!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1940 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1940* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB, 0.13399E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.16609E+00_JPRB, 0.16844E+00_JPRB, 0.16999E+00_JPRB, 0.17096E+00_JPRB, 0.17075E+00_JPRB, 0.17166E+00_JPRB, 0.17263E+00_JPRB ,&
&  0.17390E+00_JPRB, 0.17543E+00_JPRB, 0.17646E+00_JPRB, 0.17743E+00_JPRB, 0.17791E+00_JPRB, 0.17789E+00_JPRB, 0.17787E+00_JPRB ,&
&  0.17768E+00_JPRB, 0.17744E+00_JPRB, 0.17476E+00_JPRB, 0.17233E+00_JPRB, 0.17065E+00_JPRB, 0.16979E+00_JPRB, 0.16896E+00_JPRB ,&
&  0.16824E+00_JPRB, 0.16714E+00_JPRB, 0.16575E+00_JPRB, 0.16359E+00_JPRB, 0.16044E+00_JPRB, 0.15738E+00_JPRB, 0.15427E+00_JPRB ,&
&  0.15148E+00_JPRB, 0.14914E+00_JPRB, 0.14698E+00_JPRB, 0.14478E+00_JPRB, 0.14336E+00_JPRB, 0.14178E+00_JPRB, 0.14019E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13621E+00_JPRB, 0.13423E+00_JPRB, 0.13222E+00_JPRB, 0.13028E+00_JPRB, 0.12836E+00_JPRB, 0.12695E+00_JPRB ,&
&  0.12546E+00_JPRB, 0.12431E+00_JPRB, 0.12305E+00_JPRB, 0.12191E+00_JPRB, 0.12071E+00_JPRB, 0.11959E+00_JPRB, 0.11836E+00_JPRB ,&
&  0.11753E+00_JPRB, 0.11666E+00_JPRB, 0.11578E+00_JPRB, 0.11496E+00_JPRB, 0.11415E+00_JPRB, 0.11337E+00_JPRB, 0.11247E+00_JPRB ,&
&  0.11167E+00_JPRB, 0.11114E+00_JPRB, 0.11059E+00_JPRB, 0.10998E+00_JPRB, 0.10941E+00_JPRB, 0.10901E+00_JPRB, 0.10866E+00_JPRB ,&
&  0.10833E+00_JPRB, 0.10830E+00_JPRB, 0.10844E+00_JPRB, 0.10859E+00_JPRB, 0.10874E+00_JPRB, 0.10910E+00_JPRB, 0.10963E+00_JPRB ,&
&  0.11026E+00_JPRB, 0.11095E+00_JPRB, 0.11185E+00_JPRB, 0.11310E+00_JPRB, 0.11450E+00_JPRB, 0.11611E+00_JPRB, 0.11787E+00_JPRB ,&
&  0.11976E+00_JPRB, 0.12180E+00_JPRB, 0.12399E+00_JPRB, 0.12620E+00_JPRB, 0.12854E+00_JPRB, 0.13089E+00_JPRB, 0.13316E+00_JPRB ,&
&  0.13551E+00_JPRB, 0.13781E+00_JPRB, 0.14010E+00_JPRB, 0.14218E+00_JPRB, 0.14522E+00_JPRB, 0.14721E+00_JPRB, 0.14936E+00_JPRB ,&
&  0.15141E+00_JPRB, 0.15376E+00_JPRB, 0.15615E+00_JPRB, 0.15912E+00_JPRB, 0.16206E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.19250E+00_JPRB, 0.19289E+00_JPRB, 0.19408E+00_JPRB, 0.20109E+00_JPRB, 0.21604E+00_JPRB, 0.29291E+00_JPRB, 0.28477E+00_JPRB ,&
&  0.27806E+00_JPRB, 0.24948E+00_JPRB, 0.24543E+00_JPRB, 0.23802E+00_JPRB, 0.22828E+00_JPRB, 0.21996E+00_JPRB, 0.21485E+00_JPRB ,&
&  0.20942E+00_JPRB, 0.20358E+00_JPRB, 0.19990E+00_JPRB, 0.19646E+00_JPRB, 0.19318E+00_JPRB, 0.18962E+00_JPRB, 0.18692E+00_JPRB ,&
&  0.18468E+00_JPRB, 0.18314E+00_JPRB, 0.18146E+00_JPRB, 0.17830E+00_JPRB, 0.17683E+00_JPRB, 0.17385E+00_JPRB, 0.16935E+00_JPRB ,&
&  0.16356E+00_JPRB, 0.16025E+00_JPRB, 0.15658E+00_JPRB, 0.15326E+00_JPRB, 0.15540E+00_JPRB, 0.15413E+00_JPRB, 0.15183E+00_JPRB ,&
&  0.14841E+00_JPRB, 0.14488E+00_JPRB, 0.14037E+00_JPRB, 0.13609E+00_JPRB, 0.13243E+00_JPRB, 0.12942E+00_JPRB, 0.12676E+00_JPRB ,&
&  0.12463E+00_JPRB, 0.12281E+00_JPRB, 0.12053E+00_JPRB, 0.11845E+00_JPRB, 0.11625E+00_JPRB, 0.11431E+00_JPRB, 0.11240E+00_JPRB ,&
&  0.11089E+00_JPRB, 0.10946E+00_JPRB, 0.10797E+00_JPRB, 0.10642E+00_JPRB, 0.10520E+00_JPRB, 0.10360E+00_JPRB, 0.10240E+00_JPRB ,&
&  0.10131E+00_JPRB, 0.10001E+00_JPRB, 0.99060E-01_JPRB, 0.97880E-01_JPRB, 0.97008E-01_JPRB, 0.96297E-01_JPRB, 0.95605E-01_JPRB ,&
&  0.94662E-01_JPRB, 0.94038E-01_JPRB, 0.93774E-01_JPRB, 0.93558E-01_JPRB, 0.93225E-01_JPRB, 0.93221E-01_JPRB, 0.93473E-01_JPRB ,&
&  0.93843E-01_JPRB, 0.94282E-01_JPRB, 0.95049E-01_JPRB, 0.95976E-01_JPRB, 0.97346E-01_JPRB, 0.99158E-01_JPRB, 0.10205E+00_JPRB ,&
&  0.10520E+00_JPRB, 0.10870E+00_JPRB, 0.11241E+00_JPRB, 0.11681E+00_JPRB, 0.12119E+00_JPRB, 0.12570E+00_JPRB, 0.13088E+00_JPRB ,&
&  0.13648E+00_JPRB, 0.14073E+00_JPRB, 0.14455E+00_JPRB, 0.14859E+00_JPRB, 0.15652E+00_JPRB, 0.16262E+00_JPRB, 0.16747E+00_JPRB ,&
&  0.17285E+00_JPRB, 0.18003E+00_JPRB, 0.18496E+00_JPRB, 0.18849E+00_JPRB, 0.19084E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.44963E+00_JPRB, 0.46100E+00_JPRB, 0.47604E+00_JPRB, 0.48512E+00_JPRB, 0.49468E+00_JPRB, 0.46192E+00_JPRB, 0.43615E+00_JPRB ,&
&  0.38372E+00_JPRB, 0.35387E+00_JPRB, 0.32910E+00_JPRB, 0.30633E+00_JPRB, 0.28615E+00_JPRB, 0.26881E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.24064E+00_JPRB, 0.23385E+00_JPRB, 0.22460E+00_JPRB, 0.21448E+00_JPRB, 0.20378E+00_JPRB, 0.19444E+00_JPRB, 0.18922E+00_JPRB ,&
&  0.18481E+00_JPRB, 0.18162E+00_JPRB, 0.18032E+00_JPRB, 0.18080E+00_JPRB, 0.18280E+00_JPRB, 0.18333E+00_JPRB, 0.18424E+00_JPRB ,&
&  0.18189E+00_JPRB, 0.18886E+00_JPRB, 0.19795E+00_JPRB, 0.20209E+00_JPRB, 0.20951E+00_JPRB, 0.20794E+00_JPRB, 0.20317E+00_JPRB ,&
&  0.19656E+00_JPRB, 0.18840E+00_JPRB, 0.17944E+00_JPRB, 0.16836E+00_JPRB, 0.15787E+00_JPRB, 0.14992E+00_JPRB, 0.14222E+00_JPRB ,&
&  0.13744E+00_JPRB, 0.13209E+00_JPRB, 0.12619E+00_JPRB, 0.12120E+00_JPRB, 0.11687E+00_JPRB, 0.11319E+00_JPRB, 0.10992E+00_JPRB ,&
&  0.10736E+00_JPRB, 0.10574E+00_JPRB, 0.10416E+00_JPRB, 0.10346E+00_JPRB, 0.10227E+00_JPRB, 0.10117E+00_JPRB, 0.10010E+00_JPRB ,&
&  0.99460E-01_JPRB, 0.98332E-01_JPRB, 0.97480E-01_JPRB, 0.96713E-01_JPRB, 0.95591E-01_JPRB, 0.94613E-01_JPRB, 0.93491E-01_JPRB ,&
&  0.92491E-01_JPRB, 0.91921E-01_JPRB, 0.91594E-01_JPRB, 0.91469E-01_JPRB, 0.91641E-01_JPRB, 0.91558E-01_JPRB, 0.91716E-01_JPRB ,&
&  0.91942E-01_JPRB, 0.92292E-01_JPRB, 0.92301E-01_JPRB, 0.92543E-01_JPRB, 0.93286E-01_JPRB, 0.94316E-01_JPRB, 0.96788E-01_JPRB ,&
&  0.99670E-01_JPRB, 0.10341E+00_JPRB, 0.10754E+00_JPRB, 0.11246E+00_JPRB, 0.11744E+00_JPRB, 0.12255E+00_JPRB, 0.12815E+00_JPRB ,&
&  0.13541E+00_JPRB, 0.14193E+00_JPRB, 0.14932E+00_JPRB, 0.16043E+00_JPRB, 0.18308E+00_JPRB, 0.19651E+00_JPRB, 0.21227E+00_JPRB ,&
&  0.24005E+00_JPRB, 0.27938E+00_JPRB, 0.38238E+00_JPRB, 0.41548E+00_JPRB, 0.43258E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.62667E+00_JPRB, 0.65773E+00_JPRB, 0.68244E+00_JPRB, 0.68484E+00_JPRB, 0.68395E+00_JPRB, 0.64684E+00_JPRB, 0.58929E+00_JPRB ,&
&  0.48214E+00_JPRB, 0.41243E+00_JPRB, 0.36882E+00_JPRB, 0.32776E+00_JPRB, 0.29039E+00_JPRB, 0.26552E+00_JPRB, 0.24839E+00_JPRB ,&
&  0.23538E+00_JPRB, 0.22827E+00_JPRB, 0.22283E+00_JPRB, 0.21824E+00_JPRB, 0.21335E+00_JPRB, 0.20863E+00_JPRB, 0.19991E+00_JPRB ,&
&  0.19474E+00_JPRB, 0.19111E+00_JPRB, 0.18983E+00_JPRB, 0.19228E+00_JPRB, 0.19914E+00_JPRB, 0.20651E+00_JPRB, 0.21731E+00_JPRB ,&
&  0.23137E+00_JPRB, 0.24495E+00_JPRB, 0.25116E+00_JPRB, 0.25421E+00_JPRB, 0.25707E+00_JPRB, 0.25501E+00_JPRB, 0.23207E+00_JPRB ,&
&  0.23725E+00_JPRB, 0.26099E+00_JPRB, 0.27942E+00_JPRB, 0.29071E+00_JPRB, 0.24742E+00_JPRB, 0.20624E+00_JPRB, 0.16582E+00_JPRB ,&
&  0.14605E+00_JPRB, 0.13116E+00_JPRB, 0.12356E+00_JPRB, 0.11792E+00_JPRB, 0.11405E+00_JPRB, 0.11154E+00_JPRB, 0.10932E+00_JPRB ,&
&  0.10721E+00_JPRB, 0.10617E+00_JPRB, 0.10470E+00_JPRB, 0.10392E+00_JPRB, 0.10289E+00_JPRB, 0.10261E+00_JPRB, 0.10142E+00_JPRB ,&
&  0.10020E+00_JPRB, 0.99255E-01_JPRB, 0.98667E-01_JPRB, 0.98375E-01_JPRB, 0.97688E-01_JPRB, 0.96992E-01_JPRB, 0.95443E-01_JPRB ,&
&  0.94306E-01_JPRB, 0.93163E-01_JPRB, 0.92276E-01_JPRB, 0.92206E-01_JPRB, 0.92529E-01_JPRB, 0.93911E-01_JPRB, 0.94685E-01_JPRB ,&
&  0.94848E-01_JPRB, 0.94886E-01_JPRB, 0.94940E-01_JPRB, 0.95564E-01_JPRB, 0.96521E-01_JPRB, 0.98209E-01_JPRB, 0.10056E+00_JPRB ,&
&  0.10338E+00_JPRB, 0.10765E+00_JPRB, 0.11207E+00_JPRB, 0.11707E+00_JPRB, 0.12188E+00_JPRB, 0.12681E+00_JPRB, 0.13310E+00_JPRB ,&
&  0.13961E+00_JPRB, 0.14702E+00_JPRB, 0.15775E+00_JPRB, 0.16948E+00_JPRB, 0.19033E+00_JPRB, 0.20981E+00_JPRB, 0.24227E+00_JPRB ,&
&  0.31373E+00_JPRB, 0.46170E+00_JPRB, 0.54672E+00_JPRB, 0.55663E+00_JPRB, 0.58924E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.74701E+00_JPRB, 0.76543E+00_JPRB, 0.78190E+00_JPRB, 0.77931E+00_JPRB, 0.76210E+00_JPRB, 0.70441E+00_JPRB, 0.64103E+00_JPRB ,&
&  0.56235E+00_JPRB, 0.49593E+00_JPRB, 0.42674E+00_JPRB, 0.36655E+00_JPRB, 0.31632E+00_JPRB, 0.27779E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.23974E+00_JPRB, 0.23298E+00_JPRB, 0.23079E+00_JPRB, 0.23542E+00_JPRB, 0.23810E+00_JPRB, 0.24354E+00_JPRB, 0.23802E+00_JPRB ,&
&  0.23116E+00_JPRB, 0.22295E+00_JPRB, 0.22104E+00_JPRB, 0.22982E+00_JPRB, 0.24280E+00_JPRB, 0.25548E+00_JPRB, 0.27186E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.29589E+00_JPRB, 0.30955E+00_JPRB, 0.31335E+00_JPRB, 0.29850E+00_JPRB, 0.30004E+00_JPRB, 0.30585E+00_JPRB ,&
&  0.31892E+00_JPRB, 0.33748E+00_JPRB, 0.34817E+00_JPRB, 0.36855E+00_JPRB, 0.31682E+00_JPRB, 0.25669E+00_JPRB, 0.20545E+00_JPRB ,&
&  0.16749E+00_JPRB, 0.14604E+00_JPRB, 0.13150E+00_JPRB, 0.12405E+00_JPRB, 0.11721E+00_JPRB, 0.11345E+00_JPRB, 0.10938E+00_JPRB ,&
&  0.10756E+00_JPRB, 0.10735E+00_JPRB, 0.10674E+00_JPRB, 0.10626E+00_JPRB, 0.10644E+00_JPRB, 0.10685E+00_JPRB, 0.10622E+00_JPRB ,&
&  0.10540E+00_JPRB, 0.10507E+00_JPRB, 0.10460E+00_JPRB, 0.10429E+00_JPRB, 0.10354E+00_JPRB, 0.10341E+00_JPRB, 0.10215E+00_JPRB ,&
&  0.10149E+00_JPRB, 0.10031E+00_JPRB, 0.99609E-01_JPRB, 0.99443E-01_JPRB, 0.99703E-01_JPRB, 0.99983E-01_JPRB, 0.10046E+00_JPRB ,&
&  0.10145E+00_JPRB, 0.10236E+00_JPRB, 0.10391E+00_JPRB, 0.10552E+00_JPRB, 0.10804E+00_JPRB, 0.11044E+00_JPRB, 0.11382E+00_JPRB ,&
&  0.11739E+00_JPRB, 0.12172E+00_JPRB, 0.12657E+00_JPRB, 0.13199E+00_JPRB, 0.13684E+00_JPRB, 0.14163E+00_JPRB, 0.14691E+00_JPRB ,&
&  0.15114E+00_JPRB, 0.15777E+00_JPRB, 0.16526E+00_JPRB, 0.17766E+00_JPRB, 0.20348E+00_JPRB, 0.22742E+00_JPRB, 0.27671E+00_JPRB ,&
&  0.36188E+00_JPRB, 0.61725E+00_JPRB, 0.84763E+00_JPRB, 0.69777E+00_JPRB, 0.72863E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.75525E+00_JPRB, 0.76787E+00_JPRB, 0.78506E+00_JPRB, 0.76153E+00_JPRB, 0.74615E+00_JPRB, 0.74953E+00_JPRB, 0.75033E+00_JPRB ,&
&  0.72737E+00_JPRB, 0.69265E+00_JPRB, 0.64390E+00_JPRB, 0.60418E+00_JPRB, 0.53810E+00_JPRB, 0.45343E+00_JPRB, 0.41587E+00_JPRB ,&
&  0.38194E+00_JPRB, 0.37854E+00_JPRB, 0.38095E+00_JPRB, 0.39694E+00_JPRB, 0.40585E+00_JPRB, 0.40244E+00_JPRB, 0.38099E+00_JPRB ,&
&  0.35398E+00_JPRB, 0.32670E+00_JPRB, 0.30930E+00_JPRB, 0.29526E+00_JPRB, 0.29499E+00_JPRB, 0.29323E+00_JPRB, 0.28672E+00_JPRB ,&
&  0.27793E+00_JPRB, 0.28188E+00_JPRB, 0.29901E+00_JPRB, 0.32793E+00_JPRB, 0.37829E+00_JPRB, 0.35827E+00_JPRB, 0.36390E+00_JPRB ,&
&  0.39373E+00_JPRB, 0.41520E+00_JPRB, 0.42372E+00_JPRB, 0.42678E+00_JPRB, 0.42034E+00_JPRB, 0.44599E+00_JPRB, 0.38950E+00_JPRB ,&
&  0.28221E+00_JPRB, 0.22343E+00_JPRB, 0.18170E+00_JPRB, 0.15526E+00_JPRB, 0.13813E+00_JPRB, 0.12829E+00_JPRB, 0.11770E+00_JPRB ,&
&  0.11292E+00_JPRB, 0.10949E+00_JPRB, 0.10859E+00_JPRB, 0.10976E+00_JPRB, 0.11195E+00_JPRB, 0.11474E+00_JPRB, 0.11570E+00_JPRB ,&
&  0.11724E+00_JPRB, 0.11729E+00_JPRB, 0.11647E+00_JPRB, 0.11621E+00_JPRB, 0.11528E+00_JPRB, 0.11590E+00_JPRB, 0.11598E+00_JPRB ,&
&  0.11622E+00_JPRB, 0.11644E+00_JPRB, 0.11797E+00_JPRB, 0.11837E+00_JPRB, 0.11923E+00_JPRB, 0.11634E+00_JPRB, 0.11448E+00_JPRB ,&
&  0.11445E+00_JPRB, 0.11625E+00_JPRB, 0.11933E+00_JPRB, 0.12291E+00_JPRB, 0.12784E+00_JPRB, 0.13187E+00_JPRB, 0.13647E+00_JPRB ,&
&  0.14052E+00_JPRB, 0.14445E+00_JPRB, 0.14831E+00_JPRB, 0.15224E+00_JPRB, 0.15640E+00_JPRB, 0.16058E+00_JPRB, 0.16499E+00_JPRB ,&
&  0.16949E+00_JPRB, 0.17737E+00_JPRB, 0.18714E+00_JPRB, 0.20427E+00_JPRB, 0.22749E+00_JPRB, 0.25488E+00_JPRB, 0.30266E+00_JPRB ,&
&  0.36455E+00_JPRB, 0.59449E+00_JPRB, 0.89237E+00_JPRB, 0.71817E+00_JPRB, 0.73623E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.80487E+00_JPRB, 0.80285E+00_JPRB, 0.81105E+00_JPRB, 0.80071E+00_JPRB, 0.78159E+00_JPRB, 0.77246E+00_JPRB, 0.78941E+00_JPRB ,&
&  0.80919E+00_JPRB, 0.77987E+00_JPRB, 0.75840E+00_JPRB, 0.73927E+00_JPRB, 0.72586E+00_JPRB, 0.71222E+00_JPRB, 0.68989E+00_JPRB ,&
&  0.66980E+00_JPRB, 0.66252E+00_JPRB, 0.66076E+00_JPRB, 0.65578E+00_JPRB, 0.64553E+00_JPRB, 0.63547E+00_JPRB, 0.61772E+00_JPRB ,&
&  0.58539E+00_JPRB, 0.55153E+00_JPRB, 0.53351E+00_JPRB, 0.50996E+00_JPRB, 0.49313E+00_JPRB, 0.47376E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.40162E+00_JPRB, 0.34363E+00_JPRB, 0.30646E+00_JPRB, 0.34243E+00_JPRB, 0.43882E+00_JPRB, 0.43324E+00_JPRB, 0.45188E+00_JPRB ,&
&  0.48639E+00_JPRB, 0.50764E+00_JPRB, 0.51796E+00_JPRB, 0.52119E+00_JPRB, 0.53020E+00_JPRB, 0.53537E+00_JPRB, 0.53181E+00_JPRB ,&
&  0.51756E+00_JPRB, 0.46540E+00_JPRB, 0.42349E+00_JPRB, 0.32145E+00_JPRB, 0.24960E+00_JPRB, 0.21188E+00_JPRB, 0.18338E+00_JPRB ,&
&  0.16515E+00_JPRB, 0.15417E+00_JPRB, 0.15170E+00_JPRB, 0.15208E+00_JPRB, 0.15722E+00_JPRB, 0.16209E+00_JPRB, 0.16206E+00_JPRB ,&
&  0.15864E+00_JPRB, 0.15130E+00_JPRB, 0.14044E+00_JPRB, 0.13370E+00_JPRB, 0.13035E+00_JPRB, 0.13054E+00_JPRB, 0.13250E+00_JPRB ,&
&  0.13441E+00_JPRB, 0.13901E+00_JPRB, 0.15139E+00_JPRB, 0.16249E+00_JPRB, 0.17596E+00_JPRB, 0.15732E+00_JPRB, 0.14247E+00_JPRB ,&
&  0.13746E+00_JPRB, 0.14130E+00_JPRB, 0.14742E+00_JPRB, 0.15207E+00_JPRB, 0.15705E+00_JPRB, 0.16143E+00_JPRB, 0.16591E+00_JPRB ,&
&  0.17009E+00_JPRB, 0.17303E+00_JPRB, 0.17467E+00_JPRB, 0.17640E+00_JPRB, 0.17800E+00_JPRB, 0.18034E+00_JPRB, 0.18332E+00_JPRB ,&
&  0.18834E+00_JPRB, 0.19740E+00_JPRB, 0.20928E+00_JPRB, 0.22881E+00_JPRB, 0.24718E+00_JPRB, 0.26561E+00_JPRB, 0.28656E+00_JPRB ,&
&  0.30788E+00_JPRB, 0.38754E+00_JPRB, 0.58157E+00_JPRB, 0.90378E+00_JPRB, 0.89706E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.79837E+00_JPRB, 0.81839E+00_JPRB, 0.82943E+00_JPRB, 0.84348E+00_JPRB, 0.84370E+00_JPRB, 0.81338E+00_JPRB, 0.79857E+00_JPRB ,&
&  0.80324E+00_JPRB, 0.79845E+00_JPRB, 0.78787E+00_JPRB, 0.78150E+00_JPRB, 0.77265E+00_JPRB, 0.76277E+00_JPRB, 0.75422E+00_JPRB ,&
&  0.74591E+00_JPRB, 0.74102E+00_JPRB, 0.74817E+00_JPRB, 0.76233E+00_JPRB, 0.77403E+00_JPRB, 0.78054E+00_JPRB, 0.78559E+00_JPRB ,&
&  0.78506E+00_JPRB, 0.75190E+00_JPRB, 0.70688E+00_JPRB, 0.67397E+00_JPRB, 0.65722E+00_JPRB, 0.64333E+00_JPRB, 0.61058E+00_JPRB ,&
&  0.54421E+00_JPRB, 0.42253E+00_JPRB, 0.36514E+00_JPRB, 0.40591E+00_JPRB, 0.52605E+00_JPRB, 0.48996E+00_JPRB, 0.51011E+00_JPRB ,&
&  0.54032E+00_JPRB, 0.55593E+00_JPRB, 0.55171E+00_JPRB, 0.54123E+00_JPRB, 0.53776E+00_JPRB, 0.54127E+00_JPRB, 0.53771E+00_JPRB ,&
&  0.53400E+00_JPRB, 0.51782E+00_JPRB, 0.50828E+00_JPRB, 0.49791E+00_JPRB, 0.48073E+00_JPRB, 0.45513E+00_JPRB, 0.43988E+00_JPRB ,&
&  0.40237E+00_JPRB, 0.36535E+00_JPRB, 0.35493E+00_JPRB, 0.35630E+00_JPRB, 0.32578E+00_JPRB, 0.31355E+00_JPRB, 0.29933E+00_JPRB ,&
&  0.25889E+00_JPRB, 0.23746E+00_JPRB, 0.20491E+00_JPRB, 0.18434E+00_JPRB, 0.16917E+00_JPRB, 0.15576E+00_JPRB, 0.15395E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.17020E+00_JPRB, 0.19198E+00_JPRB, 0.22733E+00_JPRB, 0.25994E+00_JPRB, 0.21420E+00_JPRB, 0.19532E+00_JPRB ,&
&  0.18163E+00_JPRB, 0.17860E+00_JPRB, 0.18350E+00_JPRB, 0.18939E+00_JPRB, 0.19561E+00_JPRB, 0.20301E+00_JPRB, 0.20996E+00_JPRB ,&
&  0.21501E+00_JPRB, 0.21810E+00_JPRB, 0.21922E+00_JPRB, 0.21866E+00_JPRB, 0.21801E+00_JPRB, 0.21910E+00_JPRB, 0.22266E+00_JPRB ,&
&  0.22850E+00_JPRB, 0.24047E+00_JPRB, 0.26416E+00_JPRB, 0.29921E+00_JPRB, 0.34536E+00_JPRB, 0.40553E+00_JPRB, 0.49811E+00_JPRB ,&
&  0.62324E+00_JPRB, 0.78800E+00_JPRB, 0.83137E+00_JPRB, 0.84026E+00_JPRB, 0.80988E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.83691E+00_JPRB, 0.84760E+00_JPRB, 0.84547E+00_JPRB, 0.84445E+00_JPRB, 0.84628E+00_JPRB, 0.83926E+00_JPRB, 0.82491E+00_JPRB ,&
&  0.81063E+00_JPRB, 0.78413E+00_JPRB, 0.75389E+00_JPRB, 0.74043E+00_JPRB, 0.74819E+00_JPRB, 0.74824E+00_JPRB, 0.72951E+00_JPRB ,&
&  0.72772E+00_JPRB, 0.74163E+00_JPRB, 0.76470E+00_JPRB, 0.79048E+00_JPRB, 0.82530E+00_JPRB, 0.86142E+00_JPRB, 0.88850E+00_JPRB ,&
&  0.89913E+00_JPRB, 0.89703E+00_JPRB, 0.88928E+00_JPRB, 0.86251E+00_JPRB, 0.82683E+00_JPRB, 0.79948E+00_JPRB, 0.77873E+00_JPRB ,&
&  0.75314E+00_JPRB, 0.70609E+00_JPRB, 0.63609E+00_JPRB, 0.63062E+00_JPRB, 0.56591E+00_JPRB, 0.54480E+00_JPRB, 0.56669E+00_JPRB ,&
&  0.58203E+00_JPRB, 0.56407E+00_JPRB, 0.54044E+00_JPRB, 0.52083E+00_JPRB, 0.51717E+00_JPRB, 0.51584E+00_JPRB, 0.51073E+00_JPRB ,&
&  0.50677E+00_JPRB, 0.49997E+00_JPRB, 0.48589E+00_JPRB, 0.47285E+00_JPRB, 0.46057E+00_JPRB, 0.44832E+00_JPRB, 0.44008E+00_JPRB ,&
&  0.41247E+00_JPRB, 0.37015E+00_JPRB, 0.33880E+00_JPRB, 0.32542E+00_JPRB, 0.31814E+00_JPRB, 0.32186E+00_JPRB, 0.33430E+00_JPRB ,&
&  0.34408E+00_JPRB, 0.34423E+00_JPRB, 0.33845E+00_JPRB, 0.32636E+00_JPRB, 0.30857E+00_JPRB, 0.26252E+00_JPRB, 0.23656E+00_JPRB ,&
&  0.25186E+00_JPRB, 0.28630E+00_JPRB, 0.32191E+00_JPRB, 0.32960E+00_JPRB, 0.33924E+00_JPRB, 0.34196E+00_JPRB, 0.32775E+00_JPRB ,&
&  0.30004E+00_JPRB, 0.26797E+00_JPRB, 0.26121E+00_JPRB, 0.26316E+00_JPRB, 0.26704E+00_JPRB, 0.27467E+00_JPRB, 0.29032E+00_JPRB ,&
&  0.31411E+00_JPRB, 0.35321E+00_JPRB, 0.37513E+00_JPRB, 0.37771E+00_JPRB, 0.37040E+00_JPRB, 0.35873E+00_JPRB, 0.35693E+00_JPRB ,&
&  0.37061E+00_JPRB, 0.43178E+00_JPRB, 0.55410E+00_JPRB, 0.70241E+00_JPRB, 0.70042E+00_JPRB, 0.74412E+00_JPRB, 0.77893E+00_JPRB ,&
&  0.80860E+00_JPRB, 0.82183E+00_JPRB, 0.80837E+00_JPRB, 0.80987E+00_JPRB, 0.81676E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.84962E+00_JPRB, 0.85430E+00_JPRB, 0.84492E+00_JPRB, 0.84081E+00_JPRB, 0.83029E+00_JPRB, 0.81684E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.76796E+00_JPRB, 0.73971E+00_JPRB, 0.71319E+00_JPRB, 0.69151E+00_JPRB, 0.69566E+00_JPRB, 0.72843E+00_JPRB, 0.75599E+00_JPRB ,&
&  0.74496E+00_JPRB, 0.74098E+00_JPRB, 0.75813E+00_JPRB, 0.78885E+00_JPRB, 0.82238E+00_JPRB, 0.85338E+00_JPRB, 0.87591E+00_JPRB ,&
&  0.89429E+00_JPRB, 0.87609E+00_JPRB, 0.87946E+00_JPRB, 0.88120E+00_JPRB, 0.87496E+00_JPRB, 0.85705E+00_JPRB, 0.82830E+00_JPRB ,&
&  0.79967E+00_JPRB, 0.77249E+00_JPRB, 0.73878E+00_JPRB, 0.67924E+00_JPRB, 0.75250E+00_JPRB, 0.67463E+00_JPRB, 0.64562E+00_JPRB ,&
&  0.61850E+00_JPRB, 0.60047E+00_JPRB, 0.58026E+00_JPRB, 0.55833E+00_JPRB, 0.55073E+00_JPRB, 0.54604E+00_JPRB, 0.53806E+00_JPRB ,&
&  0.53153E+00_JPRB, 0.52467E+00_JPRB, 0.51429E+00_JPRB, 0.50080E+00_JPRB, 0.48806E+00_JPRB, 0.47436E+00_JPRB, 0.46181E+00_JPRB ,&
&  0.44379E+00_JPRB, 0.41755E+00_JPRB, 0.39000E+00_JPRB, 0.36993E+00_JPRB, 0.35962E+00_JPRB, 0.36876E+00_JPRB, 0.40073E+00_JPRB ,&
&  0.43373E+00_JPRB, 0.43921E+00_JPRB, 0.44869E+00_JPRB, 0.45109E+00_JPRB, 0.44252E+00_JPRB, 0.43281E+00_JPRB, 0.41539E+00_JPRB ,&
&  0.40171E+00_JPRB, 0.39920E+00_JPRB, 0.39997E+00_JPRB, 0.40333E+00_JPRB, 0.40864E+00_JPRB, 0.41614E+00_JPRB, 0.42662E+00_JPRB ,&
&  0.43630E+00_JPRB, 0.43950E+00_JPRB, 0.45162E+00_JPRB, 0.47826E+00_JPRB, 0.50888E+00_JPRB, 0.53987E+00_JPRB, 0.56610E+00_JPRB ,&
&  0.58365E+00_JPRB, 0.59568E+00_JPRB, 0.60374E+00_JPRB, 0.61884E+00_JPRB, 0.63443E+00_JPRB, 0.65032E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.65635E+00_JPRB, 0.65974E+00_JPRB, 0.67637E+00_JPRB, 0.70434E+00_JPRB, 0.72399E+00_JPRB, 0.75777E+00_JPRB, 0.77891E+00_JPRB ,&
&  0.80142E+00_JPRB, 0.80733E+00_JPRB, 0.80203E+00_JPRB, 0.81467E+00_JPRB, 0.83168E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.80865E+00_JPRB, 0.82662E+00_JPRB, 0.83896E+00_JPRB, 0.84020E+00_JPRB, 0.82499E+00_JPRB, 0.79839E+00_JPRB, 0.77548E+00_JPRB ,&
&  0.74931E+00_JPRB, 0.72387E+00_JPRB, 0.69971E+00_JPRB, 0.66699E+00_JPRB, 0.64937E+00_JPRB, 0.65268E+00_JPRB, 0.70158E+00_JPRB ,&
&  0.76676E+00_JPRB, 0.77159E+00_JPRB, 0.76556E+00_JPRB, 0.79097E+00_JPRB, 0.84378E+00_JPRB, 0.87585E+00_JPRB, 0.89710E+00_JPRB ,&
&  0.90078E+00_JPRB, 0.86825E+00_JPRB, 0.83498E+00_JPRB, 0.82109E+00_JPRB, 0.84407E+00_JPRB, 0.86171E+00_JPRB, 0.86686E+00_JPRB ,&
&  0.86228E+00_JPRB, 0.84057E+00_JPRB, 0.82066E+00_JPRB, 0.80365E+00_JPRB, 0.80711E+00_JPRB, 0.78188E+00_JPRB, 0.72936E+00_JPRB ,&
&  0.66306E+00_JPRB, 0.63673E+00_JPRB, 0.62572E+00_JPRB, 0.61138E+00_JPRB, 0.60045E+00_JPRB, 0.60038E+00_JPRB, 0.59094E+00_JPRB ,&
&  0.57743E+00_JPRB, 0.56199E+00_JPRB, 0.54733E+00_JPRB, 0.52985E+00_JPRB, 0.51621E+00_JPRB, 0.49695E+00_JPRB, 0.47896E+00_JPRB ,&
&  0.46229E+00_JPRB, 0.44207E+00_JPRB, 0.42359E+00_JPRB, 0.41092E+00_JPRB, 0.40559E+00_JPRB, 0.41565E+00_JPRB, 0.45499E+00_JPRB ,&
&  0.48608E+00_JPRB, 0.48300E+00_JPRB, 0.48040E+00_JPRB, 0.48393E+00_JPRB, 0.47637E+00_JPRB, 0.46553E+00_JPRB, 0.45966E+00_JPRB ,&
&  0.45456E+00_JPRB, 0.45044E+00_JPRB, 0.44850E+00_JPRB, 0.45029E+00_JPRB, 0.45245E+00_JPRB, 0.45068E+00_JPRB, 0.45340E+00_JPRB ,&
&  0.45845E+00_JPRB, 0.46382E+00_JPRB, 0.48085E+00_JPRB, 0.50531E+00_JPRB, 0.52567E+00_JPRB, 0.53983E+00_JPRB, 0.55287E+00_JPRB ,&
&  0.57339E+00_JPRB, 0.58600E+00_JPRB, 0.57319E+00_JPRB, 0.56089E+00_JPRB, 0.55430E+00_JPRB, 0.55924E+00_JPRB, 0.56966E+00_JPRB ,&
&  0.59038E+00_JPRB, 0.60346E+00_JPRB, 0.60965E+00_JPRB, 0.61123E+00_JPRB, 0.61462E+00_JPRB, 0.61557E+00_JPRB, 0.62812E+00_JPRB ,&
&  0.66176E+00_JPRB, 0.69082E+00_JPRB, 0.69841E+00_JPRB, 0.73132E+00_JPRB, 0.77343E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.73963E+00_JPRB, 0.77157E+00_JPRB, 0.79636E+00_JPRB, 0.80597E+00_JPRB, 0.80445E+00_JPRB, 0.79585E+00_JPRB, 0.77322E+00_JPRB ,&
&  0.74860E+00_JPRB, 0.73101E+00_JPRB, 0.71231E+00_JPRB, 0.68565E+00_JPRB, 0.66349E+00_JPRB, 0.65877E+00_JPRB, 0.67121E+00_JPRB ,&
&  0.71743E+00_JPRB, 0.75849E+00_JPRB, 0.77521E+00_JPRB, 0.78912E+00_JPRB, 0.83933E+00_JPRB, 0.88155E+00_JPRB, 0.89116E+00_JPRB ,&
&  0.88701E+00_JPRB, 0.88924E+00_JPRB, 0.85294E+00_JPRB, 0.81967E+00_JPRB, 0.81949E+00_JPRB, 0.82458E+00_JPRB, 0.80619E+00_JPRB ,&
&  0.83197E+00_JPRB, 0.84873E+00_JPRB, 0.84156E+00_JPRB, 0.81480E+00_JPRB, 0.80603E+00_JPRB, 0.79536E+00_JPRB, 0.78370E+00_JPRB ,&
&  0.76181E+00_JPRB, 0.71361E+00_JPRB, 0.68447E+00_JPRB, 0.68344E+00_JPRB, 0.67639E+00_JPRB, 0.66882E+00_JPRB, 0.65492E+00_JPRB ,&
&  0.61486E+00_JPRB, 0.57910E+00_JPRB, 0.55241E+00_JPRB, 0.53407E+00_JPRB, 0.52273E+00_JPRB, 0.51402E+00_JPRB, 0.50509E+00_JPRB ,&
&  0.49748E+00_JPRB, 0.47131E+00_JPRB, 0.44454E+00_JPRB, 0.43969E+00_JPRB, 0.44630E+00_JPRB, 0.45309E+00_JPRB, 0.47943E+00_JPRB ,&
&  0.50435E+00_JPRB, 0.50270E+00_JPRB, 0.49886E+00_JPRB, 0.50075E+00_JPRB, 0.50898E+00_JPRB, 0.50853E+00_JPRB, 0.49797E+00_JPRB ,&
&  0.49257E+00_JPRB, 0.48983E+00_JPRB, 0.48032E+00_JPRB, 0.47348E+00_JPRB, 0.47286E+00_JPRB, 0.47711E+00_JPRB, 0.49196E+00_JPRB ,&
&  0.50118E+00_JPRB, 0.50268E+00_JPRB, 0.51643E+00_JPRB, 0.53373E+00_JPRB, 0.54399E+00_JPRB, 0.55191E+00_JPRB, 0.55741E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.58997E+00_JPRB, 0.60495E+00_JPRB, 0.60690E+00_JPRB, 0.58379E+00_JPRB, 0.55943E+00_JPRB, 0.54026E+00_JPRB ,&
&  0.53212E+00_JPRB, 0.53674E+00_JPRB, 0.54421E+00_JPRB, 0.55122E+00_JPRB, 0.55765E+00_JPRB, 0.56178E+00_JPRB, 0.56079E+00_JPRB ,&
&  0.58021E+00_JPRB, 0.61137E+00_JPRB, 0.63679E+00_JPRB, 0.66728E+00_JPRB, 0.69902E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.69826E+00_JPRB, 0.71262E+00_JPRB, 0.72431E+00_JPRB, 0.74577E+00_JPRB, 0.76548E+00_JPRB, 0.76305E+00_JPRB, 0.75583E+00_JPRB ,&
&  0.75034E+00_JPRB, 0.74521E+00_JPRB, 0.73758E+00_JPRB, 0.72761E+00_JPRB, 0.72091E+00_JPRB, 0.72027E+00_JPRB, 0.72121E+00_JPRB ,&
&  0.72314E+00_JPRB, 0.74354E+00_JPRB, 0.77702E+00_JPRB, 0.80342E+00_JPRB, 0.83219E+00_JPRB, 0.87608E+00_JPRB, 0.89699E+00_JPRB ,&
&  0.88116E+00_JPRB, 0.88080E+00_JPRB, 0.87012E+00_JPRB, 0.85565E+00_JPRB, 0.85743E+00_JPRB, 0.84802E+00_JPRB, 0.81973E+00_JPRB ,&
&  0.81361E+00_JPRB, 0.80619E+00_JPRB, 0.80216E+00_JPRB, 0.82885E+00_JPRB, 0.84900E+00_JPRB, 0.84735E+00_JPRB, 0.84207E+00_JPRB ,&
&  0.82058E+00_JPRB, 0.78347E+00_JPRB, 0.74986E+00_JPRB, 0.73440E+00_JPRB, 0.73299E+00_JPRB, 0.71288E+00_JPRB, 0.68486E+00_JPRB ,&
&  0.62746E+00_JPRB, 0.58043E+00_JPRB, 0.55451E+00_JPRB, 0.53930E+00_JPRB, 0.53472E+00_JPRB, 0.53254E+00_JPRB, 0.52867E+00_JPRB ,&
&  0.53447E+00_JPRB, 0.52989E+00_JPRB, 0.50328E+00_JPRB, 0.47573E+00_JPRB, 0.47196E+00_JPRB, 0.48088E+00_JPRB, 0.49976E+00_JPRB ,&
&  0.51483E+00_JPRB, 0.51097E+00_JPRB, 0.51781E+00_JPRB, 0.52497E+00_JPRB, 0.54136E+00_JPRB, 0.55675E+00_JPRB, 0.55714E+00_JPRB ,&
&  0.55638E+00_JPRB, 0.55070E+00_JPRB, 0.53724E+00_JPRB, 0.52886E+00_JPRB, 0.51805E+00_JPRB, 0.51424E+00_JPRB, 0.52623E+00_JPRB ,&
&  0.54383E+00_JPRB, 0.55359E+00_JPRB, 0.56478E+00_JPRB, 0.57822E+00_JPRB, 0.58848E+00_JPRB, 0.59660E+00_JPRB, 0.60338E+00_JPRB ,&
&  0.61367E+00_JPRB, 0.62823E+00_JPRB, 0.64285E+00_JPRB, 0.65554E+00_JPRB, 0.65540E+00_JPRB, 0.64131E+00_JPRB, 0.61607E+00_JPRB ,&
&  0.59906E+00_JPRB, 0.58559E+00_JPRB, 0.58201E+00_JPRB, 0.58664E+00_JPRB, 0.58128E+00_JPRB, 0.56163E+00_JPRB, 0.55200E+00_JPRB ,&
&  0.56303E+00_JPRB, 0.58688E+00_JPRB, 0.60988E+00_JPRB, 0.64122E+00_JPRB, 0.67424E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.65259E+00_JPRB, 0.66456E+00_JPRB, 0.67792E+00_JPRB, 0.69084E+00_JPRB, 0.70411E+00_JPRB, 0.70379E+00_JPRB, 0.70481E+00_JPRB ,&
&  0.72338E+00_JPRB, 0.72571E+00_JPRB, 0.72444E+00_JPRB, 0.71559E+00_JPRB, 0.70924E+00_JPRB, 0.70934E+00_JPRB, 0.70084E+00_JPRB ,&
&  0.69702E+00_JPRB, 0.70417E+00_JPRB, 0.72362E+00_JPRB, 0.75288E+00_JPRB, 0.78134E+00_JPRB, 0.81823E+00_JPRB, 0.84542E+00_JPRB ,&
&  0.85010E+00_JPRB, 0.83980E+00_JPRB, 0.84006E+00_JPRB, 0.84331E+00_JPRB, 0.84364E+00_JPRB, 0.82680E+00_JPRB, 0.82227E+00_JPRB ,&
&  0.80292E+00_JPRB, 0.80908E+00_JPRB, 0.67907E+00_JPRB, 0.74724E+00_JPRB, 0.78980E+00_JPRB, 0.82238E+00_JPRB, 0.82895E+00_JPRB ,&
&  0.82063E+00_JPRB, 0.80812E+00_JPRB, 0.78639E+00_JPRB, 0.76184E+00_JPRB, 0.75288E+00_JPRB, 0.73265E+00_JPRB, 0.70991E+00_JPRB ,&
&  0.66149E+00_JPRB, 0.59988E+00_JPRB, 0.56856E+00_JPRB, 0.55729E+00_JPRB, 0.55209E+00_JPRB, 0.55411E+00_JPRB, 0.56032E+00_JPRB ,&
&  0.56482E+00_JPRB, 0.56607E+00_JPRB, 0.55572E+00_JPRB, 0.54016E+00_JPRB, 0.52221E+00_JPRB, 0.51286E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.50635E+00_JPRB, 0.49422E+00_JPRB, 0.49755E+00_JPRB, 0.52696E+00_JPRB, 0.55134E+00_JPRB, 0.56740E+00_JPRB, 0.57315E+00_JPRB ,&
&  0.57694E+00_JPRB, 0.56978E+00_JPRB, 0.56533E+00_JPRB, 0.55652E+00_JPRB, 0.53416E+00_JPRB, 0.52855E+00_JPRB, 0.54966E+00_JPRB ,&
&  0.57322E+00_JPRB, 0.58666E+00_JPRB, 0.59458E+00_JPRB, 0.60637E+00_JPRB, 0.61842E+00_JPRB, 0.62519E+00_JPRB, 0.63161E+00_JPRB ,&
&  0.63678E+00_JPRB, 0.64165E+00_JPRB, 0.65520E+00_JPRB, 0.66978E+00_JPRB, 0.67646E+00_JPRB, 0.67738E+00_JPRB, 0.67677E+00_JPRB ,&
&  0.67322E+00_JPRB, 0.66518E+00_JPRB, 0.65607E+00_JPRB, 0.64697E+00_JPRB, 0.62641E+00_JPRB, 0.60147E+00_JPRB, 0.58371E+00_JPRB ,&
&  0.57571E+00_JPRB, 0.58662E+00_JPRB, 0.59499E+00_JPRB, 0.60800E+00_JPRB, 0.63515E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.65463E+00_JPRB, 0.65198E+00_JPRB, 0.65143E+00_JPRB, 0.66228E+00_JPRB, 0.67908E+00_JPRB, 0.68247E+00_JPRB, 0.68289E+00_JPRB ,&
&  0.69408E+00_JPRB, 0.70240E+00_JPRB, 0.70155E+00_JPRB, 0.70021E+00_JPRB, 0.69470E+00_JPRB, 0.68183E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.67894E+00_JPRB, 0.68600E+00_JPRB, 0.69003E+00_JPRB, 0.71566E+00_JPRB, 0.74883E+00_JPRB, 0.78762E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.82377E+00_JPRB, 0.81688E+00_JPRB, 0.81338E+00_JPRB, 0.80811E+00_JPRB, 0.80278E+00_JPRB, 0.80448E+00_JPRB, 0.79891E+00_JPRB ,&
&  0.78996E+00_JPRB, 0.69594E+00_JPRB, 0.72438E+00_JPRB, 0.73245E+00_JPRB, 0.74646E+00_JPRB, 0.76875E+00_JPRB, 0.79236E+00_JPRB ,&
&  0.79043E+00_JPRB, 0.78646E+00_JPRB, 0.77840E+00_JPRB, 0.76046E+00_JPRB, 0.74394E+00_JPRB, 0.73427E+00_JPRB, 0.71810E+00_JPRB ,&
&  0.69118E+00_JPRB, 0.65457E+00_JPRB, 0.62200E+00_JPRB, 0.60009E+00_JPRB, 0.59049E+00_JPRB, 0.58883E+00_JPRB, 0.58573E+00_JPRB ,&
&  0.58530E+00_JPRB, 0.58516E+00_JPRB, 0.57576E+00_JPRB, 0.56729E+00_JPRB, 0.55876E+00_JPRB, 0.55374E+00_JPRB, 0.54182E+00_JPRB ,&
&  0.53401E+00_JPRB, 0.52359E+00_JPRB, 0.52314E+00_JPRB, 0.54256E+00_JPRB, 0.56215E+00_JPRB, 0.56496E+00_JPRB, 0.57610E+00_JPRB ,&
&  0.59084E+00_JPRB, 0.59618E+00_JPRB, 0.59669E+00_JPRB, 0.59339E+00_JPRB, 0.58586E+00_JPRB, 0.58535E+00_JPRB, 0.59273E+00_JPRB ,&
&  0.59887E+00_JPRB, 0.60808E+00_JPRB, 0.61075E+00_JPRB, 0.61487E+00_JPRB, 0.62115E+00_JPRB, 0.62592E+00_JPRB, 0.63713E+00_JPRB ,&
&  0.64300E+00_JPRB, 0.64730E+00_JPRB, 0.65907E+00_JPRB, 0.67060E+00_JPRB, 0.67504E+00_JPRB, 0.67689E+00_JPRB, 0.68512E+00_JPRB ,&
&  0.68762E+00_JPRB, 0.68485E+00_JPRB, 0.67651E+00_JPRB, 0.66400E+00_JPRB, 0.65426E+00_JPRB, 0.64022E+00_JPRB, 0.62043E+00_JPRB ,&
&  0.60501E+00_JPRB, 0.60419E+00_JPRB, 0.61475E+00_JPRB, 0.62612E+00_JPRB, 0.64432E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.65149E+00_JPRB, 0.64469E+00_JPRB, 0.63888E+00_JPRB, 0.64608E+00_JPRB, 0.65940E+00_JPRB, 0.66940E+00_JPRB, 0.66858E+00_JPRB ,&
&  0.66803E+00_JPRB, 0.67268E+00_JPRB, 0.68116E+00_JPRB, 0.68430E+00_JPRB, 0.67895E+00_JPRB, 0.66452E+00_JPRB, 0.65851E+00_JPRB ,&
&  0.65930E+00_JPRB, 0.66318E+00_JPRB, 0.66539E+00_JPRB, 0.67172E+00_JPRB, 0.69318E+00_JPRB, 0.72541E+00_JPRB, 0.75576E+00_JPRB ,&
&  0.77103E+00_JPRB, 0.76617E+00_JPRB, 0.76034E+00_JPRB, 0.75527E+00_JPRB, 0.75389E+00_JPRB, 0.75852E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.75184E+00_JPRB, 0.60673E+00_JPRB, 0.65967E+00_JPRB, 0.67764E+00_JPRB, 0.69863E+00_JPRB, 0.72397E+00_JPRB, 0.75356E+00_JPRB ,&
&  0.75558E+00_JPRB, 0.74739E+00_JPRB, 0.73097E+00_JPRB, 0.72015E+00_JPRB, 0.70675E+00_JPRB, 0.70401E+00_JPRB, 0.70293E+00_JPRB ,&
&  0.70172E+00_JPRB, 0.69903E+00_JPRB, 0.67722E+00_JPRB, 0.64220E+00_JPRB, 0.62364E+00_JPRB, 0.61332E+00_JPRB, 0.60524E+00_JPRB ,&
&  0.59794E+00_JPRB, 0.58754E+00_JPRB, 0.57712E+00_JPRB, 0.56836E+00_JPRB, 0.57099E+00_JPRB, 0.57040E+00_JPRB, 0.55896E+00_JPRB ,&
&  0.55475E+00_JPRB, 0.55338E+00_JPRB, 0.56120E+00_JPRB, 0.57085E+00_JPRB, 0.57397E+00_JPRB, 0.56762E+00_JPRB, 0.57482E+00_JPRB ,&
&  0.59291E+00_JPRB, 0.60376E+00_JPRB, 0.61333E+00_JPRB, 0.62657E+00_JPRB, 0.63253E+00_JPRB, 0.63034E+00_JPRB, 0.62962E+00_JPRB ,&
&  0.63389E+00_JPRB, 0.63120E+00_JPRB, 0.62338E+00_JPRB, 0.61727E+00_JPRB, 0.61837E+00_JPRB, 0.61926E+00_JPRB, 0.62967E+00_JPRB ,&
&  0.63923E+00_JPRB, 0.64232E+00_JPRB, 0.63843E+00_JPRB, 0.64117E+00_JPRB, 0.64679E+00_JPRB, 0.65565E+00_JPRB, 0.66143E+00_JPRB ,&
&  0.66754E+00_JPRB, 0.66571E+00_JPRB, 0.65379E+00_JPRB, 0.65312E+00_JPRB, 0.65881E+00_JPRB, 0.65523E+00_JPRB, 0.64401E+00_JPRB ,&
&  0.63531E+00_JPRB, 0.62811E+00_JPRB, 0.63245E+00_JPRB, 0.63705E+00_JPRB, 0.64491E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.67483E+00_JPRB, 0.67247E+00_JPRB, 0.66599E+00_JPRB, 0.66559E+00_JPRB, 0.66605E+00_JPRB, 0.66318E+00_JPRB, 0.66174E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.65746E+00_JPRB, 0.66679E+00_JPRB, 0.67581E+00_JPRB, 0.67874E+00_JPRB, 0.67543E+00_JPRB, 0.67306E+00_JPRB ,&
&  0.67237E+00_JPRB, 0.66811E+00_JPRB, 0.66302E+00_JPRB, 0.66241E+00_JPRB, 0.66709E+00_JPRB, 0.68274E+00_JPRB, 0.70551E+00_JPRB ,&
&  0.72256E+00_JPRB, 0.72276E+00_JPRB, 0.72684E+00_JPRB, 0.73787E+00_JPRB, 0.74000E+00_JPRB, 0.74099E+00_JPRB, 0.74729E+00_JPRB ,&
&  0.63059E+00_JPRB, 0.50596E+00_JPRB, 0.56618E+00_JPRB, 0.61926E+00_JPRB, 0.65829E+00_JPRB, 0.70250E+00_JPRB, 0.75100E+00_JPRB ,&
&  0.76619E+00_JPRB, 0.75527E+00_JPRB, 0.74547E+00_JPRB, 0.73229E+00_JPRB, 0.71401E+00_JPRB, 0.70251E+00_JPRB, 0.69318E+00_JPRB ,&
&  0.69210E+00_JPRB, 0.68222E+00_JPRB, 0.66636E+00_JPRB, 0.65375E+00_JPRB, 0.64383E+00_JPRB, 0.63112E+00_JPRB, 0.61878E+00_JPRB ,&
&  0.60596E+00_JPRB, 0.59523E+00_JPRB, 0.59331E+00_JPRB, 0.59340E+00_JPRB, 0.59743E+00_JPRB, 0.59887E+00_JPRB, 0.59540E+00_JPRB ,&
&  0.59254E+00_JPRB, 0.59457E+00_JPRB, 0.60357E+00_JPRB, 0.61091E+00_JPRB, 0.61236E+00_JPRB, 0.60911E+00_JPRB, 0.61555E+00_JPRB ,&
&  0.61831E+00_JPRB, 0.62573E+00_JPRB, 0.63565E+00_JPRB, 0.65552E+00_JPRB, 0.66992E+00_JPRB, 0.67043E+00_JPRB, 0.66911E+00_JPRB ,&
&  0.67123E+00_JPRB, 0.66463E+00_JPRB, 0.64955E+00_JPRB, 0.63952E+00_JPRB, 0.63490E+00_JPRB, 0.62833E+00_JPRB, 0.62669E+00_JPRB ,&
&  0.63169E+00_JPRB, 0.63326E+00_JPRB, 0.62767E+00_JPRB, 0.62957E+00_JPRB, 0.63838E+00_JPRB, 0.65182E+00_JPRB, 0.66440E+00_JPRB ,&
&  0.65747E+00_JPRB, 0.66534E+00_JPRB, 0.66556E+00_JPRB, 0.67478E+00_JPRB, 0.68693E+00_JPRB, 0.67900E+00_JPRB, 0.67337E+00_JPRB ,&
&  0.66805E+00_JPRB, 0.64883E+00_JPRB, 0.65327E+00_JPRB, 0.65764E+00_JPRB, 0.66299E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.65932E+00_JPRB, 0.69030E+00_JPRB, 0.69843E+00_JPRB, 0.68576E+00_JPRB, 0.66348E+00_JPRB, 0.65149E+00_JPRB, 0.64529E+00_JPRB ,&
&  0.64231E+00_JPRB, 0.63979E+00_JPRB, 0.64981E+00_JPRB, 0.65773E+00_JPRB, 0.66159E+00_JPRB, 0.65654E+00_JPRB, 0.65136E+00_JPRB ,&
&  0.64158E+00_JPRB, 0.62874E+00_JPRB, 0.62029E+00_JPRB, 0.61304E+00_JPRB, 0.61414E+00_JPRB, 0.62397E+00_JPRB, 0.64190E+00_JPRB ,&
&  0.66430E+00_JPRB, 0.68004E+00_JPRB, 0.69251E+00_JPRB, 0.70503E+00_JPRB, 0.70703E+00_JPRB, 0.70657E+00_JPRB, 0.70876E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.53190E+00_JPRB, 0.56965E+00_JPRB, 0.62076E+00_JPRB, 0.66448E+00_JPRB, 0.72745E+00_JPRB, 0.78218E+00_JPRB ,&
&  0.80384E+00_JPRB, 0.78480E+00_JPRB, 0.77032E+00_JPRB, 0.75049E+00_JPRB, 0.72527E+00_JPRB, 0.70184E+00_JPRB, 0.67420E+00_JPRB ,&
&  0.65320E+00_JPRB, 0.63742E+00_JPRB, 0.63051E+00_JPRB, 0.62465E+00_JPRB, 0.61837E+00_JPRB, 0.60987E+00_JPRB, 0.59967E+00_JPRB ,&
&  0.58956E+00_JPRB, 0.58542E+00_JPRB, 0.58387E+00_JPRB, 0.58225E+00_JPRB, 0.58549E+00_JPRB, 0.59169E+00_JPRB, 0.59740E+00_JPRB ,&
&  0.60642E+00_JPRB, 0.61317E+00_JPRB, 0.61378E+00_JPRB, 0.61140E+00_JPRB, 0.60852E+00_JPRB, 0.60898E+00_JPRB, 0.62099E+00_JPRB ,&
&  0.62521E+00_JPRB, 0.63385E+00_JPRB, 0.64349E+00_JPRB, 0.66599E+00_JPRB, 0.68643E+00_JPRB, 0.69042E+00_JPRB, 0.68848E+00_JPRB ,&
&  0.68665E+00_JPRB, 0.67821E+00_JPRB, 0.66481E+00_JPRB, 0.65022E+00_JPRB, 0.64059E+00_JPRB, 0.63826E+00_JPRB, 0.63064E+00_JPRB ,&
&  0.62848E+00_JPRB, 0.63044E+00_JPRB, 0.63072E+00_JPRB, 0.63973E+00_JPRB, 0.65561E+00_JPRB, 0.66778E+00_JPRB, 0.66522E+00_JPRB ,&
&  0.66389E+00_JPRB, 0.66801E+00_JPRB, 0.66870E+00_JPRB, 0.69162E+00_JPRB, 0.70032E+00_JPRB, 0.70149E+00_JPRB, 0.69259E+00_JPRB ,&
&  0.68482E+00_JPRB, 0.67913E+00_JPRB, 0.51245E+00_JPRB, 0.59958E+00_JPRB, 0.62540E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.68387E+00_JPRB, 0.71706E+00_JPRB, 0.71773E+00_JPRB, 0.68830E+00_JPRB, 0.65738E+00_JPRB, 0.63841E+00_JPRB, 0.62730E+00_JPRB ,&
&  0.62538E+00_JPRB, 0.62022E+00_JPRB, 0.62216E+00_JPRB, 0.63222E+00_JPRB, 0.64353E+00_JPRB, 0.64652E+00_JPRB, 0.63179E+00_JPRB ,&
&  0.61347E+00_JPRB, 0.60080E+00_JPRB, 0.59709E+00_JPRB, 0.59724E+00_JPRB, 0.59558E+00_JPRB, 0.59541E+00_JPRB, 0.60561E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.63161E+00_JPRB, 0.64582E+00_JPRB, 0.65693E+00_JPRB, 0.66461E+00_JPRB, 0.67121E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.69649E+00_JPRB, 0.52713E+00_JPRB, 0.54952E+00_JPRB, 0.64578E+00_JPRB, 0.72053E+00_JPRB, 0.81007E+00_JPRB, 0.86066E+00_JPRB ,&
&  0.86730E+00_JPRB, 0.85234E+00_JPRB, 0.82454E+00_JPRB, 0.79055E+00_JPRB, 0.75080E+00_JPRB, 0.71270E+00_JPRB, 0.67541E+00_JPRB ,&
&  0.65009E+00_JPRB, 0.63071E+00_JPRB, 0.62005E+00_JPRB, 0.60655E+00_JPRB, 0.59895E+00_JPRB, 0.59492E+00_JPRB, 0.59104E+00_JPRB ,&
&  0.58840E+00_JPRB, 0.58409E+00_JPRB, 0.58694E+00_JPRB, 0.59364E+00_JPRB, 0.59504E+00_JPRB, 0.60288E+00_JPRB, 0.60491E+00_JPRB ,&
&  0.61428E+00_JPRB, 0.62743E+00_JPRB, 0.63018E+00_JPRB, 0.62664E+00_JPRB, 0.62709E+00_JPRB, 0.63516E+00_JPRB, 0.64720E+00_JPRB ,&
&  0.65483E+00_JPRB, 0.67358E+00_JPRB, 0.68813E+00_JPRB, 0.70549E+00_JPRB, 0.71624E+00_JPRB, 0.71324E+00_JPRB, 0.70062E+00_JPRB ,&
&  0.70164E+00_JPRB, 0.69512E+00_JPRB, 0.68679E+00_JPRB, 0.67629E+00_JPRB, 0.66475E+00_JPRB, 0.65414E+00_JPRB, 0.64564E+00_JPRB ,&
&  0.64300E+00_JPRB, 0.64997E+00_JPRB, 0.65681E+00_JPRB, 0.67615E+00_JPRB, 0.68791E+00_JPRB, 0.68996E+00_JPRB, 0.69249E+00_JPRB ,&
&  0.69677E+00_JPRB, 0.69826E+00_JPRB, 0.69916E+00_JPRB, 0.67257E+00_JPRB, 0.72206E+00_JPRB, 0.73068E+00_JPRB, 0.72381E+00_JPRB ,&
&  0.71472E+00_JPRB, 0.68829E+00_JPRB, 0.68226E+00_JPRB, 0.55942E+00_JPRB, 0.64037E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.77561E+00_JPRB, 0.76789E+00_JPRB, 0.73347E+00_JPRB, 0.69048E+00_JPRB, 0.65325E+00_JPRB, 0.62412E+00_JPRB, 0.60939E+00_JPRB ,&
&  0.59698E+00_JPRB, 0.58725E+00_JPRB, 0.59008E+00_JPRB, 0.59875E+00_JPRB, 0.60891E+00_JPRB, 0.60969E+00_JPRB, 0.60265E+00_JPRB ,&
&  0.59228E+00_JPRB, 0.57907E+00_JPRB, 0.57300E+00_JPRB, 0.57068E+00_JPRB, 0.56508E+00_JPRB, 0.56424E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.57022E+00_JPRB, 0.57698E+00_JPRB, 0.58116E+00_JPRB, 0.59070E+00_JPRB, 0.60543E+00_JPRB, 0.62709E+00_JPRB, 0.64857E+00_JPRB ,&
&  0.64729E+00_JPRB, 0.54717E+00_JPRB, 0.59945E+00_JPRB, 0.71557E+00_JPRB, 0.82589E+00_JPRB, 0.88416E+00_JPRB, 0.90038E+00_JPRB ,&
&  0.88817E+00_JPRB, 0.86112E+00_JPRB, 0.82404E+00_JPRB, 0.78138E+00_JPRB, 0.73857E+00_JPRB, 0.70001E+00_JPRB, 0.66320E+00_JPRB ,&
&  0.63605E+00_JPRB, 0.61664E+00_JPRB, 0.60641E+00_JPRB, 0.59954E+00_JPRB, 0.59474E+00_JPRB, 0.58228E+00_JPRB, 0.57461E+00_JPRB ,&
&  0.57522E+00_JPRB, 0.57719E+00_JPRB, 0.58135E+00_JPRB, 0.58590E+00_JPRB, 0.59973E+00_JPRB, 0.61974E+00_JPRB, 0.62870E+00_JPRB ,&
&  0.63012E+00_JPRB, 0.63809E+00_JPRB, 0.64285E+00_JPRB, 0.63738E+00_JPRB, 0.63534E+00_JPRB, 0.65904E+00_JPRB, 0.67888E+00_JPRB ,&
&  0.70016E+00_JPRB, 0.72692E+00_JPRB, 0.75918E+00_JPRB, 0.77514E+00_JPRB, 0.77247E+00_JPRB, 0.75842E+00_JPRB, 0.73233E+00_JPRB ,&
&  0.71844E+00_JPRB, 0.70083E+00_JPRB, 0.68461E+00_JPRB, 0.67508E+00_JPRB, 0.66718E+00_JPRB, 0.65773E+00_JPRB, 0.64771E+00_JPRB ,&
&  0.64141E+00_JPRB, 0.63535E+00_JPRB, 0.63963E+00_JPRB, 0.65964E+00_JPRB, 0.67987E+00_JPRB, 0.69160E+00_JPRB, 0.69439E+00_JPRB ,&
&  0.69164E+00_JPRB, 0.69013E+00_JPRB, 0.69524E+00_JPRB, 0.74495E+00_JPRB, 0.73553E+00_JPRB, 0.73636E+00_JPRB, 0.72786E+00_JPRB ,&
&  0.71952E+00_JPRB, 0.70562E+00_JPRB, 0.68404E+00_JPRB, 0.67524E+00_JPRB, 0.70229E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.83821E+00_JPRB, 0.78473E+00_JPRB, 0.73509E+00_JPRB, 0.68240E+00_JPRB, 0.64511E+00_JPRB, 0.62995E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.60281E+00_JPRB, 0.58684E+00_JPRB, 0.58537E+00_JPRB, 0.58760E+00_JPRB, 0.58532E+00_JPRB, 0.57881E+00_JPRB, 0.56635E+00_JPRB ,&
&  0.56103E+00_JPRB, 0.56627E+00_JPRB, 0.56342E+00_JPRB, 0.55698E+00_JPRB, 0.54994E+00_JPRB, 0.54421E+00_JPRB, 0.54013E+00_JPRB ,&
&  0.53851E+00_JPRB, 0.54308E+00_JPRB, 0.54626E+00_JPRB, 0.55781E+00_JPRB, 0.57708E+00_JPRB, 0.59926E+00_JPRB, 0.62601E+00_JPRB ,&
&  0.62381E+00_JPRB, 0.55364E+00_JPRB, 0.69154E+00_JPRB, 0.86423E+00_JPRB, 0.98808E+00_JPRB, 0.10033E+01_JPRB, 0.94706E+00_JPRB ,&
&  0.90628E+00_JPRB, 0.84041E+00_JPRB, 0.78973E+00_JPRB, 0.75226E+00_JPRB, 0.71907E+00_JPRB, 0.69172E+00_JPRB, 0.67116E+00_JPRB ,&
&  0.64750E+00_JPRB, 0.62893E+00_JPRB, 0.61079E+00_JPRB, 0.59343E+00_JPRB, 0.59105E+00_JPRB, 0.58690E+00_JPRB, 0.58001E+00_JPRB ,&
&  0.57578E+00_JPRB, 0.57716E+00_JPRB, 0.58030E+00_JPRB, 0.58675E+00_JPRB, 0.60037E+00_JPRB, 0.64455E+00_JPRB, 0.67996E+00_JPRB ,&
&  0.68967E+00_JPRB, 0.70208E+00_JPRB, 0.70246E+00_JPRB, 0.69869E+00_JPRB, 0.70806E+00_JPRB, 0.72510E+00_JPRB, 0.75712E+00_JPRB ,&
&  0.79611E+00_JPRB, 0.81811E+00_JPRB, 0.82830E+00_JPRB, 0.83683E+00_JPRB, 0.82633E+00_JPRB, 0.81162E+00_JPRB, 0.79163E+00_JPRB ,&
&  0.77159E+00_JPRB, 0.74378E+00_JPRB, 0.72124E+00_JPRB, 0.70587E+00_JPRB, 0.68898E+00_JPRB, 0.67656E+00_JPRB, 0.66304E+00_JPRB ,&
&  0.64975E+00_JPRB, 0.63664E+00_JPRB, 0.64632E+00_JPRB, 0.66114E+00_JPRB, 0.67080E+00_JPRB, 0.67278E+00_JPRB, 0.68216E+00_JPRB ,&
&  0.68878E+00_JPRB, 0.70552E+00_JPRB, 0.69409E+00_JPRB, 0.75028E+00_JPRB, 0.80096E+00_JPRB, 0.78661E+00_JPRB, 0.77319E+00_JPRB ,&
&  0.75211E+00_JPRB, 0.72990E+00_JPRB, 0.70297E+00_JPRB, 0.69941E+00_JPRB, 0.83521E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.71960E+00_JPRB, 0.68150E+00_JPRB, 0.64080E+00_JPRB, 0.61515E+00_JPRB, 0.59454E+00_JPRB, 0.57723E+00_JPRB, 0.57515E+00_JPRB ,&
&  0.57168E+00_JPRB, 0.56402E+00_JPRB, 0.57017E+00_JPRB, 0.56802E+00_JPRB, 0.56098E+00_JPRB, 0.54505E+00_JPRB, 0.54768E+00_JPRB ,&
&  0.55215E+00_JPRB, 0.54812E+00_JPRB, 0.54192E+00_JPRB, 0.53600E+00_JPRB, 0.53097E+00_JPRB, 0.52248E+00_JPRB, 0.51740E+00_JPRB ,&
&  0.51330E+00_JPRB, 0.51518E+00_JPRB, 0.52371E+00_JPRB, 0.53800E+00_JPRB, 0.56143E+00_JPRB, 0.58999E+00_JPRB, 0.61777E+00_JPRB ,&
&  0.60808E+00_JPRB, 0.66983E+00_JPRB, 0.87802E+00_JPRB, 0.11899E+01_JPRB, 0.12403E+01_JPRB, 0.10807E+01_JPRB, 0.96110E+00_JPRB ,&
&  0.87391E+00_JPRB, 0.79236E+00_JPRB, 0.74740E+00_JPRB, 0.72075E+00_JPRB, 0.69510E+00_JPRB, 0.68294E+00_JPRB, 0.67012E+00_JPRB ,&
&  0.65497E+00_JPRB, 0.63407E+00_JPRB, 0.60753E+00_JPRB, 0.59118E+00_JPRB, 0.58489E+00_JPRB, 0.58570E+00_JPRB, 0.58196E+00_JPRB ,&
&  0.57898E+00_JPRB, 0.58798E+00_JPRB, 0.59359E+00_JPRB, 0.59930E+00_JPRB, 0.64808E+00_JPRB, 0.73194E+00_JPRB, 0.82433E+00_JPRB ,&
&  0.81229E+00_JPRB, 0.79977E+00_JPRB, 0.79987E+00_JPRB, 0.80039E+00_JPRB, 0.81698E+00_JPRB, 0.82803E+00_JPRB, 0.84815E+00_JPRB ,&
&  0.86839E+00_JPRB, 0.86109E+00_JPRB, 0.83849E+00_JPRB, 0.82126E+00_JPRB, 0.80250E+00_JPRB, 0.79543E+00_JPRB, 0.76894E+00_JPRB ,&
&  0.74528E+00_JPRB, 0.72733E+00_JPRB, 0.71751E+00_JPRB, 0.69456E+00_JPRB, 0.68140E+00_JPRB, 0.66746E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.63897E+00_JPRB, 0.62947E+00_JPRB, 0.65554E+00_JPRB, 0.66882E+00_JPRB, 0.67039E+00_JPRB, 0.67139E+00_JPRB, 0.68404E+00_JPRB ,&
&  0.70243E+00_JPRB, 0.72445E+00_JPRB, 0.73782E+00_JPRB, 0.77008E+00_JPRB, 0.76440E+00_JPRB, 0.81470E+00_JPRB, 0.79112E+00_JPRB ,&
&  0.76672E+00_JPRB, 0.72720E+00_JPRB, 0.68311E+00_JPRB, 0.66827E+00_JPRB, 0.73261E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.66767E+00_JPRB, 0.62017E+00_JPRB, 0.58341E+00_JPRB, 0.56267E+00_JPRB, 0.54192E+00_JPRB, 0.53390E+00_JPRB, 0.52940E+00_JPRB ,&
&  0.53813E+00_JPRB, 0.54545E+00_JPRB, 0.55950E+00_JPRB, 0.55636E+00_JPRB, 0.53543E+00_JPRB, 0.52665E+00_JPRB, 0.53411E+00_JPRB ,&
&  0.52714E+00_JPRB, 0.52786E+00_JPRB, 0.53383E+00_JPRB, 0.52895E+00_JPRB, 0.52295E+00_JPRB, 0.51458E+00_JPRB, 0.50407E+00_JPRB ,&
&  0.50841E+00_JPRB, 0.51555E+00_JPRB, 0.52969E+00_JPRB, 0.55111E+00_JPRB, 0.57884E+00_JPRB, 0.60664E+00_JPRB, 0.62670E+00_JPRB ,&
&  0.64318E+00_JPRB, 0.88726E+00_JPRB, 0.12483E+01_JPRB, 0.14700E+01_JPRB, 0.13320E+01_JPRB, 0.10710E+01_JPRB, 0.93951E+00_JPRB ,&
&  0.83693E+00_JPRB, 0.75733E+00_JPRB, 0.72827E+00_JPRB, 0.71120E+00_JPRB, 0.68803E+00_JPRB, 0.67780E+00_JPRB, 0.67993E+00_JPRB ,&
&  0.66743E+00_JPRB, 0.64399E+00_JPRB, 0.61856E+00_JPRB, 0.59815E+00_JPRB, 0.58708E+00_JPRB, 0.58808E+00_JPRB, 0.59273E+00_JPRB ,&
&  0.59953E+00_JPRB, 0.60909E+00_JPRB, 0.62413E+00_JPRB, 0.63541E+00_JPRB, 0.66691E+00_JPRB, 0.66733E+00_JPRB, 0.84838E+00_JPRB ,&
&  0.10565E+01_JPRB, 0.10182E+01_JPRB, 0.98790E+00_JPRB, 0.96432E+00_JPRB, 0.93944E+00_JPRB, 0.90930E+00_JPRB, 0.90158E+00_JPRB ,&
&  0.88072E+00_JPRB, 0.84239E+00_JPRB, 0.80786E+00_JPRB, 0.78652E+00_JPRB, 0.76002E+00_JPRB, 0.74050E+00_JPRB, 0.70914E+00_JPRB ,&
&  0.69334E+00_JPRB, 0.67923E+00_JPRB, 0.66963E+00_JPRB, 0.66645E+00_JPRB, 0.66772E+00_JPRB, 0.65556E+00_JPRB, 0.65021E+00_JPRB ,&
&  0.64059E+00_JPRB, 0.65450E+00_JPRB, 0.67887E+00_JPRB, 0.66301E+00_JPRB, 0.66744E+00_JPRB, 0.68999E+00_JPRB, 0.70258E+00_JPRB ,&
&  0.68771E+00_JPRB, 0.70326E+00_JPRB, 0.74265E+00_JPRB, 0.76000E+00_JPRB, 0.75137E+00_JPRB, 0.80920E+00_JPRB, 0.77492E+00_JPRB ,&
&  0.73814E+00_JPRB, 0.69855E+00_JPRB, 0.67311E+00_JPRB, 0.65311E+00_JPRB, 0.68719E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.61331E+00_JPRB, 0.57300E+00_JPRB, 0.54859E+00_JPRB, 0.52918E+00_JPRB, 0.53388E+00_JPRB, 0.52878E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.53796E+00_JPRB, 0.54689E+00_JPRB, 0.55853E+00_JPRB, 0.54759E+00_JPRB, 0.52620E+00_JPRB, 0.52047E+00_JPRB, 0.51383E+00_JPRB ,&
&  0.50492E+00_JPRB, 0.52588E+00_JPRB, 0.54623E+00_JPRB, 0.55674E+00_JPRB, 0.54944E+00_JPRB, 0.52642E+00_JPRB, 0.51431E+00_JPRB ,&
&  0.52160E+00_JPRB, 0.53927E+00_JPRB, 0.56606E+00_JPRB, 0.59557E+00_JPRB, 0.62159E+00_JPRB, 0.64618E+00_JPRB, 0.68046E+00_JPRB ,&
&  0.78311E+00_JPRB, 0.97346E+00_JPRB, 0.14106E+01_JPRB, 0.17208E+01_JPRB, 0.13597E+01_JPRB, 0.10468E+01_JPRB, 0.84924E+00_JPRB ,&
&  0.81164E+00_JPRB, 0.74667E+00_JPRB, 0.73697E+00_JPRB, 0.73278E+00_JPRB, 0.71561E+00_JPRB, 0.70089E+00_JPRB, 0.69649E+00_JPRB ,&
&  0.67563E+00_JPRB, 0.65710E+00_JPRB, 0.64247E+00_JPRB, 0.63228E+00_JPRB, 0.62780E+00_JPRB, 0.62807E+00_JPRB, 0.63115E+00_JPRB ,&
&  0.64483E+00_JPRB, 0.65515E+00_JPRB, 0.66894E+00_JPRB, 0.69069E+00_JPRB, 0.69584E+00_JPRB, 0.75744E+00_JPRB, 0.88260E+00_JPRB ,&
&  0.10191E+01_JPRB, 0.11385E+01_JPRB, 0.10674E+01_JPRB, 0.10193E+01_JPRB, 0.97817E+00_JPRB, 0.95146E+00_JPRB, 0.91108E+00_JPRB ,&
&  0.87767E+00_JPRB, 0.83161E+00_JPRB, 0.79142E+00_JPRB, 0.75706E+00_JPRB, 0.71304E+00_JPRB, 0.68757E+00_JPRB, 0.68059E+00_JPRB ,&
&  0.66611E+00_JPRB, 0.66410E+00_JPRB, 0.65427E+00_JPRB, 0.66265E+00_JPRB, 0.65811E+00_JPRB, 0.65612E+00_JPRB, 0.65707E+00_JPRB ,&
&  0.67013E+00_JPRB, 0.71322E+00_JPRB, 0.72355E+00_JPRB, 0.67180E+00_JPRB, 0.65008E+00_JPRB, 0.65869E+00_JPRB, 0.67557E+00_JPRB ,&
&  0.69267E+00_JPRB, 0.72585E+00_JPRB, 0.74458E+00_JPRB, 0.76518E+00_JPRB, 0.72778E+00_JPRB, 0.78291E+00_JPRB, 0.74529E+00_JPRB ,&
&  0.70542E+00_JPRB, 0.68210E+00_JPRB, 0.67170E+00_JPRB, 0.66060E+00_JPRB, 0.65902E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.61471E+00_JPRB, 0.58276E+00_JPRB, 0.54273E+00_JPRB, 0.52596E+00_JPRB, 0.53431E+00_JPRB, 0.52071E+00_JPRB, 0.52509E+00_JPRB ,&
&  0.53910E+00_JPRB, 0.53704E+00_JPRB, 0.53561E+00_JPRB, 0.52744E+00_JPRB, 0.52193E+00_JPRB, 0.51366E+00_JPRB, 0.51124E+00_JPRB ,&
&  0.52353E+00_JPRB, 0.55411E+00_JPRB, 0.55989E+00_JPRB, 0.54856E+00_JPRB, 0.54206E+00_JPRB, 0.52797E+00_JPRB, 0.52323E+00_JPRB ,&
&  0.54734E+00_JPRB, 0.59258E+00_JPRB, 0.63884E+00_JPRB, 0.66475E+00_JPRB, 0.68571E+00_JPRB, 0.71063E+00_JPRB, 0.80664E+00_JPRB ,&
&  0.10655E+01_JPRB, 0.25230E+01_JPRB, 0.11195E+01_JPRB, 0.18434E+01_JPRB, 0.13097E+01_JPRB, 0.98575E+00_JPRB, 0.75429E+00_JPRB ,&
&  0.71897E+00_JPRB, 0.75695E+00_JPRB, 0.76044E+00_JPRB, 0.78191E+00_JPRB, 0.75592E+00_JPRB, 0.71768E+00_JPRB, 0.70276E+00_JPRB ,&
&  0.68749E+00_JPRB, 0.68147E+00_JPRB, 0.68803E+00_JPRB, 0.69674E+00_JPRB, 0.69934E+00_JPRB, 0.69580E+00_JPRB, 0.69491E+00_JPRB ,&
&  0.70297E+00_JPRB, 0.70402E+00_JPRB, 0.72636E+00_JPRB, 0.74923E+00_JPRB, 0.75035E+00_JPRB, 0.82922E+00_JPRB, 0.94405E+00_JPRB ,&
&  0.98459E+00_JPRB, 0.11243E+01_JPRB, 0.10296E+01_JPRB, 0.99319E+00_JPRB, 0.95800E+00_JPRB, 0.91751E+00_JPRB, 0.85755E+00_JPRB ,&
&  0.80364E+00_JPRB, 0.77266E+00_JPRB, 0.74030E+00_JPRB, 0.71189E+00_JPRB, 0.67968E+00_JPRB, 0.66829E+00_JPRB, 0.65267E+00_JPRB ,&
&  0.65553E+00_JPRB, 0.67536E+00_JPRB, 0.66894E+00_JPRB, 0.66994E+00_JPRB, 0.66834E+00_JPRB, 0.68186E+00_JPRB, 0.69646E+00_JPRB ,&
&  0.72475E+00_JPRB, 0.77041E+00_JPRB, 0.74732E+00_JPRB, 0.65726E+00_JPRB, 0.62488E+00_JPRB, 0.63907E+00_JPRB, 0.65498E+00_JPRB ,&
&  0.70443E+00_JPRB, 0.74112E+00_JPRB, 0.74183E+00_JPRB, 0.74263E+00_JPRB, 0.73601E+00_JPRB, 0.77314E+00_JPRB, 0.72703E+00_JPRB ,&
&  0.68746E+00_JPRB, 0.67770E+00_JPRB, 0.67429E+00_JPRB, 0.65731E+00_JPRB, 0.65566E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.61772E+00_JPRB, 0.59997E+00_JPRB, 0.56196E+00_JPRB, 0.52949E+00_JPRB, 0.52877E+00_JPRB, 0.53361E+00_JPRB, 0.54006E+00_JPRB ,&
&  0.53848E+00_JPRB, 0.53461E+00_JPRB, 0.52972E+00_JPRB, 0.52520E+00_JPRB, 0.53337E+00_JPRB, 0.53119E+00_JPRB, 0.55789E+00_JPRB ,&
&  0.57703E+00_JPRB, 0.57038E+00_JPRB, 0.57330E+00_JPRB, 0.55171E+00_JPRB, 0.54862E+00_JPRB, 0.54441E+00_JPRB, 0.56619E+00_JPRB ,&
&  0.62011E+00_JPRB, 0.67955E+00_JPRB, 0.71483E+00_JPRB, 0.72926E+00_JPRB, 0.75139E+00_JPRB, 0.79372E+00_JPRB, 0.93974E+00_JPRB ,&
&  0.13283E+01_JPRB, 0.27288E+01_JPRB, 0.13767E+01_JPRB, 0.18275E+01_JPRB, 0.11716E+01_JPRB, 0.85683E+00_JPRB, 0.68846E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.75928E+00_JPRB, 0.78421E+00_JPRB, 0.80681E+00_JPRB, 0.73461E+00_JPRB, 0.69807E+00_JPRB, 0.68672E+00_JPRB ,&
&  0.69493E+00_JPRB, 0.73528E+00_JPRB, 0.77139E+00_JPRB, 0.78997E+00_JPRB, 0.78077E+00_JPRB, 0.76181E+00_JPRB, 0.76343E+00_JPRB ,&
&  0.76526E+00_JPRB, 0.75673E+00_JPRB, 0.79182E+00_JPRB, 0.79000E+00_JPRB, 0.86934E+00_JPRB, 0.92540E+00_JPRB, 0.94553E+00_JPRB ,&
&  0.94258E+00_JPRB, 0.10058E+01_JPRB, 0.97279E+00_JPRB, 0.96304E+00_JPRB, 0.91124E+00_JPRB, 0.86582E+00_JPRB, 0.80315E+00_JPRB ,&
&  0.76843E+00_JPRB, 0.74249E+00_JPRB, 0.71485E+00_JPRB, 0.68007E+00_JPRB, 0.66433E+00_JPRB, 0.66752E+00_JPRB, 0.65700E+00_JPRB ,&
&  0.67877E+00_JPRB, 0.70460E+00_JPRB, 0.70300E+00_JPRB, 0.70724E+00_JPRB, 0.71660E+00_JPRB, 0.73345E+00_JPRB, 0.75112E+00_JPRB ,&
&  0.77878E+00_JPRB, 0.81282E+00_JPRB, 0.75452E+00_JPRB, 0.65525E+00_JPRB, 0.64493E+00_JPRB, 0.65728E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.70091E+00_JPRB, 0.73294E+00_JPRB, 0.73436E+00_JPRB, 0.71228E+00_JPRB, 0.71921E+00_JPRB, 0.75011E+00_JPRB, 0.71333E+00_JPRB ,&
&  0.69497E+00_JPRB, 0.69229E+00_JPRB, 0.69950E+00_JPRB, 0.69514E+00_JPRB, 0.66601E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.57931E+00_JPRB, 0.56685E+00_JPRB, 0.55179E+00_JPRB, 0.53746E+00_JPRB, 0.53001E+00_JPRB, 0.53717E+00_JPRB, 0.54172E+00_JPRB ,&
&  0.53406E+00_JPRB, 0.53041E+00_JPRB, 0.52420E+00_JPRB, 0.53006E+00_JPRB, 0.55668E+00_JPRB, 0.60484E+00_JPRB, 0.64993E+00_JPRB ,&
&  0.64586E+00_JPRB, 0.61022E+00_JPRB, 0.60390E+00_JPRB, 0.59740E+00_JPRB, 0.61185E+00_JPRB, 0.63109E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.72016E+00_JPRB, 0.75770E+00_JPRB, 0.77882E+00_JPRB, 0.79506E+00_JPRB, 0.82568E+00_JPRB, 0.86301E+00_JPRB, 0.10774E+01_JPRB ,&
&  0.15430E+01_JPRB, 0.31641E+01_JPRB, 0.25146E+01_JPRB, 0.15383E+01_JPRB, 0.98314E+00_JPRB, 0.78443E+00_JPRB, 0.72072E+00_JPRB ,&
&  0.64060E+00_JPRB, 0.56676E+00_JPRB, 0.79626E+00_JPRB, 0.82141E+00_JPRB, 0.75464E+00_JPRB, 0.72343E+00_JPRB, 0.74182E+00_JPRB ,&
&  0.79410E+00_JPRB, 0.83874E+00_JPRB, 0.86722E+00_JPRB, 0.86654E+00_JPRB, 0.84138E+00_JPRB, 0.82980E+00_JPRB, 0.83322E+00_JPRB ,&
&  0.83219E+00_JPRB, 0.84706E+00_JPRB, 0.94089E+00_JPRB, 0.94967E+00_JPRB, 0.94767E+00_JPRB, 0.92328E+00_JPRB, 0.88688E+00_JPRB ,&
&  0.88861E+00_JPRB, 0.90280E+00_JPRB, 0.94024E+00_JPRB, 0.94668E+00_JPRB, 0.83104E+00_JPRB, 0.85856E+00_JPRB, 0.78923E+00_JPRB ,&
&  0.77519E+00_JPRB, 0.74575E+00_JPRB, 0.71220E+00_JPRB, 0.66870E+00_JPRB, 0.68218E+00_JPRB, 0.69281E+00_JPRB, 0.68016E+00_JPRB ,&
&  0.71008E+00_JPRB, 0.74416E+00_JPRB, 0.73249E+00_JPRB, 0.74228E+00_JPRB, 0.75029E+00_JPRB, 0.76862E+00_JPRB, 0.78377E+00_JPRB ,&
&  0.79381E+00_JPRB, 0.82302E+00_JPRB, 0.77104E+00_JPRB, 0.71875E+00_JPRB, 0.69967E+00_JPRB, 0.67383E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.69457E+00_JPRB, 0.74077E+00_JPRB, 0.73938E+00_JPRB, 0.74433E+00_JPRB, 0.72627E+00_JPRB, 0.72604E+00_JPRB, 0.70998E+00_JPRB ,&
&  0.71569E+00_JPRB, 0.71940E+00_JPRB, 0.77075E+00_JPRB, 0.71192E+00_JPRB, 0.63152E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.52768E+00_JPRB, 0.52467E+00_JPRB, 0.51867E+00_JPRB, 0.52283E+00_JPRB, 0.52690E+00_JPRB, 0.52255E+00_JPRB, 0.52555E+00_JPRB ,&
&  0.53400E+00_JPRB, 0.54389E+00_JPRB, 0.54241E+00_JPRB, 0.57348E+00_JPRB, 0.64238E+00_JPRB, 0.71478E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.69766E+00_JPRB, 0.68079E+00_JPRB, 0.67233E+00_JPRB, 0.68234E+00_JPRB, 0.69046E+00_JPRB, 0.72542E+00_JPRB, 0.76151E+00_JPRB ,&
&  0.79516E+00_JPRB, 0.82978E+00_JPRB, 0.86258E+00_JPRB, 0.90254E+00_JPRB, 0.96524E+00_JPRB, 0.10517E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.16721E+01_JPRB, 0.30591E+01_JPRB, 0.98149E+00_JPRB, 0.97833E+00_JPRB, 0.89727E+00_JPRB, 0.74822E+00_JPRB, 0.63700E+00_JPRB ,&
&  0.55216E+00_JPRB, 0.49851E+00_JPRB, 0.70749E+00_JPRB, 0.83685E+00_JPRB, 0.79345E+00_JPRB, 0.80967E+00_JPRB, 0.84632E+00_JPRB ,&
&  0.90747E+00_JPRB, 0.93996E+00_JPRB, 0.95093E+00_JPRB, 0.92712E+00_JPRB, 0.90432E+00_JPRB, 0.89942E+00_JPRB, 0.92299E+00_JPRB ,&
&  0.96360E+00_JPRB, 0.10272E+01_JPRB, 0.10678E+01_JPRB, 0.96576E+00_JPRB, 0.97122E+00_JPRB, 0.91178E+00_JPRB, 0.85874E+00_JPRB ,&
&  0.77994E+00_JPRB, 0.82170E+00_JPRB, 0.90309E+00_JPRB, 0.93850E+00_JPRB, 0.81954E+00_JPRB, 0.75992E+00_JPRB, 0.78998E+00_JPRB ,&
&  0.76535E+00_JPRB, 0.71939E+00_JPRB, 0.68152E+00_JPRB, 0.65216E+00_JPRB, 0.66259E+00_JPRB, 0.69168E+00_JPRB, 0.68335E+00_JPRB ,&
&  0.71246E+00_JPRB, 0.74719E+00_JPRB, 0.72684E+00_JPRB, 0.75223E+00_JPRB, 0.77115E+00_JPRB, 0.78213E+00_JPRB, 0.78633E+00_JPRB ,&
&  0.79485E+00_JPRB, 0.82445E+00_JPRB, 0.84011E+00_JPRB, 0.81738E+00_JPRB, 0.75940E+00_JPRB, 0.72203E+00_JPRB, 0.73086E+00_JPRB ,&
&  0.73806E+00_JPRB, 0.76301E+00_JPRB, 0.74405E+00_JPRB, 0.71171E+00_JPRB, 0.72994E+00_JPRB, 0.73464E+00_JPRB, 0.76087E+00_JPRB ,&
&  0.80157E+00_JPRB, 0.88977E+00_JPRB, 0.94281E+00_JPRB, 0.66340E+00_JPRB, 0.57116E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.50138E+00_JPRB, 0.50008E+00_JPRB, 0.48974E+00_JPRB, 0.50084E+00_JPRB, 0.50080E+00_JPRB, 0.50910E+00_JPRB, 0.52543E+00_JPRB ,&
&  0.55270E+00_JPRB, 0.58759E+00_JPRB, 0.61832E+00_JPRB, 0.65855E+00_JPRB, 0.71675E+00_JPRB, 0.76805E+00_JPRB, 0.76526E+00_JPRB ,&
&  0.74758E+00_JPRB, 0.73733E+00_JPRB, 0.73569E+00_JPRB, 0.73751E+00_JPRB, 0.75815E+00_JPRB, 0.78698E+00_JPRB, 0.81429E+00_JPRB ,&
&  0.85264E+00_JPRB, 0.90550E+00_JPRB, 0.95780E+00_JPRB, 0.10238E+01_JPRB, 0.11004E+01_JPRB, 0.12225E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.18205E+01_JPRB, 0.16835E+01_JPRB, 0.72009E+00_JPRB, 0.94014E+00_JPRB, 0.81554E+00_JPRB, 0.65982E+00_JPRB, 0.52695E+00_JPRB ,&
&  0.46402E+00_JPRB, 0.55496E+00_JPRB, 0.75147E+00_JPRB, 0.86829E+00_JPRB, 0.85813E+00_JPRB, 0.90422E+00_JPRB, 0.94690E+00_JPRB ,&
&  0.98699E+00_JPRB, 0.10048E+01_JPRB, 0.10009E+01_JPRB, 0.98803E+00_JPRB, 0.99110E+00_JPRB, 0.10147E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.11554E+01_JPRB, 0.11548E+01_JPRB, 0.10045E+01_JPRB, 0.10135E+01_JPRB, 0.92503E+00_JPRB, 0.88273E+00_JPRB ,&
&  0.78314E+00_JPRB, 0.75199E+00_JPRB, 0.80954E+00_JPRB, 0.88030E+00_JPRB, 0.78885E+00_JPRB, 0.71392E+00_JPRB, 0.77716E+00_JPRB ,&
&  0.73447E+00_JPRB, 0.67793E+00_JPRB, 0.66008E+00_JPRB, 0.65243E+00_JPRB, 0.65715E+00_JPRB, 0.67268E+00_JPRB, 0.69152E+00_JPRB ,&
&  0.71200E+00_JPRB, 0.70768E+00_JPRB, 0.68232E+00_JPRB, 0.70078E+00_JPRB, 0.72269E+00_JPRB, 0.74249E+00_JPRB, 0.75751E+00_JPRB ,&
&  0.78256E+00_JPRB, 0.81207E+00_JPRB, 0.83455E+00_JPRB, 0.85227E+00_JPRB, 0.81812E+00_JPRB, 0.77352E+00_JPRB, 0.78078E+00_JPRB ,&
&  0.73516E+00_JPRB, 0.72553E+00_JPRB, 0.72432E+00_JPRB, 0.71938E+00_JPRB, 0.70740E+00_JPRB, 0.74884E+00_JPRB, 0.77771E+00_JPRB ,&
&  0.82090E+00_JPRB, 0.85942E+00_JPRB, 0.78627E+00_JPRB, 0.59561E+00_JPRB, 0.52894E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.51271E+00_JPRB, 0.51553E+00_JPRB, 0.50619E+00_JPRB, 0.50388E+00_JPRB, 0.49692E+00_JPRB, 0.50113E+00_JPRB, 0.51483E+00_JPRB ,&
&  0.54481E+00_JPRB, 0.59525E+00_JPRB, 0.65623E+00_JPRB, 0.71003E+00_JPRB, 0.77523E+00_JPRB, 0.82272E+00_JPRB, 0.81842E+00_JPRB ,&
&  0.79526E+00_JPRB, 0.78467E+00_JPRB, 0.76522E+00_JPRB, 0.78224E+00_JPRB, 0.79735E+00_JPRB, 0.82447E+00_JPRB, 0.85489E+00_JPRB ,&
&  0.89068E+00_JPRB, 0.93512E+00_JPRB, 0.98826E+00_JPRB, 0.10505E+01_JPRB, 0.11405E+01_JPRB, 0.12335E+01_JPRB, 0.13052E+01_JPRB ,&
&  0.84746E+00_JPRB, 0.43360E+00_JPRB, 0.74968E+00_JPRB, 0.89163E+00_JPRB, 0.68460E+00_JPRB, 0.58178E+00_JPRB, 0.48061E+00_JPRB ,&
&  0.45741E+00_JPRB, 0.63945E+00_JPRB, 0.78367E+00_JPRB, 0.90473E+00_JPRB, 0.92405E+00_JPRB, 0.97477E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10328E+01_JPRB, 0.10484E+01_JPRB, 0.10628E+01_JPRB, 0.10752E+01_JPRB, 0.11009E+01_JPRB, 0.11332E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.12339E+01_JPRB, 0.12218E+01_JPRB, 0.10280E+01_JPRB, 0.10754E+01_JPRB, 0.10409E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.74307E+00_JPRB, 0.67355E+00_JPRB, 0.88055E+00_JPRB, 0.84856E+00_JPRB, 0.68323E+00_JPRB, 0.75795E+00_JPRB ,&
&  0.69739E+00_JPRB, 0.64425E+00_JPRB, 0.63420E+00_JPRB, 0.63871E+00_JPRB, 0.62809E+00_JPRB, 0.65591E+00_JPRB, 0.67482E+00_JPRB ,&
&  0.69504E+00_JPRB, 0.67150E+00_JPRB, 0.64478E+00_JPRB, 0.65424E+00_JPRB, 0.67350E+00_JPRB, 0.69725E+00_JPRB, 0.71499E+00_JPRB ,&
&  0.74176E+00_JPRB, 0.76202E+00_JPRB, 0.78598E+00_JPRB, 0.81684E+00_JPRB, 0.84477E+00_JPRB, 0.80371E+00_JPRB, 0.78394E+00_JPRB ,&
&  0.71293E+00_JPRB, 0.68578E+00_JPRB, 0.67117E+00_JPRB, 0.66385E+00_JPRB, 0.66796E+00_JPRB, 0.69799E+00_JPRB, 0.73160E+00_JPRB ,&
&  0.76189E+00_JPRB, 0.76726E+00_JPRB, 0.69917E+00_JPRB, 0.58642E+00_JPRB, 0.52904E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.50392E+00_JPRB, 0.50684E+00_JPRB, 0.50396E+00_JPRB, 0.51208E+00_JPRB, 0.50948E+00_JPRB, 0.50826E+00_JPRB, 0.50428E+00_JPRB ,&
&  0.52697E+00_JPRB, 0.57133E+00_JPRB, 0.63919E+00_JPRB, 0.71510E+00_JPRB, 0.76971E+00_JPRB, 0.83832E+00_JPRB, 0.86580E+00_JPRB ,&
&  0.85098E+00_JPRB, 0.82172E+00_JPRB, 0.80943E+00_JPRB, 0.82413E+00_JPRB, 0.83258E+00_JPRB, 0.85448E+00_JPRB, 0.87763E+00_JPRB ,&
&  0.90179E+00_JPRB, 0.93898E+00_JPRB, 0.98055E+00_JPRB, 0.10281E+01_JPRB, 0.10873E+01_JPRB, 0.11049E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.39059E+00_JPRB, 0.55453E+00_JPRB, 0.80472E+00_JPRB, 0.83996E+00_JPRB, 0.64286E+00_JPRB, 0.55263E+00_JPRB, 0.48051E+00_JPRB ,&
&  0.53696E+00_JPRB, 0.66995E+00_JPRB, 0.79189E+00_JPRB, 0.93376E+00_JPRB, 0.94201E+00_JPRB, 0.99122E+00_JPRB, 0.10402E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.10710E+01_JPRB, 0.11081E+01_JPRB, 0.11535E+01_JPRB, 0.11844E+01_JPRB, 0.12289E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.13127E+01_JPRB, 0.13067E+01_JPRB, 0.13204E+01_JPRB, 0.11017E+01_JPRB, 0.11269E+01_JPRB, 0.11635E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.86253E+00_JPRB, 0.74253E+00_JPRB, 0.85917E+00_JPRB, 0.85875E+00_JPRB, 0.83970E+00_JPRB, 0.76061E+00_JPRB ,&
&  0.68457E+00_JPRB, 0.61781E+00_JPRB, 0.57930E+00_JPRB, 0.56641E+00_JPRB, 0.57986E+00_JPRB, 0.62924E+00_JPRB, 0.66761E+00_JPRB ,&
&  0.65066E+00_JPRB, 0.62945E+00_JPRB, 0.61759E+00_JPRB, 0.61310E+00_JPRB, 0.63850E+00_JPRB, 0.66651E+00_JPRB, 0.67772E+00_JPRB ,&
&  0.67863E+00_JPRB, 0.68292E+00_JPRB, 0.70091E+00_JPRB, 0.72667E+00_JPRB, 0.75332E+00_JPRB, 0.77208E+00_JPRB, 0.78308E+00_JPRB ,&
&  0.69763E+00_JPRB, 0.65591E+00_JPRB, 0.66226E+00_JPRB, 0.66925E+00_JPRB, 0.70932E+00_JPRB, 0.74785E+00_JPRB, 0.70284E+00_JPRB ,&
&  0.67464E+00_JPRB, 0.66159E+00_JPRB, 0.63200E+00_JPRB, 0.57020E+00_JPRB, 0.51335E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.51724E+00_JPRB, 0.53159E+00_JPRB, 0.55030E+00_JPRB, 0.57248E+00_JPRB, 0.56793E+00_JPRB, 0.56698E+00_JPRB, 0.55347E+00_JPRB ,&
&  0.55009E+00_JPRB, 0.56686E+00_JPRB, 0.60970E+00_JPRB, 0.68140E+00_JPRB, 0.73409E+00_JPRB, 0.80781E+00_JPRB, 0.90493E+00_JPRB ,&
&  0.92563E+00_JPRB, 0.89719E+00_JPRB, 0.87641E+00_JPRB, 0.86970E+00_JPRB, 0.88053E+00_JPRB, 0.90098E+00_JPRB, 0.91734E+00_JPRB ,&
&  0.93288E+00_JPRB, 0.97310E+00_JPRB, 0.10026E+01_JPRB, 0.10482E+01_JPRB, 0.10898E+01_JPRB, 0.10892E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.63770E+00_JPRB, 0.88050E+00_JPRB, 0.84058E+00_JPRB, 0.81036E+00_JPRB, 0.68790E+00_JPRB, 0.59938E+00_JPRB, 0.57110E+00_JPRB ,&
&  0.60907E+00_JPRB, 0.65988E+00_JPRB, 0.77956E+00_JPRB, 0.82371E+00_JPRB, 0.89424E+00_JPRB, 0.93780E+00_JPRB, 0.10165E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.10696E+01_JPRB, 0.11028E+01_JPRB, 0.11388E+01_JPRB, 0.11671E+01_JPRB, 0.12194E+01_JPRB, 0.12684E+01_JPRB ,&
&  0.13103E+01_JPRB, 0.13179E+01_JPRB, 0.13218E+01_JPRB, 0.11653E+01_JPRB, 0.10218E+01_JPRB, 0.10139E+01_JPRB, 0.97299E+00_JPRB ,&
&  0.87050E+00_JPRB, 0.66099E+00_JPRB, 0.69026E+00_JPRB, 0.82085E+00_JPRB, 0.82207E+00_JPRB, 0.80036E+00_JPRB, 0.73953E+00_JPRB ,&
&  0.66425E+00_JPRB, 0.59002E+00_JPRB, 0.55226E+00_JPRB, 0.51921E+00_JPRB, 0.51753E+00_JPRB, 0.55658E+00_JPRB, 0.59935E+00_JPRB ,&
&  0.59139E+00_JPRB, 0.57413E+00_JPRB, 0.56632E+00_JPRB, 0.57981E+00_JPRB, 0.60955E+00_JPRB, 0.63408E+00_JPRB, 0.65213E+00_JPRB ,&
&  0.62681E+00_JPRB, 0.59385E+00_JPRB, 0.58596E+00_JPRB, 0.60042E+00_JPRB, 0.62688E+00_JPRB, 0.65573E+00_JPRB, 0.69139E+00_JPRB ,&
&  0.71456E+00_JPRB, 0.72544E+00_JPRB, 0.82269E+00_JPRB, 0.10179E+01_JPRB, 0.11538E+01_JPRB, 0.12416E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.81370E+00_JPRB, 0.68079E+00_JPRB, 0.60051E+00_JPRB, 0.54720E+00_JPRB, 0.50648E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.51418E+00_JPRB, 0.56064E+00_JPRB, 0.59723E+00_JPRB, 0.64949E+00_JPRB, 0.66579E+00_JPRB, 0.66751E+00_JPRB, 0.66400E+00_JPRB ,&
&  0.65175E+00_JPRB, 0.65790E+00_JPRB, 0.67917E+00_JPRB, 0.72992E+00_JPRB, 0.78333E+00_JPRB, 0.83484E+00_JPRB, 0.88842E+00_JPRB ,&
&  0.95551E+00_JPRB, 0.98982E+00_JPRB, 0.96790E+00_JPRB, 0.96443E+00_JPRB, 0.98219E+00_JPRB, 0.10054E+01_JPRB, 0.10069E+01_JPRB ,&
&  0.99510E+00_JPRB, 0.99741E+00_JPRB, 0.10016E+01_JPRB, 0.10276E+01_JPRB, 0.10601E+01_JPRB, 0.10685E+01_JPRB, 0.73351E+00_JPRB ,&
&  0.91862E+00_JPRB, 0.84868E+00_JPRB, 0.81378E+00_JPRB, 0.81370E+00_JPRB, 0.77208E+00_JPRB, 0.70705E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.66315E+00_JPRB, 0.67502E+00_JPRB, 0.70839E+00_JPRB, 0.73620E+00_JPRB, 0.80207E+00_JPRB, 0.83632E+00_JPRB, 0.90930E+00_JPRB ,&
&  0.97293E+00_JPRB, 0.99301E+00_JPRB, 0.10038E+01_JPRB, 0.10150E+01_JPRB, 0.10278E+01_JPRB, 0.10567E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11122E+01_JPRB, 0.10930E+01_JPRB, 0.10543E+01_JPRB, 0.94383E+00_JPRB, 0.92842E+00_JPRB, 0.76412E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.48508E+00_JPRB, 0.63703E+00_JPRB, 0.81500E+00_JPRB, 0.79274E+00_JPRB, 0.76432E+00_JPRB, 0.69945E+00_JPRB ,&
&  0.61307E+00_JPRB, 0.56953E+00_JPRB, 0.54437E+00_JPRB, 0.52132E+00_JPRB, 0.50599E+00_JPRB, 0.51949E+00_JPRB, 0.54102E+00_JPRB ,&
&  0.54735E+00_JPRB, 0.54336E+00_JPRB, 0.53948E+00_JPRB, 0.55252E+00_JPRB, 0.57792E+00_JPRB, 0.67623E+00_JPRB, 0.86970E+00_JPRB ,&
&  0.10496E+01_JPRB, 0.52024E+00_JPRB, 0.48078E+00_JPRB, 0.50164E+00_JPRB, 0.54693E+00_JPRB, 0.59499E+00_JPRB, 0.66088E+00_JPRB ,&
&  0.81964E+00_JPRB, 0.10710E+01_JPRB, 0.10137E+01_JPRB, 0.12115E+01_JPRB, 0.24985E+01_JPRB, 0.27334E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.98296E+00_JPRB, 0.69190E+00_JPRB, 0.55865E+00_JPRB, 0.51578E+00_JPRB, 0.50476E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.54185E+00_JPRB, 0.60203E+00_JPRB, 0.66875E+00_JPRB, 0.70978E+00_JPRB, 0.68223E+00_JPRB, 0.67463E+00_JPRB, 0.68610E+00_JPRB ,&
&  0.70614E+00_JPRB, 0.73536E+00_JPRB, 0.76697E+00_JPRB, 0.80928E+00_JPRB, 0.83868E+00_JPRB, 0.87626E+00_JPRB, 0.92637E+00_JPRB ,&
&  0.98206E+00_JPRB, 0.98665E+00_JPRB, 0.10017E+01_JPRB, 0.10180E+01_JPRB, 0.10299E+01_JPRB, 0.10453E+01_JPRB, 0.10556E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10609E+01_JPRB, 0.10546E+01_JPRB, 0.10718E+01_JPRB, 0.10690E+01_JPRB, 0.11407E+01_JPRB, 0.81823E+00_JPRB ,&
&  0.96579E+00_JPRB, 0.89323E+00_JPRB, 0.81617E+00_JPRB, 0.81451E+00_JPRB, 0.79746E+00_JPRB, 0.77615E+00_JPRB, 0.75124E+00_JPRB ,&
&  0.75075E+00_JPRB, 0.73565E+00_JPRB, 0.69322E+00_JPRB, 0.71439E+00_JPRB, 0.71616E+00_JPRB, 0.73467E+00_JPRB, 0.77759E+00_JPRB ,&
&  0.81952E+00_JPRB, 0.84897E+00_JPRB, 0.85240E+00_JPRB, 0.86688E+00_JPRB, 0.86820E+00_JPRB, 0.85875E+00_JPRB, 0.84590E+00_JPRB ,&
&  0.82756E+00_JPRB, 0.81992E+00_JPRB, 0.81279E+00_JPRB, 0.79065E+00_JPRB, 0.79200E+00_JPRB, 0.71053E+00_JPRB, 0.59193E+00_JPRB ,&
&  0.46091E+00_JPRB, 0.46116E+00_JPRB, 0.68566E+00_JPRB, 0.86851E+00_JPRB, 0.79785E+00_JPRB, 0.76869E+00_JPRB, 0.68167E+00_JPRB ,&
&  0.62342E+00_JPRB, 0.56113E+00_JPRB, 0.54000E+00_JPRB, 0.50571E+00_JPRB, 0.49416E+00_JPRB, 0.50679E+00_JPRB, 0.53936E+00_JPRB ,&
&  0.55451E+00_JPRB, 0.54570E+00_JPRB, 0.54008E+00_JPRB, 0.51885E+00_JPRB, 0.52427E+00_JPRB, 0.55260E+00_JPRB, 0.51933E+00_JPRB ,&
&  0.59600E+00_JPRB, 0.47700E+00_JPRB, 0.47860E+00_JPRB, 0.53922E+00_JPRB, 0.50389E+00_JPRB, 0.55042E+00_JPRB, 0.61800E+00_JPRB ,&
&  0.80361E+00_JPRB, 0.88300E+00_JPRB, 0.12192E+01_JPRB, 0.34173E+01_JPRB, 0.39403E+01_JPRB, 0.49409E+01_JPRB, 0.19648E+01_JPRB ,&
&  0.90874E+00_JPRB, 0.60047E+00_JPRB, 0.53151E+00_JPRB, 0.52978E+00_JPRB, 0.52265E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.55709E+00_JPRB, 0.63645E+00_JPRB, 0.70241E+00_JPRB, 0.62904E+00_JPRB, 0.63403E+00_JPRB, 0.64965E+00_JPRB, 0.67537E+00_JPRB ,&
&  0.70516E+00_JPRB, 0.74169E+00_JPRB, 0.78051E+00_JPRB, 0.82671E+00_JPRB, 0.86301E+00_JPRB, 0.89659E+00_JPRB, 0.93329E+00_JPRB ,&
&  0.97211E+00_JPRB, 0.98307E+00_JPRB, 0.10133E+01_JPRB, 0.10152E+01_JPRB, 0.10369E+01_JPRB, 0.10331E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10948E+01_JPRB, 0.11109E+01_JPRB, 0.11202E+01_JPRB, 0.11031E+01_JPRB, 0.11220E+01_JPRB, 0.85417E+00_JPRB ,&
&  0.10295E+01_JPRB, 0.93672E+00_JPRB, 0.85249E+00_JPRB, 0.83380E+00_JPRB, 0.80207E+00_JPRB, 0.79601E+00_JPRB, 0.78610E+00_JPRB ,&
&  0.77673E+00_JPRB, 0.77319E+00_JPRB, 0.75953E+00_JPRB, 0.71812E+00_JPRB, 0.68898E+00_JPRB, 0.70374E+00_JPRB, 0.72033E+00_JPRB ,&
&  0.74562E+00_JPRB, 0.77673E+00_JPRB, 0.80043E+00_JPRB, 0.81744E+00_JPRB, 0.82919E+00_JPRB, 0.82655E+00_JPRB, 0.82215E+00_JPRB ,&
&  0.81378E+00_JPRB, 0.81580E+00_JPRB, 0.76275E+00_JPRB, 0.75193E+00_JPRB, 0.78357E+00_JPRB, 0.69145E+00_JPRB, 0.56008E+00_JPRB ,&
&  0.46137E+00_JPRB, 0.59726E+00_JPRB, 0.79356E+00_JPRB, 0.95089E+00_JPRB, 0.85628E+00_JPRB, 0.79580E+00_JPRB, 0.73482E+00_JPRB ,&
&  0.65649E+00_JPRB, 0.59526E+00_JPRB, 0.57061E+00_JPRB, 0.52488E+00_JPRB, 0.50768E+00_JPRB, 0.50763E+00_JPRB, 0.51139E+00_JPRB ,&
&  0.50972E+00_JPRB, 0.50832E+00_JPRB, 0.50153E+00_JPRB, 0.49293E+00_JPRB, 0.46040E+00_JPRB, 0.40268E+00_JPRB, 0.57380E+00_JPRB ,&
&  0.48531E+00_JPRB, 0.38766E+00_JPRB, 0.56482E+00_JPRB, 0.40885E+00_JPRB, 0.54029E+00_JPRB, 0.59831E+00_JPRB, 0.64561E+00_JPRB ,&
&  0.60543E+00_JPRB, 0.76637E+00_JPRB, 0.11153E+01_JPRB, 0.11421E+01_JPRB, 0.12000E+01_JPRB, 0.12733E+01_JPRB, 0.74995E+00_JPRB ,&
&  0.59875E+00_JPRB, 0.58529E+00_JPRB, 0.58511E+00_JPRB, 0.53915E+00_JPRB, 0.53533E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.56168E+00_JPRB, 0.61723E+00_JPRB, 0.70521E+00_JPRB, 0.63327E+00_JPRB, 0.62968E+00_JPRB, 0.64238E+00_JPRB, 0.66253E+00_JPRB ,&
&  0.68334E+00_JPRB, 0.71133E+00_JPRB, 0.74143E+00_JPRB, 0.78353E+00_JPRB, 0.82017E+00_JPRB, 0.85291E+00_JPRB, 0.88386E+00_JPRB ,&
&  0.91215E+00_JPRB, 0.93612E+00_JPRB, 0.95925E+00_JPRB, 0.96905E+00_JPRB, 0.98077E+00_JPRB, 0.99970E+00_JPRB, 0.10245E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10855E+01_JPRB, 0.11103E+01_JPRB, 0.11545E+01_JPRB, 0.11853E+01_JPRB, 0.12101E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.96741E+00_JPRB, 0.89422E+00_JPRB, 0.84399E+00_JPRB, 0.80663E+00_JPRB, 0.78753E+00_JPRB, 0.78742E+00_JPRB ,&
&  0.80994E+00_JPRB, 0.76939E+00_JPRB, 0.76754E+00_JPRB, 0.74158E+00_JPRB, 0.73048E+00_JPRB, 0.75626E+00_JPRB, 0.77352E+00_JPRB ,&
&  0.81896E+00_JPRB, 0.87952E+00_JPRB, 0.93055E+00_JPRB, 0.96690E+00_JPRB, 0.10056E+01_JPRB, 0.10054E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10843E+01_JPRB, 0.10325E+01_JPRB, 0.10228E+01_JPRB, 0.10010E+01_JPRB, 0.92773E+00_JPRB, 0.76270E+00_JPRB ,&
&  0.65739E+00_JPRB, 0.74501E+00_JPRB, 0.79341E+00_JPRB, 0.93585E+00_JPRB, 0.89028E+00_JPRB, 0.82018E+00_JPRB, 0.78122E+00_JPRB ,&
&  0.72077E+00_JPRB, 0.67294E+00_JPRB, 0.64152E+00_JPRB, 0.59079E+00_JPRB, 0.59822E+00_JPRB, 0.60578E+00_JPRB, 0.60997E+00_JPRB ,&
&  0.58474E+00_JPRB, 0.55998E+00_JPRB, 0.53627E+00_JPRB, 0.52518E+00_JPRB, 0.56656E+00_JPRB, 0.53378E+00_JPRB, 0.55633E+00_JPRB ,&
&  0.57296E+00_JPRB, 0.39508E+00_JPRB, 0.48360E+00_JPRB, 0.63290E+00_JPRB, 0.67116E+00_JPRB, 0.71616E+00_JPRB, 0.77574E+00_JPRB ,&
&  0.84498E+00_JPRB, 0.83792E+00_JPRB, 0.80115E+00_JPRB, 0.79159E+00_JPRB, 0.77889E+00_JPRB, 0.77291E+00_JPRB, 0.69054E+00_JPRB ,&
&  0.65988E+00_JPRB, 0.60399E+00_JPRB, 0.57289E+00_JPRB, 0.55981E+00_JPRB, 0.53448E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.60733E+00_JPRB, 0.62378E+00_JPRB, 0.68786E+00_JPRB, 0.64599E+00_JPRB, 0.64397E+00_JPRB, 0.65747E+00_JPRB, 0.67349E+00_JPRB ,&
&  0.67307E+00_JPRB, 0.68169E+00_JPRB, 0.70006E+00_JPRB, 0.73259E+00_JPRB, 0.76429E+00_JPRB, 0.78901E+00_JPRB, 0.81612E+00_JPRB ,&
&  0.83718E+00_JPRB, 0.85036E+00_JPRB, 0.85810E+00_JPRB, 0.86665E+00_JPRB, 0.89258E+00_JPRB, 0.92198E+00_JPRB, 0.96213E+00_JPRB ,&
&  0.10094E+01_JPRB, 0.10171E+01_JPRB, 0.10087E+01_JPRB, 0.10265E+01_JPRB, 0.10444E+01_JPRB, 0.11284E+01_JPRB, 0.12511E+01_JPRB ,&
&  0.95126E+00_JPRB, 0.98972E+00_JPRB, 0.92257E+00_JPRB, 0.84040E+00_JPRB, 0.83858E+00_JPRB, 0.80981E+00_JPRB, 0.82258E+00_JPRB ,&
&  0.84252E+00_JPRB, 0.83154E+00_JPRB, 0.84620E+00_JPRB, 0.85618E+00_JPRB, 0.86122E+00_JPRB, 0.88256E+00_JPRB, 0.90970E+00_JPRB ,&
&  0.94938E+00_JPRB, 0.10098E+01_JPRB, 0.10619E+01_JPRB, 0.11397E+01_JPRB, 0.12148E+01_JPRB, 0.12848E+01_JPRB, 0.13365E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.13755E+01_JPRB, 0.12598E+01_JPRB, 0.12369E+01_JPRB, 0.12288E+01_JPRB, 0.11745E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.93080E+00_JPRB, 0.88995E+00_JPRB, 0.89285E+00_JPRB, 0.86030E+00_JPRB, 0.86917E+00_JPRB, 0.87469E+00_JPRB, 0.83492E+00_JPRB ,&
&  0.81454E+00_JPRB, 0.76286E+00_JPRB, 0.72430E+00_JPRB, 0.69395E+00_JPRB, 0.69297E+00_JPRB, 0.71366E+00_JPRB, 0.71141E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.66134E+00_JPRB, 0.64152E+00_JPRB, 0.62713E+00_JPRB, 0.61470E+00_JPRB, 0.73007E+00_JPRB, 0.63622E+00_JPRB ,&
&  0.68551E+00_JPRB, 0.62343E+00_JPRB, 0.55912E+00_JPRB, 0.76048E+00_JPRB, 0.81334E+00_JPRB, 0.80889E+00_JPRB, 0.83615E+00_JPRB ,&
&  0.84246E+00_JPRB, 0.83247E+00_JPRB, 0.85123E+00_JPRB, 0.80423E+00_JPRB, 0.76767E+00_JPRB, 0.74683E+00_JPRB, 0.73461E+00_JPRB ,&
&  0.68096E+00_JPRB, 0.63006E+00_JPRB, 0.58311E+00_JPRB, 0.58480E+00_JPRB, 0.55777E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.60105E+00_JPRB, 0.60247E+00_JPRB, 0.62377E+00_JPRB, 0.66149E+00_JPRB, 0.64027E+00_JPRB, 0.65777E+00_JPRB, 0.65293E+00_JPRB ,&
&  0.65173E+00_JPRB, 0.64539E+00_JPRB, 0.65622E+00_JPRB, 0.66807E+00_JPRB, 0.68812E+00_JPRB, 0.70748E+00_JPRB, 0.71341E+00_JPRB ,&
&  0.72485E+00_JPRB, 0.73734E+00_JPRB, 0.75728E+00_JPRB, 0.79026E+00_JPRB, 0.84779E+00_JPRB, 0.93944E+00_JPRB, 0.11163E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.13519E+01_JPRB, 0.13469E+01_JPRB, 0.13079E+01_JPRB, 0.14226E+01_JPRB, 0.17641E+01_JPRB, 0.20034E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.10015E+01_JPRB, 0.91918E+00_JPRB, 0.82979E+00_JPRB, 0.83499E+00_JPRB, 0.88382E+00_JPRB, 0.89175E+00_JPRB ,&
&  0.88712E+00_JPRB, 0.90772E+00_JPRB, 0.94510E+00_JPRB, 0.99118E+00_JPRB, 0.10244E+01_JPRB, 0.10472E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.11809E+01_JPRB, 0.12514E+01_JPRB, 0.13328E+01_JPRB, 0.13092E+01_JPRB, 0.14211E+01_JPRB, 0.14466E+01_JPRB ,&
&  0.14626E+01_JPRB, 0.13698E+01_JPRB, 0.11815E+01_JPRB, 0.11742E+01_JPRB, 0.12511E+01_JPRB, 0.12429E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.10099E+01_JPRB, 0.79343E+00_JPRB, 0.79460E+00_JPRB, 0.82948E+00_JPRB, 0.89766E+00_JPRB, 0.89017E+00_JPRB ,&
&  0.90022E+00_JPRB, 0.85585E+00_JPRB, 0.82120E+00_JPRB, 0.80339E+00_JPRB, 0.79978E+00_JPRB, 0.81711E+00_JPRB, 0.82109E+00_JPRB ,&
&  0.76280E+00_JPRB, 0.73896E+00_JPRB, 0.70457E+00_JPRB, 0.69868E+00_JPRB, 0.71629E+00_JPRB, 0.64825E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.72831E+00_JPRB, 0.75774E+00_JPRB, 0.68953E+00_JPRB, 0.84670E+00_JPRB, 0.84854E+00_JPRB, 0.82638E+00_JPRB, 0.83036E+00_JPRB ,&
&  0.85094E+00_JPRB, 0.80731E+00_JPRB, 0.78125E+00_JPRB, 0.76045E+00_JPRB, 0.74797E+00_JPRB, 0.72444E+00_JPRB, 0.67632E+00_JPRB ,&
&  0.64178E+00_JPRB, 0.61852E+00_JPRB, 0.60553E+00_JPRB, 0.59518E+00_JPRB, 0.59694E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.58687E+00_JPRB, 0.60383E+00_JPRB, 0.61483E+00_JPRB, 0.60847E+00_JPRB, 0.61310E+00_JPRB, 0.61884E+00_JPRB, 0.60679E+00_JPRB ,&
&  0.59798E+00_JPRB, 0.59241E+00_JPRB, 0.60579E+00_JPRB, 0.60646E+00_JPRB, 0.60966E+00_JPRB, 0.62370E+00_JPRB, 0.64164E+00_JPRB ,&
&  0.66188E+00_JPRB, 0.67854E+00_JPRB, 0.74375E+00_JPRB, 0.87439E+00_JPRB, 0.11115E+01_JPRB, 0.14092E+01_JPRB, 0.16059E+01_JPRB ,&
&  0.16970E+01_JPRB, 0.17700E+01_JPRB, 0.18136E+01_JPRB, 0.18314E+01_JPRB, 0.16467E+01_JPRB, 0.16039E+01_JPRB, 0.16608E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.91623E+00_JPRB, 0.97638E+00_JPRB, 0.91457E+00_JPRB, 0.90927E+00_JPRB, 0.91787E+00_JPRB, 0.91791E+00_JPRB ,&
&  0.92520E+00_JPRB, 0.94870E+00_JPRB, 0.98736E+00_JPRB, 0.10476E+01_JPRB, 0.11026E+01_JPRB, 0.11372E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.12039E+01_JPRB, 0.12538E+01_JPRB, 0.12361E+01_JPRB, 0.12115E+01_JPRB, 0.12392E+01_JPRB, 0.12417E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.11861E+01_JPRB, 0.10390E+01_JPRB, 0.10286E+01_JPRB, 0.11061E+01_JPRB, 0.10478E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10231E+01_JPRB, 0.72503E+00_JPRB, 0.70739E+00_JPRB, 0.76880E+00_JPRB, 0.86168E+00_JPRB, 0.95342E+00_JPRB ,&
&  0.96644E+00_JPRB, 0.94591E+00_JPRB, 0.91077E+00_JPRB, 0.90434E+00_JPRB, 0.88830E+00_JPRB, 0.88825E+00_JPRB, 0.88204E+00_JPRB ,&
&  0.81649E+00_JPRB, 0.79159E+00_JPRB, 0.76305E+00_JPRB, 0.73923E+00_JPRB, 0.73994E+00_JPRB, 0.73702E+00_JPRB, 0.74631E+00_JPRB ,&
&  0.77936E+00_JPRB, 0.78263E+00_JPRB, 0.79619E+00_JPRB, 0.79559E+00_JPRB, 0.77935E+00_JPRB, 0.82697E+00_JPRB, 0.80714E+00_JPRB ,&
&  0.81801E+00_JPRB, 0.79148E+00_JPRB, 0.76444E+00_JPRB, 0.73874E+00_JPRB, 0.71799E+00_JPRB, 0.69348E+00_JPRB, 0.66298E+00_JPRB ,&
&  0.63418E+00_JPRB, 0.61294E+00_JPRB, 0.59912E+00_JPRB, 0.58663E+00_JPRB, 0.57637E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.57584E+00_JPRB, 0.58896E+00_JPRB, 0.60570E+00_JPRB, 0.59833E+00_JPRB, 0.57750E+00_JPRB, 0.56701E+00_JPRB, 0.56039E+00_JPRB ,&
&  0.56383E+00_JPRB, 0.58049E+00_JPRB, 0.59085E+00_JPRB, 0.59148E+00_JPRB, 0.59206E+00_JPRB, 0.60790E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.70529E+00_JPRB, 0.77647E+00_JPRB, 0.90481E+00_JPRB, 0.11602E+01_JPRB, 0.14662E+01_JPRB, 0.16976E+01_JPRB, 0.16238E+01_JPRB ,&
&  0.14884E+01_JPRB, 0.14688E+01_JPRB, 0.14359E+01_JPRB, 0.14006E+01_JPRB, 0.13795E+01_JPRB, 0.13578E+01_JPRB, 0.13016E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.94408E+00_JPRB, 0.92336E+00_JPRB, 0.91858E+00_JPRB, 0.91794E+00_JPRB, 0.90567E+00_JPRB, 0.91360E+00_JPRB ,&
&  0.92137E+00_JPRB, 0.94115E+00_JPRB, 0.97497E+00_JPRB, 0.10322E+01_JPRB, 0.10969E+01_JPRB, 0.11349E+01_JPRB, 0.11465E+01_JPRB ,&
&  0.11477E+01_JPRB, 0.11427E+01_JPRB, 0.11606E+01_JPRB, 0.10686E+01_JPRB, 0.10881E+01_JPRB, 0.10947E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10026E+01_JPRB, 0.10366E+01_JPRB, 0.10302E+01_JPRB, 0.10178E+01_JPRB, 0.99227E+00_JPRB, 0.98237E+00_JPRB ,&
&  0.99942E+00_JPRB, 0.96146E+00_JPRB, 0.74509E+00_JPRB, 0.74476E+00_JPRB, 0.78846E+00_JPRB, 0.77904E+00_JPRB, 0.93722E+00_JPRB ,&
&  0.95180E+00_JPRB, 0.96387E+00_JPRB, 0.96923E+00_JPRB, 0.98406E+00_JPRB, 0.99922E+00_JPRB, 0.10139E+01_JPRB, 0.92883E+00_JPRB ,&
&  0.89165E+00_JPRB, 0.84677E+00_JPRB, 0.83678E+00_JPRB, 0.82503E+00_JPRB, 0.83722E+00_JPRB, 0.83950E+00_JPRB, 0.83465E+00_JPRB ,&
&  0.83966E+00_JPRB, 0.85294E+00_JPRB, 0.86697E+00_JPRB, 0.88346E+00_JPRB, 0.83612E+00_JPRB, 0.79532E+00_JPRB, 0.78044E+00_JPRB ,&
&  0.76683E+00_JPRB, 0.73057E+00_JPRB, 0.71864E+00_JPRB, 0.70325E+00_JPRB, 0.69058E+00_JPRB, 0.67810E+00_JPRB, 0.66528E+00_JPRB ,&
&  0.64571E+00_JPRB, 0.62794E+00_JPRB, 0.60993E+00_JPRB, 0.59027E+00_JPRB, 0.57648E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.58132E+00_JPRB, 0.58746E+00_JPRB, 0.60336E+00_JPRB, 0.61114E+00_JPRB, 0.59004E+00_JPRB, 0.57691E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.58386E+00_JPRB, 0.60026E+00_JPRB, 0.61406E+00_JPRB, 0.62656E+00_JPRB, 0.65554E+00_JPRB, 0.67372E+00_JPRB, 0.70470E+00_JPRB ,&
&  0.76085E+00_JPRB, 0.84958E+00_JPRB, 0.10136E+01_JPRB, 0.12690E+01_JPRB, 0.14991E+01_JPRB, 0.17123E+01_JPRB, 0.15851E+01_JPRB ,&
&  0.14619E+01_JPRB, 0.14006E+01_JPRB, 0.13618E+01_JPRB, 0.12881E+01_JPRB, 0.11159E+01_JPRB, 0.11185E+01_JPRB, 0.10946E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.10135E+01_JPRB, 0.95877E+00_JPRB, 0.91756E+00_JPRB, 0.87924E+00_JPRB, 0.87358E+00_JPRB, 0.88654E+00_JPRB ,&
&  0.90393E+00_JPRB, 0.92112E+00_JPRB, 0.95224E+00_JPRB, 0.99389E+00_JPRB, 0.10443E+01_JPRB, 0.10933E+01_JPRB, 0.11139E+01_JPRB ,&
&  0.10996E+01_JPRB, 0.10828E+01_JPRB, 0.10662E+01_JPRB, 0.10215E+01_JPRB, 0.10041E+01_JPRB, 0.10052E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.10186E+01_JPRB, 0.98009E+00_JPRB, 0.99907E+00_JPRB, 0.10207E+01_JPRB, 0.97692E+00_JPRB, 0.91491E+00_JPRB, 0.90518E+00_JPRB ,&
&  0.92078E+00_JPRB, 0.97906E+00_JPRB, 0.78969E+00_JPRB, 0.87659E+00_JPRB, 0.96426E+00_JPRB, 0.89113E+00_JPRB, 0.89206E+00_JPRB ,&
&  0.94218E+00_JPRB, 0.97387E+00_JPRB, 0.10141E+01_JPRB, 0.10511E+01_JPRB, 0.10775E+01_JPRB, 0.11022E+01_JPRB, 0.94283E+00_JPRB ,&
&  0.94200E+00_JPRB, 0.88597E+00_JPRB, 0.87611E+00_JPRB, 0.87621E+00_JPRB, 0.88945E+00_JPRB, 0.88844E+00_JPRB, 0.84807E+00_JPRB ,&
&  0.82358E+00_JPRB, 0.93386E+00_JPRB, 0.10837E+01_JPRB, 0.12644E+01_JPRB, 0.13217E+01_JPRB, 0.78047E+00_JPRB, 0.78988E+00_JPRB ,&
&  0.76682E+00_JPRB, 0.73470E+00_JPRB, 0.71932E+00_JPRB, 0.70529E+00_JPRB, 0.69537E+00_JPRB, 0.68379E+00_JPRB, 0.67619E+00_JPRB ,&
&  0.66265E+00_JPRB, 0.64397E+00_JPRB, 0.62476E+00_JPRB, 0.60672E+00_JPRB, 0.59127E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.59916E+00_JPRB, 0.59631E+00_JPRB, 0.60721E+00_JPRB, 0.61624E+00_JPRB, 0.61735E+00_JPRB, 0.60439E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.59800E+00_JPRB, 0.60945E+00_JPRB, 0.61960E+00_JPRB, 0.64811E+00_JPRB, 0.68272E+00_JPRB, 0.74162E+00_JPRB, 0.80048E+00_JPRB ,&
&  0.86765E+00_JPRB, 0.95524E+00_JPRB, 0.11191E+01_JPRB, 0.13242E+01_JPRB, 0.14833E+01_JPRB, 0.15724E+01_JPRB, 0.14970E+01_JPRB ,&
&  0.14018E+01_JPRB, 0.14090E+01_JPRB, 0.13759E+01_JPRB, 0.10711E+01_JPRB, 0.96192E+00_JPRB, 0.11182E+01_JPRB, 0.97917E+00_JPRB ,&
&  0.93630E+00_JPRB, 0.91993E+00_JPRB, 0.88809E+00_JPRB, 0.85442E+00_JPRB, 0.81980E+00_JPRB, 0.82163E+00_JPRB, 0.83661E+00_JPRB ,&
&  0.86367E+00_JPRB, 0.88906E+00_JPRB, 0.91926E+00_JPRB, 0.95172E+00_JPRB, 0.98746E+00_JPRB, 0.10448E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10622E+01_JPRB, 0.10419E+01_JPRB, 0.10189E+01_JPRB, 0.97561E+00_JPRB, 0.96378E+00_JPRB, 0.96722E+00_JPRB ,&
&  0.97094E+00_JPRB, 0.96722E+00_JPRB, 0.97462E+00_JPRB, 0.99549E+00_JPRB, 0.99727E+00_JPRB, 0.90418E+00_JPRB, 0.88156E+00_JPRB ,&
&  0.89885E+00_JPRB, 0.93641E+00_JPRB, 0.87121E+00_JPRB, 0.96130E+00_JPRB, 0.69930E+00_JPRB, 0.76582E+00_JPRB, 0.86737E+00_JPRB ,&
&  0.90262E+00_JPRB, 0.94337E+00_JPRB, 0.98386E+00_JPRB, 0.10334E+01_JPRB, 0.10612E+01_JPRB, 0.98289E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.99733E+00_JPRB, 0.92870E+00_JPRB, 0.91422E+00_JPRB, 0.90083E+00_JPRB, 0.90693E+00_JPRB, 0.90353E+00_JPRB, 0.84957E+00_JPRB ,&
&  0.79942E+00_JPRB, 0.91091E+00_JPRB, 0.10301E+01_JPRB, 0.11228E+01_JPRB, 0.10577E+01_JPRB, 0.81088E+00_JPRB, 0.79798E+00_JPRB ,&
&  0.78814E+00_JPRB, 0.74976E+00_JPRB, 0.72683E+00_JPRB, 0.71042E+00_JPRB, 0.69926E+00_JPRB, 0.68445E+00_JPRB, 0.67278E+00_JPRB ,&
&  0.66181E+00_JPRB, 0.64525E+00_JPRB, 0.63221E+00_JPRB, 0.61969E+00_JPRB, 0.60753E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.60638E+00_JPRB, 0.60902E+00_JPRB, 0.62329E+00_JPRB, 0.62330E+00_JPRB, 0.63122E+00_JPRB, 0.63917E+00_JPRB, 0.62357E+00_JPRB ,&
&  0.59377E+00_JPRB, 0.56669E+00_JPRB, 0.57634E+00_JPRB, 0.60902E+00_JPRB, 0.65091E+00_JPRB, 0.71659E+00_JPRB, 0.80200E+00_JPRB ,&
&  0.90914E+00_JPRB, 0.10007E+01_JPRB, 0.11432E+01_JPRB, 0.12688E+01_JPRB, 0.13582E+01_JPRB, 0.14148E+01_JPRB, 0.13985E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.93606E+00_JPRB, 0.13225E+01_JPRB, 0.14004E+01_JPRB, 0.12421E+01_JPRB, 0.11462E+01_JPRB, 0.97982E+00_JPRB ,&
&  0.89782E+00_JPRB, 0.90518E+00_JPRB, 0.89564E+00_JPRB, 0.84812E+00_JPRB, 0.77426E+00_JPRB, 0.77074E+00_JPRB, 0.78821E+00_JPRB ,&
&  0.81880E+00_JPRB, 0.84651E+00_JPRB, 0.88171E+00_JPRB, 0.92451E+00_JPRB, 0.95691E+00_JPRB, 0.10044E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.10550E+01_JPRB, 0.10499E+01_JPRB, 0.10196E+01_JPRB, 0.99458E+00_JPRB, 0.97198E+00_JPRB, 0.95075E+00_JPRB ,&
&  0.95320E+00_JPRB, 0.94769E+00_JPRB, 0.97118E+00_JPRB, 0.98615E+00_JPRB, 0.10149E+01_JPRB, 0.94699E+00_JPRB, 0.95755E+00_JPRB ,&
&  0.93027E+00_JPRB, 0.92263E+00_JPRB, 0.90890E+00_JPRB, 0.93995E+00_JPRB, 0.72717E+00_JPRB, 0.77970E+00_JPRB, 0.78018E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.91816E+00_JPRB, 0.95374E+00_JPRB, 0.98889E+00_JPRB, 0.10238E+01_JPRB, 0.96702E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.10050E+01_JPRB, 0.95192E+00_JPRB, 0.93827E+00_JPRB, 0.88114E+00_JPRB, 0.91919E+00_JPRB, 0.90081E+00_JPRB ,&
&  0.91863E+00_JPRB, 0.93489E+00_JPRB, 0.98431E+00_JPRB, 0.10442E+01_JPRB, 0.99028E+00_JPRB, 0.87917E+00_JPRB, 0.84164E+00_JPRB ,&
&  0.81822E+00_JPRB, 0.79007E+00_JPRB, 0.75200E+00_JPRB, 0.72314E+00_JPRB, 0.70367E+00_JPRB, 0.68475E+00_JPRB, 0.66593E+00_JPRB ,&
&  0.64672E+00_JPRB, 0.63046E+00_JPRB, 0.61886E+00_JPRB, 0.60865E+00_JPRB, 0.60485E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.59042E+00_JPRB, 0.60832E+00_JPRB, 0.63491E+00_JPRB, 0.66483E+00_JPRB, 0.77561E+00_JPRB, 0.90085E+00_JPRB, 0.97216E+00_JPRB ,&
&  0.91995E+00_JPRB, 0.55399E+00_JPRB, 0.54038E+00_JPRB, 0.55894E+00_JPRB, 0.58926E+00_JPRB, 0.64499E+00_JPRB, 0.70891E+00_JPRB ,&
&  0.80093E+00_JPRB, 0.90230E+00_JPRB, 0.10381E+01_JPRB, 0.12050E+01_JPRB, 0.12987E+01_JPRB, 0.13792E+01_JPRB, 0.12729E+01_JPRB ,&
&  0.89001E+00_JPRB, 0.96879E+00_JPRB, 0.14954E+01_JPRB, 0.14312E+01_JPRB, 0.13157E+01_JPRB, 0.12024E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.94902E+00_JPRB, 0.87755E+00_JPRB, 0.85506E+00_JPRB, 0.82237E+00_JPRB, 0.74260E+00_JPRB, 0.71620E+00_JPRB, 0.72042E+00_JPRB ,&
&  0.74507E+00_JPRB, 0.78151E+00_JPRB, 0.82934E+00_JPRB, 0.87976E+00_JPRB, 0.93424E+00_JPRB, 0.96905E+00_JPRB, 0.10060E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.10470E+01_JPRB, 0.10274E+01_JPRB, 0.99704E+00_JPRB, 0.10064E+01_JPRB, 0.10046E+01_JPRB, 0.98831E+00_JPRB ,&
&  0.98478E+00_JPRB, 0.10261E+01_JPRB, 0.10439E+01_JPRB, 0.98117E+00_JPRB, 0.90446E+00_JPRB, 0.97285E+00_JPRB, 0.93755E+00_JPRB ,&
&  0.93945E+00_JPRB, 0.89503E+00_JPRB, 0.96218E+00_JPRB, 0.80936E+00_JPRB, 0.77857E+00_JPRB, 0.82944E+00_JPRB, 0.83295E+00_JPRB ,&
&  0.82826E+00_JPRB, 0.87772E+00_JPRB, 0.91623E+00_JPRB, 0.95063E+00_JPRB, 0.97127E+00_JPRB, 0.91700E+00_JPRB, 0.98286E+00_JPRB ,&
&  0.95514E+00_JPRB, 0.10547E+01_JPRB, 0.10092E+01_JPRB, 0.97812E+00_JPRB, 0.87301E+00_JPRB, 0.87446E+00_JPRB, 0.89406E+00_JPRB ,&
&  0.94029E+00_JPRB, 0.96073E+00_JPRB, 0.97272E+00_JPRB, 0.97408E+00_JPRB, 0.92416E+00_JPRB, 0.90993E+00_JPRB, 0.90165E+00_JPRB ,&
&  0.86521E+00_JPRB, 0.81806E+00_JPRB, 0.78416E+00_JPRB, 0.74511E+00_JPRB, 0.71632E+00_JPRB, 0.68697E+00_JPRB, 0.65602E+00_JPRB ,&
&  0.63197E+00_JPRB, 0.61523E+00_JPRB, 0.60183E+00_JPRB, 0.58773E+00_JPRB, 0.58218E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.56374E+00_JPRB, 0.57708E+00_JPRB, 0.59874E+00_JPRB, 0.63412E+00_JPRB, 0.74360E+00_JPRB, 0.78736E+00_JPRB, 0.78930E+00_JPRB ,&
&  0.76734E+00_JPRB, 0.52702E+00_JPRB, 0.50105E+00_JPRB, 0.51687E+00_JPRB, 0.55059E+00_JPRB, 0.59121E+00_JPRB, 0.65461E+00_JPRB ,&
&  0.71141E+00_JPRB, 0.79855E+00_JPRB, 0.93514E+00_JPRB, 0.10940E+01_JPRB, 0.12090E+01_JPRB, 0.12271E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.79055E+00_JPRB, 0.14880E+01_JPRB, 0.15382E+01_JPRB, 0.15443E+01_JPRB, 0.14841E+01_JPRB, 0.13491E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.98925E+00_JPRB, 0.90937E+00_JPRB, 0.84197E+00_JPRB, 0.79599E+00_JPRB, 0.74352E+00_JPRB, 0.68707E+00_JPRB, 0.66963E+00_JPRB ,&
&  0.68176E+00_JPRB, 0.72219E+00_JPRB, 0.77402E+00_JPRB, 0.82823E+00_JPRB, 0.89329E+00_JPRB, 0.95190E+00_JPRB, 0.98892E+00_JPRB ,&
&  0.10336E+01_JPRB, 0.10432E+01_JPRB, 0.10035E+01_JPRB, 0.10047E+01_JPRB, 0.10160E+01_JPRB, 0.10437E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10080E+01_JPRB, 0.98669E+00_JPRB, 0.10955E+01_JPRB, 0.10904E+01_JPRB, 0.10173E+01_JPRB, 0.10242E+01_JPRB, 0.92100E+00_JPRB ,&
&  0.95521E+00_JPRB, 0.87552E+00_JPRB, 0.90497E+00_JPRB, 0.78024E+00_JPRB, 0.78426E+00_JPRB, 0.84851E+00_JPRB, 0.84321E+00_JPRB ,&
&  0.79416E+00_JPRB, 0.83884E+00_JPRB, 0.85002E+00_JPRB, 0.88820E+00_JPRB, 0.91617E+00_JPRB, 0.91618E+00_JPRB, 0.90457E+00_JPRB ,&
&  0.92817E+00_JPRB, 0.98397E+00_JPRB, 0.10565E+01_JPRB, 0.94519E+00_JPRB, 0.89724E+00_JPRB, 0.85749E+00_JPRB, 0.91099E+00_JPRB ,&
&  0.94969E+00_JPRB, 0.96469E+00_JPRB, 0.10376E+01_JPRB, 0.10375E+01_JPRB, 0.91443E+00_JPRB, 0.90338E+00_JPRB, 0.90504E+00_JPRB ,&
&  0.88289E+00_JPRB, 0.83981E+00_JPRB, 0.80936E+00_JPRB, 0.75697E+00_JPRB, 0.69959E+00_JPRB, 0.66615E+00_JPRB, 0.64098E+00_JPRB ,&
&  0.61942E+00_JPRB, 0.60435E+00_JPRB, 0.58785E+00_JPRB, 0.57053E+00_JPRB, 0.56104E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.54976E+00_JPRB, 0.56512E+00_JPRB, 0.58480E+00_JPRB, 0.61321E+00_JPRB, 0.67965E+00_JPRB, 0.70058E+00_JPRB, 0.64737E+00_JPRB ,&
&  0.58071E+00_JPRB, 0.47536E+00_JPRB, 0.46268E+00_JPRB, 0.47797E+00_JPRB, 0.50657E+00_JPRB, 0.54128E+00_JPRB, 0.60172E+00_JPRB ,&
&  0.67554E+00_JPRB, 0.73912E+00_JPRB, 0.87576E+00_JPRB, 0.99748E+00_JPRB, 0.11205E+01_JPRB, 0.12181E+01_JPRB, 0.83262E+00_JPRB ,&
&  0.86342E+00_JPRB, 0.14630E+01_JPRB, 0.15522E+01_JPRB, 0.15823E+01_JPRB, 0.15990E+01_JPRB, 0.15978E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.10167E+01_JPRB, 0.90061E+00_JPRB, 0.79885E+00_JPRB, 0.75131E+00_JPRB, 0.71141E+00_JPRB, 0.65828E+00_JPRB ,&
&  0.64649E+00_JPRB, 0.67389E+00_JPRB, 0.72623E+00_JPRB, 0.78525E+00_JPRB, 0.85314E+00_JPRB, 0.93033E+00_JPRB, 0.97918E+00_JPRB ,&
&  0.10284E+01_JPRB, 0.10453E+01_JPRB, 0.10354E+01_JPRB, 0.97936E+00_JPRB, 0.10221E+01_JPRB, 0.10587E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.10432E+01_JPRB, 0.10362E+01_JPRB, 0.12230E+01_JPRB, 0.13427E+01_JPRB, 0.12971E+01_JPRB, 0.11702E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.97725E+00_JPRB, 0.90634E+00_JPRB, 0.81836E+00_JPRB, 0.74713E+00_JPRB, 0.78429E+00_JPRB, 0.84368E+00_JPRB, 0.85164E+00_JPRB ,&
&  0.81598E+00_JPRB, 0.81092E+00_JPRB, 0.79772E+00_JPRB, 0.79506E+00_JPRB, 0.81770E+00_JPRB, 0.82885E+00_JPRB, 0.86305E+00_JPRB ,&
&  0.93369E+00_JPRB, 0.10060E+01_JPRB, 0.10282E+01_JPRB, 0.92003E+00_JPRB, 0.88789E+00_JPRB, 0.78794E+00_JPRB, 0.83766E+00_JPRB ,&
&  0.92202E+00_JPRB, 0.92386E+00_JPRB, 0.94686E+00_JPRB, 0.10378E+01_JPRB, 0.95744E+00_JPRB, 0.94854E+00_JPRB, 0.95367E+00_JPRB ,&
&  0.89537E+00_JPRB, 0.83292E+00_JPRB, 0.80102E+00_JPRB, 0.76942E+00_JPRB, 0.71892E+00_JPRB, 0.67192E+00_JPRB, 0.64486E+00_JPRB ,&
&  0.61704E+00_JPRB, 0.60027E+00_JPRB, 0.57991E+00_JPRB, 0.56025E+00_JPRB, 0.54863E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.53531E+00_JPRB, 0.54362E+00_JPRB, 0.56308E+00_JPRB, 0.57996E+00_JPRB, 0.59736E+00_JPRB, 0.60284E+00_JPRB, 0.55266E+00_JPRB ,&
&  0.48729E+00_JPRB, 0.44571E+00_JPRB, 0.42972E+00_JPRB, 0.43479E+00_JPRB, 0.46305E+00_JPRB, 0.50371E+00_JPRB, 0.54384E+00_JPRB ,&
&  0.62861E+00_JPRB, 0.70675E+00_JPRB, 0.81262E+00_JPRB, 0.91306E+00_JPRB, 0.91006E+00_JPRB, 0.78457E+00_JPRB, 0.78922E+00_JPRB ,&
&  0.11603E+01_JPRB, 0.15901E+01_JPRB, 0.15685E+01_JPRB, 0.15770E+01_JPRB, 0.16622E+01_JPRB, 0.17759E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.11327E+01_JPRB, 0.98324E+00_JPRB, 0.87696E+00_JPRB, 0.78796E+00_JPRB, 0.75233E+00_JPRB, 0.69889E+00_JPRB ,&
&  0.64722E+00_JPRB, 0.65736E+00_JPRB, 0.71702E+00_JPRB, 0.77896E+00_JPRB, 0.84606E+00_JPRB, 0.91509E+00_JPRB, 0.98912E+00_JPRB ,&
&  0.10271E+01_JPRB, 0.10335E+01_JPRB, 0.10205E+01_JPRB, 0.98498E+00_JPRB, 0.10168E+01_JPRB, 0.10696E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.13737E+01_JPRB, 0.14511E+01_JPRB, 0.15038E+01_JPRB, 0.14516E+01_JPRB, 0.13780E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.96224E+00_JPRB, 0.77724E+00_JPRB, 0.75569E+00_JPRB, 0.83694E+00_JPRB, 0.87002E+00_JPRB, 0.78462E+00_JPRB ,&
&  0.70993E+00_JPRB, 0.66578E+00_JPRB, 0.66278E+00_JPRB, 0.68650E+00_JPRB, 0.74327E+00_JPRB, 0.77269E+00_JPRB, 0.83831E+00_JPRB ,&
&  0.84622E+00_JPRB, 0.63291E+00_JPRB, 0.58048E+00_JPRB, 0.67743E+00_JPRB, 0.62801E+00_JPRB, 0.61826E+00_JPRB, 0.75816E+00_JPRB ,&
&  0.87727E+00_JPRB, 0.94819E+00_JPRB, 0.96179E+00_JPRB, 0.90667E+00_JPRB, 0.97320E+00_JPRB, 0.10266E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.95075E+00_JPRB, 0.85727E+00_JPRB, 0.80065E+00_JPRB, 0.76670E+00_JPRB, 0.72784E+00_JPRB, 0.68679E+00_JPRB, 0.65903E+00_JPRB ,&
&  0.63182E+00_JPRB, 0.60411E+00_JPRB, 0.58049E+00_JPRB, 0.55661E+00_JPRB, 0.53855E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.52120E+00_JPRB, 0.52078E+00_JPRB, 0.53911E+00_JPRB, 0.55278E+00_JPRB, 0.54581E+00_JPRB, 0.52486E+00_JPRB, 0.49372E+00_JPRB ,&
&  0.46084E+00_JPRB, 0.43484E+00_JPRB, 0.41607E+00_JPRB, 0.41447E+00_JPRB, 0.42822E+00_JPRB, 0.46908E+00_JPRB, 0.51897E+00_JPRB ,&
&  0.59266E+00_JPRB, 0.70184E+00_JPRB, 0.80068E+00_JPRB, 0.87428E+00_JPRB, 0.79429E+00_JPRB, 0.65277E+00_JPRB, 0.75519E+00_JPRB ,&
&  0.10328E+01_JPRB, 0.14706E+01_JPRB, 0.16950E+01_JPRB, 0.17121E+01_JPRB, 0.17112E+01_JPRB, 0.17249E+01_JPRB, 0.16210E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.12227E+01_JPRB, 0.10579E+01_JPRB, 0.94346E+00_JPRB, 0.83903E+00_JPRB, 0.76253E+00_JPRB, 0.73285E+00_JPRB ,&
&  0.68444E+00_JPRB, 0.64703E+00_JPRB, 0.68966E+00_JPRB, 0.76819E+00_JPRB, 0.83162E+00_JPRB, 0.89628E+00_JPRB, 0.96558E+00_JPRB ,&
&  0.99926E+00_JPRB, 0.10188E+01_JPRB, 0.10231E+01_JPRB, 0.10323E+01_JPRB, 0.87288E+00_JPRB, 0.10526E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.12766E+01_JPRB, 0.14446E+01_JPRB, 0.15216E+01_JPRB, 0.17336E+01_JPRB, 0.16240E+01_JPRB, 0.14853E+01_JPRB, 0.13499E+01_JPRB ,&
&  0.12569E+01_JPRB, 0.10746E+01_JPRB, 0.87574E+00_JPRB, 0.85009E+00_JPRB, 0.90044E+00_JPRB, 0.87670E+00_JPRB, 0.63548E+00_JPRB ,&
&  0.58916E+00_JPRB, 0.67491E+00_JPRB, 0.64010E+00_JPRB, 0.59269E+00_JPRB, 0.67684E+00_JPRB, 0.73020E+00_JPRB, 0.61736E+00_JPRB ,&
&  0.41445E+00_JPRB, 0.37025E+00_JPRB, 0.37511E+00_JPRB, 0.38557E+00_JPRB, 0.38334E+00_JPRB, 0.44285E+00_JPRB, 0.77864E+00_JPRB ,&
&  0.83573E+00_JPRB, 0.93743E+00_JPRB, 0.92760E+00_JPRB, 0.88996E+00_JPRB, 0.92784E+00_JPRB, 0.10399E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.98657E+00_JPRB, 0.90804E+00_JPRB, 0.83439E+00_JPRB, 0.78920E+00_JPRB, 0.74888E+00_JPRB, 0.71002E+00_JPRB, 0.67141E+00_JPRB ,&
&  0.64314E+00_JPRB, 0.61156E+00_JPRB, 0.58224E+00_JPRB, 0.55315E+00_JPRB, 0.52994E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.50076E+00_JPRB, 0.49332E+00_JPRB, 0.49953E+00_JPRB, 0.50048E+00_JPRB, 0.50564E+00_JPRB, 0.47848E+00_JPRB, 0.47676E+00_JPRB ,&
&  0.44904E+00_JPRB, 0.43652E+00_JPRB, 0.41016E+00_JPRB, 0.40010E+00_JPRB, 0.40239E+00_JPRB, 0.43890E+00_JPRB, 0.50165E+00_JPRB ,&
&  0.57199E+00_JPRB, 0.68587E+00_JPRB, 0.87671E+00_JPRB, 0.83908E+00_JPRB, 0.81513E+00_JPRB, 0.72912E+00_JPRB, 0.68828E+00_JPRB ,&
&  0.88551E+00_JPRB, 0.12559E+01_JPRB, 0.14734E+01_JPRB, 0.15918E+01_JPRB, 0.18346E+01_JPRB, 0.18898E+01_JPRB, 0.17985E+01_JPRB ,&
&  0.16554E+01_JPRB, 0.14096E+01_JPRB, 0.12131E+01_JPRB, 0.10638E+01_JPRB, 0.92902E+00_JPRB, 0.81696E+00_JPRB, 0.74205E+00_JPRB ,&
&  0.71823E+00_JPRB, 0.67157E+00_JPRB, 0.67251E+00_JPRB, 0.74163E+00_JPRB, 0.81975E+00_JPRB, 0.86630E+00_JPRB, 0.92004E+00_JPRB ,&
&  0.96912E+00_JPRB, 0.98806E+00_JPRB, 0.99695E+00_JPRB, 0.99159E+00_JPRB, 0.85898E+00_JPRB, 0.78915E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.15022E+01_JPRB, 0.17373E+01_JPRB, 0.19779E+01_JPRB, 0.16868E+01_JPRB, 0.15454E+01_JPRB, 0.15000E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.13428E+01_JPRB, 0.97325E+00_JPRB, 0.93126E+00_JPRB, 0.97599E+00_JPRB, 0.74989E+00_JPRB, 0.65050E+00_JPRB ,&
&  0.71016E+00_JPRB, 0.67061E+00_JPRB, 0.60307E+00_JPRB, 0.52222E+00_JPRB, 0.61954E+00_JPRB, 0.60523E+00_JPRB, 0.39541E+00_JPRB ,&
&  0.35046E+00_JPRB, 0.33879E+00_JPRB, 0.32523E+00_JPRB, 0.31633E+00_JPRB, 0.31346E+00_JPRB, 0.35188E+00_JPRB, 0.52413E+00_JPRB ,&
&  0.66831E+00_JPRB, 0.87438E+00_JPRB, 0.91508E+00_JPRB, 0.89609E+00_JPRB, 0.91533E+00_JPRB, 0.10787E+01_JPRB, 0.12952E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.95297E+00_JPRB, 0.85106E+00_JPRB, 0.79502E+00_JPRB, 0.75424E+00_JPRB, 0.72007E+00_JPRB, 0.68598E+00_JPRB ,&
&  0.64821E+00_JPRB, 0.61483E+00_JPRB, 0.58184E+00_JPRB, 0.54491E+00_JPRB, 0.51703E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.47984E+00_JPRB, 0.46638E+00_JPRB, 0.45051E+00_JPRB, 0.45519E+00_JPRB, 0.47588E+00_JPRB, 0.45613E+00_JPRB, 0.45481E+00_JPRB ,&
&  0.44373E+00_JPRB, 0.43340E+00_JPRB, 0.41645E+00_JPRB, 0.40096E+00_JPRB, 0.39416E+00_JPRB, 0.42039E+00_JPRB, 0.46942E+00_JPRB ,&
&  0.54214E+00_JPRB, 0.64871E+00_JPRB, 0.78627E+00_JPRB, 0.68502E+00_JPRB, 0.61003E+00_JPRB, 0.54948E+00_JPRB, 0.51404E+00_JPRB ,&
&  0.75394E+00_JPRB, 0.10166E+01_JPRB, 0.12381E+01_JPRB, 0.14920E+01_JPRB, 0.18204E+01_JPRB, 0.18817E+01_JPRB, 0.20227E+01_JPRB ,&
&  0.19695E+01_JPRB, 0.16656E+01_JPRB, 0.14089E+01_JPRB, 0.11956E+01_JPRB, 0.10378E+01_JPRB, 0.89142E+00_JPRB, 0.78393E+00_JPRB ,&
&  0.72121E+00_JPRB, 0.68808E+00_JPRB, 0.65967E+00_JPRB, 0.70072E+00_JPRB, 0.77672E+00_JPRB, 0.83090E+00_JPRB, 0.87888E+00_JPRB ,&
&  0.91952E+00_JPRB, 0.94382E+00_JPRB, 0.96307E+00_JPRB, 0.98976E+00_JPRB, 0.10372E+01_JPRB, 0.99016E+00_JPRB, 0.90360E+00_JPRB ,&
&  0.90030E+00_JPRB, 0.10702E+01_JPRB, 0.16896E+01_JPRB, 0.18026E+01_JPRB, 0.16312E+01_JPRB, 0.16200E+01_JPRB, 0.15877E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.14826E+01_JPRB, 0.11119E+01_JPRB, 0.11035E+01_JPRB, 0.95538E+00_JPRB, 0.68090E+00_JPRB, 0.72794E+00_JPRB ,&
&  0.77185E+00_JPRB, 0.72126E+00_JPRB, 0.60205E+00_JPRB, 0.48824E+00_JPRB, 0.48902E+00_JPRB, 0.40128E+00_JPRB, 0.32904E+00_JPRB ,&
&  0.31977E+00_JPRB, 0.30947E+00_JPRB, 0.29029E+00_JPRB, 0.29608E+00_JPRB, 0.29379E+00_JPRB, 0.33807E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.62067E+00_JPRB, 0.79259E+00_JPRB, 0.88227E+00_JPRB, 0.88641E+00_JPRB, 0.87176E+00_JPRB, 0.97830E+00_JPRB, 0.11805E+01_JPRB ,&
&  0.98787E+00_JPRB, 0.97309E+00_JPRB, 0.83994E+00_JPRB, 0.78599E+00_JPRB, 0.73892E+00_JPRB, 0.70009E+00_JPRB, 0.65791E+00_JPRB ,&
&  0.61821E+00_JPRB, 0.58923E+00_JPRB, 0.56184E+00_JPRB, 0.52720E+00_JPRB, 0.50028E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.45602E+00_JPRB, 0.43911E+00_JPRB, 0.41613E+00_JPRB, 0.42064E+00_JPRB, 0.44627E+00_JPRB, 0.44118E+00_JPRB, 0.43252E+00_JPRB ,&
&  0.43646E+00_JPRB, 0.42266E+00_JPRB, 0.41182E+00_JPRB, 0.39776E+00_JPRB, 0.38879E+00_JPRB, 0.40448E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.52736E+00_JPRB, 0.65768E+00_JPRB, 0.57158E+00_JPRB, 0.50810E+00_JPRB, 0.50790E+00_JPRB, 0.50573E+00_JPRB, 0.44748E+00_JPRB ,&
&  0.64524E+00_JPRB, 0.81538E+00_JPRB, 0.99375E+00_JPRB, 0.13727E+01_JPRB, 0.20427E+01_JPRB, 0.21919E+01_JPRB, 0.22577E+01_JPRB ,&
&  0.22180E+01_JPRB, 0.20435E+01_JPRB, 0.16908E+01_JPRB, 0.13649E+01_JPRB, 0.11321E+01_JPRB, 0.97567E+00_JPRB, 0.82814E+00_JPRB ,&
&  0.72402E+00_JPRB, 0.68851E+00_JPRB, 0.67695E+00_JPRB, 0.67565E+00_JPRB, 0.71872E+00_JPRB, 0.76009E+00_JPRB, 0.78926E+00_JPRB ,&
&  0.83735E+00_JPRB, 0.87982E+00_JPRB, 0.89339E+00_JPRB, 0.89519E+00_JPRB, 0.86954E+00_JPRB, 0.78162E+00_JPRB, 0.10261E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.12113E+01_JPRB, 0.13019E+01_JPRB, 0.17029E+01_JPRB, 0.17187E+01_JPRB, 0.15714E+01_JPRB, 0.17760E+01_JPRB ,&
&  0.13075E+01_JPRB, 0.11075E+01_JPRB, 0.10357E+01_JPRB, 0.96006E+00_JPRB, 0.76440E+00_JPRB, 0.79534E+00_JPRB, 0.10581E+01_JPRB ,&
&  0.80881E+00_JPRB, 0.85102E+00_JPRB, 0.86333E+00_JPRB, 0.86759E+00_JPRB, 0.45842E+00_JPRB, 0.35688E+00_JPRB, 0.50586E+00_JPRB ,&
&  0.54680E+00_JPRB, 0.40227E+00_JPRB, 0.32584E+00_JPRB, 0.38709E+00_JPRB, 0.32261E+00_JPRB, 0.36101E+00_JPRB, 0.48415E+00_JPRB ,&
&  0.55831E+00_JPRB, 0.64549E+00_JPRB, 0.83491E+00_JPRB, 0.86134E+00_JPRB, 0.83522E+00_JPRB, 0.95295E+00_JPRB, 0.10082E+01_JPRB ,&
&  0.98962E+00_JPRB, 0.97472E+00_JPRB, 0.89990E+00_JPRB, 0.77715E+00_JPRB, 0.69358E+00_JPRB, 0.64844E+00_JPRB, 0.61842E+00_JPRB ,&
&  0.58720E+00_JPRB, 0.55606E+00_JPRB, 0.52632E+00_JPRB, 0.49754E+00_JPRB, 0.47427E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.41735E+00_JPRB, 0.40682E+00_JPRB, 0.38724E+00_JPRB, 0.39132E+00_JPRB, 0.41297E+00_JPRB, 0.42111E+00_JPRB, 0.41801E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.40692E+00_JPRB, 0.40075E+00_JPRB, 0.39106E+00_JPRB, 0.38409E+00_JPRB, 0.39523E+00_JPRB, 0.42270E+00_JPRB ,&
&  0.48218E+00_JPRB, 0.50683E+00_JPRB, 0.44924E+00_JPRB, 0.42008E+00_JPRB, 0.50362E+00_JPRB, 0.47090E+00_JPRB, 0.47896E+00_JPRB ,&
&  0.58612E+00_JPRB, 0.68292E+00_JPRB, 0.82333E+00_JPRB, 0.11239E+01_JPRB, 0.14963E+01_JPRB, 0.20594E+01_JPRB, 0.23337E+01_JPRB ,&
&  0.23502E+01_JPRB, 0.22967E+01_JPRB, 0.19994E+01_JPRB, 0.16362E+01_JPRB, 0.12970E+01_JPRB, 0.10845E+01_JPRB, 0.92661E+00_JPRB ,&
&  0.79383E+00_JPRB, 0.70017E+00_JPRB, 0.67032E+00_JPRB, 0.67521E+00_JPRB, 0.69392E+00_JPRB, 0.71175E+00_JPRB, 0.70932E+00_JPRB ,&
&  0.72673E+00_JPRB, 0.77242E+00_JPRB, 0.79368E+00_JPRB, 0.78544E+00_JPRB, 0.72001E+00_JPRB, 0.62203E+00_JPRB, 0.74007E+00_JPRB ,&
&  0.10285E+01_JPRB, 0.10890E+01_JPRB, 0.12549E+01_JPRB, 0.15744E+01_JPRB, 0.17602E+01_JPRB, 0.16163E+01_JPRB, 0.20304E+01_JPRB ,&
&  0.16391E+01_JPRB, 0.12829E+01_JPRB, 0.10220E+01_JPRB, 0.71742E+00_JPRB, 0.83558E+00_JPRB, 0.11751E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10150E+01_JPRB, 0.96102E+00_JPRB, 0.85070E+00_JPRB, 0.72301E+00_JPRB, 0.53116E+00_JPRB, 0.60036E+00_JPRB ,&
&  0.61630E+00_JPRB, 0.60830E+00_JPRB, 0.57118E+00_JPRB, 0.50669E+00_JPRB, 0.41771E+00_JPRB, 0.47484E+00_JPRB, 0.50851E+00_JPRB ,&
&  0.54435E+00_JPRB, 0.53956E+00_JPRB, 0.63082E+00_JPRB, 0.77258E+00_JPRB, 0.81689E+00_JPRB, 0.86910E+00_JPRB, 0.93105E+00_JPRB ,&
&  0.87010E+00_JPRB, 0.86228E+00_JPRB, 0.76372E+00_JPRB, 0.67365E+00_JPRB, 0.62263E+00_JPRB, 0.59052E+00_JPRB, 0.56914E+00_JPRB ,&
&  0.53740E+00_JPRB, 0.51135E+00_JPRB, 0.48527E+00_JPRB, 0.45355E+00_JPRB, 0.42929E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.38876E+00_JPRB, 0.37966E+00_JPRB, 0.36075E+00_JPRB, 0.36132E+00_JPRB, 0.37809E+00_JPRB, 0.39633E+00_JPRB, 0.40532E+00_JPRB ,&
&  0.40206E+00_JPRB, 0.39859E+00_JPRB, 0.39127E+00_JPRB, 0.38629E+00_JPRB, 0.38004E+00_JPRB, 0.38044E+00_JPRB, 0.40329E+00_JPRB ,&
&  0.45356E+00_JPRB, 0.44624E+00_JPRB, 0.38521E+00_JPRB, 0.42129E+00_JPRB, 0.46586E+00_JPRB, 0.43633E+00_JPRB, 0.48910E+00_JPRB ,&
&  0.54805E+00_JPRB, 0.62125E+00_JPRB, 0.69313E+00_JPRB, 0.86301E+00_JPRB, 0.11093E+01_JPRB, 0.14765E+01_JPRB, 0.18677E+01_JPRB ,&
&  0.20063E+01_JPRB, 0.21310E+01_JPRB, 0.21592E+01_JPRB, 0.19528E+01_JPRB, 0.15786E+01_JPRB, 0.12744E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.93111E+00_JPRB, 0.78264E+00_JPRB, 0.69862E+00_JPRB, 0.67287E+00_JPRB, 0.68232E+00_JPRB, 0.69898E+00_JPRB, 0.71524E+00_JPRB ,&
&  0.68974E+00_JPRB, 0.68165E+00_JPRB, 0.68806E+00_JPRB, 0.75146E+00_JPRB, 0.61154E+00_JPRB, 0.59541E+00_JPRB, 0.67513E+00_JPRB ,&
&  0.11021E+01_JPRB, 0.12826E+01_JPRB, 0.14055E+01_JPRB, 0.18906E+01_JPRB, 0.16921E+01_JPRB, 0.17791E+01_JPRB, 0.21180E+01_JPRB ,&
&  0.23415E+01_JPRB, 0.23136E+01_JPRB, 0.21993E+01_JPRB, 0.12507E+01_JPRB, 0.10088E+01_JPRB, 0.13438E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.10926E+01_JPRB, 0.10019E+01_JPRB, 0.10276E+01_JPRB, 0.83142E+00_JPRB, 0.68077E+00_JPRB, 0.50664E+00_JPRB ,&
&  0.47624E+00_JPRB, 0.44183E+00_JPRB, 0.50166E+00_JPRB, 0.47451E+00_JPRB, 0.46588E+00_JPRB, 0.46769E+00_JPRB, 0.46246E+00_JPRB ,&
&  0.48671E+00_JPRB, 0.49038E+00_JPRB, 0.51030E+00_JPRB, 0.63007E+00_JPRB, 0.73041E+00_JPRB, 0.70981E+00_JPRB, 0.78775E+00_JPRB ,&
&  0.78340E+00_JPRB, 0.72326E+00_JPRB, 0.65202E+00_JPRB, 0.60158E+00_JPRB, 0.57112E+00_JPRB, 0.54475E+00_JPRB, 0.51724E+00_JPRB ,&
&  0.48661E+00_JPRB, 0.46302E+00_JPRB, 0.43934E+00_JPRB, 0.41665E+00_JPRB, 0.39788E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.37642E+00_JPRB, 0.36846E+00_JPRB, 0.34464E+00_JPRB, 0.34171E+00_JPRB, 0.35102E+00_JPRB, 0.36814E+00_JPRB, 0.38193E+00_JPRB ,&
&  0.38706E+00_JPRB, 0.38463E+00_JPRB, 0.38002E+00_JPRB, 0.37805E+00_JPRB, 0.37166E+00_JPRB, 0.37100E+00_JPRB, 0.39544E+00_JPRB ,&
&  0.44040E+00_JPRB, 0.49970E+00_JPRB, 0.42894E+00_JPRB, 0.38381E+00_JPRB, 0.35614E+00_JPRB, 0.41019E+00_JPRB, 0.47296E+00_JPRB ,&
&  0.55595E+00_JPRB, 0.60518E+00_JPRB, 0.68014E+00_JPRB, 0.75745E+00_JPRB, 0.82579E+00_JPRB, 0.10279E+01_JPRB, 0.13797E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.19140E+01_JPRB, 0.20427E+01_JPRB, 0.19137E+01_JPRB, 0.17054E+01_JPRB, 0.14687E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.89422E+00_JPRB, 0.76478E+00_JPRB, 0.68333E+00_JPRB, 0.66055E+00_JPRB, 0.65833E+00_JPRB, 0.68796E+00_JPRB ,&
&  0.69250E+00_JPRB, 0.67213E+00_JPRB, 0.64070E+00_JPRB, 0.64128E+00_JPRB, 0.68846E+00_JPRB, 0.72888E+00_JPRB, 0.86784E+00_JPRB ,&
&  0.10090E+01_JPRB, 0.10792E+01_JPRB, 0.15541E+01_JPRB, 0.19169E+01_JPRB, 0.23676E+01_JPRB, 0.25002E+01_JPRB, 0.23959E+01_JPRB ,&
&  0.24155E+01_JPRB, 0.22506E+01_JPRB, 0.21150E+01_JPRB, 0.15898E+01_JPRB, 0.17304E+01_JPRB, 0.16345E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.12648E+01_JPRB, 0.11340E+01_JPRB, 0.10427E+01_JPRB, 0.10081E+01_JPRB, 0.10095E+01_JPRB, 0.10079E+01_JPRB, 0.75449E+00_JPRB ,&
&  0.56638E+00_JPRB, 0.58902E+00_JPRB, 0.53376E+00_JPRB, 0.46433E+00_JPRB, 0.40914E+00_JPRB, 0.42637E+00_JPRB, 0.44203E+00_JPRB ,&
&  0.45549E+00_JPRB, 0.45409E+00_JPRB, 0.47312E+00_JPRB, 0.56389E+00_JPRB, 0.68098E+00_JPRB, 0.71215E+00_JPRB, 0.72115E+00_JPRB ,&
&  0.66688E+00_JPRB, 0.66058E+00_JPRB, 0.62629E+00_JPRB, 0.55719E+00_JPRB, 0.54283E+00_JPRB, 0.51137E+00_JPRB, 0.47573E+00_JPRB ,&
&  0.44516E+00_JPRB, 0.42691E+00_JPRB, 0.41069E+00_JPRB, 0.39610E+00_JPRB, 0.38088E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.36315E+00_JPRB, 0.36073E+00_JPRB, 0.34289E+00_JPRB, 0.33773E+00_JPRB, 0.34263E+00_JPRB, 0.35250E+00_JPRB, 0.36264E+00_JPRB ,&
&  0.36976E+00_JPRB, 0.36655E+00_JPRB, 0.36231E+00_JPRB, 0.36279E+00_JPRB, 0.36312E+00_JPRB, 0.36800E+00_JPRB, 0.39993E+00_JPRB ,&
&  0.47113E+00_JPRB, 0.62347E+00_JPRB, 0.49869E+00_JPRB, 0.46525E+00_JPRB, 0.40686E+00_JPRB, 0.43508E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.59835E+00_JPRB, 0.60592E+00_JPRB, 0.59584E+00_JPRB, 0.62533E+00_JPRB, 0.67525E+00_JPRB, 0.78715E+00_JPRB, 0.10233E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.15984E+01_JPRB, 0.16994E+01_JPRB, 0.18292E+01_JPRB, 0.16624E+01_JPRB, 0.13827E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.11579E+01_JPRB, 0.98728E+00_JPRB, 0.83907E+00_JPRB, 0.74648E+00_JPRB, 0.69370E+00_JPRB, 0.65812E+00_JPRB, 0.65669E+00_JPRB ,&
&  0.66276E+00_JPRB, 0.65578E+00_JPRB, 0.63930E+00_JPRB, 0.62326E+00_JPRB, 0.66242E+00_JPRB, 0.79845E+00_JPRB, 0.95845E+00_JPRB ,&
&  0.13925E+01_JPRB, 0.15376E+01_JPRB, 0.14230E+01_JPRB, 0.20368E+01_JPRB, 0.28317E+01_JPRB, 0.27254E+01_JPRB, 0.22854E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.21285E+01_JPRB, 0.20442E+01_JPRB, 0.19948E+01_JPRB, 0.18929E+01_JPRB, 0.17365E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.13231E+01_JPRB, 0.11698E+01_JPRB, 0.10534E+01_JPRB, 0.10622E+01_JPRB, 0.98358E+00_JPRB, 0.90197E+00_JPRB, 0.83129E+00_JPRB ,&
&  0.68467E+00_JPRB, 0.56407E+00_JPRB, 0.41376E+00_JPRB, 0.42832E+00_JPRB, 0.35933E+00_JPRB, 0.37017E+00_JPRB, 0.42464E+00_JPRB ,&
&  0.43010E+00_JPRB, 0.46508E+00_JPRB, 0.50060E+00_JPRB, 0.52316E+00_JPRB, 0.62115E+00_JPRB, 0.64747E+00_JPRB, 0.65573E+00_JPRB ,&
&  0.64048E+00_JPRB, 0.56586E+00_JPRB, 0.57891E+00_JPRB, 0.54894E+00_JPRB, 0.53312E+00_JPRB, 0.48202E+00_JPRB, 0.43241E+00_JPRB ,&
&  0.40071E+00_JPRB, 0.38605E+00_JPRB, 0.37715E+00_JPRB, 0.36947E+00_JPRB, 0.36281E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.33608E+00_JPRB, 0.33367E+00_JPRB, 0.32626E+00_JPRB, 0.32807E+00_JPRB, 0.33085E+00_JPRB, 0.33536E+00_JPRB, 0.34556E+00_JPRB ,&
&  0.35662E+00_JPRB, 0.35511E+00_JPRB, 0.34689E+00_JPRB, 0.34464E+00_JPRB, 0.34763E+00_JPRB, 0.35927E+00_JPRB, 0.41008E+00_JPRB ,&
&  0.53100E+00_JPRB, 0.58612E+00_JPRB, 0.48828E+00_JPRB, 0.40399E+00_JPRB, 0.45812E+00_JPRB, 0.44229E+00_JPRB, 0.48033E+00_JPRB ,&
&  0.46428E+00_JPRB, 0.51151E+00_JPRB, 0.49841E+00_JPRB, 0.49106E+00_JPRB, 0.50189E+00_JPRB, 0.57959E+00_JPRB, 0.71457E+00_JPRB ,&
&  0.85818E+00_JPRB, 0.10765E+01_JPRB, 0.12947E+01_JPRB, 0.13105E+01_JPRB, 0.13563E+01_JPRB, 0.12096E+01_JPRB, 0.11772E+01_JPRB ,&
&  0.10716E+01_JPRB, 0.93608E+00_JPRB, 0.81590E+00_JPRB, 0.74044E+00_JPRB, 0.69271E+00_JPRB, 0.66051E+00_JPRB, 0.65021E+00_JPRB ,&
&  0.65501E+00_JPRB, 0.65205E+00_JPRB, 0.63282E+00_JPRB, 0.63375E+00_JPRB, 0.73026E+00_JPRB, 0.10109E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.18103E+01_JPRB, 0.21128E+01_JPRB, 0.23087E+01_JPRB, 0.25968E+01_JPRB, 0.26480E+01_JPRB, 0.25993E+01_JPRB, 0.24218E+01_JPRB ,&
&  0.21613E+01_JPRB, 0.19133E+01_JPRB, 0.18257E+01_JPRB, 0.17406E+01_JPRB, 0.16652E+01_JPRB, 0.16228E+01_JPRB, 0.15016E+01_JPRB ,&
&  0.12898E+01_JPRB, 0.11552E+01_JPRB, 0.10660E+01_JPRB, 0.96136E+00_JPRB, 0.89606E+00_JPRB, 0.81504E+00_JPRB, 0.80065E+00_JPRB ,&
&  0.86179E+00_JPRB, 0.49754E+00_JPRB, 0.39943E+00_JPRB, 0.45208E+00_JPRB, 0.39428E+00_JPRB, 0.41119E+00_JPRB, 0.47878E+00_JPRB ,&
&  0.48740E+00_JPRB, 0.49696E+00_JPRB, 0.54357E+00_JPRB, 0.55535E+00_JPRB, 0.57204E+00_JPRB, 0.62396E+00_JPRB, 0.59869E+00_JPRB ,&
&  0.55873E+00_JPRB, 0.52631E+00_JPRB, 0.54272E+00_JPRB, 0.52543E+00_JPRB, 0.49407E+00_JPRB, 0.42829E+00_JPRB, 0.38262E+00_JPRB ,&
&  0.35991E+00_JPRB, 0.35064E+00_JPRB, 0.34475E+00_JPRB, 0.33910E+00_JPRB, 0.33410E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.31714E+00_JPRB, 0.31367E+00_JPRB, 0.31025E+00_JPRB, 0.31643E+00_JPRB, 0.32503E+00_JPRB, 0.32831E+00_JPRB, 0.33805E+00_JPRB ,&
&  0.34772E+00_JPRB, 0.34881E+00_JPRB, 0.34336E+00_JPRB, 0.33766E+00_JPRB, 0.33793E+00_JPRB, 0.35277E+00_JPRB, 0.40970E+00_JPRB ,&
&  0.38501E+00_JPRB, 0.41471E+00_JPRB, 0.36322E+00_JPRB, 0.37804E+00_JPRB, 0.45448E+00_JPRB, 0.40627E+00_JPRB, 0.39544E+00_JPRB ,&
&  0.38148E+00_JPRB, 0.37460E+00_JPRB, 0.35764E+00_JPRB, 0.35376E+00_JPRB, 0.36405E+00_JPRB, 0.40925E+00_JPRB, 0.50407E+00_JPRB ,&
&  0.56780E+00_JPRB, 0.68703E+00_JPRB, 0.81844E+00_JPRB, 0.83030E+00_JPRB, 0.90040E+00_JPRB, 0.95134E+00_JPRB, 0.98667E+00_JPRB ,&
&  0.94043E+00_JPRB, 0.83063E+00_JPRB, 0.73412E+00_JPRB, 0.67830E+00_JPRB, 0.64138E+00_JPRB, 0.62132E+00_JPRB, 0.60717E+00_JPRB ,&
&  0.59588E+00_JPRB, 0.59923E+00_JPRB, 0.62468E+00_JPRB, 0.68058E+00_JPRB, 0.77108E+00_JPRB, 0.11825E+01_JPRB, 0.18698E+01_JPRB ,&
&  0.21352E+01_JPRB, 0.22751E+01_JPRB, 0.24574E+01_JPRB, 0.27137E+01_JPRB, 0.26390E+01_JPRB, 0.23983E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.17846E+01_JPRB, 0.15809E+01_JPRB, 0.15683E+01_JPRB, 0.15808E+01_JPRB, 0.14676E+01_JPRB, 0.15268E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.11771E+01_JPRB, 0.10913E+01_JPRB, 0.10633E+01_JPRB, 0.99885E+00_JPRB, 0.10004E+01_JPRB, 0.94965E+00_JPRB ,&
&  0.94431E+00_JPRB, 0.80753E+00_JPRB, 0.60553E+00_JPRB, 0.55737E+00_JPRB, 0.44878E+00_JPRB, 0.46560E+00_JPRB, 0.57047E+00_JPRB ,&
&  0.57764E+00_JPRB, 0.57998E+00_JPRB, 0.56522E+00_JPRB, 0.55495E+00_JPRB, 0.53781E+00_JPRB, 0.57155E+00_JPRB, 0.56129E+00_JPRB ,&
&  0.50424E+00_JPRB, 0.51124E+00_JPRB, 0.50573E+00_JPRB, 0.46826E+00_JPRB, 0.42831E+00_JPRB, 0.37344E+00_JPRB, 0.33899E+00_JPRB ,&
&  0.33184E+00_JPRB, 0.32456E+00_JPRB, 0.32333E+00_JPRB, 0.32008E+00_JPRB, 0.31695E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.30509E+00_JPRB, 0.30304E+00_JPRB, 0.30130E+00_JPRB, 0.30581E+00_JPRB, 0.31787E+00_JPRB, 0.32349E+00_JPRB, 0.33335E+00_JPRB ,&
&  0.33987E+00_JPRB, 0.33949E+00_JPRB, 0.34122E+00_JPRB, 0.33423E+00_JPRB, 0.33392E+00_JPRB, 0.34568E+00_JPRB, 0.34669E+00_JPRB ,&
&  0.31630E+00_JPRB, 0.33916E+00_JPRB, 0.32915E+00_JPRB, 0.33002E+00_JPRB, 0.33170E+00_JPRB, 0.31555E+00_JPRB, 0.30217E+00_JPRB ,&
&  0.30961E+00_JPRB, 0.30301E+00_JPRB, 0.30348E+00_JPRB, 0.31273E+00_JPRB, 0.32798E+00_JPRB, 0.36085E+00_JPRB, 0.41964E+00_JPRB ,&
&  0.45311E+00_JPRB, 0.51600E+00_JPRB, 0.57780E+00_JPRB, 0.58451E+00_JPRB, 0.67158E+00_JPRB, 0.74853E+00_JPRB, 0.77080E+00_JPRB ,&
&  0.77175E+00_JPRB, 0.72323E+00_JPRB, 0.66388E+00_JPRB, 0.61701E+00_JPRB, 0.57431E+00_JPRB, 0.55111E+00_JPRB, 0.54185E+00_JPRB ,&
&  0.53606E+00_JPRB, 0.53827E+00_JPRB, 0.55497E+00_JPRB, 0.59026E+00_JPRB, 0.66320E+00_JPRB, 0.97217E+00_JPRB, 0.13292E+01_JPRB ,&
&  0.14760E+01_JPRB, 0.15801E+01_JPRB, 0.16654E+01_JPRB, 0.16911E+01_JPRB, 0.16829E+01_JPRB, 0.15756E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.13635E+01_JPRB, 0.12646E+01_JPRB, 0.14774E+01_JPRB, 0.14621E+01_JPRB, 0.13955E+01_JPRB, 0.13995E+01_JPRB, 0.13348E+01_JPRB ,&
&  0.12946E+01_JPRB, 0.12094E+01_JPRB, 0.11470E+01_JPRB, 0.10951E+01_JPRB, 0.10441E+01_JPRB, 0.10588E+01_JPRB, 0.98705E+00_JPRB ,&
&  0.98030E+00_JPRB, 0.96382E+00_JPRB, 0.82860E+00_JPRB, 0.72287E+00_JPRB, 0.61975E+00_JPRB, 0.64313E+00_JPRB, 0.60909E+00_JPRB ,&
&  0.57125E+00_JPRB, 0.48948E+00_JPRB, 0.51102E+00_JPRB, 0.51759E+00_JPRB, 0.52228E+00_JPRB, 0.50947E+00_JPRB, 0.49225E+00_JPRB ,&
&  0.49046E+00_JPRB, 0.49810E+00_JPRB, 0.44522E+00_JPRB, 0.39701E+00_JPRB, 0.36409E+00_JPRB, 0.33399E+00_JPRB, 0.29979E+00_JPRB ,&
&  0.29157E+00_JPRB, 0.30231E+00_JPRB, 0.30572E+00_JPRB, 0.30693E+00_JPRB, 0.30681E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.29858E+00_JPRB, 0.30009E+00_JPRB, 0.30098E+00_JPRB, 0.30200E+00_JPRB, 0.31243E+00_JPRB, 0.31840E+00_JPRB, 0.31857E+00_JPRB ,&
&  0.33321E+00_JPRB, 0.33109E+00_JPRB, 0.33129E+00_JPRB, 0.32857E+00_JPRB, 0.32704E+00_JPRB, 0.30237E+00_JPRB, 0.25578E+00_JPRB ,&
&  0.24584E+00_JPRB, 0.25819E+00_JPRB, 0.27869E+00_JPRB, 0.28654E+00_JPRB, 0.28174E+00_JPRB, 0.27209E+00_JPRB, 0.26460E+00_JPRB ,&
&  0.26508E+00_JPRB, 0.26900E+00_JPRB, 0.28212E+00_JPRB, 0.29529E+00_JPRB, 0.30303E+00_JPRB, 0.31490E+00_JPRB, 0.34339E+00_JPRB ,&
&  0.36483E+00_JPRB, 0.40176E+00_JPRB, 0.45594E+00_JPRB, 0.43356E+00_JPRB, 0.50978E+00_JPRB, 0.56681E+00_JPRB, 0.56620E+00_JPRB ,&
&  0.58272E+00_JPRB, 0.57046E+00_JPRB, 0.54377E+00_JPRB, 0.53024E+00_JPRB, 0.50374E+00_JPRB, 0.47835E+00_JPRB, 0.46540E+00_JPRB ,&
&  0.46090E+00_JPRB, 0.46361E+00_JPRB, 0.47764E+00_JPRB, 0.49847E+00_JPRB, 0.55046E+00_JPRB, 0.72515E+00_JPRB, 0.86631E+00_JPRB ,&
&  0.92571E+00_JPRB, 0.10156E+01_JPRB, 0.11411E+01_JPRB, 0.11164E+01_JPRB, 0.11512E+01_JPRB, 0.11648E+01_JPRB, 0.11181E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.10876E+01_JPRB, 0.11364E+01_JPRB, 0.11511E+01_JPRB, 0.11727E+01_JPRB, 0.11582E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11647E+01_JPRB, 0.12051E+01_JPRB, 0.10675E+01_JPRB, 0.98871E+00_JPRB, 0.91179E+00_JPRB, 0.89117E+00_JPRB, 0.84595E+00_JPRB ,&
&  0.84718E+00_JPRB, 0.86360E+00_JPRB, 0.84356E+00_JPRB, 0.81077E+00_JPRB, 0.75195E+00_JPRB, 0.68744E+00_JPRB, 0.62740E+00_JPRB ,&
&  0.59116E+00_JPRB, 0.53080E+00_JPRB, 0.50265E+00_JPRB, 0.50532E+00_JPRB, 0.50333E+00_JPRB, 0.45265E+00_JPRB, 0.44811E+00_JPRB ,&
&  0.45198E+00_JPRB, 0.41898E+00_JPRB, 0.39343E+00_JPRB, 0.35863E+00_JPRB, 0.34111E+00_JPRB, 0.32225E+00_JPRB, 0.29791E+00_JPRB ,&
&  0.28018E+00_JPRB, 0.29147E+00_JPRB, 0.29336E+00_JPRB, 0.29318E+00_JPRB, 0.29616E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.29546E+00_JPRB, 0.30149E+00_JPRB, 0.30128E+00_JPRB, 0.29965E+00_JPRB, 0.30458E+00_JPRB, 0.30025E+00_JPRB, 0.30113E+00_JPRB ,&
&  0.31864E+00_JPRB, 0.30834E+00_JPRB, 0.32398E+00_JPRB, 0.27591E+00_JPRB, 0.25197E+00_JPRB, 0.24703E+00_JPRB, 0.23956E+00_JPRB ,&
&  0.24040E+00_JPRB, 0.24760E+00_JPRB, 0.25306E+00_JPRB, 0.25689E+00_JPRB, 0.26008E+00_JPRB, 0.25365E+00_JPRB, 0.24915E+00_JPRB ,&
&  0.25043E+00_JPRB, 0.25851E+00_JPRB, 0.27237E+00_JPRB, 0.27567E+00_JPRB, 0.27499E+00_JPRB, 0.27634E+00_JPRB, 0.28528E+00_JPRB ,&
&  0.28846E+00_JPRB, 0.31051E+00_JPRB, 0.35539E+00_JPRB, 0.37266E+00_JPRB, 0.41156E+00_JPRB, 0.46116E+00_JPRB, 0.45738E+00_JPRB ,&
&  0.45134E+00_JPRB, 0.39742E+00_JPRB, 0.42014E+00_JPRB, 0.42652E+00_JPRB, 0.42623E+00_JPRB, 0.41473E+00_JPRB, 0.40217E+00_JPRB ,&
&  0.39525E+00_JPRB, 0.39630E+00_JPRB, 0.40531E+00_JPRB, 0.42017E+00_JPRB, 0.44652E+00_JPRB, 0.50247E+00_JPRB, 0.55060E+00_JPRB ,&
&  0.62083E+00_JPRB, 0.59344E+00_JPRB, 0.72395E+00_JPRB, 0.75328E+00_JPRB, 0.83300E+00_JPRB, 0.89817E+00_JPRB, 0.95311E+00_JPRB ,&
&  0.10808E+01_JPRB, 0.10284E+01_JPRB, 0.10324E+01_JPRB, 0.98295E+00_JPRB, 0.94696E+00_JPRB, 0.92073E+00_JPRB, 0.91540E+00_JPRB ,&
&  0.93927E+00_JPRB, 0.88220E+00_JPRB, 0.87585E+00_JPRB, 0.80131E+00_JPRB, 0.74987E+00_JPRB, 0.73690E+00_JPRB, 0.72334E+00_JPRB ,&
&  0.72374E+00_JPRB, 0.72418E+00_JPRB, 0.70668E+00_JPRB, 0.62688E+00_JPRB, 0.60287E+00_JPRB, 0.59062E+00_JPRB, 0.55335E+00_JPRB ,&
&  0.55803E+00_JPRB, 0.54757E+00_JPRB, 0.52119E+00_JPRB, 0.53937E+00_JPRB, 0.49483E+00_JPRB, 0.46551E+00_JPRB, 0.45066E+00_JPRB ,&
&  0.43234E+00_JPRB, 0.37524E+00_JPRB, 0.35604E+00_JPRB, 0.34212E+00_JPRB, 0.32350E+00_JPRB, 0.30633E+00_JPRB, 0.30542E+00_JPRB ,&
&  0.28979E+00_JPRB, 0.28743E+00_JPRB, 0.29268E+00_JPRB, 0.29077E+00_JPRB, 0.29105E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.29534E+00_JPRB, 0.30148E+00_JPRB, 0.30096E+00_JPRB, 0.29858E+00_JPRB, 0.29955E+00_JPRB, 0.29380E+00_JPRB, 0.28071E+00_JPRB ,&
&  0.28026E+00_JPRB, 0.26783E+00_JPRB, 0.25412E+00_JPRB, 0.24560E+00_JPRB, 0.24253E+00_JPRB, 0.23869E+00_JPRB, 0.23036E+00_JPRB ,&
&  0.22851E+00_JPRB, 0.23250E+00_JPRB, 0.24012E+00_JPRB, 0.24255E+00_JPRB, 0.24174E+00_JPRB, 0.23948E+00_JPRB, 0.23943E+00_JPRB ,&
&  0.24393E+00_JPRB, 0.25249E+00_JPRB, 0.26166E+00_JPRB, 0.26479E+00_JPRB, 0.26295E+00_JPRB, 0.26356E+00_JPRB, 0.26780E+00_JPRB ,&
&  0.26941E+00_JPRB, 0.27254E+00_JPRB, 0.28760E+00_JPRB, 0.31398E+00_JPRB, 0.36241E+00_JPRB, 0.40422E+00_JPRB, 0.39657E+00_JPRB ,&
&  0.28947E+00_JPRB, 0.27745E+00_JPRB, 0.33354E+00_JPRB, 0.34501E+00_JPRB, 0.35357E+00_JPRB, 0.35071E+00_JPRB, 0.34822E+00_JPRB ,&
&  0.34590E+00_JPRB, 0.34581E+00_JPRB, 0.34519E+00_JPRB, 0.35168E+00_JPRB, 0.36864E+00_JPRB, 0.39279E+00_JPRB, 0.40969E+00_JPRB ,&
&  0.44750E+00_JPRB, 0.43681E+00_JPRB, 0.45002E+00_JPRB, 0.48081E+00_JPRB, 0.62155E+00_JPRB, 0.70621E+00_JPRB, 0.77658E+00_JPRB ,&
&  0.84116E+00_JPRB, 0.88339E+00_JPRB, 0.94576E+00_JPRB, 0.94697E+00_JPRB, 0.91662E+00_JPRB, 0.85876E+00_JPRB, 0.83702E+00_JPRB ,&
&  0.81849E+00_JPRB, 0.79578E+00_JPRB, 0.76396E+00_JPRB, 0.72487E+00_JPRB, 0.67213E+00_JPRB, 0.64807E+00_JPRB, 0.63161E+00_JPRB ,&
&  0.62830E+00_JPRB, 0.62269E+00_JPRB, 0.59077E+00_JPRB, 0.55255E+00_JPRB, 0.52318E+00_JPRB, 0.51967E+00_JPRB, 0.52153E+00_JPRB ,&
&  0.52006E+00_JPRB, 0.52463E+00_JPRB, 0.51872E+00_JPRB, 0.50755E+00_JPRB, 0.47929E+00_JPRB, 0.44764E+00_JPRB, 0.42870E+00_JPRB ,&
&  0.39278E+00_JPRB, 0.35315E+00_JPRB, 0.31955E+00_JPRB, 0.31336E+00_JPRB, 0.31644E+00_JPRB, 0.31084E+00_JPRB, 0.29503E+00_JPRB ,&
&  0.28882E+00_JPRB, 0.27969E+00_JPRB, 0.27374E+00_JPRB, 0.28127E+00_JPRB, 0.28712E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.27971E+00_JPRB, 0.28384E+00_JPRB, 0.28649E+00_JPRB, 0.28875E+00_JPRB, 0.28467E+00_JPRB, 0.27797E+00_JPRB, 0.27006E+00_JPRB ,&
&  0.27015E+00_JPRB, 0.26498E+00_JPRB, 0.25690E+00_JPRB, 0.24551E+00_JPRB, 0.23735E+00_JPRB, 0.23053E+00_JPRB, 0.23012E+00_JPRB ,&
&  0.23364E+00_JPRB, 0.23721E+00_JPRB, 0.23712E+00_JPRB, 0.23378E+00_JPRB, 0.23099E+00_JPRB, 0.23096E+00_JPRB, 0.23510E+00_JPRB ,&
&  0.24343E+00_JPRB, 0.25243E+00_JPRB, 0.25764E+00_JPRB, 0.25938E+00_JPRB, 0.26106E+00_JPRB, 0.26400E+00_JPRB, 0.26466E+00_JPRB ,&
&  0.25979E+00_JPRB, 0.25854E+00_JPRB, 0.26372E+00_JPRB, 0.27956E+00_JPRB, 0.31766E+00_JPRB, 0.34975E+00_JPRB, 0.30652E+00_JPRB ,&
&  0.24022E+00_JPRB, 0.21936E+00_JPRB, 0.22637E+00_JPRB, 0.27213E+00_JPRB, 0.28579E+00_JPRB, 0.29449E+00_JPRB, 0.30308E+00_JPRB ,&
&  0.29507E+00_JPRB, 0.28701E+00_JPRB, 0.29089E+00_JPRB, 0.31127E+00_JPRB, 0.32141E+00_JPRB, 0.33486E+00_JPRB, 0.34903E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.38835E+00_JPRB, 0.40820E+00_JPRB, 0.39064E+00_JPRB, 0.45454E+00_JPRB, 0.60170E+00_JPRB, 0.64785E+00_JPRB ,&
&  0.71007E+00_JPRB, 0.76717E+00_JPRB, 0.87612E+00_JPRB, 0.88367E+00_JPRB, 0.83915E+00_JPRB, 0.80287E+00_JPRB, 0.78725E+00_JPRB ,&
&  0.74861E+00_JPRB, 0.71488E+00_JPRB, 0.68327E+00_JPRB, 0.66559E+00_JPRB, 0.63481E+00_JPRB, 0.60464E+00_JPRB, 0.57607E+00_JPRB ,&
&  0.56571E+00_JPRB, 0.55086E+00_JPRB, 0.52144E+00_JPRB, 0.48698E+00_JPRB, 0.47152E+00_JPRB, 0.46623E+00_JPRB, 0.47323E+00_JPRB ,&
&  0.47769E+00_JPRB, 0.48709E+00_JPRB, 0.49227E+00_JPRB, 0.49698E+00_JPRB, 0.45535E+00_JPRB, 0.40513E+00_JPRB, 0.37259E+00_JPRB ,&
&  0.35766E+00_JPRB, 0.33941E+00_JPRB, 0.31987E+00_JPRB, 0.32261E+00_JPRB, 0.32802E+00_JPRB, 0.32422E+00_JPRB, 0.29981E+00_JPRB ,&
&  0.27687E+00_JPRB, 0.26635E+00_JPRB, 0.26207E+00_JPRB, 0.26875E+00_JPRB, 0.27559E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.25112E+00_JPRB, 0.25794E+00_JPRB, 0.26806E+00_JPRB, 0.27461E+00_JPRB, 0.27695E+00_JPRB, 0.26221E+00_JPRB, 0.24697E+00_JPRB ,&
&  0.23845E+00_JPRB, 0.23557E+00_JPRB, 0.23688E+00_JPRB, 0.23785E+00_JPRB, 0.23848E+00_JPRB, 0.23956E+00_JPRB, 0.24042E+00_JPRB ,&
&  0.23869E+00_JPRB, 0.23547E+00_JPRB, 0.23257E+00_JPRB, 0.23063E+00_JPRB, 0.23082E+00_JPRB, 0.23427E+00_JPRB, 0.24145E+00_JPRB ,&
&  0.25139E+00_JPRB, 0.25804E+00_JPRB, 0.26023E+00_JPRB, 0.25899E+00_JPRB, 0.25924E+00_JPRB, 0.26180E+00_JPRB, 0.26092E+00_JPRB ,&
&  0.25476E+00_JPRB, 0.25357E+00_JPRB, 0.25683E+00_JPRB, 0.26750E+00_JPRB, 0.28825E+00_JPRB, 0.31256E+00_JPRB, 0.27447E+00_JPRB ,&
&  0.22402E+00_JPRB, 0.19561E+00_JPRB, 0.18580E+00_JPRB, 0.19314E+00_JPRB, 0.21578E+00_JPRB, 0.25513E+00_JPRB, 0.26565E+00_JPRB ,&
&  0.27431E+00_JPRB, 0.28429E+00_JPRB, 0.29170E+00_JPRB, 0.29932E+00_JPRB, 0.30552E+00_JPRB, 0.30963E+00_JPRB, 0.31201E+00_JPRB ,&
&  0.31737E+00_JPRB, 0.32865E+00_JPRB, 0.34315E+00_JPRB, 0.33663E+00_JPRB, 0.34340E+00_JPRB, 0.41181E+00_JPRB, 0.49492E+00_JPRB ,&
&  0.57704E+00_JPRB, 0.60532E+00_JPRB, 0.65393E+00_JPRB, 0.71799E+00_JPRB, 0.75836E+00_JPRB, 0.76060E+00_JPRB, 0.74416E+00_JPRB ,&
&  0.71684E+00_JPRB, 0.67009E+00_JPRB, 0.64141E+00_JPRB, 0.62066E+00_JPRB, 0.60852E+00_JPRB, 0.60057E+00_JPRB, 0.57683E+00_JPRB ,&
&  0.55735E+00_JPRB, 0.52928E+00_JPRB, 0.49195E+00_JPRB, 0.46034E+00_JPRB, 0.45426E+00_JPRB, 0.46525E+00_JPRB, 0.48282E+00_JPRB ,&
&  0.48470E+00_JPRB, 0.48459E+00_JPRB, 0.48611E+00_JPRB, 0.49269E+00_JPRB, 0.44702E+00_JPRB, 0.41431E+00_JPRB, 0.39029E+00_JPRB ,&
&  0.38247E+00_JPRB, 0.37745E+00_JPRB, 0.37054E+00_JPRB, 0.37765E+00_JPRB, 0.37059E+00_JPRB, 0.36108E+00_JPRB, 0.34043E+00_JPRB ,&
&  0.31012E+00_JPRB, 0.28496E+00_JPRB, 0.26900E+00_JPRB, 0.25824E+00_JPRB, 0.25349E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.26293E+00_JPRB, 0.25941E+00_JPRB, 0.25930E+00_JPRB, 0.26210E+00_JPRB, 0.26524E+00_JPRB, 0.26046E+00_JPRB, 0.25501E+00_JPRB ,&
&  0.25128E+00_JPRB, 0.25102E+00_JPRB, 0.25019E+00_JPRB, 0.24801E+00_JPRB, 0.24629E+00_JPRB, 0.24453E+00_JPRB, 0.24271E+00_JPRB ,&
&  0.24082E+00_JPRB, 0.23935E+00_JPRB, 0.23830E+00_JPRB, 0.23683E+00_JPRB, 0.23821E+00_JPRB, 0.24394E+00_JPRB, 0.25407E+00_JPRB ,&
&  0.26427E+00_JPRB, 0.26799E+00_JPRB, 0.26799E+00_JPRB, 0.26595E+00_JPRB, 0.26094E+00_JPRB, 0.25665E+00_JPRB, 0.25330E+00_JPRB ,&
&  0.24727E+00_JPRB, 0.24659E+00_JPRB, 0.25304E+00_JPRB, 0.26164E+00_JPRB, 0.27444E+00_JPRB, 0.28338E+00_JPRB, 0.24678E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.18119E+00_JPRB, 0.16885E+00_JPRB, 0.16642E+00_JPRB, 0.17375E+00_JPRB, 0.19029E+00_JPRB, 0.21658E+00_JPRB ,&
&  0.26045E+00_JPRB, 0.28868E+00_JPRB, 0.30005E+00_JPRB, 0.31223E+00_JPRB, 0.31513E+00_JPRB, 0.31406E+00_JPRB, 0.31313E+00_JPRB ,&
&  0.31826E+00_JPRB, 0.32717E+00_JPRB, 0.33882E+00_JPRB, 0.34340E+00_JPRB, 0.35365E+00_JPRB, 0.36581E+00_JPRB, 0.40587E+00_JPRB ,&
&  0.51031E+00_JPRB, 0.53165E+00_JPRB, 0.50558E+00_JPRB, 0.55746E+00_JPRB, 0.63889E+00_JPRB, 0.68777E+00_JPRB, 0.71706E+00_JPRB ,&
&  0.70240E+00_JPRB, 0.65999E+00_JPRB, 0.62664E+00_JPRB, 0.60847E+00_JPRB, 0.59803E+00_JPRB, 0.59224E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.61278E+00_JPRB, 0.64946E+00_JPRB, 0.72769E+00_JPRB, 0.61658E+00_JPRB, 0.56081E+00_JPRB, 0.53273E+00_JPRB, 0.50800E+00_JPRB ,&
&  0.49176E+00_JPRB, 0.48503E+00_JPRB, 0.48804E+00_JPRB, 0.49313E+00_JPRB, 0.46956E+00_JPRB, 0.44643E+00_JPRB, 0.42784E+00_JPRB ,&
&  0.42146E+00_JPRB, 0.41289E+00_JPRB, 0.40656E+00_JPRB, 0.40755E+00_JPRB, 0.40202E+00_JPRB, 0.38928E+00_JPRB, 0.38151E+00_JPRB ,&
&  0.36047E+00_JPRB, 0.32837E+00_JPRB, 0.30842E+00_JPRB, 0.29139E+00_JPRB, 0.27451E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.26705E+00_JPRB, 0.26109E+00_JPRB, 0.25750E+00_JPRB, 0.25615E+00_JPRB, 0.25458E+00_JPRB, 0.25392E+00_JPRB, 0.25168E+00_JPRB ,&
&  0.24941E+00_JPRB, 0.24724E+00_JPRB, 0.24525E+00_JPRB, 0.24339E+00_JPRB, 0.24228E+00_JPRB, 0.24138E+00_JPRB, 0.24118E+00_JPRB ,&
&  0.24140E+00_JPRB, 0.23942E+00_JPRB, 0.24049E+00_JPRB, 0.24525E+00_JPRB, 0.25254E+00_JPRB, 0.26003E+00_JPRB, 0.26895E+00_JPRB ,&
&  0.27246E+00_JPRB, 0.27147E+00_JPRB, 0.26947E+00_JPRB, 0.26480E+00_JPRB, 0.25879E+00_JPRB, 0.25282E+00_JPRB, 0.24932E+00_JPRB ,&
&  0.24939E+00_JPRB, 0.24681E+00_JPRB, 0.25226E+00_JPRB, 0.25621E+00_JPRB, 0.25424E+00_JPRB, 0.26174E+00_JPRB, 0.21883E+00_JPRB ,&
&  0.18686E+00_JPRB, 0.16496E+00_JPRB, 0.15769E+00_JPRB, 0.15466E+00_JPRB, 0.15887E+00_JPRB, 0.16788E+00_JPRB, 0.18868E+00_JPRB ,&
&  0.22460E+00_JPRB, 0.28894E+00_JPRB, 0.30380E+00_JPRB, 0.31023E+00_JPRB, 0.31443E+00_JPRB, 0.31345E+00_JPRB, 0.31343E+00_JPRB ,&
&  0.32062E+00_JPRB, 0.32530E+00_JPRB, 0.33212E+00_JPRB, 0.33769E+00_JPRB, 0.34767E+00_JPRB, 0.35857E+00_JPRB, 0.36592E+00_JPRB ,&
&  0.38541E+00_JPRB, 0.40138E+00_JPRB, 0.42078E+00_JPRB, 0.46502E+00_JPRB, 0.54187E+00_JPRB, 0.61728E+00_JPRB, 0.63890E+00_JPRB ,&
&  0.62451E+00_JPRB, 0.61411E+00_JPRB, 0.59145E+00_JPRB, 0.57917E+00_JPRB, 0.57016E+00_JPRB, 0.56189E+00_JPRB, 0.55968E+00_JPRB ,&
&  0.56666E+00_JPRB, 0.59459E+00_JPRB, 0.65199E+00_JPRB, 0.60383E+00_JPRB, 0.57580E+00_JPRB, 0.54584E+00_JPRB, 0.52011E+00_JPRB ,&
&  0.50090E+00_JPRB, 0.49211E+00_JPRB, 0.48927E+00_JPRB, 0.48352E+00_JPRB, 0.47541E+00_JPRB, 0.46572E+00_JPRB, 0.45971E+00_JPRB ,&
&  0.45016E+00_JPRB, 0.43843E+00_JPRB, 0.42480E+00_JPRB, 0.41865E+00_JPRB, 0.41398E+00_JPRB, 0.41051E+00_JPRB, 0.40542E+00_JPRB ,&
&  0.38335E+00_JPRB, 0.34659E+00_JPRB, 0.31749E+00_JPRB, 0.29543E+00_JPRB, 0.27847E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.28250E+00_JPRB, 0.27275E+00_JPRB, 0.26501E+00_JPRB, 0.26052E+00_JPRB, 0.25638E+00_JPRB, 0.25459E+00_JPRB, 0.25234E+00_JPRB ,&
&  0.25127E+00_JPRB, 0.25017E+00_JPRB, 0.24957E+00_JPRB, 0.24925E+00_JPRB, 0.24955E+00_JPRB, 0.25037E+00_JPRB, 0.25206E+00_JPRB ,&
&  0.25459E+00_JPRB, 0.25828E+00_JPRB, 0.26327E+00_JPRB, 0.26727E+00_JPRB, 0.27185E+00_JPRB, 0.27547E+00_JPRB, 0.27943E+00_JPRB ,&
&  0.28021E+00_JPRB, 0.28068E+00_JPRB, 0.27515E+00_JPRB, 0.26767E+00_JPRB, 0.25989E+00_JPRB, 0.25033E+00_JPRB, 0.25044E+00_JPRB ,&
&  0.24980E+00_JPRB, 0.24731E+00_JPRB, 0.24251E+00_JPRB, 0.24306E+00_JPRB, 0.24255E+00_JPRB, 0.20787E+00_JPRB, 0.19771E+00_JPRB ,&
&  0.18054E+00_JPRB, 0.16342E+00_JPRB, 0.15699E+00_JPRB, 0.15307E+00_JPRB, 0.15397E+00_JPRB, 0.16050E+00_JPRB, 0.17224E+00_JPRB ,&
&  0.19945E+00_JPRB, 0.27693E+00_JPRB, 0.29390E+00_JPRB, 0.30007E+00_JPRB, 0.29347E+00_JPRB, 0.29516E+00_JPRB, 0.29764E+00_JPRB ,&
&  0.30661E+00_JPRB, 0.31364E+00_JPRB, 0.32255E+00_JPRB, 0.32976E+00_JPRB, 0.33843E+00_JPRB, 0.34166E+00_JPRB, 0.34967E+00_JPRB ,&
&  0.36462E+00_JPRB, 0.38773E+00_JPRB, 0.40900E+00_JPRB, 0.43222E+00_JPRB, 0.45804E+00_JPRB, 0.50179E+00_JPRB, 0.54968E+00_JPRB ,&
&  0.55242E+00_JPRB, 0.54616E+00_JPRB, 0.54397E+00_JPRB, 0.53896E+00_JPRB, 0.53660E+00_JPRB, 0.53460E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.54546E+00_JPRB, 0.55787E+00_JPRB, 0.55093E+00_JPRB, 0.52206E+00_JPRB, 0.50704E+00_JPRB, 0.48727E+00_JPRB, 0.48216E+00_JPRB ,&
&  0.47836E+00_JPRB, 0.48541E+00_JPRB, 0.48221E+00_JPRB, 0.47520E+00_JPRB, 0.46864E+00_JPRB, 0.45843E+00_JPRB, 0.45094E+00_JPRB ,&
&  0.44258E+00_JPRB, 0.43338E+00_JPRB, 0.42552E+00_JPRB, 0.42148E+00_JPRB, 0.41845E+00_JPRB, 0.41601E+00_JPRB, 0.41013E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.36324E+00_JPRB, 0.33808E+00_JPRB, 0.31368E+00_JPRB, 0.29533E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.30878E+00_JPRB, 0.29620E+00_JPRB, 0.28612E+00_JPRB, 0.27891E+00_JPRB, 0.27339E+00_JPRB, 0.27013E+00_JPRB, 0.26735E+00_JPRB ,&
&  0.26617E+00_JPRB, 0.26472E+00_JPRB, 0.26450E+00_JPRB, 0.26419E+00_JPRB, 0.26476E+00_JPRB, 0.26591E+00_JPRB, 0.26785E+00_JPRB ,&
&  0.27086E+00_JPRB, 0.27381E+00_JPRB, 0.27761E+00_JPRB, 0.28001E+00_JPRB, 0.28337E+00_JPRB, 0.28552E+00_JPRB, 0.28756E+00_JPRB ,&
&  0.28753E+00_JPRB, 0.28680E+00_JPRB, 0.27893E+00_JPRB, 0.27181E+00_JPRB, 0.25802E+00_JPRB, 0.24799E+00_JPRB, 0.24154E+00_JPRB ,&
&  0.24435E+00_JPRB, 0.22691E+00_JPRB, 0.22620E+00_JPRB, 0.21370E+00_JPRB, 0.20122E+00_JPRB, 0.19001E+00_JPRB, 0.17999E+00_JPRB ,&
&  0.17337E+00_JPRB, 0.16421E+00_JPRB, 0.16012E+00_JPRB, 0.15940E+00_JPRB, 0.16098E+00_JPRB, 0.16921E+00_JPRB, 0.18167E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.26837E+00_JPRB, 0.28736E+00_JPRB, 0.29576E+00_JPRB, 0.29386E+00_JPRB, 0.29654E+00_JPRB, 0.29668E+00_JPRB ,&
&  0.30895E+00_JPRB, 0.32418E+00_JPRB, 0.33777E+00_JPRB, 0.34624E+00_JPRB, 0.35860E+00_JPRB, 0.36528E+00_JPRB, 0.37149E+00_JPRB ,&
&  0.37925E+00_JPRB, 0.39129E+00_JPRB, 0.40215E+00_JPRB, 0.41110E+00_JPRB, 0.42248E+00_JPRB, 0.43236E+00_JPRB, 0.44645E+00_JPRB ,&
&  0.45473E+00_JPRB, 0.47207E+00_JPRB, 0.47720E+00_JPRB, 0.48386E+00_JPRB, 0.48473E+00_JPRB, 0.48268E+00_JPRB, 0.48351E+00_JPRB ,&
&  0.48617E+00_JPRB, 0.48921E+00_JPRB, 0.48924E+00_JPRB, 0.48324E+00_JPRB, 0.47973E+00_JPRB, 0.47041E+00_JPRB, 0.46681E+00_JPRB ,&
&  0.46680E+00_JPRB, 0.46601E+00_JPRB, 0.46414E+00_JPRB, 0.46024E+00_JPRB, 0.45408E+00_JPRB, 0.44631E+00_JPRB, 0.44234E+00_JPRB ,&
&  0.43738E+00_JPRB, 0.43292E+00_JPRB, 0.42910E+00_JPRB, 0.42483E+00_JPRB, 0.42035E+00_JPRB, 0.41443E+00_JPRB, 0.40273E+00_JPRB ,&
&  0.38553E+00_JPRB, 0.36983E+00_JPRB, 0.35391E+00_JPRB, 0.33932E+00_JPRB, 0.32369E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.33624E+00_JPRB, 0.32699E+00_JPRB, 0.31671E+00_JPRB, 0.30791E+00_JPRB, 0.30049E+00_JPRB, 0.29533E+00_JPRB, 0.29089E+00_JPRB ,&
&  0.28776E+00_JPRB, 0.28511E+00_JPRB, 0.28350E+00_JPRB, 0.28253E+00_JPRB, 0.28216E+00_JPRB, 0.28268E+00_JPRB, 0.28349E+00_JPRB ,&
&  0.28552E+00_JPRB, 0.28768E+00_JPRB, 0.29116E+00_JPRB, 0.29439E+00_JPRB, 0.29788E+00_JPRB, 0.29871E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.29655E+00_JPRB, 0.29488E+00_JPRB, 0.28041E+00_JPRB, 0.27516E+00_JPRB, 0.26095E+00_JPRB, 0.25001E+00_JPRB, 0.24515E+00_JPRB ,&
&  0.23807E+00_JPRB, 0.23526E+00_JPRB, 0.22844E+00_JPRB, 0.22121E+00_JPRB, 0.21501E+00_JPRB, 0.20475E+00_JPRB, 0.19745E+00_JPRB ,&
&  0.19043E+00_JPRB, 0.18223E+00_JPRB, 0.17828E+00_JPRB, 0.17682E+00_JPRB, 0.17959E+00_JPRB, 0.18787E+00_JPRB, 0.20108E+00_JPRB ,&
&  0.22351E+00_JPRB, 0.24999E+00_JPRB, 0.28722E+00_JPRB, 0.29945E+00_JPRB, 0.30236E+00_JPRB, 0.31150E+00_JPRB, 0.31907E+00_JPRB ,&
&  0.32403E+00_JPRB, 0.33564E+00_JPRB, 0.34594E+00_JPRB, 0.35074E+00_JPRB, 0.35394E+00_JPRB, 0.35562E+00_JPRB, 0.35687E+00_JPRB ,&
&  0.35851E+00_JPRB, 0.36289E+00_JPRB, 0.36512E+00_JPRB, 0.36649E+00_JPRB, 0.36858E+00_JPRB, 0.37094E+00_JPRB, 0.37722E+00_JPRB ,&
&  0.38242E+00_JPRB, 0.39260E+00_JPRB, 0.40059E+00_JPRB, 0.41358E+00_JPRB, 0.41888E+00_JPRB, 0.42560E+00_JPRB, 0.42749E+00_JPRB ,&
&  0.43010E+00_JPRB, 0.43089E+00_JPRB, 0.42993E+00_JPRB, 0.42397E+00_JPRB, 0.42244E+00_JPRB, 0.42551E+00_JPRB, 0.42516E+00_JPRB ,&
&  0.42430E+00_JPRB, 0.42337E+00_JPRB, 0.42180E+00_JPRB, 0.41979E+00_JPRB, 0.41827E+00_JPRB, 0.41863E+00_JPRB, 0.41868E+00_JPRB ,&
&  0.42135E+00_JPRB, 0.42067E+00_JPRB, 0.41964E+00_JPRB, 0.41514E+00_JPRB, 0.40949E+00_JPRB, 0.39996E+00_JPRB, 0.38740E+00_JPRB ,&
&  0.37638E+00_JPRB, 0.36798E+00_JPRB, 0.35890E+00_JPRB, 0.35214E+00_JPRB, 0.34433E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.33442E+00_JPRB, 0.33122E+00_JPRB, 0.32641E+00_JPRB, 0.32089E+00_JPRB, 0.31624E+00_JPRB, 0.31309E+00_JPRB, 0.30985E+00_JPRB ,&
&  0.30741E+00_JPRB, 0.30558E+00_JPRB, 0.30461E+00_JPRB, 0.30423E+00_JPRB, 0.30399E+00_JPRB, 0.30527E+00_JPRB, 0.30670E+00_JPRB ,&
&  0.30806E+00_JPRB, 0.30886E+00_JPRB, 0.31051E+00_JPRB, 0.31147E+00_JPRB, 0.31168E+00_JPRB, 0.31044E+00_JPRB, 0.30826E+00_JPRB ,&
&  0.30577E+00_JPRB, 0.30251E+00_JPRB, 0.29895E+00_JPRB, 0.29505E+00_JPRB, 0.29115E+00_JPRB, 0.26910E+00_JPRB, 0.26358E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.25981E+00_JPRB, 0.25911E+00_JPRB, 0.25974E+00_JPRB, 0.26476E+00_JPRB, 0.26199E+00_JPRB, 0.25856E+00_JPRB ,&
&  0.24029E+00_JPRB, 0.23531E+00_JPRB, 0.23116E+00_JPRB, 0.22788E+00_JPRB, 0.22862E+00_JPRB, 0.23530E+00_JPRB, 0.24610E+00_JPRB ,&
&  0.26813E+00_JPRB, 0.27796E+00_JPRB, 0.28753E+00_JPRB, 0.29820E+00_JPRB, 0.30729E+00_JPRB, 0.31434E+00_JPRB, 0.32089E+00_JPRB ,&
&  0.32613E+00_JPRB, 0.33107E+00_JPRB, 0.33470E+00_JPRB, 0.33635E+00_JPRB, 0.33635E+00_JPRB, 0.33629E+00_JPRB, 0.33605E+00_JPRB ,&
&  0.33566E+00_JPRB, 0.33490E+00_JPRB, 0.33441E+00_JPRB, 0.33393E+00_JPRB, 0.33296E+00_JPRB, 0.33340E+00_JPRB, 0.33590E+00_JPRB ,&
&  0.33947E+00_JPRB, 0.34464E+00_JPRB, 0.34891E+00_JPRB, 0.35556E+00_JPRB, 0.36200E+00_JPRB, 0.37025E+00_JPRB, 0.37297E+00_JPRB ,&
&  0.37682E+00_JPRB, 0.37949E+00_JPRB, 0.38337E+00_JPRB, 0.38375E+00_JPRB, 0.38420E+00_JPRB, 0.38426E+00_JPRB, 0.38542E+00_JPRB ,&
&  0.38416E+00_JPRB, 0.38342E+00_JPRB, 0.38228E+00_JPRB, 0.38315E+00_JPRB, 0.38165E+00_JPRB, 0.38238E+00_JPRB, 0.38213E+00_JPRB ,&
&  0.38461E+00_JPRB, 0.38179E+00_JPRB, 0.38059E+00_JPRB, 0.37574E+00_JPRB, 0.37158E+00_JPRB, 0.36345E+00_JPRB, 0.35631E+00_JPRB ,&
&  0.35005E+00_JPRB, 0.34565E+00_JPRB, 0.34221E+00_JPRB, 0.33939E+00_JPRB, 0.33677E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.32656E+00_JPRB, 0.32485E+00_JPRB, 0.32304E+00_JPRB, 0.32144E+00_JPRB, 0.31997E+00_JPRB, 0.31895E+00_JPRB, 0.31799E+00_JPRB ,&
&  0.31699E+00_JPRB, 0.31617E+00_JPRB, 0.31539E+00_JPRB, 0.31486E+00_JPRB, 0.31453E+00_JPRB, 0.31477E+00_JPRB, 0.31489E+00_JPRB ,&
&  0.31484E+00_JPRB, 0.31435E+00_JPRB, 0.31395E+00_JPRB, 0.31293E+00_JPRB, 0.31172E+00_JPRB, 0.30995E+00_JPRB, 0.30825E+00_JPRB ,&
&  0.30596E+00_JPRB, 0.30322E+00_JPRB, 0.30088E+00_JPRB, 0.29935E+00_JPRB, 0.29762E+00_JPRB, 0.29532E+00_JPRB, 0.29382E+00_JPRB ,&
&  0.29311E+00_JPRB, 0.29193E+00_JPRB, 0.29091E+00_JPRB, 0.28989E+00_JPRB, 0.28804E+00_JPRB, 0.28632E+00_JPRB, 0.28480E+00_JPRB ,&
&  0.28271E+00_JPRB, 0.28080E+00_JPRB, 0.28026E+00_JPRB, 0.28003E+00_JPRB, 0.28037E+00_JPRB, 0.28168E+00_JPRB, 0.28470E+00_JPRB ,&
&  0.28935E+00_JPRB, 0.29420E+00_JPRB, 0.29929E+00_JPRB, 0.30375E+00_JPRB, 0.30735E+00_JPRB, 0.31019E+00_JPRB, 0.31284E+00_JPRB ,&
&  0.31443E+00_JPRB, 0.31545E+00_JPRB, 0.31582E+00_JPRB, 0.31556E+00_JPRB, 0.31477E+00_JPRB, 0.31433E+00_JPRB, 0.31416E+00_JPRB ,&
&  0.31403E+00_JPRB, 0.31246E+00_JPRB, 0.31166E+00_JPRB, 0.31163E+00_JPRB, 0.31204E+00_JPRB, 0.31285E+00_JPRB, 0.31444E+00_JPRB ,&
&  0.31644E+00_JPRB, 0.31878E+00_JPRB, 0.32105E+00_JPRB, 0.32422E+00_JPRB, 0.32805E+00_JPRB, 0.33247E+00_JPRB, 0.33542E+00_JPRB ,&
&  0.33913E+00_JPRB, 0.34332E+00_JPRB, 0.34775E+00_JPRB, 0.34945E+00_JPRB, 0.35142E+00_JPRB, 0.35324E+00_JPRB, 0.35544E+00_JPRB ,&
&  0.35499E+00_JPRB, 0.35557E+00_JPRB, 0.35623E+00_JPRB, 0.35791E+00_JPRB, 0.35688E+00_JPRB, 0.35681E+00_JPRB, 0.35653E+00_JPRB ,&
&  0.35752E+00_JPRB, 0.35435E+00_JPRB, 0.35236E+00_JPRB, 0.34942E+00_JPRB, 0.34728E+00_JPRB, 0.34279E+00_JPRB, 0.33913E+00_JPRB ,&
&  0.33592E+00_JPRB, 0.33386E+00_JPRB, 0.33160E+00_JPRB, 0.32970E+00_JPRB, 0.32820E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.31904E+00_JPRB, 0.31822E+00_JPRB, 0.31731E+00_JPRB, 0.31638E+00_JPRB, 0.31554E+00_JPRB, 0.31495E+00_JPRB, 0.31449E+00_JPRB ,&
&  0.31386E+00_JPRB, 0.31323E+00_JPRB, 0.31262E+00_JPRB, 0.31205E+00_JPRB, 0.31147E+00_JPRB, 0.31092E+00_JPRB, 0.31034E+00_JPRB ,&
&  0.30972E+00_JPRB, 0.30921E+00_JPRB, 0.30878E+00_JPRB, 0.30814E+00_JPRB, 0.30741E+00_JPRB, 0.30656E+00_JPRB, 0.30565E+00_JPRB ,&
&  0.30462E+00_JPRB, 0.30354E+00_JPRB, 0.30237E+00_JPRB, 0.30109E+00_JPRB, 0.29974E+00_JPRB, 0.29807E+00_JPRB, 0.29665E+00_JPRB ,&
&  0.29560E+00_JPRB, 0.29458E+00_JPRB, 0.29361E+00_JPRB, 0.29266E+00_JPRB, 0.29196E+00_JPRB, 0.29149E+00_JPRB, 0.29097E+00_JPRB ,&
&  0.29063E+00_JPRB, 0.29045E+00_JPRB, 0.29062E+00_JPRB, 0.29094E+00_JPRB, 0.29135E+00_JPRB, 0.29216E+00_JPRB, 0.29339E+00_JPRB ,&
&  0.29445E+00_JPRB, 0.29546E+00_JPRB, 0.29671E+00_JPRB, 0.29804E+00_JPRB, 0.29938E+00_JPRB, 0.30067E+00_JPRB, 0.30187E+00_JPRB ,&
&  0.30274E+00_JPRB, 0.30353E+00_JPRB, 0.30442E+00_JPRB, 0.30520E+00_JPRB, 0.30561E+00_JPRB, 0.30599E+00_JPRB, 0.30636E+00_JPRB ,&
&  0.30661E+00_JPRB, 0.30679E+00_JPRB, 0.30709E+00_JPRB, 0.30758E+00_JPRB, 0.30812E+00_JPRB, 0.30869E+00_JPRB, 0.30946E+00_JPRB ,&
&  0.31041E+00_JPRB, 0.31111E+00_JPRB, 0.31202E+00_JPRB, 0.31314E+00_JPRB, 0.31450E+00_JPRB, 0.31615E+00_JPRB, 0.31763E+00_JPRB ,&
&  0.31914E+00_JPRB, 0.32072E+00_JPRB, 0.32234E+00_JPRB, 0.32362E+00_JPRB, 0.32496E+00_JPRB, 0.32620E+00_JPRB, 0.32735E+00_JPRB ,&
&  0.32784E+00_JPRB, 0.32851E+00_JPRB, 0.32936E+00_JPRB, 0.33017E+00_JPRB, 0.32987E+00_JPRB, 0.32971E+00_JPRB, 0.32955E+00_JPRB ,&
&  0.32989E+00_JPRB, 0.32884E+00_JPRB, 0.32817E+00_JPRB, 0.32726E+00_JPRB, 0.32661E+00_JPRB, 0.32532E+00_JPRB, 0.32432E+00_JPRB ,&
&  0.32326E+00_JPRB, 0.32243E+00_JPRB, 0.32151E+00_JPRB, 0.32059E+00_JPRB, 0.31983E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB ,&
&  0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB, 0.30753E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB, 0.23065E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.25248E+00_JPRB, 0.25506E+00_JPRB, 0.25731E+00_JPRB, 0.25904E+00_JPRB, 0.25961E+00_JPRB, 0.26153E+00_JPRB, 0.26213E+00_JPRB ,&
&  0.26255E+00_JPRB, 0.26259E+00_JPRB, 0.26338E+00_JPRB, 0.26420E+00_JPRB, 0.26475E+00_JPRB, 0.26501E+00_JPRB, 0.26505E+00_JPRB ,&
&  0.26462E+00_JPRB, 0.26372E+00_JPRB, 0.26038E+00_JPRB, 0.25853E+00_JPRB, 0.25750E+00_JPRB, 0.25709E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25557E+00_JPRB, 0.25427E+00_JPRB, 0.25404E+00_JPRB, 0.25323E+00_JPRB, 0.25217E+00_JPRB, 0.25071E+00_JPRB ,&
&  0.24927E+00_JPRB, 0.24811E+00_JPRB, 0.24666E+00_JPRB, 0.24517E+00_JPRB, 0.24332E+00_JPRB, 0.24112E+00_JPRB, 0.23844E+00_JPRB ,&
&  0.23564E+00_JPRB, 0.23278E+00_JPRB, 0.23043E+00_JPRB, 0.22823E+00_JPRB, 0.22634E+00_JPRB, 0.22484E+00_JPRB, 0.22381E+00_JPRB ,&
&  0.22268E+00_JPRB, 0.22141E+00_JPRB, 0.22012E+00_JPRB, 0.21904E+00_JPRB, 0.21798E+00_JPRB, 0.21706E+00_JPRB, 0.21664E+00_JPRB ,&
&  0.21638E+00_JPRB, 0.21607E+00_JPRB, 0.21569E+00_JPRB, 0.21538E+00_JPRB, 0.21495E+00_JPRB, 0.21456E+00_JPRB, 0.21416E+00_JPRB ,&
&  0.21384E+00_JPRB, 0.21363E+00_JPRB, 0.21329E+00_JPRB, 0.21294E+00_JPRB, 0.21272E+00_JPRB, 0.21235E+00_JPRB, 0.21193E+00_JPRB ,&
&  0.21143E+00_JPRB, 0.21112E+00_JPRB, 0.21077E+00_JPRB, 0.21035E+00_JPRB, 0.20996E+00_JPRB, 0.20969E+00_JPRB, 0.20968E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.20952E+00_JPRB, 0.20956E+00_JPRB, 0.21025E+00_JPRB, 0.21096E+00_JPRB, 0.21182E+00_JPRB, 0.21279E+00_JPRB ,&
&  0.21408E+00_JPRB, 0.21542E+00_JPRB, 0.21690E+00_JPRB, 0.21833E+00_JPRB, 0.22020E+00_JPRB, 0.22205E+00_JPRB, 0.22390E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.22747E+00_JPRB, 0.22926E+00_JPRB, 0.23104E+00_JPRB, 0.23333E+00_JPRB, 0.23540E+00_JPRB, 0.23775E+00_JPRB ,&
&  0.23997E+00_JPRB, 0.24239E+00_JPRB, 0.24491E+00_JPRB, 0.24712E+00_JPRB, 0.24943E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.29520E+00_JPRB, 0.30569E+00_JPRB, 0.31471E+00_JPRB, 0.32354E+00_JPRB, 0.32797E+00_JPRB, 0.39265E+00_JPRB, 0.37344E+00_JPRB ,&
&  0.36412E+00_JPRB, 0.33567E+00_JPRB, 0.33629E+00_JPRB, 0.33168E+00_JPRB, 0.32499E+00_JPRB, 0.31721E+00_JPRB, 0.31063E+00_JPRB ,&
&  0.30418E+00_JPRB, 0.29632E+00_JPRB, 0.29118E+00_JPRB, 0.28505E+00_JPRB, 0.27925E+00_JPRB, 0.27353E+00_JPRB, 0.27015E+00_JPRB ,&
&  0.26764E+00_JPRB, 0.26616E+00_JPRB, 0.26504E+00_JPRB, 0.26453E+00_JPRB, 0.26726E+00_JPRB, 0.26876E+00_JPRB, 0.26713E+00_JPRB ,&
&  0.26379E+00_JPRB, 0.26058E+00_JPRB, 0.25644E+00_JPRB, 0.25218E+00_JPRB, 0.24891E+00_JPRB, 0.24605E+00_JPRB, 0.24312E+00_JPRB ,&
&  0.23931E+00_JPRB, 0.23506E+00_JPRB, 0.23086E+00_JPRB, 0.22760E+00_JPRB, 0.22458E+00_JPRB, 0.22155E+00_JPRB, 0.21876E+00_JPRB ,&
&  0.21591E+00_JPRB, 0.21310E+00_JPRB, 0.21028E+00_JPRB, 0.20841E+00_JPRB, 0.20685E+00_JPRB, 0.20538E+00_JPRB, 0.20406E+00_JPRB ,&
&  0.20346E+00_JPRB, 0.20276E+00_JPRB, 0.20189E+00_JPRB, 0.20081E+00_JPRB, 0.19977E+00_JPRB, 0.19863E+00_JPRB, 0.19770E+00_JPRB ,&
&  0.19753E+00_JPRB, 0.19696E+00_JPRB, 0.19636E+00_JPRB, 0.19560E+00_JPRB, 0.19522E+00_JPRB, 0.19447E+00_JPRB, 0.19354E+00_JPRB ,&
&  0.19268E+00_JPRB, 0.19189E+00_JPRB, 0.19168E+00_JPRB, 0.19140E+00_JPRB, 0.19110E+00_JPRB, 0.19089E+00_JPRB, 0.19125E+00_JPRB ,&
&  0.19170E+00_JPRB, 0.19224E+00_JPRB, 0.19315E+00_JPRB, 0.19428E+00_JPRB, 0.19526E+00_JPRB, 0.19623E+00_JPRB, 0.19758E+00_JPRB ,&
&  0.19945E+00_JPRB, 0.20132E+00_JPRB, 0.20339E+00_JPRB, 0.20538E+00_JPRB, 0.20864E+00_JPRB, 0.21214E+00_JPRB, 0.21596E+00_JPRB ,&
&  0.22068E+00_JPRB, 0.22711E+00_JPRB, 0.23438E+00_JPRB, 0.24085E+00_JPRB, 0.24671E+00_JPRB, 0.25367E+00_JPRB, 0.25938E+00_JPRB ,&
&  0.26444E+00_JPRB, 0.26939E+00_JPRB, 0.27537E+00_JPRB, 0.28174E+00_JPRB, 0.28868E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.42189E+00_JPRB, 0.42260E+00_JPRB, 0.42283E+00_JPRB, 0.41917E+00_JPRB, 0.41759E+00_JPRB, 0.41748E+00_JPRB, 0.41469E+00_JPRB ,&
&  0.40683E+00_JPRB, 0.40426E+00_JPRB, 0.39945E+00_JPRB, 0.39263E+00_JPRB, 0.38601E+00_JPRB, 0.37538E+00_JPRB, 0.35921E+00_JPRB ,&
&  0.33746E+00_JPRB, 0.31843E+00_JPRB, 0.30391E+00_JPRB, 0.29075E+00_JPRB, 0.27782E+00_JPRB, 0.26521E+00_JPRB, 0.25980E+00_JPRB ,&
&  0.25525E+00_JPRB, 0.25324E+00_JPRB, 0.25251E+00_JPRB, 0.25612E+00_JPRB, 0.26373E+00_JPRB, 0.27326E+00_JPRB, 0.28362E+00_JPRB ,&
&  0.29564E+00_JPRB, 0.30679E+00_JPRB, 0.30830E+00_JPRB, 0.30136E+00_JPRB, 0.29444E+00_JPRB, 0.29006E+00_JPRB, 0.28671E+00_JPRB ,&
&  0.28282E+00_JPRB, 0.27443E+00_JPRB, 0.26505E+00_JPRB, 0.25530E+00_JPRB, 0.24548E+00_JPRB, 0.23831E+00_JPRB, 0.22989E+00_JPRB ,&
&  0.22376E+00_JPRB, 0.21828E+00_JPRB, 0.21258E+00_JPRB, 0.20771E+00_JPRB, 0.20439E+00_JPRB, 0.20160E+00_JPRB, 0.19883E+00_JPRB ,&
&  0.19733E+00_JPRB, 0.19584E+00_JPRB, 0.19416E+00_JPRB, 0.19248E+00_JPRB, 0.19045E+00_JPRB, 0.18886E+00_JPRB, 0.18739E+00_JPRB ,&
&  0.18626E+00_JPRB, 0.18529E+00_JPRB, 0.18469E+00_JPRB, 0.18409E+00_JPRB, 0.18363E+00_JPRB, 0.18275E+00_JPRB, 0.18169E+00_JPRB ,&
&  0.18096E+00_JPRB, 0.18055E+00_JPRB, 0.17978E+00_JPRB, 0.17932E+00_JPRB, 0.17896E+00_JPRB, 0.17826E+00_JPRB, 0.17864E+00_JPRB ,&
&  0.17965E+00_JPRB, 0.18081E+00_JPRB, 0.18116E+00_JPRB, 0.18222E+00_JPRB, 0.18351E+00_JPRB, 0.18512E+00_JPRB, 0.18755E+00_JPRB ,&
&  0.18945E+00_JPRB, 0.19142E+00_JPRB, 0.19386E+00_JPRB, 0.19604E+00_JPRB, 0.19934E+00_JPRB, 0.20340E+00_JPRB, 0.20839E+00_JPRB ,&
&  0.21519E+00_JPRB, 0.22338E+00_JPRB, 0.23465E+00_JPRB, 0.24832E+00_JPRB, 0.26760E+00_JPRB, 0.27611E+00_JPRB, 0.28464E+00_JPRB ,&
&  0.29225E+00_JPRB, 0.30740E+00_JPRB, 0.38942E+00_JPRB, 0.40453E+00_JPRB, 0.41930E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.42049E+00_JPRB, 0.43261E+00_JPRB, 0.45021E+00_JPRB, 0.46104E+00_JPRB, 0.47554E+00_JPRB, 0.47842E+00_JPRB, 0.47362E+00_JPRB ,&
&  0.45443E+00_JPRB, 0.42411E+00_JPRB, 0.40740E+00_JPRB, 0.38717E+00_JPRB, 0.36594E+00_JPRB, 0.34474E+00_JPRB, 0.33539E+00_JPRB ,&
&  0.32420E+00_JPRB, 0.31181E+00_JPRB, 0.29900E+00_JPRB, 0.28809E+00_JPRB, 0.27764E+00_JPRB, 0.26785E+00_JPRB, 0.26203E+00_JPRB ,&
&  0.25606E+00_JPRB, 0.25528E+00_JPRB, 0.25460E+00_JPRB, 0.25879E+00_JPRB, 0.26982E+00_JPRB, 0.28888E+00_JPRB, 0.31471E+00_JPRB ,&
&  0.35380E+00_JPRB, 0.36048E+00_JPRB, 0.34554E+00_JPRB, 0.33914E+00_JPRB, 0.33187E+00_JPRB, 0.32540E+00_JPRB, 0.31082E+00_JPRB ,&
&  0.31805E+00_JPRB, 0.33156E+00_JPRB, 0.34744E+00_JPRB, 0.33772E+00_JPRB, 0.29280E+00_JPRB, 0.27221E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.23591E+00_JPRB, 0.22562E+00_JPRB, 0.21770E+00_JPRB, 0.21043E+00_JPRB, 0.20344E+00_JPRB, 0.19898E+00_JPRB, 0.19407E+00_JPRB ,&
&  0.19037E+00_JPRB, 0.18864E+00_JPRB, 0.18623E+00_JPRB, 0.18594E+00_JPRB, 0.18561E+00_JPRB, 0.18475E+00_JPRB, 0.18382E+00_JPRB ,&
&  0.18241E+00_JPRB, 0.18091E+00_JPRB, 0.18013E+00_JPRB, 0.18054E+00_JPRB, 0.18143E+00_JPRB, 0.18231E+00_JPRB, 0.18066E+00_JPRB ,&
&  0.17938E+00_JPRB, 0.17810E+00_JPRB, 0.17791E+00_JPRB, 0.17881E+00_JPRB, 0.17907E+00_JPRB, 0.17968E+00_JPRB, 0.17971E+00_JPRB ,&
&  0.17973E+00_JPRB, 0.18017E+00_JPRB, 0.17987E+00_JPRB, 0.18049E+00_JPRB, 0.18080E+00_JPRB, 0.18187E+00_JPRB, 0.18272E+00_JPRB ,&
&  0.18364E+00_JPRB, 0.18553E+00_JPRB, 0.18754E+00_JPRB, 0.19032E+00_JPRB, 0.19412E+00_JPRB, 0.19690E+00_JPRB, 0.20123E+00_JPRB ,&
&  0.20660E+00_JPRB, 0.21508E+00_JPRB, 0.23050E+00_JPRB, 0.24782E+00_JPRB, 0.26901E+00_JPRB, 0.28079E+00_JPRB, 0.29046E+00_JPRB ,&
&  0.30526E+00_JPRB, 0.34051E+00_JPRB, 0.39090E+00_JPRB, 0.40989E+00_JPRB, 0.41392E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.47736E+00_JPRB, 0.48484E+00_JPRB, 0.49628E+00_JPRB, 0.50103E+00_JPRB, 0.50934E+00_JPRB, 0.49773E+00_JPRB, 0.49093E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.43802E+00_JPRB, 0.40678E+00_JPRB, 0.36713E+00_JPRB, 0.34070E+00_JPRB, 0.32015E+00_JPRB, 0.31340E+00_JPRB ,&
&  0.30688E+00_JPRB, 0.30259E+00_JPRB, 0.29877E+00_JPRB, 0.29735E+00_JPRB, 0.29686E+00_JPRB, 0.29828E+00_JPRB, 0.29844E+00_JPRB ,&
&  0.29511E+00_JPRB, 0.28950E+00_JPRB, 0.28301E+00_JPRB, 0.28625E+00_JPRB, 0.29917E+00_JPRB, 0.31426E+00_JPRB, 0.33184E+00_JPRB ,&
&  0.35200E+00_JPRB, 0.36302E+00_JPRB, 0.37320E+00_JPRB, 0.37742E+00_JPRB, 0.36214E+00_JPRB, 0.36467E+00_JPRB, 0.36703E+00_JPRB ,&
&  0.37511E+00_JPRB, 0.38265E+00_JPRB, 0.38015E+00_JPRB, 0.37659E+00_JPRB, 0.32642E+00_JPRB, 0.29298E+00_JPRB, 0.26448E+00_JPRB ,&
&  0.24303E+00_JPRB, 0.22918E+00_JPRB, 0.21817E+00_JPRB, 0.20891E+00_JPRB, 0.19996E+00_JPRB, 0.19408E+00_JPRB, 0.18965E+00_JPRB ,&
&  0.18599E+00_JPRB, 0.18425E+00_JPRB, 0.18254E+00_JPRB, 0.18190E+00_JPRB, 0.18198E+00_JPRB, 0.18128E+00_JPRB, 0.18097E+00_JPRB ,&
&  0.18049E+00_JPRB, 0.17954E+00_JPRB, 0.17922E+00_JPRB, 0.18002E+00_JPRB, 0.18172E+00_JPRB, 0.18465E+00_JPRB, 0.18582E+00_JPRB ,&
&  0.18659E+00_JPRB, 0.18698E+00_JPRB, 0.18698E+00_JPRB, 0.18865E+00_JPRB, 0.18950E+00_JPRB, 0.18966E+00_JPRB, 0.18806E+00_JPRB ,&
&  0.18643E+00_JPRB, 0.18523E+00_JPRB, 0.18492E+00_JPRB, 0.18524E+00_JPRB, 0.18640E+00_JPRB, 0.18833E+00_JPRB, 0.18883E+00_JPRB ,&
&  0.18965E+00_JPRB, 0.18938E+00_JPRB, 0.18920E+00_JPRB, 0.19073E+00_JPRB, 0.19256E+00_JPRB, 0.19461E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19931E+00_JPRB, 0.20596E+00_JPRB, 0.21800E+00_JPRB, 0.23626E+00_JPRB, 0.26321E+00_JPRB, 0.27942E+00_JPRB, 0.31859E+00_JPRB ,&
&  0.38411E+00_JPRB, 0.53036E+00_JPRB, 0.59440E+00_JPRB, 0.44828E+00_JPRB, 0.46225E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.50533E+00_JPRB, 0.51684E+00_JPRB, 0.52962E+00_JPRB, 0.52172E+00_JPRB, 0.50797E+00_JPRB, 0.50027E+00_JPRB, 0.49989E+00_JPRB ,&
&  0.49930E+00_JPRB, 0.49649E+00_JPRB, 0.48422E+00_JPRB, 0.47233E+00_JPRB, 0.45457E+00_JPRB, 0.40606E+00_JPRB, 0.38723E+00_JPRB ,&
&  0.36967E+00_JPRB, 0.37208E+00_JPRB, 0.37701E+00_JPRB, 0.38398E+00_JPRB, 0.39858E+00_JPRB, 0.40981E+00_JPRB, 0.41243E+00_JPRB ,&
&  0.40301E+00_JPRB, 0.38496E+00_JPRB, 0.36783E+00_JPRB, 0.35426E+00_JPRB, 0.35394E+00_JPRB, 0.35434E+00_JPRB, 0.34773E+00_JPRB ,&
&  0.33412E+00_JPRB, 0.32778E+00_JPRB, 0.32726E+00_JPRB, 0.34737E+00_JPRB, 0.40097E+00_JPRB, 0.42139E+00_JPRB, 0.39554E+00_JPRB ,&
&  0.39980E+00_JPRB, 0.39523E+00_JPRB, 0.39143E+00_JPRB, 0.39313E+00_JPRB, 0.38352E+00_JPRB, 0.36596E+00_JPRB, 0.33227E+00_JPRB ,&
&  0.28765E+00_JPRB, 0.26446E+00_JPRB, 0.24352E+00_JPRB, 0.22442E+00_JPRB, 0.21030E+00_JPRB, 0.20196E+00_JPRB, 0.19481E+00_JPRB ,&
&  0.19012E+00_JPRB, 0.18625E+00_JPRB, 0.18363E+00_JPRB, 0.18323E+00_JPRB, 0.18424E+00_JPRB, 0.18463E+00_JPRB, 0.18520E+00_JPRB ,&
&  0.18620E+00_JPRB, 0.18661E+00_JPRB, 0.18674E+00_JPRB, 0.18761E+00_JPRB, 0.18877E+00_JPRB, 0.19103E+00_JPRB, 0.19437E+00_JPRB ,&
&  0.19850E+00_JPRB, 0.20218E+00_JPRB, 0.20579E+00_JPRB, 0.21143E+00_JPRB, 0.21190E+00_JPRB, 0.21083E+00_JPRB, 0.20329E+00_JPRB ,&
&  0.19789E+00_JPRB, 0.19539E+00_JPRB, 0.19287E+00_JPRB, 0.19212E+00_JPRB, 0.19304E+00_JPRB, 0.19469E+00_JPRB, 0.19634E+00_JPRB ,&
&  0.19745E+00_JPRB, 0.19750E+00_JPRB, 0.19682E+00_JPRB, 0.19687E+00_JPRB, 0.19754E+00_JPRB, 0.19774E+00_JPRB, 0.20102E+00_JPRB ,&
&  0.20812E+00_JPRB, 0.21871E+00_JPRB, 0.23049E+00_JPRB, 0.24692E+00_JPRB, 0.26216E+00_JPRB, 0.27844E+00_JPRB, 0.30577E+00_JPRB ,&
&  0.34589E+00_JPRB, 0.45521E+00_JPRB, 0.52529E+00_JPRB, 0.46125E+00_JPRB, 0.48480E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.53585E+00_JPRB, 0.53966E+00_JPRB, 0.54417E+00_JPRB, 0.54679E+00_JPRB, 0.54152E+00_JPRB, 0.53441E+00_JPRB, 0.52980E+00_JPRB ,&
&  0.53491E+00_JPRB, 0.53471E+00_JPRB, 0.51114E+00_JPRB, 0.49254E+00_JPRB, 0.48178E+00_JPRB, 0.47822E+00_JPRB, 0.47502E+00_JPRB ,&
&  0.46897E+00_JPRB, 0.46700E+00_JPRB, 0.46845E+00_JPRB, 0.48003E+00_JPRB, 0.48962E+00_JPRB, 0.49799E+00_JPRB, 0.50553E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.51490E+00_JPRB, 0.53679E+00_JPRB, 0.53635E+00_JPRB, 0.53393E+00_JPRB, 0.52703E+00_JPRB, 0.51288E+00_JPRB ,&
&  0.46556E+00_JPRB, 0.40162E+00_JPRB, 0.33875E+00_JPRB, 0.34964E+00_JPRB, 0.42858E+00_JPRB, 0.43917E+00_JPRB, 0.42279E+00_JPRB ,&
&  0.43300E+00_JPRB, 0.42931E+00_JPRB, 0.41161E+00_JPRB, 0.39877E+00_JPRB, 0.38344E+00_JPRB, 0.37344E+00_JPRB, 0.36659E+00_JPRB ,&
&  0.36385E+00_JPRB, 0.36227E+00_JPRB, 0.34824E+00_JPRB, 0.30079E+00_JPRB, 0.27234E+00_JPRB, 0.25529E+00_JPRB, 0.23787E+00_JPRB ,&
&  0.22438E+00_JPRB, 0.21550E+00_JPRB, 0.21369E+00_JPRB, 0.21385E+00_JPRB, 0.21797E+00_JPRB, 0.21939E+00_JPRB, 0.22072E+00_JPRB ,&
&  0.22012E+00_JPRB, 0.21883E+00_JPRB, 0.21473E+00_JPRB, 0.21017E+00_JPRB, 0.20703E+00_JPRB, 0.20639E+00_JPRB, 0.20617E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.21266E+00_JPRB, 0.22865E+00_JPRB, 0.24606E+00_JPRB, 0.24637E+00_JPRB, 0.23116E+00_JPRB, 0.21600E+00_JPRB ,&
&  0.20666E+00_JPRB, 0.20184E+00_JPRB, 0.19884E+00_JPRB, 0.19829E+00_JPRB, 0.20083E+00_JPRB, 0.20436E+00_JPRB, 0.20533E+00_JPRB ,&
&  0.20527E+00_JPRB, 0.20605E+00_JPRB, 0.20624E+00_JPRB, 0.20682E+00_JPRB, 0.20758E+00_JPRB, 0.21001E+00_JPRB, 0.21451E+00_JPRB ,&
&  0.21988E+00_JPRB, 0.22950E+00_JPRB, 0.24051E+00_JPRB, 0.25533E+00_JPRB, 0.26610E+00_JPRB, 0.27380E+00_JPRB, 0.28313E+00_JPRB ,&
&  0.30012E+00_JPRB, 0.33769E+00_JPRB, 0.40656E+00_JPRB, 0.53336E+00_JPRB, 0.52892E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.59173E+00_JPRB, 0.58019E+00_JPRB, 0.57254E+00_JPRB, 0.56720E+00_JPRB, 0.56067E+00_JPRB, 0.55969E+00_JPRB, 0.55919E+00_JPRB ,&
&  0.55354E+00_JPRB, 0.54522E+00_JPRB, 0.54599E+00_JPRB, 0.52852E+00_JPRB, 0.50377E+00_JPRB, 0.48721E+00_JPRB, 0.48496E+00_JPRB ,&
&  0.48899E+00_JPRB, 0.49631E+00_JPRB, 0.50562E+00_JPRB, 0.51486E+00_JPRB, 0.52000E+00_JPRB, 0.53184E+00_JPRB, 0.55292E+00_JPRB ,&
&  0.57751E+00_JPRB, 0.59143E+00_JPRB, 0.59587E+00_JPRB, 0.58816E+00_JPRB, 0.57587E+00_JPRB, 0.56305E+00_JPRB, 0.55236E+00_JPRB ,&
&  0.54504E+00_JPRB, 0.44294E+00_JPRB, 0.37376E+00_JPRB, 0.37961E+00_JPRB, 0.45103E+00_JPRB, 0.43520E+00_JPRB, 0.43273E+00_JPRB ,&
&  0.44521E+00_JPRB, 0.44599E+00_JPRB, 0.43317E+00_JPRB, 0.41641E+00_JPRB, 0.40059E+00_JPRB, 0.38901E+00_JPRB, 0.38029E+00_JPRB ,&
&  0.37603E+00_JPRB, 0.38239E+00_JPRB, 0.39076E+00_JPRB, 0.40377E+00_JPRB, 0.40741E+00_JPRB, 0.39515E+00_JPRB, 0.38039E+00_JPRB ,&
&  0.36605E+00_JPRB, 0.36099E+00_JPRB, 0.36316E+00_JPRB, 0.36903E+00_JPRB, 0.37676E+00_JPRB, 0.38282E+00_JPRB, 0.38095E+00_JPRB ,&
&  0.35168E+00_JPRB, 0.32894E+00_JPRB, 0.30091E+00_JPRB, 0.27454E+00_JPRB, 0.25156E+00_JPRB, 0.23602E+00_JPRB, 0.22817E+00_JPRB ,&
&  0.22603E+00_JPRB, 0.23260E+00_JPRB, 0.25348E+00_JPRB, 0.30228E+00_JPRB, 0.37188E+00_JPRB, 0.29330E+00_JPRB, 0.25397E+00_JPRB ,&
&  0.23205E+00_JPRB, 0.21911E+00_JPRB, 0.21428E+00_JPRB, 0.21483E+00_JPRB, 0.21850E+00_JPRB, 0.22403E+00_JPRB, 0.22836E+00_JPRB ,&
&  0.23183E+00_JPRB, 0.23526E+00_JPRB, 0.23760E+00_JPRB, 0.23881E+00_JPRB, 0.24052E+00_JPRB, 0.24162E+00_JPRB, 0.24058E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.24424E+00_JPRB, 0.26079E+00_JPRB, 0.28714E+00_JPRB, 0.31877E+00_JPRB, 0.35282E+00_JPRB, 0.40434E+00_JPRB ,&
&  0.47000E+00_JPRB, 0.54652E+00_JPRB, 0.56351E+00_JPRB, 0.57503E+00_JPRB, 0.58515E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.63092E+00_JPRB, 0.60571E+00_JPRB, 0.58889E+00_JPRB, 0.57561E+00_JPRB, 0.55736E+00_JPRB, 0.54353E+00_JPRB, 0.53799E+00_JPRB ,&
&  0.54627E+00_JPRB, 0.54510E+00_JPRB, 0.53541E+00_JPRB, 0.52723E+00_JPRB, 0.51125E+00_JPRB, 0.49693E+00_JPRB, 0.49108E+00_JPRB ,&
&  0.49981E+00_JPRB, 0.51634E+00_JPRB, 0.53650E+00_JPRB, 0.54519E+00_JPRB, 0.54487E+00_JPRB, 0.55340E+00_JPRB, 0.57263E+00_JPRB ,&
&  0.60105E+00_JPRB, 0.62090E+00_JPRB, 0.62822E+00_JPRB, 0.61552E+00_JPRB, 0.59832E+00_JPRB, 0.59031E+00_JPRB, 0.57587E+00_JPRB ,&
&  0.55914E+00_JPRB, 0.56292E+00_JPRB, 0.49491E+00_JPRB, 0.48743E+00_JPRB, 0.46394E+00_JPRB, 0.44978E+00_JPRB, 0.44630E+00_JPRB ,&
&  0.45974E+00_JPRB, 0.46424E+00_JPRB, 0.45645E+00_JPRB, 0.44485E+00_JPRB, 0.43659E+00_JPRB, 0.42528E+00_JPRB, 0.41460E+00_JPRB ,&
&  0.40690E+00_JPRB, 0.40921E+00_JPRB, 0.41820E+00_JPRB, 0.42749E+00_JPRB, 0.43866E+00_JPRB, 0.44929E+00_JPRB, 0.44571E+00_JPRB ,&
&  0.43404E+00_JPRB, 0.43190E+00_JPRB, 0.43847E+00_JPRB, 0.45251E+00_JPRB, 0.46786E+00_JPRB, 0.47519E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.47225E+00_JPRB, 0.45573E+00_JPRB, 0.44149E+00_JPRB, 0.42970E+00_JPRB, 0.40490E+00_JPRB, 0.34123E+00_JPRB, 0.31823E+00_JPRB ,&
&  0.32859E+00_JPRB, 0.38768E+00_JPRB, 0.45874E+00_JPRB, 0.44964E+00_JPRB, 0.44430E+00_JPRB, 0.43029E+00_JPRB, 0.39484E+00_JPRB ,&
&  0.35508E+00_JPRB, 0.29334E+00_JPRB, 0.27099E+00_JPRB, 0.26704E+00_JPRB, 0.26571E+00_JPRB, 0.27050E+00_JPRB, 0.28230E+00_JPRB ,&
&  0.29803E+00_JPRB, 0.32561E+00_JPRB, 0.34335E+00_JPRB, 0.35029E+00_JPRB, 0.34908E+00_JPRB, 0.33303E+00_JPRB, 0.31934E+00_JPRB ,&
&  0.30984E+00_JPRB, 0.33404E+00_JPRB, 0.39175E+00_JPRB, 0.47457E+00_JPRB, 0.47239E+00_JPRB, 0.49579E+00_JPRB, 0.51996E+00_JPRB ,&
&  0.54304E+00_JPRB, 0.58215E+00_JPRB, 0.60817E+00_JPRB, 0.61403E+00_JPRB, 0.63241E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.65438E+00_JPRB, 0.62712E+00_JPRB, 0.59840E+00_JPRB, 0.57357E+00_JPRB, 0.56131E+00_JPRB, 0.54347E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.52774E+00_JPRB, 0.53571E+00_JPRB, 0.53131E+00_JPRB, 0.52559E+00_JPRB, 0.51631E+00_JPRB, 0.50862E+00_JPRB, 0.49940E+00_JPRB ,&
&  0.49559E+00_JPRB, 0.50756E+00_JPRB, 0.53359E+00_JPRB, 0.54683E+00_JPRB, 0.55340E+00_JPRB, 0.58413E+00_JPRB, 0.61542E+00_JPRB ,&
&  0.62430E+00_JPRB, 0.63802E+00_JPRB, 0.64238E+00_JPRB, 0.61780E+00_JPRB, 0.60174E+00_JPRB, 0.59553E+00_JPRB, 0.59940E+00_JPRB ,&
&  0.59459E+00_JPRB, 0.59884E+00_JPRB, 0.59098E+00_JPRB, 0.52029E+00_JPRB, 0.51959E+00_JPRB, 0.48885E+00_JPRB, 0.47859E+00_JPRB ,&
&  0.47383E+00_JPRB, 0.47974E+00_JPRB, 0.47668E+00_JPRB, 0.46829E+00_JPRB, 0.46500E+00_JPRB, 0.46815E+00_JPRB, 0.46908E+00_JPRB ,&
&  0.46400E+00_JPRB, 0.45301E+00_JPRB, 0.45732E+00_JPRB, 0.46374E+00_JPRB, 0.46753E+00_JPRB, 0.47463E+00_JPRB, 0.48514E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.46608E+00_JPRB, 0.45751E+00_JPRB, 0.45761E+00_JPRB, 0.46392E+00_JPRB, 0.47832E+00_JPRB, 0.48353E+00_JPRB ,&
&  0.48154E+00_JPRB, 0.48025E+00_JPRB, 0.48110E+00_JPRB, 0.48730E+00_JPRB, 0.50235E+00_JPRB, 0.51746E+00_JPRB, 0.51877E+00_JPRB ,&
&  0.51975E+00_JPRB, 0.52410E+00_JPRB, 0.52858E+00_JPRB, 0.53073E+00_JPRB, 0.52701E+00_JPRB, 0.51202E+00_JPRB, 0.49455E+00_JPRB ,&
&  0.47798E+00_JPRB, 0.46167E+00_JPRB, 0.44766E+00_JPRB, 0.44291E+00_JPRB, 0.45559E+00_JPRB, 0.48437E+00_JPRB, 0.50033E+00_JPRB ,&
&  0.50395E+00_JPRB, 0.50593E+00_JPRB, 0.50958E+00_JPRB, 0.51107E+00_JPRB, 0.51344E+00_JPRB, 0.51181E+00_JPRB, 0.50560E+00_JPRB ,&
&  0.49450E+00_JPRB, 0.48300E+00_JPRB, 0.47952E+00_JPRB, 0.48290E+00_JPRB, 0.48716E+00_JPRB, 0.49817E+00_JPRB, 0.51382E+00_JPRB ,&
&  0.53514E+00_JPRB, 0.57296E+00_JPRB, 0.60518E+00_JPRB, 0.63029E+00_JPRB, 0.64858E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.67217E+00_JPRB, 0.67020E+00_JPRB, 0.64164E+00_JPRB, 0.61008E+00_JPRB, 0.59103E+00_JPRB, 0.56982E+00_JPRB, 0.55274E+00_JPRB ,&
&  0.55122E+00_JPRB, 0.54868E+00_JPRB, 0.54685E+00_JPRB, 0.54464E+00_JPRB, 0.54303E+00_JPRB, 0.53989E+00_JPRB, 0.52777E+00_JPRB ,&
&  0.52345E+00_JPRB, 0.53019E+00_JPRB, 0.55274E+00_JPRB, 0.57834E+00_JPRB, 0.59896E+00_JPRB, 0.62553E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.64797E+00_JPRB, 0.66019E+00_JPRB, 0.66260E+00_JPRB, 0.63039E+00_JPRB, 0.60387E+00_JPRB, 0.60510E+00_JPRB, 0.61610E+00_JPRB ,&
&  0.61639E+00_JPRB, 0.61962E+00_JPRB, 0.62352E+00_JPRB, 0.61965E+00_JPRB, 0.59874E+00_JPRB, 0.56061E+00_JPRB, 0.53080E+00_JPRB ,&
&  0.50659E+00_JPRB, 0.48796E+00_JPRB, 0.48271E+00_JPRB, 0.47212E+00_JPRB, 0.47112E+00_JPRB, 0.48133E+00_JPRB, 0.49393E+00_JPRB ,&
&  0.50681E+00_JPRB, 0.51240E+00_JPRB, 0.51928E+00_JPRB, 0.51600E+00_JPRB, 0.51156E+00_JPRB, 0.51405E+00_JPRB, 0.51368E+00_JPRB ,&
&  0.51273E+00_JPRB, 0.50358E+00_JPRB, 0.50097E+00_JPRB, 0.49407E+00_JPRB, 0.49022E+00_JPRB, 0.49079E+00_JPRB, 0.49393E+00_JPRB ,&
&  0.49515E+00_JPRB, 0.50053E+00_JPRB, 0.50210E+00_JPRB, 0.50415E+00_JPRB, 0.51349E+00_JPRB, 0.53004E+00_JPRB, 0.54671E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.56902E+00_JPRB, 0.57687E+00_JPRB, 0.58270E+00_JPRB, 0.58249E+00_JPRB, 0.57795E+00_JPRB, 0.56415E+00_JPRB ,&
&  0.53354E+00_JPRB, 0.50066E+00_JPRB, 0.47581E+00_JPRB, 0.45587E+00_JPRB, 0.45186E+00_JPRB, 0.46241E+00_JPRB, 0.47993E+00_JPRB ,&
&  0.49318E+00_JPRB, 0.49735E+00_JPRB, 0.49166E+00_JPRB, 0.49016E+00_JPRB, 0.49553E+00_JPRB, 0.50527E+00_JPRB, 0.51281E+00_JPRB ,&
&  0.51208E+00_JPRB, 0.50874E+00_JPRB, 0.51209E+00_JPRB, 0.51838E+00_JPRB, 0.53157E+00_JPRB, 0.54847E+00_JPRB, 0.56685E+00_JPRB ,&
&  0.58543E+00_JPRB, 0.60007E+00_JPRB, 0.61922E+00_JPRB, 0.63876E+00_JPRB, 0.66421E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.67725E+00_JPRB, 0.66902E+00_JPRB, 0.65455E+00_JPRB, 0.62840E+00_JPRB, 0.61137E+00_JPRB, 0.58436E+00_JPRB, 0.56894E+00_JPRB ,&
&  0.57106E+00_JPRB, 0.57218E+00_JPRB, 0.57722E+00_JPRB, 0.58673E+00_JPRB, 0.58494E+00_JPRB, 0.58055E+00_JPRB, 0.57632E+00_JPRB ,&
&  0.57970E+00_JPRB, 0.58026E+00_JPRB, 0.58882E+00_JPRB, 0.60455E+00_JPRB, 0.62112E+00_JPRB, 0.63034E+00_JPRB, 0.63976E+00_JPRB ,&
&  0.64696E+00_JPRB, 0.64066E+00_JPRB, 0.63640E+00_JPRB, 0.60637E+00_JPRB, 0.58897E+00_JPRB, 0.59426E+00_JPRB, 0.60161E+00_JPRB ,&
&  0.61167E+00_JPRB, 0.62227E+00_JPRB, 0.63291E+00_JPRB, 0.63832E+00_JPRB, 0.62450E+00_JPRB, 0.62101E+00_JPRB, 0.61053E+00_JPRB ,&
&  0.56868E+00_JPRB, 0.52759E+00_JPRB, 0.50552E+00_JPRB, 0.49863E+00_JPRB, 0.49433E+00_JPRB, 0.50054E+00_JPRB, 0.50756E+00_JPRB ,&
&  0.51999E+00_JPRB, 0.53146E+00_JPRB, 0.53449E+00_JPRB, 0.53791E+00_JPRB, 0.53951E+00_JPRB, 0.53854E+00_JPRB, 0.53856E+00_JPRB ,&
&  0.53526E+00_JPRB, 0.53304E+00_JPRB, 0.54003E+00_JPRB, 0.54063E+00_JPRB, 0.53209E+00_JPRB, 0.52993E+00_JPRB, 0.52828E+00_JPRB ,&
&  0.52591E+00_JPRB, 0.52345E+00_JPRB, 0.52039E+00_JPRB, 0.53033E+00_JPRB, 0.55558E+00_JPRB, 0.58467E+00_JPRB, 0.60994E+00_JPRB ,&
&  0.62422E+00_JPRB, 0.63575E+00_JPRB, 0.63882E+00_JPRB, 0.64094E+00_JPRB, 0.64051E+00_JPRB, 0.63192E+00_JPRB, 0.62886E+00_JPRB ,&
&  0.62108E+00_JPRB, 0.60210E+00_JPRB, 0.57672E+00_JPRB, 0.53723E+00_JPRB, 0.49888E+00_JPRB, 0.47703E+00_JPRB, 0.47662E+00_JPRB ,&
&  0.48873E+00_JPRB, 0.50108E+00_JPRB, 0.50746E+00_JPRB, 0.51566E+00_JPRB, 0.53063E+00_JPRB, 0.54853E+00_JPRB, 0.56721E+00_JPRB ,&
&  0.57877E+00_JPRB, 0.57846E+00_JPRB, 0.57550E+00_JPRB, 0.58297E+00_JPRB, 0.59321E+00_JPRB, 0.61038E+00_JPRB, 0.62933E+00_JPRB ,&
&  0.64613E+00_JPRB, 0.64570E+00_JPRB, 0.64613E+00_JPRB, 0.64733E+00_JPRB, 0.66690E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.68057E+00_JPRB, 0.68165E+00_JPRB, 0.66632E+00_JPRB, 0.65535E+00_JPRB, 0.63923E+00_JPRB, 0.61837E+00_JPRB, 0.59974E+00_JPRB ,&
&  0.60089E+00_JPRB, 0.61558E+00_JPRB, 0.62984E+00_JPRB, 0.64034E+00_JPRB, 0.65588E+00_JPRB, 0.65921E+00_JPRB, 0.66612E+00_JPRB ,&
&  0.66257E+00_JPRB, 0.62592E+00_JPRB, 0.62025E+00_JPRB, 0.62810E+00_JPRB, 0.63002E+00_JPRB, 0.63567E+00_JPRB, 0.64599E+00_JPRB ,&
&  0.65557E+00_JPRB, 0.65072E+00_JPRB, 0.64256E+00_JPRB, 0.61319E+00_JPRB, 0.60653E+00_JPRB, 0.61143E+00_JPRB, 0.61630E+00_JPRB ,&
&  0.62654E+00_JPRB, 0.64305E+00_JPRB, 0.64786E+00_JPRB, 0.65585E+00_JPRB, 0.66534E+00_JPRB, 0.66997E+00_JPRB, 0.66686E+00_JPRB ,&
&  0.64160E+00_JPRB, 0.61610E+00_JPRB, 0.60578E+00_JPRB, 0.58882E+00_JPRB, 0.56754E+00_JPRB, 0.55253E+00_JPRB, 0.54448E+00_JPRB ,&
&  0.53996E+00_JPRB, 0.54478E+00_JPRB, 0.55094E+00_JPRB, 0.56485E+00_JPRB, 0.56999E+00_JPRB, 0.57408E+00_JPRB, 0.57918E+00_JPRB ,&
&  0.57835E+00_JPRB, 0.57377E+00_JPRB, 0.57346E+00_JPRB, 0.58317E+00_JPRB, 0.58931E+00_JPRB, 0.58745E+00_JPRB, 0.58348E+00_JPRB ,&
&  0.57106E+00_JPRB, 0.55715E+00_JPRB, 0.56441E+00_JPRB, 0.57913E+00_JPRB, 0.58567E+00_JPRB, 0.61042E+00_JPRB, 0.64257E+00_JPRB ,&
&  0.66981E+00_JPRB, 0.68775E+00_JPRB, 0.68745E+00_JPRB, 0.68432E+00_JPRB, 0.68312E+00_JPRB, 0.69059E+00_JPRB, 0.70307E+00_JPRB ,&
&  0.68749E+00_JPRB, 0.67843E+00_JPRB, 0.67649E+00_JPRB, 0.65754E+00_JPRB, 0.62166E+00_JPRB, 0.59116E+00_JPRB, 0.56852E+00_JPRB ,&
&  0.55032E+00_JPRB, 0.53985E+00_JPRB, 0.54338E+00_JPRB, 0.54716E+00_JPRB, 0.56027E+00_JPRB, 0.59585E+00_JPRB, 0.63270E+00_JPRB ,&
&  0.67257E+00_JPRB, 0.69293E+00_JPRB, 0.69902E+00_JPRB, 0.70661E+00_JPRB, 0.71199E+00_JPRB, 0.70073E+00_JPRB, 0.69131E+00_JPRB ,&
&  0.68853E+00_JPRB, 0.68019E+00_JPRB, 0.67088E+00_JPRB, 0.66099E+00_JPRB, 0.66791E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.67519E+00_JPRB, 0.69655E+00_JPRB, 0.66359E+00_JPRB, 0.64549E+00_JPRB, 0.62534E+00_JPRB, 0.62116E+00_JPRB, 0.61209E+00_JPRB ,&
&  0.62219E+00_JPRB, 0.63875E+00_JPRB, 0.66233E+00_JPRB, 0.67029E+00_JPRB, 0.67726E+00_JPRB, 0.68025E+00_JPRB, 0.69763E+00_JPRB ,&
&  0.69347E+00_JPRB, 0.68560E+00_JPRB, 0.68904E+00_JPRB, 0.67892E+00_JPRB, 0.67028E+00_JPRB, 0.67485E+00_JPRB, 0.68314E+00_JPRB ,&
&  0.69280E+00_JPRB, 0.68408E+00_JPRB, 0.66682E+00_JPRB, 0.64028E+00_JPRB, 0.62770E+00_JPRB, 0.62509E+00_JPRB, 0.63247E+00_JPRB ,&
&  0.63779E+00_JPRB, 0.66118E+00_JPRB, 0.60028E+00_JPRB, 0.65591E+00_JPRB, 0.66546E+00_JPRB, 0.66523E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.66906E+00_JPRB, 0.65888E+00_JPRB, 0.65000E+00_JPRB, 0.64145E+00_JPRB, 0.61920E+00_JPRB, 0.60126E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.56454E+00_JPRB, 0.55403E+00_JPRB, 0.55897E+00_JPRB, 0.57603E+00_JPRB, 0.59353E+00_JPRB, 0.60346E+00_JPRB, 0.60961E+00_JPRB ,&
&  0.61291E+00_JPRB, 0.60557E+00_JPRB, 0.59561E+00_JPRB, 0.59578E+00_JPRB, 0.60252E+00_JPRB, 0.60781E+00_JPRB, 0.61155E+00_JPRB ,&
&  0.61271E+00_JPRB, 0.58963E+00_JPRB, 0.57655E+00_JPRB, 0.59257E+00_JPRB, 0.60954E+00_JPRB, 0.62782E+00_JPRB, 0.65175E+00_JPRB ,&
&  0.66978E+00_JPRB, 0.67666E+00_JPRB, 0.67561E+00_JPRB, 0.68240E+00_JPRB, 0.68947E+00_JPRB, 0.69809E+00_JPRB, 0.70501E+00_JPRB ,&
&  0.69560E+00_JPRB, 0.69389E+00_JPRB, 0.69495E+00_JPRB, 0.69520E+00_JPRB, 0.68679E+00_JPRB, 0.66461E+00_JPRB, 0.63310E+00_JPRB ,&
&  0.61869E+00_JPRB, 0.60376E+00_JPRB, 0.59260E+00_JPRB, 0.59003E+00_JPRB, 0.59269E+00_JPRB, 0.61852E+00_JPRB, 0.65509E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.72071E+00_JPRB, 0.74597E+00_JPRB, 0.76904E+00_JPRB, 0.76665E+00_JPRB, 0.74453E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.71957E+00_JPRB, 0.70342E+00_JPRB, 0.69271E+00_JPRB, 0.68395E+00_JPRB, 0.67391E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.71287E+00_JPRB, 0.71366E+00_JPRB, 0.70323E+00_JPRB, 0.66481E+00_JPRB, 0.63577E+00_JPRB, 0.64559E+00_JPRB, 0.65499E+00_JPRB ,&
&  0.65440E+00_JPRB, 0.66896E+00_JPRB, 0.69408E+00_JPRB, 0.71966E+00_JPRB, 0.73685E+00_JPRB, 0.73704E+00_JPRB, 0.74151E+00_JPRB ,&
&  0.73952E+00_JPRB, 0.73777E+00_JPRB, 0.74799E+00_JPRB, 0.75299E+00_JPRB, 0.74244E+00_JPRB, 0.72136E+00_JPRB, 0.71856E+00_JPRB ,&
&  0.72431E+00_JPRB, 0.72006E+00_JPRB, 0.70172E+00_JPRB, 0.68920E+00_JPRB, 0.67909E+00_JPRB, 0.68419E+00_JPRB, 0.69036E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.63306E+00_JPRB, 0.67118E+00_JPRB, 0.68289E+00_JPRB, 0.69173E+00_JPRB, 0.68256E+00_JPRB, 0.68795E+00_JPRB ,&
&  0.69963E+00_JPRB, 0.70694E+00_JPRB, 0.68702E+00_JPRB, 0.66330E+00_JPRB, 0.64204E+00_JPRB, 0.62455E+00_JPRB, 0.59688E+00_JPRB ,&
&  0.58632E+00_JPRB, 0.58329E+00_JPRB, 0.59638E+00_JPRB, 0.61021E+00_JPRB, 0.62620E+00_JPRB, 0.63268E+00_JPRB, 0.64299E+00_JPRB ,&
&  0.64539E+00_JPRB, 0.64025E+00_JPRB, 0.63563E+00_JPRB, 0.63242E+00_JPRB, 0.63235E+00_JPRB, 0.62697E+00_JPRB, 0.61911E+00_JPRB ,&
&  0.61336E+00_JPRB, 0.61179E+00_JPRB, 0.61612E+00_JPRB, 0.62539E+00_JPRB, 0.63660E+00_JPRB, 0.65337E+00_JPRB, 0.66609E+00_JPRB ,&
&  0.68204E+00_JPRB, 0.68678E+00_JPRB, 0.69103E+00_JPRB, 0.70392E+00_JPRB, 0.71250E+00_JPRB, 0.71111E+00_JPRB, 0.70831E+00_JPRB ,&
&  0.70599E+00_JPRB, 0.70093E+00_JPRB, 0.70596E+00_JPRB, 0.70546E+00_JPRB, 0.70644E+00_JPRB, 0.69867E+00_JPRB, 0.68425E+00_JPRB ,&
&  0.67888E+00_JPRB, 0.67474E+00_JPRB, 0.66619E+00_JPRB, 0.66354E+00_JPRB, 0.66147E+00_JPRB, 0.67891E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.72576E+00_JPRB, 0.74779E+00_JPRB, 0.77958E+00_JPRB, 0.80041E+00_JPRB, 0.79924E+00_JPRB, 0.79801E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.76657E+00_JPRB, 0.75262E+00_JPRB, 0.74000E+00_JPRB, 0.73601E+00_JPRB, 0.73262E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.74679E+00_JPRB, 0.74010E+00_JPRB, 0.72546E+00_JPRB, 0.69875E+00_JPRB, 0.66514E+00_JPRB, 0.67404E+00_JPRB, 0.68693E+00_JPRB ,&
&  0.68146E+00_JPRB, 0.67236E+00_JPRB, 0.68276E+00_JPRB, 0.70772E+00_JPRB, 0.73262E+00_JPRB, 0.74722E+00_JPRB, 0.75034E+00_JPRB ,&
&  0.74627E+00_JPRB, 0.74916E+00_JPRB, 0.75808E+00_JPRB, 0.77532E+00_JPRB, 0.77441E+00_JPRB, 0.75636E+00_JPRB, 0.74511E+00_JPRB ,&
&  0.74166E+00_JPRB, 0.73959E+00_JPRB, 0.73656E+00_JPRB, 0.71959E+00_JPRB, 0.70828E+00_JPRB, 0.70554E+00_JPRB, 0.70617E+00_JPRB ,&
&  0.70399E+00_JPRB, 0.56651E+00_JPRB, 0.66459E+00_JPRB, 0.67575E+00_JPRB, 0.68816E+00_JPRB, 0.68610E+00_JPRB, 0.68804E+00_JPRB ,&
&  0.69374E+00_JPRB, 0.70983E+00_JPRB, 0.70241E+00_JPRB, 0.67621E+00_JPRB, 0.64840E+00_JPRB, 0.62019E+00_JPRB, 0.60119E+00_JPRB ,&
&  0.59189E+00_JPRB, 0.59475E+00_JPRB, 0.60449E+00_JPRB, 0.61478E+00_JPRB, 0.62587E+00_JPRB, 0.62771E+00_JPRB, 0.63177E+00_JPRB ,&
&  0.63483E+00_JPRB, 0.64066E+00_JPRB, 0.64409E+00_JPRB, 0.65089E+00_JPRB, 0.65413E+00_JPRB, 0.65179E+00_JPRB, 0.64521E+00_JPRB ,&
&  0.63582E+00_JPRB, 0.62999E+00_JPRB, 0.62955E+00_JPRB, 0.63197E+00_JPRB, 0.63883E+00_JPRB, 0.65029E+00_JPRB, 0.66132E+00_JPRB ,&
&  0.67411E+00_JPRB, 0.68240E+00_JPRB, 0.68683E+00_JPRB, 0.68920E+00_JPRB, 0.69857E+00_JPRB, 0.70847E+00_JPRB, 0.70873E+00_JPRB ,&
&  0.70298E+00_JPRB, 0.69655E+00_JPRB, 0.69536E+00_JPRB, 0.69307E+00_JPRB, 0.69429E+00_JPRB, 0.69575E+00_JPRB, 0.69217E+00_JPRB ,&
&  0.68441E+00_JPRB, 0.68278E+00_JPRB, 0.68403E+00_JPRB, 0.67334E+00_JPRB, 0.67474E+00_JPRB, 0.68440E+00_JPRB, 0.70246E+00_JPRB ,&
&  0.72498E+00_JPRB, 0.75490E+00_JPRB, 0.78459E+00_JPRB, 0.81076E+00_JPRB, 0.82676E+00_JPRB, 0.83272E+00_JPRB, 0.82797E+00_JPRB ,&
&  0.81084E+00_JPRB, 0.78580E+00_JPRB, 0.76714E+00_JPRB, 0.75801E+00_JPRB, 0.76024E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.77549E+00_JPRB, 0.76590E+00_JPRB, 0.75298E+00_JPRB, 0.73574E+00_JPRB, 0.72047E+00_JPRB, 0.71252E+00_JPRB, 0.71871E+00_JPRB ,&
&  0.71796E+00_JPRB, 0.70633E+00_JPRB, 0.71591E+00_JPRB, 0.72918E+00_JPRB, 0.74040E+00_JPRB, 0.75129E+00_JPRB, 0.75635E+00_JPRB ,&
&  0.76189E+00_JPRB, 0.76054E+00_JPRB, 0.76475E+00_JPRB, 0.77727E+00_JPRB, 0.79578E+00_JPRB, 0.79726E+00_JPRB, 0.79420E+00_JPRB ,&
&  0.77138E+00_JPRB, 0.75963E+00_JPRB, 0.74893E+00_JPRB, 0.73606E+00_JPRB, 0.72484E+00_JPRB, 0.72167E+00_JPRB, 0.72524E+00_JPRB ,&
&  0.61438E+00_JPRB, 0.51908E+00_JPRB, 0.62006E+00_JPRB, 0.70732E+00_JPRB, 0.71431E+00_JPRB, 0.71913E+00_JPRB, 0.73648E+00_JPRB ,&
&  0.74287E+00_JPRB, 0.74543E+00_JPRB, 0.74311E+00_JPRB, 0.72983E+00_JPRB, 0.69786E+00_JPRB, 0.66506E+00_JPRB, 0.64631E+00_JPRB ,&
&  0.63445E+00_JPRB, 0.63874E+00_JPRB, 0.64044E+00_JPRB, 0.64767E+00_JPRB, 0.64804E+00_JPRB, 0.64119E+00_JPRB, 0.64033E+00_JPRB ,&
&  0.64209E+00_JPRB, 0.65325E+00_JPRB, 0.66424E+00_JPRB, 0.67270E+00_JPRB, 0.67660E+00_JPRB, 0.67848E+00_JPRB, 0.67681E+00_JPRB ,&
&  0.67578E+00_JPRB, 0.67432E+00_JPRB, 0.67082E+00_JPRB, 0.67175E+00_JPRB, 0.67368E+00_JPRB, 0.67954E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.69441E+00_JPRB, 0.69388E+00_JPRB, 0.69447E+00_JPRB, 0.69563E+00_JPRB, 0.70594E+00_JPRB, 0.71450E+00_JPRB, 0.71292E+00_JPRB ,&
&  0.70894E+00_JPRB, 0.70695E+00_JPRB, 0.71377E+00_JPRB, 0.71931E+00_JPRB, 0.72607E+00_JPRB, 0.72921E+00_JPRB, 0.72517E+00_JPRB ,&
&  0.71667E+00_JPRB, 0.70272E+00_JPRB, 0.69239E+00_JPRB, 0.69127E+00_JPRB, 0.69983E+00_JPRB, 0.70593E+00_JPRB, 0.72127E+00_JPRB ,&
&  0.75038E+00_JPRB, 0.78661E+00_JPRB, 0.81748E+00_JPRB, 0.85656E+00_JPRB, 0.87982E+00_JPRB, 0.89174E+00_JPRB, 0.88716E+00_JPRB ,&
&  0.86984E+00_JPRB, 0.83073E+00_JPRB, 0.81896E+00_JPRB, 0.79691E+00_JPRB, 0.78557E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.75752E+00_JPRB, 0.76190E+00_JPRB, 0.76434E+00_JPRB, 0.75389E+00_JPRB, 0.74028E+00_JPRB, 0.73515E+00_JPRB, 0.74148E+00_JPRB ,&
&  0.73715E+00_JPRB, 0.72380E+00_JPRB, 0.72883E+00_JPRB, 0.73338E+00_JPRB, 0.74187E+00_JPRB, 0.74710E+00_JPRB, 0.75329E+00_JPRB ,&
&  0.75962E+00_JPRB, 0.76520E+00_JPRB, 0.76446E+00_JPRB, 0.77355E+00_JPRB, 0.76672E+00_JPRB, 0.76148E+00_JPRB, 0.75866E+00_JPRB ,&
&  0.74983E+00_JPRB, 0.74046E+00_JPRB, 0.73290E+00_JPRB, 0.72286E+00_JPRB, 0.71597E+00_JPRB, 0.70828E+00_JPRB, 0.69359E+00_JPRB ,&
&  0.70677E+00_JPRB, 0.55383E+00_JPRB, 0.62147E+00_JPRB, 0.73633E+00_JPRB, 0.74200E+00_JPRB, 0.74380E+00_JPRB, 0.75782E+00_JPRB ,&
&  0.77763E+00_JPRB, 0.78878E+00_JPRB, 0.77978E+00_JPRB, 0.75670E+00_JPRB, 0.71561E+00_JPRB, 0.69385E+00_JPRB, 0.68012E+00_JPRB ,&
&  0.66530E+00_JPRB, 0.65502E+00_JPRB, 0.64930E+00_JPRB, 0.65131E+00_JPRB, 0.65134E+00_JPRB, 0.64840E+00_JPRB, 0.64629E+00_JPRB ,&
&  0.65123E+00_JPRB, 0.66246E+00_JPRB, 0.67038E+00_JPRB, 0.67811E+00_JPRB, 0.68558E+00_JPRB, 0.68413E+00_JPRB, 0.68557E+00_JPRB ,&
&  0.68572E+00_JPRB, 0.68628E+00_JPRB, 0.68454E+00_JPRB, 0.68323E+00_JPRB, 0.68253E+00_JPRB, 0.69578E+00_JPRB, 0.70611E+00_JPRB ,&
&  0.70499E+00_JPRB, 0.70262E+00_JPRB, 0.70594E+00_JPRB, 0.71190E+00_JPRB, 0.71697E+00_JPRB, 0.72044E+00_JPRB, 0.71728E+00_JPRB ,&
&  0.71429E+00_JPRB, 0.70877E+00_JPRB, 0.70896E+00_JPRB, 0.70950E+00_JPRB, 0.70585E+00_JPRB, 0.70529E+00_JPRB, 0.70884E+00_JPRB ,&
&  0.70367E+00_JPRB, 0.69251E+00_JPRB, 0.69452E+00_JPRB, 0.70737E+00_JPRB, 0.71733E+00_JPRB, 0.72206E+00_JPRB, 0.73485E+00_JPRB ,&
&  0.76340E+00_JPRB, 0.79612E+00_JPRB, 0.82249E+00_JPRB, 0.85861E+00_JPRB, 0.87894E+00_JPRB, 0.88989E+00_JPRB, 0.88973E+00_JPRB ,&
&  0.87506E+00_JPRB, 0.85552E+00_JPRB, 0.61851E+00_JPRB, 0.72388E+00_JPRB, 0.74803E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.76262E+00_JPRB, 0.77645E+00_JPRB, 0.77449E+00_JPRB, 0.76585E+00_JPRB, 0.75892E+00_JPRB, 0.75426E+00_JPRB, 0.75739E+00_JPRB ,&
&  0.75484E+00_JPRB, 0.74847E+00_JPRB, 0.75141E+00_JPRB, 0.75518E+00_JPRB, 0.75980E+00_JPRB, 0.76021E+00_JPRB, 0.76805E+00_JPRB ,&
&  0.77140E+00_JPRB, 0.77498E+00_JPRB, 0.77977E+00_JPRB, 0.77484E+00_JPRB, 0.75640E+00_JPRB, 0.74340E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.72837E+00_JPRB, 0.72800E+00_JPRB, 0.72852E+00_JPRB, 0.72236E+00_JPRB, 0.70367E+00_JPRB, 0.68806E+00_JPRB, 0.67830E+00_JPRB ,&
&  0.67925E+00_JPRB, 0.53385E+00_JPRB, 0.58374E+00_JPRB, 0.74457E+00_JPRB, 0.81695E+00_JPRB, 0.80964E+00_JPRB, 0.81177E+00_JPRB ,&
&  0.82480E+00_JPRB, 0.82344E+00_JPRB, 0.80221E+00_JPRB, 0.77347E+00_JPRB, 0.73771E+00_JPRB, 0.72854E+00_JPRB, 0.71278E+00_JPRB ,&
&  0.69607E+00_JPRB, 0.68994E+00_JPRB, 0.68507E+00_JPRB, 0.68617E+00_JPRB, 0.67829E+00_JPRB, 0.66879E+00_JPRB, 0.67194E+00_JPRB ,&
&  0.68188E+00_JPRB, 0.68333E+00_JPRB, 0.68907E+00_JPRB, 0.69558E+00_JPRB, 0.69355E+00_JPRB, 0.71138E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.72211E+00_JPRB, 0.72079E+00_JPRB, 0.71978E+00_JPRB, 0.71656E+00_JPRB, 0.71908E+00_JPRB, 0.73685E+00_JPRB, 0.74867E+00_JPRB ,&
&  0.74143E+00_JPRB, 0.73133E+00_JPRB, 0.73857E+00_JPRB, 0.74212E+00_JPRB, 0.73630E+00_JPRB, 0.72422E+00_JPRB, 0.71029E+00_JPRB ,&
&  0.70635E+00_JPRB, 0.70437E+00_JPRB, 0.70429E+00_JPRB, 0.70074E+00_JPRB, 0.69436E+00_JPRB, 0.68942E+00_JPRB, 0.68760E+00_JPRB ,&
&  0.68729E+00_JPRB, 0.68770E+00_JPRB, 0.69410E+00_JPRB, 0.70777E+00_JPRB, 0.72457E+00_JPRB, 0.74330E+00_JPRB, 0.76298E+00_JPRB ,&
&  0.78865E+00_JPRB, 0.81861E+00_JPRB, 0.84915E+00_JPRB, 0.82385E+00_JPRB, 0.88489E+00_JPRB, 0.90125E+00_JPRB, 0.91180E+00_JPRB ,&
&  0.90573E+00_JPRB, 0.87029E+00_JPRB, 0.82371E+00_JPRB, 0.65198E+00_JPRB, 0.72797E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.78670E+00_JPRB, 0.81556E+00_JPRB, 0.81000E+00_JPRB, 0.78603E+00_JPRB, 0.77278E+00_JPRB, 0.76765E+00_JPRB, 0.76639E+00_JPRB ,&
&  0.76210E+00_JPRB, 0.75162E+00_JPRB, 0.75664E+00_JPRB, 0.76421E+00_JPRB, 0.76411E+00_JPRB, 0.75542E+00_JPRB, 0.74988E+00_JPRB ,&
&  0.74971E+00_JPRB, 0.75489E+00_JPRB, 0.76107E+00_JPRB, 0.75217E+00_JPRB, 0.73281E+00_JPRB, 0.72007E+00_JPRB, 0.70871E+00_JPRB ,&
&  0.70723E+00_JPRB, 0.70730E+00_JPRB, 0.70541E+00_JPRB, 0.70749E+00_JPRB, 0.68795E+00_JPRB, 0.66620E+00_JPRB, 0.64779E+00_JPRB ,&
&  0.63285E+00_JPRB, 0.54832E+00_JPRB, 0.64856E+00_JPRB, 0.82233E+00_JPRB, 0.91156E+00_JPRB, 0.92308E+00_JPRB, 0.89533E+00_JPRB ,&
&  0.87919E+00_JPRB, 0.84684E+00_JPRB, 0.81008E+00_JPRB, 0.77673E+00_JPRB, 0.75990E+00_JPRB, 0.75042E+00_JPRB, 0.74505E+00_JPRB ,&
&  0.73966E+00_JPRB, 0.73317E+00_JPRB, 0.72942E+00_JPRB, 0.71384E+00_JPRB, 0.70127E+00_JPRB, 0.70055E+00_JPRB, 0.70368E+00_JPRB ,&
&  0.70880E+00_JPRB, 0.70600E+00_JPRB, 0.70853E+00_JPRB, 0.71311E+00_JPRB, 0.69930E+00_JPRB, 0.69953E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.71569E+00_JPRB, 0.71741E+00_JPRB, 0.74473E+00_JPRB, 0.75416E+00_JPRB, 0.76262E+00_JPRB, 0.76389E+00_JPRB ,&
&  0.75774E+00_JPRB, 0.75750E+00_JPRB, 0.75254E+00_JPRB, 0.75047E+00_JPRB, 0.74216E+00_JPRB, 0.72168E+00_JPRB, 0.70282E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.70359E+00_JPRB, 0.69618E+00_JPRB, 0.69688E+00_JPRB, 0.69940E+00_JPRB, 0.69434E+00_JPRB, 0.69127E+00_JPRB ,&
&  0.68958E+00_JPRB, 0.69255E+00_JPRB, 0.69522E+00_JPRB, 0.70604E+00_JPRB, 0.72383E+00_JPRB, 0.74455E+00_JPRB, 0.76591E+00_JPRB ,&
&  0.78665E+00_JPRB, 0.80221E+00_JPRB, 0.82701E+00_JPRB, 0.88641E+00_JPRB, 0.89431E+00_JPRB, 0.89395E+00_JPRB, 0.90769E+00_JPRB ,&
&  0.89900E+00_JPRB, 0.86959E+00_JPRB, 0.83963E+00_JPRB, 0.79463E+00_JPRB, 0.73259E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.88197E+00_JPRB, 0.86976E+00_JPRB, 0.85170E+00_JPRB, 0.82104E+00_JPRB, 0.80667E+00_JPRB, 0.79137E+00_JPRB, 0.77504E+00_JPRB ,&
&  0.76706E+00_JPRB, 0.75925E+00_JPRB, 0.75365E+00_JPRB, 0.74086E+00_JPRB, 0.72857E+00_JPRB, 0.72154E+00_JPRB, 0.71231E+00_JPRB ,&
&  0.71374E+00_JPRB, 0.71497E+00_JPRB, 0.70816E+00_JPRB, 0.70123E+00_JPRB, 0.69925E+00_JPRB, 0.69753E+00_JPRB, 0.70267E+00_JPRB ,&
&  0.69525E+00_JPRB, 0.68627E+00_JPRB, 0.68620E+00_JPRB, 0.67961E+00_JPRB, 0.66791E+00_JPRB, 0.65735E+00_JPRB, 0.62751E+00_JPRB ,&
&  0.61880E+00_JPRB, 0.62140E+00_JPRB, 0.82684E+00_JPRB, 0.10633E+01_JPRB, 0.10571E+01_JPRB, 0.10448E+01_JPRB, 0.99565E+00_JPRB ,&
&  0.95753E+00_JPRB, 0.90056E+00_JPRB, 0.83966E+00_JPRB, 0.80208E+00_JPRB, 0.79476E+00_JPRB, 0.78527E+00_JPRB, 0.77051E+00_JPRB ,&
&  0.75015E+00_JPRB, 0.73020E+00_JPRB, 0.71397E+00_JPRB, 0.71679E+00_JPRB, 0.71236E+00_JPRB, 0.71512E+00_JPRB, 0.72115E+00_JPRB ,&
&  0.72119E+00_JPRB, 0.72116E+00_JPRB, 0.71276E+00_JPRB, 0.71155E+00_JPRB, 0.71228E+00_JPRB, 0.72137E+00_JPRB, 0.73725E+00_JPRB ,&
&  0.76163E+00_JPRB, 0.76336E+00_JPRB, 0.77171E+00_JPRB, 0.77993E+00_JPRB, 0.79114E+00_JPRB, 0.78986E+00_JPRB, 0.77855E+00_JPRB ,&
&  0.76817E+00_JPRB, 0.76476E+00_JPRB, 0.74214E+00_JPRB, 0.75094E+00_JPRB, 0.74199E+00_JPRB, 0.72847E+00_JPRB, 0.72025E+00_JPRB ,&
&  0.70969E+00_JPRB, 0.71296E+00_JPRB, 0.70586E+00_JPRB, 0.70782E+00_JPRB, 0.71245E+00_JPRB, 0.71236E+00_JPRB, 0.70907E+00_JPRB ,&
&  0.71470E+00_JPRB, 0.71307E+00_JPRB, 0.71876E+00_JPRB, 0.74106E+00_JPRB, 0.76791E+00_JPRB, 0.78120E+00_JPRB, 0.79516E+00_JPRB ,&
&  0.80888E+00_JPRB, 0.83408E+00_JPRB, 0.81048E+00_JPRB, 0.83981E+00_JPRB, 0.94902E+00_JPRB, 0.95790E+00_JPRB, 0.92086E+00_JPRB ,&
&  0.89424E+00_JPRB, 0.86904E+00_JPRB, 0.85715E+00_JPRB, 0.83495E+00_JPRB, 0.85466E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.85867E+00_JPRB, 0.82946E+00_JPRB, 0.80962E+00_JPRB, 0.78630E+00_JPRB, 0.77814E+00_JPRB, 0.76367E+00_JPRB, 0.74342E+00_JPRB ,&
&  0.73137E+00_JPRB, 0.72984E+00_JPRB, 0.72312E+00_JPRB, 0.70750E+00_JPRB, 0.69377E+00_JPRB, 0.68860E+00_JPRB, 0.68796E+00_JPRB ,&
&  0.68632E+00_JPRB, 0.68491E+00_JPRB, 0.68289E+00_JPRB, 0.67661E+00_JPRB, 0.68125E+00_JPRB, 0.69151E+00_JPRB, 0.70265E+00_JPRB ,&
&  0.68813E+00_JPRB, 0.67651E+00_JPRB, 0.66581E+00_JPRB, 0.65496E+00_JPRB, 0.65155E+00_JPRB, 0.63315E+00_JPRB, 0.60945E+00_JPRB ,&
&  0.65926E+00_JPRB, 0.81573E+00_JPRB, 0.10989E+01_JPRB, 0.13345E+01_JPRB, 0.11843E+01_JPRB, 0.10337E+01_JPRB, 0.97822E+00_JPRB ,&
&  0.94432E+00_JPRB, 0.88172E+00_JPRB, 0.84148E+00_JPRB, 0.80918E+00_JPRB, 0.79795E+00_JPRB, 0.78413E+00_JPRB, 0.74881E+00_JPRB ,&
&  0.72164E+00_JPRB, 0.70461E+00_JPRB, 0.69330E+00_JPRB, 0.69096E+00_JPRB, 0.68292E+00_JPRB, 0.69139E+00_JPRB, 0.69601E+00_JPRB ,&
&  0.69935E+00_JPRB, 0.69904E+00_JPRB, 0.69820E+00_JPRB, 0.69674E+00_JPRB, 0.71445E+00_JPRB, 0.76588E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.82263E+00_JPRB, 0.82864E+00_JPRB, 0.82315E+00_JPRB, 0.82391E+00_JPRB, 0.82146E+00_JPRB, 0.79269E+00_JPRB, 0.77670E+00_JPRB ,&
&  0.76064E+00_JPRB, 0.75934E+00_JPRB, 0.73899E+00_JPRB, 0.73873E+00_JPRB, 0.74163E+00_JPRB, 0.73696E+00_JPRB, 0.73720E+00_JPRB ,&
&  0.73334E+00_JPRB, 0.73075E+00_JPRB, 0.72812E+00_JPRB, 0.73658E+00_JPRB, 0.73630E+00_JPRB, 0.72636E+00_JPRB, 0.72077E+00_JPRB ,&
&  0.72232E+00_JPRB, 0.71224E+00_JPRB, 0.72252E+00_JPRB, 0.75625E+00_JPRB, 0.78406E+00_JPRB, 0.79482E+00_JPRB, 0.80883E+00_JPRB ,&
&  0.82499E+00_JPRB, 0.83217E+00_JPRB, 0.81450E+00_JPRB, 0.83733E+00_JPRB, 0.87413E+00_JPRB, 0.96277E+00_JPRB, 0.90473E+00_JPRB ,&
&  0.87182E+00_JPRB, 0.85477E+00_JPRB, 0.84230E+00_JPRB, 0.82046E+00_JPRB, 0.87464E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.81544E+00_JPRB, 0.79521E+00_JPRB, 0.78443E+00_JPRB, 0.77512E+00_JPRB, 0.75618E+00_JPRB, 0.73504E+00_JPRB, 0.72988E+00_JPRB ,&
&  0.72677E+00_JPRB, 0.70810E+00_JPRB, 0.68611E+00_JPRB, 0.65779E+00_JPRB, 0.65012E+00_JPRB, 0.65320E+00_JPRB, 0.65641E+00_JPRB ,&
&  0.65371E+00_JPRB, 0.65373E+00_JPRB, 0.65275E+00_JPRB, 0.65284E+00_JPRB, 0.65384E+00_JPRB, 0.67826E+00_JPRB, 0.68938E+00_JPRB ,&
&  0.68317E+00_JPRB, 0.68594E+00_JPRB, 0.66601E+00_JPRB, 0.64426E+00_JPRB, 0.63400E+00_JPRB, 0.62047E+00_JPRB, 0.61204E+00_JPRB ,&
&  0.75517E+00_JPRB, 0.10680E+01_JPRB, 0.15420E+01_JPRB, 0.14582E+01_JPRB, 0.11819E+01_JPRB, 0.97562E+00_JPRB, 0.92315E+00_JPRB ,&
&  0.90928E+00_JPRB, 0.87029E+00_JPRB, 0.82929E+00_JPRB, 0.79517E+00_JPRB, 0.76598E+00_JPRB, 0.75167E+00_JPRB, 0.73175E+00_JPRB ,&
&  0.71997E+00_JPRB, 0.71138E+00_JPRB, 0.68631E+00_JPRB, 0.67382E+00_JPRB, 0.66740E+00_JPRB, 0.66833E+00_JPRB, 0.66667E+00_JPRB ,&
&  0.67195E+00_JPRB, 0.67782E+00_JPRB, 0.68276E+00_JPRB, 0.67491E+00_JPRB, 0.66091E+00_JPRB, 0.64028E+00_JPRB, 0.79655E+00_JPRB ,&
&  0.10447E+01_JPRB, 0.98255E+00_JPRB, 0.89369E+00_JPRB, 0.85645E+00_JPRB, 0.83195E+00_JPRB, 0.79052E+00_JPRB, 0.78225E+00_JPRB ,&
&  0.77656E+00_JPRB, 0.78192E+00_JPRB, 0.77117E+00_JPRB, 0.78405E+00_JPRB, 0.78572E+00_JPRB, 0.78045E+00_JPRB, 0.77250E+00_JPRB ,&
&  0.75486E+00_JPRB, 0.75299E+00_JPRB, 0.75354E+00_JPRB, 0.75376E+00_JPRB, 0.74192E+00_JPRB, 0.74074E+00_JPRB, 0.73880E+00_JPRB ,&
&  0.73638E+00_JPRB, 0.71948E+00_JPRB, 0.72753E+00_JPRB, 0.74826E+00_JPRB, 0.81001E+00_JPRB, 0.85023E+00_JPRB, 0.83603E+00_JPRB ,&
&  0.79631E+00_JPRB, 0.77359E+00_JPRB, 0.77443E+00_JPRB, 0.78953E+00_JPRB, 0.82247E+00_JPRB, 0.92078E+00_JPRB, 0.85519E+00_JPRB ,&
&  0.82342E+00_JPRB, 0.82140E+00_JPRB, 0.81145E+00_JPRB, 0.79906E+00_JPRB, 0.81944E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.80466E+00_JPRB, 0.79036E+00_JPRB, 0.77545E+00_JPRB, 0.75404E+00_JPRB, 0.72555E+00_JPRB, 0.70850E+00_JPRB, 0.69831E+00_JPRB ,&
&  0.69361E+00_JPRB, 0.66884E+00_JPRB, 0.64909E+00_JPRB, 0.64740E+00_JPRB, 0.65114E+00_JPRB, 0.63958E+00_JPRB, 0.63313E+00_JPRB ,&
&  0.63616E+00_JPRB, 0.63769E+00_JPRB, 0.62940E+00_JPRB, 0.62254E+00_JPRB, 0.62772E+00_JPRB, 0.66374E+00_JPRB, 0.67860E+00_JPRB ,&
&  0.68531E+00_JPRB, 0.67868E+00_JPRB, 0.65812E+00_JPRB, 0.62978E+00_JPRB, 0.62258E+00_JPRB, 0.62590E+00_JPRB, 0.65167E+00_JPRB ,&
&  0.89372E+00_JPRB, 0.10434E+01_JPRB, 0.17026E+01_JPRB, 0.16730E+01_JPRB, 0.11966E+01_JPRB, 0.96097E+00_JPRB, 0.83915E+00_JPRB ,&
&  0.86594E+00_JPRB, 0.84110E+00_JPRB, 0.81023E+00_JPRB, 0.77719E+00_JPRB, 0.74851E+00_JPRB, 0.72267E+00_JPRB, 0.70371E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.68570E+00_JPRB, 0.67293E+00_JPRB, 0.67111E+00_JPRB, 0.66295E+00_JPRB, 0.66129E+00_JPRB, 0.66053E+00_JPRB ,&
&  0.66511E+00_JPRB, 0.66391E+00_JPRB, 0.66074E+00_JPRB, 0.65326E+00_JPRB, 0.65446E+00_JPRB, 0.74690E+00_JPRB, 0.87476E+00_JPRB ,&
&  0.96339E+00_JPRB, 0.11031E+01_JPRB, 0.96845E+00_JPRB, 0.89482E+00_JPRB, 0.83421E+00_JPRB, 0.79431E+00_JPRB, 0.80099E+00_JPRB ,&
&  0.80900E+00_JPRB, 0.82360E+00_JPRB, 0.81696E+00_JPRB, 0.82053E+00_JPRB, 0.82318E+00_JPRB, 0.80028E+00_JPRB, 0.78716E+00_JPRB ,&
&  0.78265E+00_JPRB, 0.77908E+00_JPRB, 0.76944E+00_JPRB, 0.75775E+00_JPRB, 0.74154E+00_JPRB, 0.73287E+00_JPRB, 0.73109E+00_JPRB ,&
&  0.72985E+00_JPRB, 0.71896E+00_JPRB, 0.73213E+00_JPRB, 0.75322E+00_JPRB, 0.78714E+00_JPRB, 0.79405E+00_JPRB, 0.78508E+00_JPRB ,&
&  0.76284E+00_JPRB, 0.75000E+00_JPRB, 0.74341E+00_JPRB, 0.77420E+00_JPRB, 0.77314E+00_JPRB, 0.84597E+00_JPRB, 0.80041E+00_JPRB ,&
&  0.79155E+00_JPRB, 0.78976E+00_JPRB, 0.77491E+00_JPRB, 0.77202E+00_JPRB, 0.79392E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.76592E+00_JPRB, 0.73406E+00_JPRB, 0.71325E+00_JPRB, 0.69504E+00_JPRB, 0.67922E+00_JPRB, 0.66766E+00_JPRB, 0.67774E+00_JPRB ,&
&  0.67664E+00_JPRB, 0.65194E+00_JPRB, 0.63321E+00_JPRB, 0.63476E+00_JPRB, 0.63114E+00_JPRB, 0.62909E+00_JPRB, 0.62644E+00_JPRB ,&
&  0.61766E+00_JPRB, 0.60487E+00_JPRB, 0.60167E+00_JPRB, 0.60924E+00_JPRB, 0.63960E+00_JPRB, 0.67717E+00_JPRB, 0.68457E+00_JPRB ,&
&  0.67768E+00_JPRB, 0.67209E+00_JPRB, 0.65247E+00_JPRB, 0.63409E+00_JPRB, 0.62994E+00_JPRB, 0.64135E+00_JPRB, 0.68359E+00_JPRB ,&
&  0.10211E+01_JPRB, 0.24597E+01_JPRB, 0.12092E+01_JPRB, 0.16809E+01_JPRB, 0.11657E+01_JPRB, 0.90266E+00_JPRB, 0.71798E+00_JPRB ,&
&  0.72173E+00_JPRB, 0.78633E+00_JPRB, 0.78497E+00_JPRB, 0.75971E+00_JPRB, 0.71175E+00_JPRB, 0.68773E+00_JPRB, 0.68105E+00_JPRB ,&
&  0.70522E+00_JPRB, 0.68541E+00_JPRB, 0.67411E+00_JPRB, 0.67103E+00_JPRB, 0.65926E+00_JPRB, 0.65792E+00_JPRB, 0.66682E+00_JPRB ,&
&  0.66394E+00_JPRB, 0.65906E+00_JPRB, 0.65779E+00_JPRB, 0.70316E+00_JPRB, 0.71085E+00_JPRB, 0.87512E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.94000E+00_JPRB, 0.11614E+01_JPRB, 0.10090E+01_JPRB, 0.91054E+00_JPRB, 0.84562E+00_JPRB, 0.82528E+00_JPRB, 0.83172E+00_JPRB ,&
&  0.81785E+00_JPRB, 0.80591E+00_JPRB, 0.79395E+00_JPRB, 0.79641E+00_JPRB, 0.81212E+00_JPRB, 0.79457E+00_JPRB, 0.79503E+00_JPRB ,&
&  0.80505E+00_JPRB, 0.77402E+00_JPRB, 0.76352E+00_JPRB, 0.75430E+00_JPRB, 0.76179E+00_JPRB, 0.76500E+00_JPRB, 0.75753E+00_JPRB ,&
&  0.74724E+00_JPRB, 0.73916E+00_JPRB, 0.74022E+00_JPRB, 0.74553E+00_JPRB, 0.74949E+00_JPRB, 0.74732E+00_JPRB, 0.72488E+00_JPRB ,&
&  0.73722E+00_JPRB, 0.74175E+00_JPRB, 0.73841E+00_JPRB, 0.76082E+00_JPRB, 0.76622E+00_JPRB, 0.80694E+00_JPRB, 0.76901E+00_JPRB ,&
&  0.78002E+00_JPRB, 0.77757E+00_JPRB, 0.75741E+00_JPRB, 0.76587E+00_JPRB, 0.78808E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.72222E+00_JPRB, 0.69711E+00_JPRB, 0.67280E+00_JPRB, 0.64569E+00_JPRB, 0.62841E+00_JPRB, 0.63595E+00_JPRB, 0.65772E+00_JPRB ,&
&  0.64104E+00_JPRB, 0.61691E+00_JPRB, 0.61005E+00_JPRB, 0.62463E+00_JPRB, 0.64237E+00_JPRB, 0.63656E+00_JPRB, 0.62873E+00_JPRB ,&
&  0.61109E+00_JPRB, 0.60452E+00_JPRB, 0.59379E+00_JPRB, 0.60352E+00_JPRB, 0.65400E+00_JPRB, 0.65078E+00_JPRB, 0.64095E+00_JPRB ,&
&  0.65863E+00_JPRB, 0.65499E+00_JPRB, 0.63370E+00_JPRB, 0.63142E+00_JPRB, 0.63703E+00_JPRB, 0.66623E+00_JPRB, 0.72552E+00_JPRB ,&
&  0.11052E+01_JPRB, 0.24825E+01_JPRB, 0.13883E+01_JPRB, 0.16722E+01_JPRB, 0.11056E+01_JPRB, 0.82394E+00_JPRB, 0.62856E+00_JPRB ,&
&  0.61915E+00_JPRB, 0.75460E+00_JPRB, 0.76628E+00_JPRB, 0.73570E+00_JPRB, 0.66970E+00_JPRB, 0.64437E+00_JPRB, 0.67151E+00_JPRB ,&
&  0.69119E+00_JPRB, 0.67813E+00_JPRB, 0.67760E+00_JPRB, 0.69000E+00_JPRB, 0.69052E+00_JPRB, 0.68822E+00_JPRB, 0.69505E+00_JPRB ,&
&  0.69482E+00_JPRB, 0.70446E+00_JPRB, 0.72142E+00_JPRB, 0.73229E+00_JPRB, 0.83078E+00_JPRB, 0.10259E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10152E+01_JPRB, 0.11348E+01_JPRB, 0.99970E+00_JPRB, 0.91356E+00_JPRB, 0.83991E+00_JPRB, 0.82640E+00_JPRB, 0.80752E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.76915E+00_JPRB, 0.74685E+00_JPRB, 0.74821E+00_JPRB, 0.78932E+00_JPRB, 0.80651E+00_JPRB, 0.79126E+00_JPRB ,&
&  0.80800E+00_JPRB, 0.80829E+00_JPRB, 0.80846E+00_JPRB, 0.78810E+00_JPRB, 0.79245E+00_JPRB, 0.78753E+00_JPRB, 0.77528E+00_JPRB ,&
&  0.76604E+00_JPRB, 0.76771E+00_JPRB, 0.75009E+00_JPRB, 0.74634E+00_JPRB, 0.74408E+00_JPRB, 0.73903E+00_JPRB, 0.71156E+00_JPRB ,&
&  0.72544E+00_JPRB, 0.73952E+00_JPRB, 0.75166E+00_JPRB, 0.74665E+00_JPRB, 0.74086E+00_JPRB, 0.77058E+00_JPRB, 0.77709E+00_JPRB ,&
&  0.79592E+00_JPRB, 0.79440E+00_JPRB, 0.75713E+00_JPRB, 0.75347E+00_JPRB, 0.75849E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.69009E+00_JPRB, 0.66038E+00_JPRB, 0.63194E+00_JPRB, 0.58714E+00_JPRB, 0.59514E+00_JPRB, 0.60740E+00_JPRB, 0.58860E+00_JPRB ,&
&  0.54584E+00_JPRB, 0.54069E+00_JPRB, 0.55416E+00_JPRB, 0.57715E+00_JPRB, 0.59609E+00_JPRB, 0.59055E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.58705E+00_JPRB, 0.58338E+00_JPRB, 0.60327E+00_JPRB, 0.63578E+00_JPRB, 0.63867E+00_JPRB, 0.63603E+00_JPRB, 0.64684E+00_JPRB ,&
&  0.65176E+00_JPRB, 0.63923E+00_JPRB, 0.63075E+00_JPRB, 0.63561E+00_JPRB, 0.66595E+00_JPRB, 0.71690E+00_JPRB, 0.79969E+00_JPRB ,&
&  0.11706E+01_JPRB, 0.26626E+01_JPRB, 0.20202E+01_JPRB, 0.15112E+01_JPRB, 0.10060E+01_JPRB, 0.72842E+00_JPRB, 0.58735E+00_JPRB ,&
&  0.55130E+00_JPRB, 0.57519E+00_JPRB, 0.75233E+00_JPRB, 0.70526E+00_JPRB, 0.64327E+00_JPRB, 0.61634E+00_JPRB, 0.67162E+00_JPRB ,&
&  0.71383E+00_JPRB, 0.72040E+00_JPRB, 0.73324E+00_JPRB, 0.73857E+00_JPRB, 0.74342E+00_JPRB, 0.74390E+00_JPRB, 0.73953E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.77897E+00_JPRB, 0.81955E+00_JPRB, 0.84732E+00_JPRB, 0.94506E+00_JPRB, 0.11389E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.10582E+01_JPRB, 0.97378E+00_JPRB, 0.92612E+00_JPRB, 0.79901E+00_JPRB, 0.82372E+00_JPRB, 0.78173E+00_JPRB ,&
&  0.75710E+00_JPRB, 0.72544E+00_JPRB, 0.72171E+00_JPRB, 0.73538E+00_JPRB, 0.75401E+00_JPRB, 0.77355E+00_JPRB, 0.77974E+00_JPRB ,&
&  0.79185E+00_JPRB, 0.80157E+00_JPRB, 0.80333E+00_JPRB, 0.79014E+00_JPRB, 0.78782E+00_JPRB, 0.78980E+00_JPRB, 0.80033E+00_JPRB ,&
&  0.80035E+00_JPRB, 0.81136E+00_JPRB, 0.78044E+00_JPRB, 0.77085E+00_JPRB, 0.76705E+00_JPRB, 0.75601E+00_JPRB, 0.71654E+00_JPRB ,&
&  0.72192E+00_JPRB, 0.75729E+00_JPRB, 0.76294E+00_JPRB, 0.75151E+00_JPRB, 0.74795E+00_JPRB, 0.76300E+00_JPRB, 0.76878E+00_JPRB ,&
&  0.79186E+00_JPRB, 0.79420E+00_JPRB, 0.75335E+00_JPRB, 0.72902E+00_JPRB, 0.73361E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.67774E+00_JPRB, 0.65264E+00_JPRB, 0.59911E+00_JPRB, 0.57441E+00_JPRB, 0.57220E+00_JPRB, 0.56576E+00_JPRB, 0.55274E+00_JPRB ,&
&  0.53035E+00_JPRB, 0.52355E+00_JPRB, 0.55247E+00_JPRB, 0.58127E+00_JPRB, 0.60737E+00_JPRB, 0.60478E+00_JPRB, 0.60390E+00_JPRB ,&
&  0.60478E+00_JPRB, 0.62489E+00_JPRB, 0.66424E+00_JPRB, 0.67120E+00_JPRB, 0.65592E+00_JPRB, 0.64694E+00_JPRB, 0.64418E+00_JPRB ,&
&  0.64706E+00_JPRB, 0.65088E+00_JPRB, 0.66966E+00_JPRB, 0.69269E+00_JPRB, 0.73354E+00_JPRB, 0.80472E+00_JPRB, 0.95784E+00_JPRB ,&
&  0.13352E+01_JPRB, 0.23234E+01_JPRB, 0.68337E+00_JPRB, 0.85177E+00_JPRB, 0.77175E+00_JPRB, 0.64857E+00_JPRB, 0.56540E+00_JPRB ,&
&  0.50859E+00_JPRB, 0.51871E+00_JPRB, 0.63729E+00_JPRB, 0.68173E+00_JPRB, 0.62035E+00_JPRB, 0.66718E+00_JPRB, 0.75069E+00_JPRB ,&
&  0.81473E+00_JPRB, 0.82142E+00_JPRB, 0.81451E+00_JPRB, 0.80192E+00_JPRB, 0.80228E+00_JPRB, 0.80229E+00_JPRB, 0.81066E+00_JPRB ,&
&  0.83675E+00_JPRB, 0.89112E+00_JPRB, 0.94722E+00_JPRB, 0.92440E+00_JPRB, 0.11684E+01_JPRB, 0.13477E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.99038E+00_JPRB, 0.95941E+00_JPRB, 0.94489E+00_JPRB, 0.78136E+00_JPRB, 0.69088E+00_JPRB, 0.76595E+00_JPRB ,&
&  0.75718E+00_JPRB, 0.73114E+00_JPRB, 0.71155E+00_JPRB, 0.69816E+00_JPRB, 0.69557E+00_JPRB, 0.73951E+00_JPRB, 0.76924E+00_JPRB ,&
&  0.76651E+00_JPRB, 0.78907E+00_JPRB, 0.79207E+00_JPRB, 0.78070E+00_JPRB, 0.78514E+00_JPRB, 0.79508E+00_JPRB, 0.82690E+00_JPRB ,&
&  0.83894E+00_JPRB, 0.85159E+00_JPRB, 0.85794E+00_JPRB, 0.83751E+00_JPRB, 0.83877E+00_JPRB, 0.80179E+00_JPRB, 0.75259E+00_JPRB ,&
&  0.75081E+00_JPRB, 0.77005E+00_JPRB, 0.74330E+00_JPRB, 0.71286E+00_JPRB, 0.73853E+00_JPRB, 0.74166E+00_JPRB, 0.74514E+00_JPRB ,&
&  0.79619E+00_JPRB, 0.82530E+00_JPRB, 0.81852E+00_JPRB, 0.70388E+00_JPRB, 0.71572E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.66396E+00_JPRB, 0.61552E+00_JPRB, 0.58103E+00_JPRB, 0.57607E+00_JPRB, 0.56465E+00_JPRB, 0.57033E+00_JPRB, 0.57811E+00_JPRB ,&
&  0.57432E+00_JPRB, 0.58659E+00_JPRB, 0.58815E+00_JPRB, 0.59902E+00_JPRB, 0.61813E+00_JPRB, 0.62037E+00_JPRB, 0.63810E+00_JPRB ,&
&  0.65800E+00_JPRB, 0.68352E+00_JPRB, 0.70293E+00_JPRB, 0.66369E+00_JPRB, 0.65136E+00_JPRB, 0.64742E+00_JPRB, 0.65196E+00_JPRB ,&
&  0.67075E+00_JPRB, 0.69993E+00_JPRB, 0.73405E+00_JPRB, 0.77921E+00_JPRB, 0.84719E+00_JPRB, 0.97252E+00_JPRB, 0.12100E+01_JPRB ,&
&  0.18216E+01_JPRB, 0.14313E+01_JPRB, 0.53407E+00_JPRB, 0.66625E+00_JPRB, 0.64082E+00_JPRB, 0.60451E+00_JPRB, 0.51093E+00_JPRB ,&
&  0.48871E+00_JPRB, 0.53576E+00_JPRB, 0.59954E+00_JPRB, 0.66141E+00_JPRB, 0.70639E+00_JPRB, 0.77062E+00_JPRB, 0.83527E+00_JPRB ,&
&  0.87026E+00_JPRB, 0.88225E+00_JPRB, 0.87709E+00_JPRB, 0.87695E+00_JPRB, 0.88323E+00_JPRB, 0.89820E+00_JPRB, 0.92005E+00_JPRB ,&
&  0.96062E+00_JPRB, 0.10221E+01_JPRB, 0.10917E+01_JPRB, 0.10424E+01_JPRB, 0.13016E+01_JPRB, 0.14044E+01_JPRB, 0.13911E+01_JPRB ,&
&  0.12146E+01_JPRB, 0.98408E+00_JPRB, 0.90537E+00_JPRB, 0.88786E+00_JPRB, 0.70918E+00_JPRB, 0.62706E+00_JPRB, 0.72930E+00_JPRB ,&
&  0.77897E+00_JPRB, 0.73208E+00_JPRB, 0.71325E+00_JPRB, 0.70274E+00_JPRB, 0.66250E+00_JPRB, 0.70439E+00_JPRB, 0.76394E+00_JPRB ,&
&  0.76687E+00_JPRB, 0.76880E+00_JPRB, 0.78929E+00_JPRB, 0.77891E+00_JPRB, 0.77482E+00_JPRB, 0.78720E+00_JPRB, 0.81998E+00_JPRB ,&
&  0.84016E+00_JPRB, 0.85957E+00_JPRB, 0.86289E+00_JPRB, 0.89195E+00_JPRB, 0.91843E+00_JPRB, 0.85189E+00_JPRB, 0.76393E+00_JPRB ,&
&  0.74170E+00_JPRB, 0.75040E+00_JPRB, 0.73500E+00_JPRB, 0.72560E+00_JPRB, 0.71695E+00_JPRB, 0.71478E+00_JPRB, 0.73278E+00_JPRB ,&
&  0.82656E+00_JPRB, 0.77625E+00_JPRB, 0.72844E+00_JPRB, 0.66317E+00_JPRB, 0.68444E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.60213E+00_JPRB, 0.57610E+00_JPRB, 0.56782E+00_JPRB, 0.55332E+00_JPRB, 0.56680E+00_JPRB, 0.59113E+00_JPRB, 0.60965E+00_JPRB ,&
&  0.61181E+00_JPRB, 0.60968E+00_JPRB, 0.60991E+00_JPRB, 0.62603E+00_JPRB, 0.64892E+00_JPRB, 0.67229E+00_JPRB, 0.70189E+00_JPRB ,&
&  0.71985E+00_JPRB, 0.72940E+00_JPRB, 0.69905E+00_JPRB, 0.65651E+00_JPRB, 0.65704E+00_JPRB, 0.66654E+00_JPRB, 0.68488E+00_JPRB ,&
&  0.71368E+00_JPRB, 0.74594E+00_JPRB, 0.79096E+00_JPRB, 0.87475E+00_JPRB, 0.10057E+01_JPRB, 0.11599E+01_JPRB, 0.13173E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.48512E+00_JPRB, 0.49923E+00_JPRB, 0.56718E+00_JPRB, 0.51867E+00_JPRB, 0.51931E+00_JPRB, 0.51186E+00_JPRB ,&
&  0.49772E+00_JPRB, 0.54244E+00_JPRB, 0.58310E+00_JPRB, 0.70806E+00_JPRB, 0.80608E+00_JPRB, 0.85135E+00_JPRB, 0.88974E+00_JPRB ,&
&  0.91722E+00_JPRB, 0.93670E+00_JPRB, 0.93929E+00_JPRB, 0.94123E+00_JPRB, 0.95561E+00_JPRB, 0.98024E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.11400E+01_JPRB, 0.12353E+01_JPRB, 0.11279E+01_JPRB, 0.13070E+01_JPRB, 0.14457E+01_JPRB, 0.15889E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.10107E+01_JPRB, 0.79713E+00_JPRB, 0.85322E+00_JPRB, 0.77296E+00_JPRB, 0.64003E+00_JPRB, 0.75350E+00_JPRB ,&
&  0.79209E+00_JPRB, 0.80028E+00_JPRB, 0.78996E+00_JPRB, 0.71757E+00_JPRB, 0.65009E+00_JPRB, 0.63948E+00_JPRB, 0.69244E+00_JPRB ,&
&  0.71612E+00_JPRB, 0.73035E+00_JPRB, 0.73096E+00_JPRB, 0.73270E+00_JPRB, 0.71622E+00_JPRB, 0.72727E+00_JPRB, 0.75985E+00_JPRB ,&
&  0.79261E+00_JPRB, 0.82432E+00_JPRB, 0.83777E+00_JPRB, 0.86354E+00_JPRB, 0.89998E+00_JPRB, 0.84636E+00_JPRB, 0.76578E+00_JPRB ,&
&  0.74438E+00_JPRB, 0.73642E+00_JPRB, 0.70968E+00_JPRB, 0.70127E+00_JPRB, 0.71026E+00_JPRB, 0.76581E+00_JPRB, 0.88188E+00_JPRB ,&
&  0.87542E+00_JPRB, 0.77459E+00_JPRB, 0.66677E+00_JPRB, 0.63617E+00_JPRB, 0.64430E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.50664E+00_JPRB, 0.49885E+00_JPRB, 0.49967E+00_JPRB, 0.49775E+00_JPRB, 0.52947E+00_JPRB, 0.56391E+00_JPRB, 0.57360E+00_JPRB ,&
&  0.56679E+00_JPRB, 0.58233E+00_JPRB, 0.59728E+00_JPRB, 0.62554E+00_JPRB, 0.68496E+00_JPRB, 0.70795E+00_JPRB, 0.73541E+00_JPRB ,&
&  0.76258E+00_JPRB, 0.75416E+00_JPRB, 0.71711E+00_JPRB, 0.69181E+00_JPRB, 0.70172E+00_JPRB, 0.70475E+00_JPRB, 0.72277E+00_JPRB ,&
&  0.75473E+00_JPRB, 0.79015E+00_JPRB, 0.83394E+00_JPRB, 0.89127E+00_JPRB, 0.95740E+00_JPRB, 0.10000E+01_JPRB, 0.95518E+00_JPRB ,&
&  0.41572E+00_JPRB, 0.40768E+00_JPRB, 0.52252E+00_JPRB, 0.52890E+00_JPRB, 0.49035E+00_JPRB, 0.49268E+00_JPRB, 0.50672E+00_JPRB ,&
&  0.52365E+00_JPRB, 0.51973E+00_JPRB, 0.57895E+00_JPRB, 0.78939E+00_JPRB, 0.82616E+00_JPRB, 0.88523E+00_JPRB, 0.89771E+00_JPRB ,&
&  0.92337E+00_JPRB, 0.94983E+00_JPRB, 0.96848E+00_JPRB, 0.98171E+00_JPRB, 0.10038E+01_JPRB, 0.10353E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.12528E+01_JPRB, 0.13745E+01_JPRB, 0.12570E+01_JPRB, 0.13347E+01_JPRB, 0.15045E+01_JPRB, 0.18232E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.11099E+01_JPRB, 0.78019E+00_JPRB, 0.79809E+00_JPRB, 0.79282E+00_JPRB, 0.82527E+00_JPRB, 0.83363E+00_JPRB ,&
&  0.80484E+00_JPRB, 0.78023E+00_JPRB, 0.74272E+00_JPRB, 0.68070E+00_JPRB, 0.61468E+00_JPRB, 0.60788E+00_JPRB, 0.60831E+00_JPRB ,&
&  0.64015E+00_JPRB, 0.65090E+00_JPRB, 0.65769E+00_JPRB, 0.66474E+00_JPRB, 0.62515E+00_JPRB, 0.62691E+00_JPRB, 0.65511E+00_JPRB ,&
&  0.68565E+00_JPRB, 0.70866E+00_JPRB, 0.71643E+00_JPRB, 0.72489E+00_JPRB, 0.74242E+00_JPRB, 0.75166E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.68718E+00_JPRB, 0.66422E+00_JPRB, 0.67994E+00_JPRB, 0.68420E+00_JPRB, 0.85932E+00_JPRB, 0.11687E+01_JPRB, 0.12141E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.78703E+00_JPRB, 0.62611E+00_JPRB, 0.57639E+00_JPRB, 0.55572E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.46553E+00_JPRB, 0.45510E+00_JPRB, 0.45648E+00_JPRB, 0.45627E+00_JPRB, 0.49032E+00_JPRB, 0.51308E+00_JPRB, 0.53335E+00_JPRB ,&
&  0.56313E+00_JPRB, 0.58167E+00_JPRB, 0.61242E+00_JPRB, 0.66764E+00_JPRB, 0.72501E+00_JPRB, 0.76744E+00_JPRB, 0.78187E+00_JPRB ,&
&  0.78572E+00_JPRB, 0.77627E+00_JPRB, 0.75365E+00_JPRB, 0.72513E+00_JPRB, 0.72517E+00_JPRB, 0.71604E+00_JPRB, 0.72780E+00_JPRB ,&
&  0.75288E+00_JPRB, 0.77862E+00_JPRB, 0.81208E+00_JPRB, 0.84230E+00_JPRB, 0.86750E+00_JPRB, 0.86917E+00_JPRB, 0.80270E+00_JPRB ,&
&  0.40427E+00_JPRB, 0.58536E+00_JPRB, 0.61962E+00_JPRB, 0.60426E+00_JPRB, 0.55453E+00_JPRB, 0.51713E+00_JPRB, 0.51901E+00_JPRB ,&
&  0.54183E+00_JPRB, 0.54017E+00_JPRB, 0.58949E+00_JPRB, 0.67114E+00_JPRB, 0.78621E+00_JPRB, 0.83080E+00_JPRB, 0.86099E+00_JPRB ,&
&  0.88441E+00_JPRB, 0.89748E+00_JPRB, 0.92259E+00_JPRB, 0.94977E+00_JPRB, 0.98188E+00_JPRB, 0.10146E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11708E+01_JPRB, 0.12675E+01_JPRB, 0.11973E+01_JPRB, 0.11453E+01_JPRB, 0.12311E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.75721E+00_JPRB, 0.69353E+00_JPRB, 0.80299E+00_JPRB, 0.83428E+00_JPRB, 0.84847E+00_JPRB, 0.83607E+00_JPRB ,&
&  0.76654E+00_JPRB, 0.72612E+00_JPRB, 0.67208E+00_JPRB, 0.63067E+00_JPRB, 0.59345E+00_JPRB, 0.57184E+00_JPRB, 0.55704E+00_JPRB ,&
&  0.55326E+00_JPRB, 0.57051E+00_JPRB, 0.58553E+00_JPRB, 0.57479E+00_JPRB, 0.54231E+00_JPRB, 0.54109E+00_JPRB, 0.57590E+00_JPRB ,&
&  0.64265E+00_JPRB, 0.57310E+00_JPRB, 0.52870E+00_JPRB, 0.52875E+00_JPRB, 0.52872E+00_JPRB, 0.52833E+00_JPRB, 0.52492E+00_JPRB ,&
&  0.53860E+00_JPRB, 0.54106E+00_JPRB, 0.64755E+00_JPRB, 0.10241E+01_JPRB, 0.14264E+01_JPRB, 0.16805E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.96869E+00_JPRB, 0.70844E+00_JPRB, 0.56169E+00_JPRB, 0.52298E+00_JPRB, 0.50216E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.48859E+00_JPRB, 0.47169E+00_JPRB, 0.45598E+00_JPRB, 0.46816E+00_JPRB, 0.47640E+00_JPRB, 0.48154E+00_JPRB, 0.50655E+00_JPRB ,&
&  0.55904E+00_JPRB, 0.57205E+00_JPRB, 0.59836E+00_JPRB, 0.63978E+00_JPRB, 0.69949E+00_JPRB, 0.74224E+00_JPRB, 0.78088E+00_JPRB ,&
&  0.81958E+00_JPRB, 0.83779E+00_JPRB, 0.82021E+00_JPRB, 0.79579E+00_JPRB, 0.78293E+00_JPRB, 0.77740E+00_JPRB, 0.77933E+00_JPRB ,&
&  0.77164E+00_JPRB, 0.77406E+00_JPRB, 0.78907E+00_JPRB, 0.80803E+00_JPRB, 0.81667E+00_JPRB, 0.78109E+00_JPRB, 0.48130E+00_JPRB ,&
&  0.65788E+00_JPRB, 0.70803E+00_JPRB, 0.72896E+00_JPRB, 0.69767E+00_JPRB, 0.64080E+00_JPRB, 0.57818E+00_JPRB, 0.54997E+00_JPRB ,&
&  0.55354E+00_JPRB, 0.56654E+00_JPRB, 0.57788E+00_JPRB, 0.61123E+00_JPRB, 0.71270E+00_JPRB, 0.74958E+00_JPRB, 0.78715E+00_JPRB ,&
&  0.80513E+00_JPRB, 0.82987E+00_JPRB, 0.84951E+00_JPRB, 0.86561E+00_JPRB, 0.88951E+00_JPRB, 0.91346E+00_JPRB, 0.93265E+00_JPRB ,&
&  0.93841E+00_JPRB, 0.92661E+00_JPRB, 0.93404E+00_JPRB, 0.91022E+00_JPRB, 0.83347E+00_JPRB, 0.88607E+00_JPRB, 0.76099E+00_JPRB ,&
&  0.55927E+00_JPRB, 0.47011E+00_JPRB, 0.61384E+00_JPRB, 0.83787E+00_JPRB, 0.83795E+00_JPRB, 0.84444E+00_JPRB, 0.79206E+00_JPRB ,&
&  0.71294E+00_JPRB, 0.64522E+00_JPRB, 0.61268E+00_JPRB, 0.59057E+00_JPRB, 0.56768E+00_JPRB, 0.53893E+00_JPRB, 0.51687E+00_JPRB ,&
&  0.49186E+00_JPRB, 0.47646E+00_JPRB, 0.49234E+00_JPRB, 0.48414E+00_JPRB, 0.49170E+00_JPRB, 0.53779E+00_JPRB, 0.66863E+00_JPRB ,&
&  0.10187E+01_JPRB, 0.54454E+00_JPRB, 0.46529E+00_JPRB, 0.44450E+00_JPRB, 0.42219E+00_JPRB, 0.40863E+00_JPRB, 0.40200E+00_JPRB ,&
&  0.41527E+00_JPRB, 0.47729E+00_JPRB, 0.62733E+00_JPRB, 0.13488E+01_JPRB, 0.25064E+01_JPRB, 0.25706E+01_JPRB, 0.17153E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.69580E+00_JPRB, 0.55590E+00_JPRB, 0.52016E+00_JPRB, 0.49445E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.53320E+00_JPRB, 0.50323E+00_JPRB, 0.49491E+00_JPRB, 0.49222E+00_JPRB, 0.48303E+00_JPRB, 0.50130E+00_JPRB, 0.51519E+00_JPRB ,&
&  0.53468E+00_JPRB, 0.56062E+00_JPRB, 0.59261E+00_JPRB, 0.62048E+00_JPRB, 0.66138E+00_JPRB, 0.70011E+00_JPRB, 0.72936E+00_JPRB ,&
&  0.75124E+00_JPRB, 0.76584E+00_JPRB, 0.77302E+00_JPRB, 0.77323E+00_JPRB, 0.76681E+00_JPRB, 0.75986E+00_JPRB, 0.74970E+00_JPRB ,&
&  0.73981E+00_JPRB, 0.74400E+00_JPRB, 0.76366E+00_JPRB, 0.78546E+00_JPRB, 0.78737E+00_JPRB, 0.76994E+00_JPRB, 0.49326E+00_JPRB ,&
&  0.81407E+00_JPRB, 0.84148E+00_JPRB, 0.81051E+00_JPRB, 0.73710E+00_JPRB, 0.67449E+00_JPRB, 0.63070E+00_JPRB, 0.59171E+00_JPRB ,&
&  0.56778E+00_JPRB, 0.56502E+00_JPRB, 0.52569E+00_JPRB, 0.55986E+00_JPRB, 0.63242E+00_JPRB, 0.65032E+00_JPRB, 0.68785E+00_JPRB ,&
&  0.71662E+00_JPRB, 0.74788E+00_JPRB, 0.77127E+00_JPRB, 0.79366E+00_JPRB, 0.80813E+00_JPRB, 0.82633E+00_JPRB, 0.82593E+00_JPRB ,&
&  0.82013E+00_JPRB, 0.79486E+00_JPRB, 0.75180E+00_JPRB, 0.69492E+00_JPRB, 0.68476E+00_JPRB, 0.61105E+00_JPRB, 0.56975E+00_JPRB ,&
&  0.44006E+00_JPRB, 0.45878E+00_JPRB, 0.69602E+00_JPRB, 0.91045E+00_JPRB, 0.84492E+00_JPRB, 0.82555E+00_JPRB, 0.74791E+00_JPRB ,&
&  0.67588E+00_JPRB, 0.63030E+00_JPRB, 0.60817E+00_JPRB, 0.59012E+00_JPRB, 0.55695E+00_JPRB, 0.53258E+00_JPRB, 0.52283E+00_JPRB ,&
&  0.51847E+00_JPRB, 0.47508E+00_JPRB, 0.45761E+00_JPRB, 0.45950E+00_JPRB, 0.44517E+00_JPRB, 0.46890E+00_JPRB, 0.43679E+00_JPRB ,&
&  0.56165E+00_JPRB, 0.52696E+00_JPRB, 0.46046E+00_JPRB, 0.47101E+00_JPRB, 0.43744E+00_JPRB, 0.43275E+00_JPRB, 0.44063E+00_JPRB ,&
&  0.47059E+00_JPRB, 0.42661E+00_JPRB, 0.91913E+00_JPRB, 0.36929E+01_JPRB, 0.36550E+01_JPRB, 0.46456E+01_JPRB, 0.24022E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.67326E+00_JPRB, 0.57515E+00_JPRB, 0.56474E+00_JPRB, 0.54601E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.54898E+00_JPRB, 0.54138E+00_JPRB, 0.52484E+00_JPRB, 0.49747E+00_JPRB, 0.48700E+00_JPRB, 0.50512E+00_JPRB, 0.52970E+00_JPRB ,&
&  0.54721E+00_JPRB, 0.56930E+00_JPRB, 0.59281E+00_JPRB, 0.61821E+00_JPRB, 0.64638E+00_JPRB, 0.67239E+00_JPRB, 0.69250E+00_JPRB ,&
&  0.70605E+00_JPRB, 0.72424E+00_JPRB, 0.73926E+00_JPRB, 0.75030E+00_JPRB, 0.75098E+00_JPRB, 0.75076E+00_JPRB, 0.74717E+00_JPRB ,&
&  0.77500E+00_JPRB, 0.80004E+00_JPRB, 0.83177E+00_JPRB, 0.86132E+00_JPRB, 0.87238E+00_JPRB, 0.83420E+00_JPRB, 0.64297E+00_JPRB ,&
&  0.93619E+00_JPRB, 0.94058E+00_JPRB, 0.84913E+00_JPRB, 0.75766E+00_JPRB, 0.69174E+00_JPRB, 0.64793E+00_JPRB, 0.60678E+00_JPRB ,&
&  0.57713E+00_JPRB, 0.58957E+00_JPRB, 0.56027E+00_JPRB, 0.53648E+00_JPRB, 0.53896E+00_JPRB, 0.56468E+00_JPRB, 0.59859E+00_JPRB ,&
&  0.64905E+00_JPRB, 0.69904E+00_JPRB, 0.74565E+00_JPRB, 0.77780E+00_JPRB, 0.79911E+00_JPRB, 0.81821E+00_JPRB, 0.79241E+00_JPRB ,&
&  0.78591E+00_JPRB, 0.78142E+00_JPRB, 0.71672E+00_JPRB, 0.68865E+00_JPRB, 0.69690E+00_JPRB, 0.64206E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.49331E+00_JPRB, 0.59215E+00_JPRB, 0.81649E+00_JPRB, 0.93641E+00_JPRB, 0.82187E+00_JPRB, 0.79790E+00_JPRB, 0.74090E+00_JPRB ,&
&  0.67410E+00_JPRB, 0.65297E+00_JPRB, 0.64556E+00_JPRB, 0.63026E+00_JPRB, 0.58866E+00_JPRB, 0.56110E+00_JPRB, 0.55994E+00_JPRB ,&
&  0.56699E+00_JPRB, 0.52340E+00_JPRB, 0.49204E+00_JPRB, 0.50531E+00_JPRB, 0.49760E+00_JPRB, 0.42532E+00_JPRB, 0.52891E+00_JPRB ,&
&  0.49581E+00_JPRB, 0.40452E+00_JPRB, 0.49309E+00_JPRB, 0.37064E+00_JPRB, 0.47604E+00_JPRB, 0.48244E+00_JPRB, 0.51080E+00_JPRB ,&
&  0.44387E+00_JPRB, 0.51288E+00_JPRB, 0.86722E+00_JPRB, 0.11936E+01_JPRB, 0.13745E+01_JPRB, 0.13575E+01_JPRB, 0.75128E+00_JPRB ,&
&  0.65205E+00_JPRB, 0.62186E+00_JPRB, 0.59979E+00_JPRB, 0.57711E+00_JPRB, 0.55255E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.56665E+00_JPRB, 0.56747E+00_JPRB, 0.54943E+00_JPRB, 0.52474E+00_JPRB, 0.51309E+00_JPRB, 0.52330E+00_JPRB, 0.53903E+00_JPRB ,&
&  0.55324E+00_JPRB, 0.57250E+00_JPRB, 0.59482E+00_JPRB, 0.61630E+00_JPRB, 0.63518E+00_JPRB, 0.65536E+00_JPRB, 0.67913E+00_JPRB ,&
&  0.70149E+00_JPRB, 0.72923E+00_JPRB, 0.75670E+00_JPRB, 0.78347E+00_JPRB, 0.80568E+00_JPRB, 0.84570E+00_JPRB, 0.87092E+00_JPRB ,&
&  0.91416E+00_JPRB, 0.95742E+00_JPRB, 0.10090E+01_JPRB, 0.10557E+01_JPRB, 0.11106E+01_JPRB, 0.11447E+01_JPRB, 0.94032E+00_JPRB ,&
&  0.10307E+01_JPRB, 0.98054E+00_JPRB, 0.86737E+00_JPRB, 0.77566E+00_JPRB, 0.72276E+00_JPRB, 0.68616E+00_JPRB, 0.65348E+00_JPRB ,&
&  0.64111E+00_JPRB, 0.61283E+00_JPRB, 0.58089E+00_JPRB, 0.56182E+00_JPRB, 0.55297E+00_JPRB, 0.57569E+00_JPRB, 0.57062E+00_JPRB ,&
&  0.61025E+00_JPRB, 0.68217E+00_JPRB, 0.73345E+00_JPRB, 0.81141E+00_JPRB, 0.88266E+00_JPRB, 0.93489E+00_JPRB, 0.93360E+00_JPRB ,&
&  0.93635E+00_JPRB, 0.96891E+00_JPRB, 0.93246E+00_JPRB, 0.93142E+00_JPRB, 0.91911E+00_JPRB, 0.90073E+00_JPRB, 0.87695E+00_JPRB ,&
&  0.74001E+00_JPRB, 0.75651E+00_JPRB, 0.83635E+00_JPRB, 0.91191E+00_JPRB, 0.82248E+00_JPRB, 0.77671E+00_JPRB, 0.75534E+00_JPRB ,&
&  0.69124E+00_JPRB, 0.68005E+00_JPRB, 0.68339E+00_JPRB, 0.66057E+00_JPRB, 0.64427E+00_JPRB, 0.62473E+00_JPRB, 0.62325E+00_JPRB ,&
&  0.63336E+00_JPRB, 0.57966E+00_JPRB, 0.55883E+00_JPRB, 0.56067E+00_JPRB, 0.58553E+00_JPRB, 0.55312E+00_JPRB, 0.53931E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.40931E+00_JPRB, 0.43212E+00_JPRB, 0.55205E+00_JPRB, 0.55321E+00_JPRB, 0.57881E+00_JPRB, 0.62293E+00_JPRB ,&
&  0.66946E+00_JPRB, 0.69104E+00_JPRB, 0.67728E+00_JPRB, 0.69397E+00_JPRB, 0.72129E+00_JPRB, 0.70477E+00_JPRB, 0.64585E+00_JPRB ,&
&  0.65047E+00_JPRB, 0.61432E+00_JPRB, 0.59377E+00_JPRB, 0.58459E+00_JPRB, 0.57679E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.57865E+00_JPRB, 0.58673E+00_JPRB, 0.57722E+00_JPRB, 0.55959E+00_JPRB, 0.54584E+00_JPRB, 0.54129E+00_JPRB, 0.54957E+00_JPRB ,&
&  0.55905E+00_JPRB, 0.57858E+00_JPRB, 0.59385E+00_JPRB, 0.61051E+00_JPRB, 0.61882E+00_JPRB, 0.62526E+00_JPRB, 0.63863E+00_JPRB ,&
&  0.65628E+00_JPRB, 0.67536E+00_JPRB, 0.69868E+00_JPRB, 0.73109E+00_JPRB, 0.75886E+00_JPRB, 0.79274E+00_JPRB, 0.80303E+00_JPRB ,&
&  0.79740E+00_JPRB, 0.81523E+00_JPRB, 0.82436E+00_JPRB, 0.84960E+00_JPRB, 0.10027E+01_JPRB, 0.11538E+01_JPRB, 0.12926E+01_JPRB ,&
&  0.97474E+00_JPRB, 0.97435E+00_JPRB, 0.85633E+00_JPRB, 0.77150E+00_JPRB, 0.77299E+00_JPRB, 0.75387E+00_JPRB, 0.75468E+00_JPRB ,&
&  0.75944E+00_JPRB, 0.75109E+00_JPRB, 0.74948E+00_JPRB, 0.75396E+00_JPRB, 0.76443E+00_JPRB, 0.79479E+00_JPRB, 0.76794E+00_JPRB ,&
&  0.75441E+00_JPRB, 0.78610E+00_JPRB, 0.82502E+00_JPRB, 0.92726E+00_JPRB, 0.10669E+01_JPRB, 0.11789E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.12440E+01_JPRB, 0.12918E+01_JPRB, 0.12201E+01_JPRB, 0.12012E+01_JPRB, 0.11828E+01_JPRB, 0.11616E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.10085E+01_JPRB, 0.96436E+00_JPRB, 0.85162E+00_JPRB, 0.80956E+00_JPRB, 0.79673E+00_JPRB, 0.77554E+00_JPRB ,&
&  0.74796E+00_JPRB, 0.74147E+00_JPRB, 0.73996E+00_JPRB, 0.71686E+00_JPRB, 0.71048E+00_JPRB, 0.70099E+00_JPRB, 0.70597E+00_JPRB ,&
&  0.70520E+00_JPRB, 0.65923E+00_JPRB, 0.63431E+00_JPRB, 0.62457E+00_JPRB, 0.61583E+00_JPRB, 0.81870E+00_JPRB, 0.66850E+00_JPRB ,&
&  0.66636E+00_JPRB, 0.55918E+00_JPRB, 0.46584E+00_JPRB, 0.65288E+00_JPRB, 0.64352E+00_JPRB, 0.65057E+00_JPRB, 0.66584E+00_JPRB ,&
&  0.68706E+00_JPRB, 0.69989E+00_JPRB, 0.70717E+00_JPRB, 0.69145E+00_JPRB, 0.68624E+00_JPRB, 0.68158E+00_JPRB, 0.66326E+00_JPRB ,&
&  0.64923E+00_JPRB, 0.63214E+00_JPRB, 0.62789E+00_JPRB, 0.59713E+00_JPRB, 0.58088E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.57992E+00_JPRB, 0.57593E+00_JPRB, 0.58748E+00_JPRB, 0.57429E+00_JPRB, 0.59662E+00_JPRB, 0.58731E+00_JPRB, 0.61478E+00_JPRB ,&
&  0.59110E+00_JPRB, 0.58511E+00_JPRB, 0.58471E+00_JPRB, 0.59451E+00_JPRB, 0.60555E+00_JPRB, 0.62220E+00_JPRB, 0.63458E+00_JPRB ,&
&  0.65592E+00_JPRB, 0.69692E+00_JPRB, 0.75243E+00_JPRB, 0.81491E+00_JPRB, 0.89628E+00_JPRB, 0.97352E+00_JPRB, 0.10359E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.11899E+01_JPRB, 0.12170E+01_JPRB, 0.13152E+01_JPRB, 0.16392E+01_JPRB, 0.20430E+01_JPRB, 0.20441E+01_JPRB ,&
&  0.12975E+01_JPRB, 0.89660E+00_JPRB, 0.82244E+00_JPRB, 0.75915E+00_JPRB, 0.77616E+00_JPRB, 0.83888E+00_JPRB, 0.85282E+00_JPRB ,&
&  0.86013E+00_JPRB, 0.87345E+00_JPRB, 0.90147E+00_JPRB, 0.93083E+00_JPRB, 0.97542E+00_JPRB, 0.10362E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10690E+01_JPRB, 0.10817E+01_JPRB, 0.11633E+01_JPRB, 0.12501E+01_JPRB, 0.13585E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.14984E+01_JPRB, 0.12264E+01_JPRB, 0.11960E+01_JPRB, 0.12704E+01_JPRB, 0.12613E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.12841E+01_JPRB, 0.12500E+01_JPRB, 0.89819E+00_JPRB, 0.81236E+00_JPRB, 0.79914E+00_JPRB, 0.82532E+00_JPRB, 0.80277E+00_JPRB ,&
&  0.81113E+00_JPRB, 0.81115E+00_JPRB, 0.81453E+00_JPRB, 0.80003E+00_JPRB, 0.78246E+00_JPRB, 0.76937E+00_JPRB, 0.77763E+00_JPRB ,&
&  0.73767E+00_JPRB, 0.72291E+00_JPRB, 0.71823E+00_JPRB, 0.70687E+00_JPRB, 0.77292E+00_JPRB, 0.71261E+00_JPRB, 0.68306E+00_JPRB ,&
&  0.63818E+00_JPRB, 0.64889E+00_JPRB, 0.58104E+00_JPRB, 0.69407E+00_JPRB, 0.72544E+00_JPRB, 0.70157E+00_JPRB, 0.71774E+00_JPRB ,&
&  0.72716E+00_JPRB, 0.74337E+00_JPRB, 0.73143E+00_JPRB, 0.72836E+00_JPRB, 0.73361E+00_JPRB, 0.72419E+00_JPRB, 0.71053E+00_JPRB ,&
&  0.69624E+00_JPRB, 0.68066E+00_JPRB, 0.65191E+00_JPRB, 0.62416E+00_JPRB, 0.59892E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.62269E+00_JPRB, 0.62207E+00_JPRB, 0.62893E+00_JPRB, 0.63983E+00_JPRB, 0.63801E+00_JPRB, 0.63523E+00_JPRB, 0.62996E+00_JPRB ,&
&  0.60329E+00_JPRB, 0.60023E+00_JPRB, 0.58959E+00_JPRB, 0.59595E+00_JPRB, 0.61910E+00_JPRB, 0.64150E+00_JPRB, 0.68052E+00_JPRB ,&
&  0.72043E+00_JPRB, 0.77323E+00_JPRB, 0.85389E+00_JPRB, 0.10235E+01_JPRB, 0.12088E+01_JPRB, 0.13364E+01_JPRB, 0.14850E+01_JPRB ,&
&  0.15256E+01_JPRB, 0.15571E+01_JPRB, 0.16396E+01_JPRB, 0.17477E+01_JPRB, 0.17490E+01_JPRB, 0.17216E+01_JPRB, 0.16108E+01_JPRB ,&
&  0.11662E+01_JPRB, 0.84758E+00_JPRB, 0.87694E+00_JPRB, 0.82590E+00_JPRB, 0.83145E+00_JPRB, 0.83931E+00_JPRB, 0.85715E+00_JPRB ,&
&  0.89431E+00_JPRB, 0.92572E+00_JPRB, 0.95830E+00_JPRB, 0.98801E+00_JPRB, 0.10382E+01_JPRB, 0.10952E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11477E+01_JPRB, 0.11759E+01_JPRB, 0.11493E+01_JPRB, 0.11755E+01_JPRB, 0.12127E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.13056E+01_JPRB, 0.12478E+01_JPRB, 0.10208E+01_JPRB, 0.10257E+01_JPRB, 0.11562E+01_JPRB, 0.10750E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.13133E+01_JPRB, 0.84827E+00_JPRB, 0.72271E+00_JPRB, 0.74361E+00_JPRB, 0.78070E+00_JPRB, 0.83327E+00_JPRB ,&
&  0.82436E+00_JPRB, 0.83495E+00_JPRB, 0.85942E+00_JPRB, 0.86640E+00_JPRB, 0.87059E+00_JPRB, 0.84773E+00_JPRB, 0.82816E+00_JPRB ,&
&  0.76689E+00_JPRB, 0.73724E+00_JPRB, 0.73187E+00_JPRB, 0.72771E+00_JPRB, 0.73444E+00_JPRB, 0.74618E+00_JPRB, 0.71919E+00_JPRB ,&
&  0.69827E+00_JPRB, 0.68743E+00_JPRB, 0.73506E+00_JPRB, 0.70321E+00_JPRB, 0.69167E+00_JPRB, 0.75340E+00_JPRB, 0.73729E+00_JPRB ,&
&  0.75958E+00_JPRB, 0.76344E+00_JPRB, 0.76024E+00_JPRB, 0.75359E+00_JPRB, 0.74840E+00_JPRB, 0.73536E+00_JPRB, 0.72118E+00_JPRB ,&
&  0.70462E+00_JPRB, 0.68506E+00_JPRB, 0.66465E+00_JPRB, 0.64697E+00_JPRB, 0.63218E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.62462E+00_JPRB, 0.63040E+00_JPRB, 0.64278E+00_JPRB, 0.64734E+00_JPRB, 0.64559E+00_JPRB, 0.65317E+00_JPRB, 0.65090E+00_JPRB ,&
&  0.63922E+00_JPRB, 0.62885E+00_JPRB, 0.61556E+00_JPRB, 0.62629E+00_JPRB, 0.64521E+00_JPRB, 0.68355E+00_JPRB, 0.73303E+00_JPRB ,&
&  0.80510E+00_JPRB, 0.91313E+00_JPRB, 0.10545E+01_JPRB, 0.11624E+01_JPRB, 0.13038E+01_JPRB, 0.13597E+01_JPRB, 0.13814E+01_JPRB ,&
&  0.14261E+01_JPRB, 0.14770E+01_JPRB, 0.15007E+01_JPRB, 0.14923E+01_JPRB, 0.14201E+01_JPRB, 0.13593E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.86672E+00_JPRB, 0.83423E+00_JPRB, 0.81965E+00_JPRB, 0.80654E+00_JPRB, 0.79526E+00_JPRB, 0.81711E+00_JPRB ,&
&  0.85429E+00_JPRB, 0.89366E+00_JPRB, 0.94421E+00_JPRB, 0.98564E+00_JPRB, 0.10258E+01_JPRB, 0.10537E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.11149E+01_JPRB, 0.11603E+01_JPRB, 0.10959E+01_JPRB, 0.11445E+01_JPRB, 0.11187E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.10445E+01_JPRB, 0.94511E+00_JPRB, 0.95234E+00_JPRB, 0.95482E+00_JPRB, 0.96578E+00_JPRB, 0.95053E+00_JPRB, 0.94895E+00_JPRB ,&
&  0.10322E+01_JPRB, 0.10754E+01_JPRB, 0.85936E+00_JPRB, 0.75369E+00_JPRB, 0.76845E+00_JPRB, 0.71682E+00_JPRB, 0.82777E+00_JPRB ,&
&  0.84550E+00_JPRB, 0.85430E+00_JPRB, 0.86809E+00_JPRB, 0.90931E+00_JPRB, 0.94662E+00_JPRB, 0.97429E+00_JPRB, 0.90843E+00_JPRB ,&
&  0.85987E+00_JPRB, 0.83524E+00_JPRB, 0.82428E+00_JPRB, 0.81684E+00_JPRB, 0.82935E+00_JPRB, 0.84534E+00_JPRB, 0.85317E+00_JPRB ,&
&  0.80832E+00_JPRB, 0.78117E+00_JPRB, 0.80436E+00_JPRB, 0.78209E+00_JPRB, 0.75727E+00_JPRB, 0.80739E+00_JPRB, 0.76377E+00_JPRB ,&
&  0.75336E+00_JPRB, 0.75374E+00_JPRB, 0.74195E+00_JPRB, 0.72249E+00_JPRB, 0.71176E+00_JPRB, 0.69292E+00_JPRB, 0.68365E+00_JPRB ,&
&  0.67847E+00_JPRB, 0.66424E+00_JPRB, 0.65220E+00_JPRB, 0.64731E+00_JPRB, 0.63109E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.62748E+00_JPRB, 0.62590E+00_JPRB, 0.63859E+00_JPRB, 0.64058E+00_JPRB, 0.63795E+00_JPRB, 0.64338E+00_JPRB, 0.64863E+00_JPRB ,&
&  0.64597E+00_JPRB, 0.64600E+00_JPRB, 0.64215E+00_JPRB, 0.64136E+00_JPRB, 0.64415E+00_JPRB, 0.66676E+00_JPRB, 0.71264E+00_JPRB ,&
&  0.77382E+00_JPRB, 0.87352E+00_JPRB, 0.10081E+01_JPRB, 0.11329E+01_JPRB, 0.12707E+01_JPRB, 0.13396E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.13522E+01_JPRB, 0.14010E+01_JPRB, 0.13805E+01_JPRB, 0.13434E+01_JPRB, 0.10745E+01_JPRB, 0.10857E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.90947E+00_JPRB, 0.84166E+00_JPRB, 0.81269E+00_JPRB, 0.77539E+00_JPRB, 0.76238E+00_JPRB, 0.76603E+00_JPRB ,&
&  0.78627E+00_JPRB, 0.82061E+00_JPRB, 0.87015E+00_JPRB, 0.93606E+00_JPRB, 0.98109E+00_JPRB, 0.99704E+00_JPRB, 0.10089E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.10185E+01_JPRB, 0.10189E+01_JPRB, 0.10013E+01_JPRB, 0.98733E+00_JPRB, 0.96035E+00_JPRB, 0.94894E+00_JPRB ,&
&  0.94222E+00_JPRB, 0.88125E+00_JPRB, 0.87470E+00_JPRB, 0.87810E+00_JPRB, 0.83850E+00_JPRB, 0.79903E+00_JPRB, 0.82095E+00_JPRB ,&
&  0.88983E+00_JPRB, 0.10126E+01_JPRB, 0.85962E+00_JPRB, 0.86458E+00_JPRB, 0.93337E+00_JPRB, 0.83626E+00_JPRB, 0.79633E+00_JPRB ,&
&  0.83838E+00_JPRB, 0.86480E+00_JPRB, 0.88470E+00_JPRB, 0.92257E+00_JPRB, 0.97935E+00_JPRB, 0.10549E+01_JPRB, 0.94959E+00_JPRB ,&
&  0.94006E+00_JPRB, 0.90381E+00_JPRB, 0.91192E+00_JPRB, 0.91112E+00_JPRB, 0.89783E+00_JPRB, 0.86275E+00_JPRB, 0.86610E+00_JPRB ,&
&  0.80990E+00_JPRB, 0.86147E+00_JPRB, 0.95379E+00_JPRB, 0.10668E+01_JPRB, 0.12333E+01_JPRB, 0.84378E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.78457E+00_JPRB, 0.76792E+00_JPRB, 0.75060E+00_JPRB, 0.73092E+00_JPRB, 0.71358E+00_JPRB, 0.69761E+00_JPRB, 0.68903E+00_JPRB ,&
&  0.68051E+00_JPRB, 0.66612E+00_JPRB, 0.65638E+00_JPRB, 0.65184E+00_JPRB, 0.63529E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.64366E+00_JPRB, 0.63996E+00_JPRB, 0.64625E+00_JPRB, 0.65638E+00_JPRB, 0.66579E+00_JPRB, 0.66441E+00_JPRB, 0.65508E+00_JPRB ,&
&  0.63341E+00_JPRB, 0.62625E+00_JPRB, 0.62822E+00_JPRB, 0.63126E+00_JPRB, 0.62067E+00_JPRB, 0.61521E+00_JPRB, 0.63121E+00_JPRB ,&
&  0.67487E+00_JPRB, 0.74970E+00_JPRB, 0.85521E+00_JPRB, 0.98321E+00_JPRB, 0.11508E+01_JPRB, 0.12736E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14456E+01_JPRB, 0.13019E+01_JPRB, 0.93680E+00_JPRB, 0.81253E+00_JPRB, 0.92740E+00_JPRB, 0.90265E+00_JPRB ,&
&  0.86331E+00_JPRB, 0.85090E+00_JPRB, 0.83793E+00_JPRB, 0.81938E+00_JPRB, 0.78877E+00_JPRB, 0.76672E+00_JPRB, 0.74704E+00_JPRB ,&
&  0.74197E+00_JPRB, 0.75173E+00_JPRB, 0.78447E+00_JPRB, 0.84833E+00_JPRB, 0.91205E+00_JPRB, 0.94922E+00_JPRB, 0.96561E+00_JPRB ,&
&  0.96846E+00_JPRB, 0.95899E+00_JPRB, 0.95186E+00_JPRB, 0.93753E+00_JPRB, 0.91128E+00_JPRB, 0.90466E+00_JPRB, 0.89161E+00_JPRB ,&
&  0.87874E+00_JPRB, 0.85884E+00_JPRB, 0.84142E+00_JPRB, 0.83136E+00_JPRB, 0.81171E+00_JPRB, 0.73504E+00_JPRB, 0.74181E+00_JPRB ,&
&  0.80154E+00_JPRB, 0.89444E+00_JPRB, 0.88561E+00_JPRB, 0.97253E+00_JPRB, 0.67501E+00_JPRB, 0.71806E+00_JPRB, 0.77423E+00_JPRB ,&
&  0.77614E+00_JPRB, 0.82263E+00_JPRB, 0.85944E+00_JPRB, 0.89986E+00_JPRB, 0.95099E+00_JPRB, 0.95092E+00_JPRB, 0.10305E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.95442E+00_JPRB, 0.95216E+00_JPRB, 0.97592E+00_JPRB, 0.96501E+00_JPRB, 0.89730E+00_JPRB, 0.86083E+00_JPRB ,&
&  0.80981E+00_JPRB, 0.86663E+00_JPRB, 0.87312E+00_JPRB, 0.91970E+00_JPRB, 0.96509E+00_JPRB, 0.78722E+00_JPRB, 0.79643E+00_JPRB ,&
&  0.79634E+00_JPRB, 0.78176E+00_JPRB, 0.77416E+00_JPRB, 0.75066E+00_JPRB, 0.73435E+00_JPRB, 0.71954E+00_JPRB, 0.70794E+00_JPRB ,&
&  0.70344E+00_JPRB, 0.69450E+00_JPRB, 0.68112E+00_JPRB, 0.66795E+00_JPRB, 0.65445E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.66555E+00_JPRB, 0.66542E+00_JPRB, 0.67408E+00_JPRB, 0.67908E+00_JPRB, 0.68914E+00_JPRB, 0.69962E+00_JPRB, 0.68984E+00_JPRB ,&
&  0.63882E+00_JPRB, 0.59874E+00_JPRB, 0.59132E+00_JPRB, 0.59691E+00_JPRB, 0.59899E+00_JPRB, 0.59422E+00_JPRB, 0.59757E+00_JPRB ,&
&  0.61329E+00_JPRB, 0.64712E+00_JPRB, 0.70403E+00_JPRB, 0.82941E+00_JPRB, 0.10071E+01_JPRB, 0.11883E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.12936E+01_JPRB, 0.97846E+00_JPRB, 0.11860E+01_JPRB, 0.11623E+01_JPRB, 0.98603E+00_JPRB, 0.90904E+00_JPRB, 0.89511E+00_JPRB ,&
&  0.88184E+00_JPRB, 0.88739E+00_JPRB, 0.88053E+00_JPRB, 0.86108E+00_JPRB, 0.80737E+00_JPRB, 0.78116E+00_JPRB, 0.75192E+00_JPRB ,&
&  0.73716E+00_JPRB, 0.73019E+00_JPRB, 0.73923E+00_JPRB, 0.77887E+00_JPRB, 0.82932E+00_JPRB, 0.88585E+00_JPRB, 0.93399E+00_JPRB ,&
&  0.93817E+00_JPRB, 0.91347E+00_JPRB, 0.90316E+00_JPRB, 0.87349E+00_JPRB, 0.88005E+00_JPRB, 0.88389E+00_JPRB, 0.87074E+00_JPRB ,&
&  0.86068E+00_JPRB, 0.84535E+00_JPRB, 0.85636E+00_JPRB, 0.86062E+00_JPRB, 0.85372E+00_JPRB, 0.77835E+00_JPRB, 0.78494E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.81157E+00_JPRB, 0.85199E+00_JPRB, 0.90888E+00_JPRB, 0.71218E+00_JPRB, 0.74240E+00_JPRB, 0.69802E+00_JPRB ,&
&  0.72916E+00_JPRB, 0.74424E+00_JPRB, 0.78426E+00_JPRB, 0.81756E+00_JPRB, 0.87901E+00_JPRB, 0.90993E+00_JPRB, 0.10526E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.10222E+01_JPRB, 0.99293E+00_JPRB, 0.10205E+01_JPRB, 0.96326E+00_JPRB, 0.92617E+00_JPRB, 0.90629E+00_JPRB ,&
&  0.94597E+00_JPRB, 0.92806E+00_JPRB, 0.91489E+00_JPRB, 0.91150E+00_JPRB, 0.93449E+00_JPRB, 0.82266E+00_JPRB, 0.79403E+00_JPRB ,&
&  0.77763E+00_JPRB, 0.78681E+00_JPRB, 0.78336E+00_JPRB, 0.77259E+00_JPRB, 0.76078E+00_JPRB, 0.74227E+00_JPRB, 0.72450E+00_JPRB ,&
&  0.71670E+00_JPRB, 0.70787E+00_JPRB, 0.69858E+00_JPRB, 0.68982E+00_JPRB, 0.67773E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.68236E+00_JPRB, 0.67748E+00_JPRB, 0.68684E+00_JPRB, 0.70753E+00_JPRB, 0.72855E+00_JPRB, 0.78321E+00_JPRB, 0.83717E+00_JPRB ,&
&  0.96899E+00_JPRB, 0.67596E+00_JPRB, 0.59087E+00_JPRB, 0.56612E+00_JPRB, 0.56758E+00_JPRB, 0.57340E+00_JPRB, 0.58088E+00_JPRB ,&
&  0.58901E+00_JPRB, 0.60863E+00_JPRB, 0.63333E+00_JPRB, 0.67379E+00_JPRB, 0.82533E+00_JPRB, 0.10275E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.10479E+01_JPRB, 0.11446E+01_JPRB, 0.14001E+01_JPRB, 0.12060E+01_JPRB, 0.10913E+01_JPRB, 0.10612E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.96326E+00_JPRB, 0.92540E+00_JPRB, 0.89421E+00_JPRB, 0.85035E+00_JPRB, 0.82437E+00_JPRB, 0.80137E+00_JPRB, 0.78009E+00_JPRB ,&
&  0.75513E+00_JPRB, 0.73782E+00_JPRB, 0.73213E+00_JPRB, 0.75141E+00_JPRB, 0.77967E+00_JPRB, 0.82137E+00_JPRB, 0.88626E+00_JPRB ,&
&  0.92052E+00_JPRB, 0.89496E+00_JPRB, 0.87104E+00_JPRB, 0.83739E+00_JPRB, 0.85773E+00_JPRB, 0.87085E+00_JPRB, 0.86746E+00_JPRB ,&
&  0.85996E+00_JPRB, 0.88565E+00_JPRB, 0.90244E+00_JPRB, 0.87214E+00_JPRB, 0.81088E+00_JPRB, 0.86117E+00_JPRB, 0.80483E+00_JPRB ,&
&  0.80106E+00_JPRB, 0.76670E+00_JPRB, 0.84463E+00_JPRB, 0.73157E+00_JPRB, 0.77013E+00_JPRB, 0.78653E+00_JPRB, 0.75386E+00_JPRB ,&
&  0.70830E+00_JPRB, 0.70243E+00_JPRB, 0.70239E+00_JPRB, 0.72042E+00_JPRB, 0.78649E+00_JPRB, 0.82757E+00_JPRB, 0.97525E+00_JPRB ,&
&  0.98695E+00_JPRB, 0.10918E+01_JPRB, 0.10508E+01_JPRB, 0.10110E+01_JPRB, 0.91753E+00_JPRB, 0.86784E+00_JPRB, 0.87830E+00_JPRB ,&
&  0.96300E+00_JPRB, 0.99342E+00_JPRB, 0.98652E+00_JPRB, 0.94630E+00_JPRB, 0.91397E+00_JPRB, 0.87114E+00_JPRB, 0.85230E+00_JPRB ,&
&  0.83580E+00_JPRB, 0.81982E+00_JPRB, 0.80074E+00_JPRB, 0.79278E+00_JPRB, 0.77568E+00_JPRB, 0.75485E+00_JPRB, 0.72707E+00_JPRB ,&
&  0.71351E+00_JPRB, 0.70508E+00_JPRB, 0.69853E+00_JPRB, 0.69540E+00_JPRB, 0.69165E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.68181E+00_JPRB, 0.66986E+00_JPRB, 0.65467E+00_JPRB, 0.65775E+00_JPRB, 0.66277E+00_JPRB, 0.68244E+00_JPRB, 0.72156E+00_JPRB ,&
&  0.80750E+00_JPRB, 0.67071E+00_JPRB, 0.60178E+00_JPRB, 0.55527E+00_JPRB, 0.54117E+00_JPRB, 0.54809E+00_JPRB, 0.55406E+00_JPRB ,&
&  0.56581E+00_JPRB, 0.57695E+00_JPRB, 0.59168E+00_JPRB, 0.58412E+00_JPRB, 0.66207E+00_JPRB, 0.81944E+00_JPRB, 0.80941E+00_JPRB ,&
&  0.84601E+00_JPRB, 0.14443E+01_JPRB, 0.13793E+01_JPRB, 0.12960E+01_JPRB, 0.12050E+01_JPRB, 0.12056E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.10048E+01_JPRB, 0.96172E+00_JPRB, 0.94290E+00_JPRB, 0.89309E+00_JPRB, 0.86200E+00_JPRB, 0.82128E+00_JPRB, 0.80214E+00_JPRB ,&
&  0.78090E+00_JPRB, 0.76475E+00_JPRB, 0.74456E+00_JPRB, 0.74674E+00_JPRB, 0.76311E+00_JPRB, 0.78994E+00_JPRB, 0.83764E+00_JPRB ,&
&  0.88765E+00_JPRB, 0.89174E+00_JPRB, 0.85947E+00_JPRB, 0.85738E+00_JPRB, 0.86906E+00_JPRB, 0.88946E+00_JPRB, 0.89711E+00_JPRB ,&
&  0.86974E+00_JPRB, 0.85252E+00_JPRB, 0.94359E+00_JPRB, 0.96538E+00_JPRB, 0.94766E+00_JPRB, 0.96413E+00_JPRB, 0.85436E+00_JPRB ,&
&  0.87308E+00_JPRB, 0.77965E+00_JPRB, 0.80670E+00_JPRB, 0.70452E+00_JPRB, 0.76138E+00_JPRB, 0.80368E+00_JPRB, 0.78310E+00_JPRB ,&
&  0.71787E+00_JPRB, 0.71629E+00_JPRB, 0.69086E+00_JPRB, 0.68825E+00_JPRB, 0.72617E+00_JPRB, 0.77979E+00_JPRB, 0.84969E+00_JPRB ,&
&  0.92620E+00_JPRB, 0.10304E+01_JPRB, 0.11096E+01_JPRB, 0.94117E+00_JPRB, 0.87350E+00_JPRB, 0.82402E+00_JPRB, 0.82983E+00_JPRB ,&
&  0.94675E+00_JPRB, 0.98693E+00_JPRB, 0.10384E+01_JPRB, 0.10397E+01_JPRB, 0.93672E+00_JPRB, 0.90692E+00_JPRB, 0.90144E+00_JPRB ,&
&  0.88923E+00_JPRB, 0.87903E+00_JPRB, 0.83771E+00_JPRB, 0.79701E+00_JPRB, 0.76983E+00_JPRB, 0.75288E+00_JPRB, 0.71878E+00_JPRB ,&
&  0.70696E+00_JPRB, 0.69523E+00_JPRB, 0.68623E+00_JPRB, 0.68671E+00_JPRB, 0.68444E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.66400E+00_JPRB, 0.65194E+00_JPRB, 0.64587E+00_JPRB, 0.63561E+00_JPRB, 0.61825E+00_JPRB, 0.62646E+00_JPRB, 0.68866E+00_JPRB ,&
&  0.69722E+00_JPRB, 0.62036E+00_JPRB, 0.58018E+00_JPRB, 0.55084E+00_JPRB, 0.53267E+00_JPRB, 0.52808E+00_JPRB, 0.53558E+00_JPRB ,&
&  0.54090E+00_JPRB, 0.55330E+00_JPRB, 0.55586E+00_JPRB, 0.56058E+00_JPRB, 0.59720E+00_JPRB, 0.69940E+00_JPRB, 0.55963E+00_JPRB ,&
&  0.74350E+00_JPRB, 0.12630E+01_JPRB, 0.14099E+01_JPRB, 0.14235E+01_JPRB, 0.14038E+01_JPRB, 0.13766E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.10574E+01_JPRB, 0.98554E+00_JPRB, 0.95084E+00_JPRB, 0.92848E+00_JPRB, 0.88292E+00_JPRB, 0.83718E+00_JPRB ,&
&  0.80617E+00_JPRB, 0.77698E+00_JPRB, 0.75093E+00_JPRB, 0.74226E+00_JPRB, 0.74840E+00_JPRB, 0.76742E+00_JPRB, 0.80384E+00_JPRB ,&
&  0.84897E+00_JPRB, 0.88572E+00_JPRB, 0.89643E+00_JPRB, 0.84899E+00_JPRB, 0.86907E+00_JPRB, 0.88236E+00_JPRB, 0.90657E+00_JPRB ,&
&  0.87267E+00_JPRB, 0.85682E+00_JPRB, 0.10163E+01_JPRB, 0.11639E+01_JPRB, 0.12088E+01_JPRB, 0.11262E+01_JPRB, 0.99282E+00_JPRB ,&
&  0.95145E+00_JPRB, 0.87353E+00_JPRB, 0.79763E+00_JPRB, 0.71819E+00_JPRB, 0.75281E+00_JPRB, 0.80594E+00_JPRB, 0.82764E+00_JPRB ,&
&  0.77926E+00_JPRB, 0.73242E+00_JPRB, 0.70268E+00_JPRB, 0.67734E+00_JPRB, 0.69021E+00_JPRB, 0.72454E+00_JPRB, 0.82089E+00_JPRB ,&
&  0.92281E+00_JPRB, 0.10561E+01_JPRB, 0.10614E+01_JPRB, 0.89355E+00_JPRB, 0.82566E+00_JPRB, 0.77231E+00_JPRB, 0.73429E+00_JPRB ,&
&  0.86333E+00_JPRB, 0.89925E+00_JPRB, 0.87714E+00_JPRB, 0.10201E+01_JPRB, 0.92014E+00_JPRB, 0.91019E+00_JPRB, 0.94279E+00_JPRB ,&
&  0.92607E+00_JPRB, 0.90011E+00_JPRB, 0.86685E+00_JPRB, 0.84140E+00_JPRB, 0.79221E+00_JPRB, 0.74133E+00_JPRB, 0.70830E+00_JPRB ,&
&  0.69240E+00_JPRB, 0.68744E+00_JPRB, 0.67975E+00_JPRB, 0.67565E+00_JPRB, 0.67072E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.63153E+00_JPRB, 0.62445E+00_JPRB, 0.61594E+00_JPRB, 0.60864E+00_JPRB, 0.59117E+00_JPRB, 0.58100E+00_JPRB, 0.61262E+00_JPRB ,&
&  0.61232E+00_JPRB, 0.58347E+00_JPRB, 0.53973E+00_JPRB, 0.52737E+00_JPRB, 0.51978E+00_JPRB, 0.51335E+00_JPRB, 0.51745E+00_JPRB ,&
&  0.52183E+00_JPRB, 0.53712E+00_JPRB, 0.56450E+00_JPRB, 0.56955E+00_JPRB, 0.53073E+00_JPRB, 0.48423E+00_JPRB, 0.52936E+00_JPRB ,&
&  0.92133E+00_JPRB, 0.13558E+01_JPRB, 0.14627E+01_JPRB, 0.15738E+01_JPRB, 0.16593E+01_JPRB, 0.17579E+01_JPRB, 0.15778E+01_JPRB ,&
&  0.13909E+01_JPRB, 0.12720E+01_JPRB, 0.11606E+01_JPRB, 0.10943E+01_JPRB, 0.10277E+01_JPRB, 0.96193E+00_JPRB, 0.91448E+00_JPRB ,&
&  0.86705E+00_JPRB, 0.83375E+00_JPRB, 0.79417E+00_JPRB, 0.75070E+00_JPRB, 0.75046E+00_JPRB, 0.76762E+00_JPRB, 0.78506E+00_JPRB ,&
&  0.81003E+00_JPRB, 0.86163E+00_JPRB, 0.88381E+00_JPRB, 0.84436E+00_JPRB, 0.84472E+00_JPRB, 0.86234E+00_JPRB, 0.92752E+00_JPRB ,&
&  0.95746E+00_JPRB, 0.10445E+01_JPRB, 0.11521E+01_JPRB, 0.12476E+01_JPRB, 0.13444E+01_JPRB, 0.13392E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.10889E+01_JPRB, 0.93934E+00_JPRB, 0.78183E+00_JPRB, 0.76242E+00_JPRB, 0.80980E+00_JPRB, 0.86126E+00_JPRB, 0.78904E+00_JPRB ,&
&  0.69918E+00_JPRB, 0.64583E+00_JPRB, 0.63683E+00_JPRB, 0.64456E+00_JPRB, 0.69060E+00_JPRB, 0.71677E+00_JPRB, 0.81614E+00_JPRB ,&
&  0.80597E+00_JPRB, 0.62112E+00_JPRB, 0.56210E+00_JPRB, 0.66037E+00_JPRB, 0.61334E+00_JPRB, 0.58419E+00_JPRB, 0.68943E+00_JPRB ,&
&  0.77925E+00_JPRB, 0.86370E+00_JPRB, 0.85240E+00_JPRB, 0.82105E+00_JPRB, 0.85523E+00_JPRB, 0.87390E+00_JPRB, 0.99037E+00_JPRB ,&
&  0.93428E+00_JPRB, 0.88190E+00_JPRB, 0.86336E+00_JPRB, 0.84618E+00_JPRB, 0.79878E+00_JPRB, 0.73862E+00_JPRB, 0.68975E+00_JPRB ,&
&  0.66828E+00_JPRB, 0.65741E+00_JPRB, 0.64594E+00_JPRB, 0.63971E+00_JPRB, 0.63466E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.59338E+00_JPRB, 0.59406E+00_JPRB, 0.58661E+00_JPRB, 0.57943E+00_JPRB, 0.56647E+00_JPRB, 0.55273E+00_JPRB, 0.56080E+00_JPRB ,&
&  0.55351E+00_JPRB, 0.53569E+00_JPRB, 0.51382E+00_JPRB, 0.50679E+00_JPRB, 0.50329E+00_JPRB, 0.50178E+00_JPRB, 0.50668E+00_JPRB ,&
&  0.51184E+00_JPRB, 0.52739E+00_JPRB, 0.57886E+00_JPRB, 0.59465E+00_JPRB, 0.52669E+00_JPRB, 0.45032E+00_JPRB, 0.54552E+00_JPRB ,&
&  0.98094E+00_JPRB, 0.15115E+01_JPRB, 0.17678E+01_JPRB, 0.19673E+01_JPRB, 0.19867E+01_JPRB, 0.20156E+01_JPRB, 0.19051E+01_JPRB ,&
&  0.17403E+01_JPRB, 0.15911E+01_JPRB, 0.14353E+01_JPRB, 0.12591E+01_JPRB, 0.11116E+01_JPRB, 0.10169E+01_JPRB, 0.94511E+00_JPRB ,&
&  0.90500E+00_JPRB, 0.87297E+00_JPRB, 0.83735E+00_JPRB, 0.78880E+00_JPRB, 0.76325E+00_JPRB, 0.76845E+00_JPRB, 0.77382E+00_JPRB ,&
&  0.77920E+00_JPRB, 0.82807E+00_JPRB, 0.86444E+00_JPRB, 0.86377E+00_JPRB, 0.72618E+00_JPRB, 0.83439E+00_JPRB, 0.88782E+00_JPRB ,&
&  0.98086E+00_JPRB, 0.10889E+01_JPRB, 0.11238E+01_JPRB, 0.13747E+01_JPRB, 0.15257E+01_JPRB, 0.14485E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.10469E+01_JPRB, 0.87317E+00_JPRB, 0.87667E+00_JPRB, 0.91776E+00_JPRB, 0.91307E+00_JPRB, 0.66585E+00_JPRB ,&
&  0.62314E+00_JPRB, 0.70135E+00_JPRB, 0.67725E+00_JPRB, 0.60437E+00_JPRB, 0.68461E+00_JPRB, 0.71288E+00_JPRB, 0.61511E+00_JPRB ,&
&  0.40593E+00_JPRB, 0.36283E+00_JPRB, 0.37363E+00_JPRB, 0.39145E+00_JPRB, 0.37836E+00_JPRB, 0.44919E+00_JPRB, 0.74101E+00_JPRB ,&
&  0.71384E+00_JPRB, 0.84378E+00_JPRB, 0.84427E+00_JPRB, 0.82000E+00_JPRB, 0.82431E+00_JPRB, 0.86844E+00_JPRB, 0.10881E+01_JPRB ,&
&  0.97641E+00_JPRB, 0.92248E+00_JPRB, 0.86354E+00_JPRB, 0.82316E+00_JPRB, 0.77906E+00_JPRB, 0.72490E+00_JPRB, 0.68342E+00_JPRB ,&
&  0.65680E+00_JPRB, 0.64204E+00_JPRB, 0.62420E+00_JPRB, 0.61012E+00_JPRB, 0.59618E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.56637E+00_JPRB, 0.55839E+00_JPRB, 0.55083E+00_JPRB, 0.54309E+00_JPRB, 0.53091E+00_JPRB, 0.52088E+00_JPRB, 0.52058E+00_JPRB ,&
&  0.51103E+00_JPRB, 0.49374E+00_JPRB, 0.48025E+00_JPRB, 0.47472E+00_JPRB, 0.47999E+00_JPRB, 0.49261E+00_JPRB, 0.50043E+00_JPRB ,&
&  0.51254E+00_JPRB, 0.54675E+00_JPRB, 0.65638E+00_JPRB, 0.63362E+00_JPRB, 0.61149E+00_JPRB, 0.58461E+00_JPRB, 0.62145E+00_JPRB ,&
&  0.91132E+00_JPRB, 0.14054E+01_JPRB, 0.17983E+01_JPRB, 0.20810E+01_JPRB, 0.23979E+01_JPRB, 0.24913E+01_JPRB, 0.23939E+01_JPRB ,&
&  0.21338E+01_JPRB, 0.18993E+01_JPRB, 0.16559E+01_JPRB, 0.13920E+01_JPRB, 0.11956E+01_JPRB, 0.10705E+01_JPRB, 0.97904E+00_JPRB ,&
&  0.90747E+00_JPRB, 0.88063E+00_JPRB, 0.84754E+00_JPRB, 0.81484E+00_JPRB, 0.78627E+00_JPRB, 0.78564E+00_JPRB, 0.78916E+00_JPRB ,&
&  0.78589E+00_JPRB, 0.79732E+00_JPRB, 0.82562E+00_JPRB, 0.82602E+00_JPRB, 0.72785E+00_JPRB, 0.65391E+00_JPRB, 0.76782E+00_JPRB ,&
&  0.91725E+00_JPRB, 0.11720E+01_JPRB, 0.13106E+01_JPRB, 0.16434E+01_JPRB, 0.16638E+01_JPRB, 0.15128E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.13324E+01_JPRB, 0.12833E+01_JPRB, 0.96146E+00_JPRB, 0.97244E+00_JPRB, 0.10547E+01_JPRB, 0.83102E+00_JPRB, 0.72923E+00_JPRB ,&
&  0.80447E+00_JPRB, 0.76150E+00_JPRB, 0.70043E+00_JPRB, 0.59324E+00_JPRB, 0.69102E+00_JPRB, 0.63332E+00_JPRB, 0.40332E+00_JPRB ,&
&  0.35640E+00_JPRB, 0.34376E+00_JPRB, 0.33265E+00_JPRB, 0.32823E+00_JPRB, 0.32439E+00_JPRB, 0.36637E+00_JPRB, 0.49206E+00_JPRB ,&
&  0.55122E+00_JPRB, 0.70486E+00_JPRB, 0.79060E+00_JPRB, 0.83746E+00_JPRB, 0.85947E+00_JPRB, 0.98267E+00_JPRB, 0.12160E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.10329E+01_JPRB, 0.89538E+00_JPRB, 0.81447E+00_JPRB, 0.75586E+00_JPRB, 0.71002E+00_JPRB, 0.67145E+00_JPRB ,&
&  0.64005E+00_JPRB, 0.61760E+00_JPRB, 0.59868E+00_JPRB, 0.59020E+00_JPRB, 0.57736E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.52570E+00_JPRB, 0.51506E+00_JPRB, 0.50650E+00_JPRB, 0.49849E+00_JPRB, 0.48693E+00_JPRB, 0.47557E+00_JPRB, 0.48141E+00_JPRB ,&
&  0.47936E+00_JPRB, 0.46203E+00_JPRB, 0.45202E+00_JPRB, 0.44942E+00_JPRB, 0.46288E+00_JPRB, 0.48693E+00_JPRB, 0.49799E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.58709E+00_JPRB, 0.69282E+00_JPRB, 0.60566E+00_JPRB, 0.53939E+00_JPRB, 0.50733E+00_JPRB, 0.52185E+00_JPRB ,&
&  0.83097E+00_JPRB, 0.11608E+01_JPRB, 0.14873E+01_JPRB, 0.18944E+01_JPRB, 0.22911E+01_JPRB, 0.23235E+01_JPRB, 0.25333E+01_JPRB ,&
&  0.23838E+01_JPRB, 0.20475E+01_JPRB, 0.17372E+01_JPRB, 0.14525E+01_JPRB, 0.12517E+01_JPRB, 0.11016E+01_JPRB, 0.98911E+00_JPRB ,&
&  0.92640E+00_JPRB, 0.89314E+00_JPRB, 0.86526E+00_JPRB, 0.83927E+00_JPRB, 0.82306E+00_JPRB, 0.81396E+00_JPRB, 0.81891E+00_JPRB ,&
&  0.81074E+00_JPRB, 0.79274E+00_JPRB, 0.79135E+00_JPRB, 0.79260E+00_JPRB, 0.80092E+00_JPRB, 0.79633E+00_JPRB, 0.78442E+00_JPRB ,&
&  0.80027E+00_JPRB, 0.94796E+00_JPRB, 0.14550E+01_JPRB, 0.17078E+01_JPRB, 0.16806E+01_JPRB, 0.16126E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.14120E+01_JPRB, 0.11644E+01_JPRB, 0.11878E+01_JPRB, 0.10545E+01_JPRB, 0.74272E+00_JPRB, 0.82646E+00_JPRB ,&
&  0.89396E+00_JPRB, 0.86215E+00_JPRB, 0.73115E+00_JPRB, 0.56798E+00_JPRB, 0.55492E+00_JPRB, 0.43334E+00_JPRB, 0.34338E+00_JPRB ,&
&  0.33278E+00_JPRB, 0.32293E+00_JPRB, 0.30459E+00_JPRB, 0.30727E+00_JPRB, 0.30149E+00_JPRB, 0.33478E+00_JPRB, 0.40631E+00_JPRB ,&
&  0.48837E+00_JPRB, 0.59717E+00_JPRB, 0.71210E+00_JPRB, 0.79226E+00_JPRB, 0.83062E+00_JPRB, 0.92594E+00_JPRB, 0.10385E+01_JPRB ,&
&  0.96358E+00_JPRB, 0.98542E+00_JPRB, 0.86936E+00_JPRB, 0.77390E+00_JPRB, 0.70502E+00_JPRB, 0.66079E+00_JPRB, 0.62727E+00_JPRB ,&
&  0.60174E+00_JPRB, 0.57464E+00_JPRB, 0.55502E+00_JPRB, 0.54500E+00_JPRB, 0.53682E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.48492E+00_JPRB, 0.47677E+00_JPRB, 0.47052E+00_JPRB, 0.46182E+00_JPRB, 0.45300E+00_JPRB, 0.44218E+00_JPRB, 0.44719E+00_JPRB ,&
&  0.45472E+00_JPRB, 0.43509E+00_JPRB, 0.41901E+00_JPRB, 0.42269E+00_JPRB, 0.44343E+00_JPRB, 0.47245E+00_JPRB, 0.50445E+00_JPRB ,&
&  0.54286E+00_JPRB, 0.74722E+00_JPRB, 0.56743E+00_JPRB, 0.45285E+00_JPRB, 0.49525E+00_JPRB, 0.51933E+00_JPRB, 0.49310E+00_JPRB ,&
&  0.72707E+00_JPRB, 0.96952E+00_JPRB, 0.12374E+01_JPRB, 0.17471E+01_JPRB, 0.25103E+01_JPRB, 0.25821E+01_JPRB, 0.27160E+01_JPRB ,&
&  0.27031E+01_JPRB, 0.22803E+01_JPRB, 0.18450E+01_JPRB, 0.15027E+01_JPRB, 0.12700E+01_JPRB, 0.11130E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.95832E+00_JPRB, 0.91743E+00_JPRB, 0.88523E+00_JPRB, 0.87112E+00_JPRB, 0.85641E+00_JPRB, 0.83961E+00_JPRB, 0.83908E+00_JPRB ,&
&  0.82151E+00_JPRB, 0.79517E+00_JPRB, 0.78891E+00_JPRB, 0.78994E+00_JPRB, 0.81697E+00_JPRB, 0.79484E+00_JPRB, 0.11158E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.12323E+01_JPRB, 0.14693E+01_JPRB, 0.19141E+01_JPRB, 0.19481E+01_JPRB, 0.17411E+01_JPRB, 0.17755E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.11707E+01_JPRB, 0.11269E+01_JPRB, 0.10717E+01_JPRB, 0.83358E+00_JPRB, 0.85452E+00_JPRB, 0.11914E+01_JPRB ,&
&  0.95731E+00_JPRB, 0.10271E+01_JPRB, 0.10531E+01_JPRB, 0.10262E+01_JPRB, 0.53725E+00_JPRB, 0.39443E+00_JPRB, 0.51892E+00_JPRB ,&
&  0.54778E+00_JPRB, 0.41420E+00_JPRB, 0.33846E+00_JPRB, 0.39140E+00_JPRB, 0.32809E+00_JPRB, 0.36447E+00_JPRB, 0.43640E+00_JPRB ,&
&  0.44804E+00_JPRB, 0.49020E+00_JPRB, 0.65361E+00_JPRB, 0.74064E+00_JPRB, 0.80874E+00_JPRB, 0.88037E+00_JPRB, 0.86112E+00_JPRB ,&
&  0.85936E+00_JPRB, 0.90281E+00_JPRB, 0.87727E+00_JPRB, 0.76229E+00_JPRB, 0.67615E+00_JPRB, 0.61933E+00_JPRB, 0.58033E+00_JPRB ,&
&  0.55249E+00_JPRB, 0.52243E+00_JPRB, 0.50465E+00_JPRB, 0.49811E+00_JPRB, 0.49283E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.45320E+00_JPRB, 0.44852E+00_JPRB, 0.44421E+00_JPRB, 0.43517E+00_JPRB, 0.42477E+00_JPRB, 0.41724E+00_JPRB, 0.41983E+00_JPRB ,&
&  0.43051E+00_JPRB, 0.40936E+00_JPRB, 0.39570E+00_JPRB, 0.40000E+00_JPRB, 0.41602E+00_JPRB, 0.43972E+00_JPRB, 0.47611E+00_JPRB ,&
&  0.51914E+00_JPRB, 0.54941E+00_JPRB, 0.42130E+00_JPRB, 0.38146E+00_JPRB, 0.50758E+00_JPRB, 0.50077E+00_JPRB, 0.54863E+00_JPRB ,&
&  0.69245E+00_JPRB, 0.83878E+00_JPRB, 0.10750E+01_JPRB, 0.15438E+01_JPRB, 0.19864E+01_JPRB, 0.24624E+01_JPRB, 0.27407E+01_JPRB ,&
&  0.28529E+01_JPRB, 0.25866E+01_JPRB, 0.21657E+01_JPRB, 0.17378E+01_JPRB, 0.14076E+01_JPRB, 0.11931E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.97327E+00_JPRB, 0.90531E+00_JPRB, 0.86461E+00_JPRB, 0.82972E+00_JPRB, 0.83061E+00_JPRB, 0.82146E+00_JPRB, 0.82807E+00_JPRB ,&
&  0.82859E+00_JPRB, 0.80902E+00_JPRB, 0.81075E+00_JPRB, 0.80263E+00_JPRB, 0.79133E+00_JPRB, 0.81637E+00_JPRB, 0.96639E+00_JPRB ,&
&  0.11615E+01_JPRB, 0.13551E+01_JPRB, 0.15940E+01_JPRB, 0.17999E+01_JPRB, 0.18548E+01_JPRB, 0.16755E+01_JPRB, 0.18674E+01_JPRB ,&
&  0.15187E+01_JPRB, 0.12032E+01_JPRB, 0.96432E+00_JPRB, 0.74634E+00_JPRB, 0.87352E+00_JPRB, 0.12400E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.11907E+01_JPRB, 0.11616E+01_JPRB, 0.10064E+01_JPRB, 0.88781E+00_JPRB, 0.60748E+00_JPRB, 0.66803E+00_JPRB ,&
&  0.66117E+00_JPRB, 0.62465E+00_JPRB, 0.57098E+00_JPRB, 0.51039E+00_JPRB, 0.42021E+00_JPRB, 0.47823E+00_JPRB, 0.48568E+00_JPRB ,&
&  0.45369E+00_JPRB, 0.44925E+00_JPRB, 0.56851E+00_JPRB, 0.68198E+00_JPRB, 0.73095E+00_JPRB, 0.73620E+00_JPRB, 0.76504E+00_JPRB ,&
&  0.72983E+00_JPRB, 0.74337E+00_JPRB, 0.70877E+00_JPRB, 0.65282E+00_JPRB, 0.60449E+00_JPRB, 0.56136E+00_JPRB, 0.52293E+00_JPRB ,&
&  0.49526E+00_JPRB, 0.47506E+00_JPRB, 0.45846E+00_JPRB, 0.45561E+00_JPRB, 0.45506E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.43681E+00_JPRB, 0.43530E+00_JPRB, 0.43203E+00_JPRB, 0.42629E+00_JPRB, 0.41698E+00_JPRB, 0.40737E+00_JPRB, 0.40803E+00_JPRB ,&
&  0.41178E+00_JPRB, 0.39128E+00_JPRB, 0.38436E+00_JPRB, 0.38789E+00_JPRB, 0.40161E+00_JPRB, 0.42142E+00_JPRB, 0.45042E+00_JPRB ,&
&  0.49317E+00_JPRB, 0.45254E+00_JPRB, 0.35433E+00_JPRB, 0.38128E+00_JPRB, 0.44431E+00_JPRB, 0.45203E+00_JPRB, 0.53854E+00_JPRB ,&
&  0.63121E+00_JPRB, 0.73657E+00_JPRB, 0.88344E+00_JPRB, 0.11912E+01_JPRB, 0.15316E+01_JPRB, 0.18767E+01_JPRB, 0.21645E+01_JPRB ,&
&  0.22348E+01_JPRB, 0.23317E+01_JPRB, 0.22626E+01_JPRB, 0.20163E+01_JPRB, 0.16817E+01_JPRB, 0.13972E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.95145E+00_JPRB, 0.86496E+00_JPRB, 0.81530E+00_JPRB, 0.78955E+00_JPRB, 0.78901E+00_JPRB, 0.80568E+00_JPRB ,&
&  0.82927E+00_JPRB, 0.82026E+00_JPRB, 0.80447E+00_JPRB, 0.82362E+00_JPRB, 0.79550E+00_JPRB, 0.84315E+00_JPRB, 0.95205E+00_JPRB ,&
&  0.13792E+01_JPRB, 0.16594E+01_JPRB, 0.17923E+01_JPRB, 0.20244E+01_JPRB, 0.16792E+01_JPRB, 0.15762E+01_JPRB, 0.18091E+01_JPRB ,&
&  0.19965E+01_JPRB, 0.20007E+01_JPRB, 0.19899E+01_JPRB, 0.12597E+01_JPRB, 0.10306E+01_JPRB, 0.14039E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.11899E+01_JPRB, 0.11350E+01_JPRB, 0.11826E+01_JPRB, 0.95412E+00_JPRB, 0.79100E+00_JPRB, 0.57295E+00_JPRB ,&
&  0.51847E+00_JPRB, 0.46891E+00_JPRB, 0.51746E+00_JPRB, 0.48988E+00_JPRB, 0.47206E+00_JPRB, 0.47296E+00_JPRB, 0.46473E+00_JPRB ,&
&  0.45749E+00_JPRB, 0.44913E+00_JPRB, 0.47309E+00_JPRB, 0.55204E+00_JPRB, 0.63034E+00_JPRB, 0.57072E+00_JPRB, 0.59825E+00_JPRB ,&
&  0.59731E+00_JPRB, 0.60625E+00_JPRB, 0.59779E+00_JPRB, 0.59142E+00_JPRB, 0.58318E+00_JPRB, 0.54999E+00_JPRB, 0.50494E+00_JPRB ,&
&  0.46771E+00_JPRB, 0.44915E+00_JPRB, 0.43962E+00_JPRB, 0.43707E+00_JPRB, 0.43775E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.41785E+00_JPRB, 0.42018E+00_JPRB, 0.41942E+00_JPRB, 0.41675E+00_JPRB, 0.41295E+00_JPRB, 0.40704E+00_JPRB, 0.40586E+00_JPRB ,&
&  0.40289E+00_JPRB, 0.38849E+00_JPRB, 0.37870E+00_JPRB, 0.38064E+00_JPRB, 0.39156E+00_JPRB, 0.40483E+00_JPRB, 0.42568E+00_JPRB ,&
&  0.47527E+00_JPRB, 0.49724E+00_JPRB, 0.39103E+00_JPRB, 0.34432E+00_JPRB, 0.33103E+00_JPRB, 0.40660E+00_JPRB, 0.48872E+00_JPRB ,&
&  0.59768E+00_JPRB, 0.65382E+00_JPRB, 0.78271E+00_JPRB, 0.96917E+00_JPRB, 0.11739E+01_JPRB, 0.13674E+01_JPRB, 0.15404E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.17862E+01_JPRB, 0.18461E+01_JPRB, 0.17519E+01_JPRB, 0.16909E+01_JPRB, 0.15067E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.10071E+01_JPRB, 0.89017E+00_JPRB, 0.80075E+00_JPRB, 0.75184E+00_JPRB, 0.73433E+00_JPRB, 0.75100E+00_JPRB ,&
&  0.81452E+00_JPRB, 0.84020E+00_JPRB, 0.83196E+00_JPRB, 0.84210E+00_JPRB, 0.90373E+00_JPRB, 0.10037E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.13560E+01_JPRB, 0.20212E+01_JPRB, 0.21808E+01_JPRB, 0.22938E+01_JPRB, 0.21119E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.20827E+01_JPRB, 0.20973E+01_JPRB, 0.21124E+01_JPRB, 0.15825E+01_JPRB, 0.17203E+01_JPRB, 0.16424E+01_JPRB, 0.14450E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.12357E+01_JPRB, 0.11307E+01_JPRB, 0.10939E+01_JPRB, 0.11054E+01_JPRB, 0.11067E+01_JPRB, 0.83713E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.64429E+00_JPRB, 0.57881E+00_JPRB, 0.49306E+00_JPRB, 0.42342E+00_JPRB, 0.43318E+00_JPRB, 0.44393E+00_JPRB ,&
&  0.44707E+00_JPRB, 0.45002E+00_JPRB, 0.44417E+00_JPRB, 0.47206E+00_JPRB, 0.52034E+00_JPRB, 0.51076E+00_JPRB, 0.48743E+00_JPRB ,&
&  0.45138E+00_JPRB, 0.48748E+00_JPRB, 0.49438E+00_JPRB, 0.48675E+00_JPRB, 0.51571E+00_JPRB, 0.50530E+00_JPRB, 0.47545E+00_JPRB ,&
&  0.44187E+00_JPRB, 0.42322E+00_JPRB, 0.41679E+00_JPRB, 0.41416E+00_JPRB, 0.41438E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.38525E+00_JPRB, 0.39824E+00_JPRB, 0.40719E+00_JPRB, 0.41070E+00_JPRB, 0.41129E+00_JPRB, 0.40682E+00_JPRB, 0.40278E+00_JPRB ,&
&  0.39827E+00_JPRB, 0.38586E+00_JPRB, 0.37236E+00_JPRB, 0.37514E+00_JPRB, 0.38117E+00_JPRB, 0.39370E+00_JPRB, 0.41955E+00_JPRB ,&
&  0.47156E+00_JPRB, 0.59921E+00_JPRB, 0.46372E+00_JPRB, 0.43175E+00_JPRB, 0.39886E+00_JPRB, 0.44909E+00_JPRB, 0.50084E+00_JPRB ,&
&  0.59312E+00_JPRB, 0.60392E+00_JPRB, 0.64743E+00_JPRB, 0.74497E+00_JPRB, 0.90709E+00_JPRB, 0.10256E+01_JPRB, 0.11906E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.13724E+01_JPRB, 0.12822E+01_JPRB, 0.13164E+01_JPRB, 0.13303E+01_JPRB, 0.12203E+01_JPRB, 0.12142E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.95581E+00_JPRB, 0.87550E+00_JPRB, 0.80722E+00_JPRB, 0.76528E+00_JPRB, 0.73748E+00_JPRB, 0.73661E+00_JPRB ,&
&  0.78951E+00_JPRB, 0.85360E+00_JPRB, 0.86048E+00_JPRB, 0.86392E+00_JPRB, 0.89532E+00_JPRB, 0.10225E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.15509E+01_JPRB, 0.16275E+01_JPRB, 0.14390E+01_JPRB, 0.20627E+01_JPRB, 0.26171E+01_JPRB, 0.23105E+01_JPRB, 0.19520E+01_JPRB ,&
&  0.20468E+01_JPRB, 0.20843E+01_JPRB, 0.21354E+01_JPRB, 0.20750E+01_JPRB, 0.19831E+01_JPRB, 0.18300E+01_JPRB, 0.15970E+01_JPRB ,&
&  0.13490E+01_JPRB, 0.12127E+01_JPRB, 0.11276E+01_JPRB, 0.11490E+01_JPRB, 0.10524E+01_JPRB, 0.94825E+00_JPRB, 0.86936E+00_JPRB ,&
&  0.72403E+00_JPRB, 0.60053E+00_JPRB, 0.42645E+00_JPRB, 0.44870E+00_JPRB, 0.37212E+00_JPRB, 0.37852E+00_JPRB, 0.42783E+00_JPRB ,&
&  0.41995E+00_JPRB, 0.42479E+00_JPRB, 0.41918E+00_JPRB, 0.40387E+00_JPRB, 0.44514E+00_JPRB, 0.44354E+00_JPRB, 0.43658E+00_JPRB ,&
&  0.42064E+00_JPRB, 0.39391E+00_JPRB, 0.42124E+00_JPRB, 0.42025E+00_JPRB, 0.43277E+00_JPRB, 0.42664E+00_JPRB, 0.41318E+00_JPRB ,&
&  0.39576E+00_JPRB, 0.38589E+00_JPRB, 0.38484E+00_JPRB, 0.38367E+00_JPRB, 0.38022E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.35364E+00_JPRB, 0.37418E+00_JPRB, 0.38836E+00_JPRB, 0.39238E+00_JPRB, 0.39662E+00_JPRB, 0.38815E+00_JPRB, 0.38433E+00_JPRB ,&
&  0.38699E+00_JPRB, 0.37718E+00_JPRB, 0.36254E+00_JPRB, 0.36505E+00_JPRB, 0.36958E+00_JPRB, 0.38212E+00_JPRB, 0.41585E+00_JPRB ,&
&  0.49852E+00_JPRB, 0.54698E+00_JPRB, 0.47009E+00_JPRB, 0.40479E+00_JPRB, 0.49454E+00_JPRB, 0.46763E+00_JPRB, 0.49536E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.50732E+00_JPRB, 0.51881E+00_JPRB, 0.54617E+00_JPRB, 0.63456E+00_JPRB, 0.77057E+00_JPRB, 0.87232E+00_JPRB ,&
&  0.91494E+00_JPRB, 0.94665E+00_JPRB, 0.92586E+00_JPRB, 0.90710E+00_JPRB, 0.91928E+00_JPRB, 0.84712E+00_JPRB, 0.85429E+00_JPRB ,&
&  0.82262E+00_JPRB, 0.76742E+00_JPRB, 0.72189E+00_JPRB, 0.71376E+00_JPRB, 0.70422E+00_JPRB, 0.69588E+00_JPRB, 0.71209E+00_JPRB ,&
&  0.77777E+00_JPRB, 0.85941E+00_JPRB, 0.86830E+00_JPRB, 0.88232E+00_JPRB, 0.94745E+00_JPRB, 0.12019E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.19217E+01_JPRB, 0.21120E+01_JPRB, 0.23081E+01_JPRB, 0.25858E+01_JPRB, 0.25580E+01_JPRB, 0.23933E+01_JPRB, 0.22837E+01_JPRB ,&
&  0.22041E+01_JPRB, 0.20946E+01_JPRB, 0.20604E+01_JPRB, 0.19846E+01_JPRB, 0.19243E+01_JPRB, 0.18537E+01_JPRB, 0.16379E+01_JPRB ,&
&  0.13650E+01_JPRB, 0.11912E+01_JPRB, 0.10983E+01_JPRB, 0.10004E+01_JPRB, 0.92060E+00_JPRB, 0.82371E+00_JPRB, 0.82263E+00_JPRB ,&
&  0.90978E+00_JPRB, 0.50178E+00_JPRB, 0.39048E+00_JPRB, 0.45150E+00_JPRB, 0.39424E+00_JPRB, 0.40650E+00_JPRB, 0.47631E+00_JPRB ,&
&  0.46040E+00_JPRB, 0.43201E+00_JPRB, 0.41630E+00_JPRB, 0.39355E+00_JPRB, 0.40157E+00_JPRB, 0.43914E+00_JPRB, 0.39809E+00_JPRB ,&
&  0.37123E+00_JPRB, 0.36578E+00_JPRB, 0.37666E+00_JPRB, 0.36714E+00_JPRB, 0.37092E+00_JPRB, 0.36775E+00_JPRB, 0.36403E+00_JPRB ,&
&  0.35393E+00_JPRB, 0.34951E+00_JPRB, 0.34298E+00_JPRB, 0.34349E+00_JPRB, 0.34833E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.33718E+00_JPRB, 0.36012E+00_JPRB, 0.37196E+00_JPRB, 0.37916E+00_JPRB, 0.38477E+00_JPRB, 0.37399E+00_JPRB, 0.37081E+00_JPRB ,&
&  0.37391E+00_JPRB, 0.36741E+00_JPRB, 0.35541E+00_JPRB, 0.35551E+00_JPRB, 0.35853E+00_JPRB, 0.36041E+00_JPRB, 0.36864E+00_JPRB ,&
&  0.31970E+00_JPRB, 0.37225E+00_JPRB, 0.33920E+00_JPRB, 0.37463E+00_JPRB, 0.46737E+00_JPRB, 0.43343E+00_JPRB, 0.42886E+00_JPRB ,&
&  0.41423E+00_JPRB, 0.40120E+00_JPRB, 0.38894E+00_JPRB, 0.40277E+00_JPRB, 0.43863E+00_JPRB, 0.55539E+00_JPRB, 0.66654E+00_JPRB ,&
&  0.64485E+00_JPRB, 0.61533E+00_JPRB, 0.60223E+00_JPRB, 0.59989E+00_JPRB, 0.58275E+00_JPRB, 0.58281E+00_JPRB, 0.60090E+00_JPRB ,&
&  0.59952E+00_JPRB, 0.58605E+00_JPRB, 0.59154E+00_JPRB, 0.61477E+00_JPRB, 0.62822E+00_JPRB, 0.62424E+00_JPRB, 0.63574E+00_JPRB ,&
&  0.69893E+00_JPRB, 0.79460E+00_JPRB, 0.82858E+00_JPRB, 0.89197E+00_JPRB, 0.98770E+00_JPRB, 0.15080E+01_JPRB, 0.22151E+01_JPRB ,&
&  0.24006E+01_JPRB, 0.23546E+01_JPRB, 0.25126E+01_JPRB, 0.27411E+01_JPRB, 0.26522E+01_JPRB, 0.24363E+01_JPRB, 0.22103E+01_JPRB ,&
&  0.20424E+01_JPRB, 0.18421E+01_JPRB, 0.18139E+01_JPRB, 0.18408E+01_JPRB, 0.17266E+01_JPRB, 0.17489E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.12620E+01_JPRB, 0.11727E+01_JPRB, 0.11225E+01_JPRB, 0.10156E+01_JPRB, 0.10036E+01_JPRB, 0.92768E+00_JPRB ,&
&  0.91849E+00_JPRB, 0.79101E+00_JPRB, 0.58502E+00_JPRB, 0.53350E+00_JPRB, 0.42834E+00_JPRB, 0.44441E+00_JPRB, 0.55877E+00_JPRB ,&
&  0.53284E+00_JPRB, 0.47342E+00_JPRB, 0.42017E+00_JPRB, 0.38856E+00_JPRB, 0.39473E+00_JPRB, 0.42560E+00_JPRB, 0.37789E+00_JPRB ,&
&  0.32818E+00_JPRB, 0.33678E+00_JPRB, 0.33622E+00_JPRB, 0.32777E+00_JPRB, 0.32413E+00_JPRB, 0.32669E+00_JPRB, 0.32404E+00_JPRB ,&
&  0.31996E+00_JPRB, 0.31778E+00_JPRB, 0.31872E+00_JPRB, 0.31689E+00_JPRB, 0.32643E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.32435E+00_JPRB, 0.34217E+00_JPRB, 0.34946E+00_JPRB, 0.36202E+00_JPRB, 0.37104E+00_JPRB, 0.36331E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.35716E+00_JPRB, 0.35389E+00_JPRB, 0.34902E+00_JPRB, 0.34815E+00_JPRB, 0.34888E+00_JPRB, 0.34627E+00_JPRB, 0.31277E+00_JPRB ,&
&  0.27661E+00_JPRB, 0.30386E+00_JPRB, 0.30898E+00_JPRB, 0.32842E+00_JPRB, 0.36189E+00_JPRB, 0.35292E+00_JPRB, 0.33558E+00_JPRB ,&
&  0.33763E+00_JPRB, 0.33099E+00_JPRB, 0.33791E+00_JPRB, 0.35523E+00_JPRB, 0.37790E+00_JPRB, 0.44377E+00_JPRB, 0.54926E+00_JPRB ,&
&  0.51713E+00_JPRB, 0.50209E+00_JPRB, 0.48618E+00_JPRB, 0.48666E+00_JPRB, 0.49391E+00_JPRB, 0.49112E+00_JPRB, 0.48375E+00_JPRB ,&
&  0.47581E+00_JPRB, 0.48210E+00_JPRB, 0.49518E+00_JPRB, 0.51770E+00_JPRB, 0.53825E+00_JPRB, 0.54479E+00_JPRB, 0.54660E+00_JPRB ,&
&  0.58325E+00_JPRB, 0.65417E+00_JPRB, 0.70559E+00_JPRB, 0.77138E+00_JPRB, 0.88508E+00_JPRB, 0.13758E+01_JPRB, 0.18174E+01_JPRB ,&
&  0.18880E+01_JPRB, 0.19086E+01_JPRB, 0.18943E+01_JPRB, 0.18866E+01_JPRB, 0.19108E+01_JPRB, 0.17925E+01_JPRB, 0.16571E+01_JPRB ,&
&  0.15885E+01_JPRB, 0.14420E+01_JPRB, 0.16541E+01_JPRB, 0.17390E+01_JPRB, 0.16610E+01_JPRB, 0.16263E+01_JPRB, 0.14920E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.13041E+01_JPRB, 0.12172E+01_JPRB, 0.11433E+01_JPRB, 0.10533E+01_JPRB, 0.10237E+01_JPRB, 0.92197E+00_JPRB ,&
&  0.89741E+00_JPRB, 0.85757E+00_JPRB, 0.72389E+00_JPRB, 0.63103E+00_JPRB, 0.54293E+00_JPRB, 0.56776E+00_JPRB, 0.54189E+00_JPRB ,&
&  0.49270E+00_JPRB, 0.41365E+00_JPRB, 0.40519E+00_JPRB, 0.39491E+00_JPRB, 0.40493E+00_JPRB, 0.39723E+00_JPRB, 0.34466E+00_JPRB ,&
&  0.32189E+00_JPRB, 0.33281E+00_JPRB, 0.32122E+00_JPRB, 0.30912E+00_JPRB, 0.30241E+00_JPRB, 0.29598E+00_JPRB, 0.27683E+00_JPRB ,&
&  0.27422E+00_JPRB, 0.29863E+00_JPRB, 0.30860E+00_JPRB, 0.30706E+00_JPRB, 0.31286E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.31207E+00_JPRB, 0.32679E+00_JPRB, 0.33530E+00_JPRB, 0.34814E+00_JPRB, 0.35535E+00_JPRB, 0.35107E+00_JPRB, 0.33497E+00_JPRB ,&
&  0.34365E+00_JPRB, 0.34100E+00_JPRB, 0.33896E+00_JPRB, 0.34184E+00_JPRB, 0.34463E+00_JPRB, 0.30983E+00_JPRB, 0.25067E+00_JPRB ,&
&  0.23844E+00_JPRB, 0.25372E+00_JPRB, 0.28378E+00_JPRB, 0.30544E+00_JPRB, 0.30972E+00_JPRB, 0.30269E+00_JPRB, 0.29552E+00_JPRB ,&
&  0.29374E+00_JPRB, 0.29940E+00_JPRB, 0.32298E+00_JPRB, 0.34284E+00_JPRB, 0.35158E+00_JPRB, 0.37180E+00_JPRB, 0.45715E+00_JPRB ,&
&  0.45606E+00_JPRB, 0.44981E+00_JPRB, 0.45074E+00_JPRB, 0.42172E+00_JPRB, 0.44478E+00_JPRB, 0.42480E+00_JPRB, 0.40769E+00_JPRB ,&
&  0.40698E+00_JPRB, 0.41906E+00_JPRB, 0.43908E+00_JPRB, 0.46577E+00_JPRB, 0.48305E+00_JPRB, 0.48854E+00_JPRB, 0.49229E+00_JPRB ,&
&  0.50806E+00_JPRB, 0.54867E+00_JPRB, 0.59658E+00_JPRB, 0.66534E+00_JPRB, 0.76140E+00_JPRB, 0.10820E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.15236E+01_JPRB, 0.15559E+01_JPRB, 0.13947E+01_JPRB, 0.14666E+01_JPRB, 0.14516E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.12333E+01_JPRB, 0.12946E+01_JPRB, 0.13831E+01_JPRB, 0.14587E+01_JPRB, 0.13986E+01_JPRB, 0.13022E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.12931E+01_JPRB, 0.11485E+01_JPRB, 0.10602E+01_JPRB, 0.95894E+00_JPRB, 0.90280E+00_JPRB, 0.82064E+00_JPRB ,&
&  0.78497E+00_JPRB, 0.76838E+00_JPRB, 0.72744E+00_JPRB, 0.68463E+00_JPRB, 0.63456E+00_JPRB, 0.58948E+00_JPRB, 0.54622E+00_JPRB ,&
&  0.49838E+00_JPRB, 0.43542E+00_JPRB, 0.40443E+00_JPRB, 0.40663E+00_JPRB, 0.42197E+00_JPRB, 0.37776E+00_JPRB, 0.33444E+00_JPRB ,&
&  0.31417E+00_JPRB, 0.30564E+00_JPRB, 0.31731E+00_JPRB, 0.30008E+00_JPRB, 0.28704E+00_JPRB, 0.27931E+00_JPRB, 0.27380E+00_JPRB ,&
&  0.27038E+00_JPRB, 0.29765E+00_JPRB, 0.30636E+00_JPRB, 0.30578E+00_JPRB, 0.30468E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.30766E+00_JPRB, 0.32001E+00_JPRB, 0.32909E+00_JPRB, 0.33868E+00_JPRB, 0.34386E+00_JPRB, 0.32989E+00_JPRB, 0.32785E+00_JPRB ,&
&  0.36677E+00_JPRB, 0.31773E+00_JPRB, 0.33685E+00_JPRB, 0.27715E+00_JPRB, 0.25445E+00_JPRB, 0.25011E+00_JPRB, 0.24214E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.25925E+00_JPRB, 0.27649E+00_JPRB, 0.28967E+00_JPRB, 0.30008E+00_JPRB, 0.28886E+00_JPRB, 0.28007E+00_JPRB ,&
&  0.27423E+00_JPRB, 0.28408E+00_JPRB, 0.31530E+00_JPRB, 0.33839E+00_JPRB, 0.35365E+00_JPRB, 0.36707E+00_JPRB, 0.40714E+00_JPRB ,&
&  0.40443E+00_JPRB, 0.39769E+00_JPRB, 0.40889E+00_JPRB, 0.40611E+00_JPRB, 0.40186E+00_JPRB, 0.38301E+00_JPRB, 0.36498E+00_JPRB ,&
&  0.35860E+00_JPRB, 0.35137E+00_JPRB, 0.39798E+00_JPRB, 0.42726E+00_JPRB, 0.44345E+00_JPRB, 0.44846E+00_JPRB, 0.45876E+00_JPRB ,&
&  0.47204E+00_JPRB, 0.49578E+00_JPRB, 0.53665E+00_JPRB, 0.59769E+00_JPRB, 0.66609E+00_JPRB, 0.79965E+00_JPRB, 0.93569E+00_JPRB ,&
&  0.11245E+01_JPRB, 0.93720E+00_JPRB, 0.11323E+01_JPRB, 0.11249E+01_JPRB, 0.12496E+01_JPRB, 0.12833E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.12061E+01_JPRB, 0.12073E+01_JPRB, 0.11914E+01_JPRB, 0.11867E+01_JPRB, 0.11651E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.10051E+01_JPRB, 0.98728E+00_JPRB, 0.93192E+00_JPRB, 0.85795E+00_JPRB, 0.81260E+00_JPRB, 0.75610E+00_JPRB ,&
&  0.71976E+00_JPRB, 0.69945E+00_JPRB, 0.67121E+00_JPRB, 0.59532E+00_JPRB, 0.57100E+00_JPRB, 0.54993E+00_JPRB, 0.50294E+00_JPRB ,&
&  0.48340E+00_JPRB, 0.46059E+00_JPRB, 0.43792E+00_JPRB, 0.46393E+00_JPRB, 0.45310E+00_JPRB, 0.41098E+00_JPRB, 0.36802E+00_JPRB ,&
&  0.32720E+00_JPRB, 0.29275E+00_JPRB, 0.29206E+00_JPRB, 0.28391E+00_JPRB, 0.27193E+00_JPRB, 0.26991E+00_JPRB, 0.28955E+00_JPRB ,&
&  0.28659E+00_JPRB, 0.29464E+00_JPRB, 0.31183E+00_JPRB, 0.31193E+00_JPRB, 0.30675E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.30963E+00_JPRB, 0.31558E+00_JPRB, 0.31854E+00_JPRB, 0.32164E+00_JPRB, 0.32423E+00_JPRB, 0.31588E+00_JPRB, 0.29738E+00_JPRB ,&
&  0.30434E+00_JPRB, 0.26436E+00_JPRB, 0.24763E+00_JPRB, 0.23892E+00_JPRB, 0.23929E+00_JPRB, 0.23694E+00_JPRB, 0.23074E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.24778E+00_JPRB, 0.27190E+00_JPRB, 0.28358E+00_JPRB, 0.28483E+00_JPRB, 0.28115E+00_JPRB, 0.27563E+00_JPRB ,&
&  0.27319E+00_JPRB, 0.28462E+00_JPRB, 0.30928E+00_JPRB, 0.34446E+00_JPRB, 0.36163E+00_JPRB, 0.36737E+00_JPRB, 0.37582E+00_JPRB ,&
&  0.38077E+00_JPRB, 0.38108E+00_JPRB, 0.38216E+00_JPRB, 0.38762E+00_JPRB, 0.37460E+00_JPRB, 0.34760E+00_JPRB, 0.31692E+00_JPRB ,&
&  0.27195E+00_JPRB, 0.27679E+00_JPRB, 0.36108E+00_JPRB, 0.39216E+00_JPRB, 0.41193E+00_JPRB, 0.42120E+00_JPRB, 0.43210E+00_JPRB ,&
&  0.44984E+00_JPRB, 0.47290E+00_JPRB, 0.50293E+00_JPRB, 0.54317E+00_JPRB, 0.58324E+00_JPRB, 0.64394E+00_JPRB, 0.69889E+00_JPRB ,&
&  0.81827E+00_JPRB, 0.69361E+00_JPRB, 0.67638E+00_JPRB, 0.73546E+00_JPRB, 0.10352E+01_JPRB, 0.11186E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.10530E+01_JPRB, 0.10471E+01_JPRB, 0.10729E+01_JPRB, 0.10756E+01_JPRB, 0.10417E+01_JPRB, 0.98279E+00_JPRB ,&
&  0.92941E+00_JPRB, 0.89074E+00_JPRB, 0.83341E+00_JPRB, 0.81411E+00_JPRB, 0.77993E+00_JPRB, 0.76138E+00_JPRB, 0.72620E+00_JPRB ,&
&  0.69198E+00_JPRB, 0.67075E+00_JPRB, 0.62757E+00_JPRB, 0.58151E+00_JPRB, 0.54538E+00_JPRB, 0.52159E+00_JPRB, 0.49971E+00_JPRB ,&
&  0.48299E+00_JPRB, 0.47948E+00_JPRB, 0.48226E+00_JPRB, 0.48848E+00_JPRB, 0.46614E+00_JPRB, 0.41566E+00_JPRB, 0.37134E+00_JPRB ,&
&  0.32162E+00_JPRB, 0.28780E+00_JPRB, 0.25925E+00_JPRB, 0.25184E+00_JPRB, 0.25590E+00_JPRB, 0.26162E+00_JPRB, 0.26693E+00_JPRB ,&
&  0.27652E+00_JPRB, 0.28289E+00_JPRB, 0.28898E+00_JPRB, 0.29859E+00_JPRB, 0.30282E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.31193E+00_JPRB, 0.31683E+00_JPRB, 0.31076E+00_JPRB, 0.30630E+00_JPRB, 0.29779E+00_JPRB, 0.28891E+00_JPRB, 0.27775E+00_JPRB ,&
&  0.27994E+00_JPRB, 0.26341E+00_JPRB, 0.25031E+00_JPRB, 0.23906E+00_JPRB, 0.23328E+00_JPRB, 0.23018E+00_JPRB, 0.23562E+00_JPRB ,&
&  0.24949E+00_JPRB, 0.26743E+00_JPRB, 0.27954E+00_JPRB, 0.27988E+00_JPRB, 0.27902E+00_JPRB, 0.27764E+00_JPRB, 0.27887E+00_JPRB ,&
&  0.28031E+00_JPRB, 0.28768E+00_JPRB, 0.30219E+00_JPRB, 0.32488E+00_JPRB, 0.34782E+00_JPRB, 0.36074E+00_JPRB, 0.36241E+00_JPRB ,&
&  0.35958E+00_JPRB, 0.35960E+00_JPRB, 0.36490E+00_JPRB, 0.36938E+00_JPRB, 0.34669E+00_JPRB, 0.30471E+00_JPRB, 0.26542E+00_JPRB ,&
&  0.23506E+00_JPRB, 0.23017E+00_JPRB, 0.25853E+00_JPRB, 0.34581E+00_JPRB, 0.36891E+00_JPRB, 0.38495E+00_JPRB, 0.39951E+00_JPRB ,&
&  0.38624E+00_JPRB, 0.38994E+00_JPRB, 0.42060E+00_JPRB, 0.48196E+00_JPRB, 0.49929E+00_JPRB, 0.53453E+00_JPRB, 0.56251E+00_JPRB ,&
&  0.59949E+00_JPRB, 0.61255E+00_JPRB, 0.65606E+00_JPRB, 0.62957E+00_JPRB, 0.74101E+00_JPRB, 0.10111E+01_JPRB, 0.97693E+00_JPRB ,&
&  0.96217E+00_JPRB, 0.93630E+00_JPRB, 0.97921E+00_JPRB, 0.98016E+00_JPRB, 0.95789E+00_JPRB, 0.94565E+00_JPRB, 0.90486E+00_JPRB ,&
&  0.83571E+00_JPRB, 0.78938E+00_JPRB, 0.74749E+00_JPRB, 0.75246E+00_JPRB, 0.74940E+00_JPRB, 0.73537E+00_JPRB, 0.71469E+00_JPRB ,&
&  0.69240E+00_JPRB, 0.66609E+00_JPRB, 0.61974E+00_JPRB, 0.56158E+00_JPRB, 0.53138E+00_JPRB, 0.50422E+00_JPRB, 0.48473E+00_JPRB ,&
&  0.47096E+00_JPRB, 0.47397E+00_JPRB, 0.48454E+00_JPRB, 0.48708E+00_JPRB, 0.44240E+00_JPRB, 0.37721E+00_JPRB, 0.32710E+00_JPRB ,&
&  0.29390E+00_JPRB, 0.27460E+00_JPRB, 0.25537E+00_JPRB, 0.24956E+00_JPRB, 0.25024E+00_JPRB, 0.25242E+00_JPRB, 0.24987E+00_JPRB ,&
&  0.24823E+00_JPRB, 0.25252E+00_JPRB, 0.26902E+00_JPRB, 0.28876E+00_JPRB, 0.30186E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.28963E+00_JPRB, 0.30721E+00_JPRB, 0.31496E+00_JPRB, 0.30924E+00_JPRB, 0.30270E+00_JPRB, 0.28276E+00_JPRB, 0.26443E+00_JPRB ,&
&  0.25846E+00_JPRB, 0.25560E+00_JPRB, 0.25662E+00_JPRB, 0.25862E+00_JPRB, 0.26240E+00_JPRB, 0.26934E+00_JPRB, 0.27754E+00_JPRB ,&
&  0.28169E+00_JPRB, 0.28384E+00_JPRB, 0.28711E+00_JPRB, 0.29260E+00_JPRB, 0.29653E+00_JPRB, 0.29688E+00_JPRB, 0.29641E+00_JPRB ,&
&  0.29444E+00_JPRB, 0.29530E+00_JPRB, 0.30097E+00_JPRB, 0.31102E+00_JPRB, 0.32312E+00_JPRB, 0.33679E+00_JPRB, 0.34810E+00_JPRB ,&
&  0.35157E+00_JPRB, 0.34422E+00_JPRB, 0.34229E+00_JPRB, 0.33915E+00_JPRB, 0.31150E+00_JPRB, 0.28250E+00_JPRB, 0.24790E+00_JPRB ,&
&  0.22264E+00_JPRB, 0.20865E+00_JPRB, 0.20460E+00_JPRB, 0.21818E+00_JPRB, 0.25801E+00_JPRB, 0.34134E+00_JPRB, 0.36444E+00_JPRB ,&
&  0.38276E+00_JPRB, 0.39480E+00_JPRB, 0.40526E+00_JPRB, 0.42001E+00_JPRB, 0.43551E+00_JPRB, 0.45978E+00_JPRB, 0.47961E+00_JPRB ,&
&  0.50147E+00_JPRB, 0.52005E+00_JPRB, 0.54852E+00_JPRB, 0.52542E+00_JPRB, 0.53603E+00_JPRB, 0.61811E+00_JPRB, 0.70168E+00_JPRB ,&
&  0.74685E+00_JPRB, 0.74486E+00_JPRB, 0.75627E+00_JPRB, 0.81552E+00_JPRB, 0.84402E+00_JPRB, 0.88634E+00_JPRB, 0.88038E+00_JPRB ,&
&  0.84807E+00_JPRB, 0.80250E+00_JPRB, 0.75676E+00_JPRB, 0.73308E+00_JPRB, 0.73623E+00_JPRB, 0.74372E+00_JPRB, 0.71802E+00_JPRB ,&
&  0.69179E+00_JPRB, 0.66134E+00_JPRB, 0.60636E+00_JPRB, 0.53249E+00_JPRB, 0.50658E+00_JPRB, 0.50203E+00_JPRB, 0.50371E+00_JPRB ,&
&  0.49576E+00_JPRB, 0.49292E+00_JPRB, 0.49367E+00_JPRB, 0.49082E+00_JPRB, 0.43629E+00_JPRB, 0.38083E+00_JPRB, 0.33745E+00_JPRB ,&
&  0.31260E+00_JPRB, 0.29710E+00_JPRB, 0.28177E+00_JPRB, 0.28045E+00_JPRB, 0.27261E+00_JPRB, 0.26418E+00_JPRB, 0.25314E+00_JPRB ,&
&  0.24811E+00_JPRB, 0.24811E+00_JPRB, 0.25601E+00_JPRB, 0.26766E+00_JPRB, 0.27911E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.29995E+00_JPRB, 0.30928E+00_JPRB, 0.31498E+00_JPRB, 0.31590E+00_JPRB, 0.31294E+00_JPRB, 0.30478E+00_JPRB, 0.29934E+00_JPRB ,&
&  0.29910E+00_JPRB, 0.30271E+00_JPRB, 0.30548E+00_JPRB, 0.30617E+00_JPRB, 0.30730E+00_JPRB, 0.30850E+00_JPRB, 0.31021E+00_JPRB ,&
&  0.31216E+00_JPRB, 0.31535E+00_JPRB, 0.31973E+00_JPRB, 0.31826E+00_JPRB, 0.31553E+00_JPRB, 0.31361E+00_JPRB, 0.31120E+00_JPRB ,&
&  0.30862E+00_JPRB, 0.30750E+00_JPRB, 0.30767E+00_JPRB, 0.31135E+00_JPRB, 0.31244E+00_JPRB, 0.31529E+00_JPRB, 0.31468E+00_JPRB ,&
&  0.31165E+00_JPRB, 0.31392E+00_JPRB, 0.32095E+00_JPRB, 0.30393E+00_JPRB, 0.27855E+00_JPRB, 0.25836E+00_JPRB, 0.22943E+00_JPRB ,&
&  0.20667E+00_JPRB, 0.19033E+00_JPRB, 0.18021E+00_JPRB, 0.17819E+00_JPRB, 0.18541E+00_JPRB, 0.20512E+00_JPRB, 0.24940E+00_JPRB ,&
&  0.34022E+00_JPRB, 0.36182E+00_JPRB, 0.37156E+00_JPRB, 0.38667E+00_JPRB, 0.40428E+00_JPRB, 0.42336E+00_JPRB, 0.44140E+00_JPRB ,&
&  0.45502E+00_JPRB, 0.46325E+00_JPRB, 0.47589E+00_JPRB, 0.48993E+00_JPRB, 0.50973E+00_JPRB, 0.51011E+00_JPRB, 0.55624E+00_JPRB ,&
&  0.64887E+00_JPRB, 0.66606E+00_JPRB, 0.62399E+00_JPRB, 0.63879E+00_JPRB, 0.67730E+00_JPRB, 0.71770E+00_JPRB, 0.75646E+00_JPRB ,&
&  0.77147E+00_JPRB, 0.76854E+00_JPRB, 0.76215E+00_JPRB, 0.74619E+00_JPRB, 0.73736E+00_JPRB, 0.73030E+00_JPRB, 0.71902E+00_JPRB ,&
&  0.70082E+00_JPRB, 0.69124E+00_JPRB, 0.73773E+00_JPRB, 0.62196E+00_JPRB, 0.57024E+00_JPRB, 0.55558E+00_JPRB, 0.52693E+00_JPRB ,&
&  0.50682E+00_JPRB, 0.49931E+00_JPRB, 0.50198E+00_JPRB, 0.49861E+00_JPRB, 0.46380E+00_JPRB, 0.41968E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.34829E+00_JPRB, 0.32479E+00_JPRB, 0.30174E+00_JPRB, 0.29122E+00_JPRB, 0.28228E+00_JPRB, 0.27123E+00_JPRB, 0.26019E+00_JPRB ,&
&  0.25790E+00_JPRB, 0.25996E+00_JPRB, 0.26720E+00_JPRB, 0.28281E+00_JPRB, 0.29299E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.29556E+00_JPRB, 0.30668E+00_JPRB, 0.31559E+00_JPRB, 0.32002E+00_JPRB, 0.32066E+00_JPRB, 0.32120E+00_JPRB, 0.32102E+00_JPRB ,&
&  0.32166E+00_JPRB, 0.32280E+00_JPRB, 0.32397E+00_JPRB, 0.32422E+00_JPRB, 0.32447E+00_JPRB, 0.32409E+00_JPRB, 0.32514E+00_JPRB ,&
&  0.32645E+00_JPRB, 0.32585E+00_JPRB, 0.32663E+00_JPRB, 0.32938E+00_JPRB, 0.32935E+00_JPRB, 0.32566E+00_JPRB, 0.31890E+00_JPRB ,&
&  0.31131E+00_JPRB, 0.30744E+00_JPRB, 0.30398E+00_JPRB, 0.29863E+00_JPRB, 0.29330E+00_JPRB, 0.28889E+00_JPRB, 0.29066E+00_JPRB ,&
&  0.29945E+00_JPRB, 0.29522E+00_JPRB, 0.28963E+00_JPRB, 0.27028E+00_JPRB, 0.25109E+00_JPRB, 0.24025E+00_JPRB, 0.20845E+00_JPRB ,&
&  0.18625E+00_JPRB, 0.17336E+00_JPRB, 0.16875E+00_JPRB, 0.16776E+00_JPRB, 0.17226E+00_JPRB, 0.18428E+00_JPRB, 0.21017E+00_JPRB ,&
&  0.25572E+00_JPRB, 0.33223E+00_JPRB, 0.34647E+00_JPRB, 0.36392E+00_JPRB, 0.38539E+00_JPRB, 0.40738E+00_JPRB, 0.42360E+00_JPRB ,&
&  0.43549E+00_JPRB, 0.44369E+00_JPRB, 0.45835E+00_JPRB, 0.47135E+00_JPRB, 0.49332E+00_JPRB, 0.51293E+00_JPRB, 0.52761E+00_JPRB ,&
&  0.53893E+00_JPRB, 0.52718E+00_JPRB, 0.53826E+00_JPRB, 0.54801E+00_JPRB, 0.57840E+00_JPRB, 0.61108E+00_JPRB, 0.60933E+00_JPRB ,&
&  0.61094E+00_JPRB, 0.63742E+00_JPRB, 0.63590E+00_JPRB, 0.64326E+00_JPRB, 0.66776E+00_JPRB, 0.69917E+00_JPRB, 0.70510E+00_JPRB ,&
&  0.67749E+00_JPRB, 0.66234E+00_JPRB, 0.69975E+00_JPRB, 0.66181E+00_JPRB, 0.63108E+00_JPRB, 0.59696E+00_JPRB, 0.55731E+00_JPRB ,&
&  0.53022E+00_JPRB, 0.51184E+00_JPRB, 0.50102E+00_JPRB, 0.49153E+00_JPRB, 0.47439E+00_JPRB, 0.45102E+00_JPRB, 0.41793E+00_JPRB ,&
&  0.37889E+00_JPRB, 0.34608E+00_JPRB, 0.32095E+00_JPRB, 0.30532E+00_JPRB, 0.29336E+00_JPRB, 0.28152E+00_JPRB, 0.27337E+00_JPRB ,&
&  0.26822E+00_JPRB, 0.26907E+00_JPRB, 0.27295E+00_JPRB, 0.28049E+00_JPRB, 0.28780E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.30544E+00_JPRB, 0.31264E+00_JPRB, 0.31902E+00_JPRB, 0.32294E+00_JPRB, 0.32508E+00_JPRB, 0.32639E+00_JPRB, 0.32602E+00_JPRB ,&
&  0.32592E+00_JPRB, 0.32585E+00_JPRB, 0.32772E+00_JPRB, 0.32984E+00_JPRB, 0.33329E+00_JPRB, 0.33609E+00_JPRB, 0.34050E+00_JPRB ,&
&  0.34639E+00_JPRB, 0.35162E+00_JPRB, 0.35165E+00_JPRB, 0.34712E+00_JPRB, 0.34021E+00_JPRB, 0.33314E+00_JPRB, 0.31998E+00_JPRB ,&
&  0.30840E+00_JPRB, 0.30483E+00_JPRB, 0.29739E+00_JPRB, 0.28865E+00_JPRB, 0.28383E+00_JPRB, 0.27923E+00_JPRB, 0.28032E+00_JPRB ,&
&  0.27892E+00_JPRB, 0.27172E+00_JPRB, 0.26221E+00_JPRB, 0.24864E+00_JPRB, 0.23628E+00_JPRB, 0.19987E+00_JPRB, 0.18979E+00_JPRB ,&
&  0.17857E+00_JPRB, 0.17060E+00_JPRB, 0.16799E+00_JPRB, 0.16770E+00_JPRB, 0.17013E+00_JPRB, 0.17760E+00_JPRB, 0.18928E+00_JPRB ,&
&  0.20870E+00_JPRB, 0.30206E+00_JPRB, 0.32206E+00_JPRB, 0.33247E+00_JPRB, 0.35028E+00_JPRB, 0.37063E+00_JPRB, 0.38863E+00_JPRB ,&
&  0.40354E+00_JPRB, 0.41355E+00_JPRB, 0.42709E+00_JPRB, 0.43861E+00_JPRB, 0.45294E+00_JPRB, 0.46662E+00_JPRB, 0.47563E+00_JPRB ,&
&  0.48133E+00_JPRB, 0.48223E+00_JPRB, 0.49962E+00_JPRB, 0.50546E+00_JPRB, 0.50302E+00_JPRB, 0.52223E+00_JPRB, 0.53587E+00_JPRB ,&
&  0.53510E+00_JPRB, 0.52838E+00_JPRB, 0.53704E+00_JPRB, 0.56167E+00_JPRB, 0.59479E+00_JPRB, 0.62348E+00_JPRB, 0.62775E+00_JPRB ,&
&  0.62711E+00_JPRB, 0.61530E+00_JPRB, 0.60571E+00_JPRB, 0.59630E+00_JPRB, 0.57351E+00_JPRB, 0.54990E+00_JPRB, 0.53151E+00_JPRB ,&
&  0.51214E+00_JPRB, 0.50300E+00_JPRB, 0.48903E+00_JPRB, 0.47641E+00_JPRB, 0.46097E+00_JPRB, 0.44467E+00_JPRB, 0.42451E+00_JPRB ,&
&  0.39902E+00_JPRB, 0.37152E+00_JPRB, 0.34980E+00_JPRB, 0.33078E+00_JPRB, 0.31769E+00_JPRB, 0.30393E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.29158E+00_JPRB, 0.29173E+00_JPRB, 0.29061E+00_JPRB, 0.29396E+00_JPRB, 0.29751E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.32913E+00_JPRB, 0.33193E+00_JPRB, 0.33452E+00_JPRB, 0.33636E+00_JPRB, 0.33733E+00_JPRB, 0.33849E+00_JPRB, 0.33914E+00_JPRB ,&
&  0.33994E+00_JPRB, 0.34064E+00_JPRB, 0.34249E+00_JPRB, 0.34460E+00_JPRB, 0.34774E+00_JPRB, 0.35124E+00_JPRB, 0.35552E+00_JPRB ,&
&  0.36023E+00_JPRB, 0.36068E+00_JPRB, 0.35528E+00_JPRB, 0.34917E+00_JPRB, 0.34242E+00_JPRB, 0.33164E+00_JPRB, 0.31716E+00_JPRB ,&
&  0.30395E+00_JPRB, 0.29456E+00_JPRB, 0.28160E+00_JPRB, 0.27196E+00_JPRB, 0.26228E+00_JPRB, 0.25858E+00_JPRB, 0.25194E+00_JPRB ,&
&  0.25560E+00_JPRB, 0.23028E+00_JPRB, 0.22275E+00_JPRB, 0.20522E+00_JPRB, 0.19251E+00_JPRB, 0.18222E+00_JPRB, 0.17558E+00_JPRB ,&
&  0.17251E+00_JPRB, 0.16974E+00_JPRB, 0.16950E+00_JPRB, 0.17137E+00_JPRB, 0.17479E+00_JPRB, 0.18215E+00_JPRB, 0.19102E+00_JPRB ,&
&  0.21518E+00_JPRB, 0.29529E+00_JPRB, 0.32226E+00_JPRB, 0.32921E+00_JPRB, 0.33385E+00_JPRB, 0.34762E+00_JPRB, 0.36225E+00_JPRB ,&
&  0.38050E+00_JPRB, 0.39109E+00_JPRB, 0.39992E+00_JPRB, 0.40409E+00_JPRB, 0.41944E+00_JPRB, 0.43389E+00_JPRB, 0.44483E+00_JPRB ,&
&  0.45503E+00_JPRB, 0.46452E+00_JPRB, 0.47408E+00_JPRB, 0.47971E+00_JPRB, 0.47639E+00_JPRB, 0.47490E+00_JPRB, 0.47101E+00_JPRB ,&
&  0.47222E+00_JPRB, 0.47332E+00_JPRB, 0.48417E+00_JPRB, 0.50147E+00_JPRB, 0.52171E+00_JPRB, 0.53732E+00_JPRB, 0.54511E+00_JPRB ,&
&  0.55030E+00_JPRB, 0.55058E+00_JPRB, 0.55440E+00_JPRB, 0.55486E+00_JPRB, 0.54643E+00_JPRB, 0.53278E+00_JPRB, 0.52291E+00_JPRB ,&
&  0.51296E+00_JPRB, 0.49363E+00_JPRB, 0.47577E+00_JPRB, 0.46013E+00_JPRB, 0.44634E+00_JPRB, 0.43351E+00_JPRB, 0.42186E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.39680E+00_JPRB, 0.38326E+00_JPRB, 0.36703E+00_JPRB, 0.35283E+00_JPRB, 0.33923E+00_JPRB, 0.33170E+00_JPRB ,&
&  0.32509E+00_JPRB, 0.32397E+00_JPRB, 0.32184E+00_JPRB, 0.32366E+00_JPRB, 0.32494E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.35627E+00_JPRB, 0.35569E+00_JPRB, 0.35592E+00_JPRB, 0.35582E+00_JPRB, 0.35640E+00_JPRB, 0.35632E+00_JPRB, 0.35741E+00_JPRB ,&
&  0.35841E+00_JPRB, 0.35980E+00_JPRB, 0.36032E+00_JPRB, 0.36078E+00_JPRB, 0.36133E+00_JPRB, 0.36242E+00_JPRB, 0.36365E+00_JPRB ,&
&  0.36402E+00_JPRB, 0.36174E+00_JPRB, 0.35576E+00_JPRB, 0.34898E+00_JPRB, 0.34101E+00_JPRB, 0.33107E+00_JPRB, 0.31894E+00_JPRB ,&
&  0.30357E+00_JPRB, 0.28937E+00_JPRB, 0.26832E+00_JPRB, 0.25782E+00_JPRB, 0.24528E+00_JPRB, 0.23664E+00_JPRB, 0.23125E+00_JPRB ,&
&  0.22600E+00_JPRB, 0.22060E+00_JPRB, 0.21517E+00_JPRB, 0.20789E+00_JPRB, 0.20171E+00_JPRB, 0.19284E+00_JPRB, 0.18727E+00_JPRB ,&
&  0.18312E+00_JPRB, 0.18105E+00_JPRB, 0.18080E+00_JPRB, 0.18245E+00_JPRB, 0.18663E+00_JPRB, 0.19214E+00_JPRB, 0.20339E+00_JPRB ,&
&  0.22895E+00_JPRB, 0.26639E+00_JPRB, 0.32308E+00_JPRB, 0.33420E+00_JPRB, 0.33289E+00_JPRB, 0.34061E+00_JPRB, 0.34889E+00_JPRB ,&
&  0.35689E+00_JPRB, 0.36301E+00_JPRB, 0.36811E+00_JPRB, 0.37076E+00_JPRB, 0.37474E+00_JPRB, 0.37835E+00_JPRB, 0.38166E+00_JPRB ,&
&  0.38507E+00_JPRB, 0.39228E+00_JPRB, 0.40368E+00_JPRB, 0.41673E+00_JPRB, 0.42951E+00_JPRB, 0.43945E+00_JPRB, 0.44585E+00_JPRB ,&
&  0.45001E+00_JPRB, 0.45348E+00_JPRB, 0.46189E+00_JPRB, 0.47252E+00_JPRB, 0.48588E+00_JPRB, 0.49640E+00_JPRB, 0.50553E+00_JPRB ,&
&  0.51169E+00_JPRB, 0.50931E+00_JPRB, 0.50169E+00_JPRB, 0.49196E+00_JPRB, 0.48527E+00_JPRB, 0.48570E+00_JPRB, 0.48191E+00_JPRB ,&
&  0.47732E+00_JPRB, 0.47073E+00_JPRB, 0.46105E+00_JPRB, 0.44888E+00_JPRB, 0.43842E+00_JPRB, 0.42854E+00_JPRB, 0.42107E+00_JPRB ,&
&  0.41405E+00_JPRB, 0.40766E+00_JPRB, 0.40092E+00_JPRB, 0.39251E+00_JPRB, 0.38448E+00_JPRB, 0.37645E+00_JPRB, 0.37065E+00_JPRB ,&
&  0.36483E+00_JPRB, 0.36173E+00_JPRB, 0.35886E+00_JPRB, 0.35790E+00_JPRB, 0.35653E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.39359E+00_JPRB, 0.39249E+00_JPRB, 0.39156E+00_JPRB, 0.39056E+00_JPRB, 0.38916E+00_JPRB, 0.38722E+00_JPRB, 0.38481E+00_JPRB ,&
&  0.38148E+00_JPRB, 0.37794E+00_JPRB, 0.37417E+00_JPRB, 0.37028E+00_JPRB, 0.36640E+00_JPRB, 0.36330E+00_JPRB, 0.36096E+00_JPRB ,&
&  0.35922E+00_JPRB, 0.35617E+00_JPRB, 0.35093E+00_JPRB, 0.34394E+00_JPRB, 0.33803E+00_JPRB, 0.33308E+00_JPRB, 0.32813E+00_JPRB ,&
&  0.31828E+00_JPRB, 0.30779E+00_JPRB, 0.29802E+00_JPRB, 0.28682E+00_JPRB, 0.27788E+00_JPRB, 0.25466E+00_JPRB, 0.24727E+00_JPRB ,&
&  0.24132E+00_JPRB, 0.23762E+00_JPRB, 0.23674E+00_JPRB, 0.23671E+00_JPRB, 0.24110E+00_JPRB, 0.23720E+00_JPRB, 0.23454E+00_JPRB ,&
&  0.22338E+00_JPRB, 0.22303E+00_JPRB, 0.22477E+00_JPRB, 0.22742E+00_JPRB, 0.23387E+00_JPRB, 0.24169E+00_JPRB, 0.25485E+00_JPRB ,&
&  0.27889E+00_JPRB, 0.28963E+00_JPRB, 0.30609E+00_JPRB, 0.31885E+00_JPRB, 0.32600E+00_JPRB, 0.32954E+00_JPRB, 0.33326E+00_JPRB ,&
&  0.33748E+00_JPRB, 0.34397E+00_JPRB, 0.34777E+00_JPRB, 0.35027E+00_JPRB, 0.34969E+00_JPRB, 0.34775E+00_JPRB, 0.34658E+00_JPRB ,&
&  0.34438E+00_JPRB, 0.34472E+00_JPRB, 0.34737E+00_JPRB, 0.35412E+00_JPRB, 0.36396E+00_JPRB, 0.37524E+00_JPRB, 0.38726E+00_JPRB ,&
&  0.39825E+00_JPRB, 0.41044E+00_JPRB, 0.42071E+00_JPRB, 0.43119E+00_JPRB, 0.44324E+00_JPRB, 0.45450E+00_JPRB, 0.46355E+00_JPRB ,&
&  0.46985E+00_JPRB, 0.47079E+00_JPRB, 0.46724E+00_JPRB, 0.46269E+00_JPRB, 0.45673E+00_JPRB, 0.45215E+00_JPRB, 0.44995E+00_JPRB ,&
&  0.44764E+00_JPRB, 0.44605E+00_JPRB, 0.44454E+00_JPRB, 0.44105E+00_JPRB, 0.43720E+00_JPRB, 0.43247E+00_JPRB, 0.42759E+00_JPRB ,&
&  0.42243E+00_JPRB, 0.41774E+00_JPRB, 0.41385E+00_JPRB, 0.41068E+00_JPRB, 0.40859E+00_JPRB, 0.40569E+00_JPRB, 0.40333E+00_JPRB ,&
&  0.40046E+00_JPRB, 0.39945E+00_JPRB, 0.39749E+00_JPRB, 0.39612E+00_JPRB, 0.39455E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.38553E+00_JPRB, 0.38281E+00_JPRB, 0.38018E+00_JPRB, 0.37786E+00_JPRB, 0.37504E+00_JPRB, 0.37216E+00_JPRB, 0.36927E+00_JPRB ,&
&  0.36639E+00_JPRB, 0.36333E+00_JPRB, 0.36011E+00_JPRB, 0.35678E+00_JPRB, 0.35328E+00_JPRB, 0.35061E+00_JPRB, 0.34873E+00_JPRB ,&
&  0.34707E+00_JPRB, 0.34490E+00_JPRB, 0.34268E+00_JPRB, 0.33934E+00_JPRB, 0.33593E+00_JPRB, 0.33283E+00_JPRB, 0.33060E+00_JPRB ,&
&  0.32722E+00_JPRB, 0.32355E+00_JPRB, 0.31968E+00_JPRB, 0.31560E+00_JPRB, 0.31151E+00_JPRB, 0.30742E+00_JPRB, 0.30312E+00_JPRB ,&
&  0.29884E+00_JPRB, 0.29593E+00_JPRB, 0.29323E+00_JPRB, 0.29019E+00_JPRB, 0.28689E+00_JPRB, 0.28621E+00_JPRB, 0.28539E+00_JPRB ,&
&  0.28469E+00_JPRB, 0.28473E+00_JPRB, 0.28734E+00_JPRB, 0.28974E+00_JPRB, 0.29223E+00_JPRB, 0.29477E+00_JPRB, 0.29918E+00_JPRB ,&
&  0.30320E+00_JPRB, 0.30809E+00_JPRB, 0.31429E+00_JPRB, 0.32033E+00_JPRB, 0.32512E+00_JPRB, 0.32826E+00_JPRB, 0.33279E+00_JPRB ,&
&  0.33587E+00_JPRB, 0.33995E+00_JPRB, 0.34268E+00_JPRB, 0.34360E+00_JPRB, 0.34260E+00_JPRB, 0.34152E+00_JPRB, 0.34108E+00_JPRB ,&
&  0.33988E+00_JPRB, 0.34080E+00_JPRB, 0.34166E+00_JPRB, 0.34263E+00_JPRB, 0.34233E+00_JPRB, 0.34382E+00_JPRB, 0.34459E+00_JPRB ,&
&  0.34720E+00_JPRB, 0.35198E+00_JPRB, 0.35684E+00_JPRB, 0.36245E+00_JPRB, 0.36938E+00_JPRB, 0.37825E+00_JPRB, 0.38516E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.39629E+00_JPRB, 0.39987E+00_JPRB, 0.40084E+00_JPRB, 0.40182E+00_JPRB, 0.40293E+00_JPRB, 0.40484E+00_JPRB ,&
&  0.40539E+00_JPRB, 0.40596E+00_JPRB, 0.40662E+00_JPRB, 0.40684E+00_JPRB, 0.40561E+00_JPRB, 0.40435E+00_JPRB, 0.40336E+00_JPRB ,&
&  0.40210E+00_JPRB, 0.40084E+00_JPRB, 0.39989E+00_JPRB, 0.39942E+00_JPRB, 0.39858E+00_JPRB, 0.39748E+00_JPRB, 0.39643E+00_JPRB ,&
&  0.39550E+00_JPRB, 0.39389E+00_JPRB, 0.39206E+00_JPRB, 0.39015E+00_JPRB, 0.38827E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.34991E+00_JPRB, 0.34905E+00_JPRB, 0.34784E+00_JPRB, 0.34697E+00_JPRB, 0.34579E+00_JPRB, 0.34501E+00_JPRB, 0.34398E+00_JPRB ,&
&  0.34311E+00_JPRB, 0.34227E+00_JPRB, 0.34164E+00_JPRB, 0.34059E+00_JPRB, 0.33970E+00_JPRB, 0.33878E+00_JPRB, 0.33796E+00_JPRB ,&
&  0.33697E+00_JPRB, 0.33618E+00_JPRB, 0.33529E+00_JPRB, 0.33429E+00_JPRB, 0.33353E+00_JPRB, 0.33264E+00_JPRB, 0.33110E+00_JPRB ,&
&  0.32850E+00_JPRB, 0.32565E+00_JPRB, 0.32313E+00_JPRB, 0.32118E+00_JPRB, 0.32003E+00_JPRB, 0.31928E+00_JPRB, 0.31857E+00_JPRB ,&
&  0.31824E+00_JPRB, 0.31827E+00_JPRB, 0.31835E+00_JPRB, 0.31818E+00_JPRB, 0.31804E+00_JPRB, 0.31868E+00_JPRB, 0.31906E+00_JPRB ,&
&  0.31921E+00_JPRB, 0.31937E+00_JPRB, 0.32009E+00_JPRB, 0.32076E+00_JPRB, 0.32145E+00_JPRB, 0.32239E+00_JPRB, 0.32395E+00_JPRB ,&
&  0.32558E+00_JPRB, 0.32698E+00_JPRB, 0.32872E+00_JPRB, 0.33055E+00_JPRB, 0.33231E+00_JPRB, 0.33397E+00_JPRB, 0.33620E+00_JPRB ,&
&  0.33844E+00_JPRB, 0.34052E+00_JPRB, 0.34240E+00_JPRB, 0.34370E+00_JPRB, 0.34455E+00_JPRB, 0.34479E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34406E+00_JPRB, 0.34398E+00_JPRB, 0.34343E+00_JPRB, 0.34286E+00_JPRB, 0.34195E+00_JPRB, 0.34178E+00_JPRB, 0.34150E+00_JPRB ,&
&  0.34180E+00_JPRB, 0.34185E+00_JPRB, 0.34236E+00_JPRB, 0.34277E+00_JPRB, 0.34371E+00_JPRB, 0.34479E+00_JPRB, 0.34589E+00_JPRB ,&
&  0.34721E+00_JPRB, 0.34875E+00_JPRB, 0.35078E+00_JPRB, 0.35159E+00_JPRB, 0.35236E+00_JPRB, 0.35308E+00_JPRB, 0.35420E+00_JPRB ,&
&  0.35503E+00_JPRB, 0.35574E+00_JPRB, 0.35649E+00_JPRB, 0.35702E+00_JPRB, 0.35730E+00_JPRB, 0.35747E+00_JPRB, 0.35768E+00_JPRB ,&
&  0.35756E+00_JPRB, 0.35752E+00_JPRB, 0.35741E+00_JPRB, 0.35743E+00_JPRB, 0.35684E+00_JPRB, 0.35614E+00_JPRB, 0.35523E+00_JPRB ,&
&  0.35461E+00_JPRB, 0.35363E+00_JPRB, 0.35285E+00_JPRB, 0.35189E+00_JPRB, 0.35106E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB ,&
&  0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB, 0.33399E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB, 0.22748E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.24099E+00_JPRB, 0.24129E+00_JPRB, 0.24155E+00_JPRB, 0.24182E+00_JPRB, 0.24114E+00_JPRB, 0.24182E+00_JPRB, 0.24233E+00_JPRB ,&
&  0.24305E+00_JPRB, 0.24411E+00_JPRB, 0.24537E+00_JPRB, 0.24636E+00_JPRB, 0.24713E+00_JPRB, 0.24760E+00_JPRB, 0.24791E+00_JPRB ,&
&  0.24794E+00_JPRB, 0.24807E+00_JPRB, 0.24533E+00_JPRB, 0.24438E+00_JPRB, 0.24413E+00_JPRB, 0.24424E+00_JPRB, 0.24446E+00_JPRB ,&
&  0.24464E+00_JPRB, 0.24470E+00_JPRB, 0.24434E+00_JPRB, 0.24461E+00_JPRB, 0.24490E+00_JPRB, 0.24511E+00_JPRB, 0.24535E+00_JPRB ,&
&  0.24557E+00_JPRB, 0.24573E+00_JPRB, 0.24567E+00_JPRB, 0.24547E+00_JPRB, 0.24528E+00_JPRB, 0.24473E+00_JPRB, 0.24385E+00_JPRB ,&
&  0.24280E+00_JPRB, 0.24150E+00_JPRB, 0.24020E+00_JPRB, 0.23902E+00_JPRB, 0.23798E+00_JPRB, 0.23710E+00_JPRB, 0.23626E+00_JPRB ,&
&  0.23528E+00_JPRB, 0.23403E+00_JPRB, 0.23271E+00_JPRB, 0.23143E+00_JPRB, 0.23021E+00_JPRB, 0.22910E+00_JPRB, 0.22795E+00_JPRB ,&
&  0.22706E+00_JPRB, 0.22629E+00_JPRB, 0.22553E+00_JPRB, 0.22487E+00_JPRB, 0.22424E+00_JPRB, 0.22364E+00_JPRB, 0.22306E+00_JPRB ,&
&  0.22231E+00_JPRB, 0.22186E+00_JPRB, 0.22136E+00_JPRB, 0.22109E+00_JPRB, 0.22108E+00_JPRB, 0.22104E+00_JPRB, 0.22109E+00_JPRB ,&
&  0.22118E+00_JPRB, 0.22135E+00_JPRB, 0.22145E+00_JPRB, 0.22163E+00_JPRB, 0.22190E+00_JPRB, 0.22228E+00_JPRB, 0.22275E+00_JPRB ,&
&  0.22325E+00_JPRB, 0.22380E+00_JPRB, 0.22440E+00_JPRB, 0.22509E+00_JPRB, 0.22581E+00_JPRB, 0.22660E+00_JPRB, 0.22735E+00_JPRB ,&
&  0.22810E+00_JPRB, 0.22880E+00_JPRB, 0.22950E+00_JPRB, 0.23004E+00_JPRB, 0.23078E+00_JPRB, 0.23151E+00_JPRB, 0.23237E+00_JPRB ,&
&  0.23324E+00_JPRB, 0.23416E+00_JPRB, 0.23497E+00_JPRB, 0.23582E+00_JPRB, 0.23674E+00_JPRB, 0.23667E+00_JPRB, 0.23662E+00_JPRB ,&
&  0.23666E+00_JPRB, 0.23694E+00_JPRB, 0.23705E+00_JPRB, 0.23742E+00_JPRB, 0.23823E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.26988E+00_JPRB, 0.27641E+00_JPRB, 0.28146E+00_JPRB, 0.28681E+00_JPRB, 0.28507E+00_JPRB, 0.32206E+00_JPRB, 0.32063E+00_JPRB ,&
&  0.32096E+00_JPRB, 0.30624E+00_JPRB, 0.30583E+00_JPRB, 0.30387E+00_JPRB, 0.30058E+00_JPRB, 0.29591E+00_JPRB, 0.29158E+00_JPRB ,&
&  0.28778E+00_JPRB, 0.28294E+00_JPRB, 0.27739E+00_JPRB, 0.27272E+00_JPRB, 0.26893E+00_JPRB, 0.26569E+00_JPRB, 0.26295E+00_JPRB ,&
&  0.26233E+00_JPRB, 0.26355E+00_JPRB, 0.26593E+00_JPRB, 0.26536E+00_JPRB, 0.26674E+00_JPRB, 0.26895E+00_JPRB, 0.27144E+00_JPRB ,&
&  0.27382E+00_JPRB, 0.27577E+00_JPRB, 0.27587E+00_JPRB, 0.27429E+00_JPRB, 0.27278E+00_JPRB, 0.27027E+00_JPRB, 0.26798E+00_JPRB ,&
&  0.26531E+00_JPRB, 0.26235E+00_JPRB, 0.25891E+00_JPRB, 0.25616E+00_JPRB, 0.25320E+00_JPRB, 0.24959E+00_JPRB, 0.24609E+00_JPRB ,&
&  0.24310E+00_JPRB, 0.24043E+00_JPRB, 0.23763E+00_JPRB, 0.23502E+00_JPRB, 0.23256E+00_JPRB, 0.23005E+00_JPRB, 0.22788E+00_JPRB ,&
&  0.22626E+00_JPRB, 0.22488E+00_JPRB, 0.22348E+00_JPRB, 0.22203E+00_JPRB, 0.22058E+00_JPRB, 0.21918E+00_JPRB, 0.21801E+00_JPRB ,&
&  0.21772E+00_JPRB, 0.21701E+00_JPRB, 0.21631E+00_JPRB, 0.21586E+00_JPRB, 0.21579E+00_JPRB, 0.21530E+00_JPRB, 0.21506E+00_JPRB ,&
&  0.21498E+00_JPRB, 0.21499E+00_JPRB, 0.21521E+00_JPRB, 0.21550E+00_JPRB, 0.21591E+00_JPRB, 0.21644E+00_JPRB, 0.21729E+00_JPRB ,&
&  0.21830E+00_JPRB, 0.21951E+00_JPRB, 0.22081E+00_JPRB, 0.22215E+00_JPRB, 0.22353E+00_JPRB, 0.22498E+00_JPRB, 0.22659E+00_JPRB ,&
&  0.22824E+00_JPRB, 0.22985E+00_JPRB, 0.23145E+00_JPRB, 0.23301E+00_JPRB, 0.23544E+00_JPRB, 0.23780E+00_JPRB, 0.24034E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.24625E+00_JPRB, 0.24960E+00_JPRB, 0.25318E+00_JPRB, 0.25709E+00_JPRB, 0.25904E+00_JPRB, 0.26061E+00_JPRB ,&
&  0.26179E+00_JPRB, 0.26461E+00_JPRB, 0.26720E+00_JPRB, 0.26751E+00_JPRB, 0.26760E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.32842E+00_JPRB, 0.32601E+00_JPRB, 0.32618E+00_JPRB, 0.32677E+00_JPRB, 0.32970E+00_JPRB, 0.33446E+00_JPRB, 0.33968E+00_JPRB ,&
&  0.33911E+00_JPRB, 0.34200E+00_JPRB, 0.34131E+00_JPRB, 0.33908E+00_JPRB, 0.33522E+00_JPRB, 0.33190E+00_JPRB, 0.32707E+00_JPRB ,&
&  0.32020E+00_JPRB, 0.31143E+00_JPRB, 0.30285E+00_JPRB, 0.29421E+00_JPRB, 0.28595E+00_JPRB, 0.27735E+00_JPRB, 0.27188E+00_JPRB ,&
&  0.26869E+00_JPRB, 0.26781E+00_JPRB, 0.26978E+00_JPRB, 0.27273E+00_JPRB, 0.27814E+00_JPRB, 0.28550E+00_JPRB, 0.29520E+00_JPRB ,&
&  0.30488E+00_JPRB, 0.31536E+00_JPRB, 0.32076E+00_JPRB, 0.32065E+00_JPRB, 0.32033E+00_JPRB, 0.31817E+00_JPRB, 0.31465E+00_JPRB ,&
&  0.30833E+00_JPRB, 0.30101E+00_JPRB, 0.29271E+00_JPRB, 0.28393E+00_JPRB, 0.27497E+00_JPRB, 0.26626E+00_JPRB, 0.25817E+00_JPRB ,&
&  0.25157E+00_JPRB, 0.24596E+00_JPRB, 0.24090E+00_JPRB, 0.23668E+00_JPRB, 0.23340E+00_JPRB, 0.23092E+00_JPRB, 0.22809E+00_JPRB ,&
&  0.22567E+00_JPRB, 0.22355E+00_JPRB, 0.22106E+00_JPRB, 0.21930E+00_JPRB, 0.21773E+00_JPRB, 0.21669E+00_JPRB, 0.21583E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21544E+00_JPRB, 0.21583E+00_JPRB, 0.21621E+00_JPRB, 0.21670E+00_JPRB, 0.21653E+00_JPRB, 0.21621E+00_JPRB ,&
&  0.21574E+00_JPRB, 0.21532E+00_JPRB, 0.21413E+00_JPRB, 0.21324E+00_JPRB, 0.21264E+00_JPRB, 0.21199E+00_JPRB, 0.21203E+00_JPRB ,&
&  0.21290E+00_JPRB, 0.21380E+00_JPRB, 0.21458E+00_JPRB, 0.21613E+00_JPRB, 0.21769E+00_JPRB, 0.21990E+00_JPRB, 0.22254E+00_JPRB ,&
&  0.22451E+00_JPRB, 0.22682E+00_JPRB, 0.22942E+00_JPRB, 0.23242E+00_JPRB, 0.23596E+00_JPRB, 0.23967E+00_JPRB, 0.24399E+00_JPRB ,&
&  0.24885E+00_JPRB, 0.25389E+00_JPRB, 0.25960E+00_JPRB, 0.26585E+00_JPRB, 0.27346E+00_JPRB, 0.28094E+00_JPRB, 0.29157E+00_JPRB ,&
&  0.30036E+00_JPRB, 0.30188E+00_JPRB, 0.34215E+00_JPRB, 0.33586E+00_JPRB, 0.33150E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.32437E+00_JPRB, 0.32674E+00_JPRB, 0.32998E+00_JPRB, 0.33386E+00_JPRB, 0.33873E+00_JPRB, 0.34296E+00_JPRB, 0.34832E+00_JPRB ,&
&  0.35128E+00_JPRB, 0.34737E+00_JPRB, 0.34761E+00_JPRB, 0.34368E+00_JPRB, 0.33960E+00_JPRB, 0.33445E+00_JPRB, 0.32856E+00_JPRB ,&
&  0.32215E+00_JPRB, 0.31325E+00_JPRB, 0.30373E+00_JPRB, 0.29523E+00_JPRB, 0.28812E+00_JPRB, 0.28241E+00_JPRB, 0.27772E+00_JPRB ,&
&  0.27450E+00_JPRB, 0.27459E+00_JPRB, 0.27673E+00_JPRB, 0.28270E+00_JPRB, 0.29274E+00_JPRB, 0.30546E+00_JPRB, 0.32454E+00_JPRB ,&
&  0.35170E+00_JPRB, 0.35387E+00_JPRB, 0.35823E+00_JPRB, 0.35718E+00_JPRB, 0.35297E+00_JPRB, 0.35069E+00_JPRB, 0.34619E+00_JPRB ,&
&  0.34661E+00_JPRB, 0.34019E+00_JPRB, 0.35011E+00_JPRB, 0.33481E+00_JPRB, 0.30915E+00_JPRB, 0.29382E+00_JPRB, 0.27573E+00_JPRB ,&
&  0.26201E+00_JPRB, 0.25041E+00_JPRB, 0.24164E+00_JPRB, 0.23764E+00_JPRB, 0.23409E+00_JPRB, 0.23236E+00_JPRB, 0.22897E+00_JPRB ,&
&  0.22537E+00_JPRB, 0.22339E+00_JPRB, 0.22036E+00_JPRB, 0.21994E+00_JPRB, 0.21993E+00_JPRB, 0.21940E+00_JPRB, 0.21948E+00_JPRB ,&
&  0.21985E+00_JPRB, 0.22071E+00_JPRB, 0.22222E+00_JPRB, 0.22362E+00_JPRB, 0.22445E+00_JPRB, 0.22493E+00_JPRB, 0.22480E+00_JPRB ,&
&  0.22516E+00_JPRB, 0.22485E+00_JPRB, 0.22406E+00_JPRB, 0.22281E+00_JPRB, 0.22100E+00_JPRB, 0.21912E+00_JPRB, 0.21735E+00_JPRB ,&
&  0.21597E+00_JPRB, 0.21553E+00_JPRB, 0.21519E+00_JPRB, 0.21551E+00_JPRB, 0.21617E+00_JPRB, 0.21789E+00_JPRB, 0.21952E+00_JPRB ,&
&  0.22136E+00_JPRB, 0.22384E+00_JPRB, 0.22606E+00_JPRB, 0.22876E+00_JPRB, 0.23129E+00_JPRB, 0.23295E+00_JPRB, 0.23687E+00_JPRB ,&
&  0.24185E+00_JPRB, 0.24839E+00_JPRB, 0.25602E+00_JPRB, 0.26358E+00_JPRB, 0.27324E+00_JPRB, 0.28898E+00_JPRB, 0.31062E+00_JPRB ,&
&  0.33305E+00_JPRB, 0.35851E+00_JPRB, 0.35348E+00_JPRB, 0.32428E+00_JPRB, 0.32186E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.33443E+00_JPRB, 0.34091E+00_JPRB, 0.34795E+00_JPRB, 0.35543E+00_JPRB, 0.36476E+00_JPRB, 0.36869E+00_JPRB, 0.37067E+00_JPRB ,&
&  0.36672E+00_JPRB, 0.36181E+00_JPRB, 0.35081E+00_JPRB, 0.34027E+00_JPRB, 0.33063E+00_JPRB, 0.32288E+00_JPRB, 0.31730E+00_JPRB ,&
&  0.31433E+00_JPRB, 0.31202E+00_JPRB, 0.30875E+00_JPRB, 0.30558E+00_JPRB, 0.30299E+00_JPRB, 0.30290E+00_JPRB, 0.30292E+00_JPRB ,&
&  0.30199E+00_JPRB, 0.30195E+00_JPRB, 0.30079E+00_JPRB, 0.30597E+00_JPRB, 0.31501E+00_JPRB, 0.32624E+00_JPRB, 0.34087E+00_JPRB ,&
&  0.36139E+00_JPRB, 0.37274E+00_JPRB, 0.38827E+00_JPRB, 0.38896E+00_JPRB, 0.38705E+00_JPRB, 0.38317E+00_JPRB, 0.38008E+00_JPRB ,&
&  0.37550E+00_JPRB, 0.36894E+00_JPRB, 0.36424E+00_JPRB, 0.35853E+00_JPRB, 0.32993E+00_JPRB, 0.30711E+00_JPRB, 0.28969E+00_JPRB ,&
&  0.27428E+00_JPRB, 0.25976E+00_JPRB, 0.24720E+00_JPRB, 0.24078E+00_JPRB, 0.23588E+00_JPRB, 0.23298E+00_JPRB, 0.22998E+00_JPRB ,&
&  0.22681E+00_JPRB, 0.22547E+00_JPRB, 0.22353E+00_JPRB, 0.22285E+00_JPRB, 0.22377E+00_JPRB, 0.22382E+00_JPRB, 0.22461E+00_JPRB ,&
&  0.22526E+00_JPRB, 0.22617E+00_JPRB, 0.22826E+00_JPRB, 0.22985E+00_JPRB, 0.23155E+00_JPRB, 0.23319E+00_JPRB, 0.23407E+00_JPRB ,&
&  0.23574E+00_JPRB, 0.23712E+00_JPRB, 0.23709E+00_JPRB, 0.23640E+00_JPRB, 0.23475E+00_JPRB, 0.23121E+00_JPRB, 0.22762E+00_JPRB ,&
&  0.22381E+00_JPRB, 0.22105E+00_JPRB, 0.21983E+00_JPRB, 0.21891E+00_JPRB, 0.21965E+00_JPRB, 0.22048E+00_JPRB, 0.22148E+00_JPRB ,&
&  0.22332E+00_JPRB, 0.22502E+00_JPRB, 0.22694E+00_JPRB, 0.22956E+00_JPRB, 0.23166E+00_JPRB, 0.23353E+00_JPRB, 0.23593E+00_JPRB ,&
&  0.23656E+00_JPRB, 0.24068E+00_JPRB, 0.24585E+00_JPRB, 0.25221E+00_JPRB, 0.26266E+00_JPRB, 0.27242E+00_JPRB, 0.29461E+00_JPRB ,&
&  0.33169E+00_JPRB, 0.40218E+00_JPRB, 0.39684E+00_JPRB, 0.32680E+00_JPRB, 0.32884E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.35726E+00_JPRB, 0.36584E+00_JPRB, 0.37349E+00_JPRB, 0.37937E+00_JPRB, 0.38290E+00_JPRB, 0.38347E+00_JPRB, 0.38605E+00_JPRB ,&
&  0.39334E+00_JPRB, 0.39758E+00_JPRB, 0.40042E+00_JPRB, 0.39989E+00_JPRB, 0.39931E+00_JPRB, 0.38022E+00_JPRB, 0.37503E+00_JPRB ,&
&  0.36925E+00_JPRB, 0.36522E+00_JPRB, 0.35985E+00_JPRB, 0.35992E+00_JPRB, 0.35981E+00_JPRB, 0.36171E+00_JPRB, 0.36244E+00_JPRB ,&
&  0.36227E+00_JPRB, 0.35995E+00_JPRB, 0.35666E+00_JPRB, 0.35463E+00_JPRB, 0.35594E+00_JPRB, 0.35756E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.35065E+00_JPRB, 0.34984E+00_JPRB, 0.35318E+00_JPRB, 0.36082E+00_JPRB, 0.40186E+00_JPRB, 0.39703E+00_JPRB, 0.39432E+00_JPRB ,&
&  0.39102E+00_JPRB, 0.38692E+00_JPRB, 0.38647E+00_JPRB, 0.38016E+00_JPRB, 0.37081E+00_JPRB, 0.35059E+00_JPRB, 0.33925E+00_JPRB ,&
&  0.30454E+00_JPRB, 0.28509E+00_JPRB, 0.26794E+00_JPRB, 0.25669E+00_JPRB, 0.24691E+00_JPRB, 0.24126E+00_JPRB, 0.23607E+00_JPRB ,&
&  0.23215E+00_JPRB, 0.22913E+00_JPRB, 0.22644E+00_JPRB, 0.22680E+00_JPRB, 0.22841E+00_JPRB, 0.22943E+00_JPRB, 0.23041E+00_JPRB ,&
&  0.23100E+00_JPRB, 0.23148E+00_JPRB, 0.23211E+00_JPRB, 0.23334E+00_JPRB, 0.23495E+00_JPRB, 0.23707E+00_JPRB, 0.23975E+00_JPRB ,&
&  0.24418E+00_JPRB, 0.24773E+00_JPRB, 0.24930E+00_JPRB, 0.25172E+00_JPRB, 0.25140E+00_JPRB, 0.24626E+00_JPRB, 0.24009E+00_JPRB ,&
&  0.23375E+00_JPRB, 0.23042E+00_JPRB, 0.22721E+00_JPRB, 0.22500E+00_JPRB, 0.22449E+00_JPRB, 0.22455E+00_JPRB, 0.22638E+00_JPRB ,&
&  0.22839E+00_JPRB, 0.23012E+00_JPRB, 0.23131E+00_JPRB, 0.23262E+00_JPRB, 0.23432E+00_JPRB, 0.23568E+00_JPRB, 0.23748E+00_JPRB ,&
&  0.23975E+00_JPRB, 0.24286E+00_JPRB, 0.24749E+00_JPRB, 0.25279E+00_JPRB, 0.25929E+00_JPRB, 0.26932E+00_JPRB, 0.28737E+00_JPRB ,&
&  0.31207E+00_JPRB, 0.36896E+00_JPRB, 0.37539E+00_JPRB, 0.34420E+00_JPRB, 0.35000E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.36972E+00_JPRB, 0.37945E+00_JPRB, 0.38467E+00_JPRB, 0.38979E+00_JPRB, 0.39182E+00_JPRB, 0.39314E+00_JPRB, 0.40096E+00_JPRB ,&
&  0.40570E+00_JPRB, 0.40935E+00_JPRB, 0.41149E+00_JPRB, 0.41202E+00_JPRB, 0.40743E+00_JPRB, 0.40069E+00_JPRB, 0.39829E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.39751E+00_JPRB, 0.39641E+00_JPRB, 0.39645E+00_JPRB, 0.39698E+00_JPRB, 0.40178E+00_JPRB, 0.40846E+00_JPRB ,&
&  0.41001E+00_JPRB, 0.41470E+00_JPRB, 0.42708E+00_JPRB, 0.43483E+00_JPRB, 0.43974E+00_JPRB, 0.43960E+00_JPRB, 0.43411E+00_JPRB ,&
&  0.40759E+00_JPRB, 0.37996E+00_JPRB, 0.35214E+00_JPRB, 0.35374E+00_JPRB, 0.40788E+00_JPRB, 0.41452E+00_JPRB, 0.41350E+00_JPRB ,&
&  0.41189E+00_JPRB, 0.41635E+00_JPRB, 0.41650E+00_JPRB, 0.40204E+00_JPRB, 0.39145E+00_JPRB, 0.37660E+00_JPRB, 0.36403E+00_JPRB ,&
&  0.35362E+00_JPRB, 0.34581E+00_JPRB, 0.33292E+00_JPRB, 0.30607E+00_JPRB, 0.28815E+00_JPRB, 0.27900E+00_JPRB, 0.26536E+00_JPRB ,&
&  0.25521E+00_JPRB, 0.25062E+00_JPRB, 0.24948E+00_JPRB, 0.24934E+00_JPRB, 0.25347E+00_JPRB, 0.25534E+00_JPRB, 0.25440E+00_JPRB ,&
&  0.25166E+00_JPRB, 0.24931E+00_JPRB, 0.24587E+00_JPRB, 0.24089E+00_JPRB, 0.23818E+00_JPRB, 0.23721E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24651E+00_JPRB, 0.25923E+00_JPRB, 0.26840E+00_JPRB, 0.26585E+00_JPRB, 0.25505E+00_JPRB, 0.24456E+00_JPRB ,&
&  0.23804E+00_JPRB, 0.23532E+00_JPRB, 0.23331E+00_JPRB, 0.23128E+00_JPRB, 0.23184E+00_JPRB, 0.23371E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23560E+00_JPRB, 0.23708E+00_JPRB, 0.23788E+00_JPRB, 0.23890E+00_JPRB, 0.23979E+00_JPRB, 0.24128E+00_JPRB, 0.24338E+00_JPRB ,&
&  0.24552E+00_JPRB, 0.25059E+00_JPRB, 0.25632E+00_JPRB, 0.26267E+00_JPRB, 0.26663E+00_JPRB, 0.26959E+00_JPRB, 0.27516E+00_JPRB ,&
&  0.28152E+00_JPRB, 0.29721E+00_JPRB, 0.32745E+00_JPRB, 0.36538E+00_JPRB, 0.36114E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.37960E+00_JPRB, 0.38656E+00_JPRB, 0.39470E+00_JPRB, 0.40547E+00_JPRB, 0.40786E+00_JPRB, 0.40948E+00_JPRB, 0.41867E+00_JPRB ,&
&  0.41928E+00_JPRB, 0.41646E+00_JPRB, 0.41314E+00_JPRB, 0.41315E+00_JPRB, 0.41493E+00_JPRB, 0.41704E+00_JPRB, 0.41624E+00_JPRB ,&
&  0.40691E+00_JPRB, 0.40149E+00_JPRB, 0.40396E+00_JPRB, 0.40674E+00_JPRB, 0.41114E+00_JPRB, 0.41764E+00_JPRB, 0.42322E+00_JPRB ,&
&  0.42616E+00_JPRB, 0.42959E+00_JPRB, 0.43182E+00_JPRB, 0.43788E+00_JPRB, 0.44743E+00_JPRB, 0.45399E+00_JPRB, 0.46052E+00_JPRB ,&
&  0.44747E+00_JPRB, 0.40006E+00_JPRB, 0.36931E+00_JPRB, 0.37603E+00_JPRB, 0.42689E+00_JPRB, 0.43689E+00_JPRB, 0.43776E+00_JPRB ,&
&  0.44089E+00_JPRB, 0.44857E+00_JPRB, 0.44796E+00_JPRB, 0.44195E+00_JPRB, 0.43026E+00_JPRB, 0.41152E+00_JPRB, 0.38853E+00_JPRB ,&
&  0.37311E+00_JPRB, 0.36268E+00_JPRB, 0.35596E+00_JPRB, 0.34955E+00_JPRB, 0.34613E+00_JPRB, 0.34122E+00_JPRB, 0.33369E+00_JPRB ,&
&  0.33079E+00_JPRB, 0.32926E+00_JPRB, 0.33021E+00_JPRB, 0.33201E+00_JPRB, 0.33419E+00_JPRB, 0.33586E+00_JPRB, 0.33394E+00_JPRB ,&
&  0.32159E+00_JPRB, 0.30633E+00_JPRB, 0.29174E+00_JPRB, 0.27587E+00_JPRB, 0.26216E+00_JPRB, 0.25333E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.24348E+00_JPRB, 0.24855E+00_JPRB, 0.26436E+00_JPRB, 0.28295E+00_JPRB, 0.31625E+00_JPRB, 0.29228E+00_JPRB, 0.26869E+00_JPRB ,&
&  0.25683E+00_JPRB, 0.25115E+00_JPRB, 0.24797E+00_JPRB, 0.24776E+00_JPRB, 0.24920E+00_JPRB, 0.24963E+00_JPRB, 0.24968E+00_JPRB ,&
&  0.25077E+00_JPRB, 0.25292E+00_JPRB, 0.25474E+00_JPRB, 0.25624E+00_JPRB, 0.25862E+00_JPRB, 0.26142E+00_JPRB, 0.26290E+00_JPRB ,&
&  0.26353E+00_JPRB, 0.26789E+00_JPRB, 0.27706E+00_JPRB, 0.28762E+00_JPRB, 0.29835E+00_JPRB, 0.30955E+00_JPRB, 0.32732E+00_JPRB ,&
&  0.34978E+00_JPRB, 0.37752E+00_JPRB, 0.37644E+00_JPRB, 0.37595E+00_JPRB, 0.37629E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.40123E+00_JPRB, 0.41237E+00_JPRB, 0.42553E+00_JPRB, 0.43487E+00_JPRB, 0.43408E+00_JPRB, 0.43418E+00_JPRB, 0.44050E+00_JPRB ,&
&  0.43489E+00_JPRB, 0.42796E+00_JPRB, 0.42200E+00_JPRB, 0.41575E+00_JPRB, 0.41336E+00_JPRB, 0.41183E+00_JPRB, 0.41463E+00_JPRB ,&
&  0.41664E+00_JPRB, 0.41545E+00_JPRB, 0.41368E+00_JPRB, 0.41424E+00_JPRB, 0.42060E+00_JPRB, 0.42866E+00_JPRB, 0.43542E+00_JPRB ,&
&  0.43655E+00_JPRB, 0.43990E+00_JPRB, 0.44540E+00_JPRB, 0.45008E+00_JPRB, 0.45255E+00_JPRB, 0.45419E+00_JPRB, 0.45317E+00_JPRB ,&
&  0.45085E+00_JPRB, 0.44940E+00_JPRB, 0.43257E+00_JPRB, 0.44689E+00_JPRB, 0.45138E+00_JPRB, 0.45238E+00_JPRB, 0.45832E+00_JPRB ,&
&  0.46629E+00_JPRB, 0.47493E+00_JPRB, 0.47878E+00_JPRB, 0.47990E+00_JPRB, 0.47527E+00_JPRB, 0.46605E+00_JPRB, 0.44207E+00_JPRB ,&
&  0.42195E+00_JPRB, 0.40096E+00_JPRB, 0.38788E+00_JPRB, 0.38004E+00_JPRB, 0.37400E+00_JPRB, 0.36866E+00_JPRB, 0.36335E+00_JPRB ,&
&  0.35601E+00_JPRB, 0.34947E+00_JPRB, 0.34552E+00_JPRB, 0.34569E+00_JPRB, 0.34998E+00_JPRB, 0.36094E+00_JPRB, 0.37451E+00_JPRB ,&
&  0.38066E+00_JPRB, 0.37358E+00_JPRB, 0.36577E+00_JPRB, 0.35736E+00_JPRB, 0.34411E+00_JPRB, 0.31617E+00_JPRB, 0.29935E+00_JPRB ,&
&  0.29501E+00_JPRB, 0.30752E+00_JPRB, 0.32899E+00_JPRB, 0.33462E+00_JPRB, 0.33946E+00_JPRB, 0.33722E+00_JPRB, 0.33202E+00_JPRB ,&
&  0.32394E+00_JPRB, 0.30021E+00_JPRB, 0.28697E+00_JPRB, 0.28292E+00_JPRB, 0.28050E+00_JPRB, 0.28034E+00_JPRB, 0.28275E+00_JPRB ,&
&  0.28673E+00_JPRB, 0.29427E+00_JPRB, 0.29954E+00_JPRB, 0.30172E+00_JPRB, 0.30486E+00_JPRB, 0.30493E+00_JPRB, 0.30384E+00_JPRB ,&
&  0.30004E+00_JPRB, 0.30622E+00_JPRB, 0.32408E+00_JPRB, 0.35301E+00_JPRB, 0.35929E+00_JPRB, 0.36663E+00_JPRB, 0.37559E+00_JPRB ,&
&  0.38090E+00_JPRB, 0.38571E+00_JPRB, 0.38590E+00_JPRB, 0.38689E+00_JPRB, 0.39227E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.41942E+00_JPRB, 0.43194E+00_JPRB, 0.44792E+00_JPRB, 0.45938E+00_JPRB, 0.46449E+00_JPRB, 0.47007E+00_JPRB, 0.47262E+00_JPRB ,&
&  0.46785E+00_JPRB, 0.46596E+00_JPRB, 0.46035E+00_JPRB, 0.45211E+00_JPRB, 0.44495E+00_JPRB, 0.44126E+00_JPRB, 0.44147E+00_JPRB ,&
&  0.44358E+00_JPRB, 0.44539E+00_JPRB, 0.44187E+00_JPRB, 0.44005E+00_JPRB, 0.44384E+00_JPRB, 0.44901E+00_JPRB, 0.45234E+00_JPRB ,&
&  0.45357E+00_JPRB, 0.45484E+00_JPRB, 0.45950E+00_JPRB, 0.46253E+00_JPRB, 0.46279E+00_JPRB, 0.45642E+00_JPRB, 0.45184E+00_JPRB ,&
&  0.44713E+00_JPRB, 0.44562E+00_JPRB, 0.45267E+00_JPRB, 0.44456E+00_JPRB, 0.45964E+00_JPRB, 0.46178E+00_JPRB, 0.46753E+00_JPRB ,&
&  0.47588E+00_JPRB, 0.48940E+00_JPRB, 0.49961E+00_JPRB, 0.50736E+00_JPRB, 0.50984E+00_JPRB, 0.51314E+00_JPRB, 0.50353E+00_JPRB ,&
&  0.48996E+00_JPRB, 0.47144E+00_JPRB, 0.45294E+00_JPRB, 0.43315E+00_JPRB, 0.41451E+00_JPRB, 0.39989E+00_JPRB, 0.39258E+00_JPRB ,&
&  0.38373E+00_JPRB, 0.36999E+00_JPRB, 0.35673E+00_JPRB, 0.35145E+00_JPRB, 0.35599E+00_JPRB, 0.36611E+00_JPRB, 0.38171E+00_JPRB ,&
&  0.39118E+00_JPRB, 0.39405E+00_JPRB, 0.39088E+00_JPRB, 0.38645E+00_JPRB, 0.38191E+00_JPRB, 0.37505E+00_JPRB, 0.37114E+00_JPRB ,&
&  0.36784E+00_JPRB, 0.36213E+00_JPRB, 0.35914E+00_JPRB, 0.36095E+00_JPRB, 0.36417E+00_JPRB, 0.36613E+00_JPRB, 0.36432E+00_JPRB ,&
&  0.36107E+00_JPRB, 0.35673E+00_JPRB, 0.34869E+00_JPRB, 0.34398E+00_JPRB, 0.34386E+00_JPRB, 0.34815E+00_JPRB, 0.35580E+00_JPRB ,&
&  0.35831E+00_JPRB, 0.36413E+00_JPRB, 0.36989E+00_JPRB, 0.37211E+00_JPRB, 0.37477E+00_JPRB, 0.37704E+00_JPRB, 0.37731E+00_JPRB ,&
&  0.37565E+00_JPRB, 0.37065E+00_JPRB, 0.36356E+00_JPRB, 0.36019E+00_JPRB, 0.36089E+00_JPRB, 0.36724E+00_JPRB, 0.37617E+00_JPRB ,&
&  0.38510E+00_JPRB, 0.39857E+00_JPRB, 0.40896E+00_JPRB, 0.41133E+00_JPRB, 0.41352E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.44777E+00_JPRB, 0.45287E+00_JPRB, 0.46578E+00_JPRB, 0.47649E+00_JPRB, 0.48004E+00_JPRB, 0.48957E+00_JPRB, 0.50084E+00_JPRB ,&
&  0.51028E+00_JPRB, 0.51230E+00_JPRB, 0.50856E+00_JPRB, 0.49811E+00_JPRB, 0.48573E+00_JPRB, 0.47553E+00_JPRB, 0.47136E+00_JPRB ,&
&  0.47347E+00_JPRB, 0.47654E+00_JPRB, 0.48207E+00_JPRB, 0.48511E+00_JPRB, 0.48870E+00_JPRB, 0.48744E+00_JPRB, 0.48773E+00_JPRB ,&
&  0.48938E+00_JPRB, 0.49033E+00_JPRB, 0.49085E+00_JPRB, 0.48734E+00_JPRB, 0.48463E+00_JPRB, 0.48235E+00_JPRB, 0.47417E+00_JPRB ,&
&  0.46493E+00_JPRB, 0.45867E+00_JPRB, 0.46274E+00_JPRB, 0.47444E+00_JPRB, 0.48189E+00_JPRB, 0.47915E+00_JPRB, 0.47798E+00_JPRB ,&
&  0.47818E+00_JPRB, 0.48281E+00_JPRB, 0.49031E+00_JPRB, 0.50186E+00_JPRB, 0.51024E+00_JPRB, 0.52138E+00_JPRB, 0.52203E+00_JPRB ,&
&  0.51566E+00_JPRB, 0.50650E+00_JPRB, 0.50370E+00_JPRB, 0.50282E+00_JPRB, 0.47487E+00_JPRB, 0.44408E+00_JPRB, 0.42893E+00_JPRB ,&
&  0.42082E+00_JPRB, 0.41070E+00_JPRB, 0.39498E+00_JPRB, 0.38381E+00_JPRB, 0.38243E+00_JPRB, 0.38696E+00_JPRB, 0.39826E+00_JPRB ,&
&  0.40652E+00_JPRB, 0.41865E+00_JPRB, 0.43320E+00_JPRB, 0.44245E+00_JPRB, 0.44250E+00_JPRB, 0.43068E+00_JPRB, 0.41994E+00_JPRB ,&
&  0.41375E+00_JPRB, 0.41221E+00_JPRB, 0.41082E+00_JPRB, 0.40873E+00_JPRB, 0.40670E+00_JPRB, 0.40612E+00_JPRB, 0.40497E+00_JPRB ,&
&  0.40068E+00_JPRB, 0.39329E+00_JPRB, 0.38425E+00_JPRB, 0.37909E+00_JPRB, 0.37776E+00_JPRB, 0.37738E+00_JPRB, 0.37882E+00_JPRB ,&
&  0.38783E+00_JPRB, 0.39901E+00_JPRB, 0.40822E+00_JPRB, 0.41265E+00_JPRB, 0.40995E+00_JPRB, 0.40227E+00_JPRB, 0.39752E+00_JPRB ,&
&  0.39372E+00_JPRB, 0.39099E+00_JPRB, 0.38748E+00_JPRB, 0.38570E+00_JPRB, 0.38514E+00_JPRB, 0.39184E+00_JPRB, 0.40686E+00_JPRB ,&
&  0.42582E+00_JPRB, 0.45107E+00_JPRB, 0.46263E+00_JPRB, 0.46125E+00_JPRB, 0.45400E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.50149E+00_JPRB, 0.48477E+00_JPRB, 0.48511E+00_JPRB, 0.49208E+00_JPRB, 0.49607E+00_JPRB, 0.50397E+00_JPRB, 0.52350E+00_JPRB ,&
&  0.53516E+00_JPRB, 0.54267E+00_JPRB, 0.54559E+00_JPRB, 0.54011E+00_JPRB, 0.53890E+00_JPRB, 0.53159E+00_JPRB, 0.51884E+00_JPRB ,&
&  0.51357E+00_JPRB, 0.51249E+00_JPRB, 0.51406E+00_JPRB, 0.51773E+00_JPRB, 0.51978E+00_JPRB, 0.52236E+00_JPRB, 0.52530E+00_JPRB ,&
&  0.52687E+00_JPRB, 0.52680E+00_JPRB, 0.52599E+00_JPRB, 0.52628E+00_JPRB, 0.52549E+00_JPRB, 0.51824E+00_JPRB, 0.51006E+00_JPRB ,&
&  0.50101E+00_JPRB, 0.49200E+00_JPRB, 0.48970E+00_JPRB, 0.49437E+00_JPRB, 0.49955E+00_JPRB, 0.50274E+00_JPRB, 0.50059E+00_JPRB ,&
&  0.49586E+00_JPRB, 0.49274E+00_JPRB, 0.49461E+00_JPRB, 0.50152E+00_JPRB, 0.51018E+00_JPRB, 0.51981E+00_JPRB, 0.52485E+00_JPRB ,&
&  0.51942E+00_JPRB, 0.50214E+00_JPRB, 0.49173E+00_JPRB, 0.49145E+00_JPRB, 0.48849E+00_JPRB, 0.47490E+00_JPRB, 0.46519E+00_JPRB ,&
&  0.46045E+00_JPRB, 0.45176E+00_JPRB, 0.44253E+00_JPRB, 0.42978E+00_JPRB, 0.41802E+00_JPRB, 0.41360E+00_JPRB, 0.41214E+00_JPRB ,&
&  0.41490E+00_JPRB, 0.43185E+00_JPRB, 0.46353E+00_JPRB, 0.49506E+00_JPRB, 0.50930E+00_JPRB, 0.50335E+00_JPRB, 0.48732E+00_JPRB ,&
&  0.48038E+00_JPRB, 0.47771E+00_JPRB, 0.47925E+00_JPRB, 0.47880E+00_JPRB, 0.47514E+00_JPRB, 0.46649E+00_JPRB, 0.45958E+00_JPRB ,&
&  0.44959E+00_JPRB, 0.43854E+00_JPRB, 0.43400E+00_JPRB, 0.43115E+00_JPRB, 0.42806E+00_JPRB, 0.42795E+00_JPRB, 0.42678E+00_JPRB ,&
&  0.42693E+00_JPRB, 0.43064E+00_JPRB, 0.44228E+00_JPRB, 0.44917E+00_JPRB, 0.45071E+00_JPRB, 0.44057E+00_JPRB, 0.44127E+00_JPRB ,&
&  0.44355E+00_JPRB, 0.44105E+00_JPRB, 0.43640E+00_JPRB, 0.43606E+00_JPRB, 0.43583E+00_JPRB, 0.43744E+00_JPRB, 0.44853E+00_JPRB ,&
&  0.47467E+00_JPRB, 0.49790E+00_JPRB, 0.51714E+00_JPRB, 0.52287E+00_JPRB, 0.51884E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.57327E+00_JPRB, 0.54508E+00_JPRB, 0.53397E+00_JPRB, 0.53577E+00_JPRB, 0.53931E+00_JPRB, 0.53536E+00_JPRB, 0.54281E+00_JPRB ,&
&  0.55420E+00_JPRB, 0.56275E+00_JPRB, 0.56738E+00_JPRB, 0.56365E+00_JPRB, 0.56112E+00_JPRB, 0.55826E+00_JPRB, 0.54370E+00_JPRB ,&
&  0.53894E+00_JPRB, 0.53716E+00_JPRB, 0.54108E+00_JPRB, 0.54779E+00_JPRB, 0.55504E+00_JPRB, 0.56681E+00_JPRB, 0.57552E+00_JPRB ,&
&  0.58130E+00_JPRB, 0.58493E+00_JPRB, 0.58906E+00_JPRB, 0.58929E+00_JPRB, 0.58592E+00_JPRB, 0.57726E+00_JPRB, 0.57067E+00_JPRB ,&
&  0.56689E+00_JPRB, 0.55565E+00_JPRB, 0.54592E+00_JPRB, 0.54474E+00_JPRB, 0.54792E+00_JPRB, 0.55096E+00_JPRB, 0.55025E+00_JPRB ,&
&  0.53702E+00_JPRB, 0.52009E+00_JPRB, 0.51811E+00_JPRB, 0.51806E+00_JPRB, 0.52339E+00_JPRB, 0.53536E+00_JPRB, 0.53881E+00_JPRB ,&
&  0.53595E+00_JPRB, 0.52236E+00_JPRB, 0.50370E+00_JPRB, 0.50039E+00_JPRB, 0.50316E+00_JPRB, 0.49751E+00_JPRB, 0.49162E+00_JPRB ,&
&  0.49049E+00_JPRB, 0.48728E+00_JPRB, 0.48119E+00_JPRB, 0.47299E+00_JPRB, 0.46039E+00_JPRB, 0.44388E+00_JPRB, 0.43515E+00_JPRB ,&
&  0.43397E+00_JPRB, 0.44771E+00_JPRB, 0.47309E+00_JPRB, 0.50237E+00_JPRB, 0.53064E+00_JPRB, 0.54471E+00_JPRB, 0.54460E+00_JPRB ,&
&  0.53360E+00_JPRB, 0.53153E+00_JPRB, 0.53698E+00_JPRB, 0.54125E+00_JPRB, 0.54193E+00_JPRB, 0.53127E+00_JPRB, 0.51476E+00_JPRB ,&
&  0.50297E+00_JPRB, 0.49395E+00_JPRB, 0.49276E+00_JPRB, 0.49759E+00_JPRB, 0.50249E+00_JPRB, 0.50361E+00_JPRB, 0.49727E+00_JPRB ,&
&  0.49038E+00_JPRB, 0.48343E+00_JPRB, 0.47966E+00_JPRB, 0.48280E+00_JPRB, 0.48865E+00_JPRB, 0.49432E+00_JPRB, 0.50669E+00_JPRB ,&
&  0.52438E+00_JPRB, 0.54235E+00_JPRB, 0.54657E+00_JPRB, 0.54255E+00_JPRB, 0.54223E+00_JPRB, 0.53966E+00_JPRB, 0.53708E+00_JPRB ,&
&  0.54670E+00_JPRB, 0.56849E+00_JPRB, 0.58421E+00_JPRB, 0.59329E+00_JPRB, 0.59374E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.62063E+00_JPRB, 0.59633E+00_JPRB, 0.57864E+00_JPRB, 0.57217E+00_JPRB, 0.56876E+00_JPRB, 0.55705E+00_JPRB, 0.55210E+00_JPRB ,&
&  0.55835E+00_JPRB, 0.56039E+00_JPRB, 0.56763E+00_JPRB, 0.56746E+00_JPRB, 0.56674E+00_JPRB, 0.56919E+00_JPRB, 0.56150E+00_JPRB ,&
&  0.56136E+00_JPRB, 0.57014E+00_JPRB, 0.58041E+00_JPRB, 0.59434E+00_JPRB, 0.60606E+00_JPRB, 0.61497E+00_JPRB, 0.62259E+00_JPRB ,&
&  0.62566E+00_JPRB, 0.63223E+00_JPRB, 0.63592E+00_JPRB, 0.63196E+00_JPRB, 0.62159E+00_JPRB, 0.61365E+00_JPRB, 0.61253E+00_JPRB ,&
&  0.61172E+00_JPRB, 0.60605E+00_JPRB, 0.56231E+00_JPRB, 0.58597E+00_JPRB, 0.59291E+00_JPRB, 0.59568E+00_JPRB, 0.59721E+00_JPRB ,&
&  0.59140E+00_JPRB, 0.57604E+00_JPRB, 0.56900E+00_JPRB, 0.55759E+00_JPRB, 0.55651E+00_JPRB, 0.56242E+00_JPRB, 0.56144E+00_JPRB ,&
&  0.55806E+00_JPRB, 0.54858E+00_JPRB, 0.53528E+00_JPRB, 0.52483E+00_JPRB, 0.52245E+00_JPRB, 0.52107E+00_JPRB, 0.51424E+00_JPRB ,&
&  0.50926E+00_JPRB, 0.50826E+00_JPRB, 0.50639E+00_JPRB, 0.50071E+00_JPRB, 0.48625E+00_JPRB, 0.47267E+00_JPRB, 0.46965E+00_JPRB ,&
&  0.47225E+00_JPRB, 0.47827E+00_JPRB, 0.49114E+00_JPRB, 0.50826E+00_JPRB, 0.52709E+00_JPRB, 0.54499E+00_JPRB, 0.56144E+00_JPRB ,&
&  0.56823E+00_JPRB, 0.57225E+00_JPRB, 0.57581E+00_JPRB, 0.58021E+00_JPRB, 0.58085E+00_JPRB, 0.57318E+00_JPRB, 0.55451E+00_JPRB ,&
&  0.53470E+00_JPRB, 0.52977E+00_JPRB, 0.53395E+00_JPRB, 0.54312E+00_JPRB, 0.54791E+00_JPRB, 0.54729E+00_JPRB, 0.54039E+00_JPRB ,&
&  0.53576E+00_JPRB, 0.52716E+00_JPRB, 0.51850E+00_JPRB, 0.51434E+00_JPRB, 0.51425E+00_JPRB, 0.52042E+00_JPRB, 0.53766E+00_JPRB ,&
&  0.55819E+00_JPRB, 0.58114E+00_JPRB, 0.60192E+00_JPRB, 0.60993E+00_JPRB, 0.61425E+00_JPRB, 0.61384E+00_JPRB, 0.60500E+00_JPRB ,&
&  0.60860E+00_JPRB, 0.62680E+00_JPRB, 0.63623E+00_JPRB, 0.63755E+00_JPRB, 0.63540E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.67128E+00_JPRB, 0.65233E+00_JPRB, 0.64101E+00_JPRB, 0.62992E+00_JPRB, 0.62186E+00_JPRB, 0.60415E+00_JPRB, 0.59420E+00_JPRB ,&
&  0.58807E+00_JPRB, 0.58446E+00_JPRB, 0.58433E+00_JPRB, 0.58495E+00_JPRB, 0.59495E+00_JPRB, 0.60108E+00_JPRB, 0.60312E+00_JPRB ,&
&  0.61025E+00_JPRB, 0.62719E+00_JPRB, 0.64269E+00_JPRB, 0.65585E+00_JPRB, 0.66646E+00_JPRB, 0.67441E+00_JPRB, 0.67470E+00_JPRB ,&
&  0.67886E+00_JPRB, 0.68724E+00_JPRB, 0.69152E+00_JPRB, 0.68229E+00_JPRB, 0.67053E+00_JPRB, 0.66445E+00_JPRB, 0.66458E+00_JPRB ,&
&  0.65514E+00_JPRB, 0.60976E+00_JPRB, 0.62352E+00_JPRB, 0.63487E+00_JPRB, 0.64761E+00_JPRB, 0.65304E+00_JPRB, 0.65892E+00_JPRB ,&
&  0.65993E+00_JPRB, 0.64391E+00_JPRB, 0.62455E+00_JPRB, 0.61040E+00_JPRB, 0.61351E+00_JPRB, 0.61104E+00_JPRB, 0.60265E+00_JPRB ,&
&  0.59971E+00_JPRB, 0.59741E+00_JPRB, 0.59079E+00_JPRB, 0.57839E+00_JPRB, 0.56716E+00_JPRB, 0.55985E+00_JPRB, 0.55528E+00_JPRB ,&
&  0.54477E+00_JPRB, 0.53592E+00_JPRB, 0.53105E+00_JPRB, 0.52567E+00_JPRB, 0.52093E+00_JPRB, 0.52461E+00_JPRB, 0.52746E+00_JPRB ,&
&  0.52577E+00_JPRB, 0.52557E+00_JPRB, 0.53256E+00_JPRB, 0.54591E+00_JPRB, 0.55721E+00_JPRB, 0.57040E+00_JPRB, 0.58608E+00_JPRB ,&
&  0.60138E+00_JPRB, 0.61452E+00_JPRB, 0.61858E+00_JPRB, 0.62250E+00_JPRB, 0.61774E+00_JPRB, 0.60998E+00_JPRB, 0.59386E+00_JPRB ,&
&  0.57634E+00_JPRB, 0.57443E+00_JPRB, 0.57667E+00_JPRB, 0.58363E+00_JPRB, 0.58309E+00_JPRB, 0.58070E+00_JPRB, 0.57882E+00_JPRB ,&
&  0.57465E+00_JPRB, 0.57114E+00_JPRB, 0.56630E+00_JPRB, 0.56161E+00_JPRB, 0.56468E+00_JPRB, 0.57040E+00_JPRB, 0.58369E+00_JPRB ,&
&  0.60144E+00_JPRB, 0.62034E+00_JPRB, 0.64024E+00_JPRB, 0.65467E+00_JPRB, 0.66034E+00_JPRB, 0.66377E+00_JPRB, 0.66667E+00_JPRB ,&
&  0.66712E+00_JPRB, 0.66546E+00_JPRB, 0.67163E+00_JPRB, 0.67604E+00_JPRB, 0.68064E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.70346E+00_JPRB, 0.68615E+00_JPRB, 0.67620E+00_JPRB, 0.67202E+00_JPRB, 0.66343E+00_JPRB, 0.64446E+00_JPRB, 0.62888E+00_JPRB ,&
&  0.61475E+00_JPRB, 0.60751E+00_JPRB, 0.60482E+00_JPRB, 0.60419E+00_JPRB, 0.60980E+00_JPRB, 0.61482E+00_JPRB, 0.62097E+00_JPRB ,&
&  0.63165E+00_JPRB, 0.64921E+00_JPRB, 0.66442E+00_JPRB, 0.67883E+00_JPRB, 0.69200E+00_JPRB, 0.70524E+00_JPRB, 0.70923E+00_JPRB ,&
&  0.71253E+00_JPRB, 0.72038E+00_JPRB, 0.72249E+00_JPRB, 0.71110E+00_JPRB, 0.69881E+00_JPRB, 0.70105E+00_JPRB, 0.69769E+00_JPRB ,&
&  0.69065E+00_JPRB, 0.57815E+00_JPRB, 0.62873E+00_JPRB, 0.65085E+00_JPRB, 0.67582E+00_JPRB, 0.69844E+00_JPRB, 0.71076E+00_JPRB ,&
&  0.70731E+00_JPRB, 0.69230E+00_JPRB, 0.67486E+00_JPRB, 0.64952E+00_JPRB, 0.64174E+00_JPRB, 0.63509E+00_JPRB, 0.62717E+00_JPRB ,&
&  0.62197E+00_JPRB, 0.61682E+00_JPRB, 0.60971E+00_JPRB, 0.59940E+00_JPRB, 0.58586E+00_JPRB, 0.57591E+00_JPRB, 0.56715E+00_JPRB ,&
&  0.56527E+00_JPRB, 0.55832E+00_JPRB, 0.55171E+00_JPRB, 0.55228E+00_JPRB, 0.56362E+00_JPRB, 0.56831E+00_JPRB, 0.56747E+00_JPRB ,&
&  0.56604E+00_JPRB, 0.56537E+00_JPRB, 0.57121E+00_JPRB, 0.58304E+00_JPRB, 0.59331E+00_JPRB, 0.60023E+00_JPRB, 0.60992E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.63792E+00_JPRB, 0.63866E+00_JPRB, 0.62967E+00_JPRB, 0.62483E+00_JPRB, 0.62611E+00_JPRB, 0.61816E+00_JPRB ,&
&  0.60492E+00_JPRB, 0.59817E+00_JPRB, 0.59862E+00_JPRB, 0.59989E+00_JPRB, 0.59780E+00_JPRB, 0.59266E+00_JPRB, 0.59213E+00_JPRB ,&
&  0.59360E+00_JPRB, 0.59437E+00_JPRB, 0.59361E+00_JPRB, 0.59613E+00_JPRB, 0.60191E+00_JPRB, 0.60978E+00_JPRB, 0.62079E+00_JPRB ,&
&  0.63684E+00_JPRB, 0.65181E+00_JPRB, 0.66079E+00_JPRB, 0.67413E+00_JPRB, 0.68956E+00_JPRB, 0.69593E+00_JPRB, 0.70933E+00_JPRB ,&
&  0.70974E+00_JPRB, 0.70607E+00_JPRB, 0.70480E+00_JPRB, 0.71083E+00_JPRB, 0.71393E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.73551E+00_JPRB, 0.71828E+00_JPRB, 0.70949E+00_JPRB, 0.70387E+00_JPRB, 0.69941E+00_JPRB, 0.69252E+00_JPRB, 0.67918E+00_JPRB ,&
&  0.66576E+00_JPRB, 0.65910E+00_JPRB, 0.65310E+00_JPRB, 0.64495E+00_JPRB, 0.64592E+00_JPRB, 0.65144E+00_JPRB, 0.66205E+00_JPRB ,&
&  0.67390E+00_JPRB, 0.68925E+00_JPRB, 0.70414E+00_JPRB, 0.72321E+00_JPRB, 0.73617E+00_JPRB, 0.74549E+00_JPRB, 0.75074E+00_JPRB ,&
&  0.75440E+00_JPRB, 0.75512E+00_JPRB, 0.74839E+00_JPRB, 0.73978E+00_JPRB, 0.73735E+00_JPRB, 0.73779E+00_JPRB, 0.73251E+00_JPRB ,&
&  0.63674E+00_JPRB, 0.55004E+00_JPRB, 0.60108E+00_JPRB, 0.65978E+00_JPRB, 0.71529E+00_JPRB, 0.76099E+00_JPRB, 0.78079E+00_JPRB ,&
&  0.77093E+00_JPRB, 0.74549E+00_JPRB, 0.72155E+00_JPRB, 0.69259E+00_JPRB, 0.68147E+00_JPRB, 0.67712E+00_JPRB, 0.66882E+00_JPRB ,&
&  0.65916E+00_JPRB, 0.64789E+00_JPRB, 0.63444E+00_JPRB, 0.61360E+00_JPRB, 0.59917E+00_JPRB, 0.58861E+00_JPRB, 0.58748E+00_JPRB ,&
&  0.59015E+00_JPRB, 0.59409E+00_JPRB, 0.59742E+00_JPRB, 0.59958E+00_JPRB, 0.60583E+00_JPRB, 0.61394E+00_JPRB, 0.62261E+00_JPRB ,&
&  0.63346E+00_JPRB, 0.64110E+00_JPRB, 0.65414E+00_JPRB, 0.66359E+00_JPRB, 0.67230E+00_JPRB, 0.67895E+00_JPRB, 0.68482E+00_JPRB ,&
&  0.68442E+00_JPRB, 0.68068E+00_JPRB, 0.66563E+00_JPRB, 0.65783E+00_JPRB, 0.65858E+00_JPRB, 0.66055E+00_JPRB, 0.65821E+00_JPRB ,&
&  0.64998E+00_JPRB, 0.64013E+00_JPRB, 0.63039E+00_JPRB, 0.62635E+00_JPRB, 0.62862E+00_JPRB, 0.62836E+00_JPRB, 0.62468E+00_JPRB ,&
&  0.62161E+00_JPRB, 0.62184E+00_JPRB, 0.62761E+00_JPRB, 0.63864E+00_JPRB, 0.65317E+00_JPRB, 0.67124E+00_JPRB, 0.68613E+00_JPRB ,&
&  0.70177E+00_JPRB, 0.71766E+00_JPRB, 0.72960E+00_JPRB, 0.75064E+00_JPRB, 0.76985E+00_JPRB, 0.77845E+00_JPRB, 0.77493E+00_JPRB ,&
&  0.77352E+00_JPRB, 0.76267E+00_JPRB, 0.75622E+00_JPRB, 0.75504E+00_JPRB, 0.74873E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.71727E+00_JPRB, 0.72520E+00_JPRB, 0.73149E+00_JPRB, 0.72881E+00_JPRB, 0.72525E+00_JPRB, 0.71702E+00_JPRB, 0.70092E+00_JPRB ,&
&  0.68603E+00_JPRB, 0.68030E+00_JPRB, 0.68018E+00_JPRB, 0.68150E+00_JPRB, 0.68430E+00_JPRB, 0.69057E+00_JPRB, 0.70200E+00_JPRB ,&
&  0.71571E+00_JPRB, 0.73026E+00_JPRB, 0.74748E+00_JPRB, 0.76362E+00_JPRB, 0.76763E+00_JPRB, 0.76491E+00_JPRB, 0.76561E+00_JPRB ,&
&  0.77221E+00_JPRB, 0.77156E+00_JPRB, 0.76589E+00_JPRB, 0.76892E+00_JPRB, 0.77669E+00_JPRB, 0.76759E+00_JPRB, 0.74930E+00_JPRB ,&
&  0.73647E+00_JPRB, 0.59054E+00_JPRB, 0.62420E+00_JPRB, 0.68549E+00_JPRB, 0.74771E+00_JPRB, 0.81006E+00_JPRB, 0.83585E+00_JPRB ,&
&  0.82912E+00_JPRB, 0.80026E+00_JPRB, 0.77535E+00_JPRB, 0.74438E+00_JPRB, 0.73117E+00_JPRB, 0.72035E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.68579E+00_JPRB, 0.66454E+00_JPRB, 0.64132E+00_JPRB, 0.62161E+00_JPRB, 0.60587E+00_JPRB, 0.59934E+00_JPRB, 0.59811E+00_JPRB ,&
&  0.59923E+00_JPRB, 0.61051E+00_JPRB, 0.61640E+00_JPRB, 0.62525E+00_JPRB, 0.63898E+00_JPRB, 0.65741E+00_JPRB, 0.67644E+00_JPRB ,&
&  0.69310E+00_JPRB, 0.70475E+00_JPRB, 0.71650E+00_JPRB, 0.72632E+00_JPRB, 0.73814E+00_JPRB, 0.74735E+00_JPRB, 0.74227E+00_JPRB ,&
&  0.72132E+00_JPRB, 0.69574E+00_JPRB, 0.69366E+00_JPRB, 0.69828E+00_JPRB, 0.69594E+00_JPRB, 0.69575E+00_JPRB, 0.69476E+00_JPRB ,&
&  0.68789E+00_JPRB, 0.67896E+00_JPRB, 0.67476E+00_JPRB, 0.66829E+00_JPRB, 0.66445E+00_JPRB, 0.66469E+00_JPRB, 0.65407E+00_JPRB ,&
&  0.64695E+00_JPRB, 0.64726E+00_JPRB, 0.65593E+00_JPRB, 0.67582E+00_JPRB, 0.70242E+00_JPRB, 0.72728E+00_JPRB, 0.74341E+00_JPRB ,&
&  0.75289E+00_JPRB, 0.76336E+00_JPRB, 0.77092E+00_JPRB, 0.79915E+00_JPRB, 0.80862E+00_JPRB, 0.81271E+00_JPRB, 0.81250E+00_JPRB ,&
&  0.80868E+00_JPRB, 0.80513E+00_JPRB, 0.61590E+00_JPRB, 0.68082E+00_JPRB, 0.69701E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.73626E+00_JPRB, 0.76264E+00_JPRB, 0.77326E+00_JPRB, 0.77473E+00_JPRB, 0.77154E+00_JPRB, 0.75807E+00_JPRB, 0.74237E+00_JPRB ,&
&  0.72400E+00_JPRB, 0.71625E+00_JPRB, 0.71337E+00_JPRB, 0.71792E+00_JPRB, 0.72437E+00_JPRB, 0.73389E+00_JPRB, 0.74704E+00_JPRB ,&
&  0.75948E+00_JPRB, 0.76618E+00_JPRB, 0.77173E+00_JPRB, 0.77072E+00_JPRB, 0.76958E+00_JPRB, 0.77243E+00_JPRB, 0.77718E+00_JPRB ,&
&  0.78671E+00_JPRB, 0.79049E+00_JPRB, 0.78837E+00_JPRB, 0.79709E+00_JPRB, 0.80940E+00_JPRB, 0.79900E+00_JPRB, 0.77907E+00_JPRB ,&
&  0.76250E+00_JPRB, 0.62906E+00_JPRB, 0.65141E+00_JPRB, 0.75089E+00_JPRB, 0.82391E+00_JPRB, 0.90546E+00_JPRB, 0.93114E+00_JPRB ,&
&  0.91468E+00_JPRB, 0.87847E+00_JPRB, 0.83910E+00_JPRB, 0.80372E+00_JPRB, 0.77946E+00_JPRB, 0.75199E+00_JPRB, 0.73356E+00_JPRB ,&
&  0.72091E+00_JPRB, 0.70750E+00_JPRB, 0.68714E+00_JPRB, 0.67098E+00_JPRB, 0.65787E+00_JPRB, 0.64506E+00_JPRB, 0.63858E+00_JPRB ,&
&  0.64148E+00_JPRB, 0.65342E+00_JPRB, 0.66600E+00_JPRB, 0.68047E+00_JPRB, 0.70719E+00_JPRB, 0.73216E+00_JPRB, 0.75566E+00_JPRB ,&
&  0.76650E+00_JPRB, 0.77601E+00_JPRB, 0.79076E+00_JPRB, 0.80475E+00_JPRB, 0.82270E+00_JPRB, 0.83099E+00_JPRB, 0.81404E+00_JPRB ,&
&  0.77364E+00_JPRB, 0.74825E+00_JPRB, 0.73247E+00_JPRB, 0.72274E+00_JPRB, 0.72383E+00_JPRB, 0.73280E+00_JPRB, 0.73711E+00_JPRB ,&
&  0.73338E+00_JPRB, 0.72711E+00_JPRB, 0.71888E+00_JPRB, 0.71074E+00_JPRB, 0.71153E+00_JPRB, 0.71467E+00_JPRB, 0.71199E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.71193E+00_JPRB, 0.72298E+00_JPRB, 0.74409E+00_JPRB, 0.76957E+00_JPRB, 0.79705E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.82412E+00_JPRB, 0.83569E+00_JPRB, 0.85358E+00_JPRB, 0.81532E+00_JPRB, 0.85164E+00_JPRB, 0.86171E+00_JPRB, 0.86261E+00_JPRB ,&
&  0.84634E+00_JPRB, 0.82032E+00_JPRB, 0.78906E+00_JPRB, 0.65259E+00_JPRB, 0.70556E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.78334E+00_JPRB, 0.81110E+00_JPRB, 0.81887E+00_JPRB, 0.81849E+00_JPRB, 0.80634E+00_JPRB, 0.78892E+00_JPRB, 0.77522E+00_JPRB ,&
&  0.76915E+00_JPRB, 0.75503E+00_JPRB, 0.74934E+00_JPRB, 0.74826E+00_JPRB, 0.75321E+00_JPRB, 0.76205E+00_JPRB, 0.76409E+00_JPRB ,&
&  0.75711E+00_JPRB, 0.74525E+00_JPRB, 0.73739E+00_JPRB, 0.73777E+00_JPRB, 0.74813E+00_JPRB, 0.75906E+00_JPRB, 0.76648E+00_JPRB ,&
&  0.76998E+00_JPRB, 0.76458E+00_JPRB, 0.76248E+00_JPRB, 0.77189E+00_JPRB, 0.77922E+00_JPRB, 0.77113E+00_JPRB, 0.75037E+00_JPRB ,&
&  0.73810E+00_JPRB, 0.67545E+00_JPRB, 0.76788E+00_JPRB, 0.86946E+00_JPRB, 0.94955E+00_JPRB, 0.98248E+00_JPRB, 0.98680E+00_JPRB ,&
&  0.95696E+00_JPRB, 0.91199E+00_JPRB, 0.85818E+00_JPRB, 0.82017E+00_JPRB, 0.79172E+00_JPRB, 0.76904E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.73112E+00_JPRB, 0.71001E+00_JPRB, 0.69133E+00_JPRB, 0.68102E+00_JPRB, 0.67739E+00_JPRB, 0.67451E+00_JPRB, 0.67229E+00_JPRB ,&
&  0.67034E+00_JPRB, 0.66831E+00_JPRB, 0.67173E+00_JPRB, 0.68590E+00_JPRB, 0.70741E+00_JPRB, 0.74472E+00_JPRB, 0.78707E+00_JPRB ,&
&  0.80475E+00_JPRB, 0.82771E+00_JPRB, 0.85796E+00_JPRB, 0.89154E+00_JPRB, 0.90065E+00_JPRB, 0.89497E+00_JPRB, 0.87452E+00_JPRB ,&
&  0.84697E+00_JPRB, 0.80592E+00_JPRB, 0.77320E+00_JPRB, 0.74998E+00_JPRB, 0.73985E+00_JPRB, 0.73348E+00_JPRB, 0.74104E+00_JPRB ,&
&  0.74783E+00_JPRB, 0.75507E+00_JPRB, 0.75133E+00_JPRB, 0.74616E+00_JPRB, 0.74649E+00_JPRB, 0.73600E+00_JPRB, 0.73477E+00_JPRB ,&
&  0.73874E+00_JPRB, 0.75447E+00_JPRB, 0.78031E+00_JPRB, 0.80390E+00_JPRB, 0.83298E+00_JPRB, 0.85227E+00_JPRB, 0.85843E+00_JPRB ,&
&  0.86032E+00_JPRB, 0.85988E+00_JPRB, 0.85773E+00_JPRB, 0.89725E+00_JPRB, 0.89386E+00_JPRB, 0.89243E+00_JPRB, 0.88261E+00_JPRB ,&
&  0.86410E+00_JPRB, 0.82791E+00_JPRB, 0.78475E+00_JPRB, 0.76913E+00_JPRB, 0.72336E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.90917E+00_JPRB, 0.90042E+00_JPRB, 0.87830E+00_JPRB, 0.86115E+00_JPRB, 0.84174E+00_JPRB, 0.82746E+00_JPRB, 0.81548E+00_JPRB ,&
&  0.80934E+00_JPRB, 0.79457E+00_JPRB, 0.78746E+00_JPRB, 0.78088E+00_JPRB, 0.77916E+00_JPRB, 0.77310E+00_JPRB, 0.76305E+00_JPRB ,&
&  0.75163E+00_JPRB, 0.74528E+00_JPRB, 0.74400E+00_JPRB, 0.74920E+00_JPRB, 0.75971E+00_JPRB, 0.76618E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.75278E+00_JPRB, 0.73972E+00_JPRB, 0.72368E+00_JPRB, 0.73691E+00_JPRB, 0.75206E+00_JPRB, 0.75590E+00_JPRB, 0.74009E+00_JPRB ,&
&  0.73851E+00_JPRB, 0.76241E+00_JPRB, 0.99990E+00_JPRB, 0.10957E+01_JPRB, 0.11042E+01_JPRB, 0.11330E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10094E+01_JPRB, 0.92373E+00_JPRB, 0.86071E+00_JPRB, 0.83337E+00_JPRB, 0.81688E+00_JPRB, 0.78601E+00_JPRB, 0.76009E+00_JPRB ,&
&  0.72275E+00_JPRB, 0.70543E+00_JPRB, 0.69389E+00_JPRB, 0.68940E+00_JPRB, 0.68718E+00_JPRB, 0.68867E+00_JPRB, 0.69458E+00_JPRB ,&
&  0.70128E+00_JPRB, 0.70378E+00_JPRB, 0.70843E+00_JPRB, 0.72917E+00_JPRB, 0.75220E+00_JPRB, 0.81457E+00_JPRB, 0.87413E+00_JPRB ,&
&  0.90622E+00_JPRB, 0.93325E+00_JPRB, 0.96898E+00_JPRB, 0.98631E+00_JPRB, 0.96780E+00_JPRB, 0.93369E+00_JPRB, 0.90632E+00_JPRB ,&
&  0.87947E+00_JPRB, 0.84945E+00_JPRB, 0.81747E+00_JPRB, 0.79639E+00_JPRB, 0.77652E+00_JPRB, 0.76855E+00_JPRB, 0.78066E+00_JPRB ,&
&  0.78712E+00_JPRB, 0.79664E+00_JPRB, 0.79266E+00_JPRB, 0.78358E+00_JPRB, 0.76323E+00_JPRB, 0.74797E+00_JPRB, 0.74905E+00_JPRB ,&
&  0.76210E+00_JPRB, 0.78827E+00_JPRB, 0.82291E+00_JPRB, 0.85819E+00_JPRB, 0.88569E+00_JPRB, 0.89836E+00_JPRB, 0.90015E+00_JPRB ,&
&  0.90652E+00_JPRB, 0.90877E+00_JPRB, 0.85760E+00_JPRB, 0.87280E+00_JPRB, 0.96334E+00_JPRB, 0.95358E+00_JPRB, 0.93653E+00_JPRB ,&
&  0.90155E+00_JPRB, 0.84569E+00_JPRB, 0.80607E+00_JPRB, 0.79196E+00_JPRB, 0.86665E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.86958E+00_JPRB, 0.85929E+00_JPRB, 0.84262E+00_JPRB, 0.83316E+00_JPRB, 0.83506E+00_JPRB, 0.82673E+00_JPRB, 0.82080E+00_JPRB ,&
&  0.81846E+00_JPRB, 0.80912E+00_JPRB, 0.80085E+00_JPRB, 0.78744E+00_JPRB, 0.77145E+00_JPRB, 0.75970E+00_JPRB, 0.75768E+00_JPRB ,&
&  0.76018E+00_JPRB, 0.75236E+00_JPRB, 0.75771E+00_JPRB, 0.76069E+00_JPRB, 0.75807E+00_JPRB, 0.74502E+00_JPRB, 0.73210E+00_JPRB ,&
&  0.72589E+00_JPRB, 0.71396E+00_JPRB, 0.71139E+00_JPRB, 0.73120E+00_JPRB, 0.74925E+00_JPRB, 0.75068E+00_JPRB, 0.73262E+00_JPRB ,&
&  0.75256E+00_JPRB, 0.91468E+00_JPRB, 0.13131E+01_JPRB, 0.14305E+01_JPRB, 0.13406E+01_JPRB, 0.12251E+01_JPRB, 0.11022E+01_JPRB ,&
&  0.10015E+01_JPRB, 0.91393E+00_JPRB, 0.87374E+00_JPRB, 0.84072E+00_JPRB, 0.80453E+00_JPRB, 0.77488E+00_JPRB, 0.74896E+00_JPRB ,&
&  0.71726E+00_JPRB, 0.70541E+00_JPRB, 0.69997E+00_JPRB, 0.70098E+00_JPRB, 0.70244E+00_JPRB, 0.70235E+00_JPRB, 0.70489E+00_JPRB ,&
&  0.70846E+00_JPRB, 0.71246E+00_JPRB, 0.72483E+00_JPRB, 0.73854E+00_JPRB, 0.78292E+00_JPRB, 0.89184E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.10576E+01_JPRB, 0.10327E+01_JPRB, 0.10067E+01_JPRB, 0.98238E+00_JPRB, 0.95051E+00_JPRB, 0.92419E+00_JPRB ,&
&  0.89231E+00_JPRB, 0.86848E+00_JPRB, 0.85088E+00_JPRB, 0.83884E+00_JPRB, 0.81947E+00_JPRB, 0.82109E+00_JPRB, 0.83368E+00_JPRB ,&
&  0.84903E+00_JPRB, 0.85484E+00_JPRB, 0.83638E+00_JPRB, 0.81738E+00_JPRB, 0.79228E+00_JPRB, 0.78800E+00_JPRB, 0.79033E+00_JPRB ,&
&  0.79566E+00_JPRB, 0.81602E+00_JPRB, 0.85437E+00_JPRB, 0.89203E+00_JPRB, 0.91675E+00_JPRB, 0.92587E+00_JPRB, 0.92221E+00_JPRB ,&
&  0.92845E+00_JPRB, 0.92461E+00_JPRB, 0.90394E+00_JPRB, 0.91714E+00_JPRB, 0.91737E+00_JPRB, 0.97562E+00_JPRB, 0.93306E+00_JPRB ,&
&  0.87094E+00_JPRB, 0.82521E+00_JPRB, 0.80287E+00_JPRB, 0.79709E+00_JPRB, 0.85524E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.85283E+00_JPRB, 0.83561E+00_JPRB, 0.82521E+00_JPRB, 0.81591E+00_JPRB, 0.80444E+00_JPRB, 0.79953E+00_JPRB, 0.79297E+00_JPRB ,&
&  0.79788E+00_JPRB, 0.80077E+00_JPRB, 0.79210E+00_JPRB, 0.76712E+00_JPRB, 0.74751E+00_JPRB, 0.74554E+00_JPRB, 0.75264E+00_JPRB ,&
&  0.74828E+00_JPRB, 0.73850E+00_JPRB, 0.74279E+00_JPRB, 0.72673E+00_JPRB, 0.70502E+00_JPRB, 0.69168E+00_JPRB, 0.68636E+00_JPRB ,&
&  0.69086E+00_JPRB, 0.70187E+00_JPRB, 0.72866E+00_JPRB, 0.75376E+00_JPRB, 0.75694E+00_JPRB, 0.74808E+00_JPRB, 0.72829E+00_JPRB ,&
&  0.80833E+00_JPRB, 0.10400E+01_JPRB, 0.17123E+01_JPRB, 0.16782E+01_JPRB, 0.14627E+01_JPRB, 0.12395E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10019E+01_JPRB, 0.92887E+00_JPRB, 0.87608E+00_JPRB, 0.81351E+00_JPRB, 0.76681E+00_JPRB, 0.74228E+00_JPRB, 0.73114E+00_JPRB ,&
&  0.71322E+00_JPRB, 0.70213E+00_JPRB, 0.70638E+00_JPRB, 0.71505E+00_JPRB, 0.72097E+00_JPRB, 0.72126E+00_JPRB, 0.70924E+00_JPRB ,&
&  0.70773E+00_JPRB, 0.71051E+00_JPRB, 0.72056E+00_JPRB, 0.73007E+00_JPRB, 0.77291E+00_JPRB, 0.79052E+00_JPRB, 0.97484E+00_JPRB ,&
&  0.12529E+01_JPRB, 0.11853E+01_JPRB, 0.10933E+01_JPRB, 0.10469E+01_JPRB, 0.10201E+01_JPRB, 0.97987E+00_JPRB, 0.95187E+00_JPRB ,&
&  0.92044E+00_JPRB, 0.90104E+00_JPRB, 0.90700E+00_JPRB, 0.90520E+00_JPRB, 0.90839E+00_JPRB, 0.88834E+00_JPRB, 0.87422E+00_JPRB ,&
&  0.87883E+00_JPRB, 0.85585E+00_JPRB, 0.83059E+00_JPRB, 0.80677E+00_JPRB, 0.81391E+00_JPRB, 0.83078E+00_JPRB, 0.83808E+00_JPRB ,&
&  0.84528E+00_JPRB, 0.85074E+00_JPRB, 0.87191E+00_JPRB, 0.88234E+00_JPRB, 0.92368E+00_JPRB, 0.95882E+00_JPRB, 0.95350E+00_JPRB ,&
&  0.90643E+00_JPRB, 0.89905E+00_JPRB, 0.90628E+00_JPRB, 0.88308E+00_JPRB, 0.88310E+00_JPRB, 0.95183E+00_JPRB, 0.88348E+00_JPRB ,&
&  0.82553E+00_JPRB, 0.79273E+00_JPRB, 0.77666E+00_JPRB, 0.79325E+00_JPRB, 0.85555E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.86828E+00_JPRB, 0.85069E+00_JPRB, 0.83618E+00_JPRB, 0.81078E+00_JPRB, 0.80535E+00_JPRB, 0.79599E+00_JPRB, 0.79226E+00_JPRB ,&
&  0.79238E+00_JPRB, 0.78102E+00_JPRB, 0.76759E+00_JPRB, 0.74834E+00_JPRB, 0.74084E+00_JPRB, 0.75139E+00_JPRB, 0.74597E+00_JPRB ,&
&  0.72852E+00_JPRB, 0.70550E+00_JPRB, 0.69868E+00_JPRB, 0.69021E+00_JPRB, 0.68161E+00_JPRB, 0.68354E+00_JPRB, 0.69327E+00_JPRB ,&
&  0.69705E+00_JPRB, 0.71476E+00_JPRB, 0.74403E+00_JPRB, 0.77025E+00_JPRB, 0.76730E+00_JPRB, 0.75770E+00_JPRB, 0.77451E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.85658E+00_JPRB, 0.16805E+01_JPRB, 0.18855E+01_JPRB, 0.15626E+01_JPRB, 0.13082E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.99391E+00_JPRB, 0.90756E+00_JPRB, 0.85608E+00_JPRB, 0.79961E+00_JPRB, 0.76848E+00_JPRB, 0.72928E+00_JPRB, 0.71746E+00_JPRB ,&
&  0.72046E+00_JPRB, 0.71236E+00_JPRB, 0.71338E+00_JPRB, 0.73085E+00_JPRB, 0.73844E+00_JPRB, 0.74554E+00_JPRB, 0.73081E+00_JPRB ,&
&  0.72160E+00_JPRB, 0.71630E+00_JPRB, 0.73321E+00_JPRB, 0.76968E+00_JPRB, 0.82298E+00_JPRB, 0.89992E+00_JPRB, 0.99813E+00_JPRB ,&
&  0.10646E+01_JPRB, 0.12278E+01_JPRB, 0.11094E+01_JPRB, 0.10560E+01_JPRB, 0.10154E+01_JPRB, 0.97023E+00_JPRB, 0.95778E+00_JPRB ,&
&  0.92988E+00_JPRB, 0.92142E+00_JPRB, 0.92260E+00_JPRB, 0.95692E+00_JPRB, 0.93680E+00_JPRB, 0.93711E+00_JPRB, 0.91895E+00_JPRB ,&
&  0.88901E+00_JPRB, 0.85688E+00_JPRB, 0.82850E+00_JPRB, 0.81754E+00_JPRB, 0.83977E+00_JPRB, 0.85292E+00_JPRB, 0.84952E+00_JPRB ,&
&  0.84910E+00_JPRB, 0.87241E+00_JPRB, 0.89211E+00_JPRB, 0.89891E+00_JPRB, 0.92466E+00_JPRB, 0.93107E+00_JPRB, 0.93175E+00_JPRB ,&
&  0.92228E+00_JPRB, 0.92305E+00_JPRB, 0.87946E+00_JPRB, 0.87148E+00_JPRB, 0.83600E+00_JPRB, 0.87403E+00_JPRB, 0.82263E+00_JPRB ,&
&  0.78774E+00_JPRB, 0.76432E+00_JPRB, 0.78855E+00_JPRB, 0.84488E+00_JPRB, 0.87628E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.86135E+00_JPRB, 0.86247E+00_JPRB, 0.82514E+00_JPRB, 0.82124E+00_JPRB, 0.80850E+00_JPRB, 0.81212E+00_JPRB, 0.82745E+00_JPRB ,&
&  0.82095E+00_JPRB, 0.79659E+00_JPRB, 0.76528E+00_JPRB, 0.75229E+00_JPRB, 0.76251E+00_JPRB, 0.77347E+00_JPRB, 0.74068E+00_JPRB ,&
&  0.70285E+00_JPRB, 0.69144E+00_JPRB, 0.68632E+00_JPRB, 0.69032E+00_JPRB, 0.70118E+00_JPRB, 0.71870E+00_JPRB, 0.72750E+00_JPRB ,&
&  0.72242E+00_JPRB, 0.75925E+00_JPRB, 0.79940E+00_JPRB, 0.81236E+00_JPRB, 0.79484E+00_JPRB, 0.78329E+00_JPRB, 0.79220E+00_JPRB ,&
&  0.86602E+00_JPRB, 0.19245E+01_JPRB, 0.13227E+01_JPRB, 0.19936E+01_JPRB, 0.15560E+01_JPRB, 0.12539E+01_JPRB, 0.97057E+00_JPRB ,&
&  0.88760E+00_JPRB, 0.90202E+00_JPRB, 0.87284E+00_JPRB, 0.81074E+00_JPRB, 0.76926E+00_JPRB, 0.73273E+00_JPRB, 0.70134E+00_JPRB ,&
&  0.69335E+00_JPRB, 0.70402E+00_JPRB, 0.71143E+00_JPRB, 0.74863E+00_JPRB, 0.77868E+00_JPRB, 0.78363E+00_JPRB, 0.76626E+00_JPRB ,&
&  0.75179E+00_JPRB, 0.74610E+00_JPRB, 0.77677E+00_JPRB, 0.89172E+00_JPRB, 0.97144E+00_JPRB, 0.10130E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.12107E+01_JPRB, 0.10991E+01_JPRB, 0.10399E+01_JPRB, 0.99371E+00_JPRB, 0.95104E+00_JPRB, 0.95097E+00_JPRB ,&
&  0.92864E+00_JPRB, 0.91915E+00_JPRB, 0.92009E+00_JPRB, 0.91674E+00_JPRB, 0.93406E+00_JPRB, 0.95113E+00_JPRB, 0.92142E+00_JPRB ,&
&  0.88107E+00_JPRB, 0.84752E+00_JPRB, 0.84927E+00_JPRB, 0.86183E+00_JPRB, 0.87943E+00_JPRB, 0.86784E+00_JPRB, 0.85881E+00_JPRB ,&
&  0.86902E+00_JPRB, 0.91138E+00_JPRB, 0.91725E+00_JPRB, 0.92640E+00_JPRB, 0.91929E+00_JPRB, 0.89850E+00_JPRB, 0.87995E+00_JPRB ,&
&  0.91417E+00_JPRB, 0.90523E+00_JPRB, 0.84736E+00_JPRB, 0.83122E+00_JPRB, 0.81217E+00_JPRB, 0.83135E+00_JPRB, 0.78122E+00_JPRB ,&
&  0.76813E+00_JPRB, 0.77711E+00_JPRB, 0.85155E+00_JPRB, 0.86003E+00_JPRB, 0.87018E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.85703E+00_JPRB, 0.84656E+00_JPRB, 0.81636E+00_JPRB, 0.81815E+00_JPRB, 0.80061E+00_JPRB, 0.83096E+00_JPRB, 0.83615E+00_JPRB ,&
&  0.80188E+00_JPRB, 0.76974E+00_JPRB, 0.75187E+00_JPRB, 0.77102E+00_JPRB, 0.79082E+00_JPRB, 0.77623E+00_JPRB, 0.73572E+00_JPRB ,&
&  0.72293E+00_JPRB, 0.72183E+00_JPRB, 0.72291E+00_JPRB, 0.72674E+00_JPRB, 0.74235E+00_JPRB, 0.76462E+00_JPRB, 0.76151E+00_JPRB ,&
&  0.75793E+00_JPRB, 0.78746E+00_JPRB, 0.81323E+00_JPRB, 0.81290E+00_JPRB, 0.80208E+00_JPRB, 0.78928E+00_JPRB, 0.81313E+00_JPRB ,&
&  0.84616E+00_JPRB, 0.16373E+01_JPRB, 0.12593E+01_JPRB, 0.19630E+01_JPRB, 0.15186E+01_JPRB, 0.12320E+01_JPRB, 0.92611E+00_JPRB ,&
&  0.81537E+00_JPRB, 0.89092E+00_JPRB, 0.85648E+00_JPRB, 0.78485E+00_JPRB, 0.73237E+00_JPRB, 0.70613E+00_JPRB, 0.68546E+00_JPRB ,&
&  0.67831E+00_JPRB, 0.71126E+00_JPRB, 0.74499E+00_JPRB, 0.79208E+00_JPRB, 0.83008E+00_JPRB, 0.83521E+00_JPRB, 0.83200E+00_JPRB ,&
&  0.82827E+00_JPRB, 0.83167E+00_JPRB, 0.92045E+00_JPRB, 0.10272E+01_JPRB, 0.11578E+01_JPRB, 0.11754E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.11622E+01_JPRB, 0.10972E+01_JPRB, 0.10406E+01_JPRB, 0.98247E+00_JPRB, 0.96059E+00_JPRB, 0.94620E+00_JPRB ,&
&  0.90766E+00_JPRB, 0.86303E+00_JPRB, 0.85820E+00_JPRB, 0.87282E+00_JPRB, 0.93122E+00_JPRB, 0.94285E+00_JPRB, 0.90047E+00_JPRB ,&
&  0.85866E+00_JPRB, 0.84279E+00_JPRB, 0.84398E+00_JPRB, 0.86010E+00_JPRB, 0.87599E+00_JPRB, 0.87242E+00_JPRB, 0.87097E+00_JPRB ,&
&  0.90152E+00_JPRB, 0.95218E+00_JPRB, 0.94248E+00_JPRB, 0.93779E+00_JPRB, 0.90186E+00_JPRB, 0.87256E+00_JPRB, 0.85135E+00_JPRB ,&
&  0.87205E+00_JPRB, 0.85853E+00_JPRB, 0.82656E+00_JPRB, 0.79240E+00_JPRB, 0.78375E+00_JPRB, 0.79138E+00_JPRB, 0.77340E+00_JPRB ,&
&  0.78517E+00_JPRB, 0.83809E+00_JPRB, 0.89494E+00_JPRB, 0.85938E+00_JPRB, 0.85397E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.82551E+00_JPRB, 0.79550E+00_JPRB, 0.78395E+00_JPRB, 0.77202E+00_JPRB, 0.76000E+00_JPRB, 0.79160E+00_JPRB, 0.78175E+00_JPRB ,&
&  0.75760E+00_JPRB, 0.73308E+00_JPRB, 0.75216E+00_JPRB, 0.79565E+00_JPRB, 0.79549E+00_JPRB, 0.77358E+00_JPRB, 0.75834E+00_JPRB ,&
&  0.75694E+00_JPRB, 0.77447E+00_JPRB, 0.77825E+00_JPRB, 0.78919E+00_JPRB, 0.78878E+00_JPRB, 0.77797E+00_JPRB, 0.77055E+00_JPRB ,&
&  0.78633E+00_JPRB, 0.79519E+00_JPRB, 0.80352E+00_JPRB, 0.80675E+00_JPRB, 0.81939E+00_JPRB, 0.81626E+00_JPRB, 0.87867E+00_JPRB ,&
&  0.97897E+00_JPRB, 0.15425E+01_JPRB, 0.13052E+01_JPRB, 0.15324E+01_JPRB, 0.13712E+01_JPRB, 0.11299E+01_JPRB, 0.84017E+00_JPRB ,&
&  0.72374E+00_JPRB, 0.67831E+00_JPRB, 0.81424E+00_JPRB, 0.77650E+00_JPRB, 0.75083E+00_JPRB, 0.76640E+00_JPRB, 0.77274E+00_JPRB ,&
&  0.79434E+00_JPRB, 0.80801E+00_JPRB, 0.88463E+00_JPRB, 0.92269E+00_JPRB, 0.94186E+00_JPRB, 0.94627E+00_JPRB, 0.94032E+00_JPRB ,&
&  0.96687E+00_JPRB, 0.10149E+01_JPRB, 0.11826E+01_JPRB, 0.12963E+01_JPRB, 0.12705E+01_JPRB, 0.12284E+01_JPRB, 0.11314E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.11053E+01_JPRB, 0.11004E+01_JPRB, 0.10591E+01_JPRB, 0.95503E+00_JPRB, 0.98278E+00_JPRB, 0.91970E+00_JPRB ,&
&  0.85432E+00_JPRB, 0.81387E+00_JPRB, 0.81691E+00_JPRB, 0.84795E+00_JPRB, 0.89117E+00_JPRB, 0.89017E+00_JPRB, 0.86618E+00_JPRB ,&
&  0.85282E+00_JPRB, 0.85886E+00_JPRB, 0.84553E+00_JPRB, 0.85905E+00_JPRB, 0.86995E+00_JPRB, 0.89801E+00_JPRB, 0.92440E+00_JPRB ,&
&  0.96291E+00_JPRB, 0.99391E+00_JPRB, 0.96248E+00_JPRB, 0.92733E+00_JPRB, 0.87862E+00_JPRB, 0.84561E+00_JPRB, 0.81022E+00_JPRB ,&
&  0.81551E+00_JPRB, 0.83730E+00_JPRB, 0.83460E+00_JPRB, 0.80470E+00_JPRB, 0.78730E+00_JPRB, 0.78505E+00_JPRB, 0.80919E+00_JPRB ,&
&  0.84162E+00_JPRB, 0.90221E+00_JPRB, 0.91059E+00_JPRB, 0.85880E+00_JPRB, 0.84694E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.76019E+00_JPRB, 0.75225E+00_JPRB, 0.73013E+00_JPRB, 0.71575E+00_JPRB, 0.73859E+00_JPRB, 0.77213E+00_JPRB, 0.77137E+00_JPRB ,&
&  0.74867E+00_JPRB, 0.73696E+00_JPRB, 0.77208E+00_JPRB, 0.80600E+00_JPRB, 0.80232E+00_JPRB, 0.81697E+00_JPRB, 0.82461E+00_JPRB ,&
&  0.85410E+00_JPRB, 0.84633E+00_JPRB, 0.82602E+00_JPRB, 0.79792E+00_JPRB, 0.79484E+00_JPRB, 0.78117E+00_JPRB, 0.77929E+00_JPRB ,&
&  0.78608E+00_JPRB, 0.79279E+00_JPRB, 0.80663E+00_JPRB, 0.83120E+00_JPRB, 0.85799E+00_JPRB, 0.93001E+00_JPRB, 0.10834E+01_JPRB ,&
&  0.13162E+01_JPRB, 0.17964E+01_JPRB, 0.68595E+00_JPRB, 0.86770E+00_JPRB, 0.98260E+00_JPRB, 0.85667E+00_JPRB, 0.71369E+00_JPRB ,&
&  0.61839E+00_JPRB, 0.56127E+00_JPRB, 0.66088E+00_JPRB, 0.80709E+00_JPRB, 0.83588E+00_JPRB, 0.87652E+00_JPRB, 0.89966E+00_JPRB ,&
&  0.93512E+00_JPRB, 0.97425E+00_JPRB, 0.10363E+01_JPRB, 0.10650E+01_JPRB, 0.10793E+01_JPRB, 0.10822E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.12967E+01_JPRB, 0.14961E+01_JPRB, 0.14080E+01_JPRB, 0.13781E+01_JPRB, 0.12391E+01_JPRB, 0.10982E+01_JPRB ,&
&  0.98995E+00_JPRB, 0.10522E+01_JPRB, 0.11116E+01_JPRB, 0.10602E+01_JPRB, 0.94672E+00_JPRB, 0.88533E+00_JPRB, 0.94591E+00_JPRB ,&
&  0.86262E+00_JPRB, 0.81075E+00_JPRB, 0.79169E+00_JPRB, 0.80040E+00_JPRB, 0.85569E+00_JPRB, 0.85234E+00_JPRB, 0.83497E+00_JPRB ,&
&  0.84388E+00_JPRB, 0.83587E+00_JPRB, 0.85043E+00_JPRB, 0.87802E+00_JPRB, 0.90697E+00_JPRB, 0.96008E+00_JPRB, 0.98580E+00_JPRB ,&
&  0.98493E+00_JPRB, 0.99626E+00_JPRB, 0.99181E+00_JPRB, 0.94947E+00_JPRB, 0.88706E+00_JPRB, 0.84491E+00_JPRB, 0.83106E+00_JPRB ,&
&  0.84197E+00_JPRB, 0.85121E+00_JPRB, 0.81163E+00_JPRB, 0.77337E+00_JPRB, 0.80661E+00_JPRB, 0.83005E+00_JPRB, 0.93760E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.11286E+01_JPRB, 0.10905E+01_JPRB, 0.82144E+00_JPRB, 0.81146E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.71728E+00_JPRB, 0.67620E+00_JPRB, 0.64493E+00_JPRB, 0.65636E+00_JPRB, 0.70660E+00_JPRB, 0.75797E+00_JPRB, 0.75705E+00_JPRB ,&
&  0.74272E+00_JPRB, 0.76147E+00_JPRB, 0.83311E+00_JPRB, 0.88804E+00_JPRB, 0.89510E+00_JPRB, 0.90463E+00_JPRB, 0.91972E+00_JPRB ,&
&  0.86403E+00_JPRB, 0.82970E+00_JPRB, 0.81191E+00_JPRB, 0.79232E+00_JPRB, 0.79084E+00_JPRB, 0.78326E+00_JPRB, 0.78132E+00_JPRB ,&
&  0.78889E+00_JPRB, 0.79220E+00_JPRB, 0.81882E+00_JPRB, 0.86198E+00_JPRB, 0.94516E+00_JPRB, 0.11394E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.19711E+01_JPRB, 0.12939E+01_JPRB, 0.53478E+00_JPRB, 0.71624E+00_JPRB, 0.75045E+00_JPRB, 0.71825E+00_JPRB, 0.63318E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.61778E+00_JPRB, 0.72786E+00_JPRB, 0.87870E+00_JPRB, 0.93412E+00_JPRB, 0.98836E+00_JPRB, 0.10267E+01_JPRB ,&
&  0.10563E+01_JPRB, 0.11023E+01_JPRB, 0.11410E+01_JPRB, 0.11755E+01_JPRB, 0.12082E+01_JPRB, 0.12463E+01_JPRB, 0.13120E+01_JPRB ,&
&  0.14391E+01_JPRB, 0.15831E+01_JPRB, 0.17324E+01_JPRB, 0.15515E+01_JPRB, 0.15265E+01_JPRB, 0.12591E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.99686E+00_JPRB, 0.10134E+01_JPRB, 0.10757E+01_JPRB, 0.10106E+01_JPRB, 0.90213E+00_JPRB, 0.82821E+00_JPRB, 0.93675E+00_JPRB ,&
&  0.84443E+00_JPRB, 0.78881E+00_JPRB, 0.75100E+00_JPRB, 0.76079E+00_JPRB, 0.79455E+00_JPRB, 0.80519E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.81589E+00_JPRB, 0.81765E+00_JPRB, 0.85270E+00_JPRB, 0.87467E+00_JPRB, 0.89311E+00_JPRB, 0.93181E+00_JPRB, 0.93518E+00_JPRB ,&
&  0.92846E+00_JPRB, 0.92294E+00_JPRB, 0.93159E+00_JPRB, 0.94855E+00_JPRB, 0.90000E+00_JPRB, 0.82154E+00_JPRB, 0.80867E+00_JPRB ,&
&  0.80385E+00_JPRB, 0.80057E+00_JPRB, 0.77229E+00_JPRB, 0.78674E+00_JPRB, 0.80053E+00_JPRB, 0.87284E+00_JPRB, 0.95304E+00_JPRB ,&
&  0.98862E+00_JPRB, 0.96499E+00_JPRB, 0.90073E+00_JPRB, 0.77696E+00_JPRB, 0.76304E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.66610E+00_JPRB, 0.64009E+00_JPRB, 0.60056E+00_JPRB, 0.59913E+00_JPRB, 0.65827E+00_JPRB, 0.70537E+00_JPRB, 0.70717E+00_JPRB ,&
&  0.71340E+00_JPRB, 0.77621E+00_JPRB, 0.88798E+00_JPRB, 0.97584E+00_JPRB, 0.10022E+01_JPRB, 0.99175E+00_JPRB, 0.89168E+00_JPRB ,&
&  0.85098E+00_JPRB, 0.82461E+00_JPRB, 0.79613E+00_JPRB, 0.78091E+00_JPRB, 0.77190E+00_JPRB, 0.76941E+00_JPRB, 0.77669E+00_JPRB ,&
&  0.78918E+00_JPRB, 0.81769E+00_JPRB, 0.87771E+00_JPRB, 0.96026E+00_JPRB, 0.10891E+01_JPRB, 0.12355E+01_JPRB, 0.13604E+01_JPRB ,&
&  0.99660E+00_JPRB, 0.44671E+00_JPRB, 0.49564E+00_JPRB, 0.70951E+00_JPRB, 0.66457E+00_JPRB, 0.64880E+00_JPRB, 0.61450E+00_JPRB ,&
&  0.62635E+00_JPRB, 0.69672E+00_JPRB, 0.77094E+00_JPRB, 0.91885E+00_JPRB, 0.10021E+01_JPRB, 0.10805E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.11612E+01_JPRB, 0.12165E+01_JPRB, 0.12801E+01_JPRB, 0.13390E+01_JPRB, 0.13989E+01_JPRB, 0.14627E+01_JPRB, 0.15730E+01_JPRB ,&
&  0.17047E+01_JPRB, 0.18335E+01_JPRB, 0.19500E+01_JPRB, 0.17159E+01_JPRB, 0.17562E+01_JPRB, 0.14777E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.89438E+00_JPRB, 0.88507E+00_JPRB, 0.98526E+00_JPRB, 0.92096E+00_JPRB, 0.77971E+00_JPRB, 0.83840E+00_JPRB ,&
&  0.78655E+00_JPRB, 0.76652E+00_JPRB, 0.75033E+00_JPRB, 0.69276E+00_JPRB, 0.75109E+00_JPRB, 0.77668E+00_JPRB, 0.79437E+00_JPRB ,&
&  0.77910E+00_JPRB, 0.79553E+00_JPRB, 0.81618E+00_JPRB, 0.85179E+00_JPRB, 0.84159E+00_JPRB, 0.84020E+00_JPRB, 0.82522E+00_JPRB ,&
&  0.82678E+00_JPRB, 0.83154E+00_JPRB, 0.84622E+00_JPRB, 0.86980E+00_JPRB, 0.87641E+00_JPRB, 0.80689E+00_JPRB, 0.75586E+00_JPRB ,&
&  0.73377E+00_JPRB, 0.72700E+00_JPRB, 0.70466E+00_JPRB, 0.75343E+00_JPRB, 0.77577E+00_JPRB, 0.79884E+00_JPRB, 0.83197E+00_JPRB ,&
&  0.84750E+00_JPRB, 0.84864E+00_JPRB, 0.80097E+00_JPRB, 0.74590E+00_JPRB, 0.70933E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.56170E+00_JPRB, 0.54906E+00_JPRB, 0.53401E+00_JPRB, 0.55170E+00_JPRB, 0.58557E+00_JPRB, 0.62657E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.66599E+00_JPRB, 0.76279E+00_JPRB, 0.88972E+00_JPRB, 0.10006E+01_JPRB, 0.10151E+01_JPRB, 0.97882E+00_JPRB, 0.87333E+00_JPRB ,&
&  0.83921E+00_JPRB, 0.80710E+00_JPRB, 0.78533E+00_JPRB, 0.76011E+00_JPRB, 0.75155E+00_JPRB, 0.76406E+00_JPRB, 0.79181E+00_JPRB ,&
&  0.82856E+00_JPRB, 0.86788E+00_JPRB, 0.91460E+00_JPRB, 0.96209E+00_JPRB, 0.99492E+00_JPRB, 0.97683E+00_JPRB, 0.85294E+00_JPRB ,&
&  0.36287E+00_JPRB, 0.45070E+00_JPRB, 0.66730E+00_JPRB, 0.65749E+00_JPRB, 0.61609E+00_JPRB, 0.59308E+00_JPRB, 0.61543E+00_JPRB ,&
&  0.64509E+00_JPRB, 0.65671E+00_JPRB, 0.72932E+00_JPRB, 0.90642E+00_JPRB, 0.10222E+01_JPRB, 0.11295E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.13043E+01_JPRB, 0.13922E+01_JPRB, 0.14668E+01_JPRB, 0.15327E+01_JPRB, 0.15951E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.19801E+01_JPRB, 0.21105E+01_JPRB, 0.19121E+01_JPRB, 0.19973E+01_JPRB, 0.18053E+01_JPRB, 0.16274E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.84745E+00_JPRB, 0.86331E+00_JPRB, 0.96919E+00_JPRB, 0.92111E+00_JPRB, 0.87450E+00_JPRB, 0.83597E+00_JPRB ,&
&  0.78548E+00_JPRB, 0.71637E+00_JPRB, 0.72124E+00_JPRB, 0.72697E+00_JPRB, 0.72552E+00_JPRB, 0.76626E+00_JPRB, 0.74156E+00_JPRB ,&
&  0.75859E+00_JPRB, 0.76948E+00_JPRB, 0.76299E+00_JPRB, 0.76686E+00_JPRB, 0.76384E+00_JPRB, 0.76272E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.73708E+00_JPRB, 0.74642E+00_JPRB, 0.76450E+00_JPRB, 0.76940E+00_JPRB, 0.76219E+00_JPRB, 0.72187E+00_JPRB ,&
&  0.68428E+00_JPRB, 0.69930E+00_JPRB, 0.80569E+00_JPRB, 0.94443E+00_JPRB, 0.92590E+00_JPRB, 0.79852E+00_JPRB, 0.62821E+00_JPRB ,&
&  0.61344E+00_JPRB, 0.63437E+00_JPRB, 0.63690E+00_JPRB, 0.61665E+00_JPRB, 0.59092E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.45788E+00_JPRB, 0.44664E+00_JPRB, 0.44617E+00_JPRB, 0.45675E+00_JPRB, 0.46334E+00_JPRB, 0.48709E+00_JPRB, 0.51710E+00_JPRB ,&
&  0.53169E+00_JPRB, 0.59899E+00_JPRB, 0.74845E+00_JPRB, 0.87477E+00_JPRB, 0.92253E+00_JPRB, 0.95630E+00_JPRB, 0.89946E+00_JPRB ,&
&  0.87646E+00_JPRB, 0.83554E+00_JPRB, 0.77600E+00_JPRB, 0.75675E+00_JPRB, 0.75959E+00_JPRB, 0.77519E+00_JPRB, 0.79765E+00_JPRB ,&
&  0.82435E+00_JPRB, 0.85279E+00_JPRB, 0.87353E+00_JPRB, 0.89183E+00_JPRB, 0.91451E+00_JPRB, 0.87162E+00_JPRB, 0.78013E+00_JPRB ,&
&  0.42260E+00_JPRB, 0.71148E+00_JPRB, 0.69632E+00_JPRB, 0.64052E+00_JPRB, 0.58234E+00_JPRB, 0.55440E+00_JPRB, 0.59309E+00_JPRB ,&
&  0.63239E+00_JPRB, 0.62521E+00_JPRB, 0.67844E+00_JPRB, 0.80645E+00_JPRB, 0.10052E+01_JPRB, 0.11112E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.12822E+01_JPRB, 0.13410E+01_JPRB, 0.14085E+01_JPRB, 0.14735E+01_JPRB, 0.15323E+01_JPRB, 0.15767E+01_JPRB, 0.16407E+01_JPRB ,&
&  0.17259E+01_JPRB, 0.18573E+01_JPRB, 0.19617E+01_JPRB, 0.18508E+01_JPRB, 0.16002E+01_JPRB, 0.15055E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.78763E+00_JPRB, 0.85800E+00_JPRB, 0.92893E+00_JPRB, 0.91921E+00_JPRB, 0.90230E+00_JPRB, 0.86646E+00_JPRB ,&
&  0.81377E+00_JPRB, 0.74276E+00_JPRB, 0.66201E+00_JPRB, 0.66595E+00_JPRB, 0.69373E+00_JPRB, 0.70840E+00_JPRB, 0.70587E+00_JPRB ,&
&  0.71459E+00_JPRB, 0.70102E+00_JPRB, 0.68417E+00_JPRB, 0.65968E+00_JPRB, 0.65348E+00_JPRB, 0.67560E+00_JPRB, 0.68762E+00_JPRB ,&
&  0.70734E+00_JPRB, 0.65956E+00_JPRB, 0.62510E+00_JPRB, 0.63436E+00_JPRB, 0.64495E+00_JPRB, 0.66155E+00_JPRB, 0.66155E+00_JPRB ,&
&  0.71351E+00_JPRB, 0.86804E+00_JPRB, 0.12663E+01_JPRB, 0.16733E+01_JPRB, 0.13733E+01_JPRB, 0.10799E+01_JPRB, 0.60564E+00_JPRB ,&
&  0.47824E+00_JPRB, 0.46147E+00_JPRB, 0.47913E+00_JPRB, 0.48210E+00_JPRB, 0.48000E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.40725E+00_JPRB, 0.39827E+00_JPRB, 0.40519E+00_JPRB, 0.40363E+00_JPRB, 0.40719E+00_JPRB, 0.41394E+00_JPRB, 0.41827E+00_JPRB ,&
&  0.42158E+00_JPRB, 0.43788E+00_JPRB, 0.52402E+00_JPRB, 0.64139E+00_JPRB, 0.75070E+00_JPRB, 0.83172E+00_JPRB, 0.86256E+00_JPRB ,&
&  0.88766E+00_JPRB, 0.88549E+00_JPRB, 0.85445E+00_JPRB, 0.81939E+00_JPRB, 0.80636E+00_JPRB, 0.83027E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.85918E+00_JPRB, 0.86799E+00_JPRB, 0.87297E+00_JPRB, 0.88484E+00_JPRB, 0.89065E+00_JPRB, 0.80510E+00_JPRB, 0.48940E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.76879E+00_JPRB, 0.75145E+00_JPRB, 0.68110E+00_JPRB, 0.62249E+00_JPRB, 0.57449E+00_JPRB, 0.57550E+00_JPRB ,&
&  0.58736E+00_JPRB, 0.59084E+00_JPRB, 0.60915E+00_JPRB, 0.66819E+00_JPRB, 0.91473E+00_JPRB, 0.10121E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.12256E+01_JPRB, 0.12729E+01_JPRB, 0.13358E+01_JPRB, 0.13841E+01_JPRB, 0.14040E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.14468E+01_JPRB, 0.14759E+01_JPRB, 0.14499E+01_JPRB, 0.13606E+01_JPRB, 0.11714E+01_JPRB, 0.12022E+01_JPRB, 0.95408E+00_JPRB ,&
&  0.77649E+00_JPRB, 0.71580E+00_JPRB, 0.80115E+00_JPRB, 0.89479E+00_JPRB, 0.90014E+00_JPRB, 0.86858E+00_JPRB, 0.80421E+00_JPRB ,&
&  0.74531E+00_JPRB, 0.69521E+00_JPRB, 0.65587E+00_JPRB, 0.62866E+00_JPRB, 0.63285E+00_JPRB, 0.64288E+00_JPRB, 0.64051E+00_JPRB ,&
&  0.63823E+00_JPRB, 0.61276E+00_JPRB, 0.58058E+00_JPRB, 0.57427E+00_JPRB, 0.58192E+00_JPRB, 0.64512E+00_JPRB, 0.80168E+00_JPRB ,&
&  0.89609E+00_JPRB, 0.55064E+00_JPRB, 0.52564E+00_JPRB, 0.50421E+00_JPRB, 0.51418E+00_JPRB, 0.55405E+00_JPRB, 0.64243E+00_JPRB ,&
&  0.86153E+00_JPRB, 0.15747E+01_JPRB, 0.24801E+01_JPRB, 0.18976E+01_JPRB, 0.21762E+01_JPRB, 0.14876E+01_JPRB, 0.58808E+00_JPRB ,&
&  0.45063E+00_JPRB, 0.44117E+00_JPRB, 0.44615E+00_JPRB, 0.43534E+00_JPRB, 0.42220E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.41546E+00_JPRB, 0.40564E+00_JPRB, 0.41227E+00_JPRB, 0.42149E+00_JPRB, 0.41731E+00_JPRB, 0.41647E+00_JPRB, 0.41145E+00_JPRB ,&
&  0.40198E+00_JPRB, 0.39597E+00_JPRB, 0.41692E+00_JPRB, 0.45685E+00_JPRB, 0.51140E+00_JPRB, 0.58158E+00_JPRB, 0.64834E+00_JPRB ,&
&  0.68675E+00_JPRB, 0.72831E+00_JPRB, 0.76095E+00_JPRB, 0.78475E+00_JPRB, 0.77609E+00_JPRB, 0.76552E+00_JPRB, 0.74149E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.71848E+00_JPRB, 0.73669E+00_JPRB, 0.74461E+00_JPRB, 0.77159E+00_JPRB, 0.79795E+00_JPRB, 0.52482E+00_JPRB ,&
&  0.85759E+00_JPRB, 0.86750E+00_JPRB, 0.80457E+00_JPRB, 0.72436E+00_JPRB, 0.68197E+00_JPRB, 0.62050E+00_JPRB, 0.55788E+00_JPRB ,&
&  0.53145E+00_JPRB, 0.52480E+00_JPRB, 0.52284E+00_JPRB, 0.60037E+00_JPRB, 0.72049E+00_JPRB, 0.80580E+00_JPRB, 0.88356E+00_JPRB ,&
&  0.92516E+00_JPRB, 0.97712E+00_JPRB, 0.10184E+01_JPRB, 0.10548E+01_JPRB, 0.10846E+01_JPRB, 0.10836E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.98628E+00_JPRB, 0.93457E+00_JPRB, 0.90998E+00_JPRB, 0.94703E+00_JPRB, 0.90167E+00_JPRB, 0.86145E+00_JPRB, 0.76350E+00_JPRB ,&
&  0.61478E+00_JPRB, 0.59692E+00_JPRB, 0.66903E+00_JPRB, 0.87925E+00_JPRB, 0.90464E+00_JPRB, 0.86822E+00_JPRB, 0.81377E+00_JPRB ,&
&  0.76257E+00_JPRB, 0.71510E+00_JPRB, 0.67530E+00_JPRB, 0.63944E+00_JPRB, 0.61782E+00_JPRB, 0.60521E+00_JPRB, 0.59272E+00_JPRB ,&
&  0.57668E+00_JPRB, 0.54763E+00_JPRB, 0.51640E+00_JPRB, 0.50395E+00_JPRB, 0.49473E+00_JPRB, 0.50217E+00_JPRB, 0.50522E+00_JPRB ,&
&  0.60748E+00_JPRB, 0.48728E+00_JPRB, 0.45063E+00_JPRB, 0.45134E+00_JPRB, 0.42254E+00_JPRB, 0.47311E+00_JPRB, 0.64980E+00_JPRB ,&
&  0.10697E+01_JPRB, 0.16124E+01_JPRB, 0.27580E+01_JPRB, 0.49876E+01_JPRB, 0.27486E+01_JPRB, 0.21124E+01_JPRB, 0.48720E+00_JPRB ,&
&  0.46438E+00_JPRB, 0.46605E+00_JPRB, 0.46227E+00_JPRB, 0.44444E+00_JPRB, 0.43926E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.45494E+00_JPRB, 0.45250E+00_JPRB, 0.45886E+00_JPRB, 0.46591E+00_JPRB, 0.46705E+00_JPRB, 0.46740E+00_JPRB, 0.46683E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.46637E+00_JPRB, 0.46738E+00_JPRB, 0.47381E+00_JPRB, 0.48785E+00_JPRB, 0.51878E+00_JPRB, 0.55401E+00_JPRB ,&
&  0.58395E+00_JPRB, 0.61040E+00_JPRB, 0.63409E+00_JPRB, 0.65334E+00_JPRB, 0.67221E+00_JPRB, 0.68118E+00_JPRB, 0.71942E+00_JPRB ,&
&  0.76797E+00_JPRB, 0.82720E+00_JPRB, 0.88448E+00_JPRB, 0.91956E+00_JPRB, 0.93764E+00_JPRB, 0.86610E+00_JPRB, 0.66524E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.10289E+01_JPRB, 0.91304E+00_JPRB, 0.81066E+00_JPRB, 0.74282E+00_JPRB, 0.68658E+00_JPRB, 0.62841E+00_JPRB ,&
&  0.57228E+00_JPRB, 0.56916E+00_JPRB, 0.52967E+00_JPRB, 0.52840E+00_JPRB, 0.54162E+00_JPRB, 0.58399E+00_JPRB, 0.68239E+00_JPRB ,&
&  0.74953E+00_JPRB, 0.79735E+00_JPRB, 0.85032E+00_JPRB, 0.91513E+00_JPRB, 0.94564E+00_JPRB, 0.92553E+00_JPRB, 0.83384E+00_JPRB ,&
&  0.77741E+00_JPRB, 0.74471E+00_JPRB, 0.69349E+00_JPRB, 0.72595E+00_JPRB, 0.74233E+00_JPRB, 0.72740E+00_JPRB, 0.70388E+00_JPRB ,&
&  0.58961E+00_JPRB, 0.60628E+00_JPRB, 0.70345E+00_JPRB, 0.93934E+00_JPRB, 0.89367E+00_JPRB, 0.83699E+00_JPRB, 0.82287E+00_JPRB ,&
&  0.77338E+00_JPRB, 0.73188E+00_JPRB, 0.69950E+00_JPRB, 0.66146E+00_JPRB, 0.62918E+00_JPRB, 0.61018E+00_JPRB, 0.58653E+00_JPRB ,&
&  0.55692E+00_JPRB, 0.52006E+00_JPRB, 0.48831E+00_JPRB, 0.46513E+00_JPRB, 0.45539E+00_JPRB, 0.40795E+00_JPRB, 0.50794E+00_JPRB ,&
&  0.50460E+00_JPRB, 0.44757E+00_JPRB, 0.46986E+00_JPRB, 0.34890E+00_JPRB, 0.43546E+00_JPRB, 0.48882E+00_JPRB, 0.61325E+00_JPRB ,&
&  0.78333E+00_JPRB, 0.12612E+01_JPRB, 0.19920E+01_JPRB, 0.17953E+01_JPRB, 0.13628E+01_JPRB, 0.86336E+00_JPRB, 0.50190E+00_JPRB ,&
&  0.48010E+00_JPRB, 0.47490E+00_JPRB, 0.47858E+00_JPRB, 0.46784E+00_JPRB, 0.46329E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.48310E+00_JPRB, 0.48190E+00_JPRB, 0.49402E+00_JPRB, 0.50615E+00_JPRB, 0.51256E+00_JPRB, 0.51604E+00_JPRB, 0.52252E+00_JPRB ,&
&  0.52423E+00_JPRB, 0.52851E+00_JPRB, 0.53398E+00_JPRB, 0.54653E+00_JPRB, 0.56668E+00_JPRB, 0.59863E+00_JPRB, 0.63485E+00_JPRB ,&
&  0.67829E+00_JPRB, 0.72513E+00_JPRB, 0.77629E+00_JPRB, 0.83134E+00_JPRB, 0.90043E+00_JPRB, 0.99525E+00_JPRB, 0.11259E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.12811E+01_JPRB, 0.13809E+01_JPRB, 0.14580E+01_JPRB, 0.14501E+01_JPRB, 0.13892E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11316E+01_JPRB, 0.10117E+01_JPRB, 0.89984E+00_JPRB, 0.82653E+00_JPRB, 0.76855E+00_JPRB, 0.72697E+00_JPRB ,&
&  0.71408E+00_JPRB, 0.68483E+00_JPRB, 0.64106E+00_JPRB, 0.61095E+00_JPRB, 0.59389E+00_JPRB, 0.60071E+00_JPRB, 0.59387E+00_JPRB ,&
&  0.64631E+00_JPRB, 0.75175E+00_JPRB, 0.88597E+00_JPRB, 0.10524E+01_JPRB, 0.11414E+01_JPRB, 0.11244E+01_JPRB, 0.98371E+00_JPRB ,&
&  0.87763E+00_JPRB, 0.79880E+00_JPRB, 0.77035E+00_JPRB, 0.79955E+00_JPRB, 0.82007E+00_JPRB, 0.85679E+00_JPRB, 0.85423E+00_JPRB ,&
&  0.77563E+00_JPRB, 0.70932E+00_JPRB, 0.76049E+00_JPRB, 0.95361E+00_JPRB, 0.88851E+00_JPRB, 0.80315E+00_JPRB, 0.80445E+00_JPRB ,&
&  0.75261E+00_JPRB, 0.71259E+00_JPRB, 0.69322E+00_JPRB, 0.65828E+00_JPRB, 0.62197E+00_JPRB, 0.59855E+00_JPRB, 0.58270E+00_JPRB ,&
&  0.54767E+00_JPRB, 0.50768E+00_JPRB, 0.47551E+00_JPRB, 0.47116E+00_JPRB, 0.50711E+00_JPRB, 0.50324E+00_JPRB, 0.55811E+00_JPRB ,&
&  0.52319E+00_JPRB, 0.40906E+00_JPRB, 0.44579E+00_JPRB, 0.52941E+00_JPRB, 0.52603E+00_JPRB, 0.56173E+00_JPRB, 0.63053E+00_JPRB ,&
&  0.70368E+00_JPRB, 0.76855E+00_JPRB, 0.76099E+00_JPRB, 0.64929E+00_JPRB, 0.60490E+00_JPRB, 0.58057E+00_JPRB, 0.55433E+00_JPRB ,&
&  0.53320E+00_JPRB, 0.49633E+00_JPRB, 0.50714E+00_JPRB, 0.49279E+00_JPRB, 0.48875E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.52763E+00_JPRB, 0.52260E+00_JPRB, 0.52309E+00_JPRB, 0.52225E+00_JPRB, 0.53146E+00_JPRB, 0.53233E+00_JPRB, 0.53114E+00_JPRB ,&
&  0.52844E+00_JPRB, 0.53526E+00_JPRB, 0.53933E+00_JPRB, 0.54649E+00_JPRB, 0.55541E+00_JPRB, 0.58002E+00_JPRB, 0.60790E+00_JPRB ,&
&  0.64138E+00_JPRB, 0.68142E+00_JPRB, 0.72422E+00_JPRB, 0.78455E+00_JPRB, 0.86891E+00_JPRB, 0.92157E+00_JPRB, 0.98198E+00_JPRB ,&
&  0.10584E+01_JPRB, 0.11769E+01_JPRB, 0.13040E+01_JPRB, 0.14755E+01_JPRB, 0.16609E+01_JPRB, 0.16563E+01_JPRB, 0.15221E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.11258E+01_JPRB, 0.10400E+01_JPRB, 0.93373E+00_JPRB, 0.93312E+00_JPRB, 0.90436E+00_JPRB, 0.91278E+00_JPRB ,&
&  0.93537E+00_JPRB, 0.93449E+00_JPRB, 0.92558E+00_JPRB, 0.93124E+00_JPRB, 0.93569E+00_JPRB, 0.94174E+00_JPRB, 0.92696E+00_JPRB ,&
&  0.92153E+00_JPRB, 0.95518E+00_JPRB, 0.11030E+01_JPRB, 0.12906E+01_JPRB, 0.14174E+01_JPRB, 0.14464E+01_JPRB, 0.14056E+01_JPRB ,&
&  0.13083E+01_JPRB, 0.12104E+01_JPRB, 0.11150E+01_JPRB, 0.10879E+01_JPRB, 0.11104E+01_JPRB, 0.11219E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.90235E+00_JPRB, 0.88695E+00_JPRB, 0.88491E+00_JPRB, 0.84170E+00_JPRB, 0.80734E+00_JPRB, 0.76804E+00_JPRB ,&
&  0.76164E+00_JPRB, 0.72113E+00_JPRB, 0.69799E+00_JPRB, 0.65820E+00_JPRB, 0.61937E+00_JPRB, 0.59529E+00_JPRB, 0.58011E+00_JPRB ,&
&  0.55121E+00_JPRB, 0.52560E+00_JPRB, 0.51181E+00_JPRB, 0.52778E+00_JPRB, 0.54830E+00_JPRB, 0.71998E+00_JPRB, 0.61854E+00_JPRB ,&
&  0.59025E+00_JPRB, 0.55006E+00_JPRB, 0.47851E+00_JPRB, 0.61224E+00_JPRB, 0.61769E+00_JPRB, 0.62631E+00_JPRB, 0.63212E+00_JPRB ,&
&  0.63728E+00_JPRB, 0.63295E+00_JPRB, 0.60939E+00_JPRB, 0.60294E+00_JPRB, 0.61011E+00_JPRB, 0.61556E+00_JPRB, 0.60366E+00_JPRB ,&
&  0.56752E+00_JPRB, 0.54994E+00_JPRB, 0.57167E+00_JPRB, 0.53735E+00_JPRB, 0.53125E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.60134E+00_JPRB, 0.57543E+00_JPRB, 0.55881E+00_JPRB, 0.56523E+00_JPRB, 0.57290E+00_JPRB, 0.56169E+00_JPRB, 0.58884E+00_JPRB ,&
&  0.56465E+00_JPRB, 0.58358E+00_JPRB, 0.59451E+00_JPRB, 0.59185E+00_JPRB, 0.62675E+00_JPRB, 0.66199E+00_JPRB, 0.70655E+00_JPRB ,&
&  0.76766E+00_JPRB, 0.86528E+00_JPRB, 0.98070E+00_JPRB, 0.11209E+01_JPRB, 0.12891E+01_JPRB, 0.13604E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.15316E+01_JPRB, 0.16648E+01_JPRB, 0.18612E+01_JPRB, 0.20910E+01_JPRB, 0.23710E+01_JPRB, 0.21761E+01_JPRB, 0.18362E+01_JPRB ,&
&  0.13562E+01_JPRB, 0.10632E+01_JPRB, 0.10139E+01_JPRB, 0.93700E+00_JPRB, 0.97355E+00_JPRB, 0.10707E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11359E+01_JPRB, 0.11653E+01_JPRB, 0.11954E+01_JPRB, 0.12312E+01_JPRB, 0.12737E+01_JPRB, 0.13196E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.13763E+01_JPRB, 0.14064E+01_JPRB, 0.14921E+01_JPRB, 0.15774E+01_JPRB, 0.16463E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.17258E+01_JPRB, 0.16224E+01_JPRB, 0.13746E+01_JPRB, 0.13475E+01_JPRB, 0.13742E+01_JPRB, 0.13896E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.14949E+01_JPRB, 0.12125E+01_JPRB, 0.82841E+00_JPRB, 0.82623E+00_JPRB, 0.80783E+00_JPRB, 0.82291E+00_JPRB, 0.74925E+00_JPRB ,&
&  0.74604E+00_JPRB, 0.73511E+00_JPRB, 0.72915E+00_JPRB, 0.69203E+00_JPRB, 0.67059E+00_JPRB, 0.64851E+00_JPRB, 0.61558E+00_JPRB ,&
&  0.57168E+00_JPRB, 0.56207E+00_JPRB, 0.55029E+00_JPRB, 0.55599E+00_JPRB, 0.62493E+00_JPRB, 0.59911E+00_JPRB, 0.61888E+00_JPRB ,&
&  0.62407E+00_JPRB, 0.69259E+00_JPRB, 0.62400E+00_JPRB, 0.72088E+00_JPRB, 0.69051E+00_JPRB, 0.68107E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.68620E+00_JPRB, 0.69505E+00_JPRB, 0.70371E+00_JPRB, 0.69200E+00_JPRB, 0.70705E+00_JPRB, 0.73065E+00_JPRB, 0.72724E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.66789E+00_JPRB, 0.64439E+00_JPRB, 0.61589E+00_JPRB, 0.61076E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.64634E+00_JPRB, 0.65027E+00_JPRB, 0.64979E+00_JPRB, 0.65462E+00_JPRB, 0.65915E+00_JPRB, 0.64683E+00_JPRB, 0.66780E+00_JPRB ,&
&  0.66286E+00_JPRB, 0.68286E+00_JPRB, 0.70561E+00_JPRB, 0.72890E+00_JPRB, 0.77328E+00_JPRB, 0.81796E+00_JPRB, 0.87858E+00_JPRB ,&
&  0.97917E+00_JPRB, 0.11399E+01_JPRB, 0.13365E+01_JPRB, 0.15742E+01_JPRB, 0.16221E+01_JPRB, 0.16391E+01_JPRB, 0.17072E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.18354E+01_JPRB, 0.18175E+01_JPRB, 0.18242E+01_JPRB, 0.18317E+01_JPRB, 0.17216E+01_JPRB, 0.16042E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.10407E+01_JPRB, 0.11199E+01_JPRB, 0.10601E+01_JPRB, 0.10932E+01_JPRB, 0.11247E+01_JPRB, 0.11530E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.11932E+01_JPRB, 0.12094E+01_JPRB, 0.12329E+01_JPRB, 0.12707E+01_JPRB, 0.13030E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.13713E+01_JPRB, 0.13998E+01_JPRB, 0.14348E+01_JPRB, 0.13986E+01_JPRB, 0.14383E+01_JPRB, 0.14911E+01_JPRB, 0.15527E+01_JPRB ,&
&  0.16210E+01_JPRB, 0.15622E+01_JPRB, 0.13432E+01_JPRB, 0.13971E+01_JPRB, 0.15118E+01_JPRB, 0.14339E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.16441E+01_JPRB, 0.14992E+01_JPRB, 0.83037E+00_JPRB, 0.72213E+00_JPRB, 0.73836E+00_JPRB, 0.76496E+00_JPRB, 0.78692E+00_JPRB ,&
&  0.77670E+00_JPRB, 0.76490E+00_JPRB, 0.75783E+00_JPRB, 0.75206E+00_JPRB, 0.72748E+00_JPRB, 0.71779E+00_JPRB, 0.65930E+00_JPRB ,&
&  0.54898E+00_JPRB, 0.54072E+00_JPRB, 0.55055E+00_JPRB, 0.56822E+00_JPRB, 0.60738E+00_JPRB, 0.62089E+00_JPRB, 0.63002E+00_JPRB ,&
&  0.67845E+00_JPRB, 0.74105E+00_JPRB, 0.79273E+00_JPRB, 0.72131E+00_JPRB, 0.65798E+00_JPRB, 0.73329E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.74952E+00_JPRB, 0.75032E+00_JPRB, 0.75863E+00_JPRB, 0.76099E+00_JPRB, 0.76672E+00_JPRB, 0.75135E+00_JPRB, 0.73342E+00_JPRB ,&
&  0.71402E+00_JPRB, 0.69751E+00_JPRB, 0.68603E+00_JPRB, 0.66698E+00_JPRB, 0.65260E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.69818E+00_JPRB, 0.67945E+00_JPRB, 0.66952E+00_JPRB, 0.65789E+00_JPRB, 0.65517E+00_JPRB, 0.65116E+00_JPRB, 0.65072E+00_JPRB ,&
&  0.65189E+00_JPRB, 0.65711E+00_JPRB, 0.67664E+00_JPRB, 0.71066E+00_JPRB, 0.75788E+00_JPRB, 0.82103E+00_JPRB, 0.92681E+00_JPRB ,&
&  0.10297E+01_JPRB, 0.11815E+01_JPRB, 0.13580E+01_JPRB, 0.15551E+01_JPRB, 0.16794E+01_JPRB, 0.16981E+01_JPRB, 0.17705E+01_JPRB ,&
&  0.18062E+01_JPRB, 0.17907E+01_JPRB, 0.16987E+01_JPRB, 0.15689E+01_JPRB, 0.14540E+01_JPRB, 0.14192E+01_JPRB, 0.14352E+01_JPRB ,&
&  0.12743E+01_JPRB, 0.10549E+01_JPRB, 0.10369E+01_JPRB, 0.10531E+01_JPRB, 0.10698E+01_JPRB, 0.10870E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.11483E+01_JPRB, 0.11656E+01_JPRB, 0.11758E+01_JPRB, 0.11875E+01_JPRB, 0.12322E+01_JPRB, 0.12431E+01_JPRB, 0.12660E+01_JPRB ,&
&  0.12822E+01_JPRB, 0.12991E+01_JPRB, 0.13504E+01_JPRB, 0.12846E+01_JPRB, 0.13775E+01_JPRB, 0.13824E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.13205E+01_JPRB, 0.12258E+01_JPRB, 0.12477E+01_JPRB, 0.13191E+01_JPRB, 0.13974E+01_JPRB, 0.13712E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14053E+01_JPRB, 0.86369E+00_JPRB, 0.74882E+00_JPRB, 0.76895E+00_JPRB, 0.71952E+00_JPRB, 0.81261E+00_JPRB ,&
&  0.81004E+00_JPRB, 0.80070E+00_JPRB, 0.79038E+00_JPRB, 0.79213E+00_JPRB, 0.80311E+00_JPRB, 0.87415E+00_JPRB, 0.71285E+00_JPRB ,&
&  0.60426E+00_JPRB, 0.57022E+00_JPRB, 0.56242E+00_JPRB, 0.57612E+00_JPRB, 0.61317E+00_JPRB, 0.64382E+00_JPRB, 0.69016E+00_JPRB ,&
&  0.73181E+00_JPRB, 0.78752E+00_JPRB, 0.84356E+00_JPRB, 0.83379E+00_JPRB, 0.75357E+00_JPRB, 0.79511E+00_JPRB, 0.81387E+00_JPRB ,&
&  0.82471E+00_JPRB, 0.82522E+00_JPRB, 0.81332E+00_JPRB, 0.81087E+00_JPRB, 0.79605E+00_JPRB, 0.78860E+00_JPRB, 0.77977E+00_JPRB ,&
&  0.76784E+00_JPRB, 0.76460E+00_JPRB, 0.75525E+00_JPRB, 0.72651E+00_JPRB, 0.70959E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.74997E+00_JPRB, 0.72710E+00_JPRB, 0.70853E+00_JPRB, 0.69345E+00_JPRB, 0.68493E+00_JPRB, 0.66557E+00_JPRB, 0.63993E+00_JPRB ,&
&  0.62846E+00_JPRB, 0.62592E+00_JPRB, 0.63953E+00_JPRB, 0.66779E+00_JPRB, 0.70958E+00_JPRB, 0.75308E+00_JPRB, 0.83109E+00_JPRB ,&
&  0.92395E+00_JPRB, 0.10592E+01_JPRB, 0.12309E+01_JPRB, 0.14656E+01_JPRB, 0.16520E+01_JPRB, 0.18201E+01_JPRB, 0.18491E+01_JPRB ,&
&  0.19221E+01_JPRB, 0.19133E+01_JPRB, 0.17672E+01_JPRB, 0.15317E+01_JPRB, 0.11706E+01_JPRB, 0.11283E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.10886E+01_JPRB, 0.10120E+01_JPRB, 0.10040E+01_JPRB, 0.99540E+00_JPRB, 0.10223E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11033E+01_JPRB, 0.11213E+01_JPRB, 0.11494E+01_JPRB, 0.11746E+01_JPRB, 0.11791E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.11874E+01_JPRB, 0.11831E+01_JPRB, 0.11794E+01_JPRB, 0.11631E+01_JPRB, 0.11625E+01_JPRB, 0.11505E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.10332E+01_JPRB, 0.10639E+01_JPRB, 0.11066E+01_JPRB, 0.11034E+01_JPRB, 0.10499E+01_JPRB, 0.10551E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.13267E+01_JPRB, 0.86305E+00_JPRB, 0.86513E+00_JPRB, 0.93364E+00_JPRB, 0.85093E+00_JPRB, 0.81933E+00_JPRB ,&
&  0.82541E+00_JPRB, 0.81298E+00_JPRB, 0.80576E+00_JPRB, 0.80523E+00_JPRB, 0.83981E+00_JPRB, 0.92991E+00_JPRB, 0.74531E+00_JPRB ,&
&  0.68098E+00_JPRB, 0.64857E+00_JPRB, 0.64978E+00_JPRB, 0.66145E+00_JPRB, 0.66337E+00_JPRB, 0.62989E+00_JPRB, 0.62757E+00_JPRB ,&
&  0.64788E+00_JPRB, 0.92260E+00_JPRB, 0.11450E+01_JPRB, 0.13527E+01_JPRB, 0.12886E+01_JPRB, 0.81460E+00_JPRB, 0.88024E+00_JPRB ,&
&  0.89633E+00_JPRB, 0.89127E+00_JPRB, 0.85519E+00_JPRB, 0.84297E+00_JPRB, 0.83270E+00_JPRB, 0.81953E+00_JPRB, 0.81644E+00_JPRB ,&
&  0.80837E+00_JPRB, 0.81114E+00_JPRB, 0.81769E+00_JPRB, 0.79933E+00_JPRB, 0.77188E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.81000E+00_JPRB, 0.79196E+00_JPRB, 0.77125E+00_JPRB, 0.75105E+00_JPRB, 0.73279E+00_JPRB, 0.68835E+00_JPRB, 0.64266E+00_JPRB ,&
&  0.61295E+00_JPRB, 0.59972E+00_JPRB, 0.60490E+00_JPRB, 0.61627E+00_JPRB, 0.64365E+00_JPRB, 0.67422E+00_JPRB, 0.72580E+00_JPRB ,&
&  0.80480E+00_JPRB, 0.91674E+00_JPRB, 0.10721E+01_JPRB, 0.12642E+01_JPRB, 0.14945E+01_JPRB, 0.17605E+01_JPRB, 0.19298E+01_JPRB ,&
&  0.20820E+01_JPRB, 0.20995E+01_JPRB, 0.18453E+01_JPRB, 0.12979E+01_JPRB, 0.11201E+01_JPRB, 0.10769E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.99751E+00_JPRB, 0.97958E+00_JPRB, 0.95726E+00_JPRB, 0.94286E+00_JPRB, 0.93787E+00_JPRB, 0.95555E+00_JPRB, 0.97436E+00_JPRB ,&
&  0.99317E+00_JPRB, 0.10099E+01_JPRB, 0.10293E+01_JPRB, 0.10643E+01_JPRB, 0.10760E+01_JPRB, 0.10947E+01_JPRB, 0.11194E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.11178E+01_JPRB, 0.10770E+01_JPRB, 0.10519E+01_JPRB, 0.10187E+01_JPRB, 0.99230E+00_JPRB, 0.96619E+00_JPRB ,&
&  0.95722E+00_JPRB, 0.94137E+00_JPRB, 0.93653E+00_JPRB, 0.95516E+00_JPRB, 0.96178E+00_JPRB, 0.88435E+00_JPRB, 0.91423E+00_JPRB ,&
&  0.10263E+01_JPRB, 0.11322E+01_JPRB, 0.93788E+00_JPRB, 0.98545E+00_JPRB, 0.68908E+00_JPRB, 0.78015E+00_JPRB, 0.85762E+00_JPRB ,&
&  0.83968E+00_JPRB, 0.82438E+00_JPRB, 0.80144E+00_JPRB, 0.81911E+00_JPRB, 0.85252E+00_JPRB, 0.85138E+00_JPRB, 0.80265E+00_JPRB ,&
&  0.74752E+00_JPRB, 0.71590E+00_JPRB, 0.73050E+00_JPRB, 0.75432E+00_JPRB, 0.75258E+00_JPRB, 0.69781E+00_JPRB, 0.64127E+00_JPRB ,&
&  0.62876E+00_JPRB, 0.82714E+00_JPRB, 0.93084E+00_JPRB, 0.10267E+01_JPRB, 0.93404E+00_JPRB, 0.78903E+00_JPRB, 0.83829E+00_JPRB ,&
&  0.87369E+00_JPRB, 0.88683E+00_JPRB, 0.87679E+00_JPRB, 0.85712E+00_JPRB, 0.83324E+00_JPRB, 0.81259E+00_JPRB, 0.80229E+00_JPRB ,&
&  0.79933E+00_JPRB, 0.79603E+00_JPRB, 0.80273E+00_JPRB, 0.81336E+00_JPRB, 0.81953E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.80447E+00_JPRB, 0.80187E+00_JPRB, 0.80244E+00_JPRB, 0.80353E+00_JPRB, 0.79683E+00_JPRB, 0.76891E+00_JPRB, 0.70958E+00_JPRB ,&
&  0.62291E+00_JPRB, 0.55682E+00_JPRB, 0.54843E+00_JPRB, 0.55191E+00_JPRB, 0.56327E+00_JPRB, 0.59360E+00_JPRB, 0.64264E+00_JPRB ,&
&  0.69917E+00_JPRB, 0.79302E+00_JPRB, 0.92657E+00_JPRB, 0.10955E+01_JPRB, 0.13147E+01_JPRB, 0.16414E+01_JPRB, 0.19909E+01_JPRB ,&
&  0.19225E+01_JPRB, 0.14341E+01_JPRB, 0.17773E+01_JPRB, 0.17702E+01_JPRB, 0.15552E+01_JPRB, 0.12588E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.10084E+01_JPRB, 0.98472E+00_JPRB, 0.96675E+00_JPRB, 0.93994E+00_JPRB, 0.89371E+00_JPRB, 0.90215E+00_JPRB, 0.91490E+00_JPRB ,&
&  0.92991E+00_JPRB, 0.94386E+00_JPRB, 0.95280E+00_JPRB, 0.99227E+00_JPRB, 0.10017E+01_JPRB, 0.10137E+01_JPRB, 0.10405E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10374E+01_JPRB, 0.10030E+01_JPRB, 0.94163E+00_JPRB, 0.92249E+00_JPRB, 0.90479E+00_JPRB, 0.89222E+00_JPRB ,&
&  0.89645E+00_JPRB, 0.88824E+00_JPRB, 0.90158E+00_JPRB, 0.91257E+00_JPRB, 0.93217E+00_JPRB, 0.86460E+00_JPRB, 0.89296E+00_JPRB ,&
&  0.93785E+00_JPRB, 0.10383E+01_JPRB, 0.10100E+01_JPRB, 0.10415E+01_JPRB, 0.77014E+00_JPRB, 0.82417E+00_JPRB, 0.81476E+00_JPRB ,&
&  0.87346E+00_JPRB, 0.83559E+00_JPRB, 0.80208E+00_JPRB, 0.80436E+00_JPRB, 0.84071E+00_JPRB, 0.86559E+00_JPRB, 0.87566E+00_JPRB ,&
&  0.78646E+00_JPRB, 0.76080E+00_JPRB, 0.76529E+00_JPRB, 0.80453E+00_JPRB, 0.79461E+00_JPRB, 0.76107E+00_JPRB, 0.73328E+00_JPRB ,&
&  0.81676E+00_JPRB, 0.85463E+00_JPRB, 0.90936E+00_JPRB, 0.92575E+00_JPRB, 0.85193E+00_JPRB, 0.77535E+00_JPRB, 0.79146E+00_JPRB ,&
&  0.81373E+00_JPRB, 0.83111E+00_JPRB, 0.83000E+00_JPRB, 0.81374E+00_JPRB, 0.78473E+00_JPRB, 0.75944E+00_JPRB, 0.74619E+00_JPRB ,&
&  0.73597E+00_JPRB, 0.74262E+00_JPRB, 0.75285E+00_JPRB, 0.76664E+00_JPRB, 0.78745E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.71626E+00_JPRB, 0.73576E+00_JPRB, 0.75198E+00_JPRB, 0.79526E+00_JPRB, 0.84871E+00_JPRB, 0.91959E+00_JPRB, 0.95478E+00_JPRB ,&
&  0.81392E+00_JPRB, 0.53119E+00_JPRB, 0.52011E+00_JPRB, 0.51523E+00_JPRB, 0.51521E+00_JPRB, 0.52549E+00_JPRB, 0.56755E+00_JPRB ,&
&  0.63077E+00_JPRB, 0.71733E+00_JPRB, 0.81729E+00_JPRB, 0.93777E+00_JPRB, 0.10997E+01_JPRB, 0.14137E+01_JPRB, 0.16356E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.17184E+01_JPRB, 0.21993E+01_JPRB, 0.19876E+01_JPRB, 0.17627E+01_JPRB, 0.14975E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10208E+01_JPRB, 0.10067E+01_JPRB, 0.10103E+01_JPRB, 0.90393E+00_JPRB, 0.89257E+00_JPRB, 0.89354E+00_JPRB ,&
&  0.89909E+00_JPRB, 0.90240E+00_JPRB, 0.90589E+00_JPRB, 0.94946E+00_JPRB, 0.98175E+00_JPRB, 0.99476E+00_JPRB, 0.99687E+00_JPRB ,&
&  0.99463E+00_JPRB, 0.98885E+00_JPRB, 0.96300E+00_JPRB, 0.89327E+00_JPRB, 0.88600E+00_JPRB, 0.88448E+00_JPRB, 0.89604E+00_JPRB ,&
&  0.91521E+00_JPRB, 0.96546E+00_JPRB, 0.98889E+00_JPRB, 0.94633E+00_JPRB, 0.87814E+00_JPRB, 0.93171E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.88242E+00_JPRB, 0.92594E+00_JPRB, 0.10972E+01_JPRB, 0.88011E+00_JPRB, 0.85653E+00_JPRB, 0.90733E+00_JPRB, 0.91468E+00_JPRB ,&
&  0.89203E+00_JPRB, 0.89992E+00_JPRB, 0.84058E+00_JPRB, 0.80049E+00_JPRB, 0.81591E+00_JPRB, 0.82531E+00_JPRB, 0.93319E+00_JPRB ,&
&  0.85420E+00_JPRB, 0.89402E+00_JPRB, 0.82833E+00_JPRB, 0.82176E+00_JPRB, 0.78608E+00_JPRB, 0.73736E+00_JPRB, 0.74074E+00_JPRB ,&
&  0.88743E+00_JPRB, 0.91783E+00_JPRB, 0.92734E+00_JPRB, 0.93412E+00_JPRB, 0.89101E+00_JPRB, 0.86817E+00_JPRB, 0.82933E+00_JPRB ,&
&  0.81066E+00_JPRB, 0.80859E+00_JPRB, 0.80802E+00_JPRB, 0.77889E+00_JPRB, 0.75266E+00_JPRB, 0.71774E+00_JPRB, 0.68931E+00_JPRB ,&
&  0.67303E+00_JPRB, 0.66591E+00_JPRB, 0.66654E+00_JPRB, 0.67830E+00_JPRB, 0.69491E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.64964E+00_JPRB, 0.65829E+00_JPRB, 0.65759E+00_JPRB, 0.63396E+00_JPRB, 0.61951E+00_JPRB, 0.60717E+00_JPRB, 0.57277E+00_JPRB ,&
&  0.54676E+00_JPRB, 0.52916E+00_JPRB, 0.51231E+00_JPRB, 0.50149E+00_JPRB, 0.50170E+00_JPRB, 0.49765E+00_JPRB, 0.51083E+00_JPRB ,&
&  0.55940E+00_JPRB, 0.63587E+00_JPRB, 0.75605E+00_JPRB, 0.83711E+00_JPRB, 0.93799E+00_JPRB, 0.11412E+01_JPRB, 0.12410E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.23361E+01_JPRB, 0.22131E+01_JPRB, 0.21741E+01_JPRB, 0.20349E+01_JPRB, 0.17785E+01_JPRB, 0.13159E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.10812E+01_JPRB, 0.10127E+01_JPRB, 0.97766E+00_JPRB, 0.95905E+00_JPRB, 0.91256E+00_JPRB, 0.91121E+00_JPRB ,&
&  0.90598E+00_JPRB, 0.90045E+00_JPRB, 0.88767E+00_JPRB, 0.91621E+00_JPRB, 0.96042E+00_JPRB, 0.98481E+00_JPRB, 0.10024E+01_JPRB ,&
&  0.99797E+00_JPRB, 0.99061E+00_JPRB, 0.95880E+00_JPRB, 0.93525E+00_JPRB, 0.93081E+00_JPRB, 0.95654E+00_JPRB, 0.99136E+00_JPRB ,&
&  0.99584E+00_JPRB, 0.99527E+00_JPRB, 0.10836E+01_JPRB, 0.10847E+01_JPRB, 0.10428E+01_JPRB, 0.10469E+01_JPRB, 0.92037E+00_JPRB ,&
&  0.92003E+00_JPRB, 0.86485E+00_JPRB, 0.99868E+00_JPRB, 0.88216E+00_JPRB, 0.89936E+00_JPRB, 0.94858E+00_JPRB, 0.95720E+00_JPRB ,&
&  0.91141E+00_JPRB, 0.95127E+00_JPRB, 0.90571E+00_JPRB, 0.85515E+00_JPRB, 0.82185E+00_JPRB, 0.82877E+00_JPRB, 0.90547E+00_JPRB ,&
&  0.93893E+00_JPRB, 0.94313E+00_JPRB, 0.97107E+00_JPRB, 0.86619E+00_JPRB, 0.83689E+00_JPRB, 0.76206E+00_JPRB, 0.76986E+00_JPRB ,&
&  0.91538E+00_JPRB, 0.93830E+00_JPRB, 0.98465E+00_JPRB, 0.98787E+00_JPRB, 0.94141E+00_JPRB, 0.95131E+00_JPRB, 0.91099E+00_JPRB ,&
&  0.86861E+00_JPRB, 0.83896E+00_JPRB, 0.81771E+00_JPRB, 0.79021E+00_JPRB, 0.74862E+00_JPRB, 0.70580E+00_JPRB, 0.67408E+00_JPRB ,&
&  0.64560E+00_JPRB, 0.62771E+00_JPRB, 0.62467E+00_JPRB, 0.62729E+00_JPRB, 0.63655E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.61522E+00_JPRB, 0.62060E+00_JPRB, 0.63520E+00_JPRB, 0.62221E+00_JPRB, 0.60207E+00_JPRB, 0.58346E+00_JPRB, 0.57265E+00_JPRB ,&
&  0.55499E+00_JPRB, 0.53057E+00_JPRB, 0.51024E+00_JPRB, 0.49849E+00_JPRB, 0.49063E+00_JPRB, 0.49091E+00_JPRB, 0.49941E+00_JPRB ,&
&  0.52570E+00_JPRB, 0.56231E+00_JPRB, 0.63940E+00_JPRB, 0.73082E+00_JPRB, 0.81185E+00_JPRB, 0.95226E+00_JPRB, 0.85400E+00_JPRB ,&
&  0.11711E+01_JPRB, 0.21599E+01_JPRB, 0.22608E+01_JPRB, 0.22914E+01_JPRB, 0.22201E+01_JPRB, 0.21368E+01_JPRB, 0.16561E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.12684E+01_JPRB, 0.11006E+01_JPRB, 0.10225E+01_JPRB, 0.99302E+00_JPRB, 0.97596E+00_JPRB, 0.95515E+00_JPRB ,&
&  0.93098E+00_JPRB, 0.90781E+00_JPRB, 0.89886E+00_JPRB, 0.92410E+00_JPRB, 0.96644E+00_JPRB, 0.10159E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10221E+01_JPRB, 0.10097E+01_JPRB, 0.10148E+01_JPRB, 0.96813E+00_JPRB, 0.10066E+01_JPRB, 0.10372E+01_JPRB, 0.10733E+01_JPRB ,&
&  0.10434E+01_JPRB, 0.10207E+01_JPRB, 0.11720E+01_JPRB, 0.12883E+01_JPRB, 0.12961E+01_JPRB, 0.11852E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.98831E+00_JPRB, 0.92264E+00_JPRB, 0.93696E+00_JPRB, 0.90208E+00_JPRB, 0.93020E+00_JPRB, 0.96680E+00_JPRB, 0.10147E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10031E+01_JPRB, 0.96190E+00_JPRB, 0.88916E+00_JPRB, 0.85209E+00_JPRB, 0.81570E+00_JPRB, 0.90981E+00_JPRB ,&
&  0.10271E+01_JPRB, 0.11063E+01_JPRB, 0.11351E+01_JPRB, 0.10092E+01_JPRB, 0.91916E+00_JPRB, 0.77562E+00_JPRB, 0.73334E+00_JPRB ,&
&  0.86918E+00_JPRB, 0.89822E+00_JPRB, 0.85263E+00_JPRB, 0.94449E+00_JPRB, 0.89999E+00_JPRB, 0.92505E+00_JPRB, 0.92658E+00_JPRB ,&
&  0.86739E+00_JPRB, 0.83085E+00_JPRB, 0.79303E+00_JPRB, 0.77140E+00_JPRB, 0.76525E+00_JPRB, 0.73159E+00_JPRB, 0.69970E+00_JPRB ,&
&  0.66968E+00_JPRB, 0.64913E+00_JPRB, 0.63213E+00_JPRB, 0.62134E+00_JPRB, 0.61382E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.61232E+00_JPRB, 0.60579E+00_JPRB, 0.60741E+00_JPRB, 0.61526E+00_JPRB, 0.61683E+00_JPRB, 0.59925E+00_JPRB, 0.58066E+00_JPRB ,&
&  0.55929E+00_JPRB, 0.53333E+00_JPRB, 0.51424E+00_JPRB, 0.49765E+00_JPRB, 0.48986E+00_JPRB, 0.48878E+00_JPRB, 0.49573E+00_JPRB ,&
&  0.51306E+00_JPRB, 0.53898E+00_JPRB, 0.61189E+00_JPRB, 0.67395E+00_JPRB, 0.72068E+00_JPRB, 0.69339E+00_JPRB, 0.81153E+00_JPRB ,&
&  0.14472E+01_JPRB, 0.21580E+01_JPRB, 0.21479E+01_JPRB, 0.22435E+01_JPRB, 0.23524E+01_JPRB, 0.25000E+01_JPRB, 0.19723E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.14979E+01_JPRB, 0.12929E+01_JPRB, 0.11633E+01_JPRB, 0.10769E+01_JPRB, 0.10331E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.97990E+00_JPRB, 0.94361E+00_JPRB, 0.91884E+00_JPRB, 0.93797E+00_JPRB, 0.10034E+01_JPRB, 0.10544E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10464E+01_JPRB, 0.10123E+01_JPRB, 0.10199E+01_JPRB, 0.10119E+01_JPRB, 0.10474E+01_JPRB, 0.10705E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.12064E+01_JPRB, 0.13025E+01_JPRB, 0.13458E+01_JPRB, 0.13661E+01_JPRB, 0.13436E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.96251E+00_JPRB, 0.86220E+00_JPRB, 0.90870E+00_JPRB, 0.96797E+00_JPRB, 0.10046E+01_JPRB, 0.96144E+00_JPRB ,&
&  0.91193E+00_JPRB, 0.83287E+00_JPRB, 0.85199E+00_JPRB, 0.89061E+00_JPRB, 0.92733E+00_JPRB, 0.87019E+00_JPRB, 0.98276E+00_JPRB ,&
&  0.95586E+00_JPRB, 0.73911E+00_JPRB, 0.67294E+00_JPRB, 0.79140E+00_JPRB, 0.71353E+00_JPRB, 0.61748E+00_JPRB, 0.66926E+00_JPRB ,&
&  0.76500E+00_JPRB, 0.89133E+00_JPRB, 0.85332E+00_JPRB, 0.78281E+00_JPRB, 0.87210E+00_JPRB, 0.90180E+00_JPRB, 0.92901E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.78639E+00_JPRB, 0.76035E+00_JPRB, 0.73163E+00_JPRB, 0.71193E+00_JPRB, 0.70687E+00_JPRB, 0.70185E+00_JPRB ,&
&  0.68314E+00_JPRB, 0.66597E+00_JPRB, 0.65641E+00_JPRB, 0.64593E+00_JPRB, 0.62852E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.62606E+00_JPRB, 0.60812E+00_JPRB, 0.60039E+00_JPRB, 0.59582E+00_JPRB, 0.59183E+00_JPRB, 0.58373E+00_JPRB, 0.57033E+00_JPRB ,&
&  0.55336E+00_JPRB, 0.52943E+00_JPRB, 0.50820E+00_JPRB, 0.49288E+00_JPRB, 0.48363E+00_JPRB, 0.48318E+00_JPRB, 0.49233E+00_JPRB ,&
&  0.50187E+00_JPRB, 0.53726E+00_JPRB, 0.63648E+00_JPRB, 0.68845E+00_JPRB, 0.68664E+00_JPRB, 0.62219E+00_JPRB, 0.79185E+00_JPRB ,&
&  0.12150E+01_JPRB, 0.18239E+01_JPRB, 0.23289E+01_JPRB, 0.25040E+01_JPRB, 0.25962E+01_JPRB, 0.24938E+01_JPRB, 0.21380E+01_JPRB ,&
&  0.18482E+01_JPRB, 0.17076E+01_JPRB, 0.15285E+01_JPRB, 0.13530E+01_JPRB, 0.12055E+01_JPRB, 0.11185E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.10362E+01_JPRB, 0.97561E+00_JPRB, 0.92596E+00_JPRB, 0.93306E+00_JPRB, 0.99398E+00_JPRB, 0.10662E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10058E+01_JPRB, 0.10137E+01_JPRB, 0.10618E+01_JPRB, 0.90743E+00_JPRB, 0.10399E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10994E+01_JPRB, 0.11771E+01_JPRB, 0.11961E+01_JPRB, 0.14197E+01_JPRB, 0.14801E+01_JPRB, 0.14296E+01_JPRB, 0.13749E+01_JPRB ,&
&  0.12920E+01_JPRB, 0.10746E+01_JPRB, 0.92461E+00_JPRB, 0.97433E+00_JPRB, 0.10375E+01_JPRB, 0.10549E+01_JPRB, 0.79174E+00_JPRB ,&
&  0.78318E+00_JPRB, 0.92648E+00_JPRB, 0.92274E+00_JPRB, 0.85098E+00_JPRB, 0.95790E+00_JPRB, 0.94102E+00_JPRB, 0.77616E+00_JPRB ,&
&  0.48113E+00_JPRB, 0.41569E+00_JPRB, 0.42899E+00_JPRB, 0.43517E+00_JPRB, 0.42310E+00_JPRB, 0.45717E+00_JPRB, 0.77513E+00_JPRB ,&
&  0.71888E+00_JPRB, 0.88675E+00_JPRB, 0.87456E+00_JPRB, 0.83803E+00_JPRB, 0.86160E+00_JPRB, 0.94879E+00_JPRB, 0.11008E+01_JPRB ,&
&  0.90405E+00_JPRB, 0.84137E+00_JPRB, 0.77144E+00_JPRB, 0.72042E+00_JPRB, 0.70276E+00_JPRB, 0.69234E+00_JPRB, 0.68877E+00_JPRB ,&
&  0.67727E+00_JPRB, 0.66173E+00_JPRB, 0.65483E+00_JPRB, 0.65216E+00_JPRB, 0.64389E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.62040E+00_JPRB, 0.61331E+00_JPRB, 0.60559E+00_JPRB, 0.59632E+00_JPRB, 0.58191E+00_JPRB, 0.56629E+00_JPRB, 0.55340E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.51887E+00_JPRB, 0.49855E+00_JPRB, 0.48366E+00_JPRB, 0.47526E+00_JPRB, 0.47464E+00_JPRB, 0.48328E+00_JPRB ,&
&  0.49790E+00_JPRB, 0.54701E+00_JPRB, 0.74502E+00_JPRB, 0.70552E+00_JPRB, 0.71882E+00_JPRB, 0.67188E+00_JPRB, 0.73050E+00_JPRB ,&
&  0.10478E+01_JPRB, 0.15352E+01_JPRB, 0.20874E+01_JPRB, 0.24216E+01_JPRB, 0.28021E+01_JPRB, 0.28459E+01_JPRB, 0.26183E+01_JPRB ,&
&  0.21894E+01_JPRB, 0.18670E+01_JPRB, 0.16068E+01_JPRB, 0.13977E+01_JPRB, 0.12222E+01_JPRB, 0.11164E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.99939E+00_JPRB, 0.94143E+00_JPRB, 0.93495E+00_JPRB, 0.10016E+01_JPRB, 0.10797E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10001E+01_JPRB, 0.98078E+00_JPRB, 0.10109E+01_JPRB, 0.90470E+00_JPRB, 0.82302E+00_JPRB, 0.94667E+00_JPRB ,&
&  0.10854E+01_JPRB, 0.12667E+01_JPRB, 0.12888E+01_JPRB, 0.15055E+01_JPRB, 0.15468E+01_JPRB, 0.15001E+01_JPRB, 0.16086E+01_JPRB ,&
&  0.15214E+01_JPRB, 0.14148E+01_JPRB, 0.10441E+01_JPRB, 0.10526E+01_JPRB, 0.11436E+01_JPRB, 0.91530E+00_JPRB, 0.83509E+00_JPRB ,&
&  0.97983E+00_JPRB, 0.93801E+00_JPRB, 0.88908E+00_JPRB, 0.80210E+00_JPRB, 0.96102E+00_JPRB, 0.83285E+00_JPRB, 0.50578E+00_JPRB ,&
&  0.44120E+00_JPRB, 0.41118E+00_JPRB, 0.38862E+00_JPRB, 0.36706E+00_JPRB, 0.34818E+00_JPRB, 0.38205E+00_JPRB, 0.57318E+00_JPRB ,&
&  0.63384E+00_JPRB, 0.83412E+00_JPRB, 0.87382E+00_JPRB, 0.84256E+00_JPRB, 0.82953E+00_JPRB, 0.95813E+00_JPRB, 0.11560E+01_JPRB ,&
&  0.95161E+00_JPRB, 0.92886E+00_JPRB, 0.83995E+00_JPRB, 0.77360E+00_JPRB, 0.72416E+00_JPRB, 0.69379E+00_JPRB, 0.68396E+00_JPRB ,&
&  0.67145E+00_JPRB, 0.65861E+00_JPRB, 0.64816E+00_JPRB, 0.63901E+00_JPRB, 0.62997E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.60907E+00_JPRB, 0.60151E+00_JPRB, 0.59887E+00_JPRB, 0.59512E+00_JPRB, 0.58465E+00_JPRB, 0.56655E+00_JPRB, 0.54966E+00_JPRB ,&
&  0.53547E+00_JPRB, 0.51106E+00_JPRB, 0.49367E+00_JPRB, 0.48093E+00_JPRB, 0.47112E+00_JPRB, 0.46907E+00_JPRB, 0.47479E+00_JPRB ,&
&  0.48621E+00_JPRB, 0.57548E+00_JPRB, 0.71843E+00_JPRB, 0.62694E+00_JPRB, 0.58886E+00_JPRB, 0.55502E+00_JPRB, 0.56019E+00_JPRB ,&
&  0.90784E+00_JPRB, 0.13230E+01_JPRB, 0.17770E+01_JPRB, 0.23778E+01_JPRB, 0.28509E+01_JPRB, 0.27478E+01_JPRB, 0.28390E+01_JPRB ,&
&  0.25029E+01_JPRB, 0.20658E+01_JPRB, 0.17395E+01_JPRB, 0.14935E+01_JPRB, 0.12808E+01_JPRB, 0.11459E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.10013E+01_JPRB, 0.95753E+00_JPRB, 0.93863E+00_JPRB, 0.99509E+00_JPRB, 0.11100E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10091E+01_JPRB, 0.96251E+00_JPRB, 0.96231E+00_JPRB, 0.97882E+00_JPRB, 0.96909E+00_JPRB, 0.91151E+00_JPRB ,&
&  0.91134E+00_JPRB, 0.10411E+01_JPRB, 0.14135E+01_JPRB, 0.14327E+01_JPRB, 0.15150E+01_JPRB, 0.16044E+01_JPRB, 0.16851E+01_JPRB ,&
&  0.16197E+01_JPRB, 0.15861E+01_JPRB, 0.12344E+01_JPRB, 0.12112E+01_JPRB, 0.10513E+01_JPRB, 0.87076E+00_JPRB, 0.10139E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.10610E+01_JPRB, 0.92570E+00_JPRB, 0.75828E+00_JPRB, 0.71425E+00_JPRB, 0.55062E+00_JPRB, 0.41513E+00_JPRB ,&
&  0.40320E+00_JPRB, 0.38852E+00_JPRB, 0.35702E+00_JPRB, 0.35233E+00_JPRB, 0.33936E+00_JPRB, 0.38266E+00_JPRB, 0.50327E+00_JPRB ,&
&  0.64740E+00_JPRB, 0.73548E+00_JPRB, 0.77113E+00_JPRB, 0.74472E+00_JPRB, 0.72469E+00_JPRB, 0.80704E+00_JPRB, 0.91782E+00_JPRB ,&
&  0.84854E+00_JPRB, 0.92091E+00_JPRB, 0.85573E+00_JPRB, 0.78328E+00_JPRB, 0.74347E+00_JPRB, 0.71174E+00_JPRB, 0.69248E+00_JPRB ,&
&  0.67848E+00_JPRB, 0.66718E+00_JPRB, 0.65518E+00_JPRB, 0.63687E+00_JPRB, 0.62268E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.60376E+00_JPRB, 0.59080E+00_JPRB, 0.58656E+00_JPRB, 0.58415E+00_JPRB, 0.57964E+00_JPRB, 0.56789E+00_JPRB, 0.55194E+00_JPRB ,&
&  0.53298E+00_JPRB, 0.51101E+00_JPRB, 0.49664E+00_JPRB, 0.48581E+00_JPRB, 0.47284E+00_JPRB, 0.47110E+00_JPRB, 0.47743E+00_JPRB ,&
&  0.50070E+00_JPRB, 0.69617E+00_JPRB, 0.53439E+00_JPRB, 0.45439E+00_JPRB, 0.50767E+00_JPRB, 0.52452E+00_JPRB, 0.49239E+00_JPRB ,&
&  0.80325E+00_JPRB, 0.12243E+01_JPRB, 0.15822E+01_JPRB, 0.23545E+01_JPRB, 0.31967E+01_JPRB, 0.30259E+01_JPRB, 0.29588E+01_JPRB ,&
&  0.28632E+01_JPRB, 0.22924E+01_JPRB, 0.19093E+01_JPRB, 0.16329E+01_JPRB, 0.13690E+01_JPRB, 0.11848E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.99014E+00_JPRB, 0.98644E+00_JPRB, 0.96077E+00_JPRB, 0.96409E+00_JPRB, 0.99288E+00_JPRB, 0.10804E+01_JPRB, 0.11748E+01_JPRB ,&
&  0.11403E+01_JPRB, 0.10572E+01_JPRB, 0.99047E+00_JPRB, 0.99039E+00_JPRB, 0.95205E+00_JPRB, 0.94196E+00_JPRB, 0.11214E+01_JPRB ,&
&  0.11276E+01_JPRB, 0.12906E+01_JPRB, 0.14056E+01_JPRB, 0.16184E+01_JPRB, 0.17738E+01_JPRB, 0.17213E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.14265E+01_JPRB, 0.12531E+01_JPRB, 0.12405E+01_JPRB, 0.11588E+01_JPRB, 0.95774E+00_JPRB, 0.10826E+01_JPRB, 0.15862E+01_JPRB ,&
&  0.12194E+01_JPRB, 0.13261E+01_JPRB, 0.13659E+01_JPRB, 0.13348E+01_JPRB, 0.69418E+00_JPRB, 0.50589E+00_JPRB, 0.72250E+00_JPRB ,&
&  0.75765E+00_JPRB, 0.52270E+00_JPRB, 0.40316E+00_JPRB, 0.46035E+00_JPRB, 0.38442E+00_JPRB, 0.43775E+00_JPRB, 0.54902E+00_JPRB ,&
&  0.59375E+00_JPRB, 0.64136E+00_JPRB, 0.74720E+00_JPRB, 0.68846E+00_JPRB, 0.69453E+00_JPRB, 0.78373E+00_JPRB, 0.80381E+00_JPRB ,&
&  0.79456E+00_JPRB, 0.87177E+00_JPRB, 0.87878E+00_JPRB, 0.78533E+00_JPRB, 0.73041E+00_JPRB, 0.69741E+00_JPRB, 0.68318E+00_JPRB ,&
&  0.67885E+00_JPRB, 0.67074E+00_JPRB, 0.66007E+00_JPRB, 0.64262E+00_JPRB, 0.62051E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.58731E+00_JPRB, 0.57472E+00_JPRB, 0.56669E+00_JPRB, 0.56644E+00_JPRB, 0.57005E+00_JPRB, 0.56926E+00_JPRB, 0.55633E+00_JPRB ,&
&  0.53697E+00_JPRB, 0.51740E+00_JPRB, 0.49924E+00_JPRB, 0.48967E+00_JPRB, 0.47759E+00_JPRB, 0.47581E+00_JPRB, 0.48309E+00_JPRB ,&
&  0.50780E+00_JPRB, 0.54425E+00_JPRB, 0.41428E+00_JPRB, 0.38979E+00_JPRB, 0.51465E+00_JPRB, 0.50688E+00_JPRB, 0.52912E+00_JPRB ,&
&  0.77362E+00_JPRB, 0.10561E+01_JPRB, 0.14122E+01_JPRB, 0.21542E+01_JPRB, 0.25638E+01_JPRB, 0.29330E+01_JPRB, 0.29654E+01_JPRB ,&
&  0.30046E+01_JPRB, 0.24741E+01_JPRB, 0.19978E+01_JPRB, 0.16390E+01_JPRB, 0.13649E+01_JPRB, 0.11938E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.98040E+00_JPRB, 0.93149E+00_JPRB, 0.94250E+00_JPRB, 0.94414E+00_JPRB, 0.98223E+00_JPRB, 0.10283E+01_JPRB, 0.11087E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.10739E+01_JPRB, 0.98215E+00_JPRB, 0.97355E+00_JPRB, 0.89279E+00_JPRB, 0.84962E+00_JPRB, 0.94393E+00_JPRB ,&
&  0.11541E+01_JPRB, 0.13193E+01_JPRB, 0.14807E+01_JPRB, 0.16612E+01_JPRB, 0.19033E+01_JPRB, 0.18049E+01_JPRB, 0.20576E+01_JPRB ,&
&  0.17237E+01_JPRB, 0.15138E+01_JPRB, 0.13139E+01_JPRB, 0.10764E+01_JPRB, 0.11806E+01_JPRB, 0.18162E+01_JPRB, 0.18110E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.16245E+01_JPRB, 0.15451E+01_JPRB, 0.12837E+01_JPRB, 0.10408E+01_JPRB, 0.73209E+00_JPRB, 0.81134E+00_JPRB ,&
&  0.78432E+00_JPRB, 0.78967E+00_JPRB, 0.69771E+00_JPRB, 0.57613E+00_JPRB, 0.47882E+00_JPRB, 0.55534E+00_JPRB, 0.55247E+00_JPRB ,&
&  0.54940E+00_JPRB, 0.55289E+00_JPRB, 0.63532E+00_JPRB, 0.62794E+00_JPRB, 0.62921E+00_JPRB, 0.65770E+00_JPRB, 0.73376E+00_JPRB ,&
&  0.71966E+00_JPRB, 0.75861E+00_JPRB, 0.77547E+00_JPRB, 0.75018E+00_JPRB, 0.72724E+00_JPRB, 0.70352E+00_JPRB, 0.67822E+00_JPRB ,&
&  0.66156E+00_JPRB, 0.64877E+00_JPRB, 0.63056E+00_JPRB, 0.61669E+00_JPRB, 0.60090E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.56026E+00_JPRB, 0.54843E+00_JPRB, 0.54053E+00_JPRB, 0.53834E+00_JPRB, 0.54447E+00_JPRB, 0.55492E+00_JPRB, 0.55336E+00_JPRB ,&
&  0.54036E+00_JPRB, 0.52284E+00_JPRB, 0.50767E+00_JPRB, 0.49596E+00_JPRB, 0.48706E+00_JPRB, 0.48564E+00_JPRB, 0.48871E+00_JPRB ,&
&  0.51028E+00_JPRB, 0.47209E+00_JPRB, 0.35816E+00_JPRB, 0.38164E+00_JPRB, 0.46356E+00_JPRB, 0.46131E+00_JPRB, 0.57472E+00_JPRB ,&
&  0.71634E+00_JPRB, 0.90207E+00_JPRB, 0.11293E+01_JPRB, 0.16882E+01_JPRB, 0.20971E+01_JPRB, 0.23882E+01_JPRB, 0.24910E+01_JPRB ,&
&  0.23753E+01_JPRB, 0.22014E+01_JPRB, 0.19516E+01_JPRB, 0.16838E+01_JPRB, 0.13878E+01_JPRB, 0.12565E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.99246E+00_JPRB, 0.93884E+00_JPRB, 0.94076E+00_JPRB, 0.95913E+00_JPRB, 0.99684E+00_JPRB, 0.10741E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.10976E+01_JPRB, 0.98818E+00_JPRB, 0.94777E+00_JPRB, 0.78415E+00_JPRB, 0.76790E+00_JPRB, 0.84883E+00_JPRB ,&
&  0.12974E+01_JPRB, 0.15016E+01_JPRB, 0.15619E+01_JPRB, 0.19249E+01_JPRB, 0.17369E+01_JPRB, 0.17648E+01_JPRB, 0.21211E+01_JPRB ,&
&  0.23348E+01_JPRB, 0.23505E+01_JPRB, 0.24020E+01_JPRB, 0.18351E+01_JPRB, 0.14844E+01_JPRB, 0.20743E+01_JPRB, 0.19562E+01_JPRB ,&
&  0.17797E+01_JPRB, 0.16517E+01_JPRB, 0.14342E+01_JPRB, 0.14051E+01_JPRB, 0.11007E+01_JPRB, 0.88639E+00_JPRB, 0.63628E+00_JPRB ,&
&  0.57584E+00_JPRB, 0.53301E+00_JPRB, 0.58208E+00_JPRB, 0.54391E+00_JPRB, 0.53902E+00_JPRB, 0.51642E+00_JPRB, 0.49691E+00_JPRB ,&
&  0.50214E+00_JPRB, 0.49705E+00_JPRB, 0.51196E+00_JPRB, 0.55680E+00_JPRB, 0.58963E+00_JPRB, 0.55760E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.68715E+00_JPRB, 0.71369E+00_JPRB, 0.70019E+00_JPRB, 0.71324E+00_JPRB, 0.69933E+00_JPRB, 0.69144E+00_JPRB, 0.67615E+00_JPRB ,&
&  0.65214E+00_JPRB, 0.62991E+00_JPRB, 0.61030E+00_JPRB, 0.59112E+00_JPRB, 0.57464E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.50370E+00_JPRB, 0.50056E+00_JPRB, 0.50208E+00_JPRB, 0.50513E+00_JPRB, 0.51067E+00_JPRB, 0.52136E+00_JPRB, 0.52418E+00_JPRB ,&
&  0.52092E+00_JPRB, 0.50896E+00_JPRB, 0.49402E+00_JPRB, 0.48316E+00_JPRB, 0.47973E+00_JPRB, 0.48395E+00_JPRB, 0.49132E+00_JPRB ,&
&  0.50034E+00_JPRB, 0.51844E+00_JPRB, 0.40789E+00_JPRB, 0.36251E+00_JPRB, 0.36306E+00_JPRB, 0.44788E+00_JPRB, 0.58671E+00_JPRB ,&
&  0.70458E+00_JPRB, 0.76885E+00_JPRB, 0.96735E+00_JPRB, 0.13233E+01_JPRB, 0.16057E+01_JPRB, 0.17647E+01_JPRB, 0.19227E+01_JPRB ,&
&  0.22657E+01_JPRB, 0.17844E+01_JPRB, 0.17075E+01_JPRB, 0.15143E+01_JPRB, 0.13714E+01_JPRB, 0.12704E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.10160E+01_JPRB, 0.93555E+00_JPRB, 0.89510E+00_JPRB, 0.87714E+00_JPRB, 0.87923E+00_JPRB, 0.92185E+00_JPRB ,&
&  0.98053E+00_JPRB, 0.10252E+01_JPRB, 0.10108E+01_JPRB, 0.98579E+00_JPRB, 0.96932E+00_JPRB, 0.96908E+00_JPRB, 0.10373E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.11589E+01_JPRB, 0.16069E+01_JPRB, 0.18841E+01_JPRB, 0.21655E+01_JPRB, 0.22827E+01_JPRB, 0.23522E+01_JPRB ,&
&  0.24883E+01_JPRB, 0.24409E+01_JPRB, 0.24494E+01_JPRB, 0.21643E+01_JPRB, 0.23609E+01_JPRB, 0.22816E+01_JPRB, 0.20551E+01_JPRB ,&
&  0.17982E+01_JPRB, 0.16189E+01_JPRB, 0.13645E+01_JPRB, 0.12310E+01_JPRB, 0.11877E+01_JPRB, 0.11646E+01_JPRB, 0.87850E+00_JPRB ,&
&  0.65806E+00_JPRB, 0.68806E+00_JPRB, 0.60610E+00_JPRB, 0.54289E+00_JPRB, 0.48239E+00_JPRB, 0.49415E+00_JPRB, 0.50389E+00_JPRB ,&
&  0.50670E+00_JPRB, 0.50692E+00_JPRB, 0.49634E+00_JPRB, 0.53262E+00_JPRB, 0.58380E+00_JPRB, 0.56989E+00_JPRB, 0.60159E+00_JPRB ,&
&  0.57853E+00_JPRB, 0.62014E+00_JPRB, 0.62462E+00_JPRB, 0.61084E+00_JPRB, 0.64945E+00_JPRB, 0.65940E+00_JPRB, 0.64163E+00_JPRB ,&
&  0.61047E+00_JPRB, 0.58216E+00_JPRB, 0.55615E+00_JPRB, 0.53209E+00_JPRB, 0.51429E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.49195E+00_JPRB, 0.48864E+00_JPRB, 0.48831E+00_JPRB, 0.49084E+00_JPRB, 0.49511E+00_JPRB, 0.50705E+00_JPRB, 0.51422E+00_JPRB ,&
&  0.51362E+00_JPRB, 0.50976E+00_JPRB, 0.49860E+00_JPRB, 0.48736E+00_JPRB, 0.48189E+00_JPRB, 0.48214E+00_JPRB, 0.48968E+00_JPRB ,&
&  0.51132E+00_JPRB, 0.66691E+00_JPRB, 0.48999E+00_JPRB, 0.44736E+00_JPRB, 0.42061E+00_JPRB, 0.51738E+00_JPRB, 0.60556E+00_JPRB ,&
&  0.67226E+00_JPRB, 0.68807E+00_JPRB, 0.73294E+00_JPRB, 0.98634E+00_JPRB, 0.12425E+01_JPRB, 0.12801E+01_JPRB, 0.14420E+01_JPRB ,&
&  0.15037E+01_JPRB, 0.14957E+01_JPRB, 0.13356E+01_JPRB, 0.13570E+01_JPRB, 0.11940E+01_JPRB, 0.10329E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.99098E+00_JPRB, 0.95538E+00_JPRB, 0.90425E+00_JPRB, 0.84253E+00_JPRB, 0.81279E+00_JPRB, 0.79266E+00_JPRB, 0.79591E+00_JPRB ,&
&  0.81906E+00_JPRB, 0.85156E+00_JPRB, 0.89890E+00_JPRB, 0.93999E+00_JPRB, 0.96967E+00_JPRB, 0.10496E+01_JPRB, 0.11009E+01_JPRB ,&
&  0.13711E+01_JPRB, 0.14921E+01_JPRB, 0.13748E+01_JPRB, 0.18685E+01_JPRB, 0.26871E+01_JPRB, 0.26748E+01_JPRB, 0.24721E+01_JPRB ,&
&  0.25963E+01_JPRB, 0.25506E+01_JPRB, 0.25409E+01_JPRB, 0.25936E+01_JPRB, 0.25061E+01_JPRB, 0.23223E+01_JPRB, 0.20494E+01_JPRB ,&
&  0.17354E+01_JPRB, 0.15127E+01_JPRB, 0.13314E+01_JPRB, 0.12727E+01_JPRB, 0.11157E+01_JPRB, 0.10064E+01_JPRB, 0.91843E+00_JPRB ,&
&  0.77357E+00_JPRB, 0.65018E+00_JPRB, 0.49379E+00_JPRB, 0.51403E+00_JPRB, 0.42429E+00_JPRB, 0.43435E+00_JPRB, 0.48251E+00_JPRB ,&
&  0.47772E+00_JPRB, 0.51463E+00_JPRB, 0.53107E+00_JPRB, 0.51170E+00_JPRB, 0.56370E+00_JPRB, 0.57113E+00_JPRB, 0.58675E+00_JPRB ,&
&  0.58816E+00_JPRB, 0.53437E+00_JPRB, 0.56145E+00_JPRB, 0.54796E+00_JPRB, 0.55854E+00_JPRB, 0.55585E+00_JPRB, 0.55081E+00_JPRB ,&
&  0.54034E+00_JPRB, 0.52551E+00_JPRB, 0.51172E+00_JPRB, 0.50195E+00_JPRB, 0.49648E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.45716E+00_JPRB, 0.45247E+00_JPRB, 0.44676E+00_JPRB, 0.45151E+00_JPRB, 0.46452E+00_JPRB, 0.48030E+00_JPRB, 0.49248E+00_JPRB ,&
&  0.49331E+00_JPRB, 0.49375E+00_JPRB, 0.48728E+00_JPRB, 0.47830E+00_JPRB, 0.47353E+00_JPRB, 0.46777E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.66771E+00_JPRB, 0.52657E+00_JPRB, 0.44033E+00_JPRB, 0.52333E+00_JPRB, 0.54312E+00_JPRB, 0.60775E+00_JPRB ,&
&  0.60528E+00_JPRB, 0.66059E+00_JPRB, 0.66799E+00_JPRB, 0.72761E+00_JPRB, 0.91700E+00_JPRB, 0.10145E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10465E+01_JPRB, 0.10541E+01_JPRB, 0.10835E+01_JPRB, 0.11066E+01_JPRB, 0.97772E+00_JPRB, 0.94661E+00_JPRB ,&
&  0.88599E+00_JPRB, 0.84200E+00_JPRB, 0.79061E+00_JPRB, 0.76401E+00_JPRB, 0.75094E+00_JPRB, 0.74755E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.77680E+00_JPRB, 0.79503E+00_JPRB, 0.81608E+00_JPRB, 0.86765E+00_JPRB, 0.93178E+00_JPRB, 0.11556E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.18509E+01_JPRB, 0.21482E+01_JPRB, 0.24584E+01_JPRB, 0.28865E+01_JPRB, 0.29202E+01_JPRB, 0.28496E+01_JPRB, 0.27595E+01_JPRB ,&
&  0.26476E+01_JPRB, 0.25209E+01_JPRB, 0.24719E+01_JPRB, 0.24109E+01_JPRB, 0.23370E+01_JPRB, 0.22138E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.14473E+01_JPRB, 0.13390E+01_JPRB, 0.11993E+01_JPRB, 0.10747E+01_JPRB, 0.95612E+00_JPRB, 0.90957E+00_JPRB ,&
&  0.95748E+00_JPRB, 0.55615E+00_JPRB, 0.45599E+00_JPRB, 0.51447E+00_JPRB, 0.44474E+00_JPRB, 0.46356E+00_JPRB, 0.52528E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.51616E+00_JPRB, 0.53223E+00_JPRB, 0.51000E+00_JPRB, 0.51216E+00_JPRB, 0.56939E+00_JPRB, 0.53208E+00_JPRB ,&
&  0.50430E+00_JPRB, 0.49749E+00_JPRB, 0.51111E+00_JPRB, 0.50307E+00_JPRB, 0.50051E+00_JPRB, 0.48980E+00_JPRB, 0.47414E+00_JPRB ,&
&  0.46499E+00_JPRB, 0.46503E+00_JPRB, 0.46168E+00_JPRB, 0.46017E+00_JPRB, 0.45925E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.43387E+00_JPRB, 0.43265E+00_JPRB, 0.42597E+00_JPRB, 0.43082E+00_JPRB, 0.44946E+00_JPRB, 0.46889E+00_JPRB, 0.48191E+00_JPRB ,&
&  0.48482E+00_JPRB, 0.48749E+00_JPRB, 0.48164E+00_JPRB, 0.47521E+00_JPRB, 0.47002E+00_JPRB, 0.45632E+00_JPRB, 0.47618E+00_JPRB ,&
&  0.40134E+00_JPRB, 0.49614E+00_JPRB, 0.41453E+00_JPRB, 0.43215E+00_JPRB, 0.58159E+00_JPRB, 0.53558E+00_JPRB, 0.55682E+00_JPRB ,&
&  0.55709E+00_JPRB, 0.58013E+00_JPRB, 0.58390E+00_JPRB, 0.62193E+00_JPRB, 0.74976E+00_JPRB, 0.85750E+00_JPRB, 0.88911E+00_JPRB ,&
&  0.84889E+00_JPRB, 0.80512E+00_JPRB, 0.83965E+00_JPRB, 0.86310E+00_JPRB, 0.86814E+00_JPRB, 0.89864E+00_JPRB, 0.90321E+00_JPRB ,&
&  0.84099E+00_JPRB, 0.79177E+00_JPRB, 0.73512E+00_JPRB, 0.70906E+00_JPRB, 0.69613E+00_JPRB, 0.68887E+00_JPRB, 0.68012E+00_JPRB ,&
&  0.68399E+00_JPRB, 0.70333E+00_JPRB, 0.73083E+00_JPRB, 0.81329E+00_JPRB, 0.90419E+00_JPRB, 0.14993E+01_JPRB, 0.24223E+01_JPRB ,&
&  0.26978E+01_JPRB, 0.26797E+01_JPRB, 0.29065E+01_JPRB, 0.32909E+01_JPRB, 0.31681E+01_JPRB, 0.29608E+01_JPRB, 0.26862E+01_JPRB ,&
&  0.25371E+01_JPRB, 0.24129E+01_JPRB, 0.23223E+01_JPRB, 0.22686E+01_JPRB, 0.21151E+01_JPRB, 0.20486E+01_JPRB, 0.18522E+01_JPRB ,&
&  0.15682E+01_JPRB, 0.14273E+01_JPRB, 0.13519E+01_JPRB, 0.13127E+01_JPRB, 0.12214E+01_JPRB, 0.11943E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.87223E+00_JPRB, 0.65159E+00_JPRB, 0.59363E+00_JPRB, 0.49143E+00_JPRB, 0.50718E+00_JPRB, 0.62687E+00_JPRB ,&
&  0.60553E+00_JPRB, 0.54466E+00_JPRB, 0.50214E+00_JPRB, 0.47678E+00_JPRB, 0.46929E+00_JPRB, 0.50963E+00_JPRB, 0.47803E+00_JPRB ,&
&  0.44228E+00_JPRB, 0.46184E+00_JPRB, 0.46365E+00_JPRB, 0.45947E+00_JPRB, 0.44960E+00_JPRB, 0.43849E+00_JPRB, 0.40840E+00_JPRB ,&
&  0.40957E+00_JPRB, 0.41006E+00_JPRB, 0.41692E+00_JPRB, 0.42691E+00_JPRB, 0.43188E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.42636E+00_JPRB, 0.42394E+00_JPRB, 0.41779E+00_JPRB, 0.42429E+00_JPRB, 0.44361E+00_JPRB, 0.46182E+00_JPRB, 0.47422E+00_JPRB ,&
&  0.47685E+00_JPRB, 0.47627E+00_JPRB, 0.47431E+00_JPRB, 0.47208E+00_JPRB, 0.46269E+00_JPRB, 0.44905E+00_JPRB, 0.40572E+00_JPRB ,&
&  0.35156E+00_JPRB, 0.41689E+00_JPRB, 0.39617E+00_JPRB, 0.41932E+00_JPRB, 0.48650E+00_JPRB, 0.46563E+00_JPRB, 0.45742E+00_JPRB ,&
&  0.47689E+00_JPRB, 0.48563E+00_JPRB, 0.51790E+00_JPRB, 0.55349E+00_JPRB, 0.61142E+00_JPRB, 0.72729E+00_JPRB, 0.75909E+00_JPRB ,&
&  0.70635E+00_JPRB, 0.67624E+00_JPRB, 0.70680E+00_JPRB, 0.76611E+00_JPRB, 0.82742E+00_JPRB, 0.87551E+00_JPRB, 0.86329E+00_JPRB ,&
&  0.82492E+00_JPRB, 0.77697E+00_JPRB, 0.70586E+00_JPRB, 0.65852E+00_JPRB, 0.63140E+00_JPRB, 0.61551E+00_JPRB, 0.60583E+00_JPRB ,&
&  0.60237E+00_JPRB, 0.61490E+00_JPRB, 0.64978E+00_JPRB, 0.71200E+00_JPRB, 0.86756E+00_JPRB, 0.13952E+01_JPRB, 0.18595E+01_JPRB ,&
&  0.19038E+01_JPRB, 0.19801E+01_JPRB, 0.21484E+01_JPRB, 0.23032E+01_JPRB, 0.24447E+01_JPRB, 0.23878E+01_JPRB, 0.23156E+01_JPRB ,&
&  0.22783E+01_JPRB, 0.21775E+01_JPRB, 0.22447E+01_JPRB, 0.21542E+01_JPRB, 0.20007E+01_JPRB, 0.18886E+01_JPRB, 0.16906E+01_JPRB ,&
&  0.15273E+01_JPRB, 0.14300E+01_JPRB, 0.13766E+01_JPRB, 0.13041E+01_JPRB, 0.12408E+01_JPRB, 0.12272E+01_JPRB, 0.11255E+01_JPRB ,&
&  0.10805E+01_JPRB, 0.10229E+01_JPRB, 0.85199E+00_JPRB, 0.72995E+00_JPRB, 0.62631E+00_JPRB, 0.63445E+00_JPRB, 0.61307E+00_JPRB ,&
&  0.55824E+00_JPRB, 0.47736E+00_JPRB, 0.47182E+00_JPRB, 0.46153E+00_JPRB, 0.46097E+00_JPRB, 0.45240E+00_JPRB, 0.42431E+00_JPRB ,&
&  0.42473E+00_JPRB, 0.43511E+00_JPRB, 0.41362E+00_JPRB, 0.40425E+00_JPRB, 0.39241E+00_JPRB, 0.38301E+00_JPRB, 0.34711E+00_JPRB ,&
&  0.34605E+00_JPRB, 0.37124E+00_JPRB, 0.38931E+00_JPRB, 0.40843E+00_JPRB, 0.42092E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.43280E+00_JPRB, 0.43252E+00_JPRB, 0.43402E+00_JPRB, 0.43442E+00_JPRB, 0.44047E+00_JPRB, 0.45896E+00_JPRB, 0.44853E+00_JPRB ,&
&  0.47806E+00_JPRB, 0.48098E+00_JPRB, 0.47384E+00_JPRB, 0.47110E+00_JPRB, 0.45430E+00_JPRB, 0.38815E+00_JPRB, 0.30949E+00_JPRB ,&
&  0.29693E+00_JPRB, 0.32834E+00_JPRB, 0.36509E+00_JPRB, 0.38745E+00_JPRB, 0.41639E+00_JPRB, 0.41698E+00_JPRB, 0.41545E+00_JPRB ,&
&  0.41683E+00_JPRB, 0.42163E+00_JPRB, 0.44508E+00_JPRB, 0.48501E+00_JPRB, 0.52510E+00_JPRB, 0.58849E+00_JPRB, 0.65715E+00_JPRB ,&
&  0.61501E+00_JPRB, 0.60491E+00_JPRB, 0.64582E+00_JPRB, 0.63024E+00_JPRB, 0.72649E+00_JPRB, 0.75024E+00_JPRB, 0.74913E+00_JPRB ,&
&  0.73092E+00_JPRB, 0.70084E+00_JPRB, 0.65760E+00_JPRB, 0.63016E+00_JPRB, 0.60248E+00_JPRB, 0.57912E+00_JPRB, 0.56472E+00_JPRB ,&
&  0.56751E+00_JPRB, 0.57870E+00_JPRB, 0.60795E+00_JPRB, 0.65850E+00_JPRB, 0.72846E+00_JPRB, 0.10357E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.14239E+01_JPRB, 0.16584E+01_JPRB, 0.19046E+01_JPRB, 0.18327E+01_JPRB, 0.19712E+01_JPRB, 0.20573E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.20665E+01_JPRB, 0.19729E+01_JPRB, 0.19463E+01_JPRB, 0.18591E+01_JPRB, 0.17614E+01_JPRB, 0.16372E+01_JPRB, 0.14865E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.14359E+01_JPRB, 0.14085E+01_JPRB, 0.13092E+01_JPRB, 0.11701E+01_JPRB, 0.10787E+01_JPRB, 0.98467E+00_JPRB ,&
&  0.93709E+00_JPRB, 0.91747E+00_JPRB, 0.87059E+00_JPRB, 0.82407E+00_JPRB, 0.75767E+00_JPRB, 0.69292E+00_JPRB, 0.64110E+00_JPRB ,&
&  0.58701E+00_JPRB, 0.51335E+00_JPRB, 0.47589E+00_JPRB, 0.46681E+00_JPRB, 0.45702E+00_JPRB, 0.42187E+00_JPRB, 0.40944E+00_JPRB ,&
&  0.40483E+00_JPRB, 0.38343E+00_JPRB, 0.38324E+00_JPRB, 0.36947E+00_JPRB, 0.36232E+00_JPRB, 0.36199E+00_JPRB, 0.35717E+00_JPRB ,&
&  0.35261E+00_JPRB, 0.38434E+00_JPRB, 0.40412E+00_JPRB, 0.41895E+00_JPRB, 0.42707E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.43386E+00_JPRB, 0.44465E+00_JPRB, 0.45158E+00_JPRB, 0.44740E+00_JPRB, 0.44313E+00_JPRB, 0.43235E+00_JPRB, 0.44010E+00_JPRB ,&
&  0.47921E+00_JPRB, 0.43847E+00_JPRB, 0.47526E+00_JPRB, 0.37599E+00_JPRB, 0.33145E+00_JPRB, 0.31234E+00_JPRB, 0.29665E+00_JPRB ,&
&  0.29890E+00_JPRB, 0.31960E+00_JPRB, 0.33805E+00_JPRB, 0.35109E+00_JPRB, 0.37831E+00_JPRB, 0.38143E+00_JPRB, 0.38038E+00_JPRB ,&
&  0.38559E+00_JPRB, 0.39017E+00_JPRB, 0.41426E+00_JPRB, 0.42191E+00_JPRB, 0.44725E+00_JPRB, 0.48792E+00_JPRB, 0.55173E+00_JPRB ,&
&  0.52711E+00_JPRB, 0.53423E+00_JPRB, 0.59158E+00_JPRB, 0.60880E+00_JPRB, 0.62562E+00_JPRB, 0.61505E+00_JPRB, 0.59458E+00_JPRB ,&
&  0.57960E+00_JPRB, 0.48694E+00_JPRB, 0.50969E+00_JPRB, 0.49978E+00_JPRB, 0.48941E+00_JPRB, 0.47644E+00_JPRB, 0.45622E+00_JPRB ,&
&  0.44452E+00_JPRB, 0.45031E+00_JPRB, 0.47177E+00_JPRB, 0.50361E+00_JPRB, 0.55710E+00_JPRB, 0.69623E+00_JPRB, 0.86018E+00_JPRB ,&
&  0.10759E+01_JPRB, 0.10350E+01_JPRB, 0.13273E+01_JPRB, 0.14383E+01_JPRB, 0.15852E+01_JPRB, 0.17366E+01_JPRB, 0.18251E+01_JPRB ,&
&  0.19048E+01_JPRB, 0.17719E+01_JPRB, 0.16682E+01_JPRB, 0.15457E+01_JPRB, 0.14298E+01_JPRB, 0.13341E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.11678E+01_JPRB, 0.11959E+01_JPRB, 0.11282E+01_JPRB, 0.10181E+01_JPRB, 0.95090E+00_JPRB, 0.88625E+00_JPRB ,&
&  0.83371E+00_JPRB, 0.79865E+00_JPRB, 0.76193E+00_JPRB, 0.68657E+00_JPRB, 0.66695E+00_JPRB, 0.65187E+00_JPRB, 0.60834E+00_JPRB ,&
&  0.58584E+00_JPRB, 0.55159E+00_JPRB, 0.51231E+00_JPRB, 0.51504E+00_JPRB, 0.48040E+00_JPRB, 0.45151E+00_JPRB, 0.44085E+00_JPRB ,&
&  0.41436E+00_JPRB, 0.36601E+00_JPRB, 0.35341E+00_JPRB, 0.35158E+00_JPRB, 0.34421E+00_JPRB, 0.34582E+00_JPRB, 0.37268E+00_JPRB ,&
&  0.37420E+00_JPRB, 0.38520E+00_JPRB, 0.40940E+00_JPRB, 0.41557E+00_JPRB, 0.42699E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.42743E+00_JPRB, 0.44423E+00_JPRB, 0.45859E+00_JPRB, 0.45319E+00_JPRB, 0.44055E+00_JPRB, 0.42630E+00_JPRB, 0.42011E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.36828E+00_JPRB, 0.34010E+00_JPRB, 0.32484E+00_JPRB, 0.31813E+00_JPRB, 0.30460E+00_JPRB, 0.29089E+00_JPRB ,&
&  0.29024E+00_JPRB, 0.30052E+00_JPRB, 0.32068E+00_JPRB, 0.33643E+00_JPRB, 0.34940E+00_JPRB, 0.35744E+00_JPRB, 0.36193E+00_JPRB ,&
&  0.36758E+00_JPRB, 0.37425E+00_JPRB, 0.37868E+00_JPRB, 0.38476E+00_JPRB, 0.39218E+00_JPRB, 0.40845E+00_JPRB, 0.43141E+00_JPRB ,&
&  0.45077E+00_JPRB, 0.46257E+00_JPRB, 0.44820E+00_JPRB, 0.45121E+00_JPRB, 0.45758E+00_JPRB, 0.45739E+00_JPRB, 0.46365E+00_JPRB ,&
&  0.32822E+00_JPRB, 0.31677E+00_JPRB, 0.37518E+00_JPRB, 0.37598E+00_JPRB, 0.38283E+00_JPRB, 0.38276E+00_JPRB, 0.38194E+00_JPRB ,&
&  0.38772E+00_JPRB, 0.40460E+00_JPRB, 0.43790E+00_JPRB, 0.49395E+00_JPRB, 0.55653E+00_JPRB, 0.63292E+00_JPRB, 0.73808E+00_JPRB ,&
&  0.94176E+00_JPRB, 0.92492E+00_JPRB, 0.97335E+00_JPRB, 0.10964E+01_JPRB, 0.13330E+01_JPRB, 0.14637E+01_JPRB, 0.15016E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.14083E+01_JPRB, 0.13413E+01_JPRB, 0.13068E+01_JPRB, 0.12550E+01_JPRB, 0.11837E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.10563E+01_JPRB, 0.10029E+01_JPRB, 0.97331E+00_JPRB, 0.94886E+00_JPRB, 0.89000E+00_JPRB, 0.85087E+00_JPRB, 0.81464E+00_JPRB ,&
&  0.77397E+00_JPRB, 0.72725E+00_JPRB, 0.67415E+00_JPRB, 0.63140E+00_JPRB, 0.60013E+00_JPRB, 0.59312E+00_JPRB, 0.58532E+00_JPRB ,&
&  0.56955E+00_JPRB, 0.55830E+00_JPRB, 0.54445E+00_JPRB, 0.53163E+00_JPRB, 0.49988E+00_JPRB, 0.46525E+00_JPRB, 0.44641E+00_JPRB ,&
&  0.40059E+00_JPRB, 0.36567E+00_JPRB, 0.32703E+00_JPRB, 0.31954E+00_JPRB, 0.32957E+00_JPRB, 0.33890E+00_JPRB, 0.34519E+00_JPRB ,&
&  0.35991E+00_JPRB, 0.36889E+00_JPRB, 0.37198E+00_JPRB, 0.38222E+00_JPRB, 0.40012E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.41028E+00_JPRB, 0.42284E+00_JPRB, 0.43334E+00_JPRB, 0.43966E+00_JPRB, 0.42815E+00_JPRB, 0.42106E+00_JPRB, 0.41615E+00_JPRB ,&
&  0.42072E+00_JPRB, 0.38135E+00_JPRB, 0.35532E+00_JPRB, 0.33445E+00_JPRB, 0.32100E+00_JPRB, 0.30752E+00_JPRB, 0.30419E+00_JPRB ,&
&  0.30813E+00_JPRB, 0.31529E+00_JPRB, 0.32424E+00_JPRB, 0.33210E+00_JPRB, 0.33797E+00_JPRB, 0.34007E+00_JPRB, 0.34442E+00_JPRB ,&
&  0.35157E+00_JPRB, 0.35630E+00_JPRB, 0.35436E+00_JPRB, 0.35363E+00_JPRB, 0.35574E+00_JPRB, 0.36416E+00_JPRB, 0.36725E+00_JPRB ,&
&  0.36748E+00_JPRB, 0.36347E+00_JPRB, 0.36135E+00_JPRB, 0.37042E+00_JPRB, 0.37335E+00_JPRB, 0.37072E+00_JPRB, 0.33459E+00_JPRB ,&
&  0.27894E+00_JPRB, 0.25910E+00_JPRB, 0.27026E+00_JPRB, 0.32873E+00_JPRB, 0.34453E+00_JPRB, 0.35420E+00_JPRB, 0.36841E+00_JPRB ,&
&  0.36057E+00_JPRB, 0.36431E+00_JPRB, 0.39891E+00_JPRB, 0.48658E+00_JPRB, 0.54845E+00_JPRB, 0.61507E+00_JPRB, 0.69268E+00_JPRB ,&
&  0.81860E+00_JPRB, 0.92412E+00_JPRB, 0.10321E+01_JPRB, 0.98174E+00_JPRB, 0.10962E+01_JPRB, 0.13630E+01_JPRB, 0.12879E+01_JPRB ,&
&  0.12156E+01_JPRB, 0.11676E+01_JPRB, 0.11902E+01_JPRB, 0.11534E+01_JPRB, 0.11056E+01_JPRB, 0.10305E+01_JPRB, 0.96487E+00_JPRB ,&
&  0.90095E+00_JPRB, 0.84576E+00_JPRB, 0.81187E+00_JPRB, 0.81793E+00_JPRB, 0.79981E+00_JPRB, 0.77421E+00_JPRB, 0.75362E+00_JPRB ,&
&  0.71840E+00_JPRB, 0.66534E+00_JPRB, 0.61499E+00_JPRB, 0.57411E+00_JPRB, 0.55785E+00_JPRB, 0.55464E+00_JPRB, 0.54910E+00_JPRB ,&
&  0.53641E+00_JPRB, 0.53174E+00_JPRB, 0.53129E+00_JPRB, 0.52814E+00_JPRB, 0.48289E+00_JPRB, 0.43032E+00_JPRB, 0.39303E+00_JPRB ,&
&  0.36565E+00_JPRB, 0.34690E+00_JPRB, 0.33144E+00_JPRB, 0.33171E+00_JPRB, 0.33579E+00_JPRB, 0.34084E+00_JPRB, 0.33675E+00_JPRB ,&
&  0.33449E+00_JPRB, 0.33386E+00_JPRB, 0.34346E+00_JPRB, 0.36584E+00_JPRB, 0.38905E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.36418E+00_JPRB, 0.38272E+00_JPRB, 0.40762E+00_JPRB, 0.42114E+00_JPRB, 0.42755E+00_JPRB, 0.40927E+00_JPRB, 0.39107E+00_JPRB ,&
&  0.37350E+00_JPRB, 0.35205E+00_JPRB, 0.34119E+00_JPRB, 0.33502E+00_JPRB, 0.33266E+00_JPRB, 0.33224E+00_JPRB, 0.33338E+00_JPRB ,&
&  0.33512E+00_JPRB, 0.33569E+00_JPRB, 0.33800E+00_JPRB, 0.34421E+00_JPRB, 0.34958E+00_JPRB, 0.35007E+00_JPRB, 0.34863E+00_JPRB ,&
&  0.34824E+00_JPRB, 0.34743E+00_JPRB, 0.34213E+00_JPRB, 0.33367E+00_JPRB, 0.33181E+00_JPRB, 0.34119E+00_JPRB, 0.34558E+00_JPRB ,&
&  0.34524E+00_JPRB, 0.33688E+00_JPRB, 0.33664E+00_JPRB, 0.34486E+00_JPRB, 0.34321E+00_JPRB, 0.33540E+00_JPRB, 0.30516E+00_JPRB ,&
&  0.26863E+00_JPRB, 0.24027E+00_JPRB, 0.23025E+00_JPRB, 0.24276E+00_JPRB, 0.27170E+00_JPRB, 0.32147E+00_JPRB, 0.33593E+00_JPRB ,&
&  0.34735E+00_JPRB, 0.36674E+00_JPRB, 0.39821E+00_JPRB, 0.44648E+00_JPRB, 0.51050E+00_JPRB, 0.57539E+00_JPRB, 0.63312E+00_JPRB ,&
&  0.70461E+00_JPRB, 0.78038E+00_JPRB, 0.88607E+00_JPRB, 0.85101E+00_JPRB, 0.88265E+00_JPRB, 0.96044E+00_JPRB, 0.97909E+00_JPRB ,&
&  0.95654E+00_JPRB, 0.94743E+00_JPRB, 0.95979E+00_JPRB, 0.95449E+00_JPRB, 0.91274E+00_JPRB, 0.86081E+00_JPRB, 0.80005E+00_JPRB ,&
&  0.75483E+00_JPRB, 0.72128E+00_JPRB, 0.70241E+00_JPRB, 0.70646E+00_JPRB, 0.72182E+00_JPRB, 0.71881E+00_JPRB, 0.69763E+00_JPRB ,&
&  0.66598E+00_JPRB, 0.61927E+00_JPRB, 0.56959E+00_JPRB, 0.52994E+00_JPRB, 0.52123E+00_JPRB, 0.53010E+00_JPRB, 0.53493E+00_JPRB ,&
&  0.53309E+00_JPRB, 0.52785E+00_JPRB, 0.51498E+00_JPRB, 0.50216E+00_JPRB, 0.45837E+00_JPRB, 0.42373E+00_JPRB, 0.39908E+00_JPRB ,&
&  0.38901E+00_JPRB, 0.38024E+00_JPRB, 0.37419E+00_JPRB, 0.37849E+00_JPRB, 0.37354E+00_JPRB, 0.37087E+00_JPRB, 0.36745E+00_JPRB ,&
&  0.36062E+00_JPRB, 0.35250E+00_JPRB, 0.35140E+00_JPRB, 0.35077E+00_JPRB, 0.35473E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.38862E+00_JPRB, 0.39254E+00_JPRB, 0.39621E+00_JPRB, 0.40414E+00_JPRB, 0.41185E+00_JPRB, 0.40864E+00_JPRB, 0.40086E+00_JPRB ,&
&  0.38709E+00_JPRB, 0.37289E+00_JPRB, 0.36309E+00_JPRB, 0.35698E+00_JPRB, 0.35562E+00_JPRB, 0.35675E+00_JPRB, 0.35828E+00_JPRB ,&
&  0.35971E+00_JPRB, 0.36080E+00_JPRB, 0.35898E+00_JPRB, 0.35322E+00_JPRB, 0.34920E+00_JPRB, 0.34827E+00_JPRB, 0.34868E+00_JPRB ,&
&  0.34973E+00_JPRB, 0.34828E+00_JPRB, 0.34470E+00_JPRB, 0.33938E+00_JPRB, 0.33293E+00_JPRB, 0.33817E+00_JPRB, 0.33545E+00_JPRB ,&
&  0.32829E+00_JPRB, 0.32740E+00_JPRB, 0.33425E+00_JPRB, 0.33266E+00_JPRB, 0.32796E+00_JPRB, 0.31826E+00_JPRB, 0.28453E+00_JPRB ,&
&  0.25332E+00_JPRB, 0.22969E+00_JPRB, 0.21680E+00_JPRB, 0.21407E+00_JPRB, 0.22172E+00_JPRB, 0.24129E+00_JPRB, 0.27333E+00_JPRB ,&
&  0.31962E+00_JPRB, 0.33985E+00_JPRB, 0.36570E+00_JPRB, 0.40130E+00_JPRB, 0.44676E+00_JPRB, 0.49837E+00_JPRB, 0.54459E+00_JPRB ,&
&  0.58194E+00_JPRB, 0.62624E+00_JPRB, 0.68812E+00_JPRB, 0.75663E+00_JPRB, 0.83487E+00_JPRB, 0.80512E+00_JPRB, 0.81227E+00_JPRB ,&
&  0.89124E+00_JPRB, 0.86599E+00_JPRB, 0.83529E+00_JPRB, 0.77370E+00_JPRB, 0.72121E+00_JPRB, 0.66852E+00_JPRB, 0.63339E+00_JPRB ,&
&  0.61760E+00_JPRB, 0.61920E+00_JPRB, 0.63349E+00_JPRB, 0.64915E+00_JPRB, 0.65323E+00_JPRB, 0.65218E+00_JPRB, 0.63026E+00_JPRB ,&
&  0.60735E+00_JPRB, 0.61771E+00_JPRB, 0.74116E+00_JPRB, 0.68371E+00_JPRB, 0.62683E+00_JPRB, 0.59951E+00_JPRB, 0.56558E+00_JPRB ,&
&  0.54426E+00_JPRB, 0.52666E+00_JPRB, 0.50534E+00_JPRB, 0.48791E+00_JPRB, 0.46200E+00_JPRB, 0.43511E+00_JPRB, 0.41441E+00_JPRB ,&
&  0.40846E+00_JPRB, 0.40571E+00_JPRB, 0.40240E+00_JPRB, 0.40066E+00_JPRB, 0.39665E+00_JPRB, 0.39467E+00_JPRB, 0.39303E+00_JPRB ,&
&  0.38738E+00_JPRB, 0.38285E+00_JPRB, 0.38287E+00_JPRB, 0.38299E+00_JPRB, 0.38502E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.38287E+00_JPRB, 0.38335E+00_JPRB, 0.38428E+00_JPRB, 0.38461E+00_JPRB, 0.38618E+00_JPRB, 0.38700E+00_JPRB, 0.38533E+00_JPRB ,&
&  0.38044E+00_JPRB, 0.37723E+00_JPRB, 0.37476E+00_JPRB, 0.37528E+00_JPRB, 0.37631E+00_JPRB, 0.37691E+00_JPRB, 0.37682E+00_JPRB ,&
&  0.37403E+00_JPRB, 0.36468E+00_JPRB, 0.35797E+00_JPRB, 0.35503E+00_JPRB, 0.35337E+00_JPRB, 0.35309E+00_JPRB, 0.35307E+00_JPRB ,&
&  0.35035E+00_JPRB, 0.34553E+00_JPRB, 0.34190E+00_JPRB, 0.33452E+00_JPRB, 0.32952E+00_JPRB, 0.32905E+00_JPRB, 0.33337E+00_JPRB ,&
&  0.33558E+00_JPRB, 0.32486E+00_JPRB, 0.32348E+00_JPRB, 0.32036E+00_JPRB, 0.30988E+00_JPRB, 0.30366E+00_JPRB, 0.26135E+00_JPRB ,&
&  0.23508E+00_JPRB, 0.21868E+00_JPRB, 0.21259E+00_JPRB, 0.20990E+00_JPRB, 0.21590E+00_JPRB, 0.23089E+00_JPRB, 0.26094E+00_JPRB ,&
&  0.29467E+00_JPRB, 0.33689E+00_JPRB, 0.35421E+00_JPRB, 0.38130E+00_JPRB, 0.41193E+00_JPRB, 0.44991E+00_JPRB, 0.48233E+00_JPRB ,&
&  0.49693E+00_JPRB, 0.50691E+00_JPRB, 0.52226E+00_JPRB, 0.53286E+00_JPRB, 0.55182E+00_JPRB, 0.56341E+00_JPRB, 0.56094E+00_JPRB ,&
&  0.56720E+00_JPRB, 0.56809E+00_JPRB, 0.57662E+00_JPRB, 0.57115E+00_JPRB, 0.55488E+00_JPRB, 0.53567E+00_JPRB, 0.52358E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.52459E+00_JPRB, 0.55096E+00_JPRB, 0.58591E+00_JPRB, 0.59669E+00_JPRB, 0.59812E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.56406E+00_JPRB, 0.57439E+00_JPRB, 0.63411E+00_JPRB, 0.61591E+00_JPRB, 0.60194E+00_JPRB, 0.58745E+00_JPRB, 0.56138E+00_JPRB ,&
&  0.52845E+00_JPRB, 0.51795E+00_JPRB, 0.50749E+00_JPRB, 0.48653E+00_JPRB, 0.46634E+00_JPRB, 0.44973E+00_JPRB, 0.44060E+00_JPRB ,&
&  0.43666E+00_JPRB, 0.43543E+00_JPRB, 0.42940E+00_JPRB, 0.42400E+00_JPRB, 0.41782E+00_JPRB, 0.40962E+00_JPRB, 0.40419E+00_JPRB ,&
&  0.39854E+00_JPRB, 0.39504E+00_JPRB, 0.39181E+00_JPRB, 0.38800E+00_JPRB, 0.38518E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.37686E+00_JPRB, 0.37657E+00_JPRB, 0.37598E+00_JPRB, 0.37557E+00_JPRB, 0.37586E+00_JPRB, 0.37766E+00_JPRB, 0.38038E+00_JPRB ,&
&  0.38136E+00_JPRB, 0.38238E+00_JPRB, 0.38310E+00_JPRB, 0.38248E+00_JPRB, 0.38061E+00_JPRB, 0.37806E+00_JPRB, 0.37580E+00_JPRB ,&
&  0.37243E+00_JPRB, 0.36857E+00_JPRB, 0.36394E+00_JPRB, 0.35958E+00_JPRB, 0.35537E+00_JPRB, 0.35299E+00_JPRB, 0.34974E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34227E+00_JPRB, 0.33483E+00_JPRB, 0.32831E+00_JPRB, 0.32600E+00_JPRB, 0.32199E+00_JPRB, 0.32320E+00_JPRB ,&
&  0.31967E+00_JPRB, 0.31289E+00_JPRB, 0.30345E+00_JPRB, 0.29870E+00_JPRB, 0.29279E+00_JPRB, 0.25218E+00_JPRB, 0.24036E+00_JPRB ,&
&  0.22701E+00_JPRB, 0.21618E+00_JPRB, 0.21284E+00_JPRB, 0.21350E+00_JPRB, 0.21835E+00_JPRB, 0.23183E+00_JPRB, 0.25059E+00_JPRB ,&
&  0.27523E+00_JPRB, 0.34396E+00_JPRB, 0.36558E+00_JPRB, 0.38970E+00_JPRB, 0.40867E+00_JPRB, 0.42935E+00_JPRB, 0.44076E+00_JPRB ,&
&  0.44710E+00_JPRB, 0.45121E+00_JPRB, 0.46344E+00_JPRB, 0.48085E+00_JPRB, 0.49540E+00_JPRB, 0.49980E+00_JPRB, 0.49804E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.47468E+00_JPRB, 0.45942E+00_JPRB, 0.44977E+00_JPRB, 0.45072E+00_JPRB, 0.45363E+00_JPRB, 0.46692E+00_JPRB ,&
&  0.47685E+00_JPRB, 0.48406E+00_JPRB, 0.50040E+00_JPRB, 0.51910E+00_JPRB, 0.53096E+00_JPRB, 0.53899E+00_JPRB, 0.53936E+00_JPRB ,&
&  0.53594E+00_JPRB, 0.54278E+00_JPRB, 0.54196E+00_JPRB, 0.53892E+00_JPRB, 0.53168E+00_JPRB, 0.52006E+00_JPRB, 0.50576E+00_JPRB ,&
&  0.49973E+00_JPRB, 0.50342E+00_JPRB, 0.49887E+00_JPRB, 0.48380E+00_JPRB, 0.46973E+00_JPRB, 0.45546E+00_JPRB, 0.44512E+00_JPRB ,&
&  0.43952E+00_JPRB, 0.43688E+00_JPRB, 0.43051E+00_JPRB, 0.42378E+00_JPRB, 0.41496E+00_JPRB, 0.40621E+00_JPRB, 0.40043E+00_JPRB ,&
&  0.39505E+00_JPRB, 0.39169E+00_JPRB, 0.38695E+00_JPRB, 0.38170E+00_JPRB, 0.37922E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.37196E+00_JPRB, 0.37241E+00_JPRB, 0.37153E+00_JPRB, 0.37113E+00_JPRB, 0.37087E+00_JPRB, 0.37236E+00_JPRB, 0.37358E+00_JPRB ,&
&  0.37423E+00_JPRB, 0.37294E+00_JPRB, 0.37215E+00_JPRB, 0.37036E+00_JPRB, 0.36858E+00_JPRB, 0.36647E+00_JPRB, 0.36543E+00_JPRB ,&
&  0.36476E+00_JPRB, 0.36326E+00_JPRB, 0.36076E+00_JPRB, 0.35704E+00_JPRB, 0.35389E+00_JPRB, 0.34992E+00_JPRB, 0.34568E+00_JPRB ,&
&  0.34085E+00_JPRB, 0.33471E+00_JPRB, 0.32245E+00_JPRB, 0.31489E+00_JPRB, 0.30535E+00_JPRB, 0.30131E+00_JPRB, 0.29638E+00_JPRB ,&
&  0.30127E+00_JPRB, 0.27444E+00_JPRB, 0.26723E+00_JPRB, 0.25372E+00_JPRB, 0.24330E+00_JPRB, 0.23556E+00_JPRB, 0.22754E+00_JPRB ,&
&  0.22408E+00_JPRB, 0.22111E+00_JPRB, 0.22168E+00_JPRB, 0.22671E+00_JPRB, 0.23487E+00_JPRB, 0.25065E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.29476E+00_JPRB, 0.35130E+00_JPRB, 0.37056E+00_JPRB, 0.39043E+00_JPRB, 0.40798E+00_JPRB, 0.42631E+00_JPRB, 0.42887E+00_JPRB ,&
&  0.43592E+00_JPRB, 0.43953E+00_JPRB, 0.44870E+00_JPRB, 0.45553E+00_JPRB, 0.47085E+00_JPRB, 0.46906E+00_JPRB, 0.46423E+00_JPRB ,&
&  0.44392E+00_JPRB, 0.42462E+00_JPRB, 0.41117E+00_JPRB, 0.41095E+00_JPRB, 0.41953E+00_JPRB, 0.43137E+00_JPRB, 0.44305E+00_JPRB ,&
&  0.45330E+00_JPRB, 0.46053E+00_JPRB, 0.46469E+00_JPRB, 0.47321E+00_JPRB, 0.48356E+00_JPRB, 0.49590E+00_JPRB, 0.50077E+00_JPRB ,&
&  0.50434E+00_JPRB, 0.50770E+00_JPRB, 0.50770E+00_JPRB, 0.50760E+00_JPRB, 0.49956E+00_JPRB, 0.49169E+00_JPRB, 0.48622E+00_JPRB ,&
&  0.48793E+00_JPRB, 0.48395E+00_JPRB, 0.47662E+00_JPRB, 0.46696E+00_JPRB, 0.46012E+00_JPRB, 0.44997E+00_JPRB, 0.44111E+00_JPRB ,&
&  0.43429E+00_JPRB, 0.43084E+00_JPRB, 0.42568E+00_JPRB, 0.42030E+00_JPRB, 0.41027E+00_JPRB, 0.40153E+00_JPRB, 0.39496E+00_JPRB ,&
&  0.38850E+00_JPRB, 0.38319E+00_JPRB, 0.37721E+00_JPRB, 0.37337E+00_JPRB, 0.37258E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.36890E+00_JPRB, 0.36935E+00_JPRB, 0.36953E+00_JPRB, 0.36963E+00_JPRB, 0.36837E+00_JPRB, 0.36828E+00_JPRB, 0.36727E+00_JPRB ,&
&  0.36801E+00_JPRB, 0.36752E+00_JPRB, 0.36767E+00_JPRB, 0.36835E+00_JPRB, 0.36892E+00_JPRB, 0.36945E+00_JPRB, 0.36917E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36933E+00_JPRB, 0.36847E+00_JPRB, 0.36568E+00_JPRB, 0.36295E+00_JPRB, 0.35894E+00_JPRB, 0.35256E+00_JPRB ,&
&  0.34475E+00_JPRB, 0.33758E+00_JPRB, 0.31700E+00_JPRB, 0.30850E+00_JPRB, 0.29634E+00_JPRB, 0.28951E+00_JPRB, 0.28659E+00_JPRB ,&
&  0.28454E+00_JPRB, 0.28319E+00_JPRB, 0.27907E+00_JPRB, 0.27328E+00_JPRB, 0.26638E+00_JPRB, 0.25673E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.24485E+00_JPRB, 0.24343E+00_JPRB, 0.24586E+00_JPRB, 0.25241E+00_JPRB, 0.26364E+00_JPRB, 0.27835E+00_JPRB, 0.29531E+00_JPRB ,&
&  0.31733E+00_JPRB, 0.33960E+00_JPRB, 0.36818E+00_JPRB, 0.37933E+00_JPRB, 0.38936E+00_JPRB, 0.39926E+00_JPRB, 0.40767E+00_JPRB ,&
&  0.40720E+00_JPRB, 0.40722E+00_JPRB, 0.41485E+00_JPRB, 0.41366E+00_JPRB, 0.41211E+00_JPRB, 0.40455E+00_JPRB, 0.39866E+00_JPRB ,&
&  0.39104E+00_JPRB, 0.38941E+00_JPRB, 0.38922E+00_JPRB, 0.39343E+00_JPRB, 0.40071E+00_JPRB, 0.40996E+00_JPRB, 0.41937E+00_JPRB ,&
&  0.42801E+00_JPRB, 0.43815E+00_JPRB, 0.44497E+00_JPRB, 0.45131E+00_JPRB, 0.45724E+00_JPRB, 0.46557E+00_JPRB, 0.47188E+00_JPRB ,&
&  0.47622E+00_JPRB, 0.47547E+00_JPRB, 0.47149E+00_JPRB, 0.46223E+00_JPRB, 0.45833E+00_JPRB, 0.46390E+00_JPRB, 0.46223E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.45695E+00_JPRB, 0.45388E+00_JPRB, 0.45020E+00_JPRB, 0.44665E+00_JPRB, 0.44078E+00_JPRB, 0.43403E+00_JPRB ,&
&  0.42823E+00_JPRB, 0.42446E+00_JPRB, 0.41993E+00_JPRB, 0.41562E+00_JPRB, 0.40852E+00_JPRB, 0.40050E+00_JPRB, 0.39335E+00_JPRB ,&
&  0.38678E+00_JPRB, 0.38090E+00_JPRB, 0.37547E+00_JPRB, 0.37170E+00_JPRB, 0.36983E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.36597E+00_JPRB, 0.36546E+00_JPRB, 0.36437E+00_JPRB, 0.36307E+00_JPRB, 0.36122E+00_JPRB, 0.36042E+00_JPRB, 0.36046E+00_JPRB ,&
&  0.36118E+00_JPRB, 0.36119E+00_JPRB, 0.36142E+00_JPRB, 0.36301E+00_JPRB, 0.36540E+00_JPRB, 0.36662E+00_JPRB, 0.36804E+00_JPRB ,&
&  0.36954E+00_JPRB, 0.37021E+00_JPRB, 0.37053E+00_JPRB, 0.36995E+00_JPRB, 0.36798E+00_JPRB, 0.36454E+00_JPRB, 0.36156E+00_JPRB ,&
&  0.35803E+00_JPRB, 0.35481E+00_JPRB, 0.35095E+00_JPRB, 0.34676E+00_JPRB, 0.34260E+00_JPRB, 0.31650E+00_JPRB, 0.30853E+00_JPRB ,&
&  0.30629E+00_JPRB, 0.30674E+00_JPRB, 0.31064E+00_JPRB, 0.31637E+00_JPRB, 0.32709E+00_JPRB, 0.32563E+00_JPRB, 0.32643E+00_JPRB ,&
&  0.31221E+00_JPRB, 0.31237E+00_JPRB, 0.31268E+00_JPRB, 0.31421E+00_JPRB, 0.31884E+00_JPRB, 0.32440E+00_JPRB, 0.33351E+00_JPRB ,&
&  0.35305E+00_JPRB, 0.35727E+00_JPRB, 0.36046E+00_JPRB, 0.36417E+00_JPRB, 0.36747E+00_JPRB, 0.36975E+00_JPRB, 0.37124E+00_JPRB ,&
&  0.37151E+00_JPRB, 0.37156E+00_JPRB, 0.37119E+00_JPRB, 0.37029E+00_JPRB, 0.37028E+00_JPRB, 0.37066E+00_JPRB, 0.37321E+00_JPRB ,&
&  0.37540E+00_JPRB, 0.37937E+00_JPRB, 0.38395E+00_JPRB, 0.38884E+00_JPRB, 0.39316E+00_JPRB, 0.39829E+00_JPRB, 0.40491E+00_JPRB ,&
&  0.41135E+00_JPRB, 0.41806E+00_JPRB, 0.42283E+00_JPRB, 0.42783E+00_JPRB, 0.43138E+00_JPRB, 0.43591E+00_JPRB, 0.43761E+00_JPRB ,&
&  0.43950E+00_JPRB, 0.43885E+00_JPRB, 0.43931E+00_JPRB, 0.43868E+00_JPRB, 0.43859E+00_JPRB, 0.43895E+00_JPRB, 0.43813E+00_JPRB ,&
&  0.43736E+00_JPRB, 0.43559E+00_JPRB, 0.43410E+00_JPRB, 0.43196E+00_JPRB, 0.42875E+00_JPRB, 0.42390E+00_JPRB, 0.41905E+00_JPRB ,&
&  0.41541E+00_JPRB, 0.41252E+00_JPRB, 0.40917E+00_JPRB, 0.40627E+00_JPRB, 0.40216E+00_JPRB, 0.39617E+00_JPRB, 0.38935E+00_JPRB ,&
&  0.38328E+00_JPRB, 0.37804E+00_JPRB, 0.37338E+00_JPRB, 0.36995E+00_JPRB, 0.36764E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.36382E+00_JPRB, 0.36373E+00_JPRB, 0.36382E+00_JPRB, 0.36362E+00_JPRB, 0.36276E+00_JPRB, 0.36238E+00_JPRB, 0.36258E+00_JPRB ,&
&  0.36379E+00_JPRB, 0.36523E+00_JPRB, 0.36563E+00_JPRB, 0.36540E+00_JPRB, 0.36524E+00_JPRB, 0.36519E+00_JPRB, 0.36529E+00_JPRB ,&
&  0.36561E+00_JPRB, 0.36547E+00_JPRB, 0.36430E+00_JPRB, 0.36218E+00_JPRB, 0.36079E+00_JPRB, 0.35992E+00_JPRB, 0.35998E+00_JPRB ,&
&  0.36005E+00_JPRB, 0.35959E+00_JPRB, 0.35858E+00_JPRB, 0.35782E+00_JPRB, 0.35636E+00_JPRB, 0.35494E+00_JPRB, 0.35331E+00_JPRB ,&
&  0.35234E+00_JPRB, 0.35179E+00_JPRB, 0.35142E+00_JPRB, 0.35102E+00_JPRB, 0.35147E+00_JPRB, 0.35204E+00_JPRB, 0.35255E+00_JPRB ,&
&  0.35288E+00_JPRB, 0.35326E+00_JPRB, 0.35434E+00_JPRB, 0.35524E+00_JPRB, 0.35597E+00_JPRB, 0.35631E+00_JPRB, 0.35589E+00_JPRB ,&
&  0.35463E+00_JPRB, 0.35361E+00_JPRB, 0.35372E+00_JPRB, 0.35362E+00_JPRB, 0.35371E+00_JPRB, 0.35389E+00_JPRB, 0.35497E+00_JPRB ,&
&  0.35559E+00_JPRB, 0.35639E+00_JPRB, 0.35778E+00_JPRB, 0.36004E+00_JPRB, 0.36197E+00_JPRB, 0.36444E+00_JPRB, 0.36736E+00_JPRB ,&
&  0.37002E+00_JPRB, 0.37286E+00_JPRB, 0.37575E+00_JPRB, 0.37870E+00_JPRB, 0.38139E+00_JPRB, 0.38390E+00_JPRB, 0.38692E+00_JPRB ,&
&  0.39033E+00_JPRB, 0.39443E+00_JPRB, 0.39717E+00_JPRB, 0.40009E+00_JPRB, 0.40329E+00_JPRB, 0.40793E+00_JPRB, 0.41004E+00_JPRB ,&
&  0.41184E+00_JPRB, 0.41350E+00_JPRB, 0.41529E+00_JPRB, 0.41615E+00_JPRB, 0.41695E+00_JPRB, 0.41763E+00_JPRB, 0.41780E+00_JPRB ,&
&  0.41752E+00_JPRB, 0.41660E+00_JPRB, 0.41525E+00_JPRB, 0.41301E+00_JPRB, 0.41026E+00_JPRB, 0.40713E+00_JPRB, 0.40345E+00_JPRB ,&
&  0.40013E+00_JPRB, 0.39592E+00_JPRB, 0.39150E+00_JPRB, 0.38790E+00_JPRB, 0.38528E+00_JPRB, 0.38164E+00_JPRB, 0.37755E+00_JPRB ,&
&  0.37349E+00_JPRB, 0.37025E+00_JPRB, 0.36809E+00_JPRB, 0.36623E+00_JPRB, 0.36492E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.36581E+00_JPRB, 0.36548E+00_JPRB, 0.36530E+00_JPRB, 0.36530E+00_JPRB, 0.36512E+00_JPRB, 0.36472E+00_JPRB, 0.36414E+00_JPRB ,&
&  0.36365E+00_JPRB, 0.36366E+00_JPRB, 0.36390E+00_JPRB, 0.36432E+00_JPRB, 0.36454E+00_JPRB, 0.36435E+00_JPRB, 0.36383E+00_JPRB ,&
&  0.36306E+00_JPRB, 0.36195E+00_JPRB, 0.36049E+00_JPRB, 0.35917E+00_JPRB, 0.35893E+00_JPRB, 0.35882E+00_JPRB, 0.35855E+00_JPRB ,&
&  0.35836E+00_JPRB, 0.35826E+00_JPRB, 0.35820E+00_JPRB, 0.35810E+00_JPRB, 0.35820E+00_JPRB, 0.35816E+00_JPRB, 0.35820E+00_JPRB ,&
&  0.35821E+00_JPRB, 0.35822E+00_JPRB, 0.35798E+00_JPRB, 0.35780E+00_JPRB, 0.35746E+00_JPRB, 0.35755E+00_JPRB, 0.35748E+00_JPRB ,&
&  0.35761E+00_JPRB, 0.35748E+00_JPRB, 0.35725E+00_JPRB, 0.35687E+00_JPRB, 0.35650E+00_JPRB, 0.35572E+00_JPRB, 0.35500E+00_JPRB ,&
&  0.35447E+00_JPRB, 0.35422E+00_JPRB, 0.35444E+00_JPRB, 0.35509E+00_JPRB, 0.35578E+00_JPRB, 0.35656E+00_JPRB, 0.35737E+00_JPRB ,&
&  0.35809E+00_JPRB, 0.35894E+00_JPRB, 0.35985E+00_JPRB, 0.36074E+00_JPRB, 0.36164E+00_JPRB, 0.36247E+00_JPRB, 0.36339E+00_JPRB ,&
&  0.36427E+00_JPRB, 0.36537E+00_JPRB, 0.36686E+00_JPRB, 0.36843E+00_JPRB, 0.36995E+00_JPRB, 0.37137E+00_JPRB, 0.37297E+00_JPRB ,&
&  0.37470E+00_JPRB, 0.37643E+00_JPRB, 0.37783E+00_JPRB, 0.37932E+00_JPRB, 0.38087E+00_JPRB, 0.38240E+00_JPRB, 0.38354E+00_JPRB ,&
&  0.38470E+00_JPRB, 0.38601E+00_JPRB, 0.38743E+00_JPRB, 0.38790E+00_JPRB, 0.38827E+00_JPRB, 0.38849E+00_JPRB, 0.38842E+00_JPRB ,&
&  0.38788E+00_JPRB, 0.38719E+00_JPRB, 0.38646E+00_JPRB, 0.38551E+00_JPRB, 0.38399E+00_JPRB, 0.38235E+00_JPRB, 0.38052E+00_JPRB ,&
&  0.37873E+00_JPRB, 0.37656E+00_JPRB, 0.37442E+00_JPRB, 0.37275E+00_JPRB, 0.37150E+00_JPRB, 0.37030E+00_JPRB, 0.36906E+00_JPRB ,&
&  0.36780E+00_JPRB, 0.36704E+00_JPRB, 0.36681E+00_JPRB, 0.36661E+00_JPRB, 0.36635E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB, 0.36060E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB ,&
&  0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB, 0.22186E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.23311E+00_JPRB, 0.23340E+00_JPRB, 0.23355E+00_JPRB, 0.23363E+00_JPRB, 0.23218E+00_JPRB, 0.23273E+00_JPRB, 0.23291E+00_JPRB ,&
&  0.23325E+00_JPRB, 0.23356E+00_JPRB, 0.23412E+00_JPRB, 0.23478E+00_JPRB, 0.23541E+00_JPRB, 0.23598E+00_JPRB, 0.23646E+00_JPRB ,&
&  0.23675E+00_JPRB, 0.23712E+00_JPRB, 0.23592E+00_JPRB, 0.23524E+00_JPRB, 0.23519E+00_JPRB, 0.23532E+00_JPRB, 0.23533E+00_JPRB ,&
&  0.23531E+00_JPRB, 0.23523E+00_JPRB, 0.23484E+00_JPRB, 0.23454E+00_JPRB, 0.23441E+00_JPRB, 0.23424E+00_JPRB, 0.23396E+00_JPRB ,&
&  0.23358E+00_JPRB, 0.23329E+00_JPRB, 0.23294E+00_JPRB, 0.23255E+00_JPRB, 0.23238E+00_JPRB, 0.23214E+00_JPRB, 0.23153E+00_JPRB ,&
&  0.23080E+00_JPRB, 0.22979E+00_JPRB, 0.22885E+00_JPRB, 0.22801E+00_JPRB, 0.22734E+00_JPRB, 0.22698E+00_JPRB, 0.22661E+00_JPRB ,&
&  0.22613E+00_JPRB, 0.22548E+00_JPRB, 0.22486E+00_JPRB, 0.22413E+00_JPRB, 0.22350E+00_JPRB, 0.22293E+00_JPRB, 0.22226E+00_JPRB ,&
&  0.22152E+00_JPRB, 0.22078E+00_JPRB, 0.22001E+00_JPRB, 0.21917E+00_JPRB, 0.21837E+00_JPRB, 0.21755E+00_JPRB, 0.21667E+00_JPRB ,&
&  0.21581E+00_JPRB, 0.21541E+00_JPRB, 0.21496E+00_JPRB, 0.21458E+00_JPRB, 0.21429E+00_JPRB, 0.21407E+00_JPRB, 0.21385E+00_JPRB ,&
&  0.21368E+00_JPRB, 0.21349E+00_JPRB, 0.21340E+00_JPRB, 0.21331E+00_JPRB, 0.21331E+00_JPRB, 0.21337E+00_JPRB, 0.21353E+00_JPRB ,&
&  0.21372E+00_JPRB, 0.21396E+00_JPRB, 0.21428E+00_JPRB, 0.21480E+00_JPRB, 0.21532E+00_JPRB, 0.21592E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21717E+00_JPRB, 0.21781E+00_JPRB, 0.21846E+00_JPRB, 0.21908E+00_JPRB, 0.21991E+00_JPRB, 0.22070E+00_JPRB, 0.22157E+00_JPRB ,&
&  0.22244E+00_JPRB, 0.22338E+00_JPRB, 0.22431E+00_JPRB, 0.22530E+00_JPRB, 0.22642E+00_JPRB, 0.22677E+00_JPRB, 0.22713E+00_JPRB ,&
&  0.22758E+00_JPRB, 0.22824E+00_JPRB, 0.22884E+00_JPRB, 0.22961E+00_JPRB, 0.23090E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.25975E+00_JPRB, 0.26621E+00_JPRB, 0.27069E+00_JPRB, 0.27562E+00_JPRB, 0.27551E+00_JPRB, 0.31338E+00_JPRB, 0.31238E+00_JPRB ,&
&  0.31104E+00_JPRB, 0.29630E+00_JPRB, 0.29359E+00_JPRB, 0.28869E+00_JPRB, 0.28306E+00_JPRB, 0.27694E+00_JPRB, 0.27166E+00_JPRB ,&
&  0.26687E+00_JPRB, 0.26157E+00_JPRB, 0.25723E+00_JPRB, 0.25322E+00_JPRB, 0.24992E+00_JPRB, 0.24700E+00_JPRB, 0.24493E+00_JPRB ,&
&  0.24438E+00_JPRB, 0.24577E+00_JPRB, 0.24875E+00_JPRB, 0.25056E+00_JPRB, 0.25143E+00_JPRB, 0.25217E+00_JPRB, 0.25287E+00_JPRB ,&
&  0.25300E+00_JPRB, 0.25310E+00_JPRB, 0.25211E+00_JPRB, 0.25022E+00_JPRB, 0.24834E+00_JPRB, 0.24676E+00_JPRB, 0.24548E+00_JPRB ,&
&  0.24426E+00_JPRB, 0.24298E+00_JPRB, 0.24162E+00_JPRB, 0.24073E+00_JPRB, 0.23957E+00_JPRB, 0.23804E+00_JPRB, 0.23600E+00_JPRB ,&
&  0.23398E+00_JPRB, 0.23196E+00_JPRB, 0.22961E+00_JPRB, 0.22772E+00_JPRB, 0.22589E+00_JPRB, 0.22404E+00_JPRB, 0.22221E+00_JPRB ,&
&  0.22117E+00_JPRB, 0.22015E+00_JPRB, 0.21889E+00_JPRB, 0.21728E+00_JPRB, 0.21605E+00_JPRB, 0.21473E+00_JPRB, 0.21358E+00_JPRB ,&
&  0.21281E+00_JPRB, 0.21202E+00_JPRB, 0.21121E+00_JPRB, 0.21038E+00_JPRB, 0.20976E+00_JPRB, 0.20906E+00_JPRB, 0.20842E+00_JPRB ,&
&  0.20787E+00_JPRB, 0.20726E+00_JPRB, 0.20711E+00_JPRB, 0.20705E+00_JPRB, 0.20712E+00_JPRB, 0.20719E+00_JPRB, 0.20749E+00_JPRB ,&
&  0.20794E+00_JPRB, 0.20859E+00_JPRB, 0.20939E+00_JPRB, 0.21014E+00_JPRB, 0.21088E+00_JPRB, 0.21165E+00_JPRB, 0.21276E+00_JPRB ,&
&  0.21398E+00_JPRB, 0.21529E+00_JPRB, 0.21666E+00_JPRB, 0.21774E+00_JPRB, 0.21988E+00_JPRB, 0.22191E+00_JPRB, 0.22408E+00_JPRB ,&
&  0.22657E+00_JPRB, 0.22951E+00_JPRB, 0.23264E+00_JPRB, 0.23595E+00_JPRB, 0.24026E+00_JPRB, 0.24313E+00_JPRB, 0.24526E+00_JPRB ,&
&  0.24760E+00_JPRB, 0.25144E+00_JPRB, 0.25317E+00_JPRB, 0.25464E+00_JPRB, 0.25683E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.33079E+00_JPRB, 0.33531E+00_JPRB, 0.33919E+00_JPRB, 0.34262E+00_JPRB, 0.34560E+00_JPRB, 0.34628E+00_JPRB, 0.34530E+00_JPRB ,&
&  0.33631E+00_JPRB, 0.32959E+00_JPRB, 0.32146E+00_JPRB, 0.31432E+00_JPRB, 0.30771E+00_JPRB, 0.30019E+00_JPRB, 0.29352E+00_JPRB ,&
&  0.28672E+00_JPRB, 0.27994E+00_JPRB, 0.27345E+00_JPRB, 0.26587E+00_JPRB, 0.25795E+00_JPRB, 0.25039E+00_JPRB, 0.24566E+00_JPRB ,&
&  0.24294E+00_JPRB, 0.24276E+00_JPRB, 0.24556E+00_JPRB, 0.24936E+00_JPRB, 0.25429E+00_JPRB, 0.26065E+00_JPRB, 0.26745E+00_JPRB ,&
&  0.27178E+00_JPRB, 0.27419E+00_JPRB, 0.27382E+00_JPRB, 0.27252E+00_JPRB, 0.27061E+00_JPRB, 0.26890E+00_JPRB, 0.26627E+00_JPRB ,&
&  0.26293E+00_JPRB, 0.26042E+00_JPRB, 0.25933E+00_JPRB, 0.25991E+00_JPRB, 0.25937E+00_JPRB, 0.25731E+00_JPRB, 0.25341E+00_JPRB ,&
&  0.24918E+00_JPRB, 0.24494E+00_JPRB, 0.24067E+00_JPRB, 0.23672E+00_JPRB, 0.23347E+00_JPRB, 0.23011E+00_JPRB, 0.22653E+00_JPRB ,&
&  0.22424E+00_JPRB, 0.22180E+00_JPRB, 0.21913E+00_JPRB, 0.21656E+00_JPRB, 0.21443E+00_JPRB, 0.21279E+00_JPRB, 0.21142E+00_JPRB ,&
&  0.21033E+00_JPRB, 0.20948E+00_JPRB, 0.20879E+00_JPRB, 0.20827E+00_JPRB, 0.20778E+00_JPRB, 0.20704E+00_JPRB, 0.20632E+00_JPRB ,&
&  0.20559E+00_JPRB, 0.20531E+00_JPRB, 0.20487E+00_JPRB, 0.20489E+00_JPRB, 0.20513E+00_JPRB, 0.20528E+00_JPRB, 0.20577E+00_JPRB ,&
&  0.20686E+00_JPRB, 0.20799E+00_JPRB, 0.20865E+00_JPRB, 0.20972E+00_JPRB, 0.21077E+00_JPRB, 0.21212E+00_JPRB, 0.21385E+00_JPRB ,&
&  0.21498E+00_JPRB, 0.21624E+00_JPRB, 0.21771E+00_JPRB, 0.21928E+00_JPRB, 0.22181E+00_JPRB, 0.22456E+00_JPRB, 0.22762E+00_JPRB ,&
&  0.23155E+00_JPRB, 0.23543E+00_JPRB, 0.23983E+00_JPRB, 0.24503E+00_JPRB, 0.25403E+00_JPRB, 0.26065E+00_JPRB, 0.26934E+00_JPRB ,&
&  0.27820E+00_JPRB, 0.28414E+00_JPRB, 0.31753E+00_JPRB, 0.31995E+00_JPRB, 0.32503E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.33949E+00_JPRB, 0.34602E+00_JPRB, 0.35290E+00_JPRB, 0.35793E+00_JPRB, 0.36163E+00_JPRB, 0.36464E+00_JPRB, 0.36357E+00_JPRB ,&
&  0.35744E+00_JPRB, 0.34208E+00_JPRB, 0.33060E+00_JPRB, 0.31958E+00_JPRB, 0.31019E+00_JPRB, 0.30134E+00_JPRB, 0.29305E+00_JPRB ,&
&  0.28519E+00_JPRB, 0.27871E+00_JPRB, 0.27344E+00_JPRB, 0.26887E+00_JPRB, 0.26260E+00_JPRB, 0.25712E+00_JPRB, 0.25273E+00_JPRB ,&
&  0.24908E+00_JPRB, 0.24893E+00_JPRB, 0.25111E+00_JPRB, 0.25540E+00_JPRB, 0.26337E+00_JPRB, 0.27164E+00_JPRB, 0.28093E+00_JPRB ,&
&  0.29528E+00_JPRB, 0.29968E+00_JPRB, 0.29869E+00_JPRB, 0.29800E+00_JPRB, 0.29310E+00_JPRB, 0.28707E+00_JPRB, 0.27770E+00_JPRB ,&
&  0.27788E+00_JPRB, 0.27595E+00_JPRB, 0.28268E+00_JPRB, 0.28299E+00_JPRB, 0.28180E+00_JPRB, 0.28137E+00_JPRB, 0.27383E+00_JPRB ,&
&  0.26796E+00_JPRB, 0.25905E+00_JPRB, 0.25220E+00_JPRB, 0.24537E+00_JPRB, 0.23889E+00_JPRB, 0.23500E+00_JPRB, 0.22997E+00_JPRB ,&
&  0.22619E+00_JPRB, 0.22327E+00_JPRB, 0.21955E+00_JPRB, 0.21712E+00_JPRB, 0.21508E+00_JPRB, 0.21301E+00_JPRB, 0.21194E+00_JPRB ,&
&  0.21115E+00_JPRB, 0.21086E+00_JPRB, 0.21094E+00_JPRB, 0.21113E+00_JPRB, 0.21159E+00_JPRB, 0.21245E+00_JPRB, 0.21310E+00_JPRB ,&
&  0.21382E+00_JPRB, 0.21404E+00_JPRB, 0.21365E+00_JPRB, 0.21320E+00_JPRB, 0.21213E+00_JPRB, 0.21139E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.20994E+00_JPRB, 0.21026E+00_JPRB, 0.21057E+00_JPRB, 0.21125E+00_JPRB, 0.21196E+00_JPRB, 0.21345E+00_JPRB, 0.21486E+00_JPRB ,&
&  0.21636E+00_JPRB, 0.21818E+00_JPRB, 0.21969E+00_JPRB, 0.22158E+00_JPRB, 0.22386E+00_JPRB, 0.22547E+00_JPRB, 0.22817E+00_JPRB ,&
&  0.23085E+00_JPRB, 0.23417E+00_JPRB, 0.23967E+00_JPRB, 0.24587E+00_JPRB, 0.25421E+00_JPRB, 0.26492E+00_JPRB, 0.27862E+00_JPRB ,&
&  0.29624E+00_JPRB, 0.32141E+00_JPRB, 0.32346E+00_JPRB, 0.32971E+00_JPRB, 0.33445E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.34450E+00_JPRB, 0.34704E+00_JPRB, 0.35115E+00_JPRB, 0.35648E+00_JPRB, 0.36312E+00_JPRB, 0.36903E+00_JPRB, 0.37032E+00_JPRB ,&
&  0.36118E+00_JPRB, 0.34824E+00_JPRB, 0.33020E+00_JPRB, 0.31571E+00_JPRB, 0.30191E+00_JPRB, 0.29148E+00_JPRB, 0.28295E+00_JPRB ,&
&  0.27770E+00_JPRB, 0.27715E+00_JPRB, 0.27772E+00_JPRB, 0.27896E+00_JPRB, 0.27812E+00_JPRB, 0.27929E+00_JPRB, 0.27980E+00_JPRB ,&
&  0.27877E+00_JPRB, 0.27868E+00_JPRB, 0.27859E+00_JPRB, 0.28213E+00_JPRB, 0.28893E+00_JPRB, 0.29530E+00_JPRB, 0.30164E+00_JPRB ,&
&  0.30901E+00_JPRB, 0.31143E+00_JPRB, 0.31577E+00_JPRB, 0.31473E+00_JPRB, 0.31127E+00_JPRB, 0.30676E+00_JPRB, 0.30048E+00_JPRB ,&
&  0.29572E+00_JPRB, 0.29160E+00_JPRB, 0.29007E+00_JPRB, 0.29052E+00_JPRB, 0.29069E+00_JPRB, 0.29180E+00_JPRB, 0.28532E+00_JPRB ,&
&  0.27797E+00_JPRB, 0.26779E+00_JPRB, 0.25649E+00_JPRB, 0.24823E+00_JPRB, 0.24061E+00_JPRB, 0.23511E+00_JPRB, 0.23020E+00_JPRB ,&
&  0.22564E+00_JPRB, 0.22257E+00_JPRB, 0.21940E+00_JPRB, 0.21712E+00_JPRB, 0.21642E+00_JPRB, 0.21485E+00_JPRB, 0.21401E+00_JPRB ,&
&  0.21369E+00_JPRB, 0.21414E+00_JPRB, 0.21560E+00_JPRB, 0.21697E+00_JPRB, 0.21881E+00_JPRB, 0.22094E+00_JPRB, 0.22278E+00_JPRB ,&
&  0.22521E+00_JPRB, 0.22714E+00_JPRB, 0.22735E+00_JPRB, 0.22673E+00_JPRB, 0.22471E+00_JPRB, 0.22209E+00_JPRB, 0.21970E+00_JPRB ,&
&  0.21738E+00_JPRB, 0.21571E+00_JPRB, 0.21505E+00_JPRB, 0.21427E+00_JPRB, 0.21468E+00_JPRB, 0.21578E+00_JPRB, 0.21668E+00_JPRB ,&
&  0.21832E+00_JPRB, 0.21965E+00_JPRB, 0.22117E+00_JPRB, 0.22364E+00_JPRB, 0.22604E+00_JPRB, 0.22816E+00_JPRB, 0.23038E+00_JPRB ,&
&  0.23065E+00_JPRB, 0.23233E+00_JPRB, 0.23453E+00_JPRB, 0.23831E+00_JPRB, 0.24915E+00_JPRB, 0.25726E+00_JPRB, 0.27599E+00_JPRB ,&
&  0.29331E+00_JPRB, 0.32957E+00_JPRB, 0.33086E+00_JPRB, 0.33534E+00_JPRB, 0.34102E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.34750E+00_JPRB, 0.35190E+00_JPRB, 0.35290E+00_JPRB, 0.35618E+00_JPRB, 0.36057E+00_JPRB, 0.36574E+00_JPRB, 0.36997E+00_JPRB ,&
&  0.37492E+00_JPRB, 0.37465E+00_JPRB, 0.36950E+00_JPRB, 0.36273E+00_JPRB, 0.35696E+00_JPRB, 0.34012E+00_JPRB, 0.33106E+00_JPRB ,&
&  0.32374E+00_JPRB, 0.32309E+00_JPRB, 0.32369E+00_JPRB, 0.32526E+00_JPRB, 0.32640E+00_JPRB, 0.33137E+00_JPRB, 0.33295E+00_JPRB ,&
&  0.33062E+00_JPRB, 0.32564E+00_JPRB, 0.32065E+00_JPRB, 0.31984E+00_JPRB, 0.32242E+00_JPRB, 0.32447E+00_JPRB, 0.32181E+00_JPRB ,&
&  0.31822E+00_JPRB, 0.31795E+00_JPRB, 0.31862E+00_JPRB, 0.32009E+00_JPRB, 0.32984E+00_JPRB, 0.32574E+00_JPRB, 0.32142E+00_JPRB ,&
&  0.31432E+00_JPRB, 0.30812E+00_JPRB, 0.30375E+00_JPRB, 0.30400E+00_JPRB, 0.30664E+00_JPRB, 0.30689E+00_JPRB, 0.30939E+00_JPRB ,&
&  0.29728E+00_JPRB, 0.28726E+00_JPRB, 0.27265E+00_JPRB, 0.26126E+00_JPRB, 0.25062E+00_JPRB, 0.24332E+00_JPRB, 0.23642E+00_JPRB ,&
&  0.23053E+00_JPRB, 0.22571E+00_JPRB, 0.22111E+00_JPRB, 0.21956E+00_JPRB, 0.22040E+00_JPRB, 0.22061E+00_JPRB, 0.22040E+00_JPRB ,&
&  0.22104E+00_JPRB, 0.22202E+00_JPRB, 0.22279E+00_JPRB, 0.22374E+00_JPRB, 0.22520E+00_JPRB, 0.22764E+00_JPRB, 0.23089E+00_JPRB ,&
&  0.23604E+00_JPRB, 0.24017E+00_JPRB, 0.24243E+00_JPRB, 0.24336E+00_JPRB, 0.24028E+00_JPRB, 0.23517E+00_JPRB, 0.22982E+00_JPRB ,&
&  0.22467E+00_JPRB, 0.22224E+00_JPRB, 0.22014E+00_JPRB, 0.21915E+00_JPRB, 0.21900E+00_JPRB, 0.21988E+00_JPRB, 0.22159E+00_JPRB ,&
&  0.22328E+00_JPRB, 0.22534E+00_JPRB, 0.22670E+00_JPRB, 0.22837E+00_JPRB, 0.23041E+00_JPRB, 0.23221E+00_JPRB, 0.23408E+00_JPRB ,&
&  0.23617E+00_JPRB, 0.23780E+00_JPRB, 0.24058E+00_JPRB, 0.24398E+00_JPRB, 0.25035E+00_JPRB, 0.25817E+00_JPRB, 0.27222E+00_JPRB ,&
&  0.28909E+00_JPRB, 0.32207E+00_JPRB, 0.33365E+00_JPRB, 0.33897E+00_JPRB, 0.34538E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.34492E+00_JPRB, 0.35123E+00_JPRB, 0.35630E+00_JPRB, 0.36633E+00_JPRB, 0.36899E+00_JPRB, 0.37150E+00_JPRB, 0.37347E+00_JPRB ,&
&  0.37924E+00_JPRB, 0.38271E+00_JPRB, 0.38040E+00_JPRB, 0.37969E+00_JPRB, 0.37875E+00_JPRB, 0.37775E+00_JPRB, 0.37851E+00_JPRB ,&
&  0.37837E+00_JPRB, 0.38033E+00_JPRB, 0.37810E+00_JPRB, 0.37526E+00_JPRB, 0.37710E+00_JPRB, 0.38178E+00_JPRB, 0.38496E+00_JPRB ,&
&  0.38004E+00_JPRB, 0.37896E+00_JPRB, 0.38761E+00_JPRB, 0.39002E+00_JPRB, 0.39154E+00_JPRB, 0.39374E+00_JPRB, 0.39369E+00_JPRB ,&
&  0.38033E+00_JPRB, 0.35569E+00_JPRB, 0.32821E+00_JPRB, 0.32876E+00_JPRB, 0.34326E+00_JPRB, 0.33882E+00_JPRB, 0.33736E+00_JPRB ,&
&  0.33451E+00_JPRB, 0.33019E+00_JPRB, 0.32601E+00_JPRB, 0.32423E+00_JPRB, 0.32148E+00_JPRB, 0.31712E+00_JPRB, 0.31550E+00_JPRB ,&
&  0.31466E+00_JPRB, 0.32264E+00_JPRB, 0.31888E+00_JPRB, 0.29844E+00_JPRB, 0.28226E+00_JPRB, 0.27355E+00_JPRB, 0.26249E+00_JPRB ,&
&  0.24982E+00_JPRB, 0.24201E+00_JPRB, 0.24057E+00_JPRB, 0.23915E+00_JPRB, 0.24096E+00_JPRB, 0.24204E+00_JPRB, 0.24318E+00_JPRB ,&
&  0.24421E+00_JPRB, 0.24418E+00_JPRB, 0.23978E+00_JPRB, 0.23403E+00_JPRB, 0.23087E+00_JPRB, 0.23078E+00_JPRB, 0.23313E+00_JPRB ,&
&  0.23848E+00_JPRB, 0.24444E+00_JPRB, 0.25816E+00_JPRB, 0.26583E+00_JPRB, 0.25780E+00_JPRB, 0.25169E+00_JPRB, 0.24481E+00_JPRB ,&
&  0.23514E+00_JPRB, 0.23129E+00_JPRB, 0.22813E+00_JPRB, 0.22636E+00_JPRB, 0.22712E+00_JPRB, 0.23014E+00_JPRB, 0.23185E+00_JPRB ,&
&  0.23415E+00_JPRB, 0.23651E+00_JPRB, 0.23745E+00_JPRB, 0.23826E+00_JPRB, 0.23893E+00_JPRB, 0.24063E+00_JPRB, 0.24238E+00_JPRB ,&
&  0.24391E+00_JPRB, 0.24659E+00_JPRB, 0.24988E+00_JPRB, 0.25431E+00_JPRB, 0.25822E+00_JPRB, 0.26058E+00_JPRB, 0.26706E+00_JPRB ,&
&  0.27493E+00_JPRB, 0.29127E+00_JPRB, 0.31832E+00_JPRB, 0.34341E+00_JPRB, 0.34064E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.35214E+00_JPRB, 0.35699E+00_JPRB, 0.36361E+00_JPRB, 0.37101E+00_JPRB, 0.37623E+00_JPRB, 0.37843E+00_JPRB, 0.38075E+00_JPRB ,&
&  0.38386E+00_JPRB, 0.38517E+00_JPRB, 0.38619E+00_JPRB, 0.38895E+00_JPRB, 0.39447E+00_JPRB, 0.39723E+00_JPRB, 0.40176E+00_JPRB ,&
&  0.40345E+00_JPRB, 0.40138E+00_JPRB, 0.39475E+00_JPRB, 0.39270E+00_JPRB, 0.39608E+00_JPRB, 0.40073E+00_JPRB, 0.40601E+00_JPRB ,&
&  0.41158E+00_JPRB, 0.41673E+00_JPRB, 0.41925E+00_JPRB, 0.42011E+00_JPRB, 0.42046E+00_JPRB, 0.42048E+00_JPRB, 0.41798E+00_JPRB ,&
&  0.41362E+00_JPRB, 0.37494E+00_JPRB, 0.34829E+00_JPRB, 0.35029E+00_JPRB, 0.36225E+00_JPRB, 0.35737E+00_JPRB, 0.35589E+00_JPRB ,&
&  0.35435E+00_JPRB, 0.35109E+00_JPRB, 0.34621E+00_JPRB, 0.34234E+00_JPRB, 0.33740E+00_JPRB, 0.33109E+00_JPRB, 0.32590E+00_JPRB ,&
&  0.32492E+00_JPRB, 0.32950E+00_JPRB, 0.33123E+00_JPRB, 0.33406E+00_JPRB, 0.33359E+00_JPRB, 0.33620E+00_JPRB, 0.33559E+00_JPRB ,&
&  0.33084E+00_JPRB, 0.32534E+00_JPRB, 0.32375E+00_JPRB, 0.32359E+00_JPRB, 0.32463E+00_JPRB, 0.32721E+00_JPRB, 0.32748E+00_JPRB ,&
&  0.31420E+00_JPRB, 0.30281E+00_JPRB, 0.28453E+00_JPRB, 0.26684E+00_JPRB, 0.25382E+00_JPRB, 0.24781E+00_JPRB, 0.24340E+00_JPRB ,&
&  0.24227E+00_JPRB, 0.24743E+00_JPRB, 0.26286E+00_JPRB, 0.28204E+00_JPRB, 0.29735E+00_JPRB, 0.27927E+00_JPRB, 0.26326E+00_JPRB ,&
&  0.25336E+00_JPRB, 0.24819E+00_JPRB, 0.24391E+00_JPRB, 0.24225E+00_JPRB, 0.24243E+00_JPRB, 0.24437E+00_JPRB, 0.24729E+00_JPRB ,&
&  0.25157E+00_JPRB, 0.25568E+00_JPRB, 0.25845E+00_JPRB, 0.25978E+00_JPRB, 0.26046E+00_JPRB, 0.26113E+00_JPRB, 0.26050E+00_JPRB ,&
&  0.26010E+00_JPRB, 0.26292E+00_JPRB, 0.26915E+00_JPRB, 0.27749E+00_JPRB, 0.28546E+00_JPRB, 0.29419E+00_JPRB, 0.30657E+00_JPRB ,&
&  0.32049E+00_JPRB, 0.33265E+00_JPRB, 0.33689E+00_JPRB, 0.34257E+00_JPRB, 0.34759E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.37214E+00_JPRB, 0.37636E+00_JPRB, 0.38022E+00_JPRB, 0.38450E+00_JPRB, 0.39062E+00_JPRB, 0.39359E+00_JPRB, 0.39446E+00_JPRB ,&
&  0.39577E+00_JPRB, 0.39645E+00_JPRB, 0.39810E+00_JPRB, 0.40243E+00_JPRB, 0.40820E+00_JPRB, 0.41217E+00_JPRB, 0.41763E+00_JPRB ,&
&  0.41774E+00_JPRB, 0.40946E+00_JPRB, 0.40179E+00_JPRB, 0.40146E+00_JPRB, 0.40680E+00_JPRB, 0.41242E+00_JPRB, 0.41962E+00_JPRB ,&
&  0.42680E+00_JPRB, 0.43213E+00_JPRB, 0.43521E+00_JPRB, 0.43677E+00_JPRB, 0.43882E+00_JPRB, 0.43865E+00_JPRB, 0.43578E+00_JPRB ,&
&  0.43195E+00_JPRB, 0.42356E+00_JPRB, 0.40414E+00_JPRB, 0.40127E+00_JPRB, 0.39261E+00_JPRB, 0.38502E+00_JPRB, 0.37973E+00_JPRB ,&
&  0.37542E+00_JPRB, 0.37203E+00_JPRB, 0.36833E+00_JPRB, 0.36535E+00_JPRB, 0.36197E+00_JPRB, 0.35560E+00_JPRB, 0.35261E+00_JPRB ,&
&  0.34993E+00_JPRB, 0.34767E+00_JPRB, 0.34559E+00_JPRB, 0.34631E+00_JPRB, 0.34721E+00_JPRB, 0.35049E+00_JPRB, 0.35223E+00_JPRB ,&
&  0.34948E+00_JPRB, 0.34613E+00_JPRB, 0.34351E+00_JPRB, 0.34517E+00_JPRB, 0.34735E+00_JPRB, 0.34886E+00_JPRB, 0.34945E+00_JPRB ,&
&  0.34727E+00_JPRB, 0.34314E+00_JPRB, 0.33890E+00_JPRB, 0.33387E+00_JPRB, 0.32949E+00_JPRB, 0.30378E+00_JPRB, 0.29228E+00_JPRB ,&
&  0.28839E+00_JPRB, 0.29705E+00_JPRB, 0.30978E+00_JPRB, 0.30812E+00_JPRB, 0.30947E+00_JPRB, 0.30935E+00_JPRB, 0.30685E+00_JPRB ,&
&  0.29840E+00_JPRB, 0.28464E+00_JPRB, 0.27500E+00_JPRB, 0.27062E+00_JPRB, 0.26738E+00_JPRB, 0.26801E+00_JPRB, 0.27344E+00_JPRB ,&
&  0.28287E+00_JPRB, 0.29435E+00_JPRB, 0.30314E+00_JPRB, 0.30721E+00_JPRB, 0.30630E+00_JPRB, 0.30322E+00_JPRB, 0.29909E+00_JPRB ,&
&  0.29429E+00_JPRB, 0.29934E+00_JPRB, 0.31282E+00_JPRB, 0.33282E+00_JPRB, 0.33532E+00_JPRB, 0.33987E+00_JPRB, 0.34293E+00_JPRB ,&
&  0.34787E+00_JPRB, 0.35407E+00_JPRB, 0.35953E+00_JPRB, 0.36550E+00_JPRB, 0.36924E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.39757E+00_JPRB, 0.39993E+00_JPRB, 0.40224E+00_JPRB, 0.40378E+00_JPRB, 0.40641E+00_JPRB, 0.41112E+00_JPRB, 0.41301E+00_JPRB ,&
&  0.41124E+00_JPRB, 0.40927E+00_JPRB, 0.40912E+00_JPRB, 0.41233E+00_JPRB, 0.41858E+00_JPRB, 0.42525E+00_JPRB, 0.43289E+00_JPRB ,&
&  0.42920E+00_JPRB, 0.41728E+00_JPRB, 0.41295E+00_JPRB, 0.41490E+00_JPRB, 0.41967E+00_JPRB, 0.42500E+00_JPRB, 0.43168E+00_JPRB ,&
&  0.43803E+00_JPRB, 0.44228E+00_JPRB, 0.44670E+00_JPRB, 0.45061E+00_JPRB, 0.45317E+00_JPRB, 0.45161E+00_JPRB, 0.44738E+00_JPRB ,&
&  0.44234E+00_JPRB, 0.43736E+00_JPRB, 0.43311E+00_JPRB, 0.41726E+00_JPRB, 0.41471E+00_JPRB, 0.40576E+00_JPRB, 0.39822E+00_JPRB ,&
&  0.39292E+00_JPRB, 0.38857E+00_JPRB, 0.38591E+00_JPRB, 0.38312E+00_JPRB, 0.37930E+00_JPRB, 0.37269E+00_JPRB, 0.37125E+00_JPRB ,&
&  0.37093E+00_JPRB, 0.37258E+00_JPRB, 0.36862E+00_JPRB, 0.36679E+00_JPRB, 0.36478E+00_JPRB, 0.36328E+00_JPRB, 0.36799E+00_JPRB ,&
&  0.37341E+00_JPRB, 0.37116E+00_JPRB, 0.36738E+00_JPRB, 0.36701E+00_JPRB, 0.37118E+00_JPRB, 0.37279E+00_JPRB, 0.37383E+00_JPRB ,&
&  0.37579E+00_JPRB, 0.37663E+00_JPRB, 0.37246E+00_JPRB, 0.36765E+00_JPRB, 0.35905E+00_JPRB, 0.34868E+00_JPRB, 0.34615E+00_JPRB ,&
&  0.34328E+00_JPRB, 0.33729E+00_JPRB, 0.33339E+00_JPRB, 0.33213E+00_JPRB, 0.33163E+00_JPRB, 0.33221E+00_JPRB, 0.33202E+00_JPRB ,&
&  0.33165E+00_JPRB, 0.32799E+00_JPRB, 0.32262E+00_JPRB, 0.31782E+00_JPRB, 0.31137E+00_JPRB, 0.31335E+00_JPRB, 0.32230E+00_JPRB ,&
&  0.33370E+00_JPRB, 0.34054E+00_JPRB, 0.34504E+00_JPRB, 0.34850E+00_JPRB, 0.35161E+00_JPRB, 0.35315E+00_JPRB, 0.35552E+00_JPRB ,&
&  0.35869E+00_JPRB, 0.36013E+00_JPRB, 0.35857E+00_JPRB, 0.35792E+00_JPRB, 0.36028E+00_JPRB, 0.36480E+00_JPRB, 0.37049E+00_JPRB ,&
&  0.37631E+00_JPRB, 0.38128E+00_JPRB, 0.38548E+00_JPRB, 0.39081E+00_JPRB, 0.39425E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.42961E+00_JPRB, 0.42969E+00_JPRB, 0.43027E+00_JPRB, 0.43166E+00_JPRB, 0.43259E+00_JPRB, 0.43174E+00_JPRB, 0.43153E+00_JPRB ,&
&  0.42847E+00_JPRB, 0.42392E+00_JPRB, 0.42100E+00_JPRB, 0.42220E+00_JPRB, 0.43093E+00_JPRB, 0.44238E+00_JPRB, 0.44949E+00_JPRB ,&
&  0.44558E+00_JPRB, 0.43372E+00_JPRB, 0.43085E+00_JPRB, 0.43320E+00_JPRB, 0.43541E+00_JPRB, 0.43959E+00_JPRB, 0.44575E+00_JPRB ,&
&  0.45137E+00_JPRB, 0.45522E+00_JPRB, 0.45939E+00_JPRB, 0.46109E+00_JPRB, 0.46146E+00_JPRB, 0.45456E+00_JPRB, 0.44360E+00_JPRB ,&
&  0.43318E+00_JPRB, 0.42578E+00_JPRB, 0.42135E+00_JPRB, 0.41787E+00_JPRB, 0.41282E+00_JPRB, 0.40772E+00_JPRB, 0.40106E+00_JPRB ,&
&  0.39768E+00_JPRB, 0.39725E+00_JPRB, 0.39802E+00_JPRB, 0.39619E+00_JPRB, 0.39384E+00_JPRB, 0.38918E+00_JPRB, 0.38547E+00_JPRB ,&
&  0.38437E+00_JPRB, 0.38612E+00_JPRB, 0.38802E+00_JPRB, 0.38781E+00_JPRB, 0.38507E+00_JPRB, 0.38328E+00_JPRB, 0.38505E+00_JPRB ,&
&  0.39111E+00_JPRB, 0.39104E+00_JPRB, 0.39008E+00_JPRB, 0.39064E+00_JPRB, 0.39260E+00_JPRB, 0.39447E+00_JPRB, 0.39400E+00_JPRB ,&
&  0.39354E+00_JPRB, 0.39380E+00_JPRB, 0.39443E+00_JPRB, 0.39668E+00_JPRB, 0.39707E+00_JPRB, 0.39524E+00_JPRB, 0.38998E+00_JPRB ,&
&  0.38178E+00_JPRB, 0.37129E+00_JPRB, 0.36691E+00_JPRB, 0.36379E+00_JPRB, 0.36073E+00_JPRB, 0.35989E+00_JPRB, 0.36099E+00_JPRB ,&
&  0.35946E+00_JPRB, 0.35324E+00_JPRB, 0.35032E+00_JPRB, 0.35044E+00_JPRB, 0.34879E+00_JPRB, 0.35030E+00_JPRB, 0.35334E+00_JPRB ,&
&  0.36269E+00_JPRB, 0.36775E+00_JPRB, 0.36924E+00_JPRB, 0.36718E+00_JPRB, 0.36756E+00_JPRB, 0.36928E+00_JPRB, 0.37340E+00_JPRB ,&
&  0.37969E+00_JPRB, 0.38642E+00_JPRB, 0.39321E+00_JPRB, 0.40069E+00_JPRB, 0.41033E+00_JPRB, 0.41977E+00_JPRB, 0.42328E+00_JPRB ,&
&  0.42328E+00_JPRB, 0.42332E+00_JPRB, 0.42423E+00_JPRB, 0.42550E+00_JPRB, 0.42834E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.45523E+00_JPRB, 0.45575E+00_JPRB, 0.45477E+00_JPRB, 0.45393E+00_JPRB, 0.45290E+00_JPRB, 0.45195E+00_JPRB, 0.45013E+00_JPRB ,&
&  0.44530E+00_JPRB, 0.43908E+00_JPRB, 0.43546E+00_JPRB, 0.43376E+00_JPRB, 0.43931E+00_JPRB, 0.44761E+00_JPRB, 0.45506E+00_JPRB ,&
&  0.45449E+00_JPRB, 0.44586E+00_JPRB, 0.44338E+00_JPRB, 0.44551E+00_JPRB, 0.44819E+00_JPRB, 0.45109E+00_JPRB, 0.45498E+00_JPRB ,&
&  0.45922E+00_JPRB, 0.46358E+00_JPRB, 0.46544E+00_JPRB, 0.46313E+00_JPRB, 0.45694E+00_JPRB, 0.44357E+00_JPRB, 0.43176E+00_JPRB ,&
&  0.42407E+00_JPRB, 0.41825E+00_JPRB, 0.41499E+00_JPRB, 0.41401E+00_JPRB, 0.41181E+00_JPRB, 0.40795E+00_JPRB, 0.40251E+00_JPRB ,&
&  0.39774E+00_JPRB, 0.39543E+00_JPRB, 0.39742E+00_JPRB, 0.39897E+00_JPRB, 0.39986E+00_JPRB, 0.39982E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39613E+00_JPRB, 0.39615E+00_JPRB, 0.39642E+00_JPRB, 0.39859E+00_JPRB, 0.39687E+00_JPRB, 0.39542E+00_JPRB, 0.39781E+00_JPRB ,&
&  0.40246E+00_JPRB, 0.40636E+00_JPRB, 0.40978E+00_JPRB, 0.41223E+00_JPRB, 0.41311E+00_JPRB, 0.41509E+00_JPRB, 0.41650E+00_JPRB ,&
&  0.41583E+00_JPRB, 0.41263E+00_JPRB, 0.40966E+00_JPRB, 0.41058E+00_JPRB, 0.41328E+00_JPRB, 0.41625E+00_JPRB, 0.42000E+00_JPRB ,&
&  0.42079E+00_JPRB, 0.41577E+00_JPRB, 0.41066E+00_JPRB, 0.40606E+00_JPRB, 0.40178E+00_JPRB, 0.39833E+00_JPRB, 0.39652E+00_JPRB ,&
&  0.39584E+00_JPRB, 0.39274E+00_JPRB, 0.38684E+00_JPRB, 0.38191E+00_JPRB, 0.38083E+00_JPRB, 0.37930E+00_JPRB, 0.37502E+00_JPRB ,&
&  0.37497E+00_JPRB, 0.37706E+00_JPRB, 0.37582E+00_JPRB, 0.37393E+00_JPRB, 0.37595E+00_JPRB, 0.38109E+00_JPRB, 0.39151E+00_JPRB ,&
&  0.40551E+00_JPRB, 0.42170E+00_JPRB, 0.43671E+00_JPRB, 0.44996E+00_JPRB, 0.45796E+00_JPRB, 0.45995E+00_JPRB, 0.45823E+00_JPRB ,&
&  0.45740E+00_JPRB, 0.45780E+00_JPRB, 0.45647E+00_JPRB, 0.45447E+00_JPRB, 0.45432E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.48139E+00_JPRB, 0.47981E+00_JPRB, 0.47832E+00_JPRB, 0.47660E+00_JPRB, 0.47408E+00_JPRB, 0.47187E+00_JPRB, 0.46775E+00_JPRB ,&
&  0.46316E+00_JPRB, 0.45933E+00_JPRB, 0.45776E+00_JPRB, 0.45749E+00_JPRB, 0.45910E+00_JPRB, 0.46319E+00_JPRB, 0.46988E+00_JPRB ,&
&  0.47014E+00_JPRB, 0.46390E+00_JPRB, 0.46121E+00_JPRB, 0.46151E+00_JPRB, 0.46408E+00_JPRB, 0.46595E+00_JPRB, 0.46890E+00_JPRB ,&
&  0.47091E+00_JPRB, 0.47163E+00_JPRB, 0.47115E+00_JPRB, 0.46468E+00_JPRB, 0.44842E+00_JPRB, 0.43851E+00_JPRB, 0.42961E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.41996E+00_JPRB, 0.41765E+00_JPRB, 0.41654E+00_JPRB, 0.41543E+00_JPRB, 0.41350E+00_JPRB, 0.41326E+00_JPRB ,&
&  0.41117E+00_JPRB, 0.40841E+00_JPRB, 0.40682E+00_JPRB, 0.40727E+00_JPRB, 0.40893E+00_JPRB, 0.41061E+00_JPRB, 0.41076E+00_JPRB ,&
&  0.41203E+00_JPRB, 0.41610E+00_JPRB, 0.41756E+00_JPRB, 0.41814E+00_JPRB, 0.41830E+00_JPRB, 0.41807E+00_JPRB, 0.41838E+00_JPRB ,&
&  0.42076E+00_JPRB, 0.42574E+00_JPRB, 0.42985E+00_JPRB, 0.43387E+00_JPRB, 0.43549E+00_JPRB, 0.43628E+00_JPRB, 0.43634E+00_JPRB ,&
&  0.43352E+00_JPRB, 0.42981E+00_JPRB, 0.42878E+00_JPRB, 0.43122E+00_JPRB, 0.43599E+00_JPRB, 0.44221E+00_JPRB, 0.44646E+00_JPRB ,&
&  0.44972E+00_JPRB, 0.45082E+00_JPRB, 0.44853E+00_JPRB, 0.44711E+00_JPRB, 0.44422E+00_JPRB, 0.43949E+00_JPRB, 0.43672E+00_JPRB ,&
&  0.43406E+00_JPRB, 0.43183E+00_JPRB, 0.43010E+00_JPRB, 0.42143E+00_JPRB, 0.41371E+00_JPRB, 0.40868E+00_JPRB, 0.40522E+00_JPRB ,&
&  0.40382E+00_JPRB, 0.40284E+00_JPRB, 0.40245E+00_JPRB, 0.40371E+00_JPRB, 0.40558E+00_JPRB, 0.41137E+00_JPRB, 0.42337E+00_JPRB ,&
&  0.44167E+00_JPRB, 0.45896E+00_JPRB, 0.46861E+00_JPRB, 0.47585E+00_JPRB, 0.48076E+00_JPRB, 0.48352E+00_JPRB, 0.48613E+00_JPRB ,&
&  0.48984E+00_JPRB, 0.49093E+00_JPRB, 0.48867E+00_JPRB, 0.48563E+00_JPRB, 0.48379E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.50577E+00_JPRB, 0.49784E+00_JPRB, 0.49200E+00_JPRB, 0.48739E+00_JPRB, 0.48376E+00_JPRB, 0.48273E+00_JPRB, 0.48130E+00_JPRB ,&
&  0.47968E+00_JPRB, 0.47753E+00_JPRB, 0.47666E+00_JPRB, 0.47410E+00_JPRB, 0.47158E+00_JPRB, 0.47448E+00_JPRB, 0.47995E+00_JPRB ,&
&  0.47882E+00_JPRB, 0.47267E+00_JPRB, 0.46782E+00_JPRB, 0.46805E+00_JPRB, 0.47172E+00_JPRB, 0.47449E+00_JPRB, 0.47777E+00_JPRB ,&
&  0.47776E+00_JPRB, 0.47727E+00_JPRB, 0.47613E+00_JPRB, 0.46117E+00_JPRB, 0.44641E+00_JPRB, 0.43917E+00_JPRB, 0.43498E+00_JPRB ,&
&  0.43414E+00_JPRB, 0.43829E+00_JPRB, 0.42345E+00_JPRB, 0.43249E+00_JPRB, 0.42808E+00_JPRB, 0.42172E+00_JPRB, 0.41833E+00_JPRB ,&
&  0.41621E+00_JPRB, 0.41359E+00_JPRB, 0.41179E+00_JPRB, 0.41371E+00_JPRB, 0.41647E+00_JPRB, 0.41754E+00_JPRB, 0.41958E+00_JPRB ,&
&  0.41964E+00_JPRB, 0.42242E+00_JPRB, 0.42473E+00_JPRB, 0.42989E+00_JPRB, 0.43395E+00_JPRB, 0.43656E+00_JPRB, 0.43982E+00_JPRB ,&
&  0.44423E+00_JPRB, 0.44835E+00_JPRB, 0.45226E+00_JPRB, 0.45554E+00_JPRB, 0.45686E+00_JPRB, 0.45667E+00_JPRB, 0.45271E+00_JPRB ,&
&  0.44957E+00_JPRB, 0.44945E+00_JPRB, 0.45165E+00_JPRB, 0.45599E+00_JPRB, 0.46056E+00_JPRB, 0.46307E+00_JPRB, 0.46513E+00_JPRB ,&
&  0.46736E+00_JPRB, 0.47094E+00_JPRB, 0.47135E+00_JPRB, 0.46974E+00_JPRB, 0.46750E+00_JPRB, 0.46473E+00_JPRB, 0.46013E+00_JPRB ,&
&  0.45433E+00_JPRB, 0.44996E+00_JPRB, 0.44556E+00_JPRB, 0.43509E+00_JPRB, 0.42697E+00_JPRB, 0.42617E+00_JPRB, 0.42754E+00_JPRB ,&
&  0.42698E+00_JPRB, 0.42544E+00_JPRB, 0.42659E+00_JPRB, 0.43205E+00_JPRB, 0.43139E+00_JPRB, 0.43698E+00_JPRB, 0.44973E+00_JPRB ,&
&  0.46318E+00_JPRB, 0.47543E+00_JPRB, 0.48561E+00_JPRB, 0.49646E+00_JPRB, 0.50748E+00_JPRB, 0.51615E+00_JPRB, 0.52374E+00_JPRB ,&
&  0.52745E+00_JPRB, 0.52598E+00_JPRB, 0.52057E+00_JPRB, 0.51692E+00_JPRB, 0.51363E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.54872E+00_JPRB, 0.53567E+00_JPRB, 0.52570E+00_JPRB, 0.51842E+00_JPRB, 0.51401E+00_JPRB, 0.51351E+00_JPRB, 0.51166E+00_JPRB ,&
&  0.50993E+00_JPRB, 0.50879E+00_JPRB, 0.50338E+00_JPRB, 0.49749E+00_JPRB, 0.49727E+00_JPRB, 0.50294E+00_JPRB, 0.50774E+00_JPRB ,&
&  0.50551E+00_JPRB, 0.49595E+00_JPRB, 0.48689E+00_JPRB, 0.48535E+00_JPRB, 0.48871E+00_JPRB, 0.49147E+00_JPRB, 0.49140E+00_JPRB ,&
&  0.49314E+00_JPRB, 0.49415E+00_JPRB, 0.48599E+00_JPRB, 0.46963E+00_JPRB, 0.46182E+00_JPRB, 0.45735E+00_JPRB, 0.46150E+00_JPRB ,&
&  0.46780E+00_JPRB, 0.46028E+00_JPRB, 0.47366E+00_JPRB, 0.47083E+00_JPRB, 0.46901E+00_JPRB, 0.46441E+00_JPRB, 0.45741E+00_JPRB ,&
&  0.45195E+00_JPRB, 0.44651E+00_JPRB, 0.44320E+00_JPRB, 0.44182E+00_JPRB, 0.44251E+00_JPRB, 0.44393E+00_JPRB, 0.44870E+00_JPRB ,&
&  0.45023E+00_JPRB, 0.44891E+00_JPRB, 0.45168E+00_JPRB, 0.45885E+00_JPRB, 0.46768E+00_JPRB, 0.47267E+00_JPRB, 0.47719E+00_JPRB ,&
&  0.48091E+00_JPRB, 0.48306E+00_JPRB, 0.48604E+00_JPRB, 0.48712E+00_JPRB, 0.48381E+00_JPRB, 0.48325E+00_JPRB, 0.48337E+00_JPRB ,&
&  0.48549E+00_JPRB, 0.48831E+00_JPRB, 0.49094E+00_JPRB, 0.49321E+00_JPRB, 0.49448E+00_JPRB, 0.49257E+00_JPRB, 0.49094E+00_JPRB ,&
&  0.49200E+00_JPRB, 0.49519E+00_JPRB, 0.49887E+00_JPRB, 0.50083E+00_JPRB, 0.49970E+00_JPRB, 0.49307E+00_JPRB, 0.48534E+00_JPRB ,&
&  0.47729E+00_JPRB, 0.46901E+00_JPRB, 0.46155E+00_JPRB, 0.45716E+00_JPRB, 0.45391E+00_JPRB, 0.45206E+00_JPRB, 0.45110E+00_JPRB ,&
&  0.45091E+00_JPRB, 0.45110E+00_JPRB, 0.45420E+00_JPRB, 0.46002E+00_JPRB, 0.46265E+00_JPRB, 0.47021E+00_JPRB, 0.48024E+00_JPRB ,&
&  0.49093E+00_JPRB, 0.50143E+00_JPRB, 0.51413E+00_JPRB, 0.53217E+00_JPRB, 0.55232E+00_JPRB, 0.56762E+00_JPRB, 0.57930E+00_JPRB ,&
&  0.58675E+00_JPRB, 0.58606E+00_JPRB, 0.57894E+00_JPRB, 0.57085E+00_JPRB, 0.56123E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.59219E+00_JPRB, 0.57284E+00_JPRB, 0.55698E+00_JPRB, 0.54579E+00_JPRB, 0.53967E+00_JPRB, 0.53884E+00_JPRB, 0.53647E+00_JPRB ,&
&  0.53515E+00_JPRB, 0.53247E+00_JPRB, 0.53075E+00_JPRB, 0.53325E+00_JPRB, 0.53468E+00_JPRB, 0.53553E+00_JPRB, 0.53746E+00_JPRB ,&
&  0.53286E+00_JPRB, 0.52114E+00_JPRB, 0.51169E+00_JPRB, 0.50946E+00_JPRB, 0.51098E+00_JPRB, 0.50751E+00_JPRB, 0.50635E+00_JPRB ,&
&  0.50938E+00_JPRB, 0.50161E+00_JPRB, 0.48863E+00_JPRB, 0.48026E+00_JPRB, 0.48070E+00_JPRB, 0.47843E+00_JPRB, 0.47542E+00_JPRB ,&
&  0.49473E+00_JPRB, 0.44571E+00_JPRB, 0.49051E+00_JPRB, 0.49599E+00_JPRB, 0.49947E+00_JPRB, 0.50117E+00_JPRB, 0.49942E+00_JPRB ,&
&  0.49486E+00_JPRB, 0.48940E+00_JPRB, 0.48399E+00_JPRB, 0.47913E+00_JPRB, 0.47661E+00_JPRB, 0.47304E+00_JPRB, 0.46997E+00_JPRB ,&
&  0.47172E+00_JPRB, 0.47429E+00_JPRB, 0.48013E+00_JPRB, 0.48649E+00_JPRB, 0.49441E+00_JPRB, 0.49735E+00_JPRB, 0.49994E+00_JPRB ,&
&  0.50284E+00_JPRB, 0.50467E+00_JPRB, 0.50488E+00_JPRB, 0.50405E+00_JPRB, 0.50278E+00_JPRB, 0.50758E+00_JPRB, 0.51380E+00_JPRB ,&
&  0.51839E+00_JPRB, 0.52198E+00_JPRB, 0.52660E+00_JPRB, 0.52768E+00_JPRB, 0.52781E+00_JPRB, 0.52484E+00_JPRB, 0.51987E+00_JPRB ,&
&  0.51590E+00_JPRB, 0.51481E+00_JPRB, 0.51436E+00_JPRB, 0.50974E+00_JPRB, 0.50299E+00_JPRB, 0.49690E+00_JPRB, 0.48960E+00_JPRB ,&
&  0.48026E+00_JPRB, 0.47280E+00_JPRB, 0.46908E+00_JPRB, 0.46833E+00_JPRB, 0.46975E+00_JPRB, 0.47072E+00_JPRB, 0.47164E+00_JPRB ,&
&  0.47459E+00_JPRB, 0.47556E+00_JPRB, 0.47740E+00_JPRB, 0.48198E+00_JPRB, 0.48580E+00_JPRB, 0.49194E+00_JPRB, 0.50028E+00_JPRB ,&
&  0.51160E+00_JPRB, 0.52454E+00_JPRB, 0.53665E+00_JPRB, 0.55633E+00_JPRB, 0.58324E+00_JPRB, 0.60306E+00_JPRB, 0.61478E+00_JPRB ,&
&  0.62532E+00_JPRB, 0.62776E+00_JPRB, 0.62260E+00_JPRB, 0.61654E+00_JPRB, 0.60589E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.64406E+00_JPRB, 0.62967E+00_JPRB, 0.61041E+00_JPRB, 0.59661E+00_JPRB, 0.58923E+00_JPRB, 0.58608E+00_JPRB, 0.58268E+00_JPRB ,&
&  0.57783E+00_JPRB, 0.57804E+00_JPRB, 0.57876E+00_JPRB, 0.57944E+00_JPRB, 0.58009E+00_JPRB, 0.57779E+00_JPRB, 0.57119E+00_JPRB ,&
&  0.56343E+00_JPRB, 0.55288E+00_JPRB, 0.55150E+00_JPRB, 0.55118E+00_JPRB, 0.54525E+00_JPRB, 0.54118E+00_JPRB, 0.54093E+00_JPRB ,&
&  0.53469E+00_JPRB, 0.52514E+00_JPRB, 0.50776E+00_JPRB, 0.50212E+00_JPRB, 0.50462E+00_JPRB, 0.50472E+00_JPRB, 0.51071E+00_JPRB ,&
&  0.48959E+00_JPRB, 0.46250E+00_JPRB, 0.51083E+00_JPRB, 0.53985E+00_JPRB, 0.54992E+00_JPRB, 0.55853E+00_JPRB, 0.56871E+00_JPRB ,&
&  0.56345E+00_JPRB, 0.56485E+00_JPRB, 0.55256E+00_JPRB, 0.53199E+00_JPRB, 0.51839E+00_JPRB, 0.51413E+00_JPRB, 0.51462E+00_JPRB ,&
&  0.52142E+00_JPRB, 0.52673E+00_JPRB, 0.52888E+00_JPRB, 0.52969E+00_JPRB, 0.53151E+00_JPRB, 0.52901E+00_JPRB, 0.52737E+00_JPRB ,&
&  0.52912E+00_JPRB, 0.53265E+00_JPRB, 0.53448E+00_JPRB, 0.53860E+00_JPRB, 0.54681E+00_JPRB, 0.55931E+00_JPRB, 0.57083E+00_JPRB ,&
&  0.57936E+00_JPRB, 0.58477E+00_JPRB, 0.59204E+00_JPRB, 0.59546E+00_JPRB, 0.59342E+00_JPRB, 0.58424E+00_JPRB, 0.57281E+00_JPRB ,&
&  0.56071E+00_JPRB, 0.55151E+00_JPRB, 0.54499E+00_JPRB, 0.54054E+00_JPRB, 0.53348E+00_JPRB, 0.52658E+00_JPRB, 0.51828E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.50286E+00_JPRB, 0.49907E+00_JPRB, 0.49780E+00_JPRB, 0.50003E+00_JPRB, 0.50370E+00_JPRB, 0.50872E+00_JPRB ,&
&  0.51231E+00_JPRB, 0.51102E+00_JPRB, 0.51104E+00_JPRB, 0.51555E+00_JPRB, 0.52094E+00_JPRB, 0.53014E+00_JPRB, 0.54242E+00_JPRB ,&
&  0.55723E+00_JPRB, 0.57130E+00_JPRB, 0.58609E+00_JPRB, 0.61569E+00_JPRB, 0.64483E+00_JPRB, 0.66821E+00_JPRB, 0.68019E+00_JPRB ,&
&  0.68586E+00_JPRB, 0.67715E+00_JPRB, 0.67615E+00_JPRB, 0.66670E+00_JPRB, 0.65419E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.64501E+00_JPRB, 0.64286E+00_JPRB, 0.63119E+00_JPRB, 0.62405E+00_JPRB, 0.61870E+00_JPRB, 0.61297E+00_JPRB, 0.60385E+00_JPRB ,&
&  0.59802E+00_JPRB, 0.59681E+00_JPRB, 0.59583E+00_JPRB, 0.59652E+00_JPRB, 0.59791E+00_JPRB, 0.59408E+00_JPRB, 0.58743E+00_JPRB ,&
&  0.58019E+00_JPRB, 0.57309E+00_JPRB, 0.57468E+00_JPRB, 0.57443E+00_JPRB, 0.56987E+00_JPRB, 0.56605E+00_JPRB, 0.56077E+00_JPRB ,&
&  0.54683E+00_JPRB, 0.53321E+00_JPRB, 0.52237E+00_JPRB, 0.52517E+00_JPRB, 0.53159E+00_JPRB, 0.52686E+00_JPRB, 0.53983E+00_JPRB ,&
&  0.58297E+00_JPRB, 0.50511E+00_JPRB, 0.53136E+00_JPRB, 0.58293E+00_JPRB, 0.60506E+00_JPRB, 0.62802E+00_JPRB, 0.64319E+00_JPRB ,&
&  0.65221E+00_JPRB, 0.65671E+00_JPRB, 0.63357E+00_JPRB, 0.60679E+00_JPRB, 0.58510E+00_JPRB, 0.56815E+00_JPRB, 0.55642E+00_JPRB ,&
&  0.55291E+00_JPRB, 0.55804E+00_JPRB, 0.56223E+00_JPRB, 0.56082E+00_JPRB, 0.55930E+00_JPRB, 0.55547E+00_JPRB, 0.55299E+00_JPRB ,&
&  0.55192E+00_JPRB, 0.55377E+00_JPRB, 0.55772E+00_JPRB, 0.56728E+00_JPRB, 0.57658E+00_JPRB, 0.58737E+00_JPRB, 0.60188E+00_JPRB ,&
&  0.61925E+00_JPRB, 0.63625E+00_JPRB, 0.64944E+00_JPRB, 0.65250E+00_JPRB, 0.64712E+00_JPRB, 0.63670E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.60519E+00_JPRB, 0.58823E+00_JPRB, 0.57789E+00_JPRB, 0.57279E+00_JPRB, 0.56652E+00_JPRB, 0.56024E+00_JPRB, 0.55484E+00_JPRB ,&
&  0.55026E+00_JPRB, 0.54481E+00_JPRB, 0.54541E+00_JPRB, 0.54639E+00_JPRB, 0.54679E+00_JPRB, 0.54291E+00_JPRB, 0.53862E+00_JPRB ,&
&  0.53489E+00_JPRB, 0.53241E+00_JPRB, 0.53408E+00_JPRB, 0.53966E+00_JPRB, 0.54837E+00_JPRB, 0.56310E+00_JPRB, 0.58183E+00_JPRB ,&
&  0.59593E+00_JPRB, 0.60859E+00_JPRB, 0.62240E+00_JPRB, 0.65427E+00_JPRB, 0.67511E+00_JPRB, 0.68884E+00_JPRB, 0.69462E+00_JPRB ,&
&  0.69327E+00_JPRB, 0.69452E+00_JPRB, 0.55038E+00_JPRB, 0.61561E+00_JPRB, 0.63465E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.65924E+00_JPRB, 0.65637E+00_JPRB, 0.64972E+00_JPRB, 0.64713E+00_JPRB, 0.64655E+00_JPRB, 0.63445E+00_JPRB, 0.62447E+00_JPRB ,&
&  0.61926E+00_JPRB, 0.62040E+00_JPRB, 0.62430E+00_JPRB, 0.62623E+00_JPRB, 0.62233E+00_JPRB, 0.61384E+00_JPRB, 0.60944E+00_JPRB ,&
&  0.60171E+00_JPRB, 0.59456E+00_JPRB, 0.59321E+00_JPRB, 0.59312E+00_JPRB, 0.58815E+00_JPRB, 0.58070E+00_JPRB, 0.57444E+00_JPRB ,&
&  0.56168E+00_JPRB, 0.55384E+00_JPRB, 0.55234E+00_JPRB, 0.55541E+00_JPRB, 0.55861E+00_JPRB, 0.55726E+00_JPRB, 0.57261E+00_JPRB ,&
&  0.60796E+00_JPRB, 0.53737E+00_JPRB, 0.54485E+00_JPRB, 0.63871E+00_JPRB, 0.69441E+00_JPRB, 0.70924E+00_JPRB, 0.72621E+00_JPRB ,&
&  0.74346E+00_JPRB, 0.73661E+00_JPRB, 0.72060E+00_JPRB, 0.68616E+00_JPRB, 0.65979E+00_JPRB, 0.64418E+00_JPRB, 0.63179E+00_JPRB ,&
&  0.62448E+00_JPRB, 0.61906E+00_JPRB, 0.61904E+00_JPRB, 0.61388E+00_JPRB, 0.61208E+00_JPRB, 0.60411E+00_JPRB, 0.60173E+00_JPRB ,&
&  0.60243E+00_JPRB, 0.60753E+00_JPRB, 0.61295E+00_JPRB, 0.61617E+00_JPRB, 0.62499E+00_JPRB, 0.63924E+00_JPRB, 0.66184E+00_JPRB ,&
&  0.68918E+00_JPRB, 0.71761E+00_JPRB, 0.73098E+00_JPRB, 0.72855E+00_JPRB, 0.71799E+00_JPRB, 0.70326E+00_JPRB, 0.68234E+00_JPRB ,&
&  0.65774E+00_JPRB, 0.63414E+00_JPRB, 0.61801E+00_JPRB, 0.60998E+00_JPRB, 0.60596E+00_JPRB, 0.60359E+00_JPRB, 0.60072E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.59801E+00_JPRB, 0.59957E+00_JPRB, 0.59958E+00_JPRB, 0.59901E+00_JPRB, 0.59169E+00_JPRB, 0.57920E+00_JPRB ,&
&  0.57033E+00_JPRB, 0.56841E+00_JPRB, 0.57587E+00_JPRB, 0.58198E+00_JPRB, 0.59007E+00_JPRB, 0.60720E+00_JPRB, 0.62643E+00_JPRB ,&
&  0.63862E+00_JPRB, 0.65352E+00_JPRB, 0.67714E+00_JPRB, 0.67338E+00_JPRB, 0.70903E+00_JPRB, 0.70783E+00_JPRB, 0.70883E+00_JPRB ,&
&  0.69877E+00_JPRB, 0.68356E+00_JPRB, 0.67575E+00_JPRB, 0.58022E+00_JPRB, 0.64095E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.67097E+00_JPRB, 0.66868E+00_JPRB, 0.66326E+00_JPRB, 0.66043E+00_JPRB, 0.65639E+00_JPRB, 0.64564E+00_JPRB, 0.63764E+00_JPRB ,&
&  0.63604E+00_JPRB, 0.63315E+00_JPRB, 0.63069E+00_JPRB, 0.62527E+00_JPRB, 0.61686E+00_JPRB, 0.61134E+00_JPRB, 0.61223E+00_JPRB ,&
&  0.61030E+00_JPRB, 0.60990E+00_JPRB, 0.60991E+00_JPRB, 0.60597E+00_JPRB, 0.60049E+00_JPRB, 0.59195E+00_JPRB, 0.57836E+00_JPRB ,&
&  0.56207E+00_JPRB, 0.55757E+00_JPRB, 0.56624E+00_JPRB, 0.57302E+00_JPRB, 0.57700E+00_JPRB, 0.57918E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.61711E+00_JPRB, 0.59047E+00_JPRB, 0.66188E+00_JPRB, 0.77113E+00_JPRB, 0.83551E+00_JPRB, 0.83135E+00_JPRB, 0.82979E+00_JPRB ,&
&  0.80842E+00_JPRB, 0.78135E+00_JPRB, 0.75079E+00_JPRB, 0.71395E+00_JPRB, 0.68251E+00_JPRB, 0.66396E+00_JPRB, 0.65841E+00_JPRB ,&
&  0.65605E+00_JPRB, 0.65332E+00_JPRB, 0.65399E+00_JPRB, 0.64858E+00_JPRB, 0.64337E+00_JPRB, 0.63905E+00_JPRB, 0.64043E+00_JPRB ,&
&  0.64283E+00_JPRB, 0.64587E+00_JPRB, 0.64721E+00_JPRB, 0.64969E+00_JPRB, 0.65408E+00_JPRB, 0.66650E+00_JPRB, 0.69242E+00_JPRB ,&
&  0.72585E+00_JPRB, 0.76011E+00_JPRB, 0.77151E+00_JPRB, 0.76261E+00_JPRB, 0.75382E+00_JPRB, 0.74849E+00_JPRB, 0.72485E+00_JPRB ,&
&  0.69368E+00_JPRB, 0.67305E+00_JPRB, 0.65668E+00_JPRB, 0.64340E+00_JPRB, 0.63497E+00_JPRB, 0.63161E+00_JPRB, 0.62955E+00_JPRB ,&
&  0.62836E+00_JPRB, 0.62668E+00_JPRB, 0.62397E+00_JPRB, 0.62280E+00_JPRB, 0.62174E+00_JPRB, 0.61500E+00_JPRB, 0.60683E+00_JPRB ,&
&  0.59786E+00_JPRB, 0.59988E+00_JPRB, 0.61137E+00_JPRB, 0.61755E+00_JPRB, 0.62327E+00_JPRB, 0.63587E+00_JPRB, 0.65252E+00_JPRB ,&
&  0.66226E+00_JPRB, 0.67854E+00_JPRB, 0.69181E+00_JPRB, 0.73515E+00_JPRB, 0.73913E+00_JPRB, 0.72235E+00_JPRB, 0.71174E+00_JPRB ,&
&  0.69140E+00_JPRB, 0.67039E+00_JPRB, 0.65714E+00_JPRB, 0.66873E+00_JPRB, 0.63527E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.75335E+00_JPRB, 0.71991E+00_JPRB, 0.68683E+00_JPRB, 0.66580E+00_JPRB, 0.65719E+00_JPRB, 0.65433E+00_JPRB, 0.65289E+00_JPRB ,&
&  0.64937E+00_JPRB, 0.64470E+00_JPRB, 0.62989E+00_JPRB, 0.61949E+00_JPRB, 0.61276E+00_JPRB, 0.61164E+00_JPRB, 0.61051E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.61200E+00_JPRB, 0.60817E+00_JPRB, 0.60742E+00_JPRB, 0.60914E+00_JPRB, 0.60482E+00_JPRB, 0.58960E+00_JPRB ,&
&  0.57454E+00_JPRB, 0.57683E+00_JPRB, 0.58148E+00_JPRB, 0.58404E+00_JPRB, 0.58950E+00_JPRB, 0.59866E+00_JPRB, 0.61400E+00_JPRB ,&
&  0.63348E+00_JPRB, 0.63129E+00_JPRB, 0.80693E+00_JPRB, 0.92136E+00_JPRB, 0.97441E+00_JPRB, 0.10001E+01_JPRB, 0.96502E+00_JPRB ,&
&  0.91690E+00_JPRB, 0.85007E+00_JPRB, 0.79515E+00_JPRB, 0.74086E+00_JPRB, 0.70714E+00_JPRB, 0.68495E+00_JPRB, 0.67151E+00_JPRB ,&
&  0.66604E+00_JPRB, 0.66427E+00_JPRB, 0.66228E+00_JPRB, 0.65891E+00_JPRB, 0.65719E+00_JPRB, 0.65684E+00_JPRB, 0.65773E+00_JPRB ,&
&  0.66052E+00_JPRB, 0.66621E+00_JPRB, 0.66881E+00_JPRB, 0.67015E+00_JPRB, 0.67258E+00_JPRB, 0.70693E+00_JPRB, 0.75970E+00_JPRB ,&
&  0.80623E+00_JPRB, 0.83334E+00_JPRB, 0.82480E+00_JPRB, 0.81633E+00_JPRB, 0.81007E+00_JPRB, 0.79100E+00_JPRB, 0.76834E+00_JPRB ,&
&  0.74378E+00_JPRB, 0.71738E+00_JPRB, 0.68934E+00_JPRB, 0.66808E+00_JPRB, 0.65629E+00_JPRB, 0.65112E+00_JPRB, 0.64867E+00_JPRB ,&
&  0.64940E+00_JPRB, 0.65342E+00_JPRB, 0.65240E+00_JPRB, 0.65295E+00_JPRB, 0.64956E+00_JPRB, 0.64573E+00_JPRB, 0.64496E+00_JPRB ,&
&  0.64335E+00_JPRB, 0.64157E+00_JPRB, 0.65282E+00_JPRB, 0.66039E+00_JPRB, 0.66700E+00_JPRB, 0.67623E+00_JPRB, 0.68461E+00_JPRB ,&
&  0.69454E+00_JPRB, 0.71605E+00_JPRB, 0.69563E+00_JPRB, 0.71146E+00_JPRB, 0.77993E+00_JPRB, 0.74529E+00_JPRB, 0.70447E+00_JPRB ,&
&  0.67134E+00_JPRB, 0.65765E+00_JPRB, 0.65883E+00_JPRB, 0.68895E+00_JPRB, 0.74649E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.71787E+00_JPRB, 0.67716E+00_JPRB, 0.65559E+00_JPRB, 0.65374E+00_JPRB, 0.66017E+00_JPRB, 0.65769E+00_JPRB, 0.66126E+00_JPRB ,&
&  0.65563E+00_JPRB, 0.64395E+00_JPRB, 0.63071E+00_JPRB, 0.61815E+00_JPRB, 0.61254E+00_JPRB, 0.60834E+00_JPRB, 0.60542E+00_JPRB ,&
&  0.60378E+00_JPRB, 0.60246E+00_JPRB, 0.59827E+00_JPRB, 0.60208E+00_JPRB, 0.60441E+00_JPRB, 0.60343E+00_JPRB, 0.58330E+00_JPRB ,&
&  0.57456E+00_JPRB, 0.57640E+00_JPRB, 0.58078E+00_JPRB, 0.58490E+00_JPRB, 0.59175E+00_JPRB, 0.60357E+00_JPRB, 0.61526E+00_JPRB ,&
&  0.63166E+00_JPRB, 0.70194E+00_JPRB, 0.91224E+00_JPRB, 0.10614E+01_JPRB, 0.11003E+01_JPRB, 0.10730E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.96297E+00_JPRB, 0.92157E+00_JPRB, 0.88244E+00_JPRB, 0.80744E+00_JPRB, 0.73750E+00_JPRB, 0.70213E+00_JPRB, 0.68667E+00_JPRB ,&
&  0.68081E+00_JPRB, 0.67990E+00_JPRB, 0.67787E+00_JPRB, 0.67520E+00_JPRB, 0.67185E+00_JPRB, 0.67103E+00_JPRB, 0.67448E+00_JPRB ,&
&  0.67899E+00_JPRB, 0.67650E+00_JPRB, 0.67709E+00_JPRB, 0.67196E+00_JPRB, 0.72337E+00_JPRB, 0.78985E+00_JPRB, 0.87259E+00_JPRB ,&
&  0.90438E+00_JPRB, 0.91977E+00_JPRB, 0.89978E+00_JPRB, 0.86358E+00_JPRB, 0.82740E+00_JPRB, 0.79555E+00_JPRB, 0.76931E+00_JPRB ,&
&  0.74630E+00_JPRB, 0.72524E+00_JPRB, 0.70710E+00_JPRB, 0.70187E+00_JPRB, 0.70518E+00_JPRB, 0.69794E+00_JPRB, 0.69349E+00_JPRB ,&
&  0.69269E+00_JPRB, 0.69480E+00_JPRB, 0.69318E+00_JPRB, 0.68322E+00_JPRB, 0.66983E+00_JPRB, 0.66285E+00_JPRB, 0.66091E+00_JPRB ,&
&  0.66794E+00_JPRB, 0.67118E+00_JPRB, 0.69419E+00_JPRB, 0.69923E+00_JPRB, 0.70347E+00_JPRB, 0.70579E+00_JPRB, 0.71372E+00_JPRB ,&
&  0.73101E+00_JPRB, 0.72816E+00_JPRB, 0.72540E+00_JPRB, 0.73446E+00_JPRB, 0.73405E+00_JPRB, 0.75125E+00_JPRB, 0.68930E+00_JPRB ,&
&  0.65309E+00_JPRB, 0.64349E+00_JPRB, 0.64959E+00_JPRB, 0.67654E+00_JPRB, 0.72645E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.68282E+00_JPRB, 0.66187E+00_JPRB, 0.65457E+00_JPRB, 0.65683E+00_JPRB, 0.65330E+00_JPRB, 0.65631E+00_JPRB, 0.66140E+00_JPRB ,&
&  0.66036E+00_JPRB, 0.64788E+00_JPRB, 0.63564E+00_JPRB, 0.62156E+00_JPRB, 0.61685E+00_JPRB, 0.60961E+00_JPRB, 0.60813E+00_JPRB ,&
&  0.60191E+00_JPRB, 0.59936E+00_JPRB, 0.59416E+00_JPRB, 0.59312E+00_JPRB, 0.59370E+00_JPRB, 0.58982E+00_JPRB, 0.57200E+00_JPRB ,&
&  0.56882E+00_JPRB, 0.56813E+00_JPRB, 0.57501E+00_JPRB, 0.58539E+00_JPRB, 0.59778E+00_JPRB, 0.60486E+00_JPRB, 0.60818E+00_JPRB ,&
&  0.64467E+00_JPRB, 0.71777E+00_JPRB, 0.11021E+01_JPRB, 0.11959E+01_JPRB, 0.12129E+01_JPRB, 0.11163E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.97869E+00_JPRB, 0.94645E+00_JPRB, 0.93372E+00_JPRB, 0.87382E+00_JPRB, 0.79575E+00_JPRB, 0.73633E+00_JPRB, 0.71902E+00_JPRB ,&
&  0.70956E+00_JPRB, 0.69983E+00_JPRB, 0.69616E+00_JPRB, 0.69603E+00_JPRB, 0.69186E+00_JPRB, 0.68946E+00_JPRB, 0.69496E+00_JPRB ,&
&  0.69724E+00_JPRB, 0.69945E+00_JPRB, 0.69987E+00_JPRB, 0.72126E+00_JPRB, 0.76901E+00_JPRB, 0.73241E+00_JPRB, 0.84154E+00_JPRB ,&
&  0.10616E+01_JPRB, 0.10473E+01_JPRB, 0.96978E+00_JPRB, 0.87158E+00_JPRB, 0.81207E+00_JPRB, 0.78808E+00_JPRB, 0.77060E+00_JPRB ,&
&  0.75616E+00_JPRB, 0.74132E+00_JPRB, 0.73187E+00_JPRB, 0.74012E+00_JPRB, 0.74144E+00_JPRB, 0.73270E+00_JPRB, 0.72368E+00_JPRB ,&
&  0.72451E+00_JPRB, 0.72649E+00_JPRB, 0.71685E+00_JPRB, 0.69571E+00_JPRB, 0.68177E+00_JPRB, 0.68062E+00_JPRB, 0.67870E+00_JPRB ,&
&  0.67820E+00_JPRB, 0.68310E+00_JPRB, 0.69341E+00_JPRB, 0.68130E+00_JPRB, 0.69893E+00_JPRB, 0.72968E+00_JPRB, 0.73889E+00_JPRB ,&
&  0.72287E+00_JPRB, 0.71340E+00_JPRB, 0.73510E+00_JPRB, 0.72603E+00_JPRB, 0.73295E+00_JPRB, 0.74333E+00_JPRB, 0.67515E+00_JPRB ,&
&  0.64034E+00_JPRB, 0.63612E+00_JPRB, 0.65792E+00_JPRB, 0.68667E+00_JPRB, 0.72461E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.66472E+00_JPRB, 0.65316E+00_JPRB, 0.65038E+00_JPRB, 0.64667E+00_JPRB, 0.63773E+00_JPRB, 0.62756E+00_JPRB, 0.63147E+00_JPRB ,&
&  0.63727E+00_JPRB, 0.64127E+00_JPRB, 0.63349E+00_JPRB, 0.61839E+00_JPRB, 0.61571E+00_JPRB, 0.61156E+00_JPRB, 0.60635E+00_JPRB ,&
&  0.60188E+00_JPRB, 0.60150E+00_JPRB, 0.59439E+00_JPRB, 0.59028E+00_JPRB, 0.59289E+00_JPRB, 0.59130E+00_JPRB, 0.58432E+00_JPRB ,&
&  0.57645E+00_JPRB, 0.57096E+00_JPRB, 0.57557E+00_JPRB, 0.58044E+00_JPRB, 0.59236E+00_JPRB, 0.59506E+00_JPRB, 0.59511E+00_JPRB ,&
&  0.63950E+00_JPRB, 0.57330E+00_JPRB, 0.11643E+01_JPRB, 0.14734E+01_JPRB, 0.14282E+01_JPRB, 0.12109E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.96921E+00_JPRB, 0.93321E+00_JPRB, 0.88254E+00_JPRB, 0.83521E+00_JPRB, 0.78324E+00_JPRB, 0.76071E+00_JPRB ,&
&  0.74516E+00_JPRB, 0.72883E+00_JPRB, 0.72539E+00_JPRB, 0.72313E+00_JPRB, 0.72475E+00_JPRB, 0.72219E+00_JPRB, 0.72922E+00_JPRB ,&
&  0.73148E+00_JPRB, 0.73788E+00_JPRB, 0.76364E+00_JPRB, 0.81815E+00_JPRB, 0.84095E+00_JPRB, 0.80086E+00_JPRB, 0.84379E+00_JPRB ,&
&  0.94779E+00_JPRB, 0.10556E+01_JPRB, 0.96059E+00_JPRB, 0.84928E+00_JPRB, 0.79697E+00_JPRB, 0.78140E+00_JPRB, 0.76594E+00_JPRB ,&
&  0.74996E+00_JPRB, 0.74151E+00_JPRB, 0.75329E+00_JPRB, 0.75374E+00_JPRB, 0.73898E+00_JPRB, 0.71901E+00_JPRB, 0.72168E+00_JPRB ,&
&  0.72774E+00_JPRB, 0.72316E+00_JPRB, 0.70747E+00_JPRB, 0.70078E+00_JPRB, 0.69555E+00_JPRB, 0.68230E+00_JPRB, 0.67649E+00_JPRB ,&
&  0.67111E+00_JPRB, 0.68433E+00_JPRB, 0.68879E+00_JPRB, 0.68835E+00_JPRB, 0.69703E+00_JPRB, 0.71515E+00_JPRB, 0.72634E+00_JPRB ,&
&  0.73608E+00_JPRB, 0.74086E+00_JPRB, 0.72652E+00_JPRB, 0.72715E+00_JPRB, 0.69268E+00_JPRB, 0.71137E+00_JPRB, 0.65969E+00_JPRB ,&
&  0.64206E+00_JPRB, 0.64243E+00_JPRB, 0.65393E+00_JPRB, 0.66560E+00_JPRB, 0.68491E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.64623E+00_JPRB, 0.64085E+00_JPRB, 0.64165E+00_JPRB, 0.63519E+00_JPRB, 0.62089E+00_JPRB, 0.60870E+00_JPRB, 0.60900E+00_JPRB ,&
&  0.60823E+00_JPRB, 0.61274E+00_JPRB, 0.62004E+00_JPRB, 0.61139E+00_JPRB, 0.61046E+00_JPRB, 0.61260E+00_JPRB, 0.61093E+00_JPRB ,&
&  0.60218E+00_JPRB, 0.59805E+00_JPRB, 0.59408E+00_JPRB, 0.59093E+00_JPRB, 0.59006E+00_JPRB, 0.58196E+00_JPRB, 0.57826E+00_JPRB ,&
&  0.57735E+00_JPRB, 0.58132E+00_JPRB, 0.58534E+00_JPRB, 0.58655E+00_JPRB, 0.59461E+00_JPRB, 0.59528E+00_JPRB, 0.59510E+00_JPRB ,&
&  0.62617E+00_JPRB, 0.11085E+01_JPRB, 0.10128E+01_JPRB, 0.17192E+01_JPRB, 0.15141E+01_JPRB, 0.12284E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.97662E+00_JPRB, 0.98832E+00_JPRB, 0.93375E+00_JPRB, 0.88801E+00_JPRB, 0.85025E+00_JPRB, 0.80678E+00_JPRB, 0.76634E+00_JPRB ,&
&  0.73586E+00_JPRB, 0.72315E+00_JPRB, 0.72681E+00_JPRB, 0.73512E+00_JPRB, 0.75053E+00_JPRB, 0.76644E+00_JPRB, 0.77462E+00_JPRB ,&
&  0.79185E+00_JPRB, 0.83281E+00_JPRB, 0.89552E+00_JPRB, 0.97282E+00_JPRB, 0.89659E+00_JPRB, 0.83390E+00_JPRB, 0.88116E+00_JPRB ,&
&  0.90738E+00_JPRB, 0.10120E+01_JPRB, 0.91766E+00_JPRB, 0.82572E+00_JPRB, 0.79429E+00_JPRB, 0.77963E+00_JPRB, 0.75913E+00_JPRB ,&
&  0.74410E+00_JPRB, 0.74498E+00_JPRB, 0.76384E+00_JPRB, 0.74572E+00_JPRB, 0.71800E+00_JPRB, 0.71140E+00_JPRB, 0.71154E+00_JPRB ,&
&  0.71366E+00_JPRB, 0.70281E+00_JPRB, 0.71823E+00_JPRB, 0.72148E+00_JPRB, 0.71667E+00_JPRB, 0.70416E+00_JPRB, 0.69206E+00_JPRB ,&
&  0.68863E+00_JPRB, 0.70160E+00_JPRB, 0.69199E+00_JPRB, 0.67825E+00_JPRB, 0.68188E+00_JPRB, 0.68602E+00_JPRB, 0.68574E+00_JPRB ,&
&  0.72907E+00_JPRB, 0.73891E+00_JPRB, 0.72104E+00_JPRB, 0.70859E+00_JPRB, 0.69284E+00_JPRB, 0.71187E+00_JPRB, 0.65422E+00_JPRB ,&
&  0.64287E+00_JPRB, 0.64058E+00_JPRB, 0.64647E+00_JPRB, 0.65743E+00_JPRB, 0.65936E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.64171E+00_JPRB, 0.63750E+00_JPRB, 0.63826E+00_JPRB, 0.62334E+00_JPRB, 0.60492E+00_JPRB, 0.59437E+00_JPRB, 0.59295E+00_JPRB ,&
&  0.59635E+00_JPRB, 0.60776E+00_JPRB, 0.61576E+00_JPRB, 0.60203E+00_JPRB, 0.59383E+00_JPRB, 0.59923E+00_JPRB, 0.59874E+00_JPRB ,&
&  0.59571E+00_JPRB, 0.59102E+00_JPRB, 0.58928E+00_JPRB, 0.58940E+00_JPRB, 0.58690E+00_JPRB, 0.57744E+00_JPRB, 0.57774E+00_JPRB ,&
&  0.58714E+00_JPRB, 0.59586E+00_JPRB, 0.59977E+00_JPRB, 0.60127E+00_JPRB, 0.61146E+00_JPRB, 0.61123E+00_JPRB, 0.61539E+00_JPRB ,&
&  0.63911E+00_JPRB, 0.10884E+01_JPRB, 0.11154E+01_JPRB, 0.19361E+01_JPRB, 0.16146E+01_JPRB, 0.13075E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.97429E+00_JPRB, 0.10110E+01_JPRB, 0.92874E+00_JPRB, 0.84054E+00_JPRB, 0.79276E+00_JPRB, 0.77884E+00_JPRB, 0.75739E+00_JPRB ,&
&  0.73499E+00_JPRB, 0.74871E+00_JPRB, 0.76781E+00_JPRB, 0.79205E+00_JPRB, 0.81820E+00_JPRB, 0.83536E+00_JPRB, 0.87101E+00_JPRB ,&
&  0.91968E+00_JPRB, 0.99042E+00_JPRB, 0.10882E+01_JPRB, 0.10436E+01_JPRB, 0.92848E+00_JPRB, 0.89182E+00_JPRB, 0.88135E+00_JPRB ,&
&  0.88554E+00_JPRB, 0.94105E+00_JPRB, 0.86737E+00_JPRB, 0.80711E+00_JPRB, 0.76725E+00_JPRB, 0.75990E+00_JPRB, 0.74029E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.72494E+00_JPRB, 0.72721E+00_JPRB, 0.71687E+00_JPRB, 0.70641E+00_JPRB, 0.69965E+00_JPRB, 0.69039E+00_JPRB ,&
&  0.69145E+00_JPRB, 0.71319E+00_JPRB, 0.71794E+00_JPRB, 0.72140E+00_JPRB, 0.72702E+00_JPRB, 0.72898E+00_JPRB, 0.71818E+00_JPRB ,&
&  0.71543E+00_JPRB, 0.71962E+00_JPRB, 0.68979E+00_JPRB, 0.67961E+00_JPRB, 0.68715E+00_JPRB, 0.68438E+00_JPRB, 0.68557E+00_JPRB ,&
&  0.71312E+00_JPRB, 0.72251E+00_JPRB, 0.70204E+00_JPRB, 0.66988E+00_JPRB, 0.68297E+00_JPRB, 0.70297E+00_JPRB, 0.65554E+00_JPRB ,&
&  0.65014E+00_JPRB, 0.64814E+00_JPRB, 0.65133E+00_JPRB, 0.65117E+00_JPRB, 0.64666E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.61488E+00_JPRB, 0.61067E+00_JPRB, 0.60575E+00_JPRB, 0.57102E+00_JPRB, 0.54312E+00_JPRB, 0.53420E+00_JPRB, 0.55082E+00_JPRB ,&
&  0.58338E+00_JPRB, 0.60559E+00_JPRB, 0.60149E+00_JPRB, 0.58646E+00_JPRB, 0.57901E+00_JPRB, 0.58021E+00_JPRB, 0.58537E+00_JPRB ,&
&  0.58289E+00_JPRB, 0.57166E+00_JPRB, 0.56489E+00_JPRB, 0.57168E+00_JPRB, 0.57374E+00_JPRB, 0.58093E+00_JPRB, 0.59407E+00_JPRB ,&
&  0.61261E+00_JPRB, 0.62225E+00_JPRB, 0.62318E+00_JPRB, 0.62705E+00_JPRB, 0.64747E+00_JPRB, 0.64452E+00_JPRB, 0.64784E+00_JPRB ,&
&  0.71620E+00_JPRB, 0.11930E+01_JPRB, 0.14244E+01_JPRB, 0.17182E+01_JPRB, 0.14951E+01_JPRB, 0.11786E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.89458E+00_JPRB, 0.78870E+00_JPRB, 0.83825E+00_JPRB, 0.77848E+00_JPRB, 0.77882E+00_JPRB, 0.77348E+00_JPRB, 0.77389E+00_JPRB ,&
&  0.79663E+00_JPRB, 0.84223E+00_JPRB, 0.87285E+00_JPRB, 0.89661E+00_JPRB, 0.92069E+00_JPRB, 0.95852E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.11634E+01_JPRB, 0.12306E+01_JPRB, 0.10966E+01_JPRB, 0.90991E+00_JPRB, 0.90248E+00_JPRB, 0.88637E+00_JPRB ,&
&  0.88269E+00_JPRB, 0.87172E+00_JPRB, 0.83075E+00_JPRB, 0.77614E+00_JPRB, 0.70867E+00_JPRB, 0.74108E+00_JPRB, 0.71660E+00_JPRB ,&
&  0.68748E+00_JPRB, 0.69541E+00_JPRB, 0.69238E+00_JPRB, 0.69816E+00_JPRB, 0.69724E+00_JPRB, 0.68937E+00_JPRB, 0.68931E+00_JPRB ,&
&  0.69981E+00_JPRB, 0.70696E+00_JPRB, 0.71486E+00_JPRB, 0.72170E+00_JPRB, 0.73351E+00_JPRB, 0.74112E+00_JPRB, 0.73683E+00_JPRB ,&
&  0.72760E+00_JPRB, 0.73239E+00_JPRB, 0.70833E+00_JPRB, 0.71485E+00_JPRB, 0.71421E+00_JPRB, 0.70960E+00_JPRB, 0.68662E+00_JPRB ,&
&  0.68724E+00_JPRB, 0.69889E+00_JPRB, 0.68338E+00_JPRB, 0.67274E+00_JPRB, 0.68352E+00_JPRB, 0.67979E+00_JPRB, 0.65322E+00_JPRB ,&
&  0.65443E+00_JPRB, 0.64904E+00_JPRB, 0.65059E+00_JPRB, 0.63416E+00_JPRB, 0.62315E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.60128E+00_JPRB, 0.59289E+00_JPRB, 0.56843E+00_JPRB, 0.51512E+00_JPRB, 0.50477E+00_JPRB, 0.52233E+00_JPRB, 0.58112E+00_JPRB ,&
&  0.60500E+00_JPRB, 0.60890E+00_JPRB, 0.59440E+00_JPRB, 0.57551E+00_JPRB, 0.56316E+00_JPRB, 0.55967E+00_JPRB, 0.57046E+00_JPRB ,&
&  0.57946E+00_JPRB, 0.56863E+00_JPRB, 0.56728E+00_JPRB, 0.57380E+00_JPRB, 0.58467E+00_JPRB, 0.60314E+00_JPRB, 0.62375E+00_JPRB ,&
&  0.64156E+00_JPRB, 0.65074E+00_JPRB, 0.65420E+00_JPRB, 0.67602E+00_JPRB, 0.70404E+00_JPRB, 0.71361E+00_JPRB, 0.75175E+00_JPRB ,&
&  0.97778E+00_JPRB, 0.13998E+01_JPRB, 0.69794E+00_JPRB, 0.10555E+01_JPRB, 0.11875E+01_JPRB, 0.10286E+01_JPRB, 0.90618E+00_JPRB ,&
&  0.81586E+00_JPRB, 0.71719E+00_JPRB, 0.72754E+00_JPRB, 0.79860E+00_JPRB, 0.81574E+00_JPRB, 0.85766E+00_JPRB, 0.89954E+00_JPRB ,&
&  0.94168E+00_JPRB, 0.97123E+00_JPRB, 0.99825E+00_JPRB, 0.10239E+01_JPRB, 0.10580E+01_JPRB, 0.11136E+01_JPRB, 0.11657E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.12334E+01_JPRB, 0.12218E+01_JPRB, 0.98753E+00_JPRB, 0.88869E+00_JPRB, 0.88165E+00_JPRB, 0.84456E+00_JPRB ,&
&  0.79041E+00_JPRB, 0.80330E+00_JPRB, 0.79375E+00_JPRB, 0.73394E+00_JPRB, 0.67445E+00_JPRB, 0.65409E+00_JPRB, 0.68303E+00_JPRB ,&
&  0.66989E+00_JPRB, 0.66113E+00_JPRB, 0.65580E+00_JPRB, 0.66077E+00_JPRB, 0.66104E+00_JPRB, 0.66254E+00_JPRB, 0.66780E+00_JPRB ,&
&  0.68329E+00_JPRB, 0.69671E+00_JPRB, 0.70714E+00_JPRB, 0.71450E+00_JPRB, 0.72664E+00_JPRB, 0.73954E+00_JPRB, 0.73949E+00_JPRB ,&
&  0.74753E+00_JPRB, 0.75155E+00_JPRB, 0.75675E+00_JPRB, 0.74905E+00_JPRB, 0.72782E+00_JPRB, 0.71479E+00_JPRB, 0.70755E+00_JPRB ,&
&  0.70233E+00_JPRB, 0.69481E+00_JPRB, 0.66885E+00_JPRB, 0.66057E+00_JPRB, 0.69667E+00_JPRB, 0.68893E+00_JPRB, 0.73615E+00_JPRB ,&
&  0.80137E+00_JPRB, 0.85850E+00_JPRB, 0.81939E+00_JPRB, 0.62820E+00_JPRB, 0.60759E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.58072E+00_JPRB, 0.55453E+00_JPRB, 0.50509E+00_JPRB, 0.47511E+00_JPRB, 0.49966E+00_JPRB, 0.56276E+00_JPRB, 0.61077E+00_JPRB ,&
&  0.62681E+00_JPRB, 0.62180E+00_JPRB, 0.61522E+00_JPRB, 0.60992E+00_JPRB, 0.61663E+00_JPRB, 0.60604E+00_JPRB, 0.60836E+00_JPRB ,&
&  0.61381E+00_JPRB, 0.61233E+00_JPRB, 0.61703E+00_JPRB, 0.62239E+00_JPRB, 0.63025E+00_JPRB, 0.64947E+00_JPRB, 0.67051E+00_JPRB ,&
&  0.68739E+00_JPRB, 0.70244E+00_JPRB, 0.72639E+00_JPRB, 0.75975E+00_JPRB, 0.81248E+00_JPRB, 0.90875E+00_JPRB, 0.10793E+01_JPRB ,&
&  0.14000E+01_JPRB, 0.96498E+00_JPRB, 0.58545E+00_JPRB, 0.98328E+00_JPRB, 0.10260E+01_JPRB, 0.94533E+00_JPRB, 0.82728E+00_JPRB ,&
&  0.79324E+00_JPRB, 0.76523E+00_JPRB, 0.79174E+00_JPRB, 0.89284E+00_JPRB, 0.96296E+00_JPRB, 0.10610E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.11379E+01_JPRB, 0.11516E+01_JPRB, 0.11710E+01_JPRB, 0.12048E+01_JPRB, 0.12466E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.12673E+01_JPRB, 0.12195E+01_JPRB, 0.95293E+00_JPRB, 0.86657E+00_JPRB, 0.81860E+00_JPRB, 0.79152E+00_JPRB ,&
&  0.73822E+00_JPRB, 0.73705E+00_JPRB, 0.74251E+00_JPRB, 0.70395E+00_JPRB, 0.65872E+00_JPRB, 0.62712E+00_JPRB, 0.66322E+00_JPRB ,&
&  0.66609E+00_JPRB, 0.63419E+00_JPRB, 0.60707E+00_JPRB, 0.59265E+00_JPRB, 0.61492E+00_JPRB, 0.63423E+00_JPRB, 0.63105E+00_JPRB ,&
&  0.64209E+00_JPRB, 0.66044E+00_JPRB, 0.67392E+00_JPRB, 0.68727E+00_JPRB, 0.70728E+00_JPRB, 0.72839E+00_JPRB, 0.73967E+00_JPRB ,&
&  0.75000E+00_JPRB, 0.75016E+00_JPRB, 0.75397E+00_JPRB, 0.74764E+00_JPRB, 0.71575E+00_JPRB, 0.68205E+00_JPRB, 0.68597E+00_JPRB ,&
&  0.68455E+00_JPRB, 0.69099E+00_JPRB, 0.68372E+00_JPRB, 0.69654E+00_JPRB, 0.70789E+00_JPRB, 0.73673E+00_JPRB, 0.77141E+00_JPRB ,&
&  0.80045E+00_JPRB, 0.78922E+00_JPRB, 0.71082E+00_JPRB, 0.61763E+00_JPRB, 0.59662E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.54621E+00_JPRB, 0.48979E+00_JPRB, 0.44062E+00_JPRB, 0.45798E+00_JPRB, 0.51995E+00_JPRB, 0.57175E+00_JPRB, 0.59767E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.63187E+00_JPRB, 0.63541E+00_JPRB, 0.64108E+00_JPRB, 0.65542E+00_JPRB, 0.65213E+00_JPRB, 0.65928E+00_JPRB ,&
&  0.65474E+00_JPRB, 0.64428E+00_JPRB, 0.64140E+00_JPRB, 0.65636E+00_JPRB, 0.67697E+00_JPRB, 0.69985E+00_JPRB, 0.72530E+00_JPRB ,&
&  0.75363E+00_JPRB, 0.78268E+00_JPRB, 0.82624E+00_JPRB, 0.87940E+00_JPRB, 0.94958E+00_JPRB, 0.10251E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.78797E+00_JPRB, 0.47219E+00_JPRB, 0.82455E+00_JPRB, 0.10055E+01_JPRB, 0.93393E+00_JPRB, 0.88623E+00_JPRB, 0.84747E+00_JPRB ,&
&  0.81584E+00_JPRB, 0.81621E+00_JPRB, 0.84515E+00_JPRB, 0.10566E+01_JPRB, 0.11791E+01_JPRB, 0.12317E+01_JPRB, 0.12547E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.12838E+01_JPRB, 0.13042E+01_JPRB, 0.13375E+01_JPRB, 0.13742E+01_JPRB, 0.14017E+01_JPRB, 0.14109E+01_JPRB ,&
&  0.14176E+01_JPRB, 0.14042E+01_JPRB, 0.13537E+01_JPRB, 0.10293E+01_JPRB, 0.93421E+00_JPRB, 0.84339E+00_JPRB, 0.79119E+00_JPRB ,&
&  0.74190E+00_JPRB, 0.66449E+00_JPRB, 0.65953E+00_JPRB, 0.69445E+00_JPRB, 0.66848E+00_JPRB, 0.60057E+00_JPRB, 0.60721E+00_JPRB ,&
&  0.62088E+00_JPRB, 0.60785E+00_JPRB, 0.57972E+00_JPRB, 0.56562E+00_JPRB, 0.59314E+00_JPRB, 0.59506E+00_JPRB, 0.60214E+00_JPRB ,&
&  0.59808E+00_JPRB, 0.62317E+00_JPRB, 0.64754E+00_JPRB, 0.66630E+00_JPRB, 0.68710E+00_JPRB, 0.71130E+00_JPRB, 0.71204E+00_JPRB ,&
&  0.71826E+00_JPRB, 0.71422E+00_JPRB, 0.71891E+00_JPRB, 0.70701E+00_JPRB, 0.69549E+00_JPRB, 0.66756E+00_JPRB, 0.66216E+00_JPRB ,&
&  0.66789E+00_JPRB, 0.67571E+00_JPRB, 0.67758E+00_JPRB, 0.67332E+00_JPRB, 0.70254E+00_JPRB, 0.70728E+00_JPRB, 0.71849E+00_JPRB ,&
&  0.71717E+00_JPRB, 0.69776E+00_JPRB, 0.64049E+00_JPRB, 0.59638E+00_JPRB, 0.57850E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.46120E+00_JPRB, 0.40967E+00_JPRB, 0.40455E+00_JPRB, 0.44562E+00_JPRB, 0.48702E+00_JPRB, 0.52902E+00_JPRB, 0.56602E+00_JPRB ,&
&  0.60180E+00_JPRB, 0.62004E+00_JPRB, 0.65784E+00_JPRB, 0.67063E+00_JPRB, 0.67735E+00_JPRB, 0.66903E+00_JPRB, 0.65938E+00_JPRB ,&
&  0.66673E+00_JPRB, 0.67370E+00_JPRB, 0.67791E+00_JPRB, 0.69605E+00_JPRB, 0.71496E+00_JPRB, 0.73688E+00_JPRB, 0.76673E+00_JPRB ,&
&  0.79631E+00_JPRB, 0.83150E+00_JPRB, 0.86581E+00_JPRB, 0.90630E+00_JPRB, 0.93029E+00_JPRB, 0.92836E+00_JPRB, 0.85844E+00_JPRB ,&
&  0.48111E+00_JPRB, 0.78173E+00_JPRB, 0.97348E+00_JPRB, 0.95687E+00_JPRB, 0.92247E+00_JPRB, 0.88843E+00_JPRB, 0.90005E+00_JPRB ,&
&  0.87744E+00_JPRB, 0.84119E+00_JPRB, 0.89378E+00_JPRB, 0.11749E+01_JPRB, 0.12450E+01_JPRB, 0.12822E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.13710E+01_JPRB, 0.14056E+01_JPRB, 0.14468E+01_JPRB, 0.14896E+01_JPRB, 0.15461E+01_JPRB, 0.15966E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.16706E+01_JPRB, 0.16853E+01_JPRB, 0.16819E+01_JPRB, 0.13422E+01_JPRB, 0.12905E+01_JPRB, 0.11213E+01_JPRB, 0.99384E+00_JPRB ,&
&  0.74102E+00_JPRB, 0.63942E+00_JPRB, 0.65760E+00_JPRB, 0.71514E+00_JPRB, 0.67613E+00_JPRB, 0.64374E+00_JPRB, 0.59259E+00_JPRB ,&
&  0.59795E+00_JPRB, 0.59814E+00_JPRB, 0.59604E+00_JPRB, 0.58060E+00_JPRB, 0.58063E+00_JPRB, 0.55659E+00_JPRB, 0.59181E+00_JPRB ,&
&  0.60719E+00_JPRB, 0.59610E+00_JPRB, 0.59760E+00_JPRB, 0.61914E+00_JPRB, 0.64835E+00_JPRB, 0.67747E+00_JPRB, 0.68431E+00_JPRB ,&
&  0.67510E+00_JPRB, 0.65913E+00_JPRB, 0.66006E+00_JPRB, 0.66093E+00_JPRB, 0.66051E+00_JPRB, 0.65984E+00_JPRB, 0.64995E+00_JPRB ,&
&  0.64321E+00_JPRB, 0.66047E+00_JPRB, 0.70147E+00_JPRB, 0.65848E+00_JPRB, 0.70686E+00_JPRB, 0.64279E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.61061E+00_JPRB, 0.59116E+00_JPRB, 0.54476E+00_JPRB, 0.52205E+00_JPRB, 0.49444E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.37482E+00_JPRB, 0.35782E+00_JPRB, 0.36700E+00_JPRB, 0.39046E+00_JPRB, 0.41825E+00_JPRB, 0.45985E+00_JPRB, 0.50881E+00_JPRB ,&
&  0.54973E+00_JPRB, 0.58766E+00_JPRB, 0.65294E+00_JPRB, 0.66107E+00_JPRB, 0.66471E+00_JPRB, 0.69195E+00_JPRB, 0.70302E+00_JPRB ,&
&  0.71369E+00_JPRB, 0.71318E+00_JPRB, 0.71828E+00_JPRB, 0.72453E+00_JPRB, 0.74528E+00_JPRB, 0.76948E+00_JPRB, 0.78546E+00_JPRB ,&
&  0.81234E+00_JPRB, 0.84373E+00_JPRB, 0.89547E+00_JPRB, 0.94738E+00_JPRB, 0.99213E+00_JPRB, 0.98037E+00_JPRB, 0.95106E+00_JPRB ,&
&  0.70991E+00_JPRB, 0.94314E+00_JPRB, 0.94740E+00_JPRB, 0.93527E+00_JPRB, 0.90383E+00_JPRB, 0.87272E+00_JPRB, 0.87390E+00_JPRB ,&
&  0.88017E+00_JPRB, 0.84988E+00_JPRB, 0.88300E+00_JPRB, 0.10212E+01_JPRB, 0.12153E+01_JPRB, 0.12618E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.14551E+01_JPRB, 0.15214E+01_JPRB, 0.16014E+01_JPRB, 0.16840E+01_JPRB, 0.17621E+01_JPRB, 0.18411E+01_JPRB ,&
&  0.19076E+01_JPRB, 0.19647E+01_JPRB, 0.20497E+01_JPRB, 0.17166E+01_JPRB, 0.14281E+01_JPRB, 0.11824E+01_JPRB, 0.88326E+00_JPRB ,&
&  0.69059E+00_JPRB, 0.61931E+00_JPRB, 0.66523E+00_JPRB, 0.72410E+00_JPRB, 0.68472E+00_JPRB, 0.64807E+00_JPRB, 0.59705E+00_JPRB ,&
&  0.58858E+00_JPRB, 0.58888E+00_JPRB, 0.59193E+00_JPRB, 0.58347E+00_JPRB, 0.58910E+00_JPRB, 0.57425E+00_JPRB, 0.57773E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.58345E+00_JPRB, 0.57800E+00_JPRB, 0.58768E+00_JPRB, 0.61447E+00_JPRB, 0.64555E+00_JPRB, 0.64679E+00_JPRB ,&
&  0.65177E+00_JPRB, 0.58990E+00_JPRB, 0.56856E+00_JPRB, 0.58549E+00_JPRB, 0.59184E+00_JPRB, 0.59809E+00_JPRB, 0.62304E+00_JPRB ,&
&  0.70930E+00_JPRB, 0.81538E+00_JPRB, 0.91070E+00_JPRB, 0.90990E+00_JPRB, 0.96761E+00_JPRB, 0.75572E+00_JPRB, 0.55628E+00_JPRB ,&
&  0.52097E+00_JPRB, 0.50841E+00_JPRB, 0.46552E+00_JPRB, 0.43649E+00_JPRB, 0.40372E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.36877E+00_JPRB, 0.36272E+00_JPRB, 0.37167E+00_JPRB, 0.38792E+00_JPRB, 0.39967E+00_JPRB, 0.42691E+00_JPRB, 0.45945E+00_JPRB ,&
&  0.49491E+00_JPRB, 0.53789E+00_JPRB, 0.57038E+00_JPRB, 0.58833E+00_JPRB, 0.62334E+00_JPRB, 0.67603E+00_JPRB, 0.69103E+00_JPRB ,&
&  0.72950E+00_JPRB, 0.74663E+00_JPRB, 0.74557E+00_JPRB, 0.74136E+00_JPRB, 0.75348E+00_JPRB, 0.78112E+00_JPRB, 0.80823E+00_JPRB ,&
&  0.82660E+00_JPRB, 0.85646E+00_JPRB, 0.90438E+00_JPRB, 0.93013E+00_JPRB, 0.93161E+00_JPRB, 0.90475E+00_JPRB, 0.70536E+00_JPRB ,&
&  0.92554E+00_JPRB, 0.94427E+00_JPRB, 0.94632E+00_JPRB, 0.91286E+00_JPRB, 0.88464E+00_JPRB, 0.85781E+00_JPRB, 0.85386E+00_JPRB ,&
&  0.85939E+00_JPRB, 0.85124E+00_JPRB, 0.84666E+00_JPRB, 0.89114E+00_JPRB, 0.10783E+01_JPRB, 0.11433E+01_JPRB, 0.12408E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.13505E+01_JPRB, 0.14269E+01_JPRB, 0.15126E+01_JPRB, 0.15976E+01_JPRB, 0.16683E+01_JPRB, 0.17297E+01_JPRB ,&
&  0.17871E+01_JPRB, 0.18476E+01_JPRB, 0.18989E+01_JPRB, 0.16285E+01_JPRB, 0.13439E+01_JPRB, 0.11484E+01_JPRB, 0.86039E+00_JPRB ,&
&  0.66622E+00_JPRB, 0.61922E+00_JPRB, 0.66428E+00_JPRB, 0.70915E+00_JPRB, 0.66295E+00_JPRB, 0.62446E+00_JPRB, 0.58918E+00_JPRB ,&
&  0.58615E+00_JPRB, 0.59535E+00_JPRB, 0.60908E+00_JPRB, 0.61122E+00_JPRB, 0.59115E+00_JPRB, 0.57274E+00_JPRB, 0.56092E+00_JPRB ,&
&  0.57889E+00_JPRB, 0.57614E+00_JPRB, 0.56809E+00_JPRB, 0.58880E+00_JPRB, 0.59793E+00_JPRB, 0.66305E+00_JPRB, 0.76380E+00_JPRB ,&
&  0.76314E+00_JPRB, 0.49444E+00_JPRB, 0.50687E+00_JPRB, 0.52504E+00_JPRB, 0.56796E+00_JPRB, 0.64326E+00_JPRB, 0.79868E+00_JPRB ,&
&  0.11478E+01_JPRB, 0.19150E+01_JPRB, 0.21972E+01_JPRB, 0.14523E+01_JPRB, 0.19827E+01_JPRB, 0.12793E+01_JPRB, 0.50200E+00_JPRB ,&
&  0.47401E+00_JPRB, 0.45926E+00_JPRB, 0.43171E+00_JPRB, 0.40269E+00_JPRB, 0.38325E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.44560E+00_JPRB, 0.44670E+00_JPRB, 0.45353E+00_JPRB, 0.45796E+00_JPRB, 0.46255E+00_JPRB, 0.46525E+00_JPRB, 0.46826E+00_JPRB ,&
&  0.48076E+00_JPRB, 0.48772E+00_JPRB, 0.50577E+00_JPRB, 0.51746E+00_JPRB, 0.52774E+00_JPRB, 0.55357E+00_JPRB, 0.57304E+00_JPRB ,&
&  0.60611E+00_JPRB, 0.64817E+00_JPRB, 0.66019E+00_JPRB, 0.68110E+00_JPRB, 0.69070E+00_JPRB, 0.69838E+00_JPRB, 0.69995E+00_JPRB ,&
&  0.74130E+00_JPRB, 0.82223E+00_JPRB, 0.86972E+00_JPRB, 0.91976E+00_JPRB, 0.90413E+00_JPRB, 0.90509E+00_JPRB, 0.72423E+00_JPRB ,&
&  0.97037E+00_JPRB, 0.96378E+00_JPRB, 0.89731E+00_JPRB, 0.85921E+00_JPRB, 0.83415E+00_JPRB, 0.80764E+00_JPRB, 0.77959E+00_JPRB ,&
&  0.77926E+00_JPRB, 0.81394E+00_JPRB, 0.81437E+00_JPRB, 0.87331E+00_JPRB, 0.95337E+00_JPRB, 0.96969E+00_JPRB, 0.10473E+01_JPRB ,&
&  0.10909E+01_JPRB, 0.11181E+01_JPRB, 0.11582E+01_JPRB, 0.12251E+01_JPRB, 0.12743E+01_JPRB, 0.13030E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.13001E+01_JPRB, 0.13154E+01_JPRB, 0.13654E+01_JPRB, 0.13320E+01_JPRB, 0.11566E+01_JPRB, 0.10192E+01_JPRB, 0.85893E+00_JPRB ,&
&  0.66689E+00_JPRB, 0.59942E+00_JPRB, 0.61751E+00_JPRB, 0.69167E+00_JPRB, 0.66071E+00_JPRB, 0.62576E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.57876E+00_JPRB, 0.58245E+00_JPRB, 0.58944E+00_JPRB, 0.58566E+00_JPRB, 0.56953E+00_JPRB, 0.54278E+00_JPRB, 0.51880E+00_JPRB ,&
&  0.51713E+00_JPRB, 0.50949E+00_JPRB, 0.50638E+00_JPRB, 0.51204E+00_JPRB, 0.51750E+00_JPRB, 0.54187E+00_JPRB, 0.50667E+00_JPRB ,&
&  0.55576E+00_JPRB, 0.45386E+00_JPRB, 0.47551E+00_JPRB, 0.51817E+00_JPRB, 0.58124E+00_JPRB, 0.81068E+00_JPRB, 0.12117E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.20762E+01_JPRB, 0.27640E+01_JPRB, 0.40663E+01_JPRB, 0.25549E+01_JPRB, 0.16218E+01_JPRB, 0.51544E+00_JPRB ,&
&  0.50401E+00_JPRB, 0.47841E+00_JPRB, 0.47067E+00_JPRB, 0.45592E+00_JPRB, 0.45147E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.50741E+00_JPRB, 0.51559E+00_JPRB, 0.52500E+00_JPRB, 0.53325E+00_JPRB, 0.53822E+00_JPRB, 0.54757E+00_JPRB, 0.55401E+00_JPRB ,&
&  0.56159E+00_JPRB, 0.56349E+00_JPRB, 0.56860E+00_JPRB, 0.57242E+00_JPRB, 0.57966E+00_JPRB, 0.59112E+00_JPRB, 0.60227E+00_JPRB ,&
&  0.60992E+00_JPRB, 0.62224E+00_JPRB, 0.63771E+00_JPRB, 0.64606E+00_JPRB, 0.67081E+00_JPRB, 0.71976E+00_JPRB, 0.77335E+00_JPRB ,&
&  0.81088E+00_JPRB, 0.85815E+00_JPRB, 0.90003E+00_JPRB, 0.90817E+00_JPRB, 0.88818E+00_JPRB, 0.87711E+00_JPRB, 0.74451E+00_JPRB ,&
&  0.10697E+01_JPRB, 0.10530E+01_JPRB, 0.94409E+00_JPRB, 0.87906E+00_JPRB, 0.82294E+00_JPRB, 0.77234E+00_JPRB, 0.72186E+00_JPRB ,&
&  0.69270E+00_JPRB, 0.76002E+00_JPRB, 0.76728E+00_JPRB, 0.78560E+00_JPRB, 0.76871E+00_JPRB, 0.78284E+00_JPRB, 0.85456E+00_JPRB ,&
&  0.93070E+00_JPRB, 0.92694E+00_JPRB, 0.88217E+00_JPRB, 0.89855E+00_JPRB, 0.92647E+00_JPRB, 0.93109E+00_JPRB, 0.93675E+00_JPRB ,&
&  0.89755E+00_JPRB, 0.85592E+00_JPRB, 0.77564E+00_JPRB, 0.86296E+00_JPRB, 0.83602E+00_JPRB, 0.78228E+00_JPRB, 0.71573E+00_JPRB ,&
&  0.56499E+00_JPRB, 0.56814E+00_JPRB, 0.61545E+00_JPRB, 0.71332E+00_JPRB, 0.67692E+00_JPRB, 0.64817E+00_JPRB, 0.61207E+00_JPRB ,&
&  0.58223E+00_JPRB, 0.56844E+00_JPRB, 0.54629E+00_JPRB, 0.53638E+00_JPRB, 0.53256E+00_JPRB, 0.51737E+00_JPRB, 0.48947E+00_JPRB ,&
&  0.46726E+00_JPRB, 0.46329E+00_JPRB, 0.45763E+00_JPRB, 0.45229E+00_JPRB, 0.45399E+00_JPRB, 0.40952E+00_JPRB, 0.48632E+00_JPRB ,&
&  0.46837E+00_JPRB, 0.40831E+00_JPRB, 0.49723E+00_JPRB, 0.38652E+00_JPRB, 0.50140E+00_JPRB, 0.61967E+00_JPRB, 0.81163E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.11167E+01_JPRB, 0.14670E+01_JPRB, 0.11576E+01_JPRB, 0.87134E+00_JPRB, 0.57456E+00_JPRB, 0.53817E+00_JPRB ,&
&  0.52648E+00_JPRB, 0.51604E+00_JPRB, 0.51853E+00_JPRB, 0.51132E+00_JPRB, 0.50943E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.57578E+00_JPRB, 0.58731E+00_JPRB, 0.60129E+00_JPRB, 0.61954E+00_JPRB, 0.63029E+00_JPRB, 0.64655E+00_JPRB, 0.65855E+00_JPRB ,&
&  0.67578E+00_JPRB, 0.68806E+00_JPRB, 0.69765E+00_JPRB, 0.70752E+00_JPRB, 0.72169E+00_JPRB, 0.74107E+00_JPRB, 0.76637E+00_JPRB ,&
&  0.79389E+00_JPRB, 0.82134E+00_JPRB, 0.85161E+00_JPRB, 0.88648E+00_JPRB, 0.92711E+00_JPRB, 0.97717E+00_JPRB, 0.10236E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.11062E+01_JPRB, 0.11443E+01_JPRB, 0.11734E+01_JPRB, 0.11795E+01_JPRB, 0.11780E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.12469E+01_JPRB, 0.12208E+01_JPRB, 0.11248E+01_JPRB, 0.10195E+01_JPRB, 0.93328E+00_JPRB, 0.85978E+00_JPRB, 0.78701E+00_JPRB ,&
&  0.77949E+00_JPRB, 0.73263E+00_JPRB, 0.70934E+00_JPRB, 0.70500E+00_JPRB, 0.66958E+00_JPRB, 0.66288E+00_JPRB, 0.68675E+00_JPRB ,&
&  0.72738E+00_JPRB, 0.79716E+00_JPRB, 0.82354E+00_JPRB, 0.85662E+00_JPRB, 0.91465E+00_JPRB, 0.87667E+00_JPRB, 0.87473E+00_JPRB ,&
&  0.80657E+00_JPRB, 0.72365E+00_JPRB, 0.59370E+00_JPRB, 0.65571E+00_JPRB, 0.69799E+00_JPRB, 0.72320E+00_JPRB, 0.72132E+00_JPRB ,&
&  0.60820E+00_JPRB, 0.53904E+00_JPRB, 0.57889E+00_JPRB, 0.76434E+00_JPRB, 0.74472E+00_JPRB, 0.68034E+00_JPRB, 0.63025E+00_JPRB ,&
&  0.58141E+00_JPRB, 0.54611E+00_JPRB, 0.50864E+00_JPRB, 0.49376E+00_JPRB, 0.50985E+00_JPRB, 0.50742E+00_JPRB, 0.48460E+00_JPRB ,&
&  0.45098E+00_JPRB, 0.43464E+00_JPRB, 0.43151E+00_JPRB, 0.42915E+00_JPRB, 0.45241E+00_JPRB, 0.42710E+00_JPRB, 0.46175E+00_JPRB ,&
&  0.47256E+00_JPRB, 0.35759E+00_JPRB, 0.40706E+00_JPRB, 0.53317E+00_JPRB, 0.52463E+00_JPRB, 0.57973E+00_JPRB, 0.61418E+00_JPRB ,&
&  0.64673E+00_JPRB, 0.65614E+00_JPRB, 0.61161E+00_JPRB, 0.60765E+00_JPRB, 0.59573E+00_JPRB, 0.58336E+00_JPRB, 0.56917E+00_JPRB ,&
&  0.55620E+00_JPRB, 0.55672E+00_JPRB, 0.57014E+00_JPRB, 0.56974E+00_JPRB, 0.57024E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.58438E+00_JPRB, 0.59612E+00_JPRB, 0.59607E+00_JPRB, 0.60158E+00_JPRB, 0.61363E+00_JPRB, 0.62133E+00_JPRB, 0.63325E+00_JPRB ,&
&  0.64477E+00_JPRB, 0.64805E+00_JPRB, 0.65063E+00_JPRB, 0.65469E+00_JPRB, 0.65585E+00_JPRB, 0.67059E+00_JPRB, 0.68524E+00_JPRB ,&
&  0.70536E+00_JPRB, 0.72064E+00_JPRB, 0.73784E+00_JPRB, 0.76517E+00_JPRB, 0.79986E+00_JPRB, 0.85511E+00_JPRB, 0.91307E+00_JPRB ,&
&  0.98753E+00_JPRB, 0.10752E+01_JPRB, 0.11790E+01_JPRB, 0.12615E+01_JPRB, 0.13022E+01_JPRB, 0.13141E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.13058E+01_JPRB, 0.12211E+01_JPRB, 0.10761E+01_JPRB, 0.10528E+01_JPRB, 0.98056E+00_JPRB, 0.94826E+00_JPRB ,&
&  0.94017E+00_JPRB, 0.88991E+00_JPRB, 0.86529E+00_JPRB, 0.84195E+00_JPRB, 0.81943E+00_JPRB, 0.77462E+00_JPRB, 0.73594E+00_JPRB ,&
&  0.72680E+00_JPRB, 0.75810E+00_JPRB, 0.83635E+00_JPRB, 0.10038E+01_JPRB, 0.11695E+01_JPRB, 0.12205E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.10728E+01_JPRB, 0.98571E+00_JPRB, 0.94527E+00_JPRB, 0.98481E+00_JPRB, 0.10184E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.86155E+00_JPRB, 0.73528E+00_JPRB, 0.70438E+00_JPRB, 0.76865E+00_JPRB, 0.77858E+00_JPRB, 0.75328E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.57545E+00_JPRB, 0.52400E+00_JPRB, 0.49433E+00_JPRB, 0.49889E+00_JPRB, 0.54355E+00_JPRB, 0.55256E+00_JPRB, 0.53679E+00_JPRB ,&
&  0.49046E+00_JPRB, 0.46155E+00_JPRB, 0.44534E+00_JPRB, 0.44169E+00_JPRB, 0.42897E+00_JPRB, 0.66246E+00_JPRB, 0.54979E+00_JPRB ,&
&  0.52963E+00_JPRB, 0.49919E+00_JPRB, 0.42666E+00_JPRB, 0.57706E+00_JPRB, 0.60145E+00_JPRB, 0.61823E+00_JPRB, 0.61741E+00_JPRB ,&
&  0.60814E+00_JPRB, 0.60655E+00_JPRB, 0.60311E+00_JPRB, 0.58608E+00_JPRB, 0.57307E+00_JPRB, 0.56780E+00_JPRB, 0.58874E+00_JPRB ,&
&  0.58388E+00_JPRB, 0.58337E+00_JPRB, 0.58002E+00_JPRB, 0.59183E+00_JPRB, 0.58868E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.59936E+00_JPRB, 0.60400E+00_JPRB, 0.61348E+00_JPRB, 0.63078E+00_JPRB, 0.62766E+00_JPRB, 0.62455E+00_JPRB, 0.64948E+00_JPRB ,&
&  0.63235E+00_JPRB, 0.61892E+00_JPRB, 0.61469E+00_JPRB, 0.58766E+00_JPRB, 0.60134E+00_JPRB, 0.61356E+00_JPRB, 0.64427E+00_JPRB ,&
&  0.67612E+00_JPRB, 0.69443E+00_JPRB, 0.71063E+00_JPRB, 0.73623E+00_JPRB, 0.78546E+00_JPRB, 0.82204E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.90374E+00_JPRB, 0.99148E+00_JPRB, 0.10909E+01_JPRB, 0.13297E+01_JPRB, 0.18857E+01_JPRB, 0.23181E+01_JPRB, 0.22982E+01_JPRB ,&
&  0.17144E+01_JPRB, 0.12192E+01_JPRB, 0.11545E+01_JPRB, 0.10434E+01_JPRB, 0.10718E+01_JPRB, 0.11648E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.11528E+01_JPRB, 0.11452E+01_JPRB, 0.11418E+01_JPRB, 0.11504E+01_JPRB, 0.11539E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.10638E+01_JPRB, 0.10966E+01_JPRB, 0.12993E+01_JPRB, 0.14396E+01_JPRB, 0.16945E+01_JPRB, 0.16972E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.14982E+01_JPRB, 0.14263E+01_JPRB, 0.13941E+01_JPRB, 0.13559E+01_JPRB, 0.13319E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.11356E+01_JPRB, 0.75819E+00_JPRB, 0.80345E+00_JPRB, 0.81328E+00_JPRB, 0.83170E+00_JPRB, 0.68277E+00_JPRB ,&
&  0.56868E+00_JPRB, 0.53848E+00_JPRB, 0.53488E+00_JPRB, 0.55750E+00_JPRB, 0.60908E+00_JPRB, 0.64738E+00_JPRB, 0.61594E+00_JPRB ,&
&  0.55810E+00_JPRB, 0.50133E+00_JPRB, 0.46938E+00_JPRB, 0.47004E+00_JPRB, 0.51490E+00_JPRB, 0.55163E+00_JPRB, 0.53361E+00_JPRB ,&
&  0.53102E+00_JPRB, 0.60555E+00_JPRB, 0.55833E+00_JPRB, 0.70854E+00_JPRB, 0.72127E+00_JPRB, 0.71509E+00_JPRB, 0.70290E+00_JPRB ,&
&  0.69452E+00_JPRB, 0.67657E+00_JPRB, 0.65967E+00_JPRB, 0.63917E+00_JPRB, 0.61780E+00_JPRB, 0.63915E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.64037E+00_JPRB, 0.65562E+00_JPRB, 0.64031E+00_JPRB, 0.61634E+00_JPRB, 0.61623E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.63258E+00_JPRB, 0.62380E+00_JPRB, 0.63666E+00_JPRB, 0.65823E+00_JPRB, 0.66950E+00_JPRB, 0.68016E+00_JPRB, 0.69100E+00_JPRB ,&
&  0.69387E+00_JPRB, 0.69489E+00_JPRB, 0.69795E+00_JPRB, 0.69762E+00_JPRB, 0.72690E+00_JPRB, 0.75882E+00_JPRB, 0.79913E+00_JPRB ,&
&  0.86012E+00_JPRB, 0.91407E+00_JPRB, 0.96144E+00_JPRB, 0.10533E+01_JPRB, 0.11143E+01_JPRB, 0.11684E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.15189E+01_JPRB, 0.16463E+01_JPRB, 0.17659E+01_JPRB, 0.19008E+01_JPRB, 0.20176E+01_JPRB, 0.20006E+01_JPRB ,&
&  0.14881E+01_JPRB, 0.11832E+01_JPRB, 0.12962E+01_JPRB, 0.12356E+01_JPRB, 0.12671E+01_JPRB, 0.12843E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.13022E+01_JPRB, 0.13048E+01_JPRB, 0.13112E+01_JPRB, 0.13513E+01_JPRB, 0.14095E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.15528E+01_JPRB, 0.15748E+01_JPRB, 0.15621E+01_JPRB, 0.16210E+01_JPRB, 0.17642E+01_JPRB, 0.18889E+01_JPRB ,&
&  0.19551E+01_JPRB, 0.19525E+01_JPRB, 0.17239E+01_JPRB, 0.16783E+01_JPRB, 0.16621E+01_JPRB, 0.15421E+01_JPRB, 0.15799E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.14342E+01_JPRB, 0.88847E+00_JPRB, 0.82959E+00_JPRB, 0.81097E+00_JPRB, 0.77483E+00_JPRB, 0.70146E+00_JPRB ,&
&  0.58693E+00_JPRB, 0.56868E+00_JPRB, 0.58979E+00_JPRB, 0.66216E+00_JPRB, 0.70722E+00_JPRB, 0.73769E+00_JPRB, 0.68161E+00_JPRB ,&
&  0.59223E+00_JPRB, 0.53290E+00_JPRB, 0.48547E+00_JPRB, 0.48893E+00_JPRB, 0.48793E+00_JPRB, 0.54670E+00_JPRB, 0.53810E+00_JPRB ,&
&  0.58461E+00_JPRB, 0.64546E+00_JPRB, 0.73625E+00_JPRB, 0.74114E+00_JPRB, 0.71731E+00_JPRB, 0.82091E+00_JPRB, 0.79634E+00_JPRB ,&
&  0.77594E+00_JPRB, 0.76918E+00_JPRB, 0.75546E+00_JPRB, 0.74420E+00_JPRB, 0.73309E+00_JPRB, 0.73082E+00_JPRB, 0.71343E+00_JPRB ,&
&  0.70652E+00_JPRB, 0.69238E+00_JPRB, 0.68490E+00_JPRB, 0.65945E+00_JPRB, 0.64604E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.65632E+00_JPRB, 0.65496E+00_JPRB, 0.65785E+00_JPRB, 0.67482E+00_JPRB, 0.69782E+00_JPRB, 0.71244E+00_JPRB, 0.71734E+00_JPRB ,&
&  0.73247E+00_JPRB, 0.75317E+00_JPRB, 0.78426E+00_JPRB, 0.81153E+00_JPRB, 0.84698E+00_JPRB, 0.87965E+00_JPRB, 0.93272E+00_JPRB ,&
&  0.98183E+00_JPRB, 0.10414E+01_JPRB, 0.11473E+01_JPRB, 0.13051E+01_JPRB, 0.14363E+01_JPRB, 0.15458E+01_JPRB, 0.17167E+01_JPRB ,&
&  0.18807E+01_JPRB, 0.19526E+01_JPRB, 0.20431E+01_JPRB, 0.20468E+01_JPRB, 0.19056E+01_JPRB, 0.17468E+01_JPRB, 0.17326E+01_JPRB ,&
&  0.14191E+01_JPRB, 0.11976E+01_JPRB, 0.12248E+01_JPRB, 0.12730E+01_JPRB, 0.13058E+01_JPRB, 0.13031E+01_JPRB, 0.13075E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.13346E+01_JPRB, 0.13425E+01_JPRB, 0.13555E+01_JPRB, 0.13865E+01_JPRB, 0.14493E+01_JPRB, 0.15419E+01_JPRB ,&
&  0.15527E+01_JPRB, 0.15841E+01_JPRB, 0.16628E+01_JPRB, 0.15351E+01_JPRB, 0.16267E+01_JPRB, 0.17570E+01_JPRB, 0.18195E+01_JPRB ,&
&  0.17994E+01_JPRB, 0.16474E+01_JPRB, 0.16259E+01_JPRB, 0.16259E+01_JPRB, 0.16215E+01_JPRB, 0.15386E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.15707E+01_JPRB, 0.15189E+01_JPRB, 0.10134E+01_JPRB, 0.87896E+00_JPRB, 0.87051E+00_JPRB, 0.72874E+00_JPRB, 0.72151E+00_JPRB ,&
&  0.63768E+00_JPRB, 0.62082E+00_JPRB, 0.68117E+00_JPRB, 0.72702E+00_JPRB, 0.76040E+00_JPRB, 0.83962E+00_JPRB, 0.68254E+00_JPRB ,&
&  0.61797E+00_JPRB, 0.57169E+00_JPRB, 0.53664E+00_JPRB, 0.53992E+00_JPRB, 0.56346E+00_JPRB, 0.61459E+00_JPRB, 0.64287E+00_JPRB ,&
&  0.67049E+00_JPRB, 0.72741E+00_JPRB, 0.78682E+00_JPRB, 0.84132E+00_JPRB, 0.81520E+00_JPRB, 0.85464E+00_JPRB, 0.82783E+00_JPRB ,&
&  0.79523E+00_JPRB, 0.78911E+00_JPRB, 0.77539E+00_JPRB, 0.75205E+00_JPRB, 0.73928E+00_JPRB, 0.72353E+00_JPRB, 0.71555E+00_JPRB ,&
&  0.71197E+00_JPRB, 0.70752E+00_JPRB, 0.69859E+00_JPRB, 0.68894E+00_JPRB, 0.67081E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.68867E+00_JPRB, 0.67920E+00_JPRB, 0.67572E+00_JPRB, 0.68608E+00_JPRB, 0.70237E+00_JPRB, 0.70862E+00_JPRB, 0.71630E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.75880E+00_JPRB, 0.78360E+00_JPRB, 0.81821E+00_JPRB, 0.85324E+00_JPRB, 0.89024E+00_JPRB, 0.94848E+00_JPRB ,&
&  0.10186E+01_JPRB, 0.11162E+01_JPRB, 0.12522E+01_JPRB, 0.13972E+01_JPRB, 0.15279E+01_JPRB, 0.16749E+01_JPRB, 0.18180E+01_JPRB ,&
&  0.19819E+01_JPRB, 0.20261E+01_JPRB, 0.19281E+01_JPRB, 0.17512E+01_JPRB, 0.13613E+01_JPRB, 0.13583E+01_JPRB, 0.14612E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.12787E+01_JPRB, 0.12400E+01_JPRB, 0.12262E+01_JPRB, 0.12327E+01_JPRB, 0.12531E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.13031E+01_JPRB, 0.13211E+01_JPRB, 0.13362E+01_JPRB, 0.13530E+01_JPRB, 0.13921E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.14914E+01_JPRB, 0.15085E+01_JPRB, 0.15219E+01_JPRB, 0.14838E+01_JPRB, 0.15469E+01_JPRB, 0.16238E+01_JPRB, 0.15675E+01_JPRB ,&
&  0.15094E+01_JPRB, 0.13583E+01_JPRB, 0.13200E+01_JPRB, 0.13510E+01_JPRB, 0.13205E+01_JPRB, 0.12001E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.13059E+01_JPRB, 0.14687E+01_JPRB, 0.95068E+00_JPRB, 0.97366E+00_JPRB, 0.10833E+01_JPRB, 0.93710E+00_JPRB, 0.81631E+00_JPRB ,&
&  0.74130E+00_JPRB, 0.74188E+00_JPRB, 0.80454E+00_JPRB, 0.81405E+00_JPRB, 0.79377E+00_JPRB, 0.86995E+00_JPRB, 0.68805E+00_JPRB ,&
&  0.68569E+00_JPRB, 0.63712E+00_JPRB, 0.61668E+00_JPRB, 0.63165E+00_JPRB, 0.66591E+00_JPRB, 0.68294E+00_JPRB, 0.68238E+00_JPRB ,&
&  0.64765E+00_JPRB, 0.91038E+00_JPRB, 0.11509E+01_JPRB, 0.14270E+01_JPRB, 0.14533E+01_JPRB, 0.86091E+00_JPRB, 0.85641E+00_JPRB ,&
&  0.84661E+00_JPRB, 0.82870E+00_JPRB, 0.80595E+00_JPRB, 0.78558E+00_JPRB, 0.76963E+00_JPRB, 0.75263E+00_JPRB, 0.73915E+00_JPRB ,&
&  0.72995E+00_JPRB, 0.72385E+00_JPRB, 0.71965E+00_JPRB, 0.70837E+00_JPRB, 0.69706E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.71397E+00_JPRB, 0.71022E+00_JPRB, 0.69817E+00_JPRB, 0.69680E+00_JPRB, 0.72028E+00_JPRB, 0.73640E+00_JPRB, 0.72120E+00_JPRB ,&
&  0.72155E+00_JPRB, 0.73195E+00_JPRB, 0.74703E+00_JPRB, 0.76829E+00_JPRB, 0.78996E+00_JPRB, 0.80666E+00_JPRB, 0.85340E+00_JPRB ,&
&  0.91056E+00_JPRB, 0.10217E+01_JPRB, 0.11963E+01_JPRB, 0.13730E+01_JPRB, 0.15349E+01_JPRB, 0.17209E+01_JPRB, 0.19006E+01_JPRB ,&
&  0.21272E+01_JPRB, 0.21835E+01_JPRB, 0.19840E+01_JPRB, 0.13117E+01_JPRB, 0.10843E+01_JPRB, 0.11499E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11549E+01_JPRB, 0.11443E+01_JPRB, 0.11389E+01_JPRB, 0.11501E+01_JPRB, 0.11732E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.12125E+01_JPRB, 0.12381E+01_JPRB, 0.12687E+01_JPRB, 0.12988E+01_JPRB, 0.13320E+01_JPRB, 0.13666E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14056E+01_JPRB, 0.13898E+01_JPRB, 0.13526E+01_JPRB, 0.13364E+01_JPRB, 0.13366E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.11749E+01_JPRB, 0.11322E+01_JPRB, 0.11545E+01_JPRB, 0.11649E+01_JPRB, 0.10550E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.12276E+01_JPRB, 0.10059E+01_JPRB, 0.11010E+01_JPRB, 0.75192E+00_JPRB, 0.87569E+00_JPRB, 0.95004E+00_JPRB ,&
&  0.90132E+00_JPRB, 0.90711E+00_JPRB, 0.92958E+00_JPRB, 0.95067E+00_JPRB, 0.94990E+00_JPRB, 0.85417E+00_JPRB, 0.80353E+00_JPRB ,&
&  0.77236E+00_JPRB, 0.70046E+00_JPRB, 0.64557E+00_JPRB, 0.68010E+00_JPRB, 0.72875E+00_JPRB, 0.71884E+00_JPRB, 0.75150E+00_JPRB ,&
&  0.66475E+00_JPRB, 0.79521E+00_JPRB, 0.94539E+00_JPRB, 0.11324E+01_JPRB, 0.11261E+01_JPRB, 0.86074E+00_JPRB, 0.86794E+00_JPRB ,&
&  0.87438E+00_JPRB, 0.85223E+00_JPRB, 0.83342E+00_JPRB, 0.80281E+00_JPRB, 0.78897E+00_JPRB, 0.77291E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.75044E+00_JPRB, 0.74580E+00_JPRB, 0.74065E+00_JPRB, 0.72613E+00_JPRB, 0.71476E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.73899E+00_JPRB, 0.73709E+00_JPRB, 0.74137E+00_JPRB, 0.74326E+00_JPRB, 0.74428E+00_JPRB, 0.77904E+00_JPRB, 0.79210E+00_JPRB ,&
&  0.75329E+00_JPRB, 0.69886E+00_JPRB, 0.69741E+00_JPRB, 0.69878E+00_JPRB, 0.70369E+00_JPRB, 0.71695E+00_JPRB, 0.74401E+00_JPRB ,&
&  0.77208E+00_JPRB, 0.82354E+00_JPRB, 0.94542E+00_JPRB, 0.11600E+01_JPRB, 0.14039E+01_JPRB, 0.16475E+01_JPRB, 0.18969E+01_JPRB ,&
&  0.17799E+01_JPRB, 0.12898E+01_JPRB, 0.16075E+01_JPRB, 0.15272E+01_JPRB, 0.12894E+01_JPRB, 0.11075E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.11240E+01_JPRB, 0.11194E+01_JPRB, 0.11049E+01_JPRB, 0.10943E+01_JPRB, 0.10616E+01_JPRB, 0.10731E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.11104E+01_JPRB, 0.11229E+01_JPRB, 0.11449E+01_JPRB, 0.11945E+01_JPRB, 0.12551E+01_JPRB, 0.12882E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.12866E+01_JPRB, 0.12246E+01_JPRB, 0.11766E+01_JPRB, 0.11604E+01_JPRB, 0.11630E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.10816E+01_JPRB, 0.10821E+01_JPRB, 0.11122E+01_JPRB, 0.11267E+01_JPRB, 0.10548E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.11203E+01_JPRB, 0.10754E+01_JPRB, 0.11369E+01_JPRB, 0.85407E+00_JPRB, 0.92921E+00_JPRB, 0.88166E+00_JPRB ,&
&  0.98969E+00_JPRB, 0.99744E+00_JPRB, 0.10069E+01_JPRB, 0.10245E+01_JPRB, 0.10803E+01_JPRB, 0.97837E+00_JPRB, 0.92748E+00_JPRB ,&
&  0.81647E+00_JPRB, 0.76795E+00_JPRB, 0.70166E+00_JPRB, 0.75953E+00_JPRB, 0.77688E+00_JPRB, 0.75708E+00_JPRB, 0.83363E+00_JPRB ,&
&  0.88409E+00_JPRB, 0.81759E+00_JPRB, 0.81566E+00_JPRB, 0.89804E+00_JPRB, 0.92060E+00_JPRB, 0.83857E+00_JPRB, 0.84334E+00_JPRB ,&
&  0.84969E+00_JPRB, 0.85960E+00_JPRB, 0.83536E+00_JPRB, 0.80735E+00_JPRB, 0.78417E+00_JPRB, 0.76754E+00_JPRB, 0.76719E+00_JPRB ,&
&  0.76081E+00_JPRB, 0.75601E+00_JPRB, 0.75189E+00_JPRB, 0.74518E+00_JPRB, 0.74017E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.73967E+00_JPRB, 0.73549E+00_JPRB, 0.73222E+00_JPRB, 0.74701E+00_JPRB, 0.79388E+00_JPRB, 0.84552E+00_JPRB, 0.98208E+00_JPRB ,&
&  0.11996E+01_JPRB, 0.77577E+00_JPRB, 0.67478E+00_JPRB, 0.65544E+00_JPRB, 0.64504E+00_JPRB, 0.64130E+00_JPRB, 0.65970E+00_JPRB ,&
&  0.68186E+00_JPRB, 0.70685E+00_JPRB, 0.75783E+00_JPRB, 0.87239E+00_JPRB, 0.10920E+01_JPRB, 0.14210E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.12867E+01_JPRB, 0.14714E+01_JPRB, 0.17795E+01_JPRB, 0.14795E+01_JPRB, 0.13274E+01_JPRB, 0.11840E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.10676E+01_JPRB, 0.10717E+01_JPRB, 0.10596E+01_JPRB, 0.10335E+01_JPRB, 0.10311E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10584E+01_JPRB, 0.10584E+01_JPRB, 0.10630E+01_JPRB, 0.10840E+01_JPRB, 0.11362E+01_JPRB, 0.11804E+01_JPRB ,&
&  0.12126E+01_JPRB, 0.12183E+01_JPRB, 0.11567E+01_JPRB, 0.11019E+01_JPRB, 0.10918E+01_JPRB, 0.11084E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.10830E+01_JPRB, 0.10790E+01_JPRB, 0.10701E+01_JPRB, 0.10226E+01_JPRB, 0.10968E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.10853E+01_JPRB, 0.11805E+01_JPRB, 0.97072E+00_JPRB, 0.96993E+00_JPRB, 0.10146E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.94745E+00_JPRB, 0.10139E+01_JPRB, 0.10769E+01_JPRB, 0.10801E+01_JPRB, 0.10996E+01_JPRB, 0.10237E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.86945E+00_JPRB, 0.90978E+00_JPRB, 0.79281E+00_JPRB, 0.82244E+00_JPRB, 0.81893E+00_JPRB, 0.77997E+00_JPRB, 0.84628E+00_JPRB ,&
&  0.93010E+00_JPRB, 0.90094E+00_JPRB, 0.86700E+00_JPRB, 0.84262E+00_JPRB, 0.80019E+00_JPRB, 0.78593E+00_JPRB, 0.79487E+00_JPRB ,&
&  0.80953E+00_JPRB, 0.84766E+00_JPRB, 0.83206E+00_JPRB, 0.81859E+00_JPRB, 0.79991E+00_JPRB, 0.78309E+00_JPRB, 0.77580E+00_JPRB ,&
&  0.77233E+00_JPRB, 0.76695E+00_JPRB, 0.76171E+00_JPRB, 0.75411E+00_JPRB, 0.74957E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.73058E+00_JPRB, 0.71377E+00_JPRB, 0.69165E+00_JPRB, 0.67486E+00_JPRB, 0.66786E+00_JPRB, 0.69305E+00_JPRB, 0.75904E+00_JPRB ,&
&  0.10407E+01_JPRB, 0.77984E+00_JPRB, 0.68493E+00_JPRB, 0.65337E+00_JPRB, 0.63896E+00_JPRB, 0.61483E+00_JPRB, 0.60424E+00_JPRB ,&
&  0.61026E+00_JPRB, 0.62336E+00_JPRB, 0.65572E+00_JPRB, 0.70615E+00_JPRB, 0.80618E+00_JPRB, 0.10295E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.20003E+01_JPRB, 0.17194E+01_JPRB, 0.15921E+01_JPRB, 0.14039E+01_JPRB, 0.12841E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.10756E+01_JPRB, 0.10703E+01_JPRB, 0.10864E+01_JPRB, 0.10933E+01_JPRB, 0.10908E+01_JPRB, 0.10808E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10689E+01_JPRB, 0.10595E+01_JPRB, 0.10574E+01_JPRB, 0.10560E+01_JPRB, 0.10565E+01_JPRB, 0.10816E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.11343E+01_JPRB, 0.11875E+01_JPRB, 0.11431E+01_JPRB, 0.11017E+01_JPRB, 0.10897E+01_JPRB, 0.11109E+01_JPRB, 0.11230E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10140E+01_JPRB, 0.10588E+01_JPRB, 0.10829E+01_JPRB, 0.10882E+01_JPRB, 0.11511E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.11084E+01_JPRB, 0.11864E+01_JPRB, 0.10105E+01_JPRB, 0.99221E+00_JPRB, 0.10662E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.95990E+00_JPRB, 0.10069E+01_JPRB, 0.10637E+01_JPRB, 0.11044E+01_JPRB, 0.10808E+01_JPRB, 0.10414E+01_JPRB, 0.10035E+01_JPRB ,&
&  0.94480E+00_JPRB, 0.98370E+00_JPRB, 0.92298E+00_JPRB, 0.81543E+00_JPRB, 0.82369E+00_JPRB, 0.74217E+00_JPRB, 0.83479E+00_JPRB ,&
&  0.92265E+00_JPRB, 0.87667E+00_JPRB, 0.90460E+00_JPRB, 0.94555E+00_JPRB, 0.83887E+00_JPRB, 0.78953E+00_JPRB, 0.77493E+00_JPRB ,&
&  0.79887E+00_JPRB, 0.82301E+00_JPRB, 0.81531E+00_JPRB, 0.79414E+00_JPRB, 0.78076E+00_JPRB, 0.76863E+00_JPRB, 0.75144E+00_JPRB ,&
&  0.74395E+00_JPRB, 0.74176E+00_JPRB, 0.74038E+00_JPRB, 0.73881E+00_JPRB, 0.74270E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.74817E+00_JPRB, 0.74474E+00_JPRB, 0.71119E+00_JPRB, 0.67550E+00_JPRB, 0.65644E+00_JPRB, 0.65943E+00_JPRB, 0.68371E+00_JPRB ,&
&  0.88619E+00_JPRB, 0.77616E+00_JPRB, 0.69302E+00_JPRB, 0.65310E+00_JPRB, 0.62977E+00_JPRB, 0.61288E+00_JPRB, 0.59589E+00_JPRB ,&
&  0.58532E+00_JPRB, 0.58546E+00_JPRB, 0.61550E+00_JPRB, 0.63611E+00_JPRB, 0.67829E+00_JPRB, 0.78637E+00_JPRB, 0.71575E+00_JPRB ,&
&  0.10127E+01_JPRB, 0.19055E+01_JPRB, 0.17931E+01_JPRB, 0.17332E+01_JPRB, 0.15857E+01_JPRB, 0.14998E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.11271E+01_JPRB, 0.11374E+01_JPRB, 0.11556E+01_JPRB, 0.11734E+01_JPRB, 0.11779E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.10804E+01_JPRB, 0.10740E+01_JPRB, 0.10658E+01_JPRB, 0.10606E+01_JPRB, 0.10721E+01_JPRB, 0.10783E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.11241E+01_JPRB, 0.11601E+01_JPRB, 0.10844E+01_JPRB, 0.10826E+01_JPRB, 0.10957E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.98691E+00_JPRB, 0.10606E+01_JPRB, 0.11462E+01_JPRB, 0.12075E+01_JPRB, 0.12327E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.12412E+01_JPRB, 0.11806E+01_JPRB, 0.11265E+01_JPRB, 0.10290E+01_JPRB, 0.10441E+01_JPRB, 0.11044E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.10539E+01_JPRB, 0.10555E+01_JPRB, 0.10568E+01_JPRB, 0.10604E+01_JPRB, 0.10177E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.12766E+01_JPRB, 0.11806E+01_JPRB, 0.99792E+00_JPRB, 0.82840E+00_JPRB, 0.63211E+00_JPRB, 0.69082E+00_JPRB ,&
&  0.82954E+00_JPRB, 0.81536E+00_JPRB, 0.80237E+00_JPRB, 0.99185E+00_JPRB, 0.89972E+00_JPRB, 0.86291E+00_JPRB, 0.82011E+00_JPRB ,&
&  0.78784E+00_JPRB, 0.80116E+00_JPRB, 0.81175E+00_JPRB, 0.80009E+00_JPRB, 0.78062E+00_JPRB, 0.76276E+00_JPRB, 0.73707E+00_JPRB ,&
&  0.71603E+00_JPRB, 0.71113E+00_JPRB, 0.71878E+00_JPRB, 0.73275E+00_JPRB, 0.74639E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.74283E+00_JPRB, 0.74822E+00_JPRB, 0.71375E+00_JPRB, 0.66597E+00_JPRB, 0.63847E+00_JPRB, 0.63086E+00_JPRB, 0.63416E+00_JPRB ,&
&  0.71147E+00_JPRB, 0.71317E+00_JPRB, 0.68017E+00_JPRB, 0.64701E+00_JPRB, 0.63250E+00_JPRB, 0.61109E+00_JPRB, 0.59673E+00_JPRB ,&
&  0.57432E+00_JPRB, 0.56592E+00_JPRB, 0.61050E+00_JPRB, 0.62731E+00_JPRB, 0.64998E+00_JPRB, 0.56211E+00_JPRB, 0.65804E+00_JPRB ,&
&  0.11685E+01_JPRB, 0.19506E+01_JPRB, 0.18848E+01_JPRB, 0.19170E+01_JPRB, 0.18795E+01_JPRB, 0.18870E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.12137E+01_JPRB, 0.12049E+01_JPRB, 0.12484E+01_JPRB, 0.12643E+01_JPRB, 0.12618E+01_JPRB, 0.12480E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.11328E+01_JPRB, 0.10999E+01_JPRB, 0.10824E+01_JPRB, 0.10521E+01_JPRB, 0.10348E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10777E+01_JPRB, 0.11139E+01_JPRB, 0.10753E+01_JPRB, 0.10345E+01_JPRB, 0.10360E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10687E+01_JPRB, 0.10910E+01_JPRB, 0.11121E+01_JPRB, 0.11331E+01_JPRB, 0.12218E+01_JPRB, 0.13155E+01_JPRB, 0.13625E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.12810E+01_JPRB, 0.11166E+01_JPRB, 0.11073E+01_JPRB, 0.11812E+01_JPRB, 0.12308E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.10187E+01_JPRB, 0.89578E+00_JPRB, 0.88274E+00_JPRB, 0.95947E+00_JPRB, 0.10415E+01_JPRB, 0.10698E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.77815E+00_JPRB, 0.68091E+00_JPRB, 0.82054E+00_JPRB, 0.65296E+00_JPRB, 0.52366E+00_JPRB, 0.66562E+00_JPRB ,&
&  0.77953E+00_JPRB, 0.84996E+00_JPRB, 0.85162E+00_JPRB, 0.83559E+00_JPRB, 0.92467E+00_JPRB, 0.97607E+00_JPRB, 0.97147E+00_JPRB ,&
&  0.91545E+00_JPRB, 0.87143E+00_JPRB, 0.82679E+00_JPRB, 0.79793E+00_JPRB, 0.78432E+00_JPRB, 0.76923E+00_JPRB, 0.73862E+00_JPRB ,&
&  0.71988E+00_JPRB, 0.70753E+00_JPRB, 0.70617E+00_JPRB, 0.72373E+00_JPRB, 0.73784E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.74196E+00_JPRB, 0.73137E+00_JPRB, 0.69611E+00_JPRB, 0.65006E+00_JPRB, 0.62492E+00_JPRB, 0.61148E+00_JPRB, 0.60491E+00_JPRB ,&
&  0.62642E+00_JPRB, 0.64544E+00_JPRB, 0.64920E+00_JPRB, 0.63967E+00_JPRB, 0.63009E+00_JPRB, 0.61221E+00_JPRB, 0.59586E+00_JPRB ,&
&  0.57484E+00_JPRB, 0.55624E+00_JPRB, 0.62107E+00_JPRB, 0.66939E+00_JPRB, 0.66244E+00_JPRB, 0.59042E+00_JPRB, 0.74273E+00_JPRB ,&
&  0.11664E+01_JPRB, 0.19019E+01_JPRB, 0.22517E+01_JPRB, 0.22963E+01_JPRB, 0.21228E+01_JPRB, 0.20456E+01_JPRB, 0.16084E+01_JPRB ,&
&  0.14073E+01_JPRB, 0.13772E+01_JPRB, 0.13269E+01_JPRB, 0.13375E+01_JPRB, 0.13652E+01_JPRB, 0.13544E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.11945E+01_JPRB, 0.11451E+01_JPRB, 0.11168E+01_JPRB, 0.10779E+01_JPRB, 0.10290E+01_JPRB, 0.10106E+01_JPRB ,&
&  0.10374E+01_JPRB, 0.10478E+01_JPRB, 0.10726E+01_JPRB, 0.11023E+01_JPRB, 0.91826E+00_JPRB, 0.98804E+00_JPRB, 0.10218E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.11173E+01_JPRB, 0.11041E+01_JPRB, 0.11750E+01_JPRB, 0.12753E+01_JPRB, 0.13206E+01_JPRB, 0.14076E+01_JPRB ,&
&  0.15151E+01_JPRB, 0.14247E+01_JPRB, 0.12529E+01_JPRB, 0.12567E+01_JPRB, 0.13404E+01_JPRB, 0.13498E+01_JPRB, 0.96323E+00_JPRB ,&
&  0.89577E+00_JPRB, 0.10216E+01_JPRB, 0.96279E+00_JPRB, 0.87052E+00_JPRB, 0.99172E+00_JPRB, 0.10752E+01_JPRB, 0.94513E+00_JPRB ,&
&  0.52708E+00_JPRB, 0.43212E+00_JPRB, 0.43502E+00_JPRB, 0.44413E+00_JPRB, 0.43713E+00_JPRB, 0.47359E+00_JPRB, 0.81586E+00_JPRB ,&
&  0.78047E+00_JPRB, 0.86473E+00_JPRB, 0.87746E+00_JPRB, 0.85609E+00_JPRB, 0.92305E+00_JPRB, 0.10092E+01_JPRB, 0.12186E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.10298E+01_JPRB, 0.97091E+00_JPRB, 0.89090E+00_JPRB, 0.82571E+00_JPRB, 0.77791E+00_JPRB, 0.74661E+00_JPRB ,&
&  0.72562E+00_JPRB, 0.71429E+00_JPRB, 0.71347E+00_JPRB, 0.72488E+00_JPRB, 0.73953E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.74665E+00_JPRB, 0.71517E+00_JPRB, 0.66998E+00_JPRB, 0.62157E+00_JPRB, 0.60025E+00_JPRB, 0.58492E+00_JPRB, 0.57458E+00_JPRB ,&
&  0.58308E+00_JPRB, 0.59626E+00_JPRB, 0.61070E+00_JPRB, 0.61624E+00_JPRB, 0.61224E+00_JPRB, 0.60078E+00_JPRB, 0.58568E+00_JPRB ,&
&  0.56633E+00_JPRB, 0.55196E+00_JPRB, 0.66481E+00_JPRB, 0.67987E+00_JPRB, 0.70796E+00_JPRB, 0.66413E+00_JPRB, 0.77874E+00_JPRB ,&
&  0.11182E+01_JPRB, 0.17649E+01_JPRB, 0.22812E+01_JPRB, 0.23707E+01_JPRB, 0.24562E+01_JPRB, 0.24262E+01_JPRB, 0.21976E+01_JPRB ,&
&  0.19271E+01_JPRB, 0.17653E+01_JPRB, 0.16152E+01_JPRB, 0.15230E+01_JPRB, 0.14541E+01_JPRB, 0.14133E+01_JPRB, 0.13507E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.12566E+01_JPRB, 0.12178E+01_JPRB, 0.11926E+01_JPRB, 0.11420E+01_JPRB, 0.10817E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.10415E+01_JPRB, 0.10483E+01_JPRB, 0.10874E+01_JPRB, 0.98099E+00_JPRB, 0.90484E+00_JPRB, 0.99032E+00_JPRB ,&
&  0.10906E+01_JPRB, 0.12331E+01_JPRB, 0.12275E+01_JPRB, 0.12976E+01_JPRB, 0.13324E+01_JPRB, 0.13505E+01_JPRB, 0.14994E+01_JPRB ,&
&  0.16329E+01_JPRB, 0.17473E+01_JPRB, 0.14373E+01_JPRB, 0.14298E+01_JPRB, 0.15387E+01_JPRB, 0.12068E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.11596E+01_JPRB, 0.10508E+01_JPRB, 0.96787E+00_JPRB, 0.82791E+00_JPRB, 0.95096E+00_JPRB, 0.88735E+00_JPRB, 0.53696E+00_JPRB ,&
&  0.44073E+00_JPRB, 0.41046E+00_JPRB, 0.39298E+00_JPRB, 0.39380E+00_JPRB, 0.39238E+00_JPRB, 0.44812E+00_JPRB, 0.65666E+00_JPRB ,&
&  0.72582E+00_JPRB, 0.82691E+00_JPRB, 0.87981E+00_JPRB, 0.89640E+00_JPRB, 0.91938E+00_JPRB, 0.97062E+00_JPRB, 0.12440E+01_JPRB ,&
&  0.13664E+01_JPRB, 0.12990E+01_JPRB, 0.11526E+01_JPRB, 0.10515E+01_JPRB, 0.99483E+00_JPRB, 0.91852E+00_JPRB, 0.84150E+00_JPRB ,&
&  0.77754E+00_JPRB, 0.74672E+00_JPRB, 0.73593E+00_JPRB, 0.74515E+00_JPRB, 0.76476E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.76089E+00_JPRB, 0.73408E+00_JPRB, 0.68066E+00_JPRB, 0.62268E+00_JPRB, 0.59174E+00_JPRB, 0.56635E+00_JPRB, 0.55174E+00_JPRB ,&
&  0.55450E+00_JPRB, 0.56359E+00_JPRB, 0.57261E+00_JPRB, 0.57951E+00_JPRB, 0.57686E+00_JPRB, 0.57022E+00_JPRB, 0.56289E+00_JPRB ,&
&  0.54628E+00_JPRB, 0.56476E+00_JPRB, 0.62974E+00_JPRB, 0.58377E+00_JPRB, 0.58972E+00_JPRB, 0.59570E+00_JPRB, 0.68086E+00_JPRB ,&
&  0.10671E+01_JPRB, 0.16738E+01_JPRB, 0.22071E+01_JPRB, 0.25082E+01_JPRB, 0.27531E+01_JPRB, 0.26737E+01_JPRB, 0.28980E+01_JPRB ,&
&  0.26104E+01_JPRB, 0.22972E+01_JPRB, 0.20211E+01_JPRB, 0.18243E+01_JPRB, 0.16352E+01_JPRB, 0.15098E+01_JPRB, 0.14426E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.12855E+01_JPRB, 0.12551E+01_JPRB, 0.12249E+01_JPRB, 0.11860E+01_JPRB, 0.11422E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10861E+01_JPRB, 0.10749E+01_JPRB, 0.10635E+01_JPRB, 0.11153E+01_JPRB, 0.10792E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.11474E+01_JPRB, 0.13296E+01_JPRB, 0.12953E+01_JPRB, 0.13369E+01_JPRB, 0.14337E+01_JPRB, 0.15704E+01_JPRB ,&
&  0.17291E+01_JPRB, 0.18939E+01_JPRB, 0.16467E+01_JPRB, 0.16686E+01_JPRB, 0.15311E+01_JPRB, 0.12167E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.12017E+01_JPRB, 0.10137E+01_JPRB, 0.82164E+00_JPRB, 0.77973E+00_JPRB, 0.60414E+00_JPRB, 0.43442E+00_JPRB ,&
&  0.41738E+00_JPRB, 0.40635E+00_JPRB, 0.38645E+00_JPRB, 0.42238E+00_JPRB, 0.41848E+00_JPRB, 0.48054E+00_JPRB, 0.63834E+00_JPRB ,&
&  0.75937E+00_JPRB, 0.77186E+00_JPRB, 0.83172E+00_JPRB, 0.88215E+00_JPRB, 0.91852E+00_JPRB, 0.10073E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.12482E+01_JPRB, 0.12775E+01_JPRB, 0.12293E+01_JPRB, 0.11013E+01_JPRB, 0.10479E+01_JPRB, 0.10158E+01_JPRB, 0.96253E+00_JPRB ,&
&  0.88681E+00_JPRB, 0.81290E+00_JPRB, 0.78323E+00_JPRB, 0.78111E+00_JPRB, 0.77908E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.78562E+00_JPRB, 0.76567E+00_JPRB, 0.71415E+00_JPRB, 0.65378E+00_JPRB, 0.60929E+00_JPRB, 0.57641E+00_JPRB, 0.55992E+00_JPRB ,&
&  0.56288E+00_JPRB, 0.56438E+00_JPRB, 0.56729E+00_JPRB, 0.56734E+00_JPRB, 0.56209E+00_JPRB, 0.55272E+00_JPRB, 0.55109E+00_JPRB ,&
&  0.53713E+00_JPRB, 0.65834E+00_JPRB, 0.49630E+00_JPRB, 0.42963E+00_JPRB, 0.48927E+00_JPRB, 0.56214E+00_JPRB, 0.58167E+00_JPRB ,&
&  0.98256E+00_JPRB, 0.15977E+01_JPRB, 0.20926E+01_JPRB, 0.27157E+01_JPRB, 0.35935E+01_JPRB, 0.34249E+01_JPRB, 0.34210E+01_JPRB ,&
&  0.32626E+01_JPRB, 0.28897E+01_JPRB, 0.25073E+01_JPRB, 0.21379E+01_JPRB, 0.18138E+01_JPRB, 0.16211E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.13984E+01_JPRB, 0.13134E+01_JPRB, 0.12631E+01_JPRB, 0.12441E+01_JPRB, 0.12061E+01_JPRB, 0.11715E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.11453E+01_JPRB, 0.11363E+01_JPRB, 0.11189E+01_JPRB, 0.11151E+01_JPRB, 0.10629E+01_JPRB, 0.10217E+01_JPRB, 0.12285E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.12745E+01_JPRB, 0.12780E+01_JPRB, 0.13704E+01_JPRB, 0.14528E+01_JPRB, 0.15176E+01_JPRB, 0.17148E+01_JPRB ,&
&  0.16409E+01_JPRB, 0.16243E+01_JPRB, 0.16461E+01_JPRB, 0.16358E+01_JPRB, 0.14806E+01_JPRB, 0.15567E+01_JPRB, 0.19459E+01_JPRB ,&
&  0.15388E+01_JPRB, 0.15277E+01_JPRB, 0.15058E+01_JPRB, 0.14110E+01_JPRB, 0.78893E+00_JPRB, 0.59194E+00_JPRB, 0.80606E+00_JPRB ,&
&  0.84726E+00_JPRB, 0.59011E+00_JPRB, 0.48990E+00_JPRB, 0.62947E+00_JPRB, 0.52635E+00_JPRB, 0.57786E+00_JPRB, 0.72949E+00_JPRB ,&
&  0.73491E+00_JPRB, 0.68201E+00_JPRB, 0.79548E+00_JPRB, 0.91070E+00_JPRB, 0.10030E+01_JPRB, 0.10470E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.12293E+01_JPRB, 0.12108E+01_JPRB, 0.12483E+01_JPRB, 0.11477E+01_JPRB, 0.10665E+01_JPRB, 0.10185E+01_JPRB, 0.97971E+00_JPRB ,&
&  0.92841E+00_JPRB, 0.87103E+00_JPRB, 0.83823E+00_JPRB, 0.82810E+00_JPRB, 0.80529E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.80087E+00_JPRB, 0.78292E+00_JPRB, 0.73101E+00_JPRB, 0.67541E+00_JPRB, 0.61311E+00_JPRB, 0.56448E+00_JPRB, 0.55105E+00_JPRB ,&
&  0.55907E+00_JPRB, 0.56794E+00_JPRB, 0.56937E+00_JPRB, 0.56416E+00_JPRB, 0.55548E+00_JPRB, 0.54670E+00_JPRB, 0.53982E+00_JPRB ,&
&  0.52879E+00_JPRB, 0.50590E+00_JPRB, 0.39486E+00_JPRB, 0.35632E+00_JPRB, 0.48131E+00_JPRB, 0.51219E+00_JPRB, 0.57635E+00_JPRB ,&
&  0.87027E+00_JPRB, 0.13669E+01_JPRB, 0.19545E+01_JPRB, 0.26414E+01_JPRB, 0.33326E+01_JPRB, 0.36627E+01_JPRB, 0.36456E+01_JPRB ,&
&  0.36990E+01_JPRB, 0.33194E+01_JPRB, 0.27982E+01_JPRB, 0.23642E+01_JPRB, 0.20625E+01_JPRB, 0.18596E+01_JPRB, 0.16575E+01_JPRB ,&
&  0.14758E+01_JPRB, 0.13543E+01_JPRB, 0.12875E+01_JPRB, 0.12428E+01_JPRB, 0.12171E+01_JPRB, 0.11773E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11456E+01_JPRB, 0.11676E+01_JPRB, 0.11794E+01_JPRB, 0.11575E+01_JPRB, 0.10042E+01_JPRB, 0.95121E+00_JPRB, 0.11055E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.12590E+01_JPRB, 0.12968E+01_JPRB, 0.13931E+01_JPRB, 0.15153E+01_JPRB, 0.15239E+01_JPRB, 0.18041E+01_JPRB ,&
&  0.17502E+01_JPRB, 0.16393E+01_JPRB, 0.15528E+01_JPRB, 0.13610E+01_JPRB, 0.14979E+01_JPRB, 0.20673E+01_JPRB, 0.20878E+01_JPRB ,&
&  0.19221E+01_JPRB, 0.17983E+01_JPRB, 0.16586E+01_JPRB, 0.14326E+01_JPRB, 0.11879E+01_JPRB, 0.87612E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10680E+01_JPRB, 0.10068E+01_JPRB, 0.88075E+00_JPRB, 0.76032E+00_JPRB, 0.85833E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.66683E+00_JPRB, 0.61212E+00_JPRB, 0.70382E+00_JPRB, 0.85839E+00_JPRB, 0.94886E+00_JPRB, 0.95363E+00_JPRB, 0.11052E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11758E+01_JPRB, 0.11688E+01_JPRB, 0.11052E+01_JPRB, 0.10434E+01_JPRB, 0.99344E+00_JPRB, 0.94097E+00_JPRB ,&
&  0.90752E+00_JPRB, 0.88466E+00_JPRB, 0.85972E+00_JPRB, 0.84285E+00_JPRB, 0.81641E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.82086E+00_JPRB, 0.79495E+00_JPRB, 0.74009E+00_JPRB, 0.68112E+00_JPRB, 0.61234E+00_JPRB, 0.54930E+00_JPRB, 0.53277E+00_JPRB ,&
&  0.54036E+00_JPRB, 0.54883E+00_JPRB, 0.54757E+00_JPRB, 0.53871E+00_JPRB, 0.53247E+00_JPRB, 0.52393E+00_JPRB, 0.51932E+00_JPRB ,&
&  0.51720E+00_JPRB, 0.46232E+00_JPRB, 0.36184E+00_JPRB, 0.35850E+00_JPRB, 0.42398E+00_JPRB, 0.46788E+00_JPRB, 0.58360E+00_JPRB ,&
&  0.76793E+00_JPRB, 0.12225E+01_JPRB, 0.16561E+01_JPRB, 0.21835E+01_JPRB, 0.27360E+01_JPRB, 0.31619E+01_JPRB, 0.32073E+01_JPRB ,&
&  0.30150E+01_JPRB, 0.29946E+01_JPRB, 0.27606E+01_JPRB, 0.24504E+01_JPRB, 0.21731E+01_JPRB, 0.19636E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.15999E+01_JPRB, 0.14312E+01_JPRB, 0.13305E+01_JPRB, 0.12679E+01_JPRB, 0.12259E+01_JPRB, 0.11905E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11549E+01_JPRB, 0.11733E+01_JPRB, 0.11721E+01_JPRB, 0.11629E+01_JPRB, 0.97263E+00_JPRB, 0.99000E+00_JPRB, 0.11321E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.14095E+01_JPRB, 0.14403E+01_JPRB, 0.16715E+01_JPRB, 0.15454E+01_JPRB, 0.16183E+01_JPRB, 0.19865E+01_JPRB ,&
&  0.22945E+01_JPRB, 0.23417E+01_JPRB, 0.23721E+01_JPRB, 0.19077E+01_JPRB, 0.16809E+01_JPRB, 0.23321E+01_JPRB, 0.21965E+01_JPRB ,&
&  0.20064E+01_JPRB, 0.18881E+01_JPRB, 0.17247E+01_JPRB, 0.16712E+01_JPRB, 0.14105E+01_JPRB, 0.12120E+01_JPRB, 0.94181E+00_JPRB ,&
&  0.97587E+00_JPRB, 0.93238E+00_JPRB, 0.10328E+01_JPRB, 0.97021E+00_JPRB, 0.90240E+00_JPRB, 0.81988E+00_JPRB, 0.68900E+00_JPRB ,&
&  0.60775E+00_JPRB, 0.55270E+00_JPRB, 0.55467E+00_JPRB, 0.66427E+00_JPRB, 0.76742E+00_JPRB, 0.74946E+00_JPRB, 0.89727E+00_JPRB ,&
&  0.10205E+01_JPRB, 0.11070E+01_JPRB, 0.10548E+01_JPRB, 0.10685E+01_JPRB, 0.10433E+01_JPRB, 0.10112E+01_JPRB, 0.95868E+00_JPRB ,&
&  0.91357E+00_JPRB, 0.88453E+00_JPRB, 0.86605E+00_JPRB, 0.84904E+00_JPRB, 0.83107E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.82159E+00_JPRB, 0.79381E+00_JPRB, 0.73865E+00_JPRB, 0.68083E+00_JPRB, 0.60248E+00_JPRB, 0.52994E+00_JPRB, 0.51673E+00_JPRB ,&
&  0.53128E+00_JPRB, 0.53366E+00_JPRB, 0.52349E+00_JPRB, 0.50099E+00_JPRB, 0.49597E+00_JPRB, 0.50363E+00_JPRB, 0.50893E+00_JPRB ,&
&  0.51121E+00_JPRB, 0.58285E+00_JPRB, 0.46404E+00_JPRB, 0.38451E+00_JPRB, 0.37799E+00_JPRB, 0.45088E+00_JPRB, 0.59706E+00_JPRB ,&
&  0.75487E+00_JPRB, 0.10413E+01_JPRB, 0.13447E+01_JPRB, 0.16095E+01_JPRB, 0.19644E+01_JPRB, 0.22416E+01_JPRB, 0.23855E+01_JPRB ,&
&  0.25368E+01_JPRB, 0.21491E+01_JPRB, 0.21434E+01_JPRB, 0.19759E+01_JPRB, 0.18127E+01_JPRB, 0.16774E+01_JPRB, 0.15695E+01_JPRB ,&
&  0.14383E+01_JPRB, 0.13402E+01_JPRB, 0.12275E+01_JPRB, 0.11501E+01_JPRB, 0.11074E+01_JPRB, 0.10690E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.10606E+01_JPRB, 0.11113E+01_JPRB, 0.11619E+01_JPRB, 0.11542E+01_JPRB, 0.11790E+01_JPRB, 0.12738E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.13778E+01_JPRB, 0.13691E+01_JPRB, 0.17746E+01_JPRB, 0.19851E+01_JPRB, 0.22334E+01_JPRB, 0.23537E+01_JPRB, 0.24675E+01_JPRB ,&
&  0.27314E+01_JPRB, 0.26764E+01_JPRB, 0.25909E+01_JPRB, 0.23027E+01_JPRB, 0.24999E+01_JPRB, 0.24303E+01_JPRB, 0.22554E+01_JPRB ,&
&  0.20458E+01_JPRB, 0.19854E+01_JPRB, 0.18275E+01_JPRB, 0.17083E+01_JPRB, 0.16489E+01_JPRB, 0.16624E+01_JPRB, 0.14762E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12801E+01_JPRB, 0.11407E+01_JPRB, 0.96129E+00_JPRB, 0.75790E+00_JPRB, 0.67452E+00_JPRB, 0.61006E+00_JPRB ,&
&  0.56284E+00_JPRB, 0.53145E+00_JPRB, 0.51923E+00_JPRB, 0.56598E+00_JPRB, 0.63037E+00_JPRB, 0.65318E+00_JPRB, 0.68597E+00_JPRB ,&
&  0.72298E+00_JPRB, 0.88275E+00_JPRB, 0.91777E+00_JPRB, 0.91308E+00_JPRB, 0.94751E+00_JPRB, 0.96881E+00_JPRB, 0.95907E+00_JPRB ,&
&  0.92008E+00_JPRB, 0.89002E+00_JPRB, 0.86776E+00_JPRB, 0.85218E+00_JPRB, 0.83474E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.80461E+00_JPRB, 0.78038E+00_JPRB, 0.73573E+00_JPRB, 0.67795E+00_JPRB, 0.59694E+00_JPRB, 0.52752E+00_JPRB, 0.51140E+00_JPRB ,&
&  0.50990E+00_JPRB, 0.50616E+00_JPRB, 0.49572E+00_JPRB, 0.48901E+00_JPRB, 0.49079E+00_JPRB, 0.49673E+00_JPRB, 0.51823E+00_JPRB ,&
&  0.56286E+00_JPRB, 0.85779E+00_JPRB, 0.65428E+00_JPRB, 0.53994E+00_JPRB, 0.47777E+00_JPRB, 0.56927E+00_JPRB, 0.66361E+00_JPRB ,&
&  0.80174E+00_JPRB, 0.93874E+00_JPRB, 0.10233E+01_JPRB, 0.12254E+01_JPRB, 0.14296E+01_JPRB, 0.15031E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.16483E+01_JPRB, 0.16598E+01_JPRB, 0.15019E+01_JPRB, 0.14957E+01_JPRB, 0.14220E+01_JPRB, 0.12747E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.12044E+01_JPRB, 0.11273E+01_JPRB, 0.10500E+01_JPRB, 0.10082E+01_JPRB, 0.10066E+01_JPRB, 0.10017E+01_JPRB, 0.96872E+00_JPRB ,&
&  0.98339E+00_JPRB, 0.10287E+01_JPRB, 0.11032E+01_JPRB, 0.11648E+01_JPRB, 0.12277E+01_JPRB, 0.13665E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.17884E+01_JPRB, 0.17772E+01_JPRB, 0.16166E+01_JPRB, 0.20641E+01_JPRB, 0.27915E+01_JPRB, 0.28779E+01_JPRB, 0.26679E+01_JPRB ,&
&  0.29088E+01_JPRB, 0.28964E+01_JPRB, 0.27869E+01_JPRB, 0.26354E+01_JPRB, 0.24763E+01_JPRB, 0.23735E+01_JPRB, 0.22969E+01_JPRB ,&
&  0.20627E+01_JPRB, 0.19296E+01_JPRB, 0.18374E+01_JPRB, 0.17430E+01_JPRB, 0.16212E+01_JPRB, 0.15587E+01_JPRB, 0.15283E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.11248E+01_JPRB, 0.78827E+00_JPRB, 0.76650E+00_JPRB, 0.57813E+00_JPRB, 0.50647E+00_JPRB, 0.53825E+00_JPRB ,&
&  0.51727E+00_JPRB, 0.54462E+00_JPRB, 0.57245E+00_JPRB, 0.54573E+00_JPRB, 0.55161E+00_JPRB, 0.53067E+00_JPRB, 0.55705E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.59895E+00_JPRB, 0.73835E+00_JPRB, 0.76971E+00_JPRB, 0.80610E+00_JPRB, 0.83365E+00_JPRB, 0.84901E+00_JPRB ,&
&  0.85128E+00_JPRB, 0.84137E+00_JPRB, 0.83235E+00_JPRB, 0.82092E+00_JPRB, 0.81719E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.72793E+00_JPRB, 0.70604E+00_JPRB, 0.67440E+00_JPRB, 0.62973E+00_JPRB, 0.57030E+00_JPRB, 0.51269E+00_JPRB, 0.48641E+00_JPRB ,&
&  0.48275E+00_JPRB, 0.47417E+00_JPRB, 0.46838E+00_JPRB, 0.47018E+00_JPRB, 0.47896E+00_JPRB, 0.49365E+00_JPRB, 0.54102E+00_JPRB ,&
&  0.65629E+00_JPRB, 0.85287E+00_JPRB, 0.71242E+00_JPRB, 0.59558E+00_JPRB, 0.70924E+00_JPRB, 0.68128E+00_JPRB, 0.73329E+00_JPRB ,&
&  0.76448E+00_JPRB, 0.90636E+00_JPRB, 0.90194E+00_JPRB, 0.92709E+00_JPRB, 0.10515E+01_JPRB, 0.11300E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.10930E+01_JPRB, 0.10373E+01_JPRB, 0.10501E+01_JPRB, 0.11157E+01_JPRB, 0.10601E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.94923E+00_JPRB, 0.88067E+00_JPRB, 0.85888E+00_JPRB, 0.87780E+00_JPRB, 0.91938E+00_JPRB, 0.94290E+00_JPRB ,&
&  0.99209E+00_JPRB, 0.10477E+01_JPRB, 0.10948E+01_JPRB, 0.11842E+01_JPRB, 0.13058E+01_JPRB, 0.16067E+01_JPRB, 0.20123E+01_JPRB ,&
&  0.23686E+01_JPRB, 0.25633E+01_JPRB, 0.27881E+01_JPRB, 0.31841E+01_JPRB, 0.32444E+01_JPRB, 0.31329E+01_JPRB, 0.30173E+01_JPRB ,&
&  0.28394E+01_JPRB, 0.27423E+01_JPRB, 0.27154E+01_JPRB, 0.25496E+01_JPRB, 0.24017E+01_JPRB, 0.23268E+01_JPRB, 0.22216E+01_JPRB ,&
&  0.19332E+01_JPRB, 0.17679E+01_JPRB, 0.17320E+01_JPRB, 0.16136E+01_JPRB, 0.15200E+01_JPRB, 0.14171E+01_JPRB, 0.13937E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.82528E+00_JPRB, 0.63892E+00_JPRB, 0.67713E+00_JPRB, 0.54374E+00_JPRB, 0.50191E+00_JPRB, 0.56624E+00_JPRB ,&
&  0.55771E+00_JPRB, 0.55639E+00_JPRB, 0.57084E+00_JPRB, 0.53001E+00_JPRB, 0.49146E+00_JPRB, 0.48564E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.47301E+00_JPRB, 0.49129E+00_JPRB, 0.59591E+00_JPRB, 0.66349E+00_JPRB, 0.70538E+00_JPRB, 0.72295E+00_JPRB, 0.73875E+00_JPRB ,&
&  0.74537E+00_JPRB, 0.74802E+00_JPRB, 0.74959E+00_JPRB, 0.75423E+00_JPRB, 0.75423E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.65077E+00_JPRB, 0.64104E+00_JPRB, 0.62168E+00_JPRB, 0.59523E+00_JPRB, 0.55851E+00_JPRB, 0.51833E+00_JPRB, 0.47890E+00_JPRB ,&
&  0.47491E+00_JPRB, 0.46692E+00_JPRB, 0.46235E+00_JPRB, 0.46372E+00_JPRB, 0.47150E+00_JPRB, 0.48337E+00_JPRB, 0.50783E+00_JPRB ,&
&  0.45112E+00_JPRB, 0.55376E+00_JPRB, 0.51004E+00_JPRB, 0.60378E+00_JPRB, 0.84797E+00_JPRB, 0.68659E+00_JPRB, 0.71233E+00_JPRB ,&
&  0.73995E+00_JPRB, 0.80702E+00_JPRB, 0.82499E+00_JPRB, 0.82924E+00_JPRB, 0.87042E+00_JPRB, 0.89911E+00_JPRB, 0.88894E+00_JPRB ,&
&  0.82720E+00_JPRB, 0.73593E+00_JPRB, 0.71300E+00_JPRB, 0.71664E+00_JPRB, 0.79596E+00_JPRB, 0.85846E+00_JPRB, 0.87527E+00_JPRB ,&
&  0.84402E+00_JPRB, 0.81226E+00_JPRB, 0.78255E+00_JPRB, 0.79460E+00_JPRB, 0.82607E+00_JPRB, 0.85268E+00_JPRB, 0.90328E+00_JPRB ,&
&  0.97530E+00_JPRB, 0.10543E+01_JPRB, 0.11474E+01_JPRB, 0.12733E+01_JPRB, 0.12902E+01_JPRB, 0.19771E+01_JPRB, 0.31787E+01_JPRB ,&
&  0.34975E+01_JPRB, 0.33529E+01_JPRB, 0.35512E+01_JPRB, 0.40661E+01_JPRB, 0.39409E+01_JPRB, 0.35183E+01_JPRB, 0.30173E+01_JPRB ,&
&  0.26250E+01_JPRB, 0.24319E+01_JPRB, 0.24648E+01_JPRB, 0.23835E+01_JPRB, 0.21555E+01_JPRB, 0.21188E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.18595E+01_JPRB, 0.17190E+01_JPRB, 0.16529E+01_JPRB, 0.16106E+01_JPRB, 0.15070E+01_JPRB, 0.14987E+01_JPRB, 0.14250E+01_JPRB ,&
&  0.13583E+01_JPRB, 0.11334E+01_JPRB, 0.81562E+00_JPRB, 0.67015E+00_JPRB, 0.52952E+00_JPRB, 0.52875E+00_JPRB, 0.62331E+00_JPRB ,&
&  0.60671E+00_JPRB, 0.56537E+00_JPRB, 0.51032E+00_JPRB, 0.46245E+00_JPRB, 0.43111E+00_JPRB, 0.42735E+00_JPRB, 0.40182E+00_JPRB ,&
&  0.39437E+00_JPRB, 0.43118E+00_JPRB, 0.48919E+00_JPRB, 0.55082E+00_JPRB, 0.60160E+00_JPRB, 0.64094E+00_JPRB, 0.61111E+00_JPRB ,&
&  0.67322E+00_JPRB, 0.67822E+00_JPRB, 0.68602E+00_JPRB, 0.69605E+00_JPRB, 0.68243E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.56566E+00_JPRB, 0.57490E+00_JPRB, 0.56139E+00_JPRB, 0.55502E+00_JPRB, 0.52907E+00_JPRB, 0.50851E+00_JPRB, 0.47474E+00_JPRB ,&
&  0.48095E+00_JPRB, 0.47596E+00_JPRB, 0.46286E+00_JPRB, 0.45855E+00_JPRB, 0.46844E+00_JPRB, 0.46971E+00_JPRB, 0.42134E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.43021E+00_JPRB, 0.42699E+00_JPRB, 0.48910E+00_JPRB, 0.55130E+00_JPRB, 0.52834E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.64966E+00_JPRB, 0.68431E+00_JPRB, 0.70964E+00_JPRB, 0.72396E+00_JPRB, 0.73418E+00_JPRB, 0.74745E+00_JPRB, 0.75154E+00_JPRB ,&
&  0.69286E+00_JPRB, 0.64637E+00_JPRB, 0.61115E+00_JPRB, 0.60730E+00_JPRB, 0.68203E+00_JPRB, 0.72351E+00_JPRB, 0.70605E+00_JPRB ,&
&  0.68327E+00_JPRB, 0.67831E+00_JPRB, 0.68913E+00_JPRB, 0.74000E+00_JPRB, 0.77804E+00_JPRB, 0.79999E+00_JPRB, 0.85108E+00_JPRB ,&
&  0.92819E+00_JPRB, 0.10359E+01_JPRB, 0.11471E+01_JPRB, 0.12704E+01_JPRB, 0.13406E+01_JPRB, 0.20430E+01_JPRB, 0.29519E+01_JPRB ,&
&  0.30060E+01_JPRB, 0.31338E+01_JPRB, 0.32815E+01_JPRB, 0.33696E+01_JPRB, 0.33206E+01_JPRB, 0.28753E+01_JPRB, 0.23956E+01_JPRB ,&
&  0.21669E+01_JPRB, 0.20082E+01_JPRB, 0.23330E+01_JPRB, 0.22086E+01_JPRB, 0.20395E+01_JPRB, 0.19184E+01_JPRB, 0.17999E+01_JPRB ,&
&  0.17089E+01_JPRB, 0.16350E+01_JPRB, 0.15573E+01_JPRB, 0.14758E+01_JPRB, 0.13980E+01_JPRB, 0.13631E+01_JPRB, 0.12658E+01_JPRB ,&
&  0.12032E+01_JPRB, 0.10960E+01_JPRB, 0.88347E+00_JPRB, 0.72514E+00_JPRB, 0.60361E+00_JPRB, 0.60900E+00_JPRB, 0.56577E+00_JPRB ,&
&  0.50077E+00_JPRB, 0.43362E+00_JPRB, 0.41889E+00_JPRB, 0.39940E+00_JPRB, 0.39131E+00_JPRB, 0.38575E+00_JPRB, 0.37466E+00_JPRB ,&
&  0.39289E+00_JPRB, 0.43098E+00_JPRB, 0.43730E+00_JPRB, 0.46556E+00_JPRB, 0.50633E+00_JPRB, 0.56338E+00_JPRB, 0.49576E+00_JPRB ,&
&  0.51837E+00_JPRB, 0.58007E+00_JPRB, 0.59315E+00_JPRB, 0.59323E+00_JPRB, 0.57510E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.52699E+00_JPRB, 0.53908E+00_JPRB, 0.52624E+00_JPRB, 0.53196E+00_JPRB, 0.51711E+00_JPRB, 0.50998E+00_JPRB, 0.45572E+00_JPRB ,&
&  0.48499E+00_JPRB, 0.47293E+00_JPRB, 0.45290E+00_JPRB, 0.44915E+00_JPRB, 0.46573E+00_JPRB, 0.40525E+00_JPRB, 0.34745E+00_JPRB ,&
&  0.33085E+00_JPRB, 0.34961E+00_JPRB, 0.39938E+00_JPRB, 0.42723E+00_JPRB, 0.43872E+00_JPRB, 0.44841E+00_JPRB, 0.45328E+00_JPRB ,&
&  0.50944E+00_JPRB, 0.53459E+00_JPRB, 0.58528E+00_JPRB, 0.61498E+00_JPRB, 0.61207E+00_JPRB, 0.62223E+00_JPRB, 0.64721E+00_JPRB ,&
&  0.61144E+00_JPRB, 0.59243E+00_JPRB, 0.58789E+00_JPRB, 0.53601E+00_JPRB, 0.60224E+00_JPRB, 0.58188E+00_JPRB, 0.55488E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.58775E+00_JPRB, 0.62283E+00_JPRB, 0.66076E+00_JPRB, 0.68929E+00_JPRB, 0.72296E+00_JPRB, 0.76989E+00_JPRB ,&
&  0.83875E+00_JPRB, 0.97175E+00_JPRB, 0.11206E+01_JPRB, 0.12699E+01_JPRB, 0.14646E+01_JPRB, 0.20040E+01_JPRB, 0.26390E+01_JPRB ,&
&  0.27464E+01_JPRB, 0.28813E+01_JPRB, 0.30110E+01_JPRB, 0.26274E+01_JPRB, 0.27116E+01_JPRB, 0.25764E+01_JPRB, 0.21895E+01_JPRB ,&
&  0.19797E+01_JPRB, 0.17111E+01_JPRB, 0.16378E+01_JPRB, 0.16067E+01_JPRB, 0.16348E+01_JPRB, 0.15336E+01_JPRB, 0.14432E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.14861E+01_JPRB, 0.13643E+01_JPRB, 0.12871E+01_JPRB, 0.11674E+01_JPRB, 0.10775E+01_JPRB, 0.98015E+00_JPRB ,&
&  0.91325E+00_JPRB, 0.83583E+00_JPRB, 0.75863E+00_JPRB, 0.69827E+00_JPRB, 0.62687E+00_JPRB, 0.57318E+00_JPRB, 0.53255E+00_JPRB ,&
&  0.48985E+00_JPRB, 0.43725E+00_JPRB, 0.41657E+00_JPRB, 0.40158E+00_JPRB, 0.39354E+00_JPRB, 0.38070E+00_JPRB, 0.38419E+00_JPRB ,&
&  0.40618E+00_JPRB, 0.43476E+00_JPRB, 0.46107E+00_JPRB, 0.45867E+00_JPRB, 0.47315E+00_JPRB, 0.50457E+00_JPRB, 0.48414E+00_JPRB ,&
&  0.46298E+00_JPRB, 0.52366E+00_JPRB, 0.52956E+00_JPRB, 0.53083E+00_JPRB, 0.52543E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.51655E+00_JPRB, 0.52005E+00_JPRB, 0.51256E+00_JPRB, 0.51357E+00_JPRB, 0.51549E+00_JPRB, 0.47644E+00_JPRB, 0.43846E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.43618E+00_JPRB, 0.46548E+00_JPRB, 0.38326E+00_JPRB, 0.35741E+00_JPRB, 0.35425E+00_JPRB, 0.34227E+00_JPRB ,&
&  0.33273E+00_JPRB, 0.35586E+00_JPRB, 0.38245E+00_JPRB, 0.39383E+00_JPRB, 0.41457E+00_JPRB, 0.42524E+00_JPRB, 0.42654E+00_JPRB ,&
&  0.43920E+00_JPRB, 0.45901E+00_JPRB, 0.49720E+00_JPRB, 0.50222E+00_JPRB, 0.48868E+00_JPRB, 0.49398E+00_JPRB, 0.50774E+00_JPRB ,&
&  0.49145E+00_JPRB, 0.47654E+00_JPRB, 0.49289E+00_JPRB, 0.49611E+00_JPRB, 0.49390E+00_JPRB, 0.46901E+00_JPRB, 0.45899E+00_JPRB ,&
&  0.47333E+00_JPRB, 0.45710E+00_JPRB, 0.52294E+00_JPRB, 0.55192E+00_JPRB, 0.58725E+00_JPRB, 0.62866E+00_JPRB, 0.67025E+00_JPRB ,&
&  0.73936E+00_JPRB, 0.86086E+00_JPRB, 0.10128E+01_JPRB, 0.11463E+01_JPRB, 0.13422E+01_JPRB, 0.18528E+01_JPRB, 0.22196E+01_JPRB ,&
&  0.25111E+01_JPRB, 0.18131E+01_JPRB, 0.21728E+01_JPRB, 0.21843E+01_JPRB, 0.23260E+01_JPRB, 0.22090E+01_JPRB, 0.20000E+01_JPRB ,&
&  0.17703E+01_JPRB, 0.14599E+01_JPRB, 0.12914E+01_JPRB, 0.11809E+01_JPRB, 0.11554E+01_JPRB, 0.11894E+01_JPRB, 0.11998E+01_JPRB ,&
&  0.12178E+01_JPRB, 0.11500E+01_JPRB, 0.10407E+01_JPRB, 0.94035E+00_JPRB, 0.86603E+00_JPRB, 0.81438E+00_JPRB, 0.76344E+00_JPRB ,&
&  0.71785E+00_JPRB, 0.67016E+00_JPRB, 0.62365E+00_JPRB, 0.56748E+00_JPRB, 0.54062E+00_JPRB, 0.51713E+00_JPRB, 0.49521E+00_JPRB ,&
&  0.48266E+00_JPRB, 0.45705E+00_JPRB, 0.42976E+00_JPRB, 0.41646E+00_JPRB, 0.40370E+00_JPRB, 0.40343E+00_JPRB, 0.41580E+00_JPRB ,&
&  0.41752E+00_JPRB, 0.40511E+00_JPRB, 0.41904E+00_JPRB, 0.42586E+00_JPRB, 0.42825E+00_JPRB, 0.43870E+00_JPRB, 0.48494E+00_JPRB ,&
&  0.46514E+00_JPRB, 0.48255E+00_JPRB, 0.51655E+00_JPRB, 0.51596E+00_JPRB, 0.51544E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.51633E+00_JPRB, 0.51445E+00_JPRB, 0.50617E+00_JPRB, 0.49537E+00_JPRB, 0.49659E+00_JPRB, 0.47608E+00_JPRB, 0.41666E+00_JPRB ,&
&  0.42129E+00_JPRB, 0.38120E+00_JPRB, 0.35540E+00_JPRB, 0.34491E+00_JPRB, 0.34735E+00_JPRB, 0.34455E+00_JPRB, 0.34067E+00_JPRB ,&
&  0.34287E+00_JPRB, 0.35768E+00_JPRB, 0.37998E+00_JPRB, 0.38863E+00_JPRB, 0.39906E+00_JPRB, 0.40770E+00_JPRB, 0.40747E+00_JPRB ,&
&  0.41490E+00_JPRB, 0.42687E+00_JPRB, 0.43952E+00_JPRB, 0.44862E+00_JPRB, 0.43891E+00_JPRB, 0.42954E+00_JPRB, 0.42224E+00_JPRB ,&
&  0.41668E+00_JPRB, 0.41595E+00_JPRB, 0.41944E+00_JPRB, 0.43942E+00_JPRB, 0.42854E+00_JPRB, 0.41754E+00_JPRB, 0.42397E+00_JPRB ,&
&  0.34737E+00_JPRB, 0.36016E+00_JPRB, 0.44689E+00_JPRB, 0.47650E+00_JPRB, 0.50484E+00_JPRB, 0.53858E+00_JPRB, 0.59090E+00_JPRB ,&
&  0.65407E+00_JPRB, 0.74694E+00_JPRB, 0.86028E+00_JPRB, 0.96897E+00_JPRB, 0.11193E+01_JPRB, 0.14353E+01_JPRB, 0.17261E+01_JPRB ,&
&  0.19692E+01_JPRB, 0.15693E+01_JPRB, 0.14886E+01_JPRB, 0.15964E+01_JPRB, 0.18430E+01_JPRB, 0.15843E+01_JPRB, 0.13919E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.10926E+01_JPRB, 0.10348E+01_JPRB, 0.10730E+01_JPRB, 0.10692E+01_JPRB, 0.10933E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.96989E+00_JPRB, 0.88018E+00_JPRB, 0.79358E+00_JPRB, 0.75149E+00_JPRB, 0.72250E+00_JPRB, 0.69915E+00_JPRB, 0.67653E+00_JPRB ,&
&  0.65782E+00_JPRB, 0.62953E+00_JPRB, 0.58417E+00_JPRB, 0.53521E+00_JPRB, 0.50319E+00_JPRB, 0.48429E+00_JPRB, 0.47055E+00_JPRB ,&
&  0.45731E+00_JPRB, 0.44420E+00_JPRB, 0.43577E+00_JPRB, 0.43260E+00_JPRB, 0.43256E+00_JPRB, 0.43371E+00_JPRB, 0.44406E+00_JPRB ,&
&  0.42312E+00_JPRB, 0.40002E+00_JPRB, 0.37967E+00_JPRB, 0.38473E+00_JPRB, 0.40285E+00_JPRB, 0.42157E+00_JPRB, 0.43522E+00_JPRB ,&
&  0.45735E+00_JPRB, 0.46831E+00_JPRB, 0.46030E+00_JPRB, 0.47182E+00_JPRB, 0.48849E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.51561E+00_JPRB, 0.51153E+00_JPRB, 0.49498E+00_JPRB, 0.49026E+00_JPRB, 0.47605E+00_JPRB, 0.46746E+00_JPRB, 0.44408E+00_JPRB ,&
&  0.43813E+00_JPRB, 0.41533E+00_JPRB, 0.38912E+00_JPRB, 0.36152E+00_JPRB, 0.34926E+00_JPRB, 0.34545E+00_JPRB, 0.34861E+00_JPRB ,&
&  0.35763E+00_JPRB, 0.36824E+00_JPRB, 0.37947E+00_JPRB, 0.38194E+00_JPRB, 0.38945E+00_JPRB, 0.39587E+00_JPRB, 0.39729E+00_JPRB ,&
&  0.39913E+00_JPRB, 0.40273E+00_JPRB, 0.40095E+00_JPRB, 0.39429E+00_JPRB, 0.39167E+00_JPRB, 0.39319E+00_JPRB, 0.39101E+00_JPRB ,&
&  0.38256E+00_JPRB, 0.37854E+00_JPRB, 0.38843E+00_JPRB, 0.41706E+00_JPRB, 0.40113E+00_JPRB, 0.40467E+00_JPRB, 0.35974E+00_JPRB ,&
&  0.31514E+00_JPRB, 0.30759E+00_JPRB, 0.33395E+00_JPRB, 0.42204E+00_JPRB, 0.44829E+00_JPRB, 0.48310E+00_JPRB, 0.51276E+00_JPRB ,&
&  0.51820E+00_JPRB, 0.55602E+00_JPRB, 0.64978E+00_JPRB, 0.83647E+00_JPRB, 0.93862E+00_JPRB, 0.11467E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.15065E+01_JPRB, 0.15294E+01_JPRB, 0.12716E+01_JPRB, 0.11794E+01_JPRB, 0.12894E+01_JPRB, 0.11265E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.95593E+00_JPRB, 0.94454E+00_JPRB, 0.96985E+00_JPRB, 0.90572E+00_JPRB, 0.90772E+00_JPRB, 0.86995E+00_JPRB ,&
&  0.78821E+00_JPRB, 0.73770E+00_JPRB, 0.70002E+00_JPRB, 0.67952E+00_JPRB, 0.66287E+00_JPRB, 0.65424E+00_JPRB, 0.65524E+00_JPRB ,&
&  0.67252E+00_JPRB, 0.66591E+00_JPRB, 0.60259E+00_JPRB, 0.51351E+00_JPRB, 0.47788E+00_JPRB, 0.46234E+00_JPRB, 0.45092E+00_JPRB ,&
&  0.43835E+00_JPRB, 0.43201E+00_JPRB, 0.43485E+00_JPRB, 0.43744E+00_JPRB, 0.43058E+00_JPRB, 0.41388E+00_JPRB, 0.39857E+00_JPRB ,&
&  0.38901E+00_JPRB, 0.38016E+00_JPRB, 0.37199E+00_JPRB, 0.38797E+00_JPRB, 0.40829E+00_JPRB, 0.42351E+00_JPRB, 0.42188E+00_JPRB ,&
&  0.42044E+00_JPRB, 0.42948E+00_JPRB, 0.44042E+00_JPRB, 0.46622E+00_JPRB, 0.48938E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.44489E+00_JPRB, 0.46673E+00_JPRB, 0.49043E+00_JPRB, 0.48821E+00_JPRB, 0.48817E+00_JPRB, 0.46083E+00_JPRB, 0.43053E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.39892E+00_JPRB, 0.39513E+00_JPRB, 0.38853E+00_JPRB, 0.38406E+00_JPRB, 0.38295E+00_JPRB, 0.38344E+00_JPRB ,&
&  0.38514E+00_JPRB, 0.38692E+00_JPRB, 0.38878E+00_JPRB, 0.39074E+00_JPRB, 0.39604E+00_JPRB, 0.40218E+00_JPRB, 0.40244E+00_JPRB ,&
&  0.39898E+00_JPRB, 0.39758E+00_JPRB, 0.38888E+00_JPRB, 0.37642E+00_JPRB, 0.37149E+00_JPRB, 0.37580E+00_JPRB, 0.38486E+00_JPRB ,&
&  0.38293E+00_JPRB, 0.37505E+00_JPRB, 0.38064E+00_JPRB, 0.39463E+00_JPRB, 0.38820E+00_JPRB, 0.39509E+00_JPRB, 0.35762E+00_JPRB ,&
&  0.31344E+00_JPRB, 0.28367E+00_JPRB, 0.27267E+00_JPRB, 0.29570E+00_JPRB, 0.34760E+00_JPRB, 0.43050E+00_JPRB, 0.46774E+00_JPRB ,&
&  0.50309E+00_JPRB, 0.54922E+00_JPRB, 0.60506E+00_JPRB, 0.68063E+00_JPRB, 0.78110E+00_JPRB, 0.94588E+00_JPRB, 0.10858E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.12147E+01_JPRB, 0.12033E+01_JPRB, 0.96857E+00_JPRB, 0.86272E+00_JPRB, 0.82848E+00_JPRB, 0.88830E+00_JPRB ,&
&  0.92699E+00_JPRB, 0.82963E+00_JPRB, 0.76510E+00_JPRB, 0.75122E+00_JPRB, 0.73038E+00_JPRB, 0.73182E+00_JPRB, 0.73224E+00_JPRB ,&
&  0.71580E+00_JPRB, 0.68568E+00_JPRB, 0.65291E+00_JPRB, 0.62937E+00_JPRB, 0.62032E+00_JPRB, 0.62396E+00_JPRB, 0.62626E+00_JPRB ,&
&  0.64668E+00_JPRB, 0.69265E+00_JPRB, 0.65623E+00_JPRB, 0.51638E+00_JPRB, 0.48680E+00_JPRB, 0.47642E+00_JPRB, 0.46530E+00_JPRB ,&
&  0.44655E+00_JPRB, 0.43511E+00_JPRB, 0.43433E+00_JPRB, 0.43562E+00_JPRB, 0.42824E+00_JPRB, 0.42074E+00_JPRB, 0.41650E+00_JPRB ,&
&  0.41684E+00_JPRB, 0.41933E+00_JPRB, 0.42305E+00_JPRB, 0.44048E+00_JPRB, 0.44782E+00_JPRB, 0.44813E+00_JPRB, 0.43860E+00_JPRB ,&
&  0.43101E+00_JPRB, 0.42981E+00_JPRB, 0.42903E+00_JPRB, 0.42686E+00_JPRB, 0.43232E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.45603E+00_JPRB, 0.46203E+00_JPRB, 0.47236E+00_JPRB, 0.47982E+00_JPRB, 0.48916E+00_JPRB, 0.48177E+00_JPRB, 0.47353E+00_JPRB ,&
&  0.47063E+00_JPRB, 0.46836E+00_JPRB, 0.45663E+00_JPRB, 0.44412E+00_JPRB, 0.42822E+00_JPRB, 0.41654E+00_JPRB, 0.41371E+00_JPRB ,&
&  0.41549E+00_JPRB, 0.41646E+00_JPRB, 0.41711E+00_JPRB, 0.41463E+00_JPRB, 0.41412E+00_JPRB, 0.41644E+00_JPRB, 0.41295E+00_JPRB ,&
&  0.40655E+00_JPRB, 0.40086E+00_JPRB, 0.39336E+00_JPRB, 0.38558E+00_JPRB, 0.37466E+00_JPRB, 0.37156E+00_JPRB, 0.36952E+00_JPRB ,&
&  0.36700E+00_JPRB, 0.36854E+00_JPRB, 0.37788E+00_JPRB, 0.37331E+00_JPRB, 0.37689E+00_JPRB, 0.38453E+00_JPRB, 0.34237E+00_JPRB ,&
&  0.30117E+00_JPRB, 0.27369E+00_JPRB, 0.25925E+00_JPRB, 0.25993E+00_JPRB, 0.27698E+00_JPRB, 0.31483E+00_JPRB, 0.36791E+00_JPRB ,&
&  0.43478E+00_JPRB, 0.47902E+00_JPRB, 0.52013E+00_JPRB, 0.56056E+00_JPRB, 0.60629E+00_JPRB, 0.70376E+00_JPRB, 0.78375E+00_JPRB ,&
&  0.86356E+00_JPRB, 0.95188E+00_JPRB, 0.10087E+01_JPRB, 0.10189E+01_JPRB, 0.98517E+00_JPRB, 0.86291E+00_JPRB, 0.85758E+00_JPRB ,&
&  0.10000E+01_JPRB, 0.82414E+00_JPRB, 0.67695E+00_JPRB, 0.61149E+00_JPRB, 0.59510E+00_JPRB, 0.59556E+00_JPRB, 0.59692E+00_JPRB ,&
&  0.60153E+00_JPRB, 0.60197E+00_JPRB, 0.59501E+00_JPRB, 0.58621E+00_JPRB, 0.58197E+00_JPRB, 0.57975E+00_JPRB, 0.58445E+00_JPRB ,&
&  0.59640E+00_JPRB, 0.68095E+00_JPRB, 0.85145E+00_JPRB, 0.64601E+00_JPRB, 0.56251E+00_JPRB, 0.51409E+00_JPRB, 0.47073E+00_JPRB ,&
&  0.45251E+00_JPRB, 0.44612E+00_JPRB, 0.44819E+00_JPRB, 0.44947E+00_JPRB, 0.44792E+00_JPRB, 0.44981E+00_JPRB, 0.45511E+00_JPRB ,&
&  0.46566E+00_JPRB, 0.47368E+00_JPRB, 0.47534E+00_JPRB, 0.47478E+00_JPRB, 0.46734E+00_JPRB, 0.45554E+00_JPRB, 0.44723E+00_JPRB ,&
&  0.44749E+00_JPRB, 0.44958E+00_JPRB, 0.45249E+00_JPRB, 0.45455E+00_JPRB, 0.45514E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.46000E+00_JPRB, 0.46151E+00_JPRB, 0.46733E+00_JPRB, 0.47220E+00_JPRB, 0.47970E+00_JPRB, 0.48553E+00_JPRB, 0.48784E+00_JPRB ,&
&  0.49538E+00_JPRB, 0.49183E+00_JPRB, 0.47872E+00_JPRB, 0.46267E+00_JPRB, 0.44654E+00_JPRB, 0.43663E+00_JPRB, 0.43266E+00_JPRB ,&
&  0.43475E+00_JPRB, 0.43247E+00_JPRB, 0.43132E+00_JPRB, 0.43226E+00_JPRB, 0.43431E+00_JPRB, 0.43700E+00_JPRB, 0.43240E+00_JPRB ,&
&  0.41732E+00_JPRB, 0.40652E+00_JPRB, 0.39433E+00_JPRB, 0.37946E+00_JPRB, 0.36533E+00_JPRB, 0.35614E+00_JPRB, 0.35996E+00_JPRB ,&
&  0.36770E+00_JPRB, 0.36333E+00_JPRB, 0.36187E+00_JPRB, 0.36312E+00_JPRB, 0.36677E+00_JPRB, 0.36373E+00_JPRB, 0.30088E+00_JPRB ,&
&  0.27692E+00_JPRB, 0.26147E+00_JPRB, 0.25582E+00_JPRB, 0.25620E+00_JPRB, 0.26930E+00_JPRB, 0.29413E+00_JPRB, 0.33918E+00_JPRB ,&
&  0.38548E+00_JPRB, 0.43064E+00_JPRB, 0.46073E+00_JPRB, 0.49451E+00_JPRB, 0.54471E+00_JPRB, 0.60704E+00_JPRB, 0.65962E+00_JPRB ,&
&  0.70982E+00_JPRB, 0.77733E+00_JPRB, 0.82499E+00_JPRB, 0.85185E+00_JPRB, 0.83894E+00_JPRB, 0.79992E+00_JPRB, 0.76297E+00_JPRB ,&
&  0.73192E+00_JPRB, 0.65450E+00_JPRB, 0.61948E+00_JPRB, 0.59420E+00_JPRB, 0.57812E+00_JPRB, 0.56960E+00_JPRB, 0.55348E+00_JPRB ,&
&  0.54596E+00_JPRB, 0.54753E+00_JPRB, 0.54390E+00_JPRB, 0.54271E+00_JPRB, 0.54075E+00_JPRB, 0.53608E+00_JPRB, 0.53111E+00_JPRB ,&
&  0.53024E+00_JPRB, 0.53513E+00_JPRB, 0.55264E+00_JPRB, 0.53163E+00_JPRB, 0.50778E+00_JPRB, 0.48461E+00_JPRB, 0.47095E+00_JPRB ,&
&  0.46632E+00_JPRB, 0.46709E+00_JPRB, 0.47088E+00_JPRB, 0.47031E+00_JPRB, 0.47149E+00_JPRB, 0.47391E+00_JPRB, 0.48357E+00_JPRB ,&
&  0.48643E+00_JPRB, 0.48862E+00_JPRB, 0.48677E+00_JPRB, 0.48142E+00_JPRB, 0.47197E+00_JPRB, 0.46117E+00_JPRB, 0.45348E+00_JPRB ,&
&  0.45082E+00_JPRB, 0.45031E+00_JPRB, 0.45049E+00_JPRB, 0.45327E+00_JPRB, 0.45723E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.45571E+00_JPRB, 0.45650E+00_JPRB, 0.46099E+00_JPRB, 0.46641E+00_JPRB, 0.47266E+00_JPRB, 0.47915E+00_JPRB, 0.48218E+00_JPRB ,&
&  0.48403E+00_JPRB, 0.48215E+00_JPRB, 0.47840E+00_JPRB, 0.46730E+00_JPRB, 0.45938E+00_JPRB, 0.45588E+00_JPRB, 0.45190E+00_JPRB ,&
&  0.44747E+00_JPRB, 0.44550E+00_JPRB, 0.44489E+00_JPRB, 0.44600E+00_JPRB, 0.44466E+00_JPRB, 0.44587E+00_JPRB, 0.44120E+00_JPRB ,&
&  0.42793E+00_JPRB, 0.41712E+00_JPRB, 0.39768E+00_JPRB, 0.38015E+00_JPRB, 0.36793E+00_JPRB, 0.35966E+00_JPRB, 0.36146E+00_JPRB ,&
&  0.35944E+00_JPRB, 0.35870E+00_JPRB, 0.35909E+00_JPRB, 0.36051E+00_JPRB, 0.36487E+00_JPRB, 0.29992E+00_JPRB, 0.28138E+00_JPRB ,&
&  0.26806E+00_JPRB, 0.25946E+00_JPRB, 0.25752E+00_JPRB, 0.25823E+00_JPRB, 0.26473E+00_JPRB, 0.28359E+00_JPRB, 0.31289E+00_JPRB ,&
&  0.33851E+00_JPRB, 0.41126E+00_JPRB, 0.42910E+00_JPRB, 0.45134E+00_JPRB, 0.48831E+00_JPRB, 0.52878E+00_JPRB, 0.56439E+00_JPRB ,&
&  0.59472E+00_JPRB, 0.63311E+00_JPRB, 0.66818E+00_JPRB, 0.69517E+00_JPRB, 0.69255E+00_JPRB, 0.68752E+00_JPRB, 0.67088E+00_JPRB ,&
&  0.65428E+00_JPRB, 0.62785E+00_JPRB, 0.61042E+00_JPRB, 0.59452E+00_JPRB, 0.58153E+00_JPRB, 0.56824E+00_JPRB, 0.55668E+00_JPRB ,&
&  0.54908E+00_JPRB, 0.53802E+00_JPRB, 0.53426E+00_JPRB, 0.52749E+00_JPRB, 0.52023E+00_JPRB, 0.51540E+00_JPRB, 0.51036E+00_JPRB ,&
&  0.50461E+00_JPRB, 0.49837E+00_JPRB, 0.48903E+00_JPRB, 0.47795E+00_JPRB, 0.47104E+00_JPRB, 0.46460E+00_JPRB, 0.46308E+00_JPRB ,&
&  0.46395E+00_JPRB, 0.47072E+00_JPRB, 0.47329E+00_JPRB, 0.47578E+00_JPRB, 0.47844E+00_JPRB, 0.47745E+00_JPRB, 0.47806E+00_JPRB ,&
&  0.47891E+00_JPRB, 0.48084E+00_JPRB, 0.47896E+00_JPRB, 0.47423E+00_JPRB, 0.46613E+00_JPRB, 0.45743E+00_JPRB, 0.45096E+00_JPRB ,&
&  0.44833E+00_JPRB, 0.44853E+00_JPRB, 0.44642E+00_JPRB, 0.44828E+00_JPRB, 0.45173E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.44980E+00_JPRB, 0.44882E+00_JPRB, 0.44907E+00_JPRB, 0.45302E+00_JPRB, 0.46062E+00_JPRB, 0.46927E+00_JPRB, 0.46900E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.46662E+00_JPRB, 0.46746E+00_JPRB, 0.46630E+00_JPRB, 0.46428E+00_JPRB, 0.46060E+00_JPRB, 0.45736E+00_JPRB ,&
&  0.45459E+00_JPRB, 0.44878E+00_JPRB, 0.44162E+00_JPRB, 0.43969E+00_JPRB, 0.44078E+00_JPRB, 0.44227E+00_JPRB, 0.44227E+00_JPRB ,&
&  0.43959E+00_JPRB, 0.43133E+00_JPRB, 0.40763E+00_JPRB, 0.38943E+00_JPRB, 0.37244E+00_JPRB, 0.36763E+00_JPRB, 0.35996E+00_JPRB ,&
&  0.37036E+00_JPRB, 0.33608E+00_JPRB, 0.33271E+00_JPRB, 0.31638E+00_JPRB, 0.30188E+00_JPRB, 0.28684E+00_JPRB, 0.27569E+00_JPRB ,&
&  0.27155E+00_JPRB, 0.26776E+00_JPRB, 0.26776E+00_JPRB, 0.27325E+00_JPRB, 0.28155E+00_JPRB, 0.29763E+00_JPRB, 0.31776E+00_JPRB ,&
&  0.33753E+00_JPRB, 0.39025E+00_JPRB, 0.40735E+00_JPRB, 0.42023E+00_JPRB, 0.44208E+00_JPRB, 0.46375E+00_JPRB, 0.48438E+00_JPRB ,&
&  0.50245E+00_JPRB, 0.52061E+00_JPRB, 0.53966E+00_JPRB, 0.54694E+00_JPRB, 0.55700E+00_JPRB, 0.55309E+00_JPRB, 0.55110E+00_JPRB ,&
&  0.55049E+00_JPRB, 0.54815E+00_JPRB, 0.54337E+00_JPRB, 0.53644E+00_JPRB, 0.52920E+00_JPRB, 0.52007E+00_JPRB, 0.51355E+00_JPRB ,&
&  0.50541E+00_JPRB, 0.49790E+00_JPRB, 0.48896E+00_JPRB, 0.48133E+00_JPRB, 0.47311E+00_JPRB, 0.46809E+00_JPRB, 0.46263E+00_JPRB ,&
&  0.45821E+00_JPRB, 0.45342E+00_JPRB, 0.44843E+00_JPRB, 0.44554E+00_JPRB, 0.44531E+00_JPRB, 0.44459E+00_JPRB, 0.44661E+00_JPRB ,&
&  0.45241E+00_JPRB, 0.45684E+00_JPRB, 0.46183E+00_JPRB, 0.46616E+00_JPRB, 0.46953E+00_JPRB, 0.46934E+00_JPRB, 0.46748E+00_JPRB ,&
&  0.46735E+00_JPRB, 0.46770E+00_JPRB, 0.46625E+00_JPRB, 0.46141E+00_JPRB, 0.45522E+00_JPRB, 0.44918E+00_JPRB, 0.44581E+00_JPRB ,&
&  0.44326E+00_JPRB, 0.44306E+00_JPRB, 0.44088E+00_JPRB, 0.44356E+00_JPRB, 0.44657E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.43845E+00_JPRB, 0.43859E+00_JPRB, 0.43828E+00_JPRB, 0.44011E+00_JPRB, 0.44287E+00_JPRB, 0.44749E+00_JPRB, 0.44871E+00_JPRB ,&
&  0.45289E+00_JPRB, 0.45656E+00_JPRB, 0.46033E+00_JPRB, 0.46145E+00_JPRB, 0.46193E+00_JPRB, 0.46052E+00_JPRB, 0.45775E+00_JPRB ,&
&  0.45492E+00_JPRB, 0.45099E+00_JPRB, 0.44783E+00_JPRB, 0.44431E+00_JPRB, 0.44085E+00_JPRB, 0.43919E+00_JPRB, 0.43830E+00_JPRB ,&
&  0.43827E+00_JPRB, 0.43760E+00_JPRB, 0.40657E+00_JPRB, 0.39432E+00_JPRB, 0.37760E+00_JPRB, 0.36901E+00_JPRB, 0.36523E+00_JPRB ,&
&  0.36134E+00_JPRB, 0.35736E+00_JPRB, 0.35245E+00_JPRB, 0.34300E+00_JPRB, 0.32978E+00_JPRB, 0.31216E+00_JPRB, 0.30230E+00_JPRB ,&
&  0.29396E+00_JPRB, 0.29158E+00_JPRB, 0.29295E+00_JPRB, 0.29864E+00_JPRB, 0.30917E+00_JPRB, 0.32157E+00_JPRB, 0.33652E+00_JPRB ,&
&  0.35696E+00_JPRB, 0.37975E+00_JPRB, 0.39501E+00_JPRB, 0.39907E+00_JPRB, 0.41477E+00_JPRB, 0.42530E+00_JPRB, 0.43417E+00_JPRB ,&
&  0.44132E+00_JPRB, 0.44736E+00_JPRB, 0.45864E+00_JPRB, 0.46419E+00_JPRB, 0.46557E+00_JPRB, 0.46738E+00_JPRB, 0.47087E+00_JPRB ,&
&  0.47276E+00_JPRB, 0.47349E+00_JPRB, 0.47142E+00_JPRB, 0.46895E+00_JPRB, 0.46544E+00_JPRB, 0.46102E+00_JPRB, 0.45700E+00_JPRB ,&
&  0.45303E+00_JPRB, 0.44934E+00_JPRB, 0.44413E+00_JPRB, 0.43869E+00_JPRB, 0.43483E+00_JPRB, 0.43151E+00_JPRB, 0.42837E+00_JPRB ,&
&  0.42627E+00_JPRB, 0.42476E+00_JPRB, 0.42361E+00_JPRB, 0.42019E+00_JPRB, 0.42059E+00_JPRB, 0.42564E+00_JPRB, 0.42772E+00_JPRB ,&
&  0.43259E+00_JPRB, 0.43665E+00_JPRB, 0.44147E+00_JPRB, 0.44535E+00_JPRB, 0.44837E+00_JPRB, 0.44979E+00_JPRB, 0.44935E+00_JPRB ,&
&  0.44938E+00_JPRB, 0.44902E+00_JPRB, 0.44786E+00_JPRB, 0.44397E+00_JPRB, 0.44002E+00_JPRB, 0.43617E+00_JPRB, 0.43448E+00_JPRB ,&
&  0.43237E+00_JPRB, 0.43171E+00_JPRB, 0.43027E+00_JPRB, 0.43300E+00_JPRB, 0.43549E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.43040E+00_JPRB, 0.43182E+00_JPRB, 0.43302E+00_JPRB, 0.43458E+00_JPRB, 0.43710E+00_JPRB, 0.44017E+00_JPRB, 0.44213E+00_JPRB ,&
&  0.44491E+00_JPRB, 0.44795E+00_JPRB, 0.45153E+00_JPRB, 0.45342E+00_JPRB, 0.45462E+00_JPRB, 0.45467E+00_JPRB, 0.45404E+00_JPRB ,&
&  0.45245E+00_JPRB, 0.45086E+00_JPRB, 0.44918E+00_JPRB, 0.44794E+00_JPRB, 0.44674E+00_JPRB, 0.44558E+00_JPRB, 0.44343E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.44071E+00_JPRB, 0.43717E+00_JPRB, 0.43205E+00_JPRB, 0.42774E+00_JPRB, 0.39376E+00_JPRB, 0.38215E+00_JPRB ,&
&  0.37915E+00_JPRB, 0.37618E+00_JPRB, 0.37721E+00_JPRB, 0.37914E+00_JPRB, 0.38769E+00_JPRB, 0.38479E+00_JPRB, 0.38298E+00_JPRB ,&
&  0.36368E+00_JPRB, 0.36146E+00_JPRB, 0.35773E+00_JPRB, 0.35627E+00_JPRB, 0.35920E+00_JPRB, 0.36266E+00_JPRB, 0.37144E+00_JPRB ,&
&  0.39217E+00_JPRB, 0.39899E+00_JPRB, 0.40234E+00_JPRB, 0.40599E+00_JPRB, 0.41080E+00_JPRB, 0.41469E+00_JPRB, 0.41796E+00_JPRB ,&
&  0.42063E+00_JPRB, 0.42285E+00_JPRB, 0.42462E+00_JPRB, 0.42470E+00_JPRB, 0.42474E+00_JPRB, 0.42497E+00_JPRB, 0.42497E+00_JPRB ,&
&  0.42444E+00_JPRB, 0.42457E+00_JPRB, 0.42521E+00_JPRB, 0.42570E+00_JPRB, 0.42625E+00_JPRB, 0.42593E+00_JPRB, 0.42535E+00_JPRB ,&
&  0.42462E+00_JPRB, 0.42359E+00_JPRB, 0.42238E+00_JPRB, 0.42116E+00_JPRB, 0.42077E+00_JPRB, 0.41956E+00_JPRB, 0.41882E+00_JPRB ,&
&  0.41790E+00_JPRB, 0.41768E+00_JPRB, 0.41724E+00_JPRB, 0.41812E+00_JPRB, 0.41860E+00_JPRB, 0.42049E+00_JPRB, 0.42219E+00_JPRB ,&
&  0.42507E+00_JPRB, 0.42724E+00_JPRB, 0.42924E+00_JPRB, 0.43039E+00_JPRB, 0.43155E+00_JPRB, 0.43233E+00_JPRB, 0.43213E+00_JPRB ,&
&  0.43177E+00_JPRB, 0.43090E+00_JPRB, 0.42957E+00_JPRB, 0.42741E+00_JPRB, 0.42556E+00_JPRB, 0.42393E+00_JPRB, 0.42301E+00_JPRB ,&
&  0.42195E+00_JPRB, 0.42237E+00_JPRB, 0.42394E+00_JPRB, 0.42632E+00_JPRB, 0.42850E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.43555E+00_JPRB, 0.43697E+00_JPRB, 0.43764E+00_JPRB, 0.43815E+00_JPRB, 0.43861E+00_JPRB, 0.43986E+00_JPRB, 0.44097E+00_JPRB ,&
&  0.44213E+00_JPRB, 0.44353E+00_JPRB, 0.44557E+00_JPRB, 0.44675E+00_JPRB, 0.44766E+00_JPRB, 0.44808E+00_JPRB, 0.44859E+00_JPRB ,&
&  0.44870E+00_JPRB, 0.44848E+00_JPRB, 0.44779E+00_JPRB, 0.44740E+00_JPRB, 0.44710E+00_JPRB, 0.44703E+00_JPRB, 0.44675E+00_JPRB ,&
&  0.44583E+00_JPRB, 0.44468E+00_JPRB, 0.44338E+00_JPRB, 0.44128E+00_JPRB, 0.43855E+00_JPRB, 0.43554E+00_JPRB, 0.43209E+00_JPRB ,&
&  0.42857E+00_JPRB, 0.42586E+00_JPRB, 0.42315E+00_JPRB, 0.42059E+00_JPRB, 0.41957E+00_JPRB, 0.41864E+00_JPRB, 0.41766E+00_JPRB ,&
&  0.41671E+00_JPRB, 0.41541E+00_JPRB, 0.41527E+00_JPRB, 0.41556E+00_JPRB, 0.41623E+00_JPRB, 0.41763E+00_JPRB, 0.41893E+00_JPRB ,&
&  0.41990E+00_JPRB, 0.42001E+00_JPRB, 0.41954E+00_JPRB, 0.41974E+00_JPRB, 0.42034E+00_JPRB, 0.42053E+00_JPRB, 0.42002E+00_JPRB ,&
&  0.41967E+00_JPRB, 0.41880E+00_JPRB, 0.41765E+00_JPRB, 0.41638E+00_JPRB, 0.41582E+00_JPRB, 0.41504E+00_JPRB, 0.41408E+00_JPRB ,&
&  0.41339E+00_JPRB, 0.41300E+00_JPRB, 0.41293E+00_JPRB, 0.41297E+00_JPRB, 0.41354E+00_JPRB, 0.41412E+00_JPRB, 0.41469E+00_JPRB ,&
&  0.41525E+00_JPRB, 0.41653E+00_JPRB, 0.41775E+00_JPRB, 0.41877E+00_JPRB, 0.41984E+00_JPRB, 0.42081E+00_JPRB, 0.42178E+00_JPRB ,&
&  0.42246E+00_JPRB, 0.42283E+00_JPRB, 0.42242E+00_JPRB, 0.42253E+00_JPRB, 0.42275E+00_JPRB, 0.42353E+00_JPRB, 0.42399E+00_JPRB ,&
&  0.42513E+00_JPRB, 0.42581E+00_JPRB, 0.42628E+00_JPRB, 0.42688E+00_JPRB, 0.42734E+00_JPRB, 0.42755E+00_JPRB, 0.42740E+00_JPRB ,&
&  0.42711E+00_JPRB, 0.42710E+00_JPRB, 0.42712E+00_JPRB, 0.42712E+00_JPRB, 0.42747E+00_JPRB, 0.42793E+00_JPRB, 0.42845E+00_JPRB ,&
&  0.42919E+00_JPRB, 0.43056E+00_JPRB, 0.43214E+00_JPRB, 0.43334E+00_JPRB, 0.43435E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.43897E+00_JPRB, 0.43909E+00_JPRB, 0.43910E+00_JPRB, 0.43898E+00_JPRB, 0.43895E+00_JPRB, 0.43915E+00_JPRB, 0.43927E+00_JPRB ,&
&  0.43946E+00_JPRB, 0.43967E+00_JPRB, 0.44020E+00_JPRB, 0.44048E+00_JPRB, 0.44080E+00_JPRB, 0.44097E+00_JPRB, 0.44127E+00_JPRB ,&
&  0.44150E+00_JPRB, 0.44172E+00_JPRB, 0.44187E+00_JPRB, 0.44213E+00_JPRB, 0.44242E+00_JPRB, 0.44286E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44327E+00_JPRB, 0.44298E+00_JPRB, 0.44282E+00_JPRB, 0.44249E+00_JPRB, 0.44209E+00_JPRB, 0.44123E+00_JPRB, 0.44018E+00_JPRB ,&
&  0.43891E+00_JPRB, 0.43776E+00_JPRB, 0.43683E+00_JPRB, 0.43600E+00_JPRB, 0.43542E+00_JPRB, 0.43497E+00_JPRB, 0.43462E+00_JPRB ,&
&  0.43411E+00_JPRB, 0.43376E+00_JPRB, 0.43344E+00_JPRB, 0.43319E+00_JPRB, 0.43268E+00_JPRB, 0.43208E+00_JPRB, 0.43141E+00_JPRB ,&
&  0.43067E+00_JPRB, 0.42978E+00_JPRB, 0.42903E+00_JPRB, 0.42834E+00_JPRB, 0.42753E+00_JPRB, 0.42632E+00_JPRB, 0.42500E+00_JPRB ,&
&  0.42366E+00_JPRB, 0.42222E+00_JPRB, 0.42072E+00_JPRB, 0.41936E+00_JPRB, 0.41840E+00_JPRB, 0.41755E+00_JPRB, 0.41666E+00_JPRB ,&
&  0.41636E+00_JPRB, 0.41662E+00_JPRB, 0.41693E+00_JPRB, 0.41709E+00_JPRB, 0.41764E+00_JPRB, 0.41815E+00_JPRB, 0.41877E+00_JPRB ,&
&  0.41930E+00_JPRB, 0.42012E+00_JPRB, 0.42115E+00_JPRB, 0.42208E+00_JPRB, 0.42313E+00_JPRB, 0.42423E+00_JPRB, 0.42550E+00_JPRB ,&
&  0.42673E+00_JPRB, 0.42802E+00_JPRB, 0.42935E+00_JPRB, 0.43065E+00_JPRB, 0.43138E+00_JPRB, 0.43195E+00_JPRB, 0.43211E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.43255E+00_JPRB, 0.43289E+00_JPRB, 0.43320E+00_JPRB, 0.43363E+00_JPRB, 0.43398E+00_JPRB, 0.43437E+00_JPRB ,&
&  0.43484E+00_JPRB, 0.43546E+00_JPRB, 0.43594E+00_JPRB, 0.43639E+00_JPRB, 0.43677E+00_JPRB, 0.43728E+00_JPRB, 0.43768E+00_JPRB ,&
&  0.43801E+00_JPRB, 0.43842E+00_JPRB, 0.43880E+00_JPRB, 0.43894E+00_JPRB, 0.43896E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB, 0.19015E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.19358E+00_JPRB, 0.19388E+00_JPRB, 0.19400E+00_JPRB, 0.19404E+00_JPRB, 0.19272E+00_JPRB, 0.19229E+00_JPRB, 0.19184E+00_JPRB ,&
&  0.19151E+00_JPRB, 0.19134E+00_JPRB, 0.19171E+00_JPRB, 0.19218E+00_JPRB, 0.19265E+00_JPRB, 0.19311E+00_JPRB, 0.19346E+00_JPRB ,&
&  0.19361E+00_JPRB, 0.19376E+00_JPRB, 0.19284E+00_JPRB, 0.19241E+00_JPRB, 0.19231E+00_JPRB, 0.19239E+00_JPRB, 0.19240E+00_JPRB ,&
&  0.19253E+00_JPRB, 0.19263E+00_JPRB, 0.19264E+00_JPRB, 0.19293E+00_JPRB, 0.19346E+00_JPRB, 0.19390E+00_JPRB, 0.19436E+00_JPRB ,&
&  0.19480E+00_JPRB, 0.19542E+00_JPRB, 0.19598E+00_JPRB, 0.19650E+00_JPRB, 0.19713E+00_JPRB, 0.19770E+00_JPRB, 0.19799E+00_JPRB ,&
&  0.19818E+00_JPRB, 0.19816E+00_JPRB, 0.19820E+00_JPRB, 0.19823E+00_JPRB, 0.19837E+00_JPRB, 0.19856E+00_JPRB, 0.19857E+00_JPRB ,&
&  0.19845E+00_JPRB, 0.19827E+00_JPRB, 0.19802E+00_JPRB, 0.19751E+00_JPRB, 0.19698E+00_JPRB, 0.19647E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19555E+00_JPRB, 0.19507E+00_JPRB, 0.19454E+00_JPRB, 0.19398E+00_JPRB, 0.19339E+00_JPRB, 0.19282E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19166E+00_JPRB, 0.19128E+00_JPRB, 0.19086E+00_JPRB, 0.19051E+00_JPRB, 0.19024E+00_JPRB, 0.18997E+00_JPRB, 0.18972E+00_JPRB ,&
&  0.18946E+00_JPRB, 0.18914E+00_JPRB, 0.18886E+00_JPRB, 0.18853E+00_JPRB, 0.18825E+00_JPRB, 0.18793E+00_JPRB, 0.18768E+00_JPRB ,&
&  0.18743E+00_JPRB, 0.18722E+00_JPRB, 0.18704E+00_JPRB, 0.18696E+00_JPRB, 0.18692E+00_JPRB, 0.18696E+00_JPRB, 0.18701E+00_JPRB ,&
&  0.18707E+00_JPRB, 0.18712E+00_JPRB, 0.18716E+00_JPRB, 0.18708E+00_JPRB, 0.18726E+00_JPRB, 0.18745E+00_JPRB, 0.18772E+00_JPRB ,&
&  0.18798E+00_JPRB, 0.18838E+00_JPRB, 0.18878E+00_JPRB, 0.18923E+00_JPRB, 0.18990E+00_JPRB, 0.19014E+00_JPRB, 0.19036E+00_JPRB ,&
&  0.19061E+00_JPRB, 0.19110E+00_JPRB, 0.19155E+00_JPRB, 0.19179E+00_JPRB, 0.19232E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.20633E+00_JPRB, 0.21135E+00_JPRB, 0.21532E+00_JPRB, 0.21925E+00_JPRB, 0.21871E+00_JPRB, 0.23798E+00_JPRB, 0.23591E+00_JPRB ,&
&  0.23542E+00_JPRB, 0.22824E+00_JPRB, 0.22889E+00_JPRB, 0.22740E+00_JPRB, 0.22497E+00_JPRB, 0.22192E+00_JPRB, 0.21866E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21142E+00_JPRB, 0.20778E+00_JPRB, 0.20431E+00_JPRB, 0.20145E+00_JPRB, 0.19897E+00_JPRB, 0.19758E+00_JPRB ,&
&  0.19756E+00_JPRB, 0.19892E+00_JPRB, 0.20130E+00_JPRB, 0.20277E+00_JPRB, 0.20432E+00_JPRB, 0.20593E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20912E+00_JPRB, 0.21102E+00_JPRB, 0.21205E+00_JPRB, 0.21177E+00_JPRB, 0.21056E+00_JPRB, 0.20985E+00_JPRB, 0.20946E+00_JPRB ,&
&  0.20966E+00_JPRB, 0.20964E+00_JPRB, 0.20952E+00_JPRB, 0.20958E+00_JPRB, 0.20960E+00_JPRB, 0.20897E+00_JPRB, 0.20776E+00_JPRB ,&
&  0.20675E+00_JPRB, 0.20594E+00_JPRB, 0.20447E+00_JPRB, 0.20306E+00_JPRB, 0.20174E+00_JPRB, 0.20037E+00_JPRB, 0.19896E+00_JPRB ,&
&  0.19793E+00_JPRB, 0.19690E+00_JPRB, 0.19561E+00_JPRB, 0.19412E+00_JPRB, 0.19269E+00_JPRB, 0.19142E+00_JPRB, 0.19040E+00_JPRB ,&
&  0.19013E+00_JPRB, 0.18957E+00_JPRB, 0.18909E+00_JPRB, 0.18862E+00_JPRB, 0.18848E+00_JPRB, 0.18810E+00_JPRB, 0.18785E+00_JPRB ,&
&  0.18766E+00_JPRB, 0.18743E+00_JPRB, 0.18720E+00_JPRB, 0.18696E+00_JPRB, 0.18681E+00_JPRB, 0.18658E+00_JPRB, 0.18637E+00_JPRB ,&
&  0.18625E+00_JPRB, 0.18616E+00_JPRB, 0.18603E+00_JPRB, 0.18574E+00_JPRB, 0.18542E+00_JPRB, 0.18504E+00_JPRB, 0.18506E+00_JPRB ,&
&  0.18522E+00_JPRB, 0.18551E+00_JPRB, 0.18600E+00_JPRB, 0.18624E+00_JPRB, 0.18701E+00_JPRB, 0.18777E+00_JPRB, 0.18865E+00_JPRB ,&
&  0.18959E+00_JPRB, 0.19101E+00_JPRB, 0.19255E+00_JPRB, 0.19421E+00_JPRB, 0.19676E+00_JPRB, 0.19781E+00_JPRB, 0.19874E+00_JPRB ,&
&  0.19984E+00_JPRB, 0.20302E+00_JPRB, 0.20428E+00_JPRB, 0.20454E+00_JPRB, 0.20555E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.24875E+00_JPRB, 0.25163E+00_JPRB, 0.25362E+00_JPRB, 0.25495E+00_JPRB, 0.25620E+00_JPRB, 0.25554E+00_JPRB, 0.25410E+00_JPRB ,&
&  0.25014E+00_JPRB, 0.24872E+00_JPRB, 0.24604E+00_JPRB, 0.24328E+00_JPRB, 0.24003E+00_JPRB, 0.23668E+00_JPRB, 0.23269E+00_JPRB ,&
&  0.22830E+00_JPRB, 0.22357E+00_JPRB, 0.21783E+00_JPRB, 0.21172E+00_JPRB, 0.20614E+00_JPRB, 0.20093E+00_JPRB, 0.19763E+00_JPRB ,&
&  0.19607E+00_JPRB, 0.19666E+00_JPRB, 0.19923E+00_JPRB, 0.20258E+00_JPRB, 0.20666E+00_JPRB, 0.21130E+00_JPRB, 0.21708E+00_JPRB ,&
&  0.22211E+00_JPRB, 0.22821E+00_JPRB, 0.23193E+00_JPRB, 0.23358E+00_JPRB, 0.23219E+00_JPRB, 0.23199E+00_JPRB, 0.23096E+00_JPRB ,&
&  0.22957E+00_JPRB, 0.22729E+00_JPRB, 0.22613E+00_JPRB, 0.22584E+00_JPRB, 0.22465E+00_JPRB, 0.22245E+00_JPRB, 0.21966E+00_JPRB ,&
&  0.21717E+00_JPRB, 0.21503E+00_JPRB, 0.21254E+00_JPRB, 0.21041E+00_JPRB, 0.20879E+00_JPRB, 0.20687E+00_JPRB, 0.20442E+00_JPRB ,&
&  0.20217E+00_JPRB, 0.19990E+00_JPRB, 0.19721E+00_JPRB, 0.19486E+00_JPRB, 0.19254E+00_JPRB, 0.19085E+00_JPRB, 0.18933E+00_JPRB ,&
&  0.18864E+00_JPRB, 0.18794E+00_JPRB, 0.18770E+00_JPRB, 0.18757E+00_JPRB, 0.18793E+00_JPRB, 0.18770E+00_JPRB, 0.18736E+00_JPRB ,&
&  0.18693E+00_JPRB, 0.18682E+00_JPRB, 0.18636E+00_JPRB, 0.18620E+00_JPRB, 0.18605E+00_JPRB, 0.18551E+00_JPRB, 0.18527E+00_JPRB ,&
&  0.18543E+00_JPRB, 0.18544E+00_JPRB, 0.18506E+00_JPRB, 0.18509E+00_JPRB, 0.18486E+00_JPRB, 0.18450E+00_JPRB, 0.18478E+00_JPRB ,&
&  0.18466E+00_JPRB, 0.18488E+00_JPRB, 0.18521E+00_JPRB, 0.18572E+00_JPRB, 0.18676E+00_JPRB, 0.18810E+00_JPRB, 0.18990E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.19507E+00_JPRB, 0.19758E+00_JPRB, 0.20055E+00_JPRB, 0.20586E+00_JPRB, 0.20906E+00_JPRB, 0.21448E+00_JPRB ,&
&  0.21981E+00_JPRB, 0.22052E+00_JPRB, 0.24294E+00_JPRB, 0.24393E+00_JPRB, 0.24548E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.26185E+00_JPRB, 0.26381E+00_JPRB, 0.26501E+00_JPRB, 0.26704E+00_JPRB, 0.26932E+00_JPRB, 0.27019E+00_JPRB, 0.27114E+00_JPRB ,&
&  0.26846E+00_JPRB, 0.26159E+00_JPRB, 0.25662E+00_JPRB, 0.25028E+00_JPRB, 0.24384E+00_JPRB, 0.23735E+00_JPRB, 0.23132E+00_JPRB ,&
&  0.22528E+00_JPRB, 0.21987E+00_JPRB, 0.21445E+00_JPRB, 0.20992E+00_JPRB, 0.20686E+00_JPRB, 0.20435E+00_JPRB, 0.20151E+00_JPRB ,&
&  0.19904E+00_JPRB, 0.19983E+00_JPRB, 0.20213E+00_JPRB, 0.20674E+00_JPRB, 0.21406E+00_JPRB, 0.22108E+00_JPRB, 0.22871E+00_JPRB ,&
&  0.24251E+00_JPRB, 0.24676E+00_JPRB, 0.24655E+00_JPRB, 0.24827E+00_JPRB, 0.24754E+00_JPRB, 0.24799E+00_JPRB, 0.23778E+00_JPRB ,&
&  0.23907E+00_JPRB, 0.24105E+00_JPRB, 0.24801E+00_JPRB, 0.25137E+00_JPRB, 0.24665E+00_JPRB, 0.24205E+00_JPRB, 0.23408E+00_JPRB ,&
&  0.22712E+00_JPRB, 0.22180E+00_JPRB, 0.21828E+00_JPRB, 0.21611E+00_JPRB, 0.21211E+00_JPRB, 0.20915E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.19998E+00_JPRB, 0.19705E+00_JPRB, 0.19341E+00_JPRB, 0.19109E+00_JPRB, 0.18923E+00_JPRB, 0.18748E+00_JPRB, 0.18616E+00_JPRB ,&
&  0.18543E+00_JPRB, 0.18484E+00_JPRB, 0.18544E+00_JPRB, 0.18617E+00_JPRB, 0.18754E+00_JPRB, 0.18864E+00_JPRB, 0.18894E+00_JPRB ,&
&  0.18923E+00_JPRB, 0.18958E+00_JPRB, 0.18965E+00_JPRB, 0.19011E+00_JPRB, 0.18961E+00_JPRB, 0.18885E+00_JPRB, 0.18790E+00_JPRB ,&
&  0.18693E+00_JPRB, 0.18661E+00_JPRB, 0.18603E+00_JPRB, 0.18568E+00_JPRB, 0.18431E+00_JPRB, 0.18335E+00_JPRB, 0.18283E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.18270E+00_JPRB, 0.18284E+00_JPRB, 0.18360E+00_JPRB, 0.18480E+00_JPRB, 0.18595E+00_JPRB, 0.18792E+00_JPRB ,&
&  0.19043E+00_JPRB, 0.19333E+00_JPRB, 0.19808E+00_JPRB, 0.20328E+00_JPRB, 0.20902E+00_JPRB, 0.21680E+00_JPRB, 0.22463E+00_JPRB ,&
&  0.23540E+00_JPRB, 0.25173E+00_JPRB, 0.25394E+00_JPRB, 0.25576E+00_JPRB, 0.25959E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.27139E+00_JPRB, 0.27241E+00_JPRB, 0.27390E+00_JPRB, 0.27624E+00_JPRB, 0.27914E+00_JPRB, 0.28031E+00_JPRB, 0.28061E+00_JPRB ,&
&  0.27156E+00_JPRB, 0.26378E+00_JPRB, 0.25703E+00_JPRB, 0.24667E+00_JPRB, 0.23845E+00_JPRB, 0.23098E+00_JPRB, 0.22554E+00_JPRB ,&
&  0.22110E+00_JPRB, 0.21864E+00_JPRB, 0.21629E+00_JPRB, 0.21553E+00_JPRB, 0.21572E+00_JPRB, 0.21655E+00_JPRB, 0.21531E+00_JPRB ,&
&  0.21372E+00_JPRB, 0.21461E+00_JPRB, 0.21522E+00_JPRB, 0.21967E+00_JPRB, 0.22608E+00_JPRB, 0.23341E+00_JPRB, 0.24055E+00_JPRB ,&
&  0.24964E+00_JPRB, 0.25370E+00_JPRB, 0.25676E+00_JPRB, 0.25751E+00_JPRB, 0.25599E+00_JPRB, 0.25513E+00_JPRB, 0.25288E+00_JPRB ,&
&  0.25212E+00_JPRB, 0.25143E+00_JPRB, 0.25235E+00_JPRB, 0.25325E+00_JPRB, 0.25203E+00_JPRB, 0.24911E+00_JPRB, 0.24317E+00_JPRB ,&
&  0.23484E+00_JPRB, 0.22800E+00_JPRB, 0.22202E+00_JPRB, 0.21742E+00_JPRB, 0.21153E+00_JPRB, 0.20628E+00_JPRB, 0.20104E+00_JPRB ,&
&  0.19640E+00_JPRB, 0.19281E+00_JPRB, 0.18866E+00_JPRB, 0.18559E+00_JPRB, 0.18418E+00_JPRB, 0.18253E+00_JPRB, 0.18182E+00_JPRB ,&
&  0.18176E+00_JPRB, 0.18207E+00_JPRB, 0.18389E+00_JPRB, 0.18544E+00_JPRB, 0.18787E+00_JPRB, 0.19068E+00_JPRB, 0.19250E+00_JPRB ,&
&  0.19550E+00_JPRB, 0.19809E+00_JPRB, 0.19948E+00_JPRB, 0.20042E+00_JPRB, 0.19948E+00_JPRB, 0.19694E+00_JPRB, 0.19425E+00_JPRB ,&
&  0.19090E+00_JPRB, 0.18847E+00_JPRB, 0.18690E+00_JPRB, 0.18521E+00_JPRB, 0.18439E+00_JPRB, 0.18370E+00_JPRB, 0.18301E+00_JPRB ,&
&  0.18250E+00_JPRB, 0.18230E+00_JPRB, 0.18207E+00_JPRB, 0.18297E+00_JPRB, 0.18398E+00_JPRB, 0.18580E+00_JPRB, 0.18802E+00_JPRB ,&
&  0.19015E+00_JPRB, 0.19337E+00_JPRB, 0.19690E+00_JPRB, 0.20167E+00_JPRB, 0.20993E+00_JPRB, 0.21724E+00_JPRB, 0.23297E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.27612E+00_JPRB, 0.28762E+00_JPRB, 0.27404E+00_JPRB, 0.27202E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.28562E+00_JPRB, 0.28280E+00_JPRB, 0.28029E+00_JPRB, 0.28003E+00_JPRB, 0.28059E+00_JPRB, 0.28364E+00_JPRB, 0.28152E+00_JPRB ,&
&  0.27721E+00_JPRB, 0.27571E+00_JPRB, 0.27715E+00_JPRB, 0.27766E+00_JPRB, 0.27680E+00_JPRB, 0.26601E+00_JPRB, 0.25989E+00_JPRB ,&
&  0.25319E+00_JPRB, 0.25067E+00_JPRB, 0.24873E+00_JPRB, 0.24937E+00_JPRB, 0.25177E+00_JPRB, 0.25328E+00_JPRB, 0.24933E+00_JPRB ,&
&  0.24499E+00_JPRB, 0.24072E+00_JPRB, 0.23645E+00_JPRB, 0.23465E+00_JPRB, 0.23596E+00_JPRB, 0.23752E+00_JPRB, 0.23788E+00_JPRB ,&
&  0.23859E+00_JPRB, 0.24123E+00_JPRB, 0.24357E+00_JPRB, 0.24687E+00_JPRB, 0.25955E+00_JPRB, 0.25832E+00_JPRB, 0.25704E+00_JPRB ,&
&  0.25723E+00_JPRB, 0.25748E+00_JPRB, 0.25831E+00_JPRB, 0.25867E+00_JPRB, 0.26210E+00_JPRB, 0.26529E+00_JPRB, 0.26740E+00_JPRB ,&
&  0.25348E+00_JPRB, 0.24485E+00_JPRB, 0.23681E+00_JPRB, 0.22793E+00_JPRB, 0.21848E+00_JPRB, 0.21063E+00_JPRB, 0.20337E+00_JPRB ,&
&  0.19716E+00_JPRB, 0.19133E+00_JPRB, 0.18620E+00_JPRB, 0.18416E+00_JPRB, 0.18347E+00_JPRB, 0.18246E+00_JPRB, 0.18246E+00_JPRB ,&
&  0.18300E+00_JPRB, 0.18371E+00_JPRB, 0.18422E+00_JPRB, 0.18563E+00_JPRB, 0.18777E+00_JPRB, 0.19108E+00_JPRB, 0.19464E+00_JPRB ,&
&  0.20003E+00_JPRB, 0.20424E+00_JPRB, 0.20908E+00_JPRB, 0.21281E+00_JPRB, 0.21135E+00_JPRB, 0.20663E+00_JPRB, 0.20147E+00_JPRB ,&
&  0.19560E+00_JPRB, 0.19217E+00_JPRB, 0.18878E+00_JPRB, 0.18633E+00_JPRB, 0.18496E+00_JPRB, 0.18422E+00_JPRB, 0.18461E+00_JPRB ,&
&  0.18488E+00_JPRB, 0.18540E+00_JPRB, 0.18508E+00_JPRB, 0.18533E+00_JPRB, 0.18611E+00_JPRB, 0.18776E+00_JPRB, 0.18991E+00_JPRB ,&
&  0.19319E+00_JPRB, 0.19702E+00_JPRB, 0.20165E+00_JPRB, 0.20679E+00_JPRB, 0.21408E+00_JPRB, 0.22402E+00_JPRB, 0.23856E+00_JPRB ,&
&  0.25565E+00_JPRB, 0.28945E+00_JPRB, 0.30311E+00_JPRB, 0.29214E+00_JPRB, 0.28875E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.30201E+00_JPRB, 0.29846E+00_JPRB, 0.29662E+00_JPRB, 0.29695E+00_JPRB, 0.29718E+00_JPRB, 0.30002E+00_JPRB, 0.29968E+00_JPRB ,&
&  0.29553E+00_JPRB, 0.29095E+00_JPRB, 0.28942E+00_JPRB, 0.28906E+00_JPRB, 0.28787E+00_JPRB, 0.28671E+00_JPRB, 0.28723E+00_JPRB ,&
&  0.28568E+00_JPRB, 0.28717E+00_JPRB, 0.28968E+00_JPRB, 0.29093E+00_JPRB, 0.29167E+00_JPRB, 0.29189E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.28598E+00_JPRB, 0.28450E+00_JPRB, 0.28996E+00_JPRB, 0.28746E+00_JPRB, 0.28379E+00_JPRB, 0.28065E+00_JPRB, 0.27963E+00_JPRB ,&
&  0.27186E+00_JPRB, 0.26092E+00_JPRB, 0.24793E+00_JPRB, 0.25224E+00_JPRB, 0.26914E+00_JPRB, 0.26770E+00_JPRB, 0.26549E+00_JPRB ,&
&  0.26517E+00_JPRB, 0.26455E+00_JPRB, 0.26534E+00_JPRB, 0.26579E+00_JPRB, 0.26645E+00_JPRB, 0.26635E+00_JPRB, 0.26716E+00_JPRB ,&
&  0.26833E+00_JPRB, 0.27338E+00_JPRB, 0.27726E+00_JPRB, 0.25954E+00_JPRB, 0.24351E+00_JPRB, 0.23122E+00_JPRB, 0.21966E+00_JPRB ,&
&  0.20940E+00_JPRB, 0.20612E+00_JPRB, 0.20285E+00_JPRB, 0.19773E+00_JPRB, 0.19834E+00_JPRB, 0.19881E+00_JPRB, 0.19817E+00_JPRB ,&
&  0.19654E+00_JPRB, 0.19554E+00_JPRB, 0.19415E+00_JPRB, 0.19189E+00_JPRB, 0.19097E+00_JPRB, 0.19204E+00_JPRB, 0.19510E+00_JPRB ,&
&  0.20007E+00_JPRB, 0.20605E+00_JPRB, 0.21952E+00_JPRB, 0.23132E+00_JPRB, 0.22865E+00_JPRB, 0.22038E+00_JPRB, 0.21281E+00_JPRB ,&
&  0.20346E+00_JPRB, 0.19869E+00_JPRB, 0.19406E+00_JPRB, 0.19036E+00_JPRB, 0.18872E+00_JPRB, 0.18907E+00_JPRB, 0.19004E+00_JPRB ,&
&  0.19133E+00_JPRB, 0.19316E+00_JPRB, 0.19390E+00_JPRB, 0.19347E+00_JPRB, 0.19269E+00_JPRB, 0.19303E+00_JPRB, 0.19438E+00_JPRB ,&
&  0.19701E+00_JPRB, 0.20164E+00_JPRB, 0.20738E+00_JPRB, 0.21440E+00_JPRB, 0.22041E+00_JPRB, 0.22597E+00_JPRB, 0.23440E+00_JPRB ,&
&  0.24208E+00_JPRB, 0.25911E+00_JPRB, 0.28457E+00_JPRB, 0.31231E+00_JPRB, 0.30896E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.30370E+00_JPRB, 0.30074E+00_JPRB, 0.30105E+00_JPRB, 0.30150E+00_JPRB, 0.30055E+00_JPRB, 0.30067E+00_JPRB, 0.30024E+00_JPRB ,&
&  0.30027E+00_JPRB, 0.29850E+00_JPRB, 0.29649E+00_JPRB, 0.29664E+00_JPRB, 0.29847E+00_JPRB, 0.30058E+00_JPRB, 0.30097E+00_JPRB ,&
&  0.30055E+00_JPRB, 0.30160E+00_JPRB, 0.30462E+00_JPRB, 0.30805E+00_JPRB, 0.30886E+00_JPRB, 0.30917E+00_JPRB, 0.31092E+00_JPRB ,&
&  0.31322E+00_JPRB, 0.31401E+00_JPRB, 0.31664E+00_JPRB, 0.31455E+00_JPRB, 0.31026E+00_JPRB, 0.30841E+00_JPRB, 0.30678E+00_JPRB ,&
&  0.30115E+00_JPRB, 0.27865E+00_JPRB, 0.26347E+00_JPRB, 0.26696E+00_JPRB, 0.28330E+00_JPRB, 0.28337E+00_JPRB, 0.28144E+00_JPRB ,&
&  0.27949E+00_JPRB, 0.27695E+00_JPRB, 0.27472E+00_JPRB, 0.27386E+00_JPRB, 0.27378E+00_JPRB, 0.27346E+00_JPRB, 0.27175E+00_JPRB ,&
&  0.27178E+00_JPRB, 0.27571E+00_JPRB, 0.28162E+00_JPRB, 0.28026E+00_JPRB, 0.27725E+00_JPRB, 0.27123E+00_JPRB, 0.26738E+00_JPRB ,&
&  0.26372E+00_JPRB, 0.25959E+00_JPRB, 0.25733E+00_JPRB, 0.25651E+00_JPRB, 0.25620E+00_JPRB, 0.25438E+00_JPRB, 0.25134E+00_JPRB ,&
&  0.24327E+00_JPRB, 0.23463E+00_JPRB, 0.22535E+00_JPRB, 0.21626E+00_JPRB, 0.20918E+00_JPRB, 0.20436E+00_JPRB, 0.20086E+00_JPRB ,&
&  0.20204E+00_JPRB, 0.21174E+00_JPRB, 0.22892E+00_JPRB, 0.24812E+00_JPRB, 0.27024E+00_JPRB, 0.24730E+00_JPRB, 0.23603E+00_JPRB ,&
&  0.22525E+00_JPRB, 0.21609E+00_JPRB, 0.20981E+00_JPRB, 0.20561E+00_JPRB, 0.20268E+00_JPRB, 0.20113E+00_JPRB, 0.20121E+00_JPRB ,&
&  0.20389E+00_JPRB, 0.20763E+00_JPRB, 0.21096E+00_JPRB, 0.21150E+00_JPRB, 0.21054E+00_JPRB, 0.20875E+00_JPRB, 0.20773E+00_JPRB ,&
&  0.20857E+00_JPRB, 0.21323E+00_JPRB, 0.22139E+00_JPRB, 0.23308E+00_JPRB, 0.24402E+00_JPRB, 0.25546E+00_JPRB, 0.26924E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.29956E+00_JPRB, 0.30329E+00_JPRB, 0.30627E+00_JPRB, 0.30629E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.31209E+00_JPRB, 0.31006E+00_JPRB, 0.31038E+00_JPRB, 0.31145E+00_JPRB, 0.31138E+00_JPRB, 0.31103E+00_JPRB, 0.30879E+00_JPRB ,&
&  0.30674E+00_JPRB, 0.30719E+00_JPRB, 0.30851E+00_JPRB, 0.31089E+00_JPRB, 0.31505E+00_JPRB, 0.31942E+00_JPRB, 0.31866E+00_JPRB ,&
&  0.31545E+00_JPRB, 0.31391E+00_JPRB, 0.31587E+00_JPRB, 0.31808E+00_JPRB, 0.31857E+00_JPRB, 0.31919E+00_JPRB, 0.32265E+00_JPRB ,&
&  0.32710E+00_JPRB, 0.32906E+00_JPRB, 0.33185E+00_JPRB, 0.33141E+00_JPRB, 0.32788E+00_JPRB, 0.32807E+00_JPRB, 0.32633E+00_JPRB ,&
&  0.32155E+00_JPRB, 0.31601E+00_JPRB, 0.30722E+00_JPRB, 0.30832E+00_JPRB, 0.30477E+00_JPRB, 0.30193E+00_JPRB, 0.29778E+00_JPRB ,&
&  0.29331E+00_JPRB, 0.28978E+00_JPRB, 0.28577E+00_JPRB, 0.28249E+00_JPRB, 0.28073E+00_JPRB, 0.28065E+00_JPRB, 0.27850E+00_JPRB ,&
&  0.27700E+00_JPRB, 0.27814E+00_JPRB, 0.27961E+00_JPRB, 0.27785E+00_JPRB, 0.27860E+00_JPRB, 0.28532E+00_JPRB, 0.29036E+00_JPRB ,&
&  0.29204E+00_JPRB, 0.29187E+00_JPRB, 0.28943E+00_JPRB, 0.28882E+00_JPRB, 0.28774E+00_JPRB, 0.28576E+00_JPRB, 0.28252E+00_JPRB ,&
&  0.27784E+00_JPRB, 0.27556E+00_JPRB, 0.27401E+00_JPRB, 0.27084E+00_JPRB, 0.26629E+00_JPRB, 0.24900E+00_JPRB, 0.23947E+00_JPRB ,&
&  0.23861E+00_JPRB, 0.24726E+00_JPRB, 0.26257E+00_JPRB, 0.26849E+00_JPRB, 0.27305E+00_JPRB, 0.27776E+00_JPRB, 0.27882E+00_JPRB ,&
&  0.28125E+00_JPRB, 0.26066E+00_JPRB, 0.24955E+00_JPRB, 0.24137E+00_JPRB, 0.23398E+00_JPRB, 0.22895E+00_JPRB, 0.22765E+00_JPRB ,&
&  0.23028E+00_JPRB, 0.23624E+00_JPRB, 0.24103E+00_JPRB, 0.24320E+00_JPRB, 0.24271E+00_JPRB, 0.23928E+00_JPRB, 0.23595E+00_JPRB ,&
&  0.23517E+00_JPRB, 0.24097E+00_JPRB, 0.25298E+00_JPRB, 0.26609E+00_JPRB, 0.27460E+00_JPRB, 0.28361E+00_JPRB, 0.29133E+00_JPRB ,&
&  0.29662E+00_JPRB, 0.30157E+00_JPRB, 0.30782E+00_JPRB, 0.31024E+00_JPRB, 0.31151E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.31251E+00_JPRB, 0.31254E+00_JPRB, 0.31575E+00_JPRB, 0.31926E+00_JPRB, 0.32023E+00_JPRB, 0.32027E+00_JPRB, 0.31804E+00_JPRB ,&
&  0.31655E+00_JPRB, 0.31776E+00_JPRB, 0.31954E+00_JPRB, 0.32078E+00_JPRB, 0.32267E+00_JPRB, 0.32511E+00_JPRB, 0.32698E+00_JPRB ,&
&  0.32695E+00_JPRB, 0.32524E+00_JPRB, 0.32225E+00_JPRB, 0.32310E+00_JPRB, 0.32313E+00_JPRB, 0.32499E+00_JPRB, 0.33212E+00_JPRB ,&
&  0.33721E+00_JPRB, 0.34074E+00_JPRB, 0.34373E+00_JPRB, 0.34414E+00_JPRB, 0.34370E+00_JPRB, 0.34522E+00_JPRB, 0.34205E+00_JPRB ,&
&  0.33606E+00_JPRB, 0.33047E+00_JPRB, 0.32579E+00_JPRB, 0.31883E+00_JPRB, 0.32126E+00_JPRB, 0.31869E+00_JPRB, 0.31558E+00_JPRB ,&
&  0.31160E+00_JPRB, 0.30677E+00_JPRB, 0.30125E+00_JPRB, 0.29611E+00_JPRB, 0.29237E+00_JPRB, 0.28897E+00_JPRB, 0.28424E+00_JPRB ,&
&  0.28268E+00_JPRB, 0.28436E+00_JPRB, 0.29004E+00_JPRB, 0.29317E+00_JPRB, 0.29136E+00_JPRB, 0.29262E+00_JPRB, 0.29930E+00_JPRB ,&
&  0.30586E+00_JPRB, 0.30879E+00_JPRB, 0.30859E+00_JPRB, 0.30913E+00_JPRB, 0.30974E+00_JPRB, 0.31105E+00_JPRB, 0.31165E+00_JPRB ,&
&  0.31067E+00_JPRB, 0.30914E+00_JPRB, 0.30671E+00_JPRB, 0.30077E+00_JPRB, 0.29363E+00_JPRB, 0.28783E+00_JPRB, 0.28545E+00_JPRB ,&
&  0.28795E+00_JPRB, 0.28753E+00_JPRB, 0.28718E+00_JPRB, 0.28848E+00_JPRB, 0.28891E+00_JPRB, 0.28808E+00_JPRB, 0.28702E+00_JPRB ,&
&  0.28812E+00_JPRB, 0.28761E+00_JPRB, 0.28941E+00_JPRB, 0.28670E+00_JPRB, 0.28222E+00_JPRB, 0.27792E+00_JPRB, 0.27389E+00_JPRB ,&
&  0.27144E+00_JPRB, 0.27044E+00_JPRB, 0.27234E+00_JPRB, 0.27291E+00_JPRB, 0.27463E+00_JPRB, 0.27666E+00_JPRB, 0.27837E+00_JPRB ,&
&  0.27930E+00_JPRB, 0.28042E+00_JPRB, 0.28259E+00_JPRB, 0.28723E+00_JPRB, 0.29218E+00_JPRB, 0.29652E+00_JPRB, 0.30110E+00_JPRB ,&
&  0.30546E+00_JPRB, 0.30862E+00_JPRB, 0.31445E+00_JPRB, 0.31584E+00_JPRB, 0.31534E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.31750E+00_JPRB, 0.31958E+00_JPRB, 0.32164E+00_JPRB, 0.32351E+00_JPRB, 0.32478E+00_JPRB, 0.32578E+00_JPRB, 0.32500E+00_JPRB ,&
&  0.32486E+00_JPRB, 0.32646E+00_JPRB, 0.32855E+00_JPRB, 0.33026E+00_JPRB, 0.33120E+00_JPRB, 0.33298E+00_JPRB, 0.33465E+00_JPRB ,&
&  0.33574E+00_JPRB, 0.33491E+00_JPRB, 0.33197E+00_JPRB, 0.33088E+00_JPRB, 0.32985E+00_JPRB, 0.33462E+00_JPRB, 0.34286E+00_JPRB ,&
&  0.34650E+00_JPRB, 0.34883E+00_JPRB, 0.35171E+00_JPRB, 0.35243E+00_JPRB, 0.35291E+00_JPRB, 0.35275E+00_JPRB, 0.34837E+00_JPRB ,&
&  0.34359E+00_JPRB, 0.33915E+00_JPRB, 0.33661E+00_JPRB, 0.33333E+00_JPRB, 0.33290E+00_JPRB, 0.33023E+00_JPRB, 0.32819E+00_JPRB ,&
&  0.32677E+00_JPRB, 0.32442E+00_JPRB, 0.32180E+00_JPRB, 0.31728E+00_JPRB, 0.31152E+00_JPRB, 0.30410E+00_JPRB, 0.29914E+00_JPRB ,&
&  0.29724E+00_JPRB, 0.30064E+00_JPRB, 0.30878E+00_JPRB, 0.31288E+00_JPRB, 0.31032E+00_JPRB, 0.31013E+00_JPRB, 0.31354E+00_JPRB ,&
&  0.31823E+00_JPRB, 0.32326E+00_JPRB, 0.32484E+00_JPRB, 0.32394E+00_JPRB, 0.32213E+00_JPRB, 0.32371E+00_JPRB, 0.32776E+00_JPRB ,&
&  0.33138E+00_JPRB, 0.33342E+00_JPRB, 0.33287E+00_JPRB, 0.33208E+00_JPRB, 0.32954E+00_JPRB, 0.32355E+00_JPRB, 0.31547E+00_JPRB ,&
&  0.30816E+00_JPRB, 0.30365E+00_JPRB, 0.30029E+00_JPRB, 0.29713E+00_JPRB, 0.29604E+00_JPRB, 0.29652E+00_JPRB, 0.29872E+00_JPRB ,&
&  0.29963E+00_JPRB, 0.30150E+00_JPRB, 0.30165E+00_JPRB, 0.29754E+00_JPRB, 0.29424E+00_JPRB, 0.29543E+00_JPRB, 0.29688E+00_JPRB ,&
&  0.29932E+00_JPRB, 0.30164E+00_JPRB, 0.30405E+00_JPRB, 0.30514E+00_JPRB, 0.30672E+00_JPRB, 0.30921E+00_JPRB, 0.31060E+00_JPRB ,&
&  0.31128E+00_JPRB, 0.31185E+00_JPRB, 0.31360E+00_JPRB, 0.31643E+00_JPRB, 0.31885E+00_JPRB, 0.32067E+00_JPRB, 0.31985E+00_JPRB ,&
&  0.31866E+00_JPRB, 0.31752E+00_JPRB, 0.31820E+00_JPRB, 0.31868E+00_JPRB, 0.31737E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.33812E+00_JPRB, 0.33824E+00_JPRB, 0.33772E+00_JPRB, 0.33679E+00_JPRB, 0.33589E+00_JPRB, 0.33518E+00_JPRB, 0.33538E+00_JPRB ,&
&  0.33569E+00_JPRB, 0.33642E+00_JPRB, 0.33731E+00_JPRB, 0.33806E+00_JPRB, 0.33971E+00_JPRB, 0.34157E+00_JPRB, 0.34299E+00_JPRB ,&
&  0.34457E+00_JPRB, 0.34395E+00_JPRB, 0.34182E+00_JPRB, 0.33970E+00_JPRB, 0.34049E+00_JPRB, 0.34575E+00_JPRB, 0.35164E+00_JPRB ,&
&  0.35330E+00_JPRB, 0.35457E+00_JPRB, 0.35525E+00_JPRB, 0.35437E+00_JPRB, 0.35329E+00_JPRB, 0.35303E+00_JPRB, 0.34946E+00_JPRB ,&
&  0.34519E+00_JPRB, 0.34385E+00_JPRB, 0.34359E+00_JPRB, 0.34070E+00_JPRB, 0.33977E+00_JPRB, 0.34131E+00_JPRB, 0.33965E+00_JPRB ,&
&  0.33842E+00_JPRB, 0.33674E+00_JPRB, 0.33401E+00_JPRB, 0.33184E+00_JPRB, 0.32927E+00_JPRB, 0.32413E+00_JPRB, 0.31808E+00_JPRB ,&
&  0.31318E+00_JPRB, 0.31345E+00_JPRB, 0.31862E+00_JPRB, 0.32426E+00_JPRB, 0.32385E+00_JPRB, 0.32338E+00_JPRB, 0.32508E+00_JPRB ,&
&  0.32863E+00_JPRB, 0.33293E+00_JPRB, 0.33562E+00_JPRB, 0.33478E+00_JPRB, 0.33368E+00_JPRB, 0.33553E+00_JPRB, 0.33812E+00_JPRB ,&
&  0.33945E+00_JPRB, 0.34098E+00_JPRB, 0.34207E+00_JPRB, 0.34350E+00_JPRB, 0.34377E+00_JPRB, 0.34164E+00_JPRB, 0.33975E+00_JPRB ,&
&  0.33629E+00_JPRB, 0.33199E+00_JPRB, 0.32704E+00_JPRB, 0.32177E+00_JPRB, 0.31827E+00_JPRB, 0.31643E+00_JPRB, 0.31696E+00_JPRB ,&
&  0.31673E+00_JPRB, 0.31844E+00_JPRB, 0.31892E+00_JPRB, 0.31666E+00_JPRB, 0.31363E+00_JPRB, 0.31357E+00_JPRB, 0.31696E+00_JPRB ,&
&  0.32161E+00_JPRB, 0.32469E+00_JPRB, 0.33006E+00_JPRB, 0.33285E+00_JPRB, 0.33488E+00_JPRB, 0.33727E+00_JPRB, 0.33787E+00_JPRB ,&
&  0.33529E+00_JPRB, 0.33323E+00_JPRB, 0.33284E+00_JPRB, 0.33559E+00_JPRB, 0.33636E+00_JPRB, 0.33569E+00_JPRB, 0.33371E+00_JPRB ,&
&  0.33346E+00_JPRB, 0.33552E+00_JPRB, 0.33636E+00_JPRB, 0.33622E+00_JPRB, 0.33685E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.36326E+00_JPRB, 0.36168E+00_JPRB, 0.36020E+00_JPRB, 0.35802E+00_JPRB, 0.35570E+00_JPRB, 0.35366E+00_JPRB, 0.35184E+00_JPRB ,&
&  0.35070E+00_JPRB, 0.35064E+00_JPRB, 0.35131E+00_JPRB, 0.35155E+00_JPRB, 0.35135E+00_JPRB, 0.35211E+00_JPRB, 0.35240E+00_JPRB ,&
&  0.35388E+00_JPRB, 0.35418E+00_JPRB, 0.35238E+00_JPRB, 0.35233E+00_JPRB, 0.35502E+00_JPRB, 0.35988E+00_JPRB, 0.36202E+00_JPRB ,&
&  0.36010E+00_JPRB, 0.36024E+00_JPRB, 0.36040E+00_JPRB, 0.35873E+00_JPRB, 0.35764E+00_JPRB, 0.35685E+00_JPRB, 0.35299E+00_JPRB ,&
&  0.34851E+00_JPRB, 0.34850E+00_JPRB, 0.35051E+00_JPRB, 0.35285E+00_JPRB, 0.35168E+00_JPRB, 0.34890E+00_JPRB, 0.34895E+00_JPRB ,&
&  0.34706E+00_JPRB, 0.34684E+00_JPRB, 0.34667E+00_JPRB, 0.34734E+00_JPRB, 0.34584E+00_JPRB, 0.34374E+00_JPRB, 0.33997E+00_JPRB ,&
&  0.33190E+00_JPRB, 0.32957E+00_JPRB, 0.33184E+00_JPRB, 0.33615E+00_JPRB, 0.33593E+00_JPRB, 0.33693E+00_JPRB, 0.34004E+00_JPRB ,&
&  0.34385E+00_JPRB, 0.34663E+00_JPRB, 0.34837E+00_JPRB, 0.34964E+00_JPRB, 0.35135E+00_JPRB, 0.35301E+00_JPRB, 0.35400E+00_JPRB ,&
&  0.35528E+00_JPRB, 0.35733E+00_JPRB, 0.35962E+00_JPRB, 0.36175E+00_JPRB, 0.36225E+00_JPRB, 0.36183E+00_JPRB, 0.36062E+00_JPRB ,&
&  0.35843E+00_JPRB, 0.35449E+00_JPRB, 0.35029E+00_JPRB, 0.34489E+00_JPRB, 0.33981E+00_JPRB, 0.33736E+00_JPRB, 0.33647E+00_JPRB ,&
&  0.33372E+00_JPRB, 0.33136E+00_JPRB, 0.32966E+00_JPRB, 0.32952E+00_JPRB, 0.32993E+00_JPRB, 0.33128E+00_JPRB, 0.33364E+00_JPRB ,&
&  0.33556E+00_JPRB, 0.33835E+00_JPRB, 0.34453E+00_JPRB, 0.34890E+00_JPRB, 0.35115E+00_JPRB, 0.35420E+00_JPRB, 0.35565E+00_JPRB ,&
&  0.35404E+00_JPRB, 0.35253E+00_JPRB, 0.35140E+00_JPRB, 0.35334E+00_JPRB, 0.35572E+00_JPRB, 0.35731E+00_JPRB, 0.35877E+00_JPRB ,&
&  0.36191E+00_JPRB, 0.36502E+00_JPRB, 0.36560E+00_JPRB, 0.36529E+00_JPRB, 0.36488E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.38584E+00_JPRB, 0.38199E+00_JPRB, 0.37804E+00_JPRB, 0.37459E+00_JPRB, 0.37162E+00_JPRB, 0.36850E+00_JPRB, 0.36568E+00_JPRB ,&
&  0.36286E+00_JPRB, 0.36123E+00_JPRB, 0.36009E+00_JPRB, 0.35909E+00_JPRB, 0.35608E+00_JPRB, 0.35476E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.35806E+00_JPRB, 0.36049E+00_JPRB, 0.36262E+00_JPRB, 0.36264E+00_JPRB, 0.36521E+00_JPRB, 0.36768E+00_JPRB, 0.36817E+00_JPRB ,&
&  0.36776E+00_JPRB, 0.36741E+00_JPRB, 0.36583E+00_JPRB, 0.36356E+00_JPRB, 0.36381E+00_JPRB, 0.36424E+00_JPRB, 0.36056E+00_JPRB ,&
&  0.35417E+00_JPRB, 0.35480E+00_JPRB, 0.34761E+00_JPRB, 0.35877E+00_JPRB, 0.36083E+00_JPRB, 0.35607E+00_JPRB, 0.35046E+00_JPRB ,&
&  0.34911E+00_JPRB, 0.35039E+00_JPRB, 0.35357E+00_JPRB, 0.35549E+00_JPRB, 0.35594E+00_JPRB, 0.35594E+00_JPRB, 0.35596E+00_JPRB ,&
&  0.34924E+00_JPRB, 0.34562E+00_JPRB, 0.34579E+00_JPRB, 0.34698E+00_JPRB, 0.34731E+00_JPRB, 0.34822E+00_JPRB, 0.35025E+00_JPRB ,&
&  0.35178E+00_JPRB, 0.35285E+00_JPRB, 0.35411E+00_JPRB, 0.35685E+00_JPRB, 0.35892E+00_JPRB, 0.36004E+00_JPRB, 0.36207E+00_JPRB ,&
&  0.36506E+00_JPRB, 0.36853E+00_JPRB, 0.37118E+00_JPRB, 0.37258E+00_JPRB, 0.37496E+00_JPRB, 0.37740E+00_JPRB, 0.37874E+00_JPRB ,&
&  0.37809E+00_JPRB, 0.37598E+00_JPRB, 0.37333E+00_JPRB, 0.36871E+00_JPRB, 0.36257E+00_JPRB, 0.35794E+00_JPRB, 0.35459E+00_JPRB ,&
&  0.34930E+00_JPRB, 0.34715E+00_JPRB, 0.34632E+00_JPRB, 0.34468E+00_JPRB, 0.34333E+00_JPRB, 0.34305E+00_JPRB, 0.34361E+00_JPRB ,&
&  0.34565E+00_JPRB, 0.34929E+00_JPRB, 0.35182E+00_JPRB, 0.35499E+00_JPRB, 0.35909E+00_JPRB, 0.36315E+00_JPRB, 0.36363E+00_JPRB ,&
&  0.36252E+00_JPRB, 0.36208E+00_JPRB, 0.36392E+00_JPRB, 0.36950E+00_JPRB, 0.37630E+00_JPRB, 0.38279E+00_JPRB, 0.38897E+00_JPRB ,&
&  0.39357E+00_JPRB, 0.39548E+00_JPRB, 0.39439E+00_JPRB, 0.39286E+00_JPRB, 0.38973E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.41600E+00_JPRB, 0.40880E+00_JPRB, 0.40142E+00_JPRB, 0.39586E+00_JPRB, 0.39187E+00_JPRB, 0.38772E+00_JPRB, 0.38271E+00_JPRB ,&
&  0.37896E+00_JPRB, 0.37398E+00_JPRB, 0.37023E+00_JPRB, 0.36673E+00_JPRB, 0.36376E+00_JPRB, 0.36284E+00_JPRB, 0.36378E+00_JPRB ,&
&  0.36637E+00_JPRB, 0.37172E+00_JPRB, 0.37507E+00_JPRB, 0.37427E+00_JPRB, 0.37636E+00_JPRB, 0.37878E+00_JPRB, 0.37954E+00_JPRB ,&
&  0.37771E+00_JPRB, 0.37506E+00_JPRB, 0.37208E+00_JPRB, 0.37041E+00_JPRB, 0.37053E+00_JPRB, 0.37157E+00_JPRB, 0.36838E+00_JPRB ,&
&  0.36349E+00_JPRB, 0.35487E+00_JPRB, 0.36454E+00_JPRB, 0.36840E+00_JPRB, 0.36864E+00_JPRB, 0.36678E+00_JPRB, 0.36372E+00_JPRB ,&
&  0.35877E+00_JPRB, 0.35585E+00_JPRB, 0.35656E+00_JPRB, 0.35842E+00_JPRB, 0.36069E+00_JPRB, 0.36153E+00_JPRB, 0.36252E+00_JPRB ,&
&  0.35963E+00_JPRB, 0.35834E+00_JPRB, 0.35709E+00_JPRB, 0.35652E+00_JPRB, 0.35751E+00_JPRB, 0.35863E+00_JPRB, 0.35927E+00_JPRB ,&
&  0.35841E+00_JPRB, 0.35927E+00_JPRB, 0.36046E+00_JPRB, 0.36008E+00_JPRB, 0.36189E+00_JPRB, 0.36444E+00_JPRB, 0.36770E+00_JPRB ,&
&  0.37160E+00_JPRB, 0.37653E+00_JPRB, 0.38159E+00_JPRB, 0.38767E+00_JPRB, 0.39476E+00_JPRB, 0.40052E+00_JPRB, 0.40433E+00_JPRB ,&
&  0.40553E+00_JPRB, 0.40376E+00_JPRB, 0.39961E+00_JPRB, 0.39278E+00_JPRB, 0.38673E+00_JPRB, 0.38201E+00_JPRB, 0.37577E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.36285E+00_JPRB, 0.36018E+00_JPRB, 0.35912E+00_JPRB, 0.35772E+00_JPRB, 0.35730E+00_JPRB, 0.35861E+00_JPRB ,&
&  0.36152E+00_JPRB, 0.36446E+00_JPRB, 0.36627E+00_JPRB, 0.36748E+00_JPRB, 0.36924E+00_JPRB, 0.36884E+00_JPRB, 0.36882E+00_JPRB ,&
&  0.37124E+00_JPRB, 0.37658E+00_JPRB, 0.38602E+00_JPRB, 0.40197E+00_JPRB, 0.41550E+00_JPRB, 0.42508E+00_JPRB, 0.43098E+00_JPRB ,&
&  0.43335E+00_JPRB, 0.43342E+00_JPRB, 0.43065E+00_JPRB, 0.42718E+00_JPRB, 0.42269E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.43620E+00_JPRB, 0.42649E+00_JPRB, 0.41639E+00_JPRB, 0.40982E+00_JPRB, 0.40529E+00_JPRB, 0.40072E+00_JPRB, 0.39524E+00_JPRB ,&
&  0.38899E+00_JPRB, 0.38348E+00_JPRB, 0.37836E+00_JPRB, 0.37319E+00_JPRB, 0.37080E+00_JPRB, 0.36981E+00_JPRB, 0.37073E+00_JPRB ,&
&  0.37358E+00_JPRB, 0.37836E+00_JPRB, 0.38095E+00_JPRB, 0.38131E+00_JPRB, 0.38225E+00_JPRB, 0.38226E+00_JPRB, 0.38088E+00_JPRB ,&
&  0.37724E+00_JPRB, 0.37275E+00_JPRB, 0.37048E+00_JPRB, 0.37155E+00_JPRB, 0.37196E+00_JPRB, 0.37259E+00_JPRB, 0.37204E+00_JPRB ,&
&  0.37547E+00_JPRB, 0.34144E+00_JPRB, 0.36330E+00_JPRB, 0.37458E+00_JPRB, 0.37203E+00_JPRB, 0.37122E+00_JPRB, 0.36832E+00_JPRB ,&
&  0.36262E+00_JPRB, 0.35969E+00_JPRB, 0.35811E+00_JPRB, 0.35946E+00_JPRB, 0.36217E+00_JPRB, 0.36334E+00_JPRB, 0.36344E+00_JPRB ,&
&  0.36081E+00_JPRB, 0.36092E+00_JPRB, 0.36063E+00_JPRB, 0.36085E+00_JPRB, 0.36172E+00_JPRB, 0.36258E+00_JPRB, 0.36272E+00_JPRB ,&
&  0.36163E+00_JPRB, 0.35902E+00_JPRB, 0.35638E+00_JPRB, 0.35647E+00_JPRB, 0.35833E+00_JPRB, 0.36225E+00_JPRB, 0.36867E+00_JPRB ,&
&  0.37699E+00_JPRB, 0.38616E+00_JPRB, 0.39569E+00_JPRB, 0.40530E+00_JPRB, 0.41547E+00_JPRB, 0.42407E+00_JPRB, 0.43034E+00_JPRB ,&
&  0.43022E+00_JPRB, 0.42650E+00_JPRB, 0.41892E+00_JPRB, 0.40943E+00_JPRB, 0.40211E+00_JPRB, 0.39466E+00_JPRB, 0.38727E+00_JPRB ,&
&  0.38165E+00_JPRB, 0.37657E+00_JPRB, 0.37354E+00_JPRB, 0.37325E+00_JPRB, 0.37372E+00_JPRB, 0.37335E+00_JPRB, 0.37360E+00_JPRB ,&
&  0.37413E+00_JPRB, 0.37471E+00_JPRB, 0.37531E+00_JPRB, 0.37565E+00_JPRB, 0.37485E+00_JPRB, 0.37400E+00_JPRB, 0.37535E+00_JPRB ,&
&  0.37956E+00_JPRB, 0.38776E+00_JPRB, 0.40174E+00_JPRB, 0.42196E+00_JPRB, 0.44184E+00_JPRB, 0.45629E+00_JPRB, 0.46288E+00_JPRB ,&
&  0.46191E+00_JPRB, 0.45786E+00_JPRB, 0.45104E+00_JPRB, 0.44678E+00_JPRB, 0.44364E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.45147E+00_JPRB, 0.44646E+00_JPRB, 0.44104E+00_JPRB, 0.43525E+00_JPRB, 0.42876E+00_JPRB, 0.42106E+00_JPRB, 0.41274E+00_JPRB ,&
&  0.40602E+00_JPRB, 0.40042E+00_JPRB, 0.39504E+00_JPRB, 0.39092E+00_JPRB, 0.38726E+00_JPRB, 0.38628E+00_JPRB, 0.38662E+00_JPRB ,&
&  0.38756E+00_JPRB, 0.38742E+00_JPRB, 0.38691E+00_JPRB, 0.38769E+00_JPRB, 0.38773E+00_JPRB, 0.38581E+00_JPRB, 0.38296E+00_JPRB ,&
&  0.37803E+00_JPRB, 0.37384E+00_JPRB, 0.37261E+00_JPRB, 0.37538E+00_JPRB, 0.37478E+00_JPRB, 0.37662E+00_JPRB, 0.38328E+00_JPRB ,&
&  0.36705E+00_JPRB, 0.33808E+00_JPRB, 0.36451E+00_JPRB, 0.39218E+00_JPRB, 0.38533E+00_JPRB, 0.37872E+00_JPRB, 0.37616E+00_JPRB ,&
&  0.37282E+00_JPRB, 0.37105E+00_JPRB, 0.36944E+00_JPRB, 0.36893E+00_JPRB, 0.37006E+00_JPRB, 0.36954E+00_JPRB, 0.36771E+00_JPRB ,&
&  0.36638E+00_JPRB, 0.36846E+00_JPRB, 0.37070E+00_JPRB, 0.37192E+00_JPRB, 0.37171E+00_JPRB, 0.37086E+00_JPRB, 0.36780E+00_JPRB ,&
&  0.36504E+00_JPRB, 0.36361E+00_JPRB, 0.36410E+00_JPRB, 0.36788E+00_JPRB, 0.37391E+00_JPRB, 0.38312E+00_JPRB, 0.39618E+00_JPRB ,&
&  0.41070E+00_JPRB, 0.42449E+00_JPRB, 0.43706E+00_JPRB, 0.44777E+00_JPRB, 0.45752E+00_JPRB, 0.46482E+00_JPRB, 0.46798E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.45024E+00_JPRB, 0.43635E+00_JPRB, 0.42607E+00_JPRB, 0.41892E+00_JPRB, 0.41433E+00_JPRB, 0.41169E+00_JPRB ,&
&  0.41121E+00_JPRB, 0.41063E+00_JPRB, 0.40950E+00_JPRB, 0.40919E+00_JPRB, 0.40921E+00_JPRB, 0.40805E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.40569E+00_JPRB, 0.40325E+00_JPRB, 0.39968E+00_JPRB, 0.39619E+00_JPRB, 0.39350E+00_JPRB, 0.39350E+00_JPRB, 0.39577E+00_JPRB ,&
&  0.40201E+00_JPRB, 0.41466E+00_JPRB, 0.43053E+00_JPRB, 0.46113E+00_JPRB, 0.48206E+00_JPRB, 0.49605E+00_JPRB, 0.50106E+00_JPRB ,&
&  0.49495E+00_JPRB, 0.47888E+00_JPRB, 0.46584E+00_JPRB, 0.45885E+00_JPRB, 0.45825E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.46041E+00_JPRB, 0.45964E+00_JPRB, 0.45509E+00_JPRB, 0.44918E+00_JPRB, 0.44362E+00_JPRB, 0.43626E+00_JPRB, 0.42705E+00_JPRB ,&
&  0.41864E+00_JPRB, 0.41074E+00_JPRB, 0.40609E+00_JPRB, 0.40200E+00_JPRB, 0.39574E+00_JPRB, 0.39262E+00_JPRB, 0.39346E+00_JPRB ,&
&  0.39292E+00_JPRB, 0.38881E+00_JPRB, 0.38378E+00_JPRB, 0.38345E+00_JPRB, 0.38214E+00_JPRB, 0.38009E+00_JPRB, 0.37696E+00_JPRB ,&
&  0.37355E+00_JPRB, 0.37279E+00_JPRB, 0.37391E+00_JPRB, 0.37561E+00_JPRB, 0.37299E+00_JPRB, 0.37448E+00_JPRB, 0.38334E+00_JPRB ,&
&  0.42176E+00_JPRB, 0.35831E+00_JPRB, 0.39076E+00_JPRB, 0.42749E+00_JPRB, 0.41708E+00_JPRB, 0.40772E+00_JPRB, 0.39887E+00_JPRB ,&
&  0.39300E+00_JPRB, 0.39123E+00_JPRB, 0.39050E+00_JPRB, 0.38725E+00_JPRB, 0.38351E+00_JPRB, 0.37887E+00_JPRB, 0.37671E+00_JPRB ,&
&  0.37625E+00_JPRB, 0.37994E+00_JPRB, 0.38221E+00_JPRB, 0.38236E+00_JPRB, 0.37997E+00_JPRB, 0.37765E+00_JPRB, 0.37607E+00_JPRB ,&
&  0.37600E+00_JPRB, 0.37797E+00_JPRB, 0.38064E+00_JPRB, 0.38419E+00_JPRB, 0.39009E+00_JPRB, 0.40167E+00_JPRB, 0.41873E+00_JPRB ,&
&  0.43901E+00_JPRB, 0.45658E+00_JPRB, 0.47146E+00_JPRB, 0.48471E+00_JPRB, 0.49417E+00_JPRB, 0.49816E+00_JPRB, 0.49448E+00_JPRB ,&
&  0.48503E+00_JPRB, 0.46870E+00_JPRB, 0.45734E+00_JPRB, 0.44819E+00_JPRB, 0.43890E+00_JPRB, 0.43229E+00_JPRB, 0.42832E+00_JPRB ,&
&  0.42638E+00_JPRB, 0.42514E+00_JPRB, 0.42359E+00_JPRB, 0.42094E+00_JPRB, 0.42021E+00_JPRB, 0.42011E+00_JPRB, 0.42169E+00_JPRB ,&
&  0.42170E+00_JPRB, 0.42069E+00_JPRB, 0.41646E+00_JPRB, 0.41046E+00_JPRB, 0.40852E+00_JPRB, 0.40901E+00_JPRB, 0.41157E+00_JPRB ,&
&  0.41834E+00_JPRB, 0.43115E+00_JPRB, 0.44701E+00_JPRB, 0.48126E+00_JPRB, 0.49890E+00_JPRB, 0.50996E+00_JPRB, 0.51125E+00_JPRB ,&
&  0.50067E+00_JPRB, 0.48337E+00_JPRB, 0.39315E+00_JPRB, 0.44770E+00_JPRB, 0.46372E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.50094E+00_JPRB, 0.49426E+00_JPRB, 0.48331E+00_JPRB, 0.46675E+00_JPRB, 0.45459E+00_JPRB, 0.44534E+00_JPRB, 0.43586E+00_JPRB ,&
&  0.42742E+00_JPRB, 0.42068E+00_JPRB, 0.41521E+00_JPRB, 0.40865E+00_JPRB, 0.40130E+00_JPRB, 0.40034E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.39867E+00_JPRB, 0.39247E+00_JPRB, 0.38651E+00_JPRB, 0.38558E+00_JPRB, 0.38536E+00_JPRB, 0.38540E+00_JPRB, 0.38284E+00_JPRB ,&
&  0.37833E+00_JPRB, 0.37802E+00_JPRB, 0.38200E+00_JPRB, 0.38359E+00_JPRB, 0.38026E+00_JPRB, 0.38235E+00_JPRB, 0.39158E+00_JPRB ,&
&  0.43734E+00_JPRB, 0.38577E+00_JPRB, 0.41587E+00_JPRB, 0.48035E+00_JPRB, 0.47523E+00_JPRB, 0.47361E+00_JPRB, 0.46067E+00_JPRB ,&
&  0.44704E+00_JPRB, 0.44033E+00_JPRB, 0.43459E+00_JPRB, 0.42219E+00_JPRB, 0.40867E+00_JPRB, 0.40073E+00_JPRB, 0.39893E+00_JPRB ,&
&  0.40111E+00_JPRB, 0.40284E+00_JPRB, 0.40403E+00_JPRB, 0.40100E+00_JPRB, 0.39960E+00_JPRB, 0.39906E+00_JPRB, 0.39951E+00_JPRB ,&
&  0.40021E+00_JPRB, 0.40318E+00_JPRB, 0.40684E+00_JPRB, 0.40903E+00_JPRB, 0.41577E+00_JPRB, 0.43509E+00_JPRB, 0.46181E+00_JPRB ,&
&  0.48683E+00_JPRB, 0.50796E+00_JPRB, 0.52192E+00_JPRB, 0.53188E+00_JPRB, 0.53437E+00_JPRB, 0.53507E+00_JPRB, 0.53036E+00_JPRB ,&
&  0.51572E+00_JPRB, 0.49881E+00_JPRB, 0.48757E+00_JPRB, 0.47511E+00_JPRB, 0.46322E+00_JPRB, 0.45427E+00_JPRB, 0.44700E+00_JPRB ,&
&  0.44132E+00_JPRB, 0.43701E+00_JPRB, 0.43365E+00_JPRB, 0.42821E+00_JPRB, 0.42479E+00_JPRB, 0.42568E+00_JPRB, 0.42720E+00_JPRB ,&
&  0.42843E+00_JPRB, 0.43113E+00_JPRB, 0.43354E+00_JPRB, 0.43542E+00_JPRB, 0.43711E+00_JPRB, 0.44003E+00_JPRB, 0.44378E+00_JPRB ,&
&  0.45514E+00_JPRB, 0.47367E+00_JPRB, 0.49402E+00_JPRB, 0.51263E+00_JPRB, 0.53490E+00_JPRB, 0.53558E+00_JPRB, 0.52318E+00_JPRB ,&
&  0.50523E+00_JPRB, 0.48558E+00_JPRB, 0.47612E+00_JPRB, 0.42956E+00_JPRB, 0.50309E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.55808E+00_JPRB, 0.53743E+00_JPRB, 0.50561E+00_JPRB, 0.47519E+00_JPRB, 0.45396E+00_JPRB, 0.44059E+00_JPRB, 0.43297E+00_JPRB ,&
&  0.42752E+00_JPRB, 0.42293E+00_JPRB, 0.41765E+00_JPRB, 0.40954E+00_JPRB, 0.40476E+00_JPRB, 0.40320E+00_JPRB, 0.40242E+00_JPRB ,&
&  0.39974E+00_JPRB, 0.39177E+00_JPRB, 0.38949E+00_JPRB, 0.38746E+00_JPRB, 0.38669E+00_JPRB, 0.38578E+00_JPRB, 0.38186E+00_JPRB ,&
&  0.37670E+00_JPRB, 0.37446E+00_JPRB, 0.37871E+00_JPRB, 0.38144E+00_JPRB, 0.38401E+00_JPRB, 0.38957E+00_JPRB, 0.40318E+00_JPRB ,&
&  0.45351E+00_JPRB, 0.44774E+00_JPRB, 0.50098E+00_JPRB, 0.54366E+00_JPRB, 0.53430E+00_JPRB, 0.52174E+00_JPRB, 0.51456E+00_JPRB ,&
&  0.50318E+00_JPRB, 0.49421E+00_JPRB, 0.47579E+00_JPRB, 0.45005E+00_JPRB, 0.42895E+00_JPRB, 0.42401E+00_JPRB, 0.41917E+00_JPRB ,&
&  0.41540E+00_JPRB, 0.41456E+00_JPRB, 0.41506E+00_JPRB, 0.41625E+00_JPRB, 0.41740E+00_JPRB, 0.41747E+00_JPRB, 0.41755E+00_JPRB ,&
&  0.41880E+00_JPRB, 0.42185E+00_JPRB, 0.42465E+00_JPRB, 0.43130E+00_JPRB, 0.44835E+00_JPRB, 0.47515E+00_JPRB, 0.50324E+00_JPRB ,&
&  0.52426E+00_JPRB, 0.54018E+00_JPRB, 0.55937E+00_JPRB, 0.57240E+00_JPRB, 0.57672E+00_JPRB, 0.57318E+00_JPRB, 0.56181E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.53956E+00_JPRB, 0.51997E+00_JPRB, 0.49895E+00_JPRB, 0.48595E+00_JPRB, 0.47823E+00_JPRB, 0.47296E+00_JPRB ,&
&  0.46914E+00_JPRB, 0.46341E+00_JPRB, 0.45590E+00_JPRB, 0.44934E+00_JPRB, 0.44648E+00_JPRB, 0.44267E+00_JPRB, 0.43803E+00_JPRB ,&
&  0.43785E+00_JPRB, 0.44266E+00_JPRB, 0.45013E+00_JPRB, 0.45499E+00_JPRB, 0.45972E+00_JPRB, 0.46712E+00_JPRB, 0.47284E+00_JPRB ,&
&  0.48243E+00_JPRB, 0.50086E+00_JPRB, 0.51682E+00_JPRB, 0.54887E+00_JPRB, 0.54454E+00_JPRB, 0.53773E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.49778E+00_JPRB, 0.48158E+00_JPRB, 0.46827E+00_JPRB, 0.49061E+00_JPRB, 0.55149E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.60130E+00_JPRB, 0.55068E+00_JPRB, 0.50892E+00_JPRB, 0.47481E+00_JPRB, 0.45375E+00_JPRB, 0.44154E+00_JPRB, 0.43303E+00_JPRB ,&
&  0.42932E+00_JPRB, 0.42769E+00_JPRB, 0.42220E+00_JPRB, 0.41807E+00_JPRB, 0.41537E+00_JPRB, 0.41475E+00_JPRB, 0.41467E+00_JPRB ,&
&  0.40756E+00_JPRB, 0.40652E+00_JPRB, 0.40691E+00_JPRB, 0.40318E+00_JPRB, 0.39802E+00_JPRB, 0.39277E+00_JPRB, 0.38590E+00_JPRB ,&
&  0.38155E+00_JPRB, 0.38050E+00_JPRB, 0.38320E+00_JPRB, 0.38754E+00_JPRB, 0.39168E+00_JPRB, 0.40296E+00_JPRB, 0.42443E+00_JPRB ,&
&  0.48449E+00_JPRB, 0.51725E+00_JPRB, 0.59673E+00_JPRB, 0.62907E+00_JPRB, 0.57291E+00_JPRB, 0.55918E+00_JPRB, 0.55720E+00_JPRB ,&
&  0.56044E+00_JPRB, 0.56140E+00_JPRB, 0.53955E+00_JPRB, 0.50778E+00_JPRB, 0.47945E+00_JPRB, 0.46380E+00_JPRB, 0.45455E+00_JPRB ,&
&  0.44614E+00_JPRB, 0.44076E+00_JPRB, 0.43650E+00_JPRB, 0.43977E+00_JPRB, 0.44259E+00_JPRB, 0.44178E+00_JPRB, 0.44074E+00_JPRB ,&
&  0.44028E+00_JPRB, 0.44220E+00_JPRB, 0.44739E+00_JPRB, 0.46644E+00_JPRB, 0.50256E+00_JPRB, 0.54005E+00_JPRB, 0.57442E+00_JPRB ,&
&  0.59392E+00_JPRB, 0.61308E+00_JPRB, 0.63192E+00_JPRB, 0.64285E+00_JPRB, 0.64159E+00_JPRB, 0.63410E+00_JPRB, 0.62560E+00_JPRB ,&
&  0.61431E+00_JPRB, 0.58831E+00_JPRB, 0.55315E+00_JPRB, 0.52742E+00_JPRB, 0.51434E+00_JPRB, 0.50739E+00_JPRB, 0.50469E+00_JPRB ,&
&  0.50267E+00_JPRB, 0.50040E+00_JPRB, 0.49518E+00_JPRB, 0.48696E+00_JPRB, 0.47544E+00_JPRB, 0.46762E+00_JPRB, 0.46078E+00_JPRB ,&
&  0.45683E+00_JPRB, 0.45720E+00_JPRB, 0.46188E+00_JPRB, 0.47138E+00_JPRB, 0.47974E+00_JPRB, 0.48794E+00_JPRB, 0.49506E+00_JPRB ,&
&  0.50716E+00_JPRB, 0.53316E+00_JPRB, 0.52221E+00_JPRB, 0.53462E+00_JPRB, 0.57680E+00_JPRB, 0.55277E+00_JPRB, 0.52388E+00_JPRB ,&
&  0.49887E+00_JPRB, 0.48220E+00_JPRB, 0.48068E+00_JPRB, 0.50926E+00_JPRB, 0.63060E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.51212E+00_JPRB, 0.49486E+00_JPRB, 0.47827E+00_JPRB, 0.46064E+00_JPRB, 0.44457E+00_JPRB, 0.43666E+00_JPRB, 0.43475E+00_JPRB ,&
&  0.43690E+00_JPRB, 0.43841E+00_JPRB, 0.43658E+00_JPRB, 0.43079E+00_JPRB, 0.42729E+00_JPRB, 0.42425E+00_JPRB, 0.42208E+00_JPRB ,&
&  0.41652E+00_JPRB, 0.41730E+00_JPRB, 0.41407E+00_JPRB, 0.40878E+00_JPRB, 0.40295E+00_JPRB, 0.39585E+00_JPRB, 0.38902E+00_JPRB ,&
&  0.38512E+00_JPRB, 0.38443E+00_JPRB, 0.38622E+00_JPRB, 0.39098E+00_JPRB, 0.40289E+00_JPRB, 0.41448E+00_JPRB, 0.42778E+00_JPRB ,&
&  0.48922E+00_JPRB, 0.60359E+00_JPRB, 0.73184E+00_JPRB, 0.80551E+00_JPRB, 0.72242E+00_JPRB, 0.66448E+00_JPRB, 0.64061E+00_JPRB ,&
&  0.62900E+00_JPRB, 0.61534E+00_JPRB, 0.59067E+00_JPRB, 0.56632E+00_JPRB, 0.53960E+00_JPRB, 0.52792E+00_JPRB, 0.51478E+00_JPRB ,&
&  0.50083E+00_JPRB, 0.48951E+00_JPRB, 0.47681E+00_JPRB, 0.47338E+00_JPRB, 0.47088E+00_JPRB, 0.46646E+00_JPRB, 0.46225E+00_JPRB ,&
&  0.46238E+00_JPRB, 0.46793E+00_JPRB, 0.48074E+00_JPRB, 0.51392E+00_JPRB, 0.57202E+00_JPRB, 0.62693E+00_JPRB, 0.68853E+00_JPRB ,&
&  0.71413E+00_JPRB, 0.72655E+00_JPRB, 0.73624E+00_JPRB, 0.73035E+00_JPRB, 0.70255E+00_JPRB, 0.68237E+00_JPRB, 0.67922E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.61716E+00_JPRB, 0.56980E+00_JPRB, 0.54007E+00_JPRB, 0.52441E+00_JPRB, 0.51939E+00_JPRB, 0.51679E+00_JPRB ,&
&  0.51808E+00_JPRB, 0.51817E+00_JPRB, 0.51018E+00_JPRB, 0.49682E+00_JPRB, 0.48361E+00_JPRB, 0.47518E+00_JPRB, 0.47084E+00_JPRB ,&
&  0.46618E+00_JPRB, 0.46551E+00_JPRB, 0.47311E+00_JPRB, 0.47924E+00_JPRB, 0.48305E+00_JPRB, 0.48961E+00_JPRB, 0.50115E+00_JPRB ,&
&  0.51797E+00_JPRB, 0.53066E+00_JPRB, 0.52572E+00_JPRB, 0.53438E+00_JPRB, 0.53701E+00_JPRB, 0.54814E+00_JPRB, 0.52384E+00_JPRB ,&
&  0.49716E+00_JPRB, 0.48933E+00_JPRB, 0.50133E+00_JPRB, 0.51674E+00_JPRB, 0.52794E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.49180E+00_JPRB, 0.47872E+00_JPRB, 0.46687E+00_JPRB, 0.44742E+00_JPRB, 0.43530E+00_JPRB, 0.42882E+00_JPRB, 0.43096E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.44381E+00_JPRB, 0.44143E+00_JPRB, 0.43862E+00_JPRB, 0.43357E+00_JPRB, 0.43070E+00_JPRB, 0.42415E+00_JPRB ,&
&  0.42079E+00_JPRB, 0.41727E+00_JPRB, 0.41176E+00_JPRB, 0.40619E+00_JPRB, 0.39748E+00_JPRB, 0.39390E+00_JPRB, 0.38904E+00_JPRB ,&
&  0.38428E+00_JPRB, 0.38262E+00_JPRB, 0.38516E+00_JPRB, 0.39421E+00_JPRB, 0.40538E+00_JPRB, 0.41820E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.52038E+00_JPRB, 0.59575E+00_JPRB, 0.93337E+00_JPRB, 0.95806E+00_JPRB, 0.88265E+00_JPRB, 0.80169E+00_JPRB, 0.79285E+00_JPRB ,&
&  0.77881E+00_JPRB, 0.72922E+00_JPRB, 0.68513E+00_JPRB, 0.65399E+00_JPRB, 0.62503E+00_JPRB, 0.60527E+00_JPRB, 0.58057E+00_JPRB ,&
&  0.55914E+00_JPRB, 0.53575E+00_JPRB, 0.50694E+00_JPRB, 0.48860E+00_JPRB, 0.47654E+00_JPRB, 0.47242E+00_JPRB, 0.47684E+00_JPRB ,&
&  0.48491E+00_JPRB, 0.49651E+00_JPRB, 0.52725E+00_JPRB, 0.57731E+00_JPRB, 0.62523E+00_JPRB, 0.62431E+00_JPRB, 0.70386E+00_JPRB ,&
&  0.86054E+00_JPRB, 0.85978E+00_JPRB, 0.81598E+00_JPRB, 0.77728E+00_JPRB, 0.71777E+00_JPRB, 0.69385E+00_JPRB, 0.68602E+00_JPRB ,&
&  0.66313E+00_JPRB, 0.62697E+00_JPRB, 0.58870E+00_JPRB, 0.55966E+00_JPRB, 0.53960E+00_JPRB, 0.52796E+00_JPRB, 0.52792E+00_JPRB ,&
&  0.52578E+00_JPRB, 0.51506E+00_JPRB, 0.49674E+00_JPRB, 0.48154E+00_JPRB, 0.47363E+00_JPRB, 0.47208E+00_JPRB, 0.47289E+00_JPRB ,&
&  0.47346E+00_JPRB, 0.47539E+00_JPRB, 0.48053E+00_JPRB, 0.46965E+00_JPRB, 0.47767E+00_JPRB, 0.49527E+00_JPRB, 0.50221E+00_JPRB ,&
&  0.49387E+00_JPRB, 0.49669E+00_JPRB, 0.50963E+00_JPRB, 0.51664E+00_JPRB, 0.52474E+00_JPRB, 0.55368E+00_JPRB, 0.52990E+00_JPRB ,&
&  0.50144E+00_JPRB, 0.49785E+00_JPRB, 0.51242E+00_JPRB, 0.51820E+00_JPRB, 0.51648E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.47522E+00_JPRB, 0.46144E+00_JPRB, 0.45229E+00_JPRB, 0.43930E+00_JPRB, 0.42743E+00_JPRB, 0.42075E+00_JPRB, 0.42329E+00_JPRB ,&
&  0.43591E+00_JPRB, 0.44778E+00_JPRB, 0.44299E+00_JPRB, 0.44128E+00_JPRB, 0.44082E+00_JPRB, 0.43543E+00_JPRB, 0.42652E+00_JPRB ,&
&  0.41754E+00_JPRB, 0.40900E+00_JPRB, 0.39933E+00_JPRB, 0.38959E+00_JPRB, 0.38281E+00_JPRB, 0.38356E+00_JPRB, 0.38236E+00_JPRB ,&
&  0.38063E+00_JPRB, 0.38398E+00_JPRB, 0.38879E+00_JPRB, 0.39751E+00_JPRB, 0.40566E+00_JPRB, 0.41770E+00_JPRB, 0.43738E+00_JPRB ,&
&  0.51880E+00_JPRB, 0.42688E+00_JPRB, 0.92857E+00_JPRB, 0.11365E+01_JPRB, 0.11731E+01_JPRB, 0.10861E+01_JPRB, 0.10096E+01_JPRB ,&
&  0.97104E+00_JPRB, 0.86695E+00_JPRB, 0.79130E+00_JPRB, 0.72383E+00_JPRB, 0.64707E+00_JPRB, 0.60160E+00_JPRB, 0.57774E+00_JPRB ,&
&  0.56039E+00_JPRB, 0.54218E+00_JPRB, 0.52609E+00_JPRB, 0.51146E+00_JPRB, 0.49878E+00_JPRB, 0.49544E+00_JPRB, 0.50448E+00_JPRB ,&
&  0.51909E+00_JPRB, 0.54408E+00_JPRB, 0.59668E+00_JPRB, 0.66231E+00_JPRB, 0.67920E+00_JPRB, 0.69831E+00_JPRB, 0.73927E+00_JPRB ,&
&  0.83085E+00_JPRB, 0.92426E+00_JPRB, 0.83148E+00_JPRB, 0.73712E+00_JPRB, 0.67614E+00_JPRB, 0.65964E+00_JPRB, 0.65720E+00_JPRB ,&
&  0.63906E+00_JPRB, 0.61861E+00_JPRB, 0.59555E+00_JPRB, 0.57010E+00_JPRB, 0.54832E+00_JPRB, 0.53588E+00_JPRB, 0.52671E+00_JPRB ,&
&  0.51324E+00_JPRB, 0.49469E+00_JPRB, 0.48250E+00_JPRB, 0.47623E+00_JPRB, 0.47379E+00_JPRB, 0.47310E+00_JPRB, 0.47634E+00_JPRB ,&
&  0.48436E+00_JPRB, 0.48630E+00_JPRB, 0.47853E+00_JPRB, 0.46407E+00_JPRB, 0.46092E+00_JPRB, 0.46586E+00_JPRB, 0.46987E+00_JPRB ,&
&  0.48448E+00_JPRB, 0.49344E+00_JPRB, 0.49960E+00_JPRB, 0.51176E+00_JPRB, 0.49829E+00_JPRB, 0.53792E+00_JPRB, 0.51996E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.50191E+00_JPRB, 0.50791E+00_JPRB, 0.50817E+00_JPRB, 0.49599E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.45430E+00_JPRB, 0.44470E+00_JPRB, 0.43599E+00_JPRB, 0.43179E+00_JPRB, 0.42401E+00_JPRB, 0.42016E+00_JPRB, 0.42407E+00_JPRB ,&
&  0.43439E+00_JPRB, 0.43788E+00_JPRB, 0.43639E+00_JPRB, 0.43415E+00_JPRB, 0.42975E+00_JPRB, 0.42379E+00_JPRB, 0.41553E+00_JPRB ,&
&  0.40836E+00_JPRB, 0.39710E+00_JPRB, 0.38279E+00_JPRB, 0.37284E+00_JPRB, 0.37756E+00_JPRB, 0.38250E+00_JPRB, 0.38209E+00_JPRB ,&
&  0.38089E+00_JPRB, 0.38647E+00_JPRB, 0.38872E+00_JPRB, 0.39850E+00_JPRB, 0.40542E+00_JPRB, 0.41953E+00_JPRB, 0.44229E+00_JPRB ,&
&  0.51341E+00_JPRB, 0.78128E+00_JPRB, 0.81302E+00_JPRB, 0.12805E+01_JPRB, 0.14482E+01_JPRB, 0.13632E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.10434E+01_JPRB, 0.97033E+00_JPRB, 0.85022E+00_JPRB, 0.75140E+00_JPRB, 0.68628E+00_JPRB, 0.63787E+00_JPRB, 0.59525E+00_JPRB ,&
&  0.56087E+00_JPRB, 0.53884E+00_JPRB, 0.52595E+00_JPRB, 0.51451E+00_JPRB, 0.51208E+00_JPRB, 0.51821E+00_JPRB, 0.53338E+00_JPRB ,&
&  0.55803E+00_JPRB, 0.59713E+00_JPRB, 0.67273E+00_JPRB, 0.74279E+00_JPRB, 0.73079E+00_JPRB, 0.76303E+00_JPRB, 0.84923E+00_JPRB ,&
&  0.88034E+00_JPRB, 0.94814E+00_JPRB, 0.79053E+00_JPRB, 0.68541E+00_JPRB, 0.64659E+00_JPRB, 0.65006E+00_JPRB, 0.64571E+00_JPRB ,&
&  0.62494E+00_JPRB, 0.60910E+00_JPRB, 0.58752E+00_JPRB, 0.56513E+00_JPRB, 0.54803E+00_JPRB, 0.53462E+00_JPRB, 0.51619E+00_JPRB ,&
&  0.50119E+00_JPRB, 0.49587E+00_JPRB, 0.49397E+00_JPRB, 0.49261E+00_JPRB, 0.49358E+00_JPRB, 0.49689E+00_JPRB, 0.49957E+00_JPRB ,&
&  0.50208E+00_JPRB, 0.49407E+00_JPRB, 0.47125E+00_JPRB, 0.45832E+00_JPRB, 0.45997E+00_JPRB, 0.46451E+00_JPRB, 0.46362E+00_JPRB ,&
&  0.48808E+00_JPRB, 0.50049E+00_JPRB, 0.50006E+00_JPRB, 0.50720E+00_JPRB, 0.51511E+00_JPRB, 0.57388E+00_JPRB, 0.52651E+00_JPRB ,&
&  0.50924E+00_JPRB, 0.50045E+00_JPRB, 0.49979E+00_JPRB, 0.50010E+00_JPRB, 0.48056E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.43656E+00_JPRB, 0.42501E+00_JPRB, 0.41929E+00_JPRB, 0.41936E+00_JPRB, 0.41838E+00_JPRB, 0.41766E+00_JPRB, 0.42280E+00_JPRB ,&
&  0.42505E+00_JPRB, 0.42524E+00_JPRB, 0.42349E+00_JPRB, 0.42097E+00_JPRB, 0.41487E+00_JPRB, 0.40830E+00_JPRB, 0.40118E+00_JPRB ,&
&  0.39723E+00_JPRB, 0.38224E+00_JPRB, 0.37469E+00_JPRB, 0.38301E+00_JPRB, 0.38725E+00_JPRB, 0.38641E+00_JPRB, 0.38681E+00_JPRB ,&
&  0.38655E+00_JPRB, 0.38983E+00_JPRB, 0.38998E+00_JPRB, 0.39523E+00_JPRB, 0.40493E+00_JPRB, 0.42695E+00_JPRB, 0.45384E+00_JPRB ,&
&  0.54419E+00_JPRB, 0.72715E+00_JPRB, 0.79666E+00_JPRB, 0.14143E+01_JPRB, 0.15387E+01_JPRB, 0.14460E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.10659E+01_JPRB, 0.10521E+01_JPRB, 0.92660E+00_JPRB, 0.81998E+00_JPRB, 0.75268E+00_JPRB, 0.69104E+00_JPRB, 0.62040E+00_JPRB ,&
&  0.56191E+00_JPRB, 0.53064E+00_JPRB, 0.52089E+00_JPRB, 0.52617E+00_JPRB, 0.53860E+00_JPRB, 0.54972E+00_JPRB, 0.56756E+00_JPRB ,&
&  0.60509E+00_JPRB, 0.67220E+00_JPRB, 0.76442E+00_JPRB, 0.76969E+00_JPRB, 0.75795E+00_JPRB, 0.78576E+00_JPRB, 0.81510E+00_JPRB ,&
&  0.86218E+00_JPRB, 0.83627E+00_JPRB, 0.70760E+00_JPRB, 0.64489E+00_JPRB, 0.62329E+00_JPRB, 0.63697E+00_JPRB, 0.62934E+00_JPRB ,&
&  0.60788E+00_JPRB, 0.58964E+00_JPRB, 0.56653E+00_JPRB, 0.55076E+00_JPRB, 0.53939E+00_JPRB, 0.52332E+00_JPRB, 0.51352E+00_JPRB ,&
&  0.51097E+00_JPRB, 0.51076E+00_JPRB, 0.51527E+00_JPRB, 0.51663E+00_JPRB, 0.51824E+00_JPRB, 0.51708E+00_JPRB, 0.51388E+00_JPRB ,&
&  0.50513E+00_JPRB, 0.48690E+00_JPRB, 0.46918E+00_JPRB, 0.47245E+00_JPRB, 0.48715E+00_JPRB, 0.48917E+00_JPRB, 0.48631E+00_JPRB ,&
&  0.50287E+00_JPRB, 0.51346E+00_JPRB, 0.51519E+00_JPRB, 0.50720E+00_JPRB, 0.52059E+00_JPRB, 0.55116E+00_JPRB, 0.51717E+00_JPRB ,&
&  0.50985E+00_JPRB, 0.50036E+00_JPRB, 0.49522E+00_JPRB, 0.49086E+00_JPRB, 0.46529E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.44064E+00_JPRB, 0.42187E+00_JPRB, 0.41364E+00_JPRB, 0.40849E+00_JPRB, 0.40768E+00_JPRB, 0.41138E+00_JPRB, 0.41470E+00_JPRB ,&
&  0.41295E+00_JPRB, 0.41205E+00_JPRB, 0.41285E+00_JPRB, 0.40953E+00_JPRB, 0.40016E+00_JPRB, 0.39173E+00_JPRB, 0.38461E+00_JPRB ,&
&  0.38522E+00_JPRB, 0.38758E+00_JPRB, 0.39848E+00_JPRB, 0.40552E+00_JPRB, 0.40686E+00_JPRB, 0.40483E+00_JPRB, 0.40507E+00_JPRB ,&
&  0.40754E+00_JPRB, 0.40602E+00_JPRB, 0.40333E+00_JPRB, 0.41190E+00_JPRB, 0.42717E+00_JPRB, 0.45678E+00_JPRB, 0.50106E+00_JPRB ,&
&  0.62574E+00_JPRB, 0.75678E+00_JPRB, 0.84083E+00_JPRB, 0.12226E+01_JPRB, 0.14416E+01_JPRB, 0.13493E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.92066E+00_JPRB, 0.95406E+00_JPRB, 0.85524E+00_JPRB, 0.77252E+00_JPRB, 0.70323E+00_JPRB, 0.64402E+00_JPRB ,&
&  0.59583E+00_JPRB, 0.56999E+00_JPRB, 0.57568E+00_JPRB, 0.58899E+00_JPRB, 0.59671E+00_JPRB, 0.60332E+00_JPRB, 0.63327E+00_JPRB ,&
&  0.68722E+00_JPRB, 0.77255E+00_JPRB, 0.86387E+00_JPRB, 0.84351E+00_JPRB, 0.74902E+00_JPRB, 0.75904E+00_JPRB, 0.76612E+00_JPRB ,&
&  0.80253E+00_JPRB, 0.73267E+00_JPRB, 0.66434E+00_JPRB, 0.63053E+00_JPRB, 0.59612E+00_JPRB, 0.62272E+00_JPRB, 0.60861E+00_JPRB ,&
&  0.59224E+00_JPRB, 0.56959E+00_JPRB, 0.54814E+00_JPRB, 0.53580E+00_JPRB, 0.52600E+00_JPRB, 0.51929E+00_JPRB, 0.51362E+00_JPRB ,&
&  0.51150E+00_JPRB, 0.51154E+00_JPRB, 0.51923E+00_JPRB, 0.52217E+00_JPRB, 0.52001E+00_JPRB, 0.50876E+00_JPRB, 0.49262E+00_JPRB ,&
&  0.48185E+00_JPRB, 0.48649E+00_JPRB, 0.48566E+00_JPRB, 0.49949E+00_JPRB, 0.51195E+00_JPRB, 0.51662E+00_JPRB, 0.50649E+00_JPRB ,&
&  0.50656E+00_JPRB, 0.52135E+00_JPRB, 0.52855E+00_JPRB, 0.53378E+00_JPRB, 0.53646E+00_JPRB, 0.53704E+00_JPRB, 0.51045E+00_JPRB ,&
&  0.49837E+00_JPRB, 0.48897E+00_JPRB, 0.49641E+00_JPRB, 0.48933E+00_JPRB, 0.46099E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.44817E+00_JPRB, 0.43973E+00_JPRB, 0.40250E+00_JPRB, 0.40355E+00_JPRB, 0.40312E+00_JPRB, 0.40698E+00_JPRB, 0.40188E+00_JPRB ,&
&  0.39819E+00_JPRB, 0.39946E+00_JPRB, 0.40181E+00_JPRB, 0.39539E+00_JPRB, 0.37621E+00_JPRB, 0.36787E+00_JPRB, 0.37161E+00_JPRB ,&
&  0.38189E+00_JPRB, 0.40174E+00_JPRB, 0.42578E+00_JPRB, 0.42902E+00_JPRB, 0.43194E+00_JPRB, 0.42979E+00_JPRB, 0.42647E+00_JPRB ,&
&  0.42828E+00_JPRB, 0.43247E+00_JPRB, 0.43913E+00_JPRB, 0.45573E+00_JPRB, 0.48450E+00_JPRB, 0.52757E+00_JPRB, 0.59556E+00_JPRB ,&
&  0.76185E+00_JPRB, 0.81559E+00_JPRB, 0.43386E+00_JPRB, 0.76343E+00_JPRB, 0.11127E+01_JPRB, 0.11537E+01_JPRB, 0.10755E+01_JPRB ,&
&  0.98572E+00_JPRB, 0.86581E+00_JPRB, 0.84690E+00_JPRB, 0.83801E+00_JPRB, 0.76193E+00_JPRB, 0.71851E+00_JPRB, 0.67613E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.63494E+00_JPRB, 0.65427E+00_JPRB, 0.66278E+00_JPRB, 0.66678E+00_JPRB, 0.69535E+00_JPRB, 0.74714E+00_JPRB ,&
&  0.81290E+00_JPRB, 0.89923E+00_JPRB, 0.96365E+00_JPRB, 0.83802E+00_JPRB, 0.76774E+00_JPRB, 0.74256E+00_JPRB, 0.71012E+00_JPRB ,&
&  0.67678E+00_JPRB, 0.66863E+00_JPRB, 0.64771E+00_JPRB, 0.61791E+00_JPRB, 0.58469E+00_JPRB, 0.57372E+00_JPRB, 0.58735E+00_JPRB ,&
&  0.57019E+00_JPRB, 0.54219E+00_JPRB, 0.52414E+00_JPRB, 0.51465E+00_JPRB, 0.50645E+00_JPRB, 0.50250E+00_JPRB, 0.49735E+00_JPRB ,&
&  0.49652E+00_JPRB, 0.50226E+00_JPRB, 0.50769E+00_JPRB, 0.50535E+00_JPRB, 0.49610E+00_JPRB, 0.48278E+00_JPRB, 0.47220E+00_JPRB ,&
&  0.47760E+00_JPRB, 0.49442E+00_JPRB, 0.51645E+00_JPRB, 0.52174E+00_JPRB, 0.52409E+00_JPRB, 0.53156E+00_JPRB, 0.53095E+00_JPRB ,&
&  0.53341E+00_JPRB, 0.54419E+00_JPRB, 0.53243E+00_JPRB, 0.52681E+00_JPRB, 0.54336E+00_JPRB, 0.53301E+00_JPRB, 0.52792E+00_JPRB ,&
&  0.56715E+00_JPRB, 0.64936E+00_JPRB, 0.78384E+00_JPRB, 0.53004E+00_JPRB, 0.47840E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.45757E+00_JPRB, 0.42530E+00_JPRB, 0.39104E+00_JPRB, 0.38969E+00_JPRB, 0.40516E+00_JPRB, 0.40773E+00_JPRB, 0.40346E+00_JPRB ,&
&  0.39253E+00_JPRB, 0.38233E+00_JPRB, 0.36972E+00_JPRB, 0.36231E+00_JPRB, 0.36572E+00_JPRB, 0.37402E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.42921E+00_JPRB, 0.45301E+00_JPRB, 0.47281E+00_JPRB, 0.47671E+00_JPRB, 0.46922E+00_JPRB, 0.47045E+00_JPRB, 0.47714E+00_JPRB ,&
&  0.48041E+00_JPRB, 0.49495E+00_JPRB, 0.51502E+00_JPRB, 0.54337E+00_JPRB, 0.58523E+00_JPRB, 0.65217E+00_JPRB, 0.75733E+00_JPRB ,&
&  0.92127E+00_JPRB, 0.64062E+00_JPRB, 0.48224E+00_JPRB, 0.89048E+00_JPRB, 0.10697E+01_JPRB, 0.10887E+01_JPRB, 0.92066E+00_JPRB ,&
&  0.86870E+00_JPRB, 0.82081E+00_JPRB, 0.80879E+00_JPRB, 0.83289E+00_JPRB, 0.76259E+00_JPRB, 0.73872E+00_JPRB, 0.71306E+00_JPRB ,&
&  0.69934E+00_JPRB, 0.71239E+00_JPRB, 0.73776E+00_JPRB, 0.75611E+00_JPRB, 0.78266E+00_JPRB, 0.84152E+00_JPRB, 0.90266E+00_JPRB ,&
&  0.98220E+00_JPRB, 0.10478E+01_JPRB, 0.10820E+01_JPRB, 0.88759E+00_JPRB, 0.79289E+00_JPRB, 0.72657E+00_JPRB, 0.69017E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.62749E+00_JPRB, 0.62062E+00_JPRB, 0.59831E+00_JPRB, 0.57187E+00_JPRB, 0.55121E+00_JPRB, 0.56330E+00_JPRB ,&
&  0.53615E+00_JPRB, 0.50869E+00_JPRB, 0.49014E+00_JPRB, 0.47972E+00_JPRB, 0.47682E+00_JPRB, 0.47704E+00_JPRB, 0.47192E+00_JPRB ,&
&  0.47632E+00_JPRB, 0.47826E+00_JPRB, 0.48323E+00_JPRB, 0.47707E+00_JPRB, 0.47302E+00_JPRB, 0.47235E+00_JPRB, 0.46416E+00_JPRB ,&
&  0.48446E+00_JPRB, 0.51179E+00_JPRB, 0.52054E+00_JPRB, 0.53514E+00_JPRB, 0.53326E+00_JPRB, 0.52507E+00_JPRB, 0.53367E+00_JPRB ,&
&  0.53353E+00_JPRB, 0.54555E+00_JPRB, 0.53810E+00_JPRB, 0.54586E+00_JPRB, 0.53956E+00_JPRB, 0.54112E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.60404E+00_JPRB, 0.66254E+00_JPRB, 0.66570E+00_JPRB, 0.52124E+00_JPRB, 0.47827E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.43580E+00_JPRB, 0.41166E+00_JPRB, 0.37513E+00_JPRB, 0.36173E+00_JPRB, 0.38085E+00_JPRB, 0.39514E+00_JPRB, 0.38774E+00_JPRB ,&
&  0.38018E+00_JPRB, 0.37130E+00_JPRB, 0.36660E+00_JPRB, 0.37266E+00_JPRB, 0.40053E+00_JPRB, 0.43799E+00_JPRB, 0.46297E+00_JPRB ,&
&  0.48826E+00_JPRB, 0.50795E+00_JPRB, 0.52522E+00_JPRB, 0.53145E+00_JPRB, 0.52973E+00_JPRB, 0.53620E+00_JPRB, 0.54430E+00_JPRB ,&
&  0.55955E+00_JPRB, 0.58378E+00_JPRB, 0.61689E+00_JPRB, 0.66493E+00_JPRB, 0.72692E+00_JPRB, 0.80084E+00_JPRB, 0.87251E+00_JPRB ,&
&  0.64361E+00_JPRB, 0.37849E+00_JPRB, 0.82927E+00_JPRB, 0.97858E+00_JPRB, 0.92227E+00_JPRB, 0.90260E+00_JPRB, 0.83660E+00_JPRB ,&
&  0.78331E+00_JPRB, 0.78640E+00_JPRB, 0.75830E+00_JPRB, 0.79904E+00_JPRB, 0.75044E+00_JPRB, 0.75478E+00_JPRB, 0.76702E+00_JPRB ,&
&  0.79165E+00_JPRB, 0.82361E+00_JPRB, 0.86291E+00_JPRB, 0.90567E+00_JPRB, 0.95380E+00_JPRB, 0.10326E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.12458E+01_JPRB, 0.12683E+01_JPRB, 0.98680E+00_JPRB, 0.85895E+00_JPRB, 0.78669E+00_JPRB, 0.72236E+00_JPRB ,&
&  0.65486E+00_JPRB, 0.57817E+00_JPRB, 0.56268E+00_JPRB, 0.57744E+00_JPRB, 0.56435E+00_JPRB, 0.53075E+00_JPRB, 0.53762E+00_JPRB ,&
&  0.50055E+00_JPRB, 0.46939E+00_JPRB, 0.44979E+00_JPRB, 0.43571E+00_JPRB, 0.44141E+00_JPRB, 0.44564E+00_JPRB, 0.44744E+00_JPRB ,&
&  0.44068E+00_JPRB, 0.44295E+00_JPRB, 0.45516E+00_JPRB, 0.45836E+00_JPRB, 0.46490E+00_JPRB, 0.47446E+00_JPRB, 0.47873E+00_JPRB ,&
&  0.50643E+00_JPRB, 0.51541E+00_JPRB, 0.52576E+00_JPRB, 0.54005E+00_JPRB, 0.54554E+00_JPRB, 0.52905E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.52638E+00_JPRB, 0.53219E+00_JPRB, 0.53081E+00_JPRB, 0.54060E+00_JPRB, 0.54368E+00_JPRB, 0.55273E+00_JPRB, 0.57226E+00_JPRB ,&
&  0.59807E+00_JPRB, 0.60566E+00_JPRB, 0.57558E+00_JPRB, 0.49990E+00_JPRB, 0.46523E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.41834E+00_JPRB, 0.39621E+00_JPRB, 0.36164E+00_JPRB, 0.32910E+00_JPRB, 0.34819E+00_JPRB, 0.34564E+00_JPRB, 0.35714E+00_JPRB ,&
&  0.35990E+00_JPRB, 0.38060E+00_JPRB, 0.39990E+00_JPRB, 0.42903E+00_JPRB, 0.46251E+00_JPRB, 0.49416E+00_JPRB, 0.51867E+00_JPRB ,&
&  0.53952E+00_JPRB, 0.56138E+00_JPRB, 0.57842E+00_JPRB, 0.58350E+00_JPRB, 0.59320E+00_JPRB, 0.60936E+00_JPRB, 0.62653E+00_JPRB ,&
&  0.64913E+00_JPRB, 0.68175E+00_JPRB, 0.72887E+00_JPRB, 0.79087E+00_JPRB, 0.86317E+00_JPRB, 0.89025E+00_JPRB, 0.89046E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.80702E+00_JPRB, 0.92223E+00_JPRB, 0.87013E+00_JPRB, 0.83882E+00_JPRB, 0.81533E+00_JPRB, 0.79609E+00_JPRB ,&
&  0.76440E+00_JPRB, 0.72119E+00_JPRB, 0.70825E+00_JPRB, 0.77337E+00_JPRB, 0.78646E+00_JPRB, 0.83729E+00_JPRB, 0.88166E+00_JPRB ,&
&  0.92843E+00_JPRB, 0.97478E+00_JPRB, 0.10264E+01_JPRB, 0.10863E+01_JPRB, 0.11530E+01_JPRB, 0.12209E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.13812E+01_JPRB, 0.14498E+01_JPRB, 0.14663E+01_JPRB, 0.11686E+01_JPRB, 0.10738E+01_JPRB, 0.10175E+01_JPRB, 0.91940E+00_JPRB ,&
&  0.65800E+00_JPRB, 0.54946E+00_JPRB, 0.54979E+00_JPRB, 0.57373E+00_JPRB, 0.55533E+00_JPRB, 0.53373E+00_JPRB, 0.50694E+00_JPRB ,&
&  0.47414E+00_JPRB, 0.44409E+00_JPRB, 0.42817E+00_JPRB, 0.42332E+00_JPRB, 0.43648E+00_JPRB, 0.43626E+00_JPRB, 0.42368E+00_JPRB ,&
&  0.40757E+00_JPRB, 0.41088E+00_JPRB, 0.43661E+00_JPRB, 0.45695E+00_JPRB, 0.48618E+00_JPRB, 0.50406E+00_JPRB, 0.51472E+00_JPRB ,&
&  0.52649E+00_JPRB, 0.51714E+00_JPRB, 0.52752E+00_JPRB, 0.53794E+00_JPRB, 0.53634E+00_JPRB, 0.53076E+00_JPRB, 0.52011E+00_JPRB ,&
&  0.51489E+00_JPRB, 0.52923E+00_JPRB, 0.54131E+00_JPRB, 0.55322E+00_JPRB, 0.55454E+00_JPRB, 0.55189E+00_JPRB, 0.55536E+00_JPRB ,&
&  0.54662E+00_JPRB, 0.52812E+00_JPRB, 0.50571E+00_JPRB, 0.47327E+00_JPRB, 0.44360E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.41047E+00_JPRB, 0.39852E+00_JPRB, 0.38277E+00_JPRB, 0.37329E+00_JPRB, 0.36462E+00_JPRB, 0.35508E+00_JPRB, 0.36342E+00_JPRB ,&
&  0.38712E+00_JPRB, 0.40738E+00_JPRB, 0.44147E+00_JPRB, 0.49189E+00_JPRB, 0.53404E+00_JPRB, 0.55233E+00_JPRB, 0.57365E+00_JPRB ,&
&  0.59085E+00_JPRB, 0.61179E+00_JPRB, 0.62833E+00_JPRB, 0.64124E+00_JPRB, 0.65931E+00_JPRB, 0.68247E+00_JPRB, 0.71390E+00_JPRB ,&
&  0.75402E+00_JPRB, 0.79437E+00_JPRB, 0.84807E+00_JPRB, 0.91933E+00_JPRB, 0.98511E+00_JPRB, 0.99910E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.77693E+00_JPRB, 0.90968E+00_JPRB, 0.89034E+00_JPRB, 0.85081E+00_JPRB, 0.80803E+00_JPRB, 0.77210E+00_JPRB, 0.75269E+00_JPRB ,&
&  0.74234E+00_JPRB, 0.70570E+00_JPRB, 0.71391E+00_JPRB, 0.77619E+00_JPRB, 0.88133E+00_JPRB, 0.94038E+00_JPRB, 0.99349E+00_JPRB ,&
&  0.10501E+01_JPRB, 0.11109E+01_JPRB, 0.11727E+01_JPRB, 0.12390E+01_JPRB, 0.13017E+01_JPRB, 0.13805E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.14984E+01_JPRB, 0.15415E+01_JPRB, 0.15568E+01_JPRB, 0.12877E+01_JPRB, 0.10680E+01_JPRB, 0.94435E+00_JPRB, 0.76238E+00_JPRB ,&
&  0.60574E+00_JPRB, 0.54059E+00_JPRB, 0.56004E+00_JPRB, 0.58550E+00_JPRB, 0.56122E+00_JPRB, 0.52610E+00_JPRB, 0.48642E+00_JPRB ,&
&  0.46849E+00_JPRB, 0.43029E+00_JPRB, 0.41719E+00_JPRB, 0.42421E+00_JPRB, 0.44385E+00_JPRB, 0.42655E+00_JPRB, 0.40357E+00_JPRB ,&
&  0.38734E+00_JPRB, 0.38039E+00_JPRB, 0.41165E+00_JPRB, 0.44838E+00_JPRB, 0.49552E+00_JPRB, 0.51502E+00_JPRB, 0.50797E+00_JPRB ,&
&  0.54482E+00_JPRB, 0.50739E+00_JPRB, 0.50896E+00_JPRB, 0.51885E+00_JPRB, 0.52358E+00_JPRB, 0.52938E+00_JPRB, 0.53849E+00_JPRB ,&
&  0.57178E+00_JPRB, 0.63438E+00_JPRB, 0.65798E+00_JPRB, 0.61230E+00_JPRB, 0.57574E+00_JPRB, 0.53706E+00_JPRB, 0.49357E+00_JPRB ,&
&  0.49710E+00_JPRB, 0.48620E+00_JPRB, 0.47484E+00_JPRB, 0.45830E+00_JPRB, 0.43521E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.46401E+00_JPRB, 0.45192E+00_JPRB, 0.44460E+00_JPRB, 0.43829E+00_JPRB, 0.42307E+00_JPRB, 0.42298E+00_JPRB, 0.41943E+00_JPRB ,&
&  0.43463E+00_JPRB, 0.45319E+00_JPRB, 0.48594E+00_JPRB, 0.52405E+00_JPRB, 0.57195E+00_JPRB, 0.60159E+00_JPRB, 0.63483E+00_JPRB ,&
&  0.65046E+00_JPRB, 0.67286E+00_JPRB, 0.70230E+00_JPRB, 0.72241E+00_JPRB, 0.74863E+00_JPRB, 0.76100E+00_JPRB, 0.79321E+00_JPRB ,&
&  0.82055E+00_JPRB, 0.85309E+00_JPRB, 0.90179E+00_JPRB, 0.94294E+00_JPRB, 0.97798E+00_JPRB, 0.96649E+00_JPRB, 0.75656E+00_JPRB ,&
&  0.91418E+00_JPRB, 0.90807E+00_JPRB, 0.88386E+00_JPRB, 0.83394E+00_JPRB, 0.77844E+00_JPRB, 0.73179E+00_JPRB, 0.71948E+00_JPRB ,&
&  0.71885E+00_JPRB, 0.72023E+00_JPRB, 0.74184E+00_JPRB, 0.78256E+00_JPRB, 0.93303E+00_JPRB, 0.10152E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11963E+01_JPRB, 0.12637E+01_JPRB, 0.13386E+01_JPRB, 0.13997E+01_JPRB, 0.14688E+01_JPRB, 0.15429E+01_JPRB ,&
&  0.15853E+01_JPRB, 0.16202E+01_JPRB, 0.15603E+01_JPRB, 0.12392E+01_JPRB, 0.10597E+01_JPRB, 0.91478E+00_JPRB, 0.72587E+00_JPRB ,&
&  0.61346E+00_JPRB, 0.58078E+00_JPRB, 0.60251E+00_JPRB, 0.62311E+00_JPRB, 0.58369E+00_JPRB, 0.53689E+00_JPRB, 0.50658E+00_JPRB ,&
&  0.46609E+00_JPRB, 0.45884E+00_JPRB, 0.44799E+00_JPRB, 0.46479E+00_JPRB, 0.47280E+00_JPRB, 0.45417E+00_JPRB, 0.44205E+00_JPRB ,&
&  0.42181E+00_JPRB, 0.40493E+00_JPRB, 0.42246E+00_JPRB, 0.45638E+00_JPRB, 0.53224E+00_JPRB, 0.60086E+00_JPRB, 0.61551E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.44846E+00_JPRB, 0.47330E+00_JPRB, 0.49762E+00_JPRB, 0.54144E+00_JPRB, 0.58510E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.86525E+00_JPRB, 0.12074E+01_JPRB, 0.10762E+01_JPRB, 0.81825E+00_JPRB, 0.90712E+00_JPRB, 0.83193E+00_JPRB, 0.44515E+00_JPRB ,&
&  0.44720E+00_JPRB, 0.44860E+00_JPRB, 0.46384E+00_JPRB, 0.46824E+00_JPRB, 0.47123E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.53672E+00_JPRB, 0.53112E+00_JPRB, 0.53030E+00_JPRB, 0.52100E+00_JPRB, 0.51575E+00_JPRB, 0.51619E+00_JPRB, 0.50353E+00_JPRB ,&
&  0.51262E+00_JPRB, 0.52791E+00_JPRB, 0.53319E+00_JPRB, 0.54863E+00_JPRB, 0.55684E+00_JPRB, 0.58702E+00_JPRB, 0.63076E+00_JPRB ,&
&  0.65682E+00_JPRB, 0.71975E+00_JPRB, 0.74459E+00_JPRB, 0.77678E+00_JPRB, 0.80165E+00_JPRB, 0.82657E+00_JPRB, 0.84647E+00_JPRB ,&
&  0.87839E+00_JPRB, 0.91440E+00_JPRB, 0.95564E+00_JPRB, 0.98110E+00_JPRB, 0.96944E+00_JPRB, 0.10257E+01_JPRB, 0.80631E+00_JPRB ,&
&  0.92974E+00_JPRB, 0.91122E+00_JPRB, 0.83194E+00_JPRB, 0.78597E+00_JPRB, 0.74249E+00_JPRB, 0.71364E+00_JPRB, 0.69480E+00_JPRB ,&
&  0.69550E+00_JPRB, 0.70607E+00_JPRB, 0.73832E+00_JPRB, 0.81907E+00_JPRB, 0.92032E+00_JPRB, 0.10142E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.11739E+01_JPRB, 0.12412E+01_JPRB, 0.13161E+01_JPRB, 0.13975E+01_JPRB, 0.14544E+01_JPRB, 0.15126E+01_JPRB ,&
&  0.15323E+01_JPRB, 0.15466E+01_JPRB, 0.14411E+01_JPRB, 0.12759E+01_JPRB, 0.11330E+01_JPRB, 0.98001E+00_JPRB, 0.80239E+00_JPRB ,&
&  0.64995E+00_JPRB, 0.61403E+00_JPRB, 0.62754E+00_JPRB, 0.67108E+00_JPRB, 0.62213E+00_JPRB, 0.58348E+00_JPRB, 0.56275E+00_JPRB ,&
&  0.53614E+00_JPRB, 0.49443E+00_JPRB, 0.50712E+00_JPRB, 0.53867E+00_JPRB, 0.54677E+00_JPRB, 0.53737E+00_JPRB, 0.51848E+00_JPRB ,&
&  0.49537E+00_JPRB, 0.46324E+00_JPRB, 0.45680E+00_JPRB, 0.45808E+00_JPRB, 0.46876E+00_JPRB, 0.49476E+00_JPRB, 0.44199E+00_JPRB ,&
&  0.49754E+00_JPRB, 0.44508E+00_JPRB, 0.46574E+00_JPRB, 0.51292E+00_JPRB, 0.59829E+00_JPRB, 0.77194E+00_JPRB, 0.10981E+01_JPRB ,&
&  0.16828E+01_JPRB, 0.19441E+01_JPRB, 0.22019E+01_JPRB, 0.27576E+01_JPRB, 0.24049E+01_JPRB, 0.17654E+01_JPRB, 0.50042E+00_JPRB ,&
&  0.49707E+00_JPRB, 0.50085E+00_JPRB, 0.52091E+00_JPRB, 0.52903E+00_JPRB, 0.53358E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.57396E+00_JPRB, 0.58200E+00_JPRB, 0.59317E+00_JPRB, 0.60499E+00_JPRB, 0.61944E+00_JPRB, 0.63379E+00_JPRB, 0.64484E+00_JPRB ,&
&  0.65623E+00_JPRB, 0.67054E+00_JPRB, 0.68422E+00_JPRB, 0.69334E+00_JPRB, 0.70240E+00_JPRB, 0.72079E+00_JPRB, 0.74238E+00_JPRB ,&
&  0.74899E+00_JPRB, 0.78362E+00_JPRB, 0.79478E+00_JPRB, 0.82434E+00_JPRB, 0.84952E+00_JPRB, 0.88147E+00_JPRB, 0.90221E+00_JPRB ,&
&  0.91885E+00_JPRB, 0.94677E+00_JPRB, 0.94797E+00_JPRB, 0.93192E+00_JPRB, 0.92845E+00_JPRB, 0.96223E+00_JPRB, 0.76585E+00_JPRB ,&
&  0.97675E+00_JPRB, 0.93700E+00_JPRB, 0.81927E+00_JPRB, 0.75855E+00_JPRB, 0.71131E+00_JPRB, 0.68632E+00_JPRB, 0.67259E+00_JPRB ,&
&  0.66816E+00_JPRB, 0.69277E+00_JPRB, 0.68756E+00_JPRB, 0.71107E+00_JPRB, 0.81626E+00_JPRB, 0.93547E+00_JPRB, 0.10156E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10836E+01_JPRB, 0.10970E+01_JPRB, 0.11204E+01_JPRB, 0.11749E+01_JPRB, 0.12251E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12417E+01_JPRB, 0.12143E+01_JPRB, 0.11204E+01_JPRB, 0.11720E+01_JPRB, 0.11551E+01_JPRB, 0.10226E+01_JPRB, 0.84427E+00_JPRB ,&
&  0.63182E+00_JPRB, 0.65011E+00_JPRB, 0.69269E+00_JPRB, 0.76064E+00_JPRB, 0.72668E+00_JPRB, 0.69387E+00_JPRB, 0.62537E+00_JPRB ,&
&  0.59729E+00_JPRB, 0.58513E+00_JPRB, 0.60648E+00_JPRB, 0.63506E+00_JPRB, 0.65745E+00_JPRB, 0.64847E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.56957E+00_JPRB, 0.52196E+00_JPRB, 0.49393E+00_JPRB, 0.48022E+00_JPRB, 0.45831E+00_JPRB, 0.40374E+00_JPRB, 0.45299E+00_JPRB ,&
&  0.43587E+00_JPRB, 0.41318E+00_JPRB, 0.46969E+00_JPRB, 0.38112E+00_JPRB, 0.56640E+00_JPRB, 0.73173E+00_JPRB, 0.99579E+00_JPRB ,&
&  0.12043E+01_JPRB, 0.17138E+01_JPRB, 0.21699E+01_JPRB, 0.16429E+01_JPRB, 0.11218E+01_JPRB, 0.62553E+00_JPRB, 0.53533E+00_JPRB ,&
&  0.54197E+00_JPRB, 0.54672E+00_JPRB, 0.55471E+00_JPRB, 0.55422E+00_JPRB, 0.56470E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.58917E+00_JPRB, 0.60050E+00_JPRB, 0.61618E+00_JPRB, 0.63736E+00_JPRB, 0.65735E+00_JPRB, 0.67213E+00_JPRB, 0.68129E+00_JPRB ,&
&  0.69786E+00_JPRB, 0.72030E+00_JPRB, 0.73283E+00_JPRB, 0.73895E+00_JPRB, 0.75286E+00_JPRB, 0.76599E+00_JPRB, 0.79006E+00_JPRB ,&
&  0.80481E+00_JPRB, 0.83874E+00_JPRB, 0.86360E+00_JPRB, 0.88897E+00_JPRB, 0.90828E+00_JPRB, 0.93206E+00_JPRB, 0.95425E+00_JPRB ,&
&  0.97193E+00_JPRB, 0.10091E+01_JPRB, 0.10431E+01_JPRB, 0.10699E+01_JPRB, 0.11093E+01_JPRB, 0.11271E+01_JPRB, 0.91271E+00_JPRB ,&
&  0.10827E+01_JPRB, 0.10172E+01_JPRB, 0.87988E+00_JPRB, 0.77623E+00_JPRB, 0.71051E+00_JPRB, 0.65872E+00_JPRB, 0.62927E+00_JPRB ,&
&  0.62404E+00_JPRB, 0.63347E+00_JPRB, 0.64202E+00_JPRB, 0.63144E+00_JPRB, 0.62564E+00_JPRB, 0.68013E+00_JPRB, 0.80272E+00_JPRB ,&
&  0.87438E+00_JPRB, 0.90638E+00_JPRB, 0.92825E+00_JPRB, 0.91865E+00_JPRB, 0.94836E+00_JPRB, 0.99502E+00_JPRB, 0.10219E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.95318E+00_JPRB, 0.85723E+00_JPRB, 0.10138E+01_JPRB, 0.10432E+01_JPRB, 0.10301E+01_JPRB, 0.92790E+00_JPRB ,&
&  0.78133E+00_JPRB, 0.71026E+00_JPRB, 0.73984E+00_JPRB, 0.83520E+00_JPRB, 0.81915E+00_JPRB, 0.80152E+00_JPRB, 0.75266E+00_JPRB ,&
&  0.72342E+00_JPRB, 0.70333E+00_JPRB, 0.71000E+00_JPRB, 0.73159E+00_JPRB, 0.74532E+00_JPRB, 0.73887E+00_JPRB, 0.68274E+00_JPRB ,&
&  0.63233E+00_JPRB, 0.58291E+00_JPRB, 0.54926E+00_JPRB, 0.51930E+00_JPRB, 0.49440E+00_JPRB, 0.45326E+00_JPRB, 0.47043E+00_JPRB ,&
&  0.44101E+00_JPRB, 0.35740E+00_JPRB, 0.38101E+00_JPRB, 0.45707E+00_JPRB, 0.49911E+00_JPRB, 0.57852E+00_JPRB, 0.74066E+00_JPRB ,&
&  0.95025E+00_JPRB, 0.10945E+01_JPRB, 0.10705E+01_JPRB, 0.80709E+00_JPRB, 0.62508E+00_JPRB, 0.56692E+00_JPRB, 0.56493E+00_JPRB ,&
&  0.57234E+00_JPRB, 0.57041E+00_JPRB, 0.57268E+00_JPRB, 0.57668E+00_JPRB, 0.58396E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.57144E+00_JPRB, 0.56984E+00_JPRB, 0.57585E+00_JPRB, 0.58860E+00_JPRB, 0.59378E+00_JPRB, 0.59670E+00_JPRB, 0.59767E+00_JPRB ,&
&  0.61238E+00_JPRB, 0.62945E+00_JPRB, 0.64522E+00_JPRB, 0.66111E+00_JPRB, 0.67409E+00_JPRB, 0.68584E+00_JPRB, 0.71730E+00_JPRB ,&
&  0.73226E+00_JPRB, 0.75400E+00_JPRB, 0.76872E+00_JPRB, 0.78366E+00_JPRB, 0.79595E+00_JPRB, 0.78883E+00_JPRB, 0.79116E+00_JPRB ,&
&  0.79873E+00_JPRB, 0.82231E+00_JPRB, 0.84802E+00_JPRB, 0.89113E+00_JPRB, 0.94805E+00_JPRB, 0.95864E+00_JPRB, 0.10686E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.11320E+01_JPRB, 0.97483E+00_JPRB, 0.82531E+00_JPRB, 0.78674E+00_JPRB, 0.70785E+00_JPRB, 0.66566E+00_JPRB ,&
&  0.66368E+00_JPRB, 0.63976E+00_JPRB, 0.61276E+00_JPRB, 0.58130E+00_JPRB, 0.54740E+00_JPRB, 0.55727E+00_JPRB, 0.58533E+00_JPRB ,&
&  0.67499E+00_JPRB, 0.72977E+00_JPRB, 0.80727E+00_JPRB, 0.88065E+00_JPRB, 0.96784E+00_JPRB, 0.99919E+00_JPRB, 0.10058E+01_JPRB ,&
&  0.99653E+00_JPRB, 0.96240E+00_JPRB, 0.88417E+00_JPRB, 0.98667E+00_JPRB, 0.10745E+01_JPRB, 0.11203E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.92899E+00_JPRB, 0.91932E+00_JPRB, 0.91867E+00_JPRB, 0.89871E+00_JPRB, 0.90621E+00_JPRB, 0.86750E+00_JPRB ,&
&  0.83291E+00_JPRB, 0.79726E+00_JPRB, 0.77994E+00_JPRB, 0.78339E+00_JPRB, 0.78157E+00_JPRB, 0.76292E+00_JPRB, 0.68977E+00_JPRB ,&
&  0.64485E+00_JPRB, 0.60896E+00_JPRB, 0.58892E+00_JPRB, 0.56723E+00_JPRB, 0.51453E+00_JPRB, 0.80087E+00_JPRB, 0.62188E+00_JPRB ,&
&  0.49493E+00_JPRB, 0.48406E+00_JPRB, 0.36921E+00_JPRB, 0.47148E+00_JPRB, 0.50808E+00_JPRB, 0.55718E+00_JPRB, 0.62432E+00_JPRB ,&
&  0.63877E+00_JPRB, 0.62667E+00_JPRB, 0.56472E+00_JPRB, 0.50459E+00_JPRB, 0.51793E+00_JPRB, 0.51959E+00_JPRB, 0.52237E+00_JPRB ,&
&  0.53629E+00_JPRB, 0.54464E+00_JPRB, 0.55499E+00_JPRB, 0.57376E+00_JPRB, 0.56810E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.56159E+00_JPRB, 0.55036E+00_JPRB, 0.53928E+00_JPRB, 0.52819E+00_JPRB, 0.53333E+00_JPRB, 0.53305E+00_JPRB, 0.54063E+00_JPRB ,&
&  0.54924E+00_JPRB, 0.54648E+00_JPRB, 0.54878E+00_JPRB, 0.54895E+00_JPRB, 0.56016E+00_JPRB, 0.57022E+00_JPRB, 0.59736E+00_JPRB ,&
&  0.62254E+00_JPRB, 0.63525E+00_JPRB, 0.65332E+00_JPRB, 0.67053E+00_JPRB, 0.68337E+00_JPRB, 0.69634E+00_JPRB, 0.67985E+00_JPRB ,&
&  0.66147E+00_JPRB, 0.66151E+00_JPRB, 0.69474E+00_JPRB, 0.80638E+00_JPRB, 0.10214E+01_JPRB, 0.13044E+01_JPRB, 0.18581E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.11322E+01_JPRB, 0.10091E+01_JPRB, 0.88431E+00_JPRB, 0.91783E+00_JPRB, 0.98497E+00_JPRB, 0.97711E+00_JPRB ,&
&  0.96554E+00_JPRB, 0.95426E+00_JPRB, 0.93788E+00_JPRB, 0.91399E+00_JPRB, 0.86323E+00_JPRB, 0.79664E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.72618E+00_JPRB, 0.69980E+00_JPRB, 0.78955E+00_JPRB, 0.10161E+01_JPRB, 0.11636E+01_JPRB, 0.14249E+01_JPRB, 0.13771E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.13534E+01_JPRB, 0.12741E+01_JPRB, 0.13503E+01_JPRB, 0.13754E+01_JPRB, 0.14216E+01_JPRB, 0.14101E+01_JPRB ,&
&  0.14470E+01_JPRB, 0.12524E+01_JPRB, 0.89940E+00_JPRB, 0.99698E+00_JPRB, 0.98235E+00_JPRB, 0.10110E+01_JPRB, 0.96643E+00_JPRB ,&
&  0.92512E+00_JPRB, 0.86945E+00_JPRB, 0.83400E+00_JPRB, 0.80949E+00_JPRB, 0.78305E+00_JPRB, 0.73835E+00_JPRB, 0.65749E+00_JPRB ,&
&  0.62468E+00_JPRB, 0.60240E+00_JPRB, 0.59203E+00_JPRB, 0.56937E+00_JPRB, 0.58426E+00_JPRB, 0.60874E+00_JPRB, 0.58720E+00_JPRB ,&
&  0.52424E+00_JPRB, 0.56924E+00_JPRB, 0.46044E+00_JPRB, 0.52754E+00_JPRB, 0.53593E+00_JPRB, 0.52118E+00_JPRB, 0.52522E+00_JPRB ,&
&  0.50385E+00_JPRB, 0.48535E+00_JPRB, 0.47135E+00_JPRB, 0.47423E+00_JPRB, 0.48859E+00_JPRB, 0.51167E+00_JPRB, 0.51851E+00_JPRB ,&
&  0.52889E+00_JPRB, 0.54328E+00_JPRB, 0.56191E+00_JPRB, 0.58234E+00_JPRB, 0.57965E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.58799E+00_JPRB, 0.58842E+00_JPRB, 0.58915E+00_JPRB, 0.59569E+00_JPRB, 0.59589E+00_JPRB, 0.59089E+00_JPRB, 0.57792E+00_JPRB ,&
&  0.56932E+00_JPRB, 0.55365E+00_JPRB, 0.54315E+00_JPRB, 0.53544E+00_JPRB, 0.54543E+00_JPRB, 0.57294E+00_JPRB, 0.61687E+00_JPRB ,&
&  0.62947E+00_JPRB, 0.62762E+00_JPRB, 0.60549E+00_JPRB, 0.59407E+00_JPRB, 0.61701E+00_JPRB, 0.62210E+00_JPRB, 0.59966E+00_JPRB ,&
&  0.57948E+00_JPRB, 0.63275E+00_JPRB, 0.77733E+00_JPRB, 0.97548E+00_JPRB, 0.13218E+01_JPRB, 0.14710E+01_JPRB, 0.18259E+01_JPRB ,&
&  0.16052E+01_JPRB, 0.12055E+01_JPRB, 0.12881E+01_JPRB, 0.12362E+01_JPRB, 0.13121E+01_JPRB, 0.13537E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.14262E+01_JPRB, 0.14496E+01_JPRB, 0.14814E+01_JPRB, 0.14959E+01_JPRB, 0.14781E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.13966E+01_JPRB, 0.14088E+01_JPRB, 0.14587E+01_JPRB, 0.15923E+01_JPRB, 0.17410E+01_JPRB, 0.17516E+01_JPRB ,&
&  0.17617E+01_JPRB, 0.18244E+01_JPRB, 0.17290E+01_JPRB, 0.18021E+01_JPRB, 0.18095E+01_JPRB, 0.17405E+01_JPRB, 0.18489E+01_JPRB ,&
&  0.18950E+01_JPRB, 0.15795E+01_JPRB, 0.10063E+01_JPRB, 0.10517E+01_JPRB, 0.10166E+01_JPRB, 0.99452E+00_JPRB, 0.10323E+01_JPRB ,&
&  0.97313E+00_JPRB, 0.92425E+00_JPRB, 0.88511E+00_JPRB, 0.85623E+00_JPRB, 0.80497E+00_JPRB, 0.73623E+00_JPRB, 0.69137E+00_JPRB ,&
&  0.63552E+00_JPRB, 0.60128E+00_JPRB, 0.58010E+00_JPRB, 0.55433E+00_JPRB, 0.51829E+00_JPRB, 0.55699E+00_JPRB, 0.55636E+00_JPRB ,&
&  0.56589E+00_JPRB, 0.57004E+00_JPRB, 0.58150E+00_JPRB, 0.55162E+00_JPRB, 0.51191E+00_JPRB, 0.56065E+00_JPRB, 0.54665E+00_JPRB ,&
&  0.54379E+00_JPRB, 0.53813E+00_JPRB, 0.53567E+00_JPRB, 0.53022E+00_JPRB, 0.55159E+00_JPRB, 0.58346E+00_JPRB, 0.59018E+00_JPRB ,&
&  0.60306E+00_JPRB, 0.59023E+00_JPRB, 0.59509E+00_JPRB, 0.60297E+00_JPRB, 0.60712E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.69072E+00_JPRB, 0.68965E+00_JPRB, 0.67756E+00_JPRB, 0.68659E+00_JPRB, 0.69762E+00_JPRB, 0.69289E+00_JPRB, 0.70556E+00_JPRB ,&
&  0.72379E+00_JPRB, 0.74089E+00_JPRB, 0.75057E+00_JPRB, 0.75620E+00_JPRB, 0.79920E+00_JPRB, 0.85339E+00_JPRB, 0.91715E+00_JPRB ,&
&  0.95356E+00_JPRB, 0.98892E+00_JPRB, 0.99941E+00_JPRB, 0.98959E+00_JPRB, 0.98605E+00_JPRB, 0.95626E+00_JPRB, 0.91591E+00_JPRB ,&
&  0.97473E+00_JPRB, 0.12278E+01_JPRB, 0.16308E+01_JPRB, 0.19732E+01_JPRB, 0.22510E+01_JPRB, 0.18426E+01_JPRB, 0.18281E+01_JPRB ,&
&  0.15757E+01_JPRB, 0.13023E+01_JPRB, 0.13409E+01_JPRB, 0.14434E+01_JPRB, 0.15289E+01_JPRB, 0.15320E+01_JPRB, 0.15674E+01_JPRB ,&
&  0.15866E+01_JPRB, 0.16029E+01_JPRB, 0.16307E+01_JPRB, 0.16999E+01_JPRB, 0.17546E+01_JPRB, 0.17957E+01_JPRB, 0.18475E+01_JPRB ,&
&  0.18825E+01_JPRB, 0.18764E+01_JPRB, 0.18630E+01_JPRB, 0.17095E+01_JPRB, 0.17533E+01_JPRB, 0.18576E+01_JPRB, 0.19473E+01_JPRB ,&
&  0.19871E+01_JPRB, 0.19672E+01_JPRB, 0.20813E+01_JPRB, 0.20918E+01_JPRB, 0.20575E+01_JPRB, 0.19896E+01_JPRB, 0.19885E+01_JPRB ,&
&  0.19762E+01_JPRB, 0.17699E+01_JPRB, 0.11721E+01_JPRB, 0.12216E+01_JPRB, 0.11536E+01_JPRB, 0.99776E+00_JPRB, 0.10543E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.93040E+00_JPRB, 0.87965E+00_JPRB, 0.86192E+00_JPRB, 0.84178E+00_JPRB, 0.82381E+00_JPRB, 0.78579E+00_JPRB ,&
&  0.72618E+00_JPRB, 0.65402E+00_JPRB, 0.57066E+00_JPRB, 0.54389E+00_JPRB, 0.52441E+00_JPRB, 0.58506E+00_JPRB, 0.71043E+00_JPRB ,&
&  0.69577E+00_JPRB, 0.64966E+00_JPRB, 0.67333E+00_JPRB, 0.65770E+00_JPRB, 0.59975E+00_JPRB, 0.62384E+00_JPRB, 0.62479E+00_JPRB ,&
&  0.63164E+00_JPRB, 0.64052E+00_JPRB, 0.64627E+00_JPRB, 0.64702E+00_JPRB, 0.65690E+00_JPRB, 0.67467E+00_JPRB, 0.69105E+00_JPRB ,&
&  0.70524E+00_JPRB, 0.69957E+00_JPRB, 0.69523E+00_JPRB, 0.69150E+00_JPRB, 0.68686E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.74212E+00_JPRB, 0.71705E+00_JPRB, 0.69630E+00_JPRB, 0.68599E+00_JPRB, 0.68381E+00_JPRB, 0.69279E+00_JPRB, 0.71050E+00_JPRB ,&
&  0.74159E+00_JPRB, 0.78643E+00_JPRB, 0.83229E+00_JPRB, 0.87107E+00_JPRB, 0.93067E+00_JPRB, 0.10094E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.12196E+01_JPRB, 0.13379E+01_JPRB, 0.14029E+01_JPRB, 0.14346E+01_JPRB, 0.14991E+01_JPRB, 0.16038E+01_JPRB, 0.18331E+01_JPRB ,&
&  0.21439E+01_JPRB, 0.23908E+01_JPRB, 0.25002E+01_JPRB, 0.22269E+01_JPRB, 0.16619E+01_JPRB, 0.15484E+01_JPRB, 0.16603E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.15618E+01_JPRB, 0.14772E+01_JPRB, 0.14727E+01_JPRB, 0.14821E+01_JPRB, 0.15261E+01_JPRB, 0.15526E+01_JPRB ,&
&  0.15656E+01_JPRB, 0.15866E+01_JPRB, 0.16423E+01_JPRB, 0.17016E+01_JPRB, 0.17970E+01_JPRB, 0.18978E+01_JPRB, 0.20105E+01_JPRB ,&
&  0.20416E+01_JPRB, 0.20881E+01_JPRB, 0.20528E+01_JPRB, 0.19668E+01_JPRB, 0.19367E+01_JPRB, 0.19775E+01_JPRB, 0.20500E+01_JPRB ,&
&  0.21182E+01_JPRB, 0.20572E+01_JPRB, 0.20619E+01_JPRB, 0.20634E+01_JPRB, 0.19553E+01_JPRB, 0.17342E+01_JPRB, 0.16458E+01_JPRB ,&
&  0.16525E+01_JPRB, 0.17862E+01_JPRB, 0.11879E+01_JPRB, 0.13591E+01_JPRB, 0.14567E+01_JPRB, 0.13138E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10201E+01_JPRB, 0.97503E+00_JPRB, 0.99507E+00_JPRB, 0.96988E+00_JPRB, 0.96274E+00_JPRB, 0.85186E+00_JPRB ,&
&  0.85168E+00_JPRB, 0.74094E+00_JPRB, 0.61093E+00_JPRB, 0.52025E+00_JPRB, 0.54368E+00_JPRB, 0.62273E+00_JPRB, 0.71736E+00_JPRB ,&
&  0.69821E+00_JPRB, 0.80546E+00_JPRB, 0.93660E+00_JPRB, 0.11400E+01_JPRB, 0.12911E+01_JPRB, 0.64236E+00_JPRB, 0.65198E+00_JPRB ,&
&  0.66502E+00_JPRB, 0.69537E+00_JPRB, 0.70485E+00_JPRB, 0.71055E+00_JPRB, 0.71772E+00_JPRB, 0.73879E+00_JPRB, 0.77148E+00_JPRB ,&
&  0.78897E+00_JPRB, 0.79689E+00_JPRB, 0.80360E+00_JPRB, 0.79100E+00_JPRB, 0.76381E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.78506E+00_JPRB, 0.78006E+00_JPRB, 0.77318E+00_JPRB, 0.75695E+00_JPRB, 0.72988E+00_JPRB, 0.71058E+00_JPRB, 0.71183E+00_JPRB ,&
&  0.71568E+00_JPRB, 0.74972E+00_JPRB, 0.80934E+00_JPRB, 0.88365E+00_JPRB, 0.98171E+00_JPRB, 0.10928E+01_JPRB, 0.12097E+01_JPRB ,&
&  0.13629E+01_JPRB, 0.15193E+01_JPRB, 0.16250E+01_JPRB, 0.17133E+01_JPRB, 0.18161E+01_JPRB, 0.19878E+01_JPRB, 0.21876E+01_JPRB ,&
&  0.23367E+01_JPRB, 0.23301E+01_JPRB, 0.21791E+01_JPRB, 0.15639E+01_JPRB, 0.12601E+01_JPRB, 0.14479E+01_JPRB, 0.14806E+01_JPRB ,&
&  0.14765E+01_JPRB, 0.14506E+01_JPRB, 0.14407E+01_JPRB, 0.14210E+01_JPRB, 0.14301E+01_JPRB, 0.14711E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.15052E+01_JPRB, 0.15169E+01_JPRB, 0.15615E+01_JPRB, 0.16558E+01_JPRB, 0.17921E+01_JPRB, 0.18594E+01_JPRB, 0.19373E+01_JPRB ,&
&  0.20000E+01_JPRB, 0.21083E+01_JPRB, 0.21529E+01_JPRB, 0.21497E+01_JPRB, 0.20793E+01_JPRB, 0.20641E+01_JPRB, 0.20921E+01_JPRB ,&
&  0.20872E+01_JPRB, 0.20237E+01_JPRB, 0.20065E+01_JPRB, 0.20087E+01_JPRB, 0.19602E+01_JPRB, 0.16596E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.14863E+01_JPRB, 0.14515E+01_JPRB, 0.12556E+01_JPRB, 0.15267E+01_JPRB, 0.10482E+01_JPRB, 0.11968E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.11882E+01_JPRB, 0.12018E+01_JPRB, 0.11948E+01_JPRB, 0.11574E+01_JPRB, 0.95938E+00_JPRB, 0.92480E+00_JPRB ,&
&  0.90985E+00_JPRB, 0.76198E+00_JPRB, 0.64589E+00_JPRB, 0.50155E+00_JPRB, 0.55225E+00_JPRB, 0.63792E+00_JPRB, 0.68129E+00_JPRB ,&
&  0.64690E+00_JPRB, 0.78915E+00_JPRB, 0.95234E+00_JPRB, 0.10402E+01_JPRB, 0.10657E+01_JPRB, 0.64835E+00_JPRB, 0.64511E+00_JPRB ,&
&  0.66762E+00_JPRB, 0.68472E+00_JPRB, 0.70220E+00_JPRB, 0.70220E+00_JPRB, 0.72071E+00_JPRB, 0.75407E+00_JPRB, 0.78510E+00_JPRB ,&
&  0.79793E+00_JPRB, 0.79453E+00_JPRB, 0.80120E+00_JPRB, 0.79662E+00_JPRB, 0.79046E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.79170E+00_JPRB, 0.79902E+00_JPRB, 0.82468E+00_JPRB, 0.83886E+00_JPRB, 0.84533E+00_JPRB, 0.79998E+00_JPRB, 0.76443E+00_JPRB ,&
&  0.71778E+00_JPRB, 0.66902E+00_JPRB, 0.72568E+00_JPRB, 0.79546E+00_JPRB, 0.87827E+00_JPRB, 0.96696E+00_JPRB, 0.10603E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.13377E+01_JPRB, 0.15265E+01_JPRB, 0.16679E+01_JPRB, 0.17914E+01_JPRB, 0.19526E+01_JPRB, 0.20938E+01_JPRB ,&
&  0.17356E+01_JPRB, 0.12402E+01_JPRB, 0.15783E+01_JPRB, 0.16932E+01_JPRB, 0.14035E+01_JPRB, 0.13199E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.13526E+01_JPRB, 0.13838E+01_JPRB, 0.14024E+01_JPRB, 0.14140E+01_JPRB, 0.13766E+01_JPRB, 0.14070E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.14738E+01_JPRB, 0.14774E+01_JPRB, 0.14871E+01_JPRB, 0.15599E+01_JPRB, 0.16878E+01_JPRB, 0.17187E+01_JPRB, 0.18247E+01_JPRB ,&
&  0.19358E+01_JPRB, 0.20863E+01_JPRB, 0.21160E+01_JPRB, 0.20428E+01_JPRB, 0.20460E+01_JPRB, 0.20790E+01_JPRB, 0.20896E+01_JPRB ,&
&  0.20900E+01_JPRB, 0.20591E+01_JPRB, 0.20648E+01_JPRB, 0.20922E+01_JPRB, 0.20466E+01_JPRB, 0.17643E+01_JPRB, 0.16754E+01_JPRB ,&
&  0.15628E+01_JPRB, 0.14219E+01_JPRB, 0.13188E+01_JPRB, 0.14819E+01_JPRB, 0.11667E+01_JPRB, 0.12339E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.13026E+01_JPRB, 0.12769E+01_JPRB, 0.12221E+01_JPRB, 0.11620E+01_JPRB, 0.98872E+00_JPRB, 0.92946E+00_JPRB ,&
&  0.82805E+00_JPRB, 0.79249E+00_JPRB, 0.64823E+00_JPRB, 0.52964E+00_JPRB, 0.47606E+00_JPRB, 0.52931E+00_JPRB, 0.61150E+00_JPRB ,&
&  0.70842E+00_JPRB, 0.71125E+00_JPRB, 0.84844E+00_JPRB, 0.90719E+00_JPRB, 0.90411E+00_JPRB, 0.70277E+00_JPRB, 0.67448E+00_JPRB ,&
&  0.68118E+00_JPRB, 0.68489E+00_JPRB, 0.65935E+00_JPRB, 0.66747E+00_JPRB, 0.69440E+00_JPRB, 0.73612E+00_JPRB, 0.75656E+00_JPRB ,&
&  0.75747E+00_JPRB, 0.74223E+00_JPRB, 0.74155E+00_JPRB, 0.75242E+00_JPRB, 0.78037E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.72760E+00_JPRB, 0.75096E+00_JPRB, 0.77852E+00_JPRB, 0.82613E+00_JPRB, 0.89581E+00_JPRB, 0.98439E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.85842E+00_JPRB, 0.62300E+00_JPRB, 0.64034E+00_JPRB, 0.66812E+00_JPRB, 0.71994E+00_JPRB, 0.76921E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.91211E+00_JPRB, 0.10273E+01_JPRB, 0.12250E+01_JPRB, 0.14481E+01_JPRB, 0.16408E+01_JPRB, 0.18719E+01_JPRB, 0.17629E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.12965E+01_JPRB, 0.15589E+01_JPRB, 0.15048E+01_JPRB, 0.13280E+01_JPRB, 0.13182E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.13099E+01_JPRB, 0.13326E+01_JPRB, 0.13063E+01_JPRB, 0.12760E+01_JPRB, 0.13039E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.14092E+01_JPRB, 0.14503E+01_JPRB, 0.14759E+01_JPRB, 0.15039E+01_JPRB, 0.15612E+01_JPRB, 0.15863E+01_JPRB, 0.16888E+01_JPRB ,&
&  0.18366E+01_JPRB, 0.19682E+01_JPRB, 0.20012E+01_JPRB, 0.19479E+01_JPRB, 0.19880E+01_JPRB, 0.20898E+01_JPRB, 0.21651E+01_JPRB ,&
&  0.22111E+01_JPRB, 0.22956E+01_JPRB, 0.23241E+01_JPRB, 0.22453E+01_JPRB, 0.20170E+01_JPRB, 0.20900E+01_JPRB, 0.18137E+01_JPRB ,&
&  0.17161E+01_JPRB, 0.15086E+01_JPRB, 0.15347E+01_JPRB, 0.12237E+01_JPRB, 0.13055E+01_JPRB, 0.13251E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.14029E+01_JPRB, 0.13799E+01_JPRB, 0.12588E+01_JPRB, 0.12186E+01_JPRB, 0.10242E+01_JPRB, 0.96668E+00_JPRB ,&
&  0.78136E+00_JPRB, 0.82518E+00_JPRB, 0.66612E+00_JPRB, 0.51092E+00_JPRB, 0.40631E+00_JPRB, 0.40972E+00_JPRB, 0.49149E+00_JPRB ,&
&  0.66490E+00_JPRB, 0.74173E+00_JPRB, 0.77043E+00_JPRB, 0.79307E+00_JPRB, 0.79400E+00_JPRB, 0.70816E+00_JPRB, 0.70603E+00_JPRB ,&
&  0.71795E+00_JPRB, 0.69606E+00_JPRB, 0.64873E+00_JPRB, 0.64967E+00_JPRB, 0.67311E+00_JPRB, 0.71265E+00_JPRB, 0.71277E+00_JPRB ,&
&  0.72031E+00_JPRB, 0.71341E+00_JPRB, 0.69998E+00_JPRB, 0.69192E+00_JPRB, 0.70357E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.66111E+00_JPRB, 0.66193E+00_JPRB, 0.65094E+00_JPRB, 0.64338E+00_JPRB, 0.64275E+00_JPRB, 0.64988E+00_JPRB, 0.67225E+00_JPRB ,&
&  0.61715E+00_JPRB, 0.60585E+00_JPRB, 0.60888E+00_JPRB, 0.60429E+00_JPRB, 0.61416E+00_JPRB, 0.62635E+00_JPRB, 0.65139E+00_JPRB ,&
&  0.68233E+00_JPRB, 0.71153E+00_JPRB, 0.82846E+00_JPRB, 0.10624E+01_JPRB, 0.12989E+01_JPRB, 0.15689E+01_JPRB, 0.14465E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.18280E+01_JPRB, 0.14031E+01_JPRB, 0.14411E+01_JPRB, 0.13861E+01_JPRB, 0.13649E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.12387E+01_JPRB, 0.12434E+01_JPRB, 0.12765E+01_JPRB, 0.12574E+01_JPRB, 0.12595E+01_JPRB, 0.12698E+01_JPRB, 0.12966E+01_JPRB ,&
&  0.13452E+01_JPRB, 0.13912E+01_JPRB, 0.14213E+01_JPRB, 0.14504E+01_JPRB, 0.14914E+01_JPRB, 0.15119E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.16664E+01_JPRB, 0.18213E+01_JPRB, 0.18768E+01_JPRB, 0.19378E+01_JPRB, 0.20102E+01_JPRB, 0.21915E+01_JPRB, 0.23194E+01_JPRB ,&
&  0.22926E+01_JPRB, 0.22799E+01_JPRB, 0.25017E+01_JPRB, 0.25218E+01_JPRB, 0.24386E+01_JPRB, 0.24368E+01_JPRB, 0.20169E+01_JPRB ,&
&  0.19346E+01_JPRB, 0.16520E+01_JPRB, 0.16631E+01_JPRB, 0.13420E+01_JPRB, 0.13774E+01_JPRB, 0.14437E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.12887E+01_JPRB, 0.14216E+01_JPRB, 0.14014E+01_JPRB, 0.13365E+01_JPRB, 0.12278E+01_JPRB, 0.11192E+01_JPRB, 0.99223E+00_JPRB ,&
&  0.86642E+00_JPRB, 0.82503E+00_JPRB, 0.78917E+00_JPRB, 0.55026E+00_JPRB, 0.45618E+00_JPRB, 0.43362E+00_JPRB, 0.47077E+00_JPRB ,&
&  0.61623E+00_JPRB, 0.71313E+00_JPRB, 0.87341E+00_JPRB, 0.84834E+00_JPRB, 0.77205E+00_JPRB, 0.72773E+00_JPRB, 0.73237E+00_JPRB ,&
&  0.71752E+00_JPRB, 0.71506E+00_JPRB, 0.67753E+00_JPRB, 0.66340E+00_JPRB, 0.68484E+00_JPRB, 0.71194E+00_JPRB, 0.70752E+00_JPRB ,&
&  0.71843E+00_JPRB, 0.72138E+00_JPRB, 0.70731E+00_JPRB, 0.68305E+00_JPRB, 0.66867E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.67261E+00_JPRB, 0.65717E+00_JPRB, 0.65254E+00_JPRB, 0.63958E+00_JPRB, 0.62878E+00_JPRB, 0.61608E+00_JPRB, 0.62242E+00_JPRB ,&
&  0.60485E+00_JPRB, 0.59284E+00_JPRB, 0.59210E+00_JPRB, 0.57943E+00_JPRB, 0.57952E+00_JPRB, 0.56550E+00_JPRB, 0.57405E+00_JPRB ,&
&  0.58221E+00_JPRB, 0.55674E+00_JPRB, 0.59121E+00_JPRB, 0.73138E+00_JPRB, 0.93421E+00_JPRB, 0.12678E+01_JPRB, 0.97745E+00_JPRB ,&
&  0.12220E+01_JPRB, 0.17669E+01_JPRB, 0.15049E+01_JPRB, 0.15153E+01_JPRB, 0.15658E+01_JPRB, 0.15612E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.11745E+01_JPRB, 0.11966E+01_JPRB, 0.12425E+01_JPRB, 0.13016E+01_JPRB, 0.13618E+01_JPRB, 0.13612E+01_JPRB ,&
&  0.13788E+01_JPRB, 0.14111E+01_JPRB, 0.14281E+01_JPRB, 0.14304E+01_JPRB, 0.14472E+01_JPRB, 0.14705E+01_JPRB, 0.14837E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.16306E+01_JPRB, 0.18184E+01_JPRB, 0.18398E+01_JPRB, 0.19855E+01_JPRB, 0.22557E+01_JPRB, 0.24200E+01_JPRB ,&
&  0.23074E+01_JPRB, 0.22644E+01_JPRB, 0.26431E+01_JPRB, 0.29820E+01_JPRB, 0.30286E+01_JPRB, 0.28468E+01_JPRB, 0.23807E+01_JPRB ,&
&  0.21373E+01_JPRB, 0.18717E+01_JPRB, 0.17158E+01_JPRB, 0.14548E+01_JPRB, 0.14412E+01_JPRB, 0.15306E+01_JPRB, 0.15023E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.15287E+01_JPRB, 0.15011E+01_JPRB, 0.13704E+01_JPRB, 0.12311E+01_JPRB, 0.11359E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.11024E+01_JPRB, 0.10395E+01_JPRB, 0.69534E+00_JPRB, 0.54464E+00_JPRB, 0.43706E+00_JPRB, 0.47995E+00_JPRB ,&
&  0.63314E+00_JPRB, 0.69352E+00_JPRB, 0.71786E+00_JPRB, 0.10050E+01_JPRB, 0.88950E+00_JPRB, 0.82341E+00_JPRB, 0.80811E+00_JPRB ,&
&  0.77293E+00_JPRB, 0.78960E+00_JPRB, 0.77179E+00_JPRB, 0.73656E+00_JPRB, 0.73792E+00_JPRB, 0.74577E+00_JPRB, 0.75242E+00_JPRB ,&
&  0.74560E+00_JPRB, 0.74620E+00_JPRB, 0.74822E+00_JPRB, 0.73159E+00_JPRB, 0.70039E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.68554E+00_JPRB, 0.67291E+00_JPRB, 0.64944E+00_JPRB, 0.63422E+00_JPRB, 0.63080E+00_JPRB, 0.61035E+00_JPRB, 0.59334E+00_JPRB ,&
&  0.59349E+00_JPRB, 0.57980E+00_JPRB, 0.56604E+00_JPRB, 0.56063E+00_JPRB, 0.55751E+00_JPRB, 0.54924E+00_JPRB, 0.54776E+00_JPRB ,&
&  0.54140E+00_JPRB, 0.51329E+00_JPRB, 0.54781E+00_JPRB, 0.60894E+00_JPRB, 0.73421E+00_JPRB, 0.80888E+00_JPRB, 0.90490E+00_JPRB ,&
&  0.14759E+01_JPRB, 0.18907E+01_JPRB, 0.16566E+01_JPRB, 0.17807E+01_JPRB, 0.19919E+01_JPRB, 0.22379E+01_JPRB, 0.16552E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.12800E+01_JPRB, 0.11840E+01_JPRB, 0.12418E+01_JPRB, 0.13541E+01_JPRB, 0.14534E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.14324E+01_JPRB, 0.14068E+01_JPRB, 0.14169E+01_JPRB, 0.14191E+01_JPRB, 0.14168E+01_JPRB, 0.14533E+01_JPRB, 0.14663E+01_JPRB ,&
&  0.14749E+01_JPRB, 0.15278E+01_JPRB, 0.16785E+01_JPRB, 0.17475E+01_JPRB, 0.19315E+01_JPRB, 0.22048E+01_JPRB, 0.25283E+01_JPRB ,&
&  0.25765E+01_JPRB, 0.27022E+01_JPRB, 0.29583E+01_JPRB, 0.32559E+01_JPRB, 0.34541E+01_JPRB, 0.34058E+01_JPRB, 0.28750E+01_JPRB ,&
&  0.24781E+01_JPRB, 0.20264E+01_JPRB, 0.16735E+01_JPRB, 0.15664E+01_JPRB, 0.15908E+01_JPRB, 0.17056E+01_JPRB, 0.15356E+01_JPRB ,&
&  0.14436E+01_JPRB, 0.12676E+01_JPRB, 0.12320E+01_JPRB, 0.13223E+01_JPRB, 0.12887E+01_JPRB, 0.12133E+01_JPRB, 0.13466E+01_JPRB ,&
&  0.11875E+01_JPRB, 0.74143E+00_JPRB, 0.64452E+00_JPRB, 0.72948E+00_JPRB, 0.52257E+00_JPRB, 0.40668E+00_JPRB, 0.54507E+00_JPRB ,&
&  0.66230E+00_JPRB, 0.77096E+00_JPRB, 0.77546E+00_JPRB, 0.80333E+00_JPRB, 0.94858E+00_JPRB, 0.90012E+00_JPRB, 0.92978E+00_JPRB ,&
&  0.83759E+00_JPRB, 0.84126E+00_JPRB, 0.83771E+00_JPRB, 0.78575E+00_JPRB, 0.76350E+00_JPRB, 0.78693E+00_JPRB, 0.80080E+00_JPRB ,&
&  0.79550E+00_JPRB, 0.78715E+00_JPRB, 0.76421E+00_JPRB, 0.73872E+00_JPRB, 0.70740E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.72081E+00_JPRB, 0.70405E+00_JPRB, 0.69215E+00_JPRB, 0.66480E+00_JPRB, 0.64605E+00_JPRB, 0.62302E+00_JPRB, 0.59335E+00_JPRB ,&
&  0.57662E+00_JPRB, 0.56680E+00_JPRB, 0.56047E+00_JPRB, 0.55080E+00_JPRB, 0.54542E+00_JPRB, 0.54254E+00_JPRB, 0.54859E+00_JPRB ,&
&  0.53247E+00_JPRB, 0.51426E+00_JPRB, 0.59991E+00_JPRB, 0.66041E+00_JPRB, 0.69775E+00_JPRB, 0.71036E+00_JPRB, 0.94084E+00_JPRB ,&
&  0.15097E+01_JPRB, 0.20606E+01_JPRB, 0.24387E+01_JPRB, 0.25742E+01_JPRB, 0.27341E+01_JPRB, 0.29608E+01_JPRB, 0.25313E+01_JPRB ,&
&  0.20374E+01_JPRB, 0.16527E+01_JPRB, 0.13812E+01_JPRB, 0.13632E+01_JPRB, 0.14707E+01_JPRB, 0.15646E+01_JPRB, 0.15532E+01_JPRB ,&
&  0.15118E+01_JPRB, 0.14347E+01_JPRB, 0.13928E+01_JPRB, 0.13657E+01_JPRB, 0.13807E+01_JPRB, 0.14272E+01_JPRB, 0.14534E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14455E+01_JPRB, 0.15367E+01_JPRB, 0.17352E+01_JPRB, 0.16709E+01_JPRB, 0.21318E+01_JPRB, 0.25256E+01_JPRB ,&
&  0.28176E+01_JPRB, 0.29586E+01_JPRB, 0.29984E+01_JPRB, 0.36373E+01_JPRB, 0.40120E+01_JPRB, 0.38467E+01_JPRB, 0.32975E+01_JPRB ,&
&  0.29614E+01_JPRB, 0.23920E+01_JPRB, 0.19329E+01_JPRB, 0.17708E+01_JPRB, 0.17561E+01_JPRB, 0.18137E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.12657E+01_JPRB, 0.15227E+01_JPRB, 0.13891E+01_JPRB, 0.12408E+01_JPRB, 0.13491E+01_JPRB, 0.13189E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.55306E+00_JPRB, 0.46193E+00_JPRB, 0.46067E+00_JPRB, 0.44877E+00_JPRB, 0.40385E+00_JPRB, 0.41197E+00_JPRB, 0.76737E+00_JPRB ,&
&  0.71018E+00_JPRB, 0.85850E+00_JPRB, 0.83928E+00_JPRB, 0.84651E+00_JPRB, 0.93684E+00_JPRB, 0.93453E+00_JPRB, 0.11568E+01_JPRB ,&
&  0.99065E+00_JPRB, 0.93092E+00_JPRB, 0.89783E+00_JPRB, 0.85452E+00_JPRB, 0.83741E+00_JPRB, 0.82456E+00_JPRB, 0.83606E+00_JPRB ,&
&  0.84756E+00_JPRB, 0.85215E+00_JPRB, 0.82928E+00_JPRB, 0.78876E+00_JPRB, 0.75210E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.77070E+00_JPRB, 0.73370E+00_JPRB, 0.70941E+00_JPRB, 0.69869E+00_JPRB, 0.68369E+00_JPRB, 0.64930E+00_JPRB, 0.60205E+00_JPRB ,&
&  0.57279E+00_JPRB, 0.55778E+00_JPRB, 0.55024E+00_JPRB, 0.54600E+00_JPRB, 0.54907E+00_JPRB, 0.55157E+00_JPRB, 0.55011E+00_JPRB ,&
&  0.52503E+00_JPRB, 0.53349E+00_JPRB, 0.66440E+00_JPRB, 0.70831E+00_JPRB, 0.73537E+00_JPRB, 0.73337E+00_JPRB, 0.94082E+00_JPRB ,&
&  0.14867E+01_JPRB, 0.20835E+01_JPRB, 0.25031E+01_JPRB, 0.25971E+01_JPRB, 0.34312E+01_JPRB, 0.39187E+01_JPRB, 0.38948E+01_JPRB ,&
&  0.31727E+01_JPRB, 0.25165E+01_JPRB, 0.19672E+01_JPRB, 0.16867E+01_JPRB, 0.16799E+01_JPRB, 0.17073E+01_JPRB, 0.16870E+01_JPRB ,&
&  0.15992E+01_JPRB, 0.15169E+01_JPRB, 0.14445E+01_JPRB, 0.14008E+01_JPRB, 0.13985E+01_JPRB, 0.14308E+01_JPRB, 0.14768E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.14258E+01_JPRB, 0.14128E+01_JPRB, 0.16072E+01_JPRB, 0.17652E+01_JPRB, 0.19106E+01_JPRB, 0.23678E+01_JPRB ,&
&  0.28621E+01_JPRB, 0.32432E+01_JPRB, 0.32674E+01_JPRB, 0.42348E+01_JPRB, 0.44539E+01_JPRB, 0.41706E+01_JPRB, 0.38570E+01_JPRB ,&
&  0.34612E+01_JPRB, 0.31512E+01_JPRB, 0.22511E+01_JPRB, 0.19651E+01_JPRB, 0.19675E+01_JPRB, 0.15936E+01_JPRB, 0.14270E+01_JPRB ,&
&  0.16774E+01_JPRB, 0.15386E+01_JPRB, 0.13510E+01_JPRB, 0.11596E+01_JPRB, 0.13758E+01_JPRB, 0.11828E+01_JPRB, 0.62061E+00_JPRB ,&
&  0.51393E+00_JPRB, 0.48573E+00_JPRB, 0.43717E+00_JPRB, 0.42007E+00_JPRB, 0.40031E+00_JPRB, 0.42734E+00_JPRB, 0.64294E+00_JPRB ,&
&  0.71539E+00_JPRB, 0.84864E+00_JPRB, 0.86297E+00_JPRB, 0.88968E+00_JPRB, 0.95655E+00_JPRB, 0.10600E+01_JPRB, 0.15870E+01_JPRB ,&
&  0.13046E+01_JPRB, 0.11657E+01_JPRB, 0.10248E+01_JPRB, 0.95146E+00_JPRB, 0.89947E+00_JPRB, 0.86725E+00_JPRB, 0.86604E+00_JPRB ,&
&  0.86569E+00_JPRB, 0.86667E+00_JPRB, 0.84923E+00_JPRB, 0.82317E+00_JPRB, 0.79681E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.80038E+00_JPRB, 0.76922E+00_JPRB, 0.73236E+00_JPRB, 0.70280E+00_JPRB, 0.68548E+00_JPRB, 0.66631E+00_JPRB, 0.63460E+00_JPRB ,&
&  0.59717E+00_JPRB, 0.57669E+00_JPRB, 0.56644E+00_JPRB, 0.55336E+00_JPRB, 0.55220E+00_JPRB, 0.55778E+00_JPRB, 0.55487E+00_JPRB ,&
&  0.53478E+00_JPRB, 0.56552E+00_JPRB, 0.64691E+00_JPRB, 0.64274E+00_JPRB, 0.65039E+00_JPRB, 0.66008E+00_JPRB, 0.82352E+00_JPRB ,&
&  0.13755E+01_JPRB, 0.19966E+01_JPRB, 0.23981E+01_JPRB, 0.28147E+01_JPRB, 0.37591E+01_JPRB, 0.39078E+01_JPRB, 0.44638E+01_JPRB ,&
&  0.39899E+01_JPRB, 0.33553E+01_JPRB, 0.27952E+01_JPRB, 0.24174E+01_JPRB, 0.21768E+01_JPRB, 0.19900E+01_JPRB, 0.18883E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.16406E+01_JPRB, 0.15843E+01_JPRB, 0.15269E+01_JPRB, 0.15010E+01_JPRB, 0.14911E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.15479E+01_JPRB, 0.14832E+01_JPRB, 0.13708E+01_JPRB, 0.14450E+01_JPRB, 0.18235E+01_JPRB, 0.21409E+01_JPRB, 0.22800E+01_JPRB ,&
&  0.24603E+01_JPRB, 0.26637E+01_JPRB, 0.34071E+01_JPRB, 0.41458E+01_JPRB, 0.46510E+01_JPRB, 0.47211E+01_JPRB, 0.41394E+01_JPRB ,&
&  0.37094E+01_JPRB, 0.33056E+01_JPRB, 0.24403E+01_JPRB, 0.21392E+01_JPRB, 0.18711E+01_JPRB, 0.15449E+01_JPRB, 0.17277E+01_JPRB ,&
&  0.18852E+01_JPRB, 0.17331E+01_JPRB, 0.14374E+01_JPRB, 0.11565E+01_JPRB, 0.10924E+01_JPRB, 0.80314E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.54971E+00_JPRB, 0.51183E+00_JPRB, 0.46719E+00_JPRB, 0.48248E+00_JPRB, 0.45854E+00_JPRB, 0.50540E+00_JPRB, 0.67853E+00_JPRB ,&
&  0.82167E+00_JPRB, 0.88820E+00_JPRB, 0.95225E+00_JPRB, 0.94596E+00_JPRB, 0.96305E+00_JPRB, 0.10031E+01_JPRB, 0.15021E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11324E+01_JPRB, 0.11242E+01_JPRB, 0.10605E+01_JPRB, 0.10200E+01_JPRB, 0.96417E+00_JPRB, 0.91544E+00_JPRB ,&
&  0.89029E+00_JPRB, 0.86777E+00_JPRB, 0.84731E+00_JPRB, 0.83410E+00_JPRB, 0.82006E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.80189E+00_JPRB, 0.76453E+00_JPRB, 0.73164E+00_JPRB, 0.70035E+00_JPRB, 0.67438E+00_JPRB, 0.65525E+00_JPRB, 0.64092E+00_JPRB ,&
&  0.61988E+00_JPRB, 0.60300E+00_JPRB, 0.58234E+00_JPRB, 0.56253E+00_JPRB, 0.55288E+00_JPRB, 0.55148E+00_JPRB, 0.54294E+00_JPRB ,&
&  0.52783E+00_JPRB, 0.60124E+00_JPRB, 0.48141E+00_JPRB, 0.46109E+00_JPRB, 0.54603E+00_JPRB, 0.61673E+00_JPRB, 0.69109E+00_JPRB ,&
&  0.11183E+01_JPRB, 0.17697E+01_JPRB, 0.22066E+01_JPRB, 0.30603E+01_JPRB, 0.48395E+01_JPRB, 0.46288E+01_JPRB, 0.46719E+01_JPRB ,&
&  0.46681E+01_JPRB, 0.38787E+01_JPRB, 0.33236E+01_JPRB, 0.28441E+01_JPRB, 0.24881E+01_JPRB, 0.22103E+01_JPRB, 0.19692E+01_JPRB ,&
&  0.17696E+01_JPRB, 0.16349E+01_JPRB, 0.15669E+01_JPRB, 0.15646E+01_JPRB, 0.15797E+01_JPRB, 0.16037E+01_JPRB, 0.16253E+01_JPRB ,&
&  0.16184E+01_JPRB, 0.15678E+01_JPRB, 0.14411E+01_JPRB, 0.14382E+01_JPRB, 0.16067E+01_JPRB, 0.19326E+01_JPRB, 0.26916E+01_JPRB ,&
&  0.27301E+01_JPRB, 0.29577E+01_JPRB, 0.30169E+01_JPRB, 0.43052E+01_JPRB, 0.49633E+01_JPRB, 0.46517E+01_JPRB, 0.43576E+01_JPRB ,&
&  0.32335E+01_JPRB, 0.26392E+01_JPRB, 0.22856E+01_JPRB, 0.19871E+01_JPRB, 0.17939E+01_JPRB, 0.19711E+01_JPRB, 0.26706E+01_JPRB ,&
&  0.21126E+01_JPRB, 0.22117E+01_JPRB, 0.22336E+01_JPRB, 0.19837E+01_JPRB, 0.10956E+01_JPRB, 0.82967E+00_JPRB, 0.11849E+01_JPRB ,&
&  0.12356E+01_JPRB, 0.78743E+00_JPRB, 0.60777E+00_JPRB, 0.73153E+00_JPRB, 0.58176E+00_JPRB, 0.62304E+00_JPRB, 0.76049E+00_JPRB ,&
&  0.80384E+00_JPRB, 0.85573E+00_JPRB, 0.10530E+01_JPRB, 0.10499E+01_JPRB, 0.10331E+01_JPRB, 0.10040E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.10292E+01_JPRB, 0.10523E+01_JPRB, 0.99685E+00_JPRB, 0.97352E+00_JPRB, 0.94326E+00_JPRB, 0.90233E+00_JPRB ,&
&  0.87823E+00_JPRB, 0.85686E+00_JPRB, 0.84218E+00_JPRB, 0.83137E+00_JPRB, 0.82513E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.79851E+00_JPRB, 0.76051E+00_JPRB, 0.72027E+00_JPRB, 0.69176E+00_JPRB, 0.66765E+00_JPRB, 0.64280E+00_JPRB, 0.62147E+00_JPRB ,&
&  0.60857E+00_JPRB, 0.59534E+00_JPRB, 0.57166E+00_JPRB, 0.55491E+00_JPRB, 0.54400E+00_JPRB, 0.54606E+00_JPRB, 0.54329E+00_JPRB ,&
&  0.53087E+00_JPRB, 0.49828E+00_JPRB, 0.41672E+00_JPRB, 0.40095E+00_JPRB, 0.51146E+00_JPRB, 0.53658E+00_JPRB, 0.59903E+00_JPRB ,&
&  0.88743E+00_JPRB, 0.14950E+01_JPRB, 0.21784E+01_JPRB, 0.30825E+01_JPRB, 0.43785E+01_JPRB, 0.51051E+01_JPRB, 0.49663E+01_JPRB ,&
&  0.50232E+01_JPRB, 0.42632E+01_JPRB, 0.36397E+01_JPRB, 0.31389E+01_JPRB, 0.26701E+01_JPRB, 0.22290E+01_JPRB, 0.18842E+01_JPRB ,&
&  0.16874E+01_JPRB, 0.15540E+01_JPRB, 0.14805E+01_JPRB, 0.14124E+01_JPRB, 0.14369E+01_JPRB, 0.15060E+01_JPRB, 0.15792E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.15906E+01_JPRB, 0.15301E+01_JPRB, 0.14596E+01_JPRB, 0.16131E+01_JPRB, 0.19400E+01_JPRB, 0.22860E+01_JPRB ,&
&  0.26273E+01_JPRB, 0.28327E+01_JPRB, 0.30798E+01_JPRB, 0.36546E+01_JPRB, 0.44167E+01_JPRB, 0.41817E+01_JPRB, 0.42591E+01_JPRB ,&
&  0.33438E+01_JPRB, 0.27531E+01_JPRB, 0.22558E+01_JPRB, 0.17000E+01_JPRB, 0.19046E+01_JPRB, 0.28800E+01_JPRB, 0.29086E+01_JPRB ,&
&  0.27228E+01_JPRB, 0.26818E+01_JPRB, 0.24638E+01_JPRB, 0.19576E+01_JPRB, 0.16314E+01_JPRB, 0.12526E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.14290E+01_JPRB, 0.13626E+01_JPRB, 0.12264E+01_JPRB, 0.10364E+01_JPRB, 0.86536E+00_JPRB, 0.91852E+00_JPRB, 0.83016E+00_JPRB ,&
&  0.76911E+00_JPRB, 0.77532E+00_JPRB, 0.98327E+00_JPRB, 0.10698E+01_JPRB, 0.10492E+01_JPRB, 0.92394E+00_JPRB, 0.11193E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.10133E+01_JPRB, 0.94103E+00_JPRB, 0.89410E+00_JPRB, 0.88667E+00_JPRB, 0.88100E+00_JPRB, 0.87312E+00_JPRB ,&
&  0.84744E+00_JPRB, 0.83492E+00_JPRB, 0.82897E+00_JPRB, 0.82354E+00_JPRB, 0.81565E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.78715E+00_JPRB, 0.75254E+00_JPRB, 0.71826E+00_JPRB, 0.69199E+00_JPRB, 0.66660E+00_JPRB, 0.64165E+00_JPRB, 0.61430E+00_JPRB ,&
&  0.59415E+00_JPRB, 0.58218E+00_JPRB, 0.56598E+00_JPRB, 0.55295E+00_JPRB, 0.54781E+00_JPRB, 0.54933E+00_JPRB, 0.55400E+00_JPRB ,&
&  0.55058E+00_JPRB, 0.51319E+00_JPRB, 0.40497E+00_JPRB, 0.40124E+00_JPRB, 0.44337E+00_JPRB, 0.45248E+00_JPRB, 0.54992E+00_JPRB ,&
&  0.75382E+00_JPRB, 0.11975E+01_JPRB, 0.17784E+01_JPRB, 0.25818E+01_JPRB, 0.34808E+01_JPRB, 0.41793E+01_JPRB, 0.43246E+01_JPRB ,&
&  0.38033E+01_JPRB, 0.37031E+01_JPRB, 0.31734E+01_JPRB, 0.27166E+01_JPRB, 0.22618E+01_JPRB, 0.19270E+01_JPRB, 0.16961E+01_JPRB ,&
&  0.14992E+01_JPRB, 0.13746E+01_JPRB, 0.13153E+01_JPRB, 0.12987E+01_JPRB, 0.13234E+01_JPRB, 0.13777E+01_JPRB, 0.14526E+01_JPRB ,&
&  0.14820E+01_JPRB, 0.14939E+01_JPRB, 0.14670E+01_JPRB, 0.14997E+01_JPRB, 0.14891E+01_JPRB, 0.19359E+01_JPRB, 0.21272E+01_JPRB ,&
&  0.27390E+01_JPRB, 0.31556E+01_JPRB, 0.32703E+01_JPRB, 0.41472E+01_JPRB, 0.35204E+01_JPRB, 0.38495E+01_JPRB, 0.41168E+01_JPRB ,&
&  0.38799E+01_JPRB, 0.35354E+01_JPRB, 0.33395E+01_JPRB, 0.26793E+01_JPRB, 0.22709E+01_JPRB, 0.30472E+01_JPRB, 0.28814E+01_JPRB ,&
&  0.25974E+01_JPRB, 0.25013E+01_JPRB, 0.22550E+01_JPRB, 0.20921E+01_JPRB, 0.18044E+01_JPRB, 0.15953E+01_JPRB, 0.12225E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.11156E+01_JPRB, 0.11896E+01_JPRB, 0.10927E+01_JPRB, 0.10211E+01_JPRB, 0.95198E+00_JPRB, 0.85286E+00_JPRB ,&
&  0.77803E+00_JPRB, 0.74796E+00_JPRB, 0.81505E+00_JPRB, 0.87302E+00_JPRB, 0.91114E+00_JPRB, 0.79671E+00_JPRB, 0.90600E+00_JPRB ,&
&  0.97055E+00_JPRB, 0.97528E+00_JPRB, 0.89242E+00_JPRB, 0.90405E+00_JPRB, 0.90049E+00_JPRB, 0.87742E+00_JPRB, 0.86219E+00_JPRB ,&
&  0.84220E+00_JPRB, 0.82586E+00_JPRB, 0.81988E+00_JPRB, 0.82364E+00_JPRB, 0.81202E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.74488E+00_JPRB, 0.71897E+00_JPRB, 0.69191E+00_JPRB, 0.67122E+00_JPRB, 0.65792E+00_JPRB, 0.64338E+00_JPRB, 0.61712E+00_JPRB ,&
&  0.58635E+00_JPRB, 0.56778E+00_JPRB, 0.55735E+00_JPRB, 0.54884E+00_JPRB, 0.54508E+00_JPRB, 0.54958E+00_JPRB, 0.55654E+00_JPRB ,&
&  0.56098E+00_JPRB, 0.65269E+00_JPRB, 0.50012E+00_JPRB, 0.40403E+00_JPRB, 0.36497E+00_JPRB, 0.42896E+00_JPRB, 0.55280E+00_JPRB ,&
&  0.73092E+00_JPRB, 0.10340E+01_JPRB, 0.15397E+01_JPRB, 0.19772E+01_JPRB, 0.24688E+01_JPRB, 0.29538E+01_JPRB, 0.31224E+01_JPRB ,&
&  0.33346E+01_JPRB, 0.26251E+01_JPRB, 0.23444E+01_JPRB, 0.19180E+01_JPRB, 0.16455E+01_JPRB, 0.14706E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11338E+01_JPRB, 0.10933E+01_JPRB, 0.11018E+01_JPRB, 0.11146E+01_JPRB, 0.11490E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12732E+01_JPRB, 0.13086E+01_JPRB, 0.13093E+01_JPRB, 0.13754E+01_JPRB, 0.16148E+01_JPRB, 0.19517E+01_JPRB, 0.22042E+01_JPRB ,&
&  0.21546E+01_JPRB, 0.22801E+01_JPRB, 0.37982E+01_JPRB, 0.44553E+01_JPRB, 0.46608E+01_JPRB, 0.47022E+01_JPRB, 0.43721E+01_JPRB ,&
&  0.40258E+01_JPRB, 0.37015E+01_JPRB, 0.34608E+01_JPRB, 0.29692E+01_JPRB, 0.31859E+01_JPRB, 0.31137E+01_JPRB, 0.29522E+01_JPRB ,&
&  0.26309E+01_JPRB, 0.24102E+01_JPRB, 0.21934E+01_JPRB, 0.20449E+01_JPRB, 0.19418E+01_JPRB, 0.19137E+01_JPRB, 0.17457E+01_JPRB ,&
&  0.14827E+01_JPRB, 0.15381E+01_JPRB, 0.13406E+01_JPRB, 0.11245E+01_JPRB, 0.94633E+00_JPRB, 0.88660E+00_JPRB, 0.82668E+00_JPRB ,&
&  0.76247E+00_JPRB, 0.70901E+00_JPRB, 0.72338E+00_JPRB, 0.75066E+00_JPRB, 0.76675E+00_JPRB, 0.75391E+00_JPRB, 0.73211E+00_JPRB ,&
&  0.71763E+00_JPRB, 0.85846E+00_JPRB, 0.92046E+00_JPRB, 0.92294E+00_JPRB, 0.93532E+00_JPRB, 0.89267E+00_JPRB, 0.86336E+00_JPRB ,&
&  0.83167E+00_JPRB, 0.81159E+00_JPRB, 0.79980E+00_JPRB, 0.78732E+00_JPRB, 0.76914E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.68283E+00_JPRB, 0.67123E+00_JPRB, 0.66055E+00_JPRB, 0.64664E+00_JPRB, 0.63885E+00_JPRB, 0.62853E+00_JPRB, 0.60982E+00_JPRB ,&
&  0.59025E+00_JPRB, 0.57134E+00_JPRB, 0.55536E+00_JPRB, 0.54510E+00_JPRB, 0.54081E+00_JPRB, 0.54339E+00_JPRB, 0.54371E+00_JPRB ,&
&  0.55795E+00_JPRB, 0.84557E+00_JPRB, 0.62513E+00_JPRB, 0.51714E+00_JPRB, 0.45233E+00_JPRB, 0.51727E+00_JPRB, 0.61106E+00_JPRB ,&
&  0.77826E+00_JPRB, 0.89167E+00_JPRB, 0.11039E+01_JPRB, 0.14367E+01_JPRB, 0.17428E+01_JPRB, 0.19720E+01_JPRB, 0.21344E+01_JPRB ,&
&  0.19702E+01_JPRB, 0.18083E+01_JPRB, 0.14014E+01_JPRB, 0.13100E+01_JPRB, 0.11704E+01_JPRB, 0.10025E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.99715E+00_JPRB, 0.99472E+00_JPRB, 0.98878E+00_JPRB, 0.10051E+01_JPRB, 0.10244E+01_JPRB, 0.10498E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.11167E+01_JPRB, 0.11534E+01_JPRB, 0.11849E+01_JPRB, 0.12439E+01_JPRB, 0.14150E+01_JPRB, 0.17310E+01_JPRB, 0.19617E+01_JPRB ,&
&  0.23496E+01_JPRB, 0.23916E+01_JPRB, 0.24470E+01_JPRB, 0.35502E+01_JPRB, 0.48662E+01_JPRB, 0.46693E+01_JPRB, 0.39747E+01_JPRB ,&
&  0.37206E+01_JPRB, 0.35783E+01_JPRB, 0.34741E+01_JPRB, 0.32460E+01_JPRB, 0.28961E+01_JPRB, 0.27496E+01_JPRB, 0.25797E+01_JPRB ,&
&  0.23073E+01_JPRB, 0.21017E+01_JPRB, 0.19704E+01_JPRB, 0.18920E+01_JPRB, 0.17840E+01_JPRB, 0.17370E+01_JPRB, 0.17164E+01_JPRB ,&
&  0.16386E+01_JPRB, 0.14083E+01_JPRB, 0.97811E+00_JPRB, 0.95816E+00_JPRB, 0.76983E+00_JPRB, 0.69147E+00_JPRB, 0.69725E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.65616E+00_JPRB, 0.68420E+00_JPRB, 0.66173E+00_JPRB, 0.72561E+00_JPRB, 0.66763E+00_JPRB, 0.63111E+00_JPRB ,&
&  0.61168E+00_JPRB, 0.59860E+00_JPRB, 0.78981E+00_JPRB, 0.85700E+00_JPRB, 0.87356E+00_JPRB, 0.85317E+00_JPRB, 0.81857E+00_JPRB ,&
&  0.76441E+00_JPRB, 0.73032E+00_JPRB, 0.71769E+00_JPRB, 0.71226E+00_JPRB, 0.69744E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.61577E+00_JPRB, 0.61623E+00_JPRB, 0.61064E+00_JPRB, 0.59377E+00_JPRB, 0.58170E+00_JPRB, 0.57914E+00_JPRB, 0.58455E+00_JPRB ,&
&  0.58291E+00_JPRB, 0.56621E+00_JPRB, 0.55312E+00_JPRB, 0.53857E+00_JPRB, 0.53065E+00_JPRB, 0.52782E+00_JPRB, 0.53069E+00_JPRB ,&
&  0.60886E+00_JPRB, 0.85898E+00_JPRB, 0.67071E+00_JPRB, 0.53497E+00_JPRB, 0.64307E+00_JPRB, 0.62000E+00_JPRB, 0.66439E+00_JPRB ,&
&  0.67809E+00_JPRB, 0.80860E+00_JPRB, 0.85037E+00_JPRB, 0.96020E+00_JPRB, 0.11056E+01_JPRB, 0.13108E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.99323E+00_JPRB, 0.81873E+00_JPRB, 0.78315E+00_JPRB, 0.82989E+00_JPRB, 0.74962E+00_JPRB, 0.79915E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.84280E+00_JPRB, 0.86537E+00_JPRB, 0.87601E+00_JPRB, 0.90187E+00_JPRB, 0.94119E+00_JPRB, 0.98783E+00_JPRB ,&
&  0.10456E+01_JPRB, 0.10991E+01_JPRB, 0.11310E+01_JPRB, 0.11830E+01_JPRB, 0.13749E+01_JPRB, 0.18837E+01_JPRB, 0.23426E+01_JPRB ,&
&  0.27249E+01_JPRB, 0.30879E+01_JPRB, 0.37365E+01_JPRB, 0.45597E+01_JPRB, 0.46842E+01_JPRB, 0.45773E+01_JPRB, 0.41313E+01_JPRB ,&
&  0.35515E+01_JPRB, 0.30563E+01_JPRB, 0.28001E+01_JPRB, 0.24435E+01_JPRB, 0.22255E+01_JPRB, 0.21622E+01_JPRB, 0.20620E+01_JPRB ,&
&  0.18960E+01_JPRB, 0.17799E+01_JPRB, 0.17317E+01_JPRB, 0.16204E+01_JPRB, 0.15491E+01_JPRB, 0.14621E+01_JPRB, 0.14535E+01_JPRB ,&
&  0.15435E+01_JPRB, 0.10196E+01_JPRB, 0.80753E+00_JPRB, 0.84972E+00_JPRB, 0.70743E+00_JPRB, 0.63487E+00_JPRB, 0.66884E+00_JPRB ,&
&  0.59669E+00_JPRB, 0.60121E+00_JPRB, 0.64042E+00_JPRB, 0.58760E+00_JPRB, 0.59926E+00_JPRB, 0.61520E+00_JPRB, 0.53077E+00_JPRB ,&
&  0.50612E+00_JPRB, 0.51139E+00_JPRB, 0.62387E+00_JPRB, 0.70615E+00_JPRB, 0.74200E+00_JPRB, 0.72563E+00_JPRB, 0.68988E+00_JPRB ,&
&  0.63742E+00_JPRB, 0.61626E+00_JPRB, 0.61626E+00_JPRB, 0.61550E+00_JPRB, 0.61660E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.55067E+00_JPRB, 0.55812E+00_JPRB, 0.56059E+00_JPRB, 0.56003E+00_JPRB, 0.55949E+00_JPRB, 0.55662E+00_JPRB, 0.56181E+00_JPRB ,&
&  0.56649E+00_JPRB, 0.55729E+00_JPRB, 0.54190E+00_JPRB, 0.52847E+00_JPRB, 0.51380E+00_JPRB, 0.50867E+00_JPRB, 0.53190E+00_JPRB ,&
&  0.46112E+00_JPRB, 0.59562E+00_JPRB, 0.49274E+00_JPRB, 0.52515E+00_JPRB, 0.80494E+00_JPRB, 0.70322E+00_JPRB, 0.67630E+00_JPRB ,&
&  0.62470E+00_JPRB, 0.62489E+00_JPRB, 0.62399E+00_JPRB, 0.70022E+00_JPRB, 0.76365E+00_JPRB, 0.86197E+00_JPRB, 0.91558E+00_JPRB ,&
&  0.78972E+00_JPRB, 0.61108E+00_JPRB, 0.53540E+00_JPRB, 0.52082E+00_JPRB, 0.56612E+00_JPRB, 0.60192E+00_JPRB, 0.67335E+00_JPRB ,&
&  0.70978E+00_JPRB, 0.73842E+00_JPRB, 0.76340E+00_JPRB, 0.79986E+00_JPRB, 0.82022E+00_JPRB, 0.84546E+00_JPRB, 0.88331E+00_JPRB ,&
&  0.92953E+00_JPRB, 0.99222E+00_JPRB, 0.10553E+01_JPRB, 0.11859E+01_JPRB, 0.13252E+01_JPRB, 0.20843E+01_JPRB, 0.33766E+01_JPRB ,&
&  0.36531E+01_JPRB, 0.37993E+01_JPRB, 0.44145E+01_JPRB, 0.54165E+01_JPRB, 0.53524E+01_JPRB, 0.48620E+01_JPRB, 0.41059E+01_JPRB ,&
&  0.34020E+01_JPRB, 0.27711E+01_JPRB, 0.24997E+01_JPRB, 0.22252E+01_JPRB, 0.19560E+01_JPRB, 0.19365E+01_JPRB, 0.18349E+01_JPRB ,&
&  0.16911E+01_JPRB, 0.16375E+01_JPRB, 0.15857E+01_JPRB, 0.15654E+01_JPRB, 0.14668E+01_JPRB, 0.14402E+01_JPRB, 0.13744E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.12118E+01_JPRB, 0.91593E+00_JPRB, 0.77968E+00_JPRB, 0.65890E+00_JPRB, 0.61988E+00_JPRB, 0.71465E+00_JPRB ,&
&  0.63351E+00_JPRB, 0.58578E+00_JPRB, 0.55071E+00_JPRB, 0.50650E+00_JPRB, 0.49238E+00_JPRB, 0.51284E+00_JPRB, 0.45997E+00_JPRB ,&
&  0.43936E+00_JPRB, 0.48610E+00_JPRB, 0.53385E+00_JPRB, 0.58309E+00_JPRB, 0.60862E+00_JPRB, 0.60735E+00_JPRB, 0.52436E+00_JPRB ,&
&  0.53999E+00_JPRB, 0.52882E+00_JPRB, 0.53386E+00_JPRB, 0.53134E+00_JPRB, 0.53779E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.50640E+00_JPRB, 0.51570E+00_JPRB, 0.52344E+00_JPRB, 0.53529E+00_JPRB, 0.54928E+00_JPRB, 0.54725E+00_JPRB, 0.54695E+00_JPRB ,&
&  0.55271E+00_JPRB, 0.54670E+00_JPRB, 0.52823E+00_JPRB, 0.51549E+00_JPRB, 0.50002E+00_JPRB, 0.49988E+00_JPRB, 0.44607E+00_JPRB ,&
&  0.40551E+00_JPRB, 0.47878E+00_JPRB, 0.46495E+00_JPRB, 0.51327E+00_JPRB, 0.60298E+00_JPRB, 0.59353E+00_JPRB, 0.59951E+00_JPRB ,&
&  0.58765E+00_JPRB, 0.54906E+00_JPRB, 0.53394E+00_JPRB, 0.54699E+00_JPRB, 0.56371E+00_JPRB, 0.61308E+00_JPRB, 0.67218E+00_JPRB ,&
&  0.60335E+00_JPRB, 0.49450E+00_JPRB, 0.44629E+00_JPRB, 0.44942E+00_JPRB, 0.50651E+00_JPRB, 0.56105E+00_JPRB, 0.59657E+00_JPRB ,&
&  0.62677E+00_JPRB, 0.65932E+00_JPRB, 0.69941E+00_JPRB, 0.74768E+00_JPRB, 0.77280E+00_JPRB, 0.79498E+00_JPRB, 0.82025E+00_JPRB ,&
&  0.85006E+00_JPRB, 0.90076E+00_JPRB, 0.97196E+00_JPRB, 0.11040E+01_JPRB, 0.12614E+01_JPRB, 0.20053E+01_JPRB, 0.29780E+01_JPRB ,&
&  0.32783E+01_JPRB, 0.35235E+01_JPRB, 0.38985E+01_JPRB, 0.42812E+01_JPRB, 0.42792E+01_JPRB, 0.38123E+01_JPRB, 0.32944E+01_JPRB ,&
&  0.29206E+01_JPRB, 0.24642E+01_JPRB, 0.24089E+01_JPRB, 0.21006E+01_JPRB, 0.18580E+01_JPRB, 0.17808E+01_JPRB, 0.16542E+01_JPRB ,&
&  0.15711E+01_JPRB, 0.15623E+01_JPRB, 0.14742E+01_JPRB, 0.14146E+01_JPRB, 0.12967E+01_JPRB, 0.12621E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11404E+01_JPRB, 0.97105E+00_JPRB, 0.82491E+00_JPRB, 0.69378E+00_JPRB, 0.66420E+00_JPRB, 0.60798E+00_JPRB ,&
&  0.56823E+00_JPRB, 0.52065E+00_JPRB, 0.48311E+00_JPRB, 0.43994E+00_JPRB, 0.42007E+00_JPRB, 0.41771E+00_JPRB, 0.41145E+00_JPRB ,&
&  0.44458E+00_JPRB, 0.49825E+00_JPRB, 0.49756E+00_JPRB, 0.49875E+00_JPRB, 0.50626E+00_JPRB, 0.52533E+00_JPRB, 0.42780E+00_JPRB ,&
&  0.43298E+00_JPRB, 0.48866E+00_JPRB, 0.48831E+00_JPRB, 0.48732E+00_JPRB, 0.49116E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.47781E+00_JPRB, 0.48911E+00_JPRB, 0.50121E+00_JPRB, 0.52274E+00_JPRB, 0.54193E+00_JPRB, 0.54825E+00_JPRB, 0.51163E+00_JPRB ,&
&  0.55079E+00_JPRB, 0.52890E+00_JPRB, 0.51065E+00_JPRB, 0.50781E+00_JPRB, 0.50067E+00_JPRB, 0.43202E+00_JPRB, 0.36154E+00_JPRB ,&
&  0.35423E+00_JPRB, 0.38439E+00_JPRB, 0.43281E+00_JPRB, 0.46897E+00_JPRB, 0.50955E+00_JPRB, 0.51768E+00_JPRB, 0.51786E+00_JPRB ,&
&  0.51009E+00_JPRB, 0.48674E+00_JPRB, 0.48426E+00_JPRB, 0.48429E+00_JPRB, 0.48189E+00_JPRB, 0.48727E+00_JPRB, 0.51435E+00_JPRB ,&
&  0.47814E+00_JPRB, 0.44987E+00_JPRB, 0.45251E+00_JPRB, 0.43938E+00_JPRB, 0.49893E+00_JPRB, 0.52277E+00_JPRB, 0.55558E+00_JPRB ,&
&  0.59257E+00_JPRB, 0.63166E+00_JPRB, 0.69659E+00_JPRB, 0.74557E+00_JPRB, 0.77849E+00_JPRB, 0.79889E+00_JPRB, 0.83024E+00_JPRB ,&
&  0.85166E+00_JPRB, 0.89291E+00_JPRB, 0.97125E+00_JPRB, 0.11189E+01_JPRB, 0.13679E+01_JPRB, 0.18416E+01_JPRB, 0.27026E+01_JPRB ,&
&  0.29786E+01_JPRB, 0.33432E+01_JPRB, 0.36688E+01_JPRB, 0.32421E+01_JPRB, 0.33557E+01_JPRB, 0.34236E+01_JPRB, 0.31384E+01_JPRB ,&
&  0.29211E+01_JPRB, 0.24377E+01_JPRB, 0.21668E+01_JPRB, 0.19503E+01_JPRB, 0.17822E+01_JPRB, 0.15464E+01_JPRB, 0.13659E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.14636E+01_JPRB, 0.11710E+01_JPRB, 0.10468E+01_JPRB, 0.95013E+00_JPRB, 0.91804E+00_JPRB, 0.87127E+00_JPRB ,&
&  0.85587E+00_JPRB, 0.84853E+00_JPRB, 0.80302E+00_JPRB, 0.74656E+00_JPRB, 0.65676E+00_JPRB, 0.61730E+00_JPRB, 0.57713E+00_JPRB ,&
&  0.55719E+00_JPRB, 0.51932E+00_JPRB, 0.46188E+00_JPRB, 0.42997E+00_JPRB, 0.40573E+00_JPRB, 0.38573E+00_JPRB, 0.38586E+00_JPRB ,&
&  0.40912E+00_JPRB, 0.42785E+00_JPRB, 0.48483E+00_JPRB, 0.48461E+00_JPRB, 0.48797E+00_JPRB, 0.51132E+00_JPRB, 0.45240E+00_JPRB ,&
&  0.42899E+00_JPRB, 0.49222E+00_JPRB, 0.47827E+00_JPRB, 0.46810E+00_JPRB, 0.46804E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.45980E+00_JPRB, 0.48046E+00_JPRB, 0.49639E+00_JPRB, 0.51678E+00_JPRB, 0.53995E+00_JPRB, 0.49901E+00_JPRB, 0.46819E+00_JPRB ,&
&  0.53002E+00_JPRB, 0.45422E+00_JPRB, 0.50122E+00_JPRB, 0.41263E+00_JPRB, 0.37238E+00_JPRB, 0.36050E+00_JPRB, 0.35400E+00_JPRB ,&
&  0.35185E+00_JPRB, 0.37975E+00_JPRB, 0.40757E+00_JPRB, 0.43065E+00_JPRB, 0.47430E+00_JPRB, 0.47949E+00_JPRB, 0.46941E+00_JPRB ,&
&  0.45170E+00_JPRB, 0.43330E+00_JPRB, 0.43475E+00_JPRB, 0.43254E+00_JPRB, 0.44029E+00_JPRB, 0.44806E+00_JPRB, 0.45179E+00_JPRB ,&
&  0.42611E+00_JPRB, 0.42580E+00_JPRB, 0.45210E+00_JPRB, 0.47178E+00_JPRB, 0.48628E+00_JPRB, 0.49614E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.55225E+00_JPRB, 0.56765E+00_JPRB, 0.71988E+00_JPRB, 0.75917E+00_JPRB, 0.80209E+00_JPRB, 0.82899E+00_JPRB, 0.86152E+00_JPRB ,&
&  0.90317E+00_JPRB, 0.95517E+00_JPRB, 0.10096E+01_JPRB, 0.10918E+01_JPRB, 0.12626E+01_JPRB, 0.16585E+01_JPRB, 0.21389E+01_JPRB ,&
&  0.29015E+01_JPRB, 0.21913E+01_JPRB, 0.26884E+01_JPRB, 0.26021E+01_JPRB, 0.28287E+01_JPRB, 0.29875E+01_JPRB, 0.28699E+01_JPRB ,&
&  0.28183E+01_JPRB, 0.23553E+01_JPRB, 0.19580E+01_JPRB, 0.16028E+01_JPRB, 0.13470E+01_JPRB, 0.12476E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.10494E+01_JPRB, 0.90068E+00_JPRB, 0.81850E+00_JPRB, 0.78887E+00_JPRB, 0.77222E+00_JPRB, 0.71697E+00_JPRB ,&
&  0.68286E+00_JPRB, 0.65573E+00_JPRB, 0.61822E+00_JPRB, 0.57998E+00_JPRB, 0.55306E+00_JPRB, 0.53644E+00_JPRB, 0.51724E+00_JPRB ,&
&  0.51205E+00_JPRB, 0.49771E+00_JPRB, 0.45688E+00_JPRB, 0.43618E+00_JPRB, 0.41918E+00_JPRB, 0.39799E+00_JPRB, 0.38941E+00_JPRB ,&
&  0.38319E+00_JPRB, 0.37313E+00_JPRB, 0.39169E+00_JPRB, 0.41997E+00_JPRB, 0.42620E+00_JPRB, 0.43181E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.45067E+00_JPRB, 0.46576E+00_JPRB, 0.46855E+00_JPRB, 0.44730E+00_JPRB, 0.44434E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.44161E+00_JPRB, 0.46295E+00_JPRB, 0.48916E+00_JPRB, 0.50991E+00_JPRB, 0.53154E+00_JPRB, 0.48494E+00_JPRB, 0.43852E+00_JPRB ,&
&  0.46399E+00_JPRB, 0.40538E+00_JPRB, 0.37341E+00_JPRB, 0.36105E+00_JPRB, 0.35953E+00_JPRB, 0.35213E+00_JPRB, 0.34354E+00_JPRB ,&
&  0.34246E+00_JPRB, 0.35521E+00_JPRB, 0.38016E+00_JPRB, 0.39918E+00_JPRB, 0.41697E+00_JPRB, 0.42451E+00_JPRB, 0.41818E+00_JPRB ,&
&  0.40974E+00_JPRB, 0.40411E+00_JPRB, 0.40107E+00_JPRB, 0.40235E+00_JPRB, 0.40585E+00_JPRB, 0.41250E+00_JPRB, 0.41521E+00_JPRB ,&
&  0.41411E+00_JPRB, 0.41771E+00_JPRB, 0.43025E+00_JPRB, 0.45477E+00_JPRB, 0.45950E+00_JPRB, 0.46232E+00_JPRB, 0.46539E+00_JPRB ,&
&  0.38787E+00_JPRB, 0.45089E+00_JPRB, 0.68869E+00_JPRB, 0.75447E+00_JPRB, 0.79473E+00_JPRB, 0.85027E+00_JPRB, 0.90213E+00_JPRB ,&
&  0.95503E+00_JPRB, 0.10266E+01_JPRB, 0.11159E+01_JPRB, 0.12135E+01_JPRB, 0.13231E+01_JPRB, 0.15014E+01_JPRB, 0.17717E+01_JPRB ,&
&  0.23902E+01_JPRB, 0.19577E+01_JPRB, 0.18173E+01_JPRB, 0.18657E+01_JPRB, 0.25834E+01_JPRB, 0.25653E+01_JPRB, 0.22030E+01_JPRB ,&
&  0.19647E+01_JPRB, 0.17732E+01_JPRB, 0.16098E+01_JPRB, 0.14116E+01_JPRB, 0.12669E+01_JPRB, 0.12046E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.98912E+00_JPRB, 0.91450E+00_JPRB, 0.80003E+00_JPRB, 0.76465E+00_JPRB, 0.76045E+00_JPRB, 0.71730E+00_JPRB, 0.65704E+00_JPRB ,&
&  0.60368E+00_JPRB, 0.56047E+00_JPRB, 0.52744E+00_JPRB, 0.50369E+00_JPRB, 0.48500E+00_JPRB, 0.47520E+00_JPRB, 0.46969E+00_JPRB ,&
&  0.46669E+00_JPRB, 0.46347E+00_JPRB, 0.44829E+00_JPRB, 0.43942E+00_JPRB, 0.42616E+00_JPRB, 0.40860E+00_JPRB, 0.39653E+00_JPRB ,&
&  0.38233E+00_JPRB, 0.37166E+00_JPRB, 0.35888E+00_JPRB, 0.36119E+00_JPRB, 0.37145E+00_JPRB, 0.39185E+00_JPRB, 0.41195E+00_JPRB ,&
&  0.43739E+00_JPRB, 0.43975E+00_JPRB, 0.41628E+00_JPRB, 0.40980E+00_JPRB, 0.41434E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.41196E+00_JPRB, 0.45064E+00_JPRB, 0.48267E+00_JPRB, 0.50817E+00_JPRB, 0.50667E+00_JPRB, 0.48577E+00_JPRB, 0.43348E+00_JPRB ,&
&  0.42138E+00_JPRB, 0.39548E+00_JPRB, 0.38313E+00_JPRB, 0.36898E+00_JPRB, 0.35939E+00_JPRB, 0.35210E+00_JPRB, 0.35081E+00_JPRB ,&
&  0.35695E+00_JPRB, 0.36812E+00_JPRB, 0.37931E+00_JPRB, 0.38566E+00_JPRB, 0.39058E+00_JPRB, 0.39104E+00_JPRB, 0.38998E+00_JPRB ,&
&  0.38788E+00_JPRB, 0.38627E+00_JPRB, 0.38594E+00_JPRB, 0.38756E+00_JPRB, 0.39333E+00_JPRB, 0.40029E+00_JPRB, 0.40086E+00_JPRB ,&
&  0.39512E+00_JPRB, 0.38961E+00_JPRB, 0.40393E+00_JPRB, 0.42758E+00_JPRB, 0.42556E+00_JPRB, 0.42477E+00_JPRB, 0.38667E+00_JPRB ,&
&  0.35155E+00_JPRB, 0.36927E+00_JPRB, 0.46157E+00_JPRB, 0.66676E+00_JPRB, 0.73051E+00_JPRB, 0.79461E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.79822E+00_JPRB, 0.83129E+00_JPRB, 0.93958E+00_JPRB, 0.11986E+01_JPRB, 0.12725E+01_JPRB, 0.13915E+01_JPRB, 0.15180E+01_JPRB ,&
&  0.17830E+01_JPRB, 0.19556E+01_JPRB, 0.20215E+01_JPRB, 0.16129E+01_JPRB, 0.17563E+01_JPRB, 0.21191E+01_JPRB, 0.17927E+01_JPRB ,&
&  0.16231E+01_JPRB, 0.15061E+01_JPRB, 0.14873E+01_JPRB, 0.13300E+01_JPRB, 0.11696E+01_JPRB, 0.10644E+01_JPRB, 0.10056E+01_JPRB ,&
&  0.94360E+00_JPRB, 0.88438E+00_JPRB, 0.76250E+00_JPRB, 0.70037E+00_JPRB, 0.69014E+00_JPRB, 0.66830E+00_JPRB, 0.64655E+00_JPRB ,&
&  0.61579E+00_JPRB, 0.56255E+00_JPRB, 0.51319E+00_JPRB, 0.46507E+00_JPRB, 0.43443E+00_JPRB, 0.42918E+00_JPRB, 0.43595E+00_JPRB ,&
&  0.44174E+00_JPRB, 0.44054E+00_JPRB, 0.43669E+00_JPRB, 0.42916E+00_JPRB, 0.41238E+00_JPRB, 0.38943E+00_JPRB, 0.37399E+00_JPRB ,&
&  0.36059E+00_JPRB, 0.34790E+00_JPRB, 0.34095E+00_JPRB, 0.34516E+00_JPRB, 0.35288E+00_JPRB, 0.36654E+00_JPRB, 0.38033E+00_JPRB ,&
&  0.38815E+00_JPRB, 0.39470E+00_JPRB, 0.39013E+00_JPRB, 0.39089E+00_JPRB, 0.38997E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.37501E+00_JPRB, 0.41372E+00_JPRB, 0.47093E+00_JPRB, 0.50407E+00_JPRB, 0.50839E+00_JPRB, 0.47076E+00_JPRB, 0.42964E+00_JPRB ,&
&  0.40656E+00_JPRB, 0.39307E+00_JPRB, 0.39035E+00_JPRB, 0.38843E+00_JPRB, 0.38805E+00_JPRB, 0.38886E+00_JPRB, 0.39055E+00_JPRB ,&
&  0.39055E+00_JPRB, 0.38958E+00_JPRB, 0.38936E+00_JPRB, 0.39090E+00_JPRB, 0.39500E+00_JPRB, 0.39504E+00_JPRB, 0.39393E+00_JPRB ,&
&  0.39331E+00_JPRB, 0.39313E+00_JPRB, 0.39102E+00_JPRB, 0.38939E+00_JPRB, 0.39113E+00_JPRB, 0.39661E+00_JPRB, 0.39923E+00_JPRB ,&
&  0.39171E+00_JPRB, 0.38076E+00_JPRB, 0.38897E+00_JPRB, 0.40837E+00_JPRB, 0.40688E+00_JPRB, 0.40296E+00_JPRB, 0.37021E+00_JPRB ,&
&  0.34410E+00_JPRB, 0.32702E+00_JPRB, 0.33751E+00_JPRB, 0.38948E+00_JPRB, 0.48380E+00_JPRB, 0.66223E+00_JPRB, 0.74849E+00_JPRB ,&
&  0.83275E+00_JPRB, 0.89917E+00_JPRB, 0.96137E+00_JPRB, 0.10244E+01_JPRB, 0.11094E+01_JPRB, 0.11908E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.14225E+01_JPRB, 0.15756E+01_JPRB, 0.11988E+01_JPRB, 0.11222E+01_JPRB, 0.12143E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.11883E+01_JPRB, 0.11419E+01_JPRB, 0.11325E+01_JPRB, 0.10320E+01_JPRB, 0.94482E+00_JPRB, 0.88395E+00_JPRB ,&
&  0.83540E+00_JPRB, 0.80702E+00_JPRB, 0.73264E+00_JPRB, 0.64331E+00_JPRB, 0.61528E+00_JPRB, 0.61173E+00_JPRB, 0.61905E+00_JPRB ,&
&  0.63925E+00_JPRB, 0.64269E+00_JPRB, 0.57725E+00_JPRB, 0.45731E+00_JPRB, 0.42490E+00_JPRB, 0.41677E+00_JPRB, 0.42428E+00_JPRB ,&
&  0.43049E+00_JPRB, 0.42952E+00_JPRB, 0.42853E+00_JPRB, 0.42168E+00_JPRB, 0.40931E+00_JPRB, 0.39250E+00_JPRB, 0.37653E+00_JPRB ,&
&  0.36289E+00_JPRB, 0.35800E+00_JPRB, 0.36219E+00_JPRB, 0.36594E+00_JPRB, 0.37800E+00_JPRB, 0.38976E+00_JPRB, 0.40188E+00_JPRB ,&
&  0.39972E+00_JPRB, 0.39482E+00_JPRB, 0.38825E+00_JPRB, 0.37277E+00_JPRB, 0.36248E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.38626E+00_JPRB, 0.41839E+00_JPRB, 0.46207E+00_JPRB, 0.49267E+00_JPRB, 0.49717E+00_JPRB, 0.47349E+00_JPRB, 0.45260E+00_JPRB ,&
&  0.44077E+00_JPRB, 0.43730E+00_JPRB, 0.43449E+00_JPRB, 0.43090E+00_JPRB, 0.42735E+00_JPRB, 0.42366E+00_JPRB, 0.42005E+00_JPRB ,&
&  0.41760E+00_JPRB, 0.41631E+00_JPRB, 0.41478E+00_JPRB, 0.41004E+00_JPRB, 0.40670E+00_JPRB, 0.40647E+00_JPRB, 0.40876E+00_JPRB ,&
&  0.41199E+00_JPRB, 0.41134E+00_JPRB, 0.41042E+00_JPRB, 0.40582E+00_JPRB, 0.39567E+00_JPRB, 0.38402E+00_JPRB, 0.37481E+00_JPRB ,&
&  0.36873E+00_JPRB, 0.37254E+00_JPRB, 0.38117E+00_JPRB, 0.39674E+00_JPRB, 0.39454E+00_JPRB, 0.38837E+00_JPRB, 0.35745E+00_JPRB ,&
&  0.33707E+00_JPRB, 0.31394E+00_JPRB, 0.30474E+00_JPRB, 0.31350E+00_JPRB, 0.34539E+00_JPRB, 0.39645E+00_JPRB, 0.48115E+00_JPRB ,&
&  0.63559E+00_JPRB, 0.69713E+00_JPRB, 0.74677E+00_JPRB, 0.80468E+00_JPRB, 0.86097E+00_JPRB, 0.91486E+00_JPRB, 0.95913E+00_JPRB ,&
&  0.10196E+01_JPRB, 0.10908E+01_JPRB, 0.11656E+01_JPRB, 0.11461E+01_JPRB, 0.11476E+01_JPRB, 0.10551E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.14284E+01_JPRB, 0.12438E+01_JPRB, 0.10737E+01_JPRB, 0.96243E+00_JPRB, 0.88507E+00_JPRB, 0.81537E+00_JPRB, 0.75891E+00_JPRB ,&
&  0.71163E+00_JPRB, 0.68454E+00_JPRB, 0.66441E+00_JPRB, 0.61470E+00_JPRB, 0.56891E+00_JPRB, 0.55478E+00_JPRB, 0.57260E+00_JPRB ,&
&  0.63291E+00_JPRB, 0.72860E+00_JPRB, 0.87302E+00_JPRB, 0.61824E+00_JPRB, 0.53592E+00_JPRB, 0.49113E+00_JPRB, 0.47232E+00_JPRB ,&
&  0.46192E+00_JPRB, 0.45179E+00_JPRB, 0.44693E+00_JPRB, 0.43838E+00_JPRB, 0.42647E+00_JPRB, 0.41482E+00_JPRB, 0.40469E+00_JPRB ,&
&  0.39812E+00_JPRB, 0.39999E+00_JPRB, 0.40677E+00_JPRB, 0.41300E+00_JPRB, 0.41691E+00_JPRB, 0.41797E+00_JPRB, 0.41938E+00_JPRB ,&
&  0.41759E+00_JPRB, 0.41108E+00_JPRB, 0.40374E+00_JPRB, 0.39003E+00_JPRB, 0.38044E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.40526E+00_JPRB, 0.42904E+00_JPRB, 0.45760E+00_JPRB, 0.47700E+00_JPRB, 0.48726E+00_JPRB, 0.47646E+00_JPRB, 0.46422E+00_JPRB ,&
&  0.45422E+00_JPRB, 0.44688E+00_JPRB, 0.44199E+00_JPRB, 0.43855E+00_JPRB, 0.43436E+00_JPRB, 0.43047E+00_JPRB, 0.42701E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.41640E+00_JPRB, 0.41228E+00_JPRB, 0.41311E+00_JPRB, 0.41502E+00_JPRB, 0.41847E+00_JPRB, 0.42221E+00_JPRB ,&
&  0.42111E+00_JPRB, 0.41597E+00_JPRB, 0.41206E+00_JPRB, 0.39924E+00_JPRB, 0.38438E+00_JPRB, 0.37558E+00_JPRB, 0.37924E+00_JPRB ,&
&  0.37816E+00_JPRB, 0.36815E+00_JPRB, 0.38378E+00_JPRB, 0.39723E+00_JPRB, 0.38696E+00_JPRB, 0.37671E+00_JPRB, 0.32737E+00_JPRB ,&
&  0.31338E+00_JPRB, 0.29973E+00_JPRB, 0.29903E+00_JPRB, 0.30350E+00_JPRB, 0.32357E+00_JPRB, 0.36319E+00_JPRB, 0.42319E+00_JPRB ,&
&  0.48722E+00_JPRB, 0.59330E+00_JPRB, 0.62109E+00_JPRB, 0.67264E+00_JPRB, 0.72568E+00_JPRB, 0.77646E+00_JPRB, 0.81828E+00_JPRB ,&
&  0.87666E+00_JPRB, 0.95508E+00_JPRB, 0.10092E+01_JPRB, 0.10262E+01_JPRB, 0.10491E+01_JPRB, 0.10611E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.96097E+00_JPRB, 0.85120E+00_JPRB, 0.77377E+00_JPRB, 0.72202E+00_JPRB, 0.69923E+00_JPRB, 0.64772E+00_JPRB ,&
&  0.62595E+00_JPRB, 0.61488E+00_JPRB, 0.59901E+00_JPRB, 0.59138E+00_JPRB, 0.58118E+00_JPRB, 0.56502E+00_JPRB, 0.53989E+00_JPRB ,&
&  0.51516E+00_JPRB, 0.51104E+00_JPRB, 0.55788E+00_JPRB, 0.56833E+00_JPRB, 0.56477E+00_JPRB, 0.53444E+00_JPRB, 0.49622E+00_JPRB ,&
&  0.46370E+00_JPRB, 0.44606E+00_JPRB, 0.43982E+00_JPRB, 0.43551E+00_JPRB, 0.43257E+00_JPRB, 0.43029E+00_JPRB, 0.43458E+00_JPRB ,&
&  0.43197E+00_JPRB, 0.43598E+00_JPRB, 0.44075E+00_JPRB, 0.44281E+00_JPRB, 0.44010E+00_JPRB, 0.43373E+00_JPRB, 0.42804E+00_JPRB ,&
&  0.42217E+00_JPRB, 0.41622E+00_JPRB, 0.41085E+00_JPRB, 0.40416E+00_JPRB, 0.39984E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.42102E+00_JPRB, 0.43298E+00_JPRB, 0.45244E+00_JPRB, 0.46749E+00_JPRB, 0.47866E+00_JPRB, 0.47667E+00_JPRB, 0.46798E+00_JPRB ,&
&  0.45949E+00_JPRB, 0.45296E+00_JPRB, 0.44679E+00_JPRB, 0.44068E+00_JPRB, 0.43351E+00_JPRB, 0.42779E+00_JPRB, 0.42359E+00_JPRB ,&
&  0.42150E+00_JPRB, 0.42096E+00_JPRB, 0.42139E+00_JPRB, 0.42167E+00_JPRB, 0.42243E+00_JPRB, 0.42368E+00_JPRB, 0.42455E+00_JPRB ,&
&  0.42563E+00_JPRB, 0.42108E+00_JPRB, 0.40579E+00_JPRB, 0.38929E+00_JPRB, 0.38261E+00_JPRB, 0.37565E+00_JPRB, 0.38233E+00_JPRB ,&
&  0.38383E+00_JPRB, 0.38677E+00_JPRB, 0.38767E+00_JPRB, 0.38979E+00_JPRB, 0.37838E+00_JPRB, 0.31476E+00_JPRB, 0.31486E+00_JPRB ,&
&  0.30672E+00_JPRB, 0.29888E+00_JPRB, 0.30134E+00_JPRB, 0.30631E+00_JPRB, 0.31720E+00_JPRB, 0.34165E+00_JPRB, 0.38235E+00_JPRB ,&
&  0.42187E+00_JPRB, 0.56247E+00_JPRB, 0.59916E+00_JPRB, 0.63816E+00_JPRB, 0.67297E+00_JPRB, 0.71775E+00_JPRB, 0.77260E+00_JPRB ,&
&  0.81219E+00_JPRB, 0.85704E+00_JPRB, 0.89147E+00_JPRB, 0.91882E+00_JPRB, 0.92594E+00_JPRB, 0.91991E+00_JPRB, 0.91127E+00_JPRB ,&
&  0.88294E+00_JPRB, 0.79234E+00_JPRB, 0.72609E+00_JPRB, 0.68274E+00_JPRB, 0.64123E+00_JPRB, 0.61243E+00_JPRB, 0.59182E+00_JPRB ,&
&  0.58674E+00_JPRB, 0.56380E+00_JPRB, 0.55533E+00_JPRB, 0.53770E+00_JPRB, 0.52460E+00_JPRB, 0.51146E+00_JPRB, 0.49549E+00_JPRB ,&
&  0.48154E+00_JPRB, 0.47856E+00_JPRB, 0.48569E+00_JPRB, 0.49510E+00_JPRB, 0.50137E+00_JPRB, 0.49623E+00_JPRB, 0.48627E+00_JPRB ,&
&  0.46970E+00_JPRB, 0.46513E+00_JPRB, 0.46087E+00_JPRB, 0.45921E+00_JPRB, 0.45634E+00_JPRB, 0.45744E+00_JPRB, 0.45970E+00_JPRB ,&
&  0.46189E+00_JPRB, 0.46116E+00_JPRB, 0.45966E+00_JPRB, 0.45864E+00_JPRB, 0.45657E+00_JPRB, 0.45256E+00_JPRB, 0.44672E+00_JPRB ,&
&  0.44058E+00_JPRB, 0.43671E+00_JPRB, 0.42963E+00_JPRB, 0.42338E+00_JPRB, 0.41976E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.43560E+00_JPRB, 0.43892E+00_JPRB, 0.44550E+00_JPRB, 0.45076E+00_JPRB, 0.45450E+00_JPRB, 0.45424E+00_JPRB, 0.44848E+00_JPRB ,&
&  0.44307E+00_JPRB, 0.43823E+00_JPRB, 0.43436E+00_JPRB, 0.43157E+00_JPRB, 0.43055E+00_JPRB, 0.42803E+00_JPRB, 0.42467E+00_JPRB ,&
&  0.42190E+00_JPRB, 0.42026E+00_JPRB, 0.41971E+00_JPRB, 0.41913E+00_JPRB, 0.42023E+00_JPRB, 0.41973E+00_JPRB, 0.42007E+00_JPRB ,&
&  0.42021E+00_JPRB, 0.41239E+00_JPRB, 0.39151E+00_JPRB, 0.38099E+00_JPRB, 0.37329E+00_JPRB, 0.37474E+00_JPRB, 0.37775E+00_JPRB ,&
&  0.38282E+00_JPRB, 0.33864E+00_JPRB, 0.34130E+00_JPRB, 0.32987E+00_JPRB, 0.32127E+00_JPRB, 0.31992E+00_JPRB, 0.31415E+00_JPRB ,&
&  0.31099E+00_JPRB, 0.30718E+00_JPRB, 0.31093E+00_JPRB, 0.31910E+00_JPRB, 0.32610E+00_JPRB, 0.34144E+00_JPRB, 0.36632E+00_JPRB ,&
&  0.40541E+00_JPRB, 0.52725E+00_JPRB, 0.58008E+00_JPRB, 0.61608E+00_JPRB, 0.64291E+00_JPRB, 0.67553E+00_JPRB, 0.70844E+00_JPRB ,&
&  0.73456E+00_JPRB, 0.76755E+00_JPRB, 0.78427E+00_JPRB, 0.78058E+00_JPRB, 0.79265E+00_JPRB, 0.78205E+00_JPRB, 0.75658E+00_JPRB ,&
&  0.71447E+00_JPRB, 0.66042E+00_JPRB, 0.61893E+00_JPRB, 0.59127E+00_JPRB, 0.56862E+00_JPRB, 0.55065E+00_JPRB, 0.53876E+00_JPRB ,&
&  0.53894E+00_JPRB, 0.52809E+00_JPRB, 0.51428E+00_JPRB, 0.49268E+00_JPRB, 0.47626E+00_JPRB, 0.46567E+00_JPRB, 0.46086E+00_JPRB ,&
&  0.45997E+00_JPRB, 0.46424E+00_JPRB, 0.47283E+00_JPRB, 0.48119E+00_JPRB, 0.48244E+00_JPRB, 0.47982E+00_JPRB, 0.48143E+00_JPRB ,&
&  0.48014E+00_JPRB, 0.47749E+00_JPRB, 0.47496E+00_JPRB, 0.47156E+00_JPRB, 0.46583E+00_JPRB, 0.46036E+00_JPRB, 0.45747E+00_JPRB ,&
&  0.45666E+00_JPRB, 0.45548E+00_JPRB, 0.45452E+00_JPRB, 0.45470E+00_JPRB, 0.45692E+00_JPRB, 0.45869E+00_JPRB, 0.45861E+00_JPRB ,&
&  0.45156E+00_JPRB, 0.44462E+00_JPRB, 0.43815E+00_JPRB, 0.43639E+00_JPRB, 0.43544E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.42260E+00_JPRB, 0.42262E+00_JPRB, 0.42338E+00_JPRB, 0.42385E+00_JPRB, 0.42399E+00_JPRB, 0.42411E+00_JPRB, 0.42407E+00_JPRB ,&
&  0.42429E+00_JPRB, 0.42519E+00_JPRB, 0.42575E+00_JPRB, 0.42552E+00_JPRB, 0.42613E+00_JPRB, 0.42733E+00_JPRB, 0.42746E+00_JPRB ,&
&  0.42474E+00_JPRB, 0.42110E+00_JPRB, 0.41769E+00_JPRB, 0.41374E+00_JPRB, 0.41034E+00_JPRB, 0.40779E+00_JPRB, 0.40399E+00_JPRB ,&
&  0.40156E+00_JPRB, 0.39578E+00_JPRB, 0.37037E+00_JPRB, 0.36944E+00_JPRB, 0.36409E+00_JPRB, 0.36328E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36311E+00_JPRB, 0.36377E+00_JPRB, 0.35729E+00_JPRB, 0.34529E+00_JPRB, 0.33688E+00_JPRB, 0.32491E+00_JPRB, 0.32413E+00_JPRB ,&
&  0.32242E+00_JPRB, 0.32604E+00_JPRB, 0.33547E+00_JPRB, 0.34138E+00_JPRB, 0.34584E+00_JPRB, 0.35202E+00_JPRB, 0.36824E+00_JPRB ,&
&  0.40190E+00_JPRB, 0.44970E+00_JPRB, 0.51156E+00_JPRB, 0.54261E+00_JPRB, 0.56838E+00_JPRB, 0.59452E+00_JPRB, 0.61892E+00_JPRB ,&
&  0.63776E+00_JPRB, 0.65333E+00_JPRB, 0.65679E+00_JPRB, 0.65666E+00_JPRB, 0.64514E+00_JPRB, 0.62313E+00_JPRB, 0.59693E+00_JPRB ,&
&  0.56935E+00_JPRB, 0.54930E+00_JPRB, 0.53351E+00_JPRB, 0.52339E+00_JPRB, 0.51533E+00_JPRB, 0.50651E+00_JPRB, 0.49917E+00_JPRB ,&
&  0.48962E+00_JPRB, 0.47269E+00_JPRB, 0.45649E+00_JPRB, 0.44071E+00_JPRB, 0.43199E+00_JPRB, 0.42624E+00_JPRB, 0.42542E+00_JPRB ,&
&  0.42647E+00_JPRB, 0.43120E+00_JPRB, 0.43738E+00_JPRB, 0.43952E+00_JPRB, 0.44409E+00_JPRB, 0.45063E+00_JPRB, 0.45113E+00_JPRB ,&
&  0.45205E+00_JPRB, 0.45192E+00_JPRB, 0.44963E+00_JPRB, 0.44599E+00_JPRB, 0.44048E+00_JPRB, 0.43465E+00_JPRB, 0.42905E+00_JPRB ,&
&  0.42548E+00_JPRB, 0.42364E+00_JPRB, 0.42504E+00_JPRB, 0.42702E+00_JPRB, 0.43039E+00_JPRB, 0.43170E+00_JPRB, 0.42972E+00_JPRB ,&
&  0.42431E+00_JPRB, 0.42127E+00_JPRB, 0.41900E+00_JPRB, 0.42031E+00_JPRB, 0.42168E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.41023E+00_JPRB, 0.41191E+00_JPRB, 0.41290E+00_JPRB, 0.41324E+00_JPRB, 0.41357E+00_JPRB, 0.41438E+00_JPRB, 0.41518E+00_JPRB ,&
&  0.41605E+00_JPRB, 0.41685E+00_JPRB, 0.41781E+00_JPRB, 0.41843E+00_JPRB, 0.41869E+00_JPRB, 0.42210E+00_JPRB, 0.42318E+00_JPRB ,&
&  0.42259E+00_JPRB, 0.42109E+00_JPRB, 0.41945E+00_JPRB, 0.41750E+00_JPRB, 0.41502E+00_JPRB, 0.41168E+00_JPRB, 0.40571E+00_JPRB ,&
&  0.40240E+00_JPRB, 0.39844E+00_JPRB, 0.39386E+00_JPRB, 0.39305E+00_JPRB, 0.39171E+00_JPRB, 0.37355E+00_JPRB, 0.37263E+00_JPRB ,&
&  0.38414E+00_JPRB, 0.39035E+00_JPRB, 0.39393E+00_JPRB, 0.39831E+00_JPRB, 0.39068E+00_JPRB, 0.37280E+00_JPRB, 0.36005E+00_JPRB ,&
&  0.33790E+00_JPRB, 0.35365E+00_JPRB, 0.36478E+00_JPRB, 0.36989E+00_JPRB, 0.37254E+00_JPRB, 0.37686E+00_JPRB, 0.40074E+00_JPRB ,&
&  0.44990E+00_JPRB, 0.46058E+00_JPRB, 0.47247E+00_JPRB, 0.48550E+00_JPRB, 0.49898E+00_JPRB, 0.50901E+00_JPRB, 0.52032E+00_JPRB ,&
&  0.52777E+00_JPRB, 0.53305E+00_JPRB, 0.53301E+00_JPRB, 0.53067E+00_JPRB, 0.52198E+00_JPRB, 0.51374E+00_JPRB, 0.50514E+00_JPRB ,&
&  0.49857E+00_JPRB, 0.49225E+00_JPRB, 0.48544E+00_JPRB, 0.47687E+00_JPRB, 0.46722E+00_JPRB, 0.45731E+00_JPRB, 0.44716E+00_JPRB ,&
&  0.43630E+00_JPRB, 0.42466E+00_JPRB, 0.41596E+00_JPRB, 0.40857E+00_JPRB, 0.40546E+00_JPRB, 0.40329E+00_JPRB, 0.40410E+00_JPRB ,&
&  0.40504E+00_JPRB, 0.40762E+00_JPRB, 0.41079E+00_JPRB, 0.41585E+00_JPRB, 0.41979E+00_JPRB, 0.42305E+00_JPRB, 0.42547E+00_JPRB ,&
&  0.42748E+00_JPRB, 0.42837E+00_JPRB, 0.42865E+00_JPRB, 0.42765E+00_JPRB, 0.42647E+00_JPRB, 0.42515E+00_JPRB, 0.42326E+00_JPRB ,&
&  0.42151E+00_JPRB, 0.41968E+00_JPRB, 0.41813E+00_JPRB, 0.41635E+00_JPRB, 0.41429E+00_JPRB, 0.41098E+00_JPRB, 0.40774E+00_JPRB ,&
&  0.40487E+00_JPRB, 0.40393E+00_JPRB, 0.40384E+00_JPRB, 0.40551E+00_JPRB, 0.40773E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.41566E+00_JPRB, 0.41683E+00_JPRB, 0.41764E+00_JPRB, 0.41796E+00_JPRB, 0.41824E+00_JPRB, 0.41845E+00_JPRB, 0.41815E+00_JPRB ,&
&  0.41727E+00_JPRB, 0.41671E+00_JPRB, 0.41669E+00_JPRB, 0.41646E+00_JPRB, 0.41577E+00_JPRB, 0.41717E+00_JPRB, 0.42008E+00_JPRB ,&
&  0.42052E+00_JPRB, 0.42058E+00_JPRB, 0.42058E+00_JPRB, 0.42079E+00_JPRB, 0.42074E+00_JPRB, 0.42035E+00_JPRB, 0.42005E+00_JPRB ,&
&  0.42045E+00_JPRB, 0.42060E+00_JPRB, 0.42045E+00_JPRB, 0.41967E+00_JPRB, 0.42047E+00_JPRB, 0.42121E+00_JPRB, 0.42147E+00_JPRB ,&
&  0.42111E+00_JPRB, 0.42011E+00_JPRB, 0.41923E+00_JPRB, 0.41910E+00_JPRB, 0.42176E+00_JPRB, 0.42525E+00_JPRB, 0.42983E+00_JPRB ,&
&  0.43529E+00_JPRB, 0.44180E+00_JPRB, 0.44829E+00_JPRB, 0.45400E+00_JPRB, 0.45826E+00_JPRB, 0.46340E+00_JPRB, 0.46723E+00_JPRB ,&
&  0.46851E+00_JPRB, 0.46750E+00_JPRB, 0.46610E+00_JPRB, 0.46620E+00_JPRB, 0.46631E+00_JPRB, 0.46602E+00_JPRB, 0.46613E+00_JPRB ,&
&  0.46639E+00_JPRB, 0.46649E+00_JPRB, 0.46650E+00_JPRB, 0.46550E+00_JPRB, 0.46337E+00_JPRB, 0.46093E+00_JPRB, 0.45789E+00_JPRB ,&
&  0.45469E+00_JPRB, 0.45067E+00_JPRB, 0.44665E+00_JPRB, 0.44248E+00_JPRB, 0.43803E+00_JPRB, 0.43295E+00_JPRB, 0.42781E+00_JPRB ,&
&  0.42386E+00_JPRB, 0.42127E+00_JPRB, 0.41958E+00_JPRB, 0.41716E+00_JPRB, 0.41511E+00_JPRB, 0.41358E+00_JPRB, 0.41384E+00_JPRB ,&
&  0.41434E+00_JPRB, 0.41600E+00_JPRB, 0.41784E+00_JPRB, 0.42054E+00_JPRB, 0.42258E+00_JPRB, 0.42432E+00_JPRB, 0.42535E+00_JPRB ,&
&  0.42616E+00_JPRB, 0.42647E+00_JPRB, 0.42663E+00_JPRB, 0.42645E+00_JPRB, 0.42586E+00_JPRB, 0.42492E+00_JPRB, 0.42350E+00_JPRB ,&
&  0.42190E+00_JPRB, 0.42022E+00_JPRB, 0.41827E+00_JPRB, 0.41617E+00_JPRB, 0.41435E+00_JPRB, 0.41247E+00_JPRB, 0.41148E+00_JPRB ,&
&  0.41087E+00_JPRB, 0.41096E+00_JPRB, 0.41142E+00_JPRB, 0.41279E+00_JPRB, 0.41434E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.42086E+00_JPRB, 0.42199E+00_JPRB, 0.42307E+00_JPRB, 0.42399E+00_JPRB, 0.42477E+00_JPRB, 0.42540E+00_JPRB, 0.42592E+00_JPRB ,&
&  0.42612E+00_JPRB, 0.42678E+00_JPRB, 0.42767E+00_JPRB, 0.42854E+00_JPRB, 0.42932E+00_JPRB, 0.43011E+00_JPRB, 0.43082E+00_JPRB ,&
&  0.43185E+00_JPRB, 0.43450E+00_JPRB, 0.43664E+00_JPRB, 0.43806E+00_JPRB, 0.43948E+00_JPRB, 0.44093E+00_JPRB, 0.44230E+00_JPRB ,&
&  0.44363E+00_JPRB, 0.44485E+00_JPRB, 0.44597E+00_JPRB, 0.44695E+00_JPRB, 0.44803E+00_JPRB, 0.44907E+00_JPRB, 0.45014E+00_JPRB ,&
&  0.45132E+00_JPRB, 0.45291E+00_JPRB, 0.45470E+00_JPRB, 0.45659E+00_JPRB, 0.45853E+00_JPRB, 0.46095E+00_JPRB, 0.46320E+00_JPRB ,&
&  0.46509E+00_JPRB, 0.46639E+00_JPRB, 0.46631E+00_JPRB, 0.46523E+00_JPRB, 0.46338E+00_JPRB, 0.46107E+00_JPRB, 0.45837E+00_JPRB ,&
&  0.45522E+00_JPRB, 0.45236E+00_JPRB, 0.44986E+00_JPRB, 0.44756E+00_JPRB, 0.44543E+00_JPRB, 0.44362E+00_JPRB, 0.44178E+00_JPRB ,&
&  0.44003E+00_JPRB, 0.43824E+00_JPRB, 0.43661E+00_JPRB, 0.43500E+00_JPRB, 0.43337E+00_JPRB, 0.43164E+00_JPRB, 0.42996E+00_JPRB ,&
&  0.42859E+00_JPRB, 0.42716E+00_JPRB, 0.42604E+00_JPRB, 0.42490E+00_JPRB, 0.42394E+00_JPRB, 0.42333E+00_JPRB, 0.42289E+00_JPRB ,&
&  0.42244E+00_JPRB, 0.42216E+00_JPRB, 0.42209E+00_JPRB, 0.42180E+00_JPRB, 0.42141E+00_JPRB, 0.42091E+00_JPRB, 0.42046E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.42011E+00_JPRB, 0.42003E+00_JPRB, 0.42016E+00_JPRB, 0.42024E+00_JPRB, 0.42033E+00_JPRB, 0.42053E+00_JPRB ,&
&  0.42055E+00_JPRB, 0.42068E+00_JPRB, 0.42072E+00_JPRB, 0.42095E+00_JPRB, 0.42101E+00_JPRB, 0.42081E+00_JPRB, 0.42039E+00_JPRB ,&
&  0.41988E+00_JPRB, 0.41930E+00_JPRB, 0.41880E+00_JPRB, 0.41823E+00_JPRB, 0.41798E+00_JPRB, 0.41798E+00_JPRB, 0.41802E+00_JPRB ,&
&  0.41805E+00_JPRB, 0.41827E+00_JPRB, 0.41862E+00_JPRB, 0.41928E+00_JPRB, 0.41991E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB ,&
&  0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB, 0.43363E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB, 0.14704E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.15218E+00_JPRB, 0.15221E+00_JPRB, 0.15211E+00_JPRB, 0.15155E+00_JPRB, 0.14974E+00_JPRB, 0.14944E+00_JPRB, 0.14943E+00_JPRB ,&
&  0.14959E+00_JPRB, 0.14970E+00_JPRB, 0.15044E+00_JPRB, 0.15123E+00_JPRB, 0.15201E+00_JPRB, 0.15261E+00_JPRB, 0.15310E+00_JPRB ,&
&  0.15338E+00_JPRB, 0.15365E+00_JPRB, 0.15273E+00_JPRB, 0.15229E+00_JPRB, 0.15217E+00_JPRB, 0.15218E+00_JPRB, 0.15218E+00_JPRB ,&
&  0.15213E+00_JPRB, 0.15211E+00_JPRB, 0.15199E+00_JPRB, 0.15214E+00_JPRB, 0.15239E+00_JPRB, 0.15260E+00_JPRB, 0.15285E+00_JPRB ,&
&  0.15304E+00_JPRB, 0.15321E+00_JPRB, 0.15331E+00_JPRB, 0.15338E+00_JPRB, 0.15363E+00_JPRB, 0.15373E+00_JPRB, 0.15363E+00_JPRB ,&
&  0.15344E+00_JPRB, 0.15310E+00_JPRB, 0.15272E+00_JPRB, 0.15233E+00_JPRB, 0.15204E+00_JPRB, 0.15177E+00_JPRB, 0.15150E+00_JPRB ,&
&  0.15123E+00_JPRB, 0.15094E+00_JPRB, 0.15063E+00_JPRB, 0.15029E+00_JPRB, 0.14996E+00_JPRB, 0.14972E+00_JPRB, 0.14946E+00_JPRB ,&
&  0.14924E+00_JPRB, 0.14905E+00_JPRB, 0.14883E+00_JPRB, 0.14855E+00_JPRB, 0.14827E+00_JPRB, 0.14799E+00_JPRB, 0.14768E+00_JPRB ,&
&  0.14729E+00_JPRB, 0.14702E+00_JPRB, 0.14675E+00_JPRB, 0.14651E+00_JPRB, 0.14631E+00_JPRB, 0.14610E+00_JPRB, 0.14591E+00_JPRB ,&
&  0.14570E+00_JPRB, 0.14551E+00_JPRB, 0.14528E+00_JPRB, 0.14508E+00_JPRB, 0.14489E+00_JPRB, 0.14473E+00_JPRB, 0.14461E+00_JPRB ,&
&  0.14449E+00_JPRB, 0.14437E+00_JPRB, 0.14427E+00_JPRB, 0.14429E+00_JPRB, 0.14429E+00_JPRB, 0.14433E+00_JPRB, 0.14436E+00_JPRB ,&
&  0.14440E+00_JPRB, 0.14443E+00_JPRB, 0.14446E+00_JPRB, 0.14438E+00_JPRB, 0.14451E+00_JPRB, 0.14470E+00_JPRB, 0.14498E+00_JPRB ,&
&  0.14530E+00_JPRB, 0.14571E+00_JPRB, 0.14614E+00_JPRB, 0.14659E+00_JPRB, 0.14711E+00_JPRB, 0.14754E+00_JPRB, 0.14802E+00_JPRB ,&
&  0.14858E+00_JPRB, 0.14915E+00_JPRB, 0.14957E+00_JPRB, 0.15006E+00_JPRB, 0.15081E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.16277E+00_JPRB, 0.16560E+00_JPRB, 0.16753E+00_JPRB, 0.16936E+00_JPRB, 0.16507E+00_JPRB, 0.17619E+00_JPRB, 0.17306E+00_JPRB ,&
&  0.17245E+00_JPRB, 0.17056E+00_JPRB, 0.17177E+00_JPRB, 0.17155E+00_JPRB, 0.17077E+00_JPRB, 0.16969E+00_JPRB, 0.16840E+00_JPRB ,&
&  0.16720E+00_JPRB, 0.16574E+00_JPRB, 0.16474E+00_JPRB, 0.16369E+00_JPRB, 0.16284E+00_JPRB, 0.16203E+00_JPRB, 0.16147E+00_JPRB ,&
&  0.16142E+00_JPRB, 0.16210E+00_JPRB, 0.16333E+00_JPRB, 0.16284E+00_JPRB, 0.16292E+00_JPRB, 0.16336E+00_JPRB, 0.16397E+00_JPRB ,&
&  0.16427E+00_JPRB, 0.16444E+00_JPRB, 0.16424E+00_JPRB, 0.16332E+00_JPRB, 0.16320E+00_JPRB, 0.16312E+00_JPRB, 0.16311E+00_JPRB ,&
&  0.16295E+00_JPRB, 0.16259E+00_JPRB, 0.16173E+00_JPRB, 0.16093E+00_JPRB, 0.16011E+00_JPRB, 0.15915E+00_JPRB, 0.15804E+00_JPRB ,&
&  0.15712E+00_JPRB, 0.15639E+00_JPRB, 0.15555E+00_JPRB, 0.15473E+00_JPRB, 0.15402E+00_JPRB, 0.15333E+00_JPRB, 0.15270E+00_JPRB ,&
&  0.15229E+00_JPRB, 0.15199E+00_JPRB, 0.15166E+00_JPRB, 0.15117E+00_JPRB, 0.15062E+00_JPRB, 0.15008E+00_JPRB, 0.14960E+00_JPRB ,&
&  0.14928E+00_JPRB, 0.14877E+00_JPRB, 0.14826E+00_JPRB, 0.14771E+00_JPRB, 0.14729E+00_JPRB, 0.14661E+00_JPRB, 0.14602E+00_JPRB ,&
&  0.14546E+00_JPRB, 0.14495E+00_JPRB, 0.14457E+00_JPRB, 0.14429E+00_JPRB, 0.14405E+00_JPRB, 0.14389E+00_JPRB, 0.14381E+00_JPRB ,&
&  0.14377E+00_JPRB, 0.14376E+00_JPRB, 0.14384E+00_JPRB, 0.14385E+00_JPRB, 0.14383E+00_JPRB, 0.14387E+00_JPRB, 0.14415E+00_JPRB ,&
&  0.14439E+00_JPRB, 0.14458E+00_JPRB, 0.14471E+00_JPRB, 0.14450E+00_JPRB, 0.14486E+00_JPRB, 0.14519E+00_JPRB, 0.14557E+00_JPRB ,&
&  0.14593E+00_JPRB, 0.14664E+00_JPRB, 0.14749E+00_JPRB, 0.14851E+00_JPRB, 0.15040E+00_JPRB, 0.15213E+00_JPRB, 0.15395E+00_JPRB ,&
&  0.15599E+00_JPRB, 0.15891E+00_JPRB, 0.16024E+00_JPRB, 0.16116E+00_JPRB, 0.16196E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.18736E+00_JPRB, 0.18872E+00_JPRB, 0.19005E+00_JPRB, 0.19129E+00_JPRB, 0.19262E+00_JPRB, 0.19312E+00_JPRB, 0.19299E+00_JPRB ,&
&  0.19071E+00_JPRB, 0.19006E+00_JPRB, 0.18851E+00_JPRB, 0.18666E+00_JPRB, 0.18518E+00_JPRB, 0.18265E+00_JPRB, 0.18004E+00_JPRB ,&
&  0.17737E+00_JPRB, 0.17479E+00_JPRB, 0.17246E+00_JPRB, 0.16927E+00_JPRB, 0.16620E+00_JPRB, 0.16320E+00_JPRB, 0.16217E+00_JPRB ,&
&  0.16171E+00_JPRB, 0.16210E+00_JPRB, 0.16360E+00_JPRB, 0.16493E+00_JPRB, 0.16684E+00_JPRB, 0.16890E+00_JPRB, 0.17204E+00_JPRB ,&
&  0.17516E+00_JPRB, 0.17750E+00_JPRB, 0.17790E+00_JPRB, 0.17687E+00_JPRB, 0.17588E+00_JPRB, 0.17550E+00_JPRB, 0.17501E+00_JPRB ,&
&  0.17415E+00_JPRB, 0.17354E+00_JPRB, 0.17283E+00_JPRB, 0.17220E+00_JPRB, 0.17073E+00_JPRB, 0.16828E+00_JPRB, 0.16585E+00_JPRB ,&
&  0.16360E+00_JPRB, 0.16161E+00_JPRB, 0.15978E+00_JPRB, 0.15820E+00_JPRB, 0.15727E+00_JPRB, 0.15635E+00_JPRB, 0.15525E+00_JPRB ,&
&  0.15458E+00_JPRB, 0.15389E+00_JPRB, 0.15307E+00_JPRB, 0.15246E+00_JPRB, 0.15175E+00_JPRB, 0.15123E+00_JPRB, 0.15065E+00_JPRB ,&
&  0.15019E+00_JPRB, 0.14951E+00_JPRB, 0.14892E+00_JPRB, 0.14829E+00_JPRB, 0.14798E+00_JPRB, 0.14730E+00_JPRB, 0.14665E+00_JPRB ,&
&  0.14585E+00_JPRB, 0.14535E+00_JPRB, 0.14446E+00_JPRB, 0.14392E+00_JPRB, 0.14331E+00_JPRB, 0.14277E+00_JPRB, 0.14242E+00_JPRB ,&
&  0.14253E+00_JPRB, 0.14260E+00_JPRB, 0.14232E+00_JPRB, 0.14233E+00_JPRB, 0.14243E+00_JPRB, 0.14286E+00_JPRB, 0.14403E+00_JPRB ,&
&  0.14479E+00_JPRB, 0.14564E+00_JPRB, 0.14647E+00_JPRB, 0.14712E+00_JPRB, 0.14819E+00_JPRB, 0.14923E+00_JPRB, 0.15029E+00_JPRB ,&
&  0.15153E+00_JPRB, 0.15261E+00_JPRB, 0.15387E+00_JPRB, 0.15538E+00_JPRB, 0.15891E+00_JPRB, 0.16156E+00_JPRB, 0.16447E+00_JPRB ,&
&  0.16675E+00_JPRB, 0.16629E+00_JPRB, 0.17973E+00_JPRB, 0.18237E+00_JPRB, 0.18430E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.19565E+00_JPRB, 0.19647E+00_JPRB, 0.19653E+00_JPRB, 0.19747E+00_JPRB, 0.19832E+00_JPRB, 0.19932E+00_JPRB, 0.19963E+00_JPRB ,&
&  0.19894E+00_JPRB, 0.19454E+00_JPRB, 0.19206E+00_JPRB, 0.18882E+00_JPRB, 0.18566E+00_JPRB, 0.18244E+00_JPRB, 0.17975E+00_JPRB ,&
&  0.17800E+00_JPRB, 0.17665E+00_JPRB, 0.17542E+00_JPRB, 0.17373E+00_JPRB, 0.17172E+00_JPRB, 0.16971E+00_JPRB, 0.16787E+00_JPRB ,&
&  0.16623E+00_JPRB, 0.16622E+00_JPRB, 0.16730E+00_JPRB, 0.16937E+00_JPRB, 0.17313E+00_JPRB, 0.17623E+00_JPRB, 0.18019E+00_JPRB ,&
&  0.18661E+00_JPRB, 0.18410E+00_JPRB, 0.18422E+00_JPRB, 0.18434E+00_JPRB, 0.18397E+00_JPRB, 0.18369E+00_JPRB, 0.18042E+00_JPRB ,&
&  0.18316E+00_JPRB, 0.18333E+00_JPRB, 0.18653E+00_JPRB, 0.18596E+00_JPRB, 0.18178E+00_JPRB, 0.17810E+00_JPRB, 0.17328E+00_JPRB ,&
&  0.16962E+00_JPRB, 0.16537E+00_JPRB, 0.16252E+00_JPRB, 0.16003E+00_JPRB, 0.15728E+00_JPRB, 0.15614E+00_JPRB, 0.15425E+00_JPRB ,&
&  0.15307E+00_JPRB, 0.15280E+00_JPRB, 0.15194E+00_JPRB, 0.15187E+00_JPRB, 0.15117E+00_JPRB, 0.15017E+00_JPRB, 0.14948E+00_JPRB ,&
&  0.14886E+00_JPRB, 0.14820E+00_JPRB, 0.14796E+00_JPRB, 0.14774E+00_JPRB, 0.14787E+00_JPRB, 0.14792E+00_JPRB, 0.14802E+00_JPRB ,&
&  0.14825E+00_JPRB, 0.14840E+00_JPRB, 0.14805E+00_JPRB, 0.14788E+00_JPRB, 0.14706E+00_JPRB, 0.14636E+00_JPRB, 0.14528E+00_JPRB ,&
&  0.14463E+00_JPRB, 0.14427E+00_JPRB, 0.14377E+00_JPRB, 0.14343E+00_JPRB, 0.14337E+00_JPRB, 0.14385E+00_JPRB, 0.14444E+00_JPRB ,&
&  0.14511E+00_JPRB, 0.14628E+00_JPRB, 0.14725E+00_JPRB, 0.14854E+00_JPRB, 0.14998E+00_JPRB, 0.15113E+00_JPRB, 0.15276E+00_JPRB ,&
&  0.15438E+00_JPRB, 0.15604E+00_JPRB, 0.15880E+00_JPRB, 0.16192E+00_JPRB, 0.16606E+00_JPRB, 0.17101E+00_JPRB, 0.17412E+00_JPRB ,&
&  0.17772E+00_JPRB, 0.18512E+00_JPRB, 0.18770E+00_JPRB, 0.19392E+00_JPRB, 0.19490E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.20062E+00_JPRB, 0.20049E+00_JPRB, 0.19990E+00_JPRB, 0.19941E+00_JPRB, 0.19985E+00_JPRB, 0.20058E+00_JPRB, 0.20144E+00_JPRB ,&
&  0.19801E+00_JPRB, 0.19525E+00_JPRB, 0.19093E+00_JPRB, 0.18614E+00_JPRB, 0.18199E+00_JPRB, 0.17852E+00_JPRB, 0.17647E+00_JPRB ,&
&  0.17620E+00_JPRB, 0.17799E+00_JPRB, 0.17955E+00_JPRB, 0.18130E+00_JPRB, 0.18148E+00_JPRB, 0.18167E+00_JPRB, 0.18046E+00_JPRB ,&
&  0.17914E+00_JPRB, 0.17858E+00_JPRB, 0.17885E+00_JPRB, 0.18067E+00_JPRB, 0.18337E+00_JPRB, 0.18531E+00_JPRB, 0.18754E+00_JPRB ,&
&  0.19049E+00_JPRB, 0.19004E+00_JPRB, 0.19081E+00_JPRB, 0.19031E+00_JPRB, 0.19044E+00_JPRB, 0.19109E+00_JPRB, 0.19137E+00_JPRB ,&
&  0.19199E+00_JPRB, 0.19264E+00_JPRB, 0.19241E+00_JPRB, 0.19175E+00_JPRB, 0.18776E+00_JPRB, 0.18493E+00_JPRB, 0.18029E+00_JPRB ,&
&  0.17489E+00_JPRB, 0.17018E+00_JPRB, 0.16504E+00_JPRB, 0.16149E+00_JPRB, 0.15748E+00_JPRB, 0.15509E+00_JPRB, 0.15325E+00_JPRB ,&
&  0.15152E+00_JPRB, 0.15084E+00_JPRB, 0.14990E+00_JPRB, 0.14893E+00_JPRB, 0.14875E+00_JPRB, 0.14782E+00_JPRB, 0.14774E+00_JPRB ,&
&  0.14795E+00_JPRB, 0.14814E+00_JPRB, 0.14898E+00_JPRB, 0.14942E+00_JPRB, 0.15019E+00_JPRB, 0.15093E+00_JPRB, 0.15156E+00_JPRB ,&
&  0.15266E+00_JPRB, 0.15396E+00_JPRB, 0.15400E+00_JPRB, 0.15420E+00_JPRB, 0.15339E+00_JPRB, 0.15179E+00_JPRB, 0.15001E+00_JPRB ,&
&  0.14816E+00_JPRB, 0.14670E+00_JPRB, 0.14625E+00_JPRB, 0.14562E+00_JPRB, 0.14577E+00_JPRB, 0.14613E+00_JPRB, 0.14635E+00_JPRB ,&
&  0.14699E+00_JPRB, 0.14757E+00_JPRB, 0.14834E+00_JPRB, 0.14998E+00_JPRB, 0.15117E+00_JPRB, 0.15268E+00_JPRB, 0.15424E+00_JPRB ,&
&  0.15548E+00_JPRB, 0.15770E+00_JPRB, 0.15966E+00_JPRB, 0.16234E+00_JPRB, 0.16844E+00_JPRB, 0.17409E+00_JPRB, 0.18414E+00_JPRB ,&
&  0.18982E+00_JPRB, 0.20281E+00_JPRB, 0.20159E+00_JPRB, 0.20020E+00_JPRB, 0.20101E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.20672E+00_JPRB, 0.20549E+00_JPRB, 0.20445E+00_JPRB, 0.20325E+00_JPRB, 0.20400E+00_JPRB, 0.20506E+00_JPRB, 0.20638E+00_JPRB ,&
&  0.20790E+00_JPRB, 0.20827E+00_JPRB, 0.20796E+00_JPRB, 0.20605E+00_JPRB, 0.20481E+00_JPRB, 0.19843E+00_JPRB, 0.19605E+00_JPRB ,&
&  0.19428E+00_JPRB, 0.19581E+00_JPRB, 0.19807E+00_JPRB, 0.20152E+00_JPRB, 0.20317E+00_JPRB, 0.20486E+00_JPRB, 0.20411E+00_JPRB ,&
&  0.20207E+00_JPRB, 0.19956E+00_JPRB, 0.19724E+00_JPRB, 0.19537E+00_JPRB, 0.19485E+00_JPRB, 0.19349E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19189E+00_JPRB, 0.19318E+00_JPRB, 0.19398E+00_JPRB, 0.19481E+00_JPRB, 0.20084E+00_JPRB, 0.20169E+00_JPRB, 0.20356E+00_JPRB ,&
&  0.20396E+00_JPRB, 0.20434E+00_JPRB, 0.20353E+00_JPRB, 0.20215E+00_JPRB, 0.20089E+00_JPRB, 0.19630E+00_JPRB, 0.19412E+00_JPRB ,&
&  0.18677E+00_JPRB, 0.18237E+00_JPRB, 0.17586E+00_JPRB, 0.17017E+00_JPRB, 0.16424E+00_JPRB, 0.15982E+00_JPRB, 0.15618E+00_JPRB ,&
&  0.15330E+00_JPRB, 0.15119E+00_JPRB, 0.14939E+00_JPRB, 0.14916E+00_JPRB, 0.14929E+00_JPRB, 0.14903E+00_JPRB, 0.14919E+00_JPRB ,&
&  0.14987E+00_JPRB, 0.15062E+00_JPRB, 0.15058E+00_JPRB, 0.15112E+00_JPRB, 0.15164E+00_JPRB, 0.15239E+00_JPRB, 0.15377E+00_JPRB ,&
&  0.15589E+00_JPRB, 0.15783E+00_JPRB, 0.15872E+00_JPRB, 0.16026E+00_JPRB, 0.15958E+00_JPRB, 0.15707E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15192E+00_JPRB, 0.15074E+00_JPRB, 0.14953E+00_JPRB, 0.14884E+00_JPRB, 0.14878E+00_JPRB, 0.14899E+00_JPRB, 0.14973E+00_JPRB ,&
&  0.15052E+00_JPRB, 0.15132E+00_JPRB, 0.15167E+00_JPRB, 0.15215E+00_JPRB, 0.15258E+00_JPRB, 0.15336E+00_JPRB, 0.15477E+00_JPRB ,&
&  0.15665E+00_JPRB, 0.15883E+00_JPRB, 0.16094E+00_JPRB, 0.16354E+00_JPRB, 0.16787E+00_JPRB, 0.17383E+00_JPRB, 0.18289E+00_JPRB ,&
&  0.19052E+00_JPRB, 0.20387E+00_JPRB, 0.20505E+00_JPRB, 0.20579E+00_JPRB, 0.20695E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.21517E+00_JPRB, 0.21539E+00_JPRB, 0.21191E+00_JPRB, 0.20876E+00_JPRB, 0.20728E+00_JPRB, 0.20678E+00_JPRB, 0.20784E+00_JPRB ,&
&  0.21040E+00_JPRB, 0.21234E+00_JPRB, 0.21428E+00_JPRB, 0.21598E+00_JPRB, 0.21670E+00_JPRB, 0.21666E+00_JPRB, 0.21757E+00_JPRB ,&
&  0.21844E+00_JPRB, 0.22066E+00_JPRB, 0.22170E+00_JPRB, 0.22386E+00_JPRB, 0.22409E+00_JPRB, 0.22483E+00_JPRB, 0.22601E+00_JPRB ,&
&  0.22565E+00_JPRB, 0.22641E+00_JPRB, 0.22797E+00_JPRB, 0.22605E+00_JPRB, 0.22472E+00_JPRB, 0.22265E+00_JPRB, 0.22088E+00_JPRB ,&
&  0.21511E+00_JPRB, 0.20894E+00_JPRB, 0.20212E+00_JPRB, 0.20492E+00_JPRB, 0.21299E+00_JPRB, 0.21522E+00_JPRB, 0.21877E+00_JPRB ,&
&  0.21964E+00_JPRB, 0.21807E+00_JPRB, 0.21632E+00_JPRB, 0.21332E+00_JPRB, 0.21034E+00_JPRB, 0.20709E+00_JPRB, 0.20455E+00_JPRB ,&
&  0.20183E+00_JPRB, 0.20228E+00_JPRB, 0.19894E+00_JPRB, 0.18646E+00_JPRB, 0.17952E+00_JPRB, 0.17527E+00_JPRB, 0.16905E+00_JPRB ,&
&  0.16300E+00_JPRB, 0.16100E+00_JPRB, 0.16051E+00_JPRB, 0.15868E+00_JPRB, 0.15878E+00_JPRB, 0.15911E+00_JPRB, 0.15910E+00_JPRB ,&
&  0.15800E+00_JPRB, 0.15778E+00_JPRB, 0.15651E+00_JPRB, 0.15470E+00_JPRB, 0.15304E+00_JPRB, 0.15239E+00_JPRB, 0.15277E+00_JPRB ,&
&  0.15416E+00_JPRB, 0.15584E+00_JPRB, 0.16108E+00_JPRB, 0.16462E+00_JPRB, 0.16086E+00_JPRB, 0.15776E+00_JPRB, 0.15730E+00_JPRB ,&
&  0.15387E+00_JPRB, 0.15311E+00_JPRB, 0.15307E+00_JPRB, 0.15301E+00_JPRB, 0.15333E+00_JPRB, 0.15415E+00_JPRB, 0.15422E+00_JPRB ,&
&  0.15482E+00_JPRB, 0.15588E+00_JPRB, 0.15614E+00_JPRB, 0.15616E+00_JPRB, 0.15598E+00_JPRB, 0.15638E+00_JPRB, 0.15746E+00_JPRB ,&
&  0.15876E+00_JPRB, 0.16110E+00_JPRB, 0.16358E+00_JPRB, 0.16714E+00_JPRB, 0.16999E+00_JPRB, 0.17221E+00_JPRB, 0.17611E+00_JPRB ,&
&  0.17978E+00_JPRB, 0.18814E+00_JPRB, 0.20259E+00_JPRB, 0.21383E+00_JPRB, 0.21163E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.22061E+00_JPRB, 0.22048E+00_JPRB, 0.21925E+00_JPRB, 0.21809E+00_JPRB, 0.21656E+00_JPRB, 0.21504E+00_JPRB, 0.21555E+00_JPRB ,&
&  0.21768E+00_JPRB, 0.22050E+00_JPRB, 0.22323E+00_JPRB, 0.22535E+00_JPRB, 0.22706E+00_JPRB, 0.22788E+00_JPRB, 0.22923E+00_JPRB ,&
&  0.23083E+00_JPRB, 0.23350E+00_JPRB, 0.23418E+00_JPRB, 0.23480E+00_JPRB, 0.23421E+00_JPRB, 0.23367E+00_JPRB, 0.23482E+00_JPRB ,&
&  0.23802E+00_JPRB, 0.23967E+00_JPRB, 0.23903E+00_JPRB, 0.23718E+00_JPRB, 0.23771E+00_JPRB, 0.23940E+00_JPRB, 0.23857E+00_JPRB ,&
&  0.23266E+00_JPRB, 0.21990E+00_JPRB, 0.21063E+00_JPRB, 0.21273E+00_JPRB, 0.22162E+00_JPRB, 0.22569E+00_JPRB, 0.22821E+00_JPRB ,&
&  0.22980E+00_JPRB, 0.23007E+00_JPRB, 0.22935E+00_JPRB, 0.22802E+00_JPRB, 0.22568E+00_JPRB, 0.22164E+00_JPRB, 0.21675E+00_JPRB ,&
&  0.21231E+00_JPRB, 0.20988E+00_JPRB, 0.20722E+00_JPRB, 0.20264E+00_JPRB, 0.19802E+00_JPRB, 0.19504E+00_JPRB, 0.19191E+00_JPRB ,&
&  0.18875E+00_JPRB, 0.18614E+00_JPRB, 0.18549E+00_JPRB, 0.18526E+00_JPRB, 0.18538E+00_JPRB, 0.18513E+00_JPRB, 0.18345E+00_JPRB ,&
&  0.17990E+00_JPRB, 0.17588E+00_JPRB, 0.17032E+00_JPRB, 0.16538E+00_JPRB, 0.16181E+00_JPRB, 0.15847E+00_JPRB, 0.15464E+00_JPRB ,&
&  0.15267E+00_JPRB, 0.15473E+00_JPRB, 0.16233E+00_JPRB, 0.17128E+00_JPRB, 0.17825E+00_JPRB, 0.16668E+00_JPRB, 0.16198E+00_JPRB ,&
&  0.16048E+00_JPRB, 0.15997E+00_JPRB, 0.16012E+00_JPRB, 0.16127E+00_JPRB, 0.16189E+00_JPRB, 0.16181E+00_JPRB, 0.16233E+00_JPRB ,&
&  0.16397E+00_JPRB, 0.16572E+00_JPRB, 0.16668E+00_JPRB, 0.16763E+00_JPRB, 0.16817E+00_JPRB, 0.16795E+00_JPRB, 0.16764E+00_JPRB ,&
&  0.16815E+00_JPRB, 0.17066E+00_JPRB, 0.17499E+00_JPRB, 0.18119E+00_JPRB, 0.18595E+00_JPRB, 0.19033E+00_JPRB, 0.19609E+00_JPRB ,&
&  0.20338E+00_JPRB, 0.21100E+00_JPRB, 0.21464E+00_JPRB, 0.21789E+00_JPRB, 0.21959E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.22675E+00_JPRB, 0.22800E+00_JPRB, 0.22995E+00_JPRB, 0.23191E+00_JPRB, 0.23257E+00_JPRB, 0.23197E+00_JPRB, 0.23129E+00_JPRB ,&
&  0.23155E+00_JPRB, 0.23323E+00_JPRB, 0.23501E+00_JPRB, 0.23526E+00_JPRB, 0.23536E+00_JPRB, 0.23564E+00_JPRB, 0.23728E+00_JPRB ,&
&  0.23983E+00_JPRB, 0.24277E+00_JPRB, 0.24274E+00_JPRB, 0.24237E+00_JPRB, 0.24218E+00_JPRB, 0.24141E+00_JPRB, 0.24266E+00_JPRB ,&
&  0.24567E+00_JPRB, 0.24601E+00_JPRB, 0.24602E+00_JPRB, 0.24537E+00_JPRB, 0.24684E+00_JPRB, 0.24970E+00_JPRB, 0.25115E+00_JPRB ,&
&  0.24947E+00_JPRB, 0.24290E+00_JPRB, 0.23499E+00_JPRB, 0.23472E+00_JPRB, 0.23638E+00_JPRB, 0.23813E+00_JPRB, 0.23844E+00_JPRB ,&
&  0.23802E+00_JPRB, 0.23722E+00_JPRB, 0.23619E+00_JPRB, 0.23523E+00_JPRB, 0.23418E+00_JPRB, 0.23098E+00_JPRB, 0.22726E+00_JPRB ,&
&  0.22432E+00_JPRB, 0.22133E+00_JPRB, 0.21832E+00_JPRB, 0.21442E+00_JPRB, 0.21040E+00_JPRB, 0.20754E+00_JPRB, 0.20441E+00_JPRB ,&
&  0.20051E+00_JPRB, 0.19762E+00_JPRB, 0.19679E+00_JPRB, 0.19693E+00_JPRB, 0.19727E+00_JPRB, 0.19670E+00_JPRB, 0.19599E+00_JPRB ,&
&  0.19463E+00_JPRB, 0.19354E+00_JPRB, 0.19215E+00_JPRB, 0.19154E+00_JPRB, 0.18741E+00_JPRB, 0.17900E+00_JPRB, 0.17326E+00_JPRB ,&
&  0.17233E+00_JPRB, 0.17506E+00_JPRB, 0.17919E+00_JPRB, 0.17967E+00_JPRB, 0.18108E+00_JPRB, 0.18197E+00_JPRB, 0.18170E+00_JPRB ,&
&  0.17967E+00_JPRB, 0.17532E+00_JPRB, 0.17473E+00_JPRB, 0.17547E+00_JPRB, 0.17579E+00_JPRB, 0.17671E+00_JPRB, 0.17820E+00_JPRB ,&
&  0.18087E+00_JPRB, 0.18330E+00_JPRB, 0.18537E+00_JPRB, 0.18736E+00_JPRB, 0.18848E+00_JPRB, 0.18779E+00_JPRB, 0.18577E+00_JPRB ,&
&  0.18541E+00_JPRB, 0.19029E+00_JPRB, 0.19941E+00_JPRB, 0.21024E+00_JPRB, 0.21238E+00_JPRB, 0.21441E+00_JPRB, 0.21636E+00_JPRB ,&
&  0.21729E+00_JPRB, 0.22015E+00_JPRB, 0.22266E+00_JPRB, 0.22414E+00_JPRB, 0.22532E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.23701E+00_JPRB, 0.23866E+00_JPRB, 0.23947E+00_JPRB, 0.24045E+00_JPRB, 0.24237E+00_JPRB, 0.24374E+00_JPRB, 0.24394E+00_JPRB ,&
&  0.24403E+00_JPRB, 0.24500E+00_JPRB, 0.24563E+00_JPRB, 0.24587E+00_JPRB, 0.24357E+00_JPRB, 0.24330E+00_JPRB, 0.24493E+00_JPRB ,&
&  0.24709E+00_JPRB, 0.24948E+00_JPRB, 0.24974E+00_JPRB, 0.24972E+00_JPRB, 0.24944E+00_JPRB, 0.24866E+00_JPRB, 0.25033E+00_JPRB ,&
&  0.25170E+00_JPRB, 0.25235E+00_JPRB, 0.25373E+00_JPRB, 0.25499E+00_JPRB, 0.25766E+00_JPRB, 0.26073E+00_JPRB, 0.26226E+00_JPRB ,&
&  0.26182E+00_JPRB, 0.26007E+00_JPRB, 0.25406E+00_JPRB, 0.24762E+00_JPRB, 0.24831E+00_JPRB, 0.24961E+00_JPRB, 0.24964E+00_JPRB ,&
&  0.24766E+00_JPRB, 0.24498E+00_JPRB, 0.24314E+00_JPRB, 0.24170E+00_JPRB, 0.23998E+00_JPRB, 0.23721E+00_JPRB, 0.23425E+00_JPRB ,&
&  0.23163E+00_JPRB, 0.23074E+00_JPRB, 0.22941E+00_JPRB, 0.22730E+00_JPRB, 0.22427E+00_JPRB, 0.22192E+00_JPRB, 0.21949E+00_JPRB ,&
&  0.21506E+00_JPRB, 0.21065E+00_JPRB, 0.20830E+00_JPRB, 0.20785E+00_JPRB, 0.20841E+00_JPRB, 0.20819E+00_JPRB, 0.20677E+00_JPRB ,&
&  0.20511E+00_JPRB, 0.20282E+00_JPRB, 0.20144E+00_JPRB, 0.20104E+00_JPRB, 0.20209E+00_JPRB, 0.20121E+00_JPRB, 0.19727E+00_JPRB ,&
&  0.19490E+00_JPRB, 0.19320E+00_JPRB, 0.19304E+00_JPRB, 0.19202E+00_JPRB, 0.19107E+00_JPRB, 0.18976E+00_JPRB, 0.19005E+00_JPRB ,&
&  0.19091E+00_JPRB, 0.19129E+00_JPRB, 0.19365E+00_JPRB, 0.19782E+00_JPRB, 0.20166E+00_JPRB, 0.20390E+00_JPRB, 0.20521E+00_JPRB ,&
&  0.20567E+00_JPRB, 0.20494E+00_JPRB, 0.20420E+00_JPRB, 0.20451E+00_JPRB, 0.20617E+00_JPRB, 0.20848E+00_JPRB, 0.21078E+00_JPRB ,&
&  0.21149E+00_JPRB, 0.21248E+00_JPRB, 0.21467E+00_JPRB, 0.21803E+00_JPRB, 0.22006E+00_JPRB, 0.22183E+00_JPRB, 0.22344E+00_JPRB ,&
&  0.22431E+00_JPRB, 0.22575E+00_JPRB, 0.22929E+00_JPRB, 0.23161E+00_JPRB, 0.23404E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.24624E+00_JPRB, 0.24703E+00_JPRB, 0.24798E+00_JPRB, 0.24811E+00_JPRB, 0.24899E+00_JPRB, 0.24975E+00_JPRB, 0.25106E+00_JPRB ,&
&  0.25152E+00_JPRB, 0.25277E+00_JPRB, 0.25464E+00_JPRB, 0.25454E+00_JPRB, 0.25418E+00_JPRB, 0.25402E+00_JPRB, 0.25467E+00_JPRB ,&
&  0.25695E+00_JPRB, 0.25841E+00_JPRB, 0.25811E+00_JPRB, 0.25796E+00_JPRB, 0.25755E+00_JPRB, 0.25725E+00_JPRB, 0.25944E+00_JPRB ,&
&  0.26002E+00_JPRB, 0.26083E+00_JPRB, 0.26262E+00_JPRB, 0.26403E+00_JPRB, 0.26733E+00_JPRB, 0.27120E+00_JPRB, 0.27343E+00_JPRB ,&
&  0.27268E+00_JPRB, 0.26976E+00_JPRB, 0.26528E+00_JPRB, 0.26180E+00_JPRB, 0.26134E+00_JPRB, 0.26236E+00_JPRB, 0.26457E+00_JPRB ,&
&  0.26370E+00_JPRB, 0.25830E+00_JPRB, 0.25148E+00_JPRB, 0.24781E+00_JPRB, 0.24657E+00_JPRB, 0.24509E+00_JPRB, 0.24244E+00_JPRB ,&
&  0.24028E+00_JPRB, 0.23982E+00_JPRB, 0.23992E+00_JPRB, 0.23919E+00_JPRB, 0.23776E+00_JPRB, 0.23532E+00_JPRB, 0.23213E+00_JPRB ,&
&  0.22937E+00_JPRB, 0.22656E+00_JPRB, 0.22482E+00_JPRB, 0.22353E+00_JPRB, 0.22343E+00_JPRB, 0.22376E+00_JPRB, 0.22315E+00_JPRB ,&
&  0.22155E+00_JPRB, 0.22057E+00_JPRB, 0.21918E+00_JPRB, 0.21627E+00_JPRB, 0.21416E+00_JPRB, 0.21350E+00_JPRB, 0.21243E+00_JPRB ,&
&  0.21090E+00_JPRB, 0.21025E+00_JPRB, 0.20990E+00_JPRB, 0.20928E+00_JPRB, 0.20991E+00_JPRB, 0.21142E+00_JPRB, 0.20897E+00_JPRB ,&
&  0.20691E+00_JPRB, 0.20557E+00_JPRB, 0.20558E+00_JPRB, 0.20693E+00_JPRB, 0.20922E+00_JPRB, 0.21173E+00_JPRB, 0.21387E+00_JPRB ,&
&  0.21478E+00_JPRB, 0.21503E+00_JPRB, 0.21536E+00_JPRB, 0.21537E+00_JPRB, 0.21618E+00_JPRB, 0.21805E+00_JPRB, 0.22043E+00_JPRB ,&
&  0.22169E+00_JPRB, 0.22406E+00_JPRB, 0.22652E+00_JPRB, 0.22958E+00_JPRB, 0.23128E+00_JPRB, 0.23288E+00_JPRB, 0.23451E+00_JPRB ,&
&  0.23583E+00_JPRB, 0.23751E+00_JPRB, 0.23994E+00_JPRB, 0.24202E+00_JPRB, 0.24427E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.25817E+00_JPRB, 0.25734E+00_JPRB, 0.25699E+00_JPRB, 0.25759E+00_JPRB, 0.25972E+00_JPRB, 0.26031E+00_JPRB, 0.26023E+00_JPRB ,&
&  0.25989E+00_JPRB, 0.26153E+00_JPRB, 0.26480E+00_JPRB, 0.26675E+00_JPRB, 0.26689E+00_JPRB, 0.26665E+00_JPRB, 0.26637E+00_JPRB ,&
&  0.26643E+00_JPRB, 0.26650E+00_JPRB, 0.26563E+00_JPRB, 0.26575E+00_JPRB, 0.26608E+00_JPRB, 0.26560E+00_JPRB, 0.26670E+00_JPRB ,&
&  0.26698E+00_JPRB, 0.26747E+00_JPRB, 0.26964E+00_JPRB, 0.27134E+00_JPRB, 0.27472E+00_JPRB, 0.27972E+00_JPRB, 0.28160E+00_JPRB ,&
&  0.28039E+00_JPRB, 0.27668E+00_JPRB, 0.27375E+00_JPRB, 0.27447E+00_JPRB, 0.27602E+00_JPRB, 0.27904E+00_JPRB, 0.28206E+00_JPRB ,&
&  0.28223E+00_JPRB, 0.27609E+00_JPRB, 0.26740E+00_JPRB, 0.26187E+00_JPRB, 0.25880E+00_JPRB, 0.25709E+00_JPRB, 0.25569E+00_JPRB ,&
&  0.25358E+00_JPRB, 0.25193E+00_JPRB, 0.25119E+00_JPRB, 0.25117E+00_JPRB, 0.25102E+00_JPRB, 0.24860E+00_JPRB, 0.24591E+00_JPRB ,&
&  0.24223E+00_JPRB, 0.23969E+00_JPRB, 0.23755E+00_JPRB, 0.23684E+00_JPRB, 0.23701E+00_JPRB, 0.23690E+00_JPRB, 0.23607E+00_JPRB ,&
&  0.23496E+00_JPRB, 0.23468E+00_JPRB, 0.23443E+00_JPRB, 0.23344E+00_JPRB, 0.23219E+00_JPRB, 0.23190E+00_JPRB, 0.23090E+00_JPRB ,&
&  0.22868E+00_JPRB, 0.22725E+00_JPRB, 0.22705E+00_JPRB, 0.22777E+00_JPRB, 0.22760E+00_JPRB, 0.22824E+00_JPRB, 0.22962E+00_JPRB ,&
&  0.22868E+00_JPRB, 0.22631E+00_JPRB, 0.22515E+00_JPRB, 0.22574E+00_JPRB, 0.22747E+00_JPRB, 0.22972E+00_JPRB, 0.23082E+00_JPRB ,&
&  0.23182E+00_JPRB, 0.23178E+00_JPRB, 0.23262E+00_JPRB, 0.23382E+00_JPRB, 0.23518E+00_JPRB, 0.23581E+00_JPRB, 0.23603E+00_JPRB ,&
&  0.23639E+00_JPRB, 0.23684E+00_JPRB, 0.23865E+00_JPRB, 0.24159E+00_JPRB, 0.24418E+00_JPRB, 0.24714E+00_JPRB, 0.24945E+00_JPRB ,&
&  0.25099E+00_JPRB, 0.25269E+00_JPRB, 0.25393E+00_JPRB, 0.25496E+00_JPRB, 0.25648E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.27705E+00_JPRB, 0.27488E+00_JPRB, 0.27349E+00_JPRB, 0.27386E+00_JPRB, 0.27607E+00_JPRB, 0.27667E+00_JPRB, 0.27539E+00_JPRB ,&
&  0.27405E+00_JPRB, 0.27526E+00_JPRB, 0.27801E+00_JPRB, 0.27926E+00_JPRB, 0.27959E+00_JPRB, 0.28021E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.27643E+00_JPRB, 0.27339E+00_JPRB, 0.27380E+00_JPRB, 0.27496E+00_JPRB, 0.27611E+00_JPRB, 0.27510E+00_JPRB, 0.27381E+00_JPRB ,&
&  0.27459E+00_JPRB, 0.27470E+00_JPRB, 0.27642E+00_JPRB, 0.27842E+00_JPRB, 0.28165E+00_JPRB, 0.28630E+00_JPRB, 0.28956E+00_JPRB ,&
&  0.29000E+00_JPRB, 0.28845E+00_JPRB, 0.28809E+00_JPRB, 0.29008E+00_JPRB, 0.29212E+00_JPRB, 0.29707E+00_JPRB, 0.30168E+00_JPRB ,&
&  0.30276E+00_JPRB, 0.29931E+00_JPRB, 0.28975E+00_JPRB, 0.28244E+00_JPRB, 0.27860E+00_JPRB, 0.27499E+00_JPRB, 0.27148E+00_JPRB ,&
&  0.26735E+00_JPRB, 0.26421E+00_JPRB, 0.26233E+00_JPRB, 0.26152E+00_JPRB, 0.26129E+00_JPRB, 0.26081E+00_JPRB, 0.25893E+00_JPRB ,&
&  0.25645E+00_JPRB, 0.25454E+00_JPRB, 0.25304E+00_JPRB, 0.25186E+00_JPRB, 0.25077E+00_JPRB, 0.25032E+00_JPRB, 0.24981E+00_JPRB ,&
&  0.24961E+00_JPRB, 0.25010E+00_JPRB, 0.25100E+00_JPRB, 0.25115E+00_JPRB, 0.25113E+00_JPRB, 0.25053E+00_JPRB, 0.24975E+00_JPRB ,&
&  0.24835E+00_JPRB, 0.24584E+00_JPRB, 0.24368E+00_JPRB, 0.24293E+00_JPRB, 0.24247E+00_JPRB, 0.24269E+00_JPRB, 0.24303E+00_JPRB ,&
&  0.24277E+00_JPRB, 0.24268E+00_JPRB, 0.24347E+00_JPRB, 0.24464E+00_JPRB, 0.24556E+00_JPRB, 0.24601E+00_JPRB, 0.24660E+00_JPRB ,&
&  0.24809E+00_JPRB, 0.24982E+00_JPRB, 0.25100E+00_JPRB, 0.25179E+00_JPRB, 0.25220E+00_JPRB, 0.25315E+00_JPRB, 0.25307E+00_JPRB ,&
&  0.25314E+00_JPRB, 0.25408E+00_JPRB, 0.25668E+00_JPRB, 0.26053E+00_JPRB, 0.26435E+00_JPRB, 0.26698E+00_JPRB, 0.26966E+00_JPRB ,&
&  0.27184E+00_JPRB, 0.27365E+00_JPRB, 0.27472E+00_JPRB, 0.27538E+00_JPRB, 0.27651E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.29675E+00_JPRB, 0.29491E+00_JPRB, 0.29367E+00_JPRB, 0.29242E+00_JPRB, 0.29308E+00_JPRB, 0.29251E+00_JPRB, 0.29034E+00_JPRB ,&
&  0.28876E+00_JPRB, 0.29005E+00_JPRB, 0.29239E+00_JPRB, 0.29322E+00_JPRB, 0.29244E+00_JPRB, 0.29112E+00_JPRB, 0.28853E+00_JPRB ,&
&  0.28496E+00_JPRB, 0.28318E+00_JPRB, 0.28288E+00_JPRB, 0.28339E+00_JPRB, 0.28341E+00_JPRB, 0.28212E+00_JPRB, 0.28129E+00_JPRB ,&
&  0.28259E+00_JPRB, 0.28408E+00_JPRB, 0.28439E+00_JPRB, 0.28664E+00_JPRB, 0.29002E+00_JPRB, 0.29577E+00_JPRB, 0.30122E+00_JPRB ,&
&  0.30264E+00_JPRB, 0.30412E+00_JPRB, 0.29813E+00_JPRB, 0.30755E+00_JPRB, 0.31213E+00_JPRB, 0.31820E+00_JPRB, 0.32085E+00_JPRB ,&
&  0.32142E+00_JPRB, 0.31767E+00_JPRB, 0.30835E+00_JPRB, 0.30131E+00_JPRB, 0.29740E+00_JPRB, 0.29420E+00_JPRB, 0.28907E+00_JPRB ,&
&  0.28353E+00_JPRB, 0.27848E+00_JPRB, 0.27551E+00_JPRB, 0.27418E+00_JPRB, 0.27460E+00_JPRB, 0.27527E+00_JPRB, 0.27383E+00_JPRB ,&
&  0.27225E+00_JPRB, 0.27067E+00_JPRB, 0.27009E+00_JPRB, 0.26940E+00_JPRB, 0.26829E+00_JPRB, 0.26746E+00_JPRB, 0.26735E+00_JPRB ,&
&  0.26756E+00_JPRB, 0.26767E+00_JPRB, 0.26804E+00_JPRB, 0.26890E+00_JPRB, 0.26939E+00_JPRB, 0.26874E+00_JPRB, 0.26712E+00_JPRB ,&
&  0.26522E+00_JPRB, 0.26345E+00_JPRB, 0.26180E+00_JPRB, 0.26017E+00_JPRB, 0.25877E+00_JPRB, 0.25781E+00_JPRB, 0.25700E+00_JPRB ,&
&  0.25611E+00_JPRB, 0.25641E+00_JPRB, 0.25773E+00_JPRB, 0.25849E+00_JPRB, 0.25883E+00_JPRB, 0.25976E+00_JPRB, 0.26148E+00_JPRB ,&
&  0.26317E+00_JPRB, 0.26502E+00_JPRB, 0.26631E+00_JPRB, 0.26669E+00_JPRB, 0.26591E+00_JPRB, 0.26564E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.26756E+00_JPRB, 0.27006E+00_JPRB, 0.27391E+00_JPRB, 0.27915E+00_JPRB, 0.28482E+00_JPRB, 0.28890E+00_JPRB, 0.29268E+00_JPRB ,&
&  0.29568E+00_JPRB, 0.29661E+00_JPRB, 0.29659E+00_JPRB, 0.29758E+00_JPRB, 0.29803E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.31650E+00_JPRB, 0.31487E+00_JPRB, 0.31371E+00_JPRB, 0.31106E+00_JPRB, 0.30948E+00_JPRB, 0.30856E+00_JPRB, 0.30719E+00_JPRB ,&
&  0.30624E+00_JPRB, 0.30701E+00_JPRB, 0.30627E+00_JPRB, 0.30596E+00_JPRB, 0.30536E+00_JPRB, 0.30301E+00_JPRB, 0.29992E+00_JPRB ,&
&  0.29819E+00_JPRB, 0.29654E+00_JPRB, 0.29505E+00_JPRB, 0.29347E+00_JPRB, 0.29209E+00_JPRB, 0.29013E+00_JPRB, 0.29053E+00_JPRB ,&
&  0.29206E+00_JPRB, 0.29490E+00_JPRB, 0.29781E+00_JPRB, 0.30123E+00_JPRB, 0.30585E+00_JPRB, 0.31192E+00_JPRB, 0.31774E+00_JPRB ,&
&  0.32383E+00_JPRB, 0.31385E+00_JPRB, 0.32406E+00_JPRB, 0.32929E+00_JPRB, 0.33823E+00_JPRB, 0.34469E+00_JPRB, 0.34421E+00_JPRB ,&
&  0.34046E+00_JPRB, 0.33080E+00_JPRB, 0.32105E+00_JPRB, 0.31385E+00_JPRB, 0.31102E+00_JPRB, 0.30958E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.30237E+00_JPRB, 0.29758E+00_JPRB, 0.29379E+00_JPRB, 0.29166E+00_JPRB, 0.29104E+00_JPRB, 0.29112E+00_JPRB, 0.29011E+00_JPRB ,&
&  0.28912E+00_JPRB, 0.28802E+00_JPRB, 0.28755E+00_JPRB, 0.28738E+00_JPRB, 0.28712E+00_JPRB, 0.28753E+00_JPRB, 0.28839E+00_JPRB ,&
&  0.28905E+00_JPRB, 0.28942E+00_JPRB, 0.28970E+00_JPRB, 0.29042E+00_JPRB, 0.29063E+00_JPRB, 0.28999E+00_JPRB, 0.28881E+00_JPRB ,&
&  0.28704E+00_JPRB, 0.28543E+00_JPRB, 0.28340E+00_JPRB, 0.28195E+00_JPRB, 0.28018E+00_JPRB, 0.27798E+00_JPRB, 0.27596E+00_JPRB ,&
&  0.27489E+00_JPRB, 0.27478E+00_JPRB, 0.27504E+00_JPRB, 0.27543E+00_JPRB, 0.27610E+00_JPRB, 0.27723E+00_JPRB, 0.27850E+00_JPRB ,&
&  0.27978E+00_JPRB, 0.28089E+00_JPRB, 0.28117E+00_JPRB, 0.28107E+00_JPRB, 0.28033E+00_JPRB, 0.28028E+00_JPRB, 0.28143E+00_JPRB ,&
&  0.28452E+00_JPRB, 0.28855E+00_JPRB, 0.29446E+00_JPRB, 0.30367E+00_JPRB, 0.31206E+00_JPRB, 0.31720E+00_JPRB, 0.32102E+00_JPRB ,&
&  0.32154E+00_JPRB, 0.31962E+00_JPRB, 0.31769E+00_JPRB, 0.31781E+00_JPRB, 0.31805E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.33052E+00_JPRB, 0.32963E+00_JPRB, 0.32884E+00_JPRB, 0.32734E+00_JPRB, 0.32607E+00_JPRB, 0.32518E+00_JPRB, 0.32286E+00_JPRB ,&
&  0.32261E+00_JPRB, 0.32019E+00_JPRB, 0.31772E+00_JPRB, 0.31707E+00_JPRB, 0.31591E+00_JPRB, 0.31386E+00_JPRB, 0.31209E+00_JPRB ,&
&  0.30999E+00_JPRB, 0.30776E+00_JPRB, 0.30649E+00_JPRB, 0.30464E+00_JPRB, 0.30176E+00_JPRB, 0.29966E+00_JPRB, 0.30035E+00_JPRB ,&
&  0.30151E+00_JPRB, 0.30421E+00_JPRB, 0.31140E+00_JPRB, 0.31871E+00_JPRB, 0.32578E+00_JPRB, 0.33209E+00_JPRB, 0.33859E+00_JPRB ,&
&  0.34772E+00_JPRB, 0.31134E+00_JPRB, 0.32807E+00_JPRB, 0.33858E+00_JPRB, 0.35418E+00_JPRB, 0.36801E+00_JPRB, 0.36646E+00_JPRB ,&
&  0.35719E+00_JPRB, 0.34486E+00_JPRB, 0.33354E+00_JPRB, 0.32534E+00_JPRB, 0.32325E+00_JPRB, 0.32253E+00_JPRB, 0.32079E+00_JPRB ,&
&  0.31909E+00_JPRB, 0.31533E+00_JPRB, 0.31118E+00_JPRB, 0.30831E+00_JPRB, 0.30670E+00_JPRB, 0.30545E+00_JPRB, 0.30406E+00_JPRB ,&
&  0.30343E+00_JPRB, 0.30360E+00_JPRB, 0.30384E+00_JPRB, 0.30400E+00_JPRB, 0.30479E+00_JPRB, 0.30640E+00_JPRB, 0.30819E+00_JPRB ,&
&  0.30984E+00_JPRB, 0.31072E+00_JPRB, 0.31183E+00_JPRB, 0.31284E+00_JPRB, 0.31400E+00_JPRB, 0.31487E+00_JPRB, 0.31482E+00_JPRB ,&
&  0.31345E+00_JPRB, 0.31150E+00_JPRB, 0.30946E+00_JPRB, 0.30728E+00_JPRB, 0.30463E+00_JPRB, 0.30143E+00_JPRB, 0.29878E+00_JPRB ,&
&  0.29646E+00_JPRB, 0.29467E+00_JPRB, 0.29177E+00_JPRB, 0.29114E+00_JPRB, 0.29184E+00_JPRB, 0.29257E+00_JPRB, 0.29342E+00_JPRB ,&
&  0.29413E+00_JPRB, 0.29395E+00_JPRB, 0.29332E+00_JPRB, 0.29270E+00_JPRB, 0.29240E+00_JPRB, 0.29290E+00_JPRB, 0.29452E+00_JPRB ,&
&  0.29739E+00_JPRB, 0.30270E+00_JPRB, 0.31003E+00_JPRB, 0.32100E+00_JPRB, 0.33029E+00_JPRB, 0.33547E+00_JPRB, 0.33799E+00_JPRB ,&
&  0.33635E+00_JPRB, 0.33311E+00_JPRB, 0.33060E+00_JPRB, 0.32908E+00_JPRB, 0.33023E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.35114E+00_JPRB, 0.34910E+00_JPRB, 0.34805E+00_JPRB, 0.34601E+00_JPRB, 0.34393E+00_JPRB, 0.34176E+00_JPRB, 0.33918E+00_JPRB ,&
&  0.33604E+00_JPRB, 0.33422E+00_JPRB, 0.33240E+00_JPRB, 0.32985E+00_JPRB, 0.32721E+00_JPRB, 0.32497E+00_JPRB, 0.32258E+00_JPRB ,&
&  0.32001E+00_JPRB, 0.31728E+00_JPRB, 0.31563E+00_JPRB, 0.31461E+00_JPRB, 0.31324E+00_JPRB, 0.31291E+00_JPRB, 0.31407E+00_JPRB ,&
&  0.31446E+00_JPRB, 0.31691E+00_JPRB, 0.32612E+00_JPRB, 0.33634E+00_JPRB, 0.34721E+00_JPRB, 0.35597E+00_JPRB, 0.36872E+00_JPRB ,&
&  0.35430E+00_JPRB, 0.31716E+00_JPRB, 0.33549E+00_JPRB, 0.35517E+00_JPRB, 0.37524E+00_JPRB, 0.39308E+00_JPRB, 0.39065E+00_JPRB ,&
&  0.38495E+00_JPRB, 0.37503E+00_JPRB, 0.36428E+00_JPRB, 0.35520E+00_JPRB, 0.35122E+00_JPRB, 0.34777E+00_JPRB, 0.34424E+00_JPRB ,&
&  0.34063E+00_JPRB, 0.33565E+00_JPRB, 0.33104E+00_JPRB, 0.32698E+00_JPRB, 0.32428E+00_JPRB, 0.32199E+00_JPRB, 0.31989E+00_JPRB ,&
&  0.31929E+00_JPRB, 0.31958E+00_JPRB, 0.32000E+00_JPRB, 0.32053E+00_JPRB, 0.32150E+00_JPRB, 0.32387E+00_JPRB, 0.32722E+00_JPRB ,&
&  0.33123E+00_JPRB, 0.33607E+00_JPRB, 0.34173E+00_JPRB, 0.34695E+00_JPRB, 0.35131E+00_JPRB, 0.35347E+00_JPRB, 0.35254E+00_JPRB ,&
&  0.35036E+00_JPRB, 0.34674E+00_JPRB, 0.34256E+00_JPRB, 0.33745E+00_JPRB, 0.33130E+00_JPRB, 0.32553E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.31714E+00_JPRB, 0.31339E+00_JPRB, 0.31027E+00_JPRB, 0.30921E+00_JPRB, 0.30949E+00_JPRB, 0.30958E+00_JPRB, 0.30934E+00_JPRB ,&
&  0.30900E+00_JPRB, 0.30832E+00_JPRB, 0.30788E+00_JPRB, 0.30809E+00_JPRB, 0.30908E+00_JPRB, 0.31128E+00_JPRB, 0.31385E+00_JPRB ,&
&  0.31621E+00_JPRB, 0.32154E+00_JPRB, 0.32891E+00_JPRB, 0.34231E+00_JPRB, 0.35149E+00_JPRB, 0.35558E+00_JPRB, 0.35624E+00_JPRB ,&
&  0.35431E+00_JPRB, 0.35171E+00_JPRB, 0.35101E+00_JPRB, 0.34990E+00_JPRB, 0.35069E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.37531E+00_JPRB, 0.37518E+00_JPRB, 0.37156E+00_JPRB, 0.36407E+00_JPRB, 0.35836E+00_JPRB, 0.35426E+00_JPRB, 0.35168E+00_JPRB ,&
&  0.35038E+00_JPRB, 0.34926E+00_JPRB, 0.34744E+00_JPRB, 0.34356E+00_JPRB, 0.34029E+00_JPRB, 0.33597E+00_JPRB, 0.33144E+00_JPRB ,&
&  0.32777E+00_JPRB, 0.32441E+00_JPRB, 0.32270E+00_JPRB, 0.32021E+00_JPRB, 0.31908E+00_JPRB, 0.32038E+00_JPRB, 0.32243E+00_JPRB ,&
&  0.32517E+00_JPRB, 0.32791E+00_JPRB, 0.33636E+00_JPRB, 0.34584E+00_JPRB, 0.36218E+00_JPRB, 0.37303E+00_JPRB, 0.38667E+00_JPRB ,&
&  0.41791E+00_JPRB, 0.35007E+00_JPRB, 0.36906E+00_JPRB, 0.39432E+00_JPRB, 0.40928E+00_JPRB, 0.42610E+00_JPRB, 0.42070E+00_JPRB ,&
&  0.41620E+00_JPRB, 0.41148E+00_JPRB, 0.40347E+00_JPRB, 0.39519E+00_JPRB, 0.38974E+00_JPRB, 0.38093E+00_JPRB, 0.37097E+00_JPRB ,&
&  0.35992E+00_JPRB, 0.35215E+00_JPRB, 0.34624E+00_JPRB, 0.34088E+00_JPRB, 0.33656E+00_JPRB, 0.33406E+00_JPRB, 0.33303E+00_JPRB ,&
&  0.33337E+00_JPRB, 0.33384E+00_JPRB, 0.33387E+00_JPRB, 0.33445E+00_JPRB, 0.33593E+00_JPRB, 0.33944E+00_JPRB, 0.34538E+00_JPRB ,&
&  0.35593E+00_JPRB, 0.36893E+00_JPRB, 0.38066E+00_JPRB, 0.38920E+00_JPRB, 0.39422E+00_JPRB, 0.39345E+00_JPRB, 0.38940E+00_JPRB ,&
&  0.38241E+00_JPRB, 0.37343E+00_JPRB, 0.36526E+00_JPRB, 0.35790E+00_JPRB, 0.35138E+00_JPRB, 0.34572E+00_JPRB, 0.33982E+00_JPRB ,&
&  0.33536E+00_JPRB, 0.33227E+00_JPRB, 0.33098E+00_JPRB, 0.33010E+00_JPRB, 0.32900E+00_JPRB, 0.32800E+00_JPRB, 0.32788E+00_JPRB ,&
&  0.32758E+00_JPRB, 0.32733E+00_JPRB, 0.32739E+00_JPRB, 0.32866E+00_JPRB, 0.33098E+00_JPRB, 0.33315E+00_JPRB, 0.33483E+00_JPRB ,&
&  0.33638E+00_JPRB, 0.34094E+00_JPRB, 0.34611E+00_JPRB, 0.36174E+00_JPRB, 0.36783E+00_JPRB, 0.37018E+00_JPRB, 0.37185E+00_JPRB ,&
&  0.37307E+00_JPRB, 0.37205E+00_JPRB, 0.33414E+00_JPRB, 0.35827E+00_JPRB, 0.36974E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.40686E+00_JPRB, 0.40330E+00_JPRB, 0.39421E+00_JPRB, 0.38444E+00_JPRB, 0.37877E+00_JPRB, 0.37560E+00_JPRB, 0.37358E+00_JPRB ,&
&  0.37080E+00_JPRB, 0.36719E+00_JPRB, 0.36327E+00_JPRB, 0.36100E+00_JPRB, 0.35801E+00_JPRB, 0.35358E+00_JPRB, 0.34812E+00_JPRB ,&
&  0.34309E+00_JPRB, 0.33833E+00_JPRB, 0.33477E+00_JPRB, 0.33057E+00_JPRB, 0.32723E+00_JPRB, 0.32765E+00_JPRB, 0.32946E+00_JPRB ,&
&  0.33490E+00_JPRB, 0.34244E+00_JPRB, 0.34948E+00_JPRB, 0.36302E+00_JPRB, 0.37776E+00_JPRB, 0.39708E+00_JPRB, 0.41437E+00_JPRB ,&
&  0.44698E+00_JPRB, 0.38105E+00_JPRB, 0.39348E+00_JPRB, 0.42843E+00_JPRB, 0.44564E+00_JPRB, 0.46078E+00_JPRB, 0.46168E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.46120E+00_JPRB, 0.45715E+00_JPRB, 0.44905E+00_JPRB, 0.43513E+00_JPRB, 0.41167E+00_JPRB, 0.38945E+00_JPRB ,&
&  0.37355E+00_JPRB, 0.36804E+00_JPRB, 0.36565E+00_JPRB, 0.36262E+00_JPRB, 0.35980E+00_JPRB, 0.35911E+00_JPRB, 0.36062E+00_JPRB ,&
&  0.36157E+00_JPRB, 0.36186E+00_JPRB, 0.36133E+00_JPRB, 0.36053E+00_JPRB, 0.36136E+00_JPRB, 0.36908E+00_JPRB, 0.38539E+00_JPRB ,&
&  0.40889E+00_JPRB, 0.43102E+00_JPRB, 0.44684E+00_JPRB, 0.45563E+00_JPRB, 0.45436E+00_JPRB, 0.44486E+00_JPRB, 0.42847E+00_JPRB ,&
&  0.41025E+00_JPRB, 0.39516E+00_JPRB, 0.38505E+00_JPRB, 0.37725E+00_JPRB, 0.37101E+00_JPRB, 0.36593E+00_JPRB, 0.36292E+00_JPRB ,&
&  0.36190E+00_JPRB, 0.36154E+00_JPRB, 0.36081E+00_JPRB, 0.35885E+00_JPRB, 0.35764E+00_JPRB, 0.35665E+00_JPRB, 0.35633E+00_JPRB ,&
&  0.35599E+00_JPRB, 0.35561E+00_JPRB, 0.35629E+00_JPRB, 0.35769E+00_JPRB, 0.35859E+00_JPRB, 0.35862E+00_JPRB, 0.35902E+00_JPRB ,&
&  0.36209E+00_JPRB, 0.36840E+00_JPRB, 0.37780E+00_JPRB, 0.39850E+00_JPRB, 0.40904E+00_JPRB, 0.40325E+00_JPRB, 0.39841E+00_JPRB ,&
&  0.39731E+00_JPRB, 0.39555E+00_JPRB, 0.39826E+00_JPRB, 0.37677E+00_JPRB, 0.40515E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.43110E+00_JPRB, 0.43084E+00_JPRB, 0.42165E+00_JPRB, 0.41035E+00_JPRB, 0.39997E+00_JPRB, 0.39363E+00_JPRB, 0.38850E+00_JPRB ,&
&  0.38527E+00_JPRB, 0.38324E+00_JPRB, 0.38175E+00_JPRB, 0.37989E+00_JPRB, 0.37678E+00_JPRB, 0.37178E+00_JPRB, 0.36522E+00_JPRB ,&
&  0.35935E+00_JPRB, 0.35345E+00_JPRB, 0.34868E+00_JPRB, 0.34315E+00_JPRB, 0.33620E+00_JPRB, 0.33305E+00_JPRB, 0.33587E+00_JPRB ,&
&  0.34160E+00_JPRB, 0.34874E+00_JPRB, 0.35548E+00_JPRB, 0.36579E+00_JPRB, 0.38672E+00_JPRB, 0.40567E+00_JPRB, 0.42271E+00_JPRB ,&
&  0.46066E+00_JPRB, 0.41997E+00_JPRB, 0.43780E+00_JPRB, 0.45883E+00_JPRB, 0.47492E+00_JPRB, 0.48720E+00_JPRB, 0.50291E+00_JPRB ,&
&  0.51722E+00_JPRB, 0.52480E+00_JPRB, 0.52256E+00_JPRB, 0.50290E+00_JPRB, 0.46962E+00_JPRB, 0.44143E+00_JPRB, 0.42477E+00_JPRB ,&
&  0.41595E+00_JPRB, 0.41216E+00_JPRB, 0.40879E+00_JPRB, 0.40630E+00_JPRB, 0.40520E+00_JPRB, 0.40424E+00_JPRB, 0.40306E+00_JPRB ,&
&  0.40179E+00_JPRB, 0.40081E+00_JPRB, 0.40033E+00_JPRB, 0.40098E+00_JPRB, 0.40669E+00_JPRB, 0.42176E+00_JPRB, 0.44300E+00_JPRB ,&
&  0.47275E+00_JPRB, 0.49450E+00_JPRB, 0.50089E+00_JPRB, 0.50414E+00_JPRB, 0.50469E+00_JPRB, 0.49257E+00_JPRB, 0.46518E+00_JPRB ,&
&  0.44033E+00_JPRB, 0.41992E+00_JPRB, 0.40581E+00_JPRB, 0.39641E+00_JPRB, 0.39061E+00_JPRB, 0.38789E+00_JPRB, 0.38794E+00_JPRB ,&
&  0.38885E+00_JPRB, 0.38930E+00_JPRB, 0.39057E+00_JPRB, 0.39110E+00_JPRB, 0.39165E+00_JPRB, 0.39097E+00_JPRB, 0.38787E+00_JPRB ,&
&  0.38373E+00_JPRB, 0.38105E+00_JPRB, 0.37916E+00_JPRB, 0.37785E+00_JPRB, 0.37732E+00_JPRB, 0.37606E+00_JPRB, 0.37547E+00_JPRB ,&
&  0.37738E+00_JPRB, 0.38959E+00_JPRB, 0.40765E+00_JPRB, 0.44319E+00_JPRB, 0.43867E+00_JPRB, 0.42890E+00_JPRB, 0.41801E+00_JPRB ,&
&  0.41342E+00_JPRB, 0.41476E+00_JPRB, 0.41086E+00_JPRB, 0.41631E+00_JPRB, 0.43033E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.48841E+00_JPRB, 0.47513E+00_JPRB, 0.45991E+00_JPRB, 0.44177E+00_JPRB, 0.42563E+00_JPRB, 0.41394E+00_JPRB, 0.40552E+00_JPRB ,&
&  0.40025E+00_JPRB, 0.39722E+00_JPRB, 0.39533E+00_JPRB, 0.39399E+00_JPRB, 0.39191E+00_JPRB, 0.38954E+00_JPRB, 0.38358E+00_JPRB ,&
&  0.37642E+00_JPRB, 0.36827E+00_JPRB, 0.36205E+00_JPRB, 0.35655E+00_JPRB, 0.34627E+00_JPRB, 0.34081E+00_JPRB, 0.34314E+00_JPRB ,&
&  0.34826E+00_JPRB, 0.35466E+00_JPRB, 0.35971E+00_JPRB, 0.37051E+00_JPRB, 0.38945E+00_JPRB, 0.40717E+00_JPRB, 0.42879E+00_JPRB ,&
&  0.47488E+00_JPRB, 0.45631E+00_JPRB, 0.49286E+00_JPRB, 0.50293E+00_JPRB, 0.50669E+00_JPRB, 0.53286E+00_JPRB, 0.56741E+00_JPRB ,&
&  0.59269E+00_JPRB, 0.62620E+00_JPRB, 0.61726E+00_JPRB, 0.57386E+00_JPRB, 0.53347E+00_JPRB, 0.50366E+00_JPRB, 0.48576E+00_JPRB ,&
&  0.47176E+00_JPRB, 0.45996E+00_JPRB, 0.44979E+00_JPRB, 0.44407E+00_JPRB, 0.43823E+00_JPRB, 0.43234E+00_JPRB, 0.42643E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.42206E+00_JPRB, 0.42192E+00_JPRB, 0.42932E+00_JPRB, 0.44833E+00_JPRB, 0.47899E+00_JPRB, 0.51403E+00_JPRB ,&
&  0.54146E+00_JPRB, 0.55276E+00_JPRB, 0.56154E+00_JPRB, 0.55459E+00_JPRB, 0.53645E+00_JPRB, 0.51825E+00_JPRB, 0.50018E+00_JPRB ,&
&  0.47889E+00_JPRB, 0.45544E+00_JPRB, 0.43873E+00_JPRB, 0.42746E+00_JPRB, 0.42096E+00_JPRB, 0.41829E+00_JPRB, 0.41770E+00_JPRB ,&
&  0.41668E+00_JPRB, 0.41755E+00_JPRB, 0.42256E+00_JPRB, 0.42853E+00_JPRB, 0.42920E+00_JPRB, 0.42467E+00_JPRB, 0.41665E+00_JPRB ,&
&  0.40939E+00_JPRB, 0.40304E+00_JPRB, 0.40080E+00_JPRB, 0.40077E+00_JPRB, 0.40197E+00_JPRB, 0.40441E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.40859E+00_JPRB, 0.42899E+00_JPRB, 0.44100E+00_JPRB, 0.47457E+00_JPRB, 0.50362E+00_JPRB, 0.48680E+00_JPRB, 0.46563E+00_JPRB ,&
&  0.45046E+00_JPRB, 0.44366E+00_JPRB, 0.43471E+00_JPRB, 0.43974E+00_JPRB, 0.50396E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.46709E+00_JPRB, 0.45938E+00_JPRB, 0.45095E+00_JPRB, 0.44352E+00_JPRB, 0.43267E+00_JPRB, 0.42210E+00_JPRB, 0.41166E+00_JPRB ,&
&  0.40394E+00_JPRB, 0.40006E+00_JPRB, 0.40061E+00_JPRB, 0.40187E+00_JPRB, 0.40038E+00_JPRB, 0.39429E+00_JPRB, 0.38267E+00_JPRB ,&
&  0.37517E+00_JPRB, 0.36973E+00_JPRB, 0.36353E+00_JPRB, 0.35572E+00_JPRB, 0.34812E+00_JPRB, 0.34558E+00_JPRB, 0.34689E+00_JPRB ,&
&  0.35174E+00_JPRB, 0.35806E+00_JPRB, 0.36260E+00_JPRB, 0.37439E+00_JPRB, 0.38990E+00_JPRB, 0.40804E+00_JPRB, 0.43299E+00_JPRB ,&
&  0.47276E+00_JPRB, 0.55038E+00_JPRB, 0.58231E+00_JPRB, 0.58335E+00_JPRB, 0.58562E+00_JPRB, 0.60170E+00_JPRB, 0.65641E+00_JPRB ,&
&  0.71041E+00_JPRB, 0.74280E+00_JPRB, 0.72413E+00_JPRB, 0.66155E+00_JPRB, 0.61135E+00_JPRB, 0.57698E+00_JPRB, 0.54285E+00_JPRB ,&
&  0.50939E+00_JPRB, 0.48343E+00_JPRB, 0.46313E+00_JPRB, 0.44908E+00_JPRB, 0.43839E+00_JPRB, 0.43079E+00_JPRB, 0.42715E+00_JPRB ,&
&  0.42781E+00_JPRB, 0.43041E+00_JPRB, 0.43915E+00_JPRB, 0.45749E+00_JPRB, 0.51239E+00_JPRB, 0.57194E+00_JPRB, 0.61117E+00_JPRB ,&
&  0.62319E+00_JPRB, 0.63214E+00_JPRB, 0.63481E+00_JPRB, 0.61261E+00_JPRB, 0.58173E+00_JPRB, 0.54577E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.49928E+00_JPRB, 0.48133E+00_JPRB, 0.46522E+00_JPRB, 0.45045E+00_JPRB, 0.44122E+00_JPRB, 0.43777E+00_JPRB, 0.43725E+00_JPRB ,&
&  0.43576E+00_JPRB, 0.43663E+00_JPRB, 0.43875E+00_JPRB, 0.43997E+00_JPRB, 0.43931E+00_JPRB, 0.43715E+00_JPRB, 0.43304E+00_JPRB ,&
&  0.42744E+00_JPRB, 0.42290E+00_JPRB, 0.42809E+00_JPRB, 0.43132E+00_JPRB, 0.43409E+00_JPRB, 0.43367E+00_JPRB, 0.43060E+00_JPRB ,&
&  0.43317E+00_JPRB, 0.45179E+00_JPRB, 0.45880E+00_JPRB, 0.47768E+00_JPRB, 0.49366E+00_JPRB, 0.50447E+00_JPRB, 0.48498E+00_JPRB ,&
&  0.47229E+00_JPRB, 0.45666E+00_JPRB, 0.44377E+00_JPRB, 0.44393E+00_JPRB, 0.46957E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.45665E+00_JPRB, 0.45225E+00_JPRB, 0.44515E+00_JPRB, 0.43790E+00_JPRB, 0.42887E+00_JPRB, 0.41872E+00_JPRB, 0.41135E+00_JPRB ,&
&  0.40727E+00_JPRB, 0.40743E+00_JPRB, 0.40905E+00_JPRB, 0.40140E+00_JPRB, 0.39001E+00_JPRB, 0.37931E+00_JPRB, 0.37296E+00_JPRB ,&
&  0.36834E+00_JPRB, 0.36372E+00_JPRB, 0.35781E+00_JPRB, 0.35011E+00_JPRB, 0.34691E+00_JPRB, 0.34631E+00_JPRB, 0.34646E+00_JPRB ,&
&  0.35179E+00_JPRB, 0.35694E+00_JPRB, 0.36371E+00_JPRB, 0.37272E+00_JPRB, 0.38619E+00_JPRB, 0.40423E+00_JPRB, 0.43347E+00_JPRB ,&
&  0.49260E+00_JPRB, 0.56546E+00_JPRB, 0.68801E+00_JPRB, 0.64938E+00_JPRB, 0.66328E+00_JPRB, 0.69046E+00_JPRB, 0.78106E+00_JPRB ,&
&  0.85818E+00_JPRB, 0.87376E+00_JPRB, 0.81717E+00_JPRB, 0.73493E+00_JPRB, 0.66027E+00_JPRB, 0.60525E+00_JPRB, 0.56783E+00_JPRB ,&
&  0.53148E+00_JPRB, 0.49830E+00_JPRB, 0.47523E+00_JPRB, 0.45943E+00_JPRB, 0.44862E+00_JPRB, 0.44202E+00_JPRB, 0.43966E+00_JPRB ,&
&  0.44277E+00_JPRB, 0.45319E+00_JPRB, 0.47534E+00_JPRB, 0.51937E+00_JPRB, 0.59002E+00_JPRB, 0.58219E+00_JPRB, 0.61595E+00_JPRB ,&
&  0.70234E+00_JPRB, 0.70150E+00_JPRB, 0.68127E+00_JPRB, 0.64079E+00_JPRB, 0.60183E+00_JPRB, 0.56701E+00_JPRB, 0.53475E+00_JPRB ,&
&  0.51054E+00_JPRB, 0.49168E+00_JPRB, 0.47497E+00_JPRB, 0.46400E+00_JPRB, 0.45878E+00_JPRB, 0.45914E+00_JPRB, 0.46023E+00_JPRB ,&
&  0.45904E+00_JPRB, 0.45718E+00_JPRB, 0.45705E+00_JPRB, 0.45860E+00_JPRB, 0.46024E+00_JPRB, 0.46199E+00_JPRB, 0.45583E+00_JPRB ,&
&  0.45056E+00_JPRB, 0.44994E+00_JPRB, 0.45836E+00_JPRB, 0.44889E+00_JPRB, 0.44974E+00_JPRB, 0.45037E+00_JPRB, 0.44536E+00_JPRB ,&
&  0.43767E+00_JPRB, 0.44270E+00_JPRB, 0.45561E+00_JPRB, 0.47013E+00_JPRB, 0.50008E+00_JPRB, 0.52126E+00_JPRB, 0.48803E+00_JPRB ,&
&  0.48078E+00_JPRB, 0.46715E+00_JPRB, 0.45397E+00_JPRB, 0.45256E+00_JPRB, 0.46325E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.44854E+00_JPRB, 0.44339E+00_JPRB, 0.43733E+00_JPRB, 0.43098E+00_JPRB, 0.42289E+00_JPRB, 0.41520E+00_JPRB, 0.41061E+00_JPRB ,&
&  0.41027E+00_JPRB, 0.41346E+00_JPRB, 0.41115E+00_JPRB, 0.40075E+00_JPRB, 0.38770E+00_JPRB, 0.37585E+00_JPRB, 0.37006E+00_JPRB ,&
&  0.36532E+00_JPRB, 0.35854E+00_JPRB, 0.34959E+00_JPRB, 0.34017E+00_JPRB, 0.33754E+00_JPRB, 0.34015E+00_JPRB, 0.33739E+00_JPRB ,&
&  0.34326E+00_JPRB, 0.35020E+00_JPRB, 0.35734E+00_JPRB, 0.36396E+00_JPRB, 0.37730E+00_JPRB, 0.39695E+00_JPRB, 0.42835E+00_JPRB ,&
&  0.48722E+00_JPRB, 0.42214E+00_JPRB, 0.66528E+00_JPRB, 0.75469E+00_JPRB, 0.81184E+00_JPRB, 0.85753E+00_JPRB, 0.94656E+00_JPRB ,&
&  0.10214E+01_JPRB, 0.98154E+00_JPRB, 0.90730E+00_JPRB, 0.79357E+00_JPRB, 0.68905E+00_JPRB, 0.61621E+00_JPRB, 0.57077E+00_JPRB ,&
&  0.54206E+00_JPRB, 0.51986E+00_JPRB, 0.49903E+00_JPRB, 0.48056E+00_JPRB, 0.46758E+00_JPRB, 0.45996E+00_JPRB, 0.45914E+00_JPRB ,&
&  0.47391E+00_JPRB, 0.50429E+00_JPRB, 0.54555E+00_JPRB, 0.62659E+00_JPRB, 0.65122E+00_JPRB, 0.63910E+00_JPRB, 0.62922E+00_JPRB ,&
&  0.68098E+00_JPRB, 0.75296E+00_JPRB, 0.70144E+00_JPRB, 0.63770E+00_JPRB, 0.60445E+00_JPRB, 0.57333E+00_JPRB, 0.54223E+00_JPRB ,&
&  0.51527E+00_JPRB, 0.49740E+00_JPRB, 0.48540E+00_JPRB, 0.47858E+00_JPRB, 0.47729E+00_JPRB, 0.47535E+00_JPRB, 0.46923E+00_JPRB ,&
&  0.46558E+00_JPRB, 0.46452E+00_JPRB, 0.46596E+00_JPRB, 0.46947E+00_JPRB, 0.47018E+00_JPRB, 0.47118E+00_JPRB, 0.46993E+00_JPRB ,&
&  0.46819E+00_JPRB, 0.46924E+00_JPRB, 0.46329E+00_JPRB, 0.45136E+00_JPRB, 0.44843E+00_JPRB, 0.44393E+00_JPRB, 0.44050E+00_JPRB ,&
&  0.44474E+00_JPRB, 0.45011E+00_JPRB, 0.45796E+00_JPRB, 0.47864E+00_JPRB, 0.48905E+00_JPRB, 0.51297E+00_JPRB, 0.48579E+00_JPRB ,&
&  0.48568E+00_JPRB, 0.47750E+00_JPRB, 0.46997E+00_JPRB, 0.46088E+00_JPRB, 0.45865E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.44799E+00_JPRB, 0.44163E+00_JPRB, 0.43539E+00_JPRB, 0.42772E+00_JPRB, 0.41803E+00_JPRB, 0.41210E+00_JPRB, 0.41425E+00_JPRB ,&
&  0.41992E+00_JPRB, 0.41711E+00_JPRB, 0.41179E+00_JPRB, 0.40187E+00_JPRB, 0.38657E+00_JPRB, 0.37970E+00_JPRB, 0.37550E+00_JPRB ,&
&  0.36795E+00_JPRB, 0.36233E+00_JPRB, 0.35264E+00_JPRB, 0.34763E+00_JPRB, 0.34598E+00_JPRB, 0.34270E+00_JPRB, 0.33630E+00_JPRB ,&
&  0.33927E+00_JPRB, 0.34609E+00_JPRB, 0.35439E+00_JPRB, 0.36186E+00_JPRB, 0.37614E+00_JPRB, 0.39669E+00_JPRB, 0.42859E+00_JPRB ,&
&  0.48953E+00_JPRB, 0.69288E+00_JPRB, 0.61509E+00_JPRB, 0.96048E+00_JPRB, 0.98374E+00_JPRB, 0.10279E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.10167E+01_JPRB, 0.97720E+00_JPRB, 0.83769E+00_JPRB, 0.70466E+00_JPRB, 0.62588E+00_JPRB, 0.57610E+00_JPRB ,&
&  0.54162E+00_JPRB, 0.51411E+00_JPRB, 0.49270E+00_JPRB, 0.47736E+00_JPRB, 0.46530E+00_JPRB, 0.46348E+00_JPRB, 0.48702E+00_JPRB ,&
&  0.53454E+00_JPRB, 0.57982E+00_JPRB, 0.65644E+00_JPRB, 0.73182E+00_JPRB, 0.68014E+00_JPRB, 0.65482E+00_JPRB, 0.66208E+00_JPRB ,&
&  0.68385E+00_JPRB, 0.74442E+00_JPRB, 0.65922E+00_JPRB, 0.61207E+00_JPRB, 0.58776E+00_JPRB, 0.57203E+00_JPRB, 0.54650E+00_JPRB ,&
&  0.52253E+00_JPRB, 0.50986E+00_JPRB, 0.50216E+00_JPRB, 0.49808E+00_JPRB, 0.49251E+00_JPRB, 0.48475E+00_JPRB, 0.47620E+00_JPRB ,&
&  0.46872E+00_JPRB, 0.47205E+00_JPRB, 0.47586E+00_JPRB, 0.47799E+00_JPRB, 0.47961E+00_JPRB, 0.47811E+00_JPRB, 0.47682E+00_JPRB ,&
&  0.47824E+00_JPRB, 0.48139E+00_JPRB, 0.46094E+00_JPRB, 0.44543E+00_JPRB, 0.43952E+00_JPRB, 0.43595E+00_JPRB, 0.43415E+00_JPRB ,&
&  0.44600E+00_JPRB, 0.45667E+00_JPRB, 0.46578E+00_JPRB, 0.48270E+00_JPRB, 0.50596E+00_JPRB, 0.53237E+00_JPRB, 0.49857E+00_JPRB ,&
&  0.50472E+00_JPRB, 0.49529E+00_JPRB, 0.48376E+00_JPRB, 0.46665E+00_JPRB, 0.45812E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.45079E+00_JPRB, 0.44152E+00_JPRB, 0.43550E+00_JPRB, 0.42926E+00_JPRB, 0.42332E+00_JPRB, 0.42167E+00_JPRB, 0.42444E+00_JPRB ,&
&  0.42517E+00_JPRB, 0.41951E+00_JPRB, 0.40994E+00_JPRB, 0.40033E+00_JPRB, 0.38480E+00_JPRB, 0.37598E+00_JPRB, 0.37186E+00_JPRB ,&
&  0.36874E+00_JPRB, 0.36936E+00_JPRB, 0.36561E+00_JPRB, 0.36261E+00_JPRB, 0.35992E+00_JPRB, 0.35455E+00_JPRB, 0.35338E+00_JPRB ,&
&  0.35151E+00_JPRB, 0.35440E+00_JPRB, 0.36103E+00_JPRB, 0.37285E+00_JPRB, 0.39027E+00_JPRB, 0.41554E+00_JPRB, 0.44608E+00_JPRB ,&
&  0.52064E+00_JPRB, 0.63949E+00_JPRB, 0.66044E+00_JPRB, 0.12280E+01_JPRB, 0.12054E+01_JPRB, 0.11437E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.96121E+00_JPRB, 0.10160E+01_JPRB, 0.98500E+00_JPRB, 0.87023E+00_JPRB, 0.74501E+00_JPRB, 0.65142E+00_JPRB, 0.57566E+00_JPRB ,&
&  0.52724E+00_JPRB, 0.49577E+00_JPRB, 0.47980E+00_JPRB, 0.47273E+00_JPRB, 0.47173E+00_JPRB, 0.49970E+00_JPRB, 0.55890E+00_JPRB ,&
&  0.61061E+00_JPRB, 0.66913E+00_JPRB, 0.78249E+00_JPRB, 0.74544E+00_JPRB, 0.67224E+00_JPRB, 0.65320E+00_JPRB, 0.65162E+00_JPRB ,&
&  0.66658E+00_JPRB, 0.65959E+00_JPRB, 0.61313E+00_JPRB, 0.58175E+00_JPRB, 0.55412E+00_JPRB, 0.55816E+00_JPRB, 0.54460E+00_JPRB ,&
&  0.52856E+00_JPRB, 0.51948E+00_JPRB, 0.51173E+00_JPRB, 0.50580E+00_JPRB, 0.49717E+00_JPRB, 0.48987E+00_JPRB, 0.47676E+00_JPRB ,&
&  0.47667E+00_JPRB, 0.47880E+00_JPRB, 0.47941E+00_JPRB, 0.48384E+00_JPRB, 0.47953E+00_JPRB, 0.47841E+00_JPRB, 0.47857E+00_JPRB ,&
&  0.48503E+00_JPRB, 0.48498E+00_JPRB, 0.45789E+00_JPRB, 0.44285E+00_JPRB, 0.43959E+00_JPRB, 0.43685E+00_JPRB, 0.43686E+00_JPRB ,&
&  0.45126E+00_JPRB, 0.46799E+00_JPRB, 0.47898E+00_JPRB, 0.48561E+00_JPRB, 0.51265E+00_JPRB, 0.52672E+00_JPRB, 0.52316E+00_JPRB ,&
&  0.51663E+00_JPRB, 0.50899E+00_JPRB, 0.49785E+00_JPRB, 0.47327E+00_JPRB, 0.46238E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.44989E+00_JPRB, 0.43889E+00_JPRB, 0.43003E+00_JPRB, 0.42741E+00_JPRB, 0.42325E+00_JPRB, 0.42037E+00_JPRB, 0.42632E+00_JPRB ,&
&  0.42205E+00_JPRB, 0.41243E+00_JPRB, 0.39704E+00_JPRB, 0.38072E+00_JPRB, 0.36748E+00_JPRB, 0.36913E+00_JPRB, 0.37400E+00_JPRB ,&
&  0.37451E+00_JPRB, 0.37295E+00_JPRB, 0.37149E+00_JPRB, 0.37236E+00_JPRB, 0.37075E+00_JPRB, 0.36988E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.36841E+00_JPRB, 0.37231E+00_JPRB, 0.38511E+00_JPRB, 0.40626E+00_JPRB, 0.43096E+00_JPRB, 0.46683E+00_JPRB, 0.51438E+00_JPRB ,&
&  0.61593E+00_JPRB, 0.68251E+00_JPRB, 0.76229E+00_JPRB, 0.11432E+01_JPRB, 0.11985E+01_JPRB, 0.10861E+01_JPRB, 0.98402E+00_JPRB ,&
&  0.88105E+00_JPRB, 0.82273E+00_JPRB, 0.94903E+00_JPRB, 0.85064E+00_JPRB, 0.73908E+00_JPRB, 0.63519E+00_JPRB, 0.56118E+00_JPRB ,&
&  0.51839E+00_JPRB, 0.48858E+00_JPRB, 0.48076E+00_JPRB, 0.48843E+00_JPRB, 0.52262E+00_JPRB, 0.58696E+00_JPRB, 0.65321E+00_JPRB ,&
&  0.70021E+00_JPRB, 0.78653E+00_JPRB, 0.91237E+00_JPRB, 0.77784E+00_JPRB, 0.63845E+00_JPRB, 0.61537E+00_JPRB, 0.59836E+00_JPRB ,&
&  0.61989E+00_JPRB, 0.59872E+00_JPRB, 0.58697E+00_JPRB, 0.56348E+00_JPRB, 0.53262E+00_JPRB, 0.56027E+00_JPRB, 0.53990E+00_JPRB ,&
&  0.53057E+00_JPRB, 0.52180E+00_JPRB, 0.51283E+00_JPRB, 0.50662E+00_JPRB, 0.49492E+00_JPRB, 0.47682E+00_JPRB, 0.46596E+00_JPRB ,&
&  0.46759E+00_JPRB, 0.47396E+00_JPRB, 0.47273E+00_JPRB, 0.47310E+00_JPRB, 0.46864E+00_JPRB, 0.47082E+00_JPRB, 0.47564E+00_JPRB ,&
&  0.48575E+00_JPRB, 0.48849E+00_JPRB, 0.46989E+00_JPRB, 0.46333E+00_JPRB, 0.46103E+00_JPRB, 0.46017E+00_JPRB, 0.45864E+00_JPRB ,&
&  0.46798E+00_JPRB, 0.48827E+00_JPRB, 0.50224E+00_JPRB, 0.51534E+00_JPRB, 0.53121E+00_JPRB, 0.53852E+00_JPRB, 0.53750E+00_JPRB ,&
&  0.54017E+00_JPRB, 0.55672E+00_JPRB, 0.56347E+00_JPRB, 0.49801E+00_JPRB, 0.47011E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.43082E+00_JPRB, 0.42203E+00_JPRB, 0.41852E+00_JPRB, 0.41829E+00_JPRB, 0.41614E+00_JPRB, 0.41835E+00_JPRB, 0.42213E+00_JPRB ,&
&  0.40476E+00_JPRB, 0.38605E+00_JPRB, 0.37315E+00_JPRB, 0.36826E+00_JPRB, 0.36746E+00_JPRB, 0.37825E+00_JPRB, 0.38191E+00_JPRB ,&
&  0.37863E+00_JPRB, 0.37734E+00_JPRB, 0.38728E+00_JPRB, 0.38879E+00_JPRB, 0.38934E+00_JPRB, 0.38766E+00_JPRB, 0.38821E+00_JPRB ,&
&  0.39622E+00_JPRB, 0.41123E+00_JPRB, 0.44002E+00_JPRB, 0.48196E+00_JPRB, 0.52434E+00_JPRB, 0.58897E+00_JPRB, 0.65749E+00_JPRB ,&
&  0.77403E+00_JPRB, 0.78995E+00_JPRB, 0.43521E+00_JPRB, 0.70046E+00_JPRB, 0.92430E+00_JPRB, 0.93328E+00_JPRB, 0.86124E+00_JPRB ,&
&  0.79403E+00_JPRB, 0.74104E+00_JPRB, 0.80813E+00_JPRB, 0.80496E+00_JPRB, 0.68707E+00_JPRB, 0.61094E+00_JPRB, 0.55142E+00_JPRB ,&
&  0.51633E+00_JPRB, 0.50496E+00_JPRB, 0.51417E+00_JPRB, 0.55063E+00_JPRB, 0.62053E+00_JPRB, 0.70341E+00_JPRB, 0.75621E+00_JPRB ,&
&  0.81524E+00_JPRB, 0.94405E+00_JPRB, 0.10194E+01_JPRB, 0.76149E+00_JPRB, 0.61658E+00_JPRB, 0.58243E+00_JPRB, 0.55556E+00_JPRB ,&
&  0.54202E+00_JPRB, 0.56645E+00_JPRB, 0.58153E+00_JPRB, 0.55710E+00_JPRB, 0.53309E+00_JPRB, 0.52156E+00_JPRB, 0.53564E+00_JPRB ,&
&  0.52552E+00_JPRB, 0.51760E+00_JPRB, 0.51055E+00_JPRB, 0.49525E+00_JPRB, 0.46832E+00_JPRB, 0.44378E+00_JPRB, 0.44782E+00_JPRB ,&
&  0.45032E+00_JPRB, 0.45832E+00_JPRB, 0.45528E+00_JPRB, 0.46166E+00_JPRB, 0.46701E+00_JPRB, 0.47383E+00_JPRB, 0.47951E+00_JPRB ,&
&  0.48867E+00_JPRB, 0.49710E+00_JPRB, 0.50583E+00_JPRB, 0.49929E+00_JPRB, 0.49393E+00_JPRB, 0.49274E+00_JPRB, 0.49696E+00_JPRB ,&
&  0.50857E+00_JPRB, 0.52270E+00_JPRB, 0.52323E+00_JPRB, 0.53423E+00_JPRB, 0.56396E+00_JPRB, 0.57790E+00_JPRB, 0.58436E+00_JPRB ,&
&  0.63686E+00_JPRB, 0.68257E+00_JPRB, 0.62994E+00_JPRB, 0.48022E+00_JPRB, 0.44947E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.41774E+00_JPRB, 0.41665E+00_JPRB, 0.41397E+00_JPRB, 0.41643E+00_JPRB, 0.41251E+00_JPRB, 0.41568E+00_JPRB, 0.40516E+00_JPRB ,&
&  0.38393E+00_JPRB, 0.37588E+00_JPRB, 0.38974E+00_JPRB, 0.39470E+00_JPRB, 0.39736E+00_JPRB, 0.41008E+00_JPRB, 0.42420E+00_JPRB ,&
&  0.42771E+00_JPRB, 0.42579E+00_JPRB, 0.42269E+00_JPRB, 0.42173E+00_JPRB, 0.41800E+00_JPRB, 0.41984E+00_JPRB, 0.43306E+00_JPRB ,&
&  0.45267E+00_JPRB, 0.48864E+00_JPRB, 0.54211E+00_JPRB, 0.60749E+00_JPRB, 0.67369E+00_JPRB, 0.75651E+00_JPRB, 0.81803E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.65418E+00_JPRB, 0.46838E+00_JPRB, 0.78815E+00_JPRB, 0.89371E+00_JPRB, 0.88345E+00_JPRB, 0.77220E+00_JPRB ,&
&  0.73243E+00_JPRB, 0.71228E+00_JPRB, 0.71527E+00_JPRB, 0.69355E+00_JPRB, 0.62578E+00_JPRB, 0.59313E+00_JPRB, 0.56145E+00_JPRB ,&
&  0.54492E+00_JPRB, 0.55423E+00_JPRB, 0.58882E+00_JPRB, 0.66099E+00_JPRB, 0.75824E+00_JPRB, 0.82644E+00_JPRB, 0.87791E+00_JPRB ,&
&  0.99074E+00_JPRB, 0.11221E+01_JPRB, 0.11446E+01_JPRB, 0.82412E+00_JPRB, 0.64094E+00_JPRB, 0.58006E+00_JPRB, 0.55659E+00_JPRB ,&
&  0.53912E+00_JPRB, 0.55453E+00_JPRB, 0.57824E+00_JPRB, 0.56189E+00_JPRB, 0.53910E+00_JPRB, 0.51707E+00_JPRB, 0.53199E+00_JPRB ,&
&  0.52009E+00_JPRB, 0.50835E+00_JPRB, 0.49270E+00_JPRB, 0.46669E+00_JPRB, 0.43876E+00_JPRB, 0.43913E+00_JPRB, 0.43294E+00_JPRB ,&
&  0.43512E+00_JPRB, 0.44587E+00_JPRB, 0.44422E+00_JPRB, 0.45492E+00_JPRB, 0.46687E+00_JPRB, 0.48317E+00_JPRB, 0.49764E+00_JPRB ,&
&  0.50656E+00_JPRB, 0.51856E+00_JPRB, 0.53216E+00_JPRB, 0.53326E+00_JPRB, 0.52637E+00_JPRB, 0.51546E+00_JPRB, 0.52816E+00_JPRB ,&
&  0.53772E+00_JPRB, 0.55366E+00_JPRB, 0.55519E+00_JPRB, 0.57008E+00_JPRB, 0.55981E+00_JPRB, 0.55818E+00_JPRB, 0.53070E+00_JPRB ,&
&  0.53140E+00_JPRB, 0.50246E+00_JPRB, 0.46990E+00_JPRB, 0.42123E+00_JPRB, 0.41621E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.42344E+00_JPRB, 0.41803E+00_JPRB, 0.41044E+00_JPRB, 0.40237E+00_JPRB, 0.39417E+00_JPRB, 0.39440E+00_JPRB, 0.38452E+00_JPRB ,&
&  0.38113E+00_JPRB, 0.39924E+00_JPRB, 0.42491E+00_JPRB, 0.44565E+00_JPRB, 0.46314E+00_JPRB, 0.46859E+00_JPRB, 0.48128E+00_JPRB ,&
&  0.47940E+00_JPRB, 0.47598E+00_JPRB, 0.47380E+00_JPRB, 0.47240E+00_JPRB, 0.47675E+00_JPRB, 0.49364E+00_JPRB, 0.52430E+00_JPRB ,&
&  0.56841E+00_JPRB, 0.62578E+00_JPRB, 0.69402E+00_JPRB, 0.76417E+00_JPRB, 0.82432E+00_JPRB, 0.88138E+00_JPRB, 0.89684E+00_JPRB ,&
&  0.64711E+00_JPRB, 0.41937E+00_JPRB, 0.80166E+00_JPRB, 0.89354E+00_JPRB, 0.83285E+00_JPRB, 0.79964E+00_JPRB, 0.74282E+00_JPRB ,&
&  0.68852E+00_JPRB, 0.68079E+00_JPRB, 0.66020E+00_JPRB, 0.65409E+00_JPRB, 0.60300E+00_JPRB, 0.59056E+00_JPRB, 0.58763E+00_JPRB ,&
&  0.59569E+00_JPRB, 0.62337E+00_JPRB, 0.68762E+00_JPRB, 0.79497E+00_JPRB, 0.87880E+00_JPRB, 0.92655E+00_JPRB, 0.10154E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.13017E+01_JPRB, 0.13464E+01_JPRB, 0.96135E+00_JPRB, 0.73000E+00_JPRB, 0.67361E+00_JPRB, 0.62797E+00_JPRB ,&
&  0.60133E+00_JPRB, 0.53933E+00_JPRB, 0.54642E+00_JPRB, 0.56558E+00_JPRB, 0.54964E+00_JPRB, 0.51228E+00_JPRB, 0.51923E+00_JPRB ,&
&  0.50273E+00_JPRB, 0.47423E+00_JPRB, 0.44469E+00_JPRB, 0.42983E+00_JPRB, 0.42521E+00_JPRB, 0.43507E+00_JPRB, 0.43377E+00_JPRB ,&
&  0.43564E+00_JPRB, 0.44607E+00_JPRB, 0.44822E+00_JPRB, 0.45490E+00_JPRB, 0.48139E+00_JPRB, 0.50814E+00_JPRB, 0.52522E+00_JPRB ,&
&  0.53194E+00_JPRB, 0.54697E+00_JPRB, 0.55881E+00_JPRB, 0.56430E+00_JPRB, 0.56489E+00_JPRB, 0.54926E+00_JPRB, 0.54882E+00_JPRB ,&
&  0.55565E+00_JPRB, 0.56192E+00_JPRB, 0.55897E+00_JPRB, 0.56169E+00_JPRB, 0.54308E+00_JPRB, 0.53993E+00_JPRB, 0.48925E+00_JPRB ,&
&  0.46746E+00_JPRB, 0.43182E+00_JPRB, 0.40546E+00_JPRB, 0.39543E+00_JPRB, 0.40172E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.42469E+00_JPRB, 0.40831E+00_JPRB, 0.39827E+00_JPRB, 0.39021E+00_JPRB, 0.38045E+00_JPRB, 0.39996E+00_JPRB, 0.40351E+00_JPRB ,&
&  0.42270E+00_JPRB, 0.45531E+00_JPRB, 0.49212E+00_JPRB, 0.50769E+00_JPRB, 0.50647E+00_JPRB, 0.52175E+00_JPRB, 0.53490E+00_JPRB ,&
&  0.54138E+00_JPRB, 0.54577E+00_JPRB, 0.55337E+00_JPRB, 0.56361E+00_JPRB, 0.58339E+00_JPRB, 0.61425E+00_JPRB, 0.66048E+00_JPRB ,&
&  0.71283E+00_JPRB, 0.76802E+00_JPRB, 0.82642E+00_JPRB, 0.87827E+00_JPRB, 0.90755E+00_JPRB, 0.93221E+00_JPRB, 0.90007E+00_JPRB ,&
&  0.58502E+00_JPRB, 0.77644E+00_JPRB, 0.88160E+00_JPRB, 0.83088E+00_JPRB, 0.78490E+00_JPRB, 0.75046E+00_JPRB, 0.72428E+00_JPRB ,&
&  0.69437E+00_JPRB, 0.65609E+00_JPRB, 0.64221E+00_JPRB, 0.66492E+00_JPRB, 0.62339E+00_JPRB, 0.61390E+00_JPRB, 0.62479E+00_JPRB ,&
&  0.64789E+00_JPRB, 0.69761E+00_JPRB, 0.79331E+00_JPRB, 0.90526E+00_JPRB, 0.97602E+00_JPRB, 0.10477E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.14832E+01_JPRB, 0.15726E+01_JPRB, 0.12105E+01_JPRB, 0.10629E+01_JPRB, 0.10410E+01_JPRB, 0.95553E+00_JPRB ,&
&  0.67120E+00_JPRB, 0.54493E+00_JPRB, 0.55741E+00_JPRB, 0.57689E+00_JPRB, 0.53882E+00_JPRB, 0.51126E+00_JPRB, 0.48191E+00_JPRB ,&
&  0.44830E+00_JPRB, 0.41596E+00_JPRB, 0.39554E+00_JPRB, 0.41783E+00_JPRB, 0.44070E+00_JPRB, 0.45320E+00_JPRB, 0.44465E+00_JPRB ,&
&  0.46543E+00_JPRB, 0.45445E+00_JPRB, 0.45311E+00_JPRB, 0.46406E+00_JPRB, 0.51169E+00_JPRB, 0.53050E+00_JPRB, 0.53653E+00_JPRB ,&
&  0.54384E+00_JPRB, 0.55136E+00_JPRB, 0.55646E+00_JPRB, 0.56392E+00_JPRB, 0.56854E+00_JPRB, 0.56824E+00_JPRB, 0.56395E+00_JPRB ,&
&  0.55929E+00_JPRB, 0.56592E+00_JPRB, 0.56156E+00_JPRB, 0.55657E+00_JPRB, 0.54702E+00_JPRB, 0.57280E+00_JPRB, 0.50829E+00_JPRB ,&
&  0.46305E+00_JPRB, 0.42974E+00_JPRB, 0.40470E+00_JPRB, 0.39810E+00_JPRB, 0.40418E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.42884E+00_JPRB, 0.41695E+00_JPRB, 0.39977E+00_JPRB, 0.40686E+00_JPRB, 0.40832E+00_JPRB, 0.42304E+00_JPRB, 0.43986E+00_JPRB ,&
&  0.46174E+00_JPRB, 0.50242E+00_JPRB, 0.53757E+00_JPRB, 0.56404E+00_JPRB, 0.57531E+00_JPRB, 0.59304E+00_JPRB, 0.61061E+00_JPRB ,&
&  0.61834E+00_JPRB, 0.62864E+00_JPRB, 0.64215E+00_JPRB, 0.66480E+00_JPRB, 0.69482E+00_JPRB, 0.72871E+00_JPRB, 0.77079E+00_JPRB ,&
&  0.81756E+00_JPRB, 0.87007E+00_JPRB, 0.93876E+00_JPRB, 0.95231E+00_JPRB, 0.96703E+00_JPRB, 0.98977E+00_JPRB, 0.95978E+00_JPRB ,&
&  0.71809E+00_JPRB, 0.84134E+00_JPRB, 0.81777E+00_JPRB, 0.78325E+00_JPRB, 0.75735E+00_JPRB, 0.72223E+00_JPRB, 0.69276E+00_JPRB ,&
&  0.67038E+00_JPRB, 0.62881E+00_JPRB, 0.61199E+00_JPRB, 0.60711E+00_JPRB, 0.64601E+00_JPRB, 0.63938E+00_JPRB, 0.66414E+00_JPRB ,&
&  0.70821E+00_JPRB, 0.78433E+00_JPRB, 0.88214E+00_JPRB, 0.98205E+00_JPRB, 0.10771E+01_JPRB, 0.11761E+01_JPRB, 0.13153E+01_JPRB ,&
&  0.14678E+01_JPRB, 0.15664E+01_JPRB, 0.16156E+01_JPRB, 0.13946E+01_JPRB, 0.11454E+01_JPRB, 0.10211E+01_JPRB, 0.81284E+00_JPRB ,&
&  0.61901E+00_JPRB, 0.55037E+00_JPRB, 0.57600E+00_JPRB, 0.59454E+00_JPRB, 0.54817E+00_JPRB, 0.50040E+00_JPRB, 0.46022E+00_JPRB ,&
&  0.43096E+00_JPRB, 0.40621E+00_JPRB, 0.42175E+00_JPRB, 0.47094E+00_JPRB, 0.49528E+00_JPRB, 0.51378E+00_JPRB, 0.51325E+00_JPRB ,&
&  0.50610E+00_JPRB, 0.48035E+00_JPRB, 0.46326E+00_JPRB, 0.47479E+00_JPRB, 0.50998E+00_JPRB, 0.54365E+00_JPRB, 0.54365E+00_JPRB ,&
&  0.52920E+00_JPRB, 0.52435E+00_JPRB, 0.53425E+00_JPRB, 0.55656E+00_JPRB, 0.57485E+00_JPRB, 0.60071E+00_JPRB, 0.62265E+00_JPRB ,&
&  0.65294E+00_JPRB, 0.69805E+00_JPRB, 0.69092E+00_JPRB, 0.69255E+00_JPRB, 0.79156E+00_JPRB, 0.84728E+00_JPRB, 0.53704E+00_JPRB ,&
&  0.45425E+00_JPRB, 0.42100E+00_JPRB, 0.41619E+00_JPRB, 0.40905E+00_JPRB, 0.43628E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.46246E+00_JPRB, 0.47970E+00_JPRB, 0.46969E+00_JPRB, 0.47173E+00_JPRB, 0.47803E+00_JPRB, 0.48770E+00_JPRB, 0.50829E+00_JPRB ,&
&  0.51234E+00_JPRB, 0.53704E+00_JPRB, 0.56085E+00_JPRB, 0.57919E+00_JPRB, 0.60682E+00_JPRB, 0.64592E+00_JPRB, 0.69273E+00_JPRB ,&
&  0.73814E+00_JPRB, 0.76203E+00_JPRB, 0.77753E+00_JPRB, 0.78532E+00_JPRB, 0.80649E+00_JPRB, 0.84034E+00_JPRB, 0.85150E+00_JPRB ,&
&  0.89898E+00_JPRB, 0.93023E+00_JPRB, 0.97082E+00_JPRB, 0.10318E+01_JPRB, 0.97970E+00_JPRB, 0.97278E+00_JPRB, 0.71514E+00_JPRB ,&
&  0.80844E+00_JPRB, 0.81202E+00_JPRB, 0.79356E+00_JPRB, 0.77380E+00_JPRB, 0.74489E+00_JPRB, 0.69900E+00_JPRB, 0.66820E+00_JPRB ,&
&  0.64563E+00_JPRB, 0.62512E+00_JPRB, 0.60522E+00_JPRB, 0.59997E+00_JPRB, 0.66059E+00_JPRB, 0.66424E+00_JPRB, 0.71283E+00_JPRB ,&
&  0.78330E+00_JPRB, 0.86488E+00_JPRB, 0.94326E+00_JPRB, 0.10355E+01_JPRB, 0.11489E+01_JPRB, 0.12709E+01_JPRB, 0.13978E+01_JPRB ,&
&  0.14972E+01_JPRB, 0.15768E+01_JPRB, 0.15451E+01_JPRB, 0.12746E+01_JPRB, 0.11436E+01_JPRB, 0.91460E+00_JPRB, 0.70565E+00_JPRB ,&
&  0.60837E+00_JPRB, 0.58332E+00_JPRB, 0.61585E+00_JPRB, 0.63541E+00_JPRB, 0.57003E+00_JPRB, 0.50877E+00_JPRB, 0.46583E+00_JPRB ,&
&  0.44218E+00_JPRB, 0.43942E+00_JPRB, 0.46595E+00_JPRB, 0.49500E+00_JPRB, 0.53187E+00_JPRB, 0.56687E+00_JPRB, 0.57262E+00_JPRB ,&
&  0.55961E+00_JPRB, 0.52655E+00_JPRB, 0.51217E+00_JPRB, 0.52999E+00_JPRB, 0.59071E+00_JPRB, 0.65518E+00_JPRB, 0.70623E+00_JPRB ,&
&  0.57975E+00_JPRB, 0.49362E+00_JPRB, 0.52243E+00_JPRB, 0.56131E+00_JPRB, 0.61496E+00_JPRB, 0.69027E+00_JPRB, 0.80460E+00_JPRB ,&
&  0.97475E+00_JPRB, 0.13273E+01_JPRB, 0.13098E+01_JPRB, 0.10938E+01_JPRB, 0.16151E+01_JPRB, 0.13354E+01_JPRB, 0.47863E+00_JPRB ,&
&  0.45862E+00_JPRB, 0.45035E+00_JPRB, 0.45234E+00_JPRB, 0.45329E+00_JPRB, 0.46162E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.54796E+00_JPRB, 0.55525E+00_JPRB, 0.57060E+00_JPRB, 0.56993E+00_JPRB, 0.57696E+00_JPRB, 0.59386E+00_JPRB, 0.61062E+00_JPRB ,&
&  0.62926E+00_JPRB, 0.63572E+00_JPRB, 0.64646E+00_JPRB, 0.65878E+00_JPRB, 0.67322E+00_JPRB, 0.70774E+00_JPRB, 0.76116E+00_JPRB ,&
&  0.79460E+00_JPRB, 0.82470E+00_JPRB, 0.86478E+00_JPRB, 0.86731E+00_JPRB, 0.88453E+00_JPRB, 0.91898E+00_JPRB, 0.93291E+00_JPRB ,&
&  0.96760E+00_JPRB, 0.98323E+00_JPRB, 0.99809E+00_JPRB, 0.10435E+01_JPRB, 0.10018E+01_JPRB, 0.10080E+01_JPRB, 0.78390E+00_JPRB ,&
&  0.83423E+00_JPRB, 0.81286E+00_JPRB, 0.76645E+00_JPRB, 0.74419E+00_JPRB, 0.72190E+00_JPRB, 0.68906E+00_JPRB, 0.66406E+00_JPRB ,&
&  0.65374E+00_JPRB, 0.63061E+00_JPRB, 0.60850E+00_JPRB, 0.63602E+00_JPRB, 0.65874E+00_JPRB, 0.70076E+00_JPRB, 0.76901E+00_JPRB ,&
&  0.84079E+00_JPRB, 0.91072E+00_JPRB, 0.97245E+00_JPRB, 0.10608E+01_JPRB, 0.11746E+01_JPRB, 0.12972E+01_JPRB, 0.14301E+01_JPRB ,&
&  0.15159E+01_JPRB, 0.15777E+01_JPRB, 0.15839E+01_JPRB, 0.13459E+01_JPRB, 0.12794E+01_JPRB, 0.10126E+01_JPRB, 0.74604E+00_JPRB ,&
&  0.61416E+00_JPRB, 0.57619E+00_JPRB, 0.60314E+00_JPRB, 0.66553E+00_JPRB, 0.60655E+00_JPRB, 0.55640E+00_JPRB, 0.50964E+00_JPRB ,&
&  0.48792E+00_JPRB, 0.48384E+00_JPRB, 0.49048E+00_JPRB, 0.51687E+00_JPRB, 0.55491E+00_JPRB, 0.59644E+00_JPRB, 0.61961E+00_JPRB ,&
&  0.61432E+00_JPRB, 0.57761E+00_JPRB, 0.54230E+00_JPRB, 0.51859E+00_JPRB, 0.52288E+00_JPRB, 0.54647E+00_JPRB, 0.51766E+00_JPRB ,&
&  0.53070E+00_JPRB, 0.50996E+00_JPRB, 0.51971E+00_JPRB, 0.58172E+00_JPRB, 0.68641E+00_JPRB, 0.89805E+00_JPRB, 0.12980E+01_JPRB ,&
&  0.18889E+01_JPRB, 0.20493E+01_JPRB, 0.25463E+01_JPRB, 0.34178E+01_JPRB, 0.25149E+01_JPRB, 0.17581E+01_JPRB, 0.52683E+00_JPRB ,&
&  0.53942E+00_JPRB, 0.53341E+00_JPRB, 0.53078E+00_JPRB, 0.52784E+00_JPRB, 0.53272E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.57641E+00_JPRB, 0.58933E+00_JPRB, 0.60474E+00_JPRB, 0.61793E+00_JPRB, 0.63584E+00_JPRB, 0.66223E+00_JPRB, 0.68825E+00_JPRB ,&
&  0.70953E+00_JPRB, 0.72371E+00_JPRB, 0.73215E+00_JPRB, 0.74759E+00_JPRB, 0.76750E+00_JPRB, 0.79303E+00_JPRB, 0.82707E+00_JPRB ,&
&  0.83319E+00_JPRB, 0.86029E+00_JPRB, 0.86656E+00_JPRB, 0.88897E+00_JPRB, 0.88924E+00_JPRB, 0.91854E+00_JPRB, 0.93648E+00_JPRB ,&
&  0.94565E+00_JPRB, 0.94911E+00_JPRB, 0.94960E+00_JPRB, 0.96237E+00_JPRB, 0.93844E+00_JPRB, 0.94859E+00_JPRB, 0.73494E+00_JPRB ,&
&  0.81670E+00_JPRB, 0.78560E+00_JPRB, 0.71293E+00_JPRB, 0.68072E+00_JPRB, 0.65271E+00_JPRB, 0.63244E+00_JPRB, 0.61944E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.62131E+00_JPRB, 0.61686E+00_JPRB, 0.62958E+00_JPRB, 0.67532E+00_JPRB, 0.73860E+00_JPRB, 0.79104E+00_JPRB ,&
&  0.86173E+00_JPRB, 0.92759E+00_JPRB, 0.98703E+00_JPRB, 0.10630E+01_JPRB, 0.11617E+01_JPRB, 0.12657E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.14075E+01_JPRB, 0.12897E+01_JPRB, 0.12810E+01_JPRB, 0.13142E+01_JPRB, 0.11167E+01_JPRB, 0.83241E+00_JPRB ,&
&  0.56578E+00_JPRB, 0.56275E+00_JPRB, 0.61710E+00_JPRB, 0.71810E+00_JPRB, 0.67837E+00_JPRB, 0.63553E+00_JPRB, 0.58751E+00_JPRB ,&
&  0.56151E+00_JPRB, 0.53780E+00_JPRB, 0.54050E+00_JPRB, 0.56711E+00_JPRB, 0.60719E+00_JPRB, 0.64742E+00_JPRB, 0.66480E+00_JPRB ,&
&  0.65331E+00_JPRB, 0.61719E+00_JPRB, 0.56929E+00_JPRB, 0.52512E+00_JPRB, 0.49211E+00_JPRB, 0.45795E+00_JPRB, 0.51673E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.47689E+00_JPRB, 0.51330E+00_JPRB, 0.47335E+00_JPRB, 0.65819E+00_JPRB, 0.81388E+00_JPRB, 0.10609E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.16368E+01_JPRB, 0.20604E+01_JPRB, 0.14690E+01_JPRB, 0.10821E+01_JPRB, 0.65259E+00_JPRB, 0.57851E+00_JPRB ,&
&  0.57449E+00_JPRB, 0.55691E+00_JPRB, 0.55342E+00_JPRB, 0.55935E+00_JPRB, 0.56630E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.60652E+00_JPRB, 0.62233E+00_JPRB, 0.63825E+00_JPRB, 0.65322E+00_JPRB, 0.66760E+00_JPRB, 0.68016E+00_JPRB, 0.69413E+00_JPRB ,&
&  0.70880E+00_JPRB, 0.72449E+00_JPRB, 0.73939E+00_JPRB, 0.75792E+00_JPRB, 0.77557E+00_JPRB, 0.78396E+00_JPRB, 0.79439E+00_JPRB ,&
&  0.79807E+00_JPRB, 0.82088E+00_JPRB, 0.82696E+00_JPRB, 0.84717E+00_JPRB, 0.86533E+00_JPRB, 0.87322E+00_JPRB, 0.89687E+00_JPRB ,&
&  0.90762E+00_JPRB, 0.91582E+00_JPRB, 0.92586E+00_JPRB, 0.94692E+00_JPRB, 0.95020E+00_JPRB, 0.99253E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.77215E+00_JPRB, 0.72367E+00_JPRB, 0.66152E+00_JPRB, 0.61175E+00_JPRB, 0.57339E+00_JPRB, 0.53352E+00_JPRB, 0.51119E+00_JPRB ,&
&  0.51218E+00_JPRB, 0.52441E+00_JPRB, 0.54568E+00_JPRB, 0.57716E+00_JPRB, 0.63044E+00_JPRB, 0.70113E+00_JPRB, 0.75566E+00_JPRB ,&
&  0.81622E+00_JPRB, 0.88784E+00_JPRB, 0.95032E+00_JPRB, 0.10125E+01_JPRB, 0.10850E+01_JPRB, 0.11507E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.11304E+01_JPRB, 0.95930E+00_JPRB, 0.10395E+01_JPRB, 0.11052E+01_JPRB, 0.10307E+01_JPRB, 0.83119E+00_JPRB ,&
&  0.62396E+00_JPRB, 0.57460E+00_JPRB, 0.61045E+00_JPRB, 0.75712E+00_JPRB, 0.78699E+00_JPRB, 0.76452E+00_JPRB, 0.72778E+00_JPRB ,&
&  0.69537E+00_JPRB, 0.65882E+00_JPRB, 0.64094E+00_JPRB, 0.65673E+00_JPRB, 0.69037E+00_JPRB, 0.70519E+00_JPRB, 0.66976E+00_JPRB ,&
&  0.64115E+00_JPRB, 0.62150E+00_JPRB, 0.58678E+00_JPRB, 0.53409E+00_JPRB, 0.50452E+00_JPRB, 0.44381E+00_JPRB, 0.47695E+00_JPRB ,&
&  0.46281E+00_JPRB, 0.39945E+00_JPRB, 0.42620E+00_JPRB, 0.51651E+00_JPRB, 0.58067E+00_JPRB, 0.66312E+00_JPRB, 0.83222E+00_JPRB ,&
&  0.97413E+00_JPRB, 0.10190E+01_JPRB, 0.94989E+00_JPRB, 0.77221E+00_JPRB, 0.63287E+00_JPRB, 0.57782E+00_JPRB, 0.57531E+00_JPRB ,&
&  0.57414E+00_JPRB, 0.56798E+00_JPRB, 0.57225E+00_JPRB, 0.58364E+00_JPRB, 0.59304E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.58591E+00_JPRB, 0.60112E+00_JPRB, 0.61657E+00_JPRB, 0.63225E+00_JPRB, 0.64573E+00_JPRB, 0.65808E+00_JPRB, 0.67130E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.70035E+00_JPRB, 0.71719E+00_JPRB, 0.73504E+00_JPRB, 0.74837E+00_JPRB, 0.75215E+00_JPRB, 0.75970E+00_JPRB ,&
&  0.76373E+00_JPRB, 0.76168E+00_JPRB, 0.75219E+00_JPRB, 0.76919E+00_JPRB, 0.78682E+00_JPRB, 0.79905E+00_JPRB, 0.81919E+00_JPRB ,&
&  0.84278E+00_JPRB, 0.83340E+00_JPRB, 0.83080E+00_JPRB, 0.85027E+00_JPRB, 0.86052E+00_JPRB, 0.81462E+00_JPRB, 0.88086E+00_JPRB ,&
&  0.63231E+00_JPRB, 0.74741E+00_JPRB, 0.68715E+00_JPRB, 0.59809E+00_JPRB, 0.57444E+00_JPRB, 0.51101E+00_JPRB, 0.47690E+00_JPRB ,&
&  0.50183E+00_JPRB, 0.49540E+00_JPRB, 0.49348E+00_JPRB, 0.50398E+00_JPRB, 0.54776E+00_JPRB, 0.61787E+00_JPRB, 0.67007E+00_JPRB ,&
&  0.73757E+00_JPRB, 0.80103E+00_JPRB, 0.87511E+00_JPRB, 0.95926E+00_JPRB, 0.10571E+01_JPRB, 0.10972E+01_JPRB, 0.10398E+01_JPRB ,&
&  0.95676E+00_JPRB, 0.89478E+00_JPRB, 0.68811E+00_JPRB, 0.74623E+00_JPRB, 0.84866E+00_JPRB, 0.83687E+00_JPRB, 0.73018E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.61271E+00_JPRB, 0.72995E+00_JPRB, 0.84897E+00_JPRB, 0.90419E+00_JPRB, 0.91474E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.82511E+00_JPRB, 0.76511E+00_JPRB, 0.72621E+00_JPRB, 0.74399E+00_JPRB, 0.74603E+00_JPRB, 0.71742E+00_JPRB, 0.63524E+00_JPRB ,&
&  0.59293E+00_JPRB, 0.59206E+00_JPRB, 0.59445E+00_JPRB, 0.59002E+00_JPRB, 0.56681E+00_JPRB, 0.70736E+00_JPRB, 0.51233E+00_JPRB ,&
&  0.53941E+00_JPRB, 0.47673E+00_JPRB, 0.36802E+00_JPRB, 0.46965E+00_JPRB, 0.51286E+00_JPRB, 0.54131E+00_JPRB, 0.59251E+00_JPRB ,&
&  0.62346E+00_JPRB, 0.58214E+00_JPRB, 0.57306E+00_JPRB, 0.54987E+00_JPRB, 0.54869E+00_JPRB, 0.54363E+00_JPRB, 0.55532E+00_JPRB ,&
&  0.56174E+00_JPRB, 0.56001E+00_JPRB, 0.56484E+00_JPRB, 0.57228E+00_JPRB, 0.57665E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.55714E+00_JPRB, 0.56329E+00_JPRB, 0.56764E+00_JPRB, 0.58238E+00_JPRB, 0.60480E+00_JPRB, 0.60695E+00_JPRB, 0.61790E+00_JPRB ,&
&  0.62327E+00_JPRB, 0.63756E+00_JPRB, 0.65304E+00_JPRB, 0.67194E+00_JPRB, 0.68083E+00_JPRB, 0.68325E+00_JPRB, 0.69801E+00_JPRB ,&
&  0.70927E+00_JPRB, 0.70236E+00_JPRB, 0.70916E+00_JPRB, 0.72426E+00_JPRB, 0.72769E+00_JPRB, 0.75233E+00_JPRB, 0.75051E+00_JPRB ,&
&  0.77783E+00_JPRB, 0.78478E+00_JPRB, 0.80924E+00_JPRB, 0.87785E+00_JPRB, 0.96006E+00_JPRB, 0.11366E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.83285E+00_JPRB, 0.78366E+00_JPRB, 0.66538E+00_JPRB, 0.66072E+00_JPRB, 0.67971E+00_JPRB, 0.67395E+00_JPRB ,&
&  0.65106E+00_JPRB, 0.61229E+00_JPRB, 0.58944E+00_JPRB, 0.57217E+00_JPRB, 0.57589E+00_JPRB, 0.60830E+00_JPRB, 0.62749E+00_JPRB ,&
&  0.60978E+00_JPRB, 0.66755E+00_JPRB, 0.74278E+00_JPRB, 0.85058E+00_JPRB, 0.92618E+00_JPRB, 0.11067E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.97271E+00_JPRB, 0.82691E+00_JPRB, 0.83934E+00_JPRB, 0.89679E+00_JPRB, 0.85565E+00_JPRB, 0.74509E+00_JPRB ,&
&  0.70153E+00_JPRB, 0.64448E+00_JPRB, 0.68773E+00_JPRB, 0.94175E+00_JPRB, 0.10419E+01_JPRB, 0.10838E+01_JPRB, 0.10383E+01_JPRB ,&
&  0.97298E+00_JPRB, 0.90926E+00_JPRB, 0.83529E+00_JPRB, 0.78585E+00_JPRB, 0.76194E+00_JPRB, 0.71330E+00_JPRB, 0.63249E+00_JPRB ,&
&  0.56343E+00_JPRB, 0.56196E+00_JPRB, 0.58235E+00_JPRB, 0.58976E+00_JPRB, 0.64823E+00_JPRB, 0.60556E+00_JPRB, 0.54218E+00_JPRB ,&
&  0.43971E+00_JPRB, 0.46387E+00_JPRB, 0.38756E+00_JPRB, 0.45307E+00_JPRB, 0.42519E+00_JPRB, 0.41079E+00_JPRB, 0.42291E+00_JPRB ,&
&  0.43376E+00_JPRB, 0.44315E+00_JPRB, 0.45723E+00_JPRB, 0.46936E+00_JPRB, 0.48425E+00_JPRB, 0.50845E+00_JPRB, 0.53470E+00_JPRB ,&
&  0.55501E+00_JPRB, 0.54215E+00_JPRB, 0.54174E+00_JPRB, 0.54508E+00_JPRB, 0.55719E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.51781E+00_JPRB, 0.51392E+00_JPRB, 0.50619E+00_JPRB, 0.50623E+00_JPRB, 0.51028E+00_JPRB, 0.52240E+00_JPRB, 0.52976E+00_JPRB ,&
&  0.52742E+00_JPRB, 0.53132E+00_JPRB, 0.51482E+00_JPRB, 0.51476E+00_JPRB, 0.52778E+00_JPRB, 0.53483E+00_JPRB, 0.54137E+00_JPRB ,&
&  0.57011E+00_JPRB, 0.58203E+00_JPRB, 0.58074E+00_JPRB, 0.58411E+00_JPRB, 0.58005E+00_JPRB, 0.62077E+00_JPRB, 0.62395E+00_JPRB ,&
&  0.60985E+00_JPRB, 0.63301E+00_JPRB, 0.73308E+00_JPRB, 0.98325E+00_JPRB, 0.11929E+01_JPRB, 0.11356E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.92944E+00_JPRB, 0.10431E+01_JPRB, 0.93699E+00_JPRB, 0.96281E+00_JPRB, 0.99424E+00_JPRB, 0.98367E+00_JPRB ,&
&  0.99377E+00_JPRB, 0.97380E+00_JPRB, 0.96840E+00_JPRB, 0.96837E+00_JPRB, 0.95755E+00_JPRB, 0.94508E+00_JPRB, 0.92316E+00_JPRB ,&
&  0.86390E+00_JPRB, 0.85761E+00_JPRB, 0.90404E+00_JPRB, 0.10073E+01_JPRB, 0.11306E+01_JPRB, 0.12640E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.13316E+01_JPRB, 0.13546E+01_JPRB, 0.12551E+01_JPRB, 0.12520E+01_JPRB, 0.12938E+01_JPRB, 0.11800E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.96875E+00_JPRB, 0.74736E+00_JPRB, 0.71874E+00_JPRB, 0.10283E+01_JPRB, 0.11304E+01_JPRB, 0.11410E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.11166E+01_JPRB, 0.10805E+01_JPRB, 0.97630E+00_JPRB, 0.86804E+00_JPRB, 0.77241E+00_JPRB, 0.69109E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.53079E+00_JPRB, 0.54186E+00_JPRB, 0.54399E+00_JPRB, 0.52516E+00_JPRB, 0.55680E+00_JPRB, 0.47588E+00_JPRB ,&
&  0.40996E+00_JPRB, 0.38640E+00_JPRB, 0.42614E+00_JPRB, 0.43059E+00_JPRB, 0.34469E+00_JPRB, 0.36346E+00_JPRB, 0.35372E+00_JPRB ,&
&  0.36802E+00_JPRB, 0.38794E+00_JPRB, 0.39552E+00_JPRB, 0.40572E+00_JPRB, 0.42454E+00_JPRB, 0.44129E+00_JPRB, 0.46520E+00_JPRB ,&
&  0.48072E+00_JPRB, 0.49319E+00_JPRB, 0.50647E+00_JPRB, 0.51998E+00_JPRB, 0.51601E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.56660E+00_JPRB, 0.59300E+00_JPRB, 0.60797E+00_JPRB, 0.62649E+00_JPRB, 0.63329E+00_JPRB, 0.62634E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.55413E+00_JPRB, 0.54436E+00_JPRB, 0.52865E+00_JPRB, 0.54341E+00_JPRB, 0.57387E+00_JPRB, 0.60059E+00_JPRB ,&
&  0.66663E+00_JPRB, 0.63120E+00_JPRB, 0.63654E+00_JPRB, 0.66744E+00_JPRB, 0.67854E+00_JPRB, 0.72019E+00_JPRB, 0.83908E+00_JPRB ,&
&  0.10306E+01_JPRB, 0.12451E+01_JPRB, 0.15305E+01_JPRB, 0.19465E+01_JPRB, 0.19807E+01_JPRB, 0.14508E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.11156E+01_JPRB, 0.11759E+01_JPRB, 0.12194E+01_JPRB, 0.12703E+01_JPRB, 0.12609E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.13215E+01_JPRB, 0.13358E+01_JPRB, 0.13377E+01_JPRB, 0.13488E+01_JPRB, 0.13707E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.14337E+01_JPRB, 0.14473E+01_JPRB, 0.14163E+01_JPRB, 0.14850E+01_JPRB, 0.15491E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.16387E+01_JPRB, 0.16557E+01_JPRB, 0.17485E+01_JPRB, 0.17284E+01_JPRB, 0.16700E+01_JPRB, 0.16053E+01_JPRB, 0.14778E+01_JPRB ,&
&  0.13269E+01_JPRB, 0.10585E+01_JPRB, 0.94749E+00_JPRB, 0.13966E+01_JPRB, 0.13958E+01_JPRB, 0.12057E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.12119E+01_JPRB, 0.11470E+01_JPRB, 0.99120E+00_JPRB, 0.85896E+00_JPRB, 0.75439E+00_JPRB, 0.64584E+00_JPRB ,&
&  0.62857E+00_JPRB, 0.55680E+00_JPRB, 0.51912E+00_JPRB, 0.51229E+00_JPRB, 0.48638E+00_JPRB, 0.47994E+00_JPRB, 0.50726E+00_JPRB ,&
&  0.47884E+00_JPRB, 0.42516E+00_JPRB, 0.43986E+00_JPRB, 0.48880E+00_JPRB, 0.49269E+00_JPRB, 0.37250E+00_JPRB, 0.35497E+00_JPRB ,&
&  0.36978E+00_JPRB, 0.38380E+00_JPRB, 0.39728E+00_JPRB, 0.41132E+00_JPRB, 0.42276E+00_JPRB, 0.44317E+00_JPRB, 0.46328E+00_JPRB ,&
&  0.48598E+00_JPRB, 0.51806E+00_JPRB, 0.53273E+00_JPRB, 0.54802E+00_JPRB, 0.56325E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.79644E+00_JPRB, 0.83406E+00_JPRB, 0.86877E+00_JPRB, 0.88834E+00_JPRB, 0.89026E+00_JPRB, 0.89821E+00_JPRB, 0.89920E+00_JPRB ,&
&  0.90897E+00_JPRB, 0.91009E+00_JPRB, 0.93913E+00_JPRB, 0.10019E+01_JPRB, 0.11255E+01_JPRB, 0.11920E+01_JPRB, 0.12775E+01_JPRB ,&
&  0.12843E+01_JPRB, 0.13104E+01_JPRB, 0.13406E+01_JPRB, 0.13809E+01_JPRB, 0.14580E+01_JPRB, 0.15717E+01_JPRB, 0.18005E+01_JPRB ,&
&  0.22211E+01_JPRB, 0.24753E+01_JPRB, 0.23850E+01_JPRB, 0.19728E+01_JPRB, 0.15152E+01_JPRB, 0.12343E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13953E+01_JPRB, 0.13819E+01_JPRB, 0.13829E+01_JPRB, 0.13909E+01_JPRB, 0.13866E+01_JPRB, 0.13917E+01_JPRB, 0.13896E+01_JPRB ,&
&  0.13979E+01_JPRB, 0.14118E+01_JPRB, 0.14133E+01_JPRB, 0.14127E+01_JPRB, 0.14415E+01_JPRB, 0.14984E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.16991E+01_JPRB, 0.17843E+01_JPRB, 0.18089E+01_JPRB, 0.18027E+01_JPRB, 0.17888E+01_JPRB, 0.17919E+01_JPRB, 0.18270E+01_JPRB ,&
&  0.18727E+01_JPRB, 0.18684E+01_JPRB, 0.19758E+01_JPRB, 0.20391E+01_JPRB, 0.19448E+01_JPRB, 0.18311E+01_JPRB, 0.17397E+01_JPRB ,&
&  0.16722E+01_JPRB, 0.15907E+01_JPRB, 0.13629E+01_JPRB, 0.17705E+01_JPRB, 0.18143E+01_JPRB, 0.16010E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.13562E+01_JPRB, 0.13191E+01_JPRB, 0.12161E+01_JPRB, 0.10377E+01_JPRB, 0.90442E+00_JPRB, 0.84943E+00_JPRB, 0.67954E+00_JPRB ,&
&  0.70272E+00_JPRB, 0.61929E+00_JPRB, 0.54905E+00_JPRB, 0.52010E+00_JPRB, 0.50140E+00_JPRB, 0.48569E+00_JPRB, 0.50347E+00_JPRB ,&
&  0.47015E+00_JPRB, 0.51343E+00_JPRB, 0.60942E+00_JPRB, 0.96102E+00_JPRB, 0.17284E+01_JPRB, 0.44589E+00_JPRB, 0.37256E+00_JPRB ,&
&  0.39337E+00_JPRB, 0.42239E+00_JPRB, 0.45514E+00_JPRB, 0.48034E+00_JPRB, 0.49994E+00_JPRB, 0.53311E+00_JPRB, 0.58551E+00_JPRB ,&
&  0.62167E+00_JPRB, 0.65000E+00_JPRB, 0.68360E+00_JPRB, 0.73852E+00_JPRB, 0.76549E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.94265E+00_JPRB, 0.97829E+00_JPRB, 0.10007E+01_JPRB, 0.10422E+01_JPRB, 0.10835E+01_JPRB, 0.11281E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.12766E+01_JPRB, 0.13700E+01_JPRB, 0.14587E+01_JPRB, 0.15225E+01_JPRB, 0.15813E+01_JPRB, 0.16255E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.16923E+01_JPRB, 0.17595E+01_JPRB, 0.18355E+01_JPRB, 0.19365E+01_JPRB, 0.20553E+01_JPRB, 0.21930E+01_JPRB, 0.23346E+01_JPRB ,&
&  0.24184E+01_JPRB, 0.22952E+01_JPRB, 0.19682E+01_JPRB, 0.13645E+01_JPRB, 0.11034E+01_JPRB, 0.12243E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.13843E+01_JPRB, 0.13775E+01_JPRB, 0.14131E+01_JPRB, 0.14372E+01_JPRB, 0.14536E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.14691E+01_JPRB, 0.14895E+01_JPRB, 0.14971E+01_JPRB, 0.15001E+01_JPRB, 0.15336E+01_JPRB, 0.16241E+01_JPRB, 0.17690E+01_JPRB ,&
&  0.19450E+01_JPRB, 0.20581E+01_JPRB, 0.21328E+01_JPRB, 0.21768E+01_JPRB, 0.20949E+01_JPRB, 0.20222E+01_JPRB, 0.19901E+01_JPRB ,&
&  0.20174E+01_JPRB, 0.20585E+01_JPRB, 0.21525E+01_JPRB, 0.22259E+01_JPRB, 0.22131E+01_JPRB, 0.20168E+01_JPRB, 0.19195E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.18290E+01_JPRB, 0.17531E+01_JPRB, 0.20462E+01_JPRB, 0.13424E+01_JPRB, 0.13672E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.15525E+01_JPRB, 0.13395E+01_JPRB, 0.10795E+01_JPRB, 0.94958E+00_JPRB, 0.70471E+00_JPRB, 0.68735E+00_JPRB ,&
&  0.76888E+00_JPRB, 0.72511E+00_JPRB, 0.61349E+00_JPRB, 0.55216E+00_JPRB, 0.56631E+00_JPRB, 0.51977E+00_JPRB, 0.52056E+00_JPRB ,&
&  0.50565E+00_JPRB, 0.53362E+00_JPRB, 0.59794E+00_JPRB, 0.74910E+00_JPRB, 0.91138E+00_JPRB, 0.45968E+00_JPRB, 0.40081E+00_JPRB ,&
&  0.41928E+00_JPRB, 0.46114E+00_JPRB, 0.51110E+00_JPRB, 0.55583E+00_JPRB, 0.59795E+00_JPRB, 0.64413E+00_JPRB, 0.69477E+00_JPRB ,&
&  0.75188E+00_JPRB, 0.81965E+00_JPRB, 0.86444E+00_JPRB, 0.89428E+00_JPRB, 0.91441E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.93294E+00_JPRB, 0.97238E+00_JPRB, 0.10233E+01_JPRB, 0.10937E+01_JPRB, 0.11669E+01_JPRB, 0.11896E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.12176E+01_JPRB, 0.12791E+01_JPRB, 0.13556E+01_JPRB, 0.14123E+01_JPRB, 0.14364E+01_JPRB, 0.14863E+01_JPRB, 0.15467E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.16740E+01_JPRB, 0.17591E+01_JPRB, 0.18624E+01_JPRB, 0.20005E+01_JPRB, 0.21716E+01_JPRB, 0.22487E+01_JPRB ,&
&  0.17633E+01_JPRB, 0.12498E+01_JPRB, 0.16365E+01_JPRB, 0.15727E+01_JPRB, 0.13382E+01_JPRB, 0.11931E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.13923E+01_JPRB, 0.14065E+01_JPRB, 0.14096E+01_JPRB, 0.13938E+01_JPRB, 0.14282E+01_JPRB, 0.14626E+01_JPRB ,&
&  0.14921E+01_JPRB, 0.15161E+01_JPRB, 0.15348E+01_JPRB, 0.15495E+01_JPRB, 0.15986E+01_JPRB, 0.17053E+01_JPRB, 0.18837E+01_JPRB ,&
&  0.20658E+01_JPRB, 0.21841E+01_JPRB, 0.22896E+01_JPRB, 0.23176E+01_JPRB, 0.22328E+01_JPRB, 0.21642E+01_JPRB, 0.21548E+01_JPRB ,&
&  0.22025E+01_JPRB, 0.22560E+01_JPRB, 0.23175E+01_JPRB, 0.23502E+01_JPRB, 0.23929E+01_JPRB, 0.21563E+01_JPRB, 0.21157E+01_JPRB ,&
&  0.21209E+01_JPRB, 0.19903E+01_JPRB, 0.19728E+01_JPRB, 0.20715E+01_JPRB, 0.14365E+01_JPRB, 0.13888E+01_JPRB, 0.13028E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.16441E+01_JPRB, 0.15432E+01_JPRB, 0.13124E+01_JPRB, 0.10782E+01_JPRB, 0.75156E+00_JPRB, 0.73444E+00_JPRB ,&
&  0.80526E+00_JPRB, 0.84956E+00_JPRB, 0.69607E+00_JPRB, 0.63546E+00_JPRB, 0.52286E+00_JPRB, 0.47161E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.53137E+00_JPRB, 0.53038E+00_JPRB, 0.56433E+00_JPRB, 0.61919E+00_JPRB, 0.63689E+00_JPRB, 0.47637E+00_JPRB, 0.42146E+00_JPRB ,&
&  0.43895E+00_JPRB, 0.47957E+00_JPRB, 0.53739E+00_JPRB, 0.60318E+00_JPRB, 0.63787E+00_JPRB, 0.67411E+00_JPRB, 0.71754E+00_JPRB ,&
&  0.78354E+00_JPRB, 0.83536E+00_JPRB, 0.87121E+00_JPRB, 0.88042E+00_JPRB, 0.90812E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.85268E+00_JPRB, 0.90197E+00_JPRB, 0.97150E+00_JPRB, 0.10773E+01_JPRB, 0.12266E+01_JPRB, 0.13696E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.10142E+01_JPRB, 0.10565E+01_JPRB, 0.10978E+01_JPRB, 0.11376E+01_JPRB, 0.12061E+01_JPRB, 0.12994E+01_JPRB ,&
&  0.14015E+01_JPRB, 0.14968E+01_JPRB, 0.15899E+01_JPRB, 0.16690E+01_JPRB, 0.17172E+01_JPRB, 0.18698E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11288E+01_JPRB, 0.15915E+01_JPRB, 0.13543E+01_JPRB, 0.13369E+01_JPRB, 0.13120E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.12732E+01_JPRB, 0.13347E+01_JPRB, 0.13212E+01_JPRB, 0.13123E+01_JPRB, 0.12858E+01_JPRB, 0.13497E+01_JPRB, 0.14175E+01_JPRB ,&
&  0.14572E+01_JPRB, 0.14821E+01_JPRB, 0.15064E+01_JPRB, 0.15257E+01_JPRB, 0.15699E+01_JPRB, 0.16732E+01_JPRB, 0.18810E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.22206E+01_JPRB, 0.23875E+01_JPRB, 0.24109E+01_JPRB, 0.23483E+01_JPRB, 0.23537E+01_JPRB, 0.24233E+01_JPRB ,&
&  0.24966E+01_JPRB, 0.26128E+01_JPRB, 0.26399E+01_JPRB, 0.25664E+01_JPRB, 0.23454E+01_JPRB, 0.24817E+01_JPRB, 0.22050E+01_JPRB ,&
&  0.22382E+01_JPRB, 0.21062E+01_JPRB, 0.22771E+01_JPRB, 0.17612E+01_JPRB, 0.16392E+01_JPRB, 0.16166E+01_JPRB, 0.15607E+01_JPRB ,&
&  0.15067E+01_JPRB, 0.16509E+01_JPRB, 0.16380E+01_JPRB, 0.14303E+01_JPRB, 0.12041E+01_JPRB, 0.91021E+00_JPRB, 0.93604E+00_JPRB ,&
&  0.92303E+00_JPRB, 0.99612E+00_JPRB, 0.77247E+00_JPRB, 0.60362E+00_JPRB, 0.44649E+00_JPRB, 0.38543E+00_JPRB, 0.42689E+00_JPRB ,&
&  0.51370E+00_JPRB, 0.56554E+00_JPRB, 0.55167E+00_JPRB, 0.54227E+00_JPRB, 0.54679E+00_JPRB, 0.49916E+00_JPRB, 0.45320E+00_JPRB ,&
&  0.47471E+00_JPRB, 0.49817E+00_JPRB, 0.53502E+00_JPRB, 0.59605E+00_JPRB, 0.63490E+00_JPRB, 0.67920E+00_JPRB, 0.68720E+00_JPRB ,&
&  0.72144E+00_JPRB, 0.75350E+00_JPRB, 0.76848E+00_JPRB, 0.78443E+00_JPRB, 0.81048E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.70326E+00_JPRB, 0.71634E+00_JPRB, 0.74870E+00_JPRB, 0.80122E+00_JPRB, 0.84810E+00_JPRB, 0.89123E+00_JPRB, 0.90084E+00_JPRB ,&
&  0.87813E+00_JPRB, 0.85164E+00_JPRB, 0.84839E+00_JPRB, 0.82479E+00_JPRB, 0.81598E+00_JPRB, 0.82419E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.10086E+01_JPRB, 0.11916E+01_JPRB, 0.13720E+01_JPRB, 0.14895E+01_JPRB, 0.15322E+01_JPRB, 0.16225E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.95842E+00_JPRB, 0.16209E+01_JPRB, 0.13208E+01_JPRB, 0.13328E+01_JPRB, 0.13591E+01_JPRB, 0.14131E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.12457E+01_JPRB, 0.12421E+01_JPRB, 0.12476E+01_JPRB, 0.12326E+01_JPRB, 0.12781E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.14116E+01_JPRB, 0.14546E+01_JPRB, 0.14770E+01_JPRB, 0.14813E+01_JPRB, 0.15028E+01_JPRB, 0.15537E+01_JPRB, 0.17340E+01_JPRB ,&
&  0.19187E+01_JPRB, 0.21460E+01_JPRB, 0.23687E+01_JPRB, 0.25775E+01_JPRB, 0.25164E+01_JPRB, 0.25805E+01_JPRB, 0.27665E+01_JPRB ,&
&  0.27847E+01_JPRB, 0.26825E+01_JPRB, 0.28485E+01_JPRB, 0.28419E+01_JPRB, 0.27508E+01_JPRB, 0.28301E+01_JPRB, 0.23625E+01_JPRB ,&
&  0.24199E+01_JPRB, 0.21742E+01_JPRB, 0.23256E+01_JPRB, 0.19012E+01_JPRB, 0.18007E+01_JPRB, 0.18438E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.16829E+01_JPRB, 0.16033E+01_JPRB, 0.14691E+01_JPRB, 0.13842E+01_JPRB, 0.12238E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.11641E+01_JPRB, 0.10799E+01_JPRB, 0.89158E+00_JPRB, 0.52602E+00_JPRB, 0.40837E+00_JPRB, 0.34840E+00_JPRB, 0.37999E+00_JPRB ,&
&  0.46948E+00_JPRB, 0.53773E+00_JPRB, 0.59542E+00_JPRB, 0.57726E+00_JPRB, 0.54271E+00_JPRB, 0.51804E+00_JPRB, 0.49674E+00_JPRB ,&
&  0.49793E+00_JPRB, 0.50733E+00_JPRB, 0.51919E+00_JPRB, 0.56816E+00_JPRB, 0.62902E+00_JPRB, 0.66822E+00_JPRB, 0.64581E+00_JPRB ,&
&  0.62384E+00_JPRB, 0.62571E+00_JPRB, 0.65063E+00_JPRB, 0.66930E+00_JPRB, 0.68794E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.61369E+00_JPRB, 0.60441E+00_JPRB, 0.60597E+00_JPRB, 0.62667E+00_JPRB, 0.63312E+00_JPRB, 0.63561E+00_JPRB, 0.67459E+00_JPRB ,&
&  0.68488E+00_JPRB, 0.67339E+00_JPRB, 0.68269E+00_JPRB, 0.68395E+00_JPRB, 0.67557E+00_JPRB, 0.65145E+00_JPRB, 0.64279E+00_JPRB ,&
&  0.66472E+00_JPRB, 0.77031E+00_JPRB, 0.98285E+00_JPRB, 0.12412E+01_JPRB, 0.14109E+01_JPRB, 0.16927E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.10408E+01_JPRB, 0.13778E+01_JPRB, 0.12655E+01_JPRB, 0.13634E+01_JPRB, 0.14441E+01_JPRB, 0.15368E+01_JPRB, 0.13359E+01_JPRB ,&
&  0.12775E+01_JPRB, 0.12747E+01_JPRB, 0.12355E+01_JPRB, 0.12440E+01_JPRB, 0.12565E+01_JPRB, 0.12690E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.13745E+01_JPRB, 0.14190E+01_JPRB, 0.14296E+01_JPRB, 0.14361E+01_JPRB, 0.14610E+01_JPRB, 0.16028E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.19819E+01_JPRB, 0.22665E+01_JPRB, 0.25014E+01_JPRB, 0.26615E+01_JPRB, 0.27653E+01_JPRB, 0.30704E+01_JPRB ,&
&  0.30665E+01_JPRB, 0.29387E+01_JPRB, 0.31124E+01_JPRB, 0.32199E+01_JPRB, 0.32257E+01_JPRB, 0.32131E+01_JPRB, 0.27642E+01_JPRB ,&
&  0.25666E+01_JPRB, 0.24323E+01_JPRB, 0.23578E+01_JPRB, 0.20389E+01_JPRB, 0.19237E+01_JPRB, 0.19750E+01_JPRB, 0.19992E+01_JPRB ,&
&  0.18695E+01_JPRB, 0.18226E+01_JPRB, 0.17890E+01_JPRB, 0.15942E+01_JPRB, 0.14983E+01_JPRB, 0.13978E+01_JPRB, 0.15039E+01_JPRB ,&
&  0.14424E+01_JPRB, 0.12254E+01_JPRB, 0.75146E+00_JPRB, 0.41742E+00_JPRB, 0.33439E+00_JPRB, 0.29772E+00_JPRB, 0.34259E+00_JPRB ,&
&  0.43672E+00_JPRB, 0.50696E+00_JPRB, 0.55708E+00_JPRB, 0.70881E+00_JPRB, 0.68044E+00_JPRB, 0.62610E+00_JPRB, 0.57952E+00_JPRB ,&
&  0.55776E+00_JPRB, 0.53323E+00_JPRB, 0.52242E+00_JPRB, 0.53277E+00_JPRB, 0.58671E+00_JPRB, 0.64329E+00_JPRB, 0.66863E+00_JPRB ,&
&  0.64077E+00_JPRB, 0.60484E+00_JPRB, 0.58842E+00_JPRB, 0.58630E+00_JPRB, 0.59575E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.55754E+00_JPRB, 0.55304E+00_JPRB, 0.55525E+00_JPRB, 0.56561E+00_JPRB, 0.57332E+00_JPRB, 0.55476E+00_JPRB, 0.52436E+00_JPRB ,&
&  0.53146E+00_JPRB, 0.51758E+00_JPRB, 0.53564E+00_JPRB, 0.55117E+00_JPRB, 0.57093E+00_JPRB, 0.57104E+00_JPRB, 0.56877E+00_JPRB ,&
&  0.55992E+00_JPRB, 0.58713E+00_JPRB, 0.73987E+00_JPRB, 0.99753E+00_JPRB, 0.12150E+01_JPRB, 0.13466E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.12880E+01_JPRB, 0.13748E+01_JPRB, 0.13841E+01_JPRB, 0.16691E+01_JPRB, 0.18991E+01_JPRB, 0.22593E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.13646E+01_JPRB, 0.13328E+01_JPRB, 0.13107E+01_JPRB, 0.13138E+01_JPRB, 0.13485E+01_JPRB, 0.13307E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.12832E+01_JPRB, 0.12906E+01_JPRB, 0.13170E+01_JPRB, 0.13381E+01_JPRB, 0.13535E+01_JPRB, 0.13810E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.17154E+01_JPRB, 0.19498E+01_JPRB, 0.22094E+01_JPRB, 0.25269E+01_JPRB, 0.27312E+01_JPRB, 0.29614E+01_JPRB, 0.33952E+01_JPRB ,&
&  0.35889E+01_JPRB, 0.36968E+01_JPRB, 0.35941E+01_JPRB, 0.34696E+01_JPRB, 0.34176E+01_JPRB, 0.36502E+01_JPRB, 0.32734E+01_JPRB ,&
&  0.28542E+01_JPRB, 0.25148E+01_JPRB, 0.22337E+01_JPRB, 0.21155E+01_JPRB, 0.20689E+01_JPRB, 0.21711E+01_JPRB, 0.19433E+01_JPRB ,&
&  0.16618E+01_JPRB, 0.15108E+01_JPRB, 0.15069E+01_JPRB, 0.15744E+01_JPRB, 0.16464E+01_JPRB, 0.16463E+01_JPRB, 0.18905E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.71977E+00_JPRB, 0.43752E+00_JPRB, 0.41503E+00_JPRB, 0.28728E+00_JPRB, 0.24234E+00_JPRB, 0.36539E+00_JPRB ,&
&  0.47957E+00_JPRB, 0.59571E+00_JPRB, 0.62407E+00_JPRB, 0.68961E+00_JPRB, 0.79696E+00_JPRB, 0.76891E+00_JPRB, 0.70539E+00_JPRB ,&
&  0.63585E+00_JPRB, 0.59334E+00_JPRB, 0.57543E+00_JPRB, 0.56854E+00_JPRB, 0.60638E+00_JPRB, 0.65795E+00_JPRB, 0.69030E+00_JPRB ,&
&  0.67978E+00_JPRB, 0.64401E+00_JPRB, 0.58990E+00_JPRB, 0.54748E+00_JPRB, 0.54126E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.55418E+00_JPRB, 0.56924E+00_JPRB, 0.58469E+00_JPRB, 0.57265E+00_JPRB, 0.55016E+00_JPRB, 0.54493E+00_JPRB, 0.50981E+00_JPRB ,&
&  0.48612E+00_JPRB, 0.47065E+00_JPRB, 0.46336E+00_JPRB, 0.47360E+00_JPRB, 0.49694E+00_JPRB, 0.51386E+00_JPRB, 0.50960E+00_JPRB ,&
&  0.49822E+00_JPRB, 0.52520E+00_JPRB, 0.61835E+00_JPRB, 0.85544E+00_JPRB, 0.11364E+01_JPRB, 0.14180E+01_JPRB, 0.14724E+01_JPRB ,&
&  0.13484E+01_JPRB, 0.17863E+01_JPRB, 0.24658E+01_JPRB, 0.28171E+01_JPRB, 0.26343E+01_JPRB, 0.27774E+01_JPRB, 0.19825E+01_JPRB ,&
&  0.16550E+01_JPRB, 0.16191E+01_JPRB, 0.15865E+01_JPRB, 0.15659E+01_JPRB, 0.15149E+01_JPRB, 0.14352E+01_JPRB, 0.13941E+01_JPRB ,&
&  0.13563E+01_JPRB, 0.12780E+01_JPRB, 0.12500E+01_JPRB, 0.12480E+01_JPRB, 0.12704E+01_JPRB, 0.13062E+01_JPRB, 0.14265E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.18816E+01_JPRB, 0.21430E+01_JPRB, 0.28018E+01_JPRB, 0.24383E+01_JPRB, 0.32242E+01_JPRB, 0.36683E+01_JPRB ,&
&  0.38543E+01_JPRB, 0.39416E+01_JPRB, 0.36195E+01_JPRB, 0.39597E+01_JPRB, 0.38789E+01_JPRB, 0.39571E+01_JPRB, 0.36470E+01_JPRB ,&
&  0.31938E+01_JPRB, 0.26253E+01_JPRB, 0.23759E+01_JPRB, 0.22365E+01_JPRB, 0.21502E+01_JPRB, 0.22491E+01_JPRB, 0.15909E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.16592E+01_JPRB, 0.16297E+01_JPRB, 0.14980E+01_JPRB, 0.17176E+01_JPRB, 0.18477E+01_JPRB, 0.14271E+01_JPRB ,&
&  0.66745E+00_JPRB, 0.50756E+00_JPRB, 0.43652E+00_JPRB, 0.38060E+00_JPRB, 0.32523E+00_JPRB, 0.31730E+00_JPRB, 0.68393E+00_JPRB ,&
&  0.56462E+00_JPRB, 0.74758E+00_JPRB, 0.77576E+00_JPRB, 0.84113E+00_JPRB, 0.10389E+01_JPRB, 0.10691E+01_JPRB, 0.14695E+01_JPRB ,&
&  0.13061E+01_JPRB, 0.10915E+01_JPRB, 0.92778E+00_JPRB, 0.81118E+00_JPRB, 0.75902E+00_JPRB, 0.74651E+00_JPRB, 0.74440E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.69453E+00_JPRB, 0.65540E+00_JPRB, 0.59850E+00_JPRB, 0.55086E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.59332E+00_JPRB, 0.59446E+00_JPRB, 0.59712E+00_JPRB, 0.59281E+00_JPRB, 0.57884E+00_JPRB, 0.57617E+00_JPRB, 0.55315E+00_JPRB ,&
&  0.51667E+00_JPRB, 0.49133E+00_JPRB, 0.47071E+00_JPRB, 0.47465E+00_JPRB, 0.48481E+00_JPRB, 0.49398E+00_JPRB, 0.48473E+00_JPRB ,&
&  0.47175E+00_JPRB, 0.49607E+00_JPRB, 0.64899E+00_JPRB, 0.76519E+00_JPRB, 0.10892E+01_JPRB, 0.13764E+01_JPRB, 0.15855E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.19623E+01_JPRB, 0.26998E+01_JPRB, 0.30752E+01_JPRB, 0.40610E+01_JPRB, 0.40110E+01_JPRB, 0.34222E+01_JPRB ,&
&  0.25847E+01_JPRB, 0.24598E+01_JPRB, 0.23579E+01_JPRB, 0.21607E+01_JPRB, 0.19103E+01_JPRB, 0.16686E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.13361E+01_JPRB, 0.12463E+01_JPRB, 0.11965E+01_JPRB, 0.12064E+01_JPRB, 0.12384E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.15764E+01_JPRB, 0.18039E+01_JPRB, 0.20348E+01_JPRB, 0.26509E+01_JPRB, 0.26961E+01_JPRB, 0.30099E+01_JPRB, 0.37731E+01_JPRB ,&
&  0.41291E+01_JPRB, 0.43483E+01_JPRB, 0.39342E+01_JPRB, 0.46818E+01_JPRB, 0.42459E+01_JPRB, 0.42650E+01_JPRB, 0.41725E+01_JPRB ,&
&  0.35729E+01_JPRB, 0.31607E+01_JPRB, 0.24414E+01_JPRB, 0.22900E+01_JPRB, 0.22961E+01_JPRB, 0.19061E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.18041E+01_JPRB, 0.16765E+01_JPRB, 0.15780E+01_JPRB, 0.14658E+01_JPRB, 0.17729E+01_JPRB, 0.16454E+01_JPRB, 0.83328E+00_JPRB ,&
&  0.62569E+00_JPRB, 0.59302E+00_JPRB, 0.46749E+00_JPRB, 0.40555E+00_JPRB, 0.35673E+00_JPRB, 0.35099E+00_JPRB, 0.52286E+00_JPRB ,&
&  0.56514E+00_JPRB, 0.80901E+00_JPRB, 0.91324E+00_JPRB, 0.10743E+01_JPRB, 0.12338E+01_JPRB, 0.13272E+01_JPRB, 0.20466E+01_JPRB ,&
&  0.20177E+01_JPRB, 0.17004E+01_JPRB, 0.13595E+01_JPRB, 0.11744E+01_JPRB, 0.10234E+01_JPRB, 0.90464E+00_JPRB, 0.82640E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.72753E+00_JPRB, 0.69682E+00_JPRB, 0.67549E+00_JPRB, 0.62429E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.61670E+00_JPRB, 0.60062E+00_JPRB, 0.59561E+00_JPRB, 0.57671E+00_JPRB, 0.55964E+00_JPRB, 0.55458E+00_JPRB, 0.56651E+00_JPRB ,&
&  0.55073E+00_JPRB, 0.52574E+00_JPRB, 0.50103E+00_JPRB, 0.49188E+00_JPRB, 0.49956E+00_JPRB, 0.50434E+00_JPRB, 0.48905E+00_JPRB ,&
&  0.47913E+00_JPRB, 0.50410E+00_JPRB, 0.63463E+00_JPRB, 0.73078E+00_JPRB, 0.92127E+00_JPRB, 0.11910E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.16343E+01_JPRB, 0.20857E+01_JPRB, 0.27009E+01_JPRB, 0.37166E+01_JPRB, 0.49545E+01_JPRB, 0.55044E+01_JPRB, 0.58582E+01_JPRB ,&
&  0.48327E+01_JPRB, 0.38532E+01_JPRB, 0.32604E+01_JPRB, 0.28380E+01_JPRB, 0.23620E+01_JPRB, 0.19721E+01_JPRB, 0.16664E+01_JPRB ,&
&  0.14711E+01_JPRB, 0.13229E+01_JPRB, 0.12844E+01_JPRB, 0.12111E+01_JPRB, 0.11802E+01_JPRB, 0.11936E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.15122E+01_JPRB, 0.17533E+01_JPRB, 0.20647E+01_JPRB, 0.24658E+01_JPRB, 0.29945E+01_JPRB, 0.36777E+01_JPRB, 0.39339E+01_JPRB ,&
&  0.38140E+01_JPRB, 0.37808E+01_JPRB, 0.44087E+01_JPRB, 0.48332E+01_JPRB, 0.44951E+01_JPRB, 0.47117E+01_JPRB, 0.44610E+01_JPRB ,&
&  0.38228E+01_JPRB, 0.33002E+01_JPRB, 0.26057E+01_JPRB, 0.26069E+01_JPRB, 0.21891E+01_JPRB, 0.17390E+01_JPRB, 0.19034E+01_JPRB ,&
&  0.20240E+01_JPRB, 0.18398E+01_JPRB, 0.15937E+01_JPRB, 0.13846E+01_JPRB, 0.13733E+01_JPRB, 0.10890E+01_JPRB, 0.80139E+00_JPRB ,&
&  0.70209E+00_JPRB, 0.59965E+00_JPRB, 0.51177E+00_JPRB, 0.51560E+00_JPRB, 0.44179E+00_JPRB, 0.44669E+00_JPRB, 0.58099E+00_JPRB ,&
&  0.71980E+00_JPRB, 0.81841E+00_JPRB, 0.10634E+01_JPRB, 0.11944E+01_JPRB, 0.12619E+01_JPRB, 0.12918E+01_JPRB, 0.16840E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.13831E+01_JPRB, 0.13357E+01_JPRB, 0.12054E+01_JPRB, 0.10941E+01_JPRB, 0.99315E+00_JPRB, 0.89189E+00_JPRB ,&
&  0.80742E+00_JPRB, 0.73551E+00_JPRB, 0.68814E+00_JPRB, 0.67197E+00_JPRB, 0.64907E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.61446E+00_JPRB, 0.59604E+00_JPRB, 0.56614E+00_JPRB, 0.52213E+00_JPRB, 0.50470E+00_JPRB, 0.51217E+00_JPRB, 0.53483E+00_JPRB ,&
&  0.54305E+00_JPRB, 0.54528E+00_JPRB, 0.53423E+00_JPRB, 0.52062E+00_JPRB, 0.52160E+00_JPRB, 0.51557E+00_JPRB, 0.49784E+00_JPRB ,&
&  0.49046E+00_JPRB, 0.52466E+00_JPRB, 0.45924E+00_JPRB, 0.50688E+00_JPRB, 0.69206E+00_JPRB, 0.10596E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.20669E+01_JPRB, 0.25713E+01_JPRB, 0.37674E+01_JPRB, 0.66778E+01_JPRB, 0.68469E+01_JPRB, 0.65778E+01_JPRB ,&
&  0.60739E+01_JPRB, 0.49409E+01_JPRB, 0.39027E+01_JPRB, 0.30877E+01_JPRB, 0.24790E+01_JPRB, 0.20742E+01_JPRB, 0.17698E+01_JPRB ,&
&  0.15704E+01_JPRB, 0.13984E+01_JPRB, 0.12815E+01_JPRB, 0.12543E+01_JPRB, 0.12194E+01_JPRB, 0.11990E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.14500E+01_JPRB, 0.16831E+01_JPRB, 0.21340E+01_JPRB, 0.26111E+01_JPRB, 0.28469E+01_JPRB, 0.33584E+01_JPRB, 0.44544E+01_JPRB ,&
&  0.41090E+01_JPRB, 0.42272E+01_JPRB, 0.41419E+01_JPRB, 0.50496E+01_JPRB, 0.49073E+01_JPRB, 0.47385E+01_JPRB, 0.47147E+01_JPRB ,&
&  0.34308E+01_JPRB, 0.27649E+01_JPRB, 0.25808E+01_JPRB, 0.25532E+01_JPRB, 0.20058E+01_JPRB, 0.20790E+01_JPRB, 0.27313E+01_JPRB ,&
&  0.21429E+01_JPRB, 0.21006E+01_JPRB, 0.21088E+01_JPRB, 0.21627E+01_JPRB, 0.13600E+01_JPRB, 0.95860E+00_JPRB, 0.13648E+01_JPRB ,&
&  0.13032E+01_JPRB, 0.79460E+00_JPRB, 0.61074E+00_JPRB, 0.72326E+00_JPRB, 0.57616E+00_JPRB, 0.62117E+00_JPRB, 0.79246E+00_JPRB ,&
&  0.79889E+00_JPRB, 0.84944E+00_JPRB, 0.12276E+01_JPRB, 0.13129E+01_JPRB, 0.13216E+01_JPRB, 0.12158E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.12564E+01_JPRB, 0.12886E+01_JPRB, 0.11639E+01_JPRB, 0.10623E+01_JPRB, 0.98699E+00_JPRB, 0.90590E+00_JPRB ,&
&  0.84018E+00_JPRB, 0.76445E+00_JPRB, 0.69657E+00_JPRB, 0.64806E+00_JPRB, 0.62859E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.60471E+00_JPRB, 0.58590E+00_JPRB, 0.56246E+00_JPRB, 0.53276E+00_JPRB, 0.50159E+00_JPRB, 0.49106E+00_JPRB, 0.49613E+00_JPRB ,&
&  0.50519E+00_JPRB, 0.51981E+00_JPRB, 0.52792E+00_JPRB, 0.53110E+00_JPRB, 0.53158E+00_JPRB, 0.52691E+00_JPRB, 0.50270E+00_JPRB ,&
&  0.49480E+00_JPRB, 0.48328E+00_JPRB, 0.41315E+00_JPRB, 0.42117E+00_JPRB, 0.62108E+00_JPRB, 0.85700E+00_JPRB, 0.96014E+00_JPRB ,&
&  0.13429E+01_JPRB, 0.18901E+01_JPRB, 0.25618E+01_JPRB, 0.38522E+01_JPRB, 0.52035E+01_JPRB, 0.60251E+01_JPRB, 0.56973E+01_JPRB ,&
&  0.56605E+01_JPRB, 0.50589E+01_JPRB, 0.41822E+01_JPRB, 0.34646E+01_JPRB, 0.28146E+01_JPRB, 0.23082E+01_JPRB, 0.19313E+01_JPRB ,&
&  0.16869E+01_JPRB, 0.15186E+01_JPRB, 0.13775E+01_JPRB, 0.12956E+01_JPRB, 0.12825E+01_JPRB, 0.12537E+01_JPRB, 0.13041E+01_JPRB ,&
&  0.13869E+01_JPRB, 0.15652E+01_JPRB, 0.21153E+01_JPRB, 0.28742E+01_JPRB, 0.32481E+01_JPRB, 0.34309E+01_JPRB, 0.37187E+01_JPRB ,&
&  0.38570E+01_JPRB, 0.38728E+01_JPRB, 0.41585E+01_JPRB, 0.43163E+01_JPRB, 0.46599E+01_JPRB, 0.46048E+01_JPRB, 0.49973E+01_JPRB ,&
&  0.40502E+01_JPRB, 0.32798E+01_JPRB, 0.26146E+01_JPRB, 0.22598E+01_JPRB, 0.23333E+01_JPRB, 0.29312E+01_JPRB, 0.28110E+01_JPRB ,&
&  0.24889E+01_JPRB, 0.23258E+01_JPRB, 0.21276E+01_JPRB, 0.18740E+01_JPRB, 0.18640E+01_JPRB, 0.11990E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.13056E+01_JPRB, 0.12736E+01_JPRB, 0.11475E+01_JPRB, 0.97813E+00_JPRB, 0.85620E+00_JPRB, 0.87497E+00_JPRB, 0.86970E+00_JPRB ,&
&  0.86657E+00_JPRB, 0.79878E+00_JPRB, 0.98511E+00_JPRB, 0.11797E+01_JPRB, 0.13084E+01_JPRB, 0.11627E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.12360E+01_JPRB, 0.12244E+01_JPRB, 0.11441E+01_JPRB, 0.10364E+01_JPRB, 0.94739E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.81796E+00_JPRB, 0.75465E+00_JPRB, 0.68420E+00_JPRB, 0.63642E+00_JPRB, 0.61869E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.63396E+00_JPRB, 0.60885E+00_JPRB, 0.57697E+00_JPRB, 0.54954E+00_JPRB, 0.52149E+00_JPRB, 0.50793E+00_JPRB, 0.50696E+00_JPRB ,&
&  0.51175E+00_JPRB, 0.51449E+00_JPRB, 0.52474E+00_JPRB, 0.53607E+00_JPRB, 0.54246E+00_JPRB, 0.54008E+00_JPRB, 0.51252E+00_JPRB ,&
&  0.51500E+00_JPRB, 0.51592E+00_JPRB, 0.41051E+00_JPRB, 0.42981E+00_JPRB, 0.54060E+00_JPRB, 0.66412E+00_JPRB, 0.90535E+00_JPRB ,&
&  0.11301E+01_JPRB, 0.15642E+01_JPRB, 0.21112E+01_JPRB, 0.29913E+01_JPRB, 0.35866E+01_JPRB, 0.39066E+01_JPRB, 0.39551E+01_JPRB ,&
&  0.36173E+01_JPRB, 0.35775E+01_JPRB, 0.33296E+01_JPRB, 0.29567E+01_JPRB, 0.25878E+01_JPRB, 0.23065E+01_JPRB, 0.20739E+01_JPRB ,&
&  0.18693E+01_JPRB, 0.16995E+01_JPRB, 0.15752E+01_JPRB, 0.14747E+01_JPRB, 0.14309E+01_JPRB, 0.14035E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.14100E+01_JPRB, 0.17810E+01_JPRB, 0.26553E+01_JPRB, 0.29983E+01_JPRB, 0.34295E+01_JPRB, 0.34824E+01_JPRB ,&
&  0.40822E+01_JPRB, 0.44398E+01_JPRB, 0.45144E+01_JPRB, 0.51925E+01_JPRB, 0.45179E+01_JPRB, 0.47802E+01_JPRB, 0.50213E+01_JPRB ,&
&  0.52481E+01_JPRB, 0.47198E+01_JPRB, 0.41771E+01_JPRB, 0.34897E+01_JPRB, 0.28015E+01_JPRB, 0.32404E+01_JPRB, 0.27696E+01_JPRB ,&
&  0.24127E+01_JPRB, 0.23329E+01_JPRB, 0.21493E+01_JPRB, 0.20966E+01_JPRB, 0.17406E+01_JPRB, 0.15228E+01_JPRB, 0.10950E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.95725E+00_JPRB, 0.95726E+00_JPRB, 0.86098E+00_JPRB, 0.81982E+00_JPRB, 0.74074E+00_JPRB, 0.68471E+00_JPRB ,&
&  0.70300E+00_JPRB, 0.65118E+00_JPRB, 0.68281E+00_JPRB, 0.77792E+00_JPRB, 0.10204E+01_JPRB, 0.99377E+00_JPRB, 0.10907E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.11231E+01_JPRB, 0.10005E+01_JPRB, 0.10095E+01_JPRB, 0.93760E+00_JPRB, 0.87304E+00_JPRB, 0.83416E+00_JPRB ,&
&  0.77014E+00_JPRB, 0.72637E+00_JPRB, 0.68943E+00_JPRB, 0.65177E+00_JPRB, 0.64435E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.59290E+00_JPRB, 0.56636E+00_JPRB, 0.55164E+00_JPRB, 0.54353E+00_JPRB, 0.52484E+00_JPRB, 0.51351E+00_JPRB, 0.52576E+00_JPRB ,&
&  0.53061E+00_JPRB, 0.52006E+00_JPRB, 0.51545E+00_JPRB, 0.51978E+00_JPRB, 0.52812E+00_JPRB, 0.53298E+00_JPRB, 0.52283E+00_JPRB ,&
&  0.52856E+00_JPRB, 0.67515E+00_JPRB, 0.50414E+00_JPRB, 0.40901E+00_JPRB, 0.39017E+00_JPRB, 0.50898E+00_JPRB, 0.69680E+00_JPRB ,&
&  0.93762E+00_JPRB, 0.11911E+01_JPRB, 0.16176E+01_JPRB, 0.19416E+01_JPRB, 0.22261E+01_JPRB, 0.24042E+01_JPRB, 0.25323E+01_JPRB ,&
&  0.26981E+01_JPRB, 0.20844E+01_JPRB, 0.19897E+01_JPRB, 0.19126E+01_JPRB, 0.18671E+01_JPRB, 0.18086E+01_JPRB, 0.17884E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.16806E+01_JPRB, 0.16242E+01_JPRB, 0.15617E+01_JPRB, 0.14990E+01_JPRB, 0.14436E+01_JPRB, 0.13959E+01_JPRB ,&
&  0.13885E+01_JPRB, 0.13755E+01_JPRB, 0.14741E+01_JPRB, 0.19555E+01_JPRB, 0.28326E+01_JPRB, 0.34667E+01_JPRB, 0.39078E+01_JPRB ,&
&  0.36030E+01_JPRB, 0.35690E+01_JPRB, 0.52485E+01_JPRB, 0.56740E+01_JPRB, 0.58974E+01_JPRB, 0.61722E+01_JPRB, 0.58088E+01_JPRB ,&
&  0.56603E+01_JPRB, 0.50101E+01_JPRB, 0.42980E+01_JPRB, 0.35699E+01_JPRB, 0.35954E+01_JPRB, 0.33042E+01_JPRB, 0.29212E+01_JPRB ,&
&  0.26219E+01_JPRB, 0.26177E+01_JPRB, 0.23868E+01_JPRB, 0.21142E+01_JPRB, 0.19188E+01_JPRB, 0.18841E+01_JPRB, 0.14825E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.10548E+01_JPRB, 0.80390E+00_JPRB, 0.65902E+00_JPRB, 0.58114E+00_JPRB, 0.51858E+00_JPRB, 0.49288E+00_JPRB ,&
&  0.48968E+00_JPRB, 0.50308E+00_JPRB, 0.53826E+00_JPRB, 0.58887E+00_JPRB, 0.74428E+00_JPRB, 0.83555E+00_JPRB, 0.87089E+00_JPRB ,&
&  0.80636E+00_JPRB, 0.88929E+00_JPRB, 0.90876E+00_JPRB, 0.89572E+00_JPRB, 0.91281E+00_JPRB, 0.82292E+00_JPRB, 0.78386E+00_JPRB ,&
&  0.72913E+00_JPRB, 0.70347E+00_JPRB, 0.69239E+00_JPRB, 0.65699E+00_JPRB, 0.62847E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.58468E+00_JPRB, 0.56768E+00_JPRB, 0.56093E+00_JPRB, 0.55290E+00_JPRB, 0.54734E+00_JPRB, 0.54180E+00_JPRB, 0.54933E+00_JPRB ,&
&  0.54497E+00_JPRB, 0.53805E+00_JPRB, 0.53024E+00_JPRB, 0.52752E+00_JPRB, 0.52866E+00_JPRB, 0.52852E+00_JPRB, 0.53028E+00_JPRB ,&
&  0.55435E+00_JPRB, 0.82568E+00_JPRB, 0.64252E+00_JPRB, 0.54686E+00_JPRB, 0.46355E+00_JPRB, 0.53164E+00_JPRB, 0.64197E+00_JPRB ,&
&  0.82342E+00_JPRB, 0.91271E+00_JPRB, 0.10806E+01_JPRB, 0.12873E+01_JPRB, 0.14923E+01_JPRB, 0.14552E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.13540E+01_JPRB, 0.11323E+01_JPRB, 0.12441E+01_JPRB, 0.12329E+01_JPRB, 0.11143E+01_JPRB, 0.13334E+01_JPRB ,&
&  0.14073E+01_JPRB, 0.14665E+01_JPRB, 0.15103E+01_JPRB, 0.15338E+01_JPRB, 0.15411E+01_JPRB, 0.15054E+01_JPRB, 0.14524E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.13671E+01_JPRB, 0.14506E+01_JPRB, 0.15944E+01_JPRB, 0.19992E+01_JPRB, 0.25298E+01_JPRB, 0.35439E+01_JPRB ,&
&  0.42881E+01_JPRB, 0.41095E+01_JPRB, 0.40149E+01_JPRB, 0.51777E+01_JPRB, 0.60562E+01_JPRB, 0.56496E+01_JPRB, 0.50821E+01_JPRB ,&
&  0.50873E+01_JPRB, 0.49017E+01_JPRB, 0.45871E+01_JPRB, 0.42369E+01_JPRB, 0.38960E+01_JPRB, 0.35864E+01_JPRB, 0.32482E+01_JPRB ,&
&  0.29211E+01_JPRB, 0.28183E+01_JPRB, 0.27282E+01_JPRB, 0.23808E+01_JPRB, 0.19426E+01_JPRB, 0.17134E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.13274E+01_JPRB, 0.83921E+00_JPRB, 0.53673E+00_JPRB, 0.41451E+00_JPRB, 0.37607E+00_JPRB, 0.34912E+00_JPRB, 0.36152E+00_JPRB ,&
&  0.35569E+00_JPRB, 0.40263E+00_JPRB, 0.49753E+00_JPRB, 0.53203E+00_JPRB, 0.69061E+00_JPRB, 0.76884E+00_JPRB, 0.72559E+00_JPRB ,&
&  0.65156E+00_JPRB, 0.62689E+00_JPRB, 0.85362E+00_JPRB, 0.87542E+00_JPRB, 0.86775E+00_JPRB, 0.79395E+00_JPRB, 0.76943E+00_JPRB ,&
&  0.73294E+00_JPRB, 0.71497E+00_JPRB, 0.69319E+00_JPRB, 0.64926E+00_JPRB, 0.60878E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.58159E+00_JPRB, 0.57630E+00_JPRB, 0.57174E+00_JPRB, 0.56171E+00_JPRB, 0.55465E+00_JPRB, 0.53984E+00_JPRB, 0.54360E+00_JPRB ,&
&  0.54275E+00_JPRB, 0.54130E+00_JPRB, 0.53655E+00_JPRB, 0.53614E+00_JPRB, 0.53751E+00_JPRB, 0.53642E+00_JPRB, 0.54205E+00_JPRB ,&
&  0.63091E+00_JPRB, 0.86237E+00_JPRB, 0.74677E+00_JPRB, 0.57494E+00_JPRB, 0.66409E+00_JPRB, 0.64631E+00_JPRB, 0.70394E+00_JPRB ,&
&  0.72815E+00_JPRB, 0.81160E+00_JPRB, 0.82939E+00_JPRB, 0.84127E+00_JPRB, 0.91450E+00_JPRB, 0.99006E+00_JPRB, 0.99435E+00_JPRB ,&
&  0.92296E+00_JPRB, 0.82380E+00_JPRB, 0.77348E+00_JPRB, 0.77780E+00_JPRB, 0.87517E+00_JPRB, 0.81587E+00_JPRB, 0.95236E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.11135E+01_JPRB, 0.11899E+01_JPRB, 0.12743E+01_JPRB, 0.13394E+01_JPRB, 0.13950E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.14136E+01_JPRB, 0.14239E+01_JPRB, 0.14467E+01_JPRB, 0.15615E+01_JPRB, 0.18026E+01_JPRB, 0.24917E+01_JPRB, 0.39228E+01_JPRB ,&
&  0.43559E+01_JPRB, 0.43723E+01_JPRB, 0.51617E+01_JPRB, 0.57744E+01_JPRB, 0.53812E+01_JPRB, 0.49187E+01_JPRB, 0.45572E+01_JPRB ,&
&  0.44051E+01_JPRB, 0.43597E+01_JPRB, 0.44072E+01_JPRB, 0.43903E+01_JPRB, 0.42576E+01_JPRB, 0.38743E+01_JPRB, 0.35034E+01_JPRB ,&
&  0.31274E+01_JPRB, 0.29882E+01_JPRB, 0.28994E+01_JPRB, 0.24652E+01_JPRB, 0.19533E+01_JPRB, 0.15899E+01_JPRB, 0.13920E+01_JPRB ,&
&  0.13036E+01_JPRB, 0.68109E+00_JPRB, 0.46094E+00_JPRB, 0.36182E+00_JPRB, 0.33416E+00_JPRB, 0.33187E+00_JPRB, 0.39333E+00_JPRB ,&
&  0.36031E+00_JPRB, 0.42406E+00_JPRB, 0.53387E+00_JPRB, 0.56796E+00_JPRB, 0.59885E+00_JPRB, 0.63215E+00_JPRB, 0.50831E+00_JPRB ,&
&  0.46869E+00_JPRB, 0.54278E+00_JPRB, 0.77268E+00_JPRB, 0.81309E+00_JPRB, 0.81639E+00_JPRB, 0.78847E+00_JPRB, 0.78206E+00_JPRB ,&
&  0.74030E+00_JPRB, 0.70890E+00_JPRB, 0.65572E+00_JPRB, 0.61279E+00_JPRB, 0.59446E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.60776E+00_JPRB, 0.59142E+00_JPRB, 0.58093E+00_JPRB, 0.57025E+00_JPRB, 0.55090E+00_JPRB, 0.53248E+00_JPRB, 0.53564E+00_JPRB ,&
&  0.53349E+00_JPRB, 0.52930E+00_JPRB, 0.53100E+00_JPRB, 0.53179E+00_JPRB, 0.52828E+00_JPRB, 0.53469E+00_JPRB, 0.57162E+00_JPRB ,&
&  0.50177E+00_JPRB, 0.60902E+00_JPRB, 0.52719E+00_JPRB, 0.55355E+00_JPRB, 0.80366E+00_JPRB, 0.71341E+00_JPRB, 0.70841E+00_JPRB ,&
&  0.68403E+00_JPRB, 0.68661E+00_JPRB, 0.65739E+00_JPRB, 0.64871E+00_JPRB, 0.65524E+00_JPRB, 0.70592E+00_JPRB, 0.72982E+00_JPRB ,&
&  0.64630E+00_JPRB, 0.57142E+00_JPRB, 0.56020E+00_JPRB, 0.56758E+00_JPRB, 0.62083E+00_JPRB, 0.70551E+00_JPRB, 0.82099E+00_JPRB ,&
&  0.87065E+00_JPRB, 0.91482E+00_JPRB, 0.98081E+00_JPRB, 0.10513E+01_JPRB, 0.11162E+01_JPRB, 0.11804E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.12227E+01_JPRB, 0.12733E+01_JPRB, 0.13473E+01_JPRB, 0.15114E+01_JPRB, 0.16359E+01_JPRB, 0.25163E+01_JPRB, 0.44709E+01_JPRB ,&
&  0.50001E+01_JPRB, 0.43209E+01_JPRB, 0.44636E+01_JPRB, 0.52948E+01_JPRB, 0.51199E+01_JPRB, 0.46619E+01_JPRB, 0.41373E+01_JPRB ,&
&  0.38188E+01_JPRB, 0.36582E+01_JPRB, 0.36163E+01_JPRB, 0.36968E+01_JPRB, 0.37609E+01_JPRB, 0.37729E+01_JPRB, 0.36153E+01_JPRB ,&
&  0.33129E+01_JPRB, 0.31258E+01_JPRB, 0.29629E+01_JPRB, 0.26303E+01_JPRB, 0.21215E+01_JPRB, 0.17482E+01_JPRB, 0.13424E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.80222E+00_JPRB, 0.55252E+00_JPRB, 0.45062E+00_JPRB, 0.40776E+00_JPRB, 0.43423E+00_JPRB, 0.55844E+00_JPRB ,&
&  0.44710E+00_JPRB, 0.47799E+00_JPRB, 0.48986E+00_JPRB, 0.49135E+00_JPRB, 0.48797E+00_JPRB, 0.46571E+00_JPRB, 0.42301E+00_JPRB ,&
&  0.44223E+00_JPRB, 0.57903E+00_JPRB, 0.71529E+00_JPRB, 0.75401E+00_JPRB, 0.75400E+00_JPRB, 0.73266E+00_JPRB, 0.66092E+00_JPRB ,&
&  0.74563E+00_JPRB, 0.66329E+00_JPRB, 0.62399E+00_JPRB, 0.61441E+00_JPRB, 0.62319E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.60389E+00_JPRB, 0.59383E+00_JPRB, 0.58154E+00_JPRB, 0.56226E+00_JPRB, 0.53965E+00_JPRB, 0.52142E+00_JPRB, 0.51446E+00_JPRB ,&
&  0.51858E+00_JPRB, 0.52050E+00_JPRB, 0.52204E+00_JPRB, 0.51880E+00_JPRB, 0.50869E+00_JPRB, 0.52778E+00_JPRB, 0.48151E+00_JPRB ,&
&  0.43429E+00_JPRB, 0.47501E+00_JPRB, 0.46982E+00_JPRB, 0.50648E+00_JPRB, 0.56556E+00_JPRB, 0.55188E+00_JPRB, 0.55033E+00_JPRB ,&
&  0.56159E+00_JPRB, 0.55202E+00_JPRB, 0.55539E+00_JPRB, 0.55983E+00_JPRB, 0.56025E+00_JPRB, 0.58905E+00_JPRB, 0.63693E+00_JPRB ,&
&  0.59157E+00_JPRB, 0.54258E+00_JPRB, 0.52008E+00_JPRB, 0.53820E+00_JPRB, 0.61960E+00_JPRB, 0.72197E+00_JPRB, 0.78089E+00_JPRB ,&
&  0.80426E+00_JPRB, 0.83068E+00_JPRB, 0.89104E+00_JPRB, 0.93317E+00_JPRB, 0.96392E+00_JPRB, 0.99622E+00_JPRB, 0.10234E+01_JPRB ,&
&  0.10535E+01_JPRB, 0.11190E+01_JPRB, 0.12115E+01_JPRB, 0.13674E+01_JPRB, 0.15080E+01_JPRB, 0.20634E+01_JPRB, 0.29029E+01_JPRB ,&
&  0.31245E+01_JPRB, 0.31182E+01_JPRB, 0.30955E+01_JPRB, 0.32126E+01_JPRB, 0.34031E+01_JPRB, 0.32479E+01_JPRB, 0.30418E+01_JPRB ,&
&  0.30192E+01_JPRB, 0.28091E+01_JPRB, 0.31836E+01_JPRB, 0.33077E+01_JPRB, 0.33144E+01_JPRB, 0.34304E+01_JPRB, 0.33595E+01_JPRB ,&
&  0.32465E+01_JPRB, 0.30621E+01_JPRB, 0.29900E+01_JPRB, 0.26742E+01_JPRB, 0.22143E+01_JPRB, 0.18847E+01_JPRB, 0.14507E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.90187E+00_JPRB, 0.68016E+00_JPRB, 0.58652E+00_JPRB, 0.50422E+00_JPRB, 0.49032E+00_JPRB, 0.47568E+00_JPRB ,&
&  0.45341E+00_JPRB, 0.44547E+00_JPRB, 0.43427E+00_JPRB, 0.44361E+00_JPRB, 0.42968E+00_JPRB, 0.41152E+00_JPRB, 0.44967E+00_JPRB ,&
&  0.52578E+00_JPRB, 0.73081E+00_JPRB, 0.69667E+00_JPRB, 0.71775E+00_JPRB, 0.70171E+00_JPRB, 0.67546E+00_JPRB, 0.51993E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.64054E+00_JPRB, 0.60838E+00_JPRB, 0.60262E+00_JPRB, 0.60428E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.57629E+00_JPRB, 0.57956E+00_JPRB, 0.56062E+00_JPRB, 0.54084E+00_JPRB, 0.52861E+00_JPRB, 0.50222E+00_JPRB, 0.47761E+00_JPRB ,&
&  0.52792E+00_JPRB, 0.51769E+00_JPRB, 0.51806E+00_JPRB, 0.52227E+00_JPRB, 0.51682E+00_JPRB, 0.46020E+00_JPRB, 0.41363E+00_JPRB ,&
&  0.38702E+00_JPRB, 0.38839E+00_JPRB, 0.41891E+00_JPRB, 0.43720E+00_JPRB, 0.44963E+00_JPRB, 0.44482E+00_JPRB, 0.44746E+00_JPRB ,&
&  0.45151E+00_JPRB, 0.46365E+00_JPRB, 0.49579E+00_JPRB, 0.52088E+00_JPRB, 0.52603E+00_JPRB, 0.53873E+00_JPRB, 0.58770E+00_JPRB ,&
&  0.56687E+00_JPRB, 0.57512E+00_JPRB, 0.59227E+00_JPRB, 0.54670E+00_JPRB, 0.66323E+00_JPRB, 0.70310E+00_JPRB, 0.72742E+00_JPRB ,&
&  0.77505E+00_JPRB, 0.81988E+00_JPRB, 0.84607E+00_JPRB, 0.86254E+00_JPRB, 0.89460E+00_JPRB, 0.91038E+00_JPRB, 0.93350E+00_JPRB ,&
&  0.96576E+00_JPRB, 0.10099E+01_JPRB, 0.10660E+01_JPRB, 0.11494E+01_JPRB, 0.13487E+01_JPRB, 0.15678E+01_JPRB, 0.18702E+01_JPRB ,&
&  0.20625E+01_JPRB, 0.21904E+01_JPRB, 0.23645E+01_JPRB, 0.21616E+01_JPRB, 0.27185E+01_JPRB, 0.29766E+01_JPRB, 0.27249E+01_JPRB ,&
&  0.27130E+01_JPRB, 0.24320E+01_JPRB, 0.25945E+01_JPRB, 0.27786E+01_JPRB, 0.29395E+01_JPRB, 0.30680E+01_JPRB, 0.30941E+01_JPRB ,&
&  0.29609E+01_JPRB, 0.29689E+01_JPRB, 0.28763E+01_JPRB, 0.25520E+01_JPRB, 0.21421E+01_JPRB, 0.18469E+01_JPRB, 0.14863E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.93701E+00_JPRB, 0.78670E+00_JPRB, 0.71212E+00_JPRB, 0.59278E+00_JPRB, 0.55697E+00_JPRB, 0.52227E+00_JPRB ,&
&  0.48695E+00_JPRB, 0.45647E+00_JPRB, 0.44362E+00_JPRB, 0.42901E+00_JPRB, 0.41865E+00_JPRB, 0.42386E+00_JPRB, 0.43036E+00_JPRB ,&
&  0.47442E+00_JPRB, 0.55591E+00_JPRB, 0.72611E+00_JPRB, 0.68058E+00_JPRB, 0.64959E+00_JPRB, 0.62220E+00_JPRB, 0.53947E+00_JPRB ,&
&  0.51100E+00_JPRB, 0.59977E+00_JPRB, 0.58887E+00_JPRB, 0.57758E+00_JPRB, 0.56498E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.54266E+00_JPRB, 0.54704E+00_JPRB, 0.53511E+00_JPRB, 0.52269E+00_JPRB, 0.50609E+00_JPRB, 0.45251E+00_JPRB, 0.44720E+00_JPRB ,&
&  0.51552E+00_JPRB, 0.46271E+00_JPRB, 0.51299E+00_JPRB, 0.41188E+00_JPRB, 0.38911E+00_JPRB, 0.37213E+00_JPRB, 0.35512E+00_JPRB ,&
&  0.33695E+00_JPRB, 0.34449E+00_JPRB, 0.36549E+00_JPRB, 0.38181E+00_JPRB, 0.39317E+00_JPRB, 0.39128E+00_JPRB, 0.39870E+00_JPRB ,&
&  0.41275E+00_JPRB, 0.44027E+00_JPRB, 0.47883E+00_JPRB, 0.49783E+00_JPRB, 0.51534E+00_JPRB, 0.54242E+00_JPRB, 0.56864E+00_JPRB ,&
&  0.53700E+00_JPRB, 0.53743E+00_JPRB, 0.54880E+00_JPRB, 0.57154E+00_JPRB, 0.61433E+00_JPRB, 0.67889E+00_JPRB, 0.72044E+00_JPRB ,&
&  0.73959E+00_JPRB, 0.61632E+00_JPRB, 0.73281E+00_JPRB, 0.76473E+00_JPRB, 0.83008E+00_JPRB, 0.86591E+00_JPRB, 0.88960E+00_JPRB ,&
&  0.92455E+00_JPRB, 0.96111E+00_JPRB, 0.10014E+01_JPRB, 0.10515E+01_JPRB, 0.11151E+01_JPRB, 0.12455E+01_JPRB, 0.13458E+01_JPRB ,&
&  0.15698E+01_JPRB, 0.11874E+01_JPRB, 0.13477E+01_JPRB, 0.14852E+01_JPRB, 0.20345E+01_JPRB, 0.22556E+01_JPRB, 0.22291E+01_JPRB ,&
&  0.23872E+01_JPRB, 0.22382E+01_JPRB, 0.22412E+01_JPRB, 0.22460E+01_JPRB, 0.23429E+01_JPRB, 0.24853E+01_JPRB, 0.25624E+01_JPRB ,&
&  0.25352E+01_JPRB, 0.25318E+01_JPRB, 0.26094E+01_JPRB, 0.23673E+01_JPRB, 0.20598E+01_JPRB, 0.17622E+01_JPRB, 0.14222E+01_JPRB ,&
&  0.11942E+01_JPRB, 0.99060E+00_JPRB, 0.81577E+00_JPRB, 0.69600E+00_JPRB, 0.61166E+00_JPRB, 0.56161E+00_JPRB, 0.53756E+00_JPRB ,&
&  0.53080E+00_JPRB, 0.51691E+00_JPRB, 0.49998E+00_JPRB, 0.46846E+00_JPRB, 0.45296E+00_JPRB, 0.44749E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.45986E+00_JPRB, 0.44214E+00_JPRB, 0.47127E+00_JPRB, 0.50809E+00_JPRB, 0.50648E+00_JPRB, 0.51663E+00_JPRB, 0.57351E+00_JPRB ,&
&  0.50190E+00_JPRB, 0.52674E+00_JPRB, 0.57491E+00_JPRB, 0.57033E+00_JPRB, 0.54727E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.52714E+00_JPRB, 0.51334E+00_JPRB, 0.50619E+00_JPRB, 0.49572E+00_JPRB, 0.48031E+00_JPRB, 0.44013E+00_JPRB, 0.41362E+00_JPRB ,&
&  0.42608E+00_JPRB, 0.41073E+00_JPRB, 0.39894E+00_JPRB, 0.37210E+00_JPRB, 0.35589E+00_JPRB, 0.33948E+00_JPRB, 0.32618E+00_JPRB ,&
&  0.31873E+00_JPRB, 0.33548E+00_JPRB, 0.36671E+00_JPRB, 0.37614E+00_JPRB, 0.37876E+00_JPRB, 0.38278E+00_JPRB, 0.39777E+00_JPRB ,&
&  0.41912E+00_JPRB, 0.44389E+00_JPRB, 0.45999E+00_JPRB, 0.47755E+00_JPRB, 0.49736E+00_JPRB, 0.51988E+00_JPRB, 0.51570E+00_JPRB ,&
&  0.50839E+00_JPRB, 0.48933E+00_JPRB, 0.46995E+00_JPRB, 0.50627E+00_JPRB, 0.57263E+00_JPRB, 0.63553E+00_JPRB, 0.61629E+00_JPRB ,&
&  0.43602E+00_JPRB, 0.43750E+00_JPRB, 0.61910E+00_JPRB, 0.67701E+00_JPRB, 0.73498E+00_JPRB, 0.77077E+00_JPRB, 0.81840E+00_JPRB ,&
&  0.86253E+00_JPRB, 0.90445E+00_JPRB, 0.93775E+00_JPRB, 0.97529E+00_JPRB, 0.10287E+01_JPRB, 0.11039E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.10128E+01_JPRB, 0.97555E+00_JPRB, 0.10771E+01_JPRB, 0.15436E+01_JPRB, 0.16388E+01_JPRB, 0.16134E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.17332E+01_JPRB, 0.18016E+01_JPRB, 0.18688E+01_JPRB, 0.18492E+01_JPRB, 0.19012E+01_JPRB, 0.20020E+01_JPRB ,&
&  0.21918E+01_JPRB, 0.23573E+01_JPRB, 0.23417E+01_JPRB, 0.22444E+01_JPRB, 0.20710E+01_JPRB, 0.18738E+01_JPRB, 0.15852E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.10184E+01_JPRB, 0.81692E+00_JPRB, 0.68434E+00_JPRB, 0.61596E+00_JPRB, 0.59056E+00_JPRB, 0.57385E+00_JPRB ,&
&  0.58342E+00_JPRB, 0.58641E+00_JPRB, 0.57185E+00_JPRB, 0.52426E+00_JPRB, 0.48731E+00_JPRB, 0.47367E+00_JPRB, 0.46282E+00_JPRB ,&
&  0.44766E+00_JPRB, 0.41551E+00_JPRB, 0.37906E+00_JPRB, 0.38464E+00_JPRB, 0.42019E+00_JPRB, 0.45571E+00_JPRB, 0.47324E+00_JPRB ,&
&  0.47634E+00_JPRB, 0.47275E+00_JPRB, 0.48074E+00_JPRB, 0.50317E+00_JPRB, 0.51476E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.51693E+00_JPRB, 0.47941E+00_JPRB, 0.45796E+00_JPRB, 0.46100E+00_JPRB, 0.42566E+00_JPRB, 0.39634E+00_JPRB, 0.38238E+00_JPRB ,&
&  0.38720E+00_JPRB, 0.39027E+00_JPRB, 0.39256E+00_JPRB, 0.37352E+00_JPRB, 0.35810E+00_JPRB, 0.34960E+00_JPRB, 0.34779E+00_JPRB ,&
&  0.36104E+00_JPRB, 0.37759E+00_JPRB, 0.39057E+00_JPRB, 0.38791E+00_JPRB, 0.38823E+00_JPRB, 0.39630E+00_JPRB, 0.41321E+00_JPRB ,&
&  0.43665E+00_JPRB, 0.45098E+00_JPRB, 0.45160E+00_JPRB, 0.45353E+00_JPRB, 0.47504E+00_JPRB, 0.49759E+00_JPRB, 0.46745E+00_JPRB ,&
&  0.43500E+00_JPRB, 0.41283E+00_JPRB, 0.40966E+00_JPRB, 0.45746E+00_JPRB, 0.52835E+00_JPRB, 0.56311E+00_JPRB, 0.44094E+00_JPRB ,&
&  0.35306E+00_JPRB, 0.33717E+00_JPRB, 0.37579E+00_JPRB, 0.51550E+00_JPRB, 0.56133E+00_JPRB, 0.61088E+00_JPRB, 0.68534E+00_JPRB ,&
&  0.63245E+00_JPRB, 0.63345E+00_JPRB, 0.65797E+00_JPRB, 0.81181E+00_JPRB, 0.86878E+00_JPRB, 0.91521E+00_JPRB, 0.94712E+00_JPRB ,&
&  0.97473E+00_JPRB, 0.10480E+01_JPRB, 0.12354E+01_JPRB, 0.10772E+01_JPRB, 0.12042E+01_JPRB, 0.14655E+01_JPRB, 0.14349E+01_JPRB ,&
&  0.15445E+01_JPRB, 0.16263E+01_JPRB, 0.18693E+01_JPRB, 0.17999E+01_JPRB, 0.16375E+01_JPRB, 0.15751E+01_JPRB, 0.16540E+01_JPRB ,&
&  0.19312E+01_JPRB, 0.21347E+01_JPRB, 0.20476E+01_JPRB, 0.19226E+01_JPRB, 0.18327E+01_JPRB, 0.17158E+01_JPRB, 0.15050E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.10172E+01_JPRB, 0.84840E+00_JPRB, 0.71737E+00_JPRB, 0.66604E+00_JPRB, 0.65544E+00_JPRB, 0.63896E+00_JPRB ,&
&  0.64248E+00_JPRB, 0.64193E+00_JPRB, 0.62962E+00_JPRB, 0.56713E+00_JPRB, 0.51009E+00_JPRB, 0.46121E+00_JPRB, 0.42600E+00_JPRB ,&
&  0.40536E+00_JPRB, 0.37955E+00_JPRB, 0.36383E+00_JPRB, 0.37381E+00_JPRB, 0.41589E+00_JPRB, 0.46399E+00_JPRB, 0.48492E+00_JPRB ,&
&  0.47260E+00_JPRB, 0.46267E+00_JPRB, 0.48424E+00_JPRB, 0.51895E+00_JPRB, 0.53178E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.54483E+00_JPRB, 0.54020E+00_JPRB, 0.52285E+00_JPRB, 0.49058E+00_JPRB, 0.46242E+00_JPRB, 0.43080E+00_JPRB, 0.40909E+00_JPRB ,&
&  0.39864E+00_JPRB, 0.39398E+00_JPRB, 0.39293E+00_JPRB, 0.39545E+00_JPRB, 0.40031E+00_JPRB, 0.40891E+00_JPRB, 0.41255E+00_JPRB ,&
&  0.41792E+00_JPRB, 0.42169E+00_JPRB, 0.42809E+00_JPRB, 0.43774E+00_JPRB, 0.44896E+00_JPRB, 0.46145E+00_JPRB, 0.47356E+00_JPRB ,&
&  0.48357E+00_JPRB, 0.47748E+00_JPRB, 0.45620E+00_JPRB, 0.44302E+00_JPRB, 0.44754E+00_JPRB, 0.46206E+00_JPRB, 0.44524E+00_JPRB ,&
&  0.40549E+00_JPRB, 0.38104E+00_JPRB, 0.38117E+00_JPRB, 0.44401E+00_JPRB, 0.48240E+00_JPRB, 0.50293E+00_JPRB, 0.38669E+00_JPRB ,&
&  0.32325E+00_JPRB, 0.29584E+00_JPRB, 0.29387E+00_JPRB, 0.31874E+00_JPRB, 0.37439E+00_JPRB, 0.46043E+00_JPRB, 0.52586E+00_JPRB ,&
&  0.57337E+00_JPRB, 0.61818E+00_JPRB, 0.63347E+00_JPRB, 0.66050E+00_JPRB, 0.70527E+00_JPRB, 0.75147E+00_JPRB, 0.79938E+00_JPRB ,&
&  0.83880E+00_JPRB, 0.90829E+00_JPRB, 0.10640E+01_JPRB, 0.97481E+00_JPRB, 0.10156E+01_JPRB, 0.10785E+01_JPRB, 0.12063E+01_JPRB ,&
&  0.14416E+01_JPRB, 0.15403E+01_JPRB, 0.16416E+01_JPRB, 0.18127E+01_JPRB, 0.17193E+01_JPRB, 0.16402E+01_JPRB, 0.16426E+01_JPRB ,&
&  0.18436E+01_JPRB, 0.20141E+01_JPRB, 0.19345E+01_JPRB, 0.17253E+01_JPRB, 0.16391E+01_JPRB, 0.15835E+01_JPRB, 0.13902E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.10156E+01_JPRB, 0.89051E+00_JPRB, 0.77806E+00_JPRB, 0.74451E+00_JPRB, 0.72749E+00_JPRB, 0.71767E+00_JPRB ,&
&  0.71093E+00_JPRB, 0.69844E+00_JPRB, 0.68257E+00_JPRB, 0.62344E+00_JPRB, 0.56565E+00_JPRB, 0.50787E+00_JPRB, 0.46775E+00_JPRB ,&
&  0.44221E+00_JPRB, 0.43229E+00_JPRB, 0.43354E+00_JPRB, 0.44728E+00_JPRB, 0.47843E+00_JPRB, 0.50474E+00_JPRB, 0.52155E+00_JPRB ,&
&  0.52251E+00_JPRB, 0.50740E+00_JPRB, 0.50228E+00_JPRB, 0.50560E+00_JPRB, 0.53081E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.60933E+00_JPRB, 0.61755E+00_JPRB, 0.60661E+00_JPRB, 0.57915E+00_JPRB, 0.54494E+00_JPRB, 0.50705E+00_JPRB, 0.48542E+00_JPRB ,&
&  0.47663E+00_JPRB, 0.47673E+00_JPRB, 0.47352E+00_JPRB, 0.46951E+00_JPRB, 0.46935E+00_JPRB, 0.47103E+00_JPRB, 0.47436E+00_JPRB ,&
&  0.48066E+00_JPRB, 0.49034E+00_JPRB, 0.49927E+00_JPRB, 0.49938E+00_JPRB, 0.50507E+00_JPRB, 0.51052E+00_JPRB, 0.51594E+00_JPRB ,&
&  0.51203E+00_JPRB, 0.48891E+00_JPRB, 0.45918E+00_JPRB, 0.44258E+00_JPRB, 0.42496E+00_JPRB, 0.42081E+00_JPRB, 0.40756E+00_JPRB ,&
&  0.38054E+00_JPRB, 0.37697E+00_JPRB, 0.40741E+00_JPRB, 0.43685E+00_JPRB, 0.43355E+00_JPRB, 0.45659E+00_JPRB, 0.34127E+00_JPRB ,&
&  0.30100E+00_JPRB, 0.27968E+00_JPRB, 0.27432E+00_JPRB, 0.28239E+00_JPRB, 0.30657E+00_JPRB, 0.34118E+00_JPRB, 0.38444E+00_JPRB ,&
&  0.45753E+00_JPRB, 0.52273E+00_JPRB, 0.56452E+00_JPRB, 0.60335E+00_JPRB, 0.64317E+00_JPRB, 0.67957E+00_JPRB, 0.70433E+00_JPRB ,&
&  0.72539E+00_JPRB, 0.77026E+00_JPRB, 0.84506E+00_JPRB, 0.94843E+00_JPRB, 0.11568E+01_JPRB, 0.11409E+01_JPRB, 0.11936E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.17811E+01_JPRB, 0.17031E+01_JPRB, 0.16322E+01_JPRB, 0.16296E+01_JPRB, 0.16964E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.18980E+01_JPRB, 0.19869E+01_JPRB, 0.19351E+01_JPRB, 0.17164E+01_JPRB, 0.16002E+01_JPRB, 0.15264E+01_JPRB, 0.13615E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.11575E+01_JPRB, 0.12144E+01_JPRB, 0.10047E+01_JPRB, 0.88174E+00_JPRB, 0.82100E+00_JPRB, 0.79244E+00_JPRB ,&
&  0.76830E+00_JPRB, 0.74309E+00_JPRB, 0.71014E+00_JPRB, 0.67667E+00_JPRB, 0.64036E+00_JPRB, 0.60708E+00_JPRB, 0.57890E+00_JPRB ,&
&  0.55918E+00_JPRB, 0.54316E+00_JPRB, 0.53430E+00_JPRB, 0.52766E+00_JPRB, 0.51767E+00_JPRB, 0.51263E+00_JPRB, 0.51818E+00_JPRB ,&
&  0.52998E+00_JPRB, 0.53776E+00_JPRB, 0.55211E+00_JPRB, 0.56713E+00_JPRB, 0.58785E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.61496E+00_JPRB, 0.62635E+00_JPRB, 0.63294E+00_JPRB, 0.62898E+00_JPRB, 0.61421E+00_JPRB, 0.59562E+00_JPRB, 0.58101E+00_JPRB ,&
&  0.57010E+00_JPRB, 0.56038E+00_JPRB, 0.54989E+00_JPRB, 0.54185E+00_JPRB, 0.53515E+00_JPRB, 0.53221E+00_JPRB, 0.53042E+00_JPRB ,&
&  0.53141E+00_JPRB, 0.52184E+00_JPRB, 0.52005E+00_JPRB, 0.52393E+00_JPRB, 0.52992E+00_JPRB, 0.52958E+00_JPRB, 0.52555E+00_JPRB ,&
&  0.50444E+00_JPRB, 0.47524E+00_JPRB, 0.44835E+00_JPRB, 0.42709E+00_JPRB, 0.41113E+00_JPRB, 0.40286E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.41592E+00_JPRB, 0.41465E+00_JPRB, 0.41636E+00_JPRB, 0.43532E+00_JPRB, 0.44732E+00_JPRB, 0.38792E+00_JPRB, 0.30488E+00_JPRB ,&
&  0.28881E+00_JPRB, 0.27392E+00_JPRB, 0.27299E+00_JPRB, 0.27777E+00_JPRB, 0.29050E+00_JPRB, 0.31294E+00_JPRB, 0.35142E+00_JPRB ,&
&  0.40931E+00_JPRB, 0.52906E+00_JPRB, 0.59370E+00_JPRB, 0.63970E+00_JPRB, 0.67894E+00_JPRB, 0.71065E+00_JPRB, 0.72229E+00_JPRB ,&
&  0.73149E+00_JPRB, 0.75433E+00_JPRB, 0.79891E+00_JPRB, 0.85288E+00_JPRB, 0.93707E+00_JPRB, 0.11010E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.14447E+01_JPRB, 0.15502E+01_JPRB, 0.15463E+01_JPRB, 0.14861E+01_JPRB, 0.15365E+01_JPRB, 0.16352E+01_JPRB, 0.16675E+01_JPRB ,&
&  0.17315E+01_JPRB, 0.17954E+01_JPRB, 0.17394E+01_JPRB, 0.16241E+01_JPRB, 0.15503E+01_JPRB, 0.14953E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.12259E+01_JPRB, 0.11547E+01_JPRB, 0.11371E+01_JPRB, 0.10163E+01_JPRB, 0.94060E+00_JPRB, 0.86766E+00_JPRB, 0.81755E+00_JPRB ,&
&  0.78159E+00_JPRB, 0.75255E+00_JPRB, 0.71080E+00_JPRB, 0.67682E+00_JPRB, 0.65740E+00_JPRB, 0.64290E+00_JPRB, 0.62871E+00_JPRB ,&
&  0.61391E+00_JPRB, 0.59789E+00_JPRB, 0.58130E+00_JPRB, 0.56601E+00_JPRB, 0.54814E+00_JPRB, 0.53833E+00_JPRB, 0.53836E+00_JPRB ,&
&  0.54486E+00_JPRB, 0.55805E+00_JPRB, 0.57305E+00_JPRB, 0.58800E+00_JPRB, 0.60366E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.63967E+00_JPRB, 0.64524E+00_JPRB, 0.64574E+00_JPRB, 0.63778E+00_JPRB, 0.62375E+00_JPRB, 0.60666E+00_JPRB, 0.59148E+00_JPRB ,&
&  0.57883E+00_JPRB, 0.56947E+00_JPRB, 0.55958E+00_JPRB, 0.55254E+00_JPRB, 0.54394E+00_JPRB, 0.53928E+00_JPRB, 0.53334E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.53210E+00_JPRB, 0.52711E+00_JPRB, 0.52059E+00_JPRB, 0.50554E+00_JPRB, 0.49131E+00_JPRB, 0.48017E+00_JPRB ,&
&  0.46950E+00_JPRB, 0.46117E+00_JPRB, 0.44269E+00_JPRB, 0.42848E+00_JPRB, 0.42203E+00_JPRB, 0.41805E+00_JPRB, 0.42694E+00_JPRB ,&
&  0.42758E+00_JPRB, 0.43478E+00_JPRB, 0.44148E+00_JPRB, 0.43583E+00_JPRB, 0.42845E+00_JPRB, 0.33015E+00_JPRB, 0.30186E+00_JPRB ,&
&  0.29274E+00_JPRB, 0.28593E+00_JPRB, 0.28461E+00_JPRB, 0.29025E+00_JPRB, 0.30078E+00_JPRB, 0.32832E+00_JPRB, 0.35555E+00_JPRB ,&
&  0.38403E+00_JPRB, 0.54080E+00_JPRB, 0.60542E+00_JPRB, 0.66545E+00_JPRB, 0.70764E+00_JPRB, 0.73872E+00_JPRB, 0.75761E+00_JPRB ,&
&  0.77764E+00_JPRB, 0.80778E+00_JPRB, 0.82868E+00_JPRB, 0.86953E+00_JPRB, 0.94297E+00_JPRB, 0.10667E+01_JPRB, 0.11787E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.13623E+01_JPRB, 0.14156E+01_JPRB, 0.14372E+01_JPRB, 0.14747E+01_JPRB, 0.14952E+01_JPRB, 0.14984E+01_JPRB ,&
&  0.15077E+01_JPRB, 0.14789E+01_JPRB, 0.14861E+01_JPRB, 0.14560E+01_JPRB, 0.14277E+01_JPRB, 0.13921E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.10954E+01_JPRB, 0.10165E+01_JPRB, 0.93845E+00_JPRB, 0.88047E+00_JPRB, 0.83741E+00_JPRB, 0.81761E+00_JPRB ,&
&  0.80617E+00_JPRB, 0.79877E+00_JPRB, 0.76921E+00_JPRB, 0.73544E+00_JPRB, 0.71343E+00_JPRB, 0.69900E+00_JPRB, 0.67277E+00_JPRB ,&
&  0.64802E+00_JPRB, 0.62156E+00_JPRB, 0.60445E+00_JPRB, 0.58733E+00_JPRB, 0.57224E+00_JPRB, 0.56449E+00_JPRB, 0.56235E+00_JPRB ,&
&  0.56832E+00_JPRB, 0.57774E+00_JPRB, 0.59377E+00_JPRB, 0.61176E+00_JPRB, 0.63115E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.66860E+00_JPRB, 0.65529E+00_JPRB, 0.62850E+00_JPRB, 0.59214E+00_JPRB, 0.56602E+00_JPRB, 0.54892E+00_JPRB, 0.54229E+00_JPRB ,&
&  0.53929E+00_JPRB, 0.53891E+00_JPRB, 0.53790E+00_JPRB, 0.53607E+00_JPRB, 0.53138E+00_JPRB, 0.52470E+00_JPRB, 0.51759E+00_JPRB ,&
&  0.50640E+00_JPRB, 0.49536E+00_JPRB, 0.48359E+00_JPRB, 0.47671E+00_JPRB, 0.47059E+00_JPRB, 0.46613E+00_JPRB, 0.46237E+00_JPRB ,&
&  0.45792E+00_JPRB, 0.45231E+00_JPRB, 0.43374E+00_JPRB, 0.42563E+00_JPRB, 0.41808E+00_JPRB, 0.41940E+00_JPRB, 0.41755E+00_JPRB ,&
&  0.43390E+00_JPRB, 0.38398E+00_JPRB, 0.37622E+00_JPRB, 0.35943E+00_JPRB, 0.34521E+00_JPRB, 0.32995E+00_JPRB, 0.31817E+00_JPRB ,&
&  0.31183E+00_JPRB, 0.30610E+00_JPRB, 0.30471E+00_JPRB, 0.31157E+00_JPRB, 0.32072E+00_JPRB, 0.33948E+00_JPRB, 0.36128E+00_JPRB ,&
&  0.40536E+00_JPRB, 0.54857E+00_JPRB, 0.62027E+00_JPRB, 0.67692E+00_JPRB, 0.73124E+00_JPRB, 0.78015E+00_JPRB, 0.80878E+00_JPRB ,&
&  0.83943E+00_JPRB, 0.87962E+00_JPRB, 0.91587E+00_JPRB, 0.93456E+00_JPRB, 0.10242E+01_JPRB, 0.10889E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.12181E+01_JPRB, 0.12706E+01_JPRB, 0.13156E+01_JPRB, 0.13438E+01_JPRB, 0.13647E+01_JPRB, 0.13546E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.13089E+01_JPRB, 0.12958E+01_JPRB, 0.12896E+01_JPRB, 0.12472E+01_JPRB, 0.12205E+01_JPRB, 0.11959E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.10336E+01_JPRB, 0.97539E+00_JPRB, 0.91766E+00_JPRB, 0.86605E+00_JPRB, 0.83586E+00_JPRB, 0.82527E+00_JPRB ,&
&  0.82386E+00_JPRB, 0.81671E+00_JPRB, 0.79895E+00_JPRB, 0.77260E+00_JPRB, 0.74598E+00_JPRB, 0.72830E+00_JPRB, 0.70529E+00_JPRB ,&
&  0.68117E+00_JPRB, 0.65294E+00_JPRB, 0.63407E+00_JPRB, 0.61693E+00_JPRB, 0.60739E+00_JPRB, 0.60416E+00_JPRB, 0.60601E+00_JPRB ,&
&  0.61062E+00_JPRB, 0.61925E+00_JPRB, 0.63431E+00_JPRB, 0.65092E+00_JPRB, 0.66417E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.61834E+00_JPRB, 0.58718E+00_JPRB, 0.55416E+00_JPRB, 0.53631E+00_JPRB, 0.53271E+00_JPRB, 0.52934E+00_JPRB, 0.52707E+00_JPRB ,&
&  0.52536E+00_JPRB, 0.52801E+00_JPRB, 0.53097E+00_JPRB, 0.53219E+00_JPRB, 0.53028E+00_JPRB, 0.52637E+00_JPRB, 0.52063E+00_JPRB ,&
&  0.51260E+00_JPRB, 0.50368E+00_JPRB, 0.49502E+00_JPRB, 0.48750E+00_JPRB, 0.47919E+00_JPRB, 0.47120E+00_JPRB, 0.46084E+00_JPRB ,&
&  0.45024E+00_JPRB, 0.44028E+00_JPRB, 0.40614E+00_JPRB, 0.40268E+00_JPRB, 0.39506E+00_JPRB, 0.39345E+00_JPRB, 0.39628E+00_JPRB ,&
&  0.39622E+00_JPRB, 0.38892E+00_JPRB, 0.38133E+00_JPRB, 0.36743E+00_JPRB, 0.35544E+00_JPRB, 0.34216E+00_JPRB, 0.33473E+00_JPRB ,&
&  0.33184E+00_JPRB, 0.33383E+00_JPRB, 0.34017E+00_JPRB, 0.34916E+00_JPRB, 0.36135E+00_JPRB, 0.37610E+00_JPRB, 0.39803E+00_JPRB ,&
&  0.45160E+00_JPRB, 0.52779E+00_JPRB, 0.61743E+00_JPRB, 0.66871E+00_JPRB, 0.72668E+00_JPRB, 0.78184E+00_JPRB, 0.82115E+00_JPRB ,&
&  0.85818E+00_JPRB, 0.89852E+00_JPRB, 0.94327E+00_JPRB, 0.99369E+00_JPRB, 0.10417E+01_JPRB, 0.10804E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11982E+01_JPRB, 0.12092E+01_JPRB, 0.12110E+01_JPRB, 0.12066E+01_JPRB, 0.11897E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11501E+01_JPRB, 0.11363E+01_JPRB, 0.11031E+01_JPRB, 0.10844E+01_JPRB, 0.10715E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.10188E+01_JPRB, 0.98664E+00_JPRB, 0.95624E+00_JPRB, 0.91632E+00_JPRB, 0.88593E+00_JPRB, 0.87531E+00_JPRB, 0.86826E+00_JPRB ,&
&  0.86072E+00_JPRB, 0.85086E+00_JPRB, 0.83923E+00_JPRB, 0.82337E+00_JPRB, 0.79849E+00_JPRB, 0.77405E+00_JPRB, 0.75160E+00_JPRB ,&
&  0.73288E+00_JPRB, 0.71284E+00_JPRB, 0.69509E+00_JPRB, 0.67476E+00_JPRB, 0.66195E+00_JPRB, 0.65046E+00_JPRB, 0.64692E+00_JPRB ,&
&  0.64704E+00_JPRB, 0.65073E+00_JPRB, 0.65244E+00_JPRB, 0.65097E+00_JPRB, 0.63944E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.59543E+00_JPRB, 0.56691E+00_JPRB, 0.55171E+00_JPRB, 0.54795E+00_JPRB, 0.54543E+00_JPRB, 0.54288E+00_JPRB, 0.54066E+00_JPRB ,&
&  0.53610E+00_JPRB, 0.53250E+00_JPRB, 0.53207E+00_JPRB, 0.53202E+00_JPRB, 0.53007E+00_JPRB, 0.52763E+00_JPRB, 0.52454E+00_JPRB ,&
&  0.52099E+00_JPRB, 0.51592E+00_JPRB, 0.51092E+00_JPRB, 0.50452E+00_JPRB, 0.49776E+00_JPRB, 0.48964E+00_JPRB, 0.47918E+00_JPRB ,&
&  0.46995E+00_JPRB, 0.46125E+00_JPRB, 0.45001E+00_JPRB, 0.44517E+00_JPRB, 0.44263E+00_JPRB, 0.41841E+00_JPRB, 0.41235E+00_JPRB ,&
&  0.41597E+00_JPRB, 0.41685E+00_JPRB, 0.41812E+00_JPRB, 0.42004E+00_JPRB, 0.42437E+00_JPRB, 0.41831E+00_JPRB, 0.41741E+00_JPRB ,&
&  0.40067E+00_JPRB, 0.41204E+00_JPRB, 0.42065E+00_JPRB, 0.42992E+00_JPRB, 0.44071E+00_JPRB, 0.45829E+00_JPRB, 0.48857E+00_JPRB ,&
&  0.54620E+00_JPRB, 0.57686E+00_JPRB, 0.60972E+00_JPRB, 0.64309E+00_JPRB, 0.68329E+00_JPRB, 0.72249E+00_JPRB, 0.75593E+00_JPRB ,&
&  0.78539E+00_JPRB, 0.81767E+00_JPRB, 0.85536E+00_JPRB, 0.90391E+00_JPRB, 0.94424E+00_JPRB, 0.98065E+00_JPRB, 0.10067E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.10573E+01_JPRB, 0.10721E+01_JPRB, 0.10815E+01_JPRB, 0.10917E+01_JPRB, 0.10960E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.11020E+01_JPRB, 0.10927E+01_JPRB, 0.10760E+01_JPRB, 0.10632E+01_JPRB, 0.10543E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.10267E+01_JPRB, 0.10117E+01_JPRB, 0.99409E+00_JPRB, 0.97384E+00_JPRB, 0.95020E+00_JPRB, 0.92904E+00_JPRB, 0.90531E+00_JPRB ,&
&  0.88581E+00_JPRB, 0.87062E+00_JPRB, 0.85824E+00_JPRB, 0.84327E+00_JPRB, 0.82651E+00_JPRB, 0.81026E+00_JPRB, 0.79211E+00_JPRB ,&
&  0.77621E+00_JPRB, 0.76120E+00_JPRB, 0.74961E+00_JPRB, 0.73657E+00_JPRB, 0.72771E+00_JPRB, 0.71543E+00_JPRB, 0.70488E+00_JPRB ,&
&  0.69301E+00_JPRB, 0.68472E+00_JPRB, 0.66947E+00_JPRB, 0.64829E+00_JPRB, 0.61988E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.57966E+00_JPRB, 0.56742E+00_JPRB, 0.56173E+00_JPRB, 0.55841E+00_JPRB, 0.55558E+00_JPRB, 0.55163E+00_JPRB, 0.54727E+00_JPRB ,&
&  0.54356E+00_JPRB, 0.53929E+00_JPRB, 0.53512E+00_JPRB, 0.53340E+00_JPRB, 0.53084E+00_JPRB, 0.52938E+00_JPRB, 0.52787E+00_JPRB ,&
&  0.52663E+00_JPRB, 0.52492E+00_JPRB, 0.52370E+00_JPRB, 0.52196E+00_JPRB, 0.51938E+00_JPRB, 0.51586E+00_JPRB, 0.51271E+00_JPRB ,&
&  0.51082E+00_JPRB, 0.50948E+00_JPRB, 0.50725E+00_JPRB, 0.50551E+00_JPRB, 0.50563E+00_JPRB, 0.50504E+00_JPRB, 0.50379E+00_JPRB ,&
&  0.50542E+00_JPRB, 0.50669E+00_JPRB, 0.50657E+00_JPRB, 0.50534E+00_JPRB, 0.50368E+00_JPRB, 0.50393E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.51134E+00_JPRB, 0.52316E+00_JPRB, 0.53230E+00_JPRB, 0.54114E+00_JPRB, 0.55005E+00_JPRB, 0.56085E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.58732E+00_JPRB, 0.59862E+00_JPRB, 0.61107E+00_JPRB, 0.62756E+00_JPRB, 0.64477E+00_JPRB, 0.66210E+00_JPRB, 0.68090E+00_JPRB ,&
&  0.69676E+00_JPRB, 0.71300E+00_JPRB, 0.73164E+00_JPRB, 0.75213E+00_JPRB, 0.76935E+00_JPRB, 0.79210E+00_JPRB, 0.81958E+00_JPRB ,&
&  0.85398E+00_JPRB, 0.87201E+00_JPRB, 0.89177E+00_JPRB, 0.91157E+00_JPRB, 0.93535E+00_JPRB, 0.94934E+00_JPRB, 0.96220E+00_JPRB ,&
&  0.97137E+00_JPRB, 0.97910E+00_JPRB, 0.97674E+00_JPRB, 0.97267E+00_JPRB, 0.96957E+00_JPRB, 0.96378E+00_JPRB, 0.95660E+00_JPRB ,&
&  0.95012E+00_JPRB, 0.94513E+00_JPRB, 0.93536E+00_JPRB, 0.92532E+00_JPRB, 0.91421E+00_JPRB, 0.90378E+00_JPRB, 0.88441E+00_JPRB ,&
&  0.86655E+00_JPRB, 0.85314E+00_JPRB, 0.84398E+00_JPRB, 0.83158E+00_JPRB, 0.81954E+00_JPRB, 0.80838E+00_JPRB, 0.79759E+00_JPRB ,&
&  0.78700E+00_JPRB, 0.77483E+00_JPRB, 0.76208E+00_JPRB, 0.74845E+00_JPRB, 0.73676E+00_JPRB, 0.72087E+00_JPRB, 0.70314E+00_JPRB ,&
&  0.68263E+00_JPRB, 0.66538E+00_JPRB, 0.64433E+00_JPRB, 0.61950E+00_JPRB, 0.59496E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.59847E+00_JPRB, 0.59431E+00_JPRB, 0.59030E+00_JPRB, 0.58620E+00_JPRB, 0.58162E+00_JPRB, 0.57657E+00_JPRB, 0.57113E+00_JPRB ,&
&  0.56599E+00_JPRB, 0.56157E+00_JPRB, 0.55757E+00_JPRB, 0.55383E+00_JPRB, 0.55037E+00_JPRB, 0.54761E+00_JPRB, 0.54624E+00_JPRB ,&
&  0.54570E+00_JPRB, 0.54508E+00_JPRB, 0.54485E+00_JPRB, 0.54479E+00_JPRB, 0.54442E+00_JPRB, 0.54381E+00_JPRB, 0.54310E+00_JPRB ,&
&  0.54272E+00_JPRB, 0.54244E+00_JPRB, 0.54203E+00_JPRB, 0.54208E+00_JPRB, 0.54260E+00_JPRB, 0.54319E+00_JPRB, 0.54379E+00_JPRB ,&
&  0.54500E+00_JPRB, 0.54639E+00_JPRB, 0.54824E+00_JPRB, 0.55036E+00_JPRB, 0.55464E+00_JPRB, 0.55948E+00_JPRB, 0.56449E+00_JPRB ,&
&  0.56935E+00_JPRB, 0.57441E+00_JPRB, 0.57872E+00_JPRB, 0.58246E+00_JPRB, 0.58590E+00_JPRB, 0.58990E+00_JPRB, 0.59435E+00_JPRB ,&
&  0.59928E+00_JPRB, 0.60474E+00_JPRB, 0.61097E+00_JPRB, 0.61718E+00_JPRB, 0.62301E+00_JPRB, 0.62919E+00_JPRB, 0.63580E+00_JPRB ,&
&  0.64239E+00_JPRB, 0.64908E+00_JPRB, 0.65579E+00_JPRB, 0.66248E+00_JPRB, 0.66893E+00_JPRB, 0.67645E+00_JPRB, 0.68535E+00_JPRB ,&
&  0.69524E+00_JPRB, 0.70320E+00_JPRB, 0.71225E+00_JPRB, 0.72205E+00_JPRB, 0.73589E+00_JPRB, 0.74297E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.75611E+00_JPRB, 0.76558E+00_JPRB, 0.77132E+00_JPRB, 0.77726E+00_JPRB, 0.78271E+00_JPRB, 0.78751E+00_JPRB, 0.79038E+00_JPRB ,&
&  0.79282E+00_JPRB, 0.79485E+00_JPRB, 0.79495E+00_JPRB, 0.79394E+00_JPRB, 0.79197E+00_JPRB, 0.78948E+00_JPRB, 0.78355E+00_JPRB ,&
&  0.77798E+00_JPRB, 0.77260E+00_JPRB, 0.76766E+00_JPRB, 0.76065E+00_JPRB, 0.75363E+00_JPRB, 0.74642E+00_JPRB, 0.73902E+00_JPRB ,&
&  0.72955E+00_JPRB, 0.71991E+00_JPRB, 0.70959E+00_JPRB, 0.69869E+00_JPRB, 0.68810E+00_JPRB, 0.67716E+00_JPRB, 0.66543E+00_JPRB ,&
&  0.65230E+00_JPRB, 0.64009E+00_JPRB, 0.62647E+00_JPRB, 0.61302E+00_JPRB, 0.60299E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB, 0.62013E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB, 0.12456E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.13190E+00_JPRB, 0.13191E+00_JPRB, 0.13184E+00_JPRB, 0.13159E+00_JPRB, 0.13107E+00_JPRB, 0.13094E+00_JPRB, 0.13078E+00_JPRB ,&
&  0.13054E+00_JPRB, 0.13001E+00_JPRB, 0.13000E+00_JPRB, 0.12993E+00_JPRB, 0.12984E+00_JPRB, 0.12974E+00_JPRB, 0.12956E+00_JPRB ,&
&  0.12917E+00_JPRB, 0.12859E+00_JPRB, 0.12682E+00_JPRB, 0.12597E+00_JPRB, 0.12544E+00_JPRB, 0.12517E+00_JPRB, 0.12495E+00_JPRB ,&
&  0.12482E+00_JPRB, 0.12476E+00_JPRB, 0.12469E+00_JPRB, 0.12505E+00_JPRB, 0.12521E+00_JPRB, 0.12528E+00_JPRB, 0.12532E+00_JPRB ,&
&  0.12533E+00_JPRB, 0.12541E+00_JPRB, 0.12542E+00_JPRB, 0.12542E+00_JPRB, 0.12552E+00_JPRB, 0.12552E+00_JPRB, 0.12536E+00_JPRB ,&
&  0.12518E+00_JPRB, 0.12496E+00_JPRB, 0.12474E+00_JPRB, 0.12447E+00_JPRB, 0.12427E+00_JPRB, 0.12411E+00_JPRB, 0.12404E+00_JPRB ,&
&  0.12391E+00_JPRB, 0.12380E+00_JPRB, 0.12368E+00_JPRB, 0.12362E+00_JPRB, 0.12352E+00_JPRB, 0.12347E+00_JPRB, 0.12336E+00_JPRB ,&
&  0.12342E+00_JPRB, 0.12343E+00_JPRB, 0.12346E+00_JPRB, 0.12345E+00_JPRB, 0.12344E+00_JPRB, 0.12339E+00_JPRB, 0.12331E+00_JPRB ,&
&  0.12315E+00_JPRB, 0.12305E+00_JPRB, 0.12294E+00_JPRB, 0.12285E+00_JPRB, 0.12281E+00_JPRB, 0.12279E+00_JPRB, 0.12278E+00_JPRB ,&
&  0.12277E+00_JPRB, 0.12284E+00_JPRB, 0.12285E+00_JPRB, 0.12289E+00_JPRB, 0.12294E+00_JPRB, 0.12299E+00_JPRB, 0.12303E+00_JPRB ,&
&  0.12310E+00_JPRB, 0.12316E+00_JPRB, 0.12324E+00_JPRB, 0.12336E+00_JPRB, 0.12350E+00_JPRB, 0.12368E+00_JPRB, 0.12392E+00_JPRB ,&
&  0.12417E+00_JPRB, 0.12443E+00_JPRB, 0.12469E+00_JPRB, 0.12494E+00_JPRB, 0.12533E+00_JPRB, 0.12572E+00_JPRB, 0.12617E+00_JPRB ,&
&  0.12662E+00_JPRB, 0.12707E+00_JPRB, 0.12747E+00_JPRB, 0.12790E+00_JPRB, 0.12844E+00_JPRB, 0.12872E+00_JPRB, 0.12896E+00_JPRB ,&
&  0.12932E+00_JPRB, 0.12969E+00_JPRB, 0.13007E+00_JPRB, 0.13040E+00_JPRB, 0.13095E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.13668E+00_JPRB, 0.13705E+00_JPRB, 0.13667E+00_JPRB, 0.13631E+00_JPRB, 0.13136E+00_JPRB, 0.13716E+00_JPRB, 0.13644E+00_JPRB ,&
&  0.13669E+00_JPRB, 0.13623E+00_JPRB, 0.13717E+00_JPRB, 0.13713E+00_JPRB, 0.13671E+00_JPRB, 0.13641E+00_JPRB, 0.13607E+00_JPRB ,&
&  0.13570E+00_JPRB, 0.13499E+00_JPRB, 0.13458E+00_JPRB, 0.13406E+00_JPRB, 0.13355E+00_JPRB, 0.13276E+00_JPRB, 0.13194E+00_JPRB ,&
&  0.13159E+00_JPRB, 0.13167E+00_JPRB, 0.13202E+00_JPRB, 0.13080E+00_JPRB, 0.13032E+00_JPRB, 0.13003E+00_JPRB, 0.12991E+00_JPRB ,&
&  0.12966E+00_JPRB, 0.12964E+00_JPRB, 0.12967E+00_JPRB, 0.12962E+00_JPRB, 0.13062E+00_JPRB, 0.13091E+00_JPRB, 0.13114E+00_JPRB ,&
&  0.13116E+00_JPRB, 0.13099E+00_JPRB, 0.13037E+00_JPRB, 0.12982E+00_JPRB, 0.12927E+00_JPRB, 0.12834E+00_JPRB, 0.12735E+00_JPRB ,&
&  0.12651E+00_JPRB, 0.12585E+00_JPRB, 0.12512E+00_JPRB, 0.12455E+00_JPRB, 0.12408E+00_JPRB, 0.12363E+00_JPRB, 0.12334E+00_JPRB ,&
&  0.12324E+00_JPRB, 0.12321E+00_JPRB, 0.12316E+00_JPRB, 0.12311E+00_JPRB, 0.12302E+00_JPRB, 0.12295E+00_JPRB, 0.12287E+00_JPRB ,&
&  0.12303E+00_JPRB, 0.12295E+00_JPRB, 0.12278E+00_JPRB, 0.12251E+00_JPRB, 0.12236E+00_JPRB, 0.12205E+00_JPRB, 0.12183E+00_JPRB ,&
&  0.12165E+00_JPRB, 0.12144E+00_JPRB, 0.12134E+00_JPRB, 0.12130E+00_JPRB, 0.12127E+00_JPRB, 0.12125E+00_JPRB, 0.12132E+00_JPRB ,&
&  0.12143E+00_JPRB, 0.12159E+00_JPRB, 0.12190E+00_JPRB, 0.12216E+00_JPRB, 0.12241E+00_JPRB, 0.12262E+00_JPRB, 0.12304E+00_JPRB ,&
&  0.12351E+00_JPRB, 0.12396E+00_JPRB, 0.12442E+00_JPRB, 0.12474E+00_JPRB, 0.12538E+00_JPRB, 0.12602E+00_JPRB, 0.12672E+00_JPRB ,&
&  0.12745E+00_JPRB, 0.12837E+00_JPRB, 0.12932E+00_JPRB, 0.13026E+00_JPRB, 0.13148E+00_JPRB, 0.13248E+00_JPRB, 0.13345E+00_JPRB ,&
&  0.13452E+00_JPRB, 0.13650E+00_JPRB, 0.13714E+00_JPRB, 0.13741E+00_JPRB, 0.13720E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.15113E+00_JPRB, 0.15178E+00_JPRB, 0.15210E+00_JPRB, 0.15249E+00_JPRB, 0.15340E+00_JPRB, 0.15342E+00_JPRB, 0.15326E+00_JPRB ,&
&  0.15140E+00_JPRB, 0.15062E+00_JPRB, 0.14940E+00_JPRB, 0.14775E+00_JPRB, 0.14609E+00_JPRB, 0.14347E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.13999E+00_JPRB, 0.13816E+00_JPRB, 0.13691E+00_JPRB, 0.13536E+00_JPRB, 0.13404E+00_JPRB, 0.13260E+00_JPRB, 0.13209E+00_JPRB ,&
&  0.13140E+00_JPRB, 0.13137E+00_JPRB, 0.13208E+00_JPRB, 0.13271E+00_JPRB, 0.13381E+00_JPRB, 0.13499E+00_JPRB, 0.13649E+00_JPRB ,&
&  0.13702E+00_JPRB, 0.13785E+00_JPRB, 0.13860E+00_JPRB, 0.13907E+00_JPRB, 0.14054E+00_JPRB, 0.14161E+00_JPRB, 0.14197E+00_JPRB ,&
&  0.14154E+00_JPRB, 0.14080E+00_JPRB, 0.13931E+00_JPRB, 0.13762E+00_JPRB, 0.13562E+00_JPRB, 0.13340E+00_JPRB, 0.13127E+00_JPRB ,&
&  0.12960E+00_JPRB, 0.12806E+00_JPRB, 0.12689E+00_JPRB, 0.12594E+00_JPRB, 0.12533E+00_JPRB, 0.12486E+00_JPRB, 0.12440E+00_JPRB ,&
&  0.12411E+00_JPRB, 0.12387E+00_JPRB, 0.12352E+00_JPRB, 0.12346E+00_JPRB, 0.12333E+00_JPRB, 0.12333E+00_JPRB, 0.12332E+00_JPRB ,&
&  0.12331E+00_JPRB, 0.12322E+00_JPRB, 0.12306E+00_JPRB, 0.12275E+00_JPRB, 0.12243E+00_JPRB, 0.12211E+00_JPRB, 0.12178E+00_JPRB ,&
&  0.12142E+00_JPRB, 0.12135E+00_JPRB, 0.12108E+00_JPRB, 0.12088E+00_JPRB, 0.12063E+00_JPRB, 0.12025E+00_JPRB, 0.12014E+00_JPRB ,&
&  0.12023E+00_JPRB, 0.12036E+00_JPRB, 0.12029E+00_JPRB, 0.12044E+00_JPRB, 0.12063E+00_JPRB, 0.12092E+00_JPRB, 0.12161E+00_JPRB ,&
&  0.12205E+00_JPRB, 0.12252E+00_JPRB, 0.12296E+00_JPRB, 0.12357E+00_JPRB, 0.12458E+00_JPRB, 0.12559E+00_JPRB, 0.12672E+00_JPRB ,&
&  0.12807E+00_JPRB, 0.12935E+00_JPRB, 0.13063E+00_JPRB, 0.13189E+00_JPRB, 0.13396E+00_JPRB, 0.13549E+00_JPRB, 0.13751E+00_JPRB ,&
&  0.13931E+00_JPRB, 0.13878E+00_JPRB, 0.14629E+00_JPRB, 0.14719E+00_JPRB, 0.14849E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.15728E+00_JPRB, 0.15748E+00_JPRB, 0.15737E+00_JPRB, 0.15822E+00_JPRB, 0.15903E+00_JPRB, 0.15990E+00_JPRB, 0.15968E+00_JPRB ,&
&  0.15944E+00_JPRB, 0.15786E+00_JPRB, 0.15663E+00_JPRB, 0.15383E+00_JPRB, 0.15157E+00_JPRB, 0.14893E+00_JPRB, 0.14664E+00_JPRB ,&
&  0.14460E+00_JPRB, 0.14268E+00_JPRB, 0.14090E+00_JPRB, 0.13939E+00_JPRB, 0.13802E+00_JPRB, 0.13687E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13414E+00_JPRB, 0.13371E+00_JPRB, 0.13377E+00_JPRB, 0.13492E+00_JPRB, 0.13721E+00_JPRB, 0.13881E+00_JPRB, 0.14072E+00_JPRB ,&
&  0.14311E+00_JPRB, 0.14280E+00_JPRB, 0.14543E+00_JPRB, 0.14799E+00_JPRB, 0.15008E+00_JPRB, 0.15171E+00_JPRB, 0.15217E+00_JPRB ,&
&  0.15280E+00_JPRB, 0.14898E+00_JPRB, 0.15034E+00_JPRB, 0.14703E+00_JPRB, 0.14283E+00_JPRB, 0.14071E+00_JPRB, 0.13696E+00_JPRB ,&
&  0.13501E+00_JPRB, 0.13212E+00_JPRB, 0.13023E+00_JPRB, 0.12870E+00_JPRB, 0.12699E+00_JPRB, 0.12653E+00_JPRB, 0.12578E+00_JPRB ,&
&  0.12531E+00_JPRB, 0.12530E+00_JPRB, 0.12462E+00_JPRB, 0.12435E+00_JPRB, 0.12391E+00_JPRB, 0.12347E+00_JPRB, 0.12369E+00_JPRB ,&
&  0.12409E+00_JPRB, 0.12462E+00_JPRB, 0.12516E+00_JPRB, 0.12523E+00_JPRB, 0.12518E+00_JPRB, 0.12515E+00_JPRB, 0.12508E+00_JPRB ,&
&  0.12527E+00_JPRB, 0.12527E+00_JPRB, 0.12491E+00_JPRB, 0.12450E+00_JPRB, 0.12359E+00_JPRB, 0.12270E+00_JPRB, 0.12182E+00_JPRB ,&
&  0.12106E+00_JPRB, 0.12065E+00_JPRB, 0.12031E+00_JPRB, 0.12024E+00_JPRB, 0.12030E+00_JPRB, 0.12060E+00_JPRB, 0.12091E+00_JPRB ,&
&  0.12120E+00_JPRB, 0.12183E+00_JPRB, 0.12231E+00_JPRB, 0.12307E+00_JPRB, 0.12399E+00_JPRB, 0.12482E+00_JPRB, 0.12608E+00_JPRB ,&
&  0.12745E+00_JPRB, 0.12894E+00_JPRB, 0.13104E+00_JPRB, 0.13291E+00_JPRB, 0.13525E+00_JPRB, 0.13833E+00_JPRB, 0.14069E+00_JPRB ,&
&  0.14327E+00_JPRB, 0.14607E+00_JPRB, 0.14669E+00_JPRB, 0.15297E+00_JPRB, 0.15648E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.15615E+00_JPRB, 0.15740E+00_JPRB, 0.15880E+00_JPRB, 0.16018E+00_JPRB, 0.16140E+00_JPRB, 0.16262E+00_JPRB, 0.16341E+00_JPRB ,&
&  0.16122E+00_JPRB, 0.15985E+00_JPRB, 0.15736E+00_JPRB, 0.15500E+00_JPRB, 0.15234E+00_JPRB, 0.14950E+00_JPRB, 0.14844E+00_JPRB ,&
&  0.14708E+00_JPRB, 0.14678E+00_JPRB, 0.14573E+00_JPRB, 0.14529E+00_JPRB, 0.14446E+00_JPRB, 0.14402E+00_JPRB, 0.14326E+00_JPRB ,&
&  0.14162E+00_JPRB, 0.14075E+00_JPRB, 0.13991E+00_JPRB, 0.14031E+00_JPRB, 0.14168E+00_JPRB, 0.14319E+00_JPRB, 0.14480E+00_JPRB ,&
&  0.14708E+00_JPRB, 0.14826E+00_JPRB, 0.15048E+00_JPRB, 0.15210E+00_JPRB, 0.15464E+00_JPRB, 0.15675E+00_JPRB, 0.15717E+00_JPRB ,&
&  0.15725E+00_JPRB, 0.15601E+00_JPRB, 0.15581E+00_JPRB, 0.15421E+00_JPRB, 0.14879E+00_JPRB, 0.14558E+00_JPRB, 0.14114E+00_JPRB ,&
&  0.13620E+00_JPRB, 0.13246E+00_JPRB, 0.12992E+00_JPRB, 0.12903E+00_JPRB, 0.12822E+00_JPRB, 0.12793E+00_JPRB, 0.12767E+00_JPRB ,&
&  0.12694E+00_JPRB, 0.12610E+00_JPRB, 0.12512E+00_JPRB, 0.12453E+00_JPRB, 0.12478E+00_JPRB, 0.12537E+00_JPRB, 0.12666E+00_JPRB ,&
&  0.12798E+00_JPRB, 0.12871E+00_JPRB, 0.12952E+00_JPRB, 0.12979E+00_JPRB, 0.12990E+00_JPRB, 0.13010E+00_JPRB, 0.13016E+00_JPRB ,&
&  0.13071E+00_JPRB, 0.13085E+00_JPRB, 0.13040E+00_JPRB, 0.12965E+00_JPRB, 0.12780E+00_JPRB, 0.12538E+00_JPRB, 0.12344E+00_JPRB ,&
&  0.12180E+00_JPRB, 0.12074E+00_JPRB, 0.12035E+00_JPRB, 0.11999E+00_JPRB, 0.12021E+00_JPRB, 0.12054E+00_JPRB, 0.12086E+00_JPRB ,&
&  0.12140E+00_JPRB, 0.12187E+00_JPRB, 0.12233E+00_JPRB, 0.12332E+00_JPRB, 0.12413E+00_JPRB, 0.12533E+00_JPRB, 0.12657E+00_JPRB ,&
&  0.12744E+00_JPRB, 0.12878E+00_JPRB, 0.12989E+00_JPRB, 0.13113E+00_JPRB, 0.13477E+00_JPRB, 0.13774E+00_JPRB, 0.14377E+00_JPRB ,&
&  0.14793E+00_JPRB, 0.15021E+00_JPRB, 0.15187E+00_JPRB, 0.15630E+00_JPRB, 0.15597E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.15627E+00_JPRB, 0.15797E+00_JPRB, 0.15991E+00_JPRB, 0.16138E+00_JPRB, 0.16280E+00_JPRB, 0.16401E+00_JPRB, 0.16465E+00_JPRB ,&
&  0.16478E+00_JPRB, 0.16424E+00_JPRB, 0.16430E+00_JPRB, 0.16328E+00_JPRB, 0.16231E+00_JPRB, 0.15821E+00_JPRB, 0.15748E+00_JPRB ,&
&  0.15644E+00_JPRB, 0.15686E+00_JPRB, 0.15645E+00_JPRB, 0.15663E+00_JPRB, 0.15603E+00_JPRB, 0.15575E+00_JPRB, 0.15451E+00_JPRB ,&
&  0.15265E+00_JPRB, 0.15118E+00_JPRB, 0.14970E+00_JPRB, 0.14919E+00_JPRB, 0.14986E+00_JPRB, 0.15041E+00_JPRB, 0.15021E+00_JPRB ,&
&  0.15025E+00_JPRB, 0.15059E+00_JPRB, 0.15109E+00_JPRB, 0.15130E+00_JPRB, 0.15624E+00_JPRB, 0.15826E+00_JPRB, 0.15936E+00_JPRB ,&
&  0.15943E+00_JPRB, 0.16002E+00_JPRB, 0.16030E+00_JPRB, 0.15972E+00_JPRB, 0.15909E+00_JPRB, 0.15573E+00_JPRB, 0.15345E+00_JPRB ,&
&  0.14588E+00_JPRB, 0.14046E+00_JPRB, 0.13632E+00_JPRB, 0.13376E+00_JPRB, 0.13269E+00_JPRB, 0.13251E+00_JPRB, 0.13194E+00_JPRB ,&
&  0.13052E+00_JPRB, 0.12924E+00_JPRB, 0.12843E+00_JPRB, 0.12894E+00_JPRB, 0.12981E+00_JPRB, 0.13083E+00_JPRB, 0.13199E+00_JPRB ,&
&  0.13306E+00_JPRB, 0.13332E+00_JPRB, 0.13331E+00_JPRB, 0.13364E+00_JPRB, 0.13390E+00_JPRB, 0.13430E+00_JPRB, 0.13463E+00_JPRB ,&
&  0.13548E+00_JPRB, 0.13490E+00_JPRB, 0.13541E+00_JPRB, 0.13629E+00_JPRB, 0.13429E+00_JPRB, 0.13108E+00_JPRB, 0.12784E+00_JPRB ,&
&  0.12489E+00_JPRB, 0.12322E+00_JPRB, 0.12191E+00_JPRB, 0.12110E+00_JPRB, 0.12093E+00_JPRB, 0.12108E+00_JPRB, 0.12159E+00_JPRB ,&
&  0.12224E+00_JPRB, 0.12310E+00_JPRB, 0.12369E+00_JPRB, 0.12441E+00_JPRB, 0.12527E+00_JPRB, 0.12617E+00_JPRB, 0.12723E+00_JPRB ,&
&  0.12854E+00_JPRB, 0.12979E+00_JPRB, 0.13066E+00_JPRB, 0.13194E+00_JPRB, 0.13435E+00_JPRB, 0.13765E+00_JPRB, 0.14272E+00_JPRB ,&
&  0.14859E+00_JPRB, 0.15403E+00_JPRB, 0.15251E+00_JPRB, 0.15416E+00_JPRB, 0.15557E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.16153E+00_JPRB, 0.16376E+00_JPRB, 0.16544E+00_JPRB, 0.16697E+00_JPRB, 0.16812E+00_JPRB, 0.16943E+00_JPRB, 0.17002E+00_JPRB ,&
&  0.17055E+00_JPRB, 0.17054E+00_JPRB, 0.17090E+00_JPRB, 0.17082E+00_JPRB, 0.17094E+00_JPRB, 0.16991E+00_JPRB, 0.16928E+00_JPRB ,&
&  0.16884E+00_JPRB, 0.16903E+00_JPRB, 0.16890E+00_JPRB, 0.16865E+00_JPRB, 0.16788E+00_JPRB, 0.16692E+00_JPRB, 0.16551E+00_JPRB ,&
&  0.16274E+00_JPRB, 0.16103E+00_JPRB, 0.16145E+00_JPRB, 0.16115E+00_JPRB, 0.16158E+00_JPRB, 0.16170E+00_JPRB, 0.16215E+00_JPRB ,&
&  0.15855E+00_JPRB, 0.15708E+00_JPRB, 0.15690E+00_JPRB, 0.15614E+00_JPRB, 0.15878E+00_JPRB, 0.15994E+00_JPRB, 0.16169E+00_JPRB ,&
&  0.16236E+00_JPRB, 0.16289E+00_JPRB, 0.16366E+00_JPRB, 0.16342E+00_JPRB, 0.16386E+00_JPRB, 0.16430E+00_JPRB, 0.16328E+00_JPRB ,&
&  0.16066E+00_JPRB, 0.15847E+00_JPRB, 0.15548E+00_JPRB, 0.14797E+00_JPRB, 0.14574E+00_JPRB, 0.14566E+00_JPRB, 0.14413E+00_JPRB ,&
&  0.14155E+00_JPRB, 0.14157E+00_JPRB, 0.14115E+00_JPRB, 0.13989E+00_JPRB, 0.14137E+00_JPRB, 0.14354E+00_JPRB, 0.14369E+00_JPRB ,&
&  0.14271E+00_JPRB, 0.14247E+00_JPRB, 0.14190E+00_JPRB, 0.14080E+00_JPRB, 0.13949E+00_JPRB, 0.13892E+00_JPRB, 0.13843E+00_JPRB ,&
&  0.13822E+00_JPRB, 0.13728E+00_JPRB, 0.14041E+00_JPRB, 0.14143E+00_JPRB, 0.13725E+00_JPRB, 0.13812E+00_JPRB, 0.13582E+00_JPRB ,&
&  0.13019E+00_JPRB, 0.12759E+00_JPRB, 0.12622E+00_JPRB, 0.12512E+00_JPRB, 0.12497E+00_JPRB, 0.12519E+00_JPRB, 0.12519E+00_JPRB ,&
&  0.12601E+00_JPRB, 0.12701E+00_JPRB, 0.12751E+00_JPRB, 0.12809E+00_JPRB, 0.12888E+00_JPRB, 0.12999E+00_JPRB, 0.13145E+00_JPRB ,&
&  0.13269E+00_JPRB, 0.13445E+00_JPRB, 0.13541E+00_JPRB, 0.13696E+00_JPRB, 0.13813E+00_JPRB, 0.13952E+00_JPRB, 0.14274E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.15102E+00_JPRB, 0.15861E+00_JPRB, 0.16150E+00_JPRB, 0.15947E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.16684E+00_JPRB, 0.16898E+00_JPRB, 0.17165E+00_JPRB, 0.17439E+00_JPRB, 0.17646E+00_JPRB, 0.17851E+00_JPRB, 0.18030E+00_JPRB ,&
&  0.18184E+00_JPRB, 0.18287E+00_JPRB, 0.18351E+00_JPRB, 0.18380E+00_JPRB, 0.18395E+00_JPRB, 0.18356E+00_JPRB, 0.18273E+00_JPRB ,&
&  0.18188E+00_JPRB, 0.18048E+00_JPRB, 0.17873E+00_JPRB, 0.17727E+00_JPRB, 0.17627E+00_JPRB, 0.17510E+00_JPRB, 0.17347E+00_JPRB ,&
&  0.17175E+00_JPRB, 0.17028E+00_JPRB, 0.16926E+00_JPRB, 0.16844E+00_JPRB, 0.16856E+00_JPRB, 0.16917E+00_JPRB, 0.16967E+00_JPRB ,&
&  0.16663E+00_JPRB, 0.16171E+00_JPRB, 0.16185E+00_JPRB, 0.16284E+00_JPRB, 0.16234E+00_JPRB, 0.16372E+00_JPRB, 0.16461E+00_JPRB ,&
&  0.16583E+00_JPRB, 0.16731E+00_JPRB, 0.16867E+00_JPRB, 0.16942E+00_JPRB, 0.16995E+00_JPRB, 0.17066E+00_JPRB, 0.17007E+00_JPRB ,&
&  0.16820E+00_JPRB, 0.16652E+00_JPRB, 0.16499E+00_JPRB, 0.16346E+00_JPRB, 0.16303E+00_JPRB, 0.16423E+00_JPRB, 0.16489E+00_JPRB ,&
&  0.16398E+00_JPRB, 0.16270E+00_JPRB, 0.16152E+00_JPRB, 0.16164E+00_JPRB, 0.16226E+00_JPRB, 0.16256E+00_JPRB, 0.16086E+00_JPRB ,&
&  0.15934E+00_JPRB, 0.15678E+00_JPRB, 0.15431E+00_JPRB, 0.15193E+00_JPRB, 0.15055E+00_JPRB, 0.14748E+00_JPRB, 0.14431E+00_JPRB ,&
&  0.14357E+00_JPRB, 0.14687E+00_JPRB, 0.15229E+00_JPRB, 0.15499E+00_JPRB, 0.14816E+00_JPRB, 0.14678E+00_JPRB, 0.14588E+00_JPRB ,&
&  0.14158E+00_JPRB, 0.13711E+00_JPRB, 0.13399E+00_JPRB, 0.13268E+00_JPRB, 0.13223E+00_JPRB, 0.13188E+00_JPRB, 0.13238E+00_JPRB ,&
&  0.13328E+00_JPRB, 0.13388E+00_JPRB, 0.13451E+00_JPRB, 0.13555E+00_JPRB, 0.13653E+00_JPRB, 0.13750E+00_JPRB, 0.13833E+00_JPRB ,&
&  0.14005E+00_JPRB, 0.14219E+00_JPRB, 0.14409E+00_JPRB, 0.14671E+00_JPRB, 0.14951E+00_JPRB, 0.15237E+00_JPRB, 0.15539E+00_JPRB ,&
&  0.15874E+00_JPRB, 0.16110E+00_JPRB, 0.16243E+00_JPRB, 0.16467E+00_JPRB, 0.16571E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.17851E+00_JPRB, 0.18345E+00_JPRB, 0.18787E+00_JPRB, 0.19041E+00_JPRB, 0.19223E+00_JPRB, 0.19478E+00_JPRB, 0.19726E+00_JPRB ,&
&  0.19893E+00_JPRB, 0.19979E+00_JPRB, 0.20009E+00_JPRB, 0.19983E+00_JPRB, 0.19879E+00_JPRB, 0.19720E+00_JPRB, 0.19543E+00_JPRB ,&
&  0.19353E+00_JPRB, 0.19094E+00_JPRB, 0.18858E+00_JPRB, 0.18674E+00_JPRB, 0.18516E+00_JPRB, 0.18330E+00_JPRB, 0.18181E+00_JPRB ,&
&  0.18018E+00_JPRB, 0.17946E+00_JPRB, 0.17874E+00_JPRB, 0.17822E+00_JPRB, 0.17791E+00_JPRB, 0.17724E+00_JPRB, 0.17662E+00_JPRB ,&
&  0.17496E+00_JPRB, 0.17207E+00_JPRB, 0.17044E+00_JPRB, 0.16998E+00_JPRB, 0.17014E+00_JPRB, 0.16977E+00_JPRB, 0.16932E+00_JPRB ,&
&  0.17110E+00_JPRB, 0.17394E+00_JPRB, 0.17596E+00_JPRB, 0.17708E+00_JPRB, 0.17781E+00_JPRB, 0.17876E+00_JPRB, 0.17993E+00_JPRB ,&
&  0.17789E+00_JPRB, 0.17542E+00_JPRB, 0.17359E+00_JPRB, 0.17254E+00_JPRB, 0.17228E+00_JPRB, 0.17285E+00_JPRB, 0.17416E+00_JPRB ,&
&  0.17535E+00_JPRB, 0.17597E+00_JPRB, 0.17556E+00_JPRB, 0.17436E+00_JPRB, 0.17295E+00_JPRB, 0.17227E+00_JPRB, 0.17184E+00_JPRB ,&
&  0.17164E+00_JPRB, 0.17142E+00_JPRB, 0.17108E+00_JPRB, 0.17023E+00_JPRB, 0.16578E+00_JPRB, 0.16174E+00_JPRB, 0.15914E+00_JPRB ,&
&  0.15958E+00_JPRB, 0.16030E+00_JPRB, 0.16043E+00_JPRB, 0.16459E+00_JPRB, 0.16917E+00_JPRB, 0.16545E+00_JPRB, 0.16065E+00_JPRB ,&
&  0.15430E+00_JPRB, 0.15363E+00_JPRB, 0.15099E+00_JPRB, 0.14698E+00_JPRB, 0.14562E+00_JPRB, 0.14555E+00_JPRB, 0.14534E+00_JPRB ,&
&  0.14609E+00_JPRB, 0.14655E+00_JPRB, 0.14775E+00_JPRB, 0.14837E+00_JPRB, 0.14868E+00_JPRB, 0.14925E+00_JPRB, 0.14986E+00_JPRB ,&
&  0.15136E+00_JPRB, 0.15447E+00_JPRB, 0.15850E+00_JPRB, 0.16067E+00_JPRB, 0.16292E+00_JPRB, 0.16514E+00_JPRB, 0.16667E+00_JPRB ,&
&  0.16789E+00_JPRB, 0.16955E+00_JPRB, 0.17113E+00_JPRB, 0.17239E+00_JPRB, 0.17462E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.19536E+00_JPRB, 0.19970E+00_JPRB, 0.20237E+00_JPRB, 0.20496E+00_JPRB, 0.20774E+00_JPRB, 0.21222E+00_JPRB, 0.21529E+00_JPRB ,&
&  0.21585E+00_JPRB, 0.21574E+00_JPRB, 0.21518E+00_JPRB, 0.21395E+00_JPRB, 0.21212E+00_JPRB, 0.21049E+00_JPRB, 0.20784E+00_JPRB ,&
&  0.20459E+00_JPRB, 0.20162E+00_JPRB, 0.19896E+00_JPRB, 0.19644E+00_JPRB, 0.19409E+00_JPRB, 0.19193E+00_JPRB, 0.19018E+00_JPRB ,&
&  0.18949E+00_JPRB, 0.18900E+00_JPRB, 0.18795E+00_JPRB, 0.18763E+00_JPRB, 0.18778E+00_JPRB, 0.18742E+00_JPRB, 0.18640E+00_JPRB ,&
&  0.18509E+00_JPRB, 0.18268E+00_JPRB, 0.18019E+00_JPRB, 0.17909E+00_JPRB, 0.17936E+00_JPRB, 0.17881E+00_JPRB, 0.17807E+00_JPRB ,&
&  0.17944E+00_JPRB, 0.18157E+00_JPRB, 0.18384E+00_JPRB, 0.18495E+00_JPRB, 0.18557E+00_JPRB, 0.18654E+00_JPRB, 0.18781E+00_JPRB ,&
&  0.18805E+00_JPRB, 0.18583E+00_JPRB, 0.18380E+00_JPRB, 0.18329E+00_JPRB, 0.18303E+00_JPRB, 0.18260E+00_JPRB, 0.18281E+00_JPRB ,&
&  0.18330E+00_JPRB, 0.18444E+00_JPRB, 0.18522E+00_JPRB, 0.18439E+00_JPRB, 0.18291E+00_JPRB, 0.18183E+00_JPRB, 0.18110E+00_JPRB ,&
&  0.18069E+00_JPRB, 0.18019E+00_JPRB, 0.17931E+00_JPRB, 0.17794E+00_JPRB, 0.17649E+00_JPRB, 0.17521E+00_JPRB, 0.17398E+00_JPRB ,&
&  0.17292E+00_JPRB, 0.17313E+00_JPRB, 0.17503E+00_JPRB, 0.17647E+00_JPRB, 0.17645E+00_JPRB, 0.17523E+00_JPRB, 0.17320E+00_JPRB ,&
&  0.17173E+00_JPRB, 0.16976E+00_JPRB, 0.16821E+00_JPRB, 0.16698E+00_JPRB, 0.16598E+00_JPRB, 0.16461E+00_JPRB, 0.16407E+00_JPRB ,&
&  0.16423E+00_JPRB, 0.16562E+00_JPRB, 0.16630E+00_JPRB, 0.16669E+00_JPRB, 0.16655E+00_JPRB, 0.16641E+00_JPRB, 0.16726E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.17012E+00_JPRB, 0.17188E+00_JPRB, 0.17421E+00_JPRB, 0.17663E+00_JPRB, 0.17981E+00_JPRB, 0.18248E+00_JPRB ,&
&  0.18412E+00_JPRB, 0.18464E+00_JPRB, 0.18583E+00_JPRB, 0.18755E+00_JPRB, 0.19051E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.21106E+00_JPRB, 0.21425E+00_JPRB, 0.21694E+00_JPRB, 0.22045E+00_JPRB, 0.22357E+00_JPRB, 0.22806E+00_JPRB, 0.22989E+00_JPRB ,&
&  0.22961E+00_JPRB, 0.22899E+00_JPRB, 0.22728E+00_JPRB, 0.22473E+00_JPRB, 0.22094E+00_JPRB, 0.21718E+00_JPRB, 0.21372E+00_JPRB ,&
&  0.21112E+00_JPRB, 0.20896E+00_JPRB, 0.20721E+00_JPRB, 0.20560E+00_JPRB, 0.20335E+00_JPRB, 0.20123E+00_JPRB, 0.19956E+00_JPRB ,&
&  0.19867E+00_JPRB, 0.19779E+00_JPRB, 0.19730E+00_JPRB, 0.19702E+00_JPRB, 0.19686E+00_JPRB, 0.19685E+00_JPRB, 0.19699E+00_JPRB ,&
&  0.19611E+00_JPRB, 0.19309E+00_JPRB, 0.19142E+00_JPRB, 0.19095E+00_JPRB, 0.19060E+00_JPRB, 0.19115E+00_JPRB, 0.19109E+00_JPRB ,&
&  0.19111E+00_JPRB, 0.19198E+00_JPRB, 0.19359E+00_JPRB, 0.19531E+00_JPRB, 0.19539E+00_JPRB, 0.19581E+00_JPRB, 0.19687E+00_JPRB ,&
&  0.19732E+00_JPRB, 0.19680E+00_JPRB, 0.19517E+00_JPRB, 0.19450E+00_JPRB, 0.19484E+00_JPRB, 0.19465E+00_JPRB, 0.19493E+00_JPRB ,&
&  0.19553E+00_JPRB, 0.19652E+00_JPRB, 0.19696E+00_JPRB, 0.19557E+00_JPRB, 0.19424E+00_JPRB, 0.19375E+00_JPRB, 0.19369E+00_JPRB ,&
&  0.19332E+00_JPRB, 0.19235E+00_JPRB, 0.19045E+00_JPRB, 0.18885E+00_JPRB, 0.18639E+00_JPRB, 0.18440E+00_JPRB, 0.18341E+00_JPRB ,&
&  0.18395E+00_JPRB, 0.18442E+00_JPRB, 0.18569E+00_JPRB, 0.18686E+00_JPRB, 0.18759E+00_JPRB, 0.18773E+00_JPRB, 0.18618E+00_JPRB ,&
&  0.18411E+00_JPRB, 0.18319E+00_JPRB, 0.18229E+00_JPRB, 0.18149E+00_JPRB, 0.18161E+00_JPRB, 0.18150E+00_JPRB, 0.18109E+00_JPRB ,&
&  0.18058E+00_JPRB, 0.18083E+00_JPRB, 0.18123E+00_JPRB, 0.18145E+00_JPRB, 0.18136E+00_JPRB, 0.18084E+00_JPRB, 0.18140E+00_JPRB ,&
&  0.18269E+00_JPRB, 0.18447E+00_JPRB, 0.18655E+00_JPRB, 0.18883E+00_JPRB, 0.19136E+00_JPRB, 0.19370E+00_JPRB, 0.19592E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19994E+00_JPRB, 0.20172E+00_JPRB, 0.20448E+00_JPRB, 0.20726E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.22408E+00_JPRB, 0.22677E+00_JPRB, 0.23118E+00_JPRB, 0.23600E+00_JPRB, 0.23936E+00_JPRB, 0.24327E+00_JPRB, 0.24381E+00_JPRB ,&
&  0.24287E+00_JPRB, 0.24092E+00_JPRB, 0.23694E+00_JPRB, 0.23217E+00_JPRB, 0.22782E+00_JPRB, 0.22431E+00_JPRB, 0.22194E+00_JPRB ,&
&  0.22057E+00_JPRB, 0.21939E+00_JPRB, 0.21801E+00_JPRB, 0.21572E+00_JPRB, 0.21283E+00_JPRB, 0.21017E+00_JPRB, 0.20798E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20665E+00_JPRB, 0.20678E+00_JPRB, 0.20656E+00_JPRB, 0.20654E+00_JPRB, 0.20792E+00_JPRB, 0.20959E+00_JPRB ,&
&  0.20869E+00_JPRB, 0.20475E+00_JPRB, 0.20244E+00_JPRB, 0.20233E+00_JPRB, 0.20262E+00_JPRB, 0.20330E+00_JPRB, 0.20419E+00_JPRB ,&
&  0.20472E+00_JPRB, 0.20572E+00_JPRB, 0.20711E+00_JPRB, 0.20878E+00_JPRB, 0.20957E+00_JPRB, 0.21177E+00_JPRB, 0.21297E+00_JPRB ,&
&  0.21312E+00_JPRB, 0.21246E+00_JPRB, 0.21112E+00_JPRB, 0.20852E+00_JPRB, 0.20657E+00_JPRB, 0.20581E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.20565E+00_JPRB, 0.20626E+00_JPRB, 0.20679E+00_JPRB, 0.20610E+00_JPRB, 0.20460E+00_JPRB, 0.20440E+00_JPRB, 0.20482E+00_JPRB ,&
&  0.20469E+00_JPRB, 0.20351E+00_JPRB, 0.20186E+00_JPRB, 0.20093E+00_JPRB, 0.20001E+00_JPRB, 0.19981E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.19967E+00_JPRB, 0.19887E+00_JPRB, 0.19839E+00_JPRB, 0.19823E+00_JPRB, 0.19816E+00_JPRB, 0.19856E+00_JPRB, 0.19827E+00_JPRB ,&
&  0.19823E+00_JPRB, 0.19841E+00_JPRB, 0.19751E+00_JPRB, 0.19729E+00_JPRB, 0.19729E+00_JPRB, 0.19761E+00_JPRB, 0.19770E+00_JPRB ,&
&  0.19744E+00_JPRB, 0.19703E+00_JPRB, 0.19670E+00_JPRB, 0.19677E+00_JPRB, 0.19646E+00_JPRB, 0.19602E+00_JPRB, 0.19681E+00_JPRB ,&
&  0.19975E+00_JPRB, 0.20284E+00_JPRB, 0.20550E+00_JPRB, 0.20730E+00_JPRB, 0.20797E+00_JPRB, 0.20939E+00_JPRB, 0.21201E+00_JPRB ,&
&  0.21440E+00_JPRB, 0.21550E+00_JPRB, 0.21743E+00_JPRB, 0.21906E+00_JPRB, 0.22101E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.23903E+00_JPRB, 0.24186E+00_JPRB, 0.24594E+00_JPRB, 0.25179E+00_JPRB, 0.25526E+00_JPRB, 0.25813E+00_JPRB, 0.25762E+00_JPRB ,&
&  0.25513E+00_JPRB, 0.25163E+00_JPRB, 0.24746E+00_JPRB, 0.24248E+00_JPRB, 0.23738E+00_JPRB, 0.23353E+00_JPRB, 0.23211E+00_JPRB ,&
&  0.23197E+00_JPRB, 0.23116E+00_JPRB, 0.22888E+00_JPRB, 0.22569E+00_JPRB, 0.22205E+00_JPRB, 0.21911E+00_JPRB, 0.21673E+00_JPRB ,&
&  0.21577E+00_JPRB, 0.21587E+00_JPRB, 0.21652E+00_JPRB, 0.21687E+00_JPRB, 0.21720E+00_JPRB, 0.21968E+00_JPRB, 0.22351E+00_JPRB ,&
&  0.22146E+00_JPRB, 0.21907E+00_JPRB, 0.21457E+00_JPRB, 0.21419E+00_JPRB, 0.21548E+00_JPRB, 0.21778E+00_JPRB, 0.22004E+00_JPRB ,&
&  0.22193E+00_JPRB, 0.22468E+00_JPRB, 0.22744E+00_JPRB, 0.22974E+00_JPRB, 0.23101E+00_JPRB, 0.23227E+00_JPRB, 0.23312E+00_JPRB ,&
&  0.23332E+00_JPRB, 0.23222E+00_JPRB, 0.23036E+00_JPRB, 0.22706E+00_JPRB, 0.22284E+00_JPRB, 0.22000E+00_JPRB, 0.21846E+00_JPRB ,&
&  0.21682E+00_JPRB, 0.21647E+00_JPRB, 0.21638E+00_JPRB, 0.21586E+00_JPRB, 0.21506E+00_JPRB, 0.21485E+00_JPRB, 0.21535E+00_JPRB ,&
&  0.21561E+00_JPRB, 0.21515E+00_JPRB, 0.21431E+00_JPRB, 0.21418E+00_JPRB, 0.21468E+00_JPRB, 0.21582E+00_JPRB, 0.21693E+00_JPRB ,&
&  0.21771E+00_JPRB, 0.21754E+00_JPRB, 0.21697E+00_JPRB, 0.21539E+00_JPRB, 0.21423E+00_JPRB, 0.21436E+00_JPRB, 0.21542E+00_JPRB ,&
&  0.21606E+00_JPRB, 0.21607E+00_JPRB, 0.21632E+00_JPRB, 0.21549E+00_JPRB, 0.21478E+00_JPRB, 0.21495E+00_JPRB, 0.21543E+00_JPRB ,&
&  0.21562E+00_JPRB, 0.21548E+00_JPRB, 0.21514E+00_JPRB, 0.21492E+00_JPRB, 0.21530E+00_JPRB, 0.21541E+00_JPRB, 0.21634E+00_JPRB ,&
&  0.21871E+00_JPRB, 0.22141E+00_JPRB, 0.22418E+00_JPRB, 0.22630E+00_JPRB, 0.22760E+00_JPRB, 0.22808E+00_JPRB, 0.23033E+00_JPRB ,&
&  0.23349E+00_JPRB, 0.23562E+00_JPRB, 0.23547E+00_JPRB, 0.23509E+00_JPRB, 0.23673E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.25679E+00_JPRB, 0.25785E+00_JPRB, 0.26228E+00_JPRB, 0.26726E+00_JPRB, 0.27231E+00_JPRB, 0.27309E+00_JPRB, 0.27041E+00_JPRB ,&
&  0.26748E+00_JPRB, 0.26358E+00_JPRB, 0.25804E+00_JPRB, 0.25139E+00_JPRB, 0.24531E+00_JPRB, 0.24350E+00_JPRB, 0.24345E+00_JPRB ,&
&  0.24368E+00_JPRB, 0.24246E+00_JPRB, 0.23967E+00_JPRB, 0.23639E+00_JPRB, 0.23245E+00_JPRB, 0.22899E+00_JPRB, 0.22636E+00_JPRB ,&
&  0.22547E+00_JPRB, 0.22503E+00_JPRB, 0.22541E+00_JPRB, 0.22556E+00_JPRB, 0.22642E+00_JPRB, 0.22950E+00_JPRB, 0.23488E+00_JPRB ,&
&  0.23260E+00_JPRB, 0.23309E+00_JPRB, 0.22714E+00_JPRB, 0.22951E+00_JPRB, 0.23162E+00_JPRB, 0.23607E+00_JPRB, 0.24097E+00_JPRB ,&
&  0.24575E+00_JPRB, 0.25033E+00_JPRB, 0.25327E+00_JPRB, 0.25407E+00_JPRB, 0.25281E+00_JPRB, 0.25251E+00_JPRB, 0.25218E+00_JPRB ,&
&  0.25140E+00_JPRB, 0.25034E+00_JPRB, 0.24844E+00_JPRB, 0.24490E+00_JPRB, 0.24002E+00_JPRB, 0.23570E+00_JPRB, 0.23251E+00_JPRB ,&
&  0.23160E+00_JPRB, 0.23108E+00_JPRB, 0.22971E+00_JPRB, 0.22843E+00_JPRB, 0.22714E+00_JPRB, 0.22642E+00_JPRB, 0.22697E+00_JPRB ,&
&  0.22816E+00_JPRB, 0.22918E+00_JPRB, 0.22996E+00_JPRB, 0.23114E+00_JPRB, 0.23314E+00_JPRB, 0.23503E+00_JPRB, 0.23634E+00_JPRB ,&
&  0.23674E+00_JPRB, 0.23628E+00_JPRB, 0.23581E+00_JPRB, 0.23522E+00_JPRB, 0.23484E+00_JPRB, 0.23435E+00_JPRB, 0.23435E+00_JPRB ,&
&  0.23453E+00_JPRB, 0.23453E+00_JPRB, 0.23396E+00_JPRB, 0.23290E+00_JPRB, 0.23198E+00_JPRB, 0.23122E+00_JPRB, 0.23103E+00_JPRB ,&
&  0.23140E+00_JPRB, 0.23182E+00_JPRB, 0.23175E+00_JPRB, 0.23139E+00_JPRB, 0.23164E+00_JPRB, 0.23217E+00_JPRB, 0.23346E+00_JPRB ,&
&  0.23540E+00_JPRB, 0.23798E+00_JPRB, 0.24142E+00_JPRB, 0.24515E+00_JPRB, 0.24649E+00_JPRB, 0.24796E+00_JPRB, 0.25103E+00_JPRB ,&
&  0.25384E+00_JPRB, 0.25490E+00_JPRB, 0.25535E+00_JPRB, 0.25582E+00_JPRB, 0.25684E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.27904E+00_JPRB, 0.27732E+00_JPRB, 0.27949E+00_JPRB, 0.28405E+00_JPRB, 0.28980E+00_JPRB, 0.28805E+00_JPRB, 0.28518E+00_JPRB ,&
&  0.27956E+00_JPRB, 0.27258E+00_JPRB, 0.26587E+00_JPRB, 0.25957E+00_JPRB, 0.25635E+00_JPRB, 0.25628E+00_JPRB, 0.25582E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25210E+00_JPRB, 0.24822E+00_JPRB, 0.24434E+00_JPRB, 0.24147E+00_JPRB, 0.23994E+00_JPRB, 0.23861E+00_JPRB ,&
&  0.23775E+00_JPRB, 0.23675E+00_JPRB, 0.23769E+00_JPRB, 0.23778E+00_JPRB, 0.23877E+00_JPRB, 0.24293E+00_JPRB, 0.24618E+00_JPRB ,&
&  0.24796E+00_JPRB, 0.24981E+00_JPRB, 0.25009E+00_JPRB, 0.25045E+00_JPRB, 0.25317E+00_JPRB, 0.26015E+00_JPRB, 0.26831E+00_JPRB ,&
&  0.27541E+00_JPRB, 0.28048E+00_JPRB, 0.28301E+00_JPRB, 0.28248E+00_JPRB, 0.27982E+00_JPRB, 0.27689E+00_JPRB, 0.27495E+00_JPRB ,&
&  0.27302E+00_JPRB, 0.26956E+00_JPRB, 0.26581E+00_JPRB, 0.26264E+00_JPRB, 0.26107E+00_JPRB, 0.25978E+00_JPRB, 0.25804E+00_JPRB ,&
&  0.25586E+00_JPRB, 0.25379E+00_JPRB, 0.25107E+00_JPRB, 0.24824E+00_JPRB, 0.24579E+00_JPRB, 0.24452E+00_JPRB, 0.24485E+00_JPRB ,&
&  0.24653E+00_JPRB, 0.24909E+00_JPRB, 0.25215E+00_JPRB, 0.25574E+00_JPRB, 0.25838E+00_JPRB, 0.26050E+00_JPRB, 0.26116E+00_JPRB ,&
&  0.26116E+00_JPRB, 0.26082E+00_JPRB, 0.26044E+00_JPRB, 0.26006E+00_JPRB, 0.25940E+00_JPRB, 0.25830E+00_JPRB, 0.25702E+00_JPRB ,&
&  0.25589E+00_JPRB, 0.25508E+00_JPRB, 0.25446E+00_JPRB, 0.25391E+00_JPRB, 0.25248E+00_JPRB, 0.25061E+00_JPRB, 0.24938E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24854E+00_JPRB, 0.24838E+00_JPRB, 0.24790E+00_JPRB, 0.24771E+00_JPRB, 0.24836E+00_JPRB, 0.24996E+00_JPRB ,&
&  0.25260E+00_JPRB, 0.25631E+00_JPRB, 0.26089E+00_JPRB, 0.26618E+00_JPRB, 0.27082E+00_JPRB, 0.27529E+00_JPRB, 0.27954E+00_JPRB ,&
&  0.28203E+00_JPRB, 0.28164E+00_JPRB, 0.28175E+00_JPRB, 0.28245E+00_JPRB, 0.28152E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.30559E+00_JPRB, 0.30116E+00_JPRB, 0.30017E+00_JPRB, 0.30313E+00_JPRB, 0.30597E+00_JPRB, 0.30421E+00_JPRB, 0.29942E+00_JPRB ,&
&  0.29009E+00_JPRB, 0.28182E+00_JPRB, 0.27490E+00_JPRB, 0.27124E+00_JPRB, 0.27192E+00_JPRB, 0.27165E+00_JPRB, 0.26928E+00_JPRB ,&
&  0.26696E+00_JPRB, 0.26298E+00_JPRB, 0.25975E+00_JPRB, 0.25653E+00_JPRB, 0.25409E+00_JPRB, 0.25305E+00_JPRB, 0.25275E+00_JPRB ,&
&  0.25161E+00_JPRB, 0.25037E+00_JPRB, 0.25067E+00_JPRB, 0.25144E+00_JPRB, 0.25391E+00_JPRB, 0.25883E+00_JPRB, 0.26441E+00_JPRB ,&
&  0.27351E+00_JPRB, 0.25906E+00_JPRB, 0.27506E+00_JPRB, 0.27964E+00_JPRB, 0.28300E+00_JPRB, 0.28930E+00_JPRB, 0.29630E+00_JPRB ,&
&  0.30259E+00_JPRB, 0.30684E+00_JPRB, 0.30914E+00_JPRB, 0.30920E+00_JPRB, 0.30668E+00_JPRB, 0.30435E+00_JPRB, 0.30053E+00_JPRB ,&
&  0.29592E+00_JPRB, 0.29414E+00_JPRB, 0.29316E+00_JPRB, 0.29103E+00_JPRB, 0.28859E+00_JPRB, 0.28510E+00_JPRB, 0.28144E+00_JPRB ,&
&  0.27801E+00_JPRB, 0.27471E+00_JPRB, 0.27212E+00_JPRB, 0.26977E+00_JPRB, 0.26808E+00_JPRB, 0.26756E+00_JPRB, 0.26847E+00_JPRB ,&
&  0.26990E+00_JPRB, 0.27214E+00_JPRB, 0.27707E+00_JPRB, 0.28248E+00_JPRB, 0.28639E+00_JPRB, 0.28868E+00_JPRB, 0.28994E+00_JPRB ,&
&  0.29023E+00_JPRB, 0.28998E+00_JPRB, 0.28927E+00_JPRB, 0.28825E+00_JPRB, 0.28650E+00_JPRB, 0.28464E+00_JPRB, 0.28156E+00_JPRB ,&
&  0.27905E+00_JPRB, 0.27710E+00_JPRB, 0.27500E+00_JPRB, 0.27352E+00_JPRB, 0.27166E+00_JPRB, 0.26902E+00_JPRB, 0.26641E+00_JPRB ,&
&  0.26434E+00_JPRB, 0.26287E+00_JPRB, 0.26180E+00_JPRB, 0.26139E+00_JPRB, 0.26164E+00_JPRB, 0.26348E+00_JPRB, 0.26561E+00_JPRB ,&
&  0.26849E+00_JPRB, 0.27308E+00_JPRB, 0.27945E+00_JPRB, 0.28683E+00_JPRB, 0.29501E+00_JPRB, 0.30019E+00_JPRB, 0.30455E+00_JPRB ,&
&  0.30713E+00_JPRB, 0.30692E+00_JPRB, 0.30765E+00_JPRB, 0.30804E+00_JPRB, 0.30770E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.33398E+00_JPRB, 0.33067E+00_JPRB, 0.32655E+00_JPRB, 0.32719E+00_JPRB, 0.32472E+00_JPRB, 0.31992E+00_JPRB, 0.31288E+00_JPRB ,&
&  0.30251E+00_JPRB, 0.29489E+00_JPRB, 0.28895E+00_JPRB, 0.28680E+00_JPRB, 0.28870E+00_JPRB, 0.28685E+00_JPRB, 0.28309E+00_JPRB ,&
&  0.28032E+00_JPRB, 0.27714E+00_JPRB, 0.27495E+00_JPRB, 0.27237E+00_JPRB, 0.27039E+00_JPRB, 0.26978E+00_JPRB, 0.26941E+00_JPRB ,&
&  0.26814E+00_JPRB, 0.26621E+00_JPRB, 0.26489E+00_JPRB, 0.26487E+00_JPRB, 0.26833E+00_JPRB, 0.27513E+00_JPRB, 0.28571E+00_JPRB ,&
&  0.28327E+00_JPRB, 0.26602E+00_JPRB, 0.28209E+00_JPRB, 0.30209E+00_JPRB, 0.31439E+00_JPRB, 0.32509E+00_JPRB, 0.33243E+00_JPRB ,&
&  0.33889E+00_JPRB, 0.34079E+00_JPRB, 0.34210E+00_JPRB, 0.34224E+00_JPRB, 0.33984E+00_JPRB, 0.33502E+00_JPRB, 0.33014E+00_JPRB ,&
&  0.32717E+00_JPRB, 0.32528E+00_JPRB, 0.32292E+00_JPRB, 0.31802E+00_JPRB, 0.31246E+00_JPRB, 0.30623E+00_JPRB, 0.30034E+00_JPRB ,&
&  0.29579E+00_JPRB, 0.29137E+00_JPRB, 0.28807E+00_JPRB, 0.28659E+00_JPRB, 0.28728E+00_JPRB, 0.28943E+00_JPRB, 0.29342E+00_JPRB ,&
&  0.29814E+00_JPRB, 0.30527E+00_JPRB, 0.31523E+00_JPRB, 0.32285E+00_JPRB, 0.32743E+00_JPRB, 0.32887E+00_JPRB, 0.32931E+00_JPRB ,&
&  0.32752E+00_JPRB, 0.32535E+00_JPRB, 0.32415E+00_JPRB, 0.32215E+00_JPRB, 0.31923E+00_JPRB, 0.31519E+00_JPRB, 0.30985E+00_JPRB ,&
&  0.30536E+00_JPRB, 0.30172E+00_JPRB, 0.29823E+00_JPRB, 0.29504E+00_JPRB, 0.29185E+00_JPRB, 0.28849E+00_JPRB, 0.28553E+00_JPRB ,&
&  0.28243E+00_JPRB, 0.27924E+00_JPRB, 0.27708E+00_JPRB, 0.27699E+00_JPRB, 0.27950E+00_JPRB, 0.28277E+00_JPRB, 0.28696E+00_JPRB ,&
&  0.29171E+00_JPRB, 0.29910E+00_JPRB, 0.30716E+00_JPRB, 0.31782E+00_JPRB, 0.32674E+00_JPRB, 0.32945E+00_JPRB, 0.33196E+00_JPRB ,&
&  0.33410E+00_JPRB, 0.33257E+00_JPRB, 0.33444E+00_JPRB, 0.33425E+00_JPRB, 0.33313E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.36002E+00_JPRB, 0.36416E+00_JPRB, 0.36315E+00_JPRB, 0.35623E+00_JPRB, 0.34608E+00_JPRB, 0.33898E+00_JPRB, 0.32869E+00_JPRB ,&
&  0.31783E+00_JPRB, 0.31119E+00_JPRB, 0.30366E+00_JPRB, 0.30038E+00_JPRB, 0.30004E+00_JPRB, 0.29655E+00_JPRB, 0.29384E+00_JPRB ,&
&  0.29160E+00_JPRB, 0.28752E+00_JPRB, 0.28504E+00_JPRB, 0.28264E+00_JPRB, 0.28244E+00_JPRB, 0.28253E+00_JPRB, 0.28242E+00_JPRB ,&
&  0.28158E+00_JPRB, 0.27981E+00_JPRB, 0.27842E+00_JPRB, 0.27792E+00_JPRB, 0.28084E+00_JPRB, 0.28814E+00_JPRB, 0.30151E+00_JPRB ,&
&  0.32977E+00_JPRB, 0.29536E+00_JPRB, 0.30710E+00_JPRB, 0.32151E+00_JPRB, 0.34578E+00_JPRB, 0.36089E+00_JPRB, 0.36640E+00_JPRB ,&
&  0.37274E+00_JPRB, 0.37978E+00_JPRB, 0.38317E+00_JPRB, 0.38569E+00_JPRB, 0.37981E+00_JPRB, 0.37191E+00_JPRB, 0.36689E+00_JPRB ,&
&  0.36084E+00_JPRB, 0.35336E+00_JPRB, 0.34568E+00_JPRB, 0.33724E+00_JPRB, 0.33196E+00_JPRB, 0.32814E+00_JPRB, 0.32397E+00_JPRB ,&
&  0.31858E+00_JPRB, 0.31270E+00_JPRB, 0.30861E+00_JPRB, 0.30763E+00_JPRB, 0.31058E+00_JPRB, 0.31738E+00_JPRB, 0.32980E+00_JPRB ,&
&  0.34175E+00_JPRB, 0.35493E+00_JPRB, 0.36738E+00_JPRB, 0.37460E+00_JPRB, 0.37786E+00_JPRB, 0.37634E+00_JPRB, 0.37267E+00_JPRB ,&
&  0.36747E+00_JPRB, 0.36220E+00_JPRB, 0.35838E+00_JPRB, 0.35323E+00_JPRB, 0.34651E+00_JPRB, 0.34109E+00_JPRB, 0.33693E+00_JPRB ,&
&  0.33385E+00_JPRB, 0.32981E+00_JPRB, 0.32470E+00_JPRB, 0.31963E+00_JPRB, 0.31438E+00_JPRB, 0.31006E+00_JPRB, 0.30632E+00_JPRB ,&
&  0.30278E+00_JPRB, 0.29921E+00_JPRB, 0.29749E+00_JPRB, 0.29921E+00_JPRB, 0.30278E+00_JPRB, 0.30689E+00_JPRB, 0.31077E+00_JPRB ,&
&  0.31637E+00_JPRB, 0.32513E+00_JPRB, 0.33271E+00_JPRB, 0.34696E+00_JPRB, 0.35352E+00_JPRB, 0.35671E+00_JPRB, 0.35957E+00_JPRB ,&
&  0.35969E+00_JPRB, 0.35776E+00_JPRB, 0.32251E+00_JPRB, 0.34617E+00_JPRB, 0.35459E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.39803E+00_JPRB, 0.40148E+00_JPRB, 0.39778E+00_JPRB, 0.38320E+00_JPRB, 0.36798E+00_JPRB, 0.35864E+00_JPRB, 0.34789E+00_JPRB ,&
&  0.33645E+00_JPRB, 0.32706E+00_JPRB, 0.31622E+00_JPRB, 0.31209E+00_JPRB, 0.30944E+00_JPRB, 0.30817E+00_JPRB, 0.30661E+00_JPRB ,&
&  0.30303E+00_JPRB, 0.29714E+00_JPRB, 0.29244E+00_JPRB, 0.29041E+00_JPRB, 0.29076E+00_JPRB, 0.29179E+00_JPRB, 0.29322E+00_JPRB ,&
&  0.29290E+00_JPRB, 0.29298E+00_JPRB, 0.29273E+00_JPRB, 0.29321E+00_JPRB, 0.29549E+00_JPRB, 0.30331E+00_JPRB, 0.31844E+00_JPRB ,&
&  0.35565E+00_JPRB, 0.32277E+00_JPRB, 0.33176E+00_JPRB, 0.35062E+00_JPRB, 0.37049E+00_JPRB, 0.39569E+00_JPRB, 0.41861E+00_JPRB ,&
&  0.42781E+00_JPRB, 0.42472E+00_JPRB, 0.42930E+00_JPRB, 0.43868E+00_JPRB, 0.43534E+00_JPRB, 0.42933E+00_JPRB, 0.42008E+00_JPRB ,&
&  0.40621E+00_JPRB, 0.39059E+00_JPRB, 0.37480E+00_JPRB, 0.36429E+00_JPRB, 0.35977E+00_JPRB, 0.35689E+00_JPRB, 0.35580E+00_JPRB ,&
&  0.35098E+00_JPRB, 0.34817E+00_JPRB, 0.35157E+00_JPRB, 0.35645E+00_JPRB, 0.36150E+00_JPRB, 0.37284E+00_JPRB, 0.39025E+00_JPRB ,&
&  0.40296E+00_JPRB, 0.41688E+00_JPRB, 0.42754E+00_JPRB, 0.43141E+00_JPRB, 0.43381E+00_JPRB, 0.43058E+00_JPRB, 0.41951E+00_JPRB ,&
&  0.40798E+00_JPRB, 0.39605E+00_JPRB, 0.38578E+00_JPRB, 0.37649E+00_JPRB, 0.36964E+00_JPRB, 0.36670E+00_JPRB, 0.36494E+00_JPRB ,&
&  0.36339E+00_JPRB, 0.36222E+00_JPRB, 0.35924E+00_JPRB, 0.35503E+00_JPRB, 0.35132E+00_JPRB, 0.34877E+00_JPRB, 0.34632E+00_JPRB ,&
&  0.34358E+00_JPRB, 0.34080E+00_JPRB, 0.34017E+00_JPRB, 0.34227E+00_JPRB, 0.34461E+00_JPRB, 0.34794E+00_JPRB, 0.35237E+00_JPRB ,&
&  0.35659E+00_JPRB, 0.36510E+00_JPRB, 0.37027E+00_JPRB, 0.38896E+00_JPRB, 0.39589E+00_JPRB, 0.39546E+00_JPRB, 0.39537E+00_JPRB ,&
&  0.39144E+00_JPRB, 0.38773E+00_JPRB, 0.38525E+00_JPRB, 0.36645E+00_JPRB, 0.39087E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.44716E+00_JPRB, 0.44394E+00_JPRB, 0.42698E+00_JPRB, 0.40502E+00_JPRB, 0.38794E+00_JPRB, 0.37288E+00_JPRB, 0.36026E+00_JPRB ,&
&  0.35187E+00_JPRB, 0.34266E+00_JPRB, 0.33253E+00_JPRB, 0.32506E+00_JPRB, 0.32370E+00_JPRB, 0.32117E+00_JPRB, 0.31780E+00_JPRB ,&
&  0.31180E+00_JPRB, 0.30604E+00_JPRB, 0.30218E+00_JPRB, 0.30068E+00_JPRB, 0.30096E+00_JPRB, 0.30262E+00_JPRB, 0.30382E+00_JPRB ,&
&  0.30297E+00_JPRB, 0.30041E+00_JPRB, 0.30123E+00_JPRB, 0.30412E+00_JPRB, 0.30945E+00_JPRB, 0.31714E+00_JPRB, 0.33534E+00_JPRB ,&
&  0.37744E+00_JPRB, 0.35554E+00_JPRB, 0.37211E+00_JPRB, 0.38172E+00_JPRB, 0.39228E+00_JPRB, 0.41786E+00_JPRB, 0.44343E+00_JPRB ,&
&  0.46025E+00_JPRB, 0.47025E+00_JPRB, 0.48985E+00_JPRB, 0.50632E+00_JPRB, 0.50290E+00_JPRB, 0.48853E+00_JPRB, 0.46956E+00_JPRB ,&
&  0.45002E+00_JPRB, 0.43553E+00_JPRB, 0.42309E+00_JPRB, 0.41293E+00_JPRB, 0.40586E+00_JPRB, 0.40267E+00_JPRB, 0.40291E+00_JPRB ,&
&  0.39916E+00_JPRB, 0.39810E+00_JPRB, 0.40281E+00_JPRB, 0.41473E+00_JPRB, 0.42651E+00_JPRB, 0.43859E+00_JPRB, 0.45245E+00_JPRB ,&
&  0.46298E+00_JPRB, 0.47144E+00_JPRB, 0.47643E+00_JPRB, 0.47722E+00_JPRB, 0.47088E+00_JPRB, 0.46246E+00_JPRB, 0.44450E+00_JPRB ,&
&  0.42662E+00_JPRB, 0.41265E+00_JPRB, 0.40528E+00_JPRB, 0.40001E+00_JPRB, 0.39626E+00_JPRB, 0.39407E+00_JPRB, 0.39206E+00_JPRB ,&
&  0.39076E+00_JPRB, 0.39191E+00_JPRB, 0.39304E+00_JPRB, 0.39380E+00_JPRB, 0.39436E+00_JPRB, 0.39479E+00_JPRB, 0.39384E+00_JPRB ,&
&  0.39249E+00_JPRB, 0.39231E+00_JPRB, 0.39250E+00_JPRB, 0.39213E+00_JPRB, 0.39160E+00_JPRB, 0.39163E+00_JPRB, 0.39276E+00_JPRB ,&
&  0.39560E+00_JPRB, 0.40462E+00_JPRB, 0.40934E+00_JPRB, 0.42423E+00_JPRB, 0.42505E+00_JPRB, 0.42428E+00_JPRB, 0.42338E+00_JPRB ,&
&  0.41923E+00_JPRB, 0.41459E+00_JPRB, 0.40614E+00_JPRB, 0.40448E+00_JPRB, 0.42388E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.52915E+00_JPRB, 0.49911E+00_JPRB, 0.45951E+00_JPRB, 0.42844E+00_JPRB, 0.40621E+00_JPRB, 0.38918E+00_JPRB, 0.37871E+00_JPRB ,&
&  0.37445E+00_JPRB, 0.36511E+00_JPRB, 0.35285E+00_JPRB, 0.34270E+00_JPRB, 0.33767E+00_JPRB, 0.33378E+00_JPRB, 0.32847E+00_JPRB ,&
&  0.32376E+00_JPRB, 0.31964E+00_JPRB, 0.31711E+00_JPRB, 0.31600E+00_JPRB, 0.31533E+00_JPRB, 0.31459E+00_JPRB, 0.31216E+00_JPRB ,&
&  0.30831E+00_JPRB, 0.30835E+00_JPRB, 0.31052E+00_JPRB, 0.31719E+00_JPRB, 0.32549E+00_JPRB, 0.33460E+00_JPRB, 0.35211E+00_JPRB ,&
&  0.40217E+00_JPRB, 0.39027E+00_JPRB, 0.42183E+00_JPRB, 0.42986E+00_JPRB, 0.43455E+00_JPRB, 0.45719E+00_JPRB, 0.48176E+00_JPRB ,&
&  0.50699E+00_JPRB, 0.53866E+00_JPRB, 0.57088E+00_JPRB, 0.58346E+00_JPRB, 0.57012E+00_JPRB, 0.54639E+00_JPRB, 0.52478E+00_JPRB ,&
&  0.50410E+00_JPRB, 0.48967E+00_JPRB, 0.47595E+00_JPRB, 0.46318E+00_JPRB, 0.45419E+00_JPRB, 0.45308E+00_JPRB, 0.45724E+00_JPRB ,&
&  0.45868E+00_JPRB, 0.45135E+00_JPRB, 0.45817E+00_JPRB, 0.47081E+00_JPRB, 0.48998E+00_JPRB, 0.50393E+00_JPRB, 0.51561E+00_JPRB ,&
&  0.52651E+00_JPRB, 0.52828E+00_JPRB, 0.52552E+00_JPRB, 0.51560E+00_JPRB, 0.49450E+00_JPRB, 0.47133E+00_JPRB, 0.44912E+00_JPRB ,&
&  0.42776E+00_JPRB, 0.41422E+00_JPRB, 0.40836E+00_JPRB, 0.40554E+00_JPRB, 0.40581E+00_JPRB, 0.40814E+00_JPRB, 0.41324E+00_JPRB ,&
&  0.42148E+00_JPRB, 0.42911E+00_JPRB, 0.43497E+00_JPRB, 0.43754E+00_JPRB, 0.43634E+00_JPRB, 0.43372E+00_JPRB, 0.42904E+00_JPRB ,&
&  0.42380E+00_JPRB, 0.41907E+00_JPRB, 0.41575E+00_JPRB, 0.41466E+00_JPRB, 0.41507E+00_JPRB, 0.41699E+00_JPRB, 0.42015E+00_JPRB ,&
&  0.42476E+00_JPRB, 0.43609E+00_JPRB, 0.43363E+00_JPRB, 0.45046E+00_JPRB, 0.46663E+00_JPRB, 0.46084E+00_JPRB, 0.45359E+00_JPRB ,&
&  0.44378E+00_JPRB, 0.43727E+00_JPRB, 0.43222E+00_JPRB, 0.43538E+00_JPRB, 0.51110E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.47939E+00_JPRB, 0.46860E+00_JPRB, 0.44573E+00_JPRB, 0.42472E+00_JPRB, 0.41090E+00_JPRB, 0.40063E+00_JPRB, 0.39472E+00_JPRB ,&
&  0.39129E+00_JPRB, 0.38392E+00_JPRB, 0.37690E+00_JPRB, 0.36710E+00_JPRB, 0.35876E+00_JPRB, 0.35189E+00_JPRB, 0.34530E+00_JPRB ,&
&  0.33962E+00_JPRB, 0.33630E+00_JPRB, 0.33359E+00_JPRB, 0.33093E+00_JPRB, 0.32663E+00_JPRB, 0.32026E+00_JPRB, 0.31665E+00_JPRB ,&
&  0.31584E+00_JPRB, 0.31738E+00_JPRB, 0.32183E+00_JPRB, 0.33036E+00_JPRB, 0.33850E+00_JPRB, 0.34706E+00_JPRB, 0.36367E+00_JPRB ,&
&  0.40699E+00_JPRB, 0.49179E+00_JPRB, 0.51838E+00_JPRB, 0.51930E+00_JPRB, 0.51221E+00_JPRB, 0.50542E+00_JPRB, 0.53108E+00_JPRB ,&
&  0.57771E+00_JPRB, 0.61996E+00_JPRB, 0.65404E+00_JPRB, 0.66575E+00_JPRB, 0.65250E+00_JPRB, 0.63588E+00_JPRB, 0.61245E+00_JPRB ,&
&  0.58775E+00_JPRB, 0.55993E+00_JPRB, 0.53181E+00_JPRB, 0.50538E+00_JPRB, 0.48175E+00_JPRB, 0.47766E+00_JPRB, 0.48523E+00_JPRB ,&
&  0.49894E+00_JPRB, 0.50706E+00_JPRB, 0.50885E+00_JPRB, 0.51936E+00_JPRB, 0.54128E+00_JPRB, 0.57011E+00_JPRB, 0.60126E+00_JPRB ,&
&  0.61420E+00_JPRB, 0.59872E+00_JPRB, 0.57500E+00_JPRB, 0.54406E+00_JPRB, 0.50559E+00_JPRB, 0.47194E+00_JPRB, 0.44622E+00_JPRB ,&
&  0.42719E+00_JPRB, 0.41403E+00_JPRB, 0.40580E+00_JPRB, 0.40310E+00_JPRB, 0.40712E+00_JPRB, 0.41785E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.44256E+00_JPRB, 0.45347E+00_JPRB, 0.45890E+00_JPRB, 0.45998E+00_JPRB, 0.45550E+00_JPRB, 0.44638E+00_JPRB, 0.43188E+00_JPRB ,&
&  0.41927E+00_JPRB, 0.41278E+00_JPRB, 0.41708E+00_JPRB, 0.42126E+00_JPRB, 0.42487E+00_JPRB, 0.42891E+00_JPRB, 0.43351E+00_JPRB ,&
&  0.44256E+00_JPRB, 0.45334E+00_JPRB, 0.45416E+00_JPRB, 0.45322E+00_JPRB, 0.45642E+00_JPRB, 0.47287E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.45763E+00_JPRB, 0.45098E+00_JPRB, 0.44931E+00_JPRB, 0.44990E+00_JPRB, 0.47674E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.46962E+00_JPRB, 0.46500E+00_JPRB, 0.45098E+00_JPRB, 0.43485E+00_JPRB, 0.42351E+00_JPRB, 0.41462E+00_JPRB, 0.40821E+00_JPRB ,&
&  0.40437E+00_JPRB, 0.39762E+00_JPRB, 0.39176E+00_JPRB, 0.38397E+00_JPRB, 0.37460E+00_JPRB, 0.36693E+00_JPRB, 0.36224E+00_JPRB ,&
&  0.35576E+00_JPRB, 0.35247E+00_JPRB, 0.34877E+00_JPRB, 0.34140E+00_JPRB, 0.33353E+00_JPRB, 0.32707E+00_JPRB, 0.32297E+00_JPRB ,&
&  0.32174E+00_JPRB, 0.32461E+00_JPRB, 0.33048E+00_JPRB, 0.34273E+00_JPRB, 0.35274E+00_JPRB, 0.36213E+00_JPRB, 0.38212E+00_JPRB ,&
&  0.44100E+00_JPRB, 0.56858E+00_JPRB, 0.67108E+00_JPRB, 0.61348E+00_JPRB, 0.57751E+00_JPRB, 0.55440E+00_JPRB, 0.61235E+00_JPRB ,&
&  0.67318E+00_JPRB, 0.73439E+00_JPRB, 0.77035E+00_JPRB, 0.78360E+00_JPRB, 0.75495E+00_JPRB, 0.73108E+00_JPRB, 0.70321E+00_JPRB ,&
&  0.67233E+00_JPRB, 0.63628E+00_JPRB, 0.58967E+00_JPRB, 0.54356E+00_JPRB, 0.51663E+00_JPRB, 0.51528E+00_JPRB, 0.52774E+00_JPRB ,&
&  0.54564E+00_JPRB, 0.55626E+00_JPRB, 0.55270E+00_JPRB, 0.56234E+00_JPRB, 0.56430E+00_JPRB, 0.56407E+00_JPRB, 0.62954E+00_JPRB ,&
&  0.75579E+00_JPRB, 0.72106E+00_JPRB, 0.65863E+00_JPRB, 0.57995E+00_JPRB, 0.52348E+00_JPRB, 0.48038E+00_JPRB, 0.45539E+00_JPRB ,&
&  0.43688E+00_JPRB, 0.42430E+00_JPRB, 0.41709E+00_JPRB, 0.41560E+00_JPRB, 0.42208E+00_JPRB, 0.43265E+00_JPRB, 0.44045E+00_JPRB ,&
&  0.44871E+00_JPRB, 0.45499E+00_JPRB, 0.45741E+00_JPRB, 0.45418E+00_JPRB, 0.44670E+00_JPRB, 0.43579E+00_JPRB, 0.42505E+00_JPRB ,&
&  0.42016E+00_JPRB, 0.42290E+00_JPRB, 0.43573E+00_JPRB, 0.43382E+00_JPRB, 0.43941E+00_JPRB, 0.45033E+00_JPRB, 0.45249E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.45169E+00_JPRB, 0.45988E+00_JPRB, 0.45840E+00_JPRB, 0.46550E+00_JPRB, 0.49362E+00_JPRB, 0.47843E+00_JPRB ,&
&  0.46511E+00_JPRB, 0.45605E+00_JPRB, 0.45627E+00_JPRB, 0.45482E+00_JPRB, 0.46393E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.47134E+00_JPRB, 0.46864E+00_JPRB, 0.45743E+00_JPRB, 0.44308E+00_JPRB, 0.43192E+00_JPRB, 0.42079E+00_JPRB, 0.41259E+00_JPRB ,&
&  0.40692E+00_JPRB, 0.40242E+00_JPRB, 0.39921E+00_JPRB, 0.39515E+00_JPRB, 0.38943E+00_JPRB, 0.38166E+00_JPRB, 0.37419E+00_JPRB ,&
&  0.36605E+00_JPRB, 0.35799E+00_JPRB, 0.35020E+00_JPRB, 0.34204E+00_JPRB, 0.33614E+00_JPRB, 0.33199E+00_JPRB, 0.32924E+00_JPRB ,&
&  0.32945E+00_JPRB, 0.33265E+00_JPRB, 0.34040E+00_JPRB, 0.35580E+00_JPRB, 0.37301E+00_JPRB, 0.38183E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.43978E+00_JPRB, 0.39023E+00_JPRB, 0.63051E+00_JPRB, 0.68893E+00_JPRB, 0.67221E+00_JPRB, 0.68249E+00_JPRB, 0.76578E+00_JPRB ,&
&  0.83970E+00_JPRB, 0.86276E+00_JPRB, 0.86442E+00_JPRB, 0.83723E+00_JPRB, 0.77251E+00_JPRB, 0.71912E+00_JPRB, 0.67529E+00_JPRB ,&
&  0.63842E+00_JPRB, 0.60571E+00_JPRB, 0.56940E+00_JPRB, 0.53459E+00_JPRB, 0.50949E+00_JPRB, 0.50534E+00_JPRB, 0.51305E+00_JPRB ,&
&  0.54013E+00_JPRB, 0.55799E+00_JPRB, 0.57368E+00_JPRB, 0.59589E+00_JPRB, 0.58240E+00_JPRB, 0.62259E+00_JPRB, 0.64286E+00_JPRB ,&
&  0.75827E+00_JPRB, 0.82682E+00_JPRB, 0.71462E+00_JPRB, 0.59883E+00_JPRB, 0.52607E+00_JPRB, 0.49064E+00_JPRB, 0.46204E+00_JPRB ,&
&  0.44695E+00_JPRB, 0.43754E+00_JPRB, 0.43215E+00_JPRB, 0.43067E+00_JPRB, 0.43418E+00_JPRB, 0.43926E+00_JPRB, 0.44107E+00_JPRB ,&
&  0.44344E+00_JPRB, 0.44538E+00_JPRB, 0.44631E+00_JPRB, 0.44544E+00_JPRB, 0.44112E+00_JPRB, 0.43425E+00_JPRB, 0.43164E+00_JPRB ,&
&  0.43579E+00_JPRB, 0.44701E+00_JPRB, 0.45643E+00_JPRB, 0.45711E+00_JPRB, 0.46217E+00_JPRB, 0.46770E+00_JPRB, 0.47026E+00_JPRB ,&
&  0.47083E+00_JPRB, 0.47132E+00_JPRB, 0.46965E+00_JPRB, 0.47593E+00_JPRB, 0.46813E+00_JPRB, 0.49464E+00_JPRB, 0.47767E+00_JPRB ,&
&  0.46556E+00_JPRB, 0.46015E+00_JPRB, 0.45944E+00_JPRB, 0.46101E+00_JPRB, 0.46566E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.47694E+00_JPRB, 0.47421E+00_JPRB, 0.46376E+00_JPRB, 0.45008E+00_JPRB, 0.43572E+00_JPRB, 0.42382E+00_JPRB, 0.41884E+00_JPRB ,&
&  0.41323E+00_JPRB, 0.40708E+00_JPRB, 0.40129E+00_JPRB, 0.39846E+00_JPRB, 0.39753E+00_JPRB, 0.39152E+00_JPRB, 0.38061E+00_JPRB ,&
&  0.36889E+00_JPRB, 0.35770E+00_JPRB, 0.34882E+00_JPRB, 0.34470E+00_JPRB, 0.34014E+00_JPRB, 0.33772E+00_JPRB, 0.33784E+00_JPRB ,&
&  0.33909E+00_JPRB, 0.34486E+00_JPRB, 0.35659E+00_JPRB, 0.37596E+00_JPRB, 0.39134E+00_JPRB, 0.39622E+00_JPRB, 0.40224E+00_JPRB ,&
&  0.43976E+00_JPRB, 0.60724E+00_JPRB, 0.58975E+00_JPRB, 0.79746E+00_JPRB, 0.85603E+00_JPRB, 0.89249E+00_JPRB, 0.92189E+00_JPRB ,&
&  0.93478E+00_JPRB, 0.95170E+00_JPRB, 0.91221E+00_JPRB, 0.84722E+00_JPRB, 0.75750E+00_JPRB, 0.68674E+00_JPRB, 0.62501E+00_JPRB ,&
&  0.57511E+00_JPRB, 0.53805E+00_JPRB, 0.51223E+00_JPRB, 0.49276E+00_JPRB, 0.48110E+00_JPRB, 0.48097E+00_JPRB, 0.49642E+00_JPRB ,&
&  0.52895E+00_JPRB, 0.56651E+00_JPRB, 0.60902E+00_JPRB, 0.65083E+00_JPRB, 0.62454E+00_JPRB, 0.66847E+00_JPRB, 0.71129E+00_JPRB ,&
&  0.75664E+00_JPRB, 0.82313E+00_JPRB, 0.68694E+00_JPRB, 0.57016E+00_JPRB, 0.51661E+00_JPRB, 0.49437E+00_JPRB, 0.47661E+00_JPRB ,&
&  0.46502E+00_JPRB, 0.45731E+00_JPRB, 0.45251E+00_JPRB, 0.44821E+00_JPRB, 0.44563E+00_JPRB, 0.44487E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.44533E+00_JPRB, 0.44697E+00_JPRB, 0.44916E+00_JPRB, 0.45041E+00_JPRB, 0.44846E+00_JPRB, 0.44903E+00_JPRB, 0.45475E+00_JPRB ,&
&  0.46497E+00_JPRB, 0.47506E+00_JPRB, 0.47811E+00_JPRB, 0.47657E+00_JPRB, 0.48125E+00_JPRB, 0.48687E+00_JPRB, 0.48406E+00_JPRB ,&
&  0.49012E+00_JPRB, 0.49165E+00_JPRB, 0.49021E+00_JPRB, 0.48982E+00_JPRB, 0.48849E+00_JPRB, 0.51843E+00_JPRB, 0.48368E+00_JPRB ,&
&  0.47155E+00_JPRB, 0.46883E+00_JPRB, 0.47022E+00_JPRB, 0.46884E+00_JPRB, 0.47295E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.48283E+00_JPRB, 0.47383E+00_JPRB, 0.46316E+00_JPRB, 0.45065E+00_JPRB, 0.43699E+00_JPRB, 0.42784E+00_JPRB, 0.42376E+00_JPRB ,&
&  0.41580E+00_JPRB, 0.40845E+00_JPRB, 0.40297E+00_JPRB, 0.40096E+00_JPRB, 0.39923E+00_JPRB, 0.39259E+00_JPRB, 0.37995E+00_JPRB ,&
&  0.36728E+00_JPRB, 0.35924E+00_JPRB, 0.35543E+00_JPRB, 0.35146E+00_JPRB, 0.34660E+00_JPRB, 0.34400E+00_JPRB, 0.34512E+00_JPRB ,&
&  0.35029E+00_JPRB, 0.35871E+00_JPRB, 0.37540E+00_JPRB, 0.39469E+00_JPRB, 0.41032E+00_JPRB, 0.42219E+00_JPRB, 0.43042E+00_JPRB ,&
&  0.49117E+00_JPRB, 0.58093E+00_JPRB, 0.66031E+00_JPRB, 0.10377E+01_JPRB, 0.11006E+01_JPRB, 0.10668E+01_JPRB, 0.10251E+01_JPRB ,&
&  0.98892E+00_JPRB, 0.10216E+01_JPRB, 0.93901E+00_JPRB, 0.83538E+00_JPRB, 0.72754E+00_JPRB, 0.65135E+00_JPRB, 0.59244E+00_JPRB ,&
&  0.54215E+00_JPRB, 0.50944E+00_JPRB, 0.49438E+00_JPRB, 0.48854E+00_JPRB, 0.48615E+00_JPRB, 0.49110E+00_JPRB, 0.50912E+00_JPRB ,&
&  0.54396E+00_JPRB, 0.59467E+00_JPRB, 0.67010E+00_JPRB, 0.70637E+00_JPRB, 0.69426E+00_JPRB, 0.70648E+00_JPRB, 0.70743E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.74813E+00_JPRB, 0.62830E+00_JPRB, 0.55534E+00_JPRB, 0.51616E+00_JPRB, 0.50843E+00_JPRB, 0.49071E+00_JPRB ,&
&  0.48210E+00_JPRB, 0.47546E+00_JPRB, 0.47046E+00_JPRB, 0.46323E+00_JPRB, 0.45548E+00_JPRB, 0.45260E+00_JPRB, 0.45288E+00_JPRB ,&
&  0.45425E+00_JPRB, 0.45739E+00_JPRB, 0.45912E+00_JPRB, 0.46054E+00_JPRB, 0.46227E+00_JPRB, 0.46812E+00_JPRB, 0.47738E+00_JPRB ,&
&  0.48478E+00_JPRB, 0.49901E+00_JPRB, 0.49734E+00_JPRB, 0.49390E+00_JPRB, 0.50532E+00_JPRB, 0.51324E+00_JPRB, 0.50252E+00_JPRB ,&
&  0.50148E+00_JPRB, 0.50976E+00_JPRB, 0.50987E+00_JPRB, 0.49657E+00_JPRB, 0.50469E+00_JPRB, 0.52144E+00_JPRB, 0.49354E+00_JPRB ,&
&  0.48618E+00_JPRB, 0.48536E+00_JPRB, 0.48996E+00_JPRB, 0.48732E+00_JPRB, 0.48518E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.48651E+00_JPRB, 0.47390E+00_JPRB, 0.46361E+00_JPRB, 0.45561E+00_JPRB, 0.44515E+00_JPRB, 0.43423E+00_JPRB, 0.42595E+00_JPRB ,&
&  0.41285E+00_JPRB, 0.40429E+00_JPRB, 0.39825E+00_JPRB, 0.39136E+00_JPRB, 0.38486E+00_JPRB, 0.37900E+00_JPRB, 0.37407E+00_JPRB ,&
&  0.37093E+00_JPRB, 0.36847E+00_JPRB, 0.36671E+00_JPRB, 0.36291E+00_JPRB, 0.35967E+00_JPRB, 0.35925E+00_JPRB, 0.36185E+00_JPRB ,&
&  0.37389E+00_JPRB, 0.39103E+00_JPRB, 0.41101E+00_JPRB, 0.43191E+00_JPRB, 0.45090E+00_JPRB, 0.47437E+00_JPRB, 0.50197E+00_JPRB ,&
&  0.60066E+00_JPRB, 0.64786E+00_JPRB, 0.81343E+00_JPRB, 0.12285E+01_JPRB, 0.12456E+01_JPRB, 0.11024E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.98091E+00_JPRB, 0.87388E+00_JPRB, 0.89371E+00_JPRB, 0.78543E+00_JPRB, 0.68066E+00_JPRB, 0.62552E+00_JPRB, 0.58668E+00_JPRB ,&
&  0.55077E+00_JPRB, 0.52997E+00_JPRB, 0.52481E+00_JPRB, 0.52174E+00_JPRB, 0.51538E+00_JPRB, 0.52501E+00_JPRB, 0.55107E+00_JPRB ,&
&  0.60255E+00_JPRB, 0.67570E+00_JPRB, 0.78098E+00_JPRB, 0.84054E+00_JPRB, 0.74204E+00_JPRB, 0.71435E+00_JPRB, 0.68142E+00_JPRB ,&
&  0.70469E+00_JPRB, 0.66208E+00_JPRB, 0.59653E+00_JPRB, 0.55382E+00_JPRB, 0.52128E+00_JPRB, 0.52520E+00_JPRB, 0.50416E+00_JPRB ,&
&  0.49612E+00_JPRB, 0.48895E+00_JPRB, 0.48087E+00_JPRB, 0.46864E+00_JPRB, 0.46097E+00_JPRB, 0.45517E+00_JPRB, 0.45539E+00_JPRB ,&
&  0.46058E+00_JPRB, 0.46556E+00_JPRB, 0.47028E+00_JPRB, 0.47610E+00_JPRB, 0.48310E+00_JPRB, 0.48910E+00_JPRB, 0.49548E+00_JPRB ,&
&  0.50213E+00_JPRB, 0.51991E+00_JPRB, 0.51784E+00_JPRB, 0.52844E+00_JPRB, 0.54497E+00_JPRB, 0.55097E+00_JPRB, 0.52887E+00_JPRB ,&
&  0.51489E+00_JPRB, 0.52289E+00_JPRB, 0.52292E+00_JPRB, 0.51972E+00_JPRB, 0.52648E+00_JPRB, 0.53062E+00_JPRB, 0.51002E+00_JPRB ,&
&  0.51062E+00_JPRB, 0.51441E+00_JPRB, 0.52640E+00_JPRB, 0.51351E+00_JPRB, 0.50071E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.49299E+00_JPRB, 0.47469E+00_JPRB, 0.46157E+00_JPRB, 0.44805E+00_JPRB, 0.43355E+00_JPRB, 0.41775E+00_JPRB, 0.40531E+00_JPRB ,&
&  0.39011E+00_JPRB, 0.38686E+00_JPRB, 0.38363E+00_JPRB, 0.37875E+00_JPRB, 0.37320E+00_JPRB, 0.37383E+00_JPRB, 0.38007E+00_JPRB ,&
&  0.38555E+00_JPRB, 0.38731E+00_JPRB, 0.38920E+00_JPRB, 0.39055E+00_JPRB, 0.39467E+00_JPRB, 0.40379E+00_JPRB, 0.41765E+00_JPRB ,&
&  0.43578E+00_JPRB, 0.45630E+00_JPRB, 0.48340E+00_JPRB, 0.51565E+00_JPRB, 0.54649E+00_JPRB, 0.59124E+00_JPRB, 0.64766E+00_JPRB ,&
&  0.75605E+00_JPRB, 0.75843E+00_JPRB, 0.49461E+00_JPRB, 0.80262E+00_JPRB, 0.10128E+01_JPRB, 0.95598E+00_JPRB, 0.89550E+00_JPRB ,&
&  0.87038E+00_JPRB, 0.80745E+00_JPRB, 0.78885E+00_JPRB, 0.73382E+00_JPRB, 0.65842E+00_JPRB, 0.63786E+00_JPRB, 0.62201E+00_JPRB ,&
&  0.60052E+00_JPRB, 0.58261E+00_JPRB, 0.57070E+00_JPRB, 0.56660E+00_JPRB, 0.55970E+00_JPRB, 0.57254E+00_JPRB, 0.62719E+00_JPRB ,&
&  0.70173E+00_JPRB, 0.80915E+00_JPRB, 0.93915E+00_JPRB, 0.90059E+00_JPRB, 0.77373E+00_JPRB, 0.70441E+00_JPRB, 0.64527E+00_JPRB ,&
&  0.61579E+00_JPRB, 0.61161E+00_JPRB, 0.58984E+00_JPRB, 0.55945E+00_JPRB, 0.52992E+00_JPRB, 0.51177E+00_JPRB, 0.51271E+00_JPRB ,&
&  0.49987E+00_JPRB, 0.48891E+00_JPRB, 0.47851E+00_JPRB, 0.46272E+00_JPRB, 0.45724E+00_JPRB, 0.45537E+00_JPRB, 0.46230E+00_JPRB ,&
&  0.47112E+00_JPRB, 0.47804E+00_JPRB, 0.48613E+00_JPRB, 0.49514E+00_JPRB, 0.49949E+00_JPRB, 0.50610E+00_JPRB, 0.51246E+00_JPRB ,&
&  0.52220E+00_JPRB, 0.54088E+00_JPRB, 0.56122E+00_JPRB, 0.57331E+00_JPRB, 0.57622E+00_JPRB, 0.57295E+00_JPRB, 0.56020E+00_JPRB ,&
&  0.55540E+00_JPRB, 0.55339E+00_JPRB, 0.53863E+00_JPRB, 0.53397E+00_JPRB, 0.55719E+00_JPRB, 0.56498E+00_JPRB, 0.57620E+00_JPRB ,&
&  0.61472E+00_JPRB, 0.64900E+00_JPRB, 0.65695E+00_JPRB, 0.55242E+00_JPRB, 0.51731E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.46753E+00_JPRB, 0.44081E+00_JPRB, 0.41355E+00_JPRB, 0.39924E+00_JPRB, 0.38920E+00_JPRB, 0.38472E+00_JPRB, 0.38247E+00_JPRB ,&
&  0.38340E+00_JPRB, 0.38276E+00_JPRB, 0.37786E+00_JPRB, 0.38129E+00_JPRB, 0.39153E+00_JPRB, 0.40241E+00_JPRB, 0.41690E+00_JPRB ,&
&  0.42352E+00_JPRB, 0.42748E+00_JPRB, 0.43447E+00_JPRB, 0.45074E+00_JPRB, 0.47300E+00_JPRB, 0.49587E+00_JPRB, 0.51639E+00_JPRB ,&
&  0.53875E+00_JPRB, 0.56393E+00_JPRB, 0.59712E+00_JPRB, 0.64158E+00_JPRB, 0.70136E+00_JPRB, 0.77812E+00_JPRB, 0.85490E+00_JPRB ,&
&  0.93254E+00_JPRB, 0.71098E+00_JPRB, 0.55877E+00_JPRB, 0.89660E+00_JPRB, 0.96580E+00_JPRB, 0.91190E+00_JPRB, 0.79915E+00_JPRB ,&
&  0.77865E+00_JPRB, 0.77783E+00_JPRB, 0.75133E+00_JPRB, 0.73071E+00_JPRB, 0.67528E+00_JPRB, 0.66234E+00_JPRB, 0.65289E+00_JPRB ,&
&  0.63739E+00_JPRB, 0.61952E+00_JPRB, 0.62135E+00_JPRB, 0.62791E+00_JPRB, 0.63631E+00_JPRB, 0.67556E+00_JPRB, 0.75170E+00_JPRB ,&
&  0.85002E+00_JPRB, 0.98092E+00_JPRB, 0.11405E+01_JPRB, 0.10008E+01_JPRB, 0.84383E+00_JPRB, 0.71262E+00_JPRB, 0.64798E+00_JPRB ,&
&  0.60319E+00_JPRB, 0.59943E+00_JPRB, 0.59309E+00_JPRB, 0.56928E+00_JPRB, 0.54122E+00_JPRB, 0.51332E+00_JPRB, 0.50520E+00_JPRB ,&
&  0.49158E+00_JPRB, 0.47579E+00_JPRB, 0.46459E+00_JPRB, 0.45366E+00_JPRB, 0.45431E+00_JPRB, 0.46250E+00_JPRB, 0.47113E+00_JPRB ,&
&  0.47856E+00_JPRB, 0.48554E+00_JPRB, 0.49735E+00_JPRB, 0.51208E+00_JPRB, 0.52004E+00_JPRB, 0.52606E+00_JPRB, 0.53603E+00_JPRB ,&
&  0.54583E+00_JPRB, 0.55720E+00_JPRB, 0.57838E+00_JPRB, 0.58965E+00_JPRB, 0.58849E+00_JPRB, 0.57007E+00_JPRB, 0.57267E+00_JPRB ,&
&  0.57151E+00_JPRB, 0.57824E+00_JPRB, 0.57280E+00_JPRB, 0.58242E+00_JPRB, 0.58837E+00_JPRB, 0.61547E+00_JPRB, 0.64499E+00_JPRB ,&
&  0.66741E+00_JPRB, 0.66798E+00_JPRB, 0.63952E+00_JPRB, 0.53338E+00_JPRB, 0.49559E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.39840E+00_JPRB, 0.37949E+00_JPRB, 0.38130E+00_JPRB, 0.38615E+00_JPRB, 0.39917E+00_JPRB, 0.41052E+00_JPRB, 0.40506E+00_JPRB ,&
&  0.40877E+00_JPRB, 0.41435E+00_JPRB, 0.42443E+00_JPRB, 0.43534E+00_JPRB, 0.45009E+00_JPRB, 0.46476E+00_JPRB, 0.47212E+00_JPRB ,&
&  0.48749E+00_JPRB, 0.50734E+00_JPRB, 0.53352E+00_JPRB, 0.55930E+00_JPRB, 0.58574E+00_JPRB, 0.60640E+00_JPRB, 0.62531E+00_JPRB ,&
&  0.64882E+00_JPRB, 0.68823E+00_JPRB, 0.73957E+00_JPRB, 0.79859E+00_JPRB, 0.86912E+00_JPRB, 0.95221E+00_JPRB, 0.99742E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.49008E+00_JPRB, 0.91697E+00_JPRB, 0.10154E+01_JPRB, 0.89426E+00_JPRB, 0.83600E+00_JPRB, 0.78062E+00_JPRB ,&
&  0.73594E+00_JPRB, 0.73772E+00_JPRB, 0.70915E+00_JPRB, 0.72143E+00_JPRB, 0.68610E+00_JPRB, 0.67644E+00_JPRB, 0.67733E+00_JPRB ,&
&  0.67414E+00_JPRB, 0.67843E+00_JPRB, 0.70506E+00_JPRB, 0.72739E+00_JPRB, 0.76073E+00_JPRB, 0.82067E+00_JPRB, 0.91224E+00_JPRB ,&
&  0.10406E+01_JPRB, 0.11983E+01_JPRB, 0.13573E+01_JPRB, 0.10946E+01_JPRB, 0.92279E+00_JPRB, 0.80817E+00_JPRB, 0.71661E+00_JPRB ,&
&  0.66009E+00_JPRB, 0.58485E+00_JPRB, 0.57616E+00_JPRB, 0.57727E+00_JPRB, 0.55467E+00_JPRB, 0.50436E+00_JPRB, 0.49278E+00_JPRB ,&
&  0.46943E+00_JPRB, 0.45760E+00_JPRB, 0.45227E+00_JPRB, 0.44792E+00_JPRB, 0.44800E+00_JPRB, 0.45467E+00_JPRB, 0.46213E+00_JPRB ,&
&  0.47352E+00_JPRB, 0.47883E+00_JPRB, 0.49104E+00_JPRB, 0.50608E+00_JPRB, 0.51825E+00_JPRB, 0.53198E+00_JPRB, 0.54892E+00_JPRB ,&
&  0.55615E+00_JPRB, 0.57173E+00_JPRB, 0.58630E+00_JPRB, 0.58913E+00_JPRB, 0.59121E+00_JPRB, 0.57835E+00_JPRB, 0.57527E+00_JPRB ,&
&  0.57506E+00_JPRB, 0.57686E+00_JPRB, 0.57376E+00_JPRB, 0.58215E+00_JPRB, 0.59282E+00_JPRB, 0.60089E+00_JPRB, 0.62035E+00_JPRB ,&
&  0.61473E+00_JPRB, 0.58847E+00_JPRB, 0.53012E+00_JPRB, 0.46316E+00_JPRB, 0.43147E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.38975E+00_JPRB, 0.40711E+00_JPRB, 0.42578E+00_JPRB, 0.44250E+00_JPRB, 0.45673E+00_JPRB, 0.46793E+00_JPRB, 0.48664E+00_JPRB ,&
&  0.50675E+00_JPRB, 0.51840E+00_JPRB, 0.52019E+00_JPRB, 0.53019E+00_JPRB, 0.54177E+00_JPRB, 0.55823E+00_JPRB, 0.56920E+00_JPRB ,&
&  0.58537E+00_JPRB, 0.60956E+00_JPRB, 0.63098E+00_JPRB, 0.65370E+00_JPRB, 0.67490E+00_JPRB, 0.69687E+00_JPRB, 0.72896E+00_JPRB ,&
&  0.77060E+00_JPRB, 0.82461E+00_JPRB, 0.88464E+00_JPRB, 0.93732E+00_JPRB, 0.98069E+00_JPRB, 0.10300E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.69529E+00_JPRB, 0.89136E+00_JPRB, 0.99559E+00_JPRB, 0.91347E+00_JPRB, 0.84433E+00_JPRB, 0.80245E+00_JPRB, 0.78115E+00_JPRB ,&
&  0.74451E+00_JPRB, 0.69933E+00_JPRB, 0.68602E+00_JPRB, 0.72526E+00_JPRB, 0.69129E+00_JPRB, 0.69436E+00_JPRB, 0.70973E+00_JPRB ,&
&  0.73504E+00_JPRB, 0.76744E+00_JPRB, 0.80417E+00_JPRB, 0.85167E+00_JPRB, 0.91874E+00_JPRB, 0.10008E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.12418E+01_JPRB, 0.13899E+01_JPRB, 0.15204E+01_JPRB, 0.12468E+01_JPRB, 0.11520E+01_JPRB, 0.11115E+01_JPRB, 0.99870E+00_JPRB ,&
&  0.73174E+00_JPRB, 0.60000E+00_JPRB, 0.60181E+00_JPRB, 0.60654E+00_JPRB, 0.55952E+00_JPRB, 0.51231E+00_JPRB, 0.46700E+00_JPRB ,&
&  0.44448E+00_JPRB, 0.42750E+00_JPRB, 0.41959E+00_JPRB, 0.41332E+00_JPRB, 0.41215E+00_JPRB, 0.42301E+00_JPRB, 0.43596E+00_JPRB ,&
&  0.44809E+00_JPRB, 0.44949E+00_JPRB, 0.46628E+00_JPRB, 0.47716E+00_JPRB, 0.51184E+00_JPRB, 0.54472E+00_JPRB, 0.56092E+00_JPRB ,&
&  0.56982E+00_JPRB, 0.56625E+00_JPRB, 0.56343E+00_JPRB, 0.56130E+00_JPRB, 0.55959E+00_JPRB, 0.55599E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.54924E+00_JPRB, 0.56445E+00_JPRB, 0.55857E+00_JPRB, 0.55572E+00_JPRB, 0.56645E+00_JPRB, 0.54757E+00_JPRB, 0.53456E+00_JPRB ,&
&  0.51353E+00_JPRB, 0.46773E+00_JPRB, 0.43328E+00_JPRB, 0.40883E+00_JPRB, 0.39381E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.47196E+00_JPRB, 0.51199E+00_JPRB, 0.52772E+00_JPRB, 0.54681E+00_JPRB, 0.55877E+00_JPRB, 0.57520E+00_JPRB, 0.58767E+00_JPRB ,&
&  0.59863E+00_JPRB, 0.61252E+00_JPRB, 0.61599E+00_JPRB, 0.63238E+00_JPRB, 0.64478E+00_JPRB, 0.65534E+00_JPRB, 0.66159E+00_JPRB ,&
&  0.67047E+00_JPRB, 0.68876E+00_JPRB, 0.70887E+00_JPRB, 0.73309E+00_JPRB, 0.75700E+00_JPRB, 0.78921E+00_JPRB, 0.83481E+00_JPRB ,&
&  0.88950E+00_JPRB, 0.93985E+00_JPRB, 0.98052E+00_JPRB, 0.10111E+01_JPRB, 0.10224E+01_JPRB, 0.10553E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.78995E+00_JPRB, 0.93398E+00_JPRB, 0.90393E+00_JPRB, 0.86254E+00_JPRB, 0.82638E+00_JPRB, 0.78958E+00_JPRB, 0.76429E+00_JPRB ,&
&  0.73907E+00_JPRB, 0.69281E+00_JPRB, 0.68398E+00_JPRB, 0.69717E+00_JPRB, 0.71415E+00_JPRB, 0.72757E+00_JPRB, 0.75445E+00_JPRB ,&
&  0.79646E+00_JPRB, 0.84958E+00_JPRB, 0.90660E+00_JPRB, 0.96822E+00_JPRB, 0.10537E+01_JPRB, 0.11625E+01_JPRB, 0.12559E+01_JPRB ,&
&  0.13564E+01_JPRB, 0.14603E+01_JPRB, 0.15208E+01_JPRB, 0.13028E+01_JPRB, 0.11102E+01_JPRB, 0.10226E+01_JPRB, 0.83886E+00_JPRB ,&
&  0.67634E+00_JPRB, 0.60754E+00_JPRB, 0.63207E+00_JPRB, 0.64077E+00_JPRB, 0.57298E+00_JPRB, 0.50010E+00_JPRB, 0.44338E+00_JPRB ,&
&  0.42345E+00_JPRB, 0.39540E+00_JPRB, 0.38264E+00_JPRB, 0.38169E+00_JPRB, 0.38531E+00_JPRB, 0.39223E+00_JPRB, 0.41107E+00_JPRB ,&
&  0.42354E+00_JPRB, 0.43685E+00_JPRB, 0.45798E+00_JPRB, 0.48540E+00_JPRB, 0.53055E+00_JPRB, 0.59526E+00_JPRB, 0.60072E+00_JPRB ,&
&  0.57844E+00_JPRB, 0.54992E+00_JPRB, 0.52563E+00_JPRB, 0.52703E+00_JPRB, 0.52697E+00_JPRB, 0.53123E+00_JPRB, 0.54829E+00_JPRB ,&
&  0.57477E+00_JPRB, 0.59530E+00_JPRB, 0.56648E+00_JPRB, 0.54872E+00_JPRB, 0.55701E+00_JPRB, 0.52864E+00_JPRB, 0.47423E+00_JPRB ,&
&  0.45363E+00_JPRB, 0.43599E+00_JPRB, 0.42014E+00_JPRB, 0.42693E+00_JPRB, 0.44676E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.61752E+00_JPRB, 0.64114E+00_JPRB, 0.65625E+00_JPRB, 0.65473E+00_JPRB, 0.67705E+00_JPRB, 0.68965E+00_JPRB, 0.69432E+00_JPRB ,&
&  0.69868E+00_JPRB, 0.71013E+00_JPRB, 0.71371E+00_JPRB, 0.73002E+00_JPRB, 0.74081E+00_JPRB, 0.75213E+00_JPRB, 0.75775E+00_JPRB ,&
&  0.76511E+00_JPRB, 0.78231E+00_JPRB, 0.80428E+00_JPRB, 0.83784E+00_JPRB, 0.87036E+00_JPRB, 0.91158E+00_JPRB, 0.94512E+00_JPRB ,&
&  0.96526E+00_JPRB, 0.99492E+00_JPRB, 0.10338E+01_JPRB, 0.10542E+01_JPRB, 0.10359E+01_JPRB, 0.10154E+01_JPRB, 0.77439E+00_JPRB ,&
&  0.87922E+00_JPRB, 0.89194E+00_JPRB, 0.87827E+00_JPRB, 0.85236E+00_JPRB, 0.82002E+00_JPRB, 0.77526E+00_JPRB, 0.74400E+00_JPRB ,&
&  0.72465E+00_JPRB, 0.70462E+00_JPRB, 0.68960E+00_JPRB, 0.68052E+00_JPRB, 0.73302E+00_JPRB, 0.74991E+00_JPRB, 0.78156E+00_JPRB ,&
&  0.82580E+00_JPRB, 0.88318E+00_JPRB, 0.95111E+00_JPRB, 0.10439E+01_JPRB, 0.11453E+01_JPRB, 0.12448E+01_JPRB, 0.13481E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.14609E+01_JPRB, 0.14825E+01_JPRB, 0.12149E+01_JPRB, 0.10609E+01_JPRB, 0.90688E+00_JPRB, 0.76605E+00_JPRB ,&
&  0.67744E+00_JPRB, 0.65170E+00_JPRB, 0.68657E+00_JPRB, 0.68625E+00_JPRB, 0.60112E+00_JPRB, 0.51061E+00_JPRB, 0.44599E+00_JPRB ,&
&  0.41797E+00_JPRB, 0.38212E+00_JPRB, 0.37476E+00_JPRB, 0.37320E+00_JPRB, 0.38071E+00_JPRB, 0.38799E+00_JPRB, 0.39794E+00_JPRB ,&
&  0.41155E+00_JPRB, 0.43558E+00_JPRB, 0.48118E+00_JPRB, 0.54769E+00_JPRB, 0.63085E+00_JPRB, 0.71908E+00_JPRB, 0.68896E+00_JPRB ,&
&  0.58038E+00_JPRB, 0.52125E+00_JPRB, 0.53259E+00_JPRB, 0.54806E+00_JPRB, 0.58483E+00_JPRB, 0.64469E+00_JPRB, 0.71893E+00_JPRB ,&
&  0.84427E+00_JPRB, 0.92339E+00_JPRB, 0.84135E+00_JPRB, 0.70614E+00_JPRB, 0.98406E+00_JPRB, 0.96384E+00_JPRB, 0.49827E+00_JPRB ,&
&  0.54226E+00_JPRB, 0.53294E+00_JPRB, 0.55334E+00_JPRB, 0.57729E+00_JPRB, 0.60708E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.74275E+00_JPRB, 0.76397E+00_JPRB, 0.77186E+00_JPRB, 0.76983E+00_JPRB, 0.78147E+00_JPRB, 0.80314E+00_JPRB, 0.80823E+00_JPRB ,&
&  0.81332E+00_JPRB, 0.80608E+00_JPRB, 0.81323E+00_JPRB, 0.82184E+00_JPRB, 0.83077E+00_JPRB, 0.84122E+00_JPRB, 0.85123E+00_JPRB ,&
&  0.85534E+00_JPRB, 0.88129E+00_JPRB, 0.89194E+00_JPRB, 0.92104E+00_JPRB, 0.95117E+00_JPRB, 0.99497E+00_JPRB, 0.10144E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.10300E+01_JPRB, 0.10538E+01_JPRB, 0.10859E+01_JPRB, 0.10709E+01_JPRB, 0.10676E+01_JPRB, 0.84470E+00_JPRB ,&
&  0.92490E+00_JPRB, 0.89357E+00_JPRB, 0.84534E+00_JPRB, 0.82117E+00_JPRB, 0.79171E+00_JPRB, 0.76088E+00_JPRB, 0.73359E+00_JPRB ,&
&  0.72463E+00_JPRB, 0.71326E+00_JPRB, 0.69321E+00_JPRB, 0.72079E+00_JPRB, 0.73524E+00_JPRB, 0.75385E+00_JPRB, 0.79607E+00_JPRB ,&
&  0.83891E+00_JPRB, 0.90606E+00_JPRB, 0.98701E+00_JPRB, 0.10866E+01_JPRB, 0.11916E+01_JPRB, 0.12929E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.14896E+01_JPRB, 0.15666E+01_JPRB, 0.15821E+01_JPRB, 0.13379E+01_JPRB, 0.11964E+01_JPRB, 0.95390E+00_JPRB, 0.79584E+00_JPRB ,&
&  0.69384E+00_JPRB, 0.65389E+00_JPRB, 0.67534E+00_JPRB, 0.72471E+00_JPRB, 0.62052E+00_JPRB, 0.54180E+00_JPRB, 0.46846E+00_JPRB ,&
&  0.43806E+00_JPRB, 0.40465E+00_JPRB, 0.39653E+00_JPRB, 0.38810E+00_JPRB, 0.38223E+00_JPRB, 0.38582E+00_JPRB, 0.39398E+00_JPRB ,&
&  0.39738E+00_JPRB, 0.41455E+00_JPRB, 0.45327E+00_JPRB, 0.49609E+00_JPRB, 0.53127E+00_JPRB, 0.54711E+00_JPRB, 0.52037E+00_JPRB ,&
&  0.55209E+00_JPRB, 0.54930E+00_JPRB, 0.55519E+00_JPRB, 0.61290E+00_JPRB, 0.71774E+00_JPRB, 0.88260E+00_JPRB, 0.10905E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.14129E+01_JPRB, 0.17213E+01_JPRB, 0.22753E+01_JPRB, 0.20535E+01_JPRB, 0.17317E+01_JPRB, 0.61711E+00_JPRB ,&
&  0.64636E+00_JPRB, 0.65205E+00_JPRB, 0.67718E+00_JPRB, 0.70139E+00_JPRB, 0.73202E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.74472E+00_JPRB, 0.76241E+00_JPRB, 0.77538E+00_JPRB, 0.78372E+00_JPRB, 0.79782E+00_JPRB, 0.81170E+00_JPRB, 0.82464E+00_JPRB ,&
&  0.83565E+00_JPRB, 0.84274E+00_JPRB, 0.85411E+00_JPRB, 0.86409E+00_JPRB, 0.87656E+00_JPRB, 0.89186E+00_JPRB, 0.91011E+00_JPRB ,&
&  0.92299E+00_JPRB, 0.94081E+00_JPRB, 0.94778E+00_JPRB, 0.97657E+00_JPRB, 0.10150E+01_JPRB, 0.10306E+01_JPRB, 0.10367E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10402E+01_JPRB, 0.10515E+01_JPRB, 0.10707E+01_JPRB, 0.10730E+01_JPRB, 0.11069E+01_JPRB, 0.93817E+00_JPRB ,&
&  0.97209E+00_JPRB, 0.90488E+00_JPRB, 0.80528E+00_JPRB, 0.77619E+00_JPRB, 0.75229E+00_JPRB, 0.74124E+00_JPRB, 0.73166E+00_JPRB ,&
&  0.72385E+00_JPRB, 0.72302E+00_JPRB, 0.71430E+00_JPRB, 0.70763E+00_JPRB, 0.71902E+00_JPRB, 0.73887E+00_JPRB, 0.77321E+00_JPRB ,&
&  0.82732E+00_JPRB, 0.90240E+00_JPRB, 0.98996E+00_JPRB, 0.10884E+01_JPRB, 0.11993E+01_JPRB, 0.12920E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.14942E+01_JPRB, 0.13942E+01_JPRB, 0.13840E+01_JPRB, 0.13757E+01_JPRB, 0.10800E+01_JPRB, 0.84904E+00_JPRB ,&
&  0.63961E+00_JPRB, 0.63792E+00_JPRB, 0.67912E+00_JPRB, 0.76080E+00_JPRB, 0.66215E+00_JPRB, 0.59650E+00_JPRB, 0.51874E+00_JPRB ,&
&  0.45475E+00_JPRB, 0.42272E+00_JPRB, 0.41631E+00_JPRB, 0.44510E+00_JPRB, 0.42072E+00_JPRB, 0.40458E+00_JPRB, 0.41090E+00_JPRB ,&
&  0.39841E+00_JPRB, 0.40109E+00_JPRB, 0.41146E+00_JPRB, 0.43227E+00_JPRB, 0.45063E+00_JPRB, 0.44565E+00_JPRB, 0.50099E+00_JPRB ,&
&  0.50251E+00_JPRB, 0.49990E+00_JPRB, 0.51589E+00_JPRB, 0.46990E+00_JPRB, 0.65073E+00_JPRB, 0.74502E+00_JPRB, 0.87808E+00_JPRB ,&
&  0.10186E+01_JPRB, 0.14145E+01_JPRB, 0.16682E+01_JPRB, 0.12937E+01_JPRB, 0.97692E+00_JPRB, 0.67841E+00_JPRB, 0.64149E+00_JPRB ,&
&  0.65503E+00_JPRB, 0.65770E+00_JPRB, 0.67472E+00_JPRB, 0.69706E+00_JPRB, 0.72167E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.73862E+00_JPRB, 0.75819E+00_JPRB, 0.77402E+00_JPRB, 0.78650E+00_JPRB, 0.80253E+00_JPRB, 0.81902E+00_JPRB, 0.83278E+00_JPRB ,&
&  0.84516E+00_JPRB, 0.85630E+00_JPRB, 0.87038E+00_JPRB, 0.88291E+00_JPRB, 0.89824E+00_JPRB, 0.91710E+00_JPRB, 0.93522E+00_JPRB ,&
&  0.95254E+00_JPRB, 0.96121E+00_JPRB, 0.98675E+00_JPRB, 0.10174E+01_JPRB, 0.10360E+01_JPRB, 0.10360E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10444E+01_JPRB, 0.10591E+01_JPRB, 0.10726E+01_JPRB, 0.10952E+01_JPRB, 0.11430E+01_JPRB, 0.94285E+00_JPRB ,&
&  0.99913E+00_JPRB, 0.90698E+00_JPRB, 0.76782E+00_JPRB, 0.71382E+00_JPRB, 0.69363E+00_JPRB, 0.68235E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.69172E+00_JPRB, 0.67378E+00_JPRB, 0.66957E+00_JPRB, 0.66870E+00_JPRB, 0.68426E+00_JPRB, 0.70366E+00_JPRB, 0.72761E+00_JPRB ,&
&  0.77087E+00_JPRB, 0.83340E+00_JPRB, 0.91008E+00_JPRB, 0.99879E+00_JPRB, 0.11046E+01_JPRB, 0.11869E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.12296E+01_JPRB, 0.10765E+01_JPRB, 0.11505E+01_JPRB, 0.11972E+01_JPRB, 0.10820E+01_JPRB, 0.93135E+00_JPRB ,&
&  0.75063E+00_JPRB, 0.64788E+00_JPRB, 0.67612E+00_JPRB, 0.78738E+00_JPRB, 0.74811E+00_JPRB, 0.65922E+00_JPRB, 0.59394E+00_JPRB ,&
&  0.49914E+00_JPRB, 0.47112E+00_JPRB, 0.46028E+00_JPRB, 0.50717E+00_JPRB, 0.49539E+00_JPRB, 0.45417E+00_JPRB, 0.44368E+00_JPRB ,&
&  0.41080E+00_JPRB, 0.37735E+00_JPRB, 0.36338E+00_JPRB, 0.38566E+00_JPRB, 0.43253E+00_JPRB, 0.41433E+00_JPRB, 0.46778E+00_JPRB ,&
&  0.45697E+00_JPRB, 0.40229E+00_JPRB, 0.42551E+00_JPRB, 0.49212E+00_JPRB, 0.53334E+00_JPRB, 0.58211E+00_JPRB, 0.68762E+00_JPRB ,&
&  0.85164E+00_JPRB, 0.10554E+01_JPRB, 0.10241E+01_JPRB, 0.81101E+00_JPRB, 0.64680E+00_JPRB, 0.61281E+00_JPRB, 0.62554E+00_JPRB ,&
&  0.64422E+00_JPRB, 0.65859E+00_JPRB, 0.67598E+00_JPRB, 0.69522E+00_JPRB, 0.71582E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.72380E+00_JPRB, 0.74279E+00_JPRB, 0.75950E+00_JPRB, 0.77492E+00_JPRB, 0.79077E+00_JPRB, 0.80583E+00_JPRB, 0.81849E+00_JPRB ,&
&  0.82991E+00_JPRB, 0.84389E+00_JPRB, 0.85593E+00_JPRB, 0.86827E+00_JPRB, 0.88676E+00_JPRB, 0.90314E+00_JPRB, 0.91258E+00_JPRB ,&
&  0.92984E+00_JPRB, 0.95200E+00_JPRB, 0.97955E+00_JPRB, 0.10019E+01_JPRB, 0.10111E+01_JPRB, 0.10097E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.99173E+00_JPRB, 0.98493E+00_JPRB, 0.98928E+00_JPRB, 0.10016E+01_JPRB, 0.10283E+01_JPRB, 0.10129E+01_JPRB, 0.10826E+01_JPRB ,&
&  0.79480E+00_JPRB, 0.91201E+00_JPRB, 0.75327E+00_JPRB, 0.64971E+00_JPRB, 0.64603E+00_JPRB, 0.60939E+00_JPRB, 0.60887E+00_JPRB ,&
&  0.62927E+00_JPRB, 0.63548E+00_JPRB, 0.63864E+00_JPRB, 0.64026E+00_JPRB, 0.65409E+00_JPRB, 0.67903E+00_JPRB, 0.68531E+00_JPRB ,&
&  0.71188E+00_JPRB, 0.74464E+00_JPRB, 0.76272E+00_JPRB, 0.80894E+00_JPRB, 0.89839E+00_JPRB, 0.97636E+00_JPRB, 0.10113E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.96519E+00_JPRB, 0.75329E+00_JPRB, 0.81210E+00_JPRB, 0.90143E+00_JPRB, 0.88698E+00_JPRB, 0.81988E+00_JPRB ,&
&  0.79290E+00_JPRB, 0.71426E+00_JPRB, 0.81891E+00_JPRB, 0.86492E+00_JPRB, 0.84260E+00_JPRB, 0.77926E+00_JPRB, 0.68990E+00_JPRB ,&
&  0.60734E+00_JPRB, 0.54204E+00_JPRB, 0.55796E+00_JPRB, 0.61779E+00_JPRB, 0.60741E+00_JPRB, 0.53056E+00_JPRB, 0.49640E+00_JPRB ,&
&  0.44413E+00_JPRB, 0.39136E+00_JPRB, 0.36184E+00_JPRB, 0.37966E+00_JPRB, 0.41990E+00_JPRB, 0.60764E+00_JPRB, 0.48682E+00_JPRB ,&
&  0.51440E+00_JPRB, 0.47455E+00_JPRB, 0.35061E+00_JPRB, 0.42897E+00_JPRB, 0.43706E+00_JPRB, 0.42997E+00_JPRB, 0.48359E+00_JPRB ,&
&  0.59499E+00_JPRB, 0.68916E+00_JPRB, 0.69395E+00_JPRB, 0.62751E+00_JPRB, 0.60217E+00_JPRB, 0.60704E+00_JPRB, 0.62370E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.65676E+00_JPRB, 0.67209E+00_JPRB, 0.68859E+00_JPRB, 0.70521E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.69524E+00_JPRB, 0.71960E+00_JPRB, 0.73557E+00_JPRB, 0.74813E+00_JPRB, 0.75253E+00_JPRB, 0.75790E+00_JPRB, 0.77244E+00_JPRB ,&
&  0.78349E+00_JPRB, 0.79541E+00_JPRB, 0.80858E+00_JPRB, 0.82856E+00_JPRB, 0.85492E+00_JPRB, 0.87469E+00_JPRB, 0.88186E+00_JPRB ,&
&  0.88678E+00_JPRB, 0.91695E+00_JPRB, 0.93651E+00_JPRB, 0.97375E+00_JPRB, 0.98741E+00_JPRB, 0.10107E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10132E+01_JPRB, 0.10173E+01_JPRB, 0.10258E+01_JPRB, 0.10472E+01_JPRB, 0.10671E+01_JPRB, 0.11730E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.86668E+00_JPRB, 0.73271E+00_JPRB, 0.59327E+00_JPRB, 0.58383E+00_JPRB, 0.58657E+00_JPRB, 0.58206E+00_JPRB ,&
&  0.59034E+00_JPRB, 0.61052E+00_JPRB, 0.62384E+00_JPRB, 0.61641E+00_JPRB, 0.63365E+00_JPRB, 0.67238E+00_JPRB, 0.66849E+00_JPRB ,&
&  0.67469E+00_JPRB, 0.68234E+00_JPRB, 0.67315E+00_JPRB, 0.73079E+00_JPRB, 0.69700E+00_JPRB, 0.76174E+00_JPRB, 0.80851E+00_JPRB ,&
&  0.83337E+00_JPRB, 0.78106E+00_JPRB, 0.60076E+00_JPRB, 0.61242E+00_JPRB, 0.70459E+00_JPRB, 0.72909E+00_JPRB, 0.71851E+00_JPRB ,&
&  0.79324E+00_JPRB, 0.80474E+00_JPRB, 0.71274E+00_JPRB, 0.89735E+00_JPRB, 0.96088E+00_JPRB, 0.96463E+00_JPRB, 0.88733E+00_JPRB ,&
&  0.81049E+00_JPRB, 0.70025E+00_JPRB, 0.71266E+00_JPRB, 0.72597E+00_JPRB, 0.71097E+00_JPRB, 0.66910E+00_JPRB, 0.58692E+00_JPRB ,&
&  0.51093E+00_JPRB, 0.44790E+00_JPRB, 0.40418E+00_JPRB, 0.40672E+00_JPRB, 0.50795E+00_JPRB, 0.59888E+00_JPRB, 0.51115E+00_JPRB ,&
&  0.43315E+00_JPRB, 0.46030E+00_JPRB, 0.37398E+00_JPRB, 0.41964E+00_JPRB, 0.39656E+00_JPRB, 0.37666E+00_JPRB, 0.40404E+00_JPRB ,&
&  0.45899E+00_JPRB, 0.50300E+00_JPRB, 0.53318E+00_JPRB, 0.55429E+00_JPRB, 0.57241E+00_JPRB, 0.59389E+00_JPRB, 0.60296E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.63297E+00_JPRB, 0.64814E+00_JPRB, 0.67008E+00_JPRB, 0.67952E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.59314E+00_JPRB, 0.60265E+00_JPRB, 0.61719E+00_JPRB, 0.62568E+00_JPRB, 0.63433E+00_JPRB, 0.63984E+00_JPRB, 0.66340E+00_JPRB ,&
&  0.67788E+00_JPRB, 0.69163E+00_JPRB, 0.71266E+00_JPRB, 0.73422E+00_JPRB, 0.74329E+00_JPRB, 0.75362E+00_JPRB, 0.75734E+00_JPRB ,&
&  0.77098E+00_JPRB, 0.78665E+00_JPRB, 0.78599E+00_JPRB, 0.79306E+00_JPRB, 0.80016E+00_JPRB, 0.85947E+00_JPRB, 0.86079E+00_JPRB ,&
&  0.85163E+00_JPRB, 0.86042E+00_JPRB, 0.87177E+00_JPRB, 0.91383E+00_JPRB, 0.10389E+01_JPRB, 0.11120E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.99477E+00_JPRB, 0.87759E+00_JPRB, 0.93277E+00_JPRB, 0.76292E+00_JPRB, 0.72254E+00_JPRB, 0.71502E+00_JPRB, 0.66963E+00_JPRB ,&
&  0.65969E+00_JPRB, 0.66725E+00_JPRB, 0.69471E+00_JPRB, 0.68005E+00_JPRB, 0.68577E+00_JPRB, 0.68087E+00_JPRB, 0.65991E+00_JPRB ,&
&  0.66186E+00_JPRB, 0.66928E+00_JPRB, 0.66938E+00_JPRB, 0.69810E+00_JPRB, 0.68937E+00_JPRB, 0.73303E+00_JPRB, 0.74160E+00_JPRB ,&
&  0.74964E+00_JPRB, 0.76939E+00_JPRB, 0.71948E+00_JPRB, 0.72551E+00_JPRB, 0.81529E+00_JPRB, 0.83059E+00_JPRB, 0.81132E+00_JPRB ,&
&  0.83620E+00_JPRB, 0.78509E+00_JPRB, 0.58423E+00_JPRB, 0.85560E+00_JPRB, 0.10025E+01_JPRB, 0.10057E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.96314E+00_JPRB, 0.90584E+00_JPRB, 0.86672E+00_JPRB, 0.86054E+00_JPRB, 0.80691E+00_JPRB, 0.74785E+00_JPRB, 0.69290E+00_JPRB ,&
&  0.58962E+00_JPRB, 0.48819E+00_JPRB, 0.41221E+00_JPRB, 0.38809E+00_JPRB, 0.39041E+00_JPRB, 0.53744E+00_JPRB, 0.45628E+00_JPRB ,&
&  0.39593E+00_JPRB, 0.36202E+00_JPRB, 0.41099E+00_JPRB, 0.40188E+00_JPRB, 0.35466E+00_JPRB, 0.36741E+00_JPRB, 0.36422E+00_JPRB ,&
&  0.38384E+00_JPRB, 0.40928E+00_JPRB, 0.43359E+00_JPRB, 0.45219E+00_JPRB, 0.47482E+00_JPRB, 0.49450E+00_JPRB, 0.50992E+00_JPRB ,&
&  0.52504E+00_JPRB, 0.54371E+00_JPRB, 0.55957E+00_JPRB, 0.57721E+00_JPRB, 0.59126E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.58528E+00_JPRB, 0.59720E+00_JPRB, 0.59470E+00_JPRB, 0.59725E+00_JPRB, 0.59008E+00_JPRB, 0.58865E+00_JPRB, 0.57366E+00_JPRB ,&
&  0.56034E+00_JPRB, 0.56603E+00_JPRB, 0.59060E+00_JPRB, 0.63710E+00_JPRB, 0.64547E+00_JPRB, 0.65928E+00_JPRB, 0.65824E+00_JPRB ,&
&  0.67492E+00_JPRB, 0.66893E+00_JPRB, 0.66943E+00_JPRB, 0.65889E+00_JPRB, 0.67231E+00_JPRB, 0.71163E+00_JPRB, 0.79171E+00_JPRB ,&
&  0.90052E+00_JPRB, 0.10297E+01_JPRB, 0.12206E+01_JPRB, 0.14306E+01_JPRB, 0.18730E+01_JPRB, 0.15224E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11044E+01_JPRB, 0.10903E+01_JPRB, 0.10754E+01_JPRB, 0.10398E+01_JPRB, 0.98729E+00_JPRB, 0.10086E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.10479E+01_JPRB, 0.10786E+01_JPRB, 0.10836E+01_JPRB, 0.10864E+01_JPRB, 0.10604E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.94065E+00_JPRB, 0.95828E+00_JPRB, 0.96945E+00_JPRB, 0.10042E+01_JPRB, 0.11026E+01_JPRB, 0.11238E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.11784E+01_JPRB, 0.13066E+01_JPRB, 0.13244E+01_JPRB, 0.13251E+01_JPRB, 0.12339E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.85116E+00_JPRB, 0.67933E+00_JPRB, 0.10305E+01_JPRB, 0.11522E+01_JPRB, 0.10210E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10287E+01_JPRB, 0.10268E+01_JPRB, 0.97738E+00_JPRB, 0.87078E+00_JPRB, 0.81250E+00_JPRB, 0.71555E+00_JPRB ,&
&  0.66612E+00_JPRB, 0.54771E+00_JPRB, 0.44738E+00_JPRB, 0.39310E+00_JPRB, 0.36933E+00_JPRB, 0.39836E+00_JPRB, 0.43593E+00_JPRB ,&
&  0.42884E+00_JPRB, 0.35046E+00_JPRB, 0.37404E+00_JPRB, 0.43420E+00_JPRB, 0.52027E+00_JPRB, 0.39600E+00_JPRB, 0.36072E+00_JPRB ,&
&  0.38293E+00_JPRB, 0.40336E+00_JPRB, 0.42394E+00_JPRB, 0.43641E+00_JPRB, 0.44192E+00_JPRB, 0.44905E+00_JPRB, 0.47801E+00_JPRB ,&
&  0.50148E+00_JPRB, 0.52292E+00_JPRB, 0.54089E+00_JPRB, 0.56560E+00_JPRB, 0.57894E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.74845E+00_JPRB, 0.77034E+00_JPRB, 0.78482E+00_JPRB, 0.78745E+00_JPRB, 0.77811E+00_JPRB, 0.76758E+00_JPRB, 0.74512E+00_JPRB ,&
&  0.71934E+00_JPRB, 0.72719E+00_JPRB, 0.76054E+00_JPRB, 0.83739E+00_JPRB, 0.88785E+00_JPRB, 0.91794E+00_JPRB, 0.96413E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.10310E+01_JPRB, 0.10321E+01_JPRB, 0.10627E+01_JPRB, 0.10980E+01_JPRB, 0.11426E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.15621E+01_JPRB, 0.18591E+01_JPRB, 0.21617E+01_JPRB, 0.23842E+01_JPRB, 0.19668E+01_JPRB, 0.12372E+01_JPRB, 0.12540E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.13917E+01_JPRB, 0.13341E+01_JPRB, 0.13533E+01_JPRB, 0.13811E+01_JPRB, 0.14521E+01_JPRB, 0.15413E+01_JPRB ,&
&  0.16005E+01_JPRB, 0.16388E+01_JPRB, 0.16770E+01_JPRB, 0.17017E+01_JPRB, 0.17361E+01_JPRB, 0.17650E+01_JPRB, 0.17904E+01_JPRB ,&
&  0.17764E+01_JPRB, 0.17334E+01_JPRB, 0.17173E+01_JPRB, 0.17664E+01_JPRB, 0.17548E+01_JPRB, 0.17314E+01_JPRB, 0.17001E+01_JPRB ,&
&  0.16769E+01_JPRB, 0.16874E+01_JPRB, 0.18328E+01_JPRB, 0.18958E+01_JPRB, 0.17586E+01_JPRB, 0.16034E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.12682E+01_JPRB, 0.99730E+00_JPRB, 0.14430E+01_JPRB, 0.15012E+01_JPRB, 0.13744E+01_JPRB, 0.12465E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.11518E+01_JPRB, 0.11033E+01_JPRB, 0.10609E+01_JPRB, 0.93362E+00_JPRB, 0.88413E+00_JPRB, 0.69402E+00_JPRB ,&
&  0.70294E+00_JPRB, 0.60704E+00_JPRB, 0.49746E+00_JPRB, 0.42596E+00_JPRB, 0.40600E+00_JPRB, 0.40542E+00_JPRB, 0.43941E+00_JPRB ,&
&  0.41036E+00_JPRB, 0.40468E+00_JPRB, 0.47729E+00_JPRB, 0.83435E+00_JPRB, 0.18948E+01_JPRB, 0.49655E+00_JPRB, 0.39436E+00_JPRB ,&
&  0.42194E+00_JPRB, 0.45737E+00_JPRB, 0.49126E+00_JPRB, 0.51115E+00_JPRB, 0.52178E+00_JPRB, 0.54206E+00_JPRB, 0.58016E+00_JPRB ,&
&  0.62285E+00_JPRB, 0.65189E+00_JPRB, 0.66461E+00_JPRB, 0.69873E+00_JPRB, 0.72595E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.87845E+00_JPRB, 0.90435E+00_JPRB, 0.92728E+00_JPRB, 0.96178E+00_JPRB, 0.99389E+00_JPRB, 0.10018E+01_JPRB, 0.99496E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10407E+01_JPRB, 0.10759E+01_JPRB, 0.11066E+01_JPRB, 0.11491E+01_JPRB, 0.11852E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.13253E+01_JPRB, 0.14199E+01_JPRB, 0.15228E+01_JPRB, 0.16071E+01_JPRB, 0.17392E+01_JPRB, 0.18800E+01_JPRB ,&
&  0.20168E+01_JPRB, 0.18812E+01_JPRB, 0.18272E+01_JPRB, 0.14569E+01_JPRB, 0.10074E+01_JPRB, 0.12449E+01_JPRB, 0.13190E+01_JPRB ,&
&  0.13936E+01_JPRB, 0.14699E+01_JPRB, 0.15440E+01_JPRB, 0.16229E+01_JPRB, 0.17474E+01_JPRB, 0.18436E+01_JPRB, 0.18979E+01_JPRB ,&
&  0.19516E+01_JPRB, 0.20004E+01_JPRB, 0.20475E+01_JPRB, 0.21054E+01_JPRB, 0.21791E+01_JPRB, 0.22482E+01_JPRB, 0.22870E+01_JPRB ,&
&  0.23393E+01_JPRB, 0.23482E+01_JPRB, 0.23645E+01_JPRB, 0.23839E+01_JPRB, 0.22597E+01_JPRB, 0.21704E+01_JPRB, 0.21032E+01_JPRB ,&
&  0.20720E+01_JPRB, 0.20763E+01_JPRB, 0.21458E+01_JPRB, 0.22092E+01_JPRB, 0.21837E+01_JPRB, 0.19842E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.17279E+01_JPRB, 0.15413E+01_JPRB, 0.14860E+01_JPRB, 0.18594E+01_JPRB, 0.11869E+01_JPRB, 0.13116E+01_JPRB, 0.13808E+01_JPRB ,&
&  0.13096E+01_JPRB, 0.13202E+01_JPRB, 0.12609E+01_JPRB, 0.11161E+01_JPRB, 0.95645E+00_JPRB, 0.67415E+00_JPRB, 0.68662E+00_JPRB ,&
&  0.75260E+00_JPRB, 0.66900E+00_JPRB, 0.54540E+00_JPRB, 0.45538E+00_JPRB, 0.45885E+00_JPRB, 0.45172E+00_JPRB, 0.45821E+00_JPRB ,&
&  0.43895E+00_JPRB, 0.45332E+00_JPRB, 0.49380E+00_JPRB, 0.68949E+00_JPRB, 0.94142E+00_JPRB, 0.48171E+00_JPRB, 0.43050E+00_JPRB ,&
&  0.46755E+00_JPRB, 0.53126E+00_JPRB, 0.60146E+00_JPRB, 0.65139E+00_JPRB, 0.67129E+00_JPRB, 0.69046E+00_JPRB, 0.71102E+00_JPRB ,&
&  0.73486E+00_JPRB, 0.77314E+00_JPRB, 0.82534E+00_JPRB, 0.84805E+00_JPRB, 0.86517E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.90029E+00_JPRB, 0.94893E+00_JPRB, 0.10242E+01_JPRB, 0.10827E+01_JPRB, 0.11158E+01_JPRB, 0.11199E+01_JPRB, 0.10982E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11301E+01_JPRB, 0.11942E+01_JPRB, 0.12485E+01_JPRB, 0.13051E+01_JPRB, 0.13585E+01_JPRB, 0.14134E+01_JPRB ,&
&  0.14633E+01_JPRB, 0.15085E+01_JPRB, 0.15764E+01_JPRB, 0.16590E+01_JPRB, 0.17337E+01_JPRB, 0.18620E+01_JPRB, 0.18513E+01_JPRB ,&
&  0.15016E+01_JPRB, 0.11716E+01_JPRB, 0.14449E+01_JPRB, 0.14807E+01_JPRB, 0.13130E+01_JPRB, 0.13914E+01_JPRB, 0.14954E+01_JPRB ,&
&  0.16225E+01_JPRB, 0.17117E+01_JPRB, 0.17904E+01_JPRB, 0.18463E+01_JPRB, 0.18877E+01_JPRB, 0.19571E+01_JPRB, 0.20143E+01_JPRB ,&
&  0.20881E+01_JPRB, 0.21751E+01_JPRB, 0.22629E+01_JPRB, 0.23738E+01_JPRB, 0.24858E+01_JPRB, 0.26103E+01_JPRB, 0.27021E+01_JPRB ,&
&  0.27786E+01_JPRB, 0.28184E+01_JPRB, 0.27882E+01_JPRB, 0.27027E+01_JPRB, 0.25715E+01_JPRB, 0.24778E+01_JPRB, 0.24030E+01_JPRB ,&
&  0.23946E+01_JPRB, 0.24197E+01_JPRB, 0.25201E+01_JPRB, 0.25786E+01_JPRB, 0.25687E+01_JPRB, 0.22928E+01_JPRB, 0.21500E+01_JPRB ,&
&  0.20195E+01_JPRB, 0.18204E+01_JPRB, 0.18461E+01_JPRB, 0.19463E+01_JPRB, 0.14124E+01_JPRB, 0.14000E+01_JPRB, 0.12680E+01_JPRB ,&
&  0.14183E+01_JPRB, 0.14161E+01_JPRB, 0.13684E+01_JPRB, 0.12654E+01_JPRB, 0.10006E+01_JPRB, 0.71909E+00_JPRB, 0.75347E+00_JPRB ,&
&  0.83877E+00_JPRB, 0.81505E+00_JPRB, 0.62707E+00_JPRB, 0.52269E+00_JPRB, 0.45490E+00_JPRB, 0.45630E+00_JPRB, 0.46477E+00_JPRB ,&
&  0.45671E+00_JPRB, 0.44865E+00_JPRB, 0.49135E+00_JPRB, 0.58087E+00_JPRB, 0.61788E+00_JPRB, 0.49322E+00_JPRB, 0.48224E+00_JPRB ,&
&  0.53402E+00_JPRB, 0.58512E+00_JPRB, 0.62948E+00_JPRB, 0.66388E+00_JPRB, 0.69839E+00_JPRB, 0.73335E+00_JPRB, 0.75776E+00_JPRB ,&
&  0.80512E+00_JPRB, 0.79748E+00_JPRB, 0.83512E+00_JPRB, 0.85364E+00_JPRB, 0.88068E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.88704E+00_JPRB, 0.94842E+00_JPRB, 0.10215E+01_JPRB, 0.10911E+01_JPRB, 0.11238E+01_JPRB, 0.11521E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.11099E+01_JPRB, 0.11863E+01_JPRB, 0.12603E+01_JPRB, 0.13545E+01_JPRB, 0.14360E+01_JPRB, 0.15123E+01_JPRB ,&
&  0.15787E+01_JPRB, 0.16535E+01_JPRB, 0.17428E+01_JPRB, 0.18523E+01_JPRB, 0.19143E+01_JPRB, 0.19807E+01_JPRB, 0.17224E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.12989E+01_JPRB, 0.17845E+01_JPRB, 0.17229E+01_JPRB, 0.16192E+01_JPRB, 0.17101E+01_JPRB, 0.18127E+01_JPRB ,&
&  0.18890E+01_JPRB, 0.19345E+01_JPRB, 0.19209E+01_JPRB, 0.19296E+01_JPRB, 0.19199E+01_JPRB, 0.19410E+01_JPRB, 0.19667E+01_JPRB ,&
&  0.20303E+01_JPRB, 0.21399E+01_JPRB, 0.22724E+01_JPRB, 0.24515E+01_JPRB, 0.26360E+01_JPRB, 0.28132E+01_JPRB, 0.28993E+01_JPRB ,&
&  0.29388E+01_JPRB, 0.29971E+01_JPRB, 0.30025E+01_JPRB, 0.28008E+01_JPRB, 0.27159E+01_JPRB, 0.27334E+01_JPRB, 0.27716E+01_JPRB ,&
&  0.27823E+01_JPRB, 0.29117E+01_JPRB, 0.29144E+01_JPRB, 0.27180E+01_JPRB, 0.23986E+01_JPRB, 0.24427E+01_JPRB, 0.21662E+01_JPRB ,&
&  0.21656E+01_JPRB, 0.20282E+01_JPRB, 0.21486E+01_JPRB, 0.17146E+01_JPRB, 0.16098E+01_JPRB, 0.16034E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.14829E+01_JPRB, 0.14577E+01_JPRB, 0.11948E+01_JPRB, 0.11134E+01_JPRB, 0.83758E+00_JPRB, 0.92184E+00_JPRB ,&
&  0.93337E+00_JPRB, 0.10210E+01_JPRB, 0.70315E+00_JPRB, 0.52991E+00_JPRB, 0.44102E+00_JPRB, 0.44132E+00_JPRB, 0.46502E+00_JPRB ,&
&  0.45497E+00_JPRB, 0.46656E+00_JPRB, 0.48712E+00_JPRB, 0.50899E+00_JPRB, 0.50555E+00_JPRB, 0.48703E+00_JPRB, 0.49842E+00_JPRB ,&
&  0.55248E+00_JPRB, 0.59887E+00_JPRB, 0.61080E+00_JPRB, 0.62324E+00_JPRB, 0.64745E+00_JPRB, 0.69354E+00_JPRB, 0.72587E+00_JPRB ,&
&  0.76360E+00_JPRB, 0.78417E+00_JPRB, 0.80901E+00_JPRB, 0.82219E+00_JPRB, 0.84025E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.72385E+00_JPRB, 0.75594E+00_JPRB, 0.77986E+00_JPRB, 0.77945E+00_JPRB, 0.78371E+00_JPRB, 0.80132E+00_JPRB, 0.84397E+00_JPRB ,&
&  0.89618E+00_JPRB, 0.95748E+00_JPRB, 0.10296E+01_JPRB, 0.11191E+01_JPRB, 0.12348E+01_JPRB, 0.13349E+01_JPRB, 0.14505E+01_JPRB ,&
&  0.15656E+01_JPRB, 0.16802E+01_JPRB, 0.18018E+01_JPRB, 0.19279E+01_JPRB, 0.19864E+01_JPRB, 0.19823E+01_JPRB, 0.15796E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.19263E+01_JPRB, 0.18404E+01_JPRB, 0.19422E+01_JPRB, 0.19656E+01_JPRB, 0.20529E+01_JPRB, 0.20728E+01_JPRB ,&
&  0.20930E+01_JPRB, 0.20239E+01_JPRB, 0.19634E+01_JPRB, 0.19393E+01_JPRB, 0.19379E+01_JPRB, 0.19463E+01_JPRB, 0.19303E+01_JPRB ,&
&  0.19317E+01_JPRB, 0.19905E+01_JPRB, 0.21124E+01_JPRB, 0.22965E+01_JPRB, 0.25220E+01_JPRB, 0.27748E+01_JPRB, 0.29382E+01_JPRB ,&
&  0.30659E+01_JPRB, 0.31977E+01_JPRB, 0.32374E+01_JPRB, 0.30676E+01_JPRB, 0.29679E+01_JPRB, 0.30697E+01_JPRB, 0.33087E+01_JPRB ,&
&  0.32689E+01_JPRB, 0.31550E+01_JPRB, 0.32665E+01_JPRB, 0.31087E+01_JPRB, 0.27642E+01_JPRB, 0.26565E+01_JPRB, 0.22389E+01_JPRB ,&
&  0.23180E+01_JPRB, 0.21038E+01_JPRB, 0.21862E+01_JPRB, 0.17786E+01_JPRB, 0.17255E+01_JPRB, 0.18492E+01_JPRB, 0.18134E+01_JPRB ,&
&  0.16399E+01_JPRB, 0.16481E+01_JPRB, 0.15696E+01_JPRB, 0.13313E+01_JPRB, 0.12053E+01_JPRB, 0.10377E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.97614E+00_JPRB, 0.93262E+00_JPRB, 0.73605E+00_JPRB, 0.48040E+00_JPRB, 0.42656E+00_JPRB, 0.43712E+00_JPRB, 0.47500E+00_JPRB ,&
&  0.47311E+00_JPRB, 0.51083E+00_JPRB, 0.56287E+00_JPRB, 0.55771E+00_JPRB, 0.50542E+00_JPRB, 0.49697E+00_JPRB, 0.51227E+00_JPRB ,&
&  0.56119E+00_JPRB, 0.60460E+00_JPRB, 0.61120E+00_JPRB, 0.60532E+00_JPRB, 0.58776E+00_JPRB, 0.62020E+00_JPRB, 0.64016E+00_JPRB ,&
&  0.67309E+00_JPRB, 0.67345E+00_JPRB, 0.69422E+00_JPRB, 0.71095E+00_JPRB, 0.71723E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.63677E+00_JPRB, 0.65835E+00_JPRB, 0.69500E+00_JPRB, 0.70116E+00_JPRB, 0.68839E+00_JPRB, 0.68782E+00_JPRB, 0.70195E+00_JPRB ,&
&  0.72188E+00_JPRB, 0.74833E+00_JPRB, 0.79377E+00_JPRB, 0.85812E+00_JPRB, 0.95966E+00_JPRB, 0.10635E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.13761E+01_JPRB, 0.15785E+01_JPRB, 0.18041E+01_JPRB, 0.20445E+01_JPRB, 0.22099E+01_JPRB, 0.22084E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.19333E+01_JPRB, 0.19784E+01_JPRB, 0.21714E+01_JPRB, 0.23396E+01_JPRB, 0.24085E+01_JPRB, 0.22778E+01_JPRB ,&
&  0.22187E+01_JPRB, 0.21281E+01_JPRB, 0.20775E+01_JPRB, 0.20258E+01_JPRB, 0.19977E+01_JPRB, 0.19900E+01_JPRB, 0.19398E+01_JPRB ,&
&  0.18775E+01_JPRB, 0.18510E+01_JPRB, 0.18937E+01_JPRB, 0.20180E+01_JPRB, 0.22073E+01_JPRB, 0.24207E+01_JPRB, 0.27236E+01_JPRB ,&
&  0.30465E+01_JPRB, 0.33498E+01_JPRB, 0.36375E+01_JPRB, 0.34338E+01_JPRB, 0.33918E+01_JPRB, 0.33885E+01_JPRB, 0.35739E+01_JPRB ,&
&  0.35209E+01_JPRB, 0.33298E+01_JPRB, 0.36294E+01_JPRB, 0.38279E+01_JPRB, 0.35146E+01_JPRB, 0.30372E+01_JPRB, 0.25101E+01_JPRB ,&
&  0.24371E+01_JPRB, 0.23390E+01_JPRB, 0.22112E+01_JPRB, 0.19691E+01_JPRB, 0.19132E+01_JPRB, 0.21146E+01_JPRB, 0.22707E+01_JPRB ,&
&  0.21178E+01_JPRB, 0.20188E+01_JPRB, 0.18263E+01_JPRB, 0.15635E+01_JPRB, 0.13520E+01_JPRB, 0.12139E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.10314E+01_JPRB, 0.77102E+00_JPRB, 0.47529E+00_JPRB, 0.39154E+00_JPRB, 0.36548E+00_JPRB, 0.43048E+00_JPRB ,&
&  0.48743E+00_JPRB, 0.53675E+00_JPRB, 0.55120E+00_JPRB, 0.68780E+00_JPRB, 0.57683E+00_JPRB, 0.53898E+00_JPRB, 0.53204E+00_JPRB ,&
&  0.56654E+00_JPRB, 0.60325E+00_JPRB, 0.62116E+00_JPRB, 0.59833E+00_JPRB, 0.56561E+00_JPRB, 0.57051E+00_JPRB, 0.57528E+00_JPRB ,&
&  0.59559E+00_JPRB, 0.60496E+00_JPRB, 0.60239E+00_JPRB, 0.61369E+00_JPRB, 0.63205E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.58882E+00_JPRB, 0.60557E+00_JPRB, 0.62368E+00_JPRB, 0.63796E+00_JPRB, 0.64318E+00_JPRB, 0.63653E+00_JPRB, 0.62699E+00_JPRB ,&
&  0.62608E+00_JPRB, 0.63238E+00_JPRB, 0.66223E+00_JPRB, 0.69975E+00_JPRB, 0.76048E+00_JPRB, 0.82368E+00_JPRB, 0.92384E+00_JPRB ,&
&  0.10998E+01_JPRB, 0.13398E+01_JPRB, 0.16718E+01_JPRB, 0.20445E+01_JPRB, 0.21799E+01_JPRB, 0.19213E+01_JPRB, 0.17107E+01_JPRB ,&
&  0.16414E+01_JPRB, 0.19871E+01_JPRB, 0.20485E+01_JPRB, 0.23994E+01_JPRB, 0.27350E+01_JPRB, 0.28745E+01_JPRB, 0.23711E+01_JPRB ,&
&  0.22587E+01_JPRB, 0.21467E+01_JPRB, 0.20360E+01_JPRB, 0.19971E+01_JPRB, 0.19809E+01_JPRB, 0.19935E+01_JPRB, 0.19772E+01_JPRB ,&
&  0.19028E+01_JPRB, 0.18154E+01_JPRB, 0.17713E+01_JPRB, 0.17926E+01_JPRB, 0.19105E+01_JPRB, 0.20568E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.26410E+01_JPRB, 0.31720E+01_JPRB, 0.37527E+01_JPRB, 0.38727E+01_JPRB, 0.37470E+01_JPRB, 0.37304E+01_JPRB, 0.37578E+01_JPRB ,&
&  0.38353E+01_JPRB, 0.41480E+01_JPRB, 0.42378E+01_JPRB, 0.41512E+01_JPRB, 0.38130E+01_JPRB, 0.35424E+01_JPRB, 0.29213E+01_JPRB ,&
&  0.26386E+01_JPRB, 0.24690E+01_JPRB, 0.22483E+01_JPRB, 0.22600E+01_JPRB, 0.22792E+01_JPRB, 0.24340E+01_JPRB, 0.22762E+01_JPRB ,&
&  0.19832E+01_JPRB, 0.17678E+01_JPRB, 0.16591E+01_JPRB, 0.16583E+01_JPRB, 0.15839E+01_JPRB, 0.15297E+01_JPRB, 0.15384E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.46059E+00_JPRB, 0.29630E+00_JPRB, 0.35959E+00_JPRB, 0.26813E+00_JPRB, 0.26136E+00_JPRB, 0.39717E+00_JPRB ,&
&  0.49854E+00_JPRB, 0.60215E+00_JPRB, 0.67672E+00_JPRB, 0.73229E+00_JPRB, 0.76115E+00_JPRB, 0.67110E+00_JPRB, 0.60576E+00_JPRB ,&
&  0.59219E+00_JPRB, 0.61073E+00_JPRB, 0.64496E+00_JPRB, 0.63094E+00_JPRB, 0.58485E+00_JPRB, 0.56757E+00_JPRB, 0.55179E+00_JPRB ,&
&  0.54965E+00_JPRB, 0.57184E+00_JPRB, 0.58642E+00_JPRB, 0.58698E+00_JPRB, 0.58163E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.59578E+00_JPRB, 0.59533E+00_JPRB, 0.60302E+00_JPRB, 0.59317E+00_JPRB, 0.57678E+00_JPRB, 0.56802E+00_JPRB, 0.57280E+00_JPRB ,&
&  0.56550E+00_JPRB, 0.56189E+00_JPRB, 0.58261E+00_JPRB, 0.61570E+00_JPRB, 0.66253E+00_JPRB, 0.71432E+00_JPRB, 0.76286E+00_JPRB ,&
&  0.87330E+00_JPRB, 0.10831E+01_JPRB, 0.14367E+01_JPRB, 0.19445E+01_JPRB, 0.22307E+01_JPRB, 0.20683E+01_JPRB, 0.20047E+01_JPRB ,&
&  0.17943E+01_JPRB, 0.21162E+01_JPRB, 0.25714E+01_JPRB, 0.32741E+01_JPRB, 0.34639E+01_JPRB, 0.32564E+01_JPRB, 0.26011E+01_JPRB ,&
&  0.24886E+01_JPRB, 0.23039E+01_JPRB, 0.20982E+01_JPRB, 0.18988E+01_JPRB, 0.17900E+01_JPRB, 0.17737E+01_JPRB, 0.18093E+01_JPRB ,&
&  0.18252E+01_JPRB, 0.17703E+01_JPRB, 0.17038E+01_JPRB, 0.16780E+01_JPRB, 0.17251E+01_JPRB, 0.18352E+01_JPRB, 0.19755E+01_JPRB ,&
&  0.21765E+01_JPRB, 0.25651E+01_JPRB, 0.33916E+01_JPRB, 0.42360E+01_JPRB, 0.35591E+01_JPRB, 0.41008E+01_JPRB, 0.39251E+01_JPRB ,&
&  0.38905E+01_JPRB, 0.42929E+01_JPRB, 0.43556E+01_JPRB, 0.49227E+01_JPRB, 0.42485E+01_JPRB, 0.37047E+01_JPRB, 0.31181E+01_JPRB ,&
&  0.28499E+01_JPRB, 0.24455E+01_JPRB, 0.23513E+01_JPRB, 0.24627E+01_JPRB, 0.26149E+01_JPRB, 0.27264E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.17927E+01_JPRB, 0.20091E+01_JPRB, 0.19167E+01_JPRB, 0.16649E+01_JPRB, 0.17804E+01_JPRB, 0.19008E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.45625E+00_JPRB, 0.30864E+00_JPRB, 0.25982E+00_JPRB, 0.24417E+00_JPRB, 0.21328E+00_JPRB, 0.24946E+00_JPRB, 0.60842E+00_JPRB ,&
&  0.59280E+00_JPRB, 0.75986E+00_JPRB, 0.86220E+00_JPRB, 0.92440E+00_JPRB, 0.10280E+01_JPRB, 0.89980E+00_JPRB, 0.11405E+01_JPRB ,&
&  0.87288E+00_JPRB, 0.75026E+00_JPRB, 0.72203E+00_JPRB, 0.71229E+00_JPRB, 0.66022E+00_JPRB, 0.62977E+00_JPRB, 0.59878E+00_JPRB ,&
&  0.58990E+00_JPRB, 0.59435E+00_JPRB, 0.59241E+00_JPRB, 0.58804E+00_JPRB, 0.59834E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.62696E+00_JPRB, 0.61972E+00_JPRB, 0.60491E+00_JPRB, 0.56934E+00_JPRB, 0.54195E+00_JPRB, 0.53050E+00_JPRB, 0.52376E+00_JPRB ,&
&  0.51756E+00_JPRB, 0.51532E+00_JPRB, 0.52321E+00_JPRB, 0.54635E+00_JPRB, 0.57437E+00_JPRB, 0.61716E+00_JPRB, 0.67070E+00_JPRB ,&
&  0.76634E+00_JPRB, 0.95249E+00_JPRB, 0.13089E+01_JPRB, 0.16929E+01_JPRB, 0.21002E+01_JPRB, 0.20440E+01_JPRB, 0.21720E+01_JPRB ,&
&  0.19469E+01_JPRB, 0.21229E+01_JPRB, 0.27133E+01_JPRB, 0.37028E+01_JPRB, 0.50959E+01_JPRB, 0.50642E+01_JPRB, 0.43858E+01_JPRB ,&
&  0.37884E+01_JPRB, 0.33384E+01_JPRB, 0.29754E+01_JPRB, 0.25744E+01_JPRB, 0.19772E+01_JPRB, 0.16474E+01_JPRB, 0.15705E+01_JPRB ,&
&  0.15475E+01_JPRB, 0.15838E+01_JPRB, 0.15515E+01_JPRB, 0.15384E+01_JPRB, 0.16104E+01_JPRB, 0.17228E+01_JPRB, 0.18716E+01_JPRB ,&
&  0.19717E+01_JPRB, 0.21212E+01_JPRB, 0.26507E+01_JPRB, 0.38219E+01_JPRB, 0.40072E+01_JPRB, 0.37949E+01_JPRB, 0.38143E+01_JPRB ,&
&  0.38151E+01_JPRB, 0.44699E+01_JPRB, 0.50272E+01_JPRB, 0.59524E+01_JPRB, 0.43535E+01_JPRB, 0.37083E+01_JPRB, 0.33412E+01_JPRB ,&
&  0.30569E+01_JPRB, 0.29397E+01_JPRB, 0.23566E+01_JPRB, 0.23714E+01_JPRB, 0.27128E+01_JPRB, 0.22882E+01_JPRB, 0.19847E+01_JPRB ,&
&  0.22406E+01_JPRB, 0.19830E+01_JPRB, 0.18738E+01_JPRB, 0.16630E+01_JPRB, 0.20305E+01_JPRB, 0.16987E+01_JPRB, 0.67710E+00_JPRB ,&
&  0.41333E+00_JPRB, 0.32795E+00_JPRB, 0.24591E+00_JPRB, 0.23472E+00_JPRB, 0.22412E+00_JPRB, 0.26489E+00_JPRB, 0.51595E+00_JPRB ,&
&  0.59430E+00_JPRB, 0.87369E+00_JPRB, 0.10111E+01_JPRB, 0.11342E+01_JPRB, 0.12020E+01_JPRB, 0.13600E+01_JPRB, 0.25529E+01_JPRB ,&
&  0.23322E+01_JPRB, 0.17949E+01_JPRB, 0.13600E+01_JPRB, 0.10896E+01_JPRB, 0.88273E+00_JPRB, 0.77187E+00_JPRB, 0.70237E+00_JPRB ,&
&  0.67069E+00_JPRB, 0.65414E+00_JPRB, 0.64158E+00_JPRB, 0.63087E+00_JPRB, 0.62052E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.65253E+00_JPRB, 0.65119E+00_JPRB, 0.63458E+00_JPRB, 0.57712E+00_JPRB, 0.54165E+00_JPRB, 0.53742E+00_JPRB, 0.52459E+00_JPRB ,&
&  0.50783E+00_JPRB, 0.49336E+00_JPRB, 0.48855E+00_JPRB, 0.49749E+00_JPRB, 0.50539E+00_JPRB, 0.52462E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.69831E+00_JPRB, 0.91164E+00_JPRB, 0.12073E+01_JPRB, 0.14761E+01_JPRB, 0.17265E+01_JPRB, 0.17859E+01_JPRB, 0.21276E+01_JPRB ,&
&  0.21863E+01_JPRB, 0.21942E+01_JPRB, 0.26074E+01_JPRB, 0.37694E+01_JPRB, 0.56118E+01_JPRB, 0.67083E+01_JPRB, 0.70027E+01_JPRB ,&
&  0.59938E+01_JPRB, 0.49124E+01_JPRB, 0.41773E+01_JPRB, 0.36740E+01_JPRB, 0.29944E+01_JPRB, 0.21690E+01_JPRB, 0.16496E+01_JPRB ,&
&  0.14348E+01_JPRB, 0.14504E+01_JPRB, 0.14843E+01_JPRB, 0.14551E+01_JPRB, 0.15243E+01_JPRB, 0.16396E+01_JPRB, 0.18316E+01_JPRB ,&
&  0.19423E+01_JPRB, 0.20037E+01_JPRB, 0.22124E+01_JPRB, 0.30521E+01_JPRB, 0.42290E+01_JPRB, 0.46365E+01_JPRB, 0.39639E+01_JPRB ,&
&  0.34609E+01_JPRB, 0.36707E+01_JPRB, 0.56719E+01_JPRB, 0.60651E+01_JPRB, 0.44769E+01_JPRB, 0.39868E+01_JPRB, 0.35477E+01_JPRB ,&
&  0.34134E+01_JPRB, 0.33787E+01_JPRB, 0.27946E+01_JPRB, 0.27006E+01_JPRB, 0.24183E+01_JPRB, 0.20276E+01_JPRB, 0.22993E+01_JPRB ,&
&  0.24142E+01_JPRB, 0.21435E+01_JPRB, 0.18326E+01_JPRB, 0.16042E+01_JPRB, 0.17037E+01_JPRB, 0.11429E+01_JPRB, 0.71072E+00_JPRB ,&
&  0.55714E+00_JPRB, 0.42464E+00_JPRB, 0.33439E+00_JPRB, 0.33211E+00_JPRB, 0.28567E+00_JPRB, 0.31923E+00_JPRB, 0.43927E+00_JPRB ,&
&  0.66933E+00_JPRB, 0.86225E+00_JPRB, 0.11434E+01_JPRB, 0.13481E+01_JPRB, 0.13625E+01_JPRB, 0.13785E+01_JPRB, 0.25239E+01_JPRB ,&
&  0.18666E+01_JPRB, 0.17784E+01_JPRB, 0.17556E+01_JPRB, 0.14390E+01_JPRB, 0.11409E+01_JPRB, 0.95017E+00_JPRB, 0.81570E+00_JPRB ,&
&  0.73282E+00_JPRB, 0.68521E+00_JPRB, 0.65932E+00_JPRB, 0.65638E+00_JPRB, 0.66270E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.72052E+00_JPRB, 0.67863E+00_JPRB, 0.65704E+00_JPRB, 0.61742E+00_JPRB, 0.55970E+00_JPRB, 0.53175E+00_JPRB, 0.53427E+00_JPRB ,&
&  0.52186E+00_JPRB, 0.50154E+00_JPRB, 0.48248E+00_JPRB, 0.46808E+00_JPRB, 0.47151E+00_JPRB, 0.48334E+00_JPRB, 0.51056E+00_JPRB ,&
&  0.64171E+00_JPRB, 0.97754E+00_JPRB, 0.10039E+01_JPRB, 0.11340E+01_JPRB, 0.13619E+01_JPRB, 0.17281E+01_JPRB, 0.19224E+01_JPRB ,&
&  0.23370E+01_JPRB, 0.23955E+01_JPRB, 0.26051E+01_JPRB, 0.38169E+01_JPRB, 0.68698E+01_JPRB, 0.81753E+01_JPRB, 0.89571E+01_JPRB ,&
&  0.87567E+01_JPRB, 0.72297E+01_JPRB, 0.57477E+01_JPRB, 0.45140E+01_JPRB, 0.36844E+01_JPRB, 0.31139E+01_JPRB, 0.23696E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.16011E+01_JPRB, 0.15678E+01_JPRB, 0.15380E+01_JPRB, 0.15321E+01_JPRB, 0.15960E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.20478E+01_JPRB, 0.21696E+01_JPRB, 0.27667E+01_JPRB, 0.34737E+01_JPRB, 0.39892E+01_JPRB, 0.49167E+01_JPRB ,&
&  0.44728E+01_JPRB, 0.47190E+01_JPRB, 0.55610E+01_JPRB, 0.67425E+01_JPRB, 0.49135E+01_JPRB, 0.40503E+01_JPRB, 0.37062E+01_JPRB ,&
&  0.29672E+01_JPRB, 0.28210E+01_JPRB, 0.30240E+01_JPRB, 0.28325E+01_JPRB, 0.22643E+01_JPRB, 0.24932E+01_JPRB, 0.33322E+01_JPRB ,&
&  0.25244E+01_JPRB, 0.24209E+01_JPRB, 0.23877E+01_JPRB, 0.24573E+01_JPRB, 0.15725E+01_JPRB, 0.10215E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.14877E+01_JPRB, 0.79606E+00_JPRB, 0.55699E+00_JPRB, 0.64165E+00_JPRB, 0.45118E+00_JPRB, 0.45129E+00_JPRB, 0.52946E+00_JPRB ,&
&  0.58942E+00_JPRB, 0.64316E+00_JPRB, 0.11195E+01_JPRB, 0.12109E+01_JPRB, 0.13083E+01_JPRB, 0.12004E+01_JPRB, 0.17436E+01_JPRB ,&
&  0.18662E+01_JPRB, 0.15235E+01_JPRB, 0.16013E+01_JPRB, 0.14663E+01_JPRB, 0.12674E+01_JPRB, 0.11073E+01_JPRB, 0.10001E+01_JPRB ,&
&  0.89956E+00_JPRB, 0.82528E+00_JPRB, 0.78481E+00_JPRB, 0.76721E+00_JPRB, 0.76307E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.70488E+00_JPRB, 0.62139E+00_JPRB, 0.58429E+00_JPRB, 0.58121E+00_JPRB, 0.57367E+00_JPRB, 0.53079E+00_JPRB, 0.51733E+00_JPRB ,&
&  0.53281E+00_JPRB, 0.51123E+00_JPRB, 0.47977E+00_JPRB, 0.46421E+00_JPRB, 0.46801E+00_JPRB, 0.47325E+00_JPRB, 0.46983E+00_JPRB ,&
&  0.54691E+00_JPRB, 0.74515E+00_JPRB, 0.83940E+00_JPRB, 0.95199E+00_JPRB, 0.12171E+01_JPRB, 0.15579E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.22158E+01_JPRB, 0.24896E+01_JPRB, 0.29128E+01_JPRB, 0.42689E+01_JPRB, 0.61341E+01_JPRB, 0.79060E+01_JPRB, 0.85200E+01_JPRB ,&
&  0.92963E+01_JPRB, 0.88331E+01_JPRB, 0.70200E+01_JPRB, 0.56967E+01_JPRB, 0.47881E+01_JPRB, 0.41986E+01_JPRB, 0.35472E+01_JPRB ,&
&  0.27603E+01_JPRB, 0.21411E+01_JPRB, 0.18694E+01_JPRB, 0.17489E+01_JPRB, 0.17212E+01_JPRB, 0.16741E+01_JPRB, 0.17515E+01_JPRB ,&
&  0.19141E+01_JPRB, 0.21091E+01_JPRB, 0.21994E+01_JPRB, 0.27023E+01_JPRB, 0.35511E+01_JPRB, 0.41381E+01_JPRB, 0.47268E+01_JPRB ,&
&  0.48190E+01_JPRB, 0.46442E+01_JPRB, 0.55365E+01_JPRB, 0.57979E+01_JPRB, 0.52977E+01_JPRB, 0.44585E+01_JPRB, 0.41960E+01_JPRB ,&
&  0.32898E+01_JPRB, 0.29295E+01_JPRB, 0.28760E+01_JPRB, 0.26871E+01_JPRB, 0.27845E+01_JPRB, 0.34853E+01_JPRB, 0.33862E+01_JPRB ,&
&  0.29518E+01_JPRB, 0.26814E+01_JPRB, 0.24179E+01_JPRB, 0.20809E+01_JPRB, 0.21090E+01_JPRB, 0.15387E+01_JPRB, 0.17685E+01_JPRB ,&
&  0.17930E+01_JPRB, 0.16391E+01_JPRB, 0.13098E+01_JPRB, 0.10442E+01_JPRB, 0.77967E+00_JPRB, 0.72969E+00_JPRB, 0.62288E+00_JPRB ,&
&  0.59396E+00_JPRB, 0.58313E+00_JPRB, 0.81896E+00_JPRB, 0.99887E+00_JPRB, 0.10695E+01_JPRB, 0.99936E+00_JPRB, 0.13696E+01_JPRB ,&
&  0.14403E+01_JPRB, 0.15526E+01_JPRB, 0.14395E+01_JPRB, 0.12796E+01_JPRB, 0.11377E+01_JPRB, 0.10480E+01_JPRB, 0.10123E+01_JPRB ,&
&  0.93643E+00_JPRB, 0.89693E+00_JPRB, 0.87373E+00_JPRB, 0.84032E+00_JPRB, 0.79187E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.66397E+00_JPRB, 0.58321E+00_JPRB, 0.54257E+00_JPRB, 0.52655E+00_JPRB, 0.52926E+00_JPRB, 0.52702E+00_JPRB, 0.50987E+00_JPRB ,&
&  0.50160E+00_JPRB, 0.51172E+00_JPRB, 0.49705E+00_JPRB, 0.48356E+00_JPRB, 0.47912E+00_JPRB, 0.47610E+00_JPRB, 0.46073E+00_JPRB ,&
&  0.49596E+00_JPRB, 0.62805E+00_JPRB, 0.68628E+00_JPRB, 0.84746E+00_JPRB, 0.10649E+01_JPRB, 0.14070E+01_JPRB, 0.18057E+01_JPRB ,&
&  0.20277E+01_JPRB, 0.23018E+01_JPRB, 0.27062E+01_JPRB, 0.37234E+01_JPRB, 0.49313E+01_JPRB, 0.60105E+01_JPRB, 0.63099E+01_JPRB ,&
&  0.59666E+01_JPRB, 0.60976E+01_JPRB, 0.57216E+01_JPRB, 0.51204E+01_JPRB, 0.46065E+01_JPRB, 0.42151E+01_JPRB, 0.37372E+01_JPRB ,&
&  0.31792E+01_JPRB, 0.26168E+01_JPRB, 0.22167E+01_JPRB, 0.20053E+01_JPRB, 0.19161E+01_JPRB, 0.19756E+01_JPRB, 0.20358E+01_JPRB ,&
&  0.20644E+01_JPRB, 0.21794E+01_JPRB, 0.21523E+01_JPRB, 0.24264E+01_JPRB, 0.30456E+01_JPRB, 0.40455E+01_JPRB, 0.44901E+01_JPRB ,&
&  0.52846E+01_JPRB, 0.51365E+01_JPRB, 0.58593E+01_JPRB, 0.64084E+01_JPRB, 0.48107E+01_JPRB, 0.47855E+01_JPRB, 0.50097E+01_JPRB ,&
&  0.48089E+01_JPRB, 0.43351E+01_JPRB, 0.42227E+01_JPRB, 0.36617E+01_JPRB, 0.31473E+01_JPRB, 0.37098E+01_JPRB, 0.33714E+01_JPRB ,&
&  0.29783E+01_JPRB, 0.27498E+01_JPRB, 0.24497E+01_JPRB, 0.24229E+01_JPRB, 0.20907E+01_JPRB, 0.19958E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.16255E+01_JPRB, 0.13759E+01_JPRB, 0.14265E+01_JPRB, 0.12114E+01_JPRB, 0.10573E+01_JPRB, 0.86832E+00_JPRB, 0.67799E+00_JPRB ,&
&  0.59093E+00_JPRB, 0.56554E+00_JPRB, 0.62350E+00_JPRB, 0.80710E+00_JPRB, 0.96120E+00_JPRB, 0.88729E+00_JPRB, 0.10518E+01_JPRB ,&
&  0.12734E+01_JPRB, 0.13826E+01_JPRB, 0.12099E+01_JPRB, 0.11818E+01_JPRB, 0.10519E+01_JPRB, 0.95154E+00_JPRB, 0.92283E+00_JPRB ,&
&  0.85564E+00_JPRB, 0.83433E+00_JPRB, 0.81921E+00_JPRB, 0.79237E+00_JPRB, 0.74762E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.61688E+00_JPRB, 0.54792E+00_JPRB, 0.49887E+00_JPRB, 0.48690E+00_JPRB, 0.49883E+00_JPRB, 0.50812E+00_JPRB, 0.51041E+00_JPRB ,&
&  0.50105E+00_JPRB, 0.50608E+00_JPRB, 0.51033E+00_JPRB, 0.50387E+00_JPRB, 0.49777E+00_JPRB, 0.49161E+00_JPRB, 0.46865E+00_JPRB ,&
&  0.46423E+00_JPRB, 0.59416E+00_JPRB, 0.61151E+00_JPRB, 0.67405E+00_JPRB, 0.84196E+00_JPRB, 0.12591E+01_JPRB, 0.16636E+01_JPRB ,&
&  0.19248E+01_JPRB, 0.21362E+01_JPRB, 0.24821E+01_JPRB, 0.29880E+01_JPRB, 0.34725E+01_JPRB, 0.37328E+01_JPRB, 0.38179E+01_JPRB ,&
&  0.42255E+01_JPRB, 0.35449E+01_JPRB, 0.35891E+01_JPRB, 0.32482E+01_JPRB, 0.30569E+01_JPRB, 0.28634E+01_JPRB, 0.26889E+01_JPRB ,&
&  0.25112E+01_JPRB, 0.22981E+01_JPRB, 0.20778E+01_JPRB, 0.19356E+01_JPRB, 0.18713E+01_JPRB, 0.18645E+01_JPRB, 0.19219E+01_JPRB ,&
&  0.19932E+01_JPRB, 0.20575E+01_JPRB, 0.21160E+01_JPRB, 0.20309E+01_JPRB, 0.24348E+01_JPRB, 0.33434E+01_JPRB, 0.43137E+01_JPRB ,&
&  0.43001E+01_JPRB, 0.40088E+01_JPRB, 0.64360E+01_JPRB, 0.66938E+01_JPRB, 0.59308E+01_JPRB, 0.58116E+01_JPRB, 0.57255E+01_JPRB ,&
&  0.54811E+01_JPRB, 0.47895E+01_JPRB, 0.40039E+01_JPRB, 0.34812E+01_JPRB, 0.39171E+01_JPRB, 0.37392E+01_JPRB, 0.34240E+01_JPRB ,&
&  0.31018E+01_JPRB, 0.29170E+01_JPRB, 0.26673E+01_JPRB, 0.24609E+01_JPRB, 0.23264E+01_JPRB, 0.23692E+01_JPRB, 0.21524E+01_JPRB ,&
&  0.19547E+01_JPRB, 0.19795E+01_JPRB, 0.16169E+01_JPRB, 0.12505E+01_JPRB, 0.10342E+01_JPRB, 0.88602E+00_JPRB, 0.74041E+00_JPRB ,&
&  0.59736E+00_JPRB, 0.55428E+00_JPRB, 0.56124E+00_JPRB, 0.64765E+00_JPRB, 0.82213E+00_JPRB, 0.79360E+00_JPRB, 0.85883E+00_JPRB ,&
&  0.87968E+00_JPRB, 0.11304E+01_JPRB, 0.11039E+01_JPRB, 0.10480E+01_JPRB, 0.98675E+00_JPRB, 0.89228E+00_JPRB, 0.82284E+00_JPRB ,&
&  0.75834E+00_JPRB, 0.75928E+00_JPRB, 0.75716E+00_JPRB, 0.72831E+00_JPRB, 0.69198E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.58795E+00_JPRB, 0.52630E+00_JPRB, 0.47915E+00_JPRB, 0.46850E+00_JPRB, 0.48928E+00_JPRB, 0.50100E+00_JPRB, 0.50740E+00_JPRB ,&
&  0.50463E+00_JPRB, 0.50294E+00_JPRB, 0.51502E+00_JPRB, 0.52043E+00_JPRB, 0.52473E+00_JPRB, 0.51542E+00_JPRB, 0.48595E+00_JPRB ,&
&  0.48188E+00_JPRB, 0.58515E+00_JPRB, 0.56364E+00_JPRB, 0.63121E+00_JPRB, 0.74682E+00_JPRB, 0.10415E+01_JPRB, 0.13095E+01_JPRB ,&
&  0.15608E+01_JPRB, 0.17007E+01_JPRB, 0.18704E+01_JPRB, 0.21264E+01_JPRB, 0.22656E+01_JPRB, 0.21961E+01_JPRB, 0.21101E+01_JPRB ,&
&  0.20168E+01_JPRB, 0.20032E+01_JPRB, 0.17327E+01_JPRB, 0.18417E+01_JPRB, 0.17885E+01_JPRB, 0.15561E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.17107E+01_JPRB, 0.17174E+01_JPRB, 0.16790E+01_JPRB, 0.16507E+01_JPRB, 0.16480E+01_JPRB, 0.16452E+01_JPRB, 0.16362E+01_JPRB ,&
&  0.16383E+01_JPRB, 0.16763E+01_JPRB, 0.18741E+01_JPRB, 0.18982E+01_JPRB, 0.19563E+01_JPRB, 0.23865E+01_JPRB, 0.35067E+01_JPRB ,&
&  0.47202E+01_JPRB, 0.47698E+01_JPRB, 0.47717E+01_JPRB, 0.61504E+01_JPRB, 0.68468E+01_JPRB, 0.63397E+01_JPRB, 0.57980E+01_JPRB ,&
&  0.56304E+01_JPRB, 0.51184E+01_JPRB, 0.45894E+01_JPRB, 0.41725E+01_JPRB, 0.42104E+01_JPRB, 0.41047E+01_JPRB, 0.37784E+01_JPRB ,&
&  0.33546E+01_JPRB, 0.30181E+01_JPRB, 0.28473E+01_JPRB, 0.26367E+01_JPRB, 0.23460E+01_JPRB, 0.21976E+01_JPRB, 0.21856E+01_JPRB ,&
&  0.20905E+01_JPRB, 0.17663E+01_JPRB, 0.13123E+01_JPRB, 0.11397E+01_JPRB, 0.90177E+00_JPRB, 0.76973E+00_JPRB, 0.72170E+00_JPRB ,&
&  0.59238E+00_JPRB, 0.51782E+00_JPRB, 0.51966E+00_JPRB, 0.53946E+00_JPRB, 0.66744E+00_JPRB, 0.65821E+00_JPRB, 0.73172E+00_JPRB ,&
&  0.73099E+00_JPRB, 0.69264E+00_JPRB, 0.89143E+00_JPRB, 0.93005E+00_JPRB, 0.89585E+00_JPRB, 0.79913E+00_JPRB, 0.71498E+00_JPRB ,&
&  0.67947E+00_JPRB, 0.70230E+00_JPRB, 0.69871E+00_JPRB, 0.68123E+00_JPRB, 0.65408E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.55429E+00_JPRB, 0.49562E+00_JPRB, 0.45608E+00_JPRB, 0.44091E+00_JPRB, 0.45685E+00_JPRB, 0.47755E+00_JPRB, 0.49007E+00_JPRB ,&
&  0.50224E+00_JPRB, 0.51129E+00_JPRB, 0.51897E+00_JPRB, 0.52787E+00_JPRB, 0.53553E+00_JPRB, 0.52209E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.52186E+00_JPRB, 0.53339E+00_JPRB, 0.54070E+00_JPRB, 0.56543E+00_JPRB, 0.81099E+00_JPRB, 0.94571E+00_JPRB, 0.11114E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.13463E+01_JPRB, 0.13474E+01_JPRB, 0.13285E+01_JPRB, 0.13084E+01_JPRB, 0.12977E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.97102E+00_JPRB, 0.93847E+00_JPRB, 0.99762E+00_JPRB, 0.10882E+01_JPRB, 0.95578E+00_JPRB, 0.10620E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11896E+01_JPRB, 0.12232E+01_JPRB, 0.12331E+01_JPRB, 0.12584E+01_JPRB, 0.12939E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.13869E+01_JPRB, 0.14785E+01_JPRB, 0.17332E+01_JPRB, 0.18911E+01_JPRB, 0.23513E+01_JPRB, 0.37314E+01_JPRB ,&
&  0.48608E+01_JPRB, 0.51824E+01_JPRB, 0.62859E+01_JPRB, 0.74831E+01_JPRB, 0.68356E+01_JPRB, 0.60190E+01_JPRB, 0.54599E+01_JPRB ,&
&  0.50277E+01_JPRB, 0.46471E+01_JPRB, 0.43557E+01_JPRB, 0.41231E+01_JPRB, 0.40566E+01_JPRB, 0.41352E+01_JPRB, 0.39647E+01_JPRB ,&
&  0.35636E+01_JPRB, 0.32012E+01_JPRB, 0.30152E+01_JPRB, 0.27591E+01_JPRB, 0.25439E+01_JPRB, 0.22919E+01_JPRB, 0.22673E+01_JPRB ,&
&  0.22076E+01_JPRB, 0.15291E+01_JPRB, 0.12127E+01_JPRB, 0.11194E+01_JPRB, 0.81910E+00_JPRB, 0.69416E+00_JPRB, 0.71937E+00_JPRB ,&
&  0.57368E+00_JPRB, 0.49179E+00_JPRB, 0.52368E+00_JPRB, 0.56013E+00_JPRB, 0.57836E+00_JPRB, 0.57222E+00_JPRB, 0.55079E+00_JPRB ,&
&  0.49774E+00_JPRB, 0.52425E+00_JPRB, 0.71782E+00_JPRB, 0.75687E+00_JPRB, 0.75276E+00_JPRB, 0.70790E+00_JPRB, 0.66901E+00_JPRB ,&
&  0.62603E+00_JPRB, 0.64032E+00_JPRB, 0.63917E+00_JPRB, 0.62900E+00_JPRB, 0.61180E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.55387E+00_JPRB, 0.49358E+00_JPRB, 0.44456E+00_JPRB, 0.42998E+00_JPRB, 0.45037E+00_JPRB, 0.47035E+00_JPRB, 0.48381E+00_JPRB ,&
&  0.50224E+00_JPRB, 0.51684E+00_JPRB, 0.52886E+00_JPRB, 0.53942E+00_JPRB, 0.54658E+00_JPRB, 0.53987E+00_JPRB, 0.58958E+00_JPRB ,&
&  0.45762E+00_JPRB, 0.46677E+00_JPRB, 0.45797E+00_JPRB, 0.53211E+00_JPRB, 0.82753E+00_JPRB, 0.83241E+00_JPRB, 0.88920E+00_JPRB ,&
&  0.87678E+00_JPRB, 0.87014E+00_JPRB, 0.83612E+00_JPRB, 0.80708E+00_JPRB, 0.79210E+00_JPRB, 0.80852E+00_JPRB, 0.78713E+00_JPRB ,&
&  0.69079E+00_JPRB, 0.62379E+00_JPRB, 0.60632E+00_JPRB, 0.60943E+00_JPRB, 0.66935E+00_JPRB, 0.71534E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.84895E+00_JPRB, 0.90538E+00_JPRB, 0.96344E+00_JPRB, 0.10099E+01_JPRB, 0.10299E+01_JPRB, 0.10385E+01_JPRB, 0.10496E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.11018E+01_JPRB, 0.11842E+01_JPRB, 0.14876E+01_JPRB, 0.17499E+01_JPRB, 0.26540E+01_JPRB, 0.47485E+01_JPRB ,&
&  0.53177E+01_JPRB, 0.48769E+01_JPRB, 0.56158E+01_JPRB, 0.68645E+01_JPRB, 0.61949E+01_JPRB, 0.52618E+01_JPRB, 0.44271E+01_JPRB ,&
&  0.38829E+01_JPRB, 0.35453E+01_JPRB, 0.34581E+01_JPRB, 0.35933E+01_JPRB, 0.36548E+01_JPRB, 0.37926E+01_JPRB, 0.37995E+01_JPRB ,&
&  0.36409E+01_JPRB, 0.35151E+01_JPRB, 0.33673E+01_JPRB, 0.31634E+01_JPRB, 0.29180E+01_JPRB, 0.27397E+01_JPRB, 0.25393E+01_JPRB ,&
&  0.22805E+01_JPRB, 0.19458E+01_JPRB, 0.14827E+01_JPRB, 0.12052E+01_JPRB, 0.89128E+00_JPRB, 0.79818E+00_JPRB, 0.90954E+00_JPRB ,&
&  0.66108E+00_JPRB, 0.55832E+00_JPRB, 0.54346E+00_JPRB, 0.56312E+00_JPRB, 0.54302E+00_JPRB, 0.53242E+00_JPRB, 0.48177E+00_JPRB ,&
&  0.42602E+00_JPRB, 0.49423E+00_JPRB, 0.66619E+00_JPRB, 0.69495E+00_JPRB, 0.68575E+00_JPRB, 0.66688E+00_JPRB, 0.53552E+00_JPRB ,&
&  0.64115E+00_JPRB, 0.60748E+00_JPRB, 0.60588E+00_JPRB, 0.59764E+00_JPRB, 0.58811E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.56792E+00_JPRB, 0.51198E+00_JPRB, 0.45431E+00_JPRB, 0.44303E+00_JPRB, 0.46389E+00_JPRB, 0.47610E+00_JPRB, 0.48485E+00_JPRB ,&
&  0.50245E+00_JPRB, 0.51134E+00_JPRB, 0.52458E+00_JPRB, 0.54067E+00_JPRB, 0.54788E+00_JPRB, 0.57603E+00_JPRB, 0.51768E+00_JPRB ,&
&  0.43726E+00_JPRB, 0.44133E+00_JPRB, 0.43371E+00_JPRB, 0.46404E+00_JPRB, 0.55079E+00_JPRB, 0.56700E+00_JPRB, 0.57809E+00_JPRB ,&
&  0.60129E+00_JPRB, 0.58458E+00_JPRB, 0.59043E+00_JPRB, 0.58407E+00_JPRB, 0.57512E+00_JPRB, 0.58722E+00_JPRB, 0.60211E+00_JPRB ,&
&  0.54665E+00_JPRB, 0.50631E+00_JPRB, 0.48877E+00_JPRB, 0.48260E+00_JPRB, 0.54496E+00_JPRB, 0.62400E+00_JPRB, 0.66851E+00_JPRB ,&
&  0.70244E+00_JPRB, 0.74225E+00_JPRB, 0.78890E+00_JPRB, 0.83962E+00_JPRB, 0.88743E+00_JPRB, 0.92065E+00_JPRB, 0.93376E+00_JPRB ,&
&  0.94648E+00_JPRB, 0.97023E+00_JPRB, 0.10090E+01_JPRB, 0.11581E+01_JPRB, 0.14794E+01_JPRB, 0.22494E+01_JPRB, 0.33262E+01_JPRB ,&
&  0.35252E+01_JPRB, 0.35000E+01_JPRB, 0.36496E+01_JPRB, 0.39332E+01_JPRB, 0.39329E+01_JPRB, 0.34515E+01_JPRB, 0.29704E+01_JPRB ,&
&  0.27811E+01_JPRB, 0.26862E+01_JPRB, 0.28534E+01_JPRB, 0.30375E+01_JPRB, 0.31314E+01_JPRB, 0.32768E+01_JPRB, 0.33196E+01_JPRB ,&
&  0.34376E+01_JPRB, 0.35509E+01_JPRB, 0.35389E+01_JPRB, 0.33517E+01_JPRB, 0.31721E+01_JPRB, 0.30059E+01_JPRB, 0.27360E+01_JPRB ,&
&  0.24467E+01_JPRB, 0.21004E+01_JPRB, 0.16888E+01_JPRB, 0.13538E+01_JPRB, 0.10534E+01_JPRB, 0.97135E+00_JPRB, 0.84693E+00_JPRB ,&
&  0.64564E+00_JPRB, 0.55847E+00_JPRB, 0.53362E+00_JPRB, 0.53961E+00_JPRB, 0.51038E+00_JPRB, 0.47822E+00_JPRB, 0.47745E+00_JPRB ,&
&  0.51793E+00_JPRB, 0.64860E+00_JPRB, 0.66941E+00_JPRB, 0.67654E+00_JPRB, 0.67442E+00_JPRB, 0.67001E+00_JPRB, 0.48365E+00_JPRB ,&
&  0.52361E+00_JPRB, 0.63647E+00_JPRB, 0.62140E+00_JPRB, 0.61155E+00_JPRB, 0.59947E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.56796E+00_JPRB, 0.52711E+00_JPRB, 0.48192E+00_JPRB, 0.46488E+00_JPRB, 0.47582E+00_JPRB, 0.48880E+00_JPRB, 0.47721E+00_JPRB ,&
&  0.51388E+00_JPRB, 0.50059E+00_JPRB, 0.52517E+00_JPRB, 0.56248E+00_JPRB, 0.60132E+00_JPRB, 0.52219E+00_JPRB, 0.39943E+00_JPRB ,&
&  0.36780E+00_JPRB, 0.37738E+00_JPRB, 0.40090E+00_JPRB, 0.41448E+00_JPRB, 0.42899E+00_JPRB, 0.43119E+00_JPRB, 0.43656E+00_JPRB ,&
&  0.44722E+00_JPRB, 0.46374E+00_JPRB, 0.50209E+00_JPRB, 0.52425E+00_JPRB, 0.52923E+00_JPRB, 0.52795E+00_JPRB, 0.54203E+00_JPRB ,&
&  0.50300E+00_JPRB, 0.48945E+00_JPRB, 0.49105E+00_JPRB, 0.46478E+00_JPRB, 0.52902E+00_JPRB, 0.55767E+00_JPRB, 0.57986E+00_JPRB ,&
&  0.60404E+00_JPRB, 0.64307E+00_JPRB, 0.69344E+00_JPRB, 0.74287E+00_JPRB, 0.78979E+00_JPRB, 0.83096E+00_JPRB, 0.86487E+00_JPRB ,&
&  0.89440E+00_JPRB, 0.92831E+00_JPRB, 0.96837E+00_JPRB, 0.10461E+01_JPRB, 0.13209E+01_JPRB, 0.17908E+01_JPRB, 0.24373E+01_JPRB ,&
&  0.25927E+01_JPRB, 0.28063E+01_JPRB, 0.29982E+01_JPRB, 0.27755E+01_JPRB, 0.27980E+01_JPRB, 0.27722E+01_JPRB, 0.25187E+01_JPRB ,&
&  0.24016E+01_JPRB, 0.22836E+01_JPRB, 0.23294E+01_JPRB, 0.24616E+01_JPRB, 0.26686E+01_JPRB, 0.28329E+01_JPRB, 0.29462E+01_JPRB ,&
&  0.31290E+01_JPRB, 0.34517E+01_JPRB, 0.35538E+01_JPRB, 0.33783E+01_JPRB, 0.31972E+01_JPRB, 0.30124E+01_JPRB, 0.27481E+01_JPRB ,&
&  0.24742E+01_JPRB, 0.21848E+01_JPRB, 0.18488E+01_JPRB, 0.15052E+01_JPRB, 0.12054E+01_JPRB, 0.10368E+01_JPRB, 0.86472E+00_JPRB ,&
&  0.69303E+00_JPRB, 0.57835E+00_JPRB, 0.56892E+00_JPRB, 0.55660E+00_JPRB, 0.52234E+00_JPRB, 0.47927E+00_JPRB, 0.45134E+00_JPRB ,&
&  0.47892E+00_JPRB, 0.54465E+00_JPRB, 0.70982E+00_JPRB, 0.68274E+00_JPRB, 0.67951E+00_JPRB, 0.66721E+00_JPRB, 0.55086E+00_JPRB ,&
&  0.52024E+00_JPRB, 0.65610E+00_JPRB, 0.61981E+00_JPRB, 0.61575E+00_JPRB, 0.60117E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.59094E+00_JPRB, 0.55944E+00_JPRB, 0.52865E+00_JPRB, 0.50215E+00_JPRB, 0.48906E+00_JPRB, 0.47501E+00_JPRB, 0.46353E+00_JPRB ,&
&  0.51158E+00_JPRB, 0.43387E+00_JPRB, 0.50976E+00_JPRB, 0.40663E+00_JPRB, 0.37409E+00_JPRB, 0.37039E+00_JPRB, 0.35773E+00_JPRB ,&
&  0.35321E+00_JPRB, 0.36344E+00_JPRB, 0.37934E+00_JPRB, 0.39353E+00_JPRB, 0.39958E+00_JPRB, 0.40365E+00_JPRB, 0.41647E+00_JPRB ,&
&  0.43584E+00_JPRB, 0.46269E+00_JPRB, 0.49868E+00_JPRB, 0.51416E+00_JPRB, 0.51483E+00_JPRB, 0.51196E+00_JPRB, 0.51879E+00_JPRB ,&
&  0.48408E+00_JPRB, 0.47736E+00_JPRB, 0.48904E+00_JPRB, 0.49600E+00_JPRB, 0.49553E+00_JPRB, 0.48887E+00_JPRB, 0.50725E+00_JPRB ,&
&  0.50637E+00_JPRB, 0.47705E+00_JPRB, 0.61101E+00_JPRB, 0.65236E+00_JPRB, 0.70350E+00_JPRB, 0.75226E+00_JPRB, 0.79945E+00_JPRB ,&
&  0.84194E+00_JPRB, 0.88651E+00_JPRB, 0.93650E+00_JPRB, 0.98595E+00_JPRB, 0.11037E+01_JPRB, 0.13678E+01_JPRB, 0.16768E+01_JPRB ,&
&  0.20604E+01_JPRB, 0.16272E+01_JPRB, 0.18822E+01_JPRB, 0.18794E+01_JPRB, 0.21477E+01_JPRB, 0.23240E+01_JPRB, 0.22660E+01_JPRB ,&
&  0.22494E+01_JPRB, 0.21355E+01_JPRB, 0.20378E+01_JPRB, 0.19653E+01_JPRB, 0.20761E+01_JPRB, 0.22455E+01_JPRB, 0.24373E+01_JPRB ,&
&  0.26154E+01_JPRB, 0.27854E+01_JPRB, 0.31511E+01_JPRB, 0.30689E+01_JPRB, 0.29598E+01_JPRB, 0.28408E+01_JPRB, 0.26562E+01_JPRB ,&
&  0.24194E+01_JPRB, 0.21448E+01_JPRB, 0.18200E+01_JPRB, 0.14643E+01_JPRB, 0.12254E+01_JPRB, 0.10217E+01_JPRB, 0.85355E+00_JPRB ,&
&  0.73648E+00_JPRB, 0.66478E+00_JPRB, 0.65286E+00_JPRB, 0.62946E+00_JPRB, 0.59686E+00_JPRB, 0.55784E+00_JPRB, 0.49842E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.44943E+00_JPRB, 0.48957E+00_JPRB, 0.53404E+00_JPRB, 0.55635E+00_JPRB, 0.57295E+00_JPRB, 0.62514E+00_JPRB ,&
&  0.57229E+00_JPRB, 0.61414E+00_JPRB, 0.65815E+00_JPRB, 0.63488E+00_JPRB, 0.61811E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.62909E+00_JPRB, 0.60157E+00_JPRB, 0.57656E+00_JPRB, 0.54919E+00_JPRB, 0.53266E+00_JPRB, 0.50959E+00_JPRB, 0.45814E+00_JPRB ,&
&  0.40449E+00_JPRB, 0.38154E+00_JPRB, 0.38494E+00_JPRB, 0.37181E+00_JPRB, 0.35743E+00_JPRB, 0.35243E+00_JPRB, 0.35509E+00_JPRB ,&
&  0.36677E+00_JPRB, 0.39773E+00_JPRB, 0.42516E+00_JPRB, 0.43858E+00_JPRB, 0.43898E+00_JPRB, 0.44199E+00_JPRB, 0.45227E+00_JPRB ,&
&  0.46302E+00_JPRB, 0.47493E+00_JPRB, 0.48427E+00_JPRB, 0.47824E+00_JPRB, 0.47280E+00_JPRB, 0.47322E+00_JPRB, 0.47674E+00_JPRB ,&
&  0.48475E+00_JPRB, 0.48913E+00_JPRB, 0.47696E+00_JPRB, 0.48455E+00_JPRB, 0.46667E+00_JPRB, 0.46662E+00_JPRB, 0.44721E+00_JPRB ,&
&  0.37055E+00_JPRB, 0.38453E+00_JPRB, 0.53457E+00_JPRB, 0.58939E+00_JPRB, 0.62515E+00_JPRB, 0.67896E+00_JPRB, 0.74401E+00_JPRB ,&
&  0.79780E+00_JPRB, 0.84462E+00_JPRB, 0.89070E+00_JPRB, 0.94856E+00_JPRB, 0.10203E+01_JPRB, 0.11507E+01_JPRB, 0.13095E+01_JPRB ,&
&  0.15961E+01_JPRB, 0.13537E+01_JPRB, 0.12284E+01_JPRB, 0.12563E+01_JPRB, 0.15915E+01_JPRB, 0.16922E+01_JPRB, 0.17073E+01_JPRB ,&
&  0.16911E+01_JPRB, 0.15947E+01_JPRB, 0.15300E+01_JPRB, 0.14905E+01_JPRB, 0.15369E+01_JPRB, 0.16459E+01_JPRB, 0.18339E+01_JPRB ,&
&  0.20038E+01_JPRB, 0.22678E+01_JPRB, 0.26079E+01_JPRB, 0.26246E+01_JPRB, 0.25620E+01_JPRB, 0.25086E+01_JPRB, 0.24072E+01_JPRB ,&
&  0.22843E+01_JPRB, 0.20508E+01_JPRB, 0.16893E+01_JPRB, 0.13993E+01_JPRB, 0.11605E+01_JPRB, 0.97910E+00_JPRB, 0.83450E+00_JPRB ,&
&  0.76328E+00_JPRB, 0.74302E+00_JPRB, 0.73197E+00_JPRB, 0.70838E+00_JPRB, 0.66015E+00_JPRB, 0.60958E+00_JPRB, 0.55754E+00_JPRB ,&
&  0.49440E+00_JPRB, 0.45032E+00_JPRB, 0.42235E+00_JPRB, 0.43671E+00_JPRB, 0.48159E+00_JPRB, 0.52746E+00_JPRB, 0.54364E+00_JPRB ,&
&  0.55339E+00_JPRB, 0.55539E+00_JPRB, 0.55680E+00_JPRB, 0.57443E+00_JPRB, 0.60293E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.64128E+00_JPRB, 0.62117E+00_JPRB, 0.59131E+00_JPRB, 0.58535E+00_JPRB, 0.55964E+00_JPRB, 0.53845E+00_JPRB, 0.48421E+00_JPRB ,&
&  0.41773E+00_JPRB, 0.40147E+00_JPRB, 0.42448E+00_JPRB, 0.43160E+00_JPRB, 0.41683E+00_JPRB, 0.41574E+00_JPRB, 0.42553E+00_JPRB ,&
&  0.43799E+00_JPRB, 0.46864E+00_JPRB, 0.47211E+00_JPRB, 0.45730E+00_JPRB, 0.44847E+00_JPRB, 0.44485E+00_JPRB, 0.44977E+00_JPRB ,&
&  0.45181E+00_JPRB, 0.45263E+00_JPRB, 0.44809E+00_JPRB, 0.43567E+00_JPRB, 0.43704E+00_JPRB, 0.44223E+00_JPRB, 0.44528E+00_JPRB ,&
&  0.44706E+00_JPRB, 0.44257E+00_JPRB, 0.45392E+00_JPRB, 0.46853E+00_JPRB, 0.46165E+00_JPRB, 0.44485E+00_JPRB, 0.37712E+00_JPRB ,&
&  0.33652E+00_JPRB, 0.32703E+00_JPRB, 0.35893E+00_JPRB, 0.46618E+00_JPRB, 0.52256E+00_JPRB, 0.57958E+00_JPRB, 0.63805E+00_JPRB ,&
&  0.61028E+00_JPRB, 0.61476E+00_JPRB, 0.66464E+00_JPRB, 0.82165E+00_JPRB, 0.90291E+00_JPRB, 0.99425E+00_JPRB, 0.10707E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11974E+01_JPRB, 0.11790E+01_JPRB, 0.95085E+00_JPRB, 0.99181E+00_JPRB, 0.12629E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.13569E+01_JPRB, 0.14215E+01_JPRB, 0.13318E+01_JPRB, 0.12497E+01_JPRB, 0.12456E+01_JPRB, 0.13308E+01_JPRB ,&
&  0.14819E+01_JPRB, 0.17013E+01_JPRB, 0.20699E+01_JPRB, 0.22306E+01_JPRB, 0.21884E+01_JPRB, 0.21387E+01_JPRB, 0.20792E+01_JPRB ,&
&  0.20287E+01_JPRB, 0.18002E+01_JPRB, 0.15148E+01_JPRB, 0.12667E+01_JPRB, 0.10619E+01_JPRB, 0.90309E+00_JPRB, 0.80775E+00_JPRB ,&
&  0.77037E+00_JPRB, 0.75416E+00_JPRB, 0.75498E+00_JPRB, 0.73200E+00_JPRB, 0.67194E+00_JPRB, 0.60592E+00_JPRB, 0.55652E+00_JPRB ,&
&  0.52251E+00_JPRB, 0.48598E+00_JPRB, 0.45691E+00_JPRB, 0.45013E+00_JPRB, 0.48086E+00_JPRB, 0.52184E+00_JPRB, 0.53325E+00_JPRB ,&
&  0.51872E+00_JPRB, 0.51863E+00_JPRB, 0.55149E+00_JPRB, 0.58830E+00_JPRB, 0.60514E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.60061E+00_JPRB, 0.61399E+00_JPRB, 0.62447E+00_JPRB, 0.61746E+00_JPRB, 0.61121E+00_JPRB, 0.55233E+00_JPRB, 0.49174E+00_JPRB ,&
&  0.45094E+00_JPRB, 0.45767E+00_JPRB, 0.48005E+00_JPRB, 0.49971E+00_JPRB, 0.50300E+00_JPRB, 0.49657E+00_JPRB, 0.49803E+00_JPRB ,&
&  0.50772E+00_JPRB, 0.50798E+00_JPRB, 0.48805E+00_JPRB, 0.47230E+00_JPRB, 0.44862E+00_JPRB, 0.44414E+00_JPRB, 0.44694E+00_JPRB ,&
&  0.45027E+00_JPRB, 0.44902E+00_JPRB, 0.44808E+00_JPRB, 0.44450E+00_JPRB, 0.44573E+00_JPRB, 0.44402E+00_JPRB, 0.43928E+00_JPRB ,&
&  0.42919E+00_JPRB, 0.42154E+00_JPRB, 0.42718E+00_JPRB, 0.44556E+00_JPRB, 0.43977E+00_JPRB, 0.43458E+00_JPRB, 0.36057E+00_JPRB ,&
&  0.33176E+00_JPRB, 0.31429E+00_JPRB, 0.31265E+00_JPRB, 0.32662E+00_JPRB, 0.37198E+00_JPRB, 0.42562E+00_JPRB, 0.50439E+00_JPRB ,&
&  0.57317E+00_JPRB, 0.62950E+00_JPRB, 0.66102E+00_JPRB, 0.68750E+00_JPRB, 0.72836E+00_JPRB, 0.79412E+00_JPRB, 0.86872E+00_JPRB ,&
&  0.91034E+00_JPRB, 0.93349E+00_JPRB, 0.92439E+00_JPRB, 0.79083E+00_JPRB, 0.78043E+00_JPRB, 0.87224E+00_JPRB, 0.99794E+00_JPRB ,&
&  0.11186E+01_JPRB, 0.10895E+01_JPRB, 0.11367E+01_JPRB, 0.12053E+01_JPRB, 0.12242E+01_JPRB, 0.12295E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.15357E+01_JPRB, 0.17989E+01_JPRB, 0.19153E+01_JPRB, 0.18468E+01_JPRB, 0.18596E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.16910E+01_JPRB, 0.14698E+01_JPRB, 0.12788E+01_JPRB, 0.10947E+01_JPRB, 0.94681E+00_JPRB, 0.86147E+00_JPRB, 0.81507E+00_JPRB ,&
&  0.77577E+00_JPRB, 0.74448E+00_JPRB, 0.74488E+00_JPRB, 0.72552E+00_JPRB, 0.67902E+00_JPRB, 0.63640E+00_JPRB, 0.60318E+00_JPRB ,&
&  0.57769E+00_JPRB, 0.56146E+00_JPRB, 0.55627E+00_JPRB, 0.54196E+00_JPRB, 0.54166E+00_JPRB, 0.55036E+00_JPRB, 0.55189E+00_JPRB ,&
&  0.55028E+00_JPRB, 0.54934E+00_JPRB, 0.55776E+00_JPRB, 0.56906E+00_JPRB, 0.57978E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.62763E+00_JPRB, 0.64032E+00_JPRB, 0.64361E+00_JPRB, 0.64968E+00_JPRB, 0.66466E+00_JPRB, 0.62537E+00_JPRB, 0.59922E+00_JPRB ,&
&  0.58722E+00_JPRB, 0.60037E+00_JPRB, 0.60210E+00_JPRB, 0.59953E+00_JPRB, 0.58925E+00_JPRB, 0.57967E+00_JPRB, 0.57289E+00_JPRB ,&
&  0.56668E+00_JPRB, 0.54958E+00_JPRB, 0.51820E+00_JPRB, 0.46938E+00_JPRB, 0.45474E+00_JPRB, 0.45116E+00_JPRB, 0.45495E+00_JPRB ,&
&  0.45740E+00_JPRB, 0.45250E+00_JPRB, 0.45218E+00_JPRB, 0.44691E+00_JPRB, 0.44152E+00_JPRB, 0.42504E+00_JPRB, 0.41033E+00_JPRB ,&
&  0.41286E+00_JPRB, 0.41420E+00_JPRB, 0.43448E+00_JPRB, 0.42162E+00_JPRB, 0.40981E+00_JPRB, 0.40550E+00_JPRB, 0.34184E+00_JPRB ,&
&  0.32979E+00_JPRB, 0.31485E+00_JPRB, 0.30998E+00_JPRB, 0.31022E+00_JPRB, 0.32758E+00_JPRB, 0.34587E+00_JPRB, 0.38373E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.53279E+00_JPRB, 0.57747E+00_JPRB, 0.62566E+00_JPRB, 0.66731E+00_JPRB, 0.70920E+00_JPRB, 0.75194E+00_JPRB ,&
&  0.78729E+00_JPRB, 0.82521E+00_JPRB, 0.85926E+00_JPRB, 0.92310E+00_JPRB, 0.98288E+00_JPRB, 0.94803E+00_JPRB, 0.99666E+00_JPRB ,&
&  0.12444E+01_JPRB, 0.11816E+01_JPRB, 0.11393E+01_JPRB, 0.11054E+01_JPRB, 0.11202E+01_JPRB, 0.11437E+01_JPRB, 0.11766E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.14234E+01_JPRB, 0.16399E+01_JPRB, 0.17747E+01_JPRB, 0.17408E+01_JPRB, 0.17493E+01_JPRB, 0.17078E+01_JPRB ,&
&  0.15609E+01_JPRB, 0.14622E+01_JPRB, 0.15067E+01_JPRB, 0.12249E+01_JPRB, 0.10356E+01_JPRB, 0.93370E+00_JPRB, 0.89684E+00_JPRB ,&
&  0.86262E+00_JPRB, 0.82427E+00_JPRB, 0.80168E+00_JPRB, 0.76684E+00_JPRB, 0.71865E+00_JPRB, 0.69006E+00_JPRB, 0.68508E+00_JPRB ,&
&  0.68086E+00_JPRB, 0.67213E+00_JPRB, 0.66491E+00_JPRB, 0.65388E+00_JPRB, 0.63589E+00_JPRB, 0.61553E+00_JPRB, 0.60738E+00_JPRB ,&
&  0.60695E+00_JPRB, 0.60636E+00_JPRB, 0.60617E+00_JPRB, 0.60699E+00_JPRB, 0.61309E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.64082E+00_JPRB, 0.65712E+00_JPRB, 0.66040E+00_JPRB, 0.66170E+00_JPRB, 0.66386E+00_JPRB, 0.64924E+00_JPRB, 0.63719E+00_JPRB ,&
&  0.62971E+00_JPRB, 0.62167E+00_JPRB, 0.61099E+00_JPRB, 0.59974E+00_JPRB, 0.58782E+00_JPRB, 0.58028E+00_JPRB, 0.57297E+00_JPRB ,&
&  0.56561E+00_JPRB, 0.52434E+00_JPRB, 0.50238E+00_JPRB, 0.48668E+00_JPRB, 0.47269E+00_JPRB, 0.46638E+00_JPRB, 0.46294E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.45198E+00_JPRB, 0.45123E+00_JPRB, 0.44732E+00_JPRB, 0.44017E+00_JPRB, 0.42856E+00_JPRB, 0.43284E+00_JPRB ,&
&  0.45055E+00_JPRB, 0.44149E+00_JPRB, 0.42663E+00_JPRB, 0.41148E+00_JPRB, 0.38914E+00_JPRB, 0.36424E+00_JPRB, 0.34561E+00_JPRB ,&
&  0.33655E+00_JPRB, 0.31698E+00_JPRB, 0.31824E+00_JPRB, 0.32144E+00_JPRB, 0.32521E+00_JPRB, 0.34120E+00_JPRB, 0.37522E+00_JPRB ,&
&  0.41527E+00_JPRB, 0.51548E+00_JPRB, 0.55589E+00_JPRB, 0.60419E+00_JPRB, 0.64046E+00_JPRB, 0.67032E+00_JPRB, 0.69704E+00_JPRB ,&
&  0.71692E+00_JPRB, 0.74647E+00_JPRB, 0.78067E+00_JPRB, 0.83024E+00_JPRB, 0.88366E+00_JPRB, 0.91910E+00_JPRB, 0.94694E+00_JPRB ,&
&  0.99132E+00_JPRB, 0.97452E+00_JPRB, 0.98453E+00_JPRB, 0.97807E+00_JPRB, 0.99164E+00_JPRB, 0.10448E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.11694E+01_JPRB, 0.13004E+01_JPRB, 0.14261E+01_JPRB, 0.14698E+01_JPRB, 0.15248E+01_JPRB, 0.15350E+01_JPRB ,&
&  0.14508E+01_JPRB, 0.14026E+01_JPRB, 0.14129E+01_JPRB, 0.12929E+01_JPRB, 0.11646E+01_JPRB, 0.10654E+01_JPRB, 0.99696E+00_JPRB ,&
&  0.93161E+00_JPRB, 0.86449E+00_JPRB, 0.81905E+00_JPRB, 0.78158E+00_JPRB, 0.74212E+00_JPRB, 0.70670E+00_JPRB, 0.69959E+00_JPRB ,&
&  0.69412E+00_JPRB, 0.68695E+00_JPRB, 0.67673E+00_JPRB, 0.66445E+00_JPRB, 0.64181E+00_JPRB, 0.61860E+00_JPRB, 0.59936E+00_JPRB ,&
&  0.59602E+00_JPRB, 0.59650E+00_JPRB, 0.60505E+00_JPRB, 0.61236E+00_JPRB, 0.62438E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.64195E+00_JPRB, 0.64493E+00_JPRB, 0.63932E+00_JPRB, 0.63183E+00_JPRB, 0.62663E+00_JPRB, 0.61592E+00_JPRB, 0.61074E+00_JPRB ,&
&  0.60730E+00_JPRB, 0.60390E+00_JPRB, 0.59394E+00_JPRB, 0.58267E+00_JPRB, 0.56768E+00_JPRB, 0.55722E+00_JPRB, 0.54490E+00_JPRB ,&
&  0.53478E+00_JPRB, 0.52204E+00_JPRB, 0.50888E+00_JPRB, 0.49590E+00_JPRB, 0.48610E+00_JPRB, 0.47547E+00_JPRB, 0.46784E+00_JPRB ,&
&  0.46074E+00_JPRB, 0.45000E+00_JPRB, 0.43994E+00_JPRB, 0.43945E+00_JPRB, 0.43489E+00_JPRB, 0.43259E+00_JPRB, 0.43868E+00_JPRB ,&
&  0.44443E+00_JPRB, 0.44100E+00_JPRB, 0.43711E+00_JPRB, 0.41878E+00_JPRB, 0.38970E+00_JPRB, 0.32598E+00_JPRB, 0.33368E+00_JPRB ,&
&  0.32868E+00_JPRB, 0.32010E+00_JPRB, 0.32348E+00_JPRB, 0.33231E+00_JPRB, 0.34163E+00_JPRB, 0.36704E+00_JPRB, 0.37510E+00_JPRB ,&
&  0.37475E+00_JPRB, 0.50236E+00_JPRB, 0.53432E+00_JPRB, 0.56755E+00_JPRB, 0.59545E+00_JPRB, 0.61941E+00_JPRB, 0.64270E+00_JPRB ,&
&  0.66045E+00_JPRB, 0.68583E+00_JPRB, 0.70961E+00_JPRB, 0.74897E+00_JPRB, 0.79064E+00_JPRB, 0.84104E+00_JPRB, 0.88202E+00_JPRB ,&
&  0.89920E+00_JPRB, 0.90716E+00_JPRB, 0.92287E+00_JPRB, 0.93353E+00_JPRB, 0.95169E+00_JPRB, 0.98614E+00_JPRB, 0.10345E+01_JPRB ,&
&  0.11001E+01_JPRB, 0.11292E+01_JPRB, 0.12128E+01_JPRB, 0.12759E+01_JPRB, 0.13059E+01_JPRB, 0.13421E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.13429E+01_JPRB, 0.13314E+01_JPRB, 0.12934E+01_JPRB, 0.11906E+01_JPRB, 0.10806E+01_JPRB, 0.10082E+01_JPRB, 0.94192E+00_JPRB ,&
&  0.89069E+00_JPRB, 0.83865E+00_JPRB, 0.78520E+00_JPRB, 0.74231E+00_JPRB, 0.70799E+00_JPRB, 0.68272E+00_JPRB, 0.66910E+00_JPRB ,&
&  0.65585E+00_JPRB, 0.64451E+00_JPRB, 0.63454E+00_JPRB, 0.63131E+00_JPRB, 0.62254E+00_JPRB, 0.61630E+00_JPRB, 0.61069E+00_JPRB ,&
&  0.61149E+00_JPRB, 0.61347E+00_JPRB, 0.61985E+00_JPRB, 0.62687E+00_JPRB, 0.63169E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.63414E+00_JPRB, 0.63521E+00_JPRB, 0.63494E+00_JPRB, 0.62576E+00_JPRB, 0.62107E+00_JPRB, 0.61538E+00_JPRB, 0.61669E+00_JPRB ,&
&  0.60922E+00_JPRB, 0.60246E+00_JPRB, 0.58407E+00_JPRB, 0.57230E+00_JPRB, 0.55872E+00_JPRB, 0.55125E+00_JPRB, 0.54399E+00_JPRB ,&
&  0.53972E+00_JPRB, 0.53344E+00_JPRB, 0.53047E+00_JPRB, 0.52144E+00_JPRB, 0.51342E+00_JPRB, 0.49963E+00_JPRB, 0.48479E+00_JPRB ,&
&  0.46950E+00_JPRB, 0.45350E+00_JPRB, 0.43790E+00_JPRB, 0.43781E+00_JPRB, 0.43179E+00_JPRB, 0.42903E+00_JPRB, 0.42787E+00_JPRB ,&
&  0.43430E+00_JPRB, 0.38989E+00_JPRB, 0.39386E+00_JPRB, 0.37786E+00_JPRB, 0.36134E+00_JPRB, 0.34396E+00_JPRB, 0.33168E+00_JPRB ,&
&  0.33415E+00_JPRB, 0.33599E+00_JPRB, 0.34617E+00_JPRB, 0.36374E+00_JPRB, 0.37298E+00_JPRB, 0.39017E+00_JPRB, 0.40405E+00_JPRB ,&
&  0.40826E+00_JPRB, 0.49814E+00_JPRB, 0.54459E+00_JPRB, 0.57212E+00_JPRB, 0.59668E+00_JPRB, 0.61858E+00_JPRB, 0.63610E+00_JPRB ,&
&  0.65990E+00_JPRB, 0.68475E+00_JPRB, 0.70867E+00_JPRB, 0.73135E+00_JPRB, 0.77258E+00_JPRB, 0.80188E+00_JPRB, 0.82597E+00_JPRB ,&
&  0.84061E+00_JPRB, 0.84292E+00_JPRB, 0.85345E+00_JPRB, 0.87659E+00_JPRB, 0.91116E+00_JPRB, 0.94840E+00_JPRB, 0.98969E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.10693E+01_JPRB, 0.11121E+01_JPRB, 0.11529E+01_JPRB, 0.11860E+01_JPRB, 0.12024E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.11885E+01_JPRB, 0.11613E+01_JPRB, 0.11194E+01_JPRB, 0.10648E+01_JPRB, 0.99964E+00_JPRB, 0.92872E+00_JPRB, 0.86033E+00_JPRB ,&
&  0.82192E+00_JPRB, 0.78154E+00_JPRB, 0.74489E+00_JPRB, 0.70824E+00_JPRB, 0.67791E+00_JPRB, 0.65545E+00_JPRB, 0.64400E+00_JPRB ,&
&  0.63241E+00_JPRB, 0.62505E+00_JPRB, 0.62045E+00_JPRB, 0.61846E+00_JPRB, 0.61964E+00_JPRB, 0.61889E+00_JPRB, 0.62244E+00_JPRB ,&
&  0.62658E+00_JPRB, 0.63072E+00_JPRB, 0.63017E+00_JPRB, 0.63024E+00_JPRB, 0.63149E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.62983E+00_JPRB, 0.63267E+00_JPRB, 0.64049E+00_JPRB, 0.64165E+00_JPRB, 0.64204E+00_JPRB, 0.63705E+00_JPRB, 0.63571E+00_JPRB ,&
&  0.62794E+00_JPRB, 0.61969E+00_JPRB, 0.60071E+00_JPRB, 0.58733E+00_JPRB, 0.57402E+00_JPRB, 0.56621E+00_JPRB, 0.56032E+00_JPRB ,&
&  0.55679E+00_JPRB, 0.55158E+00_JPRB, 0.54579E+00_JPRB, 0.53678E+00_JPRB, 0.52534E+00_JPRB, 0.51130E+00_JPRB, 0.49253E+00_JPRB ,&
&  0.47603E+00_JPRB, 0.46349E+00_JPRB, 0.44515E+00_JPRB, 0.44750E+00_JPRB, 0.43922E+00_JPRB, 0.43365E+00_JPRB, 0.43117E+00_JPRB ,&
&  0.42324E+00_JPRB, 0.41314E+00_JPRB, 0.39838E+00_JPRB, 0.38725E+00_JPRB, 0.37710E+00_JPRB, 0.36607E+00_JPRB, 0.37482E+00_JPRB ,&
&  0.38224E+00_JPRB, 0.40605E+00_JPRB, 0.41813E+00_JPRB, 0.42403E+00_JPRB, 0.43027E+00_JPRB, 0.43314E+00_JPRB, 0.44420E+00_JPRB ,&
&  0.46001E+00_JPRB, 0.48232E+00_JPRB, 0.54100E+00_JPRB, 0.57625E+00_JPRB, 0.59602E+00_JPRB, 0.61720E+00_JPRB, 0.63248E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.66451E+00_JPRB, 0.67217E+00_JPRB, 0.68318E+00_JPRB, 0.70372E+00_JPRB, 0.72953E+00_JPRB, 0.75063E+00_JPRB ,&
&  0.76692E+00_JPRB, 0.77528E+00_JPRB, 0.78800E+00_JPRB, 0.80335E+00_JPRB, 0.82811E+00_JPRB, 0.85401E+00_JPRB, 0.88182E+00_JPRB ,&
&  0.90839E+00_JPRB, 0.93573E+00_JPRB, 0.95377E+00_JPRB, 0.96958E+00_JPRB, 0.97595E+00_JPRB, 0.97735E+00_JPRB, 0.97664E+00_JPRB ,&
&  0.96230E+00_JPRB, 0.94070E+00_JPRB, 0.91165E+00_JPRB, 0.86717E+00_JPRB, 0.83414E+00_JPRB, 0.81420E+00_JPRB, 0.78308E+00_JPRB ,&
&  0.75674E+00_JPRB, 0.73123E+00_JPRB, 0.70762E+00_JPRB, 0.68375E+00_JPRB, 0.66411E+00_JPRB, 0.64631E+00_JPRB, 0.63479E+00_JPRB ,&
&  0.62617E+00_JPRB, 0.62130E+00_JPRB, 0.62006E+00_JPRB, 0.62091E+00_JPRB, 0.62572E+00_JPRB, 0.62904E+00_JPRB, 0.63597E+00_JPRB ,&
&  0.63826E+00_JPRB, 0.63755E+00_JPRB, 0.63277E+00_JPRB, 0.62966E+00_JPRB, 0.62772E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.63499E+00_JPRB, 0.63480E+00_JPRB, 0.63705E+00_JPRB, 0.63834E+00_JPRB, 0.64012E+00_JPRB, 0.63890E+00_JPRB, 0.63773E+00_JPRB ,&
&  0.63335E+00_JPRB, 0.62521E+00_JPRB, 0.61104E+00_JPRB, 0.59839E+00_JPRB, 0.58698E+00_JPRB, 0.57933E+00_JPRB, 0.57295E+00_JPRB ,&
&  0.56706E+00_JPRB, 0.56223E+00_JPRB, 0.55674E+00_JPRB, 0.55047E+00_JPRB, 0.54229E+00_JPRB, 0.53308E+00_JPRB, 0.52185E+00_JPRB ,&
&  0.51339E+00_JPRB, 0.50494E+00_JPRB, 0.49779E+00_JPRB, 0.49266E+00_JPRB, 0.49049E+00_JPRB, 0.46691E+00_JPRB, 0.46160E+00_JPRB ,&
&  0.46700E+00_JPRB, 0.45761E+00_JPRB, 0.44974E+00_JPRB, 0.43760E+00_JPRB, 0.44032E+00_JPRB, 0.44923E+00_JPRB, 0.45524E+00_JPRB ,&
&  0.44284E+00_JPRB, 0.46650E+00_JPRB, 0.46923E+00_JPRB, 0.47077E+00_JPRB, 0.46981E+00_JPRB, 0.46733E+00_JPRB, 0.48092E+00_JPRB ,&
&  0.51227E+00_JPRB, 0.52713E+00_JPRB, 0.55366E+00_JPRB, 0.58304E+00_JPRB, 0.60312E+00_JPRB, 0.62409E+00_JPRB, 0.64352E+00_JPRB ,&
&  0.66315E+00_JPRB, 0.67585E+00_JPRB, 0.68377E+00_JPRB, 0.69148E+00_JPRB, 0.70486E+00_JPRB, 0.71896E+00_JPRB, 0.73089E+00_JPRB ,&
&  0.73835E+00_JPRB, 0.74125E+00_JPRB, 0.74561E+00_JPRB, 0.75008E+00_JPRB, 0.75811E+00_JPRB, 0.76429E+00_JPRB, 0.76993E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.77508E+00_JPRB, 0.77484E+00_JPRB, 0.77268E+00_JPRB, 0.76903E+00_JPRB, 0.76337E+00_JPRB, 0.76204E+00_JPRB ,&
&  0.75473E+00_JPRB, 0.74556E+00_JPRB, 0.73383E+00_JPRB, 0.71907E+00_JPRB, 0.70710E+00_JPRB, 0.69794E+00_JPRB, 0.68882E+00_JPRB ,&
&  0.67981E+00_JPRB, 0.67132E+00_JPRB, 0.66350E+00_JPRB, 0.65560E+00_JPRB, 0.64795E+00_JPRB, 0.64048E+00_JPRB, 0.63533E+00_JPRB ,&
&  0.63330E+00_JPRB, 0.63206E+00_JPRB, 0.63369E+00_JPRB, 0.63432E+00_JPRB, 0.63806E+00_JPRB, 0.63933E+00_JPRB, 0.64240E+00_JPRB ,&
&  0.64367E+00_JPRB, 0.64246E+00_JPRB, 0.63900E+00_JPRB, 0.63642E+00_JPRB, 0.63430E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.63070E+00_JPRB, 0.62894E+00_JPRB, 0.62784E+00_JPRB, 0.62805E+00_JPRB, 0.62903E+00_JPRB, 0.62926E+00_JPRB, 0.62868E+00_JPRB ,&
&  0.62636E+00_JPRB, 0.62068E+00_JPRB, 0.61405E+00_JPRB, 0.60698E+00_JPRB, 0.59893E+00_JPRB, 0.59142E+00_JPRB, 0.58670E+00_JPRB ,&
&  0.58173E+00_JPRB, 0.57677E+00_JPRB, 0.57166E+00_JPRB, 0.56789E+00_JPRB, 0.56356E+00_JPRB, 0.55908E+00_JPRB, 0.55507E+00_JPRB ,&
&  0.55243E+00_JPRB, 0.54932E+00_JPRB, 0.54586E+00_JPRB, 0.54164E+00_JPRB, 0.53859E+00_JPRB, 0.53505E+00_JPRB, 0.53170E+00_JPRB ,&
&  0.52780E+00_JPRB, 0.52612E+00_JPRB, 0.52507E+00_JPRB, 0.52489E+00_JPRB, 0.52702E+00_JPRB, 0.52991E+00_JPRB, 0.53088E+00_JPRB ,&
&  0.53081E+00_JPRB, 0.52844E+00_JPRB, 0.52973E+00_JPRB, 0.53169E+00_JPRB, 0.53537E+00_JPRB, 0.54148E+00_JPRB, 0.54993E+00_JPRB ,&
&  0.56113E+00_JPRB, 0.57427E+00_JPRB, 0.58806E+00_JPRB, 0.59863E+00_JPRB, 0.60907E+00_JPRB, 0.61851E+00_JPRB, 0.62474E+00_JPRB ,&
&  0.63165E+00_JPRB, 0.63653E+00_JPRB, 0.64217E+00_JPRB, 0.64680E+00_JPRB, 0.65332E+00_JPRB, 0.65864E+00_JPRB, 0.66332E+00_JPRB ,&
&  0.66486E+00_JPRB, 0.66359E+00_JPRB, 0.66184E+00_JPRB, 0.66038E+00_JPRB, 0.65784E+00_JPRB, 0.65588E+00_JPRB, 0.65455E+00_JPRB ,&
&  0.65375E+00_JPRB, 0.65293E+00_JPRB, 0.65340E+00_JPRB, 0.65427E+00_JPRB, 0.65539E+00_JPRB, 0.65536E+00_JPRB, 0.65630E+00_JPRB ,&
&  0.65531E+00_JPRB, 0.65381E+00_JPRB, 0.65107E+00_JPRB, 0.64684E+00_JPRB, 0.64334E+00_JPRB, 0.64095E+00_JPRB, 0.63842E+00_JPRB ,&
&  0.63593E+00_JPRB, 0.63402E+00_JPRB, 0.63279E+00_JPRB, 0.63188E+00_JPRB, 0.63007E+00_JPRB, 0.62862E+00_JPRB, 0.62790E+00_JPRB ,&
&  0.62793E+00_JPRB, 0.62754E+00_JPRB, 0.62865E+00_JPRB, 0.62988E+00_JPRB, 0.63396E+00_JPRB, 0.63580E+00_JPRB, 0.63734E+00_JPRB ,&
&  0.63781E+00_JPRB, 0.63892E+00_JPRB, 0.63802E+00_JPRB, 0.63555E+00_JPRB, 0.63266E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.63073E+00_JPRB, 0.62917E+00_JPRB, 0.62771E+00_JPRB, 0.62666E+00_JPRB, 0.62608E+00_JPRB, 0.62574E+00_JPRB, 0.62443E+00_JPRB ,&
&  0.62275E+00_JPRB, 0.62022E+00_JPRB, 0.61757E+00_JPRB, 0.61445E+00_JPRB, 0.61071E+00_JPRB, 0.60583E+00_JPRB, 0.60167E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.59491E+00_JPRB, 0.59160E+00_JPRB, 0.58933E+00_JPRB, 0.58707E+00_JPRB, 0.58514E+00_JPRB, 0.58302E+00_JPRB ,&
&  0.58140E+00_JPRB, 0.57975E+00_JPRB, 0.57819E+00_JPRB, 0.57681E+00_JPRB, 0.57623E+00_JPRB, 0.57544E+00_JPRB, 0.57456E+00_JPRB ,&
&  0.57380E+00_JPRB, 0.57335E+00_JPRB, 0.57296E+00_JPRB, 0.57265E+00_JPRB, 0.57250E+00_JPRB, 0.57284E+00_JPRB, 0.57317E+00_JPRB ,&
&  0.57378E+00_JPRB, 0.57511E+00_JPRB, 0.57703E+00_JPRB, 0.57880E+00_JPRB, 0.58078E+00_JPRB, 0.58345E+00_JPRB, 0.58671E+00_JPRB ,&
&  0.58962E+00_JPRB, 0.59223E+00_JPRB, 0.59504E+00_JPRB, 0.59821E+00_JPRB, 0.60114E+00_JPRB, 0.60414E+00_JPRB, 0.60664E+00_JPRB ,&
&  0.60905E+00_JPRB, 0.61126E+00_JPRB, 0.61371E+00_JPRB, 0.61459E+00_JPRB, 0.61634E+00_JPRB, 0.61782E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.62031E+00_JPRB, 0.62073E+00_JPRB, 0.62150E+00_JPRB, 0.62242E+00_JPRB, 0.62325E+00_JPRB, 0.62411E+00_JPRB, 0.62507E+00_JPRB ,&
&  0.62636E+00_JPRB, 0.62728E+00_JPRB, 0.62842E+00_JPRB, 0.62952E+00_JPRB, 0.63091E+00_JPRB, 0.63184E+00_JPRB, 0.63293E+00_JPRB ,&
&  0.63339E+00_JPRB, 0.63376E+00_JPRB, 0.63364E+00_JPRB, 0.63337E+00_JPRB, 0.63322E+00_JPRB, 0.63327E+00_JPRB, 0.63346E+00_JPRB ,&
&  0.63375E+00_JPRB, 0.63394E+00_JPRB, 0.63405E+00_JPRB, 0.63393E+00_JPRB, 0.63324E+00_JPRB, 0.63263E+00_JPRB, 0.63228E+00_JPRB ,&
&  0.63246E+00_JPRB, 0.63251E+00_JPRB, 0.63291E+00_JPRB, 0.63353E+00_JPRB, 0.63496E+00_JPRB, 0.63549E+00_JPRB, 0.63594E+00_JPRB ,&
&  0.63583E+00_JPRB, 0.63562E+00_JPRB, 0.63473E+00_JPRB, 0.63356E+00_JPRB, 0.63210E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB, 0.61332E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB, 0.12818E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.13001E+00_JPRB, 0.13067E+00_JPRB, 0.13115E+00_JPRB, 0.13155E+00_JPRB, 0.13144E+00_JPRB, 0.13153E+00_JPRB, 0.13162E+00_JPRB ,&
&  0.13165E+00_JPRB, 0.13167E+00_JPRB, 0.13190E+00_JPRB, 0.13211E+00_JPRB, 0.13234E+00_JPRB, 0.13258E+00_JPRB, 0.13284E+00_JPRB ,&
&  0.13295E+00_JPRB, 0.13319E+00_JPRB, 0.13229E+00_JPRB, 0.13158E+00_JPRB, 0.13116E+00_JPRB, 0.13093E+00_JPRB, 0.13082E+00_JPRB ,&
&  0.13073E+00_JPRB, 0.13068E+00_JPRB, 0.13059E+00_JPRB, 0.13126E+00_JPRB, 0.13171E+00_JPRB, 0.13211E+00_JPRB, 0.13243E+00_JPRB ,&
&  0.13258E+00_JPRB, 0.13276E+00_JPRB, 0.13292E+00_JPRB, 0.13308E+00_JPRB, 0.13326E+00_JPRB, 0.13322E+00_JPRB, 0.13310E+00_JPRB ,&
&  0.13292E+00_JPRB, 0.13263E+00_JPRB, 0.13230E+00_JPRB, 0.13199E+00_JPRB, 0.13174E+00_JPRB, 0.13145E+00_JPRB, 0.13114E+00_JPRB ,&
&  0.13084E+00_JPRB, 0.13048E+00_JPRB, 0.13014E+00_JPRB, 0.12982E+00_JPRB, 0.12951E+00_JPRB, 0.12924E+00_JPRB, 0.12895E+00_JPRB ,&
&  0.12874E+00_JPRB, 0.12856E+00_JPRB, 0.12837E+00_JPRB, 0.12823E+00_JPRB, 0.12803E+00_JPRB, 0.12794E+00_JPRB, 0.12776E+00_JPRB ,&
&  0.12768E+00_JPRB, 0.12760E+00_JPRB, 0.12753E+00_JPRB, 0.12745E+00_JPRB, 0.12743E+00_JPRB, 0.12739E+00_JPRB, 0.12737E+00_JPRB ,&
&  0.12733E+00_JPRB, 0.12740E+00_JPRB, 0.12729E+00_JPRB, 0.12715E+00_JPRB, 0.12699E+00_JPRB, 0.12683E+00_JPRB, 0.12669E+00_JPRB ,&
&  0.12656E+00_JPRB, 0.12643E+00_JPRB, 0.12634E+00_JPRB, 0.12634E+00_JPRB, 0.12635E+00_JPRB, 0.12638E+00_JPRB, 0.12643E+00_JPRB ,&
&  0.12649E+00_JPRB, 0.12658E+00_JPRB, 0.12666E+00_JPRB, 0.12664E+00_JPRB, 0.12666E+00_JPRB, 0.12668E+00_JPRB, 0.12670E+00_JPRB ,&
&  0.12672E+00_JPRB, 0.12677E+00_JPRB, 0.12684E+00_JPRB, 0.12693E+00_JPRB, 0.12713E+00_JPRB, 0.12722E+00_JPRB, 0.12735E+00_JPRB ,&
&  0.12754E+00_JPRB, 0.12786E+00_JPRB, 0.12815E+00_JPRB, 0.12846E+00_JPRB, 0.12884E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.13873E+00_JPRB, 0.14057E+00_JPRB, 0.14209E+00_JPRB, 0.14408E+00_JPRB, 0.14124E+00_JPRB, 0.14826E+00_JPRB, 0.14602E+00_JPRB ,&
&  0.14368E+00_JPRB, 0.13901E+00_JPRB, 0.13886E+00_JPRB, 0.13790E+00_JPRB, 0.13666E+00_JPRB, 0.13586E+00_JPRB, 0.13546E+00_JPRB ,&
&  0.13528E+00_JPRB, 0.13490E+00_JPRB, 0.13507E+00_JPRB, 0.13519E+00_JPRB, 0.13528E+00_JPRB, 0.13533E+00_JPRB, 0.13549E+00_JPRB ,&
&  0.13588E+00_JPRB, 0.13664E+00_JPRB, 0.13744E+00_JPRB, 0.13612E+00_JPRB, 0.13606E+00_JPRB, 0.13643E+00_JPRB, 0.13710E+00_JPRB ,&
&  0.13773E+00_JPRB, 0.13876E+00_JPRB, 0.13949E+00_JPRB, 0.13998E+00_JPRB, 0.14122E+00_JPRB, 0.14118E+00_JPRB, 0.14098E+00_JPRB ,&
&  0.14058E+00_JPRB, 0.13986E+00_JPRB, 0.13880E+00_JPRB, 0.13783E+00_JPRB, 0.13681E+00_JPRB, 0.13552E+00_JPRB, 0.13428E+00_JPRB ,&
&  0.13325E+00_JPRB, 0.13241E+00_JPRB, 0.13135E+00_JPRB, 0.13034E+00_JPRB, 0.12940E+00_JPRB, 0.12849E+00_JPRB, 0.12770E+00_JPRB ,&
&  0.12712E+00_JPRB, 0.12664E+00_JPRB, 0.12615E+00_JPRB, 0.12576E+00_JPRB, 0.12535E+00_JPRB, 0.12497E+00_JPRB, 0.12462E+00_JPRB ,&
&  0.12471E+00_JPRB, 0.12478E+00_JPRB, 0.12485E+00_JPRB, 0.12490E+00_JPRB, 0.12518E+00_JPRB, 0.12520E+00_JPRB, 0.12524E+00_JPRB ,&
&  0.12525E+00_JPRB, 0.12514E+00_JPRB, 0.12504E+00_JPRB, 0.12487E+00_JPRB, 0.12467E+00_JPRB, 0.12443E+00_JPRB, 0.12433E+00_JPRB ,&
&  0.12434E+00_JPRB, 0.12444E+00_JPRB, 0.12472E+00_JPRB, 0.12490E+00_JPRB, 0.12508E+00_JPRB, 0.12523E+00_JPRB, 0.12559E+00_JPRB ,&
&  0.12597E+00_JPRB, 0.12635E+00_JPRB, 0.12672E+00_JPRB, 0.12667E+00_JPRB, 0.12689E+00_JPRB, 0.12710E+00_JPRB, 0.12729E+00_JPRB ,&
&  0.12741E+00_JPRB, 0.12782E+00_JPRB, 0.12845E+00_JPRB, 0.12926E+00_JPRB, 0.13074E+00_JPRB, 0.13205E+00_JPRB, 0.13328E+00_JPRB ,&
&  0.13458E+00_JPRB, 0.13716E+00_JPRB, 0.13834E+00_JPRB, 0.13888E+00_JPRB, 0.13911E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.14868E+00_JPRB, 0.15043E+00_JPRB, 0.15152E+00_JPRB, 0.15229E+00_JPRB, 0.15330E+00_JPRB, 0.15373E+00_JPRB, 0.15413E+00_JPRB ,&
&  0.15239E+00_JPRB, 0.15173E+00_JPRB, 0.15066E+00_JPRB, 0.14935E+00_JPRB, 0.14835E+00_JPRB, 0.14629E+00_JPRB, 0.14416E+00_JPRB ,&
&  0.14174E+00_JPRB, 0.13955E+00_JPRB, 0.13808E+00_JPRB, 0.13667E+00_JPRB, 0.13529E+00_JPRB, 0.13413E+00_JPRB, 0.13540E+00_JPRB ,&
&  0.13615E+00_JPRB, 0.13664E+00_JPRB, 0.13771E+00_JPRB, 0.13786E+00_JPRB, 0.13913E+00_JPRB, 0.14110E+00_JPRB, 0.14371E+00_JPRB ,&
&  0.14569E+00_JPRB, 0.14844E+00_JPRB, 0.15009E+00_JPRB, 0.15092E+00_JPRB, 0.15200E+00_JPRB, 0.15270E+00_JPRB, 0.15236E+00_JPRB ,&
&  0.15118E+00_JPRB, 0.14960E+00_JPRB, 0.14781E+00_JPRB, 0.14554E+00_JPRB, 0.14293E+00_JPRB, 0.14037E+00_JPRB, 0.13782E+00_JPRB ,&
&  0.13590E+00_JPRB, 0.13417E+00_JPRB, 0.13240E+00_JPRB, 0.13082E+00_JPRB, 0.12932E+00_JPRB, 0.12796E+00_JPRB, 0.12660E+00_JPRB ,&
&  0.12552E+00_JPRB, 0.12465E+00_JPRB, 0.12361E+00_JPRB, 0.12298E+00_JPRB, 0.12237E+00_JPRB, 0.12207E+00_JPRB, 0.12189E+00_JPRB ,&
&  0.12212E+00_JPRB, 0.12247E+00_JPRB, 0.12284E+00_JPRB, 0.12302E+00_JPRB, 0.12319E+00_JPRB, 0.12327E+00_JPRB, 0.12342E+00_JPRB ,&
&  0.12356E+00_JPRB, 0.12374E+00_JPRB, 0.12341E+00_JPRB, 0.12311E+00_JPRB, 0.12271E+00_JPRB, 0.12227E+00_JPRB, 0.12215E+00_JPRB ,&
&  0.12240E+00_JPRB, 0.12263E+00_JPRB, 0.12256E+00_JPRB, 0.12282E+00_JPRB, 0.12325E+00_JPRB, 0.12369E+00_JPRB, 0.12477E+00_JPRB ,&
&  0.12535E+00_JPRB, 0.12593E+00_JPRB, 0.12648E+00_JPRB, 0.12693E+00_JPRB, 0.12758E+00_JPRB, 0.12813E+00_JPRB, 0.12879E+00_JPRB ,&
&  0.12959E+00_JPRB, 0.13041E+00_JPRB, 0.13152E+00_JPRB, 0.13289E+00_JPRB, 0.13607E+00_JPRB, 0.13835E+00_JPRB, 0.14048E+00_JPRB ,&
&  0.14179E+00_JPRB, 0.13976E+00_JPRB, 0.14693E+00_JPRB, 0.14658E+00_JPRB, 0.14660E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.15421E+00_JPRB, 0.15659E+00_JPRB, 0.15552E+00_JPRB, 0.15550E+00_JPRB, 0.15514E+00_JPRB, 0.15514E+00_JPRB, 0.15437E+00_JPRB ,&
&  0.15414E+00_JPRB, 0.15201E+00_JPRB, 0.15145E+00_JPRB, 0.14974E+00_JPRB, 0.14792E+00_JPRB, 0.14553E+00_JPRB, 0.14317E+00_JPRB ,&
&  0.14075E+00_JPRB, 0.13859E+00_JPRB, 0.13708E+00_JPRB, 0.13626E+00_JPRB, 0.13501E+00_JPRB, 0.13475E+00_JPRB, 0.13611E+00_JPRB ,&
&  0.13642E+00_JPRB, 0.13735E+00_JPRB, 0.13755E+00_JPRB, 0.13880E+00_JPRB, 0.14042E+00_JPRB, 0.14145E+00_JPRB, 0.14385E+00_JPRB ,&
&  0.14576E+00_JPRB, 0.14721E+00_JPRB, 0.15211E+00_JPRB, 0.15502E+00_JPRB, 0.15680E+00_JPRB, 0.15924E+00_JPRB, 0.15894E+00_JPRB ,&
&  0.15781E+00_JPRB, 0.15492E+00_JPRB, 0.15822E+00_JPRB, 0.15665E+00_JPRB, 0.15061E+00_JPRB, 0.14761E+00_JPRB, 0.14257E+00_JPRB ,&
&  0.13927E+00_JPRB, 0.13567E+00_JPRB, 0.13294E+00_JPRB, 0.13088E+00_JPRB, 0.12834E+00_JPRB, 0.12668E+00_JPRB, 0.12483E+00_JPRB ,&
&  0.12306E+00_JPRB, 0.12220E+00_JPRB, 0.12093E+00_JPRB, 0.12065E+00_JPRB, 0.12066E+00_JPRB, 0.12092E+00_JPRB, 0.12140E+00_JPRB ,&
&  0.12208E+00_JPRB, 0.12274E+00_JPRB, 0.12335E+00_JPRB, 0.12389E+00_JPRB, 0.12434E+00_JPRB, 0.12463E+00_JPRB, 0.12455E+00_JPRB ,&
&  0.12462E+00_JPRB, 0.12451E+00_JPRB, 0.12408E+00_JPRB, 0.12381E+00_JPRB, 0.12301E+00_JPRB, 0.12261E+00_JPRB, 0.12223E+00_JPRB ,&
&  0.12205E+00_JPRB, 0.12208E+00_JPRB, 0.12202E+00_JPRB, 0.12221E+00_JPRB, 0.12248E+00_JPRB, 0.12309E+00_JPRB, 0.12362E+00_JPRB ,&
&  0.12404E+00_JPRB, 0.12478E+00_JPRB, 0.12524E+00_JPRB, 0.12587E+00_JPRB, 0.12675E+00_JPRB, 0.12722E+00_JPRB, 0.12828E+00_JPRB ,&
&  0.12924E+00_JPRB, 0.13033E+00_JPRB, 0.13223E+00_JPRB, 0.13417E+00_JPRB, 0.13691E+00_JPRB, 0.14051E+00_JPRB, 0.14232E+00_JPRB ,&
&  0.14415E+00_JPRB, 0.14627E+00_JPRB, 0.14579E+00_JPRB, 0.15006E+00_JPRB, 0.15202E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.15611E+00_JPRB, 0.15695E+00_JPRB, 0.15539E+00_JPRB, 0.15590E+00_JPRB, 0.15671E+00_JPRB, 0.15717E+00_JPRB, 0.15762E+00_JPRB ,&
&  0.15531E+00_JPRB, 0.15429E+00_JPRB, 0.15197E+00_JPRB, 0.14971E+00_JPRB, 0.14714E+00_JPRB, 0.14439E+00_JPRB, 0.14225E+00_JPRB ,&
&  0.14044E+00_JPRB, 0.13962E+00_JPRB, 0.13880E+00_JPRB, 0.13935E+00_JPRB, 0.13939E+00_JPRB, 0.14086E+00_JPRB, 0.14209E+00_JPRB ,&
&  0.14283E+00_JPRB, 0.14375E+00_JPRB, 0.14273E+00_JPRB, 0.14279E+00_JPRB, 0.14456E+00_JPRB, 0.14631E+00_JPRB, 0.14893E+00_JPRB ,&
&  0.15176E+00_JPRB, 0.15469E+00_JPRB, 0.15935E+00_JPRB, 0.16113E+00_JPRB, 0.16315E+00_JPRB, 0.16450E+00_JPRB, 0.16456E+00_JPRB ,&
&  0.16432E+00_JPRB, 0.16456E+00_JPRB, 0.16551E+00_JPRB, 0.16537E+00_JPRB, 0.15892E+00_JPRB, 0.15384E+00_JPRB, 0.14853E+00_JPRB ,&
&  0.14349E+00_JPRB, 0.13878E+00_JPRB, 0.13463E+00_JPRB, 0.13185E+00_JPRB, 0.12912E+00_JPRB, 0.12712E+00_JPRB, 0.12556E+00_JPRB ,&
&  0.12395E+00_JPRB, 0.12294E+00_JPRB, 0.12171E+00_JPRB, 0.12130E+00_JPRB, 0.12149E+00_JPRB, 0.12211E+00_JPRB, 0.12295E+00_JPRB ,&
&  0.12395E+00_JPRB, 0.12467E+00_JPRB, 0.12558E+00_JPRB, 0.12646E+00_JPRB, 0.12709E+00_JPRB, 0.12753E+00_JPRB, 0.12735E+00_JPRB ,&
&  0.12754E+00_JPRB, 0.12775E+00_JPRB, 0.12735E+00_JPRB, 0.12725E+00_JPRB, 0.12627E+00_JPRB, 0.12514E+00_JPRB, 0.12395E+00_JPRB ,&
&  0.12243E+00_JPRB, 0.12126E+00_JPRB, 0.12081E+00_JPRB, 0.12055E+00_JPRB, 0.12111E+00_JPRB, 0.12198E+00_JPRB, 0.12258E+00_JPRB ,&
&  0.12292E+00_JPRB, 0.12289E+00_JPRB, 0.12298E+00_JPRB, 0.12397E+00_JPRB, 0.12502E+00_JPRB, 0.12633E+00_JPRB, 0.12769E+00_JPRB ,&
&  0.12820E+00_JPRB, 0.12934E+00_JPRB, 0.13037E+00_JPRB, 0.13158E+00_JPRB, 0.13551E+00_JPRB, 0.13879E+00_JPRB, 0.14507E+00_JPRB ,&
&  0.14879E+00_JPRB, 0.15194E+00_JPRB, 0.15275E+00_JPRB, 0.15594E+00_JPRB, 0.15576E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.16011E+00_JPRB, 0.16206E+00_JPRB, 0.16392E+00_JPRB, 0.16306E+00_JPRB, 0.16322E+00_JPRB, 0.16327E+00_JPRB, 0.16335E+00_JPRB ,&
&  0.16333E+00_JPRB, 0.16279E+00_JPRB, 0.16297E+00_JPRB, 0.16151E+00_JPRB, 0.16062E+00_JPRB, 0.15691E+00_JPRB, 0.15539E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15276E+00_JPRB, 0.15291E+00_JPRB, 0.15390E+00_JPRB, 0.15437E+00_JPRB, 0.15553E+00_JPRB, 0.15496E+00_JPRB ,&
&  0.15508E+00_JPRB, 0.15489E+00_JPRB, 0.15373E+00_JPRB, 0.15343E+00_JPRB, 0.15393E+00_JPRB, 0.15455E+00_JPRB, 0.15489E+00_JPRB ,&
&  0.15651E+00_JPRB, 0.15863E+00_JPRB, 0.16070E+00_JPRB, 0.16217E+00_JPRB, 0.16741E+00_JPRB, 0.16898E+00_JPRB, 0.17070E+00_JPRB ,&
&  0.17183E+00_JPRB, 0.17334E+00_JPRB, 0.17433E+00_JPRB, 0.17381E+00_JPRB, 0.17270E+00_JPRB, 0.16955E+00_JPRB, 0.16832E+00_JPRB ,&
&  0.15830E+00_JPRB, 0.15161E+00_JPRB, 0.14498E+00_JPRB, 0.13920E+00_JPRB, 0.13513E+00_JPRB, 0.13232E+00_JPRB, 0.12997E+00_JPRB ,&
&  0.12802E+00_JPRB, 0.12618E+00_JPRB, 0.12423E+00_JPRB, 0.12417E+00_JPRB, 0.12482E+00_JPRB, 0.12581E+00_JPRB, 0.12678E+00_JPRB ,&
&  0.12789E+00_JPRB, 0.12854E+00_JPRB, 0.12886E+00_JPRB, 0.12954E+00_JPRB, 0.12974E+00_JPRB, 0.13010E+00_JPRB, 0.13031E+00_JPRB ,&
&  0.13142E+00_JPRB, 0.13180E+00_JPRB, 0.13186E+00_JPRB, 0.13297E+00_JPRB, 0.13157E+00_JPRB, 0.12927E+00_JPRB, 0.12663E+00_JPRB ,&
&  0.12412E+00_JPRB, 0.12253E+00_JPRB, 0.12121E+00_JPRB, 0.12068E+00_JPRB, 0.12097E+00_JPRB, 0.12157E+00_JPRB, 0.12240E+00_JPRB ,&
&  0.12263E+00_JPRB, 0.12316E+00_JPRB, 0.12358E+00_JPRB, 0.12437E+00_JPRB, 0.12522E+00_JPRB, 0.12616E+00_JPRB, 0.12716E+00_JPRB ,&
&  0.12866E+00_JPRB, 0.13040E+00_JPRB, 0.13215E+00_JPRB, 0.13395E+00_JPRB, 0.13681E+00_JPRB, 0.14034E+00_JPRB, 0.14602E+00_JPRB ,&
&  0.15156E+00_JPRB, 0.15988E+00_JPRB, 0.15942E+00_JPRB, 0.16093E+00_JPRB, 0.16104E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.16651E+00_JPRB, 0.17007E+00_JPRB, 0.17170E+00_JPRB, 0.17177E+00_JPRB, 0.17080E+00_JPRB, 0.16999E+00_JPRB, 0.16986E+00_JPRB ,&
&  0.16979E+00_JPRB, 0.16874E+00_JPRB, 0.16794E+00_JPRB, 0.16724E+00_JPRB, 0.16650E+00_JPRB, 0.16578E+00_JPRB, 0.16604E+00_JPRB ,&
&  0.16718E+00_JPRB, 0.16844E+00_JPRB, 0.16924E+00_JPRB, 0.16942E+00_JPRB, 0.16892E+00_JPRB, 0.16918E+00_JPRB, 0.16997E+00_JPRB ,&
&  0.16994E+00_JPRB, 0.17038E+00_JPRB, 0.17363E+00_JPRB, 0.17430E+00_JPRB, 0.17530E+00_JPRB, 0.17551E+00_JPRB, 0.17559E+00_JPRB ,&
&  0.17135E+00_JPRB, 0.16636E+00_JPRB, 0.16300E+00_JPRB, 0.16488E+00_JPRB, 0.17325E+00_JPRB, 0.17496E+00_JPRB, 0.17792E+00_JPRB ,&
&  0.17991E+00_JPRB, 0.18109E+00_JPRB, 0.18231E+00_JPRB, 0.18346E+00_JPRB, 0.18485E+00_JPRB, 0.18388E+00_JPRB, 0.18175E+00_JPRB ,&
&  0.17927E+00_JPRB, 0.17830E+00_JPRB, 0.17519E+00_JPRB, 0.16191E+00_JPRB, 0.15356E+00_JPRB, 0.14884E+00_JPRB, 0.14397E+00_JPRB ,&
&  0.13958E+00_JPRB, 0.13692E+00_JPRB, 0.13609E+00_JPRB, 0.13500E+00_JPRB, 0.13580E+00_JPRB, 0.13741E+00_JPRB, 0.13811E+00_JPRB ,&
&  0.13781E+00_JPRB, 0.13815E+00_JPRB, 0.13784E+00_JPRB, 0.13698E+00_JPRB, 0.13552E+00_JPRB, 0.13459E+00_JPRB, 0.13404E+00_JPRB ,&
&  0.13446E+00_JPRB, 0.13436E+00_JPRB, 0.13802E+00_JPRB, 0.13952E+00_JPRB, 0.13565E+00_JPRB, 0.13384E+00_JPRB, 0.13169E+00_JPRB ,&
&  0.12728E+00_JPRB, 0.12478E+00_JPRB, 0.12342E+00_JPRB, 0.12287E+00_JPRB, 0.12368E+00_JPRB, 0.12487E+00_JPRB, 0.12553E+00_JPRB ,&
&  0.12577E+00_JPRB, 0.12663E+00_JPRB, 0.12706E+00_JPRB, 0.12756E+00_JPRB, 0.12790E+00_JPRB, 0.12863E+00_JPRB, 0.12968E+00_JPRB ,&
&  0.13102E+00_JPRB, 0.13340E+00_JPRB, 0.13581E+00_JPRB, 0.13851E+00_JPRB, 0.14017E+00_JPRB, 0.14148E+00_JPRB, 0.14437E+00_JPRB ,&
&  0.14645E+00_JPRB, 0.15288E+00_JPRB, 0.16241E+00_JPRB, 0.16657E+00_JPRB, 0.16253E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.17770E+00_JPRB, 0.17957E+00_JPRB, 0.18023E+00_JPRB, 0.18031E+00_JPRB, 0.18024E+00_JPRB, 0.18077E+00_JPRB, 0.18131E+00_JPRB ,&
&  0.18129E+00_JPRB, 0.18061E+00_JPRB, 0.17948E+00_JPRB, 0.17764E+00_JPRB, 0.17547E+00_JPRB, 0.17426E+00_JPRB, 0.17530E+00_JPRB ,&
&  0.17614E+00_JPRB, 0.17584E+00_JPRB, 0.17565E+00_JPRB, 0.17532E+00_JPRB, 0.17474E+00_JPRB, 0.17481E+00_JPRB, 0.17589E+00_JPRB ,&
&  0.17731E+00_JPRB, 0.17832E+00_JPRB, 0.17977E+00_JPRB, 0.18104E+00_JPRB, 0.18308E+00_JPRB, 0.18492E+00_JPRB, 0.18581E+00_JPRB ,&
&  0.18395E+00_JPRB, 0.17632E+00_JPRB, 0.17075E+00_JPRB, 0.17313E+00_JPRB, 0.18186E+00_JPRB, 0.18648E+00_JPRB, 0.18919E+00_JPRB ,&
&  0.19104E+00_JPRB, 0.19208E+00_JPRB, 0.19237E+00_JPRB, 0.19363E+00_JPRB, 0.19548E+00_JPRB, 0.19722E+00_JPRB, 0.19686E+00_JPRB ,&
&  0.19491E+00_JPRB, 0.19316E+00_JPRB, 0.19064E+00_JPRB, 0.18644E+00_JPRB, 0.18082E+00_JPRB, 0.17531E+00_JPRB, 0.17068E+00_JPRB ,&
&  0.16664E+00_JPRB, 0.16269E+00_JPRB, 0.15926E+00_JPRB, 0.15704E+00_JPRB, 0.15629E+00_JPRB, 0.15731E+00_JPRB, 0.15715E+00_JPRB ,&
&  0.15673E+00_JPRB, 0.15502E+00_JPRB, 0.15289E+00_JPRB, 0.15044E+00_JPRB, 0.14828E+00_JPRB, 0.14465E+00_JPRB, 0.14034E+00_JPRB ,&
&  0.13805E+00_JPRB, 0.14002E+00_JPRB, 0.14598E+00_JPRB, 0.15121E+00_JPRB, 0.15337E+00_JPRB, 0.14586E+00_JPRB, 0.14156E+00_JPRB ,&
&  0.13754E+00_JPRB, 0.13384E+00_JPRB, 0.13154E+00_JPRB, 0.13097E+00_JPRB, 0.13108E+00_JPRB, 0.13125E+00_JPRB, 0.13208E+00_JPRB ,&
&  0.13339E+00_JPRB, 0.13459E+00_JPRB, 0.13529E+00_JPRB, 0.13598E+00_JPRB, 0.13626E+00_JPRB, 0.13631E+00_JPRB, 0.13643E+00_JPRB ,&
&  0.13770E+00_JPRB, 0.14047E+00_JPRB, 0.14434E+00_JPRB, 0.14877E+00_JPRB, 0.15225E+00_JPRB, 0.15544E+00_JPRB, 0.15859E+00_JPRB ,&
&  0.16207E+00_JPRB, 0.16484E+00_JPRB, 0.16771E+00_JPRB, 0.17138E+00_JPRB, 0.17471E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.18750E+00_JPRB, 0.18958E+00_JPRB, 0.19041E+00_JPRB, 0.19017E+00_JPRB, 0.18987E+00_JPRB, 0.19085E+00_JPRB, 0.19214E+00_JPRB ,&
&  0.19278E+00_JPRB, 0.19283E+00_JPRB, 0.19206E+00_JPRB, 0.19057E+00_JPRB, 0.18952E+00_JPRB, 0.18991E+00_JPRB, 0.18839E+00_JPRB ,&
&  0.18680E+00_JPRB, 0.18668E+00_JPRB, 0.18513E+00_JPRB, 0.18347E+00_JPRB, 0.18104E+00_JPRB, 0.17953E+00_JPRB, 0.17998E+00_JPRB ,&
&  0.18202E+00_JPRB, 0.18277E+00_JPRB, 0.18361E+00_JPRB, 0.18474E+00_JPRB, 0.18679E+00_JPRB, 0.18913E+00_JPRB, 0.19084E+00_JPRB ,&
&  0.19221E+00_JPRB, 0.19189E+00_JPRB, 0.19015E+00_JPRB, 0.19313E+00_JPRB, 0.19596E+00_JPRB, 0.19947E+00_JPRB, 0.20184E+00_JPRB ,&
&  0.20391E+00_JPRB, 0.20547E+00_JPRB, 0.20617E+00_JPRB, 0.20607E+00_JPRB, 0.20642E+00_JPRB, 0.20804E+00_JPRB, 0.20914E+00_JPRB ,&
&  0.20941E+00_JPRB, 0.20881E+00_JPRB, 0.20731E+00_JPRB, 0.20580E+00_JPRB, 0.20245E+00_JPRB, 0.19752E+00_JPRB, 0.19035E+00_JPRB ,&
&  0.18299E+00_JPRB, 0.17714E+00_JPRB, 0.17269E+00_JPRB, 0.16907E+00_JPRB, 0.16651E+00_JPRB, 0.16608E+00_JPRB, 0.16597E+00_JPRB ,&
&  0.16560E+00_JPRB, 0.16528E+00_JPRB, 0.16668E+00_JPRB, 0.16763E+00_JPRB, 0.16452E+00_JPRB, 0.15909E+00_JPRB, 0.15486E+00_JPRB ,&
&  0.15396E+00_JPRB, 0.15552E+00_JPRB, 0.15808E+00_JPRB, 0.15972E+00_JPRB, 0.16251E+00_JPRB, 0.16322E+00_JPRB, 0.16240E+00_JPRB ,&
&  0.15773E+00_JPRB, 0.15204E+00_JPRB, 0.14885E+00_JPRB, 0.14656E+00_JPRB, 0.14507E+00_JPRB, 0.14469E+00_JPRB, 0.14521E+00_JPRB ,&
&  0.14707E+00_JPRB, 0.14907E+00_JPRB, 0.15123E+00_JPRB, 0.15222E+00_JPRB, 0.15186E+00_JPRB, 0.15125E+00_JPRB, 0.15046E+00_JPRB ,&
&  0.15132E+00_JPRB, 0.15503E+00_JPRB, 0.16096E+00_JPRB, 0.16479E+00_JPRB, 0.16820E+00_JPRB, 0.17048E+00_JPRB, 0.17171E+00_JPRB ,&
&  0.17264E+00_JPRB, 0.17334E+00_JPRB, 0.17425E+00_JPRB, 0.17856E+00_JPRB, 0.18411E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.19902E+00_JPRB, 0.20126E+00_JPRB, 0.20255E+00_JPRB, 0.20265E+00_JPRB, 0.20240E+00_JPRB, 0.20324E+00_JPRB, 0.20375E+00_JPRB ,&
&  0.20428E+00_JPRB, 0.20455E+00_JPRB, 0.20363E+00_JPRB, 0.20289E+00_JPRB, 0.20461E+00_JPRB, 0.20415E+00_JPRB, 0.20361E+00_JPRB ,&
&  0.20198E+00_JPRB, 0.20007E+00_JPRB, 0.19726E+00_JPRB, 0.19403E+00_JPRB, 0.19007E+00_JPRB, 0.18751E+00_JPRB, 0.18765E+00_JPRB ,&
&  0.18904E+00_JPRB, 0.18960E+00_JPRB, 0.19050E+00_JPRB, 0.19180E+00_JPRB, 0.19437E+00_JPRB, 0.19760E+00_JPRB, 0.20009E+00_JPRB ,&
&  0.20214E+00_JPRB, 0.20541E+00_JPRB, 0.20746E+00_JPRB, 0.20608E+00_JPRB, 0.21033E+00_JPRB, 0.21408E+00_JPRB, 0.21733E+00_JPRB ,&
&  0.21994E+00_JPRB, 0.22134E+00_JPRB, 0.22119E+00_JPRB, 0.22015E+00_JPRB, 0.21935E+00_JPRB, 0.22023E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.22071E+00_JPRB, 0.22245E+00_JPRB, 0.22461E+00_JPRB, 0.22507E+00_JPRB, 0.22327E+00_JPRB, 0.21916E+00_JPRB, 0.21290E+00_JPRB ,&
&  0.20498E+00_JPRB, 0.19674E+00_JPRB, 0.19009E+00_JPRB, 0.18480E+00_JPRB, 0.18071E+00_JPRB, 0.17730E+00_JPRB, 0.17453E+00_JPRB ,&
&  0.17152E+00_JPRB, 0.16948E+00_JPRB, 0.16952E+00_JPRB, 0.17024E+00_JPRB, 0.17068E+00_JPRB, 0.16973E+00_JPRB, 0.16906E+00_JPRB ,&
&  0.16835E+00_JPRB, 0.16882E+00_JPRB, 0.16960E+00_JPRB, 0.16992E+00_JPRB, 0.17003E+00_JPRB, 0.16964E+00_JPRB, 0.16926E+00_JPRB ,&
&  0.16993E+00_JPRB, 0.17026E+00_JPRB, 0.16947E+00_JPRB, 0.16785E+00_JPRB, 0.16646E+00_JPRB, 0.16615E+00_JPRB, 0.16624E+00_JPRB ,&
&  0.16662E+00_JPRB, 0.16749E+00_JPRB, 0.16892E+00_JPRB, 0.17043E+00_JPRB, 0.17161E+00_JPRB, 0.17209E+00_JPRB, 0.17217E+00_JPRB ,&
&  0.17223E+00_JPRB, 0.17194E+00_JPRB, 0.17255E+00_JPRB, 0.17499E+00_JPRB, 0.17776E+00_JPRB, 0.18012E+00_JPRB, 0.18152E+00_JPRB ,&
&  0.18131E+00_JPRB, 0.18216E+00_JPRB, 0.18527E+00_JPRB, 0.19114E+00_JPRB, 0.19559E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.21479E+00_JPRB, 0.21724E+00_JPRB, 0.21992E+00_JPRB, 0.22033E+00_JPRB, 0.21896E+00_JPRB, 0.21835E+00_JPRB, 0.21856E+00_JPRB ,&
&  0.21902E+00_JPRB, 0.21935E+00_JPRB, 0.21841E+00_JPRB, 0.21847E+00_JPRB, 0.21862E+00_JPRB, 0.21776E+00_JPRB, 0.21703E+00_JPRB ,&
&  0.21569E+00_JPRB, 0.21282E+00_JPRB, 0.20885E+00_JPRB, 0.20398E+00_JPRB, 0.20031E+00_JPRB, 0.19832E+00_JPRB, 0.19795E+00_JPRB ,&
&  0.19855E+00_JPRB, 0.19904E+00_JPRB, 0.20155E+00_JPRB, 0.20388E+00_JPRB, 0.20686E+00_JPRB, 0.21074E+00_JPRB, 0.21475E+00_JPRB ,&
&  0.21780E+00_JPRB, 0.22091E+00_JPRB, 0.22388E+00_JPRB, 0.22658E+00_JPRB, 0.23008E+00_JPRB, 0.23533E+00_JPRB, 0.23975E+00_JPRB ,&
&  0.24387E+00_JPRB, 0.24545E+00_JPRB, 0.24270E+00_JPRB, 0.23985E+00_JPRB, 0.23960E+00_JPRB, 0.24088E+00_JPRB, 0.24201E+00_JPRB ,&
&  0.24282E+00_JPRB, 0.24373E+00_JPRB, 0.24446E+00_JPRB, 0.24375E+00_JPRB, 0.24089E+00_JPRB, 0.23621E+00_JPRB, 0.23025E+00_JPRB ,&
&  0.22382E+00_JPRB, 0.21748E+00_JPRB, 0.21147E+00_JPRB, 0.20549E+00_JPRB, 0.20041E+00_JPRB, 0.19543E+00_JPRB, 0.19078E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18226E+00_JPRB, 0.17917E+00_JPRB, 0.17735E+00_JPRB, 0.17718E+00_JPRB, 0.17646E+00_JPRB, 0.17669E+00_JPRB ,&
&  0.17644E+00_JPRB, 0.17644E+00_JPRB, 0.17683E+00_JPRB, 0.17788E+00_JPRB, 0.17939E+00_JPRB, 0.17957E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.17788E+00_JPRB, 0.17892E+00_JPRB, 0.17867E+00_JPRB, 0.17650E+00_JPRB, 0.17378E+00_JPRB, 0.17306E+00_JPRB, 0.17424E+00_JPRB ,&
&  0.17660E+00_JPRB, 0.17888E+00_JPRB, 0.18082E+00_JPRB, 0.18261E+00_JPRB, 0.18412E+00_JPRB, 0.18600E+00_JPRB, 0.18772E+00_JPRB ,&
&  0.18912E+00_JPRB, 0.18970E+00_JPRB, 0.19098E+00_JPRB, 0.19235E+00_JPRB, 0.19385E+00_JPRB, 0.19495E+00_JPRB, 0.19485E+00_JPRB ,&
&  0.19448E+00_JPRB, 0.19592E+00_JPRB, 0.20046E+00_JPRB, 0.20753E+00_JPRB, 0.21203E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.23414E+00_JPRB, 0.23622E+00_JPRB, 0.23904E+00_JPRB, 0.24148E+00_JPRB, 0.24255E+00_JPRB, 0.24224E+00_JPRB, 0.24159E+00_JPRB ,&
&  0.24163E+00_JPRB, 0.24183E+00_JPRB, 0.24061E+00_JPRB, 0.23835E+00_JPRB, 0.23599E+00_JPRB, 0.23343E+00_JPRB, 0.23146E+00_JPRB ,&
&  0.22951E+00_JPRB, 0.22745E+00_JPRB, 0.22392E+00_JPRB, 0.21882E+00_JPRB, 0.21353E+00_JPRB, 0.21045E+00_JPRB, 0.20896E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.20941E+00_JPRB, 0.21199E+00_JPRB, 0.21389E+00_JPRB, 0.21646E+00_JPRB, 0.22065E+00_JPRB, 0.22630E+00_JPRB ,&
&  0.23165E+00_JPRB, 0.23673E+00_JPRB, 0.24181E+00_JPRB, 0.24611E+00_JPRB, 0.25048E+00_JPRB, 0.25597E+00_JPRB, 0.26147E+00_JPRB ,&
&  0.26683E+00_JPRB, 0.27031E+00_JPRB, 0.27065E+00_JPRB, 0.27141E+00_JPRB, 0.27354E+00_JPRB, 0.27591E+00_JPRB, 0.27723E+00_JPRB ,&
&  0.27805E+00_JPRB, 0.27598E+00_JPRB, 0.27305E+00_JPRB, 0.26905E+00_JPRB, 0.26490E+00_JPRB, 0.26041E+00_JPRB, 0.25450E+00_JPRB ,&
&  0.24747E+00_JPRB, 0.23983E+00_JPRB, 0.23255E+00_JPRB, 0.22565E+00_JPRB, 0.21906E+00_JPRB, 0.21294E+00_JPRB, 0.20734E+00_JPRB ,&
&  0.20185E+00_JPRB, 0.19764E+00_JPRB, 0.19361E+00_JPRB, 0.19134E+00_JPRB, 0.19018E+00_JPRB, 0.18874E+00_JPRB, 0.18870E+00_JPRB ,&
&  0.18810E+00_JPRB, 0.18783E+00_JPRB, 0.18778E+00_JPRB, 0.18945E+00_JPRB, 0.19252E+00_JPRB, 0.19432E+00_JPRB, 0.19322E+00_JPRB ,&
&  0.19002E+00_JPRB, 0.18801E+00_JPRB, 0.18689E+00_JPRB, 0.18599E+00_JPRB, 0.18372E+00_JPRB, 0.18333E+00_JPRB, 0.18502E+00_JPRB ,&
&  0.18873E+00_JPRB, 0.19286E+00_JPRB, 0.19657E+00_JPRB, 0.19876E+00_JPRB, 0.20094E+00_JPRB, 0.20257E+00_JPRB, 0.20400E+00_JPRB ,&
&  0.20569E+00_JPRB, 0.20692E+00_JPRB, 0.20788E+00_JPRB, 0.20923E+00_JPRB, 0.21156E+00_JPRB, 0.21390E+00_JPRB, 0.21623E+00_JPRB ,&
&  0.21869E+00_JPRB, 0.22239E+00_JPRB, 0.22613E+00_JPRB, 0.22883E+00_JPRB, 0.23209E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.26199E+00_JPRB, 0.26371E+00_JPRB, 0.26585E+00_JPRB, 0.26642E+00_JPRB, 0.26606E+00_JPRB, 0.26622E+00_JPRB, 0.26533E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26221E+00_JPRB, 0.26252E+00_JPRB, 0.26128E+00_JPRB, 0.25867E+00_JPRB, 0.25543E+00_JPRB, 0.25186E+00_JPRB ,&
&  0.24904E+00_JPRB, 0.24617E+00_JPRB, 0.24138E+00_JPRB, 0.23495E+00_JPRB, 0.22942E+00_JPRB, 0.22590E+00_JPRB, 0.22416E+00_JPRB ,&
&  0.22355E+00_JPRB, 0.22375E+00_JPRB, 0.22356E+00_JPRB, 0.22434E+00_JPRB, 0.22651E+00_JPRB, 0.23128E+00_JPRB, 0.23831E+00_JPRB ,&
&  0.24544E+00_JPRB, 0.25187E+00_JPRB, 0.25842E+00_JPRB, 0.26555E+00_JPRB, 0.27424E+00_JPRB, 0.28442E+00_JPRB, 0.29295E+00_JPRB ,&
&  0.30012E+00_JPRB, 0.30231E+00_JPRB, 0.30335E+00_JPRB, 0.30717E+00_JPRB, 0.31455E+00_JPRB, 0.31954E+00_JPRB, 0.32276E+00_JPRB ,&
&  0.32420E+00_JPRB, 0.32264E+00_JPRB, 0.31998E+00_JPRB, 0.31498E+00_JPRB, 0.30804E+00_JPRB, 0.29922E+00_JPRB, 0.28978E+00_JPRB ,&
&  0.28047E+00_JPRB, 0.27090E+00_JPRB, 0.26055E+00_JPRB, 0.25069E+00_JPRB, 0.24233E+00_JPRB, 0.23496E+00_JPRB, 0.22748E+00_JPRB ,&
&  0.22044E+00_JPRB, 0.21477E+00_JPRB, 0.21030E+00_JPRB, 0.20734E+00_JPRB, 0.20501E+00_JPRB, 0.20326E+00_JPRB, 0.20299E+00_JPRB ,&
&  0.20347E+00_JPRB, 0.20491E+00_JPRB, 0.20558E+00_JPRB, 0.20699E+00_JPRB, 0.20934E+00_JPRB, 0.21039E+00_JPRB, 0.20945E+00_JPRB ,&
&  0.20577E+00_JPRB, 0.20189E+00_JPRB, 0.19825E+00_JPRB, 0.19749E+00_JPRB, 0.19733E+00_JPRB, 0.19831E+00_JPRB, 0.19984E+00_JPRB ,&
&  0.20315E+00_JPRB, 0.20816E+00_JPRB, 0.21261E+00_JPRB, 0.21564E+00_JPRB, 0.21903E+00_JPRB, 0.22196E+00_JPRB, 0.22496E+00_JPRB ,&
&  0.22846E+00_JPRB, 0.23232E+00_JPRB, 0.23759E+00_JPRB, 0.24282E+00_JPRB, 0.24755E+00_JPRB, 0.25133E+00_JPRB, 0.25411E+00_JPRB ,&
&  0.25592E+00_JPRB, 0.25632E+00_JPRB, 0.25646E+00_JPRB, 0.25780E+00_JPRB, 0.26011E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.29302E+00_JPRB, 0.29356E+00_JPRB, 0.29467E+00_JPRB, 0.29546E+00_JPRB, 0.29330E+00_JPRB, 0.29082E+00_JPRB, 0.28777E+00_JPRB ,&
&  0.28649E+00_JPRB, 0.28552E+00_JPRB, 0.28515E+00_JPRB, 0.28232E+00_JPRB, 0.28013E+00_JPRB, 0.27881E+00_JPRB, 0.27554E+00_JPRB ,&
&  0.27261E+00_JPRB, 0.26579E+00_JPRB, 0.25840E+00_JPRB, 0.25286E+00_JPRB, 0.24933E+00_JPRB, 0.24703E+00_JPRB, 0.24239E+00_JPRB ,&
&  0.23923E+00_JPRB, 0.23678E+00_JPRB, 0.23574E+00_JPRB, 0.23607E+00_JPRB, 0.23822E+00_JPRB, 0.24400E+00_JPRB, 0.25366E+00_JPRB ,&
&  0.26049E+00_JPRB, 0.26885E+00_JPRB, 0.26888E+00_JPRB, 0.28175E+00_JPRB, 0.29440E+00_JPRB, 0.31128E+00_JPRB, 0.32490E+00_JPRB ,&
&  0.33474E+00_JPRB, 0.34270E+00_JPRB, 0.35233E+00_JPRB, 0.36154E+00_JPRB, 0.36544E+00_JPRB, 0.36715E+00_JPRB, 0.36651E+00_JPRB ,&
&  0.36365E+00_JPRB, 0.35885E+00_JPRB, 0.35324E+00_JPRB, 0.34643E+00_JPRB, 0.33710E+00_JPRB, 0.32629E+00_JPRB, 0.31546E+00_JPRB ,&
&  0.30542E+00_JPRB, 0.29556E+00_JPRB, 0.28587E+00_JPRB, 0.27549E+00_JPRB, 0.26525E+00_JPRB, 0.25598E+00_JPRB, 0.24653E+00_JPRB ,&
&  0.23853E+00_JPRB, 0.23224E+00_JPRB, 0.22757E+00_JPRB, 0.22433E+00_JPRB, 0.22199E+00_JPRB, 0.21968E+00_JPRB, 0.21869E+00_JPRB ,&
&  0.21895E+00_JPRB, 0.22056E+00_JPRB, 0.22273E+00_JPRB, 0.22450E+00_JPRB, 0.22663E+00_JPRB, 0.22701E+00_JPRB, 0.22468E+00_JPRB ,&
&  0.22214E+00_JPRB, 0.22079E+00_JPRB, 0.21661E+00_JPRB, 0.21409E+00_JPRB, 0.21358E+00_JPRB, 0.21396E+00_JPRB, 0.21566E+00_JPRB ,&
&  0.21910E+00_JPRB, 0.22395E+00_JPRB, 0.22841E+00_JPRB, 0.23221E+00_JPRB, 0.23539E+00_JPRB, 0.23875E+00_JPRB, 0.24279E+00_JPRB ,&
&  0.24710E+00_JPRB, 0.25131E+00_JPRB, 0.25799E+00_JPRB, 0.26544E+00_JPRB, 0.27179E+00_JPRB, 0.27857E+00_JPRB, 0.28271E+00_JPRB ,&
&  0.28422E+00_JPRB, 0.28479E+00_JPRB, 0.28686E+00_JPRB, 0.28999E+00_JPRB, 0.29230E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.33228E+00_JPRB, 0.33056E+00_JPRB, 0.32819E+00_JPRB, 0.32631E+00_JPRB, 0.32514E+00_JPRB, 0.32208E+00_JPRB, 0.31803E+00_JPRB ,&
&  0.31523E+00_JPRB, 0.31251E+00_JPRB, 0.31052E+00_JPRB, 0.30736E+00_JPRB, 0.30565E+00_JPRB, 0.30463E+00_JPRB, 0.30095E+00_JPRB ,&
&  0.29659E+00_JPRB, 0.29045E+00_JPRB, 0.28444E+00_JPRB, 0.27905E+00_JPRB, 0.27397E+00_JPRB, 0.26933E+00_JPRB, 0.26417E+00_JPRB ,&
&  0.25931E+00_JPRB, 0.25576E+00_JPRB, 0.25401E+00_JPRB, 0.25244E+00_JPRB, 0.25391E+00_JPRB, 0.25981E+00_JPRB, 0.26926E+00_JPRB ,&
&  0.27481E+00_JPRB, 0.27581E+00_JPRB, 0.28597E+00_JPRB, 0.29582E+00_JPRB, 0.31446E+00_JPRB, 0.33892E+00_JPRB, 0.35719E+00_JPRB ,&
&  0.36904E+00_JPRB, 0.38163E+00_JPRB, 0.39433E+00_JPRB, 0.40165E+00_JPRB, 0.40213E+00_JPRB, 0.39718E+00_JPRB, 0.39341E+00_JPRB ,&
&  0.38885E+00_JPRB, 0.38024E+00_JPRB, 0.37104E+00_JPRB, 0.35923E+00_JPRB, 0.34633E+00_JPRB, 0.33555E+00_JPRB, 0.32680E+00_JPRB ,&
&  0.31852E+00_JPRB, 0.31034E+00_JPRB, 0.30285E+00_JPRB, 0.29530E+00_JPRB, 0.28712E+00_JPRB, 0.27859E+00_JPRB, 0.26963E+00_JPRB ,&
&  0.26087E+00_JPRB, 0.25325E+00_JPRB, 0.24713E+00_JPRB, 0.24395E+00_JPRB, 0.24227E+00_JPRB, 0.24098E+00_JPRB, 0.24016E+00_JPRB ,&
&  0.24066E+00_JPRB, 0.24210E+00_JPRB, 0.24396E+00_JPRB, 0.24564E+00_JPRB, 0.24697E+00_JPRB, 0.24706E+00_JPRB, 0.24540E+00_JPRB ,&
&  0.24369E+00_JPRB, 0.24296E+00_JPRB, 0.24272E+00_JPRB, 0.24031E+00_JPRB, 0.23776E+00_JPRB, 0.23811E+00_JPRB, 0.23990E+00_JPRB ,&
&  0.24346E+00_JPRB, 0.24675E+00_JPRB, 0.24963E+00_JPRB, 0.25201E+00_JPRB, 0.25381E+00_JPRB, 0.25701E+00_JPRB, 0.26092E+00_JPRB ,&
&  0.26579E+00_JPRB, 0.27185E+00_JPRB, 0.28048E+00_JPRB, 0.29115E+00_JPRB, 0.30089E+00_JPRB, 0.30900E+00_JPRB, 0.31413E+00_JPRB ,&
&  0.31720E+00_JPRB, 0.31965E+00_JPRB, 0.32240E+00_JPRB, 0.32642E+00_JPRB, 0.33052E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.37369E+00_JPRB, 0.36951E+00_JPRB, 0.36355E+00_JPRB, 0.35844E+00_JPRB, 0.35602E+00_JPRB, 0.35399E+00_JPRB, 0.34981E+00_JPRB ,&
&  0.34586E+00_JPRB, 0.34219E+00_JPRB, 0.33832E+00_JPRB, 0.33495E+00_JPRB, 0.33216E+00_JPRB, 0.32999E+00_JPRB, 0.32639E+00_JPRB ,&
&  0.32113E+00_JPRB, 0.31468E+00_JPRB, 0.30772E+00_JPRB, 0.30220E+00_JPRB, 0.29541E+00_JPRB, 0.28840E+00_JPRB, 0.28329E+00_JPRB ,&
&  0.27903E+00_JPRB, 0.27617E+00_JPRB, 0.27528E+00_JPRB, 0.27389E+00_JPRB, 0.27495E+00_JPRB, 0.27832E+00_JPRB, 0.28636E+00_JPRB ,&
&  0.29585E+00_JPRB, 0.27692E+00_JPRB, 0.29065E+00_JPRB, 0.30219E+00_JPRB, 0.32668E+00_JPRB, 0.36119E+00_JPRB, 0.38553E+00_JPRB ,&
&  0.40157E+00_JPRB, 0.41294E+00_JPRB, 0.42127E+00_JPRB, 0.42387E+00_JPRB, 0.41823E+00_JPRB, 0.41389E+00_JPRB, 0.41059E+00_JPRB ,&
&  0.40353E+00_JPRB, 0.39377E+00_JPRB, 0.38082E+00_JPRB, 0.36692E+00_JPRB, 0.35403E+00_JPRB, 0.34439E+00_JPRB, 0.33656E+00_JPRB ,&
&  0.33025E+00_JPRB, 0.32433E+00_JPRB, 0.31916E+00_JPRB, 0.31286E+00_JPRB, 0.30545E+00_JPRB, 0.29773E+00_JPRB, 0.29086E+00_JPRB ,&
&  0.28416E+00_JPRB, 0.27746E+00_JPRB, 0.27153E+00_JPRB, 0.26654E+00_JPRB, 0.26391E+00_JPRB, 0.26289E+00_JPRB, 0.26296E+00_JPRB ,&
&  0.26383E+00_JPRB, 0.26582E+00_JPRB, 0.26757E+00_JPRB, 0.26869E+00_JPRB, 0.26923E+00_JPRB, 0.26843E+00_JPRB, 0.26614E+00_JPRB ,&
&  0.26595E+00_JPRB, 0.26591E+00_JPRB, 0.26581E+00_JPRB, 0.26624E+00_JPRB, 0.26587E+00_JPRB, 0.26595E+00_JPRB, 0.26781E+00_JPRB ,&
&  0.26976E+00_JPRB, 0.27103E+00_JPRB, 0.27254E+00_JPRB, 0.27437E+00_JPRB, 0.27668E+00_JPRB, 0.28027E+00_JPRB, 0.28530E+00_JPRB ,&
&  0.29098E+00_JPRB, 0.29734E+00_JPRB, 0.30657E+00_JPRB, 0.31853E+00_JPRB, 0.33029E+00_JPRB, 0.34103E+00_JPRB, 0.35043E+00_JPRB ,&
&  0.35466E+00_JPRB, 0.35867E+00_JPRB, 0.36504E+00_JPRB, 0.36953E+00_JPRB, 0.37386E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.42508E+00_JPRB, 0.41747E+00_JPRB, 0.40722E+00_JPRB, 0.40013E+00_JPRB, 0.39687E+00_JPRB, 0.39294E+00_JPRB, 0.38884E+00_JPRB ,&
&  0.38269E+00_JPRB, 0.37576E+00_JPRB, 0.36911E+00_JPRB, 0.36335E+00_JPRB, 0.35868E+00_JPRB, 0.35518E+00_JPRB, 0.35109E+00_JPRB ,&
&  0.34553E+00_JPRB, 0.33943E+00_JPRB, 0.33353E+00_JPRB, 0.32703E+00_JPRB, 0.31838E+00_JPRB, 0.31156E+00_JPRB, 0.30615E+00_JPRB ,&
&  0.30188E+00_JPRB, 0.29848E+00_JPRB, 0.29752E+00_JPRB, 0.29614E+00_JPRB, 0.29462E+00_JPRB, 0.29499E+00_JPRB, 0.30404E+00_JPRB ,&
&  0.29768E+00_JPRB, 0.27389E+00_JPRB, 0.28874E+00_JPRB, 0.30863E+00_JPRB, 0.33817E+00_JPRB, 0.38377E+00_JPRB, 0.41558E+00_JPRB ,&
&  0.43312E+00_JPRB, 0.44140E+00_JPRB, 0.44446E+00_JPRB, 0.44195E+00_JPRB, 0.43730E+00_JPRB, 0.43568E+00_JPRB, 0.42964E+00_JPRB ,&
&  0.41836E+00_JPRB, 0.40503E+00_JPRB, 0.39173E+00_JPRB, 0.38197E+00_JPRB, 0.37208E+00_JPRB, 0.36477E+00_JPRB, 0.35688E+00_JPRB ,&
&  0.34881E+00_JPRB, 0.34197E+00_JPRB, 0.33832E+00_JPRB, 0.33569E+00_JPRB, 0.33172E+00_JPRB, 0.32545E+00_JPRB, 0.31885E+00_JPRB ,&
&  0.31364E+00_JPRB, 0.30966E+00_JPRB, 0.30600E+00_JPRB, 0.30244E+00_JPRB, 0.30068E+00_JPRB, 0.29984E+00_JPRB, 0.29831E+00_JPRB ,&
&  0.29781E+00_JPRB, 0.29773E+00_JPRB, 0.29784E+00_JPRB, 0.29771E+00_JPRB, 0.29700E+00_JPRB, 0.29576E+00_JPRB, 0.29624E+00_JPRB ,&
&  0.29787E+00_JPRB, 0.29699E+00_JPRB, 0.29647E+00_JPRB, 0.29814E+00_JPRB, 0.29998E+00_JPRB, 0.30119E+00_JPRB, 0.30135E+00_JPRB ,&
&  0.30053E+00_JPRB, 0.30070E+00_JPRB, 0.30235E+00_JPRB, 0.30571E+00_JPRB, 0.30975E+00_JPRB, 0.31555E+00_JPRB, 0.32269E+00_JPRB ,&
&  0.33023E+00_JPRB, 0.33952E+00_JPRB, 0.35218E+00_JPRB, 0.37126E+00_JPRB, 0.38663E+00_JPRB, 0.39923E+00_JPRB, 0.40981E+00_JPRB ,&
&  0.41294E+00_JPRB, 0.41434E+00_JPRB, 0.42088E+00_JPRB, 0.42347E+00_JPRB, 0.42596E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.44965E+00_JPRB, 0.45111E+00_JPRB, 0.44576E+00_JPRB, 0.43824E+00_JPRB, 0.43117E+00_JPRB, 0.42352E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.40827E+00_JPRB, 0.39802E+00_JPRB, 0.39068E+00_JPRB, 0.38742E+00_JPRB, 0.38549E+00_JPRB, 0.38235E+00_JPRB, 0.37707E+00_JPRB ,&
&  0.37056E+00_JPRB, 0.36398E+00_JPRB, 0.35798E+00_JPRB, 0.35076E+00_JPRB, 0.34387E+00_JPRB, 0.33778E+00_JPRB, 0.33328E+00_JPRB ,&
&  0.32883E+00_JPRB, 0.32387E+00_JPRB, 0.31895E+00_JPRB, 0.31582E+00_JPRB, 0.31319E+00_JPRB, 0.31195E+00_JPRB, 0.32066E+00_JPRB ,&
&  0.34891E+00_JPRB, 0.30074E+00_JPRB, 0.31649E+00_JPRB, 0.33887E+00_JPRB, 0.37206E+00_JPRB, 0.41548E+00_JPRB, 0.44182E+00_JPRB ,&
&  0.45614E+00_JPRB, 0.46486E+00_JPRB, 0.46656E+00_JPRB, 0.46657E+00_JPRB, 0.46755E+00_JPRB, 0.46012E+00_JPRB, 0.44520E+00_JPRB ,&
&  0.43032E+00_JPRB, 0.41574E+00_JPRB, 0.40418E+00_JPRB, 0.39574E+00_JPRB, 0.38942E+00_JPRB, 0.38581E+00_JPRB, 0.37962E+00_JPRB ,&
&  0.36855E+00_JPRB, 0.36274E+00_JPRB, 0.35996E+00_JPRB, 0.35726E+00_JPRB, 0.35564E+00_JPRB, 0.35292E+00_JPRB, 0.34562E+00_JPRB ,&
&  0.33868E+00_JPRB, 0.33486E+00_JPRB, 0.33367E+00_JPRB, 0.33341E+00_JPRB, 0.33411E+00_JPRB, 0.33361E+00_JPRB, 0.33128E+00_JPRB ,&
&  0.32838E+00_JPRB, 0.32458E+00_JPRB, 0.32236E+00_JPRB, 0.32065E+00_JPRB, 0.31926E+00_JPRB, 0.32085E+00_JPRB, 0.32325E+00_JPRB ,&
&  0.32440E+00_JPRB, 0.32576E+00_JPRB, 0.32820E+00_JPRB, 0.32998E+00_JPRB, 0.33011E+00_JPRB, 0.32926E+00_JPRB, 0.32800E+00_JPRB ,&
&  0.32672E+00_JPRB, 0.32640E+00_JPRB, 0.32820E+00_JPRB, 0.33333E+00_JPRB, 0.34028E+00_JPRB, 0.34848E+00_JPRB, 0.35766E+00_JPRB ,&
&  0.36826E+00_JPRB, 0.37974E+00_JPRB, 0.39287E+00_JPRB, 0.41967E+00_JPRB, 0.43524E+00_JPRB, 0.44954E+00_JPRB, 0.46235E+00_JPRB ,&
&  0.46596E+00_JPRB, 0.46520E+00_JPRB, 0.39543E+00_JPRB, 0.42945E+00_JPRB, 0.44046E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.49005E+00_JPRB, 0.49436E+00_JPRB, 0.48735E+00_JPRB, 0.47432E+00_JPRB, 0.46189E+00_JPRB, 0.45023E+00_JPRB, 0.44079E+00_JPRB ,&
&  0.42897E+00_JPRB, 0.41928E+00_JPRB, 0.41554E+00_JPRB, 0.41464E+00_JPRB, 0.41337E+00_JPRB, 0.40918E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.39797E+00_JPRB, 0.39170E+00_JPRB, 0.38433E+00_JPRB, 0.37701E+00_JPRB, 0.37094E+00_JPRB, 0.36430E+00_JPRB, 0.35966E+00_JPRB ,&
&  0.35556E+00_JPRB, 0.35227E+00_JPRB, 0.34649E+00_JPRB, 0.33892E+00_JPRB, 0.33606E+00_JPRB, 0.33161E+00_JPRB, 0.34163E+00_JPRB ,&
&  0.37869E+00_JPRB, 0.33042E+00_JPRB, 0.35003E+00_JPRB, 0.38612E+00_JPRB, 0.41959E+00_JPRB, 0.45984E+00_JPRB, 0.48147E+00_JPRB ,&
&  0.49655E+00_JPRB, 0.50420E+00_JPRB, 0.50663E+00_JPRB, 0.51233E+00_JPRB, 0.50862E+00_JPRB, 0.49040E+00_JPRB, 0.46705E+00_JPRB ,&
&  0.45078E+00_JPRB, 0.43384E+00_JPRB, 0.42309E+00_JPRB, 0.41301E+00_JPRB, 0.40933E+00_JPRB, 0.40891E+00_JPRB, 0.40420E+00_JPRB ,&
&  0.39649E+00_JPRB, 0.39367E+00_JPRB, 0.38992E+00_JPRB, 0.38955E+00_JPRB, 0.39030E+00_JPRB, 0.39208E+00_JPRB, 0.38866E+00_JPRB ,&
&  0.38479E+00_JPRB, 0.38662E+00_JPRB, 0.38996E+00_JPRB, 0.39002E+00_JPRB, 0.38906E+00_JPRB, 0.38580E+00_JPRB, 0.38108E+00_JPRB ,&
&  0.37671E+00_JPRB, 0.37355E+00_JPRB, 0.37306E+00_JPRB, 0.37212E+00_JPRB, 0.37127E+00_JPRB, 0.37063E+00_JPRB, 0.37028E+00_JPRB ,&
&  0.37066E+00_JPRB, 0.37108E+00_JPRB, 0.37081E+00_JPRB, 0.36956E+00_JPRB, 0.36749E+00_JPRB, 0.36401E+00_JPRB, 0.36279E+00_JPRB ,&
&  0.36313E+00_JPRB, 0.36555E+00_JPRB, 0.37013E+00_JPRB, 0.37684E+00_JPRB, 0.38600E+00_JPRB, 0.39457E+00_JPRB, 0.40455E+00_JPRB ,&
&  0.41516E+00_JPRB, 0.42736E+00_JPRB, 0.44357E+00_JPRB, 0.46924E+00_JPRB, 0.49328E+00_JPRB, 0.50053E+00_JPRB, 0.50815E+00_JPRB ,&
&  0.51187E+00_JPRB, 0.51312E+00_JPRB, 0.51390E+00_JPRB, 0.45898E+00_JPRB, 0.48160E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.53293E+00_JPRB, 0.53677E+00_JPRB, 0.52232E+00_JPRB, 0.50355E+00_JPRB, 0.48524E+00_JPRB, 0.47021E+00_JPRB, 0.45727E+00_JPRB ,&
&  0.44843E+00_JPRB, 0.44246E+00_JPRB, 0.43846E+00_JPRB, 0.43592E+00_JPRB, 0.43240E+00_JPRB, 0.42662E+00_JPRB, 0.42093E+00_JPRB ,&
&  0.41655E+00_JPRB, 0.41144E+00_JPRB, 0.40344E+00_JPRB, 0.39812E+00_JPRB, 0.39525E+00_JPRB, 0.39174E+00_JPRB, 0.38840E+00_JPRB ,&
&  0.38418E+00_JPRB, 0.37484E+00_JPRB, 0.36539E+00_JPRB, 0.35976E+00_JPRB, 0.35562E+00_JPRB, 0.35383E+00_JPRB, 0.36383E+00_JPRB ,&
&  0.40453E+00_JPRB, 0.38192E+00_JPRB, 0.41175E+00_JPRB, 0.44264E+00_JPRB, 0.47817E+00_JPRB, 0.50202E+00_JPRB, 0.52176E+00_JPRB ,&
&  0.53398E+00_JPRB, 0.54489E+00_JPRB, 0.55342E+00_JPRB, 0.56515E+00_JPRB, 0.54674E+00_JPRB, 0.51962E+00_JPRB, 0.49764E+00_JPRB ,&
&  0.47711E+00_JPRB, 0.45727E+00_JPRB, 0.44141E+00_JPRB, 0.43114E+00_JPRB, 0.42909E+00_JPRB, 0.42876E+00_JPRB, 0.42645E+00_JPRB ,&
&  0.42512E+00_JPRB, 0.42072E+00_JPRB, 0.41508E+00_JPRB, 0.41610E+00_JPRB, 0.41502E+00_JPRB, 0.42470E+00_JPRB, 0.43458E+00_JPRB ,&
&  0.44072E+00_JPRB, 0.44826E+00_JPRB, 0.45115E+00_JPRB, 0.45205E+00_JPRB, 0.44763E+00_JPRB, 0.44140E+00_JPRB, 0.43487E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.42616E+00_JPRB, 0.42602E+00_JPRB, 0.42566E+00_JPRB, 0.42420E+00_JPRB, 0.42435E+00_JPRB, 0.42471E+00_JPRB ,&
&  0.42550E+00_JPRB, 0.42527E+00_JPRB, 0.42461E+00_JPRB, 0.41978E+00_JPRB, 0.41290E+00_JPRB, 0.40776E+00_JPRB, 0.40375E+00_JPRB ,&
&  0.40163E+00_JPRB, 0.40323E+00_JPRB, 0.40686E+00_JPRB, 0.41485E+00_JPRB, 0.42338E+00_JPRB, 0.43366E+00_JPRB, 0.44147E+00_JPRB ,&
&  0.44859E+00_JPRB, 0.46161E+00_JPRB, 0.47948E+00_JPRB, 0.50417E+00_JPRB, 0.51626E+00_JPRB, 0.52322E+00_JPRB, 0.53115E+00_JPRB ,&
&  0.53677E+00_JPRB, 0.54016E+00_JPRB, 0.53227E+00_JPRB, 0.52855E+00_JPRB, 0.51903E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.61499E+00_JPRB, 0.59226E+00_JPRB, 0.56096E+00_JPRB, 0.53030E+00_JPRB, 0.50172E+00_JPRB, 0.47911E+00_JPRB, 0.46510E+00_JPRB ,&
&  0.45639E+00_JPRB, 0.45076E+00_JPRB, 0.44716E+00_JPRB, 0.44426E+00_JPRB, 0.44007E+00_JPRB, 0.43664E+00_JPRB, 0.43181E+00_JPRB ,&
&  0.42618E+00_JPRB, 0.41926E+00_JPRB, 0.41304E+00_JPRB, 0.40921E+00_JPRB, 0.40568E+00_JPRB, 0.40417E+00_JPRB, 0.40252E+00_JPRB ,&
&  0.39764E+00_JPRB, 0.39109E+00_JPRB, 0.38229E+00_JPRB, 0.37324E+00_JPRB, 0.36837E+00_JPRB, 0.36839E+00_JPRB, 0.38580E+00_JPRB ,&
&  0.44380E+00_JPRB, 0.44255E+00_JPRB, 0.48043E+00_JPRB, 0.51103E+00_JPRB, 0.54113E+00_JPRB, 0.56763E+00_JPRB, 0.59263E+00_JPRB ,&
&  0.60606E+00_JPRB, 0.61751E+00_JPRB, 0.63697E+00_JPRB, 0.63218E+00_JPRB, 0.59699E+00_JPRB, 0.56595E+00_JPRB, 0.53479E+00_JPRB ,&
&  0.50390E+00_JPRB, 0.47574E+00_JPRB, 0.45213E+00_JPRB, 0.44325E+00_JPRB, 0.44273E+00_JPRB, 0.44611E+00_JPRB, 0.44752E+00_JPRB ,&
&  0.44886E+00_JPRB, 0.44677E+00_JPRB, 0.44900E+00_JPRB, 0.46145E+00_JPRB, 0.47759E+00_JPRB, 0.49853E+00_JPRB, 0.53133E+00_JPRB ,&
&  0.54620E+00_JPRB, 0.54073E+00_JPRB, 0.54278E+00_JPRB, 0.53941E+00_JPRB, 0.52415E+00_JPRB, 0.50986E+00_JPRB, 0.49864E+00_JPRB ,&
&  0.49052E+00_JPRB, 0.48556E+00_JPRB, 0.48369E+00_JPRB, 0.47987E+00_JPRB, 0.47722E+00_JPRB, 0.47652E+00_JPRB, 0.47775E+00_JPRB ,&
&  0.47834E+00_JPRB, 0.47931E+00_JPRB, 0.47830E+00_JPRB, 0.47313E+00_JPRB, 0.47006E+00_JPRB, 0.46727E+00_JPRB, 0.46048E+00_JPRB ,&
&  0.45224E+00_JPRB, 0.44915E+00_JPRB, 0.45236E+00_JPRB, 0.45882E+00_JPRB, 0.46796E+00_JPRB, 0.47694E+00_JPRB, 0.48375E+00_JPRB ,&
&  0.49157E+00_JPRB, 0.50743E+00_JPRB, 0.50652E+00_JPRB, 0.51593E+00_JPRB, 0.54584E+00_JPRB, 0.54973E+00_JPRB, 0.55847E+00_JPRB ,&
&  0.56717E+00_JPRB, 0.56922E+00_JPRB, 0.56083E+00_JPRB, 0.55293E+00_JPRB, 0.61234E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.58639E+00_JPRB, 0.56347E+00_JPRB, 0.53701E+00_JPRB, 0.51277E+00_JPRB, 0.49099E+00_JPRB, 0.47431E+00_JPRB, 0.46414E+00_JPRB ,&
&  0.45629E+00_JPRB, 0.45086E+00_JPRB, 0.45094E+00_JPRB, 0.45376E+00_JPRB, 0.45265E+00_JPRB, 0.44554E+00_JPRB, 0.43585E+00_JPRB ,&
&  0.42619E+00_JPRB, 0.41964E+00_JPRB, 0.41523E+00_JPRB, 0.41441E+00_JPRB, 0.41145E+00_JPRB, 0.40850E+00_JPRB, 0.40375E+00_JPRB ,&
&  0.40080E+00_JPRB, 0.39760E+00_JPRB, 0.39200E+00_JPRB, 0.38636E+00_JPRB, 0.38293E+00_JPRB, 0.38406E+00_JPRB, 0.41082E+00_JPRB ,&
&  0.47751E+00_JPRB, 0.54450E+00_JPRB, 0.57066E+00_JPRB, 0.60340E+00_JPRB, 0.63660E+00_JPRB, 0.65474E+00_JPRB, 0.70575E+00_JPRB ,&
&  0.73295E+00_JPRB, 0.74469E+00_JPRB, 0.75110E+00_JPRB, 0.72223E+00_JPRB, 0.66479E+00_JPRB, 0.61883E+00_JPRB, 0.56866E+00_JPRB ,&
&  0.52126E+00_JPRB, 0.48375E+00_JPRB, 0.46026E+00_JPRB, 0.45128E+00_JPRB, 0.45449E+00_JPRB, 0.46197E+00_JPRB, 0.46654E+00_JPRB ,&
&  0.47118E+00_JPRB, 0.47553E+00_JPRB, 0.49610E+00_JPRB, 0.53412E+00_JPRB, 0.59534E+00_JPRB, 0.63704E+00_JPRB, 0.69720E+00_JPRB ,&
&  0.72472E+00_JPRB, 0.71978E+00_JPRB, 0.70433E+00_JPRB, 0.67502E+00_JPRB, 0.65275E+00_JPRB, 0.62631E+00_JPRB, 0.60509E+00_JPRB ,&
&  0.58053E+00_JPRB, 0.55955E+00_JPRB, 0.54374E+00_JPRB, 0.53304E+00_JPRB, 0.52807E+00_JPRB, 0.52691E+00_JPRB, 0.52862E+00_JPRB ,&
&  0.52719E+00_JPRB, 0.52765E+00_JPRB, 0.52752E+00_JPRB, 0.53160E+00_JPRB, 0.53695E+00_JPRB, 0.53151E+00_JPRB, 0.51972E+00_JPRB ,&
&  0.50657E+00_JPRB, 0.49594E+00_JPRB, 0.49615E+00_JPRB, 0.50232E+00_JPRB, 0.50800E+00_JPRB, 0.51069E+00_JPRB, 0.51270E+00_JPRB ,&
&  0.52415E+00_JPRB, 0.53801E+00_JPRB, 0.54473E+00_JPRB, 0.54640E+00_JPRB, 0.55769E+00_JPRB, 0.59027E+00_JPRB, 0.59545E+00_JPRB ,&
&  0.59809E+00_JPRB, 0.59469E+00_JPRB, 0.58146E+00_JPRB, 0.56949E+00_JPRB, 0.59244E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.57806E+00_JPRB, 0.56173E+00_JPRB, 0.53721E+00_JPRB, 0.51586E+00_JPRB, 0.49914E+00_JPRB, 0.48226E+00_JPRB, 0.47074E+00_JPRB ,&
&  0.46236E+00_JPRB, 0.46271E+00_JPRB, 0.46545E+00_JPRB, 0.46679E+00_JPRB, 0.46287E+00_JPRB, 0.45254E+00_JPRB, 0.44209E+00_JPRB ,&
&  0.43569E+00_JPRB, 0.43566E+00_JPRB, 0.43651E+00_JPRB, 0.43431E+00_JPRB, 0.42982E+00_JPRB, 0.42402E+00_JPRB, 0.41891E+00_JPRB ,&
&  0.41440E+00_JPRB, 0.41020E+00_JPRB, 0.40590E+00_JPRB, 0.40368E+00_JPRB, 0.40102E+00_JPRB, 0.40503E+00_JPRB, 0.43616E+00_JPRB ,&
&  0.52039E+00_JPRB, 0.60747E+00_JPRB, 0.74429E+00_JPRB, 0.73587E+00_JPRB, 0.74773E+00_JPRB, 0.78758E+00_JPRB, 0.89501E+00_JPRB ,&
&  0.95837E+00_JPRB, 0.96940E+00_JPRB, 0.92943E+00_JPRB, 0.84360E+00_JPRB, 0.73330E+00_JPRB, 0.64869E+00_JPRB, 0.58470E+00_JPRB ,&
&  0.53198E+00_JPRB, 0.49796E+00_JPRB, 0.47879E+00_JPRB, 0.47349E+00_JPRB, 0.47750E+00_JPRB, 0.48717E+00_JPRB, 0.49319E+00_JPRB ,&
&  0.50478E+00_JPRB, 0.52072E+00_JPRB, 0.56419E+00_JPRB, 0.63667E+00_JPRB, 0.68976E+00_JPRB, 0.68871E+00_JPRB, 0.79508E+00_JPRB ,&
&  0.96170E+00_JPRB, 0.92155E+00_JPRB, 0.88095E+00_JPRB, 0.83870E+00_JPRB, 0.80197E+00_JPRB, 0.76478E+00_JPRB, 0.71053E+00_JPRB ,&
&  0.66798E+00_JPRB, 0.63185E+00_JPRB, 0.60597E+00_JPRB, 0.59190E+00_JPRB, 0.58408E+00_JPRB, 0.58199E+00_JPRB, 0.58070E+00_JPRB ,&
&  0.57994E+00_JPRB, 0.58033E+00_JPRB, 0.58550E+00_JPRB, 0.58704E+00_JPRB, 0.58269E+00_JPRB, 0.57066E+00_JPRB, 0.55470E+00_JPRB ,&
&  0.54281E+00_JPRB, 0.53479E+00_JPRB, 0.53193E+00_JPRB, 0.52551E+00_JPRB, 0.53288E+00_JPRB, 0.54583E+00_JPRB, 0.55298E+00_JPRB ,&
&  0.55262E+00_JPRB, 0.55848E+00_JPRB, 0.57236E+00_JPRB, 0.57479E+00_JPRB, 0.60020E+00_JPRB, 0.64180E+00_JPRB, 0.64315E+00_JPRB ,&
&  0.64395E+00_JPRB, 0.62294E+00_JPRB, 0.60067E+00_JPRB, 0.58551E+00_JPRB, 0.58467E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.57587E+00_JPRB, 0.55347E+00_JPRB, 0.53570E+00_JPRB, 0.51862E+00_JPRB, 0.50642E+00_JPRB, 0.49445E+00_JPRB, 0.48663E+00_JPRB ,&
&  0.48447E+00_JPRB, 0.48587E+00_JPRB, 0.48540E+00_JPRB, 0.47998E+00_JPRB, 0.47012E+00_JPRB, 0.46256E+00_JPRB, 0.45782E+00_JPRB ,&
&  0.45311E+00_JPRB, 0.45301E+00_JPRB, 0.45218E+00_JPRB, 0.44499E+00_JPRB, 0.43715E+00_JPRB, 0.43204E+00_JPRB, 0.42697E+00_JPRB ,&
&  0.42312E+00_JPRB, 0.42108E+00_JPRB, 0.41951E+00_JPRB, 0.41858E+00_JPRB, 0.41870E+00_JPRB, 0.42773E+00_JPRB, 0.45702E+00_JPRB ,&
&  0.51920E+00_JPRB, 0.45895E+00_JPRB, 0.80643E+00_JPRB, 0.90548E+00_JPRB, 0.94875E+00_JPRB, 0.10118E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.12985E+01_JPRB, 0.12679E+01_JPRB, 0.11240E+01_JPRB, 0.94386E+00_JPRB, 0.77772E+00_JPRB, 0.66241E+00_JPRB, 0.60127E+00_JPRB ,&
&  0.55867E+00_JPRB, 0.52948E+00_JPRB, 0.51664E+00_JPRB, 0.50968E+00_JPRB, 0.51304E+00_JPRB, 0.52147E+00_JPRB, 0.52946E+00_JPRB ,&
&  0.54617E+00_JPRB, 0.57103E+00_JPRB, 0.62322E+00_JPRB, 0.69292E+00_JPRB, 0.71407E+00_JPRB, 0.75897E+00_JPRB, 0.84420E+00_JPRB ,&
&  0.10068E+01_JPRB, 0.11078E+01_JPRB, 0.10180E+01_JPRB, 0.94842E+00_JPRB, 0.89545E+00_JPRB, 0.84473E+00_JPRB, 0.78696E+00_JPRB ,&
&  0.73783E+00_JPRB, 0.69221E+00_JPRB, 0.66166E+00_JPRB, 0.63895E+00_JPRB, 0.62543E+00_JPRB, 0.61921E+00_JPRB, 0.61551E+00_JPRB ,&
&  0.61237E+00_JPRB, 0.60993E+00_JPRB, 0.61028E+00_JPRB, 0.60834E+00_JPRB, 0.60365E+00_JPRB, 0.58964E+00_JPRB, 0.57098E+00_JPRB ,&
&  0.56079E+00_JPRB, 0.55829E+00_JPRB, 0.55276E+00_JPRB, 0.55616E+00_JPRB, 0.56641E+00_JPRB, 0.57152E+00_JPRB, 0.57964E+00_JPRB ,&
&  0.59619E+00_JPRB, 0.60370E+00_JPRB, 0.60862E+00_JPRB, 0.61875E+00_JPRB, 0.62472E+00_JPRB, 0.67052E+00_JPRB, 0.65363E+00_JPRB ,&
&  0.64647E+00_JPRB, 0.62918E+00_JPRB, 0.60642E+00_JPRB, 0.59485E+00_JPRB, 0.58914E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.57446E+00_JPRB, 0.55392E+00_JPRB, 0.53952E+00_JPRB, 0.52838E+00_JPRB, 0.51944E+00_JPRB, 0.51235E+00_JPRB, 0.51164E+00_JPRB ,&
&  0.51155E+00_JPRB, 0.50870E+00_JPRB, 0.49748E+00_JPRB, 0.48367E+00_JPRB, 0.47923E+00_JPRB, 0.47961E+00_JPRB, 0.47211E+00_JPRB ,&
&  0.47061E+00_JPRB, 0.47109E+00_JPRB, 0.46621E+00_JPRB, 0.45752E+00_JPRB, 0.45062E+00_JPRB, 0.44715E+00_JPRB, 0.44690E+00_JPRB ,&
&  0.44087E+00_JPRB, 0.43892E+00_JPRB, 0.44177E+00_JPRB, 0.44414E+00_JPRB, 0.44765E+00_JPRB, 0.46503E+00_JPRB, 0.47987E+00_JPRB ,&
&  0.52076E+00_JPRB, 0.66350E+00_JPRB, 0.70896E+00_JPRB, 0.99620E+00_JPRB, 0.11370E+01_JPRB, 0.12762E+01_JPRB, 0.13368E+01_JPRB ,&
&  0.13984E+01_JPRB, 0.14798E+01_JPRB, 0.12520E+01_JPRB, 0.10171E+01_JPRB, 0.80760E+00_JPRB, 0.69286E+00_JPRB, 0.63691E+00_JPRB ,&
&  0.60827E+00_JPRB, 0.58042E+00_JPRB, 0.56139E+00_JPRB, 0.55668E+00_JPRB, 0.56036E+00_JPRB, 0.56389E+00_JPRB, 0.56969E+00_JPRB ,&
&  0.58967E+00_JPRB, 0.62540E+00_JPRB, 0.69200E+00_JPRB, 0.75343E+00_JPRB, 0.78396E+00_JPRB, 0.90088E+00_JPRB, 0.10058E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.12604E+01_JPRB, 0.11731E+01_JPRB, 0.10706E+01_JPRB, 0.96715E+00_JPRB, 0.87673E+00_JPRB, 0.80262E+00_JPRB ,&
&  0.75133E+00_JPRB, 0.71048E+00_JPRB, 0.68075E+00_JPRB, 0.65919E+00_JPRB, 0.64482E+00_JPRB, 0.63611E+00_JPRB, 0.62521E+00_JPRB ,&
&  0.61230E+00_JPRB, 0.60765E+00_JPRB, 0.60980E+00_JPRB, 0.61128E+00_JPRB, 0.60719E+00_JPRB, 0.59856E+00_JPRB, 0.58875E+00_JPRB ,&
&  0.58434E+00_JPRB, 0.58668E+00_JPRB, 0.58320E+00_JPRB, 0.60087E+00_JPRB, 0.61016E+00_JPRB, 0.59713E+00_JPRB, 0.59641E+00_JPRB ,&
&  0.62688E+00_JPRB, 0.64158E+00_JPRB, 0.64355E+00_JPRB, 0.64430E+00_JPRB, 0.66861E+00_JPRB, 0.71961E+00_JPRB, 0.65720E+00_JPRB ,&
&  0.62828E+00_JPRB, 0.61739E+00_JPRB, 0.60310E+00_JPRB, 0.59438E+00_JPRB, 0.58656E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.57868E+00_JPRB, 0.56519E+00_JPRB, 0.55488E+00_JPRB, 0.54373E+00_JPRB, 0.53496E+00_JPRB, 0.53280E+00_JPRB, 0.53091E+00_JPRB ,&
&  0.52548E+00_JPRB, 0.50792E+00_JPRB, 0.48904E+00_JPRB, 0.48264E+00_JPRB, 0.48222E+00_JPRB, 0.47585E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.48229E+00_JPRB, 0.48543E+00_JPRB, 0.47735E+00_JPRB, 0.47193E+00_JPRB, 0.46871E+00_JPRB, 0.46706E+00_JPRB, 0.46828E+00_JPRB ,&
&  0.46947E+00_JPRB, 0.47067E+00_JPRB, 0.47604E+00_JPRB, 0.48177E+00_JPRB, 0.49031E+00_JPRB, 0.50863E+00_JPRB, 0.52582E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.67117E+00_JPRB, 0.74347E+00_JPRB, 0.12161E+01_JPRB, 0.14364E+01_JPRB, 0.14953E+01_JPRB, 0.13777E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.14676E+01_JPRB, 0.13234E+01_JPRB, 0.10772E+01_JPRB, 0.88193E+00_JPRB, 0.79176E+00_JPRB, 0.73296E+00_JPRB ,&
&  0.68365E+00_JPRB, 0.63768E+00_JPRB, 0.60958E+00_JPRB, 0.59304E+00_JPRB, 0.59589E+00_JPRB, 0.60243E+00_JPRB, 0.62070E+00_JPRB ,&
&  0.65085E+00_JPRB, 0.70558E+00_JPRB, 0.77349E+00_JPRB, 0.82253E+00_JPRB, 0.10081E+01_JPRB, 0.12041E+01_JPRB, 0.12644E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.13193E+01_JPRB, 0.11912E+01_JPRB, 0.10773E+01_JPRB, 0.94069E+00_JPRB, 0.87274E+00_JPRB, 0.79925E+00_JPRB ,&
&  0.74739E+00_JPRB, 0.70458E+00_JPRB, 0.67391E+00_JPRB, 0.65846E+00_JPRB, 0.64388E+00_JPRB, 0.63004E+00_JPRB, 0.61663E+00_JPRB ,&
&  0.60330E+00_JPRB, 0.60703E+00_JPRB, 0.61243E+00_JPRB, 0.60871E+00_JPRB, 0.60679E+00_JPRB, 0.60723E+00_JPRB, 0.61005E+00_JPRB ,&
&  0.61292E+00_JPRB, 0.61484E+00_JPRB, 0.61352E+00_JPRB, 0.63899E+00_JPRB, 0.65162E+00_JPRB, 0.63989E+00_JPRB, 0.64073E+00_JPRB ,&
&  0.66102E+00_JPRB, 0.67593E+00_JPRB, 0.67094E+00_JPRB, 0.65032E+00_JPRB, 0.66435E+00_JPRB, 0.67972E+00_JPRB, 0.64469E+00_JPRB ,&
&  0.63806E+00_JPRB, 0.62796E+00_JPRB, 0.60639E+00_JPRB, 0.59235E+00_JPRB, 0.58644E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.59172E+00_JPRB, 0.57721E+00_JPRB, 0.56117E+00_JPRB, 0.54955E+00_JPRB, 0.54293E+00_JPRB, 0.53651E+00_JPRB, 0.52430E+00_JPRB ,&
&  0.50731E+00_JPRB, 0.49789E+00_JPRB, 0.47969E+00_JPRB, 0.47719E+00_JPRB, 0.48456E+00_JPRB, 0.48982E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.50380E+00_JPRB, 0.50441E+00_JPRB, 0.49997E+00_JPRB, 0.49525E+00_JPRB, 0.49446E+00_JPRB, 0.49575E+00_JPRB, 0.50397E+00_JPRB ,&
&  0.50432E+00_JPRB, 0.50648E+00_JPRB, 0.51335E+00_JPRB, 0.52263E+00_JPRB, 0.54635E+00_JPRB, 0.58690E+00_JPRB, 0.64057E+00_JPRB ,&
&  0.76732E+00_JPRB, 0.80266E+00_JPRB, 0.86815E+00_JPRB, 0.12447E+01_JPRB, 0.14705E+01_JPRB, 0.13959E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.13043E+01_JPRB, 0.12392E+01_JPRB, 0.12571E+01_JPRB, 0.10948E+01_JPRB, 0.94301E+00_JPRB, 0.85843E+00_JPRB, 0.79254E+00_JPRB ,&
&  0.72844E+00_JPRB, 0.68110E+00_JPRB, 0.64607E+00_JPRB, 0.63175E+00_JPRB, 0.64301E+00_JPRB, 0.67367E+00_JPRB, 0.70080E+00_JPRB ,&
&  0.74898E+00_JPRB, 0.82333E+00_JPRB, 0.91811E+00_JPRB, 0.10599E+01_JPRB, 0.13247E+01_JPRB, 0.15024E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.13106E+01_JPRB, 0.12510E+01_JPRB, 0.11572E+01_JPRB, 0.10540E+01_JPRB, 0.87243E+00_JPRB, 0.86057E+00_JPRB, 0.79045E+00_JPRB ,&
&  0.73198E+00_JPRB, 0.69077E+00_JPRB, 0.66974E+00_JPRB, 0.65939E+00_JPRB, 0.64700E+00_JPRB, 0.62928E+00_JPRB, 0.61624E+00_JPRB ,&
&  0.60886E+00_JPRB, 0.61309E+00_JPRB, 0.61643E+00_JPRB, 0.61295E+00_JPRB, 0.61448E+00_JPRB, 0.62063E+00_JPRB, 0.62720E+00_JPRB ,&
&  0.63320E+00_JPRB, 0.64467E+00_JPRB, 0.65401E+00_JPRB, 0.68926E+00_JPRB, 0.70732E+00_JPRB, 0.70916E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.68296E+00_JPRB, 0.69170E+00_JPRB, 0.68879E+00_JPRB, 0.68476E+00_JPRB, 0.67931E+00_JPRB, 0.67742E+00_JPRB, 0.67151E+00_JPRB ,&
&  0.67114E+00_JPRB, 0.64989E+00_JPRB, 0.62346E+00_JPRB, 0.61006E+00_JPRB, 0.60099E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.59978E+00_JPRB, 0.58316E+00_JPRB, 0.56036E+00_JPRB, 0.53709E+00_JPRB, 0.52539E+00_JPRB, 0.51415E+00_JPRB, 0.50005E+00_JPRB ,&
&  0.49314E+00_JPRB, 0.48977E+00_JPRB, 0.48569E+00_JPRB, 0.49752E+00_JPRB, 0.51973E+00_JPRB, 0.53077E+00_JPRB, 0.53625E+00_JPRB ,&
&  0.54263E+00_JPRB, 0.54505E+00_JPRB, 0.53165E+00_JPRB, 0.52847E+00_JPRB, 0.53670E+00_JPRB, 0.54641E+00_JPRB, 0.55113E+00_JPRB ,&
&  0.55528E+00_JPRB, 0.56385E+00_JPRB, 0.57663E+00_JPRB, 0.60070E+00_JPRB, 0.65479E+00_JPRB, 0.73546E+00_JPRB, 0.84278E+00_JPRB ,&
&  0.97457E+00_JPRB, 0.94614E+00_JPRB, 0.57282E+00_JPRB, 0.93684E+00_JPRB, 0.12583E+01_JPRB, 0.12596E+01_JPRB, 0.11967E+01_JPRB ,&
&  0.11686E+01_JPRB, 0.11123E+01_JPRB, 0.11205E+01_JPRB, 0.10592E+01_JPRB, 0.96096E+00_JPRB, 0.87522E+00_JPRB, 0.80026E+00_JPRB ,&
&  0.73625E+00_JPRB, 0.69555E+00_JPRB, 0.68252E+00_JPRB, 0.70124E+00_JPRB, 0.73669E+00_JPRB, 0.75922E+00_JPRB, 0.79811E+00_JPRB ,&
&  0.87183E+00_JPRB, 0.96925E+00_JPRB, 0.11455E+01_JPRB, 0.13204E+01_JPRB, 0.15932E+01_JPRB, 0.15651E+01_JPRB, 0.13057E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.10873E+01_JPRB, 0.10854E+01_JPRB, 0.98010E+00_JPRB, 0.82552E+00_JPRB, 0.76535E+00_JPRB, 0.76210E+00_JPRB ,&
&  0.71154E+00_JPRB, 0.67767E+00_JPRB, 0.66382E+00_JPRB, 0.65136E+00_JPRB, 0.63462E+00_JPRB, 0.61979E+00_JPRB, 0.61048E+00_JPRB ,&
&  0.61279E+00_JPRB, 0.61478E+00_JPRB, 0.61488E+00_JPRB, 0.61445E+00_JPRB, 0.61888E+00_JPRB, 0.62567E+00_JPRB, 0.63674E+00_JPRB ,&
&  0.65623E+00_JPRB, 0.68118E+00_JPRB, 0.72084E+00_JPRB, 0.74027E+00_JPRB, 0.74572E+00_JPRB, 0.74560E+00_JPRB, 0.73606E+00_JPRB ,&
&  0.72790E+00_JPRB, 0.72462E+00_JPRB, 0.70827E+00_JPRB, 0.68993E+00_JPRB, 0.73107E+00_JPRB, 0.74415E+00_JPRB, 0.77906E+00_JPRB ,&
&  0.81269E+00_JPRB, 0.82135E+00_JPRB, 0.76917E+00_JPRB, 0.64058E+00_JPRB, 0.62201E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.56011E+00_JPRB, 0.54159E+00_JPRB, 0.51040E+00_JPRB, 0.50363E+00_JPRB, 0.50310E+00_JPRB, 0.50256E+00_JPRB, 0.51620E+00_JPRB ,&
&  0.52781E+00_JPRB, 0.54525E+00_JPRB, 0.55795E+00_JPRB, 0.57824E+00_JPRB, 0.58963E+00_JPRB, 0.58966E+00_JPRB, 0.59595E+00_JPRB ,&
&  0.59705E+00_JPRB, 0.59712E+00_JPRB, 0.59864E+00_JPRB, 0.60636E+00_JPRB, 0.61785E+00_JPRB, 0.62285E+00_JPRB, 0.62960E+00_JPRB ,&
&  0.64514E+00_JPRB, 0.66308E+00_JPRB, 0.68863E+00_JPRB, 0.74847E+00_JPRB, 0.85003E+00_JPRB, 0.97283E+00_JPRB, 0.10950E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.85521E+00_JPRB, 0.66756E+00_JPRB, 0.11150E+01_JPRB, 0.12439E+01_JPRB, 0.12186E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.10804E+01_JPRB, 0.10584E+01_JPRB, 0.10453E+01_JPRB, 0.10212E+01_JPRB, 0.90599E+00_JPRB, 0.83857E+00_JPRB, 0.78611E+00_JPRB ,&
&  0.75916E+00_JPRB, 0.74906E+00_JPRB, 0.76329E+00_JPRB, 0.79629E+00_JPRB, 0.82873E+00_JPRB, 0.86386E+00_JPRB, 0.94037E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.11732E+01_JPRB, 0.13948E+01_JPRB, 0.14570E+01_JPRB, 0.15915E+01_JPRB, 0.13861E+01_JPRB, 0.11722E+01_JPRB ,&
&  0.97937E+00_JPRB, 0.96499E+00_JPRB, 0.99076E+00_JPRB, 0.91645E+00_JPRB, 0.79457E+00_JPRB, 0.72454E+00_JPRB, 0.72840E+00_JPRB ,&
&  0.67825E+00_JPRB, 0.65588E+00_JPRB, 0.63891E+00_JPRB, 0.61955E+00_JPRB, 0.61306E+00_JPRB, 0.61435E+00_JPRB, 0.61718E+00_JPRB ,&
&  0.61955E+00_JPRB, 0.61592E+00_JPRB, 0.61394E+00_JPRB, 0.62014E+00_JPRB, 0.63160E+00_JPRB, 0.63985E+00_JPRB, 0.65662E+00_JPRB ,&
&  0.67963E+00_JPRB, 0.70980E+00_JPRB, 0.74546E+00_JPRB, 0.77100E+00_JPRB, 0.76877E+00_JPRB, 0.74599E+00_JPRB, 0.75295E+00_JPRB ,&
&  0.75375E+00_JPRB, 0.77320E+00_JPRB, 0.77632E+00_JPRB, 0.78991E+00_JPRB, 0.79674E+00_JPRB, 0.81731E+00_JPRB, 0.82455E+00_JPRB ,&
&  0.82412E+00_JPRB, 0.79959E+00_JPRB, 0.71684E+00_JPRB, 0.60994E+00_JPRB, 0.58587E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.50459E+00_JPRB, 0.49946E+00_JPRB, 0.50254E+00_JPRB, 0.51007E+00_JPRB, 0.52204E+00_JPRB, 0.54622E+00_JPRB, 0.57343E+00_JPRB ,&
&  0.59589E+00_JPRB, 0.63355E+00_JPRB, 0.65917E+00_JPRB, 0.66086E+00_JPRB, 0.65531E+00_JPRB, 0.65696E+00_JPRB, 0.66256E+00_JPRB ,&
&  0.67365E+00_JPRB, 0.68208E+00_JPRB, 0.68795E+00_JPRB, 0.69535E+00_JPRB, 0.70622E+00_JPRB, 0.71808E+00_JPRB, 0.73394E+00_JPRB ,&
&  0.75954E+00_JPRB, 0.80249E+00_JPRB, 0.86408E+00_JPRB, 0.95433E+00_JPRB, 0.10731E+01_JPRB, 0.11969E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.95018E+00_JPRB, 0.55973E+00_JPRB, 0.10760E+01_JPRB, 0.12585E+01_JPRB, 0.11556E+01_JPRB, 0.11117E+01_JPRB, 0.10701E+01_JPRB ,&
&  0.10120E+01_JPRB, 0.10102E+01_JPRB, 0.97940E+00_JPRB, 0.98356E+00_JPRB, 0.88261E+00_JPRB, 0.83883E+00_JPRB, 0.82216E+00_JPRB ,&
&  0.82397E+00_JPRB, 0.83268E+00_JPRB, 0.85161E+00_JPRB, 0.87701E+00_JPRB, 0.92030E+00_JPRB, 0.98446E+00_JPRB, 0.10843E+01_JPRB ,&
&  0.11973E+01_JPRB, 0.13318E+01_JPRB, 0.15272E+01_JPRB, 0.14135E+01_JPRB, 0.14319E+01_JPRB, 0.13145E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.97938E+00_JPRB, 0.85468E+00_JPRB, 0.85223E+00_JPRB, 0.86474E+00_JPRB, 0.78230E+00_JPRB, 0.68888E+00_JPRB, 0.69572E+00_JPRB ,&
&  0.65234E+00_JPRB, 0.63748E+00_JPRB, 0.61704E+00_JPRB, 0.59735E+00_JPRB, 0.59042E+00_JPRB, 0.59653E+00_JPRB, 0.60709E+00_JPRB ,&
&  0.61109E+00_JPRB, 0.61270E+00_JPRB, 0.61863E+00_JPRB, 0.62573E+00_JPRB, 0.64207E+00_JPRB, 0.65694E+00_JPRB, 0.66902E+00_JPRB ,&
&  0.70018E+00_JPRB, 0.73184E+00_JPRB, 0.76313E+00_JPRB, 0.79149E+00_JPRB, 0.81220E+00_JPRB, 0.79482E+00_JPRB, 0.78459E+00_JPRB ,&
&  0.79495E+00_JPRB, 0.80629E+00_JPRB, 0.82043E+00_JPRB, 0.84165E+00_JPRB, 0.84428E+00_JPRB, 0.81262E+00_JPRB, 0.80230E+00_JPRB ,&
&  0.74394E+00_JPRB, 0.67370E+00_JPRB, 0.59131E+00_JPRB, 0.54241E+00_JPRB, 0.52477E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.52921E+00_JPRB, 0.54299E+00_JPRB, 0.55917E+00_JPRB, 0.57940E+00_JPRB, 0.60847E+00_JPRB, 0.64161E+00_JPRB, 0.68378E+00_JPRB ,&
&  0.72636E+00_JPRB, 0.73733E+00_JPRB, 0.73910E+00_JPRB, 0.73988E+00_JPRB, 0.73984E+00_JPRB, 0.74470E+00_JPRB, 0.75338E+00_JPRB ,&
&  0.76264E+00_JPRB, 0.77275E+00_JPRB, 0.78334E+00_JPRB, 0.79453E+00_JPRB, 0.80916E+00_JPRB, 0.83109E+00_JPRB, 0.86164E+00_JPRB ,&
&  0.89447E+00_JPRB, 0.95463E+00_JPRB, 0.10359E+01_JPRB, 0.11269E+01_JPRB, 0.12207E+01_JPRB, 0.12916E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.81404E+00_JPRB, 0.10574E+01_JPRB, 0.12388E+01_JPRB, 0.11805E+01_JPRB, 0.10987E+01_JPRB, 0.10467E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.98987E+00_JPRB, 0.93147E+00_JPRB, 0.91841E+00_JPRB, 0.97194E+00_JPRB, 0.89959E+00_JPRB, 0.86888E+00_JPRB, 0.87184E+00_JPRB ,&
&  0.88677E+00_JPRB, 0.90787E+00_JPRB, 0.93991E+00_JPRB, 0.97511E+00_JPRB, 0.10296E+01_JPRB, 0.11096E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.14252E+01_JPRB, 0.15405E+01_JPRB, 0.13545E+01_JPRB, 0.13138E+01_JPRB, 0.13112E+01_JPRB, 0.12065E+01_JPRB ,&
&  0.92634E+00_JPRB, 0.79282E+00_JPRB, 0.81053E+00_JPRB, 0.82767E+00_JPRB, 0.76070E+00_JPRB, 0.70698E+00_JPRB, 0.66312E+00_JPRB ,&
&  0.63448E+00_JPRB, 0.60889E+00_JPRB, 0.58306E+00_JPRB, 0.57400E+00_JPRB, 0.57100E+00_JPRB, 0.57982E+00_JPRB, 0.59494E+00_JPRB ,&
&  0.60474E+00_JPRB, 0.60784E+00_JPRB, 0.61705E+00_JPRB, 0.62923E+00_JPRB, 0.65894E+00_JPRB, 0.67702E+00_JPRB, 0.68768E+00_JPRB ,&
&  0.71497E+00_JPRB, 0.73349E+00_JPRB, 0.75976E+00_JPRB, 0.78776E+00_JPRB, 0.81265E+00_JPRB, 0.82932E+00_JPRB, 0.82735E+00_JPRB ,&
&  0.82928E+00_JPRB, 0.85550E+00_JPRB, 0.88565E+00_JPRB, 0.89286E+00_JPRB, 0.86974E+00_JPRB, 0.79133E+00_JPRB, 0.71822E+00_JPRB ,&
&  0.63700E+00_JPRB, 0.61407E+00_JPRB, 0.57917E+00_JPRB, 0.55404E+00_JPRB, 0.53642E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.62336E+00_JPRB, 0.63839E+00_JPRB, 0.70443E+00_JPRB, 0.72298E+00_JPRB, 0.78555E+00_JPRB, 0.80840E+00_JPRB, 0.83022E+00_JPRB ,&
&  0.82981E+00_JPRB, 0.82934E+00_JPRB, 0.82741E+00_JPRB, 0.82866E+00_JPRB, 0.82936E+00_JPRB, 0.83524E+00_JPRB, 0.84043E+00_JPRB ,&
&  0.85053E+00_JPRB, 0.86533E+00_JPRB, 0.88321E+00_JPRB, 0.90145E+00_JPRB, 0.92653E+00_JPRB, 0.95609E+00_JPRB, 0.99220E+00_JPRB ,&
&  0.10354E+01_JPRB, 0.10881E+01_JPRB, 0.11466E+01_JPRB, 0.12170E+01_JPRB, 0.12902E+01_JPRB, 0.13405E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.12284E+01_JPRB, 0.12091E+01_JPRB, 0.11509E+01_JPRB, 0.10686E+01_JPRB, 0.10058E+01_JPRB, 0.97972E+00_JPRB ,&
&  0.95853E+00_JPRB, 0.90375E+00_JPRB, 0.89937E+00_JPRB, 0.92513E+00_JPRB, 0.92324E+00_JPRB, 0.89587E+00_JPRB, 0.91725E+00_JPRB ,&
&  0.94879E+00_JPRB, 0.98819E+00_JPRB, 0.10253E+01_JPRB, 0.10560E+01_JPRB, 0.11159E+01_JPRB, 0.11911E+01_JPRB, 0.12752E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.14493E+01_JPRB, 0.15137E+01_JPRB, 0.13331E+01_JPRB, 0.11785E+01_JPRB, 0.11102E+01_JPRB, 0.90817E+00_JPRB ,&
&  0.79318E+00_JPRB, 0.73116E+00_JPRB, 0.78679E+00_JPRB, 0.82034E+00_JPRB, 0.75522E+00_JPRB, 0.68098E+00_JPRB, 0.63540E+00_JPRB ,&
&  0.59770E+00_JPRB, 0.57111E+00_JPRB, 0.56256E+00_JPRB, 0.55197E+00_JPRB, 0.54749E+00_JPRB, 0.56245E+00_JPRB, 0.57694E+00_JPRB ,&
&  0.60029E+00_JPRB, 0.62899E+00_JPRB, 0.65402E+00_JPRB, 0.68343E+00_JPRB, 0.70201E+00_JPRB, 0.72426E+00_JPRB, 0.73646E+00_JPRB ,&
&  0.75525E+00_JPRB, 0.78963E+00_JPRB, 0.79604E+00_JPRB, 0.82371E+00_JPRB, 0.83039E+00_JPRB, 0.85692E+00_JPRB, 0.88346E+00_JPRB ,&
&  0.91169E+00_JPRB, 0.93748E+00_JPRB, 0.94298E+00_JPRB, 0.92498E+00_JPRB, 0.87251E+00_JPRB, 0.79727E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.61154E+00_JPRB, 0.59292E+00_JPRB, 0.59663E+00_JPRB, 0.59976E+00_JPRB, 0.61621E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.78165E+00_JPRB, 0.82908E+00_JPRB, 0.87614E+00_JPRB, 0.93332E+00_JPRB, 0.97749E+00_JPRB, 0.96119E+00_JPRB, 0.93078E+00_JPRB ,&
&  0.91118E+00_JPRB, 0.90166E+00_JPRB, 0.90632E+00_JPRB, 0.91833E+00_JPRB, 0.92713E+00_JPRB, 0.93045E+00_JPRB, 0.93897E+00_JPRB ,&
&  0.95344E+00_JPRB, 0.97386E+00_JPRB, 0.99548E+00_JPRB, 0.10199E+01_JPRB, 0.10384E+01_JPRB, 0.10649E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11700E+01_JPRB, 0.12060E+01_JPRB, 0.12442E+01_JPRB, 0.12797E+01_JPRB, 0.12697E+01_JPRB, 0.94958E+00_JPRB ,&
&  0.11893E+01_JPRB, 0.12330E+01_JPRB, 0.12462E+01_JPRB, 0.11821E+01_JPRB, 0.10765E+01_JPRB, 0.10043E+01_JPRB, 0.97513E+00_JPRB ,&
&  0.95042E+00_JPRB, 0.92213E+00_JPRB, 0.89774E+00_JPRB, 0.88722E+00_JPRB, 0.96290E+00_JPRB, 0.93320E+00_JPRB, 0.96186E+00_JPRB ,&
&  0.99812E+00_JPRB, 0.10257E+01_JPRB, 0.10480E+01_JPRB, 0.10836E+01_JPRB, 0.11538E+01_JPRB, 0.12196E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.13736E+01_JPRB, 0.14371E+01_JPRB, 0.14797E+01_JPRB, 0.12909E+01_JPRB, 0.12304E+01_JPRB, 0.11483E+01_JPRB, 0.89771E+00_JPRB ,&
&  0.76087E+00_JPRB, 0.73707E+00_JPRB, 0.79995E+00_JPRB, 0.82661E+00_JPRB, 0.75403E+00_JPRB, 0.66394E+00_JPRB, 0.62562E+00_JPRB ,&
&  0.58373E+00_JPRB, 0.56311E+00_JPRB, 0.55800E+00_JPRB, 0.53951E+00_JPRB, 0.53161E+00_JPRB, 0.55365E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.62562E+00_JPRB, 0.67557E+00_JPRB, 0.71507E+00_JPRB, 0.74829E+00_JPRB, 0.79286E+00_JPRB, 0.86858E+00_JPRB, 0.91139E+00_JPRB ,&
&  0.83831E+00_JPRB, 0.81072E+00_JPRB, 0.86949E+00_JPRB, 0.91725E+00_JPRB, 0.96855E+00_JPRB, 0.10473E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.13275E+01_JPRB, 0.11292E+01_JPRB, 0.95045E+00_JPRB, 0.12750E+01_JPRB, 0.13561E+01_JPRB, 0.68635E+00_JPRB ,&
&  0.65370E+00_JPRB, 0.67070E+00_JPRB, 0.68006E+00_JPRB, 0.71161E+00_JPRB, 0.74493E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.91788E+00_JPRB, 0.94375E+00_JPRB, 0.97217E+00_JPRB, 0.98770E+00_JPRB, 0.99371E+00_JPRB, 0.97659E+00_JPRB, 0.96165E+00_JPRB ,&
&  0.95937E+00_JPRB, 0.96514E+00_JPRB, 0.97789E+00_JPRB, 0.99208E+00_JPRB, 0.99940E+00_JPRB, 0.10054E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10660E+01_JPRB, 0.10980E+01_JPRB, 0.11182E+01_JPRB, 0.11379E+01_JPRB, 0.11512E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11908E+01_JPRB, 0.12268E+01_JPRB, 0.12292E+01_JPRB, 0.12494E+01_JPRB, 0.12573E+01_JPRB, 0.12814E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.12074E+01_JPRB, 0.12663E+01_JPRB, 0.12472E+01_JPRB, 0.11762E+01_JPRB, 0.10726E+01_JPRB, 0.10216E+01_JPRB, 0.98807E+00_JPRB ,&
&  0.97436E+00_JPRB, 0.95212E+00_JPRB, 0.91634E+00_JPRB, 0.96250E+00_JPRB, 0.99132E+00_JPRB, 0.95335E+00_JPRB, 0.97019E+00_JPRB ,&
&  0.99205E+00_JPRB, 0.10142E+01_JPRB, 0.10425E+01_JPRB, 0.10926E+01_JPRB, 0.11651E+01_JPRB, 0.12279E+01_JPRB, 0.12941E+01_JPRB ,&
&  0.13630E+01_JPRB, 0.14391E+01_JPRB, 0.15038E+01_JPRB, 0.13599E+01_JPRB, 0.13661E+01_JPRB, 0.12776E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.76780E+00_JPRB, 0.70927E+00_JPRB, 0.75411E+00_JPRB, 0.83904E+00_JPRB, 0.76371E+00_JPRB, 0.66198E+00_JPRB, 0.64002E+00_JPRB ,&
&  0.60331E+00_JPRB, 0.57516E+00_JPRB, 0.56709E+00_JPRB, 0.54399E+00_JPRB, 0.53133E+00_JPRB, 0.55843E+00_JPRB, 0.58559E+00_JPRB ,&
&  0.62195E+00_JPRB, 0.67013E+00_JPRB, 0.72288E+00_JPRB, 0.77507E+00_JPRB, 0.83464E+00_JPRB, 0.85201E+00_JPRB, 0.80294E+00_JPRB ,&
&  0.84689E+00_JPRB, 0.89140E+00_JPRB, 0.95588E+00_JPRB, 0.10678E+01_JPRB, 0.11497E+01_JPRB, 0.13707E+01_JPRB, 0.16102E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.17255E+01_JPRB, 0.20564E+01_JPRB, 0.28635E+01_JPRB, 0.26392E+01_JPRB, 0.21223E+01_JPRB, 0.79014E+00_JPRB ,&
&  0.77937E+00_JPRB, 0.77742E+00_JPRB, 0.80849E+00_JPRB, 0.84763E+00_JPRB, 0.89339E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.91340E+00_JPRB, 0.93453E+00_JPRB, 0.95500E+00_JPRB, 0.97408E+00_JPRB, 0.98761E+00_JPRB, 0.99694E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10335E+01_JPRB, 0.10454E+01_JPRB, 0.10516E+01_JPRB, 0.10571E+01_JPRB, 0.10732E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.11392E+01_JPRB, 0.11711E+01_JPRB, 0.11499E+01_JPRB, 0.11278E+01_JPRB, 0.11289E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11490E+01_JPRB, 0.11367E+01_JPRB, 0.11278E+01_JPRB, 0.11513E+01_JPRB, 0.11790E+01_JPRB, 0.12117E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.12504E+01_JPRB, 0.12128E+01_JPRB, 0.11442E+01_JPRB, 0.10343E+01_JPRB, 0.10002E+01_JPRB, 0.98740E+00_JPRB ,&
&  0.97677E+00_JPRB, 0.96948E+00_JPRB, 0.95638E+00_JPRB, 0.94745E+00_JPRB, 0.94962E+00_JPRB, 0.94617E+00_JPRB, 0.95600E+00_JPRB ,&
&  0.97144E+00_JPRB, 0.98579E+00_JPRB, 0.10170E+01_JPRB, 0.10587E+01_JPRB, 0.11243E+01_JPRB, 0.11902E+01_JPRB, 0.12327E+01_JPRB ,&
&  0.12854E+01_JPRB, 0.13383E+01_JPRB, 0.12688E+01_JPRB, 0.12756E+01_JPRB, 0.13569E+01_JPRB, 0.12643E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.75477E+00_JPRB, 0.72295E+00_JPRB, 0.75146E+00_JPRB, 0.84007E+00_JPRB, 0.77000E+00_JPRB, 0.69715E+00_JPRB, 0.66652E+00_JPRB ,&
&  0.60260E+00_JPRB, 0.54097E+00_JPRB, 0.55120E+00_JPRB, 0.54591E+00_JPRB, 0.53527E+00_JPRB, 0.55406E+00_JPRB, 0.57413E+00_JPRB ,&
&  0.59563E+00_JPRB, 0.63796E+00_JPRB, 0.67947E+00_JPRB, 0.72004E+00_JPRB, 0.73944E+00_JPRB, 0.70204E+00_JPRB, 0.79352E+00_JPRB ,&
&  0.79202E+00_JPRB, 0.78465E+00_JPRB, 0.82644E+00_JPRB, 0.71429E+00_JPRB, 0.10061E+01_JPRB, 0.10756E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.15798E+01_JPRB, 0.19593E+01_JPRB, 0.15203E+01_JPRB, 0.11834E+01_JPRB, 0.85003E+00_JPRB, 0.79266E+00_JPRB ,&
&  0.79514E+00_JPRB, 0.80703E+00_JPRB, 0.83121E+00_JPRB, 0.86155E+00_JPRB, 0.88868E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.91192E+00_JPRB, 0.93156E+00_JPRB, 0.95065E+00_JPRB, 0.97170E+00_JPRB, 0.99265E+00_JPRB, 0.10072E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10605E+01_JPRB, 0.10686E+01_JPRB, 0.10788E+01_JPRB, 0.10945E+01_JPRB, 0.11270E+01_JPRB, 0.11561E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11546E+01_JPRB, 0.11624E+01_JPRB, 0.11645E+01_JPRB, 0.11524E+01_JPRB, 0.11234E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11187E+01_JPRB, 0.11158E+01_JPRB, 0.11348E+01_JPRB, 0.11672E+01_JPRB, 0.12342E+01_JPRB, 0.10033E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.11581E+01_JPRB, 0.11628E+01_JPRB, 0.10640E+01_JPRB, 0.95102E+00_JPRB, 0.91743E+00_JPRB, 0.90882E+00_JPRB ,&
&  0.91420E+00_JPRB, 0.89627E+00_JPRB, 0.89547E+00_JPRB, 0.89965E+00_JPRB, 0.90924E+00_JPRB, 0.93020E+00_JPRB, 0.93375E+00_JPRB ,&
&  0.94532E+00_JPRB, 0.96293E+00_JPRB, 0.98633E+00_JPRB, 0.10197E+01_JPRB, 0.10596E+01_JPRB, 0.10966E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.11550E+01_JPRB, 0.10593E+01_JPRB, 0.10646E+01_JPRB, 0.11182E+01_JPRB, 0.10439E+01_JPRB, 0.89138E+00_JPRB ,&
&  0.78088E+00_JPRB, 0.70089E+00_JPRB, 0.67257E+00_JPRB, 0.80506E+00_JPRB, 0.77685E+00_JPRB, 0.72178E+00_JPRB, 0.65844E+00_JPRB ,&
&  0.56900E+00_JPRB, 0.51265E+00_JPRB, 0.51172E+00_JPRB, 0.54556E+00_JPRB, 0.53679E+00_JPRB, 0.53358E+00_JPRB, 0.53735E+00_JPRB ,&
&  0.54462E+00_JPRB, 0.54994E+00_JPRB, 0.56645E+00_JPRB, 0.60612E+00_JPRB, 0.63908E+00_JPRB, 0.59004E+00_JPRB, 0.72625E+00_JPRB ,&
&  0.67345E+00_JPRB, 0.55105E+00_JPRB, 0.61991E+00_JPRB, 0.76033E+00_JPRB, 0.79424E+00_JPRB, 0.81044E+00_JPRB, 0.91493E+00_JPRB ,&
&  0.10601E+01_JPRB, 0.11879E+01_JPRB, 0.11926E+01_JPRB, 0.96900E+00_JPRB, 0.84751E+00_JPRB, 0.79383E+00_JPRB, 0.78332E+00_JPRB ,&
&  0.78908E+00_JPRB, 0.80484E+00_JPRB, 0.83166E+00_JPRB, 0.86285E+00_JPRB, 0.88999E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.89853E+00_JPRB, 0.92019E+00_JPRB, 0.93724E+00_JPRB, 0.95058E+00_JPRB, 0.96338E+00_JPRB, 0.97916E+00_JPRB, 0.99408E+00_JPRB ,&
&  0.10090E+01_JPRB, 0.10210E+01_JPRB, 0.10291E+01_JPRB, 0.10442E+01_JPRB, 0.10609E+01_JPRB, 0.10951E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11434E+01_JPRB, 0.11541E+01_JPRB, 0.11520E+01_JPRB, 0.11654E+01_JPRB, 0.11581E+01_JPRB, 0.11381E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.11284E+01_JPRB, 0.11253E+01_JPRB, 0.11261E+01_JPRB, 0.11285E+01_JPRB, 0.11020E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.80609E+00_JPRB, 0.10512E+01_JPRB, 0.11180E+01_JPRB, 0.99751E+00_JPRB, 0.93444E+00_JPRB, 0.85097E+00_JPRB, 0.83936E+00_JPRB ,&
&  0.84784E+00_JPRB, 0.84454E+00_JPRB, 0.85405E+00_JPRB, 0.87462E+00_JPRB, 0.89139E+00_JPRB, 0.90955E+00_JPRB, 0.91200E+00_JPRB ,&
&  0.93051E+00_JPRB, 0.93269E+00_JPRB, 0.94274E+00_JPRB, 0.95153E+00_JPRB, 0.98139E+00_JPRB, 0.98325E+00_JPRB, 0.97889E+00_JPRB ,&
&  0.98538E+00_JPRB, 0.98925E+00_JPRB, 0.78410E+00_JPRB, 0.76108E+00_JPRB, 0.81327E+00_JPRB, 0.79341E+00_JPRB, 0.71274E+00_JPRB ,&
&  0.67498E+00_JPRB, 0.66671E+00_JPRB, 0.72294E+00_JPRB, 0.73317E+00_JPRB, 0.74481E+00_JPRB, 0.72864E+00_JPRB, 0.70236E+00_JPRB ,&
&  0.62213E+00_JPRB, 0.53580E+00_JPRB, 0.51004E+00_JPRB, 0.53033E+00_JPRB, 0.54242E+00_JPRB, 0.54525E+00_JPRB, 0.51051E+00_JPRB ,&
&  0.47978E+00_JPRB, 0.45687E+00_JPRB, 0.45725E+00_JPRB, 0.48554E+00_JPRB, 0.45840E+00_JPRB, 0.71257E+00_JPRB, 0.72529E+00_JPRB ,&
&  0.64868E+00_JPRB, 0.56823E+00_JPRB, 0.47389E+00_JPRB, 0.64165E+00_JPRB, 0.64766E+00_JPRB, 0.61158E+00_JPRB, 0.65035E+00_JPRB ,&
&  0.75101E+00_JPRB, 0.80065E+00_JPRB, 0.81681E+00_JPRB, 0.74649E+00_JPRB, 0.75626E+00_JPRB, 0.73536E+00_JPRB, 0.73581E+00_JPRB ,&
&  0.75125E+00_JPRB, 0.77838E+00_JPRB, 0.81196E+00_JPRB, 0.84449E+00_JPRB, 0.87276E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.85166E+00_JPRB, 0.86939E+00_JPRB, 0.88050E+00_JPRB, 0.89175E+00_JPRB, 0.90819E+00_JPRB, 0.92515E+00_JPRB, 0.93298E+00_JPRB ,&
&  0.95021E+00_JPRB, 0.96599E+00_JPRB, 0.97308E+00_JPRB, 0.99199E+00_JPRB, 0.10004E+01_JPRB, 0.10514E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.11289E+01_JPRB, 0.11343E+01_JPRB, 0.11359E+01_JPRB, 0.11484E+01_JPRB, 0.11476E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.12029E+01_JPRB, 0.12542E+01_JPRB, 0.13042E+01_JPRB, 0.13352E+01_JPRB, 0.14315E+01_JPRB, 0.15589E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.89042E+00_JPRB, 0.96083E+00_JPRB, 0.84488E+00_JPRB, 0.83847E+00_JPRB, 0.83822E+00_JPRB, 0.80575E+00_JPRB ,&
&  0.79176E+00_JPRB, 0.81934E+00_JPRB, 0.84948E+00_JPRB, 0.86561E+00_JPRB, 0.86919E+00_JPRB, 0.87915E+00_JPRB, 0.87068E+00_JPRB ,&
&  0.87594E+00_JPRB, 0.87900E+00_JPRB, 0.89849E+00_JPRB, 0.92654E+00_JPRB, 0.86428E+00_JPRB, 0.93022E+00_JPRB, 0.94734E+00_JPRB ,&
&  0.95607E+00_JPRB, 0.87657E+00_JPRB, 0.65295E+00_JPRB, 0.61467E+00_JPRB, 0.63822E+00_JPRB, 0.61876E+00_JPRB, 0.58225E+00_JPRB ,&
&  0.60929E+00_JPRB, 0.60361E+00_JPRB, 0.52888E+00_JPRB, 0.64226E+00_JPRB, 0.71318E+00_JPRB, 0.74549E+00_JPRB, 0.72670E+00_JPRB ,&
&  0.68772E+00_JPRB, 0.61129E+00_JPRB, 0.58255E+00_JPRB, 0.56995E+00_JPRB, 0.56245E+00_JPRB, 0.54142E+00_JPRB, 0.53226E+00_JPRB ,&
&  0.49845E+00_JPRB, 0.45618E+00_JPRB, 0.44187E+00_JPRB, 0.43394E+00_JPRB, 0.52527E+00_JPRB, 0.61356E+00_JPRB, 0.70854E+00_JPRB ,&
&  0.53142E+00_JPRB, 0.53639E+00_JPRB, 0.47830E+00_JPRB, 0.61150E+00_JPRB, 0.53025E+00_JPRB, 0.47050E+00_JPRB, 0.52349E+00_JPRB ,&
&  0.58059E+00_JPRB, 0.60957E+00_JPRB, 0.61954E+00_JPRB, 0.63444E+00_JPRB, 0.67639E+00_JPRB, 0.69754E+00_JPRB, 0.71545E+00_JPRB ,&
&  0.71566E+00_JPRB, 0.74457E+00_JPRB, 0.78476E+00_JPRB, 0.81486E+00_JPRB, 0.82607E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.73024E+00_JPRB, 0.74347E+00_JPRB, 0.76100E+00_JPRB, 0.77979E+00_JPRB, 0.79012E+00_JPRB, 0.80755E+00_JPRB, 0.82400E+00_JPRB ,&
&  0.84638E+00_JPRB, 0.87558E+00_JPRB, 0.89272E+00_JPRB, 0.89924E+00_JPRB, 0.91641E+00_JPRB, 0.93744E+00_JPRB, 0.94884E+00_JPRB ,&
&  0.97180E+00_JPRB, 0.98122E+00_JPRB, 0.10325E+01_JPRB, 0.10579E+01_JPRB, 0.10946E+01_JPRB, 0.11453E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.11349E+01_JPRB, 0.11475E+01_JPRB, 0.11452E+01_JPRB, 0.12187E+01_JPRB, 0.12169E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.86199E+00_JPRB, 0.75105E+00_JPRB, 0.92806E+00_JPRB, 0.81907E+00_JPRB, 0.81303E+00_JPRB, 0.79609E+00_JPRB, 0.72946E+00_JPRB ,&
&  0.73138E+00_JPRB, 0.76602E+00_JPRB, 0.80597E+00_JPRB, 0.83792E+00_JPRB, 0.87781E+00_JPRB, 0.88621E+00_JPRB, 0.87617E+00_JPRB ,&
&  0.87714E+00_JPRB, 0.88647E+00_JPRB, 0.88888E+00_JPRB, 0.81168E+00_JPRB, 0.72795E+00_JPRB, 0.76986E+00_JPRB, 0.78578E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.76735E+00_JPRB, 0.62098E+00_JPRB, 0.56976E+00_JPRB, 0.58252E+00_JPRB, 0.52689E+00_JPRB, 0.50791E+00_JPRB ,&
&  0.55670E+00_JPRB, 0.49759E+00_JPRB, 0.34073E+00_JPRB, 0.47654E+00_JPRB, 0.61672E+00_JPRB, 0.71281E+00_JPRB, 0.75504E+00_JPRB ,&
&  0.73994E+00_JPRB, 0.67534E+00_JPRB, 0.64334E+00_JPRB, 0.64440E+00_JPRB, 0.64679E+00_JPRB, 0.59599E+00_JPRB, 0.56485E+00_JPRB ,&
&  0.50589E+00_JPRB, 0.45036E+00_JPRB, 0.41150E+00_JPRB, 0.41781E+00_JPRB, 0.42851E+00_JPRB, 0.57834E+00_JPRB, 0.58953E+00_JPRB ,&
&  0.53130E+00_JPRB, 0.47266E+00_JPRB, 0.56250E+00_JPRB, 0.56872E+00_JPRB, 0.38542E+00_JPRB, 0.43674E+00_JPRB, 0.46512E+00_JPRB ,&
&  0.49212E+00_JPRB, 0.51027E+00_JPRB, 0.52460E+00_JPRB, 0.54870E+00_JPRB, 0.58271E+00_JPRB, 0.59500E+00_JPRB, 0.59925E+00_JPRB ,&
&  0.61573E+00_JPRB, 0.64312E+00_JPRB, 0.66327E+00_JPRB, 0.68811E+00_JPRB, 0.71034E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.56604E+00_JPRB, 0.56527E+00_JPRB, 0.56839E+00_JPRB, 0.57487E+00_JPRB, 0.57538E+00_JPRB, 0.57339E+00_JPRB, 0.58501E+00_JPRB ,&
&  0.60650E+00_JPRB, 0.64736E+00_JPRB, 0.70643E+00_JPRB, 0.74004E+00_JPRB, 0.72379E+00_JPRB, 0.67903E+00_JPRB, 0.66434E+00_JPRB ,&
&  0.66464E+00_JPRB, 0.66312E+00_JPRB, 0.69189E+00_JPRB, 0.74016E+00_JPRB, 0.80339E+00_JPRB, 0.82746E+00_JPRB, 0.88703E+00_JPRB ,&
&  0.88789E+00_JPRB, 0.90199E+00_JPRB, 0.98026E+00_JPRB, 0.10450E+01_JPRB, 0.15780E+01_JPRB, 0.14722E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.86826E+00_JPRB, 0.86978E+00_JPRB, 0.94118E+00_JPRB, 0.91931E+00_JPRB, 0.93713E+00_JPRB, 0.90112E+00_JPRB, 0.91838E+00_JPRB ,&
&  0.90618E+00_JPRB, 0.95067E+00_JPRB, 0.98339E+00_JPRB, 0.10262E+01_JPRB, 0.10551E+01_JPRB, 0.10584E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10783E+01_JPRB, 0.10761E+01_JPRB, 0.10563E+01_JPRB, 0.88911E+00_JPRB, 0.88728E+00_JPRB, 0.87303E+00_JPRB, 0.83866E+00_JPRB ,&
&  0.79627E+00_JPRB, 0.75869E+00_JPRB, 0.78213E+00_JPRB, 0.72723E+00_JPRB, 0.67445E+00_JPRB, 0.59782E+00_JPRB, 0.53148E+00_JPRB ,&
&  0.51262E+00_JPRB, 0.42182E+00_JPRB, 0.29978E+00_JPRB, 0.42681E+00_JPRB, 0.58770E+00_JPRB, 0.62499E+00_JPRB, 0.75374E+00_JPRB ,&
&  0.78804E+00_JPRB, 0.75775E+00_JPRB, 0.72879E+00_JPRB, 0.72167E+00_JPRB, 0.67861E+00_JPRB, 0.65839E+00_JPRB, 0.57988E+00_JPRB ,&
&  0.53704E+00_JPRB, 0.45726E+00_JPRB, 0.41193E+00_JPRB, 0.40258E+00_JPRB, 0.40218E+00_JPRB, 0.43101E+00_JPRB, 0.57930E+00_JPRB ,&
&  0.60281E+00_JPRB, 0.52947E+00_JPRB, 0.55671E+00_JPRB, 0.57632E+00_JPRB, 0.45874E+00_JPRB, 0.41962E+00_JPRB, 0.42462E+00_JPRB ,&
&  0.43119E+00_JPRB, 0.44630E+00_JPRB, 0.47234E+00_JPRB, 0.49047E+00_JPRB, 0.50375E+00_JPRB, 0.51537E+00_JPRB, 0.52944E+00_JPRB ,&
&  0.53848E+00_JPRB, 0.54964E+00_JPRB, 0.55417E+00_JPRB, 0.55906E+00_JPRB, 0.56379E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.59488E+00_JPRB, 0.59578E+00_JPRB, 0.58274E+00_JPRB, 0.59657E+00_JPRB, 0.62208E+00_JPRB, 0.68022E+00_JPRB, 0.73858E+00_JPRB ,&
&  0.81469E+00_JPRB, 0.90861E+00_JPRB, 0.10252E+01_JPRB, 0.11224E+01_JPRB, 0.11440E+01_JPRB, 0.11193E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.88989E+00_JPRB, 0.76018E+00_JPRB, 0.69053E+00_JPRB, 0.67386E+00_JPRB, 0.72140E+00_JPRB, 0.75006E+00_JPRB, 0.81572E+00_JPRB ,&
&  0.91509E+00_JPRB, 0.10816E+01_JPRB, 0.12711E+01_JPRB, 0.17103E+01_JPRB, 0.20556E+01_JPRB, 0.13218E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.11942E+01_JPRB, 0.11297E+01_JPRB, 0.11567E+01_JPRB, 0.12615E+01_JPRB, 0.13481E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.15013E+01_JPRB, 0.15480E+01_JPRB, 0.15816E+01_JPRB, 0.15997E+01_JPRB, 0.16193E+01_JPRB, 0.16164E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.15444E+01_JPRB, 0.14687E+01_JPRB, 0.14236E+01_JPRB, 0.14036E+01_JPRB, 0.13832E+01_JPRB, 0.12976E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.10683E+01_JPRB, 0.10125E+01_JPRB, 0.10498E+01_JPRB, 0.10116E+01_JPRB, 0.88891E+00_JPRB, 0.80012E+00_JPRB, 0.67186E+00_JPRB ,&
&  0.59375E+00_JPRB, 0.52589E+00_JPRB, 0.42448E+00_JPRB, 0.63033E+00_JPRB, 0.75227E+00_JPRB, 0.76929E+00_JPRB, 0.78019E+00_JPRB ,&
&  0.82971E+00_JPRB, 0.81640E+00_JPRB, 0.79951E+00_JPRB, 0.76561E+00_JPRB, 0.69514E+00_JPRB, 0.73180E+00_JPRB, 0.57937E+00_JPRB ,&
&  0.56268E+00_JPRB, 0.47414E+00_JPRB, 0.44127E+00_JPRB, 0.45588E+00_JPRB, 0.47361E+00_JPRB, 0.43424E+00_JPRB, 0.53029E+00_JPRB ,&
&  0.55829E+00_JPRB, 0.74606E+00_JPRB, 0.93602E+00_JPRB, 0.13130E+01_JPRB, 0.17204E+01_JPRB, 0.46661E+00_JPRB, 0.44625E+00_JPRB ,&
&  0.44046E+00_JPRB, 0.44712E+00_JPRB, 0.46612E+00_JPRB, 0.47723E+00_JPRB, 0.49546E+00_JPRB, 0.50539E+00_JPRB, 0.50754E+00_JPRB ,&
&  0.51505E+00_JPRB, 0.53441E+00_JPRB, 0.55414E+00_JPRB, 0.56363E+00_JPRB, 0.58295E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.85441E+00_JPRB, 0.89073E+00_JPRB, 0.89297E+00_JPRB, 0.95865E+00_JPRB, 0.10208E+01_JPRB, 0.10454E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.12004E+01_JPRB, 0.12660E+01_JPRB, 0.13044E+01_JPRB, 0.13440E+01_JPRB, 0.14060E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.13450E+01_JPRB, 0.13231E+01_JPRB, 0.12989E+01_JPRB, 0.12972E+01_JPRB, 0.13265E+01_JPRB, 0.14630E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.15828E+01_JPRB, 0.16018E+01_JPRB, 0.13779E+01_JPRB, 0.94850E+00_JPRB, 0.11738E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.12679E+01_JPRB, 0.13479E+01_JPRB, 0.13888E+01_JPRB, 0.14661E+01_JPRB, 0.15404E+01_JPRB, 0.16331E+01_JPRB, 0.17267E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.18587E+01_JPRB, 0.19353E+01_JPRB, 0.19915E+01_JPRB, 0.20551E+01_JPRB, 0.21429E+01_JPRB, 0.21610E+01_JPRB ,&
&  0.21487E+01_JPRB, 0.20753E+01_JPRB, 0.20045E+01_JPRB, 0.19670E+01_JPRB, 0.18237E+01_JPRB, 0.16618E+01_JPRB, 0.14614E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.13004E+01_JPRB, 0.13174E+01_JPRB, 0.13085E+01_JPRB, 0.12299E+01_JPRB, 0.11124E+01_JPRB, 0.97235E+00_JPRB ,&
&  0.87240E+00_JPRB, 0.80157E+00_JPRB, 0.80250E+00_JPRB, 0.88532E+00_JPRB, 0.66462E+00_JPRB, 0.73563E+00_JPRB, 0.90269E+00_JPRB ,&
&  0.88062E+00_JPRB, 0.83352E+00_JPRB, 0.80141E+00_JPRB, 0.71498E+00_JPRB, 0.68288E+00_JPRB, 0.57061E+00_JPRB, 0.60412E+00_JPRB ,&
&  0.60884E+00_JPRB, 0.51576E+00_JPRB, 0.48973E+00_JPRB, 0.53256E+00_JPRB, 0.56124E+00_JPRB, 0.50879E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.58530E+00_JPRB, 0.67302E+00_JPRB, 0.75292E+00_JPRB, 0.88285E+00_JPRB, 0.94238E+00_JPRB, 0.51569E+00_JPRB, 0.44072E+00_JPRB ,&
&  0.42608E+00_JPRB, 0.43802E+00_JPRB, 0.45912E+00_JPRB, 0.48323E+00_JPRB, 0.51409E+00_JPRB, 0.55163E+00_JPRB, 0.58992E+00_JPRB ,&
&  0.62095E+00_JPRB, 0.64325E+00_JPRB, 0.68192E+00_JPRB, 0.73397E+00_JPRB, 0.80366E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.11549E+01_JPRB, 0.12299E+01_JPRB, 0.12953E+01_JPRB, 0.12747E+01_JPRB, 0.12616E+01_JPRB, 0.12207E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.12652E+01_JPRB, 0.13341E+01_JPRB, 0.13869E+01_JPRB, 0.14228E+01_JPRB, 0.14951E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.16052E+01_JPRB, 0.16149E+01_JPRB, 0.16241E+01_JPRB, 0.16317E+01_JPRB, 0.15861E+01_JPRB, 0.15778E+01_JPRB, 0.15474E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.93967E+00_JPRB, 0.11232E+01_JPRB, 0.12941E+01_JPRB, 0.12333E+01_JPRB, 0.12844E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.15293E+01_JPRB, 0.15664E+01_JPRB, 0.15952E+01_JPRB, 0.16031E+01_JPRB, 0.16002E+01_JPRB, 0.17013E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.18927E+01_JPRB, 0.19703E+01_JPRB, 0.20492E+01_JPRB, 0.21311E+01_JPRB, 0.22026E+01_JPRB, 0.22834E+01_JPRB, 0.23243E+01_JPRB ,&
&  0.23347E+01_JPRB, 0.22856E+01_JPRB, 0.22245E+01_JPRB, 0.20850E+01_JPRB, 0.19426E+01_JPRB, 0.18159E+01_JPRB, 0.16755E+01_JPRB ,&
&  0.16371E+01_JPRB, 0.16579E+01_JPRB, 0.17128E+01_JPRB, 0.17259E+01_JPRB, 0.16940E+01_JPRB, 0.15511E+01_JPRB, 0.13942E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.10705E+01_JPRB, 0.11005E+01_JPRB, 0.11421E+01_JPRB, 0.91677E+00_JPRB, 0.88089E+00_JPRB, 0.88824E+00_JPRB ,&
&  0.98532E+00_JPRB, 0.89898E+00_JPRB, 0.80395E+00_JPRB, 0.75052E+00_JPRB, 0.78688E+00_JPRB, 0.70134E+00_JPRB, 0.70920E+00_JPRB ,&
&  0.65945E+00_JPRB, 0.56550E+00_JPRB, 0.51332E+00_JPRB, 0.55291E+00_JPRB, 0.48760E+00_JPRB, 0.48714E+00_JPRB, 0.56290E+00_JPRB ,&
&  0.65741E+00_JPRB, 0.61560E+00_JPRB, 0.65286E+00_JPRB, 0.64375E+00_JPRB, 0.60327E+00_JPRB, 0.48027E+00_JPRB, 0.44178E+00_JPRB ,&
&  0.42672E+00_JPRB, 0.44170E+00_JPRB, 0.47707E+00_JPRB, 0.50543E+00_JPRB, 0.55146E+00_JPRB, 0.60375E+00_JPRB, 0.69613E+00_JPRB ,&
&  0.77682E+00_JPRB, 0.85049E+00_JPRB, 0.90843E+00_JPRB, 0.98018E+00_JPRB, 0.10603E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.11336E+01_JPRB, 0.12509E+01_JPRB, 0.13685E+01_JPRB, 0.14484E+01_JPRB, 0.14060E+01_JPRB, 0.13473E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.12797E+01_JPRB, 0.12973E+01_JPRB, 0.13665E+01_JPRB, 0.14302E+01_JPRB, 0.14775E+01_JPRB, 0.15517E+01_JPRB, 0.16611E+01_JPRB ,&
&  0.17499E+01_JPRB, 0.18154E+01_JPRB, 0.18640E+01_JPRB, 0.19083E+01_JPRB, 0.18765E+01_JPRB, 0.18408E+01_JPRB, 0.15436E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10831E+01_JPRB, 0.14379E+01_JPRB, 0.13736E+01_JPRB, 0.13737E+01_JPRB, 0.14180E+01_JPRB, 0.15408E+01_JPRB ,&
&  0.17219E+01_JPRB, 0.17749E+01_JPRB, 0.17468E+01_JPRB, 0.17040E+01_JPRB, 0.16751E+01_JPRB, 0.17538E+01_JPRB, 0.18679E+01_JPRB ,&
&  0.19594E+01_JPRB, 0.20119E+01_JPRB, 0.20846E+01_JPRB, 0.21827E+01_JPRB, 0.23118E+01_JPRB, 0.24299E+01_JPRB, 0.25023E+01_JPRB ,&
&  0.24610E+01_JPRB, 0.24547E+01_JPRB, 0.23104E+01_JPRB, 0.20722E+01_JPRB, 0.19896E+01_JPRB, 0.20004E+01_JPRB, 0.20125E+01_JPRB ,&
&  0.20788E+01_JPRB, 0.22165E+01_JPRB, 0.23341E+01_JPRB, 0.22864E+01_JPRB, 0.20785E+01_JPRB, 0.19724E+01_JPRB, 0.17351E+01_JPRB ,&
&  0.16608E+01_JPRB, 0.14369E+01_JPRB, 0.14174E+01_JPRB, 0.12342E+01_JPRB, 0.12021E+01_JPRB, 0.11876E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.11218E+01_JPRB, 0.10028E+01_JPRB, 0.92393E+00_JPRB, 0.10011E+01_JPRB, 0.10029E+01_JPRB, 0.99317E+00_JPRB ,&
&  0.81200E+00_JPRB, 0.71226E+00_JPRB, 0.56436E+00_JPRB, 0.53470E+00_JPRB, 0.42631E+00_JPRB, 0.43063E+00_JPRB, 0.59951E+00_JPRB ,&
&  0.76059E+00_JPRB, 0.67922E+00_JPRB, 0.69965E+00_JPRB, 0.66892E+00_JPRB, 0.55830E+00_JPRB, 0.46065E+00_JPRB, 0.42337E+00_JPRB ,&
&  0.42470E+00_JPRB, 0.46797E+00_JPRB, 0.50702E+00_JPRB, 0.53817E+00_JPRB, 0.59535E+00_JPRB, 0.65605E+00_JPRB, 0.71976E+00_JPRB ,&
&  0.76092E+00_JPRB, 0.81670E+00_JPRB, 0.87514E+00_JPRB, 0.93440E+00_JPRB, 0.10227E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.87066E+00_JPRB, 0.91927E+00_JPRB, 0.96413E+00_JPRB, 0.10011E+01_JPRB, 0.10307E+01_JPRB, 0.10081E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.11998E+01_JPRB, 0.13108E+01_JPRB, 0.13878E+01_JPRB, 0.14172E+01_JPRB, 0.14827E+01_JPRB, 0.16140E+01_JPRB ,&
&  0.17322E+01_JPRB, 0.18571E+01_JPRB, 0.19645E+01_JPRB, 0.20670E+01_JPRB, 0.20406E+01_JPRB, 0.19337E+01_JPRB, 0.14569E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.16402E+01_JPRB, 0.13726E+01_JPRB, 0.13942E+01_JPRB, 0.14761E+01_JPRB, 0.15884E+01_JPRB, 0.16948E+01_JPRB ,&
&  0.18077E+01_JPRB, 0.18296E+01_JPRB, 0.17900E+01_JPRB, 0.17353E+01_JPRB, 0.16961E+01_JPRB, 0.17421E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.19253E+01_JPRB, 0.19915E+01_JPRB, 0.20846E+01_JPRB, 0.22229E+01_JPRB, 0.23984E+01_JPRB, 0.25818E+01_JPRB, 0.26692E+01_JPRB ,&
&  0.26531E+01_JPRB, 0.26206E+01_JPRB, 0.23958E+01_JPRB, 0.21254E+01_JPRB, 0.20496E+01_JPRB, 0.22171E+01_JPRB, 0.24485E+01_JPRB ,&
&  0.25827E+01_JPRB, 0.26837E+01_JPRB, 0.30058E+01_JPRB, 0.31101E+01_JPRB, 0.28986E+01_JPRB, 0.26179E+01_JPRB, 0.21141E+01_JPRB ,&
&  0.21204E+01_JPRB, 0.18514E+01_JPRB, 0.17240E+01_JPRB, 0.14875E+01_JPRB, 0.14294E+01_JPRB, 0.14784E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.14647E+01_JPRB, 0.14747E+01_JPRB, 0.13373E+01_JPRB, 0.12557E+01_JPRB, 0.12855E+01_JPRB, 0.13477E+01_JPRB, 0.12560E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.77885E+00_JPRB, 0.62179E+00_JPRB, 0.52627E+00_JPRB, 0.42608E+00_JPRB, 0.39048E+00_JPRB, 0.56779E+00_JPRB ,&
&  0.71759E+00_JPRB, 0.74177E+00_JPRB, 0.84933E+00_JPRB, 0.87547E+00_JPRB, 0.66697E+00_JPRB, 0.50948E+00_JPRB, 0.44592E+00_JPRB ,&
&  0.45581E+00_JPRB, 0.50703E+00_JPRB, 0.54173E+00_JPRB, 0.57342E+00_JPRB, 0.62030E+00_JPRB, 0.63577E+00_JPRB, 0.68678E+00_JPRB ,&
&  0.70483E+00_JPRB, 0.73869E+00_JPRB, 0.77166E+00_JPRB, 0.78608E+00_JPRB, 0.81988E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.67625E+00_JPRB, 0.70045E+00_JPRB, 0.74192E+00_JPRB, 0.77124E+00_JPRB, 0.79136E+00_JPRB, 0.82002E+00_JPRB, 0.82849E+00_JPRB ,&
&  0.87717E+00_JPRB, 0.97567E+00_JPRB, 0.10994E+01_JPRB, 0.12342E+01_JPRB, 0.12791E+01_JPRB, 0.13003E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.15873E+01_JPRB, 0.18405E+01_JPRB, 0.21103E+01_JPRB, 0.23382E+01_JPRB, 0.23679E+01_JPRB, 0.22034E+01_JPRB, 0.15504E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.16555E+01_JPRB, 0.15097E+01_JPRB, 0.14941E+01_JPRB, 0.16332E+01_JPRB, 0.17803E+01_JPRB, 0.17427E+01_JPRB ,&
&  0.17341E+01_JPRB, 0.17341E+01_JPRB, 0.16697E+01_JPRB, 0.16415E+01_JPRB, 0.16535E+01_JPRB, 0.16704E+01_JPRB, 0.17229E+01_JPRB ,&
&  0.18027E+01_JPRB, 0.18700E+01_JPRB, 0.19945E+01_JPRB, 0.22028E+01_JPRB, 0.24388E+01_JPRB, 0.26479E+01_JPRB, 0.27812E+01_JPRB ,&
&  0.28373E+01_JPRB, 0.27697E+01_JPRB, 0.26497E+01_JPRB, 0.22168E+01_JPRB, 0.21510E+01_JPRB, 0.23960E+01_JPRB, 0.28450E+01_JPRB ,&
&  0.30990E+01_JPRB, 0.32417E+01_JPRB, 0.37108E+01_JPRB, 0.40904E+01_JPRB, 0.39624E+01_JPRB, 0.33980E+01_JPRB, 0.27895E+01_JPRB ,&
&  0.25830E+01_JPRB, 0.23168E+01_JPRB, 0.19929E+01_JPRB, 0.18658E+01_JPRB, 0.17618E+01_JPRB, 0.17908E+01_JPRB, 0.19099E+01_JPRB ,&
&  0.18397E+01_JPRB, 0.18157E+01_JPRB, 0.16833E+01_JPRB, 0.15833E+01_JPRB, 0.15121E+01_JPRB, 0.14907E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.10719E+01_JPRB, 0.79636E+00_JPRB, 0.52602E+00_JPRB, 0.40060E+00_JPRB, 0.36703E+00_JPRB, 0.50686E+00_JPRB ,&
&  0.70429E+00_JPRB, 0.74188E+00_JPRB, 0.78441E+00_JPRB, 0.10814E+01_JPRB, 0.90904E+00_JPRB, 0.63380E+00_JPRB, 0.47084E+00_JPRB ,&
&  0.46670E+00_JPRB, 0.51189E+00_JPRB, 0.53703E+00_JPRB, 0.54308E+00_JPRB, 0.57330E+00_JPRB, 0.60157E+00_JPRB, 0.60702E+00_JPRB ,&
&  0.63047E+00_JPRB, 0.65205E+00_JPRB, 0.65352E+00_JPRB, 0.64657E+00_JPRB, 0.66041E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.58042E+00_JPRB, 0.60334E+00_JPRB, 0.61097E+00_JPRB, 0.61267E+00_JPRB, 0.62485E+00_JPRB, 0.65256E+00_JPRB, 0.66214E+00_JPRB ,&
&  0.70580E+00_JPRB, 0.77642E+00_JPRB, 0.87308E+00_JPRB, 0.10132E+01_JPRB, 0.11044E+01_JPRB, 0.11223E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.15634E+01_JPRB, 0.19873E+01_JPRB, 0.23839E+01_JPRB, 0.24354E+01_JPRB, 0.21876E+01_JPRB, 0.19852E+01_JPRB ,&
&  0.17454E+01_JPRB, 0.19251E+01_JPRB, 0.17839E+01_JPRB, 0.19724E+01_JPRB, 0.22618E+01_JPRB, 0.25896E+01_JPRB, 0.17800E+01_JPRB ,&
&  0.16422E+01_JPRB, 0.16107E+01_JPRB, 0.15504E+01_JPRB, 0.15467E+01_JPRB, 0.15202E+01_JPRB, 0.15433E+01_JPRB, 0.15634E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.17010E+01_JPRB, 0.18506E+01_JPRB, 0.20936E+01_JPRB, 0.24444E+01_JPRB, 0.27149E+01_JPRB, 0.28870E+01_JPRB ,&
&  0.29050E+01_JPRB, 0.29635E+01_JPRB, 0.29057E+01_JPRB, 0.25053E+01_JPRB, 0.22672E+01_JPRB, 0.25625E+01_JPRB, 0.31961E+01_JPRB ,&
&  0.39247E+01_JPRB, 0.44944E+01_JPRB, 0.46562E+01_JPRB, 0.46298E+01_JPRB, 0.45098E+01_JPRB, 0.42099E+01_JPRB, 0.34929E+01_JPRB ,&
&  0.30517E+01_JPRB, 0.26945E+01_JPRB, 0.22540E+01_JPRB, 0.22717E+01_JPRB, 0.22743E+01_JPRB, 0.22572E+01_JPRB, 0.19497E+01_JPRB ,&
&  0.17483E+01_JPRB, 0.16400E+01_JPRB, 0.15734E+01_JPRB, 0.16483E+01_JPRB, 0.17389E+01_JPRB, 0.16813E+01_JPRB, 0.18247E+01_JPRB ,&
&  0.14629E+01_JPRB, 0.56603E+00_JPRB, 0.38655E+00_JPRB, 0.43806E+00_JPRB, 0.28978E+00_JPRB, 0.29748E+00_JPRB, 0.52735E+00_JPRB ,&
&  0.78415E+00_JPRB, 0.88912E+00_JPRB, 0.86798E+00_JPRB, 0.91656E+00_JPRB, 0.10918E+01_JPRB, 0.83311E+00_JPRB, 0.52324E+00_JPRB ,&
&  0.50922E+00_JPRB, 0.55555E+00_JPRB, 0.56152E+00_JPRB, 0.54064E+00_JPRB, 0.53862E+00_JPRB, 0.56288E+00_JPRB, 0.53286E+00_JPRB ,&
&  0.55783E+00_JPRB, 0.57315E+00_JPRB, 0.57800E+00_JPRB, 0.57547E+00_JPRB, 0.56925E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.53960E+00_JPRB, 0.56320E+00_JPRB, 0.57819E+00_JPRB, 0.55230E+00_JPRB, 0.54170E+00_JPRB, 0.55513E+00_JPRB, 0.56205E+00_JPRB ,&
&  0.60294E+00_JPRB, 0.64807E+00_JPRB, 0.70426E+00_JPRB, 0.80147E+00_JPRB, 0.90400E+00_JPRB, 0.96115E+00_JPRB, 0.10075E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.12813E+01_JPRB, 0.16897E+01_JPRB, 0.22517E+01_JPRB, 0.25502E+01_JPRB, 0.23841E+01_JPRB, 0.24325E+01_JPRB ,&
&  0.20612E+01_JPRB, 0.23797E+01_JPRB, 0.30618E+01_JPRB, 0.37766E+01_JPRB, 0.39140E+01_JPRB, 0.36558E+01_JPRB, 0.22554E+01_JPRB ,&
&  0.17850E+01_JPRB, 0.17030E+01_JPRB, 0.15459E+01_JPRB, 0.15779E+01_JPRB, 0.14400E+01_JPRB, 0.14629E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.16390E+01_JPRB, 0.17525E+01_JPRB, 0.20112E+01_JPRB, 0.24299E+01_JPRB, 0.28025E+01_JPRB, 0.31526E+01_JPRB ,&
&  0.31237E+01_JPRB, 0.31756E+01_JPRB, 0.31304E+01_JPRB, 0.31289E+01_JPRB, 0.21997E+01_JPRB, 0.29829E+01_JPRB, 0.35824E+01_JPRB ,&
&  0.45328E+01_JPRB, 0.51592E+01_JPRB, 0.49657E+01_JPRB, 0.55395E+01_JPRB, 0.55244E+01_JPRB, 0.48185E+01_JPRB, 0.39823E+01_JPRB ,&
&  0.35228E+01_JPRB, 0.28900E+01_JPRB, 0.25590E+01_JPRB, 0.25933E+01_JPRB, 0.26558E+01_JPRB, 0.27522E+01_JPRB, 0.17793E+01_JPRB ,&
&  0.16745E+01_JPRB, 0.18945E+01_JPRB, 0.17873E+01_JPRB, 0.15514E+01_JPRB, 0.17913E+01_JPRB, 0.18827E+01_JPRB, 0.14369E+01_JPRB ,&
&  0.61693E+00_JPRB, 0.35126E+00_JPRB, 0.29219E+00_JPRB, 0.29045E+00_JPRB, 0.26148E+00_JPRB, 0.32444E+00_JPRB, 0.90355E+00_JPRB ,&
&  0.78809E+00_JPRB, 0.10158E+01_JPRB, 0.94992E+00_JPRB, 0.93723E+00_JPRB, 0.11658E+01_JPRB, 0.11262E+01_JPRB, 0.12603E+01_JPRB ,&
&  0.62119E+00_JPRB, 0.59020E+00_JPRB, 0.61322E+00_JPRB, 0.60756E+00_JPRB, 0.61424E+00_JPRB, 0.58822E+00_JPRB, 0.55283E+00_JPRB ,&
&  0.56615E+00_JPRB, 0.58590E+00_JPRB, 0.56242E+00_JPRB, 0.53952E+00_JPRB, 0.53042E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.52603E+00_JPRB, 0.53710E+00_JPRB, 0.54069E+00_JPRB, 0.53001E+00_JPRB, 0.49759E+00_JPRB, 0.49470E+00_JPRB, 0.51279E+00_JPRB ,&
&  0.53653E+00_JPRB, 0.56400E+00_JPRB, 0.59904E+00_JPRB, 0.65021E+00_JPRB, 0.72272E+00_JPRB, 0.79498E+00_JPRB, 0.86446E+00_JPRB ,&
&  0.95558E+00_JPRB, 0.11479E+01_JPRB, 0.14885E+01_JPRB, 0.19171E+01_JPRB, 0.24583E+01_JPRB, 0.23798E+01_JPRB, 0.26688E+01_JPRB ,&
&  0.23333E+01_JPRB, 0.26161E+01_JPRB, 0.36932E+01_JPRB, 0.50955E+01_JPRB, 0.67553E+01_JPRB, 0.64328E+01_JPRB, 0.46494E+01_JPRB ,&
&  0.30891E+01_JPRB, 0.24624E+01_JPRB, 0.21406E+01_JPRB, 0.18276E+01_JPRB, 0.16013E+01_JPRB, 0.15436E+01_JPRB, 0.15792E+01_JPRB ,&
&  0.15463E+01_JPRB, 0.15778E+01_JPRB, 0.16239E+01_JPRB, 0.18244E+01_JPRB, 0.22210E+01_JPRB, 0.26844E+01_JPRB, 0.32434E+01_JPRB ,&
&  0.35320E+01_JPRB, 0.36978E+01_JPRB, 0.35805E+01_JPRB, 0.34700E+01_JPRB, 0.31336E+01_JPRB, 0.30536E+01_JPRB, 0.37050E+01_JPRB ,&
&  0.49658E+01_JPRB, 0.60706E+01_JPRB, 0.58349E+01_JPRB, 0.67281E+01_JPRB, 0.60573E+01_JPRB, 0.52390E+01_JPRB, 0.44851E+01_JPRB ,&
&  0.38014E+01_JPRB, 0.36453E+01_JPRB, 0.27656E+01_JPRB, 0.27008E+01_JPRB, 0.28943E+01_JPRB, 0.23429E+01_JPRB, 0.19340E+01_JPRB ,&
&  0.22317E+01_JPRB, 0.19605E+01_JPRB, 0.17897E+01_JPRB, 0.15155E+01_JPRB, 0.18892E+01_JPRB, 0.16787E+01_JPRB, 0.83087E+00_JPRB ,&
&  0.54851E+00_JPRB, 0.40655E+00_JPRB, 0.28855E+00_JPRB, 0.29068E+00_JPRB, 0.28106E+00_JPRB, 0.34754E+00_JPRB, 0.76741E+00_JPRB ,&
&  0.89564E+00_JPRB, 0.11039E+01_JPRB, 0.10637E+01_JPRB, 0.95163E+00_JPRB, 0.10938E+01_JPRB, 0.12618E+01_JPRB, 0.24244E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.97714E+00_JPRB, 0.70935E+00_JPRB, 0.68060E+00_JPRB, 0.67840E+00_JPRB, 0.64973E+00_JPRB, 0.61063E+00_JPRB ,&
&  0.59265E+00_JPRB, 0.60426E+00_JPRB, 0.59689E+00_JPRB, 0.57867E+00_JPRB, 0.55264E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.55396E+00_JPRB, 0.54919E+00_JPRB, 0.55462E+00_JPRB, 0.55134E+00_JPRB, 0.52776E+00_JPRB, 0.51229E+00_JPRB, 0.50954E+00_JPRB ,&
&  0.51802E+00_JPRB, 0.52716E+00_JPRB, 0.54124E+00_JPRB, 0.56666E+00_JPRB, 0.60418E+00_JPRB, 0.65693E+00_JPRB, 0.72627E+00_JPRB ,&
&  0.84590E+00_JPRB, 0.10646E+01_JPRB, 0.13113E+01_JPRB, 0.15588E+01_JPRB, 0.19850E+01_JPRB, 0.21145E+01_JPRB, 0.25523E+01_JPRB ,&
&  0.25973E+01_JPRB, 0.28204E+01_JPRB, 0.39786E+01_JPRB, 0.60277E+01_JPRB, 0.92072E+01_JPRB, 0.10140E+02_JPRB, 0.91613E+01_JPRB ,&
&  0.70068E+01_JPRB, 0.46863E+01_JPRB, 0.33075E+01_JPRB, 0.24491E+01_JPRB, 0.19301E+01_JPRB, 0.16555E+01_JPRB, 0.16357E+01_JPRB ,&
&  0.15659E+01_JPRB, 0.15527E+01_JPRB, 0.15881E+01_JPRB, 0.16105E+01_JPRB, 0.18570E+01_JPRB, 0.23332E+01_JPRB, 0.30699E+01_JPRB ,&
&  0.36956E+01_JPRB, 0.41513E+01_JPRB, 0.42426E+01_JPRB, 0.37038E+01_JPRB, 0.42523E+01_JPRB, 0.43794E+01_JPRB, 0.42295E+01_JPRB ,&
&  0.46439E+01_JPRB, 0.53104E+01_JPRB, 0.69969E+01_JPRB, 0.66355E+01_JPRB, 0.58113E+01_JPRB, 0.56266E+01_JPRB, 0.47544E+01_JPRB ,&
&  0.41298E+01_JPRB, 0.38246E+01_JPRB, 0.30657E+01_JPRB, 0.29618E+01_JPRB, 0.26909E+01_JPRB, 0.21433E+01_JPRB, 0.22746E+01_JPRB ,&
&  0.25058E+01_JPRB, 0.23309E+01_JPRB, 0.19551E+01_JPRB, 0.15903E+01_JPRB, 0.15238E+01_JPRB, 0.10938E+01_JPRB, 0.70177E+00_JPRB ,&
&  0.57771E+00_JPRB, 0.47655E+00_JPRB, 0.34878E+00_JPRB, 0.41307E+00_JPRB, 0.37577E+00_JPRB, 0.46555E+00_JPRB, 0.78584E+00_JPRB ,&
&  0.11174E+01_JPRB, 0.11523E+01_JPRB, 0.11794E+01_JPRB, 0.98725E+00_JPRB, 0.98546E+00_JPRB, 0.11483E+01_JPRB, 0.27809E+01_JPRB ,&
&  0.18214E+01_JPRB, 0.15399E+01_JPRB, 0.11288E+01_JPRB, 0.85259E+00_JPRB, 0.74031E+00_JPRB, 0.68900E+00_JPRB, 0.66303E+00_JPRB ,&
&  0.63236E+00_JPRB, 0.61044E+00_JPRB, 0.60268E+00_JPRB, 0.58561E+00_JPRB, 0.56574E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.61616E+00_JPRB, 0.61192E+00_JPRB, 0.61362E+00_JPRB, 0.60928E+00_JPRB, 0.59817E+00_JPRB, 0.57454E+00_JPRB, 0.54579E+00_JPRB ,&
&  0.53681E+00_JPRB, 0.53540E+00_JPRB, 0.53700E+00_JPRB, 0.55328E+00_JPRB, 0.56371E+00_JPRB, 0.58774E+00_JPRB, 0.63576E+00_JPRB ,&
&  0.77824E+00_JPRB, 0.10636E+01_JPRB, 0.10718E+01_JPRB, 0.11946E+01_JPRB, 0.15516E+01_JPRB, 0.20749E+01_JPRB, 0.23263E+01_JPRB ,&
&  0.27025E+01_JPRB, 0.28274E+01_JPRB, 0.36866E+01_JPRB, 0.56860E+01_JPRB, 0.11592E+02_JPRB, 0.13532E+02_JPRB, 0.12667E+02_JPRB ,&
&  0.12002E+02_JPRB, 0.86790E+01_JPRB, 0.55175E+01_JPRB, 0.37975E+01_JPRB, 0.27153E+01_JPRB, 0.19771E+01_JPRB, 0.16358E+01_JPRB ,&
&  0.15107E+01_JPRB, 0.14653E+01_JPRB, 0.14607E+01_JPRB, 0.14945E+01_JPRB, 0.16172E+01_JPRB, 0.19873E+01_JPRB, 0.27490E+01_JPRB ,&
&  0.35196E+01_JPRB, 0.42797E+01_JPRB, 0.47592E+01_JPRB, 0.46348E+01_JPRB, 0.38983E+01_JPRB, 0.43260E+01_JPRB, 0.63063E+01_JPRB ,&
&  0.61665E+01_JPRB, 0.67023E+01_JPRB, 0.68429E+01_JPRB, 0.69536E+01_JPRB, 0.59380E+01_JPRB, 0.52011E+01_JPRB, 0.47318E+01_JPRB ,&
&  0.33890E+01_JPRB, 0.28591E+01_JPRB, 0.28942E+01_JPRB, 0.28489E+01_JPRB, 0.25358E+01_JPRB, 0.27494E+01_JPRB, 0.36147E+01_JPRB ,&
&  0.27789E+01_JPRB, 0.28316E+01_JPRB, 0.28467E+01_JPRB, 0.27172E+01_JPRB, 0.15838E+01_JPRB, 0.10625E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.79688E+00_JPRB, 0.61242E+00_JPRB, 0.81052E+00_JPRB, 0.57121E+00_JPRB, 0.68210E+00_JPRB, 0.99828E+00_JPRB ,&
&  0.10672E+01_JPRB, 0.10722E+01_JPRB, 0.13158E+01_JPRB, 0.11453E+01_JPRB, 0.11081E+01_JPRB, 0.10853E+01_JPRB, 0.17335E+01_JPRB ,&
&  0.20062E+01_JPRB, 0.17383E+01_JPRB, 0.15150E+01_JPRB, 0.11640E+01_JPRB, 0.92594E+00_JPRB, 0.82052E+00_JPRB, 0.77517E+00_JPRB ,&
&  0.74515E+00_JPRB, 0.71468E+00_JPRB, 0.68599E+00_JPRB, 0.64385E+00_JPRB, 0.62678E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.67496E+00_JPRB, 0.66794E+00_JPRB, 0.66585E+00_JPRB, 0.66515E+00_JPRB, 0.65889E+00_JPRB, 0.62830E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.56277E+00_JPRB, 0.55249E+00_JPRB, 0.55453E+00_JPRB, 0.55762E+00_JPRB, 0.54814E+00_JPRB, 0.55639E+00_JPRB, 0.58143E+00_JPRB ,&
&  0.70853E+00_JPRB, 0.91177E+00_JPRB, 0.96138E+00_JPRB, 0.10621E+01_JPRB, 0.14608E+01_JPRB, 0.20581E+01_JPRB, 0.24491E+01_JPRB ,&
&  0.29724E+01_JPRB, 0.29116E+01_JPRB, 0.34639E+01_JPRB, 0.54284E+01_JPRB, 0.95356E+01_JPRB, 0.12381E+02_JPRB, 0.12119E+02_JPRB ,&
&  0.11702E+02_JPRB, 0.96270E+01_JPRB, 0.72745E+01_JPRB, 0.54400E+01_JPRB, 0.39317E+01_JPRB, 0.28499E+01_JPRB, 0.20917E+01_JPRB ,&
&  0.16908E+01_JPRB, 0.15034E+01_JPRB, 0.14459E+01_JPRB, 0.14241E+01_JPRB, 0.15022E+01_JPRB, 0.16659E+01_JPRB, 0.22507E+01_JPRB ,&
&  0.29402E+01_JPRB, 0.39349E+01_JPRB, 0.50692E+01_JPRB, 0.55885E+01_JPRB, 0.47137E+01_JPRB, 0.51319E+01_JPRB, 0.67478E+01_JPRB ,&
&  0.68028E+01_JPRB, 0.67297E+01_JPRB, 0.68350E+01_JPRB, 0.65038E+01_JPRB, 0.63357E+01_JPRB, 0.55437E+01_JPRB, 0.53504E+01_JPRB ,&
&  0.40860E+01_JPRB, 0.33264E+01_JPRB, 0.29241E+01_JPRB, 0.27054E+01_JPRB, 0.31075E+01_JPRB, 0.40201E+01_JPRB, 0.38975E+01_JPRB ,&
&  0.34627E+01_JPRB, 0.33069E+01_JPRB, 0.30361E+01_JPRB, 0.24483E+01_JPRB, 0.22428E+01_JPRB, 0.18092E+01_JPRB, 0.20977E+01_JPRB ,&
&  0.20318E+01_JPRB, 0.18420E+01_JPRB, 0.15259E+01_JPRB, 0.11848E+01_JPRB, 0.93051E+00_JPRB, 0.11064E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.10370E+01_JPRB, 0.11693E+01_JPRB, 0.11530E+01_JPRB, 0.11613E+01_JPRB, 0.98512E+00_JPRB, 0.13159E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.16628E+01_JPRB, 0.16571E+01_JPRB, 0.13981E+01_JPRB, 0.11951E+01_JPRB, 0.10320E+01_JPRB, 0.94331E+00_JPRB ,&
&  0.90143E+00_JPRB, 0.86021E+00_JPRB, 0.80307E+00_JPRB, 0.74088E+00_JPRB, 0.70335E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.68848E+00_JPRB, 0.68115E+00_JPRB, 0.67117E+00_JPRB, 0.65927E+00_JPRB, 0.64108E+00_JPRB, 0.62424E+00_JPRB, 0.60090E+00_JPRB ,&
&  0.58115E+00_JPRB, 0.56615E+00_JPRB, 0.56291E+00_JPRB, 0.56024E+00_JPRB, 0.55889E+00_JPRB, 0.55060E+00_JPRB, 0.55234E+00_JPRB ,&
&  0.62810E+00_JPRB, 0.81288E+00_JPRB, 0.84976E+00_JPRB, 0.10031E+01_JPRB, 0.13766E+01_JPRB, 0.19679E+01_JPRB, 0.27557E+01_JPRB ,&
&  0.31555E+01_JPRB, 0.32500E+01_JPRB, 0.36674E+01_JPRB, 0.51436E+01_JPRB, 0.70955E+01_JPRB, 0.83687E+01_JPRB, 0.85906E+01_JPRB ,&
&  0.73877E+01_JPRB, 0.65355E+01_JPRB, 0.55866E+01_JPRB, 0.47482E+01_JPRB, 0.41098E+01_JPRB, 0.35328E+01_JPRB, 0.28573E+01_JPRB ,&
&  0.22546E+01_JPRB, 0.18609E+01_JPRB, 0.16639E+01_JPRB, 0.15678E+01_JPRB, 0.15391E+01_JPRB, 0.15941E+01_JPRB, 0.18660E+01_JPRB ,&
&  0.23452E+01_JPRB, 0.31957E+01_JPRB, 0.47510E+01_JPRB, 0.64310E+01_JPRB, 0.61113E+01_JPRB, 0.70651E+01_JPRB, 0.70474E+01_JPRB ,&
&  0.77391E+01_JPRB, 0.78167E+01_JPRB, 0.70099E+01_JPRB, 0.67887E+01_JPRB, 0.54399E+01_JPRB, 0.56574E+01_JPRB, 0.59104E+01_JPRB ,&
&  0.58800E+01_JPRB, 0.54930E+01_JPRB, 0.52872E+01_JPRB, 0.43781E+01_JPRB, 0.38713E+01_JPRB, 0.46708E+01_JPRB, 0.41843E+01_JPRB ,&
&  0.36225E+01_JPRB, 0.33964E+01_JPRB, 0.30626E+01_JPRB, 0.29360E+01_JPRB, 0.24100E+01_JPRB, 0.22958E+01_JPRB, 0.19556E+01_JPRB ,&
&  0.17867E+01_JPRB, 0.14851E+01_JPRB, 0.16344E+01_JPRB, 0.14114E+01_JPRB, 0.13148E+01_JPRB, 0.13070E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.11988E+01_JPRB, 0.10239E+01_JPRB, 0.92957E+00_JPRB, 0.10352E+01_JPRB, 0.11772E+01_JPRB, 0.10087E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.13743E+01_JPRB, 0.13070E+01_JPRB, 0.13354E+01_JPRB, 0.12830E+01_JPRB, 0.12147E+01_JPRB, 0.11066E+01_JPRB ,&
&  0.97402E+00_JPRB, 0.85959E+00_JPRB, 0.79084E+00_JPRB, 0.73386E+00_JPRB, 0.69683E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.66342E+00_JPRB, 0.66600E+00_JPRB, 0.64944E+00_JPRB, 0.62560E+00_JPRB, 0.61053E+00_JPRB, 0.59629E+00_JPRB, 0.58378E+00_JPRB ,&
&  0.56753E+00_JPRB, 0.55728E+00_JPRB, 0.55437E+00_JPRB, 0.55644E+00_JPRB, 0.55839E+00_JPRB, 0.55708E+00_JPRB, 0.55075E+00_JPRB ,&
&  0.56434E+00_JPRB, 0.75637E+00_JPRB, 0.77329E+00_JPRB, 0.81956E+00_JPRB, 0.11295E+01_JPRB, 0.19121E+01_JPRB, 0.27237E+01_JPRB ,&
&  0.31337E+01_JPRB, 0.33038E+01_JPRB, 0.36866E+01_JPRB, 0.42490E+01_JPRB, 0.49110E+01_JPRB, 0.53687E+01_JPRB, 0.55243E+01_JPRB ,&
&  0.53828E+01_JPRB, 0.41330E+01_JPRB, 0.37296E+01_JPRB, 0.33623E+01_JPRB, 0.31487E+01_JPRB, 0.29352E+01_JPRB, 0.27232E+01_JPRB ,&
&  0.24415E+01_JPRB, 0.20987E+01_JPRB, 0.18376E+01_JPRB, 0.16686E+01_JPRB, 0.15762E+01_JPRB, 0.14938E+01_JPRB, 0.15814E+01_JPRB ,&
&  0.17844E+01_JPRB, 0.20807E+01_JPRB, 0.30837E+01_JPRB, 0.46493E+01_JPRB, 0.69315E+01_JPRB, 0.82788E+01_JPRB, 0.81123E+01_JPRB ,&
&  0.66955E+01_JPRB, 0.64637E+01_JPRB, 0.82542E+01_JPRB, 0.69387E+01_JPRB, 0.65055E+01_JPRB, 0.66024E+01_JPRB, 0.62656E+01_JPRB ,&
&  0.60250E+01_JPRB, 0.55776E+01_JPRB, 0.50893E+01_JPRB, 0.45702E+01_JPRB, 0.48194E+01_JPRB, 0.46698E+01_JPRB, 0.41438E+01_JPRB ,&
&  0.35694E+01_JPRB, 0.34220E+01_JPRB, 0.31796E+01_JPRB, 0.29205E+01_JPRB, 0.27157E+01_JPRB, 0.27538E+01_JPRB, 0.24999E+01_JPRB ,&
&  0.22667E+01_JPRB, 0.24212E+01_JPRB, 0.19499E+01_JPRB, 0.15226E+01_JPRB, 0.12716E+01_JPRB, 0.13126E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.98413E+00_JPRB, 0.78170E+00_JPRB, 0.78189E+00_JPRB, 0.88255E+00_JPRB, 0.93665E+00_JPRB, 0.93377E+00_JPRB ,&
&  0.97944E+00_JPRB, 0.11280E+01_JPRB, 0.11250E+01_JPRB, 0.11328E+01_JPRB, 0.12354E+01_JPRB, 0.11560E+01_JPRB, 0.10111E+01_JPRB ,&
&  0.87716E+00_JPRB, 0.78874E+00_JPRB, 0.73651E+00_JPRB, 0.70033E+00_JPRB, 0.66911E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.65254E+00_JPRB, 0.64741E+00_JPRB, 0.63182E+00_JPRB, 0.60887E+00_JPRB, 0.59456E+00_JPRB, 0.58804E+00_JPRB, 0.57636E+00_JPRB ,&
&  0.56898E+00_JPRB, 0.56288E+00_JPRB, 0.56671E+00_JPRB, 0.56361E+00_JPRB, 0.56775E+00_JPRB, 0.57757E+00_JPRB, 0.58234E+00_JPRB ,&
&  0.58308E+00_JPRB, 0.73975E+00_JPRB, 0.71698E+00_JPRB, 0.77875E+00_JPRB, 0.96787E+00_JPRB, 0.15906E+01_JPRB, 0.21807E+01_JPRB ,&
&  0.25721E+01_JPRB, 0.27130E+01_JPRB, 0.28793E+01_JPRB, 0.31913E+01_JPRB, 0.33768E+01_JPRB, 0.33250E+01_JPRB, 0.32838E+01_JPRB ,&
&  0.31401E+01_JPRB, 0.28972E+01_JPRB, 0.24264E+01_JPRB, 0.23908E+01_JPRB, 0.21978E+01_JPRB, 0.18871E+01_JPRB, 0.20168E+01_JPRB ,&
&  0.20249E+01_JPRB, 0.19748E+01_JPRB, 0.18633E+01_JPRB, 0.17625E+01_JPRB, 0.16944E+01_JPRB, 0.15991E+01_JPRB, 0.15584E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.16516E+01_JPRB, 0.19085E+01_JPRB, 0.25728E+01_JPRB, 0.40007E+01_JPRB, 0.57164E+01_JPRB, 0.70381E+01_JPRB ,&
&  0.81667E+01_JPRB, 0.83533E+01_JPRB, 0.69477E+01_JPRB, 0.69276E+01_JPRB, 0.74671E+01_JPRB, 0.66076E+01_JPRB, 0.59689E+01_JPRB ,&
&  0.59769E+01_JPRB, 0.58754E+01_JPRB, 0.54126E+01_JPRB, 0.49957E+01_JPRB, 0.47847E+01_JPRB, 0.45846E+01_JPRB, 0.42414E+01_JPRB ,&
&  0.36034E+01_JPRB, 0.33449E+01_JPRB, 0.32429E+01_JPRB, 0.31001E+01_JPRB, 0.27494E+01_JPRB, 0.25516E+01_JPRB, 0.25067E+01_JPRB ,&
&  0.23868E+01_JPRB, 0.23073E+01_JPRB, 0.18145E+01_JPRB, 0.17064E+01_JPRB, 0.13101E+01_JPRB, 0.12687E+01_JPRB, 0.13845E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.93306E+00_JPRB, 0.70010E+00_JPRB, 0.61103E+00_JPRB, 0.60987E+00_JPRB, 0.66068E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.83104E+00_JPRB, 0.78262E+00_JPRB, 0.92043E+00_JPRB, 0.94172E+00_JPRB, 0.10018E+01_JPRB, 0.97805E+00_JPRB, 0.90940E+00_JPRB ,&
&  0.84995E+00_JPRB, 0.80264E+00_JPRB, 0.75648E+00_JPRB, 0.71038E+00_JPRB, 0.67414E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.63561E+00_JPRB, 0.60855E+00_JPRB, 0.59531E+00_JPRB, 0.57966E+00_JPRB, 0.56997E+00_JPRB, 0.56507E+00_JPRB, 0.56367E+00_JPRB ,&
&  0.57196E+00_JPRB, 0.57723E+00_JPRB, 0.58396E+00_JPRB, 0.57974E+00_JPRB, 0.57860E+00_JPRB, 0.57803E+00_JPRB, 0.60239E+00_JPRB ,&
&  0.62082E+00_JPRB, 0.68197E+00_JPRB, 0.72841E+00_JPRB, 0.77745E+00_JPRB, 0.10968E+01_JPRB, 0.13226E+01_JPRB, 0.15908E+01_JPRB ,&
&  0.17441E+01_JPRB, 0.18831E+01_JPRB, 0.19402E+01_JPRB, 0.19769E+01_JPRB, 0.19773E+01_JPRB, 0.19578E+01_JPRB, 0.18602E+01_JPRB ,&
&  0.17021E+01_JPRB, 0.15158E+01_JPRB, 0.14262E+01_JPRB, 0.13923E+01_JPRB, 0.14207E+01_JPRB, 0.12668E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.14441E+01_JPRB, 0.15279E+01_JPRB, 0.15676E+01_JPRB, 0.15860E+01_JPRB, 0.15826E+01_JPRB, 0.15866E+01_JPRB, 0.15993E+01_JPRB ,&
&  0.16424E+01_JPRB, 0.16440E+01_JPRB, 0.16787E+01_JPRB, 0.19818E+01_JPRB, 0.27968E+01_JPRB, 0.45433E+01_JPRB, 0.61821E+01_JPRB ,&
&  0.62117E+01_JPRB, 0.72491E+01_JPRB, 0.83216E+01_JPRB, 0.84108E+01_JPRB, 0.70394E+01_JPRB, 0.59290E+01_JPRB, 0.55202E+01_JPRB ,&
&  0.56305E+01_JPRB, 0.58429E+01_JPRB, 0.56087E+01_JPRB, 0.49582E+01_JPRB, 0.46709E+01_JPRB, 0.45023E+01_JPRB, 0.42939E+01_JPRB ,&
&  0.39643E+01_JPRB, 0.36054E+01_JPRB, 0.33427E+01_JPRB, 0.31495E+01_JPRB, 0.30139E+01_JPRB, 0.27131E+01_JPRB, 0.25501E+01_JPRB ,&
&  0.24998E+01_JPRB, 0.20865E+01_JPRB, 0.18548E+01_JPRB, 0.18867E+01_JPRB, 0.15329E+01_JPRB, 0.14928E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.12684E+01_JPRB, 0.92080E+00_JPRB, 0.74651E+00_JPRB, 0.63190E+00_JPRB, 0.58987E+00_JPRB, 0.59756E+00_JPRB, 0.62945E+00_JPRB ,&
&  0.67526E+00_JPRB, 0.68418E+00_JPRB, 0.79854E+00_JPRB, 0.84729E+00_JPRB, 0.87093E+00_JPRB, 0.85237E+00_JPRB, 0.81772E+00_JPRB ,&
&  0.76855E+00_JPRB, 0.73182E+00_JPRB, 0.71115E+00_JPRB, 0.69145E+00_JPRB, 0.66931E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.61742E+00_JPRB, 0.59897E+00_JPRB, 0.58609E+00_JPRB, 0.57735E+00_JPRB, 0.57066E+00_JPRB, 0.56411E+00_JPRB, 0.56071E+00_JPRB ,&
&  0.56218E+00_JPRB, 0.57487E+00_JPRB, 0.59188E+00_JPRB, 0.59037E+00_JPRB, 0.58544E+00_JPRB, 0.58737E+00_JPRB, 0.63684E+00_JPRB ,&
&  0.52324E+00_JPRB, 0.56290E+00_JPRB, 0.60480E+00_JPRB, 0.75109E+00_JPRB, 0.10965E+01_JPRB, 0.10817E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.12200E+01_JPRB, 0.12261E+01_JPRB, 0.12462E+01_JPRB, 0.12300E+01_JPRB, 0.12165E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.91619E+00_JPRB, 0.87539E+00_JPRB, 0.84816E+00_JPRB, 0.88856E+00_JPRB, 0.94619E+00_JPRB, 0.10410E+01_JPRB ,&
&  0.11197E+01_JPRB, 0.11964E+01_JPRB, 0.12578E+01_JPRB, 0.13040E+01_JPRB, 0.13326E+01_JPRB, 0.13616E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.14788E+01_JPRB, 0.15438E+01_JPRB, 0.18206E+01_JPRB, 0.23957E+01_JPRB, 0.42969E+01_JPRB, 0.59051E+01_JPRB ,&
&  0.48036E+01_JPRB, 0.44718E+01_JPRB, 0.57553E+01_JPRB, 0.69071E+01_JPRB, 0.59695E+01_JPRB, 0.49881E+01_JPRB, 0.44068E+01_JPRB ,&
&  0.43911E+01_JPRB, 0.45601E+01_JPRB, 0.46092E+01_JPRB, 0.43693E+01_JPRB, 0.39710E+01_JPRB, 0.38414E+01_JPRB, 0.37642E+01_JPRB ,&
&  0.36080E+01_JPRB, 0.34445E+01_JPRB, 0.32551E+01_JPRB, 0.30696E+01_JPRB, 0.29648E+01_JPRB, 0.29718E+01_JPRB, 0.28990E+01_JPRB ,&
&  0.27553E+01_JPRB, 0.25406E+01_JPRB, 0.22647E+01_JPRB, 0.20912E+01_JPRB, 0.17393E+01_JPRB, 0.16522E+01_JPRB, 0.17992E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.10588E+01_JPRB, 0.90159E+00_JPRB, 0.78269E+00_JPRB, 0.72784E+00_JPRB, 0.64119E+00_JPRB, 0.59885E+00_JPRB ,&
&  0.59751E+00_JPRB, 0.63114E+00_JPRB, 0.70245E+00_JPRB, 0.73263E+00_JPRB, 0.74196E+00_JPRB, 0.74859E+00_JPRB, 0.68404E+00_JPRB ,&
&  0.68972E+00_JPRB, 0.65690E+00_JPRB, 0.64077E+00_JPRB, 0.63689E+00_JPRB, 0.62901E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.59119E+00_JPRB, 0.58322E+00_JPRB, 0.57949E+00_JPRB, 0.56772E+00_JPRB, 0.55632E+00_JPRB, 0.55586E+00_JPRB, 0.56308E+00_JPRB ,&
&  0.56921E+00_JPRB, 0.58220E+00_JPRB, 0.59183E+00_JPRB, 0.59234E+00_JPRB, 0.59636E+00_JPRB, 0.62409E+00_JPRB, 0.57601E+00_JPRB ,&
&  0.49605E+00_JPRB, 0.51717E+00_JPRB, 0.53033E+00_JPRB, 0.59966E+00_JPRB, 0.68245E+00_JPRB, 0.71255E+00_JPRB, 0.75050E+00_JPRB ,&
&  0.80557E+00_JPRB, 0.80640E+00_JPRB, 0.82569E+00_JPRB, 0.84202E+00_JPRB, 0.84203E+00_JPRB, 0.83689E+00_JPRB, 0.82277E+00_JPRB ,&
&  0.74331E+00_JPRB, 0.69112E+00_JPRB, 0.65858E+00_JPRB, 0.64125E+00_JPRB, 0.67874E+00_JPRB, 0.74318E+00_JPRB, 0.81404E+00_JPRB ,&
&  0.88532E+00_JPRB, 0.94642E+00_JPRB, 0.10073E+01_JPRB, 0.10554E+01_JPRB, 0.10841E+01_JPRB, 0.11080E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.12305E+01_JPRB, 0.12716E+01_JPRB, 0.14838E+01_JPRB, 0.17514E+01_JPRB, 0.27234E+01_JPRB, 0.35041E+01_JPRB ,&
&  0.30558E+01_JPRB, 0.28741E+01_JPRB, 0.31028E+01_JPRB, 0.34353E+01_JPRB, 0.35078E+01_JPRB, 0.32801E+01_JPRB, 0.31274E+01_JPRB ,&
&  0.31658E+01_JPRB, 0.32769E+01_JPRB, 0.35395E+01_JPRB, 0.34168E+01_JPRB, 0.32610E+01_JPRB, 0.32073E+01_JPRB, 0.31097E+01_JPRB ,&
&  0.30825E+01_JPRB, 0.30162E+01_JPRB, 0.30397E+01_JPRB, 0.29519E+01_JPRB, 0.28588E+01_JPRB, 0.28366E+01_JPRB, 0.28179E+01_JPRB ,&
&  0.28070E+01_JPRB, 0.27289E+01_JPRB, 0.24974E+01_JPRB, 0.23076E+01_JPRB, 0.20580E+01_JPRB, 0.20268E+01_JPRB, 0.18458E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.11426E+01_JPRB, 0.10626E+01_JPRB, 0.92458E+00_JPRB, 0.81578E+00_JPRB, 0.70680E+00_JPRB, 0.63196E+00_JPRB ,&
&  0.61085E+00_JPRB, 0.65768E+00_JPRB, 0.67151E+00_JPRB, 0.68107E+00_JPRB, 0.69296E+00_JPRB, 0.69304E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.60499E+00_JPRB, 0.63298E+00_JPRB, 0.61731E+00_JPRB, 0.60725E+00_JPRB, 0.59776E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.56910E+00_JPRB, 0.57152E+00_JPRB, 0.57264E+00_JPRB, 0.57304E+00_JPRB, 0.55455E+00_JPRB, 0.57248E+00_JPRB, 0.52584E+00_JPRB ,&
&  0.59139E+00_JPRB, 0.60203E+00_JPRB, 0.59230E+00_JPRB, 0.61217E+00_JPRB, 0.65802E+00_JPRB, 0.61679E+00_JPRB, 0.48307E+00_JPRB ,&
&  0.43402E+00_JPRB, 0.44038E+00_JPRB, 0.46603E+00_JPRB, 0.48948E+00_JPRB, 0.51005E+00_JPRB, 0.52880E+00_JPRB, 0.55759E+00_JPRB ,&
&  0.58482E+00_JPRB, 0.59678E+00_JPRB, 0.62152E+00_JPRB, 0.63504E+00_JPRB, 0.63831E+00_JPRB, 0.63714E+00_JPRB, 0.64570E+00_JPRB ,&
&  0.60738E+00_JPRB, 0.58277E+00_JPRB, 0.56855E+00_JPRB, 0.53406E+00_JPRB, 0.57442E+00_JPRB, 0.60944E+00_JPRB, 0.65686E+00_JPRB ,&
&  0.71335E+00_JPRB, 0.77485E+00_JPRB, 0.82156E+00_JPRB, 0.86579E+00_JPRB, 0.89890E+00_JPRB, 0.92491E+00_JPRB, 0.95052E+00_JPRB ,&
&  0.98712E+00_JPRB, 0.10271E+01_JPRB, 0.10690E+01_JPRB, 0.12083E+01_JPRB, 0.14289E+01_JPRB, 0.17208E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.22151E+01_JPRB, 0.24496E+01_JPRB, 0.26474E+01_JPRB, 0.23319E+01_JPRB, 0.24976E+01_JPRB, 0.27569E+01_JPRB, 0.26473E+01_JPRB ,&
&  0.25486E+01_JPRB, 0.23393E+01_JPRB, 0.22685E+01_JPRB, 0.22668E+01_JPRB, 0.24121E+01_JPRB, 0.25189E+01_JPRB, 0.25513E+01_JPRB ,&
&  0.25206E+01_JPRB, 0.26092E+01_JPRB, 0.26810E+01_JPRB, 0.26976E+01_JPRB, 0.26674E+01_JPRB, 0.26589E+01_JPRB, 0.26934E+01_JPRB ,&
&  0.27186E+01_JPRB, 0.27095E+01_JPRB, 0.26415E+01_JPRB, 0.25014E+01_JPRB, 0.23231E+01_JPRB, 0.21179E+01_JPRB, 0.18675E+01_JPRB ,&
&  0.16420E+01_JPRB, 0.14088E+01_JPRB, 0.12133E+01_JPRB, 0.10779E+01_JPRB, 0.94067E+00_JPRB, 0.80920E+00_JPRB, 0.71254E+00_JPRB ,&
&  0.63096E+00_JPRB, 0.61856E+00_JPRB, 0.66789E+00_JPRB, 0.66803E+00_JPRB, 0.67332E+00_JPRB, 0.66587E+00_JPRB, 0.61485E+00_JPRB ,&
&  0.60201E+00_JPRB, 0.61866E+00_JPRB, 0.61101E+00_JPRB, 0.59178E+00_JPRB, 0.57601E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.57805E+00_JPRB, 0.56402E+00_JPRB, 0.55862E+00_JPRB, 0.57163E+00_JPRB, 0.56785E+00_JPRB, 0.53628E+00_JPRB, 0.50076E+00_JPRB ,&
&  0.61205E+00_JPRB, 0.57387E+00_JPRB, 0.63489E+00_JPRB, 0.49504E+00_JPRB, 0.44001E+00_JPRB, 0.42847E+00_JPRB, 0.41089E+00_JPRB ,&
&  0.40091E+00_JPRB, 0.40300E+00_JPRB, 0.41361E+00_JPRB, 0.43300E+00_JPRB, 0.45489E+00_JPRB, 0.46659E+00_JPRB, 0.47749E+00_JPRB ,&
&  0.49018E+00_JPRB, 0.50392E+00_JPRB, 0.52817E+00_JPRB, 0.54644E+00_JPRB, 0.55378E+00_JPRB, 0.55426E+00_JPRB, 0.56077E+00_JPRB ,&
&  0.53103E+00_JPRB, 0.51778E+00_JPRB, 0.52018E+00_JPRB, 0.51558E+00_JPRB, 0.52366E+00_JPRB, 0.53802E+00_JPRB, 0.56465E+00_JPRB ,&
&  0.62188E+00_JPRB, 0.58522E+00_JPRB, 0.69334E+00_JPRB, 0.72329E+00_JPRB, 0.75509E+00_JPRB, 0.78114E+00_JPRB, 0.80257E+00_JPRB ,&
&  0.82674E+00_JPRB, 0.86724E+00_JPRB, 0.91168E+00_JPRB, 0.98137E+00_JPRB, 0.11800E+01_JPRB, 0.13172E+01_JPRB, 0.14083E+01_JPRB ,&
&  0.16747E+01_JPRB, 0.15045E+01_JPRB, 0.17731E+01_JPRB, 0.18666E+01_JPRB, 0.21780E+01_JPRB, 0.24232E+01_JPRB, 0.23463E+01_JPRB ,&
&  0.25006E+01_JPRB, 0.22008E+01_JPRB, 0.19369E+01_JPRB, 0.17474E+01_JPRB, 0.17317E+01_JPRB, 0.17923E+01_JPRB, 0.18919E+01_JPRB ,&
&  0.19432E+01_JPRB, 0.19307E+01_JPRB, 0.20051E+01_JPRB, 0.20699E+01_JPRB, 0.21555E+01_JPRB, 0.22789E+01_JPRB, 0.24050E+01_JPRB ,&
&  0.25073E+01_JPRB, 0.25541E+01_JPRB, 0.25725E+01_JPRB, 0.24013E+01_JPRB, 0.22656E+01_JPRB, 0.21518E+01_JPRB, 0.18730E+01_JPRB ,&
&  0.16566E+01_JPRB, 0.15123E+01_JPRB, 0.13916E+01_JPRB, 0.13028E+01_JPRB, 0.11347E+01_JPRB, 0.98326E+00_JPRB, 0.83539E+00_JPRB ,&
&  0.66471E+00_JPRB, 0.58634E+00_JPRB, 0.58738E+00_JPRB, 0.59194E+00_JPRB, 0.59360E+00_JPRB, 0.60096E+00_JPRB, 0.60801E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.59716E+00_JPRB, 0.61521E+00_JPRB, 0.60800E+00_JPRB, 0.59271E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.63516E+00_JPRB, 0.61280E+00_JPRB, 0.58830E+00_JPRB, 0.57892E+00_JPRB, 0.57875E+00_JPRB, 0.54499E+00_JPRB, 0.48544E+00_JPRB ,&
&  0.50043E+00_JPRB, 0.45945E+00_JPRB, 0.44257E+00_JPRB, 0.42707E+00_JPRB, 0.41493E+00_JPRB, 0.39255E+00_JPRB, 0.37146E+00_JPRB ,&
&  0.37507E+00_JPRB, 0.39744E+00_JPRB, 0.43077E+00_JPRB, 0.45491E+00_JPRB, 0.46591E+00_JPRB, 0.46699E+00_JPRB, 0.46431E+00_JPRB ,&
&  0.46842E+00_JPRB, 0.47820E+00_JPRB, 0.49225E+00_JPRB, 0.51311E+00_JPRB, 0.50894E+00_JPRB, 0.50069E+00_JPRB, 0.49517E+00_JPRB ,&
&  0.48823E+00_JPRB, 0.47846E+00_JPRB, 0.46357E+00_JPRB, 0.46956E+00_JPRB, 0.47778E+00_JPRB, 0.49876E+00_JPRB, 0.52928E+00_JPRB ,&
&  0.44911E+00_JPRB, 0.45799E+00_JPRB, 0.57309E+00_JPRB, 0.60215E+00_JPRB, 0.63434E+00_JPRB, 0.66550E+00_JPRB, 0.69110E+00_JPRB ,&
&  0.71811E+00_JPRB, 0.75960E+00_JPRB, 0.80786E+00_JPRB, 0.86439E+00_JPRB, 0.10193E+01_JPRB, 0.11580E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.13878E+01_JPRB, 0.13034E+01_JPRB, 0.12738E+01_JPRB, 0.14080E+01_JPRB, 0.19489E+01_JPRB, 0.19849E+01_JPRB, 0.19102E+01_JPRB ,&
&  0.18418E+01_JPRB, 0.16796E+01_JPRB, 0.15413E+01_JPRB, 0.14849E+01_JPRB, 0.14433E+01_JPRB, 0.14605E+01_JPRB, 0.15236E+01_JPRB ,&
&  0.14846E+01_JPRB, 0.15239E+01_JPRB, 0.15872E+01_JPRB, 0.16483E+01_JPRB, 0.17278E+01_JPRB, 0.18687E+01_JPRB, 0.20487E+01_JPRB ,&
&  0.22364E+01_JPRB, 0.23867E+01_JPRB, 0.24280E+01_JPRB, 0.23731E+01_JPRB, 0.22709E+01_JPRB, 0.21554E+01_JPRB, 0.19788E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.15329E+01_JPRB, 0.14209E+01_JPRB, 0.13201E+01_JPRB, 0.12293E+01_JPRB, 0.10877E+01_JPRB, 0.88466E+00_JPRB ,&
&  0.67237E+00_JPRB, 0.56774E+00_JPRB, 0.54576E+00_JPRB, 0.54182E+00_JPRB, 0.54728E+00_JPRB, 0.55920E+00_JPRB, 0.56915E+00_JPRB ,&
&  0.57475E+00_JPRB, 0.56407E+00_JPRB, 0.56037E+00_JPRB, 0.58122E+00_JPRB, 0.60837E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.63079E+00_JPRB, 0.62263E+00_JPRB, 0.58406E+00_JPRB, 0.58905E+00_JPRB, 0.56029E+00_JPRB, 0.53708E+00_JPRB, 0.50267E+00_JPRB ,&
&  0.48416E+00_JPRB, 0.45606E+00_JPRB, 0.43680E+00_JPRB, 0.40862E+00_JPRB, 0.39329E+00_JPRB, 0.38912E+00_JPRB, 0.40752E+00_JPRB ,&
&  0.43353E+00_JPRB, 0.45529E+00_JPRB, 0.46887E+00_JPRB, 0.47297E+00_JPRB, 0.47155E+00_JPRB, 0.46313E+00_JPRB, 0.45498E+00_JPRB ,&
&  0.45274E+00_JPRB, 0.45927E+00_JPRB, 0.45824E+00_JPRB, 0.46205E+00_JPRB, 0.46030E+00_JPRB, 0.45044E+00_JPRB, 0.44433E+00_JPRB ,&
&  0.42961E+00_JPRB, 0.41646E+00_JPRB, 0.41612E+00_JPRB, 0.43573E+00_JPRB, 0.45760E+00_JPRB, 0.47675E+00_JPRB, 0.44379E+00_JPRB ,&
&  0.40166E+00_JPRB, 0.39304E+00_JPRB, 0.40894E+00_JPRB, 0.53163E+00_JPRB, 0.57550E+00_JPRB, 0.59635E+00_JPRB, 0.63522E+00_JPRB ,&
&  0.61738E+00_JPRB, 0.62189E+00_JPRB, 0.65896E+00_JPRB, 0.76724E+00_JPRB, 0.84468E+00_JPRB, 0.98518E+00_JPRB, 0.10745E+01_JPRB ,&
&  0.11795E+01_JPRB, 0.13096E+01_JPRB, 0.14369E+01_JPRB, 0.12569E+01_JPRB, 0.13316E+01_JPRB, 0.16068E+01_JPRB, 0.14171E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.12285E+01_JPRB, 0.12540E+01_JPRB, 0.12301E+01_JPRB, 0.11709E+01_JPRB, 0.11532E+01_JPRB, 0.11336E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.10880E+01_JPRB, 0.11783E+01_JPRB, 0.12718E+01_JPRB, 0.13643E+01_JPRB, 0.14964E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.18159E+01_JPRB, 0.19745E+01_JPRB, 0.20732E+01_JPRB, 0.20493E+01_JPRB, 0.20179E+01_JPRB, 0.19422E+01_JPRB, 0.17973E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.14117E+01_JPRB, 0.13063E+01_JPRB, 0.12314E+01_JPRB, 0.11409E+01_JPRB, 0.10054E+01_JPRB, 0.82741E+00_JPRB ,&
&  0.69211E+00_JPRB, 0.60058E+00_JPRB, 0.56223E+00_JPRB, 0.54978E+00_JPRB, 0.55456E+00_JPRB, 0.56129E+00_JPRB, 0.55796E+00_JPRB ,&
&  0.54793E+00_JPRB, 0.53891E+00_JPRB, 0.53539E+00_JPRB, 0.55005E+00_JPRB, 0.60129E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.58810E+00_JPRB, 0.61008E+00_JPRB, 0.63057E+00_JPRB, 0.60936E+00_JPRB, 0.58097E+00_JPRB, 0.52456E+00_JPRB, 0.48409E+00_JPRB ,&
&  0.45800E+00_JPRB, 0.45410E+00_JPRB, 0.45492E+00_JPRB, 0.45984E+00_JPRB, 0.46999E+00_JPRB, 0.48843E+00_JPRB, 0.50197E+00_JPRB ,&
&  0.51281E+00_JPRB, 0.51320E+00_JPRB, 0.51335E+00_JPRB, 0.50845E+00_JPRB, 0.49633E+00_JPRB, 0.47752E+00_JPRB, 0.46287E+00_JPRB ,&
&  0.45758E+00_JPRB, 0.45640E+00_JPRB, 0.45046E+00_JPRB, 0.43890E+00_JPRB, 0.43055E+00_JPRB, 0.41814E+00_JPRB, 0.42104E+00_JPRB ,&
&  0.41335E+00_JPRB, 0.40687E+00_JPRB, 0.41722E+00_JPRB, 0.43290E+00_JPRB, 0.44702E+00_JPRB, 0.46417E+00_JPRB, 0.42301E+00_JPRB ,&
&  0.37880E+00_JPRB, 0.35777E+00_JPRB, 0.34902E+00_JPRB, 0.35587E+00_JPRB, 0.40724E+00_JPRB, 0.53511E+00_JPRB, 0.58673E+00_JPRB ,&
&  0.61623E+00_JPRB, 0.63794E+00_JPRB, 0.66404E+00_JPRB, 0.69703E+00_JPRB, 0.71963E+00_JPRB, 0.79146E+00_JPRB, 0.89800E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10993E+01_JPRB, 0.11887E+01_JPRB, 0.10356E+01_JPRB, 0.94072E+00_JPRB, 0.96336E+00_JPRB, 0.10105E+01_JPRB ,&
&  0.10702E+01_JPRB, 0.10218E+01_JPRB, 0.10137E+01_JPRB, 0.10445E+01_JPRB, 0.10196E+01_JPRB, 0.99189E+00_JPRB, 0.95390E+00_JPRB ,&
&  0.92255E+00_JPRB, 0.93186E+00_JPRB, 0.10050E+01_JPRB, 0.10382E+01_JPRB, 0.11108E+01_JPRB, 0.12250E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.14508E+01_JPRB, 0.15456E+01_JPRB, 0.15924E+01_JPRB, 0.15322E+01_JPRB, 0.15019E+01_JPRB, 0.15110E+01_JPRB, 0.14868E+01_JPRB ,&
&  0.13621E+01_JPRB, 0.12643E+01_JPRB, 0.11828E+01_JPRB, 0.11135E+01_JPRB, 0.10168E+01_JPRB, 0.92855E+00_JPRB, 0.83105E+00_JPRB ,&
&  0.73522E+00_JPRB, 0.66536E+00_JPRB, 0.63067E+00_JPRB, 0.61091E+00_JPRB, 0.60855E+00_JPRB, 0.61175E+00_JPRB, 0.60556E+00_JPRB ,&
&  0.58950E+00_JPRB, 0.57078E+00_JPRB, 0.56189E+00_JPRB, 0.56539E+00_JPRB, 0.58289E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.63294E+00_JPRB, 0.63270E+00_JPRB, 0.63000E+00_JPRB, 0.62220E+00_JPRB, 0.60223E+00_JPRB, 0.57040E+00_JPRB, 0.55349E+00_JPRB ,&
&  0.54472E+00_JPRB, 0.54793E+00_JPRB, 0.54715E+00_JPRB, 0.54558E+00_JPRB, 0.54818E+00_JPRB, 0.55719E+00_JPRB, 0.56643E+00_JPRB ,&
&  0.56980E+00_JPRB, 0.56214E+00_JPRB, 0.54674E+00_JPRB, 0.51827E+00_JPRB, 0.49485E+00_JPRB, 0.47736E+00_JPRB, 0.46870E+00_JPRB ,&
&  0.46726E+00_JPRB, 0.45864E+00_JPRB, 0.44272E+00_JPRB, 0.42117E+00_JPRB, 0.40397E+00_JPRB, 0.38988E+00_JPRB, 0.39229E+00_JPRB ,&
&  0.40435E+00_JPRB, 0.40937E+00_JPRB, 0.41704E+00_JPRB, 0.42479E+00_JPRB, 0.43076E+00_JPRB, 0.44733E+00_JPRB, 0.39254E+00_JPRB ,&
&  0.35411E+00_JPRB, 0.33589E+00_JPRB, 0.32292E+00_JPRB, 0.32057E+00_JPRB, 0.33088E+00_JPRB, 0.35801E+00_JPRB, 0.41812E+00_JPRB ,&
&  0.54788E+00_JPRB, 0.58532E+00_JPRB, 0.60252E+00_JPRB, 0.62401E+00_JPRB, 0.64043E+00_JPRB, 0.65492E+00_JPRB, 0.70516E+00_JPRB ,&
&  0.77035E+00_JPRB, 0.85174E+00_JPRB, 0.92051E+00_JPRB, 0.93739E+00_JPRB, 0.93869E+00_JPRB, 0.91191E+00_JPRB, 0.98828E+00_JPRB ,&
&  0.12560E+01_JPRB, 0.11398E+01_JPRB, 0.10012E+01_JPRB, 0.91450E+00_JPRB, 0.85435E+00_JPRB, 0.81295E+00_JPRB, 0.82396E+00_JPRB ,&
&  0.84836E+00_JPRB, 0.86950E+00_JPRB, 0.91049E+00_JPRB, 0.93906E+00_JPRB, 0.97676E+00_JPRB, 0.10391E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.13589E+01_JPRB, 0.16100E+01_JPRB, 0.14358E+01_JPRB, 0.13459E+01_JPRB, 0.13218E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.12644E+01_JPRB, 0.12518E+01_JPRB, 0.11528E+01_JPRB, 0.10276E+01_JPRB, 0.96355E+00_JPRB, 0.88321E+00_JPRB, 0.81618E+00_JPRB ,&
&  0.75388E+00_JPRB, 0.70357E+00_JPRB, 0.67722E+00_JPRB, 0.66072E+00_JPRB, 0.66302E+00_JPRB, 0.66585E+00_JPRB, 0.66361E+00_JPRB ,&
&  0.65600E+00_JPRB, 0.64767E+00_JPRB, 0.64052E+00_JPRB, 0.63509E+00_JPRB, 0.63184E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.65540E+00_JPRB, 0.64790E+00_JPRB, 0.63851E+00_JPRB, 0.63050E+00_JPRB, 0.61902E+00_JPRB, 0.61081E+00_JPRB, 0.60392E+00_JPRB ,&
&  0.59954E+00_JPRB, 0.59786E+00_JPRB, 0.59559E+00_JPRB, 0.59343E+00_JPRB, 0.58949E+00_JPRB, 0.58519E+00_JPRB, 0.57758E+00_JPRB ,&
&  0.56431E+00_JPRB, 0.52657E+00_JPRB, 0.49832E+00_JPRB, 0.48444E+00_JPRB, 0.47320E+00_JPRB, 0.46699E+00_JPRB, 0.46114E+00_JPRB ,&
&  0.44968E+00_JPRB, 0.43688E+00_JPRB, 0.41864E+00_JPRB, 0.39410E+00_JPRB, 0.38200E+00_JPRB, 0.38537E+00_JPRB, 0.39547E+00_JPRB ,&
&  0.41067E+00_JPRB, 0.41874E+00_JPRB, 0.41249E+00_JPRB, 0.41018E+00_JPRB, 0.42213E+00_JPRB, 0.41477E+00_JPRB, 0.36157E+00_JPRB ,&
&  0.33424E+00_JPRB, 0.31596E+00_JPRB, 0.31097E+00_JPRB, 0.30974E+00_JPRB, 0.31593E+00_JPRB, 0.33517E+00_JPRB, 0.38325E+00_JPRB ,&
&  0.46595E+00_JPRB, 0.56890E+00_JPRB, 0.57811E+00_JPRB, 0.58425E+00_JPRB, 0.59370E+00_JPRB, 0.60344E+00_JPRB, 0.62064E+00_JPRB ,&
&  0.64823E+00_JPRB, 0.68275E+00_JPRB, 0.71363E+00_JPRB, 0.74745E+00_JPRB, 0.77757E+00_JPRB, 0.79052E+00_JPRB, 0.79728E+00_JPRB ,&
&  0.81701E+00_JPRB, 0.79982E+00_JPRB, 0.79677E+00_JPRB, 0.79577E+00_JPRB, 0.79928E+00_JPRB, 0.77616E+00_JPRB, 0.77886E+00_JPRB ,&
&  0.81132E+00_JPRB, 0.84648E+00_JPRB, 0.88430E+00_JPRB, 0.90946E+00_JPRB, 0.94066E+00_JPRB, 0.99396E+00_JPRB, 0.10484E+01_JPRB ,&
&  0.10951E+01_JPRB, 0.11621E+01_JPRB, 0.12608E+01_JPRB, 0.12483E+01_JPRB, 0.12312E+01_JPRB, 0.12195E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.12032E+01_JPRB, 0.12108E+01_JPRB, 0.11529E+01_JPRB, 0.10646E+01_JPRB, 0.98175E+00_JPRB, 0.88374E+00_JPRB, 0.82009E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.71636E+00_JPRB, 0.68901E+00_JPRB, 0.67791E+00_JPRB, 0.67779E+00_JPRB, 0.67640E+00_JPRB, 0.67236E+00_JPRB ,&
&  0.67269E+00_JPRB, 0.67387E+00_JPRB, 0.67416E+00_JPRB, 0.67014E+00_JPRB, 0.66352E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.64465E+00_JPRB, 0.63552E+00_JPRB, 0.62820E+00_JPRB, 0.62225E+00_JPRB, 0.61366E+00_JPRB, 0.60495E+00_JPRB, 0.59483E+00_JPRB ,&
&  0.58675E+00_JPRB, 0.57980E+00_JPRB, 0.57320E+00_JPRB, 0.56460E+00_JPRB, 0.55541E+00_JPRB, 0.54473E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.52594E+00_JPRB, 0.51442E+00_JPRB, 0.49710E+00_JPRB, 0.48102E+00_JPRB, 0.46911E+00_JPRB, 0.46123E+00_JPRB, 0.45198E+00_JPRB ,&
&  0.44466E+00_JPRB, 0.43200E+00_JPRB, 0.40839E+00_JPRB, 0.38912E+00_JPRB, 0.37241E+00_JPRB, 0.37067E+00_JPRB, 0.39914E+00_JPRB ,&
&  0.41201E+00_JPRB, 0.41366E+00_JPRB, 0.41344E+00_JPRB, 0.40603E+00_JPRB, 0.39795E+00_JPRB, 0.34470E+00_JPRB, 0.34190E+00_JPRB ,&
&  0.32606E+00_JPRB, 0.31446E+00_JPRB, 0.31220E+00_JPRB, 0.31288E+00_JPRB, 0.31846E+00_JPRB, 0.33306E+00_JPRB, 0.35373E+00_JPRB ,&
&  0.39148E+00_JPRB, 0.50938E+00_JPRB, 0.53770E+00_JPRB, 0.56064E+00_JPRB, 0.56612E+00_JPRB, 0.57011E+00_JPRB, 0.57767E+00_JPRB ,&
&  0.58884E+00_JPRB, 0.60496E+00_JPRB, 0.61736E+00_JPRB, 0.63413E+00_JPRB, 0.65115E+00_JPRB, 0.66286E+00_JPRB, 0.68706E+00_JPRB ,&
&  0.70437E+00_JPRB, 0.71695E+00_JPRB, 0.72520E+00_JPRB, 0.73100E+00_JPRB, 0.74637E+00_JPRB, 0.74048E+00_JPRB, 0.73571E+00_JPRB ,&
&  0.76931E+00_JPRB, 0.82219E+00_JPRB, 0.86685E+00_JPRB, 0.91014E+00_JPRB, 0.93602E+00_JPRB, 0.96842E+00_JPRB, 0.10126E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10853E+01_JPRB, 0.11397E+01_JPRB, 0.11486E+01_JPRB, 0.11411E+01_JPRB, 0.11303E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11266E+01_JPRB, 0.11276E+01_JPRB, 0.10901E+01_JPRB, 0.10606E+01_JPRB, 0.10094E+01_JPRB, 0.94536E+00_JPRB, 0.89359E+00_JPRB ,&
&  0.83413E+00_JPRB, 0.78525E+00_JPRB, 0.75795E+00_JPRB, 0.74204E+00_JPRB, 0.73338E+00_JPRB, 0.72888E+00_JPRB, 0.72233E+00_JPRB ,&
&  0.71576E+00_JPRB, 0.70350E+00_JPRB, 0.68792E+00_JPRB, 0.67134E+00_JPRB, 0.65624E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.62610E+00_JPRB, 0.61658E+00_JPRB, 0.60798E+00_JPRB, 0.59673E+00_JPRB, 0.58517E+00_JPRB, 0.57501E+00_JPRB, 0.56708E+00_JPRB ,&
&  0.56109E+00_JPRB, 0.55442E+00_JPRB, 0.54822E+00_JPRB, 0.54037E+00_JPRB, 0.53301E+00_JPRB, 0.52628E+00_JPRB, 0.52044E+00_JPRB ,&
&  0.51547E+00_JPRB, 0.50643E+00_JPRB, 0.49677E+00_JPRB, 0.48534E+00_JPRB, 0.47504E+00_JPRB, 0.46541E+00_JPRB, 0.45415E+00_JPRB ,&
&  0.44520E+00_JPRB, 0.42837E+00_JPRB, 0.40751E+00_JPRB, 0.39661E+00_JPRB, 0.37706E+00_JPRB, 0.36792E+00_JPRB, 0.36552E+00_JPRB ,&
&  0.38322E+00_JPRB, 0.37218E+00_JPRB, 0.37755E+00_JPRB, 0.36257E+00_JPRB, 0.34954E+00_JPRB, 0.33749E+00_JPRB, 0.32897E+00_JPRB ,&
&  0.32262E+00_JPRB, 0.31696E+00_JPRB, 0.31713E+00_JPRB, 0.32056E+00_JPRB, 0.32389E+00_JPRB, 0.33539E+00_JPRB, 0.35162E+00_JPRB ,&
&  0.38996E+00_JPRB, 0.49221E+00_JPRB, 0.51336E+00_JPRB, 0.53589E+00_JPRB, 0.54086E+00_JPRB, 0.54429E+00_JPRB, 0.54855E+00_JPRB ,&
&  0.55572E+00_JPRB, 0.56254E+00_JPRB, 0.56862E+00_JPRB, 0.57066E+00_JPRB, 0.59149E+00_JPRB, 0.60855E+00_JPRB, 0.62944E+00_JPRB ,&
&  0.64057E+00_JPRB, 0.64919E+00_JPRB, 0.65523E+00_JPRB, 0.66898E+00_JPRB, 0.69590E+00_JPRB, 0.70463E+00_JPRB, 0.70298E+00_JPRB ,&
&  0.72010E+00_JPRB, 0.74933E+00_JPRB, 0.79818E+00_JPRB, 0.85998E+00_JPRB, 0.89838E+00_JPRB, 0.93153E+00_JPRB, 0.98506E+00_JPRB ,&
&  0.10200E+01_JPRB, 0.10808E+01_JPRB, 0.11400E+01_JPRB, 0.11590E+01_JPRB, 0.11504E+01_JPRB, 0.11263E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10543E+01_JPRB, 0.10319E+01_JPRB, 0.10161E+01_JPRB, 0.99121E+00_JPRB, 0.96163E+00_JPRB, 0.93564E+00_JPRB ,&
&  0.90345E+00_JPRB, 0.87612E+00_JPRB, 0.85132E+00_JPRB, 0.82404E+00_JPRB, 0.79583E+00_JPRB, 0.76768E+00_JPRB, 0.73962E+00_JPRB ,&
&  0.71203E+00_JPRB, 0.68792E+00_JPRB, 0.66454E+00_JPRB, 0.64843E+00_JPRB, 0.63421E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.60531E+00_JPRB, 0.59194E+00_JPRB, 0.57810E+00_JPRB, 0.56583E+00_JPRB, 0.55605E+00_JPRB, 0.54901E+00_JPRB, 0.54478E+00_JPRB ,&
&  0.54168E+00_JPRB, 0.53746E+00_JPRB, 0.53276E+00_JPRB, 0.52744E+00_JPRB, 0.52226E+00_JPRB, 0.51994E+00_JPRB, 0.51769E+00_JPRB ,&
&  0.51586E+00_JPRB, 0.51094E+00_JPRB, 0.50372E+00_JPRB, 0.49399E+00_JPRB, 0.48323E+00_JPRB, 0.47423E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.45001E+00_JPRB, 0.43686E+00_JPRB, 0.41420E+00_JPRB, 0.40089E+00_JPRB, 0.37944E+00_JPRB, 0.36557E+00_JPRB, 0.35846E+00_JPRB ,&
&  0.35382E+00_JPRB, 0.35487E+00_JPRB, 0.35906E+00_JPRB, 0.36360E+00_JPRB, 0.36036E+00_JPRB, 0.35447E+00_JPRB, 0.35197E+00_JPRB ,&
&  0.34611E+00_JPRB, 0.34308E+00_JPRB, 0.34095E+00_JPRB, 0.34149E+00_JPRB, 0.34478E+00_JPRB, 0.35301E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.40319E+00_JPRB, 0.45067E+00_JPRB, 0.49664E+00_JPRB, 0.52146E+00_JPRB, 0.52310E+00_JPRB, 0.52973E+00_JPRB, 0.53228E+00_JPRB ,&
&  0.53730E+00_JPRB, 0.54138E+00_JPRB, 0.54807E+00_JPRB, 0.55604E+00_JPRB, 0.56802E+00_JPRB, 0.57900E+00_JPRB, 0.59130E+00_JPRB ,&
&  0.59691E+00_JPRB, 0.60247E+00_JPRB, 0.60312E+00_JPRB, 0.61265E+00_JPRB, 0.63054E+00_JPRB, 0.64626E+00_JPRB, 0.65182E+00_JPRB ,&
&  0.66391E+00_JPRB, 0.68262E+00_JPRB, 0.71975E+00_JPRB, 0.77689E+00_JPRB, 0.83636E+00_JPRB, 0.89128E+00_JPRB, 0.94286E+00_JPRB ,&
&  0.99276E+00_JPRB, 0.10521E+01_JPRB, 0.11056E+01_JPRB, 0.11209E+01_JPRB, 0.11143E+01_JPRB, 0.11144E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.10716E+01_JPRB, 0.10476E+01_JPRB, 0.10306E+01_JPRB, 0.10157E+01_JPRB, 0.99698E+00_JPRB, 0.96862E+00_JPRB, 0.93966E+00_JPRB ,&
&  0.90767E+00_JPRB, 0.87319E+00_JPRB, 0.83874E+00_JPRB, 0.80442E+00_JPRB, 0.77210E+00_JPRB, 0.74116E+00_JPRB, 0.71426E+00_JPRB ,&
&  0.68858E+00_JPRB, 0.66766E+00_JPRB, 0.64832E+00_JPRB, 0.63239E+00_JPRB, 0.61870E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.58953E+00_JPRB, 0.57759E+00_JPRB, 0.56675E+00_JPRB, 0.55738E+00_JPRB, 0.54944E+00_JPRB, 0.54365E+00_JPRB, 0.53946E+00_JPRB ,&
&  0.53630E+00_JPRB, 0.53516E+00_JPRB, 0.53387E+00_JPRB, 0.53216E+00_JPRB, 0.53011E+00_JPRB, 0.53008E+00_JPRB, 0.52896E+00_JPRB ,&
&  0.52683E+00_JPRB, 0.52344E+00_JPRB, 0.52014E+00_JPRB, 0.51549E+00_JPRB, 0.50729E+00_JPRB, 0.49740E+00_JPRB, 0.48379E+00_JPRB ,&
&  0.47519E+00_JPRB, 0.46379E+00_JPRB, 0.45048E+00_JPRB, 0.43592E+00_JPRB, 0.42623E+00_JPRB, 0.40409E+00_JPRB, 0.39558E+00_JPRB ,&
&  0.38987E+00_JPRB, 0.38061E+00_JPRB, 0.37591E+00_JPRB, 0.37531E+00_JPRB, 0.39174E+00_JPRB, 0.40482E+00_JPRB, 0.41157E+00_JPRB ,&
&  0.39871E+00_JPRB, 0.40185E+00_JPRB, 0.39872E+00_JPRB, 0.39733E+00_JPRB, 0.40010E+00_JPRB, 0.40719E+00_JPRB, 0.43259E+00_JPRB ,&
&  0.47755E+00_JPRB, 0.49629E+00_JPRB, 0.50778E+00_JPRB, 0.51883E+00_JPRB, 0.52534E+00_JPRB, 0.53021E+00_JPRB, 0.53418E+00_JPRB ,&
&  0.54228E+00_JPRB, 0.54970E+00_JPRB, 0.55629E+00_JPRB, 0.56003E+00_JPRB, 0.56462E+00_JPRB, 0.56941E+00_JPRB, 0.57420E+00_JPRB ,&
&  0.57104E+00_JPRB, 0.57018E+00_JPRB, 0.57149E+00_JPRB, 0.57664E+00_JPRB, 0.58432E+00_JPRB, 0.59704E+00_JPRB, 0.61153E+00_JPRB ,&
&  0.62793E+00_JPRB, 0.64063E+00_JPRB, 0.66243E+00_JPRB, 0.69292E+00_JPRB, 0.72898E+00_JPRB, 0.77310E+00_JPRB, 0.81819E+00_JPRB ,&
&  0.86131E+00_JPRB, 0.89663E+00_JPRB, 0.92846E+00_JPRB, 0.94160E+00_JPRB, 0.94673E+00_JPRB, 0.94938E+00_JPRB, 0.94477E+00_JPRB ,&
&  0.93692E+00_JPRB, 0.92481E+00_JPRB, 0.91271E+00_JPRB, 0.89714E+00_JPRB, 0.88001E+00_JPRB, 0.86058E+00_JPRB, 0.84000E+00_JPRB ,&
&  0.81554E+00_JPRB, 0.79150E+00_JPRB, 0.76753E+00_JPRB, 0.74431E+00_JPRB, 0.72152E+00_JPRB, 0.70053E+00_JPRB, 0.68160E+00_JPRB ,&
&  0.66430E+00_JPRB, 0.64715E+00_JPRB, 0.63188E+00_JPRB, 0.61755E+00_JPRB, 0.60345E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.57073E+00_JPRB, 0.56375E+00_JPRB, 0.55777E+00_JPRB, 0.55243E+00_JPRB, 0.54844E+00_JPRB, 0.54580E+00_JPRB, 0.54371E+00_JPRB ,&
&  0.54197E+00_JPRB, 0.54158E+00_JPRB, 0.54156E+00_JPRB, 0.54148E+00_JPRB, 0.54139E+00_JPRB, 0.54116E+00_JPRB, 0.54051E+00_JPRB ,&
&  0.53981E+00_JPRB, 0.53966E+00_JPRB, 0.53924E+00_JPRB, 0.53868E+00_JPRB, 0.53681E+00_JPRB, 0.53323E+00_JPRB, 0.52850E+00_JPRB ,&
&  0.52325E+00_JPRB, 0.51634E+00_JPRB, 0.50776E+00_JPRB, 0.49822E+00_JPRB, 0.49284E+00_JPRB, 0.48657E+00_JPRB, 0.48059E+00_JPRB ,&
&  0.47598E+00_JPRB, 0.47531E+00_JPRB, 0.47440E+00_JPRB, 0.47349E+00_JPRB, 0.47291E+00_JPRB, 0.47396E+00_JPRB, 0.47450E+00_JPRB ,&
&  0.47514E+00_JPRB, 0.47754E+00_JPRB, 0.48447E+00_JPRB, 0.49180E+00_JPRB, 0.50013E+00_JPRB, 0.50993E+00_JPRB, 0.52109E+00_JPRB ,&
&  0.52948E+00_JPRB, 0.53409E+00_JPRB, 0.53885E+00_JPRB, 0.54362E+00_JPRB, 0.54655E+00_JPRB, 0.54843E+00_JPRB, 0.55016E+00_JPRB ,&
&  0.55282E+00_JPRB, 0.55444E+00_JPRB, 0.55653E+00_JPRB, 0.55874E+00_JPRB, 0.55885E+00_JPRB, 0.55844E+00_JPRB, 0.55743E+00_JPRB ,&
&  0.55522E+00_JPRB, 0.55495E+00_JPRB, 0.55630E+00_JPRB, 0.55778E+00_JPRB, 0.56083E+00_JPRB, 0.56607E+00_JPRB, 0.57257E+00_JPRB ,&
&  0.58113E+00_JPRB, 0.59129E+00_JPRB, 0.60543E+00_JPRB, 0.62163E+00_JPRB, 0.64038E+00_JPRB, 0.66306E+00_JPRB, 0.68162E+00_JPRB ,&
&  0.69870E+00_JPRB, 0.71347E+00_JPRB, 0.72644E+00_JPRB, 0.73512E+00_JPRB, 0.74087E+00_JPRB, 0.74614E+00_JPRB, 0.74508E+00_JPRB ,&
&  0.74251E+00_JPRB, 0.73878E+00_JPRB, 0.73512E+00_JPRB, 0.72762E+00_JPRB, 0.71871E+00_JPRB, 0.70967E+00_JPRB, 0.70120E+00_JPRB ,&
&  0.68965E+00_JPRB, 0.67798E+00_JPRB, 0.66715E+00_JPRB, 0.65730E+00_JPRB, 0.64677E+00_JPRB, 0.63685E+00_JPRB, 0.62691E+00_JPRB ,&
&  0.61694E+00_JPRB, 0.60672E+00_JPRB, 0.59736E+00_JPRB, 0.58834E+00_JPRB, 0.57941E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.56310E+00_JPRB, 0.56181E+00_JPRB, 0.56075E+00_JPRB, 0.55944E+00_JPRB, 0.55782E+00_JPRB, 0.55674E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.55498E+00_JPRB, 0.55461E+00_JPRB, 0.55467E+00_JPRB, 0.55491E+00_JPRB, 0.55556E+00_JPRB, 0.55615E+00_JPRB, 0.55689E+00_JPRB ,&
&  0.55764E+00_JPRB, 0.55842E+00_JPRB, 0.55936E+00_JPRB, 0.56005E+00_JPRB, 0.56033E+00_JPRB, 0.56025E+00_JPRB, 0.55980E+00_JPRB ,&
&  0.55876E+00_JPRB, 0.55762E+00_JPRB, 0.55655E+00_JPRB, 0.55547E+00_JPRB, 0.55393E+00_JPRB, 0.55212E+00_JPRB, 0.55010E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.54711E+00_JPRB, 0.54611E+00_JPRB, 0.54500E+00_JPRB, 0.54435E+00_JPRB, 0.54487E+00_JPRB, 0.54537E+00_JPRB ,&
&  0.54594E+00_JPRB, 0.54682E+00_JPRB, 0.54877E+00_JPRB, 0.55069E+00_JPRB, 0.55246E+00_JPRB, 0.55425E+00_JPRB, 0.55605E+00_JPRB ,&
&  0.55698E+00_JPRB, 0.55755E+00_JPRB, 0.55785E+00_JPRB, 0.55798E+00_JPRB, 0.55748E+00_JPRB, 0.55709E+00_JPRB, 0.55669E+00_JPRB ,&
&  0.55612E+00_JPRB, 0.55564E+00_JPRB, 0.55502E+00_JPRB, 0.55388E+00_JPRB, 0.55109E+00_JPRB, 0.54917E+00_JPRB, 0.54854E+00_JPRB ,&
&  0.54895E+00_JPRB, 0.54949E+00_JPRB, 0.55014E+00_JPRB, 0.55089E+00_JPRB, 0.55226E+00_JPRB, 0.55385E+00_JPRB, 0.55584E+00_JPRB ,&
&  0.55786E+00_JPRB, 0.56034E+00_JPRB, 0.56398E+00_JPRB, 0.56784E+00_JPRB, 0.57219E+00_JPRB, 0.57702E+00_JPRB, 0.58128E+00_JPRB ,&
&  0.58584E+00_JPRB, 0.59074E+00_JPRB, 0.59487E+00_JPRB, 0.59878E+00_JPRB, 0.60280E+00_JPRB, 0.60668E+00_JPRB, 0.60932E+00_JPRB ,&
&  0.61034E+00_JPRB, 0.61108E+00_JPRB, 0.61152E+00_JPRB, 0.61037E+00_JPRB, 0.60815E+00_JPRB, 0.60614E+00_JPRB, 0.60395E+00_JPRB ,&
&  0.60146E+00_JPRB, 0.59853E+00_JPRB, 0.59576E+00_JPRB, 0.59274E+00_JPRB, 0.58913E+00_JPRB, 0.58550E+00_JPRB, 0.58201E+00_JPRB ,&
&  0.57847E+00_JPRB, 0.57477E+00_JPRB, 0.57156E+00_JPRB, 0.56850E+00_JPRB, 0.56564E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB, 0.57059E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB, 0.12082E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.12647E+00_JPRB, 0.12633E+00_JPRB, 0.12618E+00_JPRB, 0.12601E+00_JPRB, 0.12544E+00_JPRB, 0.12522E+00_JPRB, 0.12499E+00_JPRB ,&
&  0.12488E+00_JPRB, 0.12481E+00_JPRB, 0.12488E+00_JPRB, 0.12499E+00_JPRB, 0.12511E+00_JPRB, 0.12520E+00_JPRB, 0.12527E+00_JPRB ,&
&  0.12524E+00_JPRB, 0.12522E+00_JPRB, 0.12465E+00_JPRB, 0.12423E+00_JPRB, 0.12398E+00_JPRB, 0.12380E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12333E+00_JPRB, 0.12306E+00_JPRB, 0.12274E+00_JPRB, 0.12250E+00_JPRB, 0.12250E+00_JPRB, 0.12237E+00_JPRB, 0.12236E+00_JPRB ,&
&  0.12232E+00_JPRB, 0.12243E+00_JPRB, 0.12245E+00_JPRB, 0.12249E+00_JPRB, 0.12274E+00_JPRB, 0.12295E+00_JPRB, 0.12296E+00_JPRB ,&
&  0.12294E+00_JPRB, 0.12274E+00_JPRB, 0.12256E+00_JPRB, 0.12243E+00_JPRB, 0.12235E+00_JPRB, 0.12232E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12213E+00_JPRB, 0.12193E+00_JPRB, 0.12170E+00_JPRB, 0.12143E+00_JPRB, 0.12119E+00_JPRB, 0.12104E+00_JPRB, 0.12098E+00_JPRB ,&
&  0.12093E+00_JPRB, 0.12079E+00_JPRB, 0.12059E+00_JPRB, 0.12031E+00_JPRB, 0.12005E+00_JPRB, 0.11980E+00_JPRB, 0.11956E+00_JPRB ,&
&  0.11934E+00_JPRB, 0.11924E+00_JPRB, 0.11912E+00_JPRB, 0.11901E+00_JPRB, 0.11894E+00_JPRB, 0.11884E+00_JPRB, 0.11874E+00_JPRB ,&
&  0.11866E+00_JPRB, 0.11858E+00_JPRB, 0.11851E+00_JPRB, 0.11845E+00_JPRB, 0.11841E+00_JPRB, 0.11838E+00_JPRB, 0.11839E+00_JPRB ,&
&  0.11842E+00_JPRB, 0.11844E+00_JPRB, 0.11849E+00_JPRB, 0.11862E+00_JPRB, 0.11877E+00_JPRB, 0.11896E+00_JPRB, 0.11917E+00_JPRB ,&
&  0.11941E+00_JPRB, 0.11968E+00_JPRB, 0.11998E+00_JPRB, 0.12020E+00_JPRB, 0.12054E+00_JPRB, 0.12087E+00_JPRB, 0.12121E+00_JPRB ,&
&  0.12156E+00_JPRB, 0.12199E+00_JPRB, 0.12247E+00_JPRB, 0.12297E+00_JPRB, 0.12351E+00_JPRB, 0.12383E+00_JPRB, 0.12421E+00_JPRB ,&
&  0.12464E+00_JPRB, 0.12518E+00_JPRB, 0.12549E+00_JPRB, 0.12572E+00_JPRB, 0.12600E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.13637E+00_JPRB, 0.13835E+00_JPRB, 0.13985E+00_JPRB, 0.14170E+00_JPRB, 0.14163E+00_JPRB, 0.15304E+00_JPRB, 0.15116E+00_JPRB ,&
&  0.15006E+00_JPRB, 0.14514E+00_JPRB, 0.14463E+00_JPRB, 0.14299E+00_JPRB, 0.14106E+00_JPRB, 0.13895E+00_JPRB, 0.13701E+00_JPRB ,&
&  0.13523E+00_JPRB, 0.13326E+00_JPRB, 0.13181E+00_JPRB, 0.13061E+00_JPRB, 0.12953E+00_JPRB, 0.12854E+00_JPRB, 0.12774E+00_JPRB ,&
&  0.12766E+00_JPRB, 0.12813E+00_JPRB, 0.12905E+00_JPRB, 0.12894E+00_JPRB, 0.12870E+00_JPRB, 0.12836E+00_JPRB, 0.12820E+00_JPRB ,&
&  0.12770E+00_JPRB, 0.12734E+00_JPRB, 0.12702E+00_JPRB, 0.12652E+00_JPRB, 0.12698E+00_JPRB, 0.12739E+00_JPRB, 0.12782E+00_JPRB ,&
&  0.12823E+00_JPRB, 0.12862E+00_JPRB, 0.12853E+00_JPRB, 0.12836E+00_JPRB, 0.12796E+00_JPRB, 0.12741E+00_JPRB, 0.12659E+00_JPRB ,&
&  0.12579E+00_JPRB, 0.12497E+00_JPRB, 0.12416E+00_JPRB, 0.12358E+00_JPRB, 0.12306E+00_JPRB, 0.12257E+00_JPRB, 0.12216E+00_JPRB ,&
&  0.12195E+00_JPRB, 0.12181E+00_JPRB, 0.12165E+00_JPRB, 0.12139E+00_JPRB, 0.12106E+00_JPRB, 0.12072E+00_JPRB, 0.12042E+00_JPRB ,&
&  0.12019E+00_JPRB, 0.11978E+00_JPRB, 0.11940E+00_JPRB, 0.11900E+00_JPRB, 0.11870E+00_JPRB, 0.11832E+00_JPRB, 0.11801E+00_JPRB ,&
&  0.11772E+00_JPRB, 0.11750E+00_JPRB, 0.11745E+00_JPRB, 0.11742E+00_JPRB, 0.11741E+00_JPRB, 0.11745E+00_JPRB, 0.11757E+00_JPRB ,&
&  0.11774E+00_JPRB, 0.11792E+00_JPRB, 0.11816E+00_JPRB, 0.11834E+00_JPRB, 0.11851E+00_JPRB, 0.11866E+00_JPRB, 0.11901E+00_JPRB ,&
&  0.11944E+00_JPRB, 0.11990E+00_JPRB, 0.12034E+00_JPRB, 0.12059E+00_JPRB, 0.12132E+00_JPRB, 0.12208E+00_JPRB, 0.12290E+00_JPRB ,&
&  0.12377E+00_JPRB, 0.12487E+00_JPRB, 0.12606E+00_JPRB, 0.12732E+00_JPRB, 0.12876E+00_JPRB, 0.12976E+00_JPRB, 0.13082E+00_JPRB ,&
&  0.13222E+00_JPRB, 0.13451E+00_JPRB, 0.13543E+00_JPRB, 0.13586E+00_JPRB, 0.13620E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.16023E+00_JPRB, 0.16085E+00_JPRB, 0.16140E+00_JPRB, 0.16162E+00_JPRB, 0.16166E+00_JPRB, 0.16106E+00_JPRB, 0.16032E+00_JPRB ,&
&  0.15686E+00_JPRB, 0.15428E+00_JPRB, 0.15150E+00_JPRB, 0.14874E+00_JPRB, 0.14633E+00_JPRB, 0.14403E+00_JPRB, 0.14190E+00_JPRB ,&
&  0.13996E+00_JPRB, 0.13854E+00_JPRB, 0.13716E+00_JPRB, 0.13522E+00_JPRB, 0.13275E+00_JPRB, 0.13049E+00_JPRB, 0.12895E+00_JPRB ,&
&  0.12845E+00_JPRB, 0.12866E+00_JPRB, 0.12988E+00_JPRB, 0.13121E+00_JPRB, 0.13264E+00_JPRB, 0.13373E+00_JPRB, 0.13481E+00_JPRB ,&
&  0.13424E+00_JPRB, 0.13385E+00_JPRB, 0.13351E+00_JPRB, 0.13296E+00_JPRB, 0.13340E+00_JPRB, 0.13400E+00_JPRB, 0.13432E+00_JPRB ,&
&  0.13469E+00_JPRB, 0.13576E+00_JPRB, 0.13648E+00_JPRB, 0.13716E+00_JPRB, 0.13695E+00_JPRB, 0.13611E+00_JPRB, 0.13468E+00_JPRB ,&
&  0.13298E+00_JPRB, 0.13126E+00_JPRB, 0.12976E+00_JPRB, 0.12848E+00_JPRB, 0.12750E+00_JPRB, 0.12645E+00_JPRB, 0.12559E+00_JPRB ,&
&  0.12516E+00_JPRB, 0.12478E+00_JPRB, 0.12421E+00_JPRB, 0.12363E+00_JPRB, 0.12274E+00_JPRB, 0.12197E+00_JPRB, 0.12116E+00_JPRB ,&
&  0.12042E+00_JPRB, 0.11971E+00_JPRB, 0.11906E+00_JPRB, 0.11840E+00_JPRB, 0.11794E+00_JPRB, 0.11752E+00_JPRB, 0.11712E+00_JPRB ,&
&  0.11669E+00_JPRB, 0.11653E+00_JPRB, 0.11622E+00_JPRB, 0.11612E+00_JPRB, 0.11595E+00_JPRB, 0.11584E+00_JPRB, 0.11600E+00_JPRB ,&
&  0.11646E+00_JPRB, 0.11693E+00_JPRB, 0.11719E+00_JPRB, 0.11748E+00_JPRB, 0.11777E+00_JPRB, 0.11814E+00_JPRB, 0.11873E+00_JPRB ,&
&  0.11919E+00_JPRB, 0.11978E+00_JPRB, 0.12029E+00_JPRB, 0.12088E+00_JPRB, 0.12197E+00_JPRB, 0.12310E+00_JPRB, 0.12439E+00_JPRB ,&
&  0.12599E+00_JPRB, 0.12768E+00_JPRB, 0.12967E+00_JPRB, 0.13162E+00_JPRB, 0.13419E+00_JPRB, 0.13591E+00_JPRB, 0.13902E+00_JPRB ,&
&  0.14249E+00_JPRB, 0.14413E+00_JPRB, 0.15542E+00_JPRB, 0.15651E+00_JPRB, 0.15804E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.16554E+00_JPRB, 0.16664E+00_JPRB, 0.16655E+00_JPRB, 0.16692E+00_JPRB, 0.16613E+00_JPRB, 0.16483E+00_JPRB, 0.16302E+00_JPRB ,&
&  0.16072E+00_JPRB, 0.15529E+00_JPRB, 0.15159E+00_JPRB, 0.14750E+00_JPRB, 0.14405E+00_JPRB, 0.14108E+00_JPRB, 0.13901E+00_JPRB ,&
&  0.13814E+00_JPRB, 0.13793E+00_JPRB, 0.13785E+00_JPRB, 0.13711E+00_JPRB, 0.13572E+00_JPRB, 0.13480E+00_JPRB, 0.13359E+00_JPRB ,&
&  0.13271E+00_JPRB, 0.13259E+00_JPRB, 0.13331E+00_JPRB, 0.13502E+00_JPRB, 0.13799E+00_JPRB, 0.14030E+00_JPRB, 0.14235E+00_JPRB ,&
&  0.14464E+00_JPRB, 0.14163E+00_JPRB, 0.14133E+00_JPRB, 0.14151E+00_JPRB, 0.14126E+00_JPRB, 0.14040E+00_JPRB, 0.13869E+00_JPRB ,&
&  0.14094E+00_JPRB, 0.14150E+00_JPRB, 0.14578E+00_JPRB, 0.14574E+00_JPRB, 0.14469E+00_JPRB, 0.14529E+00_JPRB, 0.14365E+00_JPRB ,&
&  0.14289E+00_JPRB, 0.14002E+00_JPRB, 0.13791E+00_JPRB, 0.13516E+00_JPRB, 0.13255E+00_JPRB, 0.13125E+00_JPRB, 0.12974E+00_JPRB ,&
&  0.12925E+00_JPRB, 0.12913E+00_JPRB, 0.12798E+00_JPRB, 0.12727E+00_JPRB, 0.12600E+00_JPRB, 0.12457E+00_JPRB, 0.12364E+00_JPRB ,&
&  0.12268E+00_JPRB, 0.12191E+00_JPRB, 0.12148E+00_JPRB, 0.12110E+00_JPRB, 0.12095E+00_JPRB, 0.12089E+00_JPRB, 0.12082E+00_JPRB ,&
&  0.12084E+00_JPRB, 0.12075E+00_JPRB, 0.12028E+00_JPRB, 0.11998E+00_JPRB, 0.11914E+00_JPRB, 0.11855E+00_JPRB, 0.11808E+00_JPRB ,&
&  0.11769E+00_JPRB, 0.11756E+00_JPRB, 0.11749E+00_JPRB, 0.11754E+00_JPRB, 0.11766E+00_JPRB, 0.11804E+00_JPRB, 0.11848E+00_JPRB ,&
&  0.11893E+00_JPRB, 0.11970E+00_JPRB, 0.12031E+00_JPRB, 0.12116E+00_JPRB, 0.12218E+00_JPRB, 0.12304E+00_JPRB, 0.12442E+00_JPRB ,&
&  0.12596E+00_JPRB, 0.12776E+00_JPRB, 0.13041E+00_JPRB, 0.13295E+00_JPRB, 0.13585E+00_JPRB, 0.13995E+00_JPRB, 0.14450E+00_JPRB ,&
&  0.14956E+00_JPRB, 0.15581E+00_JPRB, 0.15665E+00_JPRB, 0.16101E+00_JPRB, 0.16393E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.16568E+00_JPRB, 0.16723E+00_JPRB, 0.16795E+00_JPRB, 0.16896E+00_JPRB, 0.16896E+00_JPRB, 0.16690E+00_JPRB, 0.16487E+00_JPRB ,&
&  0.16003E+00_JPRB, 0.15605E+00_JPRB, 0.15123E+00_JPRB, 0.14668E+00_JPRB, 0.14241E+00_JPRB, 0.13962E+00_JPRB, 0.13806E+00_JPRB ,&
&  0.13823E+00_JPRB, 0.13971E+00_JPRB, 0.14086E+00_JPRB, 0.14150E+00_JPRB, 0.14204E+00_JPRB, 0.14310E+00_JPRB, 0.14340E+00_JPRB ,&
&  0.14310E+00_JPRB, 0.14202E+00_JPRB, 0.14123E+00_JPRB, 0.14216E+00_JPRB, 0.14407E+00_JPRB, 0.14607E+00_JPRB, 0.14773E+00_JPRB ,&
&  0.14926E+00_JPRB, 0.14877E+00_JPRB, 0.14879E+00_JPRB, 0.14804E+00_JPRB, 0.14736E+00_JPRB, 0.14653E+00_JPRB, 0.14610E+00_JPRB ,&
&  0.14652E+00_JPRB, 0.14690E+00_JPRB, 0.14830E+00_JPRB, 0.14914E+00_JPRB, 0.14791E+00_JPRB, 0.14916E+00_JPRB, 0.14885E+00_JPRB ,&
&  0.14782E+00_JPRB, 0.14498E+00_JPRB, 0.14203E+00_JPRB, 0.13975E+00_JPRB, 0.13745E+00_JPRB, 0.13563E+00_JPRB, 0.13414E+00_JPRB ,&
&  0.13275E+00_JPRB, 0.13198E+00_JPRB, 0.13041E+00_JPRB, 0.12910E+00_JPRB, 0.12828E+00_JPRB, 0.12742E+00_JPRB, 0.12686E+00_JPRB ,&
&  0.12647E+00_JPRB, 0.12584E+00_JPRB, 0.12568E+00_JPRB, 0.12538E+00_JPRB, 0.12548E+00_JPRB, 0.12564E+00_JPRB, 0.12598E+00_JPRB ,&
&  0.12668E+00_JPRB, 0.12748E+00_JPRB, 0.12740E+00_JPRB, 0.12724E+00_JPRB, 0.12594E+00_JPRB, 0.12452E+00_JPRB, 0.12325E+00_JPRB ,&
&  0.12170E+00_JPRB, 0.12051E+00_JPRB, 0.11977E+00_JPRB, 0.11907E+00_JPRB, 0.11898E+00_JPRB, 0.11905E+00_JPRB, 0.11908E+00_JPRB ,&
&  0.11927E+00_JPRB, 0.11955E+00_JPRB, 0.11996E+00_JPRB, 0.12077E+00_JPRB, 0.12149E+00_JPRB, 0.12252E+00_JPRB, 0.12364E+00_JPRB ,&
&  0.12457E+00_JPRB, 0.12625E+00_JPRB, 0.12804E+00_JPRB, 0.13021E+00_JPRB, 0.13418E+00_JPRB, 0.13775E+00_JPRB, 0.14431E+00_JPRB ,&
&  0.14983E+00_JPRB, 0.15944E+00_JPRB, 0.15788E+00_JPRB, 0.16086E+00_JPRB, 0.16396E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.16777E+00_JPRB, 0.16870E+00_JPRB, 0.16845E+00_JPRB, 0.16858E+00_JPRB, 0.16816E+00_JPRB, 0.16934E+00_JPRB, 0.16872E+00_JPRB ,&
&  0.16884E+00_JPRB, 0.16806E+00_JPRB, 0.16724E+00_JPRB, 0.16510E+00_JPRB, 0.16399E+00_JPRB, 0.15819E+00_JPRB, 0.15677E+00_JPRB ,&
&  0.15572E+00_JPRB, 0.15662E+00_JPRB, 0.15692E+00_JPRB, 0.15744E+00_JPRB, 0.15843E+00_JPRB, 0.16008E+00_JPRB, 0.16024E+00_JPRB ,&
&  0.15927E+00_JPRB, 0.15725E+00_JPRB, 0.15492E+00_JPRB, 0.15279E+00_JPRB, 0.15315E+00_JPRB, 0.15290E+00_JPRB, 0.15207E+00_JPRB ,&
&  0.15109E+00_JPRB, 0.15163E+00_JPRB, 0.15213E+00_JPRB, 0.15339E+00_JPRB, 0.15827E+00_JPRB, 0.15694E+00_JPRB, 0.15565E+00_JPRB ,&
&  0.15532E+00_JPRB, 0.15507E+00_JPRB, 0.15539E+00_JPRB, 0.15559E+00_JPRB, 0.15716E+00_JPRB, 0.15660E+00_JPRB, 0.15886E+00_JPRB ,&
&  0.15603E+00_JPRB, 0.15417E+00_JPRB, 0.15184E+00_JPRB, 0.14909E+00_JPRB, 0.14581E+00_JPRB, 0.14373E+00_JPRB, 0.14173E+00_JPRB ,&
&  0.13959E+00_JPRB, 0.13780E+00_JPRB, 0.13549E+00_JPRB, 0.13466E+00_JPRB, 0.13353E+00_JPRB, 0.13251E+00_JPRB, 0.13159E+00_JPRB ,&
&  0.13086E+00_JPRB, 0.12984E+00_JPRB, 0.12893E+00_JPRB, 0.12833E+00_JPRB, 0.12812E+00_JPRB, 0.12824E+00_JPRB, 0.12915E+00_JPRB ,&
&  0.13095E+00_JPRB, 0.13249E+00_JPRB, 0.13366E+00_JPRB, 0.13452E+00_JPRB, 0.13319E+00_JPRB, 0.13076E+00_JPRB, 0.12831E+00_JPRB ,&
&  0.12526E+00_JPRB, 0.12354E+00_JPRB, 0.12197E+00_JPRB, 0.12105E+00_JPRB, 0.12083E+00_JPRB, 0.12085E+00_JPRB, 0.12114E+00_JPRB ,&
&  0.12146E+00_JPRB, 0.12189E+00_JPRB, 0.12220E+00_JPRB, 0.12258E+00_JPRB, 0.12307E+00_JPRB, 0.12356E+00_JPRB, 0.12426E+00_JPRB ,&
&  0.12538E+00_JPRB, 0.12684E+00_JPRB, 0.12889E+00_JPRB, 0.13132E+00_JPRB, 0.13410E+00_JPRB, 0.13779E+00_JPRB, 0.14328E+00_JPRB ,&
&  0.14916E+00_JPRB, 0.15948E+00_JPRB, 0.16065E+00_JPRB, 0.16462E+00_JPRB, 0.16705E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.17021E+00_JPRB, 0.17163E+00_JPRB, 0.17213E+00_JPRB, 0.17322E+00_JPRB, 0.17366E+00_JPRB, 0.17495E+00_JPRB, 0.17606E+00_JPRB ,&
&  0.17847E+00_JPRB, 0.18064E+00_JPRB, 0.18118E+00_JPRB, 0.17955E+00_JPRB, 0.17875E+00_JPRB, 0.17651E+00_JPRB, 0.17578E+00_JPRB ,&
&  0.17583E+00_JPRB, 0.17700E+00_JPRB, 0.17726E+00_JPRB, 0.17742E+00_JPRB, 0.17747E+00_JPRB, 0.17749E+00_JPRB, 0.17792E+00_JPRB ,&
&  0.17611E+00_JPRB, 0.17567E+00_JPRB, 0.17776E+00_JPRB, 0.17717E+00_JPRB, 0.17667E+00_JPRB, 0.17553E+00_JPRB, 0.17491E+00_JPRB ,&
&  0.17123E+00_JPRB, 0.16580E+00_JPRB, 0.15900E+00_JPRB, 0.16148E+00_JPRB, 0.16857E+00_JPRB, 0.16672E+00_JPRB, 0.16592E+00_JPRB ,&
&  0.16524E+00_JPRB, 0.16405E+00_JPRB, 0.16352E+00_JPRB, 0.16299E+00_JPRB, 0.16339E+00_JPRB, 0.16379E+00_JPRB, 0.16507E+00_JPRB ,&
&  0.16691E+00_JPRB, 0.16968E+00_JPRB, 0.16952E+00_JPRB, 0.16375E+00_JPRB, 0.16056E+00_JPRB, 0.15936E+00_JPRB, 0.15632E+00_JPRB ,&
&  0.15283E+00_JPRB, 0.15124E+00_JPRB, 0.15044E+00_JPRB, 0.14861E+00_JPRB, 0.14821E+00_JPRB, 0.14782E+00_JPRB, 0.14583E+00_JPRB ,&
&  0.14310E+00_JPRB, 0.14108E+00_JPRB, 0.13804E+00_JPRB, 0.13488E+00_JPRB, 0.13263E+00_JPRB, 0.13155E+00_JPRB, 0.13154E+00_JPRB ,&
&  0.13308E+00_JPRB, 0.13484E+00_JPRB, 0.14023E+00_JPRB, 0.14390E+00_JPRB, 0.14128E+00_JPRB, 0.13772E+00_JPRB, 0.13415E+00_JPRB ,&
&  0.12949E+00_JPRB, 0.12721E+00_JPRB, 0.12544E+00_JPRB, 0.12418E+00_JPRB, 0.12408E+00_JPRB, 0.12481E+00_JPRB, 0.12543E+00_JPRB ,&
&  0.12613E+00_JPRB, 0.12698E+00_JPRB, 0.12739E+00_JPRB, 0.12754E+00_JPRB, 0.12757E+00_JPRB, 0.12777E+00_JPRB, 0.12827E+00_JPRB ,&
&  0.12895E+00_JPRB, 0.13053E+00_JPRB, 0.13286E+00_JPRB, 0.13538E+00_JPRB, 0.13713E+00_JPRB, 0.13847E+00_JPRB, 0.14127E+00_JPRB ,&
&  0.14461E+00_JPRB, 0.15238E+00_JPRB, 0.16349E+00_JPRB, 0.17245E+00_JPRB, 0.16938E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.18242E+00_JPRB, 0.18323E+00_JPRB, 0.18424E+00_JPRB, 0.18548E+00_JPRB, 0.18696E+00_JPRB, 0.18859E+00_JPRB, 0.18995E+00_JPRB ,&
&  0.19141E+00_JPRB, 0.19236E+00_JPRB, 0.19260E+00_JPRB, 0.19165E+00_JPRB, 0.19096E+00_JPRB, 0.19146E+00_JPRB, 0.19282E+00_JPRB ,&
&  0.19132E+00_JPRB, 0.18872E+00_JPRB, 0.18775E+00_JPRB, 0.18778E+00_JPRB, 0.18737E+00_JPRB, 0.18714E+00_JPRB, 0.18784E+00_JPRB ,&
&  0.18875E+00_JPRB, 0.18857E+00_JPRB, 0.18837E+00_JPRB, 0.18757E+00_JPRB, 0.18687E+00_JPRB, 0.18630E+00_JPRB, 0.18546E+00_JPRB ,&
&  0.18280E+00_JPRB, 0.17310E+00_JPRB, 0.16644E+00_JPRB, 0.16917E+00_JPRB, 0.17642E+00_JPRB, 0.17587E+00_JPRB, 0.17580E+00_JPRB ,&
&  0.17526E+00_JPRB, 0.17398E+00_JPRB, 0.17255E+00_JPRB, 0.17120E+00_JPRB, 0.17058E+00_JPRB, 0.17075E+00_JPRB, 0.17136E+00_JPRB ,&
&  0.17197E+00_JPRB, 0.17363E+00_JPRB, 0.17574E+00_JPRB, 0.17680E+00_JPRB, 0.17781E+00_JPRB, 0.17824E+00_JPRB, 0.17840E+00_JPRB ,&
&  0.17821E+00_JPRB, 0.17747E+00_JPRB, 0.17708E+00_JPRB, 0.17670E+00_JPRB, 0.17634E+00_JPRB, 0.17559E+00_JPRB, 0.17262E+00_JPRB ,&
&  0.16734E+00_JPRB, 0.16246E+00_JPRB, 0.15648E+00_JPRB, 0.15049E+00_JPRB, 0.14491E+00_JPRB, 0.14086E+00_JPRB, 0.13767E+00_JPRB ,&
&  0.13709E+00_JPRB, 0.14028E+00_JPRB, 0.14731E+00_JPRB, 0.15633E+00_JPRB, 0.16342E+00_JPRB, 0.15297E+00_JPRB, 0.14670E+00_JPRB ,&
&  0.14115E+00_JPRB, 0.13688E+00_JPRB, 0.13427E+00_JPRB, 0.13291E+00_JPRB, 0.13207E+00_JPRB, 0.13216E+00_JPRB, 0.13277E+00_JPRB ,&
&  0.13391E+00_JPRB, 0.13518E+00_JPRB, 0.13638E+00_JPRB, 0.13707E+00_JPRB, 0.13730E+00_JPRB, 0.13712E+00_JPRB, 0.13673E+00_JPRB ,&
&  0.13702E+00_JPRB, 0.13894E+00_JPRB, 0.14266E+00_JPRB, 0.14715E+00_JPRB, 0.15093E+00_JPRB, 0.15442E+00_JPRB, 0.15937E+00_JPRB ,&
&  0.16564E+00_JPRB, 0.17109E+00_JPRB, 0.17506E+00_JPRB, 0.17909E+00_JPRB, 0.18115E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.19964E+00_JPRB, 0.20050E+00_JPRB, 0.20139E+00_JPRB, 0.20236E+00_JPRB, 0.20416E+00_JPRB, 0.20552E+00_JPRB, 0.20713E+00_JPRB ,&
&  0.20828E+00_JPRB, 0.20700E+00_JPRB, 0.20532E+00_JPRB, 0.20407E+00_JPRB, 0.20367E+00_JPRB, 0.20327E+00_JPRB, 0.20309E+00_JPRB ,&
&  0.20409E+00_JPRB, 0.20366E+00_JPRB, 0.20134E+00_JPRB, 0.19980E+00_JPRB, 0.19705E+00_JPRB, 0.19518E+00_JPRB, 0.19482E+00_JPRB ,&
&  0.19636E+00_JPRB, 0.19627E+00_JPRB, 0.19600E+00_JPRB, 0.19532E+00_JPRB, 0.19427E+00_JPRB, 0.19380E+00_JPRB, 0.19280E+00_JPRB ,&
&  0.19229E+00_JPRB, 0.19159E+00_JPRB, 0.18686E+00_JPRB, 0.18755E+00_JPRB, 0.18706E+00_JPRB, 0.18685E+00_JPRB, 0.18625E+00_JPRB ,&
&  0.18560E+00_JPRB, 0.18464E+00_JPRB, 0.18294E+00_JPRB, 0.18108E+00_JPRB, 0.17953E+00_JPRB, 0.17865E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.17871E+00_JPRB, 0.17941E+00_JPRB, 0.18074E+00_JPRB, 0.18266E+00_JPRB, 0.18348E+00_JPRB, 0.18405E+00_JPRB, 0.18489E+00_JPRB ,&
&  0.18525E+00_JPRB, 0.18533E+00_JPRB, 0.18539E+00_JPRB, 0.18551E+00_JPRB, 0.18549E+00_JPRB, 0.18497E+00_JPRB, 0.18455E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18129E+00_JPRB, 0.17998E+00_JPRB, 0.17829E+00_JPRB, 0.17354E+00_JPRB, 0.16341E+00_JPRB, 0.15772E+00_JPRB ,&
&  0.15761E+00_JPRB, 0.16208E+00_JPRB, 0.16811E+00_JPRB, 0.17101E+00_JPRB, 0.17539E+00_JPRB, 0.17559E+00_JPRB, 0.17247E+00_JPRB ,&
&  0.16536E+00_JPRB, 0.15929E+00_JPRB, 0.15511E+00_JPRB, 0.15132E+00_JPRB, 0.14894E+00_JPRB, 0.14833E+00_JPRB, 0.14890E+00_JPRB ,&
&  0.15077E+00_JPRB, 0.15367E+00_JPRB, 0.15632E+00_JPRB, 0.15772E+00_JPRB, 0.15777E+00_JPRB, 0.15630E+00_JPRB, 0.15473E+00_JPRB ,&
&  0.15405E+00_JPRB, 0.15754E+00_JPRB, 0.16521E+00_JPRB, 0.17453E+00_JPRB, 0.17831E+00_JPRB, 0.17964E+00_JPRB, 0.18098E+00_JPRB ,&
&  0.18390E+00_JPRB, 0.18805E+00_JPRB, 0.19235E+00_JPRB, 0.19534E+00_JPRB, 0.19779E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.21695E+00_JPRB, 0.21755E+00_JPRB, 0.21832E+00_JPRB, 0.21948E+00_JPRB, 0.22108E+00_JPRB, 0.22194E+00_JPRB, 0.22262E+00_JPRB ,&
&  0.22198E+00_JPRB, 0.22032E+00_JPRB, 0.21844E+00_JPRB, 0.21646E+00_JPRB, 0.21570E+00_JPRB, 0.21586E+00_JPRB, 0.21625E+00_JPRB ,&
&  0.21602E+00_JPRB, 0.21612E+00_JPRB, 0.21468E+00_JPRB, 0.21207E+00_JPRB, 0.20903E+00_JPRB, 0.20705E+00_JPRB, 0.20657E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.20540E+00_JPRB, 0.20521E+00_JPRB, 0.20508E+00_JPRB, 0.20344E+00_JPRB, 0.20257E+00_JPRB, 0.20298E+00_JPRB ,&
&  0.20381E+00_JPRB, 0.20518E+00_JPRB, 0.20549E+00_JPRB, 0.20282E+00_JPRB, 0.20506E+00_JPRB, 0.20474E+00_JPRB, 0.20290E+00_JPRB ,&
&  0.20058E+00_JPRB, 0.19821E+00_JPRB, 0.19608E+00_JPRB, 0.19400E+00_JPRB, 0.19192E+00_JPRB, 0.19035E+00_JPRB, 0.18933E+00_JPRB ,&
&  0.18904E+00_JPRB, 0.18913E+00_JPRB, 0.18945E+00_JPRB, 0.19027E+00_JPRB, 0.19090E+00_JPRB, 0.19125E+00_JPRB, 0.19127E+00_JPRB ,&
&  0.19136E+00_JPRB, 0.19155E+00_JPRB, 0.19191E+00_JPRB, 0.19215E+00_JPRB, 0.19244E+00_JPRB, 0.19275E+00_JPRB, 0.19279E+00_JPRB ,&
&  0.19194E+00_JPRB, 0.19043E+00_JPRB, 0.18835E+00_JPRB, 0.18773E+00_JPRB, 0.18783E+00_JPRB, 0.18668E+00_JPRB, 0.18457E+00_JPRB ,&
&  0.18244E+00_JPRB, 0.18182E+00_JPRB, 0.18268E+00_JPRB, 0.18378E+00_JPRB, 0.18464E+00_JPRB, 0.18490E+00_JPRB, 0.18459E+00_JPRB ,&
&  0.18367E+00_JPRB, 0.18173E+00_JPRB, 0.17918E+00_JPRB, 0.17722E+00_JPRB, 0.17623E+00_JPRB, 0.17621E+00_JPRB, 0.17704E+00_JPRB ,&
&  0.17717E+00_JPRB, 0.17712E+00_JPRB, 0.17806E+00_JPRB, 0.17879E+00_JPRB, 0.17924E+00_JPRB, 0.17990E+00_JPRB, 0.18124E+00_JPRB ,&
&  0.18313E+00_JPRB, 0.18606E+00_JPRB, 0.18963E+00_JPRB, 0.19332E+00_JPRB, 0.19589E+00_JPRB, 0.19501E+00_JPRB, 0.19528E+00_JPRB ,&
&  0.19921E+00_JPRB, 0.20416E+00_JPRB, 0.20838E+00_JPRB, 0.21176E+00_JPRB, 0.21479E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.23679E+00_JPRB, 0.23819E+00_JPRB, 0.23974E+00_JPRB, 0.24153E+00_JPRB, 0.24277E+00_JPRB, 0.24319E+00_JPRB, 0.24183E+00_JPRB ,&
&  0.23929E+00_JPRB, 0.23653E+00_JPRB, 0.23383E+00_JPRB, 0.23155E+00_JPRB, 0.23027E+00_JPRB, 0.22966E+00_JPRB, 0.22942E+00_JPRB ,&
&  0.22931E+00_JPRB, 0.22915E+00_JPRB, 0.22757E+00_JPRB, 0.22510E+00_JPRB, 0.22181E+00_JPRB, 0.21905E+00_JPRB, 0.21848E+00_JPRB ,&
&  0.21714E+00_JPRB, 0.21623E+00_JPRB, 0.21572E+00_JPRB, 0.21512E+00_JPRB, 0.21498E+00_JPRB, 0.21602E+00_JPRB, 0.21775E+00_JPRB ,&
&  0.21970E+00_JPRB, 0.22117E+00_JPRB, 0.22230E+00_JPRB, 0.22319E+00_JPRB, 0.22272E+00_JPRB, 0.22159E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.21871E+00_JPRB, 0.21727E+00_JPRB, 0.21603E+00_JPRB, 0.21436E+00_JPRB, 0.21188E+00_JPRB, 0.20892E+00_JPRB, 0.20683E+00_JPRB ,&
&  0.20526E+00_JPRB, 0.20382E+00_JPRB, 0.20225E+00_JPRB, 0.20143E+00_JPRB, 0.20086E+00_JPRB, 0.20064E+00_JPRB, 0.20073E+00_JPRB ,&
&  0.20115E+00_JPRB, 0.20234E+00_JPRB, 0.20396E+00_JPRB, 0.20569E+00_JPRB, 0.20611E+00_JPRB, 0.20623E+00_JPRB, 0.20649E+00_JPRB ,&
&  0.20592E+00_JPRB, 0.20474E+00_JPRB, 0.20313E+00_JPRB, 0.20143E+00_JPRB, 0.20044E+00_JPRB, 0.19970E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19719E+00_JPRB, 0.19617E+00_JPRB, 0.19556E+00_JPRB, 0.19542E+00_JPRB, 0.19563E+00_JPRB, 0.19569E+00_JPRB, 0.19505E+00_JPRB ,&
&  0.19425E+00_JPRB, 0.19353E+00_JPRB, 0.19250E+00_JPRB, 0.19139E+00_JPRB, 0.18969E+00_JPRB, 0.18994E+00_JPRB, 0.19076E+00_JPRB ,&
&  0.19123E+00_JPRB, 0.19151E+00_JPRB, 0.19172E+00_JPRB, 0.19270E+00_JPRB, 0.19371E+00_JPRB, 0.19518E+00_JPRB, 0.19744E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.20169E+00_JPRB, 0.20535E+00_JPRB, 0.20870E+00_JPRB, 0.21112E+00_JPRB, 0.21221E+00_JPRB, 0.21440E+00_JPRB ,&
&  0.21953E+00_JPRB, 0.22530E+00_JPRB, 0.22915E+00_JPRB, 0.23279E+00_JPRB, 0.23577E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.25632E+00_JPRB, 0.26099E+00_JPRB, 0.26465E+00_JPRB, 0.26672E+00_JPRB, 0.26672E+00_JPRB, 0.26546E+00_JPRB, 0.26295E+00_JPRB ,&
&  0.25945E+00_JPRB, 0.25601E+00_JPRB, 0.25250E+00_JPRB, 0.24962E+00_JPRB, 0.24791E+00_JPRB, 0.24659E+00_JPRB, 0.24567E+00_JPRB ,&
&  0.24429E+00_JPRB, 0.24252E+00_JPRB, 0.24015E+00_JPRB, 0.23805E+00_JPRB, 0.23556E+00_JPRB, 0.23198E+00_JPRB, 0.22945E+00_JPRB ,&
&  0.22775E+00_JPRB, 0.22606E+00_JPRB, 0.22567E+00_JPRB, 0.22695E+00_JPRB, 0.22924E+00_JPRB, 0.23149E+00_JPRB, 0.23400E+00_JPRB ,&
&  0.23575E+00_JPRB, 0.23644E+00_JPRB, 0.23689E+00_JPRB, 0.23671E+00_JPRB, 0.23676E+00_JPRB, 0.23648E+00_JPRB, 0.23577E+00_JPRB ,&
&  0.23502E+00_JPRB, 0.23428E+00_JPRB, 0.23299E+00_JPRB, 0.23279E+00_JPRB, 0.23270E+00_JPRB, 0.23163E+00_JPRB, 0.22973E+00_JPRB ,&
&  0.22672E+00_JPRB, 0.22244E+00_JPRB, 0.21892E+00_JPRB, 0.21705E+00_JPRB, 0.21637E+00_JPRB, 0.21654E+00_JPRB, 0.21732E+00_JPRB ,&
&  0.21840E+00_JPRB, 0.21997E+00_JPRB, 0.22141E+00_JPRB, 0.22209E+00_JPRB, 0.22189E+00_JPRB, 0.22169E+00_JPRB, 0.22049E+00_JPRB ,&
&  0.21931E+00_JPRB, 0.21857E+00_JPRB, 0.21801E+00_JPRB, 0.21740E+00_JPRB, 0.21542E+00_JPRB, 0.21301E+00_JPRB, 0.21262E+00_JPRB ,&
&  0.21142E+00_JPRB, 0.21068E+00_JPRB, 0.20959E+00_JPRB, 0.20919E+00_JPRB, 0.20862E+00_JPRB, 0.20758E+00_JPRB, 0.20638E+00_JPRB ,&
&  0.20533E+00_JPRB, 0.20474E+00_JPRB, 0.20432E+00_JPRB, 0.20333E+00_JPRB, 0.20351E+00_JPRB, 0.20397E+00_JPRB, 0.20525E+00_JPRB ,&
&  0.20644E+00_JPRB, 0.20768E+00_JPRB, 0.20921E+00_JPRB, 0.21052E+00_JPRB, 0.21257E+00_JPRB, 0.21501E+00_JPRB, 0.21782E+00_JPRB ,&
&  0.22016E+00_JPRB, 0.22352E+00_JPRB, 0.22748E+00_JPRB, 0.23117E+00_JPRB, 0.23404E+00_JPRB, 0.23608E+00_JPRB, 0.23884E+00_JPRB ,&
&  0.24348E+00_JPRB, 0.24736E+00_JPRB, 0.24928E+00_JPRB, 0.25123E+00_JPRB, 0.25283E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.28440E+00_JPRB, 0.28907E+00_JPRB, 0.29162E+00_JPRB, 0.29264E+00_JPRB, 0.29276E+00_JPRB, 0.29114E+00_JPRB, 0.28869E+00_JPRB ,&
&  0.28512E+00_JPRB, 0.28168E+00_JPRB, 0.27782E+00_JPRB, 0.27333E+00_JPRB, 0.26932E+00_JPRB, 0.26603E+00_JPRB, 0.26332E+00_JPRB ,&
&  0.26063E+00_JPRB, 0.25819E+00_JPRB, 0.25619E+00_JPRB, 0.25405E+00_JPRB, 0.25159E+00_JPRB, 0.24852E+00_JPRB, 0.24434E+00_JPRB ,&
&  0.24198E+00_JPRB, 0.24020E+00_JPRB, 0.24129E+00_JPRB, 0.24400E+00_JPRB, 0.24605E+00_JPRB, 0.24740E+00_JPRB, 0.24838E+00_JPRB ,&
&  0.24972E+00_JPRB, 0.25054E+00_JPRB, 0.25144E+00_JPRB, 0.25373E+00_JPRB, 0.25591E+00_JPRB, 0.25568E+00_JPRB, 0.25424E+00_JPRB ,&
&  0.25299E+00_JPRB, 0.25314E+00_JPRB, 0.25367E+00_JPRB, 0.25388E+00_JPRB, 0.25344E+00_JPRB, 0.25165E+00_JPRB, 0.24869E+00_JPRB ,&
&  0.24311E+00_JPRB, 0.23888E+00_JPRB, 0.23789E+00_JPRB, 0.23885E+00_JPRB, 0.24050E+00_JPRB, 0.24174E+00_JPRB, 0.24251E+00_JPRB ,&
&  0.24275E+00_JPRB, 0.24222E+00_JPRB, 0.24120E+00_JPRB, 0.23999E+00_JPRB, 0.23836E+00_JPRB, 0.23637E+00_JPRB, 0.23518E+00_JPRB ,&
&  0.23386E+00_JPRB, 0.23264E+00_JPRB, 0.23173E+00_JPRB, 0.23115E+00_JPRB, 0.23003E+00_JPRB, 0.22858E+00_JPRB, 0.22783E+00_JPRB ,&
&  0.22676E+00_JPRB, 0.22623E+00_JPRB, 0.22539E+00_JPRB, 0.22457E+00_JPRB, 0.22344E+00_JPRB, 0.22236E+00_JPRB, 0.22218E+00_JPRB ,&
&  0.22235E+00_JPRB, 0.22126E+00_JPRB, 0.21986E+00_JPRB, 0.21960E+00_JPRB, 0.21992E+00_JPRB, 0.22058E+00_JPRB, 0.22117E+00_JPRB ,&
&  0.22181E+00_JPRB, 0.22262E+00_JPRB, 0.22436E+00_JPRB, 0.22624E+00_JPRB, 0.22834E+00_JPRB, 0.23095E+00_JPRB, 0.23399E+00_JPRB ,&
&  0.23783E+00_JPRB, 0.24253E+00_JPRB, 0.24793E+00_JPRB, 0.25281E+00_JPRB, 0.25690E+00_JPRB, 0.25970E+00_JPRB, 0.26365E+00_JPRB ,&
&  0.26850E+00_JPRB, 0.27195E+00_JPRB, 0.27453E+00_JPRB, 0.27716E+00_JPRB, 0.28002E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.31258E+00_JPRB, 0.31420E+00_JPRB, 0.31422E+00_JPRB, 0.31456E+00_JPRB, 0.31408E+00_JPRB, 0.31239E+00_JPRB, 0.30948E+00_JPRB ,&
&  0.30648E+00_JPRB, 0.30297E+00_JPRB, 0.29938E+00_JPRB, 0.29470E+00_JPRB, 0.28981E+00_JPRB, 0.28597E+00_JPRB, 0.28177E+00_JPRB ,&
&  0.27747E+00_JPRB, 0.27352E+00_JPRB, 0.27090E+00_JPRB, 0.26880E+00_JPRB, 0.26678E+00_JPRB, 0.26498E+00_JPRB, 0.26243E+00_JPRB ,&
&  0.26024E+00_JPRB, 0.25815E+00_JPRB, 0.25855E+00_JPRB, 0.26083E+00_JPRB, 0.26265E+00_JPRB, 0.26406E+00_JPRB, 0.26523E+00_JPRB ,&
&  0.26554E+00_JPRB, 0.27035E+00_JPRB, 0.26478E+00_JPRB, 0.27350E+00_JPRB, 0.27549E+00_JPRB, 0.27611E+00_JPRB, 0.27622E+00_JPRB ,&
&  0.27778E+00_JPRB, 0.27970E+00_JPRB, 0.28079E+00_JPRB, 0.27834E+00_JPRB, 0.27496E+00_JPRB, 0.27244E+00_JPRB, 0.26933E+00_JPRB ,&
&  0.26501E+00_JPRB, 0.26338E+00_JPRB, 0.26360E+00_JPRB, 0.26402E+00_JPRB, 0.26410E+00_JPRB, 0.26364E+00_JPRB, 0.26260E+00_JPRB ,&
&  0.26124E+00_JPRB, 0.25927E+00_JPRB, 0.25696E+00_JPRB, 0.25456E+00_JPRB, 0.25304E+00_JPRB, 0.25116E+00_JPRB, 0.24932E+00_JPRB ,&
&  0.24842E+00_JPRB, 0.24853E+00_JPRB, 0.24858E+00_JPRB, 0.24848E+00_JPRB, 0.24788E+00_JPRB, 0.24751E+00_JPRB, 0.24743E+00_JPRB ,&
&  0.24675E+00_JPRB, 0.24590E+00_JPRB, 0.24487E+00_JPRB, 0.24351E+00_JPRB, 0.24208E+00_JPRB, 0.24144E+00_JPRB, 0.24161E+00_JPRB ,&
&  0.24147E+00_JPRB, 0.24014E+00_JPRB, 0.23800E+00_JPRB, 0.23740E+00_JPRB, 0.23711E+00_JPRB, 0.23721E+00_JPRB, 0.23758E+00_JPRB ,&
&  0.23856E+00_JPRB, 0.24039E+00_JPRB, 0.24234E+00_JPRB, 0.24412E+00_JPRB, 0.24585E+00_JPRB, 0.24860E+00_JPRB, 0.25214E+00_JPRB ,&
&  0.25638E+00_JPRB, 0.26101E+00_JPRB, 0.26550E+00_JPRB, 0.26975E+00_JPRB, 0.27365E+00_JPRB, 0.27875E+00_JPRB, 0.28588E+00_JPRB ,&
&  0.29215E+00_JPRB, 0.29684E+00_JPRB, 0.30043E+00_JPRB, 0.30454E+00_JPRB, 0.30896E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.34532E+00_JPRB, 0.34363E+00_JPRB, 0.34050E+00_JPRB, 0.33852E+00_JPRB, 0.33601E+00_JPRB, 0.33282E+00_JPRB, 0.32943E+00_JPRB ,&
&  0.32678E+00_JPRB, 0.32414E+00_JPRB, 0.32137E+00_JPRB, 0.31740E+00_JPRB, 0.31200E+00_JPRB, 0.30688E+00_JPRB, 0.30085E+00_JPRB ,&
&  0.29525E+00_JPRB, 0.29042E+00_JPRB, 0.28725E+00_JPRB, 0.28561E+00_JPRB, 0.28419E+00_JPRB, 0.28310E+00_JPRB, 0.28176E+00_JPRB ,&
&  0.28117E+00_JPRB, 0.28114E+00_JPRB, 0.28107E+00_JPRB, 0.28137E+00_JPRB, 0.28205E+00_JPRB, 0.28457E+00_JPRB, 0.28617E+00_JPRB ,&
&  0.28811E+00_JPRB, 0.28431E+00_JPRB, 0.29080E+00_JPRB, 0.29556E+00_JPRB, 0.30273E+00_JPRB, 0.30776E+00_JPRB, 0.31119E+00_JPRB ,&
&  0.31376E+00_JPRB, 0.31523E+00_JPRB, 0.31244E+00_JPRB, 0.30847E+00_JPRB, 0.30518E+00_JPRB, 0.30314E+00_JPRB, 0.29895E+00_JPRB ,&
&  0.29464E+00_JPRB, 0.29158E+00_JPRB, 0.28915E+00_JPRB, 0.28747E+00_JPRB, 0.28561E+00_JPRB, 0.28285E+00_JPRB, 0.27980E+00_JPRB ,&
&  0.27585E+00_JPRB, 0.27297E+00_JPRB, 0.27065E+00_JPRB, 0.26978E+00_JPRB, 0.26999E+00_JPRB, 0.27102E+00_JPRB, 0.27253E+00_JPRB ,&
&  0.27484E+00_JPRB, 0.27687E+00_JPRB, 0.27782E+00_JPRB, 0.27816E+00_JPRB, 0.27807E+00_JPRB, 0.27802E+00_JPRB, 0.27738E+00_JPRB ,&
&  0.27598E+00_JPRB, 0.27428E+00_JPRB, 0.27253E+00_JPRB, 0.27059E+00_JPRB, 0.26852E+00_JPRB, 0.26709E+00_JPRB, 0.26580E+00_JPRB ,&
&  0.26432E+00_JPRB, 0.26244E+00_JPRB, 0.26033E+00_JPRB, 0.25914E+00_JPRB, 0.25826E+00_JPRB, 0.25775E+00_JPRB, 0.25842E+00_JPRB ,&
&  0.26025E+00_JPRB, 0.26239E+00_JPRB, 0.26463E+00_JPRB, 0.26647E+00_JPRB, 0.26822E+00_JPRB, 0.26994E+00_JPRB, 0.27253E+00_JPRB ,&
&  0.27713E+00_JPRB, 0.28213E+00_JPRB, 0.28721E+00_JPRB, 0.29229E+00_JPRB, 0.29907E+00_JPRB, 0.30861E+00_JPRB, 0.31911E+00_JPRB ,&
&  0.32647E+00_JPRB, 0.33086E+00_JPRB, 0.33503E+00_JPRB, 0.33939E+00_JPRB, 0.34382E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.36926E+00_JPRB, 0.36671E+00_JPRB, 0.36413E+00_JPRB, 0.36108E+00_JPRB, 0.35695E+00_JPRB, 0.35130E+00_JPRB, 0.34712E+00_JPRB ,&
&  0.34440E+00_JPRB, 0.34106E+00_JPRB, 0.33819E+00_JPRB, 0.33440E+00_JPRB, 0.32980E+00_JPRB, 0.32439E+00_JPRB, 0.31744E+00_JPRB ,&
&  0.31131E+00_JPRB, 0.30571E+00_JPRB, 0.30220E+00_JPRB, 0.30063E+00_JPRB, 0.30079E+00_JPRB, 0.30109E+00_JPRB, 0.29990E+00_JPRB ,&
&  0.29925E+00_JPRB, 0.29920E+00_JPRB, 0.29872E+00_JPRB, 0.29888E+00_JPRB, 0.29950E+00_JPRB, 0.30239E+00_JPRB, 0.30587E+00_JPRB ,&
&  0.30983E+00_JPRB, 0.28665E+00_JPRB, 0.29644E+00_JPRB, 0.29989E+00_JPRB, 0.30916E+00_JPRB, 0.32376E+00_JPRB, 0.34007E+00_JPRB ,&
&  0.35148E+00_JPRB, 0.35218E+00_JPRB, 0.34839E+00_JPRB, 0.34168E+00_JPRB, 0.33627E+00_JPRB, 0.33131E+00_JPRB, 0.32598E+00_JPRB ,&
&  0.32076E+00_JPRB, 0.31749E+00_JPRB, 0.31461E+00_JPRB, 0.31161E+00_JPRB, 0.30719E+00_JPRB, 0.30143E+00_JPRB, 0.29571E+00_JPRB ,&
&  0.29156E+00_JPRB, 0.28995E+00_JPRB, 0.29041E+00_JPRB, 0.29194E+00_JPRB, 0.29397E+00_JPRB, 0.29743E+00_JPRB, 0.30164E+00_JPRB ,&
&  0.30572E+00_JPRB, 0.30752E+00_JPRB, 0.30850E+00_JPRB, 0.30988E+00_JPRB, 0.31082E+00_JPRB, 0.31072E+00_JPRB, 0.30944E+00_JPRB ,&
&  0.30739E+00_JPRB, 0.30448E+00_JPRB, 0.30130E+00_JPRB, 0.29769E+00_JPRB, 0.29416E+00_JPRB, 0.29054E+00_JPRB, 0.28724E+00_JPRB ,&
&  0.28442E+00_JPRB, 0.28144E+00_JPRB, 0.27836E+00_JPRB, 0.27603E+00_JPRB, 0.27541E+00_JPRB, 0.27600E+00_JPRB, 0.27741E+00_JPRB ,&
&  0.27996E+00_JPRB, 0.28257E+00_JPRB, 0.28424E+00_JPRB, 0.28614E+00_JPRB, 0.28817E+00_JPRB, 0.29038E+00_JPRB, 0.29394E+00_JPRB ,&
&  0.29921E+00_JPRB, 0.30353E+00_JPRB, 0.30788E+00_JPRB, 0.31475E+00_JPRB, 0.32533E+00_JPRB, 0.33787E+00_JPRB, 0.34917E+00_JPRB ,&
&  0.35620E+00_JPRB, 0.35883E+00_JPRB, 0.36201E+00_JPRB, 0.36452E+00_JPRB, 0.36937E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.40912E+00_JPRB, 0.40773E+00_JPRB, 0.40239E+00_JPRB, 0.39194E+00_JPRB, 0.38072E+00_JPRB, 0.37141E+00_JPRB, 0.36613E+00_JPRB ,&
&  0.36296E+00_JPRB, 0.35787E+00_JPRB, 0.35347E+00_JPRB, 0.35035E+00_JPRB, 0.34622E+00_JPRB, 0.34175E+00_JPRB, 0.33574E+00_JPRB ,&
&  0.33112E+00_JPRB, 0.32766E+00_JPRB, 0.32507E+00_JPRB, 0.32348E+00_JPRB, 0.32246E+00_JPRB, 0.32135E+00_JPRB, 0.31954E+00_JPRB ,&
&  0.31743E+00_JPRB, 0.31564E+00_JPRB, 0.31427E+00_JPRB, 0.31464E+00_JPRB, 0.31555E+00_JPRB, 0.31895E+00_JPRB, 0.32469E+00_JPRB ,&
&  0.31138E+00_JPRB, 0.28676E+00_JPRB, 0.29875E+00_JPRB, 0.31226E+00_JPRB, 0.32688E+00_JPRB, 0.35048E+00_JPRB, 0.37567E+00_JPRB ,&
&  0.39020E+00_JPRB, 0.39947E+00_JPRB, 0.39531E+00_JPRB, 0.38770E+00_JPRB, 0.37821E+00_JPRB, 0.36888E+00_JPRB, 0.36148E+00_JPRB ,&
&  0.35401E+00_JPRB, 0.34898E+00_JPRB, 0.34436E+00_JPRB, 0.33734E+00_JPRB, 0.32879E+00_JPRB, 0.32181E+00_JPRB, 0.31767E+00_JPRB ,&
&  0.31618E+00_JPRB, 0.31665E+00_JPRB, 0.31743E+00_JPRB, 0.31860E+00_JPRB, 0.32085E+00_JPRB, 0.32549E+00_JPRB, 0.33126E+00_JPRB ,&
&  0.33603E+00_JPRB, 0.34117E+00_JPRB, 0.34730E+00_JPRB, 0.35303E+00_JPRB, 0.35808E+00_JPRB, 0.35985E+00_JPRB, 0.35929E+00_JPRB ,&
&  0.35541E+00_JPRB, 0.34872E+00_JPRB, 0.34153E+00_JPRB, 0.33482E+00_JPRB, 0.32823E+00_JPRB, 0.32127E+00_JPRB, 0.31432E+00_JPRB ,&
&  0.30853E+00_JPRB, 0.30324E+00_JPRB, 0.29912E+00_JPRB, 0.29760E+00_JPRB, 0.29782E+00_JPRB, 0.29991E+00_JPRB, 0.30236E+00_JPRB ,&
&  0.30456E+00_JPRB, 0.30680E+00_JPRB, 0.30840E+00_JPRB, 0.31167E+00_JPRB, 0.31547E+00_JPRB, 0.32026E+00_JPRB, 0.32552E+00_JPRB ,&
&  0.33159E+00_JPRB, 0.33739E+00_JPRB, 0.34420E+00_JPRB, 0.35815E+00_JPRB, 0.37281E+00_JPRB, 0.38594E+00_JPRB, 0.39470E+00_JPRB ,&
&  0.39703E+00_JPRB, 0.39806E+00_JPRB, 0.40109E+00_JPRB, 0.40281E+00_JPRB, 0.40943E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.46233E+00_JPRB, 0.45937E+00_JPRB, 0.44551E+00_JPRB, 0.43110E+00_JPRB, 0.41724E+00_JPRB, 0.40436E+00_JPRB, 0.39559E+00_JPRB ,&
&  0.38998E+00_JPRB, 0.38184E+00_JPRB, 0.37484E+00_JPRB, 0.37027E+00_JPRB, 0.36507E+00_JPRB, 0.36010E+00_JPRB, 0.35431E+00_JPRB ,&
&  0.34990E+00_JPRB, 0.34749E+00_JPRB, 0.34599E+00_JPRB, 0.34401E+00_JPRB, 0.34136E+00_JPRB, 0.33763E+00_JPRB, 0.33358E+00_JPRB ,&
&  0.32942E+00_JPRB, 0.32628E+00_JPRB, 0.32341E+00_JPRB, 0.32242E+00_JPRB, 0.32523E+00_JPRB, 0.33083E+00_JPRB, 0.33629E+00_JPRB ,&
&  0.35448E+00_JPRB, 0.30910E+00_JPRB, 0.32230E+00_JPRB, 0.34285E+00_JPRB, 0.36430E+00_JPRB, 0.39407E+00_JPRB, 0.41944E+00_JPRB ,&
&  0.43429E+00_JPRB, 0.44844E+00_JPRB, 0.44638E+00_JPRB, 0.43884E+00_JPRB, 0.42344E+00_JPRB, 0.40689E+00_JPRB, 0.39255E+00_JPRB ,&
&  0.38313E+00_JPRB, 0.37626E+00_JPRB, 0.36793E+00_JPRB, 0.35856E+00_JPRB, 0.34962E+00_JPRB, 0.34330E+00_JPRB, 0.34096E+00_JPRB ,&
&  0.34145E+00_JPRB, 0.34214E+00_JPRB, 0.34223E+00_JPRB, 0.34471E+00_JPRB, 0.34978E+00_JPRB, 0.35646E+00_JPRB, 0.36433E+00_JPRB ,&
&  0.37462E+00_JPRB, 0.38806E+00_JPRB, 0.39981E+00_JPRB, 0.40973E+00_JPRB, 0.41424E+00_JPRB, 0.41210E+00_JPRB, 0.40041E+00_JPRB ,&
&  0.38555E+00_JPRB, 0.37209E+00_JPRB, 0.36290E+00_JPRB, 0.35750E+00_JPRB, 0.35328E+00_JPRB, 0.35007E+00_JPRB, 0.34644E+00_JPRB ,&
&  0.34154E+00_JPRB, 0.33639E+00_JPRB, 0.33278E+00_JPRB, 0.33089E+00_JPRB, 0.32978E+00_JPRB, 0.33036E+00_JPRB, 0.33196E+00_JPRB ,&
&  0.33388E+00_JPRB, 0.33602E+00_JPRB, 0.33885E+00_JPRB, 0.34340E+00_JPRB, 0.34895E+00_JPRB, 0.35509E+00_JPRB, 0.36017E+00_JPRB ,&
&  0.36601E+00_JPRB, 0.37231E+00_JPRB, 0.37912E+00_JPRB, 0.39705E+00_JPRB, 0.41375E+00_JPRB, 0.42501E+00_JPRB, 0.43450E+00_JPRB ,&
&  0.43984E+00_JPRB, 0.44931E+00_JPRB, 0.39066E+00_JPRB, 0.43035E+00_JPRB, 0.45413E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.53532E+00_JPRB, 0.52409E+00_JPRB, 0.50083E+00_JPRB, 0.47599E+00_JPRB, 0.45443E+00_JPRB, 0.43498E+00_JPRB, 0.42029E+00_JPRB ,&
&  0.41142E+00_JPRB, 0.40541E+00_JPRB, 0.39937E+00_JPRB, 0.39379E+00_JPRB, 0.38731E+00_JPRB, 0.38162E+00_JPRB, 0.37468E+00_JPRB ,&
&  0.37002E+00_JPRB, 0.36728E+00_JPRB, 0.36499E+00_JPRB, 0.36156E+00_JPRB, 0.35756E+00_JPRB, 0.35305E+00_JPRB, 0.34731E+00_JPRB ,&
&  0.34236E+00_JPRB, 0.33975E+00_JPRB, 0.33574E+00_JPRB, 0.33480E+00_JPRB, 0.34079E+00_JPRB, 0.34800E+00_JPRB, 0.35639E+00_JPRB ,&
&  0.38025E+00_JPRB, 0.34388E+00_JPRB, 0.35542E+00_JPRB, 0.39625E+00_JPRB, 0.42266E+00_JPRB, 0.45568E+00_JPRB, 0.47838E+00_JPRB ,&
&  0.48786E+00_JPRB, 0.49647E+00_JPRB, 0.49142E+00_JPRB, 0.48007E+00_JPRB, 0.45816E+00_JPRB, 0.43821E+00_JPRB, 0.42232E+00_JPRB ,&
&  0.40866E+00_JPRB, 0.39691E+00_JPRB, 0.38726E+00_JPRB, 0.37922E+00_JPRB, 0.37170E+00_JPRB, 0.36845E+00_JPRB, 0.36949E+00_JPRB ,&
&  0.37163E+00_JPRB, 0.37436E+00_JPRB, 0.37839E+00_JPRB, 0.38512E+00_JPRB, 0.39182E+00_JPRB, 0.39967E+00_JPRB, 0.41700E+00_JPRB ,&
&  0.44421E+00_JPRB, 0.46376E+00_JPRB, 0.47771E+00_JPRB, 0.48061E+00_JPRB, 0.47325E+00_JPRB, 0.45867E+00_JPRB, 0.43743E+00_JPRB ,&
&  0.41587E+00_JPRB, 0.39925E+00_JPRB, 0.38819E+00_JPRB, 0.38114E+00_JPRB, 0.37681E+00_JPRB, 0.37287E+00_JPRB, 0.36974E+00_JPRB ,&
&  0.36669E+00_JPRB, 0.36443E+00_JPRB, 0.36472E+00_JPRB, 0.36767E+00_JPRB, 0.37175E+00_JPRB, 0.37545E+00_JPRB, 0.37764E+00_JPRB ,&
&  0.37884E+00_JPRB, 0.37990E+00_JPRB, 0.38316E+00_JPRB, 0.38781E+00_JPRB, 0.39285E+00_JPRB, 0.39619E+00_JPRB, 0.40043E+00_JPRB ,&
&  0.40607E+00_JPRB, 0.41284E+00_JPRB, 0.42381E+00_JPRB, 0.44569E+00_JPRB, 0.46869E+00_JPRB, 0.48100E+00_JPRB, 0.49002E+00_JPRB ,&
&  0.50150E+00_JPRB, 0.50636E+00_JPRB, 0.51426E+00_JPRB, 0.46161E+00_JPRB, 0.52616E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.61527E+00_JPRB, 0.57933E+00_JPRB, 0.53239E+00_JPRB, 0.49972E+00_JPRB, 0.47632E+00_JPRB, 0.45200E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.42431E+00_JPRB, 0.41924E+00_JPRB, 0.41499E+00_JPRB, 0.40911E+00_JPRB, 0.40315E+00_JPRB, 0.39938E+00_JPRB, 0.39483E+00_JPRB ,&
&  0.38983E+00_JPRB, 0.38494E+00_JPRB, 0.37998E+00_JPRB, 0.37625E+00_JPRB, 0.37107E+00_JPRB, 0.36676E+00_JPRB, 0.36246E+00_JPRB ,&
&  0.35728E+00_JPRB, 0.35271E+00_JPRB, 0.34767E+00_JPRB, 0.35175E+00_JPRB, 0.35972E+00_JPRB, 0.36580E+00_JPRB, 0.37406E+00_JPRB ,&
&  0.40232E+00_JPRB, 0.39275E+00_JPRB, 0.42787E+00_JPRB, 0.46471E+00_JPRB, 0.48982E+00_JPRB, 0.51239E+00_JPRB, 0.52343E+00_JPRB ,&
&  0.52013E+00_JPRB, 0.53181E+00_JPRB, 0.52930E+00_JPRB, 0.51369E+00_JPRB, 0.50140E+00_JPRB, 0.47541E+00_JPRB, 0.45199E+00_JPRB ,&
&  0.43502E+00_JPRB, 0.42399E+00_JPRB, 0.41423E+00_JPRB, 0.40301E+00_JPRB, 0.39591E+00_JPRB, 0.39433E+00_JPRB, 0.39661E+00_JPRB ,&
&  0.40026E+00_JPRB, 0.40653E+00_JPRB, 0.41262E+00_JPRB, 0.41882E+00_JPRB, 0.42462E+00_JPRB, 0.44768E+00_JPRB, 0.49387E+00_JPRB ,&
&  0.53094E+00_JPRB, 0.55232E+00_JPRB, 0.56095E+00_JPRB, 0.56231E+00_JPRB, 0.54850E+00_JPRB, 0.52882E+00_JPRB, 0.50567E+00_JPRB ,&
&  0.48237E+00_JPRB, 0.45839E+00_JPRB, 0.43636E+00_JPRB, 0.41933E+00_JPRB, 0.40572E+00_JPRB, 0.39637E+00_JPRB, 0.39162E+00_JPRB ,&
&  0.39129E+00_JPRB, 0.39452E+00_JPRB, 0.39982E+00_JPRB, 0.40540E+00_JPRB, 0.40918E+00_JPRB, 0.41017E+00_JPRB, 0.40980E+00_JPRB ,&
&  0.40923E+00_JPRB, 0.40892E+00_JPRB, 0.41055E+00_JPRB, 0.41414E+00_JPRB, 0.41916E+00_JPRB, 0.42433E+00_JPRB, 0.42978E+00_JPRB ,&
&  0.43581E+00_JPRB, 0.44257E+00_JPRB, 0.45314E+00_JPRB, 0.48680E+00_JPRB, 0.50398E+00_JPRB, 0.52447E+00_JPRB, 0.54260E+00_JPRB ,&
&  0.56613E+00_JPRB, 0.57879E+00_JPRB, 0.57544E+00_JPRB, 0.56281E+00_JPRB, 0.59087E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.68092E+00_JPRB, 0.58995E+00_JPRB, 0.53411E+00_JPRB, 0.50635E+00_JPRB, 0.49617E+00_JPRB, 0.47978E+00_JPRB, 0.46294E+00_JPRB ,&
&  0.44784E+00_JPRB, 0.43804E+00_JPRB, 0.43268E+00_JPRB, 0.42583E+00_JPRB, 0.41855E+00_JPRB, 0.41249E+00_JPRB, 0.40582E+00_JPRB ,&
&  0.39854E+00_JPRB, 0.39416E+00_JPRB, 0.39051E+00_JPRB, 0.38800E+00_JPRB, 0.38601E+00_JPRB, 0.38155E+00_JPRB, 0.37627E+00_JPRB ,&
&  0.37222E+00_JPRB, 0.36715E+00_JPRB, 0.36666E+00_JPRB, 0.37292E+00_JPRB, 0.37807E+00_JPRB, 0.38055E+00_JPRB, 0.39232E+00_JPRB ,&
&  0.43062E+00_JPRB, 0.44237E+00_JPRB, 0.48528E+00_JPRB, 0.52130E+00_JPRB, 0.55590E+00_JPRB, 0.60847E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.63586E+00_JPRB, 0.64680E+00_JPRB, 0.63826E+00_JPRB, 0.60336E+00_JPRB, 0.56324E+00_JPRB, 0.53623E+00_JPRB, 0.51596E+00_JPRB ,&
&  0.49859E+00_JPRB, 0.47875E+00_JPRB, 0.45830E+00_JPRB, 0.44208E+00_JPRB, 0.43295E+00_JPRB, 0.43003E+00_JPRB, 0.42994E+00_JPRB ,&
&  0.43179E+00_JPRB, 0.43672E+00_JPRB, 0.44075E+00_JPRB, 0.44748E+00_JPRB, 0.47586E+00_JPRB, 0.55028E+00_JPRB, 0.65376E+00_JPRB ,&
&  0.70195E+00_JPRB, 0.71713E+00_JPRB, 0.72439E+00_JPRB, 0.72064E+00_JPRB, 0.68459E+00_JPRB, 0.64577E+00_JPRB, 0.60799E+00_JPRB ,&
&  0.57234E+00_JPRB, 0.53494E+00_JPRB, 0.50371E+00_JPRB, 0.47882E+00_JPRB, 0.45898E+00_JPRB, 0.44666E+00_JPRB, 0.44143E+00_JPRB ,&
&  0.43990E+00_JPRB, 0.43951E+00_JPRB, 0.43919E+00_JPRB, 0.43716E+00_JPRB, 0.43314E+00_JPRB, 0.43150E+00_JPRB, 0.43068E+00_JPRB ,&
&  0.43039E+00_JPRB, 0.43154E+00_JPRB, 0.43507E+00_JPRB, 0.44031E+00_JPRB, 0.44818E+00_JPRB, 0.45337E+00_JPRB, 0.46032E+00_JPRB ,&
&  0.46894E+00_JPRB, 0.48244E+00_JPRB, 0.48276E+00_JPRB, 0.51121E+00_JPRB, 0.56980E+00_JPRB, 0.59260E+00_JPRB, 0.61978E+00_JPRB ,&
&  0.64822E+00_JPRB, 0.64606E+00_JPRB, 0.62219E+00_JPRB, 0.62356E+00_JPRB, 0.74940E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.55549E+00_JPRB, 0.52978E+00_JPRB, 0.51410E+00_JPRB, 0.50862E+00_JPRB, 0.50666E+00_JPRB, 0.49666E+00_JPRB, 0.47866E+00_JPRB ,&
&  0.46065E+00_JPRB, 0.44563E+00_JPRB, 0.43492E+00_JPRB, 0.42703E+00_JPRB, 0.42168E+00_JPRB, 0.41633E+00_JPRB, 0.41047E+00_JPRB ,&
&  0.40440E+00_JPRB, 0.40174E+00_JPRB, 0.39995E+00_JPRB, 0.39682E+00_JPRB, 0.38926E+00_JPRB, 0.38308E+00_JPRB, 0.38110E+00_JPRB ,&
&  0.37994E+00_JPRB, 0.38085E+00_JPRB, 0.38606E+00_JPRB, 0.39198E+00_JPRB, 0.39427E+00_JPRB, 0.39596E+00_JPRB, 0.40678E+00_JPRB ,&
&  0.44400E+00_JPRB, 0.52857E+00_JPRB, 0.53770E+00_JPRB, 0.58494E+00_JPRB, 0.65612E+00_JPRB, 0.75374E+00_JPRB, 0.85058E+00_JPRB ,&
&  0.86743E+00_JPRB, 0.82912E+00_JPRB, 0.76045E+00_JPRB, 0.68919E+00_JPRB, 0.63294E+00_JPRB, 0.60008E+00_JPRB, 0.57490E+00_JPRB ,&
&  0.55021E+00_JPRB, 0.52527E+00_JPRB, 0.50397E+00_JPRB, 0.48808E+00_JPRB, 0.47737E+00_JPRB, 0.47352E+00_JPRB, 0.47343E+00_JPRB ,&
&  0.47348E+00_JPRB, 0.47252E+00_JPRB, 0.47416E+00_JPRB, 0.49942E+00_JPRB, 0.61386E+00_JPRB, 0.77904E+00_JPRB, 0.92327E+00_JPRB ,&
&  0.97612E+00_JPRB, 0.10127E+01_JPRB, 0.10017E+01_JPRB, 0.94803E+00_JPRB, 0.86464E+00_JPRB, 0.78225E+00_JPRB, 0.69677E+00_JPRB ,&
&  0.63500E+00_JPRB, 0.58974E+00_JPRB, 0.55724E+00_JPRB, 0.53017E+00_JPRB, 0.50593E+00_JPRB, 0.48787E+00_JPRB, 0.47618E+00_JPRB ,&
&  0.46810E+00_JPRB, 0.46128E+00_JPRB, 0.45467E+00_JPRB, 0.44830E+00_JPRB, 0.44296E+00_JPRB, 0.43943E+00_JPRB, 0.44114E+00_JPRB ,&
&  0.44305E+00_JPRB, 0.44402E+00_JPRB, 0.45479E+00_JPRB, 0.46984E+00_JPRB, 0.48477E+00_JPRB, 0.49428E+00_JPRB, 0.50012E+00_JPRB ,&
&  0.50696E+00_JPRB, 0.52249E+00_JPRB, 0.51782E+00_JPRB, 0.52934E+00_JPRB, 0.59121E+00_JPRB, 0.65055E+00_JPRB, 0.66400E+00_JPRB ,&
&  0.65712E+00_JPRB, 0.63427E+00_JPRB, 0.61451E+00_JPRB, 0.60369E+00_JPRB, 0.59602E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.55795E+00_JPRB, 0.53832E+00_JPRB, 0.52869E+00_JPRB, 0.51993E+00_JPRB, 0.51159E+00_JPRB, 0.49908E+00_JPRB, 0.48150E+00_JPRB ,&
&  0.46862E+00_JPRB, 0.45851E+00_JPRB, 0.44660E+00_JPRB, 0.43621E+00_JPRB, 0.42905E+00_JPRB, 0.42345E+00_JPRB, 0.41685E+00_JPRB ,&
&  0.41259E+00_JPRB, 0.40996E+00_JPRB, 0.40088E+00_JPRB, 0.39191E+00_JPRB, 0.38822E+00_JPRB, 0.38782E+00_JPRB, 0.39143E+00_JPRB ,&
&  0.39537E+00_JPRB, 0.40044E+00_JPRB, 0.40436E+00_JPRB, 0.40753E+00_JPRB, 0.40751E+00_JPRB, 0.41036E+00_JPRB, 0.42601E+00_JPRB ,&
&  0.47747E+00_JPRB, 0.57417E+00_JPRB, 0.66925E+00_JPRB, 0.69401E+00_JPRB, 0.76458E+00_JPRB, 0.88626E+00_JPRB, 0.10149E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.93442E+00_JPRB, 0.83238E+00_JPRB, 0.76112E+00_JPRB, 0.70761E+00_JPRB, 0.66776E+00_JPRB, 0.62510E+00_JPRB ,&
&  0.59016E+00_JPRB, 0.56721E+00_JPRB, 0.54512E+00_JPRB, 0.52524E+00_JPRB, 0.51742E+00_JPRB, 0.51528E+00_JPRB, 0.51225E+00_JPRB ,&
&  0.50804E+00_JPRB, 0.50483E+00_JPRB, 0.51950E+00_JPRB, 0.59421E+00_JPRB, 0.77520E+00_JPRB, 0.93926E+00_JPRB, 0.11623E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.13889E+01_JPRB, 0.12946E+01_JPRB, 0.11296E+01_JPRB, 0.95395E+00_JPRB, 0.80383E+00_JPRB, 0.71096E+00_JPRB ,&
&  0.65195E+00_JPRB, 0.60160E+00_JPRB, 0.57019E+00_JPRB, 0.54998E+00_JPRB, 0.53457E+00_JPRB, 0.52155E+00_JPRB, 0.50577E+00_JPRB ,&
&  0.48843E+00_JPRB, 0.47525E+00_JPRB, 0.46671E+00_JPRB, 0.45968E+00_JPRB, 0.45545E+00_JPRB, 0.45413E+00_JPRB, 0.45677E+00_JPRB ,&
&  0.46305E+00_JPRB, 0.47673E+00_JPRB, 0.50007E+00_JPRB, 0.51255E+00_JPRB, 0.53297E+00_JPRB, 0.55800E+00_JPRB, 0.57367E+00_JPRB ,&
&  0.55955E+00_JPRB, 0.55089E+00_JPRB, 0.53797E+00_JPRB, 0.54046E+00_JPRB, 0.61702E+00_JPRB, 0.68737E+00_JPRB, 0.66906E+00_JPRB ,&
&  0.65272E+00_JPRB, 0.63277E+00_JPRB, 0.61113E+00_JPRB, 0.58786E+00_JPRB, 0.58627E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.57023E+00_JPRB, 0.55094E+00_JPRB, 0.53573E+00_JPRB, 0.51913E+00_JPRB, 0.50717E+00_JPRB, 0.49880E+00_JPRB, 0.48756E+00_JPRB ,&
&  0.47689E+00_JPRB, 0.46692E+00_JPRB, 0.45637E+00_JPRB, 0.44600E+00_JPRB, 0.43727E+00_JPRB, 0.42671E+00_JPRB, 0.41833E+00_JPRB ,&
&  0.41041E+00_JPRB, 0.39998E+00_JPRB, 0.39305E+00_JPRB, 0.39120E+00_JPRB, 0.39501E+00_JPRB, 0.40018E+00_JPRB, 0.40673E+00_JPRB ,&
&  0.41459E+00_JPRB, 0.41985E+00_JPRB, 0.42109E+00_JPRB, 0.42337E+00_JPRB, 0.42151E+00_JPRB, 0.42350E+00_JPRB, 0.44300E+00_JPRB ,&
&  0.49429E+00_JPRB, 0.46795E+00_JPRB, 0.71774E+00_JPRB, 0.83807E+00_JPRB, 0.95993E+00_JPRB, 0.10723E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.10401E+01_JPRB, 0.92224E+00_JPRB, 0.84419E+00_JPRB, 0.78082E+00_JPRB, 0.73067E+00_JPRB, 0.69054E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.62302E+00_JPRB, 0.60040E+00_JPRB, 0.56858E+00_JPRB, 0.54363E+00_JPRB, 0.53023E+00_JPRB, 0.52499E+00_JPRB ,&
&  0.51952E+00_JPRB, 0.52791E+00_JPRB, 0.57057E+00_JPRB, 0.69071E+00_JPRB, 0.94319E+00_JPRB, 0.11617E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.14774E+01_JPRB, 0.13769E+01_JPRB, 0.11799E+01_JPRB, 0.98474E+00_JPRB, 0.84108E+00_JPRB, 0.72701E+00_JPRB ,&
&  0.66080E+00_JPRB, 0.62281E+00_JPRB, 0.60085E+00_JPRB, 0.58403E+00_JPRB, 0.56553E+00_JPRB, 0.54246E+00_JPRB, 0.51696E+00_JPRB ,&
&  0.49789E+00_JPRB, 0.48790E+00_JPRB, 0.48147E+00_JPRB, 0.47929E+00_JPRB, 0.47854E+00_JPRB, 0.47919E+00_JPRB, 0.48464E+00_JPRB ,&
&  0.49637E+00_JPRB, 0.51831E+00_JPRB, 0.53441E+00_JPRB, 0.54747E+00_JPRB, 0.56460E+00_JPRB, 0.58238E+00_JPRB, 0.59848E+00_JPRB ,&
&  0.60212E+00_JPRB, 0.58880E+00_JPRB, 0.57043E+00_JPRB, 0.57715E+00_JPRB, 0.60745E+00_JPRB, 0.66530E+00_JPRB, 0.66365E+00_JPRB ,&
&  0.65782E+00_JPRB, 0.63981E+00_JPRB, 0.61018E+00_JPRB, 0.59686E+00_JPRB, 0.59211E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.57749E+00_JPRB, 0.56365E+00_JPRB, 0.54620E+00_JPRB, 0.52930E+00_JPRB, 0.52227E+00_JPRB, 0.51714E+00_JPRB, 0.50448E+00_JPRB ,&
&  0.49375E+00_JPRB, 0.48591E+00_JPRB, 0.47615E+00_JPRB, 0.46345E+00_JPRB, 0.44847E+00_JPRB, 0.42903E+00_JPRB, 0.41118E+00_JPRB ,&
&  0.40271E+00_JPRB, 0.39579E+00_JPRB, 0.39877E+00_JPRB, 0.40407E+00_JPRB, 0.41236E+00_JPRB, 0.41911E+00_JPRB, 0.42683E+00_JPRB ,&
&  0.43244E+00_JPRB, 0.43495E+00_JPRB, 0.43877E+00_JPRB, 0.44184E+00_JPRB, 0.44257E+00_JPRB, 0.44775E+00_JPRB, 0.46965E+00_JPRB ,&
&  0.52545E+00_JPRB, 0.76009E+00_JPRB, 0.66859E+00_JPRB, 0.10510E+01_JPRB, 0.12451E+01_JPRB, 0.13359E+01_JPRB, 0.12652E+01_JPRB ,&
&  0.12309E+01_JPRB, 0.11655E+01_JPRB, 0.10311E+01_JPRB, 0.92210E+00_JPRB, 0.83731E+00_JPRB, 0.78103E+00_JPRB, 0.74189E+00_JPRB ,&
&  0.71441E+00_JPRB, 0.68137E+00_JPRB, 0.64063E+00_JPRB, 0.59911E+00_JPRB, 0.57165E+00_JPRB, 0.55749E+00_JPRB, 0.55344E+00_JPRB ,&
&  0.55190E+00_JPRB, 0.57589E+00_JPRB, 0.65773E+00_JPRB, 0.88444E+00_JPRB, 0.11563E+01_JPRB, 0.13134E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.13471E+01_JPRB, 0.12687E+01_JPRB, 0.10738E+01_JPRB, 0.92289E+00_JPRB, 0.79978E+00_JPRB, 0.71094E+00_JPRB ,&
&  0.65704E+00_JPRB, 0.62401E+00_JPRB, 0.60339E+00_JPRB, 0.58742E+00_JPRB, 0.56968E+00_JPRB, 0.55034E+00_JPRB, 0.53404E+00_JPRB ,&
&  0.52254E+00_JPRB, 0.51364E+00_JPRB, 0.50770E+00_JPRB, 0.50781E+00_JPRB, 0.50685E+00_JPRB, 0.50889E+00_JPRB, 0.51507E+00_JPRB ,&
&  0.52716E+00_JPRB, 0.55209E+00_JPRB, 0.56094E+00_JPRB, 0.57511E+00_JPRB, 0.58027E+00_JPRB, 0.58270E+00_JPRB, 0.58654E+00_JPRB ,&
&  0.61470E+00_JPRB, 0.61519E+00_JPRB, 0.60607E+00_JPRB, 0.60811E+00_JPRB, 0.62871E+00_JPRB, 0.66865E+00_JPRB, 0.66330E+00_JPRB ,&
&  0.66024E+00_JPRB, 0.64360E+00_JPRB, 0.62251E+00_JPRB, 0.60590E+00_JPRB, 0.59448E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.58477E+00_JPRB, 0.56893E+00_JPRB, 0.55641E+00_JPRB, 0.55138E+00_JPRB, 0.54493E+00_JPRB, 0.53846E+00_JPRB, 0.52498E+00_JPRB ,&
&  0.51041E+00_JPRB, 0.50163E+00_JPRB, 0.48936E+00_JPRB, 0.47788E+00_JPRB, 0.45478E+00_JPRB, 0.42600E+00_JPRB, 0.41480E+00_JPRB ,&
&  0.40832E+00_JPRB, 0.40967E+00_JPRB, 0.41912E+00_JPRB, 0.42301E+00_JPRB, 0.42744E+00_JPRB, 0.43641E+00_JPRB, 0.44527E+00_JPRB ,&
&  0.45168E+00_JPRB, 0.45195E+00_JPRB, 0.45667E+00_JPRB, 0.46289E+00_JPRB, 0.46856E+00_JPRB, 0.48787E+00_JPRB, 0.51253E+00_JPRB ,&
&  0.58624E+00_JPRB, 0.76215E+00_JPRB, 0.68574E+00_JPRB, 0.13148E+01_JPRB, 0.15437E+01_JPRB, 0.15554E+01_JPRB, 0.14378E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13383E+01_JPRB, 0.11305E+01_JPRB, 0.98029E+00_JPRB, 0.87124E+00_JPRB, 0.81225E+00_JPRB, 0.77617E+00_JPRB ,&
&  0.74877E+00_JPRB, 0.71745E+00_JPRB, 0.67461E+00_JPRB, 0.63471E+00_JPRB, 0.61254E+00_JPRB, 0.59958E+00_JPRB, 0.59448E+00_JPRB ,&
&  0.59607E+00_JPRB, 0.64768E+00_JPRB, 0.79144E+00_JPRB, 0.10789E+01_JPRB, 0.13499E+01_JPRB, 0.13792E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.12311E+01_JPRB, 0.11354E+01_JPRB, 0.95295E+00_JPRB, 0.82532E+00_JPRB, 0.75201E+00_JPRB, 0.67977E+00_JPRB ,&
&  0.63447E+00_JPRB, 0.60443E+00_JPRB, 0.58978E+00_JPRB, 0.57879E+00_JPRB, 0.56929E+00_JPRB, 0.56303E+00_JPRB, 0.55639E+00_JPRB ,&
&  0.55008E+00_JPRB, 0.54067E+00_JPRB, 0.53412E+00_JPRB, 0.52610E+00_JPRB, 0.52811E+00_JPRB, 0.53224E+00_JPRB, 0.53836E+00_JPRB ,&
&  0.55867E+00_JPRB, 0.59373E+00_JPRB, 0.59424E+00_JPRB, 0.60693E+00_JPRB, 0.60431E+00_JPRB, 0.59210E+00_JPRB, 0.59853E+00_JPRB ,&
&  0.62032E+00_JPRB, 0.62509E+00_JPRB, 0.63489E+00_JPRB, 0.62713E+00_JPRB, 0.64058E+00_JPRB, 0.66551E+00_JPRB, 0.66586E+00_JPRB ,&
&  0.66971E+00_JPRB, 0.65846E+00_JPRB, 0.64196E+00_JPRB, 0.61708E+00_JPRB, 0.60094E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.57933E+00_JPRB, 0.56215E+00_JPRB, 0.55666E+00_JPRB, 0.55568E+00_JPRB, 0.55153E+00_JPRB, 0.54201E+00_JPRB, 0.52567E+00_JPRB ,&
&  0.50597E+00_JPRB, 0.49212E+00_JPRB, 0.47704E+00_JPRB, 0.45859E+00_JPRB, 0.43129E+00_JPRB, 0.40555E+00_JPRB, 0.40508E+00_JPRB ,&
&  0.42055E+00_JPRB, 0.42970E+00_JPRB, 0.43786E+00_JPRB, 0.44658E+00_JPRB, 0.45619E+00_JPRB, 0.46816E+00_JPRB, 0.47712E+00_JPRB ,&
&  0.47970E+00_JPRB, 0.48441E+00_JPRB, 0.49469E+00_JPRB, 0.50740E+00_JPRB, 0.52424E+00_JPRB, 0.55022E+00_JPRB, 0.58616E+00_JPRB ,&
&  0.68284E+00_JPRB, 0.81155E+00_JPRB, 0.83418E+00_JPRB, 0.13925E+01_JPRB, 0.16707E+01_JPRB, 0.15700E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.11965E+01_JPRB, 0.11377E+01_JPRB, 0.97780E+00_JPRB, 0.87758E+00_JPRB, 0.82676E+00_JPRB, 0.79564E+00_JPRB ,&
&  0.77306E+00_JPRB, 0.74510E+00_JPRB, 0.70710E+00_JPRB, 0.68627E+00_JPRB, 0.67130E+00_JPRB, 0.65454E+00_JPRB, 0.64750E+00_JPRB ,&
&  0.66887E+00_JPRB, 0.76134E+00_JPRB, 0.98183E+00_JPRB, 0.13258E+01_JPRB, 0.14426E+01_JPRB, 0.13329E+01_JPRB, 0.11239E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.10525E+01_JPRB, 0.97101E+00_JPRB, 0.83834E+00_JPRB, 0.72604E+00_JPRB, 0.71998E+00_JPRB, 0.65324E+00_JPRB ,&
&  0.61550E+00_JPRB, 0.59141E+00_JPRB, 0.58059E+00_JPRB, 0.57287E+00_JPRB, 0.56994E+00_JPRB, 0.56753E+00_JPRB, 0.56322E+00_JPRB ,&
&  0.56077E+00_JPRB, 0.56061E+00_JPRB, 0.55412E+00_JPRB, 0.55387E+00_JPRB, 0.55696E+00_JPRB, 0.56334E+00_JPRB, 0.57776E+00_JPRB ,&
&  0.61073E+00_JPRB, 0.64582E+00_JPRB, 0.64986E+00_JPRB, 0.65691E+00_JPRB, 0.64504E+00_JPRB, 0.63088E+00_JPRB, 0.62922E+00_JPRB ,&
&  0.63177E+00_JPRB, 0.65532E+00_JPRB, 0.66964E+00_JPRB, 0.66551E+00_JPRB, 0.66888E+00_JPRB, 0.67665E+00_JPRB, 0.68424E+00_JPRB ,&
&  0.69402E+00_JPRB, 0.68562E+00_JPRB, 0.68179E+00_JPRB, 0.62505E+00_JPRB, 0.60418E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.55434E+00_JPRB, 0.54115E+00_JPRB, 0.52715E+00_JPRB, 0.52309E+00_JPRB, 0.51838E+00_JPRB, 0.50378E+00_JPRB, 0.48505E+00_JPRB ,&
&  0.46361E+00_JPRB, 0.44762E+00_JPRB, 0.42720E+00_JPRB, 0.41015E+00_JPRB, 0.40947E+00_JPRB, 0.42169E+00_JPRB, 0.44595E+00_JPRB ,&
&  0.47018E+00_JPRB, 0.48419E+00_JPRB, 0.49492E+00_JPRB, 0.50563E+00_JPRB, 0.51562E+00_JPRB, 0.52090E+00_JPRB, 0.52094E+00_JPRB ,&
&  0.52680E+00_JPRB, 0.54071E+00_JPRB, 0.56126E+00_JPRB, 0.58441E+00_JPRB, 0.61289E+00_JPRB, 0.65388E+00_JPRB, 0.72229E+00_JPRB ,&
&  0.85909E+00_JPRB, 0.92690E+00_JPRB, 0.53239E+00_JPRB, 0.10141E+01_JPRB, 0.14467E+01_JPRB, 0.14299E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.10290E+01_JPRB, 0.99214E+00_JPRB, 0.94818E+00_JPRB, 0.86423E+00_JPRB, 0.82852E+00_JPRB, 0.81278E+00_JPRB ,&
&  0.80521E+00_JPRB, 0.78420E+00_JPRB, 0.76852E+00_JPRB, 0.76384E+00_JPRB, 0.74528E+00_JPRB, 0.71970E+00_JPRB, 0.72212E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.92056E+00_JPRB, 0.12381E+01_JPRB, 0.14240E+01_JPRB, 0.14907E+01_JPRB, 0.12353E+01_JPRB, 0.98176E+00_JPRB ,&
&  0.87055E+00_JPRB, 0.88934E+00_JPRB, 0.85026E+00_JPRB, 0.76261E+00_JPRB, 0.67771E+00_JPRB, 0.64034E+00_JPRB, 0.63944E+00_JPRB ,&
&  0.60518E+00_JPRB, 0.58059E+00_JPRB, 0.56640E+00_JPRB, 0.55660E+00_JPRB, 0.55155E+00_JPRB, 0.54987E+00_JPRB, 0.55201E+00_JPRB ,&
&  0.56074E+00_JPRB, 0.56507E+00_JPRB, 0.57167E+00_JPRB, 0.58220E+00_JPRB, 0.59299E+00_JPRB, 0.60762E+00_JPRB, 0.63499E+00_JPRB ,&
&  0.67426E+00_JPRB, 0.70398E+00_JPRB, 0.72674E+00_JPRB, 0.71525E+00_JPRB, 0.69284E+00_JPRB, 0.68131E+00_JPRB, 0.69051E+00_JPRB ,&
&  0.69266E+00_JPRB, 0.70906E+00_JPRB, 0.68208E+00_JPRB, 0.67027E+00_JPRB, 0.69917E+00_JPRB, 0.70541E+00_JPRB, 0.74357E+00_JPRB ,&
&  0.79325E+00_JPRB, 0.87086E+00_JPRB, 0.88922E+00_JPRB, 0.62301E+00_JPRB, 0.58288E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.51165E+00_JPRB, 0.49368E+00_JPRB, 0.46795E+00_JPRB, 0.45836E+00_JPRB, 0.46306E+00_JPRB, 0.45696E+00_JPRB, 0.45124E+00_JPRB ,&
&  0.44561E+00_JPRB, 0.44128E+00_JPRB, 0.43688E+00_JPRB, 0.44656E+00_JPRB, 0.47281E+00_JPRB, 0.50897E+00_JPRB, 0.54127E+00_JPRB ,&
&  0.56402E+00_JPRB, 0.57459E+00_JPRB, 0.57779E+00_JPRB, 0.58299E+00_JPRB, 0.58669E+00_JPRB, 0.59022E+00_JPRB, 0.59559E+00_JPRB ,&
&  0.61538E+00_JPRB, 0.64621E+00_JPRB, 0.68536E+00_JPRB, 0.72736E+00_JPRB, 0.77683E+00_JPRB, 0.84753E+00_JPRB, 0.95957E+00_JPRB ,&
&  0.11283E+01_JPRB, 0.84550E+00_JPRB, 0.62783E+00_JPRB, 0.12889E+01_JPRB, 0.14546E+01_JPRB, 0.14140E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.98569E+00_JPRB, 0.91868E+00_JPRB, 0.94000E+00_JPRB, 0.85995E+00_JPRB, 0.83677E+00_JPRB, 0.83893E+00_JPRB ,&
&  0.84157E+00_JPRB, 0.84548E+00_JPRB, 0.84837E+00_JPRB, 0.83784E+00_JPRB, 0.81451E+00_JPRB, 0.80609E+00_JPRB, 0.83699E+00_JPRB ,&
&  0.94511E+00_JPRB, 0.11660E+01_JPRB, 0.15365E+01_JPRB, 0.15253E+01_JPRB, 0.15479E+01_JPRB, 0.11698E+01_JPRB, 0.93521E+00_JPRB ,&
&  0.80252E+00_JPRB, 0.78806E+00_JPRB, 0.77504E+00_JPRB, 0.73213E+00_JPRB, 0.66772E+00_JPRB, 0.61570E+00_JPRB, 0.62505E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.56269E+00_JPRB, 0.54254E+00_JPRB, 0.53336E+00_JPRB, 0.53550E+00_JPRB, 0.54164E+00_JPRB, 0.54470E+00_JPRB ,&
&  0.55538E+00_JPRB, 0.56294E+00_JPRB, 0.58161E+00_JPRB, 0.60239E+00_JPRB, 0.62411E+00_JPRB, 0.65124E+00_JPRB, 0.69145E+00_JPRB ,&
&  0.72694E+00_JPRB, 0.75181E+00_JPRB, 0.76750E+00_JPRB, 0.77197E+00_JPRB, 0.75238E+00_JPRB, 0.72681E+00_JPRB, 0.73172E+00_JPRB ,&
&  0.72308E+00_JPRB, 0.72670E+00_JPRB, 0.71362E+00_JPRB, 0.72587E+00_JPRB, 0.73742E+00_JPRB, 0.75080E+00_JPRB, 0.78522E+00_JPRB ,&
&  0.82824E+00_JPRB, 0.83454E+00_JPRB, 0.75904E+00_JPRB, 0.57613E+00_JPRB, 0.53302E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.49230E+00_JPRB, 0.46924E+00_JPRB, 0.45888E+00_JPRB, 0.45811E+00_JPRB, 0.46478E+00_JPRB, 0.46847E+00_JPRB, 0.46794E+00_JPRB ,&
&  0.47032E+00_JPRB, 0.48142E+00_JPRB, 0.49526E+00_JPRB, 0.53624E+00_JPRB, 0.57767E+00_JPRB, 0.60742E+00_JPRB, 0.62586E+00_JPRB ,&
&  0.64292E+00_JPRB, 0.65225E+00_JPRB, 0.65653E+00_JPRB, 0.66260E+00_JPRB, 0.66758E+00_JPRB, 0.68071E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.74811E+00_JPRB, 0.79179E+00_JPRB, 0.84562E+00_JPRB, 0.91162E+00_JPRB, 0.99266E+00_JPRB, 0.10687E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.88604E+00_JPRB, 0.53682E+00_JPRB, 0.12795E+01_JPRB, 0.15152E+01_JPRB, 0.13363E+01_JPRB, 0.12477E+01_JPRB, 0.11340E+01_JPRB ,&
&  0.99575E+00_JPRB, 0.94242E+00_JPRB, 0.90510E+00_JPRB, 0.93106E+00_JPRB, 0.86858E+00_JPRB, 0.85811E+00_JPRB, 0.86185E+00_JPRB ,&
&  0.87538E+00_JPRB, 0.89800E+00_JPRB, 0.90178E+00_JPRB, 0.89321E+00_JPRB, 0.89383E+00_JPRB, 0.92337E+00_JPRB, 0.10169E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.14783E+01_JPRB, 0.17379E+01_JPRB, 0.15370E+01_JPRB, 0.15599E+01_JPRB, 0.12733E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.82564E+00_JPRB, 0.72269E+00_JPRB, 0.71042E+00_JPRB, 0.72681E+00_JPRB, 0.68242E+00_JPRB, 0.61644E+00_JPRB, 0.61455E+00_JPRB ,&
&  0.57213E+00_JPRB, 0.54249E+00_JPRB, 0.52978E+00_JPRB, 0.53037E+00_JPRB, 0.53802E+00_JPRB, 0.54849E+00_JPRB, 0.55904E+00_JPRB ,&
&  0.56357E+00_JPRB, 0.57600E+00_JPRB, 0.59095E+00_JPRB, 0.61801E+00_JPRB, 0.64839E+00_JPRB, 0.69390E+00_JPRB, 0.73477E+00_JPRB ,&
&  0.76202E+00_JPRB, 0.78668E+00_JPRB, 0.81067E+00_JPRB, 0.82945E+00_JPRB, 0.84542E+00_JPRB, 0.81647E+00_JPRB, 0.78768E+00_JPRB ,&
&  0.78170E+00_JPRB, 0.78244E+00_JPRB, 0.79722E+00_JPRB, 0.81743E+00_JPRB, 0.78573E+00_JPRB, 0.77077E+00_JPRB, 0.79501E+00_JPRB ,&
&  0.79596E+00_JPRB, 0.75740E+00_JPRB, 0.65744E+00_JPRB, 0.55090E+00_JPRB, 0.51349E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.53444E+00_JPRB, 0.52195E+00_JPRB, 0.50784E+00_JPRB, 0.51024E+00_JPRB, 0.51322E+00_JPRB, 0.53072E+00_JPRB, 0.53146E+00_JPRB ,&
&  0.54478E+00_JPRB, 0.58449E+00_JPRB, 0.63165E+00_JPRB, 0.65537E+00_JPRB, 0.67596E+00_JPRB, 0.69664E+00_JPRB, 0.71253E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.73659E+00_JPRB, 0.74833E+00_JPRB, 0.76204E+00_JPRB, 0.78218E+00_JPRB, 0.81047E+00_JPRB, 0.85390E+00_JPRB ,&
&  0.89593E+00_JPRB, 0.94276E+00_JPRB, 0.10029E+01_JPRB, 0.10804E+01_JPRB, 0.11638E+01_JPRB, 0.12073E+01_JPRB, 0.12313E+01_JPRB ,&
&  0.79851E+00_JPRB, 0.12450E+01_JPRB, 0.14546E+01_JPRB, 0.13580E+01_JPRB, 0.12303E+01_JPRB, 0.11510E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.92629E+00_JPRB, 0.90172E+00_JPRB, 0.95544E+00_JPRB, 0.89669E+00_JPRB, 0.89468E+00_JPRB, 0.90139E+00_JPRB ,&
&  0.93987E+00_JPRB, 0.96703E+00_JPRB, 0.97774E+00_JPRB, 0.10005E+01_JPRB, 0.10400E+01_JPRB, 0.11292E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.14733E+01_JPRB, 0.16707E+01_JPRB, 0.17747E+01_JPRB, 0.15123E+01_JPRB, 0.15375E+01_JPRB, 0.14028E+01_JPRB, 0.11787E+01_JPRB ,&
&  0.83403E+00_JPRB, 0.70698E+00_JPRB, 0.72525E+00_JPRB, 0.74833E+00_JPRB, 0.69659E+00_JPRB, 0.64742E+00_JPRB, 0.61052E+00_JPRB ,&
&  0.56993E+00_JPRB, 0.54460E+00_JPRB, 0.54204E+00_JPRB, 0.54037E+00_JPRB, 0.54767E+00_JPRB, 0.55443E+00_JPRB, 0.56778E+00_JPRB ,&
&  0.57529E+00_JPRB, 0.59287E+00_JPRB, 0.61582E+00_JPRB, 0.64081E+00_JPRB, 0.67279E+00_JPRB, 0.71078E+00_JPRB, 0.75325E+00_JPRB ,&
&  0.78476E+00_JPRB, 0.81529E+00_JPRB, 0.84222E+00_JPRB, 0.86468E+00_JPRB, 0.87413E+00_JPRB, 0.87432E+00_JPRB, 0.85056E+00_JPRB ,&
&  0.83324E+00_JPRB, 0.84213E+00_JPRB, 0.84754E+00_JPRB, 0.84038E+00_JPRB, 0.81096E+00_JPRB, 0.78683E+00_JPRB, 0.76524E+00_JPRB ,&
&  0.71957E+00_JPRB, 0.64289E+00_JPRB, 0.57079E+00_JPRB, 0.54974E+00_JPRB, 0.53748E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.60885E+00_JPRB, 0.61539E+00_JPRB, 0.61546E+00_JPRB, 0.64787E+00_JPRB, 0.65387E+00_JPRB, 0.66552E+00_JPRB, 0.67639E+00_JPRB ,&
&  0.68421E+00_JPRB, 0.70155E+00_JPRB, 0.72263E+00_JPRB, 0.74629E+00_JPRB, 0.76515E+00_JPRB, 0.77960E+00_JPRB, 0.79137E+00_JPRB ,&
&  0.80596E+00_JPRB, 0.82315E+00_JPRB, 0.84415E+00_JPRB, 0.86965E+00_JPRB, 0.90879E+00_JPRB, 0.95130E+00_JPRB, 0.99539E+00_JPRB ,&
&  0.10363E+01_JPRB, 0.10959E+01_JPRB, 0.11726E+01_JPRB, 0.12668E+01_JPRB, 0.13633E+01_JPRB, 0.14299E+01_JPRB, 0.14567E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.14739E+01_JPRB, 0.14286E+01_JPRB, 0.12932E+01_JPRB, 0.11768E+01_JPRB, 0.11094E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.99887E+00_JPRB, 0.92613E+00_JPRB, 0.90269E+00_JPRB, 0.91519E+00_JPRB, 0.93122E+00_JPRB, 0.93862E+00_JPRB, 0.97095E+00_JPRB ,&
&  0.10245E+01_JPRB, 0.10690E+01_JPRB, 0.11207E+01_JPRB, 0.11697E+01_JPRB, 0.12393E+01_JPRB, 0.13462E+01_JPRB, 0.14671E+01_JPRB ,&
&  0.15926E+01_JPRB, 0.16836E+01_JPRB, 0.17628E+01_JPRB, 0.15524E+01_JPRB, 0.14386E+01_JPRB, 0.13329E+01_JPRB, 0.96690E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.69345E+00_JPRB, 0.74734E+00_JPRB, 0.79238E+00_JPRB, 0.73754E+00_JPRB, 0.67105E+00_JPRB, 0.61960E+00_JPRB ,&
&  0.57777E+00_JPRB, 0.54818E+00_JPRB, 0.55258E+00_JPRB, 0.54244E+00_JPRB, 0.54210E+00_JPRB, 0.55879E+00_JPRB, 0.57596E+00_JPRB ,&
&  0.61273E+00_JPRB, 0.62796E+00_JPRB, 0.66289E+00_JPRB, 0.69312E+00_JPRB, 0.72778E+00_JPRB, 0.76131E+00_JPRB, 0.79401E+00_JPRB ,&
&  0.81949E+00_JPRB, 0.83046E+00_JPRB, 0.83223E+00_JPRB, 0.85828E+00_JPRB, 0.86494E+00_JPRB, 0.87899E+00_JPRB, 0.90331E+00_JPRB ,&
&  0.93143E+00_JPRB, 0.94994E+00_JPRB, 0.93612E+00_JPRB, 0.91990E+00_JPRB, 0.88754E+00_JPRB, 0.85570E+00_JPRB, 0.65536E+00_JPRB ,&
&  0.62466E+00_JPRB, 0.58963E+00_JPRB, 0.60407E+00_JPRB, 0.62222E+00_JPRB, 0.61382E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.74441E+00_JPRB, 0.77429E+00_JPRB, 0.77677E+00_JPRB, 0.79782E+00_JPRB, 0.79234E+00_JPRB, 0.77792E+00_JPRB, 0.78318E+00_JPRB ,&
&  0.79564E+00_JPRB, 0.80379E+00_JPRB, 0.82376E+00_JPRB, 0.84419E+00_JPRB, 0.85948E+00_JPRB, 0.86404E+00_JPRB, 0.87692E+00_JPRB ,&
&  0.89911E+00_JPRB, 0.92877E+00_JPRB, 0.96274E+00_JPRB, 0.10075E+01_JPRB, 0.10369E+01_JPRB, 0.10534E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11771E+01_JPRB, 0.12496E+01_JPRB, 0.13216E+01_JPRB, 0.13651E+01_JPRB, 0.13793E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.15343E+01_JPRB, 0.15900E+01_JPRB, 0.15009E+01_JPRB, 0.12764E+01_JPRB, 0.11628E+01_JPRB, 0.10875E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.98376E+00_JPRB, 0.93458E+00_JPRB, 0.89499E+00_JPRB, 0.88045E+00_JPRB, 0.96016E+00_JPRB, 0.98185E+00_JPRB, 0.10357E+01_JPRB ,&
&  0.10954E+01_JPRB, 0.11538E+01_JPRB, 0.12085E+01_JPRB, 0.12617E+01_JPRB, 0.13388E+01_JPRB, 0.14272E+01_JPRB, 0.15081E+01_JPRB ,&
&  0.15742E+01_JPRB, 0.16099E+01_JPRB, 0.16924E+01_JPRB, 0.14951E+01_JPRB, 0.14577E+01_JPRB, 0.13970E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.76040E+00_JPRB, 0.71892E+00_JPRB, 0.78175E+00_JPRB, 0.80861E+00_JPRB, 0.76738E+00_JPRB, 0.68512E+00_JPRB, 0.63103E+00_JPRB ,&
&  0.57262E+00_JPRB, 0.56531E+00_JPRB, 0.56097E+00_JPRB, 0.54914E+00_JPRB, 0.54077E+00_JPRB, 0.56705E+00_JPRB, 0.59031E+00_JPRB ,&
&  0.64127E+00_JPRB, 0.67703E+00_JPRB, 0.72390E+00_JPRB, 0.76707E+00_JPRB, 0.83079E+00_JPRB, 0.91048E+00_JPRB, 0.98836E+00_JPRB ,&
&  0.86798E+00_JPRB, 0.80887E+00_JPRB, 0.84696E+00_JPRB, 0.90376E+00_JPRB, 0.94345E+00_JPRB, 0.10307E+01_JPRB, 0.11353E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.13615E+01_JPRB, 0.12476E+01_JPRB, 0.10479E+01_JPRB, 0.14270E+01_JPRB, 0.14779E+01_JPRB, 0.67893E+00_JPRB ,&
&  0.66808E+00_JPRB, 0.69857E+00_JPRB, 0.71516E+00_JPRB, 0.73492E+00_JPRB, 0.75379E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.81443E+00_JPRB, 0.82600E+00_JPRB, 0.83703E+00_JPRB, 0.84416E+00_JPRB, 0.84344E+00_JPRB, 0.85254E+00_JPRB, 0.87127E+00_JPRB ,&
&  0.88911E+00_JPRB, 0.90635E+00_JPRB, 0.92304E+00_JPRB, 0.93557E+00_JPRB, 0.94968E+00_JPRB, 0.95610E+00_JPRB, 0.97569E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.10310E+01_JPRB, 0.10411E+01_JPRB, 0.10670E+01_JPRB, 0.10780E+01_JPRB, 0.10788E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.11862E+01_JPRB, 0.12186E+01_JPRB, 0.12362E+01_JPRB, 0.12295E+01_JPRB, 0.12894E+01_JPRB, 0.11422E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.16436E+01_JPRB, 0.15160E+01_JPRB, 0.12294E+01_JPRB, 0.11201E+01_JPRB, 0.10632E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.98249E+00_JPRB, 0.94654E+00_JPRB, 0.90268E+00_JPRB, 0.94077E+00_JPRB, 0.97396E+00_JPRB, 0.10191E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.11313E+01_JPRB, 0.11847E+01_JPRB, 0.12385E+01_JPRB, 0.12893E+01_JPRB, 0.13653E+01_JPRB, 0.14303E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.15099E+01_JPRB, 0.15439E+01_JPRB, 0.15796E+01_JPRB, 0.14560E+01_JPRB, 0.14598E+01_JPRB, 0.13597E+01_JPRB, 0.11167E+01_JPRB ,&
&  0.81753E+00_JPRB, 0.72162E+00_JPRB, 0.76043E+00_JPRB, 0.81083E+00_JPRB, 0.78643E+00_JPRB, 0.68792E+00_JPRB, 0.66253E+00_JPRB ,&
&  0.58810E+00_JPRB, 0.54444E+00_JPRB, 0.54710E+00_JPRB, 0.53930E+00_JPRB, 0.53149E+00_JPRB, 0.56291E+00_JPRB, 0.58801E+00_JPRB ,&
&  0.62758E+00_JPRB, 0.68519E+00_JPRB, 0.72819E+00_JPRB, 0.75536E+00_JPRB, 0.82790E+00_JPRB, 0.89778E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.89494E+00_JPRB, 0.92202E+00_JPRB, 0.95618E+00_JPRB, 0.10706E+01_JPRB, 0.10588E+01_JPRB, 0.12610E+01_JPRB, 0.14822E+01_JPRB ,&
&  0.17837E+01_JPRB, 0.17299E+01_JPRB, 0.20398E+01_JPRB, 0.30730E+01_JPRB, 0.26360E+01_JPRB, 0.22149E+01_JPRB, 0.80933E+00_JPRB ,&
&  0.79534E+00_JPRB, 0.80631E+00_JPRB, 0.82508E+00_JPRB, 0.82362E+00_JPRB, 0.82158E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.84039E+00_JPRB, 0.85380E+00_JPRB, 0.86322E+00_JPRB, 0.87884E+00_JPRB, 0.89831E+00_JPRB, 0.92154E+00_JPRB, 0.93856E+00_JPRB ,&
&  0.95785E+00_JPRB, 0.97303E+00_JPRB, 0.98698E+00_JPRB, 0.10027E+01_JPRB, 0.10134E+01_JPRB, 0.10293E+01_JPRB, 0.10319E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.10426E+01_JPRB, 0.10484E+01_JPRB, 0.10518E+01_JPRB, 0.10538E+01_JPRB, 0.10559E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.10643E+01_JPRB, 0.10715E+01_JPRB, 0.10840E+01_JPRB, 0.10879E+01_JPRB, 0.11478E+01_JPRB, 0.10000E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.15355E+01_JPRB, 0.14515E+01_JPRB, 0.11590E+01_JPRB, 0.10436E+01_JPRB, 0.10035E+01_JPRB, 0.98069E+00_JPRB ,&
&  0.95869E+00_JPRB, 0.94781E+00_JPRB, 0.93085E+00_JPRB, 0.94028E+00_JPRB, 0.96993E+00_JPRB, 0.10149E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10839E+01_JPRB, 0.11162E+01_JPRB, 0.11583E+01_JPRB, 0.12049E+01_JPRB, 0.12798E+01_JPRB, 0.13219E+01_JPRB, 0.13322E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.13271E+01_JPRB, 0.12353E+01_JPRB, 0.12400E+01_JPRB, 0.12791E+01_JPRB, 0.10934E+01_JPRB, 0.93430E+00_JPRB ,&
&  0.79400E+00_JPRB, 0.74764E+00_JPRB, 0.77961E+00_JPRB, 0.85498E+00_JPRB, 0.79578E+00_JPRB, 0.72864E+00_JPRB, 0.69257E+00_JPRB ,&
&  0.60854E+00_JPRB, 0.56392E+00_JPRB, 0.55202E+00_JPRB, 0.52805E+00_JPRB, 0.52950E+00_JPRB, 0.54945E+00_JPRB, 0.59550E+00_JPRB ,&
&  0.61887E+00_JPRB, 0.64392E+00_JPRB, 0.67463E+00_JPRB, 0.68806E+00_JPRB, 0.72639E+00_JPRB, 0.69793E+00_JPRB, 0.82586E+00_JPRB ,&
&  0.81951E+00_JPRB, 0.81615E+00_JPRB, 0.86590E+00_JPRB, 0.74951E+00_JPRB, 0.10090E+01_JPRB, 0.10937E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.17532E+01_JPRB, 0.21266E+01_JPRB, 0.17795E+01_JPRB, 0.12872E+01_JPRB, 0.87684E+00_JPRB, 0.78218E+00_JPRB ,&
&  0.77701E+00_JPRB, 0.79477E+00_JPRB, 0.81303E+00_JPRB, 0.82100E+00_JPRB, 0.83030E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.83492E+00_JPRB, 0.84774E+00_JPRB, 0.86119E+00_JPRB, 0.88486E+00_JPRB, 0.91086E+00_JPRB, 0.93601E+00_JPRB, 0.95547E+00_JPRB ,&
&  0.97507E+00_JPRB, 0.98932E+00_JPRB, 0.10077E+01_JPRB, 0.10245E+01_JPRB, 0.10365E+01_JPRB, 0.10427E+01_JPRB, 0.10426E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10477E+01_JPRB, 0.10586E+01_JPRB, 0.10659E+01_JPRB, 0.10605E+01_JPRB, 0.10666E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.11035E+01_JPRB, 0.11322E+01_JPRB, 0.11740E+01_JPRB, 0.12302E+01_JPRB, 0.13393E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.12539E+01_JPRB, 0.13719E+01_JPRB, 0.13643E+01_JPRB, 0.10733E+01_JPRB, 0.96027E+00_JPRB, 0.92220E+00_JPRB, 0.90304E+00_JPRB ,&
&  0.89323E+00_JPRB, 0.88260E+00_JPRB, 0.88370E+00_JPRB, 0.90157E+00_JPRB, 0.93170E+00_JPRB, 0.96601E+00_JPRB, 0.98079E+00_JPRB ,&
&  0.10013E+01_JPRB, 0.10222E+01_JPRB, 0.10443E+01_JPRB, 0.10641E+01_JPRB, 0.11015E+01_JPRB, 0.11137E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.10444E+01_JPRB, 0.91896E+00_JPRB, 0.86407E+00_JPRB, 0.88430E+00_JPRB, 0.80910E+00_JPRB, 0.73477E+00_JPRB ,&
&  0.74837E+00_JPRB, 0.73108E+00_JPRB, 0.72141E+00_JPRB, 0.87627E+00_JPRB, 0.85231E+00_JPRB, 0.74021E+00_JPRB, 0.71725E+00_JPRB ,&
&  0.64097E+00_JPRB, 0.56994E+00_JPRB, 0.53717E+00_JPRB, 0.53547E+00_JPRB, 0.54251E+00_JPRB, 0.57026E+00_JPRB, 0.57152E+00_JPRB ,&
&  0.58141E+00_JPRB, 0.59815E+00_JPRB, 0.60322E+00_JPRB, 0.60195E+00_JPRB, 0.65671E+00_JPRB, 0.57728E+00_JPRB, 0.73354E+00_JPRB ,&
&  0.69718E+00_JPRB, 0.59234E+00_JPRB, 0.68401E+00_JPRB, 0.84623E+00_JPRB, 0.90977E+00_JPRB, 0.99361E+00_JPRB, 0.11504E+01_JPRB ,&
&  0.13274E+01_JPRB, 0.14594E+01_JPRB, 0.13106E+01_JPRB, 0.11047E+01_JPRB, 0.89244E+00_JPRB, 0.80158E+00_JPRB, 0.76458E+00_JPRB ,&
&  0.75866E+00_JPRB, 0.78195E+00_JPRB, 0.79985E+00_JPRB, 0.80853E+00_JPRB, 0.82118E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.79723E+00_JPRB, 0.81218E+00_JPRB, 0.82913E+00_JPRB, 0.84997E+00_JPRB, 0.86948E+00_JPRB, 0.88695E+00_JPRB, 0.90692E+00_JPRB ,&
&  0.93310E+00_JPRB, 0.95307E+00_JPRB, 0.97359E+00_JPRB, 0.98629E+00_JPRB, 0.99301E+00_JPRB, 0.10060E+01_JPRB, 0.10102E+01_JPRB ,&
&  0.10119E+01_JPRB, 0.10130E+01_JPRB, 0.10281E+01_JPRB, 0.10383E+01_JPRB, 0.10369E+01_JPRB, 0.10336E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.10752E+01_JPRB, 0.10995E+01_JPRB, 0.11299E+01_JPRB, 0.11552E+01_JPRB, 0.11637E+01_JPRB, 0.12388E+01_JPRB ,&
&  0.88889E+00_JPRB, 0.12517E+01_JPRB, 0.12487E+01_JPRB, 0.98248E+00_JPRB, 0.91378E+00_JPRB, 0.85542E+00_JPRB, 0.83104E+00_JPRB ,&
&  0.81931E+00_JPRB, 0.81498E+00_JPRB, 0.82398E+00_JPRB, 0.84650E+00_JPRB, 0.87162E+00_JPRB, 0.90065E+00_JPRB, 0.91374E+00_JPRB ,&
&  0.92349E+00_JPRB, 0.92961E+00_JPRB, 0.93040E+00_JPRB, 0.92588E+00_JPRB, 0.92725E+00_JPRB, 0.91362E+00_JPRB, 0.89403E+00_JPRB ,&
&  0.85665E+00_JPRB, 0.82652E+00_JPRB, 0.61355E+00_JPRB, 0.57489E+00_JPRB, 0.60653E+00_JPRB, 0.59798E+00_JPRB, 0.54928E+00_JPRB ,&
&  0.63830E+00_JPRB, 0.73965E+00_JPRB, 0.81193E+00_JPRB, 0.86848E+00_JPRB, 0.85847E+00_JPRB, 0.78763E+00_JPRB, 0.76510E+00_JPRB ,&
&  0.72326E+00_JPRB, 0.65344E+00_JPRB, 0.62068E+00_JPRB, 0.59701E+00_JPRB, 0.59851E+00_JPRB, 0.58817E+00_JPRB, 0.57321E+00_JPRB ,&
&  0.54763E+00_JPRB, 0.53403E+00_JPRB, 0.52081E+00_JPRB, 0.49878E+00_JPRB, 0.44654E+00_JPRB, 0.71794E+00_JPRB, 0.71378E+00_JPRB ,&
&  0.64287E+00_JPRB, 0.59680E+00_JPRB, 0.52249E+00_JPRB, 0.70381E+00_JPRB, 0.73643E+00_JPRB, 0.77742E+00_JPRB, 0.89608E+00_JPRB ,&
&  0.98555E+00_JPRB, 0.99258E+00_JPRB, 0.89611E+00_JPRB, 0.78501E+00_JPRB, 0.74234E+00_JPRB, 0.72638E+00_JPRB, 0.70383E+00_JPRB ,&
&  0.70374E+00_JPRB, 0.72682E+00_JPRB, 0.74831E+00_JPRB, 0.76414E+00_JPRB, 0.78075E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.76140E+00_JPRB, 0.75223E+00_JPRB, 0.76170E+00_JPRB, 0.77693E+00_JPRB, 0.79091E+00_JPRB, 0.80565E+00_JPRB, 0.83059E+00_JPRB ,&
&  0.86168E+00_JPRB, 0.88705E+00_JPRB, 0.91254E+00_JPRB, 0.93060E+00_JPRB, 0.93677E+00_JPRB, 0.95029E+00_JPRB, 0.96173E+00_JPRB ,&
&  0.97496E+00_JPRB, 0.97093E+00_JPRB, 0.98750E+00_JPRB, 0.10165E+01_JPRB, 0.10496E+01_JPRB, 0.10621E+01_JPRB, 0.10721E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.11236E+01_JPRB, 0.11687E+01_JPRB, 0.12223E+01_JPRB, 0.13134E+01_JPRB, 0.14746E+01_JPRB, 0.16432E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10374E+01_JPRB, 0.10444E+01_JPRB, 0.85751E+00_JPRB, 0.78395E+00_JPRB, 0.79644E+00_JPRB, 0.76255E+00_JPRB ,&
&  0.73307E+00_JPRB, 0.74371E+00_JPRB, 0.74897E+00_JPRB, 0.75482E+00_JPRB, 0.77124E+00_JPRB, 0.80464E+00_JPRB, 0.81059E+00_JPRB ,&
&  0.82401E+00_JPRB, 0.82596E+00_JPRB, 0.81206E+00_JPRB, 0.80582E+00_JPRB, 0.72906E+00_JPRB, 0.79884E+00_JPRB, 0.77414E+00_JPRB ,&
&  0.76187E+00_JPRB, 0.70460E+00_JPRB, 0.50268E+00_JPRB, 0.47100E+00_JPRB, 0.49642E+00_JPRB, 0.47304E+00_JPRB, 0.46254E+00_JPRB ,&
&  0.58589E+00_JPRB, 0.68271E+00_JPRB, 0.61483E+00_JPRB, 0.78849E+00_JPRB, 0.86120E+00_JPRB, 0.88301E+00_JPRB, 0.88034E+00_JPRB ,&
&  0.86060E+00_JPRB, 0.79954E+00_JPRB, 0.74791E+00_JPRB, 0.70117E+00_JPRB, 0.67751E+00_JPRB, 0.64646E+00_JPRB, 0.60429E+00_JPRB ,&
&  0.56598E+00_JPRB, 0.53422E+00_JPRB, 0.48771E+00_JPRB, 0.45631E+00_JPRB, 0.51893E+00_JPRB, 0.60904E+00_JPRB, 0.71595E+00_JPRB ,&
&  0.60178E+00_JPRB, 0.60956E+00_JPRB, 0.50554E+00_JPRB, 0.64530E+00_JPRB, 0.60078E+00_JPRB, 0.57490E+00_JPRB, 0.64760E+00_JPRB ,&
&  0.69355E+00_JPRB, 0.72542E+00_JPRB, 0.70862E+00_JPRB, 0.67793E+00_JPRB, 0.66544E+00_JPRB, 0.66616E+00_JPRB, 0.64642E+00_JPRB ,&
&  0.63970E+00_JPRB, 0.65787E+00_JPRB, 0.69133E+00_JPRB, 0.70244E+00_JPRB, 0.73277E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.67537E+00_JPRB, 0.67887E+00_JPRB, 0.68296E+00_JPRB, 0.69054E+00_JPRB, 0.70636E+00_JPRB, 0.72337E+00_JPRB, 0.73349E+00_JPRB ,&
&  0.75661E+00_JPRB, 0.77578E+00_JPRB, 0.79698E+00_JPRB, 0.80356E+00_JPRB, 0.81912E+00_JPRB, 0.83527E+00_JPRB, 0.84113E+00_JPRB ,&
&  0.86129E+00_JPRB, 0.88488E+00_JPRB, 0.88614E+00_JPRB, 0.89529E+00_JPRB, 0.92374E+00_JPRB, 0.94254E+00_JPRB, 0.92555E+00_JPRB ,&
&  0.90306E+00_JPRB, 0.87518E+00_JPRB, 0.89288E+00_JPRB, 0.92975E+00_JPRB, 0.11066E+01_JPRB, 0.12055E+01_JPRB, 0.11823E+01_JPRB ,&
&  0.94169E+00_JPRB, 0.90609E+00_JPRB, 0.10387E+01_JPRB, 0.85316E+00_JPRB, 0.77202E+00_JPRB, 0.71993E+00_JPRB, 0.66666E+00_JPRB ,&
&  0.67501E+00_JPRB, 0.69342E+00_JPRB, 0.70480E+00_JPRB, 0.71819E+00_JPRB, 0.74461E+00_JPRB, 0.75939E+00_JPRB, 0.79005E+00_JPRB ,&
&  0.79313E+00_JPRB, 0.80192E+00_JPRB, 0.78850E+00_JPRB, 0.73481E+00_JPRB, 0.69746E+00_JPRB, 0.76070E+00_JPRB, 0.75172E+00_JPRB ,&
&  0.75867E+00_JPRB, 0.77744E+00_JPRB, 0.67261E+00_JPRB, 0.62456E+00_JPRB, 0.59910E+00_JPRB, 0.52915E+00_JPRB, 0.49149E+00_JPRB ,&
&  0.52577E+00_JPRB, 0.54339E+00_JPRB, 0.44847E+00_JPRB, 0.64882E+00_JPRB, 0.81394E+00_JPRB, 0.89794E+00_JPRB, 0.98117E+00_JPRB ,&
&  0.99878E+00_JPRB, 0.93795E+00_JPRB, 0.84926E+00_JPRB, 0.83068E+00_JPRB, 0.81348E+00_JPRB, 0.71277E+00_JPRB, 0.66672E+00_JPRB ,&
&  0.59324E+00_JPRB, 0.54287E+00_JPRB, 0.49755E+00_JPRB, 0.47388E+00_JPRB, 0.44509E+00_JPRB, 0.52728E+00_JPRB, 0.58078E+00_JPRB ,&
&  0.54860E+00_JPRB, 0.53637E+00_JPRB, 0.57975E+00_JPRB, 0.63872E+00_JPRB, 0.47265E+00_JPRB, 0.47828E+00_JPRB, 0.51541E+00_JPRB ,&
&  0.53519E+00_JPRB, 0.53827E+00_JPRB, 0.54153E+00_JPRB, 0.54435E+00_JPRB, 0.55003E+00_JPRB, 0.56826E+00_JPRB, 0.56497E+00_JPRB ,&
&  0.56288E+00_JPRB, 0.56276E+00_JPRB, 0.58161E+00_JPRB, 0.60530E+00_JPRB, 0.64572E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.53645E+00_JPRB, 0.53708E+00_JPRB, 0.52551E+00_JPRB, 0.51765E+00_JPRB, 0.51728E+00_JPRB, 0.52195E+00_JPRB, 0.52905E+00_JPRB ,&
&  0.54173E+00_JPRB, 0.56150E+00_JPRB, 0.58347E+00_JPRB, 0.61400E+00_JPRB, 0.62297E+00_JPRB, 0.59701E+00_JPRB, 0.60579E+00_JPRB ,&
&  0.60440E+00_JPRB, 0.61874E+00_JPRB, 0.61195E+00_JPRB, 0.68562E+00_JPRB, 0.71241E+00_JPRB, 0.72319E+00_JPRB, 0.80820E+00_JPRB ,&
&  0.79459E+00_JPRB, 0.87180E+00_JPRB, 0.93280E+00_JPRB, 0.10438E+01_JPRB, 0.17335E+01_JPRB, 0.15699E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.90270E+00_JPRB, 0.99880E+00_JPRB, 0.93894E+00_JPRB, 0.83409E+00_JPRB, 0.78912E+00_JPRB, 0.71187E+00_JPRB, 0.68530E+00_JPRB ,&
&  0.71280E+00_JPRB, 0.74733E+00_JPRB, 0.81989E+00_JPRB, 0.88459E+00_JPRB, 0.93349E+00_JPRB, 0.98589E+00_JPRB, 0.10479E+01_JPRB ,&
&  0.10896E+01_JPRB, 0.11219E+01_JPRB, 0.10725E+01_JPRB, 0.95831E+00_JPRB, 0.10021E+01_JPRB, 0.10500E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.11196E+01_JPRB, 0.11511E+01_JPRB, 0.10643E+01_JPRB, 0.94358E+00_JPRB, 0.86544E+00_JPRB, 0.73312E+00_JPRB ,&
&  0.64146E+00_JPRB, 0.56859E+00_JPRB, 0.50989E+00_JPRB, 0.71354E+00_JPRB, 0.87316E+00_JPRB, 0.87081E+00_JPRB, 0.10467E+01_JPRB ,&
&  0.11071E+01_JPRB, 0.11279E+01_JPRB, 0.10496E+01_JPRB, 0.95760E+00_JPRB, 0.84986E+00_JPRB, 0.79830E+00_JPRB, 0.69788E+00_JPRB ,&
&  0.64950E+00_JPRB, 0.57718E+00_JPRB, 0.52518E+00_JPRB, 0.48780E+00_JPRB, 0.44747E+00_JPRB, 0.43578E+00_JPRB, 0.53883E+00_JPRB ,&
&  0.54790E+00_JPRB, 0.49832E+00_JPRB, 0.53851E+00_JPRB, 0.73160E+00_JPRB, 0.80149E+00_JPRB, 0.52838E+00_JPRB, 0.44787E+00_JPRB ,&
&  0.44388E+00_JPRB, 0.45143E+00_JPRB, 0.45593E+00_JPRB, 0.46150E+00_JPRB, 0.47688E+00_JPRB, 0.49006E+00_JPRB, 0.48470E+00_JPRB ,&
&  0.48923E+00_JPRB, 0.48664E+00_JPRB, 0.48517E+00_JPRB, 0.49223E+00_JPRB, 0.50851E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.46365E+00_JPRB, 0.46739E+00_JPRB, 0.47911E+00_JPRB, 0.51797E+00_JPRB, 0.54060E+00_JPRB, 0.55105E+00_JPRB, 0.57778E+00_JPRB ,&
&  0.60778E+00_JPRB, 0.66248E+00_JPRB, 0.70231E+00_JPRB, 0.71876E+00_JPRB, 0.69744E+00_JPRB, 0.67016E+00_JPRB, 0.65431E+00_JPRB ,&
&  0.64819E+00_JPRB, 0.60521E+00_JPRB, 0.57906E+00_JPRB, 0.60153E+00_JPRB, 0.66182E+00_JPRB, 0.76660E+00_JPRB, 0.88507E+00_JPRB ,&
&  0.10324E+01_JPRB, 0.11998E+01_JPRB, 0.14225E+01_JPRB, 0.19834E+01_JPRB, 0.22080E+01_JPRB, 0.97269E+00_JPRB, 0.91389E+00_JPRB ,&
&  0.10813E+01_JPRB, 0.11789E+01_JPRB, 0.10278E+01_JPRB, 0.90448E+00_JPRB, 0.83738E+00_JPRB, 0.80230E+00_JPRB, 0.80391E+00_JPRB ,&
&  0.85697E+00_JPRB, 0.90408E+00_JPRB, 0.96624E+00_JPRB, 0.10472E+01_JPRB, 0.11372E+01_JPRB, 0.12234E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.14335E+01_JPRB, 0.14189E+01_JPRB, 0.13992E+01_JPRB, 0.14342E+01_JPRB, 0.14933E+01_JPRB, 0.15417E+01_JPRB ,&
&  0.15581E+01_JPRB, 0.15506E+01_JPRB, 0.15570E+01_JPRB, 0.15153E+01_JPRB, 0.13385E+01_JPRB, 0.12018E+01_JPRB, 0.10293E+01_JPRB ,&
&  0.91938E+00_JPRB, 0.82800E+00_JPRB, 0.76694E+00_JPRB, 0.10484E+01_JPRB, 0.11775E+01_JPRB, 0.11522E+01_JPRB, 0.11730E+01_JPRB ,&
&  0.12507E+01_JPRB, 0.12333E+01_JPRB, 0.11564E+01_JPRB, 0.97466E+00_JPRB, 0.84019E+00_JPRB, 0.84948E+00_JPRB, 0.70086E+00_JPRB ,&
&  0.70522E+00_JPRB, 0.63770E+00_JPRB, 0.57935E+00_JPRB, 0.52422E+00_JPRB, 0.47023E+00_JPRB, 0.40373E+00_JPRB, 0.45861E+00_JPRB ,&
&  0.44468E+00_JPRB, 0.51781E+00_JPRB, 0.54571E+00_JPRB, 0.84903E+00_JPRB, 0.17871E+01_JPRB, 0.90665E+00_JPRB, 0.56187E+00_JPRB ,&
&  0.43467E+00_JPRB, 0.42516E+00_JPRB, 0.42897E+00_JPRB, 0.43811E+00_JPRB, 0.45130E+00_JPRB, 0.46252E+00_JPRB, 0.46599E+00_JPRB ,&
&  0.47821E+00_JPRB, 0.47904E+00_JPRB, 0.48527E+00_JPRB, 0.47716E+00_JPRB, 0.45619E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.58416E+00_JPRB, 0.61312E+00_JPRB, 0.65449E+00_JPRB, 0.70155E+00_JPRB, 0.72748E+00_JPRB, 0.74570E+00_JPRB, 0.73940E+00_JPRB ,&
&  0.75539E+00_JPRB, 0.77122E+00_JPRB, 0.80248E+00_JPRB, 0.83264E+00_JPRB, 0.84840E+00_JPRB, 0.88622E+00_JPRB, 0.91205E+00_JPRB ,&
&  0.91228E+00_JPRB, 0.89725E+00_JPRB, 0.92968E+00_JPRB, 0.98669E+00_JPRB, 0.11215E+01_JPRB, 0.13351E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.17359E+01_JPRB, 0.17813E+01_JPRB, 0.17666E+01_JPRB, 0.11711E+01_JPRB, 0.78310E+00_JPRB, 0.86373E+00_JPRB, 0.94736E+00_JPRB ,&
&  0.10081E+01_JPRB, 0.10233E+01_JPRB, 0.95991E+00_JPRB, 0.89702E+00_JPRB, 0.85441E+00_JPRB, 0.85523E+00_JPRB, 0.90736E+00_JPRB ,&
&  0.98845E+00_JPRB, 0.10771E+01_JPRB, 0.11630E+01_JPRB, 0.12200E+01_JPRB, 0.12896E+01_JPRB, 0.13953E+01_JPRB, 0.15244E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.16907E+01_JPRB, 0.16990E+01_JPRB, 0.17389E+01_JPRB, 0.17539E+01_JPRB, 0.17974E+01_JPRB, 0.18241E+01_JPRB ,&
&  0.18387E+01_JPRB, 0.18268E+01_JPRB, 0.17990E+01_JPRB, 0.17836E+01_JPRB, 0.16973E+01_JPRB, 0.14862E+01_JPRB, 0.13073E+01_JPRB ,&
&  0.12235E+01_JPRB, 0.11415E+01_JPRB, 0.11628E+01_JPRB, 0.13065E+01_JPRB, 0.98423E+00_JPRB, 0.10930E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.13484E+01_JPRB, 0.13050E+01_JPRB, 0.12009E+01_JPRB, 0.10640E+01_JPRB, 0.10012E+01_JPRB, 0.83551E+00_JPRB, 0.74112E+00_JPRB ,&
&  0.75051E+00_JPRB, 0.69284E+00_JPRB, 0.61801E+00_JPRB, 0.51584E+00_JPRB, 0.47832E+00_JPRB, 0.42538E+00_JPRB, 0.43368E+00_JPRB ,&
&  0.51589E+00_JPRB, 0.53887E+00_JPRB, 0.50886E+00_JPRB, 0.63654E+00_JPRB, 0.87844E+00_JPRB, 0.62151E+00_JPRB, 0.49878E+00_JPRB ,&
&  0.42901E+00_JPRB, 0.42038E+00_JPRB, 0.42328E+00_JPRB, 0.41756E+00_JPRB, 0.43077E+00_JPRB, 0.44843E+00_JPRB, 0.45121E+00_JPRB ,&
&  0.48775E+00_JPRB, 0.51439E+00_JPRB, 0.53973E+00_JPRB, 0.56437E+00_JPRB, 0.56506E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.70820E+00_JPRB, 0.80125E+00_JPRB, 0.87182E+00_JPRB, 0.93770E+00_JPRB, 0.97115E+00_JPRB, 0.10222E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.92424E+00_JPRB, 0.86349E+00_JPRB, 0.92046E+00_JPRB, 0.99820E+00_JPRB, 0.10688E+01_JPRB, 0.11190E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.12420E+01_JPRB, 0.12754E+01_JPRB, 0.12878E+01_JPRB, 0.13206E+01_JPRB, 0.13663E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.92721E+00_JPRB, 0.11599E+01_JPRB, 0.12411E+01_JPRB, 0.95935E+00_JPRB, 0.96623E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.10116E+01_JPRB, 0.10185E+01_JPRB, 0.10132E+01_JPRB, 0.87377E+00_JPRB, 0.91923E+00_JPRB, 0.10342E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.12551E+01_JPRB, 0.13356E+01_JPRB, 0.13888E+01_JPRB, 0.14499E+01_JPRB, 0.15504E+01_JPRB, 0.16550E+01_JPRB ,&
&  0.17549E+01_JPRB, 0.18214E+01_JPRB, 0.18530E+01_JPRB, 0.18910E+01_JPRB, 0.19483E+01_JPRB, 0.20274E+01_JPRB, 0.20602E+01_JPRB ,&
&  0.20854E+01_JPRB, 0.21096E+01_JPRB, 0.21312E+01_JPRB, 0.21504E+01_JPRB, 0.21110E+01_JPRB, 0.18208E+01_JPRB, 0.16491E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.13986E+01_JPRB, 0.14268E+01_JPRB, 0.15235E+01_JPRB, 0.11896E+01_JPRB, 0.12301E+01_JPRB, 0.13168E+01_JPRB ,&
&  0.15018E+01_JPRB, 0.14189E+01_JPRB, 0.13381E+01_JPRB, 0.13266E+01_JPRB, 0.12581E+01_JPRB, 0.11091E+01_JPRB, 0.94104E+00_JPRB ,&
&  0.75228E+00_JPRB, 0.68382E+00_JPRB, 0.59968E+00_JPRB, 0.54255E+00_JPRB, 0.43028E+00_JPRB, 0.43376E+00_JPRB, 0.49882E+00_JPRB ,&
&  0.65435E+00_JPRB, 0.53498E+00_JPRB, 0.51718E+00_JPRB, 0.57634E+00_JPRB, 0.59472E+00_JPRB, 0.48150E+00_JPRB, 0.44216E+00_JPRB ,&
&  0.42410E+00_JPRB, 0.43466E+00_JPRB, 0.43821E+00_JPRB, 0.43138E+00_JPRB, 0.43067E+00_JPRB, 0.44846E+00_JPRB, 0.46718E+00_JPRB ,&
&  0.49782E+00_JPRB, 0.53529E+00_JPRB, 0.56867E+00_JPRB, 0.61127E+00_JPRB, 0.66687E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.71220E+00_JPRB, 0.78569E+00_JPRB, 0.89967E+00_JPRB, 0.10457E+01_JPRB, 0.12372E+01_JPRB, 0.14166E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.95407E+00_JPRB, 0.10356E+01_JPRB, 0.11209E+01_JPRB, 0.11959E+01_JPRB, 0.12733E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.14118E+01_JPRB, 0.14891E+01_JPRB, 0.15601E+01_JPRB, 0.16108E+01_JPRB, 0.16184E+01_JPRB, 0.15853E+01_JPRB, 0.13870E+01_JPRB ,&
&  0.99634E+00_JPRB, 0.10676E+01_JPRB, 0.14865E+01_JPRB, 0.12253E+01_JPRB, 0.10432E+01_JPRB, 0.10650E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.10656E+01_JPRB, 0.10856E+01_JPRB, 0.10605E+01_JPRB, 0.96845E+00_JPRB, 0.10447E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.12823E+01_JPRB, 0.13766E+01_JPRB, 0.14701E+01_JPRB, 0.15437E+01_JPRB, 0.15776E+01_JPRB, 0.16462E+01_JPRB, 0.17061E+01_JPRB ,&
&  0.17544E+01_JPRB, 0.18110E+01_JPRB, 0.18480E+01_JPRB, 0.19024E+01_JPRB, 0.20443E+01_JPRB, 0.22172E+01_JPRB, 0.23554E+01_JPRB ,&
&  0.24292E+01_JPRB, 0.26235E+01_JPRB, 0.26666E+01_JPRB, 0.25368E+01_JPRB, 0.22629E+01_JPRB, 0.22834E+01_JPRB, 0.19410E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.16441E+01_JPRB, 0.17123E+01_JPRB, 0.14730E+01_JPRB, 0.14862E+01_JPRB, 0.15472E+01_JPRB, 0.16089E+01_JPRB ,&
&  0.16051E+01_JPRB, 0.16068E+01_JPRB, 0.14825E+01_JPRB, 0.14301E+01_JPRB, 0.14891E+01_JPRB, 0.13633E+01_JPRB, 0.12933E+01_JPRB ,&
&  0.85764E+00_JPRB, 0.74553E+00_JPRB, 0.58582E+00_JPRB, 0.49601E+00_JPRB, 0.40338E+00_JPRB, 0.43952E+00_JPRB, 0.59533E+00_JPRB ,&
&  0.76747E+00_JPRB, 0.72126E+00_JPRB, 0.70756E+00_JPRB, 0.69298E+00_JPRB, 0.56552E+00_JPRB, 0.48093E+00_JPRB, 0.43389E+00_JPRB ,&
&  0.41325E+00_JPRB, 0.42750E+00_JPRB, 0.45044E+00_JPRB, 0.45475E+00_JPRB, 0.44921E+00_JPRB, 0.46373E+00_JPRB, 0.48641E+00_JPRB ,&
&  0.51382E+00_JPRB, 0.54382E+00_JPRB, 0.58774E+00_JPRB, 0.61095E+00_JPRB, 0.66132E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.62326E+00_JPRB, 0.68028E+00_JPRB, 0.75347E+00_JPRB, 0.85477E+00_JPRB, 0.95537E+00_JPRB, 0.10222E+01_JPRB, 0.96603E+00_JPRB ,&
&  0.91505E+00_JPRB, 0.94868E+00_JPRB, 0.10068E+01_JPRB, 0.10801E+01_JPRB, 0.11599E+01_JPRB, 0.12467E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.14002E+01_JPRB, 0.14955E+01_JPRB, 0.16060E+01_JPRB, 0.17042E+01_JPRB, 0.17282E+01_JPRB, 0.16323E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.97575E+00_JPRB, 0.17310E+01_JPRB, 0.14502E+01_JPRB, 0.12920E+01_JPRB, 0.11200E+01_JPRB, 0.11235E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.10914E+01_JPRB, 0.11343E+01_JPRB, 0.11322E+01_JPRB, 0.11532E+01_JPRB, 0.12253E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.13504E+01_JPRB, 0.14298E+01_JPRB, 0.15233E+01_JPRB, 0.16104E+01_JPRB, 0.16360E+01_JPRB, 0.16602E+01_JPRB, 0.16791E+01_JPRB ,&
&  0.16958E+01_JPRB, 0.17147E+01_JPRB, 0.17410E+01_JPRB, 0.19032E+01_JPRB, 0.20806E+01_JPRB, 0.23158E+01_JPRB, 0.25993E+01_JPRB ,&
&  0.27358E+01_JPRB, 0.28429E+01_JPRB, 0.31602E+01_JPRB, 0.31201E+01_JPRB, 0.29388E+01_JPRB, 0.28343E+01_JPRB, 0.22110E+01_JPRB ,&
&  0.21558E+01_JPRB, 0.18489E+01_JPRB, 0.19071E+01_JPRB, 0.16296E+01_JPRB, 0.16387E+01_JPRB, 0.17781E+01_JPRB, 0.18392E+01_JPRB ,&
&  0.17403E+01_JPRB, 0.18110E+01_JPRB, 0.16848E+01_JPRB, 0.15581E+01_JPRB, 0.15135E+01_JPRB, 0.15059E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.79355E+00_JPRB, 0.63051E+00_JPRB, 0.48887E+00_JPRB, 0.42065E+00_JPRB, 0.41038E+00_JPRB, 0.57829E+00_JPRB ,&
&  0.82419E+00_JPRB, 0.99511E+00_JPRB, 0.11589E+01_JPRB, 0.99465E+00_JPRB, 0.78470E+00_JPRB, 0.66666E+00_JPRB, 0.55358E+00_JPRB ,&
&  0.44644E+00_JPRB, 0.44359E+00_JPRB, 0.48056E+00_JPRB, 0.49270E+00_JPRB, 0.48583E+00_JPRB, 0.50036E+00_JPRB, 0.50869E+00_JPRB ,&
&  0.50646E+00_JPRB, 0.51383E+00_JPRB, 0.54384E+00_JPRB, 0.55001E+00_JPRB, 0.57936E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.55019E+00_JPRB, 0.58251E+00_JPRB, 0.65083E+00_JPRB, 0.71287E+00_JPRB, 0.74789E+00_JPRB, 0.76384E+00_JPRB, 0.77254E+00_JPRB ,&
&  0.81652E+00_JPRB, 0.87492E+00_JPRB, 0.91757E+00_JPRB, 0.95289E+00_JPRB, 0.10094E+01_JPRB, 0.11026E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.15083E+01_JPRB, 0.17055E+01_JPRB, 0.19030E+01_JPRB, 0.20500E+01_JPRB, 0.19846E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.17148E+01_JPRB, 0.16832E+01_JPRB, 0.15912E+01_JPRB, 0.13591E+01_JPRB, 0.12543E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.11897E+01_JPRB, 0.12264E+01_JPRB, 0.12749E+01_JPRB, 0.13064E+01_JPRB, 0.13314E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.13118E+01_JPRB, 0.13532E+01_JPRB, 0.14548E+01_JPRB, 0.15600E+01_JPRB, 0.16327E+01_JPRB, 0.16644E+01_JPRB ,&
&  0.16670E+01_JPRB, 0.16576E+01_JPRB, 0.16832E+01_JPRB, 0.17475E+01_JPRB, 0.20797E+01_JPRB, 0.23272E+01_JPRB, 0.26605E+01_JPRB ,&
&  0.29086E+01_JPRB, 0.30591E+01_JPRB, 0.36685E+01_JPRB, 0.40721E+01_JPRB, 0.41045E+01_JPRB, 0.34996E+01_JPRB, 0.27480E+01_JPRB ,&
&  0.23979E+01_JPRB, 0.21055E+01_JPRB, 0.19356E+01_JPRB, 0.17885E+01_JPRB, 0.17962E+01_JPRB, 0.19457E+01_JPRB, 0.21777E+01_JPRB ,&
&  0.20472E+01_JPRB, 0.19637E+01_JPRB, 0.18861E+01_JPRB, 0.16525E+01_JPRB, 0.15612E+01_JPRB, 0.15236E+01_JPRB, 0.17480E+01_JPRB ,&
&  0.17659E+01_JPRB, 0.13007E+01_JPRB, 0.82828E+00_JPRB, 0.53691E+00_JPRB, 0.43667E+00_JPRB, 0.37714E+00_JPRB, 0.45941E+00_JPRB ,&
&  0.66473E+00_JPRB, 0.89497E+00_JPRB, 0.11765E+01_JPRB, 0.13909E+01_JPRB, 0.11301E+01_JPRB, 0.10172E+01_JPRB, 0.89522E+00_JPRB ,&
&  0.60545E+00_JPRB, 0.52608E+00_JPRB, 0.53009E+00_JPRB, 0.53401E+00_JPRB, 0.51969E+00_JPRB, 0.54192E+00_JPRB, 0.56671E+00_JPRB ,&
&  0.54355E+00_JPRB, 0.48901E+00_JPRB, 0.49041E+00_JPRB, 0.49716E+00_JPRB, 0.51727E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.50164E+00_JPRB, 0.50383E+00_JPRB, 0.54624E+00_JPRB, 0.58112E+00_JPRB, 0.61477E+00_JPRB, 0.64278E+00_JPRB, 0.68516E+00_JPRB ,&
&  0.74702E+00_JPRB, 0.79350E+00_JPRB, 0.82503E+00_JPRB, 0.82723E+00_JPRB, 0.82778E+00_JPRB, 0.87228E+00_JPRB, 0.95716E+00_JPRB ,&
&  0.10962E+01_JPRB, 0.13156E+01_JPRB, 0.16246E+01_JPRB, 0.18833E+01_JPRB, 0.19948E+01_JPRB, 0.17938E+01_JPRB, 0.16989E+01_JPRB ,&
&  0.17450E+01_JPRB, 0.20656E+01_JPRB, 0.21612E+01_JPRB, 0.22500E+01_JPRB, 0.21511E+01_JPRB, 0.20200E+01_JPRB, 0.13980E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.14347E+01_JPRB, 0.14581E+01_JPRB, 0.15160E+01_JPRB, 0.15356E+01_JPRB, 0.15146E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.13325E+01_JPRB, 0.12469E+01_JPRB, 0.12055E+01_JPRB, 0.12468E+01_JPRB, 0.13790E+01_JPRB, 0.15066E+01_JPRB, 0.16347E+01_JPRB ,&
&  0.17207E+01_JPRB, 0.17302E+01_JPRB, 0.17218E+01_JPRB, 0.16912E+01_JPRB, 0.19693E+01_JPRB, 0.24524E+01_JPRB, 0.28651E+01_JPRB ,&
&  0.32165E+01_JPRB, 0.37636E+01_JPRB, 0.42252E+01_JPRB, 0.45403E+01_JPRB, 0.47335E+01_JPRB, 0.43426E+01_JPRB, 0.35261E+01_JPRB ,&
&  0.28185E+01_JPRB, 0.22702E+01_JPRB, 0.18987E+01_JPRB, 0.19129E+01_JPRB, 0.20662E+01_JPRB, 0.21810E+01_JPRB, 0.20589E+01_JPRB ,&
&  0.18802E+01_JPRB, 0.17254E+01_JPRB, 0.16927E+01_JPRB, 0.16933E+01_JPRB, 0.17027E+01_JPRB, 0.16476E+01_JPRB, 0.19990E+01_JPRB ,&
&  0.18083E+01_JPRB, 0.92326E+00_JPRB, 0.61501E+00_JPRB, 0.65547E+00_JPRB, 0.43997E+00_JPRB, 0.34837E+00_JPRB, 0.48026E+00_JPRB ,&
&  0.61531E+00_JPRB, 0.96065E+00_JPRB, 0.12823E+01_JPRB, 0.12398E+01_JPRB, 0.15170E+01_JPRB, 0.15238E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.92957E+00_JPRB, 0.72465E+00_JPRB, 0.58902E+00_JPRB, 0.55586E+00_JPRB, 0.53883E+00_JPRB, 0.55264E+00_JPRB, 0.55573E+00_JPRB ,&
&  0.52937E+00_JPRB, 0.48260E+00_JPRB, 0.47750E+00_JPRB, 0.48232E+00_JPRB, 0.48770E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.53282E+00_JPRB, 0.49904E+00_JPRB, 0.49623E+00_JPRB, 0.49890E+00_JPRB, 0.55273E+00_JPRB, 0.61190E+00_JPRB, 0.63889E+00_JPRB ,&
&  0.70068E+00_JPRB, 0.75303E+00_JPRB, 0.76777E+00_JPRB, 0.75182E+00_JPRB, 0.73106E+00_JPRB, 0.73544E+00_JPRB, 0.76890E+00_JPRB ,&
&  0.84947E+00_JPRB, 0.10218E+01_JPRB, 0.13684E+01_JPRB, 0.17259E+01_JPRB, 0.19201E+01_JPRB, 0.18791E+01_JPRB, 0.20292E+01_JPRB ,&
&  0.20574E+01_JPRB, 0.24943E+01_JPRB, 0.32001E+01_JPRB, 0.34438E+01_JPRB, 0.27232E+01_JPRB, 0.23913E+01_JPRB, 0.18132E+01_JPRB ,&
&  0.17096E+01_JPRB, 0.17328E+01_JPRB, 0.17941E+01_JPRB, 0.18878E+01_JPRB, 0.19033E+01_JPRB, 0.18297E+01_JPRB, 0.17564E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.14210E+01_JPRB, 0.12516E+01_JPRB, 0.11916E+01_JPRB, 0.12744E+01_JPRB, 0.14363E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.18601E+01_JPRB, 0.18458E+01_JPRB, 0.19434E+01_JPRB, 0.16472E+01_JPRB, 0.24794E+01_JPRB, 0.32017E+01_JPRB ,&
&  0.37035E+01_JPRB, 0.42095E+01_JPRB, 0.42772E+01_JPRB, 0.50635E+01_JPRB, 0.54631E+01_JPRB, 0.48541E+01_JPRB, 0.40206E+01_JPRB ,&
&  0.33292E+01_JPRB, 0.25246E+01_JPRB, 0.21557E+01_JPRB, 0.21791E+01_JPRB, 0.22391E+01_JPRB, 0.22941E+01_JPRB, 0.16552E+01_JPRB ,&
&  0.16201E+01_JPRB, 0.20156E+01_JPRB, 0.19465E+01_JPRB, 0.16588E+01_JPRB, 0.17500E+01_JPRB, 0.17074E+01_JPRB, 0.15406E+01_JPRB ,&
&  0.76306E+00_JPRB, 0.52091E+00_JPRB, 0.48296E+00_JPRB, 0.44136E+00_JPRB, 0.38792E+00_JPRB, 0.37614E+00_JPRB, 0.70550E+00_JPRB ,&
&  0.66853E+00_JPRB, 0.11268E+01_JPRB, 0.14418E+01_JPRB, 0.14596E+01_JPRB, 0.17300E+01_JPRB, 0.20281E+01_JPRB, 0.25420E+01_JPRB ,&
&  0.13621E+01_JPRB, 0.97687E+00_JPRB, 0.69700E+00_JPRB, 0.59004E+00_JPRB, 0.57316E+00_JPRB, 0.56275E+00_JPRB, 0.54028E+00_JPRB ,&
&  0.52138E+00_JPRB, 0.51038E+00_JPRB, 0.50784E+00_JPRB, 0.49719E+00_JPRB, 0.52489E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.61205E+00_JPRB, 0.56268E+00_JPRB, 0.52918E+00_JPRB, 0.53404E+00_JPRB, 0.57458E+00_JPRB, 0.59821E+00_JPRB, 0.61626E+00_JPRB ,&
&  0.66549E+00_JPRB, 0.72285E+00_JPRB, 0.74426E+00_JPRB, 0.71007E+00_JPRB, 0.68414E+00_JPRB, 0.68382E+00_JPRB, 0.68933E+00_JPRB ,&
&  0.73249E+00_JPRB, 0.84591E+00_JPRB, 0.11394E+01_JPRB, 0.13642E+01_JPRB, 0.16538E+01_JPRB, 0.17038E+01_JPRB, 0.20554E+01_JPRB ,&
&  0.22209E+01_JPRB, 0.26193E+01_JPRB, 0.32337E+01_JPRB, 0.33605E+01_JPRB, 0.35257E+01_JPRB, 0.28748E+01_JPRB, 0.25350E+01_JPRB ,&
&  0.20916E+01_JPRB, 0.19321E+01_JPRB, 0.19025E+01_JPRB, 0.20589E+01_JPRB, 0.21816E+01_JPRB, 0.21634E+01_JPRB, 0.20344E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.15612E+01_JPRB, 0.13140E+01_JPRB, 0.12274E+01_JPRB, 0.11873E+01_JPRB, 0.12787E+01_JPRB, 0.14830E+01_JPRB ,&
&  0.18140E+01_JPRB, 0.20662E+01_JPRB, 0.20326E+01_JPRB, 0.20325E+01_JPRB, 0.19297E+01_JPRB, 0.21416E+01_JPRB, 0.33040E+01_JPRB ,&
&  0.43126E+01_JPRB, 0.50738E+01_JPRB, 0.52522E+01_JPRB, 0.62305E+01_JPRB, 0.60282E+01_JPRB, 0.51353E+01_JPRB, 0.46118E+01_JPRB ,&
&  0.38051E+01_JPRB, 0.32218E+01_JPRB, 0.24820E+01_JPRB, 0.23591E+01_JPRB, 0.23443E+01_JPRB, 0.18170E+01_JPRB, 0.16600E+01_JPRB ,&
&  0.20124E+01_JPRB, 0.20463E+01_JPRB, 0.19172E+01_JPRB, 0.16146E+01_JPRB, 0.18278E+01_JPRB, 0.15099E+01_JPRB, 0.85037E+00_JPRB ,&
&  0.69590E+00_JPRB, 0.61754E+00_JPRB, 0.49642E+00_JPRB, 0.43755E+00_JPRB, 0.39571E+00_JPRB, 0.40370E+00_JPRB, 0.61332E+00_JPRB ,&
&  0.79189E+00_JPRB, 0.13377E+01_JPRB, 0.16213E+01_JPRB, 0.16697E+01_JPRB, 0.17989E+01_JPRB, 0.23482E+01_JPRB, 0.26522E+01_JPRB ,&
&  0.19381E+01_JPRB, 0.15046E+01_JPRB, 0.82500E+00_JPRB, 0.64024E+00_JPRB, 0.59792E+00_JPRB, 0.57877E+00_JPRB, 0.55581E+00_JPRB ,&
&  0.53376E+00_JPRB, 0.53804E+00_JPRB, 0.54662E+00_JPRB, 0.55496E+00_JPRB, 0.57065E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.64421E+00_JPRB, 0.61881E+00_JPRB, 0.58011E+00_JPRB, 0.59365E+00_JPRB, 0.61781E+00_JPRB, 0.64781E+00_JPRB, 0.64247E+00_JPRB ,&
&  0.65335E+00_JPRB, 0.68036E+00_JPRB, 0.70284E+00_JPRB, 0.68873E+00_JPRB, 0.64967E+00_JPRB, 0.65831E+00_JPRB, 0.66387E+00_JPRB ,&
&  0.71072E+00_JPRB, 0.83883E+00_JPRB, 0.99860E+00_JPRB, 0.10768E+01_JPRB, 0.11896E+01_JPRB, 0.12997E+01_JPRB, 0.16754E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.25858E+01_JPRB, 0.30050E+01_JPRB, 0.34887E+01_JPRB, 0.39480E+01_JPRB, 0.34718E+01_JPRB, 0.34053E+01_JPRB ,&
&  0.28329E+01_JPRB, 0.23628E+01_JPRB, 0.21964E+01_JPRB, 0.21872E+01_JPRB, 0.23222E+01_JPRB, 0.23686E+01_JPRB, 0.22554E+01_JPRB ,&
&  0.19471E+01_JPRB, 0.16080E+01_JPRB, 0.13751E+01_JPRB, 0.12222E+01_JPRB, 0.11507E+01_JPRB, 0.11942E+01_JPRB, 0.14040E+01_JPRB ,&
&  0.18993E+01_JPRB, 0.23976E+01_JPRB, 0.24263E+01_JPRB, 0.22247E+01_JPRB, 0.25757E+01_JPRB, 0.28954E+01_JPRB, 0.36135E+01_JPRB ,&
&  0.41013E+01_JPRB, 0.44335E+01_JPRB, 0.62222E+01_JPRB, 0.65983E+01_JPRB, 0.60758E+01_JPRB, 0.54091E+01_JPRB, 0.46349E+01_JPRB ,&
&  0.39926E+01_JPRB, 0.34890E+01_JPRB, 0.27588E+01_JPRB, 0.25913E+01_JPRB, 0.21803E+01_JPRB, 0.17489E+01_JPRB, 0.20075E+01_JPRB ,&
&  0.22801E+01_JPRB, 0.22744E+01_JPRB, 0.19977E+01_JPRB, 0.16343E+01_JPRB, 0.15456E+01_JPRB, 0.11072E+01_JPRB, 0.76626E+00_JPRB ,&
&  0.66962E+00_JPRB, 0.63019E+00_JPRB, 0.53427E+00_JPRB, 0.55111E+00_JPRB, 0.49048E+00_JPRB, 0.51218E+00_JPRB, 0.67179E+00_JPRB ,&
&  0.98235E+00_JPRB, 0.14068E+01_JPRB, 0.15761E+01_JPRB, 0.15952E+01_JPRB, 0.17249E+01_JPRB, 0.19371E+01_JPRB, 0.26448E+01_JPRB ,&
&  0.18154E+01_JPRB, 0.17789E+01_JPRB, 0.11834E+01_JPRB, 0.80681E+00_JPRB, 0.66755E+00_JPRB, 0.61287E+00_JPRB, 0.60135E+00_JPRB ,&
&  0.59898E+00_JPRB, 0.59653E+00_JPRB, 0.60104E+00_JPRB, 0.60489E+00_JPRB, 0.61271E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.64974E+00_JPRB, 0.66198E+00_JPRB, 0.65064E+00_JPRB, 0.65944E+00_JPRB, 0.67570E+00_JPRB, 0.68790E+00_JPRB, 0.67738E+00_JPRB ,&
&  0.66196E+00_JPRB, 0.67166E+00_JPRB, 0.66803E+00_JPRB, 0.64967E+00_JPRB, 0.62771E+00_JPRB, 0.63466E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.72035E+00_JPRB, 0.91505E+00_JPRB, 0.76192E+00_JPRB, 0.77219E+00_JPRB, 0.89291E+00_JPRB, 0.10525E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.19459E+01_JPRB, 0.25359E+01_JPRB, 0.29553E+01_JPRB, 0.37414E+01_JPRB, 0.48865E+01_JPRB, 0.46642E+01_JPRB, 0.43829E+01_JPRB ,&
&  0.37405E+01_JPRB, 0.30861E+01_JPRB, 0.28225E+01_JPRB, 0.26476E+01_JPRB, 0.26052E+01_JPRB, 0.25691E+01_JPRB, 0.24040E+01_JPRB ,&
&  0.20900E+01_JPRB, 0.16719E+01_JPRB, 0.13670E+01_JPRB, 0.12877E+01_JPRB, 0.12219E+01_JPRB, 0.12633E+01_JPRB, 0.14158E+01_JPRB ,&
&  0.19692E+01_JPRB, 0.26446E+01_JPRB, 0.28914E+01_JPRB, 0.28234E+01_JPRB, 0.25928E+01_JPRB, 0.32775E+01_JPRB, 0.54048E+01_JPRB ,&
&  0.55473E+01_JPRB, 0.59229E+01_JPRB, 0.64198E+01_JPRB, 0.74761E+01_JPRB, 0.62199E+01_JPRB, 0.48478E+01_JPRB, 0.45430E+01_JPRB ,&
&  0.33103E+01_JPRB, 0.26434E+01_JPRB, 0.26071E+01_JPRB, 0.24715E+01_JPRB, 0.21268E+01_JPRB, 0.22726E+01_JPRB, 0.32234E+01_JPRB ,&
&  0.25985E+01_JPRB, 0.27787E+01_JPRB, 0.28515E+01_JPRB, 0.26894E+01_JPRB, 0.15982E+01_JPRB, 0.12578E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.18626E+01_JPRB, 0.11662E+01_JPRB, 0.74880E+00_JPRB, 0.97252E+00_JPRB, 0.70575E+00_JPRB, 0.74500E+00_JPRB, 0.93063E+00_JPRB ,&
&  0.10551E+01_JPRB, 0.12341E+01_JPRB, 0.17390E+01_JPRB, 0.17276E+01_JPRB, 0.17239E+01_JPRB, 0.18291E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.19819E+01_JPRB, 0.17999E+01_JPRB, 0.14917E+01_JPRB, 0.10721E+01_JPRB, 0.83865E+00_JPRB, 0.74342E+00_JPRB, 0.70646E+00_JPRB ,&
&  0.69493E+00_JPRB, 0.70953E+00_JPRB, 0.70123E+00_JPRB, 0.67312E+00_JPRB, 0.65835E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.72415E+00_JPRB, 0.73111E+00_JPRB, 0.72596E+00_JPRB, 0.73590E+00_JPRB, 0.75170E+00_JPRB, 0.71970E+00_JPRB, 0.70113E+00_JPRB ,&
&  0.67388E+00_JPRB, 0.65751E+00_JPRB, 0.65693E+00_JPRB, 0.65558E+00_JPRB, 0.63780E+00_JPRB, 0.63576E+00_JPRB, 0.64938E+00_JPRB ,&
&  0.71709E+00_JPRB, 0.77532E+00_JPRB, 0.70221E+00_JPRB, 0.69240E+00_JPRB, 0.81957E+00_JPRB, 0.92630E+00_JPRB, 0.11704E+01_JPRB ,&
&  0.17511E+01_JPRB, 0.24133E+01_JPRB, 0.31259E+01_JPRB, 0.39727E+01_JPRB, 0.46383E+01_JPRB, 0.52041E+01_JPRB, 0.54088E+01_JPRB ,&
&  0.51918E+01_JPRB, 0.43073E+01_JPRB, 0.37476E+01_JPRB, 0.33472E+01_JPRB, 0.29979E+01_JPRB, 0.27144E+01_JPRB, 0.24847E+01_JPRB ,&
&  0.22918E+01_JPRB, 0.19558E+01_JPRB, 0.15874E+01_JPRB, 0.13542E+01_JPRB, 0.13069E+01_JPRB, 0.13111E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.19294E+01_JPRB, 0.26018E+01_JPRB, 0.33049E+01_JPRB, 0.35742E+01_JPRB, 0.30919E+01_JPRB, 0.35410E+01_JPRB, 0.50466E+01_JPRB ,&
&  0.58861E+01_JPRB, 0.59944E+01_JPRB, 0.69321E+01_JPRB, 0.73754E+01_JPRB, 0.64967E+01_JPRB, 0.51124E+01_JPRB, 0.50836E+01_JPRB ,&
&  0.38890E+01_JPRB, 0.30065E+01_JPRB, 0.25416E+01_JPRB, 0.21868E+01_JPRB, 0.24421E+01_JPRB, 0.34005E+01_JPRB, 0.32671E+01_JPRB ,&
&  0.31548E+01_JPRB, 0.30794E+01_JPRB, 0.28586E+01_JPRB, 0.24170E+01_JPRB, 0.21169E+01_JPRB, 0.19101E+01_JPRB, 0.23550E+01_JPRB ,&
&  0.23282E+01_JPRB, 0.22143E+01_JPRB, 0.18318E+01_JPRB, 0.14945E+01_JPRB, 0.11804E+01_JPRB, 0.12518E+01_JPRB, 0.12282E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.11698E+01_JPRB, 0.15705E+01_JPRB, 0.16707E+01_JPRB, 0.15870E+01_JPRB, 0.14953E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.15645E+01_JPRB, 0.17272E+01_JPRB, 0.14911E+01_JPRB, 0.11778E+01_JPRB, 0.10015E+01_JPRB, 0.92641E+00_JPRB, 0.87075E+00_JPRB ,&
&  0.86395E+00_JPRB, 0.86738E+00_JPRB, 0.84584E+00_JPRB, 0.79383E+00_JPRB, 0.75862E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.81214E+00_JPRB, 0.80991E+00_JPRB, 0.80351E+00_JPRB, 0.80262E+00_JPRB, 0.80453E+00_JPRB, 0.76100E+00_JPRB, 0.72393E+00_JPRB ,&
&  0.69448E+00_JPRB, 0.67331E+00_JPRB, 0.67335E+00_JPRB, 0.66497E+00_JPRB, 0.65046E+00_JPRB, 0.64258E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.72028E+00_JPRB, 0.77743E+00_JPRB, 0.69151E+00_JPRB, 0.74209E+00_JPRB, 0.80181E+00_JPRB, 0.90506E+00_JPRB, 0.12710E+01_JPRB ,&
&  0.16639E+01_JPRB, 0.21606E+01_JPRB, 0.28252E+01_JPRB, 0.36002E+01_JPRB, 0.42200E+01_JPRB, 0.49304E+01_JPRB, 0.56631E+01_JPRB ,&
&  0.52978E+01_JPRB, 0.51983E+01_JPRB, 0.45085E+01_JPRB, 0.38146E+01_JPRB, 0.32348E+01_JPRB, 0.27954E+01_JPRB, 0.25675E+01_JPRB ,&
&  0.23871E+01_JPRB, 0.22610E+01_JPRB, 0.19988E+01_JPRB, 0.16972E+01_JPRB, 0.15284E+01_JPRB, 0.14635E+01_JPRB, 0.14883E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.23498E+01_JPRB, 0.34265E+01_JPRB, 0.44090E+01_JPRB, 0.36391E+01_JPRB, 0.41847E+01_JPRB, 0.48513E+01_JPRB ,&
&  0.70714E+01_JPRB, 0.75615E+01_JPRB, 0.72090E+01_JPRB, 0.72891E+01_JPRB, 0.53946E+01_JPRB, 0.49486E+01_JPRB, 0.49894E+01_JPRB ,&
&  0.47649E+01_JPRB, 0.43293E+01_JPRB, 0.43928E+01_JPRB, 0.33086E+01_JPRB, 0.28242E+01_JPRB, 0.34592E+01_JPRB, 0.30437E+01_JPRB ,&
&  0.28000E+01_JPRB, 0.26951E+01_JPRB, 0.25061E+01_JPRB, 0.25497E+01_JPRB, 0.22866E+01_JPRB, 0.22143E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.15189E+01_JPRB, 0.16406E+01_JPRB, 0.14579E+01_JPRB, 0.13492E+01_JPRB, 0.13038E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.12792E+01_JPRB, 0.14527E+01_JPRB, 0.15775E+01_JPRB, 0.14818E+01_JPRB, 0.12950E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.13465E+01_JPRB, 0.14678E+01_JPRB, 0.13375E+01_JPRB, 0.11701E+01_JPRB, 0.10634E+01_JPRB, 0.10200E+01_JPRB, 0.99035E+00_JPRB ,&
&  0.94920E+00_JPRB, 0.92289E+00_JPRB, 0.90064E+00_JPRB, 0.87713E+00_JPRB, 0.84996E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.82044E+00_JPRB, 0.81048E+00_JPRB, 0.85293E+00_JPRB, 0.83137E+00_JPRB, 0.83599E+00_JPRB, 0.79672E+00_JPRB, 0.76689E+00_JPRB ,&
&  0.74004E+00_JPRB, 0.72134E+00_JPRB, 0.70088E+00_JPRB, 0.67902E+00_JPRB, 0.65829E+00_JPRB, 0.64295E+00_JPRB, 0.64776E+00_JPRB ,&
&  0.67776E+00_JPRB, 0.83838E+00_JPRB, 0.76173E+00_JPRB, 0.72412E+00_JPRB, 0.75548E+00_JPRB, 0.96003E+00_JPRB, 0.12698E+01_JPRB ,&
&  0.15749E+01_JPRB, 0.18686E+01_JPRB, 0.23864E+01_JPRB, 0.28990E+01_JPRB, 0.34582E+01_JPRB, 0.40540E+01_JPRB, 0.47781E+01_JPRB ,&
&  0.53044E+01_JPRB, 0.44872E+01_JPRB, 0.42005E+01_JPRB, 0.36674E+01_JPRB, 0.32285E+01_JPRB, 0.28587E+01_JPRB, 0.26004E+01_JPRB ,&
&  0.23948E+01_JPRB, 0.22486E+01_JPRB, 0.20637E+01_JPRB, 0.18464E+01_JPRB, 0.16282E+01_JPRB, 0.15001E+01_JPRB, 0.14651E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.18953E+01_JPRB, 0.28305E+01_JPRB, 0.42514E+01_JPRB, 0.60514E+01_JPRB, 0.64534E+01_JPRB, 0.62257E+01_JPRB ,&
&  0.55285E+01_JPRB, 0.57024E+01_JPRB, 0.79818E+01_JPRB, 0.65078E+01_JPRB, 0.60713E+01_JPRB, 0.58256E+01_JPRB, 0.53131E+01_JPRB ,&
&  0.49392E+01_JPRB, 0.41472E+01_JPRB, 0.38735E+01_JPRB, 0.32248E+01_JPRB, 0.34642E+01_JPRB, 0.30452E+01_JPRB, 0.27017E+01_JPRB ,&
&  0.24484E+01_JPRB, 0.24347E+01_JPRB, 0.23334E+01_JPRB, 0.22708E+01_JPRB, 0.23041E+01_JPRB, 0.25250E+01_JPRB, 0.24694E+01_JPRB ,&
&  0.21794E+01_JPRB, 0.21961E+01_JPRB, 0.18250E+01_JPRB, 0.14735E+01_JPRB, 0.12139E+01_JPRB, 0.12006E+01_JPRB, 0.12140E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.13042E+01_JPRB, 0.14425E+01_JPRB, 0.14644E+01_JPRB, 0.13576E+01_JPRB, 0.12444E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.11569E+01_JPRB, 0.12526E+01_JPRB, 0.12836E+01_JPRB, 0.11217E+01_JPRB, 0.10834E+01_JPRB, 0.10486E+01_JPRB, 0.10158E+01_JPRB ,&
&  0.96160E+00_JPRB, 0.90797E+00_JPRB, 0.88099E+00_JPRB, 0.86742E+00_JPRB, 0.85461E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.83635E+00_JPRB, 0.80610E+00_JPRB, 0.84447E+00_JPRB, 0.82773E+00_JPRB, 0.81308E+00_JPRB, 0.79020E+00_JPRB, 0.76756E+00_JPRB ,&
&  0.74401E+00_JPRB, 0.73156E+00_JPRB, 0.71762E+00_JPRB, 0.68858E+00_JPRB, 0.66404E+00_JPRB, 0.64853E+00_JPRB, 0.64729E+00_JPRB ,&
&  0.65451E+00_JPRB, 0.84574E+00_JPRB, 0.81914E+00_JPRB, 0.86246E+00_JPRB, 0.87037E+00_JPRB, 0.10301E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.15599E+01_JPRB, 0.17676E+01_JPRB, 0.20679E+01_JPRB, 0.23944E+01_JPRB, 0.27551E+01_JPRB, 0.33239E+01_JPRB ,&
&  0.35134E+01_JPRB, 0.34873E+01_JPRB, 0.32950E+01_JPRB, 0.33531E+01_JPRB, 0.31999E+01_JPRB, 0.27479E+01_JPRB, 0.25720E+01_JPRB ,&
&  0.23003E+01_JPRB, 0.21102E+01_JPRB, 0.19612E+01_JPRB, 0.18658E+01_JPRB, 0.17735E+01_JPRB, 0.16554E+01_JPRB, 0.15745E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.15893E+01_JPRB, 0.20382E+01_JPRB, 0.31651E+01_JPRB, 0.50507E+01_JPRB, 0.64210E+01_JPRB, 0.65842E+01_JPRB ,&
&  0.70122E+01_JPRB, 0.69816E+01_JPRB, 0.54678E+01_JPRB, 0.59567E+01_JPRB, 0.70203E+01_JPRB, 0.62052E+01_JPRB, 0.53086E+01_JPRB ,&
&  0.52140E+01_JPRB, 0.48196E+01_JPRB, 0.45559E+01_JPRB, 0.39229E+01_JPRB, 0.32910E+01_JPRB, 0.29618E+01_JPRB, 0.27178E+01_JPRB ,&
&  0.25068E+01_JPRB, 0.24544E+01_JPRB, 0.24336E+01_JPRB, 0.23673E+01_JPRB, 0.22391E+01_JPRB, 0.22084E+01_JPRB, 0.23238E+01_JPRB ,&
&  0.22395E+01_JPRB, 0.20271E+01_JPRB, 0.15311E+01_JPRB, 0.14730E+01_JPRB, 0.11023E+01_JPRB, 0.10615E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.12558E+01_JPRB, 0.13447E+01_JPRB, 0.11970E+01_JPRB, 0.11391E+01_JPRB, 0.10936E+01_JPRB, 0.11114E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.10336E+01_JPRB, 0.11455E+01_JPRB, 0.10617E+01_JPRB, 0.10400E+01_JPRB, 0.10147E+01_JPRB, 0.98102E+00_JPRB ,&
&  0.96559E+00_JPRB, 0.93424E+00_JPRB, 0.90543E+00_JPRB, 0.89565E+00_JPRB, 0.87503E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.84647E+00_JPRB, 0.81547E+00_JPRB, 0.82858E+00_JPRB, 0.79152E+00_JPRB, 0.76011E+00_JPRB, 0.73497E+00_JPRB, 0.71745E+00_JPRB ,&
&  0.70665E+00_JPRB, 0.69589E+00_JPRB, 0.68450E+00_JPRB, 0.66632E+00_JPRB, 0.65059E+00_JPRB, 0.63483E+00_JPRB, 0.62976E+00_JPRB ,&
&  0.64573E+00_JPRB, 0.72205E+00_JPRB, 0.76865E+00_JPRB, 0.76745E+00_JPRB, 0.93175E+00_JPRB, 0.96071E+00_JPRB, 0.10328E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.12404E+01_JPRB, 0.12844E+01_JPRB, 0.13406E+01_JPRB, 0.14621E+01_JPRB, 0.17260E+01_JPRB, 0.20748E+01_JPRB ,&
&  0.22091E+01_JPRB, 0.21417E+01_JPRB, 0.21494E+01_JPRB, 0.22596E+01_JPRB, 0.24284E+01_JPRB, 0.21445E+01_JPRB, 0.20650E+01_JPRB ,&
&  0.19020E+01_JPRB, 0.17676E+01_JPRB, 0.16699E+01_JPRB, 0.16073E+01_JPRB, 0.15953E+01_JPRB, 0.16046E+01_JPRB, 0.16225E+01_JPRB ,&
&  0.16085E+01_JPRB, 0.15482E+01_JPRB, 0.16489E+01_JPRB, 0.23837E+01_JPRB, 0.45466E+01_JPRB, 0.71623E+01_JPRB, 0.87801E+01_JPRB ,&
&  0.82234E+01_JPRB, 0.79730E+01_JPRB, 0.82312E+01_JPRB, 0.83105E+01_JPRB, 0.72828E+01_JPRB, 0.62185E+01_JPRB, 0.55466E+01_JPRB ,&
&  0.51128E+01_JPRB, 0.48687E+01_JPRB, 0.47797E+01_JPRB, 0.43422E+01_JPRB, 0.37908E+01_JPRB, 0.35145E+01_JPRB, 0.32724E+01_JPRB ,&
&  0.30237E+01_JPRB, 0.29506E+01_JPRB, 0.29136E+01_JPRB, 0.27076E+01_JPRB, 0.25671E+01_JPRB, 0.23786E+01_JPRB, 0.24315E+01_JPRB ,&
&  0.25214E+01_JPRB, 0.18678E+01_JPRB, 0.15263E+01_JPRB, 0.16746E+01_JPRB, 0.12612E+01_JPRB, 0.12159E+01_JPRB, 0.13697E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.11899E+01_JPRB, 0.11883E+01_JPRB, 0.10498E+01_JPRB, 0.10125E+01_JPRB, 0.10322E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.10010E+01_JPRB, 0.97694E+00_JPRB, 0.10814E+01_JPRB, 0.98830E+00_JPRB, 0.96125E+00_JPRB, 0.94781E+00_JPRB, 0.92438E+00_JPRB ,&
&  0.91203E+00_JPRB, 0.89199E+00_JPRB, 0.86584E+00_JPRB, 0.85493E+00_JPRB, 0.85975E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.79866E+00_JPRB, 0.79224E+00_JPRB, 0.79379E+00_JPRB, 0.78598E+00_JPRB, 0.76020E+00_JPRB, 0.72962E+00_JPRB, 0.70436E+00_JPRB ,&
&  0.68847E+00_JPRB, 0.67578E+00_JPRB, 0.66062E+00_JPRB, 0.64199E+00_JPRB, 0.62666E+00_JPRB, 0.61194E+00_JPRB, 0.61591E+00_JPRB ,&
&  0.54515E+00_JPRB, 0.56822E+00_JPRB, 0.59423E+00_JPRB, 0.66300E+00_JPRB, 0.85284E+00_JPRB, 0.83942E+00_JPRB, 0.87244E+00_JPRB ,&
&  0.91533E+00_JPRB, 0.96595E+00_JPRB, 0.98565E+00_JPRB, 0.10218E+01_JPRB, 0.10929E+01_JPRB, 0.12458E+01_JPRB, 0.14361E+01_JPRB ,&
&  0.14313E+01_JPRB, 0.12982E+01_JPRB, 0.12644E+01_JPRB, 0.12398E+01_JPRB, 0.12966E+01_JPRB, 0.13639E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.13963E+01_JPRB, 0.13713E+01_JPRB, 0.13568E+01_JPRB, 0.13485E+01_JPRB, 0.13568E+01_JPRB, 0.13798E+01_JPRB ,&
&  0.14088E+01_JPRB, 0.14030E+01_JPRB, 0.14296E+01_JPRB, 0.18379E+01_JPRB, 0.37656E+01_JPRB, 0.73482E+01_JPRB, 0.11412E+02_JPRB ,&
&  0.10053E+02_JPRB, 0.86846E+01_JPRB, 0.86609E+01_JPRB, 0.88149E+01_JPRB, 0.75323E+01_JPRB, 0.62003E+01_JPRB, 0.51647E+01_JPRB ,&
&  0.46356E+01_JPRB, 0.42857E+01_JPRB, 0.42574E+01_JPRB, 0.42087E+01_JPRB, 0.39208E+01_JPRB, 0.37847E+01_JPRB, 0.36674E+01_JPRB ,&
&  0.34771E+01_JPRB, 0.33700E+01_JPRB, 0.31689E+01_JPRB, 0.29609E+01_JPRB, 0.28378E+01_JPRB, 0.28600E+01_JPRB, 0.27979E+01_JPRB ,&
&  0.27799E+01_JPRB, 0.25749E+01_JPRB, 0.22013E+01_JPRB, 0.21298E+01_JPRB, 0.16799E+01_JPRB, 0.15992E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.15236E+01_JPRB, 0.12587E+01_JPRB, 0.11502E+01_JPRB, 0.10984E+01_JPRB, 0.10749E+01_JPRB, 0.10929E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.95659E+00_JPRB, 0.99906E+00_JPRB, 0.10378E+01_JPRB, 0.90812E+00_JPRB, 0.86316E+00_JPRB, 0.83548E+00_JPRB, 0.77315E+00_JPRB ,&
&  0.80286E+00_JPRB, 0.78957E+00_JPRB, 0.78158E+00_JPRB, 0.78011E+00_JPRB, 0.78926E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.73047E+00_JPRB, 0.73823E+00_JPRB, 0.74494E+00_JPRB, 0.74204E+00_JPRB, 0.72343E+00_JPRB, 0.70069E+00_JPRB, 0.68430E+00_JPRB ,&
&  0.67397E+00_JPRB, 0.65468E+00_JPRB, 0.63316E+00_JPRB, 0.60992E+00_JPRB, 0.59466E+00_JPRB, 0.59206E+00_JPRB, 0.55325E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.51962E+00_JPRB, 0.52872E+00_JPRB, 0.56675E+00_JPRB, 0.62656E+00_JPRB, 0.65161E+00_JPRB, 0.68310E+00_JPRB ,&
&  0.75805E+00_JPRB, 0.81691E+00_JPRB, 0.85496E+00_JPRB, 0.89500E+00_JPRB, 0.92440E+00_JPRB, 0.96179E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.96224E+00_JPRB, 0.87774E+00_JPRB, 0.81613E+00_JPRB, 0.76610E+00_JPRB, 0.80604E+00_JPRB, 0.87819E+00_JPRB, 0.93125E+00_JPRB ,&
&  0.96199E+00_JPRB, 0.98488E+00_JPRB, 0.10129E+01_JPRB, 0.10338E+01_JPRB, 0.10421E+01_JPRB, 0.10562E+01_JPRB, 0.10714E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.11267E+01_JPRB, 0.11618E+01_JPRB, 0.13423E+01_JPRB, 0.27241E+01_JPRB, 0.55040E+01_JPRB, 0.77015E+01_JPRB ,&
&  0.67309E+01_JPRB, 0.63970E+01_JPRB, 0.62636E+01_JPRB, 0.60744E+01_JPRB, 0.55880E+01_JPRB, 0.48187E+01_JPRB, 0.41789E+01_JPRB ,&
&  0.39408E+01_JPRB, 0.36395E+01_JPRB, 0.36685E+01_JPRB, 0.35522E+01_JPRB, 0.34836E+01_JPRB, 0.34155E+01_JPRB, 0.32259E+01_JPRB ,&
&  0.31130E+01_JPRB, 0.29159E+01_JPRB, 0.28566E+01_JPRB, 0.27268E+01_JPRB, 0.26648E+01_JPRB, 0.26688E+01_JPRB, 0.26494E+01_JPRB ,&
&  0.26396E+01_JPRB, 0.26046E+01_JPRB, 0.23915E+01_JPRB, 0.22324E+01_JPRB, 0.20737E+01_JPRB, 0.21602E+01_JPRB, 0.19576E+01_JPRB ,&
&  0.16340E+01_JPRB, 0.12672E+01_JPRB, 0.12164E+01_JPRB, 0.11347E+01_JPRB, 0.10775E+01_JPRB, 0.10361E+01_JPRB, 0.98452E+00_JPRB ,&
&  0.10016E+01_JPRB, 0.10523E+01_JPRB, 0.98052E+00_JPRB, 0.85229E+00_JPRB, 0.79402E+00_JPRB, 0.74930E+00_JPRB, 0.65674E+00_JPRB ,&
&  0.64962E+00_JPRB, 0.70543E+00_JPRB, 0.71341E+00_JPRB, 0.71502E+00_JPRB, 0.71723E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.69348E+00_JPRB, 0.69874E+00_JPRB, 0.69788E+00_JPRB, 0.69853E+00_JPRB, 0.69511E+00_JPRB, 0.68756E+00_JPRB, 0.65528E+00_JPRB ,&
&  0.67717E+00_JPRB, 0.65828E+00_JPRB, 0.62639E+00_JPRB, 0.60437E+00_JPRB, 0.59773E+00_JPRB, 0.55607E+00_JPRB, 0.47810E+00_JPRB ,&
&  0.45920E+00_JPRB, 0.46999E+00_JPRB, 0.49321E+00_JPRB, 0.51522E+00_JPRB, 0.53488E+00_JPRB, 0.54723E+00_JPRB, 0.57032E+00_JPRB ,&
&  0.60301E+00_JPRB, 0.63443E+00_JPRB, 0.67266E+00_JPRB, 0.70852E+00_JPRB, 0.70998E+00_JPRB, 0.70505E+00_JPRB, 0.71525E+00_JPRB ,&
&  0.68760E+00_JPRB, 0.64737E+00_JPRB, 0.62487E+00_JPRB, 0.57012E+00_JPRB, 0.61011E+00_JPRB, 0.62991E+00_JPRB, 0.65126E+00_JPRB ,&
&  0.67826E+00_JPRB, 0.70468E+00_JPRB, 0.73512E+00_JPRB, 0.75901E+00_JPRB, 0.77384E+00_JPRB, 0.79144E+00_JPRB, 0.81035E+00_JPRB ,&
&  0.83171E+00_JPRB, 0.86631E+00_JPRB, 0.91930E+00_JPRB, 0.10137E+01_JPRB, 0.17814E+01_JPRB, 0.35223E+01_JPRB, 0.46700E+01_JPRB ,&
&  0.43559E+01_JPRB, 0.46515E+01_JPRB, 0.47307E+01_JPRB, 0.41528E+01_JPRB, 0.41242E+01_JPRB, 0.40768E+01_JPRB, 0.37526E+01_JPRB ,&
&  0.37214E+01_JPRB, 0.35440E+01_JPRB, 0.32900E+01_JPRB, 0.30655E+01_JPRB, 0.28780E+01_JPRB, 0.26893E+01_JPRB, 0.25158E+01_JPRB ,&
&  0.24221E+01_JPRB, 0.24126E+01_JPRB, 0.24189E+01_JPRB, 0.24723E+01_JPRB, 0.24693E+01_JPRB, 0.24864E+01_JPRB, 0.25023E+01_JPRB ,&
&  0.24870E+01_JPRB, 0.24758E+01_JPRB, 0.24217E+01_JPRB, 0.22979E+01_JPRB, 0.21689E+01_JPRB, 0.20314E+01_JPRB, 0.18765E+01_JPRB ,&
&  0.17315E+01_JPRB, 0.15033E+01_JPRB, 0.13324E+01_JPRB, 0.12494E+01_JPRB, 0.11491E+01_JPRB, 0.10349E+01_JPRB, 0.99784E+00_JPRB ,&
&  0.96809E+00_JPRB, 0.88313E+00_JPRB, 0.86801E+00_JPRB, 0.78289E+00_JPRB, 0.72812E+00_JPRB, 0.69348E+00_JPRB, 0.62712E+00_JPRB ,&
&  0.60039E+00_JPRB, 0.64730E+00_JPRB, 0.67451E+00_JPRB, 0.68957E+00_JPRB, 0.69146E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.66915E+00_JPRB, 0.67571E+00_JPRB, 0.67791E+00_JPRB, 0.68012E+00_JPRB, 0.67113E+00_JPRB, 0.63793E+00_JPRB, 0.60909E+00_JPRB ,&
&  0.66343E+00_JPRB, 0.62291E+00_JPRB, 0.63257E+00_JPRB, 0.53425E+00_JPRB, 0.49188E+00_JPRB, 0.48157E+00_JPRB, 0.46273E+00_JPRB ,&
&  0.45648E+00_JPRB, 0.46267E+00_JPRB, 0.46901E+00_JPRB, 0.47719E+00_JPRB, 0.48907E+00_JPRB, 0.49321E+00_JPRB, 0.49706E+00_JPRB ,&
&  0.50571E+00_JPRB, 0.51911E+00_JPRB, 0.54496E+00_JPRB, 0.55836E+00_JPRB, 0.56570E+00_JPRB, 0.56950E+00_JPRB, 0.57707E+00_JPRB ,&
&  0.54812E+00_JPRB, 0.53094E+00_JPRB, 0.53320E+00_JPRB, 0.52561E+00_JPRB, 0.52320E+00_JPRB, 0.52803E+00_JPRB, 0.54178E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.53403E+00_JPRB, 0.58405E+00_JPRB, 0.61097E+00_JPRB, 0.63261E+00_JPRB, 0.65107E+00_JPRB, 0.66878E+00_JPRB ,&
&  0.68694E+00_JPRB, 0.71130E+00_JPRB, 0.74969E+00_JPRB, 0.82836E+00_JPRB, 0.12008E+01_JPRB, 0.22236E+01_JPRB, 0.28933E+01_JPRB ,&
&  0.31507E+01_JPRB, 0.25086E+01_JPRB, 0.29025E+01_JPRB, 0.30188E+01_JPRB, 0.31871E+01_JPRB, 0.32466E+01_JPRB, 0.32012E+01_JPRB ,&
&  0.32894E+01_JPRB, 0.32280E+01_JPRB, 0.29343E+01_JPRB, 0.25429E+01_JPRB, 0.22447E+01_JPRB, 0.20723E+01_JPRB, 0.19666E+01_JPRB ,&
&  0.18922E+01_JPRB, 0.17759E+01_JPRB, 0.19438E+01_JPRB, 0.21012E+01_JPRB, 0.21749E+01_JPRB, 0.22346E+01_JPRB, 0.23151E+01_JPRB ,&
&  0.23612E+01_JPRB, 0.23265E+01_JPRB, 0.22731E+01_JPRB, 0.20916E+01_JPRB, 0.19927E+01_JPRB, 0.19125E+01_JPRB, 0.17957E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.16424E+01_JPRB, 0.15514E+01_JPRB, 0.15307E+01_JPRB, 0.13391E+01_JPRB, 0.12065E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.94383E+00_JPRB, 0.76244E+00_JPRB, 0.68849E+00_JPRB, 0.64543E+00_JPRB, 0.61190E+00_JPRB, 0.60851E+00_JPRB, 0.61592E+00_JPRB ,&
&  0.59708E+00_JPRB, 0.60843E+00_JPRB, 0.64717E+00_JPRB, 0.65796E+00_JPRB, 0.66065E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.64563E+00_JPRB, 0.66246E+00_JPRB, 0.67534E+00_JPRB, 0.67389E+00_JPRB, 0.65385E+00_JPRB, 0.61252E+00_JPRB, 0.56062E+00_JPRB ,&
&  0.54632E+00_JPRB, 0.52164E+00_JPRB, 0.49963E+00_JPRB, 0.48278E+00_JPRB, 0.47188E+00_JPRB, 0.45849E+00_JPRB, 0.44144E+00_JPRB ,&
&  0.43967E+00_JPRB, 0.44957E+00_JPRB, 0.46416E+00_JPRB, 0.46668E+00_JPRB, 0.46465E+00_JPRB, 0.46270E+00_JPRB, 0.45946E+00_JPRB ,&
&  0.45561E+00_JPRB, 0.45761E+00_JPRB, 0.46860E+00_JPRB, 0.48631E+00_JPRB, 0.49499E+00_JPRB, 0.49572E+00_JPRB, 0.49703E+00_JPRB ,&
&  0.49054E+00_JPRB, 0.48106E+00_JPRB, 0.47536E+00_JPRB, 0.47648E+00_JPRB, 0.47341E+00_JPRB, 0.48344E+00_JPRB, 0.48658E+00_JPRB ,&
&  0.44116E+00_JPRB, 0.45336E+00_JPRB, 0.51931E+00_JPRB, 0.54518E+00_JPRB, 0.56639E+00_JPRB, 0.58363E+00_JPRB, 0.60022E+00_JPRB ,&
&  0.61741E+00_JPRB, 0.64922E+00_JPRB, 0.69180E+00_JPRB, 0.75307E+00_JPRB, 0.96006E+00_JPRB, 0.14666E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.23872E+01_JPRB, 0.20905E+01_JPRB, 0.19717E+01_JPRB, 0.20860E+01_JPRB, 0.25484E+01_JPRB, 0.25985E+01_JPRB, 0.26446E+01_JPRB ,&
&  0.27029E+01_JPRB, 0.25938E+01_JPRB, 0.24299E+01_JPRB, 0.21950E+01_JPRB, 0.18979E+01_JPRB, 0.16821E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.14295E+01_JPRB, 0.13660E+01_JPRB, 0.13816E+01_JPRB, 0.14999E+01_JPRB, 0.16585E+01_JPRB, 0.17842E+01_JPRB, 0.18714E+01_JPRB ,&
&  0.19686E+01_JPRB, 0.20123E+01_JPRB, 0.19673E+01_JPRB, 0.18924E+01_JPRB, 0.17846E+01_JPRB, 0.17195E+01_JPRB, 0.16922E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.16190E+01_JPRB, 0.15461E+01_JPRB, 0.14581E+01_JPRB, 0.13185E+01_JPRB, 0.11896E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.80932E+00_JPRB, 0.67625E+00_JPRB, 0.58483E+00_JPRB, 0.55455E+00_JPRB, 0.54882E+00_JPRB, 0.54965E+00_JPRB, 0.54782E+00_JPRB ,&
&  0.56239E+00_JPRB, 0.57385E+00_JPRB, 0.57784E+00_JPRB, 0.59540E+00_JPRB, 0.61870E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.62743E+00_JPRB, 0.64265E+00_JPRB, 0.64543E+00_JPRB, 0.64510E+00_JPRB, 0.60979E+00_JPRB, 0.57630E+00_JPRB, 0.54387E+00_JPRB ,&
&  0.52887E+00_JPRB, 0.51156E+00_JPRB, 0.49776E+00_JPRB, 0.48003E+00_JPRB, 0.46603E+00_JPRB, 0.46046E+00_JPRB, 0.46310E+00_JPRB ,&
&  0.46922E+00_JPRB, 0.46948E+00_JPRB, 0.46635E+00_JPRB, 0.45505E+00_JPRB, 0.44488E+00_JPRB, 0.43604E+00_JPRB, 0.42888E+00_JPRB ,&
&  0.42257E+00_JPRB, 0.42408E+00_JPRB, 0.43082E+00_JPRB, 0.44322E+00_JPRB, 0.45023E+00_JPRB, 0.45474E+00_JPRB, 0.45259E+00_JPRB ,&
&  0.44240E+00_JPRB, 0.42993E+00_JPRB, 0.43169E+00_JPRB, 0.43928E+00_JPRB, 0.44244E+00_JPRB, 0.44859E+00_JPRB, 0.42709E+00_JPRB ,&
&  0.40894E+00_JPRB, 0.40720E+00_JPRB, 0.42486E+00_JPRB, 0.49931E+00_JPRB, 0.52855E+00_JPRB, 0.54465E+00_JPRB, 0.56342E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.56583E+00_JPRB, 0.60532E+00_JPRB, 0.70414E+00_JPRB, 0.82365E+00_JPRB, 0.10307E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.18056E+01_JPRB, 0.19552E+01_JPRB, 0.17667E+01_JPRB, 0.17625E+01_JPRB, 0.20254E+01_JPRB, 0.20808E+01_JPRB ,&
&  0.20791E+01_JPRB, 0.20204E+01_JPRB, 0.20270E+01_JPRB, 0.18354E+01_JPRB, 0.15625E+01_JPRB, 0.13722E+01_JPRB, 0.12321E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.10066E+01_JPRB, 0.96656E+00_JPRB, 0.95926E+00_JPRB, 0.99494E+00_JPRB, 0.10987E+01_JPRB, 0.12405E+01_JPRB ,&
&  0.13792E+01_JPRB, 0.14585E+01_JPRB, 0.14964E+01_JPRB, 0.14688E+01_JPRB, 0.14387E+01_JPRB, 0.14021E+01_JPRB, 0.13796E+01_JPRB ,&
&  0.13530E+01_JPRB, 0.13434E+01_JPRB, 0.13255E+01_JPRB, 0.12998E+01_JPRB, 0.11799E+01_JPRB, 0.99221E+00_JPRB, 0.81209E+00_JPRB ,&
&  0.68666E+00_JPRB, 0.61222E+00_JPRB, 0.56036E+00_JPRB, 0.54472E+00_JPRB, 0.54321E+00_JPRB, 0.54084E+00_JPRB, 0.52169E+00_JPRB ,&
&  0.51420E+00_JPRB, 0.52466E+00_JPRB, 0.54269E+00_JPRB, 0.56326E+00_JPRB, 0.59303E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.61122E+00_JPRB, 0.61438E+00_JPRB, 0.60450E+00_JPRB, 0.58619E+00_JPRB, 0.56959E+00_JPRB, 0.53008E+00_JPRB, 0.50052E+00_JPRB ,&
&  0.48625E+00_JPRB, 0.48168E+00_JPRB, 0.48229E+00_JPRB, 0.48233E+00_JPRB, 0.48400E+00_JPRB, 0.48726E+00_JPRB, 0.48604E+00_JPRB ,&
&  0.47932E+00_JPRB, 0.47347E+00_JPRB, 0.46802E+00_JPRB, 0.45871E+00_JPRB, 0.44536E+00_JPRB, 0.42965E+00_JPRB, 0.41699E+00_JPRB ,&
&  0.40955E+00_JPRB, 0.40977E+00_JPRB, 0.41925E+00_JPRB, 0.42799E+00_JPRB, 0.43294E+00_JPRB, 0.43293E+00_JPRB, 0.42869E+00_JPRB ,&
&  0.41783E+00_JPRB, 0.41114E+00_JPRB, 0.41685E+00_JPRB, 0.41946E+00_JPRB, 0.42367E+00_JPRB, 0.42584E+00_JPRB, 0.41046E+00_JPRB ,&
&  0.39520E+00_JPRB, 0.38229E+00_JPRB, 0.37942E+00_JPRB, 0.38964E+00_JPRB, 0.43091E+00_JPRB, 0.50032E+00_JPRB, 0.52350E+00_JPRB ,&
&  0.54510E+00_JPRB, 0.56860E+00_JPRB, 0.59653E+00_JPRB, 0.63564E+00_JPRB, 0.69004E+00_JPRB, 0.76765E+00_JPRB, 0.90984E+00_JPRB ,&
&  0.10891E+01_JPRB, 0.12448E+01_JPRB, 0.14027E+01_JPRB, 0.13015E+01_JPRB, 0.12253E+01_JPRB, 0.12525E+01_JPRB, 0.13179E+01_JPRB ,&
&  0.13603E+01_JPRB, 0.13353E+01_JPRB, 0.13469E+01_JPRB, 0.13640E+01_JPRB, 0.12814E+01_JPRB, 0.11726E+01_JPRB, 0.10442E+01_JPRB ,&
&  0.94691E+00_JPRB, 0.87397E+00_JPRB, 0.83276E+00_JPRB, 0.78483E+00_JPRB, 0.76531E+00_JPRB, 0.77726E+00_JPRB, 0.81155E+00_JPRB ,&
&  0.87667E+00_JPRB, 0.96150E+00_JPRB, 0.99734E+00_JPRB, 0.95555E+00_JPRB, 0.97781E+00_JPRB, 0.10295E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10858E+01_JPRB, 0.10900E+01_JPRB, 0.11034E+01_JPRB, 0.10929E+01_JPRB, 0.10092E+01_JPRB, 0.92908E+00_JPRB, 0.83601E+00_JPRB ,&
&  0.74627E+00_JPRB, 0.66604E+00_JPRB, 0.62101E+00_JPRB, 0.60438E+00_JPRB, 0.59193E+00_JPRB, 0.57843E+00_JPRB, 0.55992E+00_JPRB ,&
&  0.54521E+00_JPRB, 0.54385E+00_JPRB, 0.55681E+00_JPRB, 0.57450E+00_JPRB, 0.59712E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.60748E+00_JPRB, 0.57496E+00_JPRB, 0.55259E+00_JPRB, 0.54201E+00_JPRB, 0.53529E+00_JPRB, 0.51799E+00_JPRB, 0.50566E+00_JPRB ,&
&  0.49874E+00_JPRB, 0.50171E+00_JPRB, 0.49856E+00_JPRB, 0.49285E+00_JPRB, 0.49013E+00_JPRB, 0.48819E+00_JPRB, 0.48732E+00_JPRB ,&
&  0.48541E+00_JPRB, 0.47976E+00_JPRB, 0.46780E+00_JPRB, 0.44882E+00_JPRB, 0.43442E+00_JPRB, 0.42376E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.41480E+00_JPRB, 0.41692E+00_JPRB, 0.42446E+00_JPRB, 0.42627E+00_JPRB, 0.42175E+00_JPRB, 0.41493E+00_JPRB, 0.40676E+00_JPRB ,&
&  0.40633E+00_JPRB, 0.41588E+00_JPRB, 0.40850E+00_JPRB, 0.40998E+00_JPRB, 0.41882E+00_JPRB, 0.41104E+00_JPRB, 0.39538E+00_JPRB ,&
&  0.38153E+00_JPRB, 0.36802E+00_JPRB, 0.35938E+00_JPRB, 0.35726E+00_JPRB, 0.36963E+00_JPRB, 0.38967E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.48950E+00_JPRB, 0.51945E+00_JPRB, 0.53977E+00_JPRB, 0.56447E+00_JPRB, 0.59069E+00_JPRB, 0.62235E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.75281E+00_JPRB, 0.83966E+00_JPRB, 0.92524E+00_JPRB, 0.98820E+00_JPRB, 0.10340E+01_JPRB, 0.97458E+00_JPRB, 0.96012E+00_JPRB ,&
&  0.10622E+01_JPRB, 0.10974E+01_JPRB, 0.10420E+01_JPRB, 0.98334E+00_JPRB, 0.91821E+00_JPRB, 0.86116E+00_JPRB, 0.81704E+00_JPRB ,&
&  0.78894E+00_JPRB, 0.76967E+00_JPRB, 0.74800E+00_JPRB, 0.70636E+00_JPRB, 0.66944E+00_JPRB, 0.65467E+00_JPRB, 0.65818E+00_JPRB ,&
&  0.68311E+00_JPRB, 0.78549E+00_JPRB, 0.10247E+01_JPRB, 0.87820E+00_JPRB, 0.82054E+00_JPRB, 0.81852E+00_JPRB, 0.83439E+00_JPRB ,&
&  0.86974E+00_JPRB, 0.89999E+00_JPRB, 0.92168E+00_JPRB, 0.91254E+00_JPRB, 0.88333E+00_JPRB, 0.84504E+00_JPRB, 0.81714E+00_JPRB ,&
&  0.79284E+00_JPRB, 0.76772E+00_JPRB, 0.74937E+00_JPRB, 0.74140E+00_JPRB, 0.72691E+00_JPRB, 0.71060E+00_JPRB, 0.69538E+00_JPRB ,&
&  0.68130E+00_JPRB, 0.67319E+00_JPRB, 0.66846E+00_JPRB, 0.65825E+00_JPRB, 0.64214E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.54815E+00_JPRB, 0.53517E+00_JPRB, 0.52511E+00_JPRB, 0.51884E+00_JPRB, 0.51341E+00_JPRB, 0.50837E+00_JPRB, 0.50427E+00_JPRB ,&
&  0.50112E+00_JPRB, 0.49897E+00_JPRB, 0.49514E+00_JPRB, 0.49104E+00_JPRB, 0.48743E+00_JPRB, 0.48416E+00_JPRB, 0.48126E+00_JPRB ,&
&  0.47487E+00_JPRB, 0.45736E+00_JPRB, 0.45076E+00_JPRB, 0.44369E+00_JPRB, 0.43395E+00_JPRB, 0.42873E+00_JPRB, 0.42579E+00_JPRB ,&
&  0.42138E+00_JPRB, 0.41901E+00_JPRB, 0.41957E+00_JPRB, 0.41379E+00_JPRB, 0.40892E+00_JPRB, 0.41069E+00_JPRB, 0.41311E+00_JPRB ,&
&  0.40396E+00_JPRB, 0.40339E+00_JPRB, 0.41964E+00_JPRB, 0.41703E+00_JPRB, 0.39758E+00_JPRB, 0.39445E+00_JPRB, 0.38325E+00_JPRB ,&
&  0.37413E+00_JPRB, 0.35541E+00_JPRB, 0.34941E+00_JPRB, 0.34543E+00_JPRB, 0.35226E+00_JPRB, 0.36352E+00_JPRB, 0.39018E+00_JPRB ,&
&  0.41862E+00_JPRB, 0.47077E+00_JPRB, 0.49230E+00_JPRB, 0.50915E+00_JPRB, 0.52497E+00_JPRB, 0.54166E+00_JPRB, 0.56156E+00_JPRB ,&
&  0.58760E+00_JPRB, 0.62087E+00_JPRB, 0.66260E+00_JPRB, 0.70010E+00_JPRB, 0.73219E+00_JPRB, 0.74554E+00_JPRB, 0.74800E+00_JPRB ,&
&  0.76552E+00_JPRB, 0.76092E+00_JPRB, 0.73562E+00_JPRB, 0.70826E+00_JPRB, 0.67778E+00_JPRB, 0.64817E+00_JPRB, 0.61686E+00_JPRB ,&
&  0.60747E+00_JPRB, 0.61675E+00_JPRB, 0.60803E+00_JPRB, 0.59874E+00_JPRB, 0.58758E+00_JPRB, 0.58493E+00_JPRB, 0.58427E+00_JPRB ,&
&  0.57731E+00_JPRB, 0.59493E+00_JPRB, 0.66796E+00_JPRB, 0.67373E+00_JPRB, 0.67987E+00_JPRB, 0.68530E+00_JPRB, 0.69260E+00_JPRB ,&
&  0.70532E+00_JPRB, 0.72372E+00_JPRB, 0.74294E+00_JPRB, 0.75534E+00_JPRB, 0.76625E+00_JPRB, 0.76382E+00_JPRB, 0.77224E+00_JPRB ,&
&  0.76770E+00_JPRB, 0.76037E+00_JPRB, 0.74947E+00_JPRB, 0.74069E+00_JPRB, 0.72931E+00_JPRB, 0.71593E+00_JPRB, 0.69693E+00_JPRB ,&
&  0.67428E+00_JPRB, 0.64834E+00_JPRB, 0.62122E+00_JPRB, 0.59429E+00_JPRB, 0.56840E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.50578E+00_JPRB, 0.49963E+00_JPRB, 0.49498E+00_JPRB, 0.49090E+00_JPRB, 0.48824E+00_JPRB, 0.48555E+00_JPRB, 0.48340E+00_JPRB ,&
&  0.48193E+00_JPRB, 0.47927E+00_JPRB, 0.47584E+00_JPRB, 0.47153E+00_JPRB, 0.46790E+00_JPRB, 0.46446E+00_JPRB, 0.46079E+00_JPRB ,&
&  0.45535E+00_JPRB, 0.44932E+00_JPRB, 0.44298E+00_JPRB, 0.43777E+00_JPRB, 0.43855E+00_JPRB, 0.43789E+00_JPRB, 0.43669E+00_JPRB ,&
&  0.43172E+00_JPRB, 0.42379E+00_JPRB, 0.41364E+00_JPRB, 0.41145E+00_JPRB, 0.41071E+00_JPRB, 0.40444E+00_JPRB, 0.40712E+00_JPRB ,&
&  0.41332E+00_JPRB, 0.41204E+00_JPRB, 0.41017E+00_JPRB, 0.40787E+00_JPRB, 0.39005E+00_JPRB, 0.35585E+00_JPRB, 0.36591E+00_JPRB ,&
&  0.36209E+00_JPRB, 0.35075E+00_JPRB, 0.34679E+00_JPRB, 0.34455E+00_JPRB, 0.34887E+00_JPRB, 0.35914E+00_JPRB, 0.37038E+00_JPRB ,&
&  0.37745E+00_JPRB, 0.43432E+00_JPRB, 0.45692E+00_JPRB, 0.47216E+00_JPRB, 0.48213E+00_JPRB, 0.49081E+00_JPRB, 0.50001E+00_JPRB ,&
&  0.50839E+00_JPRB, 0.52176E+00_JPRB, 0.53944E+00_JPRB, 0.55625E+00_JPRB, 0.57202E+00_JPRB, 0.58258E+00_JPRB, 0.59321E+00_JPRB ,&
&  0.60474E+00_JPRB, 0.60516E+00_JPRB, 0.59492E+00_JPRB, 0.58348E+00_JPRB, 0.56948E+00_JPRB, 0.55498E+00_JPRB, 0.54145E+00_JPRB ,&
&  0.53437E+00_JPRB, 0.52831E+00_JPRB, 0.53096E+00_JPRB, 0.53174E+00_JPRB, 0.53110E+00_JPRB, 0.53315E+00_JPRB, 0.53855E+00_JPRB ,&
&  0.54417E+00_JPRB, 0.55193E+00_JPRB, 0.55761E+00_JPRB, 0.56683E+00_JPRB, 0.57767E+00_JPRB, 0.58505E+00_JPRB, 0.59192E+00_JPRB ,&
&  0.59578E+00_JPRB, 0.60482E+00_JPRB, 0.60920E+00_JPRB, 0.61685E+00_JPRB, 0.62670E+00_JPRB, 0.63299E+00_JPRB, 0.63886E+00_JPRB ,&
&  0.64291E+00_JPRB, 0.64382E+00_JPRB, 0.63999E+00_JPRB, 0.63124E+00_JPRB, 0.61845E+00_JPRB, 0.60270E+00_JPRB, 0.58348E+00_JPRB ,&
&  0.56316E+00_JPRB, 0.54372E+00_JPRB, 0.52885E+00_JPRB, 0.51909E+00_JPRB, 0.51131E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.48359E+00_JPRB, 0.47791E+00_JPRB, 0.47390E+00_JPRB, 0.47029E+00_JPRB, 0.46793E+00_JPRB, 0.46563E+00_JPRB, 0.46335E+00_JPRB ,&
&  0.46111E+00_JPRB, 0.45917E+00_JPRB, 0.45714E+00_JPRB, 0.45566E+00_JPRB, 0.45368E+00_JPRB, 0.45220E+00_JPRB, 0.45026E+00_JPRB ,&
&  0.44946E+00_JPRB, 0.44777E+00_JPRB, 0.44807E+00_JPRB, 0.44611E+00_JPRB, 0.44711E+00_JPRB, 0.44536E+00_JPRB, 0.43977E+00_JPRB ,&
&  0.43386E+00_JPRB, 0.42562E+00_JPRB, 0.41205E+00_JPRB, 0.41472E+00_JPRB, 0.41107E+00_JPRB, 0.41135E+00_JPRB, 0.40452E+00_JPRB ,&
&  0.40255E+00_JPRB, 0.38595E+00_JPRB, 0.39460E+00_JPRB, 0.38677E+00_JPRB, 0.38152E+00_JPRB, 0.37476E+00_JPRB, 0.36496E+00_JPRB ,&
&  0.36224E+00_JPRB, 0.35639E+00_JPRB, 0.35608E+00_JPRB, 0.35846E+00_JPRB, 0.36242E+00_JPRB, 0.36877E+00_JPRB, 0.37557E+00_JPRB ,&
&  0.37874E+00_JPRB, 0.41366E+00_JPRB, 0.43357E+00_JPRB, 0.44768E+00_JPRB, 0.46011E+00_JPRB, 0.46951E+00_JPRB, 0.47761E+00_JPRB ,&
&  0.48514E+00_JPRB, 0.49235E+00_JPRB, 0.50042E+00_JPRB, 0.50302E+00_JPRB, 0.51273E+00_JPRB, 0.51628E+00_JPRB, 0.52072E+00_JPRB ,&
&  0.52467E+00_JPRB, 0.52420E+00_JPRB, 0.52252E+00_JPRB, 0.51838E+00_JPRB, 0.51594E+00_JPRB, 0.51169E+00_JPRB, 0.50994E+00_JPRB ,&
&  0.50728E+00_JPRB, 0.50572E+00_JPRB, 0.50777E+00_JPRB, 0.50952E+00_JPRB, 0.51040E+00_JPRB, 0.51164E+00_JPRB, 0.51344E+00_JPRB ,&
&  0.51610E+00_JPRB, 0.52105E+00_JPRB, 0.52579E+00_JPRB, 0.53411E+00_JPRB, 0.54245E+00_JPRB, 0.54773E+00_JPRB, 0.55299E+00_JPRB ,&
&  0.55894E+00_JPRB, 0.55611E+00_JPRB, 0.55418E+00_JPRB, 0.55393E+00_JPRB, 0.55490E+00_JPRB, 0.55658E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.55512E+00_JPRB, 0.54981E+00_JPRB, 0.54290E+00_JPRB, 0.53580E+00_JPRB, 0.52961E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.51715E+00_JPRB, 0.51184E+00_JPRB, 0.50489E+00_JPRB, 0.49696E+00_JPRB, 0.48957E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.47211E+00_JPRB, 0.46633E+00_JPRB, 0.46157E+00_JPRB, 0.45846E+00_JPRB, 0.45552E+00_JPRB, 0.45332E+00_JPRB, 0.45156E+00_JPRB ,&
&  0.45002E+00_JPRB, 0.44871E+00_JPRB, 0.44803E+00_JPRB, 0.44746E+00_JPRB, 0.44676E+00_JPRB, 0.44632E+00_JPRB, 0.44565E+00_JPRB ,&
&  0.44668E+00_JPRB, 0.44608E+00_JPRB, 0.44725E+00_JPRB, 0.44611E+00_JPRB, 0.44537E+00_JPRB, 0.44357E+00_JPRB, 0.43873E+00_JPRB ,&
&  0.43596E+00_JPRB, 0.43213E+00_JPRB, 0.41476E+00_JPRB, 0.41919E+00_JPRB, 0.41270E+00_JPRB, 0.40977E+00_JPRB, 0.40768E+00_JPRB ,&
&  0.40033E+00_JPRB, 0.39672E+00_JPRB, 0.39173E+00_JPRB, 0.39044E+00_JPRB, 0.39083E+00_JPRB, 0.38649E+00_JPRB, 0.38665E+00_JPRB ,&
&  0.38247E+00_JPRB, 0.38154E+00_JPRB, 0.38224E+00_JPRB, 0.38313E+00_JPRB, 0.38718E+00_JPRB, 0.38886E+00_JPRB, 0.39335E+00_JPRB ,&
&  0.40237E+00_JPRB, 0.41575E+00_JPRB, 0.43861E+00_JPRB, 0.44893E+00_JPRB, 0.46138E+00_JPRB, 0.47075E+00_JPRB, 0.47964E+00_JPRB ,&
&  0.48510E+00_JPRB, 0.48894E+00_JPRB, 0.49197E+00_JPRB, 0.49412E+00_JPRB, 0.49479E+00_JPRB, 0.49265E+00_JPRB, 0.49292E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.49506E+00_JPRB, 0.49588E+00_JPRB, 0.49579E+00_JPRB, 0.49726E+00_JPRB, 0.49860E+00_JPRB, 0.50147E+00_JPRB ,&
&  0.50363E+00_JPRB, 0.50537E+00_JPRB, 0.50472E+00_JPRB, 0.50538E+00_JPRB, 0.50483E+00_JPRB, 0.50483E+00_JPRB, 0.50491E+00_JPRB ,&
&  0.50547E+00_JPRB, 0.50732E+00_JPRB, 0.51031E+00_JPRB, 0.50728E+00_JPRB, 0.50838E+00_JPRB, 0.51780E+00_JPRB, 0.52103E+00_JPRB ,&
&  0.52498E+00_JPRB, 0.52732E+00_JPRB, 0.52854E+00_JPRB, 0.52935E+00_JPRB, 0.52976E+00_JPRB, 0.53105E+00_JPRB, 0.53167E+00_JPRB ,&
&  0.53189E+00_JPRB, 0.52743E+00_JPRB, 0.52240E+00_JPRB, 0.51775E+00_JPRB, 0.51268E+00_JPRB, 0.50858E+00_JPRB, 0.50405E+00_JPRB ,&
&  0.50013E+00_JPRB, 0.49702E+00_JPRB, 0.49383E+00_JPRB, 0.48765E+00_JPRB, 0.47991E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.46352E+00_JPRB, 0.45919E+00_JPRB, 0.45619E+00_JPRB, 0.45399E+00_JPRB, 0.45168E+00_JPRB, 0.45003E+00_JPRB, 0.44929E+00_JPRB ,&
&  0.44900E+00_JPRB, 0.44795E+00_JPRB, 0.44752E+00_JPRB, 0.44710E+00_JPRB, 0.44619E+00_JPRB, 0.44651E+00_JPRB, 0.44645E+00_JPRB ,&
&  0.44689E+00_JPRB, 0.44636E+00_JPRB, 0.44766E+00_JPRB, 0.44687E+00_JPRB, 0.44667E+00_JPRB, 0.44679E+00_JPRB, 0.44549E+00_JPRB ,&
&  0.44534E+00_JPRB, 0.44409E+00_JPRB, 0.43944E+00_JPRB, 0.43448E+00_JPRB, 0.43043E+00_JPRB, 0.41240E+00_JPRB, 0.41167E+00_JPRB ,&
&  0.41584E+00_JPRB, 0.40954E+00_JPRB, 0.40730E+00_JPRB, 0.40536E+00_JPRB, 0.40528E+00_JPRB, 0.40451E+00_JPRB, 0.40701E+00_JPRB ,&
&  0.39869E+00_JPRB, 0.40733E+00_JPRB, 0.40772E+00_JPRB, 0.40800E+00_JPRB, 0.40949E+00_JPRB, 0.40905E+00_JPRB, 0.41455E+00_JPRB ,&
&  0.43412E+00_JPRB, 0.43862E+00_JPRB, 0.44712E+00_JPRB, 0.45455E+00_JPRB, 0.46320E+00_JPRB, 0.47086E+00_JPRB, 0.47825E+00_JPRB ,&
&  0.48377E+00_JPRB, 0.48735E+00_JPRB, 0.48961E+00_JPRB, 0.48991E+00_JPRB, 0.49038E+00_JPRB, 0.49030E+00_JPRB, 0.49082E+00_JPRB ,&
&  0.49135E+00_JPRB, 0.49278E+00_JPRB, 0.49487E+00_JPRB, 0.49663E+00_JPRB, 0.49851E+00_JPRB, 0.49932E+00_JPRB, 0.50023E+00_JPRB ,&
&  0.50061E+00_JPRB, 0.50042E+00_JPRB, 0.49953E+00_JPRB, 0.49896E+00_JPRB, 0.49767E+00_JPRB, 0.49730E+00_JPRB, 0.49696E+00_JPRB ,&
&  0.49759E+00_JPRB, 0.49767E+00_JPRB, 0.49861E+00_JPRB, 0.49932E+00_JPRB, 0.50060E+00_JPRB, 0.50283E+00_JPRB, 0.50424E+00_JPRB ,&
&  0.50717E+00_JPRB, 0.50900E+00_JPRB, 0.51072E+00_JPRB, 0.51215E+00_JPRB, 0.51282E+00_JPRB, 0.51415E+00_JPRB, 0.51488E+00_JPRB ,&
&  0.51639E+00_JPRB, 0.51392E+00_JPRB, 0.51059E+00_JPRB, 0.50659E+00_JPRB, 0.50293E+00_JPRB, 0.49939E+00_JPRB, 0.49555E+00_JPRB ,&
&  0.49167E+00_JPRB, 0.48654E+00_JPRB, 0.48111E+00_JPRB, 0.47495E+00_JPRB, 0.46868E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.46376E+00_JPRB, 0.46092E+00_JPRB, 0.45883E+00_JPRB, 0.45732E+00_JPRB, 0.45629E+00_JPRB, 0.45542E+00_JPRB, 0.45468E+00_JPRB ,&
&  0.45399E+00_JPRB, 0.45416E+00_JPRB, 0.45396E+00_JPRB, 0.45374E+00_JPRB, 0.45335E+00_JPRB, 0.45359E+00_JPRB, 0.45342E+00_JPRB ,&
&  0.45293E+00_JPRB, 0.45160E+00_JPRB, 0.45052E+00_JPRB, 0.44938E+00_JPRB, 0.44952E+00_JPRB, 0.44982E+00_JPRB, 0.45101E+00_JPRB ,&
&  0.45092E+00_JPRB, 0.45058E+00_JPRB, 0.44955E+00_JPRB, 0.44616E+00_JPRB, 0.44265E+00_JPRB, 0.43928E+00_JPRB, 0.43642E+00_JPRB ,&
&  0.43405E+00_JPRB, 0.43247E+00_JPRB, 0.43185E+00_JPRB, 0.43178E+00_JPRB, 0.43355E+00_JPRB, 0.43375E+00_JPRB, 0.43351E+00_JPRB ,&
&  0.43266E+00_JPRB, 0.43026E+00_JPRB, 0.43060E+00_JPRB, 0.43128E+00_JPRB, 0.43253E+00_JPRB, 0.43569E+00_JPRB, 0.43867E+00_JPRB ,&
&  0.44246E+00_JPRB, 0.44685E+00_JPRB, 0.45356E+00_JPRB, 0.45847E+00_JPRB, 0.46353E+00_JPRB, 0.46774E+00_JPRB, 0.46996E+00_JPRB ,&
&  0.47177E+00_JPRB, 0.47329E+00_JPRB, 0.47491E+00_JPRB, 0.47622E+00_JPRB, 0.47736E+00_JPRB, 0.47871E+00_JPRB, 0.48035E+00_JPRB ,&
&  0.48159E+00_JPRB, 0.48282E+00_JPRB, 0.48368E+00_JPRB, 0.48456E+00_JPRB, 0.48552E+00_JPRB, 0.48653E+00_JPRB, 0.48774E+00_JPRB ,&
&  0.48884E+00_JPRB, 0.48896E+00_JPRB, 0.48881E+00_JPRB, 0.48936E+00_JPRB, 0.48993E+00_JPRB, 0.49033E+00_JPRB, 0.49011E+00_JPRB ,&
&  0.49027E+00_JPRB, 0.49041E+00_JPRB, 0.49078E+00_JPRB, 0.49143E+00_JPRB, 0.49186E+00_JPRB, 0.49217E+00_JPRB, 0.49195E+00_JPRB ,&
&  0.49177E+00_JPRB, 0.49102E+00_JPRB, 0.49030E+00_JPRB, 0.49040E+00_JPRB, 0.49033E+00_JPRB, 0.49103E+00_JPRB, 0.49197E+00_JPRB ,&
&  0.49318E+00_JPRB, 0.49266E+00_JPRB, 0.49168E+00_JPRB, 0.48995E+00_JPRB, 0.48809E+00_JPRB, 0.48581E+00_JPRB, 0.48349E+00_JPRB ,&
&  0.48102E+00_JPRB, 0.47796E+00_JPRB, 0.47421E+00_JPRB, 0.47044E+00_JPRB, 0.46696E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.47164E+00_JPRB, 0.47042E+00_JPRB, 0.46927E+00_JPRB, 0.46809E+00_JPRB, 0.46697E+00_JPRB, 0.46611E+00_JPRB, 0.46536E+00_JPRB ,&
&  0.46468E+00_JPRB, 0.46400E+00_JPRB, 0.46321E+00_JPRB, 0.46252E+00_JPRB, 0.46198E+00_JPRB, 0.46139E+00_JPRB, 0.46090E+00_JPRB ,&
&  0.46044E+00_JPRB, 0.46013E+00_JPRB, 0.45988E+00_JPRB, 0.45940E+00_JPRB, 0.45905E+00_JPRB, 0.45867E+00_JPRB, 0.45792E+00_JPRB ,&
&  0.45695E+00_JPRB, 0.45601E+00_JPRB, 0.45496E+00_JPRB, 0.45407E+00_JPRB, 0.45338E+00_JPRB, 0.45239E+00_JPRB, 0.45119E+00_JPRB ,&
&  0.45009E+00_JPRB, 0.44935E+00_JPRB, 0.44884E+00_JPRB, 0.44858E+00_JPRB, 0.44856E+00_JPRB, 0.44848E+00_JPRB, 0.44837E+00_JPRB ,&
&  0.44836E+00_JPRB, 0.44853E+00_JPRB, 0.44890E+00_JPRB, 0.44953E+00_JPRB, 0.45036E+00_JPRB, 0.45150E+00_JPRB, 0.45330E+00_JPRB ,&
&  0.45531E+00_JPRB, 0.45737E+00_JPRB, 0.45950E+00_JPRB, 0.46145E+00_JPRB, 0.46326E+00_JPRB, 0.46462E+00_JPRB, 0.46555E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.46774E+00_JPRB, 0.46898E+00_JPRB, 0.46981E+00_JPRB, 0.47051E+00_JPRB, 0.47113E+00_JPRB, 0.47184E+00_JPRB ,&
&  0.47226E+00_JPRB, 0.47270E+00_JPRB, 0.47321E+00_JPRB, 0.47384E+00_JPRB, 0.47433E+00_JPRB, 0.47476E+00_JPRB, 0.47530E+00_JPRB ,&
&  0.47603E+00_JPRB, 0.47679E+00_JPRB, 0.47732E+00_JPRB, 0.47782E+00_JPRB, 0.47845E+00_JPRB, 0.47923E+00_JPRB, 0.48001E+00_JPRB ,&
&  0.48071E+00_JPRB, 0.48135E+00_JPRB, 0.48196E+00_JPRB, 0.48249E+00_JPRB, 0.48283E+00_JPRB, 0.48291E+00_JPRB, 0.48246E+00_JPRB ,&
&  0.48188E+00_JPRB, 0.48116E+00_JPRB, 0.48073E+00_JPRB, 0.48064E+00_JPRB, 0.48046E+00_JPRB, 0.48042E+00_JPRB, 0.48055E+00_JPRB ,&
&  0.48090E+00_JPRB, 0.48122E+00_JPRB, 0.48143E+00_JPRB, 0.48159E+00_JPRB, 0.48146E+00_JPRB, 0.48094E+00_JPRB, 0.48019E+00_JPRB ,&
&  0.47927E+00_JPRB, 0.47797E+00_JPRB, 0.47649E+00_JPRB, 0.47487E+00_JPRB, 0.47317E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB, 0.47108E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB ,&
&  0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB, 0.12838E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.13363E+00_JPRB, 0.13381E+00_JPRB, 0.13391E+00_JPRB, 0.13397E+00_JPRB, 0.13388E+00_JPRB, 0.13390E+00_JPRB, 0.13390E+00_JPRB ,&
&  0.13391E+00_JPRB, 0.13380E+00_JPRB, 0.13398E+00_JPRB, 0.13408E+00_JPRB, 0.13413E+00_JPRB, 0.13415E+00_JPRB, 0.13405E+00_JPRB ,&
&  0.13382E+00_JPRB, 0.13354E+00_JPRB, 0.13278E+00_JPRB, 0.13266E+00_JPRB, 0.13271E+00_JPRB, 0.13291E+00_JPRB, 0.13305E+00_JPRB ,&
&  0.13326E+00_JPRB, 0.13347E+00_JPRB, 0.13363E+00_JPRB, 0.13409E+00_JPRB, 0.13430E+00_JPRB, 0.13454E+00_JPRB, 0.13478E+00_JPRB ,&
&  0.13501E+00_JPRB, 0.13527E+00_JPRB, 0.13546E+00_JPRB, 0.13566E+00_JPRB, 0.13583E+00_JPRB, 0.13577E+00_JPRB, 0.13558E+00_JPRB ,&
&  0.13530E+00_JPRB, 0.13486E+00_JPRB, 0.13440E+00_JPRB, 0.13399E+00_JPRB, 0.13363E+00_JPRB, 0.13324E+00_JPRB, 0.13288E+00_JPRB ,&
&  0.13248E+00_JPRB, 0.13202E+00_JPRB, 0.13153E+00_JPRB, 0.13109E+00_JPRB, 0.13064E+00_JPRB, 0.13025E+00_JPRB, 0.12981E+00_JPRB ,&
&  0.12946E+00_JPRB, 0.12912E+00_JPRB, 0.12878E+00_JPRB, 0.12844E+00_JPRB, 0.12814E+00_JPRB, 0.12784E+00_JPRB, 0.12755E+00_JPRB ,&
&  0.12724E+00_JPRB, 0.12706E+00_JPRB, 0.12684E+00_JPRB, 0.12663E+00_JPRB, 0.12644E+00_JPRB, 0.12630E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12595E+00_JPRB, 0.12578E+00_JPRB, 0.12570E+00_JPRB, 0.12558E+00_JPRB, 0.12553E+00_JPRB, 0.12550E+00_JPRB, 0.12556E+00_JPRB ,&
&  0.12561E+00_JPRB, 0.12570E+00_JPRB, 0.12584E+00_JPRB, 0.12607E+00_JPRB, 0.12628E+00_JPRB, 0.12651E+00_JPRB, 0.12675E+00_JPRB ,&
&  0.12701E+00_JPRB, 0.12728E+00_JPRB, 0.12756E+00_JPRB, 0.12783E+00_JPRB, 0.12816E+00_JPRB, 0.12849E+00_JPRB, 0.12886E+00_JPRB ,&
&  0.12920E+00_JPRB, 0.12957E+00_JPRB, 0.12987E+00_JPRB, 0.13022E+00_JPRB, 0.13050E+00_JPRB, 0.13068E+00_JPRB, 0.13084E+00_JPRB ,&
&  0.13114E+00_JPRB, 0.13143E+00_JPRB, 0.13180E+00_JPRB, 0.13213E+00_JPRB, 0.13272E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.14696E+00_JPRB, 0.14932E+00_JPRB, 0.15107E+00_JPRB, 0.15304E+00_JPRB, 0.15238E+00_JPRB, 0.16375E+00_JPRB, 0.16191E+00_JPRB ,&
&  0.16164E+00_JPRB, 0.15632E+00_JPRB, 0.15639E+00_JPRB, 0.15545E+00_JPRB, 0.15397E+00_JPRB, 0.15233E+00_JPRB, 0.15057E+00_JPRB ,&
&  0.14887E+00_JPRB, 0.14676E+00_JPRB, 0.14495E+00_JPRB, 0.14344E+00_JPRB, 0.14207E+00_JPRB, 0.14087E+00_JPRB, 0.13965E+00_JPRB ,&
&  0.13941E+00_JPRB, 0.13995E+00_JPRB, 0.14085E+00_JPRB, 0.14035E+00_JPRB, 0.14075E+00_JPRB, 0.14158E+00_JPRB, 0.14268E+00_JPRB ,&
&  0.14363E+00_JPRB, 0.14489E+00_JPRB, 0.14568E+00_JPRB, 0.14587E+00_JPRB, 0.14596E+00_JPRB, 0.14527E+00_JPRB, 0.14481E+00_JPRB ,&
&  0.14419E+00_JPRB, 0.14335E+00_JPRB, 0.14229E+00_JPRB, 0.14139E+00_JPRB, 0.14037E+00_JPRB, 0.13903E+00_JPRB, 0.13762E+00_JPRB ,&
&  0.13635E+00_JPRB, 0.13523E+00_JPRB, 0.13408E+00_JPRB, 0.13312E+00_JPRB, 0.13220E+00_JPRB, 0.13129E+00_JPRB, 0.13046E+00_JPRB ,&
&  0.12981E+00_JPRB, 0.12923E+00_JPRB, 0.12859E+00_JPRB, 0.12791E+00_JPRB, 0.12714E+00_JPRB, 0.12639E+00_JPRB, 0.12568E+00_JPRB ,&
&  0.12530E+00_JPRB, 0.12496E+00_JPRB, 0.12462E+00_JPRB, 0.12425E+00_JPRB, 0.12409E+00_JPRB, 0.12393E+00_JPRB, 0.12375E+00_JPRB ,&
&  0.12357E+00_JPRB, 0.12331E+00_JPRB, 0.12321E+00_JPRB, 0.12312E+00_JPRB, 0.12303E+00_JPRB, 0.12297E+00_JPRB, 0.12306E+00_JPRB ,&
&  0.12322E+00_JPRB, 0.12344E+00_JPRB, 0.12386E+00_JPRB, 0.12432E+00_JPRB, 0.12482E+00_JPRB, 0.12531E+00_JPRB, 0.12600E+00_JPRB ,&
&  0.12676E+00_JPRB, 0.12749E+00_JPRB, 0.12813E+00_JPRB, 0.12867E+00_JPRB, 0.12947E+00_JPRB, 0.13023E+00_JPRB, 0.13096E+00_JPRB ,&
&  0.13178E+00_JPRB, 0.13281E+00_JPRB, 0.13389E+00_JPRB, 0.13499E+00_JPRB, 0.13625E+00_JPRB, 0.13737E+00_JPRB, 0.13847E+00_JPRB ,&
&  0.13977E+00_JPRB, 0.14212E+00_JPRB, 0.14381E+00_JPRB, 0.14496E+00_JPRB, 0.14626E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.16782E+00_JPRB, 0.16901E+00_JPRB, 0.17002E+00_JPRB, 0.17054E+00_JPRB, 0.17102E+00_JPRB, 0.17105E+00_JPRB, 0.17100E+00_JPRB ,&
&  0.16841E+00_JPRB, 0.16725E+00_JPRB, 0.16562E+00_JPRB, 0.16377E+00_JPRB, 0.16215E+00_JPRB, 0.16032E+00_JPRB, 0.15840E+00_JPRB ,&
&  0.15657E+00_JPRB, 0.15438E+00_JPRB, 0.15198E+00_JPRB, 0.14918E+00_JPRB, 0.14644E+00_JPRB, 0.14385E+00_JPRB, 0.14227E+00_JPRB ,&
&  0.14145E+00_JPRB, 0.14160E+00_JPRB, 0.14291E+00_JPRB, 0.14402E+00_JPRB, 0.14564E+00_JPRB, 0.14773E+00_JPRB, 0.15112E+00_JPRB ,&
&  0.15378E+00_JPRB, 0.15703E+00_JPRB, 0.15923E+00_JPRB, 0.15999E+00_JPRB, 0.15928E+00_JPRB, 0.15892E+00_JPRB, 0.15808E+00_JPRB ,&
&  0.15646E+00_JPRB, 0.15474E+00_JPRB, 0.15324E+00_JPRB, 0.15164E+00_JPRB, 0.14983E+00_JPRB, 0.14753E+00_JPRB, 0.14474E+00_JPRB ,&
&  0.14218E+00_JPRB, 0.13995E+00_JPRB, 0.13789E+00_JPRB, 0.13631E+00_JPRB, 0.13481E+00_JPRB, 0.13345E+00_JPRB, 0.13205E+00_JPRB ,&
&  0.13097E+00_JPRB, 0.13002E+00_JPRB, 0.12883E+00_JPRB, 0.12780E+00_JPRB, 0.12665E+00_JPRB, 0.12569E+00_JPRB, 0.12465E+00_JPRB ,&
&  0.12382E+00_JPRB, 0.12308E+00_JPRB, 0.12246E+00_JPRB, 0.12188E+00_JPRB, 0.12149E+00_JPRB, 0.12110E+00_JPRB, 0.12067E+00_JPRB ,&
&  0.12028E+00_JPRB, 0.12016E+00_JPRB, 0.12010E+00_JPRB, 0.12006E+00_JPRB, 0.12009E+00_JPRB, 0.11980E+00_JPRB, 0.11984E+00_JPRB ,&
&  0.12014E+00_JPRB, 0.12047E+00_JPRB, 0.12064E+00_JPRB, 0.12108E+00_JPRB, 0.12174E+00_JPRB, 0.12254E+00_JPRB, 0.12367E+00_JPRB ,&
&  0.12456E+00_JPRB, 0.12545E+00_JPRB, 0.12628E+00_JPRB, 0.12713E+00_JPRB, 0.12821E+00_JPRB, 0.12938E+00_JPRB, 0.13061E+00_JPRB ,&
&  0.13207E+00_JPRB, 0.13345E+00_JPRB, 0.13488E+00_JPRB, 0.13639E+00_JPRB, 0.13873E+00_JPRB, 0.14113E+00_JPRB, 0.14477E+00_JPRB ,&
&  0.14850E+00_JPRB, 0.15067E+00_JPRB, 0.16313E+00_JPRB, 0.16417E+00_JPRB, 0.16563E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.16829E+00_JPRB, 0.16972E+00_JPRB, 0.17062E+00_JPRB, 0.17177E+00_JPRB, 0.17239E+00_JPRB, 0.17394E+00_JPRB, 0.17431E+00_JPRB ,&
&  0.17445E+00_JPRB, 0.17054E+00_JPRB, 0.16816E+00_JPRB, 0.16557E+00_JPRB, 0.16333E+00_JPRB, 0.16081E+00_JPRB, 0.15835E+00_JPRB ,&
&  0.15641E+00_JPRB, 0.15475E+00_JPRB, 0.15298E+00_JPRB, 0.15051E+00_JPRB, 0.14801E+00_JPRB, 0.14612E+00_JPRB, 0.14467E+00_JPRB ,&
&  0.14324E+00_JPRB, 0.14368E+00_JPRB, 0.14493E+00_JPRB, 0.14769E+00_JPRB, 0.15159E+00_JPRB, 0.15503E+00_JPRB, 0.15952E+00_JPRB ,&
&  0.16562E+00_JPRB, 0.16745E+00_JPRB, 0.16926E+00_JPRB, 0.17016E+00_JPRB, 0.16988E+00_JPRB, 0.17028E+00_JPRB, 0.16780E+00_JPRB ,&
&  0.16804E+00_JPRB, 0.16628E+00_JPRB, 0.17075E+00_JPRB, 0.16844E+00_JPRB, 0.16236E+00_JPRB, 0.15930E+00_JPRB, 0.15348E+00_JPRB ,&
&  0.14890E+00_JPRB, 0.14418E+00_JPRB, 0.14092E+00_JPRB, 0.13904E+00_JPRB, 0.13644E+00_JPRB, 0.13499E+00_JPRB, 0.13283E+00_JPRB ,&
&  0.13086E+00_JPRB, 0.13012E+00_JPRB, 0.12882E+00_JPRB, 0.12822E+00_JPRB, 0.12749E+00_JPRB, 0.12652E+00_JPRB, 0.12547E+00_JPRB ,&
&  0.12475E+00_JPRB, 0.12409E+00_JPRB, 0.12372E+00_JPRB, 0.12323E+00_JPRB, 0.12255E+00_JPRB, 0.12188E+00_JPRB, 0.12114E+00_JPRB ,&
&  0.12062E+00_JPRB, 0.12029E+00_JPRB, 0.12001E+00_JPRB, 0.11985E+00_JPRB, 0.11969E+00_JPRB, 0.11941E+00_JPRB, 0.11944E+00_JPRB ,&
&  0.11965E+00_JPRB, 0.12019E+00_JPRB, 0.12068E+00_JPRB, 0.12133E+00_JPRB, 0.12195E+00_JPRB, 0.12277E+00_JPRB, 0.12343E+00_JPRB ,&
&  0.12412E+00_JPRB, 0.12500E+00_JPRB, 0.12576E+00_JPRB, 0.12666E+00_JPRB, 0.12766E+00_JPRB, 0.12853E+00_JPRB, 0.12984E+00_JPRB ,&
&  0.13118E+00_JPRB, 0.13251E+00_JPRB, 0.13431E+00_JPRB, 0.13623E+00_JPRB, 0.13866E+00_JPRB, 0.14288E+00_JPRB, 0.14779E+00_JPRB ,&
&  0.15390E+00_JPRB, 0.16321E+00_JPRB, 0.16360E+00_JPRB, 0.16578E+00_JPRB, 0.16716E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.17318E+00_JPRB, 0.17508E+00_JPRB, 0.17676E+00_JPRB, 0.17829E+00_JPRB, 0.17860E+00_JPRB, 0.17894E+00_JPRB, 0.17970E+00_JPRB ,&
&  0.17610E+00_JPRB, 0.17339E+00_JPRB, 0.16893E+00_JPRB, 0.16458E+00_JPRB, 0.16047E+00_JPRB, 0.15693E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15374E+00_JPRB, 0.15377E+00_JPRB, 0.15420E+00_JPRB, 0.15498E+00_JPRB, 0.15459E+00_JPRB, 0.15381E+00_JPRB, 0.15298E+00_JPRB ,&
&  0.15180E+00_JPRB, 0.15180E+00_JPRB, 0.15212E+00_JPRB, 0.15521E+00_JPRB, 0.15959E+00_JPRB, 0.16462E+00_JPRB, 0.16817E+00_JPRB ,&
&  0.17073E+00_JPRB, 0.17264E+00_JPRB, 0.17487E+00_JPRB, 0.17581E+00_JPRB, 0.17658E+00_JPRB, 0.17775E+00_JPRB, 0.17837E+00_JPRB ,&
&  0.17971E+00_JPRB, 0.18000E+00_JPRB, 0.18006E+00_JPRB, 0.18060E+00_JPRB, 0.17276E+00_JPRB, 0.16675E+00_JPRB, 0.16070E+00_JPRB ,&
&  0.15479E+00_JPRB, 0.14885E+00_JPRB, 0.14353E+00_JPRB, 0.14048E+00_JPRB, 0.13756E+00_JPRB, 0.13585E+00_JPRB, 0.13435E+00_JPRB ,&
&  0.13325E+00_JPRB, 0.13211E+00_JPRB, 0.13050E+00_JPRB, 0.12893E+00_JPRB, 0.12803E+00_JPRB, 0.12726E+00_JPRB, 0.12631E+00_JPRB ,&
&  0.12622E+00_JPRB, 0.12563E+00_JPRB, 0.12552E+00_JPRB, 0.12512E+00_JPRB, 0.12483E+00_JPRB, 0.12458E+00_JPRB, 0.12458E+00_JPRB ,&
&  0.12491E+00_JPRB, 0.12555E+00_JPRB, 0.12574E+00_JPRB, 0.12596E+00_JPRB, 0.12577E+00_JPRB, 0.12518E+00_JPRB, 0.12465E+00_JPRB ,&
&  0.12413E+00_JPRB, 0.12370E+00_JPRB, 0.12369E+00_JPRB, 0.12361E+00_JPRB, 0.12408E+00_JPRB, 0.12472E+00_JPRB, 0.12538E+00_JPRB ,&
&  0.12625E+00_JPRB, 0.12694E+00_JPRB, 0.12763E+00_JPRB, 0.12845E+00_JPRB, 0.12919E+00_JPRB, 0.13013E+00_JPRB, 0.13123E+00_JPRB ,&
&  0.13210E+00_JPRB, 0.13371E+00_JPRB, 0.13504E+00_JPRB, 0.13666E+00_JPRB, 0.13958E+00_JPRB, 0.14302E+00_JPRB, 0.15079E+00_JPRB ,&
&  0.15875E+00_JPRB, 0.17580E+00_JPRB, 0.17740E+00_JPRB, 0.17176E+00_JPRB, 0.17210E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.18117E+00_JPRB, 0.18347E+00_JPRB, 0.18593E+00_JPRB, 0.18481E+00_JPRB, 0.18491E+00_JPRB, 0.18523E+00_JPRB, 0.18645E+00_JPRB ,&
&  0.18707E+00_JPRB, 0.18677E+00_JPRB, 0.18645E+00_JPRB, 0.18523E+00_JPRB, 0.18376E+00_JPRB, 0.17676E+00_JPRB, 0.17364E+00_JPRB ,&
&  0.17126E+00_JPRB, 0.17161E+00_JPRB, 0.17209E+00_JPRB, 0.17404E+00_JPRB, 0.17441E+00_JPRB, 0.17376E+00_JPRB, 0.17101E+00_JPRB ,&
&  0.16862E+00_JPRB, 0.16695E+00_JPRB, 0.16530E+00_JPRB, 0.16531E+00_JPRB, 0.16713E+00_JPRB, 0.16920E+00_JPRB, 0.17035E+00_JPRB ,&
&  0.17058E+00_JPRB, 0.17226E+00_JPRB, 0.17367E+00_JPRB, 0.17622E+00_JPRB, 0.18338E+00_JPRB, 0.18501E+00_JPRB, 0.18565E+00_JPRB ,&
&  0.18650E+00_JPRB, 0.18786E+00_JPRB, 0.18877E+00_JPRB, 0.18881E+00_JPRB, 0.18734E+00_JPRB, 0.18315E+00_JPRB, 0.18206E+00_JPRB ,&
&  0.16942E+00_JPRB, 0.16196E+00_JPRB, 0.15476E+00_JPRB, 0.14936E+00_JPRB, 0.14466E+00_JPRB, 0.14207E+00_JPRB, 0.13918E+00_JPRB ,&
&  0.13676E+00_JPRB, 0.13415E+00_JPRB, 0.13217E+00_JPRB, 0.13123E+00_JPRB, 0.13096E+00_JPRB, 0.13073E+00_JPRB, 0.13021E+00_JPRB ,&
&  0.13028E+00_JPRB, 0.12989E+00_JPRB, 0.12958E+00_JPRB, 0.12927E+00_JPRB, 0.12907E+00_JPRB, 0.12914E+00_JPRB, 0.12980E+00_JPRB ,&
&  0.13154E+00_JPRB, 0.13325E+00_JPRB, 0.13492E+00_JPRB, 0.13631E+00_JPRB, 0.13579E+00_JPRB, 0.13424E+00_JPRB, 0.13188E+00_JPRB ,&
&  0.12989E+00_JPRB, 0.12862E+00_JPRB, 0.12725E+00_JPRB, 0.12691E+00_JPRB, 0.12745E+00_JPRB, 0.12814E+00_JPRB, 0.12931E+00_JPRB ,&
&  0.13034E+00_JPRB, 0.13117E+00_JPRB, 0.13173E+00_JPRB, 0.13213E+00_JPRB, 0.13277E+00_JPRB, 0.13341E+00_JPRB, 0.13446E+00_JPRB ,&
&  0.13562E+00_JPRB, 0.13706E+00_JPRB, 0.13840E+00_JPRB, 0.14018E+00_JPRB, 0.14276E+00_JPRB, 0.14678E+00_JPRB, 0.15364E+00_JPRB ,&
&  0.16081E+00_JPRB, 0.17966E+00_JPRB, 0.18183E+00_JPRB, 0.17962E+00_JPRB, 0.18071E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.18838E+00_JPRB, 0.19253E+00_JPRB, 0.19501E+00_JPRB, 0.19461E+00_JPRB, 0.19429E+00_JPRB, 0.19499E+00_JPRB, 0.19586E+00_JPRB ,&
&  0.19698E+00_JPRB, 0.19660E+00_JPRB, 0.19528E+00_JPRB, 0.19391E+00_JPRB, 0.19241E+00_JPRB, 0.19041E+00_JPRB, 0.18900E+00_JPRB ,&
&  0.18790E+00_JPRB, 0.18777E+00_JPRB, 0.18725E+00_JPRB, 0.18796E+00_JPRB, 0.18848E+00_JPRB, 0.19105E+00_JPRB, 0.18989E+00_JPRB ,&
&  0.18666E+00_JPRB, 0.18499E+00_JPRB, 0.18718E+00_JPRB, 0.18758E+00_JPRB, 0.18812E+00_JPRB, 0.18892E+00_JPRB, 0.19018E+00_JPRB ,&
&  0.18789E+00_JPRB, 0.18470E+00_JPRB, 0.17983E+00_JPRB, 0.18442E+00_JPRB, 0.19376E+00_JPRB, 0.19413E+00_JPRB, 0.19327E+00_JPRB ,&
&  0.19455E+00_JPRB, 0.19723E+00_JPRB, 0.19855E+00_JPRB, 0.19898E+00_JPRB, 0.19755E+00_JPRB, 0.19682E+00_JPRB, 0.19578E+00_JPRB ,&
&  0.19319E+00_JPRB, 0.19049E+00_JPRB, 0.18261E+00_JPRB, 0.16743E+00_JPRB, 0.16075E+00_JPRB, 0.15744E+00_JPRB, 0.15348E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14607E+00_JPRB, 0.14570E+00_JPRB, 0.14444E+00_JPRB, 0.14471E+00_JPRB, 0.14424E+00_JPRB, 0.14342E+00_JPRB ,&
&  0.14232E+00_JPRB, 0.14151E+00_JPRB, 0.14005E+00_JPRB, 0.13776E+00_JPRB, 0.13608E+00_JPRB, 0.13564E+00_JPRB, 0.13629E+00_JPRB ,&
&  0.13805E+00_JPRB, 0.14094E+00_JPRB, 0.14889E+00_JPRB, 0.15354E+00_JPRB, 0.15156E+00_JPRB, 0.14731E+00_JPRB, 0.14149E+00_JPRB ,&
&  0.13663E+00_JPRB, 0.13508E+00_JPRB, 0.13401E+00_JPRB, 0.13313E+00_JPRB, 0.13343E+00_JPRB, 0.13442E+00_JPRB, 0.13542E+00_JPRB ,&
&  0.13651E+00_JPRB, 0.13772E+00_JPRB, 0.13844E+00_JPRB, 0.13882E+00_JPRB, 0.13921E+00_JPRB, 0.13944E+00_JPRB, 0.13990E+00_JPRB ,&
&  0.14011E+00_JPRB, 0.14139E+00_JPRB, 0.14258E+00_JPRB, 0.14480E+00_JPRB, 0.14633E+00_JPRB, 0.14819E+00_JPRB, 0.15192E+00_JPRB ,&
&  0.15537E+00_JPRB, 0.16531E+00_JPRB, 0.17961E+00_JPRB, 0.18802E+00_JPRB, 0.18546E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.19863E+00_JPRB, 0.19976E+00_JPRB, 0.19993E+00_JPRB, 0.19910E+00_JPRB, 0.19794E+00_JPRB, 0.19823E+00_JPRB, 0.19968E+00_JPRB ,&
&  0.20256E+00_JPRB, 0.20475E+00_JPRB, 0.20536E+00_JPRB, 0.20550E+00_JPRB, 0.20461E+00_JPRB, 0.20253E+00_JPRB, 0.20073E+00_JPRB ,&
&  0.19928E+00_JPRB, 0.19838E+00_JPRB, 0.19767E+00_JPRB, 0.19739E+00_JPRB, 0.19739E+00_JPRB, 0.19856E+00_JPRB, 0.19982E+00_JPRB ,&
&  0.20370E+00_JPRB, 0.20394E+00_JPRB, 0.20332E+00_JPRB, 0.20309E+00_JPRB, 0.20282E+00_JPRB, 0.20273E+00_JPRB, 0.20265E+00_JPRB ,&
&  0.20042E+00_JPRB, 0.19260E+00_JPRB, 0.18537E+00_JPRB, 0.19031E+00_JPRB, 0.20303E+00_JPRB, 0.20343E+00_JPRB, 0.20336E+00_JPRB ,&
&  0.20467E+00_JPRB, 0.20757E+00_JPRB, 0.20951E+00_JPRB, 0.21054E+00_JPRB, 0.20937E+00_JPRB, 0.20755E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20713E+00_JPRB, 0.20529E+00_JPRB, 0.20035E+00_JPRB, 0.19484E+00_JPRB, 0.18927E+00_JPRB, 0.18705E+00_JPRB, 0.18529E+00_JPRB ,&
&  0.18293E+00_JPRB, 0.18158E+00_JPRB, 0.18105E+00_JPRB, 0.18054E+00_JPRB, 0.18094E+00_JPRB, 0.17915E+00_JPRB, 0.17687E+00_JPRB ,&
&  0.17297E+00_JPRB, 0.16912E+00_JPRB, 0.16424E+00_JPRB, 0.15905E+00_JPRB, 0.15449E+00_JPRB, 0.14982E+00_JPRB, 0.14660E+00_JPRB ,&
&  0.14588E+00_JPRB, 0.14931E+00_JPRB, 0.15831E+00_JPRB, 0.16849E+00_JPRB, 0.17744E+00_JPRB, 0.16552E+00_JPRB, 0.15824E+00_JPRB ,&
&  0.15110E+00_JPRB, 0.14570E+00_JPRB, 0.14269E+00_JPRB, 0.14195E+00_JPRB, 0.14210E+00_JPRB, 0.14291E+00_JPRB, 0.14434E+00_JPRB ,&
&  0.14611E+00_JPRB, 0.14841E+00_JPRB, 0.15021E+00_JPRB, 0.15140E+00_JPRB, 0.15219E+00_JPRB, 0.15241E+00_JPRB, 0.15223E+00_JPRB ,&
&  0.15199E+00_JPRB, 0.15310E+00_JPRB, 0.15548E+00_JPRB, 0.15953E+00_JPRB, 0.16261E+00_JPRB, 0.16647E+00_JPRB, 0.17196E+00_JPRB ,&
&  0.17976E+00_JPRB, 0.18912E+00_JPRB, 0.19187E+00_JPRB, 0.19363E+00_JPRB, 0.19635E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.20828E+00_JPRB, 0.20953E+00_JPRB, 0.20967E+00_JPRB, 0.20998E+00_JPRB, 0.21050E+00_JPRB, 0.21231E+00_JPRB, 0.21638E+00_JPRB ,&
&  0.22035E+00_JPRB, 0.22220E+00_JPRB, 0.22082E+00_JPRB, 0.21872E+00_JPRB, 0.21764E+00_JPRB, 0.21701E+00_JPRB, 0.21673E+00_JPRB ,&
&  0.21649E+00_JPRB, 0.21546E+00_JPRB, 0.21264E+00_JPRB, 0.20883E+00_JPRB, 0.20615E+00_JPRB, 0.20568E+00_JPRB, 0.20632E+00_JPRB ,&
&  0.21025E+00_JPRB, 0.21302E+00_JPRB, 0.21563E+00_JPRB, 0.21773E+00_JPRB, 0.21812E+00_JPRB, 0.21767E+00_JPRB, 0.21896E+00_JPRB ,&
&  0.22004E+00_JPRB, 0.21804E+00_JPRB, 0.21281E+00_JPRB, 0.21715E+00_JPRB, 0.21910E+00_JPRB, 0.22030E+00_JPRB, 0.21959E+00_JPRB ,&
&  0.22029E+00_JPRB, 0.22239E+00_JPRB, 0.22429E+00_JPRB, 0.22582E+00_JPRB, 0.22510E+00_JPRB, 0.22188E+00_JPRB, 0.22068E+00_JPRB ,&
&  0.22069E+00_JPRB, 0.21954E+00_JPRB, 0.21696E+00_JPRB, 0.21464E+00_JPRB, 0.21207E+00_JPRB, 0.21097E+00_JPRB, 0.21075E+00_JPRB ,&
&  0.20980E+00_JPRB, 0.20762E+00_JPRB, 0.20328E+00_JPRB, 0.19908E+00_JPRB, 0.19610E+00_JPRB, 0.19328E+00_JPRB, 0.19236E+00_JPRB ,&
&  0.19298E+00_JPRB, 0.19429E+00_JPRB, 0.19446E+00_JPRB, 0.19331E+00_JPRB, 0.18926E+00_JPRB, 0.17870E+00_JPRB, 0.17165E+00_JPRB ,&
&  0.17041E+00_JPRB, 0.17514E+00_JPRB, 0.18356E+00_JPRB, 0.18433E+00_JPRB, 0.18616E+00_JPRB, 0.18756E+00_JPRB, 0.18899E+00_JPRB ,&
&  0.18816E+00_JPRB, 0.17471E+00_JPRB, 0.16875E+00_JPRB, 0.16556E+00_JPRB, 0.16242E+00_JPRB, 0.16176E+00_JPRB, 0.16312E+00_JPRB ,&
&  0.16594E+00_JPRB, 0.16995E+00_JPRB, 0.17257E+00_JPRB, 0.17397E+00_JPRB, 0.17428E+00_JPRB, 0.17372E+00_JPRB, 0.17297E+00_JPRB ,&
&  0.17296E+00_JPRB, 0.17657E+00_JPRB, 0.18412E+00_JPRB, 0.19297E+00_JPRB, 0.19393E+00_JPRB, 0.19556E+00_JPRB, 0.19625E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.19861E+00_JPRB, 0.19971E+00_JPRB, 0.20238E+00_JPRB, 0.20566E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.21876E+00_JPRB, 0.22178E+00_JPRB, 0.22479E+00_JPRB, 0.22659E+00_JPRB, 0.22844E+00_JPRB, 0.23028E+00_JPRB, 0.23352E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23845E+00_JPRB, 0.23679E+00_JPRB, 0.23411E+00_JPRB, 0.23236E+00_JPRB, 0.23155E+00_JPRB, 0.23126E+00_JPRB ,&
&  0.23105E+00_JPRB, 0.23089E+00_JPRB, 0.23017E+00_JPRB, 0.22850E+00_JPRB, 0.22614E+00_JPRB, 0.22384E+00_JPRB, 0.22220E+00_JPRB ,&
&  0.22218E+00_JPRB, 0.22382E+00_JPRB, 0.22635E+00_JPRB, 0.22684E+00_JPRB, 0.22714E+00_JPRB, 0.22840E+00_JPRB, 0.23034E+00_JPRB ,&
&  0.23417E+00_JPRB, 0.23618E+00_JPRB, 0.23843E+00_JPRB, 0.23428E+00_JPRB, 0.23898E+00_JPRB, 0.24063E+00_JPRB, 0.24120E+00_JPRB ,&
&  0.24251E+00_JPRB, 0.24351E+00_JPRB, 0.24439E+00_JPRB, 0.24365E+00_JPRB, 0.23926E+00_JPRB, 0.23711E+00_JPRB, 0.23733E+00_JPRB ,&
&  0.23760E+00_JPRB, 0.23598E+00_JPRB, 0.23318E+00_JPRB, 0.23056E+00_JPRB, 0.22846E+00_JPRB, 0.22700E+00_JPRB, 0.22581E+00_JPRB ,&
&  0.22518E+00_JPRB, 0.22448E+00_JPRB, 0.22270E+00_JPRB, 0.21773E+00_JPRB, 0.21198E+00_JPRB, 0.20826E+00_JPRB, 0.20577E+00_JPRB ,&
&  0.20598E+00_JPRB, 0.20797E+00_JPRB, 0.20966E+00_JPRB, 0.20990E+00_JPRB, 0.20887E+00_JPRB, 0.20698E+00_JPRB, 0.20395E+00_JPRB ,&
&  0.19920E+00_JPRB, 0.19703E+00_JPRB, 0.19620E+00_JPRB, 0.19521E+00_JPRB, 0.19561E+00_JPRB, 0.19731E+00_JPRB, 0.19893E+00_JPRB ,&
&  0.20085E+00_JPRB, 0.20088E+00_JPRB, 0.19952E+00_JPRB, 0.19723E+00_JPRB, 0.19480E+00_JPRB, 0.19461E+00_JPRB, 0.19509E+00_JPRB ,&
&  0.19546E+00_JPRB, 0.19572E+00_JPRB, 0.19634E+00_JPRB, 0.19729E+00_JPRB, 0.19864E+00_JPRB, 0.20077E+00_JPRB, 0.20259E+00_JPRB ,&
&  0.20256E+00_JPRB, 0.20148E+00_JPRB, 0.20246E+00_JPRB, 0.20397E+00_JPRB, 0.20471E+00_JPRB, 0.20523E+00_JPRB, 0.20557E+00_JPRB ,&
&  0.20527E+00_JPRB, 0.20542E+00_JPRB, 0.20730E+00_JPRB, 0.21181E+00_JPRB, 0.21597E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.23055E+00_JPRB, 0.23427E+00_JPRB, 0.23937E+00_JPRB, 0.24081E+00_JPRB, 0.24358E+00_JPRB, 0.24633E+00_JPRB, 0.24979E+00_JPRB ,&
&  0.25192E+00_JPRB, 0.25197E+00_JPRB, 0.25042E+00_JPRB, 0.24888E+00_JPRB, 0.24801E+00_JPRB, 0.24755E+00_JPRB, 0.24689E+00_JPRB ,&
&  0.24661E+00_JPRB, 0.24672E+00_JPRB, 0.24671E+00_JPRB, 0.24664E+00_JPRB, 0.24568E+00_JPRB, 0.24418E+00_JPRB, 0.24252E+00_JPRB ,&
&  0.23959E+00_JPRB, 0.23775E+00_JPRB, 0.23738E+00_JPRB, 0.23748E+00_JPRB, 0.23712E+00_JPRB, 0.23787E+00_JPRB, 0.23973E+00_JPRB ,&
&  0.24355E+00_JPRB, 0.24662E+00_JPRB, 0.25051E+00_JPRB, 0.25455E+00_JPRB, 0.25701E+00_JPRB, 0.25877E+00_JPRB, 0.26083E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.26241E+00_JPRB, 0.26233E+00_JPRB, 0.26215E+00_JPRB, 0.25911E+00_JPRB, 0.25766E+00_JPRB, 0.25653E+00_JPRB ,&
&  0.25496E+00_JPRB, 0.25231E+00_JPRB, 0.24873E+00_JPRB, 0.24561E+00_JPRB, 0.24251E+00_JPRB, 0.24032E+00_JPRB, 0.23854E+00_JPRB ,&
&  0.23727E+00_JPRB, 0.23679E+00_JPRB, 0.23700E+00_JPRB, 0.23582E+00_JPRB, 0.23103E+00_JPRB, 0.22623E+00_JPRB, 0.22353E+00_JPRB ,&
&  0.22230E+00_JPRB, 0.22212E+00_JPRB, 0.22356E+00_JPRB, 0.22257E+00_JPRB, 0.21927E+00_JPRB, 0.21672E+00_JPRB, 0.21566E+00_JPRB ,&
&  0.21413E+00_JPRB, 0.21332E+00_JPRB, 0.21385E+00_JPRB, 0.21441E+00_JPRB, 0.21505E+00_JPRB, 0.21536E+00_JPRB, 0.21638E+00_JPRB ,&
&  0.21716E+00_JPRB, 0.21839E+00_JPRB, 0.21810E+00_JPRB, 0.21544E+00_JPRB, 0.21181E+00_JPRB, 0.20914E+00_JPRB, 0.20850E+00_JPRB ,&
&  0.20977E+00_JPRB, 0.21103E+00_JPRB, 0.21127E+00_JPRB, 0.21129E+00_JPRB, 0.21227E+00_JPRB, 0.21466E+00_JPRB, 0.21777E+00_JPRB ,&
&  0.21905E+00_JPRB, 0.22030E+00_JPRB, 0.22208E+00_JPRB, 0.22389E+00_JPRB, 0.22506E+00_JPRB, 0.22570E+00_JPRB, 0.22583E+00_JPRB ,&
&  0.22514E+00_JPRB, 0.22580E+00_JPRB, 0.22724E+00_JPRB, 0.22900E+00_JPRB, 0.23028E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.24920E+00_JPRB, 0.25178E+00_JPRB, 0.25515E+00_JPRB, 0.25603E+00_JPRB, 0.25663E+00_JPRB, 0.25827E+00_JPRB, 0.26134E+00_JPRB ,&
&  0.26341E+00_JPRB, 0.26395E+00_JPRB, 0.26376E+00_JPRB, 0.26341E+00_JPRB, 0.26355E+00_JPRB, 0.26484E+00_JPRB, 0.26535E+00_JPRB ,&
&  0.26462E+00_JPRB, 0.26401E+00_JPRB, 0.26324E+00_JPRB, 0.26276E+00_JPRB, 0.26154E+00_JPRB, 0.25967E+00_JPRB, 0.25821E+00_JPRB ,&
&  0.25609E+00_JPRB, 0.25422E+00_JPRB, 0.25296E+00_JPRB, 0.25209E+00_JPRB, 0.25183E+00_JPRB, 0.25224E+00_JPRB, 0.25406E+00_JPRB ,&
&  0.25623E+00_JPRB, 0.25891E+00_JPRB, 0.26205E+00_JPRB, 0.26562E+00_JPRB, 0.26923E+00_JPRB, 0.27331E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.27969E+00_JPRB, 0.28193E+00_JPRB, 0.28323E+00_JPRB, 0.28014E+00_JPRB, 0.27623E+00_JPRB, 0.27460E+00_JPRB, 0.27341E+00_JPRB ,&
&  0.27070E+00_JPRB, 0.26729E+00_JPRB, 0.26271E+00_JPRB, 0.26005E+00_JPRB, 0.25921E+00_JPRB, 0.25812E+00_JPRB, 0.25680E+00_JPRB ,&
&  0.25596E+00_JPRB, 0.25547E+00_JPRB, 0.25456E+00_JPRB, 0.25285E+00_JPRB, 0.24984E+00_JPRB, 0.24554E+00_JPRB, 0.24114E+00_JPRB ,&
&  0.23835E+00_JPRB, 0.23646E+00_JPRB, 0.23670E+00_JPRB, 0.23500E+00_JPRB, 0.23131E+00_JPRB, 0.22796E+00_JPRB, 0.22764E+00_JPRB ,&
&  0.22768E+00_JPRB, 0.22802E+00_JPRB, 0.22776E+00_JPRB, 0.22666E+00_JPRB, 0.22698E+00_JPRB, 0.22801E+00_JPRB, 0.22948E+00_JPRB ,&
&  0.23311E+00_JPRB, 0.23562E+00_JPRB, 0.23531E+00_JPRB, 0.23412E+00_JPRB, 0.23217E+00_JPRB, 0.23167E+00_JPRB, 0.23208E+00_JPRB ,&
&  0.23246E+00_JPRB, 0.23251E+00_JPRB, 0.23260E+00_JPRB, 0.23201E+00_JPRB, 0.23212E+00_JPRB, 0.23359E+00_JPRB, 0.23541E+00_JPRB ,&
&  0.23758E+00_JPRB, 0.24028E+00_JPRB, 0.24264E+00_JPRB, 0.24511E+00_JPRB, 0.24739E+00_JPRB, 0.24854E+00_JPRB, 0.24953E+00_JPRB ,&
&  0.25048E+00_JPRB, 0.25058E+00_JPRB, 0.25018E+00_JPRB, 0.25001E+00_JPRB, 0.24988E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.27429E+00_JPRB, 0.27459E+00_JPRB, 0.27570E+00_JPRB, 0.27629E+00_JPRB, 0.27626E+00_JPRB, 0.27684E+00_JPRB, 0.27934E+00_JPRB ,&
&  0.28151E+00_JPRB, 0.28270E+00_JPRB, 0.28352E+00_JPRB, 0.28355E+00_JPRB, 0.28322E+00_JPRB, 0.28438E+00_JPRB, 0.28532E+00_JPRB ,&
&  0.28533E+00_JPRB, 0.28423E+00_JPRB, 0.28308E+00_JPRB, 0.28172E+00_JPRB, 0.27991E+00_JPRB, 0.27808E+00_JPRB, 0.27675E+00_JPRB ,&
&  0.27490E+00_JPRB, 0.27285E+00_JPRB, 0.27088E+00_JPRB, 0.27049E+00_JPRB, 0.27100E+00_JPRB, 0.27164E+00_JPRB, 0.27291E+00_JPRB ,&
&  0.27414E+00_JPRB, 0.27692E+00_JPRB, 0.28006E+00_JPRB, 0.28358E+00_JPRB, 0.28708E+00_JPRB, 0.29170E+00_JPRB, 0.29677E+00_JPRB ,&
&  0.30149E+00_JPRB, 0.30383E+00_JPRB, 0.30526E+00_JPRB, 0.30298E+00_JPRB, 0.30176E+00_JPRB, 0.30047E+00_JPRB, 0.29954E+00_JPRB ,&
&  0.29833E+00_JPRB, 0.29374E+00_JPRB, 0.28991E+00_JPRB, 0.28935E+00_JPRB, 0.28983E+00_JPRB, 0.29006E+00_JPRB, 0.28990E+00_JPRB ,&
&  0.28887E+00_JPRB, 0.28745E+00_JPRB, 0.28489E+00_JPRB, 0.28132E+00_JPRB, 0.27754E+00_JPRB, 0.27281E+00_JPRB, 0.26583E+00_JPRB ,&
&  0.25921E+00_JPRB, 0.25384E+00_JPRB, 0.24986E+00_JPRB, 0.24646E+00_JPRB, 0.24420E+00_JPRB, 0.24168E+00_JPRB, 0.24098E+00_JPRB ,&
&  0.24132E+00_JPRB, 0.24313E+00_JPRB, 0.24420E+00_JPRB, 0.24358E+00_JPRB, 0.24250E+00_JPRB, 0.24269E+00_JPRB, 0.24466E+00_JPRB ,&
&  0.24828E+00_JPRB, 0.25106E+00_JPRB, 0.25184E+00_JPRB, 0.25126E+00_JPRB, 0.25032E+00_JPRB, 0.25092E+00_JPRB, 0.25246E+00_JPRB ,&
&  0.25315E+00_JPRB, 0.25358E+00_JPRB, 0.25471E+00_JPRB, 0.25465E+00_JPRB, 0.25658E+00_JPRB, 0.25980E+00_JPRB, 0.26334E+00_JPRB ,&
&  0.26673E+00_JPRB, 0.27095E+00_JPRB, 0.27523E+00_JPRB, 0.27870E+00_JPRB, 0.28176E+00_JPRB, 0.28201E+00_JPRB, 0.28243E+00_JPRB ,&
&  0.28253E+00_JPRB, 0.28136E+00_JPRB, 0.28010E+00_JPRB, 0.27822E+00_JPRB, 0.27582E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.30307E+00_JPRB, 0.30213E+00_JPRB, 0.30135E+00_JPRB, 0.30115E+00_JPRB, 0.30083E+00_JPRB, 0.30036E+00_JPRB, 0.30023E+00_JPRB ,&
&  0.30069E+00_JPRB, 0.30061E+00_JPRB, 0.30275E+00_JPRB, 0.30320E+00_JPRB, 0.30286E+00_JPRB, 0.30343E+00_JPRB, 0.30386E+00_JPRB ,&
&  0.30249E+00_JPRB, 0.30143E+00_JPRB, 0.30041E+00_JPRB, 0.29847E+00_JPRB, 0.29524E+00_JPRB, 0.29381E+00_JPRB, 0.29275E+00_JPRB ,&
&  0.29073E+00_JPRB, 0.28821E+00_JPRB, 0.28612E+00_JPRB, 0.28545E+00_JPRB, 0.28633E+00_JPRB, 0.28671E+00_JPRB, 0.28797E+00_JPRB ,&
&  0.28879E+00_JPRB, 0.29215E+00_JPRB, 0.28082E+00_JPRB, 0.29030E+00_JPRB, 0.29941E+00_JPRB, 0.30832E+00_JPRB, 0.31770E+00_JPRB ,&
&  0.32418E+00_JPRB, 0.32995E+00_JPRB, 0.33577E+00_JPRB, 0.33649E+00_JPRB, 0.33458E+00_JPRB, 0.33106E+00_JPRB, 0.32947E+00_JPRB ,&
&  0.32858E+00_JPRB, 0.32621E+00_JPRB, 0.32438E+00_JPRB, 0.32400E+00_JPRB, 0.32401E+00_JPRB, 0.32263E+00_JPRB, 0.32054E+00_JPRB ,&
&  0.31878E+00_JPRB, 0.31675E+00_JPRB, 0.31336E+00_JPRB, 0.30948E+00_JPRB, 0.30517E+00_JPRB, 0.29956E+00_JPRB, 0.29171E+00_JPRB ,&
&  0.28258E+00_JPRB, 0.27532E+00_JPRB, 0.26923E+00_JPRB, 0.26540E+00_JPRB, 0.26366E+00_JPRB, 0.26176E+00_JPRB, 0.25995E+00_JPRB ,&
&  0.25955E+00_JPRB, 0.26012E+00_JPRB, 0.26052E+00_JPRB, 0.26056E+00_JPRB, 0.26143E+00_JPRB, 0.26205E+00_JPRB, 0.26431E+00_JPRB ,&
&  0.26673E+00_JPRB, 0.26738E+00_JPRB, 0.26711E+00_JPRB, 0.26705E+00_JPRB, 0.26571E+00_JPRB, 0.26617E+00_JPRB, 0.26777E+00_JPRB ,&
&  0.26963E+00_JPRB, 0.27222E+00_JPRB, 0.27417E+00_JPRB, 0.27589E+00_JPRB, 0.27813E+00_JPRB, 0.28156E+00_JPRB, 0.28677E+00_JPRB ,&
&  0.29171E+00_JPRB, 0.29725E+00_JPRB, 0.30358E+00_JPRB, 0.30910E+00_JPRB, 0.31274E+00_JPRB, 0.31437E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.31505E+00_JPRB, 0.31185E+00_JPRB, 0.30844E+00_JPRB, 0.30620E+00_JPRB, 0.30441E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.34228E+00_JPRB, 0.33970E+00_JPRB, 0.33622E+00_JPRB, 0.33359E+00_JPRB, 0.33086E+00_JPRB, 0.32757E+00_JPRB, 0.32571E+00_JPRB ,&
&  0.32483E+00_JPRB, 0.32464E+00_JPRB, 0.32526E+00_JPRB, 0.32508E+00_JPRB, 0.32475E+00_JPRB, 0.32518E+00_JPRB, 0.32469E+00_JPRB ,&
&  0.32376E+00_JPRB, 0.32246E+00_JPRB, 0.32085E+00_JPRB, 0.31853E+00_JPRB, 0.31557E+00_JPRB, 0.31289E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.30690E+00_JPRB, 0.30584E+00_JPRB, 0.30549E+00_JPRB, 0.30581E+00_JPRB, 0.30747E+00_JPRB, 0.30964E+00_JPRB, 0.31184E+00_JPRB ,&
&  0.31260E+00_JPRB, 0.30252E+00_JPRB, 0.30767E+00_JPRB, 0.31094E+00_JPRB, 0.31946E+00_JPRB, 0.33376E+00_JPRB, 0.34667E+00_JPRB ,&
&  0.35452E+00_JPRB, 0.36201E+00_JPRB, 0.36766E+00_JPRB, 0.37019E+00_JPRB, 0.36792E+00_JPRB, 0.36549E+00_JPRB, 0.36596E+00_JPRB ,&
&  0.36637E+00_JPRB, 0.36533E+00_JPRB, 0.36387E+00_JPRB, 0.36183E+00_JPRB, 0.35888E+00_JPRB, 0.35593E+00_JPRB, 0.35140E+00_JPRB ,&
&  0.34697E+00_JPRB, 0.34400E+00_JPRB, 0.34106E+00_JPRB, 0.33802E+00_JPRB, 0.33316E+00_JPRB, 0.32720E+00_JPRB, 0.31985E+00_JPRB ,&
&  0.31179E+00_JPRB, 0.30599E+00_JPRB, 0.30222E+00_JPRB, 0.29847E+00_JPRB, 0.29470E+00_JPRB, 0.29208E+00_JPRB, 0.28913E+00_JPRB ,&
&  0.28811E+00_JPRB, 0.28892E+00_JPRB, 0.28849E+00_JPRB, 0.28828E+00_JPRB, 0.28885E+00_JPRB, 0.28844E+00_JPRB, 0.28840E+00_JPRB ,&
&  0.28689E+00_JPRB, 0.28520E+00_JPRB, 0.28413E+00_JPRB, 0.28352E+00_JPRB, 0.28308E+00_JPRB, 0.28446E+00_JPRB, 0.28687E+00_JPRB ,&
&  0.28949E+00_JPRB, 0.29327E+00_JPRB, 0.29735E+00_JPRB, 0.30039E+00_JPRB, 0.30304E+00_JPRB, 0.30738E+00_JPRB, 0.31295E+00_JPRB ,&
&  0.31941E+00_JPRB, 0.32688E+00_JPRB, 0.33413E+00_JPRB, 0.34084E+00_JPRB, 0.34455E+00_JPRB, 0.34645E+00_JPRB, 0.34808E+00_JPRB ,&
&  0.34869E+00_JPRB, 0.34563E+00_JPRB, 0.34510E+00_JPRB, 0.34519E+00_JPRB, 0.34405E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.36980E+00_JPRB, 0.36920E+00_JPRB, 0.36679E+00_JPRB, 0.36349E+00_JPRB, 0.35954E+00_JPRB, 0.35738E+00_JPRB, 0.35625E+00_JPRB ,&
&  0.35702E+00_JPRB, 0.35606E+00_JPRB, 0.35645E+00_JPRB, 0.35412E+00_JPRB, 0.35170E+00_JPRB, 0.34994E+00_JPRB, 0.34789E+00_JPRB ,&
&  0.34542E+00_JPRB, 0.34232E+00_JPRB, 0.33934E+00_JPRB, 0.33680E+00_JPRB, 0.33370E+00_JPRB, 0.32948E+00_JPRB, 0.32563E+00_JPRB ,&
&  0.32293E+00_JPRB, 0.32200E+00_JPRB, 0.32171E+00_JPRB, 0.32172E+00_JPRB, 0.32189E+00_JPRB, 0.32362E+00_JPRB, 0.32524E+00_JPRB ,&
&  0.32870E+00_JPRB, 0.30038E+00_JPRB, 0.31045E+00_JPRB, 0.31478E+00_JPRB, 0.32274E+00_JPRB, 0.33952E+00_JPRB, 0.35903E+00_JPRB ,&
&  0.37746E+00_JPRB, 0.39200E+00_JPRB, 0.40069E+00_JPRB, 0.40102E+00_JPRB, 0.39554E+00_JPRB, 0.39159E+00_JPRB, 0.39134E+00_JPRB ,&
&  0.38984E+00_JPRB, 0.38648E+00_JPRB, 0.38342E+00_JPRB, 0.38008E+00_JPRB, 0.37534E+00_JPRB, 0.36819E+00_JPRB, 0.36071E+00_JPRB ,&
&  0.35659E+00_JPRB, 0.35447E+00_JPRB, 0.35275E+00_JPRB, 0.35082E+00_JPRB, 0.34691E+00_JPRB, 0.34335E+00_JPRB, 0.34031E+00_JPRB ,&
&  0.33739E+00_JPRB, 0.33262E+00_JPRB, 0.32963E+00_JPRB, 0.32755E+00_JPRB, 0.32445E+00_JPRB, 0.32239E+00_JPRB, 0.32089E+00_JPRB ,&
&  0.31932E+00_JPRB, 0.31730E+00_JPRB, 0.31586E+00_JPRB, 0.31499E+00_JPRB, 0.31242E+00_JPRB, 0.30977E+00_JPRB, 0.30680E+00_JPRB ,&
&  0.30529E+00_JPRB, 0.30431E+00_JPRB, 0.30320E+00_JPRB, 0.30308E+00_JPRB, 0.30388E+00_JPRB, 0.30611E+00_JPRB, 0.30921E+00_JPRB ,&
&  0.31297E+00_JPRB, 0.31733E+00_JPRB, 0.32125E+00_JPRB, 0.32431E+00_JPRB, 0.32847E+00_JPRB, 0.33196E+00_JPRB, 0.33670E+00_JPRB ,&
&  0.34266E+00_JPRB, 0.34843E+00_JPRB, 0.35407E+00_JPRB, 0.36076E+00_JPRB, 0.36698E+00_JPRB, 0.37067E+00_JPRB, 0.37401E+00_JPRB ,&
&  0.37687E+00_JPRB, 0.37418E+00_JPRB, 0.37269E+00_JPRB, 0.37039E+00_JPRB, 0.36885E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.40461E+00_JPRB, 0.40330E+00_JPRB, 0.40081E+00_JPRB, 0.39675E+00_JPRB, 0.39282E+00_JPRB, 0.38975E+00_JPRB, 0.38808E+00_JPRB ,&
&  0.38826E+00_JPRB, 0.38901E+00_JPRB, 0.38786E+00_JPRB, 0.38363E+00_JPRB, 0.37966E+00_JPRB, 0.37684E+00_JPRB, 0.37520E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.36890E+00_JPRB, 0.36365E+00_JPRB, 0.35856E+00_JPRB, 0.35288E+00_JPRB, 0.34824E+00_JPRB, 0.34453E+00_JPRB ,&
&  0.34164E+00_JPRB, 0.34067E+00_JPRB, 0.34023E+00_JPRB, 0.34025E+00_JPRB, 0.34157E+00_JPRB, 0.34219E+00_JPRB, 0.34802E+00_JPRB ,&
&  0.32451E+00_JPRB, 0.29456E+00_JPRB, 0.30687E+00_JPRB, 0.31835E+00_JPRB, 0.32897E+00_JPRB, 0.35226E+00_JPRB, 0.38180E+00_JPRB ,&
&  0.41233E+00_JPRB, 0.43808E+00_JPRB, 0.45222E+00_JPRB, 0.44966E+00_JPRB, 0.43935E+00_JPRB, 0.43143E+00_JPRB, 0.42741E+00_JPRB ,&
&  0.42035E+00_JPRB, 0.41209E+00_JPRB, 0.40539E+00_JPRB, 0.39741E+00_JPRB, 0.38884E+00_JPRB, 0.38049E+00_JPRB, 0.37367E+00_JPRB ,&
&  0.36817E+00_JPRB, 0.36500E+00_JPRB, 0.36463E+00_JPRB, 0.36376E+00_JPRB, 0.36316E+00_JPRB, 0.36424E+00_JPRB, 0.36417E+00_JPRB ,&
&  0.36376E+00_JPRB, 0.36216E+00_JPRB, 0.36123E+00_JPRB, 0.36023E+00_JPRB, 0.36008E+00_JPRB, 0.35863E+00_JPRB, 0.35639E+00_JPRB ,&
&  0.35379E+00_JPRB, 0.35167E+00_JPRB, 0.35030E+00_JPRB, 0.34727E+00_JPRB, 0.34051E+00_JPRB, 0.33418E+00_JPRB, 0.33061E+00_JPRB ,&
&  0.32950E+00_JPRB, 0.32929E+00_JPRB, 0.32904E+00_JPRB, 0.32985E+00_JPRB, 0.33138E+00_JPRB, 0.33358E+00_JPRB, 0.33696E+00_JPRB ,&
&  0.34167E+00_JPRB, 0.34669E+00_JPRB, 0.35058E+00_JPRB, 0.35392E+00_JPRB, 0.35914E+00_JPRB, 0.36416E+00_JPRB, 0.36927E+00_JPRB ,&
&  0.37446E+00_JPRB, 0.37985E+00_JPRB, 0.38592E+00_JPRB, 0.39479E+00_JPRB, 0.40532E+00_JPRB, 0.41337E+00_JPRB, 0.41604E+00_JPRB ,&
&  0.41409E+00_JPRB, 0.40650E+00_JPRB, 0.40678E+00_JPRB, 0.40390E+00_JPRB, 0.40320E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.42262E+00_JPRB, 0.42536E+00_JPRB, 0.42404E+00_JPRB, 0.42127E+00_JPRB, 0.41900E+00_JPRB, 0.41524E+00_JPRB, 0.41213E+00_JPRB ,&
&  0.40805E+00_JPRB, 0.40572E+00_JPRB, 0.40104E+00_JPRB, 0.39809E+00_JPRB, 0.39605E+00_JPRB, 0.39476E+00_JPRB, 0.39268E+00_JPRB ,&
&  0.38847E+00_JPRB, 0.38176E+00_JPRB, 0.37491E+00_JPRB, 0.36945E+00_JPRB, 0.36580E+00_JPRB, 0.36216E+00_JPRB, 0.35910E+00_JPRB ,&
&  0.35643E+00_JPRB, 0.35406E+00_JPRB, 0.35186E+00_JPRB, 0.35267E+00_JPRB, 0.35390E+00_JPRB, 0.35424E+00_JPRB, 0.35330E+00_JPRB ,&
&  0.36122E+00_JPRB, 0.31435E+00_JPRB, 0.32598E+00_JPRB, 0.33737E+00_JPRB, 0.34831E+00_JPRB, 0.37654E+00_JPRB, 0.41770E+00_JPRB ,&
&  0.45253E+00_JPRB, 0.48033E+00_JPRB, 0.49033E+00_JPRB, 0.48235E+00_JPRB, 0.47036E+00_JPRB, 0.46232E+00_JPRB, 0.45516E+00_JPRB ,&
&  0.44125E+00_JPRB, 0.42891E+00_JPRB, 0.41806E+00_JPRB, 0.40769E+00_JPRB, 0.39769E+00_JPRB, 0.38955E+00_JPRB, 0.38214E+00_JPRB ,&
&  0.37771E+00_JPRB, 0.37642E+00_JPRB, 0.37541E+00_JPRB, 0.37458E+00_JPRB, 0.37557E+00_JPRB, 0.37848E+00_JPRB, 0.37948E+00_JPRB ,&
&  0.37961E+00_JPRB, 0.38127E+00_JPRB, 0.38132E+00_JPRB, 0.38117E+00_JPRB, 0.38238E+00_JPRB, 0.38663E+00_JPRB, 0.38605E+00_JPRB ,&
&  0.38168E+00_JPRB, 0.37484E+00_JPRB, 0.36842E+00_JPRB, 0.36366E+00_JPRB, 0.35965E+00_JPRB, 0.35570E+00_JPRB, 0.35424E+00_JPRB ,&
&  0.35235E+00_JPRB, 0.35225E+00_JPRB, 0.35247E+00_JPRB, 0.35428E+00_JPRB, 0.35636E+00_JPRB, 0.35926E+00_JPRB, 0.36356E+00_JPRB ,&
&  0.36886E+00_JPRB, 0.37417E+00_JPRB, 0.38058E+00_JPRB, 0.38717E+00_JPRB, 0.39281E+00_JPRB, 0.39667E+00_JPRB, 0.39908E+00_JPRB ,&
&  0.40112E+00_JPRB, 0.40521E+00_JPRB, 0.40779E+00_JPRB, 0.42107E+00_JPRB, 0.43126E+00_JPRB, 0.44183E+00_JPRB, 0.44167E+00_JPRB ,&
&  0.43631E+00_JPRB, 0.43056E+00_JPRB, 0.37100E+00_JPRB, 0.40100E+00_JPRB, 0.41399E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.46343E+00_JPRB, 0.46357E+00_JPRB, 0.46049E+00_JPRB, 0.45808E+00_JPRB, 0.45212E+00_JPRB, 0.44559E+00_JPRB, 0.43964E+00_JPRB ,&
&  0.43196E+00_JPRB, 0.42431E+00_JPRB, 0.41739E+00_JPRB, 0.41498E+00_JPRB, 0.41223E+00_JPRB, 0.40882E+00_JPRB, 0.40216E+00_JPRB ,&
&  0.39358E+00_JPRB, 0.38523E+00_JPRB, 0.37775E+00_JPRB, 0.37337E+00_JPRB, 0.37265E+00_JPRB, 0.37130E+00_JPRB, 0.36888E+00_JPRB ,&
&  0.36783E+00_JPRB, 0.36937E+00_JPRB, 0.36938E+00_JPRB, 0.36925E+00_JPRB, 0.36875E+00_JPRB, 0.36442E+00_JPRB, 0.36493E+00_JPRB ,&
&  0.37565E+00_JPRB, 0.33888E+00_JPRB, 0.36239E+00_JPRB, 0.38775E+00_JPRB, 0.40486E+00_JPRB, 0.43560E+00_JPRB, 0.48122E+00_JPRB ,&
&  0.52509E+00_JPRB, 0.54111E+00_JPRB, 0.53797E+00_JPRB, 0.51731E+00_JPRB, 0.50132E+00_JPRB, 0.48913E+00_JPRB, 0.46967E+00_JPRB ,&
&  0.45014E+00_JPRB, 0.43726E+00_JPRB, 0.42862E+00_JPRB, 0.42162E+00_JPRB, 0.41721E+00_JPRB, 0.41359E+00_JPRB, 0.41063E+00_JPRB ,&
&  0.40619E+00_JPRB, 0.40153E+00_JPRB, 0.39712E+00_JPRB, 0.39496E+00_JPRB, 0.39669E+00_JPRB, 0.40146E+00_JPRB, 0.40772E+00_JPRB ,&
&  0.41278E+00_JPRB, 0.41681E+00_JPRB, 0.41787E+00_JPRB, 0.41927E+00_JPRB, 0.42184E+00_JPRB, 0.42328E+00_JPRB, 0.41953E+00_JPRB ,&
&  0.41023E+00_JPRB, 0.40023E+00_JPRB, 0.39345E+00_JPRB, 0.38948E+00_JPRB, 0.38787E+00_JPRB, 0.38798E+00_JPRB, 0.38953E+00_JPRB ,&
&  0.38904E+00_JPRB, 0.38779E+00_JPRB, 0.38645E+00_JPRB, 0.38705E+00_JPRB, 0.38836E+00_JPRB, 0.39238E+00_JPRB, 0.39822E+00_JPRB ,&
&  0.40470E+00_JPRB, 0.41119E+00_JPRB, 0.41840E+00_JPRB, 0.42536E+00_JPRB, 0.42917E+00_JPRB, 0.43323E+00_JPRB, 0.43441E+00_JPRB ,&
&  0.43466E+00_JPRB, 0.43747E+00_JPRB, 0.44196E+00_JPRB, 0.45339E+00_JPRB, 0.47620E+00_JPRB, 0.48358E+00_JPRB, 0.47126E+00_JPRB ,&
&  0.46013E+00_JPRB, 0.44660E+00_JPRB, 0.44832E+00_JPRB, 0.41635E+00_JPRB, 0.45186E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.51172E+00_JPRB, 0.50623E+00_JPRB, 0.49765E+00_JPRB, 0.48984E+00_JPRB, 0.47905E+00_JPRB, 0.47058E+00_JPRB, 0.45974E+00_JPRB ,&
&  0.45008E+00_JPRB, 0.44046E+00_JPRB, 0.43172E+00_JPRB, 0.42894E+00_JPRB, 0.42282E+00_JPRB, 0.41367E+00_JPRB, 0.39954E+00_JPRB ,&
&  0.38764E+00_JPRB, 0.37812E+00_JPRB, 0.37256E+00_JPRB, 0.37083E+00_JPRB, 0.37006E+00_JPRB, 0.36796E+00_JPRB, 0.36725E+00_JPRB ,&
&  0.36841E+00_JPRB, 0.36833E+00_JPRB, 0.37100E+00_JPRB, 0.37115E+00_JPRB, 0.37504E+00_JPRB, 0.37160E+00_JPRB, 0.36945E+00_JPRB ,&
&  0.38490E+00_JPRB, 0.37703E+00_JPRB, 0.41931E+00_JPRB, 0.46558E+00_JPRB, 0.50720E+00_JPRB, 0.53521E+00_JPRB, 0.56630E+00_JPRB ,&
&  0.58994E+00_JPRB, 0.58668E+00_JPRB, 0.57075E+00_JPRB, 0.54143E+00_JPRB, 0.51876E+00_JPRB, 0.49293E+00_JPRB, 0.46690E+00_JPRB ,&
&  0.45415E+00_JPRB, 0.44868E+00_JPRB, 0.44398E+00_JPRB, 0.43958E+00_JPRB, 0.43434E+00_JPRB, 0.43201E+00_JPRB, 0.43210E+00_JPRB ,&
&  0.42884E+00_JPRB, 0.42414E+00_JPRB, 0.41781E+00_JPRB, 0.41596E+00_JPRB, 0.41568E+00_JPRB, 0.42481E+00_JPRB, 0.43827E+00_JPRB ,&
&  0.44567E+00_JPRB, 0.45340E+00_JPRB, 0.45935E+00_JPRB, 0.46679E+00_JPRB, 0.46758E+00_JPRB, 0.46484E+00_JPRB, 0.45209E+00_JPRB ,&
&  0.44199E+00_JPRB, 0.43543E+00_JPRB, 0.43214E+00_JPRB, 0.43036E+00_JPRB, 0.42962E+00_JPRB, 0.42747E+00_JPRB, 0.42723E+00_JPRB ,&
&  0.42461E+00_JPRB, 0.42141E+00_JPRB, 0.42075E+00_JPRB, 0.42158E+00_JPRB, 0.42292E+00_JPRB, 0.42448E+00_JPRB, 0.42669E+00_JPRB ,&
&  0.42974E+00_JPRB, 0.43245E+00_JPRB, 0.43639E+00_JPRB, 0.44020E+00_JPRB, 0.44121E+00_JPRB, 0.44345E+00_JPRB, 0.44757E+00_JPRB ,&
&  0.45053E+00_JPRB, 0.45663E+00_JPRB, 0.46249E+00_JPRB, 0.48913E+00_JPRB, 0.50466E+00_JPRB, 0.49874E+00_JPRB, 0.48293E+00_JPRB ,&
&  0.46988E+00_JPRB, 0.46438E+00_JPRB, 0.46705E+00_JPRB, 0.48681E+00_JPRB, 0.48902E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.61430E+00_JPRB, 0.57478E+00_JPRB, 0.53839E+00_JPRB, 0.51662E+00_JPRB, 0.50286E+00_JPRB, 0.49325E+00_JPRB, 0.48144E+00_JPRB ,&
&  0.47020E+00_JPRB, 0.45815E+00_JPRB, 0.44853E+00_JPRB, 0.44200E+00_JPRB, 0.43447E+00_JPRB, 0.41978E+00_JPRB, 0.40027E+00_JPRB ,&
&  0.38691E+00_JPRB, 0.38461E+00_JPRB, 0.38296E+00_JPRB, 0.37869E+00_JPRB, 0.37474E+00_JPRB, 0.37410E+00_JPRB, 0.37226E+00_JPRB ,&
&  0.37232E+00_JPRB, 0.37237E+00_JPRB, 0.37522E+00_JPRB, 0.37683E+00_JPRB, 0.37562E+00_JPRB, 0.38242E+00_JPRB, 0.38249E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.41440E+00_JPRB, 0.48964E+00_JPRB, 0.57480E+00_JPRB, 0.62805E+00_JPRB, 0.65249E+00_JPRB, 0.65194E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.62327E+00_JPRB, 0.59781E+00_JPRB, 0.57069E+00_JPRB, 0.54022E+00_JPRB, 0.50563E+00_JPRB, 0.48473E+00_JPRB ,&
&  0.47964E+00_JPRB, 0.47335E+00_JPRB, 0.46579E+00_JPRB, 0.45532E+00_JPRB, 0.44466E+00_JPRB, 0.44263E+00_JPRB, 0.44572E+00_JPRB ,&
&  0.44417E+00_JPRB, 0.44199E+00_JPRB, 0.43932E+00_JPRB, 0.44251E+00_JPRB, 0.45619E+00_JPRB, 0.48466E+00_JPRB, 0.50992E+00_JPRB ,&
&  0.52087E+00_JPRB, 0.53342E+00_JPRB, 0.53888E+00_JPRB, 0.54025E+00_JPRB, 0.53208E+00_JPRB, 0.51968E+00_JPRB, 0.50394E+00_JPRB ,&
&  0.49061E+00_JPRB, 0.48092E+00_JPRB, 0.47663E+00_JPRB, 0.47271E+00_JPRB, 0.46874E+00_JPRB, 0.46509E+00_JPRB, 0.46113E+00_JPRB ,&
&  0.45935E+00_JPRB, 0.45871E+00_JPRB, 0.45975E+00_JPRB, 0.45859E+00_JPRB, 0.45714E+00_JPRB, 0.45460E+00_JPRB, 0.45178E+00_JPRB ,&
&  0.44677E+00_JPRB, 0.44372E+00_JPRB, 0.44883E+00_JPRB, 0.46067E+00_JPRB, 0.47394E+00_JPRB, 0.48294E+00_JPRB, 0.48852E+00_JPRB ,&
&  0.49098E+00_JPRB, 0.50047E+00_JPRB, 0.49165E+00_JPRB, 0.52005E+00_JPRB, 0.56882E+00_JPRB, 0.54972E+00_JPRB, 0.53284E+00_JPRB ,&
&  0.51327E+00_JPRB, 0.50419E+00_JPRB, 0.50493E+00_JPRB, 0.52703E+00_JPRB, 0.60871E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.56640E+00_JPRB, 0.53148E+00_JPRB, 0.51403E+00_JPRB, 0.50439E+00_JPRB, 0.50029E+00_JPRB, 0.49759E+00_JPRB, 0.48866E+00_JPRB ,&
&  0.47542E+00_JPRB, 0.46334E+00_JPRB, 0.45476E+00_JPRB, 0.44836E+00_JPRB, 0.43528E+00_JPRB, 0.41516E+00_JPRB, 0.40040E+00_JPRB ,&
&  0.39808E+00_JPRB, 0.39582E+00_JPRB, 0.39453E+00_JPRB, 0.38972E+00_JPRB, 0.38467E+00_JPRB, 0.38278E+00_JPRB, 0.38014E+00_JPRB ,&
&  0.37639E+00_JPRB, 0.37240E+00_JPRB, 0.37314E+00_JPRB, 0.37666E+00_JPRB, 0.38036E+00_JPRB, 0.38471E+00_JPRB, 0.39167E+00_JPRB ,&
&  0.41280E+00_JPRB, 0.50250E+00_JPRB, 0.62602E+00_JPRB, 0.79178E+00_JPRB, 0.83502E+00_JPRB, 0.80571E+00_JPRB, 0.78617E+00_JPRB ,&
&  0.74321E+00_JPRB, 0.69177E+00_JPRB, 0.64581E+00_JPRB, 0.60312E+00_JPRB, 0.55470E+00_JPRB, 0.52871E+00_JPRB, 0.51706E+00_JPRB ,&
&  0.50833E+00_JPRB, 0.49587E+00_JPRB, 0.48201E+00_JPRB, 0.46725E+00_JPRB, 0.45764E+00_JPRB, 0.46168E+00_JPRB, 0.47148E+00_JPRB ,&
&  0.47076E+00_JPRB, 0.47227E+00_JPRB, 0.47422E+00_JPRB, 0.49053E+00_JPRB, 0.53842E+00_JPRB, 0.58511E+00_JPRB, 0.62587E+00_JPRB ,&
&  0.64018E+00_JPRB, 0.67484E+00_JPRB, 0.65454E+00_JPRB, 0.63107E+00_JPRB, 0.60504E+00_JPRB, 0.58149E+00_JPRB, 0.56198E+00_JPRB ,&
&  0.54860E+00_JPRB, 0.53783E+00_JPRB, 0.52888E+00_JPRB, 0.52088E+00_JPRB, 0.51294E+00_JPRB, 0.50499E+00_JPRB, 0.50304E+00_JPRB ,&
&  0.50265E+00_JPRB, 0.49687E+00_JPRB, 0.49164E+00_JPRB, 0.48575E+00_JPRB, 0.47574E+00_JPRB, 0.46619E+00_JPRB, 0.45875E+00_JPRB ,&
&  0.45179E+00_JPRB, 0.45245E+00_JPRB, 0.47385E+00_JPRB, 0.49775E+00_JPRB, 0.51342E+00_JPRB, 0.52604E+00_JPRB, 0.53388E+00_JPRB ,&
&  0.53714E+00_JPRB, 0.54507E+00_JPRB, 0.53650E+00_JPRB, 0.54886E+00_JPRB, 0.58109E+00_JPRB, 0.60091E+00_JPRB, 0.57260E+00_JPRB ,&
&  0.55577E+00_JPRB, 0.54803E+00_JPRB, 0.54593E+00_JPRB, 0.54776E+00_JPRB, 0.59921E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.54138E+00_JPRB, 0.51985E+00_JPRB, 0.50982E+00_JPRB, 0.50365E+00_JPRB, 0.49921E+00_JPRB, 0.49566E+00_JPRB, 0.48678E+00_JPRB ,&
&  0.47334E+00_JPRB, 0.46518E+00_JPRB, 0.45594E+00_JPRB, 0.44254E+00_JPRB, 0.42385E+00_JPRB, 0.41331E+00_JPRB, 0.41337E+00_JPRB ,&
&  0.41273E+00_JPRB, 0.40949E+00_JPRB, 0.40373E+00_JPRB, 0.40153E+00_JPRB, 0.39741E+00_JPRB, 0.39143E+00_JPRB, 0.38821E+00_JPRB ,&
&  0.37985E+00_JPRB, 0.37588E+00_JPRB, 0.37733E+00_JPRB, 0.38215E+00_JPRB, 0.38662E+00_JPRB, 0.39441E+00_JPRB, 0.40668E+00_JPRB ,&
&  0.44808E+00_JPRB, 0.60928E+00_JPRB, 0.89761E+00_JPRB, 0.10740E+01_JPRB, 0.11108E+01_JPRB, 0.10533E+01_JPRB, 0.10423E+01_JPRB ,&
&  0.93853E+00_JPRB, 0.84636E+00_JPRB, 0.76202E+00_JPRB, 0.67825E+00_JPRB, 0.61471E+00_JPRB, 0.58276E+00_JPRB, 0.56427E+00_JPRB ,&
&  0.54206E+00_JPRB, 0.52333E+00_JPRB, 0.50598E+00_JPRB, 0.49144E+00_JPRB, 0.48737E+00_JPRB, 0.49599E+00_JPRB, 0.50629E+00_JPRB ,&
&  0.51029E+00_JPRB, 0.51483E+00_JPRB, 0.52645E+00_JPRB, 0.55835E+00_JPRB, 0.59986E+00_JPRB, 0.63241E+00_JPRB, 0.76913E+00_JPRB ,&
&  0.91990E+00_JPRB, 0.90493E+00_JPRB, 0.83602E+00_JPRB, 0.77447E+00_JPRB, 0.71911E+00_JPRB, 0.67329E+00_JPRB, 0.63836E+00_JPRB ,&
&  0.61959E+00_JPRB, 0.60387E+00_JPRB, 0.59236E+00_JPRB, 0.57923E+00_JPRB, 0.56720E+00_JPRB, 0.56005E+00_JPRB, 0.55353E+00_JPRB ,&
&  0.54397E+00_JPRB, 0.53387E+00_JPRB, 0.52164E+00_JPRB, 0.50425E+00_JPRB, 0.48513E+00_JPRB, 0.47430E+00_JPRB, 0.47037E+00_JPRB ,&
&  0.47165E+00_JPRB, 0.49109E+00_JPRB, 0.51666E+00_JPRB, 0.53032E+00_JPRB, 0.55865E+00_JPRB, 0.58011E+00_JPRB, 0.59651E+00_JPRB ,&
&  0.57543E+00_JPRB, 0.57170E+00_JPRB, 0.58089E+00_JPRB, 0.59971E+00_JPRB, 0.63528E+00_JPRB, 0.65800E+00_JPRB, 0.61995E+00_JPRB ,&
&  0.60319E+00_JPRB, 0.59941E+00_JPRB, 0.58893E+00_JPRB, 0.57116E+00_JPRB, 0.57064E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.53625E+00_JPRB, 0.52336E+00_JPRB, 0.51267E+00_JPRB, 0.50765E+00_JPRB, 0.50309E+00_JPRB, 0.49287E+00_JPRB, 0.48318E+00_JPRB ,&
&  0.47346E+00_JPRB, 0.46279E+00_JPRB, 0.44840E+00_JPRB, 0.42711E+00_JPRB, 0.41337E+00_JPRB, 0.41080E+00_JPRB, 0.41706E+00_JPRB ,&
&  0.41861E+00_JPRB, 0.41189E+00_JPRB, 0.40856E+00_JPRB, 0.40849E+00_JPRB, 0.40783E+00_JPRB, 0.40211E+00_JPRB, 0.39432E+00_JPRB ,&
&  0.38762E+00_JPRB, 0.38786E+00_JPRB, 0.39005E+00_JPRB, 0.39449E+00_JPRB, 0.39835E+00_JPRB, 0.40654E+00_JPRB, 0.42442E+00_JPRB ,&
&  0.48244E+00_JPRB, 0.48031E+00_JPRB, 0.10397E+01_JPRB, 0.13889E+01_JPRB, 0.14290E+01_JPRB, 0.13704E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.10687E+01_JPRB, 0.93443E+00_JPRB, 0.81050E+00_JPRB, 0.71712E+00_JPRB, 0.65577E+00_JPRB, 0.61107E+00_JPRB ,&
&  0.57925E+00_JPRB, 0.55460E+00_JPRB, 0.53395E+00_JPRB, 0.52391E+00_JPRB, 0.52760E+00_JPRB, 0.53168E+00_JPRB, 0.53400E+00_JPRB ,&
&  0.54023E+00_JPRB, 0.55106E+00_JPRB, 0.57700E+00_JPRB, 0.62804E+00_JPRB, 0.68700E+00_JPRB, 0.79370E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.12005E+01_JPRB, 0.10559E+01_JPRB, 0.92883E+00_JPRB, 0.84277E+00_JPRB, 0.78676E+00_JPRB, 0.74560E+00_JPRB ,&
&  0.71174E+00_JPRB, 0.68592E+00_JPRB, 0.66632E+00_JPRB, 0.63862E+00_JPRB, 0.61969E+00_JPRB, 0.60736E+00_JPRB, 0.58773E+00_JPRB ,&
&  0.56622E+00_JPRB, 0.54618E+00_JPRB, 0.52573E+00_JPRB, 0.50734E+00_JPRB, 0.49187E+00_JPRB, 0.49011E+00_JPRB, 0.49454E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.52697E+00_JPRB, 0.54026E+00_JPRB, 0.55733E+00_JPRB, 0.58488E+00_JPRB, 0.60414E+00_JPRB, 0.61532E+00_JPRB ,&
&  0.60128E+00_JPRB, 0.60841E+00_JPRB, 0.62654E+00_JPRB, 0.65167E+00_JPRB, 0.62858E+00_JPRB, 0.69218E+00_JPRB, 0.66190E+00_JPRB ,&
&  0.65368E+00_JPRB, 0.63976E+00_JPRB, 0.59936E+00_JPRB, 0.57966E+00_JPRB, 0.55925E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.54403E+00_JPRB, 0.52793E+00_JPRB, 0.51474E+00_JPRB, 0.50338E+00_JPRB, 0.49489E+00_JPRB, 0.48977E+00_JPRB, 0.48261E+00_JPRB ,&
&  0.47331E+00_JPRB, 0.46035E+00_JPRB, 0.44798E+00_JPRB, 0.43108E+00_JPRB, 0.42388E+00_JPRB, 0.42596E+00_JPRB, 0.43367E+00_JPRB ,&
&  0.42879E+00_JPRB, 0.41663E+00_JPRB, 0.41590E+00_JPRB, 0.41686E+00_JPRB, 0.41364E+00_JPRB, 0.40661E+00_JPRB, 0.39948E+00_JPRB ,&
&  0.40175E+00_JPRB, 0.40423E+00_JPRB, 0.40635E+00_JPRB, 0.40952E+00_JPRB, 0.41386E+00_JPRB, 0.42562E+00_JPRB, 0.45026E+00_JPRB ,&
&  0.52076E+00_JPRB, 0.84189E+00_JPRB, 0.93895E+00_JPRB, 0.16703E+01_JPRB, 0.18374E+01_JPRB, 0.17784E+01_JPRB, 0.15711E+01_JPRB ,&
&  0.13687E+01_JPRB, 0.12567E+01_JPRB, 0.11001E+01_JPRB, 0.96358E+00_JPRB, 0.82809E+00_JPRB, 0.72817E+00_JPRB, 0.66689E+00_JPRB ,&
&  0.62608E+00_JPRB, 0.59511E+00_JPRB, 0.57710E+00_JPRB, 0.57188E+00_JPRB, 0.56802E+00_JPRB, 0.56618E+00_JPRB, 0.56814E+00_JPRB ,&
&  0.57682E+00_JPRB, 0.59817E+00_JPRB, 0.64015E+00_JPRB, 0.71277E+00_JPRB, 0.75613E+00_JPRB, 0.97690E+00_JPRB, 0.12638E+01_JPRB ,&
&  0.13990E+01_JPRB, 0.14426E+01_JPRB, 0.12376E+01_JPRB, 0.10524E+01_JPRB, 0.94352E+00_JPRB, 0.87356E+00_JPRB, 0.81561E+00_JPRB ,&
&  0.77426E+00_JPRB, 0.74226E+00_JPRB, 0.70703E+00_JPRB, 0.66284E+00_JPRB, 0.63873E+00_JPRB, 0.61982E+00_JPRB, 0.59014E+00_JPRB ,&
&  0.55874E+00_JPRB, 0.53760E+00_JPRB, 0.52535E+00_JPRB, 0.51648E+00_JPRB, 0.51226E+00_JPRB, 0.51766E+00_JPRB, 0.52726E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.56693E+00_JPRB, 0.56795E+00_JPRB, 0.57757E+00_JPRB, 0.60328E+00_JPRB, 0.63018E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.62449E+00_JPRB, 0.63456E+00_JPRB, 0.65420E+00_JPRB, 0.67327E+00_JPRB, 0.65708E+00_JPRB, 0.72363E+00_JPRB, 0.68553E+00_JPRB ,&
&  0.68285E+00_JPRB, 0.63889E+00_JPRB, 0.60073E+00_JPRB, 0.58672E+00_JPRB, 0.56820E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.54579E+00_JPRB, 0.51926E+00_JPRB, 0.50138E+00_JPRB, 0.49187E+00_JPRB, 0.48709E+00_JPRB, 0.48584E+00_JPRB, 0.47821E+00_JPRB ,&
&  0.47229E+00_JPRB, 0.46003E+00_JPRB, 0.44460E+00_JPRB, 0.43189E+00_JPRB, 0.43756E+00_JPRB, 0.44112E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.43396E+00_JPRB, 0.42969E+00_JPRB, 0.42827E+00_JPRB, 0.42526E+00_JPRB, 0.41998E+00_JPRB, 0.41533E+00_JPRB, 0.41417E+00_JPRB ,&
&  0.41956E+00_JPRB, 0.41963E+00_JPRB, 0.41909E+00_JPRB, 0.42350E+00_JPRB, 0.43410E+00_JPRB, 0.45844E+00_JPRB, 0.49365E+00_JPRB ,&
&  0.58961E+00_JPRB, 0.76053E+00_JPRB, 0.93690E+00_JPRB, 0.18600E+01_JPRB, 0.22127E+01_JPRB, 0.21106E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.14858E+01_JPRB, 0.14275E+01_JPRB, 0.12575E+01_JPRB, 0.10614E+01_JPRB, 0.88415E+00_JPRB, 0.78276E+00_JPRB, 0.71178E+00_JPRB ,&
&  0.66273E+00_JPRB, 0.63660E+00_JPRB, 0.62836E+00_JPRB, 0.62578E+00_JPRB, 0.62087E+00_JPRB, 0.62591E+00_JPRB, 0.63256E+00_JPRB ,&
&  0.64075E+00_JPRB, 0.66543E+00_JPRB, 0.70797E+00_JPRB, 0.73308E+00_JPRB, 0.89119E+00_JPRB, 0.12804E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.16198E+01_JPRB, 0.15484E+01_JPRB, 0.13387E+01_JPRB, 0.11226E+01_JPRB, 0.98211E+00_JPRB, 0.92431E+00_JPRB, 0.86132E+00_JPRB ,&
&  0.79478E+00_JPRB, 0.75187E+00_JPRB, 0.69575E+00_JPRB, 0.66310E+00_JPRB, 0.65141E+00_JPRB, 0.60814E+00_JPRB, 0.57350E+00_JPRB ,&
&  0.55608E+00_JPRB, 0.54873E+00_JPRB, 0.54453E+00_JPRB, 0.54042E+00_JPRB, 0.54289E+00_JPRB, 0.55081E+00_JPRB, 0.56512E+00_JPRB ,&
&  0.58567E+00_JPRB, 0.61598E+00_JPRB, 0.60715E+00_JPRB, 0.61709E+00_JPRB, 0.65195E+00_JPRB, 0.67845E+00_JPRB, 0.67082E+00_JPRB ,&
&  0.66264E+00_JPRB, 0.66576E+00_JPRB, 0.67507E+00_JPRB, 0.66039E+00_JPRB, 0.65211E+00_JPRB, 0.70933E+00_JPRB, 0.70340E+00_JPRB ,&
&  0.71055E+00_JPRB, 0.65733E+00_JPRB, 0.62978E+00_JPRB, 0.59410E+00_JPRB, 0.57204E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.52770E+00_JPRB, 0.50228E+00_JPRB, 0.49691E+00_JPRB, 0.49314E+00_JPRB, 0.48226E+00_JPRB, 0.47323E+00_JPRB, 0.46393E+00_JPRB ,&
&  0.45623E+00_JPRB, 0.44242E+00_JPRB, 0.43294E+00_JPRB, 0.43971E+00_JPRB, 0.45596E+00_JPRB, 0.46641E+00_JPRB, 0.45855E+00_JPRB ,&
&  0.44948E+00_JPRB, 0.44365E+00_JPRB, 0.44749E+00_JPRB, 0.44850E+00_JPRB, 0.44453E+00_JPRB, 0.44609E+00_JPRB, 0.44480E+00_JPRB ,&
&  0.44557E+00_JPRB, 0.44272E+00_JPRB, 0.44551E+00_JPRB, 0.45837E+00_JPRB, 0.48135E+00_JPRB, 0.52107E+00_JPRB, 0.57794E+00_JPRB ,&
&  0.73548E+00_JPRB, 0.83499E+00_JPRB, 0.10233E+01_JPRB, 0.16706E+01_JPRB, 0.21073E+01_JPRB, 0.19651E+01_JPRB, 0.17724E+01_JPRB ,&
&  0.15218E+01_JPRB, 0.11990E+01_JPRB, 0.11537E+01_JPRB, 0.10628E+01_JPRB, 0.91785E+00_JPRB, 0.80528E+00_JPRB, 0.73684E+00_JPRB ,&
&  0.70075E+00_JPRB, 0.67734E+00_JPRB, 0.67459E+00_JPRB, 0.68525E+00_JPRB, 0.70416E+00_JPRB, 0.72616E+00_JPRB, 0.73588E+00_JPRB ,&
&  0.74193E+00_JPRB, 0.75794E+00_JPRB, 0.80013E+00_JPRB, 0.86304E+00_JPRB, 0.11711E+01_JPRB, 0.16585E+01_JPRB, 0.18504E+01_JPRB ,&
&  0.17673E+01_JPRB, 0.15795E+01_JPRB, 0.13201E+01_JPRB, 0.11076E+01_JPRB, 0.90884E+00_JPRB, 0.91533E+00_JPRB, 0.84856E+00_JPRB ,&
&  0.77379E+00_JPRB, 0.71950E+00_JPRB, 0.68920E+00_JPRB, 0.66013E+00_JPRB, 0.62318E+00_JPRB, 0.58844E+00_JPRB, 0.56989E+00_JPRB ,&
&  0.56949E+00_JPRB, 0.57568E+00_JPRB, 0.57515E+00_JPRB, 0.56956E+00_JPRB, 0.58010E+00_JPRB, 0.58790E+00_JPRB, 0.60764E+00_JPRB ,&
&  0.62876E+00_JPRB, 0.65493E+00_JPRB, 0.65684E+00_JPRB, 0.68601E+00_JPRB, 0.72216E+00_JPRB, 0.74536E+00_JPRB, 0.73281E+00_JPRB ,&
&  0.69978E+00_JPRB, 0.71713E+00_JPRB, 0.72276E+00_JPRB, 0.70390E+00_JPRB, 0.70162E+00_JPRB, 0.70460E+00_JPRB, 0.70015E+00_JPRB ,&
&  0.72741E+00_JPRB, 0.71918E+00_JPRB, 0.68272E+00_JPRB, 0.59480E+00_JPRB, 0.55819E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.51495E+00_JPRB, 0.49756E+00_JPRB, 0.49702E+00_JPRB, 0.48991E+00_JPRB, 0.47803E+00_JPRB, 0.47123E+00_JPRB, 0.46377E+00_JPRB ,&
&  0.45687E+00_JPRB, 0.45710E+00_JPRB, 0.46042E+00_JPRB, 0.48912E+00_JPRB, 0.50182E+00_JPRB, 0.49389E+00_JPRB, 0.48463E+00_JPRB ,&
&  0.47856E+00_JPRB, 0.47891E+00_JPRB, 0.48117E+00_JPRB, 0.48236E+00_JPRB, 0.48202E+00_JPRB, 0.48742E+00_JPRB, 0.49448E+00_JPRB ,&
&  0.49329E+00_JPRB, 0.49565E+00_JPRB, 0.50724E+00_JPRB, 0.53179E+00_JPRB, 0.57809E+00_JPRB, 0.65129E+00_JPRB, 0.77074E+00_JPRB ,&
&  0.10160E+01_JPRB, 0.10167E+01_JPRB, 0.50814E+00_JPRB, 0.99497E+00_JPRB, 0.16144E+01_JPRB, 0.16864E+01_JPRB, 0.15951E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.10838E+01_JPRB, 0.10029E+01_JPRB, 0.10294E+01_JPRB, 0.90389E+00_JPRB, 0.80953E+00_JPRB, 0.75625E+00_JPRB ,&
&  0.72749E+00_JPRB, 0.73041E+00_JPRB, 0.76668E+00_JPRB, 0.80152E+00_JPRB, 0.83518E+00_JPRB, 0.84945E+00_JPRB, 0.84734E+00_JPRB ,&
&  0.85078E+00_JPRB, 0.88105E+00_JPRB, 0.97173E+00_JPRB, 0.10898E+01_JPRB, 0.16349E+01_JPRB, 0.18861E+01_JPRB, 0.19071E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.12942E+01_JPRB, 0.11601E+01_JPRB, 0.10213E+01_JPRB, 0.87605E+00_JPRB, 0.79893E+00_JPRB, 0.80313E+00_JPRB ,&
&  0.72554E+00_JPRB, 0.66999E+00_JPRB, 0.65300E+00_JPRB, 0.63233E+00_JPRB, 0.60987E+00_JPRB, 0.59722E+00_JPRB, 0.59272E+00_JPRB ,&
&  0.60303E+00_JPRB, 0.61258E+00_JPRB, 0.61582E+00_JPRB, 0.61543E+00_JPRB, 0.62093E+00_JPRB, 0.63010E+00_JPRB, 0.64951E+00_JPRB ,&
&  0.66461E+00_JPRB, 0.69602E+00_JPRB, 0.73654E+00_JPRB, 0.74123E+00_JPRB, 0.76102E+00_JPRB, 0.80408E+00_JPRB, 0.79242E+00_JPRB ,&
&  0.76670E+00_JPRB, 0.79936E+00_JPRB, 0.75976E+00_JPRB, 0.70280E+00_JPRB, 0.71338E+00_JPRB, 0.70171E+00_JPRB, 0.71255E+00_JPRB ,&
&  0.77282E+00_JPRB, 0.90785E+00_JPRB, 0.88372E+00_JPRB, 0.58270E+00_JPRB, 0.53245E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.47778E+00_JPRB, 0.46735E+00_JPRB, 0.46182E+00_JPRB, 0.46780E+00_JPRB, 0.47153E+00_JPRB, 0.47662E+00_JPRB, 0.46423E+00_JPRB ,&
&  0.47532E+00_JPRB, 0.50682E+00_JPRB, 0.53201E+00_JPRB, 0.54755E+00_JPRB, 0.54225E+00_JPRB, 0.53199E+00_JPRB, 0.52820E+00_JPRB ,&
&  0.52783E+00_JPRB, 0.53159E+00_JPRB, 0.54347E+00_JPRB, 0.54878E+00_JPRB, 0.55197E+00_JPRB, 0.56022E+00_JPRB, 0.56697E+00_JPRB ,&
&  0.57418E+00_JPRB, 0.58935E+00_JPRB, 0.61762E+00_JPRB, 0.67098E+00_JPRB, 0.75820E+00_JPRB, 0.89345E+00_JPRB, 0.10680E+01_JPRB ,&
&  0.13277E+01_JPRB, 0.92551E+00_JPRB, 0.52200E+00_JPRB, 0.12604E+01_JPRB, 0.16094E+01_JPRB, 0.16029E+01_JPRB, 0.13778E+01_JPRB ,&
&  0.12401E+01_JPRB, 0.10497E+01_JPRB, 0.95651E+00_JPRB, 0.10078E+01_JPRB, 0.88494E+00_JPRB, 0.83665E+00_JPRB, 0.82092E+00_JPRB ,&
&  0.83527E+00_JPRB, 0.87027E+00_JPRB, 0.90540E+00_JPRB, 0.93145E+00_JPRB, 0.93842E+00_JPRB, 0.93121E+00_JPRB, 0.93632E+00_JPRB ,&
&  0.98791E+00_JPRB, 0.10694E+01_JPRB, 0.12240E+01_JPRB, 0.14428E+01_JPRB, 0.19379E+01_JPRB, 0.18014E+01_JPRB, 0.17298E+01_JPRB ,&
&  0.11923E+01_JPRB, 0.10610E+01_JPRB, 0.10160E+01_JPRB, 0.95359E+00_JPRB, 0.83127E+00_JPRB, 0.72348E+00_JPRB, 0.73435E+00_JPRB ,&
&  0.67163E+00_JPRB, 0.63669E+00_JPRB, 0.62915E+00_JPRB, 0.61344E+00_JPRB, 0.60074E+00_JPRB, 0.60505E+00_JPRB, 0.62241E+00_JPRB ,&
&  0.64836E+00_JPRB, 0.65596E+00_JPRB, 0.65703E+00_JPRB, 0.65473E+00_JPRB, 0.65811E+00_JPRB, 0.66573E+00_JPRB, 0.67342E+00_JPRB ,&
&  0.69398E+00_JPRB, 0.72648E+00_JPRB, 0.76204E+00_JPRB, 0.77562E+00_JPRB, 0.77706E+00_JPRB, 0.79828E+00_JPRB, 0.83807E+00_JPRB ,&
&  0.80978E+00_JPRB, 0.81562E+00_JPRB, 0.76242E+00_JPRB, 0.74174E+00_JPRB, 0.70083E+00_JPRB, 0.69370E+00_JPRB, 0.69046E+00_JPRB ,&
&  0.67435E+00_JPRB, 0.62019E+00_JPRB, 0.55790E+00_JPRB, 0.48886E+00_JPRB, 0.47900E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.46695E+00_JPRB, 0.45793E+00_JPRB, 0.45425E+00_JPRB, 0.46964E+00_JPRB, 0.46664E+00_JPRB, 0.47744E+00_JPRB, 0.49244E+00_JPRB ,&
&  0.53422E+00_JPRB, 0.57280E+00_JPRB, 0.60457E+00_JPRB, 0.59724E+00_JPRB, 0.59605E+00_JPRB, 0.58474E+00_JPRB, 0.59002E+00_JPRB ,&
&  0.60097E+00_JPRB, 0.61935E+00_JPRB, 0.63943E+00_JPRB, 0.64760E+00_JPRB, 0.65402E+00_JPRB, 0.66274E+00_JPRB, 0.67616E+00_JPRB ,&
&  0.69712E+00_JPRB, 0.72758E+00_JPRB, 0.78122E+00_JPRB, 0.87594E+00_JPRB, 0.10223E+01_JPRB, 0.11615E+01_JPRB, 0.12593E+01_JPRB ,&
&  0.86807E+00_JPRB, 0.41609E+00_JPRB, 0.11853E+01_JPRB, 0.15805E+01_JPRB, 0.15397E+01_JPRB, 0.14510E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.11362E+01_JPRB, 0.10251E+01_JPRB, 0.93639E+00_JPRB, 0.10171E+01_JPRB, 0.92463E+00_JPRB, 0.90980E+00_JPRB, 0.92632E+00_JPRB ,&
&  0.95703E+00_JPRB, 0.98624E+00_JPRB, 0.10126E+01_JPRB, 0.10173E+01_JPRB, 0.10106E+01_JPRB, 0.10210E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.12819E+01_JPRB, 0.14511E+01_JPRB, 0.14791E+01_JPRB, 0.16654E+01_JPRB, 0.15758E+01_JPRB, 0.16234E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.89716E+00_JPRB, 0.85524E+00_JPRB, 0.88689E+00_JPRB, 0.80203E+00_JPRB, 0.67558E+00_JPRB, 0.67756E+00_JPRB ,&
&  0.63505E+00_JPRB, 0.61727E+00_JPRB, 0.62083E+00_JPRB, 0.61072E+00_JPRB, 0.59960E+00_JPRB, 0.61627E+00_JPRB, 0.64027E+00_JPRB ,&
&  0.66712E+00_JPRB, 0.67818E+00_JPRB, 0.67023E+00_JPRB, 0.66634E+00_JPRB, 0.66957E+00_JPRB, 0.66967E+00_JPRB, 0.67129E+00_JPRB ,&
&  0.69553E+00_JPRB, 0.72328E+00_JPRB, 0.75791E+00_JPRB, 0.79504E+00_JPRB, 0.84001E+00_JPRB, 0.84181E+00_JPRB, 0.82634E+00_JPRB ,&
&  0.79675E+00_JPRB, 0.78066E+00_JPRB, 0.74092E+00_JPRB, 0.72762E+00_JPRB, 0.70933E+00_JPRB, 0.68243E+00_JPRB, 0.65696E+00_JPRB ,&
&  0.60991E+00_JPRB, 0.53417E+00_JPRB, 0.49131E+00_JPRB, 0.48394E+00_JPRB, 0.47330E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.49233E+00_JPRB, 0.48373E+00_JPRB, 0.48735E+00_JPRB, 0.49527E+00_JPRB, 0.50875E+00_JPRB, 0.53160E+00_JPRB, 0.56868E+00_JPRB ,&
&  0.61176E+00_JPRB, 0.63430E+00_JPRB, 0.65318E+00_JPRB, 0.64169E+00_JPRB, 0.64276E+00_JPRB, 0.65705E+00_JPRB, 0.66513E+00_JPRB ,&
&  0.69055E+00_JPRB, 0.71781E+00_JPRB, 0.73274E+00_JPRB, 0.73552E+00_JPRB, 0.74319E+00_JPRB, 0.76706E+00_JPRB, 0.80097E+00_JPRB ,&
&  0.84304E+00_JPRB, 0.90018E+00_JPRB, 0.97848E+00_JPRB, 0.10873E+01_JPRB, 0.12099E+01_JPRB, 0.12490E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.64641E+00_JPRB, 0.11916E+01_JPRB, 0.15807E+01_JPRB, 0.15552E+01_JPRB, 0.14636E+01_JPRB, 0.13766E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.99388E+00_JPRB, 0.96984E+00_JPRB, 0.10554E+01_JPRB, 0.98777E+00_JPRB, 0.99625E+00_JPRB, 0.10185E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10739E+01_JPRB, 0.10823E+01_JPRB, 0.10848E+01_JPRB, 0.11075E+01_JPRB, 0.11533E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.12982E+01_JPRB, 0.14141E+01_JPRB, 0.15062E+01_JPRB, 0.14076E+01_JPRB, 0.13940E+01_JPRB, 0.14549E+01_JPRB, 0.16659E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.81300E+00_JPRB, 0.80812E+00_JPRB, 0.83828E+00_JPRB, 0.75546E+00_JPRB, 0.68465E+00_JPRB, 0.64503E+00_JPRB ,&
&  0.61878E+00_JPRB, 0.59377E+00_JPRB, 0.58360E+00_JPRB, 0.57704E+00_JPRB, 0.59231E+00_JPRB, 0.61057E+00_JPRB, 0.63726E+00_JPRB ,&
&  0.64676E+00_JPRB, 0.64674E+00_JPRB, 0.63665E+00_JPRB, 0.63139E+00_JPRB, 0.63665E+00_JPRB, 0.64507E+00_JPRB, 0.64503E+00_JPRB ,&
&  0.66445E+00_JPRB, 0.67187E+00_JPRB, 0.69713E+00_JPRB, 0.74316E+00_JPRB, 0.80286E+00_JPRB, 0.82617E+00_JPRB, 0.82054E+00_JPRB ,&
&  0.76510E+00_JPRB, 0.75839E+00_JPRB, 0.76166E+00_JPRB, 0.74547E+00_JPRB, 0.72860E+00_JPRB, 0.68034E+00_JPRB, 0.59001E+00_JPRB ,&
&  0.54942E+00_JPRB, 0.51130E+00_JPRB, 0.49744E+00_JPRB, 0.49220E+00_JPRB, 0.50379E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.56088E+00_JPRB, 0.54840E+00_JPRB, 0.55531E+00_JPRB, 0.57166E+00_JPRB, 0.58449E+00_JPRB, 0.61092E+00_JPRB, 0.64425E+00_JPRB ,&
&  0.68726E+00_JPRB, 0.71395E+00_JPRB, 0.72284E+00_JPRB, 0.72183E+00_JPRB, 0.72882E+00_JPRB, 0.72869E+00_JPRB, 0.74013E+00_JPRB ,&
&  0.76280E+00_JPRB, 0.78660E+00_JPRB, 0.81711E+00_JPRB, 0.84132E+00_JPRB, 0.85721E+00_JPRB, 0.89957E+00_JPRB, 0.94540E+00_JPRB ,&
&  0.10045E+01_JPRB, 0.10680E+01_JPRB, 0.11479E+01_JPRB, 0.12378E+01_JPRB, 0.13323E+01_JPRB, 0.13692E+01_JPRB, 0.13605E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.16148E+01_JPRB, 0.16343E+01_JPRB, 0.14671E+01_JPRB, 0.13133E+01_JPRB, 0.12540E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.11334E+01_JPRB, 0.10347E+01_JPRB, 0.10099E+01_JPRB, 0.10226E+01_JPRB, 0.10538E+01_JPRB, 0.10570E+01_JPRB, 0.10916E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11507E+01_JPRB, 0.11655E+01_JPRB, 0.11807E+01_JPRB, 0.12099E+01_JPRB, 0.12593E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13872E+01_JPRB, 0.14466E+01_JPRB, 0.15062E+01_JPRB, 0.14046E+01_JPRB, 0.13273E+01_JPRB, 0.14304E+01_JPRB, 0.13815E+01_JPRB ,&
&  0.99882E+00_JPRB, 0.74348E+00_JPRB, 0.76477E+00_JPRB, 0.79673E+00_JPRB, 0.72945E+00_JPRB, 0.66444E+00_JPRB, 0.61988E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.55249E+00_JPRB, 0.55552E+00_JPRB, 0.56136E+00_JPRB, 0.56877E+00_JPRB, 0.57310E+00_JPRB, 0.58821E+00_JPRB ,&
&  0.60296E+00_JPRB, 0.59499E+00_JPRB, 0.58478E+00_JPRB, 0.57799E+00_JPRB, 0.58864E+00_JPRB, 0.60755E+00_JPRB, 0.63004E+00_JPRB ,&
&  0.68296E+00_JPRB, 0.64566E+00_JPRB, 0.66070E+00_JPRB, 0.70669E+00_JPRB, 0.75432E+00_JPRB, 0.80969E+00_JPRB, 0.86025E+00_JPRB ,&
&  0.89700E+00_JPRB, 0.93465E+00_JPRB, 0.98556E+00_JPRB, 0.99547E+00_JPRB, 0.92514E+00_JPRB, 0.84067E+00_JPRB, 0.60324E+00_JPRB ,&
&  0.59139E+00_JPRB, 0.58086E+00_JPRB, 0.58464E+00_JPRB, 0.58367E+00_JPRB, 0.58152E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.68079E+00_JPRB, 0.67775E+00_JPRB, 0.70094E+00_JPRB, 0.72430E+00_JPRB, 0.74464E+00_JPRB, 0.76399E+00_JPRB, 0.77788E+00_JPRB ,&
&  0.79791E+00_JPRB, 0.81295E+00_JPRB, 0.81370E+00_JPRB, 0.80705E+00_JPRB, 0.81808E+00_JPRB, 0.81568E+00_JPRB, 0.83538E+00_JPRB ,&
&  0.86519E+00_JPRB, 0.90059E+00_JPRB, 0.94565E+00_JPRB, 0.97044E+00_JPRB, 0.99144E+00_JPRB, 0.10298E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.11548E+01_JPRB, 0.12041E+01_JPRB, 0.12618E+01_JPRB, 0.13131E+01_JPRB, 0.12967E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.17315E+01_JPRB, 0.16246E+01_JPRB, 0.13820E+01_JPRB, 0.12414E+01_JPRB, 0.11835E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.10891E+01_JPRB, 0.10314E+01_JPRB, 0.10139E+01_JPRB, 0.11237E+01_JPRB, 0.11020E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.11870E+01_JPRB, 0.12221E+01_JPRB, 0.12513E+01_JPRB, 0.12754E+01_JPRB, 0.12913E+01_JPRB, 0.13156E+01_JPRB, 0.13426E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.13943E+01_JPRB, 0.14541E+01_JPRB, 0.13202E+01_JPRB, 0.12778E+01_JPRB, 0.13546E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.94964E+00_JPRB, 0.73363E+00_JPRB, 0.74304E+00_JPRB, 0.76314E+00_JPRB, 0.72053E+00_JPRB, 0.65105E+00_JPRB, 0.61566E+00_JPRB ,&
&  0.58726E+00_JPRB, 0.56581E+00_JPRB, 0.55897E+00_JPRB, 0.54727E+00_JPRB, 0.55092E+00_JPRB, 0.56517E+00_JPRB, 0.57022E+00_JPRB ,&
&  0.59101E+00_JPRB, 0.57866E+00_JPRB, 0.57364E+00_JPRB, 0.57548E+00_JPRB, 0.60440E+00_JPRB, 0.66153E+00_JPRB, 0.78502E+00_JPRB ,&
&  0.90621E+00_JPRB, 0.58287E+00_JPRB, 0.64058E+00_JPRB, 0.69004E+00_JPRB, 0.76546E+00_JPRB, 0.89230E+00_JPRB, 0.10904E+01_JPRB ,&
&  0.14283E+01_JPRB, 0.19028E+01_JPRB, 0.18571E+01_JPRB, 0.14749E+01_JPRB, 0.18788E+01_JPRB, 0.15699E+01_JPRB, 0.68813E+00_JPRB ,&
&  0.69212E+00_JPRB, 0.67384E+00_JPRB, 0.67944E+00_JPRB, 0.68529E+00_JPRB, 0.68696E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.76978E+00_JPRB, 0.78746E+00_JPRB, 0.80752E+00_JPRB, 0.82550E+00_JPRB, 0.83566E+00_JPRB, 0.84482E+00_JPRB, 0.85669E+00_JPRB ,&
&  0.86171E+00_JPRB, 0.86136E+00_JPRB, 0.87072E+00_JPRB, 0.88661E+00_JPRB, 0.90693E+00_JPRB, 0.92796E+00_JPRB, 0.95516E+00_JPRB ,&
&  0.99156E+00_JPRB, 0.10241E+01_JPRB, 0.10654E+01_JPRB, 0.10906E+01_JPRB, 0.11013E+01_JPRB, 0.11127E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.11554E+01_JPRB, 0.11712E+01_JPRB, 0.12130E+01_JPRB, 0.12409E+01_JPRB, 0.12622E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.16536E+01_JPRB, 0.14958E+01_JPRB, 0.12775E+01_JPRB, 0.11683E+01_JPRB, 0.11500E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.11265E+01_JPRB, 0.10444E+01_JPRB, 0.10762E+01_JPRB, 0.11011E+01_JPRB, 0.10930E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.12120E+01_JPRB, 0.12544E+01_JPRB, 0.12958E+01_JPRB, 0.13199E+01_JPRB, 0.13504E+01_JPRB, 0.13418E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.13346E+01_JPRB, 0.13476E+01_JPRB, 0.12250E+01_JPRB, 0.11892E+01_JPRB, 0.11491E+01_JPRB, 0.97936E+00_JPRB ,&
&  0.83928E+00_JPRB, 0.73768E+00_JPRB, 0.70954E+00_JPRB, 0.78549E+00_JPRB, 0.76320E+00_JPRB, 0.69501E+00_JPRB, 0.64706E+00_JPRB ,&
&  0.60906E+00_JPRB, 0.57648E+00_JPRB, 0.58591E+00_JPRB, 0.57267E+00_JPRB, 0.53086E+00_JPRB, 0.54641E+00_JPRB, 0.56690E+00_JPRB ,&
&  0.55425E+00_JPRB, 0.56651E+00_JPRB, 0.58618E+00_JPRB, 0.58018E+00_JPRB, 0.58581E+00_JPRB, 0.61071E+00_JPRB, 0.60043E+00_JPRB ,&
&  0.66101E+00_JPRB, 0.61863E+00_JPRB, 0.63896E+00_JPRB, 0.69957E+00_JPRB, 0.70241E+00_JPRB, 0.91640E+00_JPRB, 0.12668E+01_JPRB ,&
&  0.18556E+01_JPRB, 0.20268E+01_JPRB, 0.26506E+01_JPRB, 0.41416E+01_JPRB, 0.31387E+01_JPRB, 0.20738E+01_JPRB, 0.77988E+00_JPRB ,&
&  0.77819E+00_JPRB, 0.76895E+00_JPRB, 0.78090E+00_JPRB, 0.77481E+00_JPRB, 0.77379E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.77620E+00_JPRB, 0.79956E+00_JPRB, 0.81835E+00_JPRB, 0.83696E+00_JPRB, 0.85315E+00_JPRB, 0.86824E+00_JPRB, 0.88564E+00_JPRB ,&
&  0.89681E+00_JPRB, 0.91407E+00_JPRB, 0.93527E+00_JPRB, 0.95643E+00_JPRB, 0.97336E+00_JPRB, 0.99873E+00_JPRB, 0.10196E+01_JPRB ,&
&  0.10502E+01_JPRB, 0.10702E+01_JPRB, 0.10872E+01_JPRB, 0.10862E+01_JPRB, 0.10804E+01_JPRB, 0.10853E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.10750E+01_JPRB, 0.10838E+01_JPRB, 0.11044E+01_JPRB, 0.11114E+01_JPRB, 0.11478E+01_JPRB, 0.98929E+00_JPRB ,&
&  0.12881E+01_JPRB, 0.14352E+01_JPRB, 0.13352E+01_JPRB, 0.11721E+01_JPRB, 0.10845E+01_JPRB, 0.10768E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.11239E+01_JPRB, 0.10406E+01_JPRB, 0.99819E+00_JPRB, 0.98630E+00_JPRB, 0.99062E+00_JPRB, 0.10151E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.10791E+01_JPRB, 0.11176E+01_JPRB, 0.11519E+01_JPRB, 0.11724E+01_JPRB, 0.11792E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.10837E+01_JPRB, 0.10034E+01_JPRB, 0.98651E+00_JPRB, 0.95249E+00_JPRB, 0.82241E+00_JPRB, 0.68453E+00_JPRB ,&
&  0.63203E+00_JPRB, 0.69463E+00_JPRB, 0.74686E+00_JPRB, 0.88206E+00_JPRB, 0.84009E+00_JPRB, 0.77377E+00_JPRB, 0.71114E+00_JPRB ,&
&  0.65260E+00_JPRB, 0.60711E+00_JPRB, 0.59984E+00_JPRB, 0.57310E+00_JPRB, 0.55683E+00_JPRB, 0.55078E+00_JPRB, 0.54425E+00_JPRB ,&
&  0.55414E+00_JPRB, 0.56258E+00_JPRB, 0.57624E+00_JPRB, 0.58136E+00_JPRB, 0.57261E+00_JPRB, 0.51636E+00_JPRB, 0.64499E+00_JPRB ,&
&  0.61821E+00_JPRB, 0.56245E+00_JPRB, 0.66038E+00_JPRB, 0.51383E+00_JPRB, 0.62576E+00_JPRB, 0.69071E+00_JPRB, 0.82099E+00_JPRB ,&
&  0.91086E+00_JPRB, 0.12840E+01_JPRB, 0.17746E+01_JPRB, 0.14071E+01_JPRB, 0.11204E+01_JPRB, 0.83638E+00_JPRB, 0.74293E+00_JPRB ,&
&  0.73824E+00_JPRB, 0.73938E+00_JPRB, 0.74611E+00_JPRB, 0.74682E+00_JPRB, 0.75535E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.78007E+00_JPRB, 0.80051E+00_JPRB, 0.81583E+00_JPRB, 0.83403E+00_JPRB, 0.85250E+00_JPRB, 0.86975E+00_JPRB, 0.88645E+00_JPRB ,&
&  0.90033E+00_JPRB, 0.91191E+00_JPRB, 0.92160E+00_JPRB, 0.93546E+00_JPRB, 0.95315E+00_JPRB, 0.96740E+00_JPRB, 0.98595E+00_JPRB ,&
&  0.10145E+01_JPRB, 0.10365E+01_JPRB, 0.10594E+01_JPRB, 0.10540E+01_JPRB, 0.10506E+01_JPRB, 0.10662E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.11004E+01_JPRB, 0.11238E+01_JPRB, 0.11508E+01_JPRB, 0.11837E+01_JPRB, 0.12391E+01_JPRB, 0.11584E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.12839E+01_JPRB, 0.12081E+01_JPRB, 0.10542E+01_JPRB, 0.98463E+00_JPRB, 0.95551E+00_JPRB, 0.94621E+00_JPRB ,&
&  0.94175E+00_JPRB, 0.90719E+00_JPRB, 0.88135E+00_JPRB, 0.85311E+00_JPRB, 0.84270E+00_JPRB, 0.84750E+00_JPRB, 0.85140E+00_JPRB ,&
&  0.85988E+00_JPRB, 0.86992E+00_JPRB, 0.89319E+00_JPRB, 0.92333E+00_JPRB, 0.95125E+00_JPRB, 0.93451E+00_JPRB, 0.90406E+00_JPRB ,&
&  0.85163E+00_JPRB, 0.80022E+00_JPRB, 0.65877E+00_JPRB, 0.61759E+00_JPRB, 0.58730E+00_JPRB, 0.53003E+00_JPRB, 0.47314E+00_JPRB ,&
&  0.52322E+00_JPRB, 0.60308E+00_JPRB, 0.75289E+00_JPRB, 0.92722E+00_JPRB, 0.87218E+00_JPRB, 0.81822E+00_JPRB, 0.75195E+00_JPRB ,&
&  0.68192E+00_JPRB, 0.63074E+00_JPRB, 0.60952E+00_JPRB, 0.57898E+00_JPRB, 0.56953E+00_JPRB, 0.57647E+00_JPRB, 0.55004E+00_JPRB ,&
&  0.54808E+00_JPRB, 0.54263E+00_JPRB, 0.54357E+00_JPRB, 0.54274E+00_JPRB, 0.56382E+00_JPRB, 0.51593E+00_JPRB, 0.60282E+00_JPRB ,&
&  0.61233E+00_JPRB, 0.47733E+00_JPRB, 0.50981E+00_JPRB, 0.59381E+00_JPRB, 0.58346E+00_JPRB, 0.59736E+00_JPRB, 0.67898E+00_JPRB ,&
&  0.83279E+00_JPRB, 0.94349E+00_JPRB, 0.96011E+00_JPRB, 0.90105E+00_JPRB, 0.80492E+00_JPRB, 0.74809E+00_JPRB, 0.71928E+00_JPRB ,&
&  0.72735E+00_JPRB, 0.73532E+00_JPRB, 0.74587E+00_JPRB, 0.74822E+00_JPRB, 0.75808E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.76455E+00_JPRB, 0.78651E+00_JPRB, 0.79927E+00_JPRB, 0.81237E+00_JPRB, 0.82919E+00_JPRB, 0.83752E+00_JPRB, 0.84444E+00_JPRB ,&
&  0.84660E+00_JPRB, 0.84332E+00_JPRB, 0.84964E+00_JPRB, 0.86235E+00_JPRB, 0.86845E+00_JPRB, 0.86622E+00_JPRB, 0.87171E+00_JPRB ,&
&  0.88399E+00_JPRB, 0.90380E+00_JPRB, 0.92430E+00_JPRB, 0.91623E+00_JPRB, 0.90993E+00_JPRB, 0.91973E+00_JPRB, 0.93509E+00_JPRB ,&
&  0.93445E+00_JPRB, 0.93072E+00_JPRB, 0.93803E+00_JPRB, 0.94732E+00_JPRB, 0.96436E+00_JPRB, 0.96742E+00_JPRB, 0.10676E+01_JPRB ,&
&  0.88911E+00_JPRB, 0.11650E+01_JPRB, 0.11078E+01_JPRB, 0.94393E+00_JPRB, 0.91516E+00_JPRB, 0.84405E+00_JPRB, 0.82295E+00_JPRB ,&
&  0.80717E+00_JPRB, 0.77969E+00_JPRB, 0.75277E+00_JPRB, 0.73060E+00_JPRB, 0.72563E+00_JPRB, 0.73064E+00_JPRB, 0.71676E+00_JPRB ,&
&  0.71491E+00_JPRB, 0.72397E+00_JPRB, 0.73846E+00_JPRB, 0.75440E+00_JPRB, 0.76051E+00_JPRB, 0.74890E+00_JPRB, 0.71315E+00_JPRB ,&
&  0.68452E+00_JPRB, 0.63485E+00_JPRB, 0.48614E+00_JPRB, 0.45793E+00_JPRB, 0.45617E+00_JPRB, 0.41057E+00_JPRB, 0.39361E+00_JPRB ,&
&  0.49005E+00_JPRB, 0.58972E+00_JPRB, 0.81240E+00_JPRB, 0.91331E+00_JPRB, 0.90125E+00_JPRB, 0.91731E+00_JPRB, 0.82121E+00_JPRB ,&
&  0.72780E+00_JPRB, 0.66132E+00_JPRB, 0.64552E+00_JPRB, 0.60712E+00_JPRB, 0.59372E+00_JPRB, 0.57999E+00_JPRB, 0.55656E+00_JPRB ,&
&  0.54386E+00_JPRB, 0.52844E+00_JPRB, 0.51935E+00_JPRB, 0.52437E+00_JPRB, 0.46761E+00_JPRB, 0.65469E+00_JPRB, 0.67589E+00_JPRB ,&
&  0.60087E+00_JPRB, 0.56265E+00_JPRB, 0.44267E+00_JPRB, 0.54870E+00_JPRB, 0.53246E+00_JPRB, 0.53366E+00_JPRB, 0.59557E+00_JPRB ,&
&  0.70835E+00_JPRB, 0.74762E+00_JPRB, 0.74816E+00_JPRB, 0.73499E+00_JPRB, 0.71006E+00_JPRB, 0.68772E+00_JPRB, 0.67869E+00_JPRB ,&
&  0.69303E+00_JPRB, 0.70693E+00_JPRB, 0.72267E+00_JPRB, 0.73053E+00_JPRB, 0.74294E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.74170E+00_JPRB, 0.75532E+00_JPRB, 0.75525E+00_JPRB, 0.76979E+00_JPRB, 0.78797E+00_JPRB, 0.79249E+00_JPRB, 0.79471E+00_JPRB ,&
&  0.78547E+00_JPRB, 0.78809E+00_JPRB, 0.78344E+00_JPRB, 0.79903E+00_JPRB, 0.79913E+00_JPRB, 0.80043E+00_JPRB, 0.80253E+00_JPRB ,&
&  0.81416E+00_JPRB, 0.82239E+00_JPRB, 0.84891E+00_JPRB, 0.85353E+00_JPRB, 0.85088E+00_JPRB, 0.85655E+00_JPRB, 0.87467E+00_JPRB ,&
&  0.89591E+00_JPRB, 0.92201E+00_JPRB, 0.95679E+00_JPRB, 0.98571E+00_JPRB, 0.10843E+01_JPRB, 0.12368E+01_JPRB, 0.14982E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.10130E+01_JPRB, 0.98344E+00_JPRB, 0.84340E+00_JPRB, 0.81378E+00_JPRB, 0.80129E+00_JPRB, 0.75808E+00_JPRB ,&
&  0.71015E+00_JPRB, 0.68381E+00_JPRB, 0.66290E+00_JPRB, 0.65407E+00_JPRB, 0.65455E+00_JPRB, 0.64557E+00_JPRB, 0.64680E+00_JPRB ,&
&  0.63369E+00_JPRB, 0.63110E+00_JPRB, 0.69804E+00_JPRB, 0.75356E+00_JPRB, 0.77066E+00_JPRB, 0.82942E+00_JPRB, 0.77736E+00_JPRB ,&
&  0.77046E+00_JPRB, 0.74414E+00_JPRB, 0.60058E+00_JPRB, 0.54605E+00_JPRB, 0.51742E+00_JPRB, 0.49254E+00_JPRB, 0.48108E+00_JPRB ,&
&  0.53892E+00_JPRB, 0.62901E+00_JPRB, 0.65783E+00_JPRB, 0.85834E+00_JPRB, 0.94752E+00_JPRB, 0.10075E+01_JPRB, 0.86102E+00_JPRB ,&
&  0.74171E+00_JPRB, 0.67347E+00_JPRB, 0.65876E+00_JPRB, 0.62619E+00_JPRB, 0.61433E+00_JPRB, 0.57665E+00_JPRB, 0.53964E+00_JPRB ,&
&  0.53367E+00_JPRB, 0.51299E+00_JPRB, 0.49623E+00_JPRB, 0.47636E+00_JPRB, 0.51436E+00_JPRB, 0.53705E+00_JPRB, 0.57336E+00_JPRB ,&
&  0.51908E+00_JPRB, 0.56261E+00_JPRB, 0.50841E+00_JPRB, 0.60257E+00_JPRB, 0.49717E+00_JPRB, 0.47413E+00_JPRB, 0.51255E+00_JPRB ,&
&  0.58124E+00_JPRB, 0.61684E+00_JPRB, 0.63032E+00_JPRB, 0.65580E+00_JPRB, 0.66174E+00_JPRB, 0.65369E+00_JPRB, 0.66066E+00_JPRB ,&
&  0.69577E+00_JPRB, 0.70111E+00_JPRB, 0.70664E+00_JPRB, 0.70770E+00_JPRB, 0.72437E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.68426E+00_JPRB, 0.69680E+00_JPRB, 0.69097E+00_JPRB, 0.69756E+00_JPRB, 0.70011E+00_JPRB, 0.71320E+00_JPRB, 0.71188E+00_JPRB ,&
&  0.70826E+00_JPRB, 0.71473E+00_JPRB, 0.69529E+00_JPRB, 0.69909E+00_JPRB, 0.69367E+00_JPRB, 0.69688E+00_JPRB, 0.68679E+00_JPRB ,&
&  0.68803E+00_JPRB, 0.70082E+00_JPRB, 0.70663E+00_JPRB, 0.71497E+00_JPRB, 0.71304E+00_JPRB, 0.69630E+00_JPRB, 0.67563E+00_JPRB ,&
&  0.70791E+00_JPRB, 0.79702E+00_JPRB, 0.89717E+00_JPRB, 0.96375E+00_JPRB, 0.11212E+01_JPRB, 0.10894E+01_JPRB, 0.11637E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.93322E+00_JPRB, 0.10118E+01_JPRB, 0.89302E+00_JPRB, 0.85687E+00_JPRB, 0.81255E+00_JPRB, 0.78243E+00_JPRB ,&
&  0.78316E+00_JPRB, 0.80069E+00_JPRB, 0.82720E+00_JPRB, 0.85977E+00_JPRB, 0.88829E+00_JPRB, 0.88583E+00_JPRB, 0.86845E+00_JPRB ,&
&  0.85701E+00_JPRB, 0.89433E+00_JPRB, 0.99911E+00_JPRB, 0.10027E+01_JPRB, 0.10339E+01_JPRB, 0.10658E+01_JPRB, 0.10589E+01_JPRB ,&
&  0.10445E+01_JPRB, 0.10202E+01_JPRB, 0.85560E+00_JPRB, 0.75693E+00_JPRB, 0.74835E+00_JPRB, 0.70038E+00_JPRB, 0.69915E+00_JPRB ,&
&  0.74949E+00_JPRB, 0.76935E+00_JPRB, 0.65035E+00_JPRB, 0.76469E+00_JPRB, 0.88249E+00_JPRB, 0.90082E+00_JPRB, 0.89739E+00_JPRB ,&
&  0.78443E+00_JPRB, 0.71964E+00_JPRB, 0.68949E+00_JPRB, 0.67116E+00_JPRB, 0.64131E+00_JPRB, 0.59588E+00_JPRB, 0.57095E+00_JPRB ,&
&  0.52809E+00_JPRB, 0.49767E+00_JPRB, 0.46861E+00_JPRB, 0.44980E+00_JPRB, 0.45171E+00_JPRB, 0.52367E+00_JPRB, 0.47768E+00_JPRB ,&
&  0.48388E+00_JPRB, 0.51401E+00_JPRB, 0.62566E+00_JPRB, 0.59168E+00_JPRB, 0.41183E+00_JPRB, 0.43627E+00_JPRB, 0.44438E+00_JPRB ,&
&  0.46802E+00_JPRB, 0.50012E+00_JPRB, 0.52403E+00_JPRB, 0.55483E+00_JPRB, 0.56999E+00_JPRB, 0.57049E+00_JPRB, 0.57908E+00_JPRB ,&
&  0.60327E+00_JPRB, 0.62744E+00_JPRB, 0.66180E+00_JPRB, 0.66691E+00_JPRB, 0.66847E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.58818E+00_JPRB, 0.59020E+00_JPRB, 0.59362E+00_JPRB, 0.59002E+00_JPRB, 0.58627E+00_JPRB, 0.58529E+00_JPRB, 0.58701E+00_JPRB ,&
&  0.58695E+00_JPRB, 0.57808E+00_JPRB, 0.56941E+00_JPRB, 0.56256E+00_JPRB, 0.54972E+00_JPRB, 0.52430E+00_JPRB, 0.48843E+00_JPRB ,&
&  0.49327E+00_JPRB, 0.50218E+00_JPRB, 0.51654E+00_JPRB, 0.55271E+00_JPRB, 0.63042E+00_JPRB, 0.74621E+00_JPRB, 0.89441E+00_JPRB ,&
&  0.11811E+01_JPRB, 0.14942E+01_JPRB, 0.17015E+01_JPRB, 0.17540E+01_JPRB, 0.19156E+01_JPRB, 0.12981E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.10154E+01_JPRB, 0.97869E+00_JPRB, 0.93100E+00_JPRB, 0.90425E+00_JPRB, 0.89358E+00_JPRB, 0.93206E+00_JPRB ,&
&  0.10050E+01_JPRB, 0.10734E+01_JPRB, 0.11320E+01_JPRB, 0.11881E+01_JPRB, 0.12300E+01_JPRB, 0.12706E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.12579E+01_JPRB, 0.12827E+01_JPRB, 0.12181E+01_JPRB, 0.12559E+01_JPRB, 0.12746E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.12220E+01_JPRB, 0.11489E+01_JPRB, 0.10977E+01_JPRB, 0.10190E+01_JPRB, 0.97234E+00_JPRB, 0.93128E+00_JPRB, 0.89309E+00_JPRB ,&
&  0.87451E+00_JPRB, 0.84766E+00_JPRB, 0.72250E+00_JPRB, 0.83453E+00_JPRB, 0.90349E+00_JPRB, 0.80506E+00_JPRB, 0.89403E+00_JPRB ,&
&  0.81134E+00_JPRB, 0.77857E+00_JPRB, 0.73681E+00_JPRB, 0.69990E+00_JPRB, 0.67261E+00_JPRB, 0.69970E+00_JPRB, 0.59940E+00_JPRB ,&
&  0.55239E+00_JPRB, 0.50250E+00_JPRB, 0.44749E+00_JPRB, 0.43785E+00_JPRB, 0.44708E+00_JPRB, 0.48444E+00_JPRB, 0.49486E+00_JPRB ,&
&  0.49210E+00_JPRB, 0.51016E+00_JPRB, 0.61740E+00_JPRB, 0.66677E+00_JPRB, 0.51122E+00_JPRB, 0.41057E+00_JPRB, 0.41224E+00_JPRB ,&
&  0.41393E+00_JPRB, 0.41998E+00_JPRB, 0.42849E+00_JPRB, 0.45080E+00_JPRB, 0.47167E+00_JPRB, 0.48863E+00_JPRB, 0.50190E+00_JPRB ,&
&  0.51008E+00_JPRB, 0.53098E+00_JPRB, 0.55128E+00_JPRB, 0.56842E+00_JPRB, 0.58402E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.50130E+00_JPRB, 0.50213E+00_JPRB, 0.51090E+00_JPRB, 0.51070E+00_JPRB, 0.52328E+00_JPRB, 0.52298E+00_JPRB, 0.52401E+00_JPRB ,&
&  0.53206E+00_JPRB, 0.54689E+00_JPRB, 0.53796E+00_JPRB, 0.51620E+00_JPRB, 0.51133E+00_JPRB, 0.52229E+00_JPRB, 0.55609E+00_JPRB ,&
&  0.63133E+00_JPRB, 0.70606E+00_JPRB, 0.77259E+00_JPRB, 0.87822E+00_JPRB, 0.10755E+01_JPRB, 0.13974E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.19199E+01_JPRB, 0.20518E+01_JPRB, 0.20261E+01_JPRB, 0.20947E+01_JPRB, 0.16486E+01_JPRB, 0.10095E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.11721E+01_JPRB, 0.10775E+01_JPRB, 0.10245E+01_JPRB, 0.99137E+00_JPRB, 0.10206E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.12405E+01_JPRB, 0.13022E+01_JPRB, 0.13458E+01_JPRB, 0.13673E+01_JPRB, 0.13950E+01_JPRB, 0.14090E+01_JPRB ,&
&  0.14421E+01_JPRB, 0.14429E+01_JPRB, 0.14335E+01_JPRB, 0.14107E+01_JPRB, 0.14032E+01_JPRB, 0.14308E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.14049E+01_JPRB, 0.13031E+01_JPRB, 0.12535E+01_JPRB, 0.12041E+01_JPRB, 0.11064E+01_JPRB, 0.10077E+01_JPRB, 0.97695E+00_JPRB ,&
&  0.95410E+00_JPRB, 0.95769E+00_JPRB, 0.80368E+00_JPRB, 0.97516E+00_JPRB, 0.10818E+01_JPRB, 0.98165E+00_JPRB, 0.92715E+00_JPRB ,&
&  0.84346E+00_JPRB, 0.80938E+00_JPRB, 0.73995E+00_JPRB, 0.72586E+00_JPRB, 0.73633E+00_JPRB, 0.80709E+00_JPRB, 0.61817E+00_JPRB ,&
&  0.62609E+00_JPRB, 0.52415E+00_JPRB, 0.50752E+00_JPRB, 0.51204E+00_JPRB, 0.51953E+00_JPRB, 0.49636E+00_JPRB, 0.48163E+00_JPRB ,&
&  0.47279E+00_JPRB, 0.58914E+00_JPRB, 0.77499E+00_JPRB, 0.11829E+01_JPRB, 0.15797E+01_JPRB, 0.43453E+00_JPRB, 0.40834E+00_JPRB ,&
&  0.41438E+00_JPRB, 0.41507E+00_JPRB, 0.41277E+00_JPRB, 0.42476E+00_JPRB, 0.43630E+00_JPRB, 0.44629E+00_JPRB, 0.46250E+00_JPRB ,&
&  0.47327E+00_JPRB, 0.48006E+00_JPRB, 0.48724E+00_JPRB, 0.48548E+00_JPRB, 0.49456E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.46926E+00_JPRB, 0.47827E+00_JPRB, 0.50481E+00_JPRB, 0.55380E+00_JPRB, 0.62091E+00_JPRB, 0.65349E+00_JPRB, 0.64675E+00_JPRB ,&
&  0.61626E+00_JPRB, 0.61826E+00_JPRB, 0.64238E+00_JPRB, 0.65086E+00_JPRB, 0.69489E+00_JPRB, 0.78164E+00_JPRB, 0.86799E+00_JPRB ,&
&  0.98550E+00_JPRB, 0.10913E+01_JPRB, 0.11905E+01_JPRB, 0.13447E+01_JPRB, 0.14918E+01_JPRB, 0.16644E+01_JPRB, 0.17634E+01_JPRB ,&
&  0.17352E+01_JPRB, 0.15160E+01_JPRB, 0.14124E+01_JPRB, 0.10904E+01_JPRB, 0.90114E+00_JPRB, 0.10265E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.10580E+01_JPRB, 0.10558E+01_JPRB, 0.10756E+01_JPRB, 0.11162E+01_JPRB, 0.11576E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12961E+01_JPRB, 0.13712E+01_JPRB, 0.14322E+01_JPRB, 0.14792E+01_JPRB, 0.14849E+01_JPRB, 0.15016E+01_JPRB, 0.15469E+01_JPRB ,&
&  0.16084E+01_JPRB, 0.16107E+01_JPRB, 0.16064E+01_JPRB, 0.16010E+01_JPRB, 0.15367E+01_JPRB, 0.15315E+01_JPRB, 0.15120E+01_JPRB ,&
&  0.14940E+01_JPRB, 0.14488E+01_JPRB, 0.14134E+01_JPRB, 0.13919E+01_JPRB, 0.13225E+01_JPRB, 0.11401E+01_JPRB, 0.10765E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.10109E+01_JPRB, 0.94044E+00_JPRB, 0.10718E+01_JPRB, 0.78262E+00_JPRB, 0.90268E+00_JPRB, 0.99410E+00_JPRB ,&
&  0.95501E+00_JPRB, 0.90442E+00_JPRB, 0.81985E+00_JPRB, 0.78806E+00_JPRB, 0.76761E+00_JPRB, 0.66922E+00_JPRB, 0.66930E+00_JPRB ,&
&  0.70490E+00_JPRB, 0.63655E+00_JPRB, 0.61083E+00_JPRB, 0.61066E+00_JPRB, 0.62891E+00_JPRB, 0.58566E+00_JPRB, 0.57644E+00_JPRB ,&
&  0.55681E+00_JPRB, 0.63858E+00_JPRB, 0.64460E+00_JPRB, 0.77126E+00_JPRB, 0.77443E+00_JPRB, 0.42012E+00_JPRB, 0.41987E+00_JPRB ,&
&  0.43102E+00_JPRB, 0.44112E+00_JPRB, 0.44358E+00_JPRB, 0.46660E+00_JPRB, 0.47939E+00_JPRB, 0.47626E+00_JPRB, 0.48216E+00_JPRB ,&
&  0.48927E+00_JPRB, 0.48810E+00_JPRB, 0.48635E+00_JPRB, 0.48370E+00_JPRB, 0.47986E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.52545E+00_JPRB, 0.51960E+00_JPRB, 0.54152E+00_JPRB, 0.63646E+00_JPRB, 0.73433E+00_JPRB, 0.79255E+00_JPRB, 0.76935E+00_JPRB ,&
&  0.72013E+00_JPRB, 0.67535E+00_JPRB, 0.71366E+00_JPRB, 0.76154E+00_JPRB, 0.83782E+00_JPRB, 0.93176E+00_JPRB, 0.10335E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.12488E+01_JPRB, 0.13190E+01_JPRB, 0.13998E+01_JPRB, 0.15040E+01_JPRB, 0.16102E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.13510E+01_JPRB, 0.10276E+01_JPRB, 0.12239E+01_JPRB, 0.11333E+01_JPRB, 0.10950E+01_JPRB, 0.10893E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.11022E+01_JPRB, 0.11190E+01_JPRB, 0.11583E+01_JPRB, 0.11942E+01_JPRB, 0.11891E+01_JPRB, 0.12367E+01_JPRB, 0.13069E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.14621E+01_JPRB, 0.14984E+01_JPRB, 0.15304E+01_JPRB, 0.15451E+01_JPRB, 0.15701E+01_JPRB, 0.16151E+01_JPRB ,&
&  0.16578E+01_JPRB, 0.16866E+01_JPRB, 0.16724E+01_JPRB, 0.16146E+01_JPRB, 0.15843E+01_JPRB, 0.15789E+01_JPRB, 0.15414E+01_JPRB ,&
&  0.15257E+01_JPRB, 0.14981E+01_JPRB, 0.15429E+01_JPRB, 0.15883E+01_JPRB, 0.15765E+01_JPRB, 0.13519E+01_JPRB, 0.13015E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11237E+01_JPRB, 0.10852E+01_JPRB, 0.11756E+01_JPRB, 0.89802E+00_JPRB, 0.96101E+00_JPRB, 0.96341E+00_JPRB ,&
&  0.10396E+01_JPRB, 0.99939E+00_JPRB, 0.94684E+00_JPRB, 0.88493E+00_JPRB, 0.88533E+00_JPRB, 0.75429E+00_JPRB, 0.71905E+00_JPRB ,&
&  0.71191E+00_JPRB, 0.75572E+00_JPRB, 0.70008E+00_JPRB, 0.68502E+00_JPRB, 0.64244E+00_JPRB, 0.67416E+00_JPRB, 0.71616E+00_JPRB ,&
&  0.81238E+00_JPRB, 0.75851E+00_JPRB, 0.70535E+00_JPRB, 0.64073E+00_JPRB, 0.51404E+00_JPRB, 0.44174E+00_JPRB, 0.44725E+00_JPRB ,&
&  0.46451E+00_JPRB, 0.48240E+00_JPRB, 0.48940E+00_JPRB, 0.50958E+00_JPRB, 0.52509E+00_JPRB, 0.52094E+00_JPRB, 0.51418E+00_JPRB ,&
&  0.51583E+00_JPRB, 0.53160E+00_JPRB, 0.52587E+00_JPRB, 0.52704E+00_JPRB, 0.53082E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.60237E+00_JPRB, 0.59995E+00_JPRB, 0.61770E+00_JPRB, 0.70458E+00_JPRB, 0.82924E+00_JPRB, 0.93590E+00_JPRB, 0.95932E+00_JPRB ,&
&  0.95997E+00_JPRB, 0.78650E+00_JPRB, 0.81652E+00_JPRB, 0.83668E+00_JPRB, 0.92458E+00_JPRB, 0.10230E+01_JPRB, 0.11274E+01_JPRB ,&
&  0.12274E+01_JPRB, 0.13334E+01_JPRB, 0.14103E+01_JPRB, 0.14799E+01_JPRB, 0.15580E+01_JPRB, 0.16316E+01_JPRB, 0.14106E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.12100E+01_JPRB, 0.17463E+01_JPRB, 0.13695E+01_JPRB, 0.11639E+01_JPRB, 0.11570E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.12060E+01_JPRB, 0.12339E+01_JPRB, 0.12809E+01_JPRB, 0.12797E+01_JPRB, 0.13366E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.14697E+01_JPRB, 0.15233E+01_JPRB, 0.15423E+01_JPRB, 0.15449E+01_JPRB, 0.15552E+01_JPRB, 0.15815E+01_JPRB, 0.16245E+01_JPRB ,&
&  0.16656E+01_JPRB, 0.16955E+01_JPRB, 0.16848E+01_JPRB, 0.16009E+01_JPRB, 0.15862E+01_JPRB, 0.16105E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.15969E+01_JPRB, 0.16612E+01_JPRB, 0.16837E+01_JPRB, 0.16217E+01_JPRB, 0.14909E+01_JPRB, 0.16117E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.12257E+01_JPRB, 0.12608E+01_JPRB, 0.10195E+01_JPRB, 0.10445E+01_JPRB, 0.10766E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.10372E+01_JPRB, 0.10064E+01_JPRB, 0.97423E+00_JPRB, 0.95630E+00_JPRB, 0.83579E+00_JPRB, 0.80712E+00_JPRB ,&
&  0.76771E+00_JPRB, 0.83186E+00_JPRB, 0.71457E+00_JPRB, 0.65432E+00_JPRB, 0.58997E+00_JPRB, 0.63230E+00_JPRB, 0.73563E+00_JPRB ,&
&  0.88144E+00_JPRB, 0.93510E+00_JPRB, 0.90601E+00_JPRB, 0.81714E+00_JPRB, 0.65230E+00_JPRB, 0.55143E+00_JPRB, 0.49573E+00_JPRB ,&
&  0.49287E+00_JPRB, 0.50198E+00_JPRB, 0.52087E+00_JPRB, 0.53762E+00_JPRB, 0.55573E+00_JPRB, 0.57141E+00_JPRB, 0.57926E+00_JPRB ,&
&  0.58215E+00_JPRB, 0.58057E+00_JPRB, 0.57613E+00_JPRB, 0.60045E+00_JPRB, 0.61466E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.65305E+00_JPRB, 0.63614E+00_JPRB, 0.62051E+00_JPRB, 0.64908E+00_JPRB, 0.71190E+00_JPRB, 0.73680E+00_JPRB, 0.74250E+00_JPRB ,&
&  0.73693E+00_JPRB, 0.72664E+00_JPRB, 0.75918E+00_JPRB, 0.77412E+00_JPRB, 0.86006E+00_JPRB, 0.96337E+00_JPRB, 0.10482E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.12224E+01_JPRB, 0.13161E+01_JPRB, 0.14299E+01_JPRB, 0.14890E+01_JPRB, 0.15460E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.95006E+00_JPRB, 0.19340E+01_JPRB, 0.17095E+01_JPRB, 0.15539E+01_JPRB, 0.12919E+01_JPRB, 0.12515E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.12121E+01_JPRB, 0.12542E+01_JPRB, 0.12978E+01_JPRB, 0.13150E+01_JPRB, 0.13531E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.14484E+01_JPRB, 0.15030E+01_JPRB, 0.15232E+01_JPRB, 0.15349E+01_JPRB, 0.15507E+01_JPRB, 0.15557E+01_JPRB, 0.15997E+01_JPRB ,&
&  0.16598E+01_JPRB, 0.17069E+01_JPRB, 0.17101E+01_JPRB, 0.17125E+01_JPRB, 0.16655E+01_JPRB, 0.17243E+01_JPRB, 0.18129E+01_JPRB ,&
&  0.17700E+01_JPRB, 0.17213E+01_JPRB, 0.18681E+01_JPRB, 0.18351E+01_JPRB, 0.17394E+01_JPRB, 0.17769E+01_JPRB, 0.15417E+01_JPRB ,&
&  0.15432E+01_JPRB, 0.13182E+01_JPRB, 0.13519E+01_JPRB, 0.10764E+01_JPRB, 0.10983E+01_JPRB, 0.11738E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.10857E+01_JPRB, 0.10517E+01_JPRB, 0.10475E+01_JPRB, 0.10223E+01_JPRB, 0.94518E+00_JPRB, 0.85548E+00_JPRB ,&
&  0.86238E+00_JPRB, 0.86863E+00_JPRB, 0.79755E+00_JPRB, 0.62193E+00_JPRB, 0.56989E+00_JPRB, 0.56449E+00_JPRB, 0.73750E+00_JPRB ,&
&  0.89123E+00_JPRB, 0.95238E+00_JPRB, 0.10208E+01_JPRB, 0.10665E+01_JPRB, 0.92724E+00_JPRB, 0.82238E+00_JPRB, 0.67893E+00_JPRB ,&
&  0.55927E+00_JPRB, 0.51711E+00_JPRB, 0.54346E+00_JPRB, 0.53948E+00_JPRB, 0.55753E+00_JPRB, 0.57905E+00_JPRB, 0.59677E+00_JPRB ,&
&  0.61299E+00_JPRB, 0.62758E+00_JPRB, 0.64026E+00_JPRB, 0.64233E+00_JPRB, 0.65275E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.64657E+00_JPRB, 0.63542E+00_JPRB, 0.62399E+00_JPRB, 0.63141E+00_JPRB, 0.69605E+00_JPRB, 0.71822E+00_JPRB, 0.68753E+00_JPRB ,&
&  0.66257E+00_JPRB, 0.66086E+00_JPRB, 0.68820E+00_JPRB, 0.67444E+00_JPRB, 0.71260E+00_JPRB, 0.81629E+00_JPRB, 0.89181E+00_JPRB ,&
&  0.96626E+00_JPRB, 0.10618E+01_JPRB, 0.11917E+01_JPRB, 0.13458E+01_JPRB, 0.14717E+01_JPRB, 0.16198E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.17096E+01_JPRB, 0.18039E+01_JPRB, 0.18047E+01_JPRB, 0.15953E+01_JPRB, 0.14633E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.13344E+01_JPRB, 0.12575E+01_JPRB, 0.12014E+01_JPRB, 0.11852E+01_JPRB, 0.11794E+01_JPRB, 0.12392E+01_JPRB, 0.12588E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.12597E+01_JPRB, 0.13387E+01_JPRB, 0.14025E+01_JPRB, 0.14520E+01_JPRB, 0.14941E+01_JPRB, 0.15474E+01_JPRB ,&
&  0.16298E+01_JPRB, 0.17126E+01_JPRB, 0.18297E+01_JPRB, 0.17934E+01_JPRB, 0.18267E+01_JPRB, 0.18908E+01_JPRB, 0.20281E+01_JPRB ,&
&  0.19845E+01_JPRB, 0.19364E+01_JPRB, 0.21846E+01_JPRB, 0.24052E+01_JPRB, 0.23487E+01_JPRB, 0.20459E+01_JPRB, 0.17384E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.14527E+01_JPRB, 0.13118E+01_JPRB, 0.11555E+01_JPRB, 0.12043E+01_JPRB, 0.13026E+01_JPRB, 0.13256E+01_JPRB ,&
&  0.12697E+01_JPRB, 0.11917E+01_JPRB, 0.11383E+01_JPRB, 0.11118E+01_JPRB, 0.10864E+01_JPRB, 0.10014E+01_JPRB, 0.97306E+00_JPRB ,&
&  0.10718E+01_JPRB, 0.11907E+01_JPRB, 0.10183E+01_JPRB, 0.73859E+00_JPRB, 0.59852E+00_JPRB, 0.48466E+00_JPRB, 0.56428E+00_JPRB ,&
&  0.84796E+00_JPRB, 0.90697E+00_JPRB, 0.94797E+00_JPRB, 0.11856E+01_JPRB, 0.11646E+01_JPRB, 0.11082E+01_JPRB, 0.96842E+00_JPRB ,&
&  0.76659E+00_JPRB, 0.66378E+00_JPRB, 0.61479E+00_JPRB, 0.57439E+00_JPRB, 0.57365E+00_JPRB, 0.58098E+00_JPRB, 0.59848E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.62557E+00_JPRB, 0.62910E+00_JPRB, 0.63087E+00_JPRB, 0.64509E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.65018E+00_JPRB, 0.63978E+00_JPRB, 0.64389E+00_JPRB, 0.64107E+00_JPRB, 0.68262E+00_JPRB, 0.70987E+00_JPRB, 0.64834E+00_JPRB ,&
&  0.63237E+00_JPRB, 0.62537E+00_JPRB, 0.63918E+00_JPRB, 0.63443E+00_JPRB, 0.63868E+00_JPRB, 0.68130E+00_JPRB, 0.73313E+00_JPRB ,&
&  0.80174E+00_JPRB, 0.88321E+00_JPRB, 0.10229E+01_JPRB, 0.11875E+01_JPRB, 0.12582E+01_JPRB, 0.12058E+01_JPRB, 0.12718E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.18419E+01_JPRB, 0.19570E+01_JPRB, 0.21670E+01_JPRB, 0.21628E+01_JPRB, 0.22490E+01_JPRB, 0.17957E+01_JPRB ,&
&  0.16451E+01_JPRB, 0.15089E+01_JPRB, 0.13362E+01_JPRB, 0.12199E+01_JPRB, 0.11153E+01_JPRB, 0.11017E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.11294E+01_JPRB, 0.11684E+01_JPRB, 0.12059E+01_JPRB, 0.12805E+01_JPRB, 0.13478E+01_JPRB, 0.14331E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.16863E+01_JPRB, 0.18260E+01_JPRB, 0.18944E+01_JPRB, 0.19253E+01_JPRB, 0.20121E+01_JPRB, 0.22212E+01_JPRB ,&
&  0.23275E+01_JPRB, 0.24348E+01_JPRB, 0.25234E+01_JPRB, 0.27147E+01_JPRB, 0.27627E+01_JPRB, 0.25221E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.18120E+01_JPRB, 0.15492E+01_JPRB, 0.12707E+01_JPRB, 0.12511E+01_JPRB, 0.13445E+01_JPRB, 0.14831E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.10473E+01_JPRB, 0.10330E+01_JPRB, 0.11085E+01_JPRB, 0.11965E+01_JPRB, 0.11583E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.73227E+00_JPRB, 0.60043E+00_JPRB, 0.71594E+00_JPRB, 0.54592E+00_JPRB, 0.42863E+00_JPRB, 0.57926E+00_JPRB ,&
&  0.78954E+00_JPRB, 0.98321E+00_JPRB, 0.10185E+01_JPRB, 0.10343E+01_JPRB, 0.12583E+01_JPRB, 0.13378E+01_JPRB, 0.12995E+01_JPRB ,&
&  0.93278E+00_JPRB, 0.79869E+00_JPRB, 0.72200E+00_JPRB, 0.66299E+00_JPRB, 0.61975E+00_JPRB, 0.60539E+00_JPRB, 0.60378E+00_JPRB ,&
&  0.59429E+00_JPRB, 0.59920E+00_JPRB, 0.60508E+00_JPRB, 0.61498E+00_JPRB, 0.63623E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.63031E+00_JPRB, 0.63400E+00_JPRB, 0.64651E+00_JPRB, 0.65716E+00_JPRB, 0.64574E+00_JPRB, 0.66377E+00_JPRB, 0.65617E+00_JPRB ,&
&  0.62705E+00_JPRB, 0.59899E+00_JPRB, 0.59426E+00_JPRB, 0.60340E+00_JPRB, 0.61436E+00_JPRB, 0.62222E+00_JPRB, 0.64444E+00_JPRB ,&
&  0.68436E+00_JPRB, 0.75106E+00_JPRB, 0.89532E+00_JPRB, 0.10329E+01_JPRB, 0.10929E+01_JPRB, 0.10351E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.19440E+01_JPRB, 0.23754E+01_JPRB, 0.25561E+01_JPRB, 0.24983E+01_JPRB, 0.24629E+01_JPRB, 0.21481E+01_JPRB ,&
&  0.18871E+01_JPRB, 0.17099E+01_JPRB, 0.15421E+01_JPRB, 0.13990E+01_JPRB, 0.12993E+01_JPRB, 0.11925E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.10859E+01_JPRB, 0.11063E+01_JPRB, 0.11163E+01_JPRB, 0.11705E+01_JPRB, 0.12288E+01_JPRB, 0.13221E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.16346E+01_JPRB, 0.18085E+01_JPRB, 0.20478E+01_JPRB, 0.17216E+01_JPRB, 0.20621E+01_JPRB, 0.21867E+01_JPRB ,&
&  0.23639E+01_JPRB, 0.24528E+01_JPRB, 0.24052E+01_JPRB, 0.30507E+01_JPRB, 0.32924E+01_JPRB, 0.28612E+01_JPRB, 0.23882E+01_JPRB ,&
&  0.20573E+01_JPRB, 0.16922E+01_JPRB, 0.14353E+01_JPRB, 0.14589E+01_JPRB, 0.15110E+01_JPRB, 0.15704E+01_JPRB, 0.11477E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.12183E+01_JPRB, 0.11803E+01_JPRB, 0.10109E+01_JPRB, 0.11574E+01_JPRB, 0.13180E+01_JPRB, 0.95303E+00_JPRB ,&
&  0.50061E+00_JPRB, 0.43782E+00_JPRB, 0.46246E+00_JPRB, 0.46482E+00_JPRB, 0.43770E+00_JPRB, 0.45650E+00_JPRB, 0.88796E+00_JPRB ,&
&  0.76562E+00_JPRB, 0.10123E+01_JPRB, 0.96569E+00_JPRB, 0.99874E+00_JPRB, 0.11520E+01_JPRB, 0.13966E+01_JPRB, 0.16402E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.91481E+00_JPRB, 0.78381E+00_JPRB, 0.71055E+00_JPRB, 0.65318E+00_JPRB, 0.62143E+00_JPRB, 0.60787E+00_JPRB ,&
&  0.59463E+00_JPRB, 0.58627E+00_JPRB, 0.58500E+00_JPRB, 0.59680E+00_JPRB, 0.61355E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.64167E+00_JPRB, 0.64254E+00_JPRB, 0.64080E+00_JPRB, 0.64000E+00_JPRB, 0.62068E+00_JPRB, 0.62798E+00_JPRB, 0.65541E+00_JPRB ,&
&  0.64170E+00_JPRB, 0.60455E+00_JPRB, 0.58635E+00_JPRB, 0.58999E+00_JPRB, 0.60447E+00_JPRB, 0.61469E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.69536E+00_JPRB, 0.88606E+00_JPRB, 0.89760E+00_JPRB, 0.95910E+00_JPRB, 0.92194E+00_JPRB, 0.11970E+01_JPRB ,&
&  0.15953E+01_JPRB, 0.20354E+01_JPRB, 0.24346E+01_JPRB, 0.26221E+01_JPRB, 0.29603E+01_JPRB, 0.30135E+01_JPRB, 0.28790E+01_JPRB ,&
&  0.24042E+01_JPRB, 0.20636E+01_JPRB, 0.18052E+01_JPRB, 0.16245E+01_JPRB, 0.14793E+01_JPRB, 0.13170E+01_JPRB, 0.12264E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.10918E+01_JPRB, 0.10508E+01_JPRB, 0.10593E+01_JPRB, 0.10887E+01_JPRB, 0.11254E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.15426E+01_JPRB, 0.17615E+01_JPRB, 0.20451E+01_JPRB, 0.18180E+01_JPRB, 0.16438E+01_JPRB, 0.19721E+01_JPRB ,&
&  0.23217E+01_JPRB, 0.26123E+01_JPRB, 0.27060E+01_JPRB, 0.36041E+01_JPRB, 0.36397E+01_JPRB, 0.30882E+01_JPRB, 0.27793E+01_JPRB ,&
&  0.23056E+01_JPRB, 0.21171E+01_JPRB, 0.16553E+01_JPRB, 0.16510E+01_JPRB, 0.17094E+01_JPRB, 0.13382E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.13579E+01_JPRB, 0.13029E+01_JPRB, 0.12012E+01_JPRB, 0.99588E+00_JPRB, 0.11618E+01_JPRB, 0.10907E+01_JPRB, 0.60759E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.51271E+00_JPRB, 0.50012E+00_JPRB, 0.47927E+00_JPRB, 0.47277E+00_JPRB, 0.50029E+00_JPRB, 0.72059E+00_JPRB ,&
&  0.72048E+00_JPRB, 0.95238E+00_JPRB, 0.95832E+00_JPRB, 0.96826E+00_JPRB, 0.10173E+01_JPRB, 0.13241E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.11333E+01_JPRB, 0.98357E+00_JPRB, 0.80437E+00_JPRB, 0.72657E+00_JPRB, 0.66120E+00_JPRB, 0.61990E+00_JPRB, 0.62378E+00_JPRB ,&
&  0.61409E+00_JPRB, 0.59468E+00_JPRB, 0.59132E+00_JPRB, 0.60388E+00_JPRB, 0.62079E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.64718E+00_JPRB, 0.65077E+00_JPRB, 0.65228E+00_JPRB, 0.66636E+00_JPRB, 0.65772E+00_JPRB, 0.64544E+00_JPRB, 0.66427E+00_JPRB ,&
&  0.64952E+00_JPRB, 0.62633E+00_JPRB, 0.60494E+00_JPRB, 0.60296E+00_JPRB, 0.59780E+00_JPRB, 0.61027E+00_JPRB, 0.61319E+00_JPRB ,&
&  0.63211E+00_JPRB, 0.71521E+00_JPRB, 0.83734E+00_JPRB, 0.80820E+00_JPRB, 0.78182E+00_JPRB, 0.77545E+00_JPRB, 0.94212E+00_JPRB ,&
&  0.14771E+01_JPRB, 0.20409E+01_JPRB, 0.24426E+01_JPRB, 0.28455E+01_JPRB, 0.32228E+01_JPRB, 0.31796E+01_JPRB, 0.32654E+01_JPRB ,&
&  0.28809E+01_JPRB, 0.24293E+01_JPRB, 0.21202E+01_JPRB, 0.18675E+01_JPRB, 0.17354E+01_JPRB, 0.16110E+01_JPRB, 0.14806E+01_JPRB ,&
&  0.13042E+01_JPRB, 0.11675E+01_JPRB, 0.10898E+01_JPRB, 0.10306E+01_JPRB, 0.10139E+01_JPRB, 0.10409E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.12026E+01_JPRB, 0.14154E+01_JPRB, 0.16324E+01_JPRB, 0.19388E+01_JPRB, 0.21087E+01_JPRB, 0.20916E+01_JPRB, 0.20084E+01_JPRB ,&
&  0.20592E+01_JPRB, 0.21945E+01_JPRB, 0.32327E+01_JPRB, 0.38282E+01_JPRB, 0.36317E+01_JPRB, 0.32589E+01_JPRB, 0.29091E+01_JPRB ,&
&  0.24391E+01_JPRB, 0.22315E+01_JPRB, 0.18155E+01_JPRB, 0.18594E+01_JPRB, 0.16439E+01_JPRB, 0.12954E+01_JPRB, 0.14361E+01_JPRB ,&
&  0.15740E+01_JPRB, 0.14580E+01_JPRB, 0.12406E+01_JPRB, 0.10415E+01_JPRB, 0.92781E+00_JPRB, 0.75198E+00_JPRB, 0.59383E+00_JPRB ,&
&  0.55869E+00_JPRB, 0.55056E+00_JPRB, 0.51811E+00_JPRB, 0.52441E+00_JPRB, 0.51477E+00_JPRB, 0.55715E+00_JPRB, 0.66691E+00_JPRB ,&
&  0.81395E+00_JPRB, 0.89363E+00_JPRB, 0.94447E+00_JPRB, 0.95637E+00_JPRB, 0.95296E+00_JPRB, 0.10547E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.10591E+01_JPRB, 0.99534E+00_JPRB, 0.85576E+00_JPRB, 0.74373E+00_JPRB, 0.68865E+00_JPRB, 0.64280E+00_JPRB, 0.61563E+00_JPRB ,&
&  0.60746E+00_JPRB, 0.61822E+00_JPRB, 0.62565E+00_JPRB, 0.62456E+00_JPRB, 0.61865E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.64978E+00_JPRB, 0.66561E+00_JPRB, 0.67464E+00_JPRB, 0.67932E+00_JPRB, 0.67235E+00_JPRB, 0.66653E+00_JPRB, 0.67239E+00_JPRB ,&
&  0.65960E+00_JPRB, 0.64532E+00_JPRB, 0.63120E+00_JPRB, 0.62625E+00_JPRB, 0.61605E+00_JPRB, 0.61585E+00_JPRB, 0.60735E+00_JPRB ,&
&  0.62312E+00_JPRB, 0.75151E+00_JPRB, 0.65267E+00_JPRB, 0.64909E+00_JPRB, 0.69300E+00_JPRB, 0.72957E+00_JPRB, 0.81667E+00_JPRB ,&
&  0.13562E+01_JPRB, 0.20603E+01_JPRB, 0.26048E+01_JPRB, 0.33501E+01_JPRB, 0.40653E+01_JPRB, 0.37685E+01_JPRB, 0.35707E+01_JPRB ,&
&  0.34056E+01_JPRB, 0.28462E+01_JPRB, 0.24909E+01_JPRB, 0.22155E+01_JPRB, 0.19967E+01_JPRB, 0.18347E+01_JPRB, 0.16404E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.12745E+01_JPRB, 0.11624E+01_JPRB, 0.10785E+01_JPRB, 0.10206E+01_JPRB, 0.10492E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.11026E+01_JPRB, 0.12636E+01_JPRB, 0.14889E+01_JPRB, 0.17363E+01_JPRB, 0.18608E+01_JPRB, 0.19729E+01_JPRB, 0.26895E+01_JPRB ,&
&  0.27301E+01_JPRB, 0.28864E+01_JPRB, 0.33936E+01_JPRB, 0.47483E+01_JPRB, 0.38266E+01_JPRB, 0.30584E+01_JPRB, 0.29676E+01_JPRB ,&
&  0.20873E+01_JPRB, 0.17602E+01_JPRB, 0.17497E+01_JPRB, 0.17546E+01_JPRB, 0.15203E+01_JPRB, 0.16070E+01_JPRB, 0.22951E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.16931E+01_JPRB, 0.17010E+01_JPRB, 0.17465E+01_JPRB, 0.90311E+00_JPRB, 0.68857E+00_JPRB, 0.99595E+00_JPRB ,&
&  0.10564E+01_JPRB, 0.72702E+00_JPRB, 0.57846E+00_JPRB, 0.71301E+00_JPRB, 0.58121E+00_JPRB, 0.63111E+00_JPRB, 0.77377E+00_JPRB ,&
&  0.78097E+00_JPRB, 0.79298E+00_JPRB, 0.95187E+00_JPRB, 0.92517E+00_JPRB, 0.91234E+00_JPRB, 0.96057E+00_JPRB, 0.10577E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10491E+01_JPRB, 0.98402E+00_JPRB, 0.84175E+00_JPRB, 0.76886E+00_JPRB, 0.71685E+00_JPRB, 0.68112E+00_JPRB ,&
&  0.64060E+00_JPRB, 0.62611E+00_JPRB, 0.63786E+00_JPRB, 0.64360E+00_JPRB, 0.62837E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.65780E+00_JPRB, 0.65389E+00_JPRB, 0.66539E+00_JPRB, 0.66688E+00_JPRB, 0.67483E+00_JPRB, 0.67756E+00_JPRB, 0.68191E+00_JPRB ,&
&  0.66682E+00_JPRB, 0.64970E+00_JPRB, 0.65203E+00_JPRB, 0.65237E+00_JPRB, 0.64018E+00_JPRB, 0.62627E+00_JPRB, 0.61217E+00_JPRB ,&
&  0.62752E+00_JPRB, 0.65375E+00_JPRB, 0.58357E+00_JPRB, 0.58366E+00_JPRB, 0.67828E+00_JPRB, 0.70639E+00_JPRB, 0.81853E+00_JPRB ,&
&  0.13068E+01_JPRB, 0.20029E+01_JPRB, 0.26911E+01_JPRB, 0.35962E+01_JPRB, 0.41955E+01_JPRB, 0.41446E+01_JPRB, 0.38219E+01_JPRB ,&
&  0.37741E+01_JPRB, 0.30757E+01_JPRB, 0.26823E+01_JPRB, 0.24460E+01_JPRB, 0.21744E+01_JPRB, 0.19197E+01_JPRB, 0.17029E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.13232E+01_JPRB, 0.12142E+01_JPRB, 0.11263E+01_JPRB, 0.10967E+01_JPRB, 0.10952E+01_JPRB, 0.10783E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.11022E+01_JPRB, 0.13633E+01_JPRB, 0.16168E+01_JPRB, 0.16908E+01_JPRB, 0.17995E+01_JPRB, 0.23278E+01_JPRB ,&
&  0.27649E+01_JPRB, 0.28731E+01_JPRB, 0.34887E+01_JPRB, 0.42313E+01_JPRB, 0.41752E+01_JPRB, 0.33725E+01_JPRB, 0.34295E+01_JPRB ,&
&  0.25038E+01_JPRB, 0.20447E+01_JPRB, 0.17586E+01_JPRB, 0.14867E+01_JPRB, 0.15813E+01_JPRB, 0.23756E+01_JPRB, 0.22103E+01_JPRB ,&
&  0.19969E+01_JPRB, 0.18122E+01_JPRB, 0.17492E+01_JPRB, 0.15051E+01_JPRB, 0.12672E+01_JPRB, 0.91264E+00_JPRB, 0.11178E+01_JPRB ,&
&  0.12022E+01_JPRB, 0.12023E+01_JPRB, 0.10766E+01_JPRB, 0.92647E+00_JPRB, 0.76439E+00_JPRB, 0.83851E+00_JPRB, 0.81936E+00_JPRB ,&
&  0.78572E+00_JPRB, 0.74521E+00_JPRB, 0.80846E+00_JPRB, 0.85015E+00_JPRB, 0.88209E+00_JPRB, 0.86474E+00_JPRB, 0.96059E+00_JPRB ,&
&  0.97916E+00_JPRB, 0.10667E+01_JPRB, 0.10475E+01_JPRB, 0.92411E+00_JPRB, 0.83472E+00_JPRB, 0.79108E+00_JPRB, 0.75600E+00_JPRB ,&
&  0.72162E+00_JPRB, 0.67410E+00_JPRB, 0.65913E+00_JPRB, 0.64647E+00_JPRB, 0.63921E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.66709E+00_JPRB, 0.66905E+00_JPRB, 0.66478E+00_JPRB, 0.65728E+00_JPRB, 0.64957E+00_JPRB, 0.64626E+00_JPRB, 0.65185E+00_JPRB ,&
&  0.64530E+00_JPRB, 0.62528E+00_JPRB, 0.62399E+00_JPRB, 0.62513E+00_JPRB, 0.61734E+00_JPRB, 0.60297E+00_JPRB, 0.59748E+00_JPRB ,&
&  0.61598E+00_JPRB, 0.61984E+00_JPRB, 0.54462E+00_JPRB, 0.59574E+00_JPRB, 0.64823E+00_JPRB, 0.67600E+00_JPRB, 0.83269E+00_JPRB ,&
&  0.10826E+01_JPRB, 0.15907E+01_JPRB, 0.21749E+01_JPRB, 0.30187E+01_JPRB, 0.37801E+01_JPRB, 0.41341E+01_JPRB, 0.41691E+01_JPRB ,&
&  0.36983E+01_JPRB, 0.33558E+01_JPRB, 0.28888E+01_JPRB, 0.25110E+01_JPRB, 0.21976E+01_JPRB, 0.19685E+01_JPRB, 0.17410E+01_JPRB ,&
&  0.15548E+01_JPRB, 0.14049E+01_JPRB, 0.13030E+01_JPRB, 0.12065E+01_JPRB, 0.11635E+01_JPRB, 0.11499E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10287E+01_JPRB, 0.12162E+01_JPRB, 0.15164E+01_JPRB, 0.15770E+01_JPRB, 0.17293E+01_JPRB, 0.19359E+01_JPRB ,&
&  0.28276E+01_JPRB, 0.32536E+01_JPRB, 0.33916E+01_JPRB, 0.45834E+01_JPRB, 0.31014E+01_JPRB, 0.35370E+01_JPRB, 0.39392E+01_JPRB ,&
&  0.36984E+01_JPRB, 0.32495E+01_JPRB, 0.30587E+01_JPRB, 0.23371E+01_JPRB, 0.18492E+01_JPRB, 0.24465E+01_JPRB, 0.21114E+01_JPRB ,&
&  0.18414E+01_JPRB, 0.18173E+01_JPRB, 0.17465E+01_JPRB, 0.17930E+01_JPRB, 0.13261E+01_JPRB, 0.11044E+01_JPRB, 0.89178E+00_JPRB ,&
&  0.92297E+00_JPRB, 0.87945E+00_JPRB, 0.10085E+01_JPRB, 0.90732E+00_JPRB, 0.88089E+00_JPRB, 0.83499E+00_JPRB, 0.78854E+00_JPRB ,&
&  0.78558E+00_JPRB, 0.75561E+00_JPRB, 0.73132E+00_JPRB, 0.78848E+00_JPRB, 0.85989E+00_JPRB, 0.80644E+00_JPRB, 0.87268E+00_JPRB ,&
&  0.92295E+00_JPRB, 0.98457E+00_JPRB, 0.98660E+00_JPRB, 0.96293E+00_JPRB, 0.87168E+00_JPRB, 0.81239E+00_JPRB, 0.78365E+00_JPRB ,&
&  0.76908E+00_JPRB, 0.73500E+00_JPRB, 0.69321E+00_JPRB, 0.68119E+00_JPRB, 0.65963E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.67563E+00_JPRB, 0.66588E+00_JPRB, 0.64443E+00_JPRB, 0.63953E+00_JPRB, 0.63374E+00_JPRB, 0.62735E+00_JPRB, 0.61613E+00_JPRB ,&
&  0.59954E+00_JPRB, 0.59143E+00_JPRB, 0.58633E+00_JPRB, 0.58308E+00_JPRB, 0.58070E+00_JPRB, 0.57295E+00_JPRB, 0.57183E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.69568E+00_JPRB, 0.61268E+00_JPRB, 0.55638E+00_JPRB, 0.53213E+00_JPRB, 0.62411E+00_JPRB, 0.76491E+00_JPRB ,&
&  0.96952E+00_JPRB, 0.12280E+01_JPRB, 0.16571E+01_JPRB, 0.20490E+01_JPRB, 0.25712E+01_JPRB, 0.30649E+01_JPRB, 0.34051E+01_JPRB ,&
&  0.35280E+01_JPRB, 0.26871E+01_JPRB, 0.24481E+01_JPRB, 0.21052E+01_JPRB, 0.18879E+01_JPRB, 0.16768E+01_JPRB, 0.15335E+01_JPRB ,&
&  0.14044E+01_JPRB, 0.13195E+01_JPRB, 0.12508E+01_JPRB, 0.11779E+01_JPRB, 0.11325E+01_JPRB, 0.11031E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.10602E+01_JPRB, 0.10142E+01_JPRB, 0.10987E+01_JPRB, 0.13332E+01_JPRB, 0.17444E+01_JPRB, 0.20748E+01_JPRB, 0.22653E+01_JPRB ,&
&  0.20835E+01_JPRB, 0.20917E+01_JPRB, 0.35616E+01_JPRB, 0.41093E+01_JPRB, 0.40138E+01_JPRB, 0.45973E+01_JPRB, 0.46780E+01_JPRB ,&
&  0.42450E+01_JPRB, 0.33782E+01_JPRB, 0.28369E+01_JPRB, 0.23767E+01_JPRB, 0.25731E+01_JPRB, 0.23410E+01_JPRB, 0.21032E+01_JPRB ,&
&  0.18835E+01_JPRB, 0.19385E+01_JPRB, 0.18826E+01_JPRB, 0.18439E+01_JPRB, 0.17454E+01_JPRB, 0.16294E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.13465E+01_JPRB, 0.11108E+01_JPRB, 0.88181E+00_JPRB, 0.74641E+00_JPRB, 0.76078E+00_JPRB, 0.80408E+00_JPRB ,&
&  0.82861E+00_JPRB, 0.82866E+00_JPRB, 0.80635E+00_JPRB, 0.81766E+00_JPRB, 0.83916E+00_JPRB, 0.82740E+00_JPRB, 0.84471E+00_JPRB ,&
&  0.84188E+00_JPRB, 0.90629E+00_JPRB, 0.92567E+00_JPRB, 0.93057E+00_JPRB, 0.90822E+00_JPRB, 0.84735E+00_JPRB, 0.81189E+00_JPRB ,&
&  0.79994E+00_JPRB, 0.79030E+00_JPRB, 0.75216E+00_JPRB, 0.71156E+00_JPRB, 0.68116E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.66956E+00_JPRB, 0.65181E+00_JPRB, 0.63070E+00_JPRB, 0.62975E+00_JPRB, 0.63149E+00_JPRB, 0.62886E+00_JPRB, 0.61031E+00_JPRB ,&
&  0.57640E+00_JPRB, 0.55743E+00_JPRB, 0.55228E+00_JPRB, 0.55655E+00_JPRB, 0.55844E+00_JPRB, 0.55950E+00_JPRB, 0.56919E+00_JPRB ,&
&  0.57012E+00_JPRB, 0.80549E+00_JPRB, 0.71093E+00_JPRB, 0.68140E+00_JPRB, 0.61078E+00_JPRB, 0.68608E+00_JPRB, 0.78772E+00_JPRB ,&
&  0.89881E+00_JPRB, 0.96906E+00_JPRB, 0.11231E+01_JPRB, 0.13610E+01_JPRB, 0.16113E+01_JPRB, 0.18365E+01_JPRB, 0.22036E+01_JPRB ,&
&  0.22022E+01_JPRB, 0.20420E+01_JPRB, 0.16997E+01_JPRB, 0.16269E+01_JPRB, 0.15329E+01_JPRB, 0.13302E+01_JPRB, 0.13303E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.11975E+01_JPRB, 0.11515E+01_JPRB, 0.11111E+01_JPRB, 0.10692E+01_JPRB, 0.10148E+01_JPRB, 0.98412E+00_JPRB ,&
&  0.96683E+00_JPRB, 0.95897E+00_JPRB, 0.10285E+01_JPRB, 0.11232E+01_JPRB, 0.14128E+01_JPRB, 0.17716E+01_JPRB, 0.20215E+01_JPRB ,&
&  0.23442E+01_JPRB, 0.21555E+01_JPRB, 0.20692E+01_JPRB, 0.29558E+01_JPRB, 0.43532E+01_JPRB, 0.46930E+01_JPRB, 0.43470E+01_JPRB ,&
&  0.45094E+01_JPRB, 0.40713E+01_JPRB, 0.34850E+01_JPRB, 0.30650E+01_JPRB, 0.27727E+01_JPRB, 0.25135E+01_JPRB, 0.23734E+01_JPRB ,&
&  0.21577E+01_JPRB, 0.20828E+01_JPRB, 0.21340E+01_JPRB, 0.21148E+01_JPRB, 0.19470E+01_JPRB, 0.17501E+01_JPRB, 0.16713E+01_JPRB ,&
&  0.15469E+01_JPRB, 0.12769E+01_JPRB, 0.89394E+00_JPRB, 0.86275E+00_JPRB, 0.69566E+00_JPRB, 0.69651E+00_JPRB, 0.80224E+00_JPRB ,&
&  0.79760E+00_JPRB, 0.84362E+00_JPRB, 0.85015E+00_JPRB, 0.80089E+00_JPRB, 0.81169E+00_JPRB, 0.78920E+00_JPRB, 0.80185E+00_JPRB ,&
&  0.82237E+00_JPRB, 0.74725E+00_JPRB, 0.78987E+00_JPRB, 0.83232E+00_JPRB, 0.86240E+00_JPRB, 0.84528E+00_JPRB, 0.81908E+00_JPRB ,&
&  0.80681E+00_JPRB, 0.79460E+00_JPRB, 0.76923E+00_JPRB, 0.72649E+00_JPRB, 0.69254E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.66918E+00_JPRB, 0.65113E+00_JPRB, 0.63189E+00_JPRB, 0.62143E+00_JPRB, 0.61584E+00_JPRB, 0.61255E+00_JPRB, 0.59597E+00_JPRB ,&
&  0.56933E+00_JPRB, 0.55022E+00_JPRB, 0.54208E+00_JPRB, 0.54487E+00_JPRB, 0.54529E+00_JPRB, 0.53915E+00_JPRB, 0.55647E+00_JPRB ,&
&  0.63870E+00_JPRB, 0.80260E+00_JPRB, 0.73823E+00_JPRB, 0.64745E+00_JPRB, 0.76244E+00_JPRB, 0.72774E+00_JPRB, 0.76012E+00_JPRB ,&
&  0.75503E+00_JPRB, 0.85963E+00_JPRB, 0.90859E+00_JPRB, 0.98754E+00_JPRB, 0.10750E+01_JPRB, 0.11971E+01_JPRB, 0.13860E+01_JPRB ,&
&  0.14282E+01_JPRB, 0.12917E+01_JPRB, 0.11891E+01_JPRB, 0.11853E+01_JPRB, 0.12458E+01_JPRB, 0.11304E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.10326E+01_JPRB, 0.10021E+01_JPRB, 0.98318E+00_JPRB, 0.96237E+00_JPRB, 0.93389E+00_JPRB, 0.91380E+00_JPRB ,&
&  0.90903E+00_JPRB, 0.90937E+00_JPRB, 0.96898E+00_JPRB, 0.10363E+01_JPRB, 0.12145E+01_JPRB, 0.16243E+01_JPRB, 0.22830E+01_JPRB ,&
&  0.25489E+01_JPRB, 0.26615E+01_JPRB, 0.29607E+01_JPRB, 0.34086E+01_JPRB, 0.36225E+01_JPRB, 0.37933E+01_JPRB, 0.39986E+01_JPRB ,&
&  0.39887E+01_JPRB, 0.39100E+01_JPRB, 0.38295E+01_JPRB, 0.35926E+01_JPRB, 0.32413E+01_JPRB, 0.29619E+01_JPRB, 0.27763E+01_JPRB ,&
&  0.25304E+01_JPRB, 0.23846E+01_JPRB, 0.23684E+01_JPRB, 0.22310E+01_JPRB, 0.21203E+01_JPRB, 0.19044E+01_JPRB, 0.18990E+01_JPRB ,&
&  0.17798E+01_JPRB, 0.10672E+01_JPRB, 0.82991E+00_JPRB, 0.90423E+00_JPRB, 0.74314E+00_JPRB, 0.76624E+00_JPRB, 0.87310E+00_JPRB ,&
&  0.84183E+00_JPRB, 0.81906E+00_JPRB, 0.82957E+00_JPRB, 0.79489E+00_JPRB, 0.76167E+00_JPRB, 0.77510E+00_JPRB, 0.74652E+00_JPRB ,&
&  0.69541E+00_JPRB, 0.65438E+00_JPRB, 0.67286E+00_JPRB, 0.71077E+00_JPRB, 0.75877E+00_JPRB, 0.76958E+00_JPRB, 0.76176E+00_JPRB ,&
&  0.76109E+00_JPRB, 0.75823E+00_JPRB, 0.75229E+00_JPRB, 0.72838E+00_JPRB, 0.69246E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.68474E+00_JPRB, 0.66884E+00_JPRB, 0.65309E+00_JPRB, 0.63991E+00_JPRB, 0.62072E+00_JPRB, 0.60116E+00_JPRB, 0.57706E+00_JPRB ,&
&  0.55800E+00_JPRB, 0.53892E+00_JPRB, 0.52853E+00_JPRB, 0.52385E+00_JPRB, 0.52165E+00_JPRB, 0.51491E+00_JPRB, 0.54560E+00_JPRB ,&
&  0.52042E+00_JPRB, 0.62155E+00_JPRB, 0.58067E+00_JPRB, 0.62548E+00_JPRB, 0.78398E+00_JPRB, 0.71251E+00_JPRB, 0.71205E+00_JPRB ,&
&  0.72111E+00_JPRB, 0.76172E+00_JPRB, 0.77028E+00_JPRB, 0.78225E+00_JPRB, 0.79284E+00_JPRB, 0.81947E+00_JPRB, 0.91058E+00_JPRB ,&
&  0.93610E+00_JPRB, 0.87070E+00_JPRB, 0.85514E+00_JPRB, 0.87621E+00_JPRB, 0.92663E+00_JPRB, 0.98404E+00_JPRB, 0.10198E+01_JPRB ,&
&  0.98479E+00_JPRB, 0.95458E+00_JPRB, 0.92993E+00_JPRB, 0.90528E+00_JPRB, 0.87727E+00_JPRB, 0.84992E+00_JPRB, 0.82747E+00_JPRB ,&
&  0.81788E+00_JPRB, 0.82637E+00_JPRB, 0.88531E+00_JPRB, 0.97795E+00_JPRB, 0.10685E+01_JPRB, 0.15174E+01_JPRB, 0.25222E+01_JPRB ,&
&  0.29911E+01_JPRB, 0.29642E+01_JPRB, 0.32566E+01_JPRB, 0.37974E+01_JPRB, 0.37906E+01_JPRB, 0.37206E+01_JPRB, 0.36481E+01_JPRB ,&
&  0.36077E+01_JPRB, 0.35316E+01_JPRB, 0.34492E+01_JPRB, 0.35224E+01_JPRB, 0.34699E+01_JPRB, 0.34162E+01_JPRB, 0.32316E+01_JPRB ,&
&  0.29090E+01_JPRB, 0.27117E+01_JPRB, 0.25952E+01_JPRB, 0.24314E+01_JPRB, 0.22402E+01_JPRB, 0.21406E+01_JPRB, 0.20001E+01_JPRB ,&
&  0.18122E+01_JPRB, 0.15628E+01_JPRB, 0.11555E+01_JPRB, 0.10369E+01_JPRB, 0.80347E+00_JPRB, 0.81522E+00_JPRB, 0.96033E+00_JPRB ,&
&  0.90719E+00_JPRB, 0.80704E+00_JPRB, 0.74867E+00_JPRB, 0.71672E+00_JPRB, 0.69620E+00_JPRB, 0.70135E+00_JPRB, 0.64381E+00_JPRB ,&
&  0.58794E+00_JPRB, 0.59368E+00_JPRB, 0.60570E+00_JPRB, 0.63172E+00_JPRB, 0.66716E+00_JPRB, 0.69290E+00_JPRB, 0.64712E+00_JPRB ,&
&  0.68384E+00_JPRB, 0.70165E+00_JPRB, 0.71012E+00_JPRB, 0.71311E+00_JPRB, 0.70259E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.66057E+00_JPRB, 0.64986E+00_JPRB, 0.64014E+00_JPRB, 0.62957E+00_JPRB, 0.60777E+00_JPRB, 0.57410E+00_JPRB, 0.54737E+00_JPRB ,&
&  0.53944E+00_JPRB, 0.52639E+00_JPRB, 0.51399E+00_JPRB, 0.50377E+00_JPRB, 0.49590E+00_JPRB, 0.48819E+00_JPRB, 0.48549E+00_JPRB ,&
&  0.47818E+00_JPRB, 0.53186E+00_JPRB, 0.53834E+00_JPRB, 0.56798E+00_JPRB, 0.59823E+00_JPRB, 0.58266E+00_JPRB, 0.58370E+00_JPRB ,&
&  0.62143E+00_JPRB, 0.63967E+00_JPRB, 0.65053E+00_JPRB, 0.64927E+00_JPRB, 0.64523E+00_JPRB, 0.66325E+00_JPRB, 0.71466E+00_JPRB ,&
&  0.70533E+00_JPRB, 0.69718E+00_JPRB, 0.69628E+00_JPRB, 0.72158E+00_JPRB, 0.79988E+00_JPRB, 0.89021E+00_JPRB, 0.91002E+00_JPRB ,&
&  0.88722E+00_JPRB, 0.86422E+00_JPRB, 0.83755E+00_JPRB, 0.81479E+00_JPRB, 0.79321E+00_JPRB, 0.77050E+00_JPRB, 0.74976E+00_JPRB ,&
&  0.74335E+00_JPRB, 0.75694E+00_JPRB, 0.80497E+00_JPRB, 0.89217E+00_JPRB, 0.99418E+00_JPRB, 0.12992E+01_JPRB, 0.17857E+01_JPRB ,&
&  0.20215E+01_JPRB, 0.21967E+01_JPRB, 0.24264E+01_JPRB, 0.26858E+01_JPRB, 0.28202E+01_JPRB, 0.28346E+01_JPRB, 0.28209E+01_JPRB ,&
&  0.28878E+01_JPRB, 0.28573E+01_JPRB, 0.29279E+01_JPRB, 0.28654E+01_JPRB, 0.28080E+01_JPRB, 0.28606E+01_JPRB, 0.28063E+01_JPRB ,&
&  0.26622E+01_JPRB, 0.24529E+01_JPRB, 0.24556E+01_JPRB, 0.23610E+01_JPRB, 0.22367E+01_JPRB, 0.21116E+01_JPRB, 0.19450E+01_JPRB ,&
&  0.17768E+01_JPRB, 0.16271E+01_JPRB, 0.13903E+01_JPRB, 0.12180E+01_JPRB, 0.10413E+01_JPRB, 0.10332E+01_JPRB, 0.93033E+00_JPRB ,&
&  0.81683E+00_JPRB, 0.67685E+00_JPRB, 0.65796E+00_JPRB, 0.64298E+00_JPRB, 0.63912E+00_JPRB, 0.60354E+00_JPRB, 0.55150E+00_JPRB ,&
&  0.54091E+00_JPRB, 0.56339E+00_JPRB, 0.57053E+00_JPRB, 0.57945E+00_JPRB, 0.60247E+00_JPRB, 0.63425E+00_JPRB, 0.56308E+00_JPRB ,&
&  0.56999E+00_JPRB, 0.62489E+00_JPRB, 0.64911E+00_JPRB, 0.66245E+00_JPRB, 0.66514E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.62000E+00_JPRB, 0.61382E+00_JPRB, 0.60277E+00_JPRB, 0.59220E+00_JPRB, 0.57195E+00_JPRB, 0.54536E+00_JPRB, 0.51294E+00_JPRB ,&
&  0.52395E+00_JPRB, 0.51439E+00_JPRB, 0.50344E+00_JPRB, 0.48954E+00_JPRB, 0.47485E+00_JPRB, 0.45080E+00_JPRB, 0.41539E+00_JPRB ,&
&  0.41718E+00_JPRB, 0.44419E+00_JPRB, 0.48425E+00_JPRB, 0.50723E+00_JPRB, 0.51033E+00_JPRB, 0.51059E+00_JPRB, 0.51830E+00_JPRB ,&
&  0.52973E+00_JPRB, 0.54202E+00_JPRB, 0.56077E+00_JPRB, 0.57230E+00_JPRB, 0.57170E+00_JPRB, 0.57305E+00_JPRB, 0.59362E+00_JPRB ,&
&  0.57289E+00_JPRB, 0.56360E+00_JPRB, 0.58345E+00_JPRB, 0.56450E+00_JPRB, 0.65310E+00_JPRB, 0.70199E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.71244E+00_JPRB, 0.68666E+00_JPRB, 0.66867E+00_JPRB, 0.66749E+00_JPRB, 0.66726E+00_JPRB, 0.66487E+00_JPRB, 0.66442E+00_JPRB ,&
&  0.67094E+00_JPRB, 0.68804E+00_JPRB, 0.72846E+00_JPRB, 0.79607E+00_JPRB, 0.88692E+00_JPRB, 0.10899E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.14926E+01_JPRB, 0.17579E+01_JPRB, 0.20254E+01_JPRB, 0.20491E+01_JPRB, 0.22426E+01_JPRB, 0.23664E+01_JPRB, 0.23381E+01_JPRB ,&
&  0.23989E+01_JPRB, 0.23523E+01_JPRB, 0.24057E+01_JPRB, 0.23941E+01_JPRB, 0.23784E+01_JPRB, 0.23623E+01_JPRB, 0.23334E+01_JPRB ,&
&  0.22865E+01_JPRB, 0.22092E+01_JPRB, 0.21965E+01_JPRB, 0.21150E+01_JPRB, 0.19841E+01_JPRB, 0.18712E+01_JPRB, 0.17555E+01_JPRB ,&
&  0.16558E+01_JPRB, 0.15514E+01_JPRB, 0.14348E+01_JPRB, 0.13289E+01_JPRB, 0.12187E+01_JPRB, 0.10922E+01_JPRB, 0.97190E+00_JPRB ,&
&  0.85791E+00_JPRB, 0.72396E+00_JPRB, 0.65419E+00_JPRB, 0.63499E+00_JPRB, 0.60925E+00_JPRB, 0.55005E+00_JPRB, 0.51522E+00_JPRB ,&
&  0.50167E+00_JPRB, 0.51601E+00_JPRB, 0.54146E+00_JPRB, 0.54753E+00_JPRB, 0.56048E+00_JPRB, 0.57317E+00_JPRB, 0.54268E+00_JPRB ,&
&  0.52220E+00_JPRB, 0.56729E+00_JPRB, 0.59140E+00_JPRB, 0.60674E+00_JPRB, 0.61723E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.57683E+00_JPRB, 0.57481E+00_JPRB, 0.56370E+00_JPRB, 0.55332E+00_JPRB, 0.54514E+00_JPRB, 0.51929E+00_JPRB, 0.49785E+00_JPRB ,&
&  0.52695E+00_JPRB, 0.49157E+00_JPRB, 0.48613E+00_JPRB, 0.44173E+00_JPRB, 0.41949E+00_JPRB, 0.41153E+00_JPRB, 0.40555E+00_JPRB ,&
&  0.41127E+00_JPRB, 0.42878E+00_JPRB, 0.44441E+00_JPRB, 0.45705E+00_JPRB, 0.47151E+00_JPRB, 0.47512E+00_JPRB, 0.47741E+00_JPRB ,&
&  0.48171E+00_JPRB, 0.48968E+00_JPRB, 0.50012E+00_JPRB, 0.50198E+00_JPRB, 0.50350E+00_JPRB, 0.50501E+00_JPRB, 0.50745E+00_JPRB ,&
&  0.49436E+00_JPRB, 0.49233E+00_JPRB, 0.51627E+00_JPRB, 0.52964E+00_JPRB, 0.55563E+00_JPRB, 0.56884E+00_JPRB, 0.56762E+00_JPRB ,&
&  0.54995E+00_JPRB, 0.52117E+00_JPRB, 0.54552E+00_JPRB, 0.55985E+00_JPRB, 0.57247E+00_JPRB, 0.58001E+00_JPRB, 0.58473E+00_JPRB ,&
&  0.58747E+00_JPRB, 0.59491E+00_JPRB, 0.61271E+00_JPRB, 0.64465E+00_JPRB, 0.69831E+00_JPRB, 0.80926E+00_JPRB, 0.91500E+00_JPRB ,&
&  0.11393E+01_JPRB, 0.10968E+01_JPRB, 0.13750E+01_JPRB, 0.15570E+01_JPRB, 0.17759E+01_JPRB, 0.18741E+01_JPRB, 0.18533E+01_JPRB ,&
&  0.18622E+01_JPRB, 0.18460E+01_JPRB, 0.18483E+01_JPRB, 0.18064E+01_JPRB, 0.17645E+01_JPRB, 0.17418E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.17531E+01_JPRB, 0.16456E+01_JPRB, 0.17146E+01_JPRB, 0.16472E+01_JPRB, 0.15662E+01_JPRB, 0.15186E+01_JPRB, 0.14784E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.13671E+01_JPRB, 0.12701E+01_JPRB, 0.11035E+01_JPRB, 0.10311E+01_JPRB, 0.96762E+00_JPRB, 0.87357E+00_JPRB ,&
&  0.81602E+00_JPRB, 0.75445E+00_JPRB, 0.69223E+00_JPRB, 0.67491E+00_JPRB, 0.61100E+00_JPRB, 0.56320E+00_JPRB, 0.52844E+00_JPRB ,&
&  0.48926E+00_JPRB, 0.47899E+00_JPRB, 0.49280E+00_JPRB, 0.50156E+00_JPRB, 0.50162E+00_JPRB, 0.49968E+00_JPRB, 0.51610E+00_JPRB ,&
&  0.50833E+00_JPRB, 0.52362E+00_JPRB, 0.55492E+00_JPRB, 0.56348E+00_JPRB, 0.57029E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.54613E+00_JPRB, 0.54740E+00_JPRB, 0.53795E+00_JPRB, 0.52884E+00_JPRB, 0.51963E+00_JPRB, 0.50258E+00_JPRB, 0.47159E+00_JPRB ,&
&  0.46559E+00_JPRB, 0.44059E+00_JPRB, 0.41759E+00_JPRB, 0.41100E+00_JPRB, 0.40971E+00_JPRB, 0.40586E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.40072E+00_JPRB, 0.41512E+00_JPRB, 0.43342E+00_JPRB, 0.44493E+00_JPRB, 0.45503E+00_JPRB, 0.45764E+00_JPRB, 0.46068E+00_JPRB ,&
&  0.46408E+00_JPRB, 0.46186E+00_JPRB, 0.46154E+00_JPRB, 0.46003E+00_JPRB, 0.45718E+00_JPRB, 0.45902E+00_JPRB, 0.45942E+00_JPRB ,&
&  0.46522E+00_JPRB, 0.47003E+00_JPRB, 0.47829E+00_JPRB, 0.49355E+00_JPRB, 0.50431E+00_JPRB, 0.51288E+00_JPRB, 0.48792E+00_JPRB ,&
&  0.42981E+00_JPRB, 0.43999E+00_JPRB, 0.47384E+00_JPRB, 0.49200E+00_JPRB, 0.51161E+00_JPRB, 0.52423E+00_JPRB, 0.53096E+00_JPRB ,&
&  0.53140E+00_JPRB, 0.52948E+00_JPRB, 0.53372E+00_JPRB, 0.55130E+00_JPRB, 0.58522E+00_JPRB, 0.64080E+00_JPRB, 0.69954E+00_JPRB ,&
&  0.82286E+00_JPRB, 0.83494E+00_JPRB, 0.87951E+00_JPRB, 0.10079E+01_JPRB, 0.12966E+01_JPRB, 0.13879E+01_JPRB, 0.14136E+01_JPRB ,&
&  0.14327E+01_JPRB, 0.14334E+01_JPRB, 0.14116E+01_JPRB, 0.13764E+01_JPRB, 0.13150E+01_JPRB, 0.12692E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.12352E+01_JPRB, 0.12416E+01_JPRB, 0.12296E+01_JPRB, 0.12166E+01_JPRB, 0.11678E+01_JPRB, 0.11426E+01_JPRB, 0.11209E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.10683E+01_JPRB, 0.99613E+00_JPRB, 0.91613E+00_JPRB, 0.84088E+00_JPRB, 0.79523E+00_JPRB, 0.75963E+00_JPRB ,&
&  0.72204E+00_JPRB, 0.69272E+00_JPRB, 0.66484E+00_JPRB, 0.63275E+00_JPRB, 0.59333E+00_JPRB, 0.55672E+00_JPRB, 0.52159E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.45825E+00_JPRB, 0.45604E+00_JPRB, 0.45454E+00_JPRB, 0.45738E+00_JPRB, 0.46819E+00_JPRB, 0.46371E+00_JPRB ,&
&  0.47847E+00_JPRB, 0.49126E+00_JPRB, 0.49553E+00_JPRB, 0.50594E+00_JPRB, 0.52319E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.51021E+00_JPRB, 0.51653E+00_JPRB, 0.51182E+00_JPRB, 0.51147E+00_JPRB, 0.49908E+00_JPRB, 0.48417E+00_JPRB, 0.46709E+00_JPRB ,&
&  0.46102E+00_JPRB, 0.44793E+00_JPRB, 0.43130E+00_JPRB, 0.41503E+00_JPRB, 0.40688E+00_JPRB, 0.40435E+00_JPRB, 0.40883E+00_JPRB ,&
&  0.41833E+00_JPRB, 0.43030E+00_JPRB, 0.44000E+00_JPRB, 0.44617E+00_JPRB, 0.44602E+00_JPRB, 0.44462E+00_JPRB, 0.44658E+00_JPRB ,&
&  0.44648E+00_JPRB, 0.44283E+00_JPRB, 0.44246E+00_JPRB, 0.44033E+00_JPRB, 0.44019E+00_JPRB, 0.44112E+00_JPRB, 0.44014E+00_JPRB ,&
&  0.43947E+00_JPRB, 0.43877E+00_JPRB, 0.44797E+00_JPRB, 0.46208E+00_JPRB, 0.47120E+00_JPRB, 0.47016E+00_JPRB, 0.43231E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.39038E+00_JPRB, 0.42368E+00_JPRB, 0.44244E+00_JPRB, 0.46255E+00_JPRB, 0.47425E+00_JPRB, 0.47708E+00_JPRB ,&
&  0.46811E+00_JPRB, 0.46314E+00_JPRB, 0.47319E+00_JPRB, 0.49856E+00_JPRB, 0.51924E+00_JPRB, 0.55040E+00_JPRB, 0.58516E+00_JPRB ,&
&  0.63414E+00_JPRB, 0.69762E+00_JPRB, 0.76557E+00_JPRB, 0.76565E+00_JPRB, 0.84214E+00_JPRB, 0.11079E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.11395E+01_JPRB, 0.11259E+01_JPRB, 0.11509E+01_JPRB, 0.11192E+01_JPRB, 0.10494E+01_JPRB, 0.99009E+00_JPRB, 0.94494E+00_JPRB ,&
&  0.89880E+00_JPRB, 0.88096E+00_JPRB, 0.87091E+00_JPRB, 0.88075E+00_JPRB, 0.87035E+00_JPRB, 0.85249E+00_JPRB, 0.83729E+00_JPRB ,&
&  0.82359E+00_JPRB, 0.78829E+00_JPRB, 0.75132E+00_JPRB, 0.69976E+00_JPRB, 0.66605E+00_JPRB, 0.64410E+00_JPRB, 0.63258E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.61312E+00_JPRB, 0.60740E+00_JPRB, 0.59601E+00_JPRB, 0.55551E+00_JPRB, 0.50946E+00_JPRB, 0.47507E+00_JPRB ,&
&  0.45212E+00_JPRB, 0.43649E+00_JPRB, 0.43299E+00_JPRB, 0.43488E+00_JPRB, 0.44064E+00_JPRB, 0.44559E+00_JPRB, 0.43871E+00_JPRB ,&
&  0.43839E+00_JPRB, 0.44464E+00_JPRB, 0.45893E+00_JPRB, 0.47740E+00_JPRB, 0.49380E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.46308E+00_JPRB, 0.48056E+00_JPRB, 0.49404E+00_JPRB, 0.49824E+00_JPRB, 0.49552E+00_JPRB, 0.46729E+00_JPRB, 0.44066E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.42253E+00_JPRB, 0.42450E+00_JPRB, 0.42698E+00_JPRB, 0.43054E+00_JPRB, 0.43535E+00_JPRB, 0.43928E+00_JPRB ,&
&  0.44105E+00_JPRB, 0.44302E+00_JPRB, 0.44697E+00_JPRB, 0.45324E+00_JPRB, 0.45276E+00_JPRB, 0.44956E+00_JPRB, 0.44884E+00_JPRB ,&
&  0.44728E+00_JPRB, 0.44266E+00_JPRB, 0.44131E+00_JPRB, 0.43638E+00_JPRB, 0.43369E+00_JPRB, 0.43145E+00_JPRB, 0.42631E+00_JPRB ,&
&  0.41913E+00_JPRB, 0.41749E+00_JPRB, 0.42555E+00_JPRB, 0.43623E+00_JPRB, 0.44505E+00_JPRB, 0.43501E+00_JPRB, 0.40500E+00_JPRB ,&
&  0.37931E+00_JPRB, 0.36600E+00_JPRB, 0.37039E+00_JPRB, 0.39486E+00_JPRB, 0.41837E+00_JPRB, 0.41440E+00_JPRB, 0.42505E+00_JPRB ,&
&  0.42878E+00_JPRB, 0.43238E+00_JPRB, 0.43923E+00_JPRB, 0.45397E+00_JPRB, 0.47185E+00_JPRB, 0.49156E+00_JPRB, 0.51132E+00_JPRB ,&
&  0.53561E+00_JPRB, 0.56570E+00_JPRB, 0.60347E+00_JPRB, 0.61310E+00_JPRB, 0.60793E+00_JPRB, 0.65569E+00_JPRB, 0.72047E+00_JPRB ,&
&  0.78203E+00_JPRB, 0.81130E+00_JPRB, 0.84036E+00_JPRB, 0.86101E+00_JPRB, 0.85822E+00_JPRB, 0.83612E+00_JPRB, 0.80469E+00_JPRB ,&
&  0.77732E+00_JPRB, 0.74428E+00_JPRB, 0.71958E+00_JPRB, 0.70979E+00_JPRB, 0.71378E+00_JPRB, 0.71228E+00_JPRB, 0.69869E+00_JPRB ,&
&  0.68109E+00_JPRB, 0.64812E+00_JPRB, 0.62053E+00_JPRB, 0.57191E+00_JPRB, 0.56124E+00_JPRB, 0.57228E+00_JPRB, 0.58655E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.57938E+00_JPRB, 0.57559E+00_JPRB, 0.56833E+00_JPRB, 0.52841E+00_JPRB, 0.49936E+00_JPRB, 0.47858E+00_JPRB ,&
&  0.46736E+00_JPRB, 0.45554E+00_JPRB, 0.45349E+00_JPRB, 0.45210E+00_JPRB, 0.45198E+00_JPRB, 0.45061E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.43563E+00_JPRB, 0.43393E+00_JPRB, 0.43823E+00_JPRB, 0.44313E+00_JPRB, 0.45160E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.46255E+00_JPRB, 0.46972E+00_JPRB, 0.47838E+00_JPRB, 0.48424E+00_JPRB, 0.48468E+00_JPRB, 0.47314E+00_JPRB, 0.46224E+00_JPRB ,&
&  0.45546E+00_JPRB, 0.45539E+00_JPRB, 0.45528E+00_JPRB, 0.45486E+00_JPRB, 0.45547E+00_JPRB, 0.45630E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.45950E+00_JPRB, 0.46175E+00_JPRB, 0.46280E+00_JPRB, 0.45798E+00_JPRB, 0.45301E+00_JPRB, 0.45125E+00_JPRB, 0.45191E+00_JPRB ,&
&  0.45199E+00_JPRB, 0.44829E+00_JPRB, 0.44591E+00_JPRB, 0.43852E+00_JPRB, 0.42929E+00_JPRB, 0.41933E+00_JPRB, 0.41067E+00_JPRB ,&
&  0.40961E+00_JPRB, 0.41202E+00_JPRB, 0.40986E+00_JPRB, 0.41425E+00_JPRB, 0.42223E+00_JPRB, 0.40485E+00_JPRB, 0.38268E+00_JPRB ,&
&  0.36429E+00_JPRB, 0.35107E+00_JPRB, 0.34494E+00_JPRB, 0.34544E+00_JPRB, 0.36120E+00_JPRB, 0.38466E+00_JPRB, 0.39674E+00_JPRB ,&
&  0.38653E+00_JPRB, 0.40038E+00_JPRB, 0.41743E+00_JPRB, 0.42870E+00_JPRB, 0.43986E+00_JPRB, 0.45390E+00_JPRB, 0.46728E+00_JPRB ,&
&  0.48033E+00_JPRB, 0.49240E+00_JPRB, 0.50383E+00_JPRB, 0.52524E+00_JPRB, 0.53892E+00_JPRB, 0.53075E+00_JPRB, 0.54273E+00_JPRB ,&
&  0.63228E+00_JPRB, 0.69500E+00_JPRB, 0.73936E+00_JPRB, 0.75354E+00_JPRB, 0.75058E+00_JPRB, 0.72809E+00_JPRB, 0.70555E+00_JPRB ,&
&  0.68628E+00_JPRB, 0.67055E+00_JPRB, 0.65962E+00_JPRB, 0.65519E+00_JPRB, 0.65553E+00_JPRB, 0.64960E+00_JPRB, 0.64189E+00_JPRB ,&
&  0.62950E+00_JPRB, 0.63672E+00_JPRB, 0.81052E+00_JPRB, 0.73636E+00_JPRB, 0.68134E+00_JPRB, 0.65522E+00_JPRB, 0.62879E+00_JPRB ,&
&  0.60738E+00_JPRB, 0.59218E+00_JPRB, 0.58352E+00_JPRB, 0.57360E+00_JPRB, 0.54702E+00_JPRB, 0.52541E+00_JPRB, 0.51153E+00_JPRB ,&
&  0.50391E+00_JPRB, 0.49525E+00_JPRB, 0.48748E+00_JPRB, 0.48119E+00_JPRB, 0.47439E+00_JPRB, 0.46672E+00_JPRB, 0.45417E+00_JPRB ,&
&  0.44527E+00_JPRB, 0.44493E+00_JPRB, 0.44759E+00_JPRB, 0.45373E+00_JPRB, 0.45803E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.45549E+00_JPRB, 0.46163E+00_JPRB, 0.46790E+00_JPRB, 0.47279E+00_JPRB, 0.47462E+00_JPRB, 0.47381E+00_JPRB, 0.47083E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.46690E+00_JPRB, 0.46631E+00_JPRB, 0.46657E+00_JPRB, 0.46707E+00_JPRB, 0.46781E+00_JPRB, 0.46852E+00_JPRB ,&
&  0.47034E+00_JPRB, 0.46159E+00_JPRB, 0.45671E+00_JPRB, 0.45726E+00_JPRB, 0.45674E+00_JPRB, 0.45685E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.45362E+00_JPRB, 0.44545E+00_JPRB, 0.43896E+00_JPRB, 0.43078E+00_JPRB, 0.42230E+00_JPRB, 0.42137E+00_JPRB, 0.41987E+00_JPRB ,&
&  0.41308E+00_JPRB, 0.40631E+00_JPRB, 0.41023E+00_JPRB, 0.41244E+00_JPRB, 0.39844E+00_JPRB, 0.38203E+00_JPRB, 0.35627E+00_JPRB ,&
&  0.35064E+00_JPRB, 0.33995E+00_JPRB, 0.33776E+00_JPRB, 0.33831E+00_JPRB, 0.34552E+00_JPRB, 0.35517E+00_JPRB, 0.37199E+00_JPRB ,&
&  0.38018E+00_JPRB, 0.38688E+00_JPRB, 0.39903E+00_JPRB, 0.41178E+00_JPRB, 0.42088E+00_JPRB, 0.42917E+00_JPRB, 0.43371E+00_JPRB ,&
&  0.44221E+00_JPRB, 0.45200E+00_JPRB, 0.46116E+00_JPRB, 0.46994E+00_JPRB, 0.48065E+00_JPRB, 0.48656E+00_JPRB, 0.49450E+00_JPRB ,&
&  0.52635E+00_JPRB, 0.55136E+00_JPRB, 0.57142E+00_JPRB, 0.58919E+00_JPRB, 0.60141E+00_JPRB, 0.60529E+00_JPRB, 0.59352E+00_JPRB ,&
&  0.58589E+00_JPRB, 0.58833E+00_JPRB, 0.59005E+00_JPRB, 0.59385E+00_JPRB, 0.59937E+00_JPRB, 0.60388E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.59103E+00_JPRB, 0.59098E+00_JPRB, 0.64210E+00_JPRB, 0.64074E+00_JPRB, 0.62596E+00_JPRB, 0.60401E+00_JPRB, 0.58113E+00_JPRB ,&
&  0.56403E+00_JPRB, 0.55430E+00_JPRB, 0.54849E+00_JPRB, 0.54423E+00_JPRB, 0.54073E+00_JPRB, 0.53507E+00_JPRB, 0.53397E+00_JPRB ,&
&  0.52822E+00_JPRB, 0.51985E+00_JPRB, 0.51070E+00_JPRB, 0.50178E+00_JPRB, 0.49349E+00_JPRB, 0.48488E+00_JPRB, 0.46835E+00_JPRB ,&
&  0.45084E+00_JPRB, 0.44023E+00_JPRB, 0.43907E+00_JPRB, 0.44237E+00_JPRB, 0.44902E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.44948E+00_JPRB, 0.45252E+00_JPRB, 0.45673E+00_JPRB, 0.45813E+00_JPRB, 0.45983E+00_JPRB, 0.45915E+00_JPRB, 0.45901E+00_JPRB ,&
&  0.45810E+00_JPRB, 0.45841E+00_JPRB, 0.45863E+00_JPRB, 0.45952E+00_JPRB, 0.46030E+00_JPRB, 0.46146E+00_JPRB, 0.46239E+00_JPRB ,&
&  0.46284E+00_JPRB, 0.46298E+00_JPRB, 0.46170E+00_JPRB, 0.46025E+00_JPRB, 0.45828E+00_JPRB, 0.45823E+00_JPRB, 0.45644E+00_JPRB ,&
&  0.45259E+00_JPRB, 0.44528E+00_JPRB, 0.43135E+00_JPRB, 0.42381E+00_JPRB, 0.42267E+00_JPRB, 0.41597E+00_JPRB, 0.41706E+00_JPRB ,&
&  0.41417E+00_JPRB, 0.41300E+00_JPRB, 0.40900E+00_JPRB, 0.40813E+00_JPRB, 0.39833E+00_JPRB, 0.34741E+00_JPRB, 0.34200E+00_JPRB ,&
&  0.33768E+00_JPRB, 0.33186E+00_JPRB, 0.33113E+00_JPRB, 0.33242E+00_JPRB, 0.33736E+00_JPRB, 0.35191E+00_JPRB, 0.36609E+00_JPRB ,&
&  0.36860E+00_JPRB, 0.38869E+00_JPRB, 0.39475E+00_JPRB, 0.40037E+00_JPRB, 0.41052E+00_JPRB, 0.41897E+00_JPRB, 0.42048E+00_JPRB ,&
&  0.42423E+00_JPRB, 0.42823E+00_JPRB, 0.43551E+00_JPRB, 0.44371E+00_JPRB, 0.45078E+00_JPRB, 0.45604E+00_JPRB, 0.46242E+00_JPRB ,&
&  0.46937E+00_JPRB, 0.47625E+00_JPRB, 0.48398E+00_JPRB, 0.49039E+00_JPRB, 0.49545E+00_JPRB, 0.50082E+00_JPRB, 0.50731E+00_JPRB ,&
&  0.51550E+00_JPRB, 0.52260E+00_JPRB, 0.53256E+00_JPRB, 0.53448E+00_JPRB, 0.54054E+00_JPRB, 0.54521E+00_JPRB, 0.55498E+00_JPRB ,&
&  0.56044E+00_JPRB, 0.57160E+00_JPRB, 0.57318E+00_JPRB, 0.57117E+00_JPRB, 0.56032E+00_JPRB, 0.55136E+00_JPRB, 0.54499E+00_JPRB ,&
&  0.54166E+00_JPRB, 0.54147E+00_JPRB, 0.53512E+00_JPRB, 0.53312E+00_JPRB, 0.53340E+00_JPRB, 0.53459E+00_JPRB, 0.53488E+00_JPRB ,&
&  0.53266E+00_JPRB, 0.52725E+00_JPRB, 0.52026E+00_JPRB, 0.51069E+00_JPRB, 0.50235E+00_JPRB, 0.49443E+00_JPRB, 0.47884E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.44656E+00_JPRB, 0.44070E+00_JPRB, 0.43932E+00_JPRB, 0.44347E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.44448E+00_JPRB, 0.44495E+00_JPRB, 0.44765E+00_JPRB, 0.44739E+00_JPRB, 0.44917E+00_JPRB, 0.44914E+00_JPRB, 0.44955E+00_JPRB ,&
&  0.44899E+00_JPRB, 0.44905E+00_JPRB, 0.44857E+00_JPRB, 0.44903E+00_JPRB, 0.44917E+00_JPRB, 0.45027E+00_JPRB, 0.45102E+00_JPRB ,&
&  0.45217E+00_JPRB, 0.45279E+00_JPRB, 0.45367E+00_JPRB, 0.45346E+00_JPRB, 0.45353E+00_JPRB, 0.45302E+00_JPRB, 0.45082E+00_JPRB ,&
&  0.44700E+00_JPRB, 0.43910E+00_JPRB, 0.42067E+00_JPRB, 0.41629E+00_JPRB, 0.41213E+00_JPRB, 0.41186E+00_JPRB, 0.41021E+00_JPRB ,&
&  0.40864E+00_JPRB, 0.37383E+00_JPRB, 0.37500E+00_JPRB, 0.36609E+00_JPRB, 0.36012E+00_JPRB, 0.35429E+00_JPRB, 0.34578E+00_JPRB ,&
&  0.34262E+00_JPRB, 0.33959E+00_JPRB, 0.33935E+00_JPRB, 0.34345E+00_JPRB, 0.34835E+00_JPRB, 0.35639E+00_JPRB, 0.36614E+00_JPRB ,&
&  0.36867E+00_JPRB, 0.38481E+00_JPRB, 0.39190E+00_JPRB, 0.39498E+00_JPRB, 0.40589E+00_JPRB, 0.41426E+00_JPRB, 0.41609E+00_JPRB ,&
&  0.41919E+00_JPRB, 0.41900E+00_JPRB, 0.42290E+00_JPRB, 0.42935E+00_JPRB, 0.43655E+00_JPRB, 0.43920E+00_JPRB, 0.44187E+00_JPRB ,&
&  0.44400E+00_JPRB, 0.44565E+00_JPRB, 0.44722E+00_JPRB, 0.44744E+00_JPRB, 0.44856E+00_JPRB, 0.45246E+00_JPRB, 0.45806E+00_JPRB ,&
&  0.46524E+00_JPRB, 0.47298E+00_JPRB, 0.48239E+00_JPRB, 0.49040E+00_JPRB, 0.49727E+00_JPRB, 0.50388E+00_JPRB, 0.50946E+00_JPRB ,&
&  0.51421E+00_JPRB, 0.51736E+00_JPRB, 0.51746E+00_JPRB, 0.51845E+00_JPRB, 0.51782E+00_JPRB, 0.51581E+00_JPRB, 0.51545E+00_JPRB ,&
&  0.51916E+00_JPRB, 0.51952E+00_JPRB, 0.51889E+00_JPRB, 0.51819E+00_JPRB, 0.51957E+00_JPRB, 0.52123E+00_JPRB, 0.52183E+00_JPRB ,&
&  0.52136E+00_JPRB, 0.51807E+00_JPRB, 0.51313E+00_JPRB, 0.50465E+00_JPRB, 0.49617E+00_JPRB, 0.49033E+00_JPRB, 0.48143E+00_JPRB ,&
&  0.47166E+00_JPRB, 0.45970E+00_JPRB, 0.44984E+00_JPRB, 0.44357E+00_JPRB, 0.44319E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.44276E+00_JPRB, 0.44306E+00_JPRB, 0.44524E+00_JPRB, 0.44626E+00_JPRB, 0.44834E+00_JPRB, 0.44864E+00_JPRB, 0.44967E+00_JPRB ,&
&  0.44918E+00_JPRB, 0.44915E+00_JPRB, 0.44884E+00_JPRB, 0.44853E+00_JPRB, 0.44825E+00_JPRB, 0.44793E+00_JPRB, 0.44791E+00_JPRB ,&
&  0.44845E+00_JPRB, 0.44843E+00_JPRB, 0.44937E+00_JPRB, 0.44943E+00_JPRB, 0.44921E+00_JPRB, 0.44873E+00_JPRB, 0.44503E+00_JPRB ,&
&  0.44060E+00_JPRB, 0.43651E+00_JPRB, 0.41068E+00_JPRB, 0.40750E+00_JPRB, 0.40185E+00_JPRB, 0.39891E+00_JPRB, 0.39929E+00_JPRB ,&
&  0.39655E+00_JPRB, 0.39322E+00_JPRB, 0.38333E+00_JPRB, 0.37275E+00_JPRB, 0.36400E+00_JPRB, 0.35325E+00_JPRB, 0.35063E+00_JPRB ,&
&  0.34812E+00_JPRB, 0.35026E+00_JPRB, 0.35305E+00_JPRB, 0.35592E+00_JPRB, 0.36304E+00_JPRB, 0.36840E+00_JPRB, 0.37656E+00_JPRB ,&
&  0.38647E+00_JPRB, 0.39394E+00_JPRB, 0.39589E+00_JPRB, 0.39240E+00_JPRB, 0.40309E+00_JPRB, 0.40950E+00_JPRB, 0.41540E+00_JPRB ,&
&  0.41873E+00_JPRB, 0.41952E+00_JPRB, 0.42244E+00_JPRB, 0.42365E+00_JPRB, 0.42544E+00_JPRB, 0.42868E+00_JPRB, 0.43022E+00_JPRB ,&
&  0.43218E+00_JPRB, 0.43227E+00_JPRB, 0.43156E+00_JPRB, 0.42986E+00_JPRB, 0.42726E+00_JPRB, 0.42613E+00_JPRB, 0.42686E+00_JPRB ,&
&  0.42933E+00_JPRB, 0.43381E+00_JPRB, 0.43946E+00_JPRB, 0.44625E+00_JPRB, 0.45274E+00_JPRB, 0.45863E+00_JPRB, 0.46342E+00_JPRB ,&
&  0.46734E+00_JPRB, 0.47025E+00_JPRB, 0.47522E+00_JPRB, 0.47499E+00_JPRB, 0.47761E+00_JPRB, 0.48533E+00_JPRB, 0.48594E+00_JPRB ,&
&  0.48874E+00_JPRB, 0.49276E+00_JPRB, 0.49532E+00_JPRB, 0.49584E+00_JPRB, 0.49626E+00_JPRB, 0.49714E+00_JPRB, 0.49794E+00_JPRB ,&
&  0.49811E+00_JPRB, 0.49598E+00_JPRB, 0.49235E+00_JPRB, 0.48748E+00_JPRB, 0.48287E+00_JPRB, 0.47822E+00_JPRB, 0.47242E+00_JPRB ,&
&  0.46683E+00_JPRB, 0.46079E+00_JPRB, 0.45374E+00_JPRB, 0.44688E+00_JPRB, 0.44389E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.44565E+00_JPRB, 0.44418E+00_JPRB, 0.44367E+00_JPRB, 0.44383E+00_JPRB, 0.44457E+00_JPRB, 0.44495E+00_JPRB, 0.44565E+00_JPRB ,&
&  0.44577E+00_JPRB, 0.44569E+00_JPRB, 0.44485E+00_JPRB, 0.44476E+00_JPRB, 0.44442E+00_JPRB, 0.44449E+00_JPRB, 0.44378E+00_JPRB ,&
&  0.44343E+00_JPRB, 0.44241E+00_JPRB, 0.44273E+00_JPRB, 0.44266E+00_JPRB, 0.44273E+00_JPRB, 0.44294E+00_JPRB, 0.44002E+00_JPRB ,&
&  0.43713E+00_JPRB, 0.43502E+00_JPRB, 0.43047E+00_JPRB, 0.42632E+00_JPRB, 0.42333E+00_JPRB, 0.40159E+00_JPRB, 0.39476E+00_JPRB ,&
&  0.39962E+00_JPRB, 0.40298E+00_JPRB, 0.40693E+00_JPRB, 0.41013E+00_JPRB, 0.40534E+00_JPRB, 0.39650E+00_JPRB, 0.39145E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.37656E+00_JPRB, 0.37855E+00_JPRB, 0.38041E+00_JPRB, 0.38458E+00_JPRB, 0.38629E+00_JPRB, 0.39078E+00_JPRB ,&
&  0.40278E+00_JPRB, 0.40550E+00_JPRB, 0.40908E+00_JPRB, 0.41155E+00_JPRB, 0.41224E+00_JPRB, 0.41345E+00_JPRB, 0.41713E+00_JPRB ,&
&  0.41879E+00_JPRB, 0.42034E+00_JPRB, 0.42146E+00_JPRB, 0.42149E+00_JPRB, 0.42222E+00_JPRB, 0.42231E+00_JPRB, 0.42202E+00_JPRB ,&
&  0.42084E+00_JPRB, 0.42045E+00_JPRB, 0.42000E+00_JPRB, 0.41941E+00_JPRB, 0.41926E+00_JPRB, 0.41814E+00_JPRB, 0.41756E+00_JPRB ,&
&  0.41777E+00_JPRB, 0.41890E+00_JPRB, 0.42053E+00_JPRB, 0.42294E+00_JPRB, 0.42607E+00_JPRB, 0.43026E+00_JPRB, 0.43457E+00_JPRB ,&
&  0.43944E+00_JPRB, 0.44412E+00_JPRB, 0.44825E+00_JPRB, 0.45183E+00_JPRB, 0.45488E+00_JPRB, 0.45787E+00_JPRB, 0.46034E+00_JPRB ,&
&  0.46341E+00_JPRB, 0.46665E+00_JPRB, 0.46851E+00_JPRB, 0.46868E+00_JPRB, 0.46992E+00_JPRB, 0.47088E+00_JPRB, 0.47148E+00_JPRB ,&
&  0.47176E+00_JPRB, 0.47197E+00_JPRB, 0.47129E+00_JPRB, 0.46979E+00_JPRB, 0.46810E+00_JPRB, 0.46637E+00_JPRB, 0.46414E+00_JPRB ,&
&  0.46218E+00_JPRB, 0.46029E+00_JPRB, 0.45748E+00_JPRB, 0.45333E+00_JPRB, 0.44904E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.45096E+00_JPRB, 0.44916E+00_JPRB, 0.44757E+00_JPRB, 0.44632E+00_JPRB, 0.44502E+00_JPRB, 0.44428E+00_JPRB, 0.44368E+00_JPRB ,&
&  0.44318E+00_JPRB, 0.44251E+00_JPRB, 0.44161E+00_JPRB, 0.44103E+00_JPRB, 0.44084E+00_JPRB, 0.44086E+00_JPRB, 0.44029E+00_JPRB ,&
&  0.43990E+00_JPRB, 0.43938E+00_JPRB, 0.43914E+00_JPRB, 0.43863E+00_JPRB, 0.43792E+00_JPRB, 0.43706E+00_JPRB, 0.43617E+00_JPRB ,&
&  0.43541E+00_JPRB, 0.43438E+00_JPRB, 0.43310E+00_JPRB, 0.43098E+00_JPRB, 0.42904E+00_JPRB, 0.42690E+00_JPRB, 0.42463E+00_JPRB ,&
&  0.42207E+00_JPRB, 0.42000E+00_JPRB, 0.41820E+00_JPRB, 0.41649E+00_JPRB, 0.41772E+00_JPRB, 0.41837E+00_JPRB, 0.41869E+00_JPRB ,&
&  0.41845E+00_JPRB, 0.41664E+00_JPRB, 0.41610E+00_JPRB, 0.41567E+00_JPRB, 0.41534E+00_JPRB, 0.41694E+00_JPRB, 0.41804E+00_JPRB ,&
&  0.41880E+00_JPRB, 0.41928E+00_JPRB, 0.42074E+00_JPRB, 0.42145E+00_JPRB, 0.42177E+00_JPRB, 0.42148E+00_JPRB, 0.42077E+00_JPRB ,&
&  0.42096E+00_JPRB, 0.42116E+00_JPRB, 0.42114E+00_JPRB, 0.42072E+00_JPRB, 0.42074E+00_JPRB, 0.42068E+00_JPRB, 0.42011E+00_JPRB ,&
&  0.41930E+00_JPRB, 0.41892E+00_JPRB, 0.41842E+00_JPRB, 0.41779E+00_JPRB, 0.41705E+00_JPRB, 0.41680E+00_JPRB, 0.41656E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.41669E+00_JPRB, 0.41714E+00_JPRB, 0.41772E+00_JPRB, 0.41851E+00_JPRB, 0.41996E+00_JPRB, 0.42214E+00_JPRB ,&
&  0.42483E+00_JPRB, 0.42805E+00_JPRB, 0.43134E+00_JPRB, 0.43414E+00_JPRB, 0.43647E+00_JPRB, 0.43862E+00_JPRB, 0.44117E+00_JPRB ,&
&  0.44366E+00_JPRB, 0.44569E+00_JPRB, 0.44760E+00_JPRB, 0.44943E+00_JPRB, 0.45150E+00_JPRB, 0.45281E+00_JPRB, 0.45369E+00_JPRB ,&
&  0.45434E+00_JPRB, 0.45544E+00_JPRB, 0.45650E+00_JPRB, 0.45757E+00_JPRB, 0.45832E+00_JPRB, 0.45850E+00_JPRB, 0.45824E+00_JPRB ,&
&  0.45777E+00_JPRB, 0.45729E+00_JPRB, 0.45644E+00_JPRB, 0.45482E+00_JPRB, 0.45306E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.44882E+00_JPRB, 0.44835E+00_JPRB, 0.44778E+00_JPRB, 0.44721E+00_JPRB, 0.44655E+00_JPRB, 0.44595E+00_JPRB, 0.44528E+00_JPRB ,&
&  0.44473E+00_JPRB, 0.44414E+00_JPRB, 0.44370E+00_JPRB, 0.44321E+00_JPRB, 0.44285E+00_JPRB, 0.44243E+00_JPRB, 0.44203E+00_JPRB ,&
&  0.44157E+00_JPRB, 0.44124E+00_JPRB, 0.44068E+00_JPRB, 0.44019E+00_JPRB, 0.43952E+00_JPRB, 0.43897E+00_JPRB, 0.43825E+00_JPRB ,&
&  0.43769E+00_JPRB, 0.43693E+00_JPRB, 0.43631E+00_JPRB, 0.43535E+00_JPRB, 0.43462E+00_JPRB, 0.43381E+00_JPRB, 0.43314E+00_JPRB ,&
&  0.43248E+00_JPRB, 0.43203E+00_JPRB, 0.43159E+00_JPRB, 0.43106E+00_JPRB, 0.43075E+00_JPRB, 0.43086E+00_JPRB, 0.43097E+00_JPRB ,&
&  0.43101E+00_JPRB, 0.43116E+00_JPRB, 0.43149E+00_JPRB, 0.43164E+00_JPRB, 0.43138E+00_JPRB, 0.43119E+00_JPRB, 0.43105E+00_JPRB ,&
&  0.43083E+00_JPRB, 0.43044E+00_JPRB, 0.43006E+00_JPRB, 0.42974E+00_JPRB, 0.42927E+00_JPRB, 0.42866E+00_JPRB, 0.42820E+00_JPRB ,&
&  0.42798E+00_JPRB, 0.42776E+00_JPRB, 0.42748E+00_JPRB, 0.42708E+00_JPRB, 0.42683E+00_JPRB, 0.42658E+00_JPRB, 0.42627E+00_JPRB ,&
&  0.42595E+00_JPRB, 0.42566E+00_JPRB, 0.42527E+00_JPRB, 0.42484E+00_JPRB, 0.42447E+00_JPRB, 0.42437E+00_JPRB, 0.42442E+00_JPRB ,&
&  0.42459E+00_JPRB, 0.42498E+00_JPRB, 0.42551E+00_JPRB, 0.42606E+00_JPRB, 0.42666E+00_JPRB, 0.42743E+00_JPRB, 0.42829E+00_JPRB ,&
&  0.42922E+00_JPRB, 0.43021E+00_JPRB, 0.43142E+00_JPRB, 0.43278E+00_JPRB, 0.43409E+00_JPRB, 0.43540E+00_JPRB, 0.43695E+00_JPRB ,&
&  0.43830E+00_JPRB, 0.43941E+00_JPRB, 0.44030E+00_JPRB, 0.44131E+00_JPRB, 0.44253E+00_JPRB, 0.44350E+00_JPRB, 0.44430E+00_JPRB ,&
&  0.44496E+00_JPRB, 0.44564E+00_JPRB, 0.44627E+00_JPRB, 0.44697E+00_JPRB, 0.44738E+00_JPRB, 0.44777E+00_JPRB, 0.44818E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44879E+00_JPRB, 0.44902E+00_JPRB, 0.44914E+00_JPRB, 0.44911E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB, 0.43829E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB ,&
&  0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB, 0.15832E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.17170E+00_JPRB, 0.17198E+00_JPRB, 0.17253E+00_JPRB, 0.17317E+00_JPRB, 0.17386E+00_JPRB, 0.17470E+00_JPRB, 0.17502E+00_JPRB ,&
&  0.17515E+00_JPRB, 0.17514E+00_JPRB, 0.17547E+00_JPRB, 0.17586E+00_JPRB, 0.17612E+00_JPRB, 0.17611E+00_JPRB, 0.17573E+00_JPRB ,&
&  0.17509E+00_JPRB, 0.17434E+00_JPRB, 0.17330E+00_JPRB, 0.17283E+00_JPRB, 0.17246E+00_JPRB, 0.17206E+00_JPRB, 0.17129E+00_JPRB ,&
&  0.17060E+00_JPRB, 0.16940E+00_JPRB, 0.16772E+00_JPRB, 0.16631E+00_JPRB, 0.16596E+00_JPRB, 0.16606E+00_JPRB, 0.16648E+00_JPRB ,&
&  0.16682E+00_JPRB, 0.16701E+00_JPRB, 0.16703E+00_JPRB, 0.16712E+00_JPRB, 0.16727E+00_JPRB, 0.16711E+00_JPRB, 0.16684E+00_JPRB ,&
&  0.16651E+00_JPRB, 0.16600E+00_JPRB, 0.16551E+00_JPRB, 0.16507E+00_JPRB, 0.16465E+00_JPRB, 0.16418E+00_JPRB, 0.16363E+00_JPRB ,&
&  0.16300E+00_JPRB, 0.16223E+00_JPRB, 0.16133E+00_JPRB, 0.16060E+00_JPRB, 0.15985E+00_JPRB, 0.15925E+00_JPRB, 0.15853E+00_JPRB ,&
&  0.15793E+00_JPRB, 0.15738E+00_JPRB, 0.15682E+00_JPRB, 0.15629E+00_JPRB, 0.15579E+00_JPRB, 0.15530E+00_JPRB, 0.15482E+00_JPRB ,&
&  0.15436E+00_JPRB, 0.15398E+00_JPRB, 0.15355E+00_JPRB, 0.15315E+00_JPRB, 0.15288E+00_JPRB, 0.15266E+00_JPRB, 0.15246E+00_JPRB ,&
&  0.15229E+00_JPRB, 0.15223E+00_JPRB, 0.15221E+00_JPRB, 0.15218E+00_JPRB, 0.15216E+00_JPRB, 0.15226E+00_JPRB, 0.15237E+00_JPRB ,&
&  0.15253E+00_JPRB, 0.15264E+00_JPRB, 0.15277E+00_JPRB, 0.15296E+00_JPRB, 0.15317E+00_JPRB, 0.15340E+00_JPRB, 0.15373E+00_JPRB ,&
&  0.15412E+00_JPRB, 0.15460E+00_JPRB, 0.15509E+00_JPRB, 0.15544E+00_JPRB, 0.15609E+00_JPRB, 0.15681E+00_JPRB, 0.15764E+00_JPRB ,&
&  0.15857E+00_JPRB, 0.15964E+00_JPRB, 0.16077E+00_JPRB, 0.16203E+00_JPRB, 0.16330E+00_JPRB, 0.16415E+00_JPRB, 0.16500E+00_JPRB ,&
&  0.16591E+00_JPRB, 0.16695E+00_JPRB, 0.16795E+00_JPRB, 0.16899E+00_JPRB, 0.17025E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.19355E+00_JPRB, 0.20054E+00_JPRB, 0.20767E+00_JPRB, 0.21779E+00_JPRB, 0.22582E+00_JPRB, 0.26103E+00_JPRB, 0.25591E+00_JPRB ,&
&  0.25227E+00_JPRB, 0.23792E+00_JPRB, 0.23611E+00_JPRB, 0.23236E+00_JPRB, 0.22778E+00_JPRB, 0.22277E+00_JPRB, 0.21778E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.20764E+00_JPRB, 0.20283E+00_JPRB, 0.19909E+00_JPRB, 0.19573E+00_JPRB, 0.19246E+00_JPRB, 0.18957E+00_JPRB ,&
&  0.18826E+00_JPRB, 0.18801E+00_JPRB, 0.18717E+00_JPRB, 0.18342E+00_JPRB, 0.18010E+00_JPRB, 0.17803E+00_JPRB, 0.17732E+00_JPRB ,&
&  0.17720E+00_JPRB, 0.17859E+00_JPRB, 0.17931E+00_JPRB, 0.17984E+00_JPRB, 0.18070E+00_JPRB, 0.17988E+00_JPRB, 0.17905E+00_JPRB ,&
&  0.17792E+00_JPRB, 0.17663E+00_JPRB, 0.17526E+00_JPRB, 0.17409E+00_JPRB, 0.17291E+00_JPRB, 0.17119E+00_JPRB, 0.16966E+00_JPRB ,&
&  0.16811E+00_JPRB, 0.16623E+00_JPRB, 0.16384E+00_JPRB, 0.16225E+00_JPRB, 0.16043E+00_JPRB, 0.15887E+00_JPRB, 0.15733E+00_JPRB ,&
&  0.15619E+00_JPRB, 0.15514E+00_JPRB, 0.15388E+00_JPRB, 0.15234E+00_JPRB, 0.15080E+00_JPRB, 0.14942E+00_JPRB, 0.14810E+00_JPRB ,&
&  0.14736E+00_JPRB, 0.14661E+00_JPRB, 0.14587E+00_JPRB, 0.14508E+00_JPRB, 0.14468E+00_JPRB, 0.14428E+00_JPRB, 0.14393E+00_JPRB ,&
&  0.14363E+00_JPRB, 0.14350E+00_JPRB, 0.14355E+00_JPRB, 0.14365E+00_JPRB, 0.14375E+00_JPRB, 0.14410E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14510E+00_JPRB, 0.14581E+00_JPRB, 0.14676E+00_JPRB, 0.14774E+00_JPRB, 0.14864E+00_JPRB, 0.14937E+00_JPRB, 0.15035E+00_JPRB ,&
&  0.15190E+00_JPRB, 0.15371E+00_JPRB, 0.15567E+00_JPRB, 0.15725E+00_JPRB, 0.15934E+00_JPRB, 0.16124E+00_JPRB, 0.16312E+00_JPRB ,&
&  0.16499E+00_JPRB, 0.16713E+00_JPRB, 0.16946E+00_JPRB, 0.17191E+00_JPRB, 0.17468E+00_JPRB, 0.17731E+00_JPRB, 0.17975E+00_JPRB ,&
&  0.18224E+00_JPRB, 0.18560E+00_JPRB, 0.18771E+00_JPRB, 0.18909E+00_JPRB, 0.19091E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.26643E+00_JPRB, 0.27112E+00_JPRB, 0.27473E+00_JPRB, 0.27703E+00_JPRB, 0.27913E+00_JPRB, 0.28067E+00_JPRB, 0.28120E+00_JPRB ,&
&  0.27480E+00_JPRB, 0.26977E+00_JPRB, 0.26288E+00_JPRB, 0.25468E+00_JPRB, 0.24722E+00_JPRB, 0.24079E+00_JPRB, 0.23306E+00_JPRB ,&
&  0.22675E+00_JPRB, 0.22078E+00_JPRB, 0.21438E+00_JPRB, 0.20774E+00_JPRB, 0.20075E+00_JPRB, 0.19373E+00_JPRB, 0.18996E+00_JPRB ,&
&  0.18866E+00_JPRB, 0.18938E+00_JPRB, 0.19091E+00_JPRB, 0.19244E+00_JPRB, 0.19337E+00_JPRB, 0.19404E+00_JPRB, 0.19293E+00_JPRB ,&
&  0.18937E+00_JPRB, 0.18983E+00_JPRB, 0.19047E+00_JPRB, 0.19085E+00_JPRB, 0.19215E+00_JPRB, 0.19331E+00_JPRB, 0.19304E+00_JPRB ,&
&  0.19135E+00_JPRB, 0.18904E+00_JPRB, 0.18588E+00_JPRB, 0.18294E+00_JPRB, 0.18167E+00_JPRB, 0.17942E+00_JPRB, 0.17688E+00_JPRB ,&
&  0.17463E+00_JPRB, 0.17079E+00_JPRB, 0.16708E+00_JPRB, 0.16447E+00_JPRB, 0.16121E+00_JPRB, 0.15895E+00_JPRB, 0.15602E+00_JPRB ,&
&  0.15363E+00_JPRB, 0.15141E+00_JPRB, 0.14897E+00_JPRB, 0.14704E+00_JPRB, 0.14522E+00_JPRB, 0.14386E+00_JPRB, 0.14252E+00_JPRB ,&
&  0.14173E+00_JPRB, 0.14102E+00_JPRB, 0.14057E+00_JPRB, 0.14009E+00_JPRB, 0.13983E+00_JPRB, 0.13956E+00_JPRB, 0.13936E+00_JPRB ,&
&  0.13925E+00_JPRB, 0.13955E+00_JPRB, 0.13977E+00_JPRB, 0.14017E+00_JPRB, 0.14064E+00_JPRB, 0.14096E+00_JPRB, 0.14149E+00_JPRB ,&
&  0.14248E+00_JPRB, 0.14354E+00_JPRB, 0.14430E+00_JPRB, 0.14525E+00_JPRB, 0.14631E+00_JPRB, 0.14762E+00_JPRB, 0.14935E+00_JPRB ,&
&  0.15117E+00_JPRB, 0.15320E+00_JPRB, 0.15547E+00_JPRB, 0.15774E+00_JPRB, 0.16060E+00_JPRB, 0.16352E+00_JPRB, 0.16660E+00_JPRB ,&
&  0.17021E+00_JPRB, 0.17407E+00_JPRB, 0.17799E+00_JPRB, 0.18231E+00_JPRB, 0.18853E+00_JPRB, 0.19462E+00_JPRB, 0.20221E+00_JPRB ,&
&  0.20975E+00_JPRB, 0.21643E+00_JPRB, 0.24798E+00_JPRB, 0.25282E+00_JPRB, 0.25946E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.27309E+00_JPRB, 0.28376E+00_JPRB, 0.29254E+00_JPRB, 0.29801E+00_JPRB, 0.30375E+00_JPRB, 0.30606E+00_JPRB, 0.31470E+00_JPRB ,&
&  0.30922E+00_JPRB, 0.29098E+00_JPRB, 0.27980E+00_JPRB, 0.26345E+00_JPRB, 0.24317E+00_JPRB, 0.22674E+00_JPRB, 0.21861E+00_JPRB ,&
&  0.21385E+00_JPRB, 0.21018E+00_JPRB, 0.20567E+00_JPRB, 0.20069E+00_JPRB, 0.19564E+00_JPRB, 0.19192E+00_JPRB, 0.18943E+00_JPRB ,&
&  0.18720E+00_JPRB, 0.18759E+00_JPRB, 0.18983E+00_JPRB, 0.19387E+00_JPRB, 0.19956E+00_JPRB, 0.20383E+00_JPRB, 0.20708E+00_JPRB ,&
&  0.20977E+00_JPRB, 0.20156E+00_JPRB, 0.20245E+00_JPRB, 0.20379E+00_JPRB, 0.20226E+00_JPRB, 0.20448E+00_JPRB, 0.20790E+00_JPRB ,&
&  0.20614E+00_JPRB, 0.20079E+00_JPRB, 0.20375E+00_JPRB, 0.19986E+00_JPRB, 0.19168E+00_JPRB, 0.18940E+00_JPRB, 0.18364E+00_JPRB ,&
&  0.18005E+00_JPRB, 0.17291E+00_JPRB, 0.16790E+00_JPRB, 0.16461E+00_JPRB, 0.15982E+00_JPRB, 0.15778E+00_JPRB, 0.15463E+00_JPRB ,&
&  0.15131E+00_JPRB, 0.14887E+00_JPRB, 0.14600E+00_JPRB, 0.14447E+00_JPRB, 0.14318E+00_JPRB, 0.14205E+00_JPRB, 0.14116E+00_JPRB ,&
&  0.14068E+00_JPRB, 0.14023E+00_JPRB, 0.14028E+00_JPRB, 0.14034E+00_JPRB, 0.14042E+00_JPRB, 0.14079E+00_JPRB, 0.14106E+00_JPRB ,&
&  0.14168E+00_JPRB, 0.14237E+00_JPRB, 0.14292E+00_JPRB, 0.14377E+00_JPRB, 0.14419E+00_JPRB, 0.14471E+00_JPRB, 0.14512E+00_JPRB ,&
&  0.14559E+00_JPRB, 0.14626E+00_JPRB, 0.14668E+00_JPRB, 0.14739E+00_JPRB, 0.14821E+00_JPRB, 0.14969E+00_JPRB, 0.15116E+00_JPRB ,&
&  0.15294E+00_JPRB, 0.15495E+00_JPRB, 0.15712E+00_JPRB, 0.15940E+00_JPRB, 0.16221E+00_JPRB, 0.16485E+00_JPRB, 0.16827E+00_JPRB ,&
&  0.17197E+00_JPRB, 0.17547E+00_JPRB, 0.17938E+00_JPRB, 0.18447E+00_JPRB, 0.19109E+00_JPRB, 0.20050E+00_JPRB, 0.20961E+00_JPRB ,&
&  0.22009E+00_JPRB, 0.24691E+00_JPRB, 0.25329E+00_JPRB, 0.25647E+00_JPRB, 0.26472E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.29951E+00_JPRB, 0.30883E+00_JPRB, 0.32287E+00_JPRB, 0.33184E+00_JPRB, 0.33915E+00_JPRB, 0.33501E+00_JPRB, 0.33112E+00_JPRB ,&
&  0.32710E+00_JPRB, 0.31285E+00_JPRB, 0.29575E+00_JPRB, 0.26653E+00_JPRB, 0.24188E+00_JPRB, 0.22302E+00_JPRB, 0.21279E+00_JPRB ,&
&  0.20687E+00_JPRB, 0.20501E+00_JPRB, 0.20340E+00_JPRB, 0.20314E+00_JPRB, 0.20226E+00_JPRB, 0.20407E+00_JPRB, 0.20560E+00_JPRB ,&
&  0.20534E+00_JPRB, 0.20420E+00_JPRB, 0.20347E+00_JPRB, 0.20652E+00_JPRB, 0.21238E+00_JPRB, 0.21701E+00_JPRB, 0.22142E+00_JPRB ,&
&  0.22475E+00_JPRB, 0.22230E+00_JPRB, 0.22322E+00_JPRB, 0.22107E+00_JPRB, 0.21847E+00_JPRB, 0.21821E+00_JPRB, 0.21876E+00_JPRB ,&
&  0.21921E+00_JPRB, 0.21996E+00_JPRB, 0.21940E+00_JPRB, 0.21673E+00_JPRB, 0.20549E+00_JPRB, 0.19957E+00_JPRB, 0.19211E+00_JPRB ,&
&  0.18411E+00_JPRB, 0.17485E+00_JPRB, 0.16756E+00_JPRB, 0.16399E+00_JPRB, 0.15999E+00_JPRB, 0.15831E+00_JPRB, 0.15525E+00_JPRB ,&
&  0.15176E+00_JPRB, 0.14841E+00_JPRB, 0.14602E+00_JPRB, 0.14480E+00_JPRB, 0.14434E+00_JPRB, 0.14346E+00_JPRB, 0.14311E+00_JPRB ,&
&  0.14330E+00_JPRB, 0.14358E+00_JPRB, 0.14430E+00_JPRB, 0.14506E+00_JPRB, 0.14594E+00_JPRB, 0.14678E+00_JPRB, 0.14783E+00_JPRB ,&
&  0.14940E+00_JPRB, 0.15123E+00_JPRB, 0.15207E+00_JPRB, 0.15276E+00_JPRB, 0.15255E+00_JPRB, 0.15219E+00_JPRB, 0.15160E+00_JPRB ,&
&  0.15113E+00_JPRB, 0.15101E+00_JPRB, 0.15101E+00_JPRB, 0.15098E+00_JPRB, 0.15146E+00_JPRB, 0.15234E+00_JPRB, 0.15332E+00_JPRB ,&
&  0.15485E+00_JPRB, 0.15625E+00_JPRB, 0.15782E+00_JPRB, 0.15960E+00_JPRB, 0.16169E+00_JPRB, 0.16412E+00_JPRB, 0.16688E+00_JPRB ,&
&  0.16946E+00_JPRB, 0.17265E+00_JPRB, 0.17536E+00_JPRB, 0.17750E+00_JPRB, 0.18208E+00_JPRB, 0.18892E+00_JPRB, 0.20245E+00_JPRB ,&
&  0.21863E+00_JPRB, 0.25944E+00_JPRB, 0.30235E+00_JPRB, 0.28696E+00_JPRB, 0.29159E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.29849E+00_JPRB, 0.31040E+00_JPRB, 0.32570E+00_JPRB, 0.33507E+00_JPRB, 0.34015E+00_JPRB, 0.34267E+00_JPRB, 0.34138E+00_JPRB ,&
&  0.33796E+00_JPRB, 0.33753E+00_JPRB, 0.34871E+00_JPRB, 0.33123E+00_JPRB, 0.31336E+00_JPRB, 0.27978E+00_JPRB, 0.26336E+00_JPRB ,&
&  0.25126E+00_JPRB, 0.24922E+00_JPRB, 0.24915E+00_JPRB, 0.25395E+00_JPRB, 0.25802E+00_JPRB, 0.26127E+00_JPRB, 0.25786E+00_JPRB ,&
&  0.25283E+00_JPRB, 0.24378E+00_JPRB, 0.23619E+00_JPRB, 0.22898E+00_JPRB, 0.22898E+00_JPRB, 0.22791E+00_JPRB, 0.22534E+00_JPRB ,&
&  0.22230E+00_JPRB, 0.22025E+00_JPRB, 0.21998E+00_JPRB, 0.21982E+00_JPRB, 0.23144E+00_JPRB, 0.23101E+00_JPRB, 0.23062E+00_JPRB ,&
&  0.23006E+00_JPRB, 0.22999E+00_JPRB, 0.22905E+00_JPRB, 0.22734E+00_JPRB, 0.22275E+00_JPRB, 0.21131E+00_JPRB, 0.21119E+00_JPRB ,&
&  0.20132E+00_JPRB, 0.19061E+00_JPRB, 0.18046E+00_JPRB, 0.17343E+00_JPRB, 0.16914E+00_JPRB, 0.16706E+00_JPRB, 0.16245E+00_JPRB ,&
&  0.15756E+00_JPRB, 0.15205E+00_JPRB, 0.14932E+00_JPRB, 0.14889E+00_JPRB, 0.14934E+00_JPRB, 0.14987E+00_JPRB, 0.15036E+00_JPRB ,&
&  0.15148E+00_JPRB, 0.15205E+00_JPRB, 0.15236E+00_JPRB, 0.15297E+00_JPRB, 0.15357E+00_JPRB, 0.15452E+00_JPRB, 0.15615E+00_JPRB ,&
&  0.15908E+00_JPRB, 0.16234E+00_JPRB, 0.16530E+00_JPRB, 0.16752E+00_JPRB, 0.16657E+00_JPRB, 0.16366E+00_JPRB, 0.15969E+00_JPRB ,&
&  0.15686E+00_JPRB, 0.15602E+00_JPRB, 0.15524E+00_JPRB, 0.15527E+00_JPRB, 0.15554E+00_JPRB, 0.15602E+00_JPRB, 0.15667E+00_JPRB ,&
&  0.15749E+00_JPRB, 0.15816E+00_JPRB, 0.15900E+00_JPRB, 0.16005E+00_JPRB, 0.16159E+00_JPRB, 0.16298E+00_JPRB, 0.16496E+00_JPRB ,&
&  0.16791E+00_JPRB, 0.17113E+00_JPRB, 0.17569E+00_JPRB, 0.17975E+00_JPRB, 0.18495E+00_JPRB, 0.19288E+00_JPRB, 0.20595E+00_JPRB ,&
&  0.22027E+00_JPRB, 0.25517E+00_JPRB, 0.27152E+00_JPRB, 0.28123E+00_JPRB, 0.28824E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.29615E+00_JPRB, 0.30493E+00_JPRB, 0.31709E+00_JPRB, 0.32744E+00_JPRB, 0.33629E+00_JPRB, 0.34474E+00_JPRB, 0.35191E+00_JPRB ,&
&  0.35555E+00_JPRB, 0.35765E+00_JPRB, 0.35837E+00_JPRB, 0.36176E+00_JPRB, 0.35883E+00_JPRB, 0.35239E+00_JPRB, 0.34700E+00_JPRB ,&
&  0.34261E+00_JPRB, 0.33909E+00_JPRB, 0.33391E+00_JPRB, 0.33119E+00_JPRB, 0.33099E+00_JPRB, 0.33119E+00_JPRB, 0.32562E+00_JPRB ,&
&  0.31547E+00_JPRB, 0.31500E+00_JPRB, 0.31992E+00_JPRB, 0.31486E+00_JPRB, 0.30941E+00_JPRB, 0.30431E+00_JPRB, 0.29862E+00_JPRB ,&
&  0.28336E+00_JPRB, 0.26379E+00_JPRB, 0.23653E+00_JPRB, 0.23137E+00_JPRB, 0.24726E+00_JPRB, 0.24157E+00_JPRB, 0.24421E+00_JPRB ,&
&  0.24445E+00_JPRB, 0.24492E+00_JPRB, 0.24344E+00_JPRB, 0.24203E+00_JPRB, 0.23848E+00_JPRB, 0.23710E+00_JPRB, 0.23579E+00_JPRB ,&
&  0.23079E+00_JPRB, 0.22608E+00_JPRB, 0.21862E+00_JPRB, 0.20150E+00_JPRB, 0.19578E+00_JPRB, 0.19335E+00_JPRB, 0.18810E+00_JPRB ,&
&  0.18045E+00_JPRB, 0.17421E+00_JPRB, 0.17134E+00_JPRB, 0.16814E+00_JPRB, 0.17017E+00_JPRB, 0.17169E+00_JPRB, 0.17169E+00_JPRB ,&
&  0.17168E+00_JPRB, 0.17133E+00_JPRB, 0.16907E+00_JPRB, 0.16538E+00_JPRB, 0.16252E+00_JPRB, 0.16139E+00_JPRB, 0.16179E+00_JPRB ,&
&  0.16392E+00_JPRB, 0.16840E+00_JPRB, 0.17874E+00_JPRB, 0.18698E+00_JPRB, 0.18631E+00_JPRB, 0.18155E+00_JPRB, 0.17291E+00_JPRB ,&
&  0.16608E+00_JPRB, 0.16341E+00_JPRB, 0.16119E+00_JPRB, 0.16009E+00_JPRB, 0.16050E+00_JPRB, 0.16194E+00_JPRB, 0.16285E+00_JPRB ,&
&  0.16387E+00_JPRB, 0.16480E+00_JPRB, 0.16562E+00_JPRB, 0.16619E+00_JPRB, 0.16708E+00_JPRB, 0.16809E+00_JPRB, 0.16990E+00_JPRB ,&
&  0.17261E+00_JPRB, 0.17689E+00_JPRB, 0.18256E+00_JPRB, 0.18861E+00_JPRB, 0.19292E+00_JPRB, 0.19498E+00_JPRB, 0.19828E+00_JPRB ,&
&  0.20393E+00_JPRB, 0.21805E+00_JPRB, 0.24608E+00_JPRB, 0.29189E+00_JPRB, 0.29635E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.29938E+00_JPRB, 0.30369E+00_JPRB, 0.30650E+00_JPRB, 0.31002E+00_JPRB, 0.31758E+00_JPRB, 0.32836E+00_JPRB, 0.34055E+00_JPRB ,&
&  0.35155E+00_JPRB, 0.36026E+00_JPRB, 0.36448E+00_JPRB, 0.36956E+00_JPRB, 0.37448E+00_JPRB, 0.37691E+00_JPRB, 0.38219E+00_JPRB ,&
&  0.38297E+00_JPRB, 0.37965E+00_JPRB, 0.37363E+00_JPRB, 0.36562E+00_JPRB, 0.36142E+00_JPRB, 0.36282E+00_JPRB, 0.36589E+00_JPRB ,&
&  0.36798E+00_JPRB, 0.36961E+00_JPRB, 0.36980E+00_JPRB, 0.36657E+00_JPRB, 0.36335E+00_JPRB, 0.36150E+00_JPRB, 0.35773E+00_JPRB ,&
&  0.34963E+00_JPRB, 0.30056E+00_JPRB, 0.26276E+00_JPRB, 0.25787E+00_JPRB, 0.27527E+00_JPRB, 0.27776E+00_JPRB, 0.27231E+00_JPRB ,&
&  0.26644E+00_JPRB, 0.26190E+00_JPRB, 0.25933E+00_JPRB, 0.25725E+00_JPRB, 0.25460E+00_JPRB, 0.25238E+00_JPRB, 0.25338E+00_JPRB ,&
&  0.25469E+00_JPRB, 0.25249E+00_JPRB, 0.24939E+00_JPRB, 0.24317E+00_JPRB, 0.23860E+00_JPRB, 0.23627E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.23007E+00_JPRB, 0.22596E+00_JPRB, 0.22560E+00_JPRB, 0.22643E+00_JPRB, 0.22791E+00_JPRB, 0.22895E+00_JPRB, 0.22645E+00_JPRB ,&
&  0.21976E+00_JPRB, 0.21344E+00_JPRB, 0.20515E+00_JPRB, 0.19536E+00_JPRB, 0.18594E+00_JPRB, 0.17947E+00_JPRB, 0.17450E+00_JPRB ,&
&  0.17332E+00_JPRB, 0.17801E+00_JPRB, 0.19007E+00_JPRB, 0.20557E+00_JPRB, 0.22727E+00_JPRB, 0.21503E+00_JPRB, 0.20338E+00_JPRB ,&
&  0.19313E+00_JPRB, 0.18525E+00_JPRB, 0.17937E+00_JPRB, 0.17662E+00_JPRB, 0.17559E+00_JPRB, 0.17570E+00_JPRB, 0.17679E+00_JPRB ,&
&  0.17872E+00_JPRB, 0.18132E+00_JPRB, 0.18376E+00_JPRB, 0.18494E+00_JPRB, 0.18569E+00_JPRB, 0.18638E+00_JPRB, 0.18648E+00_JPRB ,&
&  0.18585E+00_JPRB, 0.18795E+00_JPRB, 0.19444E+00_JPRB, 0.20280E+00_JPRB, 0.21113E+00_JPRB, 0.21874E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.24741E+00_JPRB, 0.26770E+00_JPRB, 0.27934E+00_JPRB, 0.28806E+00_JPRB, 0.29582E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.32428E+00_JPRB, 0.32573E+00_JPRB, 0.32318E+00_JPRB, 0.32025E+00_JPRB, 0.31981E+00_JPRB, 0.32165E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.33743E+00_JPRB, 0.34975E+00_JPRB, 0.36149E+00_JPRB, 0.37133E+00_JPRB, 0.38028E+00_JPRB, 0.38429E+00_JPRB, 0.39038E+00_JPRB ,&
&  0.39686E+00_JPRB, 0.40277E+00_JPRB, 0.40355E+00_JPRB, 0.39333E+00_JPRB, 0.38338E+00_JPRB, 0.37811E+00_JPRB, 0.37815E+00_JPRB ,&
&  0.38004E+00_JPRB, 0.38284E+00_JPRB, 0.38919E+00_JPRB, 0.39170E+00_JPRB, 0.38599E+00_JPRB, 0.38475E+00_JPRB, 0.38275E+00_JPRB ,&
&  0.37743E+00_JPRB, 0.36679E+00_JPRB, 0.33480E+00_JPRB, 0.32974E+00_JPRB, 0.32183E+00_JPRB, 0.31295E+00_JPRB, 0.30389E+00_JPRB ,&
&  0.29368E+00_JPRB, 0.28357E+00_JPRB, 0.27672E+00_JPRB, 0.27276E+00_JPRB, 0.27361E+00_JPRB, 0.27484E+00_JPRB, 0.27428E+00_JPRB ,&
&  0.27625E+00_JPRB, 0.27906E+00_JPRB, 0.28123E+00_JPRB, 0.28133E+00_JPRB, 0.28070E+00_JPRB, 0.27921E+00_JPRB, 0.27427E+00_JPRB ,&
&  0.26977E+00_JPRB, 0.26719E+00_JPRB, 0.26515E+00_JPRB, 0.26334E+00_JPRB, 0.26281E+00_JPRB, 0.26304E+00_JPRB, 0.26306E+00_JPRB ,&
&  0.26095E+00_JPRB, 0.25730E+00_JPRB, 0.25320E+00_JPRB, 0.24646E+00_JPRB, 0.23541E+00_JPRB, 0.21944E+00_JPRB, 0.21003E+00_JPRB ,&
&  0.20836E+00_JPRB, 0.21942E+00_JPRB, 0.24139E+00_JPRB, 0.24913E+00_JPRB, 0.25637E+00_JPRB, 0.26049E+00_JPRB, 0.26528E+00_JPRB ,&
&  0.26197E+00_JPRB, 0.23700E+00_JPRB, 0.22349E+00_JPRB, 0.21678E+00_JPRB, 0.21109E+00_JPRB, 0.20645E+00_JPRB, 0.20627E+00_JPRB ,&
&  0.20930E+00_JPRB, 0.21599E+00_JPRB, 0.22233E+00_JPRB, 0.22540E+00_JPRB, 0.22480E+00_JPRB, 0.22133E+00_JPRB, 0.21796E+00_JPRB ,&
&  0.21478E+00_JPRB, 0.21897E+00_JPRB, 0.23357E+00_JPRB, 0.25710E+00_JPRB, 0.26510E+00_JPRB, 0.27118E+00_JPRB, 0.27688E+00_JPRB ,&
&  0.28293E+00_JPRB, 0.29211E+00_JPRB, 0.30341E+00_JPRB, 0.31246E+00_JPRB, 0.32052E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.33972E+00_JPRB, 0.33946E+00_JPRB, 0.34086E+00_JPRB, 0.33971E+00_JPRB, 0.33714E+00_JPRB, 0.33542E+00_JPRB, 0.33930E+00_JPRB ,&
&  0.34637E+00_JPRB, 0.35786E+00_JPRB, 0.37300E+00_JPRB, 0.38630E+00_JPRB, 0.39441E+00_JPRB, 0.39946E+00_JPRB, 0.40721E+00_JPRB ,&
&  0.41435E+00_JPRB, 0.42227E+00_JPRB, 0.42241E+00_JPRB, 0.41353E+00_JPRB, 0.40597E+00_JPRB, 0.40197E+00_JPRB, 0.40138E+00_JPRB ,&
&  0.40074E+00_JPRB, 0.40260E+00_JPRB, 0.40594E+00_JPRB, 0.40955E+00_JPRB, 0.40911E+00_JPRB, 0.40748E+00_JPRB, 0.41139E+00_JPRB ,&
&  0.40984E+00_JPRB, 0.40075E+00_JPRB, 0.39169E+00_JPRB, 0.36704E+00_JPRB, 0.36684E+00_JPRB, 0.35686E+00_JPRB, 0.34907E+00_JPRB ,&
&  0.33860E+00_JPRB, 0.32578E+00_JPRB, 0.31654E+00_JPRB, 0.31063E+00_JPRB, 0.30802E+00_JPRB, 0.30543E+00_JPRB, 0.30334E+00_JPRB ,&
&  0.30194E+00_JPRB, 0.30156E+00_JPRB, 0.30271E+00_JPRB, 0.30047E+00_JPRB, 0.29932E+00_JPRB, 0.30014E+00_JPRB, 0.30176E+00_JPRB ,&
&  0.30118E+00_JPRB, 0.30036E+00_JPRB, 0.29970E+00_JPRB, 0.29839E+00_JPRB, 0.29658E+00_JPRB, 0.29410E+00_JPRB, 0.29168E+00_JPRB ,&
&  0.29053E+00_JPRB, 0.29132E+00_JPRB, 0.29144E+00_JPRB, 0.28754E+00_JPRB, 0.27992E+00_JPRB, 0.27111E+00_JPRB, 0.26538E+00_JPRB ,&
&  0.26283E+00_JPRB, 0.26471E+00_JPRB, 0.27230E+00_JPRB, 0.28327E+00_JPRB, 0.29361E+00_JPRB, 0.29966E+00_JPRB, 0.30327E+00_JPRB ,&
&  0.30773E+00_JPRB, 0.30642E+00_JPRB, 0.30291E+00_JPRB, 0.29963E+00_JPRB, 0.29657E+00_JPRB, 0.29258E+00_JPRB, 0.29246E+00_JPRB ,&
&  0.29479E+00_JPRB, 0.29698E+00_JPRB, 0.29754E+00_JPRB, 0.29625E+00_JPRB, 0.29409E+00_JPRB, 0.28969E+00_JPRB, 0.28354E+00_JPRB ,&
&  0.27607E+00_JPRB, 0.26746E+00_JPRB, 0.26519E+00_JPRB, 0.26551E+00_JPRB, 0.26927E+00_JPRB, 0.27713E+00_JPRB, 0.28781E+00_JPRB ,&
&  0.29909E+00_JPRB, 0.30854E+00_JPRB, 0.31865E+00_JPRB, 0.32797E+00_JPRB, 0.33620E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.36444E+00_JPRB, 0.36639E+00_JPRB, 0.36875E+00_JPRB, 0.37179E+00_JPRB, 0.37518E+00_JPRB, 0.37737E+00_JPRB, 0.38145E+00_JPRB ,&
&  0.38374E+00_JPRB, 0.39003E+00_JPRB, 0.40049E+00_JPRB, 0.41112E+00_JPRB, 0.41899E+00_JPRB, 0.42577E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.43757E+00_JPRB, 0.44458E+00_JPRB, 0.44540E+00_JPRB, 0.44287E+00_JPRB, 0.44006E+00_JPRB, 0.43785E+00_JPRB, 0.43590E+00_JPRB ,&
&  0.43335E+00_JPRB, 0.43322E+00_JPRB, 0.43704E+00_JPRB, 0.44141E+00_JPRB, 0.44597E+00_JPRB, 0.44750E+00_JPRB, 0.44659E+00_JPRB ,&
&  0.45078E+00_JPRB, 0.44195E+00_JPRB, 0.43642E+00_JPRB, 0.42976E+00_JPRB, 0.42383E+00_JPRB, 0.41272E+00_JPRB, 0.40460E+00_JPRB ,&
&  0.39499E+00_JPRB, 0.38669E+00_JPRB, 0.38325E+00_JPRB, 0.37952E+00_JPRB, 0.36998E+00_JPRB, 0.35873E+00_JPRB, 0.35021E+00_JPRB ,&
&  0.34452E+00_JPRB, 0.33883E+00_JPRB, 0.33214E+00_JPRB, 0.32677E+00_JPRB, 0.32238E+00_JPRB, 0.32089E+00_JPRB, 0.32276E+00_JPRB ,&
&  0.32676E+00_JPRB, 0.33019E+00_JPRB, 0.33194E+00_JPRB, 0.33213E+00_JPRB, 0.33046E+00_JPRB, 0.32781E+00_JPRB, 0.32613E+00_JPRB ,&
&  0.32646E+00_JPRB, 0.32761E+00_JPRB, 0.32981E+00_JPRB, 0.33500E+00_JPRB, 0.33560E+00_JPRB, 0.33407E+00_JPRB, 0.32851E+00_JPRB ,&
&  0.32573E+00_JPRB, 0.32420E+00_JPRB, 0.32564E+00_JPRB, 0.33059E+00_JPRB, 0.33880E+00_JPRB, 0.34674E+00_JPRB, 0.35246E+00_JPRB ,&
&  0.35541E+00_JPRB, 0.35735E+00_JPRB, 0.36052E+00_JPRB, 0.36408E+00_JPRB, 0.36533E+00_JPRB, 0.36441E+00_JPRB, 0.36266E+00_JPRB ,&
&  0.36518E+00_JPRB, 0.37231E+00_JPRB, 0.37987E+00_JPRB, 0.38447E+00_JPRB, 0.38653E+00_JPRB, 0.38524E+00_JPRB, 0.37827E+00_JPRB ,&
&  0.36579E+00_JPRB, 0.35152E+00_JPRB, 0.33766E+00_JPRB, 0.32585E+00_JPRB, 0.31768E+00_JPRB, 0.31387E+00_JPRB, 0.31686E+00_JPRB ,&
&  0.32475E+00_JPRB, 0.33486E+00_JPRB, 0.34430E+00_JPRB, 0.35139E+00_JPRB, 0.35909E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.38893E+00_JPRB, 0.38913E+00_JPRB, 0.39215E+00_JPRB, 0.39867E+00_JPRB, 0.40304E+00_JPRB, 0.40710E+00_JPRB, 0.41490E+00_JPRB ,&
&  0.42186E+00_JPRB, 0.42798E+00_JPRB, 0.43541E+00_JPRB, 0.44370E+00_JPRB, 0.45020E+00_JPRB, 0.45939E+00_JPRB, 0.46796E+00_JPRB ,&
&  0.47342E+00_JPRB, 0.47556E+00_JPRB, 0.47669E+00_JPRB, 0.47763E+00_JPRB, 0.47727E+00_JPRB, 0.47458E+00_JPRB, 0.47400E+00_JPRB ,&
&  0.47215E+00_JPRB, 0.47017E+00_JPRB, 0.47241E+00_JPRB, 0.47290E+00_JPRB, 0.47489E+00_JPRB, 0.47815E+00_JPRB, 0.47890E+00_JPRB ,&
&  0.47754E+00_JPRB, 0.46529E+00_JPRB, 0.46118E+00_JPRB, 0.45876E+00_JPRB, 0.46341E+00_JPRB, 0.46491E+00_JPRB, 0.46385E+00_JPRB ,&
&  0.45791E+00_JPRB, 0.45445E+00_JPRB, 0.45658E+00_JPRB, 0.45911E+00_JPRB, 0.45352E+00_JPRB, 0.44066E+00_JPRB, 0.43021E+00_JPRB ,&
&  0.42184E+00_JPRB, 0.40955E+00_JPRB, 0.39202E+00_JPRB, 0.38189E+00_JPRB, 0.37765E+00_JPRB, 0.37276E+00_JPRB, 0.36805E+00_JPRB ,&
&  0.36547E+00_JPRB, 0.36462E+00_JPRB, 0.36608E+00_JPRB, 0.36515E+00_JPRB, 0.36136E+00_JPRB, 0.35672E+00_JPRB, 0.35480E+00_JPRB ,&
&  0.35722E+00_JPRB, 0.35933E+00_JPRB, 0.36269E+00_JPRB, 0.36520E+00_JPRB, 0.36458E+00_JPRB, 0.36498E+00_JPRB, 0.36850E+00_JPRB ,&
&  0.37372E+00_JPRB, 0.37854E+00_JPRB, 0.38390E+00_JPRB, 0.38610E+00_JPRB, 0.38888E+00_JPRB, 0.39520E+00_JPRB, 0.39552E+00_JPRB ,&
&  0.39414E+00_JPRB, 0.39630E+00_JPRB, 0.40043E+00_JPRB, 0.40803E+00_JPRB, 0.41963E+00_JPRB, 0.42725E+00_JPRB, 0.42829E+00_JPRB ,&
&  0.42827E+00_JPRB, 0.43042E+00_JPRB, 0.43397E+00_JPRB, 0.44273E+00_JPRB, 0.45229E+00_JPRB, 0.45925E+00_JPRB, 0.46222E+00_JPRB ,&
&  0.45927E+00_JPRB, 0.45273E+00_JPRB, 0.44077E+00_JPRB, 0.42669E+00_JPRB, 0.41405E+00_JPRB, 0.40417E+00_JPRB, 0.39607E+00_JPRB ,&
&  0.38818E+00_JPRB, 0.38495E+00_JPRB, 0.38483E+00_JPRB, 0.38688E+00_JPRB, 0.38817E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.42152E+00_JPRB, 0.42238E+00_JPRB, 0.42729E+00_JPRB, 0.43224E+00_JPRB, 0.43582E+00_JPRB, 0.44088E+00_JPRB, 0.45041E+00_JPRB ,&
&  0.46009E+00_JPRB, 0.46944E+00_JPRB, 0.47586E+00_JPRB, 0.48367E+00_JPRB, 0.49454E+00_JPRB, 0.50184E+00_JPRB, 0.50767E+00_JPRB ,&
&  0.51009E+00_JPRB, 0.51033E+00_JPRB, 0.51110E+00_JPRB, 0.51401E+00_JPRB, 0.51347E+00_JPRB, 0.50981E+00_JPRB, 0.50849E+00_JPRB ,&
&  0.50842E+00_JPRB, 0.50685E+00_JPRB, 0.50682E+00_JPRB, 0.50830E+00_JPRB, 0.50656E+00_JPRB, 0.50344E+00_JPRB, 0.50367E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.49181E+00_JPRB, 0.48494E+00_JPRB, 0.47851E+00_JPRB, 0.48267E+00_JPRB, 0.49456E+00_JPRB, 0.50745E+00_JPRB ,&
&  0.51527E+00_JPRB, 0.52057E+00_JPRB, 0.52029E+00_JPRB, 0.51943E+00_JPRB, 0.51495E+00_JPRB, 0.50359E+00_JPRB, 0.49370E+00_JPRB ,&
&  0.48697E+00_JPRB, 0.47761E+00_JPRB, 0.45548E+00_JPRB, 0.44294E+00_JPRB, 0.43982E+00_JPRB, 0.44240E+00_JPRB, 0.44608E+00_JPRB ,&
&  0.44773E+00_JPRB, 0.44547E+00_JPRB, 0.43975E+00_JPRB, 0.43264E+00_JPRB, 0.42277E+00_JPRB, 0.41124E+00_JPRB, 0.40112E+00_JPRB ,&
&  0.39524E+00_JPRB, 0.39080E+00_JPRB, 0.38711E+00_JPRB, 0.38943E+00_JPRB, 0.38870E+00_JPRB, 0.38490E+00_JPRB, 0.38657E+00_JPRB ,&
&  0.39729E+00_JPRB, 0.41069E+00_JPRB, 0.42312E+00_JPRB, 0.42998E+00_JPRB, 0.43270E+00_JPRB, 0.43584E+00_JPRB, 0.43971E+00_JPRB ,&
&  0.44556E+00_JPRB, 0.45154E+00_JPRB, 0.45645E+00_JPRB, 0.46243E+00_JPRB, 0.46997E+00_JPRB, 0.47691E+00_JPRB, 0.47982E+00_JPRB ,&
&  0.48234E+00_JPRB, 0.48369E+00_JPRB, 0.48436E+00_JPRB, 0.48887E+00_JPRB, 0.49623E+00_JPRB, 0.50152E+00_JPRB, 0.50594E+00_JPRB ,&
&  0.51065E+00_JPRB, 0.51291E+00_JPRB, 0.51166E+00_JPRB, 0.50773E+00_JPRB, 0.50221E+00_JPRB, 0.49109E+00_JPRB, 0.47742E+00_JPRB ,&
&  0.46174E+00_JPRB, 0.44770E+00_JPRB, 0.43754E+00_JPRB, 0.42882E+00_JPRB, 0.42289E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.46521E+00_JPRB, 0.46338E+00_JPRB, 0.46400E+00_JPRB, 0.46731E+00_JPRB, 0.47100E+00_JPRB, 0.47396E+00_JPRB, 0.47997E+00_JPRB ,&
&  0.49003E+00_JPRB, 0.49791E+00_JPRB, 0.50744E+00_JPRB, 0.51706E+00_JPRB, 0.52435E+00_JPRB, 0.52927E+00_JPRB, 0.53195E+00_JPRB ,&
&  0.53026E+00_JPRB, 0.52751E+00_JPRB, 0.53012E+00_JPRB, 0.53456E+00_JPRB, 0.53588E+00_JPRB, 0.53543E+00_JPRB, 0.53545E+00_JPRB ,&
&  0.53511E+00_JPRB, 0.53316E+00_JPRB, 0.53078E+00_JPRB, 0.52720E+00_JPRB, 0.52288E+00_JPRB, 0.52131E+00_JPRB, 0.51979E+00_JPRB ,&
&  0.51393E+00_JPRB, 0.51406E+00_JPRB, 0.46463E+00_JPRB, 0.48023E+00_JPRB, 0.49347E+00_JPRB, 0.51353E+00_JPRB, 0.53283E+00_JPRB ,&
&  0.54675E+00_JPRB, 0.55584E+00_JPRB, 0.56053E+00_JPRB, 0.55930E+00_JPRB, 0.54739E+00_JPRB, 0.53953E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.51935E+00_JPRB, 0.51140E+00_JPRB, 0.49856E+00_JPRB, 0.48489E+00_JPRB, 0.48033E+00_JPRB, 0.48080E+00_JPRB, 0.48080E+00_JPRB ,&
&  0.47751E+00_JPRB, 0.47455E+00_JPRB, 0.47633E+00_JPRB, 0.47760E+00_JPRB, 0.47759E+00_JPRB, 0.47238E+00_JPRB, 0.46563E+00_JPRB ,&
&  0.45507E+00_JPRB, 0.44465E+00_JPRB, 0.43742E+00_JPRB, 0.43188E+00_JPRB, 0.42695E+00_JPRB, 0.42157E+00_JPRB, 0.41626E+00_JPRB ,&
&  0.41925E+00_JPRB, 0.42980E+00_JPRB, 0.43969E+00_JPRB, 0.44799E+00_JPRB, 0.45350E+00_JPRB, 0.45728E+00_JPRB, 0.46253E+00_JPRB ,&
&  0.46859E+00_JPRB, 0.47866E+00_JPRB, 0.48539E+00_JPRB, 0.49132E+00_JPRB, 0.49583E+00_JPRB, 0.50012E+00_JPRB, 0.50513E+00_JPRB ,&
&  0.50913E+00_JPRB, 0.51539E+00_JPRB, 0.51967E+00_JPRB, 0.52501E+00_JPRB, 0.52787E+00_JPRB, 0.53237E+00_JPRB, 0.53809E+00_JPRB ,&
&  0.54461E+00_JPRB, 0.55046E+00_JPRB, 0.55275E+00_JPRB, 0.55106E+00_JPRB, 0.54350E+00_JPRB, 0.53326E+00_JPRB, 0.52346E+00_JPRB ,&
&  0.51115E+00_JPRB, 0.49714E+00_JPRB, 0.48333E+00_JPRB, 0.47468E+00_JPRB, 0.46758E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.52374E+00_JPRB, 0.51905E+00_JPRB, 0.51390E+00_JPRB, 0.51339E+00_JPRB, 0.51273E+00_JPRB, 0.51517E+00_JPRB, 0.51858E+00_JPRB ,&
&  0.52573E+00_JPRB, 0.53034E+00_JPRB, 0.53958E+00_JPRB, 0.54695E+00_JPRB, 0.55046E+00_JPRB, 0.55460E+00_JPRB, 0.55725E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.55975E+00_JPRB, 0.56048E+00_JPRB, 0.56473E+00_JPRB, 0.56549E+00_JPRB, 0.56480E+00_JPRB, 0.56398E+00_JPRB ,&
&  0.56249E+00_JPRB, 0.56104E+00_JPRB, 0.55683E+00_JPRB, 0.55071E+00_JPRB, 0.54285E+00_JPRB, 0.53993E+00_JPRB, 0.53455E+00_JPRB ,&
&  0.53639E+00_JPRB, 0.50544E+00_JPRB, 0.51715E+00_JPRB, 0.51567E+00_JPRB, 0.52904E+00_JPRB, 0.55023E+00_JPRB, 0.58056E+00_JPRB ,&
&  0.60207E+00_JPRB, 0.61406E+00_JPRB, 0.61676E+00_JPRB, 0.61583E+00_JPRB, 0.60282E+00_JPRB, 0.58885E+00_JPRB, 0.57303E+00_JPRB ,&
&  0.55469E+00_JPRB, 0.54446E+00_JPRB, 0.53124E+00_JPRB, 0.52157E+00_JPRB, 0.51764E+00_JPRB, 0.51501E+00_JPRB, 0.51064E+00_JPRB ,&
&  0.50401E+00_JPRB, 0.49841E+00_JPRB, 0.49942E+00_JPRB, 0.50321E+00_JPRB, 0.50704E+00_JPRB, 0.50941E+00_JPRB, 0.50839E+00_JPRB ,&
&  0.50212E+00_JPRB, 0.49638E+00_JPRB, 0.49320E+00_JPRB, 0.49226E+00_JPRB, 0.48743E+00_JPRB, 0.47940E+00_JPRB, 0.47112E+00_JPRB ,&
&  0.46903E+00_JPRB, 0.47170E+00_JPRB, 0.47542E+00_JPRB, 0.48022E+00_JPRB, 0.48390E+00_JPRB, 0.48833E+00_JPRB, 0.49047E+00_JPRB ,&
&  0.49527E+00_JPRB, 0.49988E+00_JPRB, 0.50911E+00_JPRB, 0.51796E+00_JPRB, 0.52033E+00_JPRB, 0.52895E+00_JPRB, 0.53774E+00_JPRB ,&
&  0.54615E+00_JPRB, 0.55563E+00_JPRB, 0.56254E+00_JPRB, 0.56669E+00_JPRB, 0.57111E+00_JPRB, 0.57358E+00_JPRB, 0.57861E+00_JPRB ,&
&  0.58569E+00_JPRB, 0.58698E+00_JPRB, 0.59177E+00_JPRB, 0.59142E+00_JPRB, 0.58532E+00_JPRB, 0.57592E+00_JPRB, 0.56482E+00_JPRB ,&
&  0.55316E+00_JPRB, 0.53827E+00_JPRB, 0.53093E+00_JPRB, 0.52688E+00_JPRB, 0.52506E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.56634E+00_JPRB, 0.55996E+00_JPRB, 0.54962E+00_JPRB, 0.54395E+00_JPRB, 0.54216E+00_JPRB, 0.54609E+00_JPRB, 0.55119E+00_JPRB ,&
&  0.55642E+00_JPRB, 0.55856E+00_JPRB, 0.56602E+00_JPRB, 0.57341E+00_JPRB, 0.57771E+00_JPRB, 0.58052E+00_JPRB, 0.57961E+00_JPRB ,&
&  0.57956E+00_JPRB, 0.57954E+00_JPRB, 0.57524E+00_JPRB, 0.57194E+00_JPRB, 0.56710E+00_JPRB, 0.56801E+00_JPRB, 0.56842E+00_JPRB ,&
&  0.56788E+00_JPRB, 0.56571E+00_JPRB, 0.56051E+00_JPRB, 0.55294E+00_JPRB, 0.54999E+00_JPRB, 0.55025E+00_JPRB, 0.54854E+00_JPRB ,&
&  0.55584E+00_JPRB, 0.47644E+00_JPRB, 0.51209E+00_JPRB, 0.52471E+00_JPRB, 0.54248E+00_JPRB, 0.56812E+00_JPRB, 0.59839E+00_JPRB ,&
&  0.62925E+00_JPRB, 0.65431E+00_JPRB, 0.66254E+00_JPRB, 0.66237E+00_JPRB, 0.65115E+00_JPRB, 0.63349E+00_JPRB, 0.61310E+00_JPRB ,&
&  0.59552E+00_JPRB, 0.58181E+00_JPRB, 0.56493E+00_JPRB, 0.55532E+00_JPRB, 0.54762E+00_JPRB, 0.53771E+00_JPRB, 0.52960E+00_JPRB ,&
&  0.52677E+00_JPRB, 0.52533E+00_JPRB, 0.52574E+00_JPRB, 0.52788E+00_JPRB, 0.53104E+00_JPRB, 0.53322E+00_JPRB, 0.53307E+00_JPRB ,&
&  0.53177E+00_JPRB, 0.52902E+00_JPRB, 0.52438E+00_JPRB, 0.52217E+00_JPRB, 0.52126E+00_JPRB, 0.52258E+00_JPRB, 0.51880E+00_JPRB ,&
&  0.51248E+00_JPRB, 0.51403E+00_JPRB, 0.51737E+00_JPRB, 0.51964E+00_JPRB, 0.52290E+00_JPRB, 0.52636E+00_JPRB, 0.52696E+00_JPRB ,&
&  0.52742E+00_JPRB, 0.53147E+00_JPRB, 0.53583E+00_JPRB, 0.54273E+00_JPRB, 0.54760E+00_JPRB, 0.55663E+00_JPRB, 0.56400E+00_JPRB ,&
&  0.57090E+00_JPRB, 0.58139E+00_JPRB, 0.59091E+00_JPRB, 0.59677E+00_JPRB, 0.60017E+00_JPRB, 0.60409E+00_JPRB, 0.60847E+00_JPRB ,&
&  0.61155E+00_JPRB, 0.61395E+00_JPRB, 0.61607E+00_JPRB, 0.61483E+00_JPRB, 0.60865E+00_JPRB, 0.60243E+00_JPRB, 0.59821E+00_JPRB ,&
&  0.58879E+00_JPRB, 0.57731E+00_JPRB, 0.57332E+00_JPRB, 0.57057E+00_JPRB, 0.56765E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.63068E+00_JPRB, 0.62327E+00_JPRB, 0.61061E+00_JPRB, 0.60011E+00_JPRB, 0.59195E+00_JPRB, 0.58991E+00_JPRB, 0.58916E+00_JPRB ,&
&  0.59454E+00_JPRB, 0.60277E+00_JPRB, 0.61008E+00_JPRB, 0.61368E+00_JPRB, 0.61598E+00_JPRB, 0.61562E+00_JPRB, 0.61197E+00_JPRB ,&
&  0.60904E+00_JPRB, 0.59797E+00_JPRB, 0.58993E+00_JPRB, 0.59031E+00_JPRB, 0.58759E+00_JPRB, 0.58465E+00_JPRB, 0.58141E+00_JPRB ,&
&  0.57862E+00_JPRB, 0.57694E+00_JPRB, 0.56973E+00_JPRB, 0.56697E+00_JPRB, 0.56700E+00_JPRB, 0.56985E+00_JPRB, 0.58079E+00_JPRB ,&
&  0.50847E+00_JPRB, 0.43022E+00_JPRB, 0.48156E+00_JPRB, 0.53373E+00_JPRB, 0.56262E+00_JPRB, 0.60086E+00_JPRB, 0.63635E+00_JPRB ,&
&  0.68216E+00_JPRB, 0.71035E+00_JPRB, 0.72419E+00_JPRB, 0.71794E+00_JPRB, 0.70092E+00_JPRB, 0.68339E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.64395E+00_JPRB, 0.61904E+00_JPRB, 0.60361E+00_JPRB, 0.59676E+00_JPRB, 0.59091E+00_JPRB, 0.58407E+00_JPRB, 0.58059E+00_JPRB ,&
&  0.58017E+00_JPRB, 0.58005E+00_JPRB, 0.57890E+00_JPRB, 0.57827E+00_JPRB, 0.58030E+00_JPRB, 0.58255E+00_JPRB, 0.58494E+00_JPRB ,&
&  0.58451E+00_JPRB, 0.58278E+00_JPRB, 0.57939E+00_JPRB, 0.57306E+00_JPRB, 0.56834E+00_JPRB, 0.56878E+00_JPRB, 0.56716E+00_JPRB ,&
&  0.56636E+00_JPRB, 0.56525E+00_JPRB, 0.56558E+00_JPRB, 0.56665E+00_JPRB, 0.56653E+00_JPRB, 0.56670E+00_JPRB, 0.56937E+00_JPRB ,&
&  0.57501E+00_JPRB, 0.57547E+00_JPRB, 0.57860E+00_JPRB, 0.58406E+00_JPRB, 0.58957E+00_JPRB, 0.59459E+00_JPRB, 0.59747E+00_JPRB ,&
&  0.60271E+00_JPRB, 0.61404E+00_JPRB, 0.62199E+00_JPRB, 0.62689E+00_JPRB, 0.63107E+00_JPRB, 0.63604E+00_JPRB, 0.64318E+00_JPRB ,&
&  0.64859E+00_JPRB, 0.65313E+00_JPRB, 0.65890E+00_JPRB, 0.66770E+00_JPRB, 0.67034E+00_JPRB, 0.66746E+00_JPRB, 0.66099E+00_JPRB ,&
&  0.64982E+00_JPRB, 0.63382E+00_JPRB, 0.63509E+00_JPRB, 0.63325E+00_JPRB, 0.63137E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.65576E+00_JPRB, 0.65636E+00_JPRB, 0.64882E+00_JPRB, 0.63843E+00_JPRB, 0.63101E+00_JPRB, 0.62907E+00_JPRB, 0.62937E+00_JPRB ,&
&  0.63428E+00_JPRB, 0.63832E+00_JPRB, 0.63909E+00_JPRB, 0.63566E+00_JPRB, 0.63020E+00_JPRB, 0.62183E+00_JPRB, 0.61525E+00_JPRB ,&
&  0.60280E+00_JPRB, 0.58934E+00_JPRB, 0.58531E+00_JPRB, 0.58565E+00_JPRB, 0.58366E+00_JPRB, 0.57787E+00_JPRB, 0.57565E+00_JPRB ,&
&  0.57613E+00_JPRB, 0.58016E+00_JPRB, 0.57584E+00_JPRB, 0.57370E+00_JPRB, 0.57030E+00_JPRB, 0.57321E+00_JPRB, 0.57549E+00_JPRB ,&
&  0.57595E+00_JPRB, 0.45223E+00_JPRB, 0.50677E+00_JPRB, 0.55685E+00_JPRB, 0.58069E+00_JPRB, 0.62790E+00_JPRB, 0.67997E+00_JPRB ,&
&  0.71760E+00_JPRB, 0.73556E+00_JPRB, 0.73012E+00_JPRB, 0.71648E+00_JPRB, 0.69460E+00_JPRB, 0.67223E+00_JPRB, 0.65174E+00_JPRB ,&
&  0.63214E+00_JPRB, 0.61913E+00_JPRB, 0.60997E+00_JPRB, 0.60255E+00_JPRB, 0.59958E+00_JPRB, 0.59647E+00_JPRB, 0.59353E+00_JPRB ,&
&  0.59652E+00_JPRB, 0.60494E+00_JPRB, 0.60681E+00_JPRB, 0.60997E+00_JPRB, 0.61363E+00_JPRB, 0.61555E+00_JPRB, 0.62310E+00_JPRB ,&
&  0.62697E+00_JPRB, 0.62572E+00_JPRB, 0.62447E+00_JPRB, 0.62070E+00_JPRB, 0.61170E+00_JPRB, 0.60862E+00_JPRB, 0.60441E+00_JPRB ,&
&  0.60061E+00_JPRB, 0.59356E+00_JPRB, 0.58789E+00_JPRB, 0.58624E+00_JPRB, 0.58715E+00_JPRB, 0.58785E+00_JPRB, 0.59436E+00_JPRB ,&
&  0.60497E+00_JPRB, 0.60813E+00_JPRB, 0.61089E+00_JPRB, 0.61585E+00_JPRB, 0.61968E+00_JPRB, 0.62173E+00_JPRB, 0.62493E+00_JPRB ,&
&  0.63321E+00_JPRB, 0.64723E+00_JPRB, 0.66013E+00_JPRB, 0.66537E+00_JPRB, 0.66730E+00_JPRB, 0.67441E+00_JPRB, 0.68199E+00_JPRB ,&
&  0.68996E+00_JPRB, 0.69220E+00_JPRB, 0.69263E+00_JPRB, 0.71559E+00_JPRB, 0.72441E+00_JPRB, 0.71938E+00_JPRB, 0.71004E+00_JPRB ,&
&  0.69127E+00_JPRB, 0.68441E+00_JPRB, 0.55391E+00_JPRB, 0.61428E+00_JPRB, 0.63505E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.69891E+00_JPRB, 0.70272E+00_JPRB, 0.69490E+00_JPRB, 0.68298E+00_JPRB, 0.67877E+00_JPRB, 0.67770E+00_JPRB, 0.67751E+00_JPRB ,&
&  0.67179E+00_JPRB, 0.66257E+00_JPRB, 0.65184E+00_JPRB, 0.63939E+00_JPRB, 0.62646E+00_JPRB, 0.61595E+00_JPRB, 0.60559E+00_JPRB ,&
&  0.58799E+00_JPRB, 0.57846E+00_JPRB, 0.57887E+00_JPRB, 0.57635E+00_JPRB, 0.57393E+00_JPRB, 0.57057E+00_JPRB, 0.57236E+00_JPRB ,&
&  0.57668E+00_JPRB, 0.57872E+00_JPRB, 0.57976E+00_JPRB, 0.57458E+00_JPRB, 0.57094E+00_JPRB, 0.57201E+00_JPRB, 0.56917E+00_JPRB ,&
&  0.56867E+00_JPRB, 0.46585E+00_JPRB, 0.51692E+00_JPRB, 0.59349E+00_JPRB, 0.62708E+00_JPRB, 0.67763E+00_JPRB, 0.74876E+00_JPRB ,&
&  0.78953E+00_JPRB, 0.77966E+00_JPRB, 0.75398E+00_JPRB, 0.72999E+00_JPRB, 0.70204E+00_JPRB, 0.67548E+00_JPRB, 0.65252E+00_JPRB ,&
&  0.63868E+00_JPRB, 0.63214E+00_JPRB, 0.62849E+00_JPRB, 0.62352E+00_JPRB, 0.61789E+00_JPRB, 0.61729E+00_JPRB, 0.61762E+00_JPRB ,&
&  0.61995E+00_JPRB, 0.62567E+00_JPRB, 0.62903E+00_JPRB, 0.63660E+00_JPRB, 0.64309E+00_JPRB, 0.65462E+00_JPRB, 0.67122E+00_JPRB ,&
&  0.67535E+00_JPRB, 0.68423E+00_JPRB, 0.69965E+00_JPRB, 0.70072E+00_JPRB, 0.68810E+00_JPRB, 0.68077E+00_JPRB, 0.67299E+00_JPRB ,&
&  0.66256E+00_JPRB, 0.64433E+00_JPRB, 0.64081E+00_JPRB, 0.63724E+00_JPRB, 0.63338E+00_JPRB, 0.63241E+00_JPRB, 0.64176E+00_JPRB ,&
&  0.65103E+00_JPRB, 0.65425E+00_JPRB, 0.65986E+00_JPRB, 0.66860E+00_JPRB, 0.67284E+00_JPRB, 0.66916E+00_JPRB, 0.66801E+00_JPRB ,&
&  0.67445E+00_JPRB, 0.68541E+00_JPRB, 0.69053E+00_JPRB, 0.69124E+00_JPRB, 0.69878E+00_JPRB, 0.71159E+00_JPRB, 0.71983E+00_JPRB ,&
&  0.72537E+00_JPRB, 0.72589E+00_JPRB, 0.73208E+00_JPRB, 0.71557E+00_JPRB, 0.76096E+00_JPRB, 0.75507E+00_JPRB, 0.74255E+00_JPRB ,&
&  0.73193E+00_JPRB, 0.71886E+00_JPRB, 0.71734E+00_JPRB, 0.62023E+00_JPRB, 0.66569E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.74380E+00_JPRB, 0.75110E+00_JPRB, 0.73650E+00_JPRB, 0.72079E+00_JPRB, 0.71232E+00_JPRB, 0.70253E+00_JPRB, 0.68251E+00_JPRB ,&
&  0.65987E+00_JPRB, 0.63954E+00_JPRB, 0.62803E+00_JPRB, 0.61275E+00_JPRB, 0.59842E+00_JPRB, 0.59031E+00_JPRB, 0.58092E+00_JPRB ,&
&  0.56584E+00_JPRB, 0.56276E+00_JPRB, 0.56358E+00_JPRB, 0.56223E+00_JPRB, 0.55859E+00_JPRB, 0.55722E+00_JPRB, 0.56543E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.56508E+00_JPRB, 0.56501E+00_JPRB, 0.56207E+00_JPRB, 0.55818E+00_JPRB, 0.55874E+00_JPRB, 0.54884E+00_JPRB ,&
&  0.55051E+00_JPRB, 0.50697E+00_JPRB, 0.57983E+00_JPRB, 0.66500E+00_JPRB, 0.73125E+00_JPRB, 0.77510E+00_JPRB, 0.81643E+00_JPRB ,&
&  0.82985E+00_JPRB, 0.80063E+00_JPRB, 0.76307E+00_JPRB, 0.73491E+00_JPRB, 0.70179E+00_JPRB, 0.67527E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.64236E+00_JPRB, 0.63778E+00_JPRB, 0.63469E+00_JPRB, 0.63201E+00_JPRB, 0.62871E+00_JPRB, 0.62626E+00_JPRB, 0.62693E+00_JPRB ,&
&  0.62597E+00_JPRB, 0.62803E+00_JPRB, 0.63076E+00_JPRB, 0.62921E+00_JPRB, 0.63505E+00_JPRB, 0.65940E+00_JPRB, 0.68900E+00_JPRB ,&
&  0.69799E+00_JPRB, 0.71545E+00_JPRB, 0.73798E+00_JPRB, 0.74563E+00_JPRB, 0.73630E+00_JPRB, 0.73067E+00_JPRB, 0.72176E+00_JPRB ,&
&  0.71485E+00_JPRB, 0.70182E+00_JPRB, 0.69937E+00_JPRB, 0.69329E+00_JPRB, 0.68430E+00_JPRB, 0.67958E+00_JPRB, 0.67922E+00_JPRB ,&
&  0.68034E+00_JPRB, 0.68187E+00_JPRB, 0.68879E+00_JPRB, 0.69693E+00_JPRB, 0.69468E+00_JPRB, 0.68805E+00_JPRB, 0.68807E+00_JPRB ,&
&  0.69157E+00_JPRB, 0.68744E+00_JPRB, 0.67832E+00_JPRB, 0.68792E+00_JPRB, 0.70752E+00_JPRB, 0.71893E+00_JPRB, 0.72513E+00_JPRB ,&
&  0.73094E+00_JPRB, 0.73838E+00_JPRB, 0.75059E+00_JPRB, 0.78521E+00_JPRB, 0.79080E+00_JPRB, 0.77522E+00_JPRB, 0.76563E+00_JPRB ,&
&  0.76111E+00_JPRB, 0.75087E+00_JPRB, 0.74013E+00_JPRB, 0.73574E+00_JPRB, 0.68879E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.88770E+00_JPRB, 0.82869E+00_JPRB, 0.78009E+00_JPRB, 0.74547E+00_JPRB, 0.72287E+00_JPRB, 0.69360E+00_JPRB, 0.65822E+00_JPRB ,&
&  0.63287E+00_JPRB, 0.62423E+00_JPRB, 0.61492E+00_JPRB, 0.60078E+00_JPRB, 0.59128E+00_JPRB, 0.57977E+00_JPRB, 0.56966E+00_JPRB ,&
&  0.55575E+00_JPRB, 0.55183E+00_JPRB, 0.54793E+00_JPRB, 0.55247E+00_JPRB, 0.55412E+00_JPRB, 0.55997E+00_JPRB, 0.57213E+00_JPRB ,&
&  0.57436E+00_JPRB, 0.55419E+00_JPRB, 0.55120E+00_JPRB, 0.55424E+00_JPRB, 0.55656E+00_JPRB, 0.55876E+00_JPRB, 0.54774E+00_JPRB ,&
&  0.55909E+00_JPRB, 0.55941E+00_JPRB, 0.70138E+00_JPRB, 0.83350E+00_JPRB, 0.90046E+00_JPRB, 0.94333E+00_JPRB, 0.93321E+00_JPRB ,&
&  0.91426E+00_JPRB, 0.87019E+00_JPRB, 0.81610E+00_JPRB, 0.75869E+00_JPRB, 0.70586E+00_JPRB, 0.67327E+00_JPRB, 0.65790E+00_JPRB ,&
&  0.64998E+00_JPRB, 0.64948E+00_JPRB, 0.64686E+00_JPRB, 0.64624E+00_JPRB, 0.64574E+00_JPRB, 0.64345E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.65499E+00_JPRB, 0.64548E+00_JPRB, 0.64090E+00_JPRB, 0.65184E+00_JPRB, 0.66827E+00_JPRB, 0.68870E+00_JPRB, 0.73828E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.79408E+00_JPRB, 0.82209E+00_JPRB, 0.83152E+00_JPRB, 0.82498E+00_JPRB, 0.81844E+00_JPRB, 0.80139E+00_JPRB ,&
&  0.79222E+00_JPRB, 0.78122E+00_JPRB, 0.76791E+00_JPRB, 0.75350E+00_JPRB, 0.73912E+00_JPRB, 0.73079E+00_JPRB, 0.72565E+00_JPRB ,&
&  0.72056E+00_JPRB, 0.72863E+00_JPRB, 0.73477E+00_JPRB, 0.73541E+00_JPRB, 0.72540E+00_JPRB, 0.71820E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.70121E+00_JPRB, 0.68249E+00_JPRB, 0.67648E+00_JPRB, 0.68769E+00_JPRB, 0.71314E+00_JPRB, 0.72349E+00_JPRB, 0.72669E+00_JPRB ,&
&  0.74059E+00_JPRB, 0.75544E+00_JPRB, 0.74645E+00_JPRB, 0.77624E+00_JPRB, 0.84314E+00_JPRB, 0.83390E+00_JPRB, 0.81960E+00_JPRB ,&
&  0.79967E+00_JPRB, 0.79135E+00_JPRB, 0.77430E+00_JPRB, 0.76256E+00_JPRB, 0.86942E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.79111E+00_JPRB, 0.76107E+00_JPRB, 0.73394E+00_JPRB, 0.70236E+00_JPRB, 0.68128E+00_JPRB, 0.65862E+00_JPRB, 0.63664E+00_JPRB ,&
&  0.61749E+00_JPRB, 0.60631E+00_JPRB, 0.59639E+00_JPRB, 0.58687E+00_JPRB, 0.58822E+00_JPRB, 0.58319E+00_JPRB, 0.57136E+00_JPRB ,&
&  0.55414E+00_JPRB, 0.54159E+00_JPRB, 0.53834E+00_JPRB, 0.54198E+00_JPRB, 0.55525E+00_JPRB, 0.56543E+00_JPRB, 0.57443E+00_JPRB ,&
&  0.57818E+00_JPRB, 0.55731E+00_JPRB, 0.54864E+00_JPRB, 0.55209E+00_JPRB, 0.55486E+00_JPRB, 0.55939E+00_JPRB, 0.54368E+00_JPRB ,&
&  0.54914E+00_JPRB, 0.60877E+00_JPRB, 0.83314E+00_JPRB, 0.11351E+01_JPRB, 0.11809E+01_JPRB, 0.11231E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.95976E+00_JPRB, 0.88411E+00_JPRB, 0.80573E+00_JPRB, 0.74950E+00_JPRB, 0.71579E+00_JPRB, 0.69871E+00_JPRB ,&
&  0.69703E+00_JPRB, 0.69811E+00_JPRB, 0.69170E+00_JPRB, 0.68849E+00_JPRB, 0.68374E+00_JPRB, 0.68064E+00_JPRB, 0.68117E+00_JPRB ,&
&  0.67886E+00_JPRB, 0.67354E+00_JPRB, 0.66741E+00_JPRB, 0.66717E+00_JPRB, 0.70694E+00_JPRB, 0.76782E+00_JPRB, 0.86634E+00_JPRB ,&
&  0.92415E+00_JPRB, 0.96191E+00_JPRB, 0.98546E+00_JPRB, 0.98188E+00_JPRB, 0.96035E+00_JPRB, 0.93942E+00_JPRB, 0.92202E+00_JPRB ,&
&  0.89588E+00_JPRB, 0.85665E+00_JPRB, 0.82972E+00_JPRB, 0.80959E+00_JPRB, 0.79005E+00_JPRB, 0.77451E+00_JPRB, 0.76152E+00_JPRB ,&
&  0.75825E+00_JPRB, 0.76531E+00_JPRB, 0.76706E+00_JPRB, 0.76258E+00_JPRB, 0.74622E+00_JPRB, 0.72795E+00_JPRB, 0.71113E+00_JPRB ,&
&  0.68695E+00_JPRB, 0.67254E+00_JPRB, 0.67966E+00_JPRB, 0.70061E+00_JPRB, 0.72372E+00_JPRB, 0.72994E+00_JPRB, 0.73813E+00_JPRB ,&
&  0.75530E+00_JPRB, 0.77491E+00_JPRB, 0.77320E+00_JPRB, 0.77068E+00_JPRB, 0.77222E+00_JPRB, 0.83079E+00_JPRB, 0.82224E+00_JPRB ,&
&  0.79887E+00_JPRB, 0.77770E+00_JPRB, 0.76658E+00_JPRB, 0.75651E+00_JPRB, 0.81086E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.75467E+00_JPRB, 0.74464E+00_JPRB, 0.71360E+00_JPRB, 0.68319E+00_JPRB, 0.66254E+00_JPRB, 0.64775E+00_JPRB, 0.63412E+00_JPRB ,&
&  0.61530E+00_JPRB, 0.59171E+00_JPRB, 0.57774E+00_JPRB, 0.57475E+00_JPRB, 0.57607E+00_JPRB, 0.57482E+00_JPRB, 0.55951E+00_JPRB ,&
&  0.54066E+00_JPRB, 0.52805E+00_JPRB, 0.53186E+00_JPRB, 0.54040E+00_JPRB, 0.55550E+00_JPRB, 0.55513E+00_JPRB, 0.56579E+00_JPRB ,&
&  0.57123E+00_JPRB, 0.56851E+00_JPRB, 0.56602E+00_JPRB, 0.56459E+00_JPRB, 0.56047E+00_JPRB, 0.55699E+00_JPRB, 0.53833E+00_JPRB ,&
&  0.56054E+00_JPRB, 0.68805E+00_JPRB, 0.10773E+01_JPRB, 0.14018E+01_JPRB, 0.14249E+01_JPRB, 0.12826E+01_JPRB, 0.11912E+01_JPRB ,&
&  0.11188E+01_JPRB, 0.10535E+01_JPRB, 0.95699E+00_JPRB, 0.89272E+00_JPRB, 0.83854E+00_JPRB, 0.80197E+00_JPRB, 0.77950E+00_JPRB ,&
&  0.76863E+00_JPRB, 0.75524E+00_JPRB, 0.73299E+00_JPRB, 0.71320E+00_JPRB, 0.70440E+00_JPRB, 0.69871E+00_JPRB, 0.69996E+00_JPRB ,&
&  0.70388E+00_JPRB, 0.70196E+00_JPRB, 0.70058E+00_JPRB, 0.70335E+00_JPRB, 0.73950E+00_JPRB, 0.78009E+00_JPRB, 0.10412E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.13641E+01_JPRB, 0.13215E+01_JPRB, 0.12433E+01_JPRB, 0.11572E+01_JPRB, 0.10926E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.98176E+00_JPRB, 0.93501E+00_JPRB, 0.89757E+00_JPRB, 0.87431E+00_JPRB, 0.85711E+00_JPRB, 0.84230E+00_JPRB, 0.83721E+00_JPRB ,&
&  0.82284E+00_JPRB, 0.80948E+00_JPRB, 0.79459E+00_JPRB, 0.77652E+00_JPRB, 0.74942E+00_JPRB, 0.72303E+00_JPRB, 0.70216E+00_JPRB ,&
&  0.68033E+00_JPRB, 0.67779E+00_JPRB, 0.69342E+00_JPRB, 0.70176E+00_JPRB, 0.73925E+00_JPRB, 0.77093E+00_JPRB, 0.77096E+00_JPRB ,&
&  0.75436E+00_JPRB, 0.76032E+00_JPRB, 0.76073E+00_JPRB, 0.74509E+00_JPRB, 0.74452E+00_JPRB, 0.80834E+00_JPRB, 0.79131E+00_JPRB ,&
&  0.76817E+00_JPRB, 0.75723E+00_JPRB, 0.75002E+00_JPRB, 0.73939E+00_JPRB, 0.75662E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.72490E+00_JPRB, 0.69843E+00_JPRB, 0.67397E+00_JPRB, 0.65608E+00_JPRB, 0.64035E+00_JPRB, 0.62852E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.59307E+00_JPRB, 0.57369E+00_JPRB, 0.56936E+00_JPRB, 0.56659E+00_JPRB, 0.56211E+00_JPRB, 0.55801E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.52409E+00_JPRB, 0.50973E+00_JPRB, 0.52099E+00_JPRB, 0.53428E+00_JPRB, 0.54846E+00_JPRB, 0.55974E+00_JPRB, 0.57501E+00_JPRB ,&
&  0.57115E+00_JPRB, 0.57205E+00_JPRB, 0.57458E+00_JPRB, 0.57510E+00_JPRB, 0.56529E+00_JPRB, 0.56626E+00_JPRB, 0.54051E+00_JPRB ,&
&  0.59458E+00_JPRB, 0.63492E+00_JPRB, 0.12207E+01_JPRB, 0.17064E+01_JPRB, 0.15921E+01_JPRB, 0.14230E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.11307E+01_JPRB, 0.10181E+01_JPRB, 0.94704E+00_JPRB, 0.88686E+00_JPRB, 0.84117E+00_JPRB, 0.81162E+00_JPRB ,&
&  0.78753E+00_JPRB, 0.75493E+00_JPRB, 0.73338E+00_JPRB, 0.71783E+00_JPRB, 0.71736E+00_JPRB, 0.71906E+00_JPRB, 0.72617E+00_JPRB ,&
&  0.73096E+00_JPRB, 0.72824E+00_JPRB, 0.72769E+00_JPRB, 0.75059E+00_JPRB, 0.81498E+00_JPRB, 0.10230E+01_JPRB, 0.12603E+01_JPRB ,&
&  0.14769E+01_JPRB, 0.16982E+01_JPRB, 0.15711E+01_JPRB, 0.14086E+01_JPRB, 0.12786E+01_JPRB, 0.11830E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10116E+01_JPRB, 0.98155E+00_JPRB, 0.96565E+00_JPRB, 0.95303E+00_JPRB, 0.93699E+00_JPRB, 0.90764E+00_JPRB ,&
&  0.86804E+00_JPRB, 0.82799E+00_JPRB, 0.79171E+00_JPRB, 0.76004E+00_JPRB, 0.72994E+00_JPRB, 0.72012E+00_JPRB, 0.71570E+00_JPRB ,&
&  0.70346E+00_JPRB, 0.70887E+00_JPRB, 0.71470E+00_JPRB, 0.71050E+00_JPRB, 0.73559E+00_JPRB, 0.74551E+00_JPRB, 0.74728E+00_JPRB ,&
&  0.74792E+00_JPRB, 0.74794E+00_JPRB, 0.73637E+00_JPRB, 0.75246E+00_JPRB, 0.73132E+00_JPRB, 0.79431E+00_JPRB, 0.77195E+00_JPRB ,&
&  0.74434E+00_JPRB, 0.74549E+00_JPRB, 0.73846E+00_JPRB, 0.73746E+00_JPRB, 0.73808E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.69289E+00_JPRB, 0.65925E+00_JPRB, 0.63626E+00_JPRB, 0.62768E+00_JPRB, 0.62119E+00_JPRB, 0.60871E+00_JPRB, 0.59840E+00_JPRB ,&
&  0.58151E+00_JPRB, 0.56418E+00_JPRB, 0.55544E+00_JPRB, 0.54690E+00_JPRB, 0.55026E+00_JPRB, 0.54520E+00_JPRB, 0.53402E+00_JPRB ,&
&  0.52021E+00_JPRB, 0.51754E+00_JPRB, 0.52051E+00_JPRB, 0.53089E+00_JPRB, 0.55152E+00_JPRB, 0.56164E+00_JPRB, 0.57513E+00_JPRB ,&
&  0.58404E+00_JPRB, 0.59272E+00_JPRB, 0.59246E+00_JPRB, 0.58698E+00_JPRB, 0.58008E+00_JPRB, 0.57052E+00_JPRB, 0.55662E+00_JPRB ,&
&  0.62731E+00_JPRB, 0.13293E+01_JPRB, 0.11711E+01_JPRB, 0.20316E+01_JPRB, 0.17858E+01_JPRB, 0.15893E+01_JPRB, 0.12313E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11499E+01_JPRB, 0.10754E+01_JPRB, 0.97856E+00_JPRB, 0.89005E+00_JPRB, 0.81235E+00_JPRB, 0.76278E+00_JPRB ,&
&  0.74601E+00_JPRB, 0.74962E+00_JPRB, 0.75179E+00_JPRB, 0.74333E+00_JPRB, 0.74605E+00_JPRB, 0.74374E+00_JPRB, 0.74902E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.75537E+00_JPRB, 0.76911E+00_JPRB, 0.84888E+00_JPRB, 0.10104E+01_JPRB, 0.13565E+01_JPRB, 0.15393E+01_JPRB ,&
&  0.15328E+01_JPRB, 0.17528E+01_JPRB, 0.15720E+01_JPRB, 0.13777E+01_JPRB, 0.12541E+01_JPRB, 0.11608E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.10330E+01_JPRB, 0.10005E+01_JPRB, 0.99452E+00_JPRB, 0.98776E+00_JPRB, 0.97110E+00_JPRB, 0.92975E+00_JPRB, 0.87038E+00_JPRB ,&
&  0.81542E+00_JPRB, 0.78350E+00_JPRB, 0.75364E+00_JPRB, 0.73739E+00_JPRB, 0.73123E+00_JPRB, 0.73488E+00_JPRB, 0.74449E+00_JPRB ,&
&  0.74318E+00_JPRB, 0.75148E+00_JPRB, 0.73643E+00_JPRB, 0.71079E+00_JPRB, 0.72449E+00_JPRB, 0.71918E+00_JPRB, 0.70407E+00_JPRB ,&
&  0.72690E+00_JPRB, 0.73717E+00_JPRB, 0.73646E+00_JPRB, 0.75078E+00_JPRB, 0.74428E+00_JPRB, 0.81908E+00_JPRB, 0.76829E+00_JPRB ,&
&  0.74166E+00_JPRB, 0.74479E+00_JPRB, 0.75276E+00_JPRB, 0.73628E+00_JPRB, 0.72206E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.66578E+00_JPRB, 0.64607E+00_JPRB, 0.63324E+00_JPRB, 0.63812E+00_JPRB, 0.61663E+00_JPRB, 0.60186E+00_JPRB, 0.59892E+00_JPRB ,&
&  0.57213E+00_JPRB, 0.55610E+00_JPRB, 0.54709E+00_JPRB, 0.54951E+00_JPRB, 0.55401E+00_JPRB, 0.54770E+00_JPRB, 0.54637E+00_JPRB ,&
&  0.53611E+00_JPRB, 0.52787E+00_JPRB, 0.53088E+00_JPRB, 0.55134E+00_JPRB, 0.57834E+00_JPRB, 0.58562E+00_JPRB, 0.59990E+00_JPRB ,&
&  0.61451E+00_JPRB, 0.60881E+00_JPRB, 0.59871E+00_JPRB, 0.59889E+00_JPRB, 0.60476E+00_JPRB, 0.60255E+00_JPRB, 0.61730E+00_JPRB ,&
&  0.65575E+00_JPRB, 0.12359E+01_JPRB, 0.13699E+01_JPRB, 0.22779E+01_JPRB, 0.19774E+01_JPRB, 0.16406E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.11663E+01_JPRB, 0.10545E+01_JPRB, 0.94149E+00_JPRB, 0.82388E+00_JPRB, 0.73857E+00_JPRB, 0.71835E+00_JPRB ,&
&  0.74595E+00_JPRB, 0.77679E+00_JPRB, 0.79957E+00_JPRB, 0.78738E+00_JPRB, 0.77044E+00_JPRB, 0.76615E+00_JPRB, 0.76886E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.79331E+00_JPRB, 0.86432E+00_JPRB, 0.10171E+01_JPRB, 0.14396E+01_JPRB, 0.16946E+01_JPRB, 0.15863E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.15229E+01_JPRB, 0.13804E+01_JPRB, 0.12349E+01_JPRB, 0.11110E+01_JPRB, 0.10863E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.10023E+01_JPRB, 0.97267E+00_JPRB, 0.93019E+00_JPRB, 0.88763E+00_JPRB, 0.84186E+00_JPRB, 0.79457E+00_JPRB ,&
&  0.76594E+00_JPRB, 0.75187E+00_JPRB, 0.74164E+00_JPRB, 0.74571E+00_JPRB, 0.76262E+00_JPRB, 0.77914E+00_JPRB, 0.79387E+00_JPRB ,&
&  0.79156E+00_JPRB, 0.79489E+00_JPRB, 0.75883E+00_JPRB, 0.73609E+00_JPRB, 0.74655E+00_JPRB, 0.72831E+00_JPRB, 0.71515E+00_JPRB ,&
&  0.74094E+00_JPRB, 0.76673E+00_JPRB, 0.78122E+00_JPRB, 0.77711E+00_JPRB, 0.78613E+00_JPRB, 0.83696E+00_JPRB, 0.78951E+00_JPRB ,&
&  0.75922E+00_JPRB, 0.78733E+00_JPRB, 0.79902E+00_JPRB, 0.73586E+00_JPRB, 0.70072E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.67166E+00_JPRB, 0.66955E+00_JPRB, 0.65548E+00_JPRB, 0.65051E+00_JPRB, 0.61805E+00_JPRB, 0.58935E+00_JPRB, 0.56561E+00_JPRB ,&
&  0.55455E+00_JPRB, 0.55186E+00_JPRB, 0.56368E+00_JPRB, 0.56740E+00_JPRB, 0.56866E+00_JPRB, 0.56091E+00_JPRB, 0.55496E+00_JPRB ,&
&  0.54174E+00_JPRB, 0.54424E+00_JPRB, 0.55684E+00_JPRB, 0.57798E+00_JPRB, 0.60172E+00_JPRB, 0.61142E+00_JPRB, 0.62212E+00_JPRB ,&
&  0.62273E+00_JPRB, 0.61912E+00_JPRB, 0.62190E+00_JPRB, 0.63249E+00_JPRB, 0.64974E+00_JPRB, 0.66738E+00_JPRB, 0.68820E+00_JPRB ,&
&  0.71731E+00_JPRB, 0.12029E+01_JPRB, 0.18212E+01_JPRB, 0.20712E+01_JPRB, 0.17785E+01_JPRB, 0.14392E+01_JPRB, 0.12209E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.87258E+00_JPRB, 0.95683E+00_JPRB, 0.87988E+00_JPRB, 0.78663E+00_JPRB, 0.74954E+00_JPRB, 0.76060E+00_JPRB ,&
&  0.80127E+00_JPRB, 0.81607E+00_JPRB, 0.82373E+00_JPRB, 0.82972E+00_JPRB, 0.82029E+00_JPRB, 0.81879E+00_JPRB, 0.82709E+00_JPRB ,&
&  0.86877E+00_JPRB, 0.96399E+00_JPRB, 0.11390E+01_JPRB, 0.14491E+01_JPRB, 0.18532E+01_JPRB, 0.17143E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.13145E+01_JPRB, 0.12429E+01_JPRB, 0.11674E+01_JPRB, 0.11045E+01_JPRB, 0.95436E+00_JPRB, 0.10468E+01_JPRB, 0.10066E+01_JPRB ,&
&  0.96989E+00_JPRB, 0.93995E+00_JPRB, 0.89849E+00_JPRB, 0.86287E+00_JPRB, 0.83084E+00_JPRB, 0.79687E+00_JPRB, 0.77377E+00_JPRB ,&
&  0.76749E+00_JPRB, 0.76943E+00_JPRB, 0.77398E+00_JPRB, 0.78310E+00_JPRB, 0.79764E+00_JPRB, 0.81428E+00_JPRB, 0.82298E+00_JPRB ,&
&  0.82154E+00_JPRB, 0.82781E+00_JPRB, 0.81217E+00_JPRB, 0.81665E+00_JPRB, 0.81569E+00_JPRB, 0.79303E+00_JPRB, 0.77062E+00_JPRB ,&
&  0.78387E+00_JPRB, 0.81932E+00_JPRB, 0.83349E+00_JPRB, 0.83461E+00_JPRB, 0.84052E+00_JPRB, 0.86233E+00_JPRB, 0.80835E+00_JPRB ,&
&  0.79331E+00_JPRB, 0.85327E+00_JPRB, 0.81947E+00_JPRB, 0.73688E+00_JPRB, 0.69682E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.68081E+00_JPRB, 0.68363E+00_JPRB, 0.67396E+00_JPRB, 0.63798E+00_JPRB, 0.60535E+00_JPRB, 0.57159E+00_JPRB, 0.54890E+00_JPRB ,&
&  0.54731E+00_JPRB, 0.55815E+00_JPRB, 0.58959E+00_JPRB, 0.60757E+00_JPRB, 0.59750E+00_JPRB, 0.58003E+00_JPRB, 0.57402E+00_JPRB ,&
&  0.57811E+00_JPRB, 0.59082E+00_JPRB, 0.59826E+00_JPRB, 0.61379E+00_JPRB, 0.63293E+00_JPRB, 0.65109E+00_JPRB, 0.67065E+00_JPRB ,&
&  0.65898E+00_JPRB, 0.66394E+00_JPRB, 0.67832E+00_JPRB, 0.68790E+00_JPRB, 0.70414E+00_JPRB, 0.73922E+00_JPRB, 0.76549E+00_JPRB ,&
&  0.88573E+00_JPRB, 0.13601E+01_JPRB, 0.84593E+00_JPRB, 0.11738E+01_JPRB, 0.13491E+01_JPRB, 0.13340E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.85376E+00_JPRB, 0.85896E+00_JPRB, 0.89703E+00_JPRB, 0.84179E+00_JPRB, 0.84669E+00_JPRB, 0.85900E+00_JPRB ,&
&  0.85364E+00_JPRB, 0.86843E+00_JPRB, 0.87479E+00_JPRB, 0.88487E+00_JPRB, 0.88987E+00_JPRB, 0.90996E+00_JPRB, 0.97218E+00_JPRB ,&
&  0.10975E+01_JPRB, 0.12552E+01_JPRB, 0.15246E+01_JPRB, 0.17137E+01_JPRB, 0.20368E+01_JPRB, 0.16395E+01_JPRB, 0.12807E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10422E+01_JPRB, 0.10770E+01_JPRB, 0.10600E+01_JPRB, 0.96675E+00_JPRB, 0.90651E+00_JPRB, 0.94535E+00_JPRB ,&
&  0.90417E+00_JPRB, 0.87037E+00_JPRB, 0.85760E+00_JPRB, 0.84121E+00_JPRB, 0.81857E+00_JPRB, 0.80258E+00_JPRB, 0.79438E+00_JPRB ,&
&  0.78896E+00_JPRB, 0.79364E+00_JPRB, 0.80465E+00_JPRB, 0.81096E+00_JPRB, 0.82235E+00_JPRB, 0.83133E+00_JPRB, 0.84489E+00_JPRB ,&
&  0.85962E+00_JPRB, 0.88254E+00_JPRB, 0.90962E+00_JPRB, 0.91670E+00_JPRB, 0.90869E+00_JPRB, 0.88450E+00_JPRB, 0.86817E+00_JPRB ,&
&  0.86696E+00_JPRB, 0.88673E+00_JPRB, 0.87127E+00_JPRB, 0.86314E+00_JPRB, 0.91167E+00_JPRB, 0.88431E+00_JPRB, 0.84257E+00_JPRB ,&
&  0.88936E+00_JPRB, 0.10427E+01_JPRB, 0.10955E+01_JPRB, 0.73733E+00_JPRB, 0.69614E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.64642E+00_JPRB, 0.62992E+00_JPRB, 0.60185E+00_JPRB, 0.56681E+00_JPRB, 0.56268E+00_JPRB, 0.55955E+00_JPRB, 0.57148E+00_JPRB ,&
&  0.60745E+00_JPRB, 0.65628E+00_JPRB, 0.67036E+00_JPRB, 0.66273E+00_JPRB, 0.64590E+00_JPRB, 0.64081E+00_JPRB, 0.63801E+00_JPRB ,&
&  0.64115E+00_JPRB, 0.64084E+00_JPRB, 0.65677E+00_JPRB, 0.67795E+00_JPRB, 0.70181E+00_JPRB, 0.71972E+00_JPRB, 0.72200E+00_JPRB ,&
&  0.72728E+00_JPRB, 0.73236E+00_JPRB, 0.74933E+00_JPRB, 0.77009E+00_JPRB, 0.81405E+00_JPRB, 0.90034E+00_JPRB, 0.10340E+01_JPRB ,&
&  0.13590E+01_JPRB, 0.10607E+01_JPRB, 0.58499E+00_JPRB, 0.11150E+01_JPRB, 0.12861E+01_JPRB, 0.13640E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.94363E+00_JPRB, 0.91147E+00_JPRB, 0.99507E+00_JPRB, 0.98671E+00_JPRB, 0.99650E+00_JPRB, 0.99830E+00_JPRB ,&
&  0.96964E+00_JPRB, 0.97032E+00_JPRB, 0.96760E+00_JPRB, 0.97884E+00_JPRB, 0.10056E+01_JPRB, 0.10690E+01_JPRB, 0.11881E+01_JPRB ,&
&  0.13334E+01_JPRB, 0.15195E+01_JPRB, 0.17727E+01_JPRB, 0.18477E+01_JPRB, 0.19854E+01_JPRB, 0.15235E+01_JPRB, 0.12402E+01_JPRB ,&
&  0.96610E+00_JPRB, 0.96283E+00_JPRB, 0.10341E+01_JPRB, 0.10458E+01_JPRB, 0.97260E+00_JPRB, 0.86036E+00_JPRB, 0.90773E+00_JPRB ,&
&  0.88865E+00_JPRB, 0.86548E+00_JPRB, 0.85683E+00_JPRB, 0.83636E+00_JPRB, 0.81920E+00_JPRB, 0.81231E+00_JPRB, 0.81120E+00_JPRB ,&
&  0.81013E+00_JPRB, 0.81871E+00_JPRB, 0.82299E+00_JPRB, 0.82487E+00_JPRB, 0.83094E+00_JPRB, 0.83477E+00_JPRB, 0.86051E+00_JPRB ,&
&  0.88510E+00_JPRB, 0.90530E+00_JPRB, 0.93340E+00_JPRB, 0.95754E+00_JPRB, 0.94546E+00_JPRB, 0.90802E+00_JPRB, 0.90163E+00_JPRB ,&
&  0.88805E+00_JPRB, 0.89385E+00_JPRB, 0.88348E+00_JPRB, 0.91034E+00_JPRB, 0.88537E+00_JPRB, 0.83927E+00_JPRB, 0.78642E+00_JPRB ,&
&  0.78932E+00_JPRB, 0.78645E+00_JPRB, 0.74571E+00_JPRB, 0.68355E+00_JPRB, 0.65571E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.61645E+00_JPRB, 0.59635E+00_JPRB, 0.58300E+00_JPRB, 0.58871E+00_JPRB, 0.59750E+00_JPRB, 0.61661E+00_JPRB, 0.66420E+00_JPRB ,&
&  0.70165E+00_JPRB, 0.71861E+00_JPRB, 0.73938E+00_JPRB, 0.73800E+00_JPRB, 0.74467E+00_JPRB, 0.72456E+00_JPRB, 0.71650E+00_JPRB ,&
&  0.70122E+00_JPRB, 0.71044E+00_JPRB, 0.72820E+00_JPRB, 0.74468E+00_JPRB, 0.75761E+00_JPRB, 0.76868E+00_JPRB, 0.78300E+00_JPRB ,&
&  0.80401E+00_JPRB, 0.83413E+00_JPRB, 0.88074E+00_JPRB, 0.94721E+00_JPRB, 0.10567E+01_JPRB, 0.11913E+01_JPRB, 0.12634E+01_JPRB ,&
&  0.92039E+00_JPRB, 0.45588E+00_JPRB, 0.87367E+00_JPRB, 0.12924E+01_JPRB, 0.13534E+01_JPRB, 0.13764E+01_JPRB, 0.12895E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.10625E+01_JPRB, 0.10118E+01_JPRB, 0.11289E+01_JPRB, 0.11142E+01_JPRB, 0.11087E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.10832E+01_JPRB, 0.10856E+01_JPRB, 0.11111E+01_JPRB, 0.11623E+01_JPRB, 0.12477E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.15176E+01_JPRB, 0.17371E+01_JPRB, 0.19278E+01_JPRB, 0.17785E+01_JPRB, 0.18163E+01_JPRB, 0.15908E+01_JPRB, 0.13860E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.89486E+00_JPRB, 0.92904E+00_JPRB, 0.10460E+01_JPRB, 0.98925E+00_JPRB, 0.85229E+00_JPRB, 0.90222E+00_JPRB ,&
&  0.88870E+00_JPRB, 0.86530E+00_JPRB, 0.85514E+00_JPRB, 0.83260E+00_JPRB, 0.82466E+00_JPRB, 0.82351E+00_JPRB, 0.82893E+00_JPRB ,&
&  0.82803E+00_JPRB, 0.83135E+00_JPRB, 0.83264E+00_JPRB, 0.82825E+00_JPRB, 0.81313E+00_JPRB, 0.81657E+00_JPRB, 0.83898E+00_JPRB ,&
&  0.85862E+00_JPRB, 0.86449E+00_JPRB, 0.88887E+00_JPRB, 0.91319E+00_JPRB, 0.93972E+00_JPRB, 0.91900E+00_JPRB, 0.90752E+00_JPRB ,&
&  0.89003E+00_JPRB, 0.88849E+00_JPRB, 0.89860E+00_JPRB, 0.90284E+00_JPRB, 0.90267E+00_JPRB, 0.85097E+00_JPRB, 0.77448E+00_JPRB ,&
&  0.74841E+00_JPRB, 0.71851E+00_JPRB, 0.69374E+00_JPRB, 0.67089E+00_JPRB, 0.64553E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.67585E+00_JPRB, 0.68335E+00_JPRB, 0.67623E+00_JPRB, 0.69715E+00_JPRB, 0.71613E+00_JPRB, 0.74512E+00_JPRB, 0.77118E+00_JPRB ,&
&  0.79390E+00_JPRB, 0.81604E+00_JPRB, 0.84369E+00_JPRB, 0.81869E+00_JPRB, 0.82380E+00_JPRB, 0.80793E+00_JPRB, 0.78141E+00_JPRB ,&
&  0.77483E+00_JPRB, 0.78745E+00_JPRB, 0.80118E+00_JPRB, 0.81374E+00_JPRB, 0.82570E+00_JPRB, 0.85111E+00_JPRB, 0.88581E+00_JPRB ,&
&  0.93060E+00_JPRB, 0.98760E+00_JPRB, 0.10648E+01_JPRB, 0.11286E+01_JPRB, 0.11951E+01_JPRB, 0.12102E+01_JPRB, 0.11318E+01_JPRB ,&
&  0.56151E+00_JPRB, 0.78183E+00_JPRB, 0.12429E+01_JPRB, 0.14123E+01_JPRB, 0.14856E+01_JPRB, 0.14724E+01_JPRB, 0.14054E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.11118E+01_JPRB, 0.10930E+01_JPRB, 0.12316E+01_JPRB, 0.11946E+01_JPRB, 0.11928E+01_JPRB, 0.11809E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.11938E+01_JPRB, 0.12239E+01_JPRB, 0.12551E+01_JPRB, 0.13185E+01_JPRB, 0.14037E+01_JPRB, 0.15252E+01_JPRB ,&
&  0.16818E+01_JPRB, 0.18464E+01_JPRB, 0.19968E+01_JPRB, 0.17719E+01_JPRB, 0.17311E+01_JPRB, 0.16429E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.10744E+01_JPRB, 0.86786E+00_JPRB, 0.92929E+00_JPRB, 0.10424E+01_JPRB, 0.98440E+00_JPRB, 0.92982E+00_JPRB, 0.90487E+00_JPRB ,&
&  0.90217E+00_JPRB, 0.86328E+00_JPRB, 0.85191E+00_JPRB, 0.83091E+00_JPRB, 0.81840E+00_JPRB, 0.82471E+00_JPRB, 0.84080E+00_JPRB ,&
&  0.83456E+00_JPRB, 0.81922E+00_JPRB, 0.81560E+00_JPRB, 0.81969E+00_JPRB, 0.80266E+00_JPRB, 0.78651E+00_JPRB, 0.78375E+00_JPRB ,&
&  0.79124E+00_JPRB, 0.77940E+00_JPRB, 0.78973E+00_JPRB, 0.82256E+00_JPRB, 0.86513E+00_JPRB, 0.89042E+00_JPRB, 0.90281E+00_JPRB ,&
&  0.89391E+00_JPRB, 0.91717E+00_JPRB, 0.10553E+01_JPRB, 0.11110E+01_JPRB, 0.11160E+01_JPRB, 0.94315E+00_JPRB, 0.77752E+00_JPRB ,&
&  0.75671E+00_JPRB, 0.72708E+00_JPRB, 0.69944E+00_JPRB, 0.68907E+00_JPRB, 0.67795E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.74649E+00_JPRB, 0.75889E+00_JPRB, 0.77886E+00_JPRB, 0.81151E+00_JPRB, 0.84750E+00_JPRB, 0.87209E+00_JPRB, 0.89615E+00_JPRB ,&
&  0.90479E+00_JPRB, 0.91630E+00_JPRB, 0.92526E+00_JPRB, 0.87874E+00_JPRB, 0.88636E+00_JPRB, 0.86633E+00_JPRB, 0.85715E+00_JPRB ,&
&  0.84997E+00_JPRB, 0.86513E+00_JPRB, 0.88403E+00_JPRB, 0.89966E+00_JPRB, 0.92430E+00_JPRB, 0.95723E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.11248E+01_JPRB, 0.11933E+01_JPRB, 0.12303E+01_JPRB, 0.12670E+01_JPRB, 0.12809E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.78658E+00_JPRB, 0.13257E+01_JPRB, 0.14588E+01_JPRB, 0.15279E+01_JPRB, 0.15500E+01_JPRB, 0.14284E+01_JPRB, 0.13833E+01_JPRB ,&
&  0.12880E+01_JPRB, 0.11712E+01_JPRB, 0.11656E+01_JPRB, 0.12318E+01_JPRB, 0.13047E+01_JPRB, 0.12914E+01_JPRB, 0.12909E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.13254E+01_JPRB, 0.13643E+01_JPRB, 0.14075E+01_JPRB, 0.14684E+01_JPRB, 0.15397E+01_JPRB, 0.16447E+01_JPRB ,&
&  0.17503E+01_JPRB, 0.18564E+01_JPRB, 0.19450E+01_JPRB, 0.17541E+01_JPRB, 0.15570E+01_JPRB, 0.15226E+01_JPRB, 0.13218E+01_JPRB ,&
&  0.93193E+00_JPRB, 0.79416E+00_JPRB, 0.87753E+00_JPRB, 0.10087E+01_JPRB, 0.96467E+00_JPRB, 0.88530E+00_JPRB, 0.82847E+00_JPRB ,&
&  0.81228E+00_JPRB, 0.77835E+00_JPRB, 0.79194E+00_JPRB, 0.76142E+00_JPRB, 0.78454E+00_JPRB, 0.78025E+00_JPRB, 0.78906E+00_JPRB ,&
&  0.79573E+00_JPRB, 0.76867E+00_JPRB, 0.75598E+00_JPRB, 0.75093E+00_JPRB, 0.74139E+00_JPRB, 0.73087E+00_JPRB, 0.72302E+00_JPRB ,&
&  0.73204E+00_JPRB, 0.69690E+00_JPRB, 0.67692E+00_JPRB, 0.69501E+00_JPRB, 0.73230E+00_JPRB, 0.77711E+00_JPRB, 0.87153E+00_JPRB ,&
&  0.10299E+01_JPRB, 0.11992E+01_JPRB, 0.14910E+01_JPRB, 0.15848E+01_JPRB, 0.15393E+01_JPRB, 0.11559E+01_JPRB, 0.84414E+00_JPRB ,&
&  0.83613E+00_JPRB, 0.79083E+00_JPRB, 0.75500E+00_JPRB, 0.73428E+00_JPRB, 0.73595E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.89101E+00_JPRB, 0.91135E+00_JPRB, 0.93414E+00_JPRB, 0.95315E+00_JPRB, 0.95596E+00_JPRB, 0.95585E+00_JPRB, 0.95329E+00_JPRB ,&
&  0.97685E+00_JPRB, 0.95326E+00_JPRB, 0.97619E+00_JPRB, 0.95070E+00_JPRB, 0.96968E+00_JPRB, 0.94956E+00_JPRB, 0.95124E+00_JPRB ,&
&  0.95993E+00_JPRB, 0.98437E+00_JPRB, 0.10140E+01_JPRB, 0.10232E+01_JPRB, 0.10182E+01_JPRB, 0.10443E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.11581E+01_JPRB, 0.11954E+01_JPRB, 0.12093E+01_JPRB, 0.11815E+01_JPRB, 0.11515E+01_JPRB, 0.82617E+00_JPRB ,&
&  0.12249E+01_JPRB, 0.14584E+01_JPRB, 0.15818E+01_JPRB, 0.15317E+01_JPRB, 0.14057E+01_JPRB, 0.13084E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.12559E+01_JPRB, 0.12194E+01_JPRB, 0.12357E+01_JPRB, 0.13819E+01_JPRB, 0.13489E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.14141E+01_JPRB, 0.14493E+01_JPRB, 0.14918E+01_JPRB, 0.15291E+01_JPRB, 0.15683E+01_JPRB, 0.16203E+01_JPRB ,&
&  0.16523E+01_JPRB, 0.16581E+01_JPRB, 0.16839E+01_JPRB, 0.15064E+01_JPRB, 0.13925E+01_JPRB, 0.13493E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.80150E+00_JPRB, 0.74914E+00_JPRB, 0.84968E+00_JPRB, 0.94751E+00_JPRB, 0.91793E+00_JPRB, 0.82468E+00_JPRB, 0.77156E+00_JPRB ,&
&  0.74152E+00_JPRB, 0.72100E+00_JPRB, 0.69769E+00_JPRB, 0.70763E+00_JPRB, 0.70804E+00_JPRB, 0.72164E+00_JPRB, 0.73377E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.72769E+00_JPRB, 0.70511E+00_JPRB, 0.69408E+00_JPRB, 0.70213E+00_JPRB, 0.76169E+00_JPRB, 0.83959E+00_JPRB ,&
&  0.87285E+00_JPRB, 0.55864E+00_JPRB, 0.60639E+00_JPRB, 0.59248E+00_JPRB, 0.61142E+00_JPRB, 0.68254E+00_JPRB, 0.86555E+00_JPRB ,&
&  0.13267E+01_JPRB, 0.20644E+01_JPRB, 0.23357E+01_JPRB, 0.20001E+01_JPRB, 0.24673E+01_JPRB, 0.17584E+01_JPRB, 0.96754E+00_JPRB ,&
&  0.94488E+00_JPRB, 0.91282E+00_JPRB, 0.87351E+00_JPRB, 0.86091E+00_JPRB, 0.86796E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.94615E+00_JPRB, 0.95894E+00_JPRB, 0.95564E+00_JPRB, 0.96835E+00_JPRB, 0.94623E+00_JPRB, 0.96228E+00_JPRB, 0.96159E+00_JPRB ,&
&  0.97854E+00_JPRB, 0.98658E+00_JPRB, 0.10030E+01_JPRB, 0.10164E+01_JPRB, 0.10403E+01_JPRB, 0.10609E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.11187E+01_JPRB, 0.11366E+01_JPRB, 0.11467E+01_JPRB, 0.11227E+01_JPRB, 0.11212E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.11179E+01_JPRB, 0.11265E+01_JPRB, 0.11264E+01_JPRB, 0.11045E+01_JPRB, 0.11063E+01_JPRB, 0.81187E+00_JPRB ,&
&  0.12647E+01_JPRB, 0.14408E+01_JPRB, 0.14742E+01_JPRB, 0.13751E+01_JPRB, 0.12771E+01_JPRB, 0.12495E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.12814E+01_JPRB, 0.13025E+01_JPRB, 0.12568E+01_JPRB, 0.13034E+01_JPRB, 0.13256E+01_JPRB, 0.12919E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.13192E+01_JPRB, 0.13324E+01_JPRB, 0.13487E+01_JPRB, 0.13512E+01_JPRB, 0.13386E+01_JPRB, 0.13569E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.13409E+01_JPRB, 0.13488E+01_JPRB, 0.12649E+01_JPRB, 0.11877E+01_JPRB, 0.10223E+01_JPRB, 0.87844E+00_JPRB ,&
&  0.68758E+00_JPRB, 0.67574E+00_JPRB, 0.76462E+00_JPRB, 0.91877E+00_JPRB, 0.88703E+00_JPRB, 0.82518E+00_JPRB, 0.76682E+00_JPRB ,&
&  0.70292E+00_JPRB, 0.68637E+00_JPRB, 0.66147E+00_JPRB, 0.65321E+00_JPRB, 0.65213E+00_JPRB, 0.66843E+00_JPRB, 0.67789E+00_JPRB ,&
&  0.68949E+00_JPRB, 0.69845E+00_JPRB, 0.68289E+00_JPRB, 0.66425E+00_JPRB, 0.65424E+00_JPRB, 0.67917E+00_JPRB, 0.61000E+00_JPRB ,&
&  0.69082E+00_JPRB, 0.57984E+00_JPRB, 0.57412E+00_JPRB, 0.55445E+00_JPRB, 0.51724E+00_JPRB, 0.55609E+00_JPRB, 0.73784E+00_JPRB ,&
&  0.12342E+01_JPRB, 0.16065E+01_JPRB, 0.23151E+01_JPRB, 0.39384E+01_JPRB, 0.27676E+01_JPRB, 0.20347E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.96054E+00_JPRB, 0.93441E+00_JPRB, 0.91956E+00_JPRB, 0.90834E+00_JPRB, 0.92538E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.90896E+00_JPRB, 0.91872E+00_JPRB, 0.92782E+00_JPRB, 0.93747E+00_JPRB, 0.94470E+00_JPRB, 0.96143E+00_JPRB, 0.96710E+00_JPRB ,&
&  0.97815E+00_JPRB, 0.98377E+00_JPRB, 0.99869E+00_JPRB, 0.10178E+01_JPRB, 0.10318E+01_JPRB, 0.10536E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11283E+01_JPRB, 0.11514E+01_JPRB, 0.11569E+01_JPRB, 0.11418E+01_JPRB, 0.11410E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.11112E+01_JPRB, 0.11198E+01_JPRB, 0.11322E+01_JPRB, 0.11236E+01_JPRB, 0.11171E+01_JPRB, 0.85811E+00_JPRB ,&
&  0.12153E+01_JPRB, 0.13601E+01_JPRB, 0.13622E+01_JPRB, 0.12399E+01_JPRB, 0.11647E+01_JPRB, 0.11623E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.12319E+01_JPRB, 0.12834E+01_JPRB, 0.12436E+01_JPRB, 0.11702E+01_JPRB, 0.11296E+01_JPRB, 0.11229E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.11186E+01_JPRB, 0.11213E+01_JPRB, 0.11132E+01_JPRB, 0.10880E+01_JPRB, 0.10600E+01_JPRB, 0.10361E+01_JPRB, 0.10007E+01_JPRB ,&
&  0.96470E+00_JPRB, 0.94227E+00_JPRB, 0.85063E+00_JPRB, 0.88347E+00_JPRB, 0.83594E+00_JPRB, 0.72547E+00_JPRB, 0.60864E+00_JPRB ,&
&  0.51596E+00_JPRB, 0.55582E+00_JPRB, 0.69032E+00_JPRB, 0.90237E+00_JPRB, 0.87332E+00_JPRB, 0.81382E+00_JPRB, 0.78067E+00_JPRB ,&
&  0.73108E+00_JPRB, 0.68229E+00_JPRB, 0.64585E+00_JPRB, 0.62541E+00_JPRB, 0.61709E+00_JPRB, 0.63124E+00_JPRB, 0.64501E+00_JPRB ,&
&  0.64992E+00_JPRB, 0.65394E+00_JPRB, 0.64114E+00_JPRB, 0.63195E+00_JPRB, 0.61854E+00_JPRB, 0.56939E+00_JPRB, 0.65569E+00_JPRB ,&
&  0.61858E+00_JPRB, 0.50107E+00_JPRB, 0.59281E+00_JPRB, 0.39208E+00_JPRB, 0.49770E+00_JPRB, 0.52792E+00_JPRB, 0.63421E+00_JPRB ,&
&  0.76509E+00_JPRB, 0.10861E+01_JPRB, 0.15064E+01_JPRB, 0.13085E+01_JPRB, 0.11701E+01_JPRB, 0.10558E+01_JPRB, 0.96145E+00_JPRB ,&
&  0.92163E+00_JPRB, 0.90614E+00_JPRB, 0.90015E+00_JPRB, 0.89356E+00_JPRB, 0.89877E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.88485E+00_JPRB, 0.89321E+00_JPRB, 0.90093E+00_JPRB, 0.91348E+00_JPRB, 0.92530E+00_JPRB, 0.94099E+00_JPRB, 0.95192E+00_JPRB ,&
&  0.96269E+00_JPRB, 0.96989E+00_JPRB, 0.96934E+00_JPRB, 0.98210E+00_JPRB, 0.99226E+00_JPRB, 0.10128E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10879E+01_JPRB, 0.11173E+01_JPRB, 0.11315E+01_JPRB, 0.11418E+01_JPRB, 0.11563E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.11804E+01_JPRB, 0.11946E+01_JPRB, 0.12209E+01_JPRB, 0.12522E+01_JPRB, 0.12868E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.12595E+01_JPRB, 0.12037E+01_JPRB, 0.10857E+01_JPRB, 0.10429E+01_JPRB, 0.10167E+01_JPRB, 0.10116E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10070E+01_JPRB, 0.97788E+00_JPRB, 0.92432E+00_JPRB, 0.89870E+00_JPRB, 0.91635E+00_JPRB, 0.94124E+00_JPRB ,&
&  0.94854E+00_JPRB, 0.95182E+00_JPRB, 0.93802E+00_JPRB, 0.91259E+00_JPRB, 0.89289E+00_JPRB, 0.87445E+00_JPRB, 0.84062E+00_JPRB ,&
&  0.80011E+00_JPRB, 0.78964E+00_JPRB, 0.68082E+00_JPRB, 0.67900E+00_JPRB, 0.68370E+00_JPRB, 0.63827E+00_JPRB, 0.55398E+00_JPRB ,&
&  0.47439E+00_JPRB, 0.46272E+00_JPRB, 0.52935E+00_JPRB, 0.82768E+00_JPRB, 0.88918E+00_JPRB, 0.84566E+00_JPRB, 0.80155E+00_JPRB ,&
&  0.75775E+00_JPRB, 0.72342E+00_JPRB, 0.66231E+00_JPRB, 0.62047E+00_JPRB, 0.59566E+00_JPRB, 0.60517E+00_JPRB, 0.61207E+00_JPRB ,&
&  0.61078E+00_JPRB, 0.60265E+00_JPRB, 0.59159E+00_JPRB, 0.58151E+00_JPRB, 0.60714E+00_JPRB, 0.59799E+00_JPRB, 0.63808E+00_JPRB ,&
&  0.59956E+00_JPRB, 0.44881E+00_JPRB, 0.46969E+00_JPRB, 0.56953E+00_JPRB, 0.53734E+00_JPRB, 0.59532E+00_JPRB, 0.70998E+00_JPRB ,&
&  0.91302E+00_JPRB, 0.10290E+01_JPRB, 0.10046E+01_JPRB, 0.96613E+00_JPRB, 0.90666E+00_JPRB, 0.90289E+00_JPRB, 0.88034E+00_JPRB ,&
&  0.86260E+00_JPRB, 0.85968E+00_JPRB, 0.86169E+00_JPRB, 0.86540E+00_JPRB, 0.87406E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.84126E+00_JPRB, 0.85604E+00_JPRB, 0.86779E+00_JPRB, 0.87567E+00_JPRB, 0.87936E+00_JPRB, 0.88294E+00_JPRB, 0.88925E+00_JPRB ,&
&  0.89175E+00_JPRB, 0.88926E+00_JPRB, 0.88706E+00_JPRB, 0.90120E+00_JPRB, 0.91882E+00_JPRB, 0.92583E+00_JPRB, 0.94471E+00_JPRB ,&
&  0.93836E+00_JPRB, 0.94836E+00_JPRB, 0.96427E+00_JPRB, 0.97463E+00_JPRB, 0.99498E+00_JPRB, 0.99494E+00_JPRB, 0.97973E+00_JPRB ,&
&  0.97381E+00_JPRB, 0.96537E+00_JPRB, 0.96512E+00_JPRB, 0.96574E+00_JPRB, 0.97608E+00_JPRB, 0.99097E+00_JPRB, 0.11477E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.11861E+01_JPRB, 0.10749E+01_JPRB, 0.94105E+00_JPRB, 0.92967E+00_JPRB, 0.86634E+00_JPRB, 0.83736E+00_JPRB ,&
&  0.79575E+00_JPRB, 0.74815E+00_JPRB, 0.71941E+00_JPRB, 0.69751E+00_JPRB, 0.71308E+00_JPRB, 0.74475E+00_JPRB, 0.76960E+00_JPRB ,&
&  0.80724E+00_JPRB, 0.81777E+00_JPRB, 0.81885E+00_JPRB, 0.81185E+00_JPRB, 0.79766E+00_JPRB, 0.81952E+00_JPRB, 0.79573E+00_JPRB ,&
&  0.80904E+00_JPRB, 0.85693E+00_JPRB, 0.77971E+00_JPRB, 0.80402E+00_JPRB, 0.80629E+00_JPRB, 0.77664E+00_JPRB, 0.70628E+00_JPRB ,&
&  0.61085E+00_JPRB, 0.57214E+00_JPRB, 0.63895E+00_JPRB, 0.76484E+00_JPRB, 0.84989E+00_JPRB, 0.89264E+00_JPRB, 0.83247E+00_JPRB ,&
&  0.77847E+00_JPRB, 0.74684E+00_JPRB, 0.69831E+00_JPRB, 0.64113E+00_JPRB, 0.61769E+00_JPRB, 0.61128E+00_JPRB, 0.60494E+00_JPRB ,&
&  0.58911E+00_JPRB, 0.57724E+00_JPRB, 0.56533E+00_JPRB, 0.56828E+00_JPRB, 0.54320E+00_JPRB, 0.77463E+00_JPRB, 0.73914E+00_JPRB ,&
&  0.60506E+00_JPRB, 0.56649E+00_JPRB, 0.45968E+00_JPRB, 0.59270E+00_JPRB, 0.58171E+00_JPRB, 0.63567E+00_JPRB, 0.70863E+00_JPRB ,&
&  0.80079E+00_JPRB, 0.86912E+00_JPRB, 0.88785E+00_JPRB, 0.90694E+00_JPRB, 0.79846E+00_JPRB, 0.77600E+00_JPRB, 0.76753E+00_JPRB ,&
&  0.77158E+00_JPRB, 0.78238E+00_JPRB, 0.79501E+00_JPRB, 0.80820E+00_JPRB, 0.82505E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.77009E+00_JPRB, 0.78844E+00_JPRB, 0.80983E+00_JPRB, 0.81195E+00_JPRB, 0.81825E+00_JPRB, 0.81448E+00_JPRB, 0.81577E+00_JPRB ,&
&  0.80790E+00_JPRB, 0.80704E+00_JPRB, 0.80921E+00_JPRB, 0.81400E+00_JPRB, 0.82144E+00_JPRB, 0.83132E+00_JPRB, 0.85880E+00_JPRB ,&
&  0.85857E+00_JPRB, 0.86347E+00_JPRB, 0.87041E+00_JPRB, 0.88423E+00_JPRB, 0.90038E+00_JPRB, 0.90306E+00_JPRB, 0.88281E+00_JPRB ,&
&  0.86814E+00_JPRB, 0.86815E+00_JPRB, 0.87980E+00_JPRB, 0.89984E+00_JPRB, 0.10390E+01_JPRB, 0.13253E+01_JPRB, 0.17623E+01_JPRB ,&
&  0.14703E+01_JPRB, 0.10915E+01_JPRB, 0.99158E+00_JPRB, 0.85460E+00_JPRB, 0.84357E+00_JPRB, 0.86335E+00_JPRB, 0.82802E+00_JPRB ,&
&  0.80551E+00_JPRB, 0.76812E+00_JPRB, 0.75060E+00_JPRB, 0.74984E+00_JPRB, 0.75114E+00_JPRB, 0.73170E+00_JPRB, 0.72530E+00_JPRB ,&
&  0.70052E+00_JPRB, 0.69597E+00_JPRB, 0.71311E+00_JPRB, 0.74169E+00_JPRB, 0.82465E+00_JPRB, 0.89880E+00_JPRB, 0.88456E+00_JPRB ,&
&  0.97319E+00_JPRB, 0.10429E+01_JPRB, 0.98226E+00_JPRB, 0.98307E+00_JPRB, 0.99041E+00_JPRB, 0.95438E+00_JPRB, 0.90921E+00_JPRB ,&
&  0.91298E+00_JPRB, 0.79906E+00_JPRB, 0.64610E+00_JPRB, 0.72600E+00_JPRB, 0.80149E+00_JPRB, 0.91515E+00_JPRB, 0.86550E+00_JPRB ,&
&  0.79646E+00_JPRB, 0.75556E+00_JPRB, 0.71223E+00_JPRB, 0.65979E+00_JPRB, 0.63510E+00_JPRB, 0.63264E+00_JPRB, 0.62101E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.59084E+00_JPRB, 0.58774E+00_JPRB, 0.58645E+00_JPRB, 0.63009E+00_JPRB, 0.60158E+00_JPRB, 0.67467E+00_JPRB ,&
&  0.59102E+00_JPRB, 0.59354E+00_JPRB, 0.52612E+00_JPRB, 0.65350E+00_JPRB, 0.62770E+00_JPRB, 0.64307E+00_JPRB, 0.67200E+00_JPRB ,&
&  0.70553E+00_JPRB, 0.72097E+00_JPRB, 0.74172E+00_JPRB, 0.75669E+00_JPRB, 0.74752E+00_JPRB, 0.73498E+00_JPRB, 0.74089E+00_JPRB ,&
&  0.74592E+00_JPRB, 0.74217E+00_JPRB, 0.74153E+00_JPRB, 0.74288E+00_JPRB, 0.75601E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.66590E+00_JPRB, 0.67576E+00_JPRB, 0.69169E+00_JPRB, 0.70338E+00_JPRB, 0.72582E+00_JPRB, 0.72046E+00_JPRB, 0.71686E+00_JPRB ,&
&  0.69304E+00_JPRB, 0.69001E+00_JPRB, 0.68500E+00_JPRB, 0.68345E+00_JPRB, 0.68710E+00_JPRB, 0.70816E+00_JPRB, 0.74842E+00_JPRB ,&
&  0.76039E+00_JPRB, 0.76532E+00_JPRB, 0.76118E+00_JPRB, 0.75396E+00_JPRB, 0.72800E+00_JPRB, 0.70645E+00_JPRB, 0.69314E+00_JPRB ,&
&  0.71628E+00_JPRB, 0.89396E+00_JPRB, 0.11056E+01_JPRB, 0.12750E+01_JPRB, 0.14539E+01_JPRB, 0.12997E+01_JPRB, 0.14548E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.10802E+01_JPRB, 0.11031E+01_JPRB, 0.98593E+00_JPRB, 0.97599E+00_JPRB, 0.95966E+00_JPRB, 0.93975E+00_JPRB ,&
&  0.95268E+00_JPRB, 0.94792E+00_JPRB, 0.92673E+00_JPRB, 0.91522E+00_JPRB, 0.92527E+00_JPRB, 0.94627E+00_JPRB, 0.95777E+00_JPRB ,&
&  0.97021E+00_JPRB, 0.94877E+00_JPRB, 0.95111E+00_JPRB, 0.99403E+00_JPRB, 0.10352E+01_JPRB, 0.10624E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.11864E+01_JPRB, 0.10487E+01_JPRB, 0.10277E+01_JPRB, 0.10783E+01_JPRB, 0.98458E+00_JPRB, 0.98776E+00_JPRB ,&
&  0.10562E+01_JPRB, 0.93562E+00_JPRB, 0.65527E+00_JPRB, 0.71503E+00_JPRB, 0.78909E+00_JPRB, 0.83828E+00_JPRB, 0.92940E+00_JPRB ,&
&  0.84802E+00_JPRB, 0.81487E+00_JPRB, 0.72890E+00_JPRB, 0.68430E+00_JPRB, 0.67460E+00_JPRB, 0.70131E+00_JPRB, 0.68109E+00_JPRB ,&
&  0.68470E+00_JPRB, 0.61414E+00_JPRB, 0.61137E+00_JPRB, 0.62196E+00_JPRB, 0.62216E+00_JPRB, 0.62754E+00_JPRB, 0.62205E+00_JPRB ,&
&  0.61942E+00_JPRB, 0.60291E+00_JPRB, 0.60910E+00_JPRB, 0.61013E+00_JPRB, 0.55749E+00_JPRB, 0.65840E+00_JPRB, 0.66631E+00_JPRB ,&
&  0.67053E+00_JPRB, 0.66120E+00_JPRB, 0.64673E+00_JPRB, 0.64492E+00_JPRB, 0.63432E+00_JPRB, 0.62532E+00_JPRB, 0.63677E+00_JPRB ,&
&  0.65486E+00_JPRB, 0.65152E+00_JPRB, 0.64685E+00_JPRB, 0.64683E+00_JPRB, 0.65373E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.56722E+00_JPRB, 0.57001E+00_JPRB, 0.57217E+00_JPRB, 0.57932E+00_JPRB, 0.59218E+00_JPRB, 0.59758E+00_JPRB, 0.59355E+00_JPRB ,&
&  0.58022E+00_JPRB, 0.57406E+00_JPRB, 0.55511E+00_JPRB, 0.53170E+00_JPRB, 0.53184E+00_JPRB, 0.57552E+00_JPRB, 0.62230E+00_JPRB ,&
&  0.65421E+00_JPRB, 0.67183E+00_JPRB, 0.67809E+00_JPRB, 0.68719E+00_JPRB, 0.73110E+00_JPRB, 0.76987E+00_JPRB, 0.97199E+00_JPRB ,&
&  0.13166E+01_JPRB, 0.17777E+01_JPRB, 0.20735E+01_JPRB, 0.21727E+01_JPRB, 0.19597E+01_JPRB, 0.14968E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.13999E+01_JPRB, 0.11603E+01_JPRB, 0.10895E+01_JPRB, 0.10691E+01_JPRB, 0.10544E+01_JPRB, 0.10090E+01_JPRB, 0.98987E+00_JPRB ,&
&  0.99038E+00_JPRB, 0.99909E+00_JPRB, 0.10089E+01_JPRB, 0.10236E+01_JPRB, 0.10381E+01_JPRB, 0.10605E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.11022E+01_JPRB, 0.10930E+01_JPRB, 0.11396E+01_JPRB, 0.10343E+01_JPRB, 0.10436E+01_JPRB, 0.10647E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.10100E+01_JPRB, 0.10073E+01_JPRB, 0.10125E+01_JPRB, 0.10295E+01_JPRB, 0.97860E+00_JPRB, 0.93877E+00_JPRB ,&
&  0.96364E+00_JPRB, 0.92440E+00_JPRB, 0.71992E+00_JPRB, 0.75954E+00_JPRB, 0.85228E+00_JPRB, 0.78510E+00_JPRB, 0.95346E+00_JPRB ,&
&  0.91441E+00_JPRB, 0.86575E+00_JPRB, 0.80951E+00_JPRB, 0.76033E+00_JPRB, 0.76671E+00_JPRB, 0.78091E+00_JPRB, 0.73601E+00_JPRB ,&
&  0.79793E+00_JPRB, 0.69687E+00_JPRB, 0.67452E+00_JPRB, 0.68487E+00_JPRB, 0.67980E+00_JPRB, 0.67678E+00_JPRB, 0.67573E+00_JPRB ,&
&  0.67545E+00_JPRB, 0.67376E+00_JPRB, 0.63805E+00_JPRB, 0.66224E+00_JPRB, 0.60070E+00_JPRB, 0.64864E+00_JPRB, 0.65747E+00_JPRB ,&
&  0.65494E+00_JPRB, 0.63553E+00_JPRB, 0.63060E+00_JPRB, 0.62491E+00_JPRB, 0.62309E+00_JPRB, 0.60797E+00_JPRB, 0.61149E+00_JPRB ,&
&  0.60722E+00_JPRB, 0.60124E+00_JPRB, 0.59214E+00_JPRB, 0.58983E+00_JPRB, 0.57676E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.60058E+00_JPRB, 0.59342E+00_JPRB, 0.57994E+00_JPRB, 0.56399E+00_JPRB, 0.54832E+00_JPRB, 0.52581E+00_JPRB, 0.53030E+00_JPRB ,&
&  0.52306E+00_JPRB, 0.53239E+00_JPRB, 0.55469E+00_JPRB, 0.57364E+00_JPRB, 0.60888E+00_JPRB, 0.66535E+00_JPRB, 0.72192E+00_JPRB ,&
&  0.76686E+00_JPRB, 0.79304E+00_JPRB, 0.86070E+00_JPRB, 0.96484E+00_JPRB, 0.11348E+01_JPRB, 0.14098E+01_JPRB, 0.18336E+01_JPRB ,&
&  0.20150E+01_JPRB, 0.21087E+01_JPRB, 0.20323E+01_JPRB, 0.17880E+01_JPRB, 0.14987E+01_JPRB, 0.13022E+01_JPRB, 0.13907E+01_JPRB ,&
&  0.14467E+01_JPRB, 0.13690E+01_JPRB, 0.12262E+01_JPRB, 0.11666E+01_JPRB, 0.11114E+01_JPRB, 0.10689E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10505E+01_JPRB, 0.10435E+01_JPRB, 0.10571E+01_JPRB, 0.10630E+01_JPRB, 0.10703E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.10737E+01_JPRB, 0.10554E+01_JPRB, 0.10456E+01_JPRB, 0.10010E+01_JPRB, 0.99262E+00_JPRB, 0.98766E+00_JPRB, 0.96858E+00_JPRB ,&
&  0.94747E+00_JPRB, 0.89307E+00_JPRB, 0.90153E+00_JPRB, 0.92388E+00_JPRB, 0.90624E+00_JPRB, 0.85152E+00_JPRB, 0.85143E+00_JPRB ,&
&  0.88031E+00_JPRB, 0.91633E+00_JPRB, 0.74493E+00_JPRB, 0.87533E+00_JPRB, 0.10201E+01_JPRB, 0.97685E+00_JPRB, 0.97441E+00_JPRB ,&
&  0.98703E+00_JPRB, 0.94197E+00_JPRB, 0.91989E+00_JPRB, 0.86098E+00_JPRB, 0.88006E+00_JPRB, 0.87479E+00_JPRB, 0.79203E+00_JPRB ,&
&  0.91550E+00_JPRB, 0.80904E+00_JPRB, 0.79489E+00_JPRB, 0.77973E+00_JPRB, 0.75006E+00_JPRB, 0.71196E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.65332E+00_JPRB, 0.82934E+00_JPRB, 0.92428E+00_JPRB, 0.11003E+01_JPRB, 0.10959E+01_JPRB, 0.66337E+00_JPRB, 0.68495E+00_JPRB ,&
&  0.67972E+00_JPRB, 0.67544E+00_JPRB, 0.67523E+00_JPRB, 0.66958E+00_JPRB, 0.66189E+00_JPRB, 0.65714E+00_JPRB, 0.65796E+00_JPRB ,&
&  0.64922E+00_JPRB, 0.64052E+00_JPRB, 0.63108E+00_JPRB, 0.62418E+00_JPRB, 0.61356E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.69086E+00_JPRB, 0.69284E+00_JPRB, 0.68131E+00_JPRB, 0.65426E+00_JPRB, 0.61867E+00_JPRB, 0.58181E+00_JPRB, 0.56091E+00_JPRB ,&
&  0.53936E+00_JPRB, 0.55959E+00_JPRB, 0.60287E+00_JPRB, 0.66140E+00_JPRB, 0.73372E+00_JPRB, 0.81214E+00_JPRB, 0.89094E+00_JPRB ,&
&  0.94520E+00_JPRB, 0.10034E+01_JPRB, 0.10941E+01_JPRB, 0.12922E+01_JPRB, 0.15056E+01_JPRB, 0.16563E+01_JPRB, 0.17310E+01_JPRB ,&
&  0.17789E+01_JPRB, 0.18063E+01_JPRB, 0.17179E+01_JPRB, 0.13176E+01_JPRB, 0.10912E+01_JPRB, 0.12275E+01_JPRB, 0.11503E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.11430E+01_JPRB, 0.11433E+01_JPRB, 0.11410E+01_JPRB, 0.10954E+01_JPRB, 0.10566E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.10431E+01_JPRB, 0.10516E+01_JPRB, 0.10435E+01_JPRB, 0.10526E+01_JPRB, 0.10701E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.10161E+01_JPRB, 0.10002E+01_JPRB, 0.10010E+01_JPRB, 0.97677E+00_JPRB, 0.95704E+00_JPRB, 0.92308E+00_JPRB ,&
&  0.89364E+00_JPRB, 0.87139E+00_JPRB, 0.87419E+00_JPRB, 0.89636E+00_JPRB, 0.89771E+00_JPRB, 0.82451E+00_JPRB, 0.83350E+00_JPRB ,&
&  0.86965E+00_JPRB, 0.88447E+00_JPRB, 0.83017E+00_JPRB, 0.95752E+00_JPRB, 0.72209E+00_JPRB, 0.85114E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.96826E+00_JPRB, 0.91589E+00_JPRB, 0.89129E+00_JPRB, 0.85275E+00_JPRB, 0.81737E+00_JPRB, 0.90116E+00_JPRB ,&
&  0.94894E+00_JPRB, 0.90998E+00_JPRB, 0.87742E+00_JPRB, 0.84078E+00_JPRB, 0.82889E+00_JPRB, 0.79523E+00_JPRB, 0.77271E+00_JPRB ,&
&  0.73400E+00_JPRB, 0.84559E+00_JPRB, 0.86273E+00_JPRB, 0.87769E+00_JPRB, 0.86599E+00_JPRB, 0.65443E+00_JPRB, 0.68001E+00_JPRB ,&
&  0.68350E+00_JPRB, 0.67258E+00_JPRB, 0.67454E+00_JPRB, 0.67264E+00_JPRB, 0.65837E+00_JPRB, 0.65682E+00_JPRB, 0.66602E+00_JPRB ,&
&  0.66523E+00_JPRB, 0.66724E+00_JPRB, 0.67188E+00_JPRB, 0.67422E+00_JPRB, 0.68222E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.68876E+00_JPRB, 0.70541E+00_JPRB, 0.72834E+00_JPRB, 0.74093E+00_JPRB, 0.74237E+00_JPRB, 0.72582E+00_JPRB, 0.67750E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.60653E+00_JPRB, 0.63946E+00_JPRB, 0.68097E+00_JPRB, 0.73101E+00_JPRB, 0.79279E+00_JPRB, 0.88087E+00_JPRB ,&
&  0.99217E+00_JPRB, 0.10931E+01_JPRB, 0.11976E+01_JPRB, 0.13633E+01_JPRB, 0.15403E+01_JPRB, 0.15556E+01_JPRB, 0.15660E+01_JPRB ,&
&  0.13833E+01_JPRB, 0.11586E+01_JPRB, 0.16369E+01_JPRB, 0.18271E+01_JPRB, 0.16008E+01_JPRB, 0.12475E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.11249E+01_JPRB, 0.11450E+01_JPRB, 0.11478E+01_JPRB, 0.10555E+01_JPRB, 0.10419E+01_JPRB, 0.10237E+01_JPRB ,&
&  0.10151E+01_JPRB, 0.99280E+00_JPRB, 0.99929E+00_JPRB, 0.10060E+01_JPRB, 0.99729E+00_JPRB, 0.10102E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.99776E+00_JPRB, 0.97753E+00_JPRB, 0.97824E+00_JPRB, 0.97647E+00_JPRB, 0.98115E+00_JPRB, 0.96746E+00_JPRB, 0.92927E+00_JPRB ,&
&  0.89652E+00_JPRB, 0.86495E+00_JPRB, 0.88382E+00_JPRB, 0.91817E+00_JPRB, 0.94132E+00_JPRB, 0.87741E+00_JPRB, 0.90539E+00_JPRB ,&
&  0.90347E+00_JPRB, 0.88920E+00_JPRB, 0.88485E+00_JPRB, 0.96587E+00_JPRB, 0.79312E+00_JPRB, 0.87498E+00_JPRB, 0.90312E+00_JPRB ,&
&  0.10150E+01_JPRB, 0.98408E+00_JPRB, 0.92494E+00_JPRB, 0.88836E+00_JPRB, 0.87422E+00_JPRB, 0.78658E+00_JPRB, 0.81875E+00_JPRB ,&
&  0.85571E+00_JPRB, 0.87175E+00_JPRB, 0.84887E+00_JPRB, 0.84780E+00_JPRB, 0.79981E+00_JPRB, 0.79558E+00_JPRB, 0.82375E+00_JPRB ,&
&  0.91964E+00_JPRB, 0.89607E+00_JPRB, 0.83530E+00_JPRB, 0.75529E+00_JPRB, 0.71773E+00_JPRB, 0.63215E+00_JPRB, 0.66276E+00_JPRB ,&
&  0.66936E+00_JPRB, 0.66433E+00_JPRB, 0.65335E+00_JPRB, 0.65522E+00_JPRB, 0.65698E+00_JPRB, 0.65269E+00_JPRB, 0.65937E+00_JPRB ,&
&  0.66300E+00_JPRB, 0.66670E+00_JPRB, 0.66907E+00_JPRB, 0.67270E+00_JPRB, 0.67784E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.66714E+00_JPRB, 0.69585E+00_JPRB, 0.74053E+00_JPRB, 0.81082E+00_JPRB, 0.88197E+00_JPRB, 0.98190E+00_JPRB, 0.10647E+01_JPRB ,&
&  0.94223E+00_JPRB, 0.61262E+00_JPRB, 0.63280E+00_JPRB, 0.65286E+00_JPRB, 0.68577E+00_JPRB, 0.73341E+00_JPRB, 0.79793E+00_JPRB ,&
&  0.90323E+00_JPRB, 0.10453E+01_JPRB, 0.12140E+01_JPRB, 0.13563E+01_JPRB, 0.14799E+01_JPRB, 0.15013E+01_JPRB, 0.13946E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.13069E+01_JPRB, 0.21131E+01_JPRB, 0.19300E+01_JPRB, 0.17189E+01_JPRB, 0.14060E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10502E+01_JPRB, 0.10476E+01_JPRB, 0.10268E+01_JPRB, 0.97720E+00_JPRB, 0.97303E+00_JPRB, 0.97532E+00_JPRB ,&
&  0.96699E+00_JPRB, 0.94505E+00_JPRB, 0.92334E+00_JPRB, 0.93108E+00_JPRB, 0.92337E+00_JPRB, 0.92064E+00_JPRB, 0.92860E+00_JPRB ,&
&  0.94729E+00_JPRB, 0.95022E+00_JPRB, 0.94840E+00_JPRB, 0.93588E+00_JPRB, 0.96127E+00_JPRB, 0.97680E+00_JPRB, 0.94321E+00_JPRB ,&
&  0.90210E+00_JPRB, 0.89970E+00_JPRB, 0.90848E+00_JPRB, 0.89023E+00_JPRB, 0.84857E+00_JPRB, 0.91181E+00_JPRB, 0.89861E+00_JPRB ,&
&  0.92854E+00_JPRB, 0.89819E+00_JPRB, 0.94794E+00_JPRB, 0.83566E+00_JPRB, 0.87625E+00_JPRB, 0.95359E+00_JPRB, 0.96481E+00_JPRB ,&
&  0.96953E+00_JPRB, 0.10091E+01_JPRB, 0.96139E+00_JPRB, 0.95196E+00_JPRB, 0.92695E+00_JPRB, 0.82554E+00_JPRB, 0.84344E+00_JPRB ,&
&  0.81182E+00_JPRB, 0.86828E+00_JPRB, 0.84012E+00_JPRB, 0.80289E+00_JPRB, 0.70960E+00_JPRB, 0.68140E+00_JPRB, 0.70650E+00_JPRB ,&
&  0.81274E+00_JPRB, 0.91375E+00_JPRB, 0.89947E+00_JPRB, 0.83611E+00_JPRB, 0.72976E+00_JPRB, 0.66120E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.65885E+00_JPRB, 0.65004E+00_JPRB, 0.64692E+00_JPRB, 0.65252E+00_JPRB, 0.65630E+00_JPRB, 0.65279E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.64557E+00_JPRB, 0.64429E+00_JPRB, 0.64215E+00_JPRB, 0.64533E+00_JPRB, 0.65369E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.61907E+00_JPRB, 0.63370E+00_JPRB, 0.66180E+00_JPRB, 0.69581E+00_JPRB, 0.72968E+00_JPRB, 0.77043E+00_JPRB, 0.75974E+00_JPRB ,&
&  0.66856E+00_JPRB, 0.61319E+00_JPRB, 0.62928E+00_JPRB, 0.62894E+00_JPRB, 0.63511E+00_JPRB, 0.66278E+00_JPRB, 0.71045E+00_JPRB ,&
&  0.77446E+00_JPRB, 0.89884E+00_JPRB, 0.10721E+01_JPRB, 0.13020E+01_JPRB, 0.13899E+01_JPRB, 0.14193E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.10891E+01_JPRB, 0.21918E+01_JPRB, 0.19786E+01_JPRB, 0.20175E+01_JPRB, 0.18501E+01_JPRB, 0.17287E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10144E+01_JPRB, 0.10118E+01_JPRB, 0.94614E+00_JPRB, 0.90644E+00_JPRB, 0.90333E+00_JPRB, 0.91191E+00_JPRB ,&
&  0.90253E+00_JPRB, 0.89205E+00_JPRB, 0.87261E+00_JPRB, 0.87389E+00_JPRB, 0.87358E+00_JPRB, 0.87650E+00_JPRB, 0.88712E+00_JPRB ,&
&  0.90718E+00_JPRB, 0.91866E+00_JPRB, 0.90974E+00_JPRB, 0.91555E+00_JPRB, 0.93174E+00_JPRB, 0.96894E+00_JPRB, 0.97119E+00_JPRB ,&
&  0.91523E+00_JPRB, 0.86948E+00_JPRB, 0.90899E+00_JPRB, 0.91331E+00_JPRB, 0.90668E+00_JPRB, 0.95012E+00_JPRB, 0.89741E+00_JPRB ,&
&  0.95501E+00_JPRB, 0.90348E+00_JPRB, 0.94463E+00_JPRB, 0.82703E+00_JPRB, 0.86978E+00_JPRB, 0.99059E+00_JPRB, 0.10121E+01_JPRB ,&
&  0.97716E+00_JPRB, 0.10469E+01_JPRB, 0.10084E+01_JPRB, 0.99787E+00_JPRB, 0.97203E+00_JPRB, 0.89512E+00_JPRB, 0.83386E+00_JPRB ,&
&  0.83887E+00_JPRB, 0.87253E+00_JPRB, 0.86452E+00_JPRB, 0.75504E+00_JPRB, 0.70605E+00_JPRB, 0.65290E+00_JPRB, 0.67381E+00_JPRB ,&
&  0.75135E+00_JPRB, 0.84027E+00_JPRB, 0.90694E+00_JPRB, 0.92881E+00_JPRB, 0.84914E+00_JPRB, 0.78641E+00_JPRB, 0.73408E+00_JPRB ,&
&  0.69379E+00_JPRB, 0.65992E+00_JPRB, 0.64767E+00_JPRB, 0.64611E+00_JPRB, 0.64868E+00_JPRB, 0.65076E+00_JPRB, 0.64293E+00_JPRB ,&
&  0.63781E+00_JPRB, 0.63334E+00_JPRB, 0.62934E+00_JPRB, 0.62459E+00_JPRB, 0.62132E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.61687E+00_JPRB, 0.60562E+00_JPRB, 0.61842E+00_JPRB, 0.64884E+00_JPRB, 0.66129E+00_JPRB, 0.67050E+00_JPRB, 0.64862E+00_JPRB ,&
&  0.58442E+00_JPRB, 0.56880E+00_JPRB, 0.57050E+00_JPRB, 0.57497E+00_JPRB, 0.58466E+00_JPRB, 0.61576E+00_JPRB, 0.66034E+00_JPRB ,&
&  0.71571E+00_JPRB, 0.79743E+00_JPRB, 0.88381E+00_JPRB, 0.10538E+01_JPRB, 0.11992E+01_JPRB, 0.13655E+01_JPRB, 0.96671E+00_JPRB ,&
&  0.11823E+01_JPRB, 0.20558E+01_JPRB, 0.21043E+01_JPRB, 0.21325E+01_JPRB, 0.20977E+01_JPRB, 0.20278E+01_JPRB, 0.16227E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.11252E+01_JPRB, 0.10162E+01_JPRB, 0.91948E+00_JPRB, 0.84503E+00_JPRB, 0.85798E+00_JPRB, 0.86685E+00_JPRB ,&
&  0.87535E+00_JPRB, 0.89717E+00_JPRB, 0.85002E+00_JPRB, 0.83794E+00_JPRB, 0.82818E+00_JPRB, 0.81810E+00_JPRB, 0.84436E+00_JPRB ,&
&  0.87406E+00_JPRB, 0.88796E+00_JPRB, 0.89963E+00_JPRB, 0.87326E+00_JPRB, 0.92074E+00_JPRB, 0.95491E+00_JPRB, 0.97553E+00_JPRB ,&
&  0.90580E+00_JPRB, 0.85650E+00_JPRB, 0.92559E+00_JPRB, 0.10083E+01_JPRB, 0.10528E+01_JPRB, 0.10413E+01_JPRB, 0.97743E+00_JPRB ,&
&  0.97418E+00_JPRB, 0.93044E+00_JPRB, 0.88629E+00_JPRB, 0.81079E+00_JPRB, 0.86017E+00_JPRB, 0.99752E+00_JPRB, 0.10961E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.10789E+01_JPRB, 0.10584E+01_JPRB, 0.10048E+01_JPRB, 0.98631E+00_JPRB, 0.93302E+00_JPRB, 0.94516E+00_JPRB ,&
&  0.10257E+01_JPRB, 0.10985E+01_JPRB, 0.93197E+00_JPRB, 0.78604E+00_JPRB, 0.74773E+00_JPRB, 0.64548E+00_JPRB, 0.62857E+00_JPRB ,&
&  0.68501E+00_JPRB, 0.71516E+00_JPRB, 0.74589E+00_JPRB, 0.87080E+00_JPRB, 0.92148E+00_JPRB, 0.93878E+00_JPRB, 0.85770E+00_JPRB ,&
&  0.74587E+00_JPRB, 0.71922E+00_JPRB, 0.68784E+00_JPRB, 0.66520E+00_JPRB, 0.66303E+00_JPRB, 0.65764E+00_JPRB, 0.65355E+00_JPRB ,&
&  0.64323E+00_JPRB, 0.63806E+00_JPRB, 0.63632E+00_JPRB, 0.63673E+00_JPRB, 0.63241E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.63978E+00_JPRB, 0.62579E+00_JPRB, 0.60995E+00_JPRB, 0.61018E+00_JPRB, 0.63004E+00_JPRB, 0.62152E+00_JPRB, 0.59797E+00_JPRB ,&
&  0.55241E+00_JPRB, 0.52950E+00_JPRB, 0.50696E+00_JPRB, 0.50028E+00_JPRB, 0.51075E+00_JPRB, 0.54053E+00_JPRB, 0.59017E+00_JPRB ,&
&  0.65931E+00_JPRB, 0.72301E+00_JPRB, 0.78795E+00_JPRB, 0.84713E+00_JPRB, 0.94144E+00_JPRB, 0.90093E+00_JPRB, 0.99820E+00_JPRB ,&
&  0.14682E+01_JPRB, 0.19044E+01_JPRB, 0.20569E+01_JPRB, 0.21230E+01_JPRB, 0.22185E+01_JPRB, 0.23474E+01_JPRB, 0.20043E+01_JPRB ,&
&  0.16740E+01_JPRB, 0.14066E+01_JPRB, 0.11492E+01_JPRB, 0.99324E+00_JPRB, 0.88122E+00_JPRB, 0.82216E+00_JPRB, 0.82499E+00_JPRB ,&
&  0.83010E+00_JPRB, 0.84410E+00_JPRB, 0.82170E+00_JPRB, 0.79764E+00_JPRB, 0.78232E+00_JPRB, 0.76642E+00_JPRB, 0.77947E+00_JPRB ,&
&  0.81021E+00_JPRB, 0.83561E+00_JPRB, 0.84984E+00_JPRB, 0.84951E+00_JPRB, 0.88906E+00_JPRB, 0.93240E+00_JPRB, 0.98422E+00_JPRB ,&
&  0.96501E+00_JPRB, 0.97539E+00_JPRB, 0.10084E+01_JPRB, 0.10384E+01_JPRB, 0.11233E+01_JPRB, 0.11662E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.92964E+00_JPRB, 0.79764E+00_JPRB, 0.80311E+00_JPRB, 0.90017E+00_JPRB, 0.10189E+01_JPRB, 0.99499E+00_JPRB ,&
&  0.93005E+00_JPRB, 0.93299E+00_JPRB, 0.96843E+00_JPRB, 0.10643E+01_JPRB, 0.11400E+01_JPRB, 0.11112E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.66041E+00_JPRB, 0.57743E+00_JPRB, 0.65247E+00_JPRB, 0.55273E+00_JPRB, 0.54011E+00_JPRB, 0.63920E+00_JPRB ,&
&  0.66454E+00_JPRB, 0.71757E+00_JPRB, 0.74208E+00_JPRB, 0.72124E+00_JPRB, 0.83410E+00_JPRB, 0.94878E+00_JPRB, 0.94803E+00_JPRB ,&
&  0.83053E+00_JPRB, 0.79988E+00_JPRB, 0.76596E+00_JPRB, 0.74134E+00_JPRB, 0.72280E+00_JPRB, 0.69779E+00_JPRB, 0.68382E+00_JPRB ,&
&  0.66954E+00_JPRB, 0.65803E+00_JPRB, 0.65119E+00_JPRB, 0.64606E+00_JPRB, 0.64678E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.66779E+00_JPRB, 0.66036E+00_JPRB, 0.64888E+00_JPRB, 0.63120E+00_JPRB, 0.62614E+00_JPRB, 0.61917E+00_JPRB, 0.59149E+00_JPRB ,&
&  0.54611E+00_JPRB, 0.51651E+00_JPRB, 0.48854E+00_JPRB, 0.47099E+00_JPRB, 0.47597E+00_JPRB, 0.49044E+00_JPRB, 0.52858E+00_JPRB ,&
&  0.59364E+00_JPRB, 0.69271E+00_JPRB, 0.80475E+00_JPRB, 0.83126E+00_JPRB, 0.83008E+00_JPRB, 0.78351E+00_JPRB, 0.98946E+00_JPRB ,&
&  0.13061E+01_JPRB, 0.15821E+01_JPRB, 0.20551E+01_JPRB, 0.22826E+01_JPRB, 0.23429E+01_JPRB, 0.23551E+01_JPRB, 0.22274E+01_JPRB ,&
&  0.19944E+01_JPRB, 0.17155E+01_JPRB, 0.13625E+01_JPRB, 0.11155E+01_JPRB, 0.95733E+00_JPRB, 0.85329E+00_JPRB, 0.79546E+00_JPRB ,&
&  0.80374E+00_JPRB, 0.81736E+00_JPRB, 0.80306E+00_JPRB, 0.76845E+00_JPRB, 0.74473E+00_JPRB, 0.73684E+00_JPRB, 0.74202E+00_JPRB ,&
&  0.75547E+00_JPRB, 0.77751E+00_JPRB, 0.79681E+00_JPRB, 0.83424E+00_JPRB, 0.74016E+00_JPRB, 0.89105E+00_JPRB, 0.96298E+00_JPRB ,&
&  0.10366E+01_JPRB, 0.10829E+01_JPRB, 0.10468E+01_JPRB, 0.11247E+01_JPRB, 0.12604E+01_JPRB, 0.12802E+01_JPRB, 0.12666E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.10056E+01_JPRB, 0.83201E+00_JPRB, 0.84768E+00_JPRB, 0.94222E+00_JPRB, 0.10275E+01_JPRB, 0.78443E+00_JPRB ,&
&  0.74460E+00_JPRB, 0.95709E+00_JPRB, 0.99395E+00_JPRB, 0.95588E+00_JPRB, 0.12263E+01_JPRB, 0.14230E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.48292E+00_JPRB, 0.42058E+00_JPRB, 0.43917E+00_JPRB, 0.42233E+00_JPRB, 0.42658E+00_JPRB, 0.46734E+00_JPRB, 0.73978E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.72150E+00_JPRB, 0.72641E+00_JPRB, 0.74781E+00_JPRB, 0.82091E+00_JPRB, 0.96991E+00_JPRB, 0.11169E+01_JPRB ,&
&  0.87832E+00_JPRB, 0.84880E+00_JPRB, 0.77197E+00_JPRB, 0.74145E+00_JPRB, 0.73125E+00_JPRB, 0.72822E+00_JPRB, 0.71288E+00_JPRB ,&
&  0.70451E+00_JPRB, 0.70046E+00_JPRB, 0.69356E+00_JPRB, 0.68574E+00_JPRB, 0.67801E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.67685E+00_JPRB, 0.67531E+00_JPRB, 0.67449E+00_JPRB, 0.66723E+00_JPRB, 0.65390E+00_JPRB, 0.63530E+00_JPRB, 0.60958E+00_JPRB ,&
&  0.56948E+00_JPRB, 0.52862E+00_JPRB, 0.49745E+00_JPRB, 0.47896E+00_JPRB, 0.47419E+00_JPRB, 0.47417E+00_JPRB, 0.49450E+00_JPRB ,&
&  0.54631E+00_JPRB, 0.66886E+00_JPRB, 0.88902E+00_JPRB, 0.83634E+00_JPRB, 0.85731E+00_JPRB, 0.80334E+00_JPRB, 0.89063E+00_JPRB ,&
&  0.11627E+01_JPRB, 0.13954E+01_JPRB, 0.17596E+01_JPRB, 0.21181E+01_JPRB, 0.25182E+01_JPRB, 0.25600E+01_JPRB, 0.25113E+01_JPRB ,&
&  0.22892E+01_JPRB, 0.20219E+01_JPRB, 0.16343E+01_JPRB, 0.12994E+01_JPRB, 0.10463E+01_JPRB, 0.89967E+00_JPRB, 0.81456E+00_JPRB ,&
&  0.79839E+00_JPRB, 0.80913E+00_JPRB, 0.79419E+00_JPRB, 0.74730E+00_JPRB, 0.71926E+00_JPRB, 0.71291E+00_JPRB, 0.72892E+00_JPRB ,&
&  0.73276E+00_JPRB, 0.73966E+00_JPRB, 0.74346E+00_JPRB, 0.77982E+00_JPRB, 0.71540E+00_JPRB, 0.67561E+00_JPRB, 0.85853E+00_JPRB ,&
&  0.10345E+01_JPRB, 0.12364E+01_JPRB, 0.12857E+01_JPRB, 0.13629E+01_JPRB, 0.14728E+01_JPRB, 0.14777E+01_JPRB, 0.15210E+01_JPRB ,&
&  0.13799E+01_JPRB, 0.13155E+01_JPRB, 0.96340E+00_JPRB, 0.94085E+00_JPRB, 0.10481E+01_JPRB, 0.86736E+00_JPRB, 0.80288E+00_JPRB ,&
&  0.89798E+00_JPRB, 0.86788E+00_JPRB, 0.87663E+00_JPRB, 0.78302E+00_JPRB, 0.10863E+01_JPRB, 0.11295E+01_JPRB, 0.53371E+00_JPRB ,&
&  0.45143E+00_JPRB, 0.43061E+00_JPRB, 0.41083E+00_JPRB, 0.39731E+00_JPRB, 0.41316E+00_JPRB, 0.44760E+00_JPRB, 0.56196E+00_JPRB ,&
&  0.61300E+00_JPRB, 0.71131E+00_JPRB, 0.72158E+00_JPRB, 0.74825E+00_JPRB, 0.79072E+00_JPRB, 0.95210E+00_JPRB, 0.11689E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.10088E+01_JPRB, 0.87167E+00_JPRB, 0.78897E+00_JPRB, 0.75126E+00_JPRB, 0.73055E+00_JPRB, 0.71169E+00_JPRB ,&
&  0.69819E+00_JPRB, 0.69495E+00_JPRB, 0.69085E+00_JPRB, 0.68400E+00_JPRB, 0.67960E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.67442E+00_JPRB, 0.67658E+00_JPRB, 0.67842E+00_JPRB, 0.67821E+00_JPRB, 0.67299E+00_JPRB, 0.65755E+00_JPRB, 0.63163E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.56427E+00_JPRB, 0.53065E+00_JPRB, 0.50859E+00_JPRB, 0.49134E+00_JPRB, 0.48011E+00_JPRB, 0.48356E+00_JPRB ,&
&  0.51601E+00_JPRB, 0.65609E+00_JPRB, 0.78596E+00_JPRB, 0.72237E+00_JPRB, 0.70187E+00_JPRB, 0.70121E+00_JPRB, 0.72383E+00_JPRB ,&
&  0.10178E+01_JPRB, 0.12320E+01_JPRB, 0.15053E+01_JPRB, 0.20062E+01_JPRB, 0.24799E+01_JPRB, 0.24966E+01_JPRB, 0.26947E+01_JPRB ,&
&  0.25088E+01_JPRB, 0.22165E+01_JPRB, 0.18681E+01_JPRB, 0.15010E+01_JPRB, 0.12076E+01_JPRB, 0.98913E+00_JPRB, 0.88211E+00_JPRB ,&
&  0.82830E+00_JPRB, 0.82125E+00_JPRB, 0.80093E+00_JPRB, 0.75377E+00_JPRB, 0.73050E+00_JPRB, 0.71278E+00_JPRB, 0.70385E+00_JPRB ,&
&  0.69685E+00_JPRB, 0.71051E+00_JPRB, 0.71749E+00_JPRB, 0.76248E+00_JPRB, 0.83947E+00_JPRB, 0.84358E+00_JPRB, 0.87072E+00_JPRB ,&
&  0.94262E+00_JPRB, 0.10814E+01_JPRB, 0.14334E+01_JPRB, 0.14936E+01_JPRB, 0.16817E+01_JPRB, 0.18367E+01_JPRB, 0.17602E+01_JPRB ,&
&  0.15954E+01_JPRB, 0.14479E+01_JPRB, 0.11266E+01_JPRB, 0.11256E+01_JPRB, 0.99094E+00_JPRB, 0.79048E+00_JPRB, 0.87037E+00_JPRB ,&
&  0.91748E+00_JPRB, 0.88418E+00_JPRB, 0.78218E+00_JPRB, 0.66067E+00_JPRB, 0.70106E+00_JPRB, 0.58025E+00_JPRB, 0.45105E+00_JPRB ,&
&  0.43193E+00_JPRB, 0.41659E+00_JPRB, 0.40213E+00_JPRB, 0.41595E+00_JPRB, 0.41382E+00_JPRB, 0.44370E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.61963E+00_JPRB, 0.70080E+00_JPRB, 0.72412E+00_JPRB, 0.75588E+00_JPRB, 0.80940E+00_JPRB, 0.91687E+00_JPRB, 0.11531E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10812E+01_JPRB, 0.94427E+00_JPRB, 0.85525E+00_JPRB, 0.79910E+00_JPRB, 0.75864E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.69950E+00_JPRB, 0.68916E+00_JPRB, 0.68241E+00_JPRB, 0.67474E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.67431E+00_JPRB, 0.67404E+00_JPRB, 0.67581E+00_JPRB, 0.67514E+00_JPRB, 0.67151E+00_JPRB, 0.66211E+00_JPRB, 0.64749E+00_JPRB ,&
&  0.61910E+00_JPRB, 0.58918E+00_JPRB, 0.55285E+00_JPRB, 0.53143E+00_JPRB, 0.50967E+00_JPRB, 0.49381E+00_JPRB, 0.48766E+00_JPRB ,&
&  0.50776E+00_JPRB, 0.65512E+00_JPRB, 0.58229E+00_JPRB, 0.57516E+00_JPRB, 0.63184E+00_JPRB, 0.68369E+00_JPRB, 0.61967E+00_JPRB ,&
&  0.86846E+00_JPRB, 0.10910E+01_JPRB, 0.13799E+01_JPRB, 0.19672E+01_JPRB, 0.28473E+01_JPRB, 0.27264E+01_JPRB, 0.27897E+01_JPRB ,&
&  0.27584E+01_JPRB, 0.23336E+01_JPRB, 0.20266E+01_JPRB, 0.16865E+01_JPRB, 0.13528E+01_JPRB, 0.11217E+01_JPRB, 0.94462E+00_JPRB ,&
&  0.87647E+00_JPRB, 0.84871E+00_JPRB, 0.82048E+00_JPRB, 0.78496E+00_JPRB, 0.76158E+00_JPRB, 0.74094E+00_JPRB, 0.71199E+00_JPRB ,&
&  0.69475E+00_JPRB, 0.68644E+00_JPRB, 0.68195E+00_JPRB, 0.74738E+00_JPRB, 0.76345E+00_JPRB, 0.77627E+00_JPRB, 0.10364E+01_JPRB ,&
&  0.11553E+01_JPRB, 0.13023E+01_JPRB, 0.13624E+01_JPRB, 0.16578E+01_JPRB, 0.19781E+01_JPRB, 0.20088E+01_JPRB, 0.20466E+01_JPRB ,&
&  0.15640E+01_JPRB, 0.12399E+01_JPRB, 0.11337E+01_JPRB, 0.10976E+01_JPRB, 0.89272E+00_JPRB, 0.93939E+00_JPRB, 0.13376E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.10563E+01_JPRB, 0.11303E+01_JPRB, 0.11287E+01_JPRB, 0.66476E+00_JPRB, 0.51990E+00_JPRB, 0.72661E+00_JPRB ,&
&  0.74741E+00_JPRB, 0.52714E+00_JPRB, 0.44932E+00_JPRB, 0.53710E+00_JPRB, 0.45017E+00_JPRB, 0.47745E+00_JPRB, 0.55787E+00_JPRB ,&
&  0.60735E+00_JPRB, 0.64575E+00_JPRB, 0.73175E+00_JPRB, 0.78821E+00_JPRB, 0.85252E+00_JPRB, 0.95816E+00_JPRB, 0.10282E+01_JPRB ,&
&  0.10473E+01_JPRB, 0.10525E+01_JPRB, 0.10377E+01_JPRB, 0.94644E+00_JPRB, 0.87614E+00_JPRB, 0.80960E+00_JPRB, 0.75842E+00_JPRB ,&
&  0.72287E+00_JPRB, 0.69886E+00_JPRB, 0.68418E+00_JPRB, 0.67419E+00_JPRB, 0.67499E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.68719E+00_JPRB, 0.67843E+00_JPRB, 0.67973E+00_JPRB, 0.68659E+00_JPRB, 0.68917E+00_JPRB, 0.67965E+00_JPRB, 0.66428E+00_JPRB ,&
&  0.64901E+00_JPRB, 0.61680E+00_JPRB, 0.58027E+00_JPRB, 0.55148E+00_JPRB, 0.53365E+00_JPRB, 0.51472E+00_JPRB, 0.49949E+00_JPRB ,&
&  0.50121E+00_JPRB, 0.52792E+00_JPRB, 0.49398E+00_JPRB, 0.49404E+00_JPRB, 0.61934E+00_JPRB, 0.61124E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.76512E+00_JPRB, 0.94908E+00_JPRB, 0.12736E+01_JPRB, 0.17573E+01_JPRB, 0.22029E+01_JPRB, 0.26230E+01_JPRB, 0.28263E+01_JPRB ,&
&  0.28380E+01_JPRB, 0.24611E+01_JPRB, 0.20846E+01_JPRB, 0.17674E+01_JPRB, 0.14561E+01_JPRB, 0.12190E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.95038E+00_JPRB, 0.90639E+00_JPRB, 0.87319E+00_JPRB, 0.83500E+00_JPRB, 0.81744E+00_JPRB, 0.78026E+00_JPRB, 0.74495E+00_JPRB ,&
&  0.72860E+00_JPRB, 0.72313E+00_JPRB, 0.70223E+00_JPRB, 0.72512E+00_JPRB, 0.72522E+00_JPRB, 0.76094E+00_JPRB, 0.87413E+00_JPRB ,&
&  0.10272E+01_JPRB, 0.11880E+01_JPRB, 0.13098E+01_JPRB, 0.16542E+01_JPRB, 0.20979E+01_JPRB, 0.20237E+01_JPRB, 0.21917E+01_JPRB ,&
&  0.18840E+01_JPRB, 0.15391E+01_JPRB, 0.12347E+01_JPRB, 0.96661E+00_JPRB, 0.10540E+01_JPRB, 0.15329E+01_JPRB, 0.15521E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.13198E+01_JPRB, 0.12869E+01_JPRB, 0.11298E+01_JPRB, 0.94347E+00_JPRB, 0.72962E+00_JPRB, 0.90011E+00_JPRB ,&
&  0.92998E+00_JPRB, 0.89505E+00_JPRB, 0.82060E+00_JPRB, 0.71237E+00_JPRB, 0.59743E+00_JPRB, 0.66636E+00_JPRB, 0.65824E+00_JPRB ,&
&  0.63697E+00_JPRB, 0.61998E+00_JPRB, 0.68072E+00_JPRB, 0.78912E+00_JPRB, 0.82331E+00_JPRB, 0.88295E+00_JPRB, 0.91845E+00_JPRB ,&
&  0.91961E+00_JPRB, 0.96356E+00_JPRB, 0.96382E+00_JPRB, 0.93194E+00_JPRB, 0.90137E+00_JPRB, 0.85696E+00_JPRB, 0.81863E+00_JPRB ,&
&  0.78993E+00_JPRB, 0.76856E+00_JPRB, 0.74437E+00_JPRB, 0.72157E+00_JPRB, 0.70130E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.68688E+00_JPRB, 0.67377E+00_JPRB, 0.66978E+00_JPRB, 0.67700E+00_JPRB, 0.69424E+00_JPRB, 0.69452E+00_JPRB, 0.68312E+00_JPRB ,&
&  0.66556E+00_JPRB, 0.64568E+00_JPRB, 0.61699E+00_JPRB, 0.58272E+00_JPRB, 0.55920E+00_JPRB, 0.53098E+00_JPRB, 0.51077E+00_JPRB ,&
&  0.50454E+00_JPRB, 0.50236E+00_JPRB, 0.45467E+00_JPRB, 0.49751E+00_JPRB, 0.54569E+00_JPRB, 0.53331E+00_JPRB, 0.60081E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.81057E+00_JPRB, 0.10356E+01_JPRB, 0.13231E+01_JPRB, 0.16322E+01_JPRB, 0.20773E+01_JPRB, 0.24227E+01_JPRB ,&
&  0.24037E+01_JPRB, 0.23359E+01_JPRB, 0.21349E+01_JPRB, 0.19135E+01_JPRB, 0.16170E+01_JPRB, 0.13811E+01_JPRB, 0.12151E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10048E+01_JPRB, 0.95999E+00_JPRB, 0.92197E+00_JPRB, 0.89078E+00_JPRB, 0.85878E+00_JPRB, 0.83319E+00_JPRB ,&
&  0.79138E+00_JPRB, 0.78085E+00_JPRB, 0.75836E+00_JPRB, 0.76701E+00_JPRB, 0.73574E+00_JPRB, 0.77527E+00_JPRB, 0.84964E+00_JPRB ,&
&  0.10742E+01_JPRB, 0.12980E+01_JPRB, 0.14116E+01_JPRB, 0.18963E+01_JPRB, 0.17800E+01_JPRB, 0.17892E+01_JPRB, 0.20875E+01_JPRB ,&
&  0.23759E+01_JPRB, 0.23934E+01_JPRB, 0.22687E+01_JPRB, 0.17525E+01_JPRB, 0.14503E+01_JPRB, 0.19081E+01_JPRB, 0.18146E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.14782E+01_JPRB, 0.13377E+01_JPRB, 0.13451E+01_JPRB, 0.11175E+01_JPRB, 0.97718E+00_JPRB, 0.79529E+00_JPRB ,&
&  0.81035E+00_JPRB, 0.73076E+00_JPRB, 0.82012E+00_JPRB, 0.74791E+00_JPRB, 0.71070E+00_JPRB, 0.69327E+00_JPRB, 0.65621E+00_JPRB ,&
&  0.64091E+00_JPRB, 0.63095E+00_JPRB, 0.63090E+00_JPRB, 0.70169E+00_JPRB, 0.75564E+00_JPRB, 0.74587E+00_JPRB, 0.78977E+00_JPRB ,&
&  0.82089E+00_JPRB, 0.86298E+00_JPRB, 0.87986E+00_JPRB, 0.90799E+00_JPRB, 0.90473E+00_JPRB, 0.88595E+00_JPRB, 0.85510E+00_JPRB ,&
&  0.82070E+00_JPRB, 0.79636E+00_JPRB, 0.76874E+00_JPRB, 0.73687E+00_JPRB, 0.70816E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.66290E+00_JPRB, 0.65323E+00_JPRB, 0.64994E+00_JPRB, 0.66268E+00_JPRB, 0.68750E+00_JPRB, 0.70013E+00_JPRB, 0.68903E+00_JPRB ,&
&  0.67620E+00_JPRB, 0.66004E+00_JPRB, 0.64946E+00_JPRB, 0.61784E+00_JPRB, 0.58626E+00_JPRB, 0.54890E+00_JPRB, 0.52200E+00_JPRB ,&
&  0.52186E+00_JPRB, 0.57695E+00_JPRB, 0.50010E+00_JPRB, 0.45950E+00_JPRB, 0.43295E+00_JPRB, 0.48448E+00_JPRB, 0.55529E+00_JPRB ,&
&  0.63817E+00_JPRB, 0.70519E+00_JPRB, 0.87399E+00_JPRB, 0.97480E+00_JPRB, 0.11007E+01_JPRB, 0.14151E+01_JPRB, 0.18323E+01_JPRB ,&
&  0.23085E+01_JPRB, 0.19466E+01_JPRB, 0.19548E+01_JPRB, 0.18141E+01_JPRB, 0.16761E+01_JPRB, 0.15051E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.11946E+01_JPRB, 0.10714E+01_JPRB, 0.10022E+01_JPRB, 0.94666E+00_JPRB, 0.89748E+00_JPRB, 0.85370E+00_JPRB, 0.81799E+00_JPRB ,&
&  0.79732E+00_JPRB, 0.81159E+00_JPRB, 0.83779E+00_JPRB, 0.84283E+00_JPRB, 0.88345E+00_JPRB, 0.95924E+00_JPRB, 0.99988E+00_JPRB ,&
&  0.10081E+01_JPRB, 0.11091E+01_JPRB, 0.16107E+01_JPRB, 0.18751E+01_JPRB, 0.20867E+01_JPRB, 0.22087E+01_JPRB, 0.23531E+01_JPRB ,&
&  0.25429E+01_JPRB, 0.25829E+01_JPRB, 0.25060E+01_JPRB, 0.21256E+01_JPRB, 0.22691E+01_JPRB, 0.22083E+01_JPRB, 0.20594E+01_JPRB ,&
&  0.18646E+01_JPRB, 0.17727E+01_JPRB, 0.15611E+01_JPRB, 0.14386E+01_JPRB, 0.13944E+01_JPRB, 0.14079E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10791E+01_JPRB, 0.94841E+00_JPRB, 0.78104E+00_JPRB, 0.65188E+00_JPRB, 0.64706E+00_JPRB, 0.65085E+00_JPRB ,&
&  0.64777E+00_JPRB, 0.64592E+00_JPRB, 0.62895E+00_JPRB, 0.64947E+00_JPRB, 0.69118E+00_JPRB, 0.69633E+00_JPRB, 0.69137E+00_JPRB ,&
&  0.69486E+00_JPRB, 0.74855E+00_JPRB, 0.77429E+00_JPRB, 0.78003E+00_JPRB, 0.81254E+00_JPRB, 0.81128E+00_JPRB, 0.78509E+00_JPRB ,&
&  0.75015E+00_JPRB, 0.72222E+00_JPRB, 0.70673E+00_JPRB, 0.69101E+00_JPRB, 0.67636E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.61975E+00_JPRB, 0.62001E+00_JPRB, 0.62508E+00_JPRB, 0.64336E+00_JPRB, 0.67523E+00_JPRB, 0.69721E+00_JPRB, 0.69098E+00_JPRB ,&
&  0.68369E+00_JPRB, 0.67027E+00_JPRB, 0.65697E+00_JPRB, 0.63779E+00_JPRB, 0.60341E+00_JPRB, 0.56589E+00_JPRB, 0.54223E+00_JPRB ,&
&  0.55048E+00_JPRB, 0.70409E+00_JPRB, 0.58676E+00_JPRB, 0.55131E+00_JPRB, 0.50017E+00_JPRB, 0.53976E+00_JPRB, 0.58365E+00_JPRB ,&
&  0.65168E+00_JPRB, 0.67561E+00_JPRB, 0.72285E+00_JPRB, 0.77362E+00_JPRB, 0.84693E+00_JPRB, 0.10495E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.17374E+01_JPRB, 0.16305E+01_JPRB, 0.17271E+01_JPRB, 0.15877E+01_JPRB, 0.13917E+01_JPRB, 0.13893E+01_JPRB ,&
&  0.12657E+01_JPRB, 0.11496E+01_JPRB, 0.10343E+01_JPRB, 0.96026E+00_JPRB, 0.90760E+00_JPRB, 0.85777E+00_JPRB, 0.81645E+00_JPRB ,&
&  0.79697E+00_JPRB, 0.81069E+00_JPRB, 0.85654E+00_JPRB, 0.89020E+00_JPRB, 0.90750E+00_JPRB, 0.99906E+00_JPRB, 0.10911E+01_JPRB ,&
&  0.13532E+01_JPRB, 0.13945E+01_JPRB, 0.13751E+01_JPRB, 0.18997E+01_JPRB, 0.25156E+01_JPRB, 0.25744E+01_JPRB, 0.24441E+01_JPRB ,&
&  0.25726E+01_JPRB, 0.25992E+01_JPRB, 0.25780E+01_JPRB, 0.25178E+01_JPRB, 0.24580E+01_JPRB, 0.23437E+01_JPRB, 0.22115E+01_JPRB ,&
&  0.20729E+01_JPRB, 0.19723E+01_JPRB, 0.18260E+01_JPRB, 0.16508E+01_JPRB, 0.14763E+01_JPRB, 0.13823E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.10382E+01_JPRB, 0.73858E+00_JPRB, 0.73827E+00_JPRB, 0.59519E+00_JPRB, 0.57758E+00_JPRB, 0.62890E+00_JPRB ,&
&  0.61902E+00_JPRB, 0.64489E+00_JPRB, 0.64161E+00_JPRB, 0.60662E+00_JPRB, 0.62566E+00_JPRB, 0.64693E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.65346E+00_JPRB, 0.63160E+00_JPRB, 0.68077E+00_JPRB, 0.67860E+00_JPRB, 0.67451E+00_JPRB, 0.67386E+00_JPRB, 0.66634E+00_JPRB ,&
&  0.64308E+00_JPRB, 0.61733E+00_JPRB, 0.60395E+00_JPRB, 0.60682E+00_JPRB, 0.61347E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.55575E+00_JPRB, 0.56781E+00_JPRB, 0.58374E+00_JPRB, 0.60680E+00_JPRB, 0.64239E+00_JPRB, 0.67098E+00_JPRB, 0.67360E+00_JPRB ,&
&  0.66891E+00_JPRB, 0.66211E+00_JPRB, 0.64973E+00_JPRB, 0.63607E+00_JPRB, 0.61145E+00_JPRB, 0.57387E+00_JPRB, 0.55427E+00_JPRB ,&
&  0.62024E+00_JPRB, 0.69016E+00_JPRB, 0.60934E+00_JPRB, 0.52202E+00_JPRB, 0.59530E+00_JPRB, 0.57529E+00_JPRB, 0.59619E+00_JPRB ,&
&  0.58133E+00_JPRB, 0.64010E+00_JPRB, 0.66914E+00_JPRB, 0.66215E+00_JPRB, 0.67974E+00_JPRB, 0.82181E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.12663E+01_JPRB, 0.13086E+01_JPRB, 0.13274E+01_JPRB, 0.13803E+01_JPRB, 0.12407E+01_JPRB, 0.12256E+01_JPRB ,&
&  0.11510E+01_JPRB, 0.10696E+01_JPRB, 0.98741E+00_JPRB, 0.92457E+00_JPRB, 0.86973E+00_JPRB, 0.82264E+00_JPRB, 0.79455E+00_JPRB ,&
&  0.80585E+00_JPRB, 0.82391E+00_JPRB, 0.85883E+00_JPRB, 0.90512E+00_JPRB, 0.93964E+00_JPRB, 0.11371E+01_JPRB, 0.15184E+01_JPRB ,&
&  0.18800E+01_JPRB, 0.21261E+01_JPRB, 0.24936E+01_JPRB, 0.28484E+01_JPRB, 0.28530E+01_JPRB, 0.28584E+01_JPRB, 0.28972E+01_JPRB ,&
&  0.28421E+01_JPRB, 0.27377E+01_JPRB, 0.26203E+01_JPRB, 0.25108E+01_JPRB, 0.24612E+01_JPRB, 0.24104E+01_JPRB, 0.23201E+01_JPRB ,&
&  0.21020E+01_JPRB, 0.19637E+01_JPRB, 0.19186E+01_JPRB, 0.17326E+01_JPRB, 0.15796E+01_JPRB, 0.14013E+01_JPRB, 0.13848E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.81166E+00_JPRB, 0.65019E+00_JPRB, 0.73033E+00_JPRB, 0.61404E+00_JPRB, 0.60012E+00_JPRB, 0.66316E+00_JPRB ,&
&  0.65380E+00_JPRB, 0.63249E+00_JPRB, 0.62843E+00_JPRB, 0.59127E+00_JPRB, 0.58410E+00_JPRB, 0.63189E+00_JPRB, 0.59045E+00_JPRB ,&
&  0.55861E+00_JPRB, 0.56827E+00_JPRB, 0.59160E+00_JPRB, 0.57699E+00_JPRB, 0.56264E+00_JPRB, 0.56214E+00_JPRB, 0.55378E+00_JPRB ,&
&  0.53566E+00_JPRB, 0.52593E+00_JPRB, 0.51878E+00_JPRB, 0.52424E+00_JPRB, 0.54127E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.52095E+00_JPRB, 0.54200E+00_JPRB, 0.55746E+00_JPRB, 0.57859E+00_JPRB, 0.61001E+00_JPRB, 0.64540E+00_JPRB, 0.65692E+00_JPRB ,&
&  0.65530E+00_JPRB, 0.65037E+00_JPRB, 0.64127E+00_JPRB, 0.62639E+00_JPRB, 0.61025E+00_JPRB, 0.58075E+00_JPRB, 0.56970E+00_JPRB ,&
&  0.48994E+00_JPRB, 0.55101E+00_JPRB, 0.48014E+00_JPRB, 0.49818E+00_JPRB, 0.61475E+00_JPRB, 0.57049E+00_JPRB, 0.55716E+00_JPRB ,&
&  0.53991E+00_JPRB, 0.56400E+00_JPRB, 0.59373E+00_JPRB, 0.61147E+00_JPRB, 0.59537E+00_JPRB, 0.66583E+00_JPRB, 0.80441E+00_JPRB ,&
&  0.86386E+00_JPRB, 0.87172E+00_JPRB, 0.90814E+00_JPRB, 0.91666E+00_JPRB, 0.95098E+00_JPRB, 0.10374E+01_JPRB, 0.10694E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.95189E+00_JPRB, 0.89185E+00_JPRB, 0.84730E+00_JPRB, 0.80800E+00_JPRB, 0.76546E+00_JPRB, 0.73914E+00_JPRB ,&
&  0.73051E+00_JPRB, 0.73344E+00_JPRB, 0.78376E+00_JPRB, 0.86854E+00_JPRB, 0.93899E+00_JPRB, 0.12931E+01_JPRB, 0.20277E+01_JPRB ,&
&  0.23119E+01_JPRB, 0.24328E+01_JPRB, 0.27543E+01_JPRB, 0.31462E+01_JPRB, 0.31170E+01_JPRB, 0.30706E+01_JPRB, 0.29844E+01_JPRB ,&
&  0.29291E+01_JPRB, 0.28418E+01_JPRB, 0.26969E+01_JPRB, 0.26379E+01_JPRB, 0.24766E+01_JPRB, 0.24250E+01_JPRB, 0.22527E+01_JPRB ,&
&  0.19765E+01_JPRB, 0.18715E+01_JPRB, 0.18723E+01_JPRB, 0.18154E+01_JPRB, 0.17284E+01_JPRB, 0.16931E+01_JPRB, 0.16224E+01_JPRB ,&
&  0.15048E+01_JPRB, 0.12603E+01_JPRB, 0.92180E+00_JPRB, 0.82003E+00_JPRB, 0.63060E+00_JPRB, 0.62939E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.72156E+00_JPRB, 0.64596E+00_JPRB, 0.58726E+00_JPRB, 0.55733E+00_JPRB, 0.54962E+00_JPRB, 0.58291E+00_JPRB, 0.53827E+00_JPRB ,&
&  0.48985E+00_JPRB, 0.49552E+00_JPRB, 0.49969E+00_JPRB, 0.50594E+00_JPRB, 0.51223E+00_JPRB, 0.50712E+00_JPRB, 0.48789E+00_JPRB ,&
&  0.49176E+00_JPRB, 0.48126E+00_JPRB, 0.47170E+00_JPRB, 0.47650E+00_JPRB, 0.49520E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.50131E+00_JPRB, 0.52502E+00_JPRB, 0.54274E+00_JPRB, 0.55704E+00_JPRB, 0.57935E+00_JPRB, 0.61276E+00_JPRB, 0.62713E+00_JPRB ,&
&  0.62802E+00_JPRB, 0.62608E+00_JPRB, 0.62194E+00_JPRB, 0.61147E+00_JPRB, 0.59086E+00_JPRB, 0.56395E+00_JPRB, 0.50434E+00_JPRB ,&
&  0.43522E+00_JPRB, 0.48904E+00_JPRB, 0.47420E+00_JPRB, 0.48941E+00_JPRB, 0.49839E+00_JPRB, 0.49029E+00_JPRB, 0.47946E+00_JPRB ,&
&  0.49658E+00_JPRB, 0.51250E+00_JPRB, 0.54941E+00_JPRB, 0.59403E+00_JPRB, 0.59503E+00_JPRB, 0.62395E+00_JPRB, 0.71959E+00_JPRB ,&
&  0.73795E+00_JPRB, 0.74263E+00_JPRB, 0.71501E+00_JPRB, 0.70606E+00_JPRB, 0.74196E+00_JPRB, 0.80338E+00_JPRB, 0.85098E+00_JPRB ,&
&  0.84650E+00_JPRB, 0.80104E+00_JPRB, 0.75792E+00_JPRB, 0.73084E+00_JPRB, 0.71073E+00_JPRB, 0.68826E+00_JPRB, 0.66625E+00_JPRB ,&
&  0.64352E+00_JPRB, 0.63693E+00_JPRB, 0.66189E+00_JPRB, 0.70977E+00_JPRB, 0.79471E+00_JPRB, 0.11001E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.14952E+01_JPRB, 0.16556E+01_JPRB, 0.18931E+01_JPRB, 0.21323E+01_JPRB, 0.22882E+01_JPRB, 0.24012E+01_JPRB, 0.24870E+01_JPRB ,&
&  0.25952E+01_JPRB, 0.25341E+01_JPRB, 0.26640E+01_JPRB, 0.25959E+01_JPRB, 0.24216E+01_JPRB, 0.23023E+01_JPRB, 0.20974E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.17614E+01_JPRB, 0.17978E+01_JPRB, 0.17790E+01_JPRB, 0.17330E+01_JPRB, 0.16985E+01_JPRB, 0.16093E+01_JPRB ,&
&  0.15347E+01_JPRB, 0.14455E+01_JPRB, 0.11995E+01_JPRB, 0.99630E+00_JPRB, 0.81106E+00_JPRB, 0.80031E+00_JPRB, 0.72399E+00_JPRB ,&
&  0.63290E+00_JPRB, 0.53709E+00_JPRB, 0.53032E+00_JPRB, 0.52152E+00_JPRB, 0.52333E+00_JPRB, 0.49964E+00_JPRB, 0.47478E+00_JPRB ,&
&  0.47226E+00_JPRB, 0.47687E+00_JPRB, 0.46439E+00_JPRB, 0.46217E+00_JPRB, 0.47459E+00_JPRB, 0.46659E+00_JPRB, 0.44167E+00_JPRB ,&
&  0.43822E+00_JPRB, 0.44167E+00_JPRB, 0.43683E+00_JPRB, 0.44751E+00_JPRB, 0.47088E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.48167E+00_JPRB, 0.50294E+00_JPRB, 0.52053E+00_JPRB, 0.53612E+00_JPRB, 0.55509E+00_JPRB, 0.57521E+00_JPRB, 0.57315E+00_JPRB ,&
&  0.58725E+00_JPRB, 0.59167E+00_JPRB, 0.58861E+00_JPRB, 0.57708E+00_JPRB, 0.55400E+00_JPRB, 0.48624E+00_JPRB, 0.40073E+00_JPRB ,&
&  0.38310E+00_JPRB, 0.40533E+00_JPRB, 0.43308E+00_JPRB, 0.44537E+00_JPRB, 0.45235E+00_JPRB, 0.45834E+00_JPRB, 0.45688E+00_JPRB ,&
&  0.46217E+00_JPRB, 0.47481E+00_JPRB, 0.50466E+00_JPRB, 0.55153E+00_JPRB, 0.59293E+00_JPRB, 0.60839E+00_JPRB, 0.64568E+00_JPRB ,&
&  0.64107E+00_JPRB, 0.65466E+00_JPRB, 0.64969E+00_JPRB, 0.59479E+00_JPRB, 0.61626E+00_JPRB, 0.62154E+00_JPRB, 0.64708E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.65856E+00_JPRB, 0.63375E+00_JPRB, 0.61927E+00_JPRB, 0.61102E+00_JPRB, 0.59938E+00_JPRB, 0.58808E+00_JPRB ,&
&  0.57287E+00_JPRB, 0.56525E+00_JPRB, 0.58179E+00_JPRB, 0.60910E+00_JPRB, 0.65419E+00_JPRB, 0.86524E+00_JPRB, 0.98860E+00_JPRB ,&
&  0.10925E+01_JPRB, 0.12847E+01_JPRB, 0.15841E+01_JPRB, 0.16865E+01_JPRB, 0.19547E+01_JPRB, 0.21136E+01_JPRB, 0.22872E+01_JPRB ,&
&  0.24062E+01_JPRB, 0.22866E+01_JPRB, 0.23650E+01_JPRB, 0.23125E+01_JPRB, 0.22337E+01_JPRB, 0.20945E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.17858E+01_JPRB, 0.17877E+01_JPRB, 0.17985E+01_JPRB, 0.17237E+01_JPRB, 0.15873E+01_JPRB, 0.15005E+01_JPRB, 0.14145E+01_JPRB ,&
&  0.13634E+01_JPRB, 0.13076E+01_JPRB, 0.11963E+01_JPRB, 0.10639E+01_JPRB, 0.92216E+00_JPRB, 0.80449E+00_JPRB, 0.70961E+00_JPRB ,&
&  0.63200E+00_JPRB, 0.55360E+00_JPRB, 0.52255E+00_JPRB, 0.51424E+00_JPRB, 0.50643E+00_JPRB, 0.47393E+00_JPRB, 0.46043E+00_JPRB ,&
&  0.45541E+00_JPRB, 0.44730E+00_JPRB, 0.45022E+00_JPRB, 0.44450E+00_JPRB, 0.45146E+00_JPRB, 0.45280E+00_JPRB, 0.43559E+00_JPRB ,&
&  0.42340E+00_JPRB, 0.42946E+00_JPRB, 0.43251E+00_JPRB, 0.44142E+00_JPRB, 0.45970E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.46948E+00_JPRB, 0.49279E+00_JPRB, 0.50430E+00_JPRB, 0.51546E+00_JPRB, 0.52762E+00_JPRB, 0.52273E+00_JPRB, 0.51997E+00_JPRB ,&
&  0.54525E+00_JPRB, 0.51167E+00_JPRB, 0.54897E+00_JPRB, 0.43906E+00_JPRB, 0.39678E+00_JPRB, 0.38903E+00_JPRB, 0.37996E+00_JPRB ,&
&  0.38390E+00_JPRB, 0.39834E+00_JPRB, 0.41254E+00_JPRB, 0.42184E+00_JPRB, 0.43785E+00_JPRB, 0.44089E+00_JPRB, 0.44176E+00_JPRB ,&
&  0.44567E+00_JPRB, 0.45467E+00_JPRB, 0.48035E+00_JPRB, 0.49546E+00_JPRB, 0.51252E+00_JPRB, 0.52825E+00_JPRB, 0.54063E+00_JPRB ,&
&  0.53446E+00_JPRB, 0.55051E+00_JPRB, 0.58086E+00_JPRB, 0.56542E+00_JPRB, 0.54640E+00_JPRB, 0.53112E+00_JPRB, 0.53831E+00_JPRB ,&
&  0.53666E+00_JPRB, 0.48899E+00_JPRB, 0.50257E+00_JPRB, 0.50370E+00_JPRB, 0.50511E+00_JPRB, 0.50655E+00_JPRB, 0.50810E+00_JPRB ,&
&  0.50955E+00_JPRB, 0.51238E+00_JPRB, 0.52902E+00_JPRB, 0.54941E+00_JPRB, 0.59053E+00_JPRB, 0.67353E+00_JPRB, 0.73295E+00_JPRB ,&
&  0.87783E+00_JPRB, 0.88739E+00_JPRB, 0.12030E+01_JPRB, 0.14916E+01_JPRB, 0.18917E+01_JPRB, 0.20275E+01_JPRB, 0.21788E+01_JPRB ,&
&  0.23172E+01_JPRB, 0.21983E+01_JPRB, 0.21833E+01_JPRB, 0.20654E+01_JPRB, 0.19401E+01_JPRB, 0.18174E+01_JPRB, 0.17061E+01_JPRB ,&
&  0.16293E+01_JPRB, 0.15280E+01_JPRB, 0.15576E+01_JPRB, 0.14664E+01_JPRB, 0.13402E+01_JPRB, 0.12560E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.10505E+01_JPRB, 0.96251E+00_JPRB, 0.81234E+00_JPRB, 0.74212E+00_JPRB, 0.68881E+00_JPRB, 0.62111E+00_JPRB ,&
&  0.58928E+00_JPRB, 0.55692E+00_JPRB, 0.52946E+00_JPRB, 0.52505E+00_JPRB, 0.50271E+00_JPRB, 0.47748E+00_JPRB, 0.46701E+00_JPRB ,&
&  0.44527E+00_JPRB, 0.42704E+00_JPRB, 0.42718E+00_JPRB, 0.42590E+00_JPRB, 0.42403E+00_JPRB, 0.42427E+00_JPRB, 0.43535E+00_JPRB ,&
&  0.42786E+00_JPRB, 0.42387E+00_JPRB, 0.43187E+00_JPRB, 0.43427E+00_JPRB, 0.44769E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.44398E+00_JPRB, 0.47380E+00_JPRB, 0.48745E+00_JPRB, 0.49436E+00_JPRB, 0.50025E+00_JPRB, 0.49593E+00_JPRB, 0.46759E+00_JPRB ,&
&  0.46519E+00_JPRB, 0.41970E+00_JPRB, 0.39892E+00_JPRB, 0.38394E+00_JPRB, 0.37829E+00_JPRB, 0.37260E+00_JPRB, 0.36420E+00_JPRB ,&
&  0.36694E+00_JPRB, 0.37871E+00_JPRB, 0.39574E+00_JPRB, 0.40765E+00_JPRB, 0.41818E+00_JPRB, 0.42512E+00_JPRB, 0.43070E+00_JPRB ,&
&  0.43785E+00_JPRB, 0.44644E+00_JPRB, 0.45841E+00_JPRB, 0.46831E+00_JPRB, 0.47208E+00_JPRB, 0.47364E+00_JPRB, 0.47200E+00_JPRB ,&
&  0.47868E+00_JPRB, 0.49276E+00_JPRB, 0.51573E+00_JPRB, 0.52676E+00_JPRB, 0.52205E+00_JPRB, 0.50456E+00_JPRB, 0.47439E+00_JPRB ,&
&  0.40472E+00_JPRB, 0.40053E+00_JPRB, 0.42373E+00_JPRB, 0.43378E+00_JPRB, 0.44399E+00_JPRB, 0.45356E+00_JPRB, 0.46127E+00_JPRB ,&
&  0.46803E+00_JPRB, 0.47793E+00_JPRB, 0.49828E+00_JPRB, 0.52103E+00_JPRB, 0.55906E+00_JPRB, 0.61844E+00_JPRB, 0.66367E+00_JPRB ,&
&  0.73900E+00_JPRB, 0.74760E+00_JPRB, 0.88585E+00_JPRB, 0.11805E+01_JPRB, 0.17890E+01_JPRB, 0.18575E+01_JPRB, 0.19135E+01_JPRB ,&
&  0.19566E+01_JPRB, 0.19229E+01_JPRB, 0.19124E+01_JPRB, 0.18534E+01_JPRB, 0.17392E+01_JPRB, 0.16206E+01_JPRB, 0.15260E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.13532E+01_JPRB, 0.12778E+01_JPRB, 0.12293E+01_JPRB, 0.11608E+01_JPRB, 0.11078E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.96177E+00_JPRB, 0.88168E+00_JPRB, 0.78807E+00_JPRB, 0.70065E+00_JPRB, 0.63569E+00_JPRB, 0.60286E+00_JPRB, 0.57536E+00_JPRB ,&
&  0.55725E+00_JPRB, 0.54670E+00_JPRB, 0.53212E+00_JPRB, 0.52057E+00_JPRB, 0.49746E+00_JPRB, 0.46726E+00_JPRB, 0.45551E+00_JPRB ,&
&  0.42775E+00_JPRB, 0.41259E+00_JPRB, 0.39971E+00_JPRB, 0.39397E+00_JPRB, 0.39591E+00_JPRB, 0.39822E+00_JPRB, 0.39917E+00_JPRB ,&
&  0.40763E+00_JPRB, 0.40320E+00_JPRB, 0.39941E+00_JPRB, 0.40470E+00_JPRB, 0.41778E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.41167E+00_JPRB, 0.43333E+00_JPRB, 0.45233E+00_JPRB, 0.46544E+00_JPRB, 0.46651E+00_JPRB, 0.46352E+00_JPRB, 0.44876E+00_JPRB ,&
&  0.44781E+00_JPRB, 0.42580E+00_JPRB, 0.41026E+00_JPRB, 0.39352E+00_JPRB, 0.38111E+00_JPRB, 0.37290E+00_JPRB, 0.37378E+00_JPRB ,&
&  0.37946E+00_JPRB, 0.38673E+00_JPRB, 0.39481E+00_JPRB, 0.40081E+00_JPRB, 0.40843E+00_JPRB, 0.41718E+00_JPRB, 0.42777E+00_JPRB ,&
&  0.43769E+00_JPRB, 0.44333E+00_JPRB, 0.44873E+00_JPRB, 0.45333E+00_JPRB, 0.45787E+00_JPRB, 0.46017E+00_JPRB, 0.45343E+00_JPRB ,&
&  0.45568E+00_JPRB, 0.46085E+00_JPRB, 0.47996E+00_JPRB, 0.49707E+00_JPRB, 0.50487E+00_JPRB, 0.48489E+00_JPRB, 0.41754E+00_JPRB ,&
&  0.37413E+00_JPRB, 0.35726E+00_JPRB, 0.36103E+00_JPRB, 0.38728E+00_JPRB, 0.40017E+00_JPRB, 0.41213E+00_JPRB, 0.42348E+00_JPRB ,&
&  0.42315E+00_JPRB, 0.42738E+00_JPRB, 0.44873E+00_JPRB, 0.48052E+00_JPRB, 0.51197E+00_JPRB, 0.56864E+00_JPRB, 0.62145E+00_JPRB ,&
&  0.70190E+00_JPRB, 0.82502E+00_JPRB, 0.10485E+01_JPRB, 0.11088E+01_JPRB, 0.12560E+01_JPRB, 0.16201E+01_JPRB, 0.15952E+01_JPRB ,&
&  0.15951E+01_JPRB, 0.16133E+01_JPRB, 0.16622E+01_JPRB, 0.16692E+01_JPRB, 0.15528E+01_JPRB, 0.14358E+01_JPRB, 0.13157E+01_JPRB ,&
&  0.11747E+01_JPRB, 0.10952E+01_JPRB, 0.10274E+01_JPRB, 0.10150E+01_JPRB, 0.97945E+00_JPRB, 0.94132E+00_JPRB, 0.90621E+00_JPRB ,&
&  0.84165E+00_JPRB, 0.76180E+00_JPRB, 0.68283E+00_JPRB, 0.61416E+00_JPRB, 0.57952E+00_JPRB, 0.55939E+00_JPRB, 0.54632E+00_JPRB ,&
&  0.53267E+00_JPRB, 0.52192E+00_JPRB, 0.51066E+00_JPRB, 0.49496E+00_JPRB, 0.46296E+00_JPRB, 0.42530E+00_JPRB, 0.40766E+00_JPRB ,&
&  0.39682E+00_JPRB, 0.38971E+00_JPRB, 0.38385E+00_JPRB, 0.37973E+00_JPRB, 0.37901E+00_JPRB, 0.37681E+00_JPRB, 0.37055E+00_JPRB ,&
&  0.37123E+00_JPRB, 0.36941E+00_JPRB, 0.36576E+00_JPRB, 0.37109E+00_JPRB, 0.38957E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.38980E+00_JPRB, 0.39844E+00_JPRB, 0.40998E+00_JPRB, 0.42402E+00_JPRB, 0.43026E+00_JPRB, 0.42147E+00_JPRB, 0.40426E+00_JPRB ,&
&  0.39376E+00_JPRB, 0.38912E+00_JPRB, 0.39139E+00_JPRB, 0.39524E+00_JPRB, 0.39731E+00_JPRB, 0.39785E+00_JPRB, 0.39738E+00_JPRB ,&
&  0.39518E+00_JPRB, 0.39538E+00_JPRB, 0.39964E+00_JPRB, 0.40693E+00_JPRB, 0.41568E+00_JPRB, 0.42408E+00_JPRB, 0.43291E+00_JPRB ,&
&  0.43902E+00_JPRB, 0.44088E+00_JPRB, 0.44147E+00_JPRB, 0.43984E+00_JPRB, 0.44070E+00_JPRB, 0.44367E+00_JPRB, 0.44282E+00_JPRB ,&
&  0.43879E+00_JPRB, 0.43949E+00_JPRB, 0.44988E+00_JPRB, 0.46274E+00_JPRB, 0.47713E+00_JPRB, 0.46113E+00_JPRB, 0.41231E+00_JPRB ,&
&  0.36866E+00_JPRB, 0.33692E+00_JPRB, 0.32651E+00_JPRB, 0.32988E+00_JPRB, 0.34405E+00_JPRB, 0.36655E+00_JPRB, 0.38317E+00_JPRB ,&
&  0.39692E+00_JPRB, 0.41143E+00_JPRB, 0.42548E+00_JPRB, 0.43712E+00_JPRB, 0.45630E+00_JPRB, 0.48783E+00_JPRB, 0.52959E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.69521E+00_JPRB, 0.82387E+00_JPRB, 0.84312E+00_JPRB, 0.89661E+00_JPRB, 0.10217E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12194E+01_JPRB, 0.12375E+01_JPRB, 0.12681E+01_JPRB, 0.12581E+01_JPRB, 0.12179E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.93892E+00_JPRB, 0.87243E+00_JPRB, 0.83479E+00_JPRB, 0.82345E+00_JPRB, 0.81168E+00_JPRB, 0.78889E+00_JPRB ,&
&  0.74572E+00_JPRB, 0.69044E+00_JPRB, 0.62918E+00_JPRB, 0.57002E+00_JPRB, 0.55016E+00_JPRB, 0.54760E+00_JPRB, 0.54611E+00_JPRB ,&
&  0.53199E+00_JPRB, 0.51439E+00_JPRB, 0.49536E+00_JPRB, 0.47498E+00_JPRB, 0.44718E+00_JPRB, 0.41908E+00_JPRB, 0.40254E+00_JPRB ,&
&  0.39610E+00_JPRB, 0.39351E+00_JPRB, 0.39478E+00_JPRB, 0.38944E+00_JPRB, 0.38586E+00_JPRB, 0.38515E+00_JPRB, 0.37802E+00_JPRB ,&
&  0.37205E+00_JPRB, 0.36863E+00_JPRB, 0.36450E+00_JPRB, 0.36849E+00_JPRB, 0.37682E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.39058E+00_JPRB, 0.39795E+00_JPRB, 0.40387E+00_JPRB, 0.40681E+00_JPRB, 0.40364E+00_JPRB, 0.40186E+00_JPRB, 0.39526E+00_JPRB ,&
&  0.39176E+00_JPRB, 0.39273E+00_JPRB, 0.39102E+00_JPRB, 0.39139E+00_JPRB, 0.39220E+00_JPRB, 0.39327E+00_JPRB, 0.39527E+00_JPRB ,&
&  0.39871E+00_JPRB, 0.40349E+00_JPRB, 0.40914E+00_JPRB, 0.41513E+00_JPRB, 0.42135E+00_JPRB, 0.42780E+00_JPRB, 0.43376E+00_JPRB ,&
&  0.43704E+00_JPRB, 0.43663E+00_JPRB, 0.43808E+00_JPRB, 0.43736E+00_JPRB, 0.43341E+00_JPRB, 0.42710E+00_JPRB, 0.42039E+00_JPRB ,&
&  0.42082E+00_JPRB, 0.42832E+00_JPRB, 0.43140E+00_JPRB, 0.43679E+00_JPRB, 0.44630E+00_JPRB, 0.43077E+00_JPRB, 0.39326E+00_JPRB ,&
&  0.35306E+00_JPRB, 0.32036E+00_JPRB, 0.30592E+00_JPRB, 0.30112E+00_JPRB, 0.30714E+00_JPRB, 0.31967E+00_JPRB, 0.33531E+00_JPRB ,&
&  0.35131E+00_JPRB, 0.37194E+00_JPRB, 0.39225E+00_JPRB, 0.40615E+00_JPRB, 0.41908E+00_JPRB, 0.43467E+00_JPRB, 0.44785E+00_JPRB ,&
&  0.46696E+00_JPRB, 0.49991E+00_JPRB, 0.55950E+00_JPRB, 0.63496E+00_JPRB, 0.70668E+00_JPRB, 0.74618E+00_JPRB, 0.84439E+00_JPRB ,&
&  0.10284E+01_JPRB, 0.97388E+00_JPRB, 0.94721E+00_JPRB, 0.91510E+00_JPRB, 0.87789E+00_JPRB, 0.84163E+00_JPRB, 0.81641E+00_JPRB ,&
&  0.80338E+00_JPRB, 0.79122E+00_JPRB, 0.76433E+00_JPRB, 0.72835E+00_JPRB, 0.70117E+00_JPRB, 0.68712E+00_JPRB, 0.68751E+00_JPRB ,&
&  0.68380E+00_JPRB, 0.70414E+00_JPRB, 0.80950E+00_JPRB, 0.69858E+00_JPRB, 0.63170E+00_JPRB, 0.59874E+00_JPRB, 0.56703E+00_JPRB ,&
&  0.53938E+00_JPRB, 0.51761E+00_JPRB, 0.49736E+00_JPRB, 0.47505E+00_JPRB, 0.45470E+00_JPRB, 0.43502E+00_JPRB, 0.42279E+00_JPRB ,&
&  0.41746E+00_JPRB, 0.41459E+00_JPRB, 0.41500E+00_JPRB, 0.41290E+00_JPRB, 0.40492E+00_JPRB, 0.40030E+00_JPRB, 0.39711E+00_JPRB ,&
&  0.39052E+00_JPRB, 0.38376E+00_JPRB, 0.38088E+00_JPRB, 0.38330E+00_JPRB, 0.38551E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.40315E+00_JPRB, 0.40656E+00_JPRB, 0.40826E+00_JPRB, 0.41116E+00_JPRB, 0.40915E+00_JPRB, 0.40832E+00_JPRB, 0.40299E+00_JPRB ,&
&  0.39875E+00_JPRB, 0.39607E+00_JPRB, 0.39467E+00_JPRB, 0.39567E+00_JPRB, 0.39825E+00_JPRB, 0.40226E+00_JPRB, 0.40690E+00_JPRB ,&
&  0.41217E+00_JPRB, 0.41277E+00_JPRB, 0.41638E+00_JPRB, 0.42235E+00_JPRB, 0.42574E+00_JPRB, 0.43020E+00_JPRB, 0.43298E+00_JPRB ,&
&  0.43348E+00_JPRB, 0.43044E+00_JPRB, 0.42850E+00_JPRB, 0.42362E+00_JPRB, 0.41949E+00_JPRB, 0.41619E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.41153E+00_JPRB, 0.40876E+00_JPRB, 0.42117E+00_JPRB, 0.42752E+00_JPRB, 0.40904E+00_JPRB, 0.39248E+00_JPRB, 0.36330E+00_JPRB ,&
&  0.33468E+00_JPRB, 0.30753E+00_JPRB, 0.29779E+00_JPRB, 0.29245E+00_JPRB, 0.29581E+00_JPRB, 0.30209E+00_JPRB, 0.31441E+00_JPRB ,&
&  0.32697E+00_JPRB, 0.34475E+00_JPRB, 0.36047E+00_JPRB, 0.37765E+00_JPRB, 0.38953E+00_JPRB, 0.40334E+00_JPRB, 0.41349E+00_JPRB ,&
&  0.42661E+00_JPRB, 0.44466E+00_JPRB, 0.47176E+00_JPRB, 0.50601E+00_JPRB, 0.53727E+00_JPRB, 0.57346E+00_JPRB, 0.62314E+00_JPRB ,&
&  0.70129E+00_JPRB, 0.70164E+00_JPRB, 0.71235E+00_JPRB, 0.70333E+00_JPRB, 0.68836E+00_JPRB, 0.67703E+00_JPRB, 0.66079E+00_JPRB ,&
&  0.65760E+00_JPRB, 0.66806E+00_JPRB, 0.65855E+00_JPRB, 0.64655E+00_JPRB, 0.63262E+00_JPRB, 0.62445E+00_JPRB, 0.62626E+00_JPRB ,&
&  0.62556E+00_JPRB, 0.63294E+00_JPRB, 0.67924E+00_JPRB, 0.65609E+00_JPRB, 0.62980E+00_JPRB, 0.60149E+00_JPRB, 0.57193E+00_JPRB ,&
&  0.54570E+00_JPRB, 0.52313E+00_JPRB, 0.50105E+00_JPRB, 0.48087E+00_JPRB, 0.46636E+00_JPRB, 0.45239E+00_JPRB, 0.44504E+00_JPRB ,&
&  0.43822E+00_JPRB, 0.43293E+00_JPRB, 0.42954E+00_JPRB, 0.42486E+00_JPRB, 0.41433E+00_JPRB, 0.40502E+00_JPRB, 0.40002E+00_JPRB ,&
&  0.39471E+00_JPRB, 0.39387E+00_JPRB, 0.39439E+00_JPRB, 0.39761E+00_JPRB, 0.39997E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.41973E+00_JPRB, 0.42375E+00_JPRB, 0.42700E+00_JPRB, 0.42823E+00_JPRB, 0.42698E+00_JPRB, 0.42554E+00_JPRB, 0.42226E+00_JPRB ,&
&  0.41997E+00_JPRB, 0.41779E+00_JPRB, 0.41655E+00_JPRB, 0.41680E+00_JPRB, 0.41859E+00_JPRB, 0.42090E+00_JPRB, 0.42280E+00_JPRB ,&
&  0.42395E+00_JPRB, 0.42572E+00_JPRB, 0.42611E+00_JPRB, 0.42749E+00_JPRB, 0.42834E+00_JPRB, 0.43118E+00_JPRB, 0.43196E+00_JPRB ,&
&  0.43316E+00_JPRB, 0.42951E+00_JPRB, 0.42037E+00_JPRB, 0.41566E+00_JPRB, 0.41597E+00_JPRB, 0.40651E+00_JPRB, 0.40908E+00_JPRB ,&
&  0.41308E+00_JPRB, 0.41354E+00_JPRB, 0.40891E+00_JPRB, 0.40655E+00_JPRB, 0.39481E+00_JPRB, 0.34041E+00_JPRB, 0.33761E+00_JPRB ,&
&  0.32554E+00_JPRB, 0.30888E+00_JPRB, 0.30069E+00_JPRB, 0.29443E+00_JPRB, 0.29280E+00_JPRB, 0.29979E+00_JPRB, 0.31075E+00_JPRB ,&
&  0.31779E+00_JPRB, 0.34180E+00_JPRB, 0.35213E+00_JPRB, 0.36462E+00_JPRB, 0.37525E+00_JPRB, 0.38966E+00_JPRB, 0.39888E+00_JPRB ,&
&  0.41090E+00_JPRB, 0.42337E+00_JPRB, 0.44386E+00_JPRB, 0.47068E+00_JPRB, 0.49682E+00_JPRB, 0.52177E+00_JPRB, 0.54578E+00_JPRB ,&
&  0.59331E+00_JPRB, 0.60083E+00_JPRB, 0.61261E+00_JPRB, 0.61597E+00_JPRB, 0.60656E+00_JPRB, 0.59880E+00_JPRB, 0.58768E+00_JPRB ,&
&  0.58728E+00_JPRB, 0.59135E+00_JPRB, 0.59668E+00_JPRB, 0.59295E+00_JPRB, 0.58764E+00_JPRB, 0.58423E+00_JPRB, 0.58411E+00_JPRB ,&
&  0.58610E+00_JPRB, 0.58747E+00_JPRB, 0.58744E+00_JPRB, 0.57656E+00_JPRB, 0.57096E+00_JPRB, 0.55584E+00_JPRB, 0.54353E+00_JPRB ,&
&  0.52739E+00_JPRB, 0.51708E+00_JPRB, 0.49810E+00_JPRB, 0.48587E+00_JPRB, 0.47448E+00_JPRB, 0.46599E+00_JPRB, 0.45673E+00_JPRB ,&
&  0.44692E+00_JPRB, 0.43835E+00_JPRB, 0.43179E+00_JPRB, 0.42556E+00_JPRB, 0.41701E+00_JPRB, 0.40854E+00_JPRB, 0.40567E+00_JPRB ,&
&  0.40511E+00_JPRB, 0.40580E+00_JPRB, 0.40731E+00_JPRB, 0.41113E+00_JPRB, 0.41446E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.42352E+00_JPRB, 0.42662E+00_JPRB, 0.43057E+00_JPRB, 0.43336E+00_JPRB, 0.43505E+00_JPRB, 0.43615E+00_JPRB, 0.43621E+00_JPRB ,&
&  0.43532E+00_JPRB, 0.43364E+00_JPRB, 0.43240E+00_JPRB, 0.43093E+00_JPRB, 0.43120E+00_JPRB, 0.43136E+00_JPRB, 0.43253E+00_JPRB ,&
&  0.43254E+00_JPRB, 0.43235E+00_JPRB, 0.43123E+00_JPRB, 0.43026E+00_JPRB, 0.42940E+00_JPRB, 0.42950E+00_JPRB, 0.42836E+00_JPRB ,&
&  0.42842E+00_JPRB, 0.42499E+00_JPRB, 0.41427E+00_JPRB, 0.41269E+00_JPRB, 0.40846E+00_JPRB, 0.40555E+00_JPRB, 0.40139E+00_JPRB ,&
&  0.39499E+00_JPRB, 0.36960E+00_JPRB, 0.37863E+00_JPRB, 0.36736E+00_JPRB, 0.35585E+00_JPRB, 0.34550E+00_JPRB, 0.33354E+00_JPRB ,&
&  0.32656E+00_JPRB, 0.31753E+00_JPRB, 0.31106E+00_JPRB, 0.30894E+00_JPRB, 0.30948E+00_JPRB, 0.31559E+00_JPRB, 0.32370E+00_JPRB ,&
&  0.32971E+00_JPRB, 0.34603E+00_JPRB, 0.35516E+00_JPRB, 0.36568E+00_JPRB, 0.37722E+00_JPRB, 0.39094E+00_JPRB, 0.39849E+00_JPRB ,&
&  0.41057E+00_JPRB, 0.41655E+00_JPRB, 0.43156E+00_JPRB, 0.44798E+00_JPRB, 0.47264E+00_JPRB, 0.49447E+00_JPRB, 0.51637E+00_JPRB ,&
&  0.54567E+00_JPRB, 0.56144E+00_JPRB, 0.57272E+00_JPRB, 0.57662E+00_JPRB, 0.57326E+00_JPRB, 0.56670E+00_JPRB, 0.55803E+00_JPRB ,&
&  0.55137E+00_JPRB, 0.54664E+00_JPRB, 0.54574E+00_JPRB, 0.54555E+00_JPRB, 0.54252E+00_JPRB, 0.54260E+00_JPRB, 0.54373E+00_JPRB ,&
&  0.54516E+00_JPRB, 0.54310E+00_JPRB, 0.54529E+00_JPRB, 0.53741E+00_JPRB, 0.53272E+00_JPRB, 0.51997E+00_JPRB, 0.51019E+00_JPRB ,&
&  0.50255E+00_JPRB, 0.49286E+00_JPRB, 0.48227E+00_JPRB, 0.47250E+00_JPRB, 0.46452E+00_JPRB, 0.45854E+00_JPRB, 0.45425E+00_JPRB ,&
&  0.44979E+00_JPRB, 0.44475E+00_JPRB, 0.44059E+00_JPRB, 0.43253E+00_JPRB, 0.42322E+00_JPRB, 0.41677E+00_JPRB, 0.41449E+00_JPRB ,&
&  0.41398E+00_JPRB, 0.41519E+00_JPRB, 0.41656E+00_JPRB, 0.42017E+00_JPRB, 0.42206E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.43648E+00_JPRB, 0.43548E+00_JPRB, 0.43458E+00_JPRB, 0.43318E+00_JPRB, 0.43218E+00_JPRB, 0.43118E+00_JPRB, 0.43213E+00_JPRB ,&
&  0.43300E+00_JPRB, 0.43395E+00_JPRB, 0.43397E+00_JPRB, 0.43318E+00_JPRB, 0.43302E+00_JPRB, 0.43172E+00_JPRB, 0.43151E+00_JPRB ,&
&  0.43143E+00_JPRB, 0.43134E+00_JPRB, 0.43136E+00_JPRB, 0.43115E+00_JPRB, 0.43040E+00_JPRB, 0.43002E+00_JPRB, 0.42605E+00_JPRB ,&
&  0.42321E+00_JPRB, 0.42068E+00_JPRB, 0.40548E+00_JPRB, 0.40738E+00_JPRB, 0.40227E+00_JPRB, 0.39879E+00_JPRB, 0.39617E+00_JPRB ,&
&  0.38513E+00_JPRB, 0.38017E+00_JPRB, 0.36824E+00_JPRB, 0.35782E+00_JPRB, 0.35655E+00_JPRB, 0.34752E+00_JPRB, 0.34670E+00_JPRB ,&
&  0.34392E+00_JPRB, 0.34101E+00_JPRB, 0.34022E+00_JPRB, 0.34072E+00_JPRB, 0.34454E+00_JPRB, 0.34842E+00_JPRB, 0.35275E+00_JPRB ,&
&  0.35991E+00_JPRB, 0.36735E+00_JPRB, 0.37898E+00_JPRB, 0.38395E+00_JPRB, 0.39626E+00_JPRB, 0.40982E+00_JPRB, 0.42392E+00_JPRB ,&
&  0.43373E+00_JPRB, 0.43401E+00_JPRB, 0.43880E+00_JPRB, 0.44660E+00_JPRB, 0.45520E+00_JPRB, 0.46586E+00_JPRB, 0.48046E+00_JPRB ,&
&  0.49977E+00_JPRB, 0.51612E+00_JPRB, 0.52680E+00_JPRB, 0.52948E+00_JPRB, 0.52961E+00_JPRB, 0.52595E+00_JPRB, 0.52304E+00_JPRB ,&
&  0.52070E+00_JPRB, 0.51865E+00_JPRB, 0.51628E+00_JPRB, 0.51447E+00_JPRB, 0.51135E+00_JPRB, 0.51002E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.50584E+00_JPRB, 0.50272E+00_JPRB, 0.50342E+00_JPRB, 0.49591E+00_JPRB, 0.49280E+00_JPRB, 0.48927E+00_JPRB, 0.48306E+00_JPRB ,&
&  0.47637E+00_JPRB, 0.47042E+00_JPRB, 0.46390E+00_JPRB, 0.45731E+00_JPRB, 0.45071E+00_JPRB, 0.44651E+00_JPRB, 0.44413E+00_JPRB ,&
&  0.44413E+00_JPRB, 0.44478E+00_JPRB, 0.44570E+00_JPRB, 0.44167E+00_JPRB, 0.43846E+00_JPRB, 0.43481E+00_JPRB, 0.43333E+00_JPRB ,&
&  0.43204E+00_JPRB, 0.43308E+00_JPRB, 0.43362E+00_JPRB, 0.43548E+00_JPRB, 0.43676E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.43497E+00_JPRB, 0.43496E+00_JPRB, 0.43499E+00_JPRB, 0.43470E+00_JPRB, 0.43461E+00_JPRB, 0.43455E+00_JPRB, 0.43450E+00_JPRB ,&
&  0.43448E+00_JPRB, 0.43484E+00_JPRB, 0.43504E+00_JPRB, 0.43493E+00_JPRB, 0.43425E+00_JPRB, 0.43352E+00_JPRB, 0.43313E+00_JPRB ,&
&  0.43183E+00_JPRB, 0.43082E+00_JPRB, 0.43013E+00_JPRB, 0.42975E+00_JPRB, 0.42921E+00_JPRB, 0.42879E+00_JPRB, 0.42549E+00_JPRB ,&
&  0.42318E+00_JPRB, 0.42115E+00_JPRB, 0.41858E+00_JPRB, 0.41438E+00_JPRB, 0.41200E+00_JPRB, 0.39703E+00_JPRB, 0.39856E+00_JPRB ,&
&  0.40899E+00_JPRB, 0.40743E+00_JPRB, 0.40483E+00_JPRB, 0.40039E+00_JPRB, 0.38727E+00_JPRB, 0.37998E+00_JPRB, 0.37822E+00_JPRB ,&
&  0.36566E+00_JPRB, 0.37353E+00_JPRB, 0.37566E+00_JPRB, 0.37684E+00_JPRB, 0.37987E+00_JPRB, 0.37908E+00_JPRB, 0.38129E+00_JPRB ,&
&  0.39199E+00_JPRB, 0.39467E+00_JPRB, 0.40165E+00_JPRB, 0.41028E+00_JPRB, 0.41969E+00_JPRB, 0.42850E+00_JPRB, 0.43732E+00_JPRB ,&
&  0.44398E+00_JPRB, 0.44848E+00_JPRB, 0.45141E+00_JPRB, 0.45294E+00_JPRB, 0.45550E+00_JPRB, 0.45957E+00_JPRB, 0.46654E+00_JPRB ,&
&  0.47802E+00_JPRB, 0.49081E+00_JPRB, 0.50105E+00_JPRB, 0.50664E+00_JPRB, 0.51353E+00_JPRB, 0.51835E+00_JPRB, 0.52215E+00_JPRB ,&
&  0.52411E+00_JPRB, 0.52363E+00_JPRB, 0.52020E+00_JPRB, 0.51661E+00_JPRB, 0.51303E+00_JPRB, 0.50800E+00_JPRB, 0.50204E+00_JPRB ,&
&  0.49550E+00_JPRB, 0.48879E+00_JPRB, 0.48219E+00_JPRB, 0.47598E+00_JPRB, 0.47121E+00_JPRB, 0.46638E+00_JPRB, 0.46325E+00_JPRB ,&
&  0.45937E+00_JPRB, 0.45648E+00_JPRB, 0.45254E+00_JPRB, 0.44861E+00_JPRB, 0.44379E+00_JPRB, 0.44012E+00_JPRB, 0.43669E+00_JPRB ,&
&  0.43520E+00_JPRB, 0.43459E+00_JPRB, 0.43394E+00_JPRB, 0.43257E+00_JPRB, 0.43263E+00_JPRB, 0.43318E+00_JPRB, 0.43398E+00_JPRB ,&
&  0.43439E+00_JPRB, 0.43473E+00_JPRB, 0.43469E+00_JPRB, 0.43492E+00_JPRB, 0.43488E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.42619E+00_JPRB, 0.42647E+00_JPRB, 0.42655E+00_JPRB, 0.42655E+00_JPRB, 0.42672E+00_JPRB, 0.42698E+00_JPRB, 0.42717E+00_JPRB ,&
&  0.42728E+00_JPRB, 0.42760E+00_JPRB, 0.42807E+00_JPRB, 0.42841E+00_JPRB, 0.42841E+00_JPRB, 0.42857E+00_JPRB, 0.42884E+00_JPRB ,&
&  0.42887E+00_JPRB, 0.42868E+00_JPRB, 0.42808E+00_JPRB, 0.42778E+00_JPRB, 0.42755E+00_JPRB, 0.42715E+00_JPRB, 0.42746E+00_JPRB ,&
&  0.42772E+00_JPRB, 0.42739E+00_JPRB, 0.42637E+00_JPRB, 0.42275E+00_JPRB, 0.42014E+00_JPRB, 0.41772E+00_JPRB, 0.41552E+00_JPRB ,&
&  0.41297E+00_JPRB, 0.41266E+00_JPRB, 0.41264E+00_JPRB, 0.41267E+00_JPRB, 0.41445E+00_JPRB, 0.41416E+00_JPRB, 0.41363E+00_JPRB ,&
&  0.41245E+00_JPRB, 0.40956E+00_JPRB, 0.41033E+00_JPRB, 0.41134E+00_JPRB, 0.41230E+00_JPRB, 0.41467E+00_JPRB, 0.41704E+00_JPRB ,&
&  0.41983E+00_JPRB, 0.42285E+00_JPRB, 0.42750E+00_JPRB, 0.43264E+00_JPRB, 0.43749E+00_JPRB, 0.44197E+00_JPRB, 0.44582E+00_JPRB ,&
&  0.44914E+00_JPRB, 0.45267E+00_JPRB, 0.45653E+00_JPRB, 0.46053E+00_JPRB, 0.46419E+00_JPRB, 0.46867E+00_JPRB, 0.47296E+00_JPRB ,&
&  0.47676E+00_JPRB, 0.48089E+00_JPRB, 0.48738E+00_JPRB, 0.49510E+00_JPRB, 0.50339E+00_JPRB, 0.50881E+00_JPRB, 0.51281E+00_JPRB ,&
&  0.51523E+00_JPRB, 0.51629E+00_JPRB, 0.51509E+00_JPRB, 0.51277E+00_JPRB, 0.50966E+00_JPRB, 0.50456E+00_JPRB, 0.49915E+00_JPRB ,&
&  0.49369E+00_JPRB, 0.48846E+00_JPRB, 0.48305E+00_JPRB, 0.47728E+00_JPRB, 0.47191E+00_JPRB, 0.46666E+00_JPRB, 0.46223E+00_JPRB ,&
&  0.45764E+00_JPRB, 0.45373E+00_JPRB, 0.44999E+00_JPRB, 0.44676E+00_JPRB, 0.44299E+00_JPRB, 0.43962E+00_JPRB, 0.43637E+00_JPRB ,&
&  0.43396E+00_JPRB, 0.43133E+00_JPRB, 0.42873E+00_JPRB, 0.42659E+00_JPRB, 0.42570E+00_JPRB, 0.42537E+00_JPRB, 0.42526E+00_JPRB ,&
&  0.42512E+00_JPRB, 0.42530E+00_JPRB, 0.42570E+00_JPRB, 0.42588E+00_JPRB, 0.42598E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.43268E+00_JPRB, 0.43253E+00_JPRB, 0.43229E+00_JPRB, 0.43204E+00_JPRB, 0.43173E+00_JPRB, 0.43150E+00_JPRB, 0.43124E+00_JPRB ,&
&  0.43093E+00_JPRB, 0.43070E+00_JPRB, 0.43066E+00_JPRB, 0.43064E+00_JPRB, 0.43065E+00_JPRB, 0.43063E+00_JPRB, 0.43071E+00_JPRB ,&
&  0.43069E+00_JPRB, 0.43052E+00_JPRB, 0.43028E+00_JPRB, 0.43044E+00_JPRB, 0.43061E+00_JPRB, 0.43070E+00_JPRB, 0.43088E+00_JPRB ,&
&  0.43087E+00_JPRB, 0.43042E+00_JPRB, 0.42986E+00_JPRB, 0.42977E+00_JPRB, 0.42967E+00_JPRB, 0.42963E+00_JPRB, 0.42956E+00_JPRB ,&
&  0.42945E+00_JPRB, 0.42943E+00_JPRB, 0.42929E+00_JPRB, 0.42912E+00_JPRB, 0.42928E+00_JPRB, 0.42968E+00_JPRB, 0.43017E+00_JPRB ,&
&  0.43072E+00_JPRB, 0.43132E+00_JPRB, 0.43220E+00_JPRB, 0.43306E+00_JPRB, 0.43394E+00_JPRB, 0.43454E+00_JPRB, 0.43536E+00_JPRB ,&
&  0.43611E+00_JPRB, 0.43688E+00_JPRB, 0.43782E+00_JPRB, 0.43895E+00_JPRB, 0.43990E+00_JPRB, 0.44101E+00_JPRB, 0.44200E+00_JPRB ,&
&  0.44367E+00_JPRB, 0.44569E+00_JPRB, 0.44787E+00_JPRB, 0.45006E+00_JPRB, 0.45215E+00_JPRB, 0.45436E+00_JPRB, 0.45681E+00_JPRB ,&
&  0.45916E+00_JPRB, 0.46142E+00_JPRB, 0.46415E+00_JPRB, 0.46722E+00_JPRB, 0.47036E+00_JPRB, 0.47254E+00_JPRB, 0.47473E+00_JPRB ,&
&  0.47696E+00_JPRB, 0.47891E+00_JPRB, 0.47930E+00_JPRB, 0.47959E+00_JPRB, 0.47960E+00_JPRB, 0.47895E+00_JPRB, 0.47782E+00_JPRB ,&
&  0.47635E+00_JPRB, 0.47468E+00_JPRB, 0.47199E+00_JPRB, 0.46887E+00_JPRB, 0.46598E+00_JPRB, 0.46318E+00_JPRB, 0.46035E+00_JPRB ,&
&  0.45762E+00_JPRB, 0.45530E+00_JPRB, 0.45314E+00_JPRB, 0.45100E+00_JPRB, 0.44874E+00_JPRB, 0.44663E+00_JPRB, 0.44453E+00_JPRB ,&
&  0.44274E+00_JPRB, 0.44098E+00_JPRB, 0.43934E+00_JPRB, 0.43780E+00_JPRB, 0.43666E+00_JPRB, 0.43582E+00_JPRB, 0.43512E+00_JPRB ,&
&  0.43447E+00_JPRB, 0.43399E+00_JPRB, 0.43370E+00_JPRB, 0.43335E+00_JPRB, 0.43296E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB, 0.44322E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB, 0.21564E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.24221E+00_JPRB, 0.24225E+00_JPRB, 0.24230E+00_JPRB, 0.24195E+00_JPRB, 0.24049E+00_JPRB, 0.23942E+00_JPRB, 0.23849E+00_JPRB ,&
&  0.23808E+00_JPRB, 0.23730E+00_JPRB, 0.23691E+00_JPRB, 0.23679E+00_JPRB, 0.23646E+00_JPRB, 0.23597E+00_JPRB, 0.23532E+00_JPRB ,&
&  0.23449E+00_JPRB, 0.23337E+00_JPRB, 0.23230E+00_JPRB, 0.23221E+00_JPRB, 0.23246E+00_JPRB, 0.23273E+00_JPRB, 0.23295E+00_JPRB ,&
&  0.23306E+00_JPRB, 0.23312E+00_JPRB, 0.23270E+00_JPRB, 0.23274E+00_JPRB, 0.23278E+00_JPRB, 0.23278E+00_JPRB, 0.23292E+00_JPRB ,&
&  0.23318E+00_JPRB, 0.23337E+00_JPRB, 0.23311E+00_JPRB, 0.23263E+00_JPRB, 0.23171E+00_JPRB, 0.23005E+00_JPRB, 0.22745E+00_JPRB ,&
&  0.22451E+00_JPRB, 0.22155E+00_JPRB, 0.21918E+00_JPRB, 0.21701E+00_JPRB, 0.21501E+00_JPRB, 0.21318E+00_JPRB, 0.21166E+00_JPRB ,&
&  0.20987E+00_JPRB, 0.20810E+00_JPRB, 0.20648E+00_JPRB, 0.20534E+00_JPRB, 0.20412E+00_JPRB, 0.20306E+00_JPRB, 0.20212E+00_JPRB ,&
&  0.20149E+00_JPRB, 0.20093E+00_JPRB, 0.20044E+00_JPRB, 0.20020E+00_JPRB, 0.20000E+00_JPRB, 0.19981E+00_JPRB, 0.19959E+00_JPRB ,&
&  0.19940E+00_JPRB, 0.19924E+00_JPRB, 0.19911E+00_JPRB, 0.19904E+00_JPRB, 0.19909E+00_JPRB, 0.19895E+00_JPRB, 0.19878E+00_JPRB ,&
&  0.19870E+00_JPRB, 0.19892E+00_JPRB, 0.19909E+00_JPRB, 0.19939E+00_JPRB, 0.19985E+00_JPRB, 0.20052E+00_JPRB, 0.20124E+00_JPRB ,&
&  0.20197E+00_JPRB, 0.20281E+00_JPRB, 0.20374E+00_JPRB, 0.20486E+00_JPRB, 0.20606E+00_JPRB, 0.20725E+00_JPRB, 0.20840E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.21076E+00_JPRB, 0.21181E+00_JPRB, 0.21260E+00_JPRB, 0.21364E+00_JPRB, 0.21469E+00_JPRB, 0.21590E+00_JPRB ,&
&  0.21726E+00_JPRB, 0.21905E+00_JPRB, 0.22145E+00_JPRB, 0.22429E+00_JPRB, 0.22773E+00_JPRB, 0.23034E+00_JPRB, 0.23305E+00_JPRB ,&
&  0.23504E+00_JPRB, 0.23654E+00_JPRB, 0.23763E+00_JPRB, 0.23905E+00_JPRB, 0.24063E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.27752E+00_JPRB, 0.28329E+00_JPRB, 0.28744E+00_JPRB, 0.29663E+00_JPRB, 0.31113E+00_JPRB, 0.40668E+00_JPRB, 0.40166E+00_JPRB ,&
&  0.40055E+00_JPRB, 0.35783E+00_JPRB, 0.34955E+00_JPRB, 0.33729E+00_JPRB, 0.32195E+00_JPRB, 0.30294E+00_JPRB, 0.29111E+00_JPRB ,&
&  0.28126E+00_JPRB, 0.26990E+00_JPRB, 0.25916E+00_JPRB, 0.25293E+00_JPRB, 0.24798E+00_JPRB, 0.24369E+00_JPRB, 0.24135E+00_JPRB ,&
&  0.24069E+00_JPRB, 0.24143E+00_JPRB, 0.24250E+00_JPRB, 0.24424E+00_JPRB, 0.24779E+00_JPRB, 0.25160E+00_JPRB, 0.25537E+00_JPRB ,&
&  0.25745E+00_JPRB, 0.26143E+00_JPRB, 0.26205E+00_JPRB, 0.25880E+00_JPRB, 0.25338E+00_JPRB, 0.24631E+00_JPRB, 0.23918E+00_JPRB ,&
&  0.23180E+00_JPRB, 0.22487E+00_JPRB, 0.21917E+00_JPRB, 0.21398E+00_JPRB, 0.20817E+00_JPRB, 0.20282E+00_JPRB, 0.19944E+00_JPRB ,&
&  0.19645E+00_JPRB, 0.19344E+00_JPRB, 0.19110E+00_JPRB, 0.18990E+00_JPRB, 0.18855E+00_JPRB, 0.18766E+00_JPRB, 0.18701E+00_JPRB ,&
&  0.18684E+00_JPRB, 0.18674E+00_JPRB, 0.18681E+00_JPRB, 0.18710E+00_JPRB, 0.18708E+00_JPRB, 0.18701E+00_JPRB, 0.18698E+00_JPRB ,&
&  0.18751E+00_JPRB, 0.18752E+00_JPRB, 0.18741E+00_JPRB, 0.18715E+00_JPRB, 0.18769E+00_JPRB, 0.18779E+00_JPRB, 0.18806E+00_JPRB ,&
&  0.18860E+00_JPRB, 0.18939E+00_JPRB, 0.19002E+00_JPRB, 0.19075E+00_JPRB, 0.19146E+00_JPRB, 0.19229E+00_JPRB, 0.19325E+00_JPRB ,&
&  0.19422E+00_JPRB, 0.19565E+00_JPRB, 0.19711E+00_JPRB, 0.19842E+00_JPRB, 0.19958E+00_JPRB, 0.20116E+00_JPRB, 0.20295E+00_JPRB ,&
&  0.20564E+00_JPRB, 0.20828E+00_JPRB, 0.21098E+00_JPRB, 0.21275E+00_JPRB, 0.21546E+00_JPRB, 0.21772E+00_JPRB, 0.22007E+00_JPRB ,&
&  0.22274E+00_JPRB, 0.22689E+00_JPRB, 0.23272E+00_JPRB, 0.23892E+00_JPRB, 0.24560E+00_JPRB, 0.25191E+00_JPRB, 0.25613E+00_JPRB ,&
&  0.25992E+00_JPRB, 0.26397E+00_JPRB, 0.27004E+00_JPRB, 0.27331E+00_JPRB, 0.27595E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.49495E+00_JPRB, 0.49700E+00_JPRB, 0.49927E+00_JPRB, 0.50307E+00_JPRB, 0.50709E+00_JPRB, 0.51127E+00_JPRB, 0.51508E+00_JPRB ,&
&  0.49701E+00_JPRB, 0.47386E+00_JPRB, 0.45496E+00_JPRB, 0.43254E+00_JPRB, 0.41211E+00_JPRB, 0.39566E+00_JPRB, 0.37561E+00_JPRB ,&
&  0.35572E+00_JPRB, 0.33578E+00_JPRB, 0.31320E+00_JPRB, 0.29471E+00_JPRB, 0.27651E+00_JPRB, 0.26176E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.24764E+00_JPRB, 0.24644E+00_JPRB, 0.24807E+00_JPRB, 0.25162E+00_JPRB, 0.25487E+00_JPRB, 0.25976E+00_JPRB, 0.26642E+00_JPRB ,&
&  0.27560E+00_JPRB, 0.28968E+00_JPRB, 0.30242E+00_JPRB, 0.30722E+00_JPRB, 0.30404E+00_JPRB, 0.29594E+00_JPRB, 0.28900E+00_JPRB ,&
&  0.28047E+00_JPRB, 0.26818E+00_JPRB, 0.25355E+00_JPRB, 0.23913E+00_JPRB, 0.22572E+00_JPRB, 0.21594E+00_JPRB, 0.20797E+00_JPRB ,&
&  0.20131E+00_JPRB, 0.19549E+00_JPRB, 0.19109E+00_JPRB, 0.18808E+00_JPRB, 0.18551E+00_JPRB, 0.18341E+00_JPRB, 0.18146E+00_JPRB ,&
&  0.18042E+00_JPRB, 0.17931E+00_JPRB, 0.17863E+00_JPRB, 0.17825E+00_JPRB, 0.17780E+00_JPRB, 0.17768E+00_JPRB, 0.17759E+00_JPRB ,&
&  0.17728E+00_JPRB, 0.17668E+00_JPRB, 0.17633E+00_JPRB, 0.17608E+00_JPRB, 0.17643E+00_JPRB, 0.17669E+00_JPRB, 0.17714E+00_JPRB ,&
&  0.17788E+00_JPRB, 0.17952E+00_JPRB, 0.18059E+00_JPRB, 0.18158E+00_JPRB, 0.18210E+00_JPRB, 0.18250E+00_JPRB, 0.18306E+00_JPRB ,&
&  0.18400E+00_JPRB, 0.18518E+00_JPRB, 0.18626E+00_JPRB, 0.18770E+00_JPRB, 0.18918E+00_JPRB, 0.19121E+00_JPRB, 0.19336E+00_JPRB ,&
&  0.19592E+00_JPRB, 0.19910E+00_JPRB, 0.20309E+00_JPRB, 0.20679E+00_JPRB, 0.21231E+00_JPRB, 0.21785E+00_JPRB, 0.22398E+00_JPRB ,&
&  0.23018E+00_JPRB, 0.23686E+00_JPRB, 0.24545E+00_JPRB, 0.25559E+00_JPRB, 0.26806E+00_JPRB, 0.27685E+00_JPRB, 0.28875E+00_JPRB ,&
&  0.30683E+00_JPRB, 0.34511E+00_JPRB, 0.47984E+00_JPRB, 0.49325E+00_JPRB, 0.49756E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.50789E+00_JPRB, 0.51676E+00_JPRB, 0.53113E+00_JPRB, 0.54277E+00_JPRB, 0.56733E+00_JPRB, 0.58111E+00_JPRB, 0.62776E+00_JPRB ,&
&  0.58186E+00_JPRB, 0.50253E+00_JPRB, 0.47063E+00_JPRB, 0.43622E+00_JPRB, 0.39669E+00_JPRB, 0.36835E+00_JPRB, 0.34831E+00_JPRB ,&
&  0.33648E+00_JPRB, 0.32640E+00_JPRB, 0.31676E+00_JPRB, 0.30464E+00_JPRB, 0.29212E+00_JPRB, 0.27797E+00_JPRB, 0.26716E+00_JPRB ,&
&  0.26063E+00_JPRB, 0.25861E+00_JPRB, 0.25829E+00_JPRB, 0.26273E+00_JPRB, 0.26889E+00_JPRB, 0.27710E+00_JPRB, 0.28644E+00_JPRB ,&
&  0.30715E+00_JPRB, 0.31283E+00_JPRB, 0.31241E+00_JPRB, 0.31262E+00_JPRB, 0.31395E+00_JPRB, 0.31605E+00_JPRB, 0.30641E+00_JPRB ,&
&  0.31666E+00_JPRB, 0.33946E+00_JPRB, 0.34562E+00_JPRB, 0.32756E+00_JPRB, 0.27310E+00_JPRB, 0.24008E+00_JPRB, 0.21761E+00_JPRB ,&
&  0.20882E+00_JPRB, 0.19939E+00_JPRB, 0.19351E+00_JPRB, 0.18806E+00_JPRB, 0.18474E+00_JPRB, 0.18291E+00_JPRB, 0.18115E+00_JPRB ,&
&  0.17980E+00_JPRB, 0.17865E+00_JPRB, 0.17670E+00_JPRB, 0.17576E+00_JPRB, 0.17478E+00_JPRB, 0.17427E+00_JPRB, 0.17396E+00_JPRB ,&
&  0.17380E+00_JPRB, 0.17346E+00_JPRB, 0.17361E+00_JPRB, 0.17409E+00_JPRB, 0.17497E+00_JPRB, 0.17566E+00_JPRB, 0.17630E+00_JPRB ,&
&  0.17705E+00_JPRB, 0.17771E+00_JPRB, 0.17782E+00_JPRB, 0.17816E+00_JPRB, 0.17848E+00_JPRB, 0.17947E+00_JPRB, 0.17987E+00_JPRB ,&
&  0.18033E+00_JPRB, 0.18067E+00_JPRB, 0.18061E+00_JPRB, 0.18108E+00_JPRB, 0.18190E+00_JPRB, 0.18359E+00_JPRB, 0.18558E+00_JPRB ,&
&  0.18816E+00_JPRB, 0.19113E+00_JPRB, 0.19446E+00_JPRB, 0.19775E+00_JPRB, 0.20211E+00_JPRB, 0.20713E+00_JPRB, 0.21349E+00_JPRB ,&
&  0.22046E+00_JPRB, 0.22890E+00_JPRB, 0.24004E+00_JPRB, 0.25383E+00_JPRB, 0.27054E+00_JPRB, 0.28869E+00_JPRB, 0.31011E+00_JPRB ,&
&  0.34592E+00_JPRB, 0.44704E+00_JPRB, 0.50712E+00_JPRB, 0.49286E+00_JPRB, 0.50245E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.58528E+00_JPRB, 0.60138E+00_JPRB, 0.62680E+00_JPRB, 0.61949E+00_JPRB, 0.63991E+00_JPRB, 0.62641E+00_JPRB, 0.60398E+00_JPRB ,&
&  0.55669E+00_JPRB, 0.49943E+00_JPRB, 0.45311E+00_JPRB, 0.40100E+00_JPRB, 0.36080E+00_JPRB, 0.33118E+00_JPRB, 0.31683E+00_JPRB ,&
&  0.31013E+00_JPRB, 0.30807E+00_JPRB, 0.30725E+00_JPRB, 0.30660E+00_JPRB, 0.30240E+00_JPRB, 0.29934E+00_JPRB, 0.29387E+00_JPRB ,&
&  0.28797E+00_JPRB, 0.28320E+00_JPRB, 0.27992E+00_JPRB, 0.28492E+00_JPRB, 0.29125E+00_JPRB, 0.29796E+00_JPRB, 0.30181E+00_JPRB ,&
&  0.30537E+00_JPRB, 0.30669E+00_JPRB, 0.30928E+00_JPRB, 0.30752E+00_JPRB, 0.31846E+00_JPRB, 0.33997E+00_JPRB, 0.35878E+00_JPRB ,&
&  0.37697E+00_JPRB, 0.39224E+00_JPRB, 0.38849E+00_JPRB, 0.36508E+00_JPRB, 0.29965E+00_JPRB, 0.26539E+00_JPRB, 0.23581E+00_JPRB ,&
&  0.21736E+00_JPRB, 0.20374E+00_JPRB, 0.19476E+00_JPRB, 0.18959E+00_JPRB, 0.18567E+00_JPRB, 0.18334E+00_JPRB, 0.18086E+00_JPRB ,&
&  0.17845E+00_JPRB, 0.17689E+00_JPRB, 0.17523E+00_JPRB, 0.17444E+00_JPRB, 0.17357E+00_JPRB, 0.17243E+00_JPRB, 0.17149E+00_JPRB ,&
&  0.17155E+00_JPRB, 0.17132E+00_JPRB, 0.17232E+00_JPRB, 0.17325E+00_JPRB, 0.17510E+00_JPRB, 0.17669E+00_JPRB, 0.17881E+00_JPRB ,&
&  0.18050E+00_JPRB, 0.18185E+00_JPRB, 0.18167E+00_JPRB, 0.18110E+00_JPRB, 0.18020E+00_JPRB, 0.17961E+00_JPRB, 0.17866E+00_JPRB ,&
&  0.17754E+00_JPRB, 0.17761E+00_JPRB, 0.17772E+00_JPRB, 0.17797E+00_JPRB, 0.17893E+00_JPRB, 0.18036E+00_JPRB, 0.18267E+00_JPRB ,&
&  0.18570E+00_JPRB, 0.18889E+00_JPRB, 0.19156E+00_JPRB, 0.19433E+00_JPRB, 0.19794E+00_JPRB, 0.20211E+00_JPRB, 0.20736E+00_JPRB ,&
&  0.21226E+00_JPRB, 0.22028E+00_JPRB, 0.23119E+00_JPRB, 0.24235E+00_JPRB, 0.25823E+00_JPRB, 0.27500E+00_JPRB, 0.30311E+00_JPRB ,&
&  0.35303E+00_JPRB, 0.52807E+00_JPRB, 0.81256E+00_JPRB, 0.58486E+00_JPRB, 0.57181E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.62163E+00_JPRB, 0.63233E+00_JPRB, 0.64636E+00_JPRB, 0.64917E+00_JPRB, 0.65924E+00_JPRB, 0.65810E+00_JPRB, 0.64116E+00_JPRB ,&
&  0.61297E+00_JPRB, 0.58671E+00_JPRB, 0.57225E+00_JPRB, 0.55557E+00_JPRB, 0.52823E+00_JPRB, 0.46758E+00_JPRB, 0.44427E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.42572E+00_JPRB, 0.41677E+00_JPRB, 0.41706E+00_JPRB, 0.40625E+00_JPRB, 0.39505E+00_JPRB, 0.37685E+00_JPRB ,&
&  0.35989E+00_JPRB, 0.34258E+00_JPRB, 0.33076E+00_JPRB, 0.32473E+00_JPRB, 0.32265E+00_JPRB, 0.31850E+00_JPRB, 0.30693E+00_JPRB ,&
&  0.29856E+00_JPRB, 0.29690E+00_JPRB, 0.29962E+00_JPRB, 0.31024E+00_JPRB, 0.35475E+00_JPRB, 0.41061E+00_JPRB, 0.41079E+00_JPRB ,&
&  0.43396E+00_JPRB, 0.44782E+00_JPRB, 0.46128E+00_JPRB, 0.45073E+00_JPRB, 0.41851E+00_JPRB, 0.38074E+00_JPRB, 0.32641E+00_JPRB ,&
&  0.27466E+00_JPRB, 0.24607E+00_JPRB, 0.22291E+00_JPRB, 0.20700E+00_JPRB, 0.19644E+00_JPRB, 0.19198E+00_JPRB, 0.18634E+00_JPRB ,&
&  0.18257E+00_JPRB, 0.17975E+00_JPRB, 0.17856E+00_JPRB, 0.17785E+00_JPRB, 0.17711E+00_JPRB, 0.17515E+00_JPRB, 0.17456E+00_JPRB ,&
&  0.17504E+00_JPRB, 0.17526E+00_JPRB, 0.17516E+00_JPRB, 0.17591E+00_JPRB, 0.17688E+00_JPRB, 0.17823E+00_JPRB, 0.17939E+00_JPRB ,&
&  0.18111E+00_JPRB, 0.18203E+00_JPRB, 0.18358E+00_JPRB, 0.18523E+00_JPRB, 0.18431E+00_JPRB, 0.18308E+00_JPRB, 0.18027E+00_JPRB ,&
&  0.17724E+00_JPRB, 0.17674E+00_JPRB, 0.17613E+00_JPRB, 0.17664E+00_JPRB, 0.17810E+00_JPRB, 0.18014E+00_JPRB, 0.18286E+00_JPRB ,&
&  0.18549E+00_JPRB, 0.18821E+00_JPRB, 0.19033E+00_JPRB, 0.19242E+00_JPRB, 0.19586E+00_JPRB, 0.19957E+00_JPRB, 0.20428E+00_JPRB ,&
&  0.20919E+00_JPRB, 0.21469E+00_JPRB, 0.22237E+00_JPRB, 0.23231E+00_JPRB, 0.24620E+00_JPRB, 0.26220E+00_JPRB, 0.28703E+00_JPRB ,&
&  0.31114E+00_JPRB, 0.41271E+00_JPRB, 0.64981E+00_JPRB, 0.57624E+00_JPRB, 0.59422E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.69663E+00_JPRB, 0.70771E+00_JPRB, 0.71735E+00_JPRB, 0.71901E+00_JPRB, 0.71825E+00_JPRB, 0.71334E+00_JPRB, 0.70136E+00_JPRB ,&
&  0.68173E+00_JPRB, 0.65551E+00_JPRB, 0.63008E+00_JPRB, 0.60978E+00_JPRB, 0.59712E+00_JPRB, 0.60058E+00_JPRB, 0.60955E+00_JPRB ,&
&  0.62744E+00_JPRB, 0.65334E+00_JPRB, 0.65893E+00_JPRB, 0.65271E+00_JPRB, 0.61724E+00_JPRB, 0.58027E+00_JPRB, 0.54574E+00_JPRB ,&
&  0.51521E+00_JPRB, 0.48674E+00_JPRB, 0.47595E+00_JPRB, 0.45728E+00_JPRB, 0.44406E+00_JPRB, 0.42760E+00_JPRB, 0.40455E+00_JPRB ,&
&  0.36685E+00_JPRB, 0.33609E+00_JPRB, 0.31793E+00_JPRB, 0.33245E+00_JPRB, 0.42253E+00_JPRB, 0.48312E+00_JPRB, 0.49187E+00_JPRB ,&
&  0.52674E+00_JPRB, 0.55183E+00_JPRB, 0.54726E+00_JPRB, 0.52771E+00_JPRB, 0.50646E+00_JPRB, 0.48321E+00_JPRB, 0.46425E+00_JPRB ,&
&  0.44539E+00_JPRB, 0.41906E+00_JPRB, 0.36994E+00_JPRB, 0.30035E+00_JPRB, 0.26463E+00_JPRB, 0.25111E+00_JPRB, 0.23666E+00_JPRB ,&
&  0.22574E+00_JPRB, 0.21662E+00_JPRB, 0.21310E+00_JPRB, 0.21004E+00_JPRB, 0.20927E+00_JPRB, 0.20522E+00_JPRB, 0.20182E+00_JPRB ,&
&  0.19779E+00_JPRB, 0.19486E+00_JPRB, 0.19035E+00_JPRB, 0.18587E+00_JPRB, 0.18269E+00_JPRB, 0.18214E+00_JPRB, 0.18183E+00_JPRB ,&
&  0.18276E+00_JPRB, 0.18669E+00_JPRB, 0.19765E+00_JPRB, 0.20376E+00_JPRB, 0.20411E+00_JPRB, 0.19911E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18362E+00_JPRB, 0.18218E+00_JPRB, 0.18264E+00_JPRB, 0.18522E+00_JPRB, 0.18873E+00_JPRB, 0.19172E+00_JPRB ,&
&  0.19411E+00_JPRB, 0.19637E+00_JPRB, 0.19855E+00_JPRB, 0.20088E+00_JPRB, 0.20367E+00_JPRB, 0.20698E+00_JPRB, 0.21149E+00_JPRB ,&
&  0.21636E+00_JPRB, 0.22315E+00_JPRB, 0.23307E+00_JPRB, 0.24732E+00_JPRB, 0.25977E+00_JPRB, 0.26753E+00_JPRB, 0.27784E+00_JPRB ,&
&  0.29358E+00_JPRB, 0.34006E+00_JPRB, 0.47384E+00_JPRB, 0.70130E+00_JPRB, 0.73337E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.75683E+00_JPRB, 0.77468E+00_JPRB, 0.79556E+00_JPRB, 0.80662E+00_JPRB, 0.80762E+00_JPRB, 0.80469E+00_JPRB, 0.79605E+00_JPRB ,&
&  0.78119E+00_JPRB, 0.76323E+00_JPRB, 0.75004E+00_JPRB, 0.74119E+00_JPRB, 0.73741E+00_JPRB, 0.74057E+00_JPRB, 0.74636E+00_JPRB ,&
&  0.74613E+00_JPRB, 0.74243E+00_JPRB, 0.73068E+00_JPRB, 0.71537E+00_JPRB, 0.69919E+00_JPRB, 0.68659E+00_JPRB, 0.67684E+00_JPRB ,&
&  0.66716E+00_JPRB, 0.65243E+00_JPRB, 0.62942E+00_JPRB, 0.59417E+00_JPRB, 0.55258E+00_JPRB, 0.51449E+00_JPRB, 0.48575E+00_JPRB ,&
&  0.46180E+00_JPRB, 0.39430E+00_JPRB, 0.35681E+00_JPRB, 0.36959E+00_JPRB, 0.46365E+00_JPRB, 0.49354E+00_JPRB, 0.51738E+00_JPRB ,&
&  0.55486E+00_JPRB, 0.59467E+00_JPRB, 0.61365E+00_JPRB, 0.61426E+00_JPRB, 0.60885E+00_JPRB, 0.59638E+00_JPRB, 0.57686E+00_JPRB ,&
&  0.55206E+00_JPRB, 0.52162E+00_JPRB, 0.49580E+00_JPRB, 0.46715E+00_JPRB, 0.43751E+00_JPRB, 0.41556E+00_JPRB, 0.40022E+00_JPRB ,&
&  0.38961E+00_JPRB, 0.38013E+00_JPRB, 0.37269E+00_JPRB, 0.36301E+00_JPRB, 0.35384E+00_JPRB, 0.33280E+00_JPRB, 0.31123E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.27026E+00_JPRB, 0.25164E+00_JPRB, 0.23353E+00_JPRB, 0.21867E+00_JPRB, 0.20905E+00_JPRB, 0.20209E+00_JPRB ,&
&  0.19979E+00_JPRB, 0.20521E+00_JPRB, 0.22335E+00_JPRB, 0.25637E+00_JPRB, 0.29570E+00_JPRB, 0.26450E+00_JPRB, 0.24344E+00_JPRB ,&
&  0.22322E+00_JPRB, 0.21129E+00_JPRB, 0.20683E+00_JPRB, 0.20631E+00_JPRB, 0.20759E+00_JPRB, 0.20993E+00_JPRB, 0.21226E+00_JPRB ,&
&  0.21531E+00_JPRB, 0.21814E+00_JPRB, 0.22117E+00_JPRB, 0.22333E+00_JPRB, 0.22475E+00_JPRB, 0.22738E+00_JPRB, 0.23196E+00_JPRB ,&
&  0.23891E+00_JPRB, 0.25051E+00_JPRB, 0.27002E+00_JPRB, 0.29541E+00_JPRB, 0.32151E+00_JPRB, 0.34533E+00_JPRB, 0.39832E+00_JPRB ,&
&  0.48573E+00_JPRB, 0.59839E+00_JPRB, 0.65421E+00_JPRB, 0.70892E+00_JPRB, 0.73585E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.81703E+00_JPRB, 0.84316E+00_JPRB, 0.87226E+00_JPRB, 0.88893E+00_JPRB, 0.88967E+00_JPRB, 0.88640E+00_JPRB, 0.87137E+00_JPRB ,&
&  0.84036E+00_JPRB, 0.81474E+00_JPRB, 0.81029E+00_JPRB, 0.81404E+00_JPRB, 0.82578E+00_JPRB, 0.84450E+00_JPRB, 0.84650E+00_JPRB ,&
&  0.82067E+00_JPRB, 0.80179E+00_JPRB, 0.79251E+00_JPRB, 0.78603E+00_JPRB, 0.76969E+00_JPRB, 0.74556E+00_JPRB, 0.72991E+00_JPRB ,&
&  0.72562E+00_JPRB, 0.72584E+00_JPRB, 0.71701E+00_JPRB, 0.70301E+00_JPRB, 0.67154E+00_JPRB, 0.63281E+00_JPRB, 0.60265E+00_JPRB ,&
&  0.59281E+00_JPRB, 0.58481E+00_JPRB, 0.54374E+00_JPRB, 0.56140E+00_JPRB, 0.56499E+00_JPRB, 0.56524E+00_JPRB, 0.56309E+00_JPRB ,&
&  0.57603E+00_JPRB, 0.60956E+00_JPRB, 0.64251E+00_JPRB, 0.65231E+00_JPRB, 0.66188E+00_JPRB, 0.66374E+00_JPRB, 0.66543E+00_JPRB ,&
&  0.67016E+00_JPRB, 0.66221E+00_JPRB, 0.63260E+00_JPRB, 0.58876E+00_JPRB, 0.54549E+00_JPRB, 0.51181E+00_JPRB, 0.48309E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.45075E+00_JPRB, 0.44107E+00_JPRB, 0.43764E+00_JPRB, 0.44507E+00_JPRB, 0.45986E+00_JPRB, 0.45476E+00_JPRB ,&
&  0.43681E+00_JPRB, 0.42052E+00_JPRB, 0.39876E+00_JPRB, 0.36923E+00_JPRB, 0.33940E+00_JPRB, 0.30038E+00_JPRB, 0.28061E+00_JPRB ,&
&  0.28497E+00_JPRB, 0.32393E+00_JPRB, 0.37540E+00_JPRB, 0.39414E+00_JPRB, 0.42107E+00_JPRB, 0.44713E+00_JPRB, 0.43659E+00_JPRB ,&
&  0.39038E+00_JPRB, 0.32050E+00_JPRB, 0.29762E+00_JPRB, 0.28558E+00_JPRB, 0.27878E+00_JPRB, 0.27379E+00_JPRB, 0.27311E+00_JPRB ,&
&  0.27723E+00_JPRB, 0.28862E+00_JPRB, 0.29783E+00_JPRB, 0.29451E+00_JPRB, 0.28411E+00_JPRB, 0.27744E+00_JPRB, 0.27783E+00_JPRB ,&
&  0.28640E+00_JPRB, 0.31408E+00_JPRB, 0.37770E+00_JPRB, 0.49448E+00_JPRB, 0.52934E+00_JPRB, 0.56179E+00_JPRB, 0.59951E+00_JPRB ,&
&  0.63567E+00_JPRB, 0.66857E+00_JPRB, 0.69680E+00_JPRB, 0.73168E+00_JPRB, 0.77840E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.81193E+00_JPRB, 0.83320E+00_JPRB, 0.85174E+00_JPRB, 0.86432E+00_JPRB, 0.86592E+00_JPRB, 0.85908E+00_JPRB, 0.84330E+00_JPRB ,&
&  0.81344E+00_JPRB, 0.79650E+00_JPRB, 0.80553E+00_JPRB, 0.82022E+00_JPRB, 0.82923E+00_JPRB, 0.82919E+00_JPRB, 0.79779E+00_JPRB ,&
&  0.74398E+00_JPRB, 0.72368E+00_JPRB, 0.72079E+00_JPRB, 0.73057E+00_JPRB, 0.74915E+00_JPRB, 0.75661E+00_JPRB, 0.74509E+00_JPRB ,&
&  0.73321E+00_JPRB, 0.73516E+00_JPRB, 0.73817E+00_JPRB, 0.74512E+00_JPRB, 0.73431E+00_JPRB, 0.71340E+00_JPRB, 0.70462E+00_JPRB ,&
&  0.70234E+00_JPRB, 0.69540E+00_JPRB, 0.68811E+00_JPRB, 0.63922E+00_JPRB, 0.66446E+00_JPRB, 0.65297E+00_JPRB, 0.63496E+00_JPRB ,&
&  0.61868E+00_JPRB, 0.62155E+00_JPRB, 0.66307E+00_JPRB, 0.70925E+00_JPRB, 0.72060E+00_JPRB, 0.72344E+00_JPRB, 0.71891E+00_JPRB ,&
&  0.71572E+00_JPRB, 0.71611E+00_JPRB, 0.71746E+00_JPRB, 0.70926E+00_JPRB, 0.68883E+00_JPRB, 0.66254E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.61602E+00_JPRB, 0.59084E+00_JPRB, 0.54719E+00_JPRB, 0.52686E+00_JPRB, 0.52478E+00_JPRB, 0.53259E+00_JPRB, 0.53637E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.52600E+00_JPRB, 0.53038E+00_JPRB, 0.52781E+00_JPRB, 0.50625E+00_JPRB, 0.49341E+00_JPRB, 0.48754E+00_JPRB ,&
&  0.47594E+00_JPRB, 0.47443E+00_JPRB, 0.47871E+00_JPRB, 0.48995E+00_JPRB, 0.49983E+00_JPRB, 0.51313E+00_JPRB, 0.51835E+00_JPRB ,&
&  0.52034E+00_JPRB, 0.51711E+00_JPRB, 0.51158E+00_JPRB, 0.49986E+00_JPRB, 0.48170E+00_JPRB, 0.47482E+00_JPRB, 0.47624E+00_JPRB ,&
&  0.48488E+00_JPRB, 0.49046E+00_JPRB, 0.48956E+00_JPRB, 0.48824E+00_JPRB, 0.47666E+00_JPRB, 0.46399E+00_JPRB, 0.45413E+00_JPRB ,&
&  0.44932E+00_JPRB, 0.45163E+00_JPRB, 0.46620E+00_JPRB, 0.49046E+00_JPRB, 0.52187E+00_JPRB, 0.55986E+00_JPRB, 0.60433E+00_JPRB ,&
&  0.63608E+00_JPRB, 0.65505E+00_JPRB, 0.67947E+00_JPRB, 0.72669E+00_JPRB, 0.77192E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.77422E+00_JPRB, 0.78238E+00_JPRB, 0.79212E+00_JPRB, 0.80510E+00_JPRB, 0.81676E+00_JPRB, 0.81340E+00_JPRB, 0.80098E+00_JPRB ,&
&  0.77739E+00_JPRB, 0.76879E+00_JPRB, 0.77366E+00_JPRB, 0.77612E+00_JPRB, 0.77785E+00_JPRB, 0.77551E+00_JPRB, 0.75949E+00_JPRB ,&
&  0.72280E+00_JPRB, 0.69340E+00_JPRB, 0.67930E+00_JPRB, 0.67202E+00_JPRB, 0.66972E+00_JPRB, 0.67693E+00_JPRB, 0.68586E+00_JPRB ,&
&  0.70347E+00_JPRB, 0.71428E+00_JPRB, 0.72676E+00_JPRB, 0.73633E+00_JPRB, 0.73905E+00_JPRB, 0.74302E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.73061E+00_JPRB, 0.73124E+00_JPRB, 0.73325E+00_JPRB, 0.72406E+00_JPRB, 0.70328E+00_JPRB, 0.68237E+00_JPRB, 0.65642E+00_JPRB ,&
&  0.63936E+00_JPRB, 0.63091E+00_JPRB, 0.65648E+00_JPRB, 0.71798E+00_JPRB, 0.76101E+00_JPRB, 0.76909E+00_JPRB, 0.76152E+00_JPRB ,&
&  0.76383E+00_JPRB, 0.77133E+00_JPRB, 0.77482E+00_JPRB, 0.76321E+00_JPRB, 0.74344E+00_JPRB, 0.72606E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.70331E+00_JPRB, 0.70205E+00_JPRB, 0.68418E+00_JPRB, 0.65172E+00_JPRB, 0.62411E+00_JPRB, 0.60464E+00_JPRB, 0.59447E+00_JPRB ,&
&  0.58853E+00_JPRB, 0.57932E+00_JPRB, 0.57385E+00_JPRB, 0.57909E+00_JPRB, 0.55948E+00_JPRB, 0.54159E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.56172E+00_JPRB, 0.56691E+00_JPRB, 0.57572E+00_JPRB, 0.57828E+00_JPRB, 0.57456E+00_JPRB, 0.57020E+00_JPRB, 0.57094E+00_JPRB ,&
&  0.57910E+00_JPRB, 0.58844E+00_JPRB, 0.59527E+00_JPRB, 0.60011E+00_JPRB, 0.60512E+00_JPRB, 0.59357E+00_JPRB, 0.57259E+00_JPRB ,&
&  0.55485E+00_JPRB, 0.55127E+00_JPRB, 0.55553E+00_JPRB, 0.55470E+00_JPRB, 0.55298E+00_JPRB, 0.55298E+00_JPRB, 0.55134E+00_JPRB ,&
&  0.54823E+00_JPRB, 0.54051E+00_JPRB, 0.53148E+00_JPRB, 0.53182E+00_JPRB, 0.54334E+00_JPRB, 0.56561E+00_JPRB, 0.60023E+00_JPRB ,&
&  0.65218E+00_JPRB, 0.69164E+00_JPRB, 0.72922E+00_JPRB, 0.74409E+00_JPRB, 0.74703E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.74409E+00_JPRB, 0.75221E+00_JPRB, 0.75801E+00_JPRB, 0.76211E+00_JPRB, 0.77041E+00_JPRB, 0.78085E+00_JPRB, 0.78052E+00_JPRB ,&
&  0.76585E+00_JPRB, 0.75234E+00_JPRB, 0.74300E+00_JPRB, 0.74042E+00_JPRB, 0.74210E+00_JPRB, 0.74426E+00_JPRB, 0.73855E+00_JPRB ,&
&  0.71821E+00_JPRB, 0.69636E+00_JPRB, 0.68294E+00_JPRB, 0.66957E+00_JPRB, 0.66139E+00_JPRB, 0.65629E+00_JPRB, 0.66346E+00_JPRB ,&
&  0.67401E+00_JPRB, 0.69047E+00_JPRB, 0.70400E+00_JPRB, 0.71389E+00_JPRB, 0.71809E+00_JPRB, 0.70710E+00_JPRB, 0.70881E+00_JPRB ,&
&  0.71634E+00_JPRB, 0.71897E+00_JPRB, 0.73452E+00_JPRB, 0.73440E+00_JPRB, 0.72162E+00_JPRB, 0.71150E+00_JPRB, 0.70129E+00_JPRB ,&
&  0.69861E+00_JPRB, 0.70994E+00_JPRB, 0.73534E+00_JPRB, 0.77173E+00_JPRB, 0.79242E+00_JPRB, 0.80012E+00_JPRB, 0.79729E+00_JPRB ,&
&  0.80038E+00_JPRB, 0.81242E+00_JPRB, 0.81913E+00_JPRB, 0.81082E+00_JPRB, 0.79242E+00_JPRB, 0.77724E+00_JPRB, 0.76373E+00_JPRB ,&
&  0.75355E+00_JPRB, 0.74134E+00_JPRB, 0.73307E+00_JPRB, 0.72296E+00_JPRB, 0.70124E+00_JPRB, 0.67549E+00_JPRB, 0.65316E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.62067E+00_JPRB, 0.60102E+00_JPRB, 0.59513E+00_JPRB, 0.59953E+00_JPRB, 0.57659E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.55503E+00_JPRB, 0.58590E+00_JPRB, 0.60989E+00_JPRB, 0.61918E+00_JPRB, 0.61944E+00_JPRB, 0.62325E+00_JPRB, 0.62618E+00_JPRB ,&
&  0.63214E+00_JPRB, 0.64027E+00_JPRB, 0.64645E+00_JPRB, 0.65724E+00_JPRB, 0.66856E+00_JPRB, 0.66712E+00_JPRB, 0.65329E+00_JPRB ,&
&  0.63790E+00_JPRB, 0.62891E+00_JPRB, 0.62481E+00_JPRB, 0.62668E+00_JPRB, 0.62985E+00_JPRB, 0.62759E+00_JPRB, 0.62168E+00_JPRB ,&
&  0.61007E+00_JPRB, 0.59156E+00_JPRB, 0.57580E+00_JPRB, 0.56742E+00_JPRB, 0.56740E+00_JPRB, 0.57423E+00_JPRB, 0.59040E+00_JPRB ,&
&  0.62165E+00_JPRB, 0.66925E+00_JPRB, 0.70400E+00_JPRB, 0.72297E+00_JPRB, 0.72727E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.71686E+00_JPRB, 0.72308E+00_JPRB, 0.73208E+00_JPRB, 0.74511E+00_JPRB, 0.75853E+00_JPRB, 0.76706E+00_JPRB, 0.76630E+00_JPRB ,&
&  0.75528E+00_JPRB, 0.74214E+00_JPRB, 0.73793E+00_JPRB, 0.73751E+00_JPRB, 0.73472E+00_JPRB, 0.73353E+00_JPRB, 0.72933E+00_JPRB ,&
&  0.71885E+00_JPRB, 0.70259E+00_JPRB, 0.69504E+00_JPRB, 0.68163E+00_JPRB, 0.66891E+00_JPRB, 0.65862E+00_JPRB, 0.65907E+00_JPRB ,&
&  0.66811E+00_JPRB, 0.67843E+00_JPRB, 0.69113E+00_JPRB, 0.70578E+00_JPRB, 0.71414E+00_JPRB, 0.71340E+00_JPRB, 0.71078E+00_JPRB ,&
&  0.70872E+00_JPRB, 0.70647E+00_JPRB, 0.71805E+00_JPRB, 0.72750E+00_JPRB, 0.73381E+00_JPRB, 0.74508E+00_JPRB, 0.75414E+00_JPRB ,&
&  0.76150E+00_JPRB, 0.77235E+00_JPRB, 0.79066E+00_JPRB, 0.80419E+00_JPRB, 0.81370E+00_JPRB, 0.82403E+00_JPRB, 0.82793E+00_JPRB ,&
&  0.83439E+00_JPRB, 0.84556E+00_JPRB, 0.84789E+00_JPRB, 0.83884E+00_JPRB, 0.82433E+00_JPRB, 0.81478E+00_JPRB, 0.80917E+00_JPRB ,&
&  0.79952E+00_JPRB, 0.78007E+00_JPRB, 0.76548E+00_JPRB, 0.75401E+00_JPRB, 0.73540E+00_JPRB, 0.71542E+00_JPRB, 0.69201E+00_JPRB ,&
&  0.66530E+00_JPRB, 0.64542E+00_JPRB, 0.63001E+00_JPRB, 0.61611E+00_JPRB, 0.61293E+00_JPRB, 0.60452E+00_JPRB, 0.59787E+00_JPRB ,&
&  0.60236E+00_JPRB, 0.62029E+00_JPRB, 0.63353E+00_JPRB, 0.63766E+00_JPRB, 0.63962E+00_JPRB, 0.65486E+00_JPRB, 0.67346E+00_JPRB ,&
&  0.68785E+00_JPRB, 0.69362E+00_JPRB, 0.68922E+00_JPRB, 0.69114E+00_JPRB, 0.69597E+00_JPRB, 0.70023E+00_JPRB, 0.69444E+00_JPRB ,&
&  0.68604E+00_JPRB, 0.68049E+00_JPRB, 0.67929E+00_JPRB, 0.67573E+00_JPRB, 0.67725E+00_JPRB, 0.67712E+00_JPRB, 0.67428E+00_JPRB ,&
&  0.66846E+00_JPRB, 0.65887E+00_JPRB, 0.64669E+00_JPRB, 0.63983E+00_JPRB, 0.62982E+00_JPRB, 0.61464E+00_JPRB, 0.61597E+00_JPRB ,&
&  0.62757E+00_JPRB, 0.64604E+00_JPRB, 0.66317E+00_JPRB, 0.68311E+00_JPRB, 0.70620E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.69596E+00_JPRB, 0.70831E+00_JPRB, 0.72364E+00_JPRB, 0.73927E+00_JPRB, 0.74889E+00_JPRB, 0.75470E+00_JPRB, 0.75232E+00_JPRB ,&
&  0.74155E+00_JPRB, 0.73088E+00_JPRB, 0.73250E+00_JPRB, 0.73244E+00_JPRB, 0.73165E+00_JPRB, 0.73023E+00_JPRB, 0.72406E+00_JPRB ,&
&  0.71520E+00_JPRB, 0.70779E+00_JPRB, 0.70485E+00_JPRB, 0.70217E+00_JPRB, 0.69261E+00_JPRB, 0.68533E+00_JPRB, 0.67755E+00_JPRB ,&
&  0.67784E+00_JPRB, 0.68095E+00_JPRB, 0.68446E+00_JPRB, 0.68548E+00_JPRB, 0.69426E+00_JPRB, 0.70524E+00_JPRB, 0.71022E+00_JPRB ,&
&  0.70520E+00_JPRB, 0.71203E+00_JPRB, 0.63093E+00_JPRB, 0.70090E+00_JPRB, 0.72635E+00_JPRB, 0.74128E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.76537E+00_JPRB, 0.77616E+00_JPRB, 0.78368E+00_JPRB, 0.78756E+00_JPRB, 0.80396E+00_JPRB, 0.81819E+00_JPRB, 0.82218E+00_JPRB ,&
&  0.82352E+00_JPRB, 0.82277E+00_JPRB, 0.82410E+00_JPRB, 0.82388E+00_JPRB, 0.82786E+00_JPRB, 0.82716E+00_JPRB, 0.82077E+00_JPRB ,&
&  0.81248E+00_JPRB, 0.80085E+00_JPRB, 0.78382E+00_JPRB, 0.76432E+00_JPRB, 0.73338E+00_JPRB, 0.70637E+00_JPRB, 0.68922E+00_JPRB ,&
&  0.67483E+00_JPRB, 0.67046E+00_JPRB, 0.66506E+00_JPRB, 0.65931E+00_JPRB, 0.65698E+00_JPRB, 0.65914E+00_JPRB, 0.65787E+00_JPRB ,&
&  0.66440E+00_JPRB, 0.67312E+00_JPRB, 0.67743E+00_JPRB, 0.68459E+00_JPRB, 0.69383E+00_JPRB, 0.70745E+00_JPRB, 0.72020E+00_JPRB ,&
&  0.73216E+00_JPRB, 0.74018E+00_JPRB, 0.74107E+00_JPRB, 0.74088E+00_JPRB, 0.73864E+00_JPRB, 0.73770E+00_JPRB, 0.73080E+00_JPRB ,&
&  0.72525E+00_JPRB, 0.72236E+00_JPRB, 0.71622E+00_JPRB, 0.70459E+00_JPRB, 0.69772E+00_JPRB, 0.69192E+00_JPRB, 0.69061E+00_JPRB ,&
&  0.69171E+00_JPRB, 0.69271E+00_JPRB, 0.69000E+00_JPRB, 0.69081E+00_JPRB, 0.68467E+00_JPRB, 0.66484E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.64902E+00_JPRB, 0.65450E+00_JPRB, 0.65961E+00_JPRB, 0.66896E+00_JPRB, 0.68381E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.73453E+00_JPRB, 0.74005E+00_JPRB, 0.74882E+00_JPRB, 0.75881E+00_JPRB, 0.76184E+00_JPRB, 0.76104E+00_JPRB, 0.75857E+00_JPRB ,&
&  0.75924E+00_JPRB, 0.75117E+00_JPRB, 0.74528E+00_JPRB, 0.74195E+00_JPRB, 0.74162E+00_JPRB, 0.73899E+00_JPRB, 0.73077E+00_JPRB ,&
&  0.72405E+00_JPRB, 0.72197E+00_JPRB, 0.72720E+00_JPRB, 0.72925E+00_JPRB, 0.71463E+00_JPRB, 0.71013E+00_JPRB, 0.70273E+00_JPRB ,&
&  0.70184E+00_JPRB, 0.70263E+00_JPRB, 0.70623E+00_JPRB, 0.71314E+00_JPRB, 0.73022E+00_JPRB, 0.74251E+00_JPRB, 0.74931E+00_JPRB ,&
&  0.74865E+00_JPRB, 0.66982E+00_JPRB, 0.69761E+00_JPRB, 0.71038E+00_JPRB, 0.73310E+00_JPRB, 0.75536E+00_JPRB, 0.77437E+00_JPRB ,&
&  0.79916E+00_JPRB, 0.81239E+00_JPRB, 0.81728E+00_JPRB, 0.81756E+00_JPRB, 0.81894E+00_JPRB, 0.81916E+00_JPRB, 0.81279E+00_JPRB ,&
&  0.80825E+00_JPRB, 0.81504E+00_JPRB, 0.83096E+00_JPRB, 0.83856E+00_JPRB, 0.84217E+00_JPRB, 0.84394E+00_JPRB, 0.84028E+00_JPRB ,&
&  0.83311E+00_JPRB, 0.81900E+00_JPRB, 0.79577E+00_JPRB, 0.76811E+00_JPRB, 0.73684E+00_JPRB, 0.71554E+00_JPRB, 0.70372E+00_JPRB ,&
&  0.69566E+00_JPRB, 0.69527E+00_JPRB, 0.69511E+00_JPRB, 0.69548E+00_JPRB, 0.69800E+00_JPRB, 0.70573E+00_JPRB, 0.70854E+00_JPRB ,&
&  0.71591E+00_JPRB, 0.72344E+00_JPRB, 0.73560E+00_JPRB, 0.75516E+00_JPRB, 0.77499E+00_JPRB, 0.78624E+00_JPRB, 0.78709E+00_JPRB ,&
&  0.79272E+00_JPRB, 0.79520E+00_JPRB, 0.79137E+00_JPRB, 0.79124E+00_JPRB, 0.78756E+00_JPRB, 0.78890E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.77768E+00_JPRB, 0.77158E+00_JPRB, 0.76427E+00_JPRB, 0.74969E+00_JPRB, 0.73722E+00_JPRB, 0.72839E+00_JPRB, 0.72104E+00_JPRB ,&
&  0.72045E+00_JPRB, 0.72368E+00_JPRB, 0.72866E+00_JPRB, 0.72363E+00_JPRB, 0.70964E+00_JPRB, 0.70152E+00_JPRB, 0.69186E+00_JPRB ,&
&  0.68541E+00_JPRB, 0.68810E+00_JPRB, 0.68902E+00_JPRB, 0.69751E+00_JPRB, 0.71716E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.76539E+00_JPRB, 0.76909E+00_JPRB, 0.77366E+00_JPRB, 0.77431E+00_JPRB, 0.77058E+00_JPRB, 0.76844E+00_JPRB, 0.77033E+00_JPRB ,&
&  0.77163E+00_JPRB, 0.76385E+00_JPRB, 0.75599E+00_JPRB, 0.75043E+00_JPRB, 0.74888E+00_JPRB, 0.74167E+00_JPRB, 0.73044E+00_JPRB ,&
&  0.72265E+00_JPRB, 0.72277E+00_JPRB, 0.72918E+00_JPRB, 0.73318E+00_JPRB, 0.72562E+00_JPRB, 0.71730E+00_JPRB, 0.71488E+00_JPRB ,&
&  0.71642E+00_JPRB, 0.72307E+00_JPRB, 0.73394E+00_JPRB, 0.74085E+00_JPRB, 0.75041E+00_JPRB, 0.76535E+00_JPRB, 0.76743E+00_JPRB ,&
&  0.76717E+00_JPRB, 0.63214E+00_JPRB, 0.66934E+00_JPRB, 0.68415E+00_JPRB, 0.71329E+00_JPRB, 0.75496E+00_JPRB, 0.78878E+00_JPRB ,&
&  0.82031E+00_JPRB, 0.83842E+00_JPRB, 0.84658E+00_JPRB, 0.84026E+00_JPRB, 0.82586E+00_JPRB, 0.81267E+00_JPRB, 0.80873E+00_JPRB ,&
&  0.80702E+00_JPRB, 0.81343E+00_JPRB, 0.82737E+00_JPRB, 0.83713E+00_JPRB, 0.84051E+00_JPRB, 0.84128E+00_JPRB, 0.84351E+00_JPRB ,&
&  0.84039E+00_JPRB, 0.82491E+00_JPRB, 0.80317E+00_JPRB, 0.77707E+00_JPRB, 0.75711E+00_JPRB, 0.74852E+00_JPRB, 0.73662E+00_JPRB ,&
&  0.71981E+00_JPRB, 0.70896E+00_JPRB, 0.70630E+00_JPRB, 0.70829E+00_JPRB, 0.71638E+00_JPRB, 0.72563E+00_JPRB, 0.73382E+00_JPRB ,&
&  0.74348E+00_JPRB, 0.75886E+00_JPRB, 0.77535E+00_JPRB, 0.79131E+00_JPRB, 0.80821E+00_JPRB, 0.82440E+00_JPRB, 0.83727E+00_JPRB ,&
&  0.84595E+00_JPRB, 0.84462E+00_JPRB, 0.83328E+00_JPRB, 0.82421E+00_JPRB, 0.82041E+00_JPRB, 0.82437E+00_JPRB, 0.82480E+00_JPRB ,&
&  0.81699E+00_JPRB, 0.81083E+00_JPRB, 0.80860E+00_JPRB, 0.79899E+00_JPRB, 0.78809E+00_JPRB, 0.77839E+00_JPRB, 0.76823E+00_JPRB ,&
&  0.76001E+00_JPRB, 0.75655E+00_JPRB, 0.75344E+00_JPRB, 0.74748E+00_JPRB, 0.73882E+00_JPRB, 0.73572E+00_JPRB, 0.73342E+00_JPRB ,&
&  0.73112E+00_JPRB, 0.72767E+00_JPRB, 0.72943E+00_JPRB, 0.74002E+00_JPRB, 0.75508E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.78073E+00_JPRB, 0.78776E+00_JPRB, 0.79222E+00_JPRB, 0.79705E+00_JPRB, 0.79758E+00_JPRB, 0.79602E+00_JPRB, 0.80001E+00_JPRB ,&
&  0.80438E+00_JPRB, 0.79967E+00_JPRB, 0.79182E+00_JPRB, 0.78968E+00_JPRB, 0.78147E+00_JPRB, 0.76906E+00_JPRB, 0.75960E+00_JPRB ,&
&  0.74751E+00_JPRB, 0.74374E+00_JPRB, 0.74184E+00_JPRB, 0.73975E+00_JPRB, 0.73439E+00_JPRB, 0.72727E+00_JPRB, 0.73162E+00_JPRB ,&
&  0.73882E+00_JPRB, 0.74637E+00_JPRB, 0.74900E+00_JPRB, 0.75392E+00_JPRB, 0.76089E+00_JPRB, 0.76871E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.67403E+00_JPRB, 0.55076E+00_JPRB, 0.60098E+00_JPRB, 0.65199E+00_JPRB, 0.69501E+00_JPRB, 0.75966E+00_JPRB, 0.80581E+00_JPRB ,&
&  0.84128E+00_JPRB, 0.87228E+00_JPRB, 0.87855E+00_JPRB, 0.85813E+00_JPRB, 0.83690E+00_JPRB, 0.82948E+00_JPRB, 0.82857E+00_JPRB ,&
&  0.82599E+00_JPRB, 0.82669E+00_JPRB, 0.83224E+00_JPRB, 0.83765E+00_JPRB, 0.84289E+00_JPRB, 0.84937E+00_JPRB, 0.85170E+00_JPRB ,&
&  0.84966E+00_JPRB, 0.83759E+00_JPRB, 0.82561E+00_JPRB, 0.80716E+00_JPRB, 0.79127E+00_JPRB, 0.78690E+00_JPRB, 0.77820E+00_JPRB ,&
&  0.76308E+00_JPRB, 0.75280E+00_JPRB, 0.75074E+00_JPRB, 0.75366E+00_JPRB, 0.76267E+00_JPRB, 0.77331E+00_JPRB, 0.79130E+00_JPRB ,&
&  0.81186E+00_JPRB, 0.82693E+00_JPRB, 0.83736E+00_JPRB, 0.84992E+00_JPRB, 0.85708E+00_JPRB, 0.86662E+00_JPRB, 0.88388E+00_JPRB ,&
&  0.89822E+00_JPRB, 0.88948E+00_JPRB, 0.88097E+00_JPRB, 0.87193E+00_JPRB, 0.86849E+00_JPRB, 0.86581E+00_JPRB, 0.85681E+00_JPRB ,&
&  0.84843E+00_JPRB, 0.84366E+00_JPRB, 0.84162E+00_JPRB, 0.84593E+00_JPRB, 0.84457E+00_JPRB, 0.83969E+00_JPRB, 0.83811E+00_JPRB ,&
&  0.83737E+00_JPRB, 0.83031E+00_JPRB, 0.82147E+00_JPRB, 0.81717E+00_JPRB, 0.80936E+00_JPRB, 0.80499E+00_JPRB, 0.79967E+00_JPRB ,&
&  0.79492E+00_JPRB, 0.77985E+00_JPRB, 0.78424E+00_JPRB, 0.77827E+00_JPRB, 0.77544E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.77794E+00_JPRB, 0.80135E+00_JPRB, 0.80732E+00_JPRB, 0.81999E+00_JPRB, 0.82756E+00_JPRB, 0.83228E+00_JPRB, 0.84035E+00_JPRB ,&
&  0.84576E+00_JPRB, 0.84203E+00_JPRB, 0.83569E+00_JPRB, 0.82282E+00_JPRB, 0.80136E+00_JPRB, 0.77826E+00_JPRB, 0.75989E+00_JPRB ,&
&  0.74780E+00_JPRB, 0.74812E+00_JPRB, 0.74271E+00_JPRB, 0.73485E+00_JPRB, 0.73037E+00_JPRB, 0.72837E+00_JPRB, 0.73156E+00_JPRB ,&
&  0.74037E+00_JPRB, 0.74524E+00_JPRB, 0.74113E+00_JPRB, 0.74094E+00_JPRB, 0.75074E+00_JPRB, 0.76791E+00_JPRB, 0.77656E+00_JPRB ,&
&  0.77994E+00_JPRB, 0.58670E+00_JPRB, 0.62525E+00_JPRB, 0.67068E+00_JPRB, 0.70640E+00_JPRB, 0.75954E+00_JPRB, 0.81963E+00_JPRB ,&
&  0.85841E+00_JPRB, 0.88297E+00_JPRB, 0.88616E+00_JPRB, 0.87927E+00_JPRB, 0.86295E+00_JPRB, 0.84823E+00_JPRB, 0.83182E+00_JPRB ,&
&  0.81822E+00_JPRB, 0.81580E+00_JPRB, 0.82180E+00_JPRB, 0.82784E+00_JPRB, 0.83133E+00_JPRB, 0.83643E+00_JPRB, 0.83192E+00_JPRB ,&
&  0.83053E+00_JPRB, 0.82722E+00_JPRB, 0.82475E+00_JPRB, 0.82401E+00_JPRB, 0.82194E+00_JPRB, 0.82005E+00_JPRB, 0.81350E+00_JPRB ,&
&  0.79816E+00_JPRB, 0.78679E+00_JPRB, 0.77971E+00_JPRB, 0.77474E+00_JPRB, 0.77967E+00_JPRB, 0.79247E+00_JPRB, 0.81217E+00_JPRB ,&
&  0.83334E+00_JPRB, 0.83787E+00_JPRB, 0.84417E+00_JPRB, 0.85197E+00_JPRB, 0.86377E+00_JPRB, 0.87565E+00_JPRB, 0.89038E+00_JPRB ,&
&  0.89520E+00_JPRB, 0.88847E+00_JPRB, 0.88278E+00_JPRB, 0.87232E+00_JPRB, 0.87150E+00_JPRB, 0.85796E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.83979E+00_JPRB, 0.84061E+00_JPRB, 0.84237E+00_JPRB, 0.85220E+00_JPRB, 0.85937E+00_JPRB, 0.86026E+00_JPRB, 0.86824E+00_JPRB ,&
&  0.88030E+00_JPRB, 0.88925E+00_JPRB, 0.88765E+00_JPRB, 0.89014E+00_JPRB, 0.88294E+00_JPRB, 0.87691E+00_JPRB, 0.85840E+00_JPRB ,&
&  0.83396E+00_JPRB, 0.81660E+00_JPRB, 0.63711E+00_JPRB, 0.72188E+00_JPRB, 0.74694E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.81573E+00_JPRB, 0.85322E+00_JPRB, 0.87064E+00_JPRB, 0.87238E+00_JPRB, 0.87593E+00_JPRB, 0.87574E+00_JPRB, 0.87083E+00_JPRB ,&
&  0.86824E+00_JPRB, 0.86323E+00_JPRB, 0.85070E+00_JPRB, 0.83170E+00_JPRB, 0.81256E+00_JPRB, 0.79240E+00_JPRB, 0.77788E+00_JPRB ,&
&  0.77606E+00_JPRB, 0.77664E+00_JPRB, 0.76755E+00_JPRB, 0.75883E+00_JPRB, 0.74994E+00_JPRB, 0.74285E+00_JPRB, 0.74094E+00_JPRB ,&
&  0.73779E+00_JPRB, 0.73723E+00_JPRB, 0.74378E+00_JPRB, 0.75216E+00_JPRB, 0.76260E+00_JPRB, 0.77397E+00_JPRB, 0.78904E+00_JPRB ,&
&  0.79088E+00_JPRB, 0.60541E+00_JPRB, 0.62988E+00_JPRB, 0.70361E+00_JPRB, 0.75007E+00_JPRB, 0.80876E+00_JPRB, 0.86508E+00_JPRB ,&
&  0.88922E+00_JPRB, 0.91640E+00_JPRB, 0.91500E+00_JPRB, 0.90574E+00_JPRB, 0.90846E+00_JPRB, 0.89593E+00_JPRB, 0.86281E+00_JPRB ,&
&  0.83786E+00_JPRB, 0.83742E+00_JPRB, 0.85077E+00_JPRB, 0.86084E+00_JPRB, 0.86179E+00_JPRB, 0.85691E+00_JPRB, 0.85365E+00_JPRB ,&
&  0.84860E+00_JPRB, 0.84208E+00_JPRB, 0.83384E+00_JPRB, 0.82261E+00_JPRB, 0.82495E+00_JPRB, 0.83820E+00_JPRB, 0.84584E+00_JPRB ,&
&  0.84281E+00_JPRB, 0.83060E+00_JPRB, 0.81855E+00_JPRB, 0.81215E+00_JPRB, 0.81570E+00_JPRB, 0.83341E+00_JPRB, 0.85288E+00_JPRB ,&
&  0.86313E+00_JPRB, 0.86776E+00_JPRB, 0.87625E+00_JPRB, 0.87774E+00_JPRB, 0.88322E+00_JPRB, 0.88933E+00_JPRB, 0.89997E+00_JPRB ,&
&  0.90437E+00_JPRB, 0.89717E+00_JPRB, 0.88856E+00_JPRB, 0.87622E+00_JPRB, 0.85982E+00_JPRB, 0.85022E+00_JPRB, 0.84777E+00_JPRB ,&
&  0.84478E+00_JPRB, 0.85102E+00_JPRB, 0.86786E+00_JPRB, 0.88875E+00_JPRB, 0.89745E+00_JPRB, 0.90062E+00_JPRB, 0.90473E+00_JPRB ,&
&  0.91398E+00_JPRB, 0.91954E+00_JPRB, 0.92580E+00_JPRB, 0.88541E+00_JPRB, 0.92713E+00_JPRB, 0.92821E+00_JPRB, 0.91018E+00_JPRB ,&
&  0.88705E+00_JPRB, 0.85906E+00_JPRB, 0.85105E+00_JPRB, 0.71493E+00_JPRB, 0.78299E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.88198E+00_JPRB, 0.91524E+00_JPRB, 0.91238E+00_JPRB, 0.90834E+00_JPRB, 0.90234E+00_JPRB, 0.88501E+00_JPRB, 0.86363E+00_JPRB ,&
&  0.84227E+00_JPRB, 0.83106E+00_JPRB, 0.81499E+00_JPRB, 0.79433E+00_JPRB, 0.78236E+00_JPRB, 0.77679E+00_JPRB, 0.77203E+00_JPRB ,&
&  0.77241E+00_JPRB, 0.77219E+00_JPRB, 0.77272E+00_JPRB, 0.77217E+00_JPRB, 0.76637E+00_JPRB, 0.76121E+00_JPRB, 0.75106E+00_JPRB ,&
&  0.74511E+00_JPRB, 0.74635E+00_JPRB, 0.74853E+00_JPRB, 0.75359E+00_JPRB, 0.76157E+00_JPRB, 0.77028E+00_JPRB, 0.77611E+00_JPRB ,&
&  0.77085E+00_JPRB, 0.66067E+00_JPRB, 0.68673E+00_JPRB, 0.75383E+00_JPRB, 0.82628E+00_JPRB, 0.89288E+00_JPRB, 0.91663E+00_JPRB ,&
&  0.93057E+00_JPRB, 0.93633E+00_JPRB, 0.93297E+00_JPRB, 0.92485E+00_JPRB, 0.91222E+00_JPRB, 0.90467E+00_JPRB, 0.88119E+00_JPRB ,&
&  0.86818E+00_JPRB, 0.86660E+00_JPRB, 0.87508E+00_JPRB, 0.87975E+00_JPRB, 0.87792E+00_JPRB, 0.87187E+00_JPRB, 0.86938E+00_JPRB ,&
&  0.86128E+00_JPRB, 0.83777E+00_JPRB, 0.81118E+00_JPRB, 0.79057E+00_JPRB, 0.78446E+00_JPRB, 0.81088E+00_JPRB, 0.84130E+00_JPRB ,&
&  0.86531E+00_JPRB, 0.85291E+00_JPRB, 0.83347E+00_JPRB, 0.82453E+00_JPRB, 0.82681E+00_JPRB, 0.84395E+00_JPRB, 0.86925E+00_JPRB ,&
&  0.87792E+00_JPRB, 0.87987E+00_JPRB, 0.89189E+00_JPRB, 0.89367E+00_JPRB, 0.89550E+00_JPRB, 0.89321E+00_JPRB, 0.89043E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.87428E+00_JPRB, 0.86962E+00_JPRB, 0.86426E+00_JPRB, 0.86389E+00_JPRB, 0.85695E+00_JPRB, 0.84569E+00_JPRB ,&
&  0.83296E+00_JPRB, 0.84359E+00_JPRB, 0.87627E+00_JPRB, 0.90632E+00_JPRB, 0.90642E+00_JPRB, 0.89780E+00_JPRB, 0.89322E+00_JPRB ,&
&  0.89938E+00_JPRB, 0.89463E+00_JPRB, 0.88875E+00_JPRB, 0.93452E+00_JPRB, 0.93347E+00_JPRB, 0.92667E+00_JPRB, 0.92383E+00_JPRB ,&
&  0.92499E+00_JPRB, 0.91285E+00_JPRB, 0.89950E+00_JPRB, 0.88330E+00_JPRB, 0.82296E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.96764E+00_JPRB, 0.96051E+00_JPRB, 0.94846E+00_JPRB, 0.94050E+00_JPRB, 0.92369E+00_JPRB, 0.89265E+00_JPRB, 0.86164E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.81845E+00_JPRB, 0.78577E+00_JPRB, 0.77081E+00_JPRB, 0.76874E+00_JPRB, 0.77147E+00_JPRB, 0.76971E+00_JPRB ,&
&  0.76704E+00_JPRB, 0.77355E+00_JPRB, 0.77128E+00_JPRB, 0.77555E+00_JPRB, 0.77221E+00_JPRB, 0.77354E+00_JPRB, 0.75894E+00_JPRB ,&
&  0.75092E+00_JPRB, 0.74526E+00_JPRB, 0.74440E+00_JPRB, 0.75145E+00_JPRB, 0.75740E+00_JPRB, 0.76169E+00_JPRB, 0.76565E+00_JPRB ,&
&  0.75032E+00_JPRB, 0.67147E+00_JPRB, 0.74456E+00_JPRB, 0.85609E+00_JPRB, 0.92852E+00_JPRB, 0.98994E+00_JPRB, 0.97612E+00_JPRB ,&
&  0.96783E+00_JPRB, 0.95980E+00_JPRB, 0.97235E+00_JPRB, 0.96371E+00_JPRB, 0.93784E+00_JPRB, 0.91731E+00_JPRB, 0.90714E+00_JPRB ,&
&  0.91213E+00_JPRB, 0.91175E+00_JPRB, 0.91404E+00_JPRB, 0.91151E+00_JPRB, 0.90435E+00_JPRB, 0.89595E+00_JPRB, 0.89583E+00_JPRB ,&
&  0.88215E+00_JPRB, 0.84787E+00_JPRB, 0.81291E+00_JPRB, 0.78902E+00_JPRB, 0.79710E+00_JPRB, 0.82598E+00_JPRB, 0.86790E+00_JPRB ,&
&  0.89401E+00_JPRB, 0.88618E+00_JPRB, 0.87651E+00_JPRB, 0.87115E+00_JPRB, 0.87205E+00_JPRB, 0.88720E+00_JPRB, 0.89488E+00_JPRB ,&
&  0.90142E+00_JPRB, 0.89785E+00_JPRB, 0.89625E+00_JPRB, 0.89070E+00_JPRB, 0.88617E+00_JPRB, 0.88462E+00_JPRB, 0.87907E+00_JPRB ,&
&  0.87213E+00_JPRB, 0.87281E+00_JPRB, 0.86358E+00_JPRB, 0.86229E+00_JPRB, 0.85276E+00_JPRB, 0.83754E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.83263E+00_JPRB, 0.86012E+00_JPRB, 0.88024E+00_JPRB, 0.90638E+00_JPRB, 0.91282E+00_JPRB, 0.90227E+00_JPRB, 0.90420E+00_JPRB ,&
&  0.89762E+00_JPRB, 0.88949E+00_JPRB, 0.84562E+00_JPRB, 0.88886E+00_JPRB, 0.98849E+00_JPRB, 0.98112E+00_JPRB, 0.97374E+00_JPRB ,&
&  0.97299E+00_JPRB, 0.95743E+00_JPRB, 0.91934E+00_JPRB, 0.89109E+00_JPRB, 0.94785E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.89831E+00_JPRB, 0.88533E+00_JPRB, 0.89362E+00_JPRB, 0.89852E+00_JPRB, 0.88564E+00_JPRB, 0.85551E+00_JPRB, 0.83137E+00_JPRB ,&
&  0.82324E+00_JPRB, 0.79574E+00_JPRB, 0.77077E+00_JPRB, 0.75537E+00_JPRB, 0.76061E+00_JPRB, 0.76474E+00_JPRB, 0.76377E+00_JPRB ,&
&  0.75391E+00_JPRB, 0.75348E+00_JPRB, 0.74386E+00_JPRB, 0.73866E+00_JPRB, 0.73891E+00_JPRB, 0.74243E+00_JPRB, 0.73857E+00_JPRB ,&
&  0.74626E+00_JPRB, 0.74937E+00_JPRB, 0.75474E+00_JPRB, 0.76233E+00_JPRB, 0.75973E+00_JPRB, 0.76337E+00_JPRB, 0.75422E+00_JPRB ,&
&  0.72923E+00_JPRB, 0.76435E+00_JPRB, 0.88718E+00_JPRB, 0.10636E+01_JPRB, 0.11127E+01_JPRB, 0.10898E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.99875E+00_JPRB, 0.10136E+01_JPRB, 0.10023E+01_JPRB, 0.98408E+00_JPRB, 0.97489E+00_JPRB, 0.96415E+00_JPRB ,&
&  0.95246E+00_JPRB, 0.93874E+00_JPRB, 0.93403E+00_JPRB, 0.91879E+00_JPRB, 0.89737E+00_JPRB, 0.88941E+00_JPRB, 0.89149E+00_JPRB ,&
&  0.85131E+00_JPRB, 0.82171E+00_JPRB, 0.79775E+00_JPRB, 0.77854E+00_JPRB, 0.81825E+00_JPRB, 0.88922E+00_JPRB, 0.95820E+00_JPRB ,&
&  0.93651E+00_JPRB, 0.93321E+00_JPRB, 0.92998E+00_JPRB, 0.92723E+00_JPRB, 0.94208E+00_JPRB, 0.92217E+00_JPRB, 0.91050E+00_JPRB ,&
&  0.88890E+00_JPRB, 0.89215E+00_JPRB, 0.89629E+00_JPRB, 0.90611E+00_JPRB, 0.90469E+00_JPRB, 0.90208E+00_JPRB, 0.87321E+00_JPRB ,&
&  0.85850E+00_JPRB, 0.84602E+00_JPRB, 0.83891E+00_JPRB, 0.83432E+00_JPRB, 0.83406E+00_JPRB, 0.82076E+00_JPRB, 0.81038E+00_JPRB ,&
&  0.82102E+00_JPRB, 0.83676E+00_JPRB, 0.88600E+00_JPRB, 0.94878E+00_JPRB, 0.95122E+00_JPRB, 0.92715E+00_JPRB, 0.91142E+00_JPRB ,&
&  0.90093E+00_JPRB, 0.88568E+00_JPRB, 0.85793E+00_JPRB, 0.92049E+00_JPRB, 0.96797E+00_JPRB, 0.10457E+01_JPRB, 0.10216E+01_JPRB ,&
&  0.97964E+00_JPRB, 0.92380E+00_JPRB, 0.89414E+00_JPRB, 0.88109E+00_JPRB, 0.90613E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.85021E+00_JPRB, 0.84968E+00_JPRB, 0.86301E+00_JPRB, 0.87135E+00_JPRB, 0.85707E+00_JPRB, 0.82873E+00_JPRB, 0.80270E+00_JPRB ,&
&  0.79414E+00_JPRB, 0.79259E+00_JPRB, 0.78941E+00_JPRB, 0.77073E+00_JPRB, 0.75597E+00_JPRB, 0.74829E+00_JPRB, 0.74519E+00_JPRB ,&
&  0.74550E+00_JPRB, 0.73564E+00_JPRB, 0.72296E+00_JPRB, 0.71038E+00_JPRB, 0.71469E+00_JPRB, 0.72632E+00_JPRB, 0.74507E+00_JPRB ,&
&  0.74524E+00_JPRB, 0.74624E+00_JPRB, 0.75308E+00_JPRB, 0.76082E+00_JPRB, 0.77047E+00_JPRB, 0.76242E+00_JPRB, 0.74827E+00_JPRB ,&
&  0.74335E+00_JPRB, 0.91155E+00_JPRB, 0.11613E+01_JPRB, 0.12797E+01_JPRB, 0.12435E+01_JPRB, 0.11327E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.10412E+01_JPRB, 0.10717E+01_JPRB, 0.10570E+01_JPRB, 0.10289E+01_JPRB, 0.10055E+01_JPRB, 0.98822E+00_JPRB ,&
&  0.96255E+00_JPRB, 0.93429E+00_JPRB, 0.90559E+00_JPRB, 0.89544E+00_JPRB, 0.88802E+00_JPRB, 0.88855E+00_JPRB, 0.86761E+00_JPRB ,&
&  0.84122E+00_JPRB, 0.81807E+00_JPRB, 0.79776E+00_JPRB, 0.78650E+00_JPRB, 0.82240E+00_JPRB, 0.83132E+00_JPRB, 0.94605E+00_JPRB ,&
&  0.11158E+01_JPRB, 0.10336E+01_JPRB, 0.10013E+01_JPRB, 0.10035E+01_JPRB, 0.10010E+01_JPRB, 0.98042E+00_JPRB, 0.96055E+00_JPRB ,&
&  0.95826E+00_JPRB, 0.96885E+00_JPRB, 0.10009E+01_JPRB, 0.10046E+01_JPRB, 0.96804E+00_JPRB, 0.91764E+00_JPRB, 0.89426E+00_JPRB ,&
&  0.88588E+00_JPRB, 0.87592E+00_JPRB, 0.86802E+00_JPRB, 0.85721E+00_JPRB, 0.84648E+00_JPRB, 0.82547E+00_JPRB, 0.81861E+00_JPRB ,&
&  0.82643E+00_JPRB, 0.85954E+00_JPRB, 0.93250E+00_JPRB, 0.96113E+00_JPRB, 0.98657E+00_JPRB, 0.99296E+00_JPRB, 0.94153E+00_JPRB ,&
&  0.87706E+00_JPRB, 0.84937E+00_JPRB, 0.85334E+00_JPRB, 0.90054E+00_JPRB, 0.98128E+00_JPRB, 0.10603E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.94849E+00_JPRB, 0.88743E+00_JPRB, 0.85569E+00_JPRB, 0.84079E+00_JPRB, 0.86143E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.81399E+00_JPRB, 0.80613E+00_JPRB, 0.80993E+00_JPRB, 0.82356E+00_JPRB, 0.81090E+00_JPRB, 0.79040E+00_JPRB, 0.76807E+00_JPRB ,&
&  0.78981E+00_JPRB, 0.80276E+00_JPRB, 0.80822E+00_JPRB, 0.77742E+00_JPRB, 0.75683E+00_JPRB, 0.73318E+00_JPRB, 0.71886E+00_JPRB ,&
&  0.71779E+00_JPRB, 0.70960E+00_JPRB, 0.69802E+00_JPRB, 0.69388E+00_JPRB, 0.70110E+00_JPRB, 0.71068E+00_JPRB, 0.73009E+00_JPRB ,&
&  0.72424E+00_JPRB, 0.73209E+00_JPRB, 0.75651E+00_JPRB, 0.77057E+00_JPRB, 0.78194E+00_JPRB, 0.76469E+00_JPRB, 0.75288E+00_JPRB ,&
&  0.77850E+00_JPRB, 0.84523E+00_JPRB, 0.13123E+01_JPRB, 0.15471E+01_JPRB, 0.12826E+01_JPRB, 0.11132E+01_JPRB, 0.10161E+01_JPRB ,&
&  0.10461E+01_JPRB, 0.10773E+01_JPRB, 0.10987E+01_JPRB, 0.10715E+01_JPRB, 0.10534E+01_JPRB, 0.10166E+01_JPRB, 0.98618E+00_JPRB ,&
&  0.94962E+00_JPRB, 0.92486E+00_JPRB, 0.91215E+00_JPRB, 0.91193E+00_JPRB, 0.90291E+00_JPRB, 0.88587E+00_JPRB, 0.85886E+00_JPRB ,&
&  0.84331E+00_JPRB, 0.82826E+00_JPRB, 0.80820E+00_JPRB, 0.81424E+00_JPRB, 0.88050E+00_JPRB, 0.99132E+00_JPRB, 0.99549E+00_JPRB ,&
&  0.99977E+00_JPRB, 0.11282E+01_JPRB, 0.10776E+01_JPRB, 0.10778E+01_JPRB, 0.10584E+01_JPRB, 0.10451E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.99082E+00_JPRB, 0.10145E+01_JPRB, 0.10080E+01_JPRB, 0.97231E+00_JPRB, 0.93485E+00_JPRB, 0.91854E+00_JPRB, 0.89865E+00_JPRB ,&
&  0.89282E+00_JPRB, 0.90312E+00_JPRB, 0.90236E+00_JPRB, 0.88244E+00_JPRB, 0.86441E+00_JPRB, 0.84597E+00_JPRB, 0.84372E+00_JPRB ,&
&  0.86218E+00_JPRB, 0.88694E+00_JPRB, 0.94555E+00_JPRB, 0.10012E+01_JPRB, 0.98573E+00_JPRB, 0.95106E+00_JPRB, 0.91373E+00_JPRB ,&
&  0.88098E+00_JPRB, 0.85701E+00_JPRB, 0.85168E+00_JPRB, 0.91554E+00_JPRB, 0.94427E+00_JPRB, 0.10095E+01_JPRB, 0.96297E+00_JPRB ,&
&  0.88766E+00_JPRB, 0.83653E+00_JPRB, 0.81331E+00_JPRB, 0.80907E+00_JPRB, 0.81863E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.80196E+00_JPRB, 0.78227E+00_JPRB, 0.78857E+00_JPRB, 0.79419E+00_JPRB, 0.78976E+00_JPRB, 0.77008E+00_JPRB, 0.75868E+00_JPRB ,&
&  0.77832E+00_JPRB, 0.78619E+00_JPRB, 0.78385E+00_JPRB, 0.75666E+00_JPRB, 0.74069E+00_JPRB, 0.71546E+00_JPRB, 0.69666E+00_JPRB ,&
&  0.69667E+00_JPRB, 0.69927E+00_JPRB, 0.69272E+00_JPRB, 0.69389E+00_JPRB, 0.69915E+00_JPRB, 0.71725E+00_JPRB, 0.72999E+00_JPRB ,&
&  0.74291E+00_JPRB, 0.75488E+00_JPRB, 0.77437E+00_JPRB, 0.78650E+00_JPRB, 0.78291E+00_JPRB, 0.76894E+00_JPRB, 0.76425E+00_JPRB ,&
&  0.84095E+00_JPRB, 0.19274E+01_JPRB, 0.12365E+01_JPRB, 0.16953E+01_JPRB, 0.12635E+01_JPRB, 0.10591E+01_JPRB, 0.87198E+00_JPRB ,&
&  0.90932E+00_JPRB, 0.10748E+01_JPRB, 0.11338E+01_JPRB, 0.10907E+01_JPRB, 0.10733E+01_JPRB, 0.10441E+01_JPRB, 0.98791E+00_JPRB ,&
&  0.95309E+00_JPRB, 0.95029E+00_JPRB, 0.92641E+00_JPRB, 0.91778E+00_JPRB, 0.90266E+00_JPRB, 0.88374E+00_JPRB, 0.87420E+00_JPRB ,&
&  0.86166E+00_JPRB, 0.84951E+00_JPRB, 0.83816E+00_JPRB, 0.90868E+00_JPRB, 0.95917E+00_JPRB, 0.10985E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.98133E+00_JPRB, 0.11850E+01_JPRB, 0.11261E+01_JPRB, 0.10904E+01_JPRB, 0.10865E+01_JPRB, 0.10575E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.10062E+01_JPRB, 0.96081E+00_JPRB, 0.93917E+00_JPRB, 0.94174E+00_JPRB, 0.93506E+00_JPRB, 0.90609E+00_JPRB ,&
&  0.87662E+00_JPRB, 0.88004E+00_JPRB, 0.89260E+00_JPRB, 0.88923E+00_JPRB, 0.87452E+00_JPRB, 0.84843E+00_JPRB, 0.84539E+00_JPRB ,&
&  0.86818E+00_JPRB, 0.88379E+00_JPRB, 0.91765E+00_JPRB, 0.99569E+00_JPRB, 0.96929E+00_JPRB, 0.90961E+00_JPRB, 0.86999E+00_JPRB ,&
&  0.88181E+00_JPRB, 0.86863E+00_JPRB, 0.87131E+00_JPRB, 0.90730E+00_JPRB, 0.93258E+00_JPRB, 0.96166E+00_JPRB, 0.87987E+00_JPRB ,&
&  0.83683E+00_JPRB, 0.80380E+00_JPRB, 0.79042E+00_JPRB, 0.79040E+00_JPRB, 0.80435E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.82223E+00_JPRB, 0.79451E+00_JPRB, 0.77273E+00_JPRB, 0.77539E+00_JPRB, 0.77805E+00_JPRB, 0.76771E+00_JPRB, 0.75055E+00_JPRB ,&
&  0.74512E+00_JPRB, 0.73683E+00_JPRB, 0.74943E+00_JPRB, 0.73710E+00_JPRB, 0.72868E+00_JPRB, 0.71465E+00_JPRB, 0.70421E+00_JPRB ,&
&  0.70403E+00_JPRB, 0.71011E+00_JPRB, 0.69705E+00_JPRB, 0.70208E+00_JPRB, 0.73076E+00_JPRB, 0.75629E+00_JPRB, 0.76612E+00_JPRB ,&
&  0.77768E+00_JPRB, 0.79101E+00_JPRB, 0.79528E+00_JPRB, 0.79705E+00_JPRB, 0.78652E+00_JPRB, 0.78715E+00_JPRB, 0.79432E+00_JPRB ,&
&  0.89490E+00_JPRB, 0.20271E+01_JPRB, 0.15176E+01_JPRB, 0.17349E+01_JPRB, 0.12809E+01_JPRB, 0.10313E+01_JPRB, 0.81819E+00_JPRB ,&
&  0.83713E+00_JPRB, 0.10282E+01_JPRB, 0.11099E+01_JPRB, 0.11466E+01_JPRB, 0.10957E+01_JPRB, 0.10434E+01_JPRB, 0.99232E+00_JPRB ,&
&  0.96168E+00_JPRB, 0.95881E+00_JPRB, 0.94028E+00_JPRB, 0.92578E+00_JPRB, 0.90027E+00_JPRB, 0.89872E+00_JPRB, 0.89601E+00_JPRB ,&
&  0.88969E+00_JPRB, 0.89320E+00_JPRB, 0.91622E+00_JPRB, 0.95796E+00_JPRB, 0.11238E+01_JPRB, 0.12208E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.12144E+01_JPRB, 0.11819E+01_JPRB, 0.11085E+01_JPRB, 0.10784E+01_JPRB, 0.10354E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.98568E+00_JPRB, 0.95986E+00_JPRB, 0.93548E+00_JPRB, 0.92532E+00_JPRB, 0.93843E+00_JPRB, 0.93563E+00_JPRB, 0.89452E+00_JPRB ,&
&  0.86883E+00_JPRB, 0.87821E+00_JPRB, 0.88792E+00_JPRB, 0.89195E+00_JPRB, 0.87888E+00_JPRB, 0.86138E+00_JPRB, 0.86776E+00_JPRB ,&
&  0.88950E+00_JPRB, 0.91233E+00_JPRB, 0.92455E+00_JPRB, 0.97861E+00_JPRB, 0.95368E+00_JPRB, 0.90839E+00_JPRB, 0.86126E+00_JPRB ,&
&  0.87062E+00_JPRB, 0.88176E+00_JPRB, 0.88717E+00_JPRB, 0.89161E+00_JPRB, 0.90594E+00_JPRB, 0.89348E+00_JPRB, 0.83335E+00_JPRB ,&
&  0.82928E+00_JPRB, 0.83087E+00_JPRB, 0.83793E+00_JPRB, 0.82331E+00_JPRB, 0.84200E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.80242E+00_JPRB, 0.76167E+00_JPRB, 0.75307E+00_JPRB, 0.76412E+00_JPRB, 0.75465E+00_JPRB, 0.73811E+00_JPRB, 0.71118E+00_JPRB ,&
&  0.68718E+00_JPRB, 0.70670E+00_JPRB, 0.73590E+00_JPRB, 0.75192E+00_JPRB, 0.75952E+00_JPRB, 0.76555E+00_JPRB, 0.74475E+00_JPRB ,&
&  0.71613E+00_JPRB, 0.71739E+00_JPRB, 0.73273E+00_JPRB, 0.76151E+00_JPRB, 0.78221E+00_JPRB, 0.80294E+00_JPRB, 0.83424E+00_JPRB ,&
&  0.84338E+00_JPRB, 0.84019E+00_JPRB, 0.82924E+00_JPRB, 0.81890E+00_JPRB, 0.81598E+00_JPRB, 0.82975E+00_JPRB, 0.85931E+00_JPRB ,&
&  0.10548E+01_JPRB, 0.25896E+01_JPRB, 0.25725E+01_JPRB, 0.17640E+01_JPRB, 0.13164E+01_JPRB, 0.10452E+01_JPRB, 0.87792E+00_JPRB ,&
&  0.82652E+00_JPRB, 0.82445E+00_JPRB, 0.11335E+01_JPRB, 0.11657E+01_JPRB, 0.10933E+01_JPRB, 0.10467E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10158E+01_JPRB, 0.10098E+01_JPRB, 0.97726E+00_JPRB, 0.95184E+00_JPRB, 0.93154E+00_JPRB, 0.94015E+00_JPRB, 0.95446E+00_JPRB ,&
&  0.95944E+00_JPRB, 0.98489E+00_JPRB, 0.10709E+01_JPRB, 0.11653E+01_JPRB, 0.11852E+01_JPRB, 0.12505E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11344E+01_JPRB, 0.11887E+01_JPRB, 0.11653E+01_JPRB, 0.11209E+01_JPRB, 0.10210E+01_JPRB, 0.10573E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.97933E+00_JPRB, 0.93408E+00_JPRB, 0.93062E+00_JPRB, 0.94202E+00_JPRB, 0.95117E+00_JPRB, 0.95084E+00_JPRB, 0.92716E+00_JPRB ,&
&  0.90919E+00_JPRB, 0.90664E+00_JPRB, 0.90384E+00_JPRB, 0.90152E+00_JPRB, 0.89738E+00_JPRB, 0.89877E+00_JPRB, 0.92537E+00_JPRB ,&
&  0.94345E+00_JPRB, 0.97629E+00_JPRB, 0.98572E+00_JPRB, 0.10276E+01_JPRB, 0.10076E+01_JPRB, 0.94279E+00_JPRB, 0.88340E+00_JPRB ,&
&  0.88086E+00_JPRB, 0.91235E+00_JPRB, 0.92212E+00_JPRB, 0.91685E+00_JPRB, 0.89522E+00_JPRB, 0.86089E+00_JPRB, 0.85005E+00_JPRB ,&
&  0.86500E+00_JPRB, 0.88973E+00_JPRB, 0.88441E+00_JPRB, 0.83593E+00_JPRB, 0.84814E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.77513E+00_JPRB, 0.73854E+00_JPRB, 0.72925E+00_JPRB, 0.71991E+00_JPRB, 0.70239E+00_JPRB, 0.69460E+00_JPRB, 0.67156E+00_JPRB ,&
&  0.68804E+00_JPRB, 0.74651E+00_JPRB, 0.81530E+00_JPRB, 0.84285E+00_JPRB, 0.85785E+00_JPRB, 0.85491E+00_JPRB, 0.82548E+00_JPRB ,&
&  0.78875E+00_JPRB, 0.79276E+00_JPRB, 0.80803E+00_JPRB, 0.81851E+00_JPRB, 0.83637E+00_JPRB, 0.85061E+00_JPRB, 0.87623E+00_JPRB ,&
&  0.89746E+00_JPRB, 0.87833E+00_JPRB, 0.85267E+00_JPRB, 0.84444E+00_JPRB, 0.88515E+00_JPRB, 0.94323E+00_JPRB, 0.10207E+01_JPRB ,&
&  0.14619E+01_JPRB, 0.29437E+01_JPRB, 0.15765E+01_JPRB, 0.13900E+01_JPRB, 0.12529E+01_JPRB, 0.10914E+01_JPRB, 0.95954E+00_JPRB ,&
&  0.91890E+00_JPRB, 0.83308E+00_JPRB, 0.97442E+00_JPRB, 0.11512E+01_JPRB, 0.11164E+01_JPRB, 0.10827E+01_JPRB, 0.10835E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10662E+01_JPRB, 0.10556E+01_JPRB, 0.10183E+01_JPRB, 0.10121E+01_JPRB, 0.10119E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10841E+01_JPRB, 0.11804E+01_JPRB, 0.11809E+01_JPRB, 0.12762E+01_JPRB, 0.12842E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.94367E+00_JPRB, 0.10070E+01_JPRB, 0.11100E+01_JPRB, 0.11816E+01_JPRB, 0.11372E+01_JPRB, 0.98004E+00_JPRB, 0.10333E+01_JPRB ,&
&  0.95819E+00_JPRB, 0.94539E+00_JPRB, 0.95649E+00_JPRB, 0.95847E+00_JPRB, 0.98023E+00_JPRB, 0.97926E+00_JPRB, 0.98190E+00_JPRB ,&
&  0.96499E+00_JPRB, 0.94754E+00_JPRB, 0.93661E+00_JPRB, 0.92845E+00_JPRB, 0.92473E+00_JPRB, 0.94147E+00_JPRB, 0.97290E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.10461E+01_JPRB, 0.10988E+01_JPRB, 0.11160E+01_JPRB, 0.10967E+01_JPRB, 0.10323E+01_JPRB, 0.96040E+00_JPRB ,&
&  0.94705E+00_JPRB, 0.96374E+00_JPRB, 0.92623E+00_JPRB, 0.89129E+00_JPRB, 0.89368E+00_JPRB, 0.88557E+00_JPRB, 0.93608E+00_JPRB ,&
&  0.10216E+01_JPRB, 0.11579E+01_JPRB, 0.12584E+01_JPRB, 0.81758E+00_JPRB, 0.81151E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.70194E+00_JPRB, 0.69982E+00_JPRB, 0.67309E+00_JPRB, 0.65436E+00_JPRB, 0.64785E+00_JPRB, 0.67036E+00_JPRB, 0.71567E+00_JPRB ,&
&  0.78647E+00_JPRB, 0.87577E+00_JPRB, 0.95195E+00_JPRB, 0.97728E+00_JPRB, 0.96445E+00_JPRB, 0.96649E+00_JPRB, 0.91416E+00_JPRB ,&
&  0.85954E+00_JPRB, 0.84987E+00_JPRB, 0.86223E+00_JPRB, 0.87259E+00_JPRB, 0.88581E+00_JPRB, 0.90655E+00_JPRB, 0.93110E+00_JPRB ,&
&  0.94450E+00_JPRB, 0.95279E+00_JPRB, 0.95953E+00_JPRB, 0.10155E+01_JPRB, 0.11067E+01_JPRB, 0.12489E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.21001E+01_JPRB, 0.20364E+01_JPRB, 0.11143E+01_JPRB, 0.11798E+01_JPRB, 0.10142E+01_JPRB, 0.10266E+01_JPRB, 0.97518E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.98204E+00_JPRB, 0.10789E+01_JPRB, 0.12081E+01_JPRB, 0.11929E+01_JPRB, 0.11849E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11650E+01_JPRB, 0.11449E+01_JPRB, 0.11217E+01_JPRB, 0.10902E+01_JPRB, 0.10772E+01_JPRB, 0.10886E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11967E+01_JPRB, 0.13005E+01_JPRB, 0.12290E+01_JPRB, 0.13814E+01_JPRB, 0.13381E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.95891E+00_JPRB, 0.92041E+00_JPRB, 0.10773E+01_JPRB, 0.12176E+01_JPRB, 0.11854E+01_JPRB, 0.96351E+00_JPRB, 0.99917E+00_JPRB ,&
&  0.94734E+00_JPRB, 0.94675E+00_JPRB, 0.94861E+00_JPRB, 0.96647E+00_JPRB, 0.10014E+01_JPRB, 0.10060E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.98437E+00_JPRB, 0.96093E+00_JPRB, 0.95165E+00_JPRB, 0.94967E+00_JPRB, 0.95725E+00_JPRB, 0.99090E+00_JPRB, 0.10105E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.10712E+01_JPRB, 0.11188E+01_JPRB, 0.11715E+01_JPRB, 0.11612E+01_JPRB, 0.10983E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.99780E+00_JPRB, 0.99995E+00_JPRB, 0.92798E+00_JPRB, 0.88221E+00_JPRB, 0.84233E+00_JPRB, 0.85760E+00_JPRB, 0.84944E+00_JPRB ,&
&  0.85012E+00_JPRB, 0.85223E+00_JPRB, 0.81348E+00_JPRB, 0.70649E+00_JPRB, 0.70600E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.67651E+00_JPRB, 0.67956E+00_JPRB, 0.67165E+00_JPRB, 0.68786E+00_JPRB, 0.68949E+00_JPRB, 0.75132E+00_JPRB, 0.82015E+00_JPRB ,&
&  0.92445E+00_JPRB, 0.10003E+01_JPRB, 0.10456E+01_JPRB, 0.10520E+01_JPRB, 0.10301E+01_JPRB, 0.96934E+00_JPRB, 0.91250E+00_JPRB ,&
&  0.90850E+00_JPRB, 0.91072E+00_JPRB, 0.92364E+00_JPRB, 0.94408E+00_JPRB, 0.96240E+00_JPRB, 0.99230E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.11121E+01_JPRB, 0.11749E+01_JPRB, 0.12664E+01_JPRB, 0.13821E+01_JPRB, 0.15420E+01_JPRB, 0.17010E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.65693E+00_JPRB, 0.94158E+00_JPRB, 0.98430E+00_JPRB, 0.92260E+00_JPRB, 0.10455E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.10668E+01_JPRB, 0.10710E+01_JPRB, 0.12570E+01_JPRB, 0.12541E+01_JPRB, 0.12476E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.12236E+01_JPRB, 0.11983E+01_JPRB, 0.11706E+01_JPRB, 0.11669E+01_JPRB, 0.11633E+01_JPRB, 0.11886E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.13362E+01_JPRB, 0.14532E+01_JPRB, 0.12480E+01_JPRB, 0.13908E+01_JPRB, 0.14574E+01_JPRB, 0.13346E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.88058E+00_JPRB, 0.90276E+00_JPRB, 0.11894E+01_JPRB, 0.11951E+01_JPRB, 0.96263E+00_JPRB, 0.98955E+00_JPRB ,&
&  0.94884E+00_JPRB, 0.97610E+00_JPRB, 0.95612E+00_JPRB, 0.10101E+01_JPRB, 0.10355E+01_JPRB, 0.10495E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.98745E+00_JPRB, 0.95414E+00_JPRB, 0.94186E+00_JPRB, 0.94967E+00_JPRB, 0.95850E+00_JPRB, 0.98953E+00_JPRB, 0.99656E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10286E+01_JPRB, 0.10783E+01_JPRB, 0.11288E+01_JPRB, 0.11556E+01_JPRB, 0.11006E+01_JPRB, 0.10511E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.97709E+00_JPRB, 0.89552E+00_JPRB, 0.79917E+00_JPRB, 0.78508E+00_JPRB, 0.73995E+00_JPRB, 0.68625E+00_JPRB ,&
&  0.69567E+00_JPRB, 0.69264E+00_JPRB, 0.67678E+00_JPRB, 0.67434E+00_JPRB, 0.67909E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.76060E+00_JPRB, 0.75221E+00_JPRB, 0.74424E+00_JPRB, 0.74667E+00_JPRB, 0.75477E+00_JPRB, 0.80099E+00_JPRB, 0.87580E+00_JPRB ,&
&  0.94627E+00_JPRB, 0.98796E+00_JPRB, 0.10257E+01_JPRB, 0.10408E+01_JPRB, 0.10394E+01_JPRB, 0.99227E+00_JPRB, 0.98267E+00_JPRB ,&
&  0.97067E+00_JPRB, 0.96571E+00_JPRB, 0.98319E+00_JPRB, 0.10125E+01_JPRB, 0.10362E+01_JPRB, 0.10621E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.11743E+01_JPRB, 0.12252E+01_JPRB, 0.13071E+01_JPRB, 0.13589E+01_JPRB, 0.13725E+01_JPRB, 0.12974E+01_JPRB ,&
&  0.59144E+00_JPRB, 0.70168E+00_JPRB, 0.11064E+01_JPRB, 0.11045E+01_JPRB, 0.11628E+01_JPRB, 0.12275E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.12079E+01_JPRB, 0.10661E+01_JPRB, 0.10923E+01_JPRB, 0.13066E+01_JPRB, 0.12931E+01_JPRB, 0.13145E+01_JPRB, 0.13165E+01_JPRB ,&
&  0.12940E+01_JPRB, 0.12681E+01_JPRB, 0.12482E+01_JPRB, 0.12515E+01_JPRB, 0.12732E+01_JPRB, 0.13011E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.13806E+01_JPRB, 0.14438E+01_JPRB, 0.15281E+01_JPRB, 0.12745E+01_JPRB, 0.13511E+01_JPRB, 0.14805E+01_JPRB, 0.16583E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.84916E+00_JPRB, 0.85714E+00_JPRB, 0.11326E+01_JPRB, 0.11276E+01_JPRB, 0.11546E+01_JPRB, 0.97464E+00_JPRB ,&
&  0.96231E+00_JPRB, 0.91549E+00_JPRB, 0.93059E+00_JPRB, 0.98198E+00_JPRB, 0.10217E+01_JPRB, 0.10434E+01_JPRB, 0.10159E+01_JPRB ,&
&  0.95781E+00_JPRB, 0.92317E+00_JPRB, 0.90659E+00_JPRB, 0.91133E+00_JPRB, 0.93005E+00_JPRB, 0.94234E+00_JPRB, 0.93578E+00_JPRB ,&
&  0.92872E+00_JPRB, 0.93278E+00_JPRB, 0.97381E+00_JPRB, 0.10010E+01_JPRB, 0.10328E+01_JPRB, 0.10692E+01_JPRB, 0.11040E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.10637E+01_JPRB, 0.11647E+01_JPRB, 0.10848E+01_JPRB, 0.98396E+00_JPRB, 0.84732E+00_JPRB, 0.71317E+00_JPRB ,&
&  0.72933E+00_JPRB, 0.73781E+00_JPRB, 0.73378E+00_JPRB, 0.74027E+00_JPRB, 0.75760E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.84836E+00_JPRB, 0.83127E+00_JPRB, 0.81872E+00_JPRB, 0.80709E+00_JPRB, 0.81859E+00_JPRB, 0.85155E+00_JPRB, 0.88884E+00_JPRB ,&
&  0.92485E+00_JPRB, 0.96625E+00_JPRB, 0.10231E+01_JPRB, 0.10384E+01_JPRB, 0.10723E+01_JPRB, 0.10654E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10252E+01_JPRB, 0.10423E+01_JPRB, 0.10612E+01_JPRB, 0.10624E+01_JPRB, 0.10786E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.11636E+01_JPRB, 0.12074E+01_JPRB, 0.12885E+01_JPRB, 0.12993E+01_JPRB, 0.12777E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.68098E+00_JPRB, 0.11244E+01_JPRB, 0.12228E+01_JPRB, 0.11958E+01_JPRB, 0.12461E+01_JPRB, 0.12236E+01_JPRB, 0.11898E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.11640E+01_JPRB, 0.11609E+01_JPRB, 0.12573E+01_JPRB, 0.13379E+01_JPRB, 0.13656E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.13353E+01_JPRB, 0.13426E+01_JPRB, 0.13810E+01_JPRB, 0.14102E+01_JPRB, 0.14431E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.14982E+01_JPRB, 0.15055E+01_JPRB, 0.15730E+01_JPRB, 0.14026E+01_JPRB, 0.13175E+01_JPRB, 0.13644E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.94270E+00_JPRB, 0.65618E+00_JPRB, 0.78301E+00_JPRB, 0.10105E+01_JPRB, 0.92991E+00_JPRB, 0.94063E+00_JPRB, 0.88560E+00_JPRB ,&
&  0.79490E+00_JPRB, 0.76797E+00_JPRB, 0.83939E+00_JPRB, 0.90633E+00_JPRB, 0.96225E+00_JPRB, 0.10038E+01_JPRB, 0.98568E+00_JPRB ,&
&  0.93154E+00_JPRB, 0.88782E+00_JPRB, 0.85445E+00_JPRB, 0.84810E+00_JPRB, 0.87038E+00_JPRB, 0.88235E+00_JPRB, 0.86928E+00_JPRB ,&
&  0.86391E+00_JPRB, 0.85437E+00_JPRB, 0.89690E+00_JPRB, 0.93012E+00_JPRB, 0.97679E+00_JPRB, 0.10322E+01_JPRB, 0.11770E+01_JPRB ,&
&  0.14177E+01_JPRB, 0.16407E+01_JPRB, 0.19141E+01_JPRB, 0.18816E+01_JPRB, 0.17152E+01_JPRB, 0.12283E+01_JPRB, 0.82082E+00_JPRB ,&
&  0.83802E+00_JPRB, 0.85737E+00_JPRB, 0.85931E+00_JPRB, 0.86715E+00_JPRB, 0.85762E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.98079E+00_JPRB, 0.96448E+00_JPRB, 0.96090E+00_JPRB, 0.95692E+00_JPRB, 0.95463E+00_JPRB, 0.96223E+00_JPRB, 0.98437E+00_JPRB ,&
&  0.99391E+00_JPRB, 0.10277E+01_JPRB, 0.10789E+01_JPRB, 0.11095E+01_JPRB, 0.11493E+01_JPRB, 0.11466E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.11250E+01_JPRB, 0.11498E+01_JPRB, 0.11561E+01_JPRB, 0.11136E+01_JPRB, 0.10840E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.11069E+01_JPRB, 0.11253E+01_JPRB, 0.11434E+01_JPRB, 0.11781E+01_JPRB, 0.11690E+01_JPRB, 0.10840E+01_JPRB, 0.72695E+00_JPRB ,&
&  0.10697E+01_JPRB, 0.12777E+01_JPRB, 0.12955E+01_JPRB, 0.12291E+01_JPRB, 0.11973E+01_JPRB, 0.11279E+01_JPRB, 0.11292E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.12282E+01_JPRB, 0.12194E+01_JPRB, 0.11775E+01_JPRB, 0.13067E+01_JPRB, 0.13163E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.13284E+01_JPRB, 0.13326E+01_JPRB, 0.13458E+01_JPRB, 0.13657E+01_JPRB, 0.13795E+01_JPRB, 0.14047E+01_JPRB, 0.14187E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.13757E+01_JPRB, 0.13935E+01_JPRB, 0.12949E+01_JPRB, 0.12571E+01_JPRB, 0.12511E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.68352E+00_JPRB, 0.54352E+00_JPRB, 0.75026E+00_JPRB, 0.96047E+00_JPRB, 0.85278E+00_JPRB, 0.81037E+00_JPRB, 0.82045E+00_JPRB ,&
&  0.78005E+00_JPRB, 0.72295E+00_JPRB, 0.70932E+00_JPRB, 0.76124E+00_JPRB, 0.83591E+00_JPRB, 0.89362E+00_JPRB, 0.90855E+00_JPRB ,&
&  0.87521E+00_JPRB, 0.84853E+00_JPRB, 0.84515E+00_JPRB, 0.85964E+00_JPRB, 0.89336E+00_JPRB, 0.97232E+00_JPRB, 0.10818E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.81123E+00_JPRB, 0.92962E+00_JPRB, 0.99646E+00_JPRB, 0.11182E+01_JPRB, 0.12868E+01_JPRB, 0.15865E+01_JPRB ,&
&  0.20599E+01_JPRB, 0.25714E+01_JPRB, 0.25382E+01_JPRB, 0.21249E+01_JPRB, 0.26899E+01_JPRB, 0.17931E+01_JPRB, 0.94506E+00_JPRB ,&
&  0.97204E+00_JPRB, 0.98978E+00_JPRB, 0.10058E+01_JPRB, 0.99865E+00_JPRB, 0.99231E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10044E+01_JPRB, 0.10054E+01_JPRB, 0.99642E+00_JPRB, 0.10249E+01_JPRB, 0.10210E+01_JPRB, 0.10348E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10743E+01_JPRB, 0.10953E+01_JPRB, 0.11312E+01_JPRB, 0.11383E+01_JPRB, 0.11522E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.12396E+01_JPRB, 0.12353E+01_JPRB, 0.12172E+01_JPRB, 0.11781E+01_JPRB, 0.11492E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10783E+01_JPRB, 0.10780E+01_JPRB, 0.10856E+01_JPRB, 0.10867E+01_JPRB, 0.10941E+01_JPRB, 0.73829E+00_JPRB ,&
&  0.11774E+01_JPRB, 0.13200E+01_JPRB, 0.12395E+01_JPRB, 0.11685E+01_JPRB, 0.10833E+01_JPRB, 0.10864E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.11599E+01_JPRB, 0.12145E+01_JPRB, 0.12161E+01_JPRB, 0.11980E+01_JPRB, 0.11696E+01_JPRB, 0.11784E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11582E+01_JPRB, 0.11479E+01_JPRB, 0.11336E+01_JPRB, 0.11240E+01_JPRB, 0.11297E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10885E+01_JPRB, 0.10559E+01_JPRB, 0.10514E+01_JPRB, 0.10750E+01_JPRB, 0.10559E+01_JPRB, 0.92152E+00_JPRB, 0.73169E+00_JPRB ,&
&  0.49542E+00_JPRB, 0.49323E+00_JPRB, 0.68652E+00_JPRB, 0.95352E+00_JPRB, 0.87335E+00_JPRB, 0.79521E+00_JPRB, 0.74213E+00_JPRB ,&
&  0.72898E+00_JPRB, 0.68342E+00_JPRB, 0.62990E+00_JPRB, 0.62473E+00_JPRB, 0.64875E+00_JPRB, 0.70123E+00_JPRB, 0.70633E+00_JPRB ,&
&  0.72082E+00_JPRB, 0.71803E+00_JPRB, 0.69790E+00_JPRB, 0.68495E+00_JPRB, 0.72146E+00_JPRB, 0.72277E+00_JPRB, 0.63926E+00_JPRB ,&
&  0.75550E+00_JPRB, 0.70270E+00_JPRB, 0.74656E+00_JPRB, 0.83897E+00_JPRB, 0.91420E+00_JPRB, 0.11035E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.18288E+01_JPRB, 0.24413E+01_JPRB, 0.37394E+01_JPRB, 0.29292E+01_JPRB, 0.20713E+01_JPRB, 0.11002E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10506E+01_JPRB, 0.10553E+01_JPRB, 0.10327E+01_JPRB, 0.10150E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.91981E+00_JPRB, 0.92922E+00_JPRB, 0.94601E+00_JPRB, 0.97086E+00_JPRB, 0.98338E+00_JPRB, 0.10044E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10511E+01_JPRB, 0.10776E+01_JPRB, 0.11067E+01_JPRB, 0.11320E+01_JPRB, 0.11570E+01_JPRB, 0.11802E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12171E+01_JPRB, 0.12227E+01_JPRB, 0.12138E+01_JPRB, 0.11913E+01_JPRB, 0.11780E+01_JPRB, 0.11553E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11362E+01_JPRB, 0.11425E+01_JPRB, 0.11452E+01_JPRB, 0.11500E+01_JPRB, 0.11904E+01_JPRB, 0.85207E+00_JPRB ,&
&  0.12274E+01_JPRB, 0.12879E+01_JPRB, 0.12217E+01_JPRB, 0.10717E+01_JPRB, 0.99745E+00_JPRB, 0.98998E+00_JPRB, 0.10053E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.11063E+01_JPRB, 0.11467E+01_JPRB, 0.10667E+01_JPRB, 0.97994E+00_JPRB, 0.98250E+00_JPRB, 0.96935E+00_JPRB ,&
&  0.96646E+00_JPRB, 0.95503E+00_JPRB, 0.94176E+00_JPRB, 0.92189E+00_JPRB, 0.90301E+00_JPRB, 0.88252E+00_JPRB, 0.85658E+00_JPRB ,&
&  0.81783E+00_JPRB, 0.77569E+00_JPRB, 0.70589E+00_JPRB, 0.70788E+00_JPRB, 0.69520E+00_JPRB, 0.61006E+00_JPRB, 0.48561E+00_JPRB ,&
&  0.41631E+00_JPRB, 0.54738E+00_JPRB, 0.69826E+00_JPRB, 0.96277E+00_JPRB, 0.89444E+00_JPRB, 0.84121E+00_JPRB, 0.77748E+00_JPRB ,&
&  0.67504E+00_JPRB, 0.62610E+00_JPRB, 0.58542E+00_JPRB, 0.57330E+00_JPRB, 0.57328E+00_JPRB, 0.58775E+00_JPRB, 0.58331E+00_JPRB ,&
&  0.56571E+00_JPRB, 0.53921E+00_JPRB, 0.53506E+00_JPRB, 0.53700E+00_JPRB, 0.55398E+00_JPRB, 0.49012E+00_JPRB, 0.61252E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.52689E+00_JPRB, 0.68703E+00_JPRB, 0.49377E+00_JPRB, 0.64518E+00_JPRB, 0.73632E+00_JPRB, 0.85887E+00_JPRB ,&
&  0.89910E+00_JPRB, 0.11956E+01_JPRB, 0.15020E+01_JPRB, 0.13245E+01_JPRB, 0.11721E+01_JPRB, 0.10490E+01_JPRB, 0.99930E+00_JPRB ,&
&  0.98319E+00_JPRB, 0.96794E+00_JPRB, 0.95853E+00_JPRB, 0.94389E+00_JPRB, 0.92616E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.86741E+00_JPRB, 0.87407E+00_JPRB, 0.88500E+00_JPRB, 0.90052E+00_JPRB, 0.91649E+00_JPRB, 0.93113E+00_JPRB, 0.94258E+00_JPRB ,&
&  0.95129E+00_JPRB, 0.96515E+00_JPRB, 0.98667E+00_JPRB, 0.10128E+01_JPRB, 0.10231E+01_JPRB, 0.10393E+01_JPRB, 0.10432E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10877E+01_JPRB, 0.10967E+01_JPRB, 0.11066E+01_JPRB, 0.11133E+01_JPRB, 0.11261E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11579E+01_JPRB, 0.11951E+01_JPRB, 0.12498E+01_JPRB, 0.12946E+01_JPRB, 0.13550E+01_JPRB, 0.14538E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.12891E+01_JPRB, 0.13020E+01_JPRB, 0.11737E+01_JPRB, 0.99585E+00_JPRB, 0.93755E+00_JPRB, 0.89576E+00_JPRB, 0.86888E+00_JPRB ,&
&  0.85995E+00_JPRB, 0.82110E+00_JPRB, 0.82868E+00_JPRB, 0.82267E+00_JPRB, 0.81488E+00_JPRB, 0.84057E+00_JPRB, 0.83407E+00_JPRB ,&
&  0.83359E+00_JPRB, 0.83654E+00_JPRB, 0.84107E+00_JPRB, 0.83800E+00_JPRB, 0.84211E+00_JPRB, 0.82939E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.76168E+00_JPRB, 0.72032E+00_JPRB, 0.65575E+00_JPRB, 0.63054E+00_JPRB, 0.61750E+00_JPRB, 0.56415E+00_JPRB, 0.51480E+00_JPRB ,&
&  0.54572E+00_JPRB, 0.59688E+00_JPRB, 0.65379E+00_JPRB, 0.90778E+00_JPRB, 0.91099E+00_JPRB, 0.90185E+00_JPRB, 0.76599E+00_JPRB ,&
&  0.70258E+00_JPRB, 0.61642E+00_JPRB, 0.57787E+00_JPRB, 0.57984E+00_JPRB, 0.57596E+00_JPRB, 0.57068E+00_JPRB, 0.54935E+00_JPRB ,&
&  0.50763E+00_JPRB, 0.47774E+00_JPRB, 0.48728E+00_JPRB, 0.51191E+00_JPRB, 0.55467E+00_JPRB, 0.53295E+00_JPRB, 0.57712E+00_JPRB ,&
&  0.53825E+00_JPRB, 0.39342E+00_JPRB, 0.45555E+00_JPRB, 0.55113E+00_JPRB, 0.55658E+00_JPRB, 0.61493E+00_JPRB, 0.74654E+00_JPRB ,&
&  0.95619E+00_JPRB, 0.10196E+01_JPRB, 0.10340E+01_JPRB, 0.96631E+00_JPRB, 0.94725E+00_JPRB, 0.94018E+00_JPRB, 0.91797E+00_JPRB ,&
&  0.90659E+00_JPRB, 0.89921E+00_JPRB, 0.89202E+00_JPRB, 0.87834E+00_JPRB, 0.86838E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.82899E+00_JPRB, 0.83041E+00_JPRB, 0.83501E+00_JPRB, 0.84323E+00_JPRB, 0.85285E+00_JPRB, 0.86338E+00_JPRB, 0.86882E+00_JPRB ,&
&  0.86540E+00_JPRB, 0.86843E+00_JPRB, 0.87760E+00_JPRB, 0.89223E+00_JPRB, 0.90527E+00_JPRB, 0.91655E+00_JPRB, 0.92857E+00_JPRB ,&
&  0.92618E+00_JPRB, 0.92656E+00_JPRB, 0.94073E+00_JPRB, 0.95849E+00_JPRB, 0.96816E+00_JPRB, 0.96705E+00_JPRB, 0.96863E+00_JPRB ,&
&  0.96383E+00_JPRB, 0.97225E+00_JPRB, 0.99646E+00_JPRB, 0.10117E+01_JPRB, 0.10347E+01_JPRB, 0.10955E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.12492E+01_JPRB, 0.10793E+01_JPRB, 0.93870E+00_JPRB, 0.91971E+00_JPRB, 0.87897E+00_JPRB, 0.84860E+00_JPRB ,&
&  0.81477E+00_JPRB, 0.78371E+00_JPRB, 0.74098E+00_JPRB, 0.73528E+00_JPRB, 0.71688E+00_JPRB, 0.73606E+00_JPRB, 0.74573E+00_JPRB ,&
&  0.77229E+00_JPRB, 0.78584E+00_JPRB, 0.80913E+00_JPRB, 0.82539E+00_JPRB, 0.86450E+00_JPRB, 0.86422E+00_JPRB, 0.82234E+00_JPRB ,&
&  0.81325E+00_JPRB, 0.84280E+00_JPRB, 0.80710E+00_JPRB, 0.82098E+00_JPRB, 0.79762E+00_JPRB, 0.75644E+00_JPRB, 0.73468E+00_JPRB ,&
&  0.71255E+00_JPRB, 0.70970E+00_JPRB, 0.81395E+00_JPRB, 0.83412E+00_JPRB, 0.90635E+00_JPRB, 0.92977E+00_JPRB, 0.84549E+00_JPRB ,&
&  0.77330E+00_JPRB, 0.69313E+00_JPRB, 0.68424E+00_JPRB, 0.67753E+00_JPRB, 0.69895E+00_JPRB, 0.68729E+00_JPRB, 0.67429E+00_JPRB ,&
&  0.62111E+00_JPRB, 0.56965E+00_JPRB, 0.58133E+00_JPRB, 0.58612E+00_JPRB, 0.58534E+00_JPRB, 0.68555E+00_JPRB, 0.61757E+00_JPRB ,&
&  0.56943E+00_JPRB, 0.54248E+00_JPRB, 0.44632E+00_JPRB, 0.60464E+00_JPRB, 0.61597E+00_JPRB, 0.67529E+00_JPRB, 0.75307E+00_JPRB ,&
&  0.86283E+00_JPRB, 0.95570E+00_JPRB, 0.10103E+01_JPRB, 0.95837E+00_JPRB, 0.87866E+00_JPRB, 0.85879E+00_JPRB, 0.84396E+00_JPRB ,&
&  0.84271E+00_JPRB, 0.84032E+00_JPRB, 0.83679E+00_JPRB, 0.83133E+00_JPRB, 0.82301E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.77032E+00_JPRB, 0.77133E+00_JPRB, 0.78476E+00_JPRB, 0.78556E+00_JPRB, 0.78559E+00_JPRB, 0.79363E+00_JPRB, 0.79733E+00_JPRB ,&
&  0.79303E+00_JPRB, 0.80115E+00_JPRB, 0.81202E+00_JPRB, 0.82371E+00_JPRB, 0.83726E+00_JPRB, 0.84919E+00_JPRB, 0.85523E+00_JPRB ,&
&  0.84570E+00_JPRB, 0.83521E+00_JPRB, 0.85527E+00_JPRB, 0.87305E+00_JPRB, 0.87473E+00_JPRB, 0.87608E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.95497E+00_JPRB, 0.98235E+00_JPRB, 0.98000E+00_JPRB, 0.95103E+00_JPRB, 0.97002E+00_JPRB, 0.11576E+01_JPRB, 0.18056E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.11065E+01_JPRB, 0.10230E+01_JPRB, 0.92530E+00_JPRB, 0.92259E+00_JPRB, 0.94111E+00_JPRB, 0.92454E+00_JPRB ,&
&  0.87411E+00_JPRB, 0.86822E+00_JPRB, 0.86132E+00_JPRB, 0.85336E+00_JPRB, 0.86081E+00_JPRB, 0.86948E+00_JPRB, 0.87025E+00_JPRB ,&
&  0.85249E+00_JPRB, 0.85095E+00_JPRB, 0.86524E+00_JPRB, 0.87423E+00_JPRB, 0.91295E+00_JPRB, 0.10337E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10099E+01_JPRB, 0.95791E+00_JPRB, 0.94861E+00_JPRB, 0.94630E+00_JPRB, 0.90459E+00_JPRB, 0.85965E+00_JPRB ,&
&  0.84902E+00_JPRB, 0.80808E+00_JPRB, 0.70166E+00_JPRB, 0.79511E+00_JPRB, 0.88558E+00_JPRB, 0.97477E+00_JPRB, 0.93708E+00_JPRB ,&
&  0.90017E+00_JPRB, 0.82283E+00_JPRB, 0.80110E+00_JPRB, 0.83855E+00_JPRB, 0.88905E+00_JPRB, 0.87847E+00_JPRB, 0.84688E+00_JPRB ,&
&  0.77159E+00_JPRB, 0.71034E+00_JPRB, 0.70346E+00_JPRB, 0.66168E+00_JPRB, 0.64651E+00_JPRB, 0.57981E+00_JPRB, 0.60886E+00_JPRB ,&
&  0.61129E+00_JPRB, 0.64461E+00_JPRB, 0.55656E+00_JPRB, 0.70773E+00_JPRB, 0.70070E+00_JPRB, 0.71152E+00_JPRB, 0.73105E+00_JPRB ,&
&  0.74965E+00_JPRB, 0.77347E+00_JPRB, 0.77588E+00_JPRB, 0.84976E+00_JPRB, 0.83400E+00_JPRB, 0.81940E+00_JPRB, 0.81763E+00_JPRB ,&
&  0.78194E+00_JPRB, 0.76983E+00_JPRB, 0.76578E+00_JPRB, 0.77141E+00_JPRB, 0.77104E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.69759E+00_JPRB, 0.69422E+00_JPRB, 0.69376E+00_JPRB, 0.70887E+00_JPRB, 0.72248E+00_JPRB, 0.71519E+00_JPRB, 0.72407E+00_JPRB ,&
&  0.70593E+00_JPRB, 0.69154E+00_JPRB, 0.70310E+00_JPRB, 0.70985E+00_JPRB, 0.72641E+00_JPRB, 0.74033E+00_JPRB, 0.74975E+00_JPRB ,&
&  0.75066E+00_JPRB, 0.74920E+00_JPRB, 0.72497E+00_JPRB, 0.74154E+00_JPRB, 0.83050E+00_JPRB, 0.97321E+00_JPRB, 0.12274E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.15993E+01_JPRB, 0.16584E+01_JPRB, 0.16224E+01_JPRB, 0.14795E+01_JPRB, 0.12834E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.14479E+01_JPRB, 0.10881E+01_JPRB, 0.11242E+01_JPRB, 0.10565E+01_JPRB, 0.10581E+01_JPRB, 0.10288E+01_JPRB, 0.10111E+01_JPRB ,&
&  0.97565E+00_JPRB, 0.97056E+00_JPRB, 0.98045E+00_JPRB, 0.10115E+01_JPRB, 0.10455E+01_JPRB, 0.10826E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11055E+01_JPRB, 0.11426E+01_JPRB, 0.11056E+01_JPRB, 0.10756E+01_JPRB, 0.11063E+01_JPRB, 0.11332E+01_JPRB ,&
&  0.11538E+01_JPRB, 0.11494E+01_JPRB, 0.98962E+00_JPRB, 0.96403E+00_JPRB, 0.99046E+00_JPRB, 0.90495E+00_JPRB, 0.88985E+00_JPRB ,&
&  0.92024E+00_JPRB, 0.85721E+00_JPRB, 0.66394E+00_JPRB, 0.70943E+00_JPRB, 0.79506E+00_JPRB, 0.91358E+00_JPRB, 0.10180E+01_JPRB ,&
&  0.98747E+00_JPRB, 0.93541E+00_JPRB, 0.92837E+00_JPRB, 0.97368E+00_JPRB, 0.10026E+01_JPRB, 0.99558E+00_JPRB, 0.98430E+00_JPRB ,&
&  0.96809E+00_JPRB, 0.84096E+00_JPRB, 0.77997E+00_JPRB, 0.70898E+00_JPRB, 0.67257E+00_JPRB, 0.65759E+00_JPRB, 0.63033E+00_JPRB ,&
&  0.64144E+00_JPRB, 0.65664E+00_JPRB, 0.67738E+00_JPRB, 0.67012E+00_JPRB, 0.62936E+00_JPRB, 0.75110E+00_JPRB, 0.76799E+00_JPRB ,&
&  0.76949E+00_JPRB, 0.76858E+00_JPRB, 0.77901E+00_JPRB, 0.76697E+00_JPRB, 0.74536E+00_JPRB, 0.72447E+00_JPRB, 0.70463E+00_JPRB ,&
&  0.71345E+00_JPRB, 0.72363E+00_JPRB, 0.72666E+00_JPRB, 0.71218E+00_JPRB, 0.70052E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.67370E+00_JPRB, 0.67726E+00_JPRB, 0.67596E+00_JPRB, 0.68927E+00_JPRB, 0.68627E+00_JPRB, 0.70095E+00_JPRB, 0.69066E+00_JPRB ,&
&  0.67993E+00_JPRB, 0.68095E+00_JPRB, 0.68904E+00_JPRB, 0.69728E+00_JPRB, 0.72310E+00_JPRB, 0.77398E+00_JPRB, 0.83883E+00_JPRB ,&
&  0.90691E+00_JPRB, 0.93395E+00_JPRB, 0.98856E+00_JPRB, 0.11056E+01_JPRB, 0.13297E+01_JPRB, 0.15377E+01_JPRB, 0.15663E+01_JPRB ,&
&  0.16308E+01_JPRB, 0.17357E+01_JPRB, 0.18028E+01_JPRB, 0.18370E+01_JPRB, 0.17585E+01_JPRB, 0.16308E+01_JPRB, 0.14888E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.11051E+01_JPRB, 0.10879E+01_JPRB, 0.10928E+01_JPRB, 0.11002E+01_JPRB, 0.10705E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.10235E+01_JPRB, 0.10222E+01_JPRB, 0.10381E+01_JPRB, 0.10727E+01_JPRB, 0.11182E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.11152E+01_JPRB, 0.11059E+01_JPRB, 0.11098E+01_JPRB, 0.10214E+01_JPRB, 0.10550E+01_JPRB, 0.10663E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.98407E+00_JPRB, 0.98184E+00_JPRB, 0.96014E+00_JPRB, 0.93298E+00_JPRB, 0.87736E+00_JPRB, 0.85413E+00_JPRB ,&
&  0.86371E+00_JPRB, 0.82654E+00_JPRB, 0.68516E+00_JPRB, 0.74036E+00_JPRB, 0.81927E+00_JPRB, 0.83945E+00_JPRB, 0.10707E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10088E+01_JPRB, 0.10066E+01_JPRB, 0.99577E+00_JPRB, 0.10282E+01_JPRB, 0.10282E+01_JPRB, 0.10383E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.96436E+00_JPRB, 0.87697E+00_JPRB, 0.81358E+00_JPRB, 0.75490E+00_JPRB, 0.71121E+00_JPRB, 0.69016E+00_JPRB ,&
&  0.70249E+00_JPRB, 0.72430E+00_JPRB, 0.71475E+00_JPRB, 0.70357E+00_JPRB, 0.67025E+00_JPRB, 0.75791E+00_JPRB, 0.76472E+00_JPRB ,&
&  0.78369E+00_JPRB, 0.77268E+00_JPRB, 0.75380E+00_JPRB, 0.72909E+00_JPRB, 0.71364E+00_JPRB, 0.70287E+00_JPRB, 0.69889E+00_JPRB ,&
&  0.70379E+00_JPRB, 0.70852E+00_JPRB, 0.70017E+00_JPRB, 0.69065E+00_JPRB, 0.67933E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.65128E+00_JPRB, 0.64776E+00_JPRB, 0.64716E+00_JPRB, 0.67614E+00_JPRB, 0.67440E+00_JPRB, 0.66568E+00_JPRB, 0.67791E+00_JPRB ,&
&  0.69398E+00_JPRB, 0.71489E+00_JPRB, 0.74014E+00_JPRB, 0.76485E+00_JPRB, 0.79855E+00_JPRB, 0.84945E+00_JPRB, 0.90734E+00_JPRB ,&
&  0.97420E+00_JPRB, 0.10547E+01_JPRB, 0.11879E+01_JPRB, 0.13573E+01_JPRB, 0.15552E+01_JPRB, 0.16572E+01_JPRB, 0.16516E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.15603E+01_JPRB, 0.15966E+01_JPRB, 0.15703E+01_JPRB, 0.13131E+01_JPRB, 0.12993E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.11756E+01_JPRB, 0.11055E+01_JPRB, 0.10636E+01_JPRB, 0.10258E+01_JPRB, 0.10288E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.10070E+01_JPRB, 0.10057E+01_JPRB, 0.10115E+01_JPRB, 0.10333E+01_JPRB, 0.10749E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10823E+01_JPRB, 0.10583E+01_JPRB, 0.10073E+01_JPRB, 0.10057E+01_JPRB, 0.99954E+00_JPRB, 0.98906E+00_JPRB ,&
&  0.99985E+00_JPRB, 0.96013E+00_JPRB, 0.96125E+00_JPRB, 0.95533E+00_JPRB, 0.90419E+00_JPRB, 0.82697E+00_JPRB, 0.81412E+00_JPRB ,&
&  0.83179E+00_JPRB, 0.86047E+00_JPRB, 0.71609E+00_JPRB, 0.87772E+00_JPRB, 0.99269E+00_JPRB, 0.95346E+00_JPRB, 0.99639E+00_JPRB ,&
&  0.99225E+00_JPRB, 0.10000E+01_JPRB, 0.10135E+01_JPRB, 0.10198E+01_JPRB, 0.10166E+01_JPRB, 0.10133E+01_JPRB, 0.96739E+00_JPRB ,&
&  0.10629E+01_JPRB, 0.97162E+00_JPRB, 0.89408E+00_JPRB, 0.86093E+00_JPRB, 0.82402E+00_JPRB, 0.76318E+00_JPRB, 0.70894E+00_JPRB ,&
&  0.68590E+00_JPRB, 0.88365E+00_JPRB, 0.94672E+00_JPRB, 0.10637E+01_JPRB, 0.11207E+01_JPRB, 0.76621E+00_JPRB, 0.80344E+00_JPRB ,&
&  0.79369E+00_JPRB, 0.77974E+00_JPRB, 0.75467E+00_JPRB, 0.72663E+00_JPRB, 0.72177E+00_JPRB, 0.72149E+00_JPRB, 0.71863E+00_JPRB ,&
&  0.71186E+00_JPRB, 0.69724E+00_JPRB, 0.68771E+00_JPRB, 0.66836E+00_JPRB, 0.65865E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.66898E+00_JPRB, 0.66375E+00_JPRB, 0.66182E+00_JPRB, 0.67573E+00_JPRB, 0.68771E+00_JPRB, 0.68906E+00_JPRB, 0.68287E+00_JPRB ,&
&  0.69097E+00_JPRB, 0.71729E+00_JPRB, 0.74874E+00_JPRB, 0.76488E+00_JPRB, 0.77045E+00_JPRB, 0.78012E+00_JPRB, 0.82329E+00_JPRB ,&
&  0.89715E+00_JPRB, 0.10070E+01_JPRB, 0.11449E+01_JPRB, 0.12560E+01_JPRB, 0.13536E+01_JPRB, 0.14301E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.15649E+01_JPRB, 0.16056E+01_JPRB, 0.15454E+01_JPRB, 0.11982E+01_JPRB, 0.10804E+01_JPRB, 0.12557E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.11226E+01_JPRB, 0.10947E+01_JPRB, 0.10665E+01_JPRB, 0.10234E+01_JPRB, 0.98136E+00_JPRB, 0.98985E+00_JPRB, 0.98580E+00_JPRB ,&
&  0.96275E+00_JPRB, 0.95694E+00_JPRB, 0.95879E+00_JPRB, 0.96728E+00_JPRB, 0.98049E+00_JPRB, 0.99168E+00_JPRB, 0.98506E+00_JPRB ,&
&  0.99829E+00_JPRB, 0.99586E+00_JPRB, 0.10019E+01_JPRB, 0.99917E+00_JPRB, 0.98008E+00_JPRB, 0.98861E+00_JPRB, 0.98165E+00_JPRB ,&
&  0.98400E+00_JPRB, 0.96626E+00_JPRB, 0.95259E+00_JPRB, 0.95571E+00_JPRB, 0.93959E+00_JPRB, 0.83280E+00_JPRB, 0.81450E+00_JPRB ,&
&  0.83288E+00_JPRB, 0.83306E+00_JPRB, 0.77995E+00_JPRB, 0.88030E+00_JPRB, 0.66006E+00_JPRB, 0.78590E+00_JPRB, 0.94128E+00_JPRB ,&
&  0.97424E+00_JPRB, 0.98724E+00_JPRB, 0.99239E+00_JPRB, 0.99872E+00_JPRB, 0.99606E+00_JPRB, 0.92584E+00_JPRB, 0.93562E+00_JPRB ,&
&  0.94367E+00_JPRB, 0.91136E+00_JPRB, 0.88141E+00_JPRB, 0.87323E+00_JPRB, 0.85509E+00_JPRB, 0.81943E+00_JPRB, 0.77597E+00_JPRB ,&
&  0.71317E+00_JPRB, 0.85486E+00_JPRB, 0.86074E+00_JPRB, 0.86646E+00_JPRB, 0.84858E+00_JPRB, 0.74147E+00_JPRB, 0.77530E+00_JPRB ,&
&  0.79333E+00_JPRB, 0.76764E+00_JPRB, 0.73646E+00_JPRB, 0.73165E+00_JPRB, 0.72662E+00_JPRB, 0.72565E+00_JPRB, 0.71169E+00_JPRB ,&
&  0.70923E+00_JPRB, 0.70606E+00_JPRB, 0.70069E+00_JPRB, 0.68940E+00_JPRB, 0.67816E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.66853E+00_JPRB, 0.66693E+00_JPRB, 0.67111E+00_JPRB, 0.68472E+00_JPRB, 0.71075E+00_JPRB, 0.73241E+00_JPRB, 0.69258E+00_JPRB ,&
&  0.67963E+00_JPRB, 0.67382E+00_JPRB, 0.70174E+00_JPRB, 0.71310E+00_JPRB, 0.72137E+00_JPRB, 0.72974E+00_JPRB, 0.76939E+00_JPRB ,&
&  0.84254E+00_JPRB, 0.94840E+00_JPRB, 0.10530E+01_JPRB, 0.11223E+01_JPRB, 0.11958E+01_JPRB, 0.12970E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.12774E+01_JPRB, 0.11105E+01_JPRB, 0.14769E+01_JPRB, 0.15286E+01_JPRB, 0.13671E+01_JPRB, 0.12747E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11167E+01_JPRB, 0.11160E+01_JPRB, 0.10593E+01_JPRB, 0.95163E+00_JPRB, 0.93914E+00_JPRB, 0.93567E+00_JPRB ,&
&  0.92452E+00_JPRB, 0.90811E+00_JPRB, 0.90748E+00_JPRB, 0.91693E+00_JPRB, 0.92880E+00_JPRB, 0.91801E+00_JPRB, 0.90097E+00_JPRB ,&
&  0.89716E+00_JPRB, 0.89802E+00_JPRB, 0.90728E+00_JPRB, 0.90551E+00_JPRB, 0.91544E+00_JPRB, 0.93400E+00_JPRB, 0.94550E+00_JPRB ,&
&  0.95447E+00_JPRB, 0.94378E+00_JPRB, 0.96087E+00_JPRB, 0.98960E+00_JPRB, 0.10091E+01_JPRB, 0.91580E+00_JPRB, 0.91491E+00_JPRB ,&
&  0.88338E+00_JPRB, 0.84065E+00_JPRB, 0.81121E+00_JPRB, 0.85461E+00_JPRB, 0.68906E+00_JPRB, 0.77395E+00_JPRB, 0.82352E+00_JPRB ,&
&  0.95648E+00_JPRB, 0.96540E+00_JPRB, 0.97904E+00_JPRB, 0.10177E+01_JPRB, 0.10452E+01_JPRB, 0.96883E+00_JPRB, 0.93626E+00_JPRB ,&
&  0.92115E+00_JPRB, 0.91728E+00_JPRB, 0.89699E+00_JPRB, 0.88966E+00_JPRB, 0.84103E+00_JPRB, 0.84147E+00_JPRB, 0.86079E+00_JPRB ,&
&  0.89710E+00_JPRB, 0.88923E+00_JPRB, 0.87847E+00_JPRB, 0.84655E+00_JPRB, 0.78649E+00_JPRB, 0.75476E+00_JPRB, 0.76031E+00_JPRB ,&
&  0.76966E+00_JPRB, 0.78106E+00_JPRB, 0.73152E+00_JPRB, 0.70152E+00_JPRB, 0.69581E+00_JPRB, 0.71121E+00_JPRB, 0.69840E+00_JPRB ,&
&  0.69683E+00_JPRB, 0.68819E+00_JPRB, 0.68841E+00_JPRB, 0.68289E+00_JPRB, 0.67810E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.67313E+00_JPRB, 0.69007E+00_JPRB, 0.71941E+00_JPRB, 0.75498E+00_JPRB, 0.79173E+00_JPRB, 0.84820E+00_JPRB, 0.81447E+00_JPRB ,&
&  0.78112E+00_JPRB, 0.61608E+00_JPRB, 0.63163E+00_JPRB, 0.64314E+00_JPRB, 0.65826E+00_JPRB, 0.65882E+00_JPRB, 0.67937E+00_JPRB ,&
&  0.75296E+00_JPRB, 0.86318E+00_JPRB, 0.95694E+00_JPRB, 0.10350E+01_JPRB, 0.11157E+01_JPRB, 0.12231E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.93031E+00_JPRB, 0.10989E+01_JPRB, 0.16679E+01_JPRB, 0.16196E+01_JPRB, 0.14913E+01_JPRB, 0.13806E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11180E+01_JPRB, 0.11423E+01_JPRB, 0.10460E+01_JPRB, 0.92541E+00_JPRB, 0.90397E+00_JPRB, 0.90105E+00_JPRB ,&
&  0.88449E+00_JPRB, 0.86872E+00_JPRB, 0.85798E+00_JPRB, 0.86074E+00_JPRB, 0.86335E+00_JPRB, 0.85351E+00_JPRB, 0.84077E+00_JPRB ,&
&  0.83681E+00_JPRB, 0.84074E+00_JPRB, 0.84285E+00_JPRB, 0.83928E+00_JPRB, 0.86334E+00_JPRB, 0.88925E+00_JPRB, 0.90418E+00_JPRB ,&
&  0.91453E+00_JPRB, 0.94266E+00_JPRB, 0.96621E+00_JPRB, 0.96716E+00_JPRB, 0.92706E+00_JPRB, 0.99764E+00_JPRB, 0.95175E+00_JPRB ,&
&  0.93997E+00_JPRB, 0.88786E+00_JPRB, 0.89590E+00_JPRB, 0.75957E+00_JPRB, 0.76448E+00_JPRB, 0.81741E+00_JPRB, 0.87092E+00_JPRB ,&
&  0.90225E+00_JPRB, 0.94509E+00_JPRB, 0.96090E+00_JPRB, 0.10151E+01_JPRB, 0.10739E+01_JPRB, 0.10475E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.96589E+00_JPRB, 0.10045E+01_JPRB, 0.94213E+00_JPRB, 0.88556E+00_JPRB, 0.80372E+00_JPRB, 0.78562E+00_JPRB, 0.83054E+00_JPRB ,&
&  0.89789E+00_JPRB, 0.92482E+00_JPRB, 0.93584E+00_JPRB, 0.92906E+00_JPRB, 0.87306E+00_JPRB, 0.83514E+00_JPRB, 0.76462E+00_JPRB ,&
&  0.74284E+00_JPRB, 0.69921E+00_JPRB, 0.68397E+00_JPRB, 0.68402E+00_JPRB, 0.67501E+00_JPRB, 0.68762E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.66716E+00_JPRB, 0.66730E+00_JPRB, 0.66679E+00_JPRB, 0.66435E+00_JPRB, 0.66503E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.63294E+00_JPRB, 0.64739E+00_JPRB, 0.67344E+00_JPRB, 0.68683E+00_JPRB, 0.68088E+00_JPRB, 0.69596E+00_JPRB, 0.72726E+00_JPRB ,&
&  0.72439E+00_JPRB, 0.58464E+00_JPRB, 0.56676E+00_JPRB, 0.57359E+00_JPRB, 0.59293E+00_JPRB, 0.60460E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.64655E+00_JPRB, 0.73705E+00_JPRB, 0.87667E+00_JPRB, 0.93700E+00_JPRB, 0.98591E+00_JPRB, 0.10957E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.85206E+00_JPRB, 0.14900E+01_JPRB, 0.14593E+01_JPRB, 0.16260E+01_JPRB, 0.15772E+01_JPRB, 0.14676E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.10483E+01_JPRB, 0.10304E+01_JPRB, 0.98252E+00_JPRB, 0.93537E+00_JPRB, 0.88647E+00_JPRB, 0.87543E+00_JPRB ,&
&  0.85493E+00_JPRB, 0.82639E+00_JPRB, 0.80512E+00_JPRB, 0.80597E+00_JPRB, 0.80256E+00_JPRB, 0.79576E+00_JPRB, 0.79251E+00_JPRB ,&
&  0.79997E+00_JPRB, 0.80780E+00_JPRB, 0.80702E+00_JPRB, 0.82276E+00_JPRB, 0.84212E+00_JPRB, 0.87387E+00_JPRB, 0.89602E+00_JPRB ,&
&  0.88130E+00_JPRB, 0.86838E+00_JPRB, 0.93561E+00_JPRB, 0.96500E+00_JPRB, 0.97626E+00_JPRB, 0.10285E+01_JPRB, 0.96958E+00_JPRB ,&
&  0.99040E+00_JPRB, 0.91247E+00_JPRB, 0.92367E+00_JPRB, 0.75628E+00_JPRB, 0.76647E+00_JPRB, 0.82158E+00_JPRB, 0.85868E+00_JPRB ,&
&  0.85467E+00_JPRB, 0.93328E+00_JPRB, 0.96416E+00_JPRB, 0.10239E+01_JPRB, 0.10837E+01_JPRB, 0.11508E+01_JPRB, 0.10959E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.10837E+01_JPRB, 0.10753E+01_JPRB, 0.92599E+00_JPRB, 0.84751E+00_JPRB, 0.77738E+00_JPRB, 0.82634E+00_JPRB ,&
&  0.88661E+00_JPRB, 0.90385E+00_JPRB, 0.93604E+00_JPRB, 0.97598E+00_JPRB, 0.94321E+00_JPRB, 0.95692E+00_JPRB, 0.90025E+00_JPRB ,&
&  0.79575E+00_JPRB, 0.73975E+00_JPRB, 0.67532E+00_JPRB, 0.66923E+00_JPRB, 0.67090E+00_JPRB, 0.66127E+00_JPRB, 0.66423E+00_JPRB ,&
&  0.65810E+00_JPRB, 0.64677E+00_JPRB, 0.64097E+00_JPRB, 0.63367E+00_JPRB, 0.62550E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.60159E+00_JPRB, 0.61057E+00_JPRB, 0.62753E+00_JPRB, 0.63673E+00_JPRB, 0.63619E+00_JPRB, 0.62479E+00_JPRB, 0.65089E+00_JPRB ,&
&  0.67672E+00_JPRB, 0.57503E+00_JPRB, 0.53015E+00_JPRB, 0.51413E+00_JPRB, 0.52357E+00_JPRB, 0.55014E+00_JPRB, 0.57206E+00_JPRB ,&
&  0.59261E+00_JPRB, 0.63001E+00_JPRB, 0.76779E+00_JPRB, 0.85353E+00_JPRB, 0.91462E+00_JPRB, 0.10591E+01_JPRB, 0.77990E+00_JPRB ,&
&  0.85476E+00_JPRB, 0.12611E+01_JPRB, 0.14118E+01_JPRB, 0.15823E+01_JPRB, 0.16865E+01_JPRB, 0.16119E+01_JPRB, 0.14285E+01_JPRB ,&
&  0.11965E+01_JPRB, 0.10521E+01_JPRB, 0.97019E+00_JPRB, 0.91866E+00_JPRB, 0.88167E+00_JPRB, 0.85695E+00_JPRB, 0.82913E+00_JPRB ,&
&  0.81634E+00_JPRB, 0.79446E+00_JPRB, 0.76471E+00_JPRB, 0.75962E+00_JPRB, 0.75372E+00_JPRB, 0.74011E+00_JPRB, 0.75090E+00_JPRB ,&
&  0.77299E+00_JPRB, 0.78987E+00_JPRB, 0.80668E+00_JPRB, 0.79599E+00_JPRB, 0.83188E+00_JPRB, 0.86225E+00_JPRB, 0.89277E+00_JPRB ,&
&  0.86082E+00_JPRB, 0.83945E+00_JPRB, 0.95518E+00_JPRB, 0.10564E+01_JPRB, 0.11015E+01_JPRB, 0.10583E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.10065E+01_JPRB, 0.95991E+00_JPRB, 0.88349E+00_JPRB, 0.76108E+00_JPRB, 0.76611E+00_JPRB, 0.82934E+00_JPRB, 0.87704E+00_JPRB ,&
&  0.91615E+00_JPRB, 0.94995E+00_JPRB, 0.98950E+00_JPRB, 0.10099E+01_JPRB, 0.10590E+01_JPRB, 0.11636E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.12874E+01_JPRB, 0.14383E+01_JPRB, 0.12661E+01_JPRB, 0.10702E+01_JPRB, 0.92890E+00_JPRB, 0.81634E+00_JPRB, 0.79651E+00_JPRB ,&
&  0.85040E+00_JPRB, 0.84983E+00_JPRB, 0.82688E+00_JPRB, 0.88356E+00_JPRB, 0.92183E+00_JPRB, 0.10588E+01_JPRB, 0.10865E+01_JPRB ,&
&  0.93440E+00_JPRB, 0.85379E+00_JPRB, 0.76017E+00_JPRB, 0.71012E+00_JPRB, 0.68242E+00_JPRB, 0.67760E+00_JPRB, 0.67017E+00_JPRB ,&
&  0.66534E+00_JPRB, 0.65352E+00_JPRB, 0.63601E+00_JPRB, 0.62271E+00_JPRB, 0.60790E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.60180E+00_JPRB, 0.59059E+00_JPRB, 0.58921E+00_JPRB, 0.59133E+00_JPRB, 0.60356E+00_JPRB, 0.60000E+00_JPRB, 0.59826E+00_JPRB ,&
&  0.63551E+00_JPRB, 0.55277E+00_JPRB, 0.51814E+00_JPRB, 0.49330E+00_JPRB, 0.49794E+00_JPRB, 0.52244E+00_JPRB, 0.55008E+00_JPRB ,&
&  0.57141E+00_JPRB, 0.59654E+00_JPRB, 0.68228E+00_JPRB, 0.78979E+00_JPRB, 0.80092E+00_JPRB, 0.73213E+00_JPRB, 0.76067E+00_JPRB ,&
&  0.96791E+00_JPRB, 0.12350E+01_JPRB, 0.13946E+01_JPRB, 0.15723E+01_JPRB, 0.17966E+01_JPRB, 0.18917E+01_JPRB, 0.16348E+01_JPRB ,&
&  0.14183E+01_JPRB, 0.11908E+01_JPRB, 0.10162E+01_JPRB, 0.91865E+00_JPRB, 0.87413E+00_JPRB, 0.83098E+00_JPRB, 0.79891E+00_JPRB ,&
&  0.76925E+00_JPRB, 0.74705E+00_JPRB, 0.73376E+00_JPRB, 0.72336E+00_JPRB, 0.72261E+00_JPRB, 0.70465E+00_JPRB, 0.70485E+00_JPRB ,&
&  0.72698E+00_JPRB, 0.75210E+00_JPRB, 0.78128E+00_JPRB, 0.77764E+00_JPRB, 0.81342E+00_JPRB, 0.85207E+00_JPRB, 0.91176E+00_JPRB ,&
&  0.93415E+00_JPRB, 0.99128E+00_JPRB, 0.10464E+01_JPRB, 0.10647E+01_JPRB, 0.11264E+01_JPRB, 0.11402E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.96015E+00_JPRB, 0.84066E+00_JPRB, 0.80964E+00_JPRB, 0.83428E+00_JPRB, 0.88791E+00_JPRB, 0.80058E+00_JPRB ,&
&  0.75802E+00_JPRB, 0.74921E+00_JPRB, 0.82345E+00_JPRB, 0.92998E+00_JPRB, 0.10096E+01_JPRB, 0.11266E+01_JPRB, 0.13249E+01_JPRB ,&
&  0.14103E+01_JPRB, 0.91950E+00_JPRB, 0.78192E+00_JPRB, 0.88722E+00_JPRB, 0.76422E+00_JPRB, 0.64914E+00_JPRB, 0.72968E+00_JPRB ,&
&  0.78684E+00_JPRB, 0.84869E+00_JPRB, 0.85968E+00_JPRB, 0.78320E+00_JPRB, 0.83227E+00_JPRB, 0.10269E+01_JPRB, 0.12348E+01_JPRB ,&
&  0.10423E+01_JPRB, 0.96584E+00_JPRB, 0.89116E+00_JPRB, 0.81605E+00_JPRB, 0.75287E+00_JPRB, 0.71065E+00_JPRB, 0.68620E+00_JPRB ,&
&  0.67594E+00_JPRB, 0.66859E+00_JPRB, 0.65815E+00_JPRB, 0.64014E+00_JPRB, 0.61958E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.61819E+00_JPRB, 0.59919E+00_JPRB, 0.58651E+00_JPRB, 0.57638E+00_JPRB, 0.57655E+00_JPRB, 0.57289E+00_JPRB, 0.56242E+00_JPRB ,&
&  0.59579E+00_JPRB, 0.55090E+00_JPRB, 0.51270E+00_JPRB, 0.49051E+00_JPRB, 0.48929E+00_JPRB, 0.50389E+00_JPRB, 0.53355E+00_JPRB ,&
&  0.57313E+00_JPRB, 0.60129E+00_JPRB, 0.67819E+00_JPRB, 0.76274E+00_JPRB, 0.74569E+00_JPRB, 0.65114E+00_JPRB, 0.74457E+00_JPRB ,&
&  0.84165E+00_JPRB, 0.11083E+01_JPRB, 0.15372E+01_JPRB, 0.17906E+01_JPRB, 0.18538E+01_JPRB, 0.19428E+01_JPRB, 0.17807E+01_JPRB ,&
&  0.16512E+01_JPRB, 0.14320E+01_JPRB, 0.11745E+01_JPRB, 0.10093E+01_JPRB, 0.89906E+00_JPRB, 0.84974E+00_JPRB, 0.79853E+00_JPRB ,&
&  0.77502E+00_JPRB, 0.73934E+00_JPRB, 0.70806E+00_JPRB, 0.69375E+00_JPRB, 0.68708E+00_JPRB, 0.68126E+00_JPRB, 0.67081E+00_JPRB ,&
&  0.67815E+00_JPRB, 0.70086E+00_JPRB, 0.74302E+00_JPRB, 0.78149E+00_JPRB, 0.69683E+00_JPRB, 0.82935E+00_JPRB, 0.90317E+00_JPRB ,&
&  0.98730E+00_JPRB, 0.10684E+01_JPRB, 0.10605E+01_JPRB, 0.11616E+01_JPRB, 0.11974E+01_JPRB, 0.11429E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.10166E+01_JPRB, 0.90810E+00_JPRB, 0.93193E+00_JPRB, 0.93083E+00_JPRB, 0.94729E+00_JPRB, 0.69388E+00_JPRB ,&
&  0.66009E+00_JPRB, 0.76284E+00_JPRB, 0.78863E+00_JPRB, 0.78073E+00_JPRB, 0.88504E+00_JPRB, 0.10455E+01_JPRB, 0.95320E+00_JPRB ,&
&  0.53178E+00_JPRB, 0.46525E+00_JPRB, 0.49022E+00_JPRB, 0.49504E+00_JPRB, 0.48001E+00_JPRB, 0.51422E+00_JPRB, 0.74869E+00_JPRB ,&
&  0.73793E+00_JPRB, 0.81886E+00_JPRB, 0.84342E+00_JPRB, 0.80930E+00_JPRB, 0.78910E+00_JPRB, 0.87382E+00_JPRB, 0.12845E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.10315E+01_JPRB, 0.94670E+00_JPRB, 0.89148E+00_JPRB, 0.82535E+00_JPRB, 0.76503E+00_JPRB, 0.71651E+00_JPRB ,&
&  0.68406E+00_JPRB, 0.66732E+00_JPRB, 0.65896E+00_JPRB, 0.65019E+00_JPRB, 0.63600E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.61251E+00_JPRB, 0.59418E+00_JPRB, 0.58193E+00_JPRB, 0.57269E+00_JPRB, 0.56397E+00_JPRB, 0.56199E+00_JPRB, 0.54780E+00_JPRB ,&
&  0.55835E+00_JPRB, 0.55292E+00_JPRB, 0.51910E+00_JPRB, 0.49634E+00_JPRB, 0.48790E+00_JPRB, 0.50049E+00_JPRB, 0.53822E+00_JPRB ,&
&  0.56835E+00_JPRB, 0.62154E+00_JPRB, 0.74824E+00_JPRB, 0.71523E+00_JPRB, 0.72904E+00_JPRB, 0.64561E+00_JPRB, 0.68495E+00_JPRB ,&
&  0.77130E+00_JPRB, 0.94999E+00_JPRB, 0.12911E+01_JPRB, 0.15695E+01_JPRB, 0.18877E+01_JPRB, 0.20101E+01_JPRB, 0.19705E+01_JPRB ,&
&  0.18368E+01_JPRB, 0.16888E+01_JPRB, 0.14536E+01_JPRB, 0.12079E+01_JPRB, 0.10383E+01_JPRB, 0.92331E+00_JPRB, 0.84943E+00_JPRB ,&
&  0.81144E+00_JPRB, 0.78770E+00_JPRB, 0.72791E+00_JPRB, 0.69118E+00_JPRB, 0.67237E+00_JPRB, 0.66112E+00_JPRB, 0.63890E+00_JPRB ,&
&  0.63535E+00_JPRB, 0.65485E+00_JPRB, 0.69646E+00_JPRB, 0.74224E+00_JPRB, 0.69724E+00_JPRB, 0.66310E+00_JPRB, 0.81596E+00_JPRB ,&
&  0.97897E+00_JPRB, 0.11972E+01_JPRB, 0.12808E+01_JPRB, 0.14324E+01_JPRB, 0.13339E+01_JPRB, 0.12162E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.12660E+01_JPRB, 0.10140E+01_JPRB, 0.10427E+01_JPRB, 0.10989E+01_JPRB, 0.84934E+00_JPRB, 0.73237E+00_JPRB ,&
&  0.80611E+00_JPRB, 0.76972E+00_JPRB, 0.75086E+00_JPRB, 0.69432E+00_JPRB, 0.82133E+00_JPRB, 0.84643E+00_JPRB, 0.52692E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.44290E+00_JPRB, 0.42772E+00_JPRB, 0.41105E+00_JPRB, 0.41232E+00_JPRB, 0.44362E+00_JPRB, 0.57713E+00_JPRB ,&
&  0.63411E+00_JPRB, 0.72124E+00_JPRB, 0.77490E+00_JPRB, 0.81052E+00_JPRB, 0.78264E+00_JPRB, 0.87144E+00_JPRB, 0.10196E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.11489E+01_JPRB, 0.10387E+01_JPRB, 0.95374E+00_JPRB, 0.87301E+00_JPRB, 0.80444E+00_JPRB, 0.75370E+00_JPRB ,&
&  0.71508E+00_JPRB, 0.69061E+00_JPRB, 0.67167E+00_JPRB, 0.65526E+00_JPRB, 0.63311E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.62692E+00_JPRB, 0.60523E+00_JPRB, 0.58119E+00_JPRB, 0.56520E+00_JPRB, 0.55766E+00_JPRB, 0.54921E+00_JPRB, 0.53787E+00_JPRB ,&
&  0.52959E+00_JPRB, 0.54399E+00_JPRB, 0.52328E+00_JPRB, 0.50656E+00_JPRB, 0.49383E+00_JPRB, 0.49867E+00_JPRB, 0.52846E+00_JPRB ,&
&  0.56877E+00_JPRB, 0.64786E+00_JPRB, 0.70544E+00_JPRB, 0.64879E+00_JPRB, 0.61848E+00_JPRB, 0.58358E+00_JPRB, 0.57907E+00_JPRB ,&
&  0.70515E+00_JPRB, 0.82680E+00_JPRB, 0.10363E+01_JPRB, 0.13982E+01_JPRB, 0.18448E+01_JPRB, 0.20023E+01_JPRB, 0.21821E+01_JPRB ,&
&  0.20506E+01_JPRB, 0.18595E+01_JPRB, 0.16969E+01_JPRB, 0.14826E+01_JPRB, 0.12377E+01_JPRB, 0.10785E+01_JPRB, 0.97287E+00_JPRB ,&
&  0.90022E+00_JPRB, 0.85032E+00_JPRB, 0.79710E+00_JPRB, 0.73103E+00_JPRB, 0.67974E+00_JPRB, 0.65411E+00_JPRB, 0.62934E+00_JPRB ,&
&  0.61711E+00_JPRB, 0.63393E+00_JPRB, 0.66991E+00_JPRB, 0.72616E+00_JPRB, 0.78903E+00_JPRB, 0.80352E+00_JPRB, 0.79536E+00_JPRB ,&
&  0.87576E+00_JPRB, 0.10209E+01_JPRB, 0.14265E+01_JPRB, 0.14739E+01_JPRB, 0.14586E+01_JPRB, 0.14395E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.13826E+01_JPRB, 0.14932E+01_JPRB, 0.12248E+01_JPRB, 0.12917E+01_JPRB, 0.11679E+01_JPRB, 0.89034E+00_JPRB, 0.92542E+00_JPRB ,&
&  0.95852E+00_JPRB, 0.88436E+00_JPRB, 0.77962E+00_JPRB, 0.65431E+00_JPRB, 0.64526E+00_JPRB, 0.55113E+00_JPRB, 0.42475E+00_JPRB ,&
&  0.40449E+00_JPRB, 0.39246E+00_JPRB, 0.38081E+00_JPRB, 0.38334E+00_JPRB, 0.38133E+00_JPRB, 0.41879E+00_JPRB, 0.50120E+00_JPRB ,&
&  0.59706E+00_JPRB, 0.66340E+00_JPRB, 0.71968E+00_JPRB, 0.75821E+00_JPRB, 0.76137E+00_JPRB, 0.82812E+00_JPRB, 0.10564E+01_JPRB ,&
&  0.10410E+01_JPRB, 0.10563E+01_JPRB, 0.10434E+01_JPRB, 0.97861E+00_JPRB, 0.90146E+00_JPRB, 0.83473E+00_JPRB, 0.77549E+00_JPRB ,&
&  0.73332E+00_JPRB, 0.71052E+00_JPRB, 0.69301E+00_JPRB, 0.67218E+00_JPRB, 0.64996E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.62993E+00_JPRB, 0.60826E+00_JPRB, 0.58826E+00_JPRB, 0.57467E+00_JPRB, 0.56396E+00_JPRB, 0.55296E+00_JPRB, 0.53801E+00_JPRB ,&
&  0.51873E+00_JPRB, 0.52259E+00_JPRB, 0.51951E+00_JPRB, 0.49610E+00_JPRB, 0.48615E+00_JPRB, 0.49421E+00_JPRB, 0.52195E+00_JPRB ,&
&  0.56563E+00_JPRB, 0.69852E+00_JPRB, 0.59780E+00_JPRB, 0.55372E+00_JPRB, 0.57512E+00_JPRB, 0.57667E+00_JPRB, 0.53633E+00_JPRB ,&
&  0.66003E+00_JPRB, 0.77794E+00_JPRB, 0.95486E+00_JPRB, 0.13713E+01_JPRB, 0.20872E+01_JPRB, 0.22248E+01_JPRB, 0.23565E+01_JPRB ,&
&  0.24225E+01_JPRB, 0.21922E+01_JPRB, 0.19794E+01_JPRB, 0.17405E+01_JPRB, 0.14466E+01_JPRB, 0.12259E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.96980E+00_JPRB, 0.90650E+00_JPRB, 0.83189E+00_JPRB, 0.77084E+00_JPRB, 0.71715E+00_JPRB, 0.67709E+00_JPRB, 0.65265E+00_JPRB ,&
&  0.63572E+00_JPRB, 0.63687E+00_JPRB, 0.66109E+00_JPRB, 0.71377E+00_JPRB, 0.76572E+00_JPRB, 0.76773E+00_JPRB, 0.99450E+00_JPRB ,&
&  0.11678E+01_JPRB, 0.13032E+01_JPRB, 0.13896E+01_JPRB, 0.16608E+01_JPRB, 0.18299E+01_JPRB, 0.17771E+01_JPRB, 0.18619E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.14739E+01_JPRB, 0.14918E+01_JPRB, 0.14713E+01_JPRB, 0.11370E+01_JPRB, 0.11321E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.10913E+01_JPRB, 0.11273E+01_JPRB, 0.11110E+01_JPRB, 0.54947E+00_JPRB, 0.44807E+00_JPRB, 0.57037E+00_JPRB ,&
&  0.57498E+00_JPRB, 0.43859E+00_JPRB, 0.38763E+00_JPRB, 0.42786E+00_JPRB, 0.38517E+00_JPRB, 0.42743E+00_JPRB, 0.52328E+00_JPRB ,&
&  0.54008E+00_JPRB, 0.56170E+00_JPRB, 0.66644E+00_JPRB, 0.71357E+00_JPRB, 0.75433E+00_JPRB, 0.83012E+00_JPRB, 0.90734E+00_JPRB ,&
&  0.97233E+00_JPRB, 0.10711E+01_JPRB, 0.11090E+01_JPRB, 0.10312E+01_JPRB, 0.93334E+00_JPRB, 0.84760E+00_JPRB, 0.78309E+00_JPRB ,&
&  0.73163E+00_JPRB, 0.70127E+00_JPRB, 0.68258E+00_JPRB, 0.66189E+00_JPRB, 0.64501E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.59592E+00_JPRB, 0.57950E+00_JPRB, 0.56502E+00_JPRB, 0.55310E+00_JPRB, 0.54745E+00_JPRB, 0.54232E+00_JPRB, 0.53481E+00_JPRB ,&
&  0.51962E+00_JPRB, 0.50826E+00_JPRB, 0.51341E+00_JPRB, 0.49303E+00_JPRB, 0.47793E+00_JPRB, 0.48102E+00_JPRB, 0.51691E+00_JPRB ,&
&  0.55866E+00_JPRB, 0.59990E+00_JPRB, 0.50456E+00_JPRB, 0.47362E+00_JPRB, 0.55517E+00_JPRB, 0.53409E+00_JPRB, 0.55273E+00_JPRB ,&
&  0.64492E+00_JPRB, 0.74304E+00_JPRB, 0.90209E+00_JPRB, 0.12674E+01_JPRB, 0.16531E+01_JPRB, 0.20647E+01_JPRB, 0.23683E+01_JPRB ,&
&  0.24873E+01_JPRB, 0.24089E+01_JPRB, 0.22145E+01_JPRB, 0.19785E+01_JPRB, 0.16393E+01_JPRB, 0.13754E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.10496E+01_JPRB, 0.93608E+00_JPRB, 0.84902E+00_JPRB, 0.74808E+00_JPRB, 0.69948E+00_JPRB, 0.66141E+00_JPRB, 0.64178E+00_JPRB ,&
&  0.63714E+00_JPRB, 0.64473E+00_JPRB, 0.66473E+00_JPRB, 0.72068E+00_JPRB, 0.69528E+00_JPRB, 0.70624E+00_JPRB, 0.87371E+00_JPRB ,&
&  0.12289E+01_JPRB, 0.13435E+01_JPRB, 0.14670E+01_JPRB, 0.17182E+01_JPRB, 0.19585E+01_JPRB, 0.18821E+01_JPRB, 0.20238E+01_JPRB ,&
&  0.15877E+01_JPRB, 0.13137E+01_JPRB, 0.11379E+01_JPRB, 0.91518E+00_JPRB, 0.10636E+01_JPRB, 0.15369E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.12587E+01_JPRB, 0.12012E+01_JPRB, 0.97191E+00_JPRB, 0.76713E+00_JPRB, 0.58955E+00_JPRB, 0.65646E+00_JPRB ,&
&  0.66077E+00_JPRB, 0.63829E+00_JPRB, 0.57323E+00_JPRB, 0.51517E+00_JPRB, 0.46284E+00_JPRB, 0.53271E+00_JPRB, 0.53384E+00_JPRB ,&
&  0.50868E+00_JPRB, 0.50084E+00_JPRB, 0.55596E+00_JPRB, 0.62426E+00_JPRB, 0.69082E+00_JPRB, 0.70963E+00_JPRB, 0.78829E+00_JPRB ,&
&  0.81999E+00_JPRB, 0.88205E+00_JPRB, 0.89931E+00_JPRB, 0.87274E+00_JPRB, 0.82533E+00_JPRB, 0.76888E+00_JPRB, 0.72267E+00_JPRB ,&
&  0.69048E+00_JPRB, 0.66498E+00_JPRB, 0.64240E+00_JPRB, 0.62429E+00_JPRB, 0.61084E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.55741E+00_JPRB, 0.54872E+00_JPRB, 0.53980E+00_JPRB, 0.53477E+00_JPRB, 0.53024E+00_JPRB, 0.52898E+00_JPRB, 0.52728E+00_JPRB ,&
&  0.51858E+00_JPRB, 0.50256E+00_JPRB, 0.50701E+00_JPRB, 0.49821E+00_JPRB, 0.48314E+00_JPRB, 0.47878E+00_JPRB, 0.49867E+00_JPRB ,&
&  0.53901E+00_JPRB, 0.52894E+00_JPRB, 0.43404E+00_JPRB, 0.45081E+00_JPRB, 0.49460E+00_JPRB, 0.47864E+00_JPRB, 0.52662E+00_JPRB ,&
&  0.59638E+00_JPRB, 0.68935E+00_JPRB, 0.78574E+00_JPRB, 0.99668E+00_JPRB, 0.12572E+01_JPRB, 0.15938E+01_JPRB, 0.19202E+01_JPRB ,&
&  0.20208E+01_JPRB, 0.21707E+01_JPRB, 0.22236E+01_JPRB, 0.20430E+01_JPRB, 0.18059E+01_JPRB, 0.15603E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.10744E+01_JPRB, 0.92768E+00_JPRB, 0.80899E+00_JPRB, 0.73583E+00_JPRB, 0.69589E+00_JPRB, 0.67176E+00_JPRB ,&
&  0.66113E+00_JPRB, 0.65597E+00_JPRB, 0.65920E+00_JPRB, 0.71280E+00_JPRB, 0.68660E+00_JPRB, 0.75506E+00_JPRB, 0.91070E+00_JPRB ,&
&  0.13828E+01_JPRB, 0.15503E+01_JPRB, 0.16313E+01_JPRB, 0.19883E+01_JPRB, 0.16177E+01_JPRB, 0.16082E+01_JPRB, 0.18604E+01_JPRB ,&
&  0.19928E+01_JPRB, 0.19272E+01_JPRB, 0.18585E+01_JPRB, 0.13824E+01_JPRB, 0.11756E+01_JPRB, 0.15981E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.13195E+01_JPRB, 0.12380E+01_JPRB, 0.11383E+01_JPRB, 0.11804E+01_JPRB, 0.86861E+00_JPRB, 0.71169E+00_JPRB, 0.56430E+00_JPRB ,&
&  0.54729E+00_JPRB, 0.52216E+00_JPRB, 0.56963E+00_JPRB, 0.55135E+00_JPRB, 0.53979E+00_JPRB, 0.54063E+00_JPRB, 0.53150E+00_JPRB ,&
&  0.53506E+00_JPRB, 0.52408E+00_JPRB, 0.51885E+00_JPRB, 0.56177E+00_JPRB, 0.62913E+00_JPRB, 0.62082E+00_JPRB, 0.66594E+00_JPRB ,&
&  0.69139E+00_JPRB, 0.73633E+00_JPRB, 0.75142E+00_JPRB, 0.75322E+00_JPRB, 0.73703E+00_JPRB, 0.70724E+00_JPRB, 0.66676E+00_JPRB ,&
&  0.64096E+00_JPRB, 0.62257E+00_JPRB, 0.60241E+00_JPRB, 0.58508E+00_JPRB, 0.56986E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.50305E+00_JPRB, 0.49856E+00_JPRB, 0.49777E+00_JPRB, 0.50073E+00_JPRB, 0.50489E+00_JPRB, 0.50833E+00_JPRB, 0.51021E+00_JPRB ,&
&  0.50422E+00_JPRB, 0.48888E+00_JPRB, 0.48680E+00_JPRB, 0.48838E+00_JPRB, 0.47563E+00_JPRB, 0.47173E+00_JPRB, 0.48542E+00_JPRB ,&
&  0.52171E+00_JPRB, 0.55565E+00_JPRB, 0.46849E+00_JPRB, 0.41335E+00_JPRB, 0.39775E+00_JPRB, 0.44468E+00_JPRB, 0.49641E+00_JPRB ,&
&  0.58223E+00_JPRB, 0.65307E+00_JPRB, 0.74723E+00_JPRB, 0.86109E+00_JPRB, 0.10261E+01_JPRB, 0.12388E+01_JPRB, 0.15620E+01_JPRB ,&
&  0.20363E+01_JPRB, 0.18773E+01_JPRB, 0.19857E+01_JPRB, 0.19086E+01_JPRB, 0.18254E+01_JPRB, 0.16312E+01_JPRB, 0.14383E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.11293E+01_JPRB, 0.99375E+00_JPRB, 0.87374E+00_JPRB, 0.78658E+00_JPRB, 0.72422E+00_JPRB, 0.68795E+00_JPRB ,&
&  0.67639E+00_JPRB, 0.66549E+00_JPRB, 0.65417E+00_JPRB, 0.67845E+00_JPRB, 0.78328E+00_JPRB, 0.92397E+00_JPRB, 0.11146E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.12792E+01_JPRB, 0.17587E+01_JPRB, 0.19614E+01_JPRB, 0.20558E+01_JPRB, 0.20386E+01_JPRB, 0.20643E+01_JPRB ,&
&  0.21975E+01_JPRB, 0.20917E+01_JPRB, 0.19825E+01_JPRB, 0.16593E+01_JPRB, 0.18206E+01_JPRB, 0.17251E+01_JPRB, 0.15513E+01_JPRB ,&
&  0.13647E+01_JPRB, 0.12779E+01_JPRB, 0.11502E+01_JPRB, 0.10918E+01_JPRB, 0.10555E+01_JPRB, 0.10243E+01_JPRB, 0.80734E+00_JPRB ,&
&  0.64787E+00_JPRB, 0.68175E+00_JPRB, 0.62352E+00_JPRB, 0.56841E+00_JPRB, 0.51738E+00_JPRB, 0.53634E+00_JPRB, 0.55887E+00_JPRB ,&
&  0.57557E+00_JPRB, 0.57963E+00_JPRB, 0.57130E+00_JPRB, 0.59326E+00_JPRB, 0.63160E+00_JPRB, 0.64766E+00_JPRB, 0.64110E+00_JPRB ,&
&  0.62437E+00_JPRB, 0.63856E+00_JPRB, 0.64901E+00_JPRB, 0.63482E+00_JPRB, 0.65680E+00_JPRB, 0.63438E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.57898E+00_JPRB, 0.56012E+00_JPRB, 0.54399E+00_JPRB, 0.52737E+00_JPRB, 0.51163E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.45577E+00_JPRB, 0.45687E+00_JPRB, 0.45851E+00_JPRB, 0.46339E+00_JPRB, 0.46922E+00_JPRB, 0.47891E+00_JPRB, 0.48482E+00_JPRB ,&
&  0.48497E+00_JPRB, 0.47439E+00_JPRB, 0.46740E+00_JPRB, 0.47395E+00_JPRB, 0.46961E+00_JPRB, 0.46655E+00_JPRB, 0.47820E+00_JPRB ,&
&  0.51473E+00_JPRB, 0.62787E+00_JPRB, 0.52753E+00_JPRB, 0.48624E+00_JPRB, 0.44729E+00_JPRB, 0.47964E+00_JPRB, 0.51497E+00_JPRB ,&
&  0.60648E+00_JPRB, 0.65033E+00_JPRB, 0.67643E+00_JPRB, 0.73328E+00_JPRB, 0.86426E+00_JPRB, 0.10627E+01_JPRB, 0.13022E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.16244E+01_JPRB, 0.16355E+01_JPRB, 0.18297E+01_JPRB, 0.17212E+01_JPRB, 0.14694E+01_JPRB, 0.14334E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.11239E+01_JPRB, 0.98285E+00_JPRB, 0.87614E+00_JPRB, 0.79649E+00_JPRB, 0.73810E+00_JPRB, 0.69428E+00_JPRB ,&
&  0.66398E+00_JPRB, 0.63919E+00_JPRB, 0.61321E+00_JPRB, 0.60377E+00_JPRB, 0.66453E+00_JPRB, 0.87748E+00_JPRB, 0.11602E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.15475E+01_JPRB, 0.14021E+01_JPRB, 0.19097E+01_JPRB, 0.24098E+01_JPRB, 0.23682E+01_JPRB, 0.22214E+01_JPRB ,&
&  0.23956E+01_JPRB, 0.23833E+01_JPRB, 0.22799E+01_JPRB, 0.22342E+01_JPRB, 0.21109E+01_JPRB, 0.19289E+01_JPRB, 0.17083E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.12665E+01_JPRB, 0.11663E+01_JPRB, 0.11316E+01_JPRB, 0.10313E+01_JPRB, 0.95329E+00_JPRB, 0.90027E+00_JPRB ,&
&  0.78173E+00_JPRB, 0.68956E+00_JPRB, 0.54128E+00_JPRB, 0.56458E+00_JPRB, 0.48843E+00_JPRB, 0.50239E+00_JPRB, 0.56959E+00_JPRB ,&
&  0.57598E+00_JPRB, 0.61421E+00_JPRB, 0.62543E+00_JPRB, 0.60399E+00_JPRB, 0.65372E+00_JPRB, 0.64148E+00_JPRB, 0.61649E+00_JPRB ,&
&  0.60653E+00_JPRB, 0.55053E+00_JPRB, 0.56454E+00_JPRB, 0.56349E+00_JPRB, 0.54948E+00_JPRB, 0.53098E+00_JPRB, 0.51216E+00_JPRB ,&
&  0.49703E+00_JPRB, 0.48558E+00_JPRB, 0.47848E+00_JPRB, 0.46733E+00_JPRB, 0.45862E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.41720E+00_JPRB, 0.41805E+00_JPRB, 0.41773E+00_JPRB, 0.42324E+00_JPRB, 0.43225E+00_JPRB, 0.44440E+00_JPRB, 0.45681E+00_JPRB ,&
&  0.46523E+00_JPRB, 0.46381E+00_JPRB, 0.45653E+00_JPRB, 0.45793E+00_JPRB, 0.45782E+00_JPRB, 0.45809E+00_JPRB, 0.47097E+00_JPRB ,&
&  0.52283E+00_JPRB, 0.60003E+00_JPRB, 0.54185E+00_JPRB, 0.47818E+00_JPRB, 0.49991E+00_JPRB, 0.50173E+00_JPRB, 0.53758E+00_JPRB ,&
&  0.56263E+00_JPRB, 0.62419E+00_JPRB, 0.64971E+00_JPRB, 0.67535E+00_JPRB, 0.73339E+00_JPRB, 0.87820E+00_JPRB, 0.10614E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.12353E+01_JPRB, 0.13372E+01_JPRB, 0.14353E+01_JPRB, 0.14420E+01_JPRB, 0.12844E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.10388E+01_JPRB, 0.90755E+00_JPRB, 0.80195E+00_JPRB, 0.72944E+00_JPRB, 0.68206E+00_JPRB, 0.65308E+00_JPRB ,&
&  0.63633E+00_JPRB, 0.61656E+00_JPRB, 0.58754E+00_JPRB, 0.58625E+00_JPRB, 0.63538E+00_JPRB, 0.99901E+00_JPRB, 0.16369E+01_JPRB ,&
&  0.21277E+01_JPRB, 0.22655E+01_JPRB, 0.24676E+01_JPRB, 0.27445E+01_JPRB, 0.27906E+01_JPRB, 0.28440E+01_JPRB, 0.28652E+01_JPRB ,&
&  0.27550E+01_JPRB, 0.26058E+01_JPRB, 0.23837E+01_JPRB, 0.21302E+01_JPRB, 0.19795E+01_JPRB, 0.18741E+01_JPRB, 0.16974E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.12780E+01_JPRB, 0.12070E+01_JPRB, 0.11232E+01_JPRB, 0.10543E+01_JPRB, 0.95358E+00_JPRB, 0.93519E+00_JPRB ,&
&  0.95686E+00_JPRB, 0.60560E+00_JPRB, 0.51537E+00_JPRB, 0.58207E+00_JPRB, 0.51953E+00_JPRB, 0.54215E+00_JPRB, 0.62714E+00_JPRB ,&
&  0.63425E+00_JPRB, 0.63114E+00_JPRB, 0.65173E+00_JPRB, 0.62250E+00_JPRB, 0.62065E+00_JPRB, 0.62667E+00_JPRB, 0.58566E+00_JPRB ,&
&  0.55731E+00_JPRB, 0.53833E+00_JPRB, 0.54688E+00_JPRB, 0.52518E+00_JPRB, 0.49666E+00_JPRB, 0.47359E+00_JPRB, 0.45733E+00_JPRB ,&
&  0.44547E+00_JPRB, 0.43859E+00_JPRB, 0.43574E+00_JPRB, 0.42464E+00_JPRB, 0.41678E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.39020E+00_JPRB, 0.38809E+00_JPRB, 0.38935E+00_JPRB, 0.39630E+00_JPRB, 0.40631E+00_JPRB, 0.42050E+00_JPRB, 0.43586E+00_JPRB ,&
&  0.45030E+00_JPRB, 0.45573E+00_JPRB, 0.44894E+00_JPRB, 0.44423E+00_JPRB, 0.44419E+00_JPRB, 0.44687E+00_JPRB, 0.46482E+00_JPRB ,&
&  0.43240E+00_JPRB, 0.47202E+00_JPRB, 0.43056E+00_JPRB, 0.44391E+00_JPRB, 0.49962E+00_JPRB, 0.50400E+00_JPRB, 0.51828E+00_JPRB ,&
&  0.53313E+00_JPRB, 0.56365E+00_JPRB, 0.58039E+00_JPRB, 0.60033E+00_JPRB, 0.63515E+00_JPRB, 0.72220E+00_JPRB, 0.87540E+00_JPRB ,&
&  0.91927E+00_JPRB, 0.90690E+00_JPRB, 0.96263E+00_JPRB, 0.10226E+01_JPRB, 0.10175E+01_JPRB, 0.10609E+01_JPRB, 0.10859E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.94097E+00_JPRB, 0.85528E+00_JPRB, 0.77336E+00_JPRB, 0.70422E+00_JPRB, 0.64662E+00_JPRB, 0.60816E+00_JPRB ,&
&  0.58215E+00_JPRB, 0.56440E+00_JPRB, 0.55257E+00_JPRB, 0.59230E+00_JPRB, 0.66019E+00_JPRB, 0.12118E+01_JPRB, 0.22344E+01_JPRB ,&
&  0.25549E+01_JPRB, 0.25991E+01_JPRB, 0.28455E+01_JPRB, 0.32508E+01_JPRB, 0.31532E+01_JPRB, 0.30183E+01_JPRB, 0.28525E+01_JPRB ,&
&  0.26659E+01_JPRB, 0.24139E+01_JPRB, 0.22075E+01_JPRB, 0.20838E+01_JPRB, 0.18884E+01_JPRB, 0.18287E+01_JPRB, 0.16445E+01_JPRB ,&
&  0.14083E+01_JPRB, 0.12988E+01_JPRB, 0.12458E+01_JPRB, 0.12170E+01_JPRB, 0.11580E+01_JPRB, 0.11562E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.90386E+00_JPRB, 0.70187E+00_JPRB, 0.66461E+00_JPRB, 0.55781E+00_JPRB, 0.58731E+00_JPRB, 0.70052E+00_JPRB ,&
&  0.70101E+00_JPRB, 0.65803E+00_JPRB, 0.61506E+00_JPRB, 0.59273E+00_JPRB, 0.58683E+00_JPRB, 0.60986E+00_JPRB, 0.56690E+00_JPRB ,&
&  0.52612E+00_JPRB, 0.53371E+00_JPRB, 0.52932E+00_JPRB, 0.51108E+00_JPRB, 0.48683E+00_JPRB, 0.45933E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.43407E+00_JPRB, 0.42639E+00_JPRB, 0.41394E+00_JPRB, 0.39903E+00_JPRB, 0.39215E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.37517E+00_JPRB, 0.37175E+00_JPRB, 0.37137E+00_JPRB, 0.37862E+00_JPRB, 0.38966E+00_JPRB, 0.40195E+00_JPRB, 0.41746E+00_JPRB ,&
&  0.43407E+00_JPRB, 0.44355E+00_JPRB, 0.44092E+00_JPRB, 0.43374E+00_JPRB, 0.43041E+00_JPRB, 0.43336E+00_JPRB, 0.41409E+00_JPRB ,&
&  0.37982E+00_JPRB, 0.40987E+00_JPRB, 0.40551E+00_JPRB, 0.41571E+00_JPRB, 0.44599E+00_JPRB, 0.45391E+00_JPRB, 0.46547E+00_JPRB ,&
&  0.49110E+00_JPRB, 0.50280E+00_JPRB, 0.51880E+00_JPRB, 0.53787E+00_JPRB, 0.56346E+00_JPRB, 0.62535E+00_JPRB, 0.72390E+00_JPRB ,&
&  0.74177E+00_JPRB, 0.75434E+00_JPRB, 0.76815E+00_JPRB, 0.80441E+00_JPRB, 0.85173E+00_JPRB, 0.87969E+00_JPRB, 0.86087E+00_JPRB ,&
&  0.81974E+00_JPRB, 0.77386E+00_JPRB, 0.73834E+00_JPRB, 0.70582E+00_JPRB, 0.66242E+00_JPRB, 0.61571E+00_JPRB, 0.57909E+00_JPRB ,&
&  0.55034E+00_JPRB, 0.53452E+00_JPRB, 0.53734E+00_JPRB, 0.56326E+00_JPRB, 0.65959E+00_JPRB, 0.10942E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.19348E+01_JPRB, 0.21361E+01_JPRB, 0.22120E+01_JPRB, 0.22786E+01_JPRB, 0.23209E+01_JPRB, 0.23183E+01_JPRB, 0.22746E+01_JPRB ,&
&  0.22073E+01_JPRB, 0.19847E+01_JPRB, 0.19838E+01_JPRB, 0.19441E+01_JPRB, 0.18281E+01_JPRB, 0.17511E+01_JPRB, 0.15491E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.12649E+01_JPRB, 0.12551E+01_JPRB, 0.12303E+01_JPRB, 0.11744E+01_JPRB, 0.11765E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10513E+01_JPRB, 0.91090E+00_JPRB, 0.80261E+00_JPRB, 0.69421E+00_JPRB, 0.71641E+00_JPRB, 0.66932E+00_JPRB ,&
&  0.60798E+00_JPRB, 0.52753E+00_JPRB, 0.52932E+00_JPRB, 0.52856E+00_JPRB, 0.54307E+00_JPRB, 0.54997E+00_JPRB, 0.52202E+00_JPRB ,&
&  0.51631E+00_JPRB, 0.52817E+00_JPRB, 0.50935E+00_JPRB, 0.48968E+00_JPRB, 0.46884E+00_JPRB, 0.44018E+00_JPRB, 0.41859E+00_JPRB ,&
&  0.40960E+00_JPRB, 0.40583E+00_JPRB, 0.39361E+00_JPRB, 0.38125E+00_JPRB, 0.37677E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.37369E+00_JPRB, 0.37575E+00_JPRB, 0.37437E+00_JPRB, 0.37626E+00_JPRB, 0.38339E+00_JPRB, 0.39168E+00_JPRB, 0.39727E+00_JPRB ,&
&  0.41588E+00_JPRB, 0.42318E+00_JPRB, 0.42484E+00_JPRB, 0.42235E+00_JPRB, 0.41594E+00_JPRB, 0.38338E+00_JPRB, 0.33266E+00_JPRB ,&
&  0.32472E+00_JPRB, 0.34259E+00_JPRB, 0.36785E+00_JPRB, 0.38318E+00_JPRB, 0.39685E+00_JPRB, 0.40818E+00_JPRB, 0.41707E+00_JPRB ,&
&  0.42328E+00_JPRB, 0.42916E+00_JPRB, 0.44623E+00_JPRB, 0.47245E+00_JPRB, 0.50090E+00_JPRB, 0.53608E+00_JPRB, 0.59441E+00_JPRB ,&
&  0.60408E+00_JPRB, 0.62689E+00_JPRB, 0.66113E+00_JPRB, 0.63819E+00_JPRB, 0.71033E+00_JPRB, 0.72196E+00_JPRB, 0.71391E+00_JPRB ,&
&  0.69883E+00_JPRB, 0.66992E+00_JPRB, 0.63578E+00_JPRB, 0.60789E+00_JPRB, 0.58293E+00_JPRB, 0.56176E+00_JPRB, 0.53294E+00_JPRB ,&
&  0.51131E+00_JPRB, 0.49885E+00_JPRB, 0.49977E+00_JPRB, 0.51409E+00_JPRB, 0.56409E+00_JPRB, 0.84321E+00_JPRB, 0.10924E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.14457E+01_JPRB, 0.15019E+01_JPRB, 0.14911E+01_JPRB, 0.17112E+01_JPRB, 0.19021E+01_JPRB, 0.18868E+01_JPRB ,&
&  0.18929E+01_JPRB, 0.17136E+01_JPRB, 0.16572E+01_JPRB, 0.16238E+01_JPRB, 0.16086E+01_JPRB, 0.15151E+01_JPRB, 0.13509E+01_JPRB ,&
&  0.12435E+01_JPRB, 0.12628E+01_JPRB, 0.12193E+01_JPRB, 0.11494E+01_JPRB, 0.10551E+01_JPRB, 0.10128E+01_JPRB, 0.95662E+00_JPRB ,&
&  0.94339E+00_JPRB, 0.93871E+00_JPRB, 0.89447E+00_JPRB, 0.84444E+00_JPRB, 0.78522E+00_JPRB, 0.72113E+00_JPRB, 0.65823E+00_JPRB ,&
&  0.60064E+00_JPRB, 0.53402E+00_JPRB, 0.50959E+00_JPRB, 0.51086E+00_JPRB, 0.51885E+00_JPRB, 0.50346E+00_JPRB, 0.49709E+00_JPRB ,&
&  0.48858E+00_JPRB, 0.48225E+00_JPRB, 0.49142E+00_JPRB, 0.47434E+00_JPRB, 0.45115E+00_JPRB, 0.42518E+00_JPRB, 0.40755E+00_JPRB ,&
&  0.38993E+00_JPRB, 0.39232E+00_JPRB, 0.38744E+00_JPRB, 0.37868E+00_JPRB, 0.37313E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.36994E+00_JPRB, 0.37418E+00_JPRB, 0.37405E+00_JPRB, 0.37195E+00_JPRB, 0.37512E+00_JPRB, 0.37549E+00_JPRB, 0.37603E+00_JPRB ,&
&  0.41619E+00_JPRB, 0.39259E+00_JPRB, 0.40421E+00_JPRB, 0.35595E+00_JPRB, 0.33717E+00_JPRB, 0.33196E+00_JPRB, 0.32347E+00_JPRB ,&
&  0.32636E+00_JPRB, 0.33714E+00_JPRB, 0.34768E+00_JPRB, 0.35604E+00_JPRB, 0.36992E+00_JPRB, 0.37732E+00_JPRB, 0.38233E+00_JPRB ,&
&  0.38959E+00_JPRB, 0.40093E+00_JPRB, 0.42009E+00_JPRB, 0.43555E+00_JPRB, 0.45897E+00_JPRB, 0.49201E+00_JPRB, 0.53736E+00_JPRB ,&
&  0.54983E+00_JPRB, 0.58250E+00_JPRB, 0.64538E+00_JPRB, 0.66560E+00_JPRB, 0.67944E+00_JPRB, 0.67037E+00_JPRB, 0.65443E+00_JPRB ,&
&  0.63051E+00_JPRB, 0.54607E+00_JPRB, 0.56080E+00_JPRB, 0.54030E+00_JPRB, 0.52103E+00_JPRB, 0.50135E+00_JPRB, 0.48323E+00_JPRB ,&
&  0.46903E+00_JPRB, 0.46084E+00_JPRB, 0.46010E+00_JPRB, 0.46252E+00_JPRB, 0.47553E+00_JPRB, 0.55637E+00_JPRB, 0.66475E+00_JPRB ,&
&  0.87830E+00_JPRB, 0.84674E+00_JPRB, 0.10848E+01_JPRB, 0.11754E+01_JPRB, 0.14194E+01_JPRB, 0.15698E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.15370E+01_JPRB, 0.14168E+01_JPRB, 0.13341E+01_JPRB, 0.13052E+01_JPRB, 0.12516E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.10404E+01_JPRB, 0.10542E+01_JPRB, 0.99218E+00_JPRB, 0.91201E+00_JPRB, 0.87532E+00_JPRB, 0.83217E+00_JPRB ,&
&  0.80180E+00_JPRB, 0.77477E+00_JPRB, 0.74135E+00_JPRB, 0.67008E+00_JPRB, 0.64655E+00_JPRB, 0.62670E+00_JPRB, 0.58526E+00_JPRB ,&
&  0.56854E+00_JPRB, 0.54599E+00_JPRB, 0.52311E+00_JPRB, 0.53239E+00_JPRB, 0.51697E+00_JPRB, 0.50879E+00_JPRB, 0.50248E+00_JPRB ,&
&  0.48057E+00_JPRB, 0.45494E+00_JPRB, 0.44951E+00_JPRB, 0.44300E+00_JPRB, 0.42918E+00_JPRB, 0.41651E+00_JPRB, 0.41687E+00_JPRB ,&
&  0.40611E+00_JPRB, 0.39925E+00_JPRB, 0.39913E+00_JPRB, 0.38526E+00_JPRB, 0.37544E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.37479E+00_JPRB, 0.37446E+00_JPRB, 0.37256E+00_JPRB, 0.36690E+00_JPRB, 0.36460E+00_JPRB, 0.36207E+00_JPRB, 0.35369E+00_JPRB ,&
&  0.36708E+00_JPRB, 0.34027E+00_JPRB, 0.32524E+00_JPRB, 0.31940E+00_JPRB, 0.32021E+00_JPRB, 0.31856E+00_JPRB, 0.31398E+00_JPRB ,&
&  0.31630E+00_JPRB, 0.32513E+00_JPRB, 0.33703E+00_JPRB, 0.34594E+00_JPRB, 0.35414E+00_JPRB, 0.36061E+00_JPRB, 0.36664E+00_JPRB ,&
&  0.37524E+00_JPRB, 0.38666E+00_JPRB, 0.40111E+00_JPRB, 0.42055E+00_JPRB, 0.44290E+00_JPRB, 0.47260E+00_JPRB, 0.50114E+00_JPRB ,&
&  0.54239E+00_JPRB, 0.59389E+00_JPRB, 0.62953E+00_JPRB, 0.66589E+00_JPRB, 0.67365E+00_JPRB, 0.65384E+00_JPRB, 0.61141E+00_JPRB ,&
&  0.43553E+00_JPRB, 0.41039E+00_JPRB, 0.47521E+00_JPRB, 0.47085E+00_JPRB, 0.46162E+00_JPRB, 0.44886E+00_JPRB, 0.44135E+00_JPRB ,&
&  0.43740E+00_JPRB, 0.43309E+00_JPRB, 0.43253E+00_JPRB, 0.43193E+00_JPRB, 0.43557E+00_JPRB, 0.45255E+00_JPRB, 0.49329E+00_JPRB ,&
&  0.59845E+00_JPRB, 0.61682E+00_JPRB, 0.66665E+00_JPRB, 0.75539E+00_JPRB, 0.10319E+01_JPRB, 0.11059E+01_JPRB, 0.11215E+01_JPRB ,&
&  0.11715E+01_JPRB, 0.11495E+01_JPRB, 0.11143E+01_JPRB, 0.10943E+01_JPRB, 0.10873E+01_JPRB, 0.10769E+01_JPRB, 0.10633E+01_JPRB ,&
&  0.10051E+01_JPRB, 0.96900E+00_JPRB, 0.93788E+00_JPRB, 0.89965E+00_JPRB, 0.84810E+00_JPRB, 0.81459E+00_JPRB, 0.77913E+00_JPRB ,&
&  0.74439E+00_JPRB, 0.70819E+00_JPRB, 0.65846E+00_JPRB, 0.61159E+00_JPRB, 0.57386E+00_JPRB, 0.55800E+00_JPRB, 0.54880E+00_JPRB ,&
&  0.53972E+00_JPRB, 0.53498E+00_JPRB, 0.52952E+00_JPRB, 0.52328E+00_JPRB, 0.51462E+00_JPRB, 0.50195E+00_JPRB, 0.49437E+00_JPRB ,&
&  0.46272E+00_JPRB, 0.44302E+00_JPRB, 0.41681E+00_JPRB, 0.40414E+00_JPRB, 0.40100E+00_JPRB, 0.40037E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.40377E+00_JPRB, 0.39440E+00_JPRB, 0.38633E+00_JPRB, 0.38218E+00_JPRB, 0.37898E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.38152E+00_JPRB, 0.37776E+00_JPRB, 0.37120E+00_JPRB, 0.36070E+00_JPRB, 0.35157E+00_JPRB, 0.34730E+00_JPRB, 0.34472E+00_JPRB ,&
&  0.35232E+00_JPRB, 0.33929E+00_JPRB, 0.32637E+00_JPRB, 0.31653E+00_JPRB, 0.31149E+00_JPRB, 0.30983E+00_JPRB, 0.31301E+00_JPRB ,&
&  0.31903E+00_JPRB, 0.32510E+00_JPRB, 0.33076E+00_JPRB, 0.33644E+00_JPRB, 0.34220E+00_JPRB, 0.34710E+00_JPRB, 0.35298E+00_JPRB ,&
&  0.36157E+00_JPRB, 0.37089E+00_JPRB, 0.38031E+00_JPRB, 0.39431E+00_JPRB, 0.41254E+00_JPRB, 0.44011E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.48628E+00_JPRB, 0.51636E+00_JPRB, 0.56050E+00_JPRB, 0.62652E+00_JPRB, 0.65609E+00_JPRB, 0.62742E+00_JPRB, 0.47937E+00_JPRB ,&
&  0.37133E+00_JPRB, 0.34044E+00_JPRB, 0.35357E+00_JPRB, 0.40437E+00_JPRB, 0.40663E+00_JPRB, 0.40652E+00_JPRB, 0.40730E+00_JPRB ,&
&  0.39432E+00_JPRB, 0.38783E+00_JPRB, 0.39332E+00_JPRB, 0.40707E+00_JPRB, 0.40875E+00_JPRB, 0.41304E+00_JPRB, 0.43383E+00_JPRB ,&
&  0.47458E+00_JPRB, 0.52555E+00_JPRB, 0.58267E+00_JPRB, 0.58055E+00_JPRB, 0.66500E+00_JPRB, 0.93307E+00_JPRB, 0.96547E+00_JPRB ,&
&  0.93522E+00_JPRB, 0.91252E+00_JPRB, 0.92134E+00_JPRB, 0.91853E+00_JPRB, 0.89158E+00_JPRB, 0.87426E+00_JPRB, 0.85708E+00_JPRB ,&
&  0.82173E+00_JPRB, 0.79173E+00_JPRB, 0.76482E+00_JPRB, 0.75918E+00_JPRB, 0.74290E+00_JPRB, 0.72731E+00_JPRB, 0.71436E+00_JPRB ,&
&  0.69128E+00_JPRB, 0.64766E+00_JPRB, 0.60431E+00_JPRB, 0.55935E+00_JPRB, 0.53667E+00_JPRB, 0.52297E+00_JPRB, 0.51516E+00_JPRB ,&
&  0.50576E+00_JPRB, 0.50269E+00_JPRB, 0.50366E+00_JPRB, 0.50266E+00_JPRB, 0.48118E+00_JPRB, 0.45196E+00_JPRB, 0.43210E+00_JPRB ,&
&  0.42048E+00_JPRB, 0.41092E+00_JPRB, 0.39993E+00_JPRB, 0.39336E+00_JPRB, 0.39308E+00_JPRB, 0.39174E+00_JPRB, 0.38609E+00_JPRB ,&
&  0.38442E+00_JPRB, 0.37893E+00_JPRB, 0.37595E+00_JPRB, 0.37662E+00_JPRB, 0.38096E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.37920E+00_JPRB, 0.38075E+00_JPRB, 0.37373E+00_JPRB, 0.36158E+00_JPRB, 0.34455E+00_JPRB, 0.33170E+00_JPRB, 0.32344E+00_JPRB ,&
&  0.32147E+00_JPRB, 0.31855E+00_JPRB, 0.31702E+00_JPRB, 0.31705E+00_JPRB, 0.31803E+00_JPRB, 0.32055E+00_JPRB, 0.32291E+00_JPRB ,&
&  0.32473E+00_JPRB, 0.32729E+00_JPRB, 0.33062E+00_JPRB, 0.33398E+00_JPRB, 0.33760E+00_JPRB, 0.34290E+00_JPRB, 0.34929E+00_JPRB ,&
&  0.35681E+00_JPRB, 0.36191E+00_JPRB, 0.36491E+00_JPRB, 0.36938E+00_JPRB, 0.37850E+00_JPRB, 0.39581E+00_JPRB, 0.41728E+00_JPRB ,&
&  0.43895E+00_JPRB, 0.45695E+00_JPRB, 0.49746E+00_JPRB, 0.56878E+00_JPRB, 0.60501E+00_JPRB, 0.57215E+00_JPRB, 0.44641E+00_JPRB ,&
&  0.35808E+00_JPRB, 0.31347E+00_JPRB, 0.30208E+00_JPRB, 0.31128E+00_JPRB, 0.33326E+00_JPRB, 0.35989E+00_JPRB, 0.36669E+00_JPRB ,&
&  0.37109E+00_JPRB, 0.37331E+00_JPRB, 0.37555E+00_JPRB, 0.37868E+00_JPRB, 0.38059E+00_JPRB, 0.38447E+00_JPRB, 0.39790E+00_JPRB ,&
&  0.42220E+00_JPRB, 0.45288E+00_JPRB, 0.48425E+00_JPRB, 0.51303E+00_JPRB, 0.54712E+00_JPRB, 0.62211E+00_JPRB, 0.69594E+00_JPRB ,&
&  0.69853E+00_JPRB, 0.68724E+00_JPRB, 0.70361E+00_JPRB, 0.73942E+00_JPRB, 0.75754E+00_JPRB, 0.75504E+00_JPRB, 0.73868E+00_JPRB ,&
&  0.71777E+00_JPRB, 0.68193E+00_JPRB, 0.64764E+00_JPRB, 0.62360E+00_JPRB, 0.62226E+00_JPRB, 0.64081E+00_JPRB, 0.64985E+00_JPRB ,&
&  0.64848E+00_JPRB, 0.62609E+00_JPRB, 0.58739E+00_JPRB, 0.52514E+00_JPRB, 0.50306E+00_JPRB, 0.50318E+00_JPRB, 0.50878E+00_JPRB ,&
&  0.50408E+00_JPRB, 0.49699E+00_JPRB, 0.48978E+00_JPRB, 0.48494E+00_JPRB, 0.45981E+00_JPRB, 0.44362E+00_JPRB, 0.42985E+00_JPRB ,&
&  0.42094E+00_JPRB, 0.41423E+00_JPRB, 0.41483E+00_JPRB, 0.41342E+00_JPRB, 0.40908E+00_JPRB, 0.40223E+00_JPRB, 0.39347E+00_JPRB ,&
&  0.38690E+00_JPRB, 0.38111E+00_JPRB, 0.37532E+00_JPRB, 0.37218E+00_JPRB, 0.37401E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.38696E+00_JPRB, 0.38494E+00_JPRB, 0.38079E+00_JPRB, 0.37013E+00_JPRB, 0.35469E+00_JPRB, 0.34051E+00_JPRB, 0.33178E+00_JPRB ,&
&  0.32904E+00_JPRB, 0.32905E+00_JPRB, 0.32695E+00_JPRB, 0.32627E+00_JPRB, 0.32642E+00_JPRB, 0.32740E+00_JPRB, 0.32935E+00_JPRB ,&
&  0.33206E+00_JPRB, 0.33477E+00_JPRB, 0.33661E+00_JPRB, 0.33745E+00_JPRB, 0.34015E+00_JPRB, 0.34524E+00_JPRB, 0.35144E+00_JPRB ,&
&  0.35657E+00_JPRB, 0.35693E+00_JPRB, 0.35764E+00_JPRB, 0.36066E+00_JPRB, 0.36384E+00_JPRB, 0.37043E+00_JPRB, 0.37801E+00_JPRB ,&
&  0.39317E+00_JPRB, 0.41826E+00_JPRB, 0.46580E+00_JPRB, 0.52021E+00_JPRB, 0.54527E+00_JPRB, 0.50544E+00_JPRB, 0.41271E+00_JPRB ,&
&  0.34502E+00_JPRB, 0.30424E+00_JPRB, 0.28758E+00_JPRB, 0.28427E+00_JPRB, 0.29105E+00_JPRB, 0.30647E+00_JPRB, 0.32806E+00_JPRB ,&
&  0.34362E+00_JPRB, 0.34916E+00_JPRB, 0.35663E+00_JPRB, 0.36305E+00_JPRB, 0.36774E+00_JPRB, 0.37062E+00_JPRB, 0.37540E+00_JPRB ,&
&  0.38349E+00_JPRB, 0.39666E+00_JPRB, 0.41937E+00_JPRB, 0.45518E+00_JPRB, 0.50455E+00_JPRB, 0.50901E+00_JPRB, 0.53525E+00_JPRB ,&
&  0.58466E+00_JPRB, 0.59234E+00_JPRB, 0.60853E+00_JPRB, 0.61535E+00_JPRB, 0.61765E+00_JPRB, 0.60682E+00_JPRB, 0.60165E+00_JPRB ,&
&  0.59990E+00_JPRB, 0.59166E+00_JPRB, 0.57314E+00_JPRB, 0.55016E+00_JPRB, 0.53485E+00_JPRB, 0.53100E+00_JPRB, 0.53710E+00_JPRB ,&
&  0.55021E+00_JPRB, 0.58888E+00_JPRB, 0.71500E+00_JPRB, 0.61923E+00_JPRB, 0.57067E+00_JPRB, 0.55372E+00_JPRB, 0.53242E+00_JPRB ,&
&  0.51355E+00_JPRB, 0.49993E+00_JPRB, 0.49162E+00_JPRB, 0.48203E+00_JPRB, 0.46400E+00_JPRB, 0.45177E+00_JPRB, 0.44282E+00_JPRB ,&
&  0.43655E+00_JPRB, 0.42939E+00_JPRB, 0.42389E+00_JPRB, 0.41992E+00_JPRB, 0.41550E+00_JPRB, 0.41000E+00_JPRB, 0.40481E+00_JPRB ,&
&  0.39909E+00_JPRB, 0.39521E+00_JPRB, 0.39391E+00_JPRB, 0.39330E+00_JPRB, 0.39101E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.39187E+00_JPRB, 0.38788E+00_JPRB, 0.38007E+00_JPRB, 0.37462E+00_JPRB, 0.36574E+00_JPRB, 0.35733E+00_JPRB, 0.34812E+00_JPRB ,&
&  0.34164E+00_JPRB, 0.33743E+00_JPRB, 0.33379E+00_JPRB, 0.33240E+00_JPRB, 0.33209E+00_JPRB, 0.33216E+00_JPRB, 0.33364E+00_JPRB ,&
&  0.33531E+00_JPRB, 0.33404E+00_JPRB, 0.33553E+00_JPRB, 0.33869E+00_JPRB, 0.34239E+00_JPRB, 0.34613E+00_JPRB, 0.34988E+00_JPRB ,&
&  0.35029E+00_JPRB, 0.34828E+00_JPRB, 0.34781E+00_JPRB, 0.34876E+00_JPRB, 0.35150E+00_JPRB, 0.35804E+00_JPRB, 0.37369E+00_JPRB ,&
&  0.39460E+00_JPRB, 0.42158E+00_JPRB, 0.45116E+00_JPRB, 0.47921E+00_JPRB, 0.48280E+00_JPRB, 0.44976E+00_JPRB, 0.37221E+00_JPRB ,&
&  0.32978E+00_JPRB, 0.30269E+00_JPRB, 0.29089E+00_JPRB, 0.28597E+00_JPRB, 0.28955E+00_JPRB, 0.29952E+00_JPRB, 0.31678E+00_JPRB ,&
&  0.32920E+00_JPRB, 0.33703E+00_JPRB, 0.34100E+00_JPRB, 0.34483E+00_JPRB, 0.34738E+00_JPRB, 0.35060E+00_JPRB, 0.35336E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.36361E+00_JPRB, 0.36783E+00_JPRB, 0.37611E+00_JPRB, 0.38898E+00_JPRB, 0.40099E+00_JPRB, 0.41621E+00_JPRB ,&
&  0.44740E+00_JPRB, 0.46225E+00_JPRB, 0.47225E+00_JPRB, 0.47874E+00_JPRB, 0.49017E+00_JPRB, 0.49270E+00_JPRB, 0.49025E+00_JPRB ,&
&  0.48806E+00_JPRB, 0.49713E+00_JPRB, 0.49532E+00_JPRB, 0.49247E+00_JPRB, 0.48776E+00_JPRB, 0.48512E+00_JPRB, 0.48450E+00_JPRB ,&
&  0.47605E+00_JPRB, 0.48187E+00_JPRB, 0.53384E+00_JPRB, 0.54513E+00_JPRB, 0.53161E+00_JPRB, 0.51332E+00_JPRB, 0.49278E+00_JPRB ,&
&  0.47607E+00_JPRB, 0.46236E+00_JPRB, 0.45182E+00_JPRB, 0.44288E+00_JPRB, 0.43691E+00_JPRB, 0.43323E+00_JPRB, 0.43264E+00_JPRB ,&
&  0.42850E+00_JPRB, 0.42292E+00_JPRB, 0.41614E+00_JPRB, 0.41196E+00_JPRB, 0.41009E+00_JPRB, 0.40657E+00_JPRB, 0.40219E+00_JPRB ,&
&  0.39775E+00_JPRB, 0.39701E+00_JPRB, 0.39974E+00_JPRB, 0.40061E+00_JPRB, 0.39715E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.39149E+00_JPRB, 0.38908E+00_JPRB, 0.38335E+00_JPRB, 0.37808E+00_JPRB, 0.37112E+00_JPRB, 0.36562E+00_JPRB, 0.35898E+00_JPRB ,&
&  0.35378E+00_JPRB, 0.34907E+00_JPRB, 0.34502E+00_JPRB, 0.34216E+00_JPRB, 0.34020E+00_JPRB, 0.33875E+00_JPRB, 0.33851E+00_JPRB ,&
&  0.33908E+00_JPRB, 0.34029E+00_JPRB, 0.34182E+00_JPRB, 0.34352E+00_JPRB, 0.34555E+00_JPRB, 0.34812E+00_JPRB, 0.35028E+00_JPRB ,&
&  0.35136E+00_JPRB, 0.34891E+00_JPRB, 0.34476E+00_JPRB, 0.34889E+00_JPRB, 0.35619E+00_JPRB, 0.35778E+00_JPRB, 0.37628E+00_JPRB ,&
&  0.39549E+00_JPRB, 0.42058E+00_JPRB, 0.43440E+00_JPRB, 0.44043E+00_JPRB, 0.43532E+00_JPRB, 0.35590E+00_JPRB, 0.33954E+00_JPRB ,&
&  0.32120E+00_JPRB, 0.30659E+00_JPRB, 0.29954E+00_JPRB, 0.29524E+00_JPRB, 0.29455E+00_JPRB, 0.30018E+00_JPRB, 0.30943E+00_JPRB ,&
&  0.31496E+00_JPRB, 0.33528E+00_JPRB, 0.33588E+00_JPRB, 0.33475E+00_JPRB, 0.33488E+00_JPRB, 0.33652E+00_JPRB, 0.33496E+00_JPRB ,&
&  0.33621E+00_JPRB, 0.33677E+00_JPRB, 0.33752E+00_JPRB, 0.34254E+00_JPRB, 0.34699E+00_JPRB, 0.34954E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.36337E+00_JPRB, 0.37015E+00_JPRB, 0.37336E+00_JPRB, 0.37661E+00_JPRB, 0.38177E+00_JPRB, 0.38950E+00_JPRB, 0.40038E+00_JPRB ,&
&  0.40575E+00_JPRB, 0.41347E+00_JPRB, 0.41972E+00_JPRB, 0.42748E+00_JPRB, 0.43470E+00_JPRB, 0.44214E+00_JPRB, 0.45106E+00_JPRB ,&
&  0.45551E+00_JPRB, 0.46342E+00_JPRB, 0.47009E+00_JPRB, 0.48048E+00_JPRB, 0.47518E+00_JPRB, 0.46316E+00_JPRB, 0.45478E+00_JPRB ,&
&  0.44770E+00_JPRB, 0.44699E+00_JPRB, 0.43956E+00_JPRB, 0.43292E+00_JPRB, 0.42680E+00_JPRB, 0.42255E+00_JPRB, 0.41811E+00_JPRB ,&
&  0.41520E+00_JPRB, 0.41184E+00_JPRB, 0.40837E+00_JPRB, 0.40506E+00_JPRB, 0.40536E+00_JPRB, 0.40493E+00_JPRB, 0.40286E+00_JPRB ,&
&  0.39959E+00_JPRB, 0.39838E+00_JPRB, 0.39823E+00_JPRB, 0.39678E+00_JPRB, 0.39339E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.38835E+00_JPRB, 0.38416E+00_JPRB, 0.38038E+00_JPRB, 0.37566E+00_JPRB, 0.37068E+00_JPRB, 0.36632E+00_JPRB, 0.36108E+00_JPRB ,&
&  0.35703E+00_JPRB, 0.35389E+00_JPRB, 0.35152E+00_JPRB, 0.34991E+00_JPRB, 0.34880E+00_JPRB, 0.34807E+00_JPRB, 0.34833E+00_JPRB ,&
&  0.34808E+00_JPRB, 0.34882E+00_JPRB, 0.34960E+00_JPRB, 0.35064E+00_JPRB, 0.35159E+00_JPRB, 0.35230E+00_JPRB, 0.35285E+00_JPRB ,&
&  0.35113E+00_JPRB, 0.34600E+00_JPRB, 0.34245E+00_JPRB, 0.34949E+00_JPRB, 0.35157E+00_JPRB, 0.35693E+00_JPRB, 0.36462E+00_JPRB ,&
&  0.38357E+00_JPRB, 0.37174E+00_JPRB, 0.37695E+00_JPRB, 0.36000E+00_JPRB, 0.34616E+00_JPRB, 0.32830E+00_JPRB, 0.31335E+00_JPRB ,&
&  0.30744E+00_JPRB, 0.30067E+00_JPRB, 0.29720E+00_JPRB, 0.29730E+00_JPRB, 0.29931E+00_JPRB, 0.30539E+00_JPRB, 0.31299E+00_JPRB ,&
&  0.31715E+00_JPRB, 0.33187E+00_JPRB, 0.33321E+00_JPRB, 0.33310E+00_JPRB, 0.33458E+00_JPRB, 0.33563E+00_JPRB, 0.33273E+00_JPRB ,&
&  0.33250E+00_JPRB, 0.33143E+00_JPRB, 0.33070E+00_JPRB, 0.33280E+00_JPRB, 0.33799E+00_JPRB, 0.34096E+00_JPRB, 0.34403E+00_JPRB ,&
&  0.34766E+00_JPRB, 0.34959E+00_JPRB, 0.35022E+00_JPRB, 0.35082E+00_JPRB, 0.35209E+00_JPRB, 0.35380E+00_JPRB, 0.35630E+00_JPRB ,&
&  0.36009E+00_JPRB, 0.36726E+00_JPRB, 0.37363E+00_JPRB, 0.38461E+00_JPRB, 0.39541E+00_JPRB, 0.40920E+00_JPRB, 0.42118E+00_JPRB ,&
&  0.43184E+00_JPRB, 0.43990E+00_JPRB, 0.44765E+00_JPRB, 0.45575E+00_JPRB, 0.46247E+00_JPRB, 0.45884E+00_JPRB, 0.44875E+00_JPRB ,&
&  0.44232E+00_JPRB, 0.43240E+00_JPRB, 0.42520E+00_JPRB, 0.41854E+00_JPRB, 0.41307E+00_JPRB, 0.40976E+00_JPRB, 0.40659E+00_JPRB ,&
&  0.40452E+00_JPRB, 0.40227E+00_JPRB, 0.40044E+00_JPRB, 0.39932E+00_JPRB, 0.40079E+00_JPRB, 0.40111E+00_JPRB, 0.40015E+00_JPRB ,&
&  0.39869E+00_JPRB, 0.39886E+00_JPRB, 0.39764E+00_JPRB, 0.39496E+00_JPRB, 0.39168E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.38671E+00_JPRB, 0.38160E+00_JPRB, 0.37753E+00_JPRB, 0.37283E+00_JPRB, 0.36835E+00_JPRB, 0.36365E+00_JPRB, 0.36002E+00_JPRB ,&
&  0.35674E+00_JPRB, 0.35437E+00_JPRB, 0.35276E+00_JPRB, 0.35173E+00_JPRB, 0.35163E+00_JPRB, 0.35165E+00_JPRB, 0.35210E+00_JPRB ,&
&  0.35236E+00_JPRB, 0.35268E+00_JPRB, 0.35336E+00_JPRB, 0.35383E+00_JPRB, 0.35393E+00_JPRB, 0.35359E+00_JPRB, 0.35237E+00_JPRB ,&
&  0.34943E+00_JPRB, 0.34609E+00_JPRB, 0.33724E+00_JPRB, 0.34150E+00_JPRB, 0.34112E+00_JPRB, 0.34136E+00_JPRB, 0.34463E+00_JPRB ,&
&  0.34501E+00_JPRB, 0.34816E+00_JPRB, 0.34772E+00_JPRB, 0.34307E+00_JPRB, 0.33775E+00_JPRB, 0.32591E+00_JPRB, 0.31952E+00_JPRB ,&
&  0.31313E+00_JPRB, 0.30948E+00_JPRB, 0.30883E+00_JPRB, 0.30892E+00_JPRB, 0.31326E+00_JPRB, 0.31724E+00_JPRB, 0.32200E+00_JPRB ,&
&  0.32882E+00_JPRB, 0.33432E+00_JPRB, 0.34034E+00_JPRB, 0.33814E+00_JPRB, 0.33949E+00_JPRB, 0.34011E+00_JPRB, 0.34109E+00_JPRB ,&
&  0.34214E+00_JPRB, 0.34256E+00_JPRB, 0.34336E+00_JPRB, 0.34396E+00_JPRB, 0.34520E+00_JPRB, 0.34686E+00_JPRB, 0.34852E+00_JPRB ,&
&  0.35039E+00_JPRB, 0.35195E+00_JPRB, 0.35334E+00_JPRB, 0.35411E+00_JPRB, 0.35520E+00_JPRB, 0.35544E+00_JPRB, 0.35613E+00_JPRB ,&
&  0.35749E+00_JPRB, 0.35931E+00_JPRB, 0.36199E+00_JPRB, 0.36489E+00_JPRB, 0.36788E+00_JPRB, 0.37172E+00_JPRB, 0.37630E+00_JPRB ,&
&  0.38268E+00_JPRB, 0.38881E+00_JPRB, 0.39686E+00_JPRB, 0.40149E+00_JPRB, 0.41138E+00_JPRB, 0.42146E+00_JPRB, 0.42678E+00_JPRB ,&
&  0.42535E+00_JPRB, 0.42155E+00_JPRB, 0.41766E+00_JPRB, 0.41371E+00_JPRB, 0.41037E+00_JPRB, 0.40620E+00_JPRB, 0.40307E+00_JPRB ,&
&  0.39942E+00_JPRB, 0.39647E+00_JPRB, 0.39407E+00_JPRB, 0.39329E+00_JPRB, 0.39422E+00_JPRB, 0.39482E+00_JPRB, 0.39489E+00_JPRB ,&
&  0.39501E+00_JPRB, 0.39569E+00_JPRB, 0.39592E+00_JPRB, 0.39434E+00_JPRB, 0.39141E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.38170E+00_JPRB, 0.37944E+00_JPRB, 0.37681E+00_JPRB, 0.37352E+00_JPRB, 0.37085E+00_JPRB, 0.36829E+00_JPRB, 0.36622E+00_JPRB ,&
&  0.36384E+00_JPRB, 0.36113E+00_JPRB, 0.35915E+00_JPRB, 0.35795E+00_JPRB, 0.35710E+00_JPRB, 0.35670E+00_JPRB, 0.35673E+00_JPRB ,&
&  0.35681E+00_JPRB, 0.35668E+00_JPRB, 0.35692E+00_JPRB, 0.35707E+00_JPRB, 0.35712E+00_JPRB, 0.35668E+00_JPRB, 0.35479E+00_JPRB ,&
&  0.35263E+00_JPRB, 0.35025E+00_JPRB, 0.34774E+00_JPRB, 0.34567E+00_JPRB, 0.34542E+00_JPRB, 0.33349E+00_JPRB, 0.33330E+00_JPRB ,&
&  0.33886E+00_JPRB, 0.34226E+00_JPRB, 0.34726E+00_JPRB, 0.35080E+00_JPRB, 0.35082E+00_JPRB, 0.34711E+00_JPRB, 0.34508E+00_JPRB ,&
&  0.33341E+00_JPRB, 0.33642E+00_JPRB, 0.33622E+00_JPRB, 0.33648E+00_JPRB, 0.33854E+00_JPRB, 0.33854E+00_JPRB, 0.34107E+00_JPRB ,&
&  0.35029E+00_JPRB, 0.35047E+00_JPRB, 0.35119E+00_JPRB, 0.35113E+00_JPRB, 0.35077E+00_JPRB, 0.35041E+00_JPRB, 0.35051E+00_JPRB ,&
&  0.35041E+00_JPRB, 0.35085E+00_JPRB, 0.35190E+00_JPRB, 0.35202E+00_JPRB, 0.35229E+00_JPRB, 0.35305E+00_JPRB, 0.35395E+00_JPRB ,&
&  0.35418E+00_JPRB, 0.35454E+00_JPRB, 0.35503E+00_JPRB, 0.35552E+00_JPRB, 0.35640E+00_JPRB, 0.35695E+00_JPRB, 0.35767E+00_JPRB ,&
&  0.35788E+00_JPRB, 0.35750E+00_JPRB, 0.35766E+00_JPRB, 0.35776E+00_JPRB, 0.35773E+00_JPRB, 0.35732E+00_JPRB, 0.35788E+00_JPRB ,&
&  0.35861E+00_JPRB, 0.35933E+00_JPRB, 0.35991E+00_JPRB, 0.36112E+00_JPRB, 0.36358E+00_JPRB, 0.36694E+00_JPRB, 0.37292E+00_JPRB ,&
&  0.37780E+00_JPRB, 0.38157E+00_JPRB, 0.38277E+00_JPRB, 0.38381E+00_JPRB, 0.38510E+00_JPRB, 0.38506E+00_JPRB, 0.38490E+00_JPRB ,&
&  0.38390E+00_JPRB, 0.38266E+00_JPRB, 0.38136E+00_JPRB, 0.38035E+00_JPRB, 0.37927E+00_JPRB, 0.37826E+00_JPRB, 0.37830E+00_JPRB ,&
&  0.37987E+00_JPRB, 0.38188E+00_JPRB, 0.38357E+00_JPRB, 0.38378E+00_JPRB, 0.38320E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.37477E+00_JPRB, 0.37444E+00_JPRB, 0.37374E+00_JPRB, 0.37272E+00_JPRB, 0.37136E+00_JPRB, 0.36975E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.36651E+00_JPRB, 0.36503E+00_JPRB, 0.36348E+00_JPRB, 0.36202E+00_JPRB, 0.36069E+00_JPRB, 0.35988E+00_JPRB, 0.35938E+00_JPRB ,&
&  0.35936E+00_JPRB, 0.35944E+00_JPRB, 0.35962E+00_JPRB, 0.35938E+00_JPRB, 0.35899E+00_JPRB, 0.35818E+00_JPRB, 0.35718E+00_JPRB ,&
&  0.35617E+00_JPRB, 0.35485E+00_JPRB, 0.35335E+00_JPRB, 0.35125E+00_JPRB, 0.34962E+00_JPRB, 0.34828E+00_JPRB, 0.34725E+00_JPRB ,&
&  0.34643E+00_JPRB, 0.34580E+00_JPRB, 0.34562E+00_JPRB, 0.34576E+00_JPRB, 0.34751E+00_JPRB, 0.34873E+00_JPRB, 0.34956E+00_JPRB ,&
&  0.35020E+00_JPRB, 0.35004E+00_JPRB, 0.35012E+00_JPRB, 0.35032E+00_JPRB, 0.35058E+00_JPRB, 0.35121E+00_JPRB, 0.35133E+00_JPRB ,&
&  0.35151E+00_JPRB, 0.35162E+00_JPRB, 0.35196E+00_JPRB, 0.35219E+00_JPRB, 0.35238E+00_JPRB, 0.35280E+00_JPRB, 0.35229E+00_JPRB ,&
&  0.35236E+00_JPRB, 0.35324E+00_JPRB, 0.35446E+00_JPRB, 0.35578E+00_JPRB, 0.35678E+00_JPRB, 0.35754E+00_JPRB, 0.35810E+00_JPRB ,&
&  0.35854E+00_JPRB, 0.35922E+00_JPRB, 0.35987E+00_JPRB, 0.36042E+00_JPRB, 0.36072E+00_JPRB, 0.36116E+00_JPRB, 0.36129E+00_JPRB ,&
&  0.36126E+00_JPRB, 0.36117E+00_JPRB, 0.36141E+00_JPRB, 0.36151E+00_JPRB, 0.36122E+00_JPRB, 0.36085E+00_JPRB, 0.36099E+00_JPRB ,&
&  0.36086E+00_JPRB, 0.36056E+00_JPRB, 0.36023E+00_JPRB, 0.36089E+00_JPRB, 0.36132E+00_JPRB, 0.36147E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.36234E+00_JPRB, 0.36403E+00_JPRB, 0.36632E+00_JPRB, 0.36954E+00_JPRB, 0.37176E+00_JPRB, 0.37275E+00_JPRB, 0.37314E+00_JPRB ,&
&  0.37285E+00_JPRB, 0.37231E+00_JPRB, 0.37175E+00_JPRB, 0.37136E+00_JPRB, 0.37082E+00_JPRB, 0.37085E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.37201E+00_JPRB, 0.37251E+00_JPRB, 0.37330E+00_JPRB, 0.37414E+00_JPRB, 0.37482E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.36717E+00_JPRB, 0.36704E+00_JPRB, 0.36683E+00_JPRB, 0.36649E+00_JPRB, 0.36597E+00_JPRB, 0.36534E+00_JPRB, 0.36476E+00_JPRB ,&
&  0.36413E+00_JPRB, 0.36366E+00_JPRB, 0.36298E+00_JPRB, 0.36230E+00_JPRB, 0.36161E+00_JPRB, 0.36098E+00_JPRB, 0.36037E+00_JPRB ,&
&  0.35973E+00_JPRB, 0.35911E+00_JPRB, 0.35860E+00_JPRB, 0.35806E+00_JPRB, 0.35748E+00_JPRB, 0.35700E+00_JPRB, 0.35653E+00_JPRB ,&
&  0.35621E+00_JPRB, 0.35582E+00_JPRB, 0.35540E+00_JPRB, 0.35485E+00_JPRB, 0.35459E+00_JPRB, 0.35411E+00_JPRB, 0.35381E+00_JPRB ,&
&  0.35341E+00_JPRB, 0.35324E+00_JPRB, 0.35298E+00_JPRB, 0.35284E+00_JPRB, 0.35257E+00_JPRB, 0.35268E+00_JPRB, 0.35259E+00_JPRB ,&
&  0.35261E+00_JPRB, 0.35251E+00_JPRB, 0.35254E+00_JPRB, 0.35253E+00_JPRB, 0.35258E+00_JPRB, 0.35260E+00_JPRB, 0.35278E+00_JPRB ,&
&  0.35296E+00_JPRB, 0.35324E+00_JPRB, 0.35362E+00_JPRB, 0.35397E+00_JPRB, 0.35430E+00_JPRB, 0.35469E+00_JPRB, 0.35494E+00_JPRB ,&
&  0.35589E+00_JPRB, 0.35706E+00_JPRB, 0.35821E+00_JPRB, 0.35938E+00_JPRB, 0.36047E+00_JPRB, 0.36133E+00_JPRB, 0.36207E+00_JPRB ,&
&  0.36264E+00_JPRB, 0.36321E+00_JPRB, 0.36375E+00_JPRB, 0.36434E+00_JPRB, 0.36471E+00_JPRB, 0.36523E+00_JPRB, 0.36563E+00_JPRB ,&
&  0.36584E+00_JPRB, 0.36577E+00_JPRB, 0.36592E+00_JPRB, 0.36599E+00_JPRB, 0.36610E+00_JPRB, 0.36613E+00_JPRB, 0.36641E+00_JPRB ,&
&  0.36664E+00_JPRB, 0.36700E+00_JPRB, 0.36737E+00_JPRB, 0.36770E+00_JPRB, 0.36787E+00_JPRB, 0.36800E+00_JPRB, 0.36801E+00_JPRB ,&
&  0.36803E+00_JPRB, 0.36808E+00_JPRB, 0.36818E+00_JPRB, 0.36807E+00_JPRB, 0.36784E+00_JPRB, 0.36768E+00_JPRB, 0.36752E+00_JPRB ,&
&  0.36750E+00_JPRB, 0.36728E+00_JPRB, 0.36711E+00_JPRB, 0.36696E+00_JPRB, 0.36677E+00_JPRB, 0.36664E+00_JPRB, 0.36652E+00_JPRB ,&
&  0.36649E+00_JPRB, 0.36653E+00_JPRB, 0.36667E+00_JPRB, 0.36691E+00_JPRB, 0.36715E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB ,&
&  0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB, 0.36076E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1940:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1940
