!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1950 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1950* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB ,&
&  0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB, 0.13552E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.16789E+00_JPRB, 0.17025E+00_JPRB, 0.17181E+00_JPRB, 0.17279E+00_JPRB, 0.17256E+00_JPRB, 0.17347E+00_JPRB, 0.17444E+00_JPRB ,&
&  0.17571E+00_JPRB, 0.17726E+00_JPRB, 0.17830E+00_JPRB, 0.17926E+00_JPRB, 0.17974E+00_JPRB, 0.17972E+00_JPRB, 0.17970E+00_JPRB ,&
&  0.17950E+00_JPRB, 0.17927E+00_JPRB, 0.17654E+00_JPRB, 0.17408E+00_JPRB, 0.17237E+00_JPRB, 0.17148E+00_JPRB, 0.17063E+00_JPRB ,&
&  0.16989E+00_JPRB, 0.16878E+00_JPRB, 0.16738E+00_JPRB, 0.16520E+00_JPRB, 0.16204E+00_JPRB, 0.15897E+00_JPRB, 0.15585E+00_JPRB ,&
&  0.15305E+00_JPRB, 0.15070E+00_JPRB, 0.14854E+00_JPRB, 0.14633E+00_JPRB, 0.14491E+00_JPRB, 0.14334E+00_JPRB, 0.14175E+00_JPRB ,&
&  0.13981E+00_JPRB, 0.13776E+00_JPRB, 0.13577E+00_JPRB, 0.13375E+00_JPRB, 0.13180E+00_JPRB, 0.12986E+00_JPRB, 0.12844E+00_JPRB ,&
&  0.12695E+00_JPRB, 0.12580E+00_JPRB, 0.12454E+00_JPRB, 0.12339E+00_JPRB, 0.12219E+00_JPRB, 0.12106E+00_JPRB, 0.11983E+00_JPRB ,&
&  0.11899E+00_JPRB, 0.11811E+00_JPRB, 0.11723E+00_JPRB, 0.11640E+00_JPRB, 0.11558E+00_JPRB, 0.11480E+00_JPRB, 0.11389E+00_JPRB ,&
&  0.11308E+00_JPRB, 0.11256E+00_JPRB, 0.11200E+00_JPRB, 0.11138E+00_JPRB, 0.11081E+00_JPRB, 0.11041E+00_JPRB, 0.11006E+00_JPRB ,&
&  0.10974E+00_JPRB, 0.10970E+00_JPRB, 0.10985E+00_JPRB, 0.11000E+00_JPRB, 0.11015E+00_JPRB, 0.11051E+00_JPRB, 0.11105E+00_JPRB ,&
&  0.11169E+00_JPRB, 0.11238E+00_JPRB, 0.11328E+00_JPRB, 0.11455E+00_JPRB, 0.11596E+00_JPRB, 0.11757E+00_JPRB, 0.11934E+00_JPRB ,&
&  0.12124E+00_JPRB, 0.12330E+00_JPRB, 0.12549E+00_JPRB, 0.12771E+00_JPRB, 0.13006E+00_JPRB, 0.13243E+00_JPRB, 0.13470E+00_JPRB ,&
&  0.13707E+00_JPRB, 0.13937E+00_JPRB, 0.14168E+00_JPRB, 0.14378E+00_JPRB, 0.14686E+00_JPRB, 0.14887E+00_JPRB, 0.15104E+00_JPRB ,&
&  0.15310E+00_JPRB, 0.15547E+00_JPRB, 0.15787E+00_JPRB, 0.16086E+00_JPRB, 0.16382E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.19437E+00_JPRB, 0.19473E+00_JPRB, 0.19590E+00_JPRB, 0.20287E+00_JPRB, 0.21761E+00_JPRB, 0.29460E+00_JPRB, 0.28643E+00_JPRB ,&
&  0.27970E+00_JPRB, 0.25111E+00_JPRB, 0.24708E+00_JPRB, 0.23968E+00_JPRB, 0.22995E+00_JPRB, 0.22166E+00_JPRB, 0.21659E+00_JPRB ,&
&  0.21118E+00_JPRB, 0.20536E+00_JPRB, 0.20171E+00_JPRB, 0.19826E+00_JPRB, 0.19499E+00_JPRB, 0.19142E+00_JPRB, 0.18872E+00_JPRB ,&
&  0.18645E+00_JPRB, 0.18490E+00_JPRB, 0.18321E+00_JPRB, 0.18002E+00_JPRB, 0.17856E+00_JPRB, 0.17556E+00_JPRB, 0.17104E+00_JPRB ,&
&  0.16521E+00_JPRB, 0.16188E+00_JPRB, 0.15820E+00_JPRB, 0.15486E+00_JPRB, 0.15704E+00_JPRB, 0.15579E+00_JPRB, 0.15350E+00_JPRB ,&
&  0.15006E+00_JPRB, 0.14652E+00_JPRB, 0.14198E+00_JPRB, 0.13768E+00_JPRB, 0.13400E+00_JPRB, 0.13098E+00_JPRB, 0.12831E+00_JPRB ,&
&  0.12616E+00_JPRB, 0.12433E+00_JPRB, 0.12203E+00_JPRB, 0.11993E+00_JPRB, 0.11772E+00_JPRB, 0.11576E+00_JPRB, 0.11384E+00_JPRB ,&
&  0.11231E+00_JPRB, 0.11087E+00_JPRB, 0.10937E+00_JPRB, 0.10780E+00_JPRB, 0.10656E+00_JPRB, 0.10495E+00_JPRB, 0.10374E+00_JPRB ,&
&  0.10265E+00_JPRB, 0.10133E+00_JPRB, 0.10037E+00_JPRB, 0.99185E-01_JPRB, 0.98308E-01_JPRB, 0.97592E-01_JPRB, 0.96894E-01_JPRB ,&
&  0.95945E-01_JPRB, 0.95316E-01_JPRB, 0.95052E-01_JPRB, 0.94836E-01_JPRB, 0.94502E-01_JPRB, 0.94502E-01_JPRB, 0.94764E-01_JPRB ,&
&  0.95144E-01_JPRB, 0.95594E-01_JPRB, 0.96372E-01_JPRB, 0.97312E-01_JPRB, 0.98701E-01_JPRB, 0.10054E+00_JPRB, 0.10345E+00_JPRB ,&
&  0.10664E+00_JPRB, 0.11018E+00_JPRB, 0.11392E+00_JPRB, 0.11833E+00_JPRB, 0.12274E+00_JPRB, 0.12727E+00_JPRB, 0.13247E+00_JPRB ,&
&  0.13810E+00_JPRB, 0.14238E+00_JPRB, 0.14623E+00_JPRB, 0.15031E+00_JPRB, 0.15833E+00_JPRB, 0.16448E+00_JPRB, 0.16936E+00_JPRB ,&
&  0.17477E+00_JPRB, 0.18198E+00_JPRB, 0.18689E+00_JPRB, 0.19039E+00_JPRB, 0.19272E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.45163E+00_JPRB, 0.46304E+00_JPRB, 0.47810E+00_JPRB, 0.48720E+00_JPRB, 0.49681E+00_JPRB, 0.46406E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.38583E+00_JPRB, 0.35596E+00_JPRB, 0.33115E+00_JPRB, 0.30834E+00_JPRB, 0.28811E+00_JPRB, 0.27069E+00_JPRB, 0.25353E+00_JPRB ,&
&  0.24250E+00_JPRB, 0.23573E+00_JPRB, 0.22648E+00_JPRB, 0.21635E+00_JPRB, 0.20562E+00_JPRB, 0.19627E+00_JPRB, 0.19106E+00_JPRB ,&
&  0.18662E+00_JPRB, 0.18341E+00_JPRB, 0.18212E+00_JPRB, 0.18260E+00_JPRB, 0.18462E+00_JPRB, 0.18515E+00_JPRB, 0.18608E+00_JPRB ,&
&  0.18369E+00_JPRB, 0.19071E+00_JPRB, 0.19986E+00_JPRB, 0.20401E+00_JPRB, 0.21146E+00_JPRB, 0.20988E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.19840E+00_JPRB, 0.19018E+00_JPRB, 0.18116E+00_JPRB, 0.17001E+00_JPRB, 0.15951E+00_JPRB, 0.15155E+00_JPRB, 0.14384E+00_JPRB ,&
&  0.13907E+00_JPRB, 0.13369E+00_JPRB, 0.12774E+00_JPRB, 0.12268E+00_JPRB, 0.11831E+00_JPRB, 0.11460E+00_JPRB, 0.11128E+00_JPRB ,&
&  0.10870E+00_JPRB, 0.10707E+00_JPRB, 0.10549E+00_JPRB, 0.10481E+00_JPRB, 0.10362E+00_JPRB, 0.10252E+00_JPRB, 0.10145E+00_JPRB ,&
&  0.10081E+00_JPRB, 0.99675E-01_JPRB, 0.98818E-01_JPRB, 0.98049E-01_JPRB, 0.96915E-01_JPRB, 0.95923E-01_JPRB, 0.94784E-01_JPRB ,&
&  0.93767E-01_JPRB, 0.93182E-01_JPRB, 0.92843E-01_JPRB, 0.92710E-01_JPRB, 0.92879E-01_JPRB, 0.92799E-01_JPRB, 0.92967E-01_JPRB ,&
&  0.93203E-01_JPRB, 0.93566E-01_JPRB, 0.93586E-01_JPRB, 0.93846E-01_JPRB, 0.94612E-01_JPRB, 0.95669E-01_JPRB, 0.98178E-01_JPRB ,&
&  0.10110E+00_JPRB, 0.10488E+00_JPRB, 0.10905E+00_JPRB, 0.11402E+00_JPRB, 0.11904E+00_JPRB, 0.12417E+00_JPRB, 0.12981E+00_JPRB ,&
&  0.13712E+00_JPRB, 0.14367E+00_JPRB, 0.15112E+00_JPRB, 0.16232E+00_JPRB, 0.18523E+00_JPRB, 0.19869E+00_JPRB, 0.21435E+00_JPRB ,&
&  0.24199E+00_JPRB, 0.28110E+00_JPRB, 0.38420E+00_JPRB, 0.41731E+00_JPRB, 0.43447E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.62891E+00_JPRB, 0.65996E+00_JPRB, 0.68462E+00_JPRB, 0.68699E+00_JPRB, 0.68606E+00_JPRB, 0.64893E+00_JPRB, 0.59134E+00_JPRB ,&
&  0.48424E+00_JPRB, 0.41455E+00_JPRB, 0.37092E+00_JPRB, 0.32984E+00_JPRB, 0.29244E+00_JPRB, 0.26753E+00_JPRB, 0.25039E+00_JPRB ,&
&  0.23735E+00_JPRB, 0.23022E+00_JPRB, 0.22477E+00_JPRB, 0.22018E+00_JPRB, 0.21528E+00_JPRB, 0.21054E+00_JPRB, 0.20182E+00_JPRB ,&
&  0.19664E+00_JPRB, 0.19303E+00_JPRB, 0.19175E+00_JPRB, 0.19422E+00_JPRB, 0.20111E+00_JPRB, 0.20851E+00_JPRB, 0.21940E+00_JPRB ,&
&  0.23350E+00_JPRB, 0.24714E+00_JPRB, 0.25349E+00_JPRB, 0.25655E+00_JPRB, 0.25937E+00_JPRB, 0.25728E+00_JPRB, 0.23425E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.26297E+00_JPRB, 0.28137E+00_JPRB, 0.29256E+00_JPRB, 0.24914E+00_JPRB, 0.20791E+00_JPRB, 0.16745E+00_JPRB ,&
&  0.14769E+00_JPRB, 0.13275E+00_JPRB, 0.12510E+00_JPRB, 0.11941E+00_JPRB, 0.11547E+00_JPRB, 0.11294E+00_JPRB, 0.11070E+00_JPRB ,&
&  0.10858E+00_JPRB, 0.10755E+00_JPRB, 0.10608E+00_JPRB, 0.10531E+00_JPRB, 0.10428E+00_JPRB, 0.10401E+00_JPRB, 0.10281E+00_JPRB ,&
&  0.10158E+00_JPRB, 0.10062E+00_JPRB, 0.10002E+00_JPRB, 0.99718E-01_JPRB, 0.99016E-01_JPRB, 0.98304E-01_JPRB, 0.96730E-01_JPRB ,&
&  0.95580E-01_JPRB, 0.94425E-01_JPRB, 0.93521E-01_JPRB, 0.93453E-01_JPRB, 0.93779E-01_JPRB, 0.95181E-01_JPRB, 0.95970E-01_JPRB ,&
&  0.96143E-01_JPRB, 0.96192E-01_JPRB, 0.96261E-01_JPRB, 0.96908E-01_JPRB, 0.97895E-01_JPRB, 0.99617E-01_JPRB, 0.10200E+00_JPRB ,&
&  0.10486E+00_JPRB, 0.10918E+00_JPRB, 0.11364E+00_JPRB, 0.11869E+00_JPRB, 0.12353E+00_JPRB, 0.12849E+00_JPRB, 0.13482E+00_JPRB ,&
&  0.14136E+00_JPRB, 0.14882E+00_JPRB, 0.15966E+00_JPRB, 0.17149E+00_JPRB, 0.19249E+00_JPRB, 0.21198E+00_JPRB, 0.24434E+00_JPRB ,&
&  0.31566E+00_JPRB, 0.46354E+00_JPRB, 0.54865E+00_JPRB, 0.55877E+00_JPRB, 0.59144E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.74925E+00_JPRB, 0.76764E+00_JPRB, 0.78409E+00_JPRB, 0.78150E+00_JPRB, 0.76428E+00_JPRB, 0.70658E+00_JPRB, 0.64320E+00_JPRB ,&
&  0.56451E+00_JPRB, 0.49814E+00_JPRB, 0.42893E+00_JPRB, 0.36877E+00_JPRB, 0.31853E+00_JPRB, 0.28001E+00_JPRB, 0.25756E+00_JPRB ,&
&  0.24185E+00_JPRB, 0.23506E+00_JPRB, 0.23282E+00_JPRB, 0.23745E+00_JPRB, 0.24010E+00_JPRB, 0.24554E+00_JPRB, 0.24000E+00_JPRB ,&
&  0.23312E+00_JPRB, 0.22494E+00_JPRB, 0.22305E+00_JPRB, 0.23188E+00_JPRB, 0.24491E+00_JPRB, 0.25765E+00_JPRB, 0.27418E+00_JPRB ,&
&  0.29254E+00_JPRB, 0.29841E+00_JPRB, 0.31213E+00_JPRB, 0.31594E+00_JPRB, 0.30110E+00_JPRB, 0.30263E+00_JPRB, 0.30838E+00_JPRB ,&
&  0.32138E+00_JPRB, 0.33985E+00_JPRB, 0.35045E+00_JPRB, 0.37075E+00_JPRB, 0.31886E+00_JPRB, 0.25871E+00_JPRB, 0.20740E+00_JPRB ,&
&  0.16938E+00_JPRB, 0.14785E+00_JPRB, 0.13320E+00_JPRB, 0.12569E+00_JPRB, 0.11877E+00_JPRB, 0.11496E+00_JPRB, 0.11086E+00_JPRB ,&
&  0.10902E+00_JPRB, 0.10880E+00_JPRB, 0.10819E+00_JPRB, 0.10773E+00_JPRB, 0.10792E+00_JPRB, 0.10835E+00_JPRB, 0.10772E+00_JPRB ,&
&  0.10689E+00_JPRB, 0.10654E+00_JPRB, 0.10605E+00_JPRB, 0.10572E+00_JPRB, 0.10494E+00_JPRB, 0.10480E+00_JPRB, 0.10349E+00_JPRB ,&
&  0.10282E+00_JPRB, 0.10161E+00_JPRB, 0.10090E+00_JPRB, 0.10074E+00_JPRB, 0.10100E+00_JPRB, 0.10129E+00_JPRB, 0.10178E+00_JPRB ,&
&  0.10279E+00_JPRB, 0.10372E+00_JPRB, 0.10530E+00_JPRB, 0.10693E+00_JPRB, 0.10949E+00_JPRB, 0.11191E+00_JPRB, 0.11533E+00_JPRB ,&
&  0.11894E+00_JPRB, 0.12330E+00_JPRB, 0.12820E+00_JPRB, 0.13367E+00_JPRB, 0.13854E+00_JPRB, 0.14335E+00_JPRB, 0.14866E+00_JPRB ,&
&  0.15291E+00_JPRB, 0.15961E+00_JPRB, 0.16717E+00_JPRB, 0.17972E+00_JPRB, 0.20575E+00_JPRB, 0.22968E+00_JPRB, 0.27895E+00_JPRB ,&
&  0.36385E+00_JPRB, 0.61911E+00_JPRB, 0.84963E+00_JPRB, 0.70001E+00_JPRB, 0.73088E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.75752E+00_JPRB, 0.77011E+00_JPRB, 0.78728E+00_JPRB, 0.76376E+00_JPRB, 0.74838E+00_JPRB, 0.75175E+00_JPRB, 0.75256E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.69481E+00_JPRB, 0.64603E+00_JPRB, 0.60623E+00_JPRB, 0.54024E+00_JPRB, 0.45565E+00_JPRB, 0.41810E+00_JPRB ,&
&  0.38417E+00_JPRB, 0.38078E+00_JPRB, 0.38321E+00_JPRB, 0.39923E+00_JPRB, 0.40813E+00_JPRB, 0.40472E+00_JPRB, 0.38320E+00_JPRB ,&
&  0.35617E+00_JPRB, 0.32892E+00_JPRB, 0.31158E+00_JPRB, 0.29761E+00_JPRB, 0.29739E+00_JPRB, 0.29569E+00_JPRB, 0.28925E+00_JPRB ,&
&  0.28057E+00_JPRB, 0.28464E+00_JPRB, 0.30183E+00_JPRB, 0.33073E+00_JPRB, 0.38109E+00_JPRB, 0.36105E+00_JPRB, 0.36674E+00_JPRB ,&
&  0.39655E+00_JPRB, 0.41796E+00_JPRB, 0.42636E+00_JPRB, 0.42933E+00_JPRB, 0.42279E+00_JPRB, 0.44830E+00_JPRB, 0.39181E+00_JPRB ,&
&  0.28451E+00_JPRB, 0.22569E+00_JPRB, 0.18388E+00_JPRB, 0.15733E+00_JPRB, 0.14006E+00_JPRB, 0.13013E+00_JPRB, 0.11942E+00_JPRB ,&
&  0.11456E+00_JPRB, 0.11107E+00_JPRB, 0.11016E+00_JPRB, 0.11134E+00_JPRB, 0.11355E+00_JPRB, 0.11638E+00_JPRB, 0.11736E+00_JPRB ,&
&  0.11892E+00_JPRB, 0.11894E+00_JPRB, 0.11809E+00_JPRB, 0.11781E+00_JPRB, 0.11685E+00_JPRB, 0.11747E+00_JPRB, 0.11752E+00_JPRB ,&
&  0.11772E+00_JPRB, 0.11790E+00_JPRB, 0.11943E+00_JPRB, 0.11981E+00_JPRB, 0.12066E+00_JPRB, 0.11774E+00_JPRB, 0.11589E+00_JPRB ,&
&  0.11587E+00_JPRB, 0.11770E+00_JPRB, 0.12080E+00_JPRB, 0.12442E+00_JPRB, 0.12938E+00_JPRB, 0.13344E+00_JPRB, 0.13808E+00_JPRB ,&
&  0.14216E+00_JPRB, 0.14612E+00_JPRB, 0.14999E+00_JPRB, 0.15395E+00_JPRB, 0.15813E+00_JPRB, 0.16235E+00_JPRB, 0.16679E+00_JPRB ,&
&  0.17135E+00_JPRB, 0.17930E+00_JPRB, 0.18914E+00_JPRB, 0.20635E+00_JPRB, 0.22964E+00_JPRB, 0.25698E+00_JPRB, 0.30473E+00_JPRB ,&
&  0.36645E+00_JPRB, 0.59637E+00_JPRB, 0.89448E+00_JPRB, 0.72059E+00_JPRB, 0.73860E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.80730E+00_JPRB, 0.80523E+00_JPRB, 0.81339E+00_JPRB, 0.80305E+00_JPRB, 0.78392E+00_JPRB, 0.77479E+00_JPRB, 0.79175E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.78221E+00_JPRB, 0.76074E+00_JPRB, 0.74160E+00_JPRB, 0.72818E+00_JPRB, 0.71453E+00_JPRB, 0.69225E+00_JPRB ,&
&  0.67220E+00_JPRB, 0.66496E+00_JPRB, 0.66324E+00_JPRB, 0.65830E+00_JPRB, 0.64807E+00_JPRB, 0.63802E+00_JPRB, 0.62028E+00_JPRB ,&
&  0.58787E+00_JPRB, 0.55399E+00_JPRB, 0.53605E+00_JPRB, 0.51259E+00_JPRB, 0.49585E+00_JPRB, 0.47658E+00_JPRB, 0.44910E+00_JPRB ,&
&  0.40461E+00_JPRB, 0.34662E+00_JPRB, 0.30949E+00_JPRB, 0.34554E+00_JPRB, 0.44190E+00_JPRB, 0.43624E+00_JPRB, 0.45492E+00_JPRB ,&
&  0.48941E+00_JPRB, 0.51062E+00_JPRB, 0.52087E+00_JPRB, 0.52403E+00_JPRB, 0.53296E+00_JPRB, 0.53814E+00_JPRB, 0.53460E+00_JPRB ,&
&  0.52029E+00_JPRB, 0.46813E+00_JPRB, 0.42611E+00_JPRB, 0.32394E+00_JPRB, 0.25211E+00_JPRB, 0.21434E+00_JPRB, 0.18572E+00_JPRB ,&
&  0.16738E+00_JPRB, 0.15633E+00_JPRB, 0.15384E+00_JPRB, 0.15412E+00_JPRB, 0.15926E+00_JPRB, 0.16419E+00_JPRB, 0.16415E+00_JPRB ,&
&  0.16068E+00_JPRB, 0.15328E+00_JPRB, 0.14232E+00_JPRB, 0.13554E+00_JPRB, 0.13214E+00_JPRB, 0.13232E+00_JPRB, 0.13428E+00_JPRB ,&
&  0.13617E+00_JPRB, 0.14071E+00_JPRB, 0.15312E+00_JPRB, 0.16420E+00_JPRB, 0.17771E+00_JPRB, 0.15906E+00_JPRB, 0.14415E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.14300E+00_JPRB, 0.14916E+00_JPRB, 0.15385E+00_JPRB, 0.15886E+00_JPRB, 0.16328E+00_JPRB, 0.16780E+00_JPRB ,&
&  0.17202E+00_JPRB, 0.17497E+00_JPRB, 0.17659E+00_JPRB, 0.17832E+00_JPRB, 0.17991E+00_JPRB, 0.18225E+00_JPRB, 0.18524E+00_JPRB ,&
&  0.19029E+00_JPRB, 0.19940E+00_JPRB, 0.21132E+00_JPRB, 0.23091E+00_JPRB, 0.24935E+00_JPRB, 0.26782E+00_JPRB, 0.28886E+00_JPRB ,&
&  0.31015E+00_JPRB, 0.38990E+00_JPRB, 0.58393E+00_JPRB, 0.90608E+00_JPRB, 0.89941E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.80091E+00_JPRB, 0.82092E+00_JPRB, 0.83196E+00_JPRB, 0.84600E+00_JPRB, 0.84621E+00_JPRB, 0.81589E+00_JPRB, 0.80110E+00_JPRB ,&
&  0.80578E+00_JPRB, 0.80099E+00_JPRB, 0.79037E+00_JPRB, 0.78395E+00_JPRB, 0.77506E+00_JPRB, 0.76518E+00_JPRB, 0.75664E+00_JPRB ,&
&  0.74835E+00_JPRB, 0.74348E+00_JPRB, 0.75066E+00_JPRB, 0.76487E+00_JPRB, 0.77662E+00_JPRB, 0.78317E+00_JPRB, 0.78825E+00_JPRB ,&
&  0.78772E+00_JPRB, 0.75461E+00_JPRB, 0.70968E+00_JPRB, 0.67696E+00_JPRB, 0.66042E+00_JPRB, 0.64669E+00_JPRB, 0.61403E+00_JPRB ,&
&  0.54761E+00_JPRB, 0.42588E+00_JPRB, 0.36851E+00_JPRB, 0.40930E+00_JPRB, 0.52927E+00_JPRB, 0.49327E+00_JPRB, 0.51341E+00_JPRB ,&
&  0.54359E+00_JPRB, 0.55920E+00_JPRB, 0.55493E+00_JPRB, 0.54439E+00_JPRB, 0.54085E+00_JPRB, 0.54437E+00_JPRB, 0.54087E+00_JPRB ,&
&  0.53721E+00_JPRB, 0.52102E+00_JPRB, 0.51141E+00_JPRB, 0.50093E+00_JPRB, 0.48365E+00_JPRB, 0.45798E+00_JPRB, 0.44263E+00_JPRB ,&
&  0.40500E+00_JPRB, 0.36790E+00_JPRB, 0.35741E+00_JPRB, 0.35870E+00_JPRB, 0.32813E+00_JPRB, 0.31584E+00_JPRB, 0.30157E+00_JPRB ,&
&  0.26117E+00_JPRB, 0.23973E+00_JPRB, 0.20720E+00_JPRB, 0.18659E+00_JPRB, 0.17139E+00_JPRB, 0.15787E+00_JPRB, 0.15605E+00_JPRB ,&
&  0.16048E+00_JPRB, 0.17240E+00_JPRB, 0.19414E+00_JPRB, 0.22941E+00_JPRB, 0.26196E+00_JPRB, 0.21640E+00_JPRB, 0.19757E+00_JPRB ,&
&  0.18382E+00_JPRB, 0.18076E+00_JPRB, 0.18568E+00_JPRB, 0.19159E+00_JPRB, 0.19784E+00_JPRB, 0.20530E+00_JPRB, 0.21233E+00_JPRB ,&
&  0.21740E+00_JPRB, 0.22048E+00_JPRB, 0.22156E+00_JPRB, 0.22096E+00_JPRB, 0.22024E+00_JPRB, 0.22130E+00_JPRB, 0.22486E+00_JPRB ,&
&  0.23074E+00_JPRB, 0.24275E+00_JPRB, 0.26646E+00_JPRB, 0.30153E+00_JPRB, 0.34769E+00_JPRB, 0.40787E+00_JPRB, 0.50044E+00_JPRB ,&
&  0.62559E+00_JPRB, 0.79036E+00_JPRB, 0.83390E+00_JPRB, 0.84281E+00_JPRB, 0.81240E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.83962E+00_JPRB, 0.85029E+00_JPRB, 0.84816E+00_JPRB, 0.84715E+00_JPRB, 0.84899E+00_JPRB, 0.84198E+00_JPRB, 0.82765E+00_JPRB ,&
&  0.81339E+00_JPRB, 0.78688E+00_JPRB, 0.75661E+00_JPRB, 0.74309E+00_JPRB, 0.75080E+00_JPRB, 0.75080E+00_JPRB, 0.73208E+00_JPRB ,&
&  0.73025E+00_JPRB, 0.74414E+00_JPRB, 0.76723E+00_JPRB, 0.79304E+00_JPRB, 0.82790E+00_JPRB, 0.86408E+00_JPRB, 0.89119E+00_JPRB ,&
&  0.90182E+00_JPRB, 0.89976E+00_JPRB, 0.89216E+00_JPRB, 0.86566E+00_JPRB, 0.83021E+00_JPRB, 0.80304E+00_JPRB, 0.78247E+00_JPRB ,&
&  0.75706E+00_JPRB, 0.71000E+00_JPRB, 0.64007E+00_JPRB, 0.63465E+00_JPRB, 0.57006E+00_JPRB, 0.54892E+00_JPRB, 0.57062E+00_JPRB ,&
&  0.58578E+00_JPRB, 0.56768E+00_JPRB, 0.54391E+00_JPRB, 0.52423E+00_JPRB, 0.52050E+00_JPRB, 0.51920E+00_JPRB, 0.51420E+00_JPRB ,&
&  0.51032E+00_JPRB, 0.50352E+00_JPRB, 0.48939E+00_JPRB, 0.47633E+00_JPRB, 0.46396E+00_JPRB, 0.45162E+00_JPRB, 0.44329E+00_JPRB ,&
&  0.41556E+00_JPRB, 0.37312E+00_JPRB, 0.34169E+00_JPRB, 0.32824E+00_JPRB, 0.32088E+00_JPRB, 0.32455E+00_JPRB, 0.33699E+00_JPRB ,&
&  0.34681E+00_JPRB, 0.34699E+00_JPRB, 0.34117E+00_JPRB, 0.32898E+00_JPRB, 0.31103E+00_JPRB, 0.26501E+00_JPRB, 0.23899E+00_JPRB ,&
&  0.25431E+00_JPRB, 0.28877E+00_JPRB, 0.32442E+00_JPRB, 0.33234E+00_JPRB, 0.34186E+00_JPRB, 0.34450E+00_JPRB, 0.33024E+00_JPRB ,&
&  0.30250E+00_JPRB, 0.27059E+00_JPRB, 0.26385E+00_JPRB, 0.26579E+00_JPRB, 0.26973E+00_JPRB, 0.27745E+00_JPRB, 0.29313E+00_JPRB ,&
&  0.31691E+00_JPRB, 0.35590E+00_JPRB, 0.37780E+00_JPRB, 0.38035E+00_JPRB, 0.37302E+00_JPRB, 0.36137E+00_JPRB, 0.35956E+00_JPRB ,&
&  0.37325E+00_JPRB, 0.43433E+00_JPRB, 0.55655E+00_JPRB, 0.70475E+00_JPRB, 0.70292E+00_JPRB, 0.74665E+00_JPRB, 0.78147E+00_JPRB ,&
&  0.81119E+00_JPRB, 0.82447E+00_JPRB, 0.81106E+00_JPRB, 0.81259E+00_JPRB, 0.81948E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.85256E+00_JPRB, 0.85718E+00_JPRB, 0.84774E+00_JPRB, 0.84361E+00_JPRB, 0.83312E+00_JPRB, 0.81969E+00_JPRB, 0.79732E+00_JPRB ,&
&  0.77085E+00_JPRB, 0.74261E+00_JPRB, 0.71607E+00_JPRB, 0.69436E+00_JPRB, 0.69846E+00_JPRB, 0.73120E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.74768E+00_JPRB, 0.74366E+00_JPRB, 0.76078E+00_JPRB, 0.79150E+00_JPRB, 0.82507E+00_JPRB, 0.85613E+00_JPRB, 0.87871E+00_JPRB ,&
&  0.89708E+00_JPRB, 0.87888E+00_JPRB, 0.88236E+00_JPRB, 0.88431E+00_JPRB, 0.87835E+00_JPRB, 0.86064E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.80394E+00_JPRB, 0.77706E+00_JPRB, 0.74362E+00_JPRB, 0.68422E+00_JPRB, 0.75775E+00_JPRB, 0.68005E+00_JPRB, 0.65092E+00_JPRB ,&
&  0.62337E+00_JPRB, 0.60487E+00_JPRB, 0.58429E+00_JPRB, 0.56212E+00_JPRB, 0.55440E+00_JPRB, 0.54980E+00_JPRB, 0.54198E+00_JPRB ,&
&  0.53552E+00_JPRB, 0.52866E+00_JPRB, 0.51819E+00_JPRB, 0.50459E+00_JPRB, 0.49172E+00_JPRB, 0.47788E+00_JPRB, 0.46523E+00_JPRB ,&
&  0.44709E+00_JPRB, 0.42072E+00_JPRB, 0.39307E+00_JPRB, 0.37293E+00_JPRB, 0.36256E+00_JPRB, 0.37167E+00_JPRB, 0.40367E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.44218E+00_JPRB, 0.45166E+00_JPRB, 0.45409E+00_JPRB, 0.44544E+00_JPRB, 0.43565E+00_JPRB, 0.41820E+00_JPRB ,&
&  0.40456E+00_JPRB, 0.40214E+00_JPRB, 0.40294E+00_JPRB, 0.40641E+00_JPRB, 0.41172E+00_JPRB, 0.41919E+00_JPRB, 0.42960E+00_JPRB ,&
&  0.43918E+00_JPRB, 0.44233E+00_JPRB, 0.45440E+00_JPRB, 0.48101E+00_JPRB, 0.51161E+00_JPRB, 0.54263E+00_JPRB, 0.56893E+00_JPRB ,&
&  0.58655E+00_JPRB, 0.59861E+00_JPRB, 0.60665E+00_JPRB, 0.62179E+00_JPRB, 0.63739E+00_JPRB, 0.65327E+00_JPRB, 0.66157E+00_JPRB ,&
&  0.65925E+00_JPRB, 0.66262E+00_JPRB, 0.67928E+00_JPRB, 0.70723E+00_JPRB, 0.72688E+00_JPRB, 0.76067E+00_JPRB, 0.78181E+00_JPRB ,&
&  0.80434E+00_JPRB, 0.81028E+00_JPRB, 0.80500E+00_JPRB, 0.81764E+00_JPRB, 0.83463E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.81181E+00_JPRB, 0.82972E+00_JPRB, 0.84195E+00_JPRB, 0.84314E+00_JPRB, 0.82792E+00_JPRB, 0.80133E+00_JPRB, 0.77846E+00_JPRB ,&
&  0.75231E+00_JPRB, 0.72687E+00_JPRB, 0.70269E+00_JPRB, 0.66996E+00_JPRB, 0.65234E+00_JPRB, 0.65565E+00_JPRB, 0.70455E+00_JPRB ,&
&  0.76971E+00_JPRB, 0.77452E+00_JPRB, 0.76844E+00_JPRB, 0.79381E+00_JPRB, 0.84661E+00_JPRB, 0.87873E+00_JPRB, 0.90004E+00_JPRB ,&
&  0.90370E+00_JPRB, 0.87116E+00_JPRB, 0.83794E+00_JPRB, 0.82419E+00_JPRB, 0.84734E+00_JPRB, 0.86520E+00_JPRB, 0.87059E+00_JPRB ,&
&  0.86642E+00_JPRB, 0.84503E+00_JPRB, 0.82548E+00_JPRB, 0.80894E+00_JPRB, 0.81293E+00_JPRB, 0.78811E+00_JPRB, 0.73578E+00_JPRB ,&
&  0.66939E+00_JPRB, 0.64264E+00_JPRB, 0.63099E+00_JPRB, 0.61627E+00_JPRB, 0.60513E+00_JPRB, 0.60513E+00_JPRB, 0.59579E+00_JPRB ,&
&  0.58230E+00_JPRB, 0.56681E+00_JPRB, 0.55195E+00_JPRB, 0.53426E+00_JPRB, 0.52036E+00_JPRB, 0.50089E+00_JPRB, 0.48275E+00_JPRB ,&
&  0.46595E+00_JPRB, 0.44554E+00_JPRB, 0.42689E+00_JPRB, 0.41410E+00_JPRB, 0.40871E+00_JPRB, 0.41880E+00_JPRB, 0.45816E+00_JPRB ,&
&  0.48924E+00_JPRB, 0.48614E+00_JPRB, 0.48355E+00_JPRB, 0.48711E+00_JPRB, 0.47959E+00_JPRB, 0.46874E+00_JPRB, 0.46285E+00_JPRB ,&
&  0.45771E+00_JPRB, 0.45361E+00_JPRB, 0.45174E+00_JPRB, 0.45357E+00_JPRB, 0.45572E+00_JPRB, 0.45393E+00_JPRB, 0.45661E+00_JPRB ,&
&  0.46157E+00_JPRB, 0.46690E+00_JPRB, 0.48393E+00_JPRB, 0.50837E+00_JPRB, 0.52872E+00_JPRB, 0.54289E+00_JPRB, 0.55600E+00_JPRB ,&
&  0.57657E+00_JPRB, 0.58920E+00_JPRB, 0.57639E+00_JPRB, 0.56415E+00_JPRB, 0.55764E+00_JPRB, 0.56258E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.59369E+00_JPRB, 0.60669E+00_JPRB, 0.61282E+00_JPRB, 0.61440E+00_JPRB, 0.61777E+00_JPRB, 0.61872E+00_JPRB, 0.63130E+00_JPRB ,&
&  0.66499E+00_JPRB, 0.69406E+00_JPRB, 0.70164E+00_JPRB, 0.73453E+00_JPRB, 0.77661E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.74301E+00_JPRB, 0.77491E+00_JPRB, 0.79960E+00_JPRB, 0.80912E+00_JPRB, 0.80755E+00_JPRB, 0.79895E+00_JPRB, 0.77635E+00_JPRB ,&
&  0.75173E+00_JPRB, 0.73416E+00_JPRB, 0.71547E+00_JPRB, 0.68882E+00_JPRB, 0.66669E+00_JPRB, 0.66198E+00_JPRB, 0.67444E+00_JPRB ,&
&  0.72067E+00_JPRB, 0.76172E+00_JPRB, 0.77840E+00_JPRB, 0.79224E+00_JPRB, 0.84239E+00_JPRB, 0.88461E+00_JPRB, 0.89427E+00_JPRB ,&
&  0.89012E+00_JPRB, 0.89234E+00_JPRB, 0.85607E+00_JPRB, 0.82287E+00_JPRB, 0.82279E+00_JPRB, 0.82807E+00_JPRB, 0.80994E+00_JPRB ,&
&  0.83615E+00_JPRB, 0.85307E+00_JPRB, 0.84626E+00_JPRB, 0.81993E+00_JPRB, 0.81179E+00_JPRB, 0.80206E+00_JPRB, 0.79146E+00_JPRB ,&
&  0.76968E+00_JPRB, 0.72089E+00_JPRB, 0.69083E+00_JPRB, 0.68919E+00_JPRB, 0.68178E+00_JPRB, 0.67425E+00_JPRB, 0.66063E+00_JPRB ,&
&  0.62068E+00_JPRB, 0.58482E+00_JPRB, 0.55789E+00_JPRB, 0.53925E+00_JPRB, 0.52755E+00_JPRB, 0.51858E+00_JPRB, 0.50941E+00_JPRB ,&
&  0.50162E+00_JPRB, 0.47522E+00_JPRB, 0.44825E+00_JPRB, 0.44320E+00_JPRB, 0.44968E+00_JPRB, 0.45649E+00_JPRB, 0.48293E+00_JPRB ,&
&  0.50785E+00_JPRB, 0.50616E+00_JPRB, 0.50230E+00_JPRB, 0.50422E+00_JPRB, 0.51251E+00_JPRB, 0.51212E+00_JPRB, 0.50158E+00_JPRB ,&
&  0.49616E+00_JPRB, 0.49338E+00_JPRB, 0.48386E+00_JPRB, 0.47698E+00_JPRB, 0.47634E+00_JPRB, 0.48055E+00_JPRB, 0.49537E+00_JPRB ,&
&  0.50456E+00_JPRB, 0.50609E+00_JPRB, 0.51980E+00_JPRB, 0.53706E+00_JPRB, 0.54727E+00_JPRB, 0.55521E+00_JPRB, 0.56076E+00_JPRB ,&
&  0.57400E+00_JPRB, 0.59338E+00_JPRB, 0.60829E+00_JPRB, 0.61022E+00_JPRB, 0.58716E+00_JPRB, 0.56287E+00_JPRB, 0.54376E+00_JPRB ,&
&  0.53560E+00_JPRB, 0.54014E+00_JPRB, 0.54758E+00_JPRB, 0.55460E+00_JPRB, 0.56104E+00_JPRB, 0.56518E+00_JPRB, 0.56420E+00_JPRB ,&
&  0.58363E+00_JPRB, 0.61477E+00_JPRB, 0.64016E+00_JPRB, 0.67064E+00_JPRB, 0.70241E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.70193E+00_JPRB, 0.71628E+00_JPRB, 0.72794E+00_JPRB, 0.74935E+00_JPRB, 0.76898E+00_JPRB, 0.76651E+00_JPRB, 0.75930E+00_JPRB ,&
&  0.75383E+00_JPRB, 0.74872E+00_JPRB, 0.74112E+00_JPRB, 0.73120E+00_JPRB, 0.72452E+00_JPRB, 0.72389E+00_JPRB, 0.72480E+00_JPRB ,&
&  0.72670E+00_JPRB, 0.74708E+00_JPRB, 0.78051E+00_JPRB, 0.80684E+00_JPRB, 0.83553E+00_JPRB, 0.87935E+00_JPRB, 0.90028E+00_JPRB ,&
&  0.88447E+00_JPRB, 0.88410E+00_JPRB, 0.87346E+00_JPRB, 0.85904E+00_JPRB, 0.86089E+00_JPRB, 0.85162E+00_JPRB, 0.82359E+00_JPRB ,&
&  0.81774E+00_JPRB, 0.81046E+00_JPRB, 0.80698E+00_JPRB, 0.83425E+00_JPRB, 0.85513E+00_JPRB, 0.85497E+00_JPRB, 0.85246E+00_JPRB ,&
&  0.83221E+00_JPRB, 0.79422E+00_JPRB, 0.75863E+00_JPRB, 0.74210E+00_JPRB, 0.74014E+00_JPRB, 0.71992E+00_JPRB, 0.69177E+00_JPRB ,&
&  0.63435E+00_JPRB, 0.58724E+00_JPRB, 0.56111E+00_JPRB, 0.54570E+00_JPRB, 0.54061E+00_JPRB, 0.53797E+00_JPRB, 0.53371E+00_JPRB ,&
&  0.53929E+00_JPRB, 0.53456E+00_JPRB, 0.50776E+00_JPRB, 0.47997E+00_JPRB, 0.47598E+00_JPRB, 0.48480E+00_JPRB, 0.50372E+00_JPRB ,&
&  0.51881E+00_JPRB, 0.51482E+00_JPRB, 0.52159E+00_JPRB, 0.52873E+00_JPRB, 0.54518E+00_JPRB, 0.56070E+00_JPRB, 0.56113E+00_JPRB ,&
&  0.56034E+00_JPRB, 0.55460E+00_JPRB, 0.54112E+00_JPRB, 0.53267E+00_JPRB, 0.52181E+00_JPRB, 0.51801E+00_JPRB, 0.53005E+00_JPRB ,&
&  0.54775E+00_JPRB, 0.55759E+00_JPRB, 0.56865E+00_JPRB, 0.58199E+00_JPRB, 0.59225E+00_JPRB, 0.60038E+00_JPRB, 0.60716E+00_JPRB ,&
&  0.61742E+00_JPRB, 0.63195E+00_JPRB, 0.64654E+00_JPRB, 0.65923E+00_JPRB, 0.65914E+00_JPRB, 0.64511E+00_JPRB, 0.61991E+00_JPRB ,&
&  0.60291E+00_JPRB, 0.58941E+00_JPRB, 0.58585E+00_JPRB, 0.59053E+00_JPRB, 0.58516E+00_JPRB, 0.56547E+00_JPRB, 0.55578E+00_JPRB ,&
&  0.56673E+00_JPRB, 0.59048E+00_JPRB, 0.61343E+00_JPRB, 0.64475E+00_JPRB, 0.67783E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.65676E+00_JPRB, 0.66875E+00_JPRB, 0.68209E+00_JPRB, 0.69500E+00_JPRB, 0.70827E+00_JPRB, 0.70794E+00_JPRB, 0.70896E+00_JPRB ,&
&  0.72751E+00_JPRB, 0.72987E+00_JPRB, 0.72862E+00_JPRB, 0.71978E+00_JPRB, 0.71343E+00_JPRB, 0.71348E+00_JPRB, 0.70493E+00_JPRB ,&
&  0.70106E+00_JPRB, 0.70815E+00_JPRB, 0.72751E+00_JPRB, 0.75668E+00_JPRB, 0.78504E+00_JPRB, 0.82184E+00_JPRB, 0.84899E+00_JPRB ,&
&  0.85369E+00_JPRB, 0.84338E+00_JPRB, 0.84367E+00_JPRB, 0.84698E+00_JPRB, 0.84737E+00_JPRB, 0.83058E+00_JPRB, 0.82623E+00_JPRB ,&
&  0.80688E+00_JPRB, 0.81287E+00_JPRB, 0.68373E+00_JPRB, 0.75305E+00_JPRB, 0.79640E+00_JPRB, 0.83164E+00_JPRB, 0.84393E+00_JPRB ,&
&  0.83832E+00_JPRB, 0.82521E+00_JPRB, 0.80076E+00_JPRB, 0.77420E+00_JPRB, 0.76475E+00_JPRB, 0.74411E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.67123E+00_JPRB, 0.60902E+00_JPRB, 0.57708E+00_JPRB, 0.56549E+00_JPRB, 0.56014E+00_JPRB, 0.56146E+00_JPRB, 0.56673E+00_JPRB ,&
&  0.57078E+00_JPRB, 0.57168E+00_JPRB, 0.56104E+00_JPRB, 0.54520E+00_JPRB, 0.52696E+00_JPRB, 0.51742E+00_JPRB, 0.51457E+00_JPRB ,&
&  0.51088E+00_JPRB, 0.49870E+00_JPRB, 0.50194E+00_JPRB, 0.53117E+00_JPRB, 0.55544E+00_JPRB, 0.57149E+00_JPRB, 0.57737E+00_JPRB ,&
&  0.58119E+00_JPRB, 0.57407E+00_JPRB, 0.56967E+00_JPRB, 0.56080E+00_JPRB, 0.53835E+00_JPRB, 0.53280E+00_JPRB, 0.55407E+00_JPRB ,&
&  0.57780E+00_JPRB, 0.59134E+00_JPRB, 0.59923E+00_JPRB, 0.61090E+00_JPRB, 0.62290E+00_JPRB, 0.62970E+00_JPRB, 0.63605E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.64588E+00_JPRB, 0.65938E+00_JPRB, 0.67401E+00_JPRB, 0.68078E+00_JPRB, 0.68177E+00_JPRB, 0.68122E+00_JPRB ,&
&  0.67768E+00_JPRB, 0.66962E+00_JPRB, 0.66052E+00_JPRB, 0.65144E+00_JPRB, 0.63095E+00_JPRB, 0.60599E+00_JPRB, 0.58805E+00_JPRB ,&
&  0.57984E+00_JPRB, 0.59061E+00_JPRB, 0.59896E+00_JPRB, 0.61198E+00_JPRB, 0.63922E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.65964E+00_JPRB, 0.65705E+00_JPRB, 0.65653E+00_JPRB, 0.66748E+00_JPRB, 0.68434E+00_JPRB, 0.68768E+00_JPRB, 0.68802E+00_JPRB ,&
&  0.69915E+00_JPRB, 0.70742E+00_JPRB, 0.70657E+00_JPRB, 0.70522E+00_JPRB, 0.69965E+00_JPRB, 0.68667E+00_JPRB, 0.67708E+00_JPRB ,&
&  0.68352E+00_JPRB, 0.69048E+00_JPRB, 0.69439E+00_JPRB, 0.71991E+00_JPRB, 0.75298E+00_JPRB, 0.79166E+00_JPRB, 0.81801E+00_JPRB ,&
&  0.82771E+00_JPRB, 0.82081E+00_JPRB, 0.81733E+00_JPRB, 0.81210E+00_JPRB, 0.80682E+00_JPRB, 0.80856E+00_JPRB, 0.80316E+00_JPRB ,&
&  0.79405E+00_JPRB, 0.69988E+00_JPRB, 0.72953E+00_JPRB, 0.73834E+00_JPRB, 0.75465E+00_JPRB, 0.78268E+00_JPRB, 0.81611E+00_JPRB ,&
&  0.81892E+00_JPRB, 0.81387E+00_JPRB, 0.80220E+00_JPRB, 0.78115E+00_JPRB, 0.76322E+00_JPRB, 0.75266E+00_JPRB, 0.73500E+00_JPRB ,&
&  0.70640E+00_JPRB, 0.66829E+00_JPRB, 0.63430E+00_JPRB, 0.61137E+00_JPRB, 0.60128E+00_JPRB, 0.59896E+00_JPRB, 0.59482E+00_JPRB ,&
&  0.59333E+00_JPRB, 0.59240E+00_JPRB, 0.58234E+00_JPRB, 0.57336E+00_JPRB, 0.56453E+00_JPRB, 0.55926E+00_JPRB, 0.54714E+00_JPRB ,&
&  0.53929E+00_JPRB, 0.52885E+00_JPRB, 0.52830E+00_JPRB, 0.54764E+00_JPRB, 0.56705E+00_JPRB, 0.56961E+00_JPRB, 0.58075E+00_JPRB ,&
&  0.59563E+00_JPRB, 0.60109E+00_JPRB, 0.60169E+00_JPRB, 0.59839E+00_JPRB, 0.59083E+00_JPRB, 0.59033E+00_JPRB, 0.59783E+00_JPRB ,&
&  0.60418E+00_JPRB, 0.61358E+00_JPRB, 0.61631E+00_JPRB, 0.62040E+00_JPRB, 0.62665E+00_JPRB, 0.63143E+00_JPRB, 0.64260E+00_JPRB ,&
&  0.64841E+00_JPRB, 0.65263E+00_JPRB, 0.66431E+00_JPRB, 0.67580E+00_JPRB, 0.68024E+00_JPRB, 0.68211E+00_JPRB, 0.69037E+00_JPRB ,&
&  0.69289E+00_JPRB, 0.69014E+00_JPRB, 0.68188E+00_JPRB, 0.66933E+00_JPRB, 0.65969E+00_JPRB, 0.64563E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.60992E+00_JPRB, 0.60898E+00_JPRB, 0.61952E+00_JPRB, 0.63091E+00_JPRB, 0.64922E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.65790E+00_JPRB, 0.65116E+00_JPRB, 0.64546E+00_JPRB, 0.65283E+00_JPRB, 0.66625E+00_JPRB, 0.67614E+00_JPRB, 0.67508E+00_JPRB ,&
&  0.67431E+00_JPRB, 0.67880E+00_JPRB, 0.68724E+00_JPRB, 0.69032E+00_JPRB, 0.68481E+00_JPRB, 0.67016E+00_JPRB, 0.66390E+00_JPRB ,&
&  0.66447E+00_JPRB, 0.66814E+00_JPRB, 0.67021E+00_JPRB, 0.67643E+00_JPRB, 0.69777E+00_JPRB, 0.72990E+00_JPRB, 0.76015E+00_JPRB ,&
&  0.77538E+00_JPRB, 0.77053E+00_JPRB, 0.76471E+00_JPRB, 0.75969E+00_JPRB, 0.75838E+00_JPRB, 0.76305E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.61266E+00_JPRB, 0.66611E+00_JPRB, 0.68512E+00_JPRB, 0.71094E+00_JPRB, 0.74966E+00_JPRB, 0.79457E+00_JPRB ,&
&  0.80342E+00_JPRB, 0.79166E+00_JPRB, 0.77011E+00_JPRB, 0.75307E+00_JPRB, 0.73563E+00_JPRB, 0.72968E+00_JPRB, 0.72596E+00_JPRB ,&
&  0.72281E+00_JPRB, 0.71835E+00_JPRB, 0.69504E+00_JPRB, 0.65893E+00_JPRB, 0.63922E+00_JPRB, 0.62746E+00_JPRB, 0.61774E+00_JPRB ,&
&  0.60903E+00_JPRB, 0.59736E+00_JPRB, 0.58581E+00_JPRB, 0.57621E+00_JPRB, 0.57834E+00_JPRB, 0.57742E+00_JPRB, 0.56577E+00_JPRB ,&
&  0.56146E+00_JPRB, 0.56011E+00_JPRB, 0.56777E+00_JPRB, 0.57726E+00_JPRB, 0.58013E+00_JPRB, 0.57348E+00_JPRB, 0.58052E+00_JPRB ,&
&  0.59864E+00_JPRB, 0.60953E+00_JPRB, 0.61916E+00_JPRB, 0.63252E+00_JPRB, 0.63859E+00_JPRB, 0.63652E+00_JPRB, 0.63601E+00_JPRB ,&
&  0.64056E+00_JPRB, 0.63809E+00_JPRB, 0.63033E+00_JPRB, 0.62421E+00_JPRB, 0.62535E+00_JPRB, 0.62625E+00_JPRB, 0.63659E+00_JPRB ,&
&  0.64607E+00_JPRB, 0.64907E+00_JPRB, 0.64506E+00_JPRB, 0.64772E+00_JPRB, 0.65327E+00_JPRB, 0.66211E+00_JPRB, 0.66789E+00_JPRB ,&
&  0.67401E+00_JPRB, 0.67210E+00_JPRB, 0.66026E+00_JPRB, 0.65956E+00_JPRB, 0.66575E+00_JPRB, 0.66212E+00_JPRB, 0.65069E+00_JPRB ,&
&  0.64177E+00_JPRB, 0.63441E+00_JPRB, 0.63868E+00_JPRB, 0.64321E+00_JPRB, 0.65119E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.68377E+00_JPRB, 0.68125E+00_JPRB, 0.67475E+00_JPRB, 0.67440E+00_JPRB, 0.67469E+00_JPRB, 0.67140E+00_JPRB, 0.66953E+00_JPRB ,&
&  0.66736E+00_JPRB, 0.66478E+00_JPRB, 0.67399E+00_JPRB, 0.68284E+00_JPRB, 0.68552E+00_JPRB, 0.68188E+00_JPRB, 0.67917E+00_JPRB ,&
&  0.67818E+00_JPRB, 0.67368E+00_JPRB, 0.66839E+00_JPRB, 0.66760E+00_JPRB, 0.67215E+00_JPRB, 0.68770E+00_JPRB, 0.71039E+00_JPRB ,&
&  0.72741E+00_JPRB, 0.72759E+00_JPRB, 0.73169E+00_JPRB, 0.74276E+00_JPRB, 0.74498E+00_JPRB, 0.74603E+00_JPRB, 0.75247E+00_JPRB ,&
&  0.63516E+00_JPRB, 0.51283E+00_JPRB, 0.57414E+00_JPRB, 0.63056E+00_JPRB, 0.68708E+00_JPRB, 0.75179E+00_JPRB, 0.81949E+00_JPRB ,&
&  0.83968E+00_JPRB, 0.82355E+00_JPRB, 0.80452E+00_JPRB, 0.78250E+00_JPRB, 0.75754E+00_JPRB, 0.74091E+00_JPRB, 0.72780E+00_JPRB ,&
&  0.72372E+00_JPRB, 0.71131E+00_JPRB, 0.69281E+00_JPRB, 0.67755E+00_JPRB, 0.66492E+00_JPRB, 0.64956E+00_JPRB, 0.63496E+00_JPRB ,&
&  0.62028E+00_JPRB, 0.60812E+00_JPRB, 0.60509E+00_JPRB, 0.60402E+00_JPRB, 0.60724E+00_JPRB, 0.60816E+00_JPRB, 0.60454E+00_JPRB ,&
&  0.60173E+00_JPRB, 0.60369E+00_JPRB, 0.61224E+00_JPRB, 0.61897E+00_JPRB, 0.61982E+00_JPRB, 0.61610E+00_JPRB, 0.62226E+00_JPRB ,&
&  0.62495E+00_JPRB, 0.63248E+00_JPRB, 0.64268E+00_JPRB, 0.66294E+00_JPRB, 0.67775E+00_JPRB, 0.67881E+00_JPRB, 0.67800E+00_JPRB ,&
&  0.68058E+00_JPRB, 0.67415E+00_JPRB, 0.65909E+00_JPRB, 0.64901E+00_JPRB, 0.64429E+00_JPRB, 0.63758E+00_JPRB, 0.63574E+00_JPRB ,&
&  0.64060E+00_JPRB, 0.64218E+00_JPRB, 0.63663E+00_JPRB, 0.63856E+00_JPRB, 0.64734E+00_JPRB, 0.66065E+00_JPRB, 0.67312E+00_JPRB ,&
&  0.66589E+00_JPRB, 0.67383E+00_JPRB, 0.67410E+00_JPRB, 0.68325E+00_JPRB, 0.69605E+00_JPRB, 0.68844E+00_JPRB, 0.68286E+00_JPRB ,&
&  0.67753E+00_JPRB, 0.65798E+00_JPRB, 0.66230E+00_JPRB, 0.66650E+00_JPRB, 0.67190E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.67078E+00_JPRB, 0.70149E+00_JPRB, 0.70925E+00_JPRB, 0.69623E+00_JPRB, 0.67331E+00_JPRB, 0.66078E+00_JPRB, 0.65418E+00_JPRB ,&
&  0.65092E+00_JPRB, 0.64825E+00_JPRB, 0.65818E+00_JPRB, 0.66585E+00_JPRB, 0.66924E+00_JPRB, 0.66363E+00_JPRB, 0.65800E+00_JPRB ,&
&  0.64789E+00_JPRB, 0.63484E+00_JPRB, 0.62621E+00_JPRB, 0.61881E+00_JPRB, 0.61982E+00_JPRB, 0.62957E+00_JPRB, 0.64744E+00_JPRB ,&
&  0.66978E+00_JPRB, 0.68547E+00_JPRB, 0.69793E+00_JPRB, 0.71048E+00_JPRB, 0.71251E+00_JPRB, 0.71210E+00_JPRB, 0.71442E+00_JPRB ,&
&  0.72059E+00_JPRB, 0.53807E+00_JPRB, 0.58046E+00_JPRB, 0.64864E+00_JPRB, 0.72587E+00_JPRB, 0.81958E+00_JPRB, 0.89740E+00_JPRB ,&
&  0.92160E+00_JPRB, 0.88904E+00_JPRB, 0.86191E+00_JPRB, 0.82903E+00_JPRB, 0.79253E+00_JPRB, 0.75955E+00_JPRB, 0.72352E+00_JPRB ,&
&  0.69621E+00_JPRB, 0.67616E+00_JPRB, 0.66595E+00_JPRB, 0.65697E+00_JPRB, 0.64731E+00_JPRB, 0.63539E+00_JPRB, 0.62191E+00_JPRB ,&
&  0.60890E+00_JPRB, 0.60256E+00_JPRB, 0.59935E+00_JPRB, 0.59664E+00_JPRB, 0.59904E+00_JPRB, 0.60465E+00_JPRB, 0.60995E+00_JPRB ,&
&  0.61886E+00_JPRB, 0.62515E+00_JPRB, 0.62501E+00_JPRB, 0.62169E+00_JPRB, 0.61808E+00_JPRB, 0.61791E+00_JPRB, 0.62956E+00_JPRB ,&
&  0.63379E+00_JPRB, 0.64280E+00_JPRB, 0.65328E+00_JPRB, 0.67684E+00_JPRB, 0.69820E+00_JPRB, 0.70309E+00_JPRB, 0.70191E+00_JPRB ,&
&  0.70044E+00_JPRB, 0.69203E+00_JPRB, 0.67833E+00_JPRB, 0.66340E+00_JPRB, 0.65337E+00_JPRB, 0.65071E+00_JPRB, 0.64300E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.64329E+00_JPRB, 0.64392E+00_JPRB, 0.65316E+00_JPRB, 0.66921E+00_JPRB, 0.68099E+00_JPRB, 0.67791E+00_JPRB ,&
&  0.67644E+00_JPRB, 0.68053E+00_JPRB, 0.68095E+00_JPRB, 0.70406E+00_JPRB, 0.71297E+00_JPRB, 0.71467E+00_JPRB, 0.70610E+00_JPRB ,&
&  0.69845E+00_JPRB, 0.69275E+00_JPRB, 0.52294E+00_JPRB, 0.61136E+00_JPRB, 0.63694E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.69749E+00_JPRB, 0.73030E+00_JPRB, 0.73036E+00_JPRB, 0.70001E+00_JPRB, 0.66820E+00_JPRB, 0.64852E+00_JPRB, 0.63688E+00_JPRB ,&
&  0.63456E+00_JPRB, 0.62907E+00_JPRB, 0.63089E+00_JPRB, 0.64092E+00_JPRB, 0.65205E+00_JPRB, 0.65468E+00_JPRB, 0.63945E+00_JPRB ,&
&  0.62071E+00_JPRB, 0.60772E+00_JPRB, 0.60379E+00_JPRB, 0.60387E+00_JPRB, 0.60215E+00_JPRB, 0.60188E+00_JPRB, 0.61196E+00_JPRB ,&
&  0.62389E+00_JPRB, 0.63769E+00_JPRB, 0.65179E+00_JPRB, 0.66283E+00_JPRB, 0.67051E+00_JPRB, 0.67716E+00_JPRB, 0.69465E+00_JPRB ,&
&  0.70276E+00_JPRB, 0.53276E+00_JPRB, 0.56437E+00_JPRB, 0.71165E+00_JPRB, 0.84784E+00_JPRB, 0.98356E+00_JPRB, 0.10471E+01_JPRB ,&
&  0.10400E+01_JPRB, 0.10045E+01_JPRB, 0.95443E+00_JPRB, 0.89784E+00_JPRB, 0.83827E+00_JPRB, 0.78517E+00_JPRB, 0.73609E+00_JPRB ,&
&  0.70197E+00_JPRB, 0.67637E+00_JPRB, 0.66110E+00_JPRB, 0.64423E+00_JPRB, 0.63386E+00_JPRB, 0.62726E+00_JPRB, 0.62109E+00_JPRB ,&
&  0.61614E+00_JPRB, 0.60934E+00_JPRB, 0.60985E+00_JPRB, 0.61472E+00_JPRB, 0.61464E+00_JPRB, 0.62152E+00_JPRB, 0.62258E+00_JPRB ,&
&  0.63150E+00_JPRB, 0.64358E+00_JPRB, 0.64509E+00_JPRB, 0.64055E+00_JPRB, 0.64024E+00_JPRB, 0.64781E+00_JPRB, 0.65974E+00_JPRB ,&
&  0.66792E+00_JPRB, 0.68821E+00_JPRB, 0.70488E+00_JPRB, 0.72424E+00_JPRB, 0.73637E+00_JPRB, 0.73409E+00_JPRB, 0.72152E+00_JPRB ,&
&  0.72234E+00_JPRB, 0.71543E+00_JPRB, 0.70638E+00_JPRB, 0.69493E+00_JPRB, 0.68238E+00_JPRB, 0.67122E+00_JPRB, 0.66294E+00_JPRB ,&
&  0.66075E+00_JPRB, 0.66817E+00_JPRB, 0.67523E+00_JPRB, 0.69442E+00_JPRB, 0.70578E+00_JPRB, 0.70736E+00_JPRB, 0.70969E+00_JPRB ,&
&  0.71370E+00_JPRB, 0.71482E+00_JPRB, 0.71424E+00_JPRB, 0.68956E+00_JPRB, 0.74005E+00_JPRB, 0.74848E+00_JPRB, 0.74142E+00_JPRB ,&
&  0.73202E+00_JPRB, 0.70506E+00_JPRB, 0.69840E+00_JPRB, 0.57285E+00_JPRB, 0.65423E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.79250E+00_JPRB, 0.78381E+00_JPRB, 0.74806E+00_JPRB, 0.70362E+00_JPRB, 0.66511E+00_JPRB, 0.63497E+00_JPRB, 0.61937E+00_JPRB ,&
&  0.60627E+00_JPRB, 0.59610E+00_JPRB, 0.59895E+00_JPRB, 0.60786E+00_JPRB, 0.61806E+00_JPRB, 0.61857E+00_JPRB, 0.61106E+00_JPRB ,&
&  0.60023E+00_JPRB, 0.58676E+00_JPRB, 0.58052E+00_JPRB, 0.57811E+00_JPRB, 0.57239E+00_JPRB, 0.57135E+00_JPRB, 0.57186E+00_JPRB ,&
&  0.57690E+00_JPRB, 0.58352E+00_JPRB, 0.58754E+00_JPRB, 0.59694E+00_JPRB, 0.61159E+00_JPRB, 0.63329E+00_JPRB, 0.65497E+00_JPRB ,&
&  0.65373E+00_JPRB, 0.55294E+00_JPRB, 0.63906E+00_JPRB, 0.84796E+00_JPRB, 0.10479E+01_JPRB, 0.11363E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.10287E+01_JPRB, 0.96125E+00_JPRB, 0.88980E+00_JPRB, 0.82558E+00_JPRB, 0.77119E+00_JPRB, 0.72281E+00_JPRB ,&
&  0.68675E+00_JPRB, 0.66166E+00_JPRB, 0.64780E+00_JPRB, 0.63898E+00_JPRB, 0.63284E+00_JPRB, 0.61882E+00_JPRB, 0.60994E+00_JPRB ,&
&  0.60934E+00_JPRB, 0.61001E+00_JPRB, 0.61263E+00_JPRB, 0.61564E+00_JPRB, 0.62820E+00_JPRB, 0.64794E+00_JPRB, 0.65671E+00_JPRB ,&
&  0.65558E+00_JPRB, 0.66127E+00_JPRB, 0.66422E+00_JPRB, 0.65744E+00_JPRB, 0.65435E+00_JPRB, 0.67835E+00_JPRB, 0.69897E+00_JPRB ,&
&  0.72239E+00_JPRB, 0.75306E+00_JPRB, 0.79006E+00_JPRB, 0.80932E+00_JPRB, 0.80770E+00_JPRB, 0.79252E+00_JPRB, 0.76449E+00_JPRB ,&
&  0.74870E+00_JPRB, 0.72886E+00_JPRB, 0.71060E+00_JPRB, 0.69914E+00_JPRB, 0.69016E+00_JPRB, 0.68052E+00_JPRB, 0.67046E+00_JPRB ,&
&  0.66392E+00_JPRB, 0.65730E+00_JPRB, 0.66074E+00_JPRB, 0.67990E+00_JPRB, 0.69967E+00_JPRB, 0.71129E+00_JPRB, 0.71394E+00_JPRB ,&
&  0.71089E+00_JPRB, 0.70910E+00_JPRB, 0.71285E+00_JPRB, 0.76379E+00_JPRB, 0.75464E+00_JPRB, 0.75603E+00_JPRB, 0.74768E+00_JPRB ,&
&  0.73989E+00_JPRB, 0.72559E+00_JPRB, 0.70356E+00_JPRB, 0.69360E+00_JPRB, 0.71940E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.85573E+00_JPRB, 0.80127E+00_JPRB, 0.75053E+00_JPRB, 0.69606E+00_JPRB, 0.65734E+00_JPRB, 0.64143E+00_JPRB, 0.63036E+00_JPRB ,&
&  0.61268E+00_JPRB, 0.59618E+00_JPRB, 0.59458E+00_JPRB, 0.59687E+00_JPRB, 0.59455E+00_JPRB, 0.58799E+00_JPRB, 0.57521E+00_JPRB ,&
&  0.56950E+00_JPRB, 0.57448E+00_JPRB, 0.57154E+00_JPRB, 0.56500E+00_JPRB, 0.55773E+00_JPRB, 0.55167E+00_JPRB, 0.54729E+00_JPRB ,&
&  0.54546E+00_JPRB, 0.54992E+00_JPRB, 0.55297E+00_JPRB, 0.56432E+00_JPRB, 0.58345E+00_JPRB, 0.60556E+00_JPRB, 0.63248E+00_JPRB ,&
&  0.62907E+00_JPRB, 0.56179E+00_JPRB, 0.78563E+00_JPRB, 0.11137E+01_JPRB, 0.13635E+01_JPRB, 0.13581E+01_JPRB, 0.12162E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.99484E+00_JPRB, 0.90862E+00_JPRB, 0.84709E+00_JPRB, 0.79866E+00_JPRB, 0.75973E+00_JPRB, 0.73029E+00_JPRB ,&
&  0.69946E+00_JPRB, 0.67564E+00_JPRB, 0.65378E+00_JPRB, 0.63401E+00_JPRB, 0.63056E+00_JPRB, 0.62589E+00_JPRB, 0.61832E+00_JPRB ,&
&  0.61326E+00_JPRB, 0.61376E+00_JPRB, 0.61612E+00_JPRB, 0.62172E+00_JPRB, 0.63688E+00_JPRB, 0.68385E+00_JPRB, 0.72319E+00_JPRB ,&
&  0.72895E+00_JPRB, 0.73770E+00_JPRB, 0.73526E+00_JPRB, 0.72875E+00_JPRB, 0.73751E+00_JPRB, 0.75608E+00_JPRB, 0.79120E+00_JPRB ,&
&  0.83367E+00_JPRB, 0.85956E+00_JPRB, 0.87269E+00_JPRB, 0.88249E+00_JPRB, 0.87185E+00_JPRB, 0.85566E+00_JPRB, 0.83345E+00_JPRB ,&
&  0.81076E+00_JPRB, 0.77971E+00_JPRB, 0.75401E+00_JPRB, 0.73632E+00_JPRB, 0.71819E+00_JPRB, 0.70493E+00_JPRB, 0.69013E+00_JPRB ,&
&  0.67520E+00_JPRB, 0.66034E+00_JPRB, 0.66804E+00_JPRB, 0.68119E+00_JPRB, 0.69003E+00_JPRB, 0.69187E+00_JPRB, 0.70172E+00_JPRB ,&
&  0.70903E+00_JPRB, 0.72589E+00_JPRB, 0.71459E+00_JPRB, 0.77828E+00_JPRB, 0.82886E+00_JPRB, 0.81358E+00_JPRB, 0.79925E+00_JPRB ,&
&  0.77696E+00_JPRB, 0.75333E+00_JPRB, 0.72474E+00_JPRB, 0.71925E+00_JPRB, 0.85314E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.73601E+00_JPRB, 0.69630E+00_JPRB, 0.65348E+00_JPRB, 0.62623E+00_JPRB, 0.60440E+00_JPRB, 0.58625E+00_JPRB, 0.58380E+00_JPRB ,&
&  0.58031E+00_JPRB, 0.57259E+00_JPRB, 0.57876E+00_JPRB, 0.57657E+00_JPRB, 0.56945E+00_JPRB, 0.55366E+00_JPRB, 0.55615E+00_JPRB ,&
&  0.56032E+00_JPRB, 0.55602E+00_JPRB, 0.54993E+00_JPRB, 0.54403E+00_JPRB, 0.53867E+00_JPRB, 0.52978E+00_JPRB, 0.52439E+00_JPRB ,&
&  0.52023E+00_JPRB, 0.52207E+00_JPRB, 0.53040E+00_JPRB, 0.54450E+00_JPRB, 0.56783E+00_JPRB, 0.59639E+00_JPRB, 0.62419E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.67211E+00_JPRB, 0.10363E+01_JPRB, 0.16508E+01_JPRB, 0.18601E+01_JPRB, 0.14971E+01_JPRB, 0.12243E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.90440E+00_JPRB, 0.83447E+00_JPRB, 0.79391E+00_JPRB, 0.75784E+00_JPRB, 0.74061E+00_JPRB, 0.72451E+00_JPRB ,&
&  0.70604E+00_JPRB, 0.68169E+00_JPRB, 0.65133E+00_JPRB, 0.63140E+00_JPRB, 0.62275E+00_JPRB, 0.62245E+00_JPRB, 0.61796E+00_JPRB ,&
&  0.61457E+00_JPRB, 0.62372E+00_JPRB, 0.63064E+00_JPRB, 0.63948E+00_JPRB, 0.69672E+00_JPRB, 0.78482E+00_JPRB, 0.88351E+00_JPRB ,&
&  0.86790E+00_JPRB, 0.84971E+00_JPRB, 0.84765E+00_JPRB, 0.84543E+00_JPRB, 0.86113E+00_JPRB, 0.87195E+00_JPRB, 0.89267E+00_JPRB ,&
&  0.91298E+00_JPRB, 0.90451E+00_JPRB, 0.88025E+00_JPRB, 0.86168E+00_JPRB, 0.84149E+00_JPRB, 0.83272E+00_JPRB, 0.80319E+00_JPRB ,&
&  0.77720E+00_JPRB, 0.75728E+00_JPRB, 0.74651E+00_JPRB, 0.72294E+00_JPRB, 0.71007E+00_JPRB, 0.69582E+00_JPRB, 0.68384E+00_JPRB ,&
&  0.66389E+00_JPRB, 0.65192E+00_JPRB, 0.67607E+00_JPRB, 0.68803E+00_JPRB, 0.68888E+00_JPRB, 0.68968E+00_JPRB, 0.70287E+00_JPRB ,&
&  0.72219E+00_JPRB, 0.74683E+00_JPRB, 0.76215E+00_JPRB, 0.79948E+00_JPRB, 0.79625E+00_JPRB, 0.84517E+00_JPRB, 0.81942E+00_JPRB ,&
&  0.79254E+00_JPRB, 0.75066E+00_JPRB, 0.70366E+00_JPRB, 0.68667E+00_JPRB, 0.74978E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.68216E+00_JPRB, 0.63166E+00_JPRB, 0.59258E+00_JPRB, 0.57056E+00_JPRB, 0.54896E+00_JPRB, 0.54038E+00_JPRB, 0.53577E+00_JPRB ,&
&  0.54478E+00_JPRB, 0.55255E+00_JPRB, 0.56689E+00_JPRB, 0.56387E+00_JPRB, 0.54276E+00_JPRB, 0.53400E+00_JPRB, 0.54128E+00_JPRB ,&
&  0.53388E+00_JPRB, 0.53473E+00_JPRB, 0.54116E+00_JPRB, 0.53612E+00_JPRB, 0.52973E+00_JPRB, 0.52103E+00_JPRB, 0.51032E+00_JPRB ,&
&  0.51476E+00_JPRB, 0.52186E+00_JPRB, 0.53595E+00_JPRB, 0.55742E+00_JPRB, 0.58530E+00_JPRB, 0.61302E+00_JPRB, 0.63258E+00_JPRB ,&
&  0.64405E+00_JPRB, 0.87462E+00_JPRB, 0.14936E+01_JPRB, 0.19990E+01_JPRB, 0.18329E+01_JPRB, 0.13627E+01_JPRB, 0.11459E+01_JPRB ,&
&  0.96977E+00_JPRB, 0.84473E+00_JPRB, 0.80024E+00_JPRB, 0.77322E+00_JPRB, 0.74124E+00_JPRB, 0.72820E+00_JPRB, 0.73040E+00_JPRB ,&
&  0.71775E+00_JPRB, 0.69223E+00_JPRB, 0.66331E+00_JPRB, 0.63843E+00_JPRB, 0.62362E+00_JPRB, 0.62230E+00_JPRB, 0.62618E+00_JPRB ,&
&  0.63348E+00_JPRB, 0.64463E+00_JPRB, 0.66251E+00_JPRB, 0.67863E+00_JPRB, 0.71510E+00_JPRB, 0.71175E+00_JPRB, 0.90469E+00_JPRB ,&
&  0.11230E+01_JPRB, 0.10829E+01_JPRB, 0.10510E+01_JPRB, 0.10233E+01_JPRB, 0.99231E+00_JPRB, 0.95731E+00_JPRB, 0.94637E+00_JPRB ,&
&  0.92155E+00_JPRB, 0.87941E+00_JPRB, 0.84182E+00_JPRB, 0.81803E+00_JPRB, 0.78872E+00_JPRB, 0.76689E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.71731E+00_JPRB, 0.70271E+00_JPRB, 0.69333E+00_JPRB, 0.69196E+00_JPRB, 0.69474E+00_JPRB, 0.68155E+00_JPRB, 0.67426E+00_JPRB ,&
&  0.66240E+00_JPRB, 0.67479E+00_JPRB, 0.69858E+00_JPRB, 0.68059E+00_JPRB, 0.68423E+00_JPRB, 0.70703E+00_JPRB, 0.72007E+00_JPRB ,&
&  0.70602E+00_JPRB, 0.72549E+00_JPRB, 0.76851E+00_JPRB, 0.78944E+00_JPRB, 0.78273E+00_JPRB, 0.83965E+00_JPRB, 0.80181E+00_JPRB ,&
&  0.76194E+00_JPRB, 0.71949E+00_JPRB, 0.69177E+00_JPRB, 0.67026E+00_JPRB, 0.70350E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.62264E+00_JPRB, 0.58064E+00_JPRB, 0.55534E+00_JPRB, 0.53538E+00_JPRB, 0.53957E+00_JPRB, 0.53426E+00_JPRB, 0.52915E+00_JPRB ,&
&  0.54411E+00_JPRB, 0.55343E+00_JPRB, 0.56541E+00_JPRB, 0.55443E+00_JPRB, 0.53271E+00_JPRB, 0.52652E+00_JPRB, 0.51913E+00_JPRB ,&
&  0.50987E+00_JPRB, 0.53140E+00_JPRB, 0.55247E+00_JPRB, 0.56288E+00_JPRB, 0.55541E+00_JPRB, 0.53197E+00_JPRB, 0.51983E+00_JPRB ,&
&  0.52730E+00_JPRB, 0.54512E+00_JPRB, 0.57228E+00_JPRB, 0.60222E+00_JPRB, 0.62827E+00_JPRB, 0.65220E+00_JPRB, 0.68428E+00_JPRB ,&
&  0.78013E+00_JPRB, 0.98281E+00_JPRB, 0.15864E+01_JPRB, 0.22001E+01_JPRB, 0.17023E+01_JPRB, 0.12558E+01_JPRB, 0.99139E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.82267E+00_JPRB, 0.79939E+00_JPRB, 0.78373E+00_JPRB, 0.75878E+00_JPRB, 0.74255E+00_JPRB, 0.74157E+00_JPRB ,&
&  0.72206E+00_JPRB, 0.70397E+00_JPRB, 0.68737E+00_JPRB, 0.67301E+00_JPRB, 0.66484E+00_JPRB, 0.66266E+00_JPRB, 0.66550E+00_JPRB ,&
&  0.68008E+00_JPRB, 0.69126E+00_JPRB, 0.70743E+00_JPRB, 0.73240E+00_JPRB, 0.73591E+00_JPRB, 0.80168E+00_JPRB, 0.93566E+00_JPRB ,&
&  0.10870E+01_JPRB, 0.12074E+01_JPRB, 0.11330E+01_JPRB, 0.10799E+01_JPRB, 0.10339E+01_JPRB, 0.10025E+01_JPRB, 0.95614E+00_JPRB ,&
&  0.91679E+00_JPRB, 0.86526E+00_JPRB, 0.81987E+00_JPRB, 0.78212E+00_JPRB, 0.73407E+00_JPRB, 0.70687E+00_JPRB, 0.70024E+00_JPRB ,&
&  0.68605E+00_JPRB, 0.68490E+00_JPRB, 0.67665E+00_JPRB, 0.68712E+00_JPRB, 0.68149E+00_JPRB, 0.67781E+00_JPRB, 0.67721E+00_JPRB ,&
&  0.68878E+00_JPRB, 0.73183E+00_JPRB, 0.74167E+00_JPRB, 0.68765E+00_JPRB, 0.66559E+00_JPRB, 0.67433E+00_JPRB, 0.69215E+00_JPRB ,&
&  0.71207E+00_JPRB, 0.74917E+00_JPRB, 0.77112E+00_JPRB, 0.79406E+00_JPRB, 0.75432E+00_JPRB, 0.80983E+00_JPRB, 0.76828E+00_JPRB ,&
&  0.72557E+00_JPRB, 0.70057E+00_JPRB, 0.68872E+00_JPRB, 0.67550E+00_JPRB, 0.67103E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.62196E+00_JPRB, 0.58907E+00_JPRB, 0.54823E+00_JPRB, 0.53096E+00_JPRB, 0.53907E+00_JPRB, 0.52555E+00_JPRB, 0.53025E+00_JPRB ,&
&  0.54485E+00_JPRB, 0.54311E+00_JPRB, 0.54187E+00_JPRB, 0.53352E+00_JPRB, 0.52747E+00_JPRB, 0.51837E+00_JPRB, 0.51501E+00_JPRB ,&
&  0.52729E+00_JPRB, 0.55867E+00_JPRB, 0.56507E+00_JPRB, 0.55368E+00_JPRB, 0.54689E+00_JPRB, 0.53257E+00_JPRB, 0.52810E+00_JPRB ,&
&  0.55264E+00_JPRB, 0.59865E+00_JPRB, 0.64577E+00_JPRB, 0.67185E+00_JPRB, 0.69186E+00_JPRB, 0.71515E+00_JPRB, 0.80964E+00_JPRB ,&
&  0.10639E+01_JPRB, 0.25099E+01_JPRB, 0.11800E+01_JPRB, 0.22797E+01_JPRB, 0.15114E+01_JPRB, 0.11231E+01_JPRB, 0.84511E+00_JPRB ,&
&  0.79259E+00_JPRB, 0.81697E+00_JPRB, 0.80776E+00_JPRB, 0.81884E+00_JPRB, 0.78647E+00_JPRB, 0.75084E+00_JPRB, 0.74156E+00_JPRB ,&
&  0.72854E+00_JPRB, 0.72313E+00_JPRB, 0.72846E+00_JPRB, 0.73441E+00_JPRB, 0.73531E+00_JPRB, 0.73138E+00_JPRB, 0.73015E+00_JPRB ,&
&  0.73788E+00_JPRB, 0.73826E+00_JPRB, 0.76080E+00_JPRB, 0.78306E+00_JPRB, 0.78257E+00_JPRB, 0.86513E+00_JPRB, 0.98621E+00_JPRB ,&
&  0.10372E+01_JPRB, 0.11829E+01_JPRB, 0.10870E+01_JPRB, 0.10480E+01_JPRB, 0.10073E+01_JPRB, 0.96217E+00_JPRB, 0.89392E+00_JPRB ,&
&  0.83323E+00_JPRB, 0.79762E+00_JPRB, 0.76180E+00_JPRB, 0.73144E+00_JPRB, 0.69748E+00_JPRB, 0.68614E+00_JPRB, 0.67073E+00_JPRB ,&
&  0.67458E+00_JPRB, 0.69639E+00_JPRB, 0.69087E+00_JPRB, 0.69131E+00_JPRB, 0.68847E+00_JPRB, 0.70128E+00_JPRB, 0.71512E+00_JPRB ,&
&  0.74293E+00_JPRB, 0.78871E+00_JPRB, 0.76456E+00_JPRB, 0.67208E+00_JPRB, 0.63921E+00_JPRB, 0.65437E+00_JPRB, 0.67178E+00_JPRB ,&
&  0.72467E+00_JPRB, 0.76594E+00_JPRB, 0.76812E+00_JPRB, 0.76866E+00_JPRB, 0.76032E+00_JPRB, 0.79822E+00_JPRB, 0.74754E+00_JPRB ,&
&  0.70539E+00_JPRB, 0.69406E+00_JPRB, 0.68795E+00_JPRB, 0.66816E+00_JPRB, 0.66427E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.62405E+00_JPRB, 0.60569E+00_JPRB, 0.56680E+00_JPRB, 0.53373E+00_JPRB, 0.53271E+00_JPRB, 0.53768E+00_JPRB, 0.54448E+00_JPRB ,&
&  0.54339E+00_JPRB, 0.53985E+00_JPRB, 0.53501E+00_JPRB, 0.53021E+00_JPRB, 0.53781E+00_JPRB, 0.53489E+00_JPRB, 0.56117E+00_JPRB ,&
&  0.58038E+00_JPRB, 0.57408E+00_JPRB, 0.57739E+00_JPRB, 0.55566E+00_JPRB, 0.55245E+00_JPRB, 0.54841E+00_JPRB, 0.57082E+00_JPRB ,&
&  0.62571E+00_JPRB, 0.68637E+00_JPRB, 0.72177E+00_JPRB, 0.73505E+00_JPRB, 0.75550E+00_JPRB, 0.79670E+00_JPRB, 0.94245E+00_JPRB ,&
&  0.13270E+01_JPRB, 0.27125E+01_JPRB, 0.13872E+01_JPRB, 0.20338E+01_JPRB, 0.12684E+01_JPRB, 0.93421E+00_JPRB, 0.74716E+00_JPRB ,&
&  0.72727E+00_JPRB, 0.81707E+00_JPRB, 0.82720E+00_JPRB, 0.83634E+00_JPRB, 0.76171E+00_JPRB, 0.72787E+00_JPRB, 0.71844E+00_JPRB ,&
&  0.72657E+00_JPRB, 0.76716E+00_JPRB, 0.80343E+00_JPRB, 0.82415E+00_JPRB, 0.81608E+00_JPRB, 0.79643E+00_JPRB, 0.79778E+00_JPRB ,&
&  0.79852E+00_JPRB, 0.78661E+00_JPRB, 0.82079E+00_JPRB, 0.81706E+00_JPRB, 0.89903E+00_JPRB, 0.95177E+00_JPRB, 0.97429E+00_JPRB ,&
&  0.98160E+00_JPRB, 0.10601E+01_JPRB, 0.10244E+01_JPRB, 0.10129E+01_JPRB, 0.95430E+00_JPRB, 0.90344E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.79121E+00_JPRB, 0.76264E+00_JPRB, 0.73345E+00_JPRB, 0.69726E+00_JPRB, 0.68153E+00_JPRB, 0.68519E+00_JPRB, 0.67475E+00_JPRB ,&
&  0.69754E+00_JPRB, 0.72460E+00_JPRB, 0.72286E+00_JPRB, 0.72630E+00_JPRB, 0.73538E+00_JPRB, 0.75199E+00_JPRB, 0.76943E+00_JPRB ,&
&  0.79679E+00_JPRB, 0.83057E+00_JPRB, 0.77061E+00_JPRB, 0.66948E+00_JPRB, 0.65964E+00_JPRB, 0.67367E+00_JPRB, 0.67657E+00_JPRB ,&
&  0.72198E+00_JPRB, 0.75670E+00_JPRB, 0.75789E+00_JPRB, 0.73470E+00_JPRB, 0.74069E+00_JPRB, 0.77266E+00_JPRB, 0.73316E+00_JPRB ,&
&  0.71144E+00_JPRB, 0.70652E+00_JPRB, 0.71012E+00_JPRB, 0.70373E+00_JPRB, 0.67321E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.58472E+00_JPRB, 0.57144E+00_JPRB, 0.55601E+00_JPRB, 0.54143E+00_JPRB, 0.53377E+00_JPRB, 0.54081E+00_JPRB, 0.54538E+00_JPRB ,&
&  0.53797E+00_JPRB, 0.53461E+00_JPRB, 0.52830E+00_JPRB, 0.53391E+00_JPRB, 0.56013E+00_JPRB, 0.60807E+00_JPRB, 0.65327E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.61372E+00_JPRB, 0.60742E+00_JPRB, 0.60086E+00_JPRB, 0.61549E+00_JPRB, 0.63525E+00_JPRB, 0.67391E+00_JPRB ,&
&  0.72593E+00_JPRB, 0.76376E+00_JPRB, 0.78403E+00_JPRB, 0.79879E+00_JPRB, 0.82824E+00_JPRB, 0.86478E+00_JPRB, 0.10782E+01_JPRB ,&
&  0.15400E+01_JPRB, 0.31307E+01_JPRB, 0.24809E+01_JPRB, 0.15871E+01_JPRB, 0.10336E+01_JPRB, 0.84091E+00_JPRB, 0.77650E+00_JPRB ,&
&  0.69717E+00_JPRB, 0.62133E+00_JPRB, 0.83768E+00_JPRB, 0.85231E+00_JPRB, 0.78407E+00_JPRB, 0.75141E+00_JPRB, 0.76865E+00_JPRB ,&
&  0.82016E+00_JPRB, 0.86492E+00_JPRB, 0.89468E+00_JPRB, 0.89565E+00_JPRB, 0.87086E+00_JPRB, 0.85970E+00_JPRB, 0.86265E+00_JPRB ,&
&  0.85944E+00_JPRB, 0.87343E+00_JPRB, 0.97091E+00_JPRB, 0.97795E+00_JPRB, 0.96975E+00_JPRB, 0.94128E+00_JPRB, 0.90942E+00_JPRB ,&
&  0.92026E+00_JPRB, 0.95113E+00_JPRB, 0.99059E+00_JPRB, 0.99371E+00_JPRB, 0.86742E+00_JPRB, 0.89163E+00_JPRB, 0.81371E+00_JPRB ,&
&  0.79598E+00_JPRB, 0.76462E+00_JPRB, 0.72943E+00_JPRB, 0.68422E+00_JPRB, 0.69815E+00_JPRB, 0.70897E+00_JPRB, 0.69607E+00_JPRB ,&
&  0.72645E+00_JPRB, 0.76097E+00_JPRB, 0.74898E+00_JPRB, 0.75894E+00_JPRB, 0.76727E+00_JPRB, 0.78591E+00_JPRB, 0.80141E+00_JPRB ,&
&  0.81094E+00_JPRB, 0.83997E+00_JPRB, 0.78721E+00_JPRB, 0.73427E+00_JPRB, 0.71561E+00_JPRB, 0.69071E+00_JPRB, 0.68746E+00_JPRB ,&
&  0.71520E+00_JPRB, 0.76365E+00_JPRB, 0.76091E+00_JPRB, 0.76615E+00_JPRB, 0.74674E+00_JPRB, 0.74756E+00_JPRB, 0.73104E+00_JPRB ,&
&  0.73493E+00_JPRB, 0.73554E+00_JPRB, 0.77916E+00_JPRB, 0.71943E+00_JPRB, 0.63773E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.53392E+00_JPRB, 0.52831E+00_JPRB, 0.52224E+00_JPRB, 0.52639E+00_JPRB, 0.53029E+00_JPRB, 0.52561E+00_JPRB, 0.52849E+00_JPRB ,&
&  0.53698E+00_JPRB, 0.54709E+00_JPRB, 0.54551E+00_JPRB, 0.57644E+00_JPRB, 0.64541E+00_JPRB, 0.71790E+00_JPRB, 0.71693E+00_JPRB ,&
&  0.70103E+00_JPRB, 0.68421E+00_JPRB, 0.67582E+00_JPRB, 0.68593E+00_JPRB, 0.69431E+00_JPRB, 0.72953E+00_JPRB, 0.76570E+00_JPRB ,&
&  0.79928E+00_JPRB, 0.83362E+00_JPRB, 0.86559E+00_JPRB, 0.90428E+00_JPRB, 0.96616E+00_JPRB, 0.10510E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.16621E+01_JPRB, 0.30117E+01_JPRB, 0.96912E+00_JPRB, 0.99140E+00_JPRB, 0.94162E+00_JPRB, 0.79720E+00_JPRB, 0.67514E+00_JPRB ,&
&  0.58671E+00_JPRB, 0.52926E+00_JPRB, 0.73573E+00_JPRB, 0.86549E+00_JPRB, 0.81979E+00_JPRB, 0.83536E+00_JPRB, 0.87033E+00_JPRB ,&
&  0.93174E+00_JPRB, 0.96450E+00_JPRB, 0.97610E+00_JPRB, 0.95264E+00_JPRB, 0.93004E+00_JPRB, 0.92475E+00_JPRB, 0.94787E+00_JPRB ,&
&  0.98965E+00_JPRB, 0.10560E+01_JPRB, 0.10968E+01_JPRB, 0.98755E+00_JPRB, 0.98867E+00_JPRB, 0.92491E+00_JPRB, 0.87244E+00_JPRB ,&
&  0.79528E+00_JPRB, 0.85927E+00_JPRB, 0.95004E+00_JPRB, 0.98239E+00_JPRB, 0.85096E+00_JPRB, 0.78382E+00_JPRB, 0.81105E+00_JPRB ,&
&  0.78376E+00_JPRB, 0.73589E+00_JPRB, 0.69582E+00_JPRB, 0.66515E+00_JPRB, 0.67541E+00_JPRB, 0.70443E+00_JPRB, 0.69576E+00_JPRB ,&
&  0.72530E+00_JPRB, 0.76055E+00_JPRB, 0.74044E+00_JPRB, 0.76652E+00_JPRB, 0.78595E+00_JPRB, 0.79768E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.81048E+00_JPRB, 0.84076E+00_JPRB, 0.85737E+00_JPRB, 0.83442E+00_JPRB, 0.77591E+00_JPRB, 0.73885E+00_JPRB, 0.74932E+00_JPRB ,&
&  0.75910E+00_JPRB, 0.78677E+00_JPRB, 0.76312E+00_JPRB, 0.72892E+00_JPRB, 0.74634E+00_JPRB, 0.75059E+00_JPRB, 0.77594E+00_JPRB ,&
&  0.81563E+00_JPRB, 0.90057E+00_JPRB, 0.95007E+00_JPRB, 0.67087E+00_JPRB, 0.57805E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.50856E+00_JPRB, 0.50262E+00_JPRB, 0.49249E+00_JPRB, 0.50361E+00_JPRB, 0.50335E+00_JPRB, 0.51155E+00_JPRB, 0.52787E+00_JPRB ,&
&  0.55518E+00_JPRB, 0.59022E+00_JPRB, 0.62089E+00_JPRB, 0.66115E+00_JPRB, 0.71945E+00_JPRB, 0.77086E+00_JPRB, 0.76824E+00_JPRB ,&
&  0.75061E+00_JPRB, 0.74044E+00_JPRB, 0.73884E+00_JPRB, 0.74072E+00_JPRB, 0.76136E+00_JPRB, 0.78989E+00_JPRB, 0.81679E+00_JPRB ,&
&  0.85469E+00_JPRB, 0.90682E+00_JPRB, 0.95814E+00_JPRB, 0.10230E+01_JPRB, 0.10983E+01_JPRB, 0.12179E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.18012E+01_JPRB, 0.16558E+01_JPRB, 0.71464E+00_JPRB, 0.95610E+00_JPRB, 0.85787E+00_JPRB, 0.69684E+00_JPRB, 0.55273E+00_JPRB ,&
&  0.48743E+00_JPRB, 0.58166E+00_JPRB, 0.77556E+00_JPRB, 0.89260E+00_JPRB, 0.88122E+00_JPRB, 0.92853E+00_JPRB, 0.97098E+00_JPRB ,&
&  0.10114E+01_JPRB, 0.10290E+01_JPRB, 0.10245E+01_JPRB, 0.10114E+01_JPRB, 0.10147E+01_JPRB, 0.10387E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11817E+01_JPRB, 0.11815E+01_JPRB, 0.10240E+01_JPRB, 0.10306E+01_JPRB, 0.93759E+00_JPRB, 0.88964E+00_JPRB ,&
&  0.78523E+00_JPRB, 0.78030E+00_JPRB, 0.85632E+00_JPRB, 0.91741E+00_JPRB, 0.81451E+00_JPRB, 0.73376E+00_JPRB, 0.79337E+00_JPRB ,&
&  0.74850E+00_JPRB, 0.68981E+00_JPRB, 0.67101E+00_JPRB, 0.66326E+00_JPRB, 0.66789E+00_JPRB, 0.68221E+00_JPRB, 0.70071E+00_JPRB ,&
&  0.72179E+00_JPRB, 0.71802E+00_JPRB, 0.69296E+00_JPRB, 0.71189E+00_JPRB, 0.73424E+00_JPRB, 0.75493E+00_JPRB, 0.77059E+00_JPRB ,&
&  0.79604E+00_JPRB, 0.82646E+00_JPRB, 0.84964E+00_JPRB, 0.86757E+00_JPRB, 0.83293E+00_JPRB, 0.78834E+00_JPRB, 0.79658E+00_JPRB ,&
&  0.75048E+00_JPRB, 0.74099E+00_JPRB, 0.73912E+00_JPRB, 0.73322E+00_JPRB, 0.71896E+00_JPRB, 0.75934E+00_JPRB, 0.78799E+00_JPRB ,&
&  0.83138E+00_JPRB, 0.86951E+00_JPRB, 0.79583E+00_JPRB, 0.60513E+00_JPRB, 0.53739E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.51551E+00_JPRB, 0.51725E+00_JPRB, 0.50835E+00_JPRB, 0.50604E+00_JPRB, 0.49894E+00_JPRB, 0.50313E+00_JPRB, 0.51678E+00_JPRB ,&
&  0.54671E+00_JPRB, 0.59723E+00_JPRB, 0.65838E+00_JPRB, 0.71228E+00_JPRB, 0.77762E+00_JPRB, 0.82524E+00_JPRB, 0.82112E+00_JPRB ,&
&  0.79801E+00_JPRB, 0.78739E+00_JPRB, 0.76790E+00_JPRB, 0.78482E+00_JPRB, 0.79972E+00_JPRB, 0.82642E+00_JPRB, 0.85631E+00_JPRB ,&
&  0.89136E+00_JPRB, 0.93505E+00_JPRB, 0.98730E+00_JPRB, 0.10485E+01_JPRB, 0.11371E+01_JPRB, 0.12277E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.84007E+00_JPRB, 0.43233E+00_JPRB, 0.75519E+00_JPRB, 0.92293E+00_JPRB, 0.71672E+00_JPRB, 0.60924E+00_JPRB, 0.50181E+00_JPRB ,&
&  0.47764E+00_JPRB, 0.66425E+00_JPRB, 0.80711E+00_JPRB, 0.92615E+00_JPRB, 0.94535E+00_JPRB, 0.99739E+00_JPRB, 0.10363E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10702E+01_JPRB, 0.10841E+01_JPRB, 0.10964E+01_JPRB, 0.11225E+01_JPRB, 0.11552E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12538E+01_JPRB, 0.12565E+01_JPRB, 0.12455E+01_JPRB, 0.10450E+01_JPRB, 0.10913E+01_JPRB, 0.10540E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10737E+01_JPRB, 0.76464E+00_JPRB, 0.71013E+00_JPRB, 0.91714E+00_JPRB, 0.87518E+00_JPRB, 0.70029E+00_JPRB, 0.77130E+00_JPRB ,&
&  0.70744E+00_JPRB, 0.65195E+00_JPRB, 0.64211E+00_JPRB, 0.64772E+00_JPRB, 0.63694E+00_JPRB, 0.66354E+00_JPRB, 0.68183E+00_JPRB ,&
&  0.70282E+00_JPRB, 0.67992E+00_JPRB, 0.65332E+00_JPRB, 0.66298E+00_JPRB, 0.68275E+00_JPRB, 0.70736E+00_JPRB, 0.72557E+00_JPRB ,&
&  0.75279E+00_JPRB, 0.77368E+00_JPRB, 0.79802E+00_JPRB, 0.82940E+00_JPRB, 0.85753E+00_JPRB, 0.81615E+00_JPRB, 0.79645E+00_JPRB ,&
&  0.72434E+00_JPRB, 0.69689E+00_JPRB, 0.68102E+00_JPRB, 0.67274E+00_JPRB, 0.67458E+00_JPRB, 0.70357E+00_JPRB, 0.73718E+00_JPRB ,&
&  0.76794E+00_JPRB, 0.77396E+00_JPRB, 0.70639E+00_JPRB, 0.59253E+00_JPRB, 0.53340E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.50578E+00_JPRB, 0.50838E+00_JPRB, 0.50564E+00_JPRB, 0.51381E+00_JPRB, 0.51123E+00_JPRB, 0.51002E+00_JPRB, 0.50596E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.57305E+00_JPRB, 0.64107E+00_JPRB, 0.71716E+00_JPRB, 0.77196E+00_JPRB, 0.84076E+00_JPRB, 0.86841E+00_JPRB ,&
&  0.85376E+00_JPRB, 0.82452E+00_JPRB, 0.81211E+00_JPRB, 0.82662E+00_JPRB, 0.83479E+00_JPRB, 0.85638E+00_JPRB, 0.87916E+00_JPRB ,&
&  0.90278E+00_JPRB, 0.93950E+00_JPRB, 0.98053E+00_JPRB, 0.10276E+01_JPRB, 0.10863E+01_JPRB, 0.11031E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.39286E+00_JPRB, 0.56443E+00_JPRB, 0.83228E+00_JPRB, 0.87412E+00_JPRB, 0.66896E+00_JPRB, 0.57408E+00_JPRB, 0.50043E+00_JPRB ,&
&  0.56120E+00_JPRB, 0.69769E+00_JPRB, 0.81752E+00_JPRB, 0.95486E+00_JPRB, 0.96054E+00_JPRB, 0.10105E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10900E+01_JPRB, 0.11272E+01_JPRB, 0.11729E+01_JPRB, 0.12044E+01_JPRB, 0.12490E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.13316E+01_JPRB, 0.13251E+01_JPRB, 0.13384E+01_JPRB, 0.11165E+01_JPRB, 0.11393E+01_JPRB, 0.11749E+01_JPRB, 0.13737E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.86715E+00_JPRB, 0.76566E+00_JPRB, 0.89213E+00_JPRB, 0.88751E+00_JPRB, 0.86408E+00_JPRB, 0.77553E+00_JPRB ,&
&  0.69328E+00_JPRB, 0.62437E+00_JPRB, 0.58570E+00_JPRB, 0.57348E+00_JPRB, 0.58700E+00_JPRB, 0.63540E+00_JPRB, 0.67325E+00_JPRB ,&
&  0.65697E+00_JPRB, 0.63633E+00_JPRB, 0.62416E+00_JPRB, 0.61961E+00_JPRB, 0.64563E+00_JPRB, 0.67445E+00_JPRB, 0.68632E+00_JPRB ,&
&  0.68755E+00_JPRB, 0.69201E+00_JPRB, 0.71026E+00_JPRB, 0.73645E+00_JPRB, 0.76370E+00_JPRB, 0.78222E+00_JPRB, 0.79274E+00_JPRB ,&
&  0.70579E+00_JPRB, 0.66357E+00_JPRB, 0.66884E+00_JPRB, 0.67532E+00_JPRB, 0.71265E+00_JPRB, 0.75069E+00_JPRB, 0.70567E+00_JPRB ,&
&  0.67763E+00_JPRB, 0.66493E+00_JPRB, 0.63529E+00_JPRB, 0.57302E+00_JPRB, 0.51563E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.51883E+00_JPRB, 0.53312E+00_JPRB, 0.55189E+00_JPRB, 0.57415E+00_JPRB, 0.56967E+00_JPRB, 0.56876E+00_JPRB, 0.55521E+00_JPRB ,&
&  0.55176E+00_JPRB, 0.56847E+00_JPRB, 0.61132E+00_JPRB, 0.68313E+00_JPRB, 0.73589E+00_JPRB, 0.80976E+00_JPRB, 0.90718E+00_JPRB ,&
&  0.92831E+00_JPRB, 0.90014E+00_JPRB, 0.87948E+00_JPRB, 0.87275E+00_JPRB, 0.88345E+00_JPRB, 0.90369E+00_JPRB, 0.91982E+00_JPRB ,&
&  0.93524E+00_JPRB, 0.97550E+00_JPRB, 0.10052E+01_JPRB, 0.10513E+01_JPRB, 0.10940E+01_JPRB, 0.10951E+01_JPRB, 0.10885E+01_JPRB ,&
&  0.65304E+00_JPRB, 0.90882E+00_JPRB, 0.87073E+00_JPRB, 0.84031E+00_JPRB, 0.71337E+00_JPRB, 0.62199E+00_JPRB, 0.59583E+00_JPRB ,&
&  0.63928E+00_JPRB, 0.69333E+00_JPRB, 0.81274E+00_JPRB, 0.84244E+00_JPRB, 0.91228E+00_JPRB, 0.95354E+00_JPRB, 0.10336E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10876E+01_JPRB, 0.11212E+01_JPRB, 0.11575E+01_JPRB, 0.11863E+01_JPRB, 0.12391E+01_JPRB, 0.12879E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13345E+01_JPRB, 0.13374E+01_JPRB, 0.11785E+01_JPRB, 0.10316E+01_JPRB, 0.10286E+01_JPRB, 0.98965E+00_JPRB ,&
&  0.87522E+00_JPRB, 0.66668E+00_JPRB, 0.70586E+00_JPRB, 0.85043E+00_JPRB, 0.85007E+00_JPRB, 0.82103E+00_JPRB, 0.75236E+00_JPRB ,&
&  0.67177E+00_JPRB, 0.59626E+00_JPRB, 0.55751E+00_JPRB, 0.52433E+00_JPRB, 0.52292E+00_JPRB, 0.56128E+00_JPRB, 0.60373E+00_JPRB ,&
&  0.59608E+00_JPRB, 0.57875E+00_JPRB, 0.57086E+00_JPRB, 0.58462E+00_JPRB, 0.61588E+00_JPRB, 0.64215E+00_JPRB, 0.66185E+00_JPRB ,&
&  0.63692E+00_JPRB, 0.60414E+00_JPRB, 0.59596E+00_JPRB, 0.61112E+00_JPRB, 0.63661E+00_JPRB, 0.66347E+00_JPRB, 0.69828E+00_JPRB ,&
&  0.72065E+00_JPRB, 0.73154E+00_JPRB, 0.82883E+00_JPRB, 0.10242E+01_JPRB, 0.11564E+01_JPRB, 0.12433E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.81531E+00_JPRB, 0.68256E+00_JPRB, 0.60218E+00_JPRB, 0.54881E+00_JPRB, 0.50805E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.51570E+00_JPRB, 0.56215E+00_JPRB, 0.59883E+00_JPRB, 0.65127E+00_JPRB, 0.66775E+00_JPRB, 0.66959E+00_JPRB, 0.66612E+00_JPRB ,&
&  0.65385E+00_JPRB, 0.65990E+00_JPRB, 0.68101E+00_JPRB, 0.73169E+00_JPRB, 0.78511E+00_JPRB, 0.83673E+00_JPRB, 0.89060E+00_JPRB ,&
&  0.95802E+00_JPRB, 0.99272E+00_JPRB, 0.97105E+00_JPRB, 0.96790E+00_JPRB, 0.98593E+00_JPRB, 0.10092E+01_JPRB, 0.10109E+01_JPRB ,&
&  0.99935E+00_JPRB, 0.10020E+01_JPRB, 0.10070E+01_JPRB, 0.10340E+01_JPRB, 0.10683E+01_JPRB, 0.10794E+01_JPRB, 0.74656E+00_JPRB ,&
&  0.94739E+00_JPRB, 0.87595E+00_JPRB, 0.84140E+00_JPRB, 0.84264E+00_JPRB, 0.79950E+00_JPRB, 0.73195E+00_JPRB, 0.69383E+00_JPRB ,&
&  0.68860E+00_JPRB, 0.70157E+00_JPRB, 0.73326E+00_JPRB, 0.75773E+00_JPRB, 0.82274E+00_JPRB, 0.85015E+00_JPRB, 0.92540E+00_JPRB ,&
&  0.99112E+00_JPRB, 0.10126E+01_JPRB, 0.10240E+01_JPRB, 0.10357E+01_JPRB, 0.10489E+01_JPRB, 0.10773E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.11308E+01_JPRB, 0.11112E+01_JPRB, 0.10696E+01_JPRB, 0.95659E+00_JPRB, 0.95213E+00_JPRB, 0.78345E+00_JPRB ,&
&  0.57726E+00_JPRB, 0.49786E+00_JPRB, 0.66445E+00_JPRB, 0.85165E+00_JPRB, 0.81943E+00_JPRB, 0.78050E+00_JPRB, 0.70913E+00_JPRB ,&
&  0.61798E+00_JPRB, 0.57302E+00_JPRB, 0.54686E+00_JPRB, 0.52279E+00_JPRB, 0.50817E+00_JPRB, 0.52227E+00_JPRB, 0.54330E+00_JPRB ,&
&  0.54888E+00_JPRB, 0.54481E+00_JPRB, 0.54104E+00_JPRB, 0.55629E+00_JPRB, 0.58772E+00_JPRB, 0.70088E+00_JPRB, 0.91631E+00_JPRB ,&
&  0.11093E+01_JPRB, 0.54548E+00_JPRB, 0.49625E+00_JPRB, 0.51268E+00_JPRB, 0.55351E+00_JPRB, 0.59973E+00_JPRB, 0.66495E+00_JPRB ,&
&  0.82412E+00_JPRB, 0.10771E+01_JPRB, 0.10178E+01_JPRB, 0.12143E+01_JPRB, 0.25005E+01_JPRB, 0.27347E+01_JPRB, 0.15722E+01_JPRB ,&
&  0.98412E+00_JPRB, 0.69308E+00_JPRB, 0.55994E+00_JPRB, 0.51721E+00_JPRB, 0.50630E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.54332E+00_JPRB, 0.60352E+00_JPRB, 0.67042E+00_JPRB, 0.71180E+00_JPRB, 0.68465E+00_JPRB, 0.67734E+00_JPRB, 0.68894E+00_JPRB ,&
&  0.70896E+00_JPRB, 0.73819E+00_JPRB, 0.76974E+00_JPRB, 0.81202E+00_JPRB, 0.84137E+00_JPRB, 0.87890E+00_JPRB, 0.92908E+00_JPRB ,&
&  0.98499E+00_JPRB, 0.98982E+00_JPRB, 0.10051E+01_JPRB, 0.10217E+01_JPRB, 0.10339E+01_JPRB, 0.10497E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10666E+01_JPRB, 0.10608E+01_JPRB, 0.10788E+01_JPRB, 0.10770E+01_JPRB, 0.11497E+01_JPRB, 0.83114E+00_JPRB ,&
&  0.99602E+00_JPRB, 0.92108E+00_JPRB, 0.84363E+00_JPRB, 0.84310E+00_JPRB, 0.82548E+00_JPRB, 0.80276E+00_JPRB, 0.77685E+00_JPRB ,&
&  0.77583E+00_JPRB, 0.76010E+00_JPRB, 0.71669E+00_JPRB, 0.73729E+00_JPRB, 0.73634E+00_JPRB, 0.75229E+00_JPRB, 0.79688E+00_JPRB ,&
&  0.84129E+00_JPRB, 0.87354E+00_JPRB, 0.87865E+00_JPRB, 0.89485E+00_JPRB, 0.89690E+00_JPRB, 0.88557E+00_JPRB, 0.86999E+00_JPRB ,&
&  0.84885E+00_JPRB, 0.83942E+00_JPRB, 0.83197E+00_JPRB, 0.80900E+00_JPRB, 0.81268E+00_JPRB, 0.72982E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.47544E+00_JPRB, 0.47854E+00_JPRB, 0.72009E+00_JPRB, 0.91354E+00_JPRB, 0.82639E+00_JPRB, 0.78536E+00_JPRB, 0.68979E+00_JPRB ,&
&  0.62580E+00_JPRB, 0.56154E+00_JPRB, 0.53957E+00_JPRB, 0.50425E+00_JPRB, 0.49110E+00_JPRB, 0.50233E+00_JPRB, 0.53388E+00_JPRB ,&
&  0.54919E+00_JPRB, 0.54030E+00_JPRB, 0.53111E+00_JPRB, 0.50918E+00_JPRB, 0.51765E+00_JPRB, 0.55860E+00_JPRB, 0.53902E+00_JPRB ,&
&  0.63251E+00_JPRB, 0.50050E+00_JPRB, 0.49434E+00_JPRB, 0.55178E+00_JPRB, 0.50666E+00_JPRB, 0.55220E+00_JPRB, 0.61938E+00_JPRB ,&
&  0.80512E+00_JPRB, 0.88460E+00_JPRB, 0.12206E+01_JPRB, 0.34191E+01_JPRB, 0.39413E+01_JPRB, 0.49419E+01_JPRB, 0.19658E+01_JPRB ,&
&  0.90984E+00_JPRB, 0.60167E+00_JPRB, 0.53293E+00_JPRB, 0.53140E+00_JPRB, 0.52423E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.55869E+00_JPRB, 0.63804E+00_JPRB, 0.70422E+00_JPRB, 0.63116E+00_JPRB, 0.63652E+00_JPRB, 0.65253E+00_JPRB, 0.67852E+00_JPRB ,&
&  0.70850E+00_JPRB, 0.74529E+00_JPRB, 0.78439E+00_JPRB, 0.83085E+00_JPRB, 0.86734E+00_JPRB, 0.90104E+00_JPRB, 0.93786E+00_JPRB ,&
&  0.97680E+00_JPRB, 0.98788E+00_JPRB, 0.10183E+01_JPRB, 0.10205E+01_JPRB, 0.10425E+01_JPRB, 0.10393E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.11028E+01_JPRB, 0.11196E+01_JPRB, 0.11292E+01_JPRB, 0.11117E+01_JPRB, 0.11241E+01_JPRB, 0.84919E+00_JPRB ,&
&  0.10593E+01_JPRB, 0.96656E+00_JPRB, 0.88225E+00_JPRB, 0.86454E+00_JPRB, 0.83190E+00_JPRB, 0.82440E+00_JPRB, 0.81315E+00_JPRB ,&
&  0.80201E+00_JPRB, 0.79803E+00_JPRB, 0.78459E+00_JPRB, 0.74408E+00_JPRB, 0.71309E+00_JPRB, 0.72970E+00_JPRB, 0.74731E+00_JPRB ,&
&  0.77510E+00_JPRB, 0.80992E+00_JPRB, 0.83722E+00_JPRB, 0.85715E+00_JPRB, 0.87038E+00_JPRB, 0.86480E+00_JPRB, 0.85613E+00_JPRB ,&
&  0.84369E+00_JPRB, 0.84527E+00_JPRB, 0.79079E+00_JPRB, 0.78413E+00_JPRB, 0.82197E+00_JPRB, 0.72318E+00_JPRB, 0.58325E+00_JPRB ,&
&  0.48133E+00_JPRB, 0.62756E+00_JPRB, 0.83925E+00_JPRB, 0.10011E+01_JPRB, 0.88788E+00_JPRB, 0.81425E+00_JPRB, 0.74425E+00_JPRB ,&
&  0.65791E+00_JPRB, 0.59394E+00_JPRB, 0.56803E+00_JPRB, 0.52113E+00_JPRB, 0.50049E+00_JPRB, 0.49517E+00_JPRB, 0.49209E+00_JPRB ,&
&  0.48452E+00_JPRB, 0.47434E+00_JPRB, 0.45673E+00_JPRB, 0.44385E+00_JPRB, 0.40581E+00_JPRB, 0.36976E+00_JPRB, 0.54672E+00_JPRB ,&
&  0.47538E+00_JPRB, 0.39483E+00_JPRB, 0.59671E+00_JPRB, 0.41186E+00_JPRB, 0.54252E+00_JPRB, 0.59903E+00_JPRB, 0.64645E+00_JPRB ,&
&  0.60646E+00_JPRB, 0.76833E+00_JPRB, 0.11170E+01_JPRB, 0.11433E+01_JPRB, 0.12012E+01_JPRB, 0.12746E+01_JPRB, 0.75130E+00_JPRB ,&
&  0.60025E+00_JPRB, 0.58695E+00_JPRB, 0.58694E+00_JPRB, 0.54095E+00_JPRB, 0.53710E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.56356E+00_JPRB, 0.61882E+00_JPRB, 0.70711E+00_JPRB, 0.63547E+00_JPRB, 0.63214E+00_JPRB, 0.64518E+00_JPRB, 0.66567E+00_JPRB ,&
&  0.68671E+00_JPRB, 0.71503E+00_JPRB, 0.74562E+00_JPRB, 0.78832E+00_JPRB, 0.82550E+00_JPRB, 0.85863E+00_JPRB, 0.88996E+00_JPRB ,&
&  0.91865E+00_JPRB, 0.94300E+00_JPRB, 0.96651E+00_JPRB, 0.97674E+00_JPRB, 0.98877E+00_JPRB, 0.10084E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10979E+01_JPRB, 0.11244E+01_JPRB, 0.11702E+01_JPRB, 0.12015E+01_JPRB, 0.12203E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.99985E+00_JPRB, 0.92691E+00_JPRB, 0.87765E+00_JPRB, 0.83867E+00_JPRB, 0.81686E+00_JPRB, 0.81534E+00_JPRB ,&
&  0.83819E+00_JPRB, 0.79812E+00_JPRB, 0.79700E+00_JPRB, 0.77176E+00_JPRB, 0.76373E+00_JPRB, 0.79373E+00_JPRB, 0.81340E+00_JPRB ,&
&  0.86369E+00_JPRB, 0.92945E+00_JPRB, 0.98786E+00_JPRB, 0.10300E+01_JPRB, 0.10738E+01_JPRB, 0.10730E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.11414E+01_JPRB, 0.10918E+01_JPRB, 0.10871E+01_JPRB, 0.10672E+01_JPRB, 0.98817E+00_JPRB, 0.80890E+00_JPRB ,&
&  0.69444E+00_JPRB, 0.79060E+00_JPRB, 0.84369E+00_JPRB, 0.99255E+00_JPRB, 0.92801E+00_JPRB, 0.84264E+00_JPRB, 0.79431E+00_JPRB ,&
&  0.72347E+00_JPRB, 0.67135E+00_JPRB, 0.63743E+00_JPRB, 0.58516E+00_JPRB, 0.58837E+00_JPRB, 0.58818E+00_JPRB, 0.57924E+00_JPRB ,&
&  0.53724E+00_JPRB, 0.49289E+00_JPRB, 0.45097E+00_JPRB, 0.41805E+00_JPRB, 0.41798E+00_JPRB, 0.41167E+00_JPRB, 0.47919E+00_JPRB ,&
&  0.50019E+00_JPRB, 0.35848E+00_JPRB, 0.49033E+00_JPRB, 0.63200E+00_JPRB, 0.67563E+00_JPRB, 0.71618E+00_JPRB, 0.77637E+00_JPRB ,&
&  0.84560E+00_JPRB, 0.83867E+00_JPRB, 0.80209E+00_JPRB, 0.79273E+00_JPRB, 0.78018E+00_JPRB, 0.77437E+00_JPRB, 0.69221E+00_JPRB ,&
&  0.66175E+00_JPRB, 0.60592E+00_JPRB, 0.57485E+00_JPRB, 0.56178E+00_JPRB, 0.53642E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.60967E+00_JPRB, 0.62564E+00_JPRB, 0.68977E+00_JPRB, 0.64820E+00_JPRB, 0.64628E+00_JPRB, 0.65984E+00_JPRB, 0.67600E+00_JPRB ,&
&  0.67577E+00_JPRB, 0.68473E+00_JPRB, 0.70364E+00_JPRB, 0.73672E+00_JPRB, 0.76898E+00_JPRB, 0.79417E+00_JPRB, 0.82175E+00_JPRB ,&
&  0.84326E+00_JPRB, 0.85684E+00_JPRB, 0.86507E+00_JPRB, 0.87422E+00_JPRB, 0.90079E+00_JPRB, 0.93125E+00_JPRB, 0.97311E+00_JPRB ,&
&  0.10242E+01_JPRB, 0.10352E+01_JPRB, 0.10306E+01_JPRB, 0.10537E+01_JPRB, 0.10777E+01_JPRB, 0.11683E+01_JPRB, 0.13040E+01_JPRB ,&
&  0.99476E+00_JPRB, 0.10308E+01_JPRB, 0.95896E+00_JPRB, 0.87429E+00_JPRB, 0.87428E+00_JPRB, 0.84303E+00_JPRB, 0.85508E+00_JPRB ,&
&  0.87811E+00_JPRB, 0.86849E+00_JPRB, 0.88557E+00_JPRB, 0.89819E+00_JPRB, 0.90709E+00_JPRB, 0.93302E+00_JPRB, 0.96510E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10789E+01_JPRB, 0.11375E+01_JPRB, 0.12238E+01_JPRB, 0.13084E+01_JPRB, 0.13863E+01_JPRB, 0.14399E+01_JPRB ,&
&  0.14701E+01_JPRB, 0.14754E+01_JPRB, 0.13553E+01_JPRB, 0.13349E+01_JPRB, 0.13253E+01_JPRB, 0.12672E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.99524E+00_JPRB, 0.95249E+00_JPRB, 0.96231E+00_JPRB, 0.92200E+00_JPRB, 0.91697E+00_JPRB, 0.90673E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.82056E+00_JPRB, 0.76232E+00_JPRB, 0.72002E+00_JPRB, 0.68721E+00_JPRB, 0.68271E+00_JPRB, 0.69744E+00_JPRB, 0.68454E+00_JPRB ,&
&  0.64166E+00_JPRB, 0.59098E+00_JPRB, 0.54387E+00_JPRB, 0.49280E+00_JPRB, 0.44214E+00_JPRB, 0.51501E+00_JPRB, 0.55395E+00_JPRB ,&
&  0.58355E+00_JPRB, 0.52528E+00_JPRB, 0.55040E+00_JPRB, 0.75419E+00_JPRB, 0.81638E+00_JPRB, 0.80898E+00_JPRB, 0.83656E+00_JPRB ,&
&  0.84273E+00_JPRB, 0.83282E+00_JPRB, 0.85178E+00_JPRB, 0.80472E+00_JPRB, 0.76842E+00_JPRB, 0.74781E+00_JPRB, 0.73584E+00_JPRB ,&
&  0.68240E+00_JPRB, 0.63170E+00_JPRB, 0.58502E+00_JPRB, 0.58705E+00_JPRB, 0.56005E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.60358E+00_JPRB, 0.60466E+00_JPRB, 0.62558E+00_JPRB, 0.66340E+00_JPRB, 0.64214E+00_JPRB, 0.65971E+00_JPRB, 0.65488E+00_JPRB ,&
&  0.65383E+00_JPRB, 0.64788E+00_JPRB, 0.65909E+00_JPRB, 0.67138E+00_JPRB, 0.69174E+00_JPRB, 0.71160E+00_JPRB, 0.71820E+00_JPRB ,&
&  0.73036E+00_JPRB, 0.74341E+00_JPRB, 0.76398E+00_JPRB, 0.79800E+00_JPRB, 0.85738E+00_JPRB, 0.95262E+00_JPRB, 0.11362E+01_JPRB ,&
&  0.13182E+01_JPRB, 0.13992E+01_JPRB, 0.14093E+01_JPRB, 0.13860E+01_JPRB, 0.15215E+01_JPRB, 0.18909E+01_JPRB, 0.21419E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.10626E+01_JPRB, 0.96268E+00_JPRB, 0.86307E+00_JPRB, 0.87087E+00_JPRB, 0.92570E+00_JPRB, 0.93136E+00_JPRB ,&
&  0.92831E+00_JPRB, 0.95176E+00_JPRB, 0.99340E+00_JPRB, 0.10444E+01_JPRB, 0.10823E+01_JPRB, 0.11096E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.11914E+01_JPRB, 0.12590E+01_JPRB, 0.13365E+01_JPRB, 0.14256E+01_JPRB, 0.14082E+01_JPRB, 0.15345E+01_JPRB, 0.15669E+01_JPRB ,&
&  0.15866E+01_JPRB, 0.14911E+01_JPRB, 0.12925E+01_JPRB, 0.12879E+01_JPRB, 0.13684E+01_JPRB, 0.13594E+01_JPRB, 0.12530E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.10875E+01_JPRB, 0.85936E+00_JPRB, 0.86097E+00_JPRB, 0.88840E+00_JPRB, 0.94621E+00_JPRB, 0.92158E+00_JPRB ,&
&  0.91337E+00_JPRB, 0.85971E+00_JPRB, 0.81819E+00_JPRB, 0.79520E+00_JPRB, 0.78776E+00_JPRB, 0.80197E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.72886E+00_JPRB, 0.69180E+00_JPRB, 0.64088E+00_JPRB, 0.61320E+00_JPRB, 0.60694E+00_JPRB, 0.55369E+00_JPRB, 0.66600E+00_JPRB ,&
&  0.69946E+00_JPRB, 0.71621E+00_JPRB, 0.65224E+00_JPRB, 0.83449E+00_JPRB, 0.85059E+00_JPRB, 0.82633E+00_JPRB, 0.83047E+00_JPRB ,&
&  0.85078E+00_JPRB, 0.80704E+00_JPRB, 0.78122E+00_JPRB, 0.76034E+00_JPRB, 0.74814E+00_JPRB, 0.72496E+00_JPRB, 0.67714E+00_JPRB ,&
&  0.64286E+00_JPRB, 0.61974E+00_JPRB, 0.60715E+00_JPRB, 0.59728E+00_JPRB, 0.59934E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.58960E+00_JPRB, 0.60643E+00_JPRB, 0.61683E+00_JPRB, 0.61026E+00_JPRB, 0.61496E+00_JPRB, 0.62073E+00_JPRB, 0.60897E+00_JPRB ,&
&  0.60047E+00_JPRB, 0.59530E+00_JPRB, 0.60913E+00_JPRB, 0.61042E+00_JPRB, 0.61420E+00_JPRB, 0.62943E+00_JPRB, 0.64879E+00_JPRB ,&
&  0.67022E+00_JPRB, 0.68769E+00_JPRB, 0.75443E+00_JPRB, 0.88837E+00_JPRB, 0.11327E+01_JPRB, 0.14435E+01_JPRB, 0.16584E+01_JPRB ,&
&  0.17685E+01_JPRB, 0.18569E+01_JPRB, 0.19171E+01_JPRB, 0.19573E+01_JPRB, 0.17880E+01_JPRB, 0.17725E+01_JPRB, 0.18610E+01_JPRB ,&
&  0.13541E+01_JPRB, 0.10068E+01_JPRB, 0.10438E+01_JPRB, 0.95589E+00_JPRB, 0.94660E+00_JPRB, 0.95761E+00_JPRB, 0.95967E+00_JPRB ,&
&  0.96952E+00_JPRB, 0.99583E+00_JPRB, 0.10383E+01_JPRB, 0.11034E+01_JPRB, 0.11632E+01_JPRB, 0.12011E+01_JPRB, 0.12238E+01_JPRB ,&
&  0.12456E+01_JPRB, 0.12732E+01_JPRB, 0.13266E+01_JPRB, 0.13108E+01_JPRB, 0.12912E+01_JPRB, 0.13311E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.13489E+01_JPRB, 0.13035E+01_JPRB, 0.11450E+01_JPRB, 0.11404E+01_JPRB, 0.12359E+01_JPRB, 0.11712E+01_JPRB, 0.11461E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.11165E+01_JPRB, 0.78728E+00_JPRB, 0.76866E+00_JPRB, 0.83337E+00_JPRB, 0.92386E+00_JPRB, 0.10063E+01_JPRB ,&
&  0.99471E+00_JPRB, 0.95934E+00_JPRB, 0.91539E+00_JPRB, 0.90185E+00_JPRB, 0.87876E+00_JPRB, 0.87354E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.79502E+00_JPRB, 0.76566E+00_JPRB, 0.73386E+00_JPRB, 0.70453E+00_JPRB, 0.69904E+00_JPRB, 0.69347E+00_JPRB, 0.71850E+00_JPRB ,&
&  0.76509E+00_JPRB, 0.77495E+00_JPRB, 0.79162E+00_JPRB, 0.79683E+00_JPRB, 0.78034E+00_JPRB, 0.82678E+00_JPRB, 0.80654E+00_JPRB ,&
&  0.81702E+00_JPRB, 0.79048E+00_JPRB, 0.76351E+00_JPRB, 0.73789E+00_JPRB, 0.71740E+00_JPRB, 0.69320E+00_JPRB, 0.66307E+00_JPRB ,&
&  0.63462E+00_JPRB, 0.61384E+00_JPRB, 0.60056E+00_JPRB, 0.58859E+00_JPRB, 0.57878E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.57878E+00_JPRB, 0.59185E+00_JPRB, 0.60813E+00_JPRB, 0.60053E+00_JPRB, 0.57975E+00_JPRB, 0.56954E+00_JPRB, 0.56346E+00_JPRB ,&
&  0.56753E+00_JPRB, 0.58522E+00_JPRB, 0.59691E+00_JPRB, 0.59913E+00_JPRB, 0.60128E+00_JPRB, 0.61843E+00_JPRB, 0.66190E+00_JPRB ,&
&  0.71889E+00_JPRB, 0.79151E+00_JPRB, 0.92296E+00_JPRB, 0.11871E+01_JPRB, 0.15073E+01_JPRB, 0.17553E+01_JPRB, 0.16920E+01_JPRB ,&
&  0.15615E+01_JPRB, 0.15461E+01_JPRB, 0.15177E+01_JPRB, 0.14938E+01_JPRB, 0.14868E+01_JPRB, 0.14929E+01_JPRB, 0.14809E+01_JPRB ,&
&  0.13071E+01_JPRB, 0.10741E+01_JPRB, 0.10034E+01_JPRB, 0.96602E+00_JPRB, 0.95608E+00_JPRB, 0.94422E+00_JPRB, 0.95305E+00_JPRB ,&
&  0.96271E+00_JPRB, 0.98563E+00_JPRB, 0.10226E+01_JPRB, 0.10845E+01_JPRB, 0.11536E+01_JPRB, 0.11942E+01_JPRB, 0.12070E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.12039E+01_JPRB, 0.12234E+01_JPRB, 0.11296E+01_JPRB, 0.11580E+01_JPRB, 0.11754E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.10938E+01_JPRB, 0.11379E+01_JPRB, 0.11381E+01_JPRB, 0.11365E+01_JPRB, 0.11149E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.10733E+01_JPRB, 0.81918E+00_JPRB, 0.82442E+00_JPRB, 0.87745E+00_JPRB, 0.84629E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.10001E+01_JPRB, 0.99101E+00_JPRB, 0.98469E+00_JPRB, 0.99134E+00_JPRB, 0.99771E+00_JPRB, 0.10007E+01_JPRB, 0.91411E+00_JPRB ,&
&  0.87445E+00_JPRB, 0.82613E+00_JPRB, 0.81301E+00_JPRB, 0.79614E+00_JPRB, 0.80139E+00_JPRB, 0.79878E+00_JPRB, 0.80102E+00_JPRB ,&
&  0.82179E+00_JPRB, 0.84606E+00_JPRB, 0.86186E+00_JPRB, 0.88332E+00_JPRB, 0.83651E+00_JPRB, 0.79451E+00_JPRB, 0.77945E+00_JPRB ,&
&  0.76559E+00_JPRB, 0.72917E+00_JPRB, 0.71722E+00_JPRB, 0.70173E+00_JPRB, 0.68917E+00_JPRB, 0.67697E+00_JPRB, 0.66468E+00_JPRB ,&
&  0.64576E+00_JPRB, 0.62878E+00_JPRB, 0.61138E+00_JPRB, 0.59231E+00_JPRB, 0.57905E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.58487E+00_JPRB, 0.59109E+00_JPRB, 0.60657E+00_JPRB, 0.61412E+00_JPRB, 0.59314E+00_JPRB, 0.58044E+00_JPRB, 0.57834E+00_JPRB ,&
&  0.58963E+00_JPRB, 0.60787E+00_JPRB, 0.62424E+00_JPRB, 0.63918E+00_JPRB, 0.67038E+00_JPRB, 0.68995E+00_JPRB, 0.72253E+00_JPRB ,&
&  0.78116E+00_JPRB, 0.87328E+00_JPRB, 0.10434E+01_JPRB, 0.13086E+01_JPRB, 0.15506E+01_JPRB, 0.17766E+01_JPRB, 0.16554E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.14725E+01_JPRB, 0.14307E+01_JPRB, 0.13545E+01_JPRB, 0.11809E+01_JPRB, 0.12106E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12315E+01_JPRB, 0.11352E+01_JPRB, 0.10087E+01_JPRB, 0.95667E+00_JPRB, 0.91419E+00_JPRB, 0.90806E+00_JPRB, 0.92142E+00_JPRB ,&
&  0.94072E+00_JPRB, 0.96006E+00_JPRB, 0.99438E+00_JPRB, 0.10392E+01_JPRB, 0.10930E+01_JPRB, 0.11457E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.11375E+01_JPRB, 0.11215E+01_JPRB, 0.10775E+01_JPRB, 0.10623E+01_JPRB, 0.10682E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.10556E+01_JPRB, 0.10788E+01_JPRB, 0.11093E+01_JPRB, 0.10698E+01_JPRB, 0.10089E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.11140E+01_JPRB, 0.87384E+00_JPRB, 0.99334E+00_JPRB, 0.11284E+01_JPRB, 0.10196E+01_JPRB, 0.99623E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.10247E+01_JPRB, 0.10470E+01_JPRB, 0.10730E+01_JPRB, 0.10901E+01_JPRB, 0.10970E+01_JPRB, 0.93699E+00_JPRB ,&
&  0.93352E+00_JPRB, 0.87156E+00_JPRB, 0.85864E+00_JPRB, 0.85589E+00_JPRB, 0.86573E+00_JPRB, 0.86066E+00_JPRB, 0.82340E+00_JPRB ,&
&  0.79913E+00_JPRB, 0.92640E+00_JPRB, 0.10786E+01_JPRB, 0.12634E+01_JPRB, 0.13215E+01_JPRB, 0.77944E+00_JPRB, 0.78861E+00_JPRB ,&
&  0.76535E+00_JPRB, 0.73297E+00_JPRB, 0.71759E+00_JPRB, 0.70369E+00_JPRB, 0.69387E+00_JPRB, 0.68265E+00_JPRB, 0.67553E+00_JPRB ,&
&  0.66284E+00_JPRB, 0.64506E+00_JPRB, 0.62652E+00_JPRB, 0.60910E+00_JPRB, 0.59426E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.60352E+00_JPRB, 0.60090E+00_JPRB, 0.61149E+00_JPRB, 0.62004E+00_JPRB, 0.62131E+00_JPRB, 0.60906E+00_JPRB, 0.60691E+00_JPRB ,&
&  0.60542E+00_JPRB, 0.61932E+00_JPRB, 0.63311E+00_JPRB, 0.66535E+00_JPRB, 0.70320E+00_JPRB, 0.76539E+00_JPRB, 0.82747E+00_JPRB ,&
&  0.89860E+00_JPRB, 0.99202E+00_JPRB, 0.11647E+01_JPRB, 0.13816E+01_JPRB, 0.15530E+01_JPRB, 0.16515E+01_JPRB, 0.15799E+01_JPRB ,&
&  0.14880E+01_JPRB, 0.14918E+01_JPRB, 0.14498E+01_JPRB, 0.11188E+01_JPRB, 0.10125E+01_JPRB, 0.11963E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.98193E+00_JPRB, 0.93087E+00_JPRB, 0.88875E+00_JPRB, 0.85111E+00_JPRB, 0.85281E+00_JPRB, 0.86791E+00_JPRB ,&
&  0.89617E+00_JPRB, 0.92355E+00_JPRB, 0.95733E+00_JPRB, 0.99377E+00_JPRB, 0.10315E+01_JPRB, 0.10921E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.11279E+01_JPRB, 0.11126E+01_JPRB, 0.10928E+01_JPRB, 0.10701E+01_JPRB, 0.10243E+01_JPRB, 0.10135E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.10263E+01_JPRB, 0.10362E+01_JPRB, 0.10648E+01_JPRB, 0.10743E+01_JPRB, 0.97698E+00_JPRB, 0.96024E+00_JPRB ,&
&  0.10033E+01_JPRB, 0.10665E+01_JPRB, 0.98374E+00_JPRB, 0.11132E+01_JPRB, 0.77180E+00_JPRB, 0.88105E+00_JPRB, 0.10111E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10219E+01_JPRB, 0.10416E+01_JPRB, 0.10748E+01_JPRB, 0.10943E+01_JPRB, 0.99045E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.92444E+00_JPRB, 0.90212E+00_JPRB, 0.88553E+00_JPRB, 0.89023E+00_JPRB, 0.88584E+00_JPRB, 0.83238E+00_JPRB ,&
&  0.77920E+00_JPRB, 0.90085E+00_JPRB, 0.10208E+01_JPRB, 0.11195E+01_JPRB, 0.10568E+01_JPRB, 0.80938E+00_JPRB, 0.79656E+00_JPRB ,&
&  0.78652E+00_JPRB, 0.74811E+00_JPRB, 0.72528E+00_JPRB, 0.70923E+00_JPRB, 0.69831E+00_JPRB, 0.68378E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.66241E+00_JPRB, 0.64681E+00_JPRB, 0.63474E+00_JPRB, 0.62297E+00_JPRB, 0.61147E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.61189E+00_JPRB, 0.61488E+00_JPRB, 0.62887E+00_JPRB, 0.62814E+00_JPRB, 0.63583E+00_JPRB, 0.64446E+00_JPRB, 0.62989E+00_JPRB ,&
&  0.60150E+00_JPRB, 0.57637E+00_JPRB, 0.59008E+00_JPRB, 0.62802E+00_JPRB, 0.67396E+00_JPRB, 0.74404E+00_JPRB, 0.83401E+00_JPRB ,&
&  0.94649E+00_JPRB, 0.10448E+01_JPRB, 0.11989E+01_JPRB, 0.13402E+01_JPRB, 0.14472E+01_JPRB, 0.15226E+01_JPRB, 0.15233E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.99958E+00_JPRB, 0.14107E+01_JPRB, 0.14892E+01_JPRB, 0.13275E+01_JPRB, 0.12343E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.95689E+00_JPRB, 0.95722E+00_JPRB, 0.94138E+00_JPRB, 0.88592E+00_JPRB, 0.80386E+00_JPRB, 0.79952E+00_JPRB, 0.81678E+00_JPRB ,&
&  0.84855E+00_JPRB, 0.87801E+00_JPRB, 0.91598E+00_JPRB, 0.96268E+00_JPRB, 0.99867E+00_JPRB, 0.10490E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.11034E+01_JPRB, 0.11023E+01_JPRB, 0.10976E+01_JPRB, 0.10654E+01_JPRB, 0.10392E+01_JPRB, 0.10162E+01_JPRB, 0.99474E+00_JPRB ,&
&  0.99784E+00_JPRB, 0.99288E+00_JPRB, 0.10199E+01_JPRB, 0.10415E+01_JPRB, 0.10801E+01_JPRB, 0.10100E+01_JPRB, 0.10355E+01_JPRB ,&
&  0.10245E+01_JPRB, 0.10505E+01_JPRB, 0.10447E+01_JPRB, 0.11277E+01_JPRB, 0.83886E+00_JPRB, 0.93749E+00_JPRB, 0.93083E+00_JPRB ,&
&  0.10683E+01_JPRB, 0.10664E+01_JPRB, 0.10594E+01_JPRB, 0.10601E+01_JPRB, 0.10778E+01_JPRB, 0.99238E+00_JPRB, 0.10127E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10173E+01_JPRB, 0.94961E+00_JPRB, 0.92381E+00_JPRB, 0.86512E+00_JPRB, 0.90509E+00_JPRB, 0.88579E+00_JPRB ,&
&  0.90886E+00_JPRB, 0.92329E+00_JPRB, 0.97174E+00_JPRB, 0.10351E+01_JPRB, 0.98936E+00_JPRB, 0.87666E+00_JPRB, 0.84015E+00_JPRB ,&
&  0.81668E+00_JPRB, 0.78850E+00_JPRB, 0.75078E+00_JPRB, 0.72231E+00_JPRB, 0.70317E+00_JPRB, 0.68478E+00_JPRB, 0.66694E+00_JPRB ,&
&  0.64849E+00_JPRB, 0.63297E+00_JPRB, 0.62219E+00_JPRB, 0.61285E+00_JPRB, 0.60986E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.59729E+00_JPRB, 0.61545E+00_JPRB, 0.64189E+00_JPRB, 0.67097E+00_JPRB, 0.78135E+00_JPRB, 0.90710E+00_JPRB, 0.97949E+00_JPRB ,&
&  0.92798E+00_JPRB, 0.56391E+00_JPRB, 0.55419E+00_JPRB, 0.57806E+00_JPRB, 0.61364E+00_JPRB, 0.67524E+00_JPRB, 0.74374E+00_JPRB ,&
&  0.83989E+00_JPRB, 0.94923E+00_JPRB, 0.11004E+01_JPRB, 0.12869E+01_JPRB, 0.14011E+01_JPRB, 0.15085E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.99887E+00_JPRB, 0.10618E+01_JPRB, 0.16026E+01_JPRB, 0.15307E+01_JPRB, 0.14134E+01_JPRB, 0.12993E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.92391E+00_JPRB, 0.89755E+00_JPRB, 0.86153E+00_JPRB, 0.77300E+00_JPRB, 0.74428E+00_JPRB, 0.74682E+00_JPRB ,&
&  0.77202E+00_JPRB, 0.81079E+00_JPRB, 0.86141E+00_JPRB, 0.91528E+00_JPRB, 0.97421E+00_JPRB, 0.10116E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10939E+01_JPRB, 0.10735E+01_JPRB, 0.10406E+01_JPRB, 0.10501E+01_JPRB, 0.10481E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10262E+01_JPRB, 0.10689E+01_JPRB, 0.10878E+01_JPRB, 0.10256E+01_JPRB, 0.94769E+00_JPRB, 0.10255E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10082E+01_JPRB, 0.11308E+01_JPRB, 0.94102E+00_JPRB, 0.95448E+00_JPRB, 0.10333E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.10933E+01_JPRB, 0.10904E+01_JPRB, 0.10750E+01_JPRB, 0.10602E+01_JPRB, 0.95919E+00_JPRB, 0.10059E+01_JPRB ,&
&  0.97516E+00_JPRB, 0.10785E+01_JPRB, 0.10272E+01_JPRB, 0.96995E+00_JPRB, 0.86115E+00_JPRB, 0.86553E+00_JPRB, 0.87794E+00_JPRB ,&
&  0.93278E+00_JPRB, 0.94786E+00_JPRB, 0.95206E+00_JPRB, 0.95276E+00_JPRB, 0.92349E+00_JPRB, 0.90698E+00_JPRB, 0.89864E+00_JPRB ,&
&  0.86252E+00_JPRB, 0.81618E+00_JPRB, 0.78289E+00_JPRB, 0.74458E+00_JPRB, 0.71660E+00_JPRB, 0.68824E+00_JPRB, 0.65863E+00_JPRB ,&
&  0.63550E+00_JPRB, 0.61948E+00_JPRB, 0.60666E+00_JPRB, 0.59310E+00_JPRB, 0.58831E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.57231E+00_JPRB, 0.58568E+00_JPRB, 0.60699E+00_JPRB, 0.64166E+00_JPRB, 0.75065E+00_JPRB, 0.79461E+00_JPRB, 0.79719E+00_JPRB ,&
&  0.77576E+00_JPRB, 0.53745E+00_JPRB, 0.51454E+00_JPRB, 0.53533E+00_JPRB, 0.57478E+00_JPRB, 0.62174E+00_JPRB, 0.68927E+00_JPRB ,&
&  0.75015E+00_JPRB, 0.84468E+00_JPRB, 0.99545E+00_JPRB, 0.11688E+01_JPRB, 0.13012E+01_JPRB, 0.13411E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.87579E+00_JPRB, 0.16383E+01_JPRB, 0.16528E+01_JPRB, 0.16601E+01_JPRB, 0.16066E+01_JPRB, 0.14746E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.10598E+01_JPRB, 0.96745E+00_JPRB, 0.88987E+00_JPRB, 0.83864E+00_JPRB, 0.78128E+00_JPRB, 0.71699E+00_JPRB, 0.69518E+00_JPRB ,&
&  0.70694E+00_JPRB, 0.74961E+00_JPRB, 0.80449E+00_JPRB, 0.86180E+00_JPRB, 0.93123E+00_JPRB, 0.99369E+00_JPRB, 0.10323E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10900E+01_JPRB, 0.10488E+01_JPRB, 0.10494E+01_JPRB, 0.10601E+01_JPRB, 0.10884E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.10230E+01_JPRB, 0.11349E+01_JPRB, 0.11325E+01_JPRB, 0.10609E+01_JPRB, 0.10729E+01_JPRB, 0.97512E+00_JPRB ,&
&  0.10313E+01_JPRB, 0.97077E+00_JPRB, 0.10547E+01_JPRB, 0.92185E+00_JPRB, 0.96860E+00_JPRB, 0.10733E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10054E+01_JPRB, 0.10790E+01_JPRB, 0.10546E+01_JPRB, 0.10529E+01_JPRB, 0.10482E+01_JPRB, 0.99744E+00_JPRB, 0.95455E+00_JPRB ,&
&  0.96385E+00_JPRB, 0.10164E+01_JPRB, 0.10898E+01_JPRB, 0.96566E+00_JPRB, 0.90267E+00_JPRB, 0.85433E+00_JPRB, 0.89813E+00_JPRB ,&
&  0.94508E+00_JPRB, 0.95567E+00_JPRB, 0.10108E+01_JPRB, 0.99562E+00_JPRB, 0.91107E+00_JPRB, 0.89841E+00_JPRB, 0.90019E+00_JPRB ,&
&  0.87671E+00_JPRB, 0.83508E+00_JPRB, 0.80678E+00_JPRB, 0.75662E+00_JPRB, 0.70033E+00_JPRB, 0.66803E+00_JPRB, 0.64425E+00_JPRB ,&
&  0.62412E+00_JPRB, 0.61033E+00_JPRB, 0.59459E+00_JPRB, 0.57783E+00_JPRB, 0.56893E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.56046E+00_JPRB, 0.57567E+00_JPRB, 0.59499E+00_JPRB, 0.62236E+00_JPRB, 0.68789E+00_JPRB, 0.70880E+00_JPRB, 0.65550E+00_JPRB ,&
&  0.58957E+00_JPRB, 0.48586E+00_JPRB, 0.47560E+00_JPRB, 0.49514E+00_JPRB, 0.52984E+00_JPRB, 0.57120E+00_JPRB, 0.63565E+00_JPRB ,&
&  0.71373E+00_JPRB, 0.78191E+00_JPRB, 0.93194E+00_JPRB, 0.10690E+01_JPRB, 0.12054E+01_JPRB, 0.13381E+01_JPRB, 0.91543E+00_JPRB ,&
&  0.96976E+00_JPRB, 0.16093E+01_JPRB, 0.16776E+01_JPRB, 0.17059E+01_JPRB, 0.17221E+01_JPRB, 0.17206E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.10943E+01_JPRB, 0.96276E+00_JPRB, 0.84889E+00_JPRB, 0.79584E+00_JPRB, 0.75002E+00_JPRB, 0.68745E+00_JPRB ,&
&  0.67193E+00_JPRB, 0.70008E+00_JPRB, 0.75545E+00_JPRB, 0.81785E+00_JPRB, 0.88995E+00_JPRB, 0.97163E+00_JPRB, 0.10227E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.10928E+01_JPRB, 0.10831E+01_JPRB, 0.10236E+01_JPRB, 0.10668E+01_JPRB, 0.11033E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10705E+01_JPRB, 0.12615E+01_JPRB, 0.13862E+01_JPRB, 0.13455E+01_JPRB, 0.12203E+01_JPRB, 0.10705E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.10036E+01_JPRB, 0.94167E+00_JPRB, 0.88140E+00_JPRB, 0.96888E+00_JPRB, 0.10742E+01_JPRB, 0.11373E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10746E+01_JPRB, 0.10294E+01_JPRB, 0.97986E+00_JPRB, 0.98151E+00_JPRB, 0.93684E+00_JPRB, 0.94632E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.10606E+01_JPRB, 0.10746E+01_JPRB, 0.95686E+00_JPRB, 0.91383E+00_JPRB, 0.79714E+00_JPRB, 0.84353E+00_JPRB ,&
&  0.92746E+00_JPRB, 0.92750E+00_JPRB, 0.94297E+00_JPRB, 0.10175E+01_JPRB, 0.95359E+00_JPRB, 0.93819E+00_JPRB, 0.94620E+00_JPRB ,&
&  0.88604E+00_JPRB, 0.82359E+00_JPRB, 0.79406E+00_JPRB, 0.76659E+00_JPRB, 0.71916E+00_JPRB, 0.67420E+00_JPRB, 0.64892E+00_JPRB ,&
&  0.62299E+00_JPRB, 0.60804E+00_JPRB, 0.58931E+00_JPRB, 0.57032E+00_JPRB, 0.55901E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.54804E+00_JPRB, 0.55566E+00_JPRB, 0.57472E+00_JPRB, 0.59093E+00_JPRB, 0.60704E+00_JPRB, 0.61162E+00_JPRB, 0.56098E+00_JPRB ,&
&  0.49655E+00_JPRB, 0.45658E+00_JPRB, 0.44219E+00_JPRB, 0.45005E+00_JPRB, 0.48503E+00_JPRB, 0.53320E+00_JPRB, 0.57723E+00_JPRB ,&
&  0.66708E+00_JPRB, 0.74884E+00_JPRB, 0.86199E+00_JPRB, 0.97856E+00_JPRB, 0.98004E+00_JPRB, 0.84989E+00_JPRB, 0.87254E+00_JPRB ,&
&  0.13044E+01_JPRB, 0.17580E+01_JPRB, 0.17137E+01_JPRB, 0.17170E+01_JPRB, 0.18021E+01_JPRB, 0.19174E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.13761E+01_JPRB, 0.12260E+01_JPRB, 0.10602E+01_JPRB, 0.94049E+00_JPRB, 0.84071E+00_JPRB, 0.79850E+00_JPRB, 0.73707E+00_JPRB ,&
&  0.67682E+00_JPRB, 0.68443E+00_JPRB, 0.74661E+00_JPRB, 0.81290E+00_JPRB, 0.88418E+00_JPRB, 0.95692E+00_JPRB, 0.10349E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10808E+01_JPRB, 0.10678E+01_JPRB, 0.10295E+01_JPRB, 0.10612E+01_JPRB, 0.11147E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.12968E+01_JPRB, 0.14195E+01_JPRB, 0.14957E+01_JPRB, 0.15467E+01_JPRB, 0.15013E+01_JPRB, 0.14340E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.10469E+01_JPRB, 0.86245E+00_JPRB, 0.87536E+00_JPRB, 0.10101E+01_JPRB, 0.11272E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.94986E+00_JPRB, 0.84216E+00_JPRB, 0.83373E+00_JPRB, 0.86786E+00_JPRB, 0.93264E+00_JPRB, 0.90689E+00_JPRB, 0.94635E+00_JPRB ,&
&  0.92818E+00_JPRB, 0.67787E+00_JPRB, 0.61537E+00_JPRB, 0.71705E+00_JPRB, 0.65795E+00_JPRB, 0.63647E+00_JPRB, 0.77746E+00_JPRB ,&
&  0.89298E+00_JPRB, 0.96362E+00_JPRB, 0.97539E+00_JPRB, 0.91738E+00_JPRB, 0.98151E+00_JPRB, 0.10137E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.93853E+00_JPRB, 0.84580E+00_JPRB, 0.79024E+00_JPRB, 0.75909E+00_JPRB, 0.72523E+00_JPRB, 0.68874E+00_JPRB, 0.66429E+00_JPRB ,&
&  0.63984E+00_JPRB, 0.61421E+00_JPRB, 0.59273E+00_JPRB, 0.56994E+00_JPRB, 0.55193E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.53580E+00_JPRB, 0.53445E+00_JPRB, 0.55205E+00_JPRB, 0.56508E+00_JPRB, 0.55697E+00_JPRB, 0.53473E+00_JPRB, 0.50297E+00_JPRB ,&
&  0.47071E+00_JPRB, 0.44595E+00_JPRB, 0.42835E+00_JPRB, 0.42891E+00_JPRB, 0.44784E+00_JPRB, 0.49655E+00_JPRB, 0.55207E+00_JPRB ,&
&  0.63119E+00_JPRB, 0.74668E+00_JPRB, 0.85327E+00_JPRB, 0.94108E+00_JPRB, 0.86522E+00_JPRB, 0.72304E+00_JPRB, 0.83979E+00_JPRB ,&
&  0.11475E+01_JPRB, 0.16128E+01_JPRB, 0.18507E+01_JPRB, 0.18670E+01_JPRB, 0.18586E+01_JPRB, 0.18676E+01_JPRB, 0.17511E+01_JPRB ,&
&  0.15443E+01_JPRB, 0.13237E+01_JPRB, 0.11440E+01_JPRB, 0.10194E+01_JPRB, 0.90278E+00_JPRB, 0.81469E+00_JPRB, 0.77711E+00_JPRB ,&
&  0.72125E+00_JPRB, 0.67709E+00_JPRB, 0.72013E+00_JPRB, 0.80275E+00_JPRB, 0.87074E+00_JPRB, 0.93915E+00_JPRB, 0.10124E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10672E+01_JPRB, 0.10717E+01_JPRB, 0.10814E+01_JPRB, 0.91182E+00_JPRB, 0.11001E+01_JPRB, 0.11822E+01_JPRB ,&
&  0.13317E+01_JPRB, 0.15033E+01_JPRB, 0.15740E+01_JPRB, 0.17772E+01_JPRB, 0.16752E+01_JPRB, 0.15420E+01_JPRB, 0.14117E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.11688E+01_JPRB, 0.97013E+00_JPRB, 0.97007E+00_JPRB, 0.10579E+01_JPRB, 0.11167E+01_JPRB, 0.80719E+00_JPRB ,&
&  0.76039E+00_JPRB, 0.96633E+00_JPRB, 0.87246E+00_JPRB, 0.76871E+00_JPRB, 0.87550E+00_JPRB, 0.88129E+00_JPRB, 0.69641E+00_JPRB ,&
&  0.44062E+00_JPRB, 0.38773E+00_JPRB, 0.39391E+00_JPRB, 0.40677E+00_JPRB, 0.40444E+00_JPRB, 0.46418E+00_JPRB, 0.81010E+00_JPRB ,&
&  0.86418E+00_JPRB, 0.97408E+00_JPRB, 0.96088E+00_JPRB, 0.91607E+00_JPRB, 0.95068E+00_JPRB, 0.10254E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.96591E+00_JPRB, 0.88964E+00_JPRB, 0.82132E+00_JPRB, 0.77965E+00_JPRB, 0.74423E+00_JPRB, 0.71205E+00_JPRB, 0.67841E+00_JPRB ,&
&  0.65379E+00_JPRB, 0.62469E+00_JPRB, 0.59726E+00_JPRB, 0.56895E+00_JPRB, 0.54556E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.51724E+00_JPRB, 0.50813E+00_JPRB, 0.51321E+00_JPRB, 0.51316E+00_JPRB, 0.51798E+00_JPRB, 0.48961E+00_JPRB, 0.48750E+00_JPRB ,&
&  0.45967E+00_JPRB, 0.44801E+00_JPRB, 0.42217E+00_JPRB, 0.41345E+00_JPRB, 0.41940E+00_JPRB, 0.46383E+00_JPRB, 0.53418E+00_JPRB ,&
&  0.61051E+00_JPRB, 0.73182E+00_JPRB, 0.94046E+00_JPRB, 0.90085E+00_JPRB, 0.88161E+00_JPRB, 0.79327E+00_JPRB, 0.75181E+00_JPRB ,&
&  0.96702E+00_JPRB, 0.13704E+01_JPRB, 0.16005E+01_JPRB, 0.17229E+01_JPRB, 0.19737E+01_JPRB, 0.20276E+01_JPRB, 0.19300E+01_JPRB ,&
&  0.17826E+01_JPRB, 0.15227E+01_JPRB, 0.13115E+01_JPRB, 0.11505E+01_JPRB, 0.10049E+01_JPRB, 0.87836E+00_JPRB, 0.79101E+00_JPRB ,&
&  0.76007E+00_JPRB, 0.70730E+00_JPRB, 0.70502E+00_JPRB, 0.77627E+00_JPRB, 0.85936E+00_JPRB, 0.90984E+00_JPRB, 0.96731E+00_JPRB ,&
&  0.10198E+01_JPRB, 0.10394E+01_JPRB, 0.10478E+01_JPRB, 0.10422E+01_JPRB, 0.90260E+00_JPRB, 0.83038E+00_JPRB, 0.10620E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.15831E+01_JPRB, 0.18118E+01_JPRB, 0.20235E+01_JPRB, 0.17399E+01_JPRB, 0.16039E+01_JPRB, 0.15677E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.14763E+01_JPRB, 0.10815E+01_JPRB, 0.10509E+01_JPRB, 0.11341E+01_JPRB, 0.93335E+00_JPRB, 0.91180E+00_JPRB ,&
&  0.10710E+01_JPRB, 0.10311E+01_JPRB, 0.85999E+00_JPRB, 0.69183E+00_JPRB, 0.82143E+00_JPRB, 0.73994E+00_JPRB, 0.43305E+00_JPRB ,&
&  0.37415E+00_JPRB, 0.36001E+00_JPRB, 0.34538E+00_JPRB, 0.33709E+00_JPRB, 0.33456E+00_JPRB, 0.37297E+00_JPRB, 0.57580E+00_JPRB ,&
&  0.72121E+00_JPRB, 0.93505E+00_JPRB, 0.97360E+00_JPRB, 0.94482E+00_JPRB, 0.95580E+00_JPRB, 0.10601E+01_JPRB, 0.12757E+01_JPRB ,&
&  0.10114E+01_JPRB, 0.92478E+00_JPRB, 0.83301E+00_JPRB, 0.78550E+00_JPRB, 0.75172E+00_JPRB, 0.72442E+00_JPRB, 0.69687E+00_JPRB ,&
&  0.66346E+00_JPRB, 0.63271E+00_JPRB, 0.60111E+00_JPRB, 0.56377E+00_JPRB, 0.53518E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.49814E+00_JPRB, 0.48261E+00_JPRB, 0.46470E+00_JPRB, 0.46850E+00_JPRB, 0.48944E+00_JPRB, 0.46891E+00_JPRB, 0.46668E+00_JPRB ,&
&  0.45509E+00_JPRB, 0.44512E+00_JPRB, 0.42870E+00_JPRB, 0.41411E+00_JPRB, 0.40966E+00_JPRB, 0.44335E+00_JPRB, 0.49955E+00_JPRB ,&
&  0.57828E+00_JPRB, 0.69228E+00_JPRB, 0.84408E+00_JPRB, 0.73349E+00_JPRB, 0.65342E+00_JPRB, 0.58990E+00_JPRB, 0.55234E+00_JPRB ,&
&  0.81530E+00_JPRB, 0.11014E+01_JPRB, 0.13403E+01_JPRB, 0.16137E+01_JPRB, 0.19509E+01_JPRB, 0.20044E+01_JPRB, 0.21620E+01_JPRB ,&
&  0.21102E+01_JPRB, 0.17894E+01_JPRB, 0.15167E+01_JPRB, 0.12886E+01_JPRB, 0.11210E+01_JPRB, 0.96197E+00_JPRB, 0.84037E+00_JPRB ,&
&  0.76658E+00_JPRB, 0.72694E+00_JPRB, 0.69433E+00_JPRB, 0.73574E+00_JPRB, 0.81603E+00_JPRB, 0.87495E+00_JPRB, 0.92812E+00_JPRB ,&
&  0.97248E+00_JPRB, 0.99896E+00_JPRB, 0.10199E+01_JPRB, 0.10484E+01_JPRB, 0.11037E+01_JPRB, 0.10578E+01_JPRB, 0.96871E+00_JPRB ,&
&  0.96339E+00_JPRB, 0.11447E+01_JPRB, 0.17740E+01_JPRB, 0.18510E+01_JPRB, 0.16874E+01_JPRB, 0.16854E+01_JPRB, 0.16579E+01_JPRB ,&
&  0.15727E+01_JPRB, 0.16376E+01_JPRB, 0.12631E+01_JPRB, 0.12522E+01_JPRB, 0.10975E+01_JPRB, 0.86106E+00_JPRB, 0.11420E+01_JPRB ,&
&  0.11954E+01_JPRB, 0.11534E+01_JPRB, 0.89532E+00_JPRB, 0.65341E+00_JPRB, 0.63205E+00_JPRB, 0.46497E+00_JPRB, 0.35742E+00_JPRB ,&
&  0.34481E+00_JPRB, 0.33476E+00_JPRB, 0.31370E+00_JPRB, 0.32746E+00_JPRB, 0.32892E+00_JPRB, 0.38472E+00_JPRB, 0.54433E+00_JPRB ,&
&  0.70985E+00_JPRB, 0.87638E+00_JPRB, 0.96356E+00_JPRB, 0.95165E+00_JPRB, 0.92766E+00_JPRB, 0.90682E+00_JPRB, 0.10916E+01_JPRB ,&
&  0.92063E+00_JPRB, 0.92735E+00_JPRB, 0.81528E+00_JPRB, 0.77682E+00_JPRB, 0.74056E+00_JPRB, 0.70997E+00_JPRB, 0.67392E+00_JPRB ,&
&  0.63802E+00_JPRB, 0.61125E+00_JPRB, 0.58462E+00_JPRB, 0.54891E+00_JPRB, 0.52052E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.47551E+00_JPRB, 0.45656E+00_JPRB, 0.43119E+00_JPRB, 0.43512E+00_JPRB, 0.46123E+00_JPRB, 0.45602E+00_JPRB, 0.44589E+00_JPRB ,&
&  0.44897E+00_JPRB, 0.43477E+00_JPRB, 0.42448E+00_JPRB, 0.41094E+00_JPRB, 0.40326E+00_JPRB, 0.42524E+00_JPRB, 0.47379E+00_JPRB ,&
&  0.56293E+00_JPRB, 0.70401E+00_JPRB, 0.61088E+00_JPRB, 0.54159E+00_JPRB, 0.54137E+00_JPRB, 0.54055E+00_JPRB, 0.47714E+00_JPRB ,&
&  0.69592E+00_JPRB, 0.88443E+00_JPRB, 0.10787E+01_JPRB, 0.14908E+01_JPRB, 0.22028E+01_JPRB, 0.23377E+01_JPRB, 0.24052E+01_JPRB ,&
&  0.23695E+01_JPRB, 0.21840E+01_JPRB, 0.18144E+01_JPRB, 0.14692E+01_JPRB, 0.12226E+01_JPRB, 0.10558E+01_JPRB, 0.89251E+00_JPRB ,&
&  0.77349E+00_JPRB, 0.72955E+00_JPRB, 0.71413E+00_JPRB, 0.71141E+00_JPRB, 0.75604E+00_JPRB, 0.80068E+00_JPRB, 0.83492E+00_JPRB ,&
&  0.88900E+00_JPRB, 0.93640E+00_JPRB, 0.95306E+00_JPRB, 0.95632E+00_JPRB, 0.93577E+00_JPRB, 0.84976E+00_JPRB, 0.11273E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.12970E+01_JPRB, 0.13663E+01_JPRB, 0.17455E+01_JPRB, 0.17708E+01_JPRB, 0.16242E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.13678E+01_JPRB, 0.11745E+01_JPRB, 0.11286E+01_JPRB, 0.10575E+01_JPRB, 0.85753E+00_JPRB, 0.97169E+00_JPRB, 0.14013E+01_JPRB ,&
&  0.11166E+01_JPRB, 0.12405E+01_JPRB, 0.12969E+01_JPRB, 0.12998E+01_JPRB, 0.61108E+00_JPRB, 0.42282E+00_JPRB, 0.64192E+00_JPRB ,&
&  0.70450E+00_JPRB, 0.49110E+00_JPRB, 0.38239E+00_JPRB, 0.47313E+00_JPRB, 0.38216E+00_JPRB, 0.43039E+00_JPRB, 0.59016E+00_JPRB ,&
&  0.66262E+00_JPRB, 0.73688E+00_JPRB, 0.93213E+00_JPRB, 0.94326E+00_JPRB, 0.89409E+00_JPRB, 0.85825E+00_JPRB, 0.94682E+00_JPRB ,&
&  0.92997E+00_JPRB, 0.93790E+00_JPRB, 0.87870E+00_JPRB, 0.77241E+00_JPRB, 0.70079E+00_JPRB, 0.66401E+00_JPRB, 0.63910E+00_JPRB ,&
&  0.61083E+00_JPRB, 0.58103E+00_JPRB, 0.55144E+00_JPRB, 0.52094E+00_JPRB, 0.49561E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.43598E+00_JPRB, 0.42428E+00_JPRB, 0.40271E+00_JPRB, 0.40684E+00_JPRB, 0.42913E+00_JPRB, 0.43777E+00_JPRB, 0.43345E+00_JPRB ,&
&  0.43127E+00_JPRB, 0.41944E+00_JPRB, 0.41384E+00_JPRB, 0.40454E+00_JPRB, 0.39841E+00_JPRB, 0.41358E+00_JPRB, 0.44706E+00_JPRB ,&
&  0.51282E+00_JPRB, 0.53970E+00_JPRB, 0.47752E+00_JPRB, 0.44534E+00_JPRB, 0.53737E+00_JPRB, 0.50244E+00_JPRB, 0.51080E+00_JPRB ,&
&  0.63006E+00_JPRB, 0.73845E+00_JPRB, 0.89481E+00_JPRB, 0.12199E+01_JPRB, 0.16092E+01_JPRB, 0.22003E+01_JPRB, 0.24868E+01_JPRB ,&
&  0.25087E+01_JPRB, 0.24492E+01_JPRB, 0.21428E+01_JPRB, 0.17614E+01_JPRB, 0.14022E+01_JPRB, 0.11753E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.85434E+00_JPRB, 0.74660E+00_JPRB, 0.70995E+00_JPRB, 0.71199E+00_JPRB, 0.73079E+00_JPRB, 0.74852E+00_JPRB, 0.74881E+00_JPRB ,&
&  0.77146E+00_JPRB, 0.82384E+00_JPRB, 0.84877E+00_JPRB, 0.84257E+00_JPRB, 0.77774E+00_JPRB, 0.67580E+00_JPRB, 0.79943E+00_JPRB ,&
&  0.11011E+01_JPRB, 0.11486E+01_JPRB, 0.13035E+01_JPRB, 0.16154E+01_JPRB, 0.18088E+01_JPRB, 0.16663E+01_JPRB, 0.21032E+01_JPRB ,&
&  0.17135E+01_JPRB, 0.13591E+01_JPRB, 0.10957E+01_JPRB, 0.77655E+00_JPRB, 0.96640E+00_JPRB, 0.14132E+01_JPRB, 0.14491E+01_JPRB ,&
&  0.13734E+01_JPRB, 0.13501E+01_JPRB, 0.13365E+01_JPRB, 0.11868E+01_JPRB, 0.96087E+00_JPRB, 0.70943E+00_JPRB, 0.82623E+00_JPRB ,&
&  0.84157E+00_JPRB, 0.81807E+00_JPRB, 0.75733E+00_JPRB, 0.65583E+00_JPRB, 0.51641E+00_JPRB, 0.58820E+00_JPRB, 0.62414E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.62806E+00_JPRB, 0.71611E+00_JPRB, 0.86792E+00_JPRB, 0.87939E+00_JPRB, 0.82030E+00_JPRB, 0.91286E+00_JPRB ,&
&  0.85912E+00_JPRB, 0.85134E+00_JPRB, 0.75817E+00_JPRB, 0.67594E+00_JPRB, 0.63343E+00_JPRB, 0.60808E+00_JPRB, 0.59121E+00_JPRB ,&
&  0.56141E+00_JPRB, 0.53632E+00_JPRB, 0.50999E+00_JPRB, 0.47606E+00_JPRB, 0.44939E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.40761E+00_JPRB, 0.39763E+00_JPRB, 0.37671E+00_JPRB, 0.37703E+00_JPRB, 0.39464E+00_JPRB, 0.41410E+00_JPRB, 0.42295E+00_JPRB ,&
&  0.41736E+00_JPRB, 0.41211E+00_JPRB, 0.40543E+00_JPRB, 0.40099E+00_JPRB, 0.39468E+00_JPRB, 0.39640E+00_JPRB, 0.42416E+00_JPRB ,&
&  0.48118E+00_JPRB, 0.47450E+00_JPRB, 0.40745E+00_JPRB, 0.44766E+00_JPRB, 0.49650E+00_JPRB, 0.46422E+00_JPRB, 0.52346E+00_JPRB ,&
&  0.58920E+00_JPRB, 0.67010E+00_JPRB, 0.75049E+00_JPRB, 0.93360E+00_JPRB, 0.11939E+01_JPRB, 0.15774E+01_JPRB, 0.19874E+01_JPRB ,&
&  0.21346E+01_JPRB, 0.22757E+01_JPRB, 0.23141E+01_JPRB, 0.21011E+01_JPRB, 0.17073E+01_JPRB, 0.13839E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.84245E+00_JPRB, 0.74587E+00_JPRB, 0.71329E+00_JPRB, 0.72159E+00_JPRB, 0.73772E+00_JPRB, 0.75557E+00_JPRB ,&
&  0.73148E+00_JPRB, 0.72712E+00_JPRB, 0.73710E+00_JPRB, 0.81177E+00_JPRB, 0.66266E+00_JPRB, 0.64942E+00_JPRB, 0.73100E+00_JPRB ,&
&  0.11990E+01_JPRB, 0.13578E+01_JPRB, 0.14520E+01_JPRB, 0.19263E+01_JPRB, 0.17217E+01_JPRB, 0.18306E+01_JPRB, 0.21819E+01_JPRB ,&
&  0.24279E+01_JPRB, 0.24329E+01_JPRB, 0.23524E+01_JPRB, 0.13829E+01_JPRB, 0.11783E+01_JPRB, 0.15922E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.13753E+01_JPRB, 0.13200E+01_JPRB, 0.14049E+01_JPRB, 0.11121E+01_JPRB, 0.91852E+00_JPRB, 0.66931E+00_JPRB ,&
&  0.62177E+00_JPRB, 0.55980E+00_JPRB, 0.64003E+00_JPRB, 0.59239E+00_JPRB, 0.57546E+00_JPRB, 0.57066E+00_JPRB, 0.55786E+00_JPRB ,&
&  0.58570E+00_JPRB, 0.58344E+00_JPRB, 0.60039E+00_JPRB, 0.73957E+00_JPRB, 0.83955E+00_JPRB, 0.76377E+00_JPRB, 0.85691E+00_JPRB ,&
&  0.83502E+00_JPRB, 0.75484E+00_JPRB, 0.67324E+00_JPRB, 0.62165E+00_JPRB, 0.59158E+00_JPRB, 0.56765E+00_JPRB, 0.54208E+00_JPRB ,&
&  0.51211E+00_JPRB, 0.48840E+00_JPRB, 0.46333E+00_JPRB, 0.43852E+00_JPRB, 0.41771E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.39932E+00_JPRB, 0.38980E+00_JPRB, 0.36270E+00_JPRB, 0.35864E+00_JPRB, 0.36771E+00_JPRB, 0.38563E+00_JPRB, 0.39974E+00_JPRB ,&
&  0.40366E+00_JPRB, 0.39925E+00_JPRB, 0.39428E+00_JPRB, 0.39304E+00_JPRB, 0.38649E+00_JPRB, 0.38679E+00_JPRB, 0.41531E+00_JPRB ,&
&  0.46651E+00_JPRB, 0.53379E+00_JPRB, 0.45624E+00_JPRB, 0.40623E+00_JPRB, 0.37571E+00_JPRB, 0.43534E+00_JPRB, 0.50524E+00_JPRB ,&
&  0.59761E+00_JPRB, 0.65174E+00_JPRB, 0.73395E+00_JPRB, 0.81726E+00_JPRB, 0.88792E+00_JPRB, 0.10999E+01_JPRB, 0.14731E+01_JPRB ,&
&  0.20614E+01_JPRB, 0.20452E+01_JPRB, 0.21913E+01_JPRB, 0.20599E+01_JPRB, 0.18416E+01_JPRB, 0.15931E+01_JPRB, 0.13682E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.97165E+00_JPRB, 0.82428E+00_JPRB, 0.73034E+00_JPRB, 0.70129E+00_JPRB, 0.69661E+00_JPRB, 0.72711E+00_JPRB ,&
&  0.73212E+00_JPRB, 0.71361E+00_JPRB, 0.68399E+00_JPRB, 0.69110E+00_JPRB, 0.74803E+00_JPRB, 0.79450E+00_JPRB, 0.95272E+00_JPRB ,&
&  0.11174E+01_JPRB, 0.11372E+01_JPRB, 0.15729E+01_JPRB, 0.19285E+01_JPRB, 0.23873E+01_JPRB, 0.25311E+01_JPRB, 0.24325E+01_JPRB ,&
&  0.24875E+01_JPRB, 0.23658E+01_JPRB, 0.22764E+01_JPRB, 0.17852E+01_JPRB, 0.19890E+01_JPRB, 0.18955E+01_JPRB, 0.16873E+01_JPRB ,&
&  0.15020E+01_JPRB, 0.13760E+01_JPRB, 0.13120E+01_JPRB, 0.13155E+01_JPRB, 0.13478E+01_JPRB, 0.13739E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.75206E+00_JPRB, 0.77995E+00_JPRB, 0.69018E+00_JPRB, 0.58193E+00_JPRB, 0.49629E+00_JPRB, 0.51486E+00_JPRB, 0.53509E+00_JPRB ,&
&  0.55160E+00_JPRB, 0.54888E+00_JPRB, 0.57393E+00_JPRB, 0.69010E+00_JPRB, 0.83115E+00_JPRB, 0.83027E+00_JPRB, 0.83866E+00_JPRB ,&
&  0.76981E+00_JPRB, 0.74944E+00_JPRB, 0.70310E+00_JPRB, 0.61988E+00_JPRB, 0.59370E+00_JPRB, 0.55550E+00_JPRB, 0.51515E+00_JPRB ,&
&  0.48028E+00_JPRB, 0.45892E+00_JPRB, 0.44018E+00_JPRB, 0.42263E+00_JPRB, 0.40491E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.38822E+00_JPRB, 0.38426E+00_JPRB, 0.36324E+00_JPRB, 0.35632E+00_JPRB, 0.36046E+00_JPRB, 0.37022E+00_JPRB, 0.38016E+00_JPRB ,&
&  0.38655E+00_JPRB, 0.38134E+00_JPRB, 0.37620E+00_JPRB, 0.37694E+00_JPRB, 0.37782E+00_JPRB, 0.38420E+00_JPRB, 0.42034E+00_JPRB ,&
&  0.50002E+00_JPRB, 0.66732E+00_JPRB, 0.53281E+00_JPRB, 0.49698E+00_JPRB, 0.43269E+00_JPRB, 0.46423E+00_JPRB, 0.52050E+00_JPRB ,&
&  0.64462E+00_JPRB, 0.65242E+00_JPRB, 0.64180E+00_JPRB, 0.67436E+00_JPRB, 0.72783E+00_JPRB, 0.84546E+00_JPRB, 0.10965E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.17159E+01_JPRB, 0.18305E+01_JPRB, 0.19787E+01_JPRB, 0.18011E+01_JPRB, 0.15028E+01_JPRB, 0.14365E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.10800E+01_JPRB, 0.91363E+00_JPRB, 0.80667E+00_JPRB, 0.74403E+00_JPRB, 0.70223E+00_JPRB, 0.69813E+00_JPRB ,&
&  0.70211E+00_JPRB, 0.69485E+00_JPRB, 0.67901E+00_JPRB, 0.66782E+00_JPRB, 0.72012E+00_JPRB, 0.88239E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.15390E+01_JPRB, 0.16385E+01_JPRB, 0.14451E+01_JPRB, 0.20273E+01_JPRB, 0.28318E+01_JPRB, 0.27437E+01_JPRB, 0.23232E+01_JPRB ,&
&  0.23735E+01_JPRB, 0.22590E+01_JPRB, 0.22368E+01_JPRB, 0.22507E+01_JPRB, 0.21783E+01_JPRB, 0.20257E+01_JPRB, 0.18248E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.14182E+01_JPRB, 0.13042E+01_JPRB, 0.13514E+01_JPRB, 0.12778E+01_JPRB, 0.11975E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.91323E+00_JPRB, 0.73247E+00_JPRB, 0.50935E+00_JPRB, 0.52738E+00_JPRB, 0.42947E+00_JPRB, 0.44265E+00_JPRB, 0.51894E+00_JPRB ,&
&  0.52569E+00_JPRB, 0.57619E+00_JPRB, 0.62885E+00_JPRB, 0.65542E+00_JPRB, 0.77736E+00_JPRB, 0.79130E+00_JPRB, 0.78627E+00_JPRB ,&
&  0.76945E+00_JPRB, 0.67339E+00_JPRB, 0.68705E+00_JPRB, 0.64287E+00_JPRB, 0.61595E+00_JPRB, 0.54750E+00_JPRB, 0.48271E+00_JPRB ,&
&  0.44150E+00_JPRB, 0.42145E+00_JPRB, 0.40925E+00_JPRB, 0.39861E+00_JPRB, 0.38922E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.35912E+00_JPRB, 0.35525E+00_JPRB, 0.34579E+00_JPRB, 0.34640E+00_JPRB, 0.34833E+00_JPRB, 0.35242E+00_JPRB, 0.36282E+00_JPRB ,&
&  0.37361E+00_JPRB, 0.37051E+00_JPRB, 0.36070E+00_JPRB, 0.35808E+00_JPRB, 0.36161E+00_JPRB, 0.37531E+00_JPRB, 0.43247E+00_JPRB ,&
&  0.56627E+00_JPRB, 0.62691E+00_JPRB, 0.52304E+00_JPRB, 0.43013E+00_JPRB, 0.49060E+00_JPRB, 0.47302E+00_JPRB, 0.51510E+00_JPRB ,&
&  0.49842E+00_JPRB, 0.55054E+00_JPRB, 0.53693E+00_JPRB, 0.52987E+00_JPRB, 0.54248E+00_JPRB, 0.62620E+00_JPRB, 0.77105E+00_JPRB ,&
&  0.92650E+00_JPRB, 0.11641E+01_JPRB, 0.14026E+01_JPRB, 0.14235E+01_JPRB, 0.14793E+01_JPRB, 0.13214E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.10283E+01_JPRB, 0.89349E+00_JPRB, 0.80620E+00_JPRB, 0.74865E+00_JPRB, 0.71021E+00_JPRB, 0.69713E+00_JPRB ,&
&  0.69989E+00_JPRB, 0.69431E+00_JPRB, 0.67272E+00_JPRB, 0.67441E+00_JPRB, 0.77577E+00_JPRB, 0.10643E+01_JPRB, 0.14681E+01_JPRB ,&
&  0.19106E+01_JPRB, 0.21780E+01_JPRB, 0.22925E+01_JPRB, 0.25522E+01_JPRB, 0.26288E+01_JPRB, 0.26027E+01_JPRB, 0.24607E+01_JPRB ,&
&  0.22532E+01_JPRB, 0.20445E+01_JPRB, 0.20153E+01_JPRB, 0.19815E+01_JPRB, 0.19438E+01_JPRB, 0.19234E+01_JPRB, 0.17997E+01_JPRB ,&
&  0.15586E+01_JPRB, 0.14068E+01_JPRB, 0.13177E+01_JPRB, 0.12108E+01_JPRB, 0.11480E+01_JPRB, 0.10539E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.63455E+00_JPRB, 0.49268E+00_JPRB, 0.56681E+00_JPRB, 0.48474E+00_JPRB, 0.50678E+00_JPRB, 0.60253E+00_JPRB ,&
&  0.61586E+00_JPRB, 0.63106E+00_JPRB, 0.70043E+00_JPRB, 0.71277E+00_JPRB, 0.72454E+00_JPRB, 0.78222E+00_JPRB, 0.73625E+00_JPRB ,&
&  0.68241E+00_JPRB, 0.63765E+00_JPRB, 0.65449E+00_JPRB, 0.62435E+00_JPRB, 0.57990E+00_JPRB, 0.49261E+00_JPRB, 0.43238E+00_JPRB ,&
&  0.40080E+00_JPRB, 0.38624E+00_JPRB, 0.37666E+00_JPRB, 0.36757E+00_JPRB, 0.35900E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.33952E+00_JPRB, 0.33397E+00_JPRB, 0.32880E+00_JPRB, 0.33402E+00_JPRB, 0.34226E+00_JPRB, 0.34524E+00_JPRB, 0.35532E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36529E+00_JPRB, 0.35816E+00_JPRB, 0.35156E+00_JPRB, 0.35181E+00_JPRB, 0.36872E+00_JPRB, 0.43187E+00_JPRB ,&
&  0.40751E+00_JPRB, 0.44199E+00_JPRB, 0.38672E+00_JPRB, 0.40304E+00_JPRB, 0.48719E+00_JPRB, 0.43493E+00_JPRB, 0.42331E+00_JPRB ,&
&  0.40825E+00_JPRB, 0.40142E+00_JPRB, 0.38338E+00_JPRB, 0.37973E+00_JPRB, 0.39220E+00_JPRB, 0.44260E+00_JPRB, 0.54631E+00_JPRB ,&
&  0.61687E+00_JPRB, 0.74843E+00_JPRB, 0.89154E+00_JPRB, 0.90500E+00_JPRB, 0.98964E+00_JPRB, 0.10477E+01_JPRB, 0.10859E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.91527E+00_JPRB, 0.80603E+00_JPRB, 0.74090E+00_JPRB, 0.69692E+00_JPRB, 0.67197E+00_JPRB, 0.65388E+00_JPRB ,&
&  0.63814E+00_JPRB, 0.63713E+00_JPRB, 0.66031E+00_JPRB, 0.70525E+00_JPRB, 0.78826E+00_JPRB, 0.12118E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.21747E+01_JPRB, 0.23068E+01_JPRB, 0.24114E+01_JPRB, 0.26191E+01_JPRB, 0.25748E+01_JPRB, 0.23785E+01_JPRB, 0.20725E+01_JPRB ,&
&  0.18647E+01_JPRB, 0.16881E+01_JPRB, 0.17388E+01_JPRB, 0.18149E+01_JPRB, 0.17320E+01_JPRB, 0.18351E+01_JPRB, 0.17576E+01_JPRB ,&
&  0.15746E+01_JPRB, 0.14562E+01_JPRB, 0.13588E+01_JPRB, 0.13378E+01_JPRB, 0.12729E+01_JPRB, 0.12940E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.10773E+01_JPRB, 0.79326E+00_JPRB, 0.72353E+00_JPRB, 0.56316E+00_JPRB, 0.58273E+00_JPRB, 0.73237E+00_JPRB ,&
&  0.74882E+00_JPRB, 0.75876E+00_JPRB, 0.73334E+00_JPRB, 0.71248E+00_JPRB, 0.67978E+00_JPRB, 0.71596E+00_JPRB, 0.69218E+00_JPRB ,&
&  0.61198E+00_JPRB, 0.61909E+00_JPRB, 0.60714E+00_JPRB, 0.55207E+00_JPRB, 0.49890E+00_JPRB, 0.42785E+00_JPRB, 0.38369E+00_JPRB ,&
&  0.37184E+00_JPRB, 0.35947E+00_JPRB, 0.35471E+00_JPRB, 0.34813E+00_JPRB, 0.34162E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.32781E+00_JPRB, 0.32358E+00_JPRB, 0.32012E+00_JPRB, 0.32338E+00_JPRB, 0.33509E+00_JPRB, 0.34049E+00_JPRB, 0.35073E+00_JPRB ,&
&  0.35708E+00_JPRB, 0.35634E+00_JPRB, 0.35726E+00_JPRB, 0.34920E+00_JPRB, 0.34865E+00_JPRB, 0.36196E+00_JPRB, 0.36481E+00_JPRB ,&
&  0.33349E+00_JPRB, 0.35967E+00_JPRB, 0.34950E+00_JPRB, 0.35115E+00_JPRB, 0.35334E+00_JPRB, 0.33577E+00_JPRB, 0.32144E+00_JPRB ,&
&  0.33028E+00_JPRB, 0.32333E+00_JPRB, 0.32454E+00_JPRB, 0.33571E+00_JPRB, 0.35354E+00_JPRB, 0.39089E+00_JPRB, 0.45663E+00_JPRB ,&
&  0.49459E+00_JPRB, 0.56473E+00_JPRB, 0.63271E+00_JPRB, 0.63795E+00_JPRB, 0.73635E+00_JPRB, 0.82649E+00_JPRB, 0.85265E+00_JPRB ,&
&  0.85455E+00_JPRB, 0.79946E+00_JPRB, 0.73097E+00_JPRB, 0.67572E+00_JPRB, 0.62556E+00_JPRB, 0.59754E+00_JPRB, 0.58513E+00_JPRB ,&
&  0.57635E+00_JPRB, 0.57500E+00_JPRB, 0.58896E+00_JPRB, 0.61911E+00_JPRB, 0.68648E+00_JPRB, 0.10002E+01_JPRB, 0.13631E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.16051E+01_JPRB, 0.16670E+01_JPRB, 0.16828E+01_JPRB, 0.16774E+01_JPRB, 0.15964E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.13829E+01_JPRB, 0.16933E+01_JPRB, 0.17180E+01_JPRB, 0.16735E+01_JPRB, 0.17045E+01_JPRB, 0.16483E+01_JPRB ,&
&  0.16204E+01_JPRB, 0.15119E+01_JPRB, 0.14431E+01_JPRB, 0.13855E+01_JPRB, 0.13297E+01_JPRB, 0.13583E+01_JPRB, 0.12823E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.12734E+01_JPRB, 0.11013E+01_JPRB, 0.95883E+00_JPRB, 0.80854E+00_JPRB, 0.84312E+00_JPRB, 0.79465E+00_JPRB ,&
&  0.74173E+00_JPRB, 0.62130E+00_JPRB, 0.64960E+00_JPRB, 0.65459E+00_JPRB, 0.65703E+00_JPRB, 0.63320E+00_JPRB, 0.60165E+00_JPRB ,&
&  0.59602E+00_JPRB, 0.60233E+00_JPRB, 0.52829E+00_JPRB, 0.46075E+00_JPRB, 0.41793E+00_JPRB, 0.37974E+00_JPRB, 0.33542E+00_JPRB ,&
&  0.32364E+00_JPRB, 0.33527E+00_JPRB, 0.33629E+00_JPRB, 0.33468E+00_JPRB, 0.33193E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.32181E+00_JPRB, 0.32150E+00_JPRB, 0.32070E+00_JPRB, 0.32038E+00_JPRB, 0.33050E+00_JPRB, 0.33590E+00_JPRB, 0.33581E+00_JPRB ,&
&  0.35042E+00_JPRB, 0.34757E+00_JPRB, 0.34733E+00_JPRB, 0.34402E+00_JPRB, 0.34218E+00_JPRB, 0.31676E+00_JPRB, 0.26774E+00_JPRB ,&
&  0.25766E+00_JPRB, 0.27155E+00_JPRB, 0.29451E+00_JPRB, 0.30365E+00_JPRB, 0.29864E+00_JPRB, 0.28838E+00_JPRB, 0.28057E+00_JPRB ,&
&  0.28161E+00_JPRB, 0.28671E+00_JPRB, 0.30215E+00_JPRB, 0.31821E+00_JPRB, 0.32790E+00_JPRB, 0.34159E+00_JPRB, 0.37373E+00_JPRB ,&
&  0.39882E+00_JPRB, 0.44141E+00_JPRB, 0.50208E+00_JPRB, 0.47467E+00_JPRB, 0.55768E+00_JPRB, 0.62436E+00_JPRB, 0.62587E+00_JPRB ,&
&  0.64602E+00_JPRB, 0.63129E+00_JPRB, 0.59846E+00_JPRB, 0.58074E+00_JPRB, 0.54884E+00_JPRB, 0.51866E+00_JPRB, 0.50258E+00_JPRB ,&
&  0.49555E+00_JPRB, 0.49598E+00_JPRB, 0.50866E+00_JPRB, 0.52650E+00_JPRB, 0.57448E+00_JPRB, 0.75023E+00_JPRB, 0.89086E+00_JPRB ,&
&  0.94815E+00_JPRB, 0.10247E+01_JPRB, 0.11386E+01_JPRB, 0.11113E+01_JPRB, 0.11504E+01_JPRB, 0.11838E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.11918E+01_JPRB, 0.12851E+01_JPRB, 0.13435E+01_JPRB, 0.14034E+01_JPRB, 0.14118E+01_JPRB, 0.14118E+01_JPRB ,&
&  0.14750E+01_JPRB, 0.15265E+01_JPRB, 0.13628E+01_JPRB, 0.12696E+01_JPRB, 0.11801E+01_JPRB, 0.11615E+01_JPRB, 0.11074E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.11438E+01_JPRB, 0.11258E+01_JPRB, 0.10874E+01_JPRB, 0.10106E+01_JPRB, 0.91914E+00_JPRB, 0.83281E+00_JPRB ,&
&  0.77986E+00_JPRB, 0.68953E+00_JPRB, 0.64178E+00_JPRB, 0.63954E+00_JPRB, 0.63380E+00_JPRB, 0.55867E+00_JPRB, 0.54924E+00_JPRB ,&
&  0.55063E+00_JPRB, 0.49647E+00_JPRB, 0.46417E+00_JPRB, 0.41408E+00_JPRB, 0.39050E+00_JPRB, 0.36746E+00_JPRB, 0.33654E+00_JPRB ,&
&  0.31355E+00_JPRB, 0.32788E+00_JPRB, 0.32664E+00_JPRB, 0.32191E+00_JPRB, 0.32170E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.32072E+00_JPRB, 0.32475E+00_JPRB, 0.32216E+00_JPRB, 0.31885E+00_JPRB, 0.32331E+00_JPRB, 0.31758E+00_JPRB, 0.31794E+00_JPRB ,&
&  0.33409E+00_JPRB, 0.32355E+00_JPRB, 0.33948E+00_JPRB, 0.28914E+00_JPRB, 0.26371E+00_JPRB, 0.25872E+00_JPRB, 0.25102E+00_JPRB ,&
&  0.25218E+00_JPRB, 0.26027E+00_JPRB, 0.26659E+00_JPRB, 0.27126E+00_JPRB, 0.27532E+00_JPRB, 0.26878E+00_JPRB, 0.26437E+00_JPRB ,&
&  0.26657E+00_JPRB, 0.27637E+00_JPRB, 0.29327E+00_JPRB, 0.29869E+00_JPRB, 0.29896E+00_JPRB, 0.30066E+00_JPRB, 0.31040E+00_JPRB ,&
&  0.31356E+00_JPRB, 0.33928E+00_JPRB, 0.39060E+00_JPRB, 0.40899E+00_JPRB, 0.44990E+00_JPRB, 0.50567E+00_JPRB, 0.50303E+00_JPRB ,&
&  0.49867E+00_JPRB, 0.43628E+00_JPRB, 0.45974E+00_JPRB, 0.46502E+00_JPRB, 0.46296E+00_JPRB, 0.44863E+00_JPRB, 0.43325E+00_JPRB ,&
&  0.42460E+00_JPRB, 0.42437E+00_JPRB, 0.43241E+00_JPRB, 0.44597E+00_JPRB, 0.46966E+00_JPRB, 0.52405E+00_JPRB, 0.57118E+00_JPRB ,&
&  0.63617E+00_JPRB, 0.57307E+00_JPRB, 0.69163E+00_JPRB, 0.73003E+00_JPRB, 0.82611E+00_JPRB, 0.91391E+00_JPRB, 0.98259E+00_JPRB ,&
&  0.11655E+01_JPRB, 0.11368E+01_JPRB, 0.11706E+01_JPRB, 0.11423E+01_JPRB, 0.11179E+01_JPRB, 0.11107E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.11823E+01_JPRB, 0.11147E+01_JPRB, 0.11242E+01_JPRB, 0.10358E+01_JPRB, 0.97564E+00_JPRB, 0.96465E+00_JPRB, 0.94994E+00_JPRB ,&
&  0.95323E+00_JPRB, 0.95650E+00_JPRB, 0.93941E+00_JPRB, 0.82715E+00_JPRB, 0.79564E+00_JPRB, 0.78112E+00_JPRB, 0.72410E+00_JPRB ,&
&  0.72958E+00_JPRB, 0.71259E+00_JPRB, 0.66758E+00_JPRB, 0.69179E+00_JPRB, 0.62457E+00_JPRB, 0.58033E+00_JPRB, 0.55931E+00_JPRB ,&
&  0.52926E+00_JPRB, 0.44242E+00_JPRB, 0.41755E+00_JPRB, 0.39799E+00_JPRB, 0.37312E+00_JPRB, 0.35070E+00_JPRB, 0.35614E+00_JPRB ,&
&  0.33405E+00_JPRB, 0.32851E+00_JPRB, 0.33348E+00_JPRB, 0.32559E+00_JPRB, 0.32024E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.32697E+00_JPRB, 0.32845E+00_JPRB, 0.32436E+00_JPRB, 0.31955E+00_JPRB, 0.31925E+00_JPRB, 0.31199E+00_JPRB, 0.29709E+00_JPRB ,&
&  0.29490E+00_JPRB, 0.28172E+00_JPRB, 0.26641E+00_JPRB, 0.25736E+00_JPRB, 0.25425E+00_JPRB, 0.25035E+00_JPRB, 0.24163E+00_JPRB ,&
&  0.23988E+00_JPRB, 0.24454E+00_JPRB, 0.25331E+00_JPRB, 0.25648E+00_JPRB, 0.25610E+00_JPRB, 0.25424E+00_JPRB, 0.25494E+00_JPRB ,&
&  0.26088E+00_JPRB, 0.27172E+00_JPRB, 0.28380E+00_JPRB, 0.28893E+00_JPRB, 0.28726E+00_JPRB, 0.28737E+00_JPRB, 0.29123E+00_JPRB ,&
&  0.29259E+00_JPRB, 0.29612E+00_JPRB, 0.31342E+00_JPRB, 0.34415E+00_JPRB, 0.39838E+00_JPRB, 0.44332E+00_JPRB, 0.43639E+00_JPRB ,&
&  0.31558E+00_JPRB, 0.30059E+00_JPRB, 0.36256E+00_JPRB, 0.37394E+00_JPRB, 0.38197E+00_JPRB, 0.37753E+00_JPRB, 0.37399E+00_JPRB ,&
&  0.37117E+00_JPRB, 0.36989E+00_JPRB, 0.36818E+00_JPRB, 0.37358E+00_JPRB, 0.38928E+00_JPRB, 0.41200E+00_JPRB, 0.42735E+00_JPRB ,&
&  0.46343E+00_JPRB, 0.44526E+00_JPRB, 0.45006E+00_JPRB, 0.48260E+00_JPRB, 0.63265E+00_JPRB, 0.72850E+00_JPRB, 0.81101E+00_JPRB ,&
&  0.90214E+00_JPRB, 0.97453E+00_JPRB, 0.10628E+01_JPRB, 0.10889E+01_JPRB, 0.10668E+01_JPRB, 0.10173E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10039E+01_JPRB, 0.97514E+00_JPRB, 0.93661E+00_JPRB, 0.87678E+00_JPRB, 0.85141E+00_JPRB, 0.83138E+00_JPRB ,&
&  0.82701E+00_JPRB, 0.81971E+00_JPRB, 0.77666E+00_JPRB, 0.72374E+00_JPRB, 0.68127E+00_JPRB, 0.67566E+00_JPRB, 0.67690E+00_JPRB ,&
&  0.67192E+00_JPRB, 0.67493E+00_JPRB, 0.66299E+00_JPRB, 0.64385E+00_JPRB, 0.60292E+00_JPRB, 0.55881E+00_JPRB, 0.53124E+00_JPRB ,&
&  0.47365E+00_JPRB, 0.41840E+00_JPRB, 0.36912E+00_JPRB, 0.36124E+00_JPRB, 0.36589E+00_JPRB, 0.35852E+00_JPRB, 0.33901E+00_JPRB ,&
&  0.33317E+00_JPRB, 0.32025E+00_JPRB, 0.31083E+00_JPRB, 0.31984E+00_JPRB, 0.32264E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.31032E+00_JPRB, 0.31125E+00_JPRB, 0.31036E+00_JPRB, 0.31051E+00_JPRB, 0.30453E+00_JPRB, 0.29632E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.28601E+00_JPRB, 0.28004E+00_JPRB, 0.27074E+00_JPRB, 0.25832E+00_JPRB, 0.24957E+00_JPRB, 0.24246E+00_JPRB, 0.24230E+00_JPRB ,&
&  0.24656E+00_JPRB, 0.25093E+00_JPRB, 0.25130E+00_JPRB, 0.24809E+00_JPRB, 0.24554E+00_JPRB, 0.24624E+00_JPRB, 0.25179E+00_JPRB ,&
&  0.26240E+00_JPRB, 0.27434E+00_JPRB, 0.28189E+00_JPRB, 0.28438E+00_JPRB, 0.28560E+00_JPRB, 0.28791E+00_JPRB, 0.28777E+00_JPRB ,&
&  0.28171E+00_JPRB, 0.28011E+00_JPRB, 0.28617E+00_JPRB, 0.30478E+00_JPRB, 0.34892E+00_JPRB, 0.38487E+00_JPRB, 0.33653E+00_JPRB ,&
&  0.26061E+00_JPRB, 0.23608E+00_JPRB, 0.24315E+00_JPRB, 0.29390E+00_JPRB, 0.30837E+00_JPRB, 0.31791E+00_JPRB, 0.32725E+00_JPRB ,&
&  0.32384E+00_JPRB, 0.31204E+00_JPRB, 0.31484E+00_JPRB, 0.33513E+00_JPRB, 0.34410E+00_JPRB, 0.35608E+00_JPRB, 0.36829E+00_JPRB ,&
&  0.38418E+00_JPRB, 0.40190E+00_JPRB, 0.41789E+00_JPRB, 0.40054E+00_JPRB, 0.47137E+00_JPRB, 0.63313E+00_JPRB, 0.69530E+00_JPRB ,&
&  0.78574E+00_JPRB, 0.87258E+00_JPRB, 0.10044E+01_JPRB, 0.10232E+01_JPRB, 0.97752E+00_JPRB, 0.94581E+00_JPRB, 0.94770E+00_JPRB ,&
&  0.91682E+00_JPRB, 0.89026E+00_JPRB, 0.85807E+00_JPRB, 0.84830E+00_JPRB, 0.81857E+00_JPRB, 0.78723E+00_JPRB, 0.75469E+00_JPRB ,&
&  0.74300E+00_JPRB, 0.72119E+00_JPRB, 0.67990E+00_JPRB, 0.62986E+00_JPRB, 0.60721E+00_JPRB, 0.59769E+00_JPRB, 0.60453E+00_JPRB ,&
&  0.60591E+00_JPRB, 0.61449E+00_JPRB, 0.61761E+00_JPRB, 0.62321E+00_JPRB, 0.56562E+00_JPRB, 0.49627E+00_JPRB, 0.44791E+00_JPRB ,&
&  0.42311E+00_JPRB, 0.39810E+00_JPRB, 0.37123E+00_JPRB, 0.37475E+00_JPRB, 0.38027E+00_JPRB, 0.37361E+00_JPRB, 0.33922E+00_JPRB ,&
&  0.30980E+00_JPRB, 0.29704E+00_JPRB, 0.29188E+00_JPRB, 0.30056E+00_JPRB, 0.30811E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.27609E+00_JPRB, 0.28342E+00_JPRB, 0.29292E+00_JPRB, 0.29709E+00_JPRB, 0.29785E+00_JPRB, 0.28043E+00_JPRB, 0.26317E+00_JPRB ,&
&  0.25328E+00_JPRB, 0.24981E+00_JPRB, 0.25092E+00_JPRB, 0.25190E+00_JPRB, 0.25264E+00_JPRB, 0.25407E+00_JPRB, 0.25537E+00_JPRB ,&
&  0.25391E+00_JPRB, 0.25081E+00_JPRB, 0.24805E+00_JPRB, 0.24649E+00_JPRB, 0.24753E+00_JPRB, 0.25250E+00_JPRB, 0.26189E+00_JPRB ,&
&  0.27469E+00_JPRB, 0.28402E+00_JPRB, 0.28703E+00_JPRB, 0.28512E+00_JPRB, 0.28378E+00_JPRB, 0.28500E+00_JPRB, 0.28316E+00_JPRB ,&
&  0.27610E+00_JPRB, 0.27444E+00_JPRB, 0.27839E+00_JPRB, 0.29120E+00_JPRB, 0.31552E+00_JPRB, 0.34427E+00_JPRB, 0.30104E+00_JPRB ,&
&  0.24295E+00_JPRB, 0.21024E+00_JPRB, 0.19864E+00_JPRB, 0.20636E+00_JPRB, 0.23126E+00_JPRB, 0.27560E+00_JPRB, 0.28681E+00_JPRB ,&
&  0.29849E+00_JPRB, 0.30936E+00_JPRB, 0.31727E+00_JPRB, 0.32490E+00_JPRB, 0.33089E+00_JPRB, 0.33431E+00_JPRB, 0.33595E+00_JPRB ,&
&  0.33959E+00_JPRB, 0.34827E+00_JPRB, 0.36121E+00_JPRB, 0.35039E+00_JPRB, 0.36158E+00_JPRB, 0.44373E+00_JPRB, 0.55028E+00_JPRB ,&
&  0.66173E+00_JPRB, 0.71344E+00_JPRB, 0.77730E+00_JPRB, 0.86629E+00_JPRB, 0.90508E+00_JPRB, 0.91809E+00_JPRB, 0.91045E+00_JPRB ,&
&  0.88668E+00_JPRB, 0.83419E+00_JPRB, 0.79973E+00_JPRB, 0.78029E+00_JPRB, 0.77601E+00_JPRB, 0.77694E+00_JPRB, 0.75141E+00_JPRB ,&
&  0.72979E+00_JPRB, 0.69103E+00_JPRB, 0.63608E+00_JPRB, 0.58743E+00_JPRB, 0.57737E+00_JPRB, 0.59225E+00_JPRB, 0.61692E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.61432E+00_JPRB, 0.61103E+00_JPRB, 0.61788E+00_JPRB, 0.55131E+00_JPRB, 0.50560E+00_JPRB, 0.47135E+00_JPRB ,&
&  0.45947E+00_JPRB, 0.45024E+00_JPRB, 0.43699E+00_JPRB, 0.44652E+00_JPRB, 0.43774E+00_JPRB, 0.42194E+00_JPRB, 0.39132E+00_JPRB ,&
&  0.35231E+00_JPRB, 0.32015E+00_JPRB, 0.29968E+00_JPRB, 0.28544E+00_JPRB, 0.27926E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.29278E+00_JPRB, 0.28717E+00_JPRB, 0.28561E+00_JPRB, 0.28688E+00_JPRB, 0.28829E+00_JPRB, 0.28145E+00_JPRB, 0.27467E+00_JPRB ,&
&  0.27021E+00_JPRB, 0.26981E+00_JPRB, 0.26883E+00_JPRB, 0.26638E+00_JPRB, 0.26453E+00_JPRB, 0.26274E+00_JPRB, 0.26102E+00_JPRB ,&
&  0.25928E+00_JPRB, 0.25812E+00_JPRB, 0.25775E+00_JPRB, 0.25692E+00_JPRB, 0.25933E+00_JPRB, 0.26687E+00_JPRB, 0.27981E+00_JPRB ,&
&  0.29337E+00_JPRB, 0.29865E+00_JPRB, 0.29777E+00_JPRB, 0.29387E+00_JPRB, 0.28590E+00_JPRB, 0.27945E+00_JPRB, 0.27432E+00_JPRB ,&
&  0.26686E+00_JPRB, 0.26651E+00_JPRB, 0.27510E+00_JPRB, 0.28471E+00_JPRB, 0.29967E+00_JPRB, 0.31154E+00_JPRB, 0.26973E+00_JPRB ,&
&  0.22406E+00_JPRB, 0.19460E+00_JPRB, 0.18046E+00_JPRB, 0.17749E+00_JPRB, 0.18539E+00_JPRB, 0.20368E+00_JPRB, 0.23343E+00_JPRB ,&
&  0.28523E+00_JPRB, 0.31732E+00_JPRB, 0.32972E+00_JPRB, 0.34306E+00_JPRB, 0.34544E+00_JPRB, 0.34332E+00_JPRB, 0.34111E+00_JPRB ,&
&  0.34530E+00_JPRB, 0.35413E+00_JPRB, 0.36710E+00_JPRB, 0.37541E+00_JPRB, 0.39083E+00_JPRB, 0.41249E+00_JPRB, 0.47057E+00_JPRB ,&
&  0.61475E+00_JPRB, 0.65444E+00_JPRB, 0.62226E+00_JPRB, 0.68934E+00_JPRB, 0.78623E+00_JPRB, 0.85495E+00_JPRB, 0.90010E+00_JPRB ,&
&  0.88580E+00_JPRB, 0.83544E+00_JPRB, 0.79701E+00_JPRB, 0.77618E+00_JPRB, 0.76648E+00_JPRB, 0.76244E+00_JPRB, 0.76949E+00_JPRB ,&
&  0.79520E+00_JPRB, 0.84588E+00_JPRB, 0.93318E+00_JPRB, 0.78804E+00_JPRB, 0.71907E+00_JPRB, 0.68856E+00_JPRB, 0.65458E+00_JPRB ,&
&  0.62853E+00_JPRB, 0.61466E+00_JPRB, 0.61408E+00_JPRB, 0.61673E+00_JPRB, 0.58104E+00_JPRB, 0.54847E+00_JPRB, 0.52182E+00_JPRB ,&
&  0.51225E+00_JPRB, 0.49891E+00_JPRB, 0.48686E+00_JPRB, 0.48465E+00_JPRB, 0.47460E+00_JPRB, 0.45414E+00_JPRB, 0.44080E+00_JPRB ,&
&  0.41552E+00_JPRB, 0.37613E+00_JPRB, 0.35063E+00_JPRB, 0.32908E+00_JPRB, 0.30771E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.29893E+00_JPRB, 0.29141E+00_JPRB, 0.28645E+00_JPRB, 0.28382E+00_JPRB, 0.28099E+00_JPRB, 0.27930E+00_JPRB, 0.27601E+00_JPRB ,&
&  0.27294E+00_JPRB, 0.27008E+00_JPRB, 0.26764E+00_JPRB, 0.26537E+00_JPRB, 0.26409E+00_JPRB, 0.26320E+00_JPRB, 0.26325E+00_JPRB ,&
&  0.26391E+00_JPRB, 0.26222E+00_JPRB, 0.26393E+00_JPRB, 0.26995E+00_JPRB, 0.27939E+00_JPRB, 0.28920E+00_JPRB, 0.30087E+00_JPRB ,&
&  0.30576E+00_JPRB, 0.30427E+00_JPRB, 0.29975E+00_JPRB, 0.29159E+00_JPRB, 0.28276E+00_JPRB, 0.27457E+00_JPRB, 0.27017E+00_JPRB ,&
&  0.27091E+00_JPRB, 0.26822E+00_JPRB, 0.27408E+00_JPRB, 0.27882E+00_JPRB, 0.27793E+00_JPRB, 0.28740E+00_JPRB, 0.23795E+00_JPRB ,&
&  0.20130E+00_JPRB, 0.17696E+00_JPRB, 0.16872E+00_JPRB, 0.16530E+00_JPRB, 0.16999E+00_JPRB, 0.18033E+00_JPRB, 0.20423E+00_JPRB ,&
&  0.24680E+00_JPRB, 0.32322E+00_JPRB, 0.33960E+00_JPRB, 0.34662E+00_JPRB, 0.35056E+00_JPRB, 0.34895E+00_JPRB, 0.34785E+00_JPRB ,&
&  0.35594E+00_JPRB, 0.36283E+00_JPRB, 0.37450E+00_JPRB, 0.38697E+00_JPRB, 0.40541E+00_JPRB, 0.42467E+00_JPRB, 0.44004E+00_JPRB ,&
&  0.47047E+00_JPRB, 0.49658E+00_JPRB, 0.52423E+00_JPRB, 0.58206E+00_JPRB, 0.68062E+00_JPRB, 0.78142E+00_JPRB, 0.81164E+00_JPRB ,&
&  0.79761E+00_JPRB, 0.79324E+00_JPRB, 0.76753E+00_JPRB, 0.75143E+00_JPRB, 0.74135E+00_JPRB, 0.73309E+00_JPRB, 0.73146E+00_JPRB ,&
&  0.74087E+00_JPRB, 0.77829E+00_JPRB, 0.84979E+00_JPRB, 0.78584E+00_JPRB, 0.74999E+00_JPRB, 0.71186E+00_JPRB, 0.67419E+00_JPRB ,&
&  0.64590E+00_JPRB, 0.63036E+00_JPRB, 0.62123E+00_JPRB, 0.60851E+00_JPRB, 0.59362E+00_JPRB, 0.57873E+00_JPRB, 0.56898E+00_JPRB ,&
&  0.55435E+00_JPRB, 0.53641E+00_JPRB, 0.51334E+00_JPRB, 0.49929E+00_JPRB, 0.48752E+00_JPRB, 0.47780E+00_JPRB, 0.46699E+00_JPRB ,&
&  0.43860E+00_JPRB, 0.39511E+00_JPRB, 0.36079E+00_JPRB, 0.33398E+00_JPRB, 0.31300E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.31915E+00_JPRB, 0.30785E+00_JPRB, 0.29857E+00_JPRB, 0.29302E+00_JPRB, 0.28740E+00_JPRB, 0.28466E+00_JPRB, 0.28126E+00_JPRB ,&
&  0.27955E+00_JPRB, 0.27788E+00_JPRB, 0.27711E+00_JPRB, 0.27676E+00_JPRB, 0.27732E+00_JPRB, 0.27856E+00_JPRB, 0.28090E+00_JPRB ,&
&  0.28443E+00_JPRB, 0.28924E+00_JPRB, 0.29582E+00_JPRB, 0.30110E+00_JPRB, 0.30659E+00_JPRB, 0.31085E+00_JPRB, 0.31584E+00_JPRB ,&
&  0.31668E+00_JPRB, 0.31662E+00_JPRB, 0.30712E+00_JPRB, 0.29513E+00_JPRB, 0.28439E+00_JPRB, 0.27289E+00_JPRB, 0.27237E+00_JPRB ,&
&  0.27134E+00_JPRB, 0.26873E+00_JPRB, 0.26381E+00_JPRB, 0.26467E+00_JPRB, 0.26487E+00_JPRB, 0.22630E+00_JPRB, 0.21455E+00_JPRB ,&
&  0.19481E+00_JPRB, 0.17584E+00_JPRB, 0.16865E+00_JPRB, 0.16439E+00_JPRB, 0.16552E+00_JPRB, 0.17311E+00_JPRB, 0.18700E+00_JPRB ,&
&  0.22009E+00_JPRB, 0.31447E+00_JPRB, 0.33549E+00_JPRB, 0.34326E+00_JPRB, 0.33422E+00_JPRB, 0.33616E+00_JPRB, 0.33865E+00_JPRB ,&
&  0.35060E+00_JPRB, 0.36168E+00_JPRB, 0.37683E+00_JPRB, 0.39039E+00_JPRB, 0.40542E+00_JPRB, 0.41316E+00_JPRB, 0.42723E+00_JPRB ,&
&  0.44993E+00_JPRB, 0.48302E+00_JPRB, 0.51363E+00_JPRB, 0.54575E+00_JPRB, 0.57883E+00_JPRB, 0.64323E+00_JPRB, 0.71396E+00_JPRB ,&
&  0.72167E+00_JPRB, 0.71567E+00_JPRB, 0.71533E+00_JPRB, 0.70993E+00_JPRB, 0.70740E+00_JPRB, 0.70468E+00_JPRB, 0.70836E+00_JPRB ,&
&  0.71738E+00_JPRB, 0.73056E+00_JPRB, 0.71505E+00_JPRB, 0.67523E+00_JPRB, 0.65404E+00_JPRB, 0.62619E+00_JPRB, 0.61779E+00_JPRB ,&
&  0.61010E+00_JPRB, 0.61864E+00_JPRB, 0.61095E+00_JPRB, 0.59745E+00_JPRB, 0.58515E+00_JPRB, 0.56922E+00_JPRB, 0.55668E+00_JPRB ,&
&  0.54272E+00_JPRB, 0.52644E+00_JPRB, 0.51016E+00_JPRB, 0.49839E+00_JPRB, 0.48910E+00_JPRB, 0.48199E+00_JPRB, 0.47084E+00_JPRB ,&
&  0.44647E+00_JPRB, 0.41250E+00_JPRB, 0.38354E+00_JPRB, 0.35568E+00_JPRB, 0.33423E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.35078E+00_JPRB, 0.33711E+00_JPRB, 0.32602E+00_JPRB, 0.31800E+00_JPRB, 0.31117E+00_JPRB, 0.30685E+00_JPRB, 0.30297E+00_JPRB ,&
&  0.30116E+00_JPRB, 0.29914E+00_JPRB, 0.29885E+00_JPRB, 0.29856E+00_JPRB, 0.29955E+00_JPRB, 0.30130E+00_JPRB, 0.30389E+00_JPRB ,&
&  0.30775E+00_JPRB, 0.31132E+00_JPRB, 0.31565E+00_JPRB, 0.31830E+00_JPRB, 0.32219E+00_JPRB, 0.32477E+00_JPRB, 0.32787E+00_JPRB ,&
&  0.32709E+00_JPRB, 0.32493E+00_JPRB, 0.31225E+00_JPRB, 0.30030E+00_JPRB, 0.28287E+00_JPRB, 0.27046E+00_JPRB, 0.26276E+00_JPRB ,&
&  0.26617E+00_JPRB, 0.24655E+00_JPRB, 0.24499E+00_JPRB, 0.23100E+00_JPRB, 0.21717E+00_JPRB, 0.20496E+00_JPRB, 0.19419E+00_JPRB ,&
&  0.18678E+00_JPRB, 0.17690E+00_JPRB, 0.17252E+00_JPRB, 0.17185E+00_JPRB, 0.17398E+00_JPRB, 0.18417E+00_JPRB, 0.19966E+00_JPRB ,&
&  0.23127E+00_JPRB, 0.31006E+00_JPRB, 0.33448E+00_JPRB, 0.34551E+00_JPRB, 0.34303E+00_JPRB, 0.34665E+00_JPRB, 0.34650E+00_JPRB ,&
&  0.36302E+00_JPRB, 0.38436E+00_JPRB, 0.40606E+00_JPRB, 0.42096E+00_JPRB, 0.43997E+00_JPRB, 0.45126E+00_JPRB, 0.46196E+00_JPRB ,&
&  0.47416E+00_JPRB, 0.49228E+00_JPRB, 0.50833E+00_JPRB, 0.52245E+00_JPRB, 0.54005E+00_JPRB, 0.55618E+00_JPRB, 0.57736E+00_JPRB ,&
&  0.59034E+00_JPRB, 0.61483E+00_JPRB, 0.62155E+00_JPRB, 0.62956E+00_JPRB, 0.62920E+00_JPRB, 0.62520E+00_JPRB, 0.62506E+00_JPRB ,&
&  0.62759E+00_JPRB, 0.62928E+00_JPRB, 0.62814E+00_JPRB, 0.61803E+00_JPRB, 0.61183E+00_JPRB, 0.59726E+00_JPRB, 0.59074E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.58521E+00_JPRB, 0.58025E+00_JPRB, 0.57262E+00_JPRB, 0.56255E+00_JPRB, 0.55060E+00_JPRB, 0.54287E+00_JPRB ,&
&  0.53282E+00_JPRB, 0.52159E+00_JPRB, 0.51009E+00_JPRB, 0.49986E+00_JPRB, 0.49055E+00_JPRB, 0.48025E+00_JPRB, 0.46293E+00_JPRB ,&
&  0.44038E+00_JPRB, 0.42049E+00_JPRB, 0.40139E+00_JPRB, 0.38484E+00_JPRB, 0.36714E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.38552E+00_JPRB, 0.37556E+00_JPRB, 0.36452E+00_JPRB, 0.35478E+00_JPRB, 0.34589E+00_JPRB, 0.33946E+00_JPRB, 0.33367E+00_JPRB ,&
&  0.32956E+00_JPRB, 0.32599E+00_JPRB, 0.32378E+00_JPRB, 0.32251E+00_JPRB, 0.32206E+00_JPRB, 0.32303E+00_JPRB, 0.32413E+00_JPRB ,&
&  0.32663E+00_JPRB, 0.32904E+00_JPRB, 0.33293E+00_JPRB, 0.33643E+00_JPRB, 0.34057E+00_JPRB, 0.34177E+00_JPRB, 0.34106E+00_JPRB ,&
&  0.33904E+00_JPRB, 0.33574E+00_JPRB, 0.31584E+00_JPRB, 0.30609E+00_JPRB, 0.28841E+00_JPRB, 0.27487E+00_JPRB, 0.26892E+00_JPRB ,&
&  0.26083E+00_JPRB, 0.25749E+00_JPRB, 0.24986E+00_JPRB, 0.24159E+00_JPRB, 0.23407E+00_JPRB, 0.22249E+00_JPRB, 0.21398E+00_JPRB ,&
&  0.20622E+00_JPRB, 0.19743E+00_JPRB, 0.19334E+00_JPRB, 0.19226E+00_JPRB, 0.19592E+00_JPRB, 0.20648E+00_JPRB, 0.22346E+00_JPRB ,&
&  0.25394E+00_JPRB, 0.28939E+00_JPRB, 0.33869E+00_JPRB, 0.35451E+00_JPRB, 0.35784E+00_JPRB, 0.37062E+00_JPRB, 0.38184E+00_JPRB ,&
&  0.38977E+00_JPRB, 0.40622E+00_JPRB, 0.42242E+00_JPRB, 0.43054E+00_JPRB, 0.43648E+00_JPRB, 0.44008E+00_JPRB, 0.44366E+00_JPRB ,&
&  0.44720E+00_JPRB, 0.45440E+00_JPRB, 0.45865E+00_JPRB, 0.46199E+00_JPRB, 0.46620E+00_JPRB, 0.47054E+00_JPRB, 0.47927E+00_JPRB ,&
&  0.48563E+00_JPRB, 0.49780E+00_JPRB, 0.50729E+00_JPRB, 0.52368E+00_JPRB, 0.52964E+00_JPRB, 0.53774E+00_JPRB, 0.53931E+00_JPRB ,&
&  0.54218E+00_JPRB, 0.54196E+00_JPRB, 0.53995E+00_JPRB, 0.53091E+00_JPRB, 0.52832E+00_JPRB, 0.53141E+00_JPRB, 0.53010E+00_JPRB ,&
&  0.52735E+00_JPRB, 0.52440E+00_JPRB, 0.52065E+00_JPRB, 0.51608E+00_JPRB, 0.51217E+00_JPRB, 0.51023E+00_JPRB, 0.50724E+00_JPRB ,&
&  0.50630E+00_JPRB, 0.50068E+00_JPRB, 0.49511E+00_JPRB, 0.48724E+00_JPRB, 0.47809E+00_JPRB, 0.46459E+00_JPRB, 0.44762E+00_JPRB ,&
&  0.43303E+00_JPRB, 0.42223E+00_JPRB, 0.41124E+00_JPRB, 0.40315E+00_JPRB, 0.39430E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.38775E+00_JPRB, 0.38469E+00_JPRB, 0.37940E+00_JPRB, 0.37267E+00_JPRB, 0.36662E+00_JPRB, 0.36230E+00_JPRB, 0.35789E+00_JPRB ,&
&  0.35444E+00_JPRB, 0.35197E+00_JPRB, 0.35045E+00_JPRB, 0.34963E+00_JPRB, 0.34894E+00_JPRB, 0.35021E+00_JPRB, 0.35172E+00_JPRB ,&
&  0.35326E+00_JPRB, 0.35415E+00_JPRB, 0.35601E+00_JPRB, 0.35696E+00_JPRB, 0.35714E+00_JPRB, 0.35570E+00_JPRB, 0.35357E+00_JPRB ,&
&  0.35023E+00_JPRB, 0.34597E+00_JPRB, 0.34102E+00_JPRB, 0.33525E+00_JPRB, 0.32959E+00_JPRB, 0.30248E+00_JPRB, 0.29503E+00_JPRB ,&
&  0.29173E+00_JPRB, 0.28907E+00_JPRB, 0.28854E+00_JPRB, 0.28993E+00_JPRB, 0.29747E+00_JPRB, 0.29443E+00_JPRB, 0.29052E+00_JPRB ,&
&  0.26926E+00_JPRB, 0.26289E+00_JPRB, 0.25856E+00_JPRB, 0.25566E+00_JPRB, 0.25786E+00_JPRB, 0.26851E+00_JPRB, 0.28438E+00_JPRB ,&
&  0.31454E+00_JPRB, 0.32855E+00_JPRB, 0.34116E+00_JPRB, 0.35501E+00_JPRB, 0.36699E+00_JPRB, 0.37689E+00_JPRB, 0.38615E+00_JPRB ,&
&  0.39420E+00_JPRB, 0.40203E+00_JPRB, 0.40801E+00_JPRB, 0.41146E+00_JPRB, 0.41257E+00_JPRB, 0.41339E+00_JPRB, 0.41392E+00_JPRB ,&
&  0.41408E+00_JPRB, 0.41371E+00_JPRB, 0.41379E+00_JPRB, 0.41406E+00_JPRB, 0.41347E+00_JPRB, 0.41442E+00_JPRB, 0.41751E+00_JPRB ,&
&  0.42161E+00_JPRB, 0.42745E+00_JPRB, 0.43206E+00_JPRB, 0.43976E+00_JPRB, 0.44735E+00_JPRB, 0.45775E+00_JPRB, 0.46097E+00_JPRB ,&
&  0.46581E+00_JPRB, 0.46884E+00_JPRB, 0.47356E+00_JPRB, 0.47337E+00_JPRB, 0.47338E+00_JPRB, 0.47251E+00_JPRB, 0.47286E+00_JPRB ,&
&  0.46955E+00_JPRB, 0.46697E+00_JPRB, 0.46361E+00_JPRB, 0.46262E+00_JPRB, 0.45835E+00_JPRB, 0.45671E+00_JPRB, 0.45364E+00_JPRB ,&
&  0.45367E+00_JPRB, 0.44785E+00_JPRB, 0.44463E+00_JPRB, 0.43768E+00_JPRB, 0.43186E+00_JPRB, 0.42144E+00_JPRB, 0.41243E+00_JPRB ,&
&  0.40460E+00_JPRB, 0.39940E+00_JPRB, 0.39544E+00_JPRB, 0.39247E+00_JPRB, 0.38981E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.38247E+00_JPRB, 0.38076E+00_JPRB, 0.37870E+00_JPRB, 0.37658E+00_JPRB, 0.37454E+00_JPRB, 0.37298E+00_JPRB, 0.37140E+00_JPRB ,&
&  0.36972E+00_JPRB, 0.36835E+00_JPRB, 0.36707E+00_JPRB, 0.36597E+00_JPRB, 0.36506E+00_JPRB, 0.36496E+00_JPRB, 0.36482E+00_JPRB ,&
&  0.36445E+00_JPRB, 0.36355E+00_JPRB, 0.36283E+00_JPRB, 0.36153E+00_JPRB, 0.35996E+00_JPRB, 0.35769E+00_JPRB, 0.35552E+00_JPRB ,&
&  0.35276E+00_JPRB, 0.34940E+00_JPRB, 0.34655E+00_JPRB, 0.34488E+00_JPRB, 0.34296E+00_JPRB, 0.34030E+00_JPRB, 0.33864E+00_JPRB ,&
&  0.33793E+00_JPRB, 0.33678E+00_JPRB, 0.33574E+00_JPRB, 0.33463E+00_JPRB, 0.33237E+00_JPRB, 0.33083E+00_JPRB, 0.32953E+00_JPRB ,&
&  0.32767E+00_JPRB, 0.32636E+00_JPRB, 0.32676E+00_JPRB, 0.32755E+00_JPRB, 0.32912E+00_JPRB, 0.33203E+00_JPRB, 0.33733E+00_JPRB ,&
&  0.34409E+00_JPRB, 0.35062E+00_JPRB, 0.35733E+00_JPRB, 0.36365E+00_JPRB, 0.36883E+00_JPRB, 0.37304E+00_JPRB, 0.37681E+00_JPRB ,&
&  0.37930E+00_JPRB, 0.38103E+00_JPRB, 0.38183E+00_JPRB, 0.38159E+00_JPRB, 0.38066E+00_JPRB, 0.38009E+00_JPRB, 0.37981E+00_JPRB ,&
&  0.37962E+00_JPRB, 0.37783E+00_JPRB, 0.37720E+00_JPRB, 0.37780E+00_JPRB, 0.37892E+00_JPRB, 0.38024E+00_JPRB, 0.38235E+00_JPRB ,&
&  0.38475E+00_JPRB, 0.38737E+00_JPRB, 0.38961E+00_JPRB, 0.39303E+00_JPRB, 0.39723E+00_JPRB, 0.40230E+00_JPRB, 0.40528E+00_JPRB ,&
&  0.40940E+00_JPRB, 0.41414E+00_JPRB, 0.41932E+00_JPRB, 0.42058E+00_JPRB, 0.42222E+00_JPRB, 0.42361E+00_JPRB, 0.42548E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.42321E+00_JPRB, 0.42281E+00_JPRB, 0.42372E+00_JPRB, 0.42111E+00_JPRB, 0.41978E+00_JPRB, 0.41828E+00_JPRB ,&
&  0.41863E+00_JPRB, 0.41419E+00_JPRB, 0.41141E+00_JPRB, 0.40759E+00_JPRB, 0.40506E+00_JPRB, 0.39980E+00_JPRB, 0.39559E+00_JPRB ,&
&  0.39186E+00_JPRB, 0.38973E+00_JPRB, 0.38739E+00_JPRB, 0.38553E+00_JPRB, 0.38405E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.37605E+00_JPRB, 0.37516E+00_JPRB, 0.37412E+00_JPRB, 0.37303E+00_JPRB, 0.37200E+00_JPRB, 0.37128E+00_JPRB, 0.37062E+00_JPRB ,&
&  0.36974E+00_JPRB, 0.36891E+00_JPRB, 0.36808E+00_JPRB, 0.36724E+00_JPRB, 0.36637E+00_JPRB, 0.36556E+00_JPRB, 0.36474E+00_JPRB ,&
&  0.36389E+00_JPRB, 0.36317E+00_JPRB, 0.36264E+00_JPRB, 0.36190E+00_JPRB, 0.36102E+00_JPRB, 0.35997E+00_JPRB, 0.35889E+00_JPRB ,&
&  0.35766E+00_JPRB, 0.35638E+00_JPRB, 0.35496E+00_JPRB, 0.35351E+00_JPRB, 0.35201E+00_JPRB, 0.35010E+00_JPRB, 0.34845E+00_JPRB ,&
&  0.34734E+00_JPRB, 0.34630E+00_JPRB, 0.34536E+00_JPRB, 0.34439E+00_JPRB, 0.34380E+00_JPRB, 0.34356E+00_JPRB, 0.34333E+00_JPRB ,&
&  0.34325E+00_JPRB, 0.34340E+00_JPRB, 0.34397E+00_JPRB, 0.34470E+00_JPRB, 0.34554E+00_JPRB, 0.34684E+00_JPRB, 0.34870E+00_JPRB ,&
&  0.35034E+00_JPRB, 0.35191E+00_JPRB, 0.35375E+00_JPRB, 0.35572E+00_JPRB, 0.35765E+00_JPRB, 0.35951E+00_JPRB, 0.36121E+00_JPRB ,&
&  0.36239E+00_JPRB, 0.36340E+00_JPRB, 0.36446E+00_JPRB, 0.36531E+00_JPRB, 0.36566E+00_JPRB, 0.36597E+00_JPRB, 0.36625E+00_JPRB ,&
&  0.36642E+00_JPRB, 0.36658E+00_JPRB, 0.36695E+00_JPRB, 0.36763E+00_JPRB, 0.36834E+00_JPRB, 0.36909E+00_JPRB, 0.37003E+00_JPRB ,&
&  0.37122E+00_JPRB, 0.37198E+00_JPRB, 0.37289E+00_JPRB, 0.37401E+00_JPRB, 0.37542E+00_JPRB, 0.37714E+00_JPRB, 0.37860E+00_JPRB ,&
&  0.38008E+00_JPRB, 0.38164E+00_JPRB, 0.38324E+00_JPRB, 0.38435E+00_JPRB, 0.38550E+00_JPRB, 0.38654E+00_JPRB, 0.38753E+00_JPRB ,&
&  0.38770E+00_JPRB, 0.38808E+00_JPRB, 0.38869E+00_JPRB, 0.38934E+00_JPRB, 0.38872E+00_JPRB, 0.38827E+00_JPRB, 0.38790E+00_JPRB ,&
&  0.38820E+00_JPRB, 0.38686E+00_JPRB, 0.38599E+00_JPRB, 0.38479E+00_JPRB, 0.38412E+00_JPRB, 0.38270E+00_JPRB, 0.38162E+00_JPRB ,&
&  0.38045E+00_JPRB, 0.37960E+00_JPRB, 0.37866E+00_JPRB, 0.37772E+00_JPRB, 0.37687E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB ,&
&  0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB, 0.36501E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB, 0.23307E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.25523E+00_JPRB, 0.25781E+00_JPRB, 0.26007E+00_JPRB, 0.26178E+00_JPRB, 0.26231E+00_JPRB, 0.26421E+00_JPRB, 0.26479E+00_JPRB ,&
&  0.26518E+00_JPRB, 0.26520E+00_JPRB, 0.26598E+00_JPRB, 0.26680E+00_JPRB, 0.26735E+00_JPRB, 0.26761E+00_JPRB, 0.26765E+00_JPRB ,&
&  0.26720E+00_JPRB, 0.26629E+00_JPRB, 0.26290E+00_JPRB, 0.26103E+00_JPRB, 0.25999E+00_JPRB, 0.25958E+00_JPRB, 0.25940E+00_JPRB ,&
&  0.25880E+00_JPRB, 0.25805E+00_JPRB, 0.25674E+00_JPRB, 0.25650E+00_JPRB, 0.25570E+00_JPRB, 0.25465E+00_JPRB, 0.25320E+00_JPRB ,&
&  0.25176E+00_JPRB, 0.25061E+00_JPRB, 0.24916E+00_JPRB, 0.24768E+00_JPRB, 0.24584E+00_JPRB, 0.24365E+00_JPRB, 0.24096E+00_JPRB ,&
&  0.23817E+00_JPRB, 0.23531E+00_JPRB, 0.23295E+00_JPRB, 0.23075E+00_JPRB, 0.22886E+00_JPRB, 0.22736E+00_JPRB, 0.22632E+00_JPRB ,&
&  0.22518E+00_JPRB, 0.22389E+00_JPRB, 0.22258E+00_JPRB, 0.22149E+00_JPRB, 0.22041E+00_JPRB, 0.21948E+00_JPRB, 0.21904E+00_JPRB ,&
&  0.21877E+00_JPRB, 0.21845E+00_JPRB, 0.21806E+00_JPRB, 0.21773E+00_JPRB, 0.21730E+00_JPRB, 0.21689E+00_JPRB, 0.21648E+00_JPRB ,&
&  0.21615E+00_JPRB, 0.21593E+00_JPRB, 0.21559E+00_JPRB, 0.21523E+00_JPRB, 0.21500E+00_JPRB, 0.21463E+00_JPRB, 0.21420E+00_JPRB ,&
&  0.21370E+00_JPRB, 0.21338E+00_JPRB, 0.21303E+00_JPRB, 0.21261E+00_JPRB, 0.21222E+00_JPRB, 0.21195E+00_JPRB, 0.21194E+00_JPRB ,&
&  0.21189E+00_JPRB, 0.21178E+00_JPRB, 0.21182E+00_JPRB, 0.21250E+00_JPRB, 0.21322E+00_JPRB, 0.21409E+00_JPRB, 0.21507E+00_JPRB ,&
&  0.21636E+00_JPRB, 0.21771E+00_JPRB, 0.21920E+00_JPRB, 0.22063E+00_JPRB, 0.22252E+00_JPRB, 0.22439E+00_JPRB, 0.22626E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22987E+00_JPRB, 0.23169E+00_JPRB, 0.23351E+00_JPRB, 0.23586E+00_JPRB, 0.23796E+00_JPRB, 0.24033E+00_JPRB ,&
&  0.24258E+00_JPRB, 0.24503E+00_JPRB, 0.24758E+00_JPRB, 0.24981E+00_JPRB, 0.25214E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.29783E+00_JPRB, 0.30831E+00_JPRB, 0.31730E+00_JPRB, 0.32609E+00_JPRB, 0.33027E+00_JPRB, 0.39503E+00_JPRB, 0.37578E+00_JPRB ,&
&  0.36645E+00_JPRB, 0.33802E+00_JPRB, 0.33869E+00_JPRB, 0.33410E+00_JPRB, 0.32743E+00_JPRB, 0.31970E+00_JPRB, 0.31315E+00_JPRB ,&
&  0.30671E+00_JPRB, 0.29886E+00_JPRB, 0.29374E+00_JPRB, 0.28762E+00_JPRB, 0.28183E+00_JPRB, 0.27612E+00_JPRB, 0.27276E+00_JPRB ,&
&  0.27025E+00_JPRB, 0.26878E+00_JPRB, 0.26768E+00_JPRB, 0.26712E+00_JPRB, 0.26984E+00_JPRB, 0.27134E+00_JPRB, 0.26972E+00_JPRB ,&
&  0.26635E+00_JPRB, 0.26312E+00_JPRB, 0.25898E+00_JPRB, 0.25472E+00_JPRB, 0.25150E+00_JPRB, 0.24867E+00_JPRB, 0.24576E+00_JPRB ,&
&  0.24197E+00_JPRB, 0.23775E+00_JPRB, 0.23354E+00_JPRB, 0.23027E+00_JPRB, 0.22723E+00_JPRB, 0.22418E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.21847E+00_JPRB, 0.21562E+00_JPRB, 0.21276E+00_JPRB, 0.21086E+00_JPRB, 0.20928E+00_JPRB, 0.20779E+00_JPRB, 0.20644E+00_JPRB ,&
&  0.20583E+00_JPRB, 0.20512E+00_JPRB, 0.20424E+00_JPRB, 0.20315E+00_JPRB, 0.20209E+00_JPRB, 0.20094E+00_JPRB, 0.19999E+00_JPRB ,&
&  0.19981E+00_JPRB, 0.19923E+00_JPRB, 0.19862E+00_JPRB, 0.19785E+00_JPRB, 0.19746E+00_JPRB, 0.19671E+00_JPRB, 0.19577E+00_JPRB ,&
&  0.19490E+00_JPRB, 0.19411E+00_JPRB, 0.19390E+00_JPRB, 0.19362E+00_JPRB, 0.19332E+00_JPRB, 0.19311E+00_JPRB, 0.19347E+00_JPRB ,&
&  0.19392E+00_JPRB, 0.19447E+00_JPRB, 0.19539E+00_JPRB, 0.19653E+00_JPRB, 0.19752E+00_JPRB, 0.19850E+00_JPRB, 0.19986E+00_JPRB ,&
&  0.20174E+00_JPRB, 0.20361E+00_JPRB, 0.20569E+00_JPRB, 0.20768E+00_JPRB, 0.21096E+00_JPRB, 0.21449E+00_JPRB, 0.21835E+00_JPRB ,&
&  0.22313E+00_JPRB, 0.22962E+00_JPRB, 0.23696E+00_JPRB, 0.24350E+00_JPRB, 0.24947E+00_JPRB, 0.25646E+00_JPRB, 0.26221E+00_JPRB ,&
&  0.26729E+00_JPRB, 0.27225E+00_JPRB, 0.27816E+00_JPRB, 0.28447E+00_JPRB, 0.29135E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.42458E+00_JPRB, 0.42539E+00_JPRB, 0.42570E+00_JPRB, 0.42211E+00_JPRB, 0.42062E+00_JPRB, 0.42054E+00_JPRB, 0.41775E+00_JPRB ,&
&  0.40988E+00_JPRB, 0.40732E+00_JPRB, 0.40249E+00_JPRB, 0.39562E+00_JPRB, 0.38893E+00_JPRB, 0.37818E+00_JPRB, 0.36197E+00_JPRB ,&
&  0.34021E+00_JPRB, 0.32113E+00_JPRB, 0.30661E+00_JPRB, 0.29340E+00_JPRB, 0.28043E+00_JPRB, 0.26779E+00_JPRB, 0.26242E+00_JPRB ,&
&  0.25788E+00_JPRB, 0.25588E+00_JPRB, 0.25516E+00_JPRB, 0.25876E+00_JPRB, 0.26639E+00_JPRB, 0.27595E+00_JPRB, 0.28633E+00_JPRB ,&
&  0.29835E+00_JPRB, 0.30952E+00_JPRB, 0.31104E+00_JPRB, 0.30410E+00_JPRB, 0.29721E+00_JPRB, 0.29286E+00_JPRB, 0.28951E+00_JPRB ,&
&  0.28561E+00_JPRB, 0.27721E+00_JPRB, 0.26780E+00_JPRB, 0.25800E+00_JPRB, 0.24813E+00_JPRB, 0.24095E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.22636E+00_JPRB, 0.22086E+00_JPRB, 0.21512E+00_JPRB, 0.21019E+00_JPRB, 0.20683E+00_JPRB, 0.20401E+00_JPRB, 0.20120E+00_JPRB ,&
&  0.19969E+00_JPRB, 0.19820E+00_JPRB, 0.19651E+00_JPRB, 0.19483E+00_JPRB, 0.19279E+00_JPRB, 0.19118E+00_JPRB, 0.18971E+00_JPRB ,&
&  0.18857E+00_JPRB, 0.18759E+00_JPRB, 0.18699E+00_JPRB, 0.18638E+00_JPRB, 0.18591E+00_JPRB, 0.18501E+00_JPRB, 0.18394E+00_JPRB ,&
&  0.18320E+00_JPRB, 0.18279E+00_JPRB, 0.18202E+00_JPRB, 0.18155E+00_JPRB, 0.18119E+00_JPRB, 0.18048E+00_JPRB, 0.18087E+00_JPRB ,&
&  0.18190E+00_JPRB, 0.18307E+00_JPRB, 0.18342E+00_JPRB, 0.18449E+00_JPRB, 0.18578E+00_JPRB, 0.18739E+00_JPRB, 0.18984E+00_JPRB ,&
&  0.19175E+00_JPRB, 0.19373E+00_JPRB, 0.19618E+00_JPRB, 0.19838E+00_JPRB, 0.20170E+00_JPRB, 0.20579E+00_JPRB, 0.21081E+00_JPRB ,&
&  0.21767E+00_JPRB, 0.22592E+00_JPRB, 0.23728E+00_JPRB, 0.25105E+00_JPRB, 0.27050E+00_JPRB, 0.27896E+00_JPRB, 0.28745E+00_JPRB ,&
&  0.29494E+00_JPRB, 0.30985E+00_JPRB, 0.39195E+00_JPRB, 0.40703E+00_JPRB, 0.42184E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.42359E+00_JPRB, 0.43572E+00_JPRB, 0.45328E+00_JPRB, 0.46409E+00_JPRB, 0.47855E+00_JPRB, 0.48143E+00_JPRB, 0.47654E+00_JPRB ,&
&  0.45740E+00_JPRB, 0.42717E+00_JPRB, 0.41047E+00_JPRB, 0.39019E+00_JPRB, 0.36895E+00_JPRB, 0.34773E+00_JPRB, 0.33837E+00_JPRB ,&
&  0.32714E+00_JPRB, 0.31471E+00_JPRB, 0.30185E+00_JPRB, 0.29088E+00_JPRB, 0.28036E+00_JPRB, 0.27053E+00_JPRB, 0.26470E+00_JPRB ,&
&  0.25873E+00_JPRB, 0.25799E+00_JPRB, 0.25730E+00_JPRB, 0.26147E+00_JPRB, 0.27252E+00_JPRB, 0.29159E+00_JPRB, 0.31746E+00_JPRB ,&
&  0.35657E+00_JPRB, 0.36321E+00_JPRB, 0.34836E+00_JPRB, 0.34199E+00_JPRB, 0.33472E+00_JPRB, 0.32830E+00_JPRB, 0.31384E+00_JPRB ,&
&  0.32105E+00_JPRB, 0.33437E+00_JPRB, 0.35023E+00_JPRB, 0.34034E+00_JPRB, 0.29535E+00_JPRB, 0.27480E+00_JPRB, 0.25133E+00_JPRB ,&
&  0.23853E+00_JPRB, 0.22826E+00_JPRB, 0.22033E+00_JPRB, 0.21300E+00_JPRB, 0.20589E+00_JPRB, 0.20138E+00_JPRB, 0.19642E+00_JPRB ,&
&  0.19270E+00_JPRB, 0.19097E+00_JPRB, 0.18853E+00_JPRB, 0.18825E+00_JPRB, 0.18791E+00_JPRB, 0.18705E+00_JPRB, 0.18612E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18324E+00_JPRB, 0.18247E+00_JPRB, 0.18289E+00_JPRB, 0.18377E+00_JPRB, 0.18464E+00_JPRB, 0.18295E+00_JPRB ,&
&  0.18167E+00_JPRB, 0.18039E+00_JPRB, 0.18020E+00_JPRB, 0.18113E+00_JPRB, 0.18140E+00_JPRB, 0.18202E+00_JPRB, 0.18205E+00_JPRB ,&
&  0.18207E+00_JPRB, 0.18253E+00_JPRB, 0.18223E+00_JPRB, 0.18286E+00_JPRB, 0.18317E+00_JPRB, 0.18425E+00_JPRB, 0.18510E+00_JPRB ,&
&  0.18601E+00_JPRB, 0.18791E+00_JPRB, 0.18993E+00_JPRB, 0.19273E+00_JPRB, 0.19656E+00_JPRB, 0.19932E+00_JPRB, 0.20366E+00_JPRB ,&
&  0.20904E+00_JPRB, 0.21757E+00_JPRB, 0.23312E+00_JPRB, 0.25058E+00_JPRB, 0.27199E+00_JPRB, 0.28377E+00_JPRB, 0.29337E+00_JPRB ,&
&  0.30805E+00_JPRB, 0.34311E+00_JPRB, 0.39353E+00_JPRB, 0.41278E+00_JPRB, 0.41694E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.48057E+00_JPRB, 0.48807E+00_JPRB, 0.49947E+00_JPRB, 0.50420E+00_JPRB, 0.51248E+00_JPRB, 0.50086E+00_JPRB, 0.49405E+00_JPRB ,&
&  0.46392E+00_JPRB, 0.44119E+00_JPRB, 0.40995E+00_JPRB, 0.37038E+00_JPRB, 0.34396E+00_JPRB, 0.32344E+00_JPRB, 0.31665E+00_JPRB ,&
&  0.31003E+00_JPRB, 0.30569E+00_JPRB, 0.30179E+00_JPRB, 0.30033E+00_JPRB, 0.29976E+00_JPRB, 0.30114E+00_JPRB, 0.30124E+00_JPRB ,&
&  0.29787E+00_JPRB, 0.29226E+00_JPRB, 0.28573E+00_JPRB, 0.28893E+00_JPRB, 0.30186E+00_JPRB, 0.31698E+00_JPRB, 0.33460E+00_JPRB ,&
&  0.35483E+00_JPRB, 0.36590E+00_JPRB, 0.37612E+00_JPRB, 0.38035E+00_JPRB, 0.36508E+00_JPRB, 0.36764E+00_JPRB, 0.37006E+00_JPRB ,&
&  0.37814E+00_JPRB, 0.38561E+00_JPRB, 0.38304E+00_JPRB, 0.37937E+00_JPRB, 0.32907E+00_JPRB, 0.29564E+00_JPRB, 0.26711E+00_JPRB ,&
&  0.24566E+00_JPRB, 0.23183E+00_JPRB, 0.22077E+00_JPRB, 0.21146E+00_JPRB, 0.20239E+00_JPRB, 0.19646E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.18829E+00_JPRB, 0.18653E+00_JPRB, 0.18480E+00_JPRB, 0.18414E+00_JPRB, 0.18423E+00_JPRB, 0.18352E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18276E+00_JPRB, 0.18183E+00_JPRB, 0.18155E+00_JPRB, 0.18238E+00_JPRB, 0.18411E+00_JPRB, 0.18707E+00_JPRB, 0.18824E+00_JPRB ,&
&  0.18904E+00_JPRB, 0.18946E+00_JPRB, 0.18946E+00_JPRB, 0.19115E+00_JPRB, 0.19202E+00_JPRB, 0.19218E+00_JPRB, 0.19057E+00_JPRB ,&
&  0.18893E+00_JPRB, 0.18771E+00_JPRB, 0.18741E+00_JPRB, 0.18774E+00_JPRB, 0.18891E+00_JPRB, 0.19086E+00_JPRB, 0.19135E+00_JPRB ,&
&  0.19216E+00_JPRB, 0.19187E+00_JPRB, 0.19166E+00_JPRB, 0.19321E+00_JPRB, 0.19505E+00_JPRB, 0.19711E+00_JPRB, 0.19992E+00_JPRB ,&
&  0.20177E+00_JPRB, 0.20846E+00_JPRB, 0.22060E+00_JPRB, 0.23901E+00_JPRB, 0.26633E+00_JPRB, 0.28259E+00_JPRB, 0.32181E+00_JPRB ,&
&  0.38703E+00_JPRB, 0.53307E+00_JPRB, 0.59712E+00_JPRB, 0.45135E+00_JPRB, 0.46542E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.50858E+00_JPRB, 0.52006E+00_JPRB, 0.53280E+00_JPRB, 0.52489E+00_JPRB, 0.51117E+00_JPRB, 0.50348E+00_JPRB, 0.50312E+00_JPRB ,&
&  0.50255E+00_JPRB, 0.49974E+00_JPRB, 0.48747E+00_JPRB, 0.47548E+00_JPRB, 0.45775E+00_JPRB, 0.40926E+00_JPRB, 0.39047E+00_JPRB ,&
&  0.37288E+00_JPRB, 0.37530E+00_JPRB, 0.38022E+00_JPRB, 0.38718E+00_JPRB, 0.40175E+00_JPRB, 0.41296E+00_JPRB, 0.41551E+00_JPRB ,&
&  0.40603E+00_JPRB, 0.38791E+00_JPRB, 0.37070E+00_JPRB, 0.35711E+00_JPRB, 0.35679E+00_JPRB, 0.35720E+00_JPRB, 0.35060E+00_JPRB ,&
&  0.33705E+00_JPRB, 0.33078E+00_JPRB, 0.33030E+00_JPRB, 0.35043E+00_JPRB, 0.40395E+00_JPRB, 0.42433E+00_JPRB, 0.39860E+00_JPRB ,&
&  0.40288E+00_JPRB, 0.39829E+00_JPRB, 0.39444E+00_JPRB, 0.39606E+00_JPRB, 0.38636E+00_JPRB, 0.36859E+00_JPRB, 0.33493E+00_JPRB ,&
&  0.29034E+00_JPRB, 0.26717E+00_JPRB, 0.24623E+00_JPRB, 0.22707E+00_JPRB, 0.21283E+00_JPRB, 0.20443E+00_JPRB, 0.19722E+00_JPRB ,&
&  0.19249E+00_JPRB, 0.18856E+00_JPRB, 0.18590E+00_JPRB, 0.18550E+00_JPRB, 0.18653E+00_JPRB, 0.18693E+00_JPRB, 0.18753E+00_JPRB ,&
&  0.18857E+00_JPRB, 0.18901E+00_JPRB, 0.18917E+00_JPRB, 0.19009E+00_JPRB, 0.19129E+00_JPRB, 0.19359E+00_JPRB, 0.19698E+00_JPRB ,&
&  0.20117E+00_JPRB, 0.20489E+00_JPRB, 0.20853E+00_JPRB, 0.21421E+00_JPRB, 0.21468E+00_JPRB, 0.21358E+00_JPRB, 0.20599E+00_JPRB ,&
&  0.20054E+00_JPRB, 0.19803E+00_JPRB, 0.19548E+00_JPRB, 0.19472E+00_JPRB, 0.19564E+00_JPRB, 0.19731E+00_JPRB, 0.19896E+00_JPRB ,&
&  0.20009E+00_JPRB, 0.20011E+00_JPRB, 0.19940E+00_JPRB, 0.19942E+00_JPRB, 0.20005E+00_JPRB, 0.20023E+00_JPRB, 0.20350E+00_JPRB ,&
&  0.21063E+00_JPRB, 0.22127E+00_JPRB, 0.23314E+00_JPRB, 0.24969E+00_JPRB, 0.26507E+00_JPRB, 0.28140E+00_JPRB, 0.30881E+00_JPRB ,&
&  0.34889E+00_JPRB, 0.45812E+00_JPRB, 0.52814E+00_JPRB, 0.46444E+00_JPRB, 0.48806E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.53923E+00_JPRB, 0.54304E+00_JPRB, 0.54753E+00_JPRB, 0.55015E+00_JPRB, 0.54489E+00_JPRB, 0.53778E+00_JPRB, 0.53318E+00_JPRB ,&
&  0.53829E+00_JPRB, 0.53810E+00_JPRB, 0.51455E+00_JPRB, 0.49596E+00_JPRB, 0.48523E+00_JPRB, 0.48166E+00_JPRB, 0.47846E+00_JPRB ,&
&  0.47238E+00_JPRB, 0.47037E+00_JPRB, 0.47180E+00_JPRB, 0.48334E+00_JPRB, 0.49292E+00_JPRB, 0.50129E+00_JPRB, 0.50883E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.51800E+00_JPRB, 0.53982E+00_JPRB, 0.53933E+00_JPRB, 0.53689E+00_JPRB, 0.53002E+00_JPRB, 0.51588E+00_JPRB ,&
&  0.46851E+00_JPRB, 0.40459E+00_JPRB, 0.34185E+00_JPRB, 0.35283E+00_JPRB, 0.43166E+00_JPRB, 0.44213E+00_JPRB, 0.42590E+00_JPRB ,&
&  0.43616E+00_JPRB, 0.43244E+00_JPRB, 0.41471E+00_JPRB, 0.40179E+00_JPRB, 0.38642E+00_JPRB, 0.37637E+00_JPRB, 0.36948E+00_JPRB ,&
&  0.36671E+00_JPRB, 0.36519E+00_JPRB, 0.35108E+00_JPRB, 0.30349E+00_JPRB, 0.27505E+00_JPRB, 0.25797E+00_JPRB, 0.24053E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.21807E+00_JPRB, 0.21623E+00_JPRB, 0.21638E+00_JPRB, 0.22056E+00_JPRB, 0.22200E+00_JPRB, 0.22338E+00_JPRB ,&
&  0.22283E+00_JPRB, 0.22156E+00_JPRB, 0.21747E+00_JPRB, 0.21291E+00_JPRB, 0.20979E+00_JPRB, 0.20918E+00_JPRB, 0.20898E+00_JPRB ,&
&  0.21150E+00_JPRB, 0.21552E+00_JPRB, 0.23159E+00_JPRB, 0.24907E+00_JPRB, 0.24923E+00_JPRB, 0.23397E+00_JPRB, 0.21880E+00_JPRB ,&
&  0.20941E+00_JPRB, 0.20459E+00_JPRB, 0.20155E+00_JPRB, 0.20097E+00_JPRB, 0.20353E+00_JPRB, 0.20711E+00_JPRB, 0.20808E+00_JPRB ,&
&  0.20801E+00_JPRB, 0.20875E+00_JPRB, 0.20890E+00_JPRB, 0.20945E+00_JPRB, 0.21018E+00_JPRB, 0.21260E+00_JPRB, 0.21711E+00_JPRB ,&
&  0.22248E+00_JPRB, 0.23214E+00_JPRB, 0.24320E+00_JPRB, 0.25812E+00_JPRB, 0.26897E+00_JPRB, 0.27676E+00_JPRB, 0.28627E+00_JPRB ,&
&  0.30333E+00_JPRB, 0.34103E+00_JPRB, 0.40998E+00_JPRB, 0.53674E+00_JPRB, 0.53221E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.59534E+00_JPRB, 0.58376E+00_JPRB, 0.57609E+00_JPRB, 0.57075E+00_JPRB, 0.56422E+00_JPRB, 0.56324E+00_JPRB, 0.56273E+00_JPRB ,&
&  0.55706E+00_JPRB, 0.54872E+00_JPRB, 0.54948E+00_JPRB, 0.53201E+00_JPRB, 0.50725E+00_JPRB, 0.49070E+00_JPRB, 0.48844E+00_JPRB ,&
&  0.49243E+00_JPRB, 0.49972E+00_JPRB, 0.50901E+00_JPRB, 0.51820E+00_JPRB, 0.52332E+00_JPRB, 0.53515E+00_JPRB, 0.55625E+00_JPRB ,&
&  0.58082E+00_JPRB, 0.59469E+00_JPRB, 0.59904E+00_JPRB, 0.59128E+00_JPRB, 0.57902E+00_JPRB, 0.56624E+00_JPRB, 0.55557E+00_JPRB ,&
&  0.54816E+00_JPRB, 0.44611E+00_JPRB, 0.37703E+00_JPRB, 0.38297E+00_JPRB, 0.45426E+00_JPRB, 0.43848E+00_JPRB, 0.43611E+00_JPRB ,&
&  0.44862E+00_JPRB, 0.44931E+00_JPRB, 0.43640E+00_JPRB, 0.41954E+00_JPRB, 0.40367E+00_JPRB, 0.39208E+00_JPRB, 0.38338E+00_JPRB ,&
&  0.37915E+00_JPRB, 0.38554E+00_JPRB, 0.39388E+00_JPRB, 0.40686E+00_JPRB, 0.41044E+00_JPRB, 0.39812E+00_JPRB, 0.38329E+00_JPRB ,&
&  0.36890E+00_JPRB, 0.36384E+00_JPRB, 0.36603E+00_JPRB, 0.37196E+00_JPRB, 0.37975E+00_JPRB, 0.38587E+00_JPRB, 0.38399E+00_JPRB ,&
&  0.35470E+00_JPRB, 0.33191E+00_JPRB, 0.30387E+00_JPRB, 0.27747E+00_JPRB, 0.25445E+00_JPRB, 0.23892E+00_JPRB, 0.23108E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.23563E+00_JPRB, 0.25654E+00_JPRB, 0.30540E+00_JPRB, 0.37487E+00_JPRB, 0.29621E+00_JPRB, 0.25686E+00_JPRB ,&
&  0.23493E+00_JPRB, 0.22199E+00_JPRB, 0.21713E+00_JPRB, 0.21765E+00_JPRB, 0.22136E+00_JPRB, 0.22696E+00_JPRB, 0.23133E+00_JPRB ,&
&  0.23481E+00_JPRB, 0.23821E+00_JPRB, 0.24047E+00_JPRB, 0.24163E+00_JPRB, 0.24330E+00_JPRB, 0.24439E+00_JPRB, 0.24334E+00_JPRB ,&
&  0.24162E+00_JPRB, 0.24707E+00_JPRB, 0.26370E+00_JPRB, 0.29018E+00_JPRB, 0.32193E+00_JPRB, 0.35611E+00_JPRB, 0.40768E+00_JPRB ,&
&  0.47332E+00_JPRB, 0.54974E+00_JPRB, 0.56689E+00_JPRB, 0.57856E+00_JPRB, 0.58875E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.63467E+00_JPRB, 0.60939E+00_JPRB, 0.59255E+00_JPRB, 0.57927E+00_JPRB, 0.56104E+00_JPRB, 0.54722E+00_JPRB, 0.54167E+00_JPRB ,&
&  0.54993E+00_JPRB, 0.54872E+00_JPRB, 0.53900E+00_JPRB, 0.53079E+00_JPRB, 0.51479E+00_JPRB, 0.50046E+00_JPRB, 0.49458E+00_JPRB ,&
&  0.50326E+00_JPRB, 0.51973E+00_JPRB, 0.53988E+00_JPRB, 0.54853E+00_JPRB, 0.54819E+00_JPRB, 0.55672E+00_JPRB, 0.57596E+00_JPRB ,&
&  0.60440E+00_JPRB, 0.62424E+00_JPRB, 0.63150E+00_JPRB, 0.61874E+00_JPRB, 0.60152E+00_JPRB, 0.59353E+00_JPRB, 0.57914E+00_JPRB ,&
&  0.56244E+00_JPRB, 0.56623E+00_JPRB, 0.49837E+00_JPRB, 0.49106E+00_JPRB, 0.46780E+00_JPRB, 0.45373E+00_JPRB, 0.45031E+00_JPRB ,&
&  0.46374E+00_JPRB, 0.46810E+00_JPRB, 0.46006E+00_JPRB, 0.44828E+00_JPRB, 0.43992E+00_JPRB, 0.42859E+00_JPRB, 0.41793E+00_JPRB ,&
&  0.41026E+00_JPRB, 0.41257E+00_JPRB, 0.42155E+00_JPRB, 0.43084E+00_JPRB, 0.44201E+00_JPRB, 0.45262E+00_JPRB, 0.44905E+00_JPRB ,&
&  0.43736E+00_JPRB, 0.43521E+00_JPRB, 0.44179E+00_JPRB, 0.45588E+00_JPRB, 0.47129E+00_JPRB, 0.47862E+00_JPRB, 0.48276E+00_JPRB ,&
&  0.47552E+00_JPRB, 0.45895E+00_JPRB, 0.44465E+00_JPRB, 0.43284E+00_JPRB, 0.40798E+00_JPRB, 0.34430E+00_JPRB, 0.32131E+00_JPRB ,&
&  0.33170E+00_JPRB, 0.39079E+00_JPRB, 0.46183E+00_JPRB, 0.45280E+00_JPRB, 0.44749E+00_JPRB, 0.43347E+00_JPRB, 0.39793E+00_JPRB ,&
&  0.35801E+00_JPRB, 0.29630E+00_JPRB, 0.27401E+00_JPRB, 0.27010E+00_JPRB, 0.26884E+00_JPRB, 0.27371E+00_JPRB, 0.28554E+00_JPRB ,&
&  0.30127E+00_JPRB, 0.32875E+00_JPRB, 0.34643E+00_JPRB, 0.35332E+00_JPRB, 0.35208E+00_JPRB, 0.33604E+00_JPRB, 0.32233E+00_JPRB ,&
&  0.31285E+00_JPRB, 0.33709E+00_JPRB, 0.39486E+00_JPRB, 0.47767E+00_JPRB, 0.47563E+00_JPRB, 0.49914E+00_JPRB, 0.52337E+00_JPRB ,&
&  0.54653E+00_JPRB, 0.58574E+00_JPRB, 0.61184E+00_JPRB, 0.61776E+00_JPRB, 0.63618E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.65827E+00_JPRB, 0.63095E+00_JPRB, 0.60216E+00_JPRB, 0.57731E+00_JPRB, 0.56507E+00_JPRB, 0.54726E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.53152E+00_JPRB, 0.53947E+00_JPRB, 0.53503E+00_JPRB, 0.52927E+00_JPRB, 0.51997E+00_JPRB, 0.51227E+00_JPRB, 0.50305E+00_JPRB ,&
&  0.49922E+00_JPRB, 0.51112E+00_JPRB, 0.53712E+00_JPRB, 0.55032E+00_JPRB, 0.55684E+00_JPRB, 0.58755E+00_JPRB, 0.61884E+00_JPRB ,&
&  0.62774E+00_JPRB, 0.64147E+00_JPRB, 0.64581E+00_JPRB, 0.62117E+00_JPRB, 0.60509E+00_JPRB, 0.59886E+00_JPRB, 0.60276E+00_JPRB ,&
&  0.59801E+00_JPRB, 0.60233E+00_JPRB, 0.59454E+00_JPRB, 0.52418E+00_JPRB, 0.52383E+00_JPRB, 0.49331E+00_JPRB, 0.48316E+00_JPRB ,&
&  0.47840E+00_JPRB, 0.48415E+00_JPRB, 0.48075E+00_JPRB, 0.47204E+00_JPRB, 0.46850E+00_JPRB, 0.47156E+00_JPRB, 0.47250E+00_JPRB ,&
&  0.46747E+00_JPRB, 0.45649E+00_JPRB, 0.46078E+00_JPRB, 0.46717E+00_JPRB, 0.47098E+00_JPRB, 0.47809E+00_JPRB, 0.48860E+00_JPRB ,&
&  0.47842E+00_JPRB, 0.46950E+00_JPRB, 0.46092E+00_JPRB, 0.46102E+00_JPRB, 0.46732E+00_JPRB, 0.48169E+00_JPRB, 0.48686E+00_JPRB ,&
&  0.48485E+00_JPRB, 0.48356E+00_JPRB, 0.48442E+00_JPRB, 0.49061E+00_JPRB, 0.50560E+00_JPRB, 0.52068E+00_JPRB, 0.52203E+00_JPRB ,&
&  0.52307E+00_JPRB, 0.52746E+00_JPRB, 0.53198E+00_JPRB, 0.53411E+00_JPRB, 0.53036E+00_JPRB, 0.51534E+00_JPRB, 0.49785E+00_JPRB ,&
&  0.48125E+00_JPRB, 0.46490E+00_JPRB, 0.45089E+00_JPRB, 0.44618E+00_JPRB, 0.45895E+00_JPRB, 0.48779E+00_JPRB, 0.50379E+00_JPRB ,&
&  0.50741E+00_JPRB, 0.50935E+00_JPRB, 0.51294E+00_JPRB, 0.51434E+00_JPRB, 0.51666E+00_JPRB, 0.51501E+00_JPRB, 0.50875E+00_JPRB ,&
&  0.49762E+00_JPRB, 0.48614E+00_JPRB, 0.48274E+00_JPRB, 0.48620E+00_JPRB, 0.49053E+00_JPRB, 0.50164E+00_JPRB, 0.51740E+00_JPRB ,&
&  0.53881E+00_JPRB, 0.57672E+00_JPRB, 0.60903E+00_JPRB, 0.63420E+00_JPRB, 0.65247E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.67626E+00_JPRB, 0.67428E+00_JPRB, 0.64561E+00_JPRB, 0.61397E+00_JPRB, 0.59489E+00_JPRB, 0.57369E+00_JPRB, 0.55663E+00_JPRB ,&
&  0.55510E+00_JPRB, 0.55252E+00_JPRB, 0.55066E+00_JPRB, 0.54840E+00_JPRB, 0.54677E+00_JPRB, 0.54363E+00_JPRB, 0.53152E+00_JPRB ,&
&  0.52719E+00_JPRB, 0.53389E+00_JPRB, 0.55642E+00_JPRB, 0.58196E+00_JPRB, 0.60254E+00_JPRB, 0.62908E+00_JPRB, 0.64011E+00_JPRB ,&
&  0.65154E+00_JPRB, 0.66375E+00_JPRB, 0.66612E+00_JPRB, 0.63387E+00_JPRB, 0.60734E+00_JPRB, 0.60857E+00_JPRB, 0.61961E+00_JPRB ,&
&  0.61996E+00_JPRB, 0.62322E+00_JPRB, 0.62713E+00_JPRB, 0.62363E+00_JPRB, 0.60330E+00_JPRB, 0.56541E+00_JPRB, 0.53566E+00_JPRB ,&
&  0.51147E+00_JPRB, 0.49286E+00_JPRB, 0.48739E+00_JPRB, 0.47646E+00_JPRB, 0.47514E+00_JPRB, 0.48509E+00_JPRB, 0.49761E+00_JPRB ,&
&  0.51043E+00_JPRB, 0.51602E+00_JPRB, 0.52290E+00_JPRB, 0.51958E+00_JPRB, 0.51514E+00_JPRB, 0.51763E+00_JPRB, 0.51727E+00_JPRB ,&
&  0.51635E+00_JPRB, 0.50720E+00_JPRB, 0.50455E+00_JPRB, 0.49767E+00_JPRB, 0.49383E+00_JPRB, 0.49439E+00_JPRB, 0.49749E+00_JPRB ,&
&  0.49867E+00_JPRB, 0.50403E+00_JPRB, 0.50561E+00_JPRB, 0.50767E+00_JPRB, 0.51701E+00_JPRB, 0.53357E+00_JPRB, 0.55027E+00_JPRB ,&
&  0.56277E+00_JPRB, 0.57261E+00_JPRB, 0.58048E+00_JPRB, 0.58634E+00_JPRB, 0.58616E+00_JPRB, 0.58160E+00_JPRB, 0.56780E+00_JPRB ,&
&  0.53723E+00_JPRB, 0.50439E+00_JPRB, 0.47952E+00_JPRB, 0.45956E+00_JPRB, 0.45558E+00_JPRB, 0.46619E+00_JPRB, 0.48374E+00_JPRB ,&
&  0.49698E+00_JPRB, 0.50109E+00_JPRB, 0.49533E+00_JPRB, 0.49376E+00_JPRB, 0.49903E+00_JPRB, 0.50870E+00_JPRB, 0.51617E+00_JPRB ,&
&  0.51540E+00_JPRB, 0.51208E+00_JPRB, 0.51551E+00_JPRB, 0.52188E+00_JPRB, 0.53513E+00_JPRB, 0.55211E+00_JPRB, 0.57060E+00_JPRB ,&
&  0.58928E+00_JPRB, 0.60400E+00_JPRB, 0.62323E+00_JPRB, 0.64284E+00_JPRB, 0.66831E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.68153E+00_JPRB, 0.67326E+00_JPRB, 0.65868E+00_JPRB, 0.63236E+00_JPRB, 0.61523E+00_JPRB, 0.58818E+00_JPRB, 0.57277E+00_JPRB ,&
&  0.57490E+00_JPRB, 0.57604E+00_JPRB, 0.58110E+00_JPRB, 0.59057E+00_JPRB, 0.58875E+00_JPRB, 0.58434E+00_JPRB, 0.58008E+00_JPRB ,&
&  0.58343E+00_JPRB, 0.58398E+00_JPRB, 0.59253E+00_JPRB, 0.60823E+00_JPRB, 0.62476E+00_JPRB, 0.63398E+00_JPRB, 0.64344E+00_JPRB ,&
&  0.65069E+00_JPRB, 0.64438E+00_JPRB, 0.64008E+00_JPRB, 0.61000E+00_JPRB, 0.59260E+00_JPRB, 0.59790E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.61536E+00_JPRB, 0.62592E+00_JPRB, 0.63655E+00_JPRB, 0.64234E+00_JPRB, 0.62911E+00_JPRB, 0.62587E+00_JPRB, 0.61569E+00_JPRB ,&
&  0.57417E+00_JPRB, 0.53341E+00_JPRB, 0.51127E+00_JPRB, 0.50404E+00_JPRB, 0.49929E+00_JPRB, 0.50512E+00_JPRB, 0.51183E+00_JPRB ,&
&  0.52401E+00_JPRB, 0.53532E+00_JPRB, 0.53827E+00_JPRB, 0.54167E+00_JPRB, 0.54332E+00_JPRB, 0.54241E+00_JPRB, 0.54249E+00_JPRB ,&
&  0.53922E+00_JPRB, 0.53703E+00_JPRB, 0.54400E+00_JPRB, 0.54465E+00_JPRB, 0.53610E+00_JPRB, 0.53393E+00_JPRB, 0.53225E+00_JPRB ,&
&  0.52983E+00_JPRB, 0.52733E+00_JPRB, 0.52424E+00_JPRB, 0.53419E+00_JPRB, 0.55944E+00_JPRB, 0.58853E+00_JPRB, 0.61377E+00_JPRB ,&
&  0.62803E+00_JPRB, 0.63958E+00_JPRB, 0.64270E+00_JPRB, 0.64492E+00_JPRB, 0.64456E+00_JPRB, 0.63596E+00_JPRB, 0.63289E+00_JPRB ,&
&  0.62517E+00_JPRB, 0.60622E+00_JPRB, 0.58084E+00_JPRB, 0.54126E+00_JPRB, 0.50280E+00_JPRB, 0.48092E+00_JPRB, 0.48052E+00_JPRB ,&
&  0.49264E+00_JPRB, 0.50497E+00_JPRB, 0.51132E+00_JPRB, 0.51947E+00_JPRB, 0.53438E+00_JPRB, 0.55224E+00_JPRB, 0.57088E+00_JPRB ,&
&  0.58243E+00_JPRB, 0.58212E+00_JPRB, 0.57922E+00_JPRB, 0.58677E+00_JPRB, 0.59710E+00_JPRB, 0.61435E+00_JPRB, 0.63337E+00_JPRB ,&
&  0.65024E+00_JPRB, 0.64985E+00_JPRB, 0.65030E+00_JPRB, 0.65156E+00_JPRB, 0.67116E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.68492E+00_JPRB, 0.68594E+00_JPRB, 0.67050E+00_JPRB, 0.65938E+00_JPRB, 0.64315E+00_JPRB, 0.62220E+00_JPRB, 0.60353E+00_JPRB ,&
&  0.60471E+00_JPRB, 0.61944E+00_JPRB, 0.63374E+00_JPRB, 0.64424E+00_JPRB, 0.65977E+00_JPRB, 0.66310E+00_JPRB, 0.66997E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.62973E+00_JPRB, 0.62405E+00_JPRB, 0.63186E+00_JPRB, 0.63375E+00_JPRB, 0.63943E+00_JPRB, 0.64982E+00_JPRB ,&
&  0.65947E+00_JPRB, 0.65463E+00_JPRB, 0.64645E+00_JPRB, 0.61702E+00_JPRB, 0.61036E+00_JPRB, 0.61526E+00_JPRB, 0.62014E+00_JPRB ,&
&  0.63039E+00_JPRB, 0.64683E+00_JPRB, 0.65156E+00_JPRB, 0.66016E+00_JPRB, 0.67007E+00_JPRB, 0.67491E+00_JPRB, 0.67313E+00_JPRB ,&
&  0.64944E+00_JPRB, 0.62478E+00_JPRB, 0.61436E+00_JPRB, 0.59648E+00_JPRB, 0.57424E+00_JPRB, 0.55848E+00_JPRB, 0.54992E+00_JPRB ,&
&  0.54491E+00_JPRB, 0.54936E+00_JPRB, 0.55525E+00_JPRB, 0.56910E+00_JPRB, 0.57429E+00_JPRB, 0.57844E+00_JPRB, 0.58359E+00_JPRB ,&
&  0.58277E+00_JPRB, 0.57824E+00_JPRB, 0.57796E+00_JPRB, 0.58772E+00_JPRB, 0.59380E+00_JPRB, 0.59189E+00_JPRB, 0.58782E+00_JPRB ,&
&  0.57530E+00_JPRB, 0.56131E+00_JPRB, 0.56850E+00_JPRB, 0.58320E+00_JPRB, 0.58977E+00_JPRB, 0.61459E+00_JPRB, 0.64678E+00_JPRB ,&
&  0.67404E+00_JPRB, 0.69194E+00_JPRB, 0.69159E+00_JPRB, 0.68848E+00_JPRB, 0.68749E+00_JPRB, 0.69513E+00_JPRB, 0.70761E+00_JPRB ,&
&  0.69202E+00_JPRB, 0.68293E+00_JPRB, 0.68102E+00_JPRB, 0.66203E+00_JPRB, 0.62605E+00_JPRB, 0.59540E+00_JPRB, 0.57269E+00_JPRB ,&
&  0.55443E+00_JPRB, 0.54396E+00_JPRB, 0.54748E+00_JPRB, 0.55119E+00_JPRB, 0.56424E+00_JPRB, 0.59985E+00_JPRB, 0.63674E+00_JPRB ,&
&  0.67664E+00_JPRB, 0.69704E+00_JPRB, 0.70318E+00_JPRB, 0.71082E+00_JPRB, 0.71630E+00_JPRB, 0.70513E+00_JPRB, 0.69574E+00_JPRB ,&
&  0.69300E+00_JPRB, 0.68463E+00_JPRB, 0.67530E+00_JPRB, 0.66541E+00_JPRB, 0.67232E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.67977E+00_JPRB, 0.70109E+00_JPRB, 0.66803E+00_JPRB, 0.64973E+00_JPRB, 0.62944E+00_JPRB, 0.62512E+00_JPRB, 0.61599E+00_JPRB ,&
&  0.62614E+00_JPRB, 0.64273E+00_JPRB, 0.66634E+00_JPRB, 0.67432E+00_JPRB, 0.68131E+00_JPRB, 0.68431E+00_JPRB, 0.70167E+00_JPRB ,&
&  0.69750E+00_JPRB, 0.68963E+00_JPRB, 0.69306E+00_JPRB, 0.68293E+00_JPRB, 0.67427E+00_JPRB, 0.67887E+00_JPRB, 0.68722E+00_JPRB ,&
&  0.69693E+00_JPRB, 0.68822E+00_JPRB, 0.67093E+00_JPRB, 0.64436E+00_JPRB, 0.63175E+00_JPRB, 0.62911E+00_JPRB, 0.63648E+00_JPRB ,&
&  0.64180E+00_JPRB, 0.66507E+00_JPRB, 0.60406E+00_JPRB, 0.66115E+00_JPRB, 0.67126E+00_JPRB, 0.67173E+00_JPRB, 0.67327E+00_JPRB ,&
&  0.68082E+00_JPRB, 0.67193E+00_JPRB, 0.66342E+00_JPRB, 0.65344E+00_JPRB, 0.62928E+00_JPRB, 0.60966E+00_JPRB, 0.58941E+00_JPRB ,&
&  0.57135E+00_JPRB, 0.55998E+00_JPRB, 0.56453E+00_JPRB, 0.58153E+00_JPRB, 0.59900E+00_JPRB, 0.60876E+00_JPRB, 0.61482E+00_JPRB ,&
&  0.61810E+00_JPRB, 0.61076E+00_JPRB, 0.60074E+00_JPRB, 0.60085E+00_JPRB, 0.60749E+00_JPRB, 0.61266E+00_JPRB, 0.61626E+00_JPRB ,&
&  0.61726E+00_JPRB, 0.59413E+00_JPRB, 0.58097E+00_JPRB, 0.59694E+00_JPRB, 0.61394E+00_JPRB, 0.63228E+00_JPRB, 0.65629E+00_JPRB ,&
&  0.67440E+00_JPRB, 0.68130E+00_JPRB, 0.68023E+00_JPRB, 0.68694E+00_JPRB, 0.69398E+00_JPRB, 0.70284E+00_JPRB, 0.71007E+00_JPRB ,&
&  0.70075E+00_JPRB, 0.69897E+00_JPRB, 0.69994E+00_JPRB, 0.70012E+00_JPRB, 0.69158E+00_JPRB, 0.66926E+00_JPRB, 0.63766E+00_JPRB ,&
&  0.62316E+00_JPRB, 0.60818E+00_JPRB, 0.59696E+00_JPRB, 0.59428E+00_JPRB, 0.59688E+00_JPRB, 0.62279E+00_JPRB, 0.65945E+00_JPRB ,&
&  0.69573E+00_JPRB, 0.72522E+00_JPRB, 0.75059E+00_JPRB, 0.77374E+00_JPRB, 0.77143E+00_JPRB, 0.74949E+00_JPRB, 0.73976E+00_JPRB ,&
&  0.72481E+00_JPRB, 0.70861E+00_JPRB, 0.69779E+00_JPRB, 0.68890E+00_JPRB, 0.67869E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.71807E+00_JPRB, 0.71877E+00_JPRB, 0.70818E+00_JPRB, 0.66949E+00_JPRB, 0.64021E+00_JPRB, 0.64988E+00_JPRB, 0.65920E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.67322E+00_JPRB, 0.69838E+00_JPRB, 0.72399E+00_JPRB, 0.74121E+00_JPRB, 0.74142E+00_JPRB, 0.74589E+00_JPRB ,&
&  0.74389E+00_JPRB, 0.74212E+00_JPRB, 0.75234E+00_JPRB, 0.75737E+00_JPRB, 0.74680E+00_JPRB, 0.72573E+00_JPRB, 0.72296E+00_JPRB ,&
&  0.72872E+00_JPRB, 0.72445E+00_JPRB, 0.70609E+00_JPRB, 0.69356E+00_JPRB, 0.68340E+00_JPRB, 0.68846E+00_JPRB, 0.69466E+00_JPRB ,&
&  0.68945E+00_JPRB, 0.63740E+00_JPRB, 0.67618E+00_JPRB, 0.69065E+00_JPRB, 0.70245E+00_JPRB, 0.69559E+00_JPRB, 0.70363E+00_JPRB ,&
&  0.71690E+00_JPRB, 0.72509E+00_JPRB, 0.70537E+00_JPRB, 0.68141E+00_JPRB, 0.65847E+00_JPRB, 0.63857E+00_JPRB, 0.60887E+00_JPRB ,&
&  0.59720E+00_JPRB, 0.59283E+00_JPRB, 0.60481E+00_JPRB, 0.61808E+00_JPRB, 0.63368E+00_JPRB, 0.63974E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.65181E+00_JPRB, 0.64642E+00_JPRB, 0.64168E+00_JPRB, 0.63831E+00_JPRB, 0.63806E+00_JPRB, 0.63257E+00_JPRB, 0.62460E+00_JPRB ,&
&  0.61864E+00_JPRB, 0.61697E+00_JPRB, 0.62122E+00_JPRB, 0.63044E+00_JPRB, 0.64160E+00_JPRB, 0.65842E+00_JPRB, 0.67120E+00_JPRB ,&
&  0.68723E+00_JPRB, 0.69203E+00_JPRB, 0.69632E+00_JPRB, 0.70914E+00_JPRB, 0.71762E+00_JPRB, 0.71627E+00_JPRB, 0.71375E+00_JPRB ,&
&  0.71166E+00_JPRB, 0.70659E+00_JPRB, 0.71143E+00_JPRB, 0.71076E+00_JPRB, 0.71161E+00_JPRB, 0.70370E+00_JPRB, 0.68919E+00_JPRB ,&
&  0.68372E+00_JPRB, 0.67949E+00_JPRB, 0.67085E+00_JPRB, 0.66813E+00_JPRB, 0.66605E+00_JPRB, 0.68353E+00_JPRB, 0.71414E+00_JPRB ,&
&  0.73049E+00_JPRB, 0.75266E+00_JPRB, 0.78459E+00_JPRB, 0.80558E+00_JPRB, 0.80465E+00_JPRB, 0.80383E+00_JPRB, 0.79228E+00_JPRB ,&
&  0.77277E+00_JPRB, 0.75871E+00_JPRB, 0.74578E+00_JPRB, 0.74156E+00_JPRB, 0.73799E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.75289E+00_JPRB, 0.74603E+00_JPRB, 0.73115E+00_JPRB, 0.70413E+00_JPRB, 0.67024E+00_JPRB, 0.67894E+00_JPRB, 0.69168E+00_JPRB ,&
&  0.68617E+00_JPRB, 0.67704E+00_JPRB, 0.68749E+00_JPRB, 0.71251E+00_JPRB, 0.73745E+00_JPRB, 0.75206E+00_JPRB, 0.75515E+00_JPRB ,&
&  0.75104E+00_JPRB, 0.75388E+00_JPRB, 0.76276E+00_JPRB, 0.78000E+00_JPRB, 0.77910E+00_JPRB, 0.76105E+00_JPRB, 0.74980E+00_JPRB ,&
&  0.74635E+00_JPRB, 0.74425E+00_JPRB, 0.74121E+00_JPRB, 0.72421E+00_JPRB, 0.71285E+00_JPRB, 0.71002E+00_JPRB, 0.71064E+00_JPRB ,&
&  0.70798E+00_JPRB, 0.57252E+00_JPRB, 0.67315E+00_JPRB, 0.69195E+00_JPRB, 0.71182E+00_JPRB, 0.71023E+00_JPRB, 0.71219E+00_JPRB ,&
&  0.71717E+00_JPRB, 0.73234E+00_JPRB, 0.72441E+00_JPRB, 0.69879E+00_JPRB, 0.67190E+00_JPRB, 0.64315E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.61029E+00_JPRB, 0.61064E+00_JPRB, 0.61826E+00_JPRB, 0.62706E+00_JPRB, 0.63705E+00_JPRB, 0.63784E+00_JPRB, 0.64122E+00_JPRB ,&
&  0.64367E+00_JPRB, 0.64896E+00_JPRB, 0.65204E+00_JPRB, 0.65845E+00_JPRB, 0.66133E+00_JPRB, 0.65884E+00_JPRB, 0.65212E+00_JPRB ,&
&  0.64251E+00_JPRB, 0.63648E+00_JPRB, 0.63585E+00_JPRB, 0.63811E+00_JPRB, 0.64483E+00_JPRB, 0.65633E+00_JPRB, 0.66753E+00_JPRB ,&
&  0.68051E+00_JPRB, 0.68889E+00_JPRB, 0.69332E+00_JPRB, 0.69560E+00_JPRB, 0.70484E+00_JPRB, 0.71465E+00_JPRB, 0.71496E+00_JPRB ,&
&  0.70927E+00_JPRB, 0.70276E+00_JPRB, 0.70134E+00_JPRB, 0.69885E+00_JPRB, 0.69994E+00_JPRB, 0.70127E+00_JPRB, 0.69758E+00_JPRB ,&
&  0.68972E+00_JPRB, 0.68799E+00_JPRB, 0.68915E+00_JPRB, 0.67842E+00_JPRB, 0.67986E+00_JPRB, 0.68954E+00_JPRB, 0.70766E+00_JPRB ,&
&  0.73031E+00_JPRB, 0.76043E+00_JPRB, 0.79028E+00_JPRB, 0.81688E+00_JPRB, 0.83335E+00_JPRB, 0.83985E+00_JPRB, 0.83545E+00_JPRB ,&
&  0.81827E+00_JPRB, 0.79296E+00_JPRB, 0.77387E+00_JPRB, 0.76437E+00_JPRB, 0.76644E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.78265E+00_JPRB, 0.77270E+00_JPRB, 0.75944E+00_JPRB, 0.74192E+00_JPRB, 0.72633E+00_JPRB, 0.71810E+00_JPRB, 0.72415E+00_JPRB ,&
&  0.72335E+00_JPRB, 0.71167E+00_JPRB, 0.72134E+00_JPRB, 0.73465E+00_JPRB, 0.74588E+00_JPRB, 0.75674E+00_JPRB, 0.76171E+00_JPRB ,&
&  0.76712E+00_JPRB, 0.76565E+00_JPRB, 0.76976E+00_JPRB, 0.78222E+00_JPRB, 0.80070E+00_JPRB, 0.80216E+00_JPRB, 0.79907E+00_JPRB ,&
&  0.77622E+00_JPRB, 0.76447E+00_JPRB, 0.75376E+00_JPRB, 0.74086E+00_JPRB, 0.72956E+00_JPRB, 0.72624E+00_JPRB, 0.72971E+00_JPRB ,&
&  0.61829E+00_JPRB, 0.52577E+00_JPRB, 0.63412E+00_JPRB, 0.74688E+00_JPRB, 0.76493E+00_JPRB, 0.76447E+00_JPRB, 0.77833E+00_JPRB ,&
&  0.78105E+00_JPRB, 0.77949E+00_JPRB, 0.77576E+00_JPRB, 0.76264E+00_JPRB, 0.73084E+00_JPRB, 0.69730E+00_JPRB, 0.67780E+00_JPRB ,&
&  0.66416E+00_JPRB, 0.66585E+00_JPRB, 0.66375E+00_JPRB, 0.66750E+00_JPRB, 0.66534E+00_JPRB, 0.65692E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.65574E+00_JPRB, 0.66636E+00_JPRB, 0.67668E+00_JPRB, 0.68428E+00_JPRB, 0.68733E+00_JPRB, 0.68859E+00_JPRB, 0.68647E+00_JPRB ,&
&  0.68496E+00_JPRB, 0.68286E+00_JPRB, 0.67880E+00_JPRB, 0.67932E+00_JPRB, 0.68105E+00_JPRB, 0.68708E+00_JPRB, 0.69799E+00_JPRB ,&
&  0.70275E+00_JPRB, 0.70238E+00_JPRB, 0.70295E+00_JPRB, 0.70400E+00_JPRB, 0.71421E+00_JPRB, 0.72255E+00_JPRB, 0.72067E+00_JPRB ,&
&  0.71643E+00_JPRB, 0.71414E+00_JPRB, 0.72071E+00_JPRB, 0.72601E+00_JPRB, 0.73258E+00_JPRB, 0.73554E+00_JPRB, 0.73132E+00_JPRB ,&
&  0.72268E+00_JPRB, 0.70862E+00_JPRB, 0.69818E+00_JPRB, 0.69701E+00_JPRB, 0.70565E+00_JPRB, 0.71181E+00_JPRB, 0.72733E+00_JPRB ,&
&  0.75670E+00_JPRB, 0.79326E+00_JPRB, 0.82434E+00_JPRB, 0.86458E+00_JPRB, 0.88855E+00_JPRB, 0.90089E+00_JPRB, 0.89654E+00_JPRB ,&
&  0.87910E+00_JPRB, 0.83956E+00_JPRB, 0.82746E+00_JPRB, 0.80491E+00_JPRB, 0.79313E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.76620E+00_JPRB, 0.76990E+00_JPRB, 0.77185E+00_JPRB, 0.76096E+00_JPRB, 0.74693E+00_JPRB, 0.74158E+00_JPRB, 0.74782E+00_JPRB ,&
&  0.74346E+00_JPRB, 0.73012E+00_JPRB, 0.73515E+00_JPRB, 0.73961E+00_JPRB, 0.74801E+00_JPRB, 0.75307E+00_JPRB, 0.75908E+00_JPRB ,&
&  0.76523E+00_JPRB, 0.77064E+00_JPRB, 0.76975E+00_JPRB, 0.77874E+00_JPRB, 0.77182E+00_JPRB, 0.76651E+00_JPRB, 0.76363E+00_JPRB ,&
&  0.75479E+00_JPRB, 0.74543E+00_JPRB, 0.73787E+00_JPRB, 0.72778E+00_JPRB, 0.72077E+00_JPRB, 0.71289E+00_JPRB, 0.69812E+00_JPRB ,&
&  0.71116E+00_JPRB, 0.56203E+00_JPRB, 0.65518E+00_JPRB, 0.81906E+00_JPRB, 0.83364E+00_JPRB, 0.82757E+00_JPRB, 0.83127E+00_JPRB ,&
&  0.84270E+00_JPRB, 0.84776E+00_JPRB, 0.83597E+00_JPRB, 0.80934E+00_JPRB, 0.76268E+00_JPRB, 0.73944E+00_JPRB, 0.72471E+00_JPRB ,&
&  0.70757E+00_JPRB, 0.69402E+00_JPRB, 0.68503E+00_JPRB, 0.68467E+00_JPRB, 0.68269E+00_JPRB, 0.67791E+00_JPRB, 0.67351E+00_JPRB ,&
&  0.67642E+00_JPRB, 0.68593E+00_JPRB, 0.69175E+00_JPRB, 0.69717E+00_JPRB, 0.70277E+00_JPRB, 0.69986E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.69809E+00_JPRB, 0.69683E+00_JPRB, 0.69396E+00_JPRB, 0.69218E+00_JPRB, 0.69170E+00_JPRB, 0.70597E+00_JPRB, 0.71726E+00_JPRB ,&
&  0.71659E+00_JPRB, 0.71425E+00_JPRB, 0.71746E+00_JPRB, 0.72327E+00_JPRB, 0.72814E+00_JPRB, 0.73117E+00_JPRB, 0.72744E+00_JPRB ,&
&  0.72387E+00_JPRB, 0.71772E+00_JPRB, 0.71741E+00_JPRB, 0.71761E+00_JPRB, 0.71364E+00_JPRB, 0.71271E+00_JPRB, 0.71596E+00_JPRB ,&
&  0.71056E+00_JPRB, 0.69922E+00_JPRB, 0.70109E+00_JPRB, 0.71395E+00_JPRB, 0.72405E+00_JPRB, 0.72899E+00_JPRB, 0.74209E+00_JPRB ,&
&  0.77102E+00_JPRB, 0.80429E+00_JPRB, 0.83092E+00_JPRB, 0.86904E+00_JPRB, 0.89037E+00_JPRB, 0.90200E+00_JPRB, 0.90209E+00_JPRB ,&
&  0.88723E+00_JPRB, 0.86744E+00_JPRB, 0.62753E+00_JPRB, 0.73360E+00_JPRB, 0.75743E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.77297E+00_JPRB, 0.78573E+00_JPRB, 0.78292E+00_JPRB, 0.77359E+00_JPRB, 0.76629E+00_JPRB, 0.76137E+00_JPRB, 0.76444E+00_JPRB ,&
&  0.76192E+00_JPRB, 0.75563E+00_JPRB, 0.75856E+00_JPRB, 0.76219E+00_JPRB, 0.76659E+00_JPRB, 0.76671E+00_JPRB, 0.77426E+00_JPRB ,&
&  0.77735E+00_JPRB, 0.78071E+00_JPRB, 0.78535E+00_JPRB, 0.78029E+00_JPRB, 0.76171E+00_JPRB, 0.74856E+00_JPRB, 0.73482E+00_JPRB ,&
&  0.73344E+00_JPRB, 0.73308E+00_JPRB, 0.73360E+00_JPRB, 0.72740E+00_JPRB, 0.70853E+00_JPRB, 0.69272E+00_JPRB, 0.68276E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.54787E+00_JPRB, 0.64726E+00_JPRB, 0.87814E+00_JPRB, 0.98557E+00_JPRB, 0.94227E+00_JPRB, 0.92239E+00_JPRB ,&
&  0.91999E+00_JPRB, 0.90980E+00_JPRB, 0.88284E+00_JPRB, 0.84709E+00_JPRB, 0.80348E+00_JPRB, 0.78956E+00_JPRB, 0.77015E+00_JPRB ,&
&  0.75103E+00_JPRB, 0.74323E+00_JPRB, 0.73610E+00_JPRB, 0.73464E+00_JPRB, 0.72374E+00_JPRB, 0.71113E+00_JPRB, 0.71143E+00_JPRB ,&
&  0.71859E+00_JPRB, 0.71724E+00_JPRB, 0.71996E+00_JPRB, 0.72338E+00_JPRB, 0.71863E+00_JPRB, 0.73344E+00_JPRB, 0.73762E+00_JPRB ,&
&  0.73766E+00_JPRB, 0.73361E+00_JPRB, 0.73157E+00_JPRB, 0.72860E+00_JPRB, 0.73220E+00_JPRB, 0.75151E+00_JPRB, 0.76411E+00_JPRB ,&
&  0.75690E+00_JPRB, 0.74669E+00_JPRB, 0.75373E+00_JPRB, 0.75707E+00_JPRB, 0.75074E+00_JPRB, 0.73774E+00_JPRB, 0.72282E+00_JPRB ,&
&  0.71798E+00_JPRB, 0.71527E+00_JPRB, 0.71456E+00_JPRB, 0.71044E+00_JPRB, 0.70355E+00_JPRB, 0.69801E+00_JPRB, 0.69560E+00_JPRB ,&
&  0.69484E+00_JPRB, 0.69496E+00_JPRB, 0.70124E+00_JPRB, 0.71510E+00_JPRB, 0.73219E+00_JPRB, 0.75134E+00_JPRB, 0.77148E+00_JPRB ,&
&  0.79772E+00_JPRB, 0.82846E+00_JPRB, 0.85934E+00_JPRB, 0.83890E+00_JPRB, 0.90203E+00_JPRB, 0.91919E+00_JPRB, 0.92999E+00_JPRB ,&
&  0.92357E+00_JPRB, 0.88720E+00_JPRB, 0.83867E+00_JPRB, 0.66297E+00_JPRB, 0.73900E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.79907E+00_JPRB, 0.82678E+00_JPRB, 0.81977E+00_JPRB, 0.79453E+00_JPRB, 0.78065E+00_JPRB, 0.77531E+00_JPRB, 0.77410E+00_JPRB ,&
&  0.76996E+00_JPRB, 0.75947E+00_JPRB, 0.76439E+00_JPRB, 0.77173E+00_JPRB, 0.77129E+00_JPRB, 0.76224E+00_JPRB, 0.75636E+00_JPRB ,&
&  0.75593E+00_JPRB, 0.76096E+00_JPRB, 0.76702E+00_JPRB, 0.75799E+00_JPRB, 0.73839E+00_JPRB, 0.72541E+00_JPRB, 0.71392E+00_JPRB ,&
&  0.71243E+00_JPRB, 0.71250E+00_JPRB, 0.71059E+00_JPRB, 0.71258E+00_JPRB, 0.69284E+00_JPRB, 0.67077E+00_JPRB, 0.65238E+00_JPRB ,&
&  0.63980E+00_JPRB, 0.57193E+00_JPRB, 0.77476E+00_JPRB, 0.10580E+01_JPRB, 0.11842E+01_JPRB, 0.11499E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.96025E+00_JPRB, 0.91178E+00_JPRB, 0.86547E+00_JPRB, 0.83932E+00_JPRB, 0.82540E+00_JPRB, 0.81754E+00_JPRB ,&
&  0.81074E+00_JPRB, 0.80231E+00_JPRB, 0.79575E+00_JPRB, 0.77730E+00_JPRB, 0.76132E+00_JPRB, 0.75714E+00_JPRB, 0.75611E+00_JPRB ,&
&  0.75675E+00_JPRB, 0.74859E+00_JPRB, 0.74597E+00_JPRB, 0.74657E+00_JPRB, 0.72890E+00_JPRB, 0.72412E+00_JPRB, 0.71847E+00_JPRB ,&
&  0.71920E+00_JPRB, 0.73073E+00_JPRB, 0.73286E+00_JPRB, 0.76181E+00_JPRB, 0.77325E+00_JPRB, 0.78337E+00_JPRB, 0.78452E+00_JPRB ,&
&  0.77787E+00_JPRB, 0.77685E+00_JPRB, 0.77074E+00_JPRB, 0.76780E+00_JPRB, 0.75871E+00_JPRB, 0.73726E+00_JPRB, 0.71739E+00_JPRB ,&
&  0.71434E+00_JPRB, 0.71616E+00_JPRB, 0.70800E+00_JPRB, 0.70800E+00_JPRB, 0.70969E+00_JPRB, 0.70391E+00_JPRB, 0.70019E+00_JPRB ,&
&  0.69799E+00_JPRB, 0.70064E+00_JPRB, 0.70329E+00_JPRB, 0.71438E+00_JPRB, 0.73248E+00_JPRB, 0.75372E+00_JPRB, 0.77572E+00_JPRB ,&
&  0.79718E+00_JPRB, 0.81402E+00_JPRB, 0.84005E+00_JPRB, 0.90378E+00_JPRB, 0.91472E+00_JPRB, 0.91591E+00_JPRB, 0.93119E+00_JPRB ,&
&  0.92212E+00_JPRB, 0.89083E+00_JPRB, 0.85870E+00_JPRB, 0.81052E+00_JPRB, 0.74557E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.89686E+00_JPRB, 0.88295E+00_JPRB, 0.86288E+00_JPRB, 0.83061E+00_JPRB, 0.81548E+00_JPRB, 0.79984E+00_JPRB, 0.78333E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.76721E+00_JPRB, 0.76139E+00_JPRB, 0.74824E+00_JPRB, 0.73564E+00_JPRB, 0.72834E+00_JPRB, 0.71890E+00_JPRB ,&
&  0.72014E+00_JPRB, 0.72121E+00_JPRB, 0.71428E+00_JPRB, 0.70721E+00_JPRB, 0.70504E+00_JPRB, 0.70314E+00_JPRB, 0.70811E+00_JPRB ,&
&  0.70059E+00_JPRB, 0.69151E+00_JPRB, 0.69138E+00_JPRB, 0.68467E+00_JPRB, 0.67266E+00_JPRB, 0.66188E+00_JPRB, 0.63330E+00_JPRB ,&
&  0.63136E+00_JPRB, 0.66254E+00_JPRB, 0.10590E+01_JPRB, 0.14930E+01_JPRB, 0.14895E+01_JPRB, 0.13822E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.11373E+01_JPRB, 0.10425E+01_JPRB, 0.95455E+00_JPRB, 0.90184E+00_JPRB, 0.88947E+00_JPRB, 0.87515E+00_JPRB, 0.85732E+00_JPRB ,&
&  0.83381E+00_JPRB, 0.81135E+00_JPRB, 0.79415E+00_JPRB, 0.79564E+00_JPRB, 0.78716E+00_JPRB, 0.78475E+00_JPRB, 0.78580E+00_JPRB ,&
&  0.77997E+00_JPRB, 0.77432E+00_JPRB, 0.76033E+00_JPRB, 0.75319E+00_JPRB, 0.74765E+00_JPRB, 0.74823E+00_JPRB, 0.75875E+00_JPRB ,&
&  0.78326E+00_JPRB, 0.78621E+00_JPRB, 0.79588E+00_JPRB, 0.80683E+00_JPRB, 0.81964E+00_JPRB, 0.81768E+00_JPRB, 0.80382E+00_JPRB ,&
&  0.79098E+00_JPRB, 0.78546E+00_JPRB, 0.76092E+00_JPRB, 0.76912E+00_JPRB, 0.75996E+00_JPRB, 0.74605E+00_JPRB, 0.73720E+00_JPRB ,&
&  0.72568E+00_JPRB, 0.72809E+00_JPRB, 0.71992E+00_JPRB, 0.72073E+00_JPRB, 0.72426E+00_JPRB, 0.72315E+00_JPRB, 0.71906E+00_JPRB ,&
&  0.72411E+00_JPRB, 0.72219E+00_JPRB, 0.72789E+00_JPRB, 0.75041E+00_JPRB, 0.77784E+00_JPRB, 0.79174E+00_JPRB, 0.80653E+00_JPRB ,&
&  0.82124E+00_JPRB, 0.84860E+00_JPRB, 0.82644E+00_JPRB, 0.86389E+00_JPRB, 0.97893E+00_JPRB, 0.98935E+00_JPRB, 0.95074E+00_JPRB ,&
&  0.92155E+00_JPRB, 0.89353E+00_JPRB, 0.87932E+00_JPRB, 0.85392E+00_JPRB, 0.87086E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.87491E+00_JPRB, 0.84364E+00_JPRB, 0.82183E+00_JPRB, 0.79712E+00_JPRB, 0.78819E+00_JPRB, 0.77292E+00_JPRB, 0.75212E+00_JPRB ,&
&  0.73970E+00_JPRB, 0.73786E+00_JPRB, 0.73074E+00_JPRB, 0.71461E+00_JPRB, 0.70049E+00_JPRB, 0.69515E+00_JPRB, 0.69438E+00_JPRB ,&
&  0.69260E+00_JPRB, 0.69113E+00_JPRB, 0.68905E+00_JPRB, 0.68266E+00_JPRB, 0.68714E+00_JPRB, 0.69729E+00_JPRB, 0.70829E+00_JPRB ,&
&  0.69351E+00_JPRB, 0.68168E+00_JPRB, 0.67090E+00_JPRB, 0.65990E+00_JPRB, 0.65623E+00_JPRB, 0.63844E+00_JPRB, 0.61896E+00_JPRB ,&
&  0.68360E+00_JPRB, 0.85111E+00_JPRB, 0.14195E+01_JPRB, 0.18703E+01_JPRB, 0.17221E+01_JPRB, 0.13851E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.10194E+01_JPRB, 0.95860E+00_JPRB, 0.91531E+00_JPRB, 0.89735E+00_JPRB, 0.87472E+00_JPRB, 0.83106E+00_JPRB ,&
&  0.79867E+00_JPRB, 0.77919E+00_JPRB, 0.76842E+00_JPRB, 0.76574E+00_JPRB, 0.75594E+00_JPRB, 0.76256E+00_JPRB, 0.76524E+00_JPRB ,&
&  0.76497E+00_JPRB, 0.75973E+00_JPRB, 0.75279E+00_JPRB, 0.74536E+00_JPRB, 0.75667E+00_JPRB, 0.79945E+00_JPRB, 0.85680E+00_JPRB ,&
&  0.85995E+00_JPRB, 0.86980E+00_JPRB, 0.86398E+00_JPRB, 0.86413E+00_JPRB, 0.85836E+00_JPRB, 0.82353E+00_JPRB, 0.80190E+00_JPRB ,&
&  0.78227E+00_JPRB, 0.77862E+00_JPRB, 0.75694E+00_JPRB, 0.75617E+00_JPRB, 0.75896E+00_JPRB, 0.75463E+00_JPRB, 0.75501E+00_JPRB ,&
&  0.75090E+00_JPRB, 0.74789E+00_JPRB, 0.74446E+00_JPRB, 0.75183E+00_JPRB, 0.75002E+00_JPRB, 0.73862E+00_JPRB, 0.73190E+00_JPRB ,&
&  0.73275E+00_JPRB, 0.72218E+00_JPRB, 0.73251E+00_JPRB, 0.76665E+00_JPRB, 0.79519E+00_JPRB, 0.80694E+00_JPRB, 0.82221E+00_JPRB ,&
&  0.84010E+00_JPRB, 0.85129E+00_JPRB, 0.83353E+00_JPRB, 0.86128E+00_JPRB, 0.90650E+00_JPRB, 0.99865E+00_JPRB, 0.93498E+00_JPRB ,&
&  0.89866E+00_JPRB, 0.87941E+00_JPRB, 0.86456E+00_JPRB, 0.84026E+00_JPRB, 0.89277E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.83094E+00_JPRB, 0.80888E+00_JPRB, 0.79672E+00_JPRB, 0.78630E+00_JPRB, 0.76634E+00_JPRB, 0.74433E+00_JPRB, 0.73856E+00_JPRB ,&
&  0.73486E+00_JPRB, 0.71558E+00_JPRB, 0.69295E+00_JPRB, 0.66407E+00_JPRB, 0.65612E+00_JPRB, 0.65923E+00_JPRB, 0.66252E+00_JPRB ,&
&  0.65979E+00_JPRB, 0.65978E+00_JPRB, 0.65873E+00_JPRB, 0.65873E+00_JPRB, 0.65960E+00_JPRB, 0.68390E+00_JPRB, 0.69487E+00_JPRB ,&
&  0.68847E+00_JPRB, 0.69104E+00_JPRB, 0.67102E+00_JPRB, 0.64912E+00_JPRB, 0.63879E+00_JPRB, 0.62840E+00_JPRB, 0.62534E+00_JPRB ,&
&  0.78478E+00_JPRB, 0.10754E+01_JPRB, 0.19259E+01_JPRB, 0.19225E+01_JPRB, 0.15320E+01_JPRB, 0.11919E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.99222E+00_JPRB, 0.93093E+00_JPRB, 0.88387E+00_JPRB, 0.84574E+00_JPRB, 0.82281E+00_JPRB, 0.79497E+00_JPRB ,&
&  0.77861E+00_JPRB, 0.76904E+00_JPRB, 0.74570E+00_JPRB, 0.73447E+00_JPRB, 0.72828E+00_JPRB, 0.72967E+00_JPRB, 0.72899E+00_JPRB ,&
&  0.73292E+00_JPRB, 0.73490E+00_JPRB, 0.73497E+00_JPRB, 0.72072E+00_JPRB, 0.69791E+00_JPRB, 0.67285E+00_JPRB, 0.84463E+00_JPRB ,&
&  0.11132E+01_JPRB, 0.10433E+01_JPRB, 0.94320E+00_JPRB, 0.89921E+00_JPRB, 0.86775E+00_JPRB, 0.81867E+00_JPRB, 0.80552E+00_JPRB ,&
&  0.79770E+00_JPRB, 0.80141E+00_JPRB, 0.78917E+00_JPRB, 0.80078E+00_JPRB, 0.80131E+00_JPRB, 0.79544E+00_JPRB, 0.78775E+00_JPRB ,&
&  0.77052E+00_JPRB, 0.76908E+00_JPRB, 0.76965E+00_JPRB, 0.76938E+00_JPRB, 0.75646E+00_JPRB, 0.75398E+00_JPRB, 0.75091E+00_JPRB ,&
&  0.74780E+00_JPRB, 0.73069E+00_JPRB, 0.74049E+00_JPRB, 0.76023E+00_JPRB, 0.82237E+00_JPRB, 0.86397E+00_JPRB, 0.85151E+00_JPRB ,&
&  0.81327E+00_JPRB, 0.79301E+00_JPRB, 0.79369E+00_JPRB, 0.81240E+00_JPRB, 0.85164E+00_JPRB, 0.95337E+00_JPRB, 0.88186E+00_JPRB ,&
&  0.84810E+00_JPRB, 0.84493E+00_JPRB, 0.83323E+00_JPRB, 0.81888E+00_JPRB, 0.83697E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.81961E+00_JPRB, 0.80363E+00_JPRB, 0.78740E+00_JPRB, 0.76471E+00_JPRB, 0.73494E+00_JPRB, 0.71704E+00_JPRB, 0.70588E+00_JPRB ,&
&  0.70035E+00_JPRB, 0.67489E+00_JPRB, 0.65464E+00_JPRB, 0.65254E+00_JPRB, 0.65617E+00_JPRB, 0.64489E+00_JPRB, 0.63871E+00_JPRB ,&
&  0.64183E+00_JPRB, 0.64335E+00_JPRB, 0.63495E+00_JPRB, 0.62802E+00_JPRB, 0.63304E+00_JPRB, 0.66900E+00_JPRB, 0.68393E+00_JPRB ,&
&  0.69050E+00_JPRB, 0.68363E+00_JPRB, 0.66290E+00_JPRB, 0.63432E+00_JPRB, 0.62789E+00_JPRB, 0.63550E+00_JPRB, 0.66453E+00_JPRB ,&
&  0.90831E+00_JPRB, 0.10796E+01_JPRB, 0.19827E+01_JPRB, 0.20848E+01_JPRB, 0.14604E+01_JPRB, 0.11224E+01_JPRB, 0.96369E+00_JPRB ,&
&  0.97939E+00_JPRB, 0.94042E+00_JPRB, 0.89206E+00_JPRB, 0.84848E+00_JPRB, 0.81168E+00_JPRB, 0.77872E+00_JPRB, 0.75280E+00_JPRB ,&
&  0.74484E+00_JPRB, 0.73243E+00_JPRB, 0.72062E+00_JPRB, 0.72000E+00_JPRB, 0.71387E+00_JPRB, 0.71366E+00_JPRB, 0.71350E+00_JPRB ,&
&  0.71653E+00_JPRB, 0.71199E+00_JPRB, 0.70392E+00_JPRB, 0.69025E+00_JPRB, 0.68741E+00_JPRB, 0.78564E+00_JPRB, 0.93213E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.11807E+01_JPRB, 0.10299E+01_JPRB, 0.94367E+00_JPRB, 0.86909E+00_JPRB, 0.82061E+00_JPRB, 0.82490E+00_JPRB ,&
&  0.83110E+00_JPRB, 0.84319E+00_JPRB, 0.83282E+00_JPRB, 0.83426E+00_JPRB, 0.83614E+00_JPRB, 0.81321E+00_JPRB, 0.80119E+00_JPRB ,&
&  0.79768E+00_JPRB, 0.79497E+00_JPRB, 0.78535E+00_JPRB, 0.77312E+00_JPRB, 0.75640E+00_JPRB, 0.74678E+00_JPRB, 0.74409E+00_JPRB ,&
&  0.74254E+00_JPRB, 0.73185E+00_JPRB, 0.74616E+00_JPRB, 0.76641E+00_JPRB, 0.80086E+00_JPRB, 0.80875E+00_JPRB, 0.80121E+00_JPRB ,&
&  0.78092E+00_JPRB, 0.76873E+00_JPRB, 0.76253E+00_JPRB, 0.79606E+00_JPRB, 0.79627E+00_JPRB, 0.87313E+00_JPRB, 0.82411E+00_JPRB ,&
&  0.81412E+00_JPRB, 0.81139E+00_JPRB, 0.79502E+00_JPRB, 0.79022E+00_JPRB, 0.80998E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.77900E+00_JPRB, 0.74577E+00_JPRB, 0.72382E+00_JPRB, 0.70408E+00_JPRB, 0.68710E+00_JPRB, 0.67481E+00_JPRB, 0.68404E+00_JPRB ,&
&  0.68211E+00_JPRB, 0.65673E+00_JPRB, 0.63748E+00_JPRB, 0.63879E+00_JPRB, 0.63522E+00_JPRB, 0.63356E+00_JPRB, 0.63125E+00_JPRB ,&
&  0.62264E+00_JPRB, 0.60993E+00_JPRB, 0.60675E+00_JPRB, 0.61419E+00_JPRB, 0.64445E+00_JPRB, 0.68213E+00_JPRB, 0.68968E+00_JPRB ,&
&  0.68254E+00_JPRB, 0.67683E+00_JPRB, 0.65692E+00_JPRB, 0.63850E+00_JPRB, 0.63526E+00_JPRB, 0.64871E+00_JPRB, 0.69028E+00_JPRB ,&
&  0.10267E+01_JPRB, 0.24666E+01_JPRB, 0.12677E+01_JPRB, 0.20161E+01_JPRB, 0.13495E+01_JPRB, 0.10174E+01_JPRB, 0.79547E+00_JPRB ,&
&  0.79476E+00_JPRB, 0.86142E+00_JPRB, 0.84909E+00_JPRB, 0.81542E+00_JPRB, 0.76280E+00_JPRB, 0.73281E+00_JPRB, 0.72145E+00_JPRB ,&
&  0.74619E+00_JPRB, 0.72661E+00_JPRB, 0.71429E+00_JPRB, 0.71352E+00_JPRB, 0.70550E+00_JPRB, 0.70663E+00_JPRB, 0.71578E+00_JPRB ,&
&  0.70960E+00_JPRB, 0.69846E+00_JPRB, 0.68996E+00_JPRB, 0.73213E+00_JPRB, 0.73738E+00_JPRB, 0.90681E+00_JPRB, 0.10599E+01_JPRB ,&
&  0.10022E+01_JPRB, 0.12420E+01_JPRB, 0.10754E+01_JPRB, 0.95975E+00_JPRB, 0.87754E+00_JPRB, 0.85084E+00_JPRB, 0.85401E+00_JPRB ,&
&  0.83628E+00_JPRB, 0.82078E+00_JPRB, 0.80600E+00_JPRB, 0.80733E+00_JPRB, 0.82333E+00_JPRB, 0.80676E+00_JPRB, 0.80903E+00_JPRB ,&
&  0.82024E+00_JPRB, 0.78953E+00_JPRB, 0.77882E+00_JPRB, 0.76927E+00_JPRB, 0.77667E+00_JPRB, 0.77945E+00_JPRB, 0.77150E+00_JPRB ,&
&  0.76112E+00_JPRB, 0.75332E+00_JPRB, 0.75468E+00_JPRB, 0.75965E+00_JPRB, 0.76422E+00_JPRB, 0.76290E+00_JPRB, 0.74083E+00_JPRB ,&
&  0.75463E+00_JPRB, 0.76034E+00_JPRB, 0.75756E+00_JPRB, 0.78198E+00_JPRB, 0.78869E+00_JPRB, 0.83265E+00_JPRB, 0.79047E+00_JPRB ,&
&  0.80092E+00_JPRB, 0.79756E+00_JPRB, 0.77558E+00_JPRB, 0.78231E+00_JPRB, 0.80264E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.73397E+00_JPRB, 0.70782E+00_JPRB, 0.68224E+00_JPRB, 0.65338E+00_JPRB, 0.63490E+00_JPRB, 0.64180E+00_JPRB, 0.66267E+00_JPRB ,&
&  0.64509E+00_JPRB, 0.62052E+00_JPRB, 0.61317E+00_JPRB, 0.62758E+00_JPRB, 0.64553E+00_JPRB, 0.64024E+00_JPRB, 0.63285E+00_JPRB ,&
&  0.61542E+00_JPRB, 0.60888E+00_JPRB, 0.59812E+00_JPRB, 0.60778E+00_JPRB, 0.65832E+00_JPRB, 0.65539E+00_JPRB, 0.64554E+00_JPRB ,&
&  0.66319E+00_JPRB, 0.65934E+00_JPRB, 0.63785E+00_JPRB, 0.63583E+00_JPRB, 0.64202E+00_JPRB, 0.67110E+00_JPRB, 0.72917E+00_JPRB ,&
&  0.11078E+01_JPRB, 0.24803E+01_JPRB, 0.13898E+01_JPRB, 0.18060E+01_JPRB, 0.12169E+01_JPRB, 0.89870E+00_JPRB, 0.68186E+00_JPRB ,&
&  0.67271E+00_JPRB, 0.81468E+00_JPRB, 0.82249E+00_JPRB, 0.78392E+00_JPRB, 0.70786E+00_JPRB, 0.67570E+00_JPRB, 0.70222E+00_JPRB ,&
&  0.72383E+00_JPRB, 0.71092E+00_JPRB, 0.71269E+00_JPRB, 0.72873E+00_JPRB, 0.73219E+00_JPRB, 0.73217E+00_JPRB, 0.73786E+00_JPRB ,&
&  0.73224E+00_JPRB, 0.73520E+00_JPRB, 0.74766E+00_JPRB, 0.75405E+00_JPRB, 0.85190E+00_JPRB, 0.10443E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.10681E+01_JPRB, 0.12258E+01_JPRB, 0.10643E+01_JPRB, 0.95699E+00_JPRB, 0.86789E+00_JPRB, 0.84952E+00_JPRB, 0.82507E+00_JPRB ,&
&  0.79530E+00_JPRB, 0.78084E+00_JPRB, 0.75694E+00_JPRB, 0.75817E+00_JPRB, 0.80001E+00_JPRB, 0.81862E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.82264E+00_JPRB, 0.82332E+00_JPRB, 0.82365E+00_JPRB, 0.80367E+00_JPRB, 0.80823E+00_JPRB, 0.80289E+00_JPRB, 0.79008E+00_JPRB ,&
&  0.78065E+00_JPRB, 0.78245E+00_JPRB, 0.76458E+00_JPRB, 0.76105E+00_JPRB, 0.75961E+00_JPRB, 0.75567E+00_JPRB, 0.72822E+00_JPRB ,&
&  0.74308E+00_JPRB, 0.75843E+00_JPRB, 0.77112E+00_JPRB, 0.76693E+00_JPRB, 0.76055E+00_JPRB, 0.79098E+00_JPRB, 0.79600E+00_JPRB ,&
&  0.81444E+00_JPRB, 0.81168E+00_JPRB, 0.77253E+00_JPRB, 0.76742E+00_JPRB, 0.77113E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.70214E+00_JPRB, 0.67029E+00_JPRB, 0.63988E+00_JPRB, 0.59329E+00_JPRB, 0.60021E+00_JPRB, 0.61175E+00_JPRB, 0.59207E+00_JPRB ,&
&  0.54867E+00_JPRB, 0.54321E+00_JPRB, 0.55634E+00_JPRB, 0.57929E+00_JPRB, 0.59829E+00_JPRB, 0.59322E+00_JPRB, 0.60104E+00_JPRB ,&
&  0.59034E+00_JPRB, 0.58664E+00_JPRB, 0.60670E+00_JPRB, 0.63944E+00_JPRB, 0.64259E+00_JPRB, 0.64015E+00_JPRB, 0.65091E+00_JPRB ,&
&  0.65575E+00_JPRB, 0.64315E+00_JPRB, 0.63485E+00_JPRB, 0.63993E+00_JPRB, 0.66964E+00_JPRB, 0.71964E+00_JPRB, 0.80161E+00_JPRB ,&
&  0.11737E+01_JPRB, 0.26361E+01_JPRB, 0.19913E+01_JPRB, 0.15566E+01_JPRB, 0.10879E+01_JPRB, 0.78473E+00_JPRB, 0.63532E+00_JPRB ,&
&  0.60445E+00_JPRB, 0.62672E+00_JPRB, 0.80177E+00_JPRB, 0.74193E+00_JPRB, 0.66984E+00_JPRB, 0.63832E+00_JPRB, 0.69438E+00_JPRB ,&
&  0.73993E+00_JPRB, 0.74994E+00_JPRB, 0.76590E+00_JPRB, 0.77452E+00_JPRB, 0.78171E+00_JPRB, 0.78257E+00_JPRB, 0.77519E+00_JPRB ,&
&  0.77548E+00_JPRB, 0.80646E+00_JPRB, 0.84429E+00_JPRB, 0.86703E+00_JPRB, 0.95754E+00_JPRB, 0.11439E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.11435E+01_JPRB, 0.10336E+01_JPRB, 0.96463E+00_JPRB, 0.82412E+00_JPRB, 0.84460E+00_JPRB, 0.79545E+00_JPRB ,&
&  0.76816E+00_JPRB, 0.73453E+00_JPRB, 0.73052E+00_JPRB, 0.74470E+00_JPRB, 0.76417E+00_JPRB, 0.78493E+00_JPRB, 0.79256E+00_JPRB ,&
&  0.80568E+00_JPRB, 0.81589E+00_JPRB, 0.81821E+00_JPRB, 0.80577E+00_JPRB, 0.80363E+00_JPRB, 0.80516E+00_JPRB, 0.81493E+00_JPRB ,&
&  0.81475E+00_JPRB, 0.82624E+00_JPRB, 0.79507E+00_JPRB, 0.78581E+00_JPRB, 0.78324E+00_JPRB, 0.77362E+00_JPRB, 0.73360E+00_JPRB ,&
&  0.73972E+00_JPRB, 0.77740E+00_JPRB, 0.78225E+00_JPRB, 0.77030E+00_JPRB, 0.76554E+00_JPRB, 0.78032E+00_JPRB, 0.78592E+00_JPRB ,&
&  0.80988E+00_JPRB, 0.81352E+00_JPRB, 0.76705E+00_JPRB, 0.74102E+00_JPRB, 0.74486E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.69358E+00_JPRB, 0.66176E+00_JPRB, 0.60461E+00_JPRB, 0.57880E+00_JPRB, 0.57585E+00_JPRB, 0.56877E+00_JPRB, 0.55527E+00_JPRB ,&
&  0.53266E+00_JPRB, 0.52565E+00_JPRB, 0.55436E+00_JPRB, 0.58322E+00_JPRB, 0.60939E+00_JPRB, 0.60705E+00_JPRB, 0.60638E+00_JPRB ,&
&  0.60734E+00_JPRB, 0.62758E+00_JPRB, 0.66711E+00_JPRB, 0.67437E+00_JPRB, 0.65934E+00_JPRB, 0.65039E+00_JPRB, 0.64745E+00_JPRB ,&
&  0.65020E+00_JPRB, 0.65414E+00_JPRB, 0.67319E+00_JPRB, 0.69528E+00_JPRB, 0.73458E+00_JPRB, 0.80467E+00_JPRB, 0.95608E+00_JPRB ,&
&  0.13295E+01_JPRB, 0.22880E+01_JPRB, 0.67733E+00_JPRB, 0.87498E+00_JPRB, 0.81681E+00_JPRB, 0.68787E+00_JPRB, 0.60359E+00_JPRB ,&
&  0.54824E+00_JPRB, 0.55959E+00_JPRB, 0.67430E+00_JPRB, 0.70974E+00_JPRB, 0.64097E+00_JPRB, 0.68894E+00_JPRB, 0.77557E+00_JPRB ,&
&  0.84362E+00_JPRB, 0.85291E+00_JPRB, 0.84902E+00_JPRB, 0.83789E+00_JPRB, 0.83887E+00_JPRB, 0.83834E+00_JPRB, 0.84437E+00_JPRB ,&
&  0.86862E+00_JPRB, 0.92001E+00_JPRB, 0.97357E+00_JPRB, 0.94031E+00_JPRB, 0.11755E+01_JPRB, 0.13409E+01_JPRB, 0.12970E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10606E+01_JPRB, 0.10125E+01_JPRB, 0.98043E+00_JPRB, 0.80664E+00_JPRB, 0.70673E+00_JPRB, 0.77791E+00_JPRB ,&
&  0.76730E+00_JPRB, 0.73952E+00_JPRB, 0.72008E+00_JPRB, 0.70682E+00_JPRB, 0.70519E+00_JPRB, 0.75013E+00_JPRB, 0.78112E+00_JPRB ,&
&  0.77965E+00_JPRB, 0.80280E+00_JPRB, 0.80606E+00_JPRB, 0.79542E+00_JPRB, 0.79988E+00_JPRB, 0.80957E+00_JPRB, 0.84126E+00_JPRB ,&
&  0.85341E+00_JPRB, 0.86653E+00_JPRB, 0.87341E+00_JPRB, 0.85314E+00_JPRB, 0.85583E+00_JPRB, 0.81940E+00_JPRB, 0.76976E+00_JPRB ,&
&  0.76914E+00_JPRB, 0.79107E+00_JPRB, 0.76033E+00_JPRB, 0.72833E+00_JPRB, 0.75306E+00_JPRB, 0.75541E+00_JPRB, 0.75828E+00_JPRB ,&
&  0.80939E+00_JPRB, 0.83858E+00_JPRB, 0.82810E+00_JPRB, 0.71302E+00_JPRB, 0.72586E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.69050E+00_JPRB, 0.62226E+00_JPRB, 0.58493E+00_JPRB, 0.57914E+00_JPRB, 0.56727E+00_JPRB, 0.57266E+00_JPRB, 0.58024E+00_JPRB ,&
&  0.57629E+00_JPRB, 0.58874E+00_JPRB, 0.58988E+00_JPRB, 0.60080E+00_JPRB, 0.61995E+00_JPRB, 0.62230E+00_JPRB, 0.64016E+00_JPRB ,&
&  0.66016E+00_JPRB, 0.68575E+00_JPRB, 0.70531E+00_JPRB, 0.66624E+00_JPRB, 0.65399E+00_JPRB, 0.64999E+00_JPRB, 0.65438E+00_JPRB ,&
&  0.67299E+00_JPRB, 0.70190E+00_JPRB, 0.73535E+00_JPRB, 0.77910E+00_JPRB, 0.84582E+00_JPRB, 0.96943E+00_JPRB, 0.12028E+01_JPRB ,&
&  0.18027E+01_JPRB, 0.14137E+01_JPRB, 0.53243E+00_JPRB, 0.68279E+00_JPRB, 0.66587E+00_JPRB, 0.63167E+00_JPRB, 0.53750E+00_JPRB ,&
&  0.52083E+00_JPRB, 0.57076E+00_JPRB, 0.62789E+00_JPRB, 0.68456E+00_JPRB, 0.72998E+00_JPRB, 0.79756E+00_JPRB, 0.86567E+00_JPRB ,&
&  0.90160E+00_JPRB, 0.91533E+00_JPRB, 0.91140E+00_JPRB, 0.91209E+00_JPRB, 0.91860E+00_JPRB, 0.93321E+00_JPRB, 0.95530E+00_JPRB ,&
&  0.99496E+00_JPRB, 0.10537E+01_JPRB, 0.11210E+01_JPRB, 0.10570E+01_JPRB, 0.13080E+01_JPRB, 0.14011E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.10476E+01_JPRB, 0.96571E+00_JPRB, 0.92682E+00_JPRB, 0.73582E+00_JPRB, 0.64459E+00_JPRB, 0.74278E+00_JPRB ,&
&  0.79053E+00_JPRB, 0.74144E+00_JPRB, 0.72216E+00_JPRB, 0.71178E+00_JPRB, 0.67152E+00_JPRB, 0.71425E+00_JPRB, 0.77441E+00_JPRB ,&
&  0.77930E+00_JPRB, 0.78168E+00_JPRB, 0.80168E+00_JPRB, 0.79197E+00_JPRB, 0.78776E+00_JPRB, 0.80042E+00_JPRB, 0.83349E+00_JPRB ,&
&  0.85391E+00_JPRB, 0.87390E+00_JPRB, 0.87790E+00_JPRB, 0.90757E+00_JPRB, 0.93476E+00_JPRB, 0.86789E+00_JPRB, 0.77879E+00_JPRB ,&
&  0.75629E+00_JPRB, 0.76557E+00_JPRB, 0.74949E+00_JPRB, 0.73880E+00_JPRB, 0.72797E+00_JPRB, 0.72496E+00_JPRB, 0.74225E+00_JPRB ,&
&  0.83564E+00_JPRB, 0.78469E+00_JPRB, 0.73592E+00_JPRB, 0.67417E+00_JPRB, 0.70375E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.60793E+00_JPRB, 0.57870E+00_JPRB, 0.57043E+00_JPRB, 0.55557E+00_JPRB, 0.56885E+00_JPRB, 0.59310E+00_JPRB, 0.61145E+00_JPRB ,&
&  0.61352E+00_JPRB, 0.61127E+00_JPRB, 0.61139E+00_JPRB, 0.62755E+00_JPRB, 0.65049E+00_JPRB, 0.67398E+00_JPRB, 0.70365E+00_JPRB ,&
&  0.72171E+00_JPRB, 0.73133E+00_JPRB, 0.70102E+00_JPRB, 0.65852E+00_JPRB, 0.65899E+00_JPRB, 0.66830E+00_JPRB, 0.68631E+00_JPRB ,&
&  0.71470E+00_JPRB, 0.74626E+00_JPRB, 0.79055E+00_JPRB, 0.87302E+00_JPRB, 0.10022E+01_JPRB, 0.11537E+01_JPRB, 0.13073E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.48407E+00_JPRB, 0.50226E+00_JPRB, 0.58342E+00_JPRB, 0.53635E+00_JPRB, 0.53982E+00_JPRB, 0.53536E+00_JPRB ,&
&  0.52351E+00_JPRB, 0.56731E+00_JPRB, 0.60608E+00_JPRB, 0.73248E+00_JPRB, 0.83398E+00_JPRB, 0.88038E+00_JPRB, 0.92073E+00_JPRB ,&
&  0.94857E+00_JPRB, 0.96788E+00_JPRB, 0.97054E+00_JPRB, 0.97216E+00_JPRB, 0.98666E+00_JPRB, 0.10112E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.11691E+01_JPRB, 0.12626E+01_JPRB, 0.11433E+01_JPRB, 0.13170E+01_JPRB, 0.14512E+01_JPRB, 0.15859E+01_JPRB ,&
&  0.15371E+01_JPRB, 0.10752E+01_JPRB, 0.86910E+00_JPRB, 0.91671E+00_JPRB, 0.81743E+00_JPRB, 0.66593E+00_JPRB, 0.77402E+00_JPRB ,&
&  0.80860E+00_JPRB, 0.81363E+00_JPRB, 0.80224E+00_JPRB, 0.72827E+00_JPRB, 0.65976E+00_JPRB, 0.64849E+00_JPRB, 0.70099E+00_JPRB ,&
&  0.72609E+00_JPRB, 0.74095E+00_JPRB, 0.74067E+00_JPRB, 0.74295E+00_JPRB, 0.72651E+00_JPRB, 0.73839E+00_JPRB, 0.77186E+00_JPRB ,&
&  0.80546E+00_JPRB, 0.83770E+00_JPRB, 0.85165E+00_JPRB, 0.87778E+00_JPRB, 0.91437E+00_JPRB, 0.86000E+00_JPRB, 0.77881E+00_JPRB ,&
&  0.75709E+00_JPRB, 0.74896E+00_JPRB, 0.72117E+00_JPRB, 0.71131E+00_JPRB, 0.71913E+00_JPRB, 0.77375E+00_JPRB, 0.88902E+00_JPRB ,&
&  0.88184E+00_JPRB, 0.78033E+00_JPRB, 0.67233E+00_JPRB, 0.64353E+00_JPRB, 0.65218E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.50898E+00_JPRB, 0.50095E+00_JPRB, 0.50162E+00_JPRB, 0.49955E+00_JPRB, 0.53120E+00_JPRB, 0.56559E+00_JPRB, 0.57516E+00_JPRB ,&
&  0.56830E+00_JPRB, 0.58373E+00_JPRB, 0.59858E+00_JPRB, 0.62680E+00_JPRB, 0.68624E+00_JPRB, 0.70931E+00_JPRB, 0.73691E+00_JPRB ,&
&  0.76422E+00_JPRB, 0.75589E+00_JPRB, 0.71889E+00_JPRB, 0.69363E+00_JPRB, 0.70339E+00_JPRB, 0.70618E+00_JPRB, 0.72382E+00_JPRB ,&
&  0.75526E+00_JPRB, 0.79014E+00_JPRB, 0.83359E+00_JPRB, 0.89062E+00_JPRB, 0.95621E+00_JPRB, 0.99828E+00_JPRB, 0.95294E+00_JPRB ,&
&  0.41598E+00_JPRB, 0.41099E+00_JPRB, 0.53490E+00_JPRB, 0.54331E+00_JPRB, 0.50505E+00_JPRB, 0.51000E+00_JPRB, 0.52868E+00_JPRB ,&
&  0.54832E+00_JPRB, 0.54198E+00_JPRB, 0.60325E+00_JPRB, 0.81852E+00_JPRB, 0.85282E+00_JPRB, 0.91088E+00_JPRB, 0.92469E+00_JPRB ,&
&  0.94990E+00_JPRB, 0.97569E+00_JPRB, 0.99397E+00_JPRB, 0.10071E+01_JPRB, 0.10288E+01_JPRB, 0.10598E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.12724E+01_JPRB, 0.13909E+01_JPRB, 0.12676E+01_JPRB, 0.13398E+01_JPRB, 0.15052E+01_JPRB, 0.18085E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.11661E+01_JPRB, 0.90298E+00_JPRB, 0.89921E+00_JPRB, 0.85563E+00_JPRB, 0.87349E+00_JPRB, 0.86456E+00_JPRB ,&
&  0.82761E+00_JPRB, 0.79757E+00_JPRB, 0.75660E+00_JPRB, 0.69257E+00_JPRB, 0.62471E+00_JPRB, 0.61654E+00_JPRB, 0.61596E+00_JPRB ,&
&  0.64776E+00_JPRB, 0.65786E+00_JPRB, 0.66412E+00_JPRB, 0.67151E+00_JPRB, 0.63313E+00_JPRB, 0.63661E+00_JPRB, 0.66673E+00_JPRB ,&
&  0.69841E+00_JPRB, 0.72094E+00_JPRB, 0.72874E+00_JPRB, 0.73711E+00_JPRB, 0.75397E+00_JPRB, 0.76244E+00_JPRB, 0.73362E+00_JPRB ,&
&  0.69677E+00_JPRB, 0.67294E+00_JPRB, 0.68816E+00_JPRB, 0.69178E+00_JPRB, 0.86629E+00_JPRB, 0.11748E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.79065E+00_JPRB, 0.62961E+00_JPRB, 0.57974E+00_JPRB, 0.55873E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.46755E+00_JPRB, 0.45707E+00_JPRB, 0.45817E+00_JPRB, 0.45779E+00_JPRB, 0.49181E+00_JPRB, 0.51451E+00_JPRB, 0.53473E+00_JPRB ,&
&  0.56449E+00_JPRB, 0.58294E+00_JPRB, 0.61358E+00_JPRB, 0.66876E+00_JPRB, 0.72610E+00_JPRB, 0.76862E+00_JPRB, 0.78320E+00_JPRB ,&
&  0.78718E+00_JPRB, 0.77788E+00_JPRB, 0.75547E+00_JPRB, 0.72708E+00_JPRB, 0.72718E+00_JPRB, 0.71805E+00_JPRB, 0.72976E+00_JPRB ,&
&  0.75471E+00_JPRB, 0.78047E+00_JPRB, 0.81455E+00_JPRB, 0.84540E+00_JPRB, 0.87122E+00_JPRB, 0.87376E+00_JPRB, 0.80734E+00_JPRB ,&
&  0.40898E+00_JPRB, 0.59800E+00_JPRB, 0.63565E+00_JPRB, 0.61986E+00_JPRB, 0.56894E+00_JPRB, 0.53344E+00_JPRB, 0.53972E+00_JPRB ,&
&  0.56735E+00_JPRB, 0.56734E+00_JPRB, 0.62007E+00_JPRB, 0.69165E+00_JPRB, 0.80748E+00_JPRB, 0.85037E+00_JPRB, 0.88238E+00_JPRB ,&
&  0.90654E+00_JPRB, 0.91971E+00_JPRB, 0.94539E+00_JPRB, 0.97278E+00_JPRB, 0.10049E+01_JPRB, 0.10377E+01_JPRB, 0.10834E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.11929E+01_JPRB, 0.12903E+01_JPRB, 0.12184E+01_JPRB, 0.11604E+01_JPRB, 0.12572E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.79470E+00_JPRB, 0.77255E+00_JPRB, 0.88113E+00_JPRB, 0.89857E+00_JPRB, 0.90222E+00_JPRB, 0.87585E+00_JPRB ,&
&  0.79473E+00_JPRB, 0.74604E+00_JPRB, 0.68696E+00_JPRB, 0.64249E+00_JPRB, 0.60306E+00_JPRB, 0.57968E+00_JPRB, 0.56368E+00_JPRB ,&
&  0.55795E+00_JPRB, 0.57281E+00_JPRB, 0.58790E+00_JPRB, 0.57899E+00_JPRB, 0.54888E+00_JPRB, 0.55184E+00_JPRB, 0.59099E+00_JPRB ,&
&  0.66256E+00_JPRB, 0.59122E+00_JPRB, 0.54295E+00_JPRB, 0.54244E+00_JPRB, 0.53971E+00_JPRB, 0.53675E+00_JPRB, 0.53243E+00_JPRB ,&
&  0.54585E+00_JPRB, 0.54805E+00_JPRB, 0.65483E+00_JPRB, 0.10326E+01_JPRB, 0.14313E+01_JPRB, 0.16840E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.97128E+00_JPRB, 0.71066E+00_JPRB, 0.56350E+00_JPRB, 0.52488E+00_JPRB, 0.50419E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.49091E+00_JPRB, 0.47376E+00_JPRB, 0.45770E+00_JPRB, 0.46971E+00_JPRB, 0.47787E+00_JPRB, 0.48303E+00_JPRB, 0.50810E+00_JPRB ,&
&  0.56068E+00_JPRB, 0.57356E+00_JPRB, 0.59971E+00_JPRB, 0.64102E+00_JPRB, 0.70062E+00_JPRB, 0.74338E+00_JPRB, 0.78210E+00_JPRB ,&
&  0.82084E+00_JPRB, 0.83926E+00_JPRB, 0.82198E+00_JPRB, 0.79780E+00_JPRB, 0.78523E+00_JPRB, 0.77994E+00_JPRB, 0.78218E+00_JPRB ,&
&  0.77473E+00_JPRB, 0.77767E+00_JPRB, 0.79377E+00_JPRB, 0.81367E+00_JPRB, 0.82335E+00_JPRB, 0.78856E+00_JPRB, 0.48799E+00_JPRB ,&
&  0.66920E+00_JPRB, 0.72468E+00_JPRB, 0.74657E+00_JPRB, 0.71396E+00_JPRB, 0.65563E+00_JPRB, 0.59271E+00_JPRB, 0.56559E+00_JPRB ,&
&  0.57126E+00_JPRB, 0.58609E+00_JPRB, 0.59734E+00_JPRB, 0.62879E+00_JPRB, 0.73132E+00_JPRB, 0.76382E+00_JPRB, 0.80486E+00_JPRB ,&
&  0.82435E+00_JPRB, 0.85044E+00_JPRB, 0.87181E+00_JPRB, 0.88845E+00_JPRB, 0.91256E+00_JPRB, 0.93615E+00_JPRB, 0.95513E+00_JPRB ,&
&  0.96078E+00_JPRB, 0.94898E+00_JPRB, 0.95720E+00_JPRB, 0.93493E+00_JPRB, 0.85636E+00_JPRB, 0.92413E+00_JPRB, 0.78798E+00_JPRB ,&
&  0.57690E+00_JPRB, 0.49710E+00_JPRB, 0.66042E+00_JPRB, 0.90038E+00_JPRB, 0.89672E+00_JPRB, 0.89500E+00_JPRB, 0.83259E+00_JPRB ,&
&  0.74237E+00_JPRB, 0.66522E+00_JPRB, 0.62654E+00_JPRB, 0.60191E+00_JPRB, 0.57686E+00_JPRB, 0.54588E+00_JPRB, 0.52094E+00_JPRB ,&
&  0.49236E+00_JPRB, 0.47366E+00_JPRB, 0.48819E+00_JPRB, 0.48255E+00_JPRB, 0.49531E+00_JPRB, 0.55197E+00_JPRB, 0.70446E+00_JPRB ,&
&  0.10835E+01_JPRB, 0.57723E+00_JPRB, 0.48876E+00_JPRB, 0.46008E+00_JPRB, 0.43130E+00_JPRB, 0.41470E+00_JPRB, 0.40666E+00_JPRB ,&
&  0.41973E+00_JPRB, 0.48389E+00_JPRB, 0.63215E+00_JPRB, 0.13526E+01_JPRB, 0.25091E+01_JPRB, 0.25725E+01_JPRB, 0.17170E+01_JPRB ,&
&  0.10238E+01_JPRB, 0.69720E+00_JPRB, 0.55725E+00_JPRB, 0.52191E+00_JPRB, 0.49658E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.53590E+00_JPRB, 0.50568E+00_JPRB, 0.49712E+00_JPRB, 0.49420E+00_JPRB, 0.48494E+00_JPRB, 0.50326E+00_JPRB, 0.51723E+00_JPRB ,&
&  0.53685E+00_JPRB, 0.56279E+00_JPRB, 0.59476E+00_JPRB, 0.62256E+00_JPRB, 0.66332E+00_JPRB, 0.70190E+00_JPRB, 0.73098E+00_JPRB ,&
&  0.75286E+00_JPRB, 0.76747E+00_JPRB, 0.77466E+00_JPRB, 0.77490E+00_JPRB, 0.76859E+00_JPRB, 0.76183E+00_JPRB, 0.75217E+00_JPRB ,&
&  0.74296E+00_JPRB, 0.74836E+00_JPRB, 0.76916E+00_JPRB, 0.79146E+00_JPRB, 0.79459E+00_JPRB, 0.77759E+00_JPRB, 0.49973E+00_JPRB ,&
&  0.83071E+00_JPRB, 0.86264E+00_JPRB, 0.83104E+00_JPRB, 0.75559E+00_JPRB, 0.69074E+00_JPRB, 0.64546E+00_JPRB, 0.60595E+00_JPRB ,&
&  0.58188E+00_JPRB, 0.57975E+00_JPRB, 0.54108E+00_JPRB, 0.57670E+00_JPRB, 0.64685E+00_JPRB, 0.66161E+00_JPRB, 0.70124E+00_JPRB ,&
&  0.73158E+00_JPRB, 0.76608E+00_JPRB, 0.79160E+00_JPRB, 0.81556E+00_JPRB, 0.83086E+00_JPRB, 0.84945E+00_JPRB, 0.84810E+00_JPRB ,&
&  0.84164E+00_JPRB, 0.81572E+00_JPRB, 0.77138E+00_JPRB, 0.71591E+00_JPRB, 0.70599E+00_JPRB, 0.62986E+00_JPRB, 0.58754E+00_JPRB ,&
&  0.45811E+00_JPRB, 0.48294E+00_JPRB, 0.74316E+00_JPRB, 0.97703E+00_JPRB, 0.90214E+00_JPRB, 0.87395E+00_JPRB, 0.78738E+00_JPRB ,&
&  0.70534E+00_JPRB, 0.64954E+00_JPRB, 0.62095E+00_JPRB, 0.60118E+00_JPRB, 0.56560E+00_JPRB, 0.53762E+00_JPRB, 0.52380E+00_JPRB ,&
&  0.51461E+00_JPRB, 0.46586E+00_JPRB, 0.44161E+00_JPRB, 0.43620E+00_JPRB, 0.42521E+00_JPRB, 0.45798E+00_JPRB, 0.44590E+00_JPRB ,&
&  0.58362E+00_JPRB, 0.54763E+00_JPRB, 0.48086E+00_JPRB, 0.48711E+00_JPRB, 0.44280E+00_JPRB, 0.43496E+00_JPRB, 0.44216E+00_JPRB ,&
&  0.47193E+00_JPRB, 0.42794E+00_JPRB, 0.92029E+00_JPRB, 0.36948E+01_JPRB, 0.36561E+01_JPRB, 0.46466E+01_JPRB, 0.24034E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.67459E+00_JPRB, 0.57687E+00_JPRB, 0.56698E+00_JPRB, 0.54862E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.55192E+00_JPRB, 0.54427E+00_JPRB, 0.52763E+00_JPRB, 0.50002E+00_JPRB, 0.48950E+00_JPRB, 0.50768E+00_JPRB, 0.53244E+00_JPRB ,&
&  0.55020E+00_JPRB, 0.57244E+00_JPRB, 0.59609E+00_JPRB, 0.62154E+00_JPRB, 0.64972E+00_JPRB, 0.67573E+00_JPRB, 0.69580E+00_JPRB ,&
&  0.70943E+00_JPRB, 0.72774E+00_JPRB, 0.74285E+00_JPRB, 0.75409E+00_JPRB, 0.75510E+00_JPRB, 0.75534E+00_JPRB, 0.75253E+00_JPRB ,&
&  0.78171E+00_JPRB, 0.80866E+00_JPRB, 0.84242E+00_JPRB, 0.87328E+00_JPRB, 0.88534E+00_JPRB, 0.84182E+00_JPRB, 0.64021E+00_JPRB ,&
&  0.95916E+00_JPRB, 0.96786E+00_JPRB, 0.87298E+00_JPRB, 0.77898E+00_JPRB, 0.71003E+00_JPRB, 0.66362E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.58994E+00_JPRB, 0.60263E+00_JPRB, 0.57298E+00_JPRB, 0.55040E+00_JPRB, 0.54929E+00_JPRB, 0.57585E+00_JPRB, 0.61020E+00_JPRB ,&
&  0.66321E+00_JPRB, 0.71673E+00_JPRB, 0.76621E+00_JPRB, 0.80145E+00_JPRB, 0.82414E+00_JPRB, 0.84370E+00_JPRB, 0.81468E+00_JPRB ,&
&  0.80730E+00_JPRB, 0.80322E+00_JPRB, 0.73696E+00_JPRB, 0.70945E+00_JPRB, 0.71871E+00_JPRB, 0.66125E+00_JPRB, 0.61020E+00_JPRB ,&
&  0.51786E+00_JPRB, 0.62910E+00_JPRB, 0.87472E+00_JPRB, 0.10088E+01_JPRB, 0.87645E+00_JPRB, 0.83907E+00_JPRB, 0.77578E+00_JPRB ,&
&  0.69996E+00_JPRB, 0.67004E+00_JPRB, 0.65769E+00_JPRB, 0.64067E+00_JPRB, 0.59630E+00_JPRB, 0.56439E+00_JPRB, 0.55801E+00_JPRB ,&
&  0.55735E+00_JPRB, 0.50259E+00_JPRB, 0.45544E+00_JPRB, 0.44803E+00_JPRB, 0.42809E+00_JPRB, 0.38530E+00_JPRB, 0.50986E+00_JPRB ,&
&  0.48182E+00_JPRB, 0.40432E+00_JPRB, 0.52737E+00_JPRB, 0.37540E+00_JPRB, 0.48026E+00_JPRB, 0.48351E+00_JPRB, 0.51191E+00_JPRB ,&
&  0.44486E+00_JPRB, 0.51497E+00_JPRB, 0.86937E+00_JPRB, 0.11946E+01_JPRB, 0.13756E+01_JPRB, 0.13586E+01_JPRB, 0.75243E+00_JPRB ,&
&  0.65333E+00_JPRB, 0.62336E+00_JPRB, 0.60158E+00_JPRB, 0.57944E+00_JPRB, 0.55538E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.56917E+00_JPRB, 0.57046E+00_JPRB, 0.55276E+00_JPRB, 0.52787E+00_JPRB, 0.51623E+00_JPRB, 0.52664E+00_JPRB, 0.54275E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.57704E+00_JPRB, 0.59970E+00_JPRB, 0.62147E+00_JPRB, 0.64058E+00_JPRB, 0.66105E+00_JPRB, 0.68519E+00_JPRB ,&
&  0.70801E+00_JPRB, 0.73637E+00_JPRB, 0.76456E+00_JPRB, 0.79234E+00_JPRB, 0.81573E+00_JPRB, 0.85749E+00_JPRB, 0.88467E+00_JPRB ,&
&  0.93020E+00_JPRB, 0.97578E+00_JPRB, 0.10305E+01_JPRB, 0.10810E+01_JPRB, 0.11399E+01_JPRB, 0.11714E+01_JPRB, 0.95109E+00_JPRB ,&
&  0.10701E+01_JPRB, 0.10147E+01_JPRB, 0.89335E+00_JPRB, 0.79904E+00_JPRB, 0.74403E+00_JPRB, 0.70483E+00_JPRB, 0.67000E+00_JPRB ,&
&  0.65635E+00_JPRB, 0.62739E+00_JPRB, 0.59478E+00_JPRB, 0.57483E+00_JPRB, 0.56577E+00_JPRB, 0.59013E+00_JPRB, 0.58442E+00_JPRB ,&
&  0.62547E+00_JPRB, 0.70021E+00_JPRB, 0.75451E+00_JPRB, 0.83748E+00_JPRB, 0.91323E+00_JPRB, 0.96778E+00_JPRB, 0.96229E+00_JPRB ,&
&  0.96462E+00_JPRB, 0.99891E+00_JPRB, 0.96381E+00_JPRB, 0.96361E+00_JPRB, 0.95252E+00_JPRB, 0.93474E+00_JPRB, 0.91333E+00_JPRB ,&
&  0.77783E+00_JPRB, 0.80497E+00_JPRB, 0.90088E+00_JPRB, 0.98682E+00_JPRB, 0.87889E+00_JPRB, 0.81400E+00_JPRB, 0.78719E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.69633E+00_JPRB, 0.69449E+00_JPRB, 0.66875E+00_JPRB, 0.64943E+00_JPRB, 0.62556E+00_JPRB, 0.61806E+00_JPRB ,&
&  0.61624E+00_JPRB, 0.54377E+00_JPRB, 0.49836E+00_JPRB, 0.46911E+00_JPRB, 0.45835E+00_JPRB, 0.43037E+00_JPRB, 0.45984E+00_JPRB ,&
&  0.48331E+00_JPRB, 0.35370E+00_JPRB, 0.44063E+00_JPRB, 0.55136E+00_JPRB, 0.55820E+00_JPRB, 0.57940E+00_JPRB, 0.62396E+00_JPRB ,&
&  0.67037E+00_JPRB, 0.69190E+00_JPRB, 0.67813E+00_JPRB, 0.69486E+00_JPRB, 0.72217E+00_JPRB, 0.70555E+00_JPRB, 0.64664E+00_JPRB ,&
&  0.65131E+00_JPRB, 0.61538E+00_JPRB, 0.59500E+00_JPRB, 0.58610E+00_JPRB, 0.57890E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.58071E+00_JPRB, 0.58893E+00_JPRB, 0.57986E+00_JPRB, 0.56248E+00_JPRB, 0.54884E+00_JPRB, 0.54449E+00_JPRB, 0.55312E+00_JPRB ,&
&  0.56305E+00_JPRB, 0.58318E+00_JPRB, 0.59922E+00_JPRB, 0.61660E+00_JPRB, 0.62542E+00_JPRB, 0.63231E+00_JPRB, 0.64624E+00_JPRB ,&
&  0.66457E+00_JPRB, 0.68451E+00_JPRB, 0.70904E+00_JPRB, 0.74320E+00_JPRB, 0.77353E+00_JPRB, 0.81068E+00_JPRB, 0.82430E+00_JPRB ,&
&  0.82196E+00_JPRB, 0.84365E+00_JPRB, 0.85749E+00_JPRB, 0.88948E+00_JPRB, 0.10571E+01_JPRB, 0.12230E+01_JPRB, 0.13758E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.10152E+01_JPRB, 0.88429E+00_JPRB, 0.79532E+00_JPRB, 0.79776E+00_JPRB, 0.77697E+00_JPRB, 0.77639E+00_JPRB ,&
&  0.78126E+00_JPRB, 0.77307E+00_JPRB, 0.77169E+00_JPRB, 0.77652E+00_JPRB, 0.78746E+00_JPRB, 0.81948E+00_JPRB, 0.79170E+00_JPRB ,&
&  0.77749E+00_JPRB, 0.80991E+00_JPRB, 0.85054E+00_JPRB, 0.95795E+00_JPRB, 0.11058E+01_JPRB, 0.12265E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.13402E+01_JPRB, 0.12702E+01_JPRB, 0.12519E+01_JPRB, 0.12338E+01_JPRB, 0.12151E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.10743E+01_JPRB, 0.10456E+01_JPRB, 0.92882E+00_JPRB, 0.87306E+00_JPRB, 0.84032E+00_JPRB, 0.81001E+00_JPRB ,&
&  0.77441E+00_JPRB, 0.76035E+00_JPRB, 0.75151E+00_JPRB, 0.72350E+00_JPRB, 0.71403E+00_JPRB, 0.70015E+00_JPRB, 0.69854E+00_JPRB ,&
&  0.68212E+00_JPRB, 0.61202E+00_JPRB, 0.56302E+00_JPRB, 0.51588E+00_JPRB, 0.45847E+00_JPRB, 0.50260E+00_JPRB, 0.51859E+00_JPRB ,&
&  0.50210E+00_JPRB, 0.43157E+00_JPRB, 0.45724E+00_JPRB, 0.64297E+00_JPRB, 0.64757E+00_JPRB, 0.65135E+00_JPRB, 0.66697E+00_JPRB ,&
&  0.68811E+00_JPRB, 0.70090E+00_JPRB, 0.70805E+00_JPRB, 0.69220E+00_JPRB, 0.68689E+00_JPRB, 0.68230E+00_JPRB, 0.66400E+00_JPRB ,&
&  0.65004E+00_JPRB, 0.63305E+00_JPRB, 0.62892E+00_JPRB, 0.59845E+00_JPRB, 0.58262E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.58164E+00_JPRB, 0.57758E+00_JPRB, 0.58944E+00_JPRB, 0.57651E+00_JPRB, 0.59918E+00_JPRB, 0.59002E+00_JPRB, 0.61795E+00_JPRB ,&
&  0.59454E+00_JPRB, 0.58929E+00_JPRB, 0.58982E+00_JPRB, 0.60062E+00_JPRB, 0.61270E+00_JPRB, 0.63052E+00_JPRB, 0.64416E+00_JPRB ,&
&  0.66674E+00_JPRB, 0.70953E+00_JPRB, 0.76768E+00_JPRB, 0.83451E+00_JPRB, 0.92256E+00_JPRB, 0.10073E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.12541E+01_JPRB, 0.12920E+01_JPRB, 0.14076E+01_JPRB, 0.17617E+01_JPRB, 0.21923E+01_JPRB, 0.21973E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.94800E+00_JPRB, 0.85320E+00_JPRB, 0.78229E+00_JPRB, 0.80056E+00_JPRB, 0.86645E+00_JPRB, 0.87894E+00_JPRB ,&
&  0.88680E+00_JPRB, 0.90091E+00_JPRB, 0.93010E+00_JPRB, 0.96111E+00_JPRB, 0.10078E+01_JPRB, 0.10711E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.11051E+01_JPRB, 0.11178E+01_JPRB, 0.12007E+01_JPRB, 0.12968E+01_JPRB, 0.14161E+01_JPRB, 0.15452E+01_JPRB ,&
&  0.16296E+01_JPRB, 0.15682E+01_JPRB, 0.12931E+01_JPRB, 0.12633E+01_JPRB, 0.13375E+01_JPRB, 0.13324E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.13708E+01_JPRB, 0.13321E+01_JPRB, 0.97310E+00_JPRB, 0.89304E+00_JPRB, 0.87408E+00_JPRB, 0.88480E+00_JPRB, 0.84638E+00_JPRB ,&
&  0.84680E+00_JPRB, 0.83732E+00_JPRB, 0.83124E+00_JPRB, 0.80946E+00_JPRB, 0.78859E+00_JPRB, 0.77157E+00_JPRB, 0.77398E+00_JPRB ,&
&  0.71894E+00_JPRB, 0.67945E+00_JPRB, 0.65081E+00_JPRB, 0.60420E+00_JPRB, 0.61214E+00_JPRB, 0.55500E+00_JPRB, 0.61327E+00_JPRB ,&
&  0.58339E+00_JPRB, 0.58956E+00_JPRB, 0.54107E+00_JPRB, 0.68004E+00_JPRB, 0.72888E+00_JPRB, 0.70273E+00_JPRB, 0.71903E+00_JPRB ,&
&  0.72839E+00_JPRB, 0.74451E+00_JPRB, 0.73249E+00_JPRB, 0.72926E+00_JPRB, 0.73445E+00_JPRB, 0.72513E+00_JPRB, 0.71154E+00_JPRB ,&
&  0.69734E+00_JPRB, 0.68180E+00_JPRB, 0.65318E+00_JPRB, 0.62555E+00_JPRB, 0.60050E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.62485E+00_JPRB, 0.62437E+00_JPRB, 0.63139E+00_JPRB, 0.64255E+00_JPRB, 0.64109E+00_JPRB, 0.63889E+00_JPRB, 0.63427E+00_JPRB ,&
&  0.60795E+00_JPRB, 0.60565E+00_JPRB, 0.59583E+00_JPRB, 0.60349E+00_JPRB, 0.62848E+00_JPRB, 0.65348E+00_JPRB, 0.69530E+00_JPRB ,&
&  0.73786E+00_JPRB, 0.79347E+00_JPRB, 0.87792E+00_JPRB, 0.10557E+01_JPRB, 0.12525E+01_JPRB, 0.13923E+01_JPRB, 0.15547E+01_JPRB ,&
&  0.16067E+01_JPRB, 0.16505E+01_JPRB, 0.17489E+01_JPRB, 0.18814E+01_JPRB, 0.19150E+01_JPRB, 0.19170E+01_JPRB, 0.18212E+01_JPRB ,&
&  0.13125E+01_JPRB, 0.91507E+00_JPRB, 0.91982E+00_JPRB, 0.85242E+00_JPRB, 0.85609E+00_JPRB, 0.86571E+00_JPRB, 0.88424E+00_JPRB ,&
&  0.92266E+00_JPRB, 0.95544E+00_JPRB, 0.98948E+00_JPRB, 0.10206E+01_JPRB, 0.10724E+01_JPRB, 0.11319E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.11631E+01_JPRB, 0.11864E+01_JPRB, 0.12152E+01_JPRB, 0.11883E+01_JPRB, 0.12182E+01_JPRB, 0.12642E+01_JPRB, 0.13225E+01_JPRB ,&
&  0.13728E+01_JPRB, 0.13238E+01_JPRB, 0.10954E+01_JPRB, 0.11032E+01_JPRB, 0.12390E+01_JPRB, 0.11592E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.13356E+01_JPRB, 0.14118E+01_JPRB, 0.91504E+00_JPRB, 0.79743E+00_JPRB, 0.82570E+00_JPRB, 0.85333E+00_JPRB, 0.89470E+00_JPRB ,&
&  0.87130E+00_JPRB, 0.87255E+00_JPRB, 0.88679E+00_JPRB, 0.88522E+00_JPRB, 0.88489E+00_JPRB, 0.85719E+00_JPRB, 0.83536E+00_JPRB ,&
&  0.75875E+00_JPRB, 0.71220E+00_JPRB, 0.69377E+00_JPRB, 0.67495E+00_JPRB, 0.67266E+00_JPRB, 0.68253E+00_JPRB, 0.68809E+00_JPRB ,&
&  0.67568E+00_JPRB, 0.67282E+00_JPRB, 0.72434E+00_JPRB, 0.70645E+00_JPRB, 0.69442E+00_JPRB, 0.75542E+00_JPRB, 0.73896E+00_JPRB ,&
&  0.76115E+00_JPRB, 0.76493E+00_JPRB, 0.76174E+00_JPRB, 0.75505E+00_JPRB, 0.74978E+00_JPRB, 0.73678E+00_JPRB, 0.72267E+00_JPRB ,&
&  0.70613E+00_JPRB, 0.68657E+00_JPRB, 0.66621E+00_JPRB, 0.64859E+00_JPRB, 0.63404E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.62718E+00_JPRB, 0.63327E+00_JPRB, 0.64592E+00_JPRB, 0.65093E+00_JPRB, 0.64972E+00_JPRB, 0.65810E+00_JPRB, 0.65677E+00_JPRB ,&
&  0.64592E+00_JPRB, 0.63624E+00_JPRB, 0.62354E+00_JPRB, 0.63560E+00_JPRB, 0.65685E+00_JPRB, 0.69868E+00_JPRB, 0.75259E+00_JPRB ,&
&  0.82936E+00_JPRB, 0.94276E+00_JPRB, 0.10913E+01_JPRB, 0.12073E+01_JPRB, 0.13593E+01_JPRB, 0.14237E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.15575E+01_JPRB, 0.15877E+01_JPRB, 0.15880E+01_JPRB, 0.15323E+01_JPRB, 0.14976E+01_JPRB, 0.15016E+01_JPRB ,&
&  0.12364E+01_JPRB, 0.95560E+00_JPRB, 0.88774E+00_JPRB, 0.84929E+00_JPRB, 0.83020E+00_JPRB, 0.81997E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.88080E+00_JPRB, 0.92166E+00_JPRB, 0.97458E+00_JPRB, 0.10180E+01_JPRB, 0.10599E+01_JPRB, 0.10893E+01_JPRB, 0.11014E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.11533E+01_JPRB, 0.11994E+01_JPRB, 0.11343E+01_JPRB, 0.11887E+01_JPRB, 0.11713E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.10119E+01_JPRB, 0.10269E+01_JPRB, 0.10372E+01_JPRB, 0.10529E+01_JPRB, 0.10417E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11838E+01_JPRB, 0.93649E+00_JPRB, 0.84231E+00_JPRB, 0.87361E+00_JPRB, 0.79719E+00_JPRB, 0.91544E+00_JPRB ,&
&  0.91018E+00_JPRB, 0.90740E+00_JPRB, 0.91201E+00_JPRB, 0.94417E+00_JPRB, 0.97544E+00_JPRB, 0.99496E+00_JPRB, 0.92257E+00_JPRB ,&
&  0.86100E+00_JPRB, 0.81792E+00_JPRB, 0.79642E+00_JPRB, 0.77729E+00_JPRB, 0.77912E+00_JPRB, 0.78704E+00_JPRB, 0.78691E+00_JPRB ,&
&  0.78718E+00_JPRB, 0.77316E+00_JPRB, 0.79908E+00_JPRB, 0.78388E+00_JPRB, 0.75995E+00_JPRB, 0.80985E+00_JPRB, 0.76603E+00_JPRB ,&
&  0.75540E+00_JPRB, 0.75564E+00_JPRB, 0.74381E+00_JPRB, 0.72434E+00_JPRB, 0.71352E+00_JPRB, 0.69466E+00_JPRB, 0.68551E+00_JPRB ,&
&  0.68052E+00_JPRB, 0.66631E+00_JPRB, 0.65424E+00_JPRB, 0.64941E+00_JPRB, 0.63342E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.63063E+00_JPRB, 0.62922E+00_JPRB, 0.64205E+00_JPRB, 0.64446E+00_JPRB, 0.64257E+00_JPRB, 0.64910E+00_JPRB, 0.65593E+00_JPRB ,&
&  0.65477E+00_JPRB, 0.65603E+00_JPRB, 0.65311E+00_JPRB, 0.65342E+00_JPRB, 0.65810E+00_JPRB, 0.68413E+00_JPRB, 0.73423E+00_JPRB ,&
&  0.80082E+00_JPRB, 0.90709E+00_JPRB, 0.10496E+01_JPRB, 0.11818E+01_JPRB, 0.13292E+01_JPRB, 0.14007E+01_JPRB, 0.13695E+01_JPRB ,&
&  0.14126E+01_JPRB, 0.14623E+01_JPRB, 0.14376E+01_JPRB, 0.13976E+01_JPRB, 0.11252E+01_JPRB, 0.11545E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.96217E+00_JPRB, 0.86751E+00_JPRB, 0.83650E+00_JPRB, 0.79878E+00_JPRB, 0.78510E+00_JPRB, 0.78865E+00_JPRB ,&
&  0.80955E+00_JPRB, 0.84516E+00_JPRB, 0.89688E+00_JPRB, 0.96612E+00_JPRB, 0.10136E+01_JPRB, 0.10304E+01_JPRB, 0.10425E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10534E+01_JPRB, 0.10544E+01_JPRB, 0.10379E+01_JPRB, 0.10264E+01_JPRB, 0.10036E+01_JPRB, 0.99650E+00_JPRB ,&
&  0.99425E+00_JPRB, 0.93301E+00_JPRB, 0.92974E+00_JPRB, 0.94197E+00_JPRB, 0.90587E+00_JPRB, 0.86665E+00_JPRB, 0.89189E+00_JPRB ,&
&  0.98249E+00_JPRB, 0.11392E+01_JPRB, 0.95362E+00_JPRB, 0.98701E+00_JPRB, 0.11074E+01_JPRB, 0.97361E+00_JPRB, 0.90646E+00_JPRB ,&
&  0.93266E+00_JPRB, 0.94171E+00_JPRB, 0.94870E+00_JPRB, 0.97749E+00_JPRB, 0.10258E+01_JPRB, 0.10946E+01_JPRB, 0.97383E+00_JPRB ,&
&  0.95613E+00_JPRB, 0.90364E+00_JPRB, 0.90311E+00_JPRB, 0.89213E+00_JPRB, 0.86597E+00_JPRB, 0.82491E+00_JPRB, 0.80690E+00_JPRB ,&
&  0.77854E+00_JPRB, 0.85259E+00_JPRB, 0.94885E+00_JPRB, 0.10672E+01_JPRB, 0.12362E+01_JPRB, 0.84640E+00_JPRB, 0.83256E+00_JPRB ,&
&  0.78703E+00_JPRB, 0.77021E+00_JPRB, 0.75274E+00_JPRB, 0.73299E+00_JPRB, 0.71565E+00_JPRB, 0.69960E+00_JPRB, 0.69110E+00_JPRB ,&
&  0.68288E+00_JPRB, 0.66856E+00_JPRB, 0.65881E+00_JPRB, 0.65453E+00_JPRB, 0.63816E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.64771E+00_JPRB, 0.64398E+00_JPRB, 0.65013E+00_JPRB, 0.66030E+00_JPRB, 0.67021E+00_JPRB, 0.66972E+00_JPRB, 0.66182E+00_JPRB ,&
&  0.64187E+00_JPRB, 0.63668E+00_JPRB, 0.64091E+00_JPRB, 0.64582E+00_JPRB, 0.63611E+00_JPRB, 0.63285E+00_JPRB, 0.65214E+00_JPRB ,&
&  0.70105E+00_JPRB, 0.78346E+00_JPRB, 0.89747E+00_JPRB, 0.10335E+01_JPRB, 0.12125E+01_JPRB, 0.13437E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.14810E+01_JPRB, 0.15246E+01_JPRB, 0.13628E+01_JPRB, 0.97297E+00_JPRB, 0.84744E+00_JPRB, 0.97158E+00_JPRB, 0.93856E+00_JPRB ,&
&  0.89432E+00_JPRB, 0.87806E+00_JPRB, 0.86400E+00_JPRB, 0.84488E+00_JPRB, 0.81310E+00_JPRB, 0.79040E+00_JPRB, 0.76949E+00_JPRB ,&
&  0.76392E+00_JPRB, 0.77384E+00_JPRB, 0.80785E+00_JPRB, 0.87498E+00_JPRB, 0.94173E+00_JPRB, 0.98080E+00_JPRB, 0.99747E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.99167E+00_JPRB, 0.98517E+00_JPRB, 0.97100E+00_JPRB, 0.94500E+00_JPRB, 0.94038E+00_JPRB, 0.92894E+00_JPRB ,&
&  0.91749E+00_JPRB, 0.89866E+00_JPRB, 0.88229E+00_JPRB, 0.87783E+00_JPRB, 0.86333E+00_JPRB, 0.78453E+00_JPRB, 0.79690E+00_JPRB ,&
&  0.87769E+00_JPRB, 0.10041E+01_JPRB, 0.99829E+00_JPRB, 0.11226E+01_JPRB, 0.74735E+00_JPRB, 0.83225E+00_JPRB, 0.90435E+00_JPRB ,&
&  0.88764E+00_JPRB, 0.92804E+00_JPRB, 0.94604E+00_JPRB, 0.96959E+00_JPRB, 0.10119E+01_JPRB, 0.99825E+00_JPRB, 0.10658E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.97727E+00_JPRB, 0.96750E+00_JPRB, 0.98188E+00_JPRB, 0.95509E+00_JPRB, 0.88153E+00_JPRB, 0.83194E+00_JPRB ,&
&  0.77665E+00_JPRB, 0.84836E+00_JPRB, 0.86298E+00_JPRB, 0.91742E+00_JPRB, 0.96616E+00_JPRB, 0.78895E+00_JPRB, 0.79877E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.78437E+00_JPRB, 0.77669E+00_JPRB, 0.75313E+00_JPRB, 0.73684E+00_JPRB, 0.72210E+00_JPRB, 0.71061E+00_JPRB ,&
&  0.70641E+00_JPRB, 0.69769E+00_JPRB, 0.68447E+00_JPRB, 0.67155E+00_JPRB, 0.65835E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.67095E+00_JPRB, 0.67074E+00_JPRB, 0.67919E+00_JPRB, 0.68398E+00_JPRB, 0.69397E+00_JPRB, 0.70475E+00_JPRB, 0.69574E+00_JPRB ,&
&  0.64561E+00_JPRB, 0.60713E+00_JPRB, 0.60216E+00_JPRB, 0.61005E+00_JPRB, 0.61405E+00_JPRB, 0.61137E+00_JPRB, 0.61791E+00_JPRB ,&
&  0.63896E+00_JPRB, 0.67941E+00_JPRB, 0.74456E+00_JPRB, 0.88164E+00_JPRB, 0.10725E+01_JPRB, 0.12740E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.14058E+01_JPRB, 0.10534E+01_JPRB, 0.12764E+01_JPRB, 0.12424E+01_JPRB, 0.10548E+01_JPRB, 0.97346E+00_JPRB, 0.94599E+00_JPRB ,&
&  0.91934E+00_JPRB, 0.91953E+00_JPRB, 0.91442E+00_JPRB, 0.89283E+00_JPRB, 0.83426E+00_JPRB, 0.80720E+00_JPRB, 0.77624E+00_JPRB ,&
&  0.76013E+00_JPRB, 0.75214E+00_JPRB, 0.76145E+00_JPRB, 0.80317E+00_JPRB, 0.85600E+00_JPRB, 0.91497E+00_JPRB, 0.96480E+00_JPRB ,&
&  0.96838E+00_JPRB, 0.94283E+00_JPRB, 0.93243E+00_JPRB, 0.90215E+00_JPRB, 0.90946E+00_JPRB, 0.91450E+00_JPRB, 0.90226E+00_JPRB ,&
&  0.89215E+00_JPRB, 0.87638E+00_JPRB, 0.88975E+00_JPRB, 0.90009E+00_JPRB, 0.89960E+00_JPRB, 0.82343E+00_JPRB, 0.84274E+00_JPRB ,&
&  0.85147E+00_JPRB, 0.90674E+00_JPRB, 0.96703E+00_JPRB, 0.10537E+01_JPRB, 0.81301E+00_JPRB, 0.88151E+00_JPRB, 0.81220E+00_JPRB ,&
&  0.85465E+00_JPRB, 0.86106E+00_JPRB, 0.89000E+00_JPRB, 0.90654E+00_JPRB, 0.95172E+00_JPRB, 0.97204E+00_JPRB, 0.11005E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10611E+01_JPRB, 0.10258E+01_JPRB, 0.10416E+01_JPRB, 0.97070E+00_JPRB, 0.92447E+00_JPRB, 0.89326E+00_JPRB ,&
&  0.92190E+00_JPRB, 0.90678E+00_JPRB, 0.89654E+00_JPRB, 0.89934E+00_JPRB, 0.93250E+00_JPRB, 0.82192E+00_JPRB, 0.79479E+00_JPRB ,&
&  0.77963E+00_JPRB, 0.79011E+00_JPRB, 0.78749E+00_JPRB, 0.77675E+00_JPRB, 0.76476E+00_JPRB, 0.74601E+00_JPRB, 0.72810E+00_JPRB ,&
&  0.72069E+00_JPRB, 0.71240E+00_JPRB, 0.70351E+00_JPRB, 0.69500E+00_JPRB, 0.68312E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.68991E+00_JPRB, 0.68436E+00_JPRB, 0.69345E+00_JPRB, 0.71405E+00_JPRB, 0.73464E+00_JPRB, 0.78935E+00_JPRB, 0.84376E+00_JPRB ,&
&  0.97595E+00_JPRB, 0.68393E+00_JPRB, 0.60045E+00_JPRB, 0.57699E+00_JPRB, 0.58033E+00_JPRB, 0.58818E+00_JPRB, 0.59834E+00_JPRB ,&
&  0.61110E+00_JPRB, 0.63824E+00_JPRB, 0.67191E+00_JPRB, 0.72368E+00_JPRB, 0.89069E+00_JPRB, 0.11141E+01_JPRB, 0.12190E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.12848E+01_JPRB, 0.15294E+01_JPRB, 0.13166E+01_JPRB, 0.11936E+01_JPRB, 0.11725E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.96748E+00_JPRB, 0.93571E+00_JPRB, 0.88662E+00_JPRB, 0.85594E+00_JPRB, 0.83000E+00_JPRB, 0.80665E+00_JPRB ,&
&  0.77968E+00_JPRB, 0.76041E+00_JPRB, 0.75413E+00_JPRB, 0.77475E+00_JPRB, 0.80480E+00_JPRB, 0.84840E+00_JPRB, 0.91601E+00_JPRB ,&
&  0.95065E+00_JPRB, 0.92307E+00_JPRB, 0.89783E+00_JPRB, 0.86357E+00_JPRB, 0.88452E+00_JPRB, 0.89829E+00_JPRB, 0.89539E+00_JPRB ,&
&  0.88710E+00_JPRB, 0.91282E+00_JPRB, 0.93065E+00_JPRB, 0.90235E+00_JPRB, 0.84249E+00_JPRB, 0.90254E+00_JPRB, 0.85604E+00_JPRB ,&
&  0.86802E+00_JPRB, 0.85221E+00_JPRB, 0.97108E+00_JPRB, 0.82992E+00_JPRB, 0.91814E+00_JPRB, 0.95837E+00_JPRB, 0.91196E+00_JPRB ,&
&  0.83857E+00_JPRB, 0.82587E+00_JPRB, 0.81142E+00_JPRB, 0.81634E+00_JPRB, 0.87557E+00_JPRB, 0.90433E+00_JPRB, 0.10438E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.11442E+01_JPRB, 0.11039E+01_JPRB, 0.10471E+01_JPRB, 0.93871E+00_JPRB, 0.87870E+00_JPRB, 0.87216E+00_JPRB ,&
&  0.94715E+00_JPRB, 0.97180E+00_JPRB, 0.96118E+00_JPRB, 0.92260E+00_JPRB, 0.90698E+00_JPRB, 0.86629E+00_JPRB, 0.85024E+00_JPRB ,&
&  0.83614E+00_JPRB, 0.82278E+00_JPRB, 0.80627E+00_JPRB, 0.79905E+00_JPRB, 0.78194E+00_JPRB, 0.76024E+00_JPRB, 0.73162E+00_JPRB ,&
&  0.71824E+00_JPRB, 0.71089E+00_JPRB, 0.70545E+00_JPRB, 0.70317E+00_JPRB, 0.69964E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.69269E+00_JPRB, 0.67954E+00_JPRB, 0.66344E+00_JPRB, 0.66590E+00_JPRB, 0.67039E+00_JPRB, 0.69012E+00_JPRB, 0.73008E+00_JPRB ,&
&  0.81555E+00_JPRB, 0.67965E+00_JPRB, 0.61197E+00_JPRB, 0.56644E+00_JPRB, 0.55381E+00_JPRB, 0.56273E+00_JPRB, 0.57050E+00_JPRB ,&
&  0.58506E+00_JPRB, 0.60123E+00_JPRB, 0.62493E+00_JPRB, 0.62761E+00_JPRB, 0.72302E+00_JPRB, 0.89837E+00_JPRB, 0.89220E+00_JPRB ,&
&  0.95382E+00_JPRB, 0.16372E+01_JPRB, 0.15222E+01_JPRB, 0.14261E+01_JPRB, 0.13321E+01_JPRB, 0.13559E+01_JPRB, 0.11966E+01_JPRB ,&
&  0.10691E+01_JPRB, 0.10189E+01_JPRB, 0.99568E+00_JPRB, 0.93959E+00_JPRB, 0.90376E+00_JPRB, 0.85479E+00_JPRB, 0.83174E+00_JPRB ,&
&  0.80801E+00_JPRB, 0.78984E+00_JPRB, 0.76767E+00_JPRB, 0.76948E+00_JPRB, 0.78737E+00_JPRB, 0.81607E+00_JPRB, 0.86590E+00_JPRB ,&
&  0.91751E+00_JPRB, 0.92043E+00_JPRB, 0.88524E+00_JPRB, 0.88331E+00_JPRB, 0.89520E+00_JPRB, 0.91575E+00_JPRB, 0.92369E+00_JPRB ,&
&  0.89497E+00_JPRB, 0.87555E+00_JPRB, 0.96889E+00_JPRB, 0.99379E+00_JPRB, 0.98005E+00_JPRB, 0.10042E+01_JPRB, 0.90172E+00_JPRB ,&
&  0.94100E+00_JPRB, 0.86204E+00_JPRB, 0.92693E+00_JPRB, 0.82231E+00_JPRB, 0.92229E+00_JPRB, 0.99506E+00_JPRB, 0.96424E+00_JPRB ,&
&  0.86130E+00_JPRB, 0.86039E+00_JPRB, 0.80841E+00_JPRB, 0.79090E+00_JPRB, 0.82330E+00_JPRB, 0.87741E+00_JPRB, 0.93229E+00_JPRB ,&
&  0.99605E+00_JPRB, 0.10975E+01_JPRB, 0.11762E+01_JPRB, 0.99259E+00_JPRB, 0.90981E+00_JPRB, 0.84581E+00_JPRB, 0.82947E+00_JPRB ,&
&  0.93613E+00_JPRB, 0.96975E+00_JPRB, 0.10124E+01_JPRB, 0.98669E+00_JPRB, 0.91687E+00_JPRB, 0.89399E+00_JPRB, 0.89372E+00_JPRB ,&
&  0.88490E+00_JPRB, 0.87843E+00_JPRB, 0.84049E+00_JPRB, 0.80192E+00_JPRB, 0.77637E+00_JPRB, 0.75926E+00_JPRB, 0.72419E+00_JPRB ,&
&  0.71293E+00_JPRB, 0.70207E+00_JPRB, 0.69480E+00_JPRB, 0.69715E+00_JPRB, 0.69579E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.67852E+00_JPRB, 0.66507E+00_JPRB, 0.65751E+00_JPRB, 0.64606E+00_JPRB, 0.62739E+00_JPRB, 0.63504E+00_JPRB, 0.69788E+00_JPRB ,&
&  0.70631E+00_JPRB, 0.63029E+00_JPRB, 0.59130E+00_JPRB, 0.56274E+00_JPRB, 0.54572E+00_JPRB, 0.54309E+00_JPRB, 0.55299E+00_JPRB ,&
&  0.56002E+00_JPRB, 0.57553E+00_JPRB, 0.58454E+00_JPRB, 0.60120E+00_JPRB, 0.65389E+00_JPRB, 0.77975E+00_JPRB, 0.62077E+00_JPRB ,&
&  0.84885E+00_JPRB, 0.14190E+01_JPRB, 0.15570E+01_JPRB, 0.15665E+01_JPRB, 0.15403E+01_JPRB, 0.15128E+01_JPRB, 0.13753E+01_JPRB ,&
&  0.12220E+01_JPRB, 0.11293E+01_JPRB, 0.10496E+01_JPRB, 0.10085E+01_JPRB, 0.98192E+00_JPRB, 0.92834E+00_JPRB, 0.87376E+00_JPRB ,&
&  0.83740E+00_JPRB, 0.80456E+00_JPRB, 0.77549E+00_JPRB, 0.76484E+00_JPRB, 0.77113E+00_JPRB, 0.79259E+00_JPRB, 0.83078E+00_JPRB ,&
&  0.87797E+00_JPRB, 0.91491E+00_JPRB, 0.92414E+00_JPRB, 0.87421E+00_JPRB, 0.89514E+00_JPRB, 0.90844E+00_JPRB, 0.93202E+00_JPRB ,&
&  0.89552E+00_JPRB, 0.87779E+00_JPRB, 0.10405E+01_JPRB, 0.11935E+01_JPRB, 0.12454E+01_JPRB, 0.11668E+01_JPRB, 0.10419E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.96255E+00_JPRB, 0.91022E+00_JPRB, 0.84331E+00_JPRB, 0.91169E+00_JPRB, 0.99935E+00_JPRB, 0.10534E+01_JPRB ,&
&  0.98286E+00_JPRB, 0.91108E+00_JPRB, 0.84512E+00_JPRB, 0.79282E+00_JPRB, 0.79664E+00_JPRB, 0.82972E+00_JPRB, 0.92321E+00_JPRB ,&
&  0.10158E+01_JPRB, 0.11449E+01_JPRB, 0.11350E+01_JPRB, 0.94917E+00_JPRB, 0.86903E+00_JPRB, 0.80191E+00_JPRB, 0.74727E+00_JPRB ,&
&  0.86518E+00_JPRB, 0.89328E+00_JPRB, 0.86234E+00_JPRB, 0.96244E+00_JPRB, 0.88930E+00_JPRB, 0.88492E+00_JPRB, 0.92744E+00_JPRB ,&
&  0.91389E+00_JPRB, 0.89212E+00_JPRB, 0.86437E+00_JPRB, 0.84336E+00_JPRB, 0.79707E+00_JPRB, 0.74657E+00_JPRB, 0.71414E+00_JPRB ,&
&  0.69998E+00_JPRB, 0.69715E+00_JPRB, 0.69111E+00_JPRB, 0.68899E+00_JPRB, 0.68551E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.64868E+00_JPRB, 0.64104E+00_JPRB, 0.63117E+00_JPRB, 0.62195E+00_JPRB, 0.60237E+00_JPRB, 0.59079E+00_JPRB, 0.62217E+00_JPRB ,&
&  0.62185E+00_JPRB, 0.59355E+00_JPRB, 0.55051E+00_JPRB, 0.53917E+00_JPRB, 0.53306E+00_JPRB, 0.52844E+00_JPRB, 0.53506E+00_JPRB ,&
&  0.54157E+00_JPRB, 0.56000E+00_JPRB, 0.59492E+00_JPRB, 0.61050E+00_JPRB, 0.57817E+00_JPRB, 0.52701E+00_JPRB, 0.59221E+00_JPRB ,&
&  0.10417E+01_JPRB, 0.15019E+01_JPRB, 0.16111E+01_JPRB, 0.17355E+01_JPRB, 0.18211E+01_JPRB, 0.19185E+01_JPRB, 0.17287E+01_JPRB ,&
&  0.15112E+01_JPRB, 0.13720E+01_JPRB, 0.12475E+01_JPRB, 0.11708E+01_JPRB, 0.10948E+01_JPRB, 0.10195E+01_JPRB, 0.96266E+00_JPRB ,&
&  0.90661E+00_JPRB, 0.86698E+00_JPRB, 0.82257E+00_JPRB, 0.77484E+00_JPRB, 0.77404E+00_JPRB, 0.79319E+00_JPRB, 0.81202E+00_JPRB ,&
&  0.83835E+00_JPRB, 0.89163E+00_JPRB, 0.91319E+00_JPRB, 0.87105E+00_JPRB, 0.87261E+00_JPRB, 0.89128E+00_JPRB, 0.95610E+00_JPRB ,&
&  0.98325E+00_JPRB, 0.10718E+01_JPRB, 0.11803E+01_JPRB, 0.12746E+01_JPRB, 0.13791E+01_JPRB, 0.13826E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.11604E+01_JPRB, 0.10184E+01_JPRB, 0.85669E+00_JPRB, 0.86583E+00_JPRB, 0.94071E+00_JPRB, 0.10646E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.89109E+00_JPRB, 0.78803E+00_JPRB, 0.75378E+00_JPRB, 0.75586E+00_JPRB, 0.81265E+00_JPRB, 0.83203E+00_JPRB, 0.92983E+00_JPRB ,&
&  0.88885E+00_JPRB, 0.66821E+00_JPRB, 0.59695E+00_JPRB, 0.70097E+00_JPRB, 0.64643E+00_JPRB, 0.60757E+00_JPRB, 0.70971E+00_JPRB ,&
&  0.78778E+00_JPRB, 0.86204E+00_JPRB, 0.84274E+00_JPRB, 0.80066E+00_JPRB, 0.82634E+00_JPRB, 0.83909E+00_JPRB, 0.96898E+00_JPRB ,&
&  0.91432E+00_JPRB, 0.86559E+00_JPRB, 0.85424E+00_JPRB, 0.84526E+00_JPRB, 0.80243E+00_JPRB, 0.74341E+00_JPRB, 0.69676E+00_JPRB ,&
&  0.67843E+00_JPRB, 0.67033E+00_JPRB, 0.66095E+00_JPRB, 0.65587E+00_JPRB, 0.65151E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.61054E+00_JPRB, 0.61154E+00_JPRB, 0.60340E+00_JPRB, 0.59470E+00_JPRB, 0.57955E+00_JPRB, 0.56387E+00_JPRB, 0.57076E+00_JPRB ,&
&  0.56347E+00_JPRB, 0.54570E+00_JPRB, 0.52425E+00_JPRB, 0.51807E+00_JPRB, 0.51624E+00_JPRB, 0.51684E+00_JPRB, 0.52461E+00_JPRB ,&
&  0.53257E+00_JPRB, 0.55149E+00_JPRB, 0.61192E+00_JPRB, 0.64399E+00_JPRB, 0.57899E+00_JPRB, 0.50743E+00_JPRB, 0.61572E+00_JPRB ,&
&  0.10924E+01_JPRB, 0.16435E+01_JPRB, 0.19271E+01_JPRB, 0.21517E+01_JPRB, 0.21710E+01_JPRB, 0.21979E+01_JPRB, 0.20767E+01_JPRB ,&
&  0.18928E+01_JPRB, 0.17245E+01_JPRB, 0.15543E+01_JPRB, 0.13575E+01_JPRB, 0.11924E+01_JPRB, 0.10859E+01_JPRB, 0.10028E+01_JPRB ,&
&  0.95375E+00_JPRB, 0.91314E+00_JPRB, 0.87086E+00_JPRB, 0.81676E+00_JPRB, 0.78893E+00_JPRB, 0.79559E+00_JPRB, 0.80175E+00_JPRB ,&
&  0.80784E+00_JPRB, 0.85855E+00_JPRB, 0.89669E+00_JPRB, 0.89755E+00_JPRB, 0.75537E+00_JPRB, 0.86991E+00_JPRB, 0.92502E+00_JPRB ,&
&  0.10190E+01_JPRB, 0.11271E+01_JPRB, 0.11575E+01_JPRB, 0.14015E+01_JPRB, 0.15624E+01_JPRB, 0.14928E+01_JPRB, 0.13881E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.11413E+01_JPRB, 0.95998E+00_JPRB, 0.98134E+00_JPRB, 0.10453E+01_JPRB, 0.11113E+01_JPRB, 0.82302E+00_JPRB ,&
&  0.77660E+00_JPRB, 0.93028E+00_JPRB, 0.85495E+00_JPRB, 0.71890E+00_JPRB, 0.82019E+00_JPRB, 0.83931E+00_JPRB, 0.68798E+00_JPRB ,&
&  0.42941E+00_JPRB, 0.37859E+00_JPRB, 0.39145E+00_JPRB, 0.41198E+00_JPRB, 0.39706E+00_JPRB, 0.46828E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.72470E+00_JPRB, 0.84562E+00_JPRB, 0.83936E+00_JPRB, 0.80292E+00_JPRB, 0.79752E+00_JPRB, 0.82732E+00_JPRB, 0.10635E+01_JPRB ,&
&  0.94595E+00_JPRB, 0.89831E+00_JPRB, 0.85045E+00_JPRB, 0.82007E+00_JPRB, 0.78132E+00_JPRB, 0.73014E+00_JPRB, 0.69312E+00_JPRB ,&
&  0.67039E+00_JPRB, 0.65826E+00_JPRB, 0.64158E+00_JPRB, 0.62765E+00_JPRB, 0.61337E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.58303E+00_JPRB, 0.57485E+00_JPRB, 0.56688E+00_JPRB, 0.55787E+00_JPRB, 0.54432E+00_JPRB, 0.53277E+00_JPRB, 0.53114E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.50341E+00_JPRB, 0.49008E+00_JPRB, 0.48577E+00_JPRB, 0.49275E+00_JPRB, 0.50811E+00_JPRB, 0.51887E+00_JPRB ,&
&  0.53479E+00_JPRB, 0.57495E+00_JPRB, 0.70113E+00_JPRB, 0.68448E+00_JPRB, 0.66510E+00_JPRB, 0.63962E+00_JPRB, 0.68545E+00_JPRB ,&
&  0.10021E+01_JPRB, 0.15218E+01_JPRB, 0.19454E+01_JPRB, 0.22576E+01_JPRB, 0.25965E+01_JPRB, 0.26974E+01_JPRB, 0.25929E+01_JPRB ,&
&  0.23155E+01_JPRB, 0.20626E+01_JPRB, 0.17984E+01_JPRB, 0.15095E+01_JPRB, 0.12920E+01_JPRB, 0.11518E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.96289E+00_JPRB, 0.92706E+00_JPRB, 0.88611E+00_JPRB, 0.84807E+00_JPRB, 0.81686E+00_JPRB, 0.81675E+00_JPRB, 0.82095E+00_JPRB ,&
&  0.81743E+00_JPRB, 0.83020E+00_JPRB, 0.86095E+00_JPRB, 0.86392E+00_JPRB, 0.76410E+00_JPRB, 0.68475E+00_JPRB, 0.80656E+00_JPRB ,&
&  0.96159E+00_JPRB, 0.12233E+01_JPRB, 0.13575E+01_JPRB, 0.16756E+01_JPRB, 0.17059E+01_JPRB, 0.15646E+01_JPRB, 0.15281E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.14015E+01_JPRB, 0.10619E+01_JPRB, 0.10862E+01_JPRB, 0.11843E+01_JPRB, 0.97390E+00_JPRB, 0.98723E+00_JPRB ,&
&  0.11169E+01_JPRB, 0.10508E+01_JPRB, 0.91491E+00_JPRB, 0.72920E+00_JPRB, 0.85262E+00_JPRB, 0.74541E+00_JPRB, 0.44107E+00_JPRB ,&
&  0.38121E+00_JPRB, 0.36413E+00_JPRB, 0.35100E+00_JPRB, 0.34667E+00_JPRB, 0.34279E+00_JPRB, 0.38550E+00_JPRB, 0.52282E+00_JPRB ,&
&  0.57350E+00_JPRB, 0.72358E+00_JPRB, 0.80759E+00_JPRB, 0.83840E+00_JPRB, 0.84134E+00_JPRB, 0.92890E+00_JPRB, 0.11894E+01_JPRB ,&
&  0.10171E+01_JPRB, 0.99691E+00_JPRB, 0.87733E+00_JPRB, 0.80990E+00_JPRB, 0.75822E+00_JPRB, 0.71756E+00_JPRB, 0.68505E+00_JPRB ,&
&  0.65844E+00_JPRB, 0.63737E+00_JPRB, 0.61779E+00_JPRB, 0.60913E+00_JPRB, 0.59515E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.54239E+00_JPRB, 0.53074E+00_JPRB, 0.52120E+00_JPRB, 0.51160E+00_JPRB, 0.49867E+00_JPRB, 0.48640E+00_JPRB, 0.49213E+00_JPRB ,&
&  0.49035E+00_JPRB, 0.47189E+00_JPRB, 0.46170E+00_JPRB, 0.46021E+00_JPRB, 0.47575E+00_JPRB, 0.50333E+00_JPRB, 0.51780E+00_JPRB ,&
&  0.53749E+00_JPRB, 0.62110E+00_JPRB, 0.74210E+00_JPRB, 0.65096E+00_JPRB, 0.57938E+00_JPRB, 0.54577E+00_JPRB, 0.56357E+00_JPRB ,&
&  0.90097E+00_JPRB, 0.12524E+01_JPRB, 0.16039E+01_JPRB, 0.20481E+01_JPRB, 0.24669E+01_JPRB, 0.24905E+01_JPRB, 0.27180E+01_JPRB ,&
&  0.25644E+01_JPRB, 0.22121E+01_JPRB, 0.18846E+01_JPRB, 0.15784E+01_JPRB, 0.13594E+01_JPRB, 0.11914E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.98874E+00_JPRB, 0.94560E+00_JPRB, 0.90955E+00_JPRB, 0.87832E+00_JPRB, 0.85949E+00_JPRB, 0.84952E+00_JPRB, 0.85604E+00_JPRB ,&
&  0.84753E+00_JPRB, 0.82995E+00_JPRB, 0.83059E+00_JPRB, 0.83548E+00_JPRB, 0.85226E+00_JPRB, 0.84999E+00_JPRB, 0.83528E+00_JPRB ,&
&  0.84361E+00_JPRB, 0.99072E+00_JPRB, 0.15110E+01_JPRB, 0.17426E+01_JPRB, 0.17264E+01_JPRB, 0.16761E+01_JPRB, 0.15805E+01_JPRB ,&
&  0.14752E+01_JPRB, 0.15271E+01_JPRB, 0.13051E+01_JPRB, 0.13287E+01_JPRB, 0.11631E+01_JPRB, 0.88073E+00_JPRB, 0.11275E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.11978E+01_JPRB, 0.98447E+00_JPRB, 0.72711E+00_JPRB, 0.70015E+00_JPRB, 0.50792E+00_JPRB, 0.37729E+00_JPRB ,&
&  0.36025E+00_JPRB, 0.34817E+00_JPRB, 0.32649E+00_JPRB, 0.33425E+00_JPRB, 0.33071E+00_JPRB, 0.36913E+00_JPRB, 0.45071E+00_JPRB ,&
&  0.52395E+00_JPRB, 0.62943E+00_JPRB, 0.74464E+00_JPRB, 0.80717E+00_JPRB, 0.81129E+00_JPRB, 0.80617E+00_JPRB, 0.95814E+00_JPRB ,&
&  0.90355E+00_JPRB, 0.93626E+00_JPRB, 0.84763E+00_JPRB, 0.76992E+00_JPRB, 0.70967E+00_JPRB, 0.67152E+00_JPRB, 0.64434E+00_JPRB ,&
&  0.62368E+00_JPRB, 0.59713E+00_JPRB, 0.57578E+00_JPRB, 0.56430E+00_JPRB, 0.55509E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.50215E+00_JPRB, 0.49267E+00_JPRB, 0.48531E+00_JPRB, 0.47472E+00_JPRB, 0.46412E+00_JPRB, 0.45170E+00_JPRB, 0.45701E+00_JPRB ,&
&  0.46557E+00_JPRB, 0.44506E+00_JPRB, 0.42863E+00_JPRB, 0.43322E+00_JPRB, 0.45618E+00_JPRB, 0.48922E+00_JPRB, 0.52657E+00_JPRB ,&
&  0.57141E+00_JPRB, 0.79946E+00_JPRB, 0.60655E+00_JPRB, 0.48253E+00_JPRB, 0.53000E+00_JPRB, 0.55808E+00_JPRB, 0.52912E+00_JPRB ,&
&  0.78674E+00_JPRB, 0.10495E+01_JPRB, 0.13402E+01_JPRB, 0.18946E+01_JPRB, 0.27045E+01_JPRB, 0.27525E+01_JPRB, 0.28927E+01_JPRB ,&
&  0.28892E+01_JPRB, 0.24477E+01_JPRB, 0.19932E+01_JPRB, 0.16317E+01_JPRB, 0.13813E+01_JPRB, 0.12081E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.97773E+00_JPRB, 0.93642E+00_JPRB, 0.91683E+00_JPRB, 0.89873E+00_JPRB, 0.87948E+00_JPRB, 0.88017E+00_JPRB ,&
&  0.86418E+00_JPRB, 0.83773E+00_JPRB, 0.83342E+00_JPRB, 0.83909E+00_JPRB, 0.87671E+00_JPRB, 0.85071E+00_JPRB, 0.11892E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.12899E+01_JPRB, 0.15187E+01_JPRB, 0.19437E+01_JPRB, 0.19938E+01_JPRB, 0.18012E+01_JPRB, 0.18646E+01_JPRB ,&
&  0.14321E+01_JPRB, 0.12374E+01_JPRB, 0.12182E+01_JPRB, 0.11617E+01_JPRB, 0.91243E+00_JPRB, 0.96766E+00_JPRB, 0.14456E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.13393E+01_JPRB, 0.14144E+01_JPRB, 0.14273E+01_JPRB, 0.70531E+00_JPRB, 0.47070E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.49371E+00_JPRB, 0.38847E+00_JPRB, 0.46532E+00_JPRB, 0.38026E+00_JPRB, 0.42616E+00_JPRB, 0.51300E+00_JPRB ,&
&  0.50179E+00_JPRB, 0.53198E+00_JPRB, 0.69385E+00_JPRB, 0.76815E+00_JPRB, 0.78633E+00_JPRB, 0.75423E+00_JPRB, 0.80273E+00_JPRB ,&
&  0.81027E+00_JPRB, 0.86263E+00_JPRB, 0.85772E+00_JPRB, 0.76368E+00_JPRB, 0.68969E+00_JPRB, 0.63890E+00_JPRB, 0.60479E+00_JPRB ,&
&  0.57802E+00_JPRB, 0.54556E+00_JPRB, 0.52560E+00_JPRB, 0.51779E+00_JPRB, 0.51130E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.47182E+00_JPRB, 0.46599E+00_JPRB, 0.46020E+00_JPRB, 0.44854E+00_JPRB, 0.43591E+00_JPRB, 0.42665E+00_JPRB, 0.42909E+00_JPRB ,&
&  0.44023E+00_JPRB, 0.41830E+00_JPRB, 0.40456E+00_JPRB, 0.40983E+00_JPRB, 0.42787E+00_JPRB, 0.45498E+00_JPRB, 0.49701E+00_JPRB ,&
&  0.54668E+00_JPRB, 0.58519E+00_JPRB, 0.44719E+00_JPRB, 0.40437E+00_JPRB, 0.54349E+00_JPRB, 0.53773E+00_JPRB, 0.59050E+00_JPRB ,&
&  0.75109E+00_JPRB, 0.91193E+00_JPRB, 0.11687E+01_JPRB, 0.16754E+01_JPRB, 0.21407E+01_JPRB, 0.26338E+01_JPRB, 0.29160E+01_JPRB ,&
&  0.30406E+01_JPRB, 0.27601E+01_JPRB, 0.23229E+01_JPRB, 0.18775E+01_JPRB, 0.15279E+01_JPRB, 0.12948E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.96847E+00_JPRB, 0.91844E+00_JPRB, 0.87623E+00_JPRB, 0.87405E+00_JPRB, 0.86252E+00_JPRB, 0.87054E+00_JPRB ,&
&  0.87277E+00_JPRB, 0.85315E+00_JPRB, 0.85764E+00_JPRB, 0.85369E+00_JPRB, 0.84618E+00_JPRB, 0.87239E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.13979E+01_JPRB, 0.16236E+01_JPRB, 0.18211E+01_JPRB, 0.18953E+01_JPRB, 0.17312E+01_JPRB, 0.19570E+01_JPRB ,&
&  0.16127E+01_JPRB, 0.12889E+01_JPRB, 0.10434E+01_JPRB, 0.80731E+00_JPRB, 0.10014E+01_JPRB, 0.14642E+01_JPRB, 0.14761E+01_JPRB ,&
&  0.14256E+01_JPRB, 0.14732E+01_JPRB, 0.14931E+01_JPRB, 0.13300E+01_JPRB, 0.11703E+01_JPRB, 0.80925E+00_JPRB, 0.90976E+00_JPRB ,&
&  0.89289E+00_JPRB, 0.82608E+00_JPRB, 0.74377E+00_JPRB, 0.64895E+00_JPRB, 0.50977E+00_JPRB, 0.58611E+00_JPRB, 0.58822E+00_JPRB ,&
&  0.52656E+00_JPRB, 0.50438E+00_JPRB, 0.61993E+00_JPRB, 0.72886E+00_JPRB, 0.72512E+00_JPRB, 0.65409E+00_JPRB, 0.72239E+00_JPRB ,&
&  0.71681E+00_JPRB, 0.73060E+00_JPRB, 0.70405E+00_JPRB, 0.66462E+00_JPRB, 0.62748E+00_JPRB, 0.58958E+00_JPRB, 0.55252E+00_JPRB ,&
&  0.52327E+00_JPRB, 0.49899E+00_JPRB, 0.47969E+00_JPRB, 0.47596E+00_JPRB, 0.47462E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.45982E+00_JPRB, 0.45634E+00_JPRB, 0.45110E+00_JPRB, 0.44249E+00_JPRB, 0.42990E+00_JPRB, 0.41802E+00_JPRB, 0.41807E+00_JPRB ,&
&  0.42156E+00_JPRB, 0.39985E+00_JPRB, 0.39284E+00_JPRB, 0.39717E+00_JPRB, 0.41281E+00_JPRB, 0.43551E+00_JPRB, 0.46940E+00_JPRB ,&
&  0.51902E+00_JPRB, 0.47926E+00_JPRB, 0.37388E+00_JPRB, 0.40472E+00_JPRB, 0.47474E+00_JPRB, 0.48407E+00_JPRB, 0.58070E+00_JPRB ,&
&  0.68404E+00_JPRB, 0.80084E+00_JPRB, 0.95991E+00_JPRB, 0.12917E+01_JPRB, 0.16550E+01_JPRB, 0.20154E+01_JPRB, 0.23097E+01_JPRB ,&
&  0.23806E+01_JPRB, 0.24896E+01_JPRB, 0.24235E+01_JPRB, 0.21704E+01_JPRB, 0.18194E+01_JPRB, 0.15155E+01_JPRB, 0.13170E+01_JPRB ,&
&  0.11686E+01_JPRB, 0.10255E+01_JPRB, 0.92657E+00_JPRB, 0.86781E+00_JPRB, 0.83581E+00_JPRB, 0.83222E+00_JPRB, 0.85037E+00_JPRB ,&
&  0.87478E+00_JPRB, 0.86407E+00_JPRB, 0.84698E+00_JPRB, 0.87086E+00_JPRB, 0.84648E+00_JPRB, 0.90009E+00_JPRB, 0.10055E+01_JPRB ,&
&  0.14447E+01_JPRB, 0.17057E+01_JPRB, 0.18067E+01_JPRB, 0.20329E+01_JPRB, 0.17081E+01_JPRB, 0.16383E+01_JPRB, 0.18987E+01_JPRB ,&
&  0.21177E+01_JPRB, 0.21445E+01_JPRB, 0.21592E+01_JPRB, 0.14026E+01_JPRB, 0.12061E+01_JPRB, 0.16658E+01_JPRB, 0.15710E+01_JPRB ,&
&  0.14135E+01_JPRB, 0.14394E+01_JPRB, 0.14049E+01_JPRB, 0.15222E+01_JPRB, 0.12422E+01_JPRB, 0.10529E+01_JPRB, 0.75108E+00_JPRB ,&
&  0.66905E+00_JPRB, 0.58941E+00_JPRB, 0.65618E+00_JPRB, 0.61083E+00_JPRB, 0.58209E+00_JPRB, 0.58005E+00_JPRB, 0.56278E+00_JPRB ,&
&  0.54268E+00_JPRB, 0.52243E+00_JPRB, 0.53319E+00_JPRB, 0.61312E+00_JPRB, 0.66022E+00_JPRB, 0.57219E+00_JPRB, 0.61353E+00_JPRB ,&
&  0.61830E+00_JPRB, 0.62688E+00_JPRB, 0.62392E+00_JPRB, 0.63365E+00_JPRB, 0.63244E+00_JPRB, 0.59746E+00_JPRB, 0.54842E+00_JPRB ,&
&  0.50505E+00_JPRB, 0.48042E+00_JPRB, 0.46809E+00_JPRB, 0.46388E+00_JPRB, 0.46275E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.44304E+00_JPRB, 0.44360E+00_JPRB, 0.44110E+00_JPRB, 0.43547E+00_JPRB, 0.42793E+00_JPRB, 0.41974E+00_JPRB, 0.41775E+00_JPRB ,&
&  0.41406E+00_JPRB, 0.39824E+00_JPRB, 0.38778E+00_JPRB, 0.39008E+00_JPRB, 0.40264E+00_JPRB, 0.41834E+00_JPRB, 0.44337E+00_JPRB ,&
&  0.50046E+00_JPRB, 0.52858E+00_JPRB, 0.41460E+00_JPRB, 0.36425E+00_JPRB, 0.35019E+00_JPRB, 0.43442E+00_JPRB, 0.52627E+00_JPRB ,&
&  0.64752E+00_JPRB, 0.71108E+00_JPRB, 0.85209E+00_JPRB, 0.10528E+01_JPRB, 0.12735E+01_JPRB, 0.14785E+01_JPRB, 0.16600E+01_JPRB ,&
&  0.20712E+01_JPRB, 0.19183E+01_JPRB, 0.19883E+01_JPRB, 0.18918E+01_JPRB, 0.18332E+01_JPRB, 0.16357E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.10909E+01_JPRB, 0.95887E+00_JPRB, 0.85744E+00_JPRB, 0.79979E+00_JPRB, 0.77644E+00_JPRB, 0.79304E+00_JPRB ,&
&  0.85745E+00_JPRB, 0.88159E+00_JPRB, 0.87084E+00_JPRB, 0.88250E+00_JPRB, 0.95034E+00_JPRB, 0.10583E+01_JPRB, 0.12343E+01_JPRB ,&
&  0.13348E+01_JPRB, 0.13931E+01_JPRB, 0.20098E+01_JPRB, 0.21661E+01_JPRB, 0.23084E+01_JPRB, 0.21518E+01_JPRB, 0.20124E+01_JPRB ,&
&  0.21833E+01_JPRB, 0.22392E+01_JPRB, 0.22925E+01_JPRB, 0.17720E+01_JPRB, 0.19842E+01_JPRB, 0.19144E+01_JPRB, 0.16983E+01_JPRB ,&
&  0.15151E+01_JPRB, 0.14755E+01_JPRB, 0.13818E+01_JPRB, 0.13713E+01_JPRB, 0.14213E+01_JPRB, 0.14640E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.81457E+00_JPRB, 0.84114E+00_JPRB, 0.73877E+00_JPRB, 0.61250E+00_JPRB, 0.51297E+00_JPRB, 0.52558E+00_JPRB, 0.53927E+00_JPRB ,&
&  0.53874E+00_JPRB, 0.53675E+00_JPRB, 0.51773E+00_JPRB, 0.53791E+00_JPRB, 0.56354E+00_JPRB, 0.53535E+00_JPRB, 0.52734E+00_JPRB ,&
&  0.49169E+00_JPRB, 0.53367E+00_JPRB, 0.54466E+00_JPRB, 0.53787E+00_JPRB, 0.56979E+00_JPRB, 0.55883E+00_JPRB, 0.52385E+00_JPRB ,&
&  0.48276E+00_JPRB, 0.45830E+00_JPRB, 0.44965E+00_JPRB, 0.44456E+00_JPRB, 0.44198E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.40896E+00_JPRB, 0.42162E+00_JPRB, 0.43014E+00_JPRB, 0.43158E+00_JPRB, 0.42835E+00_JPRB, 0.42128E+00_JPRB, 0.41607E+00_JPRB ,&
&  0.41078E+00_JPRB, 0.39680E+00_JPRB, 0.38225E+00_JPRB, 0.38555E+00_JPRB, 0.39267E+00_JPRB, 0.40757E+00_JPRB, 0.43762E+00_JPRB ,&
&  0.49656E+00_JPRB, 0.63972E+00_JPRB, 0.49529E+00_JPRB, 0.46188E+00_JPRB, 0.42700E+00_JPRB, 0.48449E+00_JPRB, 0.54255E+00_JPRB ,&
&  0.64385E+00_JPRB, 0.65767E+00_JPRB, 0.70699E+00_JPRB, 0.81333E+00_JPRB, 0.98839E+00_JPRB, 0.11150E+01_JPRB, 0.12937E+01_JPRB ,&
&  0.13770E+01_JPRB, 0.14921E+01_JPRB, 0.13974E+01_JPRB, 0.14321E+01_JPRB, 0.14481E+01_JPRB, 0.13313E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.10397E+01_JPRB, 0.94737E+00_JPRB, 0.86874E+00_JPRB, 0.81940E+00_JPRB, 0.78460E+00_JPRB, 0.77977E+00_JPRB ,&
&  0.83097E+00_JPRB, 0.89415E+00_JPRB, 0.89898E+00_JPRB, 0.90161E+00_JPRB, 0.93643E+00_JPRB, 0.10815E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.16580E+01_JPRB, 0.16989E+01_JPRB, 0.14309E+01_JPRB, 0.20301E+01_JPRB, 0.26083E+01_JPRB, 0.23369E+01_JPRB, 0.20089E+01_JPRB ,&
&  0.21545E+01_JPRB, 0.22501E+01_JPRB, 0.23733E+01_JPRB, 0.23781E+01_JPRB, 0.23181E+01_JPRB, 0.21676E+01_JPRB, 0.19114E+01_JPRB ,&
&  0.16216E+01_JPRB, 0.14660E+01_JPRB, 0.13920E+01_JPRB, 0.14561E+01_JPRB, 0.13578E+01_JPRB, 0.12429E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.94765E+00_JPRB, 0.76800E+00_JPRB, 0.51992E+00_JPRB, 0.54956E+00_JPRB, 0.44273E+00_JPRB, 0.45135E+00_JPRB, 0.52261E+00_JPRB ,&
&  0.50958E+00_JPRB, 0.51376E+00_JPRB, 0.49828E+00_JPRB, 0.46664E+00_JPRB, 0.49548E+00_JPRB, 0.48636E+00_JPRB, 0.48940E+00_JPRB ,&
&  0.47899E+00_JPRB, 0.44812E+00_JPRB, 0.47897E+00_JPRB, 0.47192E+00_JPRB, 0.48473E+00_JPRB, 0.47659E+00_JPRB, 0.45813E+00_JPRB ,&
&  0.43435E+00_JPRB, 0.41954E+00_JPRB, 0.41661E+00_JPRB, 0.41264E+00_JPRB, 0.40608E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.37536E+00_JPRB, 0.39649E+00_JPRB, 0.41085E+00_JPRB, 0.41328E+00_JPRB, 0.41457E+00_JPRB, 0.40280E+00_JPRB, 0.39779E+00_JPRB ,&
&  0.39997E+00_JPRB, 0.38871E+00_JPRB, 0.37282E+00_JPRB, 0.37566E+00_JPRB, 0.38128E+00_JPRB, 0.39593E+00_JPRB, 0.43439E+00_JPRB ,&
&  0.52563E+00_JPRB, 0.58511E+00_JPRB, 0.50504E+00_JPRB, 0.43385E+00_JPRB, 0.53583E+00_JPRB, 0.50709E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.51490E+00_JPRB, 0.55293E+00_JPRB, 0.56563E+00_JPRB, 0.59708E+00_JPRB, 0.69436E+00_JPRB, 0.84023E+00_JPRB, 0.95346E+00_JPRB ,&
&  0.10033E+01_JPRB, 0.10400E+01_JPRB, 0.10172E+01_JPRB, 0.99443E+00_JPRB, 0.10039E+01_JPRB, 0.92551E+00_JPRB, 0.93437E+00_JPRB ,&
&  0.89988E+00_JPRB, 0.83697E+00_JPRB, 0.78331E+00_JPRB, 0.77039E+00_JPRB, 0.75606E+00_JPRB, 0.74372E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.82003E+00_JPRB, 0.90047E+00_JPRB, 0.90728E+00_JPRB, 0.92073E+00_JPRB, 0.98715E+00_JPRB, 0.12513E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.20183E+01_JPRB, 0.21657E+01_JPRB, 0.22744E+01_JPRB, 0.25269E+01_JPRB, 0.25377E+01_JPRB, 0.24110E+01_JPRB, 0.23446E+01_JPRB ,&
&  0.23364E+01_JPRB, 0.22882E+01_JPRB, 0.23317E+01_JPRB, 0.23147E+01_JPRB, 0.22887E+01_JPRB, 0.22281E+01_JPRB, 0.19816E+01_JPRB ,&
&  0.16641E+01_JPRB, 0.14592E+01_JPRB, 0.13628E+01_JPRB, 0.12703E+01_JPRB, 0.11895E+01_JPRB, 0.10711E+01_JPRB, 0.10811E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.63376E+00_JPRB, 0.47362E+00_JPRB, 0.55803E+00_JPRB, 0.47644E+00_JPRB, 0.49475E+00_JPRB, 0.59801E+00_JPRB ,&
&  0.57339E+00_JPRB, 0.53354E+00_JPRB, 0.50499E+00_JPRB, 0.46591E+00_JPRB, 0.46748E+00_JPRB, 0.51040E+00_JPRB, 0.46206E+00_JPRB ,&
&  0.43257E+00_JPRB, 0.42817E+00_JPRB, 0.43729E+00_JPRB, 0.41706E+00_JPRB, 0.41755E+00_JPRB, 0.41165E+00_JPRB, 0.40416E+00_JPRB ,&
&  0.38878E+00_JPRB, 0.38054E+00_JPRB, 0.37087E+00_JPRB, 0.36873E+00_JPRB, 0.37179E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.35861E+00_JPRB, 0.38259E+00_JPRB, 0.39404E+00_JPRB, 0.39976E+00_JPRB, 0.40336E+00_JPRB, 0.38928E+00_JPRB, 0.38480E+00_JPRB ,&
&  0.38706E+00_JPRB, 0.37925E+00_JPRB, 0.36597E+00_JPRB, 0.36605E+00_JPRB, 0.36958E+00_JPRB, 0.37257E+00_JPRB, 0.38328E+00_JPRB ,&
&  0.33488E+00_JPRB, 0.39695E+00_JPRB, 0.36222E+00_JPRB, 0.40110E+00_JPRB, 0.50744E+00_JPRB, 0.47106E+00_JPRB, 0.46634E+00_JPRB ,&
&  0.45178E+00_JPRB, 0.43893E+00_JPRB, 0.42530E+00_JPRB, 0.44065E+00_JPRB, 0.48108E+00_JPRB, 0.60803E+00_JPRB, 0.72909E+00_JPRB ,&
&  0.71148E+00_JPRB, 0.67953E+00_JPRB, 0.66105E+00_JPRB, 0.65647E+00_JPRB, 0.63634E+00_JPRB, 0.63546E+00_JPRB, 0.65499E+00_JPRB ,&
&  0.65276E+00_JPRB, 0.63647E+00_JPRB, 0.64002E+00_JPRB, 0.66222E+00_JPRB, 0.67480E+00_JPRB, 0.66806E+00_JPRB, 0.67677E+00_JPRB ,&
&  0.73853E+00_JPRB, 0.83319E+00_JPRB, 0.86679E+00_JPRB, 0.92134E+00_JPRB, 0.10095E+01_JPRB, 0.15430E+01_JPRB, 0.22726E+01_JPRB ,&
&  0.24490E+01_JPRB, 0.23845E+01_JPRB, 0.24687E+01_JPRB, 0.26495E+01_JPRB, 0.25969E+01_JPRB, 0.24341E+01_JPRB, 0.22698E+01_JPRB ,&
&  0.21865E+01_JPRB, 0.20396E+01_JPRB, 0.20891E+01_JPRB, 0.21931E+01_JPRB, 0.21049E+01_JPRB, 0.21595E+01_JPRB, 0.19834E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.15838E+01_JPRB, 0.14801E+01_JPRB, 0.14297E+01_JPRB, 0.13107E+01_JPRB, 0.13140E+01_JPRB, 0.12266E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.10534E+01_JPRB, 0.75939E+00_JPRB, 0.68254E+00_JPRB, 0.52853E+00_JPRB, 0.55163E+00_JPRB, 0.71462E+00_JPRB ,&
&  0.68054E+00_JPRB, 0.59815E+00_JPRB, 0.51989E+00_JPRB, 0.46927E+00_JPRB, 0.47283E+00_JPRB, 0.50929E+00_JPRB, 0.45098E+00_JPRB ,&
&  0.38682E+00_JPRB, 0.39823E+00_JPRB, 0.39283E+00_JPRB, 0.37417E+00_JPRB, 0.36456E+00_JPRB, 0.36416E+00_JPRB, 0.35982E+00_JPRB ,&
&  0.35167E+00_JPRB, 0.34595E+00_JPRB, 0.34402E+00_JPRB, 0.33937E+00_JPRB, 0.34853E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.34681E+00_JPRB, 0.36502E+00_JPRB, 0.37078E+00_JPRB, 0.38224E+00_JPRB, 0.38986E+00_JPRB, 0.37918E+00_JPRB, 0.37327E+00_JPRB ,&
&  0.37027E+00_JPRB, 0.36588E+00_JPRB, 0.36014E+00_JPRB, 0.35892E+00_JPRB, 0.35981E+00_JPRB, 0.35766E+00_JPRB, 0.32452E+00_JPRB ,&
&  0.28948E+00_JPRB, 0.32237E+00_JPRB, 0.33004E+00_JPRB, 0.35376E+00_JPRB, 0.39214E+00_JPRB, 0.38346E+00_JPRB, 0.36573E+00_JPRB ,&
&  0.36957E+00_JPRB, 0.36289E+00_JPRB, 0.37076E+00_JPRB, 0.39019E+00_JPRB, 0.41590E+00_JPRB, 0.48907E+00_JPRB, 0.60098E+00_JPRB ,&
&  0.56997E+00_JPRB, 0.55508E+00_JPRB, 0.53491E+00_JPRB, 0.53227E+00_JPRB, 0.53928E+00_JPRB, 0.53529E+00_JPRB, 0.52567E+00_JPRB ,&
&  0.51535E+00_JPRB, 0.52042E+00_JPRB, 0.53342E+00_JPRB, 0.55612E+00_JPRB, 0.57694E+00_JPRB, 0.58267E+00_JPRB, 0.58238E+00_JPRB ,&
&  0.61838E+00_JPRB, 0.69113E+00_JPRB, 0.74581E+00_JPRB, 0.81160E+00_JPRB, 0.92199E+00_JPRB, 0.14193E+01_JPRB, 0.18722E+01_JPRB ,&
&  0.19416E+01_JPRB, 0.19452E+01_JPRB, 0.18997E+01_JPRB, 0.18864E+01_JPRB, 0.19178E+01_JPRB, 0.18439E+01_JPRB, 0.17650E+01_JPRB ,&
&  0.17721E+01_JPRB, 0.16519E+01_JPRB, 0.19702E+01_JPRB, 0.21132E+01_JPRB, 0.20642E+01_JPRB, 0.20525E+01_JPRB, 0.19028E+01_JPRB ,&
&  0.18144E+01_JPRB, 0.16654E+01_JPRB, 0.15641E+01_JPRB, 0.14744E+01_JPRB, 0.13652E+01_JPRB, 0.13371E+01_JPRB, 0.12126E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.11382E+01_JPRB, 0.96046E+00_JPRB, 0.82931E+00_JPRB, 0.69823E+00_JPRB, 0.73097E+00_JPRB, 0.69471E+00_JPRB ,&
&  0.62806E+00_JPRB, 0.51675E+00_JPRB, 0.50793E+00_JPRB, 0.49219E+00_JPRB, 0.50419E+00_JPRB, 0.49183E+00_JPRB, 0.41817E+00_JPRB ,&
&  0.38655E+00_JPRB, 0.39739E+00_JPRB, 0.37657E+00_JPRB, 0.35525E+00_JPRB, 0.34147E+00_JPRB, 0.32959E+00_JPRB, 0.30628E+00_JPRB ,&
&  0.30085E+00_JPRB, 0.32712E+00_JPRB, 0.33499E+00_JPRB, 0.33059E+00_JPRB, 0.33536E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.33610E+00_JPRB, 0.35027E+00_JPRB, 0.35676E+00_JPRB, 0.36837E+00_JPRB, 0.37412E+00_JPRB, 0.36715E+00_JPRB, 0.34861E+00_JPRB ,&
&  0.35606E+00_JPRB, 0.35244E+00_JPRB, 0.34995E+00_JPRB, 0.35231E+00_JPRB, 0.35514E+00_JPRB, 0.31974E+00_JPRB, 0.25946E+00_JPRB ,&
&  0.24842E+00_JPRB, 0.26734E+00_JPRB, 0.30336E+00_JPRB, 0.32942E+00_JPRB, 0.33619E+00_JPRB, 0.33008E+00_JPRB, 0.32375E+00_JPRB ,&
&  0.32266E+00_JPRB, 0.32911E+00_JPRB, 0.35605E+00_JPRB, 0.37832E+00_JPRB, 0.38801E+00_JPRB, 0.41030E+00_JPRB, 0.50272E+00_JPRB ,&
&  0.50230E+00_JPRB, 0.49701E+00_JPRB, 0.49717E+00_JPRB, 0.46129E+00_JPRB, 0.48505E+00_JPRB, 0.46182E+00_JPRB, 0.44139E+00_JPRB ,&
&  0.43881E+00_JPRB, 0.45012E+00_JPRB, 0.47087E+00_JPRB, 0.49893E+00_JPRB, 0.51675E+00_JPRB, 0.52148E+00_JPRB, 0.52417E+00_JPRB ,&
&  0.53981E+00_JPRB, 0.58219E+00_JPRB, 0.63288E+00_JPRB, 0.70532E+00_JPRB, 0.80519E+00_JPRB, 0.11284E+01_JPRB, 0.14536E+01_JPRB ,&
&  0.15027E+01_JPRB, 0.15418E+01_JPRB, 0.15499E+01_JPRB, 0.13839E+01_JPRB, 0.14521E+01_JPRB, 0.14902E+01_JPRB, 0.14334E+01_JPRB ,&
&  0.14796E+01_JPRB, 0.14178E+01_JPRB, 0.15298E+01_JPRB, 0.16818E+01_JPRB, 0.18134E+01_JPRB, 0.17769E+01_JPRB, 0.16798E+01_JPRB ,&
&  0.16822E+01_JPRB, 0.16707E+01_JPRB, 0.14925E+01_JPRB, 0.13879E+01_JPRB, 0.12625E+01_JPRB, 0.11941E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.10207E+01_JPRB, 0.96870E+00_JPRB, 0.90979E+00_JPRB, 0.83728E+00_JPRB, 0.77025E+00_JPRB, 0.70704E+00_JPRB ,&
&  0.64105E+00_JPRB, 0.55352E+00_JPRB, 0.51006E+00_JPRB, 0.51260E+00_JPRB, 0.53419E+00_JPRB, 0.46769E+00_JPRB, 0.40771E+00_JPRB ,&
&  0.37941E+00_JPRB, 0.36172E+00_JPRB, 0.37425E+00_JPRB, 0.34693E+00_JPRB, 0.32703E+00_JPRB, 0.31359E+00_JPRB, 0.30651E+00_JPRB ,&
&  0.30071E+00_JPRB, 0.33178E+00_JPRB, 0.33710E+00_JPRB, 0.33291E+00_JPRB, 0.32982E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.33414E+00_JPRB, 0.34475E+00_JPRB, 0.35123E+00_JPRB, 0.35901E+00_JPRB, 0.36253E+00_JPRB, 0.34572E+00_JPRB, 0.34104E+00_JPRB ,&
&  0.37770E+00_JPRB, 0.32811E+00_JPRB, 0.34679E+00_JPRB, 0.28558E+00_JPRB, 0.26251E+00_JPRB, 0.25867E+00_JPRB, 0.25134E+00_JPRB ,&
&  0.25606E+00_JPRB, 0.27456E+00_JPRB, 0.29641E+00_JPRB, 0.31357E+00_JPRB, 0.32849E+00_JPRB, 0.31764E+00_JPRB, 0.30822E+00_JPRB ,&
&  0.30111E+00_JPRB, 0.31208E+00_JPRB, 0.34771E+00_JPRB, 0.37358E+00_JPRB, 0.39131E+00_JPRB, 0.40665E+00_JPRB, 0.45110E+00_JPRB ,&
&  0.44658E+00_JPRB, 0.43860E+00_JPRB, 0.45046E+00_JPRB, 0.44510E+00_JPRB, 0.43772E+00_JPRB, 0.41609E+00_JPRB, 0.39462E+00_JPRB ,&
&  0.38588E+00_JPRB, 0.37655E+00_JPRB, 0.42617E+00_JPRB, 0.45737E+00_JPRB, 0.47422E+00_JPRB, 0.47865E+00_JPRB, 0.48894E+00_JPRB ,&
&  0.50240E+00_JPRB, 0.52731E+00_JPRB, 0.57146E+00_JPRB, 0.63713E+00_JPRB, 0.70971E+00_JPRB, 0.84888E+00_JPRB, 0.99045E+00_JPRB ,&
&  0.11740E+01_JPRB, 0.90163E+00_JPRB, 0.10724E+01_JPRB, 0.10886E+01_JPRB, 0.12318E+01_JPRB, 0.13039E+01_JPRB, 0.12893E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.14034E+01_JPRB, 0.14340E+01_JPRB, 0.14525E+01_JPRB, 0.14795E+01_JPRB, 0.14802E+01_JPRB, 0.14291E+01_JPRB ,&
&  0.14363E+01_JPRB, 0.13071E+01_JPRB, 0.13010E+01_JPRB, 0.12348E+01_JPRB, 0.11438E+01_JPRB, 0.10870E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.95644E+00_JPRB, 0.92899E+00_JPRB, 0.89102E+00_JPRB, 0.78055E+00_JPRB, 0.74311E+00_JPRB, 0.71322E+00_JPRB, 0.64631E+00_JPRB ,&
&  0.62057E+00_JPRB, 0.59018E+00_JPRB, 0.55760E+00_JPRB, 0.59583E+00_JPRB, 0.57635E+00_JPRB, 0.51518E+00_JPRB, 0.45787E+00_JPRB ,&
&  0.40127E+00_JPRB, 0.34785E+00_JPRB, 0.34687E+00_JPRB, 0.33404E+00_JPRB, 0.31473E+00_JPRB, 0.30750E+00_JPRB, 0.33412E+00_JPRB ,&
&  0.32693E+00_JPRB, 0.33267E+00_JPRB, 0.35117E+00_JPRB, 0.34702E+00_JPRB, 0.33715E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.34015E+00_JPRB, 0.34180E+00_JPRB, 0.34103E+00_JPRB, 0.34159E+00_JPRB, 0.34231E+00_JPRB, 0.33167E+00_JPRB, 0.31032E+00_JPRB ,&
&  0.31522E+00_JPRB, 0.27431E+00_JPRB, 0.25610E+00_JPRB, 0.24703E+00_JPRB, 0.24774E+00_JPRB, 0.24594E+00_JPRB, 0.24056E+00_JPRB ,&
&  0.24609E+00_JPRB, 0.26427E+00_JPRB, 0.29461E+00_JPRB, 0.31050E+00_JPRB, 0.31403E+00_JPRB, 0.31107E+00_JPRB, 0.30436E+00_JPRB ,&
&  0.30084E+00_JPRB, 0.31365E+00_JPRB, 0.34133E+00_JPRB, 0.38161E+00_JPRB, 0.40119E+00_JPRB, 0.40754E+00_JPRB, 0.41650E+00_JPRB ,&
&  0.42103E+00_JPRB, 0.42056E+00_JPRB, 0.42057E+00_JPRB, 0.42473E+00_JPRB, 0.40781E+00_JPRB, 0.37718E+00_JPRB, 0.34171E+00_JPRB ,&
&  0.29097E+00_JPRB, 0.29520E+00_JPRB, 0.38615E+00_JPRB, 0.41985E+00_JPRB, 0.44058E+00_JPRB, 0.44999E+00_JPRB, 0.46128E+00_JPRB ,&
&  0.48062E+00_JPRB, 0.50620E+00_JPRB, 0.53908E+00_JPRB, 0.58223E+00_JPRB, 0.62516E+00_JPRB, 0.68933E+00_JPRB, 0.74801E+00_JPRB ,&
&  0.87107E+00_JPRB, 0.72649E+00_JPRB, 0.69360E+00_JPRB, 0.75556E+00_JPRB, 0.10641E+01_JPRB, 0.11693E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.12015E+01_JPRB, 0.12342E+01_JPRB, 0.13081E+01_JPRB, 0.13461E+01_JPRB, 0.13308E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.11657E+01_JPRB, 0.10970E+01_JPRB, 0.10792E+01_JPRB, 0.10396E+01_JPRB, 0.10188E+01_JPRB, 0.97046E+00_JPRB ,&
&  0.92251E+00_JPRB, 0.89137E+00_JPRB, 0.82972E+00_JPRB, 0.76147E+00_JPRB, 0.70488E+00_JPRB, 0.66986E+00_JPRB, 0.64005E+00_JPRB ,&
&  0.61789E+00_JPRB, 0.61377E+00_JPRB, 0.61727E+00_JPRB, 0.62376E+00_JPRB, 0.59199E+00_JPRB, 0.52361E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.39332E+00_JPRB, 0.34577E+00_JPRB, 0.30485E+00_JPRB, 0.29459E+00_JPRB, 0.29824E+00_JPRB, 0.30254E+00_JPRB, 0.30656E+00_JPRB ,&
&  0.31761E+00_JPRB, 0.32333E+00_JPRB, 0.32727E+00_JPRB, 0.33676E+00_JPRB, 0.33788E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.34512E+00_JPRB, 0.34701E+00_JPRB, 0.33626E+00_JPRB, 0.32825E+00_JPRB, 0.31626E+00_JPRB, 0.30469E+00_JPRB, 0.29150E+00_JPRB ,&
&  0.29233E+00_JPRB, 0.27482E+00_JPRB, 0.26079E+00_JPRB, 0.24916E+00_JPRB, 0.24357E+00_JPRB, 0.24131E+00_JPRB, 0.24927E+00_JPRB ,&
&  0.26774E+00_JPRB, 0.29122E+00_JPRB, 0.30749E+00_JPRB, 0.30934E+00_JPRB, 0.30958E+00_JPRB, 0.30860E+00_JPRB, 0.30971E+00_JPRB ,&
&  0.31063E+00_JPRB, 0.31853E+00_JPRB, 0.33409E+00_JPRB, 0.35898E+00_JPRB, 0.38476E+00_JPRB, 0.39915E+00_JPRB, 0.40063E+00_JPRB ,&
&  0.39686E+00_JPRB, 0.39590E+00_JPRB, 0.40099E+00_JPRB, 0.40450E+00_JPRB, 0.37741E+00_JPRB, 0.33023E+00_JPRB, 0.28544E+00_JPRB ,&
&  0.25094E+00_JPRB, 0.24488E+00_JPRB, 0.27515E+00_JPRB, 0.37059E+00_JPRB, 0.39589E+00_JPRB, 0.41384E+00_JPRB, 0.43058E+00_JPRB ,&
&  0.42366E+00_JPRB, 0.42544E+00_JPRB, 0.45728E+00_JPRB, 0.52366E+00_JPRB, 0.54106E+00_JPRB, 0.57852E+00_JPRB, 0.60933E+00_JPRB ,&
&  0.64873E+00_JPRB, 0.65968E+00_JPRB, 0.70007E+00_JPRB, 0.66273E+00_JPRB, 0.77419E+00_JPRB, 0.10692E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.10675E+01_JPRB, 0.10844E+01_JPRB, 0.11736E+01_JPRB, 0.12124E+01_JPRB, 0.12075E+01_JPRB, 0.12136E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.10904E+01_JPRB, 0.10332E+01_JPRB, 0.97929E+00_JPRB, 0.99416E+00_JPRB, 0.99517E+00_JPRB, 0.97828E+00_JPRB, 0.94821E+00_JPRB ,&
&  0.91846E+00_JPRB, 0.88162E+00_JPRB, 0.81688E+00_JPRB, 0.73140E+00_JPRB, 0.68408E+00_JPRB, 0.64359E+00_JPRB, 0.61638E+00_JPRB ,&
&  0.59696E+00_JPRB, 0.60045E+00_JPRB, 0.61406E+00_JPRB, 0.61848E+00_JPRB, 0.55802E+00_JPRB, 0.46884E+00_JPRB, 0.39998E+00_JPRB ,&
&  0.35503E+00_JPRB, 0.32977E+00_JPRB, 0.30281E+00_JPRB, 0.29444E+00_JPRB, 0.29241E+00_JPRB, 0.29217E+00_JPRB, 0.28534E+00_JPRB ,&
&  0.27960E+00_JPRB, 0.28308E+00_JPRB, 0.30181E+00_JPRB, 0.32525E+00_JPRB, 0.33786E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.31982E+00_JPRB, 0.33875E+00_JPRB, 0.34568E+00_JPRB, 0.33604E+00_JPRB, 0.32616E+00_JPRB, 0.30174E+00_JPRB, 0.28068E+00_JPRB ,&
&  0.27349E+00_JPRB, 0.27059E+00_JPRB, 0.27239E+00_JPRB, 0.27561E+00_JPRB, 0.28141E+00_JPRB, 0.29163E+00_JPRB, 0.30384E+00_JPRB ,&
&  0.31095E+00_JPRB, 0.31490E+00_JPRB, 0.31989E+00_JPRB, 0.32755E+00_JPRB, 0.33331E+00_JPRB, 0.33409E+00_JPRB, 0.33287E+00_JPRB ,&
&  0.32918E+00_JPRB, 0.32894E+00_JPRB, 0.33363E+00_JPRB, 0.34362E+00_JPRB, 0.35643E+00_JPRB, 0.37148E+00_JPRB, 0.38429E+00_JPRB ,&
&  0.38828E+00_JPRB, 0.37935E+00_JPRB, 0.37637E+00_JPRB, 0.37179E+00_JPRB, 0.33921E+00_JPRB, 0.30606E+00_JPRB, 0.26643E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.22192E+00_JPRB, 0.21686E+00_JPRB, 0.23153E+00_JPRB, 0.27528E+00_JPRB, 0.36815E+00_JPRB, 0.39378E+00_JPRB ,&
&  0.41664E+00_JPRB, 0.43035E+00_JPRB, 0.44224E+00_JPRB, 0.45808E+00_JPRB, 0.47435E+00_JPRB, 0.50037E+00_JPRB, 0.52237E+00_JPRB ,&
&  0.54735E+00_JPRB, 0.56885E+00_JPRB, 0.60218E+00_JPRB, 0.55195E+00_JPRB, 0.56544E+00_JPRB, 0.66642E+00_JPRB, 0.77417E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.88882E+00_JPRB, 0.91766E+00_JPRB, 0.10144E+01_JPRB, 0.10651E+01_JPRB, 0.11310E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.10400E+01_JPRB, 0.98345E+00_JPRB, 0.95879E+00_JPRB, 0.97030E+00_JPRB, 0.98452E+00_JPRB, 0.94818E+00_JPRB ,&
&  0.91401E+00_JPRB, 0.87402E+00_JPRB, 0.79781E+00_JPRB, 0.68980E+00_JPRB, 0.64782E+00_JPRB, 0.63818E+00_JPRB, 0.64136E+00_JPRB ,&
&  0.63064E+00_JPRB, 0.62542E+00_JPRB, 0.62444E+00_JPRB, 0.62266E+00_JPRB, 0.54725E+00_JPRB, 0.47280E+00_JPRB, 0.41477E+00_JPRB ,&
&  0.38254E+00_JPRB, 0.36251E+00_JPRB, 0.34050E+00_JPRB, 0.33903E+00_JPRB, 0.32588E+00_JPRB, 0.31065E+00_JPRB, 0.29264E+00_JPRB ,&
&  0.28280E+00_JPRB, 0.27964E+00_JPRB, 0.28692E+00_JPRB, 0.29857E+00_JPRB, 0.30959E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.33663E+00_JPRB, 0.34522E+00_JPRB, 0.35006E+00_JPRB, 0.34985E+00_JPRB, 0.34536E+00_JPRB, 0.33525E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.32931E+00_JPRB, 0.33414E+00_JPRB, 0.33851E+00_JPRB, 0.34007E+00_JPRB, 0.34274E+00_JPRB, 0.34551E+00_JPRB, 0.34863E+00_JPRB ,&
&  0.35171E+00_JPRB, 0.35630E+00_JPRB, 0.36241E+00_JPRB, 0.36106E+00_JPRB, 0.35822E+00_JPRB, 0.35603E+00_JPRB, 0.35287E+00_JPRB ,&
&  0.34812E+00_JPRB, 0.34446E+00_JPRB, 0.34221E+00_JPRB, 0.34490E+00_JPRB, 0.34499E+00_JPRB, 0.34765E+00_JPRB, 0.34656E+00_JPRB ,&
&  0.34267E+00_JPRB, 0.34486E+00_JPRB, 0.35285E+00_JPRB, 0.33222E+00_JPRB, 0.30241E+00_JPRB, 0.27903E+00_JPRB, 0.24618E+00_JPRB ,&
&  0.22070E+00_JPRB, 0.20238E+00_JPRB, 0.19091E+00_JPRB, 0.18847E+00_JPRB, 0.19628E+00_JPRB, 0.21801E+00_JPRB, 0.26742E+00_JPRB ,&
&  0.37204E+00_JPRB, 0.39784E+00_JPRB, 0.40987E+00_JPRB, 0.42664E+00_JPRB, 0.44578E+00_JPRB, 0.46633E+00_JPRB, 0.48615E+00_JPRB ,&
&  0.50259E+00_JPRB, 0.51381E+00_JPRB, 0.53182E+00_JPRB, 0.54777E+00_JPRB, 0.57336E+00_JPRB, 0.57647E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.76797E+00_JPRB, 0.80882E+00_JPRB, 0.76664E+00_JPRB, 0.79622E+00_JPRB, 0.85492E+00_JPRB, 0.91102E+00_JPRB, 0.95871E+00_JPRB ,&
&  0.97956E+00_JPRB, 0.97920E+00_JPRB, 0.97710E+00_JPRB, 0.96321E+00_JPRB, 0.95914E+00_JPRB, 0.95753E+00_JPRB, 0.94639E+00_JPRB ,&
&  0.92516E+00_JPRB, 0.91823E+00_JPRB, 0.97127E+00_JPRB, 0.80898E+00_JPRB, 0.73398E+00_JPRB, 0.71208E+00_JPRB, 0.67370E+00_JPRB ,&
&  0.64467E+00_JPRB, 0.63220E+00_JPRB, 0.63583E+00_JPRB, 0.63300E+00_JPRB, 0.58412E+00_JPRB, 0.52460E+00_JPRB, 0.46533E+00_JPRB ,&
&  0.43042E+00_JPRB, 0.39992E+00_JPRB, 0.36850E+00_JPRB, 0.35320E+00_JPRB, 0.33844E+00_JPRB, 0.32071E+00_JPRB, 0.30376E+00_JPRB ,&
&  0.29873E+00_JPRB, 0.29897E+00_JPRB, 0.30486E+00_JPRB, 0.32081E+00_JPRB, 0.33060E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.33490E+00_JPRB, 0.34681E+00_JPRB, 0.35626E+00_JPRB, 0.36096E+00_JPRB, 0.36150E+00_JPRB, 0.36232E+00_JPRB, 0.36265E+00_JPRB ,&
&  0.36395E+00_JPRB, 0.36591E+00_JPRB, 0.36763E+00_JPRB, 0.36799E+00_JPRB, 0.36844E+00_JPRB, 0.36839E+00_JPRB, 0.37002E+00_JPRB ,&
&  0.37189E+00_JPRB, 0.37129E+00_JPRB, 0.37241E+00_JPRB, 0.37621E+00_JPRB, 0.37721E+00_JPRB, 0.37300E+00_JPRB, 0.36446E+00_JPRB ,&
&  0.35296E+00_JPRB, 0.34488E+00_JPRB, 0.33821E+00_JPRB, 0.33029E+00_JPRB, 0.32325E+00_JPRB, 0.31747E+00_JPRB, 0.31924E+00_JPRB ,&
&  0.32956E+00_JPRB, 0.32426E+00_JPRB, 0.31709E+00_JPRB, 0.29432E+00_JPRB, 0.27202E+00_JPRB, 0.25912E+00_JPRB, 0.22340E+00_JPRB ,&
&  0.19860E+00_JPRB, 0.18431E+00_JPRB, 0.17905E+00_JPRB, 0.17782E+00_JPRB, 0.18282E+00_JPRB, 0.19646E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.27776E+00_JPRB, 0.36741E+00_JPRB, 0.38621E+00_JPRB, 0.40668E+00_JPRB, 0.43068E+00_JPRB, 0.45450E+00_JPRB, 0.47183E+00_JPRB ,&
&  0.48548E+00_JPRB, 0.49619E+00_JPRB, 0.51688E+00_JPRB, 0.53635E+00_JPRB, 0.56791E+00_JPRB, 0.59567E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.64336E+00_JPRB, 0.63907E+00_JPRB, 0.65992E+00_JPRB, 0.67820E+00_JPRB, 0.72237E+00_JPRB, 0.76660E+00_JPRB, 0.76233E+00_JPRB ,&
&  0.76380E+00_JPRB, 0.80062E+00_JPRB, 0.80200E+00_JPRB, 0.81732E+00_JPRB, 0.85695E+00_JPRB, 0.90781E+00_JPRB, 0.92008E+00_JPRB ,&
&  0.88508E+00_JPRB, 0.86864E+00_JPRB, 0.92004E+00_JPRB, 0.86844E+00_JPRB, 0.82352E+00_JPRB, 0.77119E+00_JPRB, 0.71423E+00_JPRB ,&
&  0.67659E+00_JPRB, 0.65093E+00_JPRB, 0.63584E+00_JPRB, 0.62252E+00_JPRB, 0.59779E+00_JPRB, 0.56623E+00_JPRB, 0.52129E+00_JPRB ,&
&  0.46875E+00_JPRB, 0.42523E+00_JPRB, 0.39155E+00_JPRB, 0.37064E+00_JPRB, 0.35397E+00_JPRB, 0.33653E+00_JPRB, 0.32371E+00_JPRB ,&
&  0.31428E+00_JPRB, 0.31220E+00_JPRB, 0.31382E+00_JPRB, 0.32041E+00_JPRB, 0.32723E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.35284E+00_JPRB, 0.36046E+00_JPRB, 0.36760E+00_JPRB, 0.37212E+00_JPRB, 0.37481E+00_JPRB, 0.37652E+00_JPRB, 0.37644E+00_JPRB ,&
&  0.37671E+00_JPRB, 0.37680E+00_JPRB, 0.37856E+00_JPRB, 0.38039E+00_JPRB, 0.38405E+00_JPRB, 0.38730E+00_JPRB, 0.39244E+00_JPRB ,&
&  0.39968E+00_JPRB, 0.40613E+00_JPRB, 0.40676E+00_JPRB, 0.40180E+00_JPRB, 0.39309E+00_JPRB, 0.38386E+00_JPRB, 0.36702E+00_JPRB ,&
&  0.35140E+00_JPRB, 0.34402E+00_JPRB, 0.33182E+00_JPRB, 0.31924E+00_JPRB, 0.31267E+00_JPRB, 0.30707E+00_JPRB, 0.30767E+00_JPRB ,&
&  0.30553E+00_JPRB, 0.29690E+00_JPRB, 0.28572E+00_JPRB, 0.26955E+00_JPRB, 0.25516E+00_JPRB, 0.21450E+00_JPRB, 0.20301E+00_JPRB ,&
&  0.19051E+00_JPRB, 0.18180E+00_JPRB, 0.17887E+00_JPRB, 0.17866E+00_JPRB, 0.18150E+00_JPRB, 0.19015E+00_JPRB, 0.20381E+00_JPRB ,&
&  0.22727E+00_JPRB, 0.33534E+00_JPRB, 0.36117E+00_JPRB, 0.37357E+00_JPRB, 0.39381E+00_JPRB, 0.41576E+00_JPRB, 0.43480E+00_JPRB ,&
&  0.45183E+00_JPRB, 0.46516E+00_JPRB, 0.48511E+00_JPRB, 0.50319E+00_JPRB, 0.52435E+00_JPRB, 0.54347E+00_JPRB, 0.55751E+00_JPRB ,&
&  0.56982E+00_JPRB, 0.57722E+00_JPRB, 0.60337E+00_JPRB, 0.61375E+00_JPRB, 0.61377E+00_JPRB, 0.64273E+00_JPRB, 0.66311E+00_JPRB ,&
&  0.66298E+00_JPRB, 0.65530E+00_JPRB, 0.66907E+00_JPRB, 0.70491E+00_JPRB, 0.75238E+00_JPRB, 0.79429E+00_JPRB, 0.80235E+00_JPRB ,&
&  0.80539E+00_JPRB, 0.79369E+00_JPRB, 0.78225E+00_JPRB, 0.77071E+00_JPRB, 0.73678E+00_JPRB, 0.70162E+00_JPRB, 0.67517E+00_JPRB ,&
&  0.64901E+00_JPRB, 0.63783E+00_JPRB, 0.61863E+00_JPRB, 0.59996E+00_JPRB, 0.57758E+00_JPRB, 0.55467E+00_JPRB, 0.52707E+00_JPRB ,&
&  0.49279E+00_JPRB, 0.45623E+00_JPRB, 0.42709E+00_JPRB, 0.40196E+00_JPRB, 0.38425E+00_JPRB, 0.36578E+00_JPRB, 0.35615E+00_JPRB ,&
&  0.34656E+00_JPRB, 0.34420E+00_JPRB, 0.34045E+00_JPRB, 0.34233E+00_JPRB, 0.34482E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.38751E+00_JPRB, 0.39004E+00_JPRB, 0.39278E+00_JPRB, 0.39501E+00_JPRB, 0.39647E+00_JPRB, 0.39783E+00_JPRB, 0.39838E+00_JPRB ,&
&  0.39898E+00_JPRB, 0.39963E+00_JPRB, 0.40135E+00_JPRB, 0.40339E+00_JPRB, 0.40652E+00_JPRB, 0.41033E+00_JPRB, 0.41501E+00_JPRB ,&
&  0.42042E+00_JPRB, 0.42101E+00_JPRB, 0.41432E+00_JPRB, 0.40656E+00_JPRB, 0.39762E+00_JPRB, 0.38357E+00_JPRB, 0.36500E+00_JPRB ,&
&  0.34769E+00_JPRB, 0.33478E+00_JPRB, 0.31627E+00_JPRB, 0.30165E+00_JPRB, 0.28885E+00_JPRB, 0.28359E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.27912E+00_JPRB, 0.24960E+00_JPRB, 0.23996E+00_JPRB, 0.22001E+00_JPRB, 0.20561E+00_JPRB, 0.19418E+00_JPRB, 0.18709E+00_JPRB ,&
&  0.18382E+00_JPRB, 0.18112E+00_JPRB, 0.18109E+00_JPRB, 0.18354E+00_JPRB, 0.18774E+00_JPRB, 0.19692E+00_JPRB, 0.20792E+00_JPRB ,&
&  0.23724E+00_JPRB, 0.33082E+00_JPRB, 0.36390E+00_JPRB, 0.37287E+00_JPRB, 0.37773E+00_JPRB, 0.39275E+00_JPRB, 0.40896E+00_JPRB ,&
&  0.43050E+00_JPRB, 0.44495E+00_JPRB, 0.45853E+00_JPRB, 0.46596E+00_JPRB, 0.48600E+00_JPRB, 0.50456E+00_JPRB, 0.51948E+00_JPRB ,&
&  0.53469E+00_JPRB, 0.54859E+00_JPRB, 0.56285E+00_JPRB, 0.57212E+00_JPRB, 0.57124E+00_JPRB, 0.57286E+00_JPRB, 0.57242E+00_JPRB ,&
&  0.57727E+00_JPRB, 0.58189E+00_JPRB, 0.59876E+00_JPRB, 0.62355E+00_JPRB, 0.65123E+00_JPRB, 0.67210E+00_JPRB, 0.68283E+00_JPRB ,&
&  0.69148E+00_JPRB, 0.69480E+00_JPRB, 0.70295E+00_JPRB, 0.70716E+00_JPRB, 0.69581E+00_JPRB, 0.67631E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.64811E+00_JPRB, 0.62262E+00_JPRB, 0.59844E+00_JPRB, 0.57700E+00_JPRB, 0.55760E+00_JPRB, 0.53963E+00_JPRB, 0.52325E+00_JPRB ,&
&  0.50689E+00_JPRB, 0.48883E+00_JPRB, 0.47044E+00_JPRB, 0.44864E+00_JPRB, 0.42923E+00_JPRB, 0.41075E+00_JPRB, 0.39985E+00_JPRB ,&
&  0.38994E+00_JPRB, 0.38683E+00_JPRB, 0.38268E+00_JPRB, 0.38353E+00_JPRB, 0.38365E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.42589E+00_JPRB, 0.42471E+00_JPRB, 0.42481E+00_JPRB, 0.42456E+00_JPRB, 0.42534E+00_JPRB, 0.42525E+00_JPRB, 0.42609E+00_JPRB ,&
&  0.42666E+00_JPRB, 0.42780E+00_JPRB, 0.42820E+00_JPRB, 0.42848E+00_JPRB, 0.42846E+00_JPRB, 0.42907E+00_JPRB, 0.42996E+00_JPRB ,&
&  0.42982E+00_JPRB, 0.42646E+00_JPRB, 0.41808E+00_JPRB, 0.40880E+00_JPRB, 0.39803E+00_JPRB, 0.38490E+00_JPRB, 0.36936E+00_JPRB ,&
&  0.34939E+00_JPRB, 0.33100E+00_JPRB, 0.30417E+00_JPRB, 0.28905E+00_JPRB, 0.27296E+00_JPRB, 0.26168E+00_JPRB, 0.25477E+00_JPRB ,&
&  0.24837E+00_JPRB, 0.24162E+00_JPRB, 0.23508E+00_JPRB, 0.22634E+00_JPRB, 0.21866E+00_JPRB, 0.20839E+00_JPRB, 0.20197E+00_JPRB ,&
&  0.19734E+00_JPRB, 0.19525E+00_JPRB, 0.19512E+00_JPRB, 0.19761E+00_JPRB, 0.20283E+00_JPRB, 0.21032E+00_JPRB, 0.22407E+00_JPRB ,&
&  0.25473E+00_JPRB, 0.29876E+00_JPRB, 0.36645E+00_JPRB, 0.38039E+00_JPRB, 0.37855E+00_JPRB, 0.38732E+00_JPRB, 0.39712E+00_JPRB ,&
&  0.40607E+00_JPRB, 0.41485E+00_JPRB, 0.42228E+00_JPRB, 0.42581E+00_JPRB, 0.43065E+00_JPRB, 0.43500E+00_JPRB, 0.43943E+00_JPRB ,&
&  0.44441E+00_JPRB, 0.45472E+00_JPRB, 0.47093E+00_JPRB, 0.48980E+00_JPRB, 0.50952E+00_JPRB, 0.52646E+00_JPRB, 0.53836E+00_JPRB ,&
&  0.54618E+00_JPRB, 0.55257E+00_JPRB, 0.56490E+00_JPRB, 0.57961E+00_JPRB, 0.59713E+00_JPRB, 0.61007E+00_JPRB, 0.62145E+00_JPRB ,&
&  0.62937E+00_JPRB, 0.62891E+00_JPRB, 0.62206E+00_JPRB, 0.61371E+00_JPRB, 0.60808E+00_JPRB, 0.60964E+00_JPRB, 0.60562E+00_JPRB ,&
&  0.59986E+00_JPRB, 0.59156E+00_JPRB, 0.57870E+00_JPRB, 0.56217E+00_JPRB, 0.54740E+00_JPRB, 0.53334E+00_JPRB, 0.52249E+00_JPRB ,&
&  0.51235E+00_JPRB, 0.50307E+00_JPRB, 0.49323E+00_JPRB, 0.48127E+00_JPRB, 0.46977E+00_JPRB, 0.45834E+00_JPRB, 0.44967E+00_JPRB ,&
&  0.44133E+00_JPRB, 0.43630E+00_JPRB, 0.43170E+00_JPRB, 0.42941E+00_JPRB, 0.42688E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.47600E+00_JPRB, 0.47442E+00_JPRB, 0.47304E+00_JPRB, 0.47161E+00_JPRB, 0.46954E+00_JPRB, 0.46700E+00_JPRB, 0.46381E+00_JPRB ,&
&  0.45923E+00_JPRB, 0.45407E+00_JPRB, 0.44872E+00_JPRB, 0.44330E+00_JPRB, 0.43810E+00_JPRB, 0.43392E+00_JPRB, 0.43064E+00_JPRB ,&
&  0.42797E+00_JPRB, 0.42358E+00_JPRB, 0.41600E+00_JPRB, 0.40640E+00_JPRB, 0.39813E+00_JPRB, 0.39110E+00_JPRB, 0.38419E+00_JPRB ,&
&  0.37092E+00_JPRB, 0.35667E+00_JPRB, 0.34332E+00_JPRB, 0.32844E+00_JPRB, 0.31665E+00_JPRB, 0.28806E+00_JPRB, 0.27841E+00_JPRB ,&
&  0.26986E+00_JPRB, 0.26501E+00_JPRB, 0.26365E+00_JPRB, 0.26350E+00_JPRB, 0.26940E+00_JPRB, 0.26492E+00_JPRB, 0.26176E+00_JPRB ,&
&  0.24885E+00_JPRB, 0.24807E+00_JPRB, 0.25009E+00_JPRB, 0.25344E+00_JPRB, 0.26118E+00_JPRB, 0.27111E+00_JPRB, 0.28687E+00_JPRB ,&
&  0.31518E+00_JPRB, 0.32801E+00_JPRB, 0.34743E+00_JPRB, 0.36242E+00_JPRB, 0.37098E+00_JPRB, 0.37519E+00_JPRB, 0.37984E+00_JPRB ,&
&  0.38502E+00_JPRB, 0.39320E+00_JPRB, 0.39807E+00_JPRB, 0.40138E+00_JPRB, 0.40092E+00_JPRB, 0.39906E+00_JPRB, 0.39839E+00_JPRB ,&
&  0.39674E+00_JPRB, 0.39841E+00_JPRB, 0.40297E+00_JPRB, 0.41298E+00_JPRB, 0.42717E+00_JPRB, 0.44365E+00_JPRB, 0.46068E+00_JPRB ,&
&  0.47608E+00_JPRB, 0.49262E+00_JPRB, 0.50654E+00_JPRB, 0.52061E+00_JPRB, 0.53631E+00_JPRB, 0.55070E+00_JPRB, 0.56286E+00_JPRB ,&
&  0.57149E+00_JPRB, 0.57493E+00_JPRB, 0.57320E+00_JPRB, 0.57068E+00_JPRB, 0.56567E+00_JPRB, 0.56190E+00_JPRB, 0.56045E+00_JPRB ,&
&  0.55850E+00_JPRB, 0.55700E+00_JPRB, 0.55533E+00_JPRB, 0.55090E+00_JPRB, 0.54538E+00_JPRB, 0.53850E+00_JPRB, 0.53129E+00_JPRB ,&
&  0.52376E+00_JPRB, 0.51661E+00_JPRB, 0.51031E+00_JPRB, 0.50484E+00_JPRB, 0.50076E+00_JPRB, 0.49586E+00_JPRB, 0.49179E+00_JPRB ,&
&  0.48733E+00_JPRB, 0.48525E+00_JPRB, 0.48230E+00_JPRB, 0.48001E+00_JPRB, 0.47762E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.46701E+00_JPRB, 0.46334E+00_JPRB, 0.45976E+00_JPRB, 0.45653E+00_JPRB, 0.45246E+00_JPRB, 0.44847E+00_JPRB, 0.44445E+00_JPRB ,&
&  0.44042E+00_JPRB, 0.43608E+00_JPRB, 0.43172E+00_JPRB, 0.42721E+00_JPRB, 0.42261E+00_JPRB, 0.41906E+00_JPRB, 0.41660E+00_JPRB ,&
&  0.41427E+00_JPRB, 0.41110E+00_JPRB, 0.40762E+00_JPRB, 0.40278E+00_JPRB, 0.39776E+00_JPRB, 0.39311E+00_JPRB, 0.38934E+00_JPRB ,&
&  0.38423E+00_JPRB, 0.37874E+00_JPRB, 0.37302E+00_JPRB, 0.36725E+00_JPRB, 0.36140E+00_JPRB, 0.35556E+00_JPRB, 0.34960E+00_JPRB ,&
&  0.34402E+00_JPRB, 0.34010E+00_JPRB, 0.33639E+00_JPRB, 0.33227E+00_JPRB, 0.32763E+00_JPRB, 0.32639E+00_JPRB, 0.32502E+00_JPRB ,&
&  0.32386E+00_JPRB, 0.32385E+00_JPRB, 0.32687E+00_JPRB, 0.32959E+00_JPRB, 0.33242E+00_JPRB, 0.33544E+00_JPRB, 0.34071E+00_JPRB ,&
&  0.34555E+00_JPRB, 0.35134E+00_JPRB, 0.35872E+00_JPRB, 0.36607E+00_JPRB, 0.37202E+00_JPRB, 0.37610E+00_JPRB, 0.38196E+00_JPRB ,&
&  0.38605E+00_JPRB, 0.39144E+00_JPRB, 0.39516E+00_JPRB, 0.39671E+00_JPRB, 0.39607E+00_JPRB, 0.39540E+00_JPRB, 0.39548E+00_JPRB ,&
&  0.39454E+00_JPRB, 0.39640E+00_JPRB, 0.39814E+00_JPRB, 0.40005E+00_JPRB, 0.40026E+00_JPRB, 0.40285E+00_JPRB, 0.40438E+00_JPRB ,&
&  0.40824E+00_JPRB, 0.41470E+00_JPRB, 0.42147E+00_JPRB, 0.42923E+00_JPRB, 0.43872E+00_JPRB, 0.45067E+00_JPRB, 0.46034E+00_JPRB ,&
&  0.46927E+00_JPRB, 0.47677E+00_JPRB, 0.48305E+00_JPRB, 0.48625E+00_JPRB, 0.48928E+00_JPRB, 0.49239E+00_JPRB, 0.49619E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.49957E+00_JPRB, 0.50098E+00_JPRB, 0.50169E+00_JPRB, 0.50010E+00_JPRB, 0.49834E+00_JPRB, 0.49677E+00_JPRB ,&
&  0.49496E+00_JPRB, 0.49293E+00_JPRB, 0.49126E+00_JPRB, 0.49016E+00_JPRB, 0.48857E+00_JPRB, 0.48648E+00_JPRB, 0.48440E+00_JPRB ,&
&  0.48247E+00_JPRB, 0.47978E+00_JPRB, 0.47685E+00_JPRB, 0.47382E+00_JPRB, 0.47089E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.41979E+00_JPRB, 0.41837E+00_JPRB, 0.41661E+00_JPRB, 0.41518E+00_JPRB, 0.41332E+00_JPRB, 0.41188E+00_JPRB, 0.41020E+00_JPRB ,&
&  0.40876E+00_JPRB, 0.40734E+00_JPRB, 0.40611E+00_JPRB, 0.40448E+00_JPRB, 0.40300E+00_JPRB, 0.40151E+00_JPRB, 0.40013E+00_JPRB ,&
&  0.39860E+00_JPRB, 0.39728E+00_JPRB, 0.39583E+00_JPRB, 0.39420E+00_JPRB, 0.39290E+00_JPRB, 0.39141E+00_JPRB, 0.38907E+00_JPRB ,&
&  0.38540E+00_JPRB, 0.38143E+00_JPRB, 0.37784E+00_JPRB, 0.37507E+00_JPRB, 0.37318E+00_JPRB, 0.37180E+00_JPRB, 0.37058E+00_JPRB ,&
&  0.36990E+00_JPRB, 0.36967E+00_JPRB, 0.36951E+00_JPRB, 0.36903E+00_JPRB, 0.36868E+00_JPRB, 0.36935E+00_JPRB, 0.36970E+00_JPRB ,&
&  0.36973E+00_JPRB, 0.36984E+00_JPRB, 0.37064E+00_JPRB, 0.37137E+00_JPRB, 0.37206E+00_JPRB, 0.37318E+00_JPRB, 0.37512E+00_JPRB ,&
&  0.37708E+00_JPRB, 0.37880E+00_JPRB, 0.38101E+00_JPRB, 0.38332E+00_JPRB, 0.38554E+00_JPRB, 0.38764E+00_JPRB, 0.39057E+00_JPRB ,&
&  0.39358E+00_JPRB, 0.39637E+00_JPRB, 0.39888E+00_JPRB, 0.40071E+00_JPRB, 0.40201E+00_JPRB, 0.40252E+00_JPRB, 0.40253E+00_JPRB ,&
&  0.40202E+00_JPRB, 0.40218E+00_JPRB, 0.40177E+00_JPRB, 0.40129E+00_JPRB, 0.40037E+00_JPRB, 0.40044E+00_JPRB, 0.40038E+00_JPRB ,&
&  0.40107E+00_JPRB, 0.40143E+00_JPRB, 0.40244E+00_JPRB, 0.40335E+00_JPRB, 0.40498E+00_JPRB, 0.40680E+00_JPRB, 0.40872E+00_JPRB ,&
&  0.41093E+00_JPRB, 0.41356E+00_JPRB, 0.41686E+00_JPRB, 0.41864E+00_JPRB, 0.42035E+00_JPRB, 0.42198E+00_JPRB, 0.42403E+00_JPRB ,&
&  0.42569E+00_JPRB, 0.42716E+00_JPRB, 0.42865E+00_JPRB, 0.42971E+00_JPRB, 0.43034E+00_JPRB, 0.43080E+00_JPRB, 0.43123E+00_JPRB ,&
&  0.43116E+00_JPRB, 0.43107E+00_JPRB, 0.43092E+00_JPRB, 0.43087E+00_JPRB, 0.43006E+00_JPRB, 0.42903E+00_JPRB, 0.42778E+00_JPRB ,&
&  0.42682E+00_JPRB, 0.42543E+00_JPRB, 0.42419E+00_JPRB, 0.42276E+00_JPRB, 0.42145E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB ,&
&  0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB, 0.39382E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB ,&
&  0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB, 0.22987E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.24350E+00_JPRB, 0.24379E+00_JPRB, 0.24404E+00_JPRB, 0.24429E+00_JPRB, 0.24358E+00_JPRB, 0.24425E+00_JPRB, 0.24476E+00_JPRB ,&
&  0.24548E+00_JPRB, 0.24656E+00_JPRB, 0.24786E+00_JPRB, 0.24887E+00_JPRB, 0.24966E+00_JPRB, 0.25014E+00_JPRB, 0.25046E+00_JPRB ,&
&  0.25049E+00_JPRB, 0.25063E+00_JPRB, 0.24786E+00_JPRB, 0.24689E+00_JPRB, 0.24664E+00_JPRB, 0.24674E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.24714E+00_JPRB, 0.24720E+00_JPRB, 0.24683E+00_JPRB, 0.24710E+00_JPRB, 0.24738E+00_JPRB, 0.24759E+00_JPRB, 0.24784E+00_JPRB ,&
&  0.24805E+00_JPRB, 0.24821E+00_JPRB, 0.24814E+00_JPRB, 0.24794E+00_JPRB, 0.24775E+00_JPRB, 0.24720E+00_JPRB, 0.24633E+00_JPRB ,&
&  0.24527E+00_JPRB, 0.24396E+00_JPRB, 0.24267E+00_JPRB, 0.24148E+00_JPRB, 0.24044E+00_JPRB, 0.23956E+00_JPRB, 0.23872E+00_JPRB ,&
&  0.23774E+00_JPRB, 0.23648E+00_JPRB, 0.23515E+00_JPRB, 0.23387E+00_JPRB, 0.23264E+00_JPRB, 0.23152E+00_JPRB, 0.23036E+00_JPRB ,&
&  0.22946E+00_JPRB, 0.22870E+00_JPRB, 0.22792E+00_JPRB, 0.22726E+00_JPRB, 0.22662E+00_JPRB, 0.22602E+00_JPRB, 0.22544E+00_JPRB ,&
&  0.22467E+00_JPRB, 0.22422E+00_JPRB, 0.22371E+00_JPRB, 0.22344E+00_JPRB, 0.22343E+00_JPRB, 0.22340E+00_JPRB, 0.22344E+00_JPRB ,&
&  0.22354E+00_JPRB, 0.22371E+00_JPRB, 0.22381E+00_JPRB, 0.22399E+00_JPRB, 0.22426E+00_JPRB, 0.22464E+00_JPRB, 0.22511E+00_JPRB ,&
&  0.22561E+00_JPRB, 0.22617E+00_JPRB, 0.22677E+00_JPRB, 0.22745E+00_JPRB, 0.22817E+00_JPRB, 0.22897E+00_JPRB, 0.22972E+00_JPRB ,&
&  0.23048E+00_JPRB, 0.23119E+00_JPRB, 0.23189E+00_JPRB, 0.23244E+00_JPRB, 0.23319E+00_JPRB, 0.23393E+00_JPRB, 0.23480E+00_JPRB ,&
&  0.23569E+00_JPRB, 0.23662E+00_JPRB, 0.23744E+00_JPRB, 0.23830E+00_JPRB, 0.23924E+00_JPRB, 0.23915E+00_JPRB, 0.23910E+00_JPRB ,&
&  0.23913E+00_JPRB, 0.23941E+00_JPRB, 0.23951E+00_JPRB, 0.23988E+00_JPRB, 0.24069E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.27239E+00_JPRB, 0.27895E+00_JPRB, 0.28398E+00_JPRB, 0.28929E+00_JPRB, 0.28730E+00_JPRB, 0.32437E+00_JPRB, 0.32293E+00_JPRB ,&
&  0.32331E+00_JPRB, 0.30870E+00_JPRB, 0.30838E+00_JPRB, 0.30648E+00_JPRB, 0.30322E+00_JPRB, 0.29857E+00_JPRB, 0.29427E+00_JPRB ,&
&  0.29049E+00_JPRB, 0.28565E+00_JPRB, 0.28011E+00_JPRB, 0.27543E+00_JPRB, 0.27165E+00_JPRB, 0.26840E+00_JPRB, 0.26567E+00_JPRB ,&
&  0.26506E+00_JPRB, 0.26629E+00_JPRB, 0.26868E+00_JPRB, 0.26806E+00_JPRB, 0.26941E+00_JPRB, 0.27159E+00_JPRB, 0.27406E+00_JPRB ,&
&  0.27642E+00_JPRB, 0.27834E+00_JPRB, 0.27843E+00_JPRB, 0.27683E+00_JPRB, 0.27535E+00_JPRB, 0.27285E+00_JPRB, 0.27057E+00_JPRB ,&
&  0.26790E+00_JPRB, 0.26495E+00_JPRB, 0.26151E+00_JPRB, 0.25875E+00_JPRB, 0.25578E+00_JPRB, 0.25216E+00_JPRB, 0.24866E+00_JPRB ,&
&  0.24567E+00_JPRB, 0.24299E+00_JPRB, 0.24017E+00_JPRB, 0.23754E+00_JPRB, 0.23507E+00_JPRB, 0.23253E+00_JPRB, 0.23035E+00_JPRB ,&
&  0.22870E+00_JPRB, 0.22731E+00_JPRB, 0.22590E+00_JPRB, 0.22443E+00_JPRB, 0.22297E+00_JPRB, 0.22156E+00_JPRB, 0.22038E+00_JPRB ,&
&  0.22009E+00_JPRB, 0.21938E+00_JPRB, 0.21868E+00_JPRB, 0.21823E+00_JPRB, 0.21816E+00_JPRB, 0.21767E+00_JPRB, 0.21743E+00_JPRB ,&
&  0.21735E+00_JPRB, 0.21736E+00_JPRB, 0.21759E+00_JPRB, 0.21788E+00_JPRB, 0.21829E+00_JPRB, 0.21882E+00_JPRB, 0.21967E+00_JPRB ,&
&  0.22068E+00_JPRB, 0.22190E+00_JPRB, 0.22320E+00_JPRB, 0.22455E+00_JPRB, 0.22593E+00_JPRB, 0.22738E+00_JPRB, 0.22901E+00_JPRB ,&
&  0.23066E+00_JPRB, 0.23227E+00_JPRB, 0.23387E+00_JPRB, 0.23544E+00_JPRB, 0.23789E+00_JPRB, 0.24028E+00_JPRB, 0.24283E+00_JPRB ,&
&  0.24558E+00_JPRB, 0.24880E+00_JPRB, 0.25218E+00_JPRB, 0.25579E+00_JPRB, 0.25975E+00_JPRB, 0.26167E+00_JPRB, 0.26321E+00_JPRB ,&
&  0.26436E+00_JPRB, 0.26719E+00_JPRB, 0.26973E+00_JPRB, 0.27002E+00_JPRB, 0.27009E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.33104E+00_JPRB, 0.32869E+00_JPRB, 0.32890E+00_JPRB, 0.32954E+00_JPRB, 0.33255E+00_JPRB, 0.33737E+00_JPRB, 0.34264E+00_JPRB ,&
&  0.34207E+00_JPRB, 0.34499E+00_JPRB, 0.34428E+00_JPRB, 0.34201E+00_JPRB, 0.33810E+00_JPRB, 0.33468E+00_JPRB, 0.32984E+00_JPRB ,&
&  0.32295E+00_JPRB, 0.31417E+00_JPRB, 0.30559E+00_JPRB, 0.29694E+00_JPRB, 0.28866E+00_JPRB, 0.28006E+00_JPRB, 0.27462E+00_JPRB ,&
&  0.27142E+00_JPRB, 0.27056E+00_JPRB, 0.27255E+00_JPRB, 0.27549E+00_JPRB, 0.28090E+00_JPRB, 0.28826E+00_JPRB, 0.29797E+00_JPRB ,&
&  0.30762E+00_JPRB, 0.31811E+00_JPRB, 0.32352E+00_JPRB, 0.32341E+00_JPRB, 0.32310E+00_JPRB, 0.32096E+00_JPRB, 0.31745E+00_JPRB ,&
&  0.31112E+00_JPRB, 0.30379E+00_JPRB, 0.29545E+00_JPRB, 0.28663E+00_JPRB, 0.27763E+00_JPRB, 0.26890E+00_JPRB, 0.26078E+00_JPRB ,&
&  0.25416E+00_JPRB, 0.24854E+00_JPRB, 0.24345E+00_JPRB, 0.23922E+00_JPRB, 0.23591E+00_JPRB, 0.23343E+00_JPRB, 0.23056E+00_JPRB ,&
&  0.22812E+00_JPRB, 0.22598E+00_JPRB, 0.22347E+00_JPRB, 0.22170E+00_JPRB, 0.22012E+00_JPRB, 0.21908E+00_JPRB, 0.21821E+00_JPRB ,&
&  0.21803E+00_JPRB, 0.21783E+00_JPRB, 0.21824E+00_JPRB, 0.21864E+00_JPRB, 0.21915E+00_JPRB, 0.21899E+00_JPRB, 0.21869E+00_JPRB ,&
&  0.21822E+00_JPRB, 0.21780E+00_JPRB, 0.21660E+00_JPRB, 0.21569E+00_JPRB, 0.21508E+00_JPRB, 0.21441E+00_JPRB, 0.21445E+00_JPRB ,&
&  0.21532E+00_JPRB, 0.21622E+00_JPRB, 0.21699E+00_JPRB, 0.21854E+00_JPRB, 0.22010E+00_JPRB, 0.22232E+00_JPRB, 0.22498E+00_JPRB ,&
&  0.22695E+00_JPRB, 0.22928E+00_JPRB, 0.23189E+00_JPRB, 0.23490E+00_JPRB, 0.23847E+00_JPRB, 0.24220E+00_JPRB, 0.24654E+00_JPRB ,&
&  0.25143E+00_JPRB, 0.25650E+00_JPRB, 0.26223E+00_JPRB, 0.26853E+00_JPRB, 0.27618E+00_JPRB, 0.28362E+00_JPRB, 0.29422E+00_JPRB ,&
&  0.30296E+00_JPRB, 0.30425E+00_JPRB, 0.34461E+00_JPRB, 0.33834E+00_JPRB, 0.33402E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.32734E+00_JPRB, 0.32975E+00_JPRB, 0.33299E+00_JPRB, 0.33690E+00_JPRB, 0.34177E+00_JPRB, 0.34603E+00_JPRB, 0.35133E+00_JPRB ,&
&  0.35427E+00_JPRB, 0.35042E+00_JPRB, 0.35070E+00_JPRB, 0.34673E+00_JPRB, 0.34262E+00_JPRB, 0.33742E+00_JPRB, 0.33151E+00_JPRB ,&
&  0.32510E+00_JPRB, 0.31618E+00_JPRB, 0.30664E+00_JPRB, 0.29812E+00_JPRB, 0.29098E+00_JPRB, 0.28526E+00_JPRB, 0.28057E+00_JPRB ,&
&  0.27736E+00_JPRB, 0.27748E+00_JPRB, 0.27963E+00_JPRB, 0.28561E+00_JPRB, 0.29566E+00_JPRB, 0.30836E+00_JPRB, 0.32744E+00_JPRB ,&
&  0.35458E+00_JPRB, 0.35672E+00_JPRB, 0.36118E+00_JPRB, 0.36016E+00_JPRB, 0.35596E+00_JPRB, 0.35370E+00_JPRB, 0.34923E+00_JPRB ,&
&  0.34967E+00_JPRB, 0.34315E+00_JPRB, 0.35309E+00_JPRB, 0.33767E+00_JPRB, 0.31186E+00_JPRB, 0.29652E+00_JPRB, 0.27837E+00_JPRB ,&
&  0.26466E+00_JPRB, 0.25303E+00_JPRB, 0.24427E+00_JPRB, 0.24029E+00_JPRB, 0.23668E+00_JPRB, 0.23494E+00_JPRB, 0.23151E+00_JPRB ,&
&  0.22787E+00_JPRB, 0.22587E+00_JPRB, 0.22282E+00_JPRB, 0.22240E+00_JPRB, 0.22239E+00_JPRB, 0.22185E+00_JPRB, 0.22194E+00_JPRB ,&
&  0.22230E+00_JPRB, 0.22318E+00_JPRB, 0.22473E+00_JPRB, 0.22616E+00_JPRB, 0.22701E+00_JPRB, 0.22750E+00_JPRB, 0.22737E+00_JPRB ,&
&  0.22774E+00_JPRB, 0.22742E+00_JPRB, 0.22663E+00_JPRB, 0.22536E+00_JPRB, 0.22354E+00_JPRB, 0.22165E+00_JPRB, 0.21987E+00_JPRB ,&
&  0.21847E+00_JPRB, 0.21801E+00_JPRB, 0.21766E+00_JPRB, 0.21797E+00_JPRB, 0.21862E+00_JPRB, 0.22034E+00_JPRB, 0.22196E+00_JPRB ,&
&  0.22381E+00_JPRB, 0.22631E+00_JPRB, 0.22855E+00_JPRB, 0.23127E+00_JPRB, 0.23382E+00_JPRB, 0.23548E+00_JPRB, 0.23942E+00_JPRB ,&
&  0.24443E+00_JPRB, 0.25103E+00_JPRB, 0.25872E+00_JPRB, 0.26633E+00_JPRB, 0.27606E+00_JPRB, 0.29182E+00_JPRB, 0.31345E+00_JPRB ,&
&  0.33585E+00_JPRB, 0.36116E+00_JPRB, 0.35608E+00_JPRB, 0.32707E+00_JPRB, 0.32477E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.33763E+00_JPRB, 0.34415E+00_JPRB, 0.35118E+00_JPRB, 0.35866E+00_JPRB, 0.36798E+00_JPRB, 0.37191E+00_JPRB, 0.37389E+00_JPRB ,&
&  0.36989E+00_JPRB, 0.36502E+00_JPRB, 0.35403E+00_JPRB, 0.34356E+00_JPRB, 0.33394E+00_JPRB, 0.32622E+00_JPRB, 0.32064E+00_JPRB ,&
&  0.31762E+00_JPRB, 0.31528E+00_JPRB, 0.31191E+00_JPRB, 0.30870E+00_JPRB, 0.30605E+00_JPRB, 0.30596E+00_JPRB, 0.30600E+00_JPRB ,&
&  0.30509E+00_JPRB, 0.30508E+00_JPRB, 0.30388E+00_JPRB, 0.30905E+00_JPRB, 0.31810E+00_JPRB, 0.32935E+00_JPRB, 0.34399E+00_JPRB ,&
&  0.36452E+00_JPRB, 0.37586E+00_JPRB, 0.39142E+00_JPRB, 0.39208E+00_JPRB, 0.39019E+00_JPRB, 0.38631E+00_JPRB, 0.38321E+00_JPRB ,&
&  0.37862E+00_JPRB, 0.37205E+00_JPRB, 0.36733E+00_JPRB, 0.36157E+00_JPRB, 0.33286E+00_JPRB, 0.31001E+00_JPRB, 0.29253E+00_JPRB ,&
&  0.27710E+00_JPRB, 0.26256E+00_JPRB, 0.24995E+00_JPRB, 0.24352E+00_JPRB, 0.23858E+00_JPRB, 0.23566E+00_JPRB, 0.23262E+00_JPRB ,&
&  0.22941E+00_JPRB, 0.22807E+00_JPRB, 0.22611E+00_JPRB, 0.22543E+00_JPRB, 0.22635E+00_JPRB, 0.22637E+00_JPRB, 0.22715E+00_JPRB ,&
&  0.22780E+00_JPRB, 0.22873E+00_JPRB, 0.23085E+00_JPRB, 0.23248E+00_JPRB, 0.23421E+00_JPRB, 0.23587E+00_JPRB, 0.23675E+00_JPRB ,&
&  0.23844E+00_JPRB, 0.23983E+00_JPRB, 0.23979E+00_JPRB, 0.23910E+00_JPRB, 0.23744E+00_JPRB, 0.23387E+00_JPRB, 0.23025E+00_JPRB ,&
&  0.22641E+00_JPRB, 0.22361E+00_JPRB, 0.22239E+00_JPRB, 0.22146E+00_JPRB, 0.22221E+00_JPRB, 0.22302E+00_JPRB, 0.22400E+00_JPRB ,&
&  0.22584E+00_JPRB, 0.22755E+00_JPRB, 0.22949E+00_JPRB, 0.23215E+00_JPRB, 0.23427E+00_JPRB, 0.23615E+00_JPRB, 0.23857E+00_JPRB ,&
&  0.23918E+00_JPRB, 0.24334E+00_JPRB, 0.24854E+00_JPRB, 0.25498E+00_JPRB, 0.26560E+00_JPRB, 0.27541E+00_JPRB, 0.29772E+00_JPRB ,&
&  0.33472E+00_JPRB, 0.40501E+00_JPRB, 0.39959E+00_JPRB, 0.32980E+00_JPRB, 0.33199E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.36056E+00_JPRB, 0.36917E+00_JPRB, 0.37681E+00_JPRB, 0.38267E+00_JPRB, 0.38618E+00_JPRB, 0.38674E+00_JPRB, 0.38932E+00_JPRB ,&
&  0.39663E+00_JPRB, 0.40088E+00_JPRB, 0.40376E+00_JPRB, 0.40323E+00_JPRB, 0.40269E+00_JPRB, 0.38368E+00_JPRB, 0.37857E+00_JPRB ,&
&  0.37276E+00_JPRB, 0.36868E+00_JPRB, 0.36321E+00_JPRB, 0.36324E+00_JPRB, 0.36311E+00_JPRB, 0.36504E+00_JPRB, 0.36579E+00_JPRB ,&
&  0.36565E+00_JPRB, 0.36337E+00_JPRB, 0.36006E+00_JPRB, 0.35804E+00_JPRB, 0.35934E+00_JPRB, 0.36095E+00_JPRB, 0.35818E+00_JPRB ,&
&  0.35402E+00_JPRB, 0.35322E+00_JPRB, 0.35657E+00_JPRB, 0.36418E+00_JPRB, 0.40516E+00_JPRB, 0.40027E+00_JPRB, 0.39760E+00_JPRB ,&
&  0.39429E+00_JPRB, 0.39018E+00_JPRB, 0.38972E+00_JPRB, 0.38337E+00_JPRB, 0.37400E+00_JPRB, 0.35363E+00_JPRB, 0.34222E+00_JPRB ,&
&  0.30748E+00_JPRB, 0.28805E+00_JPRB, 0.27091E+00_JPRB, 0.25966E+00_JPRB, 0.24982E+00_JPRB, 0.24414E+00_JPRB, 0.23892E+00_JPRB ,&
&  0.23496E+00_JPRB, 0.23191E+00_JPRB, 0.22919E+00_JPRB, 0.22954E+00_JPRB, 0.23116E+00_JPRB, 0.23216E+00_JPRB, 0.23313E+00_JPRB ,&
&  0.23371E+00_JPRB, 0.23417E+00_JPRB, 0.23480E+00_JPRB, 0.23602E+00_JPRB, 0.23765E+00_JPRB, 0.23979E+00_JPRB, 0.24250E+00_JPRB ,&
&  0.24697E+00_JPRB, 0.25057E+00_JPRB, 0.25214E+00_JPRB, 0.25460E+00_JPRB, 0.25425E+00_JPRB, 0.24908E+00_JPRB, 0.24290E+00_JPRB ,&
&  0.23650E+00_JPRB, 0.23316E+00_JPRB, 0.22991E+00_JPRB, 0.22767E+00_JPRB, 0.22715E+00_JPRB, 0.22720E+00_JPRB, 0.22902E+00_JPRB ,&
&  0.23102E+00_JPRB, 0.23275E+00_JPRB, 0.23396E+00_JPRB, 0.23529E+00_JPRB, 0.23700E+00_JPRB, 0.23837E+00_JPRB, 0.24019E+00_JPRB ,&
&  0.24248E+00_JPRB, 0.24562E+00_JPRB, 0.25029E+00_JPRB, 0.25563E+00_JPRB, 0.26220E+00_JPRB, 0.27231E+00_JPRB, 0.29047E+00_JPRB ,&
&  0.31522E+00_JPRB, 0.37212E+00_JPRB, 0.37844E+00_JPRB, 0.34742E+00_JPRB, 0.35329E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.37306E+00_JPRB, 0.38285E+00_JPRB, 0.38806E+00_JPRB, 0.39317E+00_JPRB, 0.39519E+00_JPRB, 0.39649E+00_JPRB, 0.40429E+00_JPRB ,&
&  0.40902E+00_JPRB, 0.41268E+00_JPRB, 0.41486E+00_JPRB, 0.41545E+00_JPRB, 0.41093E+00_JPRB, 0.40425E+00_JPRB, 0.40192E+00_JPRB ,&
&  0.40070E+00_JPRB, 0.40114E+00_JPRB, 0.39997E+00_JPRB, 0.39999E+00_JPRB, 0.40054E+00_JPRB, 0.40537E+00_JPRB, 0.41209E+00_JPRB ,&
&  0.41364E+00_JPRB, 0.41830E+00_JPRB, 0.43070E+00_JPRB, 0.43843E+00_JPRB, 0.44333E+00_JPRB, 0.44315E+00_JPRB, 0.43762E+00_JPRB ,&
&  0.41097E+00_JPRB, 0.38332E+00_JPRB, 0.35558E+00_JPRB, 0.35716E+00_JPRB, 0.41121E+00_JPRB, 0.41783E+00_JPRB, 0.41684E+00_JPRB ,&
&  0.41522E+00_JPRB, 0.41966E+00_JPRB, 0.41982E+00_JPRB, 0.40536E+00_JPRB, 0.39476E+00_JPRB, 0.37993E+00_JPRB, 0.36732E+00_JPRB ,&
&  0.35685E+00_JPRB, 0.34905E+00_JPRB, 0.33613E+00_JPRB, 0.30918E+00_JPRB, 0.29128E+00_JPRB, 0.28213E+00_JPRB, 0.26846E+00_JPRB ,&
&  0.25826E+00_JPRB, 0.25370E+00_JPRB, 0.25256E+00_JPRB, 0.25237E+00_JPRB, 0.25649E+00_JPRB, 0.25836E+00_JPRB, 0.25737E+00_JPRB ,&
&  0.25456E+00_JPRB, 0.25214E+00_JPRB, 0.24865E+00_JPRB, 0.24362E+00_JPRB, 0.24088E+00_JPRB, 0.23991E+00_JPRB, 0.24017E+00_JPRB ,&
&  0.24363E+00_JPRB, 0.24932E+00_JPRB, 0.26217E+00_JPRB, 0.27143E+00_JPRB, 0.26870E+00_JPRB, 0.25786E+00_JPRB, 0.24741E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.23816E+00_JPRB, 0.23613E+00_JPRB, 0.23408E+00_JPRB, 0.23464E+00_JPRB, 0.23652E+00_JPRB, 0.23801E+00_JPRB ,&
&  0.23835E+00_JPRB, 0.23984E+00_JPRB, 0.24063E+00_JPRB, 0.24167E+00_JPRB, 0.24257E+00_JPRB, 0.24409E+00_JPRB, 0.24623E+00_JPRB ,&
&  0.24841E+00_JPRB, 0.25353E+00_JPRB, 0.25931E+00_JPRB, 0.26569E+00_JPRB, 0.26969E+00_JPRB, 0.27267E+00_JPRB, 0.27832E+00_JPRB ,&
&  0.28469E+00_JPRB, 0.30047E+00_JPRB, 0.33085E+00_JPRB, 0.36874E+00_JPRB, 0.36438E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.38310E+00_JPRB, 0.39008E+00_JPRB, 0.39824E+00_JPRB, 0.40901E+00_JPRB, 0.41139E+00_JPRB, 0.41298E+00_JPRB, 0.42212E+00_JPRB ,&
&  0.42267E+00_JPRB, 0.41981E+00_JPRB, 0.41652E+00_JPRB, 0.41656E+00_JPRB, 0.41839E+00_JPRB, 0.42056E+00_JPRB, 0.41982E+00_JPRB ,&
&  0.41055E+00_JPRB, 0.40515E+00_JPRB, 0.40763E+00_JPRB, 0.41039E+00_JPRB, 0.41483E+00_JPRB, 0.42134E+00_JPRB, 0.42695E+00_JPRB ,&
&  0.42993E+00_JPRB, 0.43336E+00_JPRB, 0.43558E+00_JPRB, 0.44163E+00_JPRB, 0.45118E+00_JPRB, 0.45774E+00_JPRB, 0.46423E+00_JPRB ,&
&  0.45106E+00_JPRB, 0.40364E+00_JPRB, 0.37291E+00_JPRB, 0.37958E+00_JPRB, 0.43034E+00_JPRB, 0.44045E+00_JPRB, 0.44129E+00_JPRB ,&
&  0.44434E+00_JPRB, 0.45199E+00_JPRB, 0.45140E+00_JPRB, 0.44542E+00_JPRB, 0.43373E+00_JPRB, 0.41498E+00_JPRB, 0.39197E+00_JPRB ,&
&  0.37654E+00_JPRB, 0.36613E+00_JPRB, 0.35940E+00_JPRB, 0.35296E+00_JPRB, 0.34951E+00_JPRB, 0.34456E+00_JPRB, 0.33698E+00_JPRB ,&
&  0.33405E+00_JPRB, 0.33253E+00_JPRB, 0.33346E+00_JPRB, 0.33523E+00_JPRB, 0.33735E+00_JPRB, 0.33898E+00_JPRB, 0.33693E+00_JPRB ,&
&  0.32459E+00_JPRB, 0.30926E+00_JPRB, 0.29461E+00_JPRB, 0.27866E+00_JPRB, 0.26490E+00_JPRB, 0.25606E+00_JPRB, 0.24909E+00_JPRB ,&
&  0.24620E+00_JPRB, 0.25140E+00_JPRB, 0.26736E+00_JPRB, 0.28607E+00_JPRB, 0.31921E+00_JPRB, 0.29520E+00_JPRB, 0.27161E+00_JPRB ,&
&  0.25979E+00_JPRB, 0.25414E+00_JPRB, 0.25095E+00_JPRB, 0.25076E+00_JPRB, 0.25221E+00_JPRB, 0.25264E+00_JPRB, 0.25269E+00_JPRB ,&
&  0.25378E+00_JPRB, 0.25594E+00_JPRB, 0.25773E+00_JPRB, 0.25923E+00_JPRB, 0.26164E+00_JPRB, 0.26446E+00_JPRB, 0.26598E+00_JPRB ,&
&  0.26667E+00_JPRB, 0.27108E+00_JPRB, 0.28032E+00_JPRB, 0.29095E+00_JPRB, 0.30175E+00_JPRB, 0.31299E+00_JPRB, 0.33070E+00_JPRB ,&
&  0.35310E+00_JPRB, 0.38075E+00_JPRB, 0.37982E+00_JPRB, 0.37941E+00_JPRB, 0.37978E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.40485E+00_JPRB, 0.41606E+00_JPRB, 0.42926E+00_JPRB, 0.43858E+00_JPRB, 0.43775E+00_JPRB, 0.43782E+00_JPRB, 0.44409E+00_JPRB ,&
&  0.43837E+00_JPRB, 0.43139E+00_JPRB, 0.42544E+00_JPRB, 0.41924E+00_JPRB, 0.41690E+00_JPRB, 0.41541E+00_JPRB, 0.41824E+00_JPRB ,&
&  0.42030E+00_JPRB, 0.41916E+00_JPRB, 0.41744E+00_JPRB, 0.41801E+00_JPRB, 0.42437E+00_JPRB, 0.43245E+00_JPRB, 0.43925E+00_JPRB ,&
&  0.44042E+00_JPRB, 0.44380E+00_JPRB, 0.44929E+00_JPRB, 0.45393E+00_JPRB, 0.45639E+00_JPRB, 0.45803E+00_JPRB, 0.45700E+00_JPRB ,&
&  0.45468E+00_JPRB, 0.45316E+00_JPRB, 0.43632E+00_JPRB, 0.45057E+00_JPRB, 0.45515E+00_JPRB, 0.45622E+00_JPRB, 0.46209E+00_JPRB ,&
&  0.46994E+00_JPRB, 0.47853E+00_JPRB, 0.48239E+00_JPRB, 0.48352E+00_JPRB, 0.47893E+00_JPRB, 0.46977E+00_JPRB, 0.44577E+00_JPRB ,&
&  0.42561E+00_JPRB, 0.40458E+00_JPRB, 0.39147E+00_JPRB, 0.38363E+00_JPRB, 0.37758E+00_JPRB, 0.37224E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.35961E+00_JPRB, 0.35307E+00_JPRB, 0.34911E+00_JPRB, 0.34921E+00_JPRB, 0.35342E+00_JPRB, 0.36433E+00_JPRB, 0.37784E+00_JPRB ,&
&  0.38394E+00_JPRB, 0.37678E+00_JPRB, 0.36892E+00_JPRB, 0.36044E+00_JPRB, 0.34702E+00_JPRB, 0.31912E+00_JPRB, 0.30228E+00_JPRB ,&
&  0.29798E+00_JPRB, 0.31051E+00_JPRB, 0.33199E+00_JPRB, 0.33768E+00_JPRB, 0.34260E+00_JPRB, 0.34036E+00_JPRB, 0.33515E+00_JPRB ,&
&  0.32702E+00_JPRB, 0.30337E+00_JPRB, 0.29020E+00_JPRB, 0.28618E+00_JPRB, 0.28379E+00_JPRB, 0.28369E+00_JPRB, 0.28612E+00_JPRB ,&
&  0.29012E+00_JPRB, 0.29760E+00_JPRB, 0.30282E+00_JPRB, 0.30497E+00_JPRB, 0.30812E+00_JPRB, 0.30822E+00_JPRB, 0.30715E+00_JPRB ,&
&  0.30337E+00_JPRB, 0.30959E+00_JPRB, 0.32752E+00_JPRB, 0.35642E+00_JPRB, 0.36271E+00_JPRB, 0.37004E+00_JPRB, 0.37900E+00_JPRB ,&
&  0.38434E+00_JPRB, 0.38921E+00_JPRB, 0.38945E+00_JPRB, 0.39046E+00_JPRB, 0.39585E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.42315E+00_JPRB, 0.43572E+00_JPRB, 0.45174E+00_JPRB, 0.46319E+00_JPRB, 0.46828E+00_JPRB, 0.47384E+00_JPRB, 0.47634E+00_JPRB ,&
&  0.47149E+00_JPRB, 0.46956E+00_JPRB, 0.46399E+00_JPRB, 0.45582E+00_JPRB, 0.44872E+00_JPRB, 0.44506E+00_JPRB, 0.44530E+00_JPRB ,&
&  0.44743E+00_JPRB, 0.44927E+00_JPRB, 0.44578E+00_JPRB, 0.44397E+00_JPRB, 0.44776E+00_JPRB, 0.45292E+00_JPRB, 0.45627E+00_JPRB ,&
&  0.45754E+00_JPRB, 0.45887E+00_JPRB, 0.46354E+00_JPRB, 0.46654E+00_JPRB, 0.46673E+00_JPRB, 0.46034E+00_JPRB, 0.45575E+00_JPRB ,&
&  0.45104E+00_JPRB, 0.44953E+00_JPRB, 0.45652E+00_JPRB, 0.44844E+00_JPRB, 0.46358E+00_JPRB, 0.46585E+00_JPRB, 0.47163E+00_JPRB ,&
&  0.47987E+00_JPRB, 0.49327E+00_JPRB, 0.50342E+00_JPRB, 0.51120E+00_JPRB, 0.51374E+00_JPRB, 0.51709E+00_JPRB, 0.50750E+00_JPRB ,&
&  0.49391E+00_JPRB, 0.47538E+00_JPRB, 0.45688E+00_JPRB, 0.43710E+00_JPRB, 0.41843E+00_JPRB, 0.40376E+00_JPRB, 0.39642E+00_JPRB ,&
&  0.38754E+00_JPRB, 0.37374E+00_JPRB, 0.36040E+00_JPRB, 0.35505E+00_JPRB, 0.35953E+00_JPRB, 0.36960E+00_JPRB, 0.38513E+00_JPRB ,&
&  0.39457E+00_JPRB, 0.39740E+00_JPRB, 0.39422E+00_JPRB, 0.38980E+00_JPRB, 0.38521E+00_JPRB, 0.37828E+00_JPRB, 0.37431E+00_JPRB ,&
&  0.37101E+00_JPRB, 0.36534E+00_JPRB, 0.36239E+00_JPRB, 0.36419E+00_JPRB, 0.36741E+00_JPRB, 0.36940E+00_JPRB, 0.36769E+00_JPRB ,&
&  0.36452E+00_JPRB, 0.36018E+00_JPRB, 0.35217E+00_JPRB, 0.34747E+00_JPRB, 0.34739E+00_JPRB, 0.35170E+00_JPRB, 0.35935E+00_JPRB ,&
&  0.36187E+00_JPRB, 0.36769E+00_JPRB, 0.37346E+00_JPRB, 0.37566E+00_JPRB, 0.37831E+00_JPRB, 0.38056E+00_JPRB, 0.38080E+00_JPRB ,&
&  0.37913E+00_JPRB, 0.37413E+00_JPRB, 0.36705E+00_JPRB, 0.36371E+00_JPRB, 0.36442E+00_JPRB, 0.37075E+00_JPRB, 0.37968E+00_JPRB ,&
&  0.38863E+00_JPRB, 0.40215E+00_JPRB, 0.41259E+00_JPRB, 0.41499E+00_JPRB, 0.41720E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.45162E+00_JPRB, 0.45676E+00_JPRB, 0.46968E+00_JPRB, 0.48040E+00_JPRB, 0.48394E+00_JPRB, 0.49346E+00_JPRB, 0.50470E+00_JPRB ,&
&  0.51409E+00_JPRB, 0.51610E+00_JPRB, 0.51241E+00_JPRB, 0.50204E+00_JPRB, 0.48971E+00_JPRB, 0.47954E+00_JPRB, 0.47541E+00_JPRB ,&
&  0.47753E+00_JPRB, 0.48062E+00_JPRB, 0.48615E+00_JPRB, 0.48920E+00_JPRB, 0.49280E+00_JPRB, 0.49154E+00_JPRB, 0.49183E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.49451E+00_JPRB, 0.49505E+00_JPRB, 0.49150E+00_JPRB, 0.48873E+00_JPRB, 0.48638E+00_JPRB, 0.47816E+00_JPRB ,&
&  0.46890E+00_JPRB, 0.46265E+00_JPRB, 0.46669E+00_JPRB, 0.47836E+00_JPRB, 0.48590E+00_JPRB, 0.48332E+00_JPRB, 0.48234E+00_JPRB ,&
&  0.48252E+00_JPRB, 0.48695E+00_JPRB, 0.49435E+00_JPRB, 0.50600E+00_JPRB, 0.51446E+00_JPRB, 0.52566E+00_JPRB, 0.52631E+00_JPRB ,&
&  0.51993E+00_JPRB, 0.51082E+00_JPRB, 0.50813E+00_JPRB, 0.50735E+00_JPRB, 0.47931E+00_JPRB, 0.44837E+00_JPRB, 0.43311E+00_JPRB ,&
&  0.42493E+00_JPRB, 0.41472E+00_JPRB, 0.39893E+00_JPRB, 0.38770E+00_JPRB, 0.38627E+00_JPRB, 0.39076E+00_JPRB, 0.40200E+00_JPRB ,&
&  0.41019E+00_JPRB, 0.42231E+00_JPRB, 0.43689E+00_JPRB, 0.44618E+00_JPRB, 0.44621E+00_JPRB, 0.43431E+00_JPRB, 0.42350E+00_JPRB ,&
&  0.41725E+00_JPRB, 0.41567E+00_JPRB, 0.41427E+00_JPRB, 0.41220E+00_JPRB, 0.41026E+00_JPRB, 0.40975E+00_JPRB, 0.40862E+00_JPRB ,&
&  0.40435E+00_JPRB, 0.39699E+00_JPRB, 0.38798E+00_JPRB, 0.38287E+00_JPRB, 0.38156E+00_JPRB, 0.38122E+00_JPRB, 0.38268E+00_JPRB ,&
&  0.39167E+00_JPRB, 0.40286E+00_JPRB, 0.41206E+00_JPRB, 0.41645E+00_JPRB, 0.41374E+00_JPRB, 0.40602E+00_JPRB, 0.40126E+00_JPRB ,&
&  0.39745E+00_JPRB, 0.39473E+00_JPRB, 0.39121E+00_JPRB, 0.38942E+00_JPRB, 0.38885E+00_JPRB, 0.39554E+00_JPRB, 0.41058E+00_JPRB ,&
&  0.42955E+00_JPRB, 0.45483E+00_JPRB, 0.46641E+00_JPRB, 0.46505E+00_JPRB, 0.45783E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.50561E+00_JPRB, 0.48889E+00_JPRB, 0.48920E+00_JPRB, 0.49615E+00_JPRB, 0.50015E+00_JPRB, 0.50801E+00_JPRB, 0.52750E+00_JPRB ,&
&  0.53913E+00_JPRB, 0.54666E+00_JPRB, 0.54963E+00_JPRB, 0.54423E+00_JPRB, 0.54309E+00_JPRB, 0.53579E+00_JPRB, 0.52305E+00_JPRB ,&
&  0.51777E+00_JPRB, 0.51669E+00_JPRB, 0.51829E+00_JPRB, 0.52197E+00_JPRB, 0.52405E+00_JPRB, 0.52665E+00_JPRB, 0.52960E+00_JPRB ,&
&  0.53118E+00_JPRB, 0.53113E+00_JPRB, 0.53033E+00_JPRB, 0.53062E+00_JPRB, 0.52978E+00_JPRB, 0.52246E+00_JPRB, 0.51421E+00_JPRB ,&
&  0.50513E+00_JPRB, 0.49610E+00_JPRB, 0.49375E+00_JPRB, 0.49840E+00_JPRB, 0.50366E+00_JPRB, 0.50718E+00_JPRB, 0.50542E+00_JPRB ,&
&  0.50072E+00_JPRB, 0.49724E+00_JPRB, 0.49898E+00_JPRB, 0.50599E+00_JPRB, 0.51476E+00_JPRB, 0.52455E+00_JPRB, 0.52974E+00_JPRB ,&
&  0.52440E+00_JPRB, 0.50727E+00_JPRB, 0.49702E+00_JPRB, 0.49673E+00_JPRB, 0.49354E+00_JPRB, 0.47964E+00_JPRB, 0.46976E+00_JPRB ,&
&  0.46493E+00_JPRB, 0.45614E+00_JPRB, 0.44682E+00_JPRB, 0.43402E+00_JPRB, 0.42222E+00_JPRB, 0.41775E+00_JPRB, 0.41622E+00_JPRB ,&
&  0.41893E+00_JPRB, 0.43595E+00_JPRB, 0.46772E+00_JPRB, 0.49929E+00_JPRB, 0.51351E+00_JPRB, 0.50749E+00_JPRB, 0.49136E+00_JPRB ,&
&  0.48433E+00_JPRB, 0.48160E+00_JPRB, 0.48314E+00_JPRB, 0.48279E+00_JPRB, 0.47929E+00_JPRB, 0.47067E+00_JPRB, 0.46372E+00_JPRB ,&
&  0.45366E+00_JPRB, 0.44254E+00_JPRB, 0.43801E+00_JPRB, 0.43518E+00_JPRB, 0.43211E+00_JPRB, 0.43199E+00_JPRB, 0.43081E+00_JPRB ,&
&  0.43097E+00_JPRB, 0.43465E+00_JPRB, 0.44626E+00_JPRB, 0.45313E+00_JPRB, 0.45468E+00_JPRB, 0.44456E+00_JPRB, 0.44528E+00_JPRB ,&
&  0.44758E+00_JPRB, 0.44510E+00_JPRB, 0.44046E+00_JPRB, 0.44014E+00_JPRB, 0.43991E+00_JPRB, 0.44150E+00_JPRB, 0.45255E+00_JPRB ,&
&  0.47861E+00_JPRB, 0.50180E+00_JPRB, 0.52106E+00_JPRB, 0.52685E+00_JPRB, 0.52289E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.57762E+00_JPRB, 0.54944E+00_JPRB, 0.53828E+00_JPRB, 0.54006E+00_JPRB, 0.54357E+00_JPRB, 0.53955E+00_JPRB, 0.54693E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.56694E+00_JPRB, 0.57165E+00_JPRB, 0.56798E+00_JPRB, 0.56550E+00_JPRB, 0.56265E+00_JPRB, 0.54808E+00_JPRB ,&
&  0.54331E+00_JPRB, 0.54153E+00_JPRB, 0.54548E+00_JPRB, 0.55223E+00_JPRB, 0.55952E+00_JPRB, 0.57133E+00_JPRB, 0.58005E+00_JPRB ,&
&  0.58582E+00_JPRB, 0.58945E+00_JPRB, 0.59359E+00_JPRB, 0.59382E+00_JPRB, 0.59044E+00_JPRB, 0.58172E+00_JPRB, 0.57508E+00_JPRB ,&
&  0.57126E+00_JPRB, 0.55993E+00_JPRB, 0.55013E+00_JPRB, 0.54898E+00_JPRB, 0.55234E+00_JPRB, 0.55609E+00_JPRB, 0.55625E+00_JPRB ,&
&  0.54292E+00_JPRB, 0.52552E+00_JPRB, 0.52338E+00_JPRB, 0.52344E+00_JPRB, 0.52886E+00_JPRB, 0.54126E+00_JPRB, 0.54507E+00_JPRB ,&
&  0.54247E+00_JPRB, 0.52902E+00_JPRB, 0.51020E+00_JPRB, 0.50656E+00_JPRB, 0.50893E+00_JPRB, 0.50287E+00_JPRB, 0.49669E+00_JPRB ,&
&  0.49539E+00_JPRB, 0.49204E+00_JPRB, 0.48585E+00_JPRB, 0.47759E+00_JPRB, 0.46493E+00_JPRB, 0.44837E+00_JPRB, 0.43961E+00_JPRB ,&
&  0.43843E+00_JPRB, 0.45225E+00_JPRB, 0.47776E+00_JPRB, 0.50713E+00_JPRB, 0.53546E+00_JPRB, 0.54953E+00_JPRB, 0.54937E+00_JPRB ,&
&  0.53831E+00_JPRB, 0.53622E+00_JPRB, 0.54168E+00_JPRB, 0.54599E+00_JPRB, 0.54675E+00_JPRB, 0.53613E+00_JPRB, 0.51950E+00_JPRB ,&
&  0.50754E+00_JPRB, 0.49842E+00_JPRB, 0.49721E+00_JPRB, 0.50201E+00_JPRB, 0.50687E+00_JPRB, 0.50800E+00_JPRB, 0.50168E+00_JPRB ,&
&  0.49482E+00_JPRB, 0.48782E+00_JPRB, 0.48400E+00_JPRB, 0.48709E+00_JPRB, 0.49294E+00_JPRB, 0.49864E+00_JPRB, 0.51106E+00_JPRB ,&
&  0.52881E+00_JPRB, 0.54683E+00_JPRB, 0.55105E+00_JPRB, 0.54706E+00_JPRB, 0.54675E+00_JPRB, 0.54413E+00_JPRB, 0.54150E+00_JPRB ,&
&  0.55100E+00_JPRB, 0.57274E+00_JPRB, 0.58848E+00_JPRB, 0.59758E+00_JPRB, 0.59804E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.62533E+00_JPRB, 0.60096E+00_JPRB, 0.58318E+00_JPRB, 0.57669E+00_JPRB, 0.57325E+00_JPRB, 0.56147E+00_JPRB, 0.55647E+00_JPRB ,&
&  0.56274E+00_JPRB, 0.56485E+00_JPRB, 0.57213E+00_JPRB, 0.57197E+00_JPRB, 0.57128E+00_JPRB, 0.57378E+00_JPRB, 0.56613E+00_JPRB ,&
&  0.56605E+00_JPRB, 0.57487E+00_JPRB, 0.58518E+00_JPRB, 0.59914E+00_JPRB, 0.61088E+00_JPRB, 0.61979E+00_JPRB, 0.62740E+00_JPRB ,&
&  0.63043E+00_JPRB, 0.63698E+00_JPRB, 0.64064E+00_JPRB, 0.63666E+00_JPRB, 0.62623E+00_JPRB, 0.61825E+00_JPRB, 0.61709E+00_JPRB ,&
&  0.61624E+00_JPRB, 0.61035E+00_JPRB, 0.56661E+00_JPRB, 0.59043E+00_JPRB, 0.59789E+00_JPRB, 0.60256E+00_JPRB, 0.60584E+00_JPRB ,&
&  0.60015E+00_JPRB, 0.58444E+00_JPRB, 0.57733E+00_JPRB, 0.56618E+00_JPRB, 0.56542E+00_JPRB, 0.57183E+00_JPRB, 0.57099E+00_JPRB ,&
&  0.56751E+00_JPRB, 0.55733E+00_JPRB, 0.54321E+00_JPRB, 0.53203E+00_JPRB, 0.52904E+00_JPRB, 0.52721E+00_JPRB, 0.51997E+00_JPRB ,&
&  0.51469E+00_JPRB, 0.51353E+00_JPRB, 0.51159E+00_JPRB, 0.50583E+00_JPRB, 0.49134E+00_JPRB, 0.47781E+00_JPRB, 0.47486E+00_JPRB ,&
&  0.47755E+00_JPRB, 0.48370E+00_JPRB, 0.49669E+00_JPRB, 0.51393E+00_JPRB, 0.53285E+00_JPRB, 0.55081E+00_JPRB, 0.56730E+00_JPRB ,&
&  0.57409E+00_JPRB, 0.57805E+00_JPRB, 0.58160E+00_JPRB, 0.58598E+00_JPRB, 0.58659E+00_JPRB, 0.57888E+00_JPRB, 0.56004E+00_JPRB ,&
&  0.53996E+00_JPRB, 0.53484E+00_JPRB, 0.53892E+00_JPRB, 0.54804E+00_JPRB, 0.55277E+00_JPRB, 0.55212E+00_JPRB, 0.54519E+00_JPRB ,&
&  0.54054E+00_JPRB, 0.53191E+00_JPRB, 0.52321E+00_JPRB, 0.51902E+00_JPRB, 0.51893E+00_JPRB, 0.52508E+00_JPRB, 0.54237E+00_JPRB ,&
&  0.56295E+00_JPRB, 0.58595E+00_JPRB, 0.60675E+00_JPRB, 0.61483E+00_JPRB, 0.61926E+00_JPRB, 0.61889E+00_JPRB, 0.60998E+00_JPRB ,&
&  0.61347E+00_JPRB, 0.63161E+00_JPRB, 0.64108E+00_JPRB, 0.64238E+00_JPRB, 0.64017E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.67669E+00_JPRB, 0.65750E+00_JPRB, 0.64598E+00_JPRB, 0.63477E+00_JPRB, 0.62666E+00_JPRB, 0.60891E+00_JPRB, 0.59898E+00_JPRB ,&
&  0.59290E+00_JPRB, 0.58933E+00_JPRB, 0.58914E+00_JPRB, 0.58969E+00_JPRB, 0.59973E+00_JPRB, 0.60597E+00_JPRB, 0.60812E+00_JPRB ,&
&  0.61534E+00_JPRB, 0.63234E+00_JPRB, 0.64788E+00_JPRB, 0.66105E+00_JPRB, 0.67164E+00_JPRB, 0.67954E+00_JPRB, 0.67976E+00_JPRB ,&
&  0.68385E+00_JPRB, 0.69215E+00_JPRB, 0.69635E+00_JPRB, 0.68704E+00_JPRB, 0.67519E+00_JPRB, 0.66907E+00_JPRB, 0.66926E+00_JPRB ,&
&  0.65976E+00_JPRB, 0.61432E+00_JPRB, 0.62862E+00_JPRB, 0.64111E+00_JPRB, 0.65656E+00_JPRB, 0.66508E+00_JPRB, 0.67484E+00_JPRB ,&
&  0.67618E+00_JPRB, 0.66003E+00_JPRB, 0.64030E+00_JPRB, 0.62605E+00_JPRB, 0.62875E+00_JPRB, 0.62602E+00_JPRB, 0.61693E+00_JPRB ,&
&  0.61272E+00_JPRB, 0.60892E+00_JPRB, 0.60107E+00_JPRB, 0.58775E+00_JPRB, 0.57581E+00_JPRB, 0.56778E+00_JPRB, 0.56251E+00_JPRB ,&
&  0.55144E+00_JPRB, 0.54224E+00_JPRB, 0.53724E+00_JPRB, 0.53182E+00_JPRB, 0.52712E+00_JPRB, 0.53090E+00_JPRB, 0.53390E+00_JPRB ,&
&  0.53246E+00_JPRB, 0.53251E+00_JPRB, 0.53974E+00_JPRB, 0.55334E+00_JPRB, 0.56484E+00_JPRB, 0.57817E+00_JPRB, 0.59393E+00_JPRB ,&
&  0.60923E+00_JPRB, 0.62233E+00_JPRB, 0.62624E+00_JPRB, 0.62989E+00_JPRB, 0.62489E+00_JPRB, 0.61691E+00_JPRB, 0.60048E+00_JPRB ,&
&  0.58257E+00_JPRB, 0.58035E+00_JPRB, 0.58239E+00_JPRB, 0.58920E+00_JPRB, 0.58851E+00_JPRB, 0.58602E+00_JPRB, 0.58411E+00_JPRB ,&
&  0.57988E+00_JPRB, 0.57631E+00_JPRB, 0.57142E+00_JPRB, 0.56669E+00_JPRB, 0.56976E+00_JPRB, 0.57550E+00_JPRB, 0.58883E+00_JPRB ,&
&  0.60665E+00_JPRB, 0.62562E+00_JPRB, 0.64559E+00_JPRB, 0.66022E+00_JPRB, 0.66612E+00_JPRB, 0.66963E+00_JPRB, 0.67246E+00_JPRB ,&
&  0.67276E+00_JPRB, 0.67112E+00_JPRB, 0.67736E+00_JPRB, 0.68172E+00_JPRB, 0.68623E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.71007E+00_JPRB, 0.69225E+00_JPRB, 0.68193E+00_JPRB, 0.67747E+00_JPRB, 0.66873E+00_JPRB, 0.64975E+00_JPRB, 0.63422E+00_JPRB ,&
&  0.62008E+00_JPRB, 0.61274E+00_JPRB, 0.60993E+00_JPRB, 0.60925E+00_JPRB, 0.61492E+00_JPRB, 0.62004E+00_JPRB, 0.62629E+00_JPRB ,&
&  0.63705E+00_JPRB, 0.65466E+00_JPRB, 0.66987E+00_JPRB, 0.68424E+00_JPRB, 0.69734E+00_JPRB, 0.71049E+00_JPRB, 0.71438E+00_JPRB ,&
&  0.71758E+00_JPRB, 0.72538E+00_JPRB, 0.72741E+00_JPRB, 0.71593E+00_JPRB, 0.70358E+00_JPRB, 0.70590E+00_JPRB, 0.70267E+00_JPRB ,&
&  0.69535E+00_JPRB, 0.58510E+00_JPRB, 0.63858E+00_JPRB, 0.66632E+00_JPRB, 0.69811E+00_JPRB, 0.72488E+00_JPRB, 0.74402E+00_JPRB ,&
&  0.73911E+00_JPRB, 0.72127E+00_JPRB, 0.70031E+00_JPRB, 0.67317E+00_JPRB, 0.66468E+00_JPRB, 0.65694E+00_JPRB, 0.64702E+00_JPRB ,&
&  0.63940E+00_JPRB, 0.63220E+00_JPRB, 0.62346E+00_JPRB, 0.61206E+00_JPRB, 0.59761E+00_JPRB, 0.58675E+00_JPRB, 0.57701E+00_JPRB ,&
&  0.57425E+00_JPRB, 0.56664E+00_JPRB, 0.55969E+00_JPRB, 0.56015E+00_JPRB, 0.57157E+00_JPRB, 0.57643E+00_JPRB, 0.57591E+00_JPRB ,&
&  0.57497E+00_JPRB, 0.57489E+00_JPRB, 0.58141E+00_JPRB, 0.59371E+00_JPRB, 0.60420E+00_JPRB, 0.61113E+00_JPRB, 0.62089E+00_JPRB ,&
&  0.63697E+00_JPRB, 0.64893E+00_JPRB, 0.64917E+00_JPRB, 0.63943E+00_JPRB, 0.63400E+00_JPRB, 0.63490E+00_JPRB, 0.62654E+00_JPRB ,&
&  0.61272E+00_JPRB, 0.60539E+00_JPRB, 0.60542E+00_JPRB, 0.60637E+00_JPRB, 0.60402E+00_JPRB, 0.59869E+00_JPRB, 0.59804E+00_JPRB ,&
&  0.59940E+00_JPRB, 0.60008E+00_JPRB, 0.59927E+00_JPRB, 0.60176E+00_JPRB, 0.60758E+00_JPRB, 0.61554E+00_JPRB, 0.62665E+00_JPRB ,&
&  0.64281E+00_JPRB, 0.65793E+00_JPRB, 0.66700E+00_JPRB, 0.68080E+00_JPRB, 0.69663E+00_JPRB, 0.70300E+00_JPRB, 0.71627E+00_JPRB ,&
&  0.71678E+00_JPRB, 0.71343E+00_JPRB, 0.71218E+00_JPRB, 0.71809E+00_JPRB, 0.72094E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.74364E+00_JPRB, 0.72567E+00_JPRB, 0.71630E+00_JPRB, 0.71029E+00_JPRB, 0.70558E+00_JPRB, 0.69862E+00_JPRB, 0.68520E+00_JPRB ,&
&  0.67155E+00_JPRB, 0.66472E+00_JPRB, 0.65862E+00_JPRB, 0.65041E+00_JPRB, 0.65139E+00_JPRB, 0.65696E+00_JPRB, 0.66764E+00_JPRB ,&
&  0.67951E+00_JPRB, 0.69486E+00_JPRB, 0.70974E+00_JPRB, 0.72875E+00_JPRB, 0.74161E+00_JPRB, 0.75083E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.75954E+00_JPRB, 0.76022E+00_JPRB, 0.75342E+00_JPRB, 0.74473E+00_JPRB, 0.74232E+00_JPRB, 0.74287E+00_JPRB, 0.73768E+00_JPRB ,&
&  0.64129E+00_JPRB, 0.55767E+00_JPRB, 0.61121E+00_JPRB, 0.67950E+00_JPRB, 0.75232E+00_JPRB, 0.80986E+00_JPRB, 0.83772E+00_JPRB ,&
&  0.82210E+00_JPRB, 0.78802E+00_JPRB, 0.75912E+00_JPRB, 0.72754E+00_JPRB, 0.71401E+00_JPRB, 0.70640E+00_JPRB, 0.69470E+00_JPRB ,&
&  0.68195E+00_JPRB, 0.66823E+00_JPRB, 0.65278E+00_JPRB, 0.63023E+00_JPRB, 0.61415E+00_JPRB, 0.60223E+00_JPRB, 0.60008E+00_JPRB ,&
&  0.60204E+00_JPRB, 0.60540E+00_JPRB, 0.60824E+00_JPRB, 0.61021E+00_JPRB, 0.61650E+00_JPRB, 0.62496E+00_JPRB, 0.63436E+00_JPRB ,&
&  0.64636E+00_JPRB, 0.65535E+00_JPRB, 0.66963E+00_JPRB, 0.67946E+00_JPRB, 0.68818E+00_JPRB, 0.69480E+00_JPRB, 0.70100E+00_JPRB ,&
&  0.70064E+00_JPRB, 0.69629E+00_JPRB, 0.68035E+00_JPRB, 0.67192E+00_JPRB, 0.67213E+00_JPRB, 0.67343E+00_JPRB, 0.67020E+00_JPRB ,&
&  0.66090E+00_JPRB, 0.64984E+00_JPRB, 0.63896E+00_JPRB, 0.63411E+00_JPRB, 0.63598E+00_JPRB, 0.63548E+00_JPRB, 0.63156E+00_JPRB ,&
&  0.62829E+00_JPRB, 0.62839E+00_JPRB, 0.63412E+00_JPRB, 0.64515E+00_JPRB, 0.65980E+00_JPRB, 0.67806E+00_JPRB, 0.69317E+00_JPRB ,&
&  0.70906E+00_JPRB, 0.72518E+00_JPRB, 0.73723E+00_JPRB, 0.75902E+00_JPRB, 0.77857E+00_JPRB, 0.78700E+00_JPRB, 0.78356E+00_JPRB ,&
&  0.78270E+00_JPRB, 0.77243E+00_JPRB, 0.76590E+00_JPRB, 0.76444E+00_JPRB, 0.75757E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.72644E+00_JPRB, 0.73369E+00_JPRB, 0.73947E+00_JPRB, 0.73639E+00_JPRB, 0.73255E+00_JPRB, 0.72402E+00_JPRB, 0.70748E+00_JPRB ,&
&  0.69223E+00_JPRB, 0.68634E+00_JPRB, 0.68612E+00_JPRB, 0.68739E+00_JPRB, 0.69015E+00_JPRB, 0.69639E+00_JPRB, 0.70780E+00_JPRB ,&
&  0.72149E+00_JPRB, 0.73601E+00_JPRB, 0.75317E+00_JPRB, 0.76923E+00_JPRB, 0.77316E+00_JPRB, 0.77036E+00_JPRB, 0.77100E+00_JPRB ,&
&  0.77755E+00_JPRB, 0.77688E+00_JPRB, 0.77115E+00_JPRB, 0.77415E+00_JPRB, 0.78202E+00_JPRB, 0.77298E+00_JPRB, 0.75480E+00_JPRB ,&
&  0.74226E+00_JPRB, 0.60011E+00_JPRB, 0.64293E+00_JPRB, 0.72356E+00_JPRB, 0.81364E+00_JPRB, 0.89531E+00_JPRB, 0.92567E+00_JPRB ,&
&  0.90713E+00_JPRB, 0.86649E+00_JPRB, 0.83451E+00_JPRB, 0.79592E+00_JPRB, 0.77483E+00_JPRB, 0.75784E+00_JPRB, 0.73823E+00_JPRB ,&
&  0.71295E+00_JPRB, 0.68812E+00_JPRB, 0.66221E+00_JPRB, 0.64036E+00_JPRB, 0.62300E+00_JPRB, 0.61544E+00_JPRB, 0.61361E+00_JPRB ,&
&  0.61434E+00_JPRB, 0.62525E+00_JPRB, 0.63100E+00_JPRB, 0.63987E+00_JPRB, 0.65396E+00_JPRB, 0.67340E+00_JPRB, 0.69373E+00_JPRB ,&
&  0.71179E+00_JPRB, 0.72444E+00_JPRB, 0.73725E+00_JPRB, 0.74806E+00_JPRB, 0.76062E+00_JPRB, 0.77041E+00_JPRB, 0.76515E+00_JPRB ,&
&  0.74300E+00_JPRB, 0.71634E+00_JPRB, 0.71367E+00_JPRB, 0.71784E+00_JPRB, 0.71478E+00_JPRB, 0.71366E+00_JPRB, 0.71121E+00_JPRB ,&
&  0.70261E+00_JPRB, 0.69190E+00_JPRB, 0.68616E+00_JPRB, 0.67840E+00_JPRB, 0.67372E+00_JPRB, 0.67340E+00_JPRB, 0.66230E+00_JPRB ,&
&  0.65487E+00_JPRB, 0.65503E+00_JPRB, 0.66365E+00_JPRB, 0.68356E+00_JPRB, 0.71032E+00_JPRB, 0.73555E+00_JPRB, 0.75212E+00_JPRB ,&
&  0.76199E+00_JPRB, 0.77271E+00_JPRB, 0.78024E+00_JPRB, 0.80904E+00_JPRB, 0.81890E+00_JPRB, 0.82330E+00_JPRB, 0.82367E+00_JPRB ,&
&  0.82061E+00_JPRB, 0.81779E+00_JPRB, 0.62586E+00_JPRB, 0.69122E+00_JPRB, 0.70687E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.74707E+00_JPRB, 0.77284E+00_JPRB, 0.78286E+00_JPRB, 0.78382E+00_JPRB, 0.78013E+00_JPRB, 0.76601E+00_JPRB, 0.74962E+00_JPRB ,&
&  0.73074E+00_JPRB, 0.72273E+00_JPRB, 0.71971E+00_JPRB, 0.72421E+00_JPRB, 0.73061E+00_JPRB, 0.74009E+00_JPRB, 0.75317E+00_JPRB ,&
&  0.76554E+00_JPRB, 0.77209E+00_JPRB, 0.77752E+00_JPRB, 0.77639E+00_JPRB, 0.77516E+00_JPRB, 0.77793E+00_JPRB, 0.78264E+00_JPRB ,&
&  0.79215E+00_JPRB, 0.79593E+00_JPRB, 0.79379E+00_JPRB, 0.80258E+00_JPRB, 0.81500E+00_JPRB, 0.80464E+00_JPRB, 0.78484E+00_JPRB ,&
&  0.76936E+00_JPRB, 0.64877E+00_JPRB, 0.70281E+00_JPRB, 0.84258E+00_JPRB, 0.95693E+00_JPRB, 0.10555E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.98053E+00_JPRB, 0.92640E+00_JPRB, 0.87579E+00_JPRB, 0.83864E+00_JPRB, 0.80130E+00_JPRB, 0.77515E+00_JPRB ,&
&  0.75693E+00_JPRB, 0.73919E+00_JPRB, 0.71532E+00_JPRB, 0.69688E+00_JPRB, 0.68225E+00_JPRB, 0.66815E+00_JPRB, 0.66066E+00_JPRB ,&
&  0.66299E+00_JPRB, 0.67478E+00_JPRB, 0.68744E+00_JPRB, 0.70193E+00_JPRB, 0.72882E+00_JPRB, 0.75422E+00_JPRB, 0.77864E+00_JPRB ,&
&  0.78959E+00_JPRB, 0.79968E+00_JPRB, 0.81640E+00_JPRB, 0.83335E+00_JPRB, 0.85430E+00_JPRB, 0.86310E+00_JPRB, 0.84481E+00_JPRB ,&
&  0.80134E+00_JPRB, 0.77293E+00_JPRB, 0.75463E+00_JPRB, 0.74315E+00_JPRB, 0.74378E+00_JPRB, 0.75282E+00_JPRB, 0.75683E+00_JPRB ,&
&  0.75204E+00_JPRB, 0.74391E+00_JPRB, 0.73345E+00_JPRB, 0.72340E+00_JPRB, 0.72290E+00_JPRB, 0.72515E+00_JPRB, 0.72182E+00_JPRB ,&
&  0.71781E+00_JPRB, 0.72107E+00_JPRB, 0.73201E+00_JPRB, 0.75317E+00_JPRB, 0.77893E+00_JPRB, 0.80704E+00_JPRB, 0.82384E+00_JPRB ,&
&  0.83516E+00_JPRB, 0.84696E+00_JPRB, 0.86436E+00_JPRB, 0.82825E+00_JPRB, 0.86621E+00_JPRB, 0.87699E+00_JPRB, 0.87820E+00_JPRB ,&
&  0.86162E+00_JPRB, 0.83521E+00_JPRB, 0.80329E+00_JPRB, 0.66369E+00_JPRB, 0.71699E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.79587E+00_JPRB, 0.82316E+00_JPRB, 0.83045E+00_JPRB, 0.82950E+00_JPRB, 0.81653E+00_JPRB, 0.79799E+00_JPRB, 0.78348E+00_JPRB ,&
&  0.77687E+00_JPRB, 0.76233E+00_JPRB, 0.75637E+00_JPRB, 0.75513E+00_JPRB, 0.75997E+00_JPRB, 0.76869E+00_JPRB, 0.77058E+00_JPRB ,&
&  0.76336E+00_JPRB, 0.75123E+00_JPRB, 0.74310E+00_JPRB, 0.74330E+00_JPRB, 0.75354E+00_JPRB, 0.76446E+00_JPRB, 0.77187E+00_JPRB ,&
&  0.77538E+00_JPRB, 0.76995E+00_JPRB, 0.76781E+00_JPRB, 0.77739E+00_JPRB, 0.78485E+00_JPRB, 0.77679E+00_JPRB, 0.75634E+00_JPRB ,&
&  0.74736E+00_JPRB, 0.71885E+00_JPRB, 0.88534E+00_JPRB, 0.10414E+01_JPRB, 0.11680E+01_JPRB, 0.12046E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.10375E+01_JPRB, 0.96055E+00_JPRB, 0.90112E+00_JPRB, 0.85898E+00_JPRB, 0.82801E+00_JPRB, 0.80624E+00_JPRB ,&
&  0.77793E+00_JPRB, 0.75190E+00_JPRB, 0.72886E+00_JPRB, 0.71531E+00_JPRB, 0.70961E+00_JPRB, 0.70532E+00_JPRB, 0.70246E+00_JPRB ,&
&  0.70005E+00_JPRB, 0.69750E+00_JPRB, 0.70073E+00_JPRB, 0.71400E+00_JPRB, 0.73407E+00_JPRB, 0.77086E+00_JPRB, 0.81525E+00_JPRB ,&
&  0.83303E+00_JPRB, 0.85909E+00_JPRB, 0.89384E+00_JPRB, 0.93136E+00_JPRB, 0.94148E+00_JPRB, 0.93463E+00_JPRB, 0.91170E+00_JPRB ,&
&  0.88082E+00_JPRB, 0.83469E+00_JPRB, 0.79701E+00_JPRB, 0.77066E+00_JPRB, 0.75931E+00_JPRB, 0.75269E+00_JPRB, 0.76065E+00_JPRB ,&
&  0.76736E+00_JPRB, 0.77335E+00_JPRB, 0.76753E+00_JPRB, 0.76028E+00_JPRB, 0.75905E+00_JPRB, 0.74750E+00_JPRB, 0.74581E+00_JPRB ,&
&  0.74955E+00_JPRB, 0.76510E+00_JPRB, 0.79087E+00_JPRB, 0.81467E+00_JPRB, 0.84435E+00_JPRB, 0.86448E+00_JPRB, 0.87122E+00_JPRB ,&
&  0.87330E+00_JPRB, 0.87308E+00_JPRB, 0.87051E+00_JPRB, 0.91338E+00_JPRB, 0.91177E+00_JPRB, 0.91097E+00_JPRB, 0.90044E+00_JPRB ,&
&  0.88137E+00_JPRB, 0.84405E+00_JPRB, 0.79979E+00_JPRB, 0.78351E+00_JPRB, 0.73581E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.92368E+00_JPRB, 0.91429E+00_JPRB, 0.89142E+00_JPRB, 0.87339E+00_JPRB, 0.85281E+00_JPRB, 0.83771E+00_JPRB, 0.82501E+00_JPRB ,&
&  0.81824E+00_JPRB, 0.80290E+00_JPRB, 0.79538E+00_JPRB, 0.78851E+00_JPRB, 0.78650E+00_JPRB, 0.78009E+00_JPRB, 0.76961E+00_JPRB ,&
&  0.75776E+00_JPRB, 0.75102E+00_JPRB, 0.74948E+00_JPRB, 0.75454E+00_JPRB, 0.76505E+00_JPRB, 0.77154E+00_JPRB, 0.76646E+00_JPRB ,&
&  0.75813E+00_JPRB, 0.74491E+00_JPRB, 0.72885E+00_JPRB, 0.74230E+00_JPRB, 0.75759E+00_JPRB, 0.76151E+00_JPRB, 0.74612E+00_JPRB ,&
&  0.75280E+00_JPRB, 0.82220E+00_JPRB, 0.12078E+01_JPRB, 0.13880E+01_JPRB, 0.14226E+01_JPRB, 0.14166E+01_JPRB, 0.12666E+01_JPRB ,&
&  0.11533E+01_JPRB, 0.10352E+01_JPRB, 0.94787E+00_JPRB, 0.90549E+00_JPRB, 0.88108E+00_JPRB, 0.84464E+00_JPRB, 0.81431E+00_JPRB ,&
&  0.77260E+00_JPRB, 0.75136E+00_JPRB, 0.73682E+00_JPRB, 0.72968E+00_JPRB, 0.72519E+00_JPRB, 0.72506E+00_JPRB, 0.73015E+00_JPRB ,&
&  0.73558E+00_JPRB, 0.73670E+00_JPRB, 0.74078E+00_JPRB, 0.75968E+00_JPRB, 0.77973E+00_JPRB, 0.84310E+00_JPRB, 0.90755E+00_JPRB ,&
&  0.94365E+00_JPRB, 0.97582E+00_JPRB, 0.10178E+01_JPRB, 0.10372E+01_JPRB, 0.10154E+01_JPRB, 0.97707E+00_JPRB, 0.94633E+00_JPRB ,&
&  0.91620E+00_JPRB, 0.88191E+00_JPRB, 0.84586E+00_JPRB, 0.82082E+00_JPRB, 0.79728E+00_JPRB, 0.78718E+00_JPRB, 0.79919E+00_JPRB ,&
&  0.80563E+00_JPRB, 0.81437E+00_JPRB, 0.80901E+00_JPRB, 0.79840E+00_JPRB, 0.77650E+00_JPRB, 0.76037E+00_JPRB, 0.76110E+00_JPRB ,&
&  0.77391E+00_JPRB, 0.79985E+00_JPRB, 0.83478E+00_JPRB, 0.87064E+00_JPRB, 0.89893E+00_JPRB, 0.91233E+00_JPRB, 0.91438E+00_JPRB ,&
&  0.92042E+00_JPRB, 0.92294E+00_JPRB, 0.87181E+00_JPRB, 0.89414E+00_JPRB, 0.98723E+00_JPRB, 0.97670E+00_JPRB, 0.95824E+00_JPRB ,&
&  0.92164E+00_JPRB, 0.86366E+00_JPRB, 0.82255E+00_JPRB, 0.80768E+00_JPRB, 0.88110E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.88406E+00_JPRB, 0.87337E+00_JPRB, 0.85575E+00_JPRB, 0.84527E+00_JPRB, 0.84641E+00_JPRB, 0.83749E+00_JPRB, 0.83092E+00_JPRB ,&
&  0.82799E+00_JPRB, 0.81807E+00_JPRB, 0.80928E+00_JPRB, 0.79541E+00_JPRB, 0.77888E+00_JPRB, 0.76654E+00_JPRB, 0.76397E+00_JPRB ,&
&  0.76598E+00_JPRB, 0.75784E+00_JPRB, 0.76308E+00_JPRB, 0.76609E+00_JPRB, 0.76348E+00_JPRB, 0.75046E+00_JPRB, 0.73753E+00_JPRB ,&
&  0.73116E+00_JPRB, 0.71909E+00_JPRB, 0.71657E+00_JPRB, 0.73648E+00_JPRB, 0.75459E+00_JPRB, 0.75607E+00_JPRB, 0.73902E+00_JPRB ,&
&  0.77376E+00_JPRB, 0.95732E+00_JPRB, 0.15861E+01_JPRB, 0.18544E+01_JPRB, 0.18020E+01_JPRB, 0.15109E+01_JPRB, 0.12659E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.10012E+01_JPRB, 0.95154E+00_JPRB, 0.90691E+00_JPRB, 0.86175E+00_JPRB, 0.82634E+00_JPRB, 0.79768E+00_JPRB ,&
&  0.76355E+00_JPRB, 0.74887E+00_JPRB, 0.74061E+00_JPRB, 0.73995E+00_JPRB, 0.73916E+00_JPRB, 0.73718E+00_JPRB, 0.73884E+00_JPRB ,&
&  0.74219E+00_JPRB, 0.74603E+00_JPRB, 0.75764E+00_JPRB, 0.76684E+00_JPRB, 0.81098E+00_JPRB, 0.92411E+00_JPRB, 0.10589E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.11124E+01_JPRB, 0.10879E+01_JPRB, 0.10579E+01_JPRB, 0.10296E+01_JPRB, 0.99561E+00_JPRB, 0.96712E+00_JPRB ,&
&  0.93135E+00_JPRB, 0.90345E+00_JPRB, 0.88237E+00_JPRB, 0.86606E+00_JPRB, 0.84230E+00_JPRB, 0.84102E+00_JPRB, 0.85274E+00_JPRB ,&
&  0.86790E+00_JPRB, 0.87271E+00_JPRB, 0.85318E+00_JPRB, 0.83290E+00_JPRB, 0.80652E+00_JPRB, 0.80139E+00_JPRB, 0.80330E+00_JPRB ,&
&  0.80847E+00_JPRB, 0.82866E+00_JPRB, 0.86775E+00_JPRB, 0.90586E+00_JPRB, 0.93108E+00_JPRB, 0.94050E+00_JPRB, 0.93659E+00_JPRB ,&
&  0.94239E+00_JPRB, 0.94148E+00_JPRB, 0.92081E+00_JPRB, 0.93897E+00_JPRB, 0.94292E+00_JPRB, 0.10013E+01_JPRB, 0.95547E+00_JPRB ,&
&  0.88966E+00_JPRB, 0.84127E+00_JPRB, 0.81765E+00_JPRB, 0.81157E+00_JPRB, 0.86966E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.86559E+00_JPRB, 0.84802E+00_JPRB, 0.83688E+00_JPRB, 0.82687E+00_JPRB, 0.81487E+00_JPRB, 0.80937E+00_JPRB, 0.80225E+00_JPRB ,&
&  0.80675E+00_JPRB, 0.80912E+00_JPRB, 0.79985E+00_JPRB, 0.77426E+00_JPRB, 0.75402E+00_JPRB, 0.75160E+00_JPRB, 0.75833E+00_JPRB ,&
&  0.75369E+00_JPRB, 0.74383E+00_JPRB, 0.74813E+00_JPRB, 0.73205E+00_JPRB, 0.71036E+00_JPRB, 0.69703E+00_JPRB, 0.69165E+00_JPRB ,&
&  0.69608E+00_JPRB, 0.70705E+00_JPRB, 0.73380E+00_JPRB, 0.75900E+00_JPRB, 0.76194E+00_JPRB, 0.75428E+00_JPRB, 0.73802E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.10461E+01_JPRB, 0.20280E+01_JPRB, 0.20544E+01_JPRB, 0.17489E+01_JPRB, 0.14027E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10027E+01_JPRB, 0.94427E+00_JPRB, 0.87022E+00_JPRB, 0.81521E+00_JPRB, 0.78765E+00_JPRB, 0.77648E+00_JPRB ,&
&  0.75817E+00_JPRB, 0.74567E+00_JPRB, 0.74765E+00_JPRB, 0.75412E+00_JPRB, 0.75809E+00_JPRB, 0.75592E+00_JPRB, 0.74333E+00_JPRB ,&
&  0.74224E+00_JPRB, 0.74459E+00_JPRB, 0.75023E+00_JPRB, 0.75416E+00_JPRB, 0.80102E+00_JPRB, 0.82271E+00_JPRB, 0.10222E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.12481E+01_JPRB, 0.11503E+01_JPRB, 0.10992E+01_JPRB, 0.10712E+01_JPRB, 0.10305E+01_JPRB, 0.10008E+01_JPRB ,&
&  0.96427E+00_JPRB, 0.94036E+00_JPRB, 0.94191E+00_JPRB, 0.93483E+00_JPRB, 0.93412E+00_JPRB, 0.91075E+00_JPRB, 0.89308E+00_JPRB ,&
&  0.89581E+00_JPRB, 0.87184E+00_JPRB, 0.84587E+00_JPRB, 0.82130E+00_JPRB, 0.82797E+00_JPRB, 0.84440E+00_JPRB, 0.85153E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.86485E+00_JPRB, 0.88725E+00_JPRB, 0.89665E+00_JPRB, 0.93797E+00_JPRB, 0.97326E+00_JPRB, 0.96767E+00_JPRB ,&
&  0.92051E+00_JPRB, 0.91578E+00_JPRB, 0.92410E+00_JPRB, 0.90337E+00_JPRB, 0.90708E+00_JPRB, 0.97648E+00_JPRB, 0.90212E+00_JPRB ,&
&  0.84047E+00_JPRB, 0.80600E+00_JPRB, 0.78890E+00_JPRB, 0.80518E+00_JPRB, 0.86813E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.88012E+00_JPRB, 0.86200E+00_JPRB, 0.84627E+00_JPRB, 0.81966E+00_JPRB, 0.81363E+00_JPRB, 0.80374E+00_JPRB, 0.79960E+00_JPRB ,&
&  0.79940E+00_JPRB, 0.78767E+00_JPRB, 0.77389E+00_JPRB, 0.75415E+00_JPRB, 0.74633E+00_JPRB, 0.75674E+00_JPRB, 0.75126E+00_JPRB ,&
&  0.73382E+00_JPRB, 0.71089E+00_JPRB, 0.70397E+00_JPRB, 0.69544E+00_JPRB, 0.68683E+00_JPRB, 0.68866E+00_JPRB, 0.69830E+00_JPRB ,&
&  0.70212E+00_JPRB, 0.71999E+00_JPRB, 0.74955E+00_JPRB, 0.77643E+00_JPRB, 0.77405E+00_JPRB, 0.76568E+00_JPRB, 0.78470E+00_JPRB ,&
&  0.88220E+00_JPRB, 0.86063E+00_JPRB, 0.18297E+01_JPRB, 0.21707E+01_JPRB, 0.17530E+01_JPRB, 0.14324E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.97387E+00_JPRB, 0.91742E+00_JPRB, 0.85311E+00_JPRB, 0.82153E+00_JPRB, 0.77933E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.76737E+00_JPRB, 0.75655E+00_JPRB, 0.75598E+00_JPRB, 0.77239E+00_JPRB, 0.77791E+00_JPRB, 0.78267E+00_JPRB, 0.76803E+00_JPRB ,&
&  0.75794E+00_JPRB, 0.74881E+00_JPRB, 0.75940E+00_JPRB, 0.79252E+00_JPRB, 0.84779E+00_JPRB, 0.93014E+00_JPRB, 0.10450E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.12898E+01_JPRB, 0.11630E+01_JPRB, 0.11077E+01_JPRB, 0.10663E+01_JPRB, 0.10175E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.97093E+00_JPRB, 0.95566E+00_JPRB, 0.95077E+00_JPRB, 0.98268E+00_JPRB, 0.95970E+00_JPRB, 0.95540E+00_JPRB, 0.93331E+00_JPRB ,&
&  0.90201E+00_JPRB, 0.86996E+00_JPRB, 0.84154E+00_JPRB, 0.83036E+00_JPRB, 0.85279E+00_JPRB, 0.86585E+00_JPRB, 0.86275E+00_JPRB ,&
&  0.86293E+00_JPRB, 0.88683E+00_JPRB, 0.90713E+00_JPRB, 0.91411E+00_JPRB, 0.94001E+00_JPRB, 0.94612E+00_JPRB, 0.94662E+00_JPRB ,&
&  0.93834E+00_JPRB, 0.94020E+00_JPRB, 0.89645E+00_JPRB, 0.89050E+00_JPRB, 0.85541E+00_JPRB, 0.89386E+00_JPRB, 0.83785E+00_JPRB ,&
&  0.80054E+00_JPRB, 0.77597E+00_JPRB, 0.79912E+00_JPRB, 0.85545E+00_JPRB, 0.88783E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.87215E+00_JPRB, 0.87254E+00_JPRB, 0.83364E+00_JPRB, 0.82916E+00_JPRB, 0.81561E+00_JPRB, 0.81857E+00_JPRB, 0.83355E+00_JPRB ,&
&  0.82688E+00_JPRB, 0.80233E+00_JPRB, 0.77075E+00_JPRB, 0.75733E+00_JPRB, 0.76742E+00_JPRB, 0.77840E+00_JPRB, 0.74569E+00_JPRB ,&
&  0.70809E+00_JPRB, 0.69669E+00_JPRB, 0.69128E+00_JPRB, 0.69524E+00_JPRB, 0.70623E+00_JPRB, 0.72371E+00_JPRB, 0.73251E+00_JPRB ,&
&  0.72743E+00_JPRB, 0.76454E+00_JPRB, 0.80539E+00_JPRB, 0.81966E+00_JPRB, 0.80270E+00_JPRB, 0.79188E+00_JPRB, 0.80092E+00_JPRB ,&
&  0.87005E+00_JPRB, 0.18932E+01_JPRB, 0.13317E+01_JPRB, 0.22371E+01_JPRB, 0.16821E+01_JPRB, 0.13519E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.94427E+00_JPRB, 0.96886E+00_JPRB, 0.93524E+00_JPRB, 0.86879E+00_JPRB, 0.82827E+00_JPRB, 0.78840E+00_JPRB, 0.75318E+00_JPRB ,&
&  0.73917E+00_JPRB, 0.74478E+00_JPRB, 0.75192E+00_JPRB, 0.79184E+00_JPRB, 0.82199E+00_JPRB, 0.82536E+00_JPRB, 0.80556E+00_JPRB ,&
&  0.78603E+00_JPRB, 0.77220E+00_JPRB, 0.79713E+00_JPRB, 0.91016E+00_JPRB, 0.98535E+00_JPRB, 0.10283E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.12655E+01_JPRB, 0.11495E+01_JPRB, 0.10894E+01_JPRB, 0.10396E+01_JPRB, 0.99316E+00_JPRB, 0.98898E+00_JPRB ,&
&  0.96056E+00_JPRB, 0.94374E+00_JPRB, 0.94182E+00_JPRB, 0.93741E+00_JPRB, 0.95166E+00_JPRB, 0.96468E+00_JPRB, 0.93278E+00_JPRB ,&
&  0.89175E+00_JPRB, 0.85786E+00_JPRB, 0.85989E+00_JPRB, 0.87294E+00_JPRB, 0.89127E+00_JPRB, 0.88015E+00_JPRB, 0.87140E+00_JPRB ,&
&  0.88258E+00_JPRB, 0.92629E+00_JPRB, 0.93274E+00_JPRB, 0.94296E+00_JPRB, 0.93555E+00_JPRB, 0.91392E+00_JPRB, 0.89474E+00_JPRB ,&
&  0.93009E+00_JPRB, 0.92200E+00_JPRB, 0.86330E+00_JPRB, 0.84793E+00_JPRB, 0.82982E+00_JPRB, 0.84998E+00_JPRB, 0.79431E+00_JPRB ,&
&  0.77998E+00_JPRB, 0.78774E+00_JPRB, 0.86111E+00_JPRB, 0.86998E+00_JPRB, 0.88088E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.86641E+00_JPRB, 0.85509E+00_JPRB, 0.82380E+00_JPRB, 0.82499E+00_JPRB, 0.80665E+00_JPRB, 0.83662E+00_JPRB, 0.84168E+00_JPRB ,&
&  0.80737E+00_JPRB, 0.77500E+00_JPRB, 0.75655E+00_JPRB, 0.77543E+00_JPRB, 0.79516E+00_JPRB, 0.78066E+00_JPRB, 0.74043E+00_JPRB ,&
&  0.72785E+00_JPRB, 0.72661E+00_JPRB, 0.72746E+00_JPRB, 0.73141E+00_JPRB, 0.74719E+00_JPRB, 0.76961E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.76299E+00_JPRB, 0.79283E+00_JPRB, 0.81933E+00_JPRB, 0.81990E+00_JPRB, 0.80941E+00_JPRB, 0.79613E+00_JPRB, 0.81904E+00_JPRB ,&
&  0.84668E+00_JPRB, 0.16165E+01_JPRB, 0.12459E+01_JPRB, 0.20405E+01_JPRB, 0.15914E+01_JPRB, 0.13001E+01_JPRB, 0.97980E+00_JPRB ,&
&  0.86841E+00_JPRB, 0.96170E+00_JPRB, 0.92009E+00_JPRB, 0.84690E+00_JPRB, 0.79128E+00_JPRB, 0.76059E+00_JPRB, 0.73565E+00_JPRB ,&
&  0.71973E+00_JPRB, 0.75099E+00_JPRB, 0.78852E+00_JPRB, 0.84001E+00_JPRB, 0.87489E+00_JPRB, 0.87366E+00_JPRB, 0.86323E+00_JPRB ,&
&  0.85307E+00_JPRB, 0.84984E+00_JPRB, 0.93398E+00_JPRB, 0.10361E+01_JPRB, 0.11566E+01_JPRB, 0.11726E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.10883E+01_JPRB, 0.12186E+01_JPRB, 0.11448E+01_JPRB, 0.10847E+01_JPRB, 0.10207E+01_JPRB, 0.99609E+00_JPRB, 0.97472E+00_JPRB ,&
&  0.92882E+00_JPRB, 0.88040E+00_JPRB, 0.87529E+00_JPRB, 0.88898E+00_JPRB, 0.94424E+00_JPRB, 0.95335E+00_JPRB, 0.90943E+00_JPRB ,&
&  0.86699E+00_JPRB, 0.85117E+00_JPRB, 0.85277E+00_JPRB, 0.86955E+00_JPRB, 0.88646E+00_JPRB, 0.88375E+00_JPRB, 0.88310E+00_JPRB ,&
&  0.91497E+00_JPRB, 0.96797E+00_JPRB, 0.95903E+00_JPRB, 0.95520E+00_JPRB, 0.91782E+00_JPRB, 0.88752E+00_JPRB, 0.86565E+00_JPRB ,&
&  0.88710E+00_JPRB, 0.87393E+00_JPRB, 0.84133E+00_JPRB, 0.80701E+00_JPRB, 0.79853E+00_JPRB, 0.80578E+00_JPRB, 0.78491E+00_JPRB ,&
&  0.79603E+00_JPRB, 0.84740E+00_JPRB, 0.90338E+00_JPRB, 0.86830E+00_JPRB, 0.86332E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.83297E+00_JPRB, 0.80208E+00_JPRB, 0.78979E+00_JPRB, 0.77730E+00_JPRB, 0.76492E+00_JPRB, 0.79650E+00_JPRB, 0.78661E+00_JPRB ,&
&  0.76231E+00_JPRB, 0.73721E+00_JPRB, 0.75583E+00_JPRB, 0.79924E+00_JPRB, 0.79908E+00_JPRB, 0.77728E+00_JPRB, 0.76232E+00_JPRB ,&
&  0.76113E+00_JPRB, 0.77850E+00_JPRB, 0.78221E+00_JPRB, 0.79344E+00_JPRB, 0.79329E+00_JPRB, 0.78259E+00_JPRB, 0.77518E+00_JPRB ,&
&  0.79090E+00_JPRB, 0.80008E+00_JPRB, 0.80938E+00_JPRB, 0.81342E+00_JPRB, 0.82557E+00_JPRB, 0.82040E+00_JPRB, 0.87943E+00_JPRB ,&
&  0.97352E+00_JPRB, 0.15166E+01_JPRB, 0.12775E+01_JPRB, 0.15531E+01_JPRB, 0.14118E+01_JPRB, 0.11723E+01_JPRB, 0.88883E+00_JPRB ,&
&  0.78505E+00_JPRB, 0.74373E+00_JPRB, 0.88835E+00_JPRB, 0.83992E+00_JPRB, 0.80374E+00_JPRB, 0.82019E+00_JPRB, 0.82936E+00_JPRB ,&
&  0.84846E+00_JPRB, 0.86005E+00_JPRB, 0.94094E+00_JPRB, 0.97459E+00_JPRB, 0.98460E+00_JPRB, 0.97800E+00_JPRB, 0.96302E+00_JPRB ,&
&  0.98350E+00_JPRB, 0.10259E+01_JPRB, 0.11876E+01_JPRB, 0.12915E+01_JPRB, 0.12575E+01_JPRB, 0.12130E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.11225E+01_JPRB, 0.11565E+01_JPRB, 0.11415E+01_JPRB, 0.10967E+01_JPRB, 0.98367E+00_JPRB, 0.10102E+01_JPRB, 0.93940E+00_JPRB ,&
&  0.86958E+00_JPRB, 0.82728E+00_JPRB, 0.83010E+00_JPRB, 0.86077E+00_JPRB, 0.90137E+00_JPRB, 0.89838E+00_JPRB, 0.87361E+00_JPRB ,&
&  0.86044E+00_JPRB, 0.86674E+00_JPRB, 0.85378E+00_JPRB, 0.86824E+00_JPRB, 0.88032E+00_JPRB, 0.90959E+00_JPRB, 0.93725E+00_JPRB ,&
&  0.97735E+00_JPRB, 0.10102E+01_JPRB, 0.97948E+00_JPRB, 0.94434E+00_JPRB, 0.89464E+00_JPRB, 0.86089E+00_JPRB, 0.82422E+00_JPRB ,&
&  0.82951E+00_JPRB, 0.85213E+00_JPRB, 0.84867E+00_JPRB, 0.81838E+00_JPRB, 0.80075E+00_JPRB, 0.79767E+00_JPRB, 0.82046E+00_JPRB ,&
&  0.85215E+00_JPRB, 0.91188E+00_JPRB, 0.91803E+00_JPRB, 0.86658E+00_JPRB, 0.85475E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.76568E+00_JPRB, 0.75707E+00_JPRB, 0.73450E+00_JPRB, 0.71973E+00_JPRB, 0.74254E+00_JPRB, 0.77609E+00_JPRB, 0.77514E+00_JPRB ,&
&  0.75200E+00_JPRB, 0.73987E+00_JPRB, 0.77471E+00_JPRB, 0.80863E+00_JPRB, 0.80494E+00_JPRB, 0.81977E+00_JPRB, 0.82777E+00_JPRB ,&
&  0.85756E+00_JPRB, 0.84980E+00_JPRB, 0.82953E+00_JPRB, 0.80156E+00_JPRB, 0.79840E+00_JPRB, 0.78459E+00_JPRB, 0.78258E+00_JPRB ,&
&  0.78934E+00_JPRB, 0.79627E+00_JPRB, 0.81042E+00_JPRB, 0.83475E+00_JPRB, 0.85984E+00_JPRB, 0.92834E+00_JPRB, 0.10753E+01_JPRB ,&
&  0.12975E+01_JPRB, 0.17561E+01_JPRB, 0.67723E+00_JPRB, 0.87506E+00_JPRB, 0.10084E+01_JPRB, 0.88621E+00_JPRB, 0.74878E+00_JPRB ,&
&  0.66071E+00_JPRB, 0.60854E+00_JPRB, 0.71151E+00_JPRB, 0.85639E+00_JPRB, 0.88715E+00_JPRB, 0.93644E+00_JPRB, 0.96289E+00_JPRB ,&
&  0.99989E+00_JPRB, 0.10380E+01_JPRB, 0.10947E+01_JPRB, 0.11124E+01_JPRB, 0.11135E+01_JPRB, 0.11060E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.13010E+01_JPRB, 0.14906E+01_JPRB, 0.13921E+01_JPRB, 0.13565E+01_JPRB, 0.12147E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.99723E+00_JPRB, 0.10922E+01_JPRB, 0.11455E+01_JPRB, 0.10915E+01_JPRB, 0.96986E+00_JPRB, 0.90354E+00_JPRB, 0.96093E+00_JPRB ,&
&  0.87401E+00_JPRB, 0.82017E+00_JPRB, 0.80188E+00_JPRB, 0.81086E+00_JPRB, 0.86462E+00_JPRB, 0.85951E+00_JPRB, 0.84159E+00_JPRB ,&
&  0.85132E+00_JPRB, 0.84357E+00_JPRB, 0.85913E+00_JPRB, 0.88776E+00_JPRB, 0.91790E+00_JPRB, 0.97241E+00_JPRB, 0.99933E+00_JPRB ,&
&  0.99957E+00_JPRB, 0.10121E+01_JPRB, 0.10087E+01_JPRB, 0.96647E+00_JPRB, 0.90398E+00_JPRB, 0.86155E+00_JPRB, 0.84715E+00_JPRB ,&
&  0.85867E+00_JPRB, 0.86807E+00_JPRB, 0.82424E+00_JPRB, 0.78523E+00_JPRB, 0.81856E+00_JPRB, 0.84094E+00_JPRB, 0.94715E+00_JPRB ,&
&  0.10593E+01_JPRB, 0.11360E+01_JPRB, 0.10965E+01_JPRB, 0.82747E+00_JPRB, 0.81734E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.72508E+00_JPRB, 0.67960E+00_JPRB, 0.64840E+00_JPRB, 0.65960E+00_JPRB, 0.70974E+00_JPRB, 0.76095E+00_JPRB, 0.75976E+00_JPRB ,&
&  0.74508E+00_JPRB, 0.76357E+00_JPRB, 0.83490E+00_JPRB, 0.88986E+00_JPRB, 0.89691E+00_JPRB, 0.90653E+00_JPRB, 0.92215E+00_JPRB ,&
&  0.86693E+00_JPRB, 0.83266E+00_JPRB, 0.81492E+00_JPRB, 0.79515E+00_JPRB, 0.79339E+00_JPRB, 0.78547E+00_JPRB, 0.78321E+00_JPRB ,&
&  0.79054E+00_JPRB, 0.79361E+00_JPRB, 0.81973E+00_JPRB, 0.86162E+00_JPRB, 0.94151E+00_JPRB, 0.11296E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.19329E+01_JPRB, 0.12699E+01_JPRB, 0.53105E+00_JPRB, 0.72271E+00_JPRB, 0.76752E+00_JPRB, 0.74291E+00_JPRB, 0.65969E+00_JPRB ,&
&  0.63227E+00_JPRB, 0.65472E+00_JPRB, 0.76137E+00_JPRB, 0.92160E+00_JPRB, 0.98473E+00_JPRB, 0.10463E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.11607E+01_JPRB, 0.11904E+01_JPRB, 0.12126E+01_JPRB, 0.12360E+01_JPRB, 0.12670E+01_JPRB, 0.13247E+01_JPRB ,&
&  0.14448E+01_JPRB, 0.15810E+01_JPRB, 0.17201E+01_JPRB, 0.15299E+01_JPRB, 0.15043E+01_JPRB, 0.12439E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.10468E+01_JPRB, 0.11060E+01_JPRB, 0.10374E+01_JPRB, 0.92422E+00_JPRB, 0.84291E+00_JPRB, 0.94759E+00_JPRB ,&
&  0.85191E+00_JPRB, 0.79545E+00_JPRB, 0.75912E+00_JPRB, 0.77034E+00_JPRB, 0.80297E+00_JPRB, 0.81185E+00_JPRB, 0.81152E+00_JPRB ,&
&  0.82277E+00_JPRB, 0.82493E+00_JPRB, 0.86102E+00_JPRB, 0.88405E+00_JPRB, 0.90345E+00_JPRB, 0.94338E+00_JPRB, 0.94779E+00_JPRB ,&
&  0.94190E+00_JPRB, 0.93698E+00_JPRB, 0.94640E+00_JPRB, 0.96351E+00_JPRB, 0.91449E+00_JPRB, 0.83537E+00_JPRB, 0.82220E+00_JPRB ,&
&  0.81694E+00_JPRB, 0.81276E+00_JPRB, 0.78288E+00_JPRB, 0.79682E+00_JPRB, 0.81002E+00_JPRB, 0.88126E+00_JPRB, 0.96080E+00_JPRB ,&
&  0.99607E+00_JPRB, 0.97215E+00_JPRB, 0.90785E+00_JPRB, 0.78496E+00_JPRB, 0.77236E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.67226E+00_JPRB, 0.64330E+00_JPRB, 0.60386E+00_JPRB, 0.60173E+00_JPRB, 0.66040E+00_JPRB, 0.70730E+00_JPRB, 0.70888E+00_JPRB ,&
&  0.71494E+00_JPRB, 0.77763E+00_JPRB, 0.88944E+00_JPRB, 0.97729E+00_JPRB, 0.10036E+01_JPRB, 0.99325E+00_JPRB, 0.89365E+00_JPRB ,&
&  0.85330E+00_JPRB, 0.82696E+00_JPRB, 0.79841E+00_JPRB, 0.78293E+00_JPRB, 0.77366E+00_JPRB, 0.77099E+00_JPRB, 0.77802E+00_JPRB ,&
&  0.79014E+00_JPRB, 0.81782E+00_JPRB, 0.87636E+00_JPRB, 0.95594E+00_JPRB, 0.10801E+01_JPRB, 0.12209E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.98136E+00_JPRB, 0.44385E+00_JPRB, 0.49555E+00_JPRB, 0.72378E+00_JPRB, 0.68211E+00_JPRB, 0.67181E+00_JPRB, 0.64119E+00_JPRB ,&
&  0.65593E+00_JPRB, 0.72666E+00_JPRB, 0.80275E+00_JPRB, 0.95789E+00_JPRB, 0.10493E+01_JPRB, 0.11323E+01_JPRB, 0.11689E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.12650E+01_JPRB, 0.13205E+01_JPRB, 0.13689E+01_JPRB, 0.14220E+01_JPRB, 0.14805E+01_JPRB, 0.15835E+01_JPRB ,&
&  0.17081E+01_JPRB, 0.18293E+01_JPRB, 0.19387E+01_JPRB, 0.17026E+01_JPRB, 0.17423E+01_JPRB, 0.14704E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.92558E+00_JPRB, 0.91432E+00_JPRB, 0.10159E+01_JPRB, 0.94576E+00_JPRB, 0.79350E+00_JPRB, 0.84675E+00_JPRB ,&
&  0.79258E+00_JPRB, 0.77331E+00_JPRB, 0.75977E+00_JPRB, 0.70356E+00_JPRB, 0.76100E+00_JPRB, 0.78470E+00_JPRB, 0.80060E+00_JPRB ,&
&  0.78513E+00_JPRB, 0.80150E+00_JPRB, 0.82272E+00_JPRB, 0.85949E+00_JPRB, 0.85044E+00_JPRB, 0.85033E+00_JPRB, 0.83604E+00_JPRB ,&
&  0.83849E+00_JPRB, 0.84338E+00_JPRB, 0.85846E+00_JPRB, 0.88250E+00_JPRB, 0.88865E+00_JPRB, 0.81831E+00_JPRB, 0.76667E+00_JPRB ,&
&  0.74398E+00_JPRB, 0.73655E+00_JPRB, 0.71282E+00_JPRB, 0.76093E+00_JPRB, 0.78222E+00_JPRB, 0.80411E+00_JPRB, 0.83711E+00_JPRB ,&
&  0.85268E+00_JPRB, 0.85441E+00_JPRB, 0.80755E+00_JPRB, 0.75339E+00_JPRB, 0.71674E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.56500E+00_JPRB, 0.55167E+00_JPRB, 0.53623E+00_JPRB, 0.55356E+00_JPRB, 0.58724E+00_JPRB, 0.62812E+00_JPRB, 0.64529E+00_JPRB ,&
&  0.66723E+00_JPRB, 0.76402E+00_JPRB, 0.89100E+00_JPRB, 0.10018E+01_JPRB, 0.10163E+01_JPRB, 0.98007E+00_JPRB, 0.87480E+00_JPRB ,&
&  0.84095E+00_JPRB, 0.80882E+00_JPRB, 0.78699E+00_JPRB, 0.76164E+00_JPRB, 0.75295E+00_JPRB, 0.76528E+00_JPRB, 0.79266E+00_JPRB ,&
&  0.82880E+00_JPRB, 0.86704E+00_JPRB, 0.91218E+00_JPRB, 0.95768E+00_JPRB, 0.98836E+00_JPRB, 0.96873E+00_JPRB, 0.84546E+00_JPRB ,&
&  0.36210E+00_JPRB, 0.45426E+00_JPRB, 0.68243E+00_JPRB, 0.67369E+00_JPRB, 0.63301E+00_JPRB, 0.61352E+00_JPRB, 0.64057E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.68614E+00_JPRB, 0.76252E+00_JPRB, 0.94532E+00_JPRB, 0.10614E+01_JPRB, 0.11737E+01_JPRB, 0.12290E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.13449E+01_JPRB, 0.14256E+01_JPRB, 0.14939E+01_JPRB, 0.15548E+01_JPRB, 0.16127E+01_JPRB, 0.17014E+01_JPRB ,&
&  0.18353E+01_JPRB, 0.19829E+01_JPRB, 0.21081E+01_JPRB, 0.19061E+01_JPRB, 0.19855E+01_JPRB, 0.17948E+01_JPRB, 0.16181E+01_JPRB ,&
&  0.11374E+01_JPRB, 0.87138E+00_JPRB, 0.89839E+00_JPRB, 0.10076E+01_JPRB, 0.94867E+00_JPRB, 0.89675E+00_JPRB, 0.84663E+00_JPRB ,&
&  0.79428E+00_JPRB, 0.72661E+00_JPRB, 0.73372E+00_JPRB, 0.74110E+00_JPRB, 0.73838E+00_JPRB, 0.77663E+00_JPRB, 0.74814E+00_JPRB ,&
&  0.76327E+00_JPRB, 0.77303E+00_JPRB, 0.76657E+00_JPRB, 0.77227E+00_JPRB, 0.77094E+00_JPRB, 0.77198E+00_JPRB, 0.75722E+00_JPRB ,&
&  0.75453E+00_JPRB, 0.74782E+00_JPRB, 0.75633E+00_JPRB, 0.77472E+00_JPRB, 0.77891E+00_JPRB, 0.77089E+00_JPRB, 0.72988E+00_JPRB ,&
&  0.69145E+00_JPRB, 0.70617E+00_JPRB, 0.81205E+00_JPRB, 0.94978E+00_JPRB, 0.92923E+00_JPRB, 0.80146E+00_JPRB, 0.63108E+00_JPRB ,&
&  0.61656E+00_JPRB, 0.63793E+00_JPRB, 0.64087E+00_JPRB, 0.62074E+00_JPRB, 0.59475E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.45964E+00_JPRB, 0.44824E+00_JPRB, 0.44772E+00_JPRB, 0.45824E+00_JPRB, 0.46478E+00_JPRB, 0.48849E+00_JPRB, 0.51842E+00_JPRB ,&
&  0.53288E+00_JPRB, 0.60005E+00_JPRB, 0.74950E+00_JPRB, 0.87579E+00_JPRB, 0.92345E+00_JPRB, 0.95723E+00_JPRB, 0.90051E+00_JPRB ,&
&  0.87760E+00_JPRB, 0.83674E+00_JPRB, 0.77733E+00_JPRB, 0.75820E+00_JPRB, 0.76114E+00_JPRB, 0.77675E+00_JPRB, 0.79908E+00_JPRB ,&
&  0.82535E+00_JPRB, 0.85326E+00_JPRB, 0.87342E+00_JPRB, 0.89111E+00_JPRB, 0.91331E+00_JPRB, 0.87075E+00_JPRB, 0.78091E+00_JPRB ,&
&  0.42711E+00_JPRB, 0.72812E+00_JPRB, 0.71444E+00_JPRB, 0.65748E+00_JPRB, 0.59926E+00_JPRB, 0.57391E+00_JPRB, 0.61817E+00_JPRB ,&
&  0.66161E+00_JPRB, 0.65635E+00_JPRB, 0.71446E+00_JPRB, 0.83284E+00_JPRB, 0.10376E+01_JPRB, 0.11444E+01_JPRB, 0.12368E+01_JPRB ,&
&  0.13180E+01_JPRB, 0.13752E+01_JPRB, 0.14385E+01_JPRB, 0.15000E+01_JPRB, 0.15543E+01_JPRB, 0.15965E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.18765E+01_JPRB, 0.19838E+01_JPRB, 0.18740E+01_JPRB, 0.16174E+01_JPRB, 0.15293E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.10315E+01_JPRB, 0.81281E+00_JPRB, 0.89478E+00_JPRB, 0.96673E+00_JPRB, 0.95138E+00_JPRB, 0.92775E+00_JPRB, 0.88332E+00_JPRB ,&
&  0.82917E+00_JPRB, 0.76033E+00_JPRB, 0.68097E+00_JPRB, 0.68303E+00_JPRB, 0.70969E+00_JPRB, 0.72022E+00_JPRB, 0.71264E+00_JPRB ,&
&  0.71776E+00_JPRB, 0.70196E+00_JPRB, 0.68424E+00_JPRB, 0.66213E+00_JPRB, 0.65893E+00_JPRB, 0.68614E+00_JPRB, 0.70311E+00_JPRB ,&
&  0.72360E+00_JPRB, 0.67237E+00_JPRB, 0.63481E+00_JPRB, 0.64426E+00_JPRB, 0.65384E+00_JPRB, 0.66838E+00_JPRB, 0.66789E+00_JPRB ,&
&  0.71983E+00_JPRB, 0.87516E+00_JPRB, 0.12732E+01_JPRB, 0.16787E+01_JPRB, 0.13752E+01_JPRB, 0.10816E+01_JPRB, 0.60736E+00_JPRB ,&
&  0.48015E+00_JPRB, 0.46351E+00_JPRB, 0.48123E+00_JPRB, 0.48414E+00_JPRB, 0.48203E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.40858E+00_JPRB, 0.39964E+00_JPRB, 0.40673E+00_JPRB, 0.40525E+00_JPRB, 0.40885E+00_JPRB, 0.41558E+00_JPRB, 0.41982E+00_JPRB ,&
&  0.42302E+00_JPRB, 0.43913E+00_JPRB, 0.52508E+00_JPRB, 0.64234E+00_JPRB, 0.75153E+00_JPRB, 0.83246E+00_JPRB, 0.86331E+00_JPRB ,&
&  0.88850E+00_JPRB, 0.88647E+00_JPRB, 0.85575E+00_JPRB, 0.82106E+00_JPRB, 0.80822E+00_JPRB, 0.83218E+00_JPRB, 0.84846E+00_JPRB ,&
&  0.86103E+00_JPRB, 0.86979E+00_JPRB, 0.87513E+00_JPRB, 0.88735E+00_JPRB, 0.89371E+00_JPRB, 0.80911E+00_JPRB, 0.49525E+00_JPRB ,&
&  0.73843E+00_JPRB, 0.78802E+00_JPRB, 0.77118E+00_JPRB, 0.69984E+00_JPRB, 0.63945E+00_JPRB, 0.59118E+00_JPRB, 0.59483E+00_JPRB ,&
&  0.60927E+00_JPRB, 0.61376E+00_JPRB, 0.63133E+00_JPRB, 0.68832E+00_JPRB, 0.93921E+00_JPRB, 0.10312E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.12561E+01_JPRB, 0.13041E+01_JPRB, 0.13678E+01_JPRB, 0.14135E+01_JPRB, 0.14301E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.14710E+01_JPRB, 0.15057E+01_JPRB, 0.14878E+01_JPRB, 0.14059E+01_JPRB, 0.12114E+01_JPRB, 0.12542E+01_JPRB, 0.98718E+00_JPRB ,&
&  0.80487E+00_JPRB, 0.74980E+00_JPRB, 0.84784E+00_JPRB, 0.94204E+00_JPRB, 0.94805E+00_JPRB, 0.90946E+00_JPRB, 0.83479E+00_JPRB ,&
&  0.77372E+00_JPRB, 0.72271E+00_JPRB, 0.68136E+00_JPRB, 0.64904E+00_JPRB, 0.65051E+00_JPRB, 0.65521E+00_JPRB, 0.64720E+00_JPRB ,&
&  0.64049E+00_JPRB, 0.61183E+00_JPRB, 0.57796E+00_JPRB, 0.57351E+00_JPRB, 0.58604E+00_JPRB, 0.66007E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.94218E+00_JPRB, 0.57103E+00_JPRB, 0.53835E+00_JPRB, 0.51280E+00_JPRB, 0.52077E+00_JPRB, 0.55881E+00_JPRB, 0.64716E+00_JPRB ,&
&  0.86701E+00_JPRB, 0.15822E+01_JPRB, 0.24855E+01_JPRB, 0.18997E+01_JPRB, 0.21774E+01_JPRB, 0.14889E+01_JPRB, 0.58946E+00_JPRB ,&
&  0.45211E+00_JPRB, 0.44254E+00_JPRB, 0.44745E+00_JPRB, 0.43669E+00_JPRB, 0.42355E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.41681E+00_JPRB, 0.40705E+00_JPRB, 0.41394E+00_JPRB, 0.42344E+00_JPRB, 0.41940E+00_JPRB, 0.41862E+00_JPRB, 0.41357E+00_JPRB ,&
&  0.40405E+00_JPRB, 0.39791E+00_JPRB, 0.41873E+00_JPRB, 0.45846E+00_JPRB, 0.51281E+00_JPRB, 0.58282E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.68790E+00_JPRB, 0.72956E+00_JPRB, 0.76234E+00_JPRB, 0.78627E+00_JPRB, 0.77759E+00_JPRB, 0.76693E+00_JPRB, 0.74285E+00_JPRB ,&
&  0.72259E+00_JPRB, 0.71962E+00_JPRB, 0.73834E+00_JPRB, 0.74723E+00_JPRB, 0.77606E+00_JPRB, 0.80399E+00_JPRB, 0.53136E+00_JPRB ,&
&  0.87755E+00_JPRB, 0.89140E+00_JPRB, 0.82795E+00_JPRB, 0.74564E+00_JPRB, 0.70116E+00_JPRB, 0.63697E+00_JPRB, 0.57346E+00_JPRB ,&
&  0.54792E+00_JPRB, 0.54354E+00_JPRB, 0.54126E+00_JPRB, 0.61858E+00_JPRB, 0.73565E+00_JPRB, 0.81669E+00_JPRB, 0.89760E+00_JPRB ,&
&  0.94251E+00_JPRB, 0.99921E+00_JPRB, 0.10449E+01_JPRB, 0.10846E+01_JPRB, 0.11159E+01_JPRB, 0.11132E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.95742E+00_JPRB, 0.93683E+00_JPRB, 0.98323E+00_JPRB, 0.93629E+00_JPRB, 0.89673E+00_JPRB, 0.79540E+00_JPRB ,&
&  0.64392E+00_JPRB, 0.62700E+00_JPRB, 0.71002E+00_JPRB, 0.94010E+00_JPRB, 0.97296E+00_JPRB, 0.92998E+00_JPRB, 0.86638E+00_JPRB ,&
&  0.80766E+00_JPRB, 0.75253E+00_JPRB, 0.70584E+00_JPRB, 0.66282E+00_JPRB, 0.63704E+00_JPRB, 0.61946E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.57927E+00_JPRB, 0.54555E+00_JPRB, 0.51138E+00_JPRB, 0.49842E+00_JPRB, 0.48998E+00_JPRB, 0.50214E+00_JPRB, 0.51374E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.49513E+00_JPRB, 0.45837E+00_JPRB, 0.45907E+00_JPRB, 0.42606E+00_JPRB, 0.47520E+00_JPRB, 0.65172E+00_JPRB ,&
&  0.10719E+01_JPRB, 0.16145E+01_JPRB, 0.27597E+01_JPRB, 0.49894E+01_JPRB, 0.27496E+01_JPRB, 0.21135E+01_JPRB, 0.48834E+00_JPRB ,&
&  0.46554E+00_JPRB, 0.46723E+00_JPRB, 0.46345E+00_JPRB, 0.44568E+00_JPRB, 0.44055E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.45648E+00_JPRB, 0.45423E+00_JPRB, 0.46091E+00_JPRB, 0.46835E+00_JPRB, 0.46981E+00_JPRB, 0.47040E+00_JPRB, 0.47001E+00_JPRB ,&
&  0.47119E+00_JPRB, 0.46983E+00_JPRB, 0.47081E+00_JPRB, 0.47714E+00_JPRB, 0.49110E+00_JPRB, 0.52210E+00_JPRB, 0.55741E+00_JPRB ,&
&  0.58750E+00_JPRB, 0.61412E+00_JPRB, 0.63793E+00_JPRB, 0.65725E+00_JPRB, 0.67639E+00_JPRB, 0.68575E+00_JPRB, 0.72472E+00_JPRB ,&
&  0.77398E+00_JPRB, 0.83422E+00_JPRB, 0.89239E+00_JPRB, 0.92828E+00_JPRB, 0.94669E+00_JPRB, 0.87100E+00_JPRB, 0.65857E+00_JPRB ,&
&  0.10669E+01_JPRB, 0.10622E+01_JPRB, 0.94451E+00_JPRB, 0.83866E+00_JPRB, 0.76718E+00_JPRB, 0.70719E+00_JPRB, 0.64646E+00_JPRB ,&
&  0.58834E+00_JPRB, 0.58667E+00_JPRB, 0.54627E+00_JPRB, 0.54634E+00_JPRB, 0.55270E+00_JPRB, 0.59437E+00_JPRB, 0.69382E+00_JPRB ,&
&  0.76306E+00_JPRB, 0.81448E+00_JPRB, 0.87259E+00_JPRB, 0.94407E+00_JPRB, 0.97863E+00_JPRB, 0.95635E+00_JPRB, 0.85887E+00_JPRB ,&
&  0.79971E+00_JPRB, 0.76616E+00_JPRB, 0.71350E+00_JPRB, 0.74977E+00_JPRB, 0.76767E+00_JPRB, 0.75434E+00_JPRB, 0.73550E+00_JPRB ,&
&  0.62400E+00_JPRB, 0.64792E+00_JPRB, 0.75860E+00_JPRB, 0.10196E+01_JPRB, 0.97249E+00_JPRB, 0.90647E+00_JPRB, 0.88309E+00_JPRB ,&
&  0.82334E+00_JPRB, 0.77374E+00_JPRB, 0.73292E+00_JPRB, 0.68651E+00_JPRB, 0.64785E+00_JPRB, 0.62346E+00_JPRB, 0.59468E+00_JPRB ,&
&  0.55835E+00_JPRB, 0.51563E+00_JPRB, 0.47863E+00_JPRB, 0.45064E+00_JPRB, 0.42986E+00_JPRB, 0.37994E+00_JPRB, 0.48004E+00_JPRB ,&
&  0.47774E+00_JPRB, 0.43615E+00_JPRB, 0.47908E+00_JPRB, 0.34872E+00_JPRB, 0.43650E+00_JPRB, 0.48909E+00_JPRB, 0.61407E+00_JPRB ,&
&  0.78473E+00_JPRB, 0.12639E+01_JPRB, 0.19939E+01_JPRB, 0.17964E+01_JPRB, 0.13636E+01_JPRB, 0.86425E+00_JPRB, 0.50287E+00_JPRB ,&
&  0.48110E+00_JPRB, 0.47591E+00_JPRB, 0.47966E+00_JPRB, 0.46899E+00_JPRB, 0.46462E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.48460E+00_JPRB, 0.48365E+00_JPRB, 0.49619E+00_JPRB, 0.50880E+00_JPRB, 0.51578E+00_JPRB, 0.51987E+00_JPRB, 0.52688E+00_JPRB ,&
&  0.52921E+00_JPRB, 0.53402E+00_JPRB, 0.54005E+00_JPRB, 0.55307E+00_JPRB, 0.57382E+00_JPRB, 0.60658E+00_JPRB, 0.64382E+00_JPRB ,&
&  0.68864E+00_JPRB, 0.73701E+00_JPRB, 0.79008E+00_JPRB, 0.84721E+00_JPRB, 0.91895E+00_JPRB, 0.10172E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.13114E+01_JPRB, 0.14148E+01_JPRB, 0.14962E+01_JPRB, 0.14900E+01_JPRB, 0.14250E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.12229E+01_JPRB, 0.11736E+01_JPRB, 0.10509E+01_JPRB, 0.93550E+00_JPRB, 0.85804E+00_JPRB, 0.79567E+00_JPRB, 0.75086E+00_JPRB ,&
&  0.73641E+00_JPRB, 0.70547E+00_JPRB, 0.65988E+00_JPRB, 0.62825E+00_JPRB, 0.61035E+00_JPRB, 0.61798E+00_JPRB, 0.61014E+00_JPRB ,&
&  0.66411E+00_JPRB, 0.77483E+00_JPRB, 0.91796E+00_JPRB, 0.10956E+01_JPRB, 0.11916E+01_JPRB, 0.11735E+01_JPRB, 0.10201E+01_JPRB ,&
&  0.90850E+00_JPRB, 0.82509E+00_JPRB, 0.79624E+00_JPRB, 0.82836E+00_JPRB, 0.85195E+00_JPRB, 0.89321E+00_JPRB, 0.89542E+00_JPRB ,&
&  0.82460E+00_JPRB, 0.76648E+00_JPRB, 0.82661E+00_JPRB, 0.10409E+01_JPRB, 0.97242E+00_JPRB, 0.86924E+00_JPRB, 0.85965E+00_JPRB ,&
&  0.79794E+00_JPRB, 0.75095E+00_JPRB, 0.72466E+00_JPRB, 0.68157E+00_JPRB, 0.63764E+00_JPRB, 0.60773E+00_JPRB, 0.58581E+00_JPRB ,&
&  0.54367E+00_JPRB, 0.49640E+00_JPRB, 0.45578E+00_JPRB, 0.43489E+00_JPRB, 0.43583E+00_JPRB, 0.40049E+00_JPRB, 0.45968E+00_JPRB ,&
&  0.45556E+00_JPRB, 0.35961E+00_JPRB, 0.43613E+00_JPRB, 0.52371E+00_JPRB, 0.52698E+00_JPRB, 0.56151E+00_JPRB, 0.63118E+00_JPRB ,&
&  0.70442E+00_JPRB, 0.76952E+00_JPRB, 0.76196E+00_JPRB, 0.65028E+00_JPRB, 0.60587E+00_JPRB, 0.58159E+00_JPRB, 0.55535E+00_JPRB ,&
&  0.53416E+00_JPRB, 0.49730E+00_JPRB, 0.50818E+00_JPRB, 0.49381E+00_JPRB, 0.48997E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.52888E+00_JPRB, 0.52395E+00_JPRB, 0.52466E+00_JPRB, 0.52417E+00_JPRB, 0.53391E+00_JPRB, 0.53547E+00_JPRB, 0.53497E+00_JPRB ,&
&  0.53299E+00_JPRB, 0.54067E+00_JPRB, 0.54573E+00_JPRB, 0.55378E+00_JPRB, 0.56360E+00_JPRB, 0.58930E+00_JPRB, 0.61878E+00_JPRB ,&
&  0.65414E+00_JPRB, 0.69640E+00_JPRB, 0.74194E+00_JPRB, 0.80674E+00_JPRB, 0.89737E+00_JPRB, 0.95698E+00_JPRB, 0.10247E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.12413E+01_JPRB, 0.13811E+01_JPRB, 0.15684E+01_JPRB, 0.17772E+01_JPRB, 0.17851E+01_JPRB, 0.16234E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.11751E+01_JPRB, 0.10829E+01_JPRB, 0.97175E+00_JPRB, 0.97138E+00_JPRB, 0.94005E+00_JPRB, 0.94738E+00_JPRB ,&
&  0.97067E+00_JPRB, 0.96992E+00_JPRB, 0.96049E+00_JPRB, 0.96659E+00_JPRB, 0.97125E+00_JPRB, 0.97729E+00_JPRB, 0.96216E+00_JPRB ,&
&  0.95685E+00_JPRB, 0.99278E+00_JPRB, 0.11487E+01_JPRB, 0.13473E+01_JPRB, 0.14835E+01_JPRB, 0.15210E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.12632E+01_JPRB, 0.11657E+01_JPRB, 0.11372E+01_JPRB, 0.11635E+01_JPRB, 0.11793E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.97819E+00_JPRB, 0.97396E+00_JPRB, 0.97402E+00_JPRB, 0.92899E+00_JPRB, 0.87866E+00_JPRB, 0.82024E+00_JPRB ,&
&  0.80534E+00_JPRB, 0.75610E+00_JPRB, 0.72650E+00_JPRB, 0.67747E+00_JPRB, 0.63018E+00_JPRB, 0.59858E+00_JPRB, 0.57654E+00_JPRB ,&
&  0.54032E+00_JPRB, 0.50435E+00_JPRB, 0.47424E+00_JPRB, 0.46699E+00_JPRB, 0.43960E+00_JPRB, 0.46100E+00_JPRB, 0.49797E+00_JPRB ,&
&  0.49729E+00_JPRB, 0.44194E+00_JPRB, 0.45293E+00_JPRB, 0.59978E+00_JPRB, 0.61868E+00_JPRB, 0.62623E+00_JPRB, 0.63283E+00_JPRB ,&
&  0.63808E+00_JPRB, 0.63382E+00_JPRB, 0.61033E+00_JPRB, 0.60393E+00_JPRB, 0.61115E+00_JPRB, 0.61670E+00_JPRB, 0.60489E+00_JPRB ,&
&  0.56874E+00_JPRB, 0.55104E+00_JPRB, 0.57285E+00_JPRB, 0.53839E+00_JPRB, 0.53232E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.60280E+00_JPRB, 0.57700E+00_JPRB, 0.56051E+00_JPRB, 0.56726E+00_JPRB, 0.57549E+00_JPRB, 0.56489E+00_JPRB, 0.59309E+00_JPRB ,&
&  0.56968E+00_JPRB, 0.58969E+00_JPRB, 0.60189E+00_JPRB, 0.60045E+00_JPRB, 0.63730E+00_JPRB, 0.67423E+00_JPRB, 0.72085E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.88685E+00_JPRB, 0.10084E+01_JPRB, 0.11573E+01_JPRB, 0.13379E+01_JPRB, 0.14202E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.16175E+01_JPRB, 0.17713E+01_JPRB, 0.19969E+01_JPRB, 0.22607E+01_JPRB, 0.25702E+01_JPRB, 0.23756E+01_JPRB, 0.19876E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.11206E+01_JPRB, 0.10581E+01_JPRB, 0.97335E+00_JPRB, 0.10130E+01_JPRB, 0.11161E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11826E+01_JPRB, 0.12132E+01_JPRB, 0.12446E+01_JPRB, 0.12821E+01_JPRB, 0.13267E+01_JPRB, 0.13755E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.14166E+01_JPRB, 0.14351E+01_JPRB, 0.14656E+01_JPRB, 0.15532E+01_JPRB, 0.16490E+01_JPRB, 0.17338E+01_JPRB, 0.17763E+01_JPRB ,&
&  0.18115E+01_JPRB, 0.17107E+01_JPRB, 0.14600E+01_JPRB, 0.14286E+01_JPRB, 0.14571E+01_JPRB, 0.14772E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.15980E+01_JPRB, 0.13070E+01_JPRB, 0.90854E+00_JPRB, 0.91857E+00_JPRB, 0.90768E+00_JPRB, 0.91209E+00_JPRB, 0.80917E+00_JPRB ,&
&  0.78752E+00_JPRB, 0.76693E+00_JPRB, 0.75499E+00_JPRB, 0.70717E+00_JPRB, 0.67815E+00_JPRB, 0.65048E+00_JPRB, 0.61592E+00_JPRB ,&
&  0.56373E+00_JPRB, 0.53666E+00_JPRB, 0.50720E+00_JPRB, 0.49200E+00_JPRB, 0.51510E+00_JPRB, 0.47897E+00_JPRB, 0.55984E+00_JPRB ,&
&  0.58512E+00_JPRB, 0.62648E+00_JPRB, 0.56791E+00_JPRB, 0.69651E+00_JPRB, 0.69220E+00_JPRB, 0.68150E+00_JPRB, 0.67367E+00_JPRB ,&
&  0.68728E+00_JPRB, 0.69612E+00_JPRB, 0.70490E+00_JPRB, 0.69322E+00_JPRB, 0.70830E+00_JPRB, 0.73198E+00_JPRB, 0.72864E+00_JPRB ,&
&  0.68593E+00_JPRB, 0.66918E+00_JPRB, 0.64566E+00_JPRB, 0.61714E+00_JPRB, 0.61215E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.64856E+00_JPRB, 0.65289E+00_JPRB, 0.65277E+00_JPRB, 0.65790E+00_JPRB, 0.66292E+00_JPRB, 0.65154E+00_JPRB, 0.67370E+00_JPRB ,&
&  0.67013E+00_JPRB, 0.69151E+00_JPRB, 0.71598E+00_JPRB, 0.74148E+00_JPRB, 0.78842E+00_JPRB, 0.83568E+00_JPRB, 0.89947E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.11715E+01_JPRB, 0.13771E+01_JPRB, 0.16277E+01_JPRB, 0.16876E+01_JPRB, 0.17167E+01_JPRB, 0.17973E+01_JPRB ,&
&  0.18933E+01_JPRB, 0.19590E+01_JPRB, 0.19653E+01_JPRB, 0.19981E+01_JPRB, 0.20300E+01_JPRB, 0.19395E+01_JPRB, 0.18302E+01_JPRB ,&
&  0.14765E+01_JPRB, 0.11319E+01_JPRB, 0.11836E+01_JPRB, 0.11055E+01_JPRB, 0.11361E+01_JPRB, 0.11701E+01_JPRB, 0.12001E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.12434E+01_JPRB, 0.12612E+01_JPRB, 0.12870E+01_JPRB, 0.13280E+01_JPRB, 0.13629E+01_JPRB, 0.14024E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.14632E+01_JPRB, 0.14970E+01_JPRB, 0.14593E+01_JPRB, 0.15046E+01_JPRB, 0.15748E+01_JPRB, 0.16506E+01_JPRB ,&
&  0.17300E+01_JPRB, 0.16845E+01_JPRB, 0.14674E+01_JPRB, 0.15169E+01_JPRB, 0.16325E+01_JPRB, 0.15563E+01_JPRB, 0.16025E+01_JPRB ,&
&  0.17773E+01_JPRB, 0.16035E+01_JPRB, 0.90609E+00_JPRB, 0.80811E+00_JPRB, 0.84369E+00_JPRB, 0.86346E+00_JPRB, 0.86408E+00_JPRB ,&
&  0.82882E+00_JPRB, 0.80039E+00_JPRB, 0.77920E+00_JPRB, 0.76312E+00_JPRB, 0.73404E+00_JPRB, 0.72097E+00_JPRB, 0.66591E+00_JPRB ,&
&  0.54400E+00_JPRB, 0.52312E+00_JPRB, 0.52214E+00_JPRB, 0.52631E+00_JPRB, 0.55370E+00_JPRB, 0.57060E+00_JPRB, 0.60342E+00_JPRB ,&
&  0.65746E+00_JPRB, 0.71763E+00_JPRB, 0.77234E+00_JPRB, 0.72090E+00_JPRB, 0.65969E+00_JPRB, 0.73416E+00_JPRB, 0.73656E+00_JPRB ,&
&  0.75063E+00_JPRB, 0.75150E+00_JPRB, 0.75991E+00_JPRB, 0.76231E+00_JPRB, 0.76800E+00_JPRB, 0.75256E+00_JPRB, 0.73470E+00_JPRB ,&
&  0.71539E+00_JPRB, 0.69901E+00_JPRB, 0.68761E+00_JPRB, 0.66863E+00_JPRB, 0.65459E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.70073E+00_JPRB, 0.68245E+00_JPRB, 0.67310E+00_JPRB, 0.66202E+00_JPRB, 0.65977E+00_JPRB, 0.65655E+00_JPRB, 0.65711E+00_JPRB ,&
&  0.65966E+00_JPRB, 0.66654E+00_JPRB, 0.68843E+00_JPRB, 0.72537E+00_JPRB, 0.77579E+00_JPRB, 0.84262E+00_JPRB, 0.95291E+00_JPRB ,&
&  0.10603E+01_JPRB, 0.12174E+01_JPRB, 0.14011E+01_JPRB, 0.16081E+01_JPRB, 0.17445E+01_JPRB, 0.17732E+01_JPRB, 0.18535E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.18893E+01_JPRB, 0.18060E+01_JPRB, 0.16914E+01_JPRB, 0.15933E+01_JPRB, 0.15878E+01_JPRB, 0.16497E+01_JPRB ,&
&  0.14915E+01_JPRB, 0.11834E+01_JPRB, 0.11186E+01_JPRB, 0.11074E+01_JPRB, 0.11148E+01_JPRB, 0.11314E+01_JPRB, 0.11670E+01_JPRB ,&
&  0.11964E+01_JPRB, 0.12158E+01_JPRB, 0.12282E+01_JPRB, 0.12419E+01_JPRB, 0.12909E+01_JPRB, 0.13043E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.13655E+01_JPRB, 0.14163E+01_JPRB, 0.13464E+01_JPRB, 0.14460E+01_JPRB, 0.14681E+01_JPRB, 0.14643E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.13571E+01_JPRB, 0.13998E+01_JPRB, 0.14761E+01_JPRB, 0.15517E+01_JPRB, 0.15345E+01_JPRB, 0.14755E+01_JPRB ,&
&  0.15781E+01_JPRB, 0.15274E+01_JPRB, 0.94626E+00_JPRB, 0.85070E+00_JPRB, 0.89866E+00_JPRB, 0.82240E+00_JPRB, 0.91485E+00_JPRB ,&
&  0.88099E+00_JPRB, 0.85027E+00_JPRB, 0.81785E+00_JPRB, 0.80493E+00_JPRB, 0.81021E+00_JPRB, 0.87406E+00_JPRB, 0.70797E+00_JPRB ,&
&  0.59453E+00_JPRB, 0.55464E+00_JPRB, 0.54110E+00_JPRB, 0.54902E+00_JPRB, 0.58033E+00_JPRB, 0.60927E+00_JPRB, 0.65281E+00_JPRB ,&
&  0.71503E+00_JPRB, 0.77940E+00_JPRB, 0.83650E+00_JPRB, 0.83336E+00_JPRB, 0.75503E+00_JPRB, 0.79582E+00_JPRB, 0.81476E+00_JPRB ,&
&  0.82578E+00_JPRB, 0.82646E+00_JPRB, 0.81462E+00_JPRB, 0.81213E+00_JPRB, 0.79717E+00_JPRB, 0.78960E+00_JPRB, 0.78080E+00_JPRB ,&
&  0.76897E+00_JPRB, 0.76599E+00_JPRB, 0.75687E+00_JPRB, 0.72832E+00_JPRB, 0.71172E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.75239E+00_JPRB, 0.73009E+00_JPRB, 0.71209E+00_JPRB, 0.69751E+00_JPRB, 0.68957E+00_JPRB, 0.67079E+00_JPRB, 0.64573E+00_JPRB ,&
&  0.63500E+00_JPRB, 0.63365E+00_JPRB, 0.64932E+00_JPRB, 0.68045E+00_JPRB, 0.72566E+00_JPRB, 0.77304E+00_JPRB, 0.85606E+00_JPRB ,&
&  0.95466E+00_JPRB, 0.10967E+01_JPRB, 0.12756E+01_JPRB, 0.15204E+01_JPRB, 0.17165E+01_JPRB, 0.18952E+01_JPRB, 0.19304E+01_JPRB ,&
&  0.20083E+01_JPRB, 0.20014E+01_JPRB, 0.18510E+01_JPRB, 0.16128E+01_JPRB, 0.12472E+01_JPRB, 0.12277E+01_JPRB, 0.13523E+01_JPRB ,&
&  0.13920E+01_JPRB, 0.11973E+01_JPRB, 0.10694E+01_JPRB, 0.10542E+01_JPRB, 0.10421E+01_JPRB, 0.10667E+01_JPRB, 0.10951E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.11509E+01_JPRB, 0.11711E+01_JPRB, 0.12026E+01_JPRB, 0.12323E+01_JPRB, 0.12399E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.12538E+01_JPRB, 0.12514E+01_JPRB, 0.12487E+01_JPRB, 0.12356E+01_JPRB, 0.12403E+01_JPRB, 0.12369E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.11506E+01_JPRB, 0.11955E+01_JPRB, 0.12598E+01_JPRB, 0.12603E+01_JPRB, 0.12063E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.14890E+01_JPRB, 0.96163E+00_JPRB, 0.10109E+01_JPRB, 0.11410E+01_JPRB, 0.10173E+01_JPRB, 0.94472E+00_JPRB ,&
&  0.91443E+00_JPRB, 0.87943E+00_JPRB, 0.84647E+00_JPRB, 0.82577E+00_JPRB, 0.85124E+00_JPRB, 0.93041E+00_JPRB, 0.73859E+00_JPRB ,&
&  0.67155E+00_JPRB, 0.63464E+00_JPRB, 0.63201E+00_JPRB, 0.64113E+00_JPRB, 0.64016E+00_JPRB, 0.60618E+00_JPRB, 0.60231E+00_JPRB ,&
&  0.63329E+00_JPRB, 0.91602E+00_JPRB, 0.11407E+01_JPRB, 0.13523E+01_JPRB, 0.12891E+01_JPRB, 0.81502E+00_JPRB, 0.88091E+00_JPRB ,&
&  0.89731E+00_JPRB, 0.89251E+00_JPRB, 0.85645E+00_JPRB, 0.84409E+00_JPRB, 0.83366E+00_JPRB, 0.82041E+00_JPRB, 0.81728E+00_JPRB ,&
&  0.80926E+00_JPRB, 0.81224E+00_JPRB, 0.81902E+00_JPRB, 0.80093E+00_JPRB, 0.77381E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.81231E+00_JPRB, 0.79483E+00_JPRB, 0.77470E+00_JPRB, 0.75496E+00_JPRB, 0.73707E+00_JPRB, 0.69302E+00_JPRB, 0.64773E+00_JPRB ,&
&  0.61848E+00_JPRB, 0.60608E+00_JPRB, 0.61286E+00_JPRB, 0.62638E+00_JPRB, 0.65627E+00_JPRB, 0.68986E+00_JPRB, 0.74599E+00_JPRB ,&
&  0.83092E+00_JPRB, 0.95068E+00_JPRB, 0.11158E+01_JPRB, 0.13185E+01_JPRB, 0.15608E+01_JPRB, 0.18418E+01_JPRB, 0.20274E+01_JPRB ,&
&  0.21967E+01_JPRB, 0.22182E+01_JPRB, 0.19396E+01_JPRB, 0.13607E+01_JPRB, 0.11848E+01_JPRB, 0.11477E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10365E+01_JPRB, 0.10011E+01_JPRB, 0.98193E+00_JPRB, 0.97760E+00_JPRB, 0.99638E+00_JPRB, 0.10163E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10535E+01_JPRB, 0.10740E+01_JPRB, 0.11121E+01_JPRB, 0.11284E+01_JPRB, 0.11521E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.11931E+01_JPRB, 0.11882E+01_JPRB, 0.11520E+01_JPRB, 0.11291E+01_JPRB, 0.10947E+01_JPRB, 0.10723E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10365E+01_JPRB, 0.10373E+01_JPRB, 0.10738E+01_JPRB, 0.10982E+01_JPRB, 0.10107E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.13122E+01_JPRB, 0.10776E+01_JPRB, 0.11820E+01_JPRB, 0.79405E+00_JPRB, 0.94880E+00_JPRB, 0.10315E+01_JPRB ,&
&  0.95520E+00_JPRB, 0.90577E+00_JPRB, 0.85641E+00_JPRB, 0.84886E+00_JPRB, 0.87128E+00_JPRB, 0.85676E+00_JPRB, 0.79746E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.70499E+00_JPRB, 0.71735E+00_JPRB, 0.74072E+00_JPRB, 0.73820E+00_JPRB, 0.68417E+00_JPRB, 0.62440E+00_JPRB ,&
&  0.61400E+00_JPRB, 0.81912E+00_JPRB, 0.92570E+00_JPRB, 0.10250E+01_JPRB, 0.93391E+00_JPRB, 0.78920E+00_JPRB, 0.83849E+00_JPRB ,&
&  0.87422E+00_JPRB, 0.88779E+00_JPRB, 0.87792E+00_JPRB, 0.85805E+00_JPRB, 0.83404E+00_JPRB, 0.81335E+00_JPRB, 0.80308E+00_JPRB ,&
&  0.80022E+00_JPRB, 0.79711E+00_JPRB, 0.80397E+00_JPRB, 0.81486E+00_JPRB, 0.82137E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.80688E+00_JPRB, 0.80480E+00_JPRB, 0.80587E+00_JPRB, 0.80750E+00_JPRB, 0.80115E+00_JPRB, 0.77353E+00_JPRB, 0.71425E+00_JPRB ,&
&  0.62775E+00_JPRB, 0.56228E+00_JPRB, 0.55489E+00_JPRB, 0.55973E+00_JPRB, 0.57261E+00_JPRB, 0.60483E+00_JPRB, 0.65650E+00_JPRB ,&
&  0.71750E+00_JPRB, 0.81946E+00_JPRB, 0.96537E+00_JPRB, 0.11497E+01_JPRB, 0.13855E+01_JPRB, 0.17359E+01_JPRB, 0.21185E+01_JPRB ,&
&  0.20617E+01_JPRB, 0.15367E+01_JPRB, 0.18977E+01_JPRB, 0.18861E+01_JPRB, 0.16647E+01_JPRB, 0.13598E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.10706E+01_JPRB, 0.10326E+01_JPRB, 0.10073E+01_JPRB, 0.97603E+00_JPRB, 0.92648E+00_JPRB, 0.93714E+00_JPRB, 0.95181E+00_JPRB ,&
&  0.96915E+00_JPRB, 0.98518E+00_JPRB, 0.99488E+00_JPRB, 0.10367E+01_JPRB, 0.10490E+01_JPRB, 0.10651E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.11017E+01_JPRB, 0.10730E+01_JPRB, 0.10092E+01_JPRB, 0.99052E+00_JPRB, 0.97624E+00_JPRB, 0.96420E+00_JPRB ,&
&  0.96887E+00_JPRB, 0.96126E+00_JPRB, 0.98348E+00_JPRB, 0.10108E+01_JPRB, 0.10471E+01_JPRB, 0.97563E+00_JPRB, 0.10367E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.12405E+01_JPRB, 0.12046E+01_JPRB, 0.13010E+01_JPRB, 0.94743E+00_JPRB, 0.10395E+01_JPRB, 0.99806E+00_JPRB ,&
&  0.10422E+01_JPRB, 0.95236E+00_JPRB, 0.88046E+00_JPRB, 0.85320E+00_JPRB, 0.86721E+00_JPRB, 0.88381E+00_JPRB, 0.87503E+00_JPRB ,&
&  0.78080E+00_JPRB, 0.75292E+00_JPRB, 0.75579E+00_JPRB, 0.79258E+00_JPRB, 0.78229E+00_JPRB, 0.75137E+00_JPRB, 0.72135E+00_JPRB ,&
&  0.80032E+00_JPRB, 0.84315E+00_JPRB, 0.90070E+00_JPRB, 0.92036E+00_JPRB, 0.84985E+00_JPRB, 0.77361E+00_JPRB, 0.79038E+00_JPRB ,&
&  0.81332E+00_JPRB, 0.83140E+00_JPRB, 0.83103E+00_JPRB, 0.81495E+00_JPRB, 0.78586E+00_JPRB, 0.76049E+00_JPRB, 0.74734E+00_JPRB ,&
&  0.73741E+00_JPRB, 0.74427E+00_JPRB, 0.75468E+00_JPRB, 0.76860E+00_JPRB, 0.78955E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.71931E+00_JPRB, 0.73907E+00_JPRB, 0.75569E+00_JPRB, 0.79933E+00_JPRB, 0.85318E+00_JPRB, 0.92452E+00_JPRB, 0.96001E+00_JPRB ,&
&  0.81895E+00_JPRB, 0.53650E+00_JPRB, 0.52606E+00_JPRB, 0.52211E+00_JPRB, 0.52337E+00_JPRB, 0.53528E+00_JPRB, 0.57937E+00_JPRB ,&
&  0.64560E+00_JPRB, 0.73779E+00_JPRB, 0.84794E+00_JPRB, 0.98434E+00_JPRB, 0.11658E+01_JPRB, 0.15053E+01_JPRB, 0.17596E+01_JPRB ,&
&  0.15688E+01_JPRB, 0.18881E+01_JPRB, 0.23696E+01_JPRB, 0.21348E+01_JPRB, 0.19001E+01_JPRB, 0.16457E+01_JPRB, 0.12378E+01_JPRB ,&
&  0.11559E+01_JPRB, 0.10710E+01_JPRB, 0.10505E+01_JPRB, 0.10515E+01_JPRB, 0.93765E+00_JPRB, 0.92579E+00_JPRB, 0.92797E+00_JPRB ,&
&  0.93582E+00_JPRB, 0.94117E+00_JPRB, 0.94660E+00_JPRB, 0.99267E+00_JPRB, 0.10279E+01_JPRB, 0.10440E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10476E+01_JPRB, 0.10259E+01_JPRB, 0.95407E+00_JPRB, 0.94798E+00_JPRB, 0.94978E+00_JPRB, 0.96234E+00_JPRB ,&
&  0.98068E+00_JPRB, 0.10329E+01_JPRB, 0.10597E+01_JPRB, 0.10221E+01_JPRB, 0.95531E+00_JPRB, 0.10308E+01_JPRB, 0.99166E+00_JPRB ,&
&  0.10453E+01_JPRB, 0.11312E+01_JPRB, 0.13813E+01_JPRB, 0.10818E+01_JPRB, 0.11265E+01_JPRB, 0.11977E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10673E+01_JPRB, 0.95648E+00_JPRB, 0.87645E+00_JPRB, 0.85955E+00_JPRB, 0.85332E+00_JPRB, 0.94198E+00_JPRB ,&
&  0.85358E+00_JPRB, 0.89007E+00_JPRB, 0.82417E+00_JPRB, 0.81290E+00_JPRB, 0.77914E+00_JPRB, 0.73161E+00_JPRB, 0.72924E+00_JPRB ,&
&  0.87162E+00_JPRB, 0.90367E+00_JPRB, 0.91383E+00_JPRB, 0.92198E+00_JPRB, 0.88662E+00_JPRB, 0.86278E+00_JPRB, 0.82438E+00_JPRB ,&
&  0.80651E+00_JPRB, 0.80584E+00_JPRB, 0.80768E+00_JPRB, 0.78027E+00_JPRB, 0.75469E+00_JPRB, 0.71976E+00_JPRB, 0.69153E+00_JPRB ,&
&  0.67578E+00_JPRB, 0.66904E+00_JPRB, 0.66981E+00_JPRB, 0.68166E+00_JPRB, 0.69807E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.65437E+00_JPRB, 0.66278E+00_JPRB, 0.66208E+00_JPRB, 0.63880E+00_JPRB, 0.62470E+00_JPRB, 0.61274E+00_JPRB, 0.57876E+00_JPRB ,&
&  0.55260E+00_JPRB, 0.53521E+00_JPRB, 0.51872E+00_JPRB, 0.50863E+00_JPRB, 0.51010E+00_JPRB, 0.50759E+00_JPRB, 0.52255E+00_JPRB ,&
&  0.57430E+00_JPRB, 0.65599E+00_JPRB, 0.78511E+00_JPRB, 0.87757E+00_JPRB, 0.99584E+00_JPRB, 0.12185E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13605E+01_JPRB, 0.25661E+01_JPRB, 0.23996E+01_JPRB, 0.23517E+01_JPRB, 0.22167E+01_JPRB, 0.19791E+01_JPRB, 0.14929E+01_JPRB ,&
&  0.12558E+01_JPRB, 0.11433E+01_JPRB, 0.10652E+01_JPRB, 0.10211E+01_JPRB, 0.10011E+01_JPRB, 0.95123E+00_JPRB, 0.94970E+00_JPRB ,&
&  0.94506E+00_JPRB, 0.94072E+00_JPRB, 0.92913E+00_JPRB, 0.95965E+00_JPRB, 0.10069E+01_JPRB, 0.10337E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10486E+01_JPRB, 0.10177E+01_JPRB, 0.99660E+00_JPRB, 0.99283E+00_JPRB, 0.10207E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10478E+01_JPRB, 0.11421E+01_JPRB, 0.11504E+01_JPRB, 0.11167E+01_JPRB, 0.11389E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.10628E+01_JPRB, 0.12902E+01_JPRB, 0.11432E+01_JPRB, 0.12105E+01_JPRB, 0.13105E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.11604E+01_JPRB, 0.10597E+01_JPRB, 0.96922E+00_JPRB, 0.89265E+00_JPRB, 0.87378E+00_JPRB, 0.93331E+00_JPRB ,&
&  0.95173E+00_JPRB, 0.95118E+00_JPRB, 0.97932E+00_JPRB, 0.87251E+00_JPRB, 0.83890E+00_JPRB, 0.75874E+00_JPRB, 0.75806E+00_JPRB ,&
&  0.90155E+00_JPRB, 0.92542E+00_JPRB, 0.96680E+00_JPRB, 0.95935E+00_JPRB, 0.93377E+00_JPRB, 0.94322E+00_JPRB, 0.90299E+00_JPRB ,&
&  0.85998E+00_JPRB, 0.83071E+00_JPRB, 0.81213E+00_JPRB, 0.78812E+00_JPRB, 0.75014E+00_JPRB, 0.70874E+00_JPRB, 0.67761E+00_JPRB ,&
&  0.64998E+00_JPRB, 0.63265E+00_JPRB, 0.62997E+00_JPRB, 0.63288E+00_JPRB, 0.64187E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.62285E+00_JPRB, 0.62768E+00_JPRB, 0.64193E+00_JPRB, 0.62861E+00_JPRB, 0.60830E+00_JPRB, 0.58978E+00_JPRB, 0.57937E+00_JPRB ,&
&  0.56200E+00_JPRB, 0.53768E+00_JPRB, 0.51749E+00_JPRB, 0.50624E+00_JPRB, 0.49942E+00_JPRB, 0.50120E+00_JPRB, 0.51154E+00_JPRB ,&
&  0.54146E+00_JPRB, 0.58291E+00_JPRB, 0.66891E+00_JPRB, 0.77151E+00_JPRB, 0.86720E+00_JPRB, 0.10301E+01_JPRB, 0.91367E+00_JPRB ,&
&  0.12911E+01_JPRB, 0.23659E+01_JPRB, 0.24587E+01_JPRB, 0.24884E+01_JPRB, 0.24258E+01_JPRB, 0.23520E+01_JPRB, 0.18545E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.13588E+01_JPRB, 0.11718E+01_JPRB, 0.10797E+01_JPRB, 0.10468E+01_JPRB, 0.10292E+01_JPRB, 0.10049E+01_JPRB ,&
&  0.97709E+00_JPRB, 0.95168E+00_JPRB, 0.94292E+00_JPRB, 0.97054E+00_JPRB, 0.10159E+01_JPRB, 0.10688E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.10694E+01_JPRB, 0.10766E+01_JPRB, 0.10280E+01_JPRB, 0.10678E+01_JPRB, 0.10976E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.10651E+01_JPRB, 0.12240E+01_JPRB, 0.13520E+01_JPRB, 0.13707E+01_JPRB, 0.12732E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.11388E+01_JPRB, 0.12302E+01_JPRB, 0.12218E+01_JPRB, 0.12927E+01_JPRB, 0.13909E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.13927E+01_JPRB, 0.12665E+01_JPRB, 0.11502E+01_JPRB, 0.10343E+01_JPRB, 0.95412E+00_JPRB, 0.88088E+00_JPRB, 0.95974E+00_JPRB ,&
&  0.10610E+01_JPRB, 0.11352E+01_JPRB, 0.11616E+01_JPRB, 0.10327E+01_JPRB, 0.93639E+00_JPRB, 0.77885E+00_JPRB, 0.72817E+00_JPRB ,&
&  0.86101E+00_JPRB, 0.89109E+00_JPRB, 0.84420E+00_JPRB, 0.92295E+00_JPRB, 0.89653E+00_JPRB, 0.91925E+00_JPRB, 0.91950E+00_JPRB ,&
&  0.85804E+00_JPRB, 0.82039E+00_JPRB, 0.78421E+00_JPRB, 0.76595E+00_JPRB, 0.76345E+00_JPRB, 0.73314E+00_JPRB, 0.70310E+00_JPRB ,&
&  0.67429E+00_JPRB, 0.65488E+00_JPRB, 0.63888E+00_JPRB, 0.62891E+00_JPRB, 0.62190E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.62248E+00_JPRB, 0.61573E+00_JPRB, 0.61710E+00_JPRB, 0.62440E+00_JPRB, 0.62550E+00_JPRB, 0.60753E+00_JPRB, 0.58889E+00_JPRB ,&
&  0.56760E+00_JPRB, 0.54168E+00_JPRB, 0.52269E+00_JPRB, 0.50613E+00_JPRB, 0.49912E+00_JPRB, 0.49951E+00_JPRB, 0.50840E+00_JPRB ,&
&  0.52823E+00_JPRB, 0.55930E+00_JPRB, 0.64572E+00_JPRB, 0.72127E+00_JPRB, 0.77701E+00_JPRB, 0.74326E+00_JPRB, 0.87914E+00_JPRB ,&
&  0.15999E+01_JPRB, 0.23557E+01_JPRB, 0.23315E+01_JPRB, 0.24357E+01_JPRB, 0.25660E+01_JPRB, 0.27296E+01_JPRB, 0.21925E+01_JPRB ,&
&  0.18122E+01_JPRB, 0.16234E+01_JPRB, 0.13920E+01_JPRB, 0.12445E+01_JPRB, 0.11475E+01_JPRB, 0.10991E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.99563E+00_JPRB, 0.96874E+00_JPRB, 0.98970E+00_JPRB, 0.10594E+01_JPRB, 0.11141E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.10757E+01_JPRB, 0.10834E+01_JPRB, 0.10737E+01_JPRB, 0.11062E+01_JPRB, 0.11262E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.12576E+01_JPRB, 0.13575E+01_JPRB, 0.14044E+01_JPRB, 0.14334E+01_JPRB, 0.14277E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.11323E+01_JPRB, 0.10619E+01_JPRB, 0.11870E+01_JPRB, 0.13009E+01_JPRB, 0.14747E+01_JPRB, 0.14512E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.10461E+01_JPRB, 0.10149E+01_JPRB, 0.10432E+01_JPRB, 0.10661E+01_JPRB, 0.95908E+00_JPRB, 0.10601E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.77448E+00_JPRB, 0.70209E+00_JPRB, 0.82204E+00_JPRB, 0.73787E+00_JPRB, 0.62627E+00_JPRB, 0.67185E+00_JPRB ,&
&  0.76669E+00_JPRB, 0.89482E+00_JPRB, 0.85658E+00_JPRB, 0.78675E+00_JPRB, 0.88198E+00_JPRB, 0.90425E+00_JPRB, 0.92458E+00_JPRB ,&
&  0.80869E+00_JPRB, 0.77630E+00_JPRB, 0.75132E+00_JPRB, 0.72545E+00_JPRB, 0.70926E+00_JPRB, 0.70688E+00_JPRB, 0.70406E+00_JPRB ,&
&  0.68733E+00_JPRB, 0.67171E+00_JPRB, 0.66360E+00_JPRB, 0.65468E+00_JPRB, 0.63859E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.63809E+00_JPRB, 0.62069E+00_JPRB, 0.61327E+00_JPRB, 0.60835E+00_JPRB, 0.60371E+00_JPRB, 0.59492E+00_JPRB, 0.58095E+00_JPRB ,&
&  0.56374E+00_JPRB, 0.53932E+00_JPRB, 0.51779E+00_JPRB, 0.50223E+00_JPRB, 0.49338E+00_JPRB, 0.49446E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.51733E+00_JPRB, 0.55934E+00_JPRB, 0.67625E+00_JPRB, 0.74584E+00_JPRB, 0.74803E+00_JPRB, 0.68068E+00_JPRB, 0.86767E+00_JPRB ,&
&  0.13356E+01_JPRB, 0.19804E+01_JPRB, 0.25187E+01_JPRB, 0.27074E+01_JPRB, 0.28214E+01_JPRB, 0.27308E+01_JPRB, 0.23677E+01_JPRB ,&
&  0.20439E+01_JPRB, 0.18655E+01_JPRB, 0.16596E+01_JPRB, 0.14607E+01_JPRB, 0.12954E+01_JPRB, 0.11977E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.10359E+01_JPRB, 0.98144E+00_JPRB, 0.98826E+00_JPRB, 0.10530E+01_JPRB, 0.11307E+01_JPRB, 0.11521E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.10737E+01_JPRB, 0.10815E+01_JPRB, 0.11309E+01_JPRB, 0.95759E+00_JPRB, 0.10951E+01_JPRB, 0.11053E+01_JPRB ,&
&  0.11546E+01_JPRB, 0.12318E+01_JPRB, 0.12498E+01_JPRB, 0.14804E+01_JPRB, 0.15467E+01_JPRB, 0.15037E+01_JPRB, 0.14676E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.12195E+01_JPRB, 0.10800E+01_JPRB, 0.11813E+01_JPRB, 0.12951E+01_JPRB, 0.14583E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.12335E+01_JPRB, 0.11454E+01_JPRB, 0.10063E+01_JPRB, 0.11140E+01_JPRB, 0.10532E+01_JPRB, 0.84306E+00_JPRB ,&
&  0.50509E+00_JPRB, 0.43221E+00_JPRB, 0.44647E+00_JPRB, 0.45302E+00_JPRB, 0.44002E+00_JPRB, 0.46742E+00_JPRB, 0.78811E+00_JPRB ,&
&  0.73480E+00_JPRB, 0.90454E+00_JPRB, 0.89409E+00_JPRB, 0.85431E+00_JPRB, 0.87983E+00_JPRB, 0.95446E+00_JPRB, 0.10959E+01_JPRB ,&
&  0.89193E+00_JPRB, 0.82887E+00_JPRB, 0.76090E+00_JPRB, 0.71321E+00_JPRB, 0.69886E+00_JPRB, 0.69184E+00_JPRB, 0.69075E+00_JPRB ,&
&  0.68107E+00_JPRB, 0.66743E+00_JPRB, 0.66271E+00_JPRB, 0.66251E+00_JPRB, 0.65564E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.63506E+00_JPRB, 0.62876E+00_JPRB, 0.62170E+00_JPRB, 0.61241E+00_JPRB, 0.59751E+00_JPRB, 0.58111E+00_JPRB, 0.56748E+00_JPRB ,&
&  0.55488E+00_JPRB, 0.53145E+00_JPRB, 0.51020E+00_JPRB, 0.49459E+00_JPRB, 0.48595E+00_JPRB, 0.48660E+00_JPRB, 0.49712E+00_JPRB ,&
&  0.51425E+00_JPRB, 0.57237E+00_JPRB, 0.79839E+00_JPRB, 0.76309E+00_JPRB, 0.78167E+00_JPRB, 0.73465E+00_JPRB, 0.80130E+00_JPRB ,&
&  0.11461E+01_JPRB, 0.16681E+01_JPRB, 0.22622E+01_JPRB, 0.26240E+01_JPRB, 0.30372E+01_JPRB, 0.31083E+01_JPRB, 0.28754E+01_JPRB ,&
&  0.24125E+01_JPRB, 0.20455E+01_JPRB, 0.17464E+01_JPRB, 0.15133E+01_JPRB, 0.13175E+01_JPRB, 0.11985E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10639E+01_JPRB, 0.10012E+01_JPRB, 0.99380E+00_JPRB, 0.10645E+01_JPRB, 0.11474E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.11339E+01_JPRB, 0.10736E+01_JPRB, 0.10548E+01_JPRB, 0.10861E+01_JPRB, 0.96525E+00_JPRB, 0.87312E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.13367E+01_JPRB, 0.13565E+01_JPRB, 0.15738E+01_JPRB, 0.16164E+01_JPRB, 0.15668E+01_JPRB, 0.16924E+01_JPRB ,&
&  0.16207E+01_JPRB, 0.15737E+01_JPRB, 0.11850E+01_JPRB, 0.12133E+01_JPRB, 0.13601E+01_JPRB, 0.12172E+01_JPRB, 0.12925E+01_JPRB ,&
&  0.14401E+01_JPRB, 0.12864E+01_JPRB, 0.11259E+01_JPRB, 0.96231E+00_JPRB, 0.11302E+01_JPRB, 0.93847E+00_JPRB, 0.54328E+00_JPRB ,&
&  0.46717E+00_JPRB, 0.43308E+00_JPRB, 0.40804E+00_JPRB, 0.38560E+00_JPRB, 0.36495E+00_JPRB, 0.39641E+00_JPRB, 0.60302E+00_JPRB ,&
&  0.67144E+00_JPRB, 0.87915E+00_JPRB, 0.91689E+00_JPRB, 0.87414E+00_JPRB, 0.85890E+00_JPRB, 0.96641E+00_JPRB, 0.11490E+01_JPRB ,&
&  0.93325E+00_JPRB, 0.90558E+00_JPRB, 0.82130E+00_JPRB, 0.75983E+00_JPRB, 0.71512E+00_JPRB, 0.68966E+00_JPRB, 0.68433E+00_JPRB ,&
&  0.67493E+00_JPRB, 0.66502E+00_JPRB, 0.65795E+00_JPRB, 0.65202E+00_JPRB, 0.64454E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.62511E+00_JPRB, 0.61872E+00_JPRB, 0.61763E+00_JPRB, 0.61448E+00_JPRB, 0.60366E+00_JPRB, 0.58484E+00_JPRB, 0.56708E+00_JPRB ,&
&  0.55217E+00_JPRB, 0.52667E+00_JPRB, 0.50781E+00_JPRB, 0.49382E+00_JPRB, 0.48323E+00_JPRB, 0.48205E+00_JPRB, 0.48927E+00_JPRB ,&
&  0.50306E+00_JPRB, 0.60581E+00_JPRB, 0.77385E+00_JPRB, 0.67724E+00_JPRB, 0.63744E+00_JPRB, 0.60222E+00_JPRB, 0.60917E+00_JPRB ,&
&  0.98812E+00_JPRB, 0.14380E+01_JPRB, 0.19317E+01_JPRB, 0.25786E+01_JPRB, 0.30828E+01_JPRB, 0.29709E+01_JPRB, 0.30878E+01_JPRB ,&
&  0.27386E+01_JPRB, 0.22650E+01_JPRB, 0.18948E+01_JPRB, 0.16175E+01_JPRB, 0.13831E+01_JPRB, 0.12336E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.10932E+01_JPRB, 0.10660E+01_JPRB, 0.10199E+01_JPRB, 0.99974E+00_JPRB, 0.10606E+01_JPRB, 0.11828E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.10859E+01_JPRB, 0.10418E+01_JPRB, 0.10447E+01_JPRB, 0.10626E+01_JPRB, 0.10498E+01_JPRB, 0.98475E+00_JPRB ,&
&  0.97502E+00_JPRB, 0.11083E+01_JPRB, 0.15004E+01_JPRB, 0.15045E+01_JPRB, 0.15862E+01_JPRB, 0.16738E+01_JPRB, 0.17630E+01_JPRB ,&
&  0.17012E+01_JPRB, 0.17239E+01_JPRB, 0.13701E+01_JPRB, 0.13443E+01_JPRB, 0.12012E+01_JPRB, 0.10932E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15546E+01_JPRB, 0.14690E+01_JPRB, 0.11939E+01_JPRB, 0.92675E+00_JPRB, 0.83825E+00_JPRB, 0.61289E+00_JPRB, 0.44568E+00_JPRB ,&
&  0.43057E+00_JPRB, 0.41498E+00_JPRB, 0.38059E+00_JPRB, 0.37975E+00_JPRB, 0.36824E+00_JPRB, 0.41802E+00_JPRB, 0.55675E+00_JPRB ,&
&  0.70859E+00_JPRB, 0.79717E+00_JPRB, 0.82637E+00_JPRB, 0.78684E+00_JPRB, 0.75290E+00_JPRB, 0.76574E+00_JPRB, 0.87218E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.86828E+00_JPRB, 0.81634E+00_JPRB, 0.75686E+00_JPRB, 0.72577E+00_JPRB, 0.70255E+00_JPRB, 0.69046E+00_JPRB ,&
&  0.68189E+00_JPRB, 0.67501E+00_JPRB, 0.66636E+00_JPRB, 0.65037E+00_JPRB, 0.63790E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.62054E+00_JPRB, 0.60930E+00_JPRB, 0.60714E+00_JPRB, 0.60594E+00_JPRB, 0.60161E+00_JPRB, 0.58958E+00_JPRB, 0.57239E+00_JPRB ,&
&  0.55199E+00_JPRB, 0.52872E+00_JPRB, 0.51287E+00_JPRB, 0.50062E+00_JPRB, 0.48661E+00_JPRB, 0.48553E+00_JPRB, 0.49295E+00_JPRB ,&
&  0.51965E+00_JPRB, 0.74055E+00_JPRB, 0.57008E+00_JPRB, 0.48471E+00_JPRB, 0.54617E+00_JPRB, 0.56719E+00_JPRB, 0.53257E+00_JPRB ,&
&  0.87230E+00_JPRB, 0.13260E+01_JPRB, 0.17180E+01_JPRB, 0.25604E+01_JPRB, 0.34604E+01_JPRB, 0.32612E+01_JPRB, 0.31969E+01_JPRB ,&
&  0.31080E+01_JPRB, 0.24998E+01_JPRB, 0.20836E+01_JPRB, 0.17777E+01_JPRB, 0.14866E+01_JPRB, 0.12829E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.10497E+01_JPRB, 0.10226E+01_JPRB, 0.10288E+01_JPRB, 0.10605E+01_JPRB, 0.11542E+01_JPRB, 0.12539E+01_JPRB ,&
&  0.12189E+01_JPRB, 0.11358E+01_JPRB, 0.10746E+01_JPRB, 0.10811E+01_JPRB, 0.10421E+01_JPRB, 0.10280E+01_JPRB, 0.12272E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.13834E+01_JPRB, 0.14916E+01_JPRB, 0.16848E+01_JPRB, 0.18450E+01_JPRB, 0.17845E+01_JPRB, 0.19127E+01_JPRB ,&
&  0.14906E+01_JPRB, 0.13179E+01_JPRB, 0.13190E+01_JPRB, 0.12522E+01_JPRB, 0.10726E+01_JPRB, 0.12762E+01_JPRB, 0.19596E+01_JPRB ,&
&  0.15371E+01_JPRB, 0.17109E+01_JPRB, 0.17553E+01_JPRB, 0.17233E+01_JPRB, 0.86481E+00_JPRB, 0.58679E+00_JPRB, 0.86535E+00_JPRB ,&
&  0.90545E+00_JPRB, 0.60324E+00_JPRB, 0.45148E+00_JPRB, 0.52889E+00_JPRB, 0.43476E+00_JPRB, 0.49850E+00_JPRB, 0.62843E+00_JPRB ,&
&  0.67015E+00_JPRB, 0.71893E+00_JPRB, 0.82386E+00_JPRB, 0.74362E+00_JPRB, 0.71732E+00_JPRB, 0.71960E+00_JPRB, 0.77279E+00_JPRB ,&
&  0.76057E+00_JPRB, 0.83618E+00_JPRB, 0.85031E+00_JPRB, 0.76929E+00_JPRB, 0.72418E+00_JPRB, 0.69680E+00_JPRB, 0.68730E+00_JPRB ,&
&  0.68760E+00_JPRB, 0.68312E+00_JPRB, 0.67488E+00_JPRB, 0.65813E+00_JPRB, 0.63638E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.60278E+00_JPRB, 0.59183E+00_JPRB, 0.58522E+00_JPRB, 0.58706E+00_JPRB, 0.59299E+00_JPRB, 0.59331E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.55795E+00_JPRB, 0.53677E+00_JPRB, 0.51696E+00_JPRB, 0.50585E+00_JPRB, 0.49266E+00_JPRB, 0.49131E+00_JPRB, 0.49933E+00_JPRB ,&
&  0.52705E+00_JPRB, 0.57452E+00_JPRB, 0.43820E+00_JPRB, 0.41380E+00_JPRB, 0.55433E+00_JPRB, 0.54753E+00_JPRB, 0.57322E+00_JPRB ,&
&  0.84019E+00_JPRB, 0.11458E+01_JPRB, 0.15421E+01_JPRB, 0.23465E+01_JPRB, 0.27758E+01_JPRB, 0.31650E+01_JPRB, 0.31935E+01_JPRB ,&
&  0.32455E+01_JPRB, 0.26844E+01_JPRB, 0.21735E+01_JPRB, 0.17855E+01_JPRB, 0.14845E+01_JPRB, 0.12952E+01_JPRB, 0.11680E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.99256E+00_JPRB, 0.10018E+01_JPRB, 0.10055E+01_JPRB, 0.10487E+01_JPRB, 0.10988E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.11510E+01_JPRB, 0.10621E+01_JPRB, 0.10597E+01_JPRB, 0.97544E+00_JPRB, 0.93264E+00_JPRB, 0.10359E+01_JPRB ,&
&  0.12495E+01_JPRB, 0.13939E+01_JPRB, 0.15531E+01_JPRB, 0.17252E+01_JPRB, 0.19632E+01_JPRB, 0.18680E+01_JPRB, 0.21592E+01_JPRB ,&
&  0.18112E+01_JPRB, 0.15975E+01_JPRB, 0.13937E+01_JPRB, 0.11573E+01_JPRB, 0.13386E+01_JPRB, 0.21069E+01_JPRB, 0.20880E+01_JPRB ,&
&  0.19241E+01_JPRB, 0.19548E+01_JPRB, 0.19249E+01_JPRB, 0.16476E+01_JPRB, 0.13485E+01_JPRB, 0.94530E+00_JPRB, 0.10597E+01_JPRB ,&
&  0.10117E+01_JPRB, 0.99693E+00_JPRB, 0.86563E+00_JPRB, 0.69999E+00_JPRB, 0.56249E+00_JPRB, 0.65808E+00_JPRB, 0.64519E+00_JPRB ,&
&  0.63109E+00_JPRB, 0.63050E+00_JPRB, 0.71935E+00_JPRB, 0.69876E+00_JPRB, 0.66900E+00_JPRB, 0.63577E+00_JPRB, 0.73492E+00_JPRB ,&
&  0.71972E+00_JPRB, 0.74991E+00_JPRB, 0.76501E+00_JPRB, 0.74407E+00_JPRB, 0.72748E+00_JPRB, 0.70961E+00_JPRB, 0.68872E+00_JPRB ,&
&  0.67482E+00_JPRB, 0.66363E+00_JPRB, 0.64579E+00_JPRB, 0.63139E+00_JPRB, 0.61533E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.57457E+00_JPRB, 0.56360E+00_JPRB, 0.55682E+00_JPRB, 0.55647E+00_JPRB, 0.56477E+00_JPRB, 0.57751E+00_JPRB, 0.57629E+00_JPRB ,&
&  0.56176E+00_JPRB, 0.54254E+00_JPRB, 0.52573E+00_JPRB, 0.51275E+00_JPRB, 0.50285E+00_JPRB, 0.50162E+00_JPRB, 0.50521E+00_JPRB ,&
&  0.52962E+00_JPRB, 0.49604E+00_JPRB, 0.37635E+00_JPRB, 0.40498E+00_JPRB, 0.49811E+00_JPRB, 0.49655E+00_JPRB, 0.62389E+00_JPRB ,&
&  0.77920E+00_JPRB, 0.98291E+00_JPRB, 0.12374E+01_JPRB, 0.18446E+01_JPRB, 0.22829E+01_JPRB, 0.25901E+01_JPRB, 0.26906E+01_JPRB ,&
&  0.25627E+01_JPRB, 0.23800E+01_JPRB, 0.21145E+01_JPRB, 0.18250E+01_JPRB, 0.15020E+01_JPRB, 0.13600E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.10650E+01_JPRB, 0.10027E+01_JPRB, 0.10037E+01_JPRB, 0.10249E+01_JPRB, 0.10671E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.11970E+01_JPRB, 0.11752E+01_JPRB, 0.10615E+01_JPRB, 0.10262E+01_JPRB, 0.85840E+00_JPRB, 0.84789E+00_JPRB, 0.93875E+00_JPRB ,&
&  0.14278E+01_JPRB, 0.15912E+01_JPRB, 0.16323E+01_JPRB, 0.19911E+01_JPRB, 0.18019E+01_JPRB, 0.18550E+01_JPRB, 0.22575E+01_JPRB ,&
&  0.24727E+01_JPRB, 0.24907E+01_JPRB, 0.25648E+01_JPRB, 0.19986E+01_JPRB, 0.16725E+01_JPRB, 0.23510E+01_JPRB, 0.22159E+01_JPRB ,&
&  0.20281E+01_JPRB, 0.19285E+01_JPRB, 0.17400E+01_JPRB, 0.17757E+01_JPRB, 0.14195E+01_JPRB, 0.11543E+01_JPRB, 0.80665E+00_JPRB ,&
&  0.71597E+00_JPRB, 0.64780E+00_JPRB, 0.70819E+00_JPRB, 0.65224E+00_JPRB, 0.64191E+00_JPRB, 0.61024E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.58648E+00_JPRB, 0.57919E+00_JPRB, 0.59723E+00_JPRB, 0.65025E+00_JPRB, 0.66827E+00_JPRB, 0.60146E+00_JPRB, 0.68874E+00_JPRB ,&
&  0.72549E+00_JPRB, 0.73491E+00_JPRB, 0.71201E+00_JPRB, 0.72366E+00_JPRB, 0.70936E+00_JPRB, 0.70356E+00_JPRB, 0.68959E+00_JPRB ,&
&  0.66571E+00_JPRB, 0.64345E+00_JPRB, 0.62369E+00_JPRB, 0.60418E+00_JPRB, 0.58803E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.51855E+00_JPRB, 0.51530E+00_JPRB, 0.51750E+00_JPRB, 0.52163E+00_JPRB, 0.52869E+00_JPRB, 0.54122E+00_JPRB, 0.54487E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.52820E+00_JPRB, 0.51144E+00_JPRB, 0.49899E+00_JPRB, 0.49489E+00_JPRB, 0.49957E+00_JPRB, 0.50810E+00_JPRB ,&
&  0.51905E+00_JPRB, 0.54581E+00_JPRB, 0.43022E+00_JPRB, 0.38344E+00_JPRB, 0.38593E+00_JPRB, 0.48119E+00_JPRB, 0.63779E+00_JPRB ,&
&  0.76736E+00_JPRB, 0.83702E+00_JPRB, 0.10573E+01_JPRB, 0.14449E+01_JPRB, 0.17486E+01_JPRB, 0.19155E+01_JPRB, 0.20829E+01_JPRB ,&
&  0.24500E+01_JPRB, 0.19274E+01_JPRB, 0.18456E+01_JPRB, 0.16371E+01_JPRB, 0.14816E+01_JPRB, 0.13712E+01_JPRB, 0.12655E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.10956E+01_JPRB, 0.10039E+01_JPRB, 0.95622E+00_JPRB, 0.93599E+00_JPRB, 0.93906E+00_JPRB, 0.98719E+00_JPRB ,&
&  0.10517E+01_JPRB, 0.10991E+01_JPRB, 0.10838E+01_JPRB, 0.10596E+01_JPRB, 0.10509E+01_JPRB, 0.10561E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.12384E+01_JPRB, 0.16585E+01_JPRB, 0.19478E+01_JPRB, 0.22513E+01_JPRB, 0.23782E+01_JPRB, 0.24537E+01_JPRB ,&
&  0.26093E+01_JPRB, 0.25771E+01_JPRB, 0.26184E+01_JPRB, 0.23453E+01_JPRB, 0.26111E+01_JPRB, 0.25423E+01_JPRB, 0.23105E+01_JPRB ,&
&  0.20451E+01_JPRB, 0.18685E+01_JPRB, 0.16156E+01_JPRB, 0.15122E+01_JPRB, 0.15021E+01_JPRB, 0.15078E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.82929E+00_JPRB, 0.86632E+00_JPRB, 0.74443E+00_JPRB, 0.65323E+00_JPRB, 0.57120E+00_JPRB, 0.58858E+00_JPRB, 0.60477E+00_JPRB ,&
&  0.60753E+00_JPRB, 0.60800E+00_JPRB, 0.59319E+00_JPRB, 0.64003E+00_JPRB, 0.69277E+00_JPRB, 0.65254E+00_JPRB, 0.68163E+00_JPRB ,&
&  0.64512E+00_JPRB, 0.67268E+00_JPRB, 0.66783E+00_JPRB, 0.64638E+00_JPRB, 0.68042E+00_JPRB, 0.68640E+00_JPRB, 0.66457E+00_JPRB ,&
&  0.63073E+00_JPRB, 0.60059E+00_JPRB, 0.57355E+00_JPRB, 0.54870E+00_JPRB, 0.53001E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.51056E+00_JPRB, 0.50591E+00_JPRB, 0.50482E+00_JPRB, 0.50716E+00_JPRB, 0.51181E+00_JPRB, 0.52475E+00_JPRB, 0.53259E+00_JPRB ,&
&  0.53203E+00_JPRB, 0.52778E+00_JPRB, 0.51516E+00_JPRB, 0.50255E+00_JPRB, 0.49652E+00_JPRB, 0.49731E+00_JPRB, 0.50622E+00_JPRB ,&
&  0.53073E+00_JPRB, 0.70935E+00_JPRB, 0.52037E+00_JPRB, 0.47744E+00_JPRB, 0.45002E+00_JPRB, 0.55929E+00_JPRB, 0.65756E+00_JPRB ,&
&  0.72988E+00_JPRB, 0.74710E+00_JPRB, 0.79778E+00_JPRB, 0.10764E+01_JPRB, 0.13542E+01_JPRB, 0.13900E+01_JPRB, 0.15612E+01_JPRB ,&
&  0.16222E+01_JPRB, 0.16127E+01_JPRB, 0.14387E+01_JPRB, 0.14625E+01_JPRB, 0.12860E+01_JPRB, 0.11104E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.10266E+01_JPRB, 0.97110E+00_JPRB, 0.90313E+00_JPRB, 0.87046E+00_JPRB, 0.84845E+00_JPRB, 0.85190E+00_JPRB ,&
&  0.87685E+00_JPRB, 0.91199E+00_JPRB, 0.96272E+00_JPRB, 0.10064E+01_JPRB, 0.10425E+01_JPRB, 0.11428E+01_JPRB, 0.12211E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.16597E+01_JPRB, 0.14411E+01_JPRB, 0.19278E+01_JPRB, 0.27711E+01_JPRB, 0.27573E+01_JPRB, 0.25533E+01_JPRB ,&
&  0.27049E+01_JPRB, 0.26861E+01_JPRB, 0.27255E+01_JPRB, 0.28215E+01_JPRB, 0.27711E+01_JPRB, 0.26030E+01_JPRB, 0.23277E+01_JPRB ,&
&  0.19966E+01_JPRB, 0.17588E+01_JPRB, 0.15692E+01_JPRB, 0.15533E+01_JPRB, 0.13915E+01_JPRB, 0.12774E+01_JPRB, 0.11766E+01_JPRB ,&
&  0.98426E+00_JPRB, 0.81294E+00_JPRB, 0.59363E+00_JPRB, 0.62061E+00_JPRB, 0.50074E+00_JPRB, 0.51414E+00_JPRB, 0.58231E+00_JPRB ,&
&  0.57473E+00_JPRB, 0.62535E+00_JPRB, 0.64707E+00_JPRB, 0.61632E+00_JPRB, 0.67673E+00_JPRB, 0.67251E+00_JPRB, 0.68121E+00_JPRB ,&
&  0.68392E+00_JPRB, 0.61178E+00_JPRB, 0.63529E+00_JPRB, 0.60923E+00_JPRB, 0.61344E+00_JPRB, 0.60117E+00_JPRB, 0.58794E+00_JPRB ,&
&  0.57282E+00_JPRB, 0.55353E+00_JPRB, 0.53618E+00_JPRB, 0.52385E+00_JPRB, 0.51666E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.47619E+00_JPRB, 0.46986E+00_JPRB, 0.46247E+00_JPRB, 0.46640E+00_JPRB, 0.47976E+00_JPRB, 0.49639E+00_JPRB, 0.50915E+00_JPRB ,&
&  0.50972E+00_JPRB, 0.51003E+00_JPRB, 0.50270E+00_JPRB, 0.49286E+00_JPRB, 0.48783E+00_JPRB, 0.48237E+00_JPRB, 0.49747E+00_JPRB ,&
&  0.56909E+00_JPRB, 0.71560E+00_JPRB, 0.56403E+00_JPRB, 0.46985E+00_JPRB, 0.56278E+00_JPRB, 0.58587E+00_JPRB, 0.65826E+00_JPRB ,&
&  0.65463E+00_JPRB, 0.71574E+00_JPRB, 0.72310E+00_JPRB, 0.78977E+00_JPRB, 0.99842E+00_JPRB, 0.11021E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11271E+01_JPRB, 0.11365E+01_JPRB, 0.11695E+01_JPRB, 0.11937E+01_JPRB, 0.10536E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.95405E+00_JPRB, 0.90593E+00_JPRB, 0.84984E+00_JPRB, 0.82086E+00_JPRB, 0.80605E+00_JPRB, 0.80174E+00_JPRB, 0.81437E+00_JPRB ,&
&  0.83213E+00_JPRB, 0.85078E+00_JPRB, 0.87195E+00_JPRB, 0.92568E+00_JPRB, 0.99474E+00_JPRB, 0.12324E+01_JPRB, 0.16450E+01_JPRB ,&
&  0.20295E+01_JPRB, 0.23002E+01_JPRB, 0.25285E+01_JPRB, 0.29107E+01_JPRB, 0.29647E+01_JPRB, 0.28997E+01_JPRB, 0.28204E+01_JPRB ,&
&  0.27333E+01_JPRB, 0.26264E+01_JPRB, 0.26183E+01_JPRB, 0.25963E+01_JPRB, 0.25674E+01_JPRB, 0.24842E+01_JPRB, 0.22312E+01_JPRB ,&
&  0.19106E+01_JPRB, 0.17016E+01_JPRB, 0.15851E+01_JPRB, 0.14435E+01_JPRB, 0.13182E+01_JPRB, 0.11905E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.68857E+00_JPRB, 0.54513E+00_JPRB, 0.62561E+00_JPRB, 0.53148E+00_JPRB, 0.55633E+00_JPRB, 0.64423E+00_JPRB ,&
&  0.63966E+00_JPRB, 0.63085E+00_JPRB, 0.65053E+00_JPRB, 0.61630E+00_JPRB, 0.61241E+00_JPRB, 0.68178E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.59009E+00_JPRB, 0.57888E+00_JPRB, 0.58685E+00_JPRB, 0.56880E+00_JPRB, 0.56017E+00_JPRB, 0.53951E+00_JPRB, 0.51603E+00_JPRB ,&
&  0.50208E+00_JPRB, 0.49824E+00_JPRB, 0.48978E+00_JPRB, 0.48383E+00_JPRB, 0.48012E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.45196E+00_JPRB, 0.44922E+00_JPRB, 0.44125E+00_JPRB, 0.44546E+00_JPRB, 0.46443E+00_JPRB, 0.48461E+00_JPRB, 0.49818E+00_JPRB ,&
&  0.50083E+00_JPRB, 0.50357E+00_JPRB, 0.49687E+00_JPRB, 0.48988E+00_JPRB, 0.48457E+00_JPRB, 0.47040E+00_JPRB, 0.49158E+00_JPRB ,&
&  0.41706E+00_JPRB, 0.52819E+00_JPRB, 0.44058E+00_JPRB, 0.46003E+00_JPRB, 0.62613E+00_JPRB, 0.57562E+00_JPRB, 0.59963E+00_JPRB ,&
&  0.59995E+00_JPRB, 0.62537E+00_JPRB, 0.62984E+00_JPRB, 0.67191E+00_JPRB, 0.81374E+00_JPRB, 0.93662E+00_JPRB, 0.96768E+00_JPRB ,&
&  0.91791E+00_JPRB, 0.86867E+00_JPRB, 0.90787E+00_JPRB, 0.93616E+00_JPRB, 0.94259E+00_JPRB, 0.97572E+00_JPRB, 0.98008E+00_JPRB ,&
&  0.91126E+00_JPRB, 0.85606E+00_JPRB, 0.79220E+00_JPRB, 0.76237E+00_JPRB, 0.74703E+00_JPRB, 0.73791E+00_JPRB, 0.72733E+00_JPRB ,&
&  0.73031E+00_JPRB, 0.74984E+00_JPRB, 0.77930E+00_JPRB, 0.85627E+00_JPRB, 0.92256E+00_JPRB, 0.15282E+01_JPRB, 0.24835E+01_JPRB ,&
&  0.27837E+01_JPRB, 0.27674E+01_JPRB, 0.29288E+01_JPRB, 0.32457E+01_JPRB, 0.31449E+01_JPRB, 0.29552E+01_JPRB, 0.27014E+01_JPRB ,&
&  0.25829E+01_JPRB, 0.24792E+01_JPRB, 0.24292E+01_JPRB, 0.24305E+01_JPRB, 0.23310E+01_JPRB, 0.23248E+01_JPRB, 0.21496E+01_JPRB ,&
&  0.18554E+01_JPRB, 0.17111E+01_JPRB, 0.16289E+01_JPRB, 0.15937E+01_JPRB, 0.14972E+01_JPRB, 0.14866E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.11265E+01_JPRB, 0.81959E+00_JPRB, 0.73656E+00_JPRB, 0.59033E+00_JPRB, 0.61025E+00_JPRB, 0.78165E+00_JPRB ,&
&  0.76492E+00_JPRB, 0.67871E+00_JPRB, 0.61209E+00_JPRB, 0.57223E+00_JPRB, 0.55723E+00_JPRB, 0.60612E+00_JPRB, 0.56010E+00_JPRB ,&
&  0.50900E+00_JPRB, 0.52974E+00_JPRB, 0.52916E+00_JPRB, 0.51966E+00_JPRB, 0.50281E+00_JPRB, 0.48399E+00_JPRB, 0.44883E+00_JPRB ,&
&  0.44529E+00_JPRB, 0.44097E+00_JPRB, 0.44358E+00_JPRB, 0.44988E+00_JPRB, 0.45205E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.44499E+00_JPRB, 0.44064E+00_JPRB, 0.43329E+00_JPRB, 0.43926E+00_JPRB, 0.45879E+00_JPRB, 0.47776E+00_JPRB, 0.49057E+00_JPRB ,&
&  0.49315E+00_JPRB, 0.49271E+00_JPRB, 0.48996E+00_JPRB, 0.48718E+00_JPRB, 0.47742E+00_JPRB, 0.46321E+00_JPRB, 0.41934E+00_JPRB ,&
&  0.36578E+00_JPRB, 0.44096E+00_JPRB, 0.41942E+00_JPRB, 0.44504E+00_JPRB, 0.51938E+00_JPRB, 0.49712E+00_JPRB, 0.48824E+00_JPRB ,&
&  0.50990E+00_JPRB, 0.51948E+00_JPRB, 0.55546E+00_JPRB, 0.59531E+00_JPRB, 0.66015E+00_JPRB, 0.79032E+00_JPRB, 0.82735E+00_JPRB ,&
&  0.77103E+00_JPRB, 0.73722E+00_JPRB, 0.77094E+00_JPRB, 0.83669E+00_JPRB, 0.90428E+00_JPRB, 0.95756E+00_JPRB, 0.94348E+00_JPRB ,&
&  0.89985E+00_JPRB, 0.84507E+00_JPRB, 0.76430E+00_JPRB, 0.71005E+00_JPRB, 0.67840E+00_JPRB, 0.65965E+00_JPRB, 0.64829E+00_JPRB ,&
&  0.64376E+00_JPRB, 0.65595E+00_JPRB, 0.69320E+00_JPRB, 0.75477E+00_JPRB, 0.89636E+00_JPRB, 0.14245E+01_JPRB, 0.19009E+01_JPRB ,&
&  0.19509E+01_JPRB, 0.20306E+01_JPRB, 0.21833E+01_JPRB, 0.23189E+01_JPRB, 0.24526E+01_JPRB, 0.24029E+01_JPRB, 0.23476E+01_JPRB ,&
&  0.23395E+01_JPRB, 0.22501E+01_JPRB, 0.23999E+01_JPRB, 0.23566E+01_JPRB, 0.22470E+01_JPRB, 0.21776E+01_JPRB, 0.19926E+01_JPRB ,&
&  0.18376E+01_JPRB, 0.17423E+01_JPRB, 0.16939E+01_JPRB, 0.16079E+01_JPRB, 0.15308E+01_JPRB, 0.15260E+01_JPRB, 0.14079E+01_JPRB ,&
&  0.13707E+01_JPRB, 0.13152E+01_JPRB, 0.10974E+01_JPRB, 0.92874E+00_JPRB, 0.77898E+00_JPRB, 0.79007E+00_JPRB, 0.76006E+00_JPRB ,&
&  0.68508E+00_JPRB, 0.56989E+00_JPRB, 0.56084E+00_JPRB, 0.54474E+00_JPRB, 0.54484E+00_JPRB, 0.53148E+00_JPRB, 0.49136E+00_JPRB ,&
&  0.48952E+00_JPRB, 0.50035E+00_JPRB, 0.47168E+00_JPRB, 0.45581E+00_JPRB, 0.43583E+00_JPRB, 0.42200E+00_JPRB, 0.37929E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.39903E+00_JPRB, 0.41519E+00_JPRB, 0.43237E+00_JPRB, 0.44212E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.45270E+00_JPRB, 0.45031E+00_JPRB, 0.45052E+00_JPRB, 0.45012E+00_JPRB, 0.45580E+00_JPRB, 0.47485E+00_JPRB, 0.46460E+00_JPRB ,&
&  0.49390E+00_JPRB, 0.49719E+00_JPRB, 0.48968E+00_JPRB, 0.48626E+00_JPRB, 0.46858E+00_JPRB, 0.40065E+00_JPRB, 0.32009E+00_JPRB ,&
&  0.30826E+00_JPRB, 0.34384E+00_JPRB, 0.38457E+00_JPRB, 0.40923E+00_JPRB, 0.44132E+00_JPRB, 0.44258E+00_JPRB, 0.44112E+00_JPRB ,&
&  0.44254E+00_JPRB, 0.44783E+00_JPRB, 0.47422E+00_JPRB, 0.52005E+00_JPRB, 0.56685E+00_JPRB, 0.63887E+00_JPRB, 0.71607E+00_JPRB ,&
&  0.67077E+00_JPRB, 0.66237E+00_JPRB, 0.71133E+00_JPRB, 0.69625E+00_JPRB, 0.80520E+00_JPRB, 0.83006E+00_JPRB, 0.82643E+00_JPRB ,&
&  0.80384E+00_JPRB, 0.76887E+00_JPRB, 0.71944E+00_JPRB, 0.68756E+00_JPRB, 0.65460E+00_JPRB, 0.62587E+00_JPRB, 0.60774E+00_JPRB ,&
&  0.60899E+00_JPRB, 0.61849E+00_JPRB, 0.64694E+00_JPRB, 0.69664E+00_JPRB, 0.76175E+00_JPRB, 0.10644E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.16915E+01_JPRB, 0.19318E+01_JPRB, 0.18522E+01_JPRB, 0.19837E+01_JPRB, 0.20739E+01_JPRB, 0.20435E+01_JPRB ,&
&  0.21189E+01_JPRB, 0.20413E+01_JPRB, 0.20502E+01_JPRB, 0.20048E+01_JPRB, 0.19508E+01_JPRB, 0.18625E+01_JPRB, 0.17342E+01_JPRB ,&
&  0.16496E+01_JPRB, 0.17562E+01_JPRB, 0.17488E+01_JPRB, 0.16383E+01_JPRB, 0.14663E+01_JPRB, 0.13512E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11598E+01_JPRB, 0.11086E+01_JPRB, 0.10539E+01_JPRB, 0.96379E+00_JPRB, 0.87165E+00_JPRB, 0.79716E+00_JPRB ,&
&  0.72011E+00_JPRB, 0.61707E+00_JPRB, 0.56494E+00_JPRB, 0.55120E+00_JPRB, 0.53981E+00_JPRB, 0.49144E+00_JPRB, 0.47441E+00_JPRB ,&
&  0.46834E+00_JPRB, 0.43801E+00_JPRB, 0.43841E+00_JPRB, 0.41642E+00_JPRB, 0.40268E+00_JPRB, 0.39883E+00_JPRB, 0.39235E+00_JPRB ,&
&  0.38354E+00_JPRB, 0.41613E+00_JPRB, 0.43474E+00_JPRB, 0.44687E+00_JPRB, 0.45114E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.45556E+00_JPRB, 0.46376E+00_JPRB, 0.46928E+00_JPRB, 0.46396E+00_JPRB, 0.45894E+00_JPRB, 0.44797E+00_JPRB, 0.45565E+00_JPRB ,&
&  0.49349E+00_JPRB, 0.45269E+00_JPRB, 0.48975E+00_JPRB, 0.38864E+00_JPRB, 0.34286E+00_JPRB, 0.32312E+00_JPRB, 0.30717E+00_JPRB ,&
&  0.31035E+00_JPRB, 0.33358E+00_JPRB, 0.35443E+00_JPRB, 0.36909E+00_JPRB, 0.39964E+00_JPRB, 0.40344E+00_JPRB, 0.40241E+00_JPRB ,&
&  0.40803E+00_JPRB, 0.41323E+00_JPRB, 0.44069E+00_JPRB, 0.45119E+00_JPRB, 0.48184E+00_JPRB, 0.52944E+00_JPRB, 0.60228E+00_JPRB ,&
&  0.57501E+00_JPRB, 0.58370E+00_JPRB, 0.65039E+00_JPRB, 0.67226E+00_JPRB, 0.69324E+00_JPRB, 0.68142E+00_JPRB, 0.65698E+00_JPRB ,&
&  0.63755E+00_JPRB, 0.53115E+00_JPRB, 0.55583E+00_JPRB, 0.54285E+00_JPRB, 0.52981E+00_JPRB, 0.51375E+00_JPRB, 0.48936E+00_JPRB ,&
&  0.47399E+00_JPRB, 0.47653E+00_JPRB, 0.49645E+00_JPRB, 0.52731E+00_JPRB, 0.57987E+00_JPRB, 0.71818E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.10987E+01_JPRB, 0.10247E+01_JPRB, 0.13041E+01_JPRB, 0.14215E+01_JPRB, 0.15748E+01_JPRB, 0.17357E+01_JPRB, 0.18326E+01_JPRB ,&
&  0.19591E+01_JPRB, 0.18472E+01_JPRB, 0.17676E+01_JPRB, 0.16669E+01_JPRB, 0.15668E+01_JPRB, 0.14925E+01_JPRB, 0.14473E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.14044E+01_JPRB, 0.14711E+01_JPRB, 0.14070E+01_JPRB, 0.12744E+01_JPRB, 0.11912E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.98821E+00_JPRB, 0.94342E+00_JPRB, 0.84234E+00_JPRB, 0.81570E+00_JPRB, 0.79654E+00_JPRB, 0.73717E+00_JPRB ,&
&  0.70766E+00_JPRB, 0.66347E+00_JPRB, 0.60963E+00_JPRB, 0.61475E+00_JPRB, 0.56816E+00_JPRB, 0.52889E+00_JPRB, 0.51635E+00_JPRB ,&
&  0.48135E+00_JPRB, 0.41764E+00_JPRB, 0.40416E+00_JPRB, 0.39923E+00_JPRB, 0.38612E+00_JPRB, 0.38416E+00_JPRB, 0.41416E+00_JPRB ,&
&  0.41255E+00_JPRB, 0.42022E+00_JPRB, 0.44568E+00_JPRB, 0.44745E+00_JPRB, 0.45444E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.45011E+00_JPRB, 0.46442E+00_JPRB, 0.47786E+00_JPRB, 0.47124E+00_JPRB, 0.45726E+00_JPRB, 0.44261E+00_JPRB, 0.43565E+00_JPRB ,&
&  0.44257E+00_JPRB, 0.38130E+00_JPRB, 0.35134E+00_JPRB, 0.33581E+00_JPRB, 0.32909E+00_JPRB, 0.31524E+00_JPRB, 0.30126E+00_JPRB ,&
&  0.30104E+00_JPRB, 0.31257E+00_JPRB, 0.33512E+00_JPRB, 0.35282E+00_JPRB, 0.36747E+00_JPRB, 0.37670E+00_JPRB, 0.38190E+00_JPRB ,&
&  0.38831E+00_JPRB, 0.39611E+00_JPRB, 0.40238E+00_JPRB, 0.41074E+00_JPRB, 0.42028E+00_JPRB, 0.43965E+00_JPRB, 0.46653E+00_JPRB ,&
&  0.48915E+00_JPRB, 0.50251E+00_JPRB, 0.48595E+00_JPRB, 0.48971E+00_JPRB, 0.49745E+00_JPRB, 0.49732E+00_JPRB, 0.50506E+00_JPRB ,&
&  0.35159E+00_JPRB, 0.33804E+00_JPRB, 0.40264E+00_JPRB, 0.40255E+00_JPRB, 0.40963E+00_JPRB, 0.40964E+00_JPRB, 0.40884E+00_JPRB ,&
&  0.41385E+00_JPRB, 0.42638E+00_JPRB, 0.45865E+00_JPRB, 0.51408E+00_JPRB, 0.57608E+00_JPRB, 0.65233E+00_JPRB, 0.75852E+00_JPRB ,&
&  0.96457E+00_JPRB, 0.93447E+00_JPRB, 0.96866E+00_JPRB, 0.10920E+01_JPRB, 0.13323E+01_JPRB, 0.14735E+01_JPRB, 0.15189E+01_JPRB ,&
&  0.15233E+01_JPRB, 0.14529E+01_JPRB, 0.14003E+01_JPRB, 0.13873E+01_JPRB, 0.13543E+01_JPRB, 0.13000E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.12146E+01_JPRB, 0.11835E+01_JPRB, 0.11728E+01_JPRB, 0.11634E+01_JPRB, 0.10993E+01_JPRB, 0.10548E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.95316E+00_JPRB, 0.88882E+00_JPRB, 0.81636E+00_JPRB, 0.75898E+00_JPRB, 0.71699E+00_JPRB, 0.70848E+00_JPRB, 0.70085E+00_JPRB ,&
&  0.68226E+00_JPRB, 0.66926E+00_JPRB, 0.65122E+00_JPRB, 0.63267E+00_JPRB, 0.59136E+00_JPRB, 0.54707E+00_JPRB, 0.52456E+00_JPRB ,&
&  0.46242E+00_JPRB, 0.41986E+00_JPRB, 0.37045E+00_JPRB, 0.36137E+00_JPRB, 0.37355E+00_JPRB, 0.38271E+00_JPRB, 0.38624E+00_JPRB ,&
&  0.40148E+00_JPRB, 0.40682E+00_JPRB, 0.40524E+00_JPRB, 0.41496E+00_JPRB, 0.42886E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.43305E+00_JPRB, 0.44442E+00_JPRB, 0.45390E+00_JPRB, 0.45941E+00_JPRB, 0.44640E+00_JPRB, 0.43823E+00_JPRB, 0.43275E+00_JPRB ,&
&  0.43632E+00_JPRB, 0.39586E+00_JPRB, 0.36802E+00_JPRB, 0.34630E+00_JPRB, 0.33239E+00_JPRB, 0.31862E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.32032E+00_JPRB, 0.32866E+00_JPRB, 0.33898E+00_JPRB, 0.34809E+00_JPRB, 0.35487E+00_JPRB, 0.35738E+00_JPRB, 0.36245E+00_JPRB ,&
&  0.37061E+00_JPRB, 0.37646E+00_JPRB, 0.37561E+00_JPRB, 0.37593E+00_JPRB, 0.37893E+00_JPRB, 0.38885E+00_JPRB, 0.39288E+00_JPRB ,&
&  0.39345E+00_JPRB, 0.38886E+00_JPRB, 0.38665E+00_JPRB, 0.39742E+00_JPRB, 0.40109E+00_JPRB, 0.39820E+00_JPRB, 0.35785E+00_JPRB ,&
&  0.29578E+00_JPRB, 0.27362E+00_JPRB, 0.28553E+00_JPRB, 0.34926E+00_JPRB, 0.36644E+00_JPRB, 0.37903E+00_JPRB, 0.39804E+00_JPRB ,&
&  0.39414E+00_JPRB, 0.38581E+00_JPRB, 0.41913E+00_JPRB, 0.50642E+00_JPRB, 0.56781E+00_JPRB, 0.63450E+00_JPRB, 0.71266E+00_JPRB ,&
&  0.83821E+00_JPRB, 0.93786E+00_JPRB, 0.10367E+01_JPRB, 0.98405E+00_JPRB, 0.10991E+01_JPRB, 0.13703E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.12386E+01_JPRB, 0.12019E+01_JPRB, 0.12381E+01_JPRB, 0.12279E+01_JPRB, 0.11914E+01_JPRB, 0.11280E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.97996E+00_JPRB, 0.95579E+00_JPRB, 0.98216E+00_JPRB, 0.97036E+00_JPRB, 0.94492E+00_JPRB, 0.92149E+00_JPRB ,&
&  0.87602E+00_JPRB, 0.80559E+00_JPRB, 0.73735E+00_JPRB, 0.68271E+00_JPRB, 0.66277E+00_JPRB, 0.66065E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.63915E+00_JPRB, 0.63318E+00_JPRB, 0.63102E+00_JPRB, 0.62596E+00_JPRB, 0.56804E+00_JPRB, 0.50040E+00_JPRB, 0.45047E+00_JPRB ,&
&  0.41498E+00_JPRB, 0.39215E+00_JPRB, 0.37228E+00_JPRB, 0.37261E+00_JPRB, 0.37673E+00_JPRB, 0.38251E+00_JPRB, 0.37456E+00_JPRB ,&
&  0.36787E+00_JPRB, 0.36273E+00_JPRB, 0.36998E+00_JPRB, 0.39222E+00_JPRB, 0.41391E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.38519E+00_JPRB, 0.40382E+00_JPRB, 0.42933E+00_JPRB, 0.44236E+00_JPRB, 0.44815E+00_JPRB, 0.42777E+00_JPRB, 0.40792E+00_JPRB ,&
&  0.38926E+00_JPRB, 0.36710E+00_JPRB, 0.35550E+00_JPRB, 0.34903E+00_JPRB, 0.34675E+00_JPRB, 0.34665E+00_JPRB, 0.34825E+00_JPRB ,&
&  0.35045E+00_JPRB, 0.35137E+00_JPRB, 0.35430E+00_JPRB, 0.36144E+00_JPRB, 0.36755E+00_JPRB, 0.36839E+00_JPRB, 0.36738E+00_JPRB ,&
&  0.36764E+00_JPRB, 0.36750E+00_JPRB, 0.36250E+00_JPRB, 0.35415E+00_JPRB, 0.35267E+00_JPRB, 0.36347E+00_JPRB, 0.36905E+00_JPRB ,&
&  0.36909E+00_JPRB, 0.35974E+00_JPRB, 0.35956E+00_JPRB, 0.36917E+00_JPRB, 0.36741E+00_JPRB, 0.35850E+00_JPRB, 0.32434E+00_JPRB ,&
&  0.28384E+00_JPRB, 0.25262E+00_JPRB, 0.24160E+00_JPRB, 0.25478E+00_JPRB, 0.28514E+00_JPRB, 0.33785E+00_JPRB, 0.35375E+00_JPRB ,&
&  0.36664E+00_JPRB, 0.38525E+00_JPRB, 0.41581E+00_JPRB, 0.46314E+00_JPRB, 0.52652E+00_JPRB, 0.59170E+00_JPRB, 0.65037E+00_JPRB ,&
&  0.72287E+00_JPRB, 0.79729E+00_JPRB, 0.89918E+00_JPRB, 0.84801E+00_JPRB, 0.88166E+00_JPRB, 0.96499E+00_JPRB, 0.99646E+00_JPRB ,&
&  0.99012E+00_JPRB, 0.99845E+00_JPRB, 0.10150E+01_JPRB, 0.10273E+01_JPRB, 0.99169E+00_JPRB, 0.94631E+00_JPRB, 0.89244E+00_JPRB ,&
&  0.85767E+00_JPRB, 0.83230E+00_JPRB, 0.82131E+00_JPRB, 0.83744E+00_JPRB, 0.86728E+00_JPRB, 0.87268E+00_JPRB, 0.84878E+00_JPRB ,&
&  0.81131E+00_JPRB, 0.74981E+00_JPRB, 0.68401E+00_JPRB, 0.63185E+00_JPRB, 0.62113E+00_JPRB, 0.63424E+00_JPRB, 0.64337E+00_JPRB ,&
&  0.64141E+00_JPRB, 0.63382E+00_JPRB, 0.61363E+00_JPRB, 0.59560E+00_JPRB, 0.53610E+00_JPRB, 0.49116E+00_JPRB, 0.45832E+00_JPRB ,&
&  0.44477E+00_JPRB, 0.43310E+00_JPRB, 0.42349E+00_JPRB, 0.42863E+00_JPRB, 0.42086E+00_JPRB, 0.41558E+00_JPRB, 0.40822E+00_JPRB ,&
&  0.39620E+00_JPRB, 0.38295E+00_JPRB, 0.37866E+00_JPRB, 0.37525E+00_JPRB, 0.37724E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.41493E+00_JPRB, 0.41763E+00_JPRB, 0.42021E+00_JPRB, 0.42754E+00_JPRB, 0.43491E+00_JPRB, 0.43105E+00_JPRB, 0.42266E+00_JPRB ,&
&  0.40828E+00_JPRB, 0.39377E+00_JPRB, 0.38330E+00_JPRB, 0.37653E+00_JPRB, 0.37476E+00_JPRB, 0.37565E+00_JPRB, 0.37704E+00_JPRB ,&
&  0.37841E+00_JPRB, 0.37961E+00_JPRB, 0.37803E+00_JPRB, 0.37223E+00_JPRB, 0.36845E+00_JPRB, 0.36791E+00_JPRB, 0.36899E+00_JPRB ,&
&  0.37076E+00_JPRB, 0.36954E+00_JPRB, 0.36587E+00_JPRB, 0.36104E+00_JPRB, 0.35474E+00_JPRB, 0.36110E+00_JPRB, 0.35847E+00_JPRB ,&
&  0.35040E+00_JPRB, 0.34955E+00_JPRB, 0.35779E+00_JPRB, 0.35611E+00_JPRB, 0.35069E+00_JPRB, 0.33962E+00_JPRB, 0.30146E+00_JPRB ,&
&  0.26694E+00_JPRB, 0.24122E+00_JPRB, 0.22732E+00_JPRB, 0.22439E+00_JPRB, 0.23254E+00_JPRB, 0.25345E+00_JPRB, 0.28798E+00_JPRB ,&
&  0.33791E+00_JPRB, 0.35800E+00_JPRB, 0.38339E+00_JPRB, 0.41767E+00_JPRB, 0.46097E+00_JPRB, 0.51123E+00_JPRB, 0.55732E+00_JPRB ,&
&  0.59561E+00_JPRB, 0.64021E+00_JPRB, 0.70139E+00_JPRB, 0.76684E+00_JPRB, 0.84608E+00_JPRB, 0.82195E+00_JPRB, 0.83964E+00_JPRB ,&
&  0.93975E+00_JPRB, 0.92581E+00_JPRB, 0.89536E+00_JPRB, 0.83555E+00_JPRB, 0.78699E+00_JPRB, 0.74130E+00_JPRB, 0.71468E+00_JPRB ,&
&  0.70764E+00_JPRB, 0.71915E+00_JPRB, 0.74512E+00_JPRB, 0.77107E+00_JPRB, 0.78121E+00_JPRB, 0.78341E+00_JPRB, 0.76004E+00_JPRB ,&
&  0.73481E+00_JPRB, 0.75433E+00_JPRB, 0.91691E+00_JPRB, 0.83710E+00_JPRB, 0.76577E+00_JPRB, 0.73290E+00_JPRB, 0.69010E+00_JPRB ,&
&  0.66197E+00_JPRB, 0.63665E+00_JPRB, 0.60586E+00_JPRB, 0.58021E+00_JPRB, 0.54320E+00_JPRB, 0.50832E+00_JPRB, 0.48078E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.46687E+00_JPRB, 0.46054E+00_JPRB, 0.45617E+00_JPRB, 0.44875E+00_JPRB, 0.44362E+00_JPRB, 0.43765E+00_JPRB ,&
&  0.42684E+00_JPRB, 0.41825E+00_JPRB, 0.41540E+00_JPRB, 0.41298E+00_JPRB, 0.41292E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.41048E+00_JPRB, 0.41011E+00_JPRB, 0.41038E+00_JPRB, 0.41035E+00_JPRB, 0.41185E+00_JPRB, 0.41268E+00_JPRB, 0.41095E+00_JPRB ,&
&  0.40588E+00_JPRB, 0.40241E+00_JPRB, 0.39942E+00_JPRB, 0.39940E+00_JPRB, 0.40003E+00_JPRB, 0.40004E+00_JPRB, 0.39933E+00_JPRB ,&
&  0.39594E+00_JPRB, 0.38579E+00_JPRB, 0.37870E+00_JPRB, 0.37590E+00_JPRB, 0.37465E+00_JPRB, 0.37479E+00_JPRB, 0.37520E+00_JPRB ,&
&  0.37265E+00_JPRB, 0.36760E+00_JPRB, 0.36400E+00_JPRB, 0.35666E+00_JPRB, 0.35190E+00_JPRB, 0.35169E+00_JPRB, 0.35649E+00_JPRB ,&
&  0.35935E+00_JPRB, 0.34773E+00_JPRB, 0.34604E+00_JPRB, 0.34233E+00_JPRB, 0.33085E+00_JPRB, 0.32323E+00_JPRB, 0.27625E+00_JPRB ,&
&  0.24745E+00_JPRB, 0.22975E+00_JPRB, 0.22313E+00_JPRB, 0.22014E+00_JPRB, 0.22649E+00_JPRB, 0.24250E+00_JPRB, 0.27508E+00_JPRB ,&
&  0.31158E+00_JPRB, 0.35570E+00_JPRB, 0.37268E+00_JPRB, 0.39874E+00_JPRB, 0.42694E+00_JPRB, 0.46295E+00_JPRB, 0.49418E+00_JPRB ,&
&  0.50870E+00_JPRB, 0.51849E+00_JPRB, 0.53448E+00_JPRB, 0.54651E+00_JPRB, 0.56858E+00_JPRB, 0.58433E+00_JPRB, 0.58617E+00_JPRB ,&
&  0.59663E+00_JPRB, 0.60033E+00_JPRB, 0.61306E+00_JPRB, 0.61393E+00_JPRB, 0.60532E+00_JPRB, 0.59279E+00_JPRB, 0.58673E+00_JPRB ,&
&  0.58465E+00_JPRB, 0.60766E+00_JPRB, 0.64737E+00_JPRB, 0.69651E+00_JPRB, 0.71475E+00_JPRB, 0.72000E+00_JPRB, 0.70571E+00_JPRB ,&
&  0.67997E+00_JPRB, 0.69705E+00_JPRB, 0.78128E+00_JPRB, 0.76064E+00_JPRB, 0.74474E+00_JPRB, 0.72548E+00_JPRB, 0.68904E+00_JPRB ,&
&  0.64330E+00_JPRB, 0.62562E+00_JPRB, 0.60821E+00_JPRB, 0.57634E+00_JPRB, 0.54644E+00_JPRB, 0.52503E+00_JPRB, 0.51291E+00_JPRB ,&
&  0.50670E+00_JPRB, 0.50307E+00_JPRB, 0.49263E+00_JPRB, 0.48322E+00_JPRB, 0.47295E+00_JPRB, 0.45950E+00_JPRB, 0.44849E+00_JPRB ,&
&  0.43877E+00_JPRB, 0.43182E+00_JPRB, 0.42552E+00_JPRB, 0.41888E+00_JPRB, 0.41411E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.40717E+00_JPRB, 0.40639E+00_JPRB, 0.40541E+00_JPRB, 0.40475E+00_JPRB, 0.40490E+00_JPRB, 0.40647E+00_JPRB, 0.40895E+00_JPRB ,&
&  0.40972E+00_JPRB, 0.41050E+00_JPRB, 0.41086E+00_JPRB, 0.40978E+00_JPRB, 0.40731E+00_JPRB, 0.40414E+00_JPRB, 0.40138E+00_JPRB ,&
&  0.39750E+00_JPRB, 0.39315E+00_JPRB, 0.38812E+00_JPRB, 0.38347E+00_JPRB, 0.37895E+00_JPRB, 0.37645E+00_JPRB, 0.37311E+00_JPRB ,&
&  0.36989E+00_JPRB, 0.36544E+00_JPRB, 0.35755E+00_JPRB, 0.35063E+00_JPRB, 0.34832E+00_JPRB, 0.34415E+00_JPRB, 0.34554E+00_JPRB ,&
&  0.34180E+00_JPRB, 0.33412E+00_JPRB, 0.32368E+00_JPRB, 0.31817E+00_JPRB, 0.31133E+00_JPRB, 0.26657E+00_JPRB, 0.25356E+00_JPRB ,&
&  0.23899E+00_JPRB, 0.22717E+00_JPRB, 0.22344E+00_JPRB, 0.22402E+00_JPRB, 0.22913E+00_JPRB, 0.24346E+00_JPRB, 0.26400E+00_JPRB ,&
&  0.29106E+00_JPRB, 0.36386E+00_JPRB, 0.38543E+00_JPRB, 0.40931E+00_JPRB, 0.42725E+00_JPRB, 0.44729E+00_JPRB, 0.45792E+00_JPRB ,&
&  0.46438E+00_JPRB, 0.46854E+00_JPRB, 0.48190E+00_JPRB, 0.50118E+00_JPRB, 0.51758E+00_JPRB, 0.52304E+00_JPRB, 0.52263E+00_JPRB ,&
&  0.51376E+00_JPRB, 0.50500E+00_JPRB, 0.49412E+00_JPRB, 0.49018E+00_JPRB, 0.49928E+00_JPRB, 0.50841E+00_JPRB, 0.52696E+00_JPRB ,&
&  0.54301E+00_JPRB, 0.55617E+00_JPRB, 0.58107E+00_JPRB, 0.61059E+00_JPRB, 0.63179E+00_JPRB, 0.64738E+00_JPRB, 0.65151E+00_JPRB ,&
&  0.65010E+00_JPRB, 0.65930E+00_JPRB, 0.65820E+00_JPRB, 0.65249E+00_JPRB, 0.64270E+00_JPRB, 0.62544E+00_JPRB, 0.60606E+00_JPRB ,&
&  0.59648E+00_JPRB, 0.60055E+00_JPRB, 0.59286E+00_JPRB, 0.57040E+00_JPRB, 0.55003E+00_JPRB, 0.53116E+00_JPRB, 0.51674E+00_JPRB ,&
&  0.50746E+00_JPRB, 0.50148E+00_JPRB, 0.49072E+00_JPRB, 0.47946E+00_JPRB, 0.46587E+00_JPRB, 0.45194E+00_JPRB, 0.44263E+00_JPRB ,&
&  0.43487E+00_JPRB, 0.42927E+00_JPRB, 0.42208E+00_JPRB, 0.41455E+00_JPRB, 0.41047E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.40501E+00_JPRB, 0.40511E+00_JPRB, 0.40370E+00_JPRB, 0.40287E+00_JPRB, 0.40208E+00_JPRB, 0.40319E+00_JPRB, 0.40378E+00_JPRB ,&
&  0.40390E+00_JPRB, 0.40212E+00_JPRB, 0.40071E+00_JPRB, 0.39841E+00_JPRB, 0.39612E+00_JPRB, 0.39361E+00_JPRB, 0.39214E+00_JPRB ,&
&  0.39106E+00_JPRB, 0.38909E+00_JPRB, 0.38601E+00_JPRB, 0.38192E+00_JPRB, 0.37841E+00_JPRB, 0.37411E+00_JPRB, 0.36947E+00_JPRB ,&
&  0.36416E+00_JPRB, 0.35732E+00_JPRB, 0.34371E+00_JPRB, 0.33517E+00_JPRB, 0.32473E+00_JPRB, 0.32051E+00_JPRB, 0.31530E+00_JPRB ,&
&  0.32092E+00_JPRB, 0.29151E+00_JPRB, 0.28349E+00_JPRB, 0.26851E+00_JPRB, 0.25688E+00_JPRB, 0.24823E+00_JPRB, 0.23953E+00_JPRB ,&
&  0.23572E+00_JPRB, 0.23239E+00_JPRB, 0.23285E+00_JPRB, 0.23794E+00_JPRB, 0.24641E+00_JPRB, 0.26317E+00_JPRB, 0.28383E+00_JPRB ,&
&  0.31270E+00_JPRB, 0.37394E+00_JPRB, 0.39392E+00_JPRB, 0.41445E+00_JPRB, 0.43225E+00_JPRB, 0.45086E+00_JPRB, 0.45257E+00_JPRB ,&
&  0.46005E+00_JPRB, 0.46403E+00_JPRB, 0.47482E+00_JPRB, 0.48382E+00_JPRB, 0.50123E+00_JPRB, 0.50152E+00_JPRB, 0.49963E+00_JPRB ,&
&  0.48257E+00_JPRB, 0.46642E+00_JPRB, 0.45520E+00_JPRB, 0.45749E+00_JPRB, 0.47026E+00_JPRB, 0.48787E+00_JPRB, 0.50512E+00_JPRB ,&
&  0.52040E+00_JPRB, 0.53159E+00_JPRB, 0.53938E+00_JPRB, 0.55225E+00_JPRB, 0.56836E+00_JPRB, 0.58787E+00_JPRB, 0.59767E+00_JPRB ,&
&  0.60454E+00_JPRB, 0.60855E+00_JPRB, 0.60843E+00_JPRB, 0.60588E+00_JPRB, 0.59442E+00_JPRB, 0.58246E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.57655E+00_JPRB, 0.57059E+00_JPRB, 0.56062E+00_JPRB, 0.54726E+00_JPRB, 0.53732E+00_JPRB, 0.52356E+00_JPRB, 0.51021E+00_JPRB ,&
&  0.49863E+00_JPRB, 0.49060E+00_JPRB, 0.47989E+00_JPRB, 0.47034E+00_JPRB, 0.45660E+00_JPRB, 0.44480E+00_JPRB, 0.43610E+00_JPRB ,&
&  0.42775E+00_JPRB, 0.42064E+00_JPRB, 0.41282E+00_JPRB, 0.40774E+00_JPRB, 0.40600E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.40380E+00_JPRB, 0.40382E+00_JPRB, 0.40346E+00_JPRB, 0.40312E+00_JPRB, 0.40119E+00_JPRB, 0.40046E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.39884E+00_JPRB, 0.39791E+00_JPRB, 0.39762E+00_JPRB, 0.39807E+00_JPRB, 0.39827E+00_JPRB, 0.39854E+00_JPRB, 0.39775E+00_JPRB ,&
&  0.39785E+00_JPRB, 0.39703E+00_JPRB, 0.39574E+00_JPRB, 0.39253E+00_JPRB, 0.38933E+00_JPRB, 0.38474E+00_JPRB, 0.37734E+00_JPRB ,&
&  0.36819E+00_JPRB, 0.35986E+00_JPRB, 0.33720E+00_JPRB, 0.32777E+00_JPRB, 0.31451E+00_JPRB, 0.30705E+00_JPRB, 0.30402E+00_JPRB ,&
&  0.30180E+00_JPRB, 0.30023E+00_JPRB, 0.29573E+00_JPRB, 0.28918E+00_JPRB, 0.28140E+00_JPRB, 0.27074E+00_JPRB, 0.26374E+00_JPRB ,&
&  0.25745E+00_JPRB, 0.25563E+00_JPRB, 0.25800E+00_JPRB, 0.26482E+00_JPRB, 0.27680E+00_JPRB, 0.29290E+00_JPRB, 0.31208E+00_JPRB ,&
&  0.33778E+00_JPRB, 0.36292E+00_JPRB, 0.39433E+00_JPRB, 0.40654E+00_JPRB, 0.41736E+00_JPRB, 0.42859E+00_JPRB, 0.43838E+00_JPRB ,&
&  0.43874E+00_JPRB, 0.44013E+00_JPRB, 0.45053E+00_JPRB, 0.45137E+00_JPRB, 0.45185E+00_JPRB, 0.44655E+00_JPRB, 0.44282E+00_JPRB ,&
&  0.43675E+00_JPRB, 0.43631E+00_JPRB, 0.43763E+00_JPRB, 0.44403E+00_JPRB, 0.45472E+00_JPRB, 0.46820E+00_JPRB, 0.48140E+00_JPRB ,&
&  0.49328E+00_JPRB, 0.50656E+00_JPRB, 0.51594E+00_JPRB, 0.52471E+00_JPRB, 0.53328E+00_JPRB, 0.54478E+00_JPRB, 0.55361E+00_JPRB ,&
&  0.55992E+00_JPRB, 0.55893E+00_JPRB, 0.55388E+00_JPRB, 0.54193E+00_JPRB, 0.53712E+00_JPRB, 0.54361E+00_JPRB, 0.54147E+00_JPRB ,&
&  0.53919E+00_JPRB, 0.53380E+00_JPRB, 0.52927E+00_JPRB, 0.52340E+00_JPRB, 0.51721E+00_JPRB, 0.50806E+00_JPRB, 0.49672E+00_JPRB ,&
&  0.48649E+00_JPRB, 0.47870E+00_JPRB, 0.47071E+00_JPRB, 0.46342E+00_JPRB, 0.45361E+00_JPRB, 0.44335E+00_JPRB, 0.43431E+00_JPRB ,&
&  0.42611E+00_JPRB, 0.41879E+00_JPRB, 0.41213E+00_JPRB, 0.40747E+00_JPRB, 0.40519E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.40173E+00_JPRB, 0.40104E+00_JPRB, 0.39969E+00_JPRB, 0.39805E+00_JPRB, 0.39575E+00_JPRB, 0.39452E+00_JPRB, 0.39405E+00_JPRB ,&
&  0.39425E+00_JPRB, 0.39373E+00_JPRB, 0.39347E+00_JPRB, 0.39481E+00_JPRB, 0.39686E+00_JPRB, 0.39772E+00_JPRB, 0.39885E+00_JPRB ,&
&  0.40002E+00_JPRB, 0.40027E+00_JPRB, 0.40014E+00_JPRB, 0.39924E+00_JPRB, 0.39668E+00_JPRB, 0.39234E+00_JPRB, 0.38839E+00_JPRB ,&
&  0.38387E+00_JPRB, 0.37979E+00_JPRB, 0.37513E+00_JPRB, 0.37017E+00_JPRB, 0.36547E+00_JPRB, 0.33699E+00_JPRB, 0.32817E+00_JPRB ,&
&  0.32552E+00_JPRB, 0.32599E+00_JPRB, 0.33016E+00_JPRB, 0.33626E+00_JPRB, 0.34791E+00_JPRB, 0.34589E+00_JPRB, 0.34625E+00_JPRB ,&
&  0.33051E+00_JPRB, 0.33048E+00_JPRB, 0.33132E+00_JPRB, 0.33358E+00_JPRB, 0.33930E+00_JPRB, 0.34656E+00_JPRB, 0.35756E+00_JPRB ,&
&  0.37997E+00_JPRB, 0.38517E+00_JPRB, 0.38948E+00_JPRB, 0.39459E+00_JPRB, 0.39925E+00_JPRB, 0.40286E+00_JPRB, 0.40577E+00_JPRB ,&
&  0.40745E+00_JPRB, 0.40895E+00_JPRB, 0.41003E+00_JPRB, 0.41056E+00_JPRB, 0.41198E+00_JPRB, 0.41359E+00_JPRB, 0.41738E+00_JPRB ,&
&  0.42100E+00_JPRB, 0.42666E+00_JPRB, 0.43313E+00_JPRB, 0.44004E+00_JPRB, 0.44663E+00_JPRB, 0.45397E+00_JPRB, 0.46269E+00_JPRB ,&
&  0.47130E+00_JPRB, 0.48029E+00_JPRB, 0.48666E+00_JPRB, 0.49333E+00_JPRB, 0.49828E+00_JPRB, 0.50440E+00_JPRB, 0.50680E+00_JPRB ,&
&  0.50950E+00_JPRB, 0.50891E+00_JPRB, 0.50971E+00_JPRB, 0.50882E+00_JPRB, 0.50882E+00_JPRB, 0.50914E+00_JPRB, 0.50784E+00_JPRB ,&
&  0.50622E+00_JPRB, 0.50333E+00_JPRB, 0.50056E+00_JPRB, 0.49658E+00_JPRB, 0.49092E+00_JPRB, 0.48316E+00_JPRB, 0.47513E+00_JPRB ,&
&  0.46858E+00_JPRB, 0.46300E+00_JPRB, 0.45729E+00_JPRB, 0.45226E+00_JPRB, 0.44622E+00_JPRB, 0.43838E+00_JPRB, 0.42991E+00_JPRB ,&
&  0.42246E+00_JPRB, 0.41613E+00_JPRB, 0.41062E+00_JPRB, 0.40644E+00_JPRB, 0.40378E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.40150E+00_JPRB, 0.40134E+00_JPRB, 0.40135E+00_JPRB, 0.40101E+00_JPRB, 0.39985E+00_JPRB, 0.39924E+00_JPRB, 0.39920E+00_JPRB ,&
&  0.40007E+00_JPRB, 0.40110E+00_JPRB, 0.40102E+00_JPRB, 0.40026E+00_JPRB, 0.39960E+00_JPRB, 0.39910E+00_JPRB, 0.39881E+00_JPRB ,&
&  0.39870E+00_JPRB, 0.39806E+00_JPRB, 0.39638E+00_JPRB, 0.39371E+00_JPRB, 0.39173E+00_JPRB, 0.39020E+00_JPRB, 0.38969E+00_JPRB ,&
&  0.38937E+00_JPRB, 0.38858E+00_JPRB, 0.38723E+00_JPRB, 0.38619E+00_JPRB, 0.38449E+00_JPRB, 0.38289E+00_JPRB, 0.38104E+00_JPRB ,&
&  0.38000E+00_JPRB, 0.37945E+00_JPRB, 0.37909E+00_JPRB, 0.37863E+00_JPRB, 0.37915E+00_JPRB, 0.38002E+00_JPRB, 0.38082E+00_JPRB ,&
&  0.38143E+00_JPRB, 0.38230E+00_JPRB, 0.38397E+00_JPRB, 0.38544E+00_JPRB, 0.38673E+00_JPRB, 0.38778E+00_JPRB, 0.38794E+00_JPRB ,&
&  0.38713E+00_JPRB, 0.38658E+00_JPRB, 0.38730E+00_JPRB, 0.38788E+00_JPRB, 0.38862E+00_JPRB, 0.38950E+00_JPRB, 0.39148E+00_JPRB ,&
&  0.39293E+00_JPRB, 0.39456E+00_JPRB, 0.39678E+00_JPRB, 0.39991E+00_JPRB, 0.40289E+00_JPRB, 0.40649E+00_JPRB, 0.41054E+00_JPRB ,&
&  0.41448E+00_JPRB, 0.41867E+00_JPRB, 0.42290E+00_JPRB, 0.42717E+00_JPRB, 0.43117E+00_JPRB, 0.43471E+00_JPRB, 0.43879E+00_JPRB ,&
&  0.44328E+00_JPRB, 0.44858E+00_JPRB, 0.45223E+00_JPRB, 0.45617E+00_JPRB, 0.46043E+00_JPRB, 0.46637E+00_JPRB, 0.46906E+00_JPRB ,&
&  0.47146E+00_JPRB, 0.47369E+00_JPRB, 0.47606E+00_JPRB, 0.47704E+00_JPRB, 0.47799E+00_JPRB, 0.47875E+00_JPRB, 0.47875E+00_JPRB ,&
&  0.47776E+00_JPRB, 0.47603E+00_JPRB, 0.47361E+00_JPRB, 0.47001E+00_JPRB, 0.46549E+00_JPRB, 0.46057E+00_JPRB, 0.45503E+00_JPRB ,&
&  0.44992E+00_JPRB, 0.44380E+00_JPRB, 0.43760E+00_JPRB, 0.43236E+00_JPRB, 0.42845E+00_JPRB, 0.42357E+00_JPRB, 0.41833E+00_JPRB ,&
&  0.41326E+00_JPRB, 0.40925E+00_JPRB, 0.40664E+00_JPRB, 0.40435E+00_JPRB, 0.40282E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.40575E+00_JPRB, 0.40528E+00_JPRB, 0.40500E+00_JPRB, 0.40491E+00_JPRB, 0.40461E+00_JPRB, 0.40405E+00_JPRB, 0.40328E+00_JPRB ,&
&  0.40263E+00_JPRB, 0.40245E+00_JPRB, 0.40244E+00_JPRB, 0.40254E+00_JPRB, 0.40248E+00_JPRB, 0.40198E+00_JPRB, 0.40108E+00_JPRB ,&
&  0.39988E+00_JPRB, 0.39834E+00_JPRB, 0.39649E+00_JPRB, 0.39472E+00_JPRB, 0.39400E+00_JPRB, 0.39351E+00_JPRB, 0.39296E+00_JPRB ,&
&  0.39258E+00_JPRB, 0.39227E+00_JPRB, 0.39195E+00_JPRB, 0.39168E+00_JPRB, 0.39175E+00_JPRB, 0.39168E+00_JPRB, 0.39168E+00_JPRB ,&
&  0.39168E+00_JPRB, 0.39179E+00_JPRB, 0.39162E+00_JPRB, 0.39152E+00_JPRB, 0.39129E+00_JPRB, 0.39160E+00_JPRB, 0.39171E+00_JPRB ,&
&  0.39202E+00_JPRB, 0.39206E+00_JPRB, 0.39198E+00_JPRB, 0.39175E+00_JPRB, 0.39159E+00_JPRB, 0.39109E+00_JPRB, 0.39059E+00_JPRB ,&
&  0.39031E+00_JPRB, 0.39035E+00_JPRB, 0.39093E+00_JPRB, 0.39198E+00_JPRB, 0.39307E+00_JPRB, 0.39427E+00_JPRB, 0.39562E+00_JPRB ,&
&  0.39691E+00_JPRB, 0.39835E+00_JPRB, 0.39983E+00_JPRB, 0.40139E+00_JPRB, 0.40296E+00_JPRB, 0.40443E+00_JPRB, 0.40602E+00_JPRB ,&
&  0.40759E+00_JPRB, 0.40937E+00_JPRB, 0.41153E+00_JPRB, 0.41374E+00_JPRB, 0.41581E+00_JPRB, 0.41772E+00_JPRB, 0.41984E+00_JPRB ,&
&  0.42213E+00_JPRB, 0.42439E+00_JPRB, 0.42617E+00_JPRB, 0.42809E+00_JPRB, 0.43006E+00_JPRB, 0.43197E+00_JPRB, 0.43337E+00_JPRB ,&
&  0.43482E+00_JPRB, 0.43644E+00_JPRB, 0.43815E+00_JPRB, 0.43859E+00_JPRB, 0.43889E+00_JPRB, 0.43899E+00_JPRB, 0.43869E+00_JPRB ,&
&  0.43770E+00_JPRB, 0.43657E+00_JPRB, 0.43537E+00_JPRB, 0.43389E+00_JPRB, 0.43165E+00_JPRB, 0.42930E+00_JPRB, 0.42672E+00_JPRB ,&
&  0.42416E+00_JPRB, 0.42117E+00_JPRB, 0.41822E+00_JPRB, 0.41580E+00_JPRB, 0.41402E+00_JPRB, 0.41230E+00_JPRB, 0.41055E+00_JPRB ,&
&  0.40879E+00_JPRB, 0.40772E+00_JPRB, 0.40729E+00_JPRB, 0.40690E+00_JPRB, 0.40645E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB, 0.40140E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB, 0.22440E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.23578E+00_JPRB, 0.23607E+00_JPRB, 0.23621E+00_JPRB, 0.23628E+00_JPRB, 0.23478E+00_JPRB, 0.23530E+00_JPRB, 0.23547E+00_JPRB ,&
&  0.23580E+00_JPRB, 0.23610E+00_JPRB, 0.23667E+00_JPRB, 0.23734E+00_JPRB, 0.23798E+00_JPRB, 0.23856E+00_JPRB, 0.23906E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.23974E+00_JPRB, 0.23853E+00_JPRB, 0.23784E+00_JPRB, 0.23778E+00_JPRB, 0.23791E+00_JPRB, 0.23792E+00_JPRB ,&
&  0.23791E+00_JPRB, 0.23783E+00_JPRB, 0.23743E+00_JPRB, 0.23713E+00_JPRB, 0.23700E+00_JPRB, 0.23682E+00_JPRB, 0.23654E+00_JPRB ,&
&  0.23616E+00_JPRB, 0.23587E+00_JPRB, 0.23552E+00_JPRB, 0.23512E+00_JPRB, 0.23497E+00_JPRB, 0.23473E+00_JPRB, 0.23412E+00_JPRB ,&
&  0.23340E+00_JPRB, 0.23238E+00_JPRB, 0.23144E+00_JPRB, 0.23060E+00_JPRB, 0.22992E+00_JPRB, 0.22956E+00_JPRB, 0.22920E+00_JPRB ,&
&  0.22872E+00_JPRB, 0.22807E+00_JPRB, 0.22744E+00_JPRB, 0.22671E+00_JPRB, 0.22607E+00_JPRB, 0.22550E+00_JPRB, 0.22482E+00_JPRB ,&
&  0.22408E+00_JPRB, 0.22333E+00_JPRB, 0.22255E+00_JPRB, 0.22170E+00_JPRB, 0.22089E+00_JPRB, 0.22007E+00_JPRB, 0.21918E+00_JPRB ,&
&  0.21830E+00_JPRB, 0.21790E+00_JPRB, 0.21745E+00_JPRB, 0.21706E+00_JPRB, 0.21677E+00_JPRB, 0.21655E+00_JPRB, 0.21632E+00_JPRB ,&
&  0.21615E+00_JPRB, 0.21596E+00_JPRB, 0.21586E+00_JPRB, 0.21577E+00_JPRB, 0.21577E+00_JPRB, 0.21583E+00_JPRB, 0.21598E+00_JPRB ,&
&  0.21617E+00_JPRB, 0.21641E+00_JPRB, 0.21674E+00_JPRB, 0.21727E+00_JPRB, 0.21779E+00_JPRB, 0.21839E+00_JPRB, 0.21899E+00_JPRB ,&
&  0.21965E+00_JPRB, 0.22030E+00_JPRB, 0.22096E+00_JPRB, 0.22158E+00_JPRB, 0.22242E+00_JPRB, 0.22322E+00_JPRB, 0.22411E+00_JPRB ,&
&  0.22498E+00_JPRB, 0.22593E+00_JPRB, 0.22687E+00_JPRB, 0.22787E+00_JPRB, 0.22901E+00_JPRB, 0.22936E+00_JPRB, 0.22972E+00_JPRB ,&
&  0.23018E+00_JPRB, 0.23084E+00_JPRB, 0.23144E+00_JPRB, 0.23222E+00_JPRB, 0.23353E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.26246E+00_JPRB, 0.26894E+00_JPRB, 0.27341E+00_JPRB, 0.27831E+00_JPRB, 0.27799E+00_JPRB, 0.31594E+00_JPRB, 0.31494E+00_JPRB ,&
&  0.31361E+00_JPRB, 0.29893E+00_JPRB, 0.29627E+00_JPRB, 0.29139E+00_JPRB, 0.28576E+00_JPRB, 0.27964E+00_JPRB, 0.27435E+00_JPRB ,&
&  0.26956E+00_JPRB, 0.26425E+00_JPRB, 0.25989E+00_JPRB, 0.25588E+00_JPRB, 0.25259E+00_JPRB, 0.24966E+00_JPRB, 0.24760E+00_JPRB ,&
&  0.24706E+00_JPRB, 0.24848E+00_JPRB, 0.25149E+00_JPRB, 0.25330E+00_JPRB, 0.25416E+00_JPRB, 0.25489E+00_JPRB, 0.25560E+00_JPRB ,&
&  0.25571E+00_JPRB, 0.25580E+00_JPRB, 0.25480E+00_JPRB, 0.25289E+00_JPRB, 0.25101E+00_JPRB, 0.24942E+00_JPRB, 0.24816E+00_JPRB ,&
&  0.24694E+00_JPRB, 0.24567E+00_JPRB, 0.24432E+00_JPRB, 0.24344E+00_JPRB, 0.24228E+00_JPRB, 0.24075E+00_JPRB, 0.23871E+00_JPRB ,&
&  0.23667E+00_JPRB, 0.23463E+00_JPRB, 0.23225E+00_JPRB, 0.23034E+00_JPRB, 0.22850E+00_JPRB, 0.22662E+00_JPRB, 0.22477E+00_JPRB ,&
&  0.22371E+00_JPRB, 0.22269E+00_JPRB, 0.22141E+00_JPRB, 0.21979E+00_JPRB, 0.21854E+00_JPRB, 0.21722E+00_JPRB, 0.21606E+00_JPRB ,&
&  0.21529E+00_JPRB, 0.21450E+00_JPRB, 0.21369E+00_JPRB, 0.21285E+00_JPRB, 0.21223E+00_JPRB, 0.21152E+00_JPRB, 0.21088E+00_JPRB ,&
&  0.21032E+00_JPRB, 0.20970E+00_JPRB, 0.20955E+00_JPRB, 0.20948E+00_JPRB, 0.20955E+00_JPRB, 0.20962E+00_JPRB, 0.20992E+00_JPRB ,&
&  0.21037E+00_JPRB, 0.21102E+00_JPRB, 0.21182E+00_JPRB, 0.21258E+00_JPRB, 0.21332E+00_JPRB, 0.21409E+00_JPRB, 0.21521E+00_JPRB ,&
&  0.21644E+00_JPRB, 0.21776E+00_JPRB, 0.21913E+00_JPRB, 0.22021E+00_JPRB, 0.22237E+00_JPRB, 0.22443E+00_JPRB, 0.22662E+00_JPRB ,&
&  0.22913E+00_JPRB, 0.23209E+00_JPRB, 0.23525E+00_JPRB, 0.23859E+00_JPRB, 0.24295E+00_JPRB, 0.24584E+00_JPRB, 0.24797E+00_JPRB ,&
&  0.25032E+00_JPRB, 0.25420E+00_JPRB, 0.25593E+00_JPRB, 0.25739E+00_JPRB, 0.25956E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.33367E+00_JPRB, 0.33822E+00_JPRB, 0.34214E+00_JPRB, 0.34559E+00_JPRB, 0.34860E+00_JPRB, 0.34929E+00_JPRB, 0.34831E+00_JPRB ,&
&  0.33929E+00_JPRB, 0.33257E+00_JPRB, 0.32443E+00_JPRB, 0.31725E+00_JPRB, 0.31063E+00_JPRB, 0.30303E+00_JPRB, 0.29632E+00_JPRB ,&
&  0.28948E+00_JPRB, 0.28265E+00_JPRB, 0.27613E+00_JPRB, 0.26852E+00_JPRB, 0.26057E+00_JPRB, 0.25298E+00_JPRB, 0.24825E+00_JPRB ,&
&  0.24554E+00_JPRB, 0.24538E+00_JPRB, 0.24823E+00_JPRB, 0.25206E+00_JPRB, 0.25702E+00_JPRB, 0.26340E+00_JPRB, 0.27022E+00_JPRB ,&
&  0.27456E+00_JPRB, 0.27697E+00_JPRB, 0.27661E+00_JPRB, 0.27529E+00_JPRB, 0.27337E+00_JPRB, 0.27167E+00_JPRB, 0.26904E+00_JPRB ,&
&  0.26568E+00_JPRB, 0.26318E+00_JPRB, 0.26212E+00_JPRB, 0.26274E+00_JPRB, 0.26221E+00_JPRB, 0.26016E+00_JPRB, 0.25625E+00_JPRB ,&
&  0.25198E+00_JPRB, 0.24772E+00_JPRB, 0.24341E+00_JPRB, 0.23943E+00_JPRB, 0.23615E+00_JPRB, 0.23276E+00_JPRB, 0.22914E+00_JPRB ,&
&  0.22682E+00_JPRB, 0.22435E+00_JPRB, 0.22164E+00_JPRB, 0.21905E+00_JPRB, 0.21691E+00_JPRB, 0.21525E+00_JPRB, 0.21387E+00_JPRB ,&
&  0.21278E+00_JPRB, 0.21194E+00_JPRB, 0.21125E+00_JPRB, 0.21074E+00_JPRB, 0.21025E+00_JPRB, 0.20951E+00_JPRB, 0.20878E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20777E+00_JPRB, 0.20732E+00_JPRB, 0.20734E+00_JPRB, 0.20757E+00_JPRB, 0.20772E+00_JPRB, 0.20822E+00_JPRB ,&
&  0.20930E+00_JPRB, 0.21044E+00_JPRB, 0.21110E+00_JPRB, 0.21217E+00_JPRB, 0.21322E+00_JPRB, 0.21459E+00_JPRB, 0.21633E+00_JPRB ,&
&  0.21747E+00_JPRB, 0.21874E+00_JPRB, 0.22021E+00_JPRB, 0.22180E+00_JPRB, 0.22434E+00_JPRB, 0.22712E+00_JPRB, 0.23020E+00_JPRB ,&
&  0.23416E+00_JPRB, 0.23806E+00_JPRB, 0.24248E+00_JPRB, 0.24771E+00_JPRB, 0.25678E+00_JPRB, 0.26340E+00_JPRB, 0.27208E+00_JPRB ,&
&  0.28091E+00_JPRB, 0.28673E+00_JPRB, 0.32027E+00_JPRB, 0.32270E+00_JPRB, 0.32782E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.34249E+00_JPRB, 0.34904E+00_JPRB, 0.35592E+00_JPRB, 0.36096E+00_JPRB, 0.36465E+00_JPRB, 0.36765E+00_JPRB, 0.36651E+00_JPRB ,&
&  0.36038E+00_JPRB, 0.34504E+00_JPRB, 0.33356E+00_JPRB, 0.32251E+00_JPRB, 0.31311E+00_JPRB, 0.30423E+00_JPRB, 0.29592E+00_JPRB ,&
&  0.28803E+00_JPRB, 0.28151E+00_JPRB, 0.27621E+00_JPRB, 0.27161E+00_JPRB, 0.26531E+00_JPRB, 0.25980E+00_JPRB, 0.25538E+00_JPRB ,&
&  0.25173E+00_JPRB, 0.25159E+00_JPRB, 0.25381E+00_JPRB, 0.25815E+00_JPRB, 0.26617E+00_JPRB, 0.27448E+00_JPRB, 0.28379E+00_JPRB ,&
&  0.29814E+00_JPRB, 0.30246E+00_JPRB, 0.30148E+00_JPRB, 0.30081E+00_JPRB, 0.29591E+00_JPRB, 0.28990E+00_JPRB, 0.28056E+00_JPRB ,&
&  0.28081E+00_JPRB, 0.27885E+00_JPRB, 0.28564E+00_JPRB, 0.28590E+00_JPRB, 0.28467E+00_JPRB, 0.28429E+00_JPRB, 0.27673E+00_JPRB ,&
&  0.27090E+00_JPRB, 0.26194E+00_JPRB, 0.25506E+00_JPRB, 0.24818E+00_JPRB, 0.24162E+00_JPRB, 0.23769E+00_JPRB, 0.23260E+00_JPRB ,&
&  0.22878E+00_JPRB, 0.22583E+00_JPRB, 0.22207E+00_JPRB, 0.21961E+00_JPRB, 0.21756E+00_JPRB, 0.21547E+00_JPRB, 0.21440E+00_JPRB ,&
&  0.21360E+00_JPRB, 0.21332E+00_JPRB, 0.21341E+00_JPRB, 0.21361E+00_JPRB, 0.21408E+00_JPRB, 0.21496E+00_JPRB, 0.21562E+00_JPRB ,&
&  0.21635E+00_JPRB, 0.21657E+00_JPRB, 0.21618E+00_JPRB, 0.21572E+00_JPRB, 0.21464E+00_JPRB, 0.21389E+00_JPRB, 0.21296E+00_JPRB ,&
&  0.21242E+00_JPRB, 0.21273E+00_JPRB, 0.21304E+00_JPRB, 0.21372E+00_JPRB, 0.21444E+00_JPRB, 0.21594E+00_JPRB, 0.21736E+00_JPRB ,&
&  0.21887E+00_JPRB, 0.22069E+00_JPRB, 0.22221E+00_JPRB, 0.22413E+00_JPRB, 0.22643E+00_JPRB, 0.22805E+00_JPRB, 0.23076E+00_JPRB ,&
&  0.23346E+00_JPRB, 0.23681E+00_JPRB, 0.24236E+00_JPRB, 0.24860E+00_JPRB, 0.25700E+00_JPRB, 0.26772E+00_JPRB, 0.28142E+00_JPRB ,&
&  0.29903E+00_JPRB, 0.32420E+00_JPRB, 0.32623E+00_JPRB, 0.33263E+00_JPRB, 0.33742E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.34749E+00_JPRB, 0.35004E+00_JPRB, 0.35414E+00_JPRB, 0.35948E+00_JPRB, 0.36615E+00_JPRB, 0.37207E+00_JPRB, 0.37336E+00_JPRB ,&
&  0.36417E+00_JPRB, 0.35122E+00_JPRB, 0.33314E+00_JPRB, 0.31865E+00_JPRB, 0.30481E+00_JPRB, 0.29437E+00_JPRB, 0.28583E+00_JPRB ,&
&  0.28056E+00_JPRB, 0.28002E+00_JPRB, 0.28058E+00_JPRB, 0.28182E+00_JPRB, 0.28097E+00_JPRB, 0.28212E+00_JPRB, 0.28262E+00_JPRB ,&
&  0.28157E+00_JPRB, 0.28150E+00_JPRB, 0.28142E+00_JPRB, 0.28498E+00_JPRB, 0.29181E+00_JPRB, 0.29821E+00_JPRB, 0.30456E+00_JPRB ,&
&  0.31194E+00_JPRB, 0.31433E+00_JPRB, 0.31867E+00_JPRB, 0.31763E+00_JPRB, 0.31418E+00_JPRB, 0.30970E+00_JPRB, 0.30341E+00_JPRB ,&
&  0.29867E+00_JPRB, 0.29455E+00_JPRB, 0.29303E+00_JPRB, 0.29347E+00_JPRB, 0.29361E+00_JPRB, 0.29479E+00_JPRB, 0.28833E+00_JPRB ,&
&  0.28099E+00_JPRB, 0.27077E+00_JPRB, 0.25939E+00_JPRB, 0.25107E+00_JPRB, 0.24335E+00_JPRB, 0.23779E+00_JPRB, 0.23283E+00_JPRB ,&
&  0.22821E+00_JPRB, 0.22511E+00_JPRB, 0.22191E+00_JPRB, 0.21959E+00_JPRB, 0.21889E+00_JPRB, 0.21731E+00_JPRB, 0.21646E+00_JPRB ,&
&  0.21614E+00_JPRB, 0.21660E+00_JPRB, 0.21811E+00_JPRB, 0.21951E+00_JPRB, 0.22138E+00_JPRB, 0.22354E+00_JPRB, 0.22540E+00_JPRB ,&
&  0.22786E+00_JPRB, 0.22981E+00_JPRB, 0.23003E+00_JPRB, 0.22941E+00_JPRB, 0.22737E+00_JPRB, 0.22472E+00_JPRB, 0.22231E+00_JPRB ,&
&  0.21995E+00_JPRB, 0.21827E+00_JPRB, 0.21759E+00_JPRB, 0.21679E+00_JPRB, 0.21720E+00_JPRB, 0.21831E+00_JPRB, 0.21920E+00_JPRB ,&
&  0.22085E+00_JPRB, 0.22219E+00_JPRB, 0.22372E+00_JPRB, 0.22622E+00_JPRB, 0.22864E+00_JPRB, 0.23078E+00_JPRB, 0.23302E+00_JPRB ,&
&  0.23328E+00_JPRB, 0.23497E+00_JPRB, 0.23721E+00_JPRB, 0.24103E+00_JPRB, 0.25202E+00_JPRB, 0.26016E+00_JPRB, 0.27900E+00_JPRB ,&
&  0.29625E+00_JPRB, 0.33244E+00_JPRB, 0.33362E+00_JPRB, 0.33826E+00_JPRB, 0.34400E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.35051E+00_JPRB, 0.35492E+00_JPRB, 0.35590E+00_JPRB, 0.35919E+00_JPRB, 0.36360E+00_JPRB, 0.36880E+00_JPRB, 0.37305E+00_JPRB ,&
&  0.37801E+00_JPRB, 0.37774E+00_JPRB, 0.37259E+00_JPRB, 0.36577E+00_JPRB, 0.35997E+00_JPRB, 0.34313E+00_JPRB, 0.33407E+00_JPRB ,&
&  0.32675E+00_JPRB, 0.32615E+00_JPRB, 0.32674E+00_JPRB, 0.32833E+00_JPRB, 0.32947E+00_JPRB, 0.33445E+00_JPRB, 0.33602E+00_JPRB ,&
&  0.33367E+00_JPRB, 0.32868E+00_JPRB, 0.32367E+00_JPRB, 0.32286E+00_JPRB, 0.32546E+00_JPRB, 0.32753E+00_JPRB, 0.32487E+00_JPRB ,&
&  0.32129E+00_JPRB, 0.32104E+00_JPRB, 0.32172E+00_JPRB, 0.32320E+00_JPRB, 0.33297E+00_JPRB, 0.32879E+00_JPRB, 0.32444E+00_JPRB ,&
&  0.31732E+00_JPRB, 0.31110E+00_JPRB, 0.30674E+00_JPRB, 0.30698E+00_JPRB, 0.30965E+00_JPRB, 0.30981E+00_JPRB, 0.31231E+00_JPRB ,&
&  0.30024E+00_JPRB, 0.29025E+00_JPRB, 0.27560E+00_JPRB, 0.26416E+00_JPRB, 0.25339E+00_JPRB, 0.24602E+00_JPRB, 0.23906E+00_JPRB ,&
&  0.23312E+00_JPRB, 0.22824E+00_JPRB, 0.22361E+00_JPRB, 0.22205E+00_JPRB, 0.22290E+00_JPRB, 0.22311E+00_JPRB, 0.22289E+00_JPRB ,&
&  0.22354E+00_JPRB, 0.22453E+00_JPRB, 0.22534E+00_JPRB, 0.22632E+00_JPRB, 0.22780E+00_JPRB, 0.23028E+00_JPRB, 0.23356E+00_JPRB ,&
&  0.23876E+00_JPRB, 0.24294E+00_JPRB, 0.24522E+00_JPRB, 0.24618E+00_JPRB, 0.24307E+00_JPRB, 0.23791E+00_JPRB, 0.23255E+00_JPRB ,&
&  0.22735E+00_JPRB, 0.22490E+00_JPRB, 0.22277E+00_JPRB, 0.22177E+00_JPRB, 0.22162E+00_JPRB, 0.22249E+00_JPRB, 0.22421E+00_JPRB ,&
&  0.22590E+00_JPRB, 0.22798E+00_JPRB, 0.22934E+00_JPRB, 0.23101E+00_JPRB, 0.23306E+00_JPRB, 0.23486E+00_JPRB, 0.23674E+00_JPRB ,&
&  0.23884E+00_JPRB, 0.24048E+00_JPRB, 0.24328E+00_JPRB, 0.24671E+00_JPRB, 0.25316E+00_JPRB, 0.26102E+00_JPRB, 0.27514E+00_JPRB ,&
&  0.29204E+00_JPRB, 0.32498E+00_JPRB, 0.33643E+00_JPRB, 0.34195E+00_JPRB, 0.34842E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.34798E+00_JPRB, 0.35432E+00_JPRB, 0.35940E+00_JPRB, 0.36943E+00_JPRB, 0.37211E+00_JPRB, 0.37464E+00_JPRB, 0.37662E+00_JPRB ,&
&  0.38242E+00_JPRB, 0.38591E+00_JPRB, 0.38361E+00_JPRB, 0.38291E+00_JPRB, 0.38197E+00_JPRB, 0.38094E+00_JPRB, 0.38172E+00_JPRB ,&
&  0.38162E+00_JPRB, 0.38360E+00_JPRB, 0.38137E+00_JPRB, 0.37855E+00_JPRB, 0.38039E+00_JPRB, 0.38508E+00_JPRB, 0.38827E+00_JPRB ,&
&  0.38332E+00_JPRB, 0.38222E+00_JPRB, 0.39088E+00_JPRB, 0.39328E+00_JPRB, 0.39480E+00_JPRB, 0.39703E+00_JPRB, 0.39699E+00_JPRB ,&
&  0.38363E+00_JPRB, 0.35892E+00_JPRB, 0.33138E+00_JPRB, 0.33202E+00_JPRB, 0.34654E+00_JPRB, 0.34197E+00_JPRB, 0.34049E+00_JPRB ,&
&  0.33761E+00_JPRB, 0.33326E+00_JPRB, 0.32907E+00_JPRB, 0.32726E+00_JPRB, 0.32451E+00_JPRB, 0.32012E+00_JPRB, 0.31851E+00_JPRB ,&
&  0.31764E+00_JPRB, 0.32567E+00_JPRB, 0.32180E+00_JPRB, 0.30119E+00_JPRB, 0.28499E+00_JPRB, 0.27625E+00_JPRB, 0.26515E+00_JPRB ,&
&  0.25240E+00_JPRB, 0.24459E+00_JPRB, 0.24318E+00_JPRB, 0.24173E+00_JPRB, 0.24353E+00_JPRB, 0.24461E+00_JPRB, 0.24576E+00_JPRB ,&
&  0.24679E+00_JPRB, 0.24676E+00_JPRB, 0.24236E+00_JPRB, 0.23662E+00_JPRB, 0.23345E+00_JPRB, 0.23338E+00_JPRB, 0.23577E+00_JPRB ,&
&  0.24118E+00_JPRB, 0.24718E+00_JPRB, 0.26099E+00_JPRB, 0.26869E+00_JPRB, 0.26053E+00_JPRB, 0.25444E+00_JPRB, 0.24763E+00_JPRB ,&
&  0.23788E+00_JPRB, 0.23402E+00_JPRB, 0.23085E+00_JPRB, 0.22908E+00_JPRB, 0.22985E+00_JPRB, 0.23291E+00_JPRB, 0.23462E+00_JPRB ,&
&  0.23693E+00_JPRB, 0.23932E+00_JPRB, 0.24028E+00_JPRB, 0.24108E+00_JPRB, 0.24174E+00_JPRB, 0.24344E+00_JPRB, 0.24519E+00_JPRB ,&
&  0.24670E+00_JPRB, 0.24939E+00_JPRB, 0.25270E+00_JPRB, 0.25715E+00_JPRB, 0.26108E+00_JPRB, 0.26345E+00_JPRB, 0.27000E+00_JPRB ,&
&  0.27790E+00_JPRB, 0.29433E+00_JPRB, 0.32149E+00_JPRB, 0.34653E+00_JPRB, 0.34364E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.35532E+00_JPRB, 0.36019E+00_JPRB, 0.36683E+00_JPRB, 0.37424E+00_JPRB, 0.37948E+00_JPRB, 0.38170E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38714E+00_JPRB, 0.38846E+00_JPRB, 0.38949E+00_JPRB, 0.39227E+00_JPRB, 0.39780E+00_JPRB, 0.40058E+00_JPRB, 0.40513E+00_JPRB ,&
&  0.40685E+00_JPRB, 0.40478E+00_JPRB, 0.39815E+00_JPRB, 0.39611E+00_JPRB, 0.39950E+00_JPRB, 0.40417E+00_JPRB, 0.40944E+00_JPRB ,&
&  0.41500E+00_JPRB, 0.42013E+00_JPRB, 0.42265E+00_JPRB, 0.42351E+00_JPRB, 0.42386E+00_JPRB, 0.42392E+00_JPRB, 0.42147E+00_JPRB ,&
&  0.41710E+00_JPRB, 0.37831E+00_JPRB, 0.35160E+00_JPRB, 0.35370E+00_JPRB, 0.36561E+00_JPRB, 0.36067E+00_JPRB, 0.35917E+00_JPRB ,&
&  0.35760E+00_JPRB, 0.35430E+00_JPRB, 0.34939E+00_JPRB, 0.34549E+00_JPRB, 0.34053E+00_JPRB, 0.33416E+00_JPRB, 0.32895E+00_JPRB ,&
&  0.32795E+00_JPRB, 0.33255E+00_JPRB, 0.33426E+00_JPRB, 0.33702E+00_JPRB, 0.33649E+00_JPRB, 0.33905E+00_JPRB, 0.33840E+00_JPRB ,&
&  0.33360E+00_JPRB, 0.32804E+00_JPRB, 0.32641E+00_JPRB, 0.32623E+00_JPRB, 0.32727E+00_JPRB, 0.32985E+00_JPRB, 0.33009E+00_JPRB ,&
&  0.31687E+00_JPRB, 0.30547E+00_JPRB, 0.28720E+00_JPRB, 0.26949E+00_JPRB, 0.25645E+00_JPRB, 0.25044E+00_JPRB, 0.24602E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.25018E+00_JPRB, 0.26575E+00_JPRB, 0.28506E+00_JPRB, 0.30018E+00_JPRB, 0.28206E+00_JPRB, 0.26608E+00_JPRB ,&
&  0.25618E+00_JPRB, 0.25100E+00_JPRB, 0.24673E+00_JPRB, 0.24509E+00_JPRB, 0.24530E+00_JPRB, 0.24729E+00_JPRB, 0.25025E+00_JPRB ,&
&  0.25457E+00_JPRB, 0.25873E+00_JPRB, 0.26150E+00_JPRB, 0.26282E+00_JPRB, 0.26349E+00_JPRB, 0.26414E+00_JPRB, 0.26350E+00_JPRB ,&
&  0.26310E+00_JPRB, 0.26596E+00_JPRB, 0.27224E+00_JPRB, 0.28062E+00_JPRB, 0.28861E+00_JPRB, 0.29738E+00_JPRB, 0.30978E+00_JPRB ,&
&  0.32371E+00_JPRB, 0.33579E+00_JPRB, 0.34003E+00_JPRB, 0.34572E+00_JPRB, 0.35076E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.37541E+00_JPRB, 0.37965E+00_JPRB, 0.38354E+00_JPRB, 0.38785E+00_JPRB, 0.39399E+00_JPRB, 0.39699E+00_JPRB, 0.39786E+00_JPRB ,&
&  0.39918E+00_JPRB, 0.39986E+00_JPRB, 0.40151E+00_JPRB, 0.40586E+00_JPRB, 0.41166E+00_JPRB, 0.41567E+00_JPRB, 0.42114E+00_JPRB ,&
&  0.42123E+00_JPRB, 0.41291E+00_JPRB, 0.40524E+00_JPRB, 0.40493E+00_JPRB, 0.41030E+00_JPRB, 0.41594E+00_JPRB, 0.42315E+00_JPRB ,&
&  0.43033E+00_JPRB, 0.43565E+00_JPRB, 0.43870E+00_JPRB, 0.44024E+00_JPRB, 0.44230E+00_JPRB, 0.44215E+00_JPRB, 0.43934E+00_JPRB ,&
&  0.43556E+00_JPRB, 0.42717E+00_JPRB, 0.40768E+00_JPRB, 0.40473E+00_JPRB, 0.39607E+00_JPRB, 0.38847E+00_JPRB, 0.38316E+00_JPRB ,&
&  0.37881E+00_JPRB, 0.37538E+00_JPRB, 0.37165E+00_JPRB, 0.36863E+00_JPRB, 0.36520E+00_JPRB, 0.35878E+00_JPRB, 0.35576E+00_JPRB ,&
&  0.35306E+00_JPRB, 0.35078E+00_JPRB, 0.34868E+00_JPRB, 0.34936E+00_JPRB, 0.35023E+00_JPRB, 0.35349E+00_JPRB, 0.35523E+00_JPRB ,&
&  0.35247E+00_JPRB, 0.34911E+00_JPRB, 0.34649E+00_JPRB, 0.34815E+00_JPRB, 0.35035E+00_JPRB, 0.35187E+00_JPRB, 0.35246E+00_JPRB ,&
&  0.35025E+00_JPRB, 0.34608E+00_JPRB, 0.34180E+00_JPRB, 0.33672E+00_JPRB, 0.33222E+00_JPRB, 0.30652E+00_JPRB, 0.29503E+00_JPRB ,&
&  0.29121E+00_JPRB, 0.29990E+00_JPRB, 0.31259E+00_JPRB, 0.31101E+00_JPRB, 0.31244E+00_JPRB, 0.31231E+00_JPRB, 0.30978E+00_JPRB ,&
&  0.30125E+00_JPRB, 0.28751E+00_JPRB, 0.27791E+00_JPRB, 0.27357E+00_JPRB, 0.27039E+00_JPRB, 0.27110E+00_JPRB, 0.27659E+00_JPRB ,&
&  0.28610E+00_JPRB, 0.29760E+00_JPRB, 0.30639E+00_JPRB, 0.31044E+00_JPRB, 0.30948E+00_JPRB, 0.30638E+00_JPRB, 0.30223E+00_JPRB ,&
&  0.29744E+00_JPRB, 0.30253E+00_JPRB, 0.31606E+00_JPRB, 0.33604E+00_JPRB, 0.33854E+00_JPRB, 0.34313E+00_JPRB, 0.34623E+00_JPRB ,&
&  0.35120E+00_JPRB, 0.35743E+00_JPRB, 0.36290E+00_JPRB, 0.36884E+00_JPRB, 0.37255E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.40104E+00_JPRB, 0.40339E+00_JPRB, 0.40572E+00_JPRB, 0.40728E+00_JPRB, 0.40992E+00_JPRB, 0.41465E+00_JPRB, 0.41656E+00_JPRB ,&
&  0.41480E+00_JPRB, 0.41282E+00_JPRB, 0.41266E+00_JPRB, 0.41591E+00_JPRB, 0.42220E+00_JPRB, 0.42890E+00_JPRB, 0.43655E+00_JPRB ,&
&  0.43281E+00_JPRB, 0.42083E+00_JPRB, 0.41647E+00_JPRB, 0.41845E+00_JPRB, 0.42324E+00_JPRB, 0.42859E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.44164E+00_JPRB, 0.44588E+00_JPRB, 0.45029E+00_JPRB, 0.45418E+00_JPRB, 0.45674E+00_JPRB, 0.45516E+00_JPRB, 0.45098E+00_JPRB ,&
&  0.44604E+00_JPRB, 0.44114E+00_JPRB, 0.43691E+00_JPRB, 0.42091E+00_JPRB, 0.41826E+00_JPRB, 0.40933E+00_JPRB, 0.40177E+00_JPRB ,&
&  0.39645E+00_JPRB, 0.39207E+00_JPRB, 0.38936E+00_JPRB, 0.38653E+00_JPRB, 0.38264E+00_JPRB, 0.37598E+00_JPRB, 0.37451E+00_JPRB ,&
&  0.37418E+00_JPRB, 0.37583E+00_JPRB, 0.37188E+00_JPRB, 0.37004E+00_JPRB, 0.36797E+00_JPRB, 0.36642E+00_JPRB, 0.37112E+00_JPRB ,&
&  0.37660E+00_JPRB, 0.37439E+00_JPRB, 0.37062E+00_JPRB, 0.37025E+00_JPRB, 0.37441E+00_JPRB, 0.37600E+00_JPRB, 0.37705E+00_JPRB ,&
&  0.37899E+00_JPRB, 0.37980E+00_JPRB, 0.37559E+00_JPRB, 0.37076E+00_JPRB, 0.36213E+00_JPRB, 0.35171E+00_JPRB, 0.34911E+00_JPRB ,&
&  0.34623E+00_JPRB, 0.34028E+00_JPRB, 0.33643E+00_JPRB, 0.33516E+00_JPRB, 0.33465E+00_JPRB, 0.33522E+00_JPRB, 0.33506E+00_JPRB ,&
&  0.33471E+00_JPRB, 0.33106E+00_JPRB, 0.32572E+00_JPRB, 0.32096E+00_JPRB, 0.31455E+00_JPRB, 0.31659E+00_JPRB, 0.32561E+00_JPRB ,&
&  0.33706E+00_JPRB, 0.34393E+00_JPRB, 0.34844E+00_JPRB, 0.35187E+00_JPRB, 0.35495E+00_JPRB, 0.35646E+00_JPRB, 0.35880E+00_JPRB ,&
&  0.36194E+00_JPRB, 0.36336E+00_JPRB, 0.36180E+00_JPRB, 0.36120E+00_JPRB, 0.36361E+00_JPRB, 0.36817E+00_JPRB, 0.37391E+00_JPRB ,&
&  0.37975E+00_JPRB, 0.38474E+00_JPRB, 0.38894E+00_JPRB, 0.39428E+00_JPRB, 0.39772E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.43328E+00_JPRB, 0.43337E+00_JPRB, 0.43395E+00_JPRB, 0.43536E+00_JPRB, 0.43631E+00_JPRB, 0.43547E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.43220E+00_JPRB, 0.42762E+00_JPRB, 0.42469E+00_JPRB, 0.42591E+00_JPRB, 0.43467E+00_JPRB, 0.44616E+00_JPRB, 0.45327E+00_JPRB ,&
&  0.44933E+00_JPRB, 0.43741E+00_JPRB, 0.43453E+00_JPRB, 0.43689E+00_JPRB, 0.43909E+00_JPRB, 0.44326E+00_JPRB, 0.44942E+00_JPRB ,&
&  0.45503E+00_JPRB, 0.45888E+00_JPRB, 0.46303E+00_JPRB, 0.46473E+00_JPRB, 0.46506E+00_JPRB, 0.45813E+00_JPRB, 0.44715E+00_JPRB ,&
&  0.43678E+00_JPRB, 0.42955E+00_JPRB, 0.42528E+00_JPRB, 0.42176E+00_JPRB, 0.41651E+00_JPRB, 0.41135E+00_JPRB, 0.40464E+00_JPRB ,&
&  0.40126E+00_JPRB, 0.40082E+00_JPRB, 0.40155E+00_JPRB, 0.39969E+00_JPRB, 0.39727E+00_JPRB, 0.39258E+00_JPRB, 0.38885E+00_JPRB ,&
&  0.38776E+00_JPRB, 0.38952E+00_JPRB, 0.39147E+00_JPRB, 0.39126E+00_JPRB, 0.38848E+00_JPRB, 0.38664E+00_JPRB, 0.38846E+00_JPRB ,&
&  0.39459E+00_JPRB, 0.39461E+00_JPRB, 0.39368E+00_JPRB, 0.39425E+00_JPRB, 0.39621E+00_JPRB, 0.39806E+00_JPRB, 0.39757E+00_JPRB ,&
&  0.39708E+00_JPRB, 0.39730E+00_JPRB, 0.39788E+00_JPRB, 0.40009E+00_JPRB, 0.40043E+00_JPRB, 0.39857E+00_JPRB, 0.39328E+00_JPRB ,&
&  0.38505E+00_JPRB, 0.37455E+00_JPRB, 0.37018E+00_JPRB, 0.36702E+00_JPRB, 0.36392E+00_JPRB, 0.36306E+00_JPRB, 0.36417E+00_JPRB ,&
&  0.36265E+00_JPRB, 0.35644E+00_JPRB, 0.35353E+00_JPRB, 0.35370E+00_JPRB, 0.35210E+00_JPRB, 0.35365E+00_JPRB, 0.35674E+00_JPRB ,&
&  0.36615E+00_JPRB, 0.37124E+00_JPRB, 0.37274E+00_JPRB, 0.37064E+00_JPRB, 0.37100E+00_JPRB, 0.37269E+00_JPRB, 0.37680E+00_JPRB ,&
&  0.38310E+00_JPRB, 0.38983E+00_JPRB, 0.39665E+00_JPRB, 0.40417E+00_JPRB, 0.41384E+00_JPRB, 0.42330E+00_JPRB, 0.42683E+00_JPRB ,&
&  0.42685E+00_JPRB, 0.42693E+00_JPRB, 0.42786E+00_JPRB, 0.42915E+00_JPRB, 0.43201E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.45919E+00_JPRB, 0.45972E+00_JPRB, 0.45874E+00_JPRB, 0.45792E+00_JPRB, 0.45690E+00_JPRB, 0.45594E+00_JPRB, 0.45409E+00_JPRB ,&
&  0.44921E+00_JPRB, 0.44291E+00_JPRB, 0.43925E+00_JPRB, 0.43756E+00_JPRB, 0.44312E+00_JPRB, 0.45143E+00_JPRB, 0.45891E+00_JPRB ,&
&  0.45835E+00_JPRB, 0.44969E+00_JPRB, 0.44720E+00_JPRB, 0.44933E+00_JPRB, 0.45198E+00_JPRB, 0.45486E+00_JPRB, 0.45872E+00_JPRB ,&
&  0.46294E+00_JPRB, 0.46728E+00_JPRB, 0.46914E+00_JPRB, 0.46680E+00_JPRB, 0.46056E+00_JPRB, 0.44713E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.42759E+00_JPRB, 0.42185E+00_JPRB, 0.41898E+00_JPRB, 0.41815E+00_JPRB, 0.41585E+00_JPRB, 0.41176E+00_JPRB, 0.40621E+00_JPRB ,&
&  0.40134E+00_JPRB, 0.39900E+00_JPRB, 0.40098E+00_JPRB, 0.40253E+00_JPRB, 0.40342E+00_JPRB, 0.40339E+00_JPRB, 0.40088E+00_JPRB ,&
&  0.39972E+00_JPRB, 0.39978E+00_JPRB, 0.40016E+00_JPRB, 0.40240E+00_JPRB, 0.40070E+00_JPRB, 0.39925E+00_JPRB, 0.40172E+00_JPRB ,&
&  0.40645E+00_JPRB, 0.41041E+00_JPRB, 0.41383E+00_JPRB, 0.41626E+00_JPRB, 0.41713E+00_JPRB, 0.41909E+00_JPRB, 0.42048E+00_JPRB ,&
&  0.41975E+00_JPRB, 0.41649E+00_JPRB, 0.41347E+00_JPRB, 0.41434E+00_JPRB, 0.41702E+00_JPRB, 0.41996E+00_JPRB, 0.42368E+00_JPRB ,&
&  0.42442E+00_JPRB, 0.41937E+00_JPRB, 0.41424E+00_JPRB, 0.40962E+00_JPRB, 0.40530E+00_JPRB, 0.40182E+00_JPRB, 0.40000E+00_JPRB ,&
&  0.39933E+00_JPRB, 0.39624E+00_JPRB, 0.39034E+00_JPRB, 0.38541E+00_JPRB, 0.38436E+00_JPRB, 0.38287E+00_JPRB, 0.37862E+00_JPRB ,&
&  0.37859E+00_JPRB, 0.38069E+00_JPRB, 0.37943E+00_JPRB, 0.37754E+00_JPRB, 0.37954E+00_JPRB, 0.38469E+00_JPRB, 0.39515E+00_JPRB ,&
&  0.40919E+00_JPRB, 0.42543E+00_JPRB, 0.44049E+00_JPRB, 0.45377E+00_JPRB, 0.46180E+00_JPRB, 0.46381E+00_JPRB, 0.46211E+00_JPRB ,&
&  0.46131E+00_JPRB, 0.46172E+00_JPRB, 0.46039E+00_JPRB, 0.45840E+00_JPRB, 0.45826E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.48578E+00_JPRB, 0.48419E+00_JPRB, 0.48269E+00_JPRB, 0.48094E+00_JPRB, 0.47836E+00_JPRB, 0.47607E+00_JPRB, 0.47184E+00_JPRB ,&
&  0.46716E+00_JPRB, 0.46324E+00_JPRB, 0.46163E+00_JPRB, 0.46136E+00_JPRB, 0.46297E+00_JPRB, 0.46709E+00_JPRB, 0.47382E+00_JPRB ,&
&  0.47409E+00_JPRB, 0.46784E+00_JPRB, 0.46513E+00_JPRB, 0.46541E+00_JPRB, 0.46797E+00_JPRB, 0.46983E+00_JPRB, 0.47275E+00_JPRB ,&
&  0.47473E+00_JPRB, 0.47542E+00_JPRB, 0.47490E+00_JPRB, 0.46836E+00_JPRB, 0.45203E+00_JPRB, 0.44207E+00_JPRB, 0.43315E+00_JPRB ,&
&  0.42798E+00_JPRB, 0.42350E+00_JPRB, 0.42164E+00_JPRB, 0.42148E+00_JPRB, 0.42020E+00_JPRB, 0.41803E+00_JPRB, 0.41760E+00_JPRB ,&
&  0.41520E+00_JPRB, 0.41215E+00_JPRB, 0.41048E+00_JPRB, 0.41094E+00_JPRB, 0.41266E+00_JPRB, 0.41444E+00_JPRB, 0.41468E+00_JPRB ,&
&  0.41606E+00_JPRB, 0.42026E+00_JPRB, 0.42196E+00_JPRB, 0.42268E+00_JPRB, 0.42295E+00_JPRB, 0.42280E+00_JPRB, 0.42315E+00_JPRB ,&
&  0.42553E+00_JPRB, 0.43053E+00_JPRB, 0.43461E+00_JPRB, 0.43860E+00_JPRB, 0.44015E+00_JPRB, 0.44090E+00_JPRB, 0.44089E+00_JPRB ,&
&  0.43800E+00_JPRB, 0.43421E+00_JPRB, 0.43312E+00_JPRB, 0.43555E+00_JPRB, 0.44034E+00_JPRB, 0.44654E+00_JPRB, 0.45069E+00_JPRB ,&
&  0.45384E+00_JPRB, 0.45486E+00_JPRB, 0.45252E+00_JPRB, 0.45107E+00_JPRB, 0.44815E+00_JPRB, 0.44338E+00_JPRB, 0.44058E+00_JPRB ,&
&  0.43791E+00_JPRB, 0.43567E+00_JPRB, 0.43392E+00_JPRB, 0.42524E+00_JPRB, 0.41752E+00_JPRB, 0.41253E+00_JPRB, 0.40910E+00_JPRB ,&
&  0.40772E+00_JPRB, 0.40674E+00_JPRB, 0.40636E+00_JPRB, 0.40760E+00_JPRB, 0.40949E+00_JPRB, 0.41534E+00_JPRB, 0.42741E+00_JPRB ,&
&  0.44579E+00_JPRB, 0.46315E+00_JPRB, 0.47285E+00_JPRB, 0.48014E+00_JPRB, 0.48516E+00_JPRB, 0.48799E+00_JPRB, 0.49064E+00_JPRB ,&
&  0.49434E+00_JPRB, 0.49540E+00_JPRB, 0.49311E+00_JPRB, 0.49003E+00_JPRB, 0.48818E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.51084E+00_JPRB, 0.50279E+00_JPRB, 0.49682E+00_JPRB, 0.49206E+00_JPRB, 0.48827E+00_JPRB, 0.48709E+00_JPRB, 0.48553E+00_JPRB ,&
&  0.48381E+00_JPRB, 0.48160E+00_JPRB, 0.48069E+00_JPRB, 0.47812E+00_JPRB, 0.47558E+00_JPRB, 0.47848E+00_JPRB, 0.48395E+00_JPRB ,&
&  0.48280E+00_JPRB, 0.47662E+00_JPRB, 0.47175E+00_JPRB, 0.47197E+00_JPRB, 0.47563E+00_JPRB, 0.47839E+00_JPRB, 0.48164E+00_JPRB ,&
&  0.48160E+00_JPRB, 0.48108E+00_JPRB, 0.47990E+00_JPRB, 0.46487E+00_JPRB, 0.45005E+00_JPRB, 0.44279E+00_JPRB, 0.43857E+00_JPRB ,&
&  0.43778E+00_JPRB, 0.44186E+00_JPRB, 0.42743E+00_JPRB, 0.43886E+00_JPRB, 0.43448E+00_JPRB, 0.42785E+00_JPRB, 0.42398E+00_JPRB ,&
&  0.42135E+00_JPRB, 0.41828E+00_JPRB, 0.41615E+00_JPRB, 0.41801E+00_JPRB, 0.42089E+00_JPRB, 0.42210E+00_JPRB, 0.42447E+00_JPRB ,&
&  0.42490E+00_JPRB, 0.42804E+00_JPRB, 0.43077E+00_JPRB, 0.43627E+00_JPRB, 0.44044E+00_JPRB, 0.44293E+00_JPRB, 0.44607E+00_JPRB ,&
&  0.45040E+00_JPRB, 0.45444E+00_JPRB, 0.45822E+00_JPRB, 0.46134E+00_JPRB, 0.46249E+00_JPRB, 0.46215E+00_JPRB, 0.45806E+00_JPRB ,&
&  0.45479E+00_JPRB, 0.45456E+00_JPRB, 0.45676E+00_JPRB, 0.46123E+00_JPRB, 0.46587E+00_JPRB, 0.46824E+00_JPRB, 0.47003E+00_JPRB ,&
&  0.47208E+00_JPRB, 0.47556E+00_JPRB, 0.47589E+00_JPRB, 0.47421E+00_JPRB, 0.47191E+00_JPRB, 0.46908E+00_JPRB, 0.46442E+00_JPRB ,&
&  0.45857E+00_JPRB, 0.45415E+00_JPRB, 0.44971E+00_JPRB, 0.43921E+00_JPRB, 0.43107E+00_JPRB, 0.43027E+00_JPRB, 0.43168E+00_JPRB ,&
&  0.43114E+00_JPRB, 0.42963E+00_JPRB, 0.43079E+00_JPRB, 0.43627E+00_JPRB, 0.43566E+00_JPRB, 0.44135E+00_JPRB, 0.45418E+00_JPRB ,&
&  0.46771E+00_JPRB, 0.48008E+00_JPRB, 0.49048E+00_JPRB, 0.50160E+00_JPRB, 0.51291E+00_JPRB, 0.52185E+00_JPRB, 0.52958E+00_JPRB ,&
&  0.53324E+00_JPRB, 0.53160E+00_JPRB, 0.52599E+00_JPRB, 0.52220E+00_JPRB, 0.51881E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.55485E+00_JPRB, 0.54143E+00_JPRB, 0.53113E+00_JPRB, 0.52355E+00_JPRB, 0.51885E+00_JPRB, 0.51812E+00_JPRB, 0.51612E+00_JPRB ,&
&  0.51427E+00_JPRB, 0.51305E+00_JPRB, 0.50757E+00_JPRB, 0.50163E+00_JPRB, 0.50139E+00_JPRB, 0.50707E+00_JPRB, 0.51185E+00_JPRB ,&
&  0.50959E+00_JPRB, 0.49996E+00_JPRB, 0.49084E+00_JPRB, 0.48927E+00_JPRB, 0.49260E+00_JPRB, 0.49535E+00_JPRB, 0.49526E+00_JPRB ,&
&  0.49701E+00_JPRB, 0.49802E+00_JPRB, 0.48984E+00_JPRB, 0.47342E+00_JPRB, 0.46556E+00_JPRB, 0.46108E+00_JPRB, 0.46528E+00_JPRB ,&
&  0.47188E+00_JPRB, 0.46482E+00_JPRB, 0.47871E+00_JPRB, 0.47996E+00_JPRB, 0.47821E+00_JPRB, 0.47317E+00_JPRB, 0.46557E+00_JPRB ,&
&  0.45949E+00_JPRB, 0.45340E+00_JPRB, 0.44962E+00_JPRB, 0.44826E+00_JPRB, 0.44954E+00_JPRB, 0.45160E+00_JPRB, 0.45733E+00_JPRB ,&
&  0.45972E+00_JPRB, 0.45886E+00_JPRB, 0.46181E+00_JPRB, 0.46884E+00_JPRB, 0.47726E+00_JPRB, 0.48171E+00_JPRB, 0.48577E+00_JPRB ,&
&  0.48915E+00_JPRB, 0.49101E+00_JPRB, 0.49370E+00_JPRB, 0.49443E+00_JPRB, 0.49076E+00_JPRB, 0.48991E+00_JPRB, 0.48975E+00_JPRB ,&
&  0.49166E+00_JPRB, 0.49442E+00_JPRB, 0.49725E+00_JPRB, 0.49983E+00_JPRB, 0.50104E+00_JPRB, 0.49876E+00_JPRB, 0.49674E+00_JPRB ,&
&  0.49754E+00_JPRB, 0.50062E+00_JPRB, 0.50423E+00_JPRB, 0.50611E+00_JPRB, 0.50485E+00_JPRB, 0.49806E+00_JPRB, 0.49020E+00_JPRB ,&
&  0.48202E+00_JPRB, 0.47364E+00_JPRB, 0.46610E+00_JPRB, 0.46167E+00_JPRB, 0.45837E+00_JPRB, 0.45650E+00_JPRB, 0.45552E+00_JPRB ,&
&  0.45535E+00_JPRB, 0.45558E+00_JPRB, 0.45872E+00_JPRB, 0.46461E+00_JPRB, 0.46734E+00_JPRB, 0.47499E+00_JPRB, 0.48511E+00_JPRB ,&
&  0.49597E+00_JPRB, 0.50678E+00_JPRB, 0.51987E+00_JPRB, 0.53831E+00_JPRB, 0.55888E+00_JPRB, 0.57460E+00_JPRB, 0.58647E+00_JPRB ,&
&  0.59397E+00_JPRB, 0.59323E+00_JPRB, 0.58601E+00_JPRB, 0.57775E+00_JPRB, 0.56778E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.59981E+00_JPRB, 0.57987E+00_JPRB, 0.56341E+00_JPRB, 0.55164E+00_JPRB, 0.54509E+00_JPRB, 0.54395E+00_JPRB, 0.54135E+00_JPRB ,&
&  0.53988E+00_JPRB, 0.53710E+00_JPRB, 0.53530E+00_JPRB, 0.53775E+00_JPRB, 0.53913E+00_JPRB, 0.53988E+00_JPRB, 0.54171E+00_JPRB ,&
&  0.53703E+00_JPRB, 0.52522E+00_JPRB, 0.51571E+00_JPRB, 0.51343E+00_JPRB, 0.51492E+00_JPRB, 0.51144E+00_JPRB, 0.51028E+00_JPRB ,&
&  0.51334E+00_JPRB, 0.50558E+00_JPRB, 0.49254E+00_JPRB, 0.48411E+00_JPRB, 0.48455E+00_JPRB, 0.48228E+00_JPRB, 0.47952E+00_JPRB ,&
&  0.49955E+00_JPRB, 0.45346E+00_JPRB, 0.50024E+00_JPRB, 0.51289E+00_JPRB, 0.51900E+00_JPRB, 0.52008E+00_JPRB, 0.51502E+00_JPRB ,&
&  0.50794E+00_JPRB, 0.50153E+00_JPRB, 0.49846E+00_JPRB, 0.49687E+00_JPRB, 0.49623E+00_JPRB, 0.49342E+00_JPRB, 0.49019E+00_JPRB ,&
&  0.49095E+00_JPRB, 0.49212E+00_JPRB, 0.49648E+00_JPRB, 0.50160E+00_JPRB, 0.50832E+00_JPRB, 0.51004E+00_JPRB, 0.51165E+00_JPRB ,&
&  0.51391E+00_JPRB, 0.51521E+00_JPRB, 0.51482E+00_JPRB, 0.51342E+00_JPRB, 0.51167E+00_JPRB, 0.51606E+00_JPRB, 0.52202E+00_JPRB ,&
&  0.52642E+00_JPRB, 0.53019E+00_JPRB, 0.53522E+00_JPRB, 0.53652E+00_JPRB, 0.53636E+00_JPRB, 0.53280E+00_JPRB, 0.52721E+00_JPRB ,&
&  0.52287E+00_JPRB, 0.52163E+00_JPRB, 0.52107E+00_JPRB, 0.51626E+00_JPRB, 0.50927E+00_JPRB, 0.50294E+00_JPRB, 0.49536E+00_JPRB ,&
&  0.48571E+00_JPRB, 0.47804E+00_JPRB, 0.47418E+00_JPRB, 0.47336E+00_JPRB, 0.47477E+00_JPRB, 0.47570E+00_JPRB, 0.47659E+00_JPRB ,&
&  0.47950E+00_JPRB, 0.48046E+00_JPRB, 0.48234E+00_JPRB, 0.48700E+00_JPRB, 0.49093E+00_JPRB, 0.49716E+00_JPRB, 0.50567E+00_JPRB ,&
&  0.51730E+00_JPRB, 0.53069E+00_JPRB, 0.54316E+00_JPRB, 0.56338E+00_JPRB, 0.59086E+00_JPRB, 0.61106E+00_JPRB, 0.62282E+00_JPRB ,&
&  0.63341E+00_JPRB, 0.63596E+00_JPRB, 0.63083E+00_JPRB, 0.62482E+00_JPRB, 0.61392E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.65240E+00_JPRB, 0.63765E+00_JPRB, 0.61795E+00_JPRB, 0.60368E+00_JPRB, 0.59579E+00_JPRB, 0.59218E+00_JPRB, 0.58839E+00_JPRB ,&
&  0.58324E+00_JPRB, 0.58327E+00_JPRB, 0.58388E+00_JPRB, 0.58446E+00_JPRB, 0.58497E+00_JPRB, 0.58246E+00_JPRB, 0.57568E+00_JPRB ,&
&  0.56776E+00_JPRB, 0.55708E+00_JPRB, 0.55563E+00_JPRB, 0.55528E+00_JPRB, 0.54932E+00_JPRB, 0.54525E+00_JPRB, 0.54500E+00_JPRB ,&
&  0.53877E+00_JPRB, 0.52915E+00_JPRB, 0.51167E+00_JPRB, 0.50598E+00_JPRB, 0.50852E+00_JPRB, 0.50861E+00_JPRB, 0.51480E+00_JPRB ,&
&  0.49403E+00_JPRB, 0.47262E+00_JPRB, 0.52522E+00_JPRB, 0.56791E+00_JPRB, 0.58956E+00_JPRB, 0.59984E+00_JPRB, 0.60715E+00_JPRB ,&
&  0.60056E+00_JPRB, 0.60758E+00_JPRB, 0.59974E+00_JPRB, 0.57751E+00_JPRB, 0.56082E+00_JPRB, 0.55198E+00_JPRB, 0.54713E+00_JPRB ,&
&  0.54934E+00_JPRB, 0.55115E+00_JPRB, 0.55051E+00_JPRB, 0.54907E+00_JPRB, 0.54896E+00_JPRB, 0.54472E+00_JPRB, 0.54164E+00_JPRB ,&
&  0.54231E+00_JPRB, 0.54474E+00_JPRB, 0.54544E+00_JPRB, 0.54887E+00_JPRB, 0.55683E+00_JPRB, 0.56940E+00_JPRB, 0.58106E+00_JPRB ,&
&  0.58964E+00_JPRB, 0.59555E+00_JPRB, 0.60355E+00_JPRB, 0.60715E+00_JPRB, 0.60454E+00_JPRB, 0.59444E+00_JPRB, 0.58216E+00_JPRB ,&
&  0.56952E+00_JPRB, 0.56016E+00_JPRB, 0.55352E+00_JPRB, 0.54887E+00_JPRB, 0.54144E+00_JPRB, 0.53411E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.51789E+00_JPRB, 0.50977E+00_JPRB, 0.50578E+00_JPRB, 0.50426E+00_JPRB, 0.50620E+00_JPRB, 0.50966E+00_JPRB, 0.51453E+00_JPRB ,&
&  0.51800E+00_JPRB, 0.51662E+00_JPRB, 0.51661E+00_JPRB, 0.52115E+00_JPRB, 0.52658E+00_JPRB, 0.53593E+00_JPRB, 0.54852E+00_JPRB ,&
&  0.56376E+00_JPRB, 0.57831E+00_JPRB, 0.59339E+00_JPRB, 0.62381E+00_JPRB, 0.65364E+00_JPRB, 0.67731E+00_JPRB, 0.68935E+00_JPRB ,&
&  0.69506E+00_JPRB, 0.68635E+00_JPRB, 0.68524E+00_JPRB, 0.67559E+00_JPRB, 0.66282E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.65318E+00_JPRB, 0.65084E+00_JPRB, 0.63895E+00_JPRB, 0.63156E+00_JPRB, 0.62599E+00_JPRB, 0.61993E+00_JPRB, 0.61042E+00_JPRB ,&
&  0.60420E+00_JPRB, 0.60266E+00_JPRB, 0.60145E+00_JPRB, 0.60195E+00_JPRB, 0.60314E+00_JPRB, 0.59908E+00_JPRB, 0.59221E+00_JPRB ,&
&  0.58478E+00_JPRB, 0.57755E+00_JPRB, 0.57907E+00_JPRB, 0.57876E+00_JPRB, 0.57415E+00_JPRB, 0.57029E+00_JPRB, 0.56500E+00_JPRB ,&
&  0.55100E+00_JPRB, 0.53724E+00_JPRB, 0.52630E+00_JPRB, 0.52909E+00_JPRB, 0.53555E+00_JPRB, 0.53080E+00_JPRB, 0.54419E+00_JPRB ,&
&  0.58950E+00_JPRB, 0.51704E+00_JPRB, 0.55071E+00_JPRB, 0.63050E+00_JPRB, 0.66566E+00_JPRB, 0.70337E+00_JPRB, 0.73320E+00_JPRB ,&
&  0.75160E+00_JPRB, 0.76892E+00_JPRB, 0.73632E+00_JPRB, 0.69217E+00_JPRB, 0.65316E+00_JPRB, 0.62210E+00_JPRB, 0.59916E+00_JPRB ,&
&  0.58769E+00_JPRB, 0.58747E+00_JPRB, 0.58783E+00_JPRB, 0.58347E+00_JPRB, 0.57997E+00_JPRB, 0.57469E+00_JPRB, 0.57089E+00_JPRB ,&
&  0.56832E+00_JPRB, 0.56865E+00_JPRB, 0.57151E+00_JPRB, 0.58068E+00_JPRB, 0.58980E+00_JPRB, 0.60006E+00_JPRB, 0.61363E+00_JPRB ,&
&  0.63099E+00_JPRB, 0.64944E+00_JPRB, 0.66393E+00_JPRB, 0.66712E+00_JPRB, 0.66091E+00_JPRB, 0.64957E+00_JPRB, 0.63428E+00_JPRB ,&
&  0.61645E+00_JPRB, 0.59917E+00_JPRB, 0.58871E+00_JPRB, 0.58348E+00_JPRB, 0.57716E+00_JPRB, 0.57076E+00_JPRB, 0.56516E+00_JPRB ,&
&  0.56033E+00_JPRB, 0.55455E+00_JPRB, 0.55480E+00_JPRB, 0.55539E+00_JPRB, 0.55525E+00_JPRB, 0.55078E+00_JPRB, 0.54597E+00_JPRB ,&
&  0.54175E+00_JPRB, 0.53892E+00_JPRB, 0.54041E+00_JPRB, 0.54601E+00_JPRB, 0.55477E+00_JPRB, 0.56980E+00_JPRB, 0.58895E+00_JPRB ,&
&  0.60348E+00_JPRB, 0.61675E+00_JPRB, 0.63072E+00_JPRB, 0.66400E+00_JPRB, 0.68555E+00_JPRB, 0.69927E+00_JPRB, 0.70514E+00_JPRB ,&
&  0.70386E+00_JPRB, 0.70513E+00_JPRB, 0.55835E+00_JPRB, 0.62405E+00_JPRB, 0.64298E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.66726E+00_JPRB, 0.66417E+00_JPRB, 0.65735E+00_JPRB, 0.65469E+00_JPRB, 0.65410E+00_JPRB, 0.64184E+00_JPRB, 0.63159E+00_JPRB ,&
&  0.62603E+00_JPRB, 0.62688E+00_JPRB, 0.63053E+00_JPRB, 0.63220E+00_JPRB, 0.62802E+00_JPRB, 0.61924E+00_JPRB, 0.61462E+00_JPRB ,&
&  0.60668E+00_JPRB, 0.59937E+00_JPRB, 0.59790E+00_JPRB, 0.59772E+00_JPRB, 0.59267E+00_JPRB, 0.58520E+00_JPRB, 0.57888E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.55793E+00_JPRB, 0.55635E+00_JPRB, 0.55943E+00_JPRB, 0.56267E+00_JPRB, 0.56133E+00_JPRB, 0.57714E+00_JPRB ,&
&  0.61468E+00_JPRB, 0.55783E+00_JPRB, 0.57884E+00_JPRB, 0.72917E+00_JPRB, 0.81225E+00_JPRB, 0.83043E+00_JPRB, 0.85159E+00_JPRB ,&
&  0.87157E+00_JPRB, 0.86535E+00_JPRB, 0.83924E+00_JPRB, 0.78490E+00_JPRB, 0.74343E+00_JPRB, 0.71381E+00_JPRB, 0.68747E+00_JPRB ,&
&  0.66798E+00_JPRB, 0.65309E+00_JPRB, 0.64627E+00_JPRB, 0.63698E+00_JPRB, 0.63283E+00_JPRB, 0.62359E+00_JPRB, 0.62080E+00_JPRB ,&
&  0.62169E+00_JPRB, 0.62722E+00_JPRB, 0.63271E+00_JPRB, 0.63526E+00_JPRB, 0.64242E+00_JPRB, 0.65407E+00_JPRB, 0.67524E+00_JPRB ,&
&  0.70335E+00_JPRB, 0.73405E+00_JPRB, 0.74879E+00_JPRB, 0.74624E+00_JPRB, 0.73493E+00_JPRB, 0.71924E+00_JPRB, 0.69766E+00_JPRB ,&
&  0.67250E+00_JPRB, 0.64844E+00_JPRB, 0.63188E+00_JPRB, 0.62369E+00_JPRB, 0.61954E+00_JPRB, 0.61697E+00_JPRB, 0.61389E+00_JPRB ,&
&  0.61092E+00_JPRB, 0.61060E+00_JPRB, 0.61162E+00_JPRB, 0.61103E+00_JPRB, 0.60989E+00_JPRB, 0.60177E+00_JPRB, 0.58840E+00_JPRB ,&
&  0.57887E+00_JPRB, 0.57652E+00_JPRB, 0.58373E+00_JPRB, 0.58971E+00_JPRB, 0.59777E+00_JPRB, 0.61516E+00_JPRB, 0.63472E+00_JPRB ,&
&  0.64750E+00_JPRB, 0.66320E+00_JPRB, 0.68665E+00_JPRB, 0.68581E+00_JPRB, 0.72214E+00_JPRB, 0.72025E+00_JPRB, 0.72102E+00_JPRB ,&
&  0.71089E+00_JPRB, 0.69511E+00_JPRB, 0.68602E+00_JPRB, 0.58833E+00_JPRB, 0.64917E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.67927E+00_JPRB, 0.67668E+00_JPRB, 0.67116E+00_JPRB, 0.66833E+00_JPRB, 0.66433E+00_JPRB, 0.65348E+00_JPRB, 0.64521E+00_JPRB ,&
&  0.64339E+00_JPRB, 0.64024E+00_JPRB, 0.63745E+00_JPRB, 0.63172E+00_JPRB, 0.62301E+00_JPRB, 0.61720E+00_JPRB, 0.61787E+00_JPRB ,&
&  0.61574E+00_JPRB, 0.61516E+00_JPRB, 0.61503E+00_JPRB, 0.61094E+00_JPRB, 0.60535E+00_JPRB, 0.59673E+00_JPRB, 0.58299E+00_JPRB ,&
&  0.56644E+00_JPRB, 0.56176E+00_JPRB, 0.57040E+00_JPRB, 0.57722E+00_JPRB, 0.58129E+00_JPRB, 0.58348E+00_JPRB, 0.59351E+00_JPRB ,&
&  0.62704E+00_JPRB, 0.63461E+00_JPRB, 0.76667E+00_JPRB, 0.95715E+00_JPRB, 0.10418E+01_JPRB, 0.10104E+01_JPRB, 0.98775E+00_JPRB ,&
&  0.94253E+00_JPRB, 0.89685E+00_JPRB, 0.84873E+00_JPRB, 0.79564E+00_JPRB, 0.75219E+00_JPRB, 0.72475E+00_JPRB, 0.71087E+00_JPRB ,&
&  0.70137E+00_JPRB, 0.69189E+00_JPRB, 0.68650E+00_JPRB, 0.67676E+00_JPRB, 0.66872E+00_JPRB, 0.66357E+00_JPRB, 0.66526E+00_JPRB ,&
&  0.66815E+00_JPRB, 0.67200E+00_JPRB, 0.67323E+00_JPRB, 0.67351E+00_JPRB, 0.67317E+00_JPRB, 0.68190E+00_JPRB, 0.70791E+00_JPRB ,&
&  0.74391E+00_JPRB, 0.78072E+00_JPRB, 0.79270E+00_JPRB, 0.78310E+00_JPRB, 0.77428E+00_JPRB, 0.76854E+00_JPRB, 0.74378E+00_JPRB ,&
&  0.71060E+00_JPRB, 0.68918E+00_JPRB, 0.67243E+00_JPRB, 0.65858E+00_JPRB, 0.64948E+00_JPRB, 0.64553E+00_JPRB, 0.64300E+00_JPRB ,&
&  0.64146E+00_JPRB, 0.63961E+00_JPRB, 0.63669E+00_JPRB, 0.63514E+00_JPRB, 0.63363E+00_JPRB, 0.62630E+00_JPRB, 0.61738E+00_JPRB ,&
&  0.60767E+00_JPRB, 0.60938E+00_JPRB, 0.62087E+00_JPRB, 0.62691E+00_JPRB, 0.63251E+00_JPRB, 0.64515E+00_JPRB, 0.66228E+00_JPRB ,&
&  0.67256E+00_JPRB, 0.68971E+00_JPRB, 0.70289E+00_JPRB, 0.74885E+00_JPRB, 0.75382E+00_JPRB, 0.73660E+00_JPRB, 0.72566E+00_JPRB ,&
&  0.70470E+00_JPRB, 0.68221E+00_JPRB, 0.66735E+00_JPRB, 0.67807E+00_JPRB, 0.64363E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.76247E+00_JPRB, 0.72858E+00_JPRB, 0.69502E+00_JPRB, 0.67382E+00_JPRB, 0.66514E+00_JPRB, 0.66217E+00_JPRB, 0.66062E+00_JPRB ,&
&  0.65702E+00_JPRB, 0.65210E+00_JPRB, 0.63694E+00_JPRB, 0.62620E+00_JPRB, 0.61922E+00_JPRB, 0.61790E+00_JPRB, 0.61656E+00_JPRB ,&
&  0.61770E+00_JPRB, 0.61760E+00_JPRB, 0.61353E+00_JPRB, 0.61262E+00_JPRB, 0.61424E+00_JPRB, 0.60980E+00_JPRB, 0.59436E+00_JPRB ,&
&  0.57902E+00_JPRB, 0.58114E+00_JPRB, 0.58582E+00_JPRB, 0.58848E+00_JPRB, 0.59411E+00_JPRB, 0.60327E+00_JPRB, 0.61914E+00_JPRB ,&
&  0.65182E+00_JPRB, 0.68707E+00_JPRB, 0.99002E+00_JPRB, 0.11813E+01_JPRB, 0.12536E+01_JPRB, 0.12471E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.10625E+01_JPRB, 0.96396E+00_JPRB, 0.88583E+00_JPRB, 0.81257E+00_JPRB, 0.76599E+00_JPRB, 0.73453E+00_JPRB, 0.71373E+00_JPRB ,&
&  0.70274E+00_JPRB, 0.69762E+00_JPRB, 0.69376E+00_JPRB, 0.68905E+00_JPRB, 0.68609E+00_JPRB, 0.68480E+00_JPRB, 0.68523E+00_JPRB ,&
&  0.68790E+00_JPRB, 0.69375E+00_JPRB, 0.69509E+00_JPRB, 0.69151E+00_JPRB, 0.68909E+00_JPRB, 0.72280E+00_JPRB, 0.77950E+00_JPRB ,&
&  0.82992E+00_JPRB, 0.85875E+00_JPRB, 0.84937E+00_JPRB, 0.84051E+00_JPRB, 0.83477E+00_JPRB, 0.81514E+00_JPRB, 0.79152E+00_JPRB ,&
&  0.76489E+00_JPRB, 0.73633E+00_JPRB, 0.70589E+00_JPRB, 0.68265E+00_JPRB, 0.66986E+00_JPRB, 0.66423E+00_JPRB, 0.66152E+00_JPRB ,&
&  0.66227E+00_JPRB, 0.66655E+00_JPRB, 0.66553E+00_JPRB, 0.66591E+00_JPRB, 0.66206E+00_JPRB, 0.65773E+00_JPRB, 0.65649E+00_JPRB ,&
&  0.65458E+00_JPRB, 0.65257E+00_JPRB, 0.66392E+00_JPRB, 0.67135E+00_JPRB, 0.67760E+00_JPRB, 0.68680E+00_JPRB, 0.69558E+00_JPRB ,&
&  0.70602E+00_JPRB, 0.72871E+00_JPRB, 0.70899E+00_JPRB, 0.72963E+00_JPRB, 0.79930E+00_JPRB, 0.76248E+00_JPRB, 0.71938E+00_JPRB ,&
&  0.68424E+00_JPRB, 0.66872E+00_JPRB, 0.66886E+00_JPRB, 0.69857E+00_JPRB, 0.75549E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.72717E+00_JPRB, 0.68560E+00_JPRB, 0.66350E+00_JPRB, 0.66159E+00_JPRB, 0.66797E+00_JPRB, 0.66536E+00_JPRB, 0.66891E+00_JPRB ,&
&  0.66318E+00_JPRB, 0.65112E+00_JPRB, 0.63750E+00_JPRB, 0.62473E+00_JPRB, 0.61900E+00_JPRB, 0.61469E+00_JPRB, 0.61159E+00_JPRB ,&
&  0.60973E+00_JPRB, 0.60816E+00_JPRB, 0.60372E+00_JPRB, 0.60739E+00_JPRB, 0.60961E+00_JPRB, 0.60848E+00_JPRB, 0.58808E+00_JPRB ,&
&  0.57905E+00_JPRB, 0.58078E+00_JPRB, 0.58524E+00_JPRB, 0.58945E+00_JPRB, 0.59645E+00_JPRB, 0.60841E+00_JPRB, 0.62244E+00_JPRB ,&
&  0.65566E+00_JPRB, 0.75460E+00_JPRB, 0.11204E+01_JPRB, 0.13541E+01_JPRB, 0.15041E+01_JPRB, 0.13874E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.11186E+01_JPRB, 0.10455E+01_JPRB, 0.98696E+00_JPRB, 0.89044E+00_JPRB, 0.79847E+00_JPRB, 0.74930E+00_JPRB, 0.72470E+00_JPRB ,&
&  0.71370E+00_JPRB, 0.71088E+00_JPRB, 0.70728E+00_JPRB, 0.70384E+00_JPRB, 0.69962E+00_JPRB, 0.69874E+00_JPRB, 0.70289E+00_JPRB ,&
&  0.70804E+00_JPRB, 0.70507E+00_JPRB, 0.70147E+00_JPRB, 0.68933E+00_JPRB, 0.74200E+00_JPRB, 0.81042E+00_JPRB, 0.90023E+00_JPRB ,&
&  0.93565E+00_JPRB, 0.95204E+00_JPRB, 0.93007E+00_JPRB, 0.89115E+00_JPRB, 0.85334E+00_JPRB, 0.82092E+00_JPRB, 0.79360E+00_JPRB ,&
&  0.76847E+00_JPRB, 0.74492E+00_JPRB, 0.72490E+00_JPRB, 0.71872E+00_JPRB, 0.72135E+00_JPRB, 0.71323E+00_JPRB, 0.70815E+00_JPRB ,&
&  0.70736E+00_JPRB, 0.70956E+00_JPRB, 0.70769E+00_JPRB, 0.69703E+00_JPRB, 0.68281E+00_JPRB, 0.67536E+00_JPRB, 0.67322E+00_JPRB ,&
&  0.68037E+00_JPRB, 0.68370E+00_JPRB, 0.70713E+00_JPRB, 0.71151E+00_JPRB, 0.71521E+00_JPRB, 0.71771E+00_JPRB, 0.72635E+00_JPRB ,&
&  0.74462E+00_JPRB, 0.74455E+00_JPRB, 0.74161E+00_JPRB, 0.75232E+00_JPRB, 0.75485E+00_JPRB, 0.77005E+00_JPRB, 0.70415E+00_JPRB ,&
&  0.66482E+00_JPRB, 0.65351E+00_JPRB, 0.65913E+00_JPRB, 0.68618E+00_JPRB, 0.73596E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.69180E+00_JPRB, 0.66991E+00_JPRB, 0.66219E+00_JPRB, 0.66416E+00_JPRB, 0.66037E+00_JPRB, 0.66331E+00_JPRB, 0.66834E+00_JPRB ,&
&  0.66719E+00_JPRB, 0.65439E+00_JPRB, 0.64194E+00_JPRB, 0.62776E+00_JPRB, 0.62300E+00_JPRB, 0.61576E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.60788E+00_JPRB, 0.60512E+00_JPRB, 0.59967E+00_JPRB, 0.59847E+00_JPRB, 0.59894E+00_JPRB, 0.59484E+00_JPRB, 0.57671E+00_JPRB ,&
&  0.57323E+00_JPRB, 0.57249E+00_JPRB, 0.57949E+00_JPRB, 0.58998E+00_JPRB, 0.60254E+00_JPRB, 0.61029E+00_JPRB, 0.61894E+00_JPRB ,&
&  0.66957E+00_JPRB, 0.75395E+00_JPRB, 0.13423E+01_JPRB, 0.15014E+01_JPRB, 0.15169E+01_JPRB, 0.13083E+01_JPRB, 0.11839E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.10577E+01_JPRB, 0.10341E+01_JPRB, 0.95388E+00_JPRB, 0.85373E+00_JPRB, 0.77874E+00_JPRB, 0.75605E+00_JPRB ,&
&  0.74414E+00_JPRB, 0.73288E+00_JPRB, 0.72832E+00_JPRB, 0.72784E+00_JPRB, 0.72340E+00_JPRB, 0.72135E+00_JPRB, 0.72807E+00_JPRB ,&
&  0.73001E+00_JPRB, 0.72856E+00_JPRB, 0.72175E+00_JPRB, 0.73818E+00_JPRB, 0.78829E+00_JPRB, 0.75318E+00_JPRB, 0.87348E+00_JPRB ,&
&  0.11092E+01_JPRB, 0.10920E+01_JPRB, 0.10076E+01_JPRB, 0.90193E+00_JPRB, 0.83805E+00_JPRB, 0.81248E+00_JPRB, 0.79366E+00_JPRB ,&
&  0.77792E+00_JPRB, 0.76175E+00_JPRB, 0.75108E+00_JPRB, 0.75801E+00_JPRB, 0.75768E+00_JPRB, 0.74768E+00_JPRB, 0.73861E+00_JPRB ,&
&  0.74025E+00_JPRB, 0.74207E+00_JPRB, 0.73158E+00_JPRB, 0.70926E+00_JPRB, 0.69456E+00_JPRB, 0.69315E+00_JPRB, 0.69111E+00_JPRB ,&
&  0.69083E+00_JPRB, 0.69609E+00_JPRB, 0.70746E+00_JPRB, 0.69400E+00_JPRB, 0.71136E+00_JPRB, 0.74273E+00_JPRB, 0.75284E+00_JPRB ,&
&  0.73819E+00_JPRB, 0.73079E+00_JPRB, 0.75312E+00_JPRB, 0.74422E+00_JPRB, 0.75515E+00_JPRB, 0.76244E+00_JPRB, 0.68869E+00_JPRB ,&
&  0.65087E+00_JPRB, 0.64557E+00_JPRB, 0.66756E+00_JPRB, 0.69669E+00_JPRB, 0.73452E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.67296E+00_JPRB, 0.66053E+00_JPRB, 0.65735E+00_JPRB, 0.65333E+00_JPRB, 0.64406E+00_JPRB, 0.63364E+00_JPRB, 0.63728E+00_JPRB ,&
&  0.64288E+00_JPRB, 0.64672E+00_JPRB, 0.63884E+00_JPRB, 0.62374E+00_JPRB, 0.62110E+00_JPRB, 0.61710E+00_JPRB, 0.61197E+00_JPRB ,&
&  0.60750E+00_JPRB, 0.60703E+00_JPRB, 0.59973E+00_JPRB, 0.59551E+00_JPRB, 0.59803E+00_JPRB, 0.59624E+00_JPRB, 0.58897E+00_JPRB ,&
&  0.58086E+00_JPRB, 0.57537E+00_JPRB, 0.58007E+00_JPRB, 0.58503E+00_JPRB, 0.59720E+00_JPRB, 0.60163E+00_JPRB, 0.60761E+00_JPRB ,&
&  0.66325E+00_JPRB, 0.59615E+00_JPRB, 0.12791E+01_JPRB, 0.17599E+01_JPRB, 0.16484E+01_JPRB, 0.13288E+01_JPRB, 0.11429E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.10636E+01_JPRB, 0.10179E+01_JPRB, 0.95212E+00_JPRB, 0.89021E+00_JPRB, 0.82751E+00_JPRB, 0.80078E+00_JPRB ,&
&  0.78332E+00_JPRB, 0.76429E+00_JPRB, 0.75919E+00_JPRB, 0.75638E+00_JPRB, 0.75867E+00_JPRB, 0.75795E+00_JPRB, 0.76597E+00_JPRB ,&
&  0.76573E+00_JPRB, 0.76600E+00_JPRB, 0.78345E+00_JPRB, 0.83379E+00_JPRB, 0.85652E+00_JPRB, 0.82431E+00_JPRB, 0.87933E+00_JPRB ,&
&  0.99477E+00_JPRB, 0.11040E+01_JPRB, 0.10001E+01_JPRB, 0.87848E+00_JPRB, 0.82098E+00_JPRB, 0.80321E+00_JPRB, 0.78599E+00_JPRB ,&
&  0.76907E+00_JPRB, 0.75948E+00_JPRB, 0.76988E+00_JPRB, 0.76886E+00_JPRB, 0.75305E+00_JPRB, 0.73276E+00_JPRB, 0.73611E+00_JPRB ,&
&  0.74307E+00_JPRB, 0.73834E+00_JPRB, 0.72156E+00_JPRB, 0.71386E+00_JPRB, 0.70792E+00_JPRB, 0.69432E+00_JPRB, 0.68839E+00_JPRB ,&
&  0.68323E+00_JPRB, 0.69697E+00_JPRB, 0.70185E+00_JPRB, 0.70111E+00_JPRB, 0.70981E+00_JPRB, 0.72862E+00_JPRB, 0.74136E+00_JPRB ,&
&  0.75374E+00_JPRB, 0.75951E+00_JPRB, 0.74447E+00_JPRB, 0.74494E+00_JPRB, 0.70972E+00_JPRB, 0.72702E+00_JPRB, 0.67046E+00_JPRB ,&
&  0.65125E+00_JPRB, 0.65117E+00_JPRB, 0.66307E+00_JPRB, 0.67501E+00_JPRB, 0.69396E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.65349E+00_JPRB, 0.64760E+00_JPRB, 0.64810E+00_JPRB, 0.64125E+00_JPRB, 0.62650E+00_JPRB, 0.61382E+00_JPRB, 0.61362E+00_JPRB ,&
&  0.61257E+00_JPRB, 0.61694E+00_JPRB, 0.62426E+00_JPRB, 0.61567E+00_JPRB, 0.61488E+00_JPRB, 0.61729E+00_JPRB, 0.61588E+00_JPRB ,&
&  0.60731E+00_JPRB, 0.60327E+00_JPRB, 0.59921E+00_JPRB, 0.59603E+00_JPRB, 0.59511E+00_JPRB, 0.58675E+00_JPRB, 0.58279E+00_JPRB ,&
&  0.58179E+00_JPRB, 0.58581E+00_JPRB, 0.58990E+00_JPRB, 0.59120E+00_JPRB, 0.59963E+00_JPRB, 0.60267E+00_JPRB, 0.60768E+00_JPRB ,&
&  0.64359E+00_JPRB, 0.11211E+01_JPRB, 0.10194E+01_JPRB, 0.20066E+01_JPRB, 0.16607E+01_JPRB, 0.13152E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10584E+01_JPRB, 0.99756E+00_JPRB, 0.94311E+00_JPRB, 0.89886E+00_JPRB, 0.84955E+00_JPRB, 0.80436E+00_JPRB ,&
&  0.77047E+00_JPRB, 0.75573E+00_JPRB, 0.75817E+00_JPRB, 0.76793E+00_JPRB, 0.78771E+00_JPRB, 0.80810E+00_JPRB, 0.81703E+00_JPRB ,&
&  0.82741E+00_JPRB, 0.85811E+00_JPRB, 0.91240E+00_JPRB, 0.98393E+00_JPRB, 0.90775E+00_JPRB, 0.85763E+00_JPRB, 0.92189E+00_JPRB ,&
&  0.95223E+00_JPRB, 0.10605E+01_JPRB, 0.95343E+00_JPRB, 0.85101E+00_JPRB, 0.81541E+00_JPRB, 0.79864E+00_JPRB, 0.77633E+00_JPRB ,&
&  0.75964E+00_JPRB, 0.75881E+00_JPRB, 0.77665E+00_JPRB, 0.75779E+00_JPRB, 0.72994E+00_JPRB, 0.72364E+00_JPRB, 0.72421E+00_JPRB ,&
&  0.72660E+00_JPRB, 0.71563E+00_JPRB, 0.73069E+00_JPRB, 0.73328E+00_JPRB, 0.72805E+00_JPRB, 0.71538E+00_JPRB, 0.70338E+00_JPRB ,&
&  0.70031E+00_JPRB, 0.71392E+00_JPRB, 0.70454E+00_JPRB, 0.69099E+00_JPRB, 0.69491E+00_JPRB, 0.69985E+00_JPRB, 0.70052E+00_JPRB ,&
&  0.74631E+00_JPRB, 0.75686E+00_JPRB, 0.73809E+00_JPRB, 0.72485E+00_JPRB, 0.70839E+00_JPRB, 0.72712E+00_JPRB, 0.66356E+00_JPRB ,&
&  0.65131E+00_JPRB, 0.64867E+00_JPRB, 0.65469E+00_JPRB, 0.66571E+00_JPRB, 0.66720E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.64821E+00_JPRB, 0.64372E+00_JPRB, 0.64416E+00_JPRB, 0.62873E+00_JPRB, 0.60973E+00_JPRB, 0.59841E+00_JPRB, 0.59641E+00_JPRB ,&
&  0.59967E+00_JPRB, 0.61100E+00_JPRB, 0.61906E+00_JPRB, 0.60548E+00_JPRB, 0.59746E+00_JPRB, 0.60316E+00_JPRB, 0.60299E+00_JPRB ,&
&  0.60028E+00_JPRB, 0.59580E+00_JPRB, 0.59404E+00_JPRB, 0.59417E+00_JPRB, 0.59161E+00_JPRB, 0.58193E+00_JPRB, 0.58207E+00_JPRB ,&
&  0.59159E+00_JPRB, 0.60043E+00_JPRB, 0.60443E+00_JPRB, 0.60609E+00_JPRB, 0.61760E+00_JPRB, 0.61970E+00_JPRB, 0.62686E+00_JPRB ,&
&  0.64731E+00_JPRB, 0.10904E+01_JPRB, 0.11025E+01_JPRB, 0.20369E+01_JPRB, 0.16870E+01_JPRB, 0.13682E+01_JPRB, 0.11171E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10735E+01_JPRB, 0.98048E+00_JPRB, 0.88369E+00_JPRB, 0.82884E+00_JPRB, 0.81339E+00_JPRB, 0.79111E+00_JPRB ,&
&  0.76789E+00_JPRB, 0.78241E+00_JPRB, 0.80425E+00_JPRB, 0.83430E+00_JPRB, 0.86668E+00_JPRB, 0.88565E+00_JPRB, 0.91484E+00_JPRB ,&
&  0.94907E+00_JPRB, 0.10092E+01_JPRB, 0.10972E+01_JPRB, 0.10442E+01_JPRB, 0.93659E+00_JPRB, 0.91850E+00_JPRB, 0.91987E+00_JPRB ,&
&  0.92900E+00_JPRB, 0.99064E+00_JPRB, 0.89529E+00_JPRB, 0.82891E+00_JPRB, 0.78513E+00_JPRB, 0.77605E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.73313E+00_JPRB, 0.73530E+00_JPRB, 0.73693E+00_JPRB, 0.72641E+00_JPRB, 0.71615E+00_JPRB, 0.70922E+00_JPRB, 0.69963E+00_JPRB ,&
&  0.70062E+00_JPRB, 0.72279E+00_JPRB, 0.72799E+00_JPRB, 0.73155E+00_JPRB, 0.73736E+00_JPRB, 0.73959E+00_JPRB, 0.72892E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.73180E+00_JPRB, 0.70218E+00_JPRB, 0.69245E+00_JPRB, 0.70064E+00_JPRB, 0.69840E+00_JPRB, 0.70024E+00_JPRB ,&
&  0.72894E+00_JPRB, 0.73876E+00_JPRB, 0.71683E+00_JPRB, 0.68288E+00_JPRB, 0.69560E+00_JPRB, 0.71520E+00_JPRB, 0.66372E+00_JPRB ,&
&  0.65769E+00_JPRB, 0.65536E+00_JPRB, 0.65870E+00_JPRB, 0.65836E+00_JPRB, 0.65350E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.62073E+00_JPRB, 0.61624E+00_JPRB, 0.61094E+00_JPRB, 0.57546E+00_JPRB, 0.54658E+00_JPRB, 0.53699E+00_JPRB, 0.55344E+00_JPRB ,&
&  0.58609E+00_JPRB, 0.60822E+00_JPRB, 0.60410E+00_JPRB, 0.58922E+00_JPRB, 0.58202E+00_JPRB, 0.58357E+00_JPRB, 0.58912E+00_JPRB ,&
&  0.58695E+00_JPRB, 0.57592E+00_JPRB, 0.56888E+00_JPRB, 0.57560E+00_JPRB, 0.57779E+00_JPRB, 0.58506E+00_JPRB, 0.59861E+00_JPRB ,&
&  0.61759E+00_JPRB, 0.62756E+00_JPRB, 0.62885E+00_JPRB, 0.63339E+00_JPRB, 0.65521E+00_JPRB, 0.65279E+00_JPRB, 0.65383E+00_JPRB ,&
&  0.71784E+00_JPRB, 0.11801E+01_JPRB, 0.13909E+01_JPRB, 0.17281E+01_JPRB, 0.15303E+01_JPRB, 0.12166E+01_JPRB, 0.10577E+01_JPRB ,&
&  0.94484E+00_JPRB, 0.83284E+00_JPRB, 0.87711E+00_JPRB, 0.80881E+00_JPRB, 0.80798E+00_JPRB, 0.80626E+00_JPRB, 0.80867E+00_JPRB ,&
&  0.83479E+00_JPRB, 0.88646E+00_JPRB, 0.92022E+00_JPRB, 0.94794E+00_JPRB, 0.97219E+00_JPRB, 0.10020E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.11710E+01_JPRB, 0.12281E+01_JPRB, 0.10868E+01_JPRB, 0.91228E+00_JPRB, 0.92314E+00_JPRB, 0.91679E+00_JPRB ,&
&  0.91873E+00_JPRB, 0.91040E+00_JPRB, 0.85244E+00_JPRB, 0.79399E+00_JPRB, 0.72222E+00_JPRB, 0.75391E+00_JPRB, 0.72692E+00_JPRB ,&
&  0.69596E+00_JPRB, 0.70278E+00_JPRB, 0.69934E+00_JPRB, 0.70487E+00_JPRB, 0.70379E+00_JPRB, 0.69575E+00_JPRB, 0.69584E+00_JPRB ,&
&  0.70704E+00_JPRB, 0.71502E+00_JPRB, 0.72353E+00_JPRB, 0.73081E+00_JPRB, 0.74305E+00_JPRB, 0.75098E+00_JPRB, 0.74710E+00_JPRB ,&
&  0.73854E+00_JPRB, 0.74431E+00_JPRB, 0.72074E+00_JPRB, 0.72820E+00_JPRB, 0.72832E+00_JPRB, 0.72385E+00_JPRB, 0.70018E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.71280E+00_JPRB, 0.69559E+00_JPRB, 0.68406E+00_JPRB, 0.69449E+00_JPRB, 0.69009E+00_JPRB, 0.66249E+00_JPRB ,&
&  0.66442E+00_JPRB, 0.65930E+00_JPRB, 0.65727E+00_JPRB, 0.64055E+00_JPRB, 0.62918E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.60680E+00_JPRB, 0.59797E+00_JPRB, 0.57284E+00_JPRB, 0.51841E+00_JPRB, 0.50723E+00_JPRB, 0.52459E+00_JPRB, 0.58353E+00_JPRB ,&
&  0.60752E+00_JPRB, 0.61141E+00_JPRB, 0.59661E+00_JPRB, 0.57772E+00_JPRB, 0.56561E+00_JPRB, 0.56249E+00_JPRB, 0.57364E+00_JPRB ,&
&  0.58302E+00_JPRB, 0.57230E+00_JPRB, 0.57066E+00_JPRB, 0.57720E+00_JPRB, 0.58871E+00_JPRB, 0.60782E+00_JPRB, 0.62934E+00_JPRB ,&
&  0.64747E+00_JPRB, 0.65697E+00_JPRB, 0.66069E+00_JPRB, 0.68249E+00_JPRB, 0.70990E+00_JPRB, 0.71737E+00_JPRB, 0.75114E+00_JPRB ,&
&  0.97000E+00_JPRB, 0.13757E+01_JPRB, 0.69195E+00_JPRB, 0.10644E+01_JPRB, 0.12168E+01_JPRB, 0.10604E+01_JPRB, 0.93736E+00_JPRB ,&
&  0.85053E+00_JPRB, 0.74842E+00_JPRB, 0.75410E+00_JPRB, 0.82347E+00_JPRB, 0.84314E+00_JPRB, 0.88989E+00_JPRB, 0.93802E+00_JPRB ,&
&  0.98579E+00_JPRB, 0.10154E+01_JPRB, 0.10400E+01_JPRB, 0.10620E+01_JPRB, 0.10895E+01_JPRB, 0.11369E+01_JPRB, 0.11797E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.12308E+01_JPRB, 0.12116E+01_JPRB, 0.97728E+00_JPRB, 0.88529E+00_JPRB, 0.88402E+00_JPRB, 0.85241E+00_JPRB ,&
&  0.80240E+00_JPRB, 0.82996E+00_JPRB, 0.81085E+00_JPRB, 0.74835E+00_JPRB, 0.68566E+00_JPRB, 0.66288E+00_JPRB, 0.69050E+00_JPRB ,&
&  0.67620E+00_JPRB, 0.66631E+00_JPRB, 0.66069E+00_JPRB, 0.66532E+00_JPRB, 0.66567E+00_JPRB, 0.66751E+00_JPRB, 0.67346E+00_JPRB ,&
&  0.68970E+00_JPRB, 0.70391E+00_JPRB, 0.71499E+00_JPRB, 0.72283E+00_JPRB, 0.73529E+00_JPRB, 0.74879E+00_JPRB, 0.74938E+00_JPRB ,&
&  0.75840E+00_JPRB, 0.76342E+00_JPRB, 0.76960E+00_JPRB, 0.76247E+00_JPRB, 0.74139E+00_JPRB, 0.72825E+00_JPRB, 0.72125E+00_JPRB ,&
&  0.71603E+00_JPRB, 0.70806E+00_JPRB, 0.67914E+00_JPRB, 0.67056E+00_JPRB, 0.70688E+00_JPRB, 0.69857E+00_JPRB, 0.74599E+00_JPRB ,&
&  0.81171E+00_JPRB, 0.86740E+00_JPRB, 0.82527E+00_JPRB, 0.63381E+00_JPRB, 0.61303E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.59328E+00_JPRB, 0.55835E+00_JPRB, 0.50820E+00_JPRB, 0.47745E+00_JPRB, 0.50175E+00_JPRB, 0.56498E+00_JPRB, 0.61322E+00_JPRB ,&
&  0.62938E+00_JPRB, 0.62439E+00_JPRB, 0.61727E+00_JPRB, 0.61185E+00_JPRB, 0.61871E+00_JPRB, 0.60838E+00_JPRB, 0.61102E+00_JPRB ,&
&  0.61680E+00_JPRB, 0.61553E+00_JPRB, 0.62038E+00_JPRB, 0.62622E+00_JPRB, 0.63460E+00_JPRB, 0.65451E+00_JPRB, 0.67580E+00_JPRB ,&
&  0.69278E+00_JPRB, 0.70781E+00_JPRB, 0.73112E+00_JPRB, 0.76311E+00_JPRB, 0.81342E+00_JPRB, 0.90568E+00_JPRB, 0.10698E+01_JPRB ,&
&  0.13818E+01_JPRB, 0.95555E+00_JPRB, 0.58669E+00_JPRB, 0.99907E+00_JPRB, 0.10520E+01_JPRB, 0.97336E+00_JPRB, 0.85218E+00_JPRB ,&
&  0.82113E+00_JPRB, 0.79114E+00_JPRB, 0.81525E+00_JPRB, 0.91788E+00_JPRB, 0.99117E+00_JPRB, 0.10941E+01_JPRB, 0.11573E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11685E+01_JPRB, 0.11760E+01_JPRB, 0.11906E+01_JPRB, 0.12190E+01_JPRB, 0.12546E+01_JPRB, 0.12602E+01_JPRB ,&
&  0.12579E+01_JPRB, 0.12591E+01_JPRB, 0.12072E+01_JPRB, 0.94487E+00_JPRB, 0.86277E+00_JPRB, 0.81828E+00_JPRB, 0.79371E+00_JPRB ,&
&  0.74394E+00_JPRB, 0.75674E+00_JPRB, 0.75748E+00_JPRB, 0.71513E+00_JPRB, 0.66773E+00_JPRB, 0.63335E+00_JPRB, 0.66779E+00_JPRB ,&
&  0.66983E+00_JPRB, 0.63749E+00_JPRB, 0.61035E+00_JPRB, 0.59581E+00_JPRB, 0.61839E+00_JPRB, 0.63827E+00_JPRB, 0.63565E+00_JPRB ,&
&  0.64729E+00_JPRB, 0.66643E+00_JPRB, 0.68068E+00_JPRB, 0.69472E+00_JPRB, 0.71551E+00_JPRB, 0.73730E+00_JPRB, 0.74921E+00_JPRB ,&
&  0.76037E+00_JPRB, 0.76114E+00_JPRB, 0.76523E+00_JPRB, 0.75897E+00_JPRB, 0.72676E+00_JPRB, 0.69272E+00_JPRB, 0.69692E+00_JPRB ,&
&  0.69498E+00_JPRB, 0.70090E+00_JPRB, 0.69283E+00_JPRB, 0.70602E+00_JPRB, 0.71708E+00_JPRB, 0.74573E+00_JPRB, 0.78032E+00_JPRB ,&
&  0.80885E+00_JPRB, 0.79638E+00_JPRB, 0.71677E+00_JPRB, 0.62514E+00_JPRB, 0.60639E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.55438E+00_JPRB, 0.49241E+00_JPRB, 0.44284E+00_JPRB, 0.45994E+00_JPRB, 0.52198E+00_JPRB, 0.57393E+00_JPRB, 0.59998E+00_JPRB ,&
&  0.62895E+00_JPRB, 0.63424E+00_JPRB, 0.63756E+00_JPRB, 0.64310E+00_JPRB, 0.65759E+00_JPRB, 0.65449E+00_JPRB, 0.66207E+00_JPRB ,&
&  0.65785E+00_JPRB, 0.64773E+00_JPRB, 0.64492E+00_JPRB, 0.65989E+00_JPRB, 0.68063E+00_JPRB, 0.70356E+00_JPRB, 0.72884E+00_JPRB ,&
&  0.75689E+00_JPRB, 0.78513E+00_JPRB, 0.82750E+00_JPRB, 0.87900E+00_JPRB, 0.94690E+00_JPRB, 0.10196E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.78440E+00_JPRB, 0.47456E+00_JPRB, 0.83961E+00_JPRB, 0.10298E+01_JPRB, 0.95902E+00_JPRB, 0.91212E+00_JPRB, 0.87220E+00_JPRB ,&
&  0.83974E+00_JPRB, 0.83910E+00_JPRB, 0.86642E+00_JPRB, 0.10789E+01_JPRB, 0.12035E+01_JPRB, 0.12552E+01_JPRB, 0.12768E+01_JPRB ,&
&  0.12917E+01_JPRB, 0.12999E+01_JPRB, 0.13159E+01_JPRB, 0.13453E+01_JPRB, 0.13777E+01_JPRB, 0.14012E+01_JPRB, 0.14066E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.13935E+01_JPRB, 0.13431E+01_JPRB, 0.10262E+01_JPRB, 0.93607E+00_JPRB, 0.84906E+00_JPRB, 0.80045E+00_JPRB ,&
&  0.75462E+00_JPRB, 0.68040E+00_JPRB, 0.67236E+00_JPRB, 0.70409E+00_JPRB, 0.67458E+00_JPRB, 0.60362E+00_JPRB, 0.60938E+00_JPRB ,&
&  0.62287E+00_JPRB, 0.60979E+00_JPRB, 0.58191E+00_JPRB, 0.56811E+00_JPRB, 0.59612E+00_JPRB, 0.59833E+00_JPRB, 0.60568E+00_JPRB ,&
&  0.60191E+00_JPRB, 0.62784E+00_JPRB, 0.65307E+00_JPRB, 0.67286E+00_JPRB, 0.69460E+00_JPRB, 0.71958E+00_JPRB, 0.72063E+00_JPRB ,&
&  0.72738E+00_JPRB, 0.72351E+00_JPRB, 0.72828E+00_JPRB, 0.71613E+00_JPRB, 0.70438E+00_JPRB, 0.67612E+00_JPRB, 0.67066E+00_JPRB ,&
&  0.67613E+00_JPRB, 0.68385E+00_JPRB, 0.68532E+00_JPRB, 0.68104E+00_JPRB, 0.71018E+00_JPRB, 0.71422E+00_JPRB, 0.72512E+00_JPRB ,&
&  0.72341E+00_JPRB, 0.70349E+00_JPRB, 0.64707E+00_JPRB, 0.60481E+00_JPRB, 0.58833E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.46482E+00_JPRB, 0.41129E+00_JPRB, 0.40612E+00_JPRB, 0.44725E+00_JPRB, 0.48882E+00_JPRB, 0.53092E+00_JPRB, 0.56810E+00_JPRB ,&
&  0.60407E+00_JPRB, 0.62231E+00_JPRB, 0.66016E+00_JPRB, 0.67303E+00_JPRB, 0.67983E+00_JPRB, 0.67171E+00_JPRB, 0.66235E+00_JPRB ,&
&  0.66996E+00_JPRB, 0.67699E+00_JPRB, 0.68112E+00_JPRB, 0.69911E+00_JPRB, 0.71798E+00_JPRB, 0.73984E+00_JPRB, 0.76960E+00_JPRB ,&
&  0.79891E+00_JPRB, 0.83340E+00_JPRB, 0.86691E+00_JPRB, 0.90667E+00_JPRB, 0.93005E+00_JPRB, 0.92824E+00_JPRB, 0.86013E+00_JPRB ,&
&  0.48575E+00_JPRB, 0.79878E+00_JPRB, 0.99868E+00_JPRB, 0.98190E+00_JPRB, 0.94737E+00_JPRB, 0.91258E+00_JPRB, 0.92438E+00_JPRB ,&
&  0.90038E+00_JPRB, 0.86093E+00_JPRB, 0.91273E+00_JPRB, 0.11929E+01_JPRB, 0.12594E+01_JPRB, 0.12975E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.14133E+01_JPRB, 0.14512E+01_JPRB, 0.14911E+01_JPRB, 0.15448E+01_JPRB, 0.15922E+01_JPRB, 0.16340E+01_JPRB ,&
&  0.16608E+01_JPRB, 0.16737E+01_JPRB, 0.16708E+01_JPRB, 0.13388E+01_JPRB, 0.12907E+01_JPRB, 0.11246E+01_JPRB, 0.99853E+00_JPRB ,&
&  0.75092E+00_JPRB, 0.65325E+00_JPRB, 0.67030E+00_JPRB, 0.72426E+00_JPRB, 0.67914E+00_JPRB, 0.64572E+00_JPRB, 0.59385E+00_JPRB ,&
&  0.59893E+00_JPRB, 0.59939E+00_JPRB, 0.59823E+00_JPRB, 0.58363E+00_JPRB, 0.58409E+00_JPRB, 0.55999E+00_JPRB, 0.59504E+00_JPRB ,&
&  0.61053E+00_JPRB, 0.60030E+00_JPRB, 0.60267E+00_JPRB, 0.62520E+00_JPRB, 0.65547E+00_JPRB, 0.68542E+00_JPRB, 0.69222E+00_JPRB ,&
&  0.68297E+00_JPRB, 0.66702E+00_JPRB, 0.66768E+00_JPRB, 0.66841E+00_JPRB, 0.66769E+00_JPRB, 0.66669E+00_JPRB, 0.65642E+00_JPRB ,&
&  0.64901E+00_JPRB, 0.66611E+00_JPRB, 0.70713E+00_JPRB, 0.66356E+00_JPRB, 0.71174E+00_JPRB, 0.64730E+00_JPRB, 0.63117E+00_JPRB ,&
&  0.61485E+00_JPRB, 0.59579E+00_JPRB, 0.55054E+00_JPRB, 0.52837E+00_JPRB, 0.49987E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.37627E+00_JPRB, 0.35903E+00_JPRB, 0.36827E+00_JPRB, 0.39182E+00_JPRB, 0.41970E+00_JPRB, 0.46144E+00_JPRB, 0.51060E+00_JPRB ,&
&  0.55169E+00_JPRB, 0.58978E+00_JPRB, 0.65534E+00_JPRB, 0.66349E+00_JPRB, 0.66717E+00_JPRB, 0.69452E+00_JPRB, 0.70569E+00_JPRB ,&
&  0.71663E+00_JPRB, 0.71636E+00_JPRB, 0.72180E+00_JPRB, 0.72830E+00_JPRB, 0.74937E+00_JPRB, 0.77384E+00_JPRB, 0.78994E+00_JPRB ,&
&  0.81671E+00_JPRB, 0.84800E+00_JPRB, 0.89962E+00_JPRB, 0.95172E+00_JPRB, 0.99761E+00_JPRB, 0.98788E+00_JPRB, 0.96158E+00_JPRB ,&
&  0.72384E+00_JPRB, 0.96762E+00_JPRB, 0.97276E+00_JPRB, 0.96039E+00_JPRB, 0.92808E+00_JPRB, 0.89665E+00_JPRB, 0.89908E+00_JPRB ,&
&  0.90691E+00_JPRB, 0.87568E+00_JPRB, 0.90804E+00_JPRB, 0.10355E+01_JPRB, 0.12281E+01_JPRB, 0.12729E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.14019E+01_JPRB, 0.14605E+01_JPRB, 0.15248E+01_JPRB, 0.16025E+01_JPRB, 0.16833E+01_JPRB, 0.17595E+01_JPRB, 0.18376E+01_JPRB ,&
&  0.19038E+01_JPRB, 0.19613E+01_JPRB, 0.20485E+01_JPRB, 0.17224E+01_JPRB, 0.14436E+01_JPRB, 0.12004E+01_JPRB, 0.89678E+00_JPRB ,&
&  0.70192E+00_JPRB, 0.62983E+00_JPRB, 0.67549E+00_JPRB, 0.73179E+00_JPRB, 0.68619E+00_JPRB, 0.64936E+00_JPRB, 0.59783E+00_JPRB ,&
&  0.58956E+00_JPRB, 0.59093E+00_JPRB, 0.59510E+00_JPRB, 0.58757E+00_JPRB, 0.59414E+00_JPRB, 0.57864E+00_JPRB, 0.58121E+00_JPRB ,&
&  0.59598E+00_JPRB, 0.58757E+00_JPRB, 0.58321E+00_JPRB, 0.59422E+00_JPRB, 0.62264E+00_JPRB, 0.65533E+00_JPRB, 0.65685E+00_JPRB ,&
&  0.66104E+00_JPRB, 0.59808E+00_JPRB, 0.57550E+00_JPRB, 0.59228E+00_JPRB, 0.59747E+00_JPRB, 0.60254E+00_JPRB, 0.62751E+00_JPRB ,&
&  0.71401E+00_JPRB, 0.82059E+00_JPRB, 0.91592E+00_JPRB, 0.91390E+00_JPRB, 0.96992E+00_JPRB, 0.75818E+00_JPRB, 0.55879E+00_JPRB ,&
&  0.52351E+00_JPRB, 0.51130E+00_JPRB, 0.46838E+00_JPRB, 0.43906E+00_JPRB, 0.40569E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.37005E+00_JPRB, 0.36401E+00_JPRB, 0.37305E+00_JPRB, 0.38940E+00_JPRB, 0.40123E+00_JPRB, 0.42860E+00_JPRB, 0.46129E+00_JPRB ,&
&  0.49687E+00_JPRB, 0.53996E+00_JPRB, 0.57258E+00_JPRB, 0.59058E+00_JPRB, 0.62557E+00_JPRB, 0.67817E+00_JPRB, 0.69313E+00_JPRB ,&
&  0.73179E+00_JPRB, 0.74897E+00_JPRB, 0.74819E+00_JPRB, 0.74428E+00_JPRB, 0.75666E+00_JPRB, 0.78458E+00_JPRB, 0.81200E+00_JPRB ,&
&  0.83074E+00_JPRB, 0.86099E+00_JPRB, 0.90945E+00_JPRB, 0.93606E+00_JPRB, 0.93852E+00_JPRB, 0.91214E+00_JPRB, 0.71458E+00_JPRB ,&
&  0.94641E+00_JPRB, 0.96723E+00_JPRB, 0.96996E+00_JPRB, 0.93598E+00_JPRB, 0.90645E+00_JPRB, 0.87915E+00_JPRB, 0.87592E+00_JPRB ,&
&  0.88251E+00_JPRB, 0.87407E+00_JPRB, 0.86699E+00_JPRB, 0.90590E+00_JPRB, 0.10932E+01_JPRB, 0.11533E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.13582E+01_JPRB, 0.14342E+01_JPRB, 0.15188E+01_JPRB, 0.16032E+01_JPRB, 0.16739E+01_JPRB, 0.17366E+01_JPRB ,&
&  0.17964E+01_JPRB, 0.18600E+01_JPRB, 0.19164E+01_JPRB, 0.16512E+01_JPRB, 0.13734E+01_JPRB, 0.11809E+01_JPRB, 0.88077E+00_JPRB ,&
&  0.67806E+00_JPRB, 0.62907E+00_JPRB, 0.67418E+00_JPRB, 0.71620E+00_JPRB, 0.66451E+00_JPRB, 0.62558E+00_JPRB, 0.58967E+00_JPRB ,&
&  0.58769E+00_JPRB, 0.59828E+00_JPRB, 0.61409E+00_JPRB, 0.61796E+00_JPRB, 0.59842E+00_JPRB, 0.57875E+00_JPRB, 0.56532E+00_JPRB ,&
&  0.58280E+00_JPRB, 0.58072E+00_JPRB, 0.57379E+00_JPRB, 0.59704E+00_JPRB, 0.60999E+00_JPRB, 0.68405E+00_JPRB, 0.79633E+00_JPRB ,&
&  0.79122E+00_JPRB, 0.50740E+00_JPRB, 0.51543E+00_JPRB, 0.53125E+00_JPRB, 0.57206E+00_JPRB, 0.64713E+00_JPRB, 0.80331E+00_JPRB ,&
&  0.11538E+01_JPRB, 0.19221E+01_JPRB, 0.22019E+01_JPRB, 0.14549E+01_JPRB, 0.19842E+01_JPRB, 0.12811E+01_JPRB, 0.50385E+00_JPRB ,&
&  0.47594E+00_JPRB, 0.46115E+00_JPRB, 0.43347E+00_JPRB, 0.40431E+00_JPRB, 0.38468E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.44727E+00_JPRB, 0.44850E+00_JPRB, 0.45550E+00_JPRB, 0.46012E+00_JPRB, 0.46485E+00_JPRB, 0.46761E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.48325E+00_JPRB, 0.49015E+00_JPRB, 0.50820E+00_JPRB, 0.51994E+00_JPRB, 0.53024E+00_JPRB, 0.55597E+00_JPRB, 0.57525E+00_JPRB ,&
&  0.60827E+00_JPRB, 0.65054E+00_JPRB, 0.66260E+00_JPRB, 0.68345E+00_JPRB, 0.69295E+00_JPRB, 0.70069E+00_JPRB, 0.70235E+00_JPRB ,&
&  0.74411E+00_JPRB, 0.82598E+00_JPRB, 0.87448E+00_JPRB, 0.92572E+00_JPRB, 0.91060E+00_JPRB, 0.91107E+00_JPRB, 0.73257E+00_JPRB ,&
&  0.99190E+00_JPRB, 0.98622E+00_JPRB, 0.91884E+00_JPRB, 0.87995E+00_JPRB, 0.85422E+00_JPRB, 0.82706E+00_JPRB, 0.79902E+00_JPRB ,&
&  0.79911E+00_JPRB, 0.83399E+00_JPRB, 0.83166E+00_JPRB, 0.88806E+00_JPRB, 0.96462E+00_JPRB, 0.97663E+00_JPRB, 0.10542E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.11250E+01_JPRB, 0.11671E+01_JPRB, 0.12350E+01_JPRB, 0.12847E+01_JPRB, 0.13141E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.13161E+01_JPRB, 0.13361E+01_JPRB, 0.13935E+01_JPRB, 0.13638E+01_JPRB, 0.11877E+01_JPRB, 0.10483E+01_JPRB, 0.88308E+00_JPRB ,&
&  0.68230E+00_JPRB, 0.61020E+00_JPRB, 0.62790E+00_JPRB, 0.70130E+00_JPRB, 0.66875E+00_JPRB, 0.63383E+00_JPRB, 0.60689E+00_JPRB ,&
&  0.58559E+00_JPRB, 0.59014E+00_JPRB, 0.59734E+00_JPRB, 0.59368E+00_JPRB, 0.57761E+00_JPRB, 0.54926E+00_JPRB, 0.52336E+00_JPRB ,&
&  0.52093E+00_JPRB, 0.51330E+00_JPRB, 0.51017E+00_JPRB, 0.51851E+00_JPRB, 0.52550E+00_JPRB, 0.55384E+00_JPRB, 0.52531E+00_JPRB ,&
&  0.57684E+00_JPRB, 0.46568E+00_JPRB, 0.48631E+00_JPRB, 0.52805E+00_JPRB, 0.58303E+00_JPRB, 0.81322E+00_JPRB, 0.12146E+01_JPRB ,&
&  0.18104E+01_JPRB, 0.20788E+01_JPRB, 0.27660E+01_JPRB, 0.40686E+01_JPRB, 0.25564E+01_JPRB, 0.16235E+01_JPRB, 0.51720E+00_JPRB ,&
&  0.50591E+00_JPRB, 0.48037E+00_JPRB, 0.47257E+00_JPRB, 0.45777E+00_JPRB, 0.45323E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.50967E+00_JPRB, 0.51800E+00_JPRB, 0.52770E+00_JPRB, 0.53627E+00_JPRB, 0.54151E+00_JPRB, 0.55108E+00_JPRB, 0.55767E+00_JPRB ,&
&  0.56536E+00_JPRB, 0.56733E+00_JPRB, 0.57257E+00_JPRB, 0.57650E+00_JPRB, 0.58383E+00_JPRB, 0.59543E+00_JPRB, 0.60667E+00_JPRB ,&
&  0.61429E+00_JPRB, 0.62671E+00_JPRB, 0.64222E+00_JPRB, 0.65049E+00_JPRB, 0.67524E+00_JPRB, 0.72449E+00_JPRB, 0.77849E+00_JPRB ,&
&  0.81641E+00_JPRB, 0.86445E+00_JPRB, 0.90718E+00_JPRB, 0.91509E+00_JPRB, 0.89284E+00_JPRB, 0.87177E+00_JPRB, 0.73289E+00_JPRB ,&
&  0.10945E+01_JPRB, 0.10791E+01_JPRB, 0.96879E+00_JPRB, 0.90194E+00_JPRB, 0.84348E+00_JPRB, 0.79036E+00_JPRB, 0.73901E+00_JPRB ,&
&  0.71024E+00_JPRB, 0.78088E+00_JPRB, 0.78554E+00_JPRB, 0.80352E+00_JPRB, 0.77913E+00_JPRB, 0.78975E+00_JPRB, 0.86031E+00_JPRB ,&
&  0.93635E+00_JPRB, 0.93284E+00_JPRB, 0.88881E+00_JPRB, 0.90672E+00_JPRB, 0.93555E+00_JPRB, 0.94027E+00_JPRB, 0.94669E+00_JPRB ,&
&  0.90885E+00_JPRB, 0.87041E+00_JPRB, 0.79151E+00_JPRB, 0.88383E+00_JPRB, 0.85598E+00_JPRB, 0.80095E+00_JPRB, 0.73441E+00_JPRB ,&
&  0.57962E+00_JPRB, 0.58195E+00_JPRB, 0.62954E+00_JPRB, 0.72873E+00_JPRB, 0.69367E+00_JPRB, 0.66586E+00_JPRB, 0.62776E+00_JPRB ,&
&  0.59554E+00_JPRB, 0.58069E+00_JPRB, 0.55656E+00_JPRB, 0.54481E+00_JPRB, 0.53945E+00_JPRB, 0.52187E+00_JPRB, 0.49168E+00_JPRB ,&
&  0.46827E+00_JPRB, 0.46363E+00_JPRB, 0.45687E+00_JPRB, 0.45209E+00_JPRB, 0.45077E+00_JPRB, 0.40570E+00_JPRB, 0.50223E+00_JPRB ,&
&  0.47378E+00_JPRB, 0.41703E+00_JPRB, 0.52691E+00_JPRB, 0.38950E+00_JPRB, 0.50404E+00_JPRB, 0.62099E+00_JPRB, 0.81310E+00_JPRB ,&
&  0.86495E+00_JPRB, 0.11192E+01_JPRB, 0.14690E+01_JPRB, 0.11591E+01_JPRB, 0.87287E+00_JPRB, 0.57644E+00_JPRB, 0.54022E+00_JPRB ,&
&  0.52869E+00_JPRB, 0.51836E+00_JPRB, 0.52085E+00_JPRB, 0.51364E+00_JPRB, 0.51166E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.57844E+00_JPRB, 0.59018E+00_JPRB, 0.60449E+00_JPRB, 0.62322E+00_JPRB, 0.63447E+00_JPRB, 0.65129E+00_JPRB, 0.66379E+00_JPRB ,&
&  0.68149E+00_JPRB, 0.69421E+00_JPRB, 0.70427E+00_JPRB, 0.71470E+00_JPRB, 0.72954E+00_JPRB, 0.74955E+00_JPRB, 0.77549E+00_JPRB ,&
&  0.80347E+00_JPRB, 0.83126E+00_JPRB, 0.86215E+00_JPRB, 0.89763E+00_JPRB, 0.93892E+00_JPRB, 0.98977E+00_JPRB, 0.10369E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.11216E+01_JPRB, 0.11609E+01_JPRB, 0.11902E+01_JPRB, 0.11934E+01_JPRB, 0.11823E+01_JPRB, 0.10142E+01_JPRB ,&
&  0.12755E+01_JPRB, 0.12541E+01_JPRB, 0.11578E+01_JPRB, 0.10497E+01_JPRB, 0.95918E+00_JPRB, 0.88065E+00_JPRB, 0.80490E+00_JPRB ,&
&  0.79704E+00_JPRB, 0.74899E+00_JPRB, 0.72491E+00_JPRB, 0.71927E+00_JPRB, 0.68092E+00_JPRB, 0.67295E+00_JPRB, 0.69429E+00_JPRB ,&
&  0.73372E+00_JPRB, 0.80361E+00_JPRB, 0.83134E+00_JPRB, 0.86726E+00_JPRB, 0.92888E+00_JPRB, 0.88954E+00_JPRB, 0.88364E+00_JPRB ,&
&  0.81452E+00_JPRB, 0.72733E+00_JPRB, 0.59766E+00_JPRB, 0.66486E+00_JPRB, 0.71019E+00_JPRB, 0.73727E+00_JPRB, 0.73828E+00_JPRB ,&
&  0.62610E+00_JPRB, 0.55441E+00_JPRB, 0.59628E+00_JPRB, 0.78638E+00_JPRB, 0.76646E+00_JPRB, 0.70046E+00_JPRB, 0.64682E+00_JPRB ,&
&  0.59605E+00_JPRB, 0.55906E+00_JPRB, 0.51890E+00_JPRB, 0.50145E+00_JPRB, 0.51496E+00_JPRB, 0.50893E+00_JPRB, 0.48293E+00_JPRB ,&
&  0.44719E+00_JPRB, 0.42901E+00_JPRB, 0.42295E+00_JPRB, 0.41711E+00_JPRB, 0.42576E+00_JPRB, 0.37744E+00_JPRB, 0.43607E+00_JPRB ,&
&  0.44453E+00_JPRB, 0.32961E+00_JPRB, 0.41721E+00_JPRB, 0.53313E+00_JPRB, 0.53015E+00_JPRB, 0.58022E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.64759E+00_JPRB, 0.65715E+00_JPRB, 0.61271E+00_JPRB, 0.60885E+00_JPRB, 0.59714E+00_JPRB, 0.58506E+00_JPRB, 0.57126E+00_JPRB ,&
&  0.55860E+00_JPRB, 0.55929E+00_JPRB, 0.57270E+00_JPRB, 0.57229E+00_JPRB, 0.57276E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.58689E+00_JPRB, 0.59868E+00_JPRB, 0.59879E+00_JPRB, 0.60458E+00_JPRB, 0.61706E+00_JPRB, 0.62528E+00_JPRB, 0.63775E+00_JPRB ,&
&  0.64993E+00_JPRB, 0.65384E+00_JPRB, 0.65721E+00_JPRB, 0.66226E+00_JPRB, 0.66459E+00_JPRB, 0.68056E+00_JPRB, 0.69645E+00_JPRB ,&
&  0.71761E+00_JPRB, 0.73359E+00_JPRB, 0.75181E+00_JPRB, 0.78075E+00_JPRB, 0.81731E+00_JPRB, 0.87480E+00_JPRB, 0.93577E+00_JPRB ,&
&  0.10150E+01_JPRB, 0.11084E+01_JPRB, 0.12189E+01_JPRB, 0.13063E+01_JPRB, 0.13478E+01_JPRB, 0.13597E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.13480E+01_JPRB, 0.12610E+01_JPRB, 0.11108E+01_JPRB, 0.10868E+01_JPRB, 0.10088E+01_JPRB, 0.97337E+00_JPRB ,&
&  0.96456E+00_JPRB, 0.91309E+00_JPRB, 0.88708E+00_JPRB, 0.86268E+00_JPRB, 0.83883E+00_JPRB, 0.79220E+00_JPRB, 0.75016E+00_JPRB ,&
&  0.73911E+00_JPRB, 0.77000E+00_JPRB, 0.85022E+00_JPRB, 0.10232E+01_JPRB, 0.11954E+01_JPRB, 0.12495E+01_JPRB, 0.12008E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.10912E+01_JPRB, 0.10080E+01_JPRB, 0.96791E+00_JPRB, 0.10093E+01_JPRB, 0.10438E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.89012E+00_JPRB, 0.76394E+00_JPRB, 0.73503E+00_JPRB, 0.79633E+00_JPRB, 0.80427E+00_JPRB, 0.77532E+00_JPRB, 0.68451E+00_JPRB ,&
&  0.59245E+00_JPRB, 0.53724E+00_JPRB, 0.50501E+00_JPRB, 0.50800E+00_JPRB, 0.55135E+00_JPRB, 0.55633E+00_JPRB, 0.53443E+00_JPRB ,&
&  0.48385E+00_JPRB, 0.45111E+00_JPRB, 0.42772E+00_JPRB, 0.41251E+00_JPRB, 0.36518E+00_JPRB, 0.38819E+00_JPRB, 0.40798E+00_JPRB ,&
&  0.42773E+00_JPRB, 0.38851E+00_JPRB, 0.42196E+00_JPRB, 0.57091E+00_JPRB, 0.60436E+00_JPRB, 0.61881E+00_JPRB, 0.61825E+00_JPRB ,&
&  0.60908E+00_JPRB, 0.60759E+00_JPRB, 0.60420E+00_JPRB, 0.58727E+00_JPRB, 0.57440E+00_JPRB, 0.56941E+00_JPRB, 0.59075E+00_JPRB ,&
&  0.58599E+00_JPRB, 0.58582E+00_JPRB, 0.58235E+00_JPRB, 0.59415E+00_JPRB, 0.59101E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.60226E+00_JPRB, 0.60681E+00_JPRB, 0.61632E+00_JPRB, 0.63372E+00_JPRB, 0.63093E+00_JPRB, 0.62803E+00_JPRB, 0.65344E+00_JPRB ,&
&  0.63690E+00_JPRB, 0.62413E+00_JPRB, 0.62080E+00_JPRB, 0.59462E+00_JPRB, 0.60971E+00_JPRB, 0.62340E+00_JPRB, 0.65614E+00_JPRB ,&
&  0.68995E+00_JPRB, 0.70986E+00_JPRB, 0.72761E+00_JPRB, 0.75522E+00_JPRB, 0.80806E+00_JPRB, 0.84822E+00_JPRB, 0.88045E+00_JPRB ,&
&  0.94101E+00_JPRB, 0.10377E+01_JPRB, 0.11461E+01_JPRB, 0.14012E+01_JPRB, 0.19932E+01_JPRB, 0.24517E+01_JPRB, 0.24191E+01_JPRB ,&
&  0.17835E+01_JPRB, 0.12737E+01_JPRB, 0.11974E+01_JPRB, 0.10763E+01_JPRB, 0.11073E+01_JPRB, 0.12032E+01_JPRB, 0.12160E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.11881E+01_JPRB, 0.11797E+01_JPRB, 0.11756E+01_JPRB, 0.11835E+01_JPRB, 0.11866E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.10902E+01_JPRB, 0.11226E+01_JPRB, 0.13295E+01_JPRB, 0.14773E+01_JPRB, 0.17422E+01_JPRB, 0.17413E+01_JPRB ,&
&  0.16897E+01_JPRB, 0.15477E+01_JPRB, 0.14791E+01_JPRB, 0.14474E+01_JPRB, 0.14086E+01_JPRB, 0.13816E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.11841E+01_JPRB, 0.79330E+00_JPRB, 0.84174E+00_JPRB, 0.84839E+00_JPRB, 0.86060E+00_JPRB, 0.70909E+00_JPRB ,&
&  0.58832E+00_JPRB, 0.55609E+00_JPRB, 0.55146E+00_JPRB, 0.57367E+00_JPRB, 0.62685E+00_JPRB, 0.66413E+00_JPRB, 0.62595E+00_JPRB ,&
&  0.56033E+00_JPRB, 0.49612E+00_JPRB, 0.45479E+00_JPRB, 0.43596E+00_JPRB, 0.42888E+00_JPRB, 0.36068E+00_JPRB, 0.44392E+00_JPRB ,&
&  0.48799E+00_JPRB, 0.54717E+00_JPRB, 0.52082E+00_JPRB, 0.69176E+00_JPRB, 0.72404E+00_JPRB, 0.71618E+00_JPRB, 0.70420E+00_JPRB ,&
&  0.69599E+00_JPRB, 0.67826E+00_JPRB, 0.66137E+00_JPRB, 0.64088E+00_JPRB, 0.61976E+00_JPRB, 0.64166E+00_JPRB, 0.67194E+00_JPRB ,&
&  0.64315E+00_JPRB, 0.65889E+00_JPRB, 0.64355E+00_JPRB, 0.61947E+00_JPRB, 0.61922E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.63719E+00_JPRB, 0.62806E+00_JPRB, 0.64083E+00_JPRB, 0.66244E+00_JPRB, 0.67404E+00_JPRB, 0.68491E+00_JPRB, 0.69608E+00_JPRB ,&
&  0.69961E+00_JPRB, 0.70189E+00_JPRB, 0.70640E+00_JPRB, 0.70761E+00_JPRB, 0.73902E+00_JPRB, 0.77338E+00_JPRB, 0.81664E+00_JPRB ,&
&  0.88123E+00_JPRB, 0.93889E+00_JPRB, 0.98995E+00_JPRB, 0.10869E+01_JPRB, 0.11536E+01_JPRB, 0.12157E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.14650E+01_JPRB, 0.16152E+01_JPRB, 0.17654E+01_JPRB, 0.19155E+01_JPRB, 0.20899E+01_JPRB, 0.22427E+01_JPRB, 0.22327E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.12770E+01_JPRB, 0.13644E+01_JPRB, 0.12791E+01_JPRB, 0.13098E+01_JPRB, 0.13282E+01_JPRB, 0.13249E+01_JPRB ,&
&  0.13401E+01_JPRB, 0.13445E+01_JPRB, 0.13464E+01_JPRB, 0.13522E+01_JPRB, 0.13928E+01_JPRB, 0.14521E+01_JPRB, 0.15335E+01_JPRB ,&
&  0.15678E+01_JPRB, 0.15974E+01_JPRB, 0.16182E+01_JPRB, 0.16051E+01_JPRB, 0.16671E+01_JPRB, 0.18172E+01_JPRB, 0.19471E+01_JPRB ,&
&  0.20198E+01_JPRB, 0.20282E+01_JPRB, 0.18006E+01_JPRB, 0.17600E+01_JPRB, 0.17455E+01_JPRB, 0.16200E+01_JPRB, 0.16601E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.14975E+01_JPRB, 0.93726E+00_JPRB, 0.88180E+00_JPRB, 0.86657E+00_JPRB, 0.81799E+00_JPRB, 0.74203E+00_JPRB ,&
&  0.61537E+00_JPRB, 0.59527E+00_JPRB, 0.61669E+00_JPRB, 0.69279E+00_JPRB, 0.73934E+00_JPRB, 0.76651E+00_JPRB, 0.70991E+00_JPRB ,&
&  0.61309E+00_JPRB, 0.54187E+00_JPRB, 0.48124E+00_JPRB, 0.46453E+00_JPRB, 0.43851E+00_JPRB, 0.44958E+00_JPRB, 0.50487E+00_JPRB ,&
&  0.56800E+00_JPRB, 0.63602E+00_JPRB, 0.72687E+00_JPRB, 0.74309E+00_JPRB, 0.71930E+00_JPRB, 0.82284E+00_JPRB, 0.79869E+00_JPRB ,&
&  0.77865E+00_JPRB, 0.77227E+00_JPRB, 0.75877E+00_JPRB, 0.74770E+00_JPRB, 0.73701E+00_JPRB, 0.73541E+00_JPRB, 0.71878E+00_JPRB ,&
&  0.71222E+00_JPRB, 0.69821E+00_JPRB, 0.69080E+00_JPRB, 0.66508E+00_JPRB, 0.65111E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.66323E+00_JPRB, 0.66133E+00_JPRB, 0.66377E+00_JPRB, 0.68061E+00_JPRB, 0.70400E+00_JPRB, 0.71930E+00_JPRB, 0.72481E+00_JPRB ,&
&  0.74116E+00_JPRB, 0.76386E+00_JPRB, 0.79745E+00_JPRB, 0.82733E+00_JPRB, 0.86578E+00_JPRB, 0.90142E+00_JPRB, 0.95796E+00_JPRB ,&
&  0.10113E+01_JPRB, 0.10749E+01_JPRB, 0.11865E+01_JPRB, 0.13528E+01_JPRB, 0.14915E+01_JPRB, 0.16097E+01_JPRB, 0.17931E+01_JPRB ,&
&  0.19725E+01_JPRB, 0.20652E+01_JPRB, 0.21814E+01_JPRB, 0.22163E+01_JPRB, 0.21001E+01_JPRB, 0.19631E+01_JPRB, 0.19792E+01_JPRB ,&
&  0.16289E+01_JPRB, 0.13250E+01_JPRB, 0.13110E+01_JPRB, 0.13320E+01_JPRB, 0.13577E+01_JPRB, 0.13548E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.13750E+01_JPRB, 0.13825E+01_JPRB, 0.13892E+01_JPRB, 0.14015E+01_JPRB, 0.14328E+01_JPRB, 0.14971E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.16038E+01_JPRB, 0.16353E+01_JPRB, 0.17131E+01_JPRB, 0.15802E+01_JPRB, 0.16738E+01_JPRB, 0.18107E+01_JPRB, 0.18818E+01_JPRB ,&
&  0.18699E+01_JPRB, 0.17277E+01_JPRB, 0.17194E+01_JPRB, 0.17251E+01_JPRB, 0.17229E+01_JPRB, 0.16465E+01_JPRB, 0.15912E+01_JPRB ,&
&  0.16806E+01_JPRB, 0.16064E+01_JPRB, 0.10895E+01_JPRB, 0.96866E+00_JPRB, 0.97264E+00_JPRB, 0.79713E+00_JPRB, 0.78272E+00_JPRB ,&
&  0.68220E+00_JPRB, 0.66258E+00_JPRB, 0.72787E+00_JPRB, 0.77825E+00_JPRB, 0.80996E+00_JPRB, 0.87875E+00_JPRB, 0.70488E+00_JPRB ,&
&  0.63725E+00_JPRB, 0.58254E+00_JPRB, 0.53208E+00_JPRB, 0.51390E+00_JPRB, 0.51574E+00_JPRB, 0.55508E+00_JPRB, 0.60032E+00_JPRB ,&
&  0.65515E+00_JPRB, 0.72346E+00_JPRB, 0.78614E+00_JPRB, 0.84279E+00_JPRB, 0.81724E+00_JPRB, 0.85711E+00_JPRB, 0.83111E+00_JPRB ,&
&  0.79917E+00_JPRB, 0.79370E+00_JPRB, 0.78047E+00_JPRB, 0.75750E+00_JPRB, 0.74527E+00_JPRB, 0.73011E+00_JPRB, 0.72271E+00_JPRB ,&
&  0.71968E+00_JPRB, 0.71532E+00_JPRB, 0.70638E+00_JPRB, 0.69667E+00_JPRB, 0.67820E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.69674E+00_JPRB, 0.68675E+00_JPRB, 0.68288E+00_JPRB, 0.69295E+00_JPRB, 0.70919E+00_JPRB, 0.71642E+00_JPRB, 0.72559E+00_JPRB ,&
&  0.74649E+00_JPRB, 0.77252E+00_JPRB, 0.80041E+00_JPRB, 0.83842E+00_JPRB, 0.87697E+00_JPRB, 0.91786E+00_JPRB, 0.98004E+00_JPRB ,&
&  0.10543E+01_JPRB, 0.11569E+01_JPRB, 0.12996E+01_JPRB, 0.14535E+01_JPRB, 0.15924E+01_JPRB, 0.17463E+01_JPRB, 0.18968E+01_JPRB ,&
&  0.20664E+01_JPRB, 0.21158E+01_JPRB, 0.20231E+01_JPRB, 0.18562E+01_JPRB, 0.14635E+01_JPRB, 0.14917E+01_JPRB, 0.16223E+01_JPRB ,&
&  0.16434E+01_JPRB, 0.13893E+01_JPRB, 0.13015E+01_JPRB, 0.12834E+01_JPRB, 0.12898E+01_JPRB, 0.13083E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.13543E+01_JPRB, 0.13715E+01_JPRB, 0.13856E+01_JPRB, 0.14019E+01_JPRB, 0.14422E+01_JPRB, 0.15325E+01_JPRB ,&
&  0.15455E+01_JPRB, 0.15628E+01_JPRB, 0.15745E+01_JPRB, 0.15350E+01_JPRB, 0.15978E+01_JPRB, 0.16762E+01_JPRB, 0.16260E+01_JPRB ,&
&  0.15759E+01_JPRB, 0.14338E+01_JPRB, 0.14119E+01_JPRB, 0.14580E+01_JPRB, 0.14306E+01_JPRB, 0.13130E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.15974E+01_JPRB, 0.10621E+01_JPRB, 0.11234E+01_JPRB, 0.12685E+01_JPRB, 0.10770E+01_JPRB, 0.92167E+00_JPRB ,&
&  0.82424E+00_JPRB, 0.80794E+00_JPRB, 0.87009E+00_JPRB, 0.88218E+00_JPRB, 0.85760E+00_JPRB, 0.92099E+00_JPRB, 0.71075E+00_JPRB ,&
&  0.70169E+00_JPRB, 0.64215E+00_JPRB, 0.60297E+00_JPRB, 0.59790E+00_JPRB, 0.61751E+00_JPRB, 0.63369E+00_JPRB, 0.63419E+00_JPRB ,&
&  0.62719E+00_JPRB, 0.90785E+00_JPRB, 0.11506E+01_JPRB, 0.14283E+01_JPRB, 0.14552E+01_JPRB, 0.86313E+00_JPRB, 0.85977E+00_JPRB ,&
&  0.85098E+00_JPRB, 0.83415E+00_JPRB, 0.81224E+00_JPRB, 0.79283E+00_JPRB, 0.77767E+00_JPRB, 0.76117E+00_JPRB, 0.74816E+00_JPRB ,&
&  0.73933E+00_JPRB, 0.73379E+00_JPRB, 0.72990E+00_JPRB, 0.71828E+00_JPRB, 0.70614E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.72393E+00_JPRB, 0.71885E+00_JPRB, 0.70620E+00_JPRB, 0.70475E+00_JPRB, 0.72770E+00_JPRB, 0.74400E+00_JPRB, 0.73032E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.74682E+00_JPRB, 0.76517E+00_JPRB, 0.79002E+00_JPRB, 0.81520E+00_JPRB, 0.83562E+00_JPRB, 0.88753E+00_JPRB ,&
&  0.94972E+00_JPRB, 0.10667E+01_JPRB, 0.12495E+01_JPRB, 0.14367E+01_JPRB, 0.16097E+01_JPRB, 0.18073E+01_JPRB, 0.19998E+01_JPRB ,&
&  0.22380E+01_JPRB, 0.22921E+01_JPRB, 0.20660E+01_JPRB, 0.13643E+01_JPRB, 0.11474E+01_JPRB, 0.12279E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.12153E+01_JPRB, 0.11982E+01_JPRB, 0.11934E+01_JPRB, 0.12059E+01_JPRB, 0.12294E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.12653E+01_JPRB, 0.12913E+01_JPRB, 0.13214E+01_JPRB, 0.13508E+01_JPRB, 0.13845E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.14323E+01_JPRB, 0.14633E+01_JPRB, 0.14482E+01_JPRB, 0.14109E+01_JPRB, 0.13903E+01_JPRB, 0.13890E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.12423E+01_JPRB, 0.12099E+01_JPRB, 0.12463E+01_JPRB, 0.12685E+01_JPRB, 0.11542E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.13890E+01_JPRB, 0.11646E+01_JPRB, 0.13333E+01_JPRB, 0.88016E+00_JPRB, 0.10423E+01_JPRB, 0.11210E+01_JPRB ,&
&  0.10422E+01_JPRB, 0.10186E+01_JPRB, 0.10195E+01_JPRB, 0.10395E+01_JPRB, 0.10352E+01_JPRB, 0.90208E+00_JPRB, 0.82945E+00_JPRB ,&
&  0.78620E+00_JPRB, 0.69858E+00_JPRB, 0.63085E+00_JPRB, 0.64831E+00_JPRB, 0.68456E+00_JPRB, 0.67598E+00_JPRB, 0.69714E+00_JPRB ,&
&  0.62647E+00_JPRB, 0.78734E+00_JPRB, 0.94167E+00_JPRB, 0.11315E+01_JPRB, 0.11264E+01_JPRB, 0.86205E+00_JPRB, 0.87060E+00_JPRB ,&
&  0.87853E+00_JPRB, 0.85778E+00_JPRB, 0.84073E+00_JPRB, 0.81144E+00_JPRB, 0.79854E+00_JPRB, 0.78316E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.76179E+00_JPRB, 0.75795E+00_JPRB, 0.75356E+00_JPRB, 0.73885E+00_JPRB, 0.72622E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.75183E+00_JPRB, 0.74774E+00_JPRB, 0.75028E+00_JPRB, 0.75164E+00_JPRB, 0.75208E+00_JPRB, 0.78624E+00_JPRB, 0.79981E+00_JPRB ,&
&  0.76296E+00_JPRB, 0.71121E+00_JPRB, 0.71240E+00_JPRB, 0.71689E+00_JPRB, 0.72491E+00_JPRB, 0.74145E+00_JPRB, 0.77262E+00_JPRB ,&
&  0.80520E+00_JPRB, 0.86338E+00_JPRB, 0.99513E+00_JPRB, 0.12215E+01_JPRB, 0.14810E+01_JPRB, 0.17512E+01_JPRB, 0.20433E+01_JPRB ,&
&  0.19453E+01_JPRB, 0.13912E+01_JPRB, 0.17055E+01_JPRB, 0.16062E+01_JPRB, 0.13709E+01_JPRB, 0.11776E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.11809E+01_JPRB, 0.11726E+01_JPRB, 0.11601E+01_JPRB, 0.11509E+01_JPRB, 0.11166E+01_JPRB, 0.11308E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11562E+01_JPRB, 0.11652E+01_JPRB, 0.11777E+01_JPRB, 0.11994E+01_JPRB, 0.12480E+01_JPRB, 0.13079E+01_JPRB, 0.13423E+01_JPRB ,&
&  0.13479E+01_JPRB, 0.13475E+01_JPRB, 0.12858E+01_JPRB, 0.12332E+01_JPRB, 0.12143E+01_JPRB, 0.12175E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.12137E+01_JPRB, 0.11409E+01_JPRB, 0.11472E+01_JPRB, 0.11866E+01_JPRB, 0.12144E+01_JPRB, 0.11485E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.12539E+01_JPRB, 0.13058E+01_JPRB, 0.12881E+01_JPRB, 0.14504E+01_JPRB, 0.10730E+01_JPRB, 0.11649E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.11988E+01_JPRB, 0.11669E+01_JPRB, 0.11326E+01_JPRB, 0.11329E+01_JPRB, 0.11847E+01_JPRB, 0.10355E+01_JPRB, 0.95708E+00_JPRB ,&
&  0.83126E+00_JPRB, 0.76828E+00_JPRB, 0.68868E+00_JPRB, 0.73061E+00_JPRB, 0.73605E+00_JPRB, 0.72290E+00_JPRB, 0.78603E+00_JPRB ,&
&  0.84046E+00_JPRB, 0.80127E+00_JPRB, 0.80538E+00_JPRB, 0.89090E+00_JPRB, 0.91805E+00_JPRB, 0.83762E+00_JPRB, 0.84463E+00_JPRB ,&
&  0.85310E+00_JPRB, 0.86515E+00_JPRB, 0.84310E+00_JPRB, 0.81675E+00_JPRB, 0.79513E+00_JPRB, 0.77938E+00_JPRB, 0.78032E+00_JPRB ,&
&  0.77522E+00_JPRB, 0.77157E+00_JPRB, 0.76809E+00_JPRB, 0.76092E+00_JPRB, 0.75470E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.75687E+00_JPRB, 0.75004E+00_JPRB, 0.74400E+00_JPRB, 0.75701E+00_JPRB, 0.80300E+00_JPRB, 0.85379E+00_JPRB, 0.99012E+00_JPRB ,&
&  0.12076E+01_JPRB, 0.78489E+00_JPRB, 0.68623E+00_JPRB, 0.67075E+00_JPRB, 0.66419E+00_JPRB, 0.66217E+00_JPRB, 0.68084E+00_JPRB ,&
&  0.70471E+00_JPRB, 0.73248E+00_JPRB, 0.78904E+00_JPRB, 0.91432E+00_JPRB, 0.11505E+01_JPRB, 0.15086E+01_JPRB, 0.16175E+01_JPRB ,&
&  0.14762E+01_JPRB, 0.16608E+01_JPRB, 0.19232E+01_JPRB, 0.15837E+01_JPRB, 0.14276E+01_JPRB, 0.12924E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.11619E+01_JPRB, 0.11206E+01_JPRB, 0.11323E+01_JPRB, 0.11209E+01_JPRB, 0.10957E+01_JPRB, 0.10958E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11181E+01_JPRB, 0.11149E+01_JPRB, 0.11172E+01_JPRB, 0.11362E+01_JPRB, 0.11875E+01_JPRB, 0.12309E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.12783E+01_JPRB, 0.12180E+01_JPRB, 0.11569E+01_JPRB, 0.11450E+01_JPRB, 0.11640E+01_JPRB, 0.11674E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.11414E+01_JPRB, 0.11390E+01_JPRB, 0.11310E+01_JPRB, 0.10853E+01_JPRB, 0.11803E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.12825E+01_JPRB, 0.14698E+01_JPRB, 0.12186E+01_JPRB, 0.12814E+01_JPRB, 0.13351E+01_JPRB, 0.13096E+01_JPRB ,&
&  0.12009E+01_JPRB, 0.12600E+01_JPRB, 0.12766E+01_JPRB, 0.12383E+01_JPRB, 0.12396E+01_JPRB, 0.11031E+01_JPRB, 0.10833E+01_JPRB ,&
&  0.89372E+00_JPRB, 0.91851E+00_JPRB, 0.78615E+00_JPRB, 0.80481E+00_JPRB, 0.79972E+00_JPRB, 0.76063E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.89316E+00_JPRB, 0.88056E+00_JPRB, 0.84828E+00_JPRB, 0.82498E+00_JPRB, 0.79419E+00_JPRB, 0.78282E+00_JPRB, 0.79284E+00_JPRB ,&
&  0.80901E+00_JPRB, 0.85106E+00_JPRB, 0.84011E+00_JPRB, 0.82911E+00_JPRB, 0.81117E+00_JPRB, 0.79581E+00_JPRB, 0.79050E+00_JPRB ,&
&  0.78948E+00_JPRB, 0.78621E+00_JPRB, 0.78197E+00_JPRB, 0.77373E+00_JPRB, 0.76820E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.75130E+00_JPRB, 0.73271E+00_JPRB, 0.70692E+00_JPRB, 0.68772E+00_JPRB, 0.67914E+00_JPRB, 0.70317E+00_JPRB, 0.76872E+00_JPRB ,&
&  0.10493E+01_JPRB, 0.78832E+00_JPRB, 0.69431E+00_JPRB, 0.66562E+00_JPRB, 0.65712E+00_JPRB, 0.63588E+00_JPRB, 0.62378E+00_JPRB ,&
&  0.62976E+00_JPRB, 0.64459E+00_JPRB, 0.68043E+00_JPRB, 0.73589E+00_JPRB, 0.84659E+00_JPRB, 0.10906E+01_JPRB, 0.12052E+01_JPRB ,&
&  0.11963E+01_JPRB, 0.22361E+01_JPRB, 0.18726E+01_JPRB, 0.17200E+01_JPRB, 0.15264E+01_JPRB, 0.14387E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11375E+01_JPRB, 0.11610E+01_JPRB, 0.11703E+01_JPRB, 0.11703E+01_JPRB, 0.11602E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.11280E+01_JPRB, 0.11217E+01_JPRB, 0.11160E+01_JPRB, 0.11130E+01_JPRB, 0.11356E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.11866E+01_JPRB, 0.12458E+01_JPRB, 0.12043E+01_JPRB, 0.11599E+01_JPRB, 0.11453E+01_JPRB, 0.11688E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.10626E+01_JPRB, 0.11114E+01_JPRB, 0.11380E+01_JPRB, 0.11475E+01_JPRB, 0.12259E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.12971E+01_JPRB, 0.14759E+01_JPRB, 0.12909E+01_JPRB, 0.12917E+01_JPRB, 0.14445E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.13292E+01_JPRB, 0.13590E+01_JPRB, 0.13516E+01_JPRB, 0.12624E+01_JPRB, 0.11473E+01_JPRB, 0.10655E+01_JPRB ,&
&  0.98676E+00_JPRB, 0.10101E+01_JPRB, 0.93411E+00_JPRB, 0.82235E+00_JPRB, 0.83234E+00_JPRB, 0.74456E+00_JPRB, 0.81667E+00_JPRB ,&
&  0.89782E+00_JPRB, 0.86068E+00_JPRB, 0.88547E+00_JPRB, 0.90002E+00_JPRB, 0.82630E+00_JPRB, 0.78387E+00_JPRB, 0.77231E+00_JPRB ,&
&  0.79769E+00_JPRB, 0.82298E+00_JPRB, 0.81870E+00_JPRB, 0.80272E+00_JPRB, 0.79161E+00_JPRB, 0.78097E+00_JPRB, 0.76570E+00_JPRB ,&
&  0.76077E+00_JPRB, 0.76199E+00_JPRB, 0.76273E+00_JPRB, 0.76170E+00_JPRB, 0.76485E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.77095E+00_JPRB, 0.76642E+00_JPRB, 0.72952E+00_JPRB, 0.69037E+00_JPRB, 0.66974E+00_JPRB, 0.67154E+00_JPRB, 0.69477E+00_JPRB ,&
&  0.89634E+00_JPRB, 0.78584E+00_JPRB, 0.70282E+00_JPRB, 0.66406E+00_JPRB, 0.64470E+00_JPRB, 0.63260E+00_JPRB, 0.61549E+00_JPRB ,&
&  0.60470E+00_JPRB, 0.60643E+00_JPRB, 0.64147E+00_JPRB, 0.66799E+00_JPRB, 0.71979E+00_JPRB, 0.84707E+00_JPRB, 0.76856E+00_JPRB ,&
&  0.11222E+01_JPRB, 0.21075E+01_JPRB, 0.19513E+01_JPRB, 0.18730E+01_JPRB, 0.17097E+01_JPRB, 0.16315E+01_JPRB, 0.13120E+01_JPRB ,&
&  0.12036E+01_JPRB, 0.12072E+01_JPRB, 0.12244E+01_JPRB, 0.12481E+01_JPRB, 0.12687E+01_JPRB, 0.12737E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.11590E+01_JPRB, 0.11485E+01_JPRB, 0.11354E+01_JPRB, 0.11251E+01_JPRB, 0.11328E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.11362E+01_JPRB, 0.11817E+01_JPRB, 0.12247E+01_JPRB, 0.11450E+01_JPRB, 0.11411E+01_JPRB, 0.11557E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.10348E+01_JPRB, 0.11112E+01_JPRB, 0.12011E+01_JPRB, 0.12696E+01_JPRB, 0.13057E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13586E+01_JPRB, 0.13645E+01_JPRB, 0.12767E+01_JPRB, 0.13458E+01_JPRB, 0.15239E+01_JPRB, 0.15803E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.14472E+01_JPRB, 0.14269E+01_JPRB, 0.13897E+01_JPRB, 0.13183E+01_JPRB, 0.11569E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.13348E+01_JPRB, 0.12210E+01_JPRB, 0.10314E+01_JPRB, 0.85497E+00_JPRB, 0.64717E+00_JPRB, 0.70056E+00_JPRB ,&
&  0.82863E+00_JPRB, 0.81470E+00_JPRB, 0.80163E+00_JPRB, 0.93809E+00_JPRB, 0.88652E+00_JPRB, 0.85576E+00_JPRB, 0.81716E+00_JPRB ,&
&  0.78718E+00_JPRB, 0.80086E+00_JPRB, 0.81247E+00_JPRB, 0.80380E+00_JPRB, 0.78790E+00_JPRB, 0.77290E+00_JPRB, 0.74940E+00_JPRB ,&
&  0.73062E+00_JPRB, 0.72931E+00_JPRB, 0.73989E+00_JPRB, 0.75570E+00_JPRB, 0.76966E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.76581E+00_JPRB, 0.77060E+00_JPRB, 0.73366E+00_JPRB, 0.68242E+00_JPRB, 0.65288E+00_JPRB, 0.64443E+00_JPRB, 0.64695E+00_JPRB ,&
&  0.72345E+00_JPRB, 0.72480E+00_JPRB, 0.69170E+00_JPRB, 0.65918E+00_JPRB, 0.64638E+00_JPRB, 0.62899E+00_JPRB, 0.61635E+00_JPRB ,&
&  0.59426E+00_JPRB, 0.58770E+00_JPRB, 0.63993E+00_JPRB, 0.66567E+00_JPRB, 0.70026E+00_JPRB, 0.60618E+00_JPRB, 0.71717E+00_JPRB ,&
&  0.12982E+01_JPRB, 0.21386E+01_JPRB, 0.20421E+01_JPRB, 0.20741E+01_JPRB, 0.20329E+01_JPRB, 0.20475E+01_JPRB, 0.14608E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.13064E+01_JPRB, 0.13039E+01_JPRB, 0.13555E+01_JPRB, 0.13739E+01_JPRB, 0.13705E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.12978E+01_JPRB, 0.12216E+01_JPRB, 0.11819E+01_JPRB, 0.11595E+01_JPRB, 0.11227E+01_JPRB, 0.11000E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.11294E+01_JPRB, 0.11376E+01_JPRB, 0.11796E+01_JPRB, 0.11406E+01_JPRB, 0.10961E+01_JPRB, 0.10976E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11502E+01_JPRB, 0.11677E+01_JPRB, 0.11868E+01_JPRB, 0.12829E+01_JPRB, 0.13922E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.15032E+01_JPRB, 0.14341E+01_JPRB, 0.12944E+01_JPRB, 0.13241E+01_JPRB, 0.14592E+01_JPRB, 0.16990E+01_JPRB, 0.16342E+01_JPRB ,&
&  0.14274E+01_JPRB, 0.11859E+01_JPRB, 0.11483E+01_JPRB, 0.12798E+01_JPRB, 0.13524E+01_JPRB, 0.12527E+01_JPRB, 0.13797E+01_JPRB ,&
&  0.13144E+01_JPRB, 0.83997E+00_JPRB, 0.72777E+00_JPRB, 0.87100E+00_JPRB, 0.69249E+00_JPRB, 0.54888E+00_JPRB, 0.69567E+00_JPRB ,&
&  0.80084E+00_JPRB, 0.86252E+00_JPRB, 0.86242E+00_JPRB, 0.84123E+00_JPRB, 0.92994E+00_JPRB, 0.97255E+00_JPRB, 0.96829E+00_JPRB ,&
&  0.91412E+00_JPRB, 0.87111E+00_JPRB, 0.82800E+00_JPRB, 0.80187E+00_JPRB, 0.79143E+00_JPRB, 0.77923E+00_JPRB, 0.75007E+00_JPRB ,&
&  0.73271E+00_JPRB, 0.72337E+00_JPRB, 0.72534E+00_JPRB, 0.74564E+00_JPRB, 0.76087E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.76486E+00_JPRB, 0.75282E+00_JPRB, 0.71604E+00_JPRB, 0.66772E+00_JPRB, 0.64115E+00_JPRB, 0.62656E+00_JPRB, 0.61934E+00_JPRB ,&
&  0.64013E+00_JPRB, 0.65888E+00_JPRB, 0.66294E+00_JPRB, 0.65399E+00_JPRB, 0.64544E+00_JPRB, 0.62944E+00_JPRB, 0.61519E+00_JPRB ,&
&  0.59521E+00_JPRB, 0.57921E+00_JPRB, 0.65482E+00_JPRB, 0.71659E+00_JPRB, 0.71698E+00_JPRB, 0.64749E+00_JPRB, 0.82009E+00_JPRB ,&
&  0.12920E+01_JPRB, 0.20789E+01_JPRB, 0.24408E+01_JPRB, 0.24873E+01_JPRB, 0.22971E+01_JPRB, 0.22116E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.15081E+01_JPRB, 0.14842E+01_JPRB, 0.14359E+01_JPRB, 0.14525E+01_JPRB, 0.14867E+01_JPRB, 0.14760E+01_JPRB, 0.14247E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.12926E+01_JPRB, 0.12349E+01_JPRB, 0.12011E+01_JPRB, 0.11563E+01_JPRB, 0.10995E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.11112E+01_JPRB, 0.11386E+01_JPRB, 0.11738E+01_JPRB, 0.97474E+00_JPRB, 0.10527E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.11355E+01_JPRB, 0.11858E+01_JPRB, 0.11637E+01_JPRB, 0.12350E+01_JPRB, 0.13425E+01_JPRB, 0.13990E+01_JPRB, 0.15072E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.15788E+01_JPRB, 0.14321E+01_JPRB, 0.14749E+01_JPRB, 0.15927E+01_JPRB, 0.17705E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.14018E+01_JPRB, 0.12743E+01_JPRB, 0.11535E+01_JPRB, 0.13229E+01_JPRB, 0.13193E+01_JPRB, 0.10919E+01_JPRB ,&
&  0.57941E+00_JPRB, 0.46455E+00_JPRB, 0.46523E+00_JPRB, 0.47517E+00_JPRB, 0.46866E+00_JPRB, 0.50762E+00_JPRB, 0.86603E+00_JPRB ,&
&  0.82379E+00_JPRB, 0.89520E+00_JPRB, 0.90512E+00_JPRB, 0.88343E+00_JPRB, 0.94452E+00_JPRB, 0.10126E+01_JPRB, 0.12227E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.10310E+01_JPRB, 0.97495E+00_JPRB, 0.89858E+00_JPRB, 0.83615E+00_JPRB, 0.79004E+00_JPRB, 0.75979E+00_JPRB ,&
&  0.73968E+00_JPRB, 0.73114E+00_JPRB, 0.73301E+00_JPRB, 0.74666E+00_JPRB, 0.76244E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.76908E+00_JPRB, 0.73559E+00_JPRB, 0.68914E+00_JPRB, 0.63976E+00_JPRB, 0.61810E+00_JPRB, 0.60225E+00_JPRB, 0.59107E+00_JPRB ,&
&  0.59886E+00_JPRB, 0.61170E+00_JPRB, 0.62629E+00_JPRB, 0.63228E+00_JPRB, 0.62903E+00_JPRB, 0.61853E+00_JPRB, 0.60443E+00_JPRB ,&
&  0.58610E+00_JPRB, 0.57569E+00_JPRB, 0.70643E+00_JPRB, 0.73047E+00_JPRB, 0.76756E+00_JPRB, 0.72367E+00_JPRB, 0.85426E+00_JPRB ,&
&  0.12279E+01_JPRB, 0.19280E+01_JPRB, 0.24814E+01_JPRB, 0.25793E+01_JPRB, 0.26730E+01_JPRB, 0.26361E+01_JPRB, 0.23770E+01_JPRB ,&
&  0.20822E+01_JPRB, 0.19140E+01_JPRB, 0.17554E+01_JPRB, 0.16576E+01_JPRB, 0.15841E+01_JPRB, 0.15418E+01_JPRB, 0.14719E+01_JPRB ,&
&  0.14037E+01_JPRB, 0.13627E+01_JPRB, 0.13168E+01_JPRB, 0.12864E+01_JPRB, 0.12296E+01_JPRB, 0.11618E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.11101E+01_JPRB, 0.11160E+01_JPRB, 0.11599E+01_JPRB, 0.10486E+01_JPRB, 0.96683E+00_JPRB, 0.10627E+01_JPRB ,&
&  0.11731E+01_JPRB, 0.13192E+01_JPRB, 0.13003E+01_JPRB, 0.13704E+01_JPRB, 0.14085E+01_JPRB, 0.14334E+01_JPRB, 0.16116E+01_JPRB ,&
&  0.17812E+01_JPRB, 0.19383E+01_JPRB, 0.16265E+01_JPRB, 0.16590E+01_JPRB, 0.18156E+01_JPRB, 0.15269E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.16752E+01_JPRB, 0.14734E+01_JPRB, 0.12822E+01_JPRB, 0.10838E+01_JPRB, 0.12854E+01_JPRB, 0.11186E+01_JPRB, 0.62098E+00_JPRB ,&
&  0.48916E+00_JPRB, 0.44707E+00_JPRB, 0.42400E+00_JPRB, 0.42627E+00_JPRB, 0.42580E+00_JPRB, 0.48632E+00_JPRB, 0.72622E+00_JPRB ,&
&  0.79584E+00_JPRB, 0.88811E+00_JPRB, 0.93401E+00_JPRB, 0.93943E+00_JPRB, 0.94948E+00_JPRB, 0.97040E+00_JPRB, 0.12520E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.12956E+01_JPRB, 0.11567E+01_JPRB, 0.10626E+01_JPRB, 0.10109E+01_JPRB, 0.93696E+00_JPRB, 0.85984E+00_JPRB ,&
&  0.79592E+00_JPRB, 0.76666E+00_JPRB, 0.75733E+00_JPRB, 0.76804E+00_JPRB, 0.78824E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.78386E+00_JPRB, 0.75577E+00_JPRB, 0.70155E+00_JPRB, 0.64316E+00_JPRB, 0.61231E+00_JPRB, 0.58628E+00_JPRB, 0.57076E+00_JPRB ,&
&  0.57291E+00_JPRB, 0.58157E+00_JPRB, 0.59037E+00_JPRB, 0.59733E+00_JPRB, 0.59508E+00_JPRB, 0.58886E+00_JPRB, 0.58184E+00_JPRB ,&
&  0.56562E+00_JPRB, 0.59092E+00_JPRB, 0.67166E+00_JPRB, 0.62495E+00_JPRB, 0.63448E+00_JPRB, 0.64423E+00_JPRB, 0.74086E+00_JPRB ,&
&  0.11653E+01_JPRB, 0.18250E+01_JPRB, 0.23978E+01_JPRB, 0.27277E+01_JPRB, 0.29845E+01_JPRB, 0.28955E+01_JPRB, 0.31466E+01_JPRB ,&
&  0.28422E+01_JPRB, 0.25065E+01_JPRB, 0.22106E+01_JPRB, 0.19973E+01_JPRB, 0.17889E+01_JPRB, 0.16488E+01_JPRB, 0.15722E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.13924E+01_JPRB, 0.13564E+01_JPRB, 0.13219E+01_JPRB, 0.12782E+01_JPRB, 0.12300E+01_JPRB, 0.11865E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.11625E+01_JPRB, 0.11490E+01_JPRB, 0.11377E+01_JPRB, 0.11984E+01_JPRB, 0.11648E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.12313E+01_JPRB, 0.14130E+01_JPRB, 0.13702E+01_JPRB, 0.14180E+01_JPRB, 0.15290E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.21269E+01_JPRB, 0.18745E+01_JPRB, 0.19393E+01_JPRB, 0.18063E+01_JPRB, 0.15464E+01_JPRB, 0.18323E+01_JPRB ,&
&  0.19320E+01_JPRB, 0.16728E+01_JPRB, 0.13407E+01_JPRB, 0.10601E+01_JPRB, 0.10168E+01_JPRB, 0.74777E+00_JPRB, 0.49758E+00_JPRB ,&
&  0.46674E+00_JPRB, 0.44983E+00_JPRB, 0.42576E+00_JPRB, 0.47330E+00_JPRB, 0.46949E+00_JPRB, 0.54129E+00_JPRB, 0.72987E+00_JPRB ,&
&  0.85478E+00_JPRB, 0.85484E+00_JPRB, 0.91322E+00_JPRB, 0.95349E+00_JPRB, 0.94896E+00_JPRB, 0.94202E+00_JPRB, 0.12137E+01_JPRB ,&
&  0.12097E+01_JPRB, 0.12391E+01_JPRB, 0.12120E+01_JPRB, 0.11039E+01_JPRB, 0.10633E+01_JPRB, 0.10379E+01_JPRB, 0.98636E+00_JPRB ,&
&  0.91141E+00_JPRB, 0.83754E+00_JPRB, 0.80779E+00_JPRB, 0.80586E+00_JPRB, 0.80323E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.81367E+00_JPRB, 0.79379E+00_JPRB, 0.74133E+00_JPRB, 0.67986E+00_JPRB, 0.63481E+00_JPRB, 0.60097E+00_JPRB, 0.58346E+00_JPRB ,&
&  0.58582E+00_JPRB, 0.58631E+00_JPRB, 0.58834E+00_JPRB, 0.58802E+00_JPRB, 0.58268E+00_JPRB, 0.57308E+00_JPRB, 0.57138E+00_JPRB ,&
&  0.55741E+00_JPRB, 0.69433E+00_JPRB, 0.52667E+00_JPRB, 0.45634E+00_JPRB, 0.52440E+00_JPRB, 0.60693E+00_JPRB, 0.62929E+00_JPRB ,&
&  0.10705E+01_JPRB, 0.17397E+01_JPRB, 0.22729E+01_JPRB, 0.29464E+01_JPRB, 0.38811E+01_JPRB, 0.36888E+01_JPRB, 0.37011E+01_JPRB ,&
&  0.35489E+01_JPRB, 0.31620E+01_JPRB, 0.27501E+01_JPRB, 0.23496E+01_JPRB, 0.19933E+01_JPRB, 0.17778E+01_JPRB, 0.16463E+01_JPRB ,&
&  0.15228E+01_JPRB, 0.14236E+01_JPRB, 0.13641E+01_JPRB, 0.13415E+01_JPRB, 0.12987E+01_JPRB, 0.12603E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.12182E+01_JPRB, 0.11992E+01_JPRB, 0.11999E+01_JPRB, 0.11472E+01_JPRB, 0.11081E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.13514E+01_JPRB, 0.13628E+01_JPRB, 0.13513E+01_JPRB, 0.14396E+01_JPRB, 0.15361E+01_JPRB, 0.16152E+01_JPRB, 0.18511E+01_JPRB ,&
&  0.17838E+01_JPRB, 0.17871E+01_JPRB, 0.18548E+01_JPRB, 0.18979E+01_JPRB, 0.17528E+01_JPRB, 0.19693E+01_JPRB, 0.25749E+01_JPRB ,&
&  0.20312E+01_JPRB, 0.19973E+01_JPRB, 0.19565E+01_JPRB, 0.18666E+01_JPRB, 0.99616E+00_JPRB, 0.71749E+00_JPRB, 0.10281E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.72143E+00_JPRB, 0.57912E+00_JPRB, 0.76241E+00_JPRB, 0.61909E+00_JPRB, 0.67625E+00_JPRB, 0.86076E+00_JPRB ,&
&  0.84764E+00_JPRB, 0.76691E+00_JPRB, 0.88651E+00_JPRB, 0.99703E+00_JPRB, 0.10320E+01_JPRB, 0.97787E+00_JPRB, 0.11528E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11803E+01_JPRB, 0.12318E+01_JPRB, 0.11525E+01_JPRB, 0.10847E+01_JPRB, 0.10428E+01_JPRB, 0.10074E+01_JPRB ,&
&  0.95811E+00_JPRB, 0.90098E+00_JPRB, 0.86758E+00_JPRB, 0.85728E+00_JPRB, 0.83358E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.83497E+00_JPRB, 0.81875E+00_JPRB, 0.76563E+00_JPRB, 0.70748E+00_JPRB, 0.64241E+00_JPRB, 0.59181E+00_JPRB, 0.57822E+00_JPRB ,&
&  0.58619E+00_JPRB, 0.59434E+00_JPRB, 0.59422E+00_JPRB, 0.58774E+00_JPRB, 0.57805E+00_JPRB, 0.56852E+00_JPRB, 0.56084E+00_JPRB ,&
&  0.54906E+00_JPRB, 0.53170E+00_JPRB, 0.41637E+00_JPRB, 0.37642E+00_JPRB, 0.51658E+00_JPRB, 0.55184E+00_JPRB, 0.62315E+00_JPRB ,&
&  0.94727E+00_JPRB, 0.14874E+01_JPRB, 0.21255E+01_JPRB, 0.28683E+01_JPRB, 0.35960E+01_JPRB, 0.39468E+01_JPRB, 0.39276E+01_JPRB ,&
&  0.39996E+01_JPRB, 0.36102E+01_JPRB, 0.30601E+01_JPRB, 0.25973E+01_JPRB, 0.22695E+01_JPRB, 0.20452E+01_JPRB, 0.18173E+01_JPRB ,&
&  0.16100E+01_JPRB, 0.14696E+01_JPRB, 0.13897E+01_JPRB, 0.13370E+01_JPRB, 0.13075E+01_JPRB, 0.12633E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.12245E+01_JPRB, 0.12482E+01_JPRB, 0.12630E+01_JPRB, 0.12440E+01_JPRB, 0.10832E+01_JPRB, 0.10399E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.13698E+01_JPRB, 0.13391E+01_JPRB, 0.13627E+01_JPRB, 0.14586E+01_JPRB, 0.15998E+01_JPRB, 0.16257E+01_JPRB, 0.19501E+01_JPRB ,&
&  0.18945E+01_JPRB, 0.17790E+01_JPRB, 0.17086E+01_JPRB, 0.15282E+01_JPRB, 0.17821E+01_JPRB, 0.25464E+01_JPRB, 0.25828E+01_JPRB ,&
&  0.23923E+01_JPRB, 0.22434E+01_JPRB, 0.20887E+01_JPRB, 0.18307E+01_JPRB, 0.15335E+01_JPRB, 0.11088E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.13737E+01_JPRB, 0.12925E+01_JPRB, 0.11079E+01_JPRB, 0.92437E+00_JPRB, 0.10489E+01_JPRB, 0.94908E+00_JPRB ,&
&  0.77429E+00_JPRB, 0.69424E+00_JPRB, 0.79072E+00_JPRB, 0.94932E+00_JPRB, 0.99733E+00_JPRB, 0.93299E+00_JPRB, 0.11035E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11678E+01_JPRB, 0.11587E+01_JPRB, 0.11074E+01_JPRB, 0.10579E+01_JPRB, 0.10171E+01_JPRB, 0.96886E+00_JPRB ,&
&  0.93767E+00_JPRB, 0.91437E+00_JPRB, 0.88802E+00_JPRB, 0.87271E+00_JPRB, 0.84764E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.86267E+00_JPRB, 0.83742E+00_JPRB, 0.78012E+00_JPRB, 0.71675E+00_JPRB, 0.64350E+00_JPRB, 0.57758E+00_JPRB, 0.56083E+00_JPRB ,&
&  0.56870E+00_JPRB, 0.57668E+00_JPRB, 0.57374E+00_JPRB, 0.56286E+00_JPRB, 0.55496E+00_JPRB, 0.54521E+00_JPRB, 0.54018E+00_JPRB ,&
&  0.53809E+00_JPRB, 0.48568E+00_JPRB, 0.38065E+00_JPRB, 0.37888E+00_JPRB, 0.45338E+00_JPRB, 0.50311E+00_JPRB, 0.63190E+00_JPRB ,&
&  0.83613E+00_JPRB, 0.13328E+01_JPRB, 0.18049E+01_JPRB, 0.23784E+01_JPRB, 0.29671E+01_JPRB, 0.34167E+01_JPRB, 0.34557E+01_JPRB ,&
&  0.32506E+01_JPRB, 0.32419E+01_JPRB, 0.30031E+01_JPRB, 0.26783E+01_JPRB, 0.23833E+01_JPRB, 0.21550E+01_JPRB, 0.19475E+01_JPRB ,&
&  0.17439E+01_JPRB, 0.15514E+01_JPRB, 0.14338E+01_JPRB, 0.13622E+01_JPRB, 0.13150E+01_JPRB, 0.12752E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.12518E+01_JPRB, 0.12513E+01_JPRB, 0.12472E+01_JPRB, 0.10573E+01_JPRB, 0.10943E+01_JPRB, 0.12627E+01_JPRB ,&
&  0.15849E+01_JPRB, 0.15040E+01_JPRB, 0.15087E+01_JPRB, 0.17413E+01_JPRB, 0.16297E+01_JPRB, 0.17341E+01_JPRB, 0.21542E+01_JPRB ,&
&  0.24752E+01_JPRB, 0.25301E+01_JPRB, 0.26019E+01_JPRB, 0.21376E+01_JPRB, 0.19545E+01_JPRB, 0.27616E+01_JPRB, 0.26221E+01_JPRB ,&
&  0.24179E+01_JPRB, 0.22970E+01_JPRB, 0.21079E+01_JPRB, 0.20783E+01_JPRB, 0.17825E+01_JPRB, 0.15420E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11989E+01_JPRB, 0.11406E+01_JPRB, 0.12775E+01_JPRB, 0.11897E+01_JPRB, 0.10992E+01_JPRB, 0.98816E+00_JPRB, 0.81554E+00_JPRB ,&
&  0.70579E+00_JPRB, 0.62893E+00_JPRB, 0.62183E+00_JPRB, 0.74145E+00_JPRB, 0.83578E+00_JPRB, 0.78454E+00_JPRB, 0.91945E+00_JPRB ,&
&  0.10342E+01_JPRB, 0.11107E+01_JPRB, 0.10542E+01_JPRB, 0.10720E+01_JPRB, 0.10537E+01_JPRB, 0.10308E+01_JPRB, 0.98357E+00_JPRB ,&
&  0.94037E+00_JPRB, 0.91176E+00_JPRB, 0.89513E+00_JPRB, 0.88288E+00_JPRB, 0.86975E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.86924E+00_JPRB, 0.84065E+00_JPRB, 0.78073E+00_JPRB, 0.71742E+00_JPRB, 0.63317E+00_JPRB, 0.55660E+00_JPRB, 0.54355E+00_JPRB ,&
&  0.55979E+00_JPRB, 0.56122E+00_JPRB, 0.54850E+00_JPRB, 0.52262E+00_JPRB, 0.51638E+00_JPRB, 0.52445E+00_JPRB, 0.53072E+00_JPRB ,&
&  0.53331E+00_JPRB, 0.61871E+00_JPRB, 0.49310E+00_JPRB, 0.40800E+00_JPRB, 0.40203E+00_JPRB, 0.48367E+00_JPRB, 0.64608E+00_JPRB ,&
&  0.82176E+00_JPRB, 0.11351E+01_JPRB, 0.14681E+01_JPRB, 0.17557E+01_JPRB, 0.21358E+01_JPRB, 0.24322E+01_JPRB, 0.25829E+01_JPRB ,&
&  0.27428E+01_JPRB, 0.23291E+01_JPRB, 0.23318E+01_JPRB, 0.21549E+01_JPRB, 0.19813E+01_JPRB, 0.18360E+01_JPRB, 0.17164E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.14548E+01_JPRB, 0.13245E+01_JPRB, 0.12352E+01_JPRB, 0.11861E+01_JPRB, 0.11419E+01_JPRB, 0.11060E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.11822E+01_JPRB, 0.12336E+01_JPRB, 0.12302E+01_JPRB, 0.12700E+01_JPRB, 0.13843E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.15386E+01_JPRB, 0.14500E+01_JPRB, 0.18077E+01_JPRB, 0.20345E+01_JPRB, 0.23145E+01_JPRB, 0.24674E+01_JPRB, 0.26028E+01_JPRB ,&
&  0.28949E+01_JPRB, 0.28606E+01_JPRB, 0.28081E+01_JPRB, 0.25437E+01_JPRB, 0.28477E+01_JPRB, 0.28008E+01_JPRB, 0.26226E+01_JPRB ,&
&  0.24032E+01_JPRB, 0.23605E+01_JPRB, 0.21807E+01_JPRB, 0.20587E+01_JPRB, 0.20149E+01_JPRB, 0.20819E+01_JPRB, 0.18570E+01_JPRB ,&
&  0.14980E+01_JPRB, 0.15851E+01_JPRB, 0.13979E+01_JPRB, 0.11630E+01_JPRB, 0.90081E+00_JPRB, 0.79640E+00_JPRB, 0.71776E+00_JPRB ,&
&  0.65368E+00_JPRB, 0.60894E+00_JPRB, 0.58647E+00_JPRB, 0.63796E+00_JPRB, 0.70301E+00_JPRB, 0.70594E+00_JPRB, 0.72847E+00_JPRB ,&
&  0.75244E+00_JPRB, 0.89789E+00_JPRB, 0.92966E+00_JPRB, 0.92427E+00_JPRB, 0.96038E+00_JPRB, 0.98782E+00_JPRB, 0.98499E+00_JPRB ,&
&  0.95149E+00_JPRB, 0.92462E+00_JPRB, 0.90531E+00_JPRB, 0.89333E+00_JPRB, 0.87982E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.85120E+00_JPRB, 0.82461E+00_JPRB, 0.77556E+00_JPRB, 0.71317E+00_JPRB, 0.62626E+00_JPRB, 0.55293E+00_JPRB, 0.53652E+00_JPRB ,&
&  0.53488E+00_JPRB, 0.52983E+00_JPRB, 0.51739E+00_JPRB, 0.50941E+00_JPRB, 0.51100E+00_JPRB, 0.51805E+00_JPRB, 0.54189E+00_JPRB ,&
&  0.59110E+00_JPRB, 0.92330E+00_JPRB, 0.70333E+00_JPRB, 0.57966E+00_JPRB, 0.51221E+00_JPRB, 0.61422E+00_JPRB, 0.71878E+00_JPRB ,&
&  0.87258E+00_JPRB, 0.10233E+01_JPRB, 0.11145E+01_JPRB, 0.13373E+01_JPRB, 0.15585E+01_JPRB, 0.16342E+01_JPRB, 0.17775E+01_JPRB ,&
&  0.17901E+01_JPRB, 0.18084E+01_JPRB, 0.16382E+01_JPRB, 0.16310E+01_JPRB, 0.15504E+01_JPRB, 0.13919E+01_JPRB, 0.14218E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.12210E+01_JPRB, 0.11307E+01_JPRB, 0.10820E+01_JPRB, 0.10774E+01_JPRB, 0.10696E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10918E+01_JPRB, 0.11682E+01_JPRB, 0.12313E+01_JPRB, 0.13061E+01_JPRB, 0.14698E+01_JPRB, 0.16213E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.18989E+01_JPRB, 0.16542E+01_JPRB, 0.20960E+01_JPRB, 0.28736E+01_JPRB, 0.29871E+01_JPRB, 0.27840E+01_JPRB ,&
&  0.30729E+01_JPRB, 0.31159E+01_JPRB, 0.30650E+01_JPRB, 0.29398E+01_JPRB, 0.28006E+01_JPRB, 0.27226E+01_JPRB, 0.26453E+01_JPRB ,&
&  0.23837E+01_JPRB, 0.22449E+01_JPRB, 0.21538E+01_JPRB, 0.20929E+01_JPRB, 0.19686E+01_JPRB, 0.19106E+01_JPRB, 0.18915E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.13631E+01_JPRB, 0.92195E+00_JPRB, 0.90024E+00_JPRB, 0.66661E+00_JPRB, 0.57937E+00_JPRB, 0.62754E+00_JPRB ,&
&  0.59908E+00_JPRB, 0.63087E+00_JPRB, 0.66182E+00_JPRB, 0.62417E+00_JPRB, 0.62340E+00_JPRB, 0.58306E+00_JPRB, 0.60246E+00_JPRB ,&
&  0.63640E+00_JPRB, 0.62708E+00_JPRB, 0.76314E+00_JPRB, 0.79009E+00_JPRB, 0.82532E+00_JPRB, 0.85391E+00_JPRB, 0.87327E+00_JPRB ,&
&  0.88093E+00_JPRB, 0.87545E+00_JPRB, 0.87030E+00_JPRB, 0.86238E+00_JPRB, 0.86251E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.76762E+00_JPRB, 0.74289E+00_JPRB, 0.70796E+00_JPRB, 0.66068E+00_JPRB, 0.59705E+00_JPRB, 0.53655E+00_JPRB, 0.50908E+00_JPRB ,&
&  0.50462E+00_JPRB, 0.49438E+00_JPRB, 0.48730E+00_JPRB, 0.48848E+00_JPRB, 0.49775E+00_JPRB, 0.51423E+00_JPRB, 0.56663E+00_JPRB ,&
&  0.69278E+00_JPRB, 0.91806E+00_JPRB, 0.76975E+00_JPRB, 0.64200E+00_JPRB, 0.76801E+00_JPRB, 0.73812E+00_JPRB, 0.79673E+00_JPRB ,&
&  0.83226E+00_JPRB, 0.98913E+00_JPRB, 0.98129E+00_JPRB, 0.10095E+01_JPRB, 0.11484E+01_JPRB, 0.12339E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.11936E+01_JPRB, 0.11318E+01_JPRB, 0.11441E+01_JPRB, 0.12138E+01_JPRB, 0.11527E+01_JPRB, 0.11578E+01_JPRB ,&
&  0.10876E+01_JPRB, 0.10242E+01_JPRB, 0.94520E+00_JPRB, 0.91834E+00_JPRB, 0.93606E+00_JPRB, 0.97852E+00_JPRB, 0.10026E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.11136E+01_JPRB, 0.11602E+01_JPRB, 0.12521E+01_JPRB, 0.13767E+01_JPRB, 0.16917E+01_JPRB, 0.21386E+01_JPRB ,&
&  0.25302E+01_JPRB, 0.26772E+01_JPRB, 0.28085E+01_JPRB, 0.31787E+01_JPRB, 0.32940E+01_JPRB, 0.32165E+01_JPRB, 0.31406E+01_JPRB ,&
&  0.30009E+01_JPRB, 0.29408E+01_JPRB, 0.29724E+01_JPRB, 0.28203E+01_JPRB, 0.26942E+01_JPRB, 0.26569E+01_JPRB, 0.25578E+01_JPRB ,&
&  0.22475E+01_JPRB, 0.20691E+01_JPRB, 0.20330E+01_JPRB, 0.19105E+01_JPRB, 0.18132E+01_JPRB, 0.17028E+01_JPRB, 0.16857E+01_JPRB ,&
&  0.17458E+01_JPRB, 0.98816E+00_JPRB, 0.74355E+00_JPRB, 0.79741E+00_JPRB, 0.62915E+00_JPRB, 0.57691E+00_JPRB, 0.66821E+00_JPRB ,&
&  0.65741E+00_JPRB, 0.65075E+00_JPRB, 0.66602E+00_JPRB, 0.61257E+00_JPRB, 0.55731E+00_JPRB, 0.54610E+00_JPRB, 0.50973E+00_JPRB ,&
&  0.52053E+00_JPRB, 0.53400E+00_JPRB, 0.63067E+00_JPRB, 0.69339E+00_JPRB, 0.73132E+00_JPRB, 0.74633E+00_JPRB, 0.76326E+00_JPRB ,&
&  0.77226E+00_JPRB, 0.77848E+00_JPRB, 0.78393E+00_JPRB, 0.79162E+00_JPRB, 0.79412E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.68367E+00_JPRB, 0.67130E+00_JPRB, 0.64996E+00_JPRB, 0.62269E+00_JPRB, 0.58376E+00_JPRB, 0.54169E+00_JPRB, 0.50007E+00_JPRB ,&
&  0.49481E+00_JPRB, 0.48560E+00_JPRB, 0.47995E+00_JPRB, 0.48056E+00_JPRB, 0.48869E+00_JPRB, 0.50178E+00_JPRB, 0.52884E+00_JPRB ,&
&  0.47347E+00_JPRB, 0.59170E+00_JPRB, 0.54592E+00_JPRB, 0.65125E+00_JPRB, 0.92126E+00_JPRB, 0.74411E+00_JPRB, 0.77381E+00_JPRB ,&
&  0.80463E+00_JPRB, 0.87917E+00_JPRB, 0.89761E+00_JPRB, 0.90254E+00_JPRB, 0.94903E+00_JPRB, 0.98075E+00_JPRB, 0.96944E+00_JPRB ,&
&  0.90508E+00_JPRB, 0.80566E+00_JPRB, 0.77981E+00_JPRB, 0.78314E+00_JPRB, 0.86638E+00_JPRB, 0.93193E+00_JPRB, 0.94874E+00_JPRB ,&
&  0.91136E+00_JPRB, 0.87310E+00_JPRB, 0.83726E+00_JPRB, 0.84778E+00_JPRB, 0.87867E+00_JPRB, 0.90408E+00_JPRB, 0.95523E+00_JPRB ,&
&  0.10318E+01_JPRB, 0.11158E+01_JPRB, 0.12161E+01_JPRB, 0.13354E+01_JPRB, 0.13288E+01_JPRB, 0.20341E+01_JPRB, 0.32774E+01_JPRB ,&
&  0.36147E+01_JPRB, 0.34434E+01_JPRB, 0.35367E+01_JPRB, 0.39795E+01_JPRB, 0.39228E+01_JPRB, 0.35579E+01_JPRB, 0.31234E+01_JPRB ,&
&  0.27788E+01_JPRB, 0.25989E+01_JPRB, 0.26874E+01_JPRB, 0.26461E+01_JPRB, 0.24377E+01_JPRB, 0.24503E+01_JPRB, 0.23830E+01_JPRB ,&
&  0.21840E+01_JPRB, 0.20461E+01_JPRB, 0.19826E+01_JPRB, 0.19527E+01_JPRB, 0.18202E+01_JPRB, 0.18164E+01_JPRB, 0.17259E+01_JPRB ,&
&  0.16658E+01_JPRB, 0.13989E+01_JPRB, 0.99196E+00_JPRB, 0.80298E+00_JPRB, 0.61452E+00_JPRB, 0.61337E+00_JPRB, 0.74249E+00_JPRB ,&
&  0.72286E+00_JPRB, 0.66798E+00_JPRB, 0.59377E+00_JPRB, 0.53170E+00_JPRB, 0.48901E+00_JPRB, 0.47912E+00_JPRB, 0.44420E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.46729E+00_JPRB, 0.52435E+00_JPRB, 0.58292E+00_JPRB, 0.63042E+00_JPRB, 0.66596E+00_JPRB, 0.64080E+00_JPRB ,&
&  0.70049E+00_JPRB, 0.70649E+00_JPRB, 0.71671E+00_JPRB, 0.72867E+00_JPRB, 0.71640E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.59208E+00_JPRB, 0.60048E+00_JPRB, 0.58546E+00_JPRB, 0.57912E+00_JPRB, 0.55237E+00_JPRB, 0.53035E+00_JPRB, 0.49448E+00_JPRB ,&
&  0.50022E+00_JPRB, 0.49468E+00_JPRB, 0.47962E+00_JPRB, 0.47435E+00_JPRB, 0.48475E+00_JPRB, 0.48698E+00_JPRB, 0.43874E+00_JPRB ,&
&  0.40850E+00_JPRB, 0.45558E+00_JPRB, 0.45336E+00_JPRB, 0.52323E+00_JPRB, 0.59276E+00_JPRB, 0.56695E+00_JPRB, 0.60076E+00_JPRB ,&
&  0.70232E+00_JPRB, 0.74089E+00_JPRB, 0.76918E+00_JPRB, 0.78560E+00_JPRB, 0.79764E+00_JPRB, 0.81231E+00_JPRB, 0.81612E+00_JPRB ,&
&  0.75558E+00_JPRB, 0.70780E+00_JPRB, 0.67103E+00_JPRB, 0.66731E+00_JPRB, 0.74679E+00_JPRB, 0.78995E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.72723E+00_JPRB, 0.73663E+00_JPRB, 0.79051E+00_JPRB, 0.82769E+00_JPRB, 0.84539E+00_JPRB, 0.89746E+00_JPRB ,&
&  0.98083E+00_JPRB, 0.10972E+01_JPRB, 0.12188E+01_JPRB, 0.13494E+01_JPRB, 0.14061E+01_JPRB, 0.21101E+01_JPRB, 0.30434E+01_JPRB ,&
&  0.31040E+01_JPRB, 0.32094E+01_JPRB, 0.32974E+01_JPRB, 0.33712E+01_JPRB, 0.33483E+01_JPRB, 0.29403E+01_JPRB, 0.25183E+01_JPRB ,&
&  0.23552E+01_JPRB, 0.22084E+01_JPRB, 0.26449E+01_JPRB, 0.25200E+01_JPRB, 0.23505E+01_JPRB, 0.22547E+01_JPRB, 0.21199E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.19748E+01_JPRB, 0.18999E+01_JPRB, 0.18098E+01_JPRB, 0.17106E+01_JPRB, 0.16698E+01_JPRB, 0.15425E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.13538E+01_JPRB, 0.10864E+01_JPRB, 0.87968E+00_JPRB, 0.71494E+00_JPRB, 0.72016E+00_JPRB, 0.66461E+00_JPRB ,&
&  0.58208E+00_JPRB, 0.49421E+00_JPRB, 0.47588E+00_JPRB, 0.45201E+00_JPRB, 0.44253E+00_JPRB, 0.43029E+00_JPRB, 0.41244E+00_JPRB ,&
&  0.43093E+00_JPRB, 0.46846E+00_JPRB, 0.47051E+00_JPRB, 0.49548E+00_JPRB, 0.53366E+00_JPRB, 0.58907E+00_JPRB, 0.52355E+00_JPRB ,&
&  0.54198E+00_JPRB, 0.60400E+00_JPRB, 0.61775E+00_JPRB, 0.61860E+00_JPRB, 0.60142E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.54977E+00_JPRB, 0.56166E+00_JPRB, 0.54744E+00_JPRB, 0.55299E+00_JPRB, 0.53746E+00_JPRB, 0.52988E+00_JPRB, 0.47395E+00_JPRB ,&
&  0.50341E+00_JPRB, 0.49095E+00_JPRB, 0.46860E+00_JPRB, 0.46398E+00_JPRB, 0.48096E+00_JPRB, 0.41978E+00_JPRB, 0.36100E+00_JPRB ,&
&  0.34484E+00_JPRB, 0.36638E+00_JPRB, 0.42092E+00_JPRB, 0.45178E+00_JPRB, 0.46480E+00_JPRB, 0.47557E+00_JPRB, 0.48140E+00_JPRB ,&
&  0.54368E+00_JPRB, 0.57138E+00_JPRB, 0.62821E+00_JPRB, 0.66249E+00_JPRB, 0.66026E+00_JPRB, 0.67260E+00_JPRB, 0.70040E+00_JPRB ,&
&  0.66222E+00_JPRB, 0.64363E+00_JPRB, 0.64214E+00_JPRB, 0.58317E+00_JPRB, 0.65382E+00_JPRB, 0.63043E+00_JPRB, 0.59854E+00_JPRB ,&
&  0.59909E+00_JPRB, 0.63070E+00_JPRB, 0.66687E+00_JPRB, 0.70525E+00_JPRB, 0.73128E+00_JPRB, 0.76288E+00_JPRB, 0.81056E+00_JPRB ,&
&  0.88415E+00_JPRB, 0.10243E+01_JPRB, 0.11829E+01_JPRB, 0.13455E+01_JPRB, 0.15435E+01_JPRB, 0.20713E+01_JPRB, 0.27182E+01_JPRB ,&
&  0.28293E+01_JPRB, 0.29221E+01_JPRB, 0.30069E+01_JPRB, 0.26284E+01_JPRB, 0.27387E+01_JPRB, 0.26262E+01_JPRB, 0.22743E+01_JPRB ,&
&  0.21270E+01_JPRB, 0.18751E+01_JPRB, 0.18198E+01_JPRB, 0.18097E+01_JPRB, 0.18762E+01_JPRB, 0.17949E+01_JPRB, 0.17127E+01_JPRB ,&
&  0.16861E+01_JPRB, 0.18201E+01_JPRB, 0.16725E+01_JPRB, 0.15811E+01_JPRB, 0.14271E+01_JPRB, 0.13104E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.10078E+01_JPRB, 0.91500E+00_JPRB, 0.84127E+00_JPRB, 0.74733E+00_JPRB, 0.67716E+00_JPRB, 0.62221E+00_JPRB ,&
&  0.56522E+00_JPRB, 0.49738E+00_JPRB, 0.47140E+00_JPRB, 0.45267E+00_JPRB, 0.44197E+00_JPRB, 0.42127E+00_JPRB, 0.42226E+00_JPRB ,&
&  0.44640E+00_JPRB, 0.47415E+00_JPRB, 0.50038E+00_JPRB, 0.49042E+00_JPRB, 0.50034E+00_JPRB, 0.53133E+00_JPRB, 0.51205E+00_JPRB ,&
&  0.48740E+00_JPRB, 0.54698E+00_JPRB, 0.55268E+00_JPRB, 0.55367E+00_JPRB, 0.54842E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.53931E+00_JPRB, 0.54151E+00_JPRB, 0.53245E+00_JPRB, 0.53270E+00_JPRB, 0.53384E+00_JPRB, 0.49420E+00_JPRB, 0.45490E+00_JPRB ,&
&  0.51774E+00_JPRB, 0.45272E+00_JPRB, 0.48055E+00_JPRB, 0.39637E+00_JPRB, 0.37003E+00_JPRB, 0.36685E+00_JPRB, 0.35514E+00_JPRB ,&
&  0.34606E+00_JPRB, 0.37106E+00_JPRB, 0.40003E+00_JPRB, 0.41322E+00_JPRB, 0.43624E+00_JPRB, 0.44782E+00_JPRB, 0.44912E+00_JPRB ,&
&  0.46294E+00_JPRB, 0.48469E+00_JPRB, 0.52740E+00_JPRB, 0.53473E+00_JPRB, 0.52108E+00_JPRB, 0.52824E+00_JPRB, 0.54501E+00_JPRB ,&
&  0.52858E+00_JPRB, 0.51260E+00_JPRB, 0.53025E+00_JPRB, 0.53155E+00_JPRB, 0.52760E+00_JPRB, 0.50137E+00_JPRB, 0.49011E+00_JPRB ,&
&  0.50590E+00_JPRB, 0.48666E+00_JPRB, 0.55481E+00_JPRB, 0.58485E+00_JPRB, 0.62055E+00_JPRB, 0.66156E+00_JPRB, 0.70338E+00_JPRB ,&
&  0.77440E+00_JPRB, 0.89804E+00_JPRB, 0.10541E+01_JPRB, 0.11956E+01_JPRB, 0.13988E+01_JPRB, 0.19145E+01_JPRB, 0.22850E+01_JPRB ,&
&  0.25691E+01_JPRB, 0.17880E+01_JPRB, 0.21029E+01_JPRB, 0.21410E+01_JPRB, 0.23186E+01_JPRB, 0.22298E+01_JPRB, 0.20521E+01_JPRB ,&
&  0.18881E+01_JPRB, 0.15915E+01_JPRB, 0.14349E+01_JPRB, 0.13289E+01_JPRB, 0.13194E+01_JPRB, 0.13839E+01_JPRB, 0.14167E+01_JPRB ,&
&  0.14545E+01_JPRB, 0.13819E+01_JPRB, 0.12543E+01_JPRB, 0.11285E+01_JPRB, 0.10326E+01_JPRB, 0.96988E+00_JPRB, 0.90493E+00_JPRB ,&
&  0.84683E+00_JPRB, 0.78834E+00_JPRB, 0.73166E+00_JPRB, 0.65903E+00_JPRB, 0.62329E+00_JPRB, 0.59369E+00_JPRB, 0.56319E+00_JPRB ,&
&  0.54698E+00_JPRB, 0.51646E+00_JPRB, 0.48277E+00_JPRB, 0.46741E+00_JPRB, 0.44980E+00_JPRB, 0.44659E+00_JPRB, 0.46000E+00_JPRB ,&
&  0.46174E+00_JPRB, 0.44518E+00_JPRB, 0.46343E+00_JPRB, 0.46685E+00_JPRB, 0.46397E+00_JPRB, 0.47051E+00_JPRB, 0.51791E+00_JPRB ,&
&  0.49552E+00_JPRB, 0.51100E+00_JPRB, 0.54700E+00_JPRB, 0.54211E+00_JPRB, 0.53973E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.54179E+00_JPRB, 0.53688E+00_JPRB, 0.52659E+00_JPRB, 0.51417E+00_JPRB, 0.51430E+00_JPRB, 0.49328E+00_JPRB, 0.43234E+00_JPRB ,&
&  0.43648E+00_JPRB, 0.39681E+00_JPRB, 0.36904E+00_JPRB, 0.35759E+00_JPRB, 0.35984E+00_JPRB, 0.35684E+00_JPRB, 0.35319E+00_JPRB ,&
&  0.35578E+00_JPRB, 0.37170E+00_JPRB, 0.39575E+00_JPRB, 0.40590E+00_JPRB, 0.41776E+00_JPRB, 0.42731E+00_JPRB, 0.42698E+00_JPRB ,&
&  0.43506E+00_JPRB, 0.44857E+00_JPRB, 0.46353E+00_JPRB, 0.47514E+00_JPRB, 0.46562E+00_JPRB, 0.45640E+00_JPRB, 0.44953E+00_JPRB ,&
&  0.44421E+00_JPRB, 0.44273E+00_JPRB, 0.44470E+00_JPRB, 0.46384E+00_JPRB, 0.45242E+00_JPRB, 0.44097E+00_JPRB, 0.45012E+00_JPRB ,&
&  0.36744E+00_JPRB, 0.37946E+00_JPRB, 0.47039E+00_JPRB, 0.50178E+00_JPRB, 0.53147E+00_JPRB, 0.56632E+00_JPRB, 0.62130E+00_JPRB ,&
&  0.68780E+00_JPRB, 0.77961E+00_JPRB, 0.89145E+00_JPRB, 0.10021E+01_JPRB, 0.11578E+01_JPRB, 0.14784E+01_JPRB, 0.17744E+01_JPRB ,&
&  0.20087E+01_JPRB, 0.15769E+01_JPRB, 0.14674E+01_JPRB, 0.15797E+01_JPRB, 0.18493E+01_JPRB, 0.16121E+01_JPRB, 0.14392E+01_JPRB ,&
&  0.12888E+01_JPRB, 0.11766E+01_JPRB, 0.11340E+01_JPRB, 0.11914E+01_JPRB, 0.12012E+01_JPRB, 0.12440E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.10284E+01_JPRB, 0.92703E+00_JPRB, 0.87492E+00_JPRB, 0.83983E+00_JPRB, 0.81341E+00_JPRB, 0.78777E+00_JPRB ,&
&  0.76719E+00_JPRB, 0.73490E+00_JPRB, 0.67662E+00_JPRB, 0.61351E+00_JPRB, 0.57078E+00_JPRB, 0.54660E+00_JPRB, 0.53011E+00_JPRB ,&
&  0.51467E+00_JPRB, 0.49946E+00_JPRB, 0.48839E+00_JPRB, 0.48270E+00_JPRB, 0.48060E+00_JPRB, 0.48246E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.46989E+00_JPRB, 0.44638E+00_JPRB, 0.42156E+00_JPRB, 0.42488E+00_JPRB, 0.44295E+00_JPRB, 0.46149E+00_JPRB, 0.47348E+00_JPRB ,&
&  0.49769E+00_JPRB, 0.50471E+00_JPRB, 0.49190E+00_JPRB, 0.50337E+00_JPRB, 0.51837E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.54281E+00_JPRB, 0.53600E+00_JPRB, 0.51721E+00_JPRB, 0.51031E+00_JPRB, 0.49416E+00_JPRB, 0.48461E+00_JPRB, 0.46014E+00_JPRB ,&
&  0.45374E+00_JPRB, 0.43111E+00_JPRB, 0.40391E+00_JPRB, 0.37517E+00_JPRB, 0.36227E+00_JPRB, 0.35826E+00_JPRB, 0.36170E+00_JPRB ,&
&  0.37145E+00_JPRB, 0.38278E+00_JPRB, 0.39489E+00_JPRB, 0.39862E+00_JPRB, 0.40754E+00_JPRB, 0.41486E+00_JPRB, 0.41650E+00_JPRB ,&
&  0.41886E+00_JPRB, 0.42343E+00_JPRB, 0.42255E+00_JPRB, 0.41631E+00_JPRB, 0.41403E+00_JPRB, 0.41593E+00_JPRB, 0.41376E+00_JPRB ,&
&  0.40458E+00_JPRB, 0.39914E+00_JPRB, 0.40833E+00_JPRB, 0.43759E+00_JPRB, 0.42185E+00_JPRB, 0.42606E+00_JPRB, 0.37960E+00_JPRB ,&
&  0.33167E+00_JPRB, 0.32293E+00_JPRB, 0.35046E+00_JPRB, 0.44481E+00_JPRB, 0.47351E+00_JPRB, 0.51181E+00_JPRB, 0.54776E+00_JPRB ,&
&  0.56229E+00_JPRB, 0.58729E+00_JPRB, 0.67960E+00_JPRB, 0.86795E+00_JPRB, 0.97157E+00_JPRB, 0.11821E+01_JPRB, 0.13647E+01_JPRB ,&
&  0.14810E+01_JPRB, 0.15164E+01_JPRB, 0.15182E+01_JPRB, 0.12710E+01_JPRB, 0.11926E+01_JPRB, 0.13171E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.10984E+01_JPRB, 0.10268E+01_JPRB, 0.10303E+01_JPRB, 0.10739E+01_JPRB, 0.10097E+01_JPRB, 0.10219E+01_JPRB, 0.99102E+00_JPRB ,&
&  0.90123E+00_JPRB, 0.84308E+00_JPRB, 0.79853E+00_JPRB, 0.77614E+00_JPRB, 0.75702E+00_JPRB, 0.74775E+00_JPRB, 0.75082E+00_JPRB ,&
&  0.77830E+00_JPRB, 0.77495E+00_JPRB, 0.69716E+00_JPRB, 0.58343E+00_JPRB, 0.53905E+00_JPRB, 0.52084E+00_JPRB, 0.50801E+00_JPRB ,&
&  0.49246E+00_JPRB, 0.48333E+00_JPRB, 0.48444E+00_JPRB, 0.48612E+00_JPRB, 0.47629E+00_JPRB, 0.45617E+00_JPRB, 0.43699E+00_JPRB ,&
&  0.42420E+00_JPRB, 0.41512E+00_JPRB, 0.40477E+00_JPRB, 0.42212E+00_JPRB, 0.44443E+00_JPRB, 0.46146E+00_JPRB, 0.45713E+00_JPRB ,&
&  0.45298E+00_JPRB, 0.46064E+00_JPRB, 0.47068E+00_JPRB, 0.49757E+00_JPRB, 0.51974E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.46949E+00_JPRB, 0.49082E+00_JPRB, 0.51393E+00_JPRB, 0.50962E+00_JPRB, 0.50783E+00_JPRB, 0.47859E+00_JPRB, 0.44682E+00_JPRB ,&
&  0.42550E+00_JPRB, 0.41434E+00_JPRB, 0.41039E+00_JPRB, 0.40363E+00_JPRB, 0.39908E+00_JPRB, 0.39805E+00_JPRB, 0.39877E+00_JPRB ,&
&  0.40084E+00_JPRB, 0.40313E+00_JPRB, 0.40559E+00_JPRB, 0.40857E+00_JPRB, 0.41528E+00_JPRB, 0.42226E+00_JPRB, 0.42308E+00_JPRB ,&
&  0.42041E+00_JPRB, 0.41961E+00_JPRB, 0.41075E+00_JPRB, 0.39758E+00_JPRB, 0.39236E+00_JPRB, 0.39663E+00_JPRB, 0.40576E+00_JPRB ,&
&  0.40330E+00_JPRB, 0.39438E+00_JPRB, 0.39978E+00_JPRB, 0.41446E+00_JPRB, 0.40842E+00_JPRB, 0.41575E+00_JPRB, 0.37674E+00_JPRB ,&
&  0.32952E+00_JPRB, 0.29731E+00_JPRB, 0.28521E+00_JPRB, 0.30907E+00_JPRB, 0.36337E+00_JPRB, 0.45070E+00_JPRB, 0.48950E+00_JPRB ,&
&  0.52656E+00_JPRB, 0.57427E+00_JPRB, 0.63102E+00_JPRB, 0.70632E+00_JPRB, 0.80574E+00_JPRB, 0.97078E+00_JPRB, 0.11097E+01_JPRB ,&
&  0.12014E+01_JPRB, 0.12279E+01_JPRB, 0.12068E+01_JPRB, 0.96713E+00_JPRB, 0.87092E+00_JPRB, 0.85106E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.99110E+00_JPRB, 0.90853E+00_JPRB, 0.84075E+00_JPRB, 0.83242E+00_JPRB, 0.80893E+00_JPRB, 0.81210E+00_JPRB, 0.81625E+00_JPRB ,&
&  0.80187E+00_JPRB, 0.77165E+00_JPRB, 0.73694E+00_JPRB, 0.70934E+00_JPRB, 0.70061E+00_JPRB, 0.70820E+00_JPRB, 0.71359E+00_JPRB ,&
&  0.74367E+00_JPRB, 0.80922E+00_JPRB, 0.77021E+00_JPRB, 0.59116E+00_JPRB, 0.55619E+00_JPRB, 0.54413E+00_JPRB, 0.52954E+00_JPRB ,&
&  0.50346E+00_JPRB, 0.48682E+00_JPRB, 0.48312E+00_JPRB, 0.48309E+00_JPRB, 0.47138E+00_JPRB, 0.46113E+00_JPRB, 0.45440E+00_JPRB ,&
&  0.45329E+00_JPRB, 0.45587E+00_JPRB, 0.45881E+00_JPRB, 0.47744E+00_JPRB, 0.48503E+00_JPRB, 0.48536E+00_JPRB, 0.47401E+00_JPRB ,&
&  0.46442E+00_JPRB, 0.46110E+00_JPRB, 0.45826E+00_JPRB, 0.45416E+00_JPRB, 0.45853E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.48372E+00_JPRB, 0.48807E+00_JPRB, 0.49678E+00_JPRB, 0.50270E+00_JPRB, 0.51062E+00_JPRB, 0.50190E+00_JPRB, 0.49268E+00_JPRB ,&
&  0.48931E+00_JPRB, 0.48719E+00_JPRB, 0.47520E+00_JPRB, 0.46284E+00_JPRB, 0.44696E+00_JPRB, 0.43526E+00_JPRB, 0.43265E+00_JPRB ,&
&  0.43465E+00_JPRB, 0.43576E+00_JPRB, 0.43662E+00_JPRB, 0.43446E+00_JPRB, 0.43500E+00_JPRB, 0.43819E+00_JPRB, 0.43504E+00_JPRB ,&
&  0.42932E+00_JPRB, 0.42398E+00_JPRB, 0.41597E+00_JPRB, 0.40740E+00_JPRB, 0.39534E+00_JPRB, 0.39156E+00_JPRB, 0.38887E+00_JPRB ,&
&  0.38581E+00_JPRB, 0.38713E+00_JPRB, 0.39733E+00_JPRB, 0.39313E+00_JPRB, 0.39707E+00_JPRB, 0.40482E+00_JPRB, 0.36041E+00_JPRB ,&
&  0.31625E+00_JPRB, 0.28665E+00_JPRB, 0.27102E+00_JPRB, 0.27149E+00_JPRB, 0.28927E+00_JPRB, 0.32884E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.45246E+00_JPRB, 0.49855E+00_JPRB, 0.54095E+00_JPRB, 0.58132E+00_JPRB, 0.62604E+00_JPRB, 0.72300E+00_JPRB, 0.80252E+00_JPRB ,&
&  0.88035E+00_JPRB, 0.96767E+00_JPRB, 0.10211E+01_JPRB, 0.10337E+01_JPRB, 0.10063E+01_JPRB, 0.89519E+00_JPRB, 0.90550E+00_JPRB ,&
&  0.10810E+01_JPRB, 0.90405E+00_JPRB, 0.74093E+00_JPRB, 0.66999E+00_JPRB, 0.65148E+00_JPRB, 0.65240E+00_JPRB, 0.65483E+00_JPRB ,&
&  0.66190E+00_JPRB, 0.66515E+00_JPRB, 0.66054E+00_JPRB, 0.65284E+00_JPRB, 0.64978E+00_JPRB, 0.64801E+00_JPRB, 0.65651E+00_JPRB ,&
&  0.67790E+00_JPRB, 0.79813E+00_JPRB, 0.10241E+01_JPRB, 0.76645E+00_JPRB, 0.66135E+00_JPRB, 0.59531E+00_JPRB, 0.53403E+00_JPRB ,&
&  0.50769E+00_JPRB, 0.49739E+00_JPRB, 0.49797E+00_JPRB, 0.49854E+00_JPRB, 0.49443E+00_JPRB, 0.49442E+00_JPRB, 0.49758E+00_JPRB ,&
&  0.50701E+00_JPRB, 0.51444E+00_JPRB, 0.51482E+00_JPRB, 0.51310E+00_JPRB, 0.50430E+00_JPRB, 0.49126E+00_JPRB, 0.48121E+00_JPRB ,&
&  0.48073E+00_JPRB, 0.48185E+00_JPRB, 0.48379E+00_JPRB, 0.48489E+00_JPRB, 0.48445E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.48695E+00_JPRB, 0.48719E+00_JPRB, 0.49203E+00_JPRB, 0.49580E+00_JPRB, 0.50240E+00_JPRB, 0.50753E+00_JPRB, 0.50930E+00_JPRB ,&
&  0.51662E+00_JPRB, 0.51312E+00_JPRB, 0.49991E+00_JPRB, 0.48398E+00_JPRB, 0.46777E+00_JPRB, 0.45773E+00_JPRB, 0.45375E+00_JPRB ,&
&  0.45586E+00_JPRB, 0.45329E+00_JPRB, 0.45220E+00_JPRB, 0.45345E+00_JPRB, 0.45629E+00_JPRB, 0.46008E+00_JPRB, 0.45592E+00_JPRB ,&
&  0.44062E+00_JPRB, 0.42955E+00_JPRB, 0.41676E+00_JPRB, 0.40068E+00_JPRB, 0.38536E+00_JPRB, 0.37511E+00_JPRB, 0.37875E+00_JPRB ,&
&  0.38684E+00_JPRB, 0.38220E+00_JPRB, 0.38075E+00_JPRB, 0.38223E+00_JPRB, 0.38594E+00_JPRB, 0.38288E+00_JPRB, 0.31634E+00_JPRB ,&
&  0.29051E+00_JPRB, 0.27383E+00_JPRB, 0.26764E+00_JPRB, 0.26779E+00_JPRB, 0.28147E+00_JPRB, 0.30751E+00_JPRB, 0.35461E+00_JPRB ,&
&  0.40242E+00_JPRB, 0.44941E+00_JPRB, 0.48080E+00_JPRB, 0.51545E+00_JPRB, 0.56671E+00_JPRB, 0.62930E+00_JPRB, 0.68134E+00_JPRB ,&
&  0.72940E+00_JPRB, 0.79570E+00_JPRB, 0.84264E+00_JPRB, 0.87127E+00_JPRB, 0.86292E+00_JPRB, 0.83163E+00_JPRB, 0.80332E+00_JPRB ,&
&  0.77856E+00_JPRB, 0.70138E+00_JPRB, 0.66599E+00_JPRB, 0.64118E+00_JPRB, 0.62530E+00_JPRB, 0.61868E+00_JPRB, 0.60155E+00_JPRB ,&
&  0.59442E+00_JPRB, 0.59772E+00_JPRB, 0.59598E+00_JPRB, 0.59578E+00_JPRB, 0.59523E+00_JPRB, 0.59156E+00_JPRB, 0.58914E+00_JPRB ,&
&  0.59234E+00_JPRB, 0.60463E+00_JPRB, 0.63530E+00_JPRB, 0.60841E+00_JPRB, 0.57731E+00_JPRB, 0.54673E+00_JPRB, 0.52729E+00_JPRB ,&
&  0.51965E+00_JPRB, 0.51832E+00_JPRB, 0.52063E+00_JPRB, 0.51867E+00_JPRB, 0.51851E+00_JPRB, 0.51987E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.53000E+00_JPRB, 0.53080E+00_JPRB, 0.52692E+00_JPRB, 0.51942E+00_JPRB, 0.50803E+00_JPRB, 0.49584E+00_JPRB, 0.48679E+00_JPRB ,&
&  0.48279E+00_JPRB, 0.48119E+00_JPRB, 0.48040E+00_JPRB, 0.48234E+00_JPRB, 0.48528E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.48218E+00_JPRB, 0.48197E+00_JPRB, 0.48575E+00_JPRB, 0.49051E+00_JPRB, 0.49637E+00_JPRB, 0.50259E+00_JPRB, 0.50543E+00_JPRB ,&
&  0.50720E+00_JPRB, 0.50554E+00_JPRB, 0.50183E+00_JPRB, 0.49070E+00_JPRB, 0.48259E+00_JPRB, 0.47894E+00_JPRB, 0.47484E+00_JPRB ,&
&  0.47017E+00_JPRB, 0.46790E+00_JPRB, 0.46723E+00_JPRB, 0.46852E+00_JPRB, 0.46749E+00_JPRB, 0.46942E+00_JPRB, 0.46515E+00_JPRB ,&
&  0.45164E+00_JPRB, 0.44046E+00_JPRB, 0.41995E+00_JPRB, 0.40124E+00_JPRB, 0.38828E+00_JPRB, 0.37938E+00_JPRB, 0.38129E+00_JPRB ,&
&  0.37879E+00_JPRB, 0.37757E+00_JPRB, 0.37772E+00_JPRB, 0.37939E+00_JPRB, 0.38385E+00_JPRB, 0.31526E+00_JPRB, 0.29550E+00_JPRB ,&
&  0.28113E+00_JPRB, 0.27172E+00_JPRB, 0.26950E+00_JPRB, 0.27007E+00_JPRB, 0.27676E+00_JPRB, 0.29662E+00_JPRB, 0.32760E+00_JPRB ,&
&  0.35419E+00_JPRB, 0.43055E+00_JPRB, 0.44905E+00_JPRB, 0.47155E+00_JPRB, 0.50916E+00_JPRB, 0.54949E+00_JPRB, 0.58432E+00_JPRB ,&
&  0.61316E+00_JPRB, 0.65044E+00_JPRB, 0.68500E+00_JPRB, 0.71221E+00_JPRB, 0.71055E+00_JPRB, 0.70975E+00_JPRB, 0.69758E+00_JPRB ,&
&  0.68493E+00_JPRB, 0.66160E+00_JPRB, 0.64692E+00_JPRB, 0.63363E+00_JPRB, 0.62173E+00_JPRB, 0.61069E+00_JPRB, 0.59952E+00_JPRB ,&
&  0.59286E+00_JPRB, 0.58122E+00_JPRB, 0.57960E+00_JPRB, 0.57465E+00_JPRB, 0.56998E+00_JPRB, 0.56753E+00_JPRB, 0.56455E+00_JPRB ,&
&  0.56069E+00_JPRB, 0.55557E+00_JPRB, 0.54548E+00_JPRB, 0.53103E+00_JPRB, 0.52252E+00_JPRB, 0.51400E+00_JPRB, 0.51221E+00_JPRB ,&
&  0.51263E+00_JPRB, 0.51960E+00_JPRB, 0.52062E+00_JPRB, 0.52131E+00_JPRB, 0.52280E+00_JPRB, 0.52086E+00_JPRB, 0.52063E+00_JPRB ,&
&  0.52032E+00_JPRB, 0.52106E+00_JPRB, 0.51772E+00_JPRB, 0.51118E+00_JPRB, 0.50147E+00_JPRB, 0.49159E+00_JPRB, 0.48374E+00_JPRB ,&
&  0.47992E+00_JPRB, 0.47916E+00_JPRB, 0.47597E+00_JPRB, 0.47687E+00_JPRB, 0.47914E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.47595E+00_JPRB, 0.47432E+00_JPRB, 0.47417E+00_JPRB, 0.47781E+00_JPRB, 0.48544E+00_JPRB, 0.49415E+00_JPRB, 0.49391E+00_JPRB ,&
&  0.49368E+00_JPRB, 0.49154E+00_JPRB, 0.49237E+00_JPRB, 0.49112E+00_JPRB, 0.48899E+00_JPRB, 0.48513E+00_JPRB, 0.48178E+00_JPRB ,&
&  0.47901E+00_JPRB, 0.47309E+00_JPRB, 0.46581E+00_JPRB, 0.46365E+00_JPRB, 0.46463E+00_JPRB, 0.46632E+00_JPRB, 0.46646E+00_JPRB ,&
&  0.46383E+00_JPRB, 0.45518E+00_JPRB, 0.43026E+00_JPRB, 0.41107E+00_JPRB, 0.39312E+00_JPRB, 0.38784E+00_JPRB, 0.37976E+00_JPRB ,&
&  0.39056E+00_JPRB, 0.35380E+00_JPRB, 0.34988E+00_JPRB, 0.33244E+00_JPRB, 0.31702E+00_JPRB, 0.30108E+00_JPRB, 0.28916E+00_JPRB ,&
&  0.28467E+00_JPRB, 0.28045E+00_JPRB, 0.28029E+00_JPRB, 0.28591E+00_JPRB, 0.29454E+00_JPRB, 0.31162E+00_JPRB, 0.33285E+00_JPRB ,&
&  0.35370E+00_JPRB, 0.40952E+00_JPRB, 0.42728E+00_JPRB, 0.44023E+00_JPRB, 0.46242E+00_JPRB, 0.48383E+00_JPRB, 0.50368E+00_JPRB ,&
&  0.52048E+00_JPRB, 0.53736E+00_JPRB, 0.55532E+00_JPRB, 0.56259E+00_JPRB, 0.57395E+00_JPRB, 0.57302E+00_JPRB, 0.57259E+00_JPRB ,&
&  0.57361E+00_JPRB, 0.57315E+00_JPRB, 0.57070E+00_JPRB, 0.56624E+00_JPRB, 0.56096E+00_JPRB, 0.55399E+00_JPRB, 0.54857E+00_JPRB ,&
&  0.54178E+00_JPRB, 0.53527E+00_JPRB, 0.52752E+00_JPRB, 0.52052E+00_JPRB, 0.51307E+00_JPRB, 0.50885E+00_JPRB, 0.50378E+00_JPRB ,&
&  0.49959E+00_JPRB, 0.49419E+00_JPRB, 0.48887E+00_JPRB, 0.48578E+00_JPRB, 0.48648E+00_JPRB, 0.48642E+00_JPRB, 0.48963E+00_JPRB ,&
&  0.49628E+00_JPRB, 0.50068E+00_JPRB, 0.50481E+00_JPRB, 0.50819E+00_JPRB, 0.51077E+00_JPRB, 0.50972E+00_JPRB, 0.50704E+00_JPRB ,&
&  0.50594E+00_JPRB, 0.50499E+00_JPRB, 0.50240E+00_JPRB, 0.49634E+00_JPRB, 0.48903E+00_JPRB, 0.48217E+00_JPRB, 0.47779E+00_JPRB ,&
&  0.47402E+00_JPRB, 0.47285E+00_JPRB, 0.46956E+00_JPRB, 0.47135E+00_JPRB, 0.47339E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.46508E+00_JPRB, 0.46482E+00_JPRB, 0.46417E+00_JPRB, 0.46571E+00_JPRB, 0.46839E+00_JPRB, 0.47297E+00_JPRB, 0.47419E+00_JPRB ,&
&  0.47833E+00_JPRB, 0.48204E+00_JPRB, 0.48587E+00_JPRB, 0.48697E+00_JPRB, 0.48748E+00_JPRB, 0.48599E+00_JPRB, 0.48314E+00_JPRB ,&
&  0.48030E+00_JPRB, 0.47637E+00_JPRB, 0.47330E+00_JPRB, 0.46982E+00_JPRB, 0.46633E+00_JPRB, 0.46466E+00_JPRB, 0.46368E+00_JPRB ,&
&  0.46353E+00_JPRB, 0.46263E+00_JPRB, 0.42972E+00_JPRB, 0.41664E+00_JPRB, 0.39901E+00_JPRB, 0.38973E+00_JPRB, 0.38578E+00_JPRB ,&
&  0.38151E+00_JPRB, 0.37711E+00_JPRB, 0.37154E+00_JPRB, 0.36117E+00_JPRB, 0.34683E+00_JPRB, 0.32798E+00_JPRB, 0.31731E+00_JPRB ,&
&  0.30840E+00_JPRB, 0.30579E+00_JPRB, 0.30721E+00_JPRB, 0.31326E+00_JPRB, 0.32446E+00_JPRB, 0.33776E+00_JPRB, 0.35366E+00_JPRB ,&
&  0.37546E+00_JPRB, 0.39943E+00_JPRB, 0.41568E+00_JPRB, 0.41978E+00_JPRB, 0.43615E+00_JPRB, 0.44682E+00_JPRB, 0.45570E+00_JPRB ,&
&  0.46246E+00_JPRB, 0.46781E+00_JPRB, 0.47911E+00_JPRB, 0.48523E+00_JPRB, 0.48833E+00_JPRB, 0.49153E+00_JPRB, 0.49536E+00_JPRB ,&
&  0.49791E+00_JPRB, 0.49948E+00_JPRB, 0.49867E+00_JPRB, 0.49721E+00_JPRB, 0.49444E+00_JPRB, 0.49071E+00_JPRB, 0.48730E+00_JPRB ,&
&  0.48403E+00_JPRB, 0.48096E+00_JPRB, 0.47645E+00_JPRB, 0.47171E+00_JPRB, 0.46837E+00_JPRB, 0.46534E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.46039E+00_JPRB, 0.45916E+00_JPRB, 0.45836E+00_JPRB, 0.45545E+00_JPRB, 0.45675E+00_JPRB, 0.46302E+00_JPRB, 0.46584E+00_JPRB ,&
&  0.47112E+00_JPRB, 0.47534E+00_JPRB, 0.48007E+00_JPRB, 0.48378E+00_JPRB, 0.48639E+00_JPRB, 0.48723E+00_JPRB, 0.48612E+00_JPRB ,&
&  0.48544E+00_JPRB, 0.48417E+00_JPRB, 0.48219E+00_JPRB, 0.47735E+00_JPRB, 0.47259E+00_JPRB, 0.46798E+00_JPRB, 0.46538E+00_JPRB ,&
&  0.46224E+00_JPRB, 0.46071E+00_JPRB, 0.45844E+00_JPRB, 0.46056E+00_JPRB, 0.46253E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.45748E+00_JPRB, 0.45868E+00_JPRB, 0.45965E+00_JPRB, 0.46100E+00_JPRB, 0.46341E+00_JPRB, 0.46637E+00_JPRB, 0.46826E+00_JPRB ,&
&  0.47097E+00_JPRB, 0.47402E+00_JPRB, 0.47771E+00_JPRB, 0.47964E+00_JPRB, 0.48084E+00_JPRB, 0.48087E+00_JPRB, 0.48022E+00_JPRB ,&
&  0.47861E+00_JPRB, 0.47701E+00_JPRB, 0.47531E+00_JPRB, 0.47407E+00_JPRB, 0.47287E+00_JPRB, 0.47176E+00_JPRB, 0.46959E+00_JPRB ,&
&  0.46742E+00_JPRB, 0.46685E+00_JPRB, 0.46316E+00_JPRB, 0.45768E+00_JPRB, 0.45318E+00_JPRB, 0.41696E+00_JPRB, 0.40473E+00_JPRB ,&
&  0.40152E+00_JPRB, 0.39857E+00_JPRB, 0.39986E+00_JPRB, 0.40222E+00_JPRB, 0.41165E+00_JPRB, 0.40816E+00_JPRB, 0.40580E+00_JPRB ,&
&  0.38474E+00_JPRB, 0.38210E+00_JPRB, 0.37816E+00_JPRB, 0.37669E+00_JPRB, 0.37994E+00_JPRB, 0.38384E+00_JPRB, 0.39325E+00_JPRB ,&
&  0.41562E+00_JPRB, 0.42289E+00_JPRB, 0.42634E+00_JPRB, 0.43000E+00_JPRB, 0.43486E+00_JPRB, 0.43879E+00_JPRB, 0.44187E+00_JPRB ,&
&  0.44430E+00_JPRB, 0.44632E+00_JPRB, 0.44828E+00_JPRB, 0.44951E+00_JPRB, 0.45079E+00_JPRB, 0.45123E+00_JPRB, 0.45132E+00_JPRB ,&
&  0.45101E+00_JPRB, 0.45137E+00_JPRB, 0.45243E+00_JPRB, 0.45334E+00_JPRB, 0.45442E+00_JPRB, 0.45458E+00_JPRB, 0.45447E+00_JPRB ,&
&  0.45422E+00_JPRB, 0.45357E+00_JPRB, 0.45270E+00_JPRB, 0.45196E+00_JPRB, 0.45199E+00_JPRB, 0.45122E+00_JPRB, 0.45083E+00_JPRB ,&
&  0.45033E+00_JPRB, 0.45054E+00_JPRB, 0.45061E+00_JPRB, 0.45215E+00_JPRB, 0.45323E+00_JPRB, 0.45562E+00_JPRB, 0.45769E+00_JPRB ,&
&  0.46079E+00_JPRB, 0.46307E+00_JPRB, 0.46497E+00_JPRB, 0.46595E+00_JPRB, 0.46674E+00_JPRB, 0.46716E+00_JPRB, 0.46655E+00_JPRB ,&
&  0.46574E+00_JPRB, 0.46426E+00_JPRB, 0.46229E+00_JPRB, 0.45943E+00_JPRB, 0.45686E+00_JPRB, 0.45452E+00_JPRB, 0.45285E+00_JPRB ,&
&  0.45108E+00_JPRB, 0.45086E+00_JPRB, 0.45197E+00_JPRB, 0.45398E+00_JPRB, 0.45588E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.46271E+00_JPRB, 0.46401E+00_JPRB, 0.46454E+00_JPRB, 0.46492E+00_JPRB, 0.46526E+00_JPRB, 0.46651E+00_JPRB, 0.46762E+00_JPRB ,&
&  0.46878E+00_JPRB, 0.47020E+00_JPRB, 0.47231E+00_JPRB, 0.47354E+00_JPRB, 0.47448E+00_JPRB, 0.47492E+00_JPRB, 0.47545E+00_JPRB ,&
&  0.47555E+00_JPRB, 0.47529E+00_JPRB, 0.47461E+00_JPRB, 0.47422E+00_JPRB, 0.47391E+00_JPRB, 0.47384E+00_JPRB, 0.47357E+00_JPRB ,&
&  0.47267E+00_JPRB, 0.47154E+00_JPRB, 0.47025E+00_JPRB, 0.46807E+00_JPRB, 0.46525E+00_JPRB, 0.46211E+00_JPRB, 0.45853E+00_JPRB ,&
&  0.45486E+00_JPRB, 0.45204E+00_JPRB, 0.44921E+00_JPRB, 0.44655E+00_JPRB, 0.44554E+00_JPRB, 0.44468E+00_JPRB, 0.44375E+00_JPRB ,&
&  0.44288E+00_JPRB, 0.44164E+00_JPRB, 0.44154E+00_JPRB, 0.44187E+00_JPRB, 0.44257E+00_JPRB, 0.44401E+00_JPRB, 0.44534E+00_JPRB ,&
&  0.44631E+00_JPRB, 0.44639E+00_JPRB, 0.44584E+00_JPRB, 0.44610E+00_JPRB, 0.44685E+00_JPRB, 0.44711E+00_JPRB, 0.44657E+00_JPRB ,&
&  0.44619E+00_JPRB, 0.44524E+00_JPRB, 0.44413E+00_JPRB, 0.44293E+00_JPRB, 0.44261E+00_JPRB, 0.44187E+00_JPRB, 0.44092E+00_JPRB ,&
&  0.44018E+00_JPRB, 0.43982E+00_JPRB, 0.43984E+00_JPRB, 0.43998E+00_JPRB, 0.44077E+00_JPRB, 0.44157E+00_JPRB, 0.44240E+00_JPRB ,&
&  0.44323E+00_JPRB, 0.44487E+00_JPRB, 0.44640E+00_JPRB, 0.44774E+00_JPRB, 0.44921E+00_JPRB, 0.45069E+00_JPRB, 0.45213E+00_JPRB ,&
&  0.45326E+00_JPRB, 0.45404E+00_JPRB, 0.45397E+00_JPRB, 0.45434E+00_JPRB, 0.45482E+00_JPRB, 0.45585E+00_JPRB, 0.45649E+00_JPRB ,&
&  0.45770E+00_JPRB, 0.45841E+00_JPRB, 0.45882E+00_JPRB, 0.45930E+00_JPRB, 0.45953E+00_JPRB, 0.45945E+00_JPRB, 0.45892E+00_JPRB ,&
&  0.45827E+00_JPRB, 0.45787E+00_JPRB, 0.45751E+00_JPRB, 0.45709E+00_JPRB, 0.45700E+00_JPRB, 0.45707E+00_JPRB, 0.45716E+00_JPRB ,&
&  0.45754E+00_JPRB, 0.45857E+00_JPRB, 0.45992E+00_JPRB, 0.46088E+00_JPRB, 0.46170E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.46619E+00_JPRB, 0.46626E+00_JPRB, 0.46621E+00_JPRB, 0.46603E+00_JPRB, 0.46595E+00_JPRB, 0.46611E+00_JPRB, 0.46621E+00_JPRB ,&
&  0.46638E+00_JPRB, 0.46656E+00_JPRB, 0.46711E+00_JPRB, 0.46741E+00_JPRB, 0.46775E+00_JPRB, 0.46793E+00_JPRB, 0.46825E+00_JPRB ,&
&  0.46849E+00_JPRB, 0.46873E+00_JPRB, 0.46889E+00_JPRB, 0.46918E+00_JPRB, 0.46949E+00_JPRB, 0.46996E+00_JPRB, 0.47034E+00_JPRB ,&
&  0.47044E+00_JPRB, 0.47015E+00_JPRB, 0.47001E+00_JPRB, 0.46970E+00_JPRB, 0.46932E+00_JPRB, 0.46845E+00_JPRB, 0.46741E+00_JPRB ,&
&  0.46613E+00_JPRB, 0.46498E+00_JPRB, 0.46404E+00_JPRB, 0.46320E+00_JPRB, 0.46260E+00_JPRB, 0.46215E+00_JPRB, 0.46178E+00_JPRB ,&
&  0.46126E+00_JPRB, 0.46088E+00_JPRB, 0.46055E+00_JPRB, 0.46028E+00_JPRB, 0.45974E+00_JPRB, 0.45908E+00_JPRB, 0.45838E+00_JPRB ,&
&  0.45758E+00_JPRB, 0.45663E+00_JPRB, 0.45585E+00_JPRB, 0.45518E+00_JPRB, 0.45437E+00_JPRB, 0.45313E+00_JPRB, 0.45178E+00_JPRB ,&
&  0.45043E+00_JPRB, 0.44893E+00_JPRB, 0.44735E+00_JPRB, 0.44593E+00_JPRB, 0.44496E+00_JPRB, 0.44411E+00_JPRB, 0.44323E+00_JPRB ,&
&  0.44298E+00_JPRB, 0.44336E+00_JPRB, 0.44376E+00_JPRB, 0.44401E+00_JPRB, 0.44467E+00_JPRB, 0.44533E+00_JPRB, 0.44608E+00_JPRB ,&
&  0.44676E+00_JPRB, 0.44772E+00_JPRB, 0.44891E+00_JPRB, 0.44998E+00_JPRB, 0.45121E+00_JPRB, 0.45250E+00_JPRB, 0.45396E+00_JPRB ,&
&  0.45538E+00_JPRB, 0.45687E+00_JPRB, 0.45834E+00_JPRB, 0.45978E+00_JPRB, 0.46059E+00_JPRB, 0.46125E+00_JPRB, 0.46143E+00_JPRB ,&
&  0.46170E+00_JPRB, 0.46184E+00_JPRB, 0.46216E+00_JPRB, 0.46239E+00_JPRB, 0.46272E+00_JPRB, 0.46295E+00_JPRB, 0.46322E+00_JPRB ,&
&  0.46354E+00_JPRB, 0.46402E+00_JPRB, 0.46436E+00_JPRB, 0.46468E+00_JPRB, 0.46487E+00_JPRB, 0.46524E+00_JPRB, 0.46551E+00_JPRB ,&
&  0.46572E+00_JPRB, 0.46602E+00_JPRB, 0.46631E+00_JPRB, 0.46634E+00_JPRB, 0.46628E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB, 0.46085E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB ,&
&  0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB, 0.19225E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.19568E+00_JPRB, 0.19598E+00_JPRB, 0.19610E+00_JPRB, 0.19612E+00_JPRB, 0.19477E+00_JPRB, 0.19432E+00_JPRB, 0.19386E+00_JPRB ,&
&  0.19352E+00_JPRB, 0.19335E+00_JPRB, 0.19373E+00_JPRB, 0.19420E+00_JPRB, 0.19468E+00_JPRB, 0.19515E+00_JPRB, 0.19552E+00_JPRB ,&
&  0.19567E+00_JPRB, 0.19583E+00_JPRB, 0.19490E+00_JPRB, 0.19447E+00_JPRB, 0.19436E+00_JPRB, 0.19445E+00_JPRB, 0.19445E+00_JPRB ,&
&  0.19459E+00_JPRB, 0.19469E+00_JPRB, 0.19470E+00_JPRB, 0.19500E+00_JPRB, 0.19553E+00_JPRB, 0.19597E+00_JPRB, 0.19644E+00_JPRB ,&
&  0.19688E+00_JPRB, 0.19751E+00_JPRB, 0.19808E+00_JPRB, 0.19861E+00_JPRB, 0.19924E+00_JPRB, 0.19983E+00_JPRB, 0.20013E+00_JPRB ,&
&  0.20033E+00_JPRB, 0.20032E+00_JPRB, 0.20037E+00_JPRB, 0.20041E+00_JPRB, 0.20056E+00_JPRB, 0.20076E+00_JPRB, 0.20076E+00_JPRB ,&
&  0.20065E+00_JPRB, 0.20047E+00_JPRB, 0.20022E+00_JPRB, 0.19971E+00_JPRB, 0.19918E+00_JPRB, 0.19867E+00_JPRB, 0.19818E+00_JPRB ,&
&  0.19774E+00_JPRB, 0.19726E+00_JPRB, 0.19673E+00_JPRB, 0.19616E+00_JPRB, 0.19557E+00_JPRB, 0.19499E+00_JPRB, 0.19441E+00_JPRB ,&
&  0.19382E+00_JPRB, 0.19343E+00_JPRB, 0.19301E+00_JPRB, 0.19265E+00_JPRB, 0.19238E+00_JPRB, 0.19211E+00_JPRB, 0.19185E+00_JPRB ,&
&  0.19159E+00_JPRB, 0.19127E+00_JPRB, 0.19098E+00_JPRB, 0.19065E+00_JPRB, 0.19036E+00_JPRB, 0.19003E+00_JPRB, 0.18978E+00_JPRB ,&
&  0.18953E+00_JPRB, 0.18931E+00_JPRB, 0.18913E+00_JPRB, 0.18904E+00_JPRB, 0.18900E+00_JPRB, 0.18903E+00_JPRB, 0.18907E+00_JPRB ,&
&  0.18914E+00_JPRB, 0.18918E+00_JPRB, 0.18922E+00_JPRB, 0.18913E+00_JPRB, 0.18930E+00_JPRB, 0.18950E+00_JPRB, 0.18977E+00_JPRB ,&
&  0.19003E+00_JPRB, 0.19044E+00_JPRB, 0.19085E+00_JPRB, 0.19130E+00_JPRB, 0.19198E+00_JPRB, 0.19223E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.19269E+00_JPRB, 0.19319E+00_JPRB, 0.19364E+00_JPRB, 0.19388E+00_JPRB, 0.19441E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.20836E+00_JPRB, 0.21338E+00_JPRB, 0.21734E+00_JPRB, 0.22123E+00_JPRB, 0.22054E+00_JPRB, 0.23988E+00_JPRB, 0.23781E+00_JPRB ,&
&  0.23735E+00_JPRB, 0.23025E+00_JPRB, 0.23094E+00_JPRB, 0.22947E+00_JPRB, 0.22704E+00_JPRB, 0.22399E+00_JPRB, 0.22074E+00_JPRB ,&
&  0.21740E+00_JPRB, 0.21349E+00_JPRB, 0.20985E+00_JPRB, 0.20639E+00_JPRB, 0.20353E+00_JPRB, 0.20105E+00_JPRB, 0.19967E+00_JPRB ,&
&  0.19966E+00_JPRB, 0.20104E+00_JPRB, 0.20344E+00_JPRB, 0.20491E+00_JPRB, 0.20645E+00_JPRB, 0.20806E+00_JPRB, 0.20975E+00_JPRB ,&
&  0.21124E+00_JPRB, 0.21314E+00_JPRB, 0.21417E+00_JPRB, 0.21388E+00_JPRB, 0.21269E+00_JPRB, 0.21200E+00_JPRB, 0.21163E+00_JPRB ,&
&  0.21186E+00_JPRB, 0.21186E+00_JPRB, 0.21176E+00_JPRB, 0.21183E+00_JPRB, 0.21187E+00_JPRB, 0.21125E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.20903E+00_JPRB, 0.20822E+00_JPRB, 0.20674E+00_JPRB, 0.20532E+00_JPRB, 0.20399E+00_JPRB, 0.20260E+00_JPRB, 0.20119E+00_JPRB ,&
&  0.20015E+00_JPRB, 0.19911E+00_JPRB, 0.19782E+00_JPRB, 0.19631E+00_JPRB, 0.19488E+00_JPRB, 0.19360E+00_JPRB, 0.19258E+00_JPRB ,&
&  0.19231E+00_JPRB, 0.19176E+00_JPRB, 0.19127E+00_JPRB, 0.19080E+00_JPRB, 0.19066E+00_JPRB, 0.19028E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.18983E+00_JPRB, 0.18960E+00_JPRB, 0.18935E+00_JPRB, 0.18911E+00_JPRB, 0.18896E+00_JPRB, 0.18872E+00_JPRB, 0.18850E+00_JPRB ,&
&  0.18837E+00_JPRB, 0.18827E+00_JPRB, 0.18814E+00_JPRB, 0.18783E+00_JPRB, 0.18751E+00_JPRB, 0.18711E+00_JPRB, 0.18713E+00_JPRB ,&
&  0.18728E+00_JPRB, 0.18757E+00_JPRB, 0.18805E+00_JPRB, 0.18829E+00_JPRB, 0.18907E+00_JPRB, 0.18982E+00_JPRB, 0.19071E+00_JPRB ,&
&  0.19166E+00_JPRB, 0.19309E+00_JPRB, 0.19464E+00_JPRB, 0.19631E+00_JPRB, 0.19889E+00_JPRB, 0.19994E+00_JPRB, 0.20086E+00_JPRB ,&
&  0.20195E+00_JPRB, 0.20515E+00_JPRB, 0.20640E+00_JPRB, 0.20663E+00_JPRB, 0.20762E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.25087E+00_JPRB, 0.25380E+00_JPRB, 0.25583E+00_JPRB, 0.25718E+00_JPRB, 0.25846E+00_JPRB, 0.25780E+00_JPRB, 0.25636E+00_JPRB ,&
&  0.25238E+00_JPRB, 0.25096E+00_JPRB, 0.24827E+00_JPRB, 0.24549E+00_JPRB, 0.24223E+00_JPRB, 0.23881E+00_JPRB, 0.23480E+00_JPRB ,&
&  0.23039E+00_JPRB, 0.22564E+00_JPRB, 0.21989E+00_JPRB, 0.21376E+00_JPRB, 0.20816E+00_JPRB, 0.20295E+00_JPRB, 0.19965E+00_JPRB ,&
&  0.19810E+00_JPRB, 0.19871E+00_JPRB, 0.20131E+00_JPRB, 0.20467E+00_JPRB, 0.20877E+00_JPRB, 0.21343E+00_JPRB, 0.21924E+00_JPRB ,&
&  0.22428E+00_JPRB, 0.23038E+00_JPRB, 0.23412E+00_JPRB, 0.23577E+00_JPRB, 0.23438E+00_JPRB, 0.23421E+00_JPRB, 0.23319E+00_JPRB ,&
&  0.23182E+00_JPRB, 0.22957E+00_JPRB, 0.22843E+00_JPRB, 0.22816E+00_JPRB, 0.22699E+00_JPRB, 0.22481E+00_JPRB, 0.22202E+00_JPRB ,&
&  0.21952E+00_JPRB, 0.21739E+00_JPRB, 0.21487E+00_JPRB, 0.21273E+00_JPRB, 0.21110E+00_JPRB, 0.20917E+00_JPRB, 0.20669E+00_JPRB ,&
&  0.20442E+00_JPRB, 0.20213E+00_JPRB, 0.19942E+00_JPRB, 0.19707E+00_JPRB, 0.19473E+00_JPRB, 0.19303E+00_JPRB, 0.19151E+00_JPRB ,&
&  0.19082E+00_JPRB, 0.19012E+00_JPRB, 0.18989E+00_JPRB, 0.18977E+00_JPRB, 0.19014E+00_JPRB, 0.18991E+00_JPRB, 0.18957E+00_JPRB ,&
&  0.18914E+00_JPRB, 0.18903E+00_JPRB, 0.18856E+00_JPRB, 0.18840E+00_JPRB, 0.18824E+00_JPRB, 0.18769E+00_JPRB, 0.18745E+00_JPRB ,&
&  0.18760E+00_JPRB, 0.18761E+00_JPRB, 0.18721E+00_JPRB, 0.18722E+00_JPRB, 0.18698E+00_JPRB, 0.18659E+00_JPRB, 0.18687E+00_JPRB ,&
&  0.18673E+00_JPRB, 0.18695E+00_JPRB, 0.18727E+00_JPRB, 0.18778E+00_JPRB, 0.18882E+00_JPRB, 0.19016E+00_JPRB, 0.19196E+00_JPRB ,&
&  0.19466E+00_JPRB, 0.19714E+00_JPRB, 0.19965E+00_JPRB, 0.20262E+00_JPRB, 0.20796E+00_JPRB, 0.21114E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.22181E+00_JPRB, 0.22238E+00_JPRB, 0.24488E+00_JPRB, 0.24588E+00_JPRB, 0.24748E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.26407E+00_JPRB, 0.26605E+00_JPRB, 0.26724E+00_JPRB, 0.26927E+00_JPRB, 0.27153E+00_JPRB, 0.27239E+00_JPRB, 0.27331E+00_JPRB ,&
&  0.27064E+00_JPRB, 0.26381E+00_JPRB, 0.25885E+00_JPRB, 0.25248E+00_JPRB, 0.24603E+00_JPRB, 0.23951E+00_JPRB, 0.23348E+00_JPRB ,&
&  0.22742E+00_JPRB, 0.22197E+00_JPRB, 0.21654E+00_JPRB, 0.21198E+00_JPRB, 0.20891E+00_JPRB, 0.20638E+00_JPRB, 0.20352E+00_JPRB ,&
&  0.20104E+00_JPRB, 0.20185E+00_JPRB, 0.20417E+00_JPRB, 0.20881E+00_JPRB, 0.21618E+00_JPRB, 0.22323E+00_JPRB, 0.23089E+00_JPRB ,&
&  0.24468E+00_JPRB, 0.24890E+00_JPRB, 0.24871E+00_JPRB, 0.25047E+00_JPRB, 0.24976E+00_JPRB, 0.25024E+00_JPRB, 0.24007E+00_JPRB ,&
&  0.24143E+00_JPRB, 0.24340E+00_JPRB, 0.25041E+00_JPRB, 0.25373E+00_JPRB, 0.24898E+00_JPRB, 0.24440E+00_JPRB, 0.23643E+00_JPRB ,&
&  0.22950E+00_JPRB, 0.22418E+00_JPRB, 0.22067E+00_JPRB, 0.21851E+00_JPRB, 0.21445E+00_JPRB, 0.21146E+00_JPRB, 0.20647E+00_JPRB ,&
&  0.20220E+00_JPRB, 0.19926E+00_JPRB, 0.19560E+00_JPRB, 0.19327E+00_JPRB, 0.19140E+00_JPRB, 0.18965E+00_JPRB, 0.18832E+00_JPRB ,&
&  0.18759E+00_JPRB, 0.18700E+00_JPRB, 0.18761E+00_JPRB, 0.18835E+00_JPRB, 0.18974E+00_JPRB, 0.19086E+00_JPRB, 0.19116E+00_JPRB ,&
&  0.19145E+00_JPRB, 0.19181E+00_JPRB, 0.19188E+00_JPRB, 0.19235E+00_JPRB, 0.19185E+00_JPRB, 0.19108E+00_JPRB, 0.19012E+00_JPRB ,&
&  0.18913E+00_JPRB, 0.18880E+00_JPRB, 0.18819E+00_JPRB, 0.18783E+00_JPRB, 0.18643E+00_JPRB, 0.18545E+00_JPRB, 0.18491E+00_JPRB ,&
&  0.18441E+00_JPRB, 0.18476E+00_JPRB, 0.18489E+00_JPRB, 0.18565E+00_JPRB, 0.18686E+00_JPRB, 0.18801E+00_JPRB, 0.18998E+00_JPRB ,&
&  0.19248E+00_JPRB, 0.19538E+00_JPRB, 0.20016E+00_JPRB, 0.20538E+00_JPRB, 0.21115E+00_JPRB, 0.21892E+00_JPRB, 0.22670E+00_JPRB ,&
&  0.23740E+00_JPRB, 0.25367E+00_JPRB, 0.25589E+00_JPRB, 0.25787E+00_JPRB, 0.26177E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.27364E+00_JPRB, 0.27468E+00_JPRB, 0.27615E+00_JPRB, 0.27849E+00_JPRB, 0.28139E+00_JPRB, 0.28256E+00_JPRB, 0.28286E+00_JPRB ,&
&  0.27379E+00_JPRB, 0.26602E+00_JPRB, 0.25924E+00_JPRB, 0.24887E+00_JPRB, 0.24064E+00_JPRB, 0.23316E+00_JPRB, 0.22772E+00_JPRB ,&
&  0.22325E+00_JPRB, 0.22078E+00_JPRB, 0.21841E+00_JPRB, 0.21764E+00_JPRB, 0.21781E+00_JPRB, 0.21864E+00_JPRB, 0.21739E+00_JPRB ,&
&  0.21579E+00_JPRB, 0.21669E+00_JPRB, 0.21729E+00_JPRB, 0.22175E+00_JPRB, 0.22818E+00_JPRB, 0.23553E+00_JPRB, 0.24270E+00_JPRB ,&
&  0.25181E+00_JPRB, 0.25586E+00_JPRB, 0.25895E+00_JPRB, 0.25971E+00_JPRB, 0.25822E+00_JPRB, 0.25741E+00_JPRB, 0.25519E+00_JPRB ,&
&  0.25446E+00_JPRB, 0.25379E+00_JPRB, 0.25473E+00_JPRB, 0.25564E+00_JPRB, 0.25438E+00_JPRB, 0.25150E+00_JPRB, 0.24558E+00_JPRB ,&
&  0.23727E+00_JPRB, 0.23045E+00_JPRB, 0.22444E+00_JPRB, 0.21982E+00_JPRB, 0.21385E+00_JPRB, 0.20855E+00_JPRB, 0.20326E+00_JPRB ,&
&  0.19858E+00_JPRB, 0.19498E+00_JPRB, 0.19081E+00_JPRB, 0.18771E+00_JPRB, 0.18629E+00_JPRB, 0.18463E+00_JPRB, 0.18391E+00_JPRB ,&
&  0.18386E+00_JPRB, 0.18419E+00_JPRB, 0.18604E+00_JPRB, 0.18762E+00_JPRB, 0.19008E+00_JPRB, 0.19291E+00_JPRB, 0.19474E+00_JPRB ,&
&  0.19776E+00_JPRB, 0.20037E+00_JPRB, 0.20176E+00_JPRB, 0.20271E+00_JPRB, 0.20177E+00_JPRB, 0.19920E+00_JPRB, 0.19649E+00_JPRB ,&
&  0.19310E+00_JPRB, 0.19064E+00_JPRB, 0.18905E+00_JPRB, 0.18735E+00_JPRB, 0.18651E+00_JPRB, 0.18581E+00_JPRB, 0.18510E+00_JPRB ,&
&  0.18457E+00_JPRB, 0.18437E+00_JPRB, 0.18412E+00_JPRB, 0.18502E+00_JPRB, 0.18603E+00_JPRB, 0.18786E+00_JPRB, 0.19010E+00_JPRB ,&
&  0.19223E+00_JPRB, 0.19547E+00_JPRB, 0.19902E+00_JPRB, 0.20384E+00_JPRB, 0.21219E+00_JPRB, 0.21953E+00_JPRB, 0.23534E+00_JPRB ,&
&  0.24916E+00_JPRB, 0.27815E+00_JPRB, 0.28961E+00_JPRB, 0.27620E+00_JPRB, 0.27423E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.28794E+00_JPRB, 0.28515E+00_JPRB, 0.28262E+00_JPRB, 0.28237E+00_JPRB, 0.28294E+00_JPRB, 0.28598E+00_JPRB, 0.28384E+00_JPRB ,&
&  0.27952E+00_JPRB, 0.27800E+00_JPRB, 0.27942E+00_JPRB, 0.27987E+00_JPRB, 0.27901E+00_JPRB, 0.26822E+00_JPRB, 0.26211E+00_JPRB ,&
&  0.25539E+00_JPRB, 0.25289E+00_JPRB, 0.25095E+00_JPRB, 0.25161E+00_JPRB, 0.25400E+00_JPRB, 0.25551E+00_JPRB, 0.25156E+00_JPRB ,&
&  0.24721E+00_JPRB, 0.24294E+00_JPRB, 0.23866E+00_JPRB, 0.23686E+00_JPRB, 0.23817E+00_JPRB, 0.23973E+00_JPRB, 0.24008E+00_JPRB ,&
&  0.24080E+00_JPRB, 0.24345E+00_JPRB, 0.24582E+00_JPRB, 0.24916E+00_JPRB, 0.26186E+00_JPRB, 0.26061E+00_JPRB, 0.25935E+00_JPRB ,&
&  0.25956E+00_JPRB, 0.25983E+00_JPRB, 0.26068E+00_JPRB, 0.26106E+00_JPRB, 0.26452E+00_JPRB, 0.26767E+00_JPRB, 0.26980E+00_JPRB ,&
&  0.25593E+00_JPRB, 0.24736E+00_JPRB, 0.23934E+00_JPRB, 0.23042E+00_JPRB, 0.22086E+00_JPRB, 0.21295E+00_JPRB, 0.20563E+00_JPRB ,&
&  0.19939E+00_JPRB, 0.19350E+00_JPRB, 0.18832E+00_JPRB, 0.18626E+00_JPRB, 0.18557E+00_JPRB, 0.18456E+00_JPRB, 0.18455E+00_JPRB ,&
&  0.18510E+00_JPRB, 0.18583E+00_JPRB, 0.18634E+00_JPRB, 0.18777E+00_JPRB, 0.18992E+00_JPRB, 0.19327E+00_JPRB, 0.19685E+00_JPRB ,&
&  0.20229E+00_JPRB, 0.20652E+00_JPRB, 0.21137E+00_JPRB, 0.21514E+00_JPRB, 0.21367E+00_JPRB, 0.20893E+00_JPRB, 0.20374E+00_JPRB ,&
&  0.19780E+00_JPRB, 0.19435E+00_JPRB, 0.19092E+00_JPRB, 0.18845E+00_JPRB, 0.18707E+00_JPRB, 0.18632E+00_JPRB, 0.18670E+00_JPRB ,&
&  0.18696E+00_JPRB, 0.18747E+00_JPRB, 0.18714E+00_JPRB, 0.18738E+00_JPRB, 0.18816E+00_JPRB, 0.18981E+00_JPRB, 0.19198E+00_JPRB ,&
&  0.19529E+00_JPRB, 0.19916E+00_JPRB, 0.20384E+00_JPRB, 0.20901E+00_JPRB, 0.21637E+00_JPRB, 0.22637E+00_JPRB, 0.24099E+00_JPRB ,&
&  0.25804E+00_JPRB, 0.29165E+00_JPRB, 0.30522E+00_JPRB, 0.29445E+00_JPRB, 0.29109E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.30448E+00_JPRB, 0.30095E+00_JPRB, 0.29910E+00_JPRB, 0.29942E+00_JPRB, 0.29964E+00_JPRB, 0.30247E+00_JPRB, 0.30210E+00_JPRB ,&
&  0.29793E+00_JPRB, 0.29334E+00_JPRB, 0.29181E+00_JPRB, 0.29145E+00_JPRB, 0.29024E+00_JPRB, 0.28905E+00_JPRB, 0.28957E+00_JPRB ,&
&  0.28802E+00_JPRB, 0.28953E+00_JPRB, 0.29206E+00_JPRB, 0.29333E+00_JPRB, 0.29407E+00_JPRB, 0.29429E+00_JPRB, 0.29438E+00_JPRB ,&
&  0.28839E+00_JPRB, 0.28689E+00_JPRB, 0.29237E+00_JPRB, 0.28985E+00_JPRB, 0.28616E+00_JPRB, 0.28301E+00_JPRB, 0.28198E+00_JPRB ,&
&  0.27417E+00_JPRB, 0.26319E+00_JPRB, 0.25022E+00_JPRB, 0.25463E+00_JPRB, 0.27155E+00_JPRB, 0.27004E+00_JPRB, 0.26786E+00_JPRB ,&
&  0.26756E+00_JPRB, 0.26694E+00_JPRB, 0.26775E+00_JPRB, 0.26821E+00_JPRB, 0.26889E+00_JPRB, 0.26880E+00_JPRB, 0.26964E+00_JPRB ,&
&  0.27083E+00_JPRB, 0.27597E+00_JPRB, 0.27983E+00_JPRB, 0.26198E+00_JPRB, 0.24592E+00_JPRB, 0.23362E+00_JPRB, 0.22202E+00_JPRB ,&
&  0.21172E+00_JPRB, 0.20847E+00_JPRB, 0.20516E+00_JPRB, 0.19994E+00_JPRB, 0.20054E+00_JPRB, 0.20103E+00_JPRB, 0.20037E+00_JPRB ,&
&  0.19872E+00_JPRB, 0.19770E+00_JPRB, 0.19631E+00_JPRB, 0.19403E+00_JPRB, 0.19311E+00_JPRB, 0.19420E+00_JPRB, 0.19729E+00_JPRB ,&
&  0.20232E+00_JPRB, 0.20834E+00_JPRB, 0.22189E+00_JPRB, 0.23373E+00_JPRB, 0.23096E+00_JPRB, 0.22275E+00_JPRB, 0.21519E+00_JPRB ,&
&  0.20573E+00_JPRB, 0.20093E+00_JPRB, 0.19626E+00_JPRB, 0.19253E+00_JPRB, 0.19087E+00_JPRB, 0.19121E+00_JPRB, 0.19217E+00_JPRB ,&
&  0.19346E+00_JPRB, 0.19530E+00_JPRB, 0.19604E+00_JPRB, 0.19560E+00_JPRB, 0.19481E+00_JPRB, 0.19516E+00_JPRB, 0.19652E+00_JPRB ,&
&  0.19918E+00_JPRB, 0.20386E+00_JPRB, 0.20965E+00_JPRB, 0.21672E+00_JPRB, 0.22279E+00_JPRB, 0.22841E+00_JPRB, 0.23696E+00_JPRB ,&
&  0.24466E+00_JPRB, 0.26179E+00_JPRB, 0.28731E+00_JPRB, 0.31486E+00_JPRB, 0.31136E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.30639E+00_JPRB, 0.30339E+00_JPRB, 0.30368E+00_JPRB, 0.30411E+00_JPRB, 0.30314E+00_JPRB, 0.30324E+00_JPRB, 0.30279E+00_JPRB ,&
&  0.30279E+00_JPRB, 0.30100E+00_JPRB, 0.29899E+00_JPRB, 0.29914E+00_JPRB, 0.30096E+00_JPRB, 0.30307E+00_JPRB, 0.30346E+00_JPRB ,&
&  0.30305E+00_JPRB, 0.30411E+00_JPRB, 0.30715E+00_JPRB, 0.31059E+00_JPRB, 0.31139E+00_JPRB, 0.31171E+00_JPRB, 0.31347E+00_JPRB ,&
&  0.31577E+00_JPRB, 0.31658E+00_JPRB, 0.31922E+00_JPRB, 0.31712E+00_JPRB, 0.31282E+00_JPRB, 0.31097E+00_JPRB, 0.30935E+00_JPRB ,&
&  0.30365E+00_JPRB, 0.28110E+00_JPRB, 0.26594E+00_JPRB, 0.26948E+00_JPRB, 0.28578E+00_JPRB, 0.28583E+00_JPRB, 0.28392E+00_JPRB ,&
&  0.28197E+00_JPRB, 0.27942E+00_JPRB, 0.27718E+00_JPRB, 0.27633E+00_JPRB, 0.27626E+00_JPRB, 0.27596E+00_JPRB, 0.27426E+00_JPRB ,&
&  0.27431E+00_JPRB, 0.27829E+00_JPRB, 0.28422E+00_JPRB, 0.28280E+00_JPRB, 0.27975E+00_JPRB, 0.27370E+00_JPRB, 0.26983E+00_JPRB ,&
&  0.26613E+00_JPRB, 0.26197E+00_JPRB, 0.25965E+00_JPRB, 0.25883E+00_JPRB, 0.25851E+00_JPRB, 0.25668E+00_JPRB, 0.25361E+00_JPRB ,&
&  0.24558E+00_JPRB, 0.23689E+00_JPRB, 0.22759E+00_JPRB, 0.21848E+00_JPRB, 0.21142E+00_JPRB, 0.20660E+00_JPRB, 0.20308E+00_JPRB ,&
&  0.20431E+00_JPRB, 0.21416E+00_JPRB, 0.23149E+00_JPRB, 0.25076E+00_JPRB, 0.27268E+00_JPRB, 0.24979E+00_JPRB, 0.23854E+00_JPRB ,&
&  0.22769E+00_JPRB, 0.21847E+00_JPRB, 0.21217E+00_JPRB, 0.20794E+00_JPRB, 0.20497E+00_JPRB, 0.20339E+00_JPRB, 0.20346E+00_JPRB ,&
&  0.20615E+00_JPRB, 0.20990E+00_JPRB, 0.21325E+00_JPRB, 0.21381E+00_JPRB, 0.21287E+00_JPRB, 0.21106E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.21090E+00_JPRB, 0.21562E+00_JPRB, 0.22384E+00_JPRB, 0.23564E+00_JPRB, 0.24664E+00_JPRB, 0.25815E+00_JPRB, 0.27197E+00_JPRB ,&
&  0.28710E+00_JPRB, 0.30224E+00_JPRB, 0.30597E+00_JPRB, 0.30897E+00_JPRB, 0.30900E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.31492E+00_JPRB, 0.31287E+00_JPRB, 0.31316E+00_JPRB, 0.31420E+00_JPRB, 0.31411E+00_JPRB, 0.31372E+00_JPRB, 0.31145E+00_JPRB ,&
&  0.30939E+00_JPRB, 0.30982E+00_JPRB, 0.31114E+00_JPRB, 0.31351E+00_JPRB, 0.31767E+00_JPRB, 0.32205E+00_JPRB, 0.32130E+00_JPRB ,&
&  0.31809E+00_JPRB, 0.31656E+00_JPRB, 0.31852E+00_JPRB, 0.32073E+00_JPRB, 0.32123E+00_JPRB, 0.32185E+00_JPRB, 0.32533E+00_JPRB ,&
&  0.32979E+00_JPRB, 0.33175E+00_JPRB, 0.33455E+00_JPRB, 0.33412E+00_JPRB, 0.33059E+00_JPRB, 0.33078E+00_JPRB, 0.32904E+00_JPRB ,&
&  0.32424E+00_JPRB, 0.31866E+00_JPRB, 0.30986E+00_JPRB, 0.31096E+00_JPRB, 0.30738E+00_JPRB, 0.30454E+00_JPRB, 0.30039E+00_JPRB ,&
&  0.29592E+00_JPRB, 0.29237E+00_JPRB, 0.28835E+00_JPRB, 0.28504E+00_JPRB, 0.28329E+00_JPRB, 0.28323E+00_JPRB, 0.28108E+00_JPRB ,&
&  0.27957E+00_JPRB, 0.28073E+00_JPRB, 0.28220E+00_JPRB, 0.28041E+00_JPRB, 0.28115E+00_JPRB, 0.28791E+00_JPRB, 0.29302E+00_JPRB ,&
&  0.29476E+00_JPRB, 0.29460E+00_JPRB, 0.29213E+00_JPRB, 0.29148E+00_JPRB, 0.29037E+00_JPRB, 0.28834E+00_JPRB, 0.28508E+00_JPRB ,&
&  0.28036E+00_JPRB, 0.27804E+00_JPRB, 0.27647E+00_JPRB, 0.27326E+00_JPRB, 0.26863E+00_JPRB, 0.25139E+00_JPRB, 0.24189E+00_JPRB ,&
&  0.24110E+00_JPRB, 0.24978E+00_JPRB, 0.26508E+00_JPRB, 0.27107E+00_JPRB, 0.27572E+00_JPRB, 0.28045E+00_JPRB, 0.28148E+00_JPRB ,&
&  0.28380E+00_JPRB, 0.26324E+00_JPRB, 0.25216E+00_JPRB, 0.24393E+00_JPRB, 0.23649E+00_JPRB, 0.23145E+00_JPRB, 0.23013E+00_JPRB ,&
&  0.23276E+00_JPRB, 0.23870E+00_JPRB, 0.24351E+00_JPRB, 0.24570E+00_JPRB, 0.24521E+00_JPRB, 0.24179E+00_JPRB, 0.23847E+00_JPRB ,&
&  0.23771E+00_JPRB, 0.24359E+00_JPRB, 0.25567E+00_JPRB, 0.26877E+00_JPRB, 0.27733E+00_JPRB, 0.28638E+00_JPRB, 0.29413E+00_JPRB ,&
&  0.29945E+00_JPRB, 0.30442E+00_JPRB, 0.31069E+00_JPRB, 0.31311E+00_JPRB, 0.31436E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.31539E+00_JPRB, 0.31541E+00_JPRB, 0.31861E+00_JPRB, 0.32210E+00_JPRB, 0.32305E+00_JPRB, 0.32307E+00_JPRB, 0.32081E+00_JPRB ,&
&  0.31931E+00_JPRB, 0.32050E+00_JPRB, 0.32228E+00_JPRB, 0.32351E+00_JPRB, 0.32541E+00_JPRB, 0.32787E+00_JPRB, 0.32976E+00_JPRB ,&
&  0.32973E+00_JPRB, 0.32801E+00_JPRB, 0.32501E+00_JPRB, 0.32585E+00_JPRB, 0.32588E+00_JPRB, 0.32775E+00_JPRB, 0.33490E+00_JPRB ,&
&  0.34001E+00_JPRB, 0.34355E+00_JPRB, 0.34655E+00_JPRB, 0.34696E+00_JPRB, 0.34652E+00_JPRB, 0.34803E+00_JPRB, 0.34485E+00_JPRB ,&
&  0.33886E+00_JPRB, 0.33325E+00_JPRB, 0.32854E+00_JPRB, 0.32157E+00_JPRB, 0.32401E+00_JPRB, 0.32144E+00_JPRB, 0.31835E+00_JPRB ,&
&  0.31437E+00_JPRB, 0.30951E+00_JPRB, 0.30396E+00_JPRB, 0.29880E+00_JPRB, 0.29506E+00_JPRB, 0.29166E+00_JPRB, 0.28691E+00_JPRB ,&
&  0.28533E+00_JPRB, 0.28704E+00_JPRB, 0.29277E+00_JPRB, 0.29594E+00_JPRB, 0.29418E+00_JPRB, 0.29549E+00_JPRB, 0.30219E+00_JPRB ,&
&  0.30878E+00_JPRB, 0.31173E+00_JPRB, 0.31153E+00_JPRB, 0.31204E+00_JPRB, 0.31261E+00_JPRB, 0.31388E+00_JPRB, 0.31446E+00_JPRB ,&
&  0.31345E+00_JPRB, 0.31189E+00_JPRB, 0.30942E+00_JPRB, 0.30344E+00_JPRB, 0.29630E+00_JPRB, 0.29049E+00_JPRB, 0.28812E+00_JPRB ,&
&  0.29063E+00_JPRB, 0.29024E+00_JPRB, 0.28992E+00_JPRB, 0.29121E+00_JPRB, 0.29164E+00_JPRB, 0.29083E+00_JPRB, 0.28981E+00_JPRB ,&
&  0.29095E+00_JPRB, 0.29042E+00_JPRB, 0.29219E+00_JPRB, 0.28945E+00_JPRB, 0.28495E+00_JPRB, 0.28065E+00_JPRB, 0.27660E+00_JPRB ,&
&  0.27415E+00_JPRB, 0.27315E+00_JPRB, 0.27504E+00_JPRB, 0.27561E+00_JPRB, 0.27732E+00_JPRB, 0.27935E+00_JPRB, 0.28106E+00_JPRB ,&
&  0.28200E+00_JPRB, 0.28312E+00_JPRB, 0.28532E+00_JPRB, 0.29001E+00_JPRB, 0.29501E+00_JPRB, 0.29938E+00_JPRB, 0.30398E+00_JPRB ,&
&  0.30835E+00_JPRB, 0.31153E+00_JPRB, 0.31736E+00_JPRB, 0.31875E+00_JPRB, 0.31823E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.32047E+00_JPRB, 0.32255E+00_JPRB, 0.32460E+00_JPRB, 0.32645E+00_JPRB, 0.32770E+00_JPRB, 0.32868E+00_JPRB, 0.32789E+00_JPRB ,&
&  0.32774E+00_JPRB, 0.32932E+00_JPRB, 0.33141E+00_JPRB, 0.33311E+00_JPRB, 0.33406E+00_JPRB, 0.33585E+00_JPRB, 0.33753E+00_JPRB ,&
&  0.33863E+00_JPRB, 0.33779E+00_JPRB, 0.33482E+00_JPRB, 0.33371E+00_JPRB, 0.33268E+00_JPRB, 0.33746E+00_JPRB, 0.34573E+00_JPRB ,&
&  0.34939E+00_JPRB, 0.35172E+00_JPRB, 0.35460E+00_JPRB, 0.35532E+00_JPRB, 0.35579E+00_JPRB, 0.35563E+00_JPRB, 0.35124E+00_JPRB ,&
&  0.34644E+00_JPRB, 0.34198E+00_JPRB, 0.33941E+00_JPRB, 0.33614E+00_JPRB, 0.33573E+00_JPRB, 0.33309E+00_JPRB, 0.33109E+00_JPRB ,&
&  0.32970E+00_JPRB, 0.32734E+00_JPRB, 0.32468E+00_JPRB, 0.32015E+00_JPRB, 0.31441E+00_JPRB, 0.30697E+00_JPRB, 0.30196E+00_JPRB ,&
&  0.30010E+00_JPRB, 0.30362E+00_JPRB, 0.31190E+00_JPRB, 0.31614E+00_JPRB, 0.31369E+00_JPRB, 0.31359E+00_JPRB, 0.31707E+00_JPRB ,&
&  0.32178E+00_JPRB, 0.32678E+00_JPRB, 0.32833E+00_JPRB, 0.32736E+00_JPRB, 0.32547E+00_JPRB, 0.32697E+00_JPRB, 0.33097E+00_JPRB ,&
&  0.33454E+00_JPRB, 0.33652E+00_JPRB, 0.33590E+00_JPRB, 0.33505E+00_JPRB, 0.33246E+00_JPRB, 0.32648E+00_JPRB, 0.31841E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.30662E+00_JPRB, 0.30325E+00_JPRB, 0.30004E+00_JPRB, 0.29893E+00_JPRB, 0.29944E+00_JPRB, 0.30167E+00_JPRB ,&
&  0.30259E+00_JPRB, 0.30447E+00_JPRB, 0.30460E+00_JPRB, 0.30045E+00_JPRB, 0.29713E+00_JPRB, 0.29833E+00_JPRB, 0.29979E+00_JPRB ,&
&  0.30225E+00_JPRB, 0.30459E+00_JPRB, 0.30699E+00_JPRB, 0.30806E+00_JPRB, 0.30962E+00_JPRB, 0.31211E+00_JPRB, 0.31350E+00_JPRB ,&
&  0.31418E+00_JPRB, 0.31473E+00_JPRB, 0.31649E+00_JPRB, 0.31932E+00_JPRB, 0.32177E+00_JPRB, 0.32361E+00_JPRB, 0.32280E+00_JPRB ,&
&  0.32163E+00_JPRB, 0.32050E+00_JPRB, 0.32117E+00_JPRB, 0.32166E+00_JPRB, 0.32035E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.34145E+00_JPRB, 0.34154E+00_JPRB, 0.34097E+00_JPRB, 0.33998E+00_JPRB, 0.33902E+00_JPRB, 0.33826E+00_JPRB, 0.33843E+00_JPRB ,&
&  0.33870E+00_JPRB, 0.33941E+00_JPRB, 0.34029E+00_JPRB, 0.34102E+00_JPRB, 0.34266E+00_JPRB, 0.34451E+00_JPRB, 0.34594E+00_JPRB ,&
&  0.34752E+00_JPRB, 0.34690E+00_JPRB, 0.34474E+00_JPRB, 0.34260E+00_JPRB, 0.34339E+00_JPRB, 0.34866E+00_JPRB, 0.35458E+00_JPRB ,&
&  0.35626E+00_JPRB, 0.35752E+00_JPRB, 0.35819E+00_JPRB, 0.35730E+00_JPRB, 0.35622E+00_JPRB, 0.35598E+00_JPRB, 0.35241E+00_JPRB ,&
&  0.34809E+00_JPRB, 0.34671E+00_JPRB, 0.34643E+00_JPRB, 0.34358E+00_JPRB, 0.34272E+00_JPRB, 0.34434E+00_JPRB, 0.34274E+00_JPRB ,&
&  0.34154E+00_JPRB, 0.33984E+00_JPRB, 0.33705E+00_JPRB, 0.33492E+00_JPRB, 0.33250E+00_JPRB, 0.32737E+00_JPRB, 0.32139E+00_JPRB ,&
&  0.31663E+00_JPRB, 0.31708E+00_JPRB, 0.32250E+00_JPRB, 0.32853E+00_JPRB, 0.32843E+00_JPRB, 0.32814E+00_JPRB, 0.32991E+00_JPRB ,&
&  0.33341E+00_JPRB, 0.33757E+00_JPRB, 0.34007E+00_JPRB, 0.33902E+00_JPRB, 0.33772E+00_JPRB, 0.33938E+00_JPRB, 0.34183E+00_JPRB ,&
&  0.34302E+00_JPRB, 0.34444E+00_JPRB, 0.34544E+00_JPRB, 0.34680E+00_JPRB, 0.34702E+00_JPRB, 0.34489E+00_JPRB, 0.34307E+00_JPRB ,&
&  0.33968E+00_JPRB, 0.33543E+00_JPRB, 0.33049E+00_JPRB, 0.32515E+00_JPRB, 0.32160E+00_JPRB, 0.31976E+00_JPRB, 0.32027E+00_JPRB ,&
&  0.31998E+00_JPRB, 0.32167E+00_JPRB, 0.32211E+00_JPRB, 0.31983E+00_JPRB, 0.31678E+00_JPRB, 0.31672E+00_JPRB, 0.32011E+00_JPRB ,&
&  0.32478E+00_JPRB, 0.32788E+00_JPRB, 0.33327E+00_JPRB, 0.33606E+00_JPRB, 0.33807E+00_JPRB, 0.34045E+00_JPRB, 0.34104E+00_JPRB ,&
&  0.33844E+00_JPRB, 0.33637E+00_JPRB, 0.33596E+00_JPRB, 0.33871E+00_JPRB, 0.33949E+00_JPRB, 0.33887E+00_JPRB, 0.33692E+00_JPRB ,&
&  0.33669E+00_JPRB, 0.33877E+00_JPRB, 0.33963E+00_JPRB, 0.33952E+00_JPRB, 0.34018E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.36703E+00_JPRB, 0.36538E+00_JPRB, 0.36384E+00_JPRB, 0.36158E+00_JPRB, 0.35917E+00_JPRB, 0.35704E+00_JPRB, 0.35513E+00_JPRB ,&
&  0.35391E+00_JPRB, 0.35377E+00_JPRB, 0.35439E+00_JPRB, 0.35461E+00_JPRB, 0.35438E+00_JPRB, 0.35513E+00_JPRB, 0.35541E+00_JPRB ,&
&  0.35689E+00_JPRB, 0.35719E+00_JPRB, 0.35535E+00_JPRB, 0.35528E+00_JPRB, 0.35796E+00_JPRB, 0.36284E+00_JPRB, 0.36499E+00_JPRB ,&
&  0.36308E+00_JPRB, 0.36321E+00_JPRB, 0.36336E+00_JPRB, 0.36169E+00_JPRB, 0.36064E+00_JPRB, 0.35989E+00_JPRB, 0.35606E+00_JPRB ,&
&  0.35153E+00_JPRB, 0.35151E+00_JPRB, 0.35346E+00_JPRB, 0.35588E+00_JPRB, 0.35485E+00_JPRB, 0.35219E+00_JPRB, 0.35227E+00_JPRB ,&
&  0.35035E+00_JPRB, 0.35006E+00_JPRB, 0.34986E+00_JPRB, 0.35073E+00_JPRB, 0.34958E+00_JPRB, 0.34788E+00_JPRB, 0.34446E+00_JPRB ,&
&  0.33656E+00_JPRB, 0.33462E+00_JPRB, 0.33752E+00_JPRB, 0.34251E+00_JPRB, 0.34260E+00_JPRB, 0.34371E+00_JPRB, 0.34681E+00_JPRB ,&
&  0.35048E+00_JPRB, 0.35297E+00_JPRB, 0.35432E+00_JPRB, 0.35517E+00_JPRB, 0.35649E+00_JPRB, 0.35781E+00_JPRB, 0.35849E+00_JPRB ,&
&  0.35952E+00_JPRB, 0.36138E+00_JPRB, 0.36351E+00_JPRB, 0.36553E+00_JPRB, 0.36602E+00_JPRB, 0.36569E+00_JPRB, 0.36460E+00_JPRB ,&
&  0.36248E+00_JPRB, 0.35857E+00_JPRB, 0.35439E+00_JPRB, 0.34895E+00_JPRB, 0.34384E+00_JPRB, 0.34132E+00_JPRB, 0.34035E+00_JPRB ,&
&  0.33748E+00_JPRB, 0.33501E+00_JPRB, 0.33323E+00_JPRB, 0.33305E+00_JPRB, 0.33343E+00_JPRB, 0.33476E+00_JPRB, 0.33709E+00_JPRB ,&
&  0.33900E+00_JPRB, 0.34179E+00_JPRB, 0.34798E+00_JPRB, 0.35235E+00_JPRB, 0.35459E+00_JPRB, 0.35763E+00_JPRB, 0.35907E+00_JPRB ,&
&  0.35744E+00_JPRB, 0.35590E+00_JPRB, 0.35476E+00_JPRB, 0.35670E+00_JPRB, 0.35920E+00_JPRB, 0.36092E+00_JPRB, 0.36247E+00_JPRB ,&
&  0.36568E+00_JPRB, 0.36884E+00_JPRB, 0.36943E+00_JPRB, 0.36912E+00_JPRB, 0.36869E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.38995E+00_JPRB, 0.38597E+00_JPRB, 0.38191E+00_JPRB, 0.37834E+00_JPRB, 0.37526E+00_JPRB, 0.37204E+00_JPRB, 0.36913E+00_JPRB ,&
&  0.36621E+00_JPRB, 0.36450E+00_JPRB, 0.36329E+00_JPRB, 0.36223E+00_JPRB, 0.35918E+00_JPRB, 0.35783E+00_JPRB, 0.35864E+00_JPRB ,&
&  0.36110E+00_JPRB, 0.36352E+00_JPRB, 0.36562E+00_JPRB, 0.36561E+00_JPRB, 0.36817E+00_JPRB, 0.37064E+00_JPRB, 0.37113E+00_JPRB ,&
&  0.37074E+00_JPRB, 0.37040E+00_JPRB, 0.36881E+00_JPRB, 0.36658E+00_JPRB, 0.36691E+00_JPRB, 0.36743E+00_JPRB, 0.36390E+00_JPRB ,&
&  0.35756E+00_JPRB, 0.35814E+00_JPRB, 0.35091E+00_JPRB, 0.36235E+00_JPRB, 0.36446E+00_JPRB, 0.35965E+00_JPRB, 0.35399E+00_JPRB ,&
&  0.35250E+00_JPRB, 0.35371E+00_JPRB, 0.35710E+00_JPRB, 0.35975E+00_JPRB, 0.36120E+00_JPRB, 0.36210E+00_JPRB, 0.36276E+00_JPRB ,&
&  0.35691E+00_JPRB, 0.35420E+00_JPRB, 0.35509E+00_JPRB, 0.35665E+00_JPRB, 0.35705E+00_JPRB, 0.35780E+00_JPRB, 0.35953E+00_JPRB ,&
&  0.36066E+00_JPRB, 0.36127E+00_JPRB, 0.36200E+00_JPRB, 0.36419E+00_JPRB, 0.36570E+00_JPRB, 0.36628E+00_JPRB, 0.36778E+00_JPRB ,&
&  0.37029E+00_JPRB, 0.37341E+00_JPRB, 0.37583E+00_JPRB, 0.37714E+00_JPRB, 0.37964E+00_JPRB, 0.38235E+00_JPRB, 0.38392E+00_JPRB ,&
&  0.38336E+00_JPRB, 0.38124E+00_JPRB, 0.37851E+00_JPRB, 0.37375E+00_JPRB, 0.36745E+00_JPRB, 0.36267E+00_JPRB, 0.35919E+00_JPRB ,&
&  0.35375E+00_JPRB, 0.35149E+00_JPRB, 0.35055E+00_JPRB, 0.34880E+00_JPRB, 0.34734E+00_JPRB, 0.34698E+00_JPRB, 0.34748E+00_JPRB ,&
&  0.34946E+00_JPRB, 0.35305E+00_JPRB, 0.35554E+00_JPRB, 0.35865E+00_JPRB, 0.36271E+00_JPRB, 0.36674E+00_JPRB, 0.36718E+00_JPRB ,&
&  0.36603E+00_JPRB, 0.36558E+00_JPRB, 0.36744E+00_JPRB, 0.37320E+00_JPRB, 0.38033E+00_JPRB, 0.38701E+00_JPRB, 0.39333E+00_JPRB ,&
&  0.39798E+00_JPRB, 0.39990E+00_JPRB, 0.39878E+00_JPRB, 0.39720E+00_JPRB, 0.39396E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.42062E+00_JPRB, 0.41320E+00_JPRB, 0.40560E+00_JPRB, 0.39987E+00_JPRB, 0.39574E+00_JPRB, 0.39147E+00_JPRB, 0.38634E+00_JPRB ,&
&  0.38248E+00_JPRB, 0.37741E+00_JPRB, 0.37359E+00_JPRB, 0.37000E+00_JPRB, 0.36695E+00_JPRB, 0.36597E+00_JPRB, 0.36686E+00_JPRB ,&
&  0.36941E+00_JPRB, 0.37475E+00_JPRB, 0.37808E+00_JPRB, 0.37724E+00_JPRB, 0.37932E+00_JPRB, 0.38174E+00_JPRB, 0.38251E+00_JPRB ,&
&  0.38068E+00_JPRB, 0.37802E+00_JPRB, 0.37506E+00_JPRB, 0.37345E+00_JPRB, 0.37368E+00_JPRB, 0.37484E+00_JPRB, 0.37194E+00_JPRB ,&
&  0.36734E+00_JPRB, 0.35896E+00_JPRB, 0.36988E+00_JPRB, 0.37463E+00_JPRB, 0.37473E+00_JPRB, 0.37170E+00_JPRB, 0.36791E+00_JPRB ,&
&  0.36250E+00_JPRB, 0.35982E+00_JPRB, 0.36182E+00_JPRB, 0.36565E+00_JPRB, 0.36951E+00_JPRB, 0.37207E+00_JPRB, 0.37459E+00_JPRB ,&
&  0.37262E+00_JPRB, 0.37174E+00_JPRB, 0.37038E+00_JPRB, 0.36942E+00_JPRB, 0.36977E+00_JPRB, 0.37027E+00_JPRB, 0.37034E+00_JPRB ,&
&  0.36889E+00_JPRB, 0.36913E+00_JPRB, 0.36969E+00_JPRB, 0.36865E+00_JPRB, 0.36983E+00_JPRB, 0.37172E+00_JPRB, 0.37437E+00_JPRB ,&
&  0.37781E+00_JPRB, 0.38241E+00_JPRB, 0.38731E+00_JPRB, 0.39345E+00_JPRB, 0.40092E+00_JPRB, 0.40719E+00_JPRB, 0.41136E+00_JPRB ,&
&  0.41263E+00_JPRB, 0.41073E+00_JPRB, 0.40634E+00_JPRB, 0.39921E+00_JPRB, 0.39287E+00_JPRB, 0.38785E+00_JPRB, 0.38134E+00_JPRB ,&
&  0.37299E+00_JPRB, 0.36796E+00_JPRB, 0.36511E+00_JPRB, 0.36388E+00_JPRB, 0.36232E+00_JPRB, 0.36175E+00_JPRB, 0.36293E+00_JPRB ,&
&  0.36573E+00_JPRB, 0.36856E+00_JPRB, 0.37026E+00_JPRB, 0.37138E+00_JPRB, 0.37304E+00_JPRB, 0.37256E+00_JPRB, 0.37248E+00_JPRB ,&
&  0.37488E+00_JPRB, 0.38025E+00_JPRB, 0.38988E+00_JPRB, 0.40639E+00_JPRB, 0.42024E+00_JPRB, 0.43009E+00_JPRB, 0.43612E+00_JPRB ,&
&  0.43855E+00_JPRB, 0.43865E+00_JPRB, 0.43583E+00_JPRB, 0.43223E+00_JPRB, 0.42752E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.44144E+00_JPRB, 0.43142E+00_JPRB, 0.42102E+00_JPRB, 0.41423E+00_JPRB, 0.40950E+00_JPRB, 0.40477E+00_JPRB, 0.39916E+00_JPRB ,&
&  0.39282E+00_JPRB, 0.38723E+00_JPRB, 0.38199E+00_JPRB, 0.37669E+00_JPRB, 0.37417E+00_JPRB, 0.37308E+00_JPRB, 0.37392E+00_JPRB ,&
&  0.37672E+00_JPRB, 0.38146E+00_JPRB, 0.38400E+00_JPRB, 0.38432E+00_JPRB, 0.38525E+00_JPRB, 0.38524E+00_JPRB, 0.38385E+00_JPRB ,&
&  0.38019E+00_JPRB, 0.37568E+00_JPRB, 0.37341E+00_JPRB, 0.37456E+00_JPRB, 0.37505E+00_JPRB, 0.37579E+00_JPRB, 0.37549E+00_JPRB ,&
&  0.37885E+00_JPRB, 0.34752E+00_JPRB, 0.37255E+00_JPRB, 0.38816E+00_JPRB, 0.38470E+00_JPRB, 0.38041E+00_JPRB, 0.37557E+00_JPRB ,&
&  0.37018E+00_JPRB, 0.36896E+00_JPRB, 0.37038E+00_JPRB, 0.37451E+00_JPRB, 0.37963E+00_JPRB, 0.38243E+00_JPRB, 0.38276E+00_JPRB ,&
&  0.37975E+00_JPRB, 0.37916E+00_JPRB, 0.37762E+00_JPRB, 0.37653E+00_JPRB, 0.37621E+00_JPRB, 0.37613E+00_JPRB, 0.37538E+00_JPRB ,&
&  0.37347E+00_JPRB, 0.37013E+00_JPRB, 0.36675E+00_JPRB, 0.36614E+00_JPRB, 0.36728E+00_JPRB, 0.37055E+00_JPRB, 0.37655E+00_JPRB ,&
&  0.38477E+00_JPRB, 0.39402E+00_JPRB, 0.40361E+00_JPRB, 0.41342E+00_JPRB, 0.42422E+00_JPRB, 0.43350E+00_JPRB, 0.44020E+00_JPRB ,&
&  0.43995E+00_JPRB, 0.43576E+00_JPRB, 0.42756E+00_JPRB, 0.41743E+00_JPRB, 0.40954E+00_JPRB, 0.40159E+00_JPRB, 0.39378E+00_JPRB ,&
&  0.38785E+00_JPRB, 0.38255E+00_JPRB, 0.37931E+00_JPRB, 0.37880E+00_JPRB, 0.37905E+00_JPRB, 0.37842E+00_JPRB, 0.37843E+00_JPRB ,&
&  0.37873E+00_JPRB, 0.37910E+00_JPRB, 0.37954E+00_JPRB, 0.37975E+00_JPRB, 0.37885E+00_JPRB, 0.37792E+00_JPRB, 0.37923E+00_JPRB ,&
&  0.38349E+00_JPRB, 0.39193E+00_JPRB, 0.40632E+00_JPRB, 0.42725E+00_JPRB, 0.44765E+00_JPRB, 0.46244E+00_JPRB, 0.46919E+00_JPRB ,&
&  0.46829E+00_JPRB, 0.46421E+00_JPRB, 0.45716E+00_JPRB, 0.45260E+00_JPRB, 0.44918E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.45710E+00_JPRB, 0.45179E+00_JPRB, 0.44613E+00_JPRB, 0.44012E+00_JPRB, 0.43338E+00_JPRB, 0.42548E+00_JPRB, 0.41701E+00_JPRB ,&
&  0.41017E+00_JPRB, 0.40445E+00_JPRB, 0.39895E+00_JPRB, 0.39469E+00_JPRB, 0.39086E+00_JPRB, 0.38974E+00_JPRB, 0.38997E+00_JPRB ,&
&  0.39084E+00_JPRB, 0.39064E+00_JPRB, 0.39003E+00_JPRB, 0.39076E+00_JPRB, 0.39078E+00_JPRB, 0.38882E+00_JPRB, 0.38593E+00_JPRB ,&
&  0.38096E+00_JPRB, 0.37675E+00_JPRB, 0.37551E+00_JPRB, 0.37831E+00_JPRB, 0.37776E+00_JPRB, 0.37967E+00_JPRB, 0.38661E+00_JPRB ,&
&  0.37034E+00_JPRB, 0.34500E+00_JPRB, 0.37479E+00_JPRB, 0.40833E+00_JPRB, 0.40208E+00_JPRB, 0.39398E+00_JPRB, 0.39535E+00_JPRB ,&
&  0.39620E+00_JPRB, 0.39890E+00_JPRB, 0.39908E+00_JPRB, 0.39845E+00_JPRB, 0.39869E+00_JPRB, 0.39663E+00_JPRB, 0.39238E+00_JPRB ,&
&  0.38876E+00_JPRB, 0.38865E+00_JPRB, 0.38881E+00_JPRB, 0.38843E+00_JPRB, 0.38689E+00_JPRB, 0.38487E+00_JPRB, 0.38103E+00_JPRB ,&
&  0.37769E+00_JPRB, 0.37583E+00_JPRB, 0.37596E+00_JPRB, 0.37937E+00_JPRB, 0.38507E+00_JPRB, 0.39404E+00_JPRB, 0.40736E+00_JPRB ,&
&  0.42267E+00_JPRB, 0.43681E+00_JPRB, 0.44912E+00_JPRB, 0.45980E+00_JPRB, 0.46997E+00_JPRB, 0.47763E+00_JPRB, 0.48064E+00_JPRB ,&
&  0.47387E+00_JPRB, 0.46098E+00_JPRB, 0.44604E+00_JPRB, 0.43497E+00_JPRB, 0.42727E+00_JPRB, 0.42232E+00_JPRB, 0.41943E+00_JPRB ,&
&  0.41876E+00_JPRB, 0.41798E+00_JPRB, 0.41666E+00_JPRB, 0.41616E+00_JPRB, 0.41600E+00_JPRB, 0.41462E+00_JPRB, 0.41330E+00_JPRB ,&
&  0.41165E+00_JPRB, 0.40879E+00_JPRB, 0.40482E+00_JPRB, 0.40098E+00_JPRB, 0.39805E+00_JPRB, 0.39793E+00_JPRB, 0.40019E+00_JPRB ,&
&  0.40657E+00_JPRB, 0.41962E+00_JPRB, 0.43582E+00_JPRB, 0.46763E+00_JPRB, 0.48933E+00_JPRB, 0.50375E+00_JPRB, 0.50887E+00_JPRB ,&
&  0.50270E+00_JPRB, 0.48618E+00_JPRB, 0.47256E+00_JPRB, 0.46520E+00_JPRB, 0.46426E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.46587E+00_JPRB, 0.46486E+00_JPRB, 0.46013E+00_JPRB, 0.45412E+00_JPRB, 0.44845E+00_JPRB, 0.44097E+00_JPRB, 0.43160E+00_JPRB ,&
&  0.42302E+00_JPRB, 0.41498E+00_JPRB, 0.41020E+00_JPRB, 0.40597E+00_JPRB, 0.39952E+00_JPRB, 0.39620E+00_JPRB, 0.39691E+00_JPRB ,&
&  0.39629E+00_JPRB, 0.39205E+00_JPRB, 0.38692E+00_JPRB, 0.38655E+00_JPRB, 0.38520E+00_JPRB, 0.38312E+00_JPRB, 0.37994E+00_JPRB ,&
&  0.37650E+00_JPRB, 0.37573E+00_JPRB, 0.37684E+00_JPRB, 0.37853E+00_JPRB, 0.37590E+00_JPRB, 0.37736E+00_JPRB, 0.38656E+00_JPRB ,&
&  0.42563E+00_JPRB, 0.36466E+00_JPRB, 0.40351E+00_JPRB, 0.44991E+00_JPRB, 0.44230E+00_JPRB, 0.43819E+00_JPRB, 0.43595E+00_JPRB ,&
&  0.43325E+00_JPRB, 0.43243E+00_JPRB, 0.42922E+00_JPRB, 0.42293E+00_JPRB, 0.41637E+00_JPRB, 0.40892E+00_JPRB, 0.40377E+00_JPRB ,&
&  0.40041E+00_JPRB, 0.40135E+00_JPRB, 0.40078E+00_JPRB, 0.39850E+00_JPRB, 0.39410E+00_JPRB, 0.39031E+00_JPRB, 0.38783E+00_JPRB ,&
&  0.38738E+00_JPRB, 0.38922E+00_JPRB, 0.39170E+00_JPRB, 0.39509E+00_JPRB, 0.40096E+00_JPRB, 0.41326E+00_JPRB, 0.43225E+00_JPRB ,&
&  0.45495E+00_JPRB, 0.47306E+00_JPRB, 0.48757E+00_JPRB, 0.50039E+00_JPRB, 0.50973E+00_JPRB, 0.51319E+00_JPRB, 0.50855E+00_JPRB ,&
&  0.49797E+00_JPRB, 0.48029E+00_JPRB, 0.46794E+00_JPRB, 0.45797E+00_JPRB, 0.44800E+00_JPRB, 0.44084E+00_JPRB, 0.43642E+00_JPRB ,&
&  0.43409E+00_JPRB, 0.43247E+00_JPRB, 0.43068E+00_JPRB, 0.42786E+00_JPRB, 0.42708E+00_JPRB, 0.42693E+00_JPRB, 0.42849E+00_JPRB ,&
&  0.42839E+00_JPRB, 0.42720E+00_JPRB, 0.42260E+00_JPRB, 0.41614E+00_JPRB, 0.41386E+00_JPRB, 0.41420E+00_JPRB, 0.41680E+00_JPRB ,&
&  0.42375E+00_JPRB, 0.43697E+00_JPRB, 0.45309E+00_JPRB, 0.48946E+00_JPRB, 0.50825E+00_JPRB, 0.51955E+00_JPRB, 0.52070E+00_JPRB ,&
&  0.50949E+00_JPRB, 0.49108E+00_JPRB, 0.39872E+00_JPRB, 0.45369E+00_JPRB, 0.46948E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.50646E+00_JPRB, 0.49948E+00_JPRB, 0.48835E+00_JPRB, 0.47166E+00_JPRB, 0.45943E+00_JPRB, 0.45012E+00_JPRB, 0.44054E+00_JPRB ,&
&  0.43192E+00_JPRB, 0.42499E+00_JPRB, 0.41934E+00_JPRB, 0.41262E+00_JPRB, 0.40509E+00_JPRB, 0.40398E+00_JPRB, 0.40443E+00_JPRB ,&
&  0.40207E+00_JPRB, 0.39575E+00_JPRB, 0.38970E+00_JPRB, 0.38874E+00_JPRB, 0.38850E+00_JPRB, 0.38850E+00_JPRB, 0.38591E+00_JPRB ,&
&  0.38137E+00_JPRB, 0.38105E+00_JPRB, 0.38499E+00_JPRB, 0.38652E+00_JPRB, 0.38314E+00_JPRB, 0.38513E+00_JPRB, 0.39457E+00_JPRB ,&
&  0.44048E+00_JPRB, 0.39197E+00_JPRB, 0.43166E+00_JPRB, 0.51925E+00_JPRB, 0.52465E+00_JPRB, 0.53345E+00_JPRB, 0.52081E+00_JPRB ,&
&  0.50041E+00_JPRB, 0.48727E+00_JPRB, 0.47280E+00_JPRB, 0.45306E+00_JPRB, 0.43499E+00_JPRB, 0.42466E+00_JPRB, 0.42196E+00_JPRB ,&
&  0.42336E+00_JPRB, 0.42334E+00_JPRB, 0.42251E+00_JPRB, 0.41746E+00_JPRB, 0.41423E+00_JPRB, 0.41232E+00_JPRB, 0.41187E+00_JPRB ,&
&  0.41189E+00_JPRB, 0.41466E+00_JPRB, 0.41805E+00_JPRB, 0.41964E+00_JPRB, 0.42608E+00_JPRB, 0.44708E+00_JPRB, 0.47692E+00_JPRB ,&
&  0.50423E+00_JPRB, 0.52577E+00_JPRB, 0.53969E+00_JPRB, 0.54949E+00_JPRB, 0.55168E+00_JPRB, 0.55204E+00_JPRB, 0.54653E+00_JPRB ,&
&  0.53074E+00_JPRB, 0.51261E+00_JPRB, 0.50010E+00_JPRB, 0.48632E+00_JPRB, 0.47330E+00_JPRB, 0.46344E+00_JPRB, 0.45541E+00_JPRB ,&
&  0.44904E+00_JPRB, 0.44420E+00_JPRB, 0.44044E+00_JPRB, 0.43469E+00_JPRB, 0.43110E+00_JPRB, 0.43194E+00_JPRB, 0.43351E+00_JPRB ,&
&  0.43488E+00_JPRB, 0.43769E+00_JPRB, 0.43999E+00_JPRB, 0.44163E+00_JPRB, 0.44312E+00_JPRB, 0.44600E+00_JPRB, 0.44980E+00_JPRB ,&
&  0.46130E+00_JPRB, 0.48035E+00_JPRB, 0.50090E+00_JPRB, 0.52392E+00_JPRB, 0.54779E+00_JPRB, 0.54777E+00_JPRB, 0.53408E+00_JPRB ,&
&  0.51466E+00_JPRB, 0.49357E+00_JPRB, 0.48318E+00_JPRB, 0.43540E+00_JPRB, 0.50907E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.56377E+00_JPRB, 0.54278E+00_JPRB, 0.51064E+00_JPRB, 0.47996E+00_JPRB, 0.45860E+00_JPRB, 0.44517E+00_JPRB, 0.43751E+00_JPRB ,&
&  0.43194E+00_JPRB, 0.42716E+00_JPRB, 0.42169E+00_JPRB, 0.41342E+00_JPRB, 0.40853E+00_JPRB, 0.40687E+00_JPRB, 0.40599E+00_JPRB ,&
&  0.40320E+00_JPRB, 0.39515E+00_JPRB, 0.39283E+00_JPRB, 0.39076E+00_JPRB, 0.38994E+00_JPRB, 0.38897E+00_JPRB, 0.38497E+00_JPRB ,&
&  0.37974E+00_JPRB, 0.37746E+00_JPRB, 0.38165E+00_JPRB, 0.38430E+00_JPRB, 0.38681E+00_JPRB, 0.39227E+00_JPRB, 0.40593E+00_JPRB ,&
&  0.45553E+00_JPRB, 0.45719E+00_JPRB, 0.53102E+00_JPRB, 0.60826E+00_JPRB, 0.61762E+00_JPRB, 0.60510E+00_JPRB, 0.58867E+00_JPRB ,&
&  0.56683E+00_JPRB, 0.54721E+00_JPRB, 0.51671E+00_JPRB, 0.48216E+00_JPRB, 0.45508E+00_JPRB, 0.44697E+00_JPRB, 0.43991E+00_JPRB ,&
&  0.43445E+00_JPRB, 0.43227E+00_JPRB, 0.43201E+00_JPRB, 0.43219E+00_JPRB, 0.43267E+00_JPRB, 0.43200E+00_JPRB, 0.43134E+00_JPRB ,&
&  0.43201E+00_JPRB, 0.43438E+00_JPRB, 0.43606E+00_JPRB, 0.44180E+00_JPRB, 0.45965E+00_JPRB, 0.48934E+00_JPRB, 0.52051E+00_JPRB ,&
&  0.54253E+00_JPRB, 0.55812E+00_JPRB, 0.57803E+00_JPRB, 0.59174E+00_JPRB, 0.59617E+00_JPRB, 0.59216E+00_JPRB, 0.58000E+00_JPRB ,&
&  0.56562E+00_JPRB, 0.55608E+00_JPRB, 0.53449E+00_JPRB, 0.51131E+00_JPRB, 0.49702E+00_JPRB, 0.48845E+00_JPRB, 0.48255E+00_JPRB ,&
&  0.47816E+00_JPRB, 0.47171E+00_JPRB, 0.46360E+00_JPRB, 0.45656E+00_JPRB, 0.45331E+00_JPRB, 0.44915E+00_JPRB, 0.44424E+00_JPRB ,&
&  0.44402E+00_JPRB, 0.44897E+00_JPRB, 0.45664E+00_JPRB, 0.46162E+00_JPRB, 0.46645E+00_JPRB, 0.47393E+00_JPRB, 0.47965E+00_JPRB ,&
&  0.48920E+00_JPRB, 0.50837E+00_JPRB, 0.52484E+00_JPRB, 0.56007E+00_JPRB, 0.55685E+00_JPRB, 0.54960E+00_JPRB, 0.52881E+00_JPRB ,&
&  0.50677E+00_JPRB, 0.48918E+00_JPRB, 0.47505E+00_JPRB, 0.49692E+00_JPRB, 0.55739E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.60703E+00_JPRB, 0.55600E+00_JPRB, 0.51392E+00_JPRB, 0.47952E+00_JPRB, 0.45827E+00_JPRB, 0.44592E+00_JPRB, 0.43733E+00_JPRB ,&
&  0.43354E+00_JPRB, 0.43178E+00_JPRB, 0.42616E+00_JPRB, 0.42197E+00_JPRB, 0.41920E+00_JPRB, 0.41849E+00_JPRB, 0.41833E+00_JPRB ,&
&  0.41114E+00_JPRB, 0.41006E+00_JPRB, 0.41040E+00_JPRB, 0.40660E+00_JPRB, 0.40136E+00_JPRB, 0.39600E+00_JPRB, 0.38901E+00_JPRB ,&
&  0.38459E+00_JPRB, 0.38349E+00_JPRB, 0.38613E+00_JPRB, 0.39042E+00_JPRB, 0.39452E+00_JPRB, 0.40570E+00_JPRB, 0.42711E+00_JPRB ,&
&  0.48618E+00_JPRB, 0.52357E+00_JPRB, 0.64239E+00_JPRB, 0.73546E+00_JPRB, 0.69716E+00_JPRB, 0.66532E+00_JPRB, 0.63627E+00_JPRB ,&
&  0.62923E+00_JPRB, 0.62420E+00_JPRB, 0.59230E+00_JPRB, 0.55011E+00_JPRB, 0.51244E+00_JPRB, 0.49074E+00_JPRB, 0.47727E+00_JPRB ,&
&  0.46573E+00_JPRB, 0.45842E+00_JPRB, 0.45262E+00_JPRB, 0.45543E+00_JPRB, 0.45793E+00_JPRB, 0.45655E+00_JPRB, 0.45472E+00_JPRB ,&
&  0.45310E+00_JPRB, 0.45356E+00_JPRB, 0.45740E+00_JPRB, 0.47672E+00_JPRB, 0.51557E+00_JPRB, 0.55561E+00_JPRB, 0.59225E+00_JPRB ,&
&  0.61265E+00_JPRB, 0.63241E+00_JPRB, 0.65202E+00_JPRB, 0.66315E+00_JPRB, 0.66147E+00_JPRB, 0.65386E+00_JPRB, 0.64550E+00_JPRB ,&
&  0.63393E+00_JPRB, 0.60598E+00_JPRB, 0.56815E+00_JPRB, 0.54042E+00_JPRB, 0.52627E+00_JPRB, 0.51861E+00_JPRB, 0.51529E+00_JPRB ,&
&  0.51278E+00_JPRB, 0.51010E+00_JPRB, 0.50440E+00_JPRB, 0.49546E+00_JPRB, 0.48316E+00_JPRB, 0.47476E+00_JPRB, 0.46747E+00_JPRB ,&
&  0.46321E+00_JPRB, 0.46352E+00_JPRB, 0.46842E+00_JPRB, 0.47837E+00_JPRB, 0.48705E+00_JPRB, 0.49539E+00_JPRB, 0.50247E+00_JPRB ,&
&  0.51448E+00_JPRB, 0.54166E+00_JPRB, 0.53119E+00_JPRB, 0.54834E+00_JPRB, 0.59283E+00_JPRB, 0.56719E+00_JPRB, 0.53576E+00_JPRB ,&
&  0.50815E+00_JPRB, 0.48979E+00_JPRB, 0.48755E+00_JPRB, 0.51582E+00_JPRB, 0.63669E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.51738E+00_JPRB, 0.49978E+00_JPRB, 0.48299E+00_JPRB, 0.46514E+00_JPRB, 0.44889E+00_JPRB, 0.44093E+00_JPRB, 0.43899E+00_JPRB ,&
&  0.44111E+00_JPRB, 0.44252E+00_JPRB, 0.44063E+00_JPRB, 0.43481E+00_JPRB, 0.43124E+00_JPRB, 0.42811E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.42023E+00_JPRB, 0.42098E+00_JPRB, 0.41769E+00_JPRB, 0.41229E+00_JPRB, 0.40633E+00_JPRB, 0.39911E+00_JPRB, 0.39216E+00_JPRB ,&
&  0.38819E+00_JPRB, 0.38746E+00_JPRB, 0.38918E+00_JPRB, 0.39391E+00_JPRB, 0.40578E+00_JPRB, 0.41738E+00_JPRB, 0.43077E+00_JPRB ,&
&  0.49185E+00_JPRB, 0.58670E+00_JPRB, 0.78099E+00_JPRB, 0.96502E+00_JPRB, 0.95852E+00_JPRB, 0.83180E+00_JPRB, 0.74861E+00_JPRB ,&
&  0.71774E+00_JPRB, 0.69346E+00_JPRB, 0.66103E+00_JPRB, 0.62440E+00_JPRB, 0.58390E+00_JPRB, 0.56340E+00_JPRB, 0.54341E+00_JPRB ,&
&  0.52401E+00_JPRB, 0.50889E+00_JPRB, 0.49451E+00_JPRB, 0.49065E+00_JPRB, 0.48765E+00_JPRB, 0.48198E+00_JPRB, 0.47630E+00_JPRB ,&
&  0.47503E+00_JPRB, 0.47892E+00_JPRB, 0.49108E+00_JPRB, 0.52552E+00_JPRB, 0.58613E+00_JPRB, 0.64198E+00_JPRB, 0.70559E+00_JPRB ,&
&  0.73241E+00_JPRB, 0.74581E+00_JPRB, 0.75630E+00_JPRB, 0.75038E+00_JPRB, 0.72147E+00_JPRB, 0.70130E+00_JPRB, 0.69921E+00_JPRB ,&
&  0.68255E+00_JPRB, 0.63472E+00_JPRB, 0.58502E+00_JPRB, 0.55348E+00_JPRB, 0.53670E+00_JPRB, 0.53077E+00_JPRB, 0.52737E+00_JPRB ,&
&  0.52816E+00_JPRB, 0.52782E+00_JPRB, 0.51928E+00_JPRB, 0.50521E+00_JPRB, 0.49139E+00_JPRB, 0.48245E+00_JPRB, 0.47773E+00_JPRB ,&
&  0.47280E+00_JPRB, 0.47193E+00_JPRB, 0.48001E+00_JPRB, 0.48648E+00_JPRB, 0.49058E+00_JPRB, 0.49729E+00_JPRB, 0.50884E+00_JPRB ,&
&  0.52581E+00_JPRB, 0.54084E+00_JPRB, 0.53602E+00_JPRB, 0.54715E+00_JPRB, 0.55300E+00_JPRB, 0.56314E+00_JPRB, 0.53594E+00_JPRB ,&
&  0.50646E+00_JPRB, 0.49716E+00_JPRB, 0.50842E+00_JPRB, 0.52322E+00_JPRB, 0.53372E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.49653E+00_JPRB, 0.48320E+00_JPRB, 0.47125E+00_JPRB, 0.45162E+00_JPRB, 0.43945E+00_JPRB, 0.43297E+00_JPRB, 0.43515E+00_JPRB ,&
&  0.44089E+00_JPRB, 0.44797E+00_JPRB, 0.44558E+00_JPRB, 0.44274E+00_JPRB, 0.43763E+00_JPRB, 0.43465E+00_JPRB, 0.42803E+00_JPRB ,&
&  0.42463E+00_JPRB, 0.42103E+00_JPRB, 0.41539E+00_JPRB, 0.40963E+00_JPRB, 0.40072E+00_JPRB, 0.39701E+00_JPRB, 0.39206E+00_JPRB ,&
&  0.38727E+00_JPRB, 0.38563E+00_JPRB, 0.38812E+00_JPRB, 0.39712E+00_JPRB, 0.40822E+00_JPRB, 0.42102E+00_JPRB, 0.43699E+00_JPRB ,&
&  0.52155E+00_JPRB, 0.58303E+00_JPRB, 0.10141E+01_JPRB, 0.11345E+01_JPRB, 0.10670E+01_JPRB, 0.93637E+00_JPRB, 0.91029E+00_JPRB ,&
&  0.88697E+00_JPRB, 0.82384E+00_JPRB, 0.76677E+00_JPRB, 0.71975E+00_JPRB, 0.67598E+00_JPRB, 0.64670E+00_JPRB, 0.61323E+00_JPRB ,&
&  0.58605E+00_JPRB, 0.55877E+00_JPRB, 0.52702E+00_JPRB, 0.50698E+00_JPRB, 0.49322E+00_JPRB, 0.48782E+00_JPRB, 0.49136E+00_JPRB ,&
&  0.49846E+00_JPRB, 0.50875E+00_JPRB, 0.53914E+00_JPRB, 0.58918E+00_JPRB, 0.63702E+00_JPRB, 0.63497E+00_JPRB, 0.71757E+00_JPRB ,&
&  0.87904E+00_JPRB, 0.87917E+00_JPRB, 0.83442E+00_JPRB, 0.79479E+00_JPRB, 0.73298E+00_JPRB, 0.70838E+00_JPRB, 0.70136E+00_JPRB ,&
&  0.67896E+00_JPRB, 0.64212E+00_JPRB, 0.60261E+00_JPRB, 0.57203E+00_JPRB, 0.55062E+00_JPRB, 0.53797E+00_JPRB, 0.53727E+00_JPRB ,&
&  0.53457E+00_JPRB, 0.52320E+00_JPRB, 0.50428E+00_JPRB, 0.48857E+00_JPRB, 0.48043E+00_JPRB, 0.47875E+00_JPRB, 0.47945E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.48208E+00_JPRB, 0.48878E+00_JPRB, 0.47742E+00_JPRB, 0.48540E+00_JPRB, 0.50308E+00_JPRB, 0.51005E+00_JPRB ,&
&  0.50213E+00_JPRB, 0.50644E+00_JPRB, 0.52028E+00_JPRB, 0.52926E+00_JPRB, 0.54129E+00_JPRB, 0.57007E+00_JPRB, 0.54288E+00_JPRB ,&
&  0.51112E+00_JPRB, 0.50568E+00_JPRB, 0.51930E+00_JPRB, 0.52427E+00_JPRB, 0.52179E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.47948E+00_JPRB, 0.46549E+00_JPRB, 0.45625E+00_JPRB, 0.44318E+00_JPRB, 0.43129E+00_JPRB, 0.42466E+00_JPRB, 0.42728E+00_JPRB ,&
&  0.43999E+00_JPRB, 0.45193E+00_JPRB, 0.44713E+00_JPRB, 0.44539E+00_JPRB, 0.44487E+00_JPRB, 0.43935E+00_JPRB, 0.43033E+00_JPRB ,&
&  0.42127E+00_JPRB, 0.41262E+00_JPRB, 0.40272E+00_JPRB, 0.39271E+00_JPRB, 0.38575E+00_JPRB, 0.38646E+00_JPRB, 0.38524E+00_JPRB ,&
&  0.38353E+00_JPRB, 0.38694E+00_JPRB, 0.39169E+00_JPRB, 0.40032E+00_JPRB, 0.40838E+00_JPRB, 0.42037E+00_JPRB, 0.44069E+00_JPRB ,&
&  0.52308E+00_JPRB, 0.43231E+00_JPRB, 0.96538E+00_JPRB, 0.12939E+01_JPRB, 0.13444E+01_JPRB, 0.12302E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.97332E+00_JPRB, 0.87632E+00_JPRB, 0.78817E+00_JPRB, 0.69227E+00_JPRB, 0.63606E+00_JPRB, 0.60601E+00_JPRB ,&
&  0.58462E+00_JPRB, 0.56364E+00_JPRB, 0.54663E+00_JPRB, 0.53092E+00_JPRB, 0.51620E+00_JPRB, 0.51156E+00_JPRB, 0.52065E+00_JPRB ,&
&  0.53498E+00_JPRB, 0.55796E+00_JPRB, 0.60957E+00_JPRB, 0.67309E+00_JPRB, 0.68653E+00_JPRB, 0.70579E+00_JPRB, 0.74833E+00_JPRB ,&
&  0.84496E+00_JPRB, 0.94242E+00_JPRB, 0.84543E+00_JPRB, 0.74770E+00_JPRB, 0.68532E+00_JPRB, 0.66920E+00_JPRB, 0.66846E+00_JPRB ,&
&  0.65129E+00_JPRB, 0.63109E+00_JPRB, 0.60762E+00_JPRB, 0.58109E+00_JPRB, 0.55790E+00_JPRB, 0.54437E+00_JPRB, 0.53457E+00_JPRB ,&
&  0.52056E+00_JPRB, 0.50162E+00_JPRB, 0.48912E+00_JPRB, 0.48263E+00_JPRB, 0.48005E+00_JPRB, 0.47935E+00_JPRB, 0.48267E+00_JPRB ,&
&  0.49097E+00_JPRB, 0.49339E+00_JPRB, 0.48632E+00_JPRB, 0.47156E+00_JPRB, 0.46827E+00_JPRB, 0.47323E+00_JPRB, 0.47742E+00_JPRB ,&
&  0.49370E+00_JPRB, 0.50334E+00_JPRB, 0.51032E+00_JPRB, 0.52427E+00_JPRB, 0.51126E+00_JPRB, 0.55211E+00_JPRB, 0.53160E+00_JPRB ,&
&  0.51365E+00_JPRB, 0.50933E+00_JPRB, 0.51434E+00_JPRB, 0.51365E+00_JPRB, 0.50068E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.45812E+00_JPRB, 0.44827E+00_JPRB, 0.43946E+00_JPRB, 0.43524E+00_JPRB, 0.42749E+00_JPRB, 0.42377E+00_JPRB, 0.42785E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.44181E+00_JPRB, 0.44033E+00_JPRB, 0.43808E+00_JPRB, 0.43359E+00_JPRB, 0.42744E+00_JPRB, 0.41904E+00_JPRB ,&
&  0.41177E+00_JPRB, 0.40035E+00_JPRB, 0.38576E+00_JPRB, 0.37556E+00_JPRB, 0.38020E+00_JPRB, 0.38516E+00_JPRB, 0.38482E+00_JPRB ,&
&  0.38362E+00_JPRB, 0.38920E+00_JPRB, 0.39140E+00_JPRB, 0.40114E+00_JPRB, 0.40807E+00_JPRB, 0.42237E+00_JPRB, 0.44617E+00_JPRB ,&
&  0.51831E+00_JPRB, 0.77346E+00_JPRB, 0.81051E+00_JPRB, 0.14752E+01_JPRB, 0.16226E+01_JPRB, 0.15145E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.10740E+01_JPRB, 0.92735E+00_JPRB, 0.80451E+00_JPRB, 0.72650E+00_JPRB, 0.67078E+00_JPRB, 0.62253E+00_JPRB ,&
&  0.58375E+00_JPRB, 0.55939E+00_JPRB, 0.54572E+00_JPRB, 0.53329E+00_JPRB, 0.53009E+00_JPRB, 0.53598E+00_JPRB, 0.55134E+00_JPRB ,&
&  0.57494E+00_JPRB, 0.61114E+00_JPRB, 0.68462E+00_JPRB, 0.75126E+00_JPRB, 0.73434E+00_JPRB, 0.76628E+00_JPRB, 0.85665E+00_JPRB ,&
&  0.89324E+00_JPRB, 0.96597E+00_JPRB, 0.80027E+00_JPRB, 0.69258E+00_JPRB, 0.65362E+00_JPRB, 0.65792E+00_JPRB, 0.65481E+00_JPRB ,&
&  0.63469E+00_JPRB, 0.61926E+00_JPRB, 0.59733E+00_JPRB, 0.57387E+00_JPRB, 0.55584E+00_JPRB, 0.54199E+00_JPRB, 0.52317E+00_JPRB ,&
&  0.50793E+00_JPRB, 0.50248E+00_JPRB, 0.50035E+00_JPRB, 0.49876E+00_JPRB, 0.49964E+00_JPRB, 0.50307E+00_JPRB, 0.50603E+00_JPRB ,&
&  0.50890E+00_JPRB, 0.50120E+00_JPRB, 0.47844E+00_JPRB, 0.46552E+00_JPRB, 0.46745E+00_JPRB, 0.47237E+00_JPRB, 0.47160E+00_JPRB ,&
&  0.49711E+00_JPRB, 0.51052E+00_JPRB, 0.51076E+00_JPRB, 0.51938E+00_JPRB, 0.52866E+00_JPRB, 0.59176E+00_JPRB, 0.53858E+00_JPRB ,&
&  0.51826E+00_JPRB, 0.50768E+00_JPRB, 0.50574E+00_JPRB, 0.50499E+00_JPRB, 0.48479E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.43987E+00_JPRB, 0.42798E+00_JPRB, 0.42224E+00_JPRB, 0.42237E+00_JPRB, 0.42152E+00_JPRB, 0.42099E+00_JPRB, 0.42635E+00_JPRB ,&
&  0.42869E+00_JPRB, 0.42892E+00_JPRB, 0.42718E+00_JPRB, 0.42462E+00_JPRB, 0.41840E+00_JPRB, 0.41162E+00_JPRB, 0.40429E+00_JPRB ,&
&  0.40013E+00_JPRB, 0.38493E+00_JPRB, 0.37724E+00_JPRB, 0.38549E+00_JPRB, 0.38972E+00_JPRB, 0.38896E+00_JPRB, 0.38936E+00_JPRB ,&
&  0.38907E+00_JPRB, 0.39235E+00_JPRB, 0.39251E+00_JPRB, 0.39786E+00_JPRB, 0.40773E+00_JPRB, 0.43036E+00_JPRB, 0.45842E+00_JPRB ,&
&  0.54968E+00_JPRB, 0.72260E+00_JPRB, 0.78996E+00_JPRB, 0.15124E+01_JPRB, 0.16525E+01_JPRB, 0.15523E+01_JPRB, 0.12962E+01_JPRB ,&
&  0.11507E+01_JPRB, 0.11477E+01_JPRB, 0.99567E+00_JPRB, 0.86972E+00_JPRB, 0.79237E+00_JPRB, 0.72402E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.58347E+00_JPRB, 0.55006E+00_JPRB, 0.53983E+00_JPRB, 0.54573E+00_JPRB, 0.55907E+00_JPRB, 0.57016E+00_JPRB, 0.58676E+00_JPRB ,&
&  0.62125E+00_JPRB, 0.68525E+00_JPRB, 0.77368E+00_JPRB, 0.77420E+00_JPRB, 0.75966E+00_JPRB, 0.78725E+00_JPRB, 0.82048E+00_JPRB ,&
&  0.87463E+00_JPRB, 0.85114E+00_JPRB, 0.71455E+00_JPRB, 0.65098E+00_JPRB, 0.62977E+00_JPRB, 0.64398E+00_JPRB, 0.63704E+00_JPRB ,&
&  0.61581E+00_JPRB, 0.59752E+00_JPRB, 0.57385E+00_JPRB, 0.55758E+00_JPRB, 0.54601E+00_JPRB, 0.52978E+00_JPRB, 0.51979E+00_JPRB ,&
&  0.51712E+00_JPRB, 0.51689E+00_JPRB, 0.52137E+00_JPRB, 0.52268E+00_JPRB, 0.52444E+00_JPRB, 0.52356E+00_JPRB, 0.52055E+00_JPRB ,&
&  0.51197E+00_JPRB, 0.49380E+00_JPRB, 0.47640E+00_JPRB, 0.48022E+00_JPRB, 0.49576E+00_JPRB, 0.49816E+00_JPRB, 0.49533E+00_JPRB ,&
&  0.51249E+00_JPRB, 0.52387E+00_JPRB, 0.52621E+00_JPRB, 0.51872E+00_JPRB, 0.53326E+00_JPRB, 0.56536E+00_JPRB, 0.52719E+00_JPRB ,&
&  0.51778E+00_JPRB, 0.50700E+00_JPRB, 0.50052E+00_JPRB, 0.49526E+00_JPRB, 0.46910E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.44382E+00_JPRB, 0.42453E+00_JPRB, 0.41633E+00_JPRB, 0.41128E+00_JPRB, 0.41059E+00_JPRB, 0.41444E+00_JPRB, 0.41798E+00_JPRB ,&
&  0.41633E+00_JPRB, 0.41545E+00_JPRB, 0.41620E+00_JPRB, 0.41280E+00_JPRB, 0.40326E+00_JPRB, 0.39460E+00_JPRB, 0.38720E+00_JPRB ,&
&  0.38763E+00_JPRB, 0.38998E+00_JPRB, 0.40096E+00_JPRB, 0.40797E+00_JPRB, 0.40933E+00_JPRB, 0.40734E+00_JPRB, 0.40755E+00_JPRB ,&
&  0.41007E+00_JPRB, 0.40866E+00_JPRB, 0.40612E+00_JPRB, 0.41497E+00_JPRB, 0.43082E+00_JPRB, 0.46151E+00_JPRB, 0.50738E+00_JPRB ,&
&  0.63152E+00_JPRB, 0.74880E+00_JPRB, 0.82660E+00_JPRB, 0.12584E+01_JPRB, 0.15099E+01_JPRB, 0.14159E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.98379E+00_JPRB, 0.10182E+01_JPRB, 0.90377E+00_JPRB, 0.80985E+00_JPRB, 0.73369E+00_JPRB, 0.66945E+00_JPRB ,&
&  0.61751E+00_JPRB, 0.59024E+00_JPRB, 0.59695E+00_JPRB, 0.61264E+00_JPRB, 0.62040E+00_JPRB, 0.62468E+00_JPRB, 0.65110E+00_JPRB ,&
&  0.70152E+00_JPRB, 0.78301E+00_JPRB, 0.87005E+00_JPRB, 0.84581E+00_JPRB, 0.75038E+00_JPRB, 0.76056E+00_JPRB, 0.77024E+00_JPRB ,&
&  0.81046E+00_JPRB, 0.74307E+00_JPRB, 0.67030E+00_JPRB, 0.63636E+00_JPRB, 0.60236E+00_JPRB, 0.62919E+00_JPRB, 0.61542E+00_JPRB ,&
&  0.59910E+00_JPRB, 0.57607E+00_JPRB, 0.55404E+00_JPRB, 0.54147E+00_JPRB, 0.53162E+00_JPRB, 0.52491E+00_JPRB, 0.51924E+00_JPRB ,&
&  0.51706E+00_JPRB, 0.51716E+00_JPRB, 0.52508E+00_JPRB, 0.52829E+00_JPRB, 0.52633E+00_JPRB, 0.51517E+00_JPRB, 0.49906E+00_JPRB ,&
&  0.48841E+00_JPRB, 0.49364E+00_JPRB, 0.49368E+00_JPRB, 0.50840E+00_JPRB, 0.52183E+00_JPRB, 0.52707E+00_JPRB, 0.51656E+00_JPRB ,&
&  0.51636E+00_JPRB, 0.53193E+00_JPRB, 0.53977E+00_JPRB, 0.54587E+00_JPRB, 0.54863E+00_JPRB, 0.54879E+00_JPRB, 0.52026E+00_JPRB ,&
&  0.50750E+00_JPRB, 0.49790E+00_JPRB, 0.50135E+00_JPRB, 0.49361E+00_JPRB, 0.46469E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.45182E+00_JPRB, 0.44244E+00_JPRB, 0.40507E+00_JPRB, 0.40623E+00_JPRB, 0.40592E+00_JPRB, 0.40984E+00_JPRB, 0.40486E+00_JPRB ,&
&  0.40121E+00_JPRB, 0.40242E+00_JPRB, 0.40467E+00_JPRB, 0.39810E+00_JPRB, 0.37872E+00_JPRB, 0.37025E+00_JPRB, 0.37385E+00_JPRB ,&
&  0.38417E+00_JPRB, 0.40428E+00_JPRB, 0.42847E+00_JPRB, 0.43172E+00_JPRB, 0.43462E+00_JPRB, 0.43237E+00_JPRB, 0.42906E+00_JPRB ,&
&  0.43120E+00_JPRB, 0.43583E+00_JPRB, 0.44288E+00_JPRB, 0.46001E+00_JPRB, 0.48985E+00_JPRB, 0.53422E+00_JPRB, 0.60263E+00_JPRB ,&
&  0.76463E+00_JPRB, 0.80956E+00_JPRB, 0.43642E+00_JPRB, 0.78598E+00_JPRB, 0.11635E+01_JPRB, 0.12046E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.10348E+01_JPRB, 0.91496E+00_JPRB, 0.89578E+00_JPRB, 0.88066E+00_JPRB, 0.79494E+00_JPRB, 0.74804E+00_JPRB, 0.70229E+00_JPRB ,&
&  0.66302E+00_JPRB, 0.65700E+00_JPRB, 0.67693E+00_JPRB, 0.68786E+00_JPRB, 0.69004E+00_JPRB, 0.71449E+00_JPRB, 0.76278E+00_JPRB ,&
&  0.82519E+00_JPRB, 0.90753E+00_JPRB, 0.96716E+00_JPRB, 0.83936E+00_JPRB, 0.76879E+00_JPRB, 0.74279E+00_JPRB, 0.71061E+00_JPRB ,&
&  0.67854E+00_JPRB, 0.67688E+00_JPRB, 0.65332E+00_JPRB, 0.62397E+00_JPRB, 0.59134E+00_JPRB, 0.57949E+00_JPRB, 0.59367E+00_JPRB ,&
&  0.57633E+00_JPRB, 0.54768E+00_JPRB, 0.52918E+00_JPRB, 0.51961E+00_JPRB, 0.51143E+00_JPRB, 0.50754E+00_JPRB, 0.50242E+00_JPRB ,&
&  0.50163E+00_JPRB, 0.50759E+00_JPRB, 0.51345E+00_JPRB, 0.51134E+00_JPRB, 0.50217E+00_JPRB, 0.48891E+00_JPRB, 0.47844E+00_JPRB ,&
&  0.48433E+00_JPRB, 0.50210E+00_JPRB, 0.52551E+00_JPRB, 0.53155E+00_JPRB, 0.53466E+00_JPRB, 0.54296E+00_JPRB, 0.54251E+00_JPRB ,&
&  0.54528E+00_JPRB, 0.55681E+00_JPRB, 0.54273E+00_JPRB, 0.53714E+00_JPRB, 0.55426E+00_JPRB, 0.54317E+00_JPRB, 0.53706E+00_JPRB ,&
&  0.57612E+00_JPRB, 0.65756E+00_JPRB, 0.78883E+00_JPRB, 0.53472E+00_JPRB, 0.48247E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.46481E+00_JPRB, 0.42816E+00_JPRB, 0.39383E+00_JPRB, 0.39234E+00_JPRB, 0.40791E+00_JPRB, 0.41041E+00_JPRB, 0.40610E+00_JPRB ,&
&  0.39514E+00_JPRB, 0.38476E+00_JPRB, 0.37180E+00_JPRB, 0.36432E+00_JPRB, 0.36785E+00_JPRB, 0.37631E+00_JPRB, 0.40419E+00_JPRB ,&
&  0.43211E+00_JPRB, 0.45613E+00_JPRB, 0.47597E+00_JPRB, 0.47985E+00_JPRB, 0.47233E+00_JPRB, 0.47365E+00_JPRB, 0.48083E+00_JPRB ,&
&  0.48481E+00_JPRB, 0.50004E+00_JPRB, 0.52050E+00_JPRB, 0.54939E+00_JPRB, 0.59216E+00_JPRB, 0.65937E+00_JPRB, 0.76245E+00_JPRB ,&
&  0.91989E+00_JPRB, 0.64469E+00_JPRB, 0.48904E+00_JPRB, 0.92243E+00_JPRB, 0.11146E+01_JPRB, 0.11319E+01_JPRB, 0.95595E+00_JPRB ,&
&  0.90676E+00_JPRB, 0.85956E+00_JPRB, 0.84680E+00_JPRB, 0.87021E+00_JPRB, 0.79371E+00_JPRB, 0.76734E+00_JPRB, 0.73861E+00_JPRB ,&
&  0.72214E+00_JPRB, 0.73452E+00_JPRB, 0.75992E+00_JPRB, 0.77919E+00_JPRB, 0.80377E+00_JPRB, 0.85945E+00_JPRB, 0.91719E+00_JPRB ,&
&  0.99234E+00_JPRB, 0.10529E+01_JPRB, 0.10832E+01_JPRB, 0.88800E+00_JPRB, 0.79380E+00_JPRB, 0.72815E+00_JPRB, 0.69115E+00_JPRB ,&
&  0.64218E+00_JPRB, 0.63667E+00_JPRB, 0.62798E+00_JPRB, 0.60467E+00_JPRB, 0.57890E+00_JPRB, 0.55689E+00_JPRB, 0.56893E+00_JPRB ,&
&  0.54128E+00_JPRB, 0.51327E+00_JPRB, 0.49442E+00_JPRB, 0.48395E+00_JPRB, 0.48115E+00_JPRB, 0.48145E+00_JPRB, 0.47640E+00_JPRB ,&
&  0.48098E+00_JPRB, 0.48321E+00_JPRB, 0.48851E+00_JPRB, 0.48253E+00_JPRB, 0.47864E+00_JPRB, 0.47829E+00_JPRB, 0.47050E+00_JPRB ,&
&  0.49144E+00_JPRB, 0.51954E+00_JPRB, 0.52912E+00_JPRB, 0.54472E+00_JPRB, 0.54319E+00_JPRB, 0.53491E+00_JPRB, 0.54404E+00_JPRB ,&
&  0.54377E+00_JPRB, 0.55584E+00_JPRB, 0.54728E+00_JPRB, 0.55576E+00_JPRB, 0.54910E+00_JPRB, 0.55039E+00_JPRB, 0.56461E+00_JPRB ,&
&  0.61274E+00_JPRB, 0.67087E+00_JPRB, 0.67241E+00_JPRB, 0.52898E+00_JPRB, 0.48602E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.44114E+00_JPRB, 0.41411E+00_JPRB, 0.37750E+00_JPRB, 0.36371E+00_JPRB, 0.38289E+00_JPRB, 0.39730E+00_JPRB, 0.38982E+00_JPRB ,&
&  0.38229E+00_JPRB, 0.37322E+00_JPRB, 0.36832E+00_JPRB, 0.37449E+00_JPRB, 0.40272E+00_JPRB, 0.44061E+00_JPRB, 0.46602E+00_JPRB ,&
&  0.49166E+00_JPRB, 0.51145E+00_JPRB, 0.52882E+00_JPRB, 0.53534E+00_JPRB, 0.53395E+00_JPRB, 0.54097E+00_JPRB, 0.54968E+00_JPRB ,&
&  0.56551E+00_JPRB, 0.59000E+00_JPRB, 0.62322E+00_JPRB, 0.67160E+00_JPRB, 0.73461E+00_JPRB, 0.80909E+00_JPRB, 0.88006E+00_JPRB ,&
&  0.65330E+00_JPRB, 0.38601E+00_JPRB, 0.85490E+00_JPRB, 0.10196E+01_JPRB, 0.95993E+00_JPRB, 0.93709E+00_JPRB, 0.86902E+00_JPRB ,&
&  0.81343E+00_JPRB, 0.81742E+00_JPRB, 0.78838E+00_JPRB, 0.83094E+00_JPRB, 0.77804E+00_JPRB, 0.78009E+00_JPRB, 0.79088E+00_JPRB ,&
&  0.81454E+00_JPRB, 0.84494E+00_JPRB, 0.88346E+00_JPRB, 0.92534E+00_JPRB, 0.97244E+00_JPRB, 0.10494E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.12495E+01_JPRB, 0.12686E+01_JPRB, 0.98773E+00_JPRB, 0.86077E+00_JPRB, 0.79010E+00_JPRB, 0.72870E+00_JPRB ,&
&  0.66303E+00_JPRB, 0.58810E+00_JPRB, 0.57109E+00_JPRB, 0.58434E+00_JPRB, 0.57047E+00_JPRB, 0.53536E+00_JPRB, 0.54215E+00_JPRB ,&
&  0.50463E+00_JPRB, 0.47315E+00_JPRB, 0.45330E+00_JPRB, 0.43913E+00_JPRB, 0.44498E+00_JPRB, 0.44942E+00_JPRB, 0.45130E+00_JPRB ,&
&  0.44461E+00_JPRB, 0.44715E+00_JPRB, 0.45969E+00_JPRB, 0.46323E+00_JPRB, 0.47046E+00_JPRB, 0.48057E+00_JPRB, 0.48570E+00_JPRB ,&
&  0.51422E+00_JPRB, 0.52294E+00_JPRB, 0.53362E+00_JPRB, 0.54856E+00_JPRB, 0.55435E+00_JPRB, 0.53758E+00_JPRB, 0.53209E+00_JPRB ,&
&  0.53502E+00_JPRB, 0.54076E+00_JPRB, 0.53907E+00_JPRB, 0.54934E+00_JPRB, 0.55243E+00_JPRB, 0.56141E+00_JPRB, 0.58058E+00_JPRB ,&
&  0.60617E+00_JPRB, 0.61381E+00_JPRB, 0.58305E+00_JPRB, 0.50794E+00_JPRB, 0.47244E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.42137E+00_JPRB, 0.39805E+00_JPRB, 0.36324E+00_JPRB, 0.33051E+00_JPRB, 0.34975E+00_JPRB, 0.34728E+00_JPRB, 0.35879E+00_JPRB ,&
&  0.36157E+00_JPRB, 0.38240E+00_JPRB, 0.40175E+00_JPRB, 0.43102E+00_JPRB, 0.46490E+00_JPRB, 0.49698E+00_JPRB, 0.52189E+00_JPRB ,&
&  0.54306E+00_JPRB, 0.56518E+00_JPRB, 0.58246E+00_JPRB, 0.58777E+00_JPRB, 0.59783E+00_JPRB, 0.61445E+00_JPRB, 0.63208E+00_JPRB ,&
&  0.65532E+00_JPRB, 0.68867E+00_JPRB, 0.73668E+00_JPRB, 0.80050E+00_JPRB, 0.87581E+00_JPRB, 0.90574E+00_JPRB, 0.90959E+00_JPRB ,&
&  0.56128E+00_JPRB, 0.83579E+00_JPRB, 0.95916E+00_JPRB, 0.90655E+00_JPRB, 0.87134E+00_JPRB, 0.84649E+00_JPRB, 0.82734E+00_JPRB ,&
&  0.79393E+00_JPRB, 0.74786E+00_JPRB, 0.73465E+00_JPRB, 0.80278E+00_JPRB, 0.81033E+00_JPRB, 0.86070E+00_JPRB, 0.90468E+00_JPRB ,&
&  0.95098E+00_JPRB, 0.99693E+00_JPRB, 0.10474E+01_JPRB, 0.11067E+01_JPRB, 0.11716E+01_JPRB, 0.12366E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.13887E+01_JPRB, 0.14540E+01_JPRB, 0.14679E+01_JPRB, 0.11701E+01_JPRB, 0.10742E+01_JPRB, 0.10161E+01_JPRB, 0.91982E+00_JPRB ,&
&  0.66468E+00_JPRB, 0.55915E+00_JPRB, 0.55888E+00_JPRB, 0.58081E+00_JPRB, 0.55998E+00_JPRB, 0.53758E+00_JPRB, 0.51033E+00_JPRB ,&
&  0.47735E+00_JPRB, 0.44719E+00_JPRB, 0.43111E+00_JPRB, 0.42631E+00_JPRB, 0.43994E+00_JPRB, 0.43985E+00_JPRB, 0.42707E+00_JPRB ,&
&  0.41088E+00_JPRB, 0.41447E+00_JPRB, 0.44100E+00_JPRB, 0.46193E+00_JPRB, 0.49276E+00_JPRB, 0.51226E+00_JPRB, 0.52446E+00_JPRB ,&
&  0.53697E+00_JPRB, 0.52525E+00_JPRB, 0.53474E+00_JPRB, 0.54520E+00_JPRB, 0.54355E+00_JPRB, 0.53800E+00_JPRB, 0.52728E+00_JPRB ,&
&  0.52181E+00_JPRB, 0.53647E+00_JPRB, 0.54879E+00_JPRB, 0.56096E+00_JPRB, 0.56222E+00_JPRB, 0.55909E+00_JPRB, 0.56242E+00_JPRB ,&
&  0.55371E+00_JPRB, 0.53520E+00_JPRB, 0.51240E+00_JPRB, 0.47928E+00_JPRB, 0.44835E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.41242E+00_JPRB, 0.40017E+00_JPRB, 0.38422E+00_JPRB, 0.37469E+00_JPRB, 0.36606E+00_JPRB, 0.35654E+00_JPRB, 0.36490E+00_JPRB ,&
&  0.38871E+00_JPRB, 0.40910E+00_JPRB, 0.44337E+00_JPRB, 0.49403E+00_JPRB, 0.53668E+00_JPRB, 0.55545E+00_JPRB, 0.57726E+00_JPRB ,&
&  0.59488E+00_JPRB, 0.61629E+00_JPRB, 0.63342E+00_JPRB, 0.64693E+00_JPRB, 0.66560E+00_JPRB, 0.68960E+00_JPRB, 0.72234E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.80576E+00_JPRB, 0.86113E+00_JPRB, 0.93432E+00_JPRB, 0.10038E+01_JPRB, 0.10216E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.80396E+00_JPRB, 0.94613E+00_JPRB, 0.92721E+00_JPRB, 0.88549E+00_JPRB, 0.84018E+00_JPRB, 0.80291E+00_JPRB, 0.78353E+00_JPRB ,&
&  0.77383E+00_JPRB, 0.73559E+00_JPRB, 0.74176E+00_JPRB, 0.80044E+00_JPRB, 0.90426E+00_JPRB, 0.96201E+00_JPRB, 0.10156E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.11335E+01_JPRB, 0.11948E+01_JPRB, 0.12595E+01_JPRB, 0.13196E+01_JPRB, 0.13963E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.15520E+01_JPRB, 0.15663E+01_JPRB, 0.12972E+01_JPRB, 0.10794E+01_JPRB, 0.95461E+00_JPRB, 0.77054E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.54915E+00_JPRB, 0.56874E+00_JPRB, 0.59236E+00_JPRB, 0.56454E+00_JPRB, 0.52883E+00_JPRB, 0.48884E+00_JPRB ,&
&  0.47090E+00_JPRB, 0.43271E+00_JPRB, 0.41986E+00_JPRB, 0.42711E+00_JPRB, 0.44737E+00_JPRB, 0.43019E+00_JPRB, 0.40689E+00_JPRB ,&
&  0.39053E+00_JPRB, 0.38366E+00_JPRB, 0.41623E+00_JPRB, 0.45388E+00_JPRB, 0.50313E+00_JPRB, 0.52644E+00_JPRB, 0.52333E+00_JPRB ,&
&  0.56108E+00_JPRB, 0.51826E+00_JPRB, 0.51785E+00_JPRB, 0.52643E+00_JPRB, 0.52957E+00_JPRB, 0.53465E+00_JPRB, 0.54421E+00_JPRB ,&
&  0.57786E+00_JPRB, 0.64111E+00_JPRB, 0.66514E+00_JPRB, 0.61928E+00_JPRB, 0.58095E+00_JPRB, 0.54185E+00_JPRB, 0.49825E+00_JPRB ,&
&  0.50165E+00_JPRB, 0.49061E+00_JPRB, 0.47882E+00_JPRB, 0.46173E+00_JPRB, 0.43789E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.46617E+00_JPRB, 0.45390E+00_JPRB, 0.44648E+00_JPRB, 0.44013E+00_JPRB, 0.42488E+00_JPRB, 0.42479E+00_JPRB, 0.42125E+00_JPRB ,&
&  0.43651E+00_JPRB, 0.45513E+00_JPRB, 0.48802E+00_JPRB, 0.52631E+00_JPRB, 0.57452E+00_JPRB, 0.60449E+00_JPRB, 0.63812E+00_JPRB ,&
&  0.65413E+00_JPRB, 0.67714E+00_JPRB, 0.70731E+00_JPRB, 0.72839E+00_JPRB, 0.75551E+00_JPRB, 0.76910E+00_JPRB, 0.80287E+00_JPRB ,&
&  0.83135E+00_JPRB, 0.86509E+00_JPRB, 0.91572E+00_JPRB, 0.95895E+00_JPRB, 0.99580E+00_JPRB, 0.98659E+00_JPRB, 0.77694E+00_JPRB ,&
&  0.94647E+00_JPRB, 0.94322E+00_JPRB, 0.91870E+00_JPRB, 0.86678E+00_JPRB, 0.80874E+00_JPRB, 0.76043E+00_JPRB, 0.74820E+00_JPRB ,&
&  0.74763E+00_JPRB, 0.74888E+00_JPRB, 0.76785E+00_JPRB, 0.80546E+00_JPRB, 0.95602E+00_JPRB, 0.10358E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.12181E+01_JPRB, 0.12851E+01_JPRB, 0.13590E+01_JPRB, 0.14192E+01_JPRB, 0.14878E+01_JPRB, 0.15615E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.16374E+01_JPRB, 0.15771E+01_JPRB, 0.12567E+01_JPRB, 0.10805E+01_JPRB, 0.93518E+00_JPRB, 0.73867E+00_JPRB ,&
&  0.62355E+00_JPRB, 0.59032E+00_JPRB, 0.61219E+00_JPRB, 0.63024E+00_JPRB, 0.58726E+00_JPRB, 0.53964E+00_JPRB, 0.50887E+00_JPRB ,&
&  0.46840E+00_JPRB, 0.46153E+00_JPRB, 0.45132E+00_JPRB, 0.46868E+00_JPRB, 0.47666E+00_JPRB, 0.45772E+00_JPRB, 0.44567E+00_JPRB ,&
&  0.42556E+00_JPRB, 0.40923E+00_JPRB, 0.42888E+00_JPRB, 0.46438E+00_JPRB, 0.54389E+00_JPRB, 0.62004E+00_JPRB, 0.64746E+00_JPRB ,&
&  0.69324E+00_JPRB, 0.46339E+00_JPRB, 0.48434E+00_JPRB, 0.50481E+00_JPRB, 0.54575E+00_JPRB, 0.59029E+00_JPRB, 0.67962E+00_JPRB ,&
&  0.87235E+00_JPRB, 0.12159E+01_JPRB, 0.10826E+01_JPRB, 0.82292E+00_JPRB, 0.91023E+00_JPRB, 0.83486E+00_JPRB, 0.44781E+00_JPRB ,&
&  0.44977E+00_JPRB, 0.45109E+00_JPRB, 0.46629E+00_JPRB, 0.47060E+00_JPRB, 0.47352E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.53982E+00_JPRB, 0.53423E+00_JPRB, 0.53338E+00_JPRB, 0.52401E+00_JPRB, 0.51876E+00_JPRB, 0.51919E+00_JPRB, 0.50659E+00_JPRB ,&
&  0.51567E+00_JPRB, 0.53102E+00_JPRB, 0.53635E+00_JPRB, 0.55194E+00_JPRB, 0.56018E+00_JPRB, 0.59037E+00_JPRB, 0.63440E+00_JPRB ,&
&  0.66068E+00_JPRB, 0.72427E+00_JPRB, 0.74995E+00_JPRB, 0.78281E+00_JPRB, 0.80834E+00_JPRB, 0.83439E+00_JPRB, 0.85527E+00_JPRB ,&
&  0.88838E+00_JPRB, 0.92569E+00_JPRB, 0.96886E+00_JPRB, 0.99602E+00_JPRB, 0.98532E+00_JPRB, 0.10418E+01_JPRB, 0.82420E+00_JPRB ,&
&  0.95984E+00_JPRB, 0.94391E+00_JPRB, 0.86176E+00_JPRB, 0.81428E+00_JPRB, 0.76978E+00_JPRB, 0.74092E+00_JPRB, 0.72183E+00_JPRB ,&
&  0.72269E+00_JPRB, 0.73266E+00_JPRB, 0.76164E+00_JPRB, 0.84062E+00_JPRB, 0.93997E+00_JPRB, 0.10311E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11938E+01_JPRB, 0.12622E+01_JPRB, 0.13376E+01_JPRB, 0.14193E+01_JPRB, 0.14756E+01_JPRB, 0.15335E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.15686E+01_JPRB, 0.14621E+01_JPRB, 0.12984E+01_JPRB, 0.11570E+01_JPRB, 0.10026E+01_JPRB, 0.82078E+00_JPRB ,&
&  0.66510E+00_JPRB, 0.62906E+00_JPRB, 0.64383E+00_JPRB, 0.68641E+00_JPRB, 0.63361E+00_JPRB, 0.59315E+00_JPRB, 0.57096E+00_JPRB ,&
&  0.54324E+00_JPRB, 0.50070E+00_JPRB, 0.51267E+00_JPRB, 0.54465E+00_JPRB, 0.55261E+00_JPRB, 0.54166E+00_JPRB, 0.52207E+00_JPRB ,&
&  0.50058E+00_JPRB, 0.47061E+00_JPRB, 0.46607E+00_JPRB, 0.46894E+00_JPRB, 0.48070E+00_JPRB, 0.50893E+00_JPRB, 0.46044E+00_JPRB ,&
&  0.51466E+00_JPRB, 0.45681E+00_JPRB, 0.47754E+00_JPRB, 0.52095E+00_JPRB, 0.60124E+00_JPRB, 0.77601E+00_JPRB, 0.11032E+01_JPRB ,&
&  0.16891E+01_JPRB, 0.19487E+01_JPRB, 0.22052E+01_JPRB, 0.27610E+01_JPRB, 0.24071E+01_JPRB, 0.17679E+01_JPRB, 0.50285E+00_JPRB ,&
&  0.49964E+00_JPRB, 0.50350E+00_JPRB, 0.52375E+00_JPRB, 0.53191E+00_JPRB, 0.53650E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.57816E+00_JPRB, 0.58636E+00_JPRB, 0.59767E+00_JPRB, 0.60961E+00_JPRB, 0.62422E+00_JPRB, 0.63881E+00_JPRB, 0.65006E+00_JPRB ,&
&  0.66178E+00_JPRB, 0.67649E+00_JPRB, 0.69058E+00_JPRB, 0.69987E+00_JPRB, 0.70902E+00_JPRB, 0.72743E+00_JPRB, 0.74908E+00_JPRB ,&
&  0.75578E+00_JPRB, 0.79093E+00_JPRB, 0.80262E+00_JPRB, 0.83265E+00_JPRB, 0.85841E+00_JPRB, 0.89115E+00_JPRB, 0.91238E+00_JPRB ,&
&  0.92975E+00_JPRB, 0.95875E+00_JPRB, 0.96103E+00_JPRB, 0.94533E+00_JPRB, 0.94166E+00_JPRB, 0.96938E+00_JPRB, 0.76389E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.96903E+00_JPRB, 0.84651E+00_JPRB, 0.78342E+00_JPRB, 0.73465E+00_JPRB, 0.70957E+00_JPRB, 0.69635E+00_JPRB ,&
&  0.69218E+00_JPRB, 0.71836E+00_JPRB, 0.71001E+00_JPRB, 0.73085E+00_JPRB, 0.82892E+00_JPRB, 0.94720E+00_JPRB, 0.10294E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10998E+01_JPRB, 0.11144E+01_JPRB, 0.11380E+01_JPRB, 0.11926E+01_JPRB, 0.12426E+01_JPRB, 0.12731E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.12335E+01_JPRB, 0.11396E+01_JPRB, 0.11950E+01_JPRB, 0.11792E+01_JPRB, 0.10458E+01_JPRB, 0.86674E+00_JPRB ,&
&  0.65141E+00_JPRB, 0.67437E+00_JPRB, 0.72248E+00_JPRB, 0.79581E+00_JPRB, 0.75866E+00_JPRB, 0.72112E+00_JPRB, 0.64522E+00_JPRB ,&
&  0.61383E+00_JPRB, 0.59977E+00_JPRB, 0.62037E+00_JPRB, 0.64813E+00_JPRB, 0.66878E+00_JPRB, 0.65726E+00_JPRB, 0.61650E+00_JPRB ,&
&  0.57908E+00_JPRB, 0.53272E+00_JPRB, 0.50598E+00_JPRB, 0.49257E+00_JPRB, 0.46719E+00_JPRB, 0.40886E+00_JPRB, 0.47789E+00_JPRB ,&
&  0.43726E+00_JPRB, 0.41656E+00_JPRB, 0.49295E+00_JPRB, 0.38338E+00_JPRB, 0.56953E+00_JPRB, 0.73362E+00_JPRB, 0.99850E+00_JPRB ,&
&  0.12072E+01_JPRB, 0.17174E+01_JPRB, 0.21728E+01_JPRB, 0.16454E+01_JPRB, 0.11244E+01_JPRB, 0.62871E+00_JPRB, 0.53855E+00_JPRB ,&
&  0.54544E+00_JPRB, 0.55028E+00_JPRB, 0.55838E+00_JPRB, 0.55799E+00_JPRB, 0.56864E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.59398E+00_JPRB, 0.60555E+00_JPRB, 0.62153E+00_JPRB, 0.64303E+00_JPRB, 0.66338E+00_JPRB, 0.67849E+00_JPRB, 0.68803E+00_JPRB ,&
&  0.70514E+00_JPRB, 0.72822E+00_JPRB, 0.74118E+00_JPRB, 0.74730E+00_JPRB, 0.76137E+00_JPRB, 0.77457E+00_JPRB, 0.79885E+00_JPRB ,&
&  0.81380E+00_JPRB, 0.84841E+00_JPRB, 0.87395E+00_JPRB, 0.89972E+00_JPRB, 0.91947E+00_JPRB, 0.94365E+00_JPRB, 0.96624E+00_JPRB ,&
&  0.98437E+00_JPRB, 0.10227E+01_JPRB, 0.10582E+01_JPRB, 0.10865E+01_JPRB, 0.11258E+01_JPRB, 0.11401E+01_JPRB, 0.91682E+00_JPRB ,&
&  0.11192E+01_JPRB, 0.10517E+01_JPRB, 0.90933E+00_JPRB, 0.80171E+00_JPRB, 0.73262E+00_JPRB, 0.67813E+00_JPRB, 0.64738E+00_JPRB ,&
&  0.64160E+00_JPRB, 0.65134E+00_JPRB, 0.65972E+00_JPRB, 0.64624E+00_JPRB, 0.63596E+00_JPRB, 0.68933E+00_JPRB, 0.81131E+00_JPRB ,&
&  0.88357E+00_JPRB, 0.91670E+00_JPRB, 0.94051E+00_JPRB, 0.93233E+00_JPRB, 0.96294E+00_JPRB, 0.10096E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.96523E+00_JPRB, 0.86829E+00_JPRB, 0.10332E+01_JPRB, 0.10665E+01_JPRB, 0.10575E+01_JPRB, 0.95831E+00_JPRB ,&
&  0.81598E+00_JPRB, 0.74741E+00_JPRB, 0.78752E+00_JPRB, 0.88976E+00_JPRB, 0.87000E+00_JPRB, 0.84775E+00_JPRB, 0.78945E+00_JPRB ,&
&  0.75206E+00_JPRB, 0.72757E+00_JPRB, 0.73176E+00_JPRB, 0.75132E+00_JPRB, 0.76176E+00_JPRB, 0.75262E+00_JPRB, 0.69606E+00_JPRB ,&
&  0.64498E+00_JPRB, 0.59474E+00_JPRB, 0.55932E+00_JPRB, 0.52546E+00_JPRB, 0.49184E+00_JPRB, 0.41950E+00_JPRB, 0.45398E+00_JPRB ,&
&  0.41676E+00_JPRB, 0.33021E+00_JPRB, 0.38892E+00_JPRB, 0.45912E+00_JPRB, 0.50352E+00_JPRB, 0.57942E+00_JPRB, 0.74200E+00_JPRB ,&
&  0.95168E+00_JPRB, 0.10962E+01_JPRB, 0.10723E+01_JPRB, 0.80926E+00_JPRB, 0.62800E+00_JPRB, 0.57044E+00_JPRB, 0.56846E+00_JPRB ,&
&  0.57616E+00_JPRB, 0.57436E+00_JPRB, 0.57677E+00_JPRB, 0.58097E+00_JPRB, 0.58848E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.57582E+00_JPRB, 0.57443E+00_JPRB, 0.58077E+00_JPRB, 0.59381E+00_JPRB, 0.59925E+00_JPRB, 0.60253E+00_JPRB, 0.60400E+00_JPRB ,&
&  0.61928E+00_JPRB, 0.63688E+00_JPRB, 0.65302E+00_JPRB, 0.66913E+00_JPRB, 0.68245E+00_JPRB, 0.69442E+00_JPRB, 0.72631E+00_JPRB ,&
&  0.74167E+00_JPRB, 0.76410E+00_JPRB, 0.77922E+00_JPRB, 0.79455E+00_JPRB, 0.80725E+00_JPRB, 0.80019E+00_JPRB, 0.80298E+00_JPRB ,&
&  0.81111E+00_JPRB, 0.83586E+00_JPRB, 0.86361E+00_JPRB, 0.91071E+00_JPRB, 0.97093E+00_JPRB, 0.98179E+00_JPRB, 0.10936E+01_JPRB ,&
&  0.10481E+01_JPRB, 0.11770E+01_JPRB, 0.10108E+01_JPRB, 0.85500E+00_JPRB, 0.81462E+00_JPRB, 0.73051E+00_JPRB, 0.68444E+00_JPRB ,&
&  0.68111E+00_JPRB, 0.65680E+00_JPRB, 0.62813E+00_JPRB, 0.59479E+00_JPRB, 0.55901E+00_JPRB, 0.56878E+00_JPRB, 0.59431E+00_JPRB ,&
&  0.68273E+00_JPRB, 0.73723E+00_JPRB, 0.81653E+00_JPRB, 0.89347E+00_JPRB, 0.98364E+00_JPRB, 0.10178E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.10145E+01_JPRB, 0.98000E+00_JPRB, 0.90871E+00_JPRB, 0.10201E+01_JPRB, 0.11144E+01_JPRB, 0.11658E+01_JPRB, 0.11638E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.99278E+00_JPRB, 0.98946E+00_JPRB, 0.99285E+00_JPRB, 0.96405E+00_JPRB, 0.96510E+00_JPRB, 0.91264E+00_JPRB ,&
&  0.86944E+00_JPRB, 0.82666E+00_JPRB, 0.80398E+00_JPRB, 0.80145E+00_JPRB, 0.79437E+00_JPRB, 0.77764E+00_JPRB, 0.70228E+00_JPRB ,&
&  0.65356E+00_JPRB, 0.61444E+00_JPRB, 0.58524E+00_JPRB, 0.54893E+00_JPRB, 0.45464E+00_JPRB, 0.44041E+00_JPRB, 0.43211E+00_JPRB ,&
&  0.40560E+00_JPRB, 0.36605E+00_JPRB, 0.36498E+00_JPRB, 0.47016E+00_JPRB, 0.51088E+00_JPRB, 0.55772E+00_JPRB, 0.62528E+00_JPRB ,&
&  0.63976E+00_JPRB, 0.62781E+00_JPRB, 0.56609E+00_JPRB, 0.50628E+00_JPRB, 0.52016E+00_JPRB, 0.52253E+00_JPRB, 0.52556E+00_JPRB ,&
&  0.53979E+00_JPRB, 0.54822E+00_JPRB, 0.55871E+00_JPRB, 0.57772E+00_JPRB, 0.57227E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.56629E+00_JPRB, 0.55526E+00_JPRB, 0.54430E+00_JPRB, 0.53313E+00_JPRB, 0.53817E+00_JPRB, 0.53788E+00_JPRB, 0.54552E+00_JPRB ,&
&  0.55445E+00_JPRB, 0.55212E+00_JPRB, 0.55497E+00_JPRB, 0.55553E+00_JPRB, 0.56729E+00_JPRB, 0.57766E+00_JPRB, 0.60493E+00_JPRB ,&
&  0.63035E+00_JPRB, 0.64314E+00_JPRB, 0.66129E+00_JPRB, 0.67858E+00_JPRB, 0.69143E+00_JPRB, 0.70431E+00_JPRB, 0.68861E+00_JPRB ,&
&  0.67072E+00_JPRB, 0.67154E+00_JPRB, 0.70667E+00_JPRB, 0.82659E+00_JPRB, 0.10549E+01_JPRB, 0.13525E+01_JPRB, 0.19276E+01_JPRB ,&
&  0.18118E+01_JPRB, 0.11962E+01_JPRB, 0.10525E+01_JPRB, 0.91838E+00_JPRB, 0.95416E+00_JPRB, 0.10251E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.99993E+00_JPRB, 0.98754E+00_JPRB, 0.97011E+00_JPRB, 0.94467E+00_JPRB, 0.89049E+00_JPRB, 0.82026E+00_JPRB, 0.76671E+00_JPRB ,&
&  0.74252E+00_JPRB, 0.71365E+00_JPRB, 0.80448E+00_JPRB, 0.10373E+01_JPRB, 0.11949E+01_JPRB, 0.14633E+01_JPRB, 0.14123E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.14126E+01_JPRB, 0.13393E+01_JPRB, 0.14245E+01_JPRB, 0.14559E+01_JPRB, 0.15097E+01_JPRB, 0.15023E+01_JPRB ,&
&  0.15490E+01_JPRB, 0.13497E+01_JPRB, 0.97384E+00_JPRB, 0.10891E+01_JPRB, 0.10637E+01_JPRB, 0.10807E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.97177E+00_JPRB, 0.90507E+00_JPRB, 0.86023E+00_JPRB, 0.82725E+00_JPRB, 0.79736E+00_JPRB, 0.75800E+00_JPRB, 0.67330E+00_JPRB ,&
&  0.63497E+00_JPRB, 0.60807E+00_JPRB, 0.58660E+00_JPRB, 0.54369E+00_JPRB, 0.51475E+00_JPRB, 0.38098E+00_JPRB, 0.41052E+00_JPRB ,&
&  0.43485E+00_JPRB, 0.44625E+00_JPRB, 0.41356E+00_JPRB, 0.50627E+00_JPRB, 0.53978E+00_JPRB, 0.52143E+00_JPRB, 0.52581E+00_JPRB ,&
&  0.50451E+00_JPRB, 0.48612E+00_JPRB, 0.47236E+00_JPRB, 0.47560E+00_JPRB, 0.49044E+00_JPRB, 0.51433E+00_JPRB, 0.52166E+00_JPRB ,&
&  0.53252E+00_JPRB, 0.54717E+00_JPRB, 0.56604E+00_JPRB, 0.58676E+00_JPRB, 0.58420E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.59653E+00_JPRB, 0.59726E+00_JPRB, 0.59826E+00_JPRB, 0.60485E+00_JPRB, 0.60510E+00_JPRB, 0.60005E+00_JPRB, 0.58646E+00_JPRB ,&
&  0.57711E+00_JPRB, 0.56135E+00_JPRB, 0.55138E+00_JPRB, 0.54468E+00_JPRB, 0.55616E+00_JPRB, 0.58568E+00_JPRB, 0.63152E+00_JPRB ,&
&  0.64469E+00_JPRB, 0.64255E+00_JPRB, 0.61860E+00_JPRB, 0.60557E+00_JPRB, 0.62815E+00_JPRB, 0.63194E+00_JPRB, 0.60885E+00_JPRB ,&
&  0.59030E+00_JPRB, 0.64864E+00_JPRB, 0.80535E+00_JPRB, 0.10287E+01_JPRB, 0.14194E+01_JPRB, 0.16139E+01_JPRB, 0.20174E+01_JPRB ,&
&  0.18097E+01_JPRB, 0.13123E+01_JPRB, 0.13628E+01_JPRB, 0.12907E+01_JPRB, 0.13677E+01_JPRB, 0.14118E+01_JPRB, 0.14260E+01_JPRB ,&
&  0.14615E+01_JPRB, 0.14866E+01_JPRB, 0.15102E+01_JPRB, 0.15422E+01_JPRB, 0.15556E+01_JPRB, 0.15357E+01_JPRB, 0.15142E+01_JPRB ,&
&  0.14918E+01_JPRB, 0.14471E+01_JPRB, 0.14590E+01_JPRB, 0.15131E+01_JPRB, 0.16580E+01_JPRB, 0.18169E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.18524E+01_JPRB, 0.19341E+01_JPRB, 0.18459E+01_JPRB, 0.19369E+01_JPRB, 0.19527E+01_JPRB, 0.18854E+01_JPRB, 0.20097E+01_JPRB ,&
&  0.20622E+01_JPRB, 0.17184E+01_JPRB, 0.11132E+01_JPRB, 0.11752E+01_JPRB, 0.11222E+01_JPRB, 0.10718E+01_JPRB, 0.11035E+01_JPRB ,&
&  0.10289E+01_JPRB, 0.96551E+00_JPRB, 0.91449E+00_JPRB, 0.88019E+00_JPRB, 0.83298E+00_JPRB, 0.76140E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.65875E+00_JPRB, 0.61588E+00_JPRB, 0.57972E+00_JPRB, 0.53369E+00_JPRB, 0.48012E+00_JPRB, 0.45678E+00_JPRB, 0.47117E+00_JPRB ,&
&  0.50531E+00_JPRB, 0.50556E+00_JPRB, 0.53418E+00_JPRB, 0.55090E+00_JPRB, 0.51443E+00_JPRB, 0.56127E+00_JPRB, 0.54728E+00_JPRB ,&
&  0.54460E+00_JPRB, 0.53921E+00_JPRB, 0.53714E+00_JPRB, 0.53206E+00_JPRB, 0.55432E+00_JPRB, 0.58768E+00_JPRB, 0.59625E+00_JPRB ,&
&  0.61104E+00_JPRB, 0.59849E+00_JPRB, 0.60350E+00_JPRB, 0.61156E+00_JPRB, 0.61564E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.70772E+00_JPRB, 0.70644E+00_JPRB, 0.69459E+00_JPRB, 0.70418E+00_JPRB, 0.71525E+00_JPRB, 0.71083E+00_JPRB, 0.72384E+00_JPRB ,&
&  0.74233E+00_JPRB, 0.76023E+00_JPRB, 0.77175E+00_JPRB, 0.77964E+00_JPRB, 0.82676E+00_JPRB, 0.88498E+00_JPRB, 0.95313E+00_JPRB ,&
&  0.99190E+00_JPRB, 0.10287E+01_JPRB, 0.10384E+01_JPRB, 0.10259E+01_JPRB, 0.10203E+01_JPRB, 0.98558E+00_JPRB, 0.94169E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.12767E+01_JPRB, 0.17149E+01_JPRB, 0.21020E+01_JPRB, 0.24268E+01_JPRB, 0.20545E+01_JPRB, 0.21178E+01_JPRB ,&
&  0.18360E+01_JPRB, 0.14558E+01_JPRB, 0.14398E+01_JPRB, 0.15181E+01_JPRB, 0.15993E+01_JPRB, 0.16042E+01_JPRB, 0.16407E+01_JPRB ,&
&  0.16600E+01_JPRB, 0.16766E+01_JPRB, 0.17051E+01_JPRB, 0.17756E+01_JPRB, 0.18302E+01_JPRB, 0.18710E+01_JPRB, 0.19252E+01_JPRB ,&
&  0.19622E+01_JPRB, 0.19555E+01_JPRB, 0.19411E+01_JPRB, 0.17826E+01_JPRB, 0.18334E+01_JPRB, 0.19515E+01_JPRB, 0.20569E+01_JPRB ,&
&  0.21133E+01_JPRB, 0.21105E+01_JPRB, 0.22523E+01_JPRB, 0.22755E+01_JPRB, 0.22466E+01_JPRB, 0.21866E+01_JPRB, 0.21976E+01_JPRB ,&
&  0.21876E+01_JPRB, 0.19695E+01_JPRB, 0.13408E+01_JPRB, 0.14293E+01_JPRB, 0.13204E+01_JPRB, 0.11063E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.98394E+00_JPRB, 0.91155E+00_JPRB, 0.88891E+00_JPRB, 0.87737E+00_JPRB, 0.86159E+00_JPRB, 0.82873E+00_JPRB ,&
&  0.76809E+00_JPRB, 0.67983E+00_JPRB, 0.57566E+00_JPRB, 0.53290E+00_JPRB, 0.50466E+00_JPRB, 0.54223E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.62789E+00_JPRB, 0.60357E+00_JPRB, 0.63157E+00_JPRB, 0.64754E+00_JPRB, 0.60022E+00_JPRB, 0.62444E+00_JPRB, 0.62604E+00_JPRB ,&
&  0.63367E+00_JPRB, 0.64320E+00_JPRB, 0.64964E+00_JPRB, 0.65146E+00_JPRB, 0.66279E+00_JPRB, 0.68294E+00_JPRB, 0.70259E+00_JPRB ,&
&  0.72102E+00_JPRB, 0.71735E+00_JPRB, 0.71325E+00_JPRB, 0.70954E+00_JPRB, 0.70462E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.77019E+00_JPRB, 0.74381E+00_JPRB, 0.72230E+00_JPRB, 0.71250E+00_JPRB, 0.71085E+00_JPRB, 0.72040E+00_JPRB, 0.73950E+00_JPRB ,&
&  0.77366E+00_JPRB, 0.82199E+00_JPRB, 0.87150E+00_JPRB, 0.91417E+00_JPRB, 0.97840E+00_JPRB, 0.10614E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.12806E+01_JPRB, 0.14034E+01_JPRB, 0.14704E+01_JPRB, 0.15006E+01_JPRB, 0.15642E+01_JPRB, 0.16668E+01_JPRB, 0.18986E+01_JPRB ,&
&  0.22199E+01_JPRB, 0.24813E+01_JPRB, 0.26051E+01_JPRB, 0.23483E+01_JPRB, 0.17873E+01_JPRB, 0.17213E+01_JPRB, 0.18884E+01_JPRB ,&
&  0.19234E+01_JPRB, 0.17105E+01_JPRB, 0.15528E+01_JPRB, 0.15445E+01_JPRB, 0.15554E+01_JPRB, 0.16019E+01_JPRB, 0.16293E+01_JPRB ,&
&  0.16429E+01_JPRB, 0.16651E+01_JPRB, 0.17233E+01_JPRB, 0.17836E+01_JPRB, 0.18803E+01_JPRB, 0.19838E+01_JPRB, 0.21012E+01_JPRB ,&
&  0.21378E+01_JPRB, 0.21864E+01_JPRB, 0.21464E+01_JPRB, 0.20549E+01_JPRB, 0.20256E+01_JPRB, 0.20753E+01_JPRB, 0.21665E+01_JPRB ,&
&  0.22582E+01_JPRB, 0.22199E+01_JPRB, 0.22587E+01_JPRB, 0.22876E+01_JPRB, 0.21838E+01_JPRB, 0.19687E+01_JPRB, 0.18878E+01_JPRB ,&
&  0.18970E+01_JPRB, 0.20517E+01_JPRB, 0.14255E+01_JPRB, 0.16776E+01_JPRB, 0.17370E+01_JPRB, 0.15038E+01_JPRB, 0.12742E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.10991E+01_JPRB, 0.10249E+01_JPRB, 0.10468E+01_JPRB, 0.10304E+01_JPRB, 0.10232E+01_JPRB, 0.91001E+00_JPRB ,&
&  0.91278E+00_JPRB, 0.78395E+00_JPRB, 0.62873E+00_JPRB, 0.51774E+00_JPRB, 0.52899E+00_JPRB, 0.59364E+00_JPRB, 0.65198E+00_JPRB ,&
&  0.64919E+00_JPRB, 0.76847E+00_JPRB, 0.90154E+00_JPRB, 0.11208E+01_JPRB, 0.12842E+01_JPRB, 0.64034E+00_JPRB, 0.65245E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.69942E+00_JPRB, 0.71102E+00_JPRB, 0.71898E+00_JPRB, 0.72812E+00_JPRB, 0.75126E+00_JPRB, 0.78671E+00_JPRB ,&
&  0.80862E+00_JPRB, 0.82158E+00_JPRB, 0.83194E+00_JPRB, 0.82078E+00_JPRB, 0.79319E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.81723E+00_JPRB, 0.81220E+00_JPRB, 0.80541E+00_JPRB, 0.78968E+00_JPRB, 0.76353E+00_JPRB, 0.74471E+00_JPRB, 0.74678E+00_JPRB ,&
&  0.75307E+00_JPRB, 0.79058E+00_JPRB, 0.85450E+00_JPRB, 0.93416E+00_JPRB, 0.10385E+01_JPRB, 0.11577E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.14442E+01_JPRB, 0.16097E+01_JPRB, 0.17189E+01_JPRB, 0.18100E+01_JPRB, 0.19170E+01_JPRB, 0.20938E+01_JPRB, 0.22972E+01_JPRB ,&
&  0.24520E+01_JPRB, 0.24360E+01_JPRB, 0.22670E+01_JPRB, 0.16318E+01_JPRB, 0.13442E+01_JPRB, 0.15655E+01_JPRB, 0.16087E+01_JPRB ,&
&  0.15895E+01_JPRB, 0.15363E+01_JPRB, 0.15128E+01_JPRB, 0.14923E+01_JPRB, 0.15025E+01_JPRB, 0.15464E+01_JPRB, 0.15776E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.15960E+01_JPRB, 0.16438E+01_JPRB, 0.17424E+01_JPRB, 0.18832E+01_JPRB, 0.19527E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.21022E+01_JPRB, 0.22138E+01_JPRB, 0.22571E+01_JPRB, 0.22541E+01_JPRB, 0.21845E+01_JPRB, 0.21744E+01_JPRB, 0.22201E+01_JPRB ,&
&  0.22403E+01_JPRB, 0.22063E+01_JPRB, 0.22169E+01_JPRB, 0.22587E+01_JPRB, 0.22412E+01_JPRB, 0.19257E+01_JPRB, 0.17958E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.17631E+01_JPRB, 0.15864E+01_JPRB, 0.19802E+01_JPRB, 0.12975E+01_JPRB, 0.14173E+01_JPRB, 0.14042E+01_JPRB ,&
&  0.13217E+01_JPRB, 0.13015E+01_JPRB, 0.13083E+01_JPRB, 0.13040E+01_JPRB, 0.12604E+01_JPRB, 0.10405E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.98878E+00_JPRB, 0.82111E+00_JPRB, 0.67665E+00_JPRB, 0.50309E+00_JPRB, 0.54067E+00_JPRB, 0.61404E+00_JPRB, 0.64394E+00_JPRB ,&
&  0.60813E+00_JPRB, 0.75986E+00_JPRB, 0.92303E+00_JPRB, 0.10187E+01_JPRB, 0.10526E+01_JPRB, 0.64052E+00_JPRB, 0.64242E+00_JPRB ,&
&  0.66862E+00_JPRB, 0.68810E+00_JPRB, 0.70831E+00_JPRB, 0.71058E+00_JPRB, 0.73159E+00_JPRB, 0.76706E+00_JPRB, 0.80065E+00_JPRB ,&
&  0.81609E+00_JPRB, 0.81608E+00_JPRB, 0.82779E+00_JPRB, 0.82696E+00_JPRB, 0.82261E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.81758E+00_JPRB, 0.82615E+00_JPRB, 0.85369E+00_JPRB, 0.86977E+00_JPRB, 0.87840E+00_JPRB, 0.83470E+00_JPRB, 0.80004E+00_JPRB ,&
&  0.75493E+00_JPRB, 0.70748E+00_JPRB, 0.76769E+00_JPRB, 0.84240E+00_JPRB, 0.93134E+00_JPRB, 0.10271E+01_JPRB, 0.11290E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.14288E+01_JPRB, 0.16337E+01_JPRB, 0.17926E+01_JPRB, 0.19359E+01_JPRB, 0.21244E+01_JPRB, 0.22840E+01_JPRB ,&
&  0.19081E+01_JPRB, 0.13364E+01_JPRB, 0.16763E+01_JPRB, 0.17868E+01_JPRB, 0.15039E+01_JPRB, 0.14135E+01_JPRB, 0.14079E+01_JPRB ,&
&  0.14374E+01_JPRB, 0.14624E+01_JPRB, 0.14791E+01_JPRB, 0.14938E+01_JPRB, 0.14515E+01_JPRB, 0.14838E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.15559E+01_JPRB, 0.15598E+01_JPRB, 0.15713E+01_JPRB, 0.16487E+01_JPRB, 0.17830E+01_JPRB, 0.18166E+01_JPRB, 0.19265E+01_JPRB ,&
&  0.20424E+01_JPRB, 0.21980E+01_JPRB, 0.22296E+01_JPRB, 0.21590E+01_JPRB, 0.21708E+01_JPRB, 0.22130E+01_JPRB, 0.22342E+01_JPRB ,&
&  0.22522E+01_JPRB, 0.22347E+01_JPRB, 0.22575E+01_JPRB, 0.23240E+01_JPRB, 0.23153E+01_JPRB, 0.20207E+01_JPRB, 0.19748E+01_JPRB ,&
&  0.19021E+01_JPRB, 0.17915E+01_JPRB, 0.17176E+01_JPRB, 0.19885E+01_JPRB, 0.15262E+01_JPRB, 0.15368E+01_JPRB, 0.13666E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.14550E+01_JPRB, 0.14370E+01_JPRB, 0.13839E+01_JPRB, 0.12982E+01_JPRB, 0.10892E+01_JPRB, 0.10302E+01_JPRB ,&
&  0.91437E+00_JPRB, 0.86557E+00_JPRB, 0.68368E+00_JPRB, 0.53072E+00_JPRB, 0.46588E+00_JPRB, 0.51478E+00_JPRB, 0.58724E+00_JPRB ,&
&  0.67420E+00_JPRB, 0.68659E+00_JPRB, 0.82965E+00_JPRB, 0.88721E+00_JPRB, 0.88847E+00_JPRB, 0.69309E+00_JPRB, 0.67012E+00_JPRB ,&
&  0.68111E+00_JPRB, 0.68751E+00_JPRB, 0.66374E+00_JPRB, 0.67396E+00_JPRB, 0.70373E+00_JPRB, 0.74767E+00_JPRB, 0.77055E+00_JPRB ,&
&  0.77394E+00_JPRB, 0.76029E+00_JPRB, 0.76193E+00_JPRB, 0.77530E+00_JPRB, 0.80527E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.75274E+00_JPRB, 0.77577E+00_JPRB, 0.80276E+00_JPRB, 0.85072E+00_JPRB, 0.92178E+00_JPRB, 0.10126E+01_JPRB, 0.10316E+01_JPRB ,&
&  0.89207E+00_JPRB, 0.65853E+00_JPRB, 0.67755E+00_JPRB, 0.70574E+00_JPRB, 0.75954E+00_JPRB, 0.81140E+00_JPRB, 0.87281E+00_JPRB ,&
&  0.96706E+00_JPRB, 0.10942E+01_JPRB, 0.13095E+01_JPRB, 0.15605E+01_JPRB, 0.17913E+01_JPRB, 0.20717E+01_JPRB, 0.19882E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.14484E+01_JPRB, 0.16748E+01_JPRB, 0.16231E+01_JPRB, 0.14511E+01_JPRB, 0.14632E+01_JPRB, 0.14202E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.13839E+01_JPRB, 0.14141E+01_JPRB, 0.13865E+01_JPRB, 0.13527E+01_JPRB, 0.13819E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.14954E+01_JPRB, 0.15403E+01_JPRB, 0.15689E+01_JPRB, 0.16001E+01_JPRB, 0.16619E+01_JPRB, 0.16908E+01_JPRB, 0.17964E+01_JPRB ,&
&  0.19485E+01_JPRB, 0.20863E+01_JPRB, 0.21221E+01_JPRB, 0.20689E+01_JPRB, 0.21194E+01_JPRB, 0.22346E+01_JPRB, 0.23216E+01_JPRB ,&
&  0.23753E+01_JPRB, 0.24682E+01_JPRB, 0.25038E+01_JPRB, 0.24399E+01_JPRB, 0.22129E+01_JPRB, 0.23336E+01_JPRB, 0.20943E+01_JPRB ,&
&  0.20564E+01_JPRB, 0.18940E+01_JPRB, 0.20176E+01_JPRB, 0.16426E+01_JPRB, 0.17592E+01_JPRB, 0.17437E+01_JPRB, 0.16308E+01_JPRB ,&
&  0.15532E+01_JPRB, 0.16520E+01_JPRB, 0.16165E+01_JPRB, 0.14732E+01_JPRB, 0.13935E+01_JPRB, 0.11463E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.86872E+00_JPRB, 0.89958E+00_JPRB, 0.70115E+00_JPRB, 0.51496E+00_JPRB, 0.40388E+00_JPRB, 0.40698E+00_JPRB, 0.47849E+00_JPRB ,&
&  0.64494E+00_JPRB, 0.71700E+00_JPRB, 0.75009E+00_JPRB, 0.77036E+00_JPRB, 0.78115E+00_JPRB, 0.69899E+00_JPRB, 0.70127E+00_JPRB ,&
&  0.71788E+00_JPRB, 0.69938E+00_JPRB, 0.65407E+00_JPRB, 0.65657E+00_JPRB, 0.68223E+00_JPRB, 0.72438E+00_JPRB, 0.72730E+00_JPRB ,&
&  0.73792E+00_JPRB, 0.73267E+00_JPRB, 0.72085E+00_JPRB, 0.71511E+00_JPRB, 0.72813E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.69163E+00_JPRB, 0.69189E+00_JPRB, 0.67873E+00_JPRB, 0.66888E+00_JPRB, 0.66709E+00_JPRB, 0.67342E+00_JPRB, 0.69580E+00_JPRB ,&
&  0.64252E+00_JPRB, 0.63564E+00_JPRB, 0.64208E+00_JPRB, 0.63903E+00_JPRB, 0.64840E+00_JPRB, 0.65992E+00_JPRB, 0.68524E+00_JPRB ,&
&  0.71752E+00_JPRB, 0.75183E+00_JPRB, 0.88167E+00_JPRB, 0.11395E+01_JPRB, 0.14120E+01_JPRB, 0.17306E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.13262E+01_JPRB, 0.20419E+01_JPRB, 0.15228E+01_JPRB, 0.15751E+01_JPRB, 0.15535E+01_JPRB, 0.15967E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.13245E+01_JPRB, 0.13194E+01_JPRB, 0.13607E+01_JPRB, 0.13427E+01_JPRB, 0.13479E+01_JPRB, 0.13579E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.14373E+01_JPRB, 0.14870E+01_JPRB, 0.15201E+01_JPRB, 0.15521E+01_JPRB, 0.15966E+01_JPRB, 0.16211E+01_JPRB, 0.16671E+01_JPRB ,&
&  0.17849E+01_JPRB, 0.19445E+01_JPRB, 0.20017E+01_JPRB, 0.20659E+01_JPRB, 0.21493E+01_JPRB, 0.23514E+01_JPRB, 0.24910E+01_JPRB ,&
&  0.24664E+01_JPRB, 0.24401E+01_JPRB, 0.26641E+01_JPRB, 0.26873E+01_JPRB, 0.26130E+01_JPRB, 0.26562E+01_JPRB, 0.22828E+01_JPRB ,&
&  0.22746E+01_JPRB, 0.20273E+01_JPRB, 0.21383E+01_JPRB, 0.17860E+01_JPRB, 0.18457E+01_JPRB, 0.19883E+01_JPRB, 0.17977E+01_JPRB ,&
&  0.16636E+01_JPRB, 0.17924E+01_JPRB, 0.17419E+01_JPRB, 0.16129E+01_JPRB, 0.14361E+01_JPRB, 0.12787E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.97425E+00_JPRB, 0.90720E+00_JPRB, 0.83942E+00_JPRB, 0.56875E+00_JPRB, 0.46448E+00_JPRB, 0.43867E+00_JPRB, 0.46627E+00_JPRB ,&
&  0.60564E+00_JPRB, 0.69391E+00_JPRB, 0.84283E+00_JPRB, 0.79750E+00_JPRB, 0.75931E+00_JPRB, 0.72152E+00_JPRB, 0.73033E+00_JPRB ,&
&  0.71851E+00_JPRB, 0.71971E+00_JPRB, 0.68537E+00_JPRB, 0.67197E+00_JPRB, 0.69520E+00_JPRB, 0.72500E+00_JPRB, 0.72421E+00_JPRB ,&
&  0.73948E+00_JPRB, 0.74619E+00_JPRB, 0.73413E+00_JPRB, 0.71134E+00_JPRB, 0.69861E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.70859E+00_JPRB, 0.69277E+00_JPRB, 0.68542E+00_JPRB, 0.66862E+00_JPRB, 0.65511E+00_JPRB, 0.63991E+00_JPRB, 0.64494E+00_JPRB ,&
&  0.62658E+00_JPRB, 0.61591E+00_JPRB, 0.61916E+00_JPRB, 0.60962E+00_JPRB, 0.61062E+00_JPRB, 0.59629E+00_JPRB, 0.60462E+00_JPRB ,&
&  0.61189E+00_JPRB, 0.58582E+00_JPRB, 0.62590E+00_JPRB, 0.78241E+00_JPRB, 0.10125E+01_JPRB, 0.13997E+01_JPRB, 0.10893E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.19609E+01_JPRB, 0.16394E+01_JPRB, 0.16519E+01_JPRB, 0.17177E+01_JPRB, 0.17332E+01_JPRB, 0.14278E+01_JPRB ,&
&  0.12901E+01_JPRB, 0.12568E+01_JPRB, 0.12833E+01_JPRB, 0.13363E+01_JPRB, 0.14090E+01_JPRB, 0.14773E+01_JPRB, 0.14736E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.15194E+01_JPRB, 0.15351E+01_JPRB, 0.15346E+01_JPRB, 0.15516E+01_JPRB, 0.15774E+01_JPRB, 0.15958E+01_JPRB ,&
&  0.16427E+01_JPRB, 0.17539E+01_JPRB, 0.19507E+01_JPRB, 0.19710E+01_JPRB, 0.21289E+01_JPRB, 0.24297E+01_JPRB, 0.26029E+01_JPRB ,&
&  0.24889E+01_JPRB, 0.24277E+01_JPRB, 0.28007E+01_JPRB, 0.31362E+01_JPRB, 0.31906E+01_JPRB, 0.30423E+01_JPRB, 0.26241E+01_JPRB ,&
&  0.24520E+01_JPRB, 0.22307E+01_JPRB, 0.21071E+01_JPRB, 0.17890E+01_JPRB, 0.18383E+01_JPRB, 0.21333E+01_JPRB, 0.21749E+01_JPRB ,&
&  0.22404E+01_JPRB, 0.20533E+01_JPRB, 0.19474E+01_JPRB, 0.16968E+01_JPRB, 0.14675E+01_JPRB, 0.13169E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.12126E+01_JPRB, 0.11168E+01_JPRB, 0.73156E+00_JPRB, 0.56275E+00_JPRB, 0.44785E+00_JPRB, 0.48634E+00_JPRB ,&
&  0.63488E+00_JPRB, 0.69097E+00_JPRB, 0.71216E+00_JPRB, 0.94980E+00_JPRB, 0.87035E+00_JPRB, 0.81684E+00_JPRB, 0.80603E+00_JPRB ,&
&  0.77435E+00_JPRB, 0.79415E+00_JPRB, 0.78201E+00_JPRB, 0.74858E+00_JPRB, 0.75091E+00_JPRB, 0.76120E+00_JPRB, 0.77151E+00_JPRB ,&
&  0.76951E+00_JPRB, 0.77570E+00_JPRB, 0.78245E+00_JPRB, 0.76720E+00_JPRB, 0.73622E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.72298E+00_JPRB, 0.71031E+00_JPRB, 0.68426E+00_JPRB, 0.66508E+00_JPRB, 0.65852E+00_JPRB, 0.63590E+00_JPRB, 0.61715E+00_JPRB ,&
&  0.61613E+00_JPRB, 0.60181E+00_JPRB, 0.58857E+00_JPRB, 0.58536E+00_JPRB, 0.58451E+00_JPRB, 0.57804E+00_JPRB, 0.57735E+00_JPRB ,&
&  0.56979E+00_JPRB, 0.54010E+00_JPRB, 0.57904E+00_JPRB, 0.65339E+00_JPRB, 0.80076E+00_JPRB, 0.88787E+00_JPRB, 0.10098E+01_JPRB ,&
&  0.16584E+01_JPRB, 0.20844E+01_JPRB, 0.18105E+01_JPRB, 0.19549E+01_JPRB, 0.21986E+01_JPRB, 0.24752E+01_JPRB, 0.18156E+01_JPRB ,&
&  0.15424E+01_JPRB, 0.13887E+01_JPRB, 0.12855E+01_JPRB, 0.13525E+01_JPRB, 0.14811E+01_JPRB, 0.15904E+01_JPRB, 0.16195E+01_JPRB ,&
&  0.15625E+01_JPRB, 0.15264E+01_JPRB, 0.15299E+01_JPRB, 0.15248E+01_JPRB, 0.15190E+01_JPRB, 0.15571E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.15870E+01_JPRB, 0.16470E+01_JPRB, 0.18095E+01_JPRB, 0.18838E+01_JPRB, 0.20721E+01_JPRB, 0.23668E+01_JPRB, 0.27086E+01_JPRB ,&
&  0.27536E+01_JPRB, 0.28772E+01_JPRB, 0.31125E+01_JPRB, 0.33979E+01_JPRB, 0.36084E+01_JPRB, 0.36003E+01_JPRB, 0.31021E+01_JPRB ,&
&  0.27628E+01_JPRB, 0.23391E+01_JPRB, 0.19822E+01_JPRB, 0.18579E+01_JPRB, 0.18907E+01_JPRB, 0.22757E+01_JPRB, 0.23763E+01_JPRB ,&
&  0.21867E+01_JPRB, 0.16976E+01_JPRB, 0.15896E+01_JPRB, 0.16626E+01_JPRB, 0.15668E+01_JPRB, 0.14216E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.13538E+01_JPRB, 0.83305E+00_JPRB, 0.71492E+00_JPRB, 0.79266E+00_JPRB, 0.55892E+00_JPRB, 0.42509E+00_JPRB, 0.56573E+00_JPRB ,&
&  0.68140E+00_JPRB, 0.78555E+00_JPRB, 0.78598E+00_JPRB, 0.79420E+00_JPRB, 0.93573E+00_JPRB, 0.89037E+00_JPRB, 0.92474E+00_JPRB ,&
&  0.83603E+00_JPRB, 0.84319E+00_JPRB, 0.84691E+00_JPRB, 0.80172E+00_JPRB, 0.78124E+00_JPRB, 0.80540E+00_JPRB, 0.82191E+00_JPRB ,&
&  0.81975E+00_JPRB, 0.81473E+00_JPRB, 0.79657E+00_JPRB, 0.77477E+00_JPRB, 0.74486E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.75645E+00_JPRB, 0.74103E+00_JPRB, 0.72785E+00_JPRB, 0.69677E+00_JPRB, 0.67446E+00_JPRB, 0.64940E+00_JPRB, 0.61724E+00_JPRB ,&
&  0.59935E+00_JPRB, 0.58938E+00_JPRB, 0.58341E+00_JPRB, 0.57327E+00_JPRB, 0.56861E+00_JPRB, 0.56774E+00_JPRB, 0.57583E+00_JPRB ,&
&  0.55966E+00_JPRB, 0.54173E+00_JPRB, 0.63712E+00_JPRB, 0.71358E+00_JPRB, 0.76213E+00_JPRB, 0.78633E+00_JPRB, 0.10550E+01_JPRB ,&
&  0.16878E+01_JPRB, 0.22704E+01_JPRB, 0.26756E+01_JPRB, 0.28361E+01_JPRB, 0.30314E+01_JPRB, 0.32933E+01_JPRB, 0.28115E+01_JPRB ,&
&  0.22563E+01_JPRB, 0.18212E+01_JPRB, 0.15193E+01_JPRB, 0.15031E+01_JPRB, 0.16261E+01_JPRB, 0.17299E+01_JPRB, 0.17139E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.15677E+01_JPRB, 0.15119E+01_JPRB, 0.14732E+01_JPRB, 0.14834E+01_JPRB, 0.15309E+01_JPRB, 0.15606E+01_JPRB ,&
&  0.15653E+01_JPRB, 0.15594E+01_JPRB, 0.16626E+01_JPRB, 0.18809E+01_JPRB, 0.17992E+01_JPRB, 0.22823E+01_JPRB, 0.26907E+01_JPRB ,&
&  0.29920E+01_JPRB, 0.31266E+01_JPRB, 0.31427E+01_JPRB, 0.37791E+01_JPRB, 0.41721E+01_JPRB, 0.40390E+01_JPRB, 0.35213E+01_JPRB ,&
&  0.32315E+01_JPRB, 0.26945E+01_JPRB, 0.22636E+01_JPRB, 0.21063E+01_JPRB, 0.20881E+01_JPRB, 0.23311E+01_JPRB, 0.19127E+01_JPRB ,&
&  0.18867E+01_JPRB, 0.21333E+01_JPRB, 0.18543E+01_JPRB, 0.15924E+01_JPRB, 0.16861E+01_JPRB, 0.15771E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.61869E+00_JPRB, 0.50545E+00_JPRB, 0.49922E+00_JPRB, 0.48576E+00_JPRB, 0.43673E+00_JPRB, 0.43939E+00_JPRB, 0.81981E+00_JPRB ,&
&  0.76068E+00_JPRB, 0.90110E+00_JPRB, 0.87072E+00_JPRB, 0.85932E+00_JPRB, 0.93611E+00_JPRB, 0.93150E+00_JPRB, 0.11508E+01_JPRB ,&
&  0.98587E+00_JPRB, 0.93303E+00_JPRB, 0.90475E+00_JPRB, 0.87053E+00_JPRB, 0.85946E+00_JPRB, 0.84686E+00_JPRB, 0.85965E+00_JPRB ,&
&  0.87306E+00_JPRB, 0.87856E+00_JPRB, 0.85662E+00_JPRB, 0.81860E+00_JPRB, 0.78489E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.80085E+00_JPRB, 0.76643E+00_JPRB, 0.74236E+00_JPRB, 0.72948E+00_JPRB, 0.71250E+00_JPRB, 0.67638E+00_JPRB, 0.62645E+00_JPRB ,&
&  0.59517E+00_JPRB, 0.57968E+00_JPRB, 0.57297E+00_JPRB, 0.56896E+00_JPRB, 0.57213E+00_JPRB, 0.57549E+00_JPRB, 0.57542E+00_JPRB ,&
&  0.55103E+00_JPRB, 0.56350E+00_JPRB, 0.71247E+00_JPRB, 0.76789E+00_JPRB, 0.80276E+00_JPRB, 0.80516E+00_JPRB, 0.10458E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.23073E+01_JPRB, 0.27632E+01_JPRB, 0.28837E+01_JPRB, 0.38122E+01_JPRB, 0.43578E+01_JPRB, 0.43236E+01_JPRB ,&
&  0.35298E+01_JPRB, 0.28046E+01_JPRB, 0.21918E+01_JPRB, 0.18770E+01_JPRB, 0.18701E+01_JPRB, 0.18996E+01_JPRB, 0.18727E+01_JPRB ,&
&  0.17654E+01_JPRB, 0.16662E+01_JPRB, 0.15762E+01_JPRB, 0.15169E+01_JPRB, 0.15061E+01_JPRB, 0.15367E+01_JPRB, 0.15863E+01_JPRB ,&
&  0.15911E+01_JPRB, 0.15344E+01_JPRB, 0.15282E+01_JPRB, 0.17478E+01_JPRB, 0.19269E+01_JPRB, 0.20596E+01_JPRB, 0.25367E+01_JPRB ,&
&  0.30454E+01_JPRB, 0.34336E+01_JPRB, 0.34308E+01_JPRB, 0.43932E+01_JPRB, 0.46196E+01_JPRB, 0.43598E+01_JPRB, 0.40997E+01_JPRB ,&
&  0.37302E+01_JPRB, 0.34919E+01_JPRB, 0.25924E+01_JPRB, 0.23291E+01_JPRB, 0.23546E+01_JPRB, 0.20762E+01_JPRB, 0.22397E+01_JPRB ,&
&  0.25231E+01_JPRB, 0.21471E+01_JPRB, 0.17781E+01_JPRB, 0.14910E+01_JPRB, 0.17574E+01_JPRB, 0.14587E+01_JPRB, 0.72359E+00_JPRB ,&
&  0.58485E+00_JPRB, 0.54913E+00_JPRB, 0.48775E+00_JPRB, 0.46977E+00_JPRB, 0.44620E+00_JPRB, 0.47359E+00_JPRB, 0.72751E+00_JPRB ,&
&  0.80907E+00_JPRB, 0.93253E+00_JPRB, 0.92086E+00_JPRB, 0.91719E+00_JPRB, 0.96235E+00_JPRB, 0.10574E+01_JPRB, 0.15908E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.11545E+01_JPRB, 0.10289E+01_JPRB, 0.96520E+00_JPRB, 0.92087E+00_JPRB, 0.89219E+00_JPRB, 0.89374E+00_JPRB ,&
&  0.89445E+00_JPRB, 0.89520E+00_JPRB, 0.87692E+00_JPRB, 0.85082E+00_JPRB, 0.82500E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.82762E+00_JPRB, 0.79796E+00_JPRB, 0.76078E+00_JPRB, 0.72985E+00_JPRB, 0.71127E+00_JPRB, 0.69128E+00_JPRB, 0.65828E+00_JPRB ,&
&  0.61915E+00_JPRB, 0.59798E+00_JPRB, 0.58823E+00_JPRB, 0.57589E+00_JPRB, 0.57497E+00_JPRB, 0.58089E+00_JPRB, 0.57900E+00_JPRB ,&
&  0.56010E+00_JPRB, 0.59962E+00_JPRB, 0.69697E+00_JPRB, 0.69618E+00_JPRB, 0.70725E+00_JPRB, 0.72111E+00_JPRB, 0.91060E+00_JPRB ,&
&  0.15299E+01_JPRB, 0.22154E+01_JPRB, 0.26532E+01_JPRB, 0.31247E+01_JPRB, 0.41522E+01_JPRB, 0.43194E+01_JPRB, 0.49410E+01_JPRB ,&
&  0.44258E+01_JPRB, 0.37371E+01_JPRB, 0.31251E+01_JPRB, 0.27054E+01_JPRB, 0.24347E+01_JPRB, 0.22220E+01_JPRB, 0.20994E+01_JPRB ,&
&  0.19649E+01_JPRB, 0.18041E+01_JPRB, 0.17330E+01_JPRB, 0.16584E+01_JPRB, 0.16212E+01_JPRB, 0.16079E+01_JPRB, 0.16691E+01_JPRB ,&
&  0.16660E+01_JPRB, 0.15953E+01_JPRB, 0.14779E+01_JPRB, 0.15719E+01_JPRB, 0.20043E+01_JPRB, 0.23396E+01_JPRB, 0.24738E+01_JPRB ,&
&  0.26405E+01_JPRB, 0.28429E+01_JPRB, 0.36033E+01_JPRB, 0.43118E+01_JPRB, 0.48269E+01_JPRB, 0.49257E+01_JPRB, 0.44049E+01_JPRB ,&
&  0.39953E+01_JPRB, 0.36620E+01_JPRB, 0.28055E+01_JPRB, 0.25333E+01_JPRB, 0.22422E+01_JPRB, 0.20239E+01_JPRB, 0.25587E+01_JPRB ,&
&  0.27249E+01_JPRB, 0.24076E+01_JPRB, 0.19042E+01_JPRB, 0.15028E+01_JPRB, 0.14154E+01_JPRB, 0.99491E+00_JPRB, 0.68311E+00_JPRB ,&
&  0.64296E+00_JPRB, 0.59605E+00_JPRB, 0.54242E+00_JPRB, 0.56999E+00_JPRB, 0.54140E+00_JPRB, 0.59503E+00_JPRB, 0.80749E+00_JPRB ,&
&  0.96231E+00_JPRB, 0.10137E+01_JPRB, 0.10484E+01_JPRB, 0.10020E+01_JPRB, 0.98041E+00_JPRB, 0.93336E+00_JPRB, 0.14534E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.10775E+01_JPRB, 0.10957E+01_JPRB, 0.10616E+01_JPRB, 0.10357E+01_JPRB, 0.98667E+00_JPRB, 0.94154E+00_JPRB ,&
&  0.91774E+00_JPRB, 0.89439E+00_JPRB, 0.87308E+00_JPRB, 0.85944E+00_JPRB, 0.84565E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.83078E+00_JPRB, 0.79296E+00_JPRB, 0.75866E+00_JPRB, 0.72559E+00_JPRB, 0.69827E+00_JPRB, 0.67823E+00_JPRB, 0.66313E+00_JPRB ,&
&  0.64128E+00_JPRB, 0.62365E+00_JPRB, 0.60291E+00_JPRB, 0.58366E+00_JPRB, 0.57471E+00_JPRB, 0.57359E+00_JPRB, 0.56568E+00_JPRB ,&
&  0.55251E+00_JPRB, 0.63711E+00_JPRB, 0.51506E+00_JPRB, 0.49570E+00_JPRB, 0.59150E+00_JPRB, 0.67200E+00_JPRB, 0.75774E+00_JPRB ,&
&  0.12359E+01_JPRB, 0.19583E+01_JPRB, 0.24395E+01_JPRB, 0.33779E+01_JPRB, 0.52969E+01_JPRB, 0.50572E+01_JPRB, 0.51335E+01_JPRB ,&
&  0.51450E+01_JPRB, 0.42958E+01_JPRB, 0.37023E+01_JPRB, 0.31788E+01_JPRB, 0.27822E+01_JPRB, 0.24654E+01_JPRB, 0.21819E+01_JPRB ,&
&  0.19487E+01_JPRB, 0.17903E+01_JPRB, 0.17072E+01_JPRB, 0.16978E+01_JPRB, 0.17075E+01_JPRB, 0.17315E+01_JPRB, 0.17520E+01_JPRB ,&
&  0.17428E+01_JPRB, 0.16868E+01_JPRB, 0.15488E+01_JPRB, 0.15600E+01_JPRB, 0.17689E+01_JPRB, 0.21128E+01_JPRB, 0.29177E+01_JPRB ,&
&  0.29385E+01_JPRB, 0.31609E+01_JPRB, 0.31990E+01_JPRB, 0.44759E+01_JPRB, 0.51624E+01_JPRB, 0.48689E+01_JPRB, 0.46571E+01_JPRB ,&
&  0.34908E+01_JPRB, 0.28892E+01_JPRB, 0.25708E+01_JPRB, 0.23079E+01_JPRB, 0.21498E+01_JPRB, 0.25726E+01_JPRB, 0.36891E+01_JPRB ,&
&  0.28766E+01_JPRB, 0.29829E+01_JPRB, 0.29757E+01_JPRB, 0.26851E+01_JPRB, 0.14713E+01_JPRB, 0.10806E+01_JPRB, 0.16035E+01_JPRB ,&
&  0.16563E+01_JPRB, 0.10154E+01_JPRB, 0.75708E+00_JPRB, 0.93271E+00_JPRB, 0.71482E+00_JPRB, 0.75753E+00_JPRB, 0.91826E+00_JPRB ,&
&  0.95489E+00_JPRB, 0.99280E+00_JPRB, 0.11680E+01_JPRB, 0.11258E+01_JPRB, 0.10546E+01_JPRB, 0.93321E+00_JPRB, 0.11718E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.10044E+01_JPRB, 0.10410E+01_JPRB, 0.10023E+01_JPRB, 0.98848E+00_JPRB, 0.96456E+00_JPRB, 0.92738E+00_JPRB ,&
&  0.90329E+00_JPRB, 0.88183E+00_JPRB, 0.86725E+00_JPRB, 0.85815E+00_JPRB, 0.85300E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.82801E+00_JPRB, 0.78879E+00_JPRB, 0.74679E+00_JPRB, 0.71691E+00_JPRB, 0.69184E+00_JPRB, 0.66567E+00_JPRB, 0.64339E+00_JPRB ,&
&  0.63028E+00_JPRB, 0.61728E+00_JPRB, 0.59298E+00_JPRB, 0.57589E+00_JPRB, 0.56516E+00_JPRB, 0.56788E+00_JPRB, 0.56575E+00_JPRB ,&
&  0.55555E+00_JPRB, 0.52885E+00_JPRB, 0.44490E+00_JPRB, 0.42943E+00_JPRB, 0.55398E+00_JPRB, 0.58258E+00_JPRB, 0.65322E+00_JPRB ,&
&  0.97491E+00_JPRB, 0.16516E+01_JPRB, 0.24052E+01_JPRB, 0.33878E+01_JPRB, 0.47757E+01_JPRB, 0.55652E+01_JPRB, 0.54099E+01_JPRB ,&
&  0.54856E+01_JPRB, 0.46808E+01_JPRB, 0.40316E+01_JPRB, 0.34921E+01_JPRB, 0.29723E+01_JPRB, 0.24754E+01_JPRB, 0.20781E+01_JPRB ,&
&  0.18493E+01_JPRB, 0.16933E+01_JPRB, 0.16045E+01_JPRB, 0.15263E+01_JPRB, 0.15502E+01_JPRB, 0.16228E+01_JPRB, 0.17027E+01_JPRB ,&
&  0.17269E+01_JPRB, 0.17083E+01_JPRB, 0.16409E+01_JPRB, 0.15749E+01_JPRB, 0.17711E+01_JPRB, 0.21337E+01_JPRB, 0.25037E+01_JPRB ,&
&  0.28487E+01_JPRB, 0.30236E+01_JPRB, 0.32526E+01_JPRB, 0.38301E+01_JPRB, 0.46415E+01_JPRB, 0.44342E+01_JPRB, 0.46146E+01_JPRB ,&
&  0.36455E+01_JPRB, 0.30145E+01_JPRB, 0.24864E+01_JPRB, 0.19108E+01_JPRB, 0.22954E+01_JPRB, 0.36656E+01_JPRB, 0.36934E+01_JPRB ,&
&  0.34523E+01_JPRB, 0.34248E+01_JPRB, 0.31925E+01_JPRB, 0.25849E+01_JPRB, 0.22306E+01_JPRB, 0.17267E+01_JPRB, 0.20307E+01_JPRB ,&
&  0.19575E+01_JPRB, 0.18415E+01_JPRB, 0.16436E+01_JPRB, 0.13605E+01_JPRB, 0.10971E+01_JPRB, 0.11522E+01_JPRB, 0.10093E+01_JPRB ,&
&  0.91303E+00_JPRB, 0.90744E+00_JPRB, 0.11202E+01_JPRB, 0.11561E+01_JPRB, 0.10840E+01_JPRB, 0.89871E+00_JPRB, 0.11016E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.10010E+01_JPRB, 0.93722E+00_JPRB, 0.90304E+00_JPRB, 0.90416E+00_JPRB, 0.90522E+00_JPRB, 0.90050E+00_JPRB ,&
&  0.87330E+00_JPRB, 0.85791E+00_JPRB, 0.85175E+00_JPRB, 0.84910E+00_JPRB, 0.84400E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.81371E+00_JPRB, 0.77878E+00_JPRB, 0.74384E+00_JPRB, 0.71709E+00_JPRB, 0.69107E+00_JPRB, 0.66555E+00_JPRB, 0.63804E+00_JPRB ,&
&  0.61830E+00_JPRB, 0.60674E+00_JPRB, 0.59020E+00_JPRB, 0.57675E+00_JPRB, 0.57131E+00_JPRB, 0.57246E+00_JPRB, 0.57772E+00_JPRB ,&
&  0.57667E+00_JPRB, 0.54613E+00_JPRB, 0.43203E+00_JPRB, 0.42960E+00_JPRB, 0.47851E+00_JPRB, 0.48941E+00_JPRB, 0.59869E+00_JPRB ,&
&  0.82565E+00_JPRB, 0.13205E+01_JPRB, 0.19599E+01_JPRB, 0.28251E+01_JPRB, 0.37958E+01_JPRB, 0.45584E+01_JPRB, 0.47099E+01_JPRB ,&
&  0.41514E+01_JPRB, 0.40635E+01_JPRB, 0.35132E+01_JPRB, 0.30152E+01_JPRB, 0.25067E+01_JPRB, 0.21296E+01_JPRB, 0.18639E+01_JPRB ,&
&  0.16389E+01_JPRB, 0.14928E+01_JPRB, 0.14220E+01_JPRB, 0.14001E+01_JPRB, 0.14246E+01_JPRB, 0.14821E+01_JPRB, 0.15629E+01_JPRB ,&
&  0.15928E+01_JPRB, 0.15999E+01_JPRB, 0.15656E+01_JPRB, 0.16066E+01_JPRB, 0.16250E+01_JPRB, 0.21296E+01_JPRB, 0.23542E+01_JPRB ,&
&  0.30046E+01_JPRB, 0.33826E+01_JPRB, 0.34391E+01_JPRB, 0.43494E+01_JPRB, 0.37368E+01_JPRB, 0.41577E+01_JPRB, 0.45415E+01_JPRB ,&
&  0.42772E+01_JPRB, 0.38951E+01_JPRB, 0.37316E+01_JPRB, 0.30366E+01_JPRB, 0.26709E+01_JPRB, 0.36931E+01_JPRB, 0.34646E+01_JPRB ,&
&  0.31429E+01_JPRB, 0.30665E+01_JPRB, 0.28070E+01_JPRB, 0.26670E+01_JPRB, 0.23892E+01_JPRB, 0.21621E+01_JPRB, 0.16298E+01_JPRB ,&
&  0.15946E+01_JPRB, 0.14500E+01_JPRB, 0.15413E+01_JPRB, 0.13895E+01_JPRB, 0.12778E+01_JPRB, 0.11697E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.92311E+00_JPRB, 0.88209E+00_JPRB, 0.95151E+00_JPRB, 0.98128E+00_JPRB, 0.97992E+00_JPRB, 0.82761E+00_JPRB, 0.91925E+00_JPRB ,&
&  0.97486E+00_JPRB, 0.96748E+00_JPRB, 0.89170E+00_JPRB, 0.91291E+00_JPRB, 0.91739E+00_JPRB, 0.89993E+00_JPRB, 0.88835E+00_JPRB ,&
&  0.86773E+00_JPRB, 0.84807E+00_JPRB, 0.84113E+00_JPRB, 0.84707E+00_JPRB, 0.83782E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.77040E+00_JPRB, 0.74430E+00_JPRB, 0.71721E+00_JPRB, 0.69608E+00_JPRB, 0.68345E+00_JPRB, 0.67118E+00_JPRB, 0.64557E+00_JPRB ,&
&  0.61412E+00_JPRB, 0.59527E+00_JPRB, 0.58458E+00_JPRB, 0.57534E+00_JPRB, 0.57078E+00_JPRB, 0.57497E+00_JPRB, 0.58215E+00_JPRB ,&
&  0.58852E+00_JPRB, 0.69947E+00_JPRB, 0.53644E+00_JPRB, 0.43328E+00_JPRB, 0.39155E+00_JPRB, 0.46343E+00_JPRB, 0.60153E+00_JPRB ,&
&  0.80000E+00_JPRB, 0.11392E+01_JPRB, 0.16975E+01_JPRB, 0.21702E+01_JPRB, 0.26978E+01_JPRB, 0.32196E+01_JPRB, 0.33958E+01_JPRB ,&
&  0.36303E+01_JPRB, 0.28704E+01_JPRB, 0.25826E+01_JPRB, 0.21177E+01_JPRB, 0.18141E+01_JPRB, 0.16150E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.13001E+01_JPRB, 0.12252E+01_JPRB, 0.11774E+01_JPRB, 0.11826E+01_JPRB, 0.11937E+01_JPRB, 0.12281E+01_JPRB, 0.12897E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.13937E+01_JPRB, 0.13900E+01_JPRB, 0.14594E+01_JPRB, 0.17326E+01_JPRB, 0.21119E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.23744E+01_JPRB, 0.24333E+01_JPRB, 0.39524E+01_JPRB, 0.46640E+01_JPRB, 0.49315E+01_JPRB, 0.50192E+01_JPRB, 0.47384E+01_JPRB ,&
&  0.43990E+01_JPRB, 0.40661E+01_JPRB, 0.38698E+01_JPRB, 0.33587E+01_JPRB, 0.37164E+01_JPRB, 0.36888E+01_JPRB, 0.34953E+01_JPRB ,&
&  0.31335E+01_JPRB, 0.28958E+01_JPRB, 0.26581E+01_JPRB, 0.25070E+01_JPRB, 0.24351E+01_JPRB, 0.24558E+01_JPRB, 0.22671E+01_JPRB ,&
&  0.19085E+01_JPRB, 0.19738E+01_JPRB, 0.16895E+01_JPRB, 0.13860E+01_JPRB, 0.11421E+01_JPRB, 0.10596E+01_JPRB, 0.98625E+00_JPRB ,&
&  0.90676E+00_JPRB, 0.83846E+00_JPRB, 0.85021E+00_JPRB, 0.87210E+00_JPRB, 0.86236E+00_JPRB, 0.81543E+00_JPRB, 0.77114E+00_JPRB ,&
&  0.74138E+00_JPRB, 0.86360E+00_JPRB, 0.92498E+00_JPRB, 0.93299E+00_JPRB, 0.95089E+00_JPRB, 0.91295E+00_JPRB, 0.88630E+00_JPRB ,&
&  0.85457E+00_JPRB, 0.83369E+00_JPRB, 0.82176E+00_JPRB, 0.81112E+00_JPRB, 0.79440E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.70409E+00_JPRB, 0.69287E+00_JPRB, 0.68295E+00_JPRB, 0.67041E+00_JPRB, 0.66479E+00_JPRB, 0.65660E+00_JPRB, 0.63912E+00_JPRB ,&
&  0.61932E+00_JPRB, 0.59936E+00_JPRB, 0.58259E+00_JPRB, 0.57169E+00_JPRB, 0.56695E+00_JPRB, 0.56944E+00_JPRB, 0.57000E+00_JPRB ,&
&  0.58670E+00_JPRB, 0.91253E+00_JPRB, 0.67485E+00_JPRB, 0.55886E+00_JPRB, 0.48870E+00_JPRB, 0.56094E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.85248E+00_JPRB, 0.98076E+00_JPRB, 0.12160E+01_JPRB, 0.15800E+01_JPRB, 0.19084E+01_JPRB, 0.21577E+01_JPRB, 0.23312E+01_JPRB ,&
&  0.21558E+01_JPRB, 0.19886E+01_JPRB, 0.15441E+01_JPRB, 0.14421E+01_JPRB, 0.12868E+01_JPRB, 0.10974E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.10741E+01_JPRB, 0.10616E+01_JPRB, 0.10736E+01_JPRB, 0.10912E+01_JPRB, 0.11157E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.11823E+01_JPRB, 0.12193E+01_JPRB, 0.12514E+01_JPRB, 0.13106E+01_JPRB, 0.14974E+01_JPRB, 0.18614E+01_JPRB, 0.21304E+01_JPRB ,&
&  0.25946E+01_JPRB, 0.25717E+01_JPRB, 0.25458E+01_JPRB, 0.37024E+01_JPRB, 0.51233E+01_JPRB, 0.49421E+01_JPRB, 0.42651E+01_JPRB ,&
&  0.40589E+01_JPRB, 0.39573E+01_JPRB, 0.39305E+01_JPRB, 0.37253E+01_JPRB, 0.33703E+01_JPRB, 0.32265E+01_JPRB, 0.30393E+01_JPRB ,&
&  0.27220E+01_JPRB, 0.24856E+01_JPRB, 0.23405E+01_JPRB, 0.22819E+01_JPRB, 0.21621E+01_JPRB, 0.21162E+01_JPRB, 0.21118E+01_JPRB ,&
&  0.20308E+01_JPRB, 0.17264E+01_JPRB, 0.11690E+01_JPRB, 0.11432E+01_JPRB, 0.90577E+00_JPRB, 0.80990E+00_JPRB, 0.82928E+00_JPRB ,&
&  0.74385E+00_JPRB, 0.77896E+00_JPRB, 0.81056E+00_JPRB, 0.77815E+00_JPRB, 0.83910E+00_JPRB, 0.74422E+00_JPRB, 0.67979E+00_JPRB ,&
&  0.64857E+00_JPRB, 0.61669E+00_JPRB, 0.80413E+00_JPRB, 0.87188E+00_JPRB, 0.89109E+00_JPRB, 0.87171E+00_JPRB, 0.83717E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.74760E+00_JPRB, 0.73605E+00_JPRB, 0.73223E+00_JPRB, 0.71827E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.63464E+00_JPRB, 0.63624E+00_JPRB, 0.63172E+00_JPRB, 0.61530E+00_JPRB, 0.60419E+00_JPRB, 0.60352E+00_JPRB, 0.61167E+00_JPRB ,&
&  0.61146E+00_JPRB, 0.59395E+00_JPRB, 0.57988E+00_JPRB, 0.56471E+00_JPRB, 0.55632E+00_JPRB, 0.55360E+00_JPRB, 0.55799E+00_JPRB ,&
&  0.64347E+00_JPRB, 0.93081E+00_JPRB, 0.72864E+00_JPRB, 0.58092E+00_JPRB, 0.70090E+00_JPRB, 0.67456E+00_JPRB, 0.72419E+00_JPRB ,&
&  0.73955E+00_JPRB, 0.88640E+00_JPRB, 0.93351E+00_JPRB, 0.10556E+01_JPRB, 0.12159E+01_JPRB, 0.14439E+01_JPRB, 0.15198E+01_JPRB ,&
&  0.13650E+01_JPRB, 0.10975E+01_JPRB, 0.90090E+00_JPRB, 0.85953E+00_JPRB, 0.90770E+00_JPRB, 0.81574E+00_JPRB, 0.86495E+00_JPRB ,&
&  0.88011E+00_JPRB, 0.90144E+00_JPRB, 0.91956E+00_JPRB, 0.92610E+00_JPRB, 0.95001E+00_JPRB, 0.99015E+00_JPRB, 0.10394E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.11566E+01_JPRB, 0.11880E+01_JPRB, 0.12373E+01_JPRB, 0.14311E+01_JPRB, 0.19665E+01_JPRB, 0.24695E+01_JPRB ,&
&  0.28838E+01_JPRB, 0.31870E+01_JPRB, 0.37276E+01_JPRB, 0.45837E+01_JPRB, 0.48160E+01_JPRB, 0.47610E+01_JPRB, 0.43656E+01_JPRB ,&
&  0.38065E+01_JPRB, 0.33214E+01_JPRB, 0.31244E+01_JPRB, 0.27583E+01_JPRB, 0.25544E+01_JPRB, 0.25083E+01_JPRB, 0.24105E+01_JPRB ,&
&  0.22265E+01_JPRB, 0.20923E+01_JPRB, 0.20409E+01_JPRB, 0.19279E+01_JPRB, 0.18544E+01_JPRB, 0.17610E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.19356E+01_JPRB, 0.12508E+01_JPRB, 0.96175E+00_JPRB, 0.10155E+01_JPRB, 0.83159E+00_JPRB, 0.74236E+00_JPRB, 0.80433E+00_JPRB ,&
&  0.71358E+00_JPRB, 0.71583E+00_JPRB, 0.76239E+00_JPRB, 0.68971E+00_JPRB, 0.68972E+00_JPRB, 0.69792E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.54986E+00_JPRB, 0.54473E+00_JPRB, 0.64087E+00_JPRB, 0.72005E+00_JPRB, 0.75686E+00_JPRB, 0.74079E+00_JPRB, 0.70416E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.63123E+00_JPRB, 0.63259E+00_JPRB, 0.63285E+00_JPRB, 0.63473E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.56987E+00_JPRB, 0.57809E+00_JPRB, 0.58128E+00_JPRB, 0.58133E+00_JPRB, 0.58166E+00_JPRB, 0.58010E+00_JPRB, 0.58748E+00_JPRB ,&
&  0.59344E+00_JPRB, 0.58381E+00_JPRB, 0.56742E+00_JPRB, 0.55320E+00_JPRB, 0.53768E+00_JPRB, 0.53296E+00_JPRB, 0.55840E+00_JPRB ,&
&  0.48962E+00_JPRB, 0.64427E+00_JPRB, 0.53225E+00_JPRB, 0.56931E+00_JPRB, 0.87973E+00_JPRB, 0.76593E+00_JPRB, 0.73602E+00_JPRB ,&
&  0.67832E+00_JPRB, 0.67894E+00_JPRB, 0.67886E+00_JPRB, 0.76453E+00_JPRB, 0.83647E+00_JPRB, 0.94663E+00_JPRB, 0.10071E+01_JPRB ,&
&  0.87065E+00_JPRB, 0.67296E+00_JPRB, 0.58544E+00_JPRB, 0.56615E+00_JPRB, 0.61246E+00_JPRB, 0.64801E+00_JPRB, 0.72094E+00_JPRB ,&
&  0.75614E+00_JPRB, 0.78238E+00_JPRB, 0.80491E+00_JPRB, 0.83946E+00_JPRB, 0.85753E+00_JPRB, 0.88129E+00_JPRB, 0.91919E+00_JPRB ,&
&  0.96701E+00_JPRB, 0.10326E+01_JPRB, 0.10995E+01_JPRB, 0.12159E+01_JPRB, 0.13358E+01_JPRB, 0.21166E+01_JPRB, 0.34457E+01_JPRB ,&
&  0.37188E+01_JPRB, 0.38509E+01_JPRB, 0.43453E+01_JPRB, 0.52788E+01_JPRB, 0.53281E+01_JPRB, 0.49065E+01_JPRB, 0.42073E+01_JPRB ,&
&  0.35505E+01_JPRB, 0.29375E+01_JPRB, 0.27338E+01_JPRB, 0.24878E+01_JPRB, 0.22357E+01_JPRB, 0.22614E+01_JPRB, 0.21664E+01_JPRB ,&
&  0.20204E+01_JPRB, 0.19749E+01_JPRB, 0.19139E+01_JPRB, 0.19072E+01_JPRB, 0.17839E+01_JPRB, 0.17637E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.16767E+01_JPRB, 0.15140E+01_JPRB, 0.11225E+01_JPRB, 0.93815E+00_JPRB, 0.77294E+00_JPRB, 0.72822E+00_JPRB, 0.87197E+00_JPRB ,&
&  0.76933E+00_JPRB, 0.70334E+00_JPRB, 0.65026E+00_JPRB, 0.58588E+00_JPRB, 0.55887E+00_JPRB, 0.57593E+00_JPRB, 0.50754E+00_JPRB ,&
&  0.48002E+00_JPRB, 0.52114E+00_JPRB, 0.55732E+00_JPRB, 0.59930E+00_JPRB, 0.62236E+00_JPRB, 0.62167E+00_JPRB, 0.54229E+00_JPRB ,&
&  0.55611E+00_JPRB, 0.54456E+00_JPRB, 0.54991E+00_JPRB, 0.54823E+00_JPRB, 0.55574E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.52752E+00_JPRB, 0.53757E+00_JPRB, 0.54541E+00_JPRB, 0.55777E+00_JPRB, 0.57278E+00_JPRB, 0.57205E+00_JPRB, 0.57240E+00_JPRB ,&
&  0.57866E+00_JPRB, 0.57200E+00_JPRB, 0.55231E+00_JPRB, 0.53878E+00_JPRB, 0.52317E+00_JPRB, 0.52355E+00_JPRB, 0.46905E+00_JPRB ,&
&  0.43135E+00_JPRB, 0.51467E+00_JPRB, 0.50017E+00_JPRB, 0.55436E+00_JPRB, 0.65376E+00_JPRB, 0.64252E+00_JPRB, 0.64932E+00_JPRB ,&
&  0.63594E+00_JPRB, 0.59302E+00_JPRB, 0.57660E+00_JPRB, 0.59166E+00_JPRB, 0.61148E+00_JPRB, 0.66750E+00_JPRB, 0.73432E+00_JPRB ,&
&  0.66242E+00_JPRB, 0.54204E+00_JPRB, 0.48542E+00_JPRB, 0.48570E+00_JPRB, 0.54416E+00_JPRB, 0.59946E+00_JPRB, 0.63385E+00_JPRB ,&
&  0.66248E+00_JPRB, 0.69364E+00_JPRB, 0.73292E+00_JPRB, 0.78050E+00_JPRB, 0.80355E+00_JPRB, 0.82399E+00_JPRB, 0.84872E+00_JPRB ,&
&  0.87878E+00_JPRB, 0.93048E+00_JPRB, 0.10037E+01_JPRB, 0.11343E+01_JPRB, 0.12834E+01_JPRB, 0.20367E+01_JPRB, 0.30253E+01_JPRB ,&
&  0.33412E+01_JPRB, 0.35908E+01_JPRB, 0.39114E+01_JPRB, 0.42530E+01_JPRB, 0.42727E+01_JPRB, 0.38399E+01_JPRB, 0.33643E+01_JPRB ,&
&  0.30425E+01_JPRB, 0.26076E+01_JPRB, 0.26862E+01_JPRB, 0.24003E+01_JPRB, 0.21733E+01_JPRB, 0.21239E+01_JPRB, 0.19911E+01_JPRB ,&
&  0.19221E+01_JPRB, 0.19394E+01_JPRB, 0.18351E+01_JPRB, 0.17565E+01_JPRB, 0.16009E+01_JPRB, 0.15599E+01_JPRB, 0.14420E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.14163E+01_JPRB, 0.11981E+01_JPRB, 0.10043E+01_JPRB, 0.82825E+00_JPRB, 0.79320E+00_JPRB, 0.72197E+00_JPRB ,&
&  0.66904E+00_JPRB, 0.60265E+00_JPRB, 0.55439E+00_JPRB, 0.49754E+00_JPRB, 0.47167E+00_JPRB, 0.46244E+00_JPRB, 0.45178E+00_JPRB ,&
&  0.48617E+00_JPRB, 0.53795E+00_JPRB, 0.53048E+00_JPRB, 0.52295E+00_JPRB, 0.52735E+00_JPRB, 0.54441E+00_JPRB, 0.44656E+00_JPRB ,&
&  0.45017E+00_JPRB, 0.50667E+00_JPRB, 0.50696E+00_JPRB, 0.50672E+00_JPRB, 0.51119E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.50048E+00_JPRB, 0.51165E+00_JPRB, 0.52335E+00_JPRB, 0.54472E+00_JPRB, 0.56493E+00_JPRB, 0.57226E+00_JPRB, 0.53577E+00_JPRB ,&
&  0.57490E+00_JPRB, 0.55224E+00_JPRB, 0.53375E+00_JPRB, 0.53019E+00_JPRB, 0.52266E+00_JPRB, 0.45283E+00_JPRB, 0.38137E+00_JPRB ,&
&  0.37576E+00_JPRB, 0.40974E+00_JPRB, 0.46314E+00_JPRB, 0.50366E+00_JPRB, 0.54897E+00_JPRB, 0.55757E+00_JPRB, 0.55774E+00_JPRB ,&
&  0.54933E+00_JPRB, 0.52364E+00_JPRB, 0.52086E+00_JPRB, 0.52134E+00_JPRB, 0.51967E+00_JPRB, 0.52617E+00_JPRB, 0.55581E+00_JPRB ,&
&  0.51945E+00_JPRB, 0.48899E+00_JPRB, 0.49076E+00_JPRB, 0.47434E+00_JPRB, 0.53506E+00_JPRB, 0.55763E+00_JPRB, 0.58808E+00_JPRB ,&
&  0.62364E+00_JPRB, 0.66189E+00_JPRB, 0.72549E+00_JPRB, 0.77342E+00_JPRB, 0.80533E+00_JPRB, 0.82492E+00_JPRB, 0.85592E+00_JPRB ,&
&  0.87732E+00_JPRB, 0.91843E+00_JPRB, 0.99705E+00_JPRB, 0.11447E+01_JPRB, 0.13938E+01_JPRB, 0.18737E+01_JPRB, 0.27490E+01_JPRB ,&
&  0.30367E+01_JPRB, 0.33881E+01_JPRB, 0.36721E+01_JPRB, 0.32249E+01_JPRB, 0.33309E+01_JPRB, 0.34117E+01_JPRB, 0.31581E+01_JPRB ,&
&  0.29951E+01_JPRB, 0.25459E+01_JPRB, 0.23422E+01_JPRB, 0.21736E+01_JPRB, 0.20597E+01_JPRB, 0.18329E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.16864E+01_JPRB, 0.18452E+01_JPRB, 0.14658E+01_JPRB, 0.12983E+01_JPRB, 0.11645E+01_JPRB, 0.11196E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.10293E+01_JPRB, 0.97421E+00_JPRB, 0.90403E+00_JPRB, 0.78637E+00_JPRB, 0.73284E+00_JPRB, 0.67652E+00_JPRB ,&
&  0.64664E+00_JPRB, 0.59672E+00_JPRB, 0.52603E+00_JPRB, 0.48429E+00_JPRB, 0.45242E+00_JPRB, 0.42406E+00_JPRB, 0.42263E+00_JPRB ,&
&  0.44952E+00_JPRB, 0.46880E+00_JPRB, 0.52546E+00_JPRB, 0.51726E+00_JPRB, 0.51575E+00_JPRB, 0.53755E+00_JPRB, 0.47584E+00_JPRB ,&
&  0.45358E+00_JPRB, 0.51734E+00_JPRB, 0.50293E+00_JPRB, 0.49184E+00_JPRB, 0.49108E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.48428E+00_JPRB, 0.50358E+00_JPRB, 0.51832E+00_JPRB, 0.53837E+00_JPRB, 0.56242E+00_JPRB, 0.52193E+00_JPRB, 0.49023E+00_JPRB ,&
&  0.55143E+00_JPRB, 0.47489E+00_JPRB, 0.52098E+00_JPRB, 0.43205E+00_JPRB, 0.39145E+00_JPRB, 0.37914E+00_JPRB, 0.37296E+00_JPRB ,&
&  0.37190E+00_JPRB, 0.40244E+00_JPRB, 0.43332E+00_JPRB, 0.45966E+00_JPRB, 0.50842E+00_JPRB, 0.51389E+00_JPRB, 0.50279E+00_JPRB ,&
&  0.48359E+00_JPRB, 0.46359E+00_JPRB, 0.46541E+00_JPRB, 0.46343E+00_JPRB, 0.47241E+00_JPRB, 0.48120E+00_JPRB, 0.48549E+00_JPRB ,&
&  0.45873E+00_JPRB, 0.45890E+00_JPRB, 0.48702E+00_JPRB, 0.50701E+00_JPRB, 0.52056E+00_JPRB, 0.52780E+00_JPRB, 0.54916E+00_JPRB ,&
&  0.57926E+00_JPRB, 0.59326E+00_JPRB, 0.74548E+00_JPRB, 0.78403E+00_JPRB, 0.82692E+00_JPRB, 0.85431E+00_JPRB, 0.88680E+00_JPRB ,&
&  0.92846E+00_JPRB, 0.97961E+00_JPRB, 0.10340E+01_JPRB, 0.11158E+01_JPRB, 0.12855E+01_JPRB, 0.16847E+01_JPRB, 0.21770E+01_JPRB ,&
&  0.29464E+01_JPRB, 0.21439E+01_JPRB, 0.26049E+01_JPRB, 0.25389E+01_JPRB, 0.27801E+01_JPRB, 0.29560E+01_JPRB, 0.28689E+01_JPRB ,&
&  0.29355E+01_JPRB, 0.25090E+01_JPRB, 0.21457E+01_JPRB, 0.17920E+01_JPRB, 0.15285E+01_JPRB, 0.14445E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.12815E+01_JPRB, 0.10971E+01_JPRB, 0.99067E+00_JPRB, 0.94626E+00_JPRB, 0.92056E+00_JPRB, 0.84693E+00_JPRB ,&
&  0.80334E+00_JPRB, 0.76856E+00_JPRB, 0.72195E+00_JPRB, 0.67141E+00_JPRB, 0.63620E+00_JPRB, 0.61574E+00_JPRB, 0.58865E+00_JPRB ,&
&  0.58050E+00_JPRB, 0.56184E+00_JPRB, 0.51078E+00_JPRB, 0.48504E+00_JPRB, 0.46289E+00_JPRB, 0.43763E+00_JPRB, 0.42988E+00_JPRB ,&
&  0.42408E+00_JPRB, 0.41194E+00_JPRB, 0.43193E+00_JPRB, 0.45940E+00_JPRB, 0.46406E+00_JPRB, 0.46838E+00_JPRB, 0.51634E+00_JPRB ,&
&  0.48645E+00_JPRB, 0.50042E+00_JPRB, 0.50324E+00_JPRB, 0.47733E+00_JPRB, 0.47115E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.46874E+00_JPRB, 0.48490E+00_JPRB, 0.50994E+00_JPRB, 0.53078E+00_JPRB, 0.55313E+00_JPRB, 0.50655E+00_JPRB, 0.45944E+00_JPRB ,&
&  0.48417E+00_JPRB, 0.42526E+00_JPRB, 0.39150E+00_JPRB, 0.37912E+00_JPRB, 0.37784E+00_JPRB, 0.36992E+00_JPRB, 0.36140E+00_JPRB ,&
&  0.36079E+00_JPRB, 0.37489E+00_JPRB, 0.40173E+00_JPRB, 0.42237E+00_JPRB, 0.44220E+00_JPRB, 0.45099E+00_JPRB, 0.44463E+00_JPRB ,&
&  0.43599E+00_JPRB, 0.43024E+00_JPRB, 0.42705E+00_JPRB, 0.42819E+00_JPRB, 0.43173E+00_JPRB, 0.43896E+00_JPRB, 0.44234E+00_JPRB ,&
&  0.44201E+00_JPRB, 0.44681E+00_JPRB, 0.45979E+00_JPRB, 0.48557E+00_JPRB, 0.48984E+00_JPRB, 0.49023E+00_JPRB, 0.49034E+00_JPRB ,&
&  0.40821E+00_JPRB, 0.47160E+00_JPRB, 0.71242E+00_JPRB, 0.77783E+00_JPRB, 0.81873E+00_JPRB, 0.87531E+00_JPRB, 0.92748E+00_JPRB ,&
&  0.97965E+00_JPRB, 0.10477E+01_JPRB, 0.11363E+01_JPRB, 0.12328E+01_JPRB, 0.13421E+01_JPRB, 0.15215E+01_JPRB, 0.17980E+01_JPRB ,&
&  0.24199E+01_JPRB, 0.19522E+01_JPRB, 0.17866E+01_JPRB, 0.18425E+01_JPRB, 0.25584E+01_JPRB, 0.25721E+01_JPRB, 0.22366E+01_JPRB ,&
&  0.20555E+01_JPRB, 0.18937E+01_JPRB, 0.17560E+01_JPRB, 0.15610E+01_JPRB, 0.14162E+01_JPRB, 0.13635E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.10818E+01_JPRB, 0.94052E+00_JPRB, 0.89825E+00_JPRB, 0.89116E+00_JPRB, 0.83818E+00_JPRB, 0.76437E+00_JPRB ,&
&  0.69916E+00_JPRB, 0.64486E+00_JPRB, 0.60350E+00_JPRB, 0.57445E+00_JPRB, 0.55120E+00_JPRB, 0.53811E+00_JPRB, 0.52914E+00_JPRB ,&
&  0.52274E+00_JPRB, 0.51640E+00_JPRB, 0.49630E+00_JPRB, 0.48464E+00_JPRB, 0.46978E+00_JPRB, 0.45390E+00_JPRB, 0.44559E+00_JPRB ,&
&  0.42610E+00_JPRB, 0.41515E+00_JPRB, 0.39615E+00_JPRB, 0.39545E+00_JPRB, 0.40684E+00_JPRB, 0.43223E+00_JPRB, 0.45377E+00_JPRB ,&
&  0.48437E+00_JPRB, 0.48407E+00_JPRB, 0.45641E+00_JPRB, 0.44898E+00_JPRB, 0.44884E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.43481E+00_JPRB, 0.47129E+00_JPRB, 0.50300E+00_JPRB, 0.52873E+00_JPRB, 0.52789E+00_JPRB, 0.50693E+00_JPRB, 0.45353E+00_JPRB ,&
&  0.44031E+00_JPRB, 0.41439E+00_JPRB, 0.40118E+00_JPRB, 0.38676E+00_JPRB, 0.37702E+00_JPRB, 0.36960E+00_JPRB, 0.36847E+00_JPRB ,&
&  0.37500E+00_JPRB, 0.38672E+00_JPRB, 0.39854E+00_JPRB, 0.40555E+00_JPRB, 0.41138E+00_JPRB, 0.41264E+00_JPRB, 0.41230E+00_JPRB ,&
&  0.41044E+00_JPRB, 0.40897E+00_JPRB, 0.40868E+00_JPRB, 0.41026E+00_JPRB, 0.41611E+00_JPRB, 0.42332E+00_JPRB, 0.42421E+00_JPRB ,&
&  0.41880E+00_JPRB, 0.41345E+00_JPRB, 0.42825E+00_JPRB, 0.45340E+00_JPRB, 0.45176E+00_JPRB, 0.44881E+00_JPRB, 0.40665E+00_JPRB ,&
&  0.36870E+00_JPRB, 0.38586E+00_JPRB, 0.47991E+00_JPRB, 0.68703E+00_JPRB, 0.75212E+00_JPRB, 0.81946E+00_JPRB, 0.86682E+00_JPRB ,&
&  0.82923E+00_JPRB, 0.84941E+00_JPRB, 0.95746E+00_JPRB, 0.12154E+01_JPRB, 0.12896E+01_JPRB, 0.14076E+01_JPRB, 0.15329E+01_JPRB ,&
&  0.17949E+01_JPRB, 0.19563E+01_JPRB, 0.20076E+01_JPRB, 0.16002E+01_JPRB, 0.17531E+01_JPRB, 0.21413E+01_JPRB, 0.18398E+01_JPRB ,&
&  0.17046E+01_JPRB, 0.16111E+01_JPRB, 0.16165E+01_JPRB, 0.14714E+01_JPRB, 0.12968E+01_JPRB, 0.11881E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10111E+01_JPRB, 0.86879E+00_JPRB, 0.80159E+00_JPRB, 0.79280E+00_JPRB, 0.76682E+00_JPRB, 0.74156E+00_JPRB ,&
&  0.70858E+00_JPRB, 0.64637E+00_JPRB, 0.58893E+00_JPRB, 0.53154E+00_JPRB, 0.49450E+00_JPRB, 0.48461E+00_JPRB, 0.48730E+00_JPRB ,&
&  0.48911E+00_JPRB, 0.48552E+00_JPRB, 0.48005E+00_JPRB, 0.47184E+00_JPRB, 0.45340E+00_JPRB, 0.42842E+00_JPRB, 0.41148E+00_JPRB ,&
&  0.39517E+00_JPRB, 0.37951E+00_JPRB, 0.36955E+00_JPRB, 0.37310E+00_JPRB, 0.38095E+00_JPRB, 0.39651E+00_JPRB, 0.41167E+00_JPRB ,&
&  0.42095E+00_JPRB, 0.42740E+00_JPRB, 0.42183E+00_JPRB, 0.42255E+00_JPRB, 0.41808E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.39563E+00_JPRB, 0.43399E+00_JPRB, 0.49146E+00_JPRB, 0.52508E+00_JPRB, 0.52956E+00_JPRB, 0.49181E+00_JPRB, 0.44969E+00_JPRB ,&
&  0.42556E+00_JPRB, 0.41188E+00_JPRB, 0.40882E+00_JPRB, 0.40674E+00_JPRB, 0.40631E+00_JPRB, 0.40712E+00_JPRB, 0.40881E+00_JPRB ,&
&  0.40889E+00_JPRB, 0.40817E+00_JPRB, 0.40836E+00_JPRB, 0.41040E+00_JPRB, 0.41525E+00_JPRB, 0.41585E+00_JPRB, 0.41517E+00_JPRB ,&
&  0.41490E+00_JPRB, 0.41485E+00_JPRB, 0.41265E+00_JPRB, 0.41077E+00_JPRB, 0.41260E+00_JPRB, 0.41854E+00_JPRB, 0.42168E+00_JPRB ,&
&  0.41402E+00_JPRB, 0.40276E+00_JPRB, 0.41124E+00_JPRB, 0.43128E+00_JPRB, 0.43017E+00_JPRB, 0.42405E+00_JPRB, 0.38785E+00_JPRB ,&
&  0.35946E+00_JPRB, 0.34065E+00_JPRB, 0.35090E+00_JPRB, 0.40275E+00_JPRB, 0.49619E+00_JPRB, 0.67467E+00_JPRB, 0.76196E+00_JPRB ,&
&  0.84731E+00_JPRB, 0.91351E+00_JPRB, 0.97557E+00_JPRB, 0.10394E+01_JPRB, 0.11257E+01_JPRB, 0.12064E+01_JPRB, 0.12541E+01_JPRB ,&
&  0.13290E+01_JPRB, 0.14273E+01_JPRB, 0.15685E+01_JPRB, 0.11829E+01_JPRB, 0.11281E+01_JPRB, 0.12389E+01_JPRB, 0.13658E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.12857E+01_JPRB, 0.12324E+01_JPRB, 0.12351E+01_JPRB, 0.11333E+01_JPRB, 0.10456E+01_JPRB, 0.98189E+00_JPRB ,&
&  0.93111E+00_JPRB, 0.90478E+00_JPRB, 0.82165E+00_JPRB, 0.72350E+00_JPRB, 0.69641E+00_JPRB, 0.69505E+00_JPRB, 0.70840E+00_JPRB ,&
&  0.74062E+00_JPRB, 0.75618E+00_JPRB, 0.68292E+00_JPRB, 0.52990E+00_JPRB, 0.48540E+00_JPRB, 0.46750E+00_JPRB, 0.46948E+00_JPRB ,&
&  0.47335E+00_JPRB, 0.47267E+00_JPRB, 0.47147E+00_JPRB, 0.46346E+00_JPRB, 0.44871E+00_JPRB, 0.43027E+00_JPRB, 0.41211E+00_JPRB ,&
&  0.39615E+00_JPRB, 0.38906E+00_JPRB, 0.39137E+00_JPRB, 0.39460E+00_JPRB, 0.40692E+00_JPRB, 0.41933E+00_JPRB, 0.43223E+00_JPRB ,&
&  0.43002E+00_JPRB, 0.42385E+00_JPRB, 0.41536E+00_JPRB, 0.39710E+00_JPRB, 0.38432E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.40733E+00_JPRB, 0.43942E+00_JPRB, 0.48338E+00_JPRB, 0.51428E+00_JPRB, 0.51900E+00_JPRB, 0.49518E+00_JPRB, 0.47366E+00_JPRB ,&
&  0.46132E+00_JPRB, 0.45783E+00_JPRB, 0.45448E+00_JPRB, 0.45076E+00_JPRB, 0.44718E+00_JPRB, 0.44346E+00_JPRB, 0.43986E+00_JPRB ,&
&  0.43749E+00_JPRB, 0.43627E+00_JPRB, 0.43490E+00_JPRB, 0.43020E+00_JPRB, 0.42707E+00_JPRB, 0.42735E+00_JPRB, 0.43019E+00_JPRB ,&
&  0.43366E+00_JPRB, 0.43288E+00_JPRB, 0.43189E+00_JPRB, 0.42709E+00_JPRB, 0.41681E+00_JPRB, 0.40503E+00_JPRB, 0.39584E+00_JPRB ,&
&  0.38944E+00_JPRB, 0.39378E+00_JPRB, 0.40231E+00_JPRB, 0.41834E+00_JPRB, 0.41585E+00_JPRB, 0.40701E+00_JPRB, 0.37308E+00_JPRB ,&
&  0.35093E+00_JPRB, 0.32603E+00_JPRB, 0.31598E+00_JPRB, 0.32451E+00_JPRB, 0.35662E+00_JPRB, 0.40730E+00_JPRB, 0.49140E+00_JPRB ,&
&  0.64471E+00_JPRB, 0.70643E+00_JPRB, 0.75591E+00_JPRB, 0.81360E+00_JPRB, 0.87065E+00_JPRB, 0.92625E+00_JPRB, 0.97048E+00_JPRB ,&
&  0.10315E+01_JPRB, 0.11040E+01_JPRB, 0.11755E+01_JPRB, 0.11599E+01_JPRB, 0.11677E+01_JPRB, 0.10830E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.15234E+01_JPRB, 0.13368E+01_JPRB, 0.11514E+01_JPRB, 0.10382E+01_JPRB, 0.96231E+00_JPRB, 0.89207E+00_JPRB, 0.83307E+00_JPRB ,&
&  0.78204E+00_JPRB, 0.75327E+00_JPRB, 0.73471E+00_JPRB, 0.68470E+00_JPRB, 0.63602E+00_JPRB, 0.62387E+00_JPRB, 0.65504E+00_JPRB ,&
&  0.74585E+00_JPRB, 0.89079E+00_JPRB, 0.10793E+01_JPRB, 0.74391E+00_JPRB, 0.62915E+00_JPRB, 0.55890E+00_JPRB, 0.52513E+00_JPRB ,&
&  0.51016E+00_JPRB, 0.49950E+00_JPRB, 0.49360E+00_JPRB, 0.48222E+00_JPRB, 0.46711E+00_JPRB, 0.45382E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.43344E+00_JPRB, 0.43380E+00_JPRB, 0.43938E+00_JPRB, 0.44481E+00_JPRB, 0.44825E+00_JPRB, 0.44866E+00_JPRB, 0.44944E+00_JPRB ,&
&  0.44705E+00_JPRB, 0.43871E+00_JPRB, 0.42958E+00_JPRB, 0.41367E+00_JPRB, 0.40239E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.42776E+00_JPRB, 0.45119E+00_JPRB, 0.47980E+00_JPRB, 0.49939E+00_JPRB, 0.50974E+00_JPRB, 0.49886E+00_JPRB, 0.48623E+00_JPRB ,&
&  0.47582E+00_JPRB, 0.46823E+00_JPRB, 0.46302E+00_JPRB, 0.45944E+00_JPRB, 0.45514E+00_JPRB, 0.45118E+00_JPRB, 0.44775E+00_JPRB ,&
&  0.44527E+00_JPRB, 0.43695E+00_JPRB, 0.43283E+00_JPRB, 0.43391E+00_JPRB, 0.43601E+00_JPRB, 0.43968E+00_JPRB, 0.44352E+00_JPRB ,&
&  0.44200E+00_JPRB, 0.43643E+00_JPRB, 0.43240E+00_JPRB, 0.41924E+00_JPRB, 0.40436E+00_JPRB, 0.39574E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.39873E+00_JPRB, 0.38785E+00_JPRB, 0.40442E+00_JPRB, 0.41802E+00_JPRB, 0.40576E+00_JPRB, 0.39326E+00_JPRB, 0.34140E+00_JPRB ,&
&  0.32649E+00_JPRB, 0.31155E+00_JPRB, 0.31035E+00_JPRB, 0.31433E+00_JPRB, 0.33382E+00_JPRB, 0.37243E+00_JPRB, 0.43092E+00_JPRB ,&
&  0.49386E+00_JPRB, 0.59992E+00_JPRB, 0.62789E+00_JPRB, 0.67884E+00_JPRB, 0.73110E+00_JPRB, 0.78127E+00_JPRB, 0.82251E+00_JPRB ,&
&  0.88053E+00_JPRB, 0.95921E+00_JPRB, 0.10145E+01_JPRB, 0.10371E+01_JPRB, 0.10660E+01_JPRB, 0.10869E+01_JPRB, 0.11263E+01_JPRB ,&
&  0.11818E+01_JPRB, 0.10121E+01_JPRB, 0.90319E+00_JPRB, 0.82803E+00_JPRB, 0.77666E+00_JPRB, 0.75552E+00_JPRB, 0.70166E+00_JPRB ,&
&  0.68129E+00_JPRB, 0.67156E+00_JPRB, 0.65600E+00_JPRB, 0.64894E+00_JPRB, 0.64013E+00_JPRB, 0.62677E+00_JPRB, 0.60539E+00_JPRB ,&
&  0.58386E+00_JPRB, 0.58787E+00_JPRB, 0.65347E+00_JPRB, 0.65003E+00_JPRB, 0.63883E+00_JPRB, 0.59758E+00_JPRB, 0.54926E+00_JPRB ,&
&  0.51106E+00_JPRB, 0.49077E+00_JPRB, 0.48176E+00_JPRB, 0.47483E+00_JPRB, 0.46999E+00_JPRB, 0.46720E+00_JPRB, 0.47136E+00_JPRB ,&
&  0.46706E+00_JPRB, 0.46980E+00_JPRB, 0.47348E+00_JPRB, 0.47459E+00_JPRB, 0.47080E+00_JPRB, 0.46306E+00_JPRB, 0.45620E+00_JPRB ,&
&  0.44915E+00_JPRB, 0.44210E+00_JPRB, 0.43579E+00_JPRB, 0.42824E+00_JPRB, 0.42325E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.44406E+00_JPRB, 0.45536E+00_JPRB, 0.47498E+00_JPRB, 0.49022E+00_JPRB, 0.50126E+00_JPRB, 0.49934E+00_JPRB, 0.49046E+00_JPRB ,&
&  0.48191E+00_JPRB, 0.47541E+00_JPRB, 0.46910E+00_JPRB, 0.46294E+00_JPRB, 0.45565E+00_JPRB, 0.44981E+00_JPRB, 0.44553E+00_JPRB ,&
&  0.44327E+00_JPRB, 0.44272E+00_JPRB, 0.44313E+00_JPRB, 0.44339E+00_JPRB, 0.44410E+00_JPRB, 0.44533E+00_JPRB, 0.44597E+00_JPRB ,&
&  0.44661E+00_JPRB, 0.44140E+00_JPRB, 0.42533E+00_JPRB, 0.40877E+00_JPRB, 0.40247E+00_JPRB, 0.39532E+00_JPRB, 0.40245E+00_JPRB ,&
&  0.40405E+00_JPRB, 0.40676E+00_JPRB, 0.40738E+00_JPRB, 0.40905E+00_JPRB, 0.39639E+00_JPRB, 0.32887E+00_JPRB, 0.32879E+00_JPRB ,&
&  0.32000E+00_JPRB, 0.31126E+00_JPRB, 0.31332E+00_JPRB, 0.31803E+00_JPRB, 0.32840E+00_JPRB, 0.35314E+00_JPRB, 0.39280E+00_JPRB ,&
&  0.43065E+00_JPRB, 0.57126E+00_JPRB, 0.60860E+00_JPRB, 0.64746E+00_JPRB, 0.68209E+00_JPRB, 0.72634E+00_JPRB, 0.77946E+00_JPRB ,&
&  0.81926E+00_JPRB, 0.86417E+00_JPRB, 0.89972E+00_JPRB, 0.93002E+00_JPRB, 0.94155E+00_JPRB, 0.94143E+00_JPRB, 0.93902E+00_JPRB ,&
&  0.91615E+00_JPRB, 0.82575E+00_JPRB, 0.76063E+00_JPRB, 0.71896E+00_JPRB, 0.67767E+00_JPRB, 0.64911E+00_JPRB, 0.62972E+00_JPRB ,&
&  0.62568E+00_JPRB, 0.60446E+00_JPRB, 0.59881E+00_JPRB, 0.58344E+00_JPRB, 0.57246E+00_JPRB, 0.56209E+00_JPRB, 0.54928E+00_JPRB ,&
&  0.53768E+00_JPRB, 0.53604E+00_JPRB, 0.54315E+00_JPRB, 0.54513E+00_JPRB, 0.55003E+00_JPRB, 0.54351E+00_JPRB, 0.53121E+00_JPRB ,&
&  0.51252E+00_JPRB, 0.50712E+00_JPRB, 0.50069E+00_JPRB, 0.49669E+00_JPRB, 0.49165E+00_JPRB, 0.49195E+00_JPRB, 0.49361E+00_JPRB ,&
&  0.49503E+00_JPRB, 0.49307E+00_JPRB, 0.49028E+00_JPRB, 0.48824E+00_JPRB, 0.48523E+00_JPRB, 0.48017E+00_JPRB, 0.47333E+00_JPRB ,&
&  0.46629E+00_JPRB, 0.46192E+00_JPRB, 0.45425E+00_JPRB, 0.44761E+00_JPRB, 0.44365E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.45873E+00_JPRB, 0.46146E+00_JPRB, 0.46778E+00_JPRB, 0.47296E+00_JPRB, 0.47676E+00_JPRB, 0.47672E+00_JPRB, 0.47098E+00_JPRB ,&
&  0.46571E+00_JPRB, 0.46105E+00_JPRB, 0.45741E+00_JPRB, 0.45489E+00_JPRB, 0.45420E+00_JPRB, 0.45173E+00_JPRB, 0.44809E+00_JPRB ,&
&  0.44490E+00_JPRB, 0.44290E+00_JPRB, 0.44202E+00_JPRB, 0.44119E+00_JPRB, 0.44219E+00_JPRB, 0.44149E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.44109E+00_JPRB, 0.43256E+00_JPRB, 0.41091E+00_JPRB, 0.40064E+00_JPRB, 0.39284E+00_JPRB, 0.39436E+00_JPRB, 0.39737E+00_JPRB ,&
&  0.40253E+00_JPRB, 0.35586E+00_JPRB, 0.35882E+00_JPRB, 0.34652E+00_JPRB, 0.33734E+00_JPRB, 0.33577E+00_JPRB, 0.32933E+00_JPRB ,&
&  0.32571E+00_JPRB, 0.32122E+00_JPRB, 0.32469E+00_JPRB, 0.33291E+00_JPRB, 0.33962E+00_JPRB, 0.35516E+00_JPRB, 0.37977E+00_JPRB ,&
&  0.41854E+00_JPRB, 0.54146E+00_JPRB, 0.59436E+00_JPRB, 0.63042E+00_JPRB, 0.65805E+00_JPRB, 0.69136E+00_JPRB, 0.72465E+00_JPRB ,&
&  0.75145E+00_JPRB, 0.78449E+00_JPRB, 0.80196E+00_JPRB, 0.79950E+00_JPRB, 0.81367E+00_JPRB, 0.80527E+00_JPRB, 0.78175E+00_JPRB ,&
&  0.74110E+00_JPRB, 0.68718E+00_JPRB, 0.64575E+00_JPRB, 0.61871E+00_JPRB, 0.59738E+00_JPRB, 0.58134E+00_JPRB, 0.57143E+00_JPRB ,&
&  0.57378E+00_JPRB, 0.56584E+00_JPRB, 0.55474E+00_JPRB, 0.53486E+00_JPRB, 0.52052E+00_JPRB, 0.51112E+00_JPRB, 0.50702E+00_JPRB ,&
&  0.50651E+00_JPRB, 0.50962E+00_JPRB, 0.51727E+00_JPRB, 0.52359E+00_JPRB, 0.52450E+00_JPRB, 0.52146E+00_JPRB, 0.52296E+00_JPRB ,&
&  0.52155E+00_JPRB, 0.51749E+00_JPRB, 0.51325E+00_JPRB, 0.50763E+00_JPRB, 0.49996E+00_JPRB, 0.49315E+00_JPRB, 0.48925E+00_JPRB ,&
&  0.48748E+00_JPRB, 0.48518E+00_JPRB, 0.48326E+00_JPRB, 0.48257E+00_JPRB, 0.48401E+00_JPRB, 0.48504E+00_JPRB, 0.48444E+00_JPRB ,&
&  0.47696E+00_JPRB, 0.46953E+00_JPRB, 0.46266E+00_JPRB, 0.46068E+00_JPRB, 0.45932E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.44664E+00_JPRB, 0.44611E+00_JPRB, 0.44652E+00_JPRB, 0.44676E+00_JPRB, 0.44690E+00_JPRB, 0.44709E+00_JPRB, 0.44726E+00_JPRB ,&
&  0.44777E+00_JPRB, 0.44902E+00_JPRB, 0.44988E+00_JPRB, 0.44981E+00_JPRB, 0.45064E+00_JPRB, 0.45214E+00_JPRB, 0.45227E+00_JPRB ,&
&  0.44909E+00_JPRB, 0.44492E+00_JPRB, 0.44092E+00_JPRB, 0.43642E+00_JPRB, 0.43253E+00_JPRB, 0.42949E+00_JPRB, 0.42506E+00_JPRB ,&
&  0.42218E+00_JPRB, 0.41596E+00_JPRB, 0.38942E+00_JPRB, 0.38888E+00_JPRB, 0.38338E+00_JPRB, 0.38260E+00_JPRB, 0.38629E+00_JPRB ,&
&  0.38213E+00_JPRB, 0.38269E+00_JPRB, 0.37568E+00_JPRB, 0.36281E+00_JPRB, 0.35383E+00_JPRB, 0.34103E+00_JPRB, 0.34029E+00_JPRB ,&
&  0.33843E+00_JPRB, 0.34219E+00_JPRB, 0.35191E+00_JPRB, 0.35795E+00_JPRB, 0.36219E+00_JPRB, 0.36830E+00_JPRB, 0.38479E+00_JPRB ,&
&  0.41908E+00_JPRB, 0.46771E+00_JPRB, 0.53048E+00_JPRB, 0.56185E+00_JPRB, 0.58842E+00_JPRB, 0.61535E+00_JPRB, 0.64071E+00_JPRB ,&
&  0.66054E+00_JPRB, 0.67707E+00_JPRB, 0.68130E+00_JPRB, 0.68215E+00_JPRB, 0.67174E+00_JPRB, 0.65104E+00_JPRB, 0.62571E+00_JPRB ,&
&  0.59867E+00_JPRB, 0.57909E+00_JPRB, 0.56410E+00_JPRB, 0.55530E+00_JPRB, 0.54886E+00_JPRB, 0.54242E+00_JPRB, 0.53737E+00_JPRB ,&
&  0.52968E+00_JPRB, 0.51341E+00_JPRB, 0.49790E+00_JPRB, 0.48185E+00_JPRB, 0.47306E+00_JPRB, 0.46682E+00_JPRB, 0.46530E+00_JPRB ,&
&  0.46540E+00_JPRB, 0.46926E+00_JPRB, 0.47525E+00_JPRB, 0.47676E+00_JPRB, 0.48160E+00_JPRB, 0.48904E+00_JPRB, 0.48934E+00_JPRB ,&
&  0.49002E+00_JPRB, 0.48871E+00_JPRB, 0.48505E+00_JPRB, 0.47975E+00_JPRB, 0.47278E+00_JPRB, 0.46556E+00_JPRB, 0.45866E+00_JPRB ,&
&  0.45396E+00_JPRB, 0.45125E+00_JPRB, 0.45198E+00_JPRB, 0.45337E+00_JPRB, 0.45646E+00_JPRB, 0.45757E+00_JPRB, 0.45548E+00_JPRB ,&
&  0.44985E+00_JPRB, 0.44649E+00_JPRB, 0.44393E+00_JPRB, 0.44502E+00_JPRB, 0.44613E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.43596E+00_JPRB, 0.43751E+00_JPRB, 0.43833E+00_JPRB, 0.43845E+00_JPRB, 0.43869E+00_JPRB, 0.43945E+00_JPRB, 0.44029E+00_JPRB ,&
&  0.44123E+00_JPRB, 0.44222E+00_JPRB, 0.44335E+00_JPRB, 0.44405E+00_JPRB, 0.44434E+00_JPRB, 0.44821E+00_JPRB, 0.44947E+00_JPRB ,&
&  0.44881E+00_JPRB, 0.44705E+00_JPRB, 0.44509E+00_JPRB, 0.44285E+00_JPRB, 0.43999E+00_JPRB, 0.43615E+00_JPRB, 0.42949E+00_JPRB ,&
&  0.42571E+00_JPRB, 0.42133E+00_JPRB, 0.41628E+00_JPRB, 0.41529E+00_JPRB, 0.41392E+00_JPRB, 0.39445E+00_JPRB, 0.39341E+00_JPRB ,&
&  0.40582E+00_JPRB, 0.41243E+00_JPRB, 0.41610E+00_JPRB, 0.42055E+00_JPRB, 0.41231E+00_JPRB, 0.39320E+00_JPRB, 0.37955E+00_JPRB ,&
&  0.35576E+00_JPRB, 0.37235E+00_JPRB, 0.38412E+00_JPRB, 0.38944E+00_JPRB, 0.39204E+00_JPRB, 0.39613E+00_JPRB, 0.42071E+00_JPRB ,&
&  0.47172E+00_JPRB, 0.48273E+00_JPRB, 0.49513E+00_JPRB, 0.50857E+00_JPRB, 0.52237E+00_JPRB, 0.53278E+00_JPRB, 0.54459E+00_JPRB ,&
&  0.55243E+00_JPRB, 0.55813E+00_JPRB, 0.55841E+00_JPRB, 0.55650E+00_JPRB, 0.54832E+00_JPRB, 0.54097E+00_JPRB, 0.53351E+00_JPRB ,&
&  0.52848E+00_JPRB, 0.52372E+00_JPRB, 0.51842E+00_JPRB, 0.51132E+00_JPRB, 0.50270E+00_JPRB, 0.49366E+00_JPRB, 0.48390E+00_JPRB ,&
&  0.47353E+00_JPRB, 0.46190E+00_JPRB, 0.45288E+00_JPRB, 0.44504E+00_JPRB, 0.44166E+00_JPRB, 0.43889E+00_JPRB, 0.43916E+00_JPRB ,&
&  0.43974E+00_JPRB, 0.44226E+00_JPRB, 0.44546E+00_JPRB, 0.45094E+00_JPRB, 0.45523E+00_JPRB, 0.45874E+00_JPRB, 0.46091E+00_JPRB ,&
&  0.46258E+00_JPRB, 0.46275E+00_JPRB, 0.46222E+00_JPRB, 0.46006E+00_JPRB, 0.45791E+00_JPRB, 0.45569E+00_JPRB, 0.45295E+00_JPRB ,&
&  0.45050E+00_JPRB, 0.44795E+00_JPRB, 0.44584E+00_JPRB, 0.44353E+00_JPRB, 0.44107E+00_JPRB, 0.43746E+00_JPRB, 0.43398E+00_JPRB ,&
&  0.43088E+00_JPRB, 0.42980E+00_JPRB, 0.42963E+00_JPRB, 0.43128E+00_JPRB, 0.43346E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.44304E+00_JPRB, 0.44421E+00_JPRB, 0.44501E+00_JPRB, 0.44526E+00_JPRB, 0.44557E+00_JPRB, 0.44577E+00_JPRB, 0.44547E+00_JPRB ,&
&  0.44453E+00_JPRB, 0.44401E+00_JPRB, 0.44403E+00_JPRB, 0.44383E+00_JPRB, 0.44312E+00_JPRB, 0.44480E+00_JPRB, 0.44820E+00_JPRB ,&
&  0.44878E+00_JPRB, 0.44890E+00_JPRB, 0.44890E+00_JPRB, 0.44905E+00_JPRB, 0.44883E+00_JPRB, 0.44822E+00_JPRB, 0.44767E+00_JPRB ,&
&  0.44792E+00_JPRB, 0.44787E+00_JPRB, 0.44750E+00_JPRB, 0.44649E+00_JPRB, 0.44714E+00_JPRB, 0.44772E+00_JPRB, 0.44784E+00_JPRB ,&
&  0.44746E+00_JPRB, 0.44632E+00_JPRB, 0.44531E+00_JPRB, 0.44515E+00_JPRB, 0.44798E+00_JPRB, 0.45156E+00_JPRB, 0.45616E+00_JPRB ,&
&  0.46156E+00_JPRB, 0.46811E+00_JPRB, 0.47470E+00_JPRB, 0.48037E+00_JPRB, 0.48444E+00_JPRB, 0.48962E+00_JPRB, 0.49352E+00_JPRB ,&
&  0.49480E+00_JPRB, 0.49371E+00_JPRB, 0.49230E+00_JPRB, 0.49249E+00_JPRB, 0.49263E+00_JPRB, 0.49234E+00_JPRB, 0.49254E+00_JPRB ,&
&  0.49305E+00_JPRB, 0.49339E+00_JPRB, 0.49355E+00_JPRB, 0.49285E+00_JPRB, 0.49106E+00_JPRB, 0.48898E+00_JPRB, 0.48630E+00_JPRB ,&
&  0.48360E+00_JPRB, 0.48011E+00_JPRB, 0.47690E+00_JPRB, 0.47357E+00_JPRB, 0.46998E+00_JPRB, 0.46571E+00_JPRB, 0.46103E+00_JPRB ,&
&  0.45729E+00_JPRB, 0.45494E+00_JPRB, 0.45339E+00_JPRB, 0.45088E+00_JPRB, 0.44883E+00_JPRB, 0.44718E+00_JPRB, 0.44741E+00_JPRB ,&
&  0.44793E+00_JPRB, 0.44979E+00_JPRB, 0.45171E+00_JPRB, 0.45461E+00_JPRB, 0.45683E+00_JPRB, 0.45871E+00_JPRB, 0.45956E+00_JPRB ,&
&  0.46023E+00_JPRB, 0.46021E+00_JPRB, 0.45990E+00_JPRB, 0.45907E+00_JPRB, 0.45800E+00_JPRB, 0.45656E+00_JPRB, 0.45463E+00_JPRB ,&
&  0.45251E+00_JPRB, 0.45038E+00_JPRB, 0.44800E+00_JPRB, 0.44540E+00_JPRB, 0.44318E+00_JPRB, 0.44094E+00_JPRB, 0.43965E+00_JPRB ,&
&  0.43875E+00_JPRB, 0.43863E+00_JPRB, 0.43893E+00_JPRB, 0.44020E+00_JPRB, 0.44169E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.44966E+00_JPRB, 0.45071E+00_JPRB, 0.45172E+00_JPRB, 0.45258E+00_JPRB, 0.45334E+00_JPRB, 0.45393E+00_JPRB, 0.45444E+00_JPRB ,&
&  0.45461E+00_JPRB, 0.45530E+00_JPRB, 0.45624E+00_JPRB, 0.45714E+00_JPRB, 0.45796E+00_JPRB, 0.45879E+00_JPRB, 0.45955E+00_JPRB ,&
&  0.46065E+00_JPRB, 0.46360E+00_JPRB, 0.46599E+00_JPRB, 0.46755E+00_JPRB, 0.46906E+00_JPRB, 0.47058E+00_JPRB, 0.47200E+00_JPRB ,&
&  0.47333E+00_JPRB, 0.47447E+00_JPRB, 0.47550E+00_JPRB, 0.47638E+00_JPRB, 0.47737E+00_JPRB, 0.47831E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.48040E+00_JPRB, 0.48194E+00_JPRB, 0.48368E+00_JPRB, 0.48553E+00_JPRB, 0.48749E+00_JPRB, 0.48998E+00_JPRB, 0.49226E+00_JPRB ,&
&  0.49416E+00_JPRB, 0.49544E+00_JPRB, 0.49529E+00_JPRB, 0.49409E+00_JPRB, 0.49214E+00_JPRB, 0.48978E+00_JPRB, 0.48705E+00_JPRB ,&
&  0.48385E+00_JPRB, 0.48092E+00_JPRB, 0.47838E+00_JPRB, 0.47606E+00_JPRB, 0.47392E+00_JPRB, 0.47210E+00_JPRB, 0.47024E+00_JPRB ,&
&  0.46852E+00_JPRB, 0.46674E+00_JPRB, 0.46512E+00_JPRB, 0.46353E+00_JPRB, 0.46197E+00_JPRB, 0.46028E+00_JPRB, 0.45865E+00_JPRB ,&
&  0.45743E+00_JPRB, 0.45623E+00_JPRB, 0.45537E+00_JPRB, 0.45451E+00_JPRB, 0.45387E+00_JPRB, 0.45366E+00_JPRB, 0.45363E+00_JPRB ,&
&  0.45355E+00_JPRB, 0.45358E+00_JPRB, 0.45383E+00_JPRB, 0.45382E+00_JPRB, 0.45368E+00_JPRB, 0.45334E+00_JPRB, 0.45300E+00_JPRB ,&
&  0.45276E+00_JPRB, 0.45284E+00_JPRB, 0.45274E+00_JPRB, 0.45280E+00_JPRB, 0.45287E+00_JPRB, 0.45299E+00_JPRB, 0.45321E+00_JPRB ,&
&  0.45312E+00_JPRB, 0.45313E+00_JPRB, 0.45300E+00_JPRB, 0.45302E+00_JPRB, 0.45292E+00_JPRB, 0.45256E+00_JPRB, 0.45195E+00_JPRB ,&
&  0.45122E+00_JPRB, 0.45042E+00_JPRB, 0.44967E+00_JPRB, 0.44883E+00_JPRB, 0.44834E+00_JPRB, 0.44811E+00_JPRB, 0.44790E+00_JPRB ,&
&  0.44767E+00_JPRB, 0.44768E+00_JPRB, 0.44785E+00_JPRB, 0.44836E+00_JPRB, 0.44882E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB, 0.46380E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB ,&
&  0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB, 0.14861E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.15381E+00_JPRB, 0.15384E+00_JPRB, 0.15373E+00_JPRB, 0.15316E+00_JPRB, 0.15131E+00_JPRB, 0.15100E+00_JPRB, 0.15099E+00_JPRB ,&
&  0.15115E+00_JPRB, 0.15126E+00_JPRB, 0.15200E+00_JPRB, 0.15281E+00_JPRB, 0.15360E+00_JPRB, 0.15420E+00_JPRB, 0.15471E+00_JPRB ,&
&  0.15499E+00_JPRB, 0.15526E+00_JPRB, 0.15433E+00_JPRB, 0.15388E+00_JPRB, 0.15376E+00_JPRB, 0.15377E+00_JPRB, 0.15377E+00_JPRB ,&
&  0.15372E+00_JPRB, 0.15370E+00_JPRB, 0.15358E+00_JPRB, 0.15373E+00_JPRB, 0.15399E+00_JPRB, 0.15420E+00_JPRB, 0.15446E+00_JPRB ,&
&  0.15465E+00_JPRB, 0.15482E+00_JPRB, 0.15493E+00_JPRB, 0.15500E+00_JPRB, 0.15525E+00_JPRB, 0.15536E+00_JPRB, 0.15527E+00_JPRB ,&
&  0.15508E+00_JPRB, 0.15474E+00_JPRB, 0.15436E+00_JPRB, 0.15397E+00_JPRB, 0.15367E+00_JPRB, 0.15340E+00_JPRB, 0.15314E+00_JPRB ,&
&  0.15286E+00_JPRB, 0.15257E+00_JPRB, 0.15226E+00_JPRB, 0.15191E+00_JPRB, 0.15158E+00_JPRB, 0.15134E+00_JPRB, 0.15108E+00_JPRB ,&
&  0.15086E+00_JPRB, 0.15066E+00_JPRB, 0.15044E+00_JPRB, 0.15016E+00_JPRB, 0.14988E+00_JPRB, 0.14959E+00_JPRB, 0.14928E+00_JPRB ,&
&  0.14888E+00_JPRB, 0.14861E+00_JPRB, 0.14833E+00_JPRB, 0.14809E+00_JPRB, 0.14789E+00_JPRB, 0.14767E+00_JPRB, 0.14748E+00_JPRB ,&
&  0.14727E+00_JPRB, 0.14708E+00_JPRB, 0.14685E+00_JPRB, 0.14664E+00_JPRB, 0.14645E+00_JPRB, 0.14629E+00_JPRB, 0.14616E+00_JPRB ,&
&  0.14604E+00_JPRB, 0.14592E+00_JPRB, 0.14582E+00_JPRB, 0.14584E+00_JPRB, 0.14583E+00_JPRB, 0.14587E+00_JPRB, 0.14591E+00_JPRB ,&
&  0.14595E+00_JPRB, 0.14598E+00_JPRB, 0.14600E+00_JPRB, 0.14592E+00_JPRB, 0.14605E+00_JPRB, 0.14624E+00_JPRB, 0.14653E+00_JPRB ,&
&  0.14685E+00_JPRB, 0.14727E+00_JPRB, 0.14771E+00_JPRB, 0.14816E+00_JPRB, 0.14869E+00_JPRB, 0.14913E+00_JPRB, 0.14961E+00_JPRB ,&
&  0.15017E+00_JPRB, 0.15075E+00_JPRB, 0.15117E+00_JPRB, 0.15166E+00_JPRB, 0.15242E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.16442E+00_JPRB, 0.16725E+00_JPRB, 0.16917E+00_JPRB, 0.17098E+00_JPRB, 0.16658E+00_JPRB, 0.17774E+00_JPRB, 0.17461E+00_JPRB ,&
&  0.17400E+00_JPRB, 0.17215E+00_JPRB, 0.17339E+00_JPRB, 0.17319E+00_JPRB, 0.17241E+00_JPRB, 0.17134E+00_JPRB, 0.17005E+00_JPRB ,&
&  0.16885E+00_JPRB, 0.16739E+00_JPRB, 0.16640E+00_JPRB, 0.16535E+00_JPRB, 0.16450E+00_JPRB, 0.16369E+00_JPRB, 0.16314E+00_JPRB ,&
&  0.16309E+00_JPRB, 0.16378E+00_JPRB, 0.16503E+00_JPRB, 0.16454E+00_JPRB, 0.16461E+00_JPRB, 0.16504E+00_JPRB, 0.16566E+00_JPRB ,&
&  0.16596E+00_JPRB, 0.16613E+00_JPRB, 0.16593E+00_JPRB, 0.16499E+00_JPRB, 0.16488E+00_JPRB, 0.16481E+00_JPRB, 0.16481E+00_JPRB ,&
&  0.16466E+00_JPRB, 0.16431E+00_JPRB, 0.16346E+00_JPRB, 0.16266E+00_JPRB, 0.16185E+00_JPRB, 0.16089E+00_JPRB, 0.15977E+00_JPRB ,&
&  0.15885E+00_JPRB, 0.15811E+00_JPRB, 0.15727E+00_JPRB, 0.15644E+00_JPRB, 0.15572E+00_JPRB, 0.15503E+00_JPRB, 0.15439E+00_JPRB ,&
&  0.15398E+00_JPRB, 0.15367E+00_JPRB, 0.15334E+00_JPRB, 0.15284E+00_JPRB, 0.15228E+00_JPRB, 0.15174E+00_JPRB, 0.15125E+00_JPRB ,&
&  0.15092E+00_JPRB, 0.15040E+00_JPRB, 0.14989E+00_JPRB, 0.14934E+00_JPRB, 0.14891E+00_JPRB, 0.14823E+00_JPRB, 0.14763E+00_JPRB ,&
&  0.14706E+00_JPRB, 0.14654E+00_JPRB, 0.14615E+00_JPRB, 0.14588E+00_JPRB, 0.14563E+00_JPRB, 0.14547E+00_JPRB, 0.14537E+00_JPRB ,&
&  0.14533E+00_JPRB, 0.14532E+00_JPRB, 0.14539E+00_JPRB, 0.14540E+00_JPRB, 0.14537E+00_JPRB, 0.14540E+00_JPRB, 0.14568E+00_JPRB ,&
&  0.14593E+00_JPRB, 0.14611E+00_JPRB, 0.14624E+00_JPRB, 0.14603E+00_JPRB, 0.14640E+00_JPRB, 0.14673E+00_JPRB, 0.14712E+00_JPRB ,&
&  0.14748E+00_JPRB, 0.14819E+00_JPRB, 0.14905E+00_JPRB, 0.15008E+00_JPRB, 0.15200E+00_JPRB, 0.15375E+00_JPRB, 0.15558E+00_JPRB ,&
&  0.15763E+00_JPRB, 0.16058E+00_JPRB, 0.16191E+00_JPRB, 0.16282E+00_JPRB, 0.16362E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.18907E+00_JPRB, 0.19044E+00_JPRB, 0.19178E+00_JPRB, 0.19304E+00_JPRB, 0.19439E+00_JPRB, 0.19489E+00_JPRB, 0.19477E+00_JPRB ,&
&  0.19248E+00_JPRB, 0.19183E+00_JPRB, 0.19028E+00_JPRB, 0.18843E+00_JPRB, 0.18694E+00_JPRB, 0.18437E+00_JPRB, 0.18175E+00_JPRB ,&
&  0.17907E+00_JPRB, 0.17647E+00_JPRB, 0.17413E+00_JPRB, 0.17093E+00_JPRB, 0.16784E+00_JPRB, 0.16483E+00_JPRB, 0.16381E+00_JPRB ,&
&  0.16336E+00_JPRB, 0.16375E+00_JPRB, 0.16528E+00_JPRB, 0.16661E+00_JPRB, 0.16854E+00_JPRB, 0.17061E+00_JPRB, 0.17376E+00_JPRB ,&
&  0.17690E+00_JPRB, 0.17923E+00_JPRB, 0.17963E+00_JPRB, 0.17859E+00_JPRB, 0.17760E+00_JPRB, 0.17724E+00_JPRB, 0.17676E+00_JPRB ,&
&  0.17591E+00_JPRB, 0.17533E+00_JPRB, 0.17463E+00_JPRB, 0.17402E+00_JPRB, 0.17256E+00_JPRB, 0.17011E+00_JPRB, 0.16767E+00_JPRB ,&
&  0.16541E+00_JPRB, 0.16341E+00_JPRB, 0.16156E+00_JPRB, 0.15998E+00_JPRB, 0.15903E+00_JPRB, 0.15810E+00_JPRB, 0.15699E+00_JPRB ,&
&  0.15631E+00_JPRB, 0.15561E+00_JPRB, 0.15478E+00_JPRB, 0.15415E+00_JPRB, 0.15344E+00_JPRB, 0.15292E+00_JPRB, 0.15233E+00_JPRB ,&
&  0.15186E+00_JPRB, 0.15118E+00_JPRB, 0.15058E+00_JPRB, 0.14994E+00_JPRB, 0.14963E+00_JPRB, 0.14894E+00_JPRB, 0.14828E+00_JPRB ,&
&  0.14747E+00_JPRB, 0.14697E+00_JPRB, 0.14606E+00_JPRB, 0.14552E+00_JPRB, 0.14490E+00_JPRB, 0.14435E+00_JPRB, 0.14400E+00_JPRB ,&
&  0.14411E+00_JPRB, 0.14417E+00_JPRB, 0.14387E+00_JPRB, 0.14387E+00_JPRB, 0.14397E+00_JPRB, 0.14440E+00_JPRB, 0.14558E+00_JPRB ,&
&  0.14634E+00_JPRB, 0.14718E+00_JPRB, 0.14801E+00_JPRB, 0.14867E+00_JPRB, 0.14975E+00_JPRB, 0.15080E+00_JPRB, 0.15187E+00_JPRB ,&
&  0.15312E+00_JPRB, 0.15421E+00_JPRB, 0.15549E+00_JPRB, 0.15700E+00_JPRB, 0.16056E+00_JPRB, 0.16322E+00_JPRB, 0.16613E+00_JPRB ,&
&  0.16840E+00_JPRB, 0.16789E+00_JPRB, 0.18139E+00_JPRB, 0.18403E+00_JPRB, 0.18597E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.19744E+00_JPRB, 0.19826E+00_JPRB, 0.19831E+00_JPRB, 0.19925E+00_JPRB, 0.20009E+00_JPRB, 0.20109E+00_JPRB, 0.20138E+00_JPRB ,&
&  0.20070E+00_JPRB, 0.19631E+00_JPRB, 0.19385E+00_JPRB, 0.19060E+00_JPRB, 0.18744E+00_JPRB, 0.18421E+00_JPRB, 0.18152E+00_JPRB ,&
&  0.17976E+00_JPRB, 0.17840E+00_JPRB, 0.17716E+00_JPRB, 0.17546E+00_JPRB, 0.17344E+00_JPRB, 0.17142E+00_JPRB, 0.16957E+00_JPRB ,&
&  0.16792E+00_JPRB, 0.16792E+00_JPRB, 0.16901E+00_JPRB, 0.17109E+00_JPRB, 0.17488E+00_JPRB, 0.17798E+00_JPRB, 0.18194E+00_JPRB ,&
&  0.18835E+00_JPRB, 0.18579E+00_JPRB, 0.18592E+00_JPRB, 0.18605E+00_JPRB, 0.18569E+00_JPRB, 0.18542E+00_JPRB, 0.18218E+00_JPRB ,&
&  0.18499E+00_JPRB, 0.18518E+00_JPRB, 0.18842E+00_JPRB, 0.18783E+00_JPRB, 0.18363E+00_JPRB, 0.17996E+00_JPRB, 0.17513E+00_JPRB ,&
&  0.17148E+00_JPRB, 0.16722E+00_JPRB, 0.16436E+00_JPRB, 0.16185E+00_JPRB, 0.15906E+00_JPRB, 0.15790E+00_JPRB, 0.15598E+00_JPRB ,&
&  0.15478E+00_JPRB, 0.15450E+00_JPRB, 0.15362E+00_JPRB, 0.15355E+00_JPRB, 0.15284E+00_JPRB, 0.15182E+00_JPRB, 0.15113E+00_JPRB ,&
&  0.15051E+00_JPRB, 0.14984E+00_JPRB, 0.14960E+00_JPRB, 0.14937E+00_JPRB, 0.14950E+00_JPRB, 0.14955E+00_JPRB, 0.14966E+00_JPRB ,&
&  0.14989E+00_JPRB, 0.15003E+00_JPRB, 0.14967E+00_JPRB, 0.14951E+00_JPRB, 0.14868E+00_JPRB, 0.14797E+00_JPRB, 0.14688E+00_JPRB ,&
&  0.14622E+00_JPRB, 0.14586E+00_JPRB, 0.14534E+00_JPRB, 0.14500E+00_JPRB, 0.14494E+00_JPRB, 0.14541E+00_JPRB, 0.14600E+00_JPRB ,&
&  0.14667E+00_JPRB, 0.14784E+00_JPRB, 0.14881E+00_JPRB, 0.15011E+00_JPRB, 0.15155E+00_JPRB, 0.15270E+00_JPRB, 0.15434E+00_JPRB ,&
&  0.15598E+00_JPRB, 0.15766E+00_JPRB, 0.16045E+00_JPRB, 0.16359E+00_JPRB, 0.16778E+00_JPRB, 0.17273E+00_JPRB, 0.17583E+00_JPRB ,&
&  0.17939E+00_JPRB, 0.18677E+00_JPRB, 0.18936E+00_JPRB, 0.19568E+00_JPRB, 0.19667E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.20242E+00_JPRB, 0.20230E+00_JPRB, 0.20168E+00_JPRB, 0.20120E+00_JPRB, 0.20163E+00_JPRB, 0.20237E+00_JPRB, 0.20323E+00_JPRB ,&
&  0.19980E+00_JPRB, 0.19704E+00_JPRB, 0.19272E+00_JPRB, 0.18793E+00_JPRB, 0.18377E+00_JPRB, 0.18030E+00_JPRB, 0.17826E+00_JPRB ,&
&  0.17798E+00_JPRB, 0.17979E+00_JPRB, 0.18135E+00_JPRB, 0.18312E+00_JPRB, 0.18329E+00_JPRB, 0.18347E+00_JPRB, 0.18225E+00_JPRB ,&
&  0.18091E+00_JPRB, 0.18035E+00_JPRB, 0.18062E+00_JPRB, 0.18243E+00_JPRB, 0.18513E+00_JPRB, 0.18706E+00_JPRB, 0.18930E+00_JPRB ,&
&  0.19224E+00_JPRB, 0.19178E+00_JPRB, 0.19255E+00_JPRB, 0.19205E+00_JPRB, 0.19219E+00_JPRB, 0.19287E+00_JPRB, 0.19316E+00_JPRB ,&
&  0.19381E+00_JPRB, 0.19450E+00_JPRB, 0.19429E+00_JPRB, 0.19364E+00_JPRB, 0.18963E+00_JPRB, 0.18681E+00_JPRB, 0.18218E+00_JPRB ,&
&  0.17677E+00_JPRB, 0.17206E+00_JPRB, 0.16690E+00_JPRB, 0.16333E+00_JPRB, 0.15928E+00_JPRB, 0.15686E+00_JPRB, 0.15498E+00_JPRB ,&
&  0.15322E+00_JPRB, 0.15252E+00_JPRB, 0.15157E+00_JPRB, 0.15058E+00_JPRB, 0.15040E+00_JPRB, 0.14945E+00_JPRB, 0.14937E+00_JPRB ,&
&  0.14959E+00_JPRB, 0.14979E+00_JPRB, 0.15063E+00_JPRB, 0.15108E+00_JPRB, 0.15185E+00_JPRB, 0.15259E+00_JPRB, 0.15322E+00_JPRB ,&
&  0.15433E+00_JPRB, 0.15564E+00_JPRB, 0.15567E+00_JPRB, 0.15587E+00_JPRB, 0.15506E+00_JPRB, 0.15344E+00_JPRB, 0.15164E+00_JPRB ,&
&  0.14977E+00_JPRB, 0.14830E+00_JPRB, 0.14784E+00_JPRB, 0.14720E+00_JPRB, 0.14735E+00_JPRB, 0.14770E+00_JPRB, 0.14792E+00_JPRB ,&
&  0.14857E+00_JPRB, 0.14915E+00_JPRB, 0.14993E+00_JPRB, 0.15157E+00_JPRB, 0.15276E+00_JPRB, 0.15427E+00_JPRB, 0.15584E+00_JPRB ,&
&  0.15709E+00_JPRB, 0.15932E+00_JPRB, 0.16131E+00_JPRB, 0.16402E+00_JPRB, 0.17020E+00_JPRB, 0.17588E+00_JPRB, 0.18600E+00_JPRB ,&
&  0.19163E+00_JPRB, 0.20453E+00_JPRB, 0.20326E+00_JPRB, 0.20198E+00_JPRB, 0.20281E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.20856E+00_JPRB, 0.20734E+00_JPRB, 0.20628E+00_JPRB, 0.20508E+00_JPRB, 0.20583E+00_JPRB, 0.20690E+00_JPRB, 0.20823E+00_JPRB ,&
&  0.20976E+00_JPRB, 0.21013E+00_JPRB, 0.20983E+00_JPRB, 0.20789E+00_JPRB, 0.20666E+00_JPRB, 0.20029E+00_JPRB, 0.19791E+00_JPRB ,&
&  0.19614E+00_JPRB, 0.19770E+00_JPRB, 0.19997E+00_JPRB, 0.20346E+00_JPRB, 0.20510E+00_JPRB, 0.20680E+00_JPRB, 0.20603E+00_JPRB ,&
&  0.20398E+00_JPRB, 0.20146E+00_JPRB, 0.19912E+00_JPRB, 0.19724E+00_JPRB, 0.19671E+00_JPRB, 0.19533E+00_JPRB, 0.19381E+00_JPRB ,&
&  0.19372E+00_JPRB, 0.19503E+00_JPRB, 0.19583E+00_JPRB, 0.19666E+00_JPRB, 0.20267E+00_JPRB, 0.20350E+00_JPRB, 0.20539E+00_JPRB ,&
&  0.20583E+00_JPRB, 0.20623E+00_JPRB, 0.20545E+00_JPRB, 0.20409E+00_JPRB, 0.20285E+00_JPRB, 0.19821E+00_JPRB, 0.19604E+00_JPRB ,&
&  0.18869E+00_JPRB, 0.18431E+00_JPRB, 0.17778E+00_JPRB, 0.17206E+00_JPRB, 0.16610E+00_JPRB, 0.16166E+00_JPRB, 0.15798E+00_JPRB ,&
&  0.15506E+00_JPRB, 0.15291E+00_JPRB, 0.15109E+00_JPRB, 0.15084E+00_JPRB, 0.15097E+00_JPRB, 0.15070E+00_JPRB, 0.15086E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15230E+00_JPRB, 0.15224E+00_JPRB, 0.15279E+00_JPRB, 0.15331E+00_JPRB, 0.15406E+00_JPRB, 0.15545E+00_JPRB ,&
&  0.15759E+00_JPRB, 0.15953E+00_JPRB, 0.16042E+00_JPRB, 0.16197E+00_JPRB, 0.16128E+00_JPRB, 0.15876E+00_JPRB, 0.15624E+00_JPRB ,&
&  0.15356E+00_JPRB, 0.15238E+00_JPRB, 0.15114E+00_JPRB, 0.15044E+00_JPRB, 0.15037E+00_JPRB, 0.15057E+00_JPRB, 0.15131E+00_JPRB ,&
&  0.15211E+00_JPRB, 0.15293E+00_JPRB, 0.15328E+00_JPRB, 0.15376E+00_JPRB, 0.15418E+00_JPRB, 0.15496E+00_JPRB, 0.15637E+00_JPRB ,&
&  0.15827E+00_JPRB, 0.16046E+00_JPRB, 0.16260E+00_JPRB, 0.16523E+00_JPRB, 0.16960E+00_JPRB, 0.17559E+00_JPRB, 0.18470E+00_JPRB ,&
&  0.19233E+00_JPRB, 0.20563E+00_JPRB, 0.20676E+00_JPRB, 0.20761E+00_JPRB, 0.20879E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.21707E+00_JPRB, 0.21732E+00_JPRB, 0.21383E+00_JPRB, 0.21068E+00_JPRB, 0.20920E+00_JPRB, 0.20870E+00_JPRB, 0.20977E+00_JPRB ,&
&  0.21236E+00_JPRB, 0.21432E+00_JPRB, 0.21628E+00_JPRB, 0.21800E+00_JPRB, 0.21873E+00_JPRB, 0.21868E+00_JPRB, 0.21959E+00_JPRB ,&
&  0.22046E+00_JPRB, 0.22269E+00_JPRB, 0.22375E+00_JPRB, 0.22593E+00_JPRB, 0.22616E+00_JPRB, 0.22689E+00_JPRB, 0.22807E+00_JPRB ,&
&  0.22770E+00_JPRB, 0.22844E+00_JPRB, 0.23000E+00_JPRB, 0.22805E+00_JPRB, 0.22671E+00_JPRB, 0.22463E+00_JPRB, 0.22285E+00_JPRB ,&
&  0.21704E+00_JPRB, 0.21084E+00_JPRB, 0.20405E+00_JPRB, 0.20690E+00_JPRB, 0.21494E+00_JPRB, 0.21714E+00_JPRB, 0.22072E+00_JPRB ,&
&  0.22163E+00_JPRB, 0.22007E+00_JPRB, 0.21834E+00_JPRB, 0.21535E+00_JPRB, 0.21240E+00_JPRB, 0.20914E+00_JPRB, 0.20660E+00_JPRB ,&
&  0.20387E+00_JPRB, 0.20434E+00_JPRB, 0.20098E+00_JPRB, 0.18844E+00_JPRB, 0.18150E+00_JPRB, 0.17726E+00_JPRB, 0.17100E+00_JPRB ,&
&  0.16491E+00_JPRB, 0.16289E+00_JPRB, 0.16239E+00_JPRB, 0.16050E+00_JPRB, 0.16058E+00_JPRB, 0.16091E+00_JPRB, 0.16088E+00_JPRB ,&
&  0.15975E+00_JPRB, 0.15952E+00_JPRB, 0.15824E+00_JPRB, 0.15639E+00_JPRB, 0.15471E+00_JPRB, 0.15405E+00_JPRB, 0.15443E+00_JPRB ,&
&  0.15584E+00_JPRB, 0.15752E+00_JPRB, 0.16279E+00_JPRB, 0.16634E+00_JPRB, 0.16251E+00_JPRB, 0.15943E+00_JPRB, 0.15900E+00_JPRB ,&
&  0.15551E+00_JPRB, 0.15475E+00_JPRB, 0.15470E+00_JPRB, 0.15463E+00_JPRB, 0.15494E+00_JPRB, 0.15577E+00_JPRB, 0.15583E+00_JPRB ,&
&  0.15644E+00_JPRB, 0.15753E+00_JPRB, 0.15780E+00_JPRB, 0.15782E+00_JPRB, 0.15763E+00_JPRB, 0.15803E+00_JPRB, 0.15912E+00_JPRB ,&
&  0.16042E+00_JPRB, 0.16278E+00_JPRB, 0.16529E+00_JPRB, 0.16889E+00_JPRB, 0.17176E+00_JPRB, 0.17399E+00_JPRB, 0.17795E+00_JPRB ,&
&  0.18163E+00_JPRB, 0.19005E+00_JPRB, 0.20458E+00_JPRB, 0.21575E+00_JPRB, 0.21346E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.22265E+00_JPRB, 0.22254E+00_JPRB, 0.22131E+00_JPRB, 0.22016E+00_JPRB, 0.21862E+00_JPRB, 0.21710E+00_JPRB, 0.21762E+00_JPRB ,&
&  0.21977E+00_JPRB, 0.22261E+00_JPRB, 0.22536E+00_JPRB, 0.22750E+00_JPRB, 0.22922E+00_JPRB, 0.23005E+00_JPRB, 0.23139E+00_JPRB ,&
&  0.23300E+00_JPRB, 0.23567E+00_JPRB, 0.23635E+00_JPRB, 0.23697E+00_JPRB, 0.23637E+00_JPRB, 0.23582E+00_JPRB, 0.23696E+00_JPRB ,&
&  0.24015E+00_JPRB, 0.24179E+00_JPRB, 0.24113E+00_JPRB, 0.23926E+00_JPRB, 0.23979E+00_JPRB, 0.24150E+00_JPRB, 0.24069E+00_JPRB ,&
&  0.23474E+00_JPRB, 0.22194E+00_JPRB, 0.21267E+00_JPRB, 0.21483E+00_JPRB, 0.22369E+00_JPRB, 0.22777E+00_JPRB, 0.23032E+00_JPRB ,&
&  0.23194E+00_JPRB, 0.23222E+00_JPRB, 0.23154E+00_JPRB, 0.23023E+00_JPRB, 0.22792E+00_JPRB, 0.22387E+00_JPRB, 0.21894E+00_JPRB ,&
&  0.21447E+00_JPRB, 0.21205E+00_JPRB, 0.20940E+00_JPRB, 0.20480E+00_JPRB, 0.20017E+00_JPRB, 0.19720E+00_JPRB, 0.19403E+00_JPRB ,&
&  0.19083E+00_JPRB, 0.18821E+00_JPRB, 0.18753E+00_JPRB, 0.18724E+00_JPRB, 0.18731E+00_JPRB, 0.18704E+00_JPRB, 0.18532E+00_JPRB ,&
&  0.18177E+00_JPRB, 0.17771E+00_JPRB, 0.17213E+00_JPRB, 0.16715E+00_JPRB, 0.16355E+00_JPRB, 0.16017E+00_JPRB, 0.15630E+00_JPRB ,&
&  0.15431E+00_JPRB, 0.15640E+00_JPRB, 0.16407E+00_JPRB, 0.17308E+00_JPRB, 0.17991E+00_JPRB, 0.16834E+00_JPRB, 0.16367E+00_JPRB ,&
&  0.16218E+00_JPRB, 0.16167E+00_JPRB, 0.16182E+00_JPRB, 0.16296E+00_JPRB, 0.16358E+00_JPRB, 0.16347E+00_JPRB, 0.16400E+00_JPRB ,&
&  0.16567E+00_JPRB, 0.16745E+00_JPRB, 0.16842E+00_JPRB, 0.16939E+00_JPRB, 0.16995E+00_JPRB, 0.16973E+00_JPRB, 0.16942E+00_JPRB ,&
&  0.16994E+00_JPRB, 0.17248E+00_JPRB, 0.17686E+00_JPRB, 0.18312E+00_JPRB, 0.18789E+00_JPRB, 0.19228E+00_JPRB, 0.19806E+00_JPRB ,&
&  0.20536E+00_JPRB, 0.21296E+00_JPRB, 0.21664E+00_JPRB, 0.21993E+00_JPRB, 0.22163E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.22893E+00_JPRB, 0.23019E+00_JPRB, 0.23216E+00_JPRB, 0.23413E+00_JPRB, 0.23479E+00_JPRB, 0.23419E+00_JPRB, 0.23351E+00_JPRB ,&
&  0.23378E+00_JPRB, 0.23548E+00_JPRB, 0.23728E+00_JPRB, 0.23754E+00_JPRB, 0.23765E+00_JPRB, 0.23794E+00_JPRB, 0.23958E+00_JPRB ,&
&  0.24212E+00_JPRB, 0.24506E+00_JPRB, 0.24501E+00_JPRB, 0.24463E+00_JPRB, 0.24443E+00_JPRB, 0.24363E+00_JPRB, 0.24486E+00_JPRB ,&
&  0.24786E+00_JPRB, 0.24818E+00_JPRB, 0.24817E+00_JPRB, 0.24751E+00_JPRB, 0.24899E+00_JPRB, 0.25188E+00_JPRB, 0.25336E+00_JPRB ,&
&  0.25171E+00_JPRB, 0.24513E+00_JPRB, 0.23721E+00_JPRB, 0.23694E+00_JPRB, 0.23861E+00_JPRB, 0.24037E+00_JPRB, 0.24068E+00_JPRB ,&
&  0.24029E+00_JPRB, 0.23952E+00_JPRB, 0.23855E+00_JPRB, 0.23765E+00_JPRB, 0.23664E+00_JPRB, 0.23344E+00_JPRB, 0.22968E+00_JPRB ,&
&  0.22672E+00_JPRB, 0.22373E+00_JPRB, 0.22074E+00_JPRB, 0.21684E+00_JPRB, 0.21282E+00_JPRB, 0.20995E+00_JPRB, 0.20677E+00_JPRB ,&
&  0.20282E+00_JPRB, 0.19987E+00_JPRB, 0.19901E+00_JPRB, 0.19911E+00_JPRB, 0.19940E+00_JPRB, 0.19879E+00_JPRB, 0.19806E+00_JPRB ,&
&  0.19666E+00_JPRB, 0.19555E+00_JPRB, 0.19414E+00_JPRB, 0.19349E+00_JPRB, 0.18927E+00_JPRB, 0.18085E+00_JPRB, 0.17507E+00_JPRB ,&
&  0.17414E+00_JPRB, 0.17685E+00_JPRB, 0.18096E+00_JPRB, 0.18146E+00_JPRB, 0.18289E+00_JPRB, 0.18378E+00_JPRB, 0.18349E+00_JPRB ,&
&  0.18140E+00_JPRB, 0.17708E+00_JPRB, 0.17652E+00_JPRB, 0.17726E+00_JPRB, 0.17757E+00_JPRB, 0.17849E+00_JPRB, 0.18000E+00_JPRB ,&
&  0.18269E+00_JPRB, 0.18512E+00_JPRB, 0.18721E+00_JPRB, 0.18923E+00_JPRB, 0.19037E+00_JPRB, 0.18970E+00_JPRB, 0.18768E+00_JPRB ,&
&  0.18734E+00_JPRB, 0.19227E+00_JPRB, 0.20143E+00_JPRB, 0.21223E+00_JPRB, 0.21440E+00_JPRB, 0.21645E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.21935E+00_JPRB, 0.22224E+00_JPRB, 0.22479E+00_JPRB, 0.22629E+00_JPRB, 0.22748E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.23931E+00_JPRB, 0.24099E+00_JPRB, 0.24181E+00_JPRB, 0.24279E+00_JPRB, 0.24473E+00_JPRB, 0.24610E+00_JPRB, 0.24631E+00_JPRB ,&
&  0.24641E+00_JPRB, 0.24740E+00_JPRB, 0.24805E+00_JPRB, 0.24830E+00_JPRB, 0.24600E+00_JPRB, 0.24571E+00_JPRB, 0.24734E+00_JPRB ,&
&  0.24949E+00_JPRB, 0.25186E+00_JPRB, 0.25211E+00_JPRB, 0.25207E+00_JPRB, 0.25176E+00_JPRB, 0.25095E+00_JPRB, 0.25260E+00_JPRB ,&
&  0.25395E+00_JPRB, 0.25457E+00_JPRB, 0.25595E+00_JPRB, 0.25722E+00_JPRB, 0.25991E+00_JPRB, 0.26302E+00_JPRB, 0.26459E+00_JPRB ,&
&  0.26421E+00_JPRB, 0.26248E+00_JPRB, 0.25647E+00_JPRB, 0.25001E+00_JPRB, 0.25068E+00_JPRB, 0.25197E+00_JPRB, 0.25202E+00_JPRB ,&
&  0.25009E+00_JPRB, 0.24745E+00_JPRB, 0.24569E+00_JPRB, 0.24433E+00_JPRB, 0.24270E+00_JPRB, 0.23995E+00_JPRB, 0.23705E+00_JPRB ,&
&  0.23446E+00_JPRB, 0.23363E+00_JPRB, 0.23234E+00_JPRB, 0.23022E+00_JPRB, 0.22716E+00_JPRB, 0.22476E+00_JPRB, 0.22225E+00_JPRB ,&
&  0.21769E+00_JPRB, 0.21317E+00_JPRB, 0.21076E+00_JPRB, 0.21024E+00_JPRB, 0.21075E+00_JPRB, 0.21046E+00_JPRB, 0.20900E+00_JPRB ,&
&  0.20731E+00_JPRB, 0.20501E+00_JPRB, 0.20362E+00_JPRB, 0.20323E+00_JPRB, 0.20426E+00_JPRB, 0.20334E+00_JPRB, 0.19933E+00_JPRB ,&
&  0.19693E+00_JPRB, 0.19521E+00_JPRB, 0.19503E+00_JPRB, 0.19401E+00_JPRB, 0.19304E+00_JPRB, 0.19172E+00_JPRB, 0.19199E+00_JPRB ,&
&  0.19284E+00_JPRB, 0.19320E+00_JPRB, 0.19555E+00_JPRB, 0.19972E+00_JPRB, 0.20357E+00_JPRB, 0.20581E+00_JPRB, 0.20712E+00_JPRB ,&
&  0.20759E+00_JPRB, 0.20687E+00_JPRB, 0.20616E+00_JPRB, 0.20649E+00_JPRB, 0.20818E+00_JPRB, 0.21051E+00_JPRB, 0.21284E+00_JPRB ,&
&  0.21355E+00_JPRB, 0.21455E+00_JPRB, 0.21677E+00_JPRB, 0.22016E+00_JPRB, 0.22222E+00_JPRB, 0.22400E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.22649E+00_JPRB, 0.22793E+00_JPRB, 0.23150E+00_JPRB, 0.23386E+00_JPRB, 0.23632E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.24870E+00_JPRB, 0.24950E+00_JPRB, 0.25042E+00_JPRB, 0.25053E+00_JPRB, 0.25142E+00_JPRB, 0.25219E+00_JPRB, 0.25352E+00_JPRB ,&
&  0.25401E+00_JPRB, 0.25528E+00_JPRB, 0.25718E+00_JPRB, 0.25709E+00_JPRB, 0.25673E+00_JPRB, 0.25655E+00_JPRB, 0.25719E+00_JPRB ,&
&  0.25946E+00_JPRB, 0.26090E+00_JPRB, 0.26057E+00_JPRB, 0.26040E+00_JPRB, 0.25995E+00_JPRB, 0.25961E+00_JPRB, 0.26177E+00_JPRB ,&
&  0.26234E+00_JPRB, 0.26313E+00_JPRB, 0.26492E+00_JPRB, 0.26634E+00_JPRB, 0.26967E+00_JPRB, 0.27360E+00_JPRB, 0.27592E+00_JPRB ,&
&  0.27524E+00_JPRB, 0.27237E+00_JPRB, 0.26789E+00_JPRB, 0.26439E+00_JPRB, 0.26389E+00_JPRB, 0.26490E+00_JPRB, 0.26715E+00_JPRB ,&
&  0.26637E+00_JPRB, 0.26105E+00_JPRB, 0.25442E+00_JPRB, 0.25096E+00_JPRB, 0.24986E+00_JPRB, 0.24847E+00_JPRB, 0.24593E+00_JPRB ,&
&  0.24386E+00_JPRB, 0.24349E+00_JPRB, 0.24365E+00_JPRB, 0.24289E+00_JPRB, 0.24136E+00_JPRB, 0.23875E+00_JPRB, 0.23537E+00_JPRB ,&
&  0.23241E+00_JPRB, 0.22944E+00_JPRB, 0.22757E+00_JPRB, 0.22620E+00_JPRB, 0.22603E+00_JPRB, 0.22629E+00_JPRB, 0.22563E+00_JPRB ,&
&  0.22400E+00_JPRB, 0.22301E+00_JPRB, 0.22160E+00_JPRB, 0.21867E+00_JPRB, 0.21654E+00_JPRB, 0.21585E+00_JPRB, 0.21474E+00_JPRB ,&
&  0.21319E+00_JPRB, 0.21252E+00_JPRB, 0.21215E+00_JPRB, 0.21151E+00_JPRB, 0.21212E+00_JPRB, 0.21360E+00_JPRB, 0.21113E+00_JPRB ,&
&  0.20904E+00_JPRB, 0.20766E+00_JPRB, 0.20766E+00_JPRB, 0.20901E+00_JPRB, 0.21130E+00_JPRB, 0.21381E+00_JPRB, 0.21596E+00_JPRB ,&
&  0.21688E+00_JPRB, 0.21715E+00_JPRB, 0.21749E+00_JPRB, 0.21751E+00_JPRB, 0.21834E+00_JPRB, 0.22024E+00_JPRB, 0.22265E+00_JPRB ,&
&  0.22391E+00_JPRB, 0.22630E+00_JPRB, 0.22877E+00_JPRB, 0.23185E+00_JPRB, 0.23356E+00_JPRB, 0.23517E+00_JPRB, 0.23681E+00_JPRB ,&
&  0.23815E+00_JPRB, 0.23985E+00_JPRB, 0.24232E+00_JPRB, 0.24442E+00_JPRB, 0.24669E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.26082E+00_JPRB, 0.25994E+00_JPRB, 0.25952E+00_JPRB, 0.26010E+00_JPRB, 0.26223E+00_JPRB, 0.26282E+00_JPRB, 0.26274E+00_JPRB ,&
&  0.26242E+00_JPRB, 0.26410E+00_JPRB, 0.26739E+00_JPRB, 0.26937E+00_JPRB, 0.26952E+00_JPRB, 0.26928E+00_JPRB, 0.26898E+00_JPRB ,&
&  0.26902E+00_JPRB, 0.26906E+00_JPRB, 0.26817E+00_JPRB, 0.26827E+00_JPRB, 0.26856E+00_JPRB, 0.26804E+00_JPRB, 0.26910E+00_JPRB ,&
&  0.26936E+00_JPRB, 0.26984E+00_JPRB, 0.27202E+00_JPRB, 0.27373E+00_JPRB, 0.27714E+00_JPRB, 0.28223E+00_JPRB, 0.28421E+00_JPRB ,&
&  0.28310E+00_JPRB, 0.27947E+00_JPRB, 0.27658E+00_JPRB, 0.27729E+00_JPRB, 0.27882E+00_JPRB, 0.28188E+00_JPRB, 0.28509E+00_JPRB ,&
&  0.28566E+00_JPRB, 0.28028E+00_JPRB, 0.27224E+00_JPRB, 0.26691E+00_JPRB, 0.26373E+00_JPRB, 0.26202E+00_JPRB, 0.26055E+00_JPRB ,&
&  0.25835E+00_JPRB, 0.25657E+00_JPRB, 0.25565E+00_JPRB, 0.25546E+00_JPRB, 0.25509E+00_JPRB, 0.25241E+00_JPRB, 0.24948E+00_JPRB ,&
&  0.24561E+00_JPRB, 0.24289E+00_JPRB, 0.24060E+00_JPRB, 0.23978E+00_JPRB, 0.23987E+00_JPRB, 0.23969E+00_JPRB, 0.23881E+00_JPRB ,&
&  0.23766E+00_JPRB, 0.23736E+00_JPRB, 0.23710E+00_JPRB, 0.23608E+00_JPRB, 0.23480E+00_JPRB, 0.23447E+00_JPRB, 0.23342E+00_JPRB ,&
&  0.23117E+00_JPRB, 0.22972E+00_JPRB, 0.22953E+00_JPRB, 0.23023E+00_JPRB, 0.23004E+00_JPRB, 0.23067E+00_JPRB, 0.23205E+00_JPRB ,&
&  0.23109E+00_JPRB, 0.22868E+00_JPRB, 0.22750E+00_JPRB, 0.22806E+00_JPRB, 0.22978E+00_JPRB, 0.23200E+00_JPRB, 0.23310E+00_JPRB ,&
&  0.23411E+00_JPRB, 0.23408E+00_JPRB, 0.23493E+00_JPRB, 0.23616E+00_JPRB, 0.23755E+00_JPRB, 0.23821E+00_JPRB, 0.23844E+00_JPRB ,&
&  0.23881E+00_JPRB, 0.23927E+00_JPRB, 0.24109E+00_JPRB, 0.24404E+00_JPRB, 0.24666E+00_JPRB, 0.24963E+00_JPRB, 0.25196E+00_JPRB ,&
&  0.25353E+00_JPRB, 0.25527E+00_JPRB, 0.25655E+00_JPRB, 0.25760E+00_JPRB, 0.25913E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.27998E+00_JPRB, 0.27768E+00_JPRB, 0.27619E+00_JPRB, 0.27651E+00_JPRB, 0.27869E+00_JPRB, 0.27929E+00_JPRB, 0.27799E+00_JPRB ,&
&  0.27665E+00_JPRB, 0.27787E+00_JPRB, 0.28065E+00_JPRB, 0.28193E+00_JPRB, 0.28228E+00_JPRB, 0.28291E+00_JPRB, 0.28199E+00_JPRB ,&
&  0.27909E+00_JPRB, 0.27602E+00_JPRB, 0.27640E+00_JPRB, 0.27754E+00_JPRB, 0.27865E+00_JPRB, 0.27760E+00_JPRB, 0.27627E+00_JPRB ,&
&  0.27703E+00_JPRB, 0.27714E+00_JPRB, 0.27887E+00_JPRB, 0.28086E+00_JPRB, 0.28413E+00_JPRB, 0.28890E+00_JPRB, 0.29224E+00_JPRB ,&
&  0.29287E+00_JPRB, 0.29147E+00_JPRB, 0.29119E+00_JPRB, 0.29322E+00_JPRB, 0.29523E+00_JPRB, 0.30036E+00_JPRB, 0.30563E+00_JPRB ,&
&  0.30831E+00_JPRB, 0.30723E+00_JPRB, 0.29908E+00_JPRB, 0.29218E+00_JPRB, 0.28816E+00_JPRB, 0.28407E+00_JPRB, 0.27986E+00_JPRB ,&
&  0.27489E+00_JPRB, 0.27101E+00_JPRB, 0.26849E+00_JPRB, 0.26715E+00_JPRB, 0.26645E+00_JPRB, 0.26554E+00_JPRB, 0.26329E+00_JPRB ,&
&  0.26051E+00_JPRB, 0.25836E+00_JPRB, 0.25666E+00_JPRB, 0.25532E+00_JPRB, 0.25408E+00_JPRB, 0.25354E+00_JPRB, 0.25295E+00_JPRB ,&
&  0.25268E+00_JPRB, 0.25312E+00_JPRB, 0.25397E+00_JPRB, 0.25406E+00_JPRB, 0.25398E+00_JPRB, 0.25333E+00_JPRB, 0.25251E+00_JPRB ,&
&  0.25110E+00_JPRB, 0.24858E+00_JPRB, 0.24641E+00_JPRB, 0.24565E+00_JPRB, 0.24518E+00_JPRB, 0.24539E+00_JPRB, 0.24570E+00_JPRB ,&
&  0.24541E+00_JPRB, 0.24530E+00_JPRB, 0.24607E+00_JPRB, 0.24720E+00_JPRB, 0.24809E+00_JPRB, 0.24851E+00_JPRB, 0.24910E+00_JPRB ,&
&  0.25061E+00_JPRB, 0.25235E+00_JPRB, 0.25355E+00_JPRB, 0.25437E+00_JPRB, 0.25481E+00_JPRB, 0.25578E+00_JPRB, 0.25572E+00_JPRB ,&
&  0.25581E+00_JPRB, 0.25677E+00_JPRB, 0.25939E+00_JPRB, 0.26326E+00_JPRB, 0.26713E+00_JPRB, 0.26984E+00_JPRB, 0.27258E+00_JPRB ,&
&  0.27483E+00_JPRB, 0.27670E+00_JPRB, 0.27780E+00_JPRB, 0.27845E+00_JPRB, 0.27953E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.30001E+00_JPRB, 0.29798E+00_JPRB, 0.29661E+00_JPRB, 0.29527E+00_JPRB, 0.29588E+00_JPRB, 0.29528E+00_JPRB, 0.29308E+00_JPRB ,&
&  0.29147E+00_JPRB, 0.29277E+00_JPRB, 0.29512E+00_JPRB, 0.29596E+00_JPRB, 0.29518E+00_JPRB, 0.29386E+00_JPRB, 0.29126E+00_JPRB ,&
&  0.28766E+00_JPRB, 0.28587E+00_JPRB, 0.28555E+00_JPRB, 0.28605E+00_JPRB, 0.28603E+00_JPRB, 0.28470E+00_JPRB, 0.28383E+00_JPRB ,&
&  0.28510E+00_JPRB, 0.28659E+00_JPRB, 0.28691E+00_JPRB, 0.28917E+00_JPRB, 0.29262E+00_JPRB, 0.29849E+00_JPRB, 0.30404E+00_JPRB ,&
&  0.30577E+00_JPRB, 0.30730E+00_JPRB, 0.30160E+00_JPRB, 0.31131E+00_JPRB, 0.31584E+00_JPRB, 0.32250E+00_JPRB, 0.32749E+00_JPRB ,&
&  0.33191E+00_JPRB, 0.33236E+00_JPRB, 0.32487E+00_JPRB, 0.31818E+00_JPRB, 0.31380E+00_JPRB, 0.30962E+00_JPRB, 0.30269E+00_JPRB ,&
&  0.29522E+00_JPRB, 0.28837E+00_JPRB, 0.28399E+00_JPRB, 0.28166E+00_JPRB, 0.28132E+00_JPRB, 0.28138E+00_JPRB, 0.27944E+00_JPRB ,&
&  0.27742E+00_JPRB, 0.27545E+00_JPRB, 0.27456E+00_JPRB, 0.27363E+00_JPRB, 0.27234E+00_JPRB, 0.27136E+00_JPRB, 0.27114E+00_JPRB ,&
&  0.27123E+00_JPRB, 0.27120E+00_JPRB, 0.27141E+00_JPRB, 0.27212E+00_JPRB, 0.27248E+00_JPRB, 0.27175E+00_JPRB, 0.27008E+00_JPRB ,&
&  0.26816E+00_JPRB, 0.26640E+00_JPRB, 0.26476E+00_JPRB, 0.26315E+00_JPRB, 0.26174E+00_JPRB, 0.26076E+00_JPRB, 0.25993E+00_JPRB ,&
&  0.25901E+00_JPRB, 0.25929E+00_JPRB, 0.26059E+00_JPRB, 0.26130E+00_JPRB, 0.26160E+00_JPRB, 0.26252E+00_JPRB, 0.26424E+00_JPRB ,&
&  0.26593E+00_JPRB, 0.26779E+00_JPRB, 0.26910E+00_JPRB, 0.26951E+00_JPRB, 0.26877E+00_JPRB, 0.26854E+00_JPRB, 0.26902E+00_JPRB ,&
&  0.27053E+00_JPRB, 0.27306E+00_JPRB, 0.27693E+00_JPRB, 0.28224E+00_JPRB, 0.28805E+00_JPRB, 0.29227E+00_JPRB, 0.29616E+00_JPRB ,&
&  0.29923E+00_JPRB, 0.30021E+00_JPRB, 0.30018E+00_JPRB, 0.30110E+00_JPRB, 0.30144E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.31995E+00_JPRB, 0.31819E+00_JPRB, 0.31693E+00_JPRB, 0.31420E+00_JPRB, 0.31255E+00_JPRB, 0.31156E+00_JPRB, 0.31014E+00_JPRB ,&
&  0.30916E+00_JPRB, 0.30992E+00_JPRB, 0.30916E+00_JPRB, 0.30884E+00_JPRB, 0.30824E+00_JPRB, 0.30587E+00_JPRB, 0.30277E+00_JPRB ,&
&  0.30103E+00_JPRB, 0.29935E+00_JPRB, 0.29783E+00_JPRB, 0.29622E+00_JPRB, 0.29481E+00_JPRB, 0.29281E+00_JPRB, 0.29318E+00_JPRB ,&
&  0.29468E+00_JPRB, 0.29751E+00_JPRB, 0.30044E+00_JPRB, 0.30390E+00_JPRB, 0.30861E+00_JPRB, 0.31484E+00_JPRB, 0.32082E+00_JPRB ,&
&  0.32728E+00_JPRB, 0.31750E+00_JPRB, 0.32913E+00_JPRB, 0.33440E+00_JPRB, 0.34366E+00_JPRB, 0.35215E+00_JPRB, 0.35817E+00_JPRB ,&
&  0.36058E+00_JPRB, 0.35390E+00_JPRB, 0.34432E+00_JPRB, 0.33721E+00_JPRB, 0.33426E+00_JPRB, 0.33157E+00_JPRB, 0.32561E+00_JPRB ,&
&  0.31852E+00_JPRB, 0.31077E+00_JPRB, 0.30477E+00_JPRB, 0.30116E+00_JPRB, 0.29953E+00_JPRB, 0.29888E+00_JPRB, 0.29722E+00_JPRB ,&
&  0.29561E+00_JPRB, 0.29400E+00_JPRB, 0.29317E+00_JPRB, 0.29276E+00_JPRB, 0.29232E+00_JPRB, 0.29255E+00_JPRB, 0.29316E+00_JPRB ,&
&  0.29348E+00_JPRB, 0.29350E+00_JPRB, 0.29349E+00_JPRB, 0.29398E+00_JPRB, 0.29402E+00_JPRB, 0.29326E+00_JPRB, 0.29201E+00_JPRB ,&
&  0.29019E+00_JPRB, 0.28857E+00_JPRB, 0.28657E+00_JPRB, 0.28515E+00_JPRB, 0.28339E+00_JPRB, 0.28118E+00_JPRB, 0.27914E+00_JPRB ,&
&  0.27806E+00_JPRB, 0.27791E+00_JPRB, 0.27812E+00_JPRB, 0.27847E+00_JPRB, 0.27912E+00_JPRB, 0.28025E+00_JPRB, 0.28151E+00_JPRB ,&
&  0.28280E+00_JPRB, 0.28392E+00_JPRB, 0.28425E+00_JPRB, 0.28419E+00_JPRB, 0.28349E+00_JPRB, 0.28349E+00_JPRB, 0.28468E+00_JPRB ,&
&  0.28780E+00_JPRB, 0.29186E+00_JPRB, 0.29783E+00_JPRB, 0.30722E+00_JPRB, 0.31586E+00_JPRB, 0.32115E+00_JPRB, 0.32505E+00_JPRB ,&
&  0.32553E+00_JPRB, 0.32355E+00_JPRB, 0.32152E+00_JPRB, 0.32152E+00_JPRB, 0.32165E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.33413E+00_JPRB, 0.33318E+00_JPRB, 0.33234E+00_JPRB, 0.33076E+00_JPRB, 0.32942E+00_JPRB, 0.32846E+00_JPRB, 0.32606E+00_JPRB ,&
&  0.32576E+00_JPRB, 0.32327E+00_JPRB, 0.32075E+00_JPRB, 0.32007E+00_JPRB, 0.31891E+00_JPRB, 0.31686E+00_JPRB, 0.31508E+00_JPRB ,&
&  0.31296E+00_JPRB, 0.31070E+00_JPRB, 0.30938E+00_JPRB, 0.30749E+00_JPRB, 0.30457E+00_JPRB, 0.30243E+00_JPRB, 0.30309E+00_JPRB ,&
&  0.30422E+00_JPRB, 0.30692E+00_JPRB, 0.31416E+00_JPRB, 0.32155E+00_JPRB, 0.32879E+00_JPRB, 0.33536E+00_JPRB, 0.34227E+00_JPRB ,&
&  0.35185E+00_JPRB, 0.31782E+00_JPRB, 0.33665E+00_JPRB, 0.34796E+00_JPRB, 0.36576E+00_JPRB, 0.38708E+00_JPRB, 0.39388E+00_JPRB ,&
&  0.38931E+00_JPRB, 0.37570E+00_JPRB, 0.36302E+00_JPRB, 0.35365E+00_JPRB, 0.35063E+00_JPRB, 0.34786E+00_JPRB, 0.34298E+00_JPRB ,&
&  0.33788E+00_JPRB, 0.33091E+00_JPRB, 0.32420E+00_JPRB, 0.31960E+00_JPRB, 0.31683E+00_JPRB, 0.31468E+00_JPRB, 0.31250E+00_JPRB ,&
&  0.31128E+00_JPRB, 0.31102E+00_JPRB, 0.31091E+00_JPRB, 0.31080E+00_JPRB, 0.31134E+00_JPRB, 0.31261E+00_JPRB, 0.31394E+00_JPRB ,&
&  0.31515E+00_JPRB, 0.31568E+00_JPRB, 0.31653E+00_JPRB, 0.31740E+00_JPRB, 0.31845E+00_JPRB, 0.31921E+00_JPRB, 0.31901E+00_JPRB ,&
&  0.31750E+00_JPRB, 0.31545E+00_JPRB, 0.31332E+00_JPRB, 0.31103E+00_JPRB, 0.30828E+00_JPRB, 0.30500E+00_JPRB, 0.30229E+00_JPRB ,&
&  0.29991E+00_JPRB, 0.29806E+00_JPRB, 0.29506E+00_JPRB, 0.29438E+00_JPRB, 0.29507E+00_JPRB, 0.29580E+00_JPRB, 0.29665E+00_JPRB ,&
&  0.29739E+00_JPRB, 0.29727E+00_JPRB, 0.29670E+00_JPRB, 0.29613E+00_JPRB, 0.29587E+00_JPRB, 0.29641E+00_JPRB, 0.29807E+00_JPRB ,&
&  0.30093E+00_JPRB, 0.30630E+00_JPRB, 0.31374E+00_JPRB, 0.32503E+00_JPRB, 0.33462E+00_JPRB, 0.33987E+00_JPRB, 0.34235E+00_JPRB ,&
&  0.34057E+00_JPRB, 0.33717E+00_JPRB, 0.33452E+00_JPRB, 0.33284E+00_JPRB, 0.33392E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.35508E+00_JPRB, 0.35298E+00_JPRB, 0.35190E+00_JPRB, 0.34981E+00_JPRB, 0.34762E+00_JPRB, 0.34533E+00_JPRB, 0.34262E+00_JPRB ,&
&  0.33938E+00_JPRB, 0.33747E+00_JPRB, 0.33559E+00_JPRB, 0.33300E+00_JPRB, 0.33033E+00_JPRB, 0.32809E+00_JPRB, 0.32568E+00_JPRB ,&
&  0.32309E+00_JPRB, 0.32033E+00_JPRB, 0.31862E+00_JPRB, 0.31755E+00_JPRB, 0.31616E+00_JPRB, 0.31579E+00_JPRB, 0.31693E+00_JPRB ,&
&  0.31729E+00_JPRB, 0.31972E+00_JPRB, 0.32898E+00_JPRB, 0.33934E+00_JPRB, 0.35052E+00_JPRB, 0.35974E+00_JPRB, 0.37302E+00_JPRB ,&
&  0.35869E+00_JPRB, 0.32540E+00_JPRB, 0.34668E+00_JPRB, 0.37047E+00_JPRB, 0.39929E+00_JPRB, 0.43219E+00_JPRB, 0.43352E+00_JPRB ,&
&  0.42702E+00_JPRB, 0.41358E+00_JPRB, 0.39977E+00_JPRB, 0.38782E+00_JPRB, 0.38114E+00_JPRB, 0.37412E+00_JPRB, 0.36745E+00_JPRB ,&
&  0.36067E+00_JPRB, 0.35278E+00_JPRB, 0.34576E+00_JPRB, 0.33990E+00_JPRB, 0.33581E+00_JPRB, 0.33251E+00_JPRB, 0.32967E+00_JPRB ,&
&  0.32850E+00_JPRB, 0.32832E+00_JPRB, 0.32829E+00_JPRB, 0.32833E+00_JPRB, 0.32871E+00_JPRB, 0.33042E+00_JPRB, 0.33320E+00_JPRB ,&
&  0.33683E+00_JPRB, 0.34153E+00_JPRB, 0.34736E+00_JPRB, 0.35282E+00_JPRB, 0.35736E+00_JPRB, 0.35956E+00_JPRB, 0.35849E+00_JPRB ,&
&  0.35611E+00_JPRB, 0.35229E+00_JPRB, 0.34786E+00_JPRB, 0.34244E+00_JPRB, 0.33590E+00_JPRB, 0.32979E+00_JPRB, 0.32482E+00_JPRB ,&
&  0.32093E+00_JPRB, 0.31702E+00_JPRB, 0.31380E+00_JPRB, 0.31271E+00_JPRB, 0.31299E+00_JPRB, 0.31309E+00_JPRB, 0.31286E+00_JPRB ,&
&  0.31258E+00_JPRB, 0.31199E+00_JPRB, 0.31162E+00_JPRB, 0.31189E+00_JPRB, 0.31295E+00_JPRB, 0.31521E+00_JPRB, 0.31778E+00_JPRB ,&
&  0.32007E+00_JPRB, 0.32546E+00_JPRB, 0.33292E+00_JPRB, 0.34680E+00_JPRB, 0.35626E+00_JPRB, 0.36040E+00_JPRB, 0.36092E+00_JPRB ,&
&  0.35881E+00_JPRB, 0.35606E+00_JPRB, 0.35519E+00_JPRB, 0.35392E+00_JPRB, 0.35464E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.37983E+00_JPRB, 0.37967E+00_JPRB, 0.37594E+00_JPRB, 0.36828E+00_JPRB, 0.36239E+00_JPRB, 0.35812E+00_JPRB, 0.35540E+00_JPRB ,&
&  0.35401E+00_JPRB, 0.35279E+00_JPRB, 0.35089E+00_JPRB, 0.34693E+00_JPRB, 0.34359E+00_JPRB, 0.33922E+00_JPRB, 0.33465E+00_JPRB ,&
&  0.33093E+00_JPRB, 0.32751E+00_JPRB, 0.32575E+00_JPRB, 0.32320E+00_JPRB, 0.32205E+00_JPRB, 0.32333E+00_JPRB, 0.32536E+00_JPRB ,&
&  0.32808E+00_JPRB, 0.33080E+00_JPRB, 0.33928E+00_JPRB, 0.34889E+00_JPRB, 0.36551E+00_JPRB, 0.37682E+00_JPRB, 0.39153E+00_JPRB ,&
&  0.42425E+00_JPRB, 0.35959E+00_JPRB, 0.38681E+00_JPRB, 0.41685E+00_JPRB, 0.45291E+00_JPRB, 0.48828E+00_JPRB, 0.48173E+00_JPRB ,&
&  0.47160E+00_JPRB, 0.45968E+00_JPRB, 0.44765E+00_JPRB, 0.43480E+00_JPRB, 0.42439E+00_JPRB, 0.40954E+00_JPRB, 0.39435E+00_JPRB ,&
&  0.37918E+00_JPRB, 0.36889E+00_JPRB, 0.36135E+00_JPRB, 0.35442E+00_JPRB, 0.34842E+00_JPRB, 0.34466E+00_JPRB, 0.34290E+00_JPRB ,&
&  0.34277E+00_JPRB, 0.34285E+00_JPRB, 0.34248E+00_JPRB, 0.34254E+00_JPRB, 0.34333E+00_JPRB, 0.34615E+00_JPRB, 0.35167E+00_JPRB ,&
&  0.36233E+00_JPRB, 0.37599E+00_JPRB, 0.38835E+00_JPRB, 0.39739E+00_JPRB, 0.40263E+00_JPRB, 0.40177E+00_JPRB, 0.39753E+00_JPRB ,&
&  0.39015E+00_JPRB, 0.38064E+00_JPRB, 0.37184E+00_JPRB, 0.36388E+00_JPRB, 0.35679E+00_JPRB, 0.35061E+00_JPRB, 0.34427E+00_JPRB ,&
&  0.33949E+00_JPRB, 0.33620E+00_JPRB, 0.33482E+00_JPRB, 0.33395E+00_JPRB, 0.33289E+00_JPRB, 0.33194E+00_JPRB, 0.33193E+00_JPRB ,&
&  0.33173E+00_JPRB, 0.33156E+00_JPRB, 0.33166E+00_JPRB, 0.33299E+00_JPRB, 0.33539E+00_JPRB, 0.33758E+00_JPRB, 0.33922E+00_JPRB ,&
&  0.34064E+00_JPRB, 0.34531E+00_JPRB, 0.35048E+00_JPRB, 0.36699E+00_JPRB, 0.37358E+00_JPRB, 0.37591E+00_JPRB, 0.37737E+00_JPRB ,&
&  0.37845E+00_JPRB, 0.37703E+00_JPRB, 0.33833E+00_JPRB, 0.36273E+00_JPRB, 0.37421E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.41193E+00_JPRB, 0.40822E+00_JPRB, 0.39898E+00_JPRB, 0.38903E+00_JPRB, 0.38322E+00_JPRB, 0.37990E+00_JPRB, 0.37774E+00_JPRB ,&
&  0.37481E+00_JPRB, 0.37104E+00_JPRB, 0.36699E+00_JPRB, 0.36465E+00_JPRB, 0.36158E+00_JPRB, 0.35707E+00_JPRB, 0.35152E+00_JPRB ,&
&  0.34638E+00_JPRB, 0.34152E+00_JPRB, 0.33788E+00_JPRB, 0.33362E+00_JPRB, 0.33022E+00_JPRB, 0.33061E+00_JPRB, 0.33238E+00_JPRB ,&
&  0.33782E+00_JPRB, 0.34537E+00_JPRB, 0.35242E+00_JPRB, 0.36605E+00_JPRB, 0.38099E+00_JPRB, 0.40082E+00_JPRB, 0.41921E+00_JPRB ,&
&  0.45276E+00_JPRB, 0.38929E+00_JPRB, 0.41111E+00_JPRB, 0.45302E+00_JPRB, 0.48925E+00_JPRB, 0.52244E+00_JPRB, 0.52627E+00_JPRB ,&
&  0.52129E+00_JPRB, 0.51326E+00_JPRB, 0.50385E+00_JPRB, 0.49267E+00_JPRB, 0.47336E+00_JPRB, 0.44207E+00_JPRB, 0.41268E+00_JPRB ,&
&  0.39215E+00_JPRB, 0.38448E+00_JPRB, 0.38049E+00_JPRB, 0.37601E+00_JPRB, 0.37198E+00_JPRB, 0.37048E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.37184E+00_JPRB, 0.37170E+00_JPRB, 0.37071E+00_JPRB, 0.36930E+00_JPRB, 0.36939E+00_JPRB, 0.37664E+00_JPRB, 0.39302E+00_JPRB ,&
&  0.41762E+00_JPRB, 0.44087E+00_JPRB, 0.45741E+00_JPRB, 0.46674E+00_JPRB, 0.46531E+00_JPRB, 0.45528E+00_JPRB, 0.43817E+00_JPRB ,&
&  0.41920E+00_JPRB, 0.40338E+00_JPRB, 0.39256E+00_JPRB, 0.38405E+00_JPRB, 0.37718E+00_JPRB, 0.37166E+00_JPRB, 0.36828E+00_JPRB ,&
&  0.36694E+00_JPRB, 0.36634E+00_JPRB, 0.36551E+00_JPRB, 0.36350E+00_JPRB, 0.36227E+00_JPRB, 0.36136E+00_JPRB, 0.36118E+00_JPRB ,&
&  0.36096E+00_JPRB, 0.36060E+00_JPRB, 0.36123E+00_JPRB, 0.36263E+00_JPRB, 0.36351E+00_JPRB, 0.36345E+00_JPRB, 0.36376E+00_JPRB ,&
&  0.36670E+00_JPRB, 0.37319E+00_JPRB, 0.38257E+00_JPRB, 0.40664E+00_JPRB, 0.41758E+00_JPRB, 0.41120E+00_JPRB, 0.40563E+00_JPRB ,&
&  0.40407E+00_JPRB, 0.40176E+00_JPRB, 0.40406E+00_JPRB, 0.38185E+00_JPRB, 0.41030E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.43636E+00_JPRB, 0.43590E+00_JPRB, 0.42657E+00_JPRB, 0.41515E+00_JPRB, 0.40465E+00_JPRB, 0.39818E+00_JPRB, 0.39291E+00_JPRB ,&
&  0.38952E+00_JPRB, 0.38733E+00_JPRB, 0.38575E+00_JPRB, 0.38382E+00_JPRB, 0.38063E+00_JPRB, 0.37553E+00_JPRB, 0.36882E+00_JPRB ,&
&  0.36281E+00_JPRB, 0.35677E+00_JPRB, 0.35189E+00_JPRB, 0.34626E+00_JPRB, 0.33921E+00_JPRB, 0.33599E+00_JPRB, 0.33878E+00_JPRB ,&
&  0.34452E+00_JPRB, 0.35166E+00_JPRB, 0.35841E+00_JPRB, 0.36874E+00_JPRB, 0.38991E+00_JPRB, 0.40941E+00_JPRB, 0.42736E+00_JPRB ,&
&  0.46430E+00_JPRB, 0.43049E+00_JPRB, 0.45424E+00_JPRB, 0.48534E+00_JPRB, 0.52512E+00_JPRB, 0.56512E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.59203E+00_JPRB, 0.58940E+00_JPRB, 0.57324E+00_JPRB, 0.54404E+00_JPRB, 0.50357E+00_JPRB, 0.46988E+00_JPRB, 0.44962E+00_JPRB ,&
&  0.43810E+00_JPRB, 0.43187E+00_JPRB, 0.42620E+00_JPRB, 0.42180E+00_JPRB, 0.41945E+00_JPRB, 0.41756E+00_JPRB, 0.41570E+00_JPRB ,&
&  0.41378E+00_JPRB, 0.41214E+00_JPRB, 0.41096E+00_JPRB, 0.41093E+00_JPRB, 0.41612E+00_JPRB, 0.43110E+00_JPRB, 0.45248E+00_JPRB ,&
&  0.48342E+00_JPRB, 0.50598E+00_JPRB, 0.51237E+00_JPRB, 0.51602E+00_JPRB, 0.51721E+00_JPRB, 0.50494E+00_JPRB, 0.47637E+00_JPRB ,&
&  0.45046E+00_JPRB, 0.42912E+00_JPRB, 0.41425E+00_JPRB, 0.40424E+00_JPRB, 0.39801E+00_JPRB, 0.39501E+00_JPRB, 0.39489E+00_JPRB ,&
&  0.39568E+00_JPRB, 0.39598E+00_JPRB, 0.39705E+00_JPRB, 0.39743E+00_JPRB, 0.39792E+00_JPRB, 0.39715E+00_JPRB, 0.39388E+00_JPRB ,&
&  0.38948E+00_JPRB, 0.38658E+00_JPRB, 0.38463E+00_JPRB, 0.38328E+00_JPRB, 0.38263E+00_JPRB, 0.38126E+00_JPRB, 0.38054E+00_JPRB ,&
&  0.38228E+00_JPRB, 0.39500E+00_JPRB, 0.41408E+00_JPRB, 0.45281E+00_JPRB, 0.44814E+00_JPRB, 0.43809E+00_JPRB, 0.42647E+00_JPRB ,&
&  0.42121E+00_JPRB, 0.42198E+00_JPRB, 0.41739E+00_JPRB, 0.42219E+00_JPRB, 0.43565E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.49389E+00_JPRB, 0.48039E+00_JPRB, 0.46502E+00_JPRB, 0.44672E+00_JPRB, 0.43047E+00_JPRB, 0.41867E+00_JPRB, 0.41011E+00_JPRB ,&
&  0.40469E+00_JPRB, 0.40154E+00_JPRB, 0.39952E+00_JPRB, 0.39809E+00_JPRB, 0.39595E+00_JPRB, 0.39349E+00_JPRB, 0.38741E+00_JPRB ,&
&  0.38009E+00_JPRB, 0.37175E+00_JPRB, 0.36539E+00_JPRB, 0.35979E+00_JPRB, 0.34938E+00_JPRB, 0.34382E+00_JPRB, 0.34611E+00_JPRB ,&
&  0.35121E+00_JPRB, 0.35759E+00_JPRB, 0.36264E+00_JPRB, 0.37347E+00_JPRB, 0.39259E+00_JPRB, 0.41085E+00_JPRB, 0.43337E+00_JPRB ,&
&  0.47695E+00_JPRB, 0.46692E+00_JPRB, 0.51790E+00_JPRB, 0.55024E+00_JPRB, 0.59405E+00_JPRB, 0.64711E+00_JPRB, 0.66945E+00_JPRB ,&
&  0.67462E+00_JPRB, 0.70021E+00_JPRB, 0.68171E+00_JPRB, 0.62381E+00_JPRB, 0.57039E+00_JPRB, 0.53257E+00_JPRB, 0.51139E+00_JPRB ,&
&  0.49520E+00_JPRB, 0.48149E+00_JPRB, 0.46905E+00_JPRB, 0.46117E+00_JPRB, 0.45358E+00_JPRB, 0.44617E+00_JPRB, 0.43923E+00_JPRB ,&
&  0.43563E+00_JPRB, 0.43323E+00_JPRB, 0.43248E+00_JPRB, 0.43968E+00_JPRB, 0.45884E+00_JPRB, 0.48935E+00_JPRB, 0.52474E+00_JPRB ,&
&  0.55268E+00_JPRB, 0.56453E+00_JPRB, 0.57440E+00_JPRB, 0.56780E+00_JPRB, 0.54957E+00_JPRB, 0.53109E+00_JPRB, 0.51261E+00_JPRB ,&
&  0.49052E+00_JPRB, 0.46610E+00_JPRB, 0.44872E+00_JPRB, 0.43710E+00_JPRB, 0.43041E+00_JPRB, 0.42759E+00_JPRB, 0.42686E+00_JPRB ,&
&  0.42568E+00_JPRB, 0.42643E+00_JPRB, 0.43145E+00_JPRB, 0.43733E+00_JPRB, 0.43761E+00_JPRB, 0.43232E+00_JPRB, 0.42346E+00_JPRB ,&
&  0.41553E+00_JPRB, 0.40883E+00_JPRB, 0.40667E+00_JPRB, 0.40670E+00_JPRB, 0.40788E+00_JPRB, 0.41026E+00_JPRB, 0.41096E+00_JPRB ,&
&  0.41410E+00_JPRB, 0.43589E+00_JPRB, 0.44991E+00_JPRB, 0.48873E+00_JPRB, 0.51920E+00_JPRB, 0.50045E+00_JPRB, 0.47708E+00_JPRB ,&
&  0.46015E+00_JPRB, 0.45201E+00_JPRB, 0.44184E+00_JPRB, 0.44594E+00_JPRB, 0.50961E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.47264E+00_JPRB, 0.46475E+00_JPRB, 0.45615E+00_JPRB, 0.44858E+00_JPRB, 0.43759E+00_JPRB, 0.42687E+00_JPRB, 0.41630E+00_JPRB ,&
&  0.40843E+00_JPRB, 0.40444E+00_JPRB, 0.40490E+00_JPRB, 0.40607E+00_JPRB, 0.40449E+00_JPRB, 0.39828E+00_JPRB, 0.38650E+00_JPRB ,&
&  0.37885E+00_JPRB, 0.37326E+00_JPRB, 0.36687E+00_JPRB, 0.35887E+00_JPRB, 0.35113E+00_JPRB, 0.34852E+00_JPRB, 0.34979E+00_JPRB ,&
&  0.35464E+00_JPRB, 0.36095E+00_JPRB, 0.36550E+00_JPRB, 0.37733E+00_JPRB, 0.39300E+00_JPRB, 0.41157E+00_JPRB, 0.43736E+00_JPRB ,&
&  0.47614E+00_JPRB, 0.54364E+00_JPRB, 0.60365E+00_JPRB, 0.63598E+00_JPRB, 0.76255E+00_JPRB, 0.76446E+00_JPRB, 0.78631E+00_JPRB ,&
&  0.81184E+00_JPRB, 0.82820E+00_JPRB, 0.79863E+00_JPRB, 0.71997E+00_JPRB, 0.65622E+00_JPRB, 0.61297E+00_JPRB, 0.57150E+00_JPRB ,&
&  0.53242E+00_JPRB, 0.50345E+00_JPRB, 0.48079E+00_JPRB, 0.46448E+00_JPRB, 0.45206E+00_JPRB, 0.44323E+00_JPRB, 0.43877E+00_JPRB ,&
&  0.43884E+00_JPRB, 0.44112E+00_JPRB, 0.45010E+00_JPRB, 0.46843E+00_JPRB, 0.52503E+00_JPRB, 0.58401E+00_JPRB, 0.62334E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.64581E+00_JPRB, 0.65062E+00_JPRB, 0.62937E+00_JPRB, 0.59798E+00_JPRB, 0.56069E+00_JPRB, 0.53172E+00_JPRB ,&
&  0.51219E+00_JPRB, 0.49367E+00_JPRB, 0.47691E+00_JPRB, 0.46153E+00_JPRB, 0.45185E+00_JPRB, 0.44828E+00_JPRB, 0.44769E+00_JPRB ,&
&  0.44603E+00_JPRB, 0.44665E+00_JPRB, 0.44836E+00_JPRB, 0.44904E+00_JPRB, 0.44769E+00_JPRB, 0.44471E+00_JPRB, 0.43982E+00_JPRB ,&
&  0.43370E+00_JPRB, 0.42894E+00_JPRB, 0.43470E+00_JPRB, 0.43806E+00_JPRB, 0.44077E+00_JPRB, 0.44012E+00_JPRB, 0.43675E+00_JPRB ,&
&  0.43941E+00_JPRB, 0.46072E+00_JPRB, 0.46820E+00_JPRB, 0.48971E+00_JPRB, 0.50856E+00_JPRB, 0.51834E+00_JPRB, 0.49654E+00_JPRB ,&
&  0.48180E+00_JPRB, 0.46457E+00_JPRB, 0.45060E+00_JPRB, 0.45006E+00_JPRB, 0.47535E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.46192E+00_JPRB, 0.45740E+00_JPRB, 0.45015E+00_JPRB, 0.44278E+00_JPRB, 0.43363E+00_JPRB, 0.42338E+00_JPRB, 0.41593E+00_JPRB ,&
&  0.41176E+00_JPRB, 0.41178E+00_JPRB, 0.41329E+00_JPRB, 0.40545E+00_JPRB, 0.39390E+00_JPRB, 0.38304E+00_JPRB, 0.37654E+00_JPRB ,&
&  0.37180E+00_JPRB, 0.36704E+00_JPRB, 0.36096E+00_JPRB, 0.35310E+00_JPRB, 0.34980E+00_JPRB, 0.34912E+00_JPRB, 0.34921E+00_JPRB ,&
&  0.35457E+00_JPRB, 0.35973E+00_JPRB, 0.36652E+00_JPRB, 0.37556E+00_JPRB, 0.38922E+00_JPRB, 0.40763E+00_JPRB, 0.43690E+00_JPRB ,&
&  0.49325E+00_JPRB, 0.55526E+00_JPRB, 0.73129E+00_JPRB, 0.73317E+00_JPRB, 0.80319E+00_JPRB, 0.80517E+00_JPRB, 0.89926E+00_JPRB ,&
&  0.96316E+00_JPRB, 0.95887E+00_JPRB, 0.88507E+00_JPRB, 0.78886E+00_JPRB, 0.70334E+00_JPRB, 0.64015E+00_JPRB, 0.59584E+00_JPRB ,&
&  0.55411E+00_JPRB, 0.51753E+00_JPRB, 0.49213E+00_JPRB, 0.47423E+00_JPRB, 0.46172E+00_JPRB, 0.45390E+00_JPRB, 0.45086E+00_JPRB ,&
&  0.45392E+00_JPRB, 0.46508E+00_JPRB, 0.48782E+00_JPRB, 0.53185E+00_JPRB, 0.60225E+00_JPRB, 0.59118E+00_JPRB, 0.62647E+00_JPRB ,&
&  0.71823E+00_JPRB, 0.71950E+00_JPRB, 0.70082E+00_JPRB, 0.66144E+00_JPRB, 0.62123E+00_JPRB, 0.58434E+00_JPRB, 0.54989E+00_JPRB ,&
&  0.52423E+00_JPRB, 0.50458E+00_JPRB, 0.48717E+00_JPRB, 0.47569E+00_JPRB, 0.47024E+00_JPRB, 0.47064E+00_JPRB, 0.47154E+00_JPRB ,&
&  0.46992E+00_JPRB, 0.46738E+00_JPRB, 0.46660E+00_JPRB, 0.46753E+00_JPRB, 0.46848E+00_JPRB, 0.46956E+00_JPRB, 0.46285E+00_JPRB ,&
&  0.45729E+00_JPRB, 0.45666E+00_JPRB, 0.46681E+00_JPRB, 0.45666E+00_JPRB, 0.45705E+00_JPRB, 0.45728E+00_JPRB, 0.45186E+00_JPRB ,&
&  0.44422E+00_JPRB, 0.45043E+00_JPRB, 0.46404E+00_JPRB, 0.48070E+00_JPRB, 0.51511E+00_JPRB, 0.53655E+00_JPRB, 0.49899E+00_JPRB ,&
&  0.48986E+00_JPRB, 0.47485E+00_JPRB, 0.46082E+00_JPRB, 0.45854E+00_JPRB, 0.46874E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.45323E+00_JPRB, 0.44800E+00_JPRB, 0.44188E+00_JPRB, 0.43547E+00_JPRB, 0.42730E+00_JPRB, 0.41954E+00_JPRB, 0.41490E+00_JPRB ,&
&  0.41451E+00_JPRB, 0.41757E+00_JPRB, 0.41512E+00_JPRB, 0.40452E+00_JPRB, 0.39130E+00_JPRB, 0.37926E+00_JPRB, 0.37331E+00_JPRB ,&
&  0.36844E+00_JPRB, 0.36151E+00_JPRB, 0.35241E+00_JPRB, 0.34286E+00_JPRB, 0.34014E+00_JPRB, 0.34271E+00_JPRB, 0.33988E+00_JPRB ,&
&  0.34579E+00_JPRB, 0.35278E+00_JPRB, 0.35998E+00_JPRB, 0.36668E+00_JPRB, 0.38023E+00_JPRB, 0.40001E+00_JPRB, 0.43119E+00_JPRB ,&
&  0.49146E+00_JPRB, 0.42924E+00_JPRB, 0.68547E+00_JPRB, 0.83548E+00_JPRB, 0.91976E+00_JPRB, 0.95172E+00_JPRB, 0.10446E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.10742E+01_JPRB, 0.97999E+00_JPRB, 0.84935E+00_JPRB, 0.73036E+00_JPRB, 0.64892E+00_JPRB, 0.59721E+00_JPRB ,&
&  0.56360E+00_JPRB, 0.53848E+00_JPRB, 0.51526E+00_JPRB, 0.49470E+00_JPRB, 0.48013E+00_JPRB, 0.47167E+00_JPRB, 0.47094E+00_JPRB ,&
&  0.48701E+00_JPRB, 0.51846E+00_JPRB, 0.55864E+00_JPRB, 0.63763E+00_JPRB, 0.65819E+00_JPRB, 0.64557E+00_JPRB, 0.63885E+00_JPRB ,&
&  0.69802E+00_JPRB, 0.77756E+00_JPRB, 0.72392E+00_JPRB, 0.65862E+00_JPRB, 0.62454E+00_JPRB, 0.59164E+00_JPRB, 0.55795E+00_JPRB ,&
&  0.52858E+00_JPRB, 0.50958E+00_JPRB, 0.49716E+00_JPRB, 0.49022E+00_JPRB, 0.48876E+00_JPRB, 0.48642E+00_JPRB, 0.47947E+00_JPRB ,&
&  0.47491E+00_JPRB, 0.47311E+00_JPRB, 0.47403E+00_JPRB, 0.47710E+00_JPRB, 0.47744E+00_JPRB, 0.47822E+00_JPRB, 0.47689E+00_JPRB ,&
&  0.47516E+00_JPRB, 0.47637E+00_JPRB, 0.47086E+00_JPRB, 0.45843E+00_JPRB, 0.45520E+00_JPRB, 0.45040E+00_JPRB, 0.44683E+00_JPRB ,&
&  0.45182E+00_JPRB, 0.45761E+00_JPRB, 0.46630E+00_JPRB, 0.48923E+00_JPRB, 0.50137E+00_JPRB, 0.52603E+00_JPRB, 0.49489E+00_JPRB ,&
&  0.49353E+00_JPRB, 0.48443E+00_JPRB, 0.47647E+00_JPRB, 0.46633E+00_JPRB, 0.46356E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.45215E+00_JPRB, 0.44576E+00_JPRB, 0.43951E+00_JPRB, 0.43173E+00_JPRB, 0.42196E+00_JPRB, 0.41602E+00_JPRB, 0.41820E+00_JPRB ,&
&  0.42383E+00_JPRB, 0.42091E+00_JPRB, 0.41540E+00_JPRB, 0.40530E+00_JPRB, 0.38980E+00_JPRB, 0.38274E+00_JPRB, 0.37837E+00_JPRB ,&
&  0.37067E+00_JPRB, 0.36497E+00_JPRB, 0.35518E+00_JPRB, 0.35010E+00_JPRB, 0.34840E+00_JPRB, 0.34508E+00_JPRB, 0.33862E+00_JPRB ,&
&  0.34164E+00_JPRB, 0.34854E+00_JPRB, 0.35691E+00_JPRB, 0.36451E+00_JPRB, 0.37903E+00_JPRB, 0.39976E+00_JPRB, 0.43209E+00_JPRB ,&
&  0.49590E+00_JPRB, 0.69750E+00_JPRB, 0.62024E+00_JPRB, 0.10994E+01_JPRB, 0.10915E+01_JPRB, 0.11238E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.11078E+01_JPRB, 0.10543E+01_JPRB, 0.89791E+00_JPRB, 0.74895E+00_JPRB, 0.65912E+00_JPRB, 0.60174E+00_JPRB ,&
&  0.56172E+00_JPRB, 0.53030E+00_JPRB, 0.50668E+00_JPRB, 0.49040E+00_JPRB, 0.47748E+00_JPRB, 0.47548E+00_JPRB, 0.50096E+00_JPRB ,&
&  0.54968E+00_JPRB, 0.59331E+00_JPRB, 0.66723E+00_JPRB, 0.73858E+00_JPRB, 0.68339E+00_JPRB, 0.66107E+00_JPRB, 0.67884E+00_JPRB ,&
&  0.70856E+00_JPRB, 0.77470E+00_JPRB, 0.67912E+00_JPRB, 0.62919E+00_JPRB, 0.60475E+00_JPRB, 0.58862E+00_JPRB, 0.56103E+00_JPRB ,&
&  0.53502E+00_JPRB, 0.52166E+00_JPRB, 0.51369E+00_JPRB, 0.50929E+00_JPRB, 0.50306E+00_JPRB, 0.49424E+00_JPRB, 0.48476E+00_JPRB ,&
&  0.47644E+00_JPRB, 0.47929E+00_JPRB, 0.48274E+00_JPRB, 0.48464E+00_JPRB, 0.48616E+00_JPRB, 0.48465E+00_JPRB, 0.48351E+00_JPRB ,&
&  0.48520E+00_JPRB, 0.48860E+00_JPRB, 0.46785E+00_JPRB, 0.45183E+00_JPRB, 0.44564E+00_JPRB, 0.44194E+00_JPRB, 0.44012E+00_JPRB ,&
&  0.45262E+00_JPRB, 0.46421E+00_JPRB, 0.47446E+00_JPRB, 0.49298E+00_JPRB, 0.51822E+00_JPRB, 0.54640E+00_JPRB, 0.50654E+00_JPRB ,&
&  0.51154E+00_JPRB, 0.50139E+00_JPRB, 0.48949E+00_JPRB, 0.47143E+00_JPRB, 0.46245E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.45447E+00_JPRB, 0.44514E+00_JPRB, 0.43906E+00_JPRB, 0.43272E+00_JPRB, 0.42677E+00_JPRB, 0.42515E+00_JPRB, 0.42794E+00_JPRB ,&
&  0.42858E+00_JPRB, 0.42284E+00_JPRB, 0.41309E+00_JPRB, 0.40325E+00_JPRB, 0.38753E+00_JPRB, 0.37860E+00_JPRB, 0.37437E+00_JPRB ,&
&  0.37119E+00_JPRB, 0.37180E+00_JPRB, 0.36802E+00_JPRB, 0.36499E+00_JPRB, 0.36229E+00_JPRB, 0.35690E+00_JPRB, 0.35573E+00_JPRB ,&
&  0.35389E+00_JPRB, 0.35688E+00_JPRB, 0.36367E+00_JPRB, 0.37578E+00_JPRB, 0.39360E+00_JPRB, 0.41954E+00_JPRB, 0.45128E+00_JPRB ,&
&  0.52875E+00_JPRB, 0.64634E+00_JPRB, 0.65583E+00_JPRB, 0.13225E+01_JPRB, 0.12940E+01_JPRB, 0.12238E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.11025E+01_JPRB, 0.10571E+01_JPRB, 0.92780E+00_JPRB, 0.78945E+00_JPRB, 0.68358E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.54330E+00_JPRB, 0.50903E+00_JPRB, 0.49206E+00_JPRB, 0.48468E+00_JPRB, 0.48362E+00_JPRB, 0.51390E+00_JPRB, 0.57455E+00_JPRB ,&
&  0.62401E+00_JPRB, 0.67974E+00_JPRB, 0.78903E+00_JPRB, 0.74821E+00_JPRB, 0.67521E+00_JPRB, 0.66069E+00_JPRB, 0.66902E+00_JPRB ,&
&  0.69347E+00_JPRB, 0.68654E+00_JPRB, 0.62852E+00_JPRB, 0.59625E+00_JPRB, 0.56864E+00_JPRB, 0.57204E+00_JPRB, 0.55725E+00_JPRB ,&
&  0.53991E+00_JPRB, 0.53020E+00_JPRB, 0.52198E+00_JPRB, 0.51540E+00_JPRB, 0.50586E+00_JPRB, 0.49769E+00_JPRB, 0.48391E+00_JPRB ,&
&  0.48326E+00_JPRB, 0.48497E+00_JPRB, 0.48537E+00_JPRB, 0.48970E+00_JPRB, 0.48531E+00_JPRB, 0.48420E+00_JPRB, 0.48452E+00_JPRB ,&
&  0.49127E+00_JPRB, 0.49151E+00_JPRB, 0.46399E+00_JPRB, 0.44852E+00_JPRB, 0.44527E+00_JPRB, 0.44249E+00_JPRB, 0.44260E+00_JPRB ,&
&  0.45782E+00_JPRB, 0.47562E+00_JPRB, 0.48746E+00_JPRB, 0.49442E+00_JPRB, 0.52289E+00_JPRB, 0.53696E+00_JPRB, 0.52926E+00_JPRB ,&
&  0.52181E+00_JPRB, 0.51431E+00_JPRB, 0.50280E+00_JPRB, 0.47759E+00_JPRB, 0.46625E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.45331E+00_JPRB, 0.44203E+00_JPRB, 0.43294E+00_JPRB, 0.43030E+00_JPRB, 0.42624E+00_JPRB, 0.42344E+00_JPRB, 0.42942E+00_JPRB ,&
&  0.42503E+00_JPRB, 0.41527E+00_JPRB, 0.39969E+00_JPRB, 0.38314E+00_JPRB, 0.36977E+00_JPRB, 0.37145E+00_JPRB, 0.37633E+00_JPRB ,&
&  0.37684E+00_JPRB, 0.37528E+00_JPRB, 0.37384E+00_JPRB, 0.37476E+00_JPRB, 0.37320E+00_JPRB, 0.37243E+00_JPRB, 0.37139E+00_JPRB ,&
&  0.37112E+00_JPRB, 0.37522E+00_JPRB, 0.38849E+00_JPRB, 0.41035E+00_JPRB, 0.43600E+00_JPRB, 0.47355E+00_JPRB, 0.52332E+00_JPRB ,&
&  0.62675E+00_JPRB, 0.68531E+00_JPRB, 0.75264E+00_JPRB, 0.11853E+01_JPRB, 0.12638E+01_JPRB, 0.11469E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.93308E+00_JPRB, 0.87310E+00_JPRB, 0.10125E+01_JPRB, 0.89775E+00_JPRB, 0.77582E+00_JPRB, 0.66095E+00_JPRB, 0.57924E+00_JPRB ,&
&  0.53241E+00_JPRB, 0.50068E+00_JPRB, 0.49235E+00_JPRB, 0.50067E+00_JPRB, 0.53714E+00_JPRB, 0.60373E+00_JPRB, 0.66801E+00_JPRB ,&
&  0.71173E+00_JPRB, 0.79477E+00_JPRB, 0.91605E+00_JPRB, 0.77846E+00_JPRB, 0.64154E+00_JPRB, 0.62210E+00_JPRB, 0.61094E+00_JPRB ,&
&  0.63712E+00_JPRB, 0.62018E+00_JPRB, 0.60004E+00_JPRB, 0.57687E+00_JPRB, 0.54492E+00_JPRB, 0.57227E+00_JPRB, 0.55063E+00_JPRB ,&
&  0.54024E+00_JPRB, 0.53083E+00_JPRB, 0.52107E+00_JPRB, 0.51400E+00_JPRB, 0.50127E+00_JPRB, 0.48229E+00_JPRB, 0.47096E+00_JPRB ,&
&  0.47236E+00_JPRB, 0.47858E+00_JPRB, 0.47735E+00_JPRB, 0.47782E+00_JPRB, 0.47341E+00_JPRB, 0.47573E+00_JPRB, 0.48074E+00_JPRB ,&
&  0.49133E+00_JPRB, 0.49474E+00_JPRB, 0.47589E+00_JPRB, 0.46919E+00_JPRB, 0.46693E+00_JPRB, 0.46611E+00_JPRB, 0.46448E+00_JPRB ,&
&  0.47433E+00_JPRB, 0.49578E+00_JPRB, 0.51040E+00_JPRB, 0.52398E+00_JPRB, 0.54000E+00_JPRB, 0.54638E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54707E+00_JPRB, 0.56479E+00_JPRB, 0.56757E+00_JPRB, 0.50205E+00_JPRB, 0.47380E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.43442E+00_JPRB, 0.42476E+00_JPRB, 0.42103E+00_JPRB, 0.42077E+00_JPRB, 0.41875E+00_JPRB, 0.42106E+00_JPRB, 0.42484E+00_JPRB ,&
&  0.40730E+00_JPRB, 0.38847E+00_JPRB, 0.37543E+00_JPRB, 0.37044E+00_JPRB, 0.36963E+00_JPRB, 0.38050E+00_JPRB, 0.38421E+00_JPRB ,&
&  0.38097E+00_JPRB, 0.37974E+00_JPRB, 0.38985E+00_JPRB, 0.39149E+00_JPRB, 0.39230E+00_JPRB, 0.39083E+00_JPRB, 0.39148E+00_JPRB ,&
&  0.39978E+00_JPRB, 0.41550E+00_JPRB, 0.44560E+00_JPRB, 0.48931E+00_JPRB, 0.53369E+00_JPRB, 0.60106E+00_JPRB, 0.67073E+00_JPRB ,&
&  0.78290E+00_JPRB, 0.79327E+00_JPRB, 0.44155E+00_JPRB, 0.72947E+00_JPRB, 0.97846E+00_JPRB, 0.98321E+00_JPRB, 0.89836E+00_JPRB ,&
&  0.82826E+00_JPRB, 0.77395E+00_JPRB, 0.84611E+00_JPRB, 0.84049E+00_JPRB, 0.71323E+00_JPRB, 0.63046E+00_JPRB, 0.56668E+00_JPRB ,&
&  0.52939E+00_JPRB, 0.51722E+00_JPRB, 0.52701E+00_JPRB, 0.56568E+00_JPRB, 0.63896E+00_JPRB, 0.72123E+00_JPRB, 0.77008E+00_JPRB ,&
&  0.82514E+00_JPRB, 0.95069E+00_JPRB, 0.10209E+01_JPRB, 0.76205E+00_JPRB, 0.61854E+00_JPRB, 0.58474E+00_JPRB, 0.55940E+00_JPRB ,&
&  0.54778E+00_JPRB, 0.58273E+00_JPRB, 0.59304E+00_JPRB, 0.56934E+00_JPRB, 0.54407E+00_JPRB, 0.53022E+00_JPRB, 0.54425E+00_JPRB ,&
&  0.53332E+00_JPRB, 0.52451E+00_JPRB, 0.51660E+00_JPRB, 0.50043E+00_JPRB, 0.47255E+00_JPRB, 0.44732E+00_JPRB, 0.45117E+00_JPRB ,&
&  0.45355E+00_JPRB, 0.46175E+00_JPRB, 0.45905E+00_JPRB, 0.46571E+00_JPRB, 0.47130E+00_JPRB, 0.47844E+00_JPRB, 0.48457E+00_JPRB ,&
&  0.49435E+00_JPRB, 0.50354E+00_JPRB, 0.51272E+00_JPRB, 0.50613E+00_JPRB, 0.50062E+00_JPRB, 0.49962E+00_JPRB, 0.50418E+00_JPRB ,&
&  0.51656E+00_JPRB, 0.53124E+00_JPRB, 0.53022E+00_JPRB, 0.54158E+00_JPRB, 0.57157E+00_JPRB, 0.58456E+00_JPRB, 0.59045E+00_JPRB ,&
&  0.64325E+00_JPRB, 0.68842E+00_JPRB, 0.63291E+00_JPRB, 0.48370E+00_JPRB, 0.45313E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.42447E+00_JPRB, 0.41851E+00_JPRB, 0.41625E+00_JPRB, 0.41851E+00_JPRB, 0.41463E+00_JPRB, 0.41791E+00_JPRB, 0.40739E+00_JPRB ,&
&  0.38613E+00_JPRB, 0.37812E+00_JPRB, 0.39199E+00_JPRB, 0.39701E+00_JPRB, 0.39972E+00_JPRB, 0.41262E+00_JPRB, 0.42695E+00_JPRB ,&
&  0.43071E+00_JPRB, 0.42896E+00_JPRB, 0.42605E+00_JPRB, 0.42548E+00_JPRB, 0.42203E+00_JPRB, 0.42414E+00_JPRB, 0.43789E+00_JPRB ,&
&  0.45845E+00_JPRB, 0.49615E+00_JPRB, 0.55223E+00_JPRB, 0.62052E+00_JPRB, 0.68840E+00_JPRB, 0.77127E+00_JPRB, 0.82978E+00_JPRB ,&
&  0.93775E+00_JPRB, 0.66398E+00_JPRB, 0.47811E+00_JPRB, 0.82019E+00_JPRB, 0.93476E+00_JPRB, 0.92084E+00_JPRB, 0.79955E+00_JPRB ,&
&  0.75893E+00_JPRB, 0.73831E+00_JPRB, 0.74064E+00_JPRB, 0.71661E+00_JPRB, 0.64360E+00_JPRB, 0.60888E+00_JPRB, 0.57559E+00_JPRB ,&
&  0.55835E+00_JPRB, 0.56793E+00_JPRB, 0.60409E+00_JPRB, 0.67930E+00_JPRB, 0.77775E+00_JPRB, 0.84218E+00_JPRB, 0.88953E+00_JPRB ,&
&  0.99911E+00_JPRB, 0.11266E+01_JPRB, 0.11439E+01_JPRB, 0.82347E+00_JPRB, 0.63980E+00_JPRB, 0.57894E+00_JPRB, 0.55567E+00_JPRB ,&
&  0.54101E+00_JPRB, 0.57077E+00_JPRB, 0.59222E+00_JPRB, 0.57279E+00_JPRB, 0.54878E+00_JPRB, 0.52450E+00_JPRB, 0.53861E+00_JPRB ,&
&  0.52600E+00_JPRB, 0.51332E+00_JPRB, 0.49672E+00_JPRB, 0.46998E+00_JPRB, 0.44155E+00_JPRB, 0.44169E+00_JPRB, 0.43545E+00_JPRB ,&
&  0.43789E+00_JPRB, 0.44905E+00_JPRB, 0.44776E+00_JPRB, 0.45885E+00_JPRB, 0.47126E+00_JPRB, 0.48809E+00_JPRB, 0.50317E+00_JPRB ,&
&  0.51280E+00_JPRB, 0.52547E+00_JPRB, 0.53920E+00_JPRB, 0.54039E+00_JPRB, 0.53335E+00_JPRB, 0.52217E+00_JPRB, 0.53543E+00_JPRB ,&
&  0.54498E+00_JPRB, 0.56087E+00_JPRB, 0.56151E+00_JPRB, 0.57693E+00_JPRB, 0.56582E+00_JPRB, 0.56335E+00_JPRB, 0.53529E+00_JPRB ,&
&  0.53585E+00_JPRB, 0.50666E+00_JPRB, 0.47375E+00_JPRB, 0.42654E+00_JPRB, 0.42329E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.42697E+00_JPRB, 0.41974E+00_JPRB, 0.41235E+00_JPRB, 0.40406E+00_JPRB, 0.39580E+00_JPRB, 0.39617E+00_JPRB, 0.38641E+00_JPRB ,&
&  0.38312E+00_JPRB, 0.40140E+00_JPRB, 0.42737E+00_JPRB, 0.44839E+00_JPRB, 0.46620E+00_JPRB, 0.47199E+00_JPRB, 0.48506E+00_JPRB ,&
&  0.48360E+00_JPRB, 0.48060E+00_JPRB, 0.47881E+00_JPRB, 0.47779E+00_JPRB, 0.48257E+00_JPRB, 0.50041E+00_JPRB, 0.53263E+00_JPRB ,&
&  0.57895E+00_JPRB, 0.63888E+00_JPRB, 0.70949E+00_JPRB, 0.78098E+00_JPRB, 0.84069E+00_JPRB, 0.89687E+00_JPRB, 0.91179E+00_JPRB ,&
&  0.66262E+00_JPRB, 0.42900E+00_JPRB, 0.83184E+00_JPRB, 0.92850E+00_JPRB, 0.86336E+00_JPRB, 0.82720E+00_JPRB, 0.76633E+00_JPRB ,&
&  0.70882E+00_JPRB, 0.69997E+00_JPRB, 0.67784E+00_JPRB, 0.67048E+00_JPRB, 0.61755E+00_JPRB, 0.60501E+00_JPRB, 0.60171E+00_JPRB ,&
&  0.60992E+00_JPRB, 0.63843E+00_JPRB, 0.70434E+00_JPRB, 0.81354E+00_JPRB, 0.89559E+00_JPRB, 0.93956E+00_JPRB, 0.10257E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.13049E+01_JPRB, 0.13457E+01_JPRB, 0.96230E+00_JPRB, 0.73270E+00_JPRB, 0.67756E+00_JPRB, 0.63534E+00_JPRB ,&
&  0.61420E+00_JPRB, 0.55784E+00_JPRB, 0.56386E+00_JPRB, 0.57753E+00_JPRB, 0.55745E+00_JPRB, 0.51726E+00_JPRB, 0.52387E+00_JPRB ,&
&  0.50657E+00_JPRB, 0.47710E+00_JPRB, 0.44703E+00_JPRB, 0.43212E+00_JPRB, 0.42747E+00_JPRB, 0.43727E+00_JPRB, 0.43600E+00_JPRB ,&
&  0.43818E+00_JPRB, 0.44915E+00_JPRB, 0.45178E+00_JPRB, 0.45896E+00_JPRB, 0.48626E+00_JPRB, 0.51384E+00_JPRB, 0.53152E+00_JPRB ,&
&  0.53895E+00_JPRB, 0.55404E+00_JPRB, 0.56562E+00_JPRB, 0.57105E+00_JPRB, 0.57158E+00_JPRB, 0.55555E+00_JPRB, 0.55495E+00_JPRB ,&
&  0.56160E+00_JPRB, 0.56759E+00_JPRB, 0.56411E+00_JPRB, 0.56680E+00_JPRB, 0.54758E+00_JPRB, 0.54352E+00_JPRB, 0.49253E+00_JPRB ,&
&  0.47081E+00_JPRB, 0.43511E+00_JPRB, 0.40872E+00_JPRB, 0.39900E+00_JPRB, 0.40552E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.42694E+00_JPRB, 0.40995E+00_JPRB, 0.39986E+00_JPRB, 0.39173E+00_JPRB, 0.38195E+00_JPRB, 0.40173E+00_JPRB, 0.40542E+00_JPRB ,&
&  0.42481E+00_JPRB, 0.45770E+00_JPRB, 0.49496E+00_JPRB, 0.51118E+00_JPRB, 0.51052E+00_JPRB, 0.52643E+00_JPRB, 0.54024E+00_JPRB ,&
&  0.54744E+00_JPRB, 0.55248E+00_JPRB, 0.56081E+00_JPRB, 0.57180E+00_JPRB, 0.59272E+00_JPRB, 0.62527E+00_JPRB, 0.67367E+00_JPRB ,&
&  0.72808E+00_JPRB, 0.78484E+00_JPRB, 0.84451E+00_JPRB, 0.89684E+00_JPRB, 0.92565E+00_JPRB, 0.95041E+00_JPRB, 0.91999E+00_JPRB ,&
&  0.60199E+00_JPRB, 0.80338E+00_JPRB, 0.91299E+00_JPRB, 0.85926E+00_JPRB, 0.81051E+00_JPRB, 0.77373E+00_JPRB, 0.74526E+00_JPRB ,&
&  0.71332E+00_JPRB, 0.67301E+00_JPRB, 0.65845E+00_JPRB, 0.68098E+00_JPRB, 0.63685E+00_JPRB, 0.62773E+00_JPRB, 0.63912E+00_JPRB ,&
&  0.66293E+00_JPRB, 0.71337E+00_JPRB, 0.81018E+00_JPRB, 0.92201E+00_JPRB, 0.99017E+00_JPRB, 0.10591E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.13533E+01_JPRB, 0.14854E+01_JPRB, 0.15716E+01_JPRB, 0.12091E+01_JPRB, 0.10603E+01_JPRB, 0.10392E+01_JPRB, 0.95778E+00_JPRB ,&
&  0.68520E+00_JPRB, 0.56942E+00_JPRB, 0.57947E+00_JPRB, 0.59035E+00_JPRB, 0.54350E+00_JPRB, 0.51458E+00_JPRB, 0.48373E+00_JPRB ,&
&  0.44964E+00_JPRB, 0.41708E+00_JPRB, 0.39679E+00_JPRB, 0.41930E+00_JPRB, 0.44239E+00_JPRB, 0.45525E+00_JPRB, 0.44688E+00_JPRB ,&
&  0.46834E+00_JPRB, 0.45789E+00_JPRB, 0.45703E+00_JPRB, 0.46860E+00_JPRB, 0.51743E+00_JPRB, 0.53737E+00_JPRB, 0.54378E+00_JPRB ,&
&  0.55133E+00_JPRB, 0.55849E+00_JPRB, 0.56255E+00_JPRB, 0.56956E+00_JPRB, 0.57401E+00_JPRB, 0.57353E+00_JPRB, 0.56889E+00_JPRB ,&
&  0.56362E+00_JPRB, 0.56993E+00_JPRB, 0.56525E+00_JPRB, 0.55994E+00_JPRB, 0.54975E+00_JPRB, 0.57506E+00_JPRB, 0.51061E+00_JPRB ,&
&  0.46555E+00_JPRB, 0.43220E+00_JPRB, 0.40727E+00_JPRB, 0.40072E+00_JPRB, 0.40665E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.43061E+00_JPRB, 0.41865E+00_JPRB, 0.40136E+00_JPRB, 0.40852E+00_JPRB, 0.41005E+00_JPRB, 0.42493E+00_JPRB, 0.44187E+00_JPRB ,&
&  0.46398E+00_JPRB, 0.50511E+00_JPRB, 0.54101E+00_JPRB, 0.56836E+00_JPRB, 0.58048E+00_JPRB, 0.59916E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.62623E+00_JPRB, 0.63742E+00_JPRB, 0.65197E+00_JPRB, 0.67603E+00_JPRB, 0.70759E+00_JPRB, 0.74310E+00_JPRB, 0.78666E+00_JPRB ,&
&  0.83453E+00_JPRB, 0.88820E+00_JPRB, 0.95843E+00_JPRB, 0.97213E+00_JPRB, 0.98677E+00_JPRB, 0.10100E+01_JPRB, 0.98122E+00_JPRB ,&
&  0.74040E+00_JPRB, 0.87004E+00_JPRB, 0.84542E+00_JPRB, 0.80944E+00_JPRB, 0.78239E+00_JPRB, 0.74561E+00_JPRB, 0.71455E+00_JPRB ,&
&  0.69110E+00_JPRB, 0.64772E+00_JPRB, 0.63003E+00_JPRB, 0.62179E+00_JPRB, 0.65965E+00_JPRB, 0.65372E+00_JPRB, 0.67956E+00_JPRB ,&
&  0.72461E+00_JPRB, 0.80099E+00_JPRB, 0.89836E+00_JPRB, 0.99696E+00_JPRB, 0.10905E+01_JPRB, 0.11880E+01_JPRB, 0.13263E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.15736E+01_JPRB, 0.16223E+01_JPRB, 0.14028E+01_JPRB, 0.11532E+01_JPRB, 0.10309E+01_JPRB, 0.82802E+00_JPRB ,&
&  0.63811E+00_JPRB, 0.57183E+00_JPRB, 0.59784E+00_JPRB, 0.60614E+00_JPRB, 0.55036E+00_JPRB, 0.50165E+00_JPRB, 0.46028E+00_JPRB ,&
&  0.43099E+00_JPRB, 0.40633E+00_JPRB, 0.42230E+00_JPRB, 0.47244E+00_JPRB, 0.49702E+00_JPRB, 0.51647E+00_JPRB, 0.51686E+00_JPRB ,&
&  0.51065E+00_JPRB, 0.48534E+00_JPRB, 0.46834E+00_JPRB, 0.48046E+00_JPRB, 0.51666E+00_JPRB, 0.55237E+00_JPRB, 0.55239E+00_JPRB ,&
&  0.53696E+00_JPRB, 0.53198E+00_JPRB, 0.54019E+00_JPRB, 0.56165E+00_JPRB, 0.57922E+00_JPRB, 0.60465E+00_JPRB, 0.62645E+00_JPRB ,&
&  0.65652E+00_JPRB, 0.70146E+00_JPRB, 0.69441E+00_JPRB, 0.69549E+00_JPRB, 0.79331E+00_JPRB, 0.84894E+00_JPRB, 0.53867E+00_JPRB ,&
&  0.45588E+00_JPRB, 0.42259E+00_JPRB, 0.41802E+00_JPRB, 0.41089E+00_JPRB, 0.43825E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.46447E+00_JPRB, 0.48195E+00_JPRB, 0.47194E+00_JPRB, 0.47409E+00_JPRB, 0.48039E+00_JPRB, 0.49008E+00_JPRB, 0.51078E+00_JPRB ,&
&  0.51493E+00_JPRB, 0.54000E+00_JPRB, 0.56435E+00_JPRB, 0.58328E+00_JPRB, 0.61165E+00_JPRB, 0.65156E+00_JPRB, 0.69946E+00_JPRB ,&
&  0.74615E+00_JPRB, 0.77132E+00_JPRB, 0.78800E+00_JPRB, 0.79714E+00_JPRB, 0.82010E+00_JPRB, 0.85565E+00_JPRB, 0.86814E+00_JPRB ,&
&  0.91675E+00_JPRB, 0.94887E+00_JPRB, 0.99048E+00_JPRB, 0.10527E+01_JPRB, 0.99873E+00_JPRB, 0.99167E+00_JPRB, 0.73425E+00_JPRB ,&
&  0.83359E+00_JPRB, 0.84013E+00_JPRB, 0.82093E+00_JPRB, 0.80072E+00_JPRB, 0.77051E+00_JPRB, 0.72260E+00_JPRB, 0.69056E+00_JPRB ,&
&  0.66668E+00_JPRB, 0.64451E+00_JPRB, 0.62217E+00_JPRB, 0.61504E+00_JPRB, 0.67586E+00_JPRB, 0.67882E+00_JPRB, 0.72916E+00_JPRB ,&
&  0.80018E+00_JPRB, 0.88117E+00_JPRB, 0.95830E+00_JPRB, 0.10492E+01_JPRB, 0.11623E+01_JPRB, 0.12858E+01_JPRB, 0.14133E+01_JPRB ,&
&  0.15118E+01_JPRB, 0.15914E+01_JPRB, 0.15617E+01_JPRB, 0.12975E+01_JPRB, 0.11723E+01_JPRB, 0.94200E+00_JPRB, 0.72955E+00_JPRB ,&
&  0.62904E+00_JPRB, 0.60324E+00_JPRB, 0.63492E+00_JPRB, 0.64525E+00_JPRB, 0.57079E+00_JPRB, 0.50871E+00_JPRB, 0.46492E+00_JPRB ,&
&  0.44163E+00_JPRB, 0.43918E+00_JPRB, 0.46680E+00_JPRB, 0.49799E+00_JPRB, 0.53586E+00_JPRB, 0.57218E+00_JPRB, 0.57974E+00_JPRB ,&
&  0.56783E+00_JPRB, 0.53482E+00_JPRB, 0.52092E+00_JPRB, 0.54070E+00_JPRB, 0.60567E+00_JPRB, 0.67755E+00_JPRB, 0.73158E+00_JPRB ,&
&  0.59453E+00_JPRB, 0.50320E+00_JPRB, 0.52863E+00_JPRB, 0.56603E+00_JPRB, 0.61866E+00_JPRB, 0.69407E+00_JPRB, 0.80861E+00_JPRB ,&
&  0.97917E+00_JPRB, 0.13325E+01_JPRB, 0.13136E+01_JPRB, 0.10959E+01_JPRB, 0.16166E+01_JPRB, 0.13371E+01_JPRB, 0.48023E+00_JPRB ,&
&  0.46030E+00_JPRB, 0.45204E+00_JPRB, 0.45419E+00_JPRB, 0.45522E+00_JPRB, 0.46359E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.55119E+00_JPRB, 0.55879E+00_JPRB, 0.57442E+00_JPRB, 0.57397E+00_JPRB, 0.58109E+00_JPRB, 0.59803E+00_JPRB, 0.61479E+00_JPRB ,&
&  0.63341E+00_JPRB, 0.64015E+00_JPRB, 0.65102E+00_JPRB, 0.66359E+00_JPRB, 0.67850E+00_JPRB, 0.71393E+00_JPRB, 0.76835E+00_JPRB ,&
&  0.80329E+00_JPRB, 0.83479E+00_JPRB, 0.87654E+00_JPRB, 0.88045E+00_JPRB, 0.89901E+00_JPRB, 0.93476E+00_JPRB, 0.94982E+00_JPRB ,&
&  0.98617E+00_JPRB, 0.10022E+01_JPRB, 0.10175E+01_JPRB, 0.10632E+01_JPRB, 0.10195E+01_JPRB, 0.10240E+01_JPRB, 0.80008E+00_JPRB ,&
&  0.85875E+00_JPRB, 0.84239E+00_JPRB, 0.79388E+00_JPRB, 0.77053E+00_JPRB, 0.74690E+00_JPRB, 0.71250E+00_JPRB, 0.68639E+00_JPRB ,&
&  0.67527E+00_JPRB, 0.64983E+00_JPRB, 0.62473E+00_JPRB, 0.65149E+00_JPRB, 0.67401E+00_JPRB, 0.71616E+00_JPRB, 0.78615E+00_JPRB ,&
&  0.85810E+00_JPRB, 0.92728E+00_JPRB, 0.98822E+00_JPRB, 0.10770E+01_JPRB, 0.11926E+01_JPRB, 0.13167E+01_JPRB, 0.14501E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.15993E+01_JPRB, 0.16057E+01_JPRB, 0.13732E+01_JPRB, 0.13205E+01_JPRB, 0.10588E+01_JPRB, 0.78026E+00_JPRB ,&
&  0.63740E+00_JPRB, 0.59694E+00_JPRB, 0.62430E+00_JPRB, 0.67973E+00_JPRB, 0.61116E+00_JPRB, 0.55760E+00_JPRB, 0.50908E+00_JPRB ,&
&  0.48755E+00_JPRB, 0.48474E+00_JPRB, 0.49386E+00_JPRB, 0.52432E+00_JPRB, 0.56577E+00_JPRB, 0.60848E+00_JPRB, 0.63278E+00_JPRB ,&
&  0.62740E+00_JPRB, 0.59004E+00_JPRB, 0.55472E+00_JPRB, 0.53090E+00_JPRB, 0.53687E+00_JPRB, 0.56419E+00_JPRB, 0.53212E+00_JPRB ,&
&  0.54393E+00_JPRB, 0.51928E+00_JPRB, 0.52690E+00_JPRB, 0.58791E+00_JPRB, 0.68900E+00_JPRB, 0.90205E+00_JPRB, 0.13026E+01_JPRB ,&
&  0.18941E+01_JPRB, 0.20525E+01_JPRB, 0.25484E+01_JPRB, 0.34202E+01_JPRB, 0.25165E+01_JPRB, 0.17601E+01_JPRB, 0.52889E+00_JPRB ,&
&  0.54168E+00_JPRB, 0.53581E+00_JPRB, 0.53335E+00_JPRB, 0.53065E+00_JPRB, 0.53571E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.58125E+00_JPRB, 0.59456E+00_JPRB, 0.61030E+00_JPRB, 0.62370E+00_JPRB, 0.64187E+00_JPRB, 0.66863E+00_JPRB, 0.69501E+00_JPRB ,&
&  0.71663E+00_JPRB, 0.73110E+00_JPRB, 0.73991E+00_JPRB, 0.75572E+00_JPRB, 0.77604E+00_JPRB, 0.80225E+00_JPRB, 0.83701E+00_JPRB ,&
&  0.84396E+00_JPRB, 0.87183E+00_JPRB, 0.87883E+00_JPRB, 0.90219E+00_JPRB, 0.90336E+00_JPRB, 0.93366E+00_JPRB, 0.95294E+00_JPRB ,&
&  0.96337E+00_JPRB, 0.96762E+00_JPRB, 0.96849E+00_JPRB, 0.98111E+00_JPRB, 0.95550E+00_JPRB, 0.96145E+00_JPRB, 0.73428E+00_JPRB ,&
&  0.83874E+00_JPRB, 0.81523E+00_JPRB, 0.73893E+00_JPRB, 0.70461E+00_JPRB, 0.67479E+00_JPRB, 0.65313E+00_JPRB, 0.63959E+00_JPRB ,&
&  0.62678E+00_JPRB, 0.63943E+00_JPRB, 0.63222E+00_JPRB, 0.64392E+00_JPRB, 0.68949E+00_JPRB, 0.75450E+00_JPRB, 0.80910E+00_JPRB ,&
&  0.88042E+00_JPRB, 0.94683E+00_JPRB, 0.10069E+01_JPRB, 0.10840E+01_JPRB, 0.11842E+01_JPRB, 0.12886E+01_JPRB, 0.13910E+01_JPRB ,&
&  0.14171E+01_JPRB, 0.14327E+01_JPRB, 0.13136E+01_JPRB, 0.13113E+01_JPRB, 0.13565E+01_JPRB, 0.11645E+01_JPRB, 0.87349E+00_JPRB ,&
&  0.59014E+00_JPRB, 0.58814E+00_JPRB, 0.64930E+00_JPRB, 0.75121E+00_JPRB, 0.70494E+00_JPRB, 0.65545E+00_JPRB, 0.59944E+00_JPRB ,&
&  0.56883E+00_JPRB, 0.54556E+00_JPRB, 0.55323E+00_JPRB, 0.58467E+00_JPRB, 0.62968E+00_JPRB, 0.67146E+00_JPRB, 0.68624E+00_JPRB ,&
&  0.67058E+00_JPRB, 0.63165E+00_JPRB, 0.57983E+00_JPRB, 0.53145E+00_JPRB, 0.49976E+00_JPRB, 0.46675E+00_JPRB, 0.53261E+00_JPRB ,&
&  0.50694E+00_JPRB, 0.48933E+00_JPRB, 0.52610E+00_JPRB, 0.47514E+00_JPRB, 0.66054E+00_JPRB, 0.81704E+00_JPRB, 0.10644E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.16403E+01_JPRB, 0.20633E+01_JPRB, 0.14713E+01_JPRB, 0.10844E+01_JPRB, 0.65523E+00_JPRB, 0.58139E+00_JPRB ,&
&  0.57766E+00_JPRB, 0.56036E+00_JPRB, 0.55714E+00_JPRB, 0.56343E+00_JPRB, 0.57076E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.61208E+00_JPRB, 0.62833E+00_JPRB, 0.64462E+00_JPRB, 0.65987E+00_JPRB, 0.67458E+00_JPRB, 0.68744E+00_JPRB, 0.70171E+00_JPRB ,&
&  0.71665E+00_JPRB, 0.73267E+00_JPRB, 0.74783E+00_JPRB, 0.76668E+00_JPRB, 0.78459E+00_JPRB, 0.79307E+00_JPRB, 0.80411E+00_JPRB ,&
&  0.80813E+00_JPRB, 0.83171E+00_JPRB, 0.83825E+00_JPRB, 0.85920E+00_JPRB, 0.87804E+00_JPRB, 0.88636E+00_JPRB, 0.91139E+00_JPRB ,&
&  0.92338E+00_JPRB, 0.93281E+00_JPRB, 0.94375E+00_JPRB, 0.96565E+00_JPRB, 0.96877E+00_JPRB, 0.10082E+01_JPRB, 0.79463E+00_JPRB ,&
&  0.79917E+00_JPRB, 0.75250E+00_JPRB, 0.68638E+00_JPRB, 0.63415E+00_JPRB, 0.59297E+00_JPRB, 0.54977E+00_JPRB, 0.52592E+00_JPRB ,&
&  0.52649E+00_JPRB, 0.53713E+00_JPRB, 0.55745E+00_JPRB, 0.58818E+00_JPRB, 0.64138E+00_JPRB, 0.71511E+00_JPRB, 0.77181E+00_JPRB ,&
&  0.83461E+00_JPRB, 0.90867E+00_JPRB, 0.97338E+00_JPRB, 0.10382E+01_JPRB, 0.11123E+01_JPRB, 0.11782E+01_JPRB, 0.12155E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.11556E+01_JPRB, 0.98166E+00_JPRB, 0.10713E+01_JPRB, 0.11492E+01_JPRB, 0.10794E+01_JPRB, 0.87369E+00_JPRB ,&
&  0.66508E+00_JPRB, 0.61469E+00_JPRB, 0.66522E+00_JPRB, 0.82580E+00_JPRB, 0.85993E+00_JPRB, 0.83532E+00_JPRB, 0.78756E+00_JPRB ,&
&  0.74520E+00_JPRB, 0.69882E+00_JPRB, 0.67747E+00_JPRB, 0.69306E+00_JPRB, 0.72656E+00_JPRB, 0.73827E+00_JPRB, 0.69617E+00_JPRB ,&
&  0.65897E+00_JPRB, 0.63155E+00_JPRB, 0.58411E+00_JPRB, 0.51350E+00_JPRB, 0.47489E+00_JPRB, 0.42083E+00_JPRB, 0.47327E+00_JPRB ,&
&  0.45324E+00_JPRB, 0.39463E+00_JPRB, 0.43962E+00_JPRB, 0.51924E+00_JPRB, 0.58463E+00_JPRB, 0.66479E+00_JPRB, 0.83426E+00_JPRB ,&
&  0.97614E+00_JPRB, 0.10214E+01_JPRB, 0.95245E+00_JPRB, 0.77496E+00_JPRB, 0.63572E+00_JPRB, 0.58089E+00_JPRB, 0.57864E+00_JPRB ,&
&  0.57775E+00_JPRB, 0.57186E+00_JPRB, 0.57648E+00_JPRB, 0.58829E+00_JPRB, 0.59818E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.59125E+00_JPRB, 0.60690E+00_JPRB, 0.62280E+00_JPRB, 0.63892E+00_JPRB, 0.65277E+00_JPRB, 0.66550E+00_JPRB, 0.67901E+00_JPRB ,&
&  0.69258E+00_JPRB, 0.70869E+00_JPRB, 0.72586E+00_JPRB, 0.74411E+00_JPRB, 0.75774E+00_JPRB, 0.76168E+00_JPRB, 0.76953E+00_JPRB ,&
&  0.77402E+00_JPRB, 0.77245E+00_JPRB, 0.76302E+00_JPRB, 0.78032E+00_JPRB, 0.79846E+00_JPRB, 0.81074E+00_JPRB, 0.83155E+00_JPRB ,&
&  0.85619E+00_JPRB, 0.84777E+00_JPRB, 0.84650E+00_JPRB, 0.86779E+00_JPRB, 0.87875E+00_JPRB, 0.83064E+00_JPRB, 0.89776E+00_JPRB ,&
&  0.66455E+00_JPRB, 0.78404E+00_JPRB, 0.71669E+00_JPRB, 0.62332E+00_JPRB, 0.59852E+00_JPRB, 0.52946E+00_JPRB, 0.49106E+00_JPRB ,&
&  0.51404E+00_JPRB, 0.50592E+00_JPRB, 0.50246E+00_JPRB, 0.51179E+00_JPRB, 0.55540E+00_JPRB, 0.62763E+00_JPRB, 0.68118E+00_JPRB ,&
&  0.75167E+00_JPRB, 0.81819E+00_JPRB, 0.89684E+00_JPRB, 0.98608E+00_JPRB, 0.10873E+01_JPRB, 0.11315E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.98909E+00_JPRB, 0.92455E+00_JPRB, 0.71883E+00_JPRB, 0.78993E+00_JPRB, 0.90350E+00_JPRB, 0.89329E+00_JPRB, 0.78116E+00_JPRB ,&
&  0.67120E+00_JPRB, 0.67927E+00_JPRB, 0.82335E+00_JPRB, 0.97783E+00_JPRB, 0.10506E+01_JPRB, 0.10543E+01_JPRB, 0.98876E+00_JPRB ,&
&  0.92564E+00_JPRB, 0.83915E+00_JPRB, 0.78647E+00_JPRB, 0.79770E+00_JPRB, 0.79029E+00_JPRB, 0.74811E+00_JPRB, 0.65285E+00_JPRB ,&
&  0.60155E+00_JPRB, 0.58765E+00_JPRB, 0.56257E+00_JPRB, 0.50664E+00_JPRB, 0.42291E+00_JPRB, 0.41118E+00_JPRB, 0.39877E+00_JPRB ,&
&  0.41635E+00_JPRB, 0.38088E+00_JPRB, 0.37909E+00_JPRB, 0.47503E+00_JPRB, 0.51634E+00_JPRB, 0.54244E+00_JPRB, 0.59398E+00_JPRB ,&
&  0.62516E+00_JPRB, 0.58423E+00_JPRB, 0.57528E+00_JPRB, 0.55226E+00_JPRB, 0.55141E+00_JPRB, 0.54667E+00_JPRB, 0.55863E+00_JPRB ,&
&  0.56529E+00_JPRB, 0.56382E+00_JPRB, 0.56896E+00_JPRB, 0.57677E+00_JPRB, 0.58157E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.56150E+00_JPRB, 0.56802E+00_JPRB, 0.57272E+00_JPRB, 0.58783E+00_JPRB, 0.61072E+00_JPRB, 0.61330E+00_JPRB, 0.62466E+00_JPRB ,&
&  0.63035E+00_JPRB, 0.64511E+00_JPRB, 0.66104E+00_JPRB, 0.68048E+00_JPRB, 0.69005E+00_JPRB, 0.69277E+00_JPRB, 0.70803E+00_JPRB ,&
&  0.71958E+00_JPRB, 0.71255E+00_JPRB, 0.71918E+00_JPRB, 0.73442E+00_JPRB, 0.73765E+00_JPRB, 0.76247E+00_JPRB, 0.76046E+00_JPRB ,&
&  0.78814E+00_JPRB, 0.79578E+00_JPRB, 0.82161E+00_JPRB, 0.89386E+00_JPRB, 0.97964E+00_JPRB, 0.11613E+01_JPRB, 0.14845E+01_JPRB ,&
&  0.11938E+01_JPRB, 0.89792E+00_JPRB, 0.82645E+00_JPRB, 0.69918E+00_JPRB, 0.69757E+00_JPRB, 0.71736E+00_JPRB, 0.70103E+00_JPRB ,&
&  0.67410E+00_JPRB, 0.63208E+00_JPRB, 0.60618E+00_JPRB, 0.58607E+00_JPRB, 0.58805E+00_JPRB, 0.62009E+00_JPRB, 0.63770E+00_JPRB ,&
&  0.61965E+00_JPRB, 0.68064E+00_JPRB, 0.76181E+00_JPRB, 0.87895E+00_JPRB, 0.96566E+00_JPRB, 0.11577E+01_JPRB, 0.11471E+01_JPRB ,&
&  0.10978E+01_JPRB, 0.10423E+01_JPRB, 0.89960E+00_JPRB, 0.92517E+00_JPRB, 0.99526E+00_JPRB, 0.95067E+00_JPRB, 0.83029E+00_JPRB ,&
&  0.79141E+00_JPRB, 0.73670E+00_JPRB, 0.80317E+00_JPRB, 0.11285E+01_JPRB, 0.12615E+01_JPRB, 0.13018E+01_JPRB, 0.12301E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.10207E+01_JPRB, 0.91561E+00_JPRB, 0.84609E+00_JPRB, 0.80845E+00_JPRB, 0.74273E+00_JPRB, 0.64903E+00_JPRB ,&
&  0.56981E+00_JPRB, 0.55232E+00_JPRB, 0.54147E+00_JPRB, 0.48454E+00_JPRB, 0.46758E+00_JPRB, 0.35208E+00_JPRB, 0.37097E+00_JPRB ,&
&  0.35567E+00_JPRB, 0.36681E+00_JPRB, 0.34548E+00_JPRB, 0.41953E+00_JPRB, 0.43041E+00_JPRB, 0.41192E+00_JPRB, 0.42418E+00_JPRB ,&
&  0.43517E+00_JPRB, 0.44472E+00_JPRB, 0.45897E+00_JPRB, 0.47125E+00_JPRB, 0.48631E+00_JPRB, 0.51069E+00_JPRB, 0.53705E+00_JPRB ,&
&  0.55755E+00_JPRB, 0.54492E+00_JPRB, 0.54474E+00_JPRB, 0.54844E+00_JPRB, 0.56107E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.52378E+00_JPRB, 0.52034E+00_JPRB, 0.51269E+00_JPRB, 0.51296E+00_JPRB, 0.51722E+00_JPRB, 0.52963E+00_JPRB, 0.53708E+00_JPRB ,&
&  0.53467E+00_JPRB, 0.53867E+00_JPRB, 0.52213E+00_JPRB, 0.52223E+00_JPRB, 0.53606E+00_JPRB, 0.54430E+00_JPRB, 0.55183E+00_JPRB ,&
&  0.58179E+00_JPRB, 0.59326E+00_JPRB, 0.59086E+00_JPRB, 0.59267E+00_JPRB, 0.58735E+00_JPRB, 0.62855E+00_JPRB, 0.63157E+00_JPRB ,&
&  0.61726E+00_JPRB, 0.64192E+00_JPRB, 0.74501E+00_JPRB, 0.10095E+01_JPRB, 0.12452E+01_JPRB, 0.12158E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.10592E+01_JPRB, 0.11338E+01_JPRB, 0.98964E+00_JPRB, 0.10077E+01_JPRB, 0.10421E+01_JPRB, 0.10335E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10233E+01_JPRB, 0.10173E+01_JPRB, 0.10175E+01_JPRB, 0.10047E+01_JPRB, 0.98998E+00_JPRB, 0.96382E+00_JPRB ,&
&  0.89962E+00_JPRB, 0.89191E+00_JPRB, 0.94294E+00_JPRB, 0.10639E+01_JPRB, 0.12069E+01_JPRB, 0.13541E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.14825E+01_JPRB, 0.13908E+01_JPRB, 0.14068E+01_JPRB, 0.14719E+01_JPRB, 0.13499E+01_JPRB, 0.12376E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.87325E+00_JPRB, 0.86421E+00_JPRB, 0.12861E+01_JPRB, 0.14174E+01_JPRB, 0.13931E+01_JPRB, 0.14101E+01_JPRB ,&
&  0.13247E+01_JPRB, 0.12364E+01_JPRB, 0.10807E+01_JPRB, 0.93939E+00_JPRB, 0.81903E+00_JPRB, 0.71789E+00_JPRB, 0.67190E+00_JPRB ,&
&  0.58254E+00_JPRB, 0.53117E+00_JPRB, 0.51728E+00_JPRB, 0.47705E+00_JPRB, 0.43327E+00_JPRB, 0.40104E+00_JPRB, 0.37042E+00_JPRB ,&
&  0.34901E+00_JPRB, 0.34695E+00_JPRB, 0.38939E+00_JPRB, 0.42658E+00_JPRB, 0.34874E+00_JPRB, 0.36560E+00_JPRB, 0.35532E+00_JPRB ,&
&  0.36978E+00_JPRB, 0.38985E+00_JPRB, 0.39756E+00_JPRB, 0.40789E+00_JPRB, 0.42677E+00_JPRB, 0.44365E+00_JPRB, 0.46777E+00_JPRB ,&
&  0.48353E+00_JPRB, 0.49636E+00_JPRB, 0.51018E+00_JPRB, 0.52461E+00_JPRB, 0.52135E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.57788E+00_JPRB, 0.60565E+00_JPRB, 0.62195E+00_JPRB, 0.64164E+00_JPRB, 0.64910E+00_JPRB, 0.64214E+00_JPRB, 0.61409E+00_JPRB ,&
&  0.59169E+00_JPRB, 0.56828E+00_JPRB, 0.55911E+00_JPRB, 0.54393E+00_JPRB, 0.56067E+00_JPRB, 0.59442E+00_JPRB, 0.62310E+00_JPRB ,&
&  0.69382E+00_JPRB, 0.65579E+00_JPRB, 0.65984E+00_JPRB, 0.68908E+00_JPRB, 0.69808E+00_JPRB, 0.73909E+00_JPRB, 0.86089E+00_JPRB ,&
&  0.10584E+01_JPRB, 0.12802E+01_JPRB, 0.15755E+01_JPRB, 0.20259E+01_JPRB, 0.20971E+01_JPRB, 0.16264E+01_JPRB, 0.16238E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.13214E+01_JPRB, 0.13115E+01_JPRB, 0.13056E+01_JPRB, 0.13380E+01_JPRB, 0.13342E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.13755E+01_JPRB, 0.14044E+01_JPRB, 0.14208E+01_JPRB, 0.14247E+01_JPRB, 0.14380E+01_JPRB, 0.14636E+01_JPRB, 0.15044E+01_JPRB ,&
&  0.15326E+01_JPRB, 0.15326E+01_JPRB, 0.15445E+01_JPRB, 0.15147E+01_JPRB, 0.15949E+01_JPRB, 0.16719E+01_JPRB, 0.17465E+01_JPRB ,&
&  0.17940E+01_JPRB, 0.18301E+01_JPRB, 0.19537E+01_JPRB, 0.19551E+01_JPRB, 0.19162E+01_JPRB, 0.18629E+01_JPRB, 0.17449E+01_JPRB ,&
&  0.15907E+01_JPRB, 0.12801E+01_JPRB, 0.11760E+01_JPRB, 0.18149E+01_JPRB, 0.18140E+01_JPRB, 0.15144E+01_JPRB, 0.15905E+01_JPRB ,&
&  0.14780E+01_JPRB, 0.14155E+01_JPRB, 0.12855E+01_JPRB, 0.10744E+01_JPRB, 0.90873E+00_JPRB, 0.78106E+00_JPRB, 0.66104E+00_JPRB ,&
&  0.64409E+00_JPRB, 0.56289E+00_JPRB, 0.51045E+00_JPRB, 0.47943E+00_JPRB, 0.44554E+00_JPRB, 0.42662E+00_JPRB, 0.40627E+00_JPRB ,&
&  0.39883E+00_JPRB, 0.39095E+00_JPRB, 0.42200E+00_JPRB, 0.48967E+00_JPRB, 0.49618E+00_JPRB, 0.37479E+00_JPRB, 0.35733E+00_JPRB ,&
&  0.37242E+00_JPRB, 0.38672E+00_JPRB, 0.40044E+00_JPRB, 0.41462E+00_JPRB, 0.42619E+00_JPRB, 0.44699E+00_JPRB, 0.46761E+00_JPRB ,&
&  0.49103E+00_JPRB, 0.52429E+00_JPRB, 0.54023E+00_JPRB, 0.55680E+00_JPRB, 0.57336E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.82182E+00_JPRB, 0.86370E+00_JPRB, 0.90214E+00_JPRB, 0.92515E+00_JPRB, 0.92966E+00_JPRB, 0.93941E+00_JPRB, 0.94118E+00_JPRB ,&
&  0.95166E+00_JPRB, 0.95271E+00_JPRB, 0.98357E+00_JPRB, 0.10503E+01_JPRB, 0.11802E+01_JPRB, 0.12513E+01_JPRB, 0.13405E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.13737E+01_JPRB, 0.14033E+01_JPRB, 0.14443E+01_JPRB, 0.15218E+01_JPRB, 0.16356E+01_JPRB, 0.18687E+01_JPRB ,&
&  0.23017E+01_JPRB, 0.25656E+01_JPRB, 0.24779E+01_JPRB, 0.20754E+01_JPRB, 0.16460E+01_JPRB, 0.14340E+01_JPRB, 0.16333E+01_JPRB ,&
&  0.16902E+01_JPRB, 0.15760E+01_JPRB, 0.14830E+01_JPRB, 0.14741E+01_JPRB, 0.14718E+01_JPRB, 0.14796E+01_JPRB, 0.14788E+01_JPRB ,&
&  0.14886E+01_JPRB, 0.15048E+01_JPRB, 0.15089E+01_JPRB, 0.15108E+01_JPRB, 0.15439E+01_JPRB, 0.16080E+01_JPRB, 0.16973E+01_JPRB ,&
&  0.18312E+01_JPRB, 0.19232E+01_JPRB, 0.19492E+01_JPRB, 0.19427E+01_JPRB, 0.19287E+01_JPRB, 0.19371E+01_JPRB, 0.19859E+01_JPRB ,&
&  0.20506E+01_JPRB, 0.20667E+01_JPRB, 0.22151E+01_JPRB, 0.23139E+01_JPRB, 0.22360E+01_JPRB, 0.21365E+01_JPRB, 0.20812E+01_JPRB ,&
&  0.20492E+01_JPRB, 0.19887E+01_JPRB, 0.17337E+01_JPRB, 0.23446E+01_JPRB, 0.23991E+01_JPRB, 0.20641E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.16413E+01_JPRB, 0.15534E+01_JPRB, 0.13631E+01_JPRB, 0.11191E+01_JPRB, 0.95565E+00_JPRB, 0.88075E+00_JPRB, 0.69246E+00_JPRB ,&
&  0.71723E+00_JPRB, 0.62763E+00_JPRB, 0.54748E+00_JPRB, 0.50417E+00_JPRB, 0.47929E+00_JPRB, 0.46079E+00_JPRB, 0.44221E+00_JPRB ,&
&  0.40474E+00_JPRB, 0.47306E+00_JPRB, 0.59357E+00_JPRB, 0.96035E+00_JPRB, 0.17311E+01_JPRB, 0.44773E+00_JPRB, 0.37498E+00_JPRB ,&
&  0.39654E+00_JPRB, 0.42625E+00_JPRB, 0.45978E+00_JPRB, 0.48559E+00_JPRB, 0.50555E+00_JPRB, 0.53975E+00_JPRB, 0.59379E+00_JPRB ,&
&  0.63167E+00_JPRB, 0.66206E+00_JPRB, 0.69797E+00_JPRB, 0.75662E+00_JPRB, 0.78704E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.98198E+00_JPRB, 0.10233E+01_JPRB, 0.10516E+01_JPRB, 0.10978E+01_JPRB, 0.11434E+01_JPRB, 0.11920E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.13500E+01_JPRB, 0.14493E+01_JPRB, 0.15438E+01_JPRB, 0.16132E+01_JPRB, 0.16769E+01_JPRB, 0.17248E+01_JPRB, 0.17662E+01_JPRB ,&
&  0.17947E+01_JPRB, 0.18629E+01_JPRB, 0.19397E+01_JPRB, 0.20422E+01_JPRB, 0.21650E+01_JPRB, 0.23062E+01_JPRB, 0.24514E+01_JPRB ,&
&  0.25381E+01_JPRB, 0.24121E+01_JPRB, 0.20681E+01_JPRB, 0.14437E+01_JPRB, 0.12064E+01_JPRB, 0.13870E+01_JPRB, 0.14860E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.14986E+01_JPRB, 0.14671E+01_JPRB, 0.15043E+01_JPRB, 0.15311E+01_JPRB, 0.15496E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.15681E+01_JPRB, 0.15906E+01_JPRB, 0.16010E+01_JPRB, 0.16080E+01_JPRB, 0.16468E+01_JPRB, 0.17487E+01_JPRB, 0.19092E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.22258E+01_JPRB, 0.23034E+01_JPRB, 0.23494E+01_JPRB, 0.22604E+01_JPRB, 0.21830E+01_JPRB, 0.21584E+01_JPRB ,&
&  0.22022E+01_JPRB, 0.22668E+01_JPRB, 0.23967E+01_JPRB, 0.25137E+01_JPRB, 0.25318E+01_JPRB, 0.23431E+01_JPRB, 0.22971E+01_JPRB ,&
&  0.23592E+01_JPRB, 0.23106E+01_JPRB, 0.22646E+01_JPRB, 0.27321E+01_JPRB, 0.17858E+01_JPRB, 0.17882E+01_JPRB, 0.17721E+01_JPRB ,&
&  0.17874E+01_JPRB, 0.18169E+01_JPRB, 0.14906E+01_JPRB, 0.11575E+01_JPRB, 0.10015E+01_JPRB, 0.73174E+00_JPRB, 0.71481E+00_JPRB ,&
&  0.79704E+00_JPRB, 0.74228E+00_JPRB, 0.61797E+00_JPRB, 0.54543E+00_JPRB, 0.54940E+00_JPRB, 0.50069E+00_JPRB, 0.49472E+00_JPRB ,&
&  0.45554E+00_JPRB, 0.48713E+00_JPRB, 0.57429E+00_JPRB, 0.74096E+00_JPRB, 0.91089E+00_JPRB, 0.46000E+00_JPRB, 0.40254E+00_JPRB ,&
&  0.42259E+00_JPRB, 0.46557E+00_JPRB, 0.51668E+00_JPRB, 0.56279E+00_JPRB, 0.60630E+00_JPRB, 0.65433E+00_JPRB, 0.70764E+00_JPRB ,&
&  0.76782E+00_JPRB, 0.84005E+00_JPRB, 0.88951E+00_JPRB, 0.92472E+00_JPRB, 0.94919E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.97586E+00_JPRB, 0.10207E+01_JPRB, 0.10777E+01_JPRB, 0.11540E+01_JPRB, 0.12340E+01_JPRB, 0.12656E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.13104E+01_JPRB, 0.13817E+01_JPRB, 0.14677E+01_JPRB, 0.15333E+01_JPRB, 0.15650E+01_JPRB, 0.16219E+01_JPRB, 0.16910E+01_JPRB ,&
&  0.17572E+01_JPRB, 0.18340E+01_JPRB, 0.19301E+01_JPRB, 0.20470E+01_JPRB, 0.21983E+01_JPRB, 0.23861E+01_JPRB, 0.24724E+01_JPRB ,&
&  0.19439E+01_JPRB, 0.13533E+01_JPRB, 0.17529E+01_JPRB, 0.16855E+01_JPRB, 0.14657E+01_JPRB, 0.13301E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.14909E+01_JPRB, 0.15048E+01_JPRB, 0.15080E+01_JPRB, 0.14884E+01_JPRB, 0.15262E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.16204E+01_JPRB, 0.16409E+01_JPRB, 0.16588E+01_JPRB, 0.17154E+01_JPRB, 0.18359E+01_JPRB, 0.20356E+01_JPRB ,&
&  0.22389E+01_JPRB, 0.23646E+01_JPRB, 0.24724E+01_JPRB, 0.25018E+01_JPRB, 0.24095E+01_JPRB, 0.23341E+01_JPRB, 0.23298E+01_JPRB ,&
&  0.23921E+01_JPRB, 0.24659E+01_JPRB, 0.25564E+01_JPRB, 0.26230E+01_JPRB, 0.26972E+01_JPRB, 0.24717E+01_JPRB, 0.25099E+01_JPRB ,&
&  0.25891E+01_JPRB, 0.25127E+01_JPRB, 0.25866E+01_JPRB, 0.28284E+01_JPRB, 0.19745E+01_JPRB, 0.18617E+01_JPRB, 0.16679E+01_JPRB ,&
&  0.18659E+01_JPRB, 0.19202E+01_JPRB, 0.17316E+01_JPRB, 0.14290E+01_JPRB, 0.11519E+01_JPRB, 0.80032E+00_JPRB, 0.79060E+00_JPRB ,&
&  0.87164E+00_JPRB, 0.90458E+00_JPRB, 0.71420E+00_JPRB, 0.63431E+00_JPRB, 0.50905E+00_JPRB, 0.45433E+00_JPRB, 0.46743E+00_JPRB ,&
&  0.49340E+00_JPRB, 0.48734E+00_JPRB, 0.53381E+00_JPRB, 0.60181E+00_JPRB, 0.63015E+00_JPRB, 0.47328E+00_JPRB, 0.42150E+00_JPRB ,&
&  0.44199E+00_JPRB, 0.48426E+00_JPRB, 0.54370E+00_JPRB, 0.61135E+00_JPRB, 0.64793E+00_JPRB, 0.68686E+00_JPRB, 0.73269E+00_JPRB ,&
&  0.80300E+00_JPRB, 0.85861E+00_JPRB, 0.89845E+00_JPRB, 0.91262E+00_JPRB, 0.94553E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.88699E+00_JPRB, 0.94032E+00_JPRB, 0.10141E+01_JPRB, 0.11249E+01_JPRB, 0.12798E+01_JPRB, 0.14320E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.12276E+01_JPRB, 0.10981E+01_JPRB, 0.11490E+01_JPRB, 0.11988E+01_JPRB, 0.12495E+01_JPRB, 0.13291E+01_JPRB, 0.14356E+01_JPRB ,&
&  0.15560E+01_JPRB, 0.16701E+01_JPRB, 0.17854E+01_JPRB, 0.18882E+01_JPRB, 0.19600E+01_JPRB, 0.21459E+01_JPRB, 0.20294E+01_JPRB ,&
&  0.14723E+01_JPRB, 0.12756E+01_JPRB, 0.17160E+01_JPRB, 0.14808E+01_JPRB, 0.14799E+01_JPRB, 0.14879E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.14214E+01_JPRB, 0.14154E+01_JPRB, 0.14050E+01_JPRB, 0.13772E+01_JPRB, 0.14450E+01_JPRB, 0.15169E+01_JPRB ,&
&  0.15590E+01_JPRB, 0.15846E+01_JPRB, 0.16091E+01_JPRB, 0.16305E+01_JPRB, 0.16807E+01_JPRB, 0.17971E+01_JPRB, 0.20298E+01_JPRB ,&
&  0.22227E+01_JPRB, 0.24057E+01_JPRB, 0.25838E+01_JPRB, 0.26045E+01_JPRB, 0.25330E+01_JPRB, 0.25347E+01_JPRB, 0.26080E+01_JPRB ,&
&  0.26886E+01_JPRB, 0.28189E+01_JPRB, 0.28484E+01_JPRB, 0.27833E+01_JPRB, 0.25672E+01_JPRB, 0.27669E+01_JPRB, 0.25405E+01_JPRB ,&
&  0.26712E+01_JPRB, 0.26236E+01_JPRB, 0.29949E+01_JPRB, 0.24483E+01_JPRB, 0.23458E+01_JPRB, 0.22875E+01_JPRB, 0.21197E+01_JPRB ,&
&  0.19201E+01_JPRB, 0.20152E+01_JPRB, 0.19106E+01_JPRB, 0.16192E+01_JPRB, 0.13239E+01_JPRB, 0.10068E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10878E+01_JPRB, 0.80625E+00_JPRB, 0.60180E+00_JPRB, 0.43705E+00_JPRB, 0.37562E+00_JPRB, 0.40478E+00_JPRB ,&
&  0.47995E+00_JPRB, 0.51764E+00_JPRB, 0.51268E+00_JPRB, 0.51439E+00_JPRB, 0.53257E+00_JPRB, 0.48998E+00_JPRB, 0.44884E+00_JPRB ,&
&  0.47536E+00_JPRB, 0.50345E+00_JPRB, 0.54239E+00_JPRB, 0.60569E+00_JPRB, 0.64676E+00_JPRB, 0.69357E+00_JPRB, 0.70273E+00_JPRB ,&
&  0.74015E+00_JPRB, 0.77466E+00_JPRB, 0.79154E+00_JPRB, 0.81032E+00_JPRB, 0.84036E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.72776E+00_JPRB, 0.74336E+00_JPRB, 0.77822E+00_JPRB, 0.83327E+00_JPRB, 0.88294E+00_JPRB, 0.93050E+00_JPRB, 0.94769E+00_JPRB ,&
&  0.93154E+00_JPRB, 0.91072E+00_JPRB, 0.91176E+00_JPRB, 0.89058E+00_JPRB, 0.88675E+00_JPRB, 0.90150E+00_JPRB, 0.96638E+00_JPRB ,&
&  0.11135E+01_JPRB, 0.13229E+01_JPRB, 0.15356E+01_JPRB, 0.16874E+01_JPRB, 0.17575E+01_JPRB, 0.18866E+01_JPRB, 0.16345E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.18092E+01_JPRB, 0.14310E+01_JPRB, 0.14693E+01_JPRB, 0.15212E+01_JPRB, 0.16152E+01_JPRB, 0.13786E+01_JPRB ,&
&  0.13071E+01_JPRB, 0.13306E+01_JPRB, 0.13327E+01_JPRB, 0.13421E+01_JPRB, 0.13269E+01_JPRB, 0.13737E+01_JPRB, 0.14424E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.15565E+01_JPRB, 0.15772E+01_JPRB, 0.15799E+01_JPRB, 0.16043E+01_JPRB, 0.16641E+01_JPRB, 0.18671E+01_JPRB ,&
&  0.20749E+01_JPRB, 0.23256E+01_JPRB, 0.25652E+01_JPRB, 0.27850E+01_JPRB, 0.27162E+01_JPRB, 0.27792E+01_JPRB, 0.29739E+01_JPRB ,&
&  0.29863E+01_JPRB, 0.28691E+01_JPRB, 0.30301E+01_JPRB, 0.30274E+01_JPRB, 0.29458E+01_JPRB, 0.30612E+01_JPRB, 0.26172E+01_JPRB ,&
&  0.27823E+01_JPRB, 0.26310E+01_JPRB, 0.30000E+01_JPRB, 0.25844E+01_JPRB, 0.25322E+01_JPRB, 0.27230E+01_JPRB, 0.25705E+01_JPRB ,&
&  0.22403E+01_JPRB, 0.22230E+01_JPRB, 0.19979E+01_JPRB, 0.17585E+01_JPRB, 0.15860E+01_JPRB, 0.14019E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.11999E+01_JPRB, 0.94175E+00_JPRB, 0.53326E+00_JPRB, 0.40540E+00_JPRB, 0.34501E+00_JPRB, 0.36600E+00_JPRB ,&
&  0.44379E+00_JPRB, 0.50218E+00_JPRB, 0.55710E+00_JPRB, 0.52548E+00_JPRB, 0.50674E+00_JPRB, 0.49608E+00_JPRB, 0.48350E+00_JPRB ,&
&  0.49287E+00_JPRB, 0.51094E+00_JPRB, 0.52558E+00_JPRB, 0.57745E+00_JPRB, 0.64226E+00_JPRB, 0.68498E+00_JPRB, 0.66161E+00_JPRB ,&
&  0.63939E+00_JPRB, 0.64174E+00_JPRB, 0.66745E+00_JPRB, 0.68747E+00_JPRB, 0.70871E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.62976E+00_JPRB, 0.62213E+00_JPRB, 0.62554E+00_JPRB, 0.64768E+00_JPRB, 0.65470E+00_JPRB, 0.65832E+00_JPRB, 0.70133E+00_JPRB ,&
&  0.71703E+00_JPRB, 0.71146E+00_JPRB, 0.72611E+00_JPRB, 0.73055E+00_JPRB, 0.72488E+00_JPRB, 0.70376E+00_JPRB, 0.70047E+00_JPRB ,&
&  0.73116E+00_JPRB, 0.85208E+00_JPRB, 0.10929E+01_JPRB, 0.14004E+01_JPRB, 0.16143E+01_JPRB, 0.19699E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.12211E+01_JPRB, 0.15249E+01_JPRB, 0.13733E+01_JPRB, 0.14881E+01_JPRB, 0.15795E+01_JPRB, 0.16817E+01_JPRB, 0.14386E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.13696E+01_JPRB, 0.13333E+01_JPRB, 0.13463E+01_JPRB, 0.13617E+01_JPRB, 0.13726E+01_JPRB, 0.13764E+01_JPRB ,&
&  0.14129E+01_JPRB, 0.14739E+01_JPRB, 0.15175E+01_JPRB, 0.15252E+01_JPRB, 0.15310E+01_JPRB, 0.15613E+01_JPRB, 0.17215E+01_JPRB ,&
&  0.19266E+01_JPRB, 0.21489E+01_JPRB, 0.24577E+01_JPRB, 0.27011E+01_JPRB, 0.28722E+01_JPRB, 0.29790E+01_JPRB, 0.32985E+01_JPRB ,&
&  0.32845E+01_JPRB, 0.31353E+01_JPRB, 0.32957E+01_JPRB, 0.34087E+01_JPRB, 0.34300E+01_JPRB, 0.34319E+01_JPRB, 0.29910E+01_JPRB ,&
&  0.28375E+01_JPRB, 0.27959E+01_JPRB, 0.28447E+01_JPRB, 0.25544E+01_JPRB, 0.25634E+01_JPRB, 0.29162E+01_JPRB, 0.30881E+01_JPRB ,&
&  0.28141E+01_JPRB, 0.25574E+01_JPRB, 0.23458E+01_JPRB, 0.19796E+01_JPRB, 0.17974E+01_JPRB, 0.16801E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.16911E+01_JPRB, 0.13489E+01_JPRB, 0.78697E+00_JPRB, 0.42986E+00_JPRB, 0.33767E+00_JPRB, 0.29838E+00_JPRB, 0.33649E+00_JPRB ,&
&  0.41786E+00_JPRB, 0.48102E+00_JPRB, 0.52845E+00_JPRB, 0.62800E+00_JPRB, 0.61002E+00_JPRB, 0.57814E+00_JPRB, 0.55088E+00_JPRB ,&
&  0.54358E+00_JPRB, 0.53352E+00_JPRB, 0.52830E+00_JPRB, 0.54009E+00_JPRB, 0.59752E+00_JPRB, 0.65901E+00_JPRB, 0.68895E+00_JPRB ,&
&  0.66077E+00_JPRB, 0.62100E+00_JPRB, 0.60295E+00_JPRB, 0.60001E+00_JPRB, 0.60999E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.56915E+00_JPRB, 0.56540E+00_JPRB, 0.56934E+00_JPRB, 0.58129E+00_JPRB, 0.58946E+00_JPRB, 0.57187E+00_JPRB, 0.54278E+00_JPRB ,&
&  0.55203E+00_JPRB, 0.54149E+00_JPRB, 0.56367E+00_JPRB, 0.58163E+00_JPRB, 0.60357E+00_JPRB, 0.60606E+00_JPRB, 0.60766E+00_JPRB ,&
&  0.60583E+00_JPRB, 0.64288E+00_JPRB, 0.81758E+00_JPRB, 0.11196E+01_JPRB, 0.13829E+01_JPRB, 0.15559E+01_JPRB, 0.15763E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.15270E+01_JPRB, 0.15276E+01_JPRB, 0.18468E+01_JPRB, 0.20908E+01_JPRB, 0.24733E+01_JPRB, 0.16301E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.14402E+01_JPRB, 0.14205E+01_JPRB, 0.14270E+01_JPRB, 0.14674E+01_JPRB, 0.14464E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.13897E+01_JPRB, 0.14128E+01_JPRB, 0.14326E+01_JPRB, 0.14470E+01_JPRB, 0.14777E+01_JPRB, 0.16160E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.21104E+01_JPRB, 0.23984E+01_JPRB, 0.27340E+01_JPRB, 0.29466E+01_JPRB, 0.31935E+01_JPRB, 0.36411E+01_JPRB ,&
&  0.38321E+01_JPRB, 0.39409E+01_JPRB, 0.38111E+01_JPRB, 0.36653E+01_JPRB, 0.36226E+01_JPRB, 0.38879E+01_JPRB, 0.35229E+01_JPRB ,&
&  0.31075E+01_JPRB, 0.27828E+01_JPRB, 0.25469E+01_JPRB, 0.24749E+01_JPRB, 0.25473E+01_JPRB, 0.30147E+01_JPRB, 0.30543E+01_JPRB ,&
&  0.26140E+01_JPRB, 0.22394E+01_JPRB, 0.21113E+01_JPRB, 0.20836E+01_JPRB, 0.20927E+01_JPRB, 0.20591E+01_JPRB, 0.23199E+01_JPRB ,&
&  0.17238E+01_JPRB, 0.83661E+00_JPRB, 0.48879E+00_JPRB, 0.44874E+00_JPRB, 0.30093E+00_JPRB, 0.24717E+00_JPRB, 0.36579E+00_JPRB ,&
&  0.46487E+00_JPRB, 0.56844E+00_JPRB, 0.59312E+00_JPRB, 0.62016E+00_JPRB, 0.71316E+00_JPRB, 0.70503E+00_JPRB, 0.66490E+00_JPRB ,&
&  0.61385E+00_JPRB, 0.58703E+00_JPRB, 0.58052E+00_JPRB, 0.57652E+00_JPRB, 0.61712E+00_JPRB, 0.67340E+00_JPRB, 0.71188E+00_JPRB ,&
&  0.70313E+00_JPRB, 0.66609E+00_JPRB, 0.60757E+00_JPRB, 0.56081E+00_JPRB, 0.55289E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.56559E+00_JPRB, 0.58075E+00_JPRB, 0.59758E+00_JPRB, 0.58634E+00_JPRB, 0.56343E+00_JPRB, 0.55916E+00_JPRB, 0.52506E+00_JPRB ,&
&  0.50215E+00_JPRB, 0.48797E+00_JPRB, 0.48243E+00_JPRB, 0.49415E+00_JPRB, 0.51903E+00_JPRB, 0.53755E+00_JPRB, 0.53540E+00_JPRB ,&
&  0.52859E+00_JPRB, 0.56459E+00_JPRB, 0.67570E+00_JPRB, 0.95234E+00_JPRB, 0.12875E+01_JPRB, 0.16315E+01_JPRB, 0.17158E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.20098E+01_JPRB, 0.27413E+01_JPRB, 0.30970E+01_JPRB, 0.29103E+01_JPRB, 0.30680E+01_JPRB, 0.21754E+01_JPRB ,&
&  0.18055E+01_JPRB, 0.17667E+01_JPRB, 0.17347E+01_JPRB, 0.17159E+01_JPRB, 0.16590E+01_JPRB, 0.15678E+01_JPRB, 0.15193E+01_JPRB ,&
&  0.14751E+01_JPRB, 0.13827E+01_JPRB, 0.13457E+01_JPRB, 0.13410E+01_JPRB, 0.13639E+01_JPRB, 0.14016E+01_JPRB, 0.15305E+01_JPRB ,&
&  0.17670E+01_JPRB, 0.20328E+01_JPRB, 0.23269E+01_JPRB, 0.30429E+01_JPRB, 0.26288E+01_JPRB, 0.34869E+01_JPRB, 0.39418E+01_JPRB ,&
&  0.41247E+01_JPRB, 0.42154E+01_JPRB, 0.38583E+01_JPRB, 0.41851E+01_JPRB, 0.41023E+01_JPRB, 0.42033E+01_JPRB, 0.39078E+01_JPRB ,&
&  0.34554E+01_JPRB, 0.28686E+01_JPRB, 0.26680E+01_JPRB, 0.25915E+01_JPRB, 0.26172E+01_JPRB, 0.30238E+01_JPRB, 0.24275E+01_JPRB ,&
&  0.22449E+01_JPRB, 0.25439E+01_JPRB, 0.23615E+01_JPRB, 0.20756E+01_JPRB, 0.22798E+01_JPRB, 0.23642E+01_JPRB, 0.17749E+01_JPRB ,&
&  0.80579E+00_JPRB, 0.57884E+00_JPRB, 0.48289E+00_JPRB, 0.41741E+00_JPRB, 0.34865E+00_JPRB, 0.32736E+00_JPRB, 0.68552E+00_JPRB ,&
&  0.56015E+00_JPRB, 0.72151E+00_JPRB, 0.73419E+00_JPRB, 0.76827E+00_JPRB, 0.93304E+00_JPRB, 0.97933E+00_JPRB, 0.14076E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.10670E+01_JPRB, 0.92573E+00_JPRB, 0.81793E+00_JPRB, 0.77058E+00_JPRB, 0.76368E+00_JPRB, 0.76790E+00_JPRB ,&
&  0.75933E+00_JPRB, 0.72086E+00_JPRB, 0.67950E+00_JPRB, 0.61708E+00_JPRB, 0.56405E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.60923E+00_JPRB, 0.60886E+00_JPRB, 0.61126E+00_JPRB, 0.60656E+00_JPRB, 0.59193E+00_JPRB, 0.58974E+00_JPRB, 0.56760E+00_JPRB ,&
&  0.53177E+00_JPRB, 0.50666E+00_JPRB, 0.48723E+00_JPRB, 0.49259E+00_JPRB, 0.50337E+00_JPRB, 0.51344E+00_JPRB, 0.50541E+00_JPRB ,&
&  0.49444E+00_JPRB, 0.52531E+00_JPRB, 0.70180E+00_JPRB, 0.84253E+00_JPRB, 0.12225E+01_JPRB, 0.15529E+01_JPRB, 0.18173E+01_JPRB ,&
&  0.17450E+01_JPRB, 0.22256E+01_JPRB, 0.30227E+01_JPRB, 0.34095E+01_JPRB, 0.45208E+01_JPRB, 0.44644E+01_JPRB, 0.38025E+01_JPRB ,&
&  0.28591E+01_JPRB, 0.27171E+01_JPRB, 0.26050E+01_JPRB, 0.23850E+01_JPRB, 0.21047E+01_JPRB, 0.18327E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.14521E+01_JPRB, 0.13458E+01_JPRB, 0.12884E+01_JPRB, 0.12990E+01_JPRB, 0.13334E+01_JPRB, 0.14671E+01_JPRB ,&
&  0.16933E+01_JPRB, 0.19449E+01_JPRB, 0.22068E+01_JPRB, 0.28856E+01_JPRB, 0.29350E+01_JPRB, 0.32637E+01_JPRB, 0.40699E+01_JPRB ,&
&  0.44430E+01_JPRB, 0.46792E+01_JPRB, 0.42262E+01_JPRB, 0.49603E+01_JPRB, 0.45035E+01_JPRB, 0.45343E+01_JPRB, 0.44612E+01_JPRB ,&
&  0.38365E+01_JPRB, 0.34449E+01_JPRB, 0.27298E+01_JPRB, 0.26749E+01_JPRB, 0.28717E+01_JPRB, 0.26567E+01_JPRB, 0.26552E+01_JPRB ,&
&  0.27491E+01_JPRB, 0.24320E+01_JPRB, 0.22040E+01_JPRB, 0.20143E+01_JPRB, 0.24227E+01_JPRB, 0.21755E+01_JPRB, 0.10425E+01_JPRB ,&
&  0.75573E+00_JPRB, 0.69875E+00_JPRB, 0.53090E+00_JPRB, 0.45227E+00_JPRB, 0.38726E+00_JPRB, 0.36931E+00_JPRB, 0.55159E+00_JPRB ,&
&  0.58311E+00_JPRB, 0.80922E+00_JPRB, 0.88630E+00_JPRB, 0.10100E+01_JPRB, 0.11215E+01_JPRB, 0.12142E+01_JPRB, 0.19837E+01_JPRB ,&
&  0.19453E+01_JPRB, 0.16402E+01_JPRB, 0.13348E+01_JPRB, 0.11753E+01_JPRB, 0.10378E+01_JPRB, 0.92486E+00_JPRB, 0.85240E+00_JPRB ,&
&  0.81329E+00_JPRB, 0.75634E+00_JPRB, 0.72526E+00_JPRB, 0.70192E+00_JPRB, 0.64468E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.63993E+00_JPRB, 0.62015E+00_JPRB, 0.61279E+00_JPRB, 0.59149E+00_JPRB, 0.57303E+00_JPRB, 0.56787E+00_JPRB, 0.58046E+00_JPRB ,&
&  0.56566E+00_JPRB, 0.54130E+00_JPRB, 0.51734E+00_JPRB, 0.50931E+00_JPRB, 0.51814E+00_JPRB, 0.52407E+00_JPRB, 0.50986E+00_JPRB ,&
&  0.50087E+00_JPRB, 0.53191E+00_JPRB, 0.68435E+00_JPRB, 0.79921E+00_JPRB, 0.10224E+01_JPRB, 0.13320E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.18724E+01_JPRB, 0.23741E+01_JPRB, 0.30462E+01_JPRB, 0.41642E+01_JPRB, 0.55370E+01_JPRB, 0.61452E+01_JPRB, 0.65379E+01_JPRB ,&
&  0.53875E+01_JPRB, 0.42859E+01_JPRB, 0.36215E+01_JPRB, 0.31471E+01_JPRB, 0.26115E+01_JPRB, 0.21720E+01_JPRB, 0.18259E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.14390E+01_JPRB, 0.13923E+01_JPRB, 0.13068E+01_JPRB, 0.12730E+01_JPRB, 0.12885E+01_JPRB, 0.14256E+01_JPRB ,&
&  0.16276E+01_JPRB, 0.18904E+01_JPRB, 0.22389E+01_JPRB, 0.26920E+01_JPRB, 0.32893E+01_JPRB, 0.40328E+01_JPRB, 0.42824E+01_JPRB ,&
&  0.41362E+01_JPRB, 0.41024E+01_JPRB, 0.47644E+01_JPRB, 0.51343E+01_JPRB, 0.47810E+01_JPRB, 0.50228E+01_JPRB, 0.47846E+01_JPRB ,&
&  0.41084E+01_JPRB, 0.36254E+01_JPRB, 0.29457E+01_JPRB, 0.31236E+01_JPRB, 0.28348E+01_JPRB, 0.26142E+01_JPRB, 0.29886E+01_JPRB ,&
&  0.29458E+01_JPRB, 0.25898E+01_JPRB, 0.21758E+01_JPRB, 0.18649E+01_JPRB, 0.18284E+01_JPRB, 0.14172E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.86012E+00_JPRB, 0.71151E+00_JPRB, 0.59368E+00_JPRB, 0.59499E+00_JPRB, 0.49763E+00_JPRB, 0.49749E+00_JPRB, 0.64260E+00_JPRB ,&
&  0.76440E+00_JPRB, 0.84300E+00_JPRB, 0.10621E+01_JPRB, 0.11491E+01_JPRB, 0.11277E+01_JPRB, 0.10528E+01_JPRB, 0.15294E+01_JPRB ,&
&  0.12367E+01_JPRB, 0.12820E+01_JPRB, 0.12679E+01_JPRB, 0.11787E+01_JPRB, 0.10977E+01_JPRB, 0.10163E+01_JPRB, 0.92294E+00_JPRB ,&
&  0.83939E+00_JPRB, 0.76647E+00_JPRB, 0.71933E+00_JPRB, 0.70340E+00_JPRB, 0.67707E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.64322E+00_JPRB, 0.62285E+00_JPRB, 0.58928E+00_JPRB, 0.54003E+00_JPRB, 0.51968E+00_JPRB, 0.52620E+00_JPRB, 0.54871E+00_JPRB ,&
&  0.55755E+00_JPRB, 0.56051E+00_JPRB, 0.55032E+00_JPRB, 0.53774E+00_JPRB, 0.54010E+00_JPRB, 0.53546E+00_JPRB, 0.51980E+00_JPRB ,&
&  0.51387E+00_JPRB, 0.55505E+00_JPRB, 0.48923E+00_JPRB, 0.54581E+00_JPRB, 0.75717E+00_JPRB, 0.11732E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.17446E+01_JPRB, 0.23484E+01_JPRB, 0.29098E+01_JPRB, 0.42425E+01_JPRB, 0.74693E+01_JPRB, 0.76176E+01_JPRB, 0.73208E+01_JPRB ,&
&  0.67676E+01_JPRB, 0.54975E+01_JPRB, 0.43356E+01_JPRB, 0.34255E+01_JPRB, 0.27427E+01_JPRB, 0.22858E+01_JPRB, 0.19422E+01_JPRB ,&
&  0.17171E+01_JPRB, 0.15226E+01_JPRB, 0.13898E+01_JPRB, 0.13568E+01_JPRB, 0.13166E+01_JPRB, 0.12953E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.15650E+01_JPRB, 0.18163E+01_JPRB, 0.23170E+01_JPRB, 0.28649E+01_JPRB, 0.31485E+01_JPRB, 0.37066E+01_JPRB, 0.48902E+01_JPRB ,&
&  0.45039E+01_JPRB, 0.46322E+01_JPRB, 0.45000E+01_JPRB, 0.53565E+01_JPRB, 0.52163E+01_JPRB, 0.50583E+01_JPRB, 0.50792E+01_JPRB ,&
&  0.36962E+01_JPRB, 0.30104E+01_JPRB, 0.28978E+01_JPRB, 0.30882E+01_JPRB, 0.26178E+01_JPRB, 0.29435E+01_JPRB, 0.37624E+01_JPRB ,&
&  0.29628E+01_JPRB, 0.28354E+01_JPRB, 0.28081E+01_JPRB, 0.29124E+01_JPRB, 0.17858E+01_JPRB, 0.12295E+01_JPRB, 0.17962E+01_JPRB ,&
&  0.17006E+01_JPRB, 0.99359E+00_JPRB, 0.73204E+00_JPRB, 0.87255E+00_JPRB, 0.67239E+00_JPRB, 0.71869E+00_JPRB, 0.90842E+00_JPRB ,&
&  0.87830E+00_JPRB, 0.89914E+00_JPRB, 0.12368E+01_JPRB, 0.12812E+01_JPRB, 0.11931E+01_JPRB, 0.10445E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.12559E+01_JPRB, 0.12045E+01_JPRB, 0.12484E+01_JPRB, 0.11540E+01_JPRB, 0.10700E+01_JPRB, 0.10092E+01_JPRB, 0.93775E+00_JPRB ,&
&  0.87571E+00_JPRB, 0.79993E+00_JPRB, 0.73060E+00_JPRB, 0.68066E+00_JPRB, 0.65940E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.63540E+00_JPRB, 0.61513E+00_JPRB, 0.58951E+00_JPRB, 0.55538E+00_JPRB, 0.52001E+00_JPRB, 0.50718E+00_JPRB, 0.51134E+00_JPRB ,&
&  0.52005E+00_JPRB, 0.53509E+00_JPRB, 0.54409E+00_JPRB, 0.54814E+00_JPRB, 0.54997E+00_JPRB, 0.54747E+00_JPRB, 0.52552E+00_JPRB ,&
&  0.51991E+00_JPRB, 0.51208E+00_JPRB, 0.43859E+00_JPRB, 0.44910E+00_JPRB, 0.67390E+00_JPRB, 0.94184E+00_JPRB, 0.10654E+01_JPRB ,&
&  0.15098E+01_JPRB, 0.21311E+01_JPRB, 0.28850E+01_JPRB, 0.43226E+01_JPRB, 0.57986E+01_JPRB, 0.66947E+01_JPRB, 0.62889E+01_JPRB ,&
&  0.62273E+01_JPRB, 0.55805E+01_JPRB, 0.46311E+01_JPRB, 0.38440E+01_JPRB, 0.31220E+01_JPRB, 0.25534E+01_JPRB, 0.21272E+01_JPRB ,&
&  0.18497E+01_JPRB, 0.16566E+01_JPRB, 0.14963E+01_JPRB, 0.14039E+01_JPRB, 0.13874E+01_JPRB, 0.13560E+01_JPRB, 0.14118E+01_JPRB ,&
&  0.15020E+01_JPRB, 0.16910E+01_JPRB, 0.22937E+01_JPRB, 0.31594E+01_JPRB, 0.36113E+01_JPRB, 0.38195E+01_JPRB, 0.41159E+01_JPRB ,&
&  0.42562E+01_JPRB, 0.42448E+01_JPRB, 0.45006E+01_JPRB, 0.46138E+01_JPRB, 0.49642E+01_JPRB, 0.49367E+01_JPRB, 0.54037E+01_JPRB ,&
&  0.43693E+01_JPRB, 0.35572E+01_JPRB, 0.28837E+01_JPRB, 0.26557E+01_JPRB, 0.30317E+01_JPRB, 0.38922E+01_JPRB, 0.36656E+01_JPRB ,&
&  0.32422E+01_JPRB, 0.29852E+01_JPRB, 0.27117E+01_JPRB, 0.24114E+01_JPRB, 0.24649E+01_JPRB, 0.15644E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.16042E+01_JPRB, 0.14335E+01_JPRB, 0.11971E+01_JPRB, 0.10221E+01_JPRB, 0.10338E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.97481E+00_JPRB, 0.86789E+00_JPRB, 0.10165E+01_JPRB, 0.11670E+01_JPRB, 0.12439E+01_JPRB, 0.10975E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.12153E+01_JPRB, 0.12148E+01_JPRB, 0.11586E+01_JPRB, 0.10613E+01_JPRB, 0.97662E+00_JPRB, 0.91162E+00_JPRB ,&
&  0.85359E+00_JPRB, 0.79251E+00_JPRB, 0.72107E+00_JPRB, 0.67129E+00_JPRB, 0.65195E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.66623E+00_JPRB, 0.63900E+00_JPRB, 0.60485E+00_JPRB, 0.57429E+00_JPRB, 0.54196E+00_JPRB, 0.52575E+00_JPRB, 0.52391E+00_JPRB ,&
&  0.52840E+00_JPRB, 0.53103E+00_JPRB, 0.54185E+00_JPRB, 0.55453E+00_JPRB, 0.56247E+00_JPRB, 0.56204E+00_JPRB, 0.53636E+00_JPRB ,&
&  0.54277E+00_JPRB, 0.54979E+00_JPRB, 0.43637E+00_JPRB, 0.45841E+00_JPRB, 0.58337E+00_JPRB, 0.72400E+00_JPRB, 0.99786E+00_JPRB ,&
&  0.12566E+01_JPRB, 0.17468E+01_JPRB, 0.23587E+01_JPRB, 0.33358E+01_JPRB, 0.39933E+01_JPRB, 0.43277E+01_JPRB, 0.43349E+01_JPRB ,&
&  0.39666E+01_JPRB, 0.39486E+01_JPRB, 0.36953E+01_JPRB, 0.32855E+01_JPRB, 0.28758E+01_JPRB, 0.25577E+01_JPRB, 0.22899E+01_JPRB ,&
&  0.20557E+01_JPRB, 0.18631E+01_JPRB, 0.17188E+01_JPRB, 0.16034E+01_JPRB, 0.15539E+01_JPRB, 0.15247E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.14983E+01_JPRB, 0.15254E+01_JPRB, 0.19216E+01_JPRB, 0.28991E+01_JPRB, 0.33224E+01_JPRB, 0.38184E+01_JPRB, 0.38730E+01_JPRB ,&
&  0.45307E+01_JPRB, 0.48615E+01_JPRB, 0.48346E+01_JPRB, 0.54780E+01_JPRB, 0.48031E+01_JPRB, 0.51284E+01_JPRB, 0.54561E+01_JPRB ,&
&  0.56524E+01_JPRB, 0.51068E+01_JPRB, 0.45989E+01_JPRB, 0.40095E+01_JPRB, 0.34252E+01_JPRB, 0.40342E+01_JPRB, 0.34078E+01_JPRB ,&
&  0.29947E+01_JPRB, 0.28799E+01_JPRB, 0.26464E+01_JPRB, 0.26209E+01_JPRB, 0.21822E+01_JPRB, 0.19165E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.13133E+01_JPRB, 0.11528E+01_JPRB, 0.11484E+01_JPRB, 0.10145E+01_JPRB, 0.95435E+00_JPRB, 0.84962E+00_JPRB, 0.77493E+00_JPRB ,&
&  0.78969E+00_JPRB, 0.71718E+00_JPRB, 0.72975E+00_JPRB, 0.79946E+00_JPRB, 0.10249E+01_JPRB, 0.10043E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.11326E+01_JPRB, 0.10123E+01_JPRB, 0.10293E+01_JPRB, 0.96368E+00_JPRB, 0.90272E+00_JPRB, 0.86491E+00_JPRB ,&
&  0.80239E+00_JPRB, 0.76143E+00_JPRB, 0.72614E+00_JPRB, 0.68712E+00_JPRB, 0.67878E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.61756E+00_JPRB, 0.58978E+00_JPRB, 0.57436E+00_JPRB, 0.56572E+00_JPRB, 0.54529E+00_JPRB, 0.53281E+00_JPRB, 0.54510E+00_JPRB ,&
&  0.55019E+00_JPRB, 0.53902E+00_JPRB, 0.53431E+00_JPRB, 0.53932E+00_JPRB, 0.54883E+00_JPRB, 0.55557E+00_JPRB, 0.54785E+00_JPRB ,&
&  0.55814E+00_JPRB, 0.72456E+00_JPRB, 0.53988E+00_JPRB, 0.43642E+00_JPRB, 0.41678E+00_JPRB, 0.55008E+00_JPRB, 0.76103E+00_JPRB ,&
&  0.10336E+01_JPRB, 0.13198E+01_JPRB, 0.17930E+01_JPRB, 0.21480E+01_JPRB, 0.24578E+01_JPRB, 0.26376E+01_JPRB, 0.27720E+01_JPRB ,&
&  0.29560E+01_JPRB, 0.23007E+01_JPRB, 0.22033E+01_JPRB, 0.21175E+01_JPRB, 0.20674E+01_JPRB, 0.19990E+01_JPRB, 0.19707E+01_JPRB ,&
&  0.19003E+01_JPRB, 0.18412E+01_JPRB, 0.17759E+01_JPRB, 0.17023E+01_JPRB, 0.16297E+01_JPRB, 0.15675E+01_JPRB, 0.15175E+01_JPRB ,&
&  0.15123E+01_JPRB, 0.14919E+01_JPRB, 0.15863E+01_JPRB, 0.21035E+01_JPRB, 0.30736E+01_JPRB, 0.37937E+01_JPRB, 0.43291E+01_JPRB ,&
&  0.40136E+01_JPRB, 0.38475E+01_JPRB, 0.54569E+01_JPRB, 0.59068E+01_JPRB, 0.61917E+01_JPRB, 0.65083E+01_JPRB, 0.61978E+01_JPRB ,&
&  0.60569E+01_JPRB, 0.54028E+01_JPRB, 0.47125E+01_JPRB, 0.40332E+01_JPRB, 0.42586E+01_JPRB, 0.39605E+01_JPRB, 0.34691E+01_JPRB ,&
&  0.31387E+01_JPRB, 0.31363E+01_JPRB, 0.28725E+01_JPRB, 0.25651E+01_JPRB, 0.23414E+01_JPRB, 0.23214E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.14058E+01_JPRB, 0.12764E+01_JPRB, 0.94793E+00_JPRB, 0.75708E+00_JPRB, 0.65733E+00_JPRB, 0.58160E+00_JPRB, 0.54818E+00_JPRB ,&
&  0.54032E+00_JPRB, 0.55030E+00_JPRB, 0.58474E+00_JPRB, 0.62963E+00_JPRB, 0.78602E+00_JPRB, 0.87161E+00_JPRB, 0.90628E+00_JPRB ,&
&  0.83703E+00_JPRB, 0.91055E+00_JPRB, 0.93175E+00_JPRB, 0.92451E+00_JPRB, 0.94098E+00_JPRB, 0.84984E+00_JPRB, 0.81081E+00_JPRB ,&
&  0.75672E+00_JPRB, 0.73211E+00_JPRB, 0.72202E+00_JPRB, 0.68539E+00_JPRB, 0.65512E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.60281E+00_JPRB, 0.58592E+00_JPRB, 0.57994E+00_JPRB, 0.57294E+00_JPRB, 0.56811E+00_JPRB, 0.56262E+00_JPRB, 0.57053E+00_JPRB ,&
&  0.56592E+00_JPRB, 0.55845E+00_JPRB, 0.55049E+00_JPRB, 0.54824E+00_JPRB, 0.55014E+00_JPRB, 0.55092E+00_JPRB, 0.55590E+00_JPRB ,&
&  0.58560E+00_JPRB, 0.89058E+00_JPRB, 0.69275E+00_JPRB, 0.58919E+00_JPRB, 0.49842E+00_JPRB, 0.57434E+00_JPRB, 0.69808E+00_JPRB ,&
&  0.90333E+00_JPRB, 0.10051E+01_JPRB, 0.11923E+01_JPRB, 0.14197E+01_JPRB, 0.16391E+01_JPRB, 0.15946E+01_JPRB, 0.17004E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.14967E+01_JPRB, 0.12528E+01_JPRB, 0.13761E+01_JPRB, 0.13611E+01_JPRB, 0.12233E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.15371E+01_JPRB, 0.16005E+01_JPRB, 0.16476E+01_JPRB, 0.16727E+01_JPRB, 0.16788E+01_JPRB, 0.16376E+01_JPRB, 0.15784E+01_JPRB ,&
&  0.14974E+01_JPRB, 0.14814E+01_JPRB, 0.15597E+01_JPRB, 0.16989E+01_JPRB, 0.21241E+01_JPRB, 0.27172E+01_JPRB, 0.38662E+01_JPRB ,&
&  0.47019E+01_JPRB, 0.44175E+01_JPRB, 0.41782E+01_JPRB, 0.53207E+01_JPRB, 0.62861E+01_JPRB, 0.59053E+01_JPRB, 0.53824E+01_JPRB ,&
&  0.54665E+01_JPRB, 0.53374E+01_JPRB, 0.51101E+01_JPRB, 0.47949E+01_JPRB, 0.45179E+01_JPRB, 0.42097E+01_JPRB, 0.38102E+01_JPRB ,&
&  0.34493E+01_JPRB, 0.33347E+01_JPRB, 0.32400E+01_JPRB, 0.28682E+01_JPRB, 0.23336E+01_JPRB, 0.20591E+01_JPRB, 0.18327E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.99024E+00_JPRB, 0.60815E+00_JPRB, 0.46663E+00_JPRB, 0.41862E+00_JPRB, 0.38712E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.39141E+00_JPRB, 0.44335E+00_JPRB, 0.55334E+00_JPRB, 0.59047E+00_JPRB, 0.75831E+00_JPRB, 0.83537E+00_JPRB, 0.78005E+00_JPRB ,&
&  0.69821E+00_JPRB, 0.66094E+00_JPRB, 0.89408E+00_JPRB, 0.91474E+00_JPRB, 0.90369E+00_JPRB, 0.82626E+00_JPRB, 0.79950E+00_JPRB ,&
&  0.75954E+00_JPRB, 0.73968E+00_JPRB, 0.71644E+00_JPRB, 0.66953E+00_JPRB, 0.62742E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.59891E+00_JPRB, 0.59394E+00_JPRB, 0.59039E+00_JPRB, 0.58203E+00_JPRB, 0.57665E+00_JPRB, 0.56189E+00_JPRB, 0.56606E+00_JPRB ,&
&  0.56532E+00_JPRB, 0.56332E+00_JPRB, 0.55794E+00_JPRB, 0.55780E+00_JPRB, 0.55970E+00_JPRB, 0.55923E+00_JPRB, 0.56859E+00_JPRB ,&
&  0.66740E+00_JPRB, 0.93354E+00_JPRB, 0.81096E+00_JPRB, 0.62303E+00_JPRB, 0.72226E+00_JPRB, 0.70261E+00_JPRB, 0.76749E+00_JPRB ,&
&  0.79556E+00_JPRB, 0.88997E+00_JPRB, 0.91005E+00_JPRB, 0.92285E+00_JPRB, 0.10030E+01_JPRB, 0.10874E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.10160E+01_JPRB, 0.90803E+00_JPRB, 0.85257E+00_JPRB, 0.85518E+00_JPRB, 0.95865E+00_JPRB, 0.88609E+00_JPRB, 0.10296E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.12035E+01_JPRB, 0.12876E+01_JPRB, 0.13804E+01_JPRB, 0.14521E+01_JPRB, 0.15133E+01_JPRB, 0.15358E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.15367E+01_JPRB, 0.15484E+01_JPRB, 0.16483E+01_JPRB, 0.18758E+01_JPRB, 0.25959E+01_JPRB, 0.41586E+01_JPRB ,&
&  0.45895E+01_JPRB, 0.45242E+01_JPRB, 0.51617E+01_JPRB, 0.57542E+01_JPRB, 0.54526E+01_JPRB, 0.50448E+01_JPRB, 0.47784E+01_JPRB ,&
&  0.47141E+01_JPRB, 0.47391E+01_JPRB, 0.48885E+01_JPRB, 0.49303E+01_JPRB, 0.48636E+01_JPRB, 0.44934E+01_JPRB, 0.40741E+01_JPRB ,&
&  0.36632E+01_JPRB, 0.35132E+01_JPRB, 0.34121E+01_JPRB, 0.29242E+01_JPRB, 0.23377E+01_JPRB, 0.19124E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.16230E+01_JPRB, 0.79625E+00_JPRB, 0.52481E+00_JPRB, 0.41268E+00_JPRB, 0.37516E+00_JPRB, 0.37480E+00_JPRB, 0.44976E+00_JPRB ,&
&  0.40512E+00_JPRB, 0.48129E+00_JPRB, 0.61370E+00_JPRB, 0.64913E+00_JPRB, 0.67431E+00_JPRB, 0.70299E+00_JPRB, 0.55408E+00_JPRB ,&
&  0.50514E+00_JPRB, 0.58113E+00_JPRB, 0.81119E+00_JPRB, 0.85045E+00_JPRB, 0.85173E+00_JPRB, 0.82629E+00_JPRB, 0.82016E+00_JPRB ,&
&  0.77319E+00_JPRB, 0.73793E+00_JPRB, 0.67959E+00_JPRB, 0.63242E+00_JPRB, 0.61239E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.62792E+00_JPRB, 0.61156E+00_JPRB, 0.60185E+00_JPRB, 0.59248E+00_JPRB, 0.57454E+00_JPRB, 0.55624E+00_JPRB, 0.55984E+00_JPRB ,&
&  0.55773E+00_JPRB, 0.55282E+00_JPRB, 0.55375E+00_JPRB, 0.55443E+00_JPRB, 0.55087E+00_JPRB, 0.55812E+00_JPRB, 0.59880E+00_JPRB ,&
&  0.53401E+00_JPRB, 0.65737E+00_JPRB, 0.56895E+00_JPRB, 0.59987E+00_JPRB, 0.87773E+00_JPRB, 0.77734E+00_JPRB, 0.77221E+00_JPRB ,&
&  0.74526E+00_JPRB, 0.74819E+00_JPRB, 0.71557E+00_JPRB, 0.70488E+00_JPRB, 0.71188E+00_JPRB, 0.76848E+00_JPRB, 0.79475E+00_JPRB ,&
&  0.70495E+00_JPRB, 0.62233E+00_JPRB, 0.60890E+00_JPRB, 0.61398E+00_JPRB, 0.66729E+00_JPRB, 0.75273E+00_JPRB, 0.87389E+00_JPRB ,&
&  0.92588E+00_JPRB, 0.97430E+00_JPRB, 0.10469E+01_JPRB, 0.11255E+01_JPRB, 0.11978E+01_JPRB, 0.12687E+01_JPRB, 0.13026E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.13572E+01_JPRB, 0.14247E+01_JPRB, 0.15555E+01_JPRB, 0.16596E+01_JPRB, 0.25645E+01_JPRB, 0.45698E+01_JPRB ,&
&  0.51105E+01_JPRB, 0.43874E+01_JPRB, 0.43938E+01_JPRB, 0.51289E+01_JPRB, 0.50283E+01_JPRB, 0.46232E+01_JPRB, 0.41884E+01_JPRB ,&
&  0.39852E+01_JPRB, 0.39125E+01_JPRB, 0.39734E+01_JPRB, 0.41558E+01_JPRB, 0.42905E+01_JPRB, 0.43832E+01_JPRB, 0.42011E+01_JPRB ,&
&  0.38801E+01_JPRB, 0.36813E+01_JPRB, 0.34902E+01_JPRB, 0.31505E+01_JPRB, 0.25511E+01_JPRB, 0.21380E+01_JPRB, 0.16468E+01_JPRB ,&
&  0.13462E+01_JPRB, 0.99657E+00_JPRB, 0.66118E+00_JPRB, 0.52928E+00_JPRB, 0.46918E+00_JPRB, 0.50745E+00_JPRB, 0.66353E+00_JPRB ,&
&  0.51579E+00_JPRB, 0.55248E+00_JPRB, 0.56194E+00_JPRB, 0.55814E+00_JPRB, 0.54688E+00_JPRB, 0.51231E+00_JPRB, 0.45609E+00_JPRB ,&
&  0.47257E+00_JPRB, 0.61050E+00_JPRB, 0.75178E+00_JPRB, 0.78806E+00_JPRB, 0.78274E+00_JPRB, 0.76404E+00_JPRB, 0.70049E+00_JPRB ,&
&  0.78178E+00_JPRB, 0.69609E+00_JPRB, 0.65195E+00_JPRB, 0.63817E+00_JPRB, 0.64480E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.62545E+00_JPRB, 0.61588E+00_JPRB, 0.60440E+00_JPRB, 0.58615E+00_JPRB, 0.56468E+00_JPRB, 0.54696E+00_JPRB, 0.53960E+00_JPRB ,&
&  0.54427E+00_JPRB, 0.54582E+00_JPRB, 0.54651E+00_JPRB, 0.54274E+00_JPRB, 0.53224E+00_JPRB, 0.55094E+00_JPRB, 0.50615E+00_JPRB ,&
&  0.46231E+00_JPRB, 0.50936E+00_JPRB, 0.50477E+00_JPRB, 0.54610E+00_JPRB, 0.61215E+00_JPRB, 0.59686E+00_JPRB, 0.59533E+00_JPRB ,&
&  0.60731E+00_JPRB, 0.59579E+00_JPRB, 0.59880E+00_JPRB, 0.60311E+00_JPRB, 0.60339E+00_JPRB, 0.63467E+00_JPRB, 0.68602E+00_JPRB ,&
&  0.63530E+00_JPRB, 0.58155E+00_JPRB, 0.55576E+00_JPRB, 0.57350E+00_JPRB, 0.65547E+00_JPRB, 0.76000E+00_JPRB, 0.82079E+00_JPRB ,&
&  0.84538E+00_JPRB, 0.87370E+00_JPRB, 0.93831E+00_JPRB, 0.98547E+00_JPRB, 0.10209E+01_JPRB, 0.10574E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11762E+01_JPRB, 0.12634E+01_JPRB, 0.14119E+01_JPRB, 0.15432E+01_JPRB, 0.21077E+01_JPRB, 0.29694E+01_JPRB ,&
&  0.31989E+01_JPRB, 0.31701E+01_JPRB, 0.30931E+01_JPRB, 0.31808E+01_JPRB, 0.33769E+01_JPRB, 0.32340E+01_JPRB, 0.30702E+01_JPRB ,&
&  0.31250E+01_JPRB, 0.29960E+01_JPRB, 0.35368E+01_JPRB, 0.37518E+01_JPRB, 0.38221E+01_JPRB, 0.40282E+01_JPRB, 0.39575E+01_JPRB ,&
&  0.38538E+01_JPRB, 0.36711E+01_JPRB, 0.35611E+01_JPRB, 0.32048E+01_JPRB, 0.26653E+01_JPRB, 0.22912E+01_JPRB, 0.17575E+01_JPRB ,&
&  0.14059E+01_JPRB, 0.11033E+01_JPRB, 0.81965E+00_JPRB, 0.69787E+00_JPRB, 0.58665E+00_JPRB, 0.56827E+00_JPRB, 0.54361E+00_JPRB ,&
&  0.51414E+00_JPRB, 0.50240E+00_JPRB, 0.48778E+00_JPRB, 0.49626E+00_JPRB, 0.47846E+00_JPRB, 0.45103E+00_JPRB, 0.48726E+00_JPRB ,&
&  0.56224E+00_JPRB, 0.77093E+00_JPRB, 0.73902E+00_JPRB, 0.75690E+00_JPRB, 0.73435E+00_JPRB, 0.70725E+00_JPRB, 0.55350E+00_JPRB ,&
&  0.59834E+00_JPRB, 0.67391E+00_JPRB, 0.63769E+00_JPRB, 0.62794E+00_JPRB, 0.62705E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.60156E+00_JPRB, 0.60528E+00_JPRB, 0.58673E+00_JPRB, 0.56787E+00_JPRB, 0.55618E+00_JPRB, 0.52867E+00_JPRB, 0.50347E+00_JPRB ,&
&  0.55475E+00_JPRB, 0.54434E+00_JPRB, 0.54412E+00_JPRB, 0.54659E+00_JPRB, 0.54026E+00_JPRB, 0.48308E+00_JPRB, 0.43787E+00_JPRB ,&
&  0.41165E+00_JPRB, 0.41482E+00_JPRB, 0.44844E+00_JPRB, 0.46863E+00_JPRB, 0.48221E+00_JPRB, 0.47680E+00_JPRB, 0.47939E+00_JPRB ,&
&  0.48324E+00_JPRB, 0.49590E+00_JPRB, 0.53041E+00_JPRB, 0.55742E+00_JPRB, 0.56227E+00_JPRB, 0.57470E+00_JPRB, 0.62543E+00_JPRB ,&
&  0.60194E+00_JPRB, 0.60983E+00_JPRB, 0.62516E+00_JPRB, 0.57652E+00_JPRB, 0.69504E+00_JPRB, 0.73529E+00_JPRB, 0.76014E+00_JPRB ,&
&  0.80805E+00_JPRB, 0.85383E+00_JPRB, 0.88217E+00_JPRB, 0.90114E+00_JPRB, 0.93590E+00_JPRB, 0.95349E+00_JPRB, 0.97674E+00_JPRB ,&
&  0.10074E+01_JPRB, 0.10502E+01_JPRB, 0.11048E+01_JPRB, 0.11851E+01_JPRB, 0.13800E+01_JPRB, 0.16068E+01_JPRB, 0.19198E+01_JPRB ,&
&  0.21068E+01_JPRB, 0.21968E+01_JPRB, 0.23391E+01_JPRB, 0.21447E+01_JPRB, 0.26956E+01_JPRB, 0.29666E+01_JPRB, 0.27446E+01_JPRB ,&
&  0.27818E+01_JPRB, 0.25561E+01_JPRB, 0.28456E+01_JPRB, 0.31124E+01_JPRB, 0.33667E+01_JPRB, 0.35872E+01_JPRB, 0.36443E+01_JPRB ,&
&  0.35196E+01_JPRB, 0.35989E+01_JPRB, 0.34673E+01_JPRB, 0.30733E+01_JPRB, 0.25712E+01_JPRB, 0.22180E+01_JPRB, 0.17707E+01_JPRB ,&
&  0.14038E+01_JPRB, 0.11112E+01_JPRB, 0.93214E+00_JPRB, 0.84114E+00_JPRB, 0.68801E+00_JPRB, 0.63994E+00_JPRB, 0.59381E+00_JPRB ,&
&  0.54888E+00_JPRB, 0.51080E+00_JPRB, 0.49299E+00_JPRB, 0.47491E+00_JPRB, 0.46237E+00_JPRB, 0.46479E+00_JPRB, 0.46768E+00_JPRB ,&
&  0.51255E+00_JPRB, 0.59847E+00_JPRB, 0.77034E+00_JPRB, 0.72354E+00_JPRB, 0.68703E+00_JPRB, 0.65577E+00_JPRB, 0.57273E+00_JPRB ,&
&  0.54454E+00_JPRB, 0.63548E+00_JPRB, 0.62208E+00_JPRB, 0.60617E+00_JPRB, 0.59061E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.57145E+00_JPRB, 0.57558E+00_JPRB, 0.56338E+00_JPRB, 0.55085E+00_JPRB, 0.53348E+00_JPRB, 0.47822E+00_JPRB, 0.47217E+00_JPRB ,&
&  0.54111E+00_JPRB, 0.48797E+00_JPRB, 0.53689E+00_JPRB, 0.43546E+00_JPRB, 0.41220E+00_JPRB, 0.39396E+00_JPRB, 0.37664E+00_JPRB ,&
&  0.35803E+00_JPRB, 0.36601E+00_JPRB, 0.38861E+00_JPRB, 0.40680E+00_JPRB, 0.41884E+00_JPRB, 0.41647E+00_JPRB, 0.42444E+00_JPRB ,&
&  0.43970E+00_JPRB, 0.46912E+00_JPRB, 0.51014E+00_JPRB, 0.53052E+00_JPRB, 0.54829E+00_JPRB, 0.57454E+00_JPRB, 0.60063E+00_JPRB ,&
&  0.56621E+00_JPRB, 0.56590E+00_JPRB, 0.57629E+00_JPRB, 0.59801E+00_JPRB, 0.64089E+00_JPRB, 0.70607E+00_JPRB, 0.74784E+00_JPRB ,&
&  0.76655E+00_JPRB, 0.64271E+00_JPRB, 0.76110E+00_JPRB, 0.79497E+00_JPRB, 0.86187E+00_JPRB, 0.89857E+00_JPRB, 0.92259E+00_JPRB ,&
&  0.95887E+00_JPRB, 0.99406E+00_JPRB, 0.10328E+01_JPRB, 0.10808E+01_JPRB, 0.11433E+01_JPRB, 0.12748E+01_JPRB, 0.13772E+01_JPRB ,&
&  0.15929E+01_JPRB, 0.11461E+01_JPRB, 0.12869E+01_JPRB, 0.14445E+01_JPRB, 0.19946E+01_JPRB, 0.22393E+01_JPRB, 0.22535E+01_JPRB ,&
&  0.25191E+01_JPRB, 0.23865E+01_JPRB, 0.24627E+01_JPRB, 0.25085E+01_JPRB, 0.26507E+01_JPRB, 0.28641E+01_JPRB, 0.29915E+01_JPRB ,&
&  0.29993E+01_JPRB, 0.30386E+01_JPRB, 0.31264E+01_JPRB, 0.28290E+01_JPRB, 0.24564E+01_JPRB, 0.21000E+01_JPRB, 0.16819E+01_JPRB ,&
&  0.14060E+01_JPRB, 0.11609E+01_JPRB, 0.95152E+00_JPRB, 0.80427E+00_JPRB, 0.69972E+00_JPRB, 0.63678E+00_JPRB, 0.60446E+00_JPRB ,&
&  0.59273E+00_JPRB, 0.57231E+00_JPRB, 0.54939E+00_JPRB, 0.51454E+00_JPRB, 0.49666E+00_JPRB, 0.48966E+00_JPRB, 0.50398E+00_JPRB ,&
&  0.49903E+00_JPRB, 0.48235E+00_JPRB, 0.51176E+00_JPRB, 0.55187E+00_JPRB, 0.54917E+00_JPRB, 0.55566E+00_JPRB, 0.61365E+00_JPRB ,&
&  0.54278E+00_JPRB, 0.56690E+00_JPRB, 0.61611E+00_JPRB, 0.60615E+00_JPRB, 0.57810E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.56179E+00_JPRB, 0.54434E+00_JPRB, 0.53496E+00_JPRB, 0.52328E+00_JPRB, 0.50666E+00_JPRB, 0.46556E+00_JPRB, 0.43880E+00_JPRB ,&
&  0.45121E+00_JPRB, 0.43643E+00_JPRB, 0.42329E+00_JPRB, 0.39508E+00_JPRB, 0.37731E+00_JPRB, 0.35944E+00_JPRB, 0.34601E+00_JPRB ,&
&  0.33831E+00_JPRB, 0.35589E+00_JPRB, 0.38854E+00_JPRB, 0.39844E+00_JPRB, 0.40122E+00_JPRB, 0.40561E+00_JPRB, 0.42201E+00_JPRB ,&
&  0.44504E+00_JPRB, 0.47115E+00_JPRB, 0.48804E+00_JPRB, 0.50671E+00_JPRB, 0.52614E+00_JPRB, 0.54745E+00_JPRB, 0.54218E+00_JPRB ,&
&  0.53344E+00_JPRB, 0.51340E+00_JPRB, 0.49361E+00_JPRB, 0.52977E+00_JPRB, 0.59648E+00_JPRB, 0.65954E+00_JPRB, 0.63912E+00_JPRB ,&
&  0.45716E+00_JPRB, 0.45902E+00_JPRB, 0.64311E+00_JPRB, 0.70345E+00_JPRB, 0.76245E+00_JPRB, 0.79876E+00_JPRB, 0.84766E+00_JPRB ,&
&  0.89526E+00_JPRB, 0.93384E+00_JPRB, 0.96582E+00_JPRB, 0.10008E+01_JPRB, 0.10518E+01_JPRB, 0.11248E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.12680E+01_JPRB, 0.10241E+01_JPRB, 0.98021E+00_JPRB, 0.10876E+01_JPRB, 0.15497E+01_JPRB, 0.16657E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.17832E+01_JPRB, 0.18397E+01_JPRB, 0.19506E+01_JPRB, 0.20660E+01_JPRB, 0.20781E+01_JPRB, 0.21714E+01_JPRB, 0.23256E+01_JPRB ,&
&  0.25720E+01_JPRB, 0.27952E+01_JPRB, 0.27820E+01_JPRB, 0.26625E+01_JPRB, 0.24548E+01_JPRB, 0.22205E+01_JPRB, 0.18730E+01_JPRB ,&
&  0.14733E+01_JPRB, 0.11884E+01_JPRB, 0.94958E+00_JPRB, 0.78944E+00_JPRB, 0.70321E+00_JPRB, 0.66776E+00_JPRB, 0.64315E+00_JPRB ,&
&  0.64926E+00_JPRB, 0.64780E+00_JPRB, 0.62838E+00_JPRB, 0.57573E+00_JPRB, 0.53499E+00_JPRB, 0.52056E+00_JPRB, 0.50954E+00_JPRB ,&
&  0.49105E+00_JPRB, 0.46054E+00_JPRB, 0.41867E+00_JPRB, 0.42249E+00_JPRB, 0.46087E+00_JPRB, 0.50036E+00_JPRB, 0.51774E+00_JPRB ,&
&  0.52493E+00_JPRB, 0.51840E+00_JPRB, 0.52453E+00_JPRB, 0.54663E+00_JPRB, 0.55506E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.55254E+00_JPRB, 0.50991E+00_JPRB, 0.48571E+00_JPRB, 0.48777E+00_JPRB, 0.45110E+00_JPRB, 0.42132E+00_JPRB, 0.40687E+00_JPRB ,&
&  0.41118E+00_JPRB, 0.41444E+00_JPRB, 0.41631E+00_JPRB, 0.39635E+00_JPRB, 0.37987E+00_JPRB, 0.37073E+00_JPRB, 0.36880E+00_JPRB ,&
&  0.38236E+00_JPRB, 0.39930E+00_JPRB, 0.41294E+00_JPRB, 0.41056E+00_JPRB, 0.41115E+00_JPRB, 0.41982E+00_JPRB, 0.43782E+00_JPRB ,&
&  0.46253E+00_JPRB, 0.47725E+00_JPRB, 0.47777E+00_JPRB, 0.47988E+00_JPRB, 0.50089E+00_JPRB, 0.52224E+00_JPRB, 0.49104E+00_JPRB ,&
&  0.45773E+00_JPRB, 0.43455E+00_JPRB, 0.43080E+00_JPRB, 0.47864E+00_JPRB, 0.55045E+00_JPRB, 0.58496E+00_JPRB, 0.46152E+00_JPRB ,&
&  0.37164E+00_JPRB, 0.35474E+00_JPRB, 0.39484E+00_JPRB, 0.53803E+00_JPRB, 0.58664E+00_JPRB, 0.63875E+00_JPRB, 0.71740E+00_JPRB ,&
&  0.67472E+00_JPRB, 0.66179E+00_JPRB, 0.68460E+00_JPRB, 0.83627E+00_JPRB, 0.89106E+00_JPRB, 0.93525E+00_JPRB, 0.96651E+00_JPRB ,&
&  0.99309E+00_JPRB, 0.10649E+01_JPRB, 0.12502E+01_JPRB, 0.10910E+01_JPRB, 0.12226E+01_JPRB, 0.14952E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.16072E+01_JPRB, 0.17149E+01_JPRB, 0.19959E+01_JPRB, 0.19642E+01_JPRB, 0.18174E+01_JPRB, 0.17813E+01_JPRB, 0.19094E+01_JPRB ,&
&  0.22540E+01_JPRB, 0.25100E+01_JPRB, 0.24146E+01_JPRB, 0.22755E+01_JPRB, 0.21682E+01_JPRB, 0.20305E+01_JPRB, 0.17759E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.11856E+01_JPRB, 0.98985E+00_JPRB, 0.83033E+00_JPRB, 0.76212E+00_JPRB, 0.74177E+00_JPRB, 0.71585E+00_JPRB ,&
&  0.71402E+00_JPRB, 0.70851E+00_JPRB, 0.69313E+00_JPRB, 0.62579E+00_JPRB, 0.56292E+00_JPRB, 0.50748E+00_JPRB, 0.46714E+00_JPRB ,&
&  0.44333E+00_JPRB, 0.41542E+00_JPRB, 0.39807E+00_JPRB, 0.40815E+00_JPRB, 0.45249E+00_JPRB, 0.50366E+00_JPRB, 0.52681E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.50266E+00_JPRB, 0.52368E+00_JPRB, 0.55939E+00_JPRB, 0.57230E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.58491E+00_JPRB, 0.57735E+00_JPRB, 0.55715E+00_JPRB, 0.52262E+00_JPRB, 0.49242E+00_JPRB, 0.45996E+00_JPRB, 0.43739E+00_JPRB ,&
&  0.42620E+00_JPRB, 0.42096E+00_JPRB, 0.41903E+00_JPRB, 0.42108E+00_JPRB, 0.42534E+00_JPRB, 0.43342E+00_JPRB, 0.43670E+00_JPRB ,&
&  0.44209E+00_JPRB, 0.44622E+00_JPRB, 0.45307E+00_JPRB, 0.46337E+00_JPRB, 0.47508E+00_JPRB, 0.48807E+00_JPRB, 0.50067E+00_JPRB ,&
&  0.51107E+00_JPRB, 0.50435E+00_JPRB, 0.48201E+00_JPRB, 0.46844E+00_JPRB, 0.47263E+00_JPRB, 0.48598E+00_JPRB, 0.46780E+00_JPRB ,&
&  0.42698E+00_JPRB, 0.40220E+00_JPRB, 0.40120E+00_JPRB, 0.46370E+00_JPRB, 0.50248E+00_JPRB, 0.52267E+00_JPRB, 0.40499E+00_JPRB ,&
&  0.34019E+00_JPRB, 0.31187E+00_JPRB, 0.31002E+00_JPRB, 0.33606E+00_JPRB, 0.39411E+00_JPRB, 0.48224E+00_JPRB, 0.55008E+00_JPRB ,&
&  0.59911E+00_JPRB, 0.64328E+00_JPRB, 0.65742E+00_JPRB, 0.68274E+00_JPRB, 0.72648E+00_JPRB, 0.77267E+00_JPRB, 0.82197E+00_JPRB ,&
&  0.86176E+00_JPRB, 0.92882E+00_JPRB, 0.10813E+01_JPRB, 0.98507E+00_JPRB, 0.10355E+01_JPRB, 0.11061E+01_JPRB, 0.12462E+01_JPRB ,&
&  0.15118E+01_JPRB, 0.16432E+01_JPRB, 0.17499E+01_JPRB, 0.19478E+01_JPRB, 0.18768E+01_JPRB, 0.18240E+01_JPRB, 0.18637E+01_JPRB ,&
&  0.21239E+01_JPRB, 0.23428E+01_JPRB, 0.22605E+01_JPRB, 0.20234E+01_JPRB, 0.19286E+01_JPRB, 0.18678E+01_JPRB, 0.16379E+01_JPRB ,&
&  0.13544E+01_JPRB, 0.11900E+01_JPRB, 0.10525E+01_JPRB, 0.90559E+00_JPRB, 0.85186E+00_JPRB, 0.82322E+00_JPRB, 0.80596E+00_JPRB ,&
&  0.79299E+00_JPRB, 0.77527E+00_JPRB, 0.75665E+00_JPRB, 0.69256E+00_JPRB, 0.62914E+00_JPRB, 0.56404E+00_JPRB, 0.51764E+00_JPRB ,&
&  0.48741E+00_JPRB, 0.47490E+00_JPRB, 0.47501E+00_JPRB, 0.48807E+00_JPRB, 0.51988E+00_JPRB, 0.54617E+00_JPRB, 0.56273E+00_JPRB ,&
&  0.56354E+00_JPRB, 0.54704E+00_JPRB, 0.54105E+00_JPRB, 0.54512E+00_JPRB, 0.57215E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.65002E+00_JPRB, 0.65641E+00_JPRB, 0.64339E+00_JPRB, 0.61422E+00_JPRB, 0.57866E+00_JPRB, 0.53975E+00_JPRB, 0.51751E+00_JPRB ,&
&  0.50810E+00_JPRB, 0.50820E+00_JPRB, 0.50430E+00_JPRB, 0.49989E+00_JPRB, 0.49930E+00_JPRB, 0.50052E+00_JPRB, 0.50352E+00_JPRB ,&
&  0.50971E+00_JPRB, 0.51928E+00_JPRB, 0.52806E+00_JPRB, 0.52765E+00_JPRB, 0.53331E+00_JPRB, 0.53900E+00_JPRB, 0.54480E+00_JPRB ,&
&  0.54084E+00_JPRB, 0.51668E+00_JPRB, 0.48574E+00_JPRB, 0.46841E+00_JPRB, 0.44968E+00_JPRB, 0.44379E+00_JPRB, 0.42956E+00_JPRB ,&
&  0.40206E+00_JPRB, 0.39768E+00_JPRB, 0.42667E+00_JPRB, 0.45592E+00_JPRB, 0.45189E+00_JPRB, 0.47416E+00_JPRB, 0.35791E+00_JPRB ,&
&  0.31698E+00_JPRB, 0.29484E+00_JPRB, 0.28922E+00_JPRB, 0.29782E+00_JPRB, 0.32373E+00_JPRB, 0.36041E+00_JPRB, 0.40609E+00_JPRB ,&
&  0.48111E+00_JPRB, 0.54816E+00_JPRB, 0.58937E+00_JPRB, 0.62718E+00_JPRB, 0.66697E+00_JPRB, 0.70436E+00_JPRB, 0.73091E+00_JPRB ,&
&  0.75355E+00_JPRB, 0.79947E+00_JPRB, 0.87405E+00_JPRB, 0.97869E+00_JPRB, 0.11904E+01_JPRB, 0.11784E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.16407E+01_JPRB, 0.18875E+01_JPRB, 0.18187E+01_JPRB, 0.17615E+01_JPRB, 0.17737E+01_JPRB, 0.18655E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.21462E+01_JPRB, 0.22710E+01_JPRB, 0.22264E+01_JPRB, 0.19817E+01_JPRB, 0.18538E+01_JPRB, 0.17753E+01_JPRB, 0.15844E+01_JPRB ,&
&  0.13710E+01_JPRB, 0.13685E+01_JPRB, 0.14763E+01_JPRB, 0.12047E+01_JPRB, 0.10318E+01_JPRB, 0.94127E+00_JPRB, 0.89832E+00_JPRB ,&
&  0.86488E+00_JPRB, 0.83292E+00_JPRB, 0.79557E+00_JPRB, 0.75888E+00_JPRB, 0.71795E+00_JPRB, 0.67961E+00_JPRB, 0.64592E+00_JPRB ,&
&  0.62118E+00_JPRB, 0.60046E+00_JPRB, 0.58752E+00_JPRB, 0.57722E+00_JPRB, 0.56373E+00_JPRB, 0.55623E+00_JPRB, 0.56024E+00_JPRB ,&
&  0.57125E+00_JPRB, 0.57804E+00_JPRB, 0.59212E+00_JPRB, 0.60759E+00_JPRB, 0.62894E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.65716E+00_JPRB, 0.66663E+00_JPRB, 0.67186E+00_JPRB, 0.66651E+00_JPRB, 0.65052E+00_JPRB, 0.63083E+00_JPRB, 0.61547E+00_JPRB ,&
&  0.60432E+00_JPRB, 0.59454E+00_JPRB, 0.58370E+00_JPRB, 0.57520E+00_JPRB, 0.56798E+00_JPRB, 0.56457E+00_JPRB, 0.56235E+00_JPRB ,&
&  0.56309E+00_JPRB, 0.55195E+00_JPRB, 0.54982E+00_JPRB, 0.55373E+00_JPRB, 0.55975E+00_JPRB, 0.55936E+00_JPRB, 0.55537E+00_JPRB ,&
&  0.53366E+00_JPRB, 0.50332E+00_JPRB, 0.47548E+00_JPRB, 0.45352E+00_JPRB, 0.43643E+00_JPRB, 0.42682E+00_JPRB, 0.43038E+00_JPRB ,&
&  0.43807E+00_JPRB, 0.43557E+00_JPRB, 0.43645E+00_JPRB, 0.45495E+00_JPRB, 0.46542E+00_JPRB, 0.40500E+00_JPRB, 0.32069E+00_JPRB ,&
&  0.30440E+00_JPRB, 0.28878E+00_JPRB, 0.28801E+00_JPRB, 0.29295E+00_JPRB, 0.30650E+00_JPRB, 0.33050E+00_JPRB, 0.37145E+00_JPRB ,&
&  0.43193E+00_JPRB, 0.55513E+00_JPRB, 0.62103E+00_JPRB, 0.66795E+00_JPRB, 0.70899E+00_JPRB, 0.74201E+00_JPRB, 0.75486E+00_JPRB ,&
&  0.76524E+00_JPRB, 0.78941E+00_JPRB, 0.83517E+00_JPRB, 0.89062E+00_JPRB, 0.97852E+00_JPRB, 0.11489E+01_JPRB, 0.13037E+01_JPRB ,&
&  0.15188E+01_JPRB, 0.16438E+01_JPRB, 0.16575E+01_JPRB, 0.16167E+01_JPRB, 0.16860E+01_JPRB, 0.18051E+01_JPRB, 0.18502E+01_JPRB ,&
&  0.19337E+01_JPRB, 0.20204E+01_JPRB, 0.19716E+01_JPRB, 0.18498E+01_JPRB, 0.17713E+01_JPRB, 0.17157E+01_JPRB, 0.15701E+01_JPRB ,&
&  0.14136E+01_JPRB, 0.13443E+01_JPRB, 0.13529E+01_JPRB, 0.11913E+01_JPRB, 0.10873E+01_JPRB, 0.99291E+00_JPRB, 0.93001E+00_JPRB ,&
&  0.88587E+00_JPRB, 0.84982E+00_JPRB, 0.80072E+00_JPRB, 0.76046E+00_JPRB, 0.73631E+00_JPRB, 0.71777E+00_JPRB, 0.69969E+00_JPRB ,&
&  0.67960E+00_JPRB, 0.65866E+00_JPRB, 0.63669E+00_JPRB, 0.61670E+00_JPRB, 0.59499E+00_JPRB, 0.58222E+00_JPRB, 0.58046E+00_JPRB ,&
&  0.58587E+00_JPRB, 0.59891E+00_JPRB, 0.61394E+00_JPRB, 0.63030E+00_JPRB, 0.64658E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.68198E+00_JPRB, 0.68610E+00_JPRB, 0.68530E+00_JPRB, 0.67605E+00_JPRB, 0.66070E+00_JPRB, 0.64247E+00_JPRB, 0.62679E+00_JPRB ,&
&  0.61377E+00_JPRB, 0.60406E+00_JPRB, 0.59364E+00_JPRB, 0.58603E+00_JPRB, 0.57674E+00_JPRB, 0.57164E+00_JPRB, 0.56516E+00_JPRB ,&
&  0.56302E+00_JPRB, 0.56306E+00_JPRB, 0.55756E+00_JPRB, 0.55052E+00_JPRB, 0.53494E+00_JPRB, 0.52039E+00_JPRB, 0.50907E+00_JPRB ,&
&  0.49811E+00_JPRB, 0.48944E+00_JPRB, 0.47004E+00_JPRB, 0.45551E+00_JPRB, 0.44856E+00_JPRB, 0.44315E+00_JPRB, 0.45169E+00_JPRB ,&
&  0.45099E+00_JPRB, 0.45729E+00_JPRB, 0.46324E+00_JPRB, 0.45706E+00_JPRB, 0.44894E+00_JPRB, 0.34657E+00_JPRB, 0.31807E+00_JPRB ,&
&  0.30892E+00_JPRB, 0.30184E+00_JPRB, 0.30062E+00_JPRB, 0.30659E+00_JPRB, 0.31774E+00_JPRB, 0.34698E+00_JPRB, 0.37581E+00_JPRB ,&
&  0.40479E+00_JPRB, 0.56810E+00_JPRB, 0.63509E+00_JPRB, 0.69677E+00_JPRB, 0.74130E+00_JPRB, 0.77434E+00_JPRB, 0.79465E+00_JPRB ,&
&  0.81636E+00_JPRB, 0.84776E+00_JPRB, 0.86986E+00_JPRB, 0.91359E+00_JPRB, 0.99235E+00_JPRB, 0.11223E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.14550E+01_JPRB, 0.15255E+01_JPRB, 0.15668E+01_JPRB, 0.16215E+01_JPRB, 0.16557E+01_JPRB, 0.16676E+01_JPRB ,&
&  0.16858E+01_JPRB, 0.16601E+01_JPRB, 0.16747E+01_JPRB, 0.16439E+01_JPRB, 0.16153E+01_JPRB, 0.15764E+01_JPRB, 0.14728E+01_JPRB ,&
&  0.13471E+01_JPRB, 0.12583E+01_JPRB, 0.11737E+01_JPRB, 0.10745E+01_JPRB, 0.10025E+01_JPRB, 0.95006E+00_JPRB, 0.92532E+00_JPRB ,&
&  0.90890E+00_JPRB, 0.89764E+00_JPRB, 0.86122E+00_JPRB, 0.81993E+00_JPRB, 0.79198E+00_JPRB, 0.77252E+00_JPRB, 0.74162E+00_JPRB ,&
&  0.71108E+00_JPRB, 0.67964E+00_JPRB, 0.65778E+00_JPRB, 0.63706E+00_JPRB, 0.61896E+00_JPRB, 0.60879E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.60957E+00_JPRB, 0.61849E+00_JPRB, 0.63470E+00_JPRB, 0.65367E+00_JPRB, 0.67367E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.71002E+00_JPRB, 0.69617E+00_JPRB, 0.66844E+00_JPRB, 0.63102E+00_JPRB, 0.60383E+00_JPRB, 0.58563E+00_JPRB, 0.57841E+00_JPRB ,&
&  0.57500E+00_JPRB, 0.57447E+00_JPRB, 0.57309E+00_JPRB, 0.57090E+00_JPRB, 0.56556E+00_JPRB, 0.55833E+00_JPRB, 0.55045E+00_JPRB ,&
&  0.53840E+00_JPRB, 0.52633E+00_JPRB, 0.51386E+00_JPRB, 0.50646E+00_JPRB, 0.50001E+00_JPRB, 0.49542E+00_JPRB, 0.49139E+00_JPRB ,&
&  0.48673E+00_JPRB, 0.48083E+00_JPRB, 0.46123E+00_JPRB, 0.45308E+00_JPRB, 0.44508E+00_JPRB, 0.44627E+00_JPRB, 0.44376E+00_JPRB ,&
&  0.45943E+00_JPRB, 0.40547E+00_JPRB, 0.39786E+00_JPRB, 0.38017E+00_JPRB, 0.36516E+00_JPRB, 0.34910E+00_JPRB, 0.33668E+00_JPRB ,&
&  0.33019E+00_JPRB, 0.32403E+00_JPRB, 0.32247E+00_JPRB, 0.32976E+00_JPRB, 0.33941E+00_JPRB, 0.35930E+00_JPRB, 0.38235E+00_JPRB ,&
&  0.42783E+00_JPRB, 0.57767E+00_JPRB, 0.65267E+00_JPRB, 0.71204E+00_JPRB, 0.76980E+00_JPRB, 0.82211E+00_JPRB, 0.85325E+00_JPRB ,&
&  0.88714E+00_JPRB, 0.93017E+00_JPRB, 0.96952E+00_JPRB, 0.99080E+00_JPRB, 0.10891E+01_JPRB, 0.11606E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.13096E+01_JPRB, 0.13750E+01_JPRB, 0.14312E+01_JPRB, 0.14715E+01_JPRB, 0.15007E+01_JPRB, 0.14970E+01_JPRB, 0.14714E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.14428E+01_JPRB, 0.14383E+01_JPRB, 0.13926E+01_JPRB, 0.13661E+01_JPRB, 0.13408E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.11616E+01_JPRB, 0.10972E+01_JPRB, 0.10338E+01_JPRB, 0.97564E+00_JPRB, 0.94119E+00_JPRB, 0.92587E+00_JPRB ,&
&  0.92090E+00_JPRB, 0.90849E+00_JPRB, 0.88542E+00_JPRB, 0.85174E+00_JPRB, 0.81862E+00_JPRB, 0.79552E+00_JPRB, 0.76769E+00_JPRB ,&
&  0.73889E+00_JPRB, 0.70686E+00_JPRB, 0.68460E+00_JPRB, 0.66496E+00_JPRB, 0.65341E+00_JPRB, 0.64845E+00_JPRB, 0.64894E+00_JPRB ,&
&  0.65233E+00_JPRB, 0.66037E+00_JPRB, 0.67544E+00_JPRB, 0.69209E+00_JPRB, 0.70555E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.66005E+00_JPRB, 0.62907E+00_JPRB, 0.59567E+00_JPRB, 0.57725E+00_JPRB, 0.57288E+00_JPRB, 0.56852E+00_JPRB, 0.56540E+00_JPRB ,&
&  0.56302E+00_JPRB, 0.56547E+00_JPRB, 0.56836E+00_JPRB, 0.56939E+00_JPRB, 0.56714E+00_JPRB, 0.56274E+00_JPRB, 0.55630E+00_JPRB ,&
&  0.54751E+00_JPRB, 0.53773E+00_JPRB, 0.52845E+00_JPRB, 0.52029E+00_JPRB, 0.51132E+00_JPRB, 0.50271E+00_JPRB, 0.49123E+00_JPRB ,&
&  0.47979E+00_JPRB, 0.46919E+00_JPRB, 0.43293E+00_JPRB, 0.42960E+00_JPRB, 0.42164E+00_JPRB, 0.41991E+00_JPRB, 0.42276E+00_JPRB ,&
&  0.42152E+00_JPRB, 0.41291E+00_JPRB, 0.40421E+00_JPRB, 0.38940E+00_JPRB, 0.37684E+00_JPRB, 0.36280E+00_JPRB, 0.35517E+00_JPRB ,&
&  0.35225E+00_JPRB, 0.35459E+00_JPRB, 0.36147E+00_JPRB, 0.37135E+00_JPRB, 0.38443E+00_JPRB, 0.39993E+00_JPRB, 0.42305E+00_JPRB ,&
&  0.47905E+00_JPRB, 0.55810E+00_JPRB, 0.65300E+00_JPRB, 0.70743E+00_JPRB, 0.76894E+00_JPRB, 0.82820E+00_JPRB, 0.87160E+00_JPRB ,&
&  0.91339E+00_JPRB, 0.95829E+00_JPRB, 0.10077E+01_JPRB, 0.10622E+01_JPRB, 0.11153E+01_JPRB, 0.11596E+01_JPRB, 0.12146E+01_JPRB ,&
&  0.12606E+01_JPRB, 0.12965E+01_JPRB, 0.13132E+01_JPRB, 0.13194E+01_JPRB, 0.13177E+01_JPRB, 0.13029E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.12747E+01_JPRB, 0.12631E+01_JPRB, 0.12483E+01_JPRB, 0.12122E+01_JPRB, 0.11935E+01_JPRB, 0.11820E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.10932E+01_JPRB, 0.10600E+01_JPRB, 0.10161E+01_JPRB, 0.98087E+00_JPRB, 0.96814E+00_JPRB, 0.95771E+00_JPRB ,&
&  0.94699E+00_JPRB, 0.93316E+00_JPRB, 0.91793E+00_JPRB, 0.89709E+00_JPRB, 0.86662E+00_JPRB, 0.83695E+00_JPRB, 0.81074E+00_JPRB ,&
&  0.78899E+00_JPRB, 0.76621E+00_JPRB, 0.74580E+00_JPRB, 0.72302E+00_JPRB, 0.70832E+00_JPRB, 0.69508E+00_JPRB, 0.69047E+00_JPRB ,&
&  0.68973E+00_JPRB, 0.69304E+00_JPRB, 0.69445E+00_JPRB, 0.69276E+00_JPRB, 0.68113E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.63936E+00_JPRB, 0.61061E+00_JPRB, 0.59509E+00_JPRB, 0.59084E+00_JPRB, 0.58777E+00_JPRB, 0.58461E+00_JPRB, 0.58197E+00_JPRB ,&
&  0.57664E+00_JPRB, 0.57234E+00_JPRB, 0.57174E+00_JPRB, 0.57169E+00_JPRB, 0.56960E+00_JPRB, 0.56694E+00_JPRB, 0.56348E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.55379E+00_JPRB, 0.54816E+00_JPRB, 0.54089E+00_JPRB, 0.53317E+00_JPRB, 0.52404E+00_JPRB, 0.51248E+00_JPRB ,&
&  0.50249E+00_JPRB, 0.49329E+00_JPRB, 0.48122E+00_JPRB, 0.47617E+00_JPRB, 0.47371E+00_JPRB, 0.44752E+00_JPRB, 0.44107E+00_JPRB ,&
&  0.44490E+00_JPRB, 0.44546E+00_JPRB, 0.44660E+00_JPRB, 0.44853E+00_JPRB, 0.45281E+00_JPRB, 0.44575E+00_JPRB, 0.44432E+00_JPRB ,&
&  0.42604E+00_JPRB, 0.43834E+00_JPRB, 0.44766E+00_JPRB, 0.45770E+00_JPRB, 0.46929E+00_JPRB, 0.48812E+00_JPRB, 0.52086E+00_JPRB ,&
&  0.58317E+00_JPRB, 0.61587E+00_JPRB, 0.65118E+00_JPRB, 0.68683E+00_JPRB, 0.72975E+00_JPRB, 0.77220E+00_JPRB, 0.80882E+00_JPRB ,&
&  0.84212E+00_JPRB, 0.87786E+00_JPRB, 0.91910E+00_JPRB, 0.97104E+00_JPRB, 0.10148E+01_JPRB, 0.10545E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.11156E+01_JPRB, 0.11386E+01_JPRB, 0.11550E+01_JPRB, 0.11655E+01_JPRB, 0.11768E+01_JPRB, 0.11821E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.11909E+01_JPRB, 0.11812E+01_JPRB, 0.11635E+01_JPRB, 0.11506E+01_JPRB, 0.11427E+01_JPRB, 0.11304E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11008E+01_JPRB, 0.10822E+01_JPRB, 0.10604E+01_JPRB, 0.10342E+01_JPRB, 0.10104E+01_JPRB, 0.98328E+00_JPRB ,&
&  0.96045E+00_JPRB, 0.94204E+00_JPRB, 0.92682E+00_JPRB, 0.90891E+00_JPRB, 0.88956E+00_JPRB, 0.87097E+00_JPRB, 0.85048E+00_JPRB ,&
&  0.83257E+00_JPRB, 0.81550E+00_JPRB, 0.80221E+00_JPRB, 0.78760E+00_JPRB, 0.77753E+00_JPRB, 0.76391E+00_JPRB, 0.75234E+00_JPRB ,&
&  0.73955E+00_JPRB, 0.73073E+00_JPRB, 0.71464E+00_JPRB, 0.69281E+00_JPRB, 0.66401E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.62472E+00_JPRB, 0.61233E+00_JPRB, 0.60644E+00_JPRB, 0.60279E+00_JPRB, 0.59940E+00_JPRB, 0.59476E+00_JPRB, 0.58973E+00_JPRB ,&
&  0.58550E+00_JPRB, 0.58054E+00_JPRB, 0.57577E+00_JPRB, 0.57365E+00_JPRB, 0.57059E+00_JPRB, 0.56868E+00_JPRB, 0.56676E+00_JPRB ,&
&  0.56537E+00_JPRB, 0.56355E+00_JPRB, 0.56226E+00_JPRB, 0.56027E+00_JPRB, 0.55744E+00_JPRB, 0.55359E+00_JPRB, 0.55019E+00_JPRB ,&
&  0.54821E+00_JPRB, 0.54682E+00_JPRB, 0.54449E+00_JPRB, 0.54262E+00_JPRB, 0.54277E+00_JPRB, 0.54217E+00_JPRB, 0.54078E+00_JPRB ,&
&  0.54265E+00_JPRB, 0.54397E+00_JPRB, 0.54383E+00_JPRB, 0.54245E+00_JPRB, 0.54054E+00_JPRB, 0.54074E+00_JPRB, 0.54400E+00_JPRB ,&
&  0.54851E+00_JPRB, 0.56110E+00_JPRB, 0.57102E+00_JPRB, 0.58057E+00_JPRB, 0.59013E+00_JPRB, 0.60170E+00_JPRB, 0.61568E+00_JPRB ,&
&  0.63034E+00_JPRB, 0.64289E+00_JPRB, 0.65655E+00_JPRB, 0.67436E+00_JPRB, 0.69336E+00_JPRB, 0.71270E+00_JPRB, 0.73344E+00_JPRB ,&
&  0.75110E+00_JPRB, 0.76887E+00_JPRB, 0.78901E+00_JPRB, 0.81084E+00_JPRB, 0.82934E+00_JPRB, 0.85337E+00_JPRB, 0.88198E+00_JPRB ,&
&  0.91732E+00_JPRB, 0.93609E+00_JPRB, 0.95685E+00_JPRB, 0.97785E+00_JPRB, 0.10029E+01_JPRB, 0.10179E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.10501E+01_JPRB, 0.10476E+01_JPRB, 0.10434E+01_JPRB, 0.10405E+01_JPRB, 0.10348E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10160E+01_JPRB, 0.10060E+01_JPRB, 0.99509E+00_JPRB, 0.98268E+00_JPRB, 0.97085E+00_JPRB, 0.94934E+00_JPRB ,&
&  0.92937E+00_JPRB, 0.91422E+00_JPRB, 0.90367E+00_JPRB, 0.89003E+00_JPRB, 0.87686E+00_JPRB, 0.86471E+00_JPRB, 0.85290E+00_JPRB ,&
&  0.84134E+00_JPRB, 0.82814E+00_JPRB, 0.81440E+00_JPRB, 0.79978E+00_JPRB, 0.78737E+00_JPRB, 0.77045E+00_JPRB, 0.75172E+00_JPRB ,&
&  0.73027E+00_JPRB, 0.71244E+00_JPRB, 0.69069E+00_JPRB, 0.66524E+00_JPRB, 0.64028E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.64479E+00_JPRB, 0.64030E+00_JPRB, 0.63587E+00_JPRB, 0.63128E+00_JPRB, 0.62618E+00_JPRB, 0.62058E+00_JPRB, 0.61452E+00_JPRB ,&
&  0.60870E+00_JPRB, 0.60362E+00_JPRB, 0.59905E+00_JPRB, 0.59472E+00_JPRB, 0.59073E+00_JPRB, 0.58753E+00_JPRB, 0.58596E+00_JPRB ,&
&  0.58541E+00_JPRB, 0.58483E+00_JPRB, 0.58469E+00_JPRB, 0.58466E+00_JPRB, 0.58427E+00_JPRB, 0.58359E+00_JPRB, 0.58278E+00_JPRB ,&
&  0.58232E+00_JPRB, 0.58199E+00_JPRB, 0.58151E+00_JPRB, 0.58153E+00_JPRB, 0.58205E+00_JPRB, 0.58266E+00_JPRB, 0.58328E+00_JPRB ,&
&  0.58461E+00_JPRB, 0.58612E+00_JPRB, 0.58814E+00_JPRB, 0.59042E+00_JPRB, 0.59503E+00_JPRB, 0.60025E+00_JPRB, 0.60567E+00_JPRB ,&
&  0.61096E+00_JPRB, 0.61652E+00_JPRB, 0.62132E+00_JPRB, 0.62554E+00_JPRB, 0.62944E+00_JPRB, 0.63398E+00_JPRB, 0.63900E+00_JPRB ,&
&  0.64458E+00_JPRB, 0.65073E+00_JPRB, 0.65772E+00_JPRB, 0.66472E+00_JPRB, 0.67133E+00_JPRB, 0.67824E+00_JPRB, 0.68554E+00_JPRB ,&
&  0.69279E+00_JPRB, 0.70010E+00_JPRB, 0.70741E+00_JPRB, 0.71465E+00_JPRB, 0.72161E+00_JPRB, 0.72965E+00_JPRB, 0.73910E+00_JPRB ,&
&  0.74948E+00_JPRB, 0.75779E+00_JPRB, 0.76723E+00_JPRB, 0.77750E+00_JPRB, 0.79175E+00_JPRB, 0.79898E+00_JPRB, 0.80605E+00_JPRB ,&
&  0.81262E+00_JPRB, 0.82236E+00_JPRB, 0.82826E+00_JPRB, 0.83443E+00_JPRB, 0.84013E+00_JPRB, 0.84513E+00_JPRB, 0.84809E+00_JPRB ,&
&  0.85061E+00_JPRB, 0.85271E+00_JPRB, 0.85277E+00_JPRB, 0.85154E+00_JPRB, 0.84930E+00_JPRB, 0.84650E+00_JPRB, 0.84012E+00_JPRB ,&
&  0.83408E+00_JPRB, 0.82830E+00_JPRB, 0.82297E+00_JPRB, 0.81544E+00_JPRB, 0.80790E+00_JPRB, 0.80021E+00_JPRB, 0.79234E+00_JPRB ,&
&  0.78231E+00_JPRB, 0.77215E+00_JPRB, 0.76131E+00_JPRB, 0.74985E+00_JPRB, 0.73867E+00_JPRB, 0.72710E+00_JPRB, 0.71478E+00_JPRB ,&
&  0.70103E+00_JPRB, 0.68822E+00_JPRB, 0.67398E+00_JPRB, 0.65995E+00_JPRB, 0.64951E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB ,&
&  0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB, 0.66772E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB, 0.12612E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.13353E+00_JPRB, 0.13354E+00_JPRB, 0.13347E+00_JPRB, 0.13321E+00_JPRB, 0.13267E+00_JPRB, 0.13254E+00_JPRB, 0.13236E+00_JPRB ,&
&  0.13212E+00_JPRB, 0.13158E+00_JPRB, 0.13157E+00_JPRB, 0.13150E+00_JPRB, 0.13141E+00_JPRB, 0.13131E+00_JPRB, 0.13112E+00_JPRB ,&
&  0.13073E+00_JPRB, 0.13014E+00_JPRB, 0.12835E+00_JPRB, 0.12749E+00_JPRB, 0.12696E+00_JPRB, 0.12668E+00_JPRB, 0.12645E+00_JPRB ,&
&  0.12633E+00_JPRB, 0.12627E+00_JPRB, 0.12620E+00_JPRB, 0.12657E+00_JPRB, 0.12674E+00_JPRB, 0.12681E+00_JPRB, 0.12686E+00_JPRB ,&
&  0.12687E+00_JPRB, 0.12696E+00_JPRB, 0.12697E+00_JPRB, 0.12698E+00_JPRB, 0.12708E+00_JPRB, 0.12709E+00_JPRB, 0.12693E+00_JPRB ,&
&  0.12676E+00_JPRB, 0.12654E+00_JPRB, 0.12632E+00_JPRB, 0.12605E+00_JPRB, 0.12586E+00_JPRB, 0.12570E+00_JPRB, 0.12563E+00_JPRB ,&
&  0.12549E+00_JPRB, 0.12538E+00_JPRB, 0.12527E+00_JPRB, 0.12520E+00_JPRB, 0.12511E+00_JPRB, 0.12506E+00_JPRB, 0.12495E+00_JPRB ,&
&  0.12501E+00_JPRB, 0.12502E+00_JPRB, 0.12505E+00_JPRB, 0.12505E+00_JPRB, 0.12504E+00_JPRB, 0.12499E+00_JPRB, 0.12490E+00_JPRB ,&
&  0.12474E+00_JPRB, 0.12464E+00_JPRB, 0.12453E+00_JPRB, 0.12444E+00_JPRB, 0.12440E+00_JPRB, 0.12437E+00_JPRB, 0.12436E+00_JPRB ,&
&  0.12435E+00_JPRB, 0.12442E+00_JPRB, 0.12444E+00_JPRB, 0.12447E+00_JPRB, 0.12452E+00_JPRB, 0.12457E+00_JPRB, 0.12461E+00_JPRB ,&
&  0.12468E+00_JPRB, 0.12473E+00_JPRB, 0.12482E+00_JPRB, 0.12494E+00_JPRB, 0.12507E+00_JPRB, 0.12526E+00_JPRB, 0.12549E+00_JPRB ,&
&  0.12574E+00_JPRB, 0.12600E+00_JPRB, 0.12625E+00_JPRB, 0.12650E+00_JPRB, 0.12689E+00_JPRB, 0.12728E+00_JPRB, 0.12773E+00_JPRB ,&
&  0.12818E+00_JPRB, 0.12864E+00_JPRB, 0.12904E+00_JPRB, 0.12948E+00_JPRB, 0.13002E+00_JPRB, 0.13030E+00_JPRB, 0.13055E+00_JPRB ,&
&  0.13091E+00_JPRB, 0.13128E+00_JPRB, 0.13167E+00_JPRB, 0.13201E+00_JPRB, 0.13256E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.13834E+00_JPRB, 0.13870E+00_JPRB, 0.13831E+00_JPRB, 0.13793E+00_JPRB, 0.13288E+00_JPRB, 0.13873E+00_JPRB, 0.13800E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13781E+00_JPRB, 0.13876E+00_JPRB, 0.13873E+00_JPRB, 0.13831E+00_JPRB, 0.13801E+00_JPRB, 0.13766E+00_JPRB ,&
&  0.13729E+00_JPRB, 0.13657E+00_JPRB, 0.13616E+00_JPRB, 0.13563E+00_JPRB, 0.13511E+00_JPRB, 0.13431E+00_JPRB, 0.13349E+00_JPRB ,&
&  0.13313E+00_JPRB, 0.13321E+00_JPRB, 0.13356E+00_JPRB, 0.13233E+00_JPRB, 0.13185E+00_JPRB, 0.13156E+00_JPRB, 0.13144E+00_JPRB ,&
&  0.13121E+00_JPRB, 0.13120E+00_JPRB, 0.13124E+00_JPRB, 0.13121E+00_JPRB, 0.13224E+00_JPRB, 0.13254E+00_JPRB, 0.13279E+00_JPRB ,&
&  0.13282E+00_JPRB, 0.13266E+00_JPRB, 0.13205E+00_JPRB, 0.13149E+00_JPRB, 0.13093E+00_JPRB, 0.12999E+00_JPRB, 0.12900E+00_JPRB ,&
&  0.12814E+00_JPRB, 0.12748E+00_JPRB, 0.12674E+00_JPRB, 0.12617E+00_JPRB, 0.12570E+00_JPRB, 0.12526E+00_JPRB, 0.12496E+00_JPRB ,&
&  0.12486E+00_JPRB, 0.12484E+00_JPRB, 0.12479E+00_JPRB, 0.12474E+00_JPRB, 0.12465E+00_JPRB, 0.12457E+00_JPRB, 0.12449E+00_JPRB ,&
&  0.12465E+00_JPRB, 0.12457E+00_JPRB, 0.12439E+00_JPRB, 0.12412E+00_JPRB, 0.12396E+00_JPRB, 0.12364E+00_JPRB, 0.12342E+00_JPRB ,&
&  0.12323E+00_JPRB, 0.12302E+00_JPRB, 0.12291E+00_JPRB, 0.12288E+00_JPRB, 0.12286E+00_JPRB, 0.12283E+00_JPRB, 0.12290E+00_JPRB ,&
&  0.12302E+00_JPRB, 0.12318E+00_JPRB, 0.12350E+00_JPRB, 0.12377E+00_JPRB, 0.12401E+00_JPRB, 0.12423E+00_JPRB, 0.12465E+00_JPRB ,&
&  0.12511E+00_JPRB, 0.12557E+00_JPRB, 0.12602E+00_JPRB, 0.12633E+00_JPRB, 0.12696E+00_JPRB, 0.12760E+00_JPRB, 0.12830E+00_JPRB ,&
&  0.12904E+00_JPRB, 0.12996E+00_JPRB, 0.13092E+00_JPRB, 0.13188E+00_JPRB, 0.13311E+00_JPRB, 0.13413E+00_JPRB, 0.13511E+00_JPRB ,&
&  0.13620E+00_JPRB, 0.13820E+00_JPRB, 0.13884E+00_JPRB, 0.13910E+00_JPRB, 0.13887E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.15295E+00_JPRB, 0.15361E+00_JPRB, 0.15393E+00_JPRB, 0.15432E+00_JPRB, 0.15523E+00_JPRB, 0.15524E+00_JPRB, 0.15506E+00_JPRB ,&
&  0.15316E+00_JPRB, 0.15236E+00_JPRB, 0.15112E+00_JPRB, 0.14945E+00_JPRB, 0.14776E+00_JPRB, 0.14511E+00_JPRB, 0.14334E+00_JPRB ,&
&  0.14158E+00_JPRB, 0.13973E+00_JPRB, 0.13847E+00_JPRB, 0.13690E+00_JPRB, 0.13557E+00_JPRB, 0.13411E+00_JPRB, 0.13359E+00_JPRB ,&
&  0.13289E+00_JPRB, 0.13285E+00_JPRB, 0.13357E+00_JPRB, 0.13421E+00_JPRB, 0.13532E+00_JPRB, 0.13651E+00_JPRB, 0.13804E+00_JPRB ,&
&  0.13859E+00_JPRB, 0.13944E+00_JPRB, 0.14022E+00_JPRB, 0.14072E+00_JPRB, 0.14225E+00_JPRB, 0.14336E+00_JPRB, 0.14376E+00_JPRB ,&
&  0.14334E+00_JPRB, 0.14261E+00_JPRB, 0.14111E+00_JPRB, 0.13940E+00_JPRB, 0.13739E+00_JPRB, 0.13515E+00_JPRB, 0.13301E+00_JPRB ,&
&  0.13133E+00_JPRB, 0.12978E+00_JPRB, 0.12860E+00_JPRB, 0.12764E+00_JPRB, 0.12703E+00_JPRB, 0.12656E+00_JPRB, 0.12609E+00_JPRB ,&
&  0.12580E+00_JPRB, 0.12555E+00_JPRB, 0.12520E+00_JPRB, 0.12513E+00_JPRB, 0.12499E+00_JPRB, 0.12499E+00_JPRB, 0.12498E+00_JPRB ,&
&  0.12497E+00_JPRB, 0.12487E+00_JPRB, 0.12471E+00_JPRB, 0.12438E+00_JPRB, 0.12406E+00_JPRB, 0.12371E+00_JPRB, 0.12337E+00_JPRB ,&
&  0.12301E+00_JPRB, 0.12294E+00_JPRB, 0.12266E+00_JPRB, 0.12246E+00_JPRB, 0.12222E+00_JPRB, 0.12184E+00_JPRB, 0.12173E+00_JPRB ,&
&  0.12182E+00_JPRB, 0.12196E+00_JPRB, 0.12188E+00_JPRB, 0.12204E+00_JPRB, 0.12225E+00_JPRB, 0.12254E+00_JPRB, 0.12324E+00_JPRB ,&
&  0.12368E+00_JPRB, 0.12415E+00_JPRB, 0.12458E+00_JPRB, 0.12518E+00_JPRB, 0.12618E+00_JPRB, 0.12719E+00_JPRB, 0.12832E+00_JPRB ,&
&  0.12968E+00_JPRB, 0.13098E+00_JPRB, 0.13227E+00_JPRB, 0.13355E+00_JPRB, 0.13564E+00_JPRB, 0.13718E+00_JPRB, 0.13922E+00_JPRB ,&
&  0.14104E+00_JPRB, 0.14048E+00_JPRB, 0.14807E+00_JPRB, 0.14897E+00_JPRB, 0.15028E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.15925E+00_JPRB, 0.15946E+00_JPRB, 0.15934E+00_JPRB, 0.16019E+00_JPRB, 0.16098E+00_JPRB, 0.16183E+00_JPRB, 0.16156E+00_JPRB ,&
&  0.16127E+00_JPRB, 0.15967E+00_JPRB, 0.15843E+00_JPRB, 0.15560E+00_JPRB, 0.15331E+00_JPRB, 0.15064E+00_JPRB, 0.14832E+00_JPRB ,&
&  0.14626E+00_JPRB, 0.14431E+00_JPRB, 0.14250E+00_JPRB, 0.14098E+00_JPRB, 0.13959E+00_JPRB, 0.13841E+00_JPRB, 0.13731E+00_JPRB ,&
&  0.13564E+00_JPRB, 0.13520E+00_JPRB, 0.13526E+00_JPRB, 0.13643E+00_JPRB, 0.13874E+00_JPRB, 0.14035E+00_JPRB, 0.14229E+00_JPRB ,&
&  0.14469E+00_JPRB, 0.14438E+00_JPRB, 0.14707E+00_JPRB, 0.14969E+00_JPRB, 0.15185E+00_JPRB, 0.15356E+00_JPRB, 0.15409E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15090E+00_JPRB, 0.15229E+00_JPRB, 0.14896E+00_JPRB, 0.14475E+00_JPRB, 0.14265E+00_JPRB, 0.13888E+00_JPRB ,&
&  0.13694E+00_JPRB, 0.13403E+00_JPRB, 0.13214E+00_JPRB, 0.13059E+00_JPRB, 0.12885E+00_JPRB, 0.12838E+00_JPRB, 0.12760E+00_JPRB ,&
&  0.12711E+00_JPRB, 0.12708E+00_JPRB, 0.12637E+00_JPRB, 0.12609E+00_JPRB, 0.12563E+00_JPRB, 0.12518E+00_JPRB, 0.12540E+00_JPRB ,&
&  0.12580E+00_JPRB, 0.12633E+00_JPRB, 0.12686E+00_JPRB, 0.12692E+00_JPRB, 0.12685E+00_JPRB, 0.12681E+00_JPRB, 0.12673E+00_JPRB ,&
&  0.12693E+00_JPRB, 0.12694E+00_JPRB, 0.12658E+00_JPRB, 0.12617E+00_JPRB, 0.12525E+00_JPRB, 0.12435E+00_JPRB, 0.12345E+00_JPRB ,&
&  0.12269E+00_JPRB, 0.12228E+00_JPRB, 0.12194E+00_JPRB, 0.12187E+00_JPRB, 0.12193E+00_JPRB, 0.12224E+00_JPRB, 0.12256E+00_JPRB ,&
&  0.12286E+00_JPRB, 0.12349E+00_JPRB, 0.12397E+00_JPRB, 0.12472E+00_JPRB, 0.12562E+00_JPRB, 0.12645E+00_JPRB, 0.12771E+00_JPRB ,&
&  0.12909E+00_JPRB, 0.13059E+00_JPRB, 0.13271E+00_JPRB, 0.13460E+00_JPRB, 0.13697E+00_JPRB, 0.14009E+00_JPRB, 0.14248E+00_JPRB ,&
&  0.14508E+00_JPRB, 0.14790E+00_JPRB, 0.14851E+00_JPRB, 0.15488E+00_JPRB, 0.15842E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.15817E+00_JPRB, 0.15943E+00_JPRB, 0.16083E+00_JPRB, 0.16220E+00_JPRB, 0.16340E+00_JPRB, 0.16459E+00_JPRB, 0.16536E+00_JPRB ,&
&  0.16312E+00_JPRB, 0.16172E+00_JPRB, 0.15918E+00_JPRB, 0.15679E+00_JPRB, 0.15411E+00_JPRB, 0.15124E+00_JPRB, 0.15016E+00_JPRB ,&
&  0.14879E+00_JPRB, 0.14848E+00_JPRB, 0.14741E+00_JPRB, 0.14695E+00_JPRB, 0.14611E+00_JPRB, 0.14565E+00_JPRB, 0.14487E+00_JPRB ,&
&  0.14320E+00_JPRB, 0.14231E+00_JPRB, 0.14146E+00_JPRB, 0.14186E+00_JPRB, 0.14325E+00_JPRB, 0.14478E+00_JPRB, 0.14640E+00_JPRB ,&
&  0.14870E+00_JPRB, 0.14989E+00_JPRB, 0.15215E+00_JPRB, 0.15382E+00_JPRB, 0.15642E+00_JPRB, 0.15862E+00_JPRB, 0.15912E+00_JPRB ,&
&  0.15925E+00_JPRB, 0.15802E+00_JPRB, 0.15784E+00_JPRB, 0.15628E+00_JPRB, 0.15087E+00_JPRB, 0.14768E+00_JPRB, 0.14325E+00_JPRB ,&
&  0.13830E+00_JPRB, 0.13456E+00_JPRB, 0.13199E+00_JPRB, 0.13111E+00_JPRB, 0.13028E+00_JPRB, 0.12998E+00_JPRB, 0.12968E+00_JPRB ,&
&  0.12891E+00_JPRB, 0.12801E+00_JPRB, 0.12697E+00_JPRB, 0.12635E+00_JPRB, 0.12658E+00_JPRB, 0.12716E+00_JPRB, 0.12846E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.13049E+00_JPRB, 0.13129E+00_JPRB, 0.13154E+00_JPRB, 0.13165E+00_JPRB, 0.13185E+00_JPRB, 0.13192E+00_JPRB ,&
&  0.13248E+00_JPRB, 0.13263E+00_JPRB, 0.13218E+00_JPRB, 0.13143E+00_JPRB, 0.12955E+00_JPRB, 0.12710E+00_JPRB, 0.12512E+00_JPRB ,&
&  0.12346E+00_JPRB, 0.12238E+00_JPRB, 0.12199E+00_JPRB, 0.12161E+00_JPRB, 0.12184E+00_JPRB, 0.12219E+00_JPRB, 0.12252E+00_JPRB ,&
&  0.12306E+00_JPRB, 0.12354E+00_JPRB, 0.12400E+00_JPRB, 0.12499E+00_JPRB, 0.12580E+00_JPRB, 0.12700E+00_JPRB, 0.12825E+00_JPRB ,&
&  0.12913E+00_JPRB, 0.13047E+00_JPRB, 0.13159E+00_JPRB, 0.13284E+00_JPRB, 0.13653E+00_JPRB, 0.13954E+00_JPRB, 0.14567E+00_JPRB ,&
&  0.14986E+00_JPRB, 0.15214E+00_JPRB, 0.15373E+00_JPRB, 0.15825E+00_JPRB, 0.15797E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.15830E+00_JPRB, 0.16001E+00_JPRB, 0.16195E+00_JPRB, 0.16341E+00_JPRB, 0.16483E+00_JPRB, 0.16601E+00_JPRB, 0.16662E+00_JPRB ,&
&  0.16672E+00_JPRB, 0.16615E+00_JPRB, 0.16618E+00_JPRB, 0.16512E+00_JPRB, 0.16412E+00_JPRB, 0.15999E+00_JPRB, 0.15925E+00_JPRB ,&
&  0.15820E+00_JPRB, 0.15863E+00_JPRB, 0.15821E+00_JPRB, 0.15839E+00_JPRB, 0.15778E+00_JPRB, 0.15750E+00_JPRB, 0.15623E+00_JPRB ,&
&  0.15434E+00_JPRB, 0.15285E+00_JPRB, 0.15136E+00_JPRB, 0.15085E+00_JPRB, 0.15153E+00_JPRB, 0.15209E+00_JPRB, 0.15188E+00_JPRB ,&
&  0.15191E+00_JPRB, 0.15227E+00_JPRB, 0.15278E+00_JPRB, 0.15300E+00_JPRB, 0.15800E+00_JPRB, 0.16009E+00_JPRB, 0.16130E+00_JPRB ,&
&  0.16142E+00_JPRB, 0.16205E+00_JPRB, 0.16238E+00_JPRB, 0.16186E+00_JPRB, 0.16129E+00_JPRB, 0.15795E+00_JPRB, 0.15576E+00_JPRB ,&
&  0.14820E+00_JPRB, 0.14276E+00_JPRB, 0.13859E+00_JPRB, 0.13600E+00_JPRB, 0.13492E+00_JPRB, 0.13473E+00_JPRB, 0.13412E+00_JPRB ,&
&  0.13265E+00_JPRB, 0.13128E+00_JPRB, 0.13040E+00_JPRB, 0.13087E+00_JPRB, 0.13172E+00_JPRB, 0.13273E+00_JPRB, 0.13388E+00_JPRB ,&
&  0.13493E+00_JPRB, 0.13516E+00_JPRB, 0.13512E+00_JPRB, 0.13544E+00_JPRB, 0.13570E+00_JPRB, 0.13612E+00_JPRB, 0.13646E+00_JPRB ,&
&  0.13734E+00_JPRB, 0.13675E+00_JPRB, 0.13727E+00_JPRB, 0.13817E+00_JPRB, 0.13616E+00_JPRB, 0.13291E+00_JPRB, 0.12962E+00_JPRB ,&
&  0.12661E+00_JPRB, 0.12491E+00_JPRB, 0.12356E+00_JPRB, 0.12272E+00_JPRB, 0.12256E+00_JPRB, 0.12270E+00_JPRB, 0.12322E+00_JPRB ,&
&  0.12388E+00_JPRB, 0.12475E+00_JPRB, 0.12534E+00_JPRB, 0.12606E+00_JPRB, 0.12693E+00_JPRB, 0.12784E+00_JPRB, 0.12892E+00_JPRB ,&
&  0.13024E+00_JPRB, 0.13150E+00_JPRB, 0.13238E+00_JPRB, 0.13365E+00_JPRB, 0.13610E+00_JPRB, 0.13942E+00_JPRB, 0.14456E+00_JPRB ,&
&  0.15047E+00_JPRB, 0.15593E+00_JPRB, 0.15437E+00_JPRB, 0.15612E+00_JPRB, 0.15758E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.16355E+00_JPRB, 0.16581E+00_JPRB, 0.16749E+00_JPRB, 0.16903E+00_JPRB, 0.17018E+00_JPRB, 0.17147E+00_JPRB, 0.17203E+00_JPRB ,&
&  0.17253E+00_JPRB, 0.17249E+00_JPRB, 0.17283E+00_JPRB, 0.17272E+00_JPRB, 0.17282E+00_JPRB, 0.17177E+00_JPRB, 0.17112E+00_JPRB ,&
&  0.17066E+00_JPRB, 0.17085E+00_JPRB, 0.17072E+00_JPRB, 0.17048E+00_JPRB, 0.16971E+00_JPRB, 0.16874E+00_JPRB, 0.16733E+00_JPRB ,&
&  0.16454E+00_JPRB, 0.16281E+00_JPRB, 0.16323E+00_JPRB, 0.16293E+00_JPRB, 0.16337E+00_JPRB, 0.16348E+00_JPRB, 0.16393E+00_JPRB ,&
&  0.16028E+00_JPRB, 0.15881E+00_JPRB, 0.15866E+00_JPRB, 0.15790E+00_JPRB, 0.16054E+00_JPRB, 0.16175E+00_JPRB, 0.16359E+00_JPRB ,&
&  0.16433E+00_JPRB, 0.16491E+00_JPRB, 0.16575E+00_JPRB, 0.16559E+00_JPRB, 0.16612E+00_JPRB, 0.16668E+00_JPRB, 0.16578E+00_JPRB ,&
&  0.16325E+00_JPRB, 0.16114E+00_JPRB, 0.15816E+00_JPRB, 0.15054E+00_JPRB, 0.14830E+00_JPRB, 0.14822E+00_JPRB, 0.14668E+00_JPRB ,&
&  0.14404E+00_JPRB, 0.14401E+00_JPRB, 0.14347E+00_JPRB, 0.14212E+00_JPRB, 0.14358E+00_JPRB, 0.14571E+00_JPRB, 0.14579E+00_JPRB ,&
&  0.14473E+00_JPRB, 0.14444E+00_JPRB, 0.14383E+00_JPRB, 0.14269E+00_JPRB, 0.14136E+00_JPRB, 0.14079E+00_JPRB, 0.14030E+00_JPRB ,&
&  0.14010E+00_JPRB, 0.13915E+00_JPRB, 0.14232E+00_JPRB, 0.14334E+00_JPRB, 0.13912E+00_JPRB, 0.14006E+00_JPRB, 0.13771E+00_JPRB ,&
&  0.13199E+00_JPRB, 0.12936E+00_JPRB, 0.12795E+00_JPRB, 0.12681E+00_JPRB, 0.12665E+00_JPRB, 0.12686E+00_JPRB, 0.12684E+00_JPRB ,&
&  0.12767E+00_JPRB, 0.12868E+00_JPRB, 0.12918E+00_JPRB, 0.12978E+00_JPRB, 0.13057E+00_JPRB, 0.13170E+00_JPRB, 0.13318E+00_JPRB ,&
&  0.13444E+00_JPRB, 0.13624E+00_JPRB, 0.13720E+00_JPRB, 0.13876E+00_JPRB, 0.13993E+00_JPRB, 0.14132E+00_JPRB, 0.14460E+00_JPRB ,&
&  0.14745E+00_JPRB, 0.15297E+00_JPRB, 0.16062E+00_JPRB, 0.16349E+00_JPRB, 0.16145E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.16891E+00_JPRB, 0.17106E+00_JPRB, 0.17374E+00_JPRB, 0.17651E+00_JPRB, 0.17858E+00_JPRB, 0.18064E+00_JPRB, 0.18239E+00_JPRB ,&
&  0.18389E+00_JPRB, 0.18486E+00_JPRB, 0.18546E+00_JPRB, 0.18572E+00_JPRB, 0.18585E+00_JPRB, 0.18544E+00_JPRB, 0.18459E+00_JPRB ,&
&  0.18374E+00_JPRB, 0.18234E+00_JPRB, 0.18060E+00_JPRB, 0.17914E+00_JPRB, 0.17814E+00_JPRB, 0.17697E+00_JPRB, 0.17533E+00_JPRB ,&
&  0.17362E+00_JPRB, 0.17215E+00_JPRB, 0.17113E+00_JPRB, 0.17030E+00_JPRB, 0.17043E+00_JPRB, 0.17104E+00_JPRB, 0.17154E+00_JPRB ,&
&  0.16841E+00_JPRB, 0.16348E+00_JPRB, 0.16366E+00_JPRB, 0.16465E+00_JPRB, 0.16409E+00_JPRB, 0.16553E+00_JPRB, 0.16652E+00_JPRB ,&
&  0.16782E+00_JPRB, 0.16936E+00_JPRB, 0.17082E+00_JPRB, 0.17169E+00_JPRB, 0.17234E+00_JPRB, 0.17324E+00_JPRB, 0.17286E+00_JPRB ,&
&  0.17115E+00_JPRB, 0.16955E+00_JPRB, 0.16799E+00_JPRB, 0.16647E+00_JPRB, 0.16603E+00_JPRB, 0.16723E+00_JPRB, 0.16788E+00_JPRB ,&
&  0.16689E+00_JPRB, 0.16550E+00_JPRB, 0.16421E+00_JPRB, 0.16424E+00_JPRB, 0.16479E+00_JPRB, 0.16500E+00_JPRB, 0.16321E+00_JPRB ,&
&  0.16166E+00_JPRB, 0.15901E+00_JPRB, 0.15646E+00_JPRB, 0.15402E+00_JPRB, 0.15260E+00_JPRB, 0.14948E+00_JPRB, 0.14626E+00_JPRB ,&
&  0.14552E+00_JPRB, 0.14886E+00_JPRB, 0.15432E+00_JPRB, 0.15701E+00_JPRB, 0.15000E+00_JPRB, 0.14874E+00_JPRB, 0.14786E+00_JPRB ,&
&  0.14349E+00_JPRB, 0.13898E+00_JPRB, 0.13582E+00_JPRB, 0.13448E+00_JPRB, 0.13402E+00_JPRB, 0.13365E+00_JPRB, 0.13413E+00_JPRB ,&
&  0.13504E+00_JPRB, 0.13564E+00_JPRB, 0.13628E+00_JPRB, 0.13733E+00_JPRB, 0.13832E+00_JPRB, 0.13930E+00_JPRB, 0.14014E+00_JPRB ,&
&  0.14190E+00_JPRB, 0.14409E+00_JPRB, 0.14601E+00_JPRB, 0.14865E+00_JPRB, 0.15146E+00_JPRB, 0.15433E+00_JPRB, 0.15733E+00_JPRB ,&
&  0.16070E+00_JPRB, 0.16305E+00_JPRB, 0.16443E+00_JPRB, 0.16669E+00_JPRB, 0.16775E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.18065E+00_JPRB, 0.18564E+00_JPRB, 0.19010E+00_JPRB, 0.19266E+00_JPRB, 0.19451E+00_JPRB, 0.19705E+00_JPRB, 0.19947E+00_JPRB ,&
&  0.20107E+00_JPRB, 0.20187E+00_JPRB, 0.20212E+00_JPRB, 0.20182E+00_JPRB, 0.20076E+00_JPRB, 0.19916E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.19546E+00_JPRB, 0.19285E+00_JPRB, 0.19048E+00_JPRB, 0.18864E+00_JPRB, 0.18705E+00_JPRB, 0.18519E+00_JPRB, 0.18369E+00_JPRB ,&
&  0.18207E+00_JPRB, 0.18136E+00_JPRB, 0.18066E+00_JPRB, 0.18014E+00_JPRB, 0.17984E+00_JPRB, 0.17917E+00_JPRB, 0.17856E+00_JPRB ,&
&  0.17686E+00_JPRB, 0.17392E+00_JPRB, 0.17229E+00_JPRB, 0.17178E+00_JPRB, 0.17195E+00_JPRB, 0.17162E+00_JPRB, 0.17127E+00_JPRB ,&
&  0.17317E+00_JPRB, 0.17613E+00_JPRB, 0.17837E+00_JPRB, 0.17966E+00_JPRB, 0.18062E+00_JPRB, 0.18182E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18130E+00_JPRB, 0.17882E+00_JPRB, 0.17687E+00_JPRB, 0.17575E+00_JPRB, 0.17546E+00_JPRB, 0.17602E+00_JPRB, 0.17735E+00_JPRB ,&
&  0.17856E+00_JPRB, 0.17914E+00_JPRB, 0.17865E+00_JPRB, 0.17733E+00_JPRB, 0.17579E+00_JPRB, 0.17498E+00_JPRB, 0.17445E+00_JPRB ,&
&  0.17420E+00_JPRB, 0.17392E+00_JPRB, 0.17352E+00_JPRB, 0.17259E+00_JPRB, 0.16800E+00_JPRB, 0.16395E+00_JPRB, 0.16130E+00_JPRB ,&
&  0.16172E+00_JPRB, 0.16238E+00_JPRB, 0.16241E+00_JPRB, 0.16669E+00_JPRB, 0.17133E+00_JPRB, 0.16751E+00_JPRB, 0.16266E+00_JPRB ,&
&  0.15623E+00_JPRB, 0.15566E+00_JPRB, 0.15299E+00_JPRB, 0.14892E+00_JPRB, 0.14756E+00_JPRB, 0.14749E+00_JPRB, 0.14727E+00_JPRB ,&
&  0.14802E+00_JPRB, 0.14845E+00_JPRB, 0.14965E+00_JPRB, 0.15027E+00_JPRB, 0.15058E+00_JPRB, 0.15117E+00_JPRB, 0.15179E+00_JPRB ,&
&  0.15333E+00_JPRB, 0.15648E+00_JPRB, 0.16053E+00_JPRB, 0.16265E+00_JPRB, 0.16495E+00_JPRB, 0.16718E+00_JPRB, 0.16871E+00_JPRB ,&
&  0.16993E+00_JPRB, 0.17161E+00_JPRB, 0.17322E+00_JPRB, 0.17448E+00_JPRB, 0.17673E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.19764E+00_JPRB, 0.20204E+00_JPRB, 0.20475E+00_JPRB, 0.20738E+00_JPRB, 0.21016E+00_JPRB, 0.21463E+00_JPRB, 0.21761E+00_JPRB ,&
&  0.21808E+00_JPRB, 0.21793E+00_JPRB, 0.21732E+00_JPRB, 0.21607E+00_JPRB, 0.21422E+00_JPRB, 0.21258E+00_JPRB, 0.20990E+00_JPRB ,&
&  0.20662E+00_JPRB, 0.20363E+00_JPRB, 0.20095E+00_JPRB, 0.19840E+00_JPRB, 0.19603E+00_JPRB, 0.19386E+00_JPRB, 0.19210E+00_JPRB ,&
&  0.19142E+00_JPRB, 0.19094E+00_JPRB, 0.18990E+00_JPRB, 0.18960E+00_JPRB, 0.18975E+00_JPRB, 0.18940E+00_JPRB, 0.18837E+00_JPRB ,&
&  0.18705E+00_JPRB, 0.18461E+00_JPRB, 0.18208E+00_JPRB, 0.18097E+00_JPRB, 0.18123E+00_JPRB, 0.18071E+00_JPRB, 0.18007E+00_JPRB ,&
&  0.18160E+00_JPRB, 0.18393E+00_JPRB, 0.18654E+00_JPRB, 0.18812E+00_JPRB, 0.18904E+00_JPRB, 0.19015E+00_JPRB, 0.19152E+00_JPRB ,&
&  0.19174E+00_JPRB, 0.18936E+00_JPRB, 0.18716E+00_JPRB, 0.18653E+00_JPRB, 0.18624E+00_JPRB, 0.18580E+00_JPRB, 0.18604E+00_JPRB ,&
&  0.18660E+00_JPRB, 0.18779E+00_JPRB, 0.18855E+00_JPRB, 0.18761E+00_JPRB, 0.18597E+00_JPRB, 0.18475E+00_JPRB, 0.18394E+00_JPRB ,&
&  0.18348E+00_JPRB, 0.18291E+00_JPRB, 0.18197E+00_JPRB, 0.18055E+00_JPRB, 0.17900E+00_JPRB, 0.17762E+00_JPRB, 0.17632E+00_JPRB ,&
&  0.17521E+00_JPRB, 0.17539E+00_JPRB, 0.17729E+00_JPRB, 0.17873E+00_JPRB, 0.17867E+00_JPRB, 0.17744E+00_JPRB, 0.17542E+00_JPRB ,&
&  0.17396E+00_JPRB, 0.17194E+00_JPRB, 0.17036E+00_JPRB, 0.16910E+00_JPRB, 0.16808E+00_JPRB, 0.16671E+00_JPRB, 0.16615E+00_JPRB ,&
&  0.16632E+00_JPRB, 0.16773E+00_JPRB, 0.16840E+00_JPRB, 0.16877E+00_JPRB, 0.16861E+00_JPRB, 0.16846E+00_JPRB, 0.16931E+00_JPRB ,&
&  0.17047E+00_JPRB, 0.17218E+00_JPRB, 0.17398E+00_JPRB, 0.17634E+00_JPRB, 0.17879E+00_JPRB, 0.18199E+00_JPRB, 0.18467E+00_JPRB ,&
&  0.18632E+00_JPRB, 0.18684E+00_JPRB, 0.18804E+00_JPRB, 0.18977E+00_JPRB, 0.19275E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.21351E+00_JPRB, 0.21674E+00_JPRB, 0.21947E+00_JPRB, 0.22300E+00_JPRB, 0.22605E+00_JPRB, 0.23052E+00_JPRB, 0.23228E+00_JPRB ,&
&  0.23196E+00_JPRB, 0.23132E+00_JPRB, 0.22959E+00_JPRB, 0.22699E+00_JPRB, 0.22317E+00_JPRB, 0.21939E+00_JPRB, 0.21589E+00_JPRB ,&
&  0.21326E+00_JPRB, 0.21108E+00_JPRB, 0.20930E+00_JPRB, 0.20766E+00_JPRB, 0.20539E+00_JPRB, 0.20325E+00_JPRB, 0.20155E+00_JPRB ,&
&  0.20067E+00_JPRB, 0.19978E+00_JPRB, 0.19930E+00_JPRB, 0.19903E+00_JPRB, 0.19887E+00_JPRB, 0.19885E+00_JPRB, 0.19902E+00_JPRB ,&
&  0.19816E+00_JPRB, 0.19514E+00_JPRB, 0.19347E+00_JPRB, 0.19302E+00_JPRB, 0.19271E+00_JPRB, 0.19327E+00_JPRB, 0.19324E+00_JPRB ,&
&  0.19339E+00_JPRB, 0.19467E+00_JPRB, 0.19726E+00_JPRB, 0.19974E+00_JPRB, 0.19982E+00_JPRB, 0.19990E+00_JPRB, 0.20076E+00_JPRB ,&
&  0.20105E+00_JPRB, 0.20031E+00_JPRB, 0.19848E+00_JPRB, 0.19771E+00_JPRB, 0.19803E+00_JPRB, 0.19788E+00_JPRB, 0.19823E+00_JPRB ,&
&  0.19894E+00_JPRB, 0.20007E+00_JPRB, 0.20061E+00_JPRB, 0.19921E+00_JPRB, 0.19771E+00_JPRB, 0.19705E+00_JPRB, 0.19689E+00_JPRB ,&
&  0.19645E+00_JPRB, 0.19541E+00_JPRB, 0.19342E+00_JPRB, 0.19169E+00_JPRB, 0.18910E+00_JPRB, 0.18700E+00_JPRB, 0.18597E+00_JPRB ,&
&  0.18650E+00_JPRB, 0.18696E+00_JPRB, 0.18822E+00_JPRB, 0.18938E+00_JPRB, 0.19006E+00_JPRB, 0.19016E+00_JPRB, 0.18859E+00_JPRB ,&
&  0.18651E+00_JPRB, 0.18558E+00_JPRB, 0.18464E+00_JPRB, 0.18380E+00_JPRB, 0.18391E+00_JPRB, 0.18378E+00_JPRB, 0.18337E+00_JPRB ,&
&  0.18286E+00_JPRB, 0.18313E+00_JPRB, 0.18353E+00_JPRB, 0.18373E+00_JPRB, 0.18362E+00_JPRB, 0.18307E+00_JPRB, 0.18362E+00_JPRB ,&
&  0.18491E+00_JPRB, 0.18669E+00_JPRB, 0.18877E+00_JPRB, 0.19106E+00_JPRB, 0.19359E+00_JPRB, 0.19594E+00_JPRB, 0.19818E+00_JPRB ,&
&  0.20104E+00_JPRB, 0.20224E+00_JPRB, 0.20405E+00_JPRB, 0.20685E+00_JPRB, 0.20966E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.22670E+00_JPRB, 0.22940E+00_JPRB, 0.23383E+00_JPRB, 0.23863E+00_JPRB, 0.24191E+00_JPRB, 0.24581E+00_JPRB, 0.24632E+00_JPRB ,&
&  0.24535E+00_JPRB, 0.24339E+00_JPRB, 0.23938E+00_JPRB, 0.23457E+00_JPRB, 0.23018E+00_JPRB, 0.22663E+00_JPRB, 0.22423E+00_JPRB ,&
&  0.22283E+00_JPRB, 0.22163E+00_JPRB, 0.22023E+00_JPRB, 0.21792E+00_JPRB, 0.21500E+00_JPRB, 0.21229E+00_JPRB, 0.21007E+00_JPRB ,&
&  0.20906E+00_JPRB, 0.20871E+00_JPRB, 0.20884E+00_JPRB, 0.20862E+00_JPRB, 0.20859E+00_JPRB, 0.20996E+00_JPRB, 0.21167E+00_JPRB ,&
&  0.21083E+00_JPRB, 0.20694E+00_JPRB, 0.20464E+00_JPRB, 0.20466E+00_JPRB, 0.20510E+00_JPRB, 0.20587E+00_JPRB, 0.20686E+00_JPRB ,&
&  0.20761E+00_JPRB, 0.20971E+00_JPRB, 0.21264E+00_JPRB, 0.21489E+00_JPRB, 0.21503E+00_JPRB, 0.21634E+00_JPRB, 0.21718E+00_JPRB ,&
&  0.21712E+00_JPRB, 0.21624E+00_JPRB, 0.21469E+00_JPRB, 0.21195E+00_JPRB, 0.20995E+00_JPRB, 0.20922E+00_JPRB, 0.20856E+00_JPRB ,&
&  0.20932E+00_JPRB, 0.21015E+00_JPRB, 0.21086E+00_JPRB, 0.21019E+00_JPRB, 0.20857E+00_JPRB, 0.20821E+00_JPRB, 0.20846E+00_JPRB ,&
&  0.20818E+00_JPRB, 0.20688E+00_JPRB, 0.20511E+00_JPRB, 0.20408E+00_JPRB, 0.20306E+00_JPRB, 0.20280E+00_JPRB, 0.20271E+00_JPRB ,&
&  0.20257E+00_JPRB, 0.20172E+00_JPRB, 0.20119E+00_JPRB, 0.20097E+00_JPRB, 0.20085E+00_JPRB, 0.20121E+00_JPRB, 0.20090E+00_JPRB ,&
&  0.20086E+00_JPRB, 0.20102E+00_JPRB, 0.20008E+00_JPRB, 0.19982E+00_JPRB, 0.19977E+00_JPRB, 0.20006E+00_JPRB, 0.20013E+00_JPRB ,&
&  0.19985E+00_JPRB, 0.19943E+00_JPRB, 0.19911E+00_JPRB, 0.19918E+00_JPRB, 0.19886E+00_JPRB, 0.19841E+00_JPRB, 0.19919E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.20522E+00_JPRB, 0.20788E+00_JPRB, 0.20966E+00_JPRB, 0.21031E+00_JPRB, 0.21173E+00_JPRB, 0.21439E+00_JPRB ,&
&  0.21683E+00_JPRB, 0.21798E+00_JPRB, 0.21997E+00_JPRB, 0.22164E+00_JPRB, 0.22361E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.24184E+00_JPRB, 0.24463E+00_JPRB, 0.24869E+00_JPRB, 0.25450E+00_JPRB, 0.25794E+00_JPRB, 0.26082E+00_JPRB, 0.26028E+00_JPRB ,&
&  0.25776E+00_JPRB, 0.25423E+00_JPRB, 0.25004E+00_JPRB, 0.24501E+00_JPRB, 0.23986E+00_JPRB, 0.23598E+00_JPRB, 0.23453E+00_JPRB ,&
&  0.23437E+00_JPRB, 0.23355E+00_JPRB, 0.23125E+00_JPRB, 0.22803E+00_JPRB, 0.22434E+00_JPRB, 0.22134E+00_JPRB, 0.21891E+00_JPRB ,&
&  0.21791E+00_JPRB, 0.21800E+00_JPRB, 0.21863E+00_JPRB, 0.21898E+00_JPRB, 0.21931E+00_JPRB, 0.22178E+00_JPRB, 0.22566E+00_JPRB ,&
&  0.22368E+00_JPRB, 0.22134E+00_JPRB, 0.21678E+00_JPRB, 0.21655E+00_JPRB, 0.21831E+00_JPRB, 0.22110E+00_JPRB, 0.22391E+00_JPRB ,&
&  0.22684E+00_JPRB, 0.23135E+00_JPRB, 0.23533E+00_JPRB, 0.23794E+00_JPRB, 0.23782E+00_JPRB, 0.23785E+00_JPRB, 0.23814E+00_JPRB ,&
&  0.23801E+00_JPRB, 0.23667E+00_JPRB, 0.23458E+00_JPRB, 0.23111E+00_JPRB, 0.22680E+00_JPRB, 0.22401E+00_JPRB, 0.22266E+00_JPRB ,&
&  0.22126E+00_JPRB, 0.22112E+00_JPRB, 0.22110E+00_JPRB, 0.22052E+00_JPRB, 0.21955E+00_JPRB, 0.21915E+00_JPRB, 0.21945E+00_JPRB ,&
&  0.21952E+00_JPRB, 0.21891E+00_JPRB, 0.21793E+00_JPRB, 0.21766E+00_JPRB, 0.21807E+00_JPRB, 0.21918E+00_JPRB, 0.22025E+00_JPRB ,&
&  0.22100E+00_JPRB, 0.22074E+00_JPRB, 0.22007E+00_JPRB, 0.21839E+00_JPRB, 0.21715E+00_JPRB, 0.21723E+00_JPRB, 0.21827E+00_JPRB ,&
&  0.21890E+00_JPRB, 0.21889E+00_JPRB, 0.21911E+00_JPRB, 0.21825E+00_JPRB, 0.21749E+00_JPRB, 0.21762E+00_JPRB, 0.21807E+00_JPRB ,&
&  0.21823E+00_JPRB, 0.21807E+00_JPRB, 0.21771E+00_JPRB, 0.21749E+00_JPRB, 0.21785E+00_JPRB, 0.21796E+00_JPRB, 0.21888E+00_JPRB ,&
&  0.22126E+00_JPRB, 0.22397E+00_JPRB, 0.22672E+00_JPRB, 0.22882E+00_JPRB, 0.23012E+00_JPRB, 0.23063E+00_JPRB, 0.23297E+00_JPRB ,&
&  0.23623E+00_JPRB, 0.23843E+00_JPRB, 0.23833E+00_JPRB, 0.23798E+00_JPRB, 0.23960E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.25991E+00_JPRB, 0.26082E+00_JPRB, 0.26520E+00_JPRB, 0.27014E+00_JPRB, 0.27517E+00_JPRB, 0.27593E+00_JPRB, 0.27320E+00_JPRB ,&
&  0.27024E+00_JPRB, 0.26631E+00_JPRB, 0.26073E+00_JPRB, 0.25401E+00_JPRB, 0.24788E+00_JPRB, 0.24605E+00_JPRB, 0.24599E+00_JPRB ,&
&  0.24621E+00_JPRB, 0.24496E+00_JPRB, 0.24213E+00_JPRB, 0.23882E+00_JPRB, 0.23483E+00_JPRB, 0.23131E+00_JPRB, 0.22863E+00_JPRB ,&
&  0.22770E+00_JPRB, 0.22720E+00_JPRB, 0.22756E+00_JPRB, 0.22772E+00_JPRB, 0.22858E+00_JPRB, 0.23166E+00_JPRB, 0.23707E+00_JPRB ,&
&  0.23486E+00_JPRB, 0.23523E+00_JPRB, 0.22929E+00_JPRB, 0.23187E+00_JPRB, 0.23451E+00_JPRB, 0.24038E+00_JPRB, 0.24745E+00_JPRB ,&
&  0.25481E+00_JPRB, 0.26101E+00_JPRB, 0.26411E+00_JPRB, 0.26451E+00_JPRB, 0.26144E+00_JPRB, 0.25974E+00_JPRB, 0.25859E+00_JPRB ,&
&  0.25735E+00_JPRB, 0.25604E+00_JPRB, 0.25410E+00_JPRB, 0.25061E+00_JPRB, 0.24584E+00_JPRB, 0.24162E+00_JPRB, 0.23848E+00_JPRB ,&
&  0.23757E+00_JPRB, 0.23694E+00_JPRB, 0.23543E+00_JPRB, 0.23396E+00_JPRB, 0.23236E+00_JPRB, 0.23132E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.23259E+00_JPRB, 0.23346E+00_JPRB, 0.23410E+00_JPRB, 0.23515E+00_JPRB, 0.23707E+00_JPRB, 0.23890E+00_JPRB, 0.24018E+00_JPRB ,&
&  0.24054E+00_JPRB, 0.23997E+00_JPRB, 0.23938E+00_JPRB, 0.23866E+00_JPRB, 0.23818E+00_JPRB, 0.23763E+00_JPRB, 0.23759E+00_JPRB ,&
&  0.23773E+00_JPRB, 0.23769E+00_JPRB, 0.23708E+00_JPRB, 0.23595E+00_JPRB, 0.23496E+00_JPRB, 0.23414E+00_JPRB, 0.23390E+00_JPRB ,&
&  0.23423E+00_JPRB, 0.23461E+00_JPRB, 0.23451E+00_JPRB, 0.23411E+00_JPRB, 0.23434E+00_JPRB, 0.23486E+00_JPRB, 0.23616E+00_JPRB ,&
&  0.23811E+00_JPRB, 0.24069E+00_JPRB, 0.24415E+00_JPRB, 0.24789E+00_JPRB, 0.24927E+00_JPRB, 0.25085E+00_JPRB, 0.25407E+00_JPRB ,&
&  0.25701E+00_JPRB, 0.25817E+00_JPRB, 0.25869E+00_JPRB, 0.25920E+00_JPRB, 0.26014E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.28256E+00_JPRB, 0.28055E+00_JPRB, 0.28261E+00_JPRB, 0.28710E+00_JPRB, 0.29284E+00_JPRB, 0.29104E+00_JPRB, 0.28813E+00_JPRB ,&
&  0.28246E+00_JPRB, 0.27542E+00_JPRB, 0.26862E+00_JPRB, 0.26227E+00_JPRB, 0.25903E+00_JPRB, 0.25897E+00_JPRB, 0.25849E+00_JPRB ,&
&  0.25737E+00_JPRB, 0.25468E+00_JPRB, 0.25074E+00_JPRB, 0.24682E+00_JPRB, 0.24392E+00_JPRB, 0.24236E+00_JPRB, 0.24099E+00_JPRB ,&
&  0.24009E+00_JPRB, 0.23903E+00_JPRB, 0.23993E+00_JPRB, 0.24002E+00_JPRB, 0.24100E+00_JPRB, 0.24516E+00_JPRB, 0.24843E+00_JPRB ,&
&  0.25021E+00_JPRB, 0.25186E+00_JPRB, 0.25227E+00_JPRB, 0.25301E+00_JPRB, 0.25638E+00_JPRB, 0.26590E+00_JPRB, 0.27862E+00_JPRB ,&
&  0.28941E+00_JPRB, 0.29520E+00_JPRB, 0.29680E+00_JPRB, 0.29530E+00_JPRB, 0.29085E+00_JPRB, 0.28659E+00_JPRB, 0.28382E+00_JPRB ,&
&  0.28160E+00_JPRB, 0.27818E+00_JPRB, 0.27464E+00_JPRB, 0.27175E+00_JPRB, 0.27047E+00_JPRB, 0.26911E+00_JPRB, 0.26699E+00_JPRB ,&
&  0.26431E+00_JPRB, 0.26166E+00_JPRB, 0.25839E+00_JPRB, 0.25501E+00_JPRB, 0.25211E+00_JPRB, 0.25043E+00_JPRB, 0.25042E+00_JPRB ,&
&  0.25182E+00_JPRB, 0.25418E+00_JPRB, 0.25712E+00_JPRB, 0.26062E+00_JPRB, 0.26319E+00_JPRB, 0.26526E+00_JPRB, 0.26587E+00_JPRB ,&
&  0.26578E+00_JPRB, 0.26530E+00_JPRB, 0.26478E+00_JPRB, 0.26427E+00_JPRB, 0.26350E+00_JPRB, 0.26229E+00_JPRB, 0.26091E+00_JPRB ,&
&  0.25970E+00_JPRB, 0.25880E+00_JPRB, 0.25808E+00_JPRB, 0.25741E+00_JPRB, 0.25585E+00_JPRB, 0.25387E+00_JPRB, 0.25255E+00_JPRB ,&
&  0.25187E+00_JPRB, 0.25156E+00_JPRB, 0.25135E+00_JPRB, 0.25081E+00_JPRB, 0.25058E+00_JPRB, 0.25122E+00_JPRB, 0.25281E+00_JPRB ,&
&  0.25547E+00_JPRB, 0.25921E+00_JPRB, 0.26385E+00_JPRB, 0.26921E+00_JPRB, 0.27400E+00_JPRB, 0.27866E+00_JPRB, 0.28309E+00_JPRB ,&
&  0.28578E+00_JPRB, 0.28552E+00_JPRB, 0.28569E+00_JPRB, 0.28638E+00_JPRB, 0.28531E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.30951E+00_JPRB, 0.30475E+00_JPRB, 0.30357E+00_JPRB, 0.30639E+00_JPRB, 0.30917E+00_JPRB, 0.30733E+00_JPRB, 0.30248E+00_JPRB ,&
&  0.29307E+00_JPRB, 0.28471E+00_JPRB, 0.27772E+00_JPRB, 0.27405E+00_JPRB, 0.27476E+00_JPRB, 0.27450E+00_JPRB, 0.27209E+00_JPRB ,&
&  0.26975E+00_JPRB, 0.26568E+00_JPRB, 0.26238E+00_JPRB, 0.25913E+00_JPRB, 0.25667E+00_JPRB, 0.25560E+00_JPRB, 0.25527E+00_JPRB ,&
&  0.25409E+00_JPRB, 0.25281E+00_JPRB, 0.25306E+00_JPRB, 0.25379E+00_JPRB, 0.25623E+00_JPRB, 0.26112E+00_JPRB, 0.26674E+00_JPRB ,&
&  0.27549E+00_JPRB, 0.26346E+00_JPRB, 0.27980E+00_JPRB, 0.28399E+00_JPRB, 0.28839E+00_JPRB, 0.29858E+00_JPRB, 0.31015E+00_JPRB ,&
&  0.31970E+00_JPRB, 0.32444E+00_JPRB, 0.32586E+00_JPRB, 0.32544E+00_JPRB, 0.32149E+00_JPRB, 0.31800E+00_JPRB, 0.31345E+00_JPRB ,&
&  0.30889E+00_JPRB, 0.30739E+00_JPRB, 0.30665E+00_JPRB, 0.30460E+00_JPRB, 0.30221E+00_JPRB, 0.29840E+00_JPRB, 0.29387E+00_JPRB ,&
&  0.28928E+00_JPRB, 0.28477E+00_JPRB, 0.28121E+00_JPRB, 0.27805E+00_JPRB, 0.27576E+00_JPRB, 0.27479E+00_JPRB, 0.27539E+00_JPRB ,&
&  0.27652E+00_JPRB, 0.27853E+00_JPRB, 0.28338E+00_JPRB, 0.28880E+00_JPRB, 0.29278E+00_JPRB, 0.29507E+00_JPRB, 0.29625E+00_JPRB ,&
&  0.29638E+00_JPRB, 0.29593E+00_JPRB, 0.29505E+00_JPRB, 0.29386E+00_JPRB, 0.29193E+00_JPRB, 0.28988E+00_JPRB, 0.28661E+00_JPRB ,&
&  0.28389E+00_JPRB, 0.28173E+00_JPRB, 0.27940E+00_JPRB, 0.27771E+00_JPRB, 0.27565E+00_JPRB, 0.27283E+00_JPRB, 0.27004E+00_JPRB ,&
&  0.26781E+00_JPRB, 0.26621E+00_JPRB, 0.26502E+00_JPRB, 0.26452E+00_JPRB, 0.26473E+00_JPRB, 0.26656E+00_JPRB, 0.26870E+00_JPRB ,&
&  0.27160E+00_JPRB, 0.27628E+00_JPRB, 0.28272E+00_JPRB, 0.29023E+00_JPRB, 0.29869E+00_JPRB, 0.30411E+00_JPRB, 0.30869E+00_JPRB ,&
&  0.31150E+00_JPRB, 0.31141E+00_JPRB, 0.31220E+00_JPRB, 0.31246E+00_JPRB, 0.31191E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.33826E+00_JPRB, 0.33466E+00_JPRB, 0.33027E+00_JPRB, 0.33072E+00_JPRB, 0.32810E+00_JPRB, 0.32317E+00_JPRB, 0.31603E+00_JPRB ,&
&  0.30556E+00_JPRB, 0.29787E+00_JPRB, 0.29190E+00_JPRB, 0.28977E+00_JPRB, 0.29170E+00_JPRB, 0.28984E+00_JPRB, 0.28602E+00_JPRB ,&
&  0.28320E+00_JPRB, 0.27995E+00_JPRB, 0.27771E+00_JPRB, 0.27512E+00_JPRB, 0.27313E+00_JPRB, 0.27248E+00_JPRB, 0.27209E+00_JPRB ,&
&  0.27079E+00_JPRB, 0.26882E+00_JPRB, 0.26743E+00_JPRB, 0.26735E+00_JPRB, 0.27076E+00_JPRB, 0.27753E+00_JPRB, 0.28807E+00_JPRB ,&
&  0.28531E+00_JPRB, 0.27150E+00_JPRB, 0.28840E+00_JPRB, 0.30877E+00_JPRB, 0.32330E+00_JPRB, 0.33865E+00_JPRB, 0.34977E+00_JPRB ,&
&  0.36006E+00_JPRB, 0.36294E+00_JPRB, 0.36428E+00_JPRB, 0.36456E+00_JPRB, 0.36115E+00_JPRB, 0.35507E+00_JPRB, 0.34988E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.34419E+00_JPRB, 0.34110E+00_JPRB, 0.33523E+00_JPRB, 0.32850E+00_JPRB, 0.32102E+00_JPRB, 0.31359E+00_JPRB ,&
&  0.30741E+00_JPRB, 0.30159E+00_JPRB, 0.29722E+00_JPRB, 0.29505E+00_JPRB, 0.29546E+00_JPRB, 0.29748E+00_JPRB, 0.30153E+00_JPRB ,&
&  0.30634E+00_JPRB, 0.31375E+00_JPRB, 0.32433E+00_JPRB, 0.33233E+00_JPRB, 0.33685E+00_JPRB, 0.33801E+00_JPRB, 0.33814E+00_JPRB ,&
&  0.33592E+00_JPRB, 0.33337E+00_JPRB, 0.33190E+00_JPRB, 0.32964E+00_JPRB, 0.32642E+00_JPRB, 0.32203E+00_JPRB, 0.31627E+00_JPRB ,&
&  0.31137E+00_JPRB, 0.30736E+00_JPRB, 0.30353E+00_JPRB, 0.30003E+00_JPRB, 0.29656E+00_JPRB, 0.29293E+00_JPRB, 0.28970E+00_JPRB ,&
&  0.28635E+00_JPRB, 0.28292E+00_JPRB, 0.28057E+00_JPRB, 0.28039E+00_JPRB, 0.28291E+00_JPRB, 0.28622E+00_JPRB, 0.29046E+00_JPRB ,&
&  0.29525E+00_JPRB, 0.30276E+00_JPRB, 0.31085E+00_JPRB, 0.32184E+00_JPRB, 0.33119E+00_JPRB, 0.33412E+00_JPRB, 0.33684E+00_JPRB ,&
&  0.33918E+00_JPRB, 0.33769E+00_JPRB, 0.33952E+00_JPRB, 0.33912E+00_JPRB, 0.33769E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.36455E+00_JPRB, 0.36847E+00_JPRB, 0.36722E+00_JPRB, 0.36005E+00_JPRB, 0.34967E+00_JPRB, 0.34242E+00_JPRB, 0.33199E+00_JPRB ,&
&  0.32101E+00_JPRB, 0.31432E+00_JPRB, 0.30674E+00_JPRB, 0.30343E+00_JPRB, 0.30311E+00_JPRB, 0.29960E+00_JPRB, 0.29685E+00_JPRB ,&
&  0.29457E+00_JPRB, 0.29040E+00_JPRB, 0.28787E+00_JPRB, 0.28545E+00_JPRB, 0.28526E+00_JPRB, 0.28535E+00_JPRB, 0.28522E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.28252E+00_JPRB, 0.28108E+00_JPRB, 0.28052E+00_JPRB, 0.28338E+00_JPRB, 0.29066E+00_JPRB, 0.30406E+00_JPRB ,&
&  0.33246E+00_JPRB, 0.30093E+00_JPRB, 0.31803E+00_JPRB, 0.33252E+00_JPRB, 0.36077E+00_JPRB, 0.38344E+00_JPRB, 0.39407E+00_JPRB ,&
&  0.40612E+00_JPRB, 0.41458E+00_JPRB, 0.41926E+00_JPRB, 0.42133E+00_JPRB, 0.41331E+00_JPRB, 0.40154E+00_JPRB, 0.39324E+00_JPRB ,&
&  0.38418E+00_JPRB, 0.37485E+00_JPRB, 0.36570E+00_JPRB, 0.35553E+00_JPRB, 0.34862E+00_JPRB, 0.34364E+00_JPRB, 0.33794E+00_JPRB ,&
&  0.33090E+00_JPRB, 0.32353E+00_JPRB, 0.31835E+00_JPRB, 0.31678E+00_JPRB, 0.31968E+00_JPRB, 0.32689E+00_JPRB, 0.34047E+00_JPRB ,&
&  0.35355E+00_JPRB, 0.36774E+00_JPRB, 0.38077E+00_JPRB, 0.38793E+00_JPRB, 0.39082E+00_JPRB, 0.38873E+00_JPRB, 0.38434E+00_JPRB ,&
&  0.37836E+00_JPRB, 0.37243E+00_JPRB, 0.36810E+00_JPRB, 0.36242E+00_JPRB, 0.35512E+00_JPRB, 0.34918E+00_JPRB, 0.34453E+00_JPRB ,&
&  0.34106E+00_JPRB, 0.33666E+00_JPRB, 0.33117E+00_JPRB, 0.32570E+00_JPRB, 0.32001E+00_JPRB, 0.31525E+00_JPRB, 0.31113E+00_JPRB ,&
&  0.30728E+00_JPRB, 0.30345E+00_JPRB, 0.30156E+00_JPRB, 0.30327E+00_JPRB, 0.30687E+00_JPRB, 0.31102E+00_JPRB, 0.31493E+00_JPRB ,&
&  0.32052E+00_JPRB, 0.32941E+00_JPRB, 0.33691E+00_JPRB, 0.35206E+00_JPRB, 0.35912E+00_JPRB, 0.36252E+00_JPRB, 0.36550E+00_JPRB ,&
&  0.36565E+00_JPRB, 0.36355E+00_JPRB, 0.32712E+00_JPRB, 0.35119E+00_JPRB, 0.35944E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.40277E+00_JPRB, 0.40597E+00_JPRB, 0.40205E+00_JPRB, 0.38720E+00_JPRB, 0.37175E+00_JPRB, 0.36226E+00_JPRB, 0.35137E+00_JPRB ,&
&  0.33983E+00_JPRB, 0.33035E+00_JPRB, 0.31939E+00_JPRB, 0.31518E+00_JPRB, 0.31253E+00_JPRB, 0.31127E+00_JPRB, 0.30971E+00_JPRB ,&
&  0.30607E+00_JPRB, 0.30008E+00_JPRB, 0.29531E+00_JPRB, 0.29326E+00_JPRB, 0.29363E+00_JPRB, 0.29469E+00_JPRB, 0.29609E+00_JPRB ,&
&  0.29573E+00_JPRB, 0.29576E+00_JPRB, 0.29547E+00_JPRB, 0.29587E+00_JPRB, 0.29812E+00_JPRB, 0.30596E+00_JPRB, 0.32116E+00_JPRB ,&
&  0.35863E+00_JPRB, 0.32769E+00_JPRB, 0.34275E+00_JPRB, 0.36418E+00_JPRB, 0.38909E+00_JPRB, 0.42566E+00_JPRB, 0.46600E+00_JPRB ,&
&  0.48422E+00_JPRB, 0.48165E+00_JPRB, 0.48208E+00_JPRB, 0.48682E+00_JPRB, 0.47563E+00_JPRB, 0.46330E+00_JPRB, 0.44950E+00_JPRB ,&
&  0.43213E+00_JPRB, 0.41388E+00_JPRB, 0.39567E+00_JPRB, 0.38332E+00_JPRB, 0.37745E+00_JPRB, 0.37366E+00_JPRB, 0.37145E+00_JPRB ,&
&  0.36492E+00_JPRB, 0.36087E+00_JPRB, 0.36383E+00_JPRB, 0.36865E+00_JPRB, 0.37407E+00_JPRB, 0.38651E+00_JPRB, 0.40542E+00_JPRB ,&
&  0.41873E+00_JPRB, 0.43268E+00_JPRB, 0.44295E+00_JPRB, 0.44647E+00_JPRB, 0.44870E+00_JPRB, 0.44497E+00_JPRB, 0.43289E+00_JPRB ,&
&  0.42037E+00_JPRB, 0.40740E+00_JPRB, 0.39620E+00_JPRB, 0.38610E+00_JPRB, 0.37864E+00_JPRB, 0.37534E+00_JPRB, 0.37331E+00_JPRB ,&
&  0.37153E+00_JPRB, 0.37015E+00_JPRB, 0.36686E+00_JPRB, 0.36224E+00_JPRB, 0.35814E+00_JPRB, 0.35525E+00_JPRB, 0.35252E+00_JPRB ,&
&  0.34954E+00_JPRB, 0.34655E+00_JPRB, 0.34573E+00_JPRB, 0.34773E+00_JPRB, 0.34994E+00_JPRB, 0.35316E+00_JPRB, 0.35752E+00_JPRB ,&
&  0.36163E+00_JPRB, 0.37024E+00_JPRB, 0.37514E+00_JPRB, 0.39741E+00_JPRB, 0.40470E+00_JPRB, 0.40396E+00_JPRB, 0.40339E+00_JPRB ,&
&  0.39886E+00_JPRB, 0.39459E+00_JPRB, 0.39155E+00_JPRB, 0.37158E+00_JPRB, 0.39602E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.45220E+00_JPRB, 0.44868E+00_JPRB, 0.43142E+00_JPRB, 0.40915E+00_JPRB, 0.39185E+00_JPRB, 0.37663E+00_JPRB, 0.36388E+00_JPRB ,&
&  0.35542E+00_JPRB, 0.34609E+00_JPRB, 0.33579E+00_JPRB, 0.32823E+00_JPRB, 0.32687E+00_JPRB, 0.32433E+00_JPRB, 0.32093E+00_JPRB ,&
&  0.31487E+00_JPRB, 0.30901E+00_JPRB, 0.30509E+00_JPRB, 0.30358E+00_JPRB, 0.30386E+00_JPRB, 0.30552E+00_JPRB, 0.30670E+00_JPRB ,&
&  0.30579E+00_JPRB, 0.30316E+00_JPRB, 0.30393E+00_JPRB, 0.30678E+00_JPRB, 0.31213E+00_JPRB, 0.31988E+00_JPRB, 0.33819E+00_JPRB ,&
&  0.37989E+00_JPRB, 0.36446E+00_JPRB, 0.38651E+00_JPRB, 0.39887E+00_JPRB, 0.42042E+00_JPRB, 0.46052E+00_JPRB, 0.50364E+00_JPRB ,&
&  0.53291E+00_JPRB, 0.53711E+00_JPRB, 0.54698E+00_JPRB, 0.55641E+00_JPRB, 0.54586E+00_JPRB, 0.52570E+00_JPRB, 0.50203E+00_JPRB ,&
&  0.47915E+00_JPRB, 0.46201E+00_JPRB, 0.44721E+00_JPRB, 0.43512E+00_JPRB, 0.42653E+00_JPRB, 0.42231E+00_JPRB, 0.42143E+00_JPRB ,&
&  0.41616E+00_JPRB, 0.41428E+00_JPRB, 0.41932E+00_JPRB, 0.43207E+00_JPRB, 0.44471E+00_JPRB, 0.45711E+00_JPRB, 0.47068E+00_JPRB ,&
&  0.48073E+00_JPRB, 0.48828E+00_JPRB, 0.49250E+00_JPRB, 0.49265E+00_JPRB, 0.48541E+00_JPRB, 0.47611E+00_JPRB, 0.45702E+00_JPRB ,&
&  0.43793E+00_JPRB, 0.42298E+00_JPRB, 0.41494E+00_JPRB, 0.40921E+00_JPRB, 0.40514E+00_JPRB, 0.40276E+00_JPRB, 0.40061E+00_JPRB ,&
&  0.39923E+00_JPRB, 0.40030E+00_JPRB, 0.40126E+00_JPRB, 0.40186E+00_JPRB, 0.40228E+00_JPRB, 0.40257E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.40000E+00_JPRB, 0.39965E+00_JPRB, 0.39968E+00_JPRB, 0.39910E+00_JPRB, 0.39830E+00_JPRB, 0.39806E+00_JPRB, 0.39893E+00_JPRB ,&
&  0.40151E+00_JPRB, 0.41067E+00_JPRB, 0.41546E+00_JPRB, 0.43207E+00_JPRB, 0.43372E+00_JPRB, 0.43327E+00_JPRB, 0.43230E+00_JPRB ,&
&  0.42760E+00_JPRB, 0.42231E+00_JPRB, 0.41296E+00_JPRB, 0.41036E+00_JPRB, 0.42901E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.53472E+00_JPRB, 0.50423E+00_JPRB, 0.46420E+00_JPRB, 0.43278E+00_JPRB, 0.41029E+00_JPRB, 0.39309E+00_JPRB, 0.38255E+00_JPRB ,&
&  0.37827E+00_JPRB, 0.36877E+00_JPRB, 0.35632E+00_JPRB, 0.34603E+00_JPRB, 0.34095E+00_JPRB, 0.33703E+00_JPRB, 0.33168E+00_JPRB ,&
&  0.32691E+00_JPRB, 0.32270E+00_JPRB, 0.32008E+00_JPRB, 0.31895E+00_JPRB, 0.31827E+00_JPRB, 0.31749E+00_JPRB, 0.31498E+00_JPRB ,&
&  0.31102E+00_JPRB, 0.31100E+00_JPRB, 0.31314E+00_JPRB, 0.31980E+00_JPRB, 0.32820E+00_JPRB, 0.33748E+00_JPRB, 0.35527E+00_JPRB ,&
&  0.40560E+00_JPRB, 0.40461E+00_JPRB, 0.44406E+00_JPRB, 0.46098E+00_JPRB, 0.49160E+00_JPRB, 0.53776E+00_JPRB, 0.56536E+00_JPRB ,&
&  0.59043E+00_JPRB, 0.60618E+00_JPRB, 0.63314E+00_JPRB, 0.64069E+00_JPRB, 0.62077E+00_JPRB, 0.59113E+00_JPRB, 0.56400E+00_JPRB ,&
&  0.53821E+00_JPRB, 0.51948E+00_JPRB, 0.50188E+00_JPRB, 0.48614E+00_JPRB, 0.47539E+00_JPRB, 0.47353E+00_JPRB, 0.47728E+00_JPRB ,&
&  0.47847E+00_JPRB, 0.47055E+00_JPRB, 0.47872E+00_JPRB, 0.49270E+00_JPRB, 0.51277E+00_JPRB, 0.52442E+00_JPRB, 0.53390E+00_JPRB ,&
&  0.54367E+00_JPRB, 0.54448E+00_JPRB, 0.54114E+00_JPRB, 0.53038E+00_JPRB, 0.50786E+00_JPRB, 0.48318E+00_JPRB, 0.45949E+00_JPRB ,&
&  0.43681E+00_JPRB, 0.42234E+00_JPRB, 0.41599E+00_JPRB, 0.41298E+00_JPRB, 0.41330E+00_JPRB, 0.41580E+00_JPRB, 0.42114E+00_JPRB ,&
&  0.42966E+00_JPRB, 0.43754E+00_JPRB, 0.44364E+00_JPRB, 0.44632E+00_JPRB, 0.44504E+00_JPRB, 0.44224E+00_JPRB, 0.43732E+00_JPRB ,&
&  0.43180E+00_JPRB, 0.42673E+00_JPRB, 0.42321E+00_JPRB, 0.42192E+00_JPRB, 0.42214E+00_JPRB, 0.42384E+00_JPRB, 0.42676E+00_JPRB ,&
&  0.43113E+00_JPRB, 0.44300E+00_JPRB, 0.44091E+00_JPRB, 0.46150E+00_JPRB, 0.47852E+00_JPRB, 0.47214E+00_JPRB, 0.46409E+00_JPRB ,&
&  0.45328E+00_JPRB, 0.44574E+00_JPRB, 0.43976E+00_JPRB, 0.44191E+00_JPRB, 0.51693E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.48496E+00_JPRB, 0.47376E+00_JPRB, 0.45049E+00_JPRB, 0.42919E+00_JPRB, 0.41517E+00_JPRB, 0.40475E+00_JPRB, 0.39880E+00_JPRB ,&
&  0.39536E+00_JPRB, 0.38784E+00_JPRB, 0.38067E+00_JPRB, 0.37072E+00_JPRB, 0.36231E+00_JPRB, 0.35535E+00_JPRB, 0.34868E+00_JPRB ,&
&  0.34289E+00_JPRB, 0.33944E+00_JPRB, 0.33663E+00_JPRB, 0.33389E+00_JPRB, 0.32952E+00_JPRB, 0.32305E+00_JPRB, 0.31934E+00_JPRB ,&
&  0.31844E+00_JPRB, 0.31994E+00_JPRB, 0.32439E+00_JPRB, 0.33292E+00_JPRB, 0.34124E+00_JPRB, 0.35005E+00_JPRB, 0.36693E+00_JPRB ,&
&  0.41149E+00_JPRB, 0.49242E+00_JPRB, 0.54037E+00_JPRB, 0.56691E+00_JPRB, 0.65409E+00_JPRB, 0.64889E+00_JPRB, 0.64806E+00_JPRB ,&
&  0.67179E+00_JPRB, 0.69541E+00_JPRB, 0.72010E+00_JPRB, 0.72802E+00_JPRB, 0.70823E+00_JPRB, 0.68569E+00_JPRB, 0.65427E+00_JPRB ,&
&  0.62251E+00_JPRB, 0.58858E+00_JPRB, 0.55598E+00_JPRB, 0.52601E+00_JPRB, 0.50003E+00_JPRB, 0.49513E+00_JPRB, 0.50337E+00_JPRB ,&
&  0.51899E+00_JPRB, 0.52867E+00_JPRB, 0.53111E+00_JPRB, 0.54237E+00_JPRB, 0.56319E+00_JPRB, 0.58973E+00_JPRB, 0.62092E+00_JPRB ,&
&  0.63343E+00_JPRB, 0.61643E+00_JPRB, 0.59079E+00_JPRB, 0.55823E+00_JPRB, 0.51797E+00_JPRB, 0.48261E+00_JPRB, 0.45530E+00_JPRB ,&
&  0.43511E+00_JPRB, 0.42122E+00_JPRB, 0.41255E+00_JPRB, 0.40973E+00_JPRB, 0.41397E+00_JPRB, 0.42518E+00_JPRB, 0.43783E+00_JPRB ,&
&  0.45109E+00_JPRB, 0.46261E+00_JPRB, 0.46838E+00_JPRB, 0.46951E+00_JPRB, 0.46467E+00_JPRB, 0.45488E+00_JPRB, 0.43954E+00_JPRB ,&
&  0.42633E+00_JPRB, 0.41932E+00_JPRB, 0.42383E+00_JPRB, 0.42792E+00_JPRB, 0.43145E+00_JPRB, 0.43541E+00_JPRB, 0.43995E+00_JPRB ,&
&  0.44905E+00_JPRB, 0.46139E+00_JPRB, 0.46227E+00_JPRB, 0.46231E+00_JPRB, 0.46777E+00_JPRB, 0.48463E+00_JPRB, 0.47515E+00_JPRB ,&
&  0.46731E+00_JPRB, 0.45963E+00_JPRB, 0.45697E+00_JPRB, 0.45660E+00_JPRB, 0.48265E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.47501E+00_JPRB, 0.47007E+00_JPRB, 0.45574E+00_JPRB, 0.43940E+00_JPRB, 0.42793E+00_JPRB, 0.41894E+00_JPRB, 0.41249E+00_JPRB ,&
&  0.40860E+00_JPRB, 0.40168E+00_JPRB, 0.39567E+00_JPRB, 0.38774E+00_JPRB, 0.37828E+00_JPRB, 0.37051E+00_JPRB, 0.36572E+00_JPRB ,&
&  0.35910E+00_JPRB, 0.35565E+00_JPRB, 0.35183E+00_JPRB, 0.34433E+00_JPRB, 0.33633E+00_JPRB, 0.32975E+00_JPRB, 0.32555E+00_JPRB ,&
&  0.32425E+00_JPRB, 0.32711E+00_JPRB, 0.33294E+00_JPRB, 0.34526E+00_JPRB, 0.35545E+00_JPRB, 0.36502E+00_JPRB, 0.38554E+00_JPRB ,&
&  0.44829E+00_JPRB, 0.56727E+00_JPRB, 0.74784E+00_JPRB, 0.70880E+00_JPRB, 0.71399E+00_JPRB, 0.67168E+00_JPRB, 0.72581E+00_JPRB ,&
&  0.76183E+00_JPRB, 0.80817E+00_JPRB, 0.83792E+00_JPRB, 0.84549E+00_JPRB, 0.80587E+00_JPRB, 0.77335E+00_JPRB, 0.73851E+00_JPRB ,&
&  0.70229E+00_JPRB, 0.66214E+00_JPRB, 0.61234E+00_JPRB, 0.56343E+00_JPRB, 0.53447E+00_JPRB, 0.53222E+00_JPRB, 0.54569E+00_JPRB ,&
&  0.56646E+00_JPRB, 0.57940E+00_JPRB, 0.57445E+00_JPRB, 0.58185E+00_JPRB, 0.58138E+00_JPRB, 0.57844E+00_JPRB, 0.64713E+00_JPRB ,&
&  0.77876E+00_JPRB, 0.74211E+00_JPRB, 0.67626E+00_JPRB, 0.59443E+00_JPRB, 0.53549E+00_JPRB, 0.49008E+00_JPRB, 0.46380E+00_JPRB ,&
&  0.44451E+00_JPRB, 0.43148E+00_JPRB, 0.42392E+00_JPRB, 0.42226E+00_JPRB, 0.42891E+00_JPRB, 0.43990E+00_JPRB, 0.44805E+00_JPRB ,&
&  0.45676E+00_JPRB, 0.46349E+00_JPRB, 0.46608E+00_JPRB, 0.46261E+00_JPRB, 0.45449E+00_JPRB, 0.44279E+00_JPRB, 0.43144E+00_JPRB ,&
&  0.42637E+00_JPRB, 0.42915E+00_JPRB, 0.44314E+00_JPRB, 0.44044E+00_JPRB, 0.44586E+00_JPRB, 0.45683E+00_JPRB, 0.45894E+00_JPRB ,&
&  0.45543E+00_JPRB, 0.45901E+00_JPRB, 0.46759E+00_JPRB, 0.46681E+00_JPRB, 0.47688E+00_JPRB, 0.50643E+00_JPRB, 0.48955E+00_JPRB ,&
&  0.47460E+00_JPRB, 0.46430E+00_JPRB, 0.46356E+00_JPRB, 0.46124E+00_JPRB, 0.46966E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.47657E+00_JPRB, 0.47356E+00_JPRB, 0.46212E+00_JPRB, 0.44758E+00_JPRB, 0.43632E+00_JPRB, 0.42509E+00_JPRB, 0.41682E+00_JPRB ,&
&  0.41104E+00_JPRB, 0.40640E+00_JPRB, 0.40306E+00_JPRB, 0.39890E+00_JPRB, 0.39310E+00_JPRB, 0.38522E+00_JPRB, 0.37761E+00_JPRB ,&
&  0.36930E+00_JPRB, 0.36106E+00_JPRB, 0.35309E+00_JPRB, 0.34477E+00_JPRB, 0.33870E+00_JPRB, 0.33446E+00_JPRB, 0.33165E+00_JPRB ,&
&  0.33183E+00_JPRB, 0.33505E+00_JPRB, 0.34274E+00_JPRB, 0.35824E+00_JPRB, 0.37573E+00_JPRB, 0.38520E+00_JPRB, 0.39859E+00_JPRB ,&
&  0.45050E+00_JPRB, 0.40221E+00_JPRB, 0.68333E+00_JPRB, 0.80499E+00_JPRB, 0.79872E+00_JPRB, 0.79160E+00_JPRB, 0.86955E+00_JPRB ,&
&  0.93582E+00_JPRB, 0.93836E+00_JPRB, 0.92599E+00_JPRB, 0.88772E+00_JPRB, 0.81250E+00_JPRB, 0.75182E+00_JPRB, 0.70254E+00_JPRB ,&
&  0.66184E+00_JPRB, 0.62753E+00_JPRB, 0.59006E+00_JPRB, 0.55346E+00_JPRB, 0.52595E+00_JPRB, 0.52054E+00_JPRB, 0.52857E+00_JPRB ,&
&  0.55851E+00_JPRB, 0.57673E+00_JPRB, 0.59060E+00_JPRB, 0.61140E+00_JPRB, 0.59486E+00_JPRB, 0.63425E+00_JPRB, 0.65619E+00_JPRB ,&
&  0.77965E+00_JPRB, 0.84958E+00_JPRB, 0.73254E+00_JPRB, 0.61281E+00_JPRB, 0.53694E+00_JPRB, 0.49959E+00_JPRB, 0.46986E+00_JPRB ,&
&  0.45428E+00_JPRB, 0.44458E+00_JPRB, 0.43893E+00_JPRB, 0.43732E+00_JPRB, 0.44082E+00_JPRB, 0.44603E+00_JPRB, 0.44795E+00_JPRB ,&
&  0.45047E+00_JPRB, 0.45246E+00_JPRB, 0.45327E+00_JPRB, 0.45225E+00_JPRB, 0.44759E+00_JPRB, 0.44042E+00_JPRB, 0.43766E+00_JPRB ,&
&  0.44193E+00_JPRB, 0.45350E+00_JPRB, 0.46335E+00_JPRB, 0.46376E+00_JPRB, 0.46877E+00_JPRB, 0.47427E+00_JPRB, 0.47678E+00_JPRB ,&
&  0.47795E+00_JPRB, 0.47858E+00_JPRB, 0.47722E+00_JPRB, 0.48456E+00_JPRB, 0.47754E+00_JPRB, 0.50572E+00_JPRB, 0.48714E+00_JPRB ,&
&  0.47378E+00_JPRB, 0.46748E+00_JPRB, 0.46613E+00_JPRB, 0.46707E+00_JPRB, 0.47122E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.48195E+00_JPRB, 0.47890E+00_JPRB, 0.46823E+00_JPRB, 0.45438E+00_JPRB, 0.43988E+00_JPRB, 0.42786E+00_JPRB, 0.42277E+00_JPRB ,&
&  0.41701E+00_JPRB, 0.41075E+00_JPRB, 0.40487E+00_JPRB, 0.40198E+00_JPRB, 0.40104E+00_JPRB, 0.39492E+00_JPRB, 0.38381E+00_JPRB ,&
&  0.37188E+00_JPRB, 0.36051E+00_JPRB, 0.35145E+00_JPRB, 0.34717E+00_JPRB, 0.34244E+00_JPRB, 0.33999E+00_JPRB, 0.34012E+00_JPRB ,&
&  0.34139E+00_JPRB, 0.34718E+00_JPRB, 0.35892E+00_JPRB, 0.37847E+00_JPRB, 0.39440E+00_JPRB, 0.40010E+00_JPRB, 0.40782E+00_JPRB ,&
&  0.44992E+00_JPRB, 0.62530E+00_JPRB, 0.60212E+00_JPRB, 0.93775E+00_JPRB, 0.97383E+00_JPRB, 0.99332E+00_JPRB, 0.99793E+00_JPRB ,&
&  0.10051E+01_JPRB, 0.10169E+01_JPRB, 0.96585E+00_JPRB, 0.89166E+00_JPRB, 0.79273E+00_JPRB, 0.71526E+00_JPRB, 0.64764E+00_JPRB ,&
&  0.59400E+00_JPRB, 0.55521E+00_JPRB, 0.52788E+00_JPRB, 0.50702E+00_JPRB, 0.49400E+00_JPRB, 0.49331E+00_JPRB, 0.50946E+00_JPRB ,&
&  0.54303E+00_JPRB, 0.58044E+00_JPRB, 0.62306E+00_JPRB, 0.66436E+00_JPRB, 0.63593E+00_JPRB, 0.67921E+00_JPRB, 0.72626E+00_JPRB ,&
&  0.77868E+00_JPRB, 0.84693E+00_JPRB, 0.70248E+00_JPRB, 0.58080E+00_JPRB, 0.52547E+00_JPRB, 0.50247E+00_JPRB, 0.48416E+00_JPRB ,&
&  0.47216E+00_JPRB, 0.46407E+00_JPRB, 0.45895E+00_JPRB, 0.45441E+00_JPRB, 0.45173E+00_JPRB, 0.45095E+00_JPRB, 0.45049E+00_JPRB ,&
&  0.45128E+00_JPRB, 0.45288E+00_JPRB, 0.45507E+00_JPRB, 0.45634E+00_JPRB, 0.45441E+00_JPRB, 0.45506E+00_JPRB, 0.46098E+00_JPRB ,&
&  0.47143E+00_JPRB, 0.48196E+00_JPRB, 0.48515E+00_JPRB, 0.48339E+00_JPRB, 0.48808E+00_JPRB, 0.49356E+00_JPRB, 0.49049E+00_JPRB ,&
&  0.49699E+00_JPRB, 0.49901E+00_JPRB, 0.49799E+00_JPRB, 0.49849E+00_JPRB, 0.49814E+00_JPRB, 0.53053E+00_JPRB, 0.49240E+00_JPRB ,&
&  0.47915E+00_JPRB, 0.47565E+00_JPRB, 0.47651E+00_JPRB, 0.47465E+00_JPRB, 0.47827E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.48748E+00_JPRB, 0.47804E+00_JPRB, 0.46716E+00_JPRB, 0.45450E+00_JPRB, 0.44068E+00_JPRB, 0.43134E+00_JPRB, 0.42715E+00_JPRB ,&
&  0.41908E+00_JPRB, 0.41168E+00_JPRB, 0.40616E+00_JPRB, 0.40413E+00_JPRB, 0.40238E+00_JPRB, 0.39563E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.36995E+00_JPRB, 0.36172E+00_JPRB, 0.35779E+00_JPRB, 0.35369E+00_JPRB, 0.34881E+00_JPRB, 0.34627E+00_JPRB, 0.34746E+00_JPRB ,&
&  0.35266E+00_JPRB, 0.36112E+00_JPRB, 0.37799E+00_JPRB, 0.39771E+00_JPRB, 0.41416E+00_JPRB, 0.42724E+00_JPRB, 0.43720E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.59637E+00_JPRB, 0.65642E+00_JPRB, 0.11551E+01_JPRB, 0.12008E+01_JPRB, 0.11475E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10823E+01_JPRB, 0.98804E+00_JPRB, 0.87348E+00_JPRB, 0.75755E+00_JPRB, 0.67596E+00_JPRB, 0.61215E+00_JPRB ,&
&  0.55850E+00_JPRB, 0.52330E+00_JPRB, 0.50666E+00_JPRB, 0.50005E+00_JPRB, 0.49721E+00_JPRB, 0.50239E+00_JPRB, 0.52085E+00_JPRB ,&
&  0.55558E+00_JPRB, 0.60683E+00_JPRB, 0.68286E+00_JPRB, 0.71924E+00_JPRB, 0.70644E+00_JPRB, 0.71746E+00_JPRB, 0.71987E+00_JPRB ,&
&  0.77149E+00_JPRB, 0.77119E+00_JPRB, 0.64182E+00_JPRB, 0.56536E+00_JPRB, 0.52520E+00_JPRB, 0.51667E+00_JPRB, 0.49828E+00_JPRB ,&
&  0.48897E+00_JPRB, 0.48163E+00_JPRB, 0.47615E+00_JPRB, 0.46860E+00_JPRB, 0.46070E+00_JPRB, 0.45776E+00_JPRB, 0.45796E+00_JPRB ,&
&  0.45928E+00_JPRB, 0.46256E+00_JPRB, 0.46449E+00_JPRB, 0.46612E+00_JPRB, 0.46810E+00_JPRB, 0.47420E+00_JPRB, 0.48375E+00_JPRB ,&
&  0.49151E+00_JPRB, 0.50623E+00_JPRB, 0.50449E+00_JPRB, 0.50085E+00_JPRB, 0.51248E+00_JPRB, 0.52031E+00_JPRB, 0.50933E+00_JPRB ,&
&  0.50871E+00_JPRB, 0.51767E+00_JPRB, 0.51813E+00_JPRB, 0.50496E+00_JPRB, 0.51396E+00_JPRB, 0.53140E+00_JPRB, 0.50105E+00_JPRB ,&
&  0.49289E+00_JPRB, 0.49141E+00_JPRB, 0.49566E+00_JPRB, 0.49271E+00_JPRB, 0.49013E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.49080E+00_JPRB, 0.47759E+00_JPRB, 0.46709E+00_JPRB, 0.45888E+00_JPRB, 0.44820E+00_JPRB, 0.43709E+00_JPRB, 0.42871E+00_JPRB ,&
&  0.41553E+00_JPRB, 0.40697E+00_JPRB, 0.40094E+00_JPRB, 0.39404E+00_JPRB, 0.38748E+00_JPRB, 0.38155E+00_JPRB, 0.37651E+00_JPRB ,&
&  0.37324E+00_JPRB, 0.37063E+00_JPRB, 0.36881E+00_JPRB, 0.36506E+00_JPRB, 0.36202E+00_JPRB, 0.36178E+00_JPRB, 0.36451E+00_JPRB ,&
&  0.37670E+00_JPRB, 0.39408E+00_JPRB, 0.41457E+00_JPRB, 0.43629E+00_JPRB, 0.45647E+00_JPRB, 0.48178E+00_JPRB, 0.51103E+00_JPRB ,&
&  0.61304E+00_JPRB, 0.65958E+00_JPRB, 0.80559E+00_JPRB, 0.12854E+01_JPRB, 0.13255E+01_JPRB, 0.11656E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.91789E+00_JPRB, 0.93751E+00_JPRB, 0.81673E+00_JPRB, 0.70600E+00_JPRB, 0.64648E+00_JPRB, 0.60429E+00_JPRB ,&
&  0.56502E+00_JPRB, 0.54213E+00_JPRB, 0.53632E+00_JPRB, 0.53298E+00_JPRB, 0.52650E+00_JPRB, 0.53654E+00_JPRB, 0.56233E+00_JPRB ,&
&  0.61402E+00_JPRB, 0.68800E+00_JPRB, 0.79473E+00_JPRB, 0.85483E+00_JPRB, 0.75339E+00_JPRB, 0.72365E+00_JPRB, 0.69185E+00_JPRB ,&
&  0.72033E+00_JPRB, 0.68259E+00_JPRB, 0.60858E+00_JPRB, 0.56403E+00_JPRB, 0.53081E+00_JPRB, 0.53300E+00_JPRB, 0.51097E+00_JPRB ,&
&  0.50196E+00_JPRB, 0.49407E+00_JPRB, 0.48558E+00_JPRB, 0.47320E+00_JPRB, 0.46546E+00_JPRB, 0.45957E+00_JPRB, 0.45972E+00_JPRB ,&
&  0.46494E+00_JPRB, 0.47017E+00_JPRB, 0.47532E+00_JPRB, 0.48159E+00_JPRB, 0.48900E+00_JPRB, 0.49527E+00_JPRB, 0.50190E+00_JPRB ,&
&  0.50890E+00_JPRB, 0.52720E+00_JPRB, 0.52499E+00_JPRB, 0.53568E+00_JPRB, 0.55272E+00_JPRB, 0.55903E+00_JPRB, 0.53667E+00_JPRB ,&
&  0.52278E+00_JPRB, 0.53134E+00_JPRB, 0.53118E+00_JPRB, 0.52826E+00_JPRB, 0.53521E+00_JPRB, 0.53920E+00_JPRB, 0.51768E+00_JPRB ,&
&  0.51873E+00_JPRB, 0.52339E+00_JPRB, 0.53210E+00_JPRB, 0.51866E+00_JPRB, 0.50540E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.49721E+00_JPRB, 0.47800E+00_JPRB, 0.46429E+00_JPRB, 0.45038E+00_JPRB, 0.43569E+00_JPRB, 0.41984E+00_JPRB, 0.40735E+00_JPRB ,&
&  0.39212E+00_JPRB, 0.38895E+00_JPRB, 0.38574E+00_JPRB, 0.38086E+00_JPRB, 0.37530E+00_JPRB, 0.37589E+00_JPRB, 0.38213E+00_JPRB ,&
&  0.38761E+00_JPRB, 0.38944E+00_JPRB, 0.39156E+00_JPRB, 0.39327E+00_JPRB, 0.39786E+00_JPRB, 0.40735E+00_JPRB, 0.42161E+00_JPRB ,&
&  0.44028E+00_JPRB, 0.46160E+00_JPRB, 0.48983E+00_JPRB, 0.52335E+00_JPRB, 0.55614E+00_JPRB, 0.60338E+00_JPRB, 0.66057E+00_JPRB ,&
&  0.77062E+00_JPRB, 0.77159E+00_JPRB, 0.50385E+00_JPRB, 0.84757E+00_JPRB, 0.10855E+01_JPRB, 0.10115E+01_JPRB, 0.93142E+00_JPRB ,&
&  0.90287E+00_JPRB, 0.83984E+00_JPRB, 0.82008E+00_JPRB, 0.75903E+00_JPRB, 0.67872E+00_JPRB, 0.65601E+00_JPRB, 0.63748E+00_JPRB ,&
&  0.61355E+00_JPRB, 0.59444E+00_JPRB, 0.58267E+00_JPRB, 0.57868E+00_JPRB, 0.57173E+00_JPRB, 0.58409E+00_JPRB, 0.63885E+00_JPRB ,&
&  0.71385E+00_JPRB, 0.82259E+00_JPRB, 0.95360E+00_JPRB, 0.91316E+00_JPRB, 0.78226E+00_JPRB, 0.71023E+00_JPRB, 0.65159E+00_JPRB ,&
&  0.62292E+00_JPRB, 0.62872E+00_JPRB, 0.60124E+00_JPRB, 0.57005E+00_JPRB, 0.53960E+00_JPRB, 0.51891E+00_JPRB, 0.51865E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.49303E+00_JPRB, 0.48238E+00_JPRB, 0.46645E+00_JPRB, 0.46098E+00_JPRB, 0.45926E+00_JPRB, 0.46628E+00_JPRB ,&
&  0.47527E+00_JPRB, 0.48263E+00_JPRB, 0.49119E+00_JPRB, 0.50075E+00_JPRB, 0.50545E+00_JPRB, 0.51219E+00_JPRB, 0.51875E+00_JPRB ,&
&  0.52892E+00_JPRB, 0.54799E+00_JPRB, 0.56864E+00_JPRB, 0.58104E+00_JPRB, 0.58449E+00_JPRB, 0.58168E+00_JPRB, 0.56948E+00_JPRB ,&
&  0.56523E+00_JPRB, 0.56326E+00_JPRB, 0.54623E+00_JPRB, 0.54153E+00_JPRB, 0.56518E+00_JPRB, 0.57268E+00_JPRB, 0.58382E+00_JPRB ,&
&  0.62307E+00_JPRB, 0.65759E+00_JPRB, 0.66183E+00_JPRB, 0.55699E+00_JPRB, 0.52154E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.47532E+00_JPRB, 0.44256E+00_JPRB, 0.41533E+00_JPRB, 0.40079E+00_JPRB, 0.39063E+00_JPRB, 0.38616E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38508E+00_JPRB, 0.38461E+00_JPRB, 0.37961E+00_JPRB, 0.38315E+00_JPRB, 0.39349E+00_JPRB, 0.40449E+00_JPRB, 0.41918E+00_JPRB ,&
&  0.42603E+00_JPRB, 0.43043E+00_JPRB, 0.43814E+00_JPRB, 0.45539E+00_JPRB, 0.47865E+00_JPRB, 0.50236E+00_JPRB, 0.52371E+00_JPRB ,&
&  0.54697E+00_JPRB, 0.57327E+00_JPRB, 0.60795E+00_JPRB, 0.65430E+00_JPRB, 0.71617E+00_JPRB, 0.79504E+00_JPRB, 0.87266E+00_JPRB ,&
&  0.95199E+00_JPRB, 0.72659E+00_JPRB, 0.57384E+00_JPRB, 0.94521E+00_JPRB, 0.10215E+01_JPRB, 0.95501E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.80392E+00_JPRB, 0.80457E+00_JPRB, 0.77536E+00_JPRB, 0.75235E+00_JPRB, 0.69274E+00_JPRB, 0.67792E+00_JPRB, 0.66640E+00_JPRB ,&
&  0.64986E+00_JPRB, 0.63153E+00_JPRB, 0.63406E+00_JPRB, 0.64089E+00_JPRB, 0.64897E+00_JPRB, 0.68771E+00_JPRB, 0.76391E+00_JPRB ,&
&  0.86239E+00_JPRB, 0.99382E+00_JPRB, 0.11525E+01_JPRB, 0.10103E+01_JPRB, 0.85035E+00_JPRB, 0.71710E+00_JPRB, 0.65306E+00_JPRB ,&
&  0.60997E+00_JPRB, 0.61718E+00_JPRB, 0.60667E+00_JPRB, 0.58058E+00_JPRB, 0.55109E+00_JPRB, 0.52006E+00_JPRB, 0.50985E+00_JPRB ,&
&  0.49542E+00_JPRB, 0.47912E+00_JPRB, 0.46778E+00_JPRB, 0.45676E+00_JPRB, 0.45761E+00_JPRB, 0.46602E+00_JPRB, 0.47482E+00_JPRB ,&
&  0.48263E+00_JPRB, 0.49013E+00_JPRB, 0.50251E+00_JPRB, 0.51765E+00_JPRB, 0.52569E+00_JPRB, 0.53175E+00_JPRB, 0.54202E+00_JPRB ,&
&  0.55227E+00_JPRB, 0.56400E+00_JPRB, 0.58547E+00_JPRB, 0.59702E+00_JPRB, 0.59609E+00_JPRB, 0.57753E+00_JPRB, 0.58068E+00_JPRB ,&
&  0.57956E+00_JPRB, 0.58615E+00_JPRB, 0.57971E+00_JPRB, 0.58952E+00_JPRB, 0.59502E+00_JPRB, 0.62209E+00_JPRB, 0.65172E+00_JPRB ,&
&  0.67433E+00_JPRB, 0.67461E+00_JPRB, 0.64487E+00_JPRB, 0.53973E+00_JPRB, 0.50348E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.40169E+00_JPRB, 0.38046E+00_JPRB, 0.38268E+00_JPRB, 0.38739E+00_JPRB, 0.40044E+00_JPRB, 0.41193E+00_JPRB, 0.40657E+00_JPRB ,&
&  0.41045E+00_JPRB, 0.41623E+00_JPRB, 0.42648E+00_JPRB, 0.43758E+00_JPRB, 0.45254E+00_JPRB, 0.46760E+00_JPRB, 0.47559E+00_JPRB ,&
&  0.49184E+00_JPRB, 0.51304E+00_JPRB, 0.54070E+00_JPRB, 0.56808E+00_JPRB, 0.59569E+00_JPRB, 0.61708E+00_JPRB, 0.63663E+00_JPRB ,&
&  0.66107E+00_JPRB, 0.70186E+00_JPRB, 0.75526E+00_JPRB, 0.81663E+00_JPRB, 0.88967E+00_JPRB, 0.97554E+00_JPRB, 0.10220E+01_JPRB ,&
&  0.79479E+00_JPRB, 0.50357E+00_JPRB, 0.95876E+00_JPRB, 0.10670E+01_JPRB, 0.93540E+00_JPRB, 0.86815E+00_JPRB, 0.80413E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.75888E+00_JPRB, 0.72864E+00_JPRB, 0.73987E+00_JPRB, 0.70176E+00_JPRB, 0.69090E+00_JPRB, 0.69066E+00_JPRB ,&
&  0.68691E+00_JPRB, 0.69121E+00_JPRB, 0.71813E+00_JPRB, 0.74008E+00_JPRB, 0.77309E+00_JPRB, 0.83266E+00_JPRB, 0.92389E+00_JPRB ,&
&  0.10524E+01_JPRB, 0.12099E+01_JPRB, 0.13682E+01_JPRB, 0.11056E+01_JPRB, 0.93344E+00_JPRB, 0.81622E+00_JPRB, 0.72760E+00_JPRB ,&
&  0.67634E+00_JPRB, 0.60658E+00_JPRB, 0.59431E+00_JPRB, 0.58945E+00_JPRB, 0.56294E+00_JPRB, 0.50954E+00_JPRB, 0.49651E+00_JPRB ,&
&  0.47246E+00_JPRB, 0.46031E+00_JPRB, 0.45488E+00_JPRB, 0.45063E+00_JPRB, 0.45078E+00_JPRB, 0.45752E+00_JPRB, 0.46525E+00_JPRB ,&
&  0.47712E+00_JPRB, 0.48284E+00_JPRB, 0.49535E+00_JPRB, 0.51057E+00_JPRB, 0.52282E+00_JPRB, 0.53665E+00_JPRB, 0.55404E+00_JPRB ,&
&  0.56181E+00_JPRB, 0.57773E+00_JPRB, 0.59237E+00_JPRB, 0.59524E+00_JPRB, 0.59736E+00_JPRB, 0.58439E+00_JPRB, 0.58141E+00_JPRB ,&
&  0.58116E+00_JPRB, 0.58261E+00_JPRB, 0.57890E+00_JPRB, 0.58727E+00_JPRB, 0.59776E+00_JPRB, 0.60572E+00_JPRB, 0.62543E+00_JPRB ,&
&  0.61986E+00_JPRB, 0.59337E+00_JPRB, 0.53467E+00_JPRB, 0.46807E+00_JPRB, 0.43633E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.39146E+00_JPRB, 0.40834E+00_JPRB, 0.42730E+00_JPRB, 0.44408E+00_JPRB, 0.45841E+00_JPRB, 0.46980E+00_JPRB, 0.48874E+00_JPRB ,&
&  0.50931E+00_JPRB, 0.52133E+00_JPRB, 0.52331E+00_JPRB, 0.53366E+00_JPRB, 0.54593E+00_JPRB, 0.56339E+00_JPRB, 0.57567E+00_JPRB ,&
&  0.59338E+00_JPRB, 0.61930E+00_JPRB, 0.64220E+00_JPRB, 0.66645E+00_JPRB, 0.68858E+00_JPRB, 0.71131E+00_JPRB, 0.74451E+00_JPRB ,&
&  0.78747E+00_JPRB, 0.84334E+00_JPRB, 0.90567E+00_JPRB, 0.96073E+00_JPRB, 0.10060E+01_JPRB, 0.10576E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.71773E+00_JPRB, 0.93000E+00_JPRB, 0.10432E+01_JPRB, 0.95350E+00_JPRB, 0.87550E+00_JPRB, 0.82893E+00_JPRB, 0.80406E+00_JPRB ,&
&  0.76599E+00_JPRB, 0.71904E+00_JPRB, 0.70449E+00_JPRB, 0.74313E+00_JPRB, 0.70663E+00_JPRB, 0.70907E+00_JPRB, 0.72395E+00_JPRB ,&
&  0.74869E+00_JPRB, 0.78088E+00_JPRB, 0.81699E+00_JPRB, 0.86370E+00_JPRB, 0.93027E+00_JPRB, 0.10118E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.12527E+01_JPRB, 0.14000E+01_JPRB, 0.15293E+01_JPRB, 0.12556E+01_JPRB, 0.11592E+01_JPRB, 0.11160E+01_JPRB, 0.10053E+01_JPRB ,&
&  0.74643E+00_JPRB, 0.62432E+00_JPRB, 0.62408E+00_JPRB, 0.61848E+00_JPRB, 0.56562E+00_JPRB, 0.51654E+00_JPRB, 0.46998E+00_JPRB ,&
&  0.44686E+00_JPRB, 0.42967E+00_JPRB, 0.42165E+00_JPRB, 0.41547E+00_JPRB, 0.41431E+00_JPRB, 0.42522E+00_JPRB, 0.43829E+00_JPRB ,&
&  0.45084E+00_JPRB, 0.45247E+00_JPRB, 0.46932E+00_JPRB, 0.48013E+00_JPRB, 0.51491E+00_JPRB, 0.54843E+00_JPRB, 0.56539E+00_JPRB ,&
&  0.57459E+00_JPRB, 0.57109E+00_JPRB, 0.56814E+00_JPRB, 0.56585E+00_JPRB, 0.56409E+00_JPRB, 0.56050E+00_JPRB, 0.55733E+00_JPRB ,&
&  0.55313E+00_JPRB, 0.56812E+00_JPRB, 0.56197E+00_JPRB, 0.55914E+00_JPRB, 0.56998E+00_JPRB, 0.55098E+00_JPRB, 0.53805E+00_JPRB ,&
&  0.51696E+00_JPRB, 0.47092E+00_JPRB, 0.43645E+00_JPRB, 0.41209E+00_JPRB, 0.39647E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.47376E+00_JPRB, 0.51407E+00_JPRB, 0.52997E+00_JPRB, 0.54924E+00_JPRB, 0.56141E+00_JPRB, 0.57818E+00_JPRB, 0.59110E+00_JPRB ,&
&  0.60265E+00_JPRB, 0.61704E+00_JPRB, 0.62107E+00_JPRB, 0.63823E+00_JPRB, 0.65165E+00_JPRB, 0.66337E+00_JPRB, 0.67126E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.70146E+00_JPRB, 0.72306E+00_JPRB, 0.74875E+00_JPRB, 0.77376E+00_JPRB, 0.80714E+00_JPRB, 0.85413E+00_JPRB ,&
&  0.91053E+00_JPRB, 0.96273E+00_JPRB, 0.10054E+01_JPRB, 0.10380E+01_JPRB, 0.10503E+01_JPRB, 0.10852E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.81743E+00_JPRB, 0.97119E+00_JPRB, 0.93941E+00_JPRB, 0.89409E+00_JPRB, 0.85491E+00_JPRB, 0.81545E+00_JPRB, 0.78904E+00_JPRB ,&
&  0.76316E+00_JPRB, 0.71464E+00_JPRB, 0.70407E+00_JPRB, 0.71394E+00_JPRB, 0.73009E+00_JPRB, 0.74305E+00_JPRB, 0.76976E+00_JPRB ,&
&  0.81121E+00_JPRB, 0.86327E+00_JPRB, 0.91953E+00_JPRB, 0.98049E+00_JPRB, 0.10661E+01_JPRB, 0.11755E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.13705E+01_JPRB, 0.14748E+01_JPRB, 0.15358E+01_JPRB, 0.13184E+01_JPRB, 0.11254E+01_JPRB, 0.10379E+01_JPRB, 0.85575E+00_JPRB ,&
&  0.69439E+00_JPRB, 0.62828E+00_JPRB, 0.65304E+00_JPRB, 0.65136E+00_JPRB, 0.57738E+00_JPRB, 0.50319E+00_JPRB, 0.44545E+00_JPRB ,&
&  0.42517E+00_JPRB, 0.39707E+00_JPRB, 0.38436E+00_JPRB, 0.38335E+00_JPRB, 0.38692E+00_JPRB, 0.39385E+00_JPRB, 0.41270E+00_JPRB ,&
&  0.42552E+00_JPRB, 0.43923E+00_JPRB, 0.46056E+00_JPRB, 0.48830E+00_JPRB, 0.53417E+00_JPRB, 0.59993E+00_JPRB, 0.60602E+00_JPRB ,&
&  0.58324E+00_JPRB, 0.55466E+00_JPRB, 0.52973E+00_JPRB, 0.53081E+00_JPRB, 0.53023E+00_JPRB, 0.53411E+00_JPRB, 0.55106E+00_JPRB ,&
&  0.57740E+00_JPRB, 0.59798E+00_JPRB, 0.56928E+00_JPRB, 0.55143E+00_JPRB, 0.55922E+00_JPRB, 0.53078E+00_JPRB, 0.47628E+00_JPRB ,&
&  0.45550E+00_JPRB, 0.43785E+00_JPRB, 0.42204E+00_JPRB, 0.42888E+00_JPRB, 0.44867E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.62067E+00_JPRB, 0.64454E+00_JPRB, 0.65993E+00_JPRB, 0.65866E+00_JPRB, 0.68116E+00_JPRB, 0.69426E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.70437E+00_JPRB, 0.71642E+00_JPRB, 0.72074E+00_JPRB, 0.73808E+00_JPRB, 0.75007E+00_JPRB, 0.76286E+00_JPRB, 0.77018E+00_JPRB ,&
&  0.77900E+00_JPRB, 0.79807E+00_JPRB, 0.82167E+00_JPRB, 0.85698E+00_JPRB, 0.89072E+00_JPRB, 0.93328E+00_JPRB, 0.96777E+00_JPRB ,&
&  0.98852E+00_JPRB, 0.10190E+01_JPRB, 0.10593E+01_JPRB, 0.10814E+01_JPRB, 0.10628E+01_JPRB, 0.10421E+01_JPRB, 0.79892E+00_JPRB ,&
&  0.90811E+00_JPRB, 0.92497E+00_JPRB, 0.91082E+00_JPRB, 0.88230E+00_JPRB, 0.84728E+00_JPRB, 0.80034E+00_JPRB, 0.76816E+00_JPRB ,&
&  0.74797E+00_JPRB, 0.72619E+00_JPRB, 0.70904E+00_JPRB, 0.69790E+00_JPRB, 0.75007E+00_JPRB, 0.76578E+00_JPRB, 0.79736E+00_JPRB ,&
&  0.84146E+00_JPRB, 0.89846E+00_JPRB, 0.96567E+00_JPRB, 0.10586E+01_JPRB, 0.11611E+01_JPRB, 0.12616E+01_JPRB, 0.13663E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.14812E+01_JPRB, 0.15051E+01_JPRB, 0.12385E+01_JPRB, 0.10860E+01_JPRB, 0.93067E+00_JPRB, 0.78803E+00_JPRB ,&
&  0.69803E+00_JPRB, 0.67159E+00_JPRB, 0.70535E+00_JPRB, 0.69551E+00_JPRB, 0.60443E+00_JPRB, 0.51307E+00_JPRB, 0.44756E+00_JPRB ,&
&  0.41929E+00_JPRB, 0.38342E+00_JPRB, 0.37624E+00_JPRB, 0.37480E+00_JPRB, 0.38216E+00_JPRB, 0.38935E+00_JPRB, 0.39935E+00_JPRB ,&
&  0.41355E+00_JPRB, 0.43873E+00_JPRB, 0.48642E+00_JPRB, 0.55598E+00_JPRB, 0.64292E+00_JPRB, 0.73546E+00_JPRB, 0.70567E+00_JPRB ,&
&  0.58972E+00_JPRB, 0.52740E+00_JPRB, 0.53679E+00_JPRB, 0.55152E+00_JPRB, 0.58773E+00_JPRB, 0.64767E+00_JPRB, 0.72208E+00_JPRB ,&
&  0.84769E+00_JPRB, 0.92703E+00_JPRB, 0.84411E+00_JPRB, 0.70816E+00_JPRB, 0.98568E+00_JPRB, 0.96547E+00_JPRB, 0.49971E+00_JPRB ,&
&  0.54389E+00_JPRB, 0.53469E+00_JPRB, 0.55542E+00_JPRB, 0.57978E+00_JPRB, 0.61000E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.74835E+00_JPRB, 0.76973E+00_JPRB, 0.77787E+00_JPRB, 0.77608E+00_JPRB, 0.78790E+00_JPRB, 0.80988E+00_JPRB, 0.81533E+00_JPRB ,&
&  0.82125E+00_JPRB, 0.81479E+00_JPRB, 0.82280E+00_JPRB, 0.83258E+00_JPRB, 0.84297E+00_JPRB, 0.85470E+00_JPRB, 0.86619E+00_JPRB ,&
&  0.87172E+00_JPRB, 0.89923E+00_JPRB, 0.91110E+00_JPRB, 0.94142E+00_JPRB, 0.97279E+00_JPRB, 0.10176E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10543E+01_JPRB, 0.10793E+01_JPRB, 0.11127E+01_JPRB, 0.10977E+01_JPRB, 0.10937E+01_JPRB, 0.86774E+00_JPRB ,&
&  0.95366E+00_JPRB, 0.92743E+00_JPRB, 0.87706E+00_JPRB, 0.84998E+00_JPRB, 0.81732E+00_JPRB, 0.78547E+00_JPRB, 0.75729E+00_JPRB ,&
&  0.74748E+00_JPRB, 0.73491E+00_JPRB, 0.71261E+00_JPRB, 0.73921E+00_JPRB, 0.75304E+00_JPRB, 0.77015E+00_JPRB, 0.81263E+00_JPRB ,&
&  0.85541E+00_JPRB, 0.92270E+00_JPRB, 0.10040E+01_JPRB, 0.11042E+01_JPRB, 0.12104E+01_JPRB, 0.13132E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.15931E+01_JPRB, 0.16090E+01_JPRB, 0.13652E+01_JPRB, 0.12268E+01_JPRB, 0.98194E+00_JPRB, 0.82102E+00_JPRB ,&
&  0.71417E+00_JPRB, 0.67140E+00_JPRB, 0.69017E+00_JPRB, 0.73282E+00_JPRB, 0.62327E+00_JPRB, 0.54411E+00_JPRB, 0.46991E+00_JPRB ,&
&  0.43906E+00_JPRB, 0.40553E+00_JPRB, 0.39745E+00_JPRB, 0.38947E+00_JPRB, 0.38383E+00_JPRB, 0.38706E+00_JPRB, 0.39551E+00_JPRB ,&
&  0.39988E+00_JPRB, 0.41858E+00_JPRB, 0.46030E+00_JPRB, 0.50740E+00_JPRB, 0.54657E+00_JPRB, 0.56381E+00_JPRB, 0.53307E+00_JPRB ,&
&  0.56211E+00_JPRB, 0.55615E+00_JPRB, 0.56117E+00_JPRB, 0.61800E+00_JPRB, 0.72007E+00_JPRB, 0.88605E+00_JPRB, 0.10942E+01_JPRB ,&
&  0.13488E+01_JPRB, 0.14152E+01_JPRB, 0.17230E+01_JPRB, 0.22773E+01_JPRB, 0.20550E+01_JPRB, 0.17337E+01_JPRB, 0.61928E+00_JPRB ,&
&  0.64909E+00_JPRB, 0.65530E+00_JPRB, 0.68121E+00_JPRB, 0.70605E+00_JPRB, 0.73724E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.75259E+00_JPRB, 0.77072E+00_JPRB, 0.78397E+00_JPRB, 0.79261E+00_JPRB, 0.80701E+00_JPRB, 0.82124E+00_JPRB, 0.83459E+00_JPRB ,&
&  0.84622E+00_JPRB, 0.85412E+00_JPRB, 0.86643E+00_JPRB, 0.87759E+00_JPRB, 0.89122E+00_JPRB, 0.90722E+00_JPRB, 0.92644E+00_JPRB ,&
&  0.94027E+00_JPRB, 0.95887E+00_JPRB, 0.96654E+00_JPRB, 0.99621E+00_JPRB, 0.10358E+01_JPRB, 0.10524E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.10640E+01_JPRB, 0.10762E+01_JPRB, 0.10959E+01_JPRB, 0.10984E+01_JPRB, 0.11285E+01_JPRB, 0.94807E+00_JPRB ,&
&  0.99960E+00_JPRB, 0.94161E+00_JPRB, 0.83633E+00_JPRB, 0.80364E+00_JPRB, 0.77677E+00_JPRB, 0.76549E+00_JPRB, 0.75524E+00_JPRB ,&
&  0.74644E+00_JPRB, 0.74475E+00_JPRB, 0.73423E+00_JPRB, 0.72646E+00_JPRB, 0.73676E+00_JPRB, 0.75646E+00_JPRB, 0.79131E+00_JPRB ,&
&  0.84559E+00_JPRB, 0.92132E+00_JPRB, 0.10095E+01_JPRB, 0.11086E+01_JPRB, 0.12207E+01_JPRB, 0.13149E+01_JPRB, 0.14026E+01_JPRB ,&
&  0.14720E+01_JPRB, 0.15204E+01_JPRB, 0.14173E+01_JPRB, 0.14089E+01_JPRB, 0.14034E+01_JPRB, 0.11047E+01_JPRB, 0.87153E+00_JPRB ,&
&  0.65697E+00_JPRB, 0.65459E+00_JPRB, 0.69657E+00_JPRB, 0.77209E+00_JPRB, 0.66777E+00_JPRB, 0.60059E+00_JPRB, 0.52092E+00_JPRB ,&
&  0.45584E+00_JPRB, 0.42359E+00_JPRB, 0.41733E+00_JPRB, 0.44649E+00_JPRB, 0.42300E+00_JPRB, 0.40626E+00_JPRB, 0.41248E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.40504E+00_JPRB, 0.41771E+00_JPRB, 0.44251E+00_JPRB, 0.46426E+00_JPRB, 0.46001E+00_JPRB, 0.51929E+00_JPRB ,&
&  0.51478E+00_JPRB, 0.51149E+00_JPRB, 0.52521E+00_JPRB, 0.47224E+00_JPRB, 0.65312E+00_JPRB, 0.74818E+00_JPRB, 0.88133E+00_JPRB ,&
&  0.10216E+01_JPRB, 0.14178E+01_JPRB, 0.16708E+01_JPRB, 0.12962E+01_JPRB, 0.98002E+00_JPRB, 0.68221E+00_JPRB, 0.64568E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.66304E+00_JPRB, 0.68070E+00_JPRB, 0.70368E+00_JPRB, 0.72893E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.74827E+00_JPRB, 0.76837E+00_JPRB, 0.78464E+00_JPRB, 0.79756E+00_JPRB, 0.81400E+00_JPRB, 0.83085E+00_JPRB, 0.84500E+00_JPRB ,&
&  0.85783E+00_JPRB, 0.86944E+00_JPRB, 0.88417E+00_JPRB, 0.89757E+00_JPRB, 0.91355E+00_JPRB, 0.93290E+00_JPRB, 0.95187E+00_JPRB ,&
&  0.96990E+00_JPRB, 0.97921E+00_JPRB, 0.10052E+01_JPRB, 0.10364E+01_JPRB, 0.10556E+01_JPRB, 0.10563E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.10668E+01_JPRB, 0.10821E+01_JPRB, 0.10964E+01_JPRB, 0.11194E+01_JPRB, 0.11643E+01_JPRB, 0.95556E+00_JPRB ,&
&  0.10388E+01_JPRB, 0.95112E+00_JPRB, 0.79943E+00_JPRB, 0.74040E+00_JPRB, 0.71813E+00_JPRB, 0.70577E+00_JPRB, 0.70226E+00_JPRB ,&
&  0.71424E+00_JPRB, 0.69472E+00_JPRB, 0.68910E+00_JPRB, 0.68703E+00_JPRB, 0.70211E+00_JPRB, 0.72193E+00_JPRB, 0.74655E+00_JPRB ,&
&  0.79076E+00_JPRB, 0.85482E+00_JPRB, 0.93262E+00_JPRB, 0.10218E+01_JPRB, 0.11289E+01_JPRB, 0.12120E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.12478E+01_JPRB, 0.10908E+01_JPRB, 0.11668E+01_JPRB, 0.12169E+01_JPRB, 0.11011E+01_JPRB, 0.95018E+00_JPRB ,&
&  0.77225E+00_JPRB, 0.66803E+00_JPRB, 0.70357E+00_JPRB, 0.81732E+00_JPRB, 0.77663E+00_JPRB, 0.68131E+00_JPRB, 0.60696E+00_JPRB ,&
&  0.50590E+00_JPRB, 0.47565E+00_JPRB, 0.46546E+00_JPRB, 0.51239E+00_JPRB, 0.50120E+00_JPRB, 0.45650E+00_JPRB, 0.44370E+00_JPRB ,&
&  0.41039E+00_JPRB, 0.37763E+00_JPRB, 0.36413E+00_JPRB, 0.38874E+00_JPRB, 0.43770E+00_JPRB, 0.42141E+00_JPRB, 0.48176E+00_JPRB ,&
&  0.45385E+00_JPRB, 0.40278E+00_JPRB, 0.43805E+00_JPRB, 0.49486E+00_JPRB, 0.53713E+00_JPRB, 0.58433E+00_JPRB, 0.69007E+00_JPRB ,&
&  0.85419E+00_JPRB, 0.10585E+01_JPRB, 0.10276E+01_JPRB, 0.81511E+00_JPRB, 0.65151E+00_JPRB, 0.61792E+00_JPRB, 0.63118E+00_JPRB ,&
&  0.65041E+00_JPRB, 0.66540E+00_JPRB, 0.68348E+00_JPRB, 0.70345E+00_JPRB, 0.72479E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.73376E+00_JPRB, 0.75343E+00_JPRB, 0.77068E+00_JPRB, 0.78661E+00_JPRB, 0.80296E+00_JPRB, 0.81844E+00_JPRB, 0.83141E+00_JPRB ,&
&  0.84312E+00_JPRB, 0.85752E+00_JPRB, 0.87015E+00_JPRB, 0.88314E+00_JPRB, 0.90238E+00_JPRB, 0.91957E+00_JPRB, 0.92966E+00_JPRB ,&
&  0.94711E+00_JPRB, 0.96952E+00_JPRB, 0.99712E+00_JPRB, 0.10198E+01_JPRB, 0.10292E+01_JPRB, 0.10283E+01_JPRB, 0.10244E+01_JPRB ,&
&  0.10115E+01_JPRB, 0.10055E+01_JPRB, 0.10110E+01_JPRB, 0.10244E+01_JPRB, 0.10518E+01_JPRB, 0.10348E+01_JPRB, 0.11042E+01_JPRB ,&
&  0.84076E+00_JPRB, 0.97424E+00_JPRB, 0.79079E+00_JPRB, 0.67925E+00_JPRB, 0.67396E+00_JPRB, 0.63281E+00_JPRB, 0.63078E+00_JPRB ,&
&  0.65117E+00_JPRB, 0.65574E+00_JPRB, 0.65686E+00_JPRB, 0.65691E+00_JPRB, 0.67021E+00_JPRB, 0.69585E+00_JPRB, 0.70307E+00_JPRB ,&
&  0.73080E+00_JPRB, 0.76583E+00_JPRB, 0.78653E+00_JPRB, 0.83407E+00_JPRB, 0.92520E+00_JPRB, 0.10042E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.98480E+00_JPRB, 0.76895E+00_JPRB, 0.82738E+00_JPRB, 0.91635E+00_JPRB, 0.90142E+00_JPRB, 0.83511E+00_JPRB ,&
&  0.81531E+00_JPRB, 0.74299E+00_JPRB, 0.86829E+00_JPRB, 0.93615E+00_JPRB, 0.92465E+00_JPRB, 0.85232E+00_JPRB, 0.74361E+00_JPRB ,&
&  0.64280E+00_JPRB, 0.56501E+00_JPRB, 0.57704E+00_JPRB, 0.63560E+00_JPRB, 0.62453E+00_JPRB, 0.53813E+00_JPRB, 0.49453E+00_JPRB ,&
&  0.43720E+00_JPRB, 0.37933E+00_JPRB, 0.34074E+00_JPRB, 0.34005E+00_JPRB, 0.32958E+00_JPRB, 0.38666E+00_JPRB, 0.42212E+00_JPRB ,&
&  0.41698E+00_JPRB, 0.36880E+00_JPRB, 0.36901E+00_JPRB, 0.43535E+00_JPRB, 0.44265E+00_JPRB, 0.43193E+00_JPRB, 0.48573E+00_JPRB ,&
&  0.59739E+00_JPRB, 0.69201E+00_JPRB, 0.69748E+00_JPRB, 0.63165E+00_JPRB, 0.60685E+00_JPRB, 0.61217E+00_JPRB, 0.62937E+00_JPRB ,&
&  0.64731E+00_JPRB, 0.66377E+00_JPRB, 0.67979E+00_JPRB, 0.69701E+00_JPRB, 0.71438E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.70298E+00_JPRB, 0.72804E+00_JPRB, 0.74459E+00_JPRB, 0.75773E+00_JPRB, 0.76295E+00_JPRB, 0.76910E+00_JPRB, 0.78427E+00_JPRB ,&
&  0.79601E+00_JPRB, 0.80861E+00_JPRB, 0.82237E+00_JPRB, 0.84293E+00_JPRB, 0.86993E+00_JPRB, 0.89048E+00_JPRB, 0.89818E+00_JPRB ,&
&  0.90324E+00_JPRB, 0.93361E+00_JPRB, 0.95305E+00_JPRB, 0.99034E+00_JPRB, 0.10040E+01_JPRB, 0.10282E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.10362E+01_JPRB, 0.10456E+01_JPRB, 0.10680E+01_JPRB, 0.10891E+01_JPRB, 0.11992E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.10862E+01_JPRB, 0.95955E+00_JPRB, 0.78061E+00_JPRB, 0.62483E+00_JPRB, 0.61527E+00_JPRB, 0.61786E+00_JPRB, 0.60391E+00_JPRB ,&
&  0.61177E+00_JPRB, 0.62902E+00_JPRB, 0.64041E+00_JPRB, 0.63091E+00_JPRB, 0.64797E+00_JPRB, 0.68819E+00_JPRB, 0.68424E+00_JPRB ,&
&  0.69151E+00_JPRB, 0.70094E+00_JPRB, 0.69497E+00_JPRB, 0.75763E+00_JPRB, 0.72895E+00_JPRB, 0.79954E+00_JPRB, 0.84893E+00_JPRB ,&
&  0.87219E+00_JPRB, 0.82213E+00_JPRB, 0.63542E+00_JPRB, 0.64675E+00_JPRB, 0.73800E+00_JPRB, 0.75609E+00_JPRB, 0.73978E+00_JPRB ,&
&  0.82229E+00_JPRB, 0.84413E+00_JPRB, 0.76623E+00_JPRB, 0.10055E+01_JPRB, 0.10997E+01_JPRB, 0.11023E+01_JPRB, 0.10032E+01_JPRB ,&
&  0.90243E+00_JPRB, 0.75565E+00_JPRB, 0.75665E+00_JPRB, 0.76216E+00_JPRB, 0.74160E+00_JPRB, 0.68594E+00_JPRB, 0.59135E+00_JPRB ,&
&  0.50388E+00_JPRB, 0.42921E+00_JPRB, 0.37234E+00_JPRB, 0.34906E+00_JPRB, 0.38446E+00_JPRB, 0.31400E+00_JPRB, 0.37580E+00_JPRB ,&
&  0.35643E+00_JPRB, 0.36335E+00_JPRB, 0.32935E+00_JPRB, 0.38718E+00_JPRB, 0.40657E+00_JPRB, 0.37944E+00_JPRB, 0.40642E+00_JPRB ,&
&  0.46124E+00_JPRB, 0.50553E+00_JPRB, 0.53607E+00_JPRB, 0.55754E+00_JPRB, 0.57610E+00_JPRB, 0.59793E+00_JPRB, 0.60732E+00_JPRB ,&
&  0.62200E+00_JPRB, 0.63816E+00_JPRB, 0.65378E+00_JPRB, 0.67633E+00_JPRB, 0.68653E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.60021E+00_JPRB, 0.61016E+00_JPRB, 0.62520E+00_JPRB, 0.63391E+00_JPRB, 0.64280E+00_JPRB, 0.64870E+00_JPRB, 0.67278E+00_JPRB ,&
&  0.68803E+00_JPRB, 0.70264E+00_JPRB, 0.72486E+00_JPRB, 0.74757E+00_JPRB, 0.75716E+00_JPRB, 0.76771E+00_JPRB, 0.77120E+00_JPRB ,&
&  0.78562E+00_JPRB, 0.80114E+00_JPRB, 0.79997E+00_JPRB, 0.80599E+00_JPRB, 0.81237E+00_JPRB, 0.87261E+00_JPRB, 0.87374E+00_JPRB ,&
&  0.86486E+00_JPRB, 0.87511E+00_JPRB, 0.88830E+00_JPRB, 0.93519E+00_JPRB, 0.10713E+01_JPRB, 0.11682E+01_JPRB, 0.11626E+01_JPRB ,&
&  0.11342E+01_JPRB, 0.10300E+01_JPRB, 0.10382E+01_JPRB, 0.81218E+00_JPRB, 0.75929E+00_JPRB, 0.75124E+00_JPRB, 0.70139E+00_JPRB ,&
&  0.69018E+00_JPRB, 0.69750E+00_JPRB, 0.72536E+00_JPRB, 0.70906E+00_JPRB, 0.71458E+00_JPRB, 0.70764E+00_JPRB, 0.68236E+00_JPRB ,&
&  0.68076E+00_JPRB, 0.68856E+00_JPRB, 0.69538E+00_JPRB, 0.74288E+00_JPRB, 0.75169E+00_JPRB, 0.80886E+00_JPRB, 0.82426E+00_JPRB ,&
&  0.83334E+00_JPRB, 0.86101E+00_JPRB, 0.81166E+00_JPRB, 0.82725E+00_JPRB, 0.92561E+00_JPRB, 0.93604E+00_JPRB, 0.90295E+00_JPRB ,&
&  0.90646E+00_JPRB, 0.83344E+00_JPRB, 0.63498E+00_JPRB, 0.99163E+00_JPRB, 0.11785E+01_JPRB, 0.11576E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.10085E+01_JPRB, 0.94128E+00_JPRB, 0.91787E+00_JPRB, 0.84658E+00_JPRB, 0.76976E+00_JPRB, 0.71229E+00_JPRB ,&
&  0.60268E+00_JPRB, 0.48131E+00_JPRB, 0.39077E+00_JPRB, 0.35267E+00_JPRB, 0.33690E+00_JPRB, 0.34065E+00_JPRB, 0.33752E+00_JPRB ,&
&  0.32884E+00_JPRB, 0.32151E+00_JPRB, 0.36554E+00_JPRB, 0.39285E+00_JPRB, 0.36001E+00_JPRB, 0.37085E+00_JPRB, 0.36674E+00_JPRB ,&
&  0.38593E+00_JPRB, 0.41147E+00_JPRB, 0.43595E+00_JPRB, 0.45475E+00_JPRB, 0.47770E+00_JPRB, 0.49781E+00_JPRB, 0.51382E+00_JPRB ,&
&  0.52943E+00_JPRB, 0.54864E+00_JPRB, 0.56503E+00_JPRB, 0.58320E+00_JPRB, 0.59788E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.59870E+00_JPRB, 0.61193E+00_JPRB, 0.61009E+00_JPRB, 0.61335E+00_JPRB, 0.60647E+00_JPRB, 0.60536E+00_JPRB, 0.59012E+00_JPRB ,&
&  0.57644E+00_JPRB, 0.58352E+00_JPRB, 0.61062E+00_JPRB, 0.66136E+00_JPRB, 0.67077E+00_JPRB, 0.68514E+00_JPRB, 0.68390E+00_JPRB ,&
&  0.70250E+00_JPRB, 0.69352E+00_JPRB, 0.69168E+00_JPRB, 0.67721E+00_JPRB, 0.68833E+00_JPRB, 0.72652E+00_JPRB, 0.80678E+00_JPRB ,&
&  0.91963E+00_JPRB, 0.10559E+01_JPRB, 0.12550E+01_JPRB, 0.14800E+01_JPRB, 0.19468E+01_JPRB, 0.16406E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.14987E+01_JPRB, 0.13473E+01_JPRB, 0.12402E+01_JPRB, 0.11656E+01_JPRB, 0.11032E+01_JPRB, 0.10508E+01_JPRB, 0.10774E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.11252E+01_JPRB, 0.11604E+01_JPRB, 0.11664E+01_JPRB, 0.11695E+01_JPRB, 0.11406E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.10064E+01_JPRB, 0.10270E+01_JPRB, 0.10480E+01_JPRB, 0.11051E+01_JPRB, 0.12350E+01_JPRB, 0.12806E+01_JPRB, 0.13053E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.13826E+01_JPRB, 0.15493E+01_JPRB, 0.15873E+01_JPRB, 0.15976E+01_JPRB, 0.14851E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.12310E+01_JPRB, 0.98235E+00_JPRB, 0.79124E+00_JPRB, 0.12784E+01_JPRB, 0.14188E+01_JPRB, 0.12076E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.11711E+01_JPRB, 0.11405E+01_JPRB, 0.10473E+01_JPRB, 0.90484E+00_JPRB, 0.82789E+00_JPRB, 0.72522E+00_JPRB ,&
&  0.67996E+00_JPRB, 0.54943E+00_JPRB, 0.43675E+00_JPRB, 0.37422E+00_JPRB, 0.34601E+00_JPRB, 0.35134E+00_JPRB, 0.34176E+00_JPRB ,&
&  0.34113E+00_JPRB, 0.31589E+00_JPRB, 0.34634E+00_JPRB, 0.42750E+00_JPRB, 0.52249E+00_JPRB, 0.39810E+00_JPRB, 0.36305E+00_JPRB ,&
&  0.38535E+00_JPRB, 0.40598E+00_JPRB, 0.42680E+00_JPRB, 0.43941E+00_JPRB, 0.44528E+00_JPRB, 0.45313E+00_JPRB, 0.48329E+00_JPRB ,&
&  0.50805E+00_JPRB, 0.53095E+00_JPRB, 0.55036E+00_JPRB, 0.57674E+00_JPRB, 0.59130E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.77741E+00_JPRB, 0.80280E+00_JPRB, 0.82004E+00_JPRB, 0.82455E+00_JPRB, 0.81740E+00_JPRB, 0.80818E+00_JPRB, 0.78662E+00_JPRB ,&
&  0.76119E+00_JPRB, 0.77135E+00_JPRB, 0.80975E+00_JPRB, 0.89728E+00_JPRB, 0.95775E+00_JPRB, 0.99387E+00_JPRB, 0.10455E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11180E+01_JPRB, 0.11135E+01_JPRB, 0.11392E+01_JPRB, 0.11663E+01_JPRB, 0.12027E+01_JPRB, 0.13786E+01_JPRB ,&
&  0.16300E+01_JPRB, 0.19350E+01_JPRB, 0.22393E+01_JPRB, 0.24605E+01_JPRB, 0.20455E+01_JPRB, 0.13745E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.16260E+01_JPRB, 0.14452E+01_JPRB, 0.14530E+01_JPRB, 0.14850E+01_JPRB, 0.15664E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.17380E+01_JPRB, 0.17835E+01_JPRB, 0.18277E+01_JPRB, 0.18574E+01_JPRB, 0.18966E+01_JPRB, 0.19308E+01_JPRB, 0.19615E+01_JPRB ,&
&  0.19516E+01_JPRB, 0.19094E+01_JPRB, 0.19022E+01_JPRB, 0.19720E+01_JPRB, 0.19872E+01_JPRB, 0.19965E+01_JPRB, 0.19899E+01_JPRB ,&
&  0.19837E+01_JPRB, 0.20122E+01_JPRB, 0.22088E+01_JPRB, 0.23077E+01_JPRB, 0.21571E+01_JPRB, 0.19730E+01_JPRB, 0.19042E+01_JPRB ,&
&  0.17951E+01_JPRB, 0.16008E+01_JPRB, 0.12668E+01_JPRB, 0.18641E+01_JPRB, 0.19047E+01_JPRB, 0.16755E+01_JPRB, 0.14766E+01_JPRB ,&
&  0.13661E+01_JPRB, 0.13114E+01_JPRB, 0.12208E+01_JPRB, 0.11288E+01_JPRB, 0.96331E+00_JPRB, 0.89611E+00_JPRB, 0.69755E+00_JPRB ,&
&  0.70999E+00_JPRB, 0.60826E+00_JPRB, 0.48896E+00_JPRB, 0.41188E+00_JPRB, 0.38830E+00_JPRB, 0.38652E+00_JPRB, 0.38152E+00_JPRB ,&
&  0.34393E+00_JPRB, 0.36280E+00_JPRB, 0.45424E+00_JPRB, 0.82333E+00_JPRB, 0.18924E+01_JPRB, 0.49727E+00_JPRB, 0.39620E+00_JPRB ,&
&  0.42485E+00_JPRB, 0.46105E+00_JPRB, 0.49589E+00_JPRB, 0.51628E+00_JPRB, 0.52726E+00_JPRB, 0.54902E+00_JPRB, 0.58937E+00_JPRB ,&
&  0.63515E+00_JPRB, 0.66821E+00_JPRB, 0.68434E+00_JPRB, 0.72198E+00_JPRB, 0.75181E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.92369E+00_JPRB, 0.95434E+00_JPRB, 0.98181E+00_JPRB, 0.10207E+01_JPRB, 0.10582E+01_JPRB, 0.10709E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.10871E+01_JPRB, 0.11255E+01_JPRB, 0.11662E+01_JPRB, 0.12017E+01_JPRB, 0.12516E+01_JPRB, 0.12948E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.14557E+01_JPRB, 0.15568E+01_JPRB, 0.16634E+01_JPRB, 0.17456E+01_JPRB, 0.18742E+01_JPRB, 0.20142E+01_JPRB ,&
&  0.21532E+01_JPRB, 0.19980E+01_JPRB, 0.19212E+01_JPRB, 0.15217E+01_JPRB, 0.10796E+01_JPRB, 0.13727E+01_JPRB, 0.14731E+01_JPRB ,&
&  0.15590E+01_JPRB, 0.16132E+01_JPRB, 0.16676E+01_JPRB, 0.17570E+01_JPRB, 0.18966E+01_JPRB, 0.20076E+01_JPRB, 0.20718E+01_JPRB ,&
&  0.21337E+01_JPRB, 0.21889E+01_JPRB, 0.22425E+01_JPRB, 0.23080E+01_JPRB, 0.23916E+01_JPRB, 0.24693E+01_JPRB, 0.25174E+01_JPRB ,&
&  0.25824E+01_JPRB, 0.26033E+01_JPRB, 0.26316E+01_JPRB, 0.26676E+01_JPRB, 0.25609E+01_JPRB, 0.25026E+01_JPRB, 0.24629E+01_JPRB ,&
&  0.24547E+01_JPRB, 0.24733E+01_JPRB, 0.25814E+01_JPRB, 0.26884E+01_JPRB, 0.26773E+01_JPRB, 0.24447E+01_JPRB, 0.23199E+01_JPRB ,&
&  0.22220E+01_JPRB, 0.20093E+01_JPRB, 0.19523E+01_JPRB, 0.24371E+01_JPRB, 0.15058E+01_JPRB, 0.16090E+01_JPRB, 0.16482E+01_JPRB ,&
&  0.15169E+01_JPRB, 0.14970E+01_JPRB, 0.13778E+01_JPRB, 0.11735E+01_JPRB, 0.97862E+00_JPRB, 0.68122E+00_JPRB, 0.69482E+00_JPRB ,&
&  0.76579E+00_JPRB, 0.68523E+00_JPRB, 0.53921E+00_JPRB, 0.44006E+00_JPRB, 0.43600E+00_JPRB, 0.42443E+00_JPRB, 0.42252E+00_JPRB ,&
&  0.38882E+00_JPRB, 0.40605E+00_JPRB, 0.46341E+00_JPRB, 0.66486E+00_JPRB, 0.93124E+00_JPRB, 0.47876E+00_JPRB, 0.43072E+00_JPRB ,&
&  0.47058E+00_JPRB, 0.53604E+00_JPRB, 0.60825E+00_JPRB, 0.66037E+00_JPRB, 0.68233E+00_JPRB, 0.70449E+00_JPRB, 0.72823E+00_JPRB ,&
&  0.75476E+00_JPRB, 0.79755E+00_JPRB, 0.85484E+00_JPRB, 0.88289E+00_JPRB, 0.90537E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.94674E+00_JPRB, 0.10009E+01_JPRB, 0.10833E+01_JPRB, 0.11481E+01_JPRB, 0.11875E+01_JPRB, 0.11965E+01_JPRB, 0.11792E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.12249E+01_JPRB, 0.12971E+01_JPRB, 0.13595E+01_JPRB, 0.14264E+01_JPRB, 0.14915E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.16212E+01_JPRB, 0.16784E+01_JPRB, 0.17572E+01_JPRB, 0.18503E+01_JPRB, 0.19351E+01_JPRB, 0.20779E+01_JPRB, 0.20694E+01_JPRB ,&
&  0.16724E+01_JPRB, 0.12749E+01_JPRB, 0.15566E+01_JPRB, 0.15874E+01_JPRB, 0.14383E+01_JPRB, 0.15389E+01_JPRB, 0.16532E+01_JPRB ,&
&  0.17761E+01_JPRB, 0.18593E+01_JPRB, 0.19467E+01_JPRB, 0.20127E+01_JPRB, 0.20593E+01_JPRB, 0.21386E+01_JPRB, 0.22032E+01_JPRB ,&
&  0.22875E+01_JPRB, 0.23846E+01_JPRB, 0.24826E+01_JPRB, 0.26046E+01_JPRB, 0.27269E+01_JPRB, 0.28655E+01_JPRB, 0.29725E+01_JPRB ,&
&  0.30653E+01_JPRB, 0.31174E+01_JPRB, 0.30846E+01_JPRB, 0.30022E+01_JPRB, 0.28934E+01_JPRB, 0.28271E+01_JPRB, 0.27773E+01_JPRB ,&
&  0.27924E+01_JPRB, 0.28468E+01_JPRB, 0.30103E+01_JPRB, 0.31166E+01_JPRB, 0.31352E+01_JPRB, 0.28535E+01_JPRB, 0.27526E+01_JPRB ,&
&  0.26378E+01_JPRB, 0.24191E+01_JPRB, 0.24742E+01_JPRB, 0.26091E+01_JPRB, 0.18377E+01_JPRB, 0.17456E+01_JPRB, 0.15165E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16056E+01_JPRB, 0.14917E+01_JPRB, 0.13276E+01_JPRB, 0.10291E+01_JPRB, 0.74010E+00_JPRB, 0.78675E+00_JPRB ,&
&  0.88875E+00_JPRB, 0.86775E+00_JPRB, 0.64501E+00_JPRB, 0.51333E+00_JPRB, 0.43522E+00_JPRB, 0.43044E+00_JPRB, 0.43366E+00_JPRB ,&
&  0.42010E+00_JPRB, 0.41544E+00_JPRB, 0.46429E+00_JPRB, 0.55626E+00_JPRB, 0.60663E+00_JPRB, 0.48710E+00_JPRB, 0.47995E+00_JPRB ,&
&  0.53582E+00_JPRB, 0.59000E+00_JPRB, 0.63691E+00_JPRB, 0.67425E+00_JPRB, 0.71217E+00_JPRB, 0.75149E+00_JPRB, 0.77847E+00_JPRB ,&
&  0.82921E+00_JPRB, 0.82416E+00_JPRB, 0.86608E+00_JPRB, 0.88906E+00_JPRB, 0.92141E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.93038E+00_JPRB, 0.99639E+00_JPRB, 0.10771E+01_JPRB, 0.11526E+01_JPRB, 0.11909E+01_JPRB, 0.12232E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.12034E+01_JPRB, 0.12898E+01_JPRB, 0.13735E+01_JPRB, 0.14795E+01_JPRB, 0.15770E+01_JPRB, 0.16705E+01_JPRB ,&
&  0.17548E+01_JPRB, 0.18474E+01_JPRB, 0.19535E+01_JPRB, 0.20826E+01_JPRB, 0.21626E+01_JPRB, 0.22554E+01_JPRB, 0.19853E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.14469E+01_JPRB, 0.19373E+01_JPRB, 0.18916E+01_JPRB, 0.17961E+01_JPRB, 0.19093E+01_JPRB, 0.20128E+01_JPRB ,&
&  0.20693E+01_JPRB, 0.20988E+01_JPRB, 0.20963E+01_JPRB, 0.21100E+01_JPRB, 0.21014E+01_JPRB, 0.21241E+01_JPRB, 0.21518E+01_JPRB ,&
&  0.22235E+01_JPRB, 0.23448E+01_JPRB, 0.24886E+01_JPRB, 0.26774E+01_JPRB, 0.28745E+01_JPRB, 0.30701E+01_JPRB, 0.31732E+01_JPRB ,&
&  0.32242E+01_JPRB, 0.32882E+01_JPRB, 0.32883E+01_JPRB, 0.30853E+01_JPRB, 0.30323E+01_JPRB, 0.30786E+01_JPRB, 0.31324E+01_JPRB ,&
&  0.31581E+01_JPRB, 0.33289E+01_JPRB, 0.33892E+01_JPRB, 0.32354E+01_JPRB, 0.29300E+01_JPRB, 0.30572E+01_JPRB, 0.28120E+01_JPRB ,&
&  0.28787E+01_JPRB, 0.27661E+01_JPRB, 0.29928E+01_JPRB, 0.24007E+01_JPRB, 0.21992E+01_JPRB, 0.20941E+01_JPRB, 0.19120E+01_JPRB ,&
&  0.17533E+01_JPRB, 0.17349E+01_JPRB, 0.16284E+01_JPRB, 0.12705E+01_JPRB, 0.11628E+01_JPRB, 0.88836E+00_JPRB, 0.99821E+00_JPRB ,&
&  0.10293E+01_JPRB, 0.11153E+01_JPRB, 0.74627E+00_JPRB, 0.53883E+00_JPRB, 0.43785E+00_JPRB, 0.43216E+00_JPRB, 0.43386E+00_JPRB ,&
&  0.42919E+00_JPRB, 0.44263E+00_JPRB, 0.46379E+00_JPRB, 0.48711E+00_JPRB, 0.49196E+00_JPRB, 0.47817E+00_JPRB, 0.49265E+00_JPRB ,&
&  0.55176E+00_JPRB, 0.60171E+00_JPRB, 0.61637E+00_JPRB, 0.63154E+00_JPRB, 0.65886E+00_JPRB, 0.70950E+00_JPRB, 0.74563E+00_JPRB ,&
&  0.78719E+00_JPRB, 0.81169E+00_JPRB, 0.84034E+00_JPRB, 0.85670E+00_JPRB, 0.87869E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.76324E+00_JPRB, 0.79902E+00_JPRB, 0.82815E+00_JPRB, 0.83143E+00_JPRB, 0.84053E+00_JPRB, 0.86176E+00_JPRB, 0.90854E+00_JPRB ,&
&  0.96631E+00_JPRB, 0.10358E+01_JPRB, 0.11153E+01_JPRB, 0.12128E+01_JPRB, 0.13389E+01_JPRB, 0.14527E+01_JPRB, 0.15876E+01_JPRB ,&
&  0.17258E+01_JPRB, 0.18647E+01_JPRB, 0.20096E+01_JPRB, 0.21618E+01_JPRB, 0.22407E+01_JPRB, 0.22513E+01_JPRB, 0.18032E+01_JPRB ,&
&  0.12445E+01_JPRB, 0.21229E+01_JPRB, 0.20013E+01_JPRB, 0.21339E+01_JPRB, 0.21810E+01_JPRB, 0.22870E+01_JPRB, 0.22711E+01_JPRB ,&
&  0.22703E+01_JPRB, 0.22066E+01_JPRB, 0.21470E+01_JPRB, 0.21245E+01_JPRB, 0.21227E+01_JPRB, 0.21278E+01_JPRB, 0.21080E+01_JPRB ,&
&  0.21113E+01_JPRB, 0.21757E+01_JPRB, 0.23056E+01_JPRB, 0.24960E+01_JPRB, 0.27320E+01_JPRB, 0.30013E+01_JPRB, 0.31856E+01_JPRB ,&
&  0.33320E+01_JPRB, 0.34763E+01_JPRB, 0.35107E+01_JPRB, 0.33372E+01_JPRB, 0.32448E+01_JPRB, 0.33558E+01_JPRB, 0.36125E+01_JPRB ,&
&  0.35696E+01_JPRB, 0.34499E+01_JPRB, 0.36004E+01_JPRB, 0.35062E+01_JPRB, 0.32354E+01_JPRB, 0.32496E+01_JPRB, 0.28947E+01_JPRB ,&
&  0.31116E+01_JPRB, 0.29508E+01_JPRB, 0.31744E+01_JPRB, 0.26093E+01_JPRB, 0.24826E+01_JPRB, 0.26070E+01_JPRB, 0.24558E+01_JPRB ,&
&  0.21262E+01_JPRB, 0.20507E+01_JPRB, 0.18476E+01_JPRB, 0.14883E+01_JPRB, 0.13090E+01_JPRB, 0.11434E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11141E+01_JPRB, 0.10474E+01_JPRB, 0.79131E+00_JPRB, 0.50364E+00_JPRB, 0.44069E+00_JPRB, 0.44160E+00_JPRB, 0.45106E+00_JPRB ,&
&  0.45152E+00_JPRB, 0.48518E+00_JPRB, 0.52853E+00_JPRB, 0.51550E+00_JPRB, 0.48243E+00_JPRB, 0.48127E+00_JPRB, 0.50300E+00_JPRB ,&
&  0.55782E+00_JPRB, 0.60469E+00_JPRB, 0.61397E+00_JPRB, 0.61061E+00_JPRB, 0.59517E+00_JPRB, 0.63209E+00_JPRB, 0.65521E+00_JPRB ,&
&  0.69215E+00_JPRB, 0.69671E+00_JPRB, 0.72168E+00_JPRB, 0.74304E+00_JPRB, 0.75293E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.66836E+00_JPRB, 0.69431E+00_JPRB, 0.73555E+00_JPRB, 0.74581E+00_JPRB, 0.73579E+00_JPRB, 0.73730E+00_JPRB, 0.75387E+00_JPRB ,&
&  0.77664E+00_JPRB, 0.80766E+00_JPRB, 0.85808E+00_JPRB, 0.92708E+00_JPRB, 0.10357E+01_JPRB, 0.11505E+01_JPRB, 0.13038E+01_JPRB ,&
&  0.15062E+01_JPRB, 0.17420E+01_JPRB, 0.20089E+01_JPRB, 0.22923E+01_JPRB, 0.24888E+01_JPRB, 0.24999E+01_JPRB, 0.16045E+01_JPRB ,&
&  0.14454E+01_JPRB, 0.21112E+01_JPRB, 0.21398E+01_JPRB, 0.23542E+01_JPRB, 0.25452E+01_JPRB, 0.26238E+01_JPRB, 0.24692E+01_JPRB ,&
&  0.24116E+01_JPRB, 0.23206E+01_JPRB, 0.22665E+01_JPRB, 0.22124E+01_JPRB, 0.21810E+01_JPRB, 0.21677E+01_JPRB, 0.21079E+01_JPRB ,&
&  0.20373E+01_JPRB, 0.20090E+01_JPRB, 0.20556E+01_JPRB, 0.21824E+01_JPRB, 0.23737E+01_JPRB, 0.25939E+01_JPRB, 0.29175E+01_JPRB ,&
&  0.32760E+01_JPRB, 0.36181E+01_JPRB, 0.39158E+01_JPRB, 0.36870E+01_JPRB, 0.36404E+01_JPRB, 0.36285E+01_JPRB, 0.38158E+01_JPRB ,&
&  0.37529E+01_JPRB, 0.35331E+01_JPRB, 0.38498E+01_JPRB, 0.40950E+01_JPRB, 0.38580E+01_JPRB, 0.34954E+01_JPRB, 0.31142E+01_JPRB ,&
&  0.32403E+01_JPRB, 0.32501E+01_JPRB, 0.32130E+01_JPRB, 0.28981E+01_JPRB, 0.28986E+01_JPRB, 0.32260E+01_JPRB, 0.33710E+01_JPRB ,&
&  0.29266E+01_JPRB, 0.26346E+01_JPRB, 0.22831E+01_JPRB, 0.18851E+01_JPRB, 0.15599E+01_JPRB, 0.14211E+01_JPRB, 0.14236E+01_JPRB ,&
&  0.12823E+01_JPRB, 0.11611E+01_JPRB, 0.81410E+00_JPRB, 0.49428E+00_JPRB, 0.40370E+00_JPRB, 0.36774E+00_JPRB, 0.41504E+00_JPRB ,&
&  0.46501E+00_JPRB, 0.50960E+00_JPRB, 0.52278E+00_JPRB, 0.62642E+00_JPRB, 0.53778E+00_JPRB, 0.51138E+00_JPRB, 0.51392E+00_JPRB ,&
&  0.55697E+00_JPRB, 0.59974E+00_JPRB, 0.62104E+00_JPRB, 0.60101E+00_JPRB, 0.56991E+00_JPRB, 0.57785E+00_JPRB, 0.58493E+00_JPRB ,&
&  0.60880E+00_JPRB, 0.62231E+00_JPRB, 0.62296E+00_JPRB, 0.63796E+00_JPRB, 0.66050E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.61322E+00_JPRB, 0.63408E+00_JPRB, 0.65584E+00_JPRB, 0.67282E+00_JPRB, 0.68045E+00_JPRB, 0.67589E+00_JPRB, 0.66832E+00_JPRB ,&
&  0.66845E+00_JPRB, 0.67681E+00_JPRB, 0.71025E+00_JPRB, 0.75070E+00_JPRB, 0.81462E+00_JPRB, 0.88365E+00_JPRB, 0.99671E+00_JPRB ,&
&  0.11967E+01_JPRB, 0.14714E+01_JPRB, 0.18559E+01_JPRB, 0.22854E+01_JPRB, 0.24418E+01_JPRB, 0.21561E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.18265E+01_JPRB, 0.21624E+01_JPRB, 0.22167E+01_JPRB, 0.26043E+01_JPRB, 0.29765E+01_JPRB, 0.31209E+01_JPRB, 0.25608E+01_JPRB ,&
&  0.24473E+01_JPRB, 0.23311E+01_JPRB, 0.22131E+01_JPRB, 0.21699E+01_JPRB, 0.21509E+01_JPRB, 0.21595E+01_JPRB, 0.21352E+01_JPRB ,&
&  0.20500E+01_JPRB, 0.19546E+01_JPRB, 0.19068E+01_JPRB, 0.19279E+01_JPRB, 0.20435E+01_JPRB, 0.21872E+01_JPRB, 0.24042E+01_JPRB ,&
&  0.28023E+01_JPRB, 0.33862E+01_JPRB, 0.40345E+01_JPRB, 0.41522E+01_JPRB, 0.40062E+01_JPRB, 0.39807E+01_JPRB, 0.39946E+01_JPRB ,&
&  0.40622E+01_JPRB, 0.43793E+01_JPRB, 0.44708E+01_JPRB, 0.43821E+01_JPRB, 0.40823E+01_JPRB, 0.38901E+01_JPRB, 0.33810E+01_JPRB ,&
&  0.33184E+01_JPRB, 0.33441E+01_JPRB, 0.31310E+01_JPRB, 0.32062E+01_JPRB, 0.34063E+01_JPRB, 0.38878E+01_JPRB, 0.36883E+01_JPRB ,&
&  0.29856E+01_JPRB, 0.24791E+01_JPRB, 0.22308E+01_JPRB, 0.21673E+01_JPRB, 0.19622E+01_JPRB, 0.18910E+01_JPRB, 0.18868E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.51201E+00_JPRB, 0.31850E+00_JPRB, 0.37973E+00_JPRB, 0.27883E+00_JPRB, 0.26506E+00_JPRB, 0.39330E+00_JPRB ,&
&  0.48121E+00_JPRB, 0.57666E+00_JPRB, 0.64423E+00_JPRB, 0.67570E+00_JPRB, 0.67840E+00_JPRB, 0.61368E+00_JPRB, 0.57106E+00_JPRB ,&
&  0.57275E+00_JPRB, 0.60027E+00_JPRB, 0.64014E+00_JPRB, 0.63127E+00_JPRB, 0.58761E+00_JPRB, 0.57230E+00_JPRB, 0.55880E+00_JPRB ,&
&  0.55842E+00_JPRB, 0.58368E+00_JPRB, 0.60069E+00_JPRB, 0.60485E+00_JPRB, 0.60285E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.61582E+00_JPRB, 0.61807E+00_JPRB, 0.62832E+00_JPRB, 0.61943E+00_JPRB, 0.60349E+00_JPRB, 0.59614E+00_JPRB, 0.60488E+00_JPRB ,&
&  0.59944E+00_JPRB, 0.59691E+00_JPRB, 0.62000E+00_JPRB, 0.65602E+00_JPRB, 0.70547E+00_JPRB, 0.76027E+00_JPRB, 0.81433E+00_JPRB ,&
&  0.94014E+00_JPRB, 0.11781E+01_JPRB, 0.15841E+01_JPRB, 0.21667E+01_JPRB, 0.24904E+01_JPRB, 0.23202E+01_JPRB, 0.22592E+01_JPRB ,&
&  0.19904E+01_JPRB, 0.23088E+01_JPRB, 0.28064E+01_JPRB, 0.35878E+01_JPRB, 0.37988E+01_JPRB, 0.35636E+01_JPRB, 0.28283E+01_JPRB ,&
&  0.27069E+01_JPRB, 0.25076E+01_JPRB, 0.22808E+01_JPRB, 0.20593E+01_JPRB, 0.19375E+01_JPRB, 0.19154E+01_JPRB, 0.19481E+01_JPRB ,&
&  0.19618E+01_JPRB, 0.19009E+01_JPRB, 0.18282E+01_JPRB, 0.17992E+01_JPRB, 0.18402E+01_JPRB, 0.19446E+01_JPRB, 0.20825E+01_JPRB ,&
&  0.22924E+01_JPRB, 0.27122E+01_JPRB, 0.36163E+01_JPRB, 0.45427E+01_JPRB, 0.38298E+01_JPRB, 0.43936E+01_JPRB, 0.41998E+01_JPRB ,&
&  0.41477E+01_JPRB, 0.45560E+01_JPRB, 0.46097E+01_JPRB, 0.51857E+01_JPRB, 0.45439E+01_JPRB, 0.40267E+01_JPRB, 0.34758E+01_JPRB ,&
&  0.33730E+01_JPRB, 0.31373E+01_JPRB, 0.32048E+01_JPRB, 0.34721E+01_JPRB, 0.38453E+01_JPRB, 0.43169E+01_JPRB, 0.32068E+01_JPRB ,&
&  0.27635E+01_JPRB, 0.28680E+01_JPRB, 0.26013E+01_JPRB, 0.22375E+01_JPRB, 0.23136E+01_JPRB, 0.24264E+01_JPRB, 0.15613E+01_JPRB ,&
&  0.52657E+00_JPRB, 0.33818E+00_JPRB, 0.27846E+00_JPRB, 0.25837E+00_JPRB, 0.22314E+00_JPRB, 0.25380E+00_JPRB, 0.60074E+00_JPRB ,&
&  0.58016E+00_JPRB, 0.72981E+00_JPRB, 0.81378E+00_JPRB, 0.84266E+00_JPRB, 0.88136E+00_JPRB, 0.78649E+00_JPRB, 0.10790E+01_JPRB ,&
&  0.83903E+00_JPRB, 0.73069E+00_JPRB, 0.70963E+00_JPRB, 0.70725E+00_JPRB, 0.66114E+00_JPRB, 0.63272E+00_JPRB, 0.60345E+00_JPRB ,&
&  0.59727E+00_JPRB, 0.60449E+00_JPRB, 0.60469E+00_JPRB, 0.60260E+00_JPRB, 0.61618E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.64727E+00_JPRB, 0.64154E+00_JPRB, 0.62856E+00_JPRB, 0.59223E+00_JPRB, 0.56274E+00_JPRB, 0.55122E+00_JPRB, 0.54647E+00_JPRB ,&
&  0.54242E+00_JPRB, 0.54186E+00_JPRB, 0.55126E+00_JPRB, 0.57665E+00_JPRB, 0.60629E+00_JPRB, 0.65176E+00_JPRB, 0.71017E+00_JPRB ,&
&  0.81807E+00_JPRB, 0.10248E+01_JPRB, 0.14300E+01_JPRB, 0.18754E+01_JPRB, 0.23360E+01_JPRB, 0.22691E+01_JPRB, 0.24388E+01_JPRB ,&
&  0.21660E+01_JPRB, 0.23249E+01_JPRB, 0.29733E+01_JPRB, 0.40764E+01_JPRB, 0.56464E+01_JPRB, 0.56164E+01_JPRB, 0.48487E+01_JPRB ,&
&  0.41774E+01_JPRB, 0.36762E+01_JPRB, 0.32713E+01_JPRB, 0.28208E+01_JPRB, 0.21529E+01_JPRB, 0.17856E+01_JPRB, 0.16951E+01_JPRB ,&
&  0.16645E+01_JPRB, 0.17017E+01_JPRB, 0.16663E+01_JPRB, 0.16513E+01_JPRB, 0.17190E+01_JPRB, 0.18259E+01_JPRB, 0.19681E+01_JPRB ,&
&  0.20722E+01_JPRB, 0.22335E+01_JPRB, 0.28027E+01_JPRB, 0.40801E+01_JPRB, 0.43292E+01_JPRB, 0.41009E+01_JPRB, 0.41090E+01_JPRB ,&
&  0.40971E+01_JPRB, 0.47769E+01_JPRB, 0.53381E+01_JPRB, 0.62532E+01_JPRB, 0.46694E+01_JPRB, 0.40460E+01_JPRB, 0.36965E+01_JPRB ,&
&  0.34918E+01_JPRB, 0.35773E+01_JPRB, 0.30675E+01_JPRB, 0.32428E+01_JPRB, 0.38757E+01_JPRB, 0.37197E+01_JPRB, 0.34382E+01_JPRB ,&
&  0.33175E+01_JPRB, 0.27736E+01_JPRB, 0.25207E+01_JPRB, 0.22100E+01_JPRB, 0.26783E+01_JPRB, 0.21738E+01_JPRB, 0.81168E+00_JPRB ,&
&  0.46972E+00_JPRB, 0.36316E+00_JPRB, 0.26454E+00_JPRB, 0.25052E+00_JPRB, 0.23548E+00_JPRB, 0.27473E+00_JPRB, 0.53479E+00_JPRB ,&
&  0.59861E+00_JPRB, 0.85584E+00_JPRB, 0.96117E+00_JPRB, 0.10398E+01_JPRB, 0.10373E+01_JPRB, 0.11482E+01_JPRB, 0.24623E+01_JPRB ,&
&  0.22499E+01_JPRB, 0.17455E+01_JPRB, 0.13332E+01_JPRB, 0.10751E+01_JPRB, 0.88005E+00_JPRB, 0.77409E+00_JPRB, 0.70644E+00_JPRB ,&
&  0.67727E+00_JPRB, 0.66311E+00_JPRB, 0.65295E+00_JPRB, 0.64537E+00_JPRB, 0.63834E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.67586E+00_JPRB, 0.67444E+00_JPRB, 0.65750E+00_JPRB, 0.59759E+00_JPRB, 0.56014E+00_JPRB, 0.55574E+00_JPRB, 0.54331E+00_JPRB ,&
&  0.52714E+00_JPRB, 0.51257E+00_JPRB, 0.50881E+00_JPRB, 0.51879E+00_JPRB, 0.52710E+00_JPRB, 0.54809E+00_JPRB, 0.60719E+00_JPRB ,&
&  0.74113E+00_JPRB, 0.97773E+00_JPRB, 0.13148E+01_JPRB, 0.16238E+01_JPRB, 0.19079E+01_JPRB, 0.19724E+01_JPRB, 0.23801E+01_JPRB ,&
&  0.24468E+01_JPRB, 0.24185E+01_JPRB, 0.28653E+01_JPRB, 0.41583E+01_JPRB, 0.62329E+01_JPRB, 0.74743E+01_JPRB, 0.78085E+01_JPRB ,&
&  0.66681E+01_JPRB, 0.54507E+01_JPRB, 0.46265E+01_JPRB, 0.40602E+01_JPRB, 0.32968E+01_JPRB, 0.23707E+01_JPRB, 0.17898E+01_JPRB ,&
&  0.15465E+01_JPRB, 0.15624E+01_JPRB, 0.15987E+01_JPRB, 0.15669E+01_JPRB, 0.16349E+01_JPRB, 0.17423E+01_JPRB, 0.19259E+01_JPRB ,&
&  0.20410E+01_JPRB, 0.21090E+01_JPRB, 0.23330E+01_JPRB, 0.32400E+01_JPRB, 0.45455E+01_JPRB, 0.50323E+01_JPRB, 0.43299E+01_JPRB ,&
&  0.37613E+01_JPRB, 0.39718E+01_JPRB, 0.60374E+01_JPRB, 0.63573E+01_JPRB, 0.48160E+01_JPRB, 0.43844E+01_JPRB, 0.39451E+01_JPRB ,&
&  0.38994E+01_JPRB, 0.40872E+01_JPRB, 0.35676E+01_JPRB, 0.36021E+01_JPRB, 0.34004E+01_JPRB, 0.34343E+01_JPRB, 0.37776E+01_JPRB ,&
&  0.34857E+01_JPRB, 0.29763E+01_JPRB, 0.24446E+01_JPRB, 0.21062E+01_JPRB, 0.22266E+01_JPRB, 0.14326E+01_JPRB, 0.84344E+00_JPRB ,&
&  0.64434E+00_JPRB, 0.47632E+00_JPRB, 0.36701E+00_JPRB, 0.36646E+00_JPRB, 0.31059E+00_JPRB, 0.34503E+00_JPRB, 0.47102E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.86881E+00_JPRB, 0.11109E+01_JPRB, 0.12566E+01_JPRB, 0.11672E+01_JPRB, 0.10294E+01_JPRB, 0.22740E+01_JPRB ,&
&  0.17178E+01_JPRB, 0.16196E+01_JPRB, 0.16360E+01_JPRB, 0.13814E+01_JPRB, 0.11207E+01_JPRB, 0.94796E+00_JPRB, 0.81779E+00_JPRB ,&
&  0.73760E+00_JPRB, 0.69298E+00_JPRB, 0.67095E+00_JPRB, 0.67279E+00_JPRB, 0.68384E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.74933E+00_JPRB, 0.70586E+00_JPRB, 0.68402E+00_JPRB, 0.64259E+00_JPRB, 0.58236E+00_JPRB, 0.55126E+00_JPRB, 0.55156E+00_JPRB ,&
&  0.53849E+00_JPRB, 0.51778E+00_JPRB, 0.49826E+00_JPRB, 0.48383E+00_JPRB, 0.48821E+00_JPRB, 0.50176E+00_JPRB, 0.53345E+00_JPRB ,&
&  0.67785E+00_JPRB, 0.10492E+01_JPRB, 0.10864E+01_JPRB, 0.12362E+01_JPRB, 0.14938E+01_JPRB, 0.19034E+01_JPRB, 0.21364E+01_JPRB ,&
&  0.26185E+01_JPRB, 0.26627E+01_JPRB, 0.28774E+01_JPRB, 0.42270E+01_JPRB, 0.76585E+01_JPRB, 0.91295E+01_JPRB, 0.10014E+02_JPRB ,&
&  0.97800E+01_JPRB, 0.80574E+01_JPRB, 0.63949E+01_JPRB, 0.50097E+01_JPRB, 0.40773E+01_JPRB, 0.34333E+01_JPRB, 0.25963E+01_JPRB ,&
&  0.19874E+01_JPRB, 0.17338E+01_JPRB, 0.16971E+01_JPRB, 0.16623E+01_JPRB, 0.16500E+01_JPRB, 0.17045E+01_JPRB, 0.18485E+01_JPRB ,&
&  0.20188E+01_JPRB, 0.21541E+01_JPRB, 0.22852E+01_JPRB, 0.29325E+01_JPRB, 0.37428E+01_JPRB, 0.43423E+01_JPRB, 0.53776E+01_JPRB ,&
&  0.48737E+01_JPRB, 0.50853E+01_JPRB, 0.58689E+01_JPRB, 0.70397E+01_JPRB, 0.53030E+01_JPRB, 0.44900E+01_JPRB, 0.41470E+01_JPRB ,&
&  0.33789E+01_JPRB, 0.33722E+01_JPRB, 0.38293E+01_JPRB, 0.38330E+01_JPRB, 0.32545E+01_JPRB, 0.38992E+01_JPRB, 0.48674E+01_JPRB ,&
&  0.35802E+01_JPRB, 0.32866E+01_JPRB, 0.31615E+01_JPRB, 0.32600E+01_JPRB, 0.20250E+01_JPRB, 0.12629E+01_JPRB, 0.18784E+01_JPRB ,&
&  0.18592E+01_JPRB, 0.95379E+00_JPRB, 0.64088E+00_JPRB, 0.74825E+00_JPRB, 0.50949E+00_JPRB, 0.50420E+00_JPRB, 0.57990E+00_JPRB ,&
&  0.62840E+00_JPRB, 0.66810E+00_JPRB, 0.11126E+01_JPRB, 0.11613E+01_JPRB, 0.11506E+01_JPRB, 0.96444E+00_JPRB, 0.15938E+01_JPRB ,&
&  0.16826E+01_JPRB, 0.13852E+01_JPRB, 0.14874E+01_JPRB, 0.14050E+01_JPRB, 0.12373E+01_JPRB, 0.11002E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.90668E+00_JPRB, 0.83740E+00_JPRB, 0.80229E+00_JPRB, 0.79010E+00_JPRB, 0.79068E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.74022E+00_JPRB, 0.65249E+00_JPRB, 0.61284E+00_JPRB, 0.60955E+00_JPRB, 0.60076E+00_JPRB, 0.55344E+00_JPRB, 0.53569E+00_JPRB ,&
&  0.54889E+00_JPRB, 0.52674E+00_JPRB, 0.49490E+00_JPRB, 0.47923E+00_JPRB, 0.48399E+00_JPRB, 0.49095E+00_JPRB, 0.48962E+00_JPRB ,&
&  0.57698E+00_JPRB, 0.80139E+00_JPRB, 0.90765E+00_JPRB, 0.10346E+01_JPRB, 0.13324E+01_JPRB, 0.17169E+01_JPRB, 0.19399E+01_JPRB ,&
&  0.24775E+01_JPRB, 0.27796E+01_JPRB, 0.32419E+01_JPRB, 0.47551E+01_JPRB, 0.68540E+01_JPRB, 0.88460E+01_JPRB, 0.95199E+01_JPRB ,&
&  0.10357E+02_JPRB, 0.98407E+01_JPRB, 0.78214E+01_JPRB, 0.63415E+01_JPRB, 0.53200E+01_JPRB, 0.46541E+01_JPRB, 0.39175E+01_JPRB ,&
&  0.30360E+01_JPRB, 0.23407E+01_JPRB, 0.20348E+01_JPRB, 0.18989E+01_JPRB, 0.18650E+01_JPRB, 0.18027E+01_JPRB, 0.18675E+01_JPRB ,&
&  0.20241E+01_JPRB, 0.22158E+01_JPRB, 0.23099E+01_JPRB, 0.28643E+01_JPRB, 0.38471E+01_JPRB, 0.45350E+01_JPRB, 0.51678E+01_JPRB ,&
&  0.52181E+01_JPRB, 0.49450E+01_JPRB, 0.58056E+01_JPRB, 0.60818E+01_JPRB, 0.57205E+01_JPRB, 0.49466E+01_JPRB, 0.46807E+01_JPRB ,&
&  0.36959E+01_JPRB, 0.34062E+01_JPRB, 0.35454E+01_JPRB, 0.36228E+01_JPRB, 0.39911E+01_JPRB, 0.50410E+01_JPRB, 0.47198E+01_JPRB ,&
&  0.39784E+01_JPRB, 0.35294E+01_JPRB, 0.31393E+01_JPRB, 0.26681E+01_JPRB, 0.27476E+01_JPRB, 0.19558E+01_JPRB, 0.22374E+01_JPRB ,&
&  0.22478E+01_JPRB, 0.20353E+01_JPRB, 0.16057E+01_JPRB, 0.12601E+01_JPRB, 0.92103E+00_JPRB, 0.84656E+00_JPRB, 0.69874E+00_JPRB ,&
&  0.64798E+00_JPRB, 0.62510E+00_JPRB, 0.86136E+00_JPRB, 0.10041E+01_JPRB, 0.10099E+01_JPRB, 0.93038E+00_JPRB, 0.12640E+01_JPRB ,&
&  0.13316E+01_JPRB, 0.14152E+01_JPRB, 0.13430E+01_JPRB, 0.12324E+01_JPRB, 0.11141E+01_JPRB, 0.10453E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.95358E+00_JPRB, 0.91802E+00_JPRB, 0.90184E+00_JPRB, 0.87477E+00_JPRB, 0.83000E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.70510E+00_JPRB, 0.61776E+00_JPRB, 0.57286E+00_JPRB, 0.55358E+00_JPRB, 0.55409E+00_JPRB, 0.54968E+00_JPRB, 0.52952E+00_JPRB ,&
&  0.51876E+00_JPRB, 0.52795E+00_JPRB, 0.51343E+00_JPRB, 0.50054E+00_JPRB, 0.49665E+00_JPRB, 0.49421E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.52169E+00_JPRB, 0.67467E+00_JPRB, 0.74242E+00_JPRB, 0.92297E+00_JPRB, 0.11668E+01_JPRB, 0.15532E+01_JPRB, 0.20070E+01_JPRB ,&
&  0.22652E+01_JPRB, 0.25735E+01_JPRB, 0.30229E+01_JPRB, 0.41623E+01_JPRB, 0.55182E+01_JPRB, 0.67259E+01_JPRB, 0.70368E+01_JPRB ,&
&  0.66276E+01_JPRB, 0.67687E+01_JPRB, 0.63592E+01_JPRB, 0.56932E+01_JPRB, 0.51200E+01_JPRB, 0.46816E+01_JPRB, 0.41433E+01_JPRB ,&
&  0.35145E+01_JPRB, 0.28818E+01_JPRB, 0.24290E+01_JPRB, 0.21897E+01_JPRB, 0.20883E+01_JPRB, 0.21469E+01_JPRB, 0.21974E+01_JPRB ,&
&  0.22045E+01_JPRB, 0.22940E+01_JPRB, 0.22535E+01_JPRB, 0.25582E+01_JPRB, 0.32723E+01_JPRB, 0.43998E+01_JPRB, 0.48958E+01_JPRB ,&
&  0.57065E+01_JPRB, 0.54386E+01_JPRB, 0.61090E+01_JPRB, 0.67311E+01_JPRB, 0.51711E+01_JPRB, 0.52880E+01_JPRB, 0.55893E+01_JPRB ,&
&  0.53458E+01_JPRB, 0.48933E+01_JPRB, 0.49951E+01_JPRB, 0.46937E+01_JPRB, 0.42868E+01_JPRB, 0.49995E+01_JPRB, 0.45158E+01_JPRB ,&
&  0.38750E+01_JPRB, 0.35100E+01_JPRB, 0.30761E+01_JPRB, 0.30368E+01_JPRB, 0.26051E+01_JPRB, 0.24979E+01_JPRB, 0.19852E+01_JPRB ,&
&  0.20167E+01_JPRB, 0.16887E+01_JPRB, 0.17520E+01_JPRB, 0.14720E+01_JPRB, 0.12757E+01_JPRB, 0.10280E+01_JPRB, 0.77674E+00_JPRB ,&
&  0.65959E+00_JPRB, 0.62560E+00_JPRB, 0.69122E+00_JPRB, 0.87862E+00_JPRB, 0.98549E+00_JPRB, 0.90446E+00_JPRB, 0.10450E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.12926E+01_JPRB, 0.11418E+01_JPRB, 0.11469E+01_JPRB, 0.10393E+01_JPRB, 0.95652E+00_JPRB, 0.94238E+00_JPRB ,&
&  0.87970E+00_JPRB, 0.86342E+00_JPRB, 0.85440E+00_JPRB, 0.83401E+00_JPRB, 0.79266E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.65699E+00_JPRB, 0.58212E+00_JPRB, 0.52753E+00_JPRB, 0.51202E+00_JPRB, 0.52149E+00_JPRB, 0.52934E+00_JPRB, 0.53052E+00_JPRB ,&
&  0.52014E+00_JPRB, 0.52471E+00_JPRB, 0.52905E+00_JPRB, 0.52258E+00_JPRB, 0.51627E+00_JPRB, 0.51045E+00_JPRB, 0.48788E+00_JPRB ,&
&  0.48628E+00_JPRB, 0.63599E+00_JPRB, 0.66035E+00_JPRB, 0.73271E+00_JPRB, 0.92105E+00_JPRB, 0.13899E+01_JPRB, 0.18492E+01_JPRB ,&
&  0.21482E+01_JPRB, 0.23869E+01_JPRB, 0.27750E+01_JPRB, 0.33405E+01_JPRB, 0.38800E+01_JPRB, 0.41668E+01_JPRB, 0.42455E+01_JPRB ,&
&  0.46831E+01_JPRB, 0.39098E+01_JPRB, 0.39647E+01_JPRB, 0.35919E+01_JPRB, 0.33811E+01_JPRB, 0.31666E+01_JPRB, 0.29733E+01_JPRB ,&
&  0.27732E+01_JPRB, 0.25323E+01_JPRB, 0.22821E+01_JPRB, 0.21175E+01_JPRB, 0.20409E+01_JPRB, 0.20319E+01_JPRB, 0.20890E+01_JPRB ,&
&  0.21547E+01_JPRB, 0.21869E+01_JPRB, 0.22065E+01_JPRB, 0.21179E+01_JPRB, 0.25622E+01_JPRB, 0.35816E+01_JPRB, 0.46745E+01_JPRB ,&
&  0.46456E+01_JPRB, 0.42318E+01_JPRB, 0.66479E+01_JPRB, 0.70509E+01_JPRB, 0.63792E+01_JPRB, 0.63452E+01_JPRB, 0.63314E+01_JPRB ,&
&  0.60861E+01_JPRB, 0.53651E+01_JPRB, 0.45840E+01_JPRB, 0.42247E+01_JPRB, 0.49620E+01_JPRB, 0.47522E+01_JPRB, 0.43433E+01_JPRB ,&
&  0.38896E+01_JPRB, 0.36274E+01_JPRB, 0.32915E+01_JPRB, 0.30233E+01_JPRB, 0.28599E+01_JPRB, 0.29352E+01_JPRB, 0.26729E+01_JPRB ,&
&  0.24305E+01_JPRB, 0.24789E+01_JPRB, 0.20056E+01_JPRB, 0.15246E+01_JPRB, 0.12430E+01_JPRB, 0.10540E+01_JPRB, 0.86587E+00_JPRB ,&
&  0.68093E+00_JPRB, 0.62704E+00_JPRB, 0.63275E+00_JPRB, 0.72483E+00_JPRB, 0.88491E+00_JPRB, 0.81928E+00_JPRB, 0.88275E+00_JPRB ,&
&  0.87792E+00_JPRB, 0.10809E+01_JPRB, 0.10604E+01_JPRB, 0.10254E+01_JPRB, 0.98430E+00_JPRB, 0.90523E+00_JPRB, 0.84615E+00_JPRB ,&
&  0.78352E+00_JPRB, 0.79114E+00_JPRB, 0.79580E+00_JPRB, 0.77096E+00_JPRB, 0.73666E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.62487E+00_JPRB, 0.55909E+00_JPRB, 0.50660E+00_JPRB, 0.49200E+00_JPRB, 0.51012E+00_JPRB, 0.52087E+00_JPRB, 0.52716E+00_JPRB ,&
&  0.52404E+00_JPRB, 0.52244E+00_JPRB, 0.53514E+00_JPRB, 0.54092E+00_JPRB, 0.54502E+00_JPRB, 0.53545E+00_JPRB, 0.50575E+00_JPRB ,&
&  0.50304E+00_JPRB, 0.62467E+00_JPRB, 0.60655E+00_JPRB, 0.68488E+00_JPRB, 0.81583E+00_JPRB, 0.11455E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.17327E+01_JPRB, 0.18899E+01_JPRB, 0.20802E+01_JPRB, 0.23662E+01_JPRB, 0.25205E+01_JPRB, 0.24384E+01_JPRB, 0.23349E+01_JPRB ,&
&  0.22201E+01_JPRB, 0.22024E+01_JPRB, 0.19018E+01_JPRB, 0.20240E+01_JPRB, 0.19677E+01_JPRB, 0.17109E+01_JPRB, 0.19014E+01_JPRB ,&
&  0.18788E+01_JPRB, 0.18847E+01_JPRB, 0.18401E+01_JPRB, 0.18049E+01_JPRB, 0.17977E+01_JPRB, 0.17910E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.17786E+01_JPRB, 0.18086E+01_JPRB, 0.19642E+01_JPRB, 0.19608E+01_JPRB, 0.20291E+01_JPRB, 0.25222E+01_JPRB, 0.37405E+01_JPRB ,&
&  0.50198E+01_JPRB, 0.49791E+01_JPRB, 0.49067E+01_JPRB, 0.64271E+01_JPRB, 0.73608E+01_JPRB, 0.68657E+01_JPRB, 0.63636E+01_JPRB ,&
&  0.62491E+01_JPRB, 0.57567E+01_JPRB, 0.52492E+01_JPRB, 0.49060E+01_JPRB, 0.50747E+01_JPRB, 0.49713E+01_JPRB, 0.46121E+01_JPRB ,&
&  0.41178E+01_JPRB, 0.37040E+01_JPRB, 0.34878E+01_JPRB, 0.32497E+01_JPRB, 0.28781E+01_JPRB, 0.27091E+01_JPRB, 0.27190E+01_JPRB ,&
&  0.26274E+01_JPRB, 0.22028E+01_JPRB, 0.16028E+01_JPRB, 0.13881E+01_JPRB, 0.10759E+01_JPRB, 0.91074E+00_JPRB, 0.85580E+00_JPRB ,&
&  0.68859E+00_JPRB, 0.59281E+00_JPRB, 0.59167E+00_JPRB, 0.61000E+00_JPRB, 0.74365E+00_JPRB, 0.70340E+00_JPRB, 0.76747E+00_JPRB ,&
&  0.75119E+00_JPRB, 0.68494E+00_JPRB, 0.87742E+00_JPRB, 0.91560E+00_JPRB, 0.89607E+00_JPRB, 0.81277E+00_JPRB, 0.73809E+00_JPRB ,&
&  0.70630E+00_JPRB, 0.73535E+00_JPRB, 0.73627E+00_JPRB, 0.72102E+00_JPRB, 0.69420E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.58461E+00_JPRB, 0.52274E+00_JPRB, 0.47968E+00_JPRB, 0.46106E+00_JPRB, 0.47509E+00_JPRB, 0.49523E+00_JPRB, 0.50810E+00_JPRB ,&
&  0.52123E+00_JPRB, 0.53133E+00_JPRB, 0.53994E+00_JPRB, 0.54951E+00_JPRB, 0.55735E+00_JPRB, 0.54352E+00_JPRB, 0.53334E+00_JPRB ,&
&  0.54483E+00_JPRB, 0.57102E+00_JPRB, 0.58286E+00_JPRB, 0.61265E+00_JPRB, 0.88713E+00_JPRB, 0.10378E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.14858E+01_JPRB, 0.14867E+01_JPRB, 0.14654E+01_JPRB, 0.14427E+01_JPRB, 0.14291E+01_JPRB, 0.13060E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.10579E+01_JPRB, 0.10206E+01_JPRB, 0.10882E+01_JPRB, 0.11888E+01_JPRB, 0.10420E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.12213E+01_JPRB, 0.12951E+01_JPRB, 0.13316E+01_JPRB, 0.13413E+01_JPRB, 0.13665E+01_JPRB, 0.14032E+01_JPRB, 0.14443E+01_JPRB ,&
&  0.14930E+01_JPRB, 0.15017E+01_JPRB, 0.15699E+01_JPRB, 0.17819E+01_JPRB, 0.19282E+01_JPRB, 0.24083E+01_JPRB, 0.38536E+01_JPRB ,&
&  0.49744E+01_JPRB, 0.51820E+01_JPRB, 0.62097E+01_JPRB, 0.75353E+01_JPRB, 0.71350E+01_JPRB, 0.63963E+01_JPRB, 0.59016E+01_JPRB ,&
&  0.55134E+01_JPRB, 0.51617E+01_JPRB, 0.49174E+01_JPRB, 0.47181E+01_JPRB, 0.47495E+01_JPRB, 0.49071E+01_JPRB, 0.47581E+01_JPRB ,&
&  0.43210E+01_JPRB, 0.39052E+01_JPRB, 0.36862E+01_JPRB, 0.33898E+01_JPRB, 0.31289E+01_JPRB, 0.28239E+01_JPRB, 0.28145E+01_JPRB ,&
&  0.28054E+01_JPRB, 0.18910E+01_JPRB, 0.14792E+01_JPRB, 0.13753E+01_JPRB, 0.98309E+00_JPRB, 0.82695E+00_JPRB, 0.86394E+00_JPRB ,&
&  0.67308E+00_JPRB, 0.56708E+00_JPRB, 0.60203E+00_JPRB, 0.63973E+00_JPRB, 0.65314E+00_JPRB, 0.63344E+00_JPRB, 0.59471E+00_JPRB ,&
&  0.52385E+00_JPRB, 0.54008E+00_JPRB, 0.71965E+00_JPRB, 0.75804E+00_JPRB, 0.75463E+00_JPRB, 0.71789E+00_JPRB, 0.68609E+00_JPRB ,&
&  0.64829E+00_JPRB, 0.66928E+00_JPRB, 0.67096E+00_JPRB, 0.66209E+00_JPRB, 0.64475E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.58165E+00_JPRB, 0.51840E+00_JPRB, 0.46641E+00_JPRB, 0.44993E+00_JPRB, 0.47033E+00_JPRB, 0.49047E+00_JPRB, 0.50409E+00_JPRB ,&
&  0.52323E+00_JPRB, 0.53879E+00_JPRB, 0.55156E+00_JPRB, 0.56259E+00_JPRB, 0.57012E+00_JPRB, 0.56367E+00_JPRB, 0.61257E+00_JPRB ,&
&  0.48139E+00_JPRB, 0.49718E+00_JPRB, 0.49027E+00_JPRB, 0.57464E+00_JPRB, 0.90384E+00_JPRB, 0.90884E+00_JPRB, 0.97181E+00_JPRB ,&
&  0.95717E+00_JPRB, 0.94838E+00_JPRB, 0.90967E+00_JPRB, 0.87712E+00_JPRB, 0.86041E+00_JPRB, 0.87849E+00_JPRB, 0.85410E+00_JPRB ,&
&  0.74701E+00_JPRB, 0.67354E+00_JPRB, 0.65447E+00_JPRB, 0.65877E+00_JPRB, 0.72411E+00_JPRB, 0.77296E+00_JPRB, 0.86579E+00_JPRB ,&
&  0.91553E+00_JPRB, 0.97698E+00_JPRB, 0.10402E+01_JPRB, 0.10904E+01_JPRB, 0.11108E+01_JPRB, 0.11183E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.11813E+01_JPRB, 0.12607E+01_JPRB, 0.15254E+01_JPRB, 0.17601E+01_JPRB, 0.26793E+01_JPRB, 0.48100E+01_JPRB ,&
&  0.53399E+01_JPRB, 0.47895E+01_JPRB, 0.53196E+01_JPRB, 0.65638E+01_JPRB, 0.61676E+01_JPRB, 0.53922E+01_JPRB, 0.46648E+01_JPRB ,&
&  0.42040E+01_JPRB, 0.39129E+01_JPRB, 0.39185E+01_JPRB, 0.41192E+01_JPRB, 0.42546E+01_JPRB, 0.44927E+01_JPRB, 0.45455E+01_JPRB ,&
&  0.44146E+01_JPRB, 0.42989E+01_JPRB, 0.41358E+01_JPRB, 0.39220E+01_JPRB, 0.36211E+01_JPRB, 0.34125E+01_JPRB, 0.31609E+01_JPRB ,&
&  0.28542E+01_JPRB, 0.24476E+01_JPRB, 0.18496E+01_JPRB, 0.14955E+01_JPRB, 0.10788E+01_JPRB, 0.96077E+00_JPRB, 0.11114E+01_JPRB ,&
&  0.78597E+00_JPRB, 0.65536E+00_JPRB, 0.62846E+00_JPRB, 0.64313E+00_JPRB, 0.61153E+00_JPRB, 0.59408E+00_JPRB, 0.52781E+00_JPRB ,&
&  0.45772E+00_JPRB, 0.51811E+00_JPRB, 0.68490E+00_JPRB, 0.71230E+00_JPRB, 0.69752E+00_JPRB, 0.68036E+00_JPRB, 0.55724E+00_JPRB ,&
&  0.66441E+00_JPRB, 0.63301E+00_JPRB, 0.63336E+00_JPRB, 0.62646E+00_JPRB, 0.61705E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.60169E+00_JPRB, 0.54294E+00_JPRB, 0.48064E+00_JPRB, 0.46752E+00_JPRB, 0.48822E+00_JPRB, 0.49984E+00_JPRB, 0.50805E+00_JPRB ,&
&  0.52574E+00_JPRB, 0.53485E+00_JPRB, 0.54807E+00_JPRB, 0.56421E+00_JPRB, 0.57197E+00_JPRB, 0.59962E+00_JPRB, 0.53978E+00_JPRB ,&
&  0.46016E+00_JPRB, 0.46727E+00_JPRB, 0.46065E+00_JPRB, 0.49528E+00_JPRB, 0.59290E+00_JPRB, 0.61147E+00_JPRB, 0.62382E+00_JPRB ,&
&  0.64825E+00_JPRB, 0.62837E+00_JPRB, 0.63321E+00_JPRB, 0.62589E+00_JPRB, 0.61543E+00_JPRB, 0.62848E+00_JPRB, 0.64385E+00_JPRB ,&
&  0.58364E+00_JPRB, 0.54048E+00_JPRB, 0.52221E+00_JPRB, 0.51644E+00_JPRB, 0.58369E+00_JPRB, 0.66824E+00_JPRB, 0.71549E+00_JPRB ,&
&  0.75165E+00_JPRB, 0.79409E+00_JPRB, 0.84427E+00_JPRB, 0.89872E+00_JPRB, 0.94962E+00_JPRB, 0.98418E+00_JPRB, 0.99659E+00_JPRB ,&
&  0.10081E+01_JPRB, 0.10303E+01_JPRB, 0.10678E+01_JPRB, 0.12097E+01_JPRB, 0.15159E+01_JPRB, 0.22924E+01_JPRB, 0.33953E+01_JPRB ,&
&  0.35902E+01_JPRB, 0.35065E+01_JPRB, 0.35691E+01_JPRB, 0.38500E+01_JPRB, 0.39043E+01_JPRB, 0.35100E+01_JPRB, 0.31105E+01_JPRB ,&
&  0.29897E+01_JPRB, 0.29294E+01_JPRB, 0.32388E+01_JPRB, 0.35009E+01_JPRB, 0.36692E+01_JPRB, 0.39028E+01_JPRB, 0.39953E+01_JPRB ,&
&  0.42049E+01_JPRB, 0.44067E+01_JPRB, 0.44017E+01_JPRB, 0.41806E+01_JPRB, 0.39610E+01_JPRB, 0.37612E+01_JPRB, 0.34146E+01_JPRB ,&
&  0.30592E+01_JPRB, 0.26250E+01_JPRB, 0.21022E+01_JPRB, 0.16808E+01_JPRB, 0.12897E+01_JPRB, 0.11842E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.76382E+00_JPRB, 0.65318E+00_JPRB, 0.61742E+00_JPRB, 0.61457E+00_JPRB, 0.57306E+00_JPRB, 0.53055E+00_JPRB, 0.52417E+00_JPRB ,&
&  0.55945E+00_JPRB, 0.68454E+00_JPRB, 0.70363E+00_JPRB, 0.70898E+00_JPRB, 0.70082E+00_JPRB, 0.69301E+00_JPRB, 0.51176E+00_JPRB ,&
&  0.55245E+00_JPRB, 0.66807E+00_JPRB, 0.65486E+00_JPRB, 0.64599E+00_JPRB, 0.63443E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.60419E+00_JPRB, 0.56084E+00_JPRB, 0.51179E+00_JPRB, 0.49248E+00_JPRB, 0.50239E+00_JPRB, 0.51393E+00_JPRB, 0.50128E+00_JPRB ,&
&  0.53728E+00_JPRB, 0.52397E+00_JPRB, 0.54782E+00_JPRB, 0.58460E+00_JPRB, 0.62255E+00_JPRB, 0.54252E+00_JPRB, 0.41956E+00_JPRB ,&
&  0.38812E+00_JPRB, 0.39931E+00_JPRB, 0.42487E+00_JPRB, 0.44032E+00_JPRB, 0.45701E+00_JPRB, 0.45984E+00_JPRB, 0.46544E+00_JPRB ,&
&  0.47595E+00_JPRB, 0.49260E+00_JPRB, 0.53213E+00_JPRB, 0.55635E+00_JPRB, 0.56170E+00_JPRB, 0.56034E+00_JPRB, 0.57412E+00_JPRB ,&
&  0.53242E+00_JPRB, 0.51791E+00_JPRB, 0.51911E+00_JPRB, 0.49349E+00_JPRB, 0.56111E+00_JPRB, 0.59192E+00_JPRB, 0.61556E+00_JPRB ,&
&  0.64104E+00_JPRB, 0.68130E+00_JPRB, 0.73409E+00_JPRB, 0.78658E+00_JPRB, 0.83686E+00_JPRB, 0.88053E+00_JPRB, 0.91528E+00_JPRB ,&
&  0.94434E+00_JPRB, 0.97723E+00_JPRB, 0.10163E+01_JPRB, 0.10957E+01_JPRB, 0.13740E+01_JPRB, 0.18571E+01_JPRB, 0.25167E+01_JPRB ,&
&  0.26671E+01_JPRB, 0.28102E+01_JPRB, 0.29508E+01_JPRB, 0.27457E+01_JPRB, 0.27745E+01_JPRB, 0.27988E+01_JPRB, 0.25974E+01_JPRB ,&
&  0.25511E+01_JPRB, 0.24659E+01_JPRB, 0.25648E+01_JPRB, 0.27550E+01_JPRB, 0.30363E+01_JPRB, 0.32714E+01_JPRB, 0.34527E+01_JPRB ,&
&  0.37411E+01_JPRB, 0.42463E+01_JPRB, 0.43999E+01_JPRB, 0.41936E+01_JPRB, 0.39688E+01_JPRB, 0.37464E+01_JPRB, 0.34198E+01_JPRB ,&
&  0.30845E+01_JPRB, 0.27176E+01_JPRB, 0.22903E+01_JPRB, 0.18585E+01_JPRB, 0.14749E+01_JPRB, 0.12594E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.82653E+00_JPRB, 0.68326E+00_JPRB, 0.66399E+00_JPRB, 0.63883E+00_JPRB, 0.58896E+00_JPRB, 0.53070E+00_JPRB, 0.49382E+00_JPRB ,&
&  0.51747E+00_JPRB, 0.58290E+00_JPRB, 0.74510E+00_JPRB, 0.71932E+00_JPRB, 0.71618E+00_JPRB, 0.70186E+00_JPRB, 0.58350E+00_JPRB ,&
&  0.55438E+00_JPRB, 0.69228E+00_JPRB, 0.65783E+00_JPRB, 0.65419E+00_JPRB, 0.63923E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.63178E+00_JPRB, 0.59833E+00_JPRB, 0.56491E+00_JPRB, 0.53514E+00_JPRB, 0.51800E+00_JPRB, 0.50083E+00_JPRB, 0.48603E+00_JPRB ,&
&  0.53371E+00_JPRB, 0.45678E+00_JPRB, 0.53137E+00_JPRB, 0.42717E+00_JPRB, 0.39468E+00_JPRB, 0.39056E+00_JPRB, 0.37759E+00_JPRB ,&
&  0.37320E+00_JPRB, 0.38444E+00_JPRB, 0.40174E+00_JPRB, 0.41763E+00_JPRB, 0.42423E+00_JPRB, 0.42843E+00_JPRB, 0.44156E+00_JPRB ,&
&  0.46166E+00_JPRB, 0.48997E+00_JPRB, 0.52809E+00_JPRB, 0.54475E+00_JPRB, 0.54529E+00_JPRB, 0.54185E+00_JPRB, 0.54776E+00_JPRB ,&
&  0.51142E+00_JPRB, 0.50414E+00_JPRB, 0.51535E+00_JPRB, 0.52285E+00_JPRB, 0.52283E+00_JPRB, 0.51636E+00_JPRB, 0.53438E+00_JPRB ,&
&  0.53207E+00_JPRB, 0.50243E+00_JPRB, 0.64028E+00_JPRB, 0.68588E+00_JPRB, 0.74033E+00_JPRB, 0.79141E+00_JPRB, 0.83985E+00_JPRB ,&
&  0.88283E+00_JPRB, 0.92692E+00_JPRB, 0.97661E+00_JPRB, 0.10275E+01_JPRB, 0.11525E+01_JPRB, 0.14318E+01_JPRB, 0.17567E+01_JPRB ,&
&  0.21149E+01_JPRB, 0.16207E+01_JPRB, 0.18253E+01_JPRB, 0.18611E+01_JPRB, 0.21427E+01_JPRB, 0.23490E+01_JPRB, 0.23401E+01_JPRB ,&
&  0.24684E+01_JPRB, 0.23685E+01_JPRB, 0.22858E+01_JPRB, 0.22217E+01_JPRB, 0.23614E+01_JPRB, 0.25766E+01_JPRB, 0.28208E+01_JPRB ,&
&  0.30726E+01_JPRB, 0.33386E+01_JPRB, 0.38354E+01_JPRB, 0.37850E+01_JPRB, 0.36578E+01_JPRB, 0.35138E+01_JPRB, 0.32856E+01_JPRB ,&
&  0.29994E+01_JPRB, 0.26588E+01_JPRB, 0.22488E+01_JPRB, 0.17933E+01_JPRB, 0.14880E+01_JPRB, 0.12328E+01_JPRB, 0.10221E+01_JPRB ,&
&  0.87652E+00_JPRB, 0.78711E+00_JPRB, 0.76392E+00_JPRB, 0.72531E+00_JPRB, 0.67728E+00_JPRB, 0.62357E+00_JPRB, 0.54910E+00_JPRB ,&
&  0.48564E+00_JPRB, 0.48447E+00_JPRB, 0.52484E+00_JPRB, 0.57361E+00_JPRB, 0.60125E+00_JPRB, 0.61919E+00_JPRB, 0.67135E+00_JPRB ,&
&  0.61875E+00_JPRB, 0.65956E+00_JPRB, 0.70439E+00_JPRB, 0.67885E+00_JPRB, 0.66082E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.68077E+00_JPRB, 0.64731E+00_JPRB, 0.61835E+00_JPRB, 0.58782E+00_JPRB, 0.56773E+00_JPRB, 0.54018E+00_JPRB, 0.48487E+00_JPRB ,&
&  0.42796E+00_JPRB, 0.40613E+00_JPRB, 0.40943E+00_JPRB, 0.39546E+00_JPRB, 0.37997E+00_JPRB, 0.37445E+00_JPRB, 0.37784E+00_JPRB ,&
&  0.39066E+00_JPRB, 0.42380E+00_JPRB, 0.45276E+00_JPRB, 0.46681E+00_JPRB, 0.46688E+00_JPRB, 0.46906E+00_JPRB, 0.47935E+00_JPRB ,&
&  0.49050E+00_JPRB, 0.50311E+00_JPRB, 0.51338E+00_JPRB, 0.50662E+00_JPRB, 0.50093E+00_JPRB, 0.50110E+00_JPRB, 0.50425E+00_JPRB ,&
&  0.51138E+00_JPRB, 0.51474E+00_JPRB, 0.50203E+00_JPRB, 0.50958E+00_JPRB, 0.49161E+00_JPRB, 0.49086E+00_JPRB, 0.46834E+00_JPRB ,&
&  0.39072E+00_JPRB, 0.40480E+00_JPRB, 0.55844E+00_JPRB, 0.61743E+00_JPRB, 0.65512E+00_JPRB, 0.71084E+00_JPRB, 0.77790E+00_JPRB ,&
&  0.83216E+00_JPRB, 0.87806E+00_JPRB, 0.92450E+00_JPRB, 0.98357E+00_JPRB, 0.10596E+01_JPRB, 0.11988E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.16557E+01_JPRB, 0.13874E+01_JPRB, 0.12537E+01_JPRB, 0.13039E+01_JPRB, 0.16629E+01_JPRB, 0.17838E+01_JPRB, 0.18034E+01_JPRB ,&
&  0.18090E+01_JPRB, 0.17394E+01_JPRB, 0.16883E+01_JPRB, 0.16730E+01_JPRB, 0.17396E+01_JPRB, 0.18758E+01_JPRB, 0.21123E+01_JPRB ,&
&  0.23182E+01_JPRB, 0.26520E+01_JPRB, 0.31203E+01_JPRB, 0.32014E+01_JPRB, 0.31260E+01_JPRB, 0.30609E+01_JPRB, 0.29349E+01_JPRB ,&
&  0.27866E+01_JPRB, 0.25049E+01_JPRB, 0.20633E+01_JPRB, 0.17002E+01_JPRB, 0.13993E+01_JPRB, 0.11725E+01_JPRB, 0.99282E+00_JPRB ,&
&  0.90295E+00_JPRB, 0.87466E+00_JPRB, 0.85385E+00_JPRB, 0.81555E+00_JPRB, 0.74965E+00_JPRB, 0.68391E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.54295E+00_JPRB, 0.49478E+00_JPRB, 0.46398E+00_JPRB, 0.47929E+00_JPRB, 0.52968E+00_JPRB, 0.58346E+00_JPRB, 0.60016E+00_JPRB ,&
&  0.61314E+00_JPRB, 0.61104E+00_JPRB, 0.60963E+00_JPRB, 0.63037E+00_JPRB, 0.66091E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.69060E+00_JPRB, 0.66628E+00_JPRB, 0.63309E+00_JPRB, 0.62514E+00_JPRB, 0.59773E+00_JPRB, 0.57393E+00_JPRB, 0.51492E+00_JPRB ,&
&  0.44351E+00_JPRB, 0.42723E+00_JPRB, 0.45251E+00_JPRB, 0.46123E+00_JPRB, 0.44632E+00_JPRB, 0.44513E+00_JPRB, 0.45536E+00_JPRB ,&
&  0.46800E+00_JPRB, 0.49968E+00_JPRB, 0.50277E+00_JPRB, 0.48689E+00_JPRB, 0.47701E+00_JPRB, 0.47246E+00_JPRB, 0.47699E+00_JPRB ,&
&  0.47884E+00_JPRB, 0.47993E+00_JPRB, 0.47561E+00_JPRB, 0.46268E+00_JPRB, 0.46436E+00_JPRB, 0.46971E+00_JPRB, 0.47269E+00_JPRB ,&
&  0.47398E+00_JPRB, 0.46856E+00_JPRB, 0.47944E+00_JPRB, 0.49342E+00_JPRB, 0.48594E+00_JPRB, 0.46678E+00_JPRB, 0.39655E+00_JPRB ,&
&  0.35553E+00_JPRB, 0.34576E+00_JPRB, 0.37844E+00_JPRB, 0.48898E+00_JPRB, 0.54892E+00_JPRB, 0.60863E+00_JPRB, 0.66962E+00_JPRB ,&
&  0.64944E+00_JPRB, 0.64737E+00_JPRB, 0.69620E+00_JPRB, 0.85161E+00_JPRB, 0.93430E+00_JPRB, 0.10267E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.11958E+01_JPRB, 0.12333E+01_JPRB, 0.12123E+01_JPRB, 0.98759E+00_JPRB, 0.10365E+01_JPRB, 0.13283E+01_JPRB, 0.13051E+01_JPRB ,&
&  0.13913E+01_JPRB, 0.14623E+01_JPRB, 0.15411E+01_JPRB, 0.14753E+01_JPRB, 0.13782E+01_JPRB, 0.13817E+01_JPRB, 0.14934E+01_JPRB ,&
&  0.16797E+01_JPRB, 0.19552E+01_JPRB, 0.24463E+01_JPRB, 0.27022E+01_JPRB, 0.26518E+01_JPRB, 0.25864E+01_JPRB, 0.25127E+01_JPRB ,&
&  0.24554E+01_JPRB, 0.21831E+01_JPRB, 0.18398E+01_JPRB, 0.15333E+01_JPRB, 0.12759E+01_JPRB, 0.10757E+01_JPRB, 0.95391E+00_JPRB ,&
&  0.90413E+00_JPRB, 0.87915E+00_JPRB, 0.87345E+00_JPRB, 0.83929E+00_JPRB, 0.76182E+00_JPRB, 0.67965E+00_JPRB, 0.61846E+00_JPRB ,&
&  0.57688E+00_JPRB, 0.53527E+00_JPRB, 0.50303E+00_JPRB, 0.49563E+00_JPRB, 0.53001E+00_JPRB, 0.57583E+00_JPRB, 0.58770E+00_JPRB ,&
&  0.57144E+00_JPRB, 0.56875E+00_JPRB, 0.60203E+00_JPRB, 0.64060E+00_JPRB, 0.65724E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.65017E+00_JPRB, 0.66273E+00_JPRB, 0.67015E+00_JPRB, 0.66060E+00_JPRB, 0.65146E+00_JPRB, 0.58945E+00_JPRB, 0.52606E+00_JPRB ,&
&  0.48332E+00_JPRB, 0.49101E+00_JPRB, 0.51561E+00_JPRB, 0.53703E+00_JPRB, 0.54040E+00_JPRB, 0.53215E+00_JPRB, 0.53222E+00_JPRB ,&
&  0.54205E+00_JPRB, 0.54228E+00_JPRB, 0.52050E+00_JPRB, 0.50314E+00_JPRB, 0.47664E+00_JPRB, 0.47122E+00_JPRB, 0.47396E+00_JPRB ,&
&  0.47762E+00_JPRB, 0.47673E+00_JPRB, 0.47647E+00_JPRB, 0.47321E+00_JPRB, 0.47490E+00_JPRB, 0.47249E+00_JPRB, 0.46658E+00_JPRB ,&
&  0.45538E+00_JPRB, 0.44738E+00_JPRB, 0.45260E+00_JPRB, 0.46994E+00_JPRB, 0.46314E+00_JPRB, 0.45608E+00_JPRB, 0.38038E+00_JPRB ,&
&  0.35104E+00_JPRB, 0.33322E+00_JPRB, 0.33142E+00_JPRB, 0.34573E+00_JPRB, 0.39387E+00_JPRB, 0.44970E+00_JPRB, 0.53200E+00_JPRB ,&
&  0.60603E+00_JPRB, 0.66264E+00_JPRB, 0.69177E+00_JPRB, 0.71590E+00_JPRB, 0.75500E+00_JPRB, 0.82227E+00_JPRB, 0.89898E+00_JPRB ,&
&  0.94235E+00_JPRB, 0.96661E+00_JPRB, 0.95695E+00_JPRB, 0.81039E+00_JPRB, 0.80796E+00_JPRB, 0.91512E+00_JPRB, 0.10618E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.12065E+01_JPRB, 0.12309E+01_JPRB, 0.13058E+01_JPRB, 0.13277E+01_JPRB, 0.13404E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.15210E+01_JPRB, 0.17512E+01_JPRB, 0.21085E+01_JPRB, 0.22860E+01_JPRB, 0.22235E+01_JPRB, 0.22386E+01_JPRB, 0.21373E+01_JPRB ,&
&  0.20435E+01_JPRB, 0.17896E+01_JPRB, 0.15694E+01_JPRB, 0.13231E+01_JPRB, 0.11320E+01_JPRB, 0.10222E+01_JPRB, 0.96004E+00_JPRB ,&
&  0.90691E+00_JPRB, 0.86505E+00_JPRB, 0.86094E+00_JPRB, 0.83360E+00_JPRB, 0.77160E+00_JPRB, 0.71718E+00_JPRB, 0.67613E+00_JPRB ,&
&  0.64600E+00_JPRB, 0.62739E+00_JPRB, 0.62142E+00_JPRB, 0.60446E+00_JPRB, 0.60275E+00_JPRB, 0.61011E+00_JPRB, 0.60847E+00_JPRB ,&
&  0.60360E+00_JPRB, 0.59966E+00_JPRB, 0.60665E+00_JPRB, 0.61769E+00_JPRB, 0.62878E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.67691E+00_JPRB, 0.68969E+00_JPRB, 0.69108E+00_JPRB, 0.69604E+00_JPRB, 0.70926E+00_JPRB, 0.66816E+00_JPRB, 0.64086E+00_JPRB ,&
&  0.62844E+00_JPRB, 0.64347E+00_JPRB, 0.64548E+00_JPRB, 0.64322E+00_JPRB, 0.63264E+00_JPRB, 0.62229E+00_JPRB, 0.61482E+00_JPRB ,&
&  0.60803E+00_JPRB, 0.58938E+00_JPRB, 0.55468E+00_JPRB, 0.50175E+00_JPRB, 0.48558E+00_JPRB, 0.48127E+00_JPRB, 0.48506E+00_JPRB ,&
&  0.48736E+00_JPRB, 0.48221E+00_JPRB, 0.48212E+00_JPRB, 0.47594E+00_JPRB, 0.46996E+00_JPRB, 0.45190E+00_JPRB, 0.43597E+00_JPRB ,&
&  0.43850E+00_JPRB, 0.43929E+00_JPRB, 0.45944E+00_JPRB, 0.44571E+00_JPRB, 0.43196E+00_JPRB, 0.42591E+00_JPRB, 0.36185E+00_JPRB ,&
&  0.34976E+00_JPRB, 0.33425E+00_JPRB, 0.32918E+00_JPRB, 0.32956E+00_JPRB, 0.34836E+00_JPRB, 0.36787E+00_JPRB, 0.40874E+00_JPRB ,&
&  0.46856E+00_JPRB, 0.57083E+00_JPRB, 0.61675E+00_JPRB, 0.66291E+00_JPRB, 0.70157E+00_JPRB, 0.74283E+00_JPRB, 0.78719E+00_JPRB ,&
&  0.82601E+00_JPRB, 0.86651E+00_JPRB, 0.90427E+00_JPRB, 0.96848E+00_JPRB, 0.10290E+01_JPRB, 0.99835E+00_JPRB, 0.10616E+01_JPRB ,&
&  0.13437E+01_JPRB, 0.12902E+01_JPRB, 0.12336E+01_JPRB, 0.11987E+01_JPRB, 0.12156E+01_JPRB, 0.12472E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.14193E+01_JPRB, 0.16115E+01_JPRB, 0.18927E+01_JPRB, 0.20797E+01_JPRB, 0.20743E+01_JPRB, 0.20938E+01_JPRB, 0.20565E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.18054E+01_JPRB, 0.19076E+01_JPRB, 0.15256E+01_JPRB, 0.12733E+01_JPRB, 0.11338E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.96768E+00_JPRB, 0.93512E+00_JPRB, 0.88891E+00_JPRB, 0.82606E+00_JPRB, 0.78890E+00_JPRB, 0.78047E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.76099E+00_JPRB, 0.74979E+00_JPRB, 0.73419E+00_JPRB, 0.70992E+00_JPRB, 0.68238E+00_JPRB, 0.66871E+00_JPRB ,&
&  0.66405E+00_JPRB, 0.65937E+00_JPRB, 0.65631E+00_JPRB, 0.65568E+00_JPRB, 0.66171E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.68846E+00_JPRB, 0.70529E+00_JPRB, 0.70771E+00_JPRB, 0.70885E+00_JPRB, 0.71072E+00_JPRB, 0.69552E+00_JPRB, 0.68316E+00_JPRB ,&
&  0.67610E+00_JPRB, 0.66909E+00_JPRB, 0.65884E+00_JPRB, 0.64740E+00_JPRB, 0.63463E+00_JPRB, 0.62636E+00_JPRB, 0.61835E+00_JPRB ,&
&  0.61016E+00_JPRB, 0.56524E+00_JPRB, 0.54123E+00_JPRB, 0.52354E+00_JPRB, 0.50755E+00_JPRB, 0.49998E+00_JPRB, 0.49553E+00_JPRB ,&
&  0.48760E+00_JPRB, 0.48280E+00_JPRB, 0.48145E+00_JPRB, 0.47663E+00_JPRB, 0.46865E+00_JPRB, 0.45571E+00_JPRB, 0.45944E+00_JPRB ,&
&  0.47775E+00_JPRB, 0.46798E+00_JPRB, 0.45216E+00_JPRB, 0.43541E+00_JPRB, 0.41017E+00_JPRB, 0.38545E+00_JPRB, 0.36776E+00_JPRB ,&
&  0.35813E+00_JPRB, 0.33700E+00_JPRB, 0.33850E+00_JPRB, 0.34189E+00_JPRB, 0.34596E+00_JPRB, 0.36357E+00_JPRB, 0.40090E+00_JPRB ,&
&  0.44506E+00_JPRB, 0.55448E+00_JPRB, 0.59777E+00_JPRB, 0.64871E+00_JPRB, 0.68547E+00_JPRB, 0.71464E+00_JPRB, 0.73987E+00_JPRB ,&
&  0.76162E+00_JPRB, 0.79330E+00_JPRB, 0.83026E+00_JPRB, 0.88278E+00_JPRB, 0.94038E+00_JPRB, 0.98067E+00_JPRB, 0.10132E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.10495E+01_JPRB, 0.10612E+01_JPRB, 0.10597E+01_JPRB, 0.10791E+01_JPRB, 0.11412E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.13204E+01_JPRB, 0.14861E+01_JPRB, 0.16503E+01_JPRB, 0.17282E+01_JPRB, 0.18092E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.17368E+01_JPRB, 0.16829E+01_JPRB, 0.17194E+01_JPRB, 0.15717E+01_JPRB, 0.14118E+01_JPRB, 0.12818E+01_JPRB, 0.11892E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.10137E+01_JPRB, 0.95481E+00_JPRB, 0.90555E+00_JPRB, 0.85438E+00_JPRB, 0.80833E+00_JPRB, 0.79672E+00_JPRB ,&
&  0.78670E+00_JPRB, 0.77470E+00_JPRB, 0.75840E+00_JPRB, 0.73949E+00_JPRB, 0.70926E+00_JPRB, 0.67908E+00_JPRB, 0.65427E+00_JPRB ,&
&  0.64736E+00_JPRB, 0.64499E+00_JPRB, 0.65216E+00_JPRB, 0.65880E+00_JPRB, 0.67081E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.68968E+00_JPRB, 0.69300E+00_JPRB, 0.68660E+00_JPRB, 0.67857E+00_JPRB, 0.67360E+00_JPRB, 0.66269E+00_JPRB, 0.65820E+00_JPRB ,&
&  0.65561E+00_JPRB, 0.65279E+00_JPRB, 0.64238E+00_JPRB, 0.63007E+00_JPRB, 0.61389E+00_JPRB, 0.60274E+00_JPRB, 0.58952E+00_JPRB ,&
&  0.57844E+00_JPRB, 0.56428E+00_JPRB, 0.54948E+00_JPRB, 0.53476E+00_JPRB, 0.52347E+00_JPRB, 0.51108E+00_JPRB, 0.50201E+00_JPRB ,&
&  0.49353E+00_JPRB, 0.48133E+00_JPRB, 0.47008E+00_JPRB, 0.46914E+00_JPRB, 0.46386E+00_JPRB, 0.46056E+00_JPRB, 0.46674E+00_JPRB ,&
&  0.47209E+00_JPRB, 0.46835E+00_JPRB, 0.46429E+00_JPRB, 0.44505E+00_JPRB, 0.41330E+00_JPRB, 0.34633E+00_JPRB, 0.35541E+00_JPRB ,&
&  0.35013E+00_JPRB, 0.34076E+00_JPRB, 0.34450E+00_JPRB, 0.35415E+00_JPRB, 0.36428E+00_JPRB, 0.39213E+00_JPRB, 0.40094E+00_JPRB ,&
&  0.40044E+00_JPRB, 0.54047E+00_JPRB, 0.57490E+00_JPRB, 0.61053E+00_JPRB, 0.64012E+00_JPRB, 0.66412E+00_JPRB, 0.68726E+00_JPRB ,&
&  0.70660E+00_JPRB, 0.73357E+00_JPRB, 0.75923E+00_JPRB, 0.80132E+00_JPRB, 0.84736E+00_JPRB, 0.90352E+00_JPRB, 0.94961E+00_JPRB ,&
&  0.97006E+00_JPRB, 0.98101E+00_JPRB, 0.99905E+00_JPRB, 0.10136E+01_JPRB, 0.10369E+01_JPRB, 0.10793E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.12135E+01_JPRB, 0.12543E+01_JPRB, 0.13571E+01_JPRB, 0.14371E+01_JPRB, 0.14849E+01_JPRB, 0.15443E+01_JPRB, 0.15957E+01_JPRB ,&
&  0.15731E+01_JPRB, 0.15672E+01_JPRB, 0.15332E+01_JPRB, 0.14127E+01_JPRB, 0.12793E+01_JPRB, 0.11889E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.97385E+00_JPRB, 0.90671E+00_JPRB, 0.85190E+00_JPRB, 0.80804E+00_JPRB, 0.77415E+00_JPRB, 0.75424E+00_JPRB ,&
&  0.73384E+00_JPRB, 0.71627E+00_JPRB, 0.70020E+00_JPRB, 0.69235E+00_JPRB, 0.67931E+00_JPRB, 0.66953E+00_JPRB, 0.66126E+00_JPRB ,&
&  0.66021E+00_JPRB, 0.66115E+00_JPRB, 0.66696E+00_JPRB, 0.67377E+00_JPRB, 0.67847E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.68278E+00_JPRB, 0.68446E+00_JPRB, 0.68446E+00_JPRB, 0.67470E+00_JPRB, 0.66985E+00_JPRB, 0.66430E+00_JPRB, 0.66650E+00_JPRB ,&
&  0.65911E+00_JPRB, 0.65202E+00_JPRB, 0.63192E+00_JPRB, 0.61889E+00_JPRB, 0.60429E+00_JPRB, 0.59654E+00_JPRB, 0.58919E+00_JPRB ,&
&  0.58481E+00_JPRB, 0.57818E+00_JPRB, 0.57480E+00_JPRB, 0.56468E+00_JPRB, 0.55538E+00_JPRB, 0.53968E+00_JPRB, 0.52251E+00_JPRB ,&
&  0.50495E+00_JPRB, 0.48677E+00_JPRB, 0.46926E+00_JPRB, 0.46872E+00_JPRB, 0.46203E+00_JPRB, 0.45887E+00_JPRB, 0.45710E+00_JPRB ,&
&  0.46221E+00_JPRB, 0.41518E+00_JPRB, 0.42082E+00_JPRB, 0.40366E+00_JPRB, 0.38594E+00_JPRB, 0.36739E+00_JPRB, 0.35406E+00_JPRB ,&
&  0.35674E+00_JPRB, 0.35870E+00_JPRB, 0.36982E+00_JPRB, 0.38922E+00_JPRB, 0.39945E+00_JPRB, 0.41830E+00_JPRB, 0.43333E+00_JPRB ,&
&  0.43772E+00_JPRB, 0.53615E+00_JPRB, 0.58628E+00_JPRB, 0.61655E+00_JPRB, 0.64327E+00_JPRB, 0.66669E+00_JPRB, 0.68509E+00_JPRB ,&
&  0.71075E+00_JPRB, 0.73723E+00_JPRB, 0.76383E+00_JPRB, 0.78927E+00_JPRB, 0.83534E+00_JPRB, 0.86758E+00_JPRB, 0.89419E+00_JPRB ,&
&  0.91068E+00_JPRB, 0.91465E+00_JPRB, 0.92736E+00_JPRB, 0.95480E+00_JPRB, 0.99370E+00_JPRB, 0.10360E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.11742E+01_JPRB, 0.12272E+01_JPRB, 0.12764E+01_JPRB, 0.13198E+01_JPRB, 0.13484E+01_JPRB, 0.13719E+01_JPRB ,&
&  0.13559E+01_JPRB, 0.13322E+01_JPRB, 0.12902E+01_JPRB, 0.12295E+01_JPRB, 0.11530E+01_JPRB, 0.10694E+01_JPRB, 0.98647E+00_JPRB ,&
&  0.93967E+00_JPRB, 0.88853E+00_JPRB, 0.84168E+00_JPRB, 0.79515E+00_JPRB, 0.75670E+00_JPRB, 0.72713E+00_JPRB, 0.71040E+00_JPRB ,&
&  0.69390E+00_JPRB, 0.68285E+00_JPRB, 0.67537E+00_JPRB, 0.67116E+00_JPRB, 0.67087E+00_JPRB, 0.66837E+00_JPRB, 0.67092E+00_JPRB ,&
&  0.67444E+00_JPRB, 0.67867E+00_JPRB, 0.67776E+00_JPRB, 0.67796E+00_JPRB, 0.67954E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.67995E+00_JPRB, 0.68348E+00_JPRB, 0.69232E+00_JPRB, 0.69378E+00_JPRB, 0.69429E+00_JPRB, 0.68914E+00_JPRB, 0.68778E+00_JPRB ,&
&  0.67953E+00_JPRB, 0.67040E+00_JPRB, 0.64950E+00_JPRB, 0.63493E+00_JPRB, 0.62097E+00_JPRB, 0.61301E+00_JPRB, 0.60713E+00_JPRB ,&
&  0.60343E+00_JPRB, 0.59802E+00_JPRB, 0.59161E+00_JPRB, 0.58161E+00_JPRB, 0.56877E+00_JPRB, 0.55307E+00_JPRB, 0.53191E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.49954E+00_JPRB, 0.47942E+00_JPRB, 0.48208E+00_JPRB, 0.47272E+00_JPRB, 0.46646E+00_JPRB, 0.46355E+00_JPRB ,&
&  0.45415E+00_JPRB, 0.44299E+00_JPRB, 0.42645E+00_JPRB, 0.41446E+00_JPRB, 0.40363E+00_JPRB, 0.39165E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.40970E+00_JPRB, 0.43617E+00_JPRB, 0.44971E+00_JPRB, 0.45640E+00_JPRB, 0.46332E+00_JPRB, 0.46619E+00_JPRB, 0.47800E+00_JPRB ,&
&  0.49516E+00_JPRB, 0.51969E+00_JPRB, 0.58361E+00_JPRB, 0.62239E+00_JPRB, 0.64428E+00_JPRB, 0.66756E+00_JPRB, 0.68443E+00_JPRB ,&
&  0.70672E+00_JPRB, 0.72041E+00_JPRB, 0.72932E+00_JPRB, 0.74168E+00_JPRB, 0.76472E+00_JPRB, 0.79307E+00_JPRB, 0.81668E+00_JPRB ,&
&  0.83555E+00_JPRB, 0.84594E+00_JPRB, 0.86101E+00_JPRB, 0.87883E+00_JPRB, 0.90654E+00_JPRB, 0.93543E+00_JPRB, 0.96588E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.10256E+01_JPRB, 0.10470E+01_JPRB, 0.10679E+01_JPRB, 0.10782E+01_JPRB, 0.10831E+01_JPRB, 0.10848E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.10502E+01_JPRB, 0.10196E+01_JPRB, 0.97056E+00_JPRB, 0.93305E+00_JPRB, 0.91098E+00_JPRB, 0.87405E+00_JPRB ,&
&  0.84247E+00_JPRB, 0.81108E+00_JPRB, 0.78226E+00_JPRB, 0.75275E+00_JPRB, 0.72839E+00_JPRB, 0.70627E+00_JPRB, 0.69165E+00_JPRB ,&
&  0.68070E+00_JPRB, 0.67411E+00_JPRB, 0.67167E+00_JPRB, 0.67152E+00_JPRB, 0.67567E+00_JPRB, 0.67822E+00_JPRB, 0.68521E+00_JPRB ,&
&  0.68747E+00_JPRB, 0.68669E+00_JPRB, 0.68153E+00_JPRB, 0.67866E+00_JPRB, 0.67713E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.68570E+00_JPRB, 0.68579E+00_JPRB, 0.68847E+00_JPRB, 0.68989E+00_JPRB, 0.69145E+00_JPRB, 0.69016E+00_JPRB, 0.68911E+00_JPRB ,&
&  0.68463E+00_JPRB, 0.67591E+00_JPRB, 0.66073E+00_JPRB, 0.64718E+00_JPRB, 0.63492E+00_JPRB, 0.62660E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.61335E+00_JPRB, 0.60820E+00_JPRB, 0.60215E+00_JPRB, 0.59545E+00_JPRB, 0.58673E+00_JPRB, 0.57677E+00_JPRB, 0.56436E+00_JPRB ,&
&  0.55494E+00_JPRB, 0.54567E+00_JPRB, 0.53795E+00_JPRB, 0.53244E+00_JPRB, 0.53013E+00_JPRB, 0.50403E+00_JPRB, 0.49818E+00_JPRB ,&
&  0.50404E+00_JPRB, 0.49356E+00_JPRB, 0.48477E+00_JPRB, 0.47135E+00_JPRB, 0.47413E+00_JPRB, 0.48374E+00_JPRB, 0.49025E+00_JPRB ,&
&  0.47635E+00_JPRB, 0.50247E+00_JPRB, 0.50567E+00_JPRB, 0.50750E+00_JPRB, 0.50655E+00_JPRB, 0.50372E+00_JPRB, 0.51872E+00_JPRB ,&
&  0.55358E+00_JPRB, 0.56993E+00_JPRB, 0.59898E+00_JPRB, 0.63124E+00_JPRB, 0.65387E+00_JPRB, 0.67737E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.72157E+00_JPRB, 0.73636E+00_JPRB, 0.74547E+00_JPRB, 0.75414E+00_JPRB, 0.76906E+00_JPRB, 0.78459E+00_JPRB, 0.79785E+00_JPRB ,&
&  0.80639E+00_JPRB, 0.81000E+00_JPRB, 0.81505E+00_JPRB, 0.82000E+00_JPRB, 0.82856E+00_JPRB, 0.83477E+00_JPRB, 0.84065E+00_JPRB ,&
&  0.84363E+00_JPRB, 0.84609E+00_JPRB, 0.84577E+00_JPRB, 0.84370E+00_JPRB, 0.83993E+00_JPRB, 0.83398E+00_JPRB, 0.83231E+00_JPRB ,&
&  0.82457E+00_JPRB, 0.81529E+00_JPRB, 0.80300E+00_JPRB, 0.78704E+00_JPRB, 0.77387E+00_JPRB, 0.76372E+00_JPRB, 0.75310E+00_JPRB ,&
&  0.74262E+00_JPRB, 0.73251E+00_JPRB, 0.72318E+00_JPRB, 0.71361E+00_JPRB, 0.70452E+00_JPRB, 0.69559E+00_JPRB, 0.68918E+00_JPRB ,&
&  0.68621E+00_JPRB, 0.68409E+00_JPRB, 0.68522E+00_JPRB, 0.68536E+00_JPRB, 0.68894E+00_JPRB, 0.69001E+00_JPRB, 0.69324E+00_JPRB ,&
&  0.69462E+00_JPRB, 0.69324E+00_JPRB, 0.68940E+00_JPRB, 0.68671E+00_JPRB, 0.68471E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.68082E+00_JPRB, 0.67912E+00_JPRB, 0.67803E+00_JPRB, 0.67823E+00_JPRB, 0.67923E+00_JPRB, 0.67953E+00_JPRB, 0.67900E+00_JPRB ,&
&  0.67665E+00_JPRB, 0.67053E+00_JPRB, 0.66347E+00_JPRB, 0.65584E+00_JPRB, 0.64709E+00_JPRB, 0.63886E+00_JPRB, 0.63374E+00_JPRB ,&
&  0.62845E+00_JPRB, 0.62327E+00_JPRB, 0.61786E+00_JPRB, 0.61397E+00_JPRB, 0.60949E+00_JPRB, 0.60483E+00_JPRB, 0.60042E+00_JPRB ,&
&  0.59754E+00_JPRB, 0.59415E+00_JPRB, 0.59040E+00_JPRB, 0.58585E+00_JPRB, 0.58250E+00_JPRB, 0.57864E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.57074E+00_JPRB, 0.56882E+00_JPRB, 0.56757E+00_JPRB, 0.56729E+00_JPRB, 0.56961E+00_JPRB, 0.57282E+00_JPRB, 0.57392E+00_JPRB ,&
&  0.57391E+00_JPRB, 0.57141E+00_JPRB, 0.57297E+00_JPRB, 0.57525E+00_JPRB, 0.57939E+00_JPRB, 0.58617E+00_JPRB, 0.59555E+00_JPRB ,&
&  0.60800E+00_JPRB, 0.62259E+00_JPRB, 0.63790E+00_JPRB, 0.64985E+00_JPRB, 0.66170E+00_JPRB, 0.67239E+00_JPRB, 0.67959E+00_JPRB ,&
&  0.68764E+00_JPRB, 0.69327E+00_JPRB, 0.69943E+00_JPRB, 0.70443E+00_JPRB, 0.71157E+00_JPRB, 0.71726E+00_JPRB, 0.72205E+00_JPRB ,&
&  0.72333E+00_JPRB, 0.72180E+00_JPRB, 0.71980E+00_JPRB, 0.71809E+00_JPRB, 0.71513E+00_JPRB, 0.71270E+00_JPRB, 0.71105E+00_JPRB ,&
&  0.71003E+00_JPRB, 0.70899E+00_JPRB, 0.70925E+00_JPRB, 0.71000E+00_JPRB, 0.71098E+00_JPRB, 0.71054E+00_JPRB, 0.71098E+00_JPRB ,&
&  0.70963E+00_JPRB, 0.70808E+00_JPRB, 0.70521E+00_JPRB, 0.70059E+00_JPRB, 0.69673E+00_JPRB, 0.69411E+00_JPRB, 0.69119E+00_JPRB ,&
&  0.68832E+00_JPRB, 0.68609E+00_JPRB, 0.68464E+00_JPRB, 0.68342E+00_JPRB, 0.68123E+00_JPRB, 0.67938E+00_JPRB, 0.67836E+00_JPRB ,&
&  0.67816E+00_JPRB, 0.67749E+00_JPRB, 0.67849E+00_JPRB, 0.67967E+00_JPRB, 0.68402E+00_JPRB, 0.68596E+00_JPRB, 0.68758E+00_JPRB ,&
&  0.68802E+00_JPRB, 0.68918E+00_JPRB, 0.68833E+00_JPRB, 0.68585E+00_JPRB, 0.68287E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.68150E+00_JPRB, 0.67989E+00_JPRB, 0.67837E+00_JPRB, 0.67726E+00_JPRB, 0.67668E+00_JPRB, 0.67637E+00_JPRB, 0.67494E+00_JPRB ,&
&  0.67311E+00_JPRB, 0.67045E+00_JPRB, 0.66769E+00_JPRB, 0.66436E+00_JPRB, 0.66034E+00_JPRB, 0.65512E+00_JPRB, 0.65068E+00_JPRB ,&
&  0.64667E+00_JPRB, 0.64345E+00_JPRB, 0.63996E+00_JPRB, 0.63759E+00_JPRB, 0.63522E+00_JPRB, 0.63321E+00_JPRB, 0.63105E+00_JPRB ,&
&  0.62935E+00_JPRB, 0.62762E+00_JPRB, 0.62597E+00_JPRB, 0.62448E+00_JPRB, 0.62390E+00_JPRB, 0.62310E+00_JPRB, 0.62219E+00_JPRB ,&
&  0.62143E+00_JPRB, 0.62100E+00_JPRB, 0.62061E+00_JPRB, 0.62032E+00_JPRB, 0.62023E+00_JPRB, 0.62067E+00_JPRB, 0.62109E+00_JPRB ,&
&  0.62182E+00_JPRB, 0.62336E+00_JPRB, 0.62553E+00_JPRB, 0.62754E+00_JPRB, 0.62978E+00_JPRB, 0.63280E+00_JPRB, 0.63648E+00_JPRB ,&
&  0.63978E+00_JPRB, 0.64276E+00_JPRB, 0.64598E+00_JPRB, 0.64962E+00_JPRB, 0.65296E+00_JPRB, 0.65633E+00_JPRB, 0.65916E+00_JPRB ,&
&  0.66181E+00_JPRB, 0.66417E+00_JPRB, 0.66674E+00_JPRB, 0.66762E+00_JPRB, 0.66945E+00_JPRB, 0.67102E+00_JPRB, 0.67305E+00_JPRB ,&
&  0.67356E+00_JPRB, 0.67386E+00_JPRB, 0.67448E+00_JPRB, 0.67532E+00_JPRB, 0.67595E+00_JPRB, 0.67662E+00_JPRB, 0.67741E+00_JPRB ,&
&  0.67857E+00_JPRB, 0.67932E+00_JPRB, 0.68032E+00_JPRB, 0.68121E+00_JPRB, 0.68235E+00_JPRB, 0.68306E+00_JPRB, 0.68398E+00_JPRB ,&
&  0.68430E+00_JPRB, 0.68465E+00_JPRB, 0.68447E+00_JPRB, 0.68416E+00_JPRB, 0.68395E+00_JPRB, 0.68397E+00_JPRB, 0.68412E+00_JPRB ,&
&  0.68437E+00_JPRB, 0.68451E+00_JPRB, 0.68457E+00_JPRB, 0.68432E+00_JPRB, 0.68354E+00_JPRB, 0.68287E+00_JPRB, 0.68246E+00_JPRB ,&
&  0.68260E+00_JPRB, 0.68259E+00_JPRB, 0.68298E+00_JPRB, 0.68360E+00_JPRB, 0.68513E+00_JPRB, 0.68569E+00_JPRB, 0.68621E+00_JPRB ,&
&  0.68618E+00_JPRB, 0.68613E+00_JPRB, 0.68535E+00_JPRB, 0.68426E+00_JPRB, 0.68284E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB, 0.66399E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB, 0.12978E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.13164E+00_JPRB, 0.13230E+00_JPRB, 0.13279E+00_JPRB, 0.13319E+00_JPRB, 0.13308E+00_JPRB, 0.13316E+00_JPRB, 0.13325E+00_JPRB ,&
&  0.13328E+00_JPRB, 0.13329E+00_JPRB, 0.13352E+00_JPRB, 0.13374E+00_JPRB, 0.13397E+00_JPRB, 0.13422E+00_JPRB, 0.13448E+00_JPRB ,&
&  0.13460E+00_JPRB, 0.13484E+00_JPRB, 0.13392E+00_JPRB, 0.13320E+00_JPRB, 0.13277E+00_JPRB, 0.13253E+00_JPRB, 0.13242E+00_JPRB ,&
&  0.13232E+00_JPRB, 0.13227E+00_JPRB, 0.13218E+00_JPRB, 0.13287E+00_JPRB, 0.13332E+00_JPRB, 0.13373E+00_JPRB, 0.13406E+00_JPRB ,&
&  0.13421E+00_JPRB, 0.13440E+00_JPRB, 0.13456E+00_JPRB, 0.13473E+00_JPRB, 0.13491E+00_JPRB, 0.13488E+00_JPRB, 0.13476E+00_JPRB ,&
&  0.13457E+00_JPRB, 0.13429E+00_JPRB, 0.13395E+00_JPRB, 0.13364E+00_JPRB, 0.13338E+00_JPRB, 0.13310E+00_JPRB, 0.13279E+00_JPRB ,&
&  0.13248E+00_JPRB, 0.13212E+00_JPRB, 0.13178E+00_JPRB, 0.13145E+00_JPRB, 0.13114E+00_JPRB, 0.13088E+00_JPRB, 0.13058E+00_JPRB ,&
&  0.13037E+00_JPRB, 0.13018E+00_JPRB, 0.13000E+00_JPRB, 0.12985E+00_JPRB, 0.12965E+00_JPRB, 0.12956E+00_JPRB, 0.12938E+00_JPRB ,&
&  0.12930E+00_JPRB, 0.12922E+00_JPRB, 0.12915E+00_JPRB, 0.12907E+00_JPRB, 0.12905E+00_JPRB, 0.12901E+00_JPRB, 0.12898E+00_JPRB ,&
&  0.12894E+00_JPRB, 0.12902E+00_JPRB, 0.12890E+00_JPRB, 0.12876E+00_JPRB, 0.12860E+00_JPRB, 0.12844E+00_JPRB, 0.12829E+00_JPRB ,&
&  0.12816E+00_JPRB, 0.12803E+00_JPRB, 0.12794E+00_JPRB, 0.12794E+00_JPRB, 0.12795E+00_JPRB, 0.12798E+00_JPRB, 0.12803E+00_JPRB ,&
&  0.12809E+00_JPRB, 0.12818E+00_JPRB, 0.12826E+00_JPRB, 0.12823E+00_JPRB, 0.12826E+00_JPRB, 0.12827E+00_JPRB, 0.12829E+00_JPRB ,&
&  0.12832E+00_JPRB, 0.12836E+00_JPRB, 0.12844E+00_JPRB, 0.12853E+00_JPRB, 0.12873E+00_JPRB, 0.12882E+00_JPRB, 0.12895E+00_JPRB ,&
&  0.12914E+00_JPRB, 0.12947E+00_JPRB, 0.12976E+00_JPRB, 0.13007E+00_JPRB, 0.13046E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.14040E+00_JPRB, 0.14225E+00_JPRB, 0.14376E+00_JPRB, 0.14574E+00_JPRB, 0.14281E+00_JPRB, 0.14985E+00_JPRB, 0.14759E+00_JPRB ,&
&  0.14522E+00_JPRB, 0.14054E+00_JPRB, 0.14041E+00_JPRB, 0.13945E+00_JPRB, 0.13821E+00_JPRB, 0.13741E+00_JPRB, 0.13703E+00_JPRB ,&
&  0.13685E+00_JPRB, 0.13648E+00_JPRB, 0.13666E+00_JPRB, 0.13679E+00_JPRB, 0.13689E+00_JPRB, 0.13694E+00_JPRB, 0.13711E+00_JPRB ,&
&  0.13751E+00_JPRB, 0.13828E+00_JPRB, 0.13910E+00_JPRB, 0.13774E+00_JPRB, 0.13769E+00_JPRB, 0.13806E+00_JPRB, 0.13874E+00_JPRB ,&
&  0.13937E+00_JPRB, 0.14041E+00_JPRB, 0.14116E+00_JPRB, 0.14166E+00_JPRB, 0.14292E+00_JPRB, 0.14290E+00_JPRB, 0.14270E+00_JPRB ,&
&  0.14231E+00_JPRB, 0.14159E+00_JPRB, 0.14053E+00_JPRB, 0.13955E+00_JPRB, 0.13853E+00_JPRB, 0.13723E+00_JPRB, 0.13597E+00_JPRB ,&
&  0.13494E+00_JPRB, 0.13408E+00_JPRB, 0.13302E+00_JPRB, 0.13199E+00_JPRB, 0.13105E+00_JPRB, 0.13013E+00_JPRB, 0.12933E+00_JPRB ,&
&  0.12875E+00_JPRB, 0.12827E+00_JPRB, 0.12777E+00_JPRB, 0.12737E+00_JPRB, 0.12696E+00_JPRB, 0.12657E+00_JPRB, 0.12621E+00_JPRB ,&
&  0.12630E+00_JPRB, 0.12638E+00_JPRB, 0.12645E+00_JPRB, 0.12650E+00_JPRB, 0.12678E+00_JPRB, 0.12681E+00_JPRB, 0.12686E+00_JPRB ,&
&  0.12686E+00_JPRB, 0.12675E+00_JPRB, 0.12665E+00_JPRB, 0.12648E+00_JPRB, 0.12628E+00_JPRB, 0.12604E+00_JPRB, 0.12593E+00_JPRB ,&
&  0.12594E+00_JPRB, 0.12604E+00_JPRB, 0.12632E+00_JPRB, 0.12650E+00_JPRB, 0.12668E+00_JPRB, 0.12683E+00_JPRB, 0.12720E+00_JPRB ,&
&  0.12757E+00_JPRB, 0.12796E+00_JPRB, 0.12832E+00_JPRB, 0.12827E+00_JPRB, 0.12849E+00_JPRB, 0.12869E+00_JPRB, 0.12889E+00_JPRB ,&
&  0.12902E+00_JPRB, 0.12944E+00_JPRB, 0.13007E+00_JPRB, 0.13090E+00_JPRB, 0.13240E+00_JPRB, 0.13373E+00_JPRB, 0.13496E+00_JPRB ,&
&  0.13628E+00_JPRB, 0.13889E+00_JPRB, 0.14007E+00_JPRB, 0.14059E+00_JPRB, 0.14081E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.15035E+00_JPRB, 0.15212E+00_JPRB, 0.15322E+00_JPRB, 0.15400E+00_JPRB, 0.15503E+00_JPRB, 0.15547E+00_JPRB, 0.15588E+00_JPRB ,&
&  0.15412E+00_JPRB, 0.15346E+00_JPRB, 0.15238E+00_JPRB, 0.15106E+00_JPRB, 0.15006E+00_JPRB, 0.14796E+00_JPRB, 0.14581E+00_JPRB ,&
&  0.14337E+00_JPRB, 0.14117E+00_JPRB, 0.13969E+00_JPRB, 0.13826E+00_JPRB, 0.13687E+00_JPRB, 0.13571E+00_JPRB, 0.13701E+00_JPRB ,&
&  0.13777E+00_JPRB, 0.13828E+00_JPRB, 0.13935E+00_JPRB, 0.13949E+00_JPRB, 0.14076E+00_JPRB, 0.14274E+00_JPRB, 0.14537E+00_JPRB ,&
&  0.14736E+00_JPRB, 0.15013E+00_JPRB, 0.15179E+00_JPRB, 0.15265E+00_JPRB, 0.15374E+00_JPRB, 0.15446E+00_JPRB, 0.15413E+00_JPRB ,&
&  0.15296E+00_JPRB, 0.15137E+00_JPRB, 0.14957E+00_JPRB, 0.14729E+00_JPRB, 0.14467E+00_JPRB, 0.14209E+00_JPRB, 0.13953E+00_JPRB ,&
&  0.13760E+00_JPRB, 0.13585E+00_JPRB, 0.13406E+00_JPRB, 0.13246E+00_JPRB, 0.13095E+00_JPRB, 0.12958E+00_JPRB, 0.12821E+00_JPRB ,&
&  0.12712E+00_JPRB, 0.12624E+00_JPRB, 0.12519E+00_JPRB, 0.12456E+00_JPRB, 0.12394E+00_JPRB, 0.12363E+00_JPRB, 0.12345E+00_JPRB ,&
&  0.12370E+00_JPRB, 0.12405E+00_JPRB, 0.12442E+00_JPRB, 0.12461E+00_JPRB, 0.12479E+00_JPRB, 0.12487E+00_JPRB, 0.12503E+00_JPRB ,&
&  0.12518E+00_JPRB, 0.12537E+00_JPRB, 0.12503E+00_JPRB, 0.12473E+00_JPRB, 0.12433E+00_JPRB, 0.12389E+00_JPRB, 0.12377E+00_JPRB ,&
&  0.12403E+00_JPRB, 0.12426E+00_JPRB, 0.12418E+00_JPRB, 0.12444E+00_JPRB, 0.12487E+00_JPRB, 0.12532E+00_JPRB, 0.12640E+00_JPRB ,&
&  0.12699E+00_JPRB, 0.12756E+00_JPRB, 0.12810E+00_JPRB, 0.12854E+00_JPRB, 0.12919E+00_JPRB, 0.12974E+00_JPRB, 0.13042E+00_JPRB ,&
&  0.13122E+00_JPRB, 0.13205E+00_JPRB, 0.13317E+00_JPRB, 0.13456E+00_JPRB, 0.13778E+00_JPRB, 0.14007E+00_JPRB, 0.14220E+00_JPRB ,&
&  0.14350E+00_JPRB, 0.14138E+00_JPRB, 0.14859E+00_JPRB, 0.14822E+00_JPRB, 0.14824E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.15594E+00_JPRB, 0.15834E+00_JPRB, 0.15727E+00_JPRB, 0.15724E+00_JPRB, 0.15686E+00_JPRB, 0.15686E+00_JPRB, 0.15605E+00_JPRB ,&
&  0.15583E+00_JPRB, 0.15370E+00_JPRB, 0.15314E+00_JPRB, 0.15142E+00_JPRB, 0.14960E+00_JPRB, 0.14720E+00_JPRB, 0.14482E+00_JPRB ,&
&  0.14239E+00_JPRB, 0.14022E+00_JPRB, 0.13870E+00_JPRB, 0.13787E+00_JPRB, 0.13661E+00_JPRB, 0.13636E+00_JPRB, 0.13774E+00_JPRB ,&
&  0.13805E+00_JPRB, 0.13899E+00_JPRB, 0.13919E+00_JPRB, 0.14044E+00_JPRB, 0.14206E+00_JPRB, 0.14307E+00_JPRB, 0.14546E+00_JPRB ,&
&  0.14733E+00_JPRB, 0.14879E+00_JPRB, 0.15376E+00_JPRB, 0.15670E+00_JPRB, 0.15850E+00_JPRB, 0.16099E+00_JPRB, 0.16071E+00_JPRB ,&
&  0.15959E+00_JPRB, 0.15665E+00_JPRB, 0.15998E+00_JPRB, 0.15838E+00_JPRB, 0.15232E+00_JPRB, 0.14934E+00_JPRB, 0.14427E+00_JPRB ,&
&  0.14097E+00_JPRB, 0.13735E+00_JPRB, 0.13460E+00_JPRB, 0.13251E+00_JPRB, 0.12995E+00_JPRB, 0.12827E+00_JPRB, 0.12640E+00_JPRB ,&
&  0.12461E+00_JPRB, 0.12374E+00_JPRB, 0.12246E+00_JPRB, 0.12219E+00_JPRB, 0.12220E+00_JPRB, 0.12247E+00_JPRB, 0.12297E+00_JPRB ,&
&  0.12367E+00_JPRB, 0.12435E+00_JPRB, 0.12498E+00_JPRB, 0.12552E+00_JPRB, 0.12598E+00_JPRB, 0.12628E+00_JPRB, 0.12620E+00_JPRB ,&
&  0.12629E+00_JPRB, 0.12618E+00_JPRB, 0.12575E+00_JPRB, 0.12549E+00_JPRB, 0.12468E+00_JPRB, 0.12428E+00_JPRB, 0.12390E+00_JPRB ,&
&  0.12372E+00_JPRB, 0.12375E+00_JPRB, 0.12369E+00_JPRB, 0.12387E+00_JPRB, 0.12413E+00_JPRB, 0.12475E+00_JPRB, 0.12527E+00_JPRB ,&
&  0.12569E+00_JPRB, 0.12642E+00_JPRB, 0.12687E+00_JPRB, 0.12750E+00_JPRB, 0.12839E+00_JPRB, 0.12885E+00_JPRB, 0.12992E+00_JPRB ,&
&  0.13088E+00_JPRB, 0.13198E+00_JPRB, 0.13390E+00_JPRB, 0.13587E+00_JPRB, 0.13864E+00_JPRB, 0.14227E+00_JPRB, 0.14407E+00_JPRB ,&
&  0.14586E+00_JPRB, 0.14791E+00_JPRB, 0.14738E+00_JPRB, 0.15172E+00_JPRB, 0.15372E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.15783E+00_JPRB, 0.15868E+00_JPRB, 0.15710E+00_JPRB, 0.15761E+00_JPRB, 0.15842E+00_JPRB, 0.15888E+00_JPRB, 0.15933E+00_JPRB ,&
&  0.15700E+00_JPRB, 0.15599E+00_JPRB, 0.15365E+00_JPRB, 0.15138E+00_JPRB, 0.14880E+00_JPRB, 0.14605E+00_JPRB, 0.14391E+00_JPRB ,&
&  0.14209E+00_JPRB, 0.14127E+00_JPRB, 0.14043E+00_JPRB, 0.14099E+00_JPRB, 0.14102E+00_JPRB, 0.14250E+00_JPRB, 0.14372E+00_JPRB ,&
&  0.14446E+00_JPRB, 0.14539E+00_JPRB, 0.14435E+00_JPRB, 0.14439E+00_JPRB, 0.14616E+00_JPRB, 0.14789E+00_JPRB, 0.15051E+00_JPRB ,&
&  0.15333E+00_JPRB, 0.15627E+00_JPRB, 0.16095E+00_JPRB, 0.16274E+00_JPRB, 0.16480E+00_JPRB, 0.16618E+00_JPRB, 0.16626E+00_JPRB ,&
&  0.16604E+00_JPRB, 0.16630E+00_JPRB, 0.16728E+00_JPRB, 0.16718E+00_JPRB, 0.16071E+00_JPRB, 0.15560E+00_JPRB, 0.15027E+00_JPRB ,&
&  0.14523E+00_JPRB, 0.14049E+00_JPRB, 0.13632E+00_JPRB, 0.13352E+00_JPRB, 0.13076E+00_JPRB, 0.12874E+00_JPRB, 0.12716E+00_JPRB ,&
&  0.12553E+00_JPRB, 0.12451E+00_JPRB, 0.12328E+00_JPRB, 0.12287E+00_JPRB, 0.12307E+00_JPRB, 0.12371E+00_JPRB, 0.12456E+00_JPRB ,&
&  0.12559E+00_JPRB, 0.12633E+00_JPRB, 0.12726E+00_JPRB, 0.12816E+00_JPRB, 0.12880E+00_JPRB, 0.12924E+00_JPRB, 0.12906E+00_JPRB ,&
&  0.12927E+00_JPRB, 0.12949E+00_JPRB, 0.12910E+00_JPRB, 0.12900E+00_JPRB, 0.12801E+00_JPRB, 0.12687E+00_JPRB, 0.12567E+00_JPRB ,&
&  0.12412E+00_JPRB, 0.12293E+00_JPRB, 0.12246E+00_JPRB, 0.12218E+00_JPRB, 0.12273E+00_JPRB, 0.12360E+00_JPRB, 0.12420E+00_JPRB ,&
&  0.12454E+00_JPRB, 0.12450E+00_JPRB, 0.12459E+00_JPRB, 0.12559E+00_JPRB, 0.12665E+00_JPRB, 0.12798E+00_JPRB, 0.12936E+00_JPRB ,&
&  0.12987E+00_JPRB, 0.13102E+00_JPRB, 0.13205E+00_JPRB, 0.13327E+00_JPRB, 0.13725E+00_JPRB, 0.14056E+00_JPRB, 0.14692E+00_JPRB ,&
&  0.15061E+00_JPRB, 0.15362E+00_JPRB, 0.15431E+00_JPRB, 0.15760E+00_JPRB, 0.15746E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.16186E+00_JPRB, 0.16382E+00_JPRB, 0.16567E+00_JPRB, 0.16481E+00_JPRB, 0.16498E+00_JPRB, 0.16502E+00_JPRB, 0.16510E+00_JPRB ,&
&  0.16509E+00_JPRB, 0.16454E+00_JPRB, 0.16471E+00_JPRB, 0.16320E+00_JPRB, 0.16229E+00_JPRB, 0.15858E+00_JPRB, 0.15707E+00_JPRB ,&
&  0.15508E+00_JPRB, 0.15445E+00_JPRB, 0.15460E+00_JPRB, 0.15560E+00_JPRB, 0.15606E+00_JPRB, 0.15723E+00_JPRB, 0.15664E+00_JPRB ,&
&  0.15675E+00_JPRB, 0.15658E+00_JPRB, 0.15541E+00_JPRB, 0.15510E+00_JPRB, 0.15560E+00_JPRB, 0.15620E+00_JPRB, 0.15653E+00_JPRB ,&
&  0.15817E+00_JPRB, 0.16031E+00_JPRB, 0.16239E+00_JPRB, 0.16387E+00_JPRB, 0.16912E+00_JPRB, 0.17071E+00_JPRB, 0.17249E+00_JPRB ,&
&  0.17363E+00_JPRB, 0.17516E+00_JPRB, 0.17617E+00_JPRB, 0.17567E+00_JPRB, 0.17457E+00_JPRB, 0.17143E+00_JPRB, 0.17021E+00_JPRB ,&
&  0.16015E+00_JPRB, 0.15344E+00_JPRB, 0.14678E+00_JPRB, 0.14098E+00_JPRB, 0.13688E+00_JPRB, 0.13406E+00_JPRB, 0.13169E+00_JPRB ,&
&  0.12974E+00_JPRB, 0.12788E+00_JPRB, 0.12590E+00_JPRB, 0.12585E+00_JPRB, 0.12652E+00_JPRB, 0.12753E+00_JPRB, 0.12851E+00_JPRB ,&
&  0.12964E+00_JPRB, 0.13031E+00_JPRB, 0.13064E+00_JPRB, 0.13133E+00_JPRB, 0.13153E+00_JPRB, 0.13189E+00_JPRB, 0.13208E+00_JPRB ,&
&  0.13321E+00_JPRB, 0.13360E+00_JPRB, 0.13368E+00_JPRB, 0.13481E+00_JPRB, 0.13338E+00_JPRB, 0.13106E+00_JPRB, 0.12841E+00_JPRB ,&
&  0.12585E+00_JPRB, 0.12423E+00_JPRB, 0.12287E+00_JPRB, 0.12231E+00_JPRB, 0.12259E+00_JPRB, 0.12318E+00_JPRB, 0.12401E+00_JPRB ,&
&  0.12424E+00_JPRB, 0.12477E+00_JPRB, 0.12519E+00_JPRB, 0.12599E+00_JPRB, 0.12686E+00_JPRB, 0.12782E+00_JPRB, 0.12884E+00_JPRB ,&
&  0.13036E+00_JPRB, 0.13212E+00_JPRB, 0.13389E+00_JPRB, 0.13570E+00_JPRB, 0.13858E+00_JPRB, 0.14214E+00_JPRB, 0.14787E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.16165E+00_JPRB, 0.16107E+00_JPRB, 0.16268E+00_JPRB, 0.16280E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.16833E+00_JPRB, 0.17189E+00_JPRB, 0.17351E+00_JPRB, 0.17359E+00_JPRB, 0.17261E+00_JPRB, 0.17180E+00_JPRB, 0.17167E+00_JPRB ,&
&  0.17160E+00_JPRB, 0.17055E+00_JPRB, 0.16974E+00_JPRB, 0.16904E+00_JPRB, 0.16829E+00_JPRB, 0.16755E+00_JPRB, 0.16780E+00_JPRB ,&
&  0.16893E+00_JPRB, 0.17020E+00_JPRB, 0.17102E+00_JPRB, 0.17120E+00_JPRB, 0.17070E+00_JPRB, 0.17096E+00_JPRB, 0.17176E+00_JPRB ,&
&  0.17172E+00_JPRB, 0.17214E+00_JPRB, 0.17539E+00_JPRB, 0.17603E+00_JPRB, 0.17702E+00_JPRB, 0.17722E+00_JPRB, 0.17730E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.16800E+00_JPRB, 0.16468E+00_JPRB, 0.16659E+00_JPRB, 0.17499E+00_JPRB, 0.17678E+00_JPRB, 0.17987E+00_JPRB ,&
&  0.18199E+00_JPRB, 0.18328E+00_JPRB, 0.18455E+00_JPRB, 0.18566E+00_JPRB, 0.18699E+00_JPRB, 0.18599E+00_JPRB, 0.18387E+00_JPRB ,&
&  0.18136E+00_JPRB, 0.18041E+00_JPRB, 0.17732E+00_JPRB, 0.16396E+00_JPRB, 0.15558E+00_JPRB, 0.15085E+00_JPRB, 0.14595E+00_JPRB ,&
&  0.14153E+00_JPRB, 0.13885E+00_JPRB, 0.13803E+00_JPRB, 0.13692E+00_JPRB, 0.13776E+00_JPRB, 0.13941E+00_JPRB, 0.14013E+00_JPRB ,&
&  0.13984E+00_JPRB, 0.14019E+00_JPRB, 0.13988E+00_JPRB, 0.13899E+00_JPRB, 0.13750E+00_JPRB, 0.13655E+00_JPRB, 0.13597E+00_JPRB ,&
&  0.13637E+00_JPRB, 0.13625E+00_JPRB, 0.13994E+00_JPRB, 0.14143E+00_JPRB, 0.13748E+00_JPRB, 0.13572E+00_JPRB, 0.13357E+00_JPRB ,&
&  0.12908E+00_JPRB, 0.12655E+00_JPRB, 0.12515E+00_JPRB, 0.12456E+00_JPRB, 0.12535E+00_JPRB, 0.12654E+00_JPRB, 0.12719E+00_JPRB ,&
&  0.12742E+00_JPRB, 0.12830E+00_JPRB, 0.12874E+00_JPRB, 0.12925E+00_JPRB, 0.12960E+00_JPRB, 0.13034E+00_JPRB, 0.13141E+00_JPRB ,&
&  0.13277E+00_JPRB, 0.13518E+00_JPRB, 0.13759E+00_JPRB, 0.14030E+00_JPRB, 0.14195E+00_JPRB, 0.14326E+00_JPRB, 0.14617E+00_JPRB ,&
&  0.14825E+00_JPRB, 0.15474E+00_JPRB, 0.16432E+00_JPRB, 0.16842E+00_JPRB, 0.16431E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.17959E+00_JPRB, 0.18146E+00_JPRB, 0.18211E+00_JPRB, 0.18219E+00_JPRB, 0.18211E+00_JPRB, 0.18265E+00_JPRB, 0.18319E+00_JPRB ,&
&  0.18318E+00_JPRB, 0.18250E+00_JPRB, 0.18137E+00_JPRB, 0.17952E+00_JPRB, 0.17732E+00_JPRB, 0.17607E+00_JPRB, 0.17711E+00_JPRB ,&
&  0.17795E+00_JPRB, 0.17765E+00_JPRB, 0.17748E+00_JPRB, 0.17715E+00_JPRB, 0.17657E+00_JPRB, 0.17666E+00_JPRB, 0.17776E+00_JPRB ,&
&  0.17916E+00_JPRB, 0.18016E+00_JPRB, 0.18160E+00_JPRB, 0.18286E+00_JPRB, 0.18491E+00_JPRB, 0.18676E+00_JPRB, 0.18764E+00_JPRB ,&
&  0.18573E+00_JPRB, 0.17807E+00_JPRB, 0.17252E+00_JPRB, 0.17495E+00_JPRB, 0.18373E+00_JPRB, 0.18845E+00_JPRB, 0.19132E+00_JPRB ,&
&  0.19354E+00_JPRB, 0.19499E+00_JPRB, 0.19550E+00_JPRB, 0.19675E+00_JPRB, 0.19849E+00_JPRB, 0.20008E+00_JPRB, 0.19951E+00_JPRB ,&
&  0.19743E+00_JPRB, 0.19565E+00_JPRB, 0.19310E+00_JPRB, 0.18886E+00_JPRB, 0.18323E+00_JPRB, 0.17770E+00_JPRB, 0.17301E+00_JPRB ,&
&  0.16893E+00_JPRB, 0.16494E+00_JPRB, 0.16151E+00_JPRB, 0.15930E+00_JPRB, 0.15856E+00_JPRB, 0.15962E+00_JPRB, 0.15947E+00_JPRB ,&
&  0.15906E+00_JPRB, 0.15733E+00_JPRB, 0.15518E+00_JPRB, 0.15271E+00_JPRB, 0.15053E+00_JPRB, 0.14685E+00_JPRB, 0.14244E+00_JPRB ,&
&  0.14010E+00_JPRB, 0.14209E+00_JPRB, 0.14812E+00_JPRB, 0.15337E+00_JPRB, 0.15535E+00_JPRB, 0.14788E+00_JPRB, 0.14359E+00_JPRB ,&
&  0.13954E+00_JPRB, 0.13580E+00_JPRB, 0.13343E+00_JPRB, 0.13281E+00_JPRB, 0.13289E+00_JPRB, 0.13304E+00_JPRB, 0.13385E+00_JPRB ,&
&  0.13518E+00_JPRB, 0.13639E+00_JPRB, 0.13710E+00_JPRB, 0.13781E+00_JPRB, 0.13809E+00_JPRB, 0.13813E+00_JPRB, 0.13823E+00_JPRB ,&
&  0.13951E+00_JPRB, 0.14230E+00_JPRB, 0.14618E+00_JPRB, 0.15062E+00_JPRB, 0.15412E+00_JPRB, 0.15734E+00_JPRB, 0.16051E+00_JPRB ,&
&  0.16399E+00_JPRB, 0.16671E+00_JPRB, 0.16959E+00_JPRB, 0.17329E+00_JPRB, 0.17661E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.18946E+00_JPRB, 0.19156E+00_JPRB, 0.19238E+00_JPRB, 0.19213E+00_JPRB, 0.19181E+00_JPRB, 0.19279E+00_JPRB, 0.19409E+00_JPRB ,&
&  0.19473E+00_JPRB, 0.19479E+00_JPRB, 0.19402E+00_JPRB, 0.19250E+00_JPRB, 0.19143E+00_JPRB, 0.19179E+00_JPRB, 0.19026E+00_JPRB ,&
&  0.18866E+00_JPRB, 0.18855E+00_JPRB, 0.18699E+00_JPRB, 0.18533E+00_JPRB, 0.18289E+00_JPRB, 0.18139E+00_JPRB, 0.18186E+00_JPRB ,&
&  0.18392E+00_JPRB, 0.18466E+00_JPRB, 0.18548E+00_JPRB, 0.18661E+00_JPRB, 0.18865E+00_JPRB, 0.19099E+00_JPRB, 0.19270E+00_JPRB ,&
&  0.19406E+00_JPRB, 0.19371E+00_JPRB, 0.19199E+00_JPRB, 0.19500E+00_JPRB, 0.19789E+00_JPRB, 0.20151E+00_JPRB, 0.20425E+00_JPRB ,&
&  0.20719E+00_JPRB, 0.20957E+00_JPRB, 0.21077E+00_JPRB, 0.21078E+00_JPRB, 0.21095E+00_JPRB, 0.21226E+00_JPRB, 0.21296E+00_JPRB ,&
&  0.21283E+00_JPRB, 0.21194E+00_JPRB, 0.21031E+00_JPRB, 0.20877E+00_JPRB, 0.20538E+00_JPRB, 0.20038E+00_JPRB, 0.19311E+00_JPRB ,&
&  0.18563E+00_JPRB, 0.17969E+00_JPRB, 0.17523E+00_JPRB, 0.17162E+00_JPRB, 0.16908E+00_JPRB, 0.16870E+00_JPRB, 0.16861E+00_JPRB ,&
&  0.16820E+00_JPRB, 0.16783E+00_JPRB, 0.16921E+00_JPRB, 0.17011E+00_JPRB, 0.16691E+00_JPRB, 0.16147E+00_JPRB, 0.15717E+00_JPRB ,&
&  0.15623E+00_JPRB, 0.15774E+00_JPRB, 0.16024E+00_JPRB, 0.16193E+00_JPRB, 0.16476E+00_JPRB, 0.16546E+00_JPRB, 0.16460E+00_JPRB ,&
&  0.15984E+00_JPRB, 0.15416E+00_JPRB, 0.15093E+00_JPRB, 0.14859E+00_JPRB, 0.14705E+00_JPRB, 0.14666E+00_JPRB, 0.14717E+00_JPRB ,&
&  0.14904E+00_JPRB, 0.15103E+00_JPRB, 0.15319E+00_JPRB, 0.15418E+00_JPRB, 0.15380E+00_JPRB, 0.15318E+00_JPRB, 0.15237E+00_JPRB ,&
&  0.15323E+00_JPRB, 0.15695E+00_JPRB, 0.16292E+00_JPRB, 0.16669E+00_JPRB, 0.17015E+00_JPRB, 0.17246E+00_JPRB, 0.17370E+00_JPRB ,&
&  0.17463E+00_JPRB, 0.17531E+00_JPRB, 0.17619E+00_JPRB, 0.18051E+00_JPRB, 0.18606E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.20109E+00_JPRB, 0.20333E+00_JPRB, 0.20462E+00_JPRB, 0.20469E+00_JPRB, 0.20442E+00_JPRB, 0.20524E+00_JPRB, 0.20575E+00_JPRB ,&
&  0.20629E+00_JPRB, 0.20655E+00_JPRB, 0.20561E+00_JPRB, 0.20485E+00_JPRB, 0.20657E+00_JPRB, 0.20610E+00_JPRB, 0.20555E+00_JPRB ,&
&  0.20391E+00_JPRB, 0.20198E+00_JPRB, 0.19915E+00_JPRB, 0.19591E+00_JPRB, 0.19194E+00_JPRB, 0.18937E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.19093E+00_JPRB, 0.19150E+00_JPRB, 0.19240E+00_JPRB, 0.19369E+00_JPRB, 0.19625E+00_JPRB, 0.19947E+00_JPRB, 0.20195E+00_JPRB ,&
&  0.20399E+00_JPRB, 0.20725E+00_JPRB, 0.20929E+00_JPRB, 0.20792E+00_JPRB, 0.21221E+00_JPRB, 0.21633E+00_JPRB, 0.22078E+00_JPRB ,&
&  0.22516E+00_JPRB, 0.22795E+00_JPRB, 0.22846E+00_JPRB, 0.22748E+00_JPRB, 0.22635E+00_JPRB, 0.22663E+00_JPRB, 0.22616E+00_JPRB ,&
&  0.22565E+00_JPRB, 0.22689E+00_JPRB, 0.22870E+00_JPRB, 0.22897E+00_JPRB, 0.22706E+00_JPRB, 0.22283E+00_JPRB, 0.21648E+00_JPRB ,&
&  0.20847E+00_JPRB, 0.20008E+00_JPRB, 0.19332E+00_JPRB, 0.18800E+00_JPRB, 0.18387E+00_JPRB, 0.18043E+00_JPRB, 0.17761E+00_JPRB ,&
&  0.17453E+00_JPRB, 0.17238E+00_JPRB, 0.17234E+00_JPRB, 0.17300E+00_JPRB, 0.17336E+00_JPRB, 0.17231E+00_JPRB, 0.17155E+00_JPRB ,&
&  0.17076E+00_JPRB, 0.17120E+00_JPRB, 0.17198E+00_JPRB, 0.17228E+00_JPRB, 0.17238E+00_JPRB, 0.17198E+00_JPRB, 0.17159E+00_JPRB ,&
&  0.17224E+00_JPRB, 0.17250E+00_JPRB, 0.17166E+00_JPRB, 0.16999E+00_JPRB, 0.16856E+00_JPRB, 0.16823E+00_JPRB, 0.16830E+00_JPRB ,&
&  0.16867E+00_JPRB, 0.16955E+00_JPRB, 0.17098E+00_JPRB, 0.17250E+00_JPRB, 0.17367E+00_JPRB, 0.17414E+00_JPRB, 0.17421E+00_JPRB ,&
&  0.17425E+00_JPRB, 0.17395E+00_JPRB, 0.17457E+00_JPRB, 0.17703E+00_JPRB, 0.17982E+00_JPRB, 0.18218E+00_JPRB, 0.18358E+00_JPRB ,&
&  0.18335E+00_JPRB, 0.18418E+00_JPRB, 0.18730E+00_JPRB, 0.19319E+00_JPRB, 0.19765E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.21693E+00_JPRB, 0.21938E+00_JPRB, 0.22205E+00_JPRB, 0.22244E+00_JPRB, 0.22105E+00_JPRB, 0.22042E+00_JPRB, 0.22062E+00_JPRB ,&
&  0.22107E+00_JPRB, 0.22139E+00_JPRB, 0.22044E+00_JPRB, 0.22048E+00_JPRB, 0.22063E+00_JPRB, 0.21977E+00_JPRB, 0.21903E+00_JPRB ,&
&  0.21769E+00_JPRB, 0.21480E+00_JPRB, 0.21080E+00_JPRB, 0.20589E+00_JPRB, 0.20220E+00_JPRB, 0.20020E+00_JPRB, 0.19983E+00_JPRB ,&
&  0.20044E+00_JPRB, 0.20095E+00_JPRB, 0.20347E+00_JPRB, 0.20579E+00_JPRB, 0.20876E+00_JPRB, 0.21263E+00_JPRB, 0.21664E+00_JPRB ,&
&  0.21969E+00_JPRB, 0.22279E+00_JPRB, 0.22575E+00_JPRB, 0.22845E+00_JPRB, 0.23216E+00_JPRB, 0.23866E+00_JPRB, 0.24602E+00_JPRB ,&
&  0.25297E+00_JPRB, 0.25608E+00_JPRB, 0.25364E+00_JPRB, 0.25029E+00_JPRB, 0.24945E+00_JPRB, 0.25001E+00_JPRB, 0.25043E+00_JPRB ,&
&  0.25066E+00_JPRB, 0.25094E+00_JPRB, 0.25098E+00_JPRB, 0.24959E+00_JPRB, 0.24629E+00_JPRB, 0.24138E+00_JPRB, 0.23532E+00_JPRB ,&
&  0.22891E+00_JPRB, 0.22260E+00_JPRB, 0.21649E+00_JPRB, 0.21017E+00_JPRB, 0.20474E+00_JPRB, 0.19950E+00_JPRB, 0.19464E+00_JPRB ,&
&  0.19007E+00_JPRB, 0.18576E+00_JPRB, 0.18250E+00_JPRB, 0.18051E+00_JPRB, 0.18018E+00_JPRB, 0.17932E+00_JPRB, 0.17946E+00_JPRB ,&
&  0.17914E+00_JPRB, 0.17909E+00_JPRB, 0.17941E+00_JPRB, 0.18041E+00_JPRB, 0.18189E+00_JPRB, 0.18206E+00_JPRB, 0.18096E+00_JPRB ,&
&  0.18030E+00_JPRB, 0.18129E+00_JPRB, 0.18096E+00_JPRB, 0.17873E+00_JPRB, 0.17596E+00_JPRB, 0.17522E+00_JPRB, 0.17639E+00_JPRB ,&
&  0.17873E+00_JPRB, 0.18102E+00_JPRB, 0.18297E+00_JPRB, 0.18476E+00_JPRB, 0.18626E+00_JPRB, 0.18814E+00_JPRB, 0.18987E+00_JPRB ,&
&  0.19128E+00_JPRB, 0.19185E+00_JPRB, 0.19312E+00_JPRB, 0.19449E+00_JPRB, 0.19599E+00_JPRB, 0.19708E+00_JPRB, 0.19698E+00_JPRB ,&
&  0.19661E+00_JPRB, 0.19804E+00_JPRB, 0.20258E+00_JPRB, 0.20966E+00_JPRB, 0.21416E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.23633E+00_JPRB, 0.23842E+00_JPRB, 0.24125E+00_JPRB, 0.24369E+00_JPRB, 0.24475E+00_JPRB, 0.24443E+00_JPRB, 0.24378E+00_JPRB ,&
&  0.24381E+00_JPRB, 0.24399E+00_JPRB, 0.24276E+00_JPRB, 0.24048E+00_JPRB, 0.23810E+00_JPRB, 0.23552E+00_JPRB, 0.23353E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.22949E+00_JPRB, 0.22595E+00_JPRB, 0.22081E+00_JPRB, 0.21547E+00_JPRB, 0.21236E+00_JPRB, 0.21086E+00_JPRB ,&
&  0.21077E+00_JPRB, 0.21132E+00_JPRB, 0.21390E+00_JPRB, 0.21580E+00_JPRB, 0.21837E+00_JPRB, 0.22255E+00_JPRB, 0.22821E+00_JPRB ,&
&  0.23358E+00_JPRB, 0.23865E+00_JPRB, 0.24375E+00_JPRB, 0.24806E+00_JPRB, 0.25311E+00_JPRB, 0.26164E+00_JPRB, 0.27242E+00_JPRB ,&
&  0.28074E+00_JPRB, 0.28537E+00_JPRB, 0.28510E+00_JPRB, 0.28497E+00_JPRB, 0.28645E+00_JPRB, 0.28835E+00_JPRB, 0.28929E+00_JPRB ,&
&  0.28958E+00_JPRB, 0.28660E+00_JPRB, 0.28259E+00_JPRB, 0.27754E+00_JPRB, 0.27248E+00_JPRB, 0.26738E+00_JPRB, 0.26139E+00_JPRB ,&
&  0.25448E+00_JPRB, 0.24684E+00_JPRB, 0.23924E+00_JPRB, 0.23186E+00_JPRB, 0.22466E+00_JPRB, 0.21798E+00_JPRB, 0.21197E+00_JPRB ,&
&  0.20619E+00_JPRB, 0.20177E+00_JPRB, 0.19752E+00_JPRB, 0.19503E+00_JPRB, 0.19363E+00_JPRB, 0.19199E+00_JPRB, 0.19182E+00_JPRB ,&
&  0.19113E+00_JPRB, 0.19079E+00_JPRB, 0.19067E+00_JPRB, 0.19228E+00_JPRB, 0.19530E+00_JPRB, 0.19704E+00_JPRB, 0.19587E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.19052E+00_JPRB, 0.18932E+00_JPRB, 0.18836E+00_JPRB, 0.18604E+00_JPRB, 0.18562E+00_JPRB, 0.18729E+00_JPRB ,&
&  0.19100E+00_JPRB, 0.19514E+00_JPRB, 0.19885E+00_JPRB, 0.20102E+00_JPRB, 0.20319E+00_JPRB, 0.20484E+00_JPRB, 0.20629E+00_JPRB ,&
&  0.20797E+00_JPRB, 0.20919E+00_JPRB, 0.21013E+00_JPRB, 0.21146E+00_JPRB, 0.21377E+00_JPRB, 0.21610E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.22087E+00_JPRB, 0.22458E+00_JPRB, 0.22831E+00_JPRB, 0.23100E+00_JPRB, 0.23428E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.26437E+00_JPRB, 0.26610E+00_JPRB, 0.26825E+00_JPRB, 0.26880E+00_JPRB, 0.26842E+00_JPRB, 0.26857E+00_JPRB, 0.26767E+00_JPRB ,&
&  0.26561E+00_JPRB, 0.26452E+00_JPRB, 0.26483E+00_JPRB, 0.26357E+00_JPRB, 0.26093E+00_JPRB, 0.25765E+00_JPRB, 0.25404E+00_JPRB ,&
&  0.25120E+00_JPRB, 0.24830E+00_JPRB, 0.24347E+00_JPRB, 0.23701E+00_JPRB, 0.23145E+00_JPRB, 0.22791E+00_JPRB, 0.22615E+00_JPRB ,&
&  0.22552E+00_JPRB, 0.22571E+00_JPRB, 0.22552E+00_JPRB, 0.22628E+00_JPRB, 0.22844E+00_JPRB, 0.23322E+00_JPRB, 0.24026E+00_JPRB ,&
&  0.24738E+00_JPRB, 0.25381E+00_JPRB, 0.26046E+00_JPRB, 0.26778E+00_JPRB, 0.27798E+00_JPRB, 0.29372E+00_JPRB, 0.31018E+00_JPRB ,&
&  0.31983E+00_JPRB, 0.32177E+00_JPRB, 0.32107E+00_JPRB, 0.32348E+00_JPRB, 0.33018E+00_JPRB, 0.33581E+00_JPRB, 0.34048E+00_JPRB ,&
&  0.34184E+00_JPRB, 0.33888E+00_JPRB, 0.33456E+00_JPRB, 0.32774E+00_JPRB, 0.31920E+00_JPRB, 0.30947E+00_JPRB, 0.29973E+00_JPRB ,&
&  0.29004E+00_JPRB, 0.27987E+00_JPRB, 0.26859E+00_JPRB, 0.25778E+00_JPRB, 0.24860E+00_JPRB, 0.24061E+00_JPRB, 0.23265E+00_JPRB ,&
&  0.22525E+00_JPRB, 0.21928E+00_JPRB, 0.21455E+00_JPRB, 0.21138E+00_JPRB, 0.20885E+00_JPRB, 0.20689E+00_JPRB, 0.20645E+00_JPRB ,&
&  0.20680E+00_JPRB, 0.20813E+00_JPRB, 0.20871E+00_JPRB, 0.21005E+00_JPRB, 0.21235E+00_JPRB, 0.21331E+00_JPRB, 0.21226E+00_JPRB ,&
&  0.20848E+00_JPRB, 0.20451E+00_JPRB, 0.20078E+00_JPRB, 0.19997E+00_JPRB, 0.19976E+00_JPRB, 0.20073E+00_JPRB, 0.20226E+00_JPRB ,&
&  0.20559E+00_JPRB, 0.21059E+00_JPRB, 0.21502E+00_JPRB, 0.21803E+00_JPRB, 0.22142E+00_JPRB, 0.22437E+00_JPRB, 0.22738E+00_JPRB ,&
&  0.23088E+00_JPRB, 0.23474E+00_JPRB, 0.24000E+00_JPRB, 0.24522E+00_JPRB, 0.24992E+00_JPRB, 0.25368E+00_JPRB, 0.25644E+00_JPRB ,&
&  0.25823E+00_JPRB, 0.25862E+00_JPRB, 0.25876E+00_JPRB, 0.26011E+00_JPRB, 0.26246E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.29571E+00_JPRB, 0.29625E+00_JPRB, 0.29736E+00_JPRB, 0.29813E+00_JPRB, 0.29595E+00_JPRB, 0.29344E+00_JPRB, 0.29035E+00_JPRB ,&
&  0.28904E+00_JPRB, 0.28805E+00_JPRB, 0.28765E+00_JPRB, 0.28480E+00_JPRB, 0.28256E+00_JPRB, 0.28122E+00_JPRB, 0.27790E+00_JPRB ,&
&  0.27491E+00_JPRB, 0.26804E+00_JPRB, 0.26060E+00_JPRB, 0.25504E+00_JPRB, 0.25149E+00_JPRB, 0.24918E+00_JPRB, 0.24451E+00_JPRB ,&
&  0.24132E+00_JPRB, 0.23884E+00_JPRB, 0.23777E+00_JPRB, 0.23809E+00_JPRB, 0.24024E+00_JPRB, 0.24602E+00_JPRB, 0.25569E+00_JPRB ,&
&  0.26252E+00_JPRB, 0.27074E+00_JPRB, 0.27108E+00_JPRB, 0.28457E+00_JPRB, 0.30008E+00_JPRB, 0.32551E+00_JPRB, 0.34817E+00_JPRB ,&
&  0.35881E+00_JPRB, 0.36282E+00_JPRB, 0.36959E+00_JPRB, 0.37697E+00_JPRB, 0.38041E+00_JPRB, 0.38343E+00_JPRB, 0.38346E+00_JPRB ,&
&  0.38090E+00_JPRB, 0.37639E+00_JPRB, 0.37055E+00_JPRB, 0.36268E+00_JPRB, 0.35204E+00_JPRB, 0.34003E+00_JPRB, 0.32780E+00_JPRB ,&
&  0.31637E+00_JPRB, 0.30484E+00_JPRB, 0.29367E+00_JPRB, 0.28238E+00_JPRB, 0.27151E+00_JPRB, 0.26171E+00_JPRB, 0.25183E+00_JPRB ,&
&  0.24345E+00_JPRB, 0.23688E+00_JPRB, 0.23201E+00_JPRB, 0.22858E+00_JPRB, 0.22605E+00_JPRB, 0.22355E+00_JPRB, 0.22237E+00_JPRB ,&
&  0.22249E+00_JPRB, 0.22399E+00_JPRB, 0.22606E+00_JPRB, 0.22773E+00_JPRB, 0.22978E+00_JPRB, 0.23006E+00_JPRB, 0.22760E+00_JPRB ,&
&  0.22496E+00_JPRB, 0.22357E+00_JPRB, 0.21931E+00_JPRB, 0.21675E+00_JPRB, 0.21619E+00_JPRB, 0.21655E+00_JPRB, 0.21825E+00_JPRB ,&
&  0.22170E+00_JPRB, 0.22655E+00_JPRB, 0.23099E+00_JPRB, 0.23479E+00_JPRB, 0.23797E+00_JPRB, 0.24133E+00_JPRB, 0.24538E+00_JPRB ,&
&  0.24969E+00_JPRB, 0.25389E+00_JPRB, 0.26056E+00_JPRB, 0.26799E+00_JPRB, 0.27430E+00_JPRB, 0.28106E+00_JPRB, 0.28520E+00_JPRB ,&
&  0.28672E+00_JPRB, 0.28729E+00_JPRB, 0.28939E+00_JPRB, 0.29258E+00_JPRB, 0.29496E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.33550E+00_JPRB, 0.33377E+00_JPRB, 0.33137E+00_JPRB, 0.32946E+00_JPRB, 0.32826E+00_JPRB, 0.32515E+00_JPRB, 0.32101E+00_JPRB ,&
&  0.31814E+00_JPRB, 0.31535E+00_JPRB, 0.31331E+00_JPRB, 0.31007E+00_JPRB, 0.30831E+00_JPRB, 0.30724E+00_JPRB, 0.30350E+00_JPRB ,&
&  0.29909E+00_JPRB, 0.29288E+00_JPRB, 0.28681E+00_JPRB, 0.28139E+00_JPRB, 0.27628E+00_JPRB, 0.27161E+00_JPRB, 0.26641E+00_JPRB ,&
&  0.26151E+00_JPRB, 0.25793E+00_JPRB, 0.25614E+00_JPRB, 0.25455E+00_JPRB, 0.25601E+00_JPRB, 0.26191E+00_JPRB, 0.27139E+00_JPRB ,&
&  0.27688E+00_JPRB, 0.27795E+00_JPRB, 0.28891E+00_JPRB, 0.29953E+00_JPRB, 0.32194E+00_JPRB, 0.35746E+00_JPRB, 0.38481E+00_JPRB ,&
&  0.39517E+00_JPRB, 0.40405E+00_JPRB, 0.41489E+00_JPRB, 0.42393E+00_JPRB, 0.42726E+00_JPRB, 0.42210E+00_JPRB, 0.41608E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.39902E+00_JPRB, 0.38937E+00_JPRB, 0.37734E+00_JPRB, 0.36349E+00_JPRB, 0.35092E+00_JPRB, 0.33986E+00_JPRB ,&
&  0.32943E+00_JPRB, 0.31972E+00_JPRB, 0.31121E+00_JPRB, 0.30295E+00_JPRB, 0.29413E+00_JPRB, 0.28494E+00_JPRB, 0.27535E+00_JPRB ,&
&  0.26604E+00_JPRB, 0.25802E+00_JPRB, 0.25161E+00_JPRB, 0.24822E+00_JPRB, 0.24637E+00_JPRB, 0.24493E+00_JPRB, 0.24400E+00_JPRB ,&
&  0.24439E+00_JPRB, 0.24570E+00_JPRB, 0.24743E+00_JPRB, 0.24899E+00_JPRB, 0.25023E+00_JPRB, 0.25024E+00_JPRB, 0.24849E+00_JPRB ,&
&  0.24672E+00_JPRB, 0.24599E+00_JPRB, 0.24572E+00_JPRB, 0.24325E+00_JPRB, 0.24065E+00_JPRB, 0.24097E+00_JPRB, 0.24275E+00_JPRB ,&
&  0.24633E+00_JPRB, 0.24962E+00_JPRB, 0.25249E+00_JPRB, 0.25486E+00_JPRB, 0.25665E+00_JPRB, 0.25983E+00_JPRB, 0.26372E+00_JPRB ,&
&  0.26856E+00_JPRB, 0.27459E+00_JPRB, 0.28318E+00_JPRB, 0.29381E+00_JPRB, 0.30351E+00_JPRB, 0.31163E+00_JPRB, 0.31682E+00_JPRB ,&
&  0.31996E+00_JPRB, 0.32251E+00_JPRB, 0.32540E+00_JPRB, 0.32953E+00_JPRB, 0.33371E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.37777E+00_JPRB, 0.37350E+00_JPRB, 0.36744E+00_JPRB, 0.36223E+00_JPRB, 0.35974E+00_JPRB, 0.35761E+00_JPRB, 0.35329E+00_JPRB ,&
&  0.34920E+00_JPRB, 0.34541E+00_JPRB, 0.34143E+00_JPRB, 0.33796E+00_JPRB, 0.33508E+00_JPRB, 0.33283E+00_JPRB, 0.32916E+00_JPRB ,&
&  0.32384E+00_JPRB, 0.31733E+00_JPRB, 0.31030E+00_JPRB, 0.30474E+00_JPRB, 0.29788E+00_JPRB, 0.29082E+00_JPRB, 0.28567E+00_JPRB ,&
&  0.28138E+00_JPRB, 0.27849E+00_JPRB, 0.27758E+00_JPRB, 0.27616E+00_JPRB, 0.27720E+00_JPRB, 0.28053E+00_JPRB, 0.28863E+00_JPRB ,&
&  0.29789E+00_JPRB, 0.28146E+00_JPRB, 0.29557E+00_JPRB, 0.30740E+00_JPRB, 0.33632E+00_JPRB, 0.38552E+00_JPRB, 0.42017E+00_JPRB ,&
&  0.43853E+00_JPRB, 0.45023E+00_JPRB, 0.46233E+00_JPRB, 0.46667E+00_JPRB, 0.45796E+00_JPRB, 0.44837E+00_JPRB, 0.43992E+00_JPRB ,&
&  0.42886E+00_JPRB, 0.41632E+00_JPRB, 0.40165E+00_JPRB, 0.38634E+00_JPRB, 0.37188E+00_JPRB, 0.36074E+00_JPRB, 0.35096E+00_JPRB ,&
&  0.34283E+00_JPRB, 0.33537E+00_JPRB, 0.32896E+00_JPRB, 0.32154E+00_JPRB, 0.31310E+00_JPRB, 0.30451E+00_JPRB, 0.29688E+00_JPRB ,&
&  0.28953E+00_JPRB, 0.28231E+00_JPRB, 0.27599E+00_JPRB, 0.27072E+00_JPRB, 0.26791E+00_JPRB, 0.26681E+00_JPRB, 0.26678E+00_JPRB ,&
&  0.26752E+00_JPRB, 0.26939E+00_JPRB, 0.27108E+00_JPRB, 0.27213E+00_JPRB, 0.27261E+00_JPRB, 0.27175E+00_JPRB, 0.26940E+00_JPRB ,&
&  0.26916E+00_JPRB, 0.26912E+00_JPRB, 0.26903E+00_JPRB, 0.26947E+00_JPRB, 0.26911E+00_JPRB, 0.26920E+00_JPRB, 0.27106E+00_JPRB ,&
&  0.27301E+00_JPRB, 0.27426E+00_JPRB, 0.27575E+00_JPRB, 0.27755E+00_JPRB, 0.27982E+00_JPRB, 0.28335E+00_JPRB, 0.28832E+00_JPRB ,&
&  0.29395E+00_JPRB, 0.30026E+00_JPRB, 0.30946E+00_JPRB, 0.32138E+00_JPRB, 0.33315E+00_JPRB, 0.34401E+00_JPRB, 0.35364E+00_JPRB ,&
&  0.35811E+00_JPRB, 0.36242E+00_JPRB, 0.36908E+00_JPRB, 0.37368E+00_JPRB, 0.37801E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.43023E+00_JPRB, 0.42238E+00_JPRB, 0.41190E+00_JPRB, 0.40463E+00_JPRB, 0.40120E+00_JPRB, 0.39710E+00_JPRB, 0.39285E+00_JPRB ,&
&  0.38651E+00_JPRB, 0.37940E+00_JPRB, 0.37257E+00_JPRB, 0.36667E+00_JPRB, 0.36190E+00_JPRB, 0.35831E+00_JPRB, 0.35415E+00_JPRB ,&
&  0.34852E+00_JPRB, 0.34236E+00_JPRB, 0.33639E+00_JPRB, 0.32981E+00_JPRB, 0.32107E+00_JPRB, 0.31416E+00_JPRB, 0.30870E+00_JPRB ,&
&  0.30439E+00_JPRB, 0.30097E+00_JPRB, 0.29999E+00_JPRB, 0.29858E+00_JPRB, 0.29702E+00_JPRB, 0.29733E+00_JPRB, 0.30651E+00_JPRB ,&
&  0.30005E+00_JPRB, 0.27932E+00_JPRB, 0.29467E+00_JPRB, 0.31471E+00_JPRB, 0.34927E+00_JPRB, 0.41564E+00_JPRB, 0.46247E+00_JPRB ,&
&  0.48972E+00_JPRB, 0.50202E+00_JPRB, 0.50641E+00_JPRB, 0.49914E+00_JPRB, 0.48657E+00_JPRB, 0.47848E+00_JPRB, 0.46680E+00_JPRB ,&
&  0.45065E+00_JPRB, 0.43280E+00_JPRB, 0.41635E+00_JPRB, 0.40454E+00_JPRB, 0.39290E+00_JPRB, 0.38377E+00_JPRB, 0.37388E+00_JPRB ,&
&  0.36373E+00_JPRB, 0.35516E+00_JPRB, 0.34987E+00_JPRB, 0.34575E+00_JPRB, 0.34058E+00_JPRB, 0.33325E+00_JPRB, 0.32560E+00_JPRB ,&
&  0.31954E+00_JPRB, 0.31492E+00_JPRB, 0.31084E+00_JPRB, 0.30702E+00_JPRB, 0.30503E+00_JPRB, 0.30396E+00_JPRB, 0.30227E+00_JPRB ,&
&  0.30168E+00_JPRB, 0.30155E+00_JPRB, 0.30164E+00_JPRB, 0.30152E+00_JPRB, 0.30079E+00_JPRB, 0.29952E+00_JPRB, 0.29993E+00_JPRB ,&
&  0.30149E+00_JPRB, 0.30052E+00_JPRB, 0.29997E+00_JPRB, 0.30168E+00_JPRB, 0.30358E+00_JPRB, 0.30483E+00_JPRB, 0.30499E+00_JPRB ,&
&  0.30415E+00_JPRB, 0.30426E+00_JPRB, 0.30585E+00_JPRB, 0.30915E+00_JPRB, 0.31312E+00_JPRB, 0.31885E+00_JPRB, 0.32593E+00_JPRB ,&
&  0.33342E+00_JPRB, 0.34270E+00_JPRB, 0.35530E+00_JPRB, 0.37454E+00_JPRB, 0.39019E+00_JPRB, 0.40320E+00_JPRB, 0.41434E+00_JPRB ,&
&  0.41791E+00_JPRB, 0.41970E+00_JPRB, 0.42643E+00_JPRB, 0.42895E+00_JPRB, 0.43130E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.45535E+00_JPRB, 0.45651E+00_JPRB, 0.45084E+00_JPRB, 0.44303E+00_JPRB, 0.43574E+00_JPRB, 0.42790E+00_JPRB, 0.42132E+00_JPRB ,&
&  0.41234E+00_JPRB, 0.40189E+00_JPRB, 0.39441E+00_JPRB, 0.39104E+00_JPRB, 0.38905E+00_JPRB, 0.38583E+00_JPRB, 0.38049E+00_JPRB ,&
&  0.37391E+00_JPRB, 0.36725E+00_JPRB, 0.36117E+00_JPRB, 0.35385E+00_JPRB, 0.34685E+00_JPRB, 0.34069E+00_JPRB, 0.33612E+00_JPRB ,&
&  0.33160E+00_JPRB, 0.32658E+00_JPRB, 0.32159E+00_JPRB, 0.31839E+00_JPRB, 0.31571E+00_JPRB, 0.31447E+00_JPRB, 0.32357E+00_JPRB ,&
&  0.35276E+00_JPRB, 0.30612E+00_JPRB, 0.32411E+00_JPRB, 0.34701E+00_JPRB, 0.38843E+00_JPRB, 0.45486E+00_JPRB, 0.50060E+00_JPRB ,&
&  0.52393E+00_JPRB, 0.53751E+00_JPRB, 0.53515E+00_JPRB, 0.52846E+00_JPRB, 0.52424E+00_JPRB, 0.51316E+00_JPRB, 0.49196E+00_JPRB ,&
&  0.46982E+00_JPRB, 0.44844E+00_JPRB, 0.43240E+00_JPRB, 0.42095E+00_JPRB, 0.41193E+00_JPRB, 0.40614E+00_JPRB, 0.39814E+00_JPRB ,&
&  0.38522E+00_JPRB, 0.37771E+00_JPRB, 0.37333E+00_JPRB, 0.36913E+00_JPRB, 0.36626E+00_JPRB, 0.36227E+00_JPRB, 0.35351E+00_JPRB ,&
&  0.34541E+00_JPRB, 0.34083E+00_JPRB, 0.33924E+00_JPRB, 0.33878E+00_JPRB, 0.33924E+00_JPRB, 0.33841E+00_JPRB, 0.33592E+00_JPRB ,&
&  0.33294E+00_JPRB, 0.32903E+00_JPRB, 0.32673E+00_JPRB, 0.32497E+00_JPRB, 0.32356E+00_JPRB, 0.32511E+00_JPRB, 0.32743E+00_JPRB ,&
&  0.32849E+00_JPRB, 0.32976E+00_JPRB, 0.33216E+00_JPRB, 0.33394E+00_JPRB, 0.33410E+00_JPRB, 0.33323E+00_JPRB, 0.33194E+00_JPRB ,&
&  0.33058E+00_JPRB, 0.33018E+00_JPRB, 0.33189E+00_JPRB, 0.33694E+00_JPRB, 0.34384E+00_JPRB, 0.35201E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.37184E+00_JPRB, 0.38336E+00_JPRB, 0.39634E+00_JPRB, 0.42409E+00_JPRB, 0.44052E+00_JPRB, 0.45555E+00_JPRB, 0.46901E+00_JPRB ,&
&  0.47301E+00_JPRB, 0.47237E+00_JPRB, 0.40092E+00_JPRB, 0.43541E+00_JPRB, 0.44636E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.49601E+00_JPRB, 0.49998E+00_JPRB, 0.49263E+00_JPRB, 0.47928E+00_JPRB, 0.46664E+00_JPRB, 0.45484E+00_JPRB, 0.44525E+00_JPRB ,&
&  0.43326E+00_JPRB, 0.42341E+00_JPRB, 0.41960E+00_JPRB, 0.41865E+00_JPRB, 0.41733E+00_JPRB, 0.41305E+00_JPRB, 0.40740E+00_JPRB ,&
&  0.40169E+00_JPRB, 0.39533E+00_JPRB, 0.38786E+00_JPRB, 0.38042E+00_JPRB, 0.37425E+00_JPRB, 0.36754E+00_JPRB, 0.36281E+00_JPRB ,&
&  0.35863E+00_JPRB, 0.35525E+00_JPRB, 0.34934E+00_JPRB, 0.34165E+00_JPRB, 0.33873E+00_JPRB, 0.33431E+00_JPRB, 0.34504E+00_JPRB ,&
&  0.38381E+00_JPRB, 0.33715E+00_JPRB, 0.36152E+00_JPRB, 0.40264E+00_JPRB, 0.45102E+00_JPRB, 0.51202E+00_JPRB, 0.54671E+00_JPRB ,&
&  0.56784E+00_JPRB, 0.57733E+00_JPRB, 0.57480E+00_JPRB, 0.57524E+00_JPRB, 0.56785E+00_JPRB, 0.54511E+00_JPRB, 0.51655E+00_JPRB ,&
&  0.49423E+00_JPRB, 0.47148E+00_JPRB, 0.45529E+00_JPRB, 0.44010E+00_JPRB, 0.43257E+00_JPRB, 0.42980E+00_JPRB, 0.42348E+00_JPRB ,&
&  0.41398E+00_JPRB, 0.40954E+00_JPRB, 0.40437E+00_JPRB, 0.40264E+00_JPRB, 0.40227E+00_JPRB, 0.40302E+00_JPRB, 0.39811E+00_JPRB ,&
&  0.39311E+00_JPRB, 0.39446E+00_JPRB, 0.39759E+00_JPRB, 0.39721E+00_JPRB, 0.39567E+00_JPRB, 0.39194E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.38261E+00_JPRB, 0.37937E+00_JPRB, 0.37877E+00_JPRB, 0.37771E+00_JPRB, 0.37672E+00_JPRB, 0.37592E+00_JPRB, 0.37538E+00_JPRB ,&
&  0.37554E+00_JPRB, 0.37577E+00_JPRB, 0.37538E+00_JPRB, 0.37406E+00_JPRB, 0.37191E+00_JPRB, 0.36835E+00_JPRB, 0.36707E+00_JPRB ,&
&  0.36734E+00_JPRB, 0.36971E+00_JPRB, 0.37423E+00_JPRB, 0.38091E+00_JPRB, 0.39007E+00_JPRB, 0.39869E+00_JPRB, 0.40873E+00_JPRB ,&
&  0.41936E+00_JPRB, 0.43160E+00_JPRB, 0.44758E+00_JPRB, 0.47700E+00_JPRB, 0.50200E+00_JPRB, 0.50953E+00_JPRB, 0.51728E+00_JPRB ,&
&  0.52095E+00_JPRB, 0.52191E+00_JPRB, 0.52212E+00_JPRB, 0.46531E+00_JPRB, 0.48793E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.53911E+00_JPRB, 0.54263E+00_JPRB, 0.52784E+00_JPRB, 0.50877E+00_JPRB, 0.49027E+00_JPRB, 0.47509E+00_JPRB, 0.46200E+00_JPRB ,&
&  0.45303E+00_JPRB, 0.44698E+00_JPRB, 0.44293E+00_JPRB, 0.44035E+00_JPRB, 0.43677E+00_JPRB, 0.43090E+00_JPRB, 0.42513E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.41546E+00_JPRB, 0.40734E+00_JPRB, 0.40190E+00_JPRB, 0.39894E+00_JPRB, 0.39532E+00_JPRB, 0.39188E+00_JPRB ,&
&  0.38753E+00_JPRB, 0.37803E+00_JPRB, 0.36843E+00_JPRB, 0.36269E+00_JPRB, 0.35848E+00_JPRB, 0.35673E+00_JPRB, 0.36777E+00_JPRB ,&
&  0.41030E+00_JPRB, 0.39543E+00_JPRB, 0.43598E+00_JPRB, 0.48467E+00_JPRB, 0.54487E+00_JPRB, 0.58213E+00_JPRB, 0.59987E+00_JPRB ,&
&  0.61106E+00_JPRB, 0.62021E+00_JPRB, 0.62440E+00_JPRB, 0.63433E+00_JPRB, 0.61021E+00_JPRB, 0.57484E+00_JPRB, 0.54623E+00_JPRB ,&
&  0.51974E+00_JPRB, 0.49449E+00_JPRB, 0.47283E+00_JPRB, 0.45710E+00_JPRB, 0.45146E+00_JPRB, 0.44905E+00_JPRB, 0.44523E+00_JPRB ,&
&  0.44234E+00_JPRB, 0.43640E+00_JPRB, 0.42939E+00_JPRB, 0.42926E+00_JPRB, 0.42693E+00_JPRB, 0.43613E+00_JPRB, 0.44582E+00_JPRB ,&
&  0.45161E+00_JPRB, 0.45876E+00_JPRB, 0.46066E+00_JPRB, 0.46075E+00_JPRB, 0.45556E+00_JPRB, 0.44899E+00_JPRB, 0.44230E+00_JPRB ,&
&  0.43658E+00_JPRB, 0.43335E+00_JPRB, 0.43308E+00_JPRB, 0.43257E+00_JPRB, 0.43096E+00_JPRB, 0.43099E+00_JPRB, 0.43118E+00_JPRB ,&
&  0.43185E+00_JPRB, 0.43144E+00_JPRB, 0.43060E+00_JPRB, 0.42550E+00_JPRB, 0.41832E+00_JPRB, 0.41293E+00_JPRB, 0.40877E+00_JPRB ,&
&  0.40648E+00_JPRB, 0.40799E+00_JPRB, 0.41160E+00_JPRB, 0.41964E+00_JPRB, 0.42823E+00_JPRB, 0.43856E+00_JPRB, 0.44640E+00_JPRB ,&
&  0.45344E+00_JPRB, 0.46661E+00_JPRB, 0.48487E+00_JPRB, 0.51111E+00_JPRB, 0.52446E+00_JPRB, 0.53235E+00_JPRB, 0.54097E+00_JPRB ,&
&  0.54677E+00_JPRB, 0.54995E+00_JPRB, 0.54128E+00_JPRB, 0.53644E+00_JPRB, 0.52561E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.62174E+00_JPRB, 0.59848E+00_JPRB, 0.56678E+00_JPRB, 0.53583E+00_JPRB, 0.50699E+00_JPRB, 0.48416E+00_JPRB, 0.46999E+00_JPRB ,&
&  0.46117E+00_JPRB, 0.45548E+00_JPRB, 0.45189E+00_JPRB, 0.44901E+00_JPRB, 0.44480E+00_JPRB, 0.44131E+00_JPRB, 0.43637E+00_JPRB ,&
&  0.43063E+00_JPRB, 0.42355E+00_JPRB, 0.41719E+00_JPRB, 0.41324E+00_JPRB, 0.40961E+00_JPRB, 0.40798E+00_JPRB, 0.40617E+00_JPRB ,&
&  0.40112E+00_JPRB, 0.39441E+00_JPRB, 0.38546E+00_JPRB, 0.37629E+00_JPRB, 0.37134E+00_JPRB, 0.37140E+00_JPRB, 0.38996E+00_JPRB ,&
&  0.44959E+00_JPRB, 0.45972E+00_JPRB, 0.50954E+00_JPRB, 0.56703E+00_JPRB, 0.64451E+00_JPRB, 0.68577E+00_JPRB, 0.68971E+00_JPRB ,&
&  0.69294E+00_JPRB, 0.69547E+00_JPRB, 0.71263E+00_JPRB, 0.70332E+00_JPRB, 0.65790E+00_JPRB, 0.61669E+00_JPRB, 0.57641E+00_JPRB ,&
&  0.53866E+00_JPRB, 0.50531E+00_JPRB, 0.47720E+00_JPRB, 0.46525E+00_JPRB, 0.46245E+00_JPRB, 0.46428E+00_JPRB, 0.46473E+00_JPRB ,&
&  0.46506E+00_JPRB, 0.46184E+00_JPRB, 0.46303E+00_JPRB, 0.47511E+00_JPRB, 0.49151E+00_JPRB, 0.51343E+00_JPRB, 0.54804E+00_JPRB ,&
&  0.56250E+00_JPRB, 0.55411E+00_JPRB, 0.55451E+00_JPRB, 0.55036E+00_JPRB, 0.53438E+00_JPRB, 0.51960E+00_JPRB, 0.50803E+00_JPRB ,&
&  0.49962E+00_JPRB, 0.49444E+00_JPRB, 0.49248E+00_JPRB, 0.48854E+00_JPRB, 0.48578E+00_JPRB, 0.48493E+00_JPRB, 0.48599E+00_JPRB ,&
&  0.48644E+00_JPRB, 0.48730E+00_JPRB, 0.48607E+00_JPRB, 0.48053E+00_JPRB, 0.47713E+00_JPRB, 0.47402E+00_JPRB, 0.46684E+00_JPRB ,&
&  0.45816E+00_JPRB, 0.45476E+00_JPRB, 0.45790E+00_JPRB, 0.46441E+00_JPRB, 0.47362E+00_JPRB, 0.48263E+00_JPRB, 0.48943E+00_JPRB ,&
&  0.49714E+00_JPRB, 0.51355E+00_JPRB, 0.51341E+00_JPRB, 0.52614E+00_JPRB, 0.55775E+00_JPRB, 0.56205E+00_JPRB, 0.57102E+00_JPRB ,&
&  0.57956E+00_JPRB, 0.58072E+00_JPRB, 0.57095E+00_JPRB, 0.56150E+00_JPRB, 0.61955E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.59358E+00_JPRB, 0.57008E+00_JPRB, 0.54318E+00_JPRB, 0.51859E+00_JPRB, 0.49647E+00_JPRB, 0.47954E+00_JPRB, 0.46922E+00_JPRB ,&
&  0.46123E+00_JPRB, 0.45575E+00_JPRB, 0.45587E+00_JPRB, 0.45877E+00_JPRB, 0.45764E+00_JPRB, 0.45040E+00_JPRB, 0.44052E+00_JPRB ,&
&  0.43066E+00_JPRB, 0.42391E+00_JPRB, 0.41937E+00_JPRB, 0.41846E+00_JPRB, 0.41539E+00_JPRB, 0.41229E+00_JPRB, 0.40737E+00_JPRB ,&
&  0.40425E+00_JPRB, 0.40094E+00_JPRB, 0.39523E+00_JPRB, 0.38946E+00_JPRB, 0.38597E+00_JPRB, 0.38716E+00_JPRB, 0.41547E+00_JPRB ,&
&  0.49003E+00_JPRB, 0.55066E+00_JPRB, 0.61117E+00_JPRB, 0.68505E+00_JPRB, 0.81723E+00_JPRB, 0.81384E+00_JPRB, 0.82905E+00_JPRB ,&
&  0.83785E+00_JPRB, 0.83796E+00_JPRB, 0.83994E+00_JPRB, 0.79995E+00_JPRB, 0.72481E+00_JPRB, 0.66476E+00_JPRB, 0.60361E+00_JPRB ,&
&  0.54844E+00_JPRB, 0.50633E+00_JPRB, 0.47996E+00_JPRB, 0.46918E+00_JPRB, 0.47087E+00_JPRB, 0.47718E+00_JPRB, 0.48108E+00_JPRB ,&
&  0.48539E+00_JPRB, 0.48959E+00_JPRB, 0.51090E+00_JPRB, 0.55149E+00_JPRB, 0.61829E+00_JPRB, 0.66002E+00_JPRB, 0.72125E+00_JPRB ,&
&  0.74783E+00_JPRB, 0.73994E+00_JPRB, 0.72200E+00_JPRB, 0.69110E+00_JPRB, 0.66754E+00_JPRB, 0.64008E+00_JPRB, 0.61822E+00_JPRB ,&
&  0.59267E+00_JPRB, 0.57053E+00_JPRB, 0.55409E+00_JPRB, 0.54335E+00_JPRB, 0.53832E+00_JPRB, 0.53708E+00_JPRB, 0.53876E+00_JPRB ,&
&  0.53705E+00_JPRB, 0.53732E+00_JPRB, 0.53680E+00_JPRB, 0.54048E+00_JPRB, 0.54541E+00_JPRB, 0.53946E+00_JPRB, 0.52706E+00_JPRB ,&
&  0.51336E+00_JPRB, 0.50228E+00_JPRB, 0.50260E+00_JPRB, 0.50888E+00_JPRB, 0.51457E+00_JPRB, 0.51717E+00_JPRB, 0.51916E+00_JPRB ,&
&  0.53075E+00_JPRB, 0.54655E+00_JPRB, 0.55395E+00_JPRB, 0.55659E+00_JPRB, 0.57145E+00_JPRB, 0.60537E+00_JPRB, 0.61035E+00_JPRB ,&
&  0.61210E+00_JPRB, 0.60709E+00_JPRB, 0.59217E+00_JPRB, 0.57868E+00_JPRB, 0.60043E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.58535E+00_JPRB, 0.56863E+00_JPRB, 0.54373E+00_JPRB, 0.52200E+00_JPRB, 0.50493E+00_JPRB, 0.48775E+00_JPRB, 0.47600E+00_JPRB ,&
&  0.46746E+00_JPRB, 0.46779E+00_JPRB, 0.47054E+00_JPRB, 0.47184E+00_JPRB, 0.46776E+00_JPRB, 0.45721E+00_JPRB, 0.44651E+00_JPRB ,&
&  0.43990E+00_JPRB, 0.43976E+00_JPRB, 0.44056E+00_JPRB, 0.43828E+00_JPRB, 0.43367E+00_JPRB, 0.42773E+00_JPRB, 0.42249E+00_JPRB ,&
&  0.41786E+00_JPRB, 0.41357E+00_JPRB, 0.40916E+00_JPRB, 0.40683E+00_JPRB, 0.40414E+00_JPRB, 0.40831E+00_JPRB, 0.44217E+00_JPRB ,&
&  0.53749E+00_JPRB, 0.61582E+00_JPRB, 0.82741E+00_JPRB, 0.85131E+00_JPRB, 0.89748E+00_JPRB, 0.91789E+00_JPRB, 0.10198E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.10695E+01_JPRB, 0.10155E+01_JPRB, 0.90857E+00_JPRB, 0.78025E+00_JPRB, 0.68377E+00_JPRB, 0.61180E+00_JPRB ,&
&  0.55315E+00_JPRB, 0.51572E+00_JPRB, 0.49474E+00_JPRB, 0.48866E+00_JPRB, 0.49195E+00_JPRB, 0.50088E+00_JPRB, 0.50666E+00_JPRB ,&
&  0.51817E+00_JPRB, 0.53471E+00_JPRB, 0.58052E+00_JPRB, 0.65843E+00_JPRB, 0.71600E+00_JPRB, 0.71184E+00_JPRB, 0.82101E+00_JPRB ,&
&  0.99288E+00_JPRB, 0.94704E+00_JPRB, 0.90343E+00_JPRB, 0.85855E+00_JPRB, 0.82086E+00_JPRB, 0.78339E+00_JPRB, 0.72760E+00_JPRB ,&
&  0.68312E+00_JPRB, 0.64496E+00_JPRB, 0.61786E+00_JPRB, 0.60377E+00_JPRB, 0.59583E+00_JPRB, 0.59367E+00_JPRB, 0.59231E+00_JPRB ,&
&  0.59134E+00_JPRB, 0.59133E+00_JPRB, 0.59600E+00_JPRB, 0.59684E+00_JPRB, 0.59167E+00_JPRB, 0.57890E+00_JPRB, 0.56230E+00_JPRB ,&
&  0.55001E+00_JPRB, 0.54168E+00_JPRB, 0.53975E+00_JPRB, 0.53304E+00_JPRB, 0.54032E+00_JPRB, 0.55329E+00_JPRB, 0.56054E+00_JPRB ,&
&  0.56071E+00_JPRB, 0.56802E+00_JPRB, 0.58267E+00_JPRB, 0.58604E+00_JPRB, 0.61614E+00_JPRB, 0.65973E+00_JPRB, 0.66002E+00_JPRB ,&
&  0.65883E+00_JPRB, 0.63513E+00_JPRB, 0.61075E+00_JPRB, 0.59422E+00_JPRB, 0.59255E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.58298E+00_JPRB, 0.56023E+00_JPRB, 0.54212E+00_JPRB, 0.52468E+00_JPRB, 0.51222E+00_JPRB, 0.49998E+00_JPRB, 0.49197E+00_JPRB ,&
&  0.48969E+00_JPRB, 0.49098E+00_JPRB, 0.49037E+00_JPRB, 0.48470E+00_JPRB, 0.47458E+00_JPRB, 0.46679E+00_JPRB, 0.46184E+00_JPRB ,&
&  0.45698E+00_JPRB, 0.45680E+00_JPRB, 0.45591E+00_JPRB, 0.44866E+00_JPRB, 0.44078E+00_JPRB, 0.43560E+00_JPRB, 0.43044E+00_JPRB ,&
&  0.42650E+00_JPRB, 0.42444E+00_JPRB, 0.42285E+00_JPRB, 0.42184E+00_JPRB, 0.42207E+00_JPRB, 0.43174E+00_JPRB, 0.46484E+00_JPRB ,&
&  0.53593E+00_JPRB, 0.47339E+00_JPRB, 0.85457E+00_JPRB, 0.10262E+01_JPRB, 0.10882E+01_JPRB, 0.11431E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.14269E+01_JPRB, 0.13803E+01_JPRB, 0.12097E+01_JPRB, 0.10008E+01_JPRB, 0.81531E+00_JPRB, 0.68883E+00_JPRB, 0.62240E+00_JPRB ,&
&  0.57623E+00_JPRB, 0.54515E+00_JPRB, 0.53142E+00_JPRB, 0.52396E+00_JPRB, 0.52691E+00_JPRB, 0.53503E+00_JPRB, 0.54292E+00_JPRB ,&
&  0.55958E+00_JPRB, 0.58486E+00_JPRB, 0.63970E+00_JPRB, 0.71381E+00_JPRB, 0.73340E+00_JPRB, 0.77786E+00_JPRB, 0.86510E+00_JPRB ,&
&  0.10364E+01_JPRB, 0.11374E+01_JPRB, 0.10430E+01_JPRB, 0.97107E+00_JPRB, 0.91808E+00_JPRB, 0.86716E+00_JPRB, 0.80810E+00_JPRB ,&
&  0.75611E+00_JPRB, 0.70760E+00_JPRB, 0.67579E+00_JPRB, 0.65223E+00_JPRB, 0.63802E+00_JPRB, 0.63146E+00_JPRB, 0.62729E+00_JPRB ,&
&  0.62349E+00_JPRB, 0.62028E+00_JPRB, 0.61991E+00_JPRB, 0.61726E+00_JPRB, 0.61190E+00_JPRB, 0.59742E+00_JPRB, 0.57833E+00_JPRB ,&
&  0.56789E+00_JPRB, 0.56543E+00_JPRB, 0.56045E+00_JPRB, 0.56388E+00_JPRB, 0.57420E+00_JPRB, 0.57947E+00_JPRB, 0.58811E+00_JPRB ,&
&  0.60616E+00_JPRB, 0.61446E+00_JPRB, 0.62024E+00_JPRB, 0.63151E+00_JPRB, 0.63908E+00_JPRB, 0.68780E+00_JPRB, 0.66815E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.63920E+00_JPRB, 0.61513E+00_JPRB, 0.60281E+00_JPRB, 0.59662E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.58090E+00_JPRB, 0.56006E+00_JPRB, 0.54539E+00_JPRB, 0.53405E+00_JPRB, 0.52496E+00_JPRB, 0.51768E+00_JPRB, 0.51684E+00_JPRB ,&
&  0.51655E+00_JPRB, 0.51347E+00_JPRB, 0.50188E+00_JPRB, 0.48774E+00_JPRB, 0.48311E+00_JPRB, 0.48331E+00_JPRB, 0.47565E+00_JPRB ,&
&  0.47408E+00_JPRB, 0.47453E+00_JPRB, 0.46966E+00_JPRB, 0.46098E+00_JPRB, 0.45412E+00_JPRB, 0.45067E+00_JPRB, 0.45040E+00_JPRB ,&
&  0.44437E+00_JPRB, 0.44256E+00_JPRB, 0.44546E+00_JPRB, 0.44790E+00_JPRB, 0.45188E+00_JPRB, 0.47082E+00_JPRB, 0.48860E+00_JPRB ,&
&  0.53213E+00_JPRB, 0.67136E+00_JPRB, 0.71674E+00_JPRB, 0.11317E+01_JPRB, 0.12677E+01_JPRB, 0.14055E+01_JPRB, 0.14417E+01_JPRB ,&
&  0.14997E+01_JPRB, 0.15843E+01_JPRB, 0.13292E+01_JPRB, 0.10666E+01_JPRB, 0.83836E+00_JPRB, 0.71475E+00_JPRB, 0.65486E+00_JPRB ,&
&  0.62434E+00_JPRB, 0.59525E+00_JPRB, 0.57533E+00_JPRB, 0.57043E+00_JPRB, 0.57419E+00_JPRB, 0.57783E+00_JPRB, 0.58340E+00_JPRB ,&
&  0.60315E+00_JPRB, 0.63929E+00_JPRB, 0.70865E+00_JPRB, 0.77143E+00_JPRB, 0.79828E+00_JPRB, 0.91759E+00_JPRB, 0.10279E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.12957E+01_JPRB, 0.12037E+01_JPRB, 0.10991E+01_JPRB, 0.99523E+00_JPRB, 0.90257E+00_JPRB, 0.82528E+00_JPRB ,&
&  0.77064E+00_JPRB, 0.72753E+00_JPRB, 0.69574E+00_JPRB, 0.67239E+00_JPRB, 0.65659E+00_JPRB, 0.64697E+00_JPRB, 0.63506E+00_JPRB ,&
&  0.62119E+00_JPRB, 0.61587E+00_JPRB, 0.61761E+00_JPRB, 0.61879E+00_JPRB, 0.61464E+00_JPRB, 0.60592E+00_JPRB, 0.59603E+00_JPRB ,&
&  0.59164E+00_JPRB, 0.59431E+00_JPRB, 0.59099E+00_JPRB, 0.60874E+00_JPRB, 0.61834E+00_JPRB, 0.60566E+00_JPRB, 0.60547E+00_JPRB ,&
&  0.63729E+00_JPRB, 0.65315E+00_JPRB, 0.65566E+00_JPRB, 0.65687E+00_JPRB, 0.68336E+00_JPRB, 0.73821E+00_JPRB, 0.66859E+00_JPRB ,&
&  0.63738E+00_JPRB, 0.62511E+00_JPRB, 0.61016E+00_JPRB, 0.60121E+00_JPRB, 0.59319E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.58404E+00_JPRB, 0.57041E+00_JPRB, 0.56005E+00_JPRB, 0.54883E+00_JPRB, 0.53991E+00_JPRB, 0.53755E+00_JPRB, 0.53544E+00_JPRB ,&
&  0.52977E+00_JPRB, 0.51185E+00_JPRB, 0.49260E+00_JPRB, 0.48595E+00_JPRB, 0.48543E+00_JPRB, 0.47896E+00_JPRB, 0.48243E+00_JPRB ,&
&  0.48535E+00_JPRB, 0.48856E+00_JPRB, 0.48057E+00_JPRB, 0.47528E+00_JPRB, 0.47221E+00_JPRB, 0.47075E+00_JPRB, 0.47211E+00_JPRB ,&
&  0.47348E+00_JPRB, 0.47480E+00_JPRB, 0.48026E+00_JPRB, 0.48635E+00_JPRB, 0.49595E+00_JPRB, 0.51616E+00_JPRB, 0.53533E+00_JPRB ,&
&  0.61687E+00_JPRB, 0.67795E+00_JPRB, 0.74143E+00_JPRB, 0.13187E+01_JPRB, 0.15541E+01_JPRB, 0.16035E+01_JPRB, 0.14598E+01_JPRB ,&
&  0.14339E+01_JPRB, 0.15561E+01_JPRB, 0.13909E+01_JPRB, 0.11207E+01_JPRB, 0.91180E+00_JPRB, 0.81532E+00_JPRB, 0.75285E+00_JPRB ,&
&  0.70097E+00_JPRB, 0.65320E+00_JPRB, 0.62388E+00_JPRB, 0.60680E+00_JPRB, 0.60975E+00_JPRB, 0.61630E+00_JPRB, 0.63402E+00_JPRB ,&
&  0.66368E+00_JPRB, 0.71909E+00_JPRB, 0.78870E+00_JPRB, 0.83632E+00_JPRB, 0.10223E+01_JPRB, 0.12224E+01_JPRB, 0.12856E+01_JPRB ,&
&  0.12728E+01_JPRB, 0.13603E+01_JPRB, 0.12229E+01_JPRB, 0.11099E+01_JPRB, 0.97132E+00_JPRB, 0.89985E+00_JPRB, 0.82249E+00_JPRB ,&
&  0.76684E+00_JPRB, 0.72090E+00_JPRB, 0.68741E+00_JPRB, 0.66983E+00_JPRB, 0.65388E+00_JPRB, 0.63884E+00_JPRB, 0.62432E+00_JPRB ,&
&  0.61028E+00_JPRB, 0.61378E+00_JPRB, 0.61915E+00_JPRB, 0.61545E+00_JPRB, 0.61370E+00_JPRB, 0.61436E+00_JPRB, 0.61734E+00_JPRB ,&
&  0.62036E+00_JPRB, 0.62242E+00_JPRB, 0.62092E+00_JPRB, 0.64653E+00_JPRB, 0.65995E+00_JPRB, 0.64884E+00_JPRB, 0.65032E+00_JPRB ,&
&  0.67192E+00_JPRB, 0.68786E+00_JPRB, 0.68257E+00_JPRB, 0.66137E+00_JPRB, 0.67581E+00_JPRB, 0.69137E+00_JPRB, 0.65263E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.63405E+00_JPRB, 0.61213E+00_JPRB, 0.59798E+00_JPRB, 0.59194E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.59602E+00_JPRB, 0.58137E+00_JPRB, 0.56536E+00_JPRB, 0.55364E+00_JPRB, 0.54685E+00_JPRB, 0.54023E+00_JPRB, 0.52779E+00_JPRB ,&
&  0.51055E+00_JPRB, 0.50087E+00_JPRB, 0.48240E+00_JPRB, 0.47974E+00_JPRB, 0.48714E+00_JPRB, 0.49244E+00_JPRB, 0.50143E+00_JPRB ,&
&  0.50663E+00_JPRB, 0.50745E+00_JPRB, 0.50328E+00_JPRB, 0.49880E+00_JPRB, 0.49825E+00_JPRB, 0.49983E+00_JPRB, 0.50829E+00_JPRB ,&
&  0.50889E+00_JPRB, 0.51126E+00_JPRB, 0.51843E+00_JPRB, 0.52842E+00_JPRB, 0.55370E+00_JPRB, 0.59652E+00_JPRB, 0.65233E+00_JPRB ,&
&  0.78067E+00_JPRB, 0.81061E+00_JPRB, 0.86371E+00_JPRB, 0.13034E+01_JPRB, 0.15615E+01_JPRB, 0.14694E+01_JPRB, 0.14014E+01_JPRB ,&
&  0.13632E+01_JPRB, 0.12999E+01_JPRB, 0.13138E+01_JPRB, 0.11343E+01_JPRB, 0.97234E+00_JPRB, 0.88204E+00_JPRB, 0.81276E+00_JPRB ,&
&  0.74627E+00_JPRB, 0.69712E+00_JPRB, 0.66063E+00_JPRB, 0.64580E+00_JPRB, 0.65693E+00_JPRB, 0.68691E+00_JPRB, 0.71307E+00_JPRB ,&
&  0.76135E+00_JPRB, 0.83696E+00_JPRB, 0.93441E+00_JPRB, 0.10742E+01_JPRB, 0.13396E+01_JPRB, 0.15190E+01_JPRB, 0.14239E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.12929E+01_JPRB, 0.11909E+01_JPRB, 0.10887E+01_JPRB, 0.90150E+00_JPRB, 0.88697E+00_JPRB, 0.81246E+00_JPRB ,&
&  0.74909E+00_JPRB, 0.70393E+00_JPRB, 0.68007E+00_JPRB, 0.66809E+00_JPRB, 0.65463E+00_JPRB, 0.63555E+00_JPRB, 0.62162E+00_JPRB ,&
&  0.61399E+00_JPRB, 0.61833E+00_JPRB, 0.62196E+00_JPRB, 0.61876E+00_JPRB, 0.62063E+00_JPRB, 0.62710E+00_JPRB, 0.63390E+00_JPRB ,&
&  0.64003E+00_JPRB, 0.65151E+00_JPRB, 0.66095E+00_JPRB, 0.69679E+00_JPRB, 0.71584E+00_JPRB, 0.71844E+00_JPRB, 0.70205E+00_JPRB ,&
&  0.69350E+00_JPRB, 0.70278E+00_JPRB, 0.69885E+00_JPRB, 0.69479E+00_JPRB, 0.68895E+00_JPRB, 0.68658E+00_JPRB, 0.68017E+00_JPRB ,&
&  0.68035E+00_JPRB, 0.65834E+00_JPRB, 0.62816E+00_JPRB, 0.61481E+00_JPRB, 0.60549E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.60307E+00_JPRB, 0.58612E+00_JPRB, 0.56323E+00_JPRB, 0.53981E+00_JPRB, 0.52801E+00_JPRB, 0.51667E+00_JPRB, 0.50245E+00_JPRB ,&
&  0.49540E+00_JPRB, 0.49187E+00_JPRB, 0.48766E+00_JPRB, 0.49955E+00_JPRB, 0.52199E+00_JPRB, 0.53318E+00_JPRB, 0.53888E+00_JPRB ,&
&  0.54562E+00_JPRB, 0.54839E+00_JPRB, 0.53534E+00_JPRB, 0.53253E+00_JPRB, 0.54120E+00_JPRB, 0.55129E+00_JPRB, 0.55632E+00_JPRB ,&
&  0.56087E+00_JPRB, 0.57001E+00_JPRB, 0.58352E+00_JPRB, 0.60905E+00_JPRB, 0.66562E+00_JPRB, 0.74937E+00_JPRB, 0.85926E+00_JPRB ,&
&  0.99043E+00_JPRB, 0.95757E+00_JPRB, 0.58241E+00_JPRB, 0.97868E+00_JPRB, 0.13285E+01_JPRB, 0.13167E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.12068E+01_JPRB, 0.11512E+01_JPRB, 0.11589E+01_JPRB, 0.10901E+01_JPRB, 0.98646E+00_JPRB, 0.89714E+00_JPRB, 0.81932E+00_JPRB ,&
&  0.75317E+00_JPRB, 0.71118E+00_JPRB, 0.69704E+00_JPRB, 0.71514E+00_JPRB, 0.74927E+00_JPRB, 0.77097E+00_JPRB, 0.81045E+00_JPRB ,&
&  0.88581E+00_JPRB, 0.98493E+00_JPRB, 0.11629E+01_JPRB, 0.13334E+01_JPRB, 0.16052E+01_JPRB, 0.15768E+01_JPRB, 0.13165E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.11232E+01_JPRB, 0.11177E+01_JPRB, 0.10122E+01_JPRB, 0.85234E+00_JPRB, 0.78638E+00_JPRB, 0.78011E+00_JPRB ,&
&  0.72464E+00_JPRB, 0.68695E+00_JPRB, 0.67093E+00_JPRB, 0.65710E+00_JPRB, 0.63927E+00_JPRB, 0.62323E+00_JPRB, 0.61355E+00_JPRB ,&
&  0.61607E+00_JPRB, 0.61850E+00_JPRB, 0.61904E+00_JPRB, 0.61893E+00_JPRB, 0.62367E+00_JPRB, 0.63078E+00_JPRB, 0.64212E+00_JPRB ,&
&  0.66190E+00_JPRB, 0.68720E+00_JPRB, 0.72752E+00_JPRB, 0.74766E+00_JPRB, 0.75383E+00_JPRB, 0.75446E+00_JPRB, 0.74619E+00_JPRB ,&
&  0.73928E+00_JPRB, 0.73579E+00_JPRB, 0.71591E+00_JPRB, 0.69742E+00_JPRB, 0.73862E+00_JPRB, 0.75143E+00_JPRB, 0.78665E+00_JPRB ,&
&  0.82032E+00_JPRB, 0.82743E+00_JPRB, 0.77274E+00_JPRB, 0.64415E+00_JPRB, 0.62529E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.56542E+00_JPRB, 0.54308E+00_JPRB, 0.51212E+00_JPRB, 0.50527E+00_JPRB, 0.50482E+00_JPRB, 0.50424E+00_JPRB, 0.51785E+00_JPRB ,&
&  0.52946E+00_JPRB, 0.54701E+00_JPRB, 0.55979E+00_JPRB, 0.58038E+00_JPRB, 0.59199E+00_JPRB, 0.59225E+00_JPRB, 0.59903E+00_JPRB ,&
&  0.60069E+00_JPRB, 0.60127E+00_JPRB, 0.60345E+00_JPRB, 0.61189E+00_JPRB, 0.62402E+00_JPRB, 0.62962E+00_JPRB, 0.63703E+00_JPRB ,&
&  0.65332E+00_JPRB, 0.67216E+00_JPRB, 0.69908E+00_JPRB, 0.76133E+00_JPRB, 0.86613E+00_JPRB, 0.99168E+00_JPRB, 0.11147E+01_JPRB ,&
&  0.11650E+01_JPRB, 0.87052E+00_JPRB, 0.68290E+00_JPRB, 0.11623E+01_JPRB, 0.12974E+01_JPRB, 0.12614E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11101E+01_JPRB, 0.10877E+01_JPRB, 0.10737E+01_JPRB, 0.10466E+01_JPRB, 0.92674E+00_JPRB, 0.85734E+00_JPRB, 0.80300E+00_JPRB ,&
&  0.77459E+00_JPRB, 0.76349E+00_JPRB, 0.77620E+00_JPRB, 0.80816E+00_JPRB, 0.84012E+00_JPRB, 0.87567E+00_JPRB, 0.95339E+00_JPRB ,&
&  0.10553E+01_JPRB, 0.11876E+01_JPRB, 0.14084E+01_JPRB, 0.14665E+01_JPRB, 0.16012E+01_JPRB, 0.13960E+01_JPRB, 0.11839E+01_JPRB ,&
&  0.99369E+00_JPRB, 0.99992E+00_JPRB, 0.10261E+01_JPRB, 0.94472E+00_JPRB, 0.81831E+00_JPRB, 0.74153E+00_JPRB, 0.74123E+00_JPRB ,&
&  0.68717E+00_JPRB, 0.66218E+00_JPRB, 0.64338E+00_JPRB, 0.62241E+00_JPRB, 0.61506E+00_JPRB, 0.61588E+00_JPRB, 0.61867E+00_JPRB ,&
&  0.62147E+00_JPRB, 0.61841E+00_JPRB, 0.61677E+00_JPRB, 0.62310E+00_JPRB, 0.63480E+00_JPRB, 0.64339E+00_JPRB, 0.66050E+00_JPRB ,&
&  0.68398E+00_JPRB, 0.71468E+00_JPRB, 0.75097E+00_JPRB, 0.77708E+00_JPRB, 0.77505E+00_JPRB, 0.75235E+00_JPRB, 0.76013E+00_JPRB ,&
&  0.76086E+00_JPRB, 0.77994E+00_JPRB, 0.78194E+00_JPRB, 0.79566E+00_JPRB, 0.80199E+00_JPRB, 0.82259E+00_JPRB, 0.82980E+00_JPRB ,&
&  0.82911E+00_JPRB, 0.80392E+00_JPRB, 0.72099E+00_JPRB, 0.61494E+00_JPRB, 0.59176E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.50733E+00_JPRB, 0.50038E+00_JPRB, 0.50395E+00_JPRB, 0.51146E+00_JPRB, 0.52341E+00_JPRB, 0.54756E+00_JPRB, 0.57487E+00_JPRB ,&
&  0.59761E+00_JPRB, 0.63577E+00_JPRB, 0.66187E+00_JPRB, 0.66382E+00_JPRB, 0.65845E+00_JPRB, 0.66053E+00_JPRB, 0.66682E+00_JPRB ,&
&  0.67876E+00_JPRB, 0.68800E+00_JPRB, 0.69467E+00_JPRB, 0.70290E+00_JPRB, 0.71464E+00_JPRB, 0.72753E+00_JPRB, 0.74445E+00_JPRB ,&
&  0.77125E+00_JPRB, 0.81576E+00_JPRB, 0.87947E+00_JPRB, 0.97280E+00_JPRB, 0.10941E+01_JPRB, 0.12195E+01_JPRB, 0.12819E+01_JPRB ,&
&  0.97088E+00_JPRB, 0.57226E+00_JPRB, 0.11119E+01_JPRB, 0.13042E+01_JPRB, 0.11925E+01_JPRB, 0.11439E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.10350E+01_JPRB, 0.10033E+01_JPRB, 0.10053E+01_JPRB, 0.90078E+00_JPRB, 0.85541E+00_JPRB, 0.83699E+00_JPRB ,&
&  0.83722E+00_JPRB, 0.84450E+00_JPRB, 0.86260E+00_JPRB, 0.88779E+00_JPRB, 0.93118E+00_JPRB, 0.99588E+00_JPRB, 0.10963E+01_JPRB ,&
&  0.12089E+01_JPRB, 0.13427E+01_JPRB, 0.15370E+01_JPRB, 0.14224E+01_JPRB, 0.14437E+01_JPRB, 0.13282E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.10046E+01_JPRB, 0.88669E+00_JPRB, 0.88495E+00_JPRB, 0.89085E+00_JPRB, 0.79989E+00_JPRB, 0.69971E+00_JPRB, 0.70396E+00_JPRB ,&
&  0.65785E+00_JPRB, 0.64111E+00_JPRB, 0.61912E+00_JPRB, 0.59850E+00_JPRB, 0.59108E+00_JPRB, 0.59692E+00_JPRB, 0.60762E+00_JPRB ,&
&  0.61195E+00_JPRB, 0.61402E+00_JPRB, 0.62026E+00_JPRB, 0.62736E+00_JPRB, 0.64378E+00_JPRB, 0.65883E+00_JPRB, 0.67133E+00_JPRB ,&
&  0.70314E+00_JPRB, 0.73539E+00_JPRB, 0.76715E+00_JPRB, 0.79586E+00_JPRB, 0.81678E+00_JPRB, 0.79921E+00_JPRB, 0.78905E+00_JPRB ,&
&  0.79910E+00_JPRB, 0.81004E+00_JPRB, 0.82363E+00_JPRB, 0.84498E+00_JPRB, 0.84763E+00_JPRB, 0.81598E+00_JPRB, 0.80559E+00_JPRB ,&
&  0.74718E+00_JPRB, 0.67698E+00_JPRB, 0.59520E+00_JPRB, 0.54673E+00_JPRB, 0.52919E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.53096E+00_JPRB, 0.54446E+00_JPRB, 0.56095E+00_JPRB, 0.58124E+00_JPRB, 0.61037E+00_JPRB, 0.64372E+00_JPRB, 0.68621E+00_JPRB ,&
&  0.72937E+00_JPRB, 0.74092E+00_JPRB, 0.74308E+00_JPRB, 0.74425E+00_JPRB, 0.74469E+00_JPRB, 0.75029E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.76995E+00_JPRB, 0.78097E+00_JPRB, 0.79241E+00_JPRB, 0.80446E+00_JPRB, 0.82010E+00_JPRB, 0.84334E+00_JPRB, 0.87556E+00_JPRB ,&
&  0.91039E+00_JPRB, 0.97298E+00_JPRB, 0.10564E+01_JPRB, 0.11496E+01_JPRB, 0.12449E+01_JPRB, 0.13169E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.83435E+00_JPRB, 0.10873E+01_JPRB, 0.12778E+01_JPRB, 0.12147E+01_JPRB, 0.11280E+01_JPRB, 0.10735E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.95397E+00_JPRB, 0.93979E+00_JPRB, 0.99130E+00_JPRB, 0.91545E+00_JPRB, 0.88325E+00_JPRB, 0.88518E+00_JPRB ,&
&  0.89908E+00_JPRB, 0.91906E+00_JPRB, 0.95054E+00_JPRB, 0.98549E+00_JPRB, 0.10400E+01_JPRB, 0.11203E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.13238E+01_JPRB, 0.14351E+01_JPRB, 0.15494E+01_JPRB, 0.13630E+01_JPRB, 0.13232E+01_JPRB, 0.13198E+01_JPRB, 0.12143E+01_JPRB ,&
&  0.94361E+00_JPRB, 0.82049E+00_JPRB, 0.84107E+00_JPRB, 0.84946E+00_JPRB, 0.77120E+00_JPRB, 0.71429E+00_JPRB, 0.66790E+00_JPRB ,&
&  0.63748E+00_JPRB, 0.61053E+00_JPRB, 0.58378E+00_JPRB, 0.57439E+00_JPRB, 0.57112E+00_JPRB, 0.57969E+00_JPRB, 0.59464E+00_JPRB ,&
&  0.60451E+00_JPRB, 0.60815E+00_JPRB, 0.61777E+00_JPRB, 0.62986E+00_JPRB, 0.65953E+00_JPRB, 0.67783E+00_JPRB, 0.68890E+00_JPRB ,&
&  0.71658E+00_JPRB, 0.73541E+00_JPRB, 0.76198E+00_JPRB, 0.79031E+00_JPRB, 0.81530E+00_JPRB, 0.83179E+00_JPRB, 0.82954E+00_JPRB ,&
&  0.83102E+00_JPRB, 0.85717E+00_JPRB, 0.88743E+00_JPRB, 0.89484E+00_JPRB, 0.87201E+00_JPRB, 0.79366E+00_JPRB, 0.72061E+00_JPRB ,&
&  0.63955E+00_JPRB, 0.61698E+00_JPRB, 0.58240E+00_JPRB, 0.55713E+00_JPRB, 0.53889E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.62569E+00_JPRB, 0.64092E+00_JPRB, 0.70738E+00_JPRB, 0.72613E+00_JPRB, 0.78901E+00_JPRB, 0.81216E+00_JPRB, 0.83444E+00_JPRB ,&
&  0.83453E+00_JPRB, 0.83458E+00_JPRB, 0.83333E+00_JPRB, 0.83521E+00_JPRB, 0.83657E+00_JPRB, 0.84321E+00_JPRB, 0.84929E+00_JPRB ,&
&  0.86039E+00_JPRB, 0.87622E+00_JPRB, 0.89517E+00_JPRB, 0.91443E+00_JPRB, 0.94074E+00_JPRB, 0.97159E+00_JPRB, 0.10095E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.11096E+01_JPRB, 0.11695E+01_JPRB, 0.12412E+01_JPRB, 0.13157E+01_JPRB, 0.13673E+01_JPRB, 0.13647E+01_JPRB ,&
&  0.10314E+01_JPRB, 0.12627E+01_JPRB, 0.12424E+01_JPRB, 0.11807E+01_JPRB, 0.10947E+01_JPRB, 0.10305E+01_JPRB, 0.10037E+01_JPRB ,&
&  0.98260E+00_JPRB, 0.92632E+00_JPRB, 0.92002E+00_JPRB, 0.94186E+00_JPRB, 0.93849E+00_JPRB, 0.90973E+00_JPRB, 0.93049E+00_JPRB ,&
&  0.96107E+00_JPRB, 0.99988E+00_JPRB, 0.10366E+01_JPRB, 0.10669E+01_JPRB, 0.11272E+01_JPRB, 0.12029E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.14621E+01_JPRB, 0.15265E+01_JPRB, 0.13459E+01_JPRB, 0.11941E+01_JPRB, 0.11281E+01_JPRB, 0.92374E+00_JPRB ,&
&  0.81121E+00_JPRB, 0.75133E+00_JPRB, 0.80907E+00_JPRB, 0.83621E+00_JPRB, 0.76111E+00_JPRB, 0.68522E+00_JPRB, 0.63794E+00_JPRB ,&
&  0.59905E+00_JPRB, 0.57169E+00_JPRB, 0.56262E+00_JPRB, 0.55185E+00_JPRB, 0.54733E+00_JPRB, 0.56197E+00_JPRB, 0.57596E+00_JPRB ,&
&  0.59901E+00_JPRB, 0.62776E+00_JPRB, 0.65312E+00_JPRB, 0.68294E+00_JPRB, 0.70214E+00_JPRB, 0.72462E+00_JPRB, 0.73698E+00_JPRB ,&
&  0.75602E+00_JPRB, 0.79109E+00_JPRB, 0.79768E+00_JPRB, 0.82544E+00_JPRB, 0.83186E+00_JPRB, 0.85802E+00_JPRB, 0.88456E+00_JPRB ,&
&  0.91287E+00_JPRB, 0.93885E+00_JPRB, 0.94474E+00_JPRB, 0.92702E+00_JPRB, 0.87406E+00_JPRB, 0.79900E+00_JPRB, 0.64016E+00_JPRB ,&
&  0.61358E+00_JPRB, 0.59502E+00_JPRB, 0.59892E+00_JPRB, 0.60209E+00_JPRB, 0.61848E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.78575E+00_JPRB, 0.83358E+00_JPRB, 0.88112E+00_JPRB, 0.93895E+00_JPRB, 0.98369E+00_JPRB, 0.96788E+00_JPRB, 0.93779E+00_JPRB ,&
&  0.91844E+00_JPRB, 0.90939E+00_JPRB, 0.91456E+00_JPRB, 0.92723E+00_JPRB, 0.93668E+00_JPRB, 0.94083E+00_JPRB, 0.95026E+00_JPRB ,&
&  0.96586E+00_JPRB, 0.98741E+00_JPRB, 0.10101E+01_JPRB, 0.10356E+01_JPRB, 0.10551E+01_JPRB, 0.10830E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11489E+01_JPRB, 0.11920E+01_JPRB, 0.12290E+01_JPRB, 0.12682E+01_JPRB, 0.13044E+01_JPRB, 0.12946E+01_JPRB, 0.97205E+00_JPRB ,&
&  0.12180E+01_JPRB, 0.12656E+01_JPRB, 0.12781E+01_JPRB, 0.12109E+01_JPRB, 0.11016E+01_JPRB, 0.10279E+01_JPRB, 0.99816E+00_JPRB ,&
&  0.97292E+00_JPRB, 0.94344E+00_JPRB, 0.91724E+00_JPRB, 0.90494E+00_JPRB, 0.98007E+00_JPRB, 0.94824E+00_JPRB, 0.97680E+00_JPRB ,&
&  0.10121E+01_JPRB, 0.10390E+01_JPRB, 0.10606E+01_JPRB, 0.10960E+01_JPRB, 0.11662E+01_JPRB, 0.12326E+01_JPRB, 0.13117E+01_JPRB ,&
&  0.13880E+01_JPRB, 0.14526E+01_JPRB, 0.14958E+01_JPRB, 0.13070E+01_JPRB, 0.12503E+01_JPRB, 0.11738E+01_JPRB, 0.91782E+00_JPRB ,&
&  0.77806E+00_JPRB, 0.75340E+00_JPRB, 0.81645E+00_JPRB, 0.83750E+00_JPRB, 0.75760E+00_JPRB, 0.66660E+00_JPRB, 0.62700E+00_JPRB ,&
&  0.58438E+00_JPRB, 0.56315E+00_JPRB, 0.55764E+00_JPRB, 0.53906E+00_JPRB, 0.53119E+00_JPRB, 0.55313E+00_JPRB, 0.58451E+00_JPRB ,&
&  0.62456E+00_JPRB, 0.67487E+00_JPRB, 0.71484E+00_JPRB, 0.74936E+00_JPRB, 0.79624E+00_JPRB, 0.87503E+00_JPRB, 0.92043E+00_JPRB ,&
&  0.84423E+00_JPRB, 0.81420E+00_JPRB, 0.87190E+00_JPRB, 0.91928E+00_JPRB, 0.97032E+00_JPRB, 0.10492E+01_JPRB, 0.11309E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.13309E+01_JPRB, 0.11318E+01_JPRB, 0.95240E+00_JPRB, 0.12765E+01_JPRB, 0.13579E+01_JPRB, 0.68839E+00_JPRB ,&
&  0.65597E+00_JPRB, 0.67316E+00_JPRB, 0.68289E+00_JPRB, 0.71488E+00_JPRB, 0.74859E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.92575E+00_JPRB, 0.95224E+00_JPRB, 0.98116E+00_JPRB, 0.99716E+00_JPRB, 0.10035E+01_JPRB, 0.98663E+00_JPRB, 0.97182E+00_JPRB ,&
&  0.96978E+00_JPRB, 0.97576E+00_JPRB, 0.98873E+00_JPRB, 0.10033E+01_JPRB, 0.10112E+01_JPRB, 0.10181E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10822E+01_JPRB, 0.11154E+01_JPRB, 0.11368E+01_JPRB, 0.11576E+01_JPRB, 0.11716E+01_JPRB, 0.11949E+01_JPRB ,&
&  0.12128E+01_JPRB, 0.12495E+01_JPRB, 0.12524E+01_JPRB, 0.12731E+01_JPRB, 0.12811E+01_JPRB, 0.13053E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.13012E+01_JPRB, 0.12795E+01_JPRB, 0.12054E+01_JPRB, 0.10986E+01_JPRB, 0.10465E+01_JPRB, 0.10121E+01_JPRB ,&
&  0.99755E+00_JPRB, 0.97416E+00_JPRB, 0.93670E+00_JPRB, 0.98217E+00_JPRB, 0.10109E+01_JPRB, 0.97107E+00_JPRB, 0.98772E+00_JPRB ,&
&  0.10087E+01_JPRB, 0.10299E+01_JPRB, 0.10574E+01_JPRB, 0.11070E+01_JPRB, 0.11791E+01_JPRB, 0.12421E+01_JPRB, 0.13083E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.14552E+01_JPRB, 0.15198E+01_JPRB, 0.13751E+01_JPRB, 0.13834E+01_JPRB, 0.12956E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.78285E+00_JPRB, 0.72089E+00_JPRB, 0.76478E+00_JPRB, 0.84710E+00_JPRB, 0.76653E+00_JPRB, 0.66402E+00_JPRB, 0.64093E+00_JPRB ,&
&  0.60348E+00_JPRB, 0.57483E+00_JPRB, 0.56644E+00_JPRB, 0.54334E+00_JPRB, 0.53072E+00_JPRB, 0.55809E+00_JPRB, 0.58528E+00_JPRB ,&
&  0.62212E+00_JPRB, 0.67123E+00_JPRB, 0.72541E+00_JPRB, 0.78090E+00_JPRB, 0.84507E+00_JPRB, 0.86463E+00_JPRB, 0.81380E+00_JPRB ,&
&  0.85635E+00_JPRB, 0.89547E+00_JPRB, 0.96006E+00_JPRB, 0.10719E+01_JPRB, 0.11516E+01_JPRB, 0.13737E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.18364E+01_JPRB, 0.17278E+01_JPRB, 0.20580E+01_JPRB, 0.28660E+01_JPRB, 0.26410E+01_JPRB, 0.21251E+01_JPRB, 0.79324E+00_JPRB ,&
&  0.78301E+00_JPRB, 0.78164E+00_JPRB, 0.81363E+00_JPRB, 0.85383E+00_JPRB, 0.90047E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.92455E+00_JPRB, 0.94638E+00_JPRB, 0.96742E+00_JPRB, 0.98707E+00_JPRB, 0.10011E+01_JPRB, 0.10108E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.10478E+01_JPRB, 0.10598E+01_JPRB, 0.10660E+01_JPRB, 0.10717E+01_JPRB, 0.10887E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.11440E+01_JPRB, 0.11583E+01_JPRB, 0.11909E+01_JPRB, 0.11701E+01_JPRB, 0.11485E+01_JPRB, 0.11498E+01_JPRB, 0.11695E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11594E+01_JPRB, 0.11510E+01_JPRB, 0.11746E+01_JPRB, 0.12018E+01_JPRB, 0.12332E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.12908E+01_JPRB, 0.12467E+01_JPRB, 0.11743E+01_JPRB, 0.10616E+01_JPRB, 0.10262E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.99395E+00_JPRB, 0.98024E+00_JPRB, 0.97130E+00_JPRB, 0.97316E+00_JPRB, 0.96882E+00_JPRB, 0.97857E+00_JPRB ,&
&  0.99299E+00_JPRB, 0.10060E+01_JPRB, 0.10365E+01_JPRB, 0.10778E+01_JPRB, 0.11425E+01_JPRB, 0.12072E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.13545E+01_JPRB, 0.12827E+01_JPRB, 0.12887E+01_JPRB, 0.13707E+01_JPRB, 0.12777E+01_JPRB, 0.10399E+01_JPRB ,&
&  0.76806E+00_JPRB, 0.73264E+00_JPRB, 0.75957E+00_JPRB, 0.84632E+00_JPRB, 0.77246E+00_JPRB, 0.69922E+00_JPRB, 0.66749E+00_JPRB ,&
&  0.60268E+00_JPRB, 0.54058E+00_JPRB, 0.55056E+00_JPRB, 0.54521E+00_JPRB, 0.53469E+00_JPRB, 0.55370E+00_JPRB, 0.57408E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.63967E+00_JPRB, 0.68274E+00_JPRB, 0.72584E+00_JPRB, 0.74841E+00_JPRB, 0.71320E+00_JPRB, 0.80808E+00_JPRB ,&
&  0.80192E+00_JPRB, 0.79294E+00_JPRB, 0.83486E+00_JPRB, 0.71663E+00_JPRB, 0.10082E+01_JPRB, 0.10786E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.15835E+01_JPRB, 0.19624E+01_JPRB, 0.15232E+01_JPRB, 0.11870E+01_JPRB, 0.85459E+00_JPRB, 0.79766E+00_JPRB ,&
&  0.80098E+00_JPRB, 0.81399E+00_JPRB, 0.83952E+00_JPRB, 0.87089E+00_JPRB, 0.89895E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.92502E+00_JPRB, 0.94549E+00_JPRB, 0.96539E+00_JPRB, 0.98729E+00_JPRB, 0.10090E+01_JPRB, 0.10241E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.10781E+01_JPRB, 0.10861E+01_JPRB, 0.10957E+01_JPRB, 0.11114E+01_JPRB, 0.11447E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11749E+01_JPRB, 0.11827E+01_JPRB, 0.11852E+01_JPRB, 0.11732E+01_JPRB, 0.11438E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.11401E+01_JPRB, 0.11378E+01_JPRB, 0.11571E+01_JPRB, 0.11896E+01_JPRB, 0.12559E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.12091E+01_JPRB, 0.12016E+01_JPRB, 0.10960E+01_JPRB, 0.98000E+00_JPRB, 0.94513E+00_JPRB, 0.93647E+00_JPRB ,&
&  0.94255E+00_JPRB, 0.92458E+00_JPRB, 0.92378E+00_JPRB, 0.92788E+00_JPRB, 0.93785E+00_JPRB, 0.95907E+00_JPRB, 0.96244E+00_JPRB ,&
&  0.97286E+00_JPRB, 0.99011E+00_JPRB, 0.10131E+01_JPRB, 0.10460E+01_JPRB, 0.10851E+01_JPRB, 0.11197E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.11708E+01_JPRB, 0.10720E+01_JPRB, 0.10755E+01_JPRB, 0.11291E+01_JPRB, 0.10546E+01_JPRB, 0.90228E+00_JPRB ,&
&  0.79368E+00_JPRB, 0.70990E+00_JPRB, 0.67873E+00_JPRB, 0.81056E+00_JPRB, 0.77998E+00_JPRB, 0.72441E+00_JPRB, 0.65963E+00_JPRB ,&
&  0.56899E+00_JPRB, 0.51209E+00_JPRB, 0.51103E+00_JPRB, 0.54456E+00_JPRB, 0.53597E+00_JPRB, 0.53303E+00_JPRB, 0.53655E+00_JPRB ,&
&  0.54340E+00_JPRB, 0.54841E+00_JPRB, 0.56621E+00_JPRB, 0.60785E+00_JPRB, 0.64347E+00_JPRB, 0.59866E+00_JPRB, 0.74462E+00_JPRB ,&
&  0.67234E+00_JPRB, 0.55029E+00_JPRB, 0.63080E+00_JPRB, 0.76302E+00_JPRB, 0.79800E+00_JPRB, 0.81242E+00_JPRB, 0.91717E+00_JPRB ,&
&  0.10628E+01_JPRB, 0.11914E+01_JPRB, 0.11962E+01_JPRB, 0.97325E+00_JPRB, 0.85250E+00_JPRB, 0.79936E+00_JPRB, 0.78954E+00_JPRB ,&
&  0.79626E+00_JPRB, 0.81327E+00_JPRB, 0.84135E+00_JPRB, 0.87367E+00_JPRB, 0.90206E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.91198E+00_JPRB, 0.93455E+00_JPRB, 0.95246E+00_JPRB, 0.96661E+00_JPRB, 0.98023E+00_JPRB, 0.99683E+00_JPRB, 0.10123E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10397E+01_JPRB, 0.10477E+01_JPRB, 0.10624E+01_JPRB, 0.10789E+01_JPRB, 0.11139E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.11748E+01_JPRB, 0.11726E+01_JPRB, 0.11869E+01_JPRB, 0.11794E+01_JPRB, 0.11588E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11504E+01_JPRB, 0.11477E+01_JPRB, 0.11488E+01_JPRB, 0.11511E+01_JPRB, 0.11230E+01_JPRB, 0.11862E+01_JPRB ,&
&  0.84154E+00_JPRB, 0.11155E+01_JPRB, 0.11656E+01_JPRB, 0.10328E+01_JPRB, 0.96880E+00_JPRB, 0.88143E+00_JPRB, 0.86920E+00_JPRB ,&
&  0.87862E+00_JPRB, 0.87476E+00_JPRB, 0.88414E+00_JPRB, 0.90523E+00_JPRB, 0.92321E+00_JPRB, 0.94240E+00_JPRB, 0.94604E+00_JPRB ,&
&  0.96442E+00_JPRB, 0.96695E+00_JPRB, 0.97646E+00_JPRB, 0.98642E+00_JPRB, 0.10168E+01_JPRB, 0.10166E+01_JPRB, 0.10069E+01_JPRB ,&
&  0.10084E+01_JPRB, 0.10097E+01_JPRB, 0.79853E+00_JPRB, 0.77334E+00_JPRB, 0.82460E+00_JPRB, 0.80342E+00_JPRB, 0.72109E+00_JPRB ,&
&  0.68408E+00_JPRB, 0.67454E+00_JPRB, 0.72933E+00_JPRB, 0.73826E+00_JPRB, 0.74844E+00_JPRB, 0.73163E+00_JPRB, 0.70368E+00_JPRB ,&
&  0.62212E+00_JPRB, 0.53538E+00_JPRB, 0.50952E+00_JPRB, 0.52907E+00_JPRB, 0.54143E+00_JPRB, 0.54402E+00_JPRB, 0.50768E+00_JPRB ,&
&  0.47480E+00_JPRB, 0.44608E+00_JPRB, 0.43806E+00_JPRB, 0.45298E+00_JPRB, 0.39448E+00_JPRB, 0.51255E+00_JPRB, 0.66457E+00_JPRB ,&
&  0.57332E+00_JPRB, 0.46471E+00_JPRB, 0.48098E+00_JPRB, 0.64630E+00_JPRB, 0.65252E+00_JPRB, 0.61299E+00_JPRB, 0.65177E+00_JPRB ,&
&  0.75288E+00_JPRB, 0.80363E+00_JPRB, 0.82037E+00_JPRB, 0.75071E+00_JPRB, 0.76120E+00_JPRB, 0.74101E+00_JPRB, 0.74219E+00_JPRB ,&
&  0.75866E+00_JPRB, 0.78691E+00_JPRB, 0.82169E+00_JPRB, 0.85554E+00_JPRB, 0.88509E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.86393E+00_JPRB, 0.88240E+00_JPRB, 0.89415E+00_JPRB, 0.90603E+00_JPRB, 0.92323E+00_JPRB, 0.94111E+00_JPRB, 0.94997E+00_JPRB ,&
&  0.96823E+00_JPRB, 0.98464E+00_JPRB, 0.99175E+00_JPRB, 0.10103E+01_JPRB, 0.10183E+01_JPRB, 0.10696E+01_JPRB, 0.10892E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11479E+01_JPRB, 0.11537E+01_JPRB, 0.11557E+01_JPRB, 0.11689E+01_JPRB, 0.11674E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.12241E+01_JPRB, 0.12760E+01_JPRB, 0.13260E+01_JPRB, 0.13568E+01_JPRB, 0.14542E+01_JPRB, 0.15838E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.97229E+00_JPRB, 0.10168E+01_JPRB, 0.87772E+00_JPRB, 0.87351E+00_JPRB, 0.87484E+00_JPRB, 0.83657E+00_JPRB ,&
&  0.82063E+00_JPRB, 0.84757E+00_JPRB, 0.87737E+00_JPRB, 0.89366E+00_JPRB, 0.89837E+00_JPRB, 0.91045E+00_JPRB, 0.90341E+00_JPRB ,&
&  0.91126E+00_JPRB, 0.91758E+00_JPRB, 0.93886E+00_JPRB, 0.96927E+00_JPRB, 0.90832E+00_JPRB, 0.98022E+00_JPRB, 0.99523E+00_JPRB ,&
&  0.99903E+00_JPRB, 0.91425E+00_JPRB, 0.67990E+00_JPRB, 0.63695E+00_JPRB, 0.65704E+00_JPRB, 0.63275E+00_JPRB, 0.59044E+00_JPRB ,&
&  0.61747E+00_JPRB, 0.61076E+00_JPRB, 0.53256E+00_JPRB, 0.64740E+00_JPRB, 0.71893E+00_JPRB, 0.74956E+00_JPRB, 0.72904E+00_JPRB ,&
&  0.68884E+00_JPRB, 0.61174E+00_JPRB, 0.58271E+00_JPRB, 0.56921E+00_JPRB, 0.56217E+00_JPRB, 0.53922E+00_JPRB, 0.52698E+00_JPRB ,&
&  0.48878E+00_JPRB, 0.43590E+00_JPRB, 0.41053E+00_JPRB, 0.38698E+00_JPRB, 0.42518E+00_JPRB, 0.37323E+00_JPRB, 0.56847E+00_JPRB ,&
&  0.46553E+00_JPRB, 0.43957E+00_JPRB, 0.41707E+00_JPRB, 0.56821E+00_JPRB, 0.53235E+00_JPRB, 0.47218E+00_JPRB, 0.52459E+00_JPRB ,&
&  0.58181E+00_JPRB, 0.61149E+00_JPRB, 0.62193E+00_JPRB, 0.63744E+00_JPRB, 0.68005E+00_JPRB, 0.70179E+00_JPRB, 0.72047E+00_JPRB ,&
&  0.72173E+00_JPRB, 0.75203E+00_JPRB, 0.79369E+00_JPRB, 0.82503E+00_JPRB, 0.83729E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.73905E+00_JPRB, 0.75278E+00_JPRB, 0.77087E+00_JPRB, 0.79008E+00_JPRB, 0.80088E+00_JPRB, 0.81921E+00_JPRB, 0.83729E+00_JPRB ,&
&  0.86164E+00_JPRB, 0.89316E+00_JPRB, 0.91156E+00_JPRB, 0.91783E+00_JPRB, 0.93418E+00_JPRB, 0.95417E+00_JPRB, 0.96413E+00_JPRB ,&
&  0.98652E+00_JPRB, 0.99566E+00_JPRB, 0.10476E+01_JPRB, 0.10734E+01_JPRB, 0.11106E+01_JPRB, 0.11617E+01_JPRB, 0.11697E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.11523E+01_JPRB, 0.11655E+01_JPRB, 0.11641E+01_JPRB, 0.12397E+01_JPRB, 0.12449E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.93902E+00_JPRB, 0.86403E+00_JPRB, 0.10172E+01_JPRB, 0.85731E+00_JPRB, 0.84673E+00_JPRB, 0.83014E+00_JPRB, 0.75765E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.79400E+00_JPRB, 0.83717E+00_JPRB, 0.87337E+00_JPRB, 0.91757E+00_JPRB, 0.92670E+00_JPRB, 0.91750E+00_JPRB ,&
&  0.91895E+00_JPRB, 0.93140E+00_JPRB, 0.93706E+00_JPRB, 0.86316E+00_JPRB, 0.78668E+00_JPRB, 0.83944E+00_JPRB, 0.85846E+00_JPRB ,&
&  0.85803E+00_JPRB, 0.83761E+00_JPRB, 0.67808E+00_JPRB, 0.61821E+00_JPRB, 0.62679E+00_JPRB, 0.55548E+00_JPRB, 0.52585E+00_JPRB ,&
&  0.56674E+00_JPRB, 0.50383E+00_JPRB, 0.34246E+00_JPRB, 0.48274E+00_JPRB, 0.62786E+00_JPRB, 0.72031E+00_JPRB, 0.76055E+00_JPRB ,&
&  0.74395E+00_JPRB, 0.67851E+00_JPRB, 0.64657E+00_JPRB, 0.64908E+00_JPRB, 0.65319E+00_JPRB, 0.59691E+00_JPRB, 0.57048E+00_JPRB ,&
&  0.51358E+00_JPRB, 0.44424E+00_JPRB, 0.39638E+00_JPRB, 0.38716E+00_JPRB, 0.37914E+00_JPRB, 0.39151E+00_JPRB, 0.47128E+00_JPRB ,&
&  0.45883E+00_JPRB, 0.43109E+00_JPRB, 0.50371E+00_JPRB, 0.54640E+00_JPRB, 0.38561E+00_JPRB, 0.43805E+00_JPRB, 0.46601E+00_JPRB ,&
&  0.49314E+00_JPRB, 0.51157E+00_JPRB, 0.52641E+00_JPRB, 0.55108E+00_JPRB, 0.58569E+00_JPRB, 0.59853E+00_JPRB, 0.60325E+00_JPRB ,&
&  0.62038E+00_JPRB, 0.64867E+00_JPRB, 0.66950E+00_JPRB, 0.69516E+00_JPRB, 0.71820E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.57476E+00_JPRB, 0.57457E+00_JPRB, 0.57785E+00_JPRB, 0.58456E+00_JPRB, 0.58552E+00_JPRB, 0.58436E+00_JPRB, 0.59771E+00_JPRB ,&
&  0.62215E+00_JPRB, 0.66681E+00_JPRB, 0.72970E+00_JPRB, 0.76550E+00_JPRB, 0.74830E+00_JPRB, 0.70023E+00_JPRB, 0.68264E+00_JPRB ,&
&  0.67969E+00_JPRB, 0.67501E+00_JPRB, 0.70241E+00_JPRB, 0.75048E+00_JPRB, 0.81418E+00_JPRB, 0.83878E+00_JPRB, 0.89948E+00_JPRB ,&
&  0.90128E+00_JPRB, 0.91803E+00_JPRB, 0.10005E+01_JPRB, 0.10706E+01_JPRB, 0.16153E+01_JPRB, 0.15307E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.10633E+01_JPRB, 0.10617E+01_JPRB, 0.98424E+00_JPRB, 0.98666E+00_JPRB, 0.95071E+00_JPRB, 0.97049E+00_JPRB ,&
&  0.95851E+00_JPRB, 0.10099E+01_JPRB, 0.10487E+01_JPRB, 0.10971E+01_JPRB, 0.11292E+01_JPRB, 0.11325E+01_JPRB, 0.11468E+01_JPRB ,&
&  0.11545E+01_JPRB, 0.11539E+01_JPRB, 0.11368E+01_JPRB, 0.97041E+00_JPRB, 0.98409E+00_JPRB, 0.98567E+00_JPRB, 0.96183E+00_JPRB ,&
&  0.92442E+00_JPRB, 0.89276E+00_JPRB, 0.92741E+00_JPRB, 0.86107E+00_JPRB, 0.79407E+00_JPRB, 0.69779E+00_JPRB, 0.61056E+00_JPRB ,&
&  0.56545E+00_JPRB, 0.44198E+00_JPRB, 0.30836E+00_JPRB, 0.46664E+00_JPRB, 0.64026E+00_JPRB, 0.65394E+00_JPRB, 0.77726E+00_JPRB ,&
&  0.80361E+00_JPRB, 0.77209E+00_JPRB, 0.74365E+00_JPRB, 0.73730E+00_JPRB, 0.69301E+00_JPRB, 0.66651E+00_JPRB, 0.58515E+00_JPRB ,&
&  0.54708E+00_JPRB, 0.45957E+00_JPRB, 0.40625E+00_JPRB, 0.38709E+00_JPRB, 0.37685E+00_JPRB, 0.36751E+00_JPRB, 0.42484E+00_JPRB ,&
&  0.48821E+00_JPRB, 0.49244E+00_JPRB, 0.53315E+00_JPRB, 0.56427E+00_JPRB, 0.45854E+00_JPRB, 0.42045E+00_JPRB, 0.42573E+00_JPRB ,&
&  0.43249E+00_JPRB, 0.44764E+00_JPRB, 0.47408E+00_JPRB, 0.49282E+00_JPRB, 0.50668E+00_JPRB, 0.51884E+00_JPRB, 0.53360E+00_JPRB ,&
&  0.54342E+00_JPRB, 0.55533E+00_JPRB, 0.56051E+00_JPRB, 0.56604E+00_JPRB, 0.57159E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.61098E+00_JPRB, 0.61339E+00_JPRB, 0.60117E+00_JPRB, 0.61756E+00_JPRB, 0.64755E+00_JPRB, 0.71365E+00_JPRB, 0.78073E+00_JPRB ,&
&  0.86808E+00_JPRB, 0.97445E+00_JPRB, 0.11061E+01_JPRB, 0.12159E+01_JPRB, 0.12429E+01_JPRB, 0.12167E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.96064E+00_JPRB, 0.81381E+00_JPRB, 0.73122E+00_JPRB, 0.70660E+00_JPRB, 0.75147E+00_JPRB, 0.77926E+00_JPRB, 0.84691E+00_JPRB ,&
&  0.95147E+00_JPRB, 0.11256E+01_JPRB, 0.13195E+01_JPRB, 0.17634E+01_JPRB, 0.21139E+01_JPRB, 0.14197E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.13488E+01_JPRB, 0.14309E+01_JPRB, 0.12257E+01_JPRB, 0.12367E+01_JPRB, 0.13498E+01_JPRB, 0.14474E+01_JPRB, 0.15656E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.16823E+01_JPRB, 0.17238E+01_JPRB, 0.17453E+01_JPRB, 0.17658E+01_JPRB, 0.17598E+01_JPRB, 0.17289E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.15961E+01_JPRB, 0.15566E+01_JPRB, 0.15514E+01_JPRB, 0.15525E+01_JPRB, 0.14874E+01_JPRB, 0.13685E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.12351E+01_JPRB, 0.13006E+01_JPRB, 0.12635E+01_JPRB, 0.11215E+01_JPRB, 0.10198E+01_JPRB, 0.86884E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.64532E+00_JPRB, 0.51576E+00_JPRB, 0.79538E+00_JPRB, 0.91570E+00_JPRB, 0.87249E+00_JPRB, 0.84635E+00_JPRB ,&
&  0.87667E+00_JPRB, 0.85352E+00_JPRB, 0.82969E+00_JPRB, 0.79257E+00_JPRB, 0.72123E+00_JPRB, 0.75499E+00_JPRB, 0.59401E+00_JPRB ,&
&  0.57862E+00_JPRB, 0.48038E+00_JPRB, 0.43976E+00_JPRB, 0.44279E+00_JPRB, 0.44860E+00_JPRB, 0.40138E+00_JPRB, 0.43301E+00_JPRB ,&
&  0.47897E+00_JPRB, 0.70906E+00_JPRB, 0.92439E+00_JPRB, 0.13093E+01_JPRB, 0.17210E+01_JPRB, 0.46780E+00_JPRB, 0.44763E+00_JPRB ,&
&  0.44213E+00_JPRB, 0.44914E+00_JPRB, 0.46872E+00_JPRB, 0.48046E+00_JPRB, 0.49928E+00_JPRB, 0.51034E+00_JPRB, 0.51353E+00_JPRB ,&
&  0.52223E+00_JPRB, 0.54288E+00_JPRB, 0.56404E+00_JPRB, 0.57506E+00_JPRB, 0.59669E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.88814E+00_JPRB, 0.93003E+00_JPRB, 0.93595E+00_JPRB, 0.10111E+01_JPRB, 0.10846E+01_JPRB, 0.11203E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.13152E+01_JPRB, 0.13931E+01_JPRB, 0.14385E+01_JPRB, 0.14838E+01_JPRB, 0.15526E+01_JPRB, 0.15630E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.14898E+01_JPRB, 0.14637E+01_JPRB, 0.14331E+01_JPRB, 0.14237E+01_JPRB, 0.14459E+01_JPRB, 0.15865E+01_JPRB ,&
&  0.16929E+01_JPRB, 0.16929E+01_JPRB, 0.16921E+01_JPRB, 0.14417E+01_JPRB, 0.10204E+01_JPRB, 0.12867E+01_JPRB, 0.13095E+01_JPRB ,&
&  0.14298E+01_JPRB, 0.15035E+01_JPRB, 0.15047E+01_JPRB, 0.15863E+01_JPRB, 0.16709E+01_JPRB, 0.17768E+01_JPRB, 0.18837E+01_JPRB ,&
&  0.19565E+01_JPRB, 0.20380E+01_JPRB, 0.21247E+01_JPRB, 0.21898E+01_JPRB, 0.22590E+01_JPRB, 0.23518E+01_JPRB, 0.23701E+01_JPRB ,&
&  0.23604E+01_JPRB, 0.22837E+01_JPRB, 0.22108E+01_JPRB, 0.21865E+01_JPRB, 0.20542E+01_JPRB, 0.19076E+01_JPRB, 0.17148E+01_JPRB ,&
&  0.16210E+01_JPRB, 0.16011E+01_JPRB, 0.16544E+01_JPRB, 0.16702E+01_JPRB, 0.15899E+01_JPRB, 0.14523E+01_JPRB, 0.13003E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.10874E+01_JPRB, 0.11064E+01_JPRB, 0.11880E+01_JPRB, 0.84136E+00_JPRB, 0.87452E+00_JPRB, 0.10177E+01_JPRB ,&
&  0.96283E+00_JPRB, 0.89785E+00_JPRB, 0.85101E+00_JPRB, 0.75456E+00_JPRB, 0.72081E+00_JPRB, 0.60125E+00_JPRB, 0.63502E+00_JPRB ,&
&  0.63830E+00_JPRB, 0.53084E+00_JPRB, 0.48800E+00_JPRB, 0.51541E+00_JPRB, 0.52615E+00_JPRB, 0.47495E+00_JPRB, 0.46350E+00_JPRB ,&
&  0.50420E+00_JPRB, 0.64069E+00_JPRB, 0.74561E+00_JPRB, 0.88285E+00_JPRB, 0.94353E+00_JPRB, 0.51716E+00_JPRB, 0.44216E+00_JPRB ,&
&  0.42769E+00_JPRB, 0.44012E+00_JPRB, 0.46214E+00_JPRB, 0.48707E+00_JPRB, 0.51895E+00_JPRB, 0.55887E+00_JPRB, 0.59933E+00_JPRB ,&
&  0.63303E+00_JPRB, 0.65803E+00_JPRB, 0.69959E+00_JPRB, 0.75556E+00_JPRB, 0.83138E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.12064E+01_JPRB, 0.12912E+01_JPRB, 0.13667E+01_JPRB, 0.13528E+01_JPRB, 0.13445E+01_JPRB, 0.13115E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.13358E+01_JPRB, 0.13926E+01_JPRB, 0.14740E+01_JPRB, 0.15362E+01_JPRB, 0.15780E+01_JPRB, 0.16591E+01_JPRB, 0.17525E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.18075E+01_JPRB, 0.18242E+01_JPRB, 0.18428E+01_JPRB, 0.18061E+01_JPRB, 0.18108E+01_JPRB, 0.17924E+01_JPRB ,&
&  0.13688E+01_JPRB, 0.10540E+01_JPRB, 0.12464E+01_JPRB, 0.14140E+01_JPRB, 0.13710E+01_JPRB, 0.14413E+01_JPRB, 0.15382E+01_JPRB ,&
&  0.16932E+01_JPRB, 0.17280E+01_JPRB, 0.17451E+01_JPRB, 0.17525E+01_JPRB, 0.17491E+01_JPRB, 0.18656E+01_JPRB, 0.19864E+01_JPRB ,&
&  0.20816E+01_JPRB, 0.21694E+01_JPRB, 0.22589E+01_JPRB, 0.23512E+01_JPRB, 0.24289E+01_JPRB, 0.25144E+01_JPRB, 0.25674E+01_JPRB ,&
&  0.25880E+01_JPRB, 0.25384E+01_JPRB, 0.24793E+01_JPRB, 0.23530E+01_JPRB, 0.22207E+01_JPRB, 0.20996E+01_JPRB, 0.19692E+01_JPRB ,&
&  0.19589E+01_JPRB, 0.20226E+01_JPRB, 0.21390E+01_JPRB, 0.22059E+01_JPRB, 0.22031E+01_JPRB, 0.20530E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.15178E+01_JPRB, 0.15798E+01_JPRB, 0.16234E+01_JPRB, 0.12405E+01_JPRB, 0.10993E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.10041E+01_JPRB, 0.88513E+00_JPRB, 0.82437E+00_JPRB, 0.85916E+00_JPRB, 0.76222E+00_JPRB, 0.76431E+00_JPRB ,&
&  0.70543E+00_JPRB, 0.59773E+00_JPRB, 0.51931E+00_JPRB, 0.53102E+00_JPRB, 0.44909E+00_JPRB, 0.44989E+00_JPRB, 0.49265E+00_JPRB ,&
&  0.56333E+00_JPRB, 0.58711E+00_JPRB, 0.64081E+00_JPRB, 0.64096E+00_JPRB, 0.60451E+00_JPRB, 0.48150E+00_JPRB, 0.44317E+00_JPRB ,&
&  0.42832E+00_JPRB, 0.44372E+00_JPRB, 0.48050E+00_JPRB, 0.50960E+00_JPRB, 0.55681E+00_JPRB, 0.61170E+00_JPRB, 0.70752E+00_JPRB ,&
&  0.79297E+00_JPRB, 0.87247E+00_JPRB, 0.93536E+00_JPRB, 0.10140E+01_JPRB, 0.11019E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.11803E+01_JPRB, 0.13065E+01_JPRB, 0.14345E+01_JPRB, 0.15236E+01_JPRB, 0.14879E+01_JPRB, 0.14366E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.13946E+01_JPRB, 0.14285E+01_JPRB, 0.15117E+01_JPRB, 0.15881E+01_JPRB, 0.16430E+01_JPRB, 0.17274E+01_JPRB, 0.18530E+01_JPRB ,&
&  0.19612E+01_JPRB, 0.20452E+01_JPRB, 0.21109E+01_JPRB, 0.21763E+01_JPRB, 0.21669E+01_JPRB, 0.21625E+01_JPRB, 0.18570E+01_JPRB ,&
&  0.12600E+01_JPRB, 0.12594E+01_JPRB, 0.16157E+01_JPRB, 0.15567E+01_JPRB, 0.15651E+01_JPRB, 0.16305E+01_JPRB, 0.17566E+01_JPRB ,&
&  0.19271E+01_JPRB, 0.19475E+01_JPRB, 0.19147E+01_JPRB, 0.18672E+01_JPRB, 0.18402E+01_JPRB, 0.19324E+01_JPRB, 0.20588E+01_JPRB ,&
&  0.21593E+01_JPRB, 0.22174E+01_JPRB, 0.22972E+01_JPRB, 0.24087E+01_JPRB, 0.25494E+01_JPRB, 0.26747E+01_JPRB, 0.27681E+01_JPRB ,&
&  0.27352E+01_JPRB, 0.27382E+01_JPRB, 0.26081E+01_JPRB, 0.23781E+01_JPRB, 0.23083E+01_JPRB, 0.23292E+01_JPRB, 0.23570E+01_JPRB ,&
&  0.24506E+01_JPRB, 0.26371E+01_JPRB, 0.28160E+01_JPRB, 0.28330E+01_JPRB, 0.26542E+01_JPRB, 0.26213E+01_JPRB, 0.24118E+01_JPRB ,&
&  0.23810E+01_JPRB, 0.21321E+01_JPRB, 0.21535E+01_JPRB, 0.18644E+01_JPRB, 0.17533E+01_JPRB, 0.15975E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.13273E+01_JPRB, 0.11532E+01_JPRB, 0.10590E+01_JPRB, 0.11314E+01_JPRB, 0.11341E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.88942E+00_JPRB, 0.76264E+00_JPRB, 0.58731E+00_JPRB, 0.51895E+00_JPRB, 0.40015E+00_JPRB, 0.39522E+00_JPRB, 0.50552E+00_JPRB ,&
&  0.66511E+00_JPRB, 0.63887E+00_JPRB, 0.66524E+00_JPRB, 0.64211E+00_JPRB, 0.55523E+00_JPRB, 0.46081E+00_JPRB, 0.42415E+00_JPRB ,&
&  0.42591E+00_JPRB, 0.47002E+00_JPRB, 0.50980E+00_JPRB, 0.54166E+00_JPRB, 0.60045E+00_JPRB, 0.66459E+00_JPRB, 0.73123E+00_JPRB ,&
&  0.77646E+00_JPRB, 0.83743E+00_JPRB, 0.90087E+00_JPRB, 0.96476E+00_JPRB, 0.10602E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.90697E+00_JPRB, 0.96114E+00_JPRB, 0.10129E+01_JPRB, 0.10562E+01_JPRB, 0.10948E+01_JPRB, 0.10793E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.11863E+01_JPRB, 0.13140E+01_JPRB, 0.14452E+01_JPRB, 0.15377E+01_JPRB, 0.15736E+01_JPRB, 0.16468E+01_JPRB, 0.17969E+01_JPRB ,&
&  0.19380E+01_JPRB, 0.20900E+01_JPRB, 0.22220E+01_JPRB, 0.23509E+01_JPRB, 0.23382E+01_JPRB, 0.22455E+01_JPRB, 0.17129E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.18580E+01_JPRB, 0.15405E+01_JPRB, 0.15935E+01_JPRB, 0.17143E+01_JPRB, 0.18667E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.19987E+01_JPRB, 0.20104E+01_JPRB, 0.19612E+01_JPRB, 0.18978E+01_JPRB, 0.18556E+01_JPRB, 0.19096E+01_JPRB, 0.20108E+01_JPRB ,&
&  0.21134E+01_JPRB, 0.21866E+01_JPRB, 0.22874E+01_JPRB, 0.24415E+01_JPRB, 0.26318E+01_JPRB, 0.28288E+01_JPRB, 0.29403E+01_JPRB ,&
&  0.29361E+01_JPRB, 0.29234E+01_JPRB, 0.27171E+01_JPRB, 0.24564E+01_JPRB, 0.23926E+01_JPRB, 0.25779E+01_JPRB, 0.28256E+01_JPRB ,&
&  0.29651E+01_JPRB, 0.30800E+01_JPRB, 0.34649E+01_JPRB, 0.36419E+01_JPRB, 0.35083E+01_JPRB, 0.33450E+01_JPRB, 0.28868E+01_JPRB ,&
&  0.30427E+01_JPRB, 0.28059E+01_JPRB, 0.27474E+01_JPRB, 0.23702E+01_JPRB, 0.22515E+01_JPRB, 0.22490E+01_JPRB, 0.22294E+01_JPRB ,&
&  0.20414E+01_JPRB, 0.19470E+01_JPRB, 0.16833E+01_JPRB, 0.15351E+01_JPRB, 0.15229E+01_JPRB, 0.15741E+01_JPRB, 0.14375E+01_JPRB ,&
&  0.12135E+01_JPRB, 0.85132E+00_JPRB, 0.65383E+00_JPRB, 0.53341E+00_JPRB, 0.41775E+00_JPRB, 0.37534E+00_JPRB, 0.50323E+00_JPRB ,&
&  0.65624E+00_JPRB, 0.70218E+00_JPRB, 0.79270E+00_JPRB, 0.77422E+00_JPRB, 0.64070E+00_JPRB, 0.50645E+00_JPRB, 0.44609E+00_JPRB ,&
&  0.45670E+00_JPRB, 0.50908E+00_JPRB, 0.54441E+00_JPRB, 0.57609E+00_JPRB, 0.62496E+00_JPRB, 0.64155E+00_JPRB, 0.69588E+00_JPRB ,&
&  0.71750E+00_JPRB, 0.75621E+00_JPRB, 0.79310E+00_JPRB, 0.81079E+00_JPRB, 0.84972E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.70133E+00_JPRB, 0.72868E+00_JPRB, 0.77506E+00_JPRB, 0.80902E+00_JPRB, 0.83456E+00_JPRB, 0.86951E+00_JPRB, 0.88407E+00_JPRB ,&
&  0.94384E+00_JPRB, 0.10577E+01_JPRB, 0.12009E+01_JPRB, 0.13581E+01_JPRB, 0.14109E+01_JPRB, 0.14345E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.17647E+01_JPRB, 0.20599E+01_JPRB, 0.23774E+01_JPRB, 0.26497E+01_JPRB, 0.26917E+01_JPRB, 0.25238E+01_JPRB, 0.17788E+01_JPRB ,&
&  0.14665E+01_JPRB, 0.18471E+01_JPRB, 0.16785E+01_JPRB, 0.16836E+01_JPRB, 0.18536E+01_JPRB, 0.20228E+01_JPRB, 0.19529E+01_JPRB ,&
&  0.19205E+01_JPRB, 0.19103E+01_JPRB, 0.18305E+01_JPRB, 0.17963E+01_JPRB, 0.18055E+01_JPRB, 0.18212E+01_JPRB, 0.18762E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.20367E+01_JPRB, 0.21746E+01_JPRB, 0.24023E+01_JPRB, 0.26573E+01_JPRB, 0.28827E+01_JPRB, 0.30420E+01_JPRB ,&
&  0.31150E+01_JPRB, 0.30663E+01_JPRB, 0.29806E+01_JPRB, 0.25458E+01_JPRB, 0.25061E+01_JPRB, 0.27760E+01_JPRB, 0.32311E+01_JPRB ,&
&  0.34686E+01_JPRB, 0.36028E+01_JPRB, 0.41173E+01_JPRB, 0.45618E+01_JPRB, 0.45063E+01_JPRB, 0.40404E+01_JPRB, 0.35442E+01_JPRB ,&
&  0.35243E+01_JPRB, 0.34098E+01_JPRB, 0.31564E+01_JPRB, 0.29589E+01_JPRB, 0.29145E+01_JPRB, 0.30404E+01_JPRB, 0.31942E+01_JPRB ,&
&  0.28444E+01_JPRB, 0.26005E+01_JPRB, 0.23169E+01_JPRB, 0.20908E+01_JPRB, 0.19078E+01_JPRB, 0.18178E+01_JPRB, 0.18417E+01_JPRB ,&
&  0.16307E+01_JPRB, 0.11584E+01_JPRB, 0.82687E+00_JPRB, 0.54438E+00_JPRB, 0.41211E+00_JPRB, 0.38142E+00_JPRB, 0.51532E+00_JPRB ,&
&  0.69493E+00_JPRB, 0.72498E+00_JPRB, 0.75743E+00_JPRB, 0.95488E+00_JPRB, 0.84918E+00_JPRB, 0.62259E+00_JPRB, 0.46984E+00_JPRB ,&
&  0.46653E+00_JPRB, 0.51384E+00_JPRB, 0.54006E+00_JPRB, 0.54545E+00_JPRB, 0.57545E+00_JPRB, 0.60496E+00_JPRB, 0.61411E+00_JPRB ,&
&  0.64212E+00_JPRB, 0.66740E+00_JPRB, 0.67032E+00_JPRB, 0.66498E+00_JPRB, 0.68222E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.59888E+00_JPRB, 0.62432E+00_JPRB, 0.63461E+00_JPRB, 0.63836E+00_JPRB, 0.65349E+00_JPRB, 0.68618E+00_JPRB, 0.69981E+00_JPRB ,&
&  0.74973E+00_JPRB, 0.83111E+00_JPRB, 0.94148E+00_JPRB, 0.11021E+01_JPRB, 0.12071E+01_JPRB, 0.12264E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.17308E+01_JPRB, 0.22192E+01_JPRB, 0.26799E+01_JPRB, 0.27440E+01_JPRB, 0.24724E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.19581E+01_JPRB, 0.21317E+01_JPRB, 0.19708E+01_JPRB, 0.22095E+01_JPRB, 0.25384E+01_JPRB, 0.28949E+01_JPRB, 0.19784E+01_JPRB ,&
&  0.18118E+01_JPRB, 0.17740E+01_JPRB, 0.17031E+01_JPRB, 0.16956E+01_JPRB, 0.16579E+01_JPRB, 0.16726E+01_JPRB, 0.16869E+01_JPRB ,&
&  0.17139E+01_JPRB, 0.18344E+01_JPRB, 0.20002E+01_JPRB, 0.22649E+01_JPRB, 0.26403E+01_JPRB, 0.29288E+01_JPRB, 0.31157E+01_JPRB ,&
&  0.31529E+01_JPRB, 0.32338E+01_JPRB, 0.32155E+01_JPRB, 0.28225E+01_JPRB, 0.26067E+01_JPRB, 0.29371E+01_JPRB, 0.35847E+01_JPRB ,&
&  0.43337E+01_JPRB, 0.49164E+01_JPRB, 0.50668E+01_JPRB, 0.50408E+01_JPRB, 0.49510E+01_JPRB, 0.47192E+01_JPRB, 0.40988E+01_JPRB ,&
&  0.38372E+01_JPRB, 0.36852E+01_JPRB, 0.32912E+01_JPRB, 0.33834E+01_JPRB, 0.35973E+01_JPRB, 0.39137E+01_JPRB, 0.35181E+01_JPRB ,&
&  0.29691E+01_JPRB, 0.25703E+01_JPRB, 0.23297E+01_JPRB, 0.23568E+01_JPRB, 0.23282E+01_JPRB, 0.21449E+01_JPRB, 0.21831E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.62932E+00_JPRB, 0.42059E+00_JPRB, 0.46243E+00_JPRB, 0.31073E+00_JPRB, 0.32305E+00_JPRB, 0.57223E+00_JPRB ,&
&  0.82529E+00_JPRB, 0.89824E+00_JPRB, 0.85767E+00_JPRB, 0.86487E+00_JPRB, 0.10428E+01_JPRB, 0.81812E+00_JPRB, 0.51554E+00_JPRB ,&
&  0.50416E+00_JPRB, 0.55317E+00_JPRB, 0.56213E+00_JPRB, 0.54173E+00_JPRB, 0.53985E+00_JPRB, 0.56581E+00_JPRB, 0.53882E+00_JPRB ,&
&  0.56793E+00_JPRB, 0.58637E+00_JPRB, 0.59318E+00_JPRB, 0.59110E+00_JPRB, 0.58571E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.55420E+00_JPRB, 0.58000E+00_JPRB, 0.59776E+00_JPRB, 0.57193E+00_JPRB, 0.56262E+00_JPRB, 0.57898E+00_JPRB, 0.58792E+00_JPRB ,&
&  0.63350E+00_JPRB, 0.68480E+00_JPRB, 0.74954E+00_JPRB, 0.85860E+00_JPRB, 0.97524E+00_JPRB, 0.10391E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.13995E+01_JPRB, 0.18649E+01_JPRB, 0.25109E+01_JPRB, 0.28581E+01_JPRB, 0.26879E+01_JPRB, 0.27440E+01_JPRB ,&
&  0.22941E+01_JPRB, 0.26258E+01_JPRB, 0.33781E+01_JPRB, 0.42041E+01_JPRB, 0.43826E+01_JPRB, 0.40792E+01_JPRB, 0.24969E+01_JPRB ,&
&  0.19621E+01_JPRB, 0.18739E+01_JPRB, 0.17005E+01_JPRB, 0.17331E+01_JPRB, 0.15705E+01_JPRB, 0.15836E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.17550E+01_JPRB, 0.18756E+01_JPRB, 0.21507E+01_JPRB, 0.25984E+01_JPRB, 0.29981E+01_JPRB, 0.33665E+01_JPRB ,&
&  0.33478E+01_JPRB, 0.34157E+01_JPRB, 0.33977E+01_JPRB, 0.34403E+01_JPRB, 0.24746E+01_JPRB, 0.33306E+01_JPRB, 0.39617E+01_JPRB ,&
&  0.49634E+01_JPRB, 0.56152E+01_JPRB, 0.53739E+01_JPRB, 0.59498E+01_JPRB, 0.59553E+01_JPRB, 0.52625E+01_JPRB, 0.44479E+01_JPRB ,&
&  0.41050E+01_JPRB, 0.36066E+01_JPRB, 0.34053E+01_JPRB, 0.35457E+01_JPRB, 0.38719E+01_JPRB, 0.45395E+01_JPRB, 0.32091E+01_JPRB ,&
&  0.28496E+01_JPRB, 0.30883E+01_JPRB, 0.26920E+01_JPRB, 0.22533E+01_JPRB, 0.24812E+01_JPRB, 0.24976E+01_JPRB, 0.18053E+01_JPRB ,&
&  0.74163E+00_JPRB, 0.40032E+00_JPRB, 0.32491E+00_JPRB, 0.32157E+00_JPRB, 0.28820E+00_JPRB, 0.35828E+00_JPRB, 0.99559E+00_JPRB ,&
&  0.85821E+00_JPRB, 0.10671E+01_JPRB, 0.96159E+00_JPRB, 0.91871E+00_JPRB, 0.11493E+01_JPRB, 0.11056E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.60668E+00_JPRB, 0.57948E+00_JPRB, 0.60824E+00_JPRB, 0.60554E+00_JPRB, 0.61435E+00_JPRB, 0.58959E+00_JPRB, 0.55658E+00_JPRB ,&
&  0.57441E+00_JPRB, 0.59871E+00_JPRB, 0.57625E+00_JPRB, 0.55270E+00_JPRB, 0.54413E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.54051E+00_JPRB, 0.55306E+00_JPRB, 0.55760E+00_JPRB, 0.54739E+00_JPRB, 0.51496E+00_JPRB, 0.51313E+00_JPRB, 0.53281E+00_JPRB ,&
&  0.55937E+00_JPRB, 0.59111E+00_JPRB, 0.63099E+00_JPRB, 0.68772E+00_JPRB, 0.76781E+00_JPRB, 0.84891E+00_JPRB, 0.92639E+00_JPRB ,&
&  0.10265E+01_JPRB, 0.12384E+01_JPRB, 0.16246E+01_JPRB, 0.21152E+01_JPRB, 0.27297E+01_JPRB, 0.26518E+01_JPRB, 0.29961E+01_JPRB ,&
&  0.25947E+01_JPRB, 0.28894E+01_JPRB, 0.40962E+01_JPRB, 0.56875E+01_JPRB, 0.75628E+01_JPRB, 0.71866E+01_JPRB, 0.51678E+01_JPRB ,&
&  0.34191E+01_JPRB, 0.27188E+01_JPRB, 0.23609E+01_JPRB, 0.20057E+01_JPRB, 0.17495E+01_JPRB, 0.16780E+01_JPRB, 0.17102E+01_JPRB ,&
&  0.16684E+01_JPRB, 0.16944E+01_JPRB, 0.17389E+01_JPRB, 0.19499E+01_JPRB, 0.23683E+01_JPRB, 0.28608E+01_JPRB, 0.34495E+01_JPRB ,&
&  0.37587E+01_JPRB, 0.39386E+01_JPRB, 0.38327E+01_JPRB, 0.37516E+01_JPRB, 0.34132E+01_JPRB, 0.33417E+01_JPRB, 0.40582E+01_JPRB ,&
&  0.54355E+01_JPRB, 0.66242E+01_JPRB, 0.63143E+01_JPRB, 0.71937E+01_JPRB, 0.64954E+01_JPRB, 0.56706E+01_JPRB, 0.49152E+01_JPRB ,&
&  0.42470E+01_JPRB, 0.42135E+01_JPRB, 0.33825E+01_JPRB, 0.34318E+01_JPRB, 0.39111E+01_JPRB, 0.36647E+01_JPRB, 0.34582E+01_JPRB ,&
&  0.36141E+01_JPRB, 0.30418E+01_JPRB, 0.26107E+01_JPRB, 0.21518E+01_JPRB, 0.26552E+01_JPRB, 0.22868E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.67253E+00_JPRB, 0.48202E+00_JPRB, 0.32926E+00_JPRB, 0.33196E+00_JPRB, 0.31496E+00_JPRB, 0.39226E+00_JPRB, 0.88932E+00_JPRB ,&
&  0.10168E+01_JPRB, 0.11956E+01_JPRB, 0.10999E+01_JPRB, 0.94348E+00_JPRB, 0.10821E+01_JPRB, 0.12238E+01_JPRB, 0.24009E+01_JPRB ,&
&  0.15752E+01_JPRB, 0.95047E+00_JPRB, 0.69605E+00_JPRB, 0.67372E+00_JPRB, 0.67552E+00_JPRB, 0.64917E+00_JPRB, 0.61182E+00_JPRB ,&
&  0.59709E+00_JPRB, 0.61377E+00_JPRB, 0.60902E+00_JPRB, 0.59190E+00_JPRB, 0.56704E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.57117E+00_JPRB, 0.56675E+00_JPRB, 0.57313E+00_JPRB, 0.57017E+00_JPRB, 0.54562E+00_JPRB, 0.52985E+00_JPRB, 0.52755E+00_JPRB ,&
&  0.53757E+00_JPRB, 0.54990E+00_JPRB, 0.56738E+00_JPRB, 0.59527E+00_JPRB, 0.63595E+00_JPRB, 0.69372E+00_JPRB, 0.77063E+00_JPRB ,&
&  0.90185E+00_JPRB, 0.11417E+01_JPRB, 0.14220E+01_JPRB, 0.17034E+01_JPRB, 0.21852E+01_JPRB, 0.23385E+01_JPRB, 0.28480E+01_JPRB ,&
&  0.28901E+01_JPRB, 0.31205E+01_JPRB, 0.44158E+01_JPRB, 0.67216E+01_JPRB, 0.10295E+02_JPRB, 0.11317E+02_JPRB, 0.10196E+02_JPRB ,&
&  0.77922E+01_JPRB, 0.52068E+01_JPRB, 0.36628E+01_JPRB, 0.26973E+01_JPRB, 0.21136E+01_JPRB, 0.18049E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.16977E+01_JPRB, 0.16784E+01_JPRB, 0.17132E+01_JPRB, 0.17321E+01_JPRB, 0.19915E+01_JPRB, 0.24998E+01_JPRB, 0.32786E+01_JPRB ,&
&  0.39324E+01_JPRB, 0.44119E+01_JPRB, 0.45100E+01_JPRB, 0.39690E+01_JPRB, 0.45708E+01_JPRB, 0.47690E+01_JPRB, 0.46613E+01_JPRB ,&
&  0.51241E+01_JPRB, 0.58228E+01_JPRB, 0.75556E+01_JPRB, 0.70925E+01_JPRB, 0.62540E+01_JPRB, 0.61050E+01_JPRB, 0.51931E+01_JPRB ,&
&  0.45618E+01_JPRB, 0.43428E+01_JPRB, 0.36133E+01_JPRB, 0.36445E+01_JPRB, 0.35601E+01_JPRB, 0.34126E+01_JPRB, 0.37585E+01_JPRB ,&
&  0.37779E+01_JPRB, 0.33835E+01_JPRB, 0.27102E+01_JPRB, 0.21690E+01_JPRB, 0.20684E+01_JPRB, 0.14450E+01_JPRB, 0.88652E+00_JPRB ,&
&  0.71225E+00_JPRB, 0.57698E+00_JPRB, 0.41093E+00_JPRB, 0.49099E+00_JPRB, 0.43917E+00_JPRB, 0.54907E+00_JPRB, 0.93888E+00_JPRB ,&
&  0.13048E+01_JPRB, 0.12897E+01_JPRB, 0.12568E+01_JPRB, 0.10078E+01_JPRB, 0.97340E+00_JPRB, 0.10084E+01_JPRB, 0.26418E+01_JPRB ,&
&  0.17320E+01_JPRB, 0.14404E+01_JPRB, 0.10707E+01_JPRB, 0.82043E+00_JPRB, 0.72463E+00_JPRB, 0.68513E+00_JPRB, 0.66425E+00_JPRB ,&
&  0.63572E+00_JPRB, 0.61736E+00_JPRB, 0.61269E+00_JPRB, 0.59824E+00_JPRB, 0.58185E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.63791E+00_JPRB, 0.63436E+00_JPRB, 0.63561E+00_JPRB, 0.63063E+00_JPRB, 0.61838E+00_JPRB, 0.59408E+00_JPRB, 0.56488E+00_JPRB ,&
&  0.55638E+00_JPRB, 0.55714E+00_JPRB, 0.56160E+00_JPRB, 0.58083E+00_JPRB, 0.59356E+00_JPRB, 0.61958E+00_JPRB, 0.67142E+00_JPRB ,&
&  0.82560E+00_JPRB, 0.11384E+01_JPRB, 0.11533E+01_JPRB, 0.12929E+01_JPRB, 0.16945E+01_JPRB, 0.22851E+01_JPRB, 0.25802E+01_JPRB ,&
&  0.30021E+01_JPRB, 0.31284E+01_JPRB, 0.40865E+01_JPRB, 0.63279E+01_JPRB, 0.12934E+02_JPRB, 0.15032E+02_JPRB, 0.14064E+02_JPRB ,&
&  0.13357E+02_JPRB, 0.96690E+01_JPRB, 0.61325E+01_JPRB, 0.42068E+01_JPRB, 0.29925E+01_JPRB, 0.21646E+01_JPRB, 0.17818E+01_JPRB ,&
&  0.16406E+01_JPRB, 0.15881E+01_JPRB, 0.15821E+01_JPRB, 0.16174E+01_JPRB, 0.17467E+01_JPRB, 0.21476E+01_JPRB, 0.29564E+01_JPRB ,&
&  0.37617E+01_JPRB, 0.45568E+01_JPRB, 0.50554E+01_JPRB, 0.49322E+01_JPRB, 0.41969E+01_JPRB, 0.47567E+01_JPRB, 0.69960E+01_JPRB ,&
&  0.67933E+01_JPRB, 0.72655E+01_JPRB, 0.73330E+01_JPRB, 0.74288E+01_JPRB, 0.64470E+01_JPRB, 0.56941E+01_JPRB, 0.52310E+01_JPRB ,&
&  0.37741E+01_JPRB, 0.32446E+01_JPRB, 0.34169E+01_JPRB, 0.35821E+01_JPRB, 0.34067E+01_JPRB, 0.39885E+01_JPRB, 0.50708E+01_JPRB ,&
&  0.38916E+01_JPRB, 0.38312E+01_JPRB, 0.37894E+01_JPRB, 0.36582E+01_JPRB, 0.20881E+01_JPRB, 0.13661E+01_JPRB, 0.19243E+01_JPRB ,&
&  0.18368E+01_JPRB, 0.10080E+01_JPRB, 0.75266E+00_JPRB, 0.10018E+01_JPRB, 0.68495E+00_JPRB, 0.82309E+00_JPRB, 0.12097E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.12330E+01_JPRB, 0.14177E+01_JPRB, 0.11994E+01_JPRB, 0.10943E+01_JPRB, 0.92193E+00_JPRB, 0.15856E+01_JPRB ,&
&  0.18528E+01_JPRB, 0.16332E+01_JPRB, 0.14388E+01_JPRB, 0.11203E+01_JPRB, 0.90190E+00_JPRB, 0.80922E+00_JPRB, 0.77523E+00_JPRB ,&
&  0.75120E+00_JPRB, 0.72371E+00_JPRB, 0.69709E+00_JPRB, 0.65674E+00_JPRB, 0.64479E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.69855E+00_JPRB, 0.69375E+00_JPRB, 0.69220E+00_JPRB, 0.69134E+00_JPRB, 0.68505E+00_JPRB, 0.65374E+00_JPRB, 0.61276E+00_JPRB ,&
&  0.58539E+00_JPRB, 0.57619E+00_JPRB, 0.58083E+00_JPRB, 0.58664E+00_JPRB, 0.57907E+00_JPRB, 0.58834E+00_JPRB, 0.61420E+00_JPRB ,&
&  0.75044E+00_JPRB, 0.97563E+00_JPRB, 0.10336E+01_JPRB, 0.11463E+01_JPRB, 0.15916E+01_JPRB, 0.22620E+01_JPRB, 0.27072E+01_JPRB ,&
&  0.32947E+01_JPRB, 0.32189E+01_JPRB, 0.38329E+01_JPRB, 0.60334E+01_JPRB, 0.10630E+02_JPRB, 0.13788E+02_JPRB, 0.13463E+02_JPRB ,&
&  0.13039E+02_JPRB, 0.10731E+02_JPRB, 0.80975E+01_JPRB, 0.60466E+01_JPRB, 0.43557E+01_JPRB, 0.31406E+01_JPRB, 0.22909E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.16303E+01_JPRB, 0.15669E+01_JPRB, 0.15442E+01_JPRB, 0.16301E+01_JPRB, 0.18112E+01_JPRB, 0.24435E+01_JPRB ,&
&  0.31672E+01_JPRB, 0.42020E+01_JPRB, 0.53853E+01_JPRB, 0.59368E+01_JPRB, 0.50752E+01_JPRB, 0.56694E+01_JPRB, 0.74519E+01_JPRB ,&
&  0.73935E+01_JPRB, 0.72185E+01_JPRB, 0.73280E+01_JPRB, 0.70273E+01_JPRB, 0.69105E+01_JPRB, 0.60877E+01_JPRB, 0.59069E+01_JPRB ,&
&  0.45260E+01_JPRB, 0.37482E+01_JPRB, 0.33883E+01_JPRB, 0.33445E+01_JPRB, 0.41594E+01_JPRB, 0.54761E+01_JPRB, 0.51412E+01_JPRB ,&
&  0.45029E+01_JPRB, 0.42418E+01_JPRB, 0.38832E+01_JPRB, 0.31384E+01_JPRB, 0.29482E+01_JPRB, 0.23656E+01_JPRB, 0.27473E+01_JPRB ,&
&  0.26305E+01_JPRB, 0.23651E+01_JPRB, 0.19383E+01_JPRB, 0.14768E+01_JPRB, 0.11393E+01_JPRB, 0.13623E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.13588E+01_JPRB, 0.12123E+01_JPRB, 0.12885E+01_JPRB, 0.12231E+01_JPRB, 0.11412E+01_JPRB, 0.85147E+00_JPRB, 0.11725E+01_JPRB ,&
&  0.13303E+01_JPRB, 0.15450E+01_JPRB, 0.15605E+01_JPRB, 0.13428E+01_JPRB, 0.11634E+01_JPRB, 0.10189E+01_JPRB, 0.94504E+00_JPRB ,&
&  0.91219E+00_JPRB, 0.87420E+00_JPRB, 0.81918E+00_JPRB, 0.75625E+00_JPRB, 0.72243E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.71308E+00_JPRB, 0.70971E+00_JPRB, 0.70186E+00_JPRB, 0.69006E+00_JPRB, 0.67178E+00_JPRB, 0.65354E+00_JPRB, 0.62704E+00_JPRB ,&
&  0.60696E+00_JPRB, 0.59237E+00_JPRB, 0.59055E+00_JPRB, 0.59088E+00_JPRB, 0.59159E+00_JPRB, 0.58282E+00_JPRB, 0.58440E+00_JPRB ,&
&  0.66560E+00_JPRB, 0.87098E+00_JPRB, 0.91359E+00_JPRB, 0.10829E+01_JPRB, 0.14988E+01_JPRB, 0.21591E+01_JPRB, 0.30397E+01_JPRB ,&
&  0.34883E+01_JPRB, 0.35907E+01_JPRB, 0.40555E+01_JPRB, 0.57047E+01_JPRB, 0.78883E+01_JPRB, 0.93126E+01_JPRB, 0.95565E+01_JPRB ,&
&  0.82167E+01_JPRB, 0.72725E+01_JPRB, 0.62095E+01_JPRB, 0.52729E+01_JPRB, 0.45575E+01_JPRB, 0.39094E+01_JPRB, 0.31511E+01_JPRB ,&
&  0.24721E+01_JPRB, 0.20270E+01_JPRB, 0.18067E+01_JPRB, 0.17027E+01_JPRB, 0.16750E+01_JPRB, 0.17380E+01_JPRB, 0.20384E+01_JPRB ,&
&  0.25493E+01_JPRB, 0.34331E+01_JPRB, 0.50522E+01_JPRB, 0.68127E+01_JPRB, 0.65615E+01_JPRB, 0.76967E+01_JPRB, 0.77218E+01_JPRB ,&
&  0.83740E+01_JPRB, 0.83430E+01_JPRB, 0.74558E+01_JPRB, 0.73235E+01_JPRB, 0.59388E+01_JPRB, 0.62301E+01_JPRB, 0.65288E+01_JPRB ,&
&  0.64880E+01_JPRB, 0.61674E+01_JPRB, 0.61020E+01_JPRB, 0.52967E+01_JPRB, 0.49430E+01_JPRB, 0.58932E+01_JPRB, 0.51405E+01_JPRB ,&
&  0.44361E+01_JPRB, 0.41702E+01_JPRB, 0.37623E+01_JPRB, 0.36583E+01_JPRB, 0.30310E+01_JPRB, 0.29351E+01_JPRB, 0.25031E+01_JPRB ,&
&  0.22690E+01_JPRB, 0.18558E+01_JPRB, 0.20477E+01_JPRB, 0.17537E+01_JPRB, 0.16220E+01_JPRB, 0.16082E+01_JPRB, 0.15839E+01_JPRB ,&
&  0.14494E+01_JPRB, 0.12090E+01_JPRB, 0.10546E+01_JPRB, 0.11424E+01_JPRB, 0.12183E+01_JPRB, 0.98213E+00_JPRB, 0.10365E+01_JPRB ,&
&  0.11650E+01_JPRB, 0.12804E+01_JPRB, 0.12396E+01_JPRB, 0.12901E+01_JPRB, 0.12563E+01_JPRB, 0.12042E+01_JPRB, 0.11131E+01_JPRB ,&
&  0.99169E+00_JPRB, 0.88269E+00_JPRB, 0.81654E+00_JPRB, 0.75613E+00_JPRB, 0.71752E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.69058E+00_JPRB, 0.69587E+00_JPRB, 0.68110E+00_JPRB, 0.65750E+00_JPRB, 0.64189E+00_JPRB, 0.62545E+00_JPRB, 0.61159E+00_JPRB ,&
&  0.59555E+00_JPRB, 0.58560E+00_JPRB, 0.58340E+00_JPRB, 0.58713E+00_JPRB, 0.59057E+00_JPRB, 0.59030E+00_JPRB, 0.58426E+00_JPRB ,&
&  0.59990E+00_JPRB, 0.81186E+00_JPRB, 0.83179E+00_JPRB, 0.88210E+00_JPRB, 0.12250E+01_JPRB, 0.20945E+01_JPRB, 0.29990E+01_JPRB ,&
&  0.34577E+01_JPRB, 0.36485E+01_JPRB, 0.40771E+01_JPRB, 0.47082E+01_JPRB, 0.54511E+01_JPRB, 0.59639E+01_JPRB, 0.61375E+01_JPRB ,&
&  0.59793E+01_JPRB, 0.45847E+01_JPRB, 0.41346E+01_JPRB, 0.37233E+01_JPRB, 0.34830E+01_JPRB, 0.32466E+01_JPRB, 0.30079E+01_JPRB ,&
&  0.26870E+01_JPRB, 0.22973E+01_JPRB, 0.20013E+01_JPRB, 0.18127E+01_JPRB, 0.17133E+01_JPRB, 0.16251E+01_JPRB, 0.17263E+01_JPRB ,&
&  0.19508E+01_JPRB, 0.22562E+01_JPRB, 0.32853E+01_JPRB, 0.49177E+01_JPRB, 0.73140E+01_JPRB, 0.88012E+01_JPRB, 0.87777E+01_JPRB ,&
&  0.72446E+01_JPRB, 0.68158E+01_JPRB, 0.86934E+01_JPRB, 0.74292E+01_JPRB, 0.70283E+01_JPRB, 0.72015E+01_JPRB, 0.68601E+01_JPRB ,&
&  0.65731E+01_JPRB, 0.61255E+01_JPRB, 0.57172E+01_JPRB, 0.53530E+01_JPRB, 0.59234E+01_JPRB, 0.56593E+01_JPRB, 0.49362E+01_JPRB ,&
&  0.42567E+01_JPRB, 0.41054E+01_JPRB, 0.38467E+01_JPRB, 0.35770E+01_JPRB, 0.33584E+01_JPRB, 0.34599E+01_JPRB, 0.31721E+01_JPRB ,&
&  0.28837E+01_JPRB, 0.30877E+01_JPRB, 0.24682E+01_JPRB, 0.19072E+01_JPRB, 0.15697E+01_JPRB, 0.16176E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.14642E+01_JPRB, 0.11657E+01_JPRB, 0.89672E+00_JPRB, 0.87507E+00_JPRB, 0.93790E+00_JPRB, 0.94405E+00_JPRB, 0.90986E+00_JPRB ,&
&  0.95574E+00_JPRB, 0.10799E+01_JPRB, 0.10935E+01_JPRB, 0.11215E+01_JPRB, 0.12361E+01_JPRB, 0.11677E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.90122E+00_JPRB, 0.81546E+00_JPRB, 0.76599E+00_JPRB, 0.73030E+00_JPRB, 0.69634E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.68500E+00_JPRB, 0.68014E+00_JPRB, 0.66527E+00_JPRB, 0.64254E+00_JPRB, 0.62676E+00_JPRB, 0.61807E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.59840E+00_JPRB, 0.59267E+00_JPRB, 0.59736E+00_JPRB, 0.59544E+00_JPRB, 0.60108E+00_JPRB, 0.61231E+00_JPRB, 0.61823E+00_JPRB ,&
&  0.62035E+00_JPRB, 0.79739E+00_JPRB, 0.77373E+00_JPRB, 0.84135E+00_JPRB, 0.10491E+01_JPRB, 0.17388E+01_JPRB, 0.23965E+01_JPRB ,&
&  0.28350E+01_JPRB, 0.29923E+01_JPRB, 0.31807E+01_JPRB, 0.35325E+01_JPRB, 0.37434E+01_JPRB, 0.36887E+01_JPRB, 0.36418E+01_JPRB ,&
&  0.34784E+01_JPRB, 0.32062E+01_JPRB, 0.26803E+01_JPRB, 0.26420E+01_JPRB, 0.24259E+01_JPRB, 0.20806E+01_JPRB, 0.22211E+01_JPRB ,&
&  0.22265E+01_JPRB, 0.21655E+01_JPRB, 0.20351E+01_JPRB, 0.19191E+01_JPRB, 0.18434E+01_JPRB, 0.17395E+01_JPRB, 0.16975E+01_JPRB ,&
&  0.17282E+01_JPRB, 0.18018E+01_JPRB, 0.20488E+01_JPRB, 0.27143E+01_JPRB, 0.41966E+01_JPRB, 0.60217E+01_JPRB, 0.74506E+01_JPRB ,&
&  0.86525E+01_JPRB, 0.86896E+01_JPRB, 0.72440E+01_JPRB, 0.73163E+01_JPRB, 0.78946E+01_JPRB, 0.70259E+01_JPRB, 0.64008E+01_JPRB ,&
&  0.64705E+01_JPRB, 0.64279E+01_JPRB, 0.60912E+01_JPRB, 0.57939E+01_JPRB, 0.56987E+01_JPRB, 0.54455E+01_JPRB, 0.50105E+01_JPRB ,&
&  0.42631E+01_JPRB, 0.39683E+01_JPRB, 0.38867E+01_JPRB, 0.37820E+01_JPRB, 0.33721E+01_JPRB, 0.31651E+01_JPRB, 0.31538E+01_JPRB ,&
&  0.30294E+01_JPRB, 0.29248E+01_JPRB, 0.22760E+01_JPRB, 0.21447E+01_JPRB, 0.16185E+01_JPRB, 0.15560E+01_JPRB, 0.17106E+01_JPRB ,&
&  0.14460E+01_JPRB, 0.10968E+01_JPRB, 0.80049E+00_JPRB, 0.68196E+00_JPRB, 0.65549E+00_JPRB, 0.68868E+00_JPRB, 0.77361E+00_JPRB ,&
&  0.85620E+00_JPRB, 0.80228E+00_JPRB, 0.94510E+00_JPRB, 0.96658E+00_JPRB, 0.10355E+01_JPRB, 0.10176E+01_JPRB, 0.95128E+00_JPRB ,&
&  0.89066E+00_JPRB, 0.84165E+00_JPRB, 0.79461E+00_JPRB, 0.74735E+00_JPRB, 0.70895E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.67238E+00_JPRB, 0.64176E+00_JPRB, 0.62737E+00_JPRB, 0.61191E+00_JPRB, 0.60236E+00_JPRB, 0.59650E+00_JPRB, 0.59433E+00_JPRB ,&
&  0.60394E+00_JPRB, 0.61075E+00_JPRB, 0.61877E+00_JPRB, 0.61538E+00_JPRB, 0.61529E+00_JPRB, 0.61588E+00_JPRB, 0.64138E+00_JPRB ,&
&  0.66200E+00_JPRB, 0.74003E+00_JPRB, 0.79210E+00_JPRB, 0.84536E+00_JPRB, 0.11972E+01_JPRB, 0.14466E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.19173E+01_JPRB, 0.20731E+01_JPRB, 0.21383E+01_JPRB, 0.21816E+01_JPRB, 0.21837E+01_JPRB, 0.21626E+01_JPRB, 0.20534E+01_JPRB ,&
&  0.18771E+01_JPRB, 0.16704E+01_JPRB, 0.15694E+01_JPRB, 0.15322E+01_JPRB, 0.15614E+01_JPRB, 0.13897E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.15819E+01_JPRB, 0.16710E+01_JPRB, 0.17110E+01_JPRB, 0.17271E+01_JPRB, 0.17221E+01_JPRB, 0.17272E+01_JPRB, 0.17432E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.17972E+01_JPRB, 0.18202E+01_JPRB, 0.20948E+01_JPRB, 0.29021E+01_JPRB, 0.46893E+01_JPRB, 0.63946E+01_JPRB ,&
&  0.63784E+01_JPRB, 0.72925E+01_JPRB, 0.83898E+01_JPRB, 0.85974E+01_JPRB, 0.72629E+01_JPRB, 0.61590E+01_JPRB, 0.57927E+01_JPRB ,&
&  0.59574E+01_JPRB, 0.62258E+01_JPRB, 0.61553E+01_JPRB, 0.56206E+01_JPRB, 0.54539E+01_JPRB, 0.52906E+01_JPRB, 0.50373E+01_JPRB ,&
&  0.46723E+01_JPRB, 0.42682E+01_JPRB, 0.39915E+01_JPRB, 0.37899E+01_JPRB, 0.36602E+01_JPRB, 0.33439E+01_JPRB, 0.32040E+01_JPRB ,&
&  0.32096E+01_JPRB, 0.26466E+01_JPRB, 0.23362E+01_JPRB, 0.24100E+01_JPRB, 0.19230E+01_JPRB, 0.18588E+01_JPRB, 0.19623E+01_JPRB ,&
&  0.15397E+01_JPRB, 0.10806E+01_JPRB, 0.86343E+00_JPRB, 0.71239E+00_JPRB, 0.65172E+00_JPRB, 0.64507E+00_JPRB, 0.66490E+00_JPRB ,&
&  0.72549E+00_JPRB, 0.73566E+00_JPRB, 0.85139E+00_JPRB, 0.90602E+00_JPRB, 0.93237E+00_JPRB, 0.91690E+00_JPRB, 0.87967E+00_JPRB ,&
&  0.82193E+00_JPRB, 0.77944E+00_JPRB, 0.75594E+00_JPRB, 0.73475E+00_JPRB, 0.71039E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.66075E+00_JPRB, 0.63818E+00_JPRB, 0.62236E+00_JPRB, 0.61317E+00_JPRB, 0.60700E+00_JPRB, 0.60011E+00_JPRB, 0.59605E+00_JPRB ,&
&  0.59892E+00_JPRB, 0.61376E+00_JPRB, 0.63276E+00_JPRB, 0.63224E+00_JPRB, 0.62762E+00_JPRB, 0.62922E+00_JPRB, 0.67711E+00_JPRB ,&
&  0.56545E+00_JPRB, 0.61079E+00_JPRB, 0.65698E+00_JPRB, 0.81858E+00_JPRB, 0.12008E+01_JPRB, 0.11836E+01_JPRB, 0.12845E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.13386E+01_JPRB, 0.13458E+01_JPRB, 0.13685E+01_JPRB, 0.13509E+01_JPRB, 0.13356E+01_JPRB, 0.12807E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.99867E+00_JPRB, 0.95250E+00_JPRB, 0.92291E+00_JPRB, 0.96755E+00_JPRB, 0.10310E+01_JPRB, 0.11338E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.13020E+01_JPRB, 0.13678E+01_JPRB, 0.14161E+01_JPRB, 0.14458E+01_JPRB, 0.14782E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.15708E+01_JPRB, 0.16118E+01_JPRB, 0.16808E+01_JPRB, 0.19233E+01_JPRB, 0.24707E+01_JPRB, 0.44022E+01_JPRB, 0.60177E+01_JPRB ,&
&  0.48543E+01_JPRB, 0.44629E+01_JPRB, 0.56682E+01_JPRB, 0.68217E+01_JPRB, 0.59938E+01_JPRB, 0.50601E+01_JPRB, 0.45273E+01_JPRB ,&
&  0.45682E+01_JPRB, 0.47982E+01_JPRB, 0.49945E+01_JPRB, 0.48754E+01_JPRB, 0.45807E+01_JPRB, 0.45166E+01_JPRB, 0.44256E+01_JPRB ,&
&  0.42635E+01_JPRB, 0.41117E+01_JPRB, 0.39078E+01_JPRB, 0.37207E+01_JPRB, 0.36012E+01_JPRB, 0.36550E+01_JPRB, 0.36056E+01_JPRB ,&
&  0.34819E+01_JPRB, 0.32470E+01_JPRB, 0.29156E+01_JPRB, 0.27107E+01_JPRB, 0.22035E+01_JPRB, 0.20956E+01_JPRB, 0.23500E+01_JPRB ,&
&  0.17527E+01_JPRB, 0.12724E+01_JPRB, 0.10602E+01_JPRB, 0.89860E+00_JPRB, 0.82159E+00_JPRB, 0.70422E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.65202E+00_JPRB, 0.68857E+00_JPRB, 0.76911E+00_JPRB, 0.80459E+00_JPRB, 0.81879E+00_JPRB, 0.82606E+00_JPRB, 0.76274E+00_JPRB ,&
&  0.75127E+00_JPRB, 0.71127E+00_JPRB, 0.69165E+00_JPRB, 0.68560E+00_JPRB, 0.67576E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.63790E+00_JPRB, 0.62722E+00_JPRB, 0.62039E+00_JPRB, 0.60707E+00_JPRB, 0.59537E+00_JPRB, 0.59501E+00_JPRB, 0.60316E+00_JPRB ,&
&  0.61203E+00_JPRB, 0.62788E+00_JPRB, 0.63925E+00_JPRB, 0.64007E+00_JPRB, 0.64304E+00_JPRB, 0.66788E+00_JPRB, 0.61632E+00_JPRB ,&
&  0.53425E+00_JPRB, 0.55758E+00_JPRB, 0.57192E+00_JPRB, 0.64822E+00_JPRB, 0.73997E+00_JPRB, 0.77348E+00_JPRB, 0.81576E+00_JPRB ,&
&  0.87740E+00_JPRB, 0.87892E+00_JPRB, 0.90036E+00_JPRB, 0.91839E+00_JPRB, 0.91803E+00_JPRB, 0.91119E+00_JPRB, 0.89398E+00_JPRB ,&
&  0.80504E+00_JPRB, 0.74682E+00_JPRB, 0.71095E+00_JPRB, 0.69239E+00_JPRB, 0.73339E+00_JPRB, 0.80410E+00_JPRB, 0.88036E+00_JPRB ,&
&  0.95661E+00_JPRB, 0.10233E+01_JPRB, 0.10900E+01_JPRB, 0.11426E+01_JPRB, 0.11739E+01_JPRB, 0.12004E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12848E+01_JPRB, 0.13359E+01_JPRB, 0.13800E+01_JPRB, 0.15793E+01_JPRB, 0.18155E+01_JPRB, 0.28055E+01_JPRB, 0.36080E+01_JPRB ,&
&  0.31463E+01_JPRB, 0.29450E+01_JPRB, 0.31503E+01_JPRB, 0.34808E+01_JPRB, 0.35504E+01_JPRB, 0.33340E+01_JPRB, 0.32072E+01_JPRB ,&
&  0.32906E+01_JPRB, 0.34431E+01_JPRB, 0.38419E+01_JPRB, 0.37609E+01_JPRB, 0.36762E+01_JPRB, 0.36977E+01_JPRB, 0.36242E+01_JPRB ,&
&  0.36507E+01_JPRB, 0.36409E+01_JPRB, 0.36849E+01_JPRB, 0.35876E+01_JPRB, 0.34816E+01_JPRB, 0.34821E+01_JPRB, 0.34647E+01_JPRB ,&
&  0.34838E+01_JPRB, 0.34374E+01_JPRB, 0.31808E+01_JPRB, 0.29614E+01_JPRB, 0.26198E+01_JPRB, 0.25758E+01_JPRB, 0.23175E+01_JPRB ,&
&  0.18548E+01_JPRB, 0.13625E+01_JPRB, 0.12538E+01_JPRB, 0.10723E+01_JPRB, 0.93089E+00_JPRB, 0.79036E+00_JPRB, 0.69674E+00_JPRB ,&
&  0.67204E+00_JPRB, 0.72399E+00_JPRB, 0.74442E+00_JPRB, 0.76016E+00_JPRB, 0.77686E+00_JPRB, 0.77409E+00_JPRB, 0.69039E+00_JPRB ,&
&  0.67609E+00_JPRB, 0.69867E+00_JPRB, 0.67731E+00_JPRB, 0.66110E+00_JPRB, 0.64753E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.61695E+00_JPRB, 0.61813E+00_JPRB, 0.61771E+00_JPRB, 0.61491E+00_JPRB, 0.59422E+00_JPRB, 0.61074E+00_JPRB, 0.56597E+00_JPRB ,&
&  0.63460E+00_JPRB, 0.64877E+00_JPRB, 0.64091E+00_JPRB, 0.66007E+00_JPRB, 0.70289E+00_JPRB, 0.65605E+00_JPRB, 0.51887E+00_JPRB ,&
&  0.46732E+00_JPRB, 0.47367E+00_JPRB, 0.50065E+00_JPRB, 0.52591E+00_JPRB, 0.54830E+00_JPRB, 0.56905E+00_JPRB, 0.60093E+00_JPRB ,&
&  0.63167E+00_JPRB, 0.64551E+00_JPRB, 0.67269E+00_JPRB, 0.68716E+00_JPRB, 0.69011E+00_JPRB, 0.68812E+00_JPRB, 0.69591E+00_JPRB ,&
&  0.65378E+00_JPRB, 0.62707E+00_JPRB, 0.61109E+00_JPRB, 0.57438E+00_JPRB, 0.61750E+00_JPRB, 0.65600E+00_JPRB, 0.70686E+00_JPRB ,&
&  0.76696E+00_JPRB, 0.83260E+00_JPRB, 0.88397E+00_JPRB, 0.93306E+00_JPRB, 0.96995E+00_JPRB, 0.99893E+00_JPRB, 0.10270E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.11104E+01_JPRB, 0.11546E+01_JPRB, 0.12874E+01_JPRB, 0.14839E+01_JPRB, 0.17781E+01_JPRB, 0.21874E+01_JPRB ,&
&  0.23028E+01_JPRB, 0.25328E+01_JPRB, 0.27142E+01_JPRB, 0.23861E+01_JPRB, 0.25382E+01_JPRB, 0.27916E+01_JPRB, 0.26955E+01_JPRB ,&
&  0.26271E+01_JPRB, 0.24463E+01_JPRB, 0.24139E+01_JPRB, 0.24536E+01_JPRB, 0.26714E+01_JPRB, 0.28336E+01_JPRB, 0.29017E+01_JPRB ,&
&  0.29260E+01_JPRB, 0.31341E+01_JPRB, 0.32344E+01_JPRB, 0.32705E+01_JPRB, 0.32482E+01_JPRB, 0.32556E+01_JPRB, 0.33026E+01_JPRB ,&
&  0.33407E+01_JPRB, 0.33565E+01_JPRB, 0.32973E+01_JPRB, 0.31361E+01_JPRB, 0.29014E+01_JPRB, 0.26201E+01_JPRB, 0.22842E+01_JPRB ,&
&  0.19876E+01_JPRB, 0.16858E+01_JPRB, 0.14352E+01_JPRB, 0.12595E+01_JPRB, 0.10798E+01_JPRB, 0.90925E+00_JPRB, 0.78970E+00_JPRB ,&
&  0.69660E+00_JPRB, 0.68550E+00_JPRB, 0.74377E+00_JPRB, 0.75176E+00_JPRB, 0.76270E+00_JPRB, 0.75298E+00_JPRB, 0.69459E+00_JPRB ,&
&  0.68177E+00_JPRB, 0.69556E+00_JPRB, 0.68106E+00_JPRB, 0.65179E+00_JPRB, 0.62814E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.63223E+00_JPRB, 0.61391E+00_JPRB, 0.60711E+00_JPRB, 0.61795E+00_JPRB, 0.61014E+00_JPRB, 0.57562E+00_JPRB, 0.53833E+00_JPRB ,&
&  0.65015E+00_JPRB, 0.61533E+00_JPRB, 0.67408E+00_JPRB, 0.53121E+00_JPRB, 0.47604E+00_JPRB, 0.46286E+00_JPRB, 0.44266E+00_JPRB ,&
&  0.43087E+00_JPRB, 0.43258E+00_JPRB, 0.44356E+00_JPRB, 0.46453E+00_JPRB, 0.48776E+00_JPRB, 0.50030E+00_JPRB, 0.51232E+00_JPRB ,&
&  0.52669E+00_JPRB, 0.54213E+00_JPRB, 0.56870E+00_JPRB, 0.58848E+00_JPRB, 0.59612E+00_JPRB, 0.59607E+00_JPRB, 0.60194E+00_JPRB ,&
&  0.56982E+00_JPRB, 0.55530E+00_JPRB, 0.55707E+00_JPRB, 0.55163E+00_JPRB, 0.56014E+00_JPRB, 0.57634E+00_JPRB, 0.60487E+00_JPRB ,&
&  0.66360E+00_JPRB, 0.62689E+00_JPRB, 0.74098E+00_JPRB, 0.77493E+00_JPRB, 0.81036E+00_JPRB, 0.83948E+00_JPRB, 0.86323E+00_JPRB ,&
&  0.89040E+00_JPRB, 0.93397E+00_JPRB, 0.98074E+00_JPRB, 0.10497E+01_JPRB, 0.12314E+01_JPRB, 0.13602E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.17309E+01_JPRB, 0.15426E+01_JPRB, 0.18017E+01_JPRB, 0.18898E+01_JPRB, 0.22041E+01_JPRB, 0.24457E+01_JPRB, 0.23709E+01_JPRB ,&
&  0.25768E+01_JPRB, 0.22976E+01_JPRB, 0.20526E+01_JPRB, 0.18734E+01_JPRB, 0.18860E+01_JPRB, 0.19894E+01_JPRB, 0.21260E+01_JPRB ,&
&  0.22265E+01_JPRB, 0.22742E+01_JPRB, 0.24058E+01_JPRB, 0.25006E+01_JPRB, 0.26232E+01_JPRB, 0.27900E+01_JPRB, 0.29539E+01_JPRB ,&
&  0.30897E+01_JPRB, 0.31606E+01_JPRB, 0.31875E+01_JPRB, 0.29675E+01_JPRB, 0.27819E+01_JPRB, 0.26114E+01_JPRB, 0.22610E+01_JPRB ,&
&  0.19923E+01_JPRB, 0.18068E+01_JPRB, 0.16481E+01_JPRB, 0.15282E+01_JPRB, 0.13049E+01_JPRB, 0.11099E+01_JPRB, 0.93137E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.65407E+00_JPRB, 0.66265E+00_JPRB, 0.67594E+00_JPRB, 0.68187E+00_JPRB, 0.69068E+00_JPRB, 0.70163E+00_JPRB ,&
&  0.68959E+00_JPRB, 0.68101E+00_JPRB, 0.69852E+00_JPRB, 0.68026E+00_JPRB, 0.65548E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.70499E+00_JPRB, 0.67370E+00_JPRB, 0.64350E+00_JPRB, 0.63083E+00_JPRB, 0.62585E+00_JPRB, 0.58733E+00_JPRB, 0.52369E+00_JPRB ,&
&  0.53652E+00_JPRB, 0.49579E+00_JPRB, 0.47745E+00_JPRB, 0.46049E+00_JPRB, 0.44638E+00_JPRB, 0.42125E+00_JPRB, 0.39923E+00_JPRB ,&
&  0.40398E+00_JPRB, 0.42848E+00_JPRB, 0.46355E+00_JPRB, 0.48917E+00_JPRB, 0.50064E+00_JPRB, 0.50142E+00_JPRB, 0.49867E+00_JPRB ,&
&  0.50340E+00_JPRB, 0.51425E+00_JPRB, 0.52940E+00_JPRB, 0.55204E+00_JPRB, 0.54718E+00_JPRB, 0.53753E+00_JPRB, 0.53058E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.51054E+00_JPRB, 0.49498E+00_JPRB, 0.50045E+00_JPRB, 0.50940E+00_JPRB, 0.53234E+00_JPRB, 0.56205E+00_JPRB ,&
&  0.48015E+00_JPRB, 0.49040E+00_JPRB, 0.60887E+00_JPRB, 0.64282E+00_JPRB, 0.67831E+00_JPRB, 0.71231E+00_JPRB, 0.74075E+00_JPRB ,&
&  0.76979E+00_JPRB, 0.81234E+00_JPRB, 0.86163E+00_JPRB, 0.91942E+00_JPRB, 0.10680E+01_JPRB, 0.11975E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.14283E+01_JPRB, 0.13420E+01_JPRB, 0.13087E+01_JPRB, 0.14428E+01_JPRB, 0.19899E+01_JPRB, 0.20230E+01_JPRB, 0.19441E+01_JPRB ,&
&  0.18917E+01_JPRB, 0.17468E+01_JPRB, 0.16217E+01_JPRB, 0.15840E+01_JPRB, 0.15595E+01_JPRB, 0.16094E+01_JPRB, 0.17128E+01_JPRB ,&
&  0.16982E+01_JPRB, 0.17871E+01_JPRB, 0.18995E+01_JPRB, 0.19928E+01_JPRB, 0.21051E+01_JPRB, 0.22928E+01_JPRB, 0.25237E+01_JPRB ,&
&  0.27628E+01_JPRB, 0.29565E+01_JPRB, 0.30115E+01_JPRB, 0.29292E+01_JPRB, 0.27737E+01_JPRB, 0.26014E+01_JPRB, 0.23707E+01_JPRB ,&
&  0.20497E+01_JPRB, 0.18259E+01_JPRB, 0.16833E+01_JPRB, 0.15483E+01_JPRB, 0.14206E+01_JPRB, 0.12414E+01_JPRB, 0.10019E+01_JPRB ,&
&  0.75517E+00_JPRB, 0.64372E+00_JPRB, 0.62138E+00_JPRB, 0.61972E+00_JPRB, 0.62938E+00_JPRB, 0.64670E+00_JPRB, 0.65852E+00_JPRB ,&
&  0.66478E+00_JPRB, 0.64872E+00_JPRB, 0.63931E+00_JPRB, 0.66091E+00_JPRB, 0.68789E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.69552E+00_JPRB, 0.68299E+00_JPRB, 0.63996E+00_JPRB, 0.64133E+00_JPRB, 0.60946E+00_JPRB, 0.58211E+00_JPRB, 0.54337E+00_JPRB ,&
&  0.52058E+00_JPRB, 0.48970E+00_JPRB, 0.46803E+00_JPRB, 0.43805E+00_JPRB, 0.42240E+00_JPRB, 0.41892E+00_JPRB, 0.43955E+00_JPRB ,&
&  0.46831E+00_JPRB, 0.49182E+00_JPRB, 0.50642E+00_JPRB, 0.51063E+00_JPRB, 0.50876E+00_JPRB, 0.49937E+00_JPRB, 0.49028E+00_JPRB ,&
&  0.48754E+00_JPRB, 0.49404E+00_JPRB, 0.49251E+00_JPRB, 0.49622E+00_JPRB, 0.49389E+00_JPRB, 0.48230E+00_JPRB, 0.47476E+00_JPRB ,&
&  0.45887E+00_JPRB, 0.44512E+00_JPRB, 0.44430E+00_JPRB, 0.46342E+00_JPRB, 0.48683E+00_JPRB, 0.50573E+00_JPRB, 0.47151E+00_JPRB ,&
&  0.42854E+00_JPRB, 0.41970E+00_JPRB, 0.43613E+00_JPRB, 0.56245E+00_JPRB, 0.61132E+00_JPRB, 0.63612E+00_JPRB, 0.67941E+00_JPRB ,&
&  0.67436E+00_JPRB, 0.66978E+00_JPRB, 0.70559E+00_JPRB, 0.81524E+00_JPRB, 0.89230E+00_JPRB, 0.10260E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.12189E+01_JPRB, 0.13542E+01_JPRB, 0.14782E+01_JPRB, 0.12896E+01_JPRB, 0.13680E+01_JPRB, 0.16506E+01_JPRB, 0.14599E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.12822E+01_JPRB, 0.13091E+01_JPRB, 0.13021E+01_JPRB, 0.12530E+01_JPRB, 0.12546E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.12165E+01_JPRB, 0.12644E+01_JPRB, 0.13958E+01_JPRB, 0.15250E+01_JPRB, 0.16507E+01_JPRB, 0.18221E+01_JPRB, 0.20037E+01_JPRB ,&
&  0.22313E+01_JPRB, 0.24287E+01_JPRB, 0.25548E+01_JPRB, 0.25109E+01_JPRB, 0.24463E+01_JPRB, 0.23267E+01_JPRB, 0.21364E+01_JPRB ,&
&  0.18629E+01_JPRB, 0.16655E+01_JPRB, 0.15356E+01_JPRB, 0.14385E+01_JPRB, 0.13182E+01_JPRB, 0.11509E+01_JPRB, 0.94031E+00_JPRB ,&
&  0.78359E+00_JPRB, 0.68214E+00_JPRB, 0.64189E+00_JPRB, 0.63140E+00_JPRB, 0.63761E+00_JPRB, 0.64372E+00_JPRB, 0.63644E+00_JPRB ,&
&  0.62164E+00_JPRB, 0.60921E+00_JPRB, 0.60250E+00_JPRB, 0.61535E+00_JPRB, 0.66928E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.65070E+00_JPRB, 0.67092E+00_JPRB, 0.68869E+00_JPRB, 0.66397E+00_JPRB, 0.63115E+00_JPRB, 0.57077E+00_JPRB, 0.52689E+00_JPRB ,&
&  0.49769E+00_JPRB, 0.49235E+00_JPRB, 0.49266E+00_JPRB, 0.49824E+00_JPRB, 0.51010E+00_JPRB, 0.53036E+00_JPRB, 0.54536E+00_JPRB ,&
&  0.55743E+00_JPRB, 0.55791E+00_JPRB, 0.55750E+00_JPRB, 0.55063E+00_JPRB, 0.53581E+00_JPRB, 0.51467E+00_JPRB, 0.49863E+00_JPRB ,&
&  0.49254E+00_JPRB, 0.49073E+00_JPRB, 0.48345E+00_JPRB, 0.47005E+00_JPRB, 0.46049E+00_JPRB, 0.44632E+00_JPRB, 0.44892E+00_JPRB ,&
&  0.44073E+00_JPRB, 0.43454E+00_JPRB, 0.44498E+00_JPRB, 0.45962E+00_JPRB, 0.47481E+00_JPRB, 0.49054E+00_JPRB, 0.44814E+00_JPRB ,&
&  0.40272E+00_JPRB, 0.38105E+00_JPRB, 0.37218E+00_JPRB, 0.37896E+00_JPRB, 0.43295E+00_JPRB, 0.56533E+00_JPRB, 0.62186E+00_JPRB ,&
&  0.65886E+00_JPRB, 0.68075E+00_JPRB, 0.70698E+00_JPRB, 0.74013E+00_JPRB, 0.76333E+00_JPRB, 0.83326E+00_JPRB, 0.93704E+00_JPRB ,&
&  0.10427E+01_JPRB, 0.11441E+01_JPRB, 0.12303E+01_JPRB, 0.10603E+01_JPRB, 0.96648E+00_JPRB, 0.99558E+00_JPRB, 0.10531E+01_JPRB ,&
&  0.11222E+01_JPRB, 0.10762E+01_JPRB, 0.10579E+01_JPRB, 0.10935E+01_JPRB, 0.10786E+01_JPRB, 0.10623E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10625E+01_JPRB, 0.11696E+01_JPRB, 0.12233E+01_JPRB, 0.13217E+01_JPRB, 0.14706E+01_JPRB, 0.16008E+01_JPRB ,&
&  0.17610E+01_JPRB, 0.18826E+01_JPRB, 0.19450E+01_JPRB, 0.18562E+01_JPRB, 0.18049E+01_JPRB, 0.18051E+01_JPRB, 0.17648E+01_JPRB ,&
&  0.16081E+01_JPRB, 0.14878E+01_JPRB, 0.13865E+01_JPRB, 0.12970E+01_JPRB, 0.11732E+01_JPRB, 0.10661E+01_JPRB, 0.95288E+00_JPRB ,&
&  0.84547E+00_JPRB, 0.77007E+00_JPRB, 0.73341E+00_JPRB, 0.71024E+00_JPRB, 0.70331E+00_JPRB, 0.70027E+00_JPRB, 0.68671E+00_JPRB ,&
&  0.66407E+00_JPRB, 0.64005E+00_JPRB, 0.62744E+00_JPRB, 0.62909E+00_JPRB, 0.64723E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.69911E+00_JPRB, 0.69687E+00_JPRB, 0.69162E+00_JPRB, 0.68159E+00_JPRB, 0.65874E+00_JPRB, 0.62425E+00_JPRB, 0.60478E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.59763E+00_JPRB, 0.59741E+00_JPRB, 0.59719E+00_JPRB, 0.60137E+00_JPRB, 0.61138E+00_JPRB, 0.62089E+00_JPRB ,&
&  0.62327E+00_JPRB, 0.61314E+00_JPRB, 0.59398E+00_JPRB, 0.56092E+00_JPRB, 0.53395E+00_JPRB, 0.51422E+00_JPRB, 0.50436E+00_JPRB ,&
&  0.50172E+00_JPRB, 0.49164E+00_JPRB, 0.47384E+00_JPRB, 0.44967E+00_JPRB, 0.43059E+00_JPRB, 0.41512E+00_JPRB, 0.41783E+00_JPRB ,&
&  0.43137E+00_JPRB, 0.43705E+00_JPRB, 0.44255E+00_JPRB, 0.45126E+00_JPRB, 0.45663E+00_JPRB, 0.47100E+00_JPRB, 0.41538E+00_JPRB ,&
&  0.37576E+00_JPRB, 0.35671E+00_JPRB, 0.34329E+00_JPRB, 0.34100E+00_JPRB, 0.35245E+00_JPRB, 0.38107E+00_JPRB, 0.44440E+00_JPRB ,&
&  0.57922E+00_JPRB, 0.62055E+00_JPRB, 0.63911E+00_JPRB, 0.66158E+00_JPRB, 0.67912E+00_JPRB, 0.69273E+00_JPRB, 0.74136E+00_JPRB ,&
&  0.80700E+00_JPRB, 0.89031E+00_JPRB, 0.95926E+00_JPRB, 0.97253E+00_JPRB, 0.97183E+00_JPRB, 0.94799E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.11994E+01_JPRB, 0.10480E+01_JPRB, 0.95649E+00_JPRB, 0.89787E+00_JPRB, 0.86264E+00_JPRB, 0.88794E+00_JPRB ,&
&  0.93043E+00_JPRB, 0.97145E+00_JPRB, 0.10335E+01_JPRB, 0.10798E+01_JPRB, 0.11364E+01_JPRB, 0.12220E+01_JPRB, 0.13502E+01_JPRB ,&
&  0.14837E+01_JPRB, 0.16540E+01_JPRB, 0.19904E+01_JPRB, 0.17595E+01_JPRB, 0.16332E+01_JPRB, 0.15884E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.14885E+01_JPRB, 0.13639E+01_JPRB, 0.12049E+01_JPRB, 0.11219E+01_JPRB, 0.10276E+01_JPRB, 0.95360E+00_JPRB ,&
&  0.88526E+00_JPRB, 0.82749E+00_JPRB, 0.79562E+00_JPRB, 0.77229E+00_JPRB, 0.76647E+00_JPRB, 0.76058E+00_JPRB, 0.75109E+00_JPRB ,&
&  0.73770E+00_JPRB, 0.72411E+00_JPRB, 0.71320E+00_JPRB, 0.70491E+00_JPRB, 0.69967E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.72900E+00_JPRB, 0.71786E+00_JPRB, 0.70509E+00_JPRB, 0.69456E+00_JPRB, 0.68095E+00_JPRB, 0.67126E+00_JPRB, 0.66365E+00_JPRB ,&
&  0.65920E+00_JPRB, 0.65850E+00_JPRB, 0.65658E+00_JPRB, 0.65454E+00_JPRB, 0.64950E+00_JPRB, 0.64308E+00_JPRB, 0.63268E+00_JPRB ,&
&  0.61605E+00_JPRB, 0.57272E+00_JPRB, 0.54007E+00_JPRB, 0.52337E+00_JPRB, 0.50959E+00_JPRB, 0.50143E+00_JPRB, 0.49383E+00_JPRB ,&
&  0.48043E+00_JPRB, 0.46613E+00_JPRB, 0.44619E+00_JPRB, 0.41970E+00_JPRB, 0.40655E+00_JPRB, 0.41019E+00_JPRB, 0.42030E+00_JPRB ,&
&  0.43519E+00_JPRB, 0.44406E+00_JPRB, 0.43771E+00_JPRB, 0.43393E+00_JPRB, 0.44385E+00_JPRB, 0.43707E+00_JPRB, 0.38377E+00_JPRB ,&
&  0.35496E+00_JPRB, 0.33516E+00_JPRB, 0.33027E+00_JPRB, 0.32906E+00_JPRB, 0.33579E+00_JPRB, 0.35604E+00_JPRB, 0.40635E+00_JPRB ,&
&  0.49162E+00_JPRB, 0.59955E+00_JPRB, 0.61103E+00_JPRB, 0.61759E+00_JPRB, 0.62717E+00_JPRB, 0.63697E+00_JPRB, 0.65478E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.71742E+00_JPRB, 0.74852E+00_JPRB, 0.78168E+00_JPRB, 0.81139E+00_JPRB, 0.82392E+00_JPRB, 0.83123E+00_JPRB ,&
&  0.85341E+00_JPRB, 0.83835E+00_JPRB, 0.83785E+00_JPRB, 0.83801E+00_JPRB, 0.84185E+00_JPRB, 0.82086E+00_JPRB, 0.83156E+00_JPRB ,&
&  0.87939E+00_JPRB, 0.93164E+00_JPRB, 0.98480E+00_JPRB, 0.10231E+01_JPRB, 0.10693E+01_JPRB, 0.11405E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.13806E+01_JPRB, 0.15159E+01_JPRB, 0.14979E+01_JPRB, 0.14714E+01_JPRB, 0.14543E+01_JPRB, 0.14515E+01_JPRB ,&
&  0.14397E+01_JPRB, 0.14478E+01_JPRB, 0.13706E+01_JPRB, 0.12571E+01_JPRB, 0.11533E+01_JPRB, 0.10368E+01_JPRB, 0.96523E+00_JPRB ,&
&  0.91471E+00_JPRB, 0.84471E+00_JPRB, 0.80952E+00_JPRB, 0.79034E+00_JPRB, 0.78083E+00_JPRB, 0.77043E+00_JPRB, 0.75996E+00_JPRB ,&
&  0.75596E+00_JPRB, 0.75427E+00_JPRB, 0.75304E+00_JPRB, 0.74743E+00_JPRB, 0.73923E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.72079E+00_JPRB, 0.70835E+00_JPRB, 0.69745E+00_JPRB, 0.68792E+00_JPRB, 0.67721E+00_JPRB, 0.66658E+00_JPRB, 0.65586E+00_JPRB ,&
&  0.64690E+00_JPRB, 0.63925E+00_JPRB, 0.63124E+00_JPRB, 0.62060E+00_JPRB, 0.60903E+00_JPRB, 0.59546E+00_JPRB, 0.58365E+00_JPRB ,&
&  0.57074E+00_JPRB, 0.55619E+00_JPRB, 0.53568E+00_JPRB, 0.51668E+00_JPRB, 0.50262E+00_JPRB, 0.49301E+00_JPRB, 0.48219E+00_JPRB ,&
&  0.47356E+00_JPRB, 0.45956E+00_JPRB, 0.43423E+00_JPRB, 0.41364E+00_JPRB, 0.39548E+00_JPRB, 0.39322E+00_JPRB, 0.42291E+00_JPRB ,&
&  0.43594E+00_JPRB, 0.43696E+00_JPRB, 0.43569E+00_JPRB, 0.42751E+00_JPRB, 0.41882E+00_JPRB, 0.36452E+00_JPRB, 0.36291E+00_JPRB ,&
&  0.34616E+00_JPRB, 0.33360E+00_JPRB, 0.33144E+00_JPRB, 0.33231E+00_JPRB, 0.33824E+00_JPRB, 0.35394E+00_JPRB, 0.37524E+00_JPRB ,&
&  0.41377E+00_JPRB, 0.53649E+00_JPRB, 0.56698E+00_JPRB, 0.59092E+00_JPRB, 0.59698E+00_JPRB, 0.60167E+00_JPRB, 0.60981E+00_JPRB ,&
&  0.62144E+00_JPRB, 0.63761E+00_JPRB, 0.65014E+00_JPRB, 0.66692E+00_JPRB, 0.68426E+00_JPRB, 0.69620E+00_JPRB, 0.72165E+00_JPRB ,&
&  0.74006E+00_JPRB, 0.75490E+00_JPRB, 0.76601E+00_JPRB, 0.77432E+00_JPRB, 0.79264E+00_JPRB, 0.78870E+00_JPRB, 0.78650E+00_JPRB ,&
&  0.82975E+00_JPRB, 0.89510E+00_JPRB, 0.95091E+00_JPRB, 0.10049E+01_JPRB, 0.10437E+01_JPRB, 0.10901E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.12547E+01_JPRB, 0.13285E+01_JPRB, 0.13459E+01_JPRB, 0.13383E+01_JPRB, 0.13266E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.13245E+01_JPRB, 0.12796E+01_JPRB, 0.12471E+01_JPRB, 0.11869E+01_JPRB, 0.11120E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.98000E+00_JPRB, 0.91937E+00_JPRB, 0.88191E+00_JPRB, 0.85604E+00_JPRB, 0.83789E+00_JPRB, 0.82606E+00_JPRB, 0.81432E+00_JPRB ,&
&  0.80360E+00_JPRB, 0.78815E+00_JPRB, 0.77006E+00_JPRB, 0.75169E+00_JPRB, 0.73453E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.69898E+00_JPRB, 0.68764E+00_JPRB, 0.67727E+00_JPRB, 0.66318E+00_JPRB, 0.64922E+00_JPRB, 0.63631E+00_JPRB, 0.62646E+00_JPRB ,&
&  0.61855E+00_JPRB, 0.60990E+00_JPRB, 0.60168E+00_JPRB, 0.59165E+00_JPRB, 0.58200E+00_JPRB, 0.57284E+00_JPRB, 0.56466E+00_JPRB ,&
&  0.55726E+00_JPRB, 0.54561E+00_JPRB, 0.53366E+00_JPRB, 0.52007E+00_JPRB, 0.50790E+00_JPRB, 0.49650E+00_JPRB, 0.48352E+00_JPRB ,&
&  0.47331E+00_JPRB, 0.45496E+00_JPRB, 0.43247E+00_JPRB, 0.42051E+00_JPRB, 0.39976E+00_JPRB, 0.38987E+00_JPRB, 0.38678E+00_JPRB ,&
&  0.40460E+00_JPRB, 0.39313E+00_JPRB, 0.39963E+00_JPRB, 0.38389E+00_JPRB, 0.37062E+00_JPRB, 0.35842E+00_JPRB, 0.34936E+00_JPRB ,&
&  0.34258E+00_JPRB, 0.33616E+00_JPRB, 0.33632E+00_JPRB, 0.34002E+00_JPRB, 0.34348E+00_JPRB, 0.35563E+00_JPRB, 0.37235E+00_JPRB ,&
&  0.41127E+00_JPRB, 0.51687E+00_JPRB, 0.53983E+00_JPRB, 0.56395E+00_JPRB, 0.57026E+00_JPRB, 0.57488E+00_JPRB, 0.57982E+00_JPRB ,&
&  0.58793E+00_JPRB, 0.59506E+00_JPRB, 0.60181E+00_JPRB, 0.60465E+00_JPRB, 0.62669E+00_JPRB, 0.64445E+00_JPRB, 0.66601E+00_JPRB ,&
&  0.67739E+00_JPRB, 0.68743E+00_JPRB, 0.69518E+00_JPRB, 0.71183E+00_JPRB, 0.74300E+00_JPRB, 0.75588E+00_JPRB, 0.75756E+00_JPRB ,&
&  0.78002E+00_JPRB, 0.81560E+00_JPRB, 0.87277E+00_JPRB, 0.94402E+00_JPRB, 0.99224E+00_JPRB, 0.10327E+01_JPRB, 0.10964E+01_JPRB ,&
&  0.11385E+01_JPRB, 0.12116E+01_JPRB, 0.12843E+01_JPRB, 0.13109E+01_JPRB, 0.13054E+01_JPRB, 0.12821E+01_JPRB, 0.12557E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.12112E+01_JPRB, 0.11899E+01_JPRB, 0.11755E+01_JPRB, 0.11468E+01_JPRB, 0.11120E+01_JPRB, 0.10785E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10025E+01_JPRB, 0.97079E+00_JPRB, 0.93544E+00_JPRB, 0.89881E+00_JPRB, 0.86306E+00_JPRB, 0.82866E+00_JPRB ,&
&  0.79571E+00_JPRB, 0.76777E+00_JPRB, 0.74127E+00_JPRB, 0.72372E+00_JPRB, 0.70785E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.67453E+00_JPRB, 0.65896E+00_JPRB, 0.64286E+00_JPRB, 0.62816E+00_JPRB, 0.61630E+00_JPRB, 0.60724E+00_JPRB, 0.60087E+00_JPRB ,&
&  0.59570E+00_JPRB, 0.58941E+00_JPRB, 0.58276E+00_JPRB, 0.57548E+00_JPRB, 0.56836E+00_JPRB, 0.56425E+00_JPRB, 0.56028E+00_JPRB ,&
&  0.55685E+00_JPRB, 0.55004E+00_JPRB, 0.54082E+00_JPRB, 0.52892E+00_JPRB, 0.51593E+00_JPRB, 0.50505E+00_JPRB, 0.49065E+00_JPRB ,&
&  0.47781E+00_JPRB, 0.46334E+00_JPRB, 0.43880E+00_JPRB, 0.42443E+00_JPRB, 0.40150E+00_JPRB, 0.38669E+00_JPRB, 0.37892E+00_JPRB ,&
&  0.37378E+00_JPRB, 0.37486E+00_JPRB, 0.37906E+00_JPRB, 0.38390E+00_JPRB, 0.38119E+00_JPRB, 0.37532E+00_JPRB, 0.37311E+00_JPRB ,&
&  0.36701E+00_JPRB, 0.36383E+00_JPRB, 0.36157E+00_JPRB, 0.36214E+00_JPRB, 0.36558E+00_JPRB, 0.37411E+00_JPRB, 0.39038E+00_JPRB ,&
&  0.42563E+00_JPRB, 0.47472E+00_JPRB, 0.52257E+00_JPRB, 0.54880E+00_JPRB, 0.55208E+00_JPRB, 0.56018E+00_JPRB, 0.56370E+00_JPRB ,&
&  0.56971E+00_JPRB, 0.57453E+00_JPRB, 0.58238E+00_JPRB, 0.59139E+00_JPRB, 0.60422E+00_JPRB, 0.61594E+00_JPRB, 0.62921E+00_JPRB ,&
&  0.63546E+00_JPRB, 0.64188E+00_JPRB, 0.64383E+00_JPRB, 0.65562E+00_JPRB, 0.67714E+00_JPRB, 0.69762E+00_JPRB, 0.70662E+00_JPRB ,&
&  0.72276E+00_JPRB, 0.74629E+00_JPRB, 0.79009E+00_JPRB, 0.85624E+00_JPRB, 0.92604E+00_JPRB, 0.98979E+00_JPRB, 0.10495E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.11742E+01_JPRB, 0.12361E+01_JPRB, 0.12555E+01_JPRB, 0.12488E+01_JPRB, 0.12496E+01_JPRB, 0.12287E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.11865E+01_JPRB, 0.11711E+01_JPRB, 0.11569E+01_JPRB, 0.11342E+01_JPRB, 0.11001E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.98433E+00_JPRB, 0.94410E+00_JPRB, 0.90387E+00_JPRB, 0.86579E+00_JPRB, 0.82918E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.76811E+00_JPRB, 0.74437E+00_JPRB, 0.72268E+00_JPRB, 0.70505E+00_JPRB, 0.68966E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.65439E+00_JPRB, 0.64027E+00_JPRB, 0.62738E+00_JPRB, 0.61603E+00_JPRB, 0.60605E+00_JPRB, 0.59825E+00_JPRB, 0.59209E+00_JPRB ,&
&  0.58709E+00_JPRB, 0.58459E+00_JPRB, 0.58205E+00_JPRB, 0.57902E+00_JPRB, 0.57556E+00_JPRB, 0.57432E+00_JPRB, 0.57168E+00_JPRB ,&
&  0.56789E+00_JPRB, 0.56279E+00_JPRB, 0.55778E+00_JPRB, 0.55142E+00_JPRB, 0.54125E+00_JPRB, 0.52936E+00_JPRB, 0.51389E+00_JPRB ,&
&  0.50405E+00_JPRB, 0.49135E+00_JPRB, 0.47692E+00_JPRB, 0.46129E+00_JPRB, 0.45076E+00_JPRB, 0.42701E+00_JPRB, 0.41786E+00_JPRB ,&
&  0.41173E+00_JPRB, 0.40179E+00_JPRB, 0.39674E+00_JPRB, 0.39597E+00_JPRB, 0.41290E+00_JPRB, 0.42678E+00_JPRB, 0.43393E+00_JPRB ,&
&  0.42032E+00_JPRB, 0.42395E+00_JPRB, 0.42094E+00_JPRB, 0.41962E+00_JPRB, 0.42263E+00_JPRB, 0.43007E+00_JPRB, 0.45628E+00_JPRB ,&
&  0.50294E+00_JPRB, 0.52256E+00_JPRB, 0.53517E+00_JPRB, 0.54765E+00_JPRB, 0.55558E+00_JPRB, 0.56166E+00_JPRB, 0.56694E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.58479E+00_JPRB, 0.59235E+00_JPRB, 0.59660E+00_JPRB, 0.60193E+00_JPRB, 0.60735E+00_JPRB, 0.61278E+00_JPRB ,&
&  0.60968E+00_JPRB, 0.60917E+00_JPRB, 0.61175E+00_JPRB, 0.61861E+00_JPRB, 0.62890E+00_JPRB, 0.64533E+00_JPRB, 0.66372E+00_JPRB ,&
&  0.68426E+00_JPRB, 0.70097E+00_JPRB, 0.72741E+00_JPRB, 0.76319E+00_JPRB, 0.80552E+00_JPRB, 0.85656E+00_JPRB, 0.90825E+00_JPRB ,&
&  0.95771E+00_JPRB, 0.99815E+00_JPRB, 0.10348E+01_JPRB, 0.10502E+01_JPRB, 0.10561E+01_JPRB, 0.10589E+01_JPRB, 0.10542E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10341E+01_JPRB, 0.10217E+01_JPRB, 0.10056E+01_JPRB, 0.98717E+00_JPRB, 0.96520E+00_JPRB, 0.94090E+00_JPRB ,&
&  0.91281E+00_JPRB, 0.88532E+00_JPRB, 0.85804E+00_JPRB, 0.83159E+00_JPRB, 0.80565E+00_JPRB, 0.78162E+00_JPRB, 0.76008E+00_JPRB ,&
&  0.74034E+00_JPRB, 0.72086E+00_JPRB, 0.70329E+00_JPRB, 0.68680E+00_JPRB, 0.67055E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.62953E+00_JPRB, 0.62086E+00_JPRB, 0.61319E+00_JPRB, 0.60624E+00_JPRB, 0.60070E+00_JPRB, 0.59664E+00_JPRB, 0.59320E+00_JPRB ,&
&  0.59027E+00_JPRB, 0.58883E+00_JPRB, 0.58777E+00_JPRB, 0.58662E+00_JPRB, 0.58547E+00_JPRB, 0.58416E+00_JPRB, 0.58243E+00_JPRB ,&
&  0.58070E+00_JPRB, 0.57956E+00_JPRB, 0.57834E+00_JPRB, 0.57693E+00_JPRB, 0.57402E+00_JPRB, 0.56927E+00_JPRB, 0.56339E+00_JPRB ,&
&  0.55710E+00_JPRB, 0.54905E+00_JPRB, 0.53929E+00_JPRB, 0.52859E+00_JPRB, 0.52253E+00_JPRB, 0.51554E+00_JPRB, 0.50887E+00_JPRB ,&
&  0.50361E+00_JPRB, 0.50255E+00_JPRB, 0.50133E+00_JPRB, 0.50021E+00_JPRB, 0.49962E+00_JPRB, 0.50083E+00_JPRB, 0.50143E+00_JPRB ,&
&  0.50204E+00_JPRB, 0.50444E+00_JPRB, 0.51161E+00_JPRB, 0.51932E+00_JPRB, 0.52816E+00_JPRB, 0.53872E+00_JPRB, 0.55083E+00_JPRB ,&
&  0.56006E+00_JPRB, 0.56554E+00_JPRB, 0.57137E+00_JPRB, 0.57723E+00_JPRB, 0.58114E+00_JPRB, 0.58389E+00_JPRB, 0.58645E+00_JPRB ,&
&  0.58989E+00_JPRB, 0.59212E+00_JPRB, 0.59461E+00_JPRB, 0.59701E+00_JPRB, 0.59714E+00_JPRB, 0.59668E+00_JPRB, 0.59552E+00_JPRB ,&
&  0.59318E+00_JPRB, 0.59323E+00_JPRB, 0.59526E+00_JPRB, 0.59775E+00_JPRB, 0.60238E+00_JPRB, 0.60948E+00_JPRB, 0.61786E+00_JPRB ,&
&  0.62884E+00_JPRB, 0.64239E+00_JPRB, 0.65981E+00_JPRB, 0.67932E+00_JPRB, 0.70174E+00_JPRB, 0.72877E+00_JPRB, 0.75059E+00_JPRB ,&
&  0.77035E+00_JPRB, 0.78719E+00_JPRB, 0.80179E+00_JPRB, 0.81140E+00_JPRB, 0.81764E+00_JPRB, 0.82323E+00_JPRB, 0.82172E+00_JPRB ,&
&  0.81847E+00_JPRB, 0.81425E+00_JPRB, 0.81028E+00_JPRB, 0.80243E+00_JPRB, 0.79323E+00_JPRB, 0.78396E+00_JPRB, 0.77513E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.75080E+00_JPRB, 0.73917E+00_JPRB, 0.72836E+00_JPRB, 0.71669E+00_JPRB, 0.70554E+00_JPRB, 0.69437E+00_JPRB ,&
&  0.68314E+00_JPRB, 0.67152E+00_JPRB, 0.66071E+00_JPRB, 0.65025E+00_JPRB, 0.63988E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.61287E+00_JPRB, 0.61086E+00_JPRB, 0.60917E+00_JPRB, 0.60723E+00_JPRB, 0.60495E+00_JPRB, 0.60324E+00_JPRB, 0.60176E+00_JPRB ,&
&  0.60035E+00_JPRB, 0.59946E+00_JPRB, 0.59904E+00_JPRB, 0.59886E+00_JPRB, 0.59913E+00_JPRB, 0.59929E+00_JPRB, 0.59961E+00_JPRB ,&
&  0.59996E+00_JPRB, 0.60036E+00_JPRB, 0.60095E+00_JPRB, 0.60124E+00_JPRB, 0.60110E+00_JPRB, 0.60067E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.59852E+00_JPRB, 0.59696E+00_JPRB, 0.59542E+00_JPRB, 0.59386E+00_JPRB, 0.59197E+00_JPRB, 0.58981E+00_JPRB, 0.58740E+00_JPRB ,&
&  0.58526E+00_JPRB, 0.58383E+00_JPRB, 0.58262E+00_JPRB, 0.58127E+00_JPRB, 0.58049E+00_JPRB, 0.58106E+00_JPRB, 0.58159E+00_JPRB ,&
&  0.58218E+00_JPRB, 0.58312E+00_JPRB, 0.58523E+00_JPRB, 0.58730E+00_JPRB, 0.58921E+00_JPRB, 0.59120E+00_JPRB, 0.59331E+00_JPRB ,&
&  0.59453E+00_JPRB, 0.59537E+00_JPRB, 0.59597E+00_JPRB, 0.59641E+00_JPRB, 0.59619E+00_JPRB, 0.59607E+00_JPRB, 0.59588E+00_JPRB ,&
&  0.59549E+00_JPRB, 0.59517E+00_JPRB, 0.59464E+00_JPRB, 0.59347E+00_JPRB, 0.59049E+00_JPRB, 0.58840E+00_JPRB, 0.58772E+00_JPRB ,&
&  0.58825E+00_JPRB, 0.58907E+00_JPRB, 0.59008E+00_JPRB, 0.59136E+00_JPRB, 0.59344E+00_JPRB, 0.59586E+00_JPRB, 0.59869E+00_JPRB ,&
&  0.60169E+00_JPRB, 0.60522E+00_JPRB, 0.61000E+00_JPRB, 0.61504E+00_JPRB, 0.62062E+00_JPRB, 0.62681E+00_JPRB, 0.63228E+00_JPRB ,&
&  0.63804E+00_JPRB, 0.64411E+00_JPRB, 0.64916E+00_JPRB, 0.65380E+00_JPRB, 0.65849E+00_JPRB, 0.66300E+00_JPRB, 0.66610E+00_JPRB ,&
&  0.66722E+00_JPRB, 0.66804E+00_JPRB, 0.66852E+00_JPRB, 0.66724E+00_JPRB, 0.66478E+00_JPRB, 0.66265E+00_JPRB, 0.66037E+00_JPRB ,&
&  0.65771E+00_JPRB, 0.65450E+00_JPRB, 0.65148E+00_JPRB, 0.64816E+00_JPRB, 0.64402E+00_JPRB, 0.63973E+00_JPRB, 0.63564E+00_JPRB ,&
&  0.63150E+00_JPRB, 0.62712E+00_JPRB, 0.62321E+00_JPRB, 0.61949E+00_JPRB, 0.61607E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB, 0.61495E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB ,&
&  0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB, 0.12226E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.12795E+00_JPRB, 0.12780E+00_JPRB, 0.12764E+00_JPRB, 0.12747E+00_JPRB, 0.12689E+00_JPRB, 0.12666E+00_JPRB, 0.12643E+00_JPRB ,&
&  0.12631E+00_JPRB, 0.12624E+00_JPRB, 0.12631E+00_JPRB, 0.12642E+00_JPRB, 0.12654E+00_JPRB, 0.12664E+00_JPRB, 0.12672E+00_JPRB ,&
&  0.12668E+00_JPRB, 0.12667E+00_JPRB, 0.12610E+00_JPRB, 0.12567E+00_JPRB, 0.12541E+00_JPRB, 0.12524E+00_JPRB, 0.12497E+00_JPRB ,&
&  0.12476E+00_JPRB, 0.12450E+00_JPRB, 0.12418E+00_JPRB, 0.12394E+00_JPRB, 0.12394E+00_JPRB, 0.12381E+00_JPRB, 0.12380E+00_JPRB ,&
&  0.12376E+00_JPRB, 0.12388E+00_JPRB, 0.12390E+00_JPRB, 0.12394E+00_JPRB, 0.12420E+00_JPRB, 0.12442E+00_JPRB, 0.12443E+00_JPRB ,&
&  0.12441E+00_JPRB, 0.12422E+00_JPRB, 0.12404E+00_JPRB, 0.12391E+00_JPRB, 0.12383E+00_JPRB, 0.12380E+00_JPRB, 0.12374E+00_JPRB ,&
&  0.12361E+00_JPRB, 0.12341E+00_JPRB, 0.12317E+00_JPRB, 0.12291E+00_JPRB, 0.12266E+00_JPRB, 0.12251E+00_JPRB, 0.12245E+00_JPRB ,&
&  0.12241E+00_JPRB, 0.12227E+00_JPRB, 0.12206E+00_JPRB, 0.12178E+00_JPRB, 0.12151E+00_JPRB, 0.12126E+00_JPRB, 0.12101E+00_JPRB ,&
&  0.12079E+00_JPRB, 0.12068E+00_JPRB, 0.12056E+00_JPRB, 0.12045E+00_JPRB, 0.12037E+00_JPRB, 0.12027E+00_JPRB, 0.12018E+00_JPRB ,&
&  0.12009E+00_JPRB, 0.12001E+00_JPRB, 0.11994E+00_JPRB, 0.11988E+00_JPRB, 0.11984E+00_JPRB, 0.11981E+00_JPRB, 0.11981E+00_JPRB ,&
&  0.11984E+00_JPRB, 0.11986E+00_JPRB, 0.11991E+00_JPRB, 0.12004E+00_JPRB, 0.12019E+00_JPRB, 0.12037E+00_JPRB, 0.12059E+00_JPRB ,&
&  0.12083E+00_JPRB, 0.12110E+00_JPRB, 0.12139E+00_JPRB, 0.12162E+00_JPRB, 0.12195E+00_JPRB, 0.12229E+00_JPRB, 0.12263E+00_JPRB ,&
&  0.12298E+00_JPRB, 0.12341E+00_JPRB, 0.12389E+00_JPRB, 0.12440E+00_JPRB, 0.12495E+00_JPRB, 0.12527E+00_JPRB, 0.12565E+00_JPRB ,&
&  0.12609E+00_JPRB, 0.12663E+00_JPRB, 0.12695E+00_JPRB, 0.12719E+00_JPRB, 0.12746E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.13789E+00_JPRB, 0.13988E+00_JPRB, 0.14137E+00_JPRB, 0.14322E+00_JPRB, 0.14309E+00_JPRB, 0.15455E+00_JPRB, 0.15265E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.14663E+00_JPRB, 0.14613E+00_JPRB, 0.14449E+00_JPRB, 0.14255E+00_JPRB, 0.14042E+00_JPRB, 0.13848E+00_JPRB ,&
&  0.13668E+00_JPRB, 0.13470E+00_JPRB, 0.13325E+00_JPRB, 0.13205E+00_JPRB, 0.13097E+00_JPRB, 0.12998E+00_JPRB, 0.12919E+00_JPRB ,&
&  0.12911E+00_JPRB, 0.12960E+00_JPRB, 0.13053E+00_JPRB, 0.13043E+00_JPRB, 0.13019E+00_JPRB, 0.12985E+00_JPRB, 0.12969E+00_JPRB ,&
&  0.12919E+00_JPRB, 0.12883E+00_JPRB, 0.12851E+00_JPRB, 0.12801E+00_JPRB, 0.12848E+00_JPRB, 0.12891E+00_JPRB, 0.12936E+00_JPRB ,&
&  0.12977E+00_JPRB, 0.13018E+00_JPRB, 0.13010E+00_JPRB, 0.12993E+00_JPRB, 0.12953E+00_JPRB, 0.12898E+00_JPRB, 0.12815E+00_JPRB ,&
&  0.12735E+00_JPRB, 0.12652E+00_JPRB, 0.12570E+00_JPRB, 0.12511E+00_JPRB, 0.12459E+00_JPRB, 0.12410E+00_JPRB, 0.12368E+00_JPRB ,&
&  0.12346E+00_JPRB, 0.12332E+00_JPRB, 0.12316E+00_JPRB, 0.12289E+00_JPRB, 0.12256E+00_JPRB, 0.12221E+00_JPRB, 0.12191E+00_JPRB ,&
&  0.12167E+00_JPRB, 0.12125E+00_JPRB, 0.12086E+00_JPRB, 0.12045E+00_JPRB, 0.12014E+00_JPRB, 0.11976E+00_JPRB, 0.11944E+00_JPRB ,&
&  0.11914E+00_JPRB, 0.11893E+00_JPRB, 0.11887E+00_JPRB, 0.11884E+00_JPRB, 0.11883E+00_JPRB, 0.11887E+00_JPRB, 0.11899E+00_JPRB ,&
&  0.11916E+00_JPRB, 0.11934E+00_JPRB, 0.11958E+00_JPRB, 0.11976E+00_JPRB, 0.11993E+00_JPRB, 0.12008E+00_JPRB, 0.12043E+00_JPRB ,&
&  0.12086E+00_JPRB, 0.12132E+00_JPRB, 0.12176E+00_JPRB, 0.12200E+00_JPRB, 0.12274E+00_JPRB, 0.12350E+00_JPRB, 0.12433E+00_JPRB ,&
&  0.12520E+00_JPRB, 0.12632E+00_JPRB, 0.12752E+00_JPRB, 0.12880E+00_JPRB, 0.13025E+00_JPRB, 0.13126E+00_JPRB, 0.13233E+00_JPRB ,&
&  0.13375E+00_JPRB, 0.13606E+00_JPRB, 0.13698E+00_JPRB, 0.13741E+00_JPRB, 0.13774E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.16193E+00_JPRB, 0.16255E+00_JPRB, 0.16310E+00_JPRB, 0.16332E+00_JPRB, 0.16336E+00_JPRB, 0.16275E+00_JPRB, 0.16199E+00_JPRB ,&
&  0.15850E+00_JPRB, 0.15591E+00_JPRB, 0.15310E+00_JPRB, 0.15032E+00_JPRB, 0.14789E+00_JPRB, 0.14556E+00_JPRB, 0.14340E+00_JPRB ,&
&  0.14144E+00_JPRB, 0.14000E+00_JPRB, 0.13861E+00_JPRB, 0.13666E+00_JPRB, 0.13418E+00_JPRB, 0.13190E+00_JPRB, 0.13037E+00_JPRB ,&
&  0.12987E+00_JPRB, 0.13009E+00_JPRB, 0.13134E+00_JPRB, 0.13268E+00_JPRB, 0.13412E+00_JPRB, 0.13523E+00_JPRB, 0.13633E+00_JPRB ,&
&  0.13578E+00_JPRB, 0.13540E+00_JPRB, 0.13507E+00_JPRB, 0.13452E+00_JPRB, 0.13497E+00_JPRB, 0.13560E+00_JPRB, 0.13593E+00_JPRB ,&
&  0.13631E+00_JPRB, 0.13741E+00_JPRB, 0.13815E+00_JPRB, 0.13885E+00_JPRB, 0.13865E+00_JPRB, 0.13781E+00_JPRB, 0.13637E+00_JPRB ,&
&  0.13466E+00_JPRB, 0.13292E+00_JPRB, 0.13140E+00_JPRB, 0.13010E+00_JPRB, 0.12911E+00_JPRB, 0.12804E+00_JPRB, 0.12717E+00_JPRB ,&
&  0.12673E+00_JPRB, 0.12633E+00_JPRB, 0.12575E+00_JPRB, 0.12516E+00_JPRB, 0.12425E+00_JPRB, 0.12346E+00_JPRB, 0.12264E+00_JPRB ,&
&  0.12189E+00_JPRB, 0.12117E+00_JPRB, 0.12051E+00_JPRB, 0.11984E+00_JPRB, 0.11937E+00_JPRB, 0.11895E+00_JPRB, 0.11853E+00_JPRB ,&
&  0.11810E+00_JPRB, 0.11794E+00_JPRB, 0.11762E+00_JPRB, 0.11752E+00_JPRB, 0.11735E+00_JPRB, 0.11724E+00_JPRB, 0.11740E+00_JPRB ,&
&  0.11787E+00_JPRB, 0.11834E+00_JPRB, 0.11859E+00_JPRB, 0.11889E+00_JPRB, 0.11918E+00_JPRB, 0.11955E+00_JPRB, 0.12015E+00_JPRB ,&
&  0.12060E+00_JPRB, 0.12119E+00_JPRB, 0.12171E+00_JPRB, 0.12231E+00_JPRB, 0.12340E+00_JPRB, 0.12454E+00_JPRB, 0.12584E+00_JPRB ,&
&  0.12745E+00_JPRB, 0.12916E+00_JPRB, 0.13118E+00_JPRB, 0.13315E+00_JPRB, 0.13574E+00_JPRB, 0.13746E+00_JPRB, 0.14059E+00_JPRB ,&
&  0.14408E+00_JPRB, 0.14570E+00_JPRB, 0.15707E+00_JPRB, 0.15817E+00_JPRB, 0.15971E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.16737E+00_JPRB, 0.16847E+00_JPRB, 0.16836E+00_JPRB, 0.16873E+00_JPRB, 0.16791E+00_JPRB, 0.16658E+00_JPRB, 0.16472E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.15694E+00_JPRB, 0.15321E+00_JPRB, 0.14910E+00_JPRB, 0.14562E+00_JPRB, 0.14263E+00_JPRB, 0.14053E+00_JPRB ,&
&  0.13965E+00_JPRB, 0.13943E+00_JPRB, 0.13934E+00_JPRB, 0.13859E+00_JPRB, 0.13719E+00_JPRB, 0.13625E+00_JPRB, 0.13504E+00_JPRB ,&
&  0.13415E+00_JPRB, 0.13404E+00_JPRB, 0.13478E+00_JPRB, 0.13650E+00_JPRB, 0.13950E+00_JPRB, 0.14182E+00_JPRB, 0.14390E+00_JPRB ,&
&  0.14620E+00_JPRB, 0.14315E+00_JPRB, 0.14288E+00_JPRB, 0.14307E+00_JPRB, 0.14283E+00_JPRB, 0.14200E+00_JPRB, 0.14034E+00_JPRB ,&
&  0.14263E+00_JPRB, 0.14320E+00_JPRB, 0.14755E+00_JPRB, 0.14752E+00_JPRB, 0.14649E+00_JPRB, 0.14711E+00_JPRB, 0.14546E+00_JPRB ,&
&  0.14471E+00_JPRB, 0.14181E+00_JPRB, 0.13968E+00_JPRB, 0.13689E+00_JPRB, 0.13425E+00_JPRB, 0.13292E+00_JPRB, 0.13140E+00_JPRB ,&
&  0.13089E+00_JPRB, 0.13076E+00_JPRB, 0.12959E+00_JPRB, 0.12886E+00_JPRB, 0.12756E+00_JPRB, 0.12609E+00_JPRB, 0.12515E+00_JPRB ,&
&  0.12418E+00_JPRB, 0.12339E+00_JPRB, 0.12296E+00_JPRB, 0.12257E+00_JPRB, 0.12241E+00_JPRB, 0.12235E+00_JPRB, 0.12228E+00_JPRB ,&
&  0.12229E+00_JPRB, 0.12220E+00_JPRB, 0.12173E+00_JPRB, 0.12143E+00_JPRB, 0.12057E+00_JPRB, 0.11998E+00_JPRB, 0.11950E+00_JPRB ,&
&  0.11910E+00_JPRB, 0.11896E+00_JPRB, 0.11889E+00_JPRB, 0.11894E+00_JPRB, 0.11906E+00_JPRB, 0.11945E+00_JPRB, 0.11989E+00_JPRB ,&
&  0.12034E+00_JPRB, 0.12111E+00_JPRB, 0.12173E+00_JPRB, 0.12259E+00_JPRB, 0.12361E+00_JPRB, 0.12449E+00_JPRB, 0.12588E+00_JPRB ,&
&  0.12744E+00_JPRB, 0.12926E+00_JPRB, 0.13194E+00_JPRB, 0.13450E+00_JPRB, 0.13743E+00_JPRB, 0.14156E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.15124E+00_JPRB, 0.15752E+00_JPRB, 0.15834E+00_JPRB, 0.16278E+00_JPRB, 0.16574E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.16752E+00_JPRB, 0.16908E+00_JPRB, 0.16979E+00_JPRB, 0.17079E+00_JPRB, 0.17077E+00_JPRB, 0.16869E+00_JPRB, 0.16662E+00_JPRB ,&
&  0.16173E+00_JPRB, 0.15772E+00_JPRB, 0.15287E+00_JPRB, 0.14828E+00_JPRB, 0.14399E+00_JPRB, 0.14119E+00_JPRB, 0.13961E+00_JPRB ,&
&  0.13976E+00_JPRB, 0.14124E+00_JPRB, 0.14238E+00_JPRB, 0.14303E+00_JPRB, 0.14355E+00_JPRB, 0.14462E+00_JPRB, 0.14490E+00_JPRB ,&
&  0.14459E+00_JPRB, 0.14352E+00_JPRB, 0.14274E+00_JPRB, 0.14367E+00_JPRB, 0.14559E+00_JPRB, 0.14760E+00_JPRB, 0.14927E+00_JPRB ,&
&  0.15081E+00_JPRB, 0.15032E+00_JPRB, 0.15036E+00_JPRB, 0.14961E+00_JPRB, 0.14895E+00_JPRB, 0.14815E+00_JPRB, 0.14775E+00_JPRB ,&
&  0.14821E+00_JPRB, 0.14861E+00_JPRB, 0.15006E+00_JPRB, 0.15093E+00_JPRB, 0.14970E+00_JPRB, 0.15100E+00_JPRB, 0.15072E+00_JPRB ,&
&  0.14970E+00_JPRB, 0.14683E+00_JPRB, 0.14384E+00_JPRB, 0.14155E+00_JPRB, 0.13921E+00_JPRB, 0.13737E+00_JPRB, 0.13586E+00_JPRB ,&
&  0.13445E+00_JPRB, 0.13366E+00_JPRB, 0.13206E+00_JPRB, 0.13071E+00_JPRB, 0.12987E+00_JPRB, 0.12899E+00_JPRB, 0.12842E+00_JPRB ,&
&  0.12802E+00_JPRB, 0.12738E+00_JPRB, 0.12722E+00_JPRB, 0.12690E+00_JPRB, 0.12700E+00_JPRB, 0.12716E+00_JPRB, 0.12750E+00_JPRB ,&
&  0.12820E+00_JPRB, 0.12902E+00_JPRB, 0.12894E+00_JPRB, 0.12877E+00_JPRB, 0.12746E+00_JPRB, 0.12602E+00_JPRB, 0.12473E+00_JPRB ,&
&  0.12316E+00_JPRB, 0.12195E+00_JPRB, 0.12120E+00_JPRB, 0.12049E+00_JPRB, 0.12040E+00_JPRB, 0.12047E+00_JPRB, 0.12049E+00_JPRB ,&
&  0.12068E+00_JPRB, 0.12096E+00_JPRB, 0.12137E+00_JPRB, 0.12219E+00_JPRB, 0.12291E+00_JPRB, 0.12395E+00_JPRB, 0.12510E+00_JPRB ,&
&  0.12603E+00_JPRB, 0.12773E+00_JPRB, 0.12954E+00_JPRB, 0.13173E+00_JPRB, 0.13576E+00_JPRB, 0.13938E+00_JPRB, 0.14600E+00_JPRB ,&
&  0.15155E+00_JPRB, 0.16117E+00_JPRB, 0.15960E+00_JPRB, 0.16265E+00_JPRB, 0.16579E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.16962E+00_JPRB, 0.17056E+00_JPRB, 0.17030E+00_JPRB, 0.17043E+00_JPRB, 0.17001E+00_JPRB, 0.17118E+00_JPRB, 0.17054E+00_JPRB ,&
&  0.17063E+00_JPRB, 0.16982E+00_JPRB, 0.16898E+00_JPRB, 0.16681E+00_JPRB, 0.16568E+00_JPRB, 0.15986E+00_JPRB, 0.15842E+00_JPRB ,&
&  0.15735E+00_JPRB, 0.15825E+00_JPRB, 0.15855E+00_JPRB, 0.15907E+00_JPRB, 0.16005E+00_JPRB, 0.16170E+00_JPRB, 0.16185E+00_JPRB ,&
&  0.16087E+00_JPRB, 0.15884E+00_JPRB, 0.15652E+00_JPRB, 0.15438E+00_JPRB, 0.15475E+00_JPRB, 0.15449E+00_JPRB, 0.15365E+00_JPRB ,&
&  0.15267E+00_JPRB, 0.15322E+00_JPRB, 0.15374E+00_JPRB, 0.15502E+00_JPRB, 0.15992E+00_JPRB, 0.15857E+00_JPRB, 0.15731E+00_JPRB ,&
&  0.15701E+00_JPRB, 0.15679E+00_JPRB, 0.15715E+00_JPRB, 0.15739E+00_JPRB, 0.15899E+00_JPRB, 0.15847E+00_JPRB, 0.16083E+00_JPRB ,&
&  0.15799E+00_JPRB, 0.15616E+00_JPRB, 0.15382E+00_JPRB, 0.15106E+00_JPRB, 0.14775E+00_JPRB, 0.14565E+00_JPRB, 0.14363E+00_JPRB ,&
&  0.14146E+00_JPRB, 0.13963E+00_JPRB, 0.13726E+00_JPRB, 0.13638E+00_JPRB, 0.13519E+00_JPRB, 0.13416E+00_JPRB, 0.13321E+00_JPRB ,&
&  0.13247E+00_JPRB, 0.13143E+00_JPRB, 0.13050E+00_JPRB, 0.12988E+00_JPRB, 0.12966E+00_JPRB, 0.12977E+00_JPRB, 0.13070E+00_JPRB ,&
&  0.13252E+00_JPRB, 0.13406E+00_JPRB, 0.13525E+00_JPRB, 0.13613E+00_JPRB, 0.13478E+00_JPRB, 0.13234E+00_JPRB, 0.12987E+00_JPRB ,&
&  0.12678E+00_JPRB, 0.12504E+00_JPRB, 0.12345E+00_JPRB, 0.12251E+00_JPRB, 0.12228E+00_JPRB, 0.12229E+00_JPRB, 0.12258E+00_JPRB ,&
&  0.12289E+00_JPRB, 0.12333E+00_JPRB, 0.12363E+00_JPRB, 0.12402E+00_JPRB, 0.12451E+00_JPRB, 0.12501E+00_JPRB, 0.12573E+00_JPRB ,&
&  0.12686E+00_JPRB, 0.12834E+00_JPRB, 0.13040E+00_JPRB, 0.13286E+00_JPRB, 0.13567E+00_JPRB, 0.13939E+00_JPRB, 0.14492E+00_JPRB ,&
&  0.15083E+00_JPRB, 0.16117E+00_JPRB, 0.16233E+00_JPRB, 0.16642E+00_JPRB, 0.16889E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.17209E+00_JPRB, 0.17353E+00_JPRB, 0.17404E+00_JPRB, 0.17515E+00_JPRB, 0.17558E+00_JPRB, 0.17687E+00_JPRB, 0.17796E+00_JPRB ,&
&  0.18036E+00_JPRB, 0.18252E+00_JPRB, 0.18305E+00_JPRB, 0.18142E+00_JPRB, 0.18060E+00_JPRB, 0.17833E+00_JPRB, 0.17757E+00_JPRB ,&
&  0.17760E+00_JPRB, 0.17876E+00_JPRB, 0.17902E+00_JPRB, 0.17918E+00_JPRB, 0.17921E+00_JPRB, 0.17924E+00_JPRB, 0.17966E+00_JPRB ,&
&  0.17784E+00_JPRB, 0.17738E+00_JPRB, 0.17947E+00_JPRB, 0.17887E+00_JPRB, 0.17836E+00_JPRB, 0.17720E+00_JPRB, 0.17657E+00_JPRB ,&
&  0.17285E+00_JPRB, 0.16739E+00_JPRB, 0.16061E+00_JPRB, 0.16315E+00_JPRB, 0.17026E+00_JPRB, 0.16838E+00_JPRB, 0.16760E+00_JPRB ,&
&  0.16696E+00_JPRB, 0.16580E+00_JPRB, 0.16529E+00_JPRB, 0.16477E+00_JPRB, 0.16520E+00_JPRB, 0.16569E+00_JPRB, 0.16708E+00_JPRB ,&
&  0.16906E+00_JPRB, 0.17196E+00_JPRB, 0.17185E+00_JPRB, 0.16600E+00_JPRB, 0.16277E+00_JPRB, 0.16158E+00_JPRB, 0.15851E+00_JPRB ,&
&  0.15498E+00_JPRB, 0.15336E+00_JPRB, 0.15254E+00_JPRB, 0.15064E+00_JPRB, 0.15019E+00_JPRB, 0.14976E+00_JPRB, 0.14770E+00_JPRB ,&
&  0.14491E+00_JPRB, 0.14285E+00_JPRB, 0.13976E+00_JPRB, 0.13656E+00_JPRB, 0.13428E+00_JPRB, 0.13317E+00_JPRB, 0.13315E+00_JPRB ,&
&  0.13469E+00_JPRB, 0.13646E+00_JPRB, 0.14189E+00_JPRB, 0.14557E+00_JPRB, 0.14291E+00_JPRB, 0.13936E+00_JPRB, 0.13577E+00_JPRB ,&
&  0.13105E+00_JPRB, 0.12876E+00_JPRB, 0.12697E+00_JPRB, 0.12569E+00_JPRB, 0.12557E+00_JPRB, 0.12630E+00_JPRB, 0.12691E+00_JPRB ,&
&  0.12761E+00_JPRB, 0.12847E+00_JPRB, 0.12887E+00_JPRB, 0.12904E+00_JPRB, 0.12906E+00_JPRB, 0.12927E+00_JPRB, 0.12978E+00_JPRB ,&
&  0.13047E+00_JPRB, 0.13208E+00_JPRB, 0.13443E+00_JPRB, 0.13697E+00_JPRB, 0.13874E+00_JPRB, 0.14009E+00_JPRB, 0.14293E+00_JPRB ,&
&  0.14630E+00_JPRB, 0.15415E+00_JPRB, 0.16536E+00_JPRB, 0.17435E+00_JPRB, 0.17123E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.18445E+00_JPRB, 0.18527E+00_JPRB, 0.18630E+00_JPRB, 0.18753E+00_JPRB, 0.18900E+00_JPRB, 0.19063E+00_JPRB, 0.19197E+00_JPRB ,&
&  0.19343E+00_JPRB, 0.19437E+00_JPRB, 0.19460E+00_JPRB, 0.19363E+00_JPRB, 0.19293E+00_JPRB, 0.19341E+00_JPRB, 0.19475E+00_JPRB ,&
&  0.19322E+00_JPRB, 0.19060E+00_JPRB, 0.18962E+00_JPRB, 0.18963E+00_JPRB, 0.18921E+00_JPRB, 0.18896E+00_JPRB, 0.18967E+00_JPRB ,&
&  0.19058E+00_JPRB, 0.19038E+00_JPRB, 0.19017E+00_JPRB, 0.18935E+00_JPRB, 0.18864E+00_JPRB, 0.18806E+00_JPRB, 0.18722E+00_JPRB ,&
&  0.18449E+00_JPRB, 0.17477E+00_JPRB, 0.16812E+00_JPRB, 0.17091E+00_JPRB, 0.17814E+00_JPRB, 0.17759E+00_JPRB, 0.17754E+00_JPRB ,&
&  0.17702E+00_JPRB, 0.17577E+00_JPRB, 0.17435E+00_JPRB, 0.17297E+00_JPRB, 0.17238E+00_JPRB, 0.17264E+00_JPRB, 0.17340E+00_JPRB ,&
&  0.17419E+00_JPRB, 0.17600E+00_JPRB, 0.17817E+00_JPRB, 0.17925E+00_JPRB, 0.18029E+00_JPRB, 0.18074E+00_JPRB, 0.18089E+00_JPRB ,&
&  0.18067E+00_JPRB, 0.17988E+00_JPRB, 0.17945E+00_JPRB, 0.17901E+00_JPRB, 0.17859E+00_JPRB, 0.17776E+00_JPRB, 0.17468E+00_JPRB ,&
&  0.16934E+00_JPRB, 0.16439E+00_JPRB, 0.15835E+00_JPRB, 0.15231E+00_JPRB, 0.14669E+00_JPRB, 0.14260E+00_JPRB, 0.13939E+00_JPRB ,&
&  0.13880E+00_JPRB, 0.14205E+00_JPRB, 0.14915E+00_JPRB, 0.15821E+00_JPRB, 0.16517E+00_JPRB, 0.15473E+00_JPRB, 0.14844E+00_JPRB ,&
&  0.14285E+00_JPRB, 0.13853E+00_JPRB, 0.13590E+00_JPRB, 0.13452E+00_JPRB, 0.13365E+00_JPRB, 0.13373E+00_JPRB, 0.13433E+00_JPRB ,&
&  0.13547E+00_JPRB, 0.13675E+00_JPRB, 0.13795E+00_JPRB, 0.13865E+00_JPRB, 0.13889E+00_JPRB, 0.13871E+00_JPRB, 0.13832E+00_JPRB ,&
&  0.13862E+00_JPRB, 0.14058E+00_JPRB, 0.14434E+00_JPRB, 0.14886E+00_JPRB, 0.15268E+00_JPRB, 0.15621E+00_JPRB, 0.16120E+00_JPRB ,&
&  0.16752E+00_JPRB, 0.17298E+00_JPRB, 0.17699E+00_JPRB, 0.18106E+00_JPRB, 0.18316E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.20179E+00_JPRB, 0.20267E+00_JPRB, 0.20357E+00_JPRB, 0.20455E+00_JPRB, 0.20634E+00_JPRB, 0.20769E+00_JPRB, 0.20929E+00_JPRB ,&
&  0.21043E+00_JPRB, 0.20914E+00_JPRB, 0.20745E+00_JPRB, 0.20617E+00_JPRB, 0.20576E+00_JPRB, 0.20533E+00_JPRB, 0.20513E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.20566E+00_JPRB, 0.20331E+00_JPRB, 0.20174E+00_JPRB, 0.19896E+00_JPRB, 0.19707E+00_JPRB, 0.19671E+00_JPRB ,&
&  0.19825E+00_JPRB, 0.19815E+00_JPRB, 0.19786E+00_JPRB, 0.19715E+00_JPRB, 0.19609E+00_JPRB, 0.19561E+00_JPRB, 0.19461E+00_JPRB ,&
&  0.19409E+00_JPRB, 0.19335E+00_JPRB, 0.18863E+00_JPRB, 0.18932E+00_JPRB, 0.18885E+00_JPRB, 0.18864E+00_JPRB, 0.18805E+00_JPRB ,&
&  0.18742E+00_JPRB, 0.18648E+00_JPRB, 0.18477E+00_JPRB, 0.18288E+00_JPRB, 0.18134E+00_JPRB, 0.18056E+00_JPRB, 0.18056E+00_JPRB ,&
&  0.18094E+00_JPRB, 0.18180E+00_JPRB, 0.18327E+00_JPRB, 0.18524E+00_JPRB, 0.18607E+00_JPRB, 0.18663E+00_JPRB, 0.18746E+00_JPRB ,&
&  0.18781E+00_JPRB, 0.18788E+00_JPRB, 0.18795E+00_JPRB, 0.18805E+00_JPRB, 0.18799E+00_JPRB, 0.18742E+00_JPRB, 0.18693E+00_JPRB ,&
&  0.18562E+00_JPRB, 0.18350E+00_JPRB, 0.18213E+00_JPRB, 0.18037E+00_JPRB, 0.17552E+00_JPRB, 0.16534E+00_JPRB, 0.15962E+00_JPRB ,&
&  0.15953E+00_JPRB, 0.16401E+00_JPRB, 0.17003E+00_JPRB, 0.17295E+00_JPRB, 0.17738E+00_JPRB, 0.17757E+00_JPRB, 0.17438E+00_JPRB ,&
&  0.16715E+00_JPRB, 0.16111E+00_JPRB, 0.15692E+00_JPRB, 0.15307E+00_JPRB, 0.15066E+00_JPRB, 0.15004E+00_JPRB, 0.15058E+00_JPRB ,&
&  0.15245E+00_JPRB, 0.15534E+00_JPRB, 0.15800E+00_JPRB, 0.15942E+00_JPRB, 0.15948E+00_JPRB, 0.15803E+00_JPRB, 0.15645E+00_JPRB ,&
&  0.15580E+00_JPRB, 0.15933E+00_JPRB, 0.16706E+00_JPRB, 0.17637E+00_JPRB, 0.18019E+00_JPRB, 0.18154E+00_JPRB, 0.18289E+00_JPRB ,&
&  0.18586E+00_JPRB, 0.19006E+00_JPRB, 0.19440E+00_JPRB, 0.19743E+00_JPRB, 0.19990E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.21926E+00_JPRB, 0.21989E+00_JPRB, 0.22068E+00_JPRB, 0.22184E+00_JPRB, 0.22344E+00_JPRB, 0.22430E+00_JPRB, 0.22497E+00_JPRB ,&
&  0.22432E+00_JPRB, 0.22263E+00_JPRB, 0.22071E+00_JPRB, 0.21870E+00_JPRB, 0.21791E+00_JPRB, 0.21805E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.21815E+00_JPRB, 0.21821E+00_JPRB, 0.21675E+00_JPRB, 0.21411E+00_JPRB, 0.21104E+00_JPRB, 0.20903E+00_JPRB, 0.20853E+00_JPRB ,&
&  0.20807E+00_JPRB, 0.20735E+00_JPRB, 0.20714E+00_JPRB, 0.20699E+00_JPRB, 0.20533E+00_JPRB, 0.20445E+00_JPRB, 0.20486E+00_JPRB ,&
&  0.20569E+00_JPRB, 0.20705E+00_JPRB, 0.20735E+00_JPRB, 0.20470E+00_JPRB, 0.20695E+00_JPRB, 0.20664E+00_JPRB, 0.20479E+00_JPRB ,&
&  0.20246E+00_JPRB, 0.20010E+00_JPRB, 0.19800E+00_JPRB, 0.19599E+00_JPRB, 0.19403E+00_JPRB, 0.19263E+00_JPRB, 0.19180E+00_JPRB ,&
&  0.19169E+00_JPRB, 0.19192E+00_JPRB, 0.19230E+00_JPRB, 0.19313E+00_JPRB, 0.19373E+00_JPRB, 0.19405E+00_JPRB, 0.19403E+00_JPRB ,&
&  0.19407E+00_JPRB, 0.19422E+00_JPRB, 0.19456E+00_JPRB, 0.19480E+00_JPRB, 0.19508E+00_JPRB, 0.19535E+00_JPRB, 0.19533E+00_JPRB ,&
&  0.19440E+00_JPRB, 0.19281E+00_JPRB, 0.19067E+00_JPRB, 0.19000E+00_JPRB, 0.19004E+00_JPRB, 0.18882E+00_JPRB, 0.18666E+00_JPRB ,&
&  0.18449E+00_JPRB, 0.18387E+00_JPRB, 0.18474E+00_JPRB, 0.18584E+00_JPRB, 0.18671E+00_JPRB, 0.18698E+00_JPRB, 0.18668E+00_JPRB ,&
&  0.18573E+00_JPRB, 0.18375E+00_JPRB, 0.18115E+00_JPRB, 0.17914E+00_JPRB, 0.17809E+00_JPRB, 0.17805E+00_JPRB, 0.17888E+00_JPRB ,&
&  0.17900E+00_JPRB, 0.17895E+00_JPRB, 0.17988E+00_JPRB, 0.18063E+00_JPRB, 0.18110E+00_JPRB, 0.18176E+00_JPRB, 0.18311E+00_JPRB ,&
&  0.18502E+00_JPRB, 0.18799E+00_JPRB, 0.19159E+00_JPRB, 0.19532E+00_JPRB, 0.19792E+00_JPRB, 0.19704E+00_JPRB, 0.19733E+00_JPRB ,&
&  0.20132E+00_JPRB, 0.20632E+00_JPRB, 0.21058E+00_JPRB, 0.21400E+00_JPRB, 0.21706E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.23934E+00_JPRB, 0.24078E+00_JPRB, 0.24236E+00_JPRB, 0.24417E+00_JPRB, 0.24541E+00_JPRB, 0.24583E+00_JPRB, 0.24445E+00_JPRB ,&
&  0.24188E+00_JPRB, 0.23907E+00_JPRB, 0.23631E+00_JPRB, 0.23397E+00_JPRB, 0.23265E+00_JPRB, 0.23199E+00_JPRB, 0.23171E+00_JPRB ,&
&  0.23155E+00_JPRB, 0.23136E+00_JPRB, 0.22974E+00_JPRB, 0.22723E+00_JPRB, 0.22391E+00_JPRB, 0.22112E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.21916E+00_JPRB, 0.21825E+00_JPRB, 0.21772E+00_JPRB, 0.21711E+00_JPRB, 0.21696E+00_JPRB, 0.21799E+00_JPRB, 0.21974E+00_JPRB ,&
&  0.22169E+00_JPRB, 0.22317E+00_JPRB, 0.22431E+00_JPRB, 0.22522E+00_JPRB, 0.22477E+00_JPRB, 0.22369E+00_JPRB, 0.22247E+00_JPRB ,&
&  0.22108E+00_JPRB, 0.21991E+00_JPRB, 0.21900E+00_JPRB, 0.21761E+00_JPRB, 0.21530E+00_JPRB, 0.21245E+00_JPRB, 0.21046E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.20736E+00_JPRB, 0.20570E+00_JPRB, 0.20483E+00_JPRB, 0.20425E+00_JPRB, 0.20402E+00_JPRB, 0.20411E+00_JPRB ,&
&  0.20452E+00_JPRB, 0.20568E+00_JPRB, 0.20726E+00_JPRB, 0.20893E+00_JPRB, 0.20927E+00_JPRB, 0.20929E+00_JPRB, 0.20941E+00_JPRB ,&
&  0.20871E+00_JPRB, 0.20742E+00_JPRB, 0.20573E+00_JPRB, 0.20395E+00_JPRB, 0.20288E+00_JPRB, 0.20205E+00_JPRB, 0.20112E+00_JPRB ,&
&  0.19940E+00_JPRB, 0.19835E+00_JPRB, 0.19771E+00_JPRB, 0.19756E+00_JPRB, 0.19776E+00_JPRB, 0.19783E+00_JPRB, 0.19718E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19564E+00_JPRB, 0.19458E+00_JPRB, 0.19344E+00_JPRB, 0.19170E+00_JPRB, 0.19193E+00_JPRB, 0.19276E+00_JPRB ,&
&  0.19322E+00_JPRB, 0.19348E+00_JPRB, 0.19368E+00_JPRB, 0.19466E+00_JPRB, 0.19570E+00_JPRB, 0.19718E+00_JPRB, 0.19947E+00_JPRB ,&
&  0.20134E+00_JPRB, 0.20378E+00_JPRB, 0.20747E+00_JPRB, 0.21085E+00_JPRB, 0.21327E+00_JPRB, 0.21438E+00_JPRB, 0.21660E+00_JPRB ,&
&  0.22181E+00_JPRB, 0.22766E+00_JPRB, 0.23157E+00_JPRB, 0.23526E+00_JPRB, 0.23828E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.25919E+00_JPRB, 0.26391E+00_JPRB, 0.26760E+00_JPRB, 0.26968E+00_JPRB, 0.26969E+00_JPRB, 0.26841E+00_JPRB, 0.26587E+00_JPRB ,&
&  0.26230E+00_JPRB, 0.25878E+00_JPRB, 0.25520E+00_JPRB, 0.25225E+00_JPRB, 0.25047E+00_JPRB, 0.24909E+00_JPRB, 0.24810E+00_JPRB ,&
&  0.24667E+00_JPRB, 0.24483E+00_JPRB, 0.24243E+00_JPRB, 0.24028E+00_JPRB, 0.23776E+00_JPRB, 0.23413E+00_JPRB, 0.23156E+00_JPRB ,&
&  0.22983E+00_JPRB, 0.22812E+00_JPRB, 0.22772E+00_JPRB, 0.22899E+00_JPRB, 0.23128E+00_JPRB, 0.23353E+00_JPRB, 0.23605E+00_JPRB ,&
&  0.23781E+00_JPRB, 0.23852E+00_JPRB, 0.23901E+00_JPRB, 0.23887E+00_JPRB, 0.23900E+00_JPRB, 0.23889E+00_JPRB, 0.23845E+00_JPRB ,&
&  0.23823E+00_JPRB, 0.23839E+00_JPRB, 0.23798E+00_JPRB, 0.23826E+00_JPRB, 0.23829E+00_JPRB, 0.23711E+00_JPRB, 0.23488E+00_JPRB ,&
&  0.23140E+00_JPRB, 0.22678E+00_JPRB, 0.22309E+00_JPRB, 0.22117E+00_JPRB, 0.22047E+00_JPRB, 0.22066E+00_JPRB, 0.22145E+00_JPRB ,&
&  0.22254E+00_JPRB, 0.22403E+00_JPRB, 0.22535E+00_JPRB, 0.22588E+00_JPRB, 0.22551E+00_JPRB, 0.22516E+00_JPRB, 0.22381E+00_JPRB ,&
&  0.22250E+00_JPRB, 0.22163E+00_JPRB, 0.22095E+00_JPRB, 0.22024E+00_JPRB, 0.21816E+00_JPRB, 0.21566E+00_JPRB, 0.21520E+00_JPRB ,&
&  0.21391E+00_JPRB, 0.21311E+00_JPRB, 0.21195E+00_JPRB, 0.21149E+00_JPRB, 0.21087E+00_JPRB, 0.20979E+00_JPRB, 0.20857E+00_JPRB ,&
&  0.20751E+00_JPRB, 0.20691E+00_JPRB, 0.20647E+00_JPRB, 0.20546E+00_JPRB, 0.20563E+00_JPRB, 0.20608E+00_JPRB, 0.20736E+00_JPRB ,&
&  0.20854E+00_JPRB, 0.20977E+00_JPRB, 0.21131E+00_JPRB, 0.21264E+00_JPRB, 0.21471E+00_JPRB, 0.21718E+00_JPRB, 0.22002E+00_JPRB ,&
&  0.22240E+00_JPRB, 0.22579E+00_JPRB, 0.22977E+00_JPRB, 0.23347E+00_JPRB, 0.23636E+00_JPRB, 0.23842E+00_JPRB, 0.24125E+00_JPRB ,&
&  0.24601E+00_JPRB, 0.25001E+00_JPRB, 0.25201E+00_JPRB, 0.25402E+00_JPRB, 0.25566E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.28758E+00_JPRB, 0.29229E+00_JPRB, 0.29485E+00_JPRB, 0.29588E+00_JPRB, 0.29600E+00_JPRB, 0.29437E+00_JPRB, 0.29187E+00_JPRB ,&
&  0.28822E+00_JPRB, 0.28469E+00_JPRB, 0.28074E+00_JPRB, 0.27615E+00_JPRB, 0.27204E+00_JPRB, 0.26868E+00_JPRB, 0.26589E+00_JPRB ,&
&  0.26314E+00_JPRB, 0.26065E+00_JPRB, 0.25860E+00_JPRB, 0.25640E+00_JPRB, 0.25389E+00_JPRB, 0.25076E+00_JPRB, 0.24653E+00_JPRB ,&
&  0.24413E+00_JPRB, 0.24231E+00_JPRB, 0.24339E+00_JPRB, 0.24611E+00_JPRB, 0.24817E+00_JPRB, 0.24952E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.25185E+00_JPRB, 0.25271E+00_JPRB, 0.25366E+00_JPRB, 0.25603E+00_JPRB, 0.25848E+00_JPRB, 0.25883E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25855E+00_JPRB, 0.26051E+00_JPRB, 0.26204E+00_JPRB, 0.26238E+00_JPRB, 0.26140E+00_JPRB, 0.25879E+00_JPRB, 0.25495E+00_JPRB ,&
&  0.24869E+00_JPRB, 0.24412E+00_JPRB, 0.24307E+00_JPRB, 0.24405E+00_JPRB, 0.24572E+00_JPRB, 0.24695E+00_JPRB, 0.24766E+00_JPRB ,&
&  0.24776E+00_JPRB, 0.24699E+00_JPRB, 0.24567E+00_JPRB, 0.24415E+00_JPRB, 0.24223E+00_JPRB, 0.24002E+00_JPRB, 0.23865E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.23584E+00_JPRB, 0.23486E+00_JPRB, 0.23421E+00_JPRB, 0.23304E+00_JPRB, 0.23151E+00_JPRB, 0.23067E+00_JPRB ,&
&  0.22952E+00_JPRB, 0.22892E+00_JPRB, 0.22802E+00_JPRB, 0.22713E+00_JPRB, 0.22594E+00_JPRB, 0.22480E+00_JPRB, 0.22456E+00_JPRB ,&
&  0.22471E+00_JPRB, 0.22359E+00_JPRB, 0.22216E+00_JPRB, 0.22188E+00_JPRB, 0.22220E+00_JPRB, 0.22285E+00_JPRB, 0.22343E+00_JPRB ,&
&  0.22406E+00_JPRB, 0.22488E+00_JPRB, 0.22663E+00_JPRB, 0.22855E+00_JPRB, 0.23068E+00_JPRB, 0.23332E+00_JPRB, 0.23640E+00_JPRB ,&
&  0.24026E+00_JPRB, 0.24498E+00_JPRB, 0.25038E+00_JPRB, 0.25527E+00_JPRB, 0.25937E+00_JPRB, 0.26223E+00_JPRB, 0.26633E+00_JPRB ,&
&  0.27137E+00_JPRB, 0.27497E+00_JPRB, 0.27763E+00_JPRB, 0.28028E+00_JPRB, 0.28316E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.31611E+00_JPRB, 0.31773E+00_JPRB, 0.31772E+00_JPRB, 0.31806E+00_JPRB, 0.31756E+00_JPRB, 0.31583E+00_JPRB, 0.31285E+00_JPRB ,&
&  0.30978E+00_JPRB, 0.30619E+00_JPRB, 0.30251E+00_JPRB, 0.29771E+00_JPRB, 0.29271E+00_JPRB, 0.28877E+00_JPRB, 0.28448E+00_JPRB ,&
&  0.28011E+00_JPRB, 0.27610E+00_JPRB, 0.27341E+00_JPRB, 0.27125E+00_JPRB, 0.26918E+00_JPRB, 0.26732E+00_JPRB, 0.26473E+00_JPRB ,&
&  0.26250E+00_JPRB, 0.26038E+00_JPRB, 0.26078E+00_JPRB, 0.26307E+00_JPRB, 0.26490E+00_JPRB, 0.26629E+00_JPRB, 0.26746E+00_JPRB ,&
&  0.26778E+00_JPRB, 0.27258E+00_JPRB, 0.26708E+00_JPRB, 0.27649E+00_JPRB, 0.28084E+00_JPRB, 0.28384E+00_JPRB, 0.28573E+00_JPRB ,&
&  0.28932E+00_JPRB, 0.29263E+00_JPRB, 0.29367E+00_JPRB, 0.28985E+00_JPRB, 0.28481E+00_JPRB, 0.28094E+00_JPRB, 0.27690E+00_JPRB ,&
&  0.27202E+00_JPRB, 0.27022E+00_JPRB, 0.27034E+00_JPRB, 0.27061E+00_JPRB, 0.27057E+00_JPRB, 0.26988E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26671E+00_JPRB, 0.26430E+00_JPRB, 0.26170E+00_JPRB, 0.25906E+00_JPRB, 0.25734E+00_JPRB, 0.25526E+00_JPRB, 0.25328E+00_JPRB ,&
&  0.25228E+00_JPRB, 0.25231E+00_JPRB, 0.25227E+00_JPRB, 0.25208E+00_JPRB, 0.25138E+00_JPRB, 0.25092E+00_JPRB, 0.25077E+00_JPRB ,&
&  0.25001E+00_JPRB, 0.24908E+00_JPRB, 0.24798E+00_JPRB, 0.24654E+00_JPRB, 0.24502E+00_JPRB, 0.24431E+00_JPRB, 0.24440E+00_JPRB ,&
&  0.24420E+00_JPRB, 0.24281E+00_JPRB, 0.24061E+00_JPRB, 0.23997E+00_JPRB, 0.23966E+00_JPRB, 0.23973E+00_JPRB, 0.24008E+00_JPRB ,&
&  0.24107E+00_JPRB, 0.24291E+00_JPRB, 0.24489E+00_JPRB, 0.24669E+00_JPRB, 0.24844E+00_JPRB, 0.25121E+00_JPRB, 0.25476E+00_JPRB ,&
&  0.25900E+00_JPRB, 0.26363E+00_JPRB, 0.26813E+00_JPRB, 0.27237E+00_JPRB, 0.27630E+00_JPRB, 0.28154E+00_JPRB, 0.28895E+00_JPRB ,&
&  0.29549E+00_JPRB, 0.30035E+00_JPRB, 0.30394E+00_JPRB, 0.30803E+00_JPRB, 0.31247E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.34926E+00_JPRB, 0.34753E+00_JPRB, 0.34436E+00_JPRB, 0.34231E+00_JPRB, 0.33972E+00_JPRB, 0.33644E+00_JPRB, 0.33298E+00_JPRB ,&
&  0.33027E+00_JPRB, 0.32755E+00_JPRB, 0.32469E+00_JPRB, 0.32061E+00_JPRB, 0.31508E+00_JPRB, 0.30983E+00_JPRB, 0.30370E+00_JPRB ,&
&  0.29800E+00_JPRB, 0.29309E+00_JPRB, 0.28986E+00_JPRB, 0.28817E+00_JPRB, 0.28670E+00_JPRB, 0.28557E+00_JPRB, 0.28420E+00_JPRB ,&
&  0.28358E+00_JPRB, 0.28354E+00_JPRB, 0.28348E+00_JPRB, 0.28377E+00_JPRB, 0.28445E+00_JPRB, 0.28696E+00_JPRB, 0.28855E+00_JPRB ,&
&  0.29035E+00_JPRB, 0.28677E+00_JPRB, 0.29469E+00_JPRB, 0.30572E+00_JPRB, 0.31816E+00_JPRB, 0.32504E+00_JPRB, 0.32944E+00_JPRB ,&
&  0.33454E+00_JPRB, 0.33665E+00_JPRB, 0.33195E+00_JPRB, 0.32512E+00_JPRB, 0.31956E+00_JPRB, 0.31577E+00_JPRB, 0.31020E+00_JPRB ,&
&  0.30497E+00_JPRB, 0.30124E+00_JPRB, 0.29822E+00_JPRB, 0.29606E+00_JPRB, 0.29376E+00_JPRB, 0.29033E+00_JPRB, 0.28647E+00_JPRB ,&
&  0.28187E+00_JPRB, 0.27858E+00_JPRB, 0.27596E+00_JPRB, 0.27493E+00_JPRB, 0.27506E+00_JPRB, 0.27606E+00_JPRB, 0.27753E+00_JPRB ,&
&  0.27979E+00_JPRB, 0.28176E+00_JPRB, 0.28256E+00_JPRB, 0.28271E+00_JPRB, 0.28244E+00_JPRB, 0.28225E+00_JPRB, 0.28149E+00_JPRB ,&
&  0.27998E+00_JPRB, 0.27816E+00_JPRB, 0.27628E+00_JPRB, 0.27421E+00_JPRB, 0.27202E+00_JPRB, 0.27048E+00_JPRB, 0.26911E+00_JPRB ,&
&  0.26755E+00_JPRB, 0.26559E+00_JPRB, 0.26340E+00_JPRB, 0.26214E+00_JPRB, 0.26121E+00_JPRB, 0.26066E+00_JPRB, 0.26131E+00_JPRB ,&
&  0.26314E+00_JPRB, 0.26530E+00_JPRB, 0.26754E+00_JPRB, 0.26940E+00_JPRB, 0.27116E+00_JPRB, 0.27287E+00_JPRB, 0.27546E+00_JPRB ,&
&  0.28005E+00_JPRB, 0.28506E+00_JPRB, 0.29013E+00_JPRB, 0.29522E+00_JPRB, 0.30214E+00_JPRB, 0.31204E+00_JPRB, 0.32299E+00_JPRB ,&
&  0.33060E+00_JPRB, 0.33499E+00_JPRB, 0.33905E+00_JPRB, 0.34338E+00_JPRB, 0.34779E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.37369E+00_JPRB, 0.37107E+00_JPRB, 0.36838E+00_JPRB, 0.36520E+00_JPRB, 0.36094E+00_JPRB, 0.35518E+00_JPRB, 0.35092E+00_JPRB ,&
&  0.34814E+00_JPRB, 0.34470E+00_JPRB, 0.34172E+00_JPRB, 0.33780E+00_JPRB, 0.33307E+00_JPRB, 0.32753E+00_JPRB, 0.32045E+00_JPRB ,&
&  0.31420E+00_JPRB, 0.30850E+00_JPRB, 0.30492E+00_JPRB, 0.30330E+00_JPRB, 0.30342E+00_JPRB, 0.30370E+00_JPRB, 0.30250E+00_JPRB ,&
&  0.30183E+00_JPRB, 0.30178E+00_JPRB, 0.30130E+00_JPRB, 0.30143E+00_JPRB, 0.30202E+00_JPRB, 0.30493E+00_JPRB, 0.30861E+00_JPRB ,&
&  0.31320E+00_JPRB, 0.29346E+00_JPRB, 0.30963E+00_JPRB, 0.32131E+00_JPRB, 0.33392E+00_JPRB, 0.34986E+00_JPRB, 0.37167E+00_JPRB ,&
&  0.38813E+00_JPRB, 0.38701E+00_JPRB, 0.37827E+00_JPRB, 0.36667E+00_JPRB, 0.35771E+00_JPRB, 0.34988E+00_JPRB, 0.34226E+00_JPRB ,&
&  0.33520E+00_JPRB, 0.33064E+00_JPRB, 0.32682E+00_JPRB, 0.32308E+00_JPRB, 0.31771E+00_JPRB, 0.31082E+00_JPRB, 0.30407E+00_JPRB ,&
&  0.29922E+00_JPRB, 0.29710E+00_JPRB, 0.29719E+00_JPRB, 0.29846E+00_JPRB, 0.30028E+00_JPRB, 0.30367E+00_JPRB, 0.30792E+00_JPRB ,&
&  0.31204E+00_JPRB, 0.31372E+00_JPRB, 0.31453E+00_JPRB, 0.31585E+00_JPRB, 0.31675E+00_JPRB, 0.31655E+00_JPRB, 0.31511E+00_JPRB ,&
&  0.31289E+00_JPRB, 0.30971E+00_JPRB, 0.30625E+00_JPRB, 0.30236E+00_JPRB, 0.29857E+00_JPRB, 0.29471E+00_JPRB, 0.29122E+00_JPRB ,&
&  0.28824E+00_JPRB, 0.28512E+00_JPRB, 0.28193E+00_JPRB, 0.27950E+00_JPRB, 0.27881E+00_JPRB, 0.27935E+00_JPRB, 0.28073E+00_JPRB ,&
&  0.28329E+00_JPRB, 0.28592E+00_JPRB, 0.28762E+00_JPRB, 0.28953E+00_JPRB, 0.29157E+00_JPRB, 0.29377E+00_JPRB, 0.29732E+00_JPRB ,&
&  0.30259E+00_JPRB, 0.30691E+00_JPRB, 0.31123E+00_JPRB, 0.31820E+00_JPRB, 0.32921E+00_JPRB, 0.34237E+00_JPRB, 0.35410E+00_JPRB ,&
&  0.36124E+00_JPRB, 0.36365E+00_JPRB, 0.36668E+00_JPRB, 0.36907E+00_JPRB, 0.37386E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.41427E+00_JPRB, 0.41275E+00_JPRB, 0.40727E+00_JPRB, 0.39661E+00_JPRB, 0.38520E+00_JPRB, 0.37571E+00_JPRB, 0.37030E+00_JPRB ,&
&  0.36700E+00_JPRB, 0.36175E+00_JPRB, 0.35723E+00_JPRB, 0.35401E+00_JPRB, 0.34976E+00_JPRB, 0.34515E+00_JPRB, 0.33898E+00_JPRB ,&
&  0.33423E+00_JPRB, 0.33067E+00_JPRB, 0.32801E+00_JPRB, 0.32636E+00_JPRB, 0.32530E+00_JPRB, 0.32415E+00_JPRB, 0.32231E+00_JPRB ,&
&  0.32017E+00_JPRB, 0.31836E+00_JPRB, 0.31695E+00_JPRB, 0.31728E+00_JPRB, 0.31823E+00_JPRB, 0.32179E+00_JPRB, 0.32790E+00_JPRB ,&
&  0.31546E+00_JPRB, 0.29507E+00_JPRB, 0.31701E+00_JPRB, 0.34257E+00_JPRB, 0.36218E+00_JPRB, 0.39088E+00_JPRB, 0.42599E+00_JPRB ,&
&  0.44128E+00_JPRB, 0.44527E+00_JPRB, 0.43350E+00_JPRB, 0.41968E+00_JPRB, 0.40516E+00_JPRB, 0.39128E+00_JPRB, 0.38033E+00_JPRB ,&
&  0.37028E+00_JPRB, 0.36344E+00_JPRB, 0.35756E+00_JPRB, 0.34935E+00_JPRB, 0.33933E+00_JPRB, 0.33107E+00_JPRB, 0.32615E+00_JPRB ,&
&  0.32421E+00_JPRB, 0.32444E+00_JPRB, 0.32503E+00_JPRB, 0.32607E+00_JPRB, 0.32826E+00_JPRB, 0.33297E+00_JPRB, 0.33885E+00_JPRB ,&
&  0.34369E+00_JPRB, 0.34912E+00_JPRB, 0.35572E+00_JPRB, 0.36183E+00_JPRB, 0.36720E+00_JPRB, 0.36904E+00_JPRB, 0.36834E+00_JPRB ,&
&  0.36409E+00_JPRB, 0.35685E+00_JPRB, 0.34909E+00_JPRB, 0.34181E+00_JPRB, 0.33462E+00_JPRB, 0.32703E+00_JPRB, 0.31949E+00_JPRB ,&
&  0.31321E+00_JPRB, 0.30757E+00_JPRB, 0.30321E+00_JPRB, 0.30156E+00_JPRB, 0.30169E+00_JPRB, 0.30375E+00_JPRB, 0.30620E+00_JPRB ,&
&  0.30842E+00_JPRB, 0.31070E+00_JPRB, 0.31233E+00_JPRB, 0.31563E+00_JPRB, 0.31946E+00_JPRB, 0.32425E+00_JPRB, 0.32953E+00_JPRB ,&
&  0.33562E+00_JPRB, 0.34145E+00_JPRB, 0.34826E+00_JPRB, 0.36268E+00_JPRB, 0.37810E+00_JPRB, 0.39184E+00_JPRB, 0.40082E+00_JPRB ,&
&  0.40304E+00_JPRB, 0.40383E+00_JPRB, 0.40671E+00_JPRB, 0.40820E+00_JPRB, 0.41467E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.46885E+00_JPRB, 0.46569E+00_JPRB, 0.45154E+00_JPRB, 0.43684E+00_JPRB, 0.42266E+00_JPRB, 0.40944E+00_JPRB, 0.40041E+00_JPRB ,&
&  0.39458E+00_JPRB, 0.38623E+00_JPRB, 0.37906E+00_JPRB, 0.37434E+00_JPRB, 0.36898E+00_JPRB, 0.36384E+00_JPRB, 0.35786E+00_JPRB ,&
&  0.35329E+00_JPRB, 0.35078E+00_JPRB, 0.34920E+00_JPRB, 0.34715E+00_JPRB, 0.34443E+00_JPRB, 0.34062E+00_JPRB, 0.33647E+00_JPRB ,&
&  0.33225E+00_JPRB, 0.32904E+00_JPRB, 0.32609E+00_JPRB, 0.32508E+00_JPRB, 0.32798E+00_JPRB, 0.33375E+00_JPRB, 0.33951E+00_JPRB ,&
&  0.35858E+00_JPRB, 0.31705E+00_JPRB, 0.34551E+00_JPRB, 0.38401E+00_JPRB, 0.41695E+00_JPRB, 0.45690E+00_JPRB, 0.48877E+00_JPRB ,&
&  0.49951E+00_JPRB, 0.50828E+00_JPRB, 0.49534E+00_JPRB, 0.47842E+00_JPRB, 0.45560E+00_JPRB, 0.43292E+00_JPRB, 0.41415E+00_JPRB ,&
&  0.40177E+00_JPRB, 0.39270E+00_JPRB, 0.38271E+00_JPRB, 0.37161E+00_JPRB, 0.36090E+00_JPRB, 0.35325E+00_JPRB, 0.35022E+00_JPRB ,&
&  0.35047E+00_JPRB, 0.35107E+00_JPRB, 0.35110E+00_JPRB, 0.35354E+00_JPRB, 0.35858E+00_JPRB, 0.36523E+00_JPRB, 0.37318E+00_JPRB ,&
&  0.38412E+00_JPRB, 0.39879E+00_JPRB, 0.41145E+00_JPRB, 0.42213E+00_JPRB, 0.42688E+00_JPRB, 0.42461E+00_JPRB, 0.41189E+00_JPRB ,&
&  0.39583E+00_JPRB, 0.38128E+00_JPRB, 0.37134E+00_JPRB, 0.36543E+00_JPRB, 0.36077E+00_JPRB, 0.35720E+00_JPRB, 0.35314E+00_JPRB ,&
&  0.34782E+00_JPRB, 0.34227E+00_JPRB, 0.33834E+00_JPRB, 0.33614E+00_JPRB, 0.33480E+00_JPRB, 0.33528E+00_JPRB, 0.33681E+00_JPRB ,&
&  0.33871E+00_JPRB, 0.34084E+00_JPRB, 0.34365E+00_JPRB, 0.34819E+00_JPRB, 0.35375E+00_JPRB, 0.35989E+00_JPRB, 0.36499E+00_JPRB ,&
&  0.37087E+00_JPRB, 0.37728E+00_JPRB, 0.38413E+00_JPRB, 0.40324E+00_JPRB, 0.42092E+00_JPRB, 0.43258E+00_JPRB, 0.44215E+00_JPRB ,&
&  0.44744E+00_JPRB, 0.45694E+00_JPRB, 0.39663E+00_JPRB, 0.43686E+00_JPRB, 0.46065E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.54329E+00_JPRB, 0.53165E+00_JPRB, 0.50787E+00_JPRB, 0.48253E+00_JPRB, 0.46054E+00_JPRB, 0.44072E+00_JPRB, 0.42573E+00_JPRB ,&
&  0.41666E+00_JPRB, 0.41048E+00_JPRB, 0.40424E+00_JPRB, 0.39844E+00_JPRB, 0.39168E+00_JPRB, 0.38570E+00_JPRB, 0.37849E+00_JPRB ,&
&  0.37366E+00_JPRB, 0.37080E+00_JPRB, 0.36841E+00_JPRB, 0.36489E+00_JPRB, 0.36077E+00_JPRB, 0.35616E+00_JPRB, 0.35031E+00_JPRB ,&
&  0.34528E+00_JPRB, 0.34260E+00_JPRB, 0.33850E+00_JPRB, 0.33757E+00_JPRB, 0.34364E+00_JPRB, 0.35098E+00_JPRB, 0.35963E+00_JPRB ,&
&  0.38347E+00_JPRB, 0.35311E+00_JPRB, 0.38464E+00_JPRB, 0.45864E+00_JPRB, 0.50531E+00_JPRB, 0.55482E+00_JPRB, 0.57079E+00_JPRB ,&
&  0.56918E+00_JPRB, 0.56608E+00_JPRB, 0.54730E+00_JPRB, 0.52457E+00_JPRB, 0.49298E+00_JPRB, 0.46632E+00_JPRB, 0.44612E+00_JPRB ,&
&  0.42904E+00_JPRB, 0.41459E+00_JPRB, 0.40298E+00_JPRB, 0.39315E+00_JPRB, 0.38407E+00_JPRB, 0.37974E+00_JPRB, 0.38024E+00_JPRB ,&
&  0.38207E+00_JPRB, 0.38453E+00_JPRB, 0.38833E+00_JPRB, 0.39488E+00_JPRB, 0.40126E+00_JPRB, 0.40861E+00_JPRB, 0.42681E+00_JPRB ,&
&  0.45596E+00_JPRB, 0.47682E+00_JPRB, 0.49180E+00_JPRB, 0.49503E+00_JPRB, 0.48738E+00_JPRB, 0.47216E+00_JPRB, 0.44960E+00_JPRB ,&
&  0.42658E+00_JPRB, 0.40880E+00_JPRB, 0.39694E+00_JPRB, 0.38930E+00_JPRB, 0.38454E+00_JPRB, 0.38026E+00_JPRB, 0.37683E+00_JPRB ,&
&  0.37349E+00_JPRB, 0.37103E+00_JPRB, 0.37120E+00_JPRB, 0.37412E+00_JPRB, 0.37820E+00_JPRB, 0.38186E+00_JPRB, 0.38395E+00_JPRB ,&
&  0.38500E+00_JPRB, 0.38590E+00_JPRB, 0.38900E+00_JPRB, 0.39349E+00_JPRB, 0.39840E+00_JPRB, 0.40166E+00_JPRB, 0.40592E+00_JPRB ,&
&  0.41163E+00_JPRB, 0.41858E+00_JPRB, 0.42960E+00_JPRB, 0.45531E+00_JPRB, 0.47898E+00_JPRB, 0.49117E+00_JPRB, 0.49993E+00_JPRB ,&
&  0.51147E+00_JPRB, 0.51621E+00_JPRB, 0.52392E+00_JPRB, 0.46914E+00_JPRB, 0.53412E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.62439E+00_JPRB, 0.58762E+00_JPRB, 0.53977E+00_JPRB, 0.50659E+00_JPRB, 0.48290E+00_JPRB, 0.45823E+00_JPRB, 0.44117E+00_JPRB ,&
&  0.43003E+00_JPRB, 0.42477E+00_JPRB, 0.42030E+00_JPRB, 0.41413E+00_JPRB, 0.40783E+00_JPRB, 0.40377E+00_JPRB, 0.39897E+00_JPRB ,&
&  0.39377E+00_JPRB, 0.38871E+00_JPRB, 0.38360E+00_JPRB, 0.37974E+00_JPRB, 0.37445E+00_JPRB, 0.37004E+00_JPRB, 0.36567E+00_JPRB ,&
&  0.36041E+00_JPRB, 0.35573E+00_JPRB, 0.35059E+00_JPRB, 0.35469E+00_JPRB, 0.36273E+00_JPRB, 0.36891E+00_JPRB, 0.37731E+00_JPRB ,&
&  0.40541E+00_JPRB, 0.40806E+00_JPRB, 0.48672E+00_JPRB, 0.57185E+00_JPRB, 0.60996E+00_JPRB, 0.63389E+00_JPRB, 0.62664E+00_JPRB ,&
&  0.60095E+00_JPRB, 0.60045E+00_JPRB, 0.58578E+00_JPRB, 0.55860E+00_JPRB, 0.53894E+00_JPRB, 0.50589E+00_JPRB, 0.47687E+00_JPRB ,&
&  0.45616E+00_JPRB, 0.44253E+00_JPRB, 0.43076E+00_JPRB, 0.41784E+00_JPRB, 0.40930E+00_JPRB, 0.40674E+00_JPRB, 0.40832E+00_JPRB ,&
&  0.41146E+00_JPRB, 0.41727E+00_JPRB, 0.42299E+00_JPRB, 0.42865E+00_JPRB, 0.43380E+00_JPRB, 0.45754E+00_JPRB, 0.50683E+00_JPRB ,&
&  0.54602E+00_JPRB, 0.56805E+00_JPRB, 0.57649E+00_JPRB, 0.57753E+00_JPRB, 0.56315E+00_JPRB, 0.54277E+00_JPRB, 0.51879E+00_JPRB ,&
&  0.49451E+00_JPRB, 0.46934E+00_JPRB, 0.44617E+00_JPRB, 0.42822E+00_JPRB, 0.41391E+00_JPRB, 0.40403E+00_JPRB, 0.39898E+00_JPRB ,&
&  0.39853E+00_JPRB, 0.40180E+00_JPRB, 0.40722E+00_JPRB, 0.41287E+00_JPRB, 0.41659E+00_JPRB, 0.41741E+00_JPRB, 0.41683E+00_JPRB ,&
&  0.41599E+00_JPRB, 0.41540E+00_JPRB, 0.41678E+00_JPRB, 0.42019E+00_JPRB, 0.42509E+00_JPRB, 0.43021E+00_JPRB, 0.43567E+00_JPRB ,&
&  0.44173E+00_JPRB, 0.44879E+00_JPRB, 0.45967E+00_JPRB, 0.49571E+00_JPRB, 0.51388E+00_JPRB, 0.53522E+00_JPRB, 0.55417E+00_JPRB ,&
&  0.57856E+00_JPRB, 0.59151E+00_JPRB, 0.58748E+00_JPRB, 0.57360E+00_JPRB, 0.60002E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.68956E+00_JPRB, 0.59760E+00_JPRB, 0.54106E+00_JPRB, 0.51311E+00_JPRB, 0.50300E+00_JPRB, 0.48653E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.45400E+00_JPRB, 0.44386E+00_JPRB, 0.43820E+00_JPRB, 0.43103E+00_JPRB, 0.42343E+00_JPRB, 0.41708E+00_JPRB, 0.41014E+00_JPRB ,&
&  0.40261E+00_JPRB, 0.39806E+00_JPRB, 0.39426E+00_JPRB, 0.39164E+00_JPRB, 0.38959E+00_JPRB, 0.38502E+00_JPRB, 0.37965E+00_JPRB ,&
&  0.37551E+00_JPRB, 0.37031E+00_JPRB, 0.36975E+00_JPRB, 0.37603E+00_JPRB, 0.38123E+00_JPRB, 0.38380E+00_JPRB, 0.39566E+00_JPRB ,&
&  0.43300E+00_JPRB, 0.45852E+00_JPRB, 0.56847E+00_JPRB, 0.66778E+00_JPRB, 0.71942E+00_JPRB, 0.75999E+00_JPRB, 0.75602E+00_JPRB ,&
&  0.73292E+00_JPRB, 0.72849E+00_JPRB, 0.70541E+00_JPRB, 0.65423E+00_JPRB, 0.60301E+00_JPRB, 0.56891E+00_JPRB, 0.54387E+00_JPRB ,&
&  0.52242E+00_JPRB, 0.49903E+00_JPRB, 0.47574E+00_JPRB, 0.45742E+00_JPRB, 0.44662E+00_JPRB, 0.44284E+00_JPRB, 0.44206E+00_JPRB ,&
&  0.44319E+00_JPRB, 0.44765E+00_JPRB, 0.45112E+00_JPRB, 0.45724E+00_JPRB, 0.48661E+00_JPRB, 0.56445E+00_JPRB, 0.67293E+00_JPRB ,&
&  0.72295E+00_JPRB, 0.73762E+00_JPRB, 0.74388E+00_JPRB, 0.73907E+00_JPRB, 0.70181E+00_JPRB, 0.66200E+00_JPRB, 0.62305E+00_JPRB ,&
&  0.58614E+00_JPRB, 0.54740E+00_JPRB, 0.51498E+00_JPRB, 0.48912E+00_JPRB, 0.46847E+00_JPRB, 0.45558E+00_JPRB, 0.44996E+00_JPRB ,&
&  0.44824E+00_JPRB, 0.44773E+00_JPRB, 0.44729E+00_JPRB, 0.44501E+00_JPRB, 0.44064E+00_JPRB, 0.43873E+00_JPRB, 0.43770E+00_JPRB ,&
&  0.43717E+00_JPRB, 0.43808E+00_JPRB, 0.44151E+00_JPRB, 0.44664E+00_JPRB, 0.45448E+00_JPRB, 0.45965E+00_JPRB, 0.46664E+00_JPRB ,&
&  0.47540E+00_JPRB, 0.48956E+00_JPRB, 0.49060E+00_JPRB, 0.52335E+00_JPRB, 0.58435E+00_JPRB, 0.60752E+00_JPRB, 0.63541E+00_JPRB ,&
&  0.66445E+00_JPRB, 0.66138E+00_JPRB, 0.63527E+00_JPRB, 0.63495E+00_JPRB, 0.75887E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.56301E+00_JPRB, 0.53667E+00_JPRB, 0.52078E+00_JPRB, 0.51544E+00_JPRB, 0.51371E+00_JPRB, 0.50374E+00_JPRB, 0.48539E+00_JPRB ,&
&  0.46675E+00_JPRB, 0.45117E+00_JPRB, 0.44005E+00_JPRB, 0.43187E+00_JPRB, 0.42629E+00_JPRB, 0.42073E+00_JPRB, 0.41467E+00_JPRB ,&
&  0.40845E+00_JPRB, 0.40564E+00_JPRB, 0.40373E+00_JPRB, 0.40051E+00_JPRB, 0.39282E+00_JPRB, 0.38654E+00_JPRB, 0.38447E+00_JPRB ,&
&  0.38325E+00_JPRB, 0.38411E+00_JPRB, 0.38930E+00_JPRB, 0.39523E+00_JPRB, 0.39753E+00_JPRB, 0.39934E+00_JPRB, 0.41022E+00_JPRB ,&
&  0.44685E+00_JPRB, 0.52490E+00_JPRB, 0.61368E+00_JPRB, 0.74799E+00_JPRB, 0.90888E+00_JPRB, 0.97708E+00_JPRB, 0.10132E+01_JPRB ,&
&  0.98928E+00_JPRB, 0.92089E+00_JPRB, 0.82899E+00_JPRB, 0.74167E+00_JPRB, 0.67332E+00_JPRB, 0.63325E+00_JPRB, 0.60346E+00_JPRB ,&
&  0.57481E+00_JPRB, 0.54632E+00_JPRB, 0.52208E+00_JPRB, 0.50400E+00_JPRB, 0.49145E+00_JPRB, 0.48671E+00_JPRB, 0.48624E+00_JPRB ,&
&  0.48578E+00_JPRB, 0.48422E+00_JPRB, 0.48524E+00_JPRB, 0.51103E+00_JPRB, 0.63123E+00_JPRB, 0.80075E+00_JPRB, 0.94969E+00_JPRB ,&
&  0.10033E+01_JPRB, 0.10393E+01_JPRB, 0.10258E+01_JPRB, 0.96977E+00_JPRB, 0.88390E+00_JPRB, 0.79922E+00_JPRB, 0.71120E+00_JPRB ,&
&  0.64760E+00_JPRB, 0.60123E+00_JPRB, 0.56797E+00_JPRB, 0.54028E+00_JPRB, 0.51540E+00_JPRB, 0.49680E+00_JPRB, 0.48472E+00_JPRB ,&
&  0.47635E+00_JPRB, 0.46922E+00_JPRB, 0.46227E+00_JPRB, 0.45557E+00_JPRB, 0.44996E+00_JPRB, 0.44623E+00_JPRB, 0.44782E+00_JPRB ,&
&  0.44961E+00_JPRB, 0.45045E+00_JPRB, 0.46146E+00_JPRB, 0.47654E+00_JPRB, 0.49156E+00_JPRB, 0.50110E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.51421E+00_JPRB, 0.53181E+00_JPRB, 0.52716E+00_JPRB, 0.54091E+00_JPRB, 0.60869E+00_JPRB, 0.66902E+00_JPRB, 0.68121E+00_JPRB ,&
&  0.67232E+00_JPRB, 0.64722E+00_JPRB, 0.62535E+00_JPRB, 0.61293E+00_JPRB, 0.60426E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.56465E+00_JPRB, 0.54476E+00_JPRB, 0.53509E+00_JPRB, 0.52644E+00_JPRB, 0.51815E+00_JPRB, 0.50552E+00_JPRB, 0.48753E+00_JPRB ,&
&  0.47420E+00_JPRB, 0.46364E+00_JPRB, 0.45134E+00_JPRB, 0.44069E+00_JPRB, 0.43334E+00_JPRB, 0.42757E+00_JPRB, 0.42082E+00_JPRB ,&
&  0.41642E+00_JPRB, 0.41365E+00_JPRB, 0.40445E+00_JPRB, 0.39538E+00_JPRB, 0.39163E+00_JPRB, 0.39116E+00_JPRB, 0.39476E+00_JPRB ,&
&  0.39869E+00_JPRB, 0.40375E+00_JPRB, 0.40764E+00_JPRB, 0.41078E+00_JPRB, 0.41076E+00_JPRB, 0.41376E+00_JPRB, 0.42943E+00_JPRB ,&
&  0.47679E+00_JPRB, 0.56373E+00_JPRB, 0.77876E+00_JPRB, 0.88260E+00_JPRB, 0.98125E+00_JPRB, 0.10604E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.11320E+01_JPRB, 0.10201E+01_JPRB, 0.89415E+00_JPRB, 0.81116E+00_JPRB, 0.74961E+00_JPRB, 0.70480E+00_JPRB, 0.65765E+00_JPRB ,&
&  0.61843E+00_JPRB, 0.59194E+00_JPRB, 0.56587E+00_JPRB, 0.54249E+00_JPRB, 0.53271E+00_JPRB, 0.52987E+00_JPRB, 0.52662E+00_JPRB ,&
&  0.52196E+00_JPRB, 0.51803E+00_JPRB, 0.53254E+00_JPRB, 0.61004E+00_JPRB, 0.79763E+00_JPRB, 0.96148E+00_JPRB, 0.11891E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.14203E+01_JPRB, 0.13219E+01_JPRB, 0.11521E+01_JPRB, 0.97166E+00_JPRB, 0.81786E+00_JPRB, 0.72286E+00_JPRB ,&
&  0.66270E+00_JPRB, 0.61153E+00_JPRB, 0.57967E+00_JPRB, 0.55924E+00_JPRB, 0.54363E+00_JPRB, 0.53039E+00_JPRB, 0.51418E+00_JPRB ,&
&  0.49625E+00_JPRB, 0.48248E+00_JPRB, 0.47356E+00_JPRB, 0.46625E+00_JPRB, 0.46187E+00_JPRB, 0.46051E+00_JPRB, 0.46319E+00_JPRB ,&
&  0.46961E+00_JPRB, 0.48382E+00_JPRB, 0.50850E+00_JPRB, 0.51995E+00_JPRB, 0.54025E+00_JPRB, 0.56554E+00_JPRB, 0.58176E+00_JPRB ,&
&  0.56806E+00_JPRB, 0.56062E+00_JPRB, 0.54731E+00_JPRB, 0.55135E+00_JPRB, 0.63495E+00_JPRB, 0.70667E+00_JPRB, 0.68445E+00_JPRB ,&
&  0.66564E+00_JPRB, 0.64361E+00_JPRB, 0.62020E+00_JPRB, 0.59566E+00_JPRB, 0.59338E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.57624E+00_JPRB, 0.55671E+00_JPRB, 0.54141E+00_JPRB, 0.52473E+00_JPRB, 0.51268E+00_JPRB, 0.50421E+00_JPRB, 0.49280E+00_JPRB ,&
&  0.48185E+00_JPRB, 0.47152E+00_JPRB, 0.46067E+00_JPRB, 0.45003E+00_JPRB, 0.44109E+00_JPRB, 0.43036E+00_JPRB, 0.42180E+00_JPRB ,&
&  0.41370E+00_JPRB, 0.40316E+00_JPRB, 0.39622E+00_JPRB, 0.39437E+00_JPRB, 0.39822E+00_JPRB, 0.40340E+00_JPRB, 0.40996E+00_JPRB ,&
&  0.41781E+00_JPRB, 0.42304E+00_JPRB, 0.42423E+00_JPRB, 0.42648E+00_JPRB, 0.42463E+00_JPRB, 0.42666E+00_JPRB, 0.44529E+00_JPRB ,&
&  0.49420E+00_JPRB, 0.47228E+00_JPRB, 0.75417E+00_JPRB, 0.10038E+01_JPRB, 0.11343E+01_JPRB, 0.12130E+01_JPRB, 0.12450E+01_JPRB ,&
&  0.12577E+01_JPRB, 0.11246E+01_JPRB, 0.98457E+00_JPRB, 0.89505E+00_JPRB, 0.82442E+00_JPRB, 0.76949E+00_JPRB, 0.72486E+00_JPRB ,&
&  0.68115E+00_JPRB, 0.64788E+00_JPRB, 0.62144E+00_JPRB, 0.58634E+00_JPRB, 0.55960E+00_JPRB, 0.54561E+00_JPRB, 0.53963E+00_JPRB ,&
&  0.53311E+00_JPRB, 0.54090E+00_JPRB, 0.58308E+00_JPRB, 0.70528E+00_JPRB, 0.96051E+00_JPRB, 0.11785E+01_JPRB, 0.12843E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.15056E+01_JPRB, 0.14010E+01_JPRB, 0.11988E+01_JPRB, 0.99920E+00_JPRB, 0.85306E+00_JPRB, 0.73713E+00_JPRB ,&
&  0.67019E+00_JPRB, 0.63204E+00_JPRB, 0.61013E+00_JPRB, 0.59323E+00_JPRB, 0.57447E+00_JPRB, 0.55084E+00_JPRB, 0.52457E+00_JPRB ,&
&  0.50488E+00_JPRB, 0.49454E+00_JPRB, 0.48789E+00_JPRB, 0.48562E+00_JPRB, 0.48489E+00_JPRB, 0.48565E+00_JPRB, 0.49127E+00_JPRB ,&
&  0.50338E+00_JPRB, 0.52583E+00_JPRB, 0.54209E+00_JPRB, 0.55450E+00_JPRB, 0.57171E+00_JPRB, 0.58997E+00_JPRB, 0.60690E+00_JPRB ,&
&  0.61163E+00_JPRB, 0.59843E+00_JPRB, 0.57979E+00_JPRB, 0.58807E+00_JPRB, 0.62068E+00_JPRB, 0.68004E+00_JPRB, 0.67581E+00_JPRB ,&
&  0.66841E+00_JPRB, 0.64873E+00_JPRB, 0.61761E+00_JPRB, 0.60350E+00_JPRB, 0.59839E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.58273E+00_JPRB, 0.56867E+00_JPRB, 0.55107E+00_JPRB, 0.53400E+00_JPRB, 0.52688E+00_JPRB, 0.52164E+00_JPRB, 0.50887E+00_JPRB ,&
&  0.49792E+00_JPRB, 0.48990E+00_JPRB, 0.47987E+00_JPRB, 0.46687E+00_JPRB, 0.45165E+00_JPRB, 0.43200E+00_JPRB, 0.41388E+00_JPRB ,&
&  0.40530E+00_JPRB, 0.39833E+00_JPRB, 0.40137E+00_JPRB, 0.40678E+00_JPRB, 0.41521E+00_JPRB, 0.42202E+00_JPRB, 0.42972E+00_JPRB ,&
&  0.43533E+00_JPRB, 0.43783E+00_JPRB, 0.44160E+00_JPRB, 0.44465E+00_JPRB, 0.44536E+00_JPRB, 0.45043E+00_JPRB, 0.47193E+00_JPRB ,&
&  0.52741E+00_JPRB, 0.75272E+00_JPRB, 0.66184E+00_JPRB, 0.12289E+01_JPRB, 0.13951E+01_JPRB, 0.14531E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.13115E+01_JPRB, 0.12398E+01_JPRB, 0.10888E+01_JPRB, 0.96774E+00_JPRB, 0.87500E+00_JPRB, 0.81366E+00_JPRB, 0.77055E+00_JPRB ,&
&  0.74015E+00_JPRB, 0.70363E+00_JPRB, 0.65938E+00_JPRB, 0.61525E+00_JPRB, 0.58652E+00_JPRB, 0.57159E+00_JPRB, 0.56648E+00_JPRB ,&
&  0.56402E+00_JPRB, 0.58738E+00_JPRB, 0.66892E+00_JPRB, 0.89884E+00_JPRB, 0.11692E+01_JPRB, 0.13245E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.13710E+01_JPRB, 0.12889E+01_JPRB, 0.10892E+01_JPRB, 0.93536E+00_JPRB, 0.81050E+00_JPRB, 0.72056E+00_JPRB ,&
&  0.66587E+00_JPRB, 0.63249E+00_JPRB, 0.61183E+00_JPRB, 0.59576E+00_JPRB, 0.57773E+00_JPRB, 0.55794E+00_JPRB, 0.54108E+00_JPRB ,&
&  0.52916E+00_JPRB, 0.51997E+00_JPRB, 0.51381E+00_JPRB, 0.51392E+00_JPRB, 0.51300E+00_JPRB, 0.51503E+00_JPRB, 0.52131E+00_JPRB ,&
&  0.53359E+00_JPRB, 0.55876E+00_JPRB, 0.56737E+00_JPRB, 0.58144E+00_JPRB, 0.58689E+00_JPRB, 0.58991E+00_JPRB, 0.59436E+00_JPRB ,&
&  0.62372E+00_JPRB, 0.62455E+00_JPRB, 0.61545E+00_JPRB, 0.61821E+00_JPRB, 0.64051E+00_JPRB, 0.68234E+00_JPRB, 0.67298E+00_JPRB ,&
&  0.66881E+00_JPRB, 0.65076E+00_JPRB, 0.62863E+00_JPRB, 0.61156E+00_JPRB, 0.59990E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.58891E+00_JPRB, 0.57283E+00_JPRB, 0.56018E+00_JPRB, 0.55498E+00_JPRB, 0.54843E+00_JPRB, 0.54187E+00_JPRB, 0.52821E+00_JPRB ,&
&  0.51342E+00_JPRB, 0.50450E+00_JPRB, 0.49204E+00_JPRB, 0.48045E+00_JPRB, 0.45716E+00_JPRB, 0.42809E+00_JPRB, 0.41672E+00_JPRB ,&
&  0.41024E+00_JPRB, 0.41168E+00_JPRB, 0.42127E+00_JPRB, 0.42523E+00_JPRB, 0.42973E+00_JPRB, 0.43878E+00_JPRB, 0.44769E+00_JPRB ,&
&  0.45418E+00_JPRB, 0.45452E+00_JPRB, 0.45924E+00_JPRB, 0.46550E+00_JPRB, 0.47120E+00_JPRB, 0.49059E+00_JPRB, 0.51551E+00_JPRB ,&
&  0.58967E+00_JPRB, 0.76074E+00_JPRB, 0.67843E+00_JPRB, 0.14229E+01_JPRB, 0.16477E+01_JPRB, 0.16393E+01_JPRB, 0.14977E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.14020E+01_JPRB, 0.11784E+01_JPRB, 0.10172E+01_JPRB, 0.90001E+00_JPRB, 0.83721E+00_JPRB, 0.79898E+00_JPRB ,&
&  0.76961E+00_JPRB, 0.73628E+00_JPRB, 0.69045E+00_JPRB, 0.64871E+00_JPRB, 0.62554E+00_JPRB, 0.61180E+00_JPRB, 0.60632E+00_JPRB ,&
&  0.60744E+00_JPRB, 0.65776E+00_JPRB, 0.80158E+00_JPRB, 0.10906E+01_JPRB, 0.13604E+01_JPRB, 0.13875E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.11821E+01_JPRB, 0.12549E+01_JPRB, 0.11518E+01_JPRB, 0.96607E+00_JPRB, 0.83639E+00_JPRB, 0.76159E+00_JPRB, 0.68838E+00_JPRB ,&
&  0.64251E+00_JPRB, 0.61215E+00_JPRB, 0.59740E+00_JPRB, 0.58625E+00_JPRB, 0.57653E+00_JPRB, 0.56994E+00_JPRB, 0.56283E+00_JPRB ,&
&  0.55613E+00_JPRB, 0.54637E+00_JPRB, 0.53958E+00_JPRB, 0.53137E+00_JPRB, 0.53321E+00_JPRB, 0.53722E+00_JPRB, 0.54336E+00_JPRB ,&
&  0.56395E+00_JPRB, 0.59932E+00_JPRB, 0.59967E+00_JPRB, 0.61258E+00_JPRB, 0.61052E+00_JPRB, 0.59895E+00_JPRB, 0.60591E+00_JPRB ,&
&  0.62860E+00_JPRB, 0.63362E+00_JPRB, 0.64350E+00_JPRB, 0.63548E+00_JPRB, 0.64946E+00_JPRB, 0.67451E+00_JPRB, 0.67275E+00_JPRB ,&
&  0.67597E+00_JPRB, 0.66426E+00_JPRB, 0.64699E+00_JPRB, 0.62171E+00_JPRB, 0.60530E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.58221E+00_JPRB, 0.56470E+00_JPRB, 0.55906E+00_JPRB, 0.55800E+00_JPRB, 0.55382E+00_JPRB, 0.54423E+00_JPRB, 0.52768E+00_JPRB ,&
&  0.50778E+00_JPRB, 0.49381E+00_JPRB, 0.47867E+00_JPRB, 0.46018E+00_JPRB, 0.43267E+00_JPRB, 0.40677E+00_JPRB, 0.40636E+00_JPRB ,&
&  0.42203E+00_JPRB, 0.43136E+00_JPRB, 0.43966E+00_JPRB, 0.44842E+00_JPRB, 0.45816E+00_JPRB, 0.47023E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.48201E+00_JPRB, 0.48685E+00_JPRB, 0.49725E+00_JPRB, 0.51013E+00_JPRB, 0.52714E+00_JPRB, 0.55338E+00_JPRB, 0.58987E+00_JPRB ,&
&  0.68658E+00_JPRB, 0.81080E+00_JPRB, 0.82364E+00_JPRB, 0.14342E+01_JPRB, 0.17338E+01_JPRB, 0.16225E+01_JPRB, 0.15650E+01_JPRB ,&
&  0.14512E+01_JPRB, 0.12415E+01_JPRB, 0.11756E+01_JPRB, 0.10054E+01_JPRB, 0.90020E+00_JPRB, 0.84727E+00_JPRB, 0.81511E+00_JPRB ,&
&  0.79135E+00_JPRB, 0.76177E+00_JPRB, 0.72160E+00_JPRB, 0.69940E+00_JPRB, 0.68318E+00_JPRB, 0.66600E+00_JPRB, 0.65892E+00_JPRB ,&
&  0.67902E+00_JPRB, 0.76998E+00_JPRB, 0.99119E+00_JPRB, 0.13355E+01_JPRB, 0.14515E+01_JPRB, 0.13381E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10734E+01_JPRB, 0.98483E+00_JPRB, 0.85025E+00_JPRB, 0.73620E+00_JPRB, 0.72839E+00_JPRB, 0.66094E+00_JPRB ,&
&  0.62270E+00_JPRB, 0.59798E+00_JPRB, 0.58676E+00_JPRB, 0.57880E+00_JPRB, 0.57570E+00_JPRB, 0.57295E+00_JPRB, 0.56827E+00_JPRB ,&
&  0.56555E+00_JPRB, 0.56520E+00_JPRB, 0.55848E+00_JPRB, 0.55800E+00_JPRB, 0.56101E+00_JPRB, 0.56745E+00_JPRB, 0.58216E+00_JPRB ,&
&  0.61543E+00_JPRB, 0.65076E+00_JPRB, 0.65484E+00_JPRB, 0.66224E+00_JPRB, 0.65108E+00_JPRB, 0.63764E+00_JPRB, 0.63615E+00_JPRB ,&
&  0.63912E+00_JPRB, 0.66327E+00_JPRB, 0.67694E+00_JPRB, 0.67275E+00_JPRB, 0.67627E+00_JPRB, 0.68389E+00_JPRB, 0.69113E+00_JPRB ,&
&  0.70184E+00_JPRB, 0.69391E+00_JPRB, 0.68562E+00_JPRB, 0.62846E+00_JPRB, 0.60722E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.55725E+00_JPRB, 0.54309E+00_JPRB, 0.52881E+00_JPRB, 0.52456E+00_JPRB, 0.51976E+00_JPRB, 0.50493E+00_JPRB, 0.48601E+00_JPRB ,&
&  0.46452E+00_JPRB, 0.44857E+00_JPRB, 0.42818E+00_JPRB, 0.41120E+00_JPRB, 0.41053E+00_JPRB, 0.42276E+00_JPRB, 0.44718E+00_JPRB ,&
&  0.47164E+00_JPRB, 0.48583E+00_JPRB, 0.49671E+00_JPRB, 0.50753E+00_JPRB, 0.51762E+00_JPRB, 0.52299E+00_JPRB, 0.52315E+00_JPRB ,&
&  0.52923E+00_JPRB, 0.54346E+00_JPRB, 0.56428E+00_JPRB, 0.58772E+00_JPRB, 0.61657E+00_JPRB, 0.65816E+00_JPRB, 0.72691E+00_JPRB ,&
&  0.86263E+00_JPRB, 0.92783E+00_JPRB, 0.53690E+00_JPRB, 0.10432E+01_JPRB, 0.14910E+01_JPRB, 0.14648E+01_JPRB, 0.13697E+01_JPRB ,&
&  0.12451E+01_JPRB, 0.10552E+01_JPRB, 0.10179E+01_JPRB, 0.96983E+00_JPRB, 0.88374E+00_JPRB, 0.84696E+00_JPRB, 0.83075E+00_JPRB ,&
&  0.82274E+00_JPRB, 0.80049E+00_JPRB, 0.78354E+00_JPRB, 0.77757E+00_JPRB, 0.75771E+00_JPRB, 0.73146E+00_JPRB, 0.73238E+00_JPRB ,&
&  0.78311E+00_JPRB, 0.92836E+00_JPRB, 0.12474E+01_JPRB, 0.14333E+01_JPRB, 0.14991E+01_JPRB, 0.12394E+01_JPRB, 0.98431E+00_JPRB ,&
&  0.87607E+00_JPRB, 0.90710E+00_JPRB, 0.86341E+00_JPRB, 0.77466E+00_JPRB, 0.68782E+00_JPRB, 0.64732E+00_JPRB, 0.64626E+00_JPRB ,&
&  0.61107E+00_JPRB, 0.58543E+00_JPRB, 0.57057E+00_JPRB, 0.56032E+00_JPRB, 0.55503E+00_JPRB, 0.55320E+00_JPRB, 0.55535E+00_JPRB ,&
&  0.56418E+00_JPRB, 0.56845E+00_JPRB, 0.57506E+00_JPRB, 0.58564E+00_JPRB, 0.59661E+00_JPRB, 0.61153E+00_JPRB, 0.63920E+00_JPRB ,&
&  0.67860E+00_JPRB, 0.70852E+00_JPRB, 0.73173E+00_JPRB, 0.72048E+00_JPRB, 0.69864E+00_JPRB, 0.68789E+00_JPRB, 0.69787E+00_JPRB ,&
&  0.70083E+00_JPRB, 0.71789E+00_JPRB, 0.68771E+00_JPRB, 0.67605E+00_JPRB, 0.70554E+00_JPRB, 0.71194E+00_JPRB, 0.75079E+00_JPRB ,&
&  0.80109E+00_JPRB, 0.87730E+00_JPRB, 0.89149E+00_JPRB, 0.62552E+00_JPRB, 0.58538E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.51877E+00_JPRB, 0.49537E+00_JPRB, 0.46924E+00_JPRB, 0.45936E+00_JPRB, 0.46389E+00_JPRB, 0.45766E+00_JPRB, 0.45191E+00_JPRB ,&
&  0.44636E+00_JPRB, 0.44213E+00_JPRB, 0.43780E+00_JPRB, 0.44771E+00_JPRB, 0.47417E+00_JPRB, 0.51040E+00_JPRB, 0.54281E+00_JPRB ,&
&  0.56582E+00_JPRB, 0.57655E+00_JPRB, 0.57993E+00_JPRB, 0.58535E+00_JPRB, 0.58916E+00_JPRB, 0.59286E+00_JPRB, 0.59849E+00_JPRB ,&
&  0.61872E+00_JPRB, 0.64993E+00_JPRB, 0.68942E+00_JPRB, 0.73179E+00_JPRB, 0.78159E+00_JPRB, 0.85228E+00_JPRB, 0.96385E+00_JPRB ,&
&  0.11327E+01_JPRB, 0.85284E+00_JPRB, 0.63669E+00_JPRB, 0.13231E+01_JPRB, 0.14887E+01_JPRB, 0.14426E+01_JPRB, 0.12261E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.10074E+01_JPRB, 0.93880E+00_JPRB, 0.96045E+00_JPRB, 0.87857E+00_JPRB, 0.85485E+00_JPRB, 0.85649E+00_JPRB ,&
&  0.85850E+00_JPRB, 0.86121E+00_JPRB, 0.86270E+00_JPRB, 0.85069E+00_JPRB, 0.82595E+00_JPRB, 0.81583E+00_JPRB, 0.84500E+00_JPRB ,&
&  0.95236E+00_JPRB, 0.11733E+01_JPRB, 0.15436E+01_JPRB, 0.15326E+01_JPRB, 0.15562E+01_JPRB, 0.11775E+01_JPRB, 0.94105E+00_JPRB ,&
&  0.81090E+00_JPRB, 0.80714E+00_JPRB, 0.79111E+00_JPRB, 0.74447E+00_JPRB, 0.67759E+00_JPRB, 0.62171E+00_JPRB, 0.63015E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.56579E+00_JPRB, 0.54492E+00_JPRB, 0.53537E+00_JPRB, 0.53736E+00_JPRB, 0.54360E+00_JPRB, 0.54692E+00_JPRB ,&
&  0.55779E+00_JPRB, 0.56557E+00_JPRB, 0.58439E+00_JPRB, 0.60547E+00_JPRB, 0.62752E+00_JPRB, 0.65489E+00_JPRB, 0.69516E+00_JPRB ,&
&  0.73068E+00_JPRB, 0.75566E+00_JPRB, 0.77149E+00_JPRB, 0.77620E+00_JPRB, 0.75677E+00_JPRB, 0.73145E+00_JPRB, 0.73726E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.73199E+00_JPRB, 0.71824E+00_JPRB, 0.73120E+00_JPRB, 0.74279E+00_JPRB, 0.75679E+00_JPRB, 0.79181E+00_JPRB ,&
&  0.83410E+00_JPRB, 0.83907E+00_JPRB, 0.76210E+00_JPRB, 0.58015E+00_JPRB, 0.53859E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.49476E+00_JPRB, 0.47008E+00_JPRB, 0.45985E+00_JPRB, 0.45899E+00_JPRB, 0.46568E+00_JPRB, 0.46945E+00_JPRB, 0.46895E+00_JPRB ,&
&  0.47137E+00_JPRB, 0.48255E+00_JPRB, 0.49654E+00_JPRB, 0.53788E+00_JPRB, 0.57971E+00_JPRB, 0.60976E+00_JPRB, 0.62837E+00_JPRB ,&
&  0.64558E+00_JPRB, 0.65511E+00_JPRB, 0.65960E+00_JPRB, 0.66585E+00_JPRB, 0.67103E+00_JPRB, 0.68446E+00_JPRB, 0.71116E+00_JPRB ,&
&  0.75265E+00_JPRB, 0.79674E+00_JPRB, 0.85105E+00_JPRB, 0.91749E+00_JPRB, 0.99903E+00_JPRB, 0.10755E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.89721E+00_JPRB, 0.54426E+00_JPRB, 0.13061E+01_JPRB, 0.15464E+01_JPRB, 0.13619E+01_JPRB, 0.12709E+01_JPRB, 0.11560E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.96284E+00_JPRB, 0.92461E+00_JPRB, 0.95061E+00_JPRB, 0.88592E+00_JPRB, 0.87409E+00_JPRB, 0.87650E+00_JPRB ,&
&  0.88899E+00_JPRB, 0.91061E+00_JPRB, 0.91348E+00_JPRB, 0.90356E+00_JPRB, 0.90272E+00_JPRB, 0.93124E+00_JPRB, 0.10241E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.14847E+01_JPRB, 0.17435E+01_JPRB, 0.15430E+01_JPRB, 0.15683E+01_JPRB, 0.12836E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.84007E+00_JPRB, 0.74045E+00_JPRB, 0.72646E+00_JPRB, 0.73917E+00_JPRB, 0.68992E+00_JPRB, 0.62080E+00_JPRB, 0.61788E+00_JPRB ,&
&  0.57447E+00_JPRB, 0.54416E+00_JPRB, 0.53097E+00_JPRB, 0.53142E+00_JPRB, 0.53911E+00_JPRB, 0.54978E+00_JPRB, 0.56061E+00_JPRB ,&
&  0.56519E+00_JPRB, 0.57774E+00_JPRB, 0.59295E+00_JPRB, 0.62036E+00_JPRB, 0.65116E+00_JPRB, 0.69681E+00_JPRB, 0.73768E+00_JPRB ,&
&  0.76490E+00_JPRB, 0.78943E+00_JPRB, 0.81347E+00_JPRB, 0.83222E+00_JPRB, 0.84829E+00_JPRB, 0.81956E+00_JPRB, 0.79117E+00_JPRB ,&
&  0.78503E+00_JPRB, 0.78556E+00_JPRB, 0.80021E+00_JPRB, 0.82113E+00_JPRB, 0.79021E+00_JPRB, 0.77596E+00_JPRB, 0.80040E+00_JPRB ,&
&  0.80076E+00_JPRB, 0.76133E+00_JPRB, 0.66116E+00_JPRB, 0.55454E+00_JPRB, 0.51739E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.53587E+00_JPRB, 0.52307E+00_JPRB, 0.50914E+00_JPRB, 0.51162E+00_JPRB, 0.51471E+00_JPRB, 0.53228E+00_JPRB, 0.53304E+00_JPRB ,&
&  0.54647E+00_JPRB, 0.58639E+00_JPRB, 0.63406E+00_JPRB, 0.65827E+00_JPRB, 0.67925E+00_JPRB, 0.70009E+00_JPRB, 0.71621E+00_JPRB ,&
&  0.72864E+00_JPRB, 0.74069E+00_JPRB, 0.75253E+00_JPRB, 0.76625E+00_JPRB, 0.78664E+00_JPRB, 0.81531E+00_JPRB, 0.85928E+00_JPRB ,&
&  0.90178E+00_JPRB, 0.94919E+00_JPRB, 0.10100E+01_JPRB, 0.10883E+01_JPRB, 0.11725E+01_JPRB, 0.12180E+01_JPRB, 0.12453E+01_JPRB ,&
&  0.81082E+00_JPRB, 0.12657E+01_JPRB, 0.14815E+01_JPRB, 0.13829E+01_JPRB, 0.12526E+01_JPRB, 0.11732E+01_JPRB, 0.11091E+01_JPRB ,&
&  0.10252E+01_JPRB, 0.94673E+00_JPRB, 0.92071E+00_JPRB, 0.97336E+00_JPRB, 0.91132E+00_JPRB, 0.90768E+00_JPRB, 0.91357E+00_JPRB ,&
&  0.95147E+00_JPRB, 0.97782E+00_JPRB, 0.98745E+00_JPRB, 0.10092E+01_JPRB, 0.10477E+01_JPRB, 0.11362E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.14795E+01_JPRB, 0.16763E+01_JPRB, 0.17797E+01_JPRB, 0.15173E+01_JPRB, 0.15435E+01_JPRB, 0.14090E+01_JPRB, 0.11815E+01_JPRB ,&
&  0.84295E+00_JPRB, 0.72135E+00_JPRB, 0.74000E+00_JPRB, 0.75845E+00_JPRB, 0.70122E+00_JPRB, 0.65069E+00_JPRB, 0.61232E+00_JPRB ,&
&  0.57099E+00_JPRB, 0.54519E+00_JPRB, 0.54237E+00_JPRB, 0.54076E+00_JPRB, 0.54832E+00_JPRB, 0.55530E+00_JPRB, 0.56886E+00_JPRB ,&
&  0.57649E+00_JPRB, 0.59424E+00_JPRB, 0.61747E+00_JPRB, 0.64275E+00_JPRB, 0.67505E+00_JPRB, 0.71315E+00_JPRB, 0.75559E+00_JPRB ,&
&  0.78687E+00_JPRB, 0.81700E+00_JPRB, 0.84379E+00_JPRB, 0.86617E+00_JPRB, 0.87569E+00_JPRB, 0.87606E+00_JPRB, 0.85228E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.84352E+00_JPRB, 0.84933E+00_JPRB, 0.84289E+00_JPRB, 0.81472E+00_JPRB, 0.79090E+00_JPRB, 0.76918E+00_JPRB ,&
&  0.72285E+00_JPRB, 0.64573E+00_JPRB, 0.57354E+00_JPRB, 0.55254E+00_JPRB, 0.53974E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.61087E+00_JPRB, 0.61746E+00_JPRB, 0.61762E+00_JPRB, 0.65024E+00_JPRB, 0.65630E+00_JPRB, 0.66811E+00_JPRB, 0.67916E+00_JPRB ,&
&  0.68724E+00_JPRB, 0.70485E+00_JPRB, 0.72634E+00_JPRB, 0.75047E+00_JPRB, 0.76965E+00_JPRB, 0.78417E+00_JPRB, 0.79615E+00_JPRB ,&
&  0.81101E+00_JPRB, 0.82833E+00_JPRB, 0.84940E+00_JPRB, 0.87505E+00_JPRB, 0.91471E+00_JPRB, 0.95787E+00_JPRB, 0.10026E+01_JPRB ,&
&  0.10440E+01_JPRB, 0.11045E+01_JPRB, 0.11824E+01_JPRB, 0.12777E+01_JPRB, 0.13759E+01_JPRB, 0.14447E+01_JPRB, 0.14727E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.15001E+01_JPRB, 0.14542E+01_JPRB, 0.13163E+01_JPRB, 0.11988E+01_JPRB, 0.11316E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.94714E+00_JPRB, 0.92182E+00_JPRB, 0.93043E+00_JPRB, 0.94490E+00_JPRB, 0.95094E+00_JPRB, 0.98256E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.10795E+01_JPRB, 0.11307E+01_JPRB, 0.11788E+01_JPRB, 0.12476E+01_JPRB, 0.13543E+01_JPRB, 0.14749E+01_JPRB ,&
&  0.15999E+01_JPRB, 0.16906E+01_JPRB, 0.17699E+01_JPRB, 0.15604E+01_JPRB, 0.14498E+01_JPRB, 0.13477E+01_JPRB, 0.97625E+00_JPRB ,&
&  0.76294E+00_JPRB, 0.70332E+00_JPRB, 0.75748E+00_JPRB, 0.79986E+00_JPRB, 0.74046E+00_JPRB, 0.67315E+00_JPRB, 0.62060E+00_JPRB ,&
&  0.57818E+00_JPRB, 0.54811E+00_JPRB, 0.55233E+00_JPRB, 0.54228E+00_JPRB, 0.54209E+00_JPRB, 0.55890E+00_JPRB, 0.57626E+00_JPRB ,&
&  0.61348E+00_JPRB, 0.62925E+00_JPRB, 0.66434E+00_JPRB, 0.69474E+00_JPRB, 0.72987E+00_JPRB, 0.76372E+00_JPRB, 0.79651E+00_JPRB ,&
&  0.82149E+00_JPRB, 0.83258E+00_JPRB, 0.83346E+00_JPRB, 0.85898E+00_JPRB, 0.86511E+00_JPRB, 0.87895E+00_JPRB, 0.90355E+00_JPRB ,&
&  0.93204E+00_JPRB, 0.95122E+00_JPRB, 0.93840E+00_JPRB, 0.92300E+00_JPRB, 0.88975E+00_JPRB, 0.85805E+00_JPRB, 0.65744E+00_JPRB ,&
&  0.62665E+00_JPRB, 0.59161E+00_JPRB, 0.60621E+00_JPRB, 0.62445E+00_JPRB, 0.61591E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.74757E+00_JPRB, 0.77765E+00_JPRB, 0.78034E+00_JPRB, 0.80183E+00_JPRB, 0.79650E+00_JPRB, 0.78221E+00_JPRB, 0.78788E+00_JPRB ,&
&  0.80062E+00_JPRB, 0.80914E+00_JPRB, 0.82946E+00_JPRB, 0.85012E+00_JPRB, 0.86550E+00_JPRB, 0.87004E+00_JPRB, 0.88318E+00_JPRB ,&
&  0.90562E+00_JPRB, 0.93547E+00_JPRB, 0.96970E+00_JPRB, 0.10149E+01_JPRB, 0.10448E+01_JPRB, 0.10620E+01_JPRB, 0.10978E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11877E+01_JPRB, 0.12611E+01_JPRB, 0.13340E+01_JPRB, 0.13787E+01_JPRB, 0.13942E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.16193E+01_JPRB, 0.15279E+01_JPRB, 0.12997E+01_JPRB, 0.11845E+01_JPRB, 0.11082E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.95384E+00_JPRB, 0.91250E+00_JPRB, 0.89575E+00_JPRB, 0.97467E+00_JPRB, 0.99520E+00_JPRB, 0.10486E+01_JPRB ,&
&  0.11078E+01_JPRB, 0.11657E+01_JPRB, 0.12198E+01_JPRB, 0.12724E+01_JPRB, 0.13492E+01_JPRB, 0.14372E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.15833E+01_JPRB, 0.16195E+01_JPRB, 0.17024E+01_JPRB, 0.15053E+01_JPRB, 0.14707E+01_JPRB, 0.14137E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.76849E+00_JPRB, 0.72704E+00_JPRB, 0.78954E+00_JPRB, 0.81380E+00_JPRB, 0.76961E+00_JPRB, 0.68676E+00_JPRB, 0.63195E+00_JPRB ,&
&  0.57285E+00_JPRB, 0.56493E+00_JPRB, 0.56047E+00_JPRB, 0.54864E+00_JPRB, 0.54027E+00_JPRB, 0.56657E+00_JPRB, 0.59006E+00_JPRB ,&
&  0.64161E+00_JPRB, 0.67816E+00_JPRB, 0.72554E+00_JPRB, 0.76979E+00_JPRB, 0.83609E+00_JPRB, 0.91954E+00_JPRB, 0.10017E+01_JPRB ,&
&  0.87625E+00_JPRB, 0.81312E+00_JPRB, 0.84868E+00_JPRB, 0.90465E+00_JPRB, 0.94372E+00_JPRB, 0.10312E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.13657E+01_JPRB, 0.12513E+01_JPRB, 0.10502E+01_JPRB, 0.14285E+01_JPRB, 0.14797E+01_JPRB, 0.68087E+00_JPRB ,&
&  0.67035E+00_JPRB, 0.70109E+00_JPRB, 0.71798E+00_JPRB, 0.73804E+00_JPRB, 0.75698E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.81965E+00_JPRB, 0.83134E+00_JPRB, 0.84267E+00_JPRB, 0.85016E+00_JPRB, 0.84971E+00_JPRB, 0.85911E+00_JPRB, 0.87820E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.91390E+00_JPRB, 0.93083E+00_JPRB, 0.94354E+00_JPRB, 0.95764E+00_JPRB, 0.96437E+00_JPRB, 0.98405E+00_JPRB ,&
&  0.10129E+01_JPRB, 0.10396E+01_JPRB, 0.10500E+01_JPRB, 0.10762E+01_JPRB, 0.10876E+01_JPRB, 0.10889E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.11514E+01_JPRB, 0.11982E+01_JPRB, 0.12313E+01_JPRB, 0.12492E+01_JPRB, 0.12429E+01_JPRB, 0.13044E+01_JPRB, 0.11586E+01_JPRB ,&
&  0.14851E+01_JPRB, 0.16776E+01_JPRB, 0.15466E+01_JPRB, 0.12535E+01_JPRB, 0.11424E+01_JPRB, 0.10844E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10025E+01_JPRB, 0.96599E+00_JPRB, 0.92041E+00_JPRB, 0.95724E+00_JPRB, 0.99020E+00_JPRB, 0.10350E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11990E+01_JPRB, 0.12523E+01_JPRB, 0.13026E+01_JPRB, 0.13782E+01_JPRB, 0.14429E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.15553E+01_JPRB, 0.15909E+01_JPRB, 0.14665E+01_JPRB, 0.14713E+01_JPRB, 0.13701E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.82514E+00_JPRB, 0.72839E+00_JPRB, 0.76641E+00_JPRB, 0.81491E+00_JPRB, 0.78853E+00_JPRB, 0.68953E+00_JPRB, 0.66348E+00_JPRB ,&
&  0.58825E+00_JPRB, 0.54399E+00_JPRB, 0.54665E+00_JPRB, 0.53891E+00_JPRB, 0.53091E+00_JPRB, 0.56232E+00_JPRB, 0.58778E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.68727E+00_JPRB, 0.73226E+00_JPRB, 0.76207E+00_JPRB, 0.83970E+00_JPRB, 0.91476E+00_JPRB, 0.86142E+00_JPRB ,&
&  0.90545E+00_JPRB, 0.92596E+00_JPRB, 0.95947E+00_JPRB, 0.10740E+01_JPRB, 0.10598E+01_JPRB, 0.12630E+01_JPRB, 0.14854E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.17332E+01_JPRB, 0.20422E+01_JPRB, 0.30764E+01_JPRB, 0.26378E+01_JPRB, 0.22176E+01_JPRB, 0.81250E+00_JPRB ,&
&  0.79909E+00_JPRB, 0.81055E+00_JPRB, 0.82993E+00_JPRB, 0.82879E+00_JPRB, 0.82682E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.84754E+00_JPRB, 0.86139E+00_JPRB, 0.87138E+00_JPRB, 0.88754E+00_JPRB, 0.90751E+00_JPRB, 0.93106E+00_JPRB, 0.94833E+00_JPRB ,&
&  0.96780E+00_JPRB, 0.98308E+00_JPRB, 0.99708E+00_JPRB, 0.10130E+01_JPRB, 0.10238E+01_JPRB, 0.10397E+01_JPRB, 0.10420E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10528E+01_JPRB, 0.10589E+01_JPRB, 0.10629E+01_JPRB, 0.10653E+01_JPRB, 0.10677E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10782E+01_JPRB, 0.10856E+01_JPRB, 0.10978E+01_JPRB, 0.11015E+01_JPRB, 0.11619E+01_JPRB, 0.10104E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.15776E+01_JPRB, 0.14860E+01_JPRB, 0.11849E+01_JPRB, 0.10672E+01_JPRB, 0.10255E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.97936E+00_JPRB, 0.96788E+00_JPRB, 0.95006E+00_JPRB, 0.95937E+00_JPRB, 0.98911E+00_JPRB, 0.10342E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.11341E+01_JPRB, 0.11764E+01_JPRB, 0.12232E+01_JPRB, 0.12982E+01_JPRB, 0.13398E+01_JPRB, 0.13482E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.13407E+01_JPRB, 0.12462E+01_JPRB, 0.12491E+01_JPRB, 0.12877E+01_JPRB, 0.11004E+01_JPRB, 0.94068E+00_JPRB ,&
&  0.80166E+00_JPRB, 0.75396E+00_JPRB, 0.78497E+00_JPRB, 0.85924E+00_JPRB, 0.79874E+00_JPRB, 0.73137E+00_JPRB, 0.69428E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.56362E+00_JPRB, 0.55194E+00_JPRB, 0.52859E+00_JPRB, 0.52936E+00_JPRB, 0.54931E+00_JPRB, 0.59551E+00_JPRB ,&
&  0.61987E+00_JPRB, 0.64642E+00_JPRB, 0.67923E+00_JPRB, 0.69506E+00_JPRB, 0.73709E+00_JPRB, 0.70997E+00_JPRB, 0.84257E+00_JPRB ,&
&  0.82931E+00_JPRB, 0.82338E+00_JPRB, 0.87366E+00_JPRB, 0.75157E+00_JPRB, 0.10107E+01_JPRB, 0.10964E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.17574E+01_JPRB, 0.21302E+01_JPRB, 0.17828E+01_JPRB, 0.12908E+01_JPRB, 0.88166E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.78291E+00_JPRB, 0.80119E+00_JPRB, 0.81987E+00_JPRB, 0.82800E+00_JPRB, 0.83731E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.84380E+00_JPRB, 0.85727E+00_JPRB, 0.87141E+00_JPRB, 0.89576E+00_JPRB, 0.92230E+00_JPRB, 0.94782E+00_JPRB, 0.96735E+00_JPRB ,&
&  0.98678E+00_JPRB, 0.10010E+01_JPRB, 0.10195E+01_JPRB, 0.10363E+01_JPRB, 0.10484E+01_JPRB, 0.10544E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10565E+01_JPRB, 0.10595E+01_JPRB, 0.10710E+01_JPRB, 0.10786E+01_JPRB, 0.10738E+01_JPRB, 0.10800E+01_JPRB, 0.10959E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11191E+01_JPRB, 0.11482E+01_JPRB, 0.11905E+01_JPRB, 0.12474E+01_JPRB, 0.13576E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.14283E+01_JPRB, 0.14056E+01_JPRB, 0.11030E+01_JPRB, 0.98683E+00_JPRB, 0.94661E+00_JPRB, 0.92678E+00_JPRB ,&
&  0.91652E+00_JPRB, 0.90532E+00_JPRB, 0.90585E+00_JPRB, 0.92388E+00_JPRB, 0.95432E+00_JPRB, 0.98871E+00_JPRB, 0.10037E+01_JPRB ,&
&  0.10242E+01_JPRB, 0.10454E+01_JPRB, 0.10681E+01_JPRB, 0.10886E+01_JPRB, 0.11269E+01_JPRB, 0.11384E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10601E+01_JPRB, 0.92930E+00_JPRB, 0.87129E+00_JPRB, 0.89103E+00_JPRB, 0.81460E+00_JPRB, 0.73986E+00_JPRB ,&
&  0.75566E+00_JPRB, 0.73723E+00_JPRB, 0.72691E+00_JPRB, 0.88321E+00_JPRB, 0.86085E+00_JPRB, 0.74938E+00_JPRB, 0.72420E+00_JPRB ,&
&  0.64444E+00_JPRB, 0.57127E+00_JPRB, 0.53829E+00_JPRB, 0.53790E+00_JPRB, 0.54412E+00_JPRB, 0.57054E+00_JPRB, 0.57155E+00_JPRB ,&
&  0.58183E+00_JPRB, 0.59920E+00_JPRB, 0.60524E+00_JPRB, 0.60540E+00_JPRB, 0.66390E+00_JPRB, 0.58827E+00_JPRB, 0.75232E+00_JPRB ,&
&  0.69680E+00_JPRB, 0.58851E+00_JPRB, 0.69400E+00_JPRB, 0.84941E+00_JPRB, 0.91383E+00_JPRB, 0.99578E+00_JPRB, 0.11528E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.14631E+01_JPRB, 0.13150E+01_JPRB, 0.11097E+01_JPRB, 0.89842E+00_JPRB, 0.80818E+00_JPRB, 0.77145E+00_JPRB ,&
&  0.76573E+00_JPRB, 0.78939E+00_JPRB, 0.80765E+00_JPRB, 0.81655E+00_JPRB, 0.82948E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.80601E+00_JPRB, 0.82162E+00_JPRB, 0.83918E+00_JPRB, 0.86049E+00_JPRB, 0.88047E+00_JPRB, 0.89836E+00_JPRB, 0.91847E+00_JPRB ,&
&  0.94470E+00_JPRB, 0.96478E+00_JPRB, 0.98548E+00_JPRB, 0.99827E+00_JPRB, 0.10048E+01_JPRB, 0.10180E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.10255E+01_JPRB, 0.10412E+01_JPRB, 0.10516E+01_JPRB, 0.10507E+01_JPRB, 0.10474E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10913E+01_JPRB, 0.11167E+01_JPRB, 0.11481E+01_JPRB, 0.11745E+01_JPRB, 0.11839E+01_JPRB, 0.12598E+01_JPRB ,&
&  0.92998E+00_JPRB, 0.13310E+01_JPRB, 0.12985E+01_JPRB, 0.10164E+01_JPRB, 0.94567E+00_JPRB, 0.88337E+00_JPRB, 0.85784E+00_JPRB ,&
&  0.84553E+00_JPRB, 0.84044E+00_JPRB, 0.84885E+00_JPRB, 0.87106E+00_JPRB, 0.89607E+00_JPRB, 0.92483E+00_JPRB, 0.93798E+00_JPRB ,&
&  0.94885E+00_JPRB, 0.95647E+00_JPRB, 0.95952E+00_JPRB, 0.95725E+00_JPRB, 0.95957E+00_JPRB, 0.94714E+00_JPRB, 0.92393E+00_JPRB ,&
&  0.88141E+00_JPRB, 0.84682E+00_JPRB, 0.62678E+00_JPRB, 0.58729E+00_JPRB, 0.61680E+00_JPRB, 0.60508E+00_JPRB, 0.55371E+00_JPRB ,&
&  0.64394E+00_JPRB, 0.74697E+00_JPRB, 0.82259E+00_JPRB, 0.88540E+00_JPRB, 0.88319E+00_JPRB, 0.81542E+00_JPRB, 0.78999E+00_JPRB ,&
&  0.74204E+00_JPRB, 0.66465E+00_JPRB, 0.62955E+00_JPRB, 0.60525E+00_JPRB, 0.60559E+00_JPRB, 0.59027E+00_JPRB, 0.57230E+00_JPRB ,&
&  0.54608E+00_JPRB, 0.53068E+00_JPRB, 0.51238E+00_JPRB, 0.47777E+00_JPRB, 0.39575E+00_JPRB, 0.48993E+00_JPRB, 0.65623E+00_JPRB ,&
&  0.59023E+00_JPRB, 0.50370E+00_JPRB, 0.52641E+00_JPRB, 0.70951E+00_JPRB, 0.74134E+00_JPRB, 0.77931E+00_JPRB, 0.89837E+00_JPRB ,&
&  0.98862E+00_JPRB, 0.99647E+00_JPRB, 0.90099E+00_JPRB, 0.79101E+00_JPRB, 0.74893E+00_JPRB, 0.73330E+00_JPRB, 0.71069E+00_JPRB ,&
&  0.71042E+00_JPRB, 0.73376E+00_JPRB, 0.75566E+00_JPRB, 0.77192E+00_JPRB, 0.78898E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.76886E+00_JPRB, 0.76020E+00_JPRB, 0.76989E+00_JPRB, 0.78544E+00_JPRB, 0.79991E+00_JPRB, 0.81516E+00_JPRB, 0.84063E+00_JPRB ,&
&  0.87222E+00_JPRB, 0.89784E+00_JPRB, 0.92345E+00_JPRB, 0.94152E+00_JPRB, 0.94749E+00_JPRB, 0.96089E+00_JPRB, 0.97256E+00_JPRB ,&
&  0.98617E+00_JPRB, 0.98236E+00_JPRB, 0.99958E+00_JPRB, 0.10290E+01_JPRB, 0.10624E+01_JPRB, 0.10752E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.10999E+01_JPRB, 0.11385E+01_JPRB, 0.11845E+01_JPRB, 0.12393E+01_JPRB, 0.13322E+01_JPRB, 0.14969E+01_JPRB, 0.16698E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11408E+01_JPRB, 0.11003E+01_JPRB, 0.89306E+00_JPRB, 0.81774E+00_JPRB, 0.82882E+00_JPRB, 0.79154E+00_JPRB ,&
&  0.76060E+00_JPRB, 0.76978E+00_JPRB, 0.77447E+00_JPRB, 0.78003E+00_JPRB, 0.79638E+00_JPRB, 0.82961E+00_JPRB, 0.83508E+00_JPRB ,&
&  0.84933E+00_JPRB, 0.85335E+00_JPRB, 0.84441E+00_JPRB, 0.84508E+00_JPRB, 0.77391E+00_JPRB, 0.85660E+00_JPRB, 0.82804E+00_JPRB ,&
&  0.80937E+00_JPRB, 0.75496E+00_JPRB, 0.54740E+00_JPRB, 0.52177E+00_JPRB, 0.53989E+00_JPRB, 0.50247E+00_JPRB, 0.48115E+00_JPRB ,&
&  0.59884E+00_JPRB, 0.69627E+00_JPRB, 0.63592E+00_JPRB, 0.81937E+00_JPRB, 0.90384E+00_JPRB, 0.92891E+00_JPRB, 0.92693E+00_JPRB ,&
&  0.90574E+00_JPRB, 0.83297E+00_JPRB, 0.77142E+00_JPRB, 0.72007E+00_JPRB, 0.69300E+00_JPRB, 0.65394E+00_JPRB, 0.60465E+00_JPRB ,&
&  0.56263E+00_JPRB, 0.52643E+00_JPRB, 0.47477E+00_JPRB, 0.43191E+00_JPRB, 0.47083E+00_JPRB, 0.37491E+00_JPRB, 0.57489E+00_JPRB ,&
&  0.53745E+00_JPRB, 0.51817E+00_JPRB, 0.45425E+00_JPRB, 0.60104E+00_JPRB, 0.60577E+00_JPRB, 0.57725E+00_JPRB, 0.64972E+00_JPRB ,&
&  0.69613E+00_JPRB, 0.72849E+00_JPRB, 0.71246E+00_JPRB, 0.68273E+00_JPRB, 0.67101E+00_JPRB, 0.67215E+00_JPRB, 0.65244E+00_JPRB ,&
&  0.64552E+00_JPRB, 0.66367E+00_JPRB, 0.69740E+00_JPRB, 0.70884E+00_JPRB, 0.73972E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.68060E+00_JPRB, 0.68425E+00_JPRB, 0.68844E+00_JPRB, 0.69622E+00_JPRB, 0.71258E+00_JPRB, 0.73033E+00_JPRB, 0.74118E+00_JPRB ,&
&  0.76516E+00_JPRB, 0.78475E+00_JPRB, 0.80616E+00_JPRB, 0.81305E+00_JPRB, 0.82835E+00_JPRB, 0.84389E+00_JPRB, 0.84969E+00_JPRB ,&
&  0.87015E+00_JPRB, 0.89411E+00_JPRB, 0.89563E+00_JPRB, 0.90497E+00_JPRB, 0.93325E+00_JPRB, 0.95212E+00_JPRB, 0.93550E+00_JPRB ,&
&  0.91333E+00_JPRB, 0.88640E+00_JPRB, 0.90534E+00_JPRB, 0.94393E+00_JPRB, 0.11237E+01_JPRB, 0.12309E+01_JPRB, 0.12137E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10481E+01_JPRB, 0.11292E+01_JPRB, 0.89411E+00_JPRB, 0.80659E+00_JPRB, 0.75214E+00_JPRB, 0.69663E+00_JPRB ,&
&  0.70445E+00_JPRB, 0.72382E+00_JPRB, 0.73899E+00_JPRB, 0.75627E+00_JPRB, 0.78666E+00_JPRB, 0.80240E+00_JPRB, 0.83621E+00_JPRB ,&
&  0.83863E+00_JPRB, 0.84964E+00_JPRB, 0.84424E+00_JPRB, 0.80850E+00_JPRB, 0.79779E+00_JPRB, 0.88109E+00_JPRB, 0.87571E+00_JPRB ,&
&  0.88553E+00_JPRB, 0.92023E+00_JPRB, 0.81613E+00_JPRB, 0.77043E+00_JPRB, 0.73701E+00_JPRB, 0.64161E+00_JPRB, 0.57439E+00_JPRB ,&
&  0.58673E+00_JPRB, 0.58943E+00_JPRB, 0.50614E+00_JPRB, 0.71003E+00_JPRB, 0.88094E+00_JPRB, 0.95624E+00_JPRB, 0.10480E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.99303E+00_JPRB, 0.88347E+00_JPRB, 0.86154E+00_JPRB, 0.84113E+00_JPRB, 0.72266E+00_JPRB, 0.67521E+00_JPRB ,&
&  0.59974E+00_JPRB, 0.54057E+00_JPRB, 0.49046E+00_JPRB, 0.46228E+00_JPRB, 0.42935E+00_JPRB, 0.41715E+00_JPRB, 0.46740E+00_JPRB ,&
&  0.47578E+00_JPRB, 0.47635E+00_JPRB, 0.52673E+00_JPRB, 0.61849E+00_JPRB, 0.47421E+00_JPRB, 0.48025E+00_JPRB, 0.51727E+00_JPRB ,&
&  0.53737E+00_JPRB, 0.54075E+00_JPRB, 0.54442E+00_JPRB, 0.54773E+00_JPRB, 0.55397E+00_JPRB, 0.57273E+00_JPRB, 0.56966E+00_JPRB ,&
&  0.56749E+00_JPRB, 0.56725E+00_JPRB, 0.58610E+00_JPRB, 0.60988E+00_JPRB, 0.65060E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.54052E+00_JPRB, 0.54091E+00_JPRB, 0.52917E+00_JPRB, 0.52147E+00_JPRB, 0.52177E+00_JPRB, 0.52761E+00_JPRB, 0.53599E+00_JPRB ,&
&  0.55019E+00_JPRB, 0.57149E+00_JPRB, 0.59502E+00_JPRB, 0.62712E+00_JPRB, 0.63569E+00_JPRB, 0.60717E+00_JPRB, 0.61463E+00_JPRB ,&
&  0.61262E+00_JPRB, 0.62651E+00_JPRB, 0.61951E+00_JPRB, 0.69408E+00_JPRB, 0.72096E+00_JPRB, 0.73352E+00_JPRB, 0.82162E+00_JPRB ,&
&  0.81072E+00_JPRB, 0.89287E+00_JPRB, 0.95741E+00_JPRB, 0.10703E+01_JPRB, 0.17675E+01_JPRB, 0.16260E+01_JPRB, 0.10974E+01_JPRB ,&
&  0.11434E+01_JPRB, 0.12185E+01_JPRB, 0.10453E+01_JPRB, 0.88654E+00_JPRB, 0.83203E+00_JPRB, 0.75170E+00_JPRB, 0.72397E+00_JPRB ,&
&  0.75687E+00_JPRB, 0.80015E+00_JPRB, 0.88591E+00_JPRB, 0.96394E+00_JPRB, 0.10238E+01_JPRB, 0.10862E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.12486E+01_JPRB, 0.12012E+01_JPRB, 0.11055E+01_JPRB, 0.11883E+01_JPRB, 0.12673E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.13608E+01_JPRB, 0.14094E+01_JPRB, 0.14747E+01_JPRB, 0.13832E+01_JPRB, 0.12400E+01_JPRB, 0.11369E+01_JPRB, 0.95275E+00_JPRB ,&
&  0.80765E+00_JPRB, 0.69108E+00_JPRB, 0.63442E+00_JPRB, 0.85869E+00_JPRB, 0.10094E+01_JPRB, 0.97014E+00_JPRB, 0.11518E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.12167E+01_JPRB, 0.11085E+01_JPRB, 0.10015E+01_JPRB, 0.88054E+00_JPRB, 0.80923E+00_JPRB, 0.70393E+00_JPRB ,&
&  0.65814E+00_JPRB, 0.58152E+00_JPRB, 0.52485E+00_JPRB, 0.48491E+00_JPRB, 0.44025E+00_JPRB, 0.41044E+00_JPRB, 0.40696E+00_JPRB ,&
&  0.42633E+00_JPRB, 0.43066E+00_JPRB, 0.50002E+00_JPRB, 0.71405E+00_JPRB, 0.79935E+00_JPRB, 0.52905E+00_JPRB, 0.44980E+00_JPRB ,&
&  0.44603E+00_JPRB, 0.45377E+00_JPRB, 0.45838E+00_JPRB, 0.46407E+00_JPRB, 0.47964E+00_JPRB, 0.49329E+00_JPRB, 0.48867E+00_JPRB ,&
&  0.49369E+00_JPRB, 0.49113E+00_JPRB, 0.48962E+00_JPRB, 0.49649E+00_JPRB, 0.51256E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.47083E+00_JPRB, 0.47471E+00_JPRB, 0.48804E+00_JPRB, 0.53017E+00_JPRB, 0.55659E+00_JPRB, 0.57089E+00_JPRB, 0.60279E+00_JPRB ,&
&  0.63985E+00_JPRB, 0.70292E+00_JPRB, 0.75055E+00_JPRB, 0.77184E+00_JPRB, 0.75061E+00_JPRB, 0.72325E+00_JPRB, 0.70716E+00_JPRB ,&
&  0.69982E+00_JPRB, 0.65052E+00_JPRB, 0.61625E+00_JPRB, 0.63378E+00_JPRB, 0.69292E+00_JPRB, 0.80027E+00_JPRB, 0.92516E+00_JPRB ,&
&  0.10832E+01_JPRB, 0.12568E+01_JPRB, 0.14782E+01_JPRB, 0.20389E+01_JPRB, 0.22654E+01_JPRB, 0.10647E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.13455E+01_JPRB, 0.14133E+01_JPRB, 0.11107E+01_JPRB, 0.96738E+00_JPRB, 0.89590E+00_JPRB, 0.85954E+00_JPRB, 0.86599E+00_JPRB ,&
&  0.93044E+00_JPRB, 0.98980E+00_JPRB, 0.10644E+01_JPRB, 0.11610E+01_JPRB, 0.12681E+01_JPRB, 0.13692E+01_JPRB, 0.14941E+01_JPRB ,&
&  0.15985E+01_JPRB, 0.16226E+01_JPRB, 0.16202E+01_JPRB, 0.16243E+01_JPRB, 0.16941E+01_JPRB, 0.17935E+01_JPRB, 0.18801E+01_JPRB ,&
&  0.19267E+01_JPRB, 0.19344E+01_JPRB, 0.19733E+01_JPRB, 0.19514E+01_JPRB, 0.17486E+01_JPRB, 0.15821E+01_JPRB, 0.13814E+01_JPRB ,&
&  0.12388E+01_JPRB, 0.10901E+01_JPRB, 0.10088E+01_JPRB, 0.13229E+01_JPRB, 0.14408E+01_JPRB, 0.13456E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.13900E+01_JPRB, 0.13535E+01_JPRB, 0.12494E+01_JPRB, 0.10366E+01_JPRB, 0.87504E+00_JPRB, 0.86708E+00_JPRB, 0.70776E+00_JPRB ,&
&  0.71248E+00_JPRB, 0.64314E+00_JPRB, 0.58157E+00_JPRB, 0.52248E+00_JPRB, 0.46092E+00_JPRB, 0.38588E+00_JPRB, 0.36656E+00_JPRB ,&
&  0.34950E+00_JPRB, 0.43907E+00_JPRB, 0.51308E+00_JPRB, 0.83112E+00_JPRB, 0.17822E+01_JPRB, 0.90650E+00_JPRB, 0.56321E+00_JPRB ,&
&  0.43673E+00_JPRB, 0.42765E+00_JPRB, 0.43155E+00_JPRB, 0.44062E+00_JPRB, 0.45382E+00_JPRB, 0.46526E+00_JPRB, 0.46920E+00_JPRB ,&
&  0.48311E+00_JPRB, 0.48510E+00_JPRB, 0.49255E+00_JPRB, 0.48461E+00_JPRB, 0.46327E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.60526E+00_JPRB, 0.63613E+00_JPRB, 0.68216E+00_JPRB, 0.73555E+00_JPRB, 0.76754E+00_JPRB, 0.79220E+00_JPRB, 0.79285E+00_JPRB ,&
&  0.81849E+00_JPRB, 0.84174E+00_JPRB, 0.88027E+00_JPRB, 0.91597E+00_JPRB, 0.93673E+00_JPRB, 0.98364E+00_JPRB, 0.10152E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.10007E+01_JPRB, 0.10315E+01_JPRB, 0.10858E+01_JPRB, 0.12231E+01_JPRB, 0.14409E+01_JPRB, 0.16591E+01_JPRB ,&
&  0.18592E+01_JPRB, 0.18938E+01_JPRB, 0.18566E+01_JPRB, 0.12282E+01_JPRB, 0.84648E+00_JPRB, 0.96244E+00_JPRB, 0.10616E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.11320E+01_JPRB, 0.10318E+01_JPRB, 0.96429E+00_JPRB, 0.91997E+00_JPRB, 0.92433E+00_JPRB, 0.98750E+00_JPRB ,&
&  0.10849E+01_JPRB, 0.11927E+01_JPRB, 0.12979E+01_JPRB, 0.13684E+01_JPRB, 0.14521E+01_JPRB, 0.15770E+01_JPRB, 0.17281E+01_JPRB ,&
&  0.18563E+01_JPRB, 0.19265E+01_JPRB, 0.19477E+01_JPRB, 0.20107E+01_JPRB, 0.20450E+01_JPRB, 0.21154E+01_JPRB, 0.21667E+01_JPRB ,&
&  0.22060E+01_JPRB, 0.22110E+01_JPRB, 0.22137E+01_JPRB, 0.22326E+01_JPRB, 0.21579E+01_JPRB, 0.19181E+01_JPRB, 0.17484E+01_JPRB ,&
&  0.16797E+01_JPRB, 0.15654E+01_JPRB, 0.15953E+01_JPRB, 0.17327E+01_JPRB, 0.12497E+01_JPRB, 0.13239E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.15414E+01_JPRB, 0.14692E+01_JPRB, 0.13337E+01_JPRB, 0.11599E+01_JPRB, 0.10645E+01_JPRB, 0.86737E+00_JPRB, 0.76298E+00_JPRB ,&
&  0.76357E+00_JPRB, 0.69934E+00_JPRB, 0.62023E+00_JPRB, 0.50980E+00_JPRB, 0.45792E+00_JPRB, 0.39488E+00_JPRB, 0.37859E+00_JPRB ,&
&  0.42125E+00_JPRB, 0.47949E+00_JPRB, 0.48452E+00_JPRB, 0.62603E+00_JPRB, 0.87521E+00_JPRB, 0.62187E+00_JPRB, 0.49982E+00_JPRB ,&
&  0.43074E+00_JPRB, 0.42285E+00_JPRB, 0.42606E+00_JPRB, 0.42031E+00_JPRB, 0.43339E+00_JPRB, 0.45114E+00_JPRB, 0.45439E+00_JPRB ,&
&  0.49435E+00_JPRB, 0.52478E+00_JPRB, 0.55418E+00_JPRB, 0.58232E+00_JPRB, 0.58428E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.73779E+00_JPRB, 0.83553E+00_JPRB, 0.91134E+00_JPRB, 0.98441E+00_JPRB, 0.10278E+01_JPRB, 0.10901E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.10080E+01_JPRB, 0.95345E+00_JPRB, 0.10208E+01_JPRB, 0.11089E+01_JPRB, 0.11872E+01_JPRB, 0.12444E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.13856E+01_JPRB, 0.14255E+01_JPRB, 0.14427E+01_JPRB, 0.14818E+01_JPRB, 0.15395E+01_JPRB, 0.15339E+01_JPRB ,&
&  0.12621E+01_JPRB, 0.10321E+01_JPRB, 0.12713E+01_JPRB, 0.13392E+01_JPRB, 0.10708E+01_JPRB, 0.10871E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11348E+01_JPRB, 0.10957E+01_JPRB, 0.11017E+01_JPRB, 0.10994E+01_JPRB, 0.94012E+00_JPRB, 0.99618E+00_JPRB, 0.11317E+01_JPRB ,&
&  0.12702E+01_JPRB, 0.13961E+01_JPRB, 0.14964E+01_JPRB, 0.15655E+01_JPRB, 0.16428E+01_JPRB, 0.17640E+01_JPRB, 0.18892E+01_JPRB ,&
&  0.20081E+01_JPRB, 0.20866E+01_JPRB, 0.21269E+01_JPRB, 0.21709E+01_JPRB, 0.22377E+01_JPRB, 0.23347E+01_JPRB, 0.23825E+01_JPRB ,&
&  0.24252E+01_JPRB, 0.24661E+01_JPRB, 0.25282E+01_JPRB, 0.25963E+01_JPRB, 0.25979E+01_JPRB, 0.23098E+01_JPRB, 0.22006E+01_JPRB ,&
&  0.21356E+01_JPRB, 0.19966E+01_JPRB, 0.20496E+01_JPRB, 0.21342E+01_JPRB, 0.16038E+01_JPRB, 0.15736E+01_JPRB, 0.16089E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.16464E+01_JPRB, 0.15087E+01_JPRB, 0.14728E+01_JPRB, 0.13493E+01_JPRB, 0.11693E+01_JPRB, 0.99960E+00_JPRB ,&
&  0.78900E+00_JPRB, 0.70383E+00_JPRB, 0.60263E+00_JPRB, 0.52378E+00_JPRB, 0.39624E+00_JPRB, 0.39278E+00_JPRB, 0.43865E+00_JPRB ,&
&  0.56149E+00_JPRB, 0.49635E+00_JPRB, 0.49015E+00_JPRB, 0.55911E+00_JPRB, 0.59386E+00_JPRB, 0.48296E+00_JPRB, 0.44428E+00_JPRB ,&
&  0.42634E+00_JPRB, 0.43752E+00_JPRB, 0.44165E+00_JPRB, 0.43535E+00_JPRB, 0.43487E+00_JPRB, 0.45281E+00_JPRB, 0.47183E+00_JPRB ,&
&  0.50771E+00_JPRB, 0.54865E+00_JPRB, 0.58540E+00_JPRB, 0.63299E+00_JPRB, 0.69355E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.73996E+00_JPRB, 0.81772E+00_JPRB, 0.93841E+00_JPRB, 0.10925E+01_JPRB, 0.12940E+01_JPRB, 0.14871E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.12961E+01_JPRB, 0.10616E+01_JPRB, 0.11600E+01_JPRB, 0.12586E+01_JPRB, 0.13426E+01_JPRB, 0.14311E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.15953E+01_JPRB, 0.16874E+01_JPRB, 0.17729E+01_JPRB, 0.18392E+01_JPRB, 0.18618E+01_JPRB, 0.18484E+01_JPRB, 0.16472E+01_JPRB ,&
&  0.11937E+01_JPRB, 0.12139E+01_JPRB, 0.16340E+01_JPRB, 0.13792E+01_JPRB, 0.11910E+01_JPRB, 0.12192E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.12214E+01_JPRB, 0.11445E+01_JPRB, 0.11726E+01_JPRB, 0.11466E+01_JPRB, 0.10426E+01_JPRB, 0.11353E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.15279E+01_JPRB, 0.16432E+01_JPRB, 0.17355E+01_JPRB, 0.17828E+01_JPRB, 0.18680E+01_JPRB, 0.19454E+01_JPRB ,&
&  0.20086E+01_JPRB, 0.20793E+01_JPRB, 0.21226E+01_JPRB, 0.21766E+01_JPRB, 0.23294E+01_JPRB, 0.25151E+01_JPRB, 0.26617E+01_JPRB ,&
&  0.27325E+01_JPRB, 0.29301E+01_JPRB, 0.29908E+01_JPRB, 0.28912E+01_JPRB, 0.26353E+01_JPRB, 0.27519E+01_JPRB, 0.24995E+01_JPRB ,&
&  0.25232E+01_JPRB, 0.23933E+01_JPRB, 0.25540E+01_JPRB, 0.21330E+01_JPRB, 0.21145E+01_JPRB, 0.21250E+01_JPRB, 0.20919E+01_JPRB ,&
&  0.19869E+01_JPRB, 0.19391E+01_JPRB, 0.16991E+01_JPRB, 0.15707E+01_JPRB, 0.15866E+01_JPRB, 0.14512E+01_JPRB, 0.14154E+01_JPRB ,&
&  0.93661E+00_JPRB, 0.77893E+00_JPRB, 0.58732E+00_JPRB, 0.47083E+00_JPRB, 0.37239E+00_JPRB, 0.39796E+00_JPRB, 0.51166E+00_JPRB ,&
&  0.67432E+00_JPRB, 0.65445E+00_JPRB, 0.63674E+00_JPRB, 0.63229E+00_JPRB, 0.55828E+00_JPRB, 0.48075E+00_JPRB, 0.43581E+00_JPRB ,&
&  0.41590E+00_JPRB, 0.43068E+00_JPRB, 0.45497E+00_JPRB, 0.46035E+00_JPRB, 0.45579E+00_JPRB, 0.47117E+00_JPRB, 0.49331E+00_JPRB ,&
&  0.52367E+00_JPRB, 0.55785E+00_JPRB, 0.60534E+00_JPRB, 0.63073E+00_JPRB, 0.68535E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.64717E+00_JPRB, 0.70736E+00_JPRB, 0.78541E+00_JPRB, 0.89353E+00_JPRB, 0.10029E+01_JPRB, 0.10809E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10040E+01_JPRB, 0.10518E+01_JPRB, 0.11233E+01_JPRB, 0.12092E+01_JPRB, 0.12995E+01_JPRB, 0.13994E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.15831E+01_JPRB, 0.16947E+01_JPRB, 0.18253E+01_JPRB, 0.19439E+01_JPRB, 0.19834E+01_JPRB, 0.18857E+01_JPRB, 0.13730E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.19189E+01_JPRB, 0.15911E+01_JPRB, 0.14624E+01_JPRB, 0.13182E+01_JPRB, 0.13475E+01_JPRB, 0.12523E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11673E+01_JPRB, 0.12172E+01_JPRB, 0.12165E+01_JPRB, 0.12453E+01_JPRB, 0.13344E+01_JPRB, 0.14326E+01_JPRB ,&
&  0.14877E+01_JPRB, 0.15825E+01_JPRB, 0.16968E+01_JPRB, 0.18038E+01_JPRB, 0.18403E+01_JPRB, 0.18747E+01_JPRB, 0.19033E+01_JPRB ,&
&  0.19315E+01_JPRB, 0.19644E+01_JPRB, 0.19989E+01_JPRB, 0.21785E+01_JPRB, 0.23620E+01_JPRB, 0.26068E+01_JPRB, 0.28959E+01_JPRB ,&
&  0.30127E+01_JPRB, 0.30865E+01_JPRB, 0.34098E+01_JPRB, 0.33826E+01_JPRB, 0.32360E+01_JPRB, 0.32157E+01_JPRB, 0.26655E+01_JPRB ,&
&  0.27784E+01_JPRB, 0.26083E+01_JPRB, 0.28614E+01_JPRB, 0.23972E+01_JPRB, 0.23839E+01_JPRB, 0.26141E+01_JPRB, 0.25867E+01_JPRB ,&
&  0.23156E+01_JPRB, 0.22877E+01_JPRB, 0.20157E+01_JPRB, 0.17348E+01_JPRB, 0.16240E+01_JPRB, 0.16223E+01_JPRB, 0.16979E+01_JPRB ,&
&  0.13890E+01_JPRB, 0.84606E+00_JPRB, 0.64402E+00_JPRB, 0.48150E+00_JPRB, 0.39876E+00_JPRB, 0.38255E+00_JPRB, 0.50214E+00_JPRB ,&
&  0.73585E+00_JPRB, 0.89222E+00_JPRB, 0.10207E+01_JPRB, 0.88541E+00_JPRB, 0.74577E+00_JPRB, 0.65773E+00_JPRB, 0.55496E+00_JPRB ,&
&  0.44986E+00_JPRB, 0.44766E+00_JPRB, 0.48698E+00_JPRB, 0.50099E+00_JPRB, 0.49521E+00_JPRB, 0.51079E+00_JPRB, 0.51840E+00_JPRB ,&
&  0.51705E+00_JPRB, 0.52720E+00_JPRB, 0.55907E+00_JPRB, 0.56699E+00_JPRB, 0.59962E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.57221E+00_JPRB, 0.60539E+00_JPRB, 0.67570E+00_JPRB, 0.74157E+00_JPRB, 0.78365E+00_JPRB, 0.80897E+00_JPRB, 0.82892E+00_JPRB ,&
&  0.88789E+00_JPRB, 0.96062E+00_JPRB, 0.10137E+01_JPRB, 0.10558E+01_JPRB, 0.11184E+01_JPRB, 0.12240E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.15169E+01_JPRB, 0.17048E+01_JPRB, 0.19335E+01_JPRB, 0.21644E+01_JPRB, 0.23405E+01_JPRB, 0.22766E+01_JPRB, 0.14317E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.18763E+01_JPRB, 0.18328E+01_JPRB, 0.17463E+01_JPRB, 0.15316E+01_JPRB, 0.14442E+01_JPRB, 0.12828E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.12709E+01_JPRB, 0.13134E+01_JPRB, 0.13704E+01_JPRB, 0.14129E+01_JPRB, 0.14506E+01_JPRB, 0.14347E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.14438E+01_JPRB, 0.14974E+01_JPRB, 0.16183E+01_JPRB, 0.17440E+01_JPRB, 0.18325E+01_JPRB, 0.18751E+01_JPRB ,&
&  0.18862E+01_JPRB, 0.18855E+01_JPRB, 0.19265E+01_JPRB, 0.20043E+01_JPRB, 0.23624E+01_JPRB, 0.26128E+01_JPRB, 0.29413E+01_JPRB ,&
&  0.31634E+01_JPRB, 0.32876E+01_JPRB, 0.39141E+01_JPRB, 0.43384E+01_JPRB, 0.43968E+01_JPRB, 0.38134E+01_JPRB, 0.31048E+01_JPRB ,&
&  0.28812E+01_JPRB, 0.27609E+01_JPRB, 0.27745E+01_JPRB, 0.25396E+01_JPRB, 0.25513E+01_JPRB, 0.29575E+01_JPRB, 0.32970E+01_JPRB ,&
&  0.29082E+01_JPRB, 0.26163E+01_JPRB, 0.23896E+01_JPRB, 0.19861E+01_JPRB, 0.17750E+01_JPRB, 0.17077E+01_JPRB, 0.19791E+01_JPRB ,&
&  0.19772E+01_JPRB, 0.13871E+01_JPRB, 0.85255E+00_JPRB, 0.54095E+00_JPRB, 0.42953E+00_JPRB, 0.37105E+00_JPRB, 0.43779E+00_JPRB ,&
&  0.62067E+00_JPRB, 0.83041E+00_JPRB, 0.10720E+01_JPRB, 0.12634E+01_JPRB, 0.10562E+01_JPRB, 0.96479E+00_JPRB, 0.89501E+00_JPRB ,&
&  0.61296E+00_JPRB, 0.53341E+00_JPRB, 0.53902E+00_JPRB, 0.54499E+00_JPRB, 0.53172E+00_JPRB, 0.55556E+00_JPRB, 0.58284E+00_JPRB ,&
&  0.56147E+00_JPRB, 0.50384E+00_JPRB, 0.50484E+00_JPRB, 0.51233E+00_JPRB, 0.53556E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.52037E+00_JPRB, 0.52336E+00_JPRB, 0.56610E+00_JPRB, 0.60263E+00_JPRB, 0.64162E+00_JPRB, 0.67775E+00_JPRB, 0.72986E+00_JPRB ,&
&  0.80307E+00_JPRB, 0.86042E+00_JPRB, 0.89870E+00_JPRB, 0.90328E+00_JPRB, 0.90259E+00_JPRB, 0.95107E+00_JPRB, 0.10483E+01_JPRB ,&
&  0.12117E+01_JPRB, 0.14698E+01_JPRB, 0.18301E+01_JPRB, 0.21310E+01_JPRB, 0.22637E+01_JPRB, 0.20358E+01_JPRB, 0.19109E+01_JPRB ,&
&  0.19266E+01_JPRB, 0.22591E+01_JPRB, 0.23568E+01_JPRB, 0.24540E+01_JPRB, 0.23573E+01_JPRB, 0.22182E+01_JPRB, 0.15228E+01_JPRB ,&
&  0.15017E+01_JPRB, 0.15423E+01_JPRB, 0.15686E+01_JPRB, 0.16374E+01_JPRB, 0.16659E+01_JPRB, 0.16519E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.14576E+01_JPRB, 0.13678E+01_JPRB, 0.13261E+01_JPRB, 0.13771E+01_JPRB, 0.15314E+01_JPRB, 0.16798E+01_JPRB, 0.18279E+01_JPRB ,&
&  0.19323E+01_JPRB, 0.19523E+01_JPRB, 0.19541E+01_JPRB, 0.19252E+01_JPRB, 0.22247E+01_JPRB, 0.27182E+01_JPRB, 0.31358E+01_JPRB ,&
&  0.34729E+01_JPRB, 0.40243E+01_JPRB, 0.44988E+01_JPRB, 0.48315E+01_JPRB, 0.50466E+01_JPRB, 0.46579E+01_JPRB, 0.38533E+01_JPRB ,&
&  0.31975E+01_JPRB, 0.27563E+01_JPRB, 0.24735E+01_JPRB, 0.25087E+01_JPRB, 0.27120E+01_JPRB, 0.32403E+01_JPRB, 0.32851E+01_JPRB ,&
&  0.27929E+01_JPRB, 0.23454E+01_JPRB, 0.21726E+01_JPRB, 0.21385E+01_JPRB, 0.21106E+01_JPRB, 0.19511E+01_JPRB, 0.23048E+01_JPRB ,&
&  0.20257E+01_JPRB, 0.10104E+01_JPRB, 0.65802E+00_JPRB, 0.68642E+00_JPRB, 0.45373E+00_JPRB, 0.35497E+00_JPRB, 0.47673E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.92478E+00_JPRB, 0.12165E+01_JPRB, 0.11647E+01_JPRB, 0.14281E+01_JPRB, 0.14244E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.94002E+00_JPRB, 0.73876E+00_JPRB, 0.60030E+00_JPRB, 0.56744E+00_JPRB, 0.55198E+00_JPRB, 0.56721E+00_JPRB, 0.57257E+00_JPRB ,&
&  0.54914E+00_JPRB, 0.49958E+00_JPRB, 0.49152E+00_JPRB, 0.49635E+00_JPRB, 0.50360E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.54852E+00_JPRB, 0.51474E+00_JPRB, 0.51159E+00_JPRB, 0.51465E+00_JPRB, 0.57337E+00_JPRB, 0.64114E+00_JPRB, 0.67657E+00_JPRB ,&
&  0.74794E+00_JPRB, 0.80894E+00_JPRB, 0.82542E+00_JPRB, 0.80890E+00_JPRB, 0.78441E+00_JPRB, 0.78716E+00_JPRB, 0.82485E+00_JPRB ,&
&  0.91857E+00_JPRB, 0.11208E+01_JPRB, 0.15216E+01_JPRB, 0.19422E+01_JPRB, 0.21685E+01_JPRB, 0.21330E+01_JPRB, 0.22860E+01_JPRB ,&
&  0.22726E+01_JPRB, 0.27361E+01_JPRB, 0.35082E+01_JPRB, 0.37701E+01_JPRB, 0.29777E+01_JPRB, 0.26080E+01_JPRB, 0.19665E+01_JPRB ,&
&  0.18572E+01_JPRB, 0.18810E+01_JPRB, 0.19473E+01_JPRB, 0.20531E+01_JPRB, 0.20745E+01_JPRB, 0.19990E+01_JPRB, 0.19212E+01_JPRB ,&
&  0.17262E+01_JPRB, 0.15516E+01_JPRB, 0.13714E+01_JPRB, 0.13094E+01_JPRB, 0.14057E+01_JPRB, 0.15912E+01_JPRB, 0.17922E+01_JPRB ,&
&  0.19883E+01_JPRB, 0.20870E+01_JPRB, 0.20809E+01_JPRB, 0.21977E+01_JPRB, 0.18416E+01_JPRB, 0.27204E+01_JPRB, 0.34724E+01_JPRB ,&
&  0.39855E+01_JPRB, 0.45030E+01_JPRB, 0.45683E+01_JPRB, 0.54104E+01_JPRB, 0.58331E+01_JPRB, 0.51858E+01_JPRB, 0.43380E+01_JPRB ,&
&  0.36699E+01_JPRB, 0.28853E+01_JPRB, 0.26165E+01_JPRB, 0.27043E+01_JPRB, 0.28171E+01_JPRB, 0.32279E+01_JPRB, 0.25200E+01_JPRB ,&
&  0.23737E+01_JPRB, 0.27962E+01_JPRB, 0.25356E+01_JPRB, 0.21072E+01_JPRB, 0.22221E+01_JPRB, 0.20856E+01_JPRB, 0.18445E+01_JPRB ,&
&  0.85778E+00_JPRB, 0.56152E+00_JPRB, 0.51440E+00_JPRB, 0.46507E+00_JPRB, 0.40655E+00_JPRB, 0.38423E+00_JPRB, 0.69995E+00_JPRB ,&
&  0.66703E+00_JPRB, 0.11123E+01_JPRB, 0.14082E+01_JPRB, 0.13983E+01_JPRB, 0.16519E+01_JPRB, 0.18968E+01_JPRB, 0.24700E+01_JPRB ,&
&  0.13216E+01_JPRB, 0.98529E+00_JPRB, 0.71071E+00_JPRB, 0.60066E+00_JPRB, 0.58484E+00_JPRB, 0.57499E+00_JPRB, 0.55457E+00_JPRB ,&
&  0.53892E+00_JPRB, 0.52893E+00_JPRB, 0.52309E+00_JPRB, 0.50926E+00_JPRB, 0.53833E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.62868E+00_JPRB, 0.58005E+00_JPRB, 0.54580E+00_JPRB, 0.55213E+00_JPRB, 0.59729E+00_JPRB, 0.62345E+00_JPRB, 0.64614E+00_JPRB ,&
&  0.70456E+00_JPRB, 0.77072E+00_JPRB, 0.79481E+00_JPRB, 0.75540E+00_JPRB, 0.72354E+00_JPRB, 0.71968E+00_JPRB, 0.72804E+00_JPRB ,&
&  0.77872E+00_JPRB, 0.90890E+00_JPRB, 0.12471E+01_JPRB, 0.15136E+01_JPRB, 0.18514E+01_JPRB, 0.19085E+01_JPRB, 0.23087E+01_JPRB ,&
&  0.24602E+01_JPRB, 0.28766E+01_JPRB, 0.35490E+01_JPRB, 0.36880E+01_JPRB, 0.38787E+01_JPRB, 0.31530E+01_JPRB, 0.27683E+01_JPRB ,&
&  0.22793E+01_JPRB, 0.21033E+01_JPRB, 0.20716E+01_JPRB, 0.22467E+01_JPRB, 0.23868E+01_JPRB, 0.23701E+01_JPRB, 0.22287E+01_JPRB ,&
&  0.19081E+01_JPRB, 0.17044E+01_JPRB, 0.14348E+01_JPRB, 0.13445E+01_JPRB, 0.13031E+01_JPRB, 0.14066E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.20165E+01_JPRB, 0.23096E+01_JPRB, 0.22825E+01_JPRB, 0.22910E+01_JPRB, 0.21612E+01_JPRB, 0.23480E+01_JPRB, 0.35707E+01_JPRB ,&
&  0.46298E+01_JPRB, 0.54342E+01_JPRB, 0.56253E+01_JPRB, 0.66809E+01_JPRB, 0.64467E+01_JPRB, 0.54919E+01_JPRB, 0.49592E+01_JPRB ,&
&  0.41408E+01_JPRB, 0.36000E+01_JPRB, 0.28657E+01_JPRB, 0.28197E+01_JPRB, 0.29233E+01_JPRB, 0.25658E+01_JPRB, 0.26008E+01_JPRB ,&
&  0.29675E+01_JPRB, 0.28362E+01_JPRB, 0.25050E+01_JPRB, 0.20407E+01_JPRB, 0.23141E+01_JPRB, 0.18559E+01_JPRB, 0.99111E+00_JPRB ,&
&  0.78626E+00_JPRB, 0.68675E+00_JPRB, 0.53891E+00_JPRB, 0.46996E+00_JPRB, 0.41971E+00_JPRB, 0.42102E+00_JPRB, 0.64682E+00_JPRB ,&
&  0.83188E+00_JPRB, 0.13607E+01_JPRB, 0.16199E+01_JPRB, 0.16342E+01_JPRB, 0.17106E+01_JPRB, 0.21663E+01_JPRB, 0.25544E+01_JPRB ,&
&  0.17889E+01_JPRB, 0.14145E+01_JPRB, 0.81703E+00_JPRB, 0.64376E+00_JPRB, 0.60554E+00_JPRB, 0.58973E+00_JPRB, 0.56870E+00_JPRB ,&
&  0.54881E+00_JPRB, 0.55660E+00_JPRB, 0.56448E+00_JPRB, 0.56940E+00_JPRB, 0.58401E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.66725E+00_JPRB, 0.64264E+00_JPRB, 0.60175E+00_JPRB, 0.61575E+00_JPRB, 0.64338E+00_JPRB, 0.67759E+00_JPRB, 0.67375E+00_JPRB ,&
&  0.68748E+00_JPRB, 0.71926E+00_JPRB, 0.74465E+00_JPRB, 0.72726E+00_JPRB, 0.68251E+00_JPRB, 0.68846E+00_JPRB, 0.69628E+00_JPRB ,&
&  0.75020E+00_JPRB, 0.89397E+00_JPRB, 0.10821E+01_JPRB, 0.11768E+01_JPRB, 0.13121E+01_JPRB, 0.14391E+01_JPRB, 0.18690E+01_JPRB ,&
&  0.24311E+01_JPRB, 0.28440E+01_JPRB, 0.32977E+01_JPRB, 0.38316E+01_JPRB, 0.43457E+01_JPRB, 0.38173E+01_JPRB, 0.37422E+01_JPRB ,&
&  0.31026E+01_JPRB, 0.25802E+01_JPRB, 0.23988E+01_JPRB, 0.23904E+01_JPRB, 0.25432E+01_JPRB, 0.25977E+01_JPRB, 0.24737E+01_JPRB ,&
&  0.21321E+01_JPRB, 0.17548E+01_JPRB, 0.14997E+01_JPRB, 0.13359E+01_JPRB, 0.12587E+01_JPRB, 0.13057E+01_JPRB, 0.15411E+01_JPRB ,&
&  0.20999E+01_JPRB, 0.26672E+01_JPRB, 0.27148E+01_JPRB, 0.25021E+01_JPRB, 0.29023E+01_JPRB, 0.32092E+01_JPRB, 0.39303E+01_JPRB ,&
&  0.44131E+01_JPRB, 0.47687E+01_JPRB, 0.67059E+01_JPRB, 0.70868E+01_JPRB, 0.65184E+01_JPRB, 0.58046E+01_JPRB, 0.50101E+01_JPRB ,&
&  0.43752E+01_JPRB, 0.39625E+01_JPRB, 0.32300E+01_JPRB, 0.31623E+01_JPRB, 0.27852E+01_JPRB, 0.25614E+01_JPRB, 0.30611E+01_JPRB ,&
&  0.32408E+01_JPRB, 0.30877E+01_JPRB, 0.25692E+01_JPRB, 0.20560E+01_JPRB, 0.19293E+01_JPRB, 0.13283E+01_JPRB, 0.87838E+00_JPRB ,&
&  0.75314E+00_JPRB, 0.70501E+00_JPRB, 0.58626E+00_JPRB, 0.61459E+00_JPRB, 0.54403E+00_JPRB, 0.57001E+00_JPRB, 0.74801E+00_JPRB ,&
&  0.10543E+01_JPRB, 0.14725E+01_JPRB, 0.16139E+01_JPRB, 0.15826E+01_JPRB, 0.16220E+01_JPRB, 0.17095E+01_JPRB, 0.23960E+01_JPRB ,&
&  0.16435E+01_JPRB, 0.16101E+01_JPRB, 0.11043E+01_JPRB, 0.79114E+00_JPRB, 0.66903E+00_JPRB, 0.62460E+00_JPRB, 0.61997E+00_JPRB ,&
&  0.62002E+00_JPRB, 0.61878E+00_JPRB, 0.62364E+00_JPRB, 0.62495E+00_JPRB, 0.63114E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.67747E+00_JPRB, 0.69144E+00_JPRB, 0.67843E+00_JPRB, 0.68789E+00_JPRB, 0.70830E+00_JPRB, 0.72448E+00_JPRB, 0.71534E+00_JPRB ,&
&  0.70011E+00_JPRB, 0.70976E+00_JPRB, 0.70534E+00_JPRB, 0.68371E+00_JPRB, 0.65888E+00_JPRB, 0.66449E+00_JPRB, 0.68214E+00_JPRB ,&
&  0.76006E+00_JPRB, 0.97777E+00_JPRB, 0.81526E+00_JPRB, 0.83021E+00_JPRB, 0.96981E+00_JPRB, 0.11504E+01_JPRB, 0.13998E+01_JPRB ,&
&  0.21538E+01_JPRB, 0.27926E+01_JPRB, 0.32473E+01_JPRB, 0.41189E+01_JPRB, 0.53948E+01_JPRB, 0.51362E+01_JPRB, 0.48236E+01_JPRB ,&
&  0.41063E+01_JPRB, 0.33799E+01_JPRB, 0.30963E+01_JPRB, 0.29070E+01_JPRB, 0.28619E+01_JPRB, 0.28216E+01_JPRB, 0.26383E+01_JPRB ,&
&  0.22891E+01_JPRB, 0.18245E+01_JPRB, 0.14882E+01_JPRB, 0.14061E+01_JPRB, 0.13365E+01_JPRB, 0.13820E+01_JPRB, 0.15515E+01_JPRB ,&
&  0.21720E+01_JPRB, 0.29308E+01_JPRB, 0.32213E+01_JPRB, 0.31748E+01_JPRB, 0.29106E+01_JPRB, 0.35949E+01_JPRB, 0.58460E+01_JPRB ,&
&  0.59708E+01_JPRB, 0.63474E+01_JPRB, 0.68879E+01_JPRB, 0.79705E+01_JPRB, 0.66883E+01_JPRB, 0.52549E+01_JPRB, 0.49944E+01_JPRB ,&
&  0.36697E+01_JPRB, 0.30244E+01_JPRB, 0.31496E+01_JPRB, 0.31835E+01_JPRB, 0.28011E+01_JPRB, 0.31502E+01_JPRB, 0.43789E+01_JPRB ,&
&  0.35405E+01_JPRB, 0.36435E+01_JPRB, 0.36247E+01_JPRB, 0.34645E+01_JPRB, 0.20122E+01_JPRB, 0.15411E+01_JPRB, 0.21919E+01_JPRB ,&
&  0.23077E+01_JPRB, 0.14034E+01_JPRB, 0.86917E+00_JPRB, 0.11608E+01_JPRB, 0.82381E+00_JPRB, 0.86774E+00_JPRB, 0.10768E+01_JPRB ,&
&  0.11914E+01_JPRB, 0.13480E+01_JPRB, 0.18397E+01_JPRB, 0.17620E+01_JPRB, 0.15987E+01_JPRB, 0.15466E+01_JPRB, 0.18636E+01_JPRB ,&
&  0.18095E+01_JPRB, 0.16792E+01_JPRB, 0.14006E+01_JPRB, 0.10553E+01_JPRB, 0.84359E+00_JPRB, 0.75604E+00_JPRB, 0.73124E+00_JPRB ,&
&  0.72745E+00_JPRB, 0.74555E+00_JPRB, 0.73594E+00_JPRB, 0.70291E+00_JPRB, 0.68543E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.76172E+00_JPRB, 0.76845E+00_JPRB, 0.76141E+00_JPRB, 0.77418E+00_JPRB, 0.79514E+00_JPRB, 0.76440E+00_JPRB, 0.74433E+00_JPRB ,&
&  0.71479E+00_JPRB, 0.69673E+00_JPRB, 0.69525E+00_JPRB, 0.69089E+00_JPRB, 0.67052E+00_JPRB, 0.66744E+00_JPRB, 0.68253E+00_JPRB ,&
&  0.75937E+00_JPRB, 0.82796E+00_JPRB, 0.75030E+00_JPRB, 0.74073E+00_JPRB, 0.88353E+00_JPRB, 0.10047E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.19303E+01_JPRB, 0.26590E+01_JPRB, 0.34436E+01_JPRB, 0.43850E+01_JPRB, 0.51186E+01_JPRB, 0.57255E+01_JPRB, 0.59348E+01_JPRB ,&
&  0.56931E+01_JPRB, 0.47290E+01_JPRB, 0.41275E+01_JPRB, 0.36895E+01_JPRB, 0.33033E+01_JPRB, 0.29882E+01_JPRB, 0.27309E+01_JPRB ,&
&  0.25141E+01_JPRB, 0.21397E+01_JPRB, 0.17310E+01_JPRB, 0.14774E+01_JPRB, 0.14299E+01_JPRB, 0.14351E+01_JPRB, 0.15723E+01_JPRB ,&
&  0.21259E+01_JPRB, 0.28780E+01_JPRB, 0.36646E+01_JPRB, 0.40018E+01_JPRB, 0.34613E+01_JPRB, 0.38846E+01_JPRB, 0.54810E+01_JPRB ,&
&  0.63631E+01_JPRB, 0.63955E+01_JPRB, 0.73505E+01_JPRB, 0.78241E+01_JPRB, 0.69789E+01_JPRB, 0.55647E+01_JPRB, 0.55682E+01_JPRB ,&
&  0.42507E+01_JPRB, 0.33617E+01_JPRB, 0.29775E+01_JPRB, 0.27778E+01_JPRB, 0.32712E+01_JPRB, 0.45515E+01_JPRB, 0.42701E+01_JPRB ,&
&  0.40551E+01_JPRB, 0.38350E+01_JPRB, 0.35173E+01_JPRB, 0.30028E+01_JPRB, 0.27132E+01_JPRB, 0.24334E+01_JPRB, 0.29965E+01_JPRB ,&
&  0.29294E+01_JPRB, 0.27602E+01_JPRB, 0.22687E+01_JPRB, 0.18334E+01_JPRB, 0.14247E+01_JPRB, 0.15023E+01_JPRB, 0.14524E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.13176E+01_JPRB, 0.17289E+01_JPRB, 0.17986E+01_JPRB, 0.15503E+01_JPRB, 0.13413E+01_JPRB, 0.15947E+01_JPRB ,&
&  0.15086E+01_JPRB, 0.16248E+01_JPRB, 0.14200E+01_JPRB, 0.11755E+01_JPRB, 0.10320E+01_JPRB, 0.96305E+00_JPRB, 0.91362E+00_JPRB ,&
&  0.91522E+00_JPRB, 0.92286E+00_JPRB, 0.90043E+00_JPRB, 0.84137E+00_JPRB, 0.80047E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.86848E+00_JPRB, 0.86255E+00_JPRB, 0.85365E+00_JPRB, 0.85350E+00_JPRB, 0.85887E+00_JPRB, 0.81495E+00_JPRB, 0.77416E+00_JPRB ,&
&  0.74055E+00_JPRB, 0.71612E+00_JPRB, 0.71459E+00_JPRB, 0.70282E+00_JPRB, 0.68461E+00_JPRB, 0.67524E+00_JPRB, 0.68995E+00_JPRB ,&
&  0.76462E+00_JPRB, 0.83324E+00_JPRB, 0.74124E+00_JPRB, 0.79750E+00_JPRB, 0.86447E+00_JPRB, 0.98143E+00_JPRB, 0.13922E+01_JPRB ,&
&  0.18313E+01_JPRB, 0.23795E+01_JPRB, 0.31136E+01_JPRB, 0.39655E+01_JPRB, 0.46501E+01_JPRB, 0.54318E+01_JPRB, 0.62206E+01_JPRB ,&
&  0.58145E+01_JPRB, 0.57181E+01_JPRB, 0.49658E+01_JPRB, 0.42046E+01_JPRB, 0.35655E+01_JPRB, 0.30798E+01_JPRB, 0.28263E+01_JPRB ,&
&  0.26243E+01_JPRB, 0.24822E+01_JPRB, 0.21904E+01_JPRB, 0.18578E+01_JPRB, 0.16739E+01_JPRB, 0.16050E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.18247E+01_JPRB, 0.25964E+01_JPRB, 0.37890E+01_JPRB, 0.48993E+01_JPRB, 0.40854E+01_JPRB, 0.46566E+01_JPRB, 0.53462E+01_JPRB ,&
&  0.77314E+01_JPRB, 0.80324E+01_JPRB, 0.75413E+01_JPRB, 0.76103E+01_JPRB, 0.57179E+01_JPRB, 0.53538E+01_JPRB, 0.54118E+01_JPRB ,&
&  0.51432E+01_JPRB, 0.47482E+01_JPRB, 0.49831E+01_JPRB, 0.39945E+01_JPRB, 0.35639E+01_JPRB, 0.42632E+01_JPRB, 0.36861E+01_JPRB ,&
&  0.33590E+01_JPRB, 0.31901E+01_JPRB, 0.29463E+01_JPRB, 0.30577E+01_JPRB, 0.28087E+01_JPRB, 0.27717E+01_JPRB, 0.23102E+01_JPRB ,&
&  0.22059E+01_JPRB, 0.18530E+01_JPRB, 0.20090E+01_JPRB, 0.17719E+01_JPRB, 0.16291E+01_JPRB, 0.15566E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.15229E+01_JPRB, 0.14719E+01_JPRB, 0.16511E+01_JPRB, 0.17740E+01_JPRB, 0.15744E+01_JPRB, 0.13176E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.14379E+01_JPRB, 0.13043E+01_JPRB, 0.11748E+01_JPRB, 0.11093E+01_JPRB, 0.10827E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.99452E+00_JPRB, 0.97260E+00_JPRB, 0.94702E+00_JPRB, 0.91480E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.89024E+00_JPRB, 0.87414E+00_JPRB, 0.91759E+00_JPRB, 0.89552E+00_JPRB, 0.89970E+00_JPRB, 0.85771E+00_JPRB, 0.82360E+00_JPRB ,&
&  0.79148E+00_JPRB, 0.76881E+00_JPRB, 0.74469E+00_JPRB, 0.71917E+00_JPRB, 0.69459E+00_JPRB, 0.67715E+00_JPRB, 0.68283E+00_JPRB ,&
&  0.71919E+00_JPRB, 0.90086E+00_JPRB, 0.81976E+00_JPRB, 0.78042E+00_JPRB, 0.81551E+00_JPRB, 0.10432E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.17302E+01_JPRB, 0.20532E+01_JPRB, 0.26247E+01_JPRB, 0.31951E+01_JPRB, 0.38169E+01_JPRB, 0.44706E+01_JPRB, 0.52649E+01_JPRB ,&
&  0.58383E+01_JPRB, 0.49335E+01_JPRB, 0.46188E+01_JPRB, 0.40352E+01_JPRB, 0.35524E+01_JPRB, 0.31431E+01_JPRB, 0.28592E+01_JPRB ,&
&  0.26308E+01_JPRB, 0.24692E+01_JPRB, 0.22635E+01_JPRB, 0.20230E+01_JPRB, 0.17834E+01_JPRB, 0.16439E+01_JPRB, 0.16069E+01_JPRB ,&
&  0.16885E+01_JPRB, 0.20881E+01_JPRB, 0.31298E+01_JPRB, 0.46840E+01_JPRB, 0.66843E+01_JPRB, 0.71933E+01_JPRB, 0.69447E+01_JPRB ,&
&  0.60898E+01_JPRB, 0.60234E+01_JPRB, 0.81733E+01_JPRB, 0.67237E+01_JPRB, 0.63561E+01_JPRB, 0.61699E+01_JPRB, 0.56633E+01_JPRB ,&
&  0.52719E+01_JPRB, 0.44589E+01_JPRB, 0.42276E+01_JPRB, 0.36547E+01_JPRB, 0.40892E+01_JPRB, 0.35657E+01_JPRB, 0.31198E+01_JPRB ,&
&  0.28180E+01_JPRB, 0.27965E+01_JPRB, 0.26848E+01_JPRB, 0.26391E+01_JPRB, 0.27272E+01_JPRB, 0.30623E+01_JPRB, 0.30353E+01_JPRB ,&
&  0.26757E+01_JPRB, 0.27074E+01_JPRB, 0.22295E+01_JPRB, 0.17800E+01_JPRB, 0.14447E+01_JPRB, 0.14181E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.14636E+01_JPRB, 0.15121E+01_JPRB, 0.16606E+01_JPRB, 0.16635E+01_JPRB, 0.14929E+01_JPRB, 0.13304E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.12422E+01_JPRB, 0.12981E+01_JPRB, 0.12901E+01_JPRB, 0.11377E+01_JPRB, 0.11312E+01_JPRB, 0.11237E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.98754E+00_JPRB, 0.96078E+00_JPRB, 0.94832E+00_JPRB, 0.93377E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.91474E+00_JPRB, 0.87384E+00_JPRB, 0.91292E+00_JPRB, 0.89668E+00_JPRB, 0.87938E+00_JPRB, 0.85227E+00_JPRB, 0.82450E+00_JPRB ,&
&  0.79574E+00_JPRB, 0.78021E+00_JPRB, 0.76376E+00_JPRB, 0.73091E+00_JPRB, 0.70324E+00_JPRB, 0.68598E+00_JPRB, 0.68499E+00_JPRB ,&
&  0.69536E+00_JPRB, 0.91033E+00_JPRB, 0.88325E+00_JPRB, 0.93298E+00_JPRB, 0.94316E+00_JPRB, 0.11210E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.15260E+01_JPRB, 0.17110E+01_JPRB, 0.19420E+01_JPRB, 0.22785E+01_JPRB, 0.26451E+01_JPRB, 0.30445E+01_JPRB, 0.36720E+01_JPRB ,&
&  0.38751E+01_JPRB, 0.38361E+01_JPRB, 0.36198E+01_JPRB, 0.36861E+01_JPRB, 0.35200E+01_JPRB, 0.30216E+01_JPRB, 0.28275E+01_JPRB ,&
&  0.25260E+01_JPRB, 0.23151E+01_JPRB, 0.21493E+01_JPRB, 0.20439E+01_JPRB, 0.19432E+01_JPRB, 0.18152E+01_JPRB, 0.17277E+01_JPRB ,&
&  0.16587E+01_JPRB, 0.17460E+01_JPRB, 0.22484E+01_JPRB, 0.35013E+01_JPRB, 0.55540E+01_JPRB, 0.70845E+01_JPRB, 0.73145E+01_JPRB ,&
&  0.78243E+01_JPRB, 0.74861E+01_JPRB, 0.56693E+01_JPRB, 0.61413E+01_JPRB, 0.73142E+01_JPRB, 0.64964E+01_JPRB, 0.55891E+01_JPRB ,&
&  0.55418E+01_JPRB, 0.51775E+01_JPRB, 0.49597E+01_JPRB, 0.43220E+01_JPRB, 0.36913E+01_JPRB, 0.33422E+01_JPRB, 0.30610E+01_JPRB ,&
&  0.28202E+01_JPRB, 0.27633E+01_JPRB, 0.27537E+01_JPRB, 0.27188E+01_JPRB, 0.25816E+01_JPRB, 0.25814E+01_JPRB, 0.27698E+01_JPRB ,&
&  0.27021E+01_JPRB, 0.24528E+01_JPRB, 0.18304E+01_JPRB, 0.17620E+01_JPRB, 0.12948E+01_JPRB, 0.12377E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.13191E+01_JPRB, 0.14536E+01_JPRB, 0.15490E+01_JPRB, 0.13588E+01_JPRB, 0.12659E+01_JPRB, 0.12044E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.11331E+01_JPRB, 0.12167E+01_JPRB, 0.11053E+01_JPRB, 0.10972E+01_JPRB, 0.10941E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.10536E+01_JPRB, 0.10202E+01_JPRB, 0.99039E+00_JPRB, 0.98272E+00_JPRB, 0.96163E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.92705E+00_JPRB, 0.88610E+00_JPRB, 0.89614E+00_JPRB, 0.85829E+00_JPRB, 0.82276E+00_JPRB, 0.79294E+00_JPRB, 0.77203E+00_JPRB ,&
&  0.75859E+00_JPRB, 0.74567E+00_JPRB, 0.73202E+00_JPRB, 0.71055E+00_JPRB, 0.69193E+00_JPRB, 0.67438E+00_JPRB, 0.66897E+00_JPRB ,&
&  0.68639E+00_JPRB, 0.77974E+00_JPRB, 0.83174E+00_JPRB, 0.83098E+00_JPRB, 0.10118E+01_JPRB, 0.10444E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.13565E+01_JPRB, 0.14067E+01_JPRB, 0.14704E+01_JPRB, 0.16068E+01_JPRB, 0.19021E+01_JPRB, 0.22899E+01_JPRB ,&
&  0.24351E+01_JPRB, 0.23567E+01_JPRB, 0.23611E+01_JPRB, 0.24807E+01_JPRB, 0.26649E+01_JPRB, 0.23545E+01_JPRB, 0.22660E+01_JPRB ,&
&  0.20847E+01_JPRB, 0.19348E+01_JPRB, 0.18255E+01_JPRB, 0.17560E+01_JPRB, 0.17433E+01_JPRB, 0.17560E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.17650E+01_JPRB, 0.16981E+01_JPRB, 0.18110E+01_JPRB, 0.26319E+01_JPRB, 0.50245E+01_JPRB, 0.77806E+01_JPRB, 0.96061E+01_JPRB ,&
&  0.89959E+01_JPRB, 0.84825E+01_JPRB, 0.83769E+01_JPRB, 0.84245E+01_JPRB, 0.74801E+01_JPRB, 0.64160E+01_JPRB, 0.57600E+01_JPRB ,&
&  0.53551E+01_JPRB, 0.51417E+01_JPRB, 0.50983E+01_JPRB, 0.46719E+01_JPRB, 0.41409E+01_JPRB, 0.38695E+01_JPRB, 0.36118E+01_JPRB ,&
&  0.33553E+01_JPRB, 0.32887E+01_JPRB, 0.32563E+01_JPRB, 0.30464E+01_JPRB, 0.29111E+01_JPRB, 0.27140E+01_JPRB, 0.28088E+01_JPRB ,&
&  0.29919E+01_JPRB, 0.22076E+01_JPRB, 0.17905E+01_JPRB, 0.19821E+01_JPRB, 0.14774E+01_JPRB, 0.14219E+01_JPRB, 0.16145E+01_JPRB ,&
&  0.14634E+01_JPRB, 0.13770E+01_JPRB, 0.13709E+01_JPRB, 0.11958E+01_JPRB, 0.11400E+01_JPRB, 0.11610E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.11303E+01_JPRB, 0.10993E+01_JPRB, 0.11894E+01_JPRB, 0.10571E+01_JPRB, 0.10273E+01_JPRB, 0.10282E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.97833E+00_JPRB, 0.94804E+00_JPRB, 0.93596E+00_JPRB, 0.94260E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.86655E+00_JPRB, 0.85776E+00_JPRB, 0.85660E+00_JPRB, 0.84707E+00_JPRB, 0.81888E+00_JPRB, 0.78556E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.74120E+00_JPRB, 0.72675E+00_JPRB, 0.70877E+00_JPRB, 0.68606E+00_JPRB, 0.66737E+00_JPRB, 0.65060E+00_JPRB, 0.65275E+00_JPRB ,&
&  0.58160E+00_JPRB, 0.60956E+00_JPRB, 0.63852E+00_JPRB, 0.71402E+00_JPRB, 0.92315E+00_JPRB, 0.90831E+00_JPRB, 0.94539E+00_JPRB ,&
&  0.99347E+00_JPRB, 0.10500E+01_JPRB, 0.10727E+01_JPRB, 0.11135E+01_JPRB, 0.11934E+01_JPRB, 0.13648E+01_JPRB, 0.15778E+01_JPRB ,&
&  0.15727E+01_JPRB, 0.14244E+01_JPRB, 0.13856E+01_JPRB, 0.13573E+01_JPRB, 0.14192E+01_JPRB, 0.14946E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.15728E+01_JPRB, 0.15257E+01_JPRB, 0.14968E+01_JPRB, 0.14805E+01_JPRB, 0.14712E+01_JPRB, 0.14813E+01_JPRB, 0.15085E+01_JPRB ,&
&  0.15418E+01_JPRB, 0.15360E+01_JPRB, 0.15668E+01_JPRB, 0.19961E+01_JPRB, 0.40515E+01_JPRB, 0.77996E+01_JPRB, 0.11980E+02_JPRB ,&
&  0.10560E+02_JPRB, 0.91014E+01_JPRB, 0.87271E+01_JPRB, 0.87250E+01_JPRB, 0.75621E+01_JPRB, 0.62658E+01_JPRB, 0.52634E+01_JPRB ,&
&  0.47712E+01_JPRB, 0.44444E+01_JPRB, 0.44659E+01_JPRB, 0.44602E+01_JPRB, 0.42067E+01_JPRB, 0.41145E+01_JPRB, 0.40010E+01_JPRB ,&
&  0.38235E+01_JPRB, 0.37397E+01_JPRB, 0.35386E+01_JPRB, 0.33369E+01_JPRB, 0.32149E+01_JPRB, 0.32600E+01_JPRB, 0.31971E+01_JPRB ,&
&  0.32063E+01_JPRB, 0.30043E+01_JPRB, 0.25814E+01_JPRB, 0.25090E+01_JPRB, 0.19667E+01_JPRB, 0.18680E+01_JPRB, 0.21696E+01_JPRB ,&
&  0.17911E+01_JPRB, 0.14689E+01_JPRB, 0.13323E+01_JPRB, 0.12614E+01_JPRB, 0.12244E+01_JPRB, 0.12433E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.11283E+01_JPRB, 0.11580E+01_JPRB, 0.98685E+00_JPRB, 0.92904E+00_JPRB, 0.90501E+00_JPRB, 0.84786E+00_JPRB ,&
&  0.87868E+00_JPRB, 0.86296E+00_JPRB, 0.85187E+00_JPRB, 0.84759E+00_JPRB, 0.85633E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.78914E+00_JPRB, 0.79668E+00_JPRB, 0.80216E+00_JPRB, 0.79811E+00_JPRB, 0.77844E+00_JPRB, 0.75467E+00_JPRB, 0.73704E+00_JPRB ,&
&  0.72552E+00_JPRB, 0.70386E+00_JPRB, 0.67953E+00_JPRB, 0.65240E+00_JPRB, 0.63436E+00_JPRB, 0.62938E+00_JPRB, 0.58800E+00_JPRB ,&
&  0.53779E+00_JPRB, 0.55542E+00_JPRB, 0.56544E+00_JPRB, 0.60642E+00_JPRB, 0.67253E+00_JPRB, 0.70010E+00_JPRB, 0.73499E+00_JPRB ,&
&  0.81787E+00_JPRB, 0.88356E+00_JPRB, 0.92629E+00_JPRB, 0.97141E+00_JPRB, 0.10049E+01_JPRB, 0.10466E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10479E+01_JPRB, 0.95432E+00_JPRB, 0.88605E+00_JPRB, 0.83097E+00_JPRB, 0.87452E+00_JPRB, 0.95489E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10717E+01_JPRB, 0.11012E+01_JPRB, 0.11232E+01_JPRB, 0.11320E+01_JPRB, 0.11479E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.11990E+01_JPRB, 0.12281E+01_JPRB, 0.12682E+01_JPRB, 0.14623E+01_JPRB, 0.29533E+01_JPRB, 0.58280E+01_JPRB, 0.80717E+01_JPRB ,&
&  0.70537E+01_JPRB, 0.66969E+01_JPRB, 0.64312E+01_JPRB, 0.61605E+01_JPRB, 0.56730E+01_JPRB, 0.49047E+01_JPRB, 0.42742E+01_JPRB ,&
&  0.40609E+01_JPRB, 0.37657E+01_JPRB, 0.38694E+01_JPRB, 0.37669E+01_JPRB, 0.37181E+01_JPRB, 0.36872E+01_JPRB, 0.35243E+01_JPRB ,&
&  0.34495E+01_JPRB, 0.32822E+01_JPRB, 0.32337E+01_JPRB, 0.31020E+01_JPRB, 0.30435E+01_JPRB, 0.30621E+01_JPRB, 0.30474E+01_JPRB ,&
&  0.30522E+01_JPRB, 0.30304E+01_JPRB, 0.27935E+01_JPRB, 0.26202E+01_JPRB, 0.24339E+01_JPRB, 0.25395E+01_JPRB, 0.22998E+01_JPRB ,&
&  0.19167E+01_JPRB, 0.14711E+01_JPRB, 0.14071E+01_JPRB, 0.13037E+01_JPRB, 0.12309E+01_JPRB, 0.11770E+01_JPRB, 0.11136E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.11912E+01_JPRB, 0.10993E+01_JPRB, 0.93687E+00_JPRB, 0.86104E+00_JPRB, 0.81331E+00_JPRB, 0.71755E+00_JPRB ,&
&  0.70817E+00_JPRB, 0.76773E+00_JPRB, 0.77604E+00_JPRB, 0.77562E+00_JPRB, 0.77587E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.75329E+00_JPRB, 0.75699E+00_JPRB, 0.75500E+00_JPRB, 0.75480E+00_JPRB, 0.75026E+00_JPRB, 0.74084E+00_JPRB, 0.70690E+00_JPRB ,&
&  0.72631E+00_JPRB, 0.70581E+00_JPRB, 0.67196E+00_JPRB, 0.64664E+00_JPRB, 0.63623E+00_JPRB, 0.59136E+00_JPRB, 0.51108E+00_JPRB ,&
&  0.49155E+00_JPRB, 0.50323E+00_JPRB, 0.52752E+00_JPRB, 0.55057E+00_JPRB, 0.57144E+00_JPRB, 0.58475E+00_JPRB, 0.61052E+00_JPRB ,&
&  0.64701E+00_JPRB, 0.68221E+00_JPRB, 0.72461E+00_JPRB, 0.76434E+00_JPRB, 0.76666E+00_JPRB, 0.76161E+00_JPRB, 0.77303E+00_JPRB ,&
&  0.74284E+00_JPRB, 0.69867E+00_JPRB, 0.67392E+00_JPRB, 0.61466E+00_JPRB, 0.65808E+00_JPRB, 0.68032E+00_JPRB, 0.70419E+00_JPRB ,&
&  0.73344E+00_JPRB, 0.76122E+00_JPRB, 0.79386E+00_JPRB, 0.81923E+00_JPRB, 0.83491E+00_JPRB, 0.85414E+00_JPRB, 0.87505E+00_JPRB ,&
&  0.89902E+00_JPRB, 0.93773E+00_JPRB, 0.99698E+00_JPRB, 0.11006E+01_JPRB, 0.19440E+01_JPRB, 0.37320E+01_JPRB, 0.48900E+01_JPRB ,&
&  0.45514E+01_JPRB, 0.48263E+01_JPRB, 0.48463E+01_JPRB, 0.42347E+01_JPRB, 0.41877E+01_JPRB, 0.41314E+01_JPRB, 0.38126E+01_JPRB ,&
&  0.38031E+01_JPRB, 0.36422E+01_JPRB, 0.34199E+01_JPRB, 0.32205E+01_JPRB, 0.30551E+01_JPRB, 0.28798E+01_JPRB, 0.27265E+01_JPRB ,&
&  0.26666E+01_JPRB, 0.27209E+01_JPRB, 0.27480E+01_JPRB, 0.28251E+01_JPRB, 0.28333E+01_JPRB, 0.28647E+01_JPRB, 0.28914E+01_JPRB ,&
&  0.28813E+01_JPRB, 0.28782E+01_JPRB, 0.28254E+01_JPRB, 0.26905E+01_JPRB, 0.25445E+01_JPRB, 0.23861E+01_JPRB, 0.22054E+01_JPRB ,&
&  0.20411E+01_JPRB, 0.17701E+01_JPRB, 0.15583E+01_JPRB, 0.14508E+01_JPRB, 0.13239E+01_JPRB, 0.11802E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.99668E+00_JPRB, 0.97474E+00_JPRB, 0.86936E+00_JPRB, 0.79854E+00_JPRB, 0.75842E+00_JPRB, 0.68606E+00_JPRB ,&
&  0.65710E+00_JPRB, 0.70628E+00_JPRB, 0.73674E+00_JPRB, 0.75233E+00_JPRB, 0.75296E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.73303E+00_JPRB, 0.73783E+00_JPRB, 0.73884E+00_JPRB, 0.73971E+00_JPRB, 0.72673E+00_JPRB, 0.68946E+00_JPRB, 0.65693E+00_JPRB ,&
&  0.70924E+00_JPRB, 0.66859E+00_JPRB, 0.67587E+00_JPRB, 0.57291E+00_JPRB, 0.52869E+00_JPRB, 0.51678E+00_JPRB, 0.49599E+00_JPRB ,&
&  0.48871E+00_JPRB, 0.49447E+00_JPRB, 0.50032E+00_JPRB, 0.50876E+00_JPRB, 0.52098E+00_JPRB, 0.52547E+00_JPRB, 0.53020E+00_JPRB ,&
&  0.54034E+00_JPRB, 0.55558E+00_JPRB, 0.58411E+00_JPRB, 0.59945E+00_JPRB, 0.60854E+00_JPRB, 0.61348E+00_JPRB, 0.62185E+00_JPRB ,&
&  0.59031E+00_JPRB, 0.57157E+00_JPRB, 0.57375E+00_JPRB, 0.56502E+00_JPRB, 0.56222E+00_JPRB, 0.56760E+00_JPRB, 0.58205E+00_JPRB ,&
&  0.59963E+00_JPRB, 0.57308E+00_JPRB, 0.62723E+00_JPRB, 0.65665E+00_JPRB, 0.68026E+00_JPRB, 0.70035E+00_JPRB, 0.71965E+00_JPRB ,&
&  0.73935E+00_JPRB, 0.76582E+00_JPRB, 0.80727E+00_JPRB, 0.89194E+00_JPRB, 0.13011E+01_JPRB, 0.23790E+01_JPRB, 0.30441E+01_JPRB ,&
&  0.32750E+01_JPRB, 0.25344E+01_JPRB, 0.28977E+01_JPRB, 0.30244E+01_JPRB, 0.31990E+01_JPRB, 0.32579E+01_JPRB, 0.32183E+01_JPRB ,&
&  0.33662E+01_JPRB, 0.33347E+01_JPRB, 0.30571E+01_JPRB, 0.26749E+01_JPRB, 0.23801E+01_JPRB, 0.22145E+01_JPRB, 0.21191E+01_JPRB ,&
&  0.20691E+01_JPRB, 0.19688E+01_JPRB, 0.21833E+01_JPRB, 0.23857E+01_JPRB, 0.24895E+01_JPRB, 0.25758E+01_JPRB, 0.26821E+01_JPRB ,&
&  0.27422E+01_JPRB, 0.27089E+01_JPRB, 0.26560E+01_JPRB, 0.24499E+01_JPRB, 0.23342E+01_JPRB, 0.22430E+01_JPRB, 0.21078E+01_JPRB ,&
&  0.20172E+01_JPRB, 0.19383E+01_JPRB, 0.18283E+01_JPRB, 0.18010E+01_JPRB, 0.15624E+01_JPRB, 0.13960E+01_JPRB, 0.12753E+01_JPRB ,&
&  0.10740E+01_JPRB, 0.85937E+00_JPRB, 0.77198E+00_JPRB, 0.72107E+00_JPRB, 0.68064E+00_JPRB, 0.67407E+00_JPRB, 0.67980E+00_JPRB ,&
&  0.65875E+00_JPRB, 0.67044E+00_JPRB, 0.71326E+00_JPRB, 0.72364E+00_JPRB, 0.72593E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.71386E+00_JPRB, 0.72998E+00_JPRB, 0.74202E+00_JPRB, 0.73848E+00_JPRB, 0.71252E+00_JPRB, 0.66406E+00_JPRB, 0.60542E+00_JPRB ,&
&  0.58726E+00_JPRB, 0.56177E+00_JPRB, 0.53791E+00_JPRB, 0.51948E+00_JPRB, 0.50715E+00_JPRB, 0.49156E+00_JPRB, 0.47276E+00_JPRB ,&
&  0.47037E+00_JPRB, 0.48037E+00_JPRB, 0.49477E+00_JPRB, 0.49702E+00_JPRB, 0.49496E+00_JPRB, 0.49319E+00_JPRB, 0.49036E+00_JPRB ,&
&  0.48675E+00_JPRB, 0.48930E+00_JPRB, 0.50153E+00_JPRB, 0.52110E+00_JPRB, 0.53098E+00_JPRB, 0.53199E+00_JPRB, 0.53345E+00_JPRB ,&
&  0.52617E+00_JPRB, 0.51561E+00_JPRB, 0.50956E+00_JPRB, 0.51066E+00_JPRB, 0.50746E+00_JPRB, 0.51791E+00_JPRB, 0.51969E+00_JPRB ,&
&  0.47204E+00_JPRB, 0.48608E+00_JPRB, 0.55638E+00_JPRB, 0.58521E+00_JPRB, 0.60866E+00_JPRB, 0.62798E+00_JPRB, 0.64660E+00_JPRB ,&
&  0.66593E+00_JPRB, 0.69597E+00_JPRB, 0.73919E+00_JPRB, 0.80225E+00_JPRB, 0.10254E+01_JPRB, 0.15646E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.24900E+01_JPRB, 0.21565E+01_JPRB, 0.20089E+01_JPRB, 0.21180E+01_JPRB, 0.25763E+01_JPRB, 0.26305E+01_JPRB, 0.26766E+01_JPRB ,&
&  0.27586E+01_JPRB, 0.26714E+01_JPRB, 0.25197E+01_JPRB, 0.23034E+01_JPRB, 0.20134E+01_JPRB, 0.18019E+01_JPRB, 0.16847E+01_JPRB ,&
&  0.15648E+01_JPRB, 0.15118E+01_JPRB, 0.15515E+01_JPRB, 0.17107E+01_JPRB, 0.19157E+01_JPRB, 0.20821E+01_JPRB, 0.21972E+01_JPRB ,&
&  0.23160E+01_JPRB, 0.23660E+01_JPRB, 0.23131E+01_JPRB, 0.22266E+01_JPRB, 0.20966E+01_JPRB, 0.20181E+01_JPRB, 0.19869E+01_JPRB ,&
&  0.19478E+01_JPRB, 0.19066E+01_JPRB, 0.18235E+01_JPRB, 0.17153E+01_JPRB, 0.15426E+01_JPRB, 0.13827E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.92146E+00_JPRB, 0.76575E+00_JPRB, 0.65824E+00_JPRB, 0.62110E+00_JPRB, 0.61277E+00_JPRB, 0.61415E+00_JPRB, 0.60987E+00_JPRB ,&
&  0.62691E+00_JPRB, 0.64019E+00_JPRB, 0.64241E+00_JPRB, 0.66233E+00_JPRB, 0.68767E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.69041E+00_JPRB, 0.70547E+00_JPRB, 0.70656E+00_JPRB, 0.70372E+00_JPRB, 0.66307E+00_JPRB, 0.62444E+00_JPRB, 0.58694E+00_JPRB ,&
&  0.56809E+00_JPRB, 0.54860E+00_JPRB, 0.53285E+00_JPRB, 0.51358E+00_JPRB, 0.49839E+00_JPRB, 0.49207E+00_JPRB, 0.49444E+00_JPRB ,&
&  0.50059E+00_JPRB, 0.50026E+00_JPRB, 0.49657E+00_JPRB, 0.48505E+00_JPRB, 0.47490E+00_JPRB, 0.46610E+00_JPRB, 0.45879E+00_JPRB ,&
&  0.45207E+00_JPRB, 0.45359E+00_JPRB, 0.46078E+00_JPRB, 0.47426E+00_JPRB, 0.48194E+00_JPRB, 0.48656E+00_JPRB, 0.48413E+00_JPRB ,&
&  0.47312E+00_JPRB, 0.45972E+00_JPRB, 0.46182E+00_JPRB, 0.46973E+00_JPRB, 0.47336E+00_JPRB, 0.47887E+00_JPRB, 0.45552E+00_JPRB ,&
&  0.43699E+00_JPRB, 0.43530E+00_JPRB, 0.45411E+00_JPRB, 0.53379E+00_JPRB, 0.56663E+00_JPRB, 0.58817E+00_JPRB, 0.61437E+00_JPRB ,&
&  0.61244E+00_JPRB, 0.60310E+00_JPRB, 0.64447E+00_JPRB, 0.74459E+00_JPRB, 0.87109E+00_JPRB, 0.10920E+01_JPRB, 0.13932E+01_JPRB ,&
&  0.16709E+01_JPRB, 0.18868E+01_JPRB, 0.20171E+01_JPRB, 0.18135E+01_JPRB, 0.18006E+01_JPRB, 0.20613E+01_JPRB, 0.21227E+01_JPRB ,&
&  0.21337E+01_JPRB, 0.20927E+01_JPRB, 0.21114E+01_JPRB, 0.19392E+01_JPRB, 0.16676E+01_JPRB, 0.14772E+01_JPRB, 0.13397E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.11074E+01_JPRB, 0.10728E+01_JPRB, 0.10801E+01_JPRB, 0.11344E+01_JPRB, 0.12694E+01_JPRB, 0.14493E+01_JPRB ,&
&  0.16227E+01_JPRB, 0.17233E+01_JPRB, 0.17724E+01_JPRB, 0.17366E+01_JPRB, 0.16970E+01_JPRB, 0.16481E+01_JPRB, 0.16166E+01_JPRB ,&
&  0.15810E+01_JPRB, 0.15686E+01_JPRB, 0.15493E+01_JPRB, 0.15209E+01_JPRB, 0.13756E+01_JPRB, 0.11477E+01_JPRB, 0.92907E+00_JPRB ,&
&  0.77712E+00_JPRB, 0.68791E+00_JPRB, 0.62647E+00_JPRB, 0.60845E+00_JPRB, 0.60617E+00_JPRB, 0.60291E+00_JPRB, 0.57887E+00_JPRB ,&
&  0.56848E+00_JPRB, 0.57893E+00_JPRB, 0.59828E+00_JPRB, 0.62081E+00_JPRB, 0.65369E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.67201E+00_JPRB, 0.67361E+00_JPRB, 0.65973E+00_JPRB, 0.63655E+00_JPRB, 0.61573E+00_JPRB, 0.57228E+00_JPRB, 0.53978E+00_JPRB ,&
&  0.52366E+00_JPRB, 0.51832E+00_JPRB, 0.51839E+00_JPRB, 0.51775E+00_JPRB, 0.51863E+00_JPRB, 0.52092E+00_JPRB, 0.51863E+00_JPRB ,&
&  0.51099E+00_JPRB, 0.50475E+00_JPRB, 0.49927E+00_JPRB, 0.48972E+00_JPRB, 0.47587E+00_JPRB, 0.45951E+00_JPRB, 0.44607E+00_JPRB ,&
&  0.43799E+00_JPRB, 0.43806E+00_JPRB, 0.44831E+00_JPRB, 0.45789E+00_JPRB, 0.46326E+00_JPRB, 0.46299E+00_JPRB, 0.45792E+00_JPRB ,&
&  0.44585E+00_JPRB, 0.43919E+00_JPRB, 0.44541E+00_JPRB, 0.44789E+00_JPRB, 0.45254E+00_JPRB, 0.45356E+00_JPRB, 0.43688E+00_JPRB ,&
&  0.42091E+00_JPRB, 0.40742E+00_JPRB, 0.40428E+00_JPRB, 0.41460E+00_JPRB, 0.45779E+00_JPRB, 0.52858E+00_JPRB, 0.55413E+00_JPRB ,&
&  0.57825E+00_JPRB, 0.60199E+00_JPRB, 0.63076E+00_JPRB, 0.67062E+00_JPRB, 0.72737E+00_JPRB, 0.80900E+00_JPRB, 0.95993E+00_JPRB ,&
&  0.11482E+01_JPRB, 0.13061E+01_JPRB, 0.14556E+01_JPRB, 0.13222E+01_JPRB, 0.12551E+01_JPRB, 0.12898E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.14196E+01_JPRB, 0.14271E+01_JPRB, 0.14364E+01_JPRB, 0.14627E+01_JPRB, 0.13747E+01_JPRB, 0.12592E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.94735E+00_JPRB, 0.90986E+00_JPRB, 0.86205E+00_JPRB, 0.84822E+00_JPRB, 0.87274E+00_JPRB, 0.92213E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.11232E+01_JPRB, 0.11775E+01_JPRB, 0.11237E+01_JPRB, 0.11463E+01_JPRB, 0.12061E+01_JPRB, 0.12587E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.12701E+01_JPRB, 0.12833E+01_JPRB, 0.12702E+01_JPRB, 0.11663E+01_JPRB, 0.10683E+01_JPRB, 0.95513E+00_JPRB ,&
&  0.84745E+00_JPRB, 0.75196E+00_JPRB, 0.69790E+00_JPRB, 0.67864E+00_JPRB, 0.66349E+00_JPRB, 0.64661E+00_JPRB, 0.62328E+00_JPRB ,&
&  0.60401E+00_JPRB, 0.60071E+00_JPRB, 0.61433E+00_JPRB, 0.63343E+00_JPRB, 0.65791E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.66503E+00_JPRB, 0.62541E+00_JPRB, 0.59808E+00_JPRB, 0.58484E+00_JPRB, 0.57618E+00_JPRB, 0.55734E+00_JPRB, 0.54372E+00_JPRB ,&
&  0.53569E+00_JPRB, 0.53853E+00_JPRB, 0.53409E+00_JPRB, 0.52736E+00_JPRB, 0.52395E+00_JPRB, 0.52135E+00_JPRB, 0.52020E+00_JPRB ,&
&  0.51809E+00_JPRB, 0.51205E+00_JPRB, 0.49921E+00_JPRB, 0.47964E+00_JPRB, 0.46487E+00_JPRB, 0.45373E+00_JPRB, 0.44651E+00_JPRB ,&
&  0.44378E+00_JPRB, 0.44605E+00_JPRB, 0.45450E+00_JPRB, 0.45627E+00_JPRB, 0.45126E+00_JPRB, 0.44374E+00_JPRB, 0.43493E+00_JPRB ,&
&  0.43445E+00_JPRB, 0.44480E+00_JPRB, 0.43611E+00_JPRB, 0.43782E+00_JPRB, 0.44700E+00_JPRB, 0.43709E+00_JPRB, 0.42049E+00_JPRB ,&
&  0.40567E+00_JPRB, 0.39147E+00_JPRB, 0.38224E+00_JPRB, 0.38001E+00_JPRB, 0.39352E+00_JPRB, 0.41460E+00_JPRB, 0.45512E+00_JPRB ,&
&  0.51775E+00_JPRB, 0.54926E+00_JPRB, 0.57051E+00_JPRB, 0.59601E+00_JPRB, 0.62251E+00_JPRB, 0.65473E+00_JPRB, 0.70857E+00_JPRB ,&
&  0.79211E+00_JPRB, 0.88304E+00_JPRB, 0.97072E+00_JPRB, 0.10327E+01_JPRB, 0.10808E+01_JPRB, 0.10246E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.11344E+01_JPRB, 0.11864E+01_JPRB, 0.11227E+01_JPRB, 0.10595E+01_JPRB, 0.98739E+00_JPRB, 0.92448E+00_JPRB, 0.87700E+00_JPRB ,&
&  0.84795E+00_JPRB, 0.82927E+00_JPRB, 0.80810E+00_JPRB, 0.76540E+00_JPRB, 0.72907E+00_JPRB, 0.71812E+00_JPRB, 0.72807E+00_JPRB ,&
&  0.76665E+00_JPRB, 0.91309E+00_JPRB, 0.12401E+01_JPRB, 0.10690E+01_JPRB, 0.99341E+00_JPRB, 0.98194E+00_JPRB, 0.98977E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.10525E+01_JPRB, 0.10731E+01_JPRB, 0.10584E+01_JPRB, 0.10191E+01_JPRB, 0.97033E+00_JPRB, 0.93466E+00_JPRB ,&
&  0.90439E+00_JPRB, 0.87338E+00_JPRB, 0.85046E+00_JPRB, 0.84016E+00_JPRB, 0.82162E+00_JPRB, 0.80056E+00_JPRB, 0.78129E+00_JPRB ,&
&  0.76324E+00_JPRB, 0.75122E+00_JPRB, 0.74311E+00_JPRB, 0.72863E+00_JPRB, 0.70733E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.59524E+00_JPRB, 0.57876E+00_JPRB, 0.56613E+00_JPRB, 0.55831E+00_JPRB, 0.55171E+00_JPRB, 0.54592E+00_JPRB, 0.54108E+00_JPRB ,&
&  0.53729E+00_JPRB, 0.53458E+00_JPRB, 0.53010E+00_JPRB, 0.52540E+00_JPRB, 0.52140E+00_JPRB, 0.51764E+00_JPRB, 0.51440E+00_JPRB ,&
&  0.50764E+00_JPRB, 0.48925E+00_JPRB, 0.48260E+00_JPRB, 0.47535E+00_JPRB, 0.46497E+00_JPRB, 0.45947E+00_JPRB, 0.45632E+00_JPRB ,&
&  0.45161E+00_JPRB, 0.44905E+00_JPRB, 0.44958E+00_JPRB, 0.44330E+00_JPRB, 0.43782E+00_JPRB, 0.43950E+00_JPRB, 0.44158E+00_JPRB ,&
&  0.43117E+00_JPRB, 0.43043E+00_JPRB, 0.44826E+00_JPRB, 0.44490E+00_JPRB, 0.42265E+00_JPRB, 0.41900E+00_JPRB, 0.40827E+00_JPRB ,&
&  0.39845E+00_JPRB, 0.37779E+00_JPRB, 0.37153E+00_JPRB, 0.36746E+00_JPRB, 0.37497E+00_JPRB, 0.38698E+00_JPRB, 0.41519E+00_JPRB ,&
&  0.44457E+00_JPRB, 0.49896E+00_JPRB, 0.52183E+00_JPRB, 0.53975E+00_JPRB, 0.55607E+00_JPRB, 0.57339E+00_JPRB, 0.59431E+00_JPRB ,&
&  0.62212E+00_JPRB, 0.65828E+00_JPRB, 0.70367E+00_JPRB, 0.74435E+00_JPRB, 0.77996E+00_JPRB, 0.79622E+00_JPRB, 0.80050E+00_JPRB ,&
&  0.82181E+00_JPRB, 0.81940E+00_JPRB, 0.79034E+00_JPRB, 0.76044E+00_JPRB, 0.72680E+00_JPRB, 0.69450E+00_JPRB, 0.66097E+00_JPRB ,&
&  0.65189E+00_JPRB, 0.66347E+00_JPRB, 0.65559E+00_JPRB, 0.64641E+00_JPRB, 0.63675E+00_JPRB, 0.63595E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.63434E+00_JPRB, 0.66338E+00_JPRB, 0.76377E+00_JPRB, 0.77898E+00_JPRB, 0.78986E+00_JPRB, 0.79694E+00_JPRB, 0.80348E+00_JPRB ,&
&  0.81677E+00_JPRB, 0.83607E+00_JPRB, 0.85682E+00_JPRB, 0.86878E+00_JPRB, 0.87914E+00_JPRB, 0.87428E+00_JPRB, 0.88360E+00_JPRB ,&
&  0.87736E+00_JPRB, 0.86715E+00_JPRB, 0.85238E+00_JPRB, 0.83983E+00_JPRB, 0.82428E+00_JPRB, 0.80672E+00_JPRB, 0.78202E+00_JPRB ,&
&  0.75282E+00_JPRB, 0.71971E+00_JPRB, 0.68569E+00_JPRB, 0.65207E+00_JPRB, 0.62032E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.54684E+00_JPRB, 0.53896E+00_JPRB, 0.53289E+00_JPRB, 0.52783E+00_JPRB, 0.52449E+00_JPRB, 0.52124E+00_JPRB, 0.51864E+00_JPRB ,&
&  0.51693E+00_JPRB, 0.51381E+00_JPRB, 0.51004E+00_JPRB, 0.50529E+00_JPRB, 0.50136E+00_JPRB, 0.49760E+00_JPRB, 0.49355E+00_JPRB ,&
&  0.48772E+00_JPRB, 0.48141E+00_JPRB, 0.47478E+00_JPRB, 0.46937E+00_JPRB, 0.47065E+00_JPRB, 0.47020E+00_JPRB, 0.46897E+00_JPRB ,&
&  0.46346E+00_JPRB, 0.45435E+00_JPRB, 0.44306E+00_JPRB, 0.44077E+00_JPRB, 0.43963E+00_JPRB, 0.43231E+00_JPRB, 0.43482E+00_JPRB ,&
&  0.44116E+00_JPRB, 0.43960E+00_JPRB, 0.43691E+00_JPRB, 0.43388E+00_JPRB, 0.41448E+00_JPRB, 0.37817E+00_JPRB, 0.38948E+00_JPRB ,&
&  0.38547E+00_JPRB, 0.37293E+00_JPRB, 0.36889E+00_JPRB, 0.36672E+00_JPRB, 0.37160E+00_JPRB, 0.38291E+00_JPRB, 0.39499E+00_JPRB ,&
&  0.40231E+00_JPRB, 0.46220E+00_JPRB, 0.48599E+00_JPRB, 0.50197E+00_JPRB, 0.51261E+00_JPRB, 0.52193E+00_JPRB, 0.53160E+00_JPRB ,&
&  0.54060E+00_JPRB, 0.55482E+00_JPRB, 0.57378E+00_JPRB, 0.59190E+00_JPRB, 0.60948E+00_JPRB, 0.62137E+00_JPRB, 0.63376E+00_JPRB ,&
&  0.64696E+00_JPRB, 0.64782E+00_JPRB, 0.63569E+00_JPRB, 0.62225E+00_JPRB, 0.60649E+00_JPRB, 0.59072E+00_JPRB, 0.57657E+00_JPRB ,&
&  0.56960E+00_JPRB, 0.56392E+00_JPRB, 0.56786E+00_JPRB, 0.57008E+00_JPRB, 0.57149E+00_JPRB, 0.57592E+00_JPRB, 0.58506E+00_JPRB ,&
&  0.59432E+00_JPRB, 0.60627E+00_JPRB, 0.61602E+00_JPRB, 0.62893E+00_JPRB, 0.64319E+00_JPRB, 0.65322E+00_JPRB, 0.66253E+00_JPRB ,&
&  0.66817E+00_JPRB, 0.68058E+00_JPRB, 0.68687E+00_JPRB, 0.69570E+00_JPRB, 0.70685E+00_JPRB, 0.71318E+00_JPRB, 0.71923E+00_JPRB ,&
&  0.72291E+00_JPRB, 0.72280E+00_JPRB, 0.71716E+00_JPRB, 0.70595E+00_JPRB, 0.68994E+00_JPRB, 0.67058E+00_JPRB, 0.64665E+00_JPRB ,&
&  0.62161E+00_JPRB, 0.59656E+00_JPRB, 0.57748E+00_JPRB, 0.56467E+00_JPRB, 0.55444E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.52221E+00_JPRB, 0.51555E+00_JPRB, 0.51062E+00_JPRB, 0.50625E+00_JPRB, 0.50322E+00_JPRB, 0.50036E+00_JPRB, 0.49752E+00_JPRB ,&
&  0.49511E+00_JPRB, 0.49294E+00_JPRB, 0.49083E+00_JPRB, 0.48920E+00_JPRB, 0.48712E+00_JPRB, 0.48545E+00_JPRB, 0.48333E+00_JPRB ,&
&  0.48250E+00_JPRB, 0.48082E+00_JPRB, 0.48137E+00_JPRB, 0.47953E+00_JPRB, 0.48087E+00_JPRB, 0.47913E+00_JPRB, 0.47281E+00_JPRB ,&
&  0.46601E+00_JPRB, 0.45649E+00_JPRB, 0.44157E+00_JPRB, 0.44444E+00_JPRB, 0.43998E+00_JPRB, 0.43988E+00_JPRB, 0.43204E+00_JPRB ,&
&  0.42948E+00_JPRB, 0.41167E+00_JPRB, 0.42142E+00_JPRB, 0.41276E+00_JPRB, 0.40704E+00_JPRB, 0.39985E+00_JPRB, 0.38902E+00_JPRB ,&
&  0.38619E+00_JPRB, 0.37963E+00_JPRB, 0.37925E+00_JPRB, 0.38182E+00_JPRB, 0.38617E+00_JPRB, 0.39332E+00_JPRB, 0.40079E+00_JPRB ,&
&  0.40407E+00_JPRB, 0.44095E+00_JPRB, 0.46191E+00_JPRB, 0.47668E+00_JPRB, 0.49003E+00_JPRB, 0.50029E+00_JPRB, 0.50892E+00_JPRB ,&
&  0.51707E+00_JPRB, 0.52452E+00_JPRB, 0.53304E+00_JPRB, 0.53572E+00_JPRB, 0.54630E+00_JPRB, 0.55003E+00_JPRB, 0.55493E+00_JPRB ,&
&  0.55928E+00_JPRB, 0.55877E+00_JPRB, 0.55670E+00_JPRB, 0.55180E+00_JPRB, 0.54849E+00_JPRB, 0.54357E+00_JPRB, 0.54146E+00_JPRB ,&
&  0.53885E+00_JPRB, 0.53764E+00_JPRB, 0.54068E+00_JPRB, 0.54353E+00_JPRB, 0.54576E+00_JPRB, 0.54845E+00_JPRB, 0.55167E+00_JPRB ,&
&  0.55604E+00_JPRB, 0.56290E+00_JPRB, 0.57001E+00_JPRB, 0.58110E+00_JPRB, 0.59196E+00_JPRB, 0.59884E+00_JPRB, 0.60559E+00_JPRB ,&
&  0.61337E+00_JPRB, 0.61117E+00_JPRB, 0.60976E+00_JPRB, 0.61003E+00_JPRB, 0.61115E+00_JPRB, 0.61290E+00_JPRB, 0.61512E+00_JPRB ,&
&  0.61523E+00_JPRB, 0.61081E+00_JPRB, 0.60470E+00_JPRB, 0.59687E+00_JPRB, 0.58870E+00_JPRB, 0.58133E+00_JPRB, 0.57379E+00_JPRB ,&
&  0.56583E+00_JPRB, 0.55806E+00_JPRB, 0.54884E+00_JPRB, 0.53875E+00_JPRB, 0.52958E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.50996E+00_JPRB, 0.50354E+00_JPRB, 0.49814E+00_JPRB, 0.49460E+00_JPRB, 0.49109E+00_JPRB, 0.48840E+00_JPRB, 0.48617E+00_JPRB ,&
&  0.48435E+00_JPRB, 0.48284E+00_JPRB, 0.48218E+00_JPRB, 0.48157E+00_JPRB, 0.48082E+00_JPRB, 0.48023E+00_JPRB, 0.47948E+00_JPRB ,&
&  0.48059E+00_JPRB, 0.48003E+00_JPRB, 0.48144E+00_JPRB, 0.48038E+00_JPRB, 0.47969E+00_JPRB, 0.47768E+00_JPRB, 0.47197E+00_JPRB ,&
&  0.46842E+00_JPRB, 0.46386E+00_JPRB, 0.44468E+00_JPRB, 0.44948E+00_JPRB, 0.44198E+00_JPRB, 0.43850E+00_JPRB, 0.43589E+00_JPRB ,&
&  0.42745E+00_JPRB, 0.42331E+00_JPRB, 0.41777E+00_JPRB, 0.41633E+00_JPRB, 0.41691E+00_JPRB, 0.41219E+00_JPRB, 0.41251E+00_JPRB ,&
&  0.40801E+00_JPRB, 0.40710E+00_JPRB, 0.40792E+00_JPRB, 0.40899E+00_JPRB, 0.41355E+00_JPRB, 0.41550E+00_JPRB, 0.42042E+00_JPRB ,&
&  0.42991E+00_JPRB, 0.44401E+00_JPRB, 0.46792E+00_JPRB, 0.47816E+00_JPRB, 0.49140E+00_JPRB, 0.50154E+00_JPRB, 0.51119E+00_JPRB ,&
&  0.51724E+00_JPRB, 0.52141E+00_JPRB, 0.52474E+00_JPRB, 0.52692E+00_JPRB, 0.52759E+00_JPRB, 0.52518E+00_JPRB, 0.52548E+00_JPRB ,&
&  0.52657E+00_JPRB, 0.52792E+00_JPRB, 0.52919E+00_JPRB, 0.52902E+00_JPRB, 0.53035E+00_JPRB, 0.53127E+00_JPRB, 0.53388E+00_JPRB ,&
&  0.53572E+00_JPRB, 0.53722E+00_JPRB, 0.53661E+00_JPRB, 0.53740E+00_JPRB, 0.53744E+00_JPRB, 0.53805E+00_JPRB, 0.53880E+00_JPRB ,&
&  0.54016E+00_JPRB, 0.54302E+00_JPRB, 0.54717E+00_JPRB, 0.54470E+00_JPRB, 0.54692E+00_JPRB, 0.55813E+00_JPRB, 0.56245E+00_JPRB ,&
&  0.56761E+00_JPRB, 0.57059E+00_JPRB, 0.57215E+00_JPRB, 0.57349E+00_JPRB, 0.57432E+00_JPRB, 0.57614E+00_JPRB, 0.57719E+00_JPRB ,&
&  0.57776E+00_JPRB, 0.57303E+00_JPRB, 0.56770E+00_JPRB, 0.56294E+00_JPRB, 0.55764E+00_JPRB, 0.55320E+00_JPRB, 0.54810E+00_JPRB ,&
&  0.54362E+00_JPRB, 0.53950E+00_JPRB, 0.53520E+00_JPRB, 0.52759E+00_JPRB, 0.51878E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.50093E+00_JPRB, 0.49626E+00_JPRB, 0.49295E+00_JPRB, 0.49048E+00_JPRB, 0.48781E+00_JPRB, 0.48583E+00_JPRB, 0.48474E+00_JPRB ,&
&  0.48422E+00_JPRB, 0.48300E+00_JPRB, 0.48257E+00_JPRB, 0.48214E+00_JPRB, 0.48120E+00_JPRB, 0.48154E+00_JPRB, 0.48144E+00_JPRB ,&
&  0.48186E+00_JPRB, 0.48125E+00_JPRB, 0.48265E+00_JPRB, 0.48175E+00_JPRB, 0.48142E+00_JPRB, 0.48140E+00_JPRB, 0.47954E+00_JPRB ,&
&  0.47887E+00_JPRB, 0.47712E+00_JPRB, 0.47171E+00_JPRB, 0.46606E+00_JPRB, 0.46148E+00_JPRB, 0.44153E+00_JPRB, 0.44048E+00_JPRB ,&
&  0.44489E+00_JPRB, 0.43782E+00_JPRB, 0.43513E+00_JPRB, 0.43287E+00_JPRB, 0.43253E+00_JPRB, 0.43153E+00_JPRB, 0.43400E+00_JPRB ,&
&  0.42479E+00_JPRB, 0.43424E+00_JPRB, 0.43469E+00_JPRB, 0.43500E+00_JPRB, 0.43668E+00_JPRB, 0.43622E+00_JPRB, 0.44215E+00_JPRB ,&
&  0.46296E+00_JPRB, 0.46775E+00_JPRB, 0.47689E+00_JPRB, 0.48485E+00_JPRB, 0.49403E+00_JPRB, 0.50220E+00_JPRB, 0.51009E+00_JPRB ,&
&  0.51605E+00_JPRB, 0.51998E+00_JPRB, 0.52253E+00_JPRB, 0.52288E+00_JPRB, 0.52337E+00_JPRB, 0.52324E+00_JPRB, 0.52394E+00_JPRB ,&
&  0.52445E+00_JPRB, 0.52606E+00_JPRB, 0.52828E+00_JPRB, 0.53007E+00_JPRB, 0.53189E+00_JPRB, 0.53255E+00_JPRB, 0.53335E+00_JPRB ,&
&  0.53357E+00_JPRB, 0.53318E+00_JPRB, 0.53212E+00_JPRB, 0.53132E+00_JPRB, 0.52982E+00_JPRB, 0.52928E+00_JPRB, 0.52909E+00_JPRB ,&
&  0.53003E+00_JPRB, 0.53046E+00_JPRB, 0.53184E+00_JPRB, 0.53315E+00_JPRB, 0.53516E+00_JPRB, 0.53828E+00_JPRB, 0.54042E+00_JPRB ,&
&  0.54433E+00_JPRB, 0.54683E+00_JPRB, 0.54916E+00_JPRB, 0.55109E+00_JPRB, 0.55224E+00_JPRB, 0.55413E+00_JPRB, 0.55538E+00_JPRB ,&
&  0.55745E+00_JPRB, 0.55502E+00_JPRB, 0.55166E+00_JPRB, 0.54775E+00_JPRB, 0.54412E+00_JPRB, 0.54053E+00_JPRB, 0.53649E+00_JPRB ,&
&  0.53245E+00_JPRB, 0.52682E+00_JPRB, 0.52069E+00_JPRB, 0.51364E+00_JPRB, 0.50667E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.50146E+00_JPRB, 0.49832E+00_JPRB, 0.49598E+00_JPRB, 0.49418E+00_JPRB, 0.49291E+00_JPRB, 0.49184E+00_JPRB, 0.49096E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.49034E+00_JPRB, 0.49015E+00_JPRB, 0.48993E+00_JPRB, 0.48949E+00_JPRB, 0.48977E+00_JPRB, 0.48964E+00_JPRB ,&
&  0.48908E+00_JPRB, 0.48748E+00_JPRB, 0.48607E+00_JPRB, 0.48466E+00_JPRB, 0.48463E+00_JPRB, 0.48469E+00_JPRB, 0.48561E+00_JPRB ,&
&  0.48521E+00_JPRB, 0.48457E+00_JPRB, 0.48323E+00_JPRB, 0.47933E+00_JPRB, 0.47524E+00_JPRB, 0.47129E+00_JPRB, 0.46791E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.46319E+00_JPRB, 0.46230E+00_JPRB, 0.46205E+00_JPRB, 0.46385E+00_JPRB, 0.46400E+00_JPRB, 0.46369E+00_JPRB ,&
&  0.46272E+00_JPRB, 0.46011E+00_JPRB, 0.46047E+00_JPRB, 0.46117E+00_JPRB, 0.46247E+00_JPRB, 0.46583E+00_JPRB, 0.46902E+00_JPRB ,&
&  0.47305E+00_JPRB, 0.47771E+00_JPRB, 0.48485E+00_JPRB, 0.49009E+00_JPRB, 0.49546E+00_JPRB, 0.49994E+00_JPRB, 0.50235E+00_JPRB ,&
&  0.50433E+00_JPRB, 0.50603E+00_JPRB, 0.50781E+00_JPRB, 0.50922E+00_JPRB, 0.51041E+00_JPRB, 0.51184E+00_JPRB, 0.51356E+00_JPRB ,&
&  0.51480E+00_JPRB, 0.51596E+00_JPRB, 0.51684E+00_JPRB, 0.51775E+00_JPRB, 0.51863E+00_JPRB, 0.51961E+00_JPRB, 0.52084E+00_JPRB ,&
&  0.52195E+00_JPRB, 0.52212E+00_JPRB, 0.52196E+00_JPRB, 0.52254E+00_JPRB, 0.52316E+00_JPRB, 0.52367E+00_JPRB, 0.52361E+00_JPRB ,&
&  0.52399E+00_JPRB, 0.52434E+00_JPRB, 0.52501E+00_JPRB, 0.52613E+00_JPRB, 0.52691E+00_JPRB, 0.52756E+00_JPRB, 0.52755E+00_JPRB ,&
&  0.52764E+00_JPRB, 0.52700E+00_JPRB, 0.52641E+00_JPRB, 0.52670E+00_JPRB, 0.52686E+00_JPRB, 0.52790E+00_JPRB, 0.52922E+00_JPRB ,&
&  0.53088E+00_JPRB, 0.53067E+00_JPRB, 0.53001E+00_JPRB, 0.52856E+00_JPRB, 0.52695E+00_JPRB, 0.52482E+00_JPRB, 0.52256E+00_JPRB ,&
&  0.52008E+00_JPRB, 0.51690E+00_JPRB, 0.51288E+00_JPRB, 0.50880E+00_JPRB, 0.50499E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.50858E+00_JPRB, 0.50731E+00_JPRB, 0.50608E+00_JPRB, 0.50481E+00_JPRB, 0.50360E+00_JPRB, 0.50267E+00_JPRB, 0.50187E+00_JPRB ,&
&  0.50113E+00_JPRB, 0.50037E+00_JPRB, 0.49948E+00_JPRB, 0.49867E+00_JPRB, 0.49802E+00_JPRB, 0.49732E+00_JPRB, 0.49674E+00_JPRB ,&
&  0.49621E+00_JPRB, 0.49580E+00_JPRB, 0.49546E+00_JPRB, 0.49484E+00_JPRB, 0.49436E+00_JPRB, 0.49384E+00_JPRB, 0.49292E+00_JPRB ,&
&  0.49173E+00_JPRB, 0.49058E+00_JPRB, 0.48929E+00_JPRB, 0.48818E+00_JPRB, 0.48731E+00_JPRB, 0.48610E+00_JPRB, 0.48466E+00_JPRB ,&
&  0.48334E+00_JPRB, 0.48241E+00_JPRB, 0.48174E+00_JPRB, 0.48131E+00_JPRB, 0.48116E+00_JPRB, 0.48097E+00_JPRB, 0.48075E+00_JPRB ,&
&  0.48064E+00_JPRB, 0.48075E+00_JPRB, 0.48108E+00_JPRB, 0.48168E+00_JPRB, 0.48252E+00_JPRB, 0.48367E+00_JPRB, 0.48556E+00_JPRB ,&
&  0.48764E+00_JPRB, 0.48975E+00_JPRB, 0.49196E+00_JPRB, 0.49403E+00_JPRB, 0.49597E+00_JPRB, 0.49746E+00_JPRB, 0.49849E+00_JPRB ,&
&  0.49967E+00_JPRB, 0.50092E+00_JPRB, 0.50228E+00_JPRB, 0.50320E+00_JPRB, 0.50398E+00_JPRB, 0.50466E+00_JPRB, 0.50545E+00_JPRB ,&
&  0.50589E+00_JPRB, 0.50637E+00_JPRB, 0.50694E+00_JPRB, 0.50764E+00_JPRB, 0.50819E+00_JPRB, 0.50866E+00_JPRB, 0.50922E+00_JPRB ,&
&  0.51001E+00_JPRB, 0.51083E+00_JPRB, 0.51142E+00_JPRB, 0.51198E+00_JPRB, 0.51267E+00_JPRB, 0.51353E+00_JPRB, 0.51440E+00_JPRB ,&
&  0.51521E+00_JPRB, 0.51595E+00_JPRB, 0.51666E+00_JPRB, 0.51729E+00_JPRB, 0.51769E+00_JPRB, 0.51781E+00_JPRB, 0.51733E+00_JPRB ,&
&  0.51672E+00_JPRB, 0.51596E+00_JPRB, 0.51552E+00_JPRB, 0.51547E+00_JPRB, 0.51535E+00_JPRB, 0.51538E+00_JPRB, 0.51564E+00_JPRB ,&
&  0.51613E+00_JPRB, 0.51664E+00_JPRB, 0.51707E+00_JPRB, 0.51747E+00_JPRB, 0.51756E+00_JPRB, 0.51720E+00_JPRB, 0.51661E+00_JPRB ,&
&  0.51584E+00_JPRB, 0.51469E+00_JPRB, 0.51330E+00_JPRB, 0.51175E+00_JPRB, 0.51007E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB, 0.50579E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB ,&
&  0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB, 0.13009E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.13542E+00_JPRB, 0.13560E+00_JPRB, 0.13569E+00_JPRB, 0.13575E+00_JPRB, 0.13565E+00_JPRB, 0.13566E+00_JPRB, 0.13566E+00_JPRB ,&
&  0.13567E+00_JPRB, 0.13555E+00_JPRB, 0.13573E+00_JPRB, 0.13583E+00_JPRB, 0.13589E+00_JPRB, 0.13590E+00_JPRB, 0.13580E+00_JPRB ,&
&  0.13556E+00_JPRB, 0.13528E+00_JPRB, 0.13450E+00_JPRB, 0.13438E+00_JPRB, 0.13443E+00_JPRB, 0.13463E+00_JPRB, 0.13477E+00_JPRB ,&
&  0.13499E+00_JPRB, 0.13520E+00_JPRB, 0.13536E+00_JPRB, 0.13584E+00_JPRB, 0.13605E+00_JPRB, 0.13629E+00_JPRB, 0.13653E+00_JPRB ,&
&  0.13677E+00_JPRB, 0.13703E+00_JPRB, 0.13723E+00_JPRB, 0.13743E+00_JPRB, 0.13761E+00_JPRB, 0.13755E+00_JPRB, 0.13736E+00_JPRB ,&
&  0.13708E+00_JPRB, 0.13663E+00_JPRB, 0.13617E+00_JPRB, 0.13575E+00_JPRB, 0.13538E+00_JPRB, 0.13499E+00_JPRB, 0.13462E+00_JPRB ,&
&  0.13422E+00_JPRB, 0.13375E+00_JPRB, 0.13326E+00_JPRB, 0.13281E+00_JPRB, 0.13236E+00_JPRB, 0.13197E+00_JPRB, 0.13153E+00_JPRB ,&
&  0.13118E+00_JPRB, 0.13083E+00_JPRB, 0.13049E+00_JPRB, 0.13014E+00_JPRB, 0.12984E+00_JPRB, 0.12954E+00_JPRB, 0.12924E+00_JPRB ,&
&  0.12893E+00_JPRB, 0.12876E+00_JPRB, 0.12852E+00_JPRB, 0.12832E+00_JPRB, 0.12813E+00_JPRB, 0.12798E+00_JPRB, 0.12780E+00_JPRB ,&
&  0.12763E+00_JPRB, 0.12746E+00_JPRB, 0.12738E+00_JPRB, 0.12725E+00_JPRB, 0.12720E+00_JPRB, 0.12717E+00_JPRB, 0.12723E+00_JPRB ,&
&  0.12728E+00_JPRB, 0.12737E+00_JPRB, 0.12751E+00_JPRB, 0.12774E+00_JPRB, 0.12795E+00_JPRB, 0.12818E+00_JPRB, 0.12842E+00_JPRB ,&
&  0.12869E+00_JPRB, 0.12896E+00_JPRB, 0.12925E+00_JPRB, 0.12952E+00_JPRB, 0.12985E+00_JPRB, 0.13019E+00_JPRB, 0.13057E+00_JPRB ,&
&  0.13091E+00_JPRB, 0.13128E+00_JPRB, 0.13159E+00_JPRB, 0.13195E+00_JPRB, 0.13223E+00_JPRB, 0.13242E+00_JPRB, 0.13258E+00_JPRB ,&
&  0.13289E+00_JPRB, 0.13319E+00_JPRB, 0.13356E+00_JPRB, 0.13389E+00_JPRB, 0.13449E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.14884E+00_JPRB, 0.15120E+00_JPRB, 0.15294E+00_JPRB, 0.15489E+00_JPRB, 0.15413E+00_JPRB, 0.16554E+00_JPRB, 0.16368E+00_JPRB ,&
&  0.16340E+00_JPRB, 0.15808E+00_JPRB, 0.15816E+00_JPRB, 0.15723E+00_JPRB, 0.15574E+00_JPRB, 0.15410E+00_JPRB, 0.15234E+00_JPRB ,&
&  0.15062E+00_JPRB, 0.14851E+00_JPRB, 0.14669E+00_JPRB, 0.14518E+00_JPRB, 0.14381E+00_JPRB, 0.14261E+00_JPRB, 0.14139E+00_JPRB ,&
&  0.14116E+00_JPRB, 0.14172E+00_JPRB, 0.14264E+00_JPRB, 0.14213E+00_JPRB, 0.14254E+00_JPRB, 0.14337E+00_JPRB, 0.14448E+00_JPRB ,&
&  0.14544E+00_JPRB, 0.14671E+00_JPRB, 0.14751E+00_JPRB, 0.14772E+00_JPRB, 0.14783E+00_JPRB, 0.14714E+00_JPRB, 0.14668E+00_JPRB ,&
&  0.14606E+00_JPRB, 0.14520E+00_JPRB, 0.14412E+00_JPRB, 0.14321E+00_JPRB, 0.14218E+00_JPRB, 0.14082E+00_JPRB, 0.13940E+00_JPRB ,&
&  0.13812E+00_JPRB, 0.13699E+00_JPRB, 0.13583E+00_JPRB, 0.13487E+00_JPRB, 0.13394E+00_JPRB, 0.13302E+00_JPRB, 0.13219E+00_JPRB ,&
&  0.13153E+00_JPRB, 0.13094E+00_JPRB, 0.13030E+00_JPRB, 0.12961E+00_JPRB, 0.12884E+00_JPRB, 0.12809E+00_JPRB, 0.12737E+00_JPRB ,&
&  0.12700E+00_JPRB, 0.12665E+00_JPRB, 0.12631E+00_JPRB, 0.12595E+00_JPRB, 0.12579E+00_JPRB, 0.12563E+00_JPRB, 0.12545E+00_JPRB ,&
&  0.12526E+00_JPRB, 0.12501E+00_JPRB, 0.12490E+00_JPRB, 0.12481E+00_JPRB, 0.12472E+00_JPRB, 0.12465E+00_JPRB, 0.12474E+00_JPRB ,&
&  0.12490E+00_JPRB, 0.12511E+00_JPRB, 0.12554E+00_JPRB, 0.12601E+00_JPRB, 0.12651E+00_JPRB, 0.12700E+00_JPRB, 0.12770E+00_JPRB ,&
&  0.12846E+00_JPRB, 0.12920E+00_JPRB, 0.12984E+00_JPRB, 0.13038E+00_JPRB, 0.13119E+00_JPRB, 0.13196E+00_JPRB, 0.13270E+00_JPRB ,&
&  0.13353E+00_JPRB, 0.13458E+00_JPRB, 0.13566E+00_JPRB, 0.13678E+00_JPRB, 0.13807E+00_JPRB, 0.13920E+00_JPRB, 0.14031E+00_JPRB ,&
&  0.14162E+00_JPRB, 0.14400E+00_JPRB, 0.14570E+00_JPRB, 0.14686E+00_JPRB, 0.14816E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.16977E+00_JPRB, 0.17096E+00_JPRB, 0.17198E+00_JPRB, 0.17249E+00_JPRB, 0.17298E+00_JPRB, 0.17300E+00_JPRB, 0.17294E+00_JPRB ,&
&  0.17032E+00_JPRB, 0.16915E+00_JPRB, 0.16750E+00_JPRB, 0.16563E+00_JPRB, 0.16401E+00_JPRB, 0.16213E+00_JPRB, 0.16019E+00_JPRB ,&
&  0.15833E+00_JPRB, 0.15612E+00_JPRB, 0.15370E+00_JPRB, 0.15088E+00_JPRB, 0.14813E+00_JPRB, 0.14554E+00_JPRB, 0.14397E+00_JPRB ,&
&  0.14316E+00_JPRB, 0.14333E+00_JPRB, 0.14468E+00_JPRB, 0.14580E+00_JPRB, 0.14746E+00_JPRB, 0.14958E+00_JPRB, 0.15300E+00_JPRB ,&
&  0.15568E+00_JPRB, 0.15895E+00_JPRB, 0.16117E+00_JPRB, 0.16193E+00_JPRB, 0.16123E+00_JPRB, 0.16088E+00_JPRB, 0.16004E+00_JPRB ,&
&  0.15840E+00_JPRB, 0.15665E+00_JPRB, 0.15513E+00_JPRB, 0.15350E+00_JPRB, 0.15167E+00_JPRB, 0.14936E+00_JPRB, 0.14655E+00_JPRB ,&
&  0.14400E+00_JPRB, 0.14176E+00_JPRB, 0.13969E+00_JPRB, 0.13811E+00_JPRB, 0.13659E+00_JPRB, 0.13523E+00_JPRB, 0.13381E+00_JPRB ,&
&  0.13271E+00_JPRB, 0.13176E+00_JPRB, 0.13055E+00_JPRB, 0.12952E+00_JPRB, 0.12836E+00_JPRB, 0.12739E+00_JPRB, 0.12634E+00_JPRB ,&
&  0.12551E+00_JPRB, 0.12477E+00_JPRB, 0.12415E+00_JPRB, 0.12356E+00_JPRB, 0.12317E+00_JPRB, 0.12279E+00_JPRB, 0.12235E+00_JPRB ,&
&  0.12196E+00_JPRB, 0.12185E+00_JPRB, 0.12179E+00_JPRB, 0.12175E+00_JPRB, 0.12179E+00_JPRB, 0.12149E+00_JPRB, 0.12153E+00_JPRB ,&
&  0.12183E+00_JPRB, 0.12215E+00_JPRB, 0.12231E+00_JPRB, 0.12275E+00_JPRB, 0.12342E+00_JPRB, 0.12422E+00_JPRB, 0.12535E+00_JPRB ,&
&  0.12624E+00_JPRB, 0.12714E+00_JPRB, 0.12797E+00_JPRB, 0.12883E+00_JPRB, 0.12992E+00_JPRB, 0.13110E+00_JPRB, 0.13234E+00_JPRB ,&
&  0.13381E+00_JPRB, 0.13520E+00_JPRB, 0.13665E+00_JPRB, 0.13816E+00_JPRB, 0.14052E+00_JPRB, 0.14293E+00_JPRB, 0.14659E+00_JPRB ,&
&  0.15033E+00_JPRB, 0.15249E+00_JPRB, 0.16503E+00_JPRB, 0.16607E+00_JPRB, 0.16754E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.17024E+00_JPRB, 0.17167E+00_JPRB, 0.17256E+00_JPRB, 0.17369E+00_JPRB, 0.17429E+00_JPRB, 0.17583E+00_JPRB, 0.17617E+00_JPRB ,&
&  0.17630E+00_JPRB, 0.17238E+00_JPRB, 0.17001E+00_JPRB, 0.16741E+00_JPRB, 0.16516E+00_JPRB, 0.16263E+00_JPRB, 0.16015E+00_JPRB ,&
&  0.15819E+00_JPRB, 0.15651E+00_JPRB, 0.15472E+00_JPRB, 0.15223E+00_JPRB, 0.14971E+00_JPRB, 0.14781E+00_JPRB, 0.14637E+00_JPRB ,&
&  0.14494E+00_JPRB, 0.14540E+00_JPRB, 0.14668E+00_JPRB, 0.14948E+00_JPRB, 0.15343E+00_JPRB, 0.15691E+00_JPRB, 0.16144E+00_JPRB ,&
&  0.16756E+00_JPRB, 0.16941E+00_JPRB, 0.17127E+00_JPRB, 0.17218E+00_JPRB, 0.17191E+00_JPRB, 0.17232E+00_JPRB, 0.16985E+00_JPRB ,&
&  0.17009E+00_JPRB, 0.16826E+00_JPRB, 0.17272E+00_JPRB, 0.17036E+00_JPRB, 0.16422E+00_JPRB, 0.16116E+00_JPRB, 0.15531E+00_JPRB ,&
&  0.15073E+00_JPRB, 0.14600E+00_JPRB, 0.14275E+00_JPRB, 0.14088E+00_JPRB, 0.13825E+00_JPRB, 0.13678E+00_JPRB, 0.13460E+00_JPRB ,&
&  0.13260E+00_JPRB, 0.13186E+00_JPRB, 0.13055E+00_JPRB, 0.12995E+00_JPRB, 0.12921E+00_JPRB, 0.12823E+00_JPRB, 0.12717E+00_JPRB ,&
&  0.12645E+00_JPRB, 0.12579E+00_JPRB, 0.12542E+00_JPRB, 0.12493E+00_JPRB, 0.12425E+00_JPRB, 0.12358E+00_JPRB, 0.12284E+00_JPRB ,&
&  0.12232E+00_JPRB, 0.12200E+00_JPRB, 0.12172E+00_JPRB, 0.12157E+00_JPRB, 0.12142E+00_JPRB, 0.12113E+00_JPRB, 0.12116E+00_JPRB ,&
&  0.12137E+00_JPRB, 0.12190E+00_JPRB, 0.12239E+00_JPRB, 0.12304E+00_JPRB, 0.12365E+00_JPRB, 0.12447E+00_JPRB, 0.12512E+00_JPRB ,&
&  0.12581E+00_JPRB, 0.12669E+00_JPRB, 0.12746E+00_JPRB, 0.12836E+00_JPRB, 0.12936E+00_JPRB, 0.13022E+00_JPRB, 0.13155E+00_JPRB ,&
&  0.13290E+00_JPRB, 0.13423E+00_JPRB, 0.13605E+00_JPRB, 0.13797E+00_JPRB, 0.14042E+00_JPRB, 0.14466E+00_JPRB, 0.14958E+00_JPRB ,&
&  0.15573E+00_JPRB, 0.16506E+00_JPRB, 0.16546E+00_JPRB, 0.16770E+00_JPRB, 0.16909E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.17517E+00_JPRB, 0.17707E+00_JPRB, 0.17872E+00_JPRB, 0.18022E+00_JPRB, 0.18052E+00_JPRB, 0.18085E+00_JPRB, 0.18160E+00_JPRB ,&
&  0.17795E+00_JPRB, 0.17524E+00_JPRB, 0.17076E+00_JPRB, 0.16642E+00_JPRB, 0.16231E+00_JPRB, 0.15877E+00_JPRB, 0.15665E+00_JPRB ,&
&  0.15558E+00_JPRB, 0.15561E+00_JPRB, 0.15602E+00_JPRB, 0.15678E+00_JPRB, 0.15636E+00_JPRB, 0.15557E+00_JPRB, 0.15472E+00_JPRB ,&
&  0.15354E+00_JPRB, 0.15357E+00_JPRB, 0.15390E+00_JPRB, 0.15702E+00_JPRB, 0.16144E+00_JPRB, 0.16650E+00_JPRB, 0.17009E+00_JPRB ,&
&  0.17268E+00_JPRB, 0.17461E+00_JPRB, 0.17689E+00_JPRB, 0.17785E+00_JPRB, 0.17864E+00_JPRB, 0.17983E+00_JPRB, 0.18043E+00_JPRB ,&
&  0.18177E+00_JPRB, 0.18204E+00_JPRB, 0.18209E+00_JPRB, 0.18260E+00_JPRB, 0.17470E+00_JPRB, 0.16866E+00_JPRB, 0.16259E+00_JPRB ,&
&  0.15666E+00_JPRB, 0.15071E+00_JPRB, 0.14537E+00_JPRB, 0.14231E+00_JPRB, 0.13936E+00_JPRB, 0.13765E+00_JPRB, 0.13614E+00_JPRB ,&
&  0.13504E+00_JPRB, 0.13388E+00_JPRB, 0.13226E+00_JPRB, 0.13068E+00_JPRB, 0.12976E+00_JPRB, 0.12898E+00_JPRB, 0.12802E+00_JPRB ,&
&  0.12792E+00_JPRB, 0.12733E+00_JPRB, 0.12722E+00_JPRB, 0.12683E+00_JPRB, 0.12655E+00_JPRB, 0.12631E+00_JPRB, 0.12633E+00_JPRB ,&
&  0.12668E+00_JPRB, 0.12733E+00_JPRB, 0.12753E+00_JPRB, 0.12778E+00_JPRB, 0.12758E+00_JPRB, 0.12698E+00_JPRB, 0.12644E+00_JPRB ,&
&  0.12590E+00_JPRB, 0.12546E+00_JPRB, 0.12544E+00_JPRB, 0.12535E+00_JPRB, 0.12582E+00_JPRB, 0.12646E+00_JPRB, 0.12710E+00_JPRB ,&
&  0.12798E+00_JPRB, 0.12867E+00_JPRB, 0.12936E+00_JPRB, 0.13018E+00_JPRB, 0.13092E+00_JPRB, 0.13186E+00_JPRB, 0.13296E+00_JPRB ,&
&  0.13383E+00_JPRB, 0.13544E+00_JPRB, 0.13678E+00_JPRB, 0.13841E+00_JPRB, 0.14137E+00_JPRB, 0.14482E+00_JPRB, 0.15265E+00_JPRB ,&
&  0.16062E+00_JPRB, 0.17768E+00_JPRB, 0.17928E+00_JPRB, 0.17371E+00_JPRB, 0.17408E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.18323E+00_JPRB, 0.18553E+00_JPRB, 0.18797E+00_JPRB, 0.18682E+00_JPRB, 0.18691E+00_JPRB, 0.18722E+00_JPRB, 0.18843E+00_JPRB ,&
&  0.18902E+00_JPRB, 0.18870E+00_JPRB, 0.18837E+00_JPRB, 0.18712E+00_JPRB, 0.18565E+00_JPRB, 0.17867E+00_JPRB, 0.17556E+00_JPRB ,&
&  0.17319E+00_JPRB, 0.17355E+00_JPRB, 0.17403E+00_JPRB, 0.17599E+00_JPRB, 0.17634E+00_JPRB, 0.17566E+00_JPRB, 0.17288E+00_JPRB ,&
&  0.17047E+00_JPRB, 0.16880E+00_JPRB, 0.16716E+00_JPRB, 0.16718E+00_JPRB, 0.16903E+00_JPRB, 0.17112E+00_JPRB, 0.17230E+00_JPRB ,&
&  0.17257E+00_JPRB, 0.17428E+00_JPRB, 0.17573E+00_JPRB, 0.17831E+00_JPRB, 0.18546E+00_JPRB, 0.18704E+00_JPRB, 0.18771E+00_JPRB ,&
&  0.18856E+00_JPRB, 0.18990E+00_JPRB, 0.19080E+00_JPRB, 0.19082E+00_JPRB, 0.18933E+00_JPRB, 0.18507E+00_JPRB, 0.18397E+00_JPRB ,&
&  0.17133E+00_JPRB, 0.16388E+00_JPRB, 0.15669E+00_JPRB, 0.15131E+00_JPRB, 0.14655E+00_JPRB, 0.14395E+00_JPRB, 0.14103E+00_JPRB ,&
&  0.13859E+00_JPRB, 0.13595E+00_JPRB, 0.13395E+00_JPRB, 0.13300E+00_JPRB, 0.13274E+00_JPRB, 0.13251E+00_JPRB, 0.13198E+00_JPRB ,&
&  0.13205E+00_JPRB, 0.13165E+00_JPRB, 0.13134E+00_JPRB, 0.13103E+00_JPRB, 0.13083E+00_JPRB, 0.13091E+00_JPRB, 0.13159E+00_JPRB ,&
&  0.13336E+00_JPRB, 0.13510E+00_JPRB, 0.13680E+00_JPRB, 0.13822E+00_JPRB, 0.13770E+00_JPRB, 0.13614E+00_JPRB, 0.13375E+00_JPRB ,&
&  0.13172E+00_JPRB, 0.13042E+00_JPRB, 0.12902E+00_JPRB, 0.12867E+00_JPRB, 0.12921E+00_JPRB, 0.12990E+00_JPRB, 0.13108E+00_JPRB ,&
&  0.13211E+00_JPRB, 0.13296E+00_JPRB, 0.13351E+00_JPRB, 0.13391E+00_JPRB, 0.13455E+00_JPRB, 0.13518E+00_JPRB, 0.13624E+00_JPRB ,&
&  0.13741E+00_JPRB, 0.13885E+00_JPRB, 0.14020E+00_JPRB, 0.14199E+00_JPRB, 0.14458E+00_JPRB, 0.14862E+00_JPRB, 0.15553E+00_JPRB ,&
&  0.16268E+00_JPRB, 0.18155E+00_JPRB, 0.18373E+00_JPRB, 0.18167E+00_JPRB, 0.18279E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.19052E+00_JPRB, 0.19468E+00_JPRB, 0.19715E+00_JPRB, 0.19673E+00_JPRB, 0.19639E+00_JPRB, 0.19707E+00_JPRB, 0.19792E+00_JPRB ,&
&  0.19902E+00_JPRB, 0.19861E+00_JPRB, 0.19727E+00_JPRB, 0.19588E+00_JPRB, 0.19437E+00_JPRB, 0.19238E+00_JPRB, 0.19099E+00_JPRB ,&
&  0.18990E+00_JPRB, 0.18979E+00_JPRB, 0.18930E+00_JPRB, 0.19003E+00_JPRB, 0.19055E+00_JPRB, 0.19309E+00_JPRB, 0.19189E+00_JPRB ,&
&  0.18863E+00_JPRB, 0.18693E+00_JPRB, 0.18912E+00_JPRB, 0.18952E+00_JPRB, 0.19006E+00_JPRB, 0.19087E+00_JPRB, 0.19214E+00_JPRB ,&
&  0.18984E+00_JPRB, 0.18666E+00_JPRB, 0.18185E+00_JPRB, 0.18651E+00_JPRB, 0.19581E+00_JPRB, 0.19613E+00_JPRB, 0.19529E+00_JPRB ,&
&  0.19659E+00_JPRB, 0.19927E+00_JPRB, 0.20059E+00_JPRB, 0.20101E+00_JPRB, 0.19959E+00_JPRB, 0.19888E+00_JPRB, 0.19783E+00_JPRB ,&
&  0.19521E+00_JPRB, 0.19250E+00_JPRB, 0.18458E+00_JPRB, 0.16932E+00_JPRB, 0.16265E+00_JPRB, 0.15936E+00_JPRB, 0.15541E+00_JPRB ,&
&  0.15052E+00_JPRB, 0.14799E+00_JPRB, 0.14763E+00_JPRB, 0.14634E+00_JPRB, 0.14662E+00_JPRB, 0.14615E+00_JPRB, 0.14532E+00_JPRB ,&
&  0.14420E+00_JPRB, 0.14339E+00_JPRB, 0.14192E+00_JPRB, 0.13962E+00_JPRB, 0.13791E+00_JPRB, 0.13747E+00_JPRB, 0.13813E+00_JPRB ,&
&  0.13991E+00_JPRB, 0.14281E+00_JPRB, 0.15084E+00_JPRB, 0.15554E+00_JPRB, 0.15358E+00_JPRB, 0.14934E+00_JPRB, 0.14346E+00_JPRB ,&
&  0.13853E+00_JPRB, 0.13696E+00_JPRB, 0.13587E+00_JPRB, 0.13496E+00_JPRB, 0.13525E+00_JPRB, 0.13625E+00_JPRB, 0.13725E+00_JPRB ,&
&  0.13836E+00_JPRB, 0.13958E+00_JPRB, 0.14030E+00_JPRB, 0.14069E+00_JPRB, 0.14108E+00_JPRB, 0.14132E+00_JPRB, 0.14178E+00_JPRB ,&
&  0.14199E+00_JPRB, 0.14328E+00_JPRB, 0.14448E+00_JPRB, 0.14671E+00_JPRB, 0.14825E+00_JPRB, 0.15013E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15738E+00_JPRB, 0.16741E+00_JPRB, 0.18177E+00_JPRB, 0.19011E+00_JPRB, 0.18752E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.20083E+00_JPRB, 0.20197E+00_JPRB, 0.20213E+00_JPRB, 0.20129E+00_JPRB, 0.20011E+00_JPRB, 0.20038E+00_JPRB, 0.20182E+00_JPRB ,&
&  0.20469E+00_JPRB, 0.20686E+00_JPRB, 0.20745E+00_JPRB, 0.20756E+00_JPRB, 0.20665E+00_JPRB, 0.20456E+00_JPRB, 0.20276E+00_JPRB ,&
&  0.20133E+00_JPRB, 0.20044E+00_JPRB, 0.19973E+00_JPRB, 0.19946E+00_JPRB, 0.19946E+00_JPRB, 0.20062E+00_JPRB, 0.20185E+00_JPRB ,&
&  0.20571E+00_JPRB, 0.20593E+00_JPRB, 0.20529E+00_JPRB, 0.20506E+00_JPRB, 0.20480E+00_JPRB, 0.20472E+00_JPRB, 0.20464E+00_JPRB ,&
&  0.20238E+00_JPRB, 0.19455E+00_JPRB, 0.18735E+00_JPRB, 0.19235E+00_JPRB, 0.20503E+00_JPRB, 0.20543E+00_JPRB, 0.20539E+00_JPRB ,&
&  0.20670E+00_JPRB, 0.20960E+00_JPRB, 0.21156E+00_JPRB, 0.21260E+00_JPRB, 0.21144E+00_JPRB, 0.20966E+00_JPRB, 0.20913E+00_JPRB ,&
&  0.20926E+00_JPRB, 0.20742E+00_JPRB, 0.20246E+00_JPRB, 0.19693E+00_JPRB, 0.19134E+00_JPRB, 0.18912E+00_JPRB, 0.18739E+00_JPRB ,&
&  0.18503E+00_JPRB, 0.18370E+00_JPRB, 0.18319E+00_JPRB, 0.18271E+00_JPRB, 0.18312E+00_JPRB, 0.18131E+00_JPRB, 0.17897E+00_JPRB ,&
&  0.17508E+00_JPRB, 0.17119E+00_JPRB, 0.16629E+00_JPRB, 0.16108E+00_JPRB, 0.15649E+00_JPRB, 0.15177E+00_JPRB, 0.14852E+00_JPRB ,&
&  0.14780E+00_JPRB, 0.15128E+00_JPRB, 0.16035E+00_JPRB, 0.17056E+00_JPRB, 0.17941E+00_JPRB, 0.16761E+00_JPRB, 0.16035E+00_JPRB ,&
&  0.15315E+00_JPRB, 0.14769E+00_JPRB, 0.14463E+00_JPRB, 0.14387E+00_JPRB, 0.14403E+00_JPRB, 0.14484E+00_JPRB, 0.14629E+00_JPRB ,&
&  0.14809E+00_JPRB, 0.15042E+00_JPRB, 0.15223E+00_JPRB, 0.15343E+00_JPRB, 0.15424E+00_JPRB, 0.15447E+00_JPRB, 0.15428E+00_JPRB ,&
&  0.15405E+00_JPRB, 0.15519E+00_JPRB, 0.15759E+00_JPRB, 0.16167E+00_JPRB, 0.16476E+00_JPRB, 0.16862E+00_JPRB, 0.17410E+00_JPRB ,&
&  0.18188E+00_JPRB, 0.19120E+00_JPRB, 0.19401E+00_JPRB, 0.19581E+00_JPRB, 0.19855E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.21055E+00_JPRB, 0.21180E+00_JPRB, 0.21193E+00_JPRB, 0.21223E+00_JPRB, 0.21275E+00_JPRB, 0.21455E+00_JPRB, 0.21862E+00_JPRB ,&
&  0.22260E+00_JPRB, 0.22444E+00_JPRB, 0.22303E+00_JPRB, 0.22089E+00_JPRB, 0.21978E+00_JPRB, 0.21912E+00_JPRB, 0.21884E+00_JPRB ,&
&  0.21859E+00_JPRB, 0.21756E+00_JPRB, 0.21474E+00_JPRB, 0.21090E+00_JPRB, 0.20820E+00_JPRB, 0.20770E+00_JPRB, 0.20832E+00_JPRB ,&
&  0.21224E+00_JPRB, 0.21499E+00_JPRB, 0.21758E+00_JPRB, 0.21969E+00_JPRB, 0.22009E+00_JPRB, 0.21966E+00_JPRB, 0.22097E+00_JPRB ,&
&  0.22206E+00_JPRB, 0.22004E+00_JPRB, 0.21480E+00_JPRB, 0.21913E+00_JPRB, 0.22111E+00_JPRB, 0.22232E+00_JPRB, 0.22162E+00_JPRB ,&
&  0.22233E+00_JPRB, 0.22445E+00_JPRB, 0.22638E+00_JPRB, 0.22791E+00_JPRB, 0.22722E+00_JPRB, 0.22404E+00_JPRB, 0.22291E+00_JPRB ,&
&  0.22297E+00_JPRB, 0.22183E+00_JPRB, 0.21922E+00_JPRB, 0.21688E+00_JPRB, 0.21431E+00_JPRB, 0.21323E+00_JPRB, 0.21304E+00_JPRB ,&
&  0.21212E+00_JPRB, 0.20995E+00_JPRB, 0.20565E+00_JPRB, 0.20146E+00_JPRB, 0.19849E+00_JPRB, 0.19566E+00_JPRB, 0.19471E+00_JPRB ,&
&  0.19531E+00_JPRB, 0.19660E+00_JPRB, 0.19673E+00_JPRB, 0.19555E+00_JPRB, 0.19141E+00_JPRB, 0.18084E+00_JPRB, 0.17374E+00_JPRB ,&
&  0.17248E+00_JPRB, 0.17719E+00_JPRB, 0.18558E+00_JPRB, 0.18644E+00_JPRB, 0.18831E+00_JPRB, 0.18974E+00_JPRB, 0.19115E+00_JPRB ,&
&  0.19026E+00_JPRB, 0.17683E+00_JPRB, 0.17087E+00_JPRB, 0.16763E+00_JPRB, 0.16447E+00_JPRB, 0.16383E+00_JPRB, 0.16521E+00_JPRB ,&
&  0.16808E+00_JPRB, 0.17210E+00_JPRB, 0.17475E+00_JPRB, 0.17617E+00_JPRB, 0.17648E+00_JPRB, 0.17593E+00_JPRB, 0.17517E+00_JPRB ,&
&  0.17518E+00_JPRB, 0.17882E+00_JPRB, 0.18642E+00_JPRB, 0.19524E+00_JPRB, 0.19621E+00_JPRB, 0.19785E+00_JPRB, 0.19853E+00_JPRB ,&
&  0.19910E+00_JPRB, 0.20088E+00_JPRB, 0.20197E+00_JPRB, 0.20464E+00_JPRB, 0.20792E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.22115E+00_JPRB, 0.22417E+00_JPRB, 0.22716E+00_JPRB, 0.22894E+00_JPRB, 0.23077E+00_JPRB, 0.23259E+00_JPRB, 0.23582E+00_JPRB ,&
&  0.23962E+00_JPRB, 0.24076E+00_JPRB, 0.23907E+00_JPRB, 0.23637E+00_JPRB, 0.23459E+00_JPRB, 0.23376E+00_JPRB, 0.23345E+00_JPRB ,&
&  0.23322E+00_JPRB, 0.23306E+00_JPRB, 0.23232E+00_JPRB, 0.23063E+00_JPRB, 0.22825E+00_JPRB, 0.22591E+00_JPRB, 0.22424E+00_JPRB ,&
&  0.22419E+00_JPRB, 0.22581E+00_JPRB, 0.22831E+00_JPRB, 0.22878E+00_JPRB, 0.22907E+00_JPRB, 0.23034E+00_JPRB, 0.23229E+00_JPRB ,&
&  0.23615E+00_JPRB, 0.23818E+00_JPRB, 0.24044E+00_JPRB, 0.23632E+00_JPRB, 0.24102E+00_JPRB, 0.24268E+00_JPRB, 0.24325E+00_JPRB ,&
&  0.24457E+00_JPRB, 0.24560E+00_JPRB, 0.24652E+00_JPRB, 0.24579E+00_JPRB, 0.24143E+00_JPRB, 0.23933E+00_JPRB, 0.23963E+00_JPRB ,&
&  0.24000E+00_JPRB, 0.23844E+00_JPRB, 0.23563E+00_JPRB, 0.23301E+00_JPRB, 0.23092E+00_JPRB, 0.22952E+00_JPRB, 0.22836E+00_JPRB ,&
&  0.22777E+00_JPRB, 0.22708E+00_JPRB, 0.22531E+00_JPRB, 0.22035E+00_JPRB, 0.21459E+00_JPRB, 0.21086E+00_JPRB, 0.20836E+00_JPRB ,&
&  0.20855E+00_JPRB, 0.21052E+00_JPRB, 0.21219E+00_JPRB, 0.21238E+00_JPRB, 0.21129E+00_JPRB, 0.20931E+00_JPRB, 0.20620E+00_JPRB ,&
&  0.20141E+00_JPRB, 0.19923E+00_JPRB, 0.19840E+00_JPRB, 0.19743E+00_JPRB, 0.19784E+00_JPRB, 0.19957E+00_JPRB, 0.20121E+00_JPRB ,&
&  0.20315E+00_JPRB, 0.20317E+00_JPRB, 0.20179E+00_JPRB, 0.19945E+00_JPRB, 0.19699E+00_JPRB, 0.19678E+00_JPRB, 0.19728E+00_JPRB ,&
&  0.19768E+00_JPRB, 0.19798E+00_JPRB, 0.19863E+00_JPRB, 0.19960E+00_JPRB, 0.20096E+00_JPRB, 0.20308E+00_JPRB, 0.20490E+00_JPRB ,&
&  0.20485E+00_JPRB, 0.20377E+00_JPRB, 0.20476E+00_JPRB, 0.20629E+00_JPRB, 0.20707E+00_JPRB, 0.20761E+00_JPRB, 0.20797E+00_JPRB ,&
&  0.20767E+00_JPRB, 0.20780E+00_JPRB, 0.20968E+00_JPRB, 0.21420E+00_JPRB, 0.21836E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.23303E+00_JPRB, 0.23677E+00_JPRB, 0.24188E+00_JPRB, 0.24330E+00_JPRB, 0.24604E+00_JPRB, 0.24877E+00_JPRB, 0.25219E+00_JPRB ,&
&  0.25430E+00_JPRB, 0.25434E+00_JPRB, 0.25278E+00_JPRB, 0.25122E+00_JPRB, 0.25033E+00_JPRB, 0.24987E+00_JPRB, 0.24920E+00_JPRB ,&
&  0.24890E+00_JPRB, 0.24900E+00_JPRB, 0.24897E+00_JPRB, 0.24888E+00_JPRB, 0.24788E+00_JPRB, 0.24634E+00_JPRB, 0.24464E+00_JPRB ,&
&  0.24167E+00_JPRB, 0.23979E+00_JPRB, 0.23939E+00_JPRB, 0.23947E+00_JPRB, 0.23908E+00_JPRB, 0.23983E+00_JPRB, 0.24168E+00_JPRB ,&
&  0.24551E+00_JPRB, 0.24861E+00_JPRB, 0.25253E+00_JPRB, 0.25660E+00_JPRB, 0.25908E+00_JPRB, 0.26087E+00_JPRB, 0.26294E+00_JPRB ,&
&  0.26427E+00_JPRB, 0.26462E+00_JPRB, 0.26461E+00_JPRB, 0.26448E+00_JPRB, 0.26148E+00_JPRB, 0.26012E+00_JPRB, 0.25911E+00_JPRB ,&
&  0.25766E+00_JPRB, 0.25512E+00_JPRB, 0.25162E+00_JPRB, 0.24852E+00_JPRB, 0.24545E+00_JPRB, 0.24326E+00_JPRB, 0.24148E+00_JPRB ,&
&  0.24023E+00_JPRB, 0.23976E+00_JPRB, 0.23998E+00_JPRB, 0.23880E+00_JPRB, 0.23397E+00_JPRB, 0.22913E+00_JPRB, 0.22640E+00_JPRB ,&
&  0.22515E+00_JPRB, 0.22495E+00_JPRB, 0.22637E+00_JPRB, 0.22534E+00_JPRB, 0.22193E+00_JPRB, 0.21923E+00_JPRB, 0.21808E+00_JPRB ,&
&  0.21650E+00_JPRB, 0.21567E+00_JPRB, 0.21618E+00_JPRB, 0.21673E+00_JPRB, 0.21737E+00_JPRB, 0.21768E+00_JPRB, 0.21871E+00_JPRB ,&
&  0.21950E+00_JPRB, 0.22074E+00_JPRB, 0.22046E+00_JPRB, 0.21778E+00_JPRB, 0.21411E+00_JPRB, 0.21143E+00_JPRB, 0.21081E+00_JPRB ,&
&  0.21211E+00_JPRB, 0.21341E+00_JPRB, 0.21366E+00_JPRB, 0.21368E+00_JPRB, 0.21466E+00_JPRB, 0.21704E+00_JPRB, 0.22014E+00_JPRB ,&
&  0.22142E+00_JPRB, 0.22266E+00_JPRB, 0.22444E+00_JPRB, 0.22626E+00_JPRB, 0.22746E+00_JPRB, 0.22813E+00_JPRB, 0.22829E+00_JPRB ,&
&  0.22762E+00_JPRB, 0.22828E+00_JPRB, 0.22974E+00_JPRB, 0.23151E+00_JPRB, 0.23277E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.25183E+00_JPRB, 0.25439E+00_JPRB, 0.25775E+00_JPRB, 0.25861E+00_JPRB, 0.25918E+00_JPRB, 0.26080E+00_JPRB, 0.26384E+00_JPRB ,&
&  0.26590E+00_JPRB, 0.26643E+00_JPRB, 0.26622E+00_JPRB, 0.26586E+00_JPRB, 0.26599E+00_JPRB, 0.26730E+00_JPRB, 0.26781E+00_JPRB ,&
&  0.26708E+00_JPRB, 0.26645E+00_JPRB, 0.26564E+00_JPRB, 0.26512E+00_JPRB, 0.26386E+00_JPRB, 0.26195E+00_JPRB, 0.26046E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25638E+00_JPRB, 0.25507E+00_JPRB, 0.25417E+00_JPRB, 0.25388E+00_JPRB, 0.25427E+00_JPRB, 0.25608E+00_JPRB ,&
&  0.25824E+00_JPRB, 0.26093E+00_JPRB, 0.26411E+00_JPRB, 0.26772E+00_JPRB, 0.27136E+00_JPRB, 0.27549E+00_JPRB, 0.27915E+00_JPRB ,&
&  0.28204E+00_JPRB, 0.28437E+00_JPRB, 0.28573E+00_JPRB, 0.28267E+00_JPRB, 0.27881E+00_JPRB, 0.27733E+00_JPRB, 0.27632E+00_JPRB ,&
&  0.27371E+00_JPRB, 0.27034E+00_JPRB, 0.26584E+00_JPRB, 0.26322E+00_JPRB, 0.26239E+00_JPRB, 0.26129E+00_JPRB, 0.26000E+00_JPRB ,&
&  0.25920E+00_JPRB, 0.25876E+00_JPRB, 0.25788E+00_JPRB, 0.25617E+00_JPRB, 0.25312E+00_JPRB, 0.24876E+00_JPRB, 0.24432E+00_JPRB ,&
&  0.24152E+00_JPRB, 0.23961E+00_JPRB, 0.23983E+00_JPRB, 0.23809E+00_JPRB, 0.23427E+00_JPRB, 0.23073E+00_JPRB, 0.23028E+00_JPRB ,&
&  0.23028E+00_JPRB, 0.23057E+00_JPRB, 0.23027E+00_JPRB, 0.22912E+00_JPRB, 0.22941E+00_JPRB, 0.23043E+00_JPRB, 0.23188E+00_JPRB ,&
&  0.23553E+00_JPRB, 0.23805E+00_JPRB, 0.23776E+00_JPRB, 0.23656E+00_JPRB, 0.23459E+00_JPRB, 0.23409E+00_JPRB, 0.23453E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.23500E+00_JPRB, 0.23511E+00_JPRB, 0.23451E+00_JPRB, 0.23462E+00_JPRB, 0.23608E+00_JPRB, 0.23789E+00_JPRB ,&
&  0.24005E+00_JPRB, 0.24274E+00_JPRB, 0.24511E+00_JPRB, 0.24760E+00_JPRB, 0.24992E+00_JPRB, 0.25113E+00_JPRB, 0.25217E+00_JPRB ,&
&  0.25315E+00_JPRB, 0.25327E+00_JPRB, 0.25287E+00_JPRB, 0.25269E+00_JPRB, 0.25253E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.27713E+00_JPRB, 0.27738E+00_JPRB, 0.27844E+00_JPRB, 0.27899E+00_JPRB, 0.27891E+00_JPRB, 0.27945E+00_JPRB, 0.28192E+00_JPRB ,&
&  0.28408E+00_JPRB, 0.28527E+00_JPRB, 0.28609E+00_JPRB, 0.28614E+00_JPRB, 0.28582E+00_JPRB, 0.28700E+00_JPRB, 0.28794E+00_JPRB ,&
&  0.28795E+00_JPRB, 0.28681E+00_JPRB, 0.28563E+00_JPRB, 0.28424E+00_JPRB, 0.28239E+00_JPRB, 0.28051E+00_JPRB, 0.27914E+00_JPRB ,&
&  0.27726E+00_JPRB, 0.27516E+00_JPRB, 0.27313E+00_JPRB, 0.27269E+00_JPRB, 0.27317E+00_JPRB, 0.27378E+00_JPRB, 0.27504E+00_JPRB ,&
&  0.27626E+00_JPRB, 0.27901E+00_JPRB, 0.28218E+00_JPRB, 0.28576E+00_JPRB, 0.28931E+00_JPRB, 0.29404E+00_JPRB, 0.29928E+00_JPRB ,&
&  0.30410E+00_JPRB, 0.30642E+00_JPRB, 0.30788E+00_JPRB, 0.30562E+00_JPRB, 0.30461E+00_JPRB, 0.30355E+00_JPRB, 0.30292E+00_JPRB ,&
&  0.30177E+00_JPRB, 0.29724E+00_JPRB, 0.29348E+00_JPRB, 0.29303E+00_JPRB, 0.29357E+00_JPRB, 0.29386E+00_JPRB, 0.29383E+00_JPRB ,&
&  0.29295E+00_JPRB, 0.29165E+00_JPRB, 0.28909E+00_JPRB, 0.28545E+00_JPRB, 0.28154E+00_JPRB, 0.27668E+00_JPRB, 0.26959E+00_JPRB ,&
&  0.26290E+00_JPRB, 0.25748E+00_JPRB, 0.25343E+00_JPRB, 0.24992E+00_JPRB, 0.24751E+00_JPRB, 0.24477E+00_JPRB, 0.24394E+00_JPRB ,&
&  0.24421E+00_JPRB, 0.24594E+00_JPRB, 0.24693E+00_JPRB, 0.24624E+00_JPRB, 0.24510E+00_JPRB, 0.24528E+00_JPRB, 0.24725E+00_JPRB ,&
&  0.25090E+00_JPRB, 0.25368E+00_JPRB, 0.25446E+00_JPRB, 0.25386E+00_JPRB, 0.25291E+00_JPRB, 0.25353E+00_JPRB, 0.25510E+00_JPRB ,&
&  0.25581E+00_JPRB, 0.25624E+00_JPRB, 0.25739E+00_JPRB, 0.25733E+00_JPRB, 0.25927E+00_JPRB, 0.26249E+00_JPRB, 0.26602E+00_JPRB ,&
&  0.26941E+00_JPRB, 0.27363E+00_JPRB, 0.27793E+00_JPRB, 0.28147E+00_JPRB, 0.28461E+00_JPRB, 0.28494E+00_JPRB, 0.28541E+00_JPRB ,&
&  0.28554E+00_JPRB, 0.28437E+00_JPRB, 0.28308E+00_JPRB, 0.28115E+00_JPRB, 0.27871E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.30623E+00_JPRB, 0.30521E+00_JPRB, 0.30434E+00_JPRB, 0.30406E+00_JPRB, 0.30368E+00_JPRB, 0.30314E+00_JPRB, 0.30296E+00_JPRB ,&
&  0.30339E+00_JPRB, 0.30330E+00_JPRB, 0.30545E+00_JPRB, 0.30593E+00_JPRB, 0.30563E+00_JPRB, 0.30622E+00_JPRB, 0.30665E+00_JPRB ,&
&  0.30525E+00_JPRB, 0.30417E+00_JPRB, 0.30312E+00_JPRB, 0.30114E+00_JPRB, 0.29788E+00_JPRB, 0.29641E+00_JPRB, 0.29531E+00_JPRB ,&
&  0.29324E+00_JPRB, 0.29067E+00_JPRB, 0.28852E+00_JPRB, 0.28780E+00_JPRB, 0.28865E+00_JPRB, 0.28900E+00_JPRB, 0.29023E+00_JPRB ,&
&  0.29104E+00_JPRB, 0.29434E+00_JPRB, 0.28301E+00_JPRB, 0.29265E+00_JPRB, 0.30192E+00_JPRB, 0.31117E+00_JPRB, 0.32080E+00_JPRB ,&
&  0.32738E+00_JPRB, 0.33308E+00_JPRB, 0.33896E+00_JPRB, 0.33975E+00_JPRB, 0.33808E+00_JPRB, 0.33486E+00_JPRB, 0.33365E+00_JPRB ,&
&  0.33295E+00_JPRB, 0.33070E+00_JPRB, 0.32901E+00_JPRB, 0.32868E+00_JPRB, 0.32888E+00_JPRB, 0.32769E+00_JPRB, 0.32577E+00_JPRB ,&
&  0.32421E+00_JPRB, 0.32218E+00_JPRB, 0.31863E+00_JPRB, 0.31452E+00_JPRB, 0.30997E+00_JPRB, 0.30416E+00_JPRB, 0.29616E+00_JPRB ,&
&  0.28690E+00_JPRB, 0.27952E+00_JPRB, 0.27326E+00_JPRB, 0.26927E+00_JPRB, 0.26732E+00_JPRB, 0.26522E+00_JPRB, 0.26325E+00_JPRB ,&
&  0.26275E+00_JPRB, 0.26321E+00_JPRB, 0.26353E+00_JPRB, 0.26349E+00_JPRB, 0.26429E+00_JPRB, 0.26488E+00_JPRB, 0.26716E+00_JPRB ,&
&  0.26961E+00_JPRB, 0.27027E+00_JPRB, 0.26997E+00_JPRB, 0.26990E+00_JPRB, 0.26854E+00_JPRB, 0.26901E+00_JPRB, 0.27062E+00_JPRB ,&
&  0.27250E+00_JPRB, 0.27509E+00_JPRB, 0.27704E+00_JPRB, 0.27877E+00_JPRB, 0.28102E+00_JPRB, 0.28447E+00_JPRB, 0.28968E+00_JPRB ,&
&  0.29463E+00_JPRB, 0.30021E+00_JPRB, 0.30659E+00_JPRB, 0.31222E+00_JPRB, 0.31596E+00_JPRB, 0.31765E+00_JPRB, 0.31893E+00_JPRB ,&
&  0.31844E+00_JPRB, 0.31523E+00_JPRB, 0.31177E+00_JPRB, 0.30948E+00_JPRB, 0.30764E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.34571E+00_JPRB, 0.34303E+00_JPRB, 0.33947E+00_JPRB, 0.33676E+00_JPRB, 0.33395E+00_JPRB, 0.33059E+00_JPRB, 0.32866E+00_JPRB ,&
&  0.32773E+00_JPRB, 0.32752E+00_JPRB, 0.32815E+00_JPRB, 0.32799E+00_JPRB, 0.32768E+00_JPRB, 0.32812E+00_JPRB, 0.32762E+00_JPRB ,&
&  0.32667E+00_JPRB, 0.32534E+00_JPRB, 0.32369E+00_JPRB, 0.32133E+00_JPRB, 0.31833E+00_JPRB, 0.31562E+00_JPRB, 0.31193E+00_JPRB ,&
&  0.30953E+00_JPRB, 0.30842E+00_JPRB, 0.30803E+00_JPRB, 0.30832E+00_JPRB, 0.30996E+00_JPRB, 0.31210E+00_JPRB, 0.31429E+00_JPRB ,&
&  0.31502E+00_JPRB, 0.30511E+00_JPRB, 0.31035E+00_JPRB, 0.31407E+00_JPRB, 0.32406E+00_JPRB, 0.34000E+00_JPRB, 0.35377E+00_JPRB ,&
&  0.36154E+00_JPRB, 0.36859E+00_JPRB, 0.37380E+00_JPRB, 0.37581E+00_JPRB, 0.37341E+00_JPRB, 0.37133E+00_JPRB, 0.37225E+00_JPRB ,&
&  0.37299E+00_JPRB, 0.37197E+00_JPRB, 0.37051E+00_JPRB, 0.36858E+00_JPRB, 0.36588E+00_JPRB, 0.36305E+00_JPRB, 0.35853E+00_JPRB ,&
&  0.35400E+00_JPRB, 0.35082E+00_JPRB, 0.34760E+00_JPRB, 0.34427E+00_JPRB, 0.33912E+00_JPRB, 0.33286E+00_JPRB, 0.32523E+00_JPRB ,&
&  0.31694E+00_JPRB, 0.31088E+00_JPRB, 0.30680E+00_JPRB, 0.30276E+00_JPRB, 0.29871E+00_JPRB, 0.29589E+00_JPRB, 0.29276E+00_JPRB ,&
&  0.29162E+00_JPRB, 0.29231E+00_JPRB, 0.29180E+00_JPRB, 0.29154E+00_JPRB, 0.29206E+00_JPRB, 0.29164E+00_JPRB, 0.29159E+00_JPRB ,&
&  0.29008E+00_JPRB, 0.28837E+00_JPRB, 0.28728E+00_JPRB, 0.28665E+00_JPRB, 0.28619E+00_JPRB, 0.28756E+00_JPRB, 0.28998E+00_JPRB ,&
&  0.29262E+00_JPRB, 0.29642E+00_JPRB, 0.30053E+00_JPRB, 0.30358E+00_JPRB, 0.30623E+00_JPRB, 0.31058E+00_JPRB, 0.31619E+00_JPRB ,&
&  0.32267E+00_JPRB, 0.33020E+00_JPRB, 0.33751E+00_JPRB, 0.34432E+00_JPRB, 0.34816E+00_JPRB, 0.35010E+00_JPRB, 0.35174E+00_JPRB ,&
&  0.35239E+00_JPRB, 0.34930E+00_JPRB, 0.34874E+00_JPRB, 0.34879E+00_JPRB, 0.34757E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.37352E+00_JPRB, 0.37284E+00_JPRB, 0.37035E+00_JPRB, 0.36695E+00_JPRB, 0.36291E+00_JPRB, 0.36069E+00_JPRB, 0.35951E+00_JPRB ,&
&  0.36027E+00_JPRB, 0.35929E+00_JPRB, 0.35968E+00_JPRB, 0.35733E+00_JPRB, 0.35487E+00_JPRB, 0.35307E+00_JPRB, 0.35098E+00_JPRB ,&
&  0.34849E+00_JPRB, 0.34536E+00_JPRB, 0.34234E+00_JPRB, 0.33976E+00_JPRB, 0.33663E+00_JPRB, 0.33236E+00_JPRB, 0.32847E+00_JPRB ,&
&  0.32571E+00_JPRB, 0.32473E+00_JPRB, 0.32440E+00_JPRB, 0.32438E+00_JPRB, 0.32451E+00_JPRB, 0.32621E+00_JPRB, 0.32784E+00_JPRB ,&
&  0.33113E+00_JPRB, 0.30474E+00_JPRB, 0.31647E+00_JPRB, 0.32481E+00_JPRB, 0.33610E+00_JPRB, 0.35459E+00_JPRB, 0.37491E+00_JPRB ,&
&  0.39429E+00_JPRB, 0.40996E+00_JPRB, 0.41920E+00_JPRB, 0.41794E+00_JPRB, 0.41003E+00_JPRB, 0.40422E+00_JPRB, 0.40310E+00_JPRB ,&
&  0.40109E+00_JPRB, 0.39715E+00_JPRB, 0.39371E+00_JPRB, 0.39025E+00_JPRB, 0.38541E+00_JPRB, 0.37802E+00_JPRB, 0.37019E+00_JPRB ,&
&  0.36579E+00_JPRB, 0.36346E+00_JPRB, 0.36141E+00_JPRB, 0.35905E+00_JPRB, 0.35471E+00_JPRB, 0.35071E+00_JPRB, 0.34726E+00_JPRB ,&
&  0.34385E+00_JPRB, 0.33851E+00_JPRB, 0.33503E+00_JPRB, 0.33255E+00_JPRB, 0.32911E+00_JPRB, 0.32678E+00_JPRB, 0.32503E+00_JPRB ,&
&  0.32326E+00_JPRB, 0.32113E+00_JPRB, 0.31963E+00_JPRB, 0.31874E+00_JPRB, 0.31616E+00_JPRB, 0.31350E+00_JPRB, 0.31049E+00_JPRB ,&
&  0.30890E+00_JPRB, 0.30787E+00_JPRB, 0.30674E+00_JPRB, 0.30660E+00_JPRB, 0.30742E+00_JPRB, 0.30967E+00_JPRB, 0.31280E+00_JPRB ,&
&  0.31659E+00_JPRB, 0.32096E+00_JPRB, 0.32488E+00_JPRB, 0.32793E+00_JPRB, 0.33209E+00_JPRB, 0.33560E+00_JPRB, 0.34040E+00_JPRB ,&
&  0.34638E+00_JPRB, 0.35219E+00_JPRB, 0.35790E+00_JPRB, 0.36476E+00_JPRB, 0.37122E+00_JPRB, 0.37500E+00_JPRB, 0.37838E+00_JPRB ,&
&  0.38125E+00_JPRB, 0.37845E+00_JPRB, 0.37683E+00_JPRB, 0.37437E+00_JPRB, 0.37267E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.40882E+00_JPRB, 0.40738E+00_JPRB, 0.40477E+00_JPRB, 0.40059E+00_JPRB, 0.39660E+00_JPRB, 0.39349E+00_JPRB, 0.39179E+00_JPRB ,&
&  0.39196E+00_JPRB, 0.39272E+00_JPRB, 0.39154E+00_JPRB, 0.38723E+00_JPRB, 0.38317E+00_JPRB, 0.38029E+00_JPRB, 0.37861E+00_JPRB ,&
&  0.37618E+00_JPRB, 0.37224E+00_JPRB, 0.36694E+00_JPRB, 0.36180E+00_JPRB, 0.35607E+00_JPRB, 0.35138E+00_JPRB, 0.34761E+00_JPRB ,&
&  0.34463E+00_JPRB, 0.34358E+00_JPRB, 0.34307E+00_JPRB, 0.34303E+00_JPRB, 0.34429E+00_JPRB, 0.34486E+00_JPRB, 0.35068E+00_JPRB ,&
&  0.32690E+00_JPRB, 0.30021E+00_JPRB, 0.32160E+00_JPRB, 0.34194E+00_JPRB, 0.35355E+00_JPRB, 0.37673E+00_JPRB, 0.40766E+00_JPRB ,&
&  0.44066E+00_JPRB, 0.46921E+00_JPRB, 0.48738E+00_JPRB, 0.48395E+00_JPRB, 0.47030E+00_JPRB, 0.45899E+00_JPRB, 0.45140E+00_JPRB ,&
&  0.44073E+00_JPRB, 0.43008E+00_JPRB, 0.42211E+00_JPRB, 0.41340E+00_JPRB, 0.40400E+00_JPRB, 0.39487E+00_JPRB, 0.38720E+00_JPRB ,&
&  0.38082E+00_JPRB, 0.37689E+00_JPRB, 0.37584E+00_JPRB, 0.37439E+00_JPRB, 0.37333E+00_JPRB, 0.37388E+00_JPRB, 0.37313E+00_JPRB ,&
&  0.37183E+00_JPRB, 0.36937E+00_JPRB, 0.36784E+00_JPRB, 0.36636E+00_JPRB, 0.36576E+00_JPRB, 0.36397E+00_JPRB, 0.36146E+00_JPRB ,&
&  0.35867E+00_JPRB, 0.35644E+00_JPRB, 0.35502E+00_JPRB, 0.35192E+00_JPRB, 0.34505E+00_JPRB, 0.33863E+00_JPRB, 0.33498E+00_JPRB ,&
&  0.33380E+00_JPRB, 0.33354E+00_JPRB, 0.33325E+00_JPRB, 0.33407E+00_JPRB, 0.33564E+00_JPRB, 0.33789E+00_JPRB, 0.34132E+00_JPRB ,&
&  0.34604E+00_JPRB, 0.35104E+00_JPRB, 0.35491E+00_JPRB, 0.35824E+00_JPRB, 0.36351E+00_JPRB, 0.36859E+00_JPRB, 0.37370E+00_JPRB ,&
&  0.37887E+00_JPRB, 0.38429E+00_JPRB, 0.39043E+00_JPRB, 0.39957E+00_JPRB, 0.41055E+00_JPRB, 0.41895E+00_JPRB, 0.42168E+00_JPRB ,&
&  0.41960E+00_JPRB, 0.41163E+00_JPRB, 0.41161E+00_JPRB, 0.40844E+00_JPRB, 0.40754E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.42734E+00_JPRB, 0.42999E+00_JPRB, 0.42857E+00_JPRB, 0.42572E+00_JPRB, 0.42341E+00_JPRB, 0.41958E+00_JPRB, 0.41642E+00_JPRB ,&
&  0.41227E+00_JPRB, 0.40985E+00_JPRB, 0.40505E+00_JPRB, 0.40200E+00_JPRB, 0.39990E+00_JPRB, 0.39859E+00_JPRB, 0.39651E+00_JPRB ,&
&  0.39225E+00_JPRB, 0.38546E+00_JPRB, 0.37854E+00_JPRB, 0.37302E+00_JPRB, 0.36929E+00_JPRB, 0.36554E+00_JPRB, 0.36236E+00_JPRB ,&
&  0.35958E+00_JPRB, 0.35710E+00_JPRB, 0.35481E+00_JPRB, 0.35553E+00_JPRB, 0.35667E+00_JPRB, 0.35694E+00_JPRB, 0.35593E+00_JPRB ,&
&  0.36432E+00_JPRB, 0.32316E+00_JPRB, 0.35461E+00_JPRB, 0.37749E+00_JPRB, 0.38770E+00_JPRB, 0.41700E+00_JPRB, 0.46076E+00_JPRB ,&
&  0.49796E+00_JPRB, 0.53140E+00_JPRB, 0.55101E+00_JPRB, 0.54367E+00_JPRB, 0.52709E+00_JPRB, 0.51549E+00_JPRB, 0.50108E+00_JPRB ,&
&  0.47866E+00_JPRB, 0.46067E+00_JPRB, 0.44713E+00_JPRB, 0.43409E+00_JPRB, 0.42156E+00_JPRB, 0.41085E+00_JPRB, 0.40106E+00_JPRB ,&
&  0.39469E+00_JPRB, 0.39190E+00_JPRB, 0.38975E+00_JPRB, 0.38805E+00_JPRB, 0.38835E+00_JPRB, 0.39051E+00_JPRB, 0.39022E+00_JPRB ,&
&  0.38922E+00_JPRB, 0.39018E+00_JPRB, 0.38960E+00_JPRB, 0.38877E+00_JPRB, 0.38944E+00_JPRB, 0.39342E+00_JPRB, 0.39266E+00_JPRB ,&
&  0.38810E+00_JPRB, 0.38102E+00_JPRB, 0.37440E+00_JPRB, 0.36943E+00_JPRB, 0.36526E+00_JPRB, 0.36117E+00_JPRB, 0.35964E+00_JPRB ,&
&  0.35772E+00_JPRB, 0.35760E+00_JPRB, 0.35780E+00_JPRB, 0.35962E+00_JPRB, 0.36175E+00_JPRB, 0.36471E+00_JPRB, 0.36904E+00_JPRB ,&
&  0.37434E+00_JPRB, 0.37966E+00_JPRB, 0.38610E+00_JPRB, 0.39276E+00_JPRB, 0.39843E+00_JPRB, 0.40224E+00_JPRB, 0.40454E+00_JPRB ,&
&  0.40649E+00_JPRB, 0.41062E+00_JPRB, 0.41327E+00_JPRB, 0.42747E+00_JPRB, 0.43817E+00_JPRB, 0.44911E+00_JPRB, 0.44878E+00_JPRB ,&
&  0.44292E+00_JPRB, 0.43653E+00_JPRB, 0.37580E+00_JPRB, 0.40594E+00_JPRB, 0.41882E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.46888E+00_JPRB, 0.46891E+00_JPRB, 0.46577E+00_JPRB, 0.46331E+00_JPRB, 0.45726E+00_JPRB, 0.45060E+00_JPRB, 0.44451E+00_JPRB ,&
&  0.43667E+00_JPRB, 0.42884E+00_JPRB, 0.42179E+00_JPRB, 0.41930E+00_JPRB, 0.41651E+00_JPRB, 0.41306E+00_JPRB, 0.40631E+00_JPRB ,&
&  0.39758E+00_JPRB, 0.38908E+00_JPRB, 0.38149E+00_JPRB, 0.37699E+00_JPRB, 0.37616E+00_JPRB, 0.37469E+00_JPRB, 0.37219E+00_JPRB ,&
&  0.37105E+00_JPRB, 0.37250E+00_JPRB, 0.37241E+00_JPRB, 0.37218E+00_JPRB, 0.37159E+00_JPRB, 0.36718E+00_JPRB, 0.36765E+00_JPRB ,&
&  0.37886E+00_JPRB, 0.35080E+00_JPRB, 0.40793E+00_JPRB, 0.45748E+00_JPRB, 0.47869E+00_JPRB, 0.51066E+00_JPRB, 0.56048E+00_JPRB ,&
&  0.61740E+00_JPRB, 0.63946E+00_JPRB, 0.63814E+00_JPRB, 0.60852E+00_JPRB, 0.58367E+00_JPRB, 0.56353E+00_JPRB, 0.53568E+00_JPRB ,&
&  0.50925E+00_JPRB, 0.49074E+00_JPRB, 0.47595E+00_JPRB, 0.46256E+00_JPRB, 0.45218E+00_JPRB, 0.44377E+00_JPRB, 0.43697E+00_JPRB ,&
&  0.42929E+00_JPRB, 0.42187E+00_JPRB, 0.41518E+00_JPRB, 0.41130E+00_JPRB, 0.41164E+00_JPRB, 0.41523E+00_JPRB, 0.42007E+00_JPRB ,&
&  0.42438E+00_JPRB, 0.42776E+00_JPRB, 0.42801E+00_JPRB, 0.42888E+00_JPRB, 0.43117E+00_JPRB, 0.43245E+00_JPRB, 0.42838E+00_JPRB ,&
&  0.41849E+00_JPRB, 0.40784E+00_JPRB, 0.40059E+00_JPRB, 0.39631E+00_JPRB, 0.39456E+00_JPRB, 0.39469E+00_JPRB, 0.39639E+00_JPRB ,&
&  0.39598E+00_JPRB, 0.39472E+00_JPRB, 0.39336E+00_JPRB, 0.39391E+00_JPRB, 0.39522E+00_JPRB, 0.39925E+00_JPRB, 0.40511E+00_JPRB ,&
&  0.41160E+00_JPRB, 0.41815E+00_JPRB, 0.42545E+00_JPRB, 0.43247E+00_JPRB, 0.43612E+00_JPRB, 0.43993E+00_JPRB, 0.44089E+00_JPRB ,&
&  0.44107E+00_JPRB, 0.44421E+00_JPRB, 0.44902E+00_JPRB, 0.46394E+00_JPRB, 0.48711E+00_JPRB, 0.49393E+00_JPRB, 0.48013E+00_JPRB ,&
&  0.46768E+00_JPRB, 0.45315E+00_JPRB, 0.45434E+00_JPRB, 0.42165E+00_JPRB, 0.45740E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.51807E+00_JPRB, 0.51246E+00_JPRB, 0.50380E+00_JPRB, 0.49594E+00_JPRB, 0.48501E+00_JPRB, 0.47635E+00_JPRB, 0.46529E+00_JPRB ,&
&  0.45541E+00_JPRB, 0.44559E+00_JPRB, 0.43669E+00_JPRB, 0.43377E+00_JPRB, 0.42751E+00_JPRB, 0.41818E+00_JPRB, 0.40373E+00_JPRB ,&
&  0.39158E+00_JPRB, 0.38188E+00_JPRB, 0.37620E+00_JPRB, 0.37436E+00_JPRB, 0.37349E+00_JPRB, 0.37131E+00_JPRB, 0.37053E+00_JPRB ,&
&  0.37164E+00_JPRB, 0.37151E+00_JPRB, 0.37411E+00_JPRB, 0.37416E+00_JPRB, 0.37796E+00_JPRB, 0.37445E+00_JPRB, 0.37223E+00_JPRB ,&
&  0.38867E+00_JPRB, 0.39090E+00_JPRB, 0.47748E+00_JPRB, 0.56534E+00_JPRB, 0.63291E+00_JPRB, 0.67481E+00_JPRB, 0.70157E+00_JPRB ,&
&  0.72716E+00_JPRB, 0.71572E+00_JPRB, 0.68759E+00_JPRB, 0.64232E+00_JPRB, 0.60321E+00_JPRB, 0.56331E+00_JPRB, 0.52612E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.49711E+00_JPRB, 0.48866E+00_JPRB, 0.48081E+00_JPRB, 0.47230E+00_JPRB, 0.46679E+00_JPRB, 0.46390E+00_JPRB ,&
&  0.45768E+00_JPRB, 0.45009E+00_JPRB, 0.44125E+00_JPRB, 0.43743E+00_JPRB, 0.43507E+00_JPRB, 0.44306E+00_JPRB, 0.45587E+00_JPRB ,&
&  0.46244E+00_JPRB, 0.46913E+00_JPRB, 0.47422E+00_JPRB, 0.48104E+00_JPRB, 0.48124E+00_JPRB, 0.47770E+00_JPRB, 0.46367E+00_JPRB ,&
&  0.45241E+00_JPRB, 0.44497E+00_JPRB, 0.44115E+00_JPRB, 0.43910E+00_JPRB, 0.43834E+00_JPRB, 0.43626E+00_JPRB, 0.43605E+00_JPRB ,&
&  0.43332E+00_JPRB, 0.43001E+00_JPRB, 0.42930E+00_JPRB, 0.43010E+00_JPRB, 0.43133E+00_JPRB, 0.43268E+00_JPRB, 0.43469E+00_JPRB ,&
&  0.43766E+00_JPRB, 0.44039E+00_JPRB, 0.44431E+00_JPRB, 0.44793E+00_JPRB, 0.44852E+00_JPRB, 0.45052E+00_JPRB, 0.45476E+00_JPRB ,&
&  0.45807E+00_JPRB, 0.46489E+00_JPRB, 0.47129E+00_JPRB, 0.50054E+00_JPRB, 0.51684E+00_JPRB, 0.50990E+00_JPRB, 0.49216E+00_JPRB ,&
&  0.47761E+00_JPRB, 0.47135E+00_JPRB, 0.47382E+00_JPRB, 0.49358E+00_JPRB, 0.49525E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.62147E+00_JPRB, 0.58170E+00_JPRB, 0.54512E+00_JPRB, 0.52317E+00_JPRB, 0.50922E+00_JPRB, 0.49941E+00_JPRB, 0.48737E+00_JPRB ,&
&  0.47591E+00_JPRB, 0.46366E+00_JPRB, 0.45384E+00_JPRB, 0.44709E+00_JPRB, 0.43936E+00_JPRB, 0.42435E+00_JPRB, 0.40448E+00_JPRB ,&
&  0.39086E+00_JPRB, 0.38841E+00_JPRB, 0.38665E+00_JPRB, 0.38226E+00_JPRB, 0.37820E+00_JPRB, 0.37748E+00_JPRB, 0.37557E+00_JPRB ,&
&  0.37559E+00_JPRB, 0.37561E+00_JPRB, 0.37844E+00_JPRB, 0.37999E+00_JPRB, 0.37870E+00_JPRB, 0.38539E+00_JPRB, 0.38556E+00_JPRB ,&
&  0.40614E+00_JPRB, 0.42752E+00_JPRB, 0.56361E+00_JPRB, 0.71490E+00_JPRB, 0.83478E+00_JPRB, 0.87162E+00_JPRB, 0.83266E+00_JPRB ,&
&  0.79845E+00_JPRB, 0.76153E+00_JPRB, 0.71715E+00_JPRB, 0.67241E+00_JPRB, 0.62436E+00_JPRB, 0.57426E+00_JPRB, 0.54444E+00_JPRB ,&
&  0.53302E+00_JPRB, 0.52048E+00_JPRB, 0.50742E+00_JPRB, 0.49125E+00_JPRB, 0.47589E+00_JPRB, 0.47051E+00_JPRB, 0.47122E+00_JPRB ,&
&  0.46772E+00_JPRB, 0.46403E+00_JPRB, 0.46020E+00_JPRB, 0.46291E+00_JPRB, 0.47743E+00_JPRB, 0.50873E+00_JPRB, 0.53559E+00_JPRB ,&
&  0.54586E+00_JPRB, 0.55694E+00_JPRB, 0.56056E+00_JPRB, 0.55906E+00_JPRB, 0.54812E+00_JPRB, 0.53331E+00_JPRB, 0.51589E+00_JPRB ,&
&  0.50163E+00_JPRB, 0.49138E+00_JPRB, 0.48668E+00_JPRB, 0.48261E+00_JPRB, 0.47864E+00_JPRB, 0.47490E+00_JPRB, 0.47085E+00_JPRB ,&
&  0.46899E+00_JPRB, 0.46831E+00_JPRB, 0.46927E+00_JPRB, 0.46789E+00_JPRB, 0.46607E+00_JPRB, 0.46312E+00_JPRB, 0.45999E+00_JPRB ,&
&  0.45483E+00_JPRB, 0.45157E+00_JPRB, 0.45661E+00_JPRB, 0.46837E+00_JPRB, 0.48177E+00_JPRB, 0.49106E+00_JPRB, 0.49701E+00_JPRB ,&
&  0.49992E+00_JPRB, 0.51065E+00_JPRB, 0.50259E+00_JPRB, 0.53548E+00_JPRB, 0.58568E+00_JPRB, 0.56337E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.52239E+00_JPRB, 0.51245E+00_JPRB, 0.51286E+00_JPRB, 0.53484E+00_JPRB, 0.61591E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.57350E+00_JPRB, 0.53822E+00_JPRB, 0.52057E+00_JPRB, 0.51081E+00_JPRB, 0.50657E+00_JPRB, 0.50369E+00_JPRB, 0.49460E+00_JPRB ,&
&  0.48117E+00_JPRB, 0.46890E+00_JPRB, 0.46009E+00_JPRB, 0.45346E+00_JPRB, 0.44007E+00_JPRB, 0.41954E+00_JPRB, 0.40448E+00_JPRB ,&
&  0.40200E+00_JPRB, 0.39962E+00_JPRB, 0.39822E+00_JPRB, 0.39325E+00_JPRB, 0.38809E+00_JPRB, 0.38612E+00_JPRB, 0.38342E+00_JPRB ,&
&  0.37962E+00_JPRB, 0.37561E+00_JPRB, 0.37636E+00_JPRB, 0.37988E+00_JPRB, 0.38355E+00_JPRB, 0.38756E+00_JPRB, 0.39355E+00_JPRB ,&
&  0.41645E+00_JPRB, 0.51088E+00_JPRB, 0.73956E+00_JPRB, 0.10149E+01_JPRB, 0.11809E+01_JPRB, 0.10902E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.91644E+00_JPRB, 0.82421E+00_JPRB, 0.75134E+00_JPRB, 0.69198E+00_JPRB, 0.62863E+00_JPRB, 0.59260E+00_JPRB, 0.57477E+00_JPRB ,&
&  0.55958E+00_JPRB, 0.53978E+00_JPRB, 0.51941E+00_JPRB, 0.49874E+00_JPRB, 0.48466E+00_JPRB, 0.48626E+00_JPRB, 0.49472E+00_JPRB ,&
&  0.49291E+00_JPRB, 0.49347E+00_JPRB, 0.49520E+00_JPRB, 0.51372E+00_JPRB, 0.56724E+00_JPRB, 0.61709E+00_JPRB, 0.65930E+00_JPRB ,&
&  0.67216E+00_JPRB, 0.70404E+00_JPRB, 0.67772E+00_JPRB, 0.64948E+00_JPRB, 0.62023E+00_JPRB, 0.59471E+00_JPRB, 0.57401E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.54831E+00_JPRB, 0.53888E+00_JPRB, 0.53065E+00_JPRB, 0.52265E+00_JPRB, 0.51458E+00_JPRB, 0.51270E+00_JPRB ,&
&  0.51246E+00_JPRB, 0.50660E+00_JPRB, 0.50108E+00_JPRB, 0.49475E+00_JPRB, 0.48421E+00_JPRB, 0.47427E+00_JPRB, 0.46657E+00_JPRB ,&
&  0.45933E+00_JPRB, 0.45988E+00_JPRB, 0.48173E+00_JPRB, 0.50605E+00_JPRB, 0.52217E+00_JPRB, 0.53532E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54791E+00_JPRB, 0.55820E+00_JPRB, 0.54936E+00_JPRB, 0.56422E+00_JPRB, 0.60081E+00_JPRB, 0.61607E+00_JPRB, 0.58412E+00_JPRB ,&
&  0.56592E+00_JPRB, 0.55726E+00_JPRB, 0.55439E+00_JPRB, 0.55562E+00_JPRB, 0.60670E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.54802E+00_JPRB, 0.52620E+00_JPRB, 0.51603E+00_JPRB, 0.50978E+00_JPRB, 0.50522E+00_JPRB, 0.50154E+00_JPRB, 0.49247E+00_JPRB ,&
&  0.47881E+00_JPRB, 0.47044E+00_JPRB, 0.46090E+00_JPRB, 0.44721E+00_JPRB, 0.42818E+00_JPRB, 0.41731E+00_JPRB, 0.41717E+00_JPRB ,&
&  0.41644E+00_JPRB, 0.41310E+00_JPRB, 0.40719E+00_JPRB, 0.40486E+00_JPRB, 0.40067E+00_JPRB, 0.39463E+00_JPRB, 0.39137E+00_JPRB ,&
&  0.38295E+00_JPRB, 0.37898E+00_JPRB, 0.38047E+00_JPRB, 0.38533E+00_JPRB, 0.38974E+00_JPRB, 0.39657E+00_JPRB, 0.40718E+00_JPRB ,&
&  0.44992E+00_JPRB, 0.61052E+00_JPRB, 0.10765E+01_JPRB, 0.13276E+01_JPRB, 0.13657E+01_JPRB, 0.12567E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.10989E+01_JPRB, 0.96122E+00_JPRB, 0.84915E+00_JPRB, 0.75075E+00_JPRB, 0.67620E+00_JPRB, 0.63707E+00_JPRB, 0.61410E+00_JPRB ,&
&  0.58685E+00_JPRB, 0.56261E+00_JPRB, 0.53980E+00_JPRB, 0.51973E+00_JPRB, 0.51190E+00_JPRB, 0.51880E+00_JPRB, 0.52822E+00_JPRB ,&
&  0.53117E+00_JPRB, 0.53507E+00_JPRB, 0.54752E+00_JPRB, 0.58390E+00_JPRB, 0.63087E+00_JPRB, 0.66371E+00_JPRB, 0.80426E+00_JPRB ,&
&  0.95561E+00_JPRB, 0.93438E+00_JPRB, 0.85880E+00_JPRB, 0.79339E+00_JPRB, 0.73525E+00_JPRB, 0.68752E+00_JPRB, 0.65128E+00_JPRB ,&
&  0.63167E+00_JPRB, 0.61522E+00_JPRB, 0.60313E+00_JPRB, 0.58956E+00_JPRB, 0.57722E+00_JPRB, 0.56990E+00_JPRB, 0.56330E+00_JPRB ,&
&  0.55366E+00_JPRB, 0.54330E+00_JPRB, 0.53069E+00_JPRB, 0.51277E+00_JPRB, 0.49307E+00_JPRB, 0.48193E+00_JPRB, 0.47787E+00_JPRB ,&
&  0.47926E+00_JPRB, 0.49938E+00_JPRB, 0.52616E+00_JPRB, 0.53953E+00_JPRB, 0.56855E+00_JPRB, 0.59065E+00_JPRB, 0.60792E+00_JPRB ,&
&  0.58735E+00_JPRB, 0.58524E+00_JPRB, 0.59457E+00_JPRB, 0.61592E+00_JPRB, 0.65643E+00_JPRB, 0.67460E+00_JPRB, 0.63173E+00_JPRB ,&
&  0.61324E+00_JPRB, 0.60837E+00_JPRB, 0.59702E+00_JPRB, 0.57856E+00_JPRB, 0.57766E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.54213E+00_JPRB, 0.52899E+00_JPRB, 0.51820E+00_JPRB, 0.51309E+00_JPRB, 0.50846E+00_JPRB, 0.49815E+00_JPRB, 0.48834E+00_JPRB ,&
&  0.47841E+00_JPRB, 0.46746E+00_JPRB, 0.45274E+00_JPRB, 0.43104E+00_JPRB, 0.41692E+00_JPRB, 0.41404E+00_JPRB, 0.42019E+00_JPRB ,&
&  0.42173E+00_JPRB, 0.41498E+00_JPRB, 0.41159E+00_JPRB, 0.41151E+00_JPRB, 0.41087E+00_JPRB, 0.40517E+00_JPRB, 0.39735E+00_JPRB ,&
&  0.39064E+00_JPRB, 0.39093E+00_JPRB, 0.39322E+00_JPRB, 0.39763E+00_JPRB, 0.40133E+00_JPRB, 0.40859E+00_JPRB, 0.42579E+00_JPRB ,&
&  0.48432E+00_JPRB, 0.48419E+00_JPRB, 0.11359E+01_JPRB, 0.15993E+01_JPRB, 0.16156E+01_JPRB, 0.15289E+01_JPRB, 0.14755E+01_JPRB ,&
&  0.13476E+01_JPRB, 0.11729E+01_JPRB, 0.10178E+01_JPRB, 0.88306E+00_JPRB, 0.78111E+00_JPRB, 0.71219E+00_JPRB, 0.65987E+00_JPRB ,&
&  0.62079E+00_JPRB, 0.59016E+00_JPRB, 0.56409E+00_JPRB, 0.54978E+00_JPRB, 0.55090E+00_JPRB, 0.55352E+00_JPRB, 0.55492E+00_JPRB ,&
&  0.56025E+00_JPRB, 0.57066E+00_JPRB, 0.59810E+00_JPRB, 0.65419E+00_JPRB, 0.71678E+00_JPRB, 0.82213E+00_JPRB, 0.10309E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.12307E+01_JPRB, 0.10796E+01_JPRB, 0.94948E+00_JPRB, 0.86158E+00_JPRB, 0.80375E+00_JPRB, 0.76122E+00_JPRB ,&
&  0.72619E+00_JPRB, 0.69928E+00_JPRB, 0.67860E+00_JPRB, 0.64993E+00_JPRB, 0.63026E+00_JPRB, 0.61744E+00_JPRB, 0.59744E+00_JPRB ,&
&  0.57547E+00_JPRB, 0.55490E+00_JPRB, 0.53388E+00_JPRB, 0.51494E+00_JPRB, 0.49907E+00_JPRB, 0.49744E+00_JPRB, 0.50224E+00_JPRB ,&
&  0.51284E+00_JPRB, 0.53566E+00_JPRB, 0.54944E+00_JPRB, 0.56676E+00_JPRB, 0.59490E+00_JPRB, 0.61459E+00_JPRB, 0.62682E+00_JPRB ,&
&  0.61418E+00_JPRB, 0.62231E+00_JPRB, 0.64076E+00_JPRB, 0.66753E+00_JPRB, 0.64387E+00_JPRB, 0.70761E+00_JPRB, 0.67280E+00_JPRB ,&
&  0.66303E+00_JPRB, 0.64811E+00_JPRB, 0.60671E+00_JPRB, 0.58643E+00_JPRB, 0.56554E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.54875E+00_JPRB, 0.53251E+00_JPRB, 0.51929E+00_JPRB, 0.50787E+00_JPRB, 0.49936E+00_JPRB, 0.49424E+00_JPRB, 0.48704E+00_JPRB ,&
&  0.47752E+00_JPRB, 0.46426E+00_JPRB, 0.45161E+00_JPRB, 0.43433E+00_JPRB, 0.42686E+00_JPRB, 0.42874E+00_JPRB, 0.43637E+00_JPRB ,&
&  0.43143E+00_JPRB, 0.41927E+00_JPRB, 0.41859E+00_JPRB, 0.41961E+00_JPRB, 0.41646E+00_JPRB, 0.40947E+00_JPRB, 0.40235E+00_JPRB ,&
&  0.40469E+00_JPRB, 0.40730E+00_JPRB, 0.40952E+00_JPRB, 0.41259E+00_JPRB, 0.41663E+00_JPRB, 0.42782E+00_JPRB, 0.45235E+00_JPRB ,&
&  0.52349E+00_JPRB, 0.82799E+00_JPRB, 0.95057E+00_JPRB, 0.18526E+01_JPRB, 0.19762E+01_JPRB, 0.19093E+01_JPRB, 0.16932E+01_JPRB ,&
&  0.14789E+01_JPRB, 0.13527E+01_JPRB, 0.11814E+01_JPRB, 0.10388E+01_JPRB, 0.89274E+00_JPRB, 0.78183E+00_JPRB, 0.71089E+00_JPRB ,&
&  0.66289E+00_JPRB, 0.62615E+00_JPRB, 0.60405E+00_JPRB, 0.59571E+00_JPRB, 0.58990E+00_JPRB, 0.58660E+00_JPRB, 0.58764E+00_JPRB ,&
&  0.59552E+00_JPRB, 0.61705E+00_JPRB, 0.66088E+00_JPRB, 0.73749E+00_JPRB, 0.77561E+00_JPRB, 0.99433E+00_JPRB, 0.12816E+01_JPRB ,&
&  0.14200E+01_JPRB, 0.14699E+01_JPRB, 0.12607E+01_JPRB, 0.10748E+01_JPRB, 0.96489E+00_JPRB, 0.89265E+00_JPRB, 0.83254E+00_JPRB ,&
&  0.78965E+00_JPRB, 0.75634E+00_JPRB, 0.71976E+00_JPRB, 0.67410E+00_JPRB, 0.64887E+00_JPRB, 0.62918E+00_JPRB, 0.59885E+00_JPRB ,&
&  0.56677E+00_JPRB, 0.54495E+00_JPRB, 0.53227E+00_JPRB, 0.52318E+00_JPRB, 0.51903E+00_JPRB, 0.52492E+00_JPRB, 0.53518E+00_JPRB ,&
&  0.54918E+00_JPRB, 0.57572E+00_JPRB, 0.57662E+00_JPRB, 0.58646E+00_JPRB, 0.61256E+00_JPRB, 0.64023E+00_JPRB, 0.63420E+00_JPRB ,&
&  0.63724E+00_JPRB, 0.64823E+00_JPRB, 0.66835E+00_JPRB, 0.68861E+00_JPRB, 0.67161E+00_JPRB, 0.73995E+00_JPRB, 0.69524E+00_JPRB ,&
&  0.69106E+00_JPRB, 0.64559E+00_JPRB, 0.60641E+00_JPRB, 0.59196E+00_JPRB, 0.57313E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.54967E+00_JPRB, 0.52304E+00_JPRB, 0.50514E+00_JPRB, 0.49558E+00_JPRB, 0.49078E+00_JPRB, 0.48950E+00_JPRB, 0.48187E+00_JPRB ,&
&  0.47579E+00_JPRB, 0.46328E+00_JPRB, 0.44754E+00_JPRB, 0.43454E+00_JPRB, 0.44010E+00_JPRB, 0.44349E+00_JPRB, 0.44327E+00_JPRB ,&
&  0.43626E+00_JPRB, 0.43205E+00_JPRB, 0.43072E+00_JPRB, 0.42775E+00_JPRB, 0.42257E+00_JPRB, 0.41802E+00_JPRB, 0.41692E+00_JPRB ,&
&  0.42237E+00_JPRB, 0.42257E+00_JPRB, 0.42208E+00_JPRB, 0.42638E+00_JPRB, 0.43685E+00_JPRB, 0.46131E+00_JPRB, 0.49685E+00_JPRB ,&
&  0.59366E+00_JPRB, 0.75618E+00_JPRB, 0.92712E+00_JPRB, 0.19370E+01_JPRB, 0.22962E+01_JPRB, 0.22030E+01_JPRB, 0.18727E+01_JPRB ,&
&  0.15686E+01_JPRB, 0.15160E+01_JPRB, 0.13369E+01_JPRB, 0.11276E+01_JPRB, 0.93604E+00_JPRB, 0.82514E+00_JPRB, 0.74582E+00_JPRB ,&
&  0.69061E+00_JPRB, 0.66025E+00_JPRB, 0.65008E+00_JPRB, 0.64620E+00_JPRB, 0.63999E+00_JPRB, 0.64463E+00_JPRB, 0.65121E+00_JPRB ,&
&  0.65906E+00_JPRB, 0.68407E+00_JPRB, 0.72743E+00_JPRB, 0.74906E+00_JPRB, 0.90177E+00_JPRB, 0.12878E+01_JPRB, 0.15674E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.15771E+01_JPRB, 0.13638E+01_JPRB, 0.11474E+01_JPRB, 0.10048E+01_JPRB, 0.94408E+00_JPRB, 0.87810E+00_JPRB ,&
&  0.80899E+00_JPRB, 0.76448E+00_JPRB, 0.70668E+00_JPRB, 0.67261E+00_JPRB, 0.66011E+00_JPRB, 0.61601E+00_JPRB, 0.58055E+00_JPRB ,&
&  0.56261E+00_JPRB, 0.55499E+00_JPRB, 0.55071E+00_JPRB, 0.54669E+00_JPRB, 0.54950E+00_JPRB, 0.55790E+00_JPRB, 0.57274E+00_JPRB ,&
&  0.59362E+00_JPRB, 0.62428E+00_JPRB, 0.61514E+00_JPRB, 0.62536E+00_JPRB, 0.66093E+00_JPRB, 0.68862E+00_JPRB, 0.68199E+00_JPRB ,&
&  0.67487E+00_JPRB, 0.67889E+00_JPRB, 0.68846E+00_JPRB, 0.67318E+00_JPRB, 0.66333E+00_JPRB, 0.72035E+00_JPRB, 0.71094E+00_JPRB ,&
&  0.71690E+00_JPRB, 0.66235E+00_JPRB, 0.63407E+00_JPRB, 0.59814E+00_JPRB, 0.57594E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.53128E+00_JPRB, 0.50565E+00_JPRB, 0.50018E+00_JPRB, 0.49633E+00_JPRB, 0.48529E+00_JPRB, 0.47610E+00_JPRB, 0.46678E+00_JPRB ,&
&  0.45892E+00_JPRB, 0.44495E+00_JPRB, 0.43521E+00_JPRB, 0.44177E+00_JPRB, 0.45800E+00_JPRB, 0.46852E+00_JPRB, 0.46070E+00_JPRB ,&
&  0.45166E+00_JPRB, 0.44587E+00_JPRB, 0.44976E+00_JPRB, 0.45081E+00_JPRB, 0.44691E+00_JPRB, 0.44862E+00_JPRB, 0.44741E+00_JPRB ,&
&  0.44821E+00_JPRB, 0.44546E+00_JPRB, 0.44830E+00_JPRB, 0.46124E+00_JPRB, 0.48442E+00_JPRB, 0.52455E+00_JPRB, 0.58203E+00_JPRB ,&
&  0.74040E+00_JPRB, 0.82863E+00_JPRB, 0.10019E+01_JPRB, 0.16953E+01_JPRB, 0.21526E+01_JPRB, 0.20159E+01_JPRB, 0.18243E+01_JPRB ,&
&  0.15795E+01_JPRB, 0.12562E+01_JPRB, 0.12041E+01_JPRB, 0.11061E+01_JPRB, 0.95370E+00_JPRB, 0.83453E+00_JPRB, 0.76024E+00_JPRB ,&
&  0.72065E+00_JPRB, 0.69495E+00_JPRB, 0.69166E+00_JPRB, 0.70289E+00_JPRB, 0.72257E+00_JPRB, 0.74557E+00_JPRB, 0.75577E+00_JPRB ,&
&  0.76141E+00_JPRB, 0.77640E+00_JPRB, 0.81683E+00_JPRB, 0.87360E+00_JPRB, 0.11761E+01_JPRB, 0.16604E+01_JPRB, 0.18519E+01_JPRB ,&
&  0.17785E+01_JPRB, 0.16104E+01_JPRB, 0.13457E+01_JPRB, 0.11316E+01_JPRB, 0.92798E+00_JPRB, 0.93219E+00_JPRB, 0.86249E+00_JPRB ,&
&  0.78527E+00_JPRB, 0.72914E+00_JPRB, 0.69754E+00_JPRB, 0.66746E+00_JPRB, 0.62990E+00_JPRB, 0.59462E+00_JPRB, 0.57571E+00_JPRB ,&
&  0.57508E+00_JPRB, 0.58114E+00_JPRB, 0.58060E+00_JPRB, 0.57501E+00_JPRB, 0.58583E+00_JPRB, 0.59408E+00_JPRB, 0.61423E+00_JPRB ,&
&  0.63565E+00_JPRB, 0.66215E+00_JPRB, 0.66394E+00_JPRB, 0.69382E+00_JPRB, 0.73100E+00_JPRB, 0.75544E+00_JPRB, 0.74353E+00_JPRB ,&
&  0.71113E+00_JPRB, 0.73036E+00_JPRB, 0.73543E+00_JPRB, 0.71573E+00_JPRB, 0.71183E+00_JPRB, 0.71368E+00_JPRB, 0.70744E+00_JPRB ,&
&  0.73436E+00_JPRB, 0.72574E+00_JPRB, 0.68621E+00_JPRB, 0.59827E+00_JPRB, 0.56157E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.51922E+00_JPRB, 0.50068E+00_JPRB, 0.49994E+00_JPRB, 0.49260E+00_JPRB, 0.48051E+00_JPRB, 0.47354E+00_JPRB, 0.46596E+00_JPRB ,&
&  0.45895E+00_JPRB, 0.45910E+00_JPRB, 0.46215E+00_JPRB, 0.49072E+00_JPRB, 0.50361E+00_JPRB, 0.49584E+00_JPRB, 0.48676E+00_JPRB ,&
&  0.48076E+00_JPRB, 0.48112E+00_JPRB, 0.48338E+00_JPRB, 0.48461E+00_JPRB, 0.48441E+00_JPRB, 0.48990E+00_JPRB, 0.49705E+00_JPRB ,&
&  0.49597E+00_JPRB, 0.49846E+00_JPRB, 0.51015E+00_JPRB, 0.53503E+00_JPRB, 0.58188E+00_JPRB, 0.65588E+00_JPRB, 0.77608E+00_JPRB ,&
&  0.10195E+01_JPRB, 0.10087E+01_JPRB, 0.50997E+00_JPRB, 0.10129E+01_JPRB, 0.16468E+01_JPRB, 0.17203E+01_JPRB, 0.16262E+01_JPRB ,&
&  0.14402E+01_JPRB, 0.11109E+01_JPRB, 0.10294E+01_JPRB, 0.10564E+01_JPRB, 0.92658E+00_JPRB, 0.82870E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.74182E+00_JPRB, 0.74493E+00_JPRB, 0.78279E+00_JPRB, 0.81930E+00_JPRB, 0.85460E+00_JPRB, 0.86986E+00_JPRB, 0.86812E+00_JPRB ,&
&  0.87073E+00_JPRB, 0.89807E+00_JPRB, 0.98455E+00_JPRB, 0.10963E+01_JPRB, 0.16388E+01_JPRB, 0.18882E+01_JPRB, 0.19059E+01_JPRB ,&
&  0.14534E+01_JPRB, 0.13204E+01_JPRB, 0.11817E+01_JPRB, 0.10415E+01_JPRB, 0.89315E+00_JPRB, 0.81184E+00_JPRB, 0.81455E+00_JPRB ,&
&  0.73488E+00_JPRB, 0.67738E+00_JPRB, 0.65942E+00_JPRB, 0.63812E+00_JPRB, 0.61514E+00_JPRB, 0.60211E+00_JPRB, 0.59731E+00_JPRB ,&
&  0.60756E+00_JPRB, 0.61719E+00_JPRB, 0.62042E+00_JPRB, 0.62006E+00_JPRB, 0.62573E+00_JPRB, 0.63519E+00_JPRB, 0.65487E+00_JPRB ,&
&  0.67023E+00_JPRB, 0.70193E+00_JPRB, 0.74298E+00_JPRB, 0.74830E+00_JPRB, 0.76883E+00_JPRB, 0.81307E+00_JPRB, 0.80273E+00_JPRB ,&
&  0.77929E+00_JPRB, 0.81547E+00_JPRB, 0.77065E+00_JPRB, 0.71215E+00_JPRB, 0.72211E+00_JPRB, 0.70960E+00_JPRB, 0.71918E+00_JPRB ,&
&  0.77938E+00_JPRB, 0.91347E+00_JPRB, 0.88680E+00_JPRB, 0.58603E+00_JPRB, 0.53610E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.48438E+00_JPRB, 0.46953E+00_JPRB, 0.46406E+00_JPRB, 0.46979E+00_JPRB, 0.47342E+00_JPRB, 0.47847E+00_JPRB, 0.46597E+00_JPRB ,&
&  0.47712E+00_JPRB, 0.50867E+00_JPRB, 0.53365E+00_JPRB, 0.54925E+00_JPRB, 0.54412E+00_JPRB, 0.53410E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.53019E+00_JPRB, 0.53393E+00_JPRB, 0.54589E+00_JPRB, 0.55128E+00_JPRB, 0.55461E+00_JPRB, 0.56293E+00_JPRB, 0.56996E+00_JPRB ,&
&  0.57733E+00_JPRB, 0.59262E+00_JPRB, 0.62102E+00_JPRB, 0.67491E+00_JPRB, 0.76275E+00_JPRB, 0.89823E+00_JPRB, 0.10706E+01_JPRB ,&
&  0.13244E+01_JPRB, 0.92418E+00_JPRB, 0.52787E+00_JPRB, 0.12830E+01_JPRB, 0.16379E+01_JPRB, 0.16303E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.10699E+01_JPRB, 0.97573E+00_JPRB, 0.10278E+01_JPRB, 0.90028E+00_JPRB, 0.84984E+00_JPRB, 0.83382E+00_JPRB ,&
&  0.84938E+00_JPRB, 0.88643E+00_JPRB, 0.92389E+00_JPRB, 0.95136E+00_JPRB, 0.95884E+00_JPRB, 0.95132E+00_JPRB, 0.95471E+00_JPRB ,&
&  0.10036E+01_JPRB, 0.10820E+01_JPRB, 0.12335E+01_JPRB, 0.14490E+01_JPRB, 0.19440E+01_JPRB, 0.18064E+01_JPRB, 0.17341E+01_JPRB ,&
&  0.11997E+01_JPRB, 0.10839E+01_JPRB, 0.10368E+01_JPRB, 0.97072E+00_JPRB, 0.84573E+00_JPRB, 0.73356E+00_JPRB, 0.74335E+00_JPRB ,&
&  0.67888E+00_JPRB, 0.64226E+00_JPRB, 0.63403E+00_JPRB, 0.61782E+00_JPRB, 0.60431E+00_JPRB, 0.60835E+00_JPRB, 0.62584E+00_JPRB ,&
&  0.65203E+00_JPRB, 0.65978E+00_JPRB, 0.66080E+00_JPRB, 0.65857E+00_JPRB, 0.66198E+00_JPRB, 0.66977E+00_JPRB, 0.67760E+00_JPRB ,&
&  0.69841E+00_JPRB, 0.73132E+00_JPRB, 0.76739E+00_JPRB, 0.78152E+00_JPRB, 0.78321E+00_JPRB, 0.80471E+00_JPRB, 0.84645E+00_JPRB ,&
&  0.81997E+00_JPRB, 0.82644E+00_JPRB, 0.77013E+00_JPRB, 0.74918E+00_JPRB, 0.70742E+00_JPRB, 0.69968E+00_JPRB, 0.69611E+00_JPRB ,&
&  0.68021E+00_JPRB, 0.62572E+00_JPRB, 0.56264E+00_JPRB, 0.49430E+00_JPRB, 0.48568E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.46964E+00_JPRB, 0.45946E+00_JPRB, 0.45593E+00_JPRB, 0.47130E+00_JPRB, 0.46818E+00_JPRB, 0.47900E+00_JPRB, 0.49409E+00_JPRB ,&
&  0.53601E+00_JPRB, 0.57466E+00_JPRB, 0.60649E+00_JPRB, 0.59928E+00_JPRB, 0.59832E+00_JPRB, 0.58724E+00_JPRB, 0.59271E+00_JPRB ,&
&  0.60383E+00_JPRB, 0.62237E+00_JPRB, 0.64258E+00_JPRB, 0.65092E+00_JPRB, 0.65745E+00_JPRB, 0.66638E+00_JPRB, 0.67993E+00_JPRB ,&
&  0.70089E+00_JPRB, 0.73135E+00_JPRB, 0.78527E+00_JPRB, 0.88041E+00_JPRB, 0.10271E+01_JPRB, 0.11661E+01_JPRB, 0.12633E+01_JPRB ,&
&  0.87279E+00_JPRB, 0.42009E+00_JPRB, 0.12016E+01_JPRB, 0.16056E+01_JPRB, 0.15630E+01_JPRB, 0.14720E+01_JPRB, 0.13748E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.10440E+01_JPRB, 0.95256E+00_JPRB, 0.10322E+01_JPRB, 0.93767E+00_JPRB, 0.92337E+00_JPRB, 0.94115E+00_JPRB ,&
&  0.97322E+00_JPRB, 0.10036E+01_JPRB, 0.10308E+01_JPRB, 0.10350E+01_JPRB, 0.10271E+01_JPRB, 0.10361E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.12920E+01_JPRB, 0.14596E+01_JPRB, 0.14851E+01_JPRB, 0.16719E+01_JPRB, 0.15831E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.91596E+00_JPRB, 0.87352E+00_JPRB, 0.90411E+00_JPRB, 0.81360E+00_JPRB, 0.68297E+00_JPRB, 0.68428E+00_JPRB ,&
&  0.64011E+00_JPRB, 0.62109E+00_JPRB, 0.62417E+00_JPRB, 0.61355E+00_JPRB, 0.60169E+00_JPRB, 0.61842E+00_JPRB, 0.64289E+00_JPRB ,&
&  0.66998E+00_JPRB, 0.68090E+00_JPRB, 0.67300E+00_JPRB, 0.66942E+00_JPRB, 0.67263E+00_JPRB, 0.67273E+00_JPRB, 0.67439E+00_JPRB ,&
&  0.69893E+00_JPRB, 0.72700E+00_JPRB, 0.76199E+00_JPRB, 0.79955E+00_JPRB, 0.84481E+00_JPRB, 0.84689E+00_JPRB, 0.83237E+00_JPRB ,&
&  0.80367E+00_JPRB, 0.78748E+00_JPRB, 0.74581E+00_JPRB, 0.73251E+00_JPRB, 0.71371E+00_JPRB, 0.68658E+00_JPRB, 0.66130E+00_JPRB ,&
&  0.61436E+00_JPRB, 0.53827E+00_JPRB, 0.49489E+00_JPRB, 0.48738E+00_JPRB, 0.47649E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.49412E+00_JPRB, 0.48534E+00_JPRB, 0.48898E+00_JPRB, 0.49690E+00_JPRB, 0.51036E+00_JPRB, 0.53331E+00_JPRB, 0.57053E+00_JPRB ,&
&  0.61377E+00_JPRB, 0.63642E+00_JPRB, 0.65541E+00_JPRB, 0.64408E+00_JPRB, 0.64539E+00_JPRB, 0.65999E+00_JPRB, 0.66839E+00_JPRB ,&
&  0.69415E+00_JPRB, 0.72172E+00_JPRB, 0.73681E+00_JPRB, 0.73958E+00_JPRB, 0.74717E+00_JPRB, 0.77101E+00_JPRB, 0.80497E+00_JPRB ,&
&  0.84719E+00_JPRB, 0.90451E+00_JPRB, 0.98343E+00_JPRB, 0.10932E+01_JPRB, 0.12171E+01_JPRB, 0.12576E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.65398E+00_JPRB, 0.12058E+01_JPRB, 0.16016E+01_JPRB, 0.15765E+01_JPRB, 0.14825E+01_JPRB, 0.13948E+01_JPRB, 0.13120E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.10114E+01_JPRB, 0.98435E+00_JPRB, 0.10692E+01_JPRB, 0.10010E+01_JPRB, 0.10107E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10902E+01_JPRB, 0.10983E+01_JPRB, 0.11002E+01_JPRB, 0.11218E+01_JPRB, 0.11667E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.13089E+01_JPRB, 0.14231E+01_JPRB, 0.15138E+01_JPRB, 0.14132E+01_JPRB, 0.13991E+01_JPRB, 0.14607E+01_JPRB, 0.16676E+01_JPRB ,&
&  0.11664E+01_JPRB, 0.82670E+00_JPRB, 0.82353E+00_JPRB, 0.85220E+00_JPRB, 0.76302E+00_JPRB, 0.69012E+00_JPRB, 0.64898E+00_JPRB ,&
&  0.62146E+00_JPRB, 0.59583E+00_JPRB, 0.58557E+00_JPRB, 0.57883E+00_JPRB, 0.59358E+00_JPRB, 0.61175E+00_JPRB, 0.63883E+00_JPRB ,&
&  0.64850E+00_JPRB, 0.64870E+00_JPRB, 0.63911E+00_JPRB, 0.63400E+00_JPRB, 0.63916E+00_JPRB, 0.64749E+00_JPRB, 0.64780E+00_JPRB ,&
&  0.66773E+00_JPRB, 0.67513E+00_JPRB, 0.70048E+00_JPRB, 0.74668E+00_JPRB, 0.80660E+00_JPRB, 0.83010E+00_JPRB, 0.82477E+00_JPRB ,&
&  0.76913E+00_JPRB, 0.76236E+00_JPRB, 0.76486E+00_JPRB, 0.74849E+00_JPRB, 0.73146E+00_JPRB, 0.68298E+00_JPRB, 0.59269E+00_JPRB ,&
&  0.55197E+00_JPRB, 0.51362E+00_JPRB, 0.49952E+00_JPRB, 0.49423E+00_JPRB, 0.50572E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.56282E+00_JPRB, 0.55033E+00_JPRB, 0.55723E+00_JPRB, 0.57362E+00_JPRB, 0.58647E+00_JPRB, 0.61298E+00_JPRB, 0.64645E+00_JPRB ,&
&  0.68959E+00_JPRB, 0.71646E+00_JPRB, 0.72547E+00_JPRB, 0.72469E+00_JPRB, 0.73192E+00_JPRB, 0.73216E+00_JPRB, 0.74394E+00_JPRB ,&
&  0.76700E+00_JPRB, 0.79107E+00_JPRB, 0.82172E+00_JPRB, 0.84594E+00_JPRB, 0.86189E+00_JPRB, 0.90446E+00_JPRB, 0.95047E+00_JPRB ,&
&  0.10100E+01_JPRB, 0.10741E+01_JPRB, 0.11551E+01_JPRB, 0.12462E+01_JPRB, 0.13426E+01_JPRB, 0.13810E+01_JPRB, 0.13732E+01_JPRB ,&
&  0.11315E+01_JPRB, 0.16357E+01_JPRB, 0.16561E+01_JPRB, 0.14867E+01_JPRB, 0.13310E+01_JPRB, 0.12721E+01_JPRB, 0.12116E+01_JPRB ,&
&  0.11527E+01_JPRB, 0.10525E+01_JPRB, 0.10256E+01_JPRB, 0.10356E+01_JPRB, 0.10676E+01_JPRB, 0.10717E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.11665E+01_JPRB, 0.11806E+01_JPRB, 0.11952E+01_JPRB, 0.12237E+01_JPRB, 0.12723E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.14554E+01_JPRB, 0.15139E+01_JPRB, 0.14115E+01_JPRB, 0.13351E+01_JPRB, 0.14439E+01_JPRB, 0.13944E+01_JPRB ,&
&  0.10066E+01_JPRB, 0.75290E+00_JPRB, 0.77511E+00_JPRB, 0.80557E+00_JPRB, 0.73416E+00_JPRB, 0.66747E+00_JPRB, 0.62177E+00_JPRB ,&
&  0.58636E+00_JPRB, 0.55357E+00_JPRB, 0.55731E+00_JPRB, 0.56340E+00_JPRB, 0.56998E+00_JPRB, 0.57388E+00_JPRB, 0.58926E+00_JPRB ,&
&  0.60425E+00_JPRB, 0.59696E+00_JPRB, 0.58700E+00_JPRB, 0.58019E+00_JPRB, 0.59087E+00_JPRB, 0.61003E+00_JPRB, 0.63317E+00_JPRB ,&
&  0.68676E+00_JPRB, 0.64956E+00_JPRB, 0.66428E+00_JPRB, 0.71043E+00_JPRB, 0.75771E+00_JPRB, 0.81254E+00_JPRB, 0.86313E+00_JPRB ,&
&  0.89970E+00_JPRB, 0.93727E+00_JPRB, 0.98806E+00_JPRB, 0.99847E+00_JPRB, 0.92722E+00_JPRB, 0.84246E+00_JPRB, 0.60503E+00_JPRB ,&
&  0.59326E+00_JPRB, 0.58279E+00_JPRB, 0.58657E+00_JPRB, 0.58565E+00_JPRB, 0.58350E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.68342E+00_JPRB, 0.68042E+00_JPRB, 0.70371E+00_JPRB, 0.72727E+00_JPRB, 0.74776E+00_JPRB, 0.76716E+00_JPRB, 0.78099E+00_JPRB ,&
&  0.80107E+00_JPRB, 0.81631E+00_JPRB, 0.81727E+00_JPRB, 0.81083E+00_JPRB, 0.82203E+00_JPRB, 0.82000E+00_JPRB, 0.84003E+00_JPRB ,&
&  0.87016E+00_JPRB, 0.90581E+00_JPRB, 0.95110E+00_JPRB, 0.97591E+00_JPRB, 0.99696E+00_JPRB, 0.10355E+01_JPRB, 0.10697E+01_JPRB ,&
&  0.11098E+01_JPRB, 0.11632E+01_JPRB, 0.12137E+01_JPRB, 0.12724E+01_JPRB, 0.13252E+01_JPRB, 0.13100E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.15104E+01_JPRB, 0.17550E+01_JPRB, 0.16469E+01_JPRB, 0.14015E+01_JPRB, 0.12594E+01_JPRB, 0.12014E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.11058E+01_JPRB, 0.10467E+01_JPRB, 0.10279E+01_JPRB, 0.11383E+01_JPRB, 0.11163E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.12022E+01_JPRB, 0.12372E+01_JPRB, 0.12659E+01_JPRB, 0.12894E+01_JPRB, 0.13048E+01_JPRB, 0.13281E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.13809E+01_JPRB, 0.14036E+01_JPRB, 0.14633E+01_JPRB, 0.13288E+01_JPRB, 0.12872E+01_JPRB, 0.13684E+01_JPRB, 0.12818E+01_JPRB ,&
&  0.95783E+00_JPRB, 0.74191E+00_JPRB, 0.75101E+00_JPRB, 0.76900E+00_JPRB, 0.72386E+00_JPRB, 0.65368E+00_JPRB, 0.61768E+00_JPRB ,&
&  0.58881E+00_JPRB, 0.56788E+00_JPRB, 0.56222E+00_JPRB, 0.55043E+00_JPRB, 0.55282E+00_JPRB, 0.56630E+00_JPRB, 0.57110E+00_JPRB ,&
&  0.59208E+00_JPRB, 0.58049E+00_JPRB, 0.57600E+00_JPRB, 0.57847E+00_JPRB, 0.60890E+00_JPRB, 0.67027E+00_JPRB, 0.80388E+00_JPRB ,&
&  0.92632E+00_JPRB, 0.59070E+00_JPRB, 0.64466E+00_JPRB, 0.69315E+00_JPRB, 0.76777E+00_JPRB, 0.89442E+00_JPRB, 0.10926E+01_JPRB ,&
&  0.14307E+01_JPRB, 0.19061E+01_JPRB, 0.18599E+01_JPRB, 0.14769E+01_JPRB, 0.18803E+01_JPRB, 0.15714E+01_JPRB, 0.68994E+00_JPRB ,&
&  0.69421E+00_JPRB, 0.67600E+00_JPRB, 0.68182E+00_JPRB, 0.68782E+00_JPRB, 0.68952E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.77372E+00_JPRB, 0.79155E+00_JPRB, 0.81182E+00_JPRB, 0.83000E+00_JPRB, 0.84033E+00_JPRB, 0.84951E+00_JPRB, 0.86134E+00_JPRB ,&
&  0.86644E+00_JPRB, 0.86618E+00_JPRB, 0.87578E+00_JPRB, 0.89201E+00_JPRB, 0.91263E+00_JPRB, 0.93395E+00_JPRB, 0.96140E+00_JPRB ,&
&  0.99799E+00_JPRB, 0.10308E+01_JPRB, 0.10721E+01_JPRB, 0.10973E+01_JPRB, 0.11080E+01_JPRB, 0.11196E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.11512E+01_JPRB, 0.11656E+01_JPRB, 0.11822E+01_JPRB, 0.12245E+01_JPRB, 0.12536E+01_JPRB, 0.12760E+01_JPRB, 0.10718E+01_JPRB ,&
&  0.14209E+01_JPRB, 0.16802E+01_JPRB, 0.15204E+01_JPRB, 0.12988E+01_JPRB, 0.11876E+01_JPRB, 0.11687E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.11616E+01_JPRB, 0.11446E+01_JPRB, 0.10611E+01_JPRB, 0.10921E+01_JPRB, 0.11172E+01_JPRB, 0.11087E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.12285E+01_JPRB, 0.12703E+01_JPRB, 0.13114E+01_JPRB, 0.13349E+01_JPRB, 0.13647E+01_JPRB, 0.13545E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.13461E+01_JPRB, 0.13586E+01_JPRB, 0.12348E+01_JPRB, 0.11988E+01_JPRB, 0.11579E+01_JPRB, 0.98642E+00_JPRB ,&
&  0.84685E+00_JPRB, 0.74526E+00_JPRB, 0.71600E+00_JPRB, 0.79103E+00_JPRB, 0.76838E+00_JPRB, 0.70199E+00_JPRB, 0.65414E+00_JPRB ,&
&  0.61531E+00_JPRB, 0.58251E+00_JPRB, 0.59230E+00_JPRB, 0.57829E+00_JPRB, 0.53407E+00_JPRB, 0.54779E+00_JPRB, 0.56723E+00_JPRB ,&
&  0.55475E+00_JPRB, 0.56795E+00_JPRB, 0.58872E+00_JPRB, 0.58464E+00_JPRB, 0.59343E+00_JPRB, 0.62300E+00_JPRB, 0.61678E+00_JPRB ,&
&  0.67986E+00_JPRB, 0.62489E+00_JPRB, 0.64397E+00_JPRB, 0.70496E+00_JPRB, 0.70405E+00_JPRB, 0.91771E+00_JPRB, 0.12682E+01_JPRB ,&
&  0.18576E+01_JPRB, 0.20283E+01_JPRB, 0.26517E+01_JPRB, 0.41434E+01_JPRB, 0.31400E+01_JPRB, 0.20755E+01_JPRB, 0.78214E+00_JPRB ,&
&  0.78085E+00_JPRB, 0.77193E+00_JPRB, 0.78427E+00_JPRB, 0.77846E+00_JPRB, 0.77759E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.78115E+00_JPRB, 0.80480E+00_JPRB, 0.82378E+00_JPRB, 0.84260E+00_JPRB, 0.85912E+00_JPRB, 0.87442E+00_JPRB, 0.89204E+00_JPRB ,&
&  0.90353E+00_JPRB, 0.92098E+00_JPRB, 0.94244E+00_JPRB, 0.96390E+00_JPRB, 0.98108E+00_JPRB, 0.10066E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.10781E+01_JPRB, 0.10952E+01_JPRB, 0.10941E+01_JPRB, 0.10885E+01_JPRB, 0.10942E+01_JPRB, 0.10958E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10869E+01_JPRB, 0.10961E+01_JPRB, 0.11166E+01_JPRB, 0.11243E+01_JPRB, 0.11604E+01_JPRB, 0.99486E+00_JPRB ,&
&  0.13111E+01_JPRB, 0.14685E+01_JPRB, 0.13634E+01_JPRB, 0.11957E+01_JPRB, 0.11053E+01_JPRB, 0.10960E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11431E+01_JPRB, 0.10591E+01_JPRB, 0.10166E+01_JPRB, 0.10047E+01_JPRB, 0.10087E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10996E+01_JPRB, 0.11386E+01_JPRB, 0.11737E+01_JPRB, 0.11942E+01_JPRB, 0.11996E+01_JPRB, 0.11779E+01_JPRB ,&
&  0.11366E+01_JPRB, 0.10988E+01_JPRB, 0.10149E+01_JPRB, 0.99575E+00_JPRB, 0.96035E+00_JPRB, 0.82867E+00_JPRB, 0.68961E+00_JPRB ,&
&  0.63874E+00_JPRB, 0.70153E+00_JPRB, 0.75322E+00_JPRB, 0.89108E+00_JPRB, 0.85542E+00_JPRB, 0.79126E+00_JPRB, 0.72625E+00_JPRB ,&
&  0.66488E+00_JPRB, 0.61653E+00_JPRB, 0.60833E+00_JPRB, 0.57986E+00_JPRB, 0.56141E+00_JPRB, 0.55269E+00_JPRB, 0.54382E+00_JPRB ,&
&  0.55320E+00_JPRB, 0.56229E+00_JPRB, 0.57712E+00_JPRB, 0.58427E+00_JPRB, 0.57679E+00_JPRB, 0.51966E+00_JPRB, 0.66496E+00_JPRB ,&
&  0.63157E+00_JPRB, 0.56783E+00_JPRB, 0.67407E+00_JPRB, 0.51589E+00_JPRB, 0.62651E+00_JPRB, 0.69113E+00_JPRB, 0.82153E+00_JPRB ,&
&  0.91153E+00_JPRB, 0.12859E+01_JPRB, 0.17765E+01_JPRB, 0.14086E+01_JPRB, 0.11222E+01_JPRB, 0.83878E+00_JPRB, 0.74597E+00_JPRB ,&
&  0.74169E+00_JPRB, 0.74319E+00_JPRB, 0.75037E+00_JPRB, 0.75133E+00_JPRB, 0.76000E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.78606E+00_JPRB, 0.80690E+00_JPRB, 0.82243E+00_JPRB, 0.84086E+00_JPRB, 0.85966E+00_JPRB, 0.87723E+00_JPRB, 0.89431E+00_JPRB ,&
&  0.90853E+00_JPRB, 0.92026E+00_JPRB, 0.93016E+00_JPRB, 0.94417E+00_JPRB, 0.96195E+00_JPRB, 0.97626E+00_JPRB, 0.99497E+00_JPRB ,&
&  0.10238E+01_JPRB, 0.10459E+01_JPRB, 0.10692E+01_JPRB, 0.10642E+01_JPRB, 0.10613E+01_JPRB, 0.10779E+01_JPRB, 0.10952E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.11152E+01_JPRB, 0.11394E+01_JPRB, 0.11667E+01_JPRB, 0.12007E+01_JPRB, 0.12563E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.13287E+01_JPRB, 0.12417E+01_JPRB, 0.10813E+01_JPRB, 0.10085E+01_JPRB, 0.97633E+00_JPRB, 0.96602E+00_JPRB ,&
&  0.96143E+00_JPRB, 0.92669E+00_JPRB, 0.90122E+00_JPRB, 0.87316E+00_JPRB, 0.86270E+00_JPRB, 0.86790E+00_JPRB, 0.87338E+00_JPRB ,&
&  0.88383E+00_JPRB, 0.89640E+00_JPRB, 0.92245E+00_JPRB, 0.95551E+00_JPRB, 0.98484E+00_JPRB, 0.96513E+00_JPRB, 0.93012E+00_JPRB ,&
&  0.87421E+00_JPRB, 0.81930E+00_JPRB, 0.67108E+00_JPRB, 0.62798E+00_JPRB, 0.59664E+00_JPRB, 0.53680E+00_JPRB, 0.47779E+00_JPRB ,&
&  0.52922E+00_JPRB, 0.60963E+00_JPRB, 0.76050E+00_JPRB, 0.94859E+00_JPRB, 0.90266E+00_JPRB, 0.84906E+00_JPRB, 0.77666E+00_JPRB ,&
&  0.69990E+00_JPRB, 0.64451E+00_JPRB, 0.62039E+00_JPRB, 0.58682E+00_JPRB, 0.57511E+00_JPRB, 0.57963E+00_JPRB, 0.55048E+00_JPRB ,&
&  0.54699E+00_JPRB, 0.54077E+00_JPRB, 0.54166E+00_JPRB, 0.53999E+00_JPRB, 0.55712E+00_JPRB, 0.49196E+00_JPRB, 0.59890E+00_JPRB ,&
&  0.60378E+00_JPRB, 0.45894E+00_JPRB, 0.51298E+00_JPRB, 0.59329E+00_JPRB, 0.58349E+00_JPRB, 0.59731E+00_JPRB, 0.67936E+00_JPRB ,&
&  0.83345E+00_JPRB, 0.94457E+00_JPRB, 0.96145E+00_JPRB, 0.90271E+00_JPRB, 0.80704E+00_JPRB, 0.75093E+00_JPRB, 0.72271E+00_JPRB ,&
&  0.73133E+00_JPRB, 0.73977E+00_JPRB, 0.75074E+00_JPRB, 0.75341E+00_JPRB, 0.76363E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.77110E+00_JPRB, 0.79346E+00_JPRB, 0.80641E+00_JPRB, 0.81966E+00_JPRB, 0.83675E+00_JPRB, 0.84548E+00_JPRB, 0.85293E+00_JPRB ,&
&  0.85541E+00_JPRB, 0.85222E+00_JPRB, 0.85863E+00_JPRB, 0.87141E+00_JPRB, 0.87749E+00_JPRB, 0.87537E+00_JPRB, 0.88111E+00_JPRB ,&
&  0.89393E+00_JPRB, 0.91432E+00_JPRB, 0.93568E+00_JPRB, 0.92819E+00_JPRB, 0.92217E+00_JPRB, 0.93273E+00_JPRB, 0.94917E+00_JPRB ,&
&  0.94965E+00_JPRB, 0.94708E+00_JPRB, 0.95540E+00_JPRB, 0.96537E+00_JPRB, 0.98354E+00_JPRB, 0.98758E+00_JPRB, 0.10860E+01_JPRB ,&
&  0.93269E+00_JPRB, 0.12298E+01_JPRB, 0.11488E+01_JPRB, 0.97573E+00_JPRB, 0.94558E+00_JPRB, 0.86913E+00_JPRB, 0.84500E+00_JPRB ,&
&  0.82817E+00_JPRB, 0.79960E+00_JPRB, 0.77144E+00_JPRB, 0.74848E+00_JPRB, 0.74384E+00_JPRB, 0.74968E+00_JPRB, 0.73626E+00_JPRB ,&
&  0.73634E+00_JPRB, 0.74831E+00_JPRB, 0.76957E+00_JPRB, 0.79427E+00_JPRB, 0.80637E+00_JPRB, 0.79528E+00_JPRB, 0.75228E+00_JPRB ,&
&  0.72057E+00_JPRB, 0.66731E+00_JPRB, 0.51754E+00_JPRB, 0.49121E+00_JPRB, 0.48493E+00_JPRB, 0.43192E+00_JPRB, 0.40918E+00_JPRB ,&
&  0.50447E+00_JPRB, 0.60783E+00_JPRB, 0.83836E+00_JPRB, 0.95699E+00_JPRB, 0.94958E+00_JPRB, 0.96712E+00_JPRB, 0.85768E+00_JPRB ,&
&  0.75401E+00_JPRB, 0.68144E+00_JPRB, 0.66199E+00_JPRB, 0.62018E+00_JPRB, 0.60309E+00_JPRB, 0.58571E+00_JPRB, 0.55895E+00_JPRB ,&
&  0.54319E+00_JPRB, 0.52540E+00_JPRB, 0.51326E+00_JPRB, 0.51140E+00_JPRB, 0.43501E+00_JPRB, 0.44280E+00_JPRB, 0.53896E+00_JPRB ,&
&  0.53132E+00_JPRB, 0.44891E+00_JPRB, 0.42458E+00_JPRB, 0.54120E+00_JPRB, 0.53070E+00_JPRB, 0.53335E+00_JPRB, 0.59611E+00_JPRB ,&
&  0.70927E+00_JPRB, 0.74883E+00_JPRB, 0.74978E+00_JPRB, 0.73705E+00_JPRB, 0.71274E+00_JPRB, 0.69100E+00_JPRB, 0.68241E+00_JPRB ,&
&  0.69732E+00_JPRB, 0.71169E+00_JPRB, 0.72792E+00_JPRB, 0.73622E+00_JPRB, 0.74908E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.74798E+00_JPRB, 0.76190E+00_JPRB, 0.76197E+00_JPRB, 0.77656E+00_JPRB, 0.79498E+00_JPRB, 0.79996E+00_JPRB, 0.80275E+00_JPRB ,&
&  0.79390E+00_JPRB, 0.79659E+00_JPRB, 0.79194E+00_JPRB, 0.80758E+00_JPRB, 0.80778E+00_JPRB, 0.80930E+00_JPRB, 0.81159E+00_JPRB ,&
&  0.82372E+00_JPRB, 0.83255E+00_JPRB, 0.85964E+00_JPRB, 0.86440E+00_JPRB, 0.86167E+00_JPRB, 0.86766E+00_JPRB, 0.88662E+00_JPRB ,&
&  0.90887E+00_JPRB, 0.93639E+00_JPRB, 0.97312E+00_JPRB, 0.10041E+01_JPRB, 0.11078E+01_JPRB, 0.12652E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.10996E+01_JPRB, 0.10323E+01_JPRB, 0.88002E+00_JPRB, 0.85011E+00_JPRB, 0.83687E+00_JPRB, 0.78705E+00_JPRB ,&
&  0.73751E+00_JPRB, 0.70991E+00_JPRB, 0.68791E+00_JPRB, 0.67873E+00_JPRB, 0.67967E+00_JPRB, 0.67051E+00_JPRB, 0.67127E+00_JPRB ,&
&  0.65646E+00_JPRB, 0.65671E+00_JPRB, 0.74031E+00_JPRB, 0.82214E+00_JPRB, 0.86528E+00_JPRB, 0.93759E+00_JPRB, 0.87377E+00_JPRB ,&
&  0.86436E+00_JPRB, 0.84940E+00_JPRB, 0.70778E+00_JPRB, 0.65140E+00_JPRB, 0.60909E+00_JPRB, 0.57130E+00_JPRB, 0.55085E+00_JPRB ,&
&  0.61011E+00_JPRB, 0.71021E+00_JPRB, 0.74530E+00_JPRB, 0.93906E+00_JPRB, 0.10243E+01_JPRB, 0.10680E+01_JPRB, 0.90414E+00_JPRB ,&
&  0.77256E+00_JPRB, 0.69728E+00_JPRB, 0.67929E+00_JPRB, 0.64433E+00_JPRB, 0.62878E+00_JPRB, 0.58546E+00_JPRB, 0.54602E+00_JPRB ,&
&  0.53700E+00_JPRB, 0.51056E+00_JPRB, 0.48796E+00_JPRB, 0.45651E+00_JPRB, 0.46795E+00_JPRB, 0.37072E+00_JPRB, 0.44233E+00_JPRB ,&
&  0.45312E+00_JPRB, 0.47642E+00_JPRB, 0.44253E+00_JPRB, 0.53653E+00_JPRB, 0.49047E+00_JPRB, 0.47383E+00_JPRB, 0.51359E+00_JPRB ,&
&  0.58253E+00_JPRB, 0.61829E+00_JPRB, 0.63203E+00_JPRB, 0.65793E+00_JPRB, 0.66430E+00_JPRB, 0.65679E+00_JPRB, 0.66434E+00_JPRB ,&
&  0.69998E+00_JPRB, 0.70577E+00_JPRB, 0.71175E+00_JPRB, 0.71321E+00_JPRB, 0.73033E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.68965E+00_JPRB, 0.70247E+00_JPRB, 0.69679E+00_JPRB, 0.70339E+00_JPRB, 0.70617E+00_JPRB, 0.71970E+00_JPRB, 0.71881E+00_JPRB ,&
&  0.71544E+00_JPRB, 0.72194E+00_JPRB, 0.70255E+00_JPRB, 0.70647E+00_JPRB, 0.70129E+00_JPRB, 0.70457E+00_JPRB, 0.69447E+00_JPRB ,&
&  0.69594E+00_JPRB, 0.70908E+00_JPRB, 0.71500E+00_JPRB, 0.72331E+00_JPRB, 0.72126E+00_JPRB, 0.70467E+00_JPRB, 0.68539E+00_JPRB ,&
&  0.72092E+00_JPRB, 0.81538E+00_JPRB, 0.92281E+00_JPRB, 0.99643E+00_JPRB, 0.11696E+01_JPRB, 0.11568E+01_JPRB, 0.12487E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.10508E+01_JPRB, 0.10862E+01_JPRB, 0.94310E+00_JPRB, 0.90123E+00_JPRB, 0.85534E+00_JPRB, 0.82503E+00_JPRB ,&
&  0.82895E+00_JPRB, 0.85278E+00_JPRB, 0.88674E+00_JPRB, 0.92716E+00_JPRB, 0.96192E+00_JPRB, 0.96268E+00_JPRB, 0.94816E+00_JPRB ,&
&  0.93729E+00_JPRB, 0.98307E+00_JPRB, 0.11174E+01_JPRB, 0.11521E+01_JPRB, 0.12172E+01_JPRB, 0.12659E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.12567E+01_JPRB, 0.12409E+01_JPRB, 0.10601E+01_JPRB, 0.94686E+00_JPRB, 0.93418E+00_JPRB, 0.86494E+00_JPRB, 0.85245E+00_JPRB ,&
&  0.90461E+00_JPRB, 0.92410E+00_JPRB, 0.78174E+00_JPRB, 0.88403E+00_JPRB, 0.98548E+00_JPRB, 0.96373E+00_JPRB, 0.94758E+00_JPRB ,&
&  0.82127E+00_JPRB, 0.74670E+00_JPRB, 0.71246E+00_JPRB, 0.69369E+00_JPRB, 0.65869E+00_JPRB, 0.60685E+00_JPRB, 0.58407E+00_JPRB ,&
&  0.53573E+00_JPRB, 0.49756E+00_JPRB, 0.46208E+00_JPRB, 0.43192E+00_JPRB, 0.41833E+00_JPRB, 0.43418E+00_JPRB, 0.41817E+00_JPRB ,&
&  0.43552E+00_JPRB, 0.46391E+00_JPRB, 0.55231E+00_JPRB, 0.55924E+00_JPRB, 0.40851E+00_JPRB, 0.43626E+00_JPRB, 0.44535E+00_JPRB ,&
&  0.46932E+00_JPRB, 0.50157E+00_JPRB, 0.52559E+00_JPRB, 0.55666E+00_JPRB, 0.57210E+00_JPRB, 0.57304E+00_JPRB, 0.58170E+00_JPRB ,&
&  0.60625E+00_JPRB, 0.63081E+00_JPRB, 0.66563E+00_JPRB, 0.67137E+00_JPRB, 0.67336E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.59235E+00_JPRB, 0.59474E+00_JPRB, 0.59863E+00_JPRB, 0.59510E+00_JPRB, 0.59135E+00_JPRB, 0.59061E+00_JPRB, 0.59261E+00_JPRB ,&
&  0.59266E+00_JPRB, 0.58378E+00_JPRB, 0.57502E+00_JPRB, 0.56820E+00_JPRB, 0.55565E+00_JPRB, 0.53019E+00_JPRB, 0.49489E+00_JPRB ,&
&  0.50180E+00_JPRB, 0.51264E+00_JPRB, 0.52808E+00_JPRB, 0.56547E+00_JPRB, 0.64593E+00_JPRB, 0.76649E+00_JPRB, 0.92325E+00_JPRB ,&
&  0.12251E+01_JPRB, 0.15548E+01_JPRB, 0.17758E+01_JPRB, 0.18342E+01_JPRB, 0.20141E+01_JPRB, 0.14354E+01_JPRB, 0.12546E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.11688E+01_JPRB, 0.10748E+01_JPRB, 0.10004E+01_JPRB, 0.96020E+00_JPRB, 0.95436E+00_JPRB, 0.10006E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.11686E+01_JPRB, 0.12394E+01_JPRB, 0.13068E+01_JPRB, 0.13590E+01_JPRB, 0.14115E+01_JPRB, 0.14295E+01_JPRB ,&
&  0.14233E+01_JPRB, 0.14206E+01_JPRB, 0.14530E+01_JPRB, 0.14023E+01_JPRB, 0.14741E+01_JPRB, 0.15160E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.14086E+01_JPRB, 0.13733E+01_JPRB, 0.12956E+01_JPRB, 0.12438E+01_JPRB, 0.11924E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.10489E+01_JPRB, 0.88944E+00_JPRB, 0.99696E+00_JPRB, 0.10361E+01_JPRB, 0.88023E+00_JPRB, 0.95025E+00_JPRB ,&
&  0.85466E+00_JPRB, 0.81620E+00_JPRB, 0.76893E+00_JPRB, 0.72626E+00_JPRB, 0.69126E+00_JPRB, 0.71248E+00_JPRB, 0.60670E+00_JPRB ,&
&  0.55432E+00_JPRB, 0.49902E+00_JPRB, 0.43987E+00_JPRB, 0.42391E+00_JPRB, 0.42329E+00_JPRB, 0.44378E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.46123E+00_JPRB, 0.48201E+00_JPRB, 0.57568E+00_JPRB, 0.64379E+00_JPRB, 0.50595E+00_JPRB, 0.40982E+00_JPRB, 0.41289E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.42152E+00_JPRB, 0.43009E+00_JPRB, 0.45259E+00_JPRB, 0.47370E+00_JPRB, 0.49099E+00_JPRB, 0.50461E+00_JPRB ,&
&  0.51287E+00_JPRB, 0.53389E+00_JPRB, 0.55415E+00_JPRB, 0.57163E+00_JPRB, 0.58775E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.50535E+00_JPRB, 0.50641E+00_JPRB, 0.51544E+00_JPRB, 0.51570E+00_JPRB, 0.52889E+00_JPRB, 0.52907E+00_JPRB, 0.53089E+00_JPRB ,&
&  0.53940E+00_JPRB, 0.55453E+00_JPRB, 0.54585E+00_JPRB, 0.52504E+00_JPRB, 0.52387E+00_JPRB, 0.54064E+00_JPRB, 0.58091E+00_JPRB ,&
&  0.66483E+00_JPRB, 0.74855E+00_JPRB, 0.81895E+00_JPRB, 0.92773E+00_JPRB, 0.11306E+01_JPRB, 0.14648E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.20183E+01_JPRB, 0.21619E+01_JPRB, 0.21301E+01_JPRB, 0.21981E+01_JPRB, 0.17417E+01_JPRB, 0.11415E+01_JPRB, 0.12715E+01_JPRB ,&
&  0.13534E+01_JPRB, 0.13176E+01_JPRB, 0.11509E+01_JPRB, 0.10925E+01_JPRB, 0.10600E+01_JPRB, 0.10955E+01_JPRB, 0.11765E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.13545E+01_JPRB, 0.14255E+01_JPRB, 0.14750E+01_JPRB, 0.15009E+01_JPRB, 0.15349E+01_JPRB, 0.15569E+01_JPRB ,&
&  0.15984E+01_JPRB, 0.16050E+01_JPRB, 0.16036E+01_JPRB, 0.16015E+01_JPRB, 0.16220E+01_JPRB, 0.16746E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.16829E+01_JPRB, 0.15760E+01_JPRB, 0.15415E+01_JPRB, 0.15095E+01_JPRB, 0.14052E+01_JPRB, 0.12876E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12169E+01_JPRB, 0.10053E+01_JPRB, 0.11848E+01_JPRB, 0.12658E+01_JPRB, 0.11009E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.90532E+00_JPRB, 0.85974E+00_JPRB, 0.77899E+00_JPRB, 0.75712E+00_JPRB, 0.75854E+00_JPRB, 0.82091E+00_JPRB, 0.62107E+00_JPRB ,&
&  0.62465E+00_JPRB, 0.51876E+00_JPRB, 0.49918E+00_JPRB, 0.49860E+00_JPRB, 0.50090E+00_JPRB, 0.47580E+00_JPRB, 0.46024E+00_JPRB ,&
&  0.45308E+00_JPRB, 0.56915E+00_JPRB, 0.74910E+00_JPRB, 0.11613E+01_JPRB, 0.15714E+01_JPRB, 0.43240E+00_JPRB, 0.40890E+00_JPRB ,&
&  0.41616E+00_JPRB, 0.41694E+00_JPRB, 0.41458E+00_JPRB, 0.42673E+00_JPRB, 0.43866E+00_JPRB, 0.44951E+00_JPRB, 0.46658E+00_JPRB ,&
&  0.47796E+00_JPRB, 0.48475E+00_JPRB, 0.49145E+00_JPRB, 0.48905E+00_JPRB, 0.49831E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.47499E+00_JPRB, 0.48412E+00_JPRB, 0.51079E+00_JPRB, 0.56133E+00_JPRB, 0.63102E+00_JPRB, 0.66543E+00_JPRB, 0.66066E+00_JPRB ,&
&  0.63157E+00_JPRB, 0.63531E+00_JPRB, 0.66424E+00_JPRB, 0.68246E+00_JPRB, 0.73871E+00_JPRB, 0.83660E+00_JPRB, 0.93306E+00_JPRB ,&
&  0.10623E+01_JPRB, 0.11791E+01_JPRB, 0.12850E+01_JPRB, 0.14488E+01_JPRB, 0.16059E+01_JPRB, 0.17927E+01_JPRB, 0.19068E+01_JPRB ,&
&  0.18884E+01_JPRB, 0.16531E+01_JPRB, 0.15260E+01_JPRB, 0.11637E+01_JPRB, 0.97815E+00_JPRB, 0.11286E+01_JPRB, 0.11228E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.11416E+01_JPRB, 0.11328E+01_JPRB, 0.11534E+01_JPRB, 0.11986E+01_JPRB, 0.12486E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.14101E+01_JPRB, 0.14945E+01_JPRB, 0.15614E+01_JPRB, 0.16109E+01_JPRB, 0.16168E+01_JPRB, 0.16364E+01_JPRB, 0.16874E+01_JPRB ,&
&  0.17580E+01_JPRB, 0.17675E+01_JPRB, 0.17745E+01_JPRB, 0.17960E+01_JPRB, 0.17518E+01_JPRB, 0.17603E+01_JPRB, 0.17447E+01_JPRB ,&
&  0.17352E+01_JPRB, 0.16950E+01_JPRB, 0.16717E+01_JPRB, 0.16755E+01_JPRB, 0.16234E+01_JPRB, 0.14193E+01_JPRB, 0.13692E+01_JPRB ,&
&  0.13781E+01_JPRB, 0.13192E+01_JPRB, 0.12165E+01_JPRB, 0.13578E+01_JPRB, 0.93295E+00_JPRB, 0.10458E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.10427E+01_JPRB, 0.97239E+00_JPRB, 0.86387E+00_JPRB, 0.81718E+00_JPRB, 0.78605E+00_JPRB, 0.67582E+00_JPRB, 0.67120E+00_JPRB ,&
&  0.70311E+00_JPRB, 0.62998E+00_JPRB, 0.60053E+00_JPRB, 0.59681E+00_JPRB, 0.61192E+00_JPRB, 0.56733E+00_JPRB, 0.55466E+00_JPRB ,&
&  0.53380E+00_JPRB, 0.61707E+00_JPRB, 0.62430E+00_JPRB, 0.75457E+00_JPRB, 0.76735E+00_JPRB, 0.41756E+00_JPRB, 0.42011E+00_JPRB ,&
&  0.43275E+00_JPRB, 0.44328E+00_JPRB, 0.44590E+00_JPRB, 0.46999E+00_JPRB, 0.48385E+00_JPRB, 0.48146E+00_JPRB, 0.48818E+00_JPRB ,&
&  0.49676E+00_JPRB, 0.49635E+00_JPRB, 0.49382E+00_JPRB, 0.49016E+00_JPRB, 0.48587E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.53643E+00_JPRB, 0.52974E+00_JPRB, 0.55145E+00_JPRB, 0.64974E+00_JPRB, 0.75200E+00_JPRB, 0.81299E+00_JPRB, 0.79200E+00_JPRB ,&
&  0.74597E+00_JPRB, 0.70513E+00_JPRB, 0.75270E+00_JPRB, 0.81439E+00_JPRB, 0.90711E+00_JPRB, 0.10201E+01_JPRB, 0.11405E+01_JPRB ,&
&  0.12800E+01_JPRB, 0.13942E+01_JPRB, 0.14789E+01_JPRB, 0.15734E+01_JPRB, 0.16962E+01_JPRB, 0.18246E+01_JPRB, 0.18907E+01_JPRB ,&
&  0.15361E+01_JPRB, 0.11401E+01_JPRB, 0.13597E+01_JPRB, 0.12518E+01_JPRB, 0.12096E+01_JPRB, 0.12014E+01_JPRB, 0.12120E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.12063E+01_JPRB, 0.12502E+01_JPRB, 0.12897E+01_JPRB, 0.12807E+01_JPRB, 0.13368E+01_JPRB, 0.14176E+01_JPRB ,&
&  0.15172E+01_JPRB, 0.15898E+01_JPRB, 0.16289E+01_JPRB, 0.16628E+01_JPRB, 0.16769E+01_JPRB, 0.17016E+01_JPRB, 0.17486E+01_JPRB ,&
&  0.17956E+01_JPRB, 0.18295E+01_JPRB, 0.18232E+01_JPRB, 0.17733E+01_JPRB, 0.17500E+01_JPRB, 0.17521E+01_JPRB, 0.17123E+01_JPRB ,&
&  0.16970E+01_JPRB, 0.16715E+01_JPRB, 0.17417E+01_JPRB, 0.18227E+01_JPRB, 0.18409E+01_JPRB, 0.16147E+01_JPRB, 0.16242E+01_JPRB ,&
&  0.15679E+01_JPRB, 0.14860E+01_JPRB, 0.14320E+01_JPRB, 0.15324E+01_JPRB, 0.11150E+01_JPRB, 0.11526E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.11554E+01_JPRB, 0.10899E+01_JPRB, 0.10033E+01_JPRB, 0.91775E+00_JPRB, 0.90887E+00_JPRB, 0.76385E+00_JPRB, 0.72374E+00_JPRB ,&
&  0.71221E+00_JPRB, 0.74878E+00_JPRB, 0.68809E+00_JPRB, 0.67058E+00_JPRB, 0.62852E+00_JPRB, 0.65690E+00_JPRB, 0.69000E+00_JPRB ,&
&  0.78187E+00_JPRB, 0.72670E+00_JPRB, 0.67562E+00_JPRB, 0.62054E+00_JPRB, 0.50596E+00_JPRB, 0.43848E+00_JPRB, 0.44738E+00_JPRB ,&
&  0.46644E+00_JPRB, 0.48564E+00_JPRB, 0.49354E+00_JPRB, 0.51483E+00_JPRB, 0.53169E+00_JPRB, 0.52819E+00_JPRB, 0.52200E+00_JPRB ,&
&  0.52519E+00_JPRB, 0.54304E+00_JPRB, 0.53735E+00_JPRB, 0.53832E+00_JPRB, 0.54209E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.61995E+00_JPRB, 0.61770E+00_JPRB, 0.63574E+00_JPRB, 0.72452E+00_JPRB, 0.85282E+00_JPRB, 0.96572E+00_JPRB, 0.99298E+00_JPRB ,&
&  0.99619E+00_JPRB, 0.82743E+00_JPRB, 0.86821E+00_JPRB, 0.90059E+00_JPRB, 0.10062E+01_JPRB, 0.11269E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.13780E+01_JPRB, 0.15076E+01_JPRB, 0.16054E+01_JPRB, 0.16961E+01_JPRB, 0.17987E+01_JPRB, 0.19025E+01_JPRB, 0.16571E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.13664E+01_JPRB, 0.19428E+01_JPRB, 0.15446E+01_JPRB, 0.13099E+01_JPRB, 0.13014E+01_JPRB, 0.13231E+01_JPRB ,&
&  0.13492E+01_JPRB, 0.12979E+01_JPRB, 0.13307E+01_JPRB, 0.13814E+01_JPRB, 0.13766E+01_JPRB, 0.14416E+01_JPRB, 0.15359E+01_JPRB ,&
&  0.15927E+01_JPRB, 0.16506E+01_JPRB, 0.16705E+01_JPRB, 0.16738E+01_JPRB, 0.16841E+01_JPRB, 0.17103E+01_JPRB, 0.17531E+01_JPRB ,&
&  0.17953E+01_JPRB, 0.18276E+01_JPRB, 0.18186E+01_JPRB, 0.17319E+01_JPRB, 0.17149E+01_JPRB, 0.17414E+01_JPRB, 0.17440E+01_JPRB ,&
&  0.17244E+01_JPRB, 0.17899E+01_JPRB, 0.18212E+01_JPRB, 0.17719E+01_JPRB, 0.16488E+01_JPRB, 0.18227E+01_JPRB, 0.17066E+01_JPRB ,&
&  0.17073E+01_JPRB, 0.16129E+01_JPRB, 0.17131E+01_JPRB, 0.13265E+01_JPRB, 0.13402E+01_JPRB, 0.13409E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.11518E+01_JPRB, 0.11483E+01_JPRB, 0.10784E+01_JPRB, 0.10210E+01_JPRB, 0.99240E+00_JPRB, 0.85431E+00_JPRB, 0.81734E+00_JPRB ,&
&  0.77454E+00_JPRB, 0.83165E+00_JPRB, 0.70975E+00_JPRB, 0.64598E+00_JPRB, 0.58508E+00_JPRB, 0.62548E+00_JPRB, 0.71711E+00_JPRB ,&
&  0.85373E+00_JPRB, 0.88781E+00_JPRB, 0.84968E+00_JPRB, 0.76641E+00_JPRB, 0.63839E+00_JPRB, 0.54460E+00_JPRB, 0.49632E+00_JPRB ,&
&  0.49520E+00_JPRB, 0.50586E+00_JPRB, 0.52642E+00_JPRB, 0.54427E+00_JPRB, 0.56307E+00_JPRB, 0.57972E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.59432E+00_JPRB, 0.59495E+00_JPRB, 0.59127E+00_JPRB, 0.61662E+00_JPRB, 0.63174E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.67561E+00_JPRB, 0.65822E+00_JPRB, 0.64297E+00_JPRB, 0.67199E+00_JPRB, 0.73816E+00_JPRB, 0.77004E+00_JPRB, 0.77962E+00_JPRB ,&
&  0.77506E+00_JPRB, 0.76808E+00_JPRB, 0.80625E+00_JPRB, 0.83054E+00_JPRB, 0.93407E+00_JPRB, 0.10581E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.13753E+01_JPRB, 0.14879E+01_JPRB, 0.16255E+01_JPRB, 0.17153E+01_JPRB, 0.18011E+01_JPRB, 0.14085E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.21593E+01_JPRB, 0.18960E+01_JPRB, 0.17592E+01_JPRB, 0.14871E+01_JPRB, 0.14568E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13658E+01_JPRB, 0.13042E+01_JPRB, 0.13510E+01_JPRB, 0.13983E+01_JPRB, 0.14149E+01_JPRB, 0.14577E+01_JPRB, 0.15158E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.16212E+01_JPRB, 0.16427E+01_JPRB, 0.16540E+01_JPRB, 0.16702E+01_JPRB, 0.16755E+01_JPRB, 0.17203E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.18312E+01_JPRB, 0.18318E+01_JPRB, 0.18341E+01_JPRB, 0.17799E+01_JPRB, 0.18398E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.18833E+01_JPRB, 0.18240E+01_JPRB, 0.19791E+01_JPRB, 0.19540E+01_JPRB, 0.18695E+01_JPRB, 0.19455E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.16964E+01_JPRB, 0.18751E+01_JPRB, 0.14664E+01_JPRB, 0.14560E+01_JPRB, 0.15261E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.12290E+01_JPRB, 0.11508E+01_JPRB, 0.11219E+01_JPRB, 0.10837E+01_JPRB, 0.98524E+00_JPRB, 0.87622E+00_JPRB ,&
&  0.88147E+00_JPRB, 0.88304E+00_JPRB, 0.80566E+00_JPRB, 0.62633E+00_JPRB, 0.57339E+00_JPRB, 0.57091E+00_JPRB, 0.73280E+00_JPRB ,&
&  0.87858E+00_JPRB, 0.91606E+00_JPRB, 0.95952E+00_JPRB, 0.98262E+00_JPRB, 0.88955E+00_JPRB, 0.79327E+00_JPRB, 0.67520E+00_JPRB ,&
&  0.56376E+00_JPRB, 0.52294E+00_JPRB, 0.55038E+00_JPRB, 0.54653E+00_JPRB, 0.56540E+00_JPRB, 0.58851E+00_JPRB, 0.60792E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.64465E+00_JPRB, 0.66065E+00_JPRB, 0.66406E+00_JPRB, 0.67528E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.67223E+00_JPRB, 0.65977E+00_JPRB, 0.64761E+00_JPRB, 0.65460E+00_JPRB, 0.72215E+00_JPRB, 0.74898E+00_JPRB, 0.71933E+00_JPRB ,&
&  0.69386E+00_JPRB, 0.69255E+00_JPRB, 0.72029E+00_JPRB, 0.70955E+00_JPRB, 0.75717E+00_JPRB, 0.87820E+00_JPRB, 0.97528E+00_JPRB ,&
&  0.10718E+01_JPRB, 0.11864E+01_JPRB, 0.13382E+01_JPRB, 0.15239E+01_JPRB, 0.16910E+01_JPRB, 0.18951E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.12654E+01_JPRB, 0.18910E+01_JPRB, 0.19792E+01_JPRB, 0.19949E+01_JPRB, 0.17763E+01_JPRB, 0.16461E+01_JPRB, 0.14718E+01_JPRB ,&
&  0.14443E+01_JPRB, 0.13538E+01_JPRB, 0.12929E+01_JPRB, 0.12764E+01_JPRB, 0.12680E+01_JPRB, 0.13337E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.13332E+01_JPRB, 0.13547E+01_JPRB, 0.14375E+01_JPRB, 0.15033E+01_JPRB, 0.15527E+01_JPRB, 0.15968E+01_JPRB, 0.16549E+01_JPRB ,&
&  0.17425E+01_JPRB, 0.18283E+01_JPRB, 0.19502E+01_JPRB, 0.19104E+01_JPRB, 0.19425E+01_JPRB, 0.20075E+01_JPRB, 0.21528E+01_JPRB ,&
&  0.21026E+01_JPRB, 0.20410E+01_JPRB, 0.22922E+01_JPRB, 0.25258E+01_JPRB, 0.24865E+01_JPRB, 0.21964E+01_JPRB, 0.19151E+01_JPRB ,&
&  0.18823E+01_JPRB, 0.18042E+01_JPRB, 0.18173E+01_JPRB, 0.16418E+01_JPRB, 0.16634E+01_JPRB, 0.17895E+01_JPRB, 0.17943E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.14127E+01_JPRB, 0.12936E+01_JPRB, 0.12370E+01_JPRB, 0.12001E+01_JPRB, 0.10743E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.12229E+01_JPRB, 0.10421E+01_JPRB, 0.75797E+00_JPRB, 0.61219E+00_JPRB, 0.49926E+00_JPRB, 0.57885E+00_JPRB ,&
&  0.85824E+00_JPRB, 0.90323E+00_JPRB, 0.91758E+00_JPRB, 0.11176E+01_JPRB, 0.10992E+01_JPRB, 0.10536E+01_JPRB, 0.93858E+00_JPRB ,&
&  0.76464E+00_JPRB, 0.67199E+00_JPRB, 0.62421E+00_JPRB, 0.58298E+00_JPRB, 0.58230E+00_JPRB, 0.59138E+00_JPRB, 0.61125E+00_JPRB ,&
&  0.63158E+00_JPRB, 0.64372E+00_JPRB, 0.65107E+00_JPRB, 0.65574E+00_JPRB, 0.67140E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.67676E+00_JPRB, 0.66562E+00_JPRB, 0.66939E+00_JPRB, 0.66632E+00_JPRB, 0.70984E+00_JPRB, 0.73849E+00_JPRB, 0.67586E+00_JPRB ,&
&  0.65983E+00_JPRB, 0.65192E+00_JPRB, 0.66507E+00_JPRB, 0.66089E+00_JPRB, 0.67006E+00_JPRB, 0.71981E+00_JPRB, 0.78311E+00_JPRB ,&
&  0.87066E+00_JPRB, 0.97313E+00_JPRB, 0.11357E+01_JPRB, 0.13313E+01_JPRB, 0.14322E+01_JPRB, 0.13768E+01_JPRB, 0.14518E+01_JPRB ,&
&  0.17114E+01_JPRB, 0.20318E+01_JPRB, 0.21364E+01_JPRB, 0.23677E+01_JPRB, 0.23698E+01_JPRB, 0.24703E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.18016E+01_JPRB, 0.16353E+01_JPRB, 0.14411E+01_JPRB, 0.13126E+01_JPRB, 0.11963E+01_JPRB, 0.11816E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.12124E+01_JPRB, 0.12537E+01_JPRB, 0.12907E+01_JPRB, 0.13653E+01_JPRB, 0.14328E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.16584E+01_JPRB, 0.17954E+01_JPRB, 0.19410E+01_JPRB, 0.20145E+01_JPRB, 0.20490E+01_JPRB, 0.21386E+01_JPRB, 0.23552E+01_JPRB ,&
&  0.24587E+01_JPRB, 0.25553E+01_JPRB, 0.26339E+01_JPRB, 0.28276E+01_JPRB, 0.28979E+01_JPRB, 0.26798E+01_JPRB, 0.22761E+01_JPRB ,&
&  0.20354E+01_JPRB, 0.18359E+01_JPRB, 0.16548E+01_JPRB, 0.17171E+01_JPRB, 0.18311E+01_JPRB, 0.21492E+01_JPRB, 0.20578E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.13041E+01_JPRB, 0.12058E+01_JPRB, 0.12746E+01_JPRB, 0.13921E+01_JPRB, 0.12950E+01_JPRB, 0.12689E+01_JPRB ,&
&  0.12024E+01_JPRB, 0.78039E+00_JPRB, 0.63349E+00_JPRB, 0.74811E+00_JPRB, 0.56985E+00_JPRB, 0.44627E+00_JPRB, 0.61126E+00_JPRB ,&
&  0.81653E+00_JPRB, 0.10027E+01_JPRB, 0.10153E+01_JPRB, 0.10042E+01_JPRB, 0.11954E+01_JPRB, 0.12612E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.90264E+00_JPRB, 0.79861E+00_JPRB, 0.72847E+00_JPRB, 0.67206E+00_JPRB, 0.62916E+00_JPRB, 0.61566E+00_JPRB, 0.61736E+00_JPRB ,&
&  0.61028E+00_JPRB, 0.61754E+00_JPRB, 0.62478E+00_JPRB, 0.63763E+00_JPRB, 0.66123E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.65374E+00_JPRB, 0.65814E+00_JPRB, 0.67176E+00_JPRB, 0.68276E+00_JPRB, 0.67233E+00_JPRB, 0.69179E+00_JPRB, 0.68422E+00_JPRB ,&
&  0.65492E+00_JPRB, 0.62378E+00_JPRB, 0.61706E+00_JPRB, 0.62681E+00_JPRB, 0.64153E+00_JPRB, 0.65446E+00_JPRB, 0.68179E+00_JPRB ,&
&  0.73085E+00_JPRB, 0.81330E+00_JPRB, 0.98099E+00_JPRB, 0.11457E+01_JPRB, 0.12268E+01_JPRB, 0.11770E+01_JPRB, 0.15446E+01_JPRB ,&
&  0.18088E+01_JPRB, 0.21473E+01_JPRB, 0.26116E+01_JPRB, 0.27929E+01_JPRB, 0.27233E+01_JPRB, 0.26864E+01_JPRB, 0.23424E+01_JPRB ,&
&  0.20623E+01_JPRB, 0.18607E+01_JPRB, 0.16699E+01_JPRB, 0.15103E+01_JPRB, 0.13992E+01_JPRB, 0.12809E+01_JPRB, 0.12354E+01_JPRB ,&
&  0.11889E+01_JPRB, 0.11643E+01_JPRB, 0.11898E+01_JPRB, 0.11987E+01_JPRB, 0.12506E+01_JPRB, 0.13057E+01_JPRB, 0.14026E+01_JPRB ,&
&  0.15583E+01_JPRB, 0.17390E+01_JPRB, 0.19221E+01_JPRB, 0.21817E+01_JPRB, 0.18393E+01_JPRB, 0.21990E+01_JPRB, 0.23276E+01_JPRB ,&
&  0.25047E+01_JPRB, 0.25825E+01_JPRB, 0.25186E+01_JPRB, 0.31714E+01_JPRB, 0.34355E+01_JPRB, 0.30154E+01_JPRB, 0.25646E+01_JPRB ,&
&  0.22761E+01_JPRB, 0.19478E+01_JPRB, 0.17607E+01_JPRB, 0.18789E+01_JPRB, 0.19716E+01_JPRB, 0.21985E+01_JPRB, 0.16392E+01_JPRB ,&
&  0.14984E+01_JPRB, 0.17010E+01_JPRB, 0.14964E+01_JPRB, 0.11935E+01_JPRB, 0.14031E+01_JPRB, 0.15742E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.54134E+00_JPRB, 0.46850E+00_JPRB, 0.49455E+00_JPRB, 0.49623E+00_JPRB, 0.46694E+00_JPRB, 0.48193E+00_JPRB, 0.93958E+00_JPRB ,&
&  0.79983E+00_JPRB, 0.10513E+01_JPRB, 0.98326E+00_JPRB, 0.98514E+00_JPRB, 0.11107E+01_JPRB, 0.13286E+01_JPRB, 0.15948E+01_JPRB ,&
&  0.10234E+01_JPRB, 0.89440E+00_JPRB, 0.78364E+00_JPRB, 0.71488E+00_JPRB, 0.65905E+00_JPRB, 0.63009E+00_JPRB, 0.62142E+00_JPRB ,&
&  0.61136E+00_JPRB, 0.60481E+00_JPRB, 0.60455E+00_JPRB, 0.61700E+00_JPRB, 0.63538E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.66426E+00_JPRB, 0.66483E+00_JPRB, 0.66388E+00_JPRB, 0.66408E+00_JPRB, 0.64636E+00_JPRB, 0.65442E+00_JPRB, 0.68342E+00_JPRB ,&
&  0.66978E+00_JPRB, 0.62990E+00_JPRB, 0.61035E+00_JPRB, 0.61387E+00_JPRB, 0.63114E+00_JPRB, 0.64555E+00_JPRB, 0.64818E+00_JPRB ,&
&  0.68010E+00_JPRB, 0.74595E+00_JPRB, 0.96548E+00_JPRB, 0.98329E+00_JPRB, 0.10586E+01_JPRB, 0.10269E+01_JPRB, 0.13410E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.22526E+01_JPRB, 0.26801E+01_JPRB, 0.28710E+01_JPRB, 0.32339E+01_JPRB, 0.32916E+01_JPRB, 0.31416E+01_JPRB ,&
&  0.26285E+01_JPRB, 0.22520E+01_JPRB, 0.19634E+01_JPRB, 0.17623E+01_JPRB, 0.16020E+01_JPRB, 0.14218E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.12262E+01_JPRB, 0.11734E+01_JPRB, 0.11296E+01_JPRB, 0.11393E+01_JPRB, 0.11689E+01_JPRB, 0.12003E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.14167E+01_JPRB, 0.16430E+01_JPRB, 0.18742E+01_JPRB, 0.21829E+01_JPRB, 0.19528E+01_JPRB, 0.17638E+01_JPRB, 0.21136E+01_JPRB ,&
&  0.24751E+01_JPRB, 0.27635E+01_JPRB, 0.28465E+01_JPRB, 0.37521E+01_JPRB, 0.37967E+01_JPRB, 0.32416E+01_JPRB, 0.29629E+01_JPRB ,&
&  0.25107E+01_JPRB, 0.24066E+01_JPRB, 0.19413E+01_JPRB, 0.20123E+01_JPRB, 0.22105E+01_JPRB, 0.18734E+01_JPRB, 0.18892E+01_JPRB ,&
&  0.20944E+01_JPRB, 0.19170E+01_JPRB, 0.16194E+01_JPRB, 0.12382E+01_JPRB, 0.14690E+01_JPRB, 0.13376E+01_JPRB, 0.68714E+00_JPRB ,&
&  0.58392E+00_JPRB, 0.56566E+00_JPRB, 0.54829E+00_JPRB, 0.52527E+00_JPRB, 0.51899E+00_JPRB, 0.54690E+00_JPRB, 0.80585E+00_JPRB ,&
&  0.79037E+00_JPRB, 0.10137E+01_JPRB, 0.10055E+01_JPRB, 0.98645E+00_JPRB, 0.10043E+01_JPRB, 0.12675E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.95929E+00_JPRB, 0.80515E+00_JPRB, 0.73253E+00_JPRB, 0.66742E+00_JPRB, 0.62900E+00_JPRB, 0.63758E+00_JPRB ,&
&  0.63202E+00_JPRB, 0.61496E+00_JPRB, 0.61277E+00_JPRB, 0.62576E+00_JPRB, 0.64258E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.67266E+00_JPRB, 0.67509E+00_JPRB, 0.67625E+00_JPRB, 0.69213E+00_JPRB, 0.68528E+00_JPRB, 0.67410E+00_JPRB, 0.69345E+00_JPRB ,&
&  0.67830E+00_JPRB, 0.65214E+00_JPRB, 0.62938E+00_JPRB, 0.62806E+00_JPRB, 0.62446E+00_JPRB, 0.64157E+00_JPRB, 0.64803E+00_JPRB ,&
&  0.67113E+00_JPRB, 0.76673E+00_JPRB, 0.90800E+00_JPRB, 0.87676E+00_JPRB, 0.85184E+00_JPRB, 0.85268E+00_JPRB, 0.10463E+01_JPRB ,&
&  0.16459E+01_JPRB, 0.22621E+01_JPRB, 0.26949E+01_JPRB, 0.31245E+01_JPRB, 0.35208E+01_JPRB, 0.34674E+01_JPRB, 0.35688E+01_JPRB ,&
&  0.31517E+01_JPRB, 0.26557E+01_JPRB, 0.23119E+01_JPRB, 0.20323E+01_JPRB, 0.18865E+01_JPRB, 0.17468E+01_JPRB, 0.16003E+01_JPRB ,&
&  0.14072E+01_JPRB, 0.12590E+01_JPRB, 0.11740E+01_JPRB, 0.11091E+01_JPRB, 0.10900E+01_JPRB, 0.11167E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.15084E+01_JPRB, 0.17410E+01_JPRB, 0.20776E+01_JPRB, 0.22825E+01_JPRB, 0.22755E+01_JPRB, 0.21880E+01_JPRB ,&
&  0.22247E+01_JPRB, 0.23475E+01_JPRB, 0.34166E+01_JPRB, 0.39860E+01_JPRB, 0.37946E+01_JPRB, 0.34300E+01_JPRB, 0.31030E+01_JPRB ,&
&  0.26299E+01_JPRB, 0.25028E+01_JPRB, 0.21004E+01_JPRB, 0.22069E+01_JPRB, 0.20601E+01_JPRB, 0.17951E+01_JPRB, 0.23102E+01_JPRB ,&
&  0.23998E+01_JPRB, 0.21696E+01_JPRB, 0.17245E+01_JPRB, 0.13469E+01_JPRB, 0.11676E+01_JPRB, 0.89606E+00_JPRB, 0.67097E+00_JPRB ,&
&  0.62705E+00_JPRB, 0.62050E+00_JPRB, 0.58017E+00_JPRB, 0.59619E+00_JPRB, 0.58617E+00_JPRB, 0.63937E+00_JPRB, 0.77435E+00_JPRB ,&
&  0.92789E+00_JPRB, 0.98278E+00_JPRB, 0.10193E+01_JPRB, 0.10041E+01_JPRB, 0.95997E+00_JPRB, 0.99162E+00_JPRB, 0.12192E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.96285E+00_JPRB, 0.85339E+00_JPRB, 0.75511E+00_JPRB, 0.70381E+00_JPRB, 0.65977E+00_JPRB, 0.63403E+00_JPRB ,&
&  0.62732E+00_JPRB, 0.64061E+00_JPRB, 0.65015E+00_JPRB, 0.65003E+00_JPRB, 0.64349E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.67863E+00_JPRB, 0.69450E+00_JPRB, 0.70355E+00_JPRB, 0.70983E+00_JPRB, 0.70454E+00_JPRB, 0.69923E+00_JPRB, 0.70431E+00_JPRB ,&
&  0.69010E+00_JPRB, 0.67381E+00_JPRB, 0.65795E+00_JPRB, 0.65241E+00_JPRB, 0.64311E+00_JPRB, 0.64680E+00_JPRB, 0.64119E+00_JPRB ,&
&  0.66132E+00_JPRB, 0.80653E+00_JPRB, 0.70094E+00_JPRB, 0.69753E+00_JPRB, 0.74848E+00_JPRB, 0.79426E+00_JPRB, 0.89571E+00_JPRB ,&
&  0.15008E+01_JPRB, 0.22754E+01_JPRB, 0.28703E+01_JPRB, 0.36899E+01_JPRB, 0.44516E+01_JPRB, 0.41014E+01_JPRB, 0.38937E+01_JPRB ,&
&  0.37203E+01_JPRB, 0.31071E+01_JPRB, 0.27128E+01_JPRB, 0.24085E+01_JPRB, 0.21665E+01_JPRB, 0.19855E+01_JPRB, 0.17753E+01_JPRB ,&
&  0.15475E+01_JPRB, 0.13790E+01_JPRB, 0.12567E+01_JPRB, 0.11640E+01_JPRB, 0.11004E+01_JPRB, 0.11310E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.13480E+01_JPRB, 0.15875E+01_JPRB, 0.18621E+01_JPRB, 0.20267E+01_JPRB, 0.21545E+01_JPRB, 0.29275E+01_JPRB ,&
&  0.29523E+01_JPRB, 0.30881E+01_JPRB, 0.35681E+01_JPRB, 0.49002E+01_JPRB, 0.39963E+01_JPRB, 0.32340E+01_JPRB, 0.31819E+01_JPRB ,&
&  0.22404E+01_JPRB, 0.19147E+01_JPRB, 0.19651E+01_JPRB, 0.20224E+01_JPRB, 0.18048E+01_JPRB, 0.20444E+01_JPRB, 0.30714E+01_JPRB ,&
&  0.24500E+01_JPRB, 0.23420E+01_JPRB, 0.23374E+01_JPRB, 0.24227E+01_JPRB, 0.11808E+01_JPRB, 0.85111E+00_JPRB, 0.12745E+01_JPRB ,&
&  0.13568E+01_JPRB, 0.89765E+00_JPRB, 0.68790E+00_JPRB, 0.87135E+00_JPRB, 0.68734E+00_JPRB, 0.74803E+00_JPRB, 0.92117E+00_JPRB ,&
&  0.91195E+00_JPRB, 0.90133E+00_JPRB, 0.10533E+01_JPRB, 0.10001E+01_JPRB, 0.93466E+00_JPRB, 0.89265E+00_JPRB, 0.10429E+01_JPRB ,&
&  0.10747E+01_JPRB, 0.10380E+01_JPRB, 0.98700E+00_JPRB, 0.85957E+00_JPRB, 0.79383E+00_JPRB, 0.74442E+00_JPRB, 0.70970E+00_JPRB ,&
&  0.66832E+00_JPRB, 0.65282E+00_JPRB, 0.66547E+00_JPRB, 0.67173E+00_JPRB, 0.65670E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.69198E+00_JPRB, 0.68842E+00_JPRB, 0.70111E+00_JPRB, 0.70398E+00_JPRB, 0.71277E+00_JPRB, 0.71460E+00_JPRB, 0.71801E+00_JPRB ,&
&  0.70069E+00_JPRB, 0.68117E+00_JPRB, 0.68181E+00_JPRB, 0.68048E+00_JPRB, 0.66843E+00_JPRB, 0.65677E+00_JPRB, 0.64485E+00_JPRB ,&
&  0.66435E+00_JPRB, 0.69772E+00_JPRB, 0.62364E+00_JPRB, 0.62444E+00_JPRB, 0.73102E+00_JPRB, 0.76427E+00_JPRB, 0.89017E+00_JPRB ,&
&  0.14366E+01_JPRB, 0.22060E+01_JPRB, 0.29656E+01_JPRB, 0.39644E+01_JPRB, 0.45992E+01_JPRB, 0.45143E+01_JPRB, 0.41553E+01_JPRB ,&
&  0.41075E+01_JPRB, 0.33478E+01_JPRB, 0.29257E+01_JPRB, 0.26640E+01_JPRB, 0.23618E+01_JPRB, 0.20823E+01_JPRB, 0.18471E+01_JPRB ,&
&  0.16259E+01_JPRB, 0.14323E+01_JPRB, 0.13132E+01_JPRB, 0.12169E+01_JPRB, 0.11845E+01_JPRB, 0.11834E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11783E+01_JPRB, 0.14507E+01_JPRB, 0.17267E+01_JPRB, 0.18289E+01_JPRB, 0.19709E+01_JPRB, 0.25268E+01_JPRB ,&
&  0.29719E+01_JPRB, 0.30394E+01_JPRB, 0.36425E+01_JPRB, 0.43838E+01_JPRB, 0.43643E+01_JPRB, 0.35738E+01_JPRB, 0.36933E+01_JPRB ,&
&  0.27010E+01_JPRB, 0.22218E+01_JPRB, 0.19407E+01_JPRB, 0.16846E+01_JPRB, 0.18590E+01_JPRB, 0.29046E+01_JPRB, 0.26710E+01_JPRB ,&
&  0.24708E+01_JPRB, 0.22742E+01_JPRB, 0.22320E+01_JPRB, 0.19753E+01_JPRB, 0.16952E+01_JPRB, 0.12096E+01_JPRB, 0.15077E+01_JPRB ,&
&  0.16001E+01_JPRB, 0.15752E+01_JPRB, 0.13910E+01_JPRB, 0.11715E+01_JPRB, 0.93888E+00_JPRB, 0.10280E+01_JPRB, 0.98721E+00_JPRB ,&
&  0.92865E+00_JPRB, 0.86147E+00_JPRB, 0.91566E+00_JPRB, 0.94935E+00_JPRB, 0.93972E+00_JPRB, 0.84927E+00_JPRB, 0.97146E+00_JPRB ,&
&  0.10013E+01_JPRB, 0.10862E+01_JPRB, 0.10714E+01_JPRB, 0.95377E+00_JPRB, 0.86927E+00_JPRB, 0.83159E+00_JPRB, 0.79835E+00_JPRB ,&
&  0.76159E+00_JPRB, 0.70986E+00_JPRB, 0.69284E+00_JPRB, 0.67910E+00_JPRB, 0.67174E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.70768E+00_JPRB, 0.70990E+00_JPRB, 0.70623E+00_JPRB, 0.69958E+00_JPRB, 0.69167E+00_JPRB, 0.68638E+00_JPRB, 0.69080E+00_JPRB ,&
&  0.68229E+00_JPRB, 0.65961E+00_JPRB, 0.65613E+00_JPRB, 0.65534E+00_JPRB, 0.64721E+00_JPRB, 0.63386E+00_JPRB, 0.62953E+00_JPRB ,&
&  0.65161E+00_JPRB, 0.66127E+00_JPRB, 0.58165E+00_JPRB, 0.63838E+00_JPRB, 0.69742E+00_JPRB, 0.72892E+00_JPRB, 0.90321E+00_JPRB ,&
&  0.11836E+01_JPRB, 0.17545E+01_JPRB, 0.24086E+01_JPRB, 0.33439E+01_JPRB, 0.41744E+01_JPRB, 0.45406E+01_JPRB, 0.45402E+01_JPRB ,&
&  0.40149E+01_JPRB, 0.36411E+01_JPRB, 0.31375E+01_JPRB, 0.27288E+01_JPRB, 0.23866E+01_JPRB, 0.21369E+01_JPRB, 0.18889E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.15173E+01_JPRB, 0.14056E+01_JPRB, 0.13020E+01_JPRB, 0.12574E+01_JPRB, 0.12455E+01_JPRB, 0.12216E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.11086E+01_JPRB, 0.12954E+01_JPRB, 0.16107E+01_JPRB, 0.16885E+01_JPRB, 0.18738E+01_JPRB, 0.21149E+01_JPRB ,&
&  0.30731E+01_JPRB, 0.34621E+01_JPRB, 0.35309E+01_JPRB, 0.47389E+01_JPRB, 0.32504E+01_JPRB, 0.37458E+01_JPRB, 0.42264E+01_JPRB ,&
&  0.39763E+01_JPRB, 0.35243E+01_JPRB, 0.33692E+01_JPRB, 0.26278E+01_JPRB, 0.21205E+01_JPRB, 0.28357E+01_JPRB, 0.24290E+01_JPRB ,&
&  0.21382E+01_JPRB, 0.21422E+01_JPRB, 0.20913E+01_JPRB, 0.22167E+01_JPRB, 0.16952E+01_JPRB, 0.14457E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.11027E+01_JPRB, 0.12626E+01_JPRB, 0.11172E+01_JPRB, 0.10744E+01_JPRB, 0.10076E+01_JPRB, 0.94156E+00_JPRB ,&
&  0.93016E+00_JPRB, 0.88124E+00_JPRB, 0.83820E+00_JPRB, 0.90029E+00_JPRB, 0.95610E+00_JPRB, 0.85548E+00_JPRB, 0.92617E+00_JPRB ,&
&  0.97409E+00_JPRB, 0.10303E+01_JPRB, 0.10306E+01_JPRB, 0.10075E+01_JPRB, 0.91567E+00_JPRB, 0.85834E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.82191E+00_JPRB, 0.78414E+00_JPRB, 0.73753E+00_JPRB, 0.72257E+00_JPRB, 0.69916E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.72303E+00_JPRB, 0.71222E+00_JPRB, 0.68893E+00_JPRB, 0.68477E+00_JPRB, 0.67935E+00_JPRB, 0.67062E+00_JPRB, 0.65660E+00_JPRB ,&
&  0.63739E+00_JPRB, 0.62727E+00_JPRB, 0.61931E+00_JPRB, 0.61419E+00_JPRB, 0.61160E+00_JPRB, 0.60539E+00_JPRB, 0.60426E+00_JPRB ,&
&  0.61406E+00_JPRB, 0.74502E+00_JPRB, 0.65655E+00_JPRB, 0.59604E+00_JPRB, 0.56963E+00_JPRB, 0.67146E+00_JPRB, 0.82808E+00_JPRB ,&
&  0.10577E+01_JPRB, 0.13486E+01_JPRB, 0.18275E+01_JPRB, 0.22602E+01_JPRB, 0.28370E+01_JPRB, 0.33730E+01_JPRB, 0.37243E+01_JPRB ,&
&  0.38330E+01_JPRB, 0.29048E+01_JPRB, 0.26429E+01_JPRB, 0.22739E+01_JPRB, 0.20421E+01_JPRB, 0.18154E+01_JPRB, 0.16599E+01_JPRB ,&
&  0.15179E+01_JPRB, 0.14231E+01_JPRB, 0.13466E+01_JPRB, 0.12684E+01_JPRB, 0.12212E+01_JPRB, 0.11923E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.10983E+01_JPRB, 0.11753E+01_JPRB, 0.14167E+01_JPRB, 0.18486E+01_JPRB, 0.22066E+01_JPRB, 0.24650E+01_JPRB ,&
&  0.22898E+01_JPRB, 0.22394E+01_JPRB, 0.36977E+01_JPRB, 0.42649E+01_JPRB, 0.41841E+01_JPRB, 0.48109E+01_JPRB, 0.49232E+01_JPRB ,&
&  0.45224E+01_JPRB, 0.36299E+01_JPRB, 0.30708E+01_JPRB, 0.26121E+01_JPRB, 0.28852E+01_JPRB, 0.26282E+01_JPRB, 0.23671E+01_JPRB ,&
&  0.21340E+01_JPRB, 0.22173E+01_JPRB, 0.21694E+01_JPRB, 0.21463E+01_JPRB, 0.20897E+01_JPRB, 0.20122E+01_JPRB, 0.16915E+01_JPRB ,&
&  0.14719E+01_JPRB, 0.17015E+01_JPRB, 0.13826E+01_JPRB, 0.10754E+01_JPRB, 0.89272E+00_JPRB, 0.90718E+00_JPRB, 0.95667E+00_JPRB ,&
&  0.97982E+00_JPRB, 0.97246E+00_JPRB, 0.93825E+00_JPRB, 0.94562E+00_JPRB, 0.95865E+00_JPRB, 0.91137E+00_JPRB, 0.92340E+00_JPRB ,&
&  0.91737E+00_JPRB, 0.97583E+00_JPRB, 0.98824E+00_JPRB, 0.98896E+00_JPRB, 0.96249E+00_JPRB, 0.89919E+00_JPRB, 0.86715E+00_JPRB ,&
&  0.86067E+00_JPRB, 0.85310E+00_JPRB, 0.81116E+00_JPRB, 0.76441E+00_JPRB, 0.72957E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.71993E+00_JPRB, 0.70044E+00_JPRB, 0.67739E+00_JPRB, 0.67679E+00_JPRB, 0.67967E+00_JPRB, 0.67574E+00_JPRB, 0.65350E+00_JPRB ,&
&  0.61444E+00_JPRB, 0.59247E+00_JPRB, 0.58542E+00_JPRB, 0.58917E+00_JPRB, 0.59164E+00_JPRB, 0.59452E+00_JPRB, 0.60513E+00_JPRB ,&
&  0.60682E+00_JPRB, 0.86726E+00_JPRB, 0.76479E+00_JPRB, 0.73356E+00_JPRB, 0.65657E+00_JPRB, 0.74091E+00_JPRB, 0.85477E+00_JPRB ,&
&  0.97958E+00_JPRB, 0.10568E+01_JPRB, 0.12274E+01_JPRB, 0.14910E+01_JPRB, 0.17667E+01_JPRB, 0.20079E+01_JPRB, 0.23984E+01_JPRB ,&
&  0.23863E+01_JPRB, 0.22056E+01_JPRB, 0.18336E+01_JPRB, 0.17581E+01_JPRB, 0.16558E+01_JPRB, 0.14364E+01_JPRB, 0.14325E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.12875E+01_JPRB, 0.12359E+01_JPRB, 0.11934E+01_JPRB, 0.11499E+01_JPRB, 0.10926E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.10365E+01_JPRB, 0.10996E+01_JPRB, 0.11873E+01_JPRB, 0.14874E+01_JPRB, 0.18772E+01_JPRB, 0.21930E+01_JPRB ,&
&  0.25985E+01_JPRB, 0.23488E+01_JPRB, 0.21924E+01_JPRB, 0.30785E+01_JPRB, 0.45359E+01_JPRB, 0.48978E+01_JPRB, 0.45582E+01_JPRB ,&
&  0.47871E+01_JPRB, 0.43643E+01_JPRB, 0.37769E+01_JPRB, 0.33451E+01_JPRB, 0.30558E+01_JPRB, 0.27814E+01_JPRB, 0.26421E+01_JPRB ,&
&  0.24160E+01_JPRB, 0.23437E+01_JPRB, 0.24166E+01_JPRB, 0.24342E+01_JPRB, 0.22635E+01_JPRB, 0.20625E+01_JPRB, 0.20112E+01_JPRB ,&
&  0.18876E+01_JPRB, 0.15567E+01_JPRB, 0.10726E+01_JPRB, 0.10366E+01_JPRB, 0.81736E+00_JPRB, 0.81393E+00_JPRB, 0.94482E+00_JPRB ,&
&  0.93428E+00_JPRB, 0.99208E+00_JPRB, 0.10011E+01_JPRB, 0.93339E+00_JPRB, 0.93590E+00_JPRB, 0.89093E+00_JPRB, 0.89498E+00_JPRB ,&
&  0.92061E+00_JPRB, 0.82725E+00_JPRB, 0.86431E+00_JPRB, 0.89805E+00_JPRB, 0.91985E+00_JPRB, 0.89780E+00_JPRB, 0.87168E+00_JPRB ,&
&  0.86310E+00_JPRB, 0.85462E+00_JPRB, 0.82979E+00_JPRB, 0.78316E+00_JPRB, 0.74551E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.72043E+00_JPRB, 0.70082E+00_JPRB, 0.67975E+00_JPRB, 0.66909E+00_JPRB, 0.66371E+00_JPRB, 0.65888E+00_JPRB, 0.63888E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.58595E+00_JPRB, 0.57659E+00_JPRB, 0.57936E+00_JPRB, 0.58041E+00_JPRB, 0.57546E+00_JPRB, 0.59576E+00_JPRB ,&
&  0.68705E+00_JPRB, 0.86649E+00_JPRB, 0.79716E+00_JPRB, 0.69819E+00_JPRB, 0.82527E+00_JPRB, 0.78744E+00_JPRB, 0.82354E+00_JPRB ,&
&  0.81773E+00_JPRB, 0.93320E+00_JPRB, 0.98632E+00_JPRB, 0.10723E+01_JPRB, 0.11662E+01_JPRB, 0.12983E+01_JPRB, 0.15047E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.13988E+01_JPRB, 0.12887E+01_JPRB, 0.12807E+01_JPRB, 0.13418E+01_JPRB, 0.12160E+01_JPRB, 0.11987E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.11032E+01_JPRB, 0.10693E+01_JPRB, 0.10491E+01_JPRB, 0.10280E+01_JPRB, 0.99939E+00_JPRB, 0.97952E+00_JPRB ,&
&  0.97680E+00_JPRB, 0.97930E+00_JPRB, 0.10345E+01_JPRB, 0.10962E+01_JPRB, 0.12679E+01_JPRB, 0.16964E+01_JPRB, 0.24268E+01_JPRB ,&
&  0.27474E+01_JPRB, 0.28300E+01_JPRB, 0.30670E+01_JPRB, 0.34988E+01_JPRB, 0.37554E+01_JPRB, 0.39467E+01_JPRB, 0.41805E+01_JPRB ,&
&  0.42043E+01_JPRB, 0.41433E+01_JPRB, 0.40931E+01_JPRB, 0.38607E+01_JPRB, 0.35126E+01_JPRB, 0.32369E+01_JPRB, 0.30696E+01_JPRB ,&
&  0.28305E+01_JPRB, 0.26899E+01_JPRB, 0.26894E+01_JPRB, 0.25551E+01_JPRB, 0.24477E+01_JPRB, 0.22225E+01_JPRB, 0.22465E+01_JPRB ,&
&  0.21592E+01_JPRB, 0.12833E+01_JPRB, 0.98576E+00_JPRB, 0.10868E+01_JPRB, 0.87613E+00_JPRB, 0.90028E+00_JPRB, 0.10358E+01_JPRB ,&
&  0.99484E+00_JPRB, 0.96515E+00_JPRB, 0.97899E+00_JPRB, 0.93064E+00_JPRB, 0.87873E+00_JPRB, 0.88775E+00_JPRB, 0.84366E+00_JPRB ,&
&  0.78213E+00_JPRB, 0.73176E+00_JPRB, 0.74529E+00_JPRB, 0.77705E+00_JPRB, 0.81813E+00_JPRB, 0.82120E+00_JPRB, 0.81074E+00_JPRB ,&
&  0.81047E+00_JPRB, 0.80931E+00_JPRB, 0.80585E+00_JPRB, 0.78279E+00_JPRB, 0.74545E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.73646E+00_JPRB, 0.71995E+00_JPRB, 0.70329E+00_JPRB, 0.68934E+00_JPRB, 0.66856E+00_JPRB, 0.64619E+00_JPRB, 0.61846E+00_JPRB ,&
&  0.59582E+00_JPRB, 0.57459E+00_JPRB, 0.56352E+00_JPRB, 0.55877E+00_JPRB, 0.55647E+00_JPRB, 0.54997E+00_JPRB, 0.58492E+00_JPRB ,&
&  0.55890E+00_JPRB, 0.67036E+00_JPRB, 0.62484E+00_JPRB, 0.67403E+00_JPRB, 0.84928E+00_JPRB, 0.76961E+00_JPRB, 0.76926E+00_JPRB ,&
&  0.78016E+00_JPRB, 0.82562E+00_JPRB, 0.83495E+00_JPRB, 0.84782E+00_JPRB, 0.85917E+00_JPRB, 0.88905E+00_JPRB, 0.98912E+00_JPRB ,&
&  0.10163E+01_JPRB, 0.94342E+00_JPRB, 0.92530E+00_JPRB, 0.94726E+00_JPRB, 0.99971E+00_JPRB, 0.10596E+01_JPRB, 0.10948E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10184E+01_JPRB, 0.99033E+00_JPRB, 0.96359E+00_JPRB, 0.93431E+00_JPRB, 0.90607E+00_JPRB, 0.88236E+00_JPRB ,&
&  0.87307E+00_JPRB, 0.88358E+00_JPRB, 0.94265E+00_JPRB, 0.10273E+01_JPRB, 0.11086E+01_JPRB, 0.15674E+01_JPRB, 0.26077E+01_JPRB ,&
&  0.30974E+01_JPRB, 0.30820E+01_JPRB, 0.32980E+01_JPRB, 0.37886E+01_JPRB, 0.38294E+01_JPRB, 0.37902E+01_JPRB, 0.37452E+01_JPRB ,&
&  0.37387E+01_JPRB, 0.36841E+01_JPRB, 0.36389E+01_JPRB, 0.37562E+01_JPRB, 0.37356E+01_JPRB, 0.37237E+01_JPRB, 0.35625E+01_JPRB ,&
&  0.32582E+01_JPRB, 0.30854E+01_JPRB, 0.29748E+01_JPRB, 0.28047E+01_JPRB, 0.25970E+01_JPRB, 0.25086E+01_JPRB, 0.23705E+01_JPRB ,&
&  0.21834E+01_JPRB, 0.19086E+01_JPRB, 0.14064E+01_JPRB, 0.12554E+01_JPRB, 0.94697E+00_JPRB, 0.95713E+00_JPRB, 0.11496E+01_JPRB ,&
&  0.10849E+01_JPRB, 0.95853E+00_JPRB, 0.87825E+00_JPRB, 0.83087E+00_JPRB, 0.79770E+00_JPRB, 0.79946E+00_JPRB, 0.72452E+00_JPRB ,&
&  0.65497E+00_JPRB, 0.66179E+00_JPRB, 0.67491E+00_JPRB, 0.69974E+00_JPRB, 0.72999E+00_JPRB, 0.74804E+00_JPRB, 0.69884E+00_JPRB ,&
&  0.73251E+00_JPRB, 0.74912E+00_JPRB, 0.75871E+00_JPRB, 0.76393E+00_JPRB, 0.75442E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.70781E+00_JPRB, 0.69701E+00_JPRB, 0.68734E+00_JPRB, 0.67653E+00_JPRB, 0.65260E+00_JPRB, 0.61538E+00_JPRB, 0.58556E+00_JPRB ,&
&  0.57583E+00_JPRB, 0.56162E+00_JPRB, 0.54861E+00_JPRB, 0.53826E+00_JPRB, 0.52997E+00_JPRB, 0.52192E+00_JPRB, 0.51982E+00_JPRB ,&
&  0.51262E+00_JPRB, 0.57175E+00_JPRB, 0.57775E+00_JPRB, 0.60969E+00_JPRB, 0.64235E+00_JPRB, 0.62546E+00_JPRB, 0.62724E+00_JPRB ,&
&  0.67010E+00_JPRB, 0.69157E+00_JPRB, 0.70394E+00_JPRB, 0.70278E+00_JPRB, 0.69842E+00_JPRB, 0.71825E+00_JPRB, 0.77552E+00_JPRB ,&
&  0.76475E+00_JPRB, 0.75441E+00_JPRB, 0.75420E+00_JPRB, 0.78349E+00_JPRB, 0.86783E+00_JPRB, 0.96312E+00_JPRB, 0.98129E+00_JPRB ,&
&  0.95454E+00_JPRB, 0.92664E+00_JPRB, 0.89573E+00_JPRB, 0.86966E+00_JPRB, 0.84603E+00_JPRB, 0.82195E+00_JPRB, 0.79973E+00_JPRB ,&
&  0.79283E+00_JPRB, 0.80763E+00_JPRB, 0.85723E+00_JPRB, 0.94294E+00_JPRB, 0.10421E+01_JPRB, 0.13525E+01_JPRB, 0.18519E+01_JPRB ,&
&  0.20984E+01_JPRB, 0.22822E+01_JPRB, 0.24997E+01_JPRB, 0.27563E+01_JPRB, 0.28952E+01_JPRB, 0.29292E+01_JPRB, 0.29353E+01_JPRB ,&
&  0.30322E+01_JPRB, 0.30073E+01_JPRB, 0.31472E+01_JPRB, 0.31026E+01_JPRB, 0.30646E+01_JPRB, 0.31443E+01_JPRB, 0.31023E+01_JPRB ,&
&  0.29833E+01_JPRB, 0.27933E+01_JPRB, 0.28206E+01_JPRB, 0.27276E+01_JPRB, 0.25960E+01_JPRB, 0.24661E+01_JPRB, 0.22830E+01_JPRB ,&
&  0.21079E+01_JPRB, 0.19546E+01_JPRB, 0.16804E+01_JPRB, 0.14723E+01_JPRB, 0.12457E+01_JPRB, 0.12367E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.96416E+00_JPRB, 0.78011E+00_JPRB, 0.75406E+00_JPRB, 0.73221E+00_JPRB, 0.72808E+00_JPRB, 0.68208E+00_JPRB, 0.61600E+00_JPRB ,&
&  0.60319E+00_JPRB, 0.63060E+00_JPRB, 0.63865E+00_JPRB, 0.64690E+00_JPRB, 0.66897E+00_JPRB, 0.69493E+00_JPRB, 0.61328E+00_JPRB ,&
&  0.61531E+00_JPRB, 0.67008E+00_JPRB, 0.69397E+00_JPRB, 0.70867E+00_JPRB, 0.71221E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.66468E+00_JPRB, 0.65778E+00_JPRB, 0.64591E+00_JPRB, 0.63430E+00_JPRB, 0.61214E+00_JPRB, 0.58296E+00_JPRB, 0.54785E+00_JPRB ,&
&  0.55759E+00_JPRB, 0.54790E+00_JPRB, 0.53670E+00_JPRB, 0.52237E+00_JPRB, 0.50686E+00_JPRB, 0.48106E+00_JPRB, 0.44315E+00_JPRB ,&
&  0.44521E+00_JPRB, 0.47469E+00_JPRB, 0.51807E+00_JPRB, 0.54262E+00_JPRB, 0.54544E+00_JPRB, 0.54538E+00_JPRB, 0.55438E+00_JPRB ,&
&  0.56802E+00_JPRB, 0.58255E+00_JPRB, 0.60374E+00_JPRB, 0.61635E+00_JPRB, 0.61557E+00_JPRB, 0.61616E+00_JPRB, 0.63861E+00_JPRB ,&
&  0.61603E+00_JPRB, 0.60623E+00_JPRB, 0.62902E+00_JPRB, 0.60975E+00_JPRB, 0.70695E+00_JPRB, 0.76082E+00_JPRB, 0.78306E+00_JPRB ,&
&  0.76939E+00_JPRB, 0.73998E+00_JPRB, 0.71934E+00_JPRB, 0.71642E+00_JPRB, 0.71483E+00_JPRB, 0.71152E+00_JPRB, 0.71039E+00_JPRB ,&
&  0.71634E+00_JPRB, 0.73389E+00_JPRB, 0.77605E+00_JPRB, 0.84520E+00_JPRB, 0.93463E+00_JPRB, 0.11404E+01_JPRB, 0.13499E+01_JPRB ,&
&  0.15473E+01_JPRB, 0.18102E+01_JPRB, 0.20765E+01_JPRB, 0.21027E+01_JPRB, 0.22985E+01_JPRB, 0.24381E+01_JPRB, 0.24252E+01_JPRB ,&
&  0.25235E+01_JPRB, 0.24975E+01_JPRB, 0.25835E+01_JPRB, 0.25972E+01_JPRB, 0.26049E+01_JPRB, 0.26007E+01_JPRB, 0.25810E+01_JPRB ,&
&  0.25490E+01_JPRB, 0.25191E+01_JPRB, 0.25168E+01_JPRB, 0.24337E+01_JPRB, 0.22895E+01_JPRB, 0.21643E+01_JPRB, 0.20323E+01_JPRB ,&
&  0.19246E+01_JPRB, 0.18163E+01_JPRB, 0.16966E+01_JPRB, 0.15855E+01_JPRB, 0.14585E+01_JPRB, 0.13045E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.10082E+01_JPRB, 0.83666E+00_JPRB, 0.74847E+00_JPRB, 0.72387E+00_JPRB, 0.69427E+00_JPRB, 0.61931E+00_JPRB, 0.57537E+00_JPRB ,&
&  0.55920E+00_JPRB, 0.57380E+00_JPRB, 0.60484E+00_JPRB, 0.61001E+00_JPRB, 0.62301E+00_JPRB, 0.63243E+00_JPRB, 0.59464E+00_JPRB ,&
&  0.56812E+00_JPRB, 0.61278E+00_JPRB, 0.63606E+00_JPRB, 0.65103E+00_JPRB, 0.66207E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.62182E+00_JPRB, 0.61798E+00_JPRB, 0.60514E+00_JPRB, 0.59341E+00_JPRB, 0.58387E+00_JPRB, 0.55527E+00_JPRB, 0.53095E+00_JPRB ,&
&  0.55752E+00_JPRB, 0.52237E+00_JPRB, 0.51613E+00_JPRB, 0.47004E+00_JPRB, 0.44680E+00_JPRB, 0.43811E+00_JPRB, 0.43173E+00_JPRB ,&
&  0.43787E+00_JPRB, 0.45655E+00_JPRB, 0.47321E+00_JPRB, 0.48685E+00_JPRB, 0.50227E+00_JPRB, 0.50613E+00_JPRB, 0.50912E+00_JPRB ,&
&  0.51418E+00_JPRB, 0.52325E+00_JPRB, 0.53476E+00_JPRB, 0.53702E+00_JPRB, 0.53856E+00_JPRB, 0.54001E+00_JPRB, 0.54279E+00_JPRB ,&
&  0.52883E+00_JPRB, 0.52686E+00_JPRB, 0.55334E+00_JPRB, 0.56876E+00_JPRB, 0.59890E+00_JPRB, 0.61511E+00_JPRB, 0.61346E+00_JPRB ,&
&  0.59337E+00_JPRB, 0.56222E+00_JPRB, 0.58914E+00_JPRB, 0.60396E+00_JPRB, 0.61669E+00_JPRB, 0.62387E+00_JPRB, 0.62817E+00_JPRB ,&
&  0.63004E+00_JPRB, 0.63641E+00_JPRB, 0.65374E+00_JPRB, 0.68508E+00_JPRB, 0.73756E+00_JPRB, 0.84909E+00_JPRB, 0.95418E+00_JPRB ,&
&  0.11714E+01_JPRB, 0.10967E+01_JPRB, 0.13610E+01_JPRB, 0.15501E+01_JPRB, 0.17824E+01_JPRB, 0.19029E+01_JPRB, 0.19000E+01_JPRB ,&
&  0.19714E+01_JPRB, 0.19784E+01_JPRB, 0.19935E+01_JPRB, 0.19632E+01_JPRB, 0.19304E+01_JPRB, 0.19185E+01_JPRB, 0.19446E+01_JPRB ,&
&  0.19603E+01_JPRB, 0.18663E+01_JPRB, 0.19641E+01_JPRB, 0.18949E+01_JPRB, 0.18077E+01_JPRB, 0.17593E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.15829E+01_JPRB, 0.14758E+01_JPRB, 0.12814E+01_JPRB, 0.12016E+01_JPRB, 0.11311E+01_JPRB, 0.10163E+01_JPRB ,&
&  0.94685E+00_JPRB, 0.87166E+00_JPRB, 0.79450E+00_JPRB, 0.77556E+00_JPRB, 0.69693E+00_JPRB, 0.63808E+00_JPRB, 0.59616E+00_JPRB ,&
&  0.54771E+00_JPRB, 0.53139E+00_JPRB, 0.54695E+00_JPRB, 0.55593E+00_JPRB, 0.55576E+00_JPRB, 0.55177E+00_JPRB, 0.57356E+00_JPRB ,&
&  0.56000E+00_JPRB, 0.57183E+00_JPRB, 0.60550E+00_JPRB, 0.61188E+00_JPRB, 0.61704E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.59304E+00_JPRB, 0.59149E+00_JPRB, 0.57942E+00_JPRB, 0.56852E+00_JPRB, 0.55743E+00_JPRB, 0.53796E+00_JPRB, 0.50337E+00_JPRB ,&
&  0.49446E+00_JPRB, 0.46856E+00_JPRB, 0.44413E+00_JPRB, 0.43691E+00_JPRB, 0.43526E+00_JPRB, 0.43083E+00_JPRB, 0.42234E+00_JPRB ,&
&  0.42541E+00_JPRB, 0.44083E+00_JPRB, 0.46025E+00_JPRB, 0.47249E+00_JPRB, 0.48321E+00_JPRB, 0.48624E+00_JPRB, 0.49009E+00_JPRB ,&
&  0.49426E+00_JPRB, 0.49226E+00_JPRB, 0.49207E+00_JPRB, 0.49046E+00_JPRB, 0.48753E+00_JPRB, 0.48971E+00_JPRB, 0.49052E+00_JPRB ,&
&  0.49686E+00_JPRB, 0.50206E+00_JPRB, 0.51168E+00_JPRB, 0.52941E+00_JPRB, 0.54288E+00_JPRB, 0.55367E+00_JPRB, 0.52574E+00_JPRB ,&
&  0.46086E+00_JPRB, 0.47216E+00_JPRB, 0.51061E+00_JPRB, 0.53039E+00_JPRB, 0.55210E+00_JPRB, 0.56585E+00_JPRB, 0.57279E+00_JPRB ,&
&  0.57253E+00_JPRB, 0.56789E+00_JPRB, 0.57060E+00_JPRB, 0.58658E+00_JPRB, 0.61939E+00_JPRB, 0.67381E+00_JPRB, 0.73194E+00_JPRB ,&
&  0.85488E+00_JPRB, 0.85802E+00_JPRB, 0.89525E+00_JPRB, 0.10250E+01_JPRB, 0.13201E+01_JPRB, 0.14243E+01_JPRB, 0.14569E+01_JPRB ,&
&  0.14908E+01_JPRB, 0.15073E+01_JPRB, 0.14979E+01_JPRB, 0.14876E+01_JPRB, 0.14436E+01_JPRB, 0.14121E+01_JPRB, 0.14182E+01_JPRB ,&
&  0.14048E+01_JPRB, 0.14233E+01_JPRB, 0.14212E+01_JPRB, 0.14147E+01_JPRB, 0.13645E+01_JPRB, 0.13404E+01_JPRB, 0.13146E+01_JPRB ,&
&  0.12866E+01_JPRB, 0.12410E+01_JPRB, 0.11511E+01_JPRB, 0.10564E+01_JPRB, 0.96976E+00_JPRB, 0.91994E+00_JPRB, 0.87970E+00_JPRB ,&
&  0.83539E+00_JPRB, 0.80093E+00_JPRB, 0.76741E+00_JPRB, 0.72792E+00_JPRB, 0.67871E+00_JPRB, 0.63321E+00_JPRB, 0.59162E+00_JPRB ,&
&  0.53015E+00_JPRB, 0.51000E+00_JPRB, 0.50422E+00_JPRB, 0.50020E+00_JPRB, 0.50369E+00_JPRB, 0.51788E+00_JPRB, 0.51193E+00_JPRB ,&
&  0.53021E+00_JPRB, 0.54120E+00_JPRB, 0.54198E+00_JPRB, 0.55335E+00_JPRB, 0.57086E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.55113E+00_JPRB, 0.55631E+00_JPRB, 0.55030E+00_JPRB, 0.54934E+00_JPRB, 0.53511E+00_JPRB, 0.51848E+00_JPRB, 0.49915E+00_JPRB ,&
&  0.49079E+00_JPRB, 0.47610E+00_JPRB, 0.45788E+00_JPRB, 0.44044E+00_JPRB, 0.43184E+00_JPRB, 0.42924E+00_JPRB, 0.43405E+00_JPRB ,&
&  0.44413E+00_JPRB, 0.45671E+00_JPRB, 0.46691E+00_JPRB, 0.47347E+00_JPRB, 0.47352E+00_JPRB, 0.47224E+00_JPRB, 0.47468E+00_JPRB ,&
&  0.47499E+00_JPRB, 0.47141E+00_JPRB, 0.47131E+00_JPRB, 0.46929E+00_JPRB, 0.46935E+00_JPRB, 0.47055E+00_JPRB, 0.46980E+00_JPRB ,&
&  0.46924E+00_JPRB, 0.46842E+00_JPRB, 0.47834E+00_JPRB, 0.49413E+00_JPRB, 0.50528E+00_JPRB, 0.50556E+00_JPRB, 0.46369E+00_JPRB ,&
&  0.42448E+00_JPRB, 0.41693E+00_JPRB, 0.45334E+00_JPRB, 0.47528E+00_JPRB, 0.49821E+00_JPRB, 0.51233E+00_JPRB, 0.51812E+00_JPRB ,&
&  0.51414E+00_JPRB, 0.49939E+00_JPRB, 0.50793E+00_JPRB, 0.53206E+00_JPRB, 0.55105E+00_JPRB, 0.58050E+00_JPRB, 0.61396E+00_JPRB ,&
&  0.66274E+00_JPRB, 0.72458E+00_JPRB, 0.78847E+00_JPRB, 0.78460E+00_JPRB, 0.86477E+00_JPRB, 0.11400E+01_JPRB, 0.11972E+01_JPRB ,&
&  0.11867E+01_JPRB, 0.11861E+01_JPRB, 0.12209E+01_JPRB, 0.12146E+01_JPRB, 0.11517E+01_JPRB, 0.11001E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.10153E+01_JPRB, 0.10116E+01_JPRB, 0.10284E+01_JPRB, 0.10190E+01_JPRB, 0.10023E+01_JPRB, 0.98551E+00_JPRB ,&
&  0.96723E+00_JPRB, 0.91916E+00_JPRB, 0.87114E+00_JPRB, 0.80960E+00_JPRB, 0.77147E+00_JPRB, 0.74495E+00_JPRB, 0.72959E+00_JPRB ,&
&  0.71217E+00_JPRB, 0.70344E+00_JPRB, 0.69658E+00_JPRB, 0.68377E+00_JPRB, 0.63200E+00_JPRB, 0.57201E+00_JPRB, 0.52686E+00_JPRB ,&
&  0.49758E+00_JPRB, 0.47937E+00_JPRB, 0.47442E+00_JPRB, 0.47652E+00_JPRB, 0.48284E+00_JPRB, 0.48882E+00_JPRB, 0.47965E+00_JPRB ,&
&  0.47878E+00_JPRB, 0.48498E+00_JPRB, 0.49819E+00_JPRB, 0.51743E+00_JPRB, 0.53477E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.49840E+00_JPRB, 0.51658E+00_JPRB, 0.53069E+00_JPRB, 0.53455E+00_JPRB, 0.53080E+00_JPRB, 0.49981E+00_JPRB, 0.47065E+00_JPRB ,&
&  0.45441E+00_JPRB, 0.45046E+00_JPRB, 0.45236E+00_JPRB, 0.45478E+00_JPRB, 0.45831E+00_JPRB, 0.46314E+00_JPRB, 0.46697E+00_JPRB ,&
&  0.46867E+00_JPRB, 0.47087E+00_JPRB, 0.47516E+00_JPRB, 0.48177E+00_JPRB, 0.48126E+00_JPRB, 0.47793E+00_JPRB, 0.47751E+00_JPRB ,&
&  0.47612E+00_JPRB, 0.47142E+00_JPRB, 0.47032E+00_JPRB, 0.46551E+00_JPRB, 0.46323E+00_JPRB, 0.46098E+00_JPRB, 0.45539E+00_JPRB ,&
&  0.44761E+00_JPRB, 0.44584E+00_JPRB, 0.45415E+00_JPRB, 0.46530E+00_JPRB, 0.47572E+00_JPRB, 0.46548E+00_JPRB, 0.43329E+00_JPRB ,&
&  0.40496E+00_JPRB, 0.39014E+00_JPRB, 0.39479E+00_JPRB, 0.42125E+00_JPRB, 0.44677E+00_JPRB, 0.44239E+00_JPRB, 0.45403E+00_JPRB ,&
&  0.45823E+00_JPRB, 0.46233E+00_JPRB, 0.46888E+00_JPRB, 0.48335E+00_JPRB, 0.50049E+00_JPRB, 0.51909E+00_JPRB, 0.53741E+00_JPRB ,&
&  0.55998E+00_JPRB, 0.58827E+00_JPRB, 0.62464E+00_JPRB, 0.62716E+00_JPRB, 0.62759E+00_JPRB, 0.68318E+00_JPRB, 0.75743E+00_JPRB ,&
&  0.82918E+00_JPRB, 0.87547E+00_JPRB, 0.90252E+00_JPRB, 0.93178E+00_JPRB, 0.93595E+00_JPRB, 0.92061E+00_JPRB, 0.89629E+00_JPRB ,&
&  0.87428E+00_JPRB, 0.84632E+00_JPRB, 0.82603E+00_JPRB, 0.81792E+00_JPRB, 0.82878E+00_JPRB, 0.83557E+00_JPRB, 0.82138E+00_JPRB ,&
&  0.79982E+00_JPRB, 0.75732E+00_JPRB, 0.72871E+00_JPRB, 0.66883E+00_JPRB, 0.65144E+00_JPRB, 0.66006E+00_JPRB, 0.67428E+00_JPRB ,&
&  0.67018E+00_JPRB, 0.66195E+00_JPRB, 0.65658E+00_JPRB, 0.64901E+00_JPRB, 0.59705E+00_JPRB, 0.55905E+00_JPRB, 0.53129E+00_JPRB ,&
&  0.51666E+00_JPRB, 0.50266E+00_JPRB, 0.49917E+00_JPRB, 0.49817E+00_JPRB, 0.49766E+00_JPRB, 0.49544E+00_JPRB, 0.48372E+00_JPRB ,&
&  0.47556E+00_JPRB, 0.47212E+00_JPRB, 0.47493E+00_JPRB, 0.47847E+00_JPRB, 0.48691E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.49825E+00_JPRB, 0.50498E+00_JPRB, 0.51311E+00_JPRB, 0.51855E+00_JPRB, 0.51826E+00_JPRB, 0.50553E+00_JPRB, 0.49343E+00_JPRB ,&
&  0.48580E+00_JPRB, 0.48563E+00_JPRB, 0.48528E+00_JPRB, 0.48454E+00_JPRB, 0.48492E+00_JPRB, 0.48559E+00_JPRB, 0.48675E+00_JPRB ,&
&  0.48868E+00_JPRB, 0.49093E+00_JPRB, 0.49187E+00_JPRB, 0.48696E+00_JPRB, 0.48205E+00_JPRB, 0.48047E+00_JPRB, 0.48151E+00_JPRB ,&
&  0.48171E+00_JPRB, 0.47799E+00_JPRB, 0.47590E+00_JPRB, 0.46832E+00_JPRB, 0.45900E+00_JPRB, 0.44843E+00_JPRB, 0.43925E+00_JPRB ,&
&  0.43807E+00_JPRB, 0.44062E+00_JPRB, 0.43736E+00_JPRB, 0.44179E+00_JPRB, 0.45080E+00_JPRB, 0.43209E+00_JPRB, 0.40894E+00_JPRB ,&
&  0.38871E+00_JPRB, 0.37401E+00_JPRB, 0.36715E+00_JPRB, 0.36763E+00_JPRB, 0.38479E+00_JPRB, 0.41021E+00_JPRB, 0.42330E+00_JPRB ,&
&  0.41227E+00_JPRB, 0.42697E+00_JPRB, 0.44500E+00_JPRB, 0.45629E+00_JPRB, 0.46665E+00_JPRB, 0.47992E+00_JPRB, 0.49268E+00_JPRB ,&
&  0.50518E+00_JPRB, 0.51689E+00_JPRB, 0.52836E+00_JPRB, 0.55051E+00_JPRB, 0.56625E+00_JPRB, 0.55961E+00_JPRB, 0.57463E+00_JPRB ,&
&  0.67550E+00_JPRB, 0.75180E+00_JPRB, 0.80024E+00_JPRB, 0.81875E+00_JPRB, 0.81828E+00_JPRB, 0.79696E+00_JPRB, 0.77659E+00_JPRB ,&
&  0.75989E+00_JPRB, 0.74840E+00_JPRB, 0.74376E+00_JPRB, 0.74449E+00_JPRB, 0.75012E+00_JPRB, 0.74793E+00_JPRB, 0.74333E+00_JPRB ,&
&  0.73462E+00_JPRB, 0.75381E+00_JPRB, 0.98263E+00_JPRB, 0.89633E+00_JPRB, 0.81764E+00_JPRB, 0.77680E+00_JPRB, 0.73662E+00_JPRB ,&
&  0.70459E+00_JPRB, 0.68234E+00_JPRB, 0.66992E+00_JPRB, 0.65683E+00_JPRB, 0.62140E+00_JPRB, 0.59316E+00_JPRB, 0.57445E+00_JPRB ,&
&  0.56410E+00_JPRB, 0.55289E+00_JPRB, 0.54253E+00_JPRB, 0.53424E+00_JPRB, 0.52516E+00_JPRB, 0.51476E+00_JPRB, 0.49899E+00_JPRB ,&
&  0.48782E+00_JPRB, 0.48595E+00_JPRB, 0.48694E+00_JPRB, 0.49145E+00_JPRB, 0.49470E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.48937E+00_JPRB, 0.49502E+00_JPRB, 0.50103E+00_JPRB, 0.50573E+00_JPRB, 0.50701E+00_JPRB, 0.50586E+00_JPRB, 0.50228E+00_JPRB ,&
&  0.49880E+00_JPRB, 0.49763E+00_JPRB, 0.49683E+00_JPRB, 0.49696E+00_JPRB, 0.49738E+00_JPRB, 0.49800E+00_JPRB, 0.49864E+00_JPRB ,&
&  0.50042E+00_JPRB, 0.49122E+00_JPRB, 0.48635E+00_JPRB, 0.48720E+00_JPRB, 0.48684E+00_JPRB, 0.48711E+00_JPRB, 0.48814E+00_JPRB ,&
&  0.48404E+00_JPRB, 0.47562E+00_JPRB, 0.46906E+00_JPRB, 0.46077E+00_JPRB, 0.45206E+00_JPRB, 0.45123E+00_JPRB, 0.44944E+00_JPRB ,&
&  0.44177E+00_JPRB, 0.43405E+00_JPRB, 0.43808E+00_JPRB, 0.44024E+00_JPRB, 0.42454E+00_JPRB, 0.40705E+00_JPRB, 0.37998E+00_JPRB ,&
&  0.37373E+00_JPRB, 0.36183E+00_JPRB, 0.35939E+00_JPRB, 0.35979E+00_JPRB, 0.36754E+00_JPRB, 0.37807E+00_JPRB, 0.39642E+00_JPRB ,&
&  0.40564E+00_JPRB, 0.41317E+00_JPRB, 0.42648E+00_JPRB, 0.43999E+00_JPRB, 0.44928E+00_JPRB, 0.45761E+00_JPRB, 0.46172E+00_JPRB ,&
&  0.47000E+00_JPRB, 0.47938E+00_JPRB, 0.48853E+00_JPRB, 0.49772E+00_JPRB, 0.50960E+00_JPRB, 0.51664E+00_JPRB, 0.52622E+00_JPRB ,&
&  0.56327E+00_JPRB, 0.59296E+00_JPRB, 0.61683E+00_JPRB, 0.63838E+00_JPRB, 0.65396E+00_JPRB, 0.65993E+00_JPRB, 0.64778E+00_JPRB ,&
&  0.64171E+00_JPRB, 0.64948E+00_JPRB, 0.65713E+00_JPRB, 0.66697E+00_JPRB, 0.67850E+00_JPRB, 0.68813E+00_JPRB, 0.69097E+00_JPRB ,&
&  0.68122E+00_JPRB, 0.68832E+00_JPRB, 0.75868E+00_JPRB, 0.76024E+00_JPRB, 0.74290E+00_JPRB, 0.71510E+00_JPRB, 0.68343E+00_JPRB ,&
&  0.65883E+00_JPRB, 0.64331E+00_JPRB, 0.63313E+00_JPRB, 0.62454E+00_JPRB, 0.61726E+00_JPRB, 0.60871E+00_JPRB, 0.60570E+00_JPRB ,&
&  0.59743E+00_JPRB, 0.58603E+00_JPRB, 0.57312E+00_JPRB, 0.56067E+00_JPRB, 0.54908E+00_JPRB, 0.53711E+00_JPRB, 0.51611E+00_JPRB ,&
&  0.49425E+00_JPRB, 0.48042E+00_JPRB, 0.47683E+00_JPRB, 0.47826E+00_JPRB, 0.48375E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.48308E+00_JPRB, 0.48535E+00_JPRB, 0.48926E+00_JPRB, 0.49037E+00_JPRB, 0.49178E+00_JPRB, 0.49078E+00_JPRB, 0.49032E+00_JPRB ,&
&  0.48902E+00_JPRB, 0.48910E+00_JPRB, 0.48917E+00_JPRB, 0.48999E+00_JPRB, 0.49077E+00_JPRB, 0.49199E+00_JPRB, 0.49297E+00_JPRB ,&
&  0.49348E+00_JPRB, 0.49366E+00_JPRB, 0.49236E+00_JPRB, 0.49087E+00_JPRB, 0.48902E+00_JPRB, 0.48919E+00_JPRB, 0.48754E+00_JPRB ,&
&  0.48359E+00_JPRB, 0.47584E+00_JPRB, 0.46100E+00_JPRB, 0.45323E+00_JPRB, 0.45203E+00_JPRB, 0.44473E+00_JPRB, 0.44563E+00_JPRB ,&
&  0.44218E+00_JPRB, 0.44060E+00_JPRB, 0.43619E+00_JPRB, 0.43527E+00_JPRB, 0.42442E+00_JPRB, 0.36974E+00_JPRB, 0.36407E+00_JPRB ,&
&  0.35942E+00_JPRB, 0.35298E+00_JPRB, 0.35212E+00_JPRB, 0.35342E+00_JPRB, 0.35870E+00_JPRB, 0.37440E+00_JPRB, 0.38982E+00_JPRB ,&
&  0.39284E+00_JPRB, 0.41535E+00_JPRB, 0.42221E+00_JPRB, 0.42856E+00_JPRB, 0.44001E+00_JPRB, 0.44932E+00_JPRB, 0.45085E+00_JPRB ,&
&  0.45456E+00_JPRB, 0.45856E+00_JPRB, 0.46661E+00_JPRB, 0.47538E+00_JPRB, 0.48291E+00_JPRB, 0.48889E+00_JPRB, 0.49620E+00_JPRB ,&
&  0.50447E+00_JPRB, 0.51284E+00_JPRB, 0.52244E+00_JPRB, 0.53041E+00_JPRB, 0.53731E+00_JPRB, 0.54522E+00_JPRB, 0.55425E+00_JPRB ,&
&  0.56589E+00_JPRB, 0.57648E+00_JPRB, 0.59109E+00_JPRB, 0.59724E+00_JPRB, 0.60817E+00_JPRB, 0.61704E+00_JPRB, 0.63163E+00_JPRB ,&
&  0.64190E+00_JPRB, 0.65900E+00_JPRB, 0.66247E+00_JPRB, 0.66076E+00_JPRB, 0.64745E+00_JPRB, 0.63576E+00_JPRB, 0.62800E+00_JPRB ,&
&  0.62317E+00_JPRB, 0.62407E+00_JPRB, 0.61628E+00_JPRB, 0.61151E+00_JPRB, 0.60944E+00_JPRB, 0.60839E+00_JPRB, 0.60676E+00_JPRB ,&
&  0.60233E+00_JPRB, 0.59446E+00_JPRB, 0.58442E+00_JPRB, 0.57126E+00_JPRB, 0.55954E+00_JPRB, 0.54822E+00_JPRB, 0.52821E+00_JPRB ,&
&  0.50712E+00_JPRB, 0.48767E+00_JPRB, 0.47889E+00_JPRB, 0.47543E+00_JPRB, 0.47803E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.47962E+00_JPRB, 0.47901E+00_JPRB, 0.48111E+00_JPRB, 0.48039E+00_JPRB, 0.48183E+00_JPRB, 0.48147E+00_JPRB, 0.48168E+00_JPRB ,&
&  0.48083E+00_JPRB, 0.48067E+00_JPRB, 0.47992E+00_JPRB, 0.48025E+00_JPRB, 0.48032E+00_JPRB, 0.48144E+00_JPRB, 0.48225E+00_JPRB ,&
&  0.48352E+00_JPRB, 0.48417E+00_JPRB, 0.48514E+00_JPRB, 0.48498E+00_JPRB, 0.48518E+00_JPRB, 0.48474E+00_JPRB, 0.48236E+00_JPRB ,&
&  0.47815E+00_JPRB, 0.46937E+00_JPRB, 0.44945E+00_JPRB, 0.44475E+00_JPRB, 0.44006E+00_JPRB, 0.43961E+00_JPRB, 0.43744E+00_JPRB ,&
&  0.43506E+00_JPRB, 0.39776E+00_JPRB, 0.39950E+00_JPRB, 0.39010E+00_JPRB, 0.38370E+00_JPRB, 0.37735E+00_JPRB, 0.36807E+00_JPRB ,&
&  0.36466E+00_JPRB, 0.36129E+00_JPRB, 0.36100E+00_JPRB, 0.36532E+00_JPRB, 0.37060E+00_JPRB, 0.37936E+00_JPRB, 0.39007E+00_JPRB ,&
&  0.39315E+00_JPRB, 0.41143E+00_JPRB, 0.41961E+00_JPRB, 0.42370E+00_JPRB, 0.43613E+00_JPRB, 0.44572E+00_JPRB, 0.44770E+00_JPRB ,&
&  0.45119E+00_JPRB, 0.45129E+00_JPRB, 0.45598E+00_JPRB, 0.46288E+00_JPRB, 0.47080E+00_JPRB, 0.47382E+00_JPRB, 0.47706E+00_JPRB ,&
&  0.47993E+00_JPRB, 0.48250E+00_JPRB, 0.48523E+00_JPRB, 0.48665E+00_JPRB, 0.48928E+00_JPRB, 0.49520E+00_JPRB, 0.50302E+00_JPRB ,&
&  0.51268E+00_JPRB, 0.52314E+00_JPRB, 0.53509E+00_JPRB, 0.54648E+00_JPRB, 0.55642E+00_JPRB, 0.56572E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.58098E+00_JPRB, 0.58595E+00_JPRB, 0.58728E+00_JPRB, 0.58932E+00_JPRB, 0.58806E+00_JPRB, 0.58527E+00_JPRB, 0.58455E+00_JPRB ,&
&  0.58869E+00_JPRB, 0.58944E+00_JPRB, 0.58912E+00_JPRB, 0.58815E+00_JPRB, 0.58901E+00_JPRB, 0.58958E+00_JPRB, 0.58893E+00_JPRB ,&
&  0.58695E+00_JPRB, 0.58189E+00_JPRB, 0.57470E+00_JPRB, 0.56336E+00_JPRB, 0.55183E+00_JPRB, 0.54338E+00_JPRB, 0.53182E+00_JPRB ,&
&  0.51882E+00_JPRB, 0.50339E+00_JPRB, 0.49041E+00_JPRB, 0.48164E+00_JPRB, 0.47959E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.48040E+00_JPRB, 0.48003E+00_JPRB, 0.48170E+00_JPRB, 0.48226E+00_JPRB, 0.48382E+00_JPRB, 0.48369E+00_JPRB, 0.48438E+00_JPRB ,&
&  0.48349E+00_JPRB, 0.48314E+00_JPRB, 0.48251E+00_JPRB, 0.48198E+00_JPRB, 0.48153E+00_JPRB, 0.48113E+00_JPRB, 0.48104E+00_JPRB ,&
&  0.48159E+00_JPRB, 0.48153E+00_JPRB, 0.48254E+00_JPRB, 0.48258E+00_JPRB, 0.48224E+00_JPRB, 0.48159E+00_JPRB, 0.47725E+00_JPRB ,&
&  0.47212E+00_JPRB, 0.46720E+00_JPRB, 0.43904E+00_JPRB, 0.43544E+00_JPRB, 0.42906E+00_JPRB, 0.42563E+00_JPRB, 0.42587E+00_JPRB ,&
&  0.42278E+00_JPRB, 0.41912E+00_JPRB, 0.40834E+00_JPRB, 0.39694E+00_JPRB, 0.38762E+00_JPRB, 0.37611E+00_JPRB, 0.37332E+00_JPRB ,&
&  0.37062E+00_JPRB, 0.37292E+00_JPRB, 0.37597E+00_JPRB, 0.37921E+00_JPRB, 0.38708E+00_JPRB, 0.39310E+00_JPRB, 0.40212E+00_JPRB ,&
&  0.41302E+00_JPRB, 0.42134E+00_JPRB, 0.42422E+00_JPRB, 0.42167E+00_JPRB, 0.43370E+00_JPRB, 0.44128E+00_JPRB, 0.44798E+00_JPRB ,&
&  0.45182E+00_JPRB, 0.45272E+00_JPRB, 0.45595E+00_JPRB, 0.45724E+00_JPRB, 0.45959E+00_JPRB, 0.46360E+00_JPRB, 0.46602E+00_JPRB ,&
&  0.46885E+00_JPRB, 0.46984E+00_JPRB, 0.47011E+00_JPRB, 0.46935E+00_JPRB, 0.46740E+00_JPRB, 0.46684E+00_JPRB, 0.46827E+00_JPRB ,&
&  0.47169E+00_JPRB, 0.47746E+00_JPRB, 0.48471E+00_JPRB, 0.49335E+00_JPRB, 0.50181E+00_JPRB, 0.50953E+00_JPRB, 0.51551E+00_JPRB ,&
&  0.52041E+00_JPRB, 0.52403E+00_JPRB, 0.52974E+00_JPRB, 0.52999E+00_JPRB, 0.53336E+00_JPRB, 0.54277E+00_JPRB, 0.54394E+00_JPRB ,&
&  0.54721E+00_JPRB, 0.55176E+00_JPRB, 0.55478E+00_JPRB, 0.55613E+00_JPRB, 0.55702E+00_JPRB, 0.55772E+00_JPRB, 0.55798E+00_JPRB ,&
&  0.55727E+00_JPRB, 0.55389E+00_JPRB, 0.54880E+00_JPRB, 0.54222E+00_JPRB, 0.53599E+00_JPRB, 0.52953E+00_JPRB, 0.52184E+00_JPRB ,&
&  0.51387E+00_JPRB, 0.50549E+00_JPRB, 0.49605E+00_JPRB, 0.48708E+00_JPRB, 0.48260E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.48578E+00_JPRB, 0.48369E+00_JPRB, 0.48263E+00_JPRB, 0.48229E+00_JPRB, 0.48258E+00_JPRB, 0.48256E+00_JPRB, 0.48290E+00_JPRB ,&
&  0.48263E+00_JPRB, 0.48215E+00_JPRB, 0.48094E+00_JPRB, 0.48058E+00_JPRB, 0.48003E+00_JPRB, 0.47996E+00_JPRB, 0.47903E+00_JPRB ,&
&  0.47850E+00_JPRB, 0.47726E+00_JPRB, 0.47737E+00_JPRB, 0.47708E+00_JPRB, 0.47688E+00_JPRB, 0.47679E+00_JPRB, 0.47316E+00_JPRB ,&
&  0.46961E+00_JPRB, 0.46684E+00_JPRB, 0.46164E+00_JPRB, 0.45697E+00_JPRB, 0.45358E+00_JPRB, 0.42980E+00_JPRB, 0.42228E+00_JPRB ,&
&  0.42738E+00_JPRB, 0.43099E+00_JPRB, 0.43532E+00_JPRB, 0.43882E+00_JPRB, 0.43397E+00_JPRB, 0.42443E+00_JPRB, 0.41889E+00_JPRB ,&
&  0.39968E+00_JPRB, 0.40266E+00_JPRB, 0.40492E+00_JPRB, 0.40706E+00_JPRB, 0.41163E+00_JPRB, 0.41363E+00_JPRB, 0.41875E+00_JPRB ,&
&  0.43220E+00_JPRB, 0.43526E+00_JPRB, 0.43929E+00_JPRB, 0.44230E+00_JPRB, 0.44333E+00_JPRB, 0.44490E+00_JPRB, 0.44895E+00_JPRB ,&
&  0.45083E+00_JPRB, 0.45259E+00_JPRB, 0.45394E+00_JPRB, 0.45416E+00_JPRB, 0.45528E+00_JPRB, 0.45586E+00_JPRB, 0.45607E+00_JPRB ,&
&  0.45527E+00_JPRB, 0.45526E+00_JPRB, 0.45530E+00_JPRB, 0.45530E+00_JPRB, 0.45580E+00_JPRB, 0.45516E+00_JPRB, 0.45508E+00_JPRB ,&
&  0.45580E+00_JPRB, 0.45755E+00_JPRB, 0.45980E+00_JPRB, 0.46301E+00_JPRB, 0.46705E+00_JPRB, 0.47242E+00_JPRB, 0.47789E+00_JPRB ,&
&  0.48414E+00_JPRB, 0.49010E+00_JPRB, 0.49535E+00_JPRB, 0.49984E+00_JPRB, 0.50366E+00_JPRB, 0.50741E+00_JPRB, 0.51045E+00_JPRB ,&
&  0.51424E+00_JPRB, 0.51811E+00_JPRB, 0.52037E+00_JPRB, 0.52077E+00_JPRB, 0.52249E+00_JPRB, 0.52391E+00_JPRB, 0.52435E+00_JPRB ,&
&  0.52420E+00_JPRB, 0.52380E+00_JPRB, 0.52248E+00_JPRB, 0.52020E+00_JPRB, 0.51782E+00_JPRB, 0.51512E+00_JPRB, 0.51175E+00_JPRB ,&
&  0.50851E+00_JPRB, 0.50530E+00_JPRB, 0.50104E+00_JPRB, 0.49553E+00_JPRB, 0.49012E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.49198E+00_JPRB, 0.48964E+00_JPRB, 0.48752E+00_JPRB, 0.48576E+00_JPRB, 0.48397E+00_JPRB, 0.48283E+00_JPRB, 0.48183E+00_JPRB ,&
&  0.48092E+00_JPRB, 0.47988E+00_JPRB, 0.47865E+00_JPRB, 0.47776E+00_JPRB, 0.47729E+00_JPRB, 0.47707E+00_JPRB, 0.47626E+00_JPRB ,&
&  0.47562E+00_JPRB, 0.47483E+00_JPRB, 0.47432E+00_JPRB, 0.47358E+00_JPRB, 0.47260E+00_JPRB, 0.47143E+00_JPRB, 0.47013E+00_JPRB ,&
&  0.46905E+00_JPRB, 0.46770E+00_JPRB, 0.46612E+00_JPRB, 0.46367E+00_JPRB, 0.46149E+00_JPRB, 0.45912E+00_JPRB, 0.45659E+00_JPRB ,&
&  0.45378E+00_JPRB, 0.45154E+00_JPRB, 0.44958E+00_JPRB, 0.44772E+00_JPRB, 0.44902E+00_JPRB, 0.44973E+00_JPRB, 0.45007E+00_JPRB ,&
&  0.44979E+00_JPRB, 0.44788E+00_JPRB, 0.44728E+00_JPRB, 0.44681E+00_JPRB, 0.44643E+00_JPRB, 0.44820E+00_JPRB, 0.44943E+00_JPRB ,&
&  0.45027E+00_JPRB, 0.45072E+00_JPRB, 0.45220E+00_JPRB, 0.45293E+00_JPRB, 0.45321E+00_JPRB, 0.45285E+00_JPRB, 0.45202E+00_JPRB ,&
&  0.45221E+00_JPRB, 0.45239E+00_JPRB, 0.45235E+00_JPRB, 0.45189E+00_JPRB, 0.45202E+00_JPRB, 0.45217E+00_JPRB, 0.45187E+00_JPRB ,&
&  0.45122E+00_JPRB, 0.45104E+00_JPRB, 0.45079E+00_JPRB, 0.45046E+00_JPRB, 0.45000E+00_JPRB, 0.45006E+00_JPRB, 0.45022E+00_JPRB ,&
&  0.45061E+00_JPRB, 0.45136E+00_JPRB, 0.45241E+00_JPRB, 0.45368E+00_JPRB, 0.45520E+00_JPRB, 0.45744E+00_JPRB, 0.46044E+00_JPRB ,&
&  0.46412E+00_JPRB, 0.46837E+00_JPRB, 0.47278E+00_JPRB, 0.47636E+00_JPRB, 0.47933E+00_JPRB, 0.48201E+00_JPRB, 0.48504E+00_JPRB ,&
&  0.48795E+00_JPRB, 0.49037E+00_JPRB, 0.49270E+00_JPRB, 0.49491E+00_JPRB, 0.49730E+00_JPRB, 0.49891E+00_JPRB, 0.49998E+00_JPRB ,&
&  0.50067E+00_JPRB, 0.50162E+00_JPRB, 0.50259E+00_JPRB, 0.50361E+00_JPRB, 0.50419E+00_JPRB, 0.50397E+00_JPRB, 0.50323E+00_JPRB ,&
&  0.50220E+00_JPRB, 0.50104E+00_JPRB, 0.49951E+00_JPRB, 0.49716E+00_JPRB, 0.49468E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.48818E+00_JPRB, 0.48746E+00_JPRB, 0.48666E+00_JPRB, 0.48585E+00_JPRB, 0.48493E+00_JPRB, 0.48408E+00_JPRB, 0.48315E+00_JPRB ,&
&  0.48238E+00_JPRB, 0.48153E+00_JPRB, 0.48085E+00_JPRB, 0.48013E+00_JPRB, 0.47957E+00_JPRB, 0.47896E+00_JPRB, 0.47837E+00_JPRB ,&
&  0.47773E+00_JPRB, 0.47722E+00_JPRB, 0.47644E+00_JPRB, 0.47579E+00_JPRB, 0.47493E+00_JPRB, 0.47417E+00_JPRB, 0.47326E+00_JPRB ,&
&  0.47254E+00_JPRB, 0.47160E+00_JPRB, 0.47083E+00_JPRB, 0.46965E+00_JPRB, 0.46876E+00_JPRB, 0.46778E+00_JPRB, 0.46695E+00_JPRB ,&
&  0.46617E+00_JPRB, 0.46566E+00_JPRB, 0.46515E+00_JPRB, 0.46455E+00_JPRB, 0.46422E+00_JPRB, 0.46433E+00_JPRB, 0.46445E+00_JPRB ,&
&  0.46449E+00_JPRB, 0.46468E+00_JPRB, 0.46505E+00_JPRB, 0.46523E+00_JPRB, 0.46497E+00_JPRB, 0.46475E+00_JPRB, 0.46462E+00_JPRB ,&
&  0.46438E+00_JPRB, 0.46392E+00_JPRB, 0.46344E+00_JPRB, 0.46302E+00_JPRB, 0.46245E+00_JPRB, 0.46172E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.46096E+00_JPRB, 0.46076E+00_JPRB, 0.46048E+00_JPRB, 0.46010E+00_JPRB, 0.45993E+00_JPRB, 0.45980E+00_JPRB, 0.45962E+00_JPRB ,&
&  0.45944E+00_JPRB, 0.45930E+00_JPRB, 0.45906E+00_JPRB, 0.45879E+00_JPRB, 0.45861E+00_JPRB, 0.45875E+00_JPRB, 0.45905E+00_JPRB ,&
&  0.45946E+00_JPRB, 0.46009E+00_JPRB, 0.46097E+00_JPRB, 0.46186E+00_JPRB, 0.46281E+00_JPRB, 0.46396E+00_JPRB, 0.46520E+00_JPRB ,&
&  0.46653E+00_JPRB, 0.46794E+00_JPRB, 0.46958E+00_JPRB, 0.47134E+00_JPRB, 0.47302E+00_JPRB, 0.47469E+00_JPRB, 0.47660E+00_JPRB ,&
&  0.47823E+00_JPRB, 0.47960E+00_JPRB, 0.48075E+00_JPRB, 0.48195E+00_JPRB, 0.48329E+00_JPRB, 0.48441E+00_JPRB, 0.48534E+00_JPRB ,&
&  0.48607E+00_JPRB, 0.48677E+00_JPRB, 0.48742E+00_JPRB, 0.48813E+00_JPRB, 0.48842E+00_JPRB, 0.48867E+00_JPRB, 0.48890E+00_JPRB ,&
&  0.48905E+00_JPRB, 0.48910E+00_JPRB, 0.48906E+00_JPRB, 0.48895E+00_JPRB, 0.48870E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB, 0.47457E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB, 0.16012E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.17360E+00_JPRB, 0.17387E+00_JPRB, 0.17441E+00_JPRB, 0.17505E+00_JPRB, 0.17572E+00_JPRB, 0.17654E+00_JPRB, 0.17686E+00_JPRB ,&
&  0.17698E+00_JPRB, 0.17697E+00_JPRB, 0.17730E+00_JPRB, 0.17769E+00_JPRB, 0.17794E+00_JPRB, 0.17794E+00_JPRB, 0.17756E+00_JPRB ,&
&  0.17691E+00_JPRB, 0.17616E+00_JPRB, 0.17511E+00_JPRB, 0.17463E+00_JPRB, 0.17426E+00_JPRB, 0.17386E+00_JPRB, 0.17308E+00_JPRB ,&
&  0.17240E+00_JPRB, 0.17119E+00_JPRB, 0.16952E+00_JPRB, 0.16811E+00_JPRB, 0.16777E+00_JPRB, 0.16788E+00_JPRB, 0.16832E+00_JPRB ,&
&  0.16867E+00_JPRB, 0.16887E+00_JPRB, 0.16890E+00_JPRB, 0.16899E+00_JPRB, 0.16916E+00_JPRB, 0.16899E+00_JPRB, 0.16872E+00_JPRB ,&
&  0.16840E+00_JPRB, 0.16788E+00_JPRB, 0.16739E+00_JPRB, 0.16695E+00_JPRB, 0.16653E+00_JPRB, 0.16606E+00_JPRB, 0.16551E+00_JPRB ,&
&  0.16487E+00_JPRB, 0.16409E+00_JPRB, 0.16319E+00_JPRB, 0.16245E+00_JPRB, 0.16169E+00_JPRB, 0.16108E+00_JPRB, 0.16036E+00_JPRB ,&
&  0.15975E+00_JPRB, 0.15919E+00_JPRB, 0.15863E+00_JPRB, 0.15810E+00_JPRB, 0.15760E+00_JPRB, 0.15711E+00_JPRB, 0.15662E+00_JPRB ,&
&  0.15616E+00_JPRB, 0.15577E+00_JPRB, 0.15535E+00_JPRB, 0.15494E+00_JPRB, 0.15467E+00_JPRB, 0.15445E+00_JPRB, 0.15425E+00_JPRB ,&
&  0.15408E+00_JPRB, 0.15402E+00_JPRB, 0.15400E+00_JPRB, 0.15397E+00_JPRB, 0.15395E+00_JPRB, 0.15405E+00_JPRB, 0.15416E+00_JPRB ,&
&  0.15431E+00_JPRB, 0.15442E+00_JPRB, 0.15456E+00_JPRB, 0.15474E+00_JPRB, 0.15495E+00_JPRB, 0.15518E+00_JPRB, 0.15551E+00_JPRB ,&
&  0.15590E+00_JPRB, 0.15638E+00_JPRB, 0.15687E+00_JPRB, 0.15722E+00_JPRB, 0.15787E+00_JPRB, 0.15859E+00_JPRB, 0.15942E+00_JPRB ,&
&  0.16036E+00_JPRB, 0.16143E+00_JPRB, 0.16257E+00_JPRB, 0.16384E+00_JPRB, 0.16513E+00_JPRB, 0.16599E+00_JPRB, 0.16685E+00_JPRB ,&
&  0.16776E+00_JPRB, 0.16881E+00_JPRB, 0.16982E+00_JPRB, 0.17086E+00_JPRB, 0.17214E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.19542E+00_JPRB, 0.20239E+00_JPRB, 0.20948E+00_JPRB, 0.21956E+00_JPRB, 0.22746E+00_JPRB, 0.26271E+00_JPRB, 0.25759E+00_JPRB ,&
&  0.25397E+00_JPRB, 0.23968E+00_JPRB, 0.23789E+00_JPRB, 0.23415E+00_JPRB, 0.22958E+00_JPRB, 0.22458E+00_JPRB, 0.21960E+00_JPRB ,&
&  0.21492E+00_JPRB, 0.20946E+00_JPRB, 0.20467E+00_JPRB, 0.20093E+00_JPRB, 0.19758E+00_JPRB, 0.19432E+00_JPRB, 0.19144E+00_JPRB ,&
&  0.19014E+00_JPRB, 0.18990E+00_JPRB, 0.18907E+00_JPRB, 0.18529E+00_JPRB, 0.18197E+00_JPRB, 0.17990E+00_JPRB, 0.17922E+00_JPRB ,&
&  0.17911E+00_JPRB, 0.18053E+00_JPRB, 0.18129E+00_JPRB, 0.18186E+00_JPRB, 0.18276E+00_JPRB, 0.18194E+00_JPRB, 0.18112E+00_JPRB ,&
&  0.17999E+00_JPRB, 0.17870E+00_JPRB, 0.17732E+00_JPRB, 0.17615E+00_JPRB, 0.17497E+00_JPRB, 0.17323E+00_JPRB, 0.17169E+00_JPRB ,&
&  0.17014E+00_JPRB, 0.16824E+00_JPRB, 0.16583E+00_JPRB, 0.16421E+00_JPRB, 0.16236E+00_JPRB, 0.16078E+00_JPRB, 0.15921E+00_JPRB ,&
&  0.15806E+00_JPRB, 0.15700E+00_JPRB, 0.15572E+00_JPRB, 0.15417E+00_JPRB, 0.15262E+00_JPRB, 0.15123E+00_JPRB, 0.14990E+00_JPRB ,&
&  0.14916E+00_JPRB, 0.14841E+00_JPRB, 0.14766E+00_JPRB, 0.14686E+00_JPRB, 0.14646E+00_JPRB, 0.14607E+00_JPRB, 0.14571E+00_JPRB ,&
&  0.14541E+00_JPRB, 0.14527E+00_JPRB, 0.14532E+00_JPRB, 0.14542E+00_JPRB, 0.14552E+00_JPRB, 0.14588E+00_JPRB, 0.14631E+00_JPRB ,&
&  0.14688E+00_JPRB, 0.14759E+00_JPRB, 0.14855E+00_JPRB, 0.14953E+00_JPRB, 0.15043E+00_JPRB, 0.15116E+00_JPRB, 0.15214E+00_JPRB ,&
&  0.15369E+00_JPRB, 0.15550E+00_JPRB, 0.15746E+00_JPRB, 0.15903E+00_JPRB, 0.16113E+00_JPRB, 0.16303E+00_JPRB, 0.16492E+00_JPRB ,&
&  0.16679E+00_JPRB, 0.16895E+00_JPRB, 0.17130E+00_JPRB, 0.17377E+00_JPRB, 0.17658E+00_JPRB, 0.17923E+00_JPRB, 0.18167E+00_JPRB ,&
&  0.18417E+00_JPRB, 0.18756E+00_JPRB, 0.18966E+00_JPRB, 0.19102E+00_JPRB, 0.19282E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.26838E+00_JPRB, 0.27309E+00_JPRB, 0.27671E+00_JPRB, 0.27902E+00_JPRB, 0.28113E+00_JPRB, 0.28268E+00_JPRB, 0.28321E+00_JPRB ,&
&  0.27680E+00_JPRB, 0.27177E+00_JPRB, 0.26487E+00_JPRB, 0.25665E+00_JPRB, 0.24918E+00_JPRB, 0.24269E+00_JPRB, 0.23493E+00_JPRB ,&
&  0.22859E+00_JPRB, 0.22259E+00_JPRB, 0.21619E+00_JPRB, 0.20952E+00_JPRB, 0.20252E+00_JPRB, 0.19548E+00_JPRB, 0.19174E+00_JPRB ,&
&  0.19047E+00_JPRB, 0.19125E+00_JPRB, 0.19284E+00_JPRB, 0.19438E+00_JPRB, 0.19534E+00_JPRB, 0.19603E+00_JPRB, 0.19494E+00_JPRB ,&
&  0.19139E+00_JPRB, 0.19190E+00_JPRB, 0.19260E+00_JPRB, 0.19302E+00_JPRB, 0.19437E+00_JPRB, 0.19557E+00_JPRB, 0.19531E+00_JPRB ,&
&  0.19361E+00_JPRB, 0.19129E+00_JPRB, 0.18810E+00_JPRB, 0.18514E+00_JPRB, 0.18387E+00_JPRB, 0.18161E+00_JPRB, 0.17905E+00_JPRB ,&
&  0.17681E+00_JPRB, 0.17295E+00_JPRB, 0.16919E+00_JPRB, 0.16654E+00_JPRB, 0.16323E+00_JPRB, 0.16093E+00_JPRB, 0.15796E+00_JPRB ,&
&  0.15553E+00_JPRB, 0.15329E+00_JPRB, 0.15083E+00_JPRB, 0.14888E+00_JPRB, 0.14704E+00_JPRB, 0.14568E+00_JPRB, 0.14433E+00_JPRB ,&
&  0.14354E+00_JPRB, 0.14281E+00_JPRB, 0.14236E+00_JPRB, 0.14188E+00_JPRB, 0.14162E+00_JPRB, 0.14135E+00_JPRB, 0.14114E+00_JPRB ,&
&  0.14102E+00_JPRB, 0.14133E+00_JPRB, 0.14156E+00_JPRB, 0.14196E+00_JPRB, 0.14242E+00_JPRB, 0.14274E+00_JPRB, 0.14328E+00_JPRB ,&
&  0.14427E+00_JPRB, 0.14534E+00_JPRB, 0.14610E+00_JPRB, 0.14705E+00_JPRB, 0.14811E+00_JPRB, 0.14941E+00_JPRB, 0.15115E+00_JPRB ,&
&  0.15297E+00_JPRB, 0.15500E+00_JPRB, 0.15727E+00_JPRB, 0.15954E+00_JPRB, 0.16241E+00_JPRB, 0.16534E+00_JPRB, 0.16843E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17594E+00_JPRB, 0.17989E+00_JPRB, 0.18423E+00_JPRB, 0.19049E+00_JPRB, 0.19658E+00_JPRB, 0.20416E+00_JPRB ,&
&  0.21167E+00_JPRB, 0.21826E+00_JPRB, 0.24987E+00_JPRB, 0.25470E+00_JPRB, 0.26136E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.27514E+00_JPRB, 0.28582E+00_JPRB, 0.29459E+00_JPRB, 0.30006E+00_JPRB, 0.30579E+00_JPRB, 0.30808E+00_JPRB, 0.31667E+00_JPRB ,&
&  0.31120E+00_JPRB, 0.29298E+00_JPRB, 0.28179E+00_JPRB, 0.26542E+00_JPRB, 0.24514E+00_JPRB, 0.22870E+00_JPRB, 0.22056E+00_JPRB ,&
&  0.21578E+00_JPRB, 0.21208E+00_JPRB, 0.20754E+00_JPRB, 0.20254E+00_JPRB, 0.19746E+00_JPRB, 0.19373E+00_JPRB, 0.19125E+00_JPRB ,&
&  0.18904E+00_JPRB, 0.18948E+00_JPRB, 0.19178E+00_JPRB, 0.19585E+00_JPRB, 0.20160E+00_JPRB, 0.20588E+00_JPRB, 0.20918E+00_JPRB ,&
&  0.21187E+00_JPRB, 0.20365E+00_JPRB, 0.20463E+00_JPRB, 0.20606E+00_JPRB, 0.20458E+00_JPRB, 0.20690E+00_JPRB, 0.21046E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.20319E+00_JPRB, 0.20617E+00_JPRB, 0.20216E+00_JPRB, 0.19394E+00_JPRB, 0.19167E+00_JPRB, 0.18588E+00_JPRB ,&
&  0.18233E+00_JPRB, 0.17514E+00_JPRB, 0.17007E+00_JPRB, 0.16674E+00_JPRB, 0.16184E+00_JPRB, 0.15975E+00_JPRB, 0.15656E+00_JPRB ,&
&  0.15320E+00_JPRB, 0.15075E+00_JPRB, 0.14786E+00_JPRB, 0.14633E+00_JPRB, 0.14505E+00_JPRB, 0.14390E+00_JPRB, 0.14300E+00_JPRB ,&
&  0.14252E+00_JPRB, 0.14206E+00_JPRB, 0.14211E+00_JPRB, 0.14217E+00_JPRB, 0.14225E+00_JPRB, 0.14262E+00_JPRB, 0.14289E+00_JPRB ,&
&  0.14351E+00_JPRB, 0.14420E+00_JPRB, 0.14475E+00_JPRB, 0.14560E+00_JPRB, 0.14603E+00_JPRB, 0.14655E+00_JPRB, 0.14696E+00_JPRB ,&
&  0.14743E+00_JPRB, 0.14811E+00_JPRB, 0.14852E+00_JPRB, 0.14923E+00_JPRB, 0.15005E+00_JPRB, 0.15153E+00_JPRB, 0.15300E+00_JPRB ,&
&  0.15478E+00_JPRB, 0.15679E+00_JPRB, 0.15896E+00_JPRB, 0.16125E+00_JPRB, 0.16407E+00_JPRB, 0.16671E+00_JPRB, 0.17014E+00_JPRB ,&
&  0.17385E+00_JPRB, 0.17736E+00_JPRB, 0.18130E+00_JPRB, 0.18641E+00_JPRB, 0.19307E+00_JPRB, 0.20249E+00_JPRB, 0.21159E+00_JPRB ,&
&  0.22205E+00_JPRB, 0.24881E+00_JPRB, 0.25519E+00_JPRB, 0.25848E+00_JPRB, 0.26675E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.30158E+00_JPRB, 0.31091E+00_JPRB, 0.32494E+00_JPRB, 0.33393E+00_JPRB, 0.34123E+00_JPRB, 0.33709E+00_JPRB, 0.33319E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.31492E+00_JPRB, 0.29780E+00_JPRB, 0.26858E+00_JPRB, 0.24394E+00_JPRB, 0.22509E+00_JPRB, 0.21485E+00_JPRB ,&
&  0.20890E+00_JPRB, 0.20703E+00_JPRB, 0.20538E+00_JPRB, 0.20510E+00_JPRB, 0.20418E+00_JPRB, 0.20598E+00_JPRB, 0.20751E+00_JPRB ,&
&  0.20725E+00_JPRB, 0.20614E+00_JPRB, 0.20543E+00_JPRB, 0.20850E+00_JPRB, 0.21438E+00_JPRB, 0.21904E+00_JPRB, 0.22350E+00_JPRB ,&
&  0.22686E+00_JPRB, 0.22445E+00_JPRB, 0.22544E+00_JPRB, 0.22335E+00_JPRB, 0.22085E+00_JPRB, 0.22068E+00_JPRB, 0.22131E+00_JPRB ,&
&  0.22179E+00_JPRB, 0.22253E+00_JPRB, 0.22198E+00_JPRB, 0.21926E+00_JPRB, 0.20794E+00_JPRB, 0.20199E+00_JPRB, 0.19446E+00_JPRB ,&
&  0.18643E+00_JPRB, 0.17710E+00_JPRB, 0.16974E+00_JPRB, 0.16611E+00_JPRB, 0.16203E+00_JPRB, 0.16032E+00_JPRB, 0.15721E+00_JPRB ,&
&  0.15369E+00_JPRB, 0.15031E+00_JPRB, 0.14792E+00_JPRB, 0.14670E+00_JPRB, 0.14624E+00_JPRB, 0.14535E+00_JPRB, 0.14500E+00_JPRB ,&
&  0.14520E+00_JPRB, 0.14549E+00_JPRB, 0.14622E+00_JPRB, 0.14700E+00_JPRB, 0.14789E+00_JPRB, 0.14873E+00_JPRB, 0.14977E+00_JPRB ,&
&  0.15134E+00_JPRB, 0.15318E+00_JPRB, 0.15402E+00_JPRB, 0.15471E+00_JPRB, 0.15450E+00_JPRB, 0.15413E+00_JPRB, 0.15352E+00_JPRB ,&
&  0.15303E+00_JPRB, 0.15291E+00_JPRB, 0.15291E+00_JPRB, 0.15287E+00_JPRB, 0.15335E+00_JPRB, 0.15424E+00_JPRB, 0.15522E+00_JPRB ,&
&  0.15675E+00_JPRB, 0.15815E+00_JPRB, 0.15972E+00_JPRB, 0.16152E+00_JPRB, 0.16361E+00_JPRB, 0.16605E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.17139E+00_JPRB, 0.17459E+00_JPRB, 0.17730E+00_JPRB, 0.17944E+00_JPRB, 0.18408E+00_JPRB, 0.19093E+00_JPRB, 0.20452E+00_JPRB ,&
&  0.22068E+00_JPRB, 0.26140E+00_JPRB, 0.30428E+00_JPRB, 0.28898E+00_JPRB, 0.29365E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.30057E+00_JPRB, 0.31250E+00_JPRB, 0.32779E+00_JPRB, 0.33717E+00_JPRB, 0.34226E+00_JPRB, 0.34480E+00_JPRB, 0.34352E+00_JPRB ,&
&  0.34010E+00_JPRB, 0.33966E+00_JPRB, 0.35082E+00_JPRB, 0.33330E+00_JPRB, 0.31545E+00_JPRB, 0.28190E+00_JPRB, 0.26548E+00_JPRB ,&
&  0.25336E+00_JPRB, 0.25132E+00_JPRB, 0.25124E+00_JPRB, 0.25604E+00_JPRB, 0.26010E+00_JPRB, 0.26333E+00_JPRB, 0.25991E+00_JPRB ,&
&  0.25487E+00_JPRB, 0.24583E+00_JPRB, 0.23823E+00_JPRB, 0.23102E+00_JPRB, 0.23104E+00_JPRB, 0.22998E+00_JPRB, 0.22742E+00_JPRB ,&
&  0.22440E+00_JPRB, 0.22239E+00_JPRB, 0.22216E+00_JPRB, 0.22209E+00_JPRB, 0.23380E+00_JPRB, 0.23337E+00_JPRB, 0.23305E+00_JPRB ,&
&  0.23253E+00_JPRB, 0.23250E+00_JPRB, 0.23157E+00_JPRB, 0.22986E+00_JPRB, 0.22530E+00_JPRB, 0.21374E+00_JPRB, 0.21359E+00_JPRB ,&
&  0.20371E+00_JPRB, 0.19297E+00_JPRB, 0.18276E+00_JPRB, 0.17565E+00_JPRB, 0.17132E+00_JPRB, 0.16922E+00_JPRB, 0.16454E+00_JPRB ,&
&  0.15961E+00_JPRB, 0.15404E+00_JPRB, 0.15130E+00_JPRB, 0.15087E+00_JPRB, 0.15134E+00_JPRB, 0.15188E+00_JPRB, 0.15238E+00_JPRB ,&
&  0.15351E+00_JPRB, 0.15409E+00_JPRB, 0.15441E+00_JPRB, 0.15502E+00_JPRB, 0.15562E+00_JPRB, 0.15658E+00_JPRB, 0.15821E+00_JPRB ,&
&  0.16116E+00_JPRB, 0.16443E+00_JPRB, 0.16742E+00_JPRB, 0.16967E+00_JPRB, 0.16872E+00_JPRB, 0.16576E+00_JPRB, 0.16174E+00_JPRB ,&
&  0.15886E+00_JPRB, 0.15799E+00_JPRB, 0.15719E+00_JPRB, 0.15721E+00_JPRB, 0.15748E+00_JPRB, 0.15796E+00_JPRB, 0.15863E+00_JPRB ,&
&  0.15945E+00_JPRB, 0.16013E+00_JPRB, 0.16096E+00_JPRB, 0.16202E+00_JPRB, 0.16355E+00_JPRB, 0.16494E+00_JPRB, 0.16692E+00_JPRB ,&
&  0.16987E+00_JPRB, 0.17311E+00_JPRB, 0.17768E+00_JPRB, 0.18174E+00_JPRB, 0.18696E+00_JPRB, 0.19489E+00_JPRB, 0.20797E+00_JPRB ,&
&  0.22228E+00_JPRB, 0.25710E+00_JPRB, 0.27343E+00_JPRB, 0.28329E+00_JPRB, 0.29033E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.29832E+00_JPRB, 0.30712E+00_JPRB, 0.31927E+00_JPRB, 0.32962E+00_JPRB, 0.33849E+00_JPRB, 0.34695E+00_JPRB, 0.35414E+00_JPRB ,&
&  0.35780E+00_JPRB, 0.35991E+00_JPRB, 0.36062E+00_JPRB, 0.36402E+00_JPRB, 0.36107E+00_JPRB, 0.35459E+00_JPRB, 0.34919E+00_JPRB ,&
&  0.34479E+00_JPRB, 0.34127E+00_JPRB, 0.33610E+00_JPRB, 0.33338E+00_JPRB, 0.33317E+00_JPRB, 0.33337E+00_JPRB, 0.32783E+00_JPRB ,&
&  0.31767E+00_JPRB, 0.31717E+00_JPRB, 0.32210E+00_JPRB, 0.31702E+00_JPRB, 0.31156E+00_JPRB, 0.30645E+00_JPRB, 0.30075E+00_JPRB ,&
&  0.28545E+00_JPRB, 0.26585E+00_JPRB, 0.23862E+00_JPRB, 0.23358E+00_JPRB, 0.24954E+00_JPRB, 0.24381E+00_JPRB, 0.24650E+00_JPRB ,&
&  0.24679E+00_JPRB, 0.24726E+00_JPRB, 0.24581E+00_JPRB, 0.24443E+00_JPRB, 0.24096E+00_JPRB, 0.23966E+00_JPRB, 0.23839E+00_JPRB ,&
&  0.23340E+00_JPRB, 0.22872E+00_JPRB, 0.22121E+00_JPRB, 0.20391E+00_JPRB, 0.19816E+00_JPRB, 0.19574E+00_JPRB, 0.19047E+00_JPRB ,&
&  0.18274E+00_JPRB, 0.17646E+00_JPRB, 0.17358E+00_JPRB, 0.17032E+00_JPRB, 0.17236E+00_JPRB, 0.17391E+00_JPRB, 0.17391E+00_JPRB ,&
&  0.17391E+00_JPRB, 0.17356E+00_JPRB, 0.17129E+00_JPRB, 0.16757E+00_JPRB, 0.16468E+00_JPRB, 0.16356E+00_JPRB, 0.16398E+00_JPRB ,&
&  0.16611E+00_JPRB, 0.17059E+00_JPRB, 0.18099E+00_JPRB, 0.18925E+00_JPRB, 0.18855E+00_JPRB, 0.18381E+00_JPRB, 0.17511E+00_JPRB ,&
&  0.16820E+00_JPRB, 0.16549E+00_JPRB, 0.16324E+00_JPRB, 0.16212E+00_JPRB, 0.16253E+00_JPRB, 0.16398E+00_JPRB, 0.16489E+00_JPRB ,&
&  0.16593E+00_JPRB, 0.16687E+00_JPRB, 0.16769E+00_JPRB, 0.16824E+00_JPRB, 0.16912E+00_JPRB, 0.17011E+00_JPRB, 0.17192E+00_JPRB ,&
&  0.17462E+00_JPRB, 0.17891E+00_JPRB, 0.18459E+00_JPRB, 0.19066E+00_JPRB, 0.19498E+00_JPRB, 0.19706E+00_JPRB, 0.20042E+00_JPRB ,&
&  0.20608E+00_JPRB, 0.22026E+00_JPRB, 0.24829E+00_JPRB, 0.29404E+00_JPRB, 0.29846E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.30173E+00_JPRB, 0.30603E+00_JPRB, 0.30882E+00_JPRB, 0.31234E+00_JPRB, 0.31991E+00_JPRB, 0.33069E+00_JPRB, 0.34288E+00_JPRB ,&
&  0.35388E+00_JPRB, 0.36260E+00_JPRB, 0.36681E+00_JPRB, 0.37188E+00_JPRB, 0.37678E+00_JPRB, 0.37919E+00_JPRB, 0.38446E+00_JPRB ,&
&  0.38523E+00_JPRB, 0.38190E+00_JPRB, 0.37588E+00_JPRB, 0.36786E+00_JPRB, 0.36366E+00_JPRB, 0.36506E+00_JPRB, 0.36812E+00_JPRB ,&
&  0.37022E+00_JPRB, 0.37186E+00_JPRB, 0.37207E+00_JPRB, 0.36883E+00_JPRB, 0.36563E+00_JPRB, 0.36378E+00_JPRB, 0.36000E+00_JPRB ,&
&  0.35182E+00_JPRB, 0.30272E+00_JPRB, 0.26494E+00_JPRB, 0.26013E+00_JPRB, 0.27750E+00_JPRB, 0.27998E+00_JPRB, 0.27455E+00_JPRB ,&
&  0.26870E+00_JPRB, 0.26415E+00_JPRB, 0.26159E+00_JPRB, 0.25957E+00_JPRB, 0.25702E+00_JPRB, 0.25488E+00_JPRB, 0.25598E+00_JPRB ,&
&  0.25743E+00_JPRB, 0.25534E+00_JPRB, 0.25234E+00_JPRB, 0.24612E+00_JPRB, 0.24143E+00_JPRB, 0.23899E+00_JPRB, 0.23701E+00_JPRB ,&
&  0.23271E+00_JPRB, 0.22852E+00_JPRB, 0.22808E+00_JPRB, 0.22889E+00_JPRB, 0.23037E+00_JPRB, 0.23140E+00_JPRB, 0.22887E+00_JPRB ,&
&  0.22222E+00_JPRB, 0.21586E+00_JPRB, 0.20755E+00_JPRB, 0.19775E+00_JPRB, 0.18830E+00_JPRB, 0.18180E+00_JPRB, 0.17681E+00_JPRB ,&
&  0.17565E+00_JPRB, 0.18040E+00_JPRB, 0.19251E+00_JPRB, 0.20801E+00_JPRB, 0.22950E+00_JPRB, 0.21740E+00_JPRB, 0.20577E+00_JPRB ,&
&  0.19548E+00_JPRB, 0.18752E+00_JPRB, 0.18160E+00_JPRB, 0.17883E+00_JPRB, 0.17779E+00_JPRB, 0.17791E+00_JPRB, 0.17901E+00_JPRB ,&
&  0.18096E+00_JPRB, 0.18358E+00_JPRB, 0.18602E+00_JPRB, 0.18721E+00_JPRB, 0.18792E+00_JPRB, 0.18858E+00_JPRB, 0.18866E+00_JPRB ,&
&  0.18802E+00_JPRB, 0.19014E+00_JPRB, 0.19665E+00_JPRB, 0.20504E+00_JPRB, 0.21337E+00_JPRB, 0.22101E+00_JPRB, 0.23390E+00_JPRB ,&
&  0.24970E+00_JPRB, 0.26993E+00_JPRB, 0.28162E+00_JPRB, 0.29039E+00_JPRB, 0.29817E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.32675E+00_JPRB, 0.32818E+00_JPRB, 0.32561E+00_JPRB, 0.32269E+00_JPRB, 0.32225E+00_JPRB, 0.32408E+00_JPRB, 0.33003E+00_JPRB ,&
&  0.33982E+00_JPRB, 0.35213E+00_JPRB, 0.36388E+00_JPRB, 0.37371E+00_JPRB, 0.38265E+00_JPRB, 0.38664E+00_JPRB, 0.39271E+00_JPRB ,&
&  0.39917E+00_JPRB, 0.40508E+00_JPRB, 0.40586E+00_JPRB, 0.39563E+00_JPRB, 0.38569E+00_JPRB, 0.38042E+00_JPRB, 0.38045E+00_JPRB ,&
&  0.38234E+00_JPRB, 0.38513E+00_JPRB, 0.39149E+00_JPRB, 0.39401E+00_JPRB, 0.38832E+00_JPRB, 0.38707E+00_JPRB, 0.38507E+00_JPRB ,&
&  0.37974E+00_JPRB, 0.36905E+00_JPRB, 0.33706E+00_JPRB, 0.33198E+00_JPRB, 0.32406E+00_JPRB, 0.31516E+00_JPRB, 0.30611E+00_JPRB ,&
&  0.29590E+00_JPRB, 0.28578E+00_JPRB, 0.27893E+00_JPRB, 0.27501E+00_JPRB, 0.27591E+00_JPRB, 0.27723E+00_JPRB, 0.27684E+00_JPRB ,&
&  0.27904E+00_JPRB, 0.28206E+00_JPRB, 0.28449E+00_JPRB, 0.28474E+00_JPRB, 0.28403E+00_JPRB, 0.28243E+00_JPRB, 0.27736E+00_JPRB ,&
&  0.27281E+00_JPRB, 0.27019E+00_JPRB, 0.26810E+00_JPRB, 0.26624E+00_JPRB, 0.26566E+00_JPRB, 0.26588E+00_JPRB, 0.26585E+00_JPRB ,&
&  0.26367E+00_JPRB, 0.25996E+00_JPRB, 0.25581E+00_JPRB, 0.24900E+00_JPRB, 0.23784E+00_JPRB, 0.22192E+00_JPRB, 0.21250E+00_JPRB ,&
&  0.21082E+00_JPRB, 0.22184E+00_JPRB, 0.24379E+00_JPRB, 0.25164E+00_JPRB, 0.25887E+00_JPRB, 0.26296E+00_JPRB, 0.26771E+00_JPRB ,&
&  0.26429E+00_JPRB, 0.23939E+00_JPRB, 0.22590E+00_JPRB, 0.21915E+00_JPRB, 0.21347E+00_JPRB, 0.20887E+00_JPRB, 0.20868E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.21841E+00_JPRB, 0.22476E+00_JPRB, 0.22786E+00_JPRB, 0.22725E+00_JPRB, 0.22377E+00_JPRB, 0.22039E+00_JPRB ,&
&  0.21722E+00_JPRB, 0.22144E+00_JPRB, 0.23604E+00_JPRB, 0.25947E+00_JPRB, 0.26750E+00_JPRB, 0.27361E+00_JPRB, 0.27933E+00_JPRB ,&
&  0.28541E+00_JPRB, 0.29461E+00_JPRB, 0.30591E+00_JPRB, 0.31496E+00_JPRB, 0.32300E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.34230E+00_JPRB, 0.34203E+00_JPRB, 0.34342E+00_JPRB, 0.34226E+00_JPRB, 0.33968E+00_JPRB, 0.33793E+00_JPRB, 0.34178E+00_JPRB ,&
&  0.34881E+00_JPRB, 0.36029E+00_JPRB, 0.37544E+00_JPRB, 0.38874E+00_JPRB, 0.39683E+00_JPRB, 0.40186E+00_JPRB, 0.40958E+00_JPRB ,&
&  0.41672E+00_JPRB, 0.42464E+00_JPRB, 0.42479E+00_JPRB, 0.41592E+00_JPRB, 0.40837E+00_JPRB, 0.40437E+00_JPRB, 0.40379E+00_JPRB ,&
&  0.40314E+00_JPRB, 0.40499E+00_JPRB, 0.40832E+00_JPRB, 0.41192E+00_JPRB, 0.41149E+00_JPRB, 0.40986E+00_JPRB, 0.41377E+00_JPRB ,&
&  0.41221E+00_JPRB, 0.40310E+00_JPRB, 0.39401E+00_JPRB, 0.36936E+00_JPRB, 0.36915E+00_JPRB, 0.35916E+00_JPRB, 0.35137E+00_JPRB ,&
&  0.34089E+00_JPRB, 0.32805E+00_JPRB, 0.31878E+00_JPRB, 0.31287E+00_JPRB, 0.31029E+00_JPRB, 0.30778E+00_JPRB, 0.30588E+00_JPRB ,&
&  0.30468E+00_JPRB, 0.30460E+00_JPRB, 0.30614E+00_JPRB, 0.30420E+00_JPRB, 0.30305E+00_JPRB, 0.30375E+00_JPRB, 0.30514E+00_JPRB ,&
&  0.30446E+00_JPRB, 0.30360E+00_JPRB, 0.30292E+00_JPRB, 0.30157E+00_JPRB, 0.29969E+00_JPRB, 0.29711E+00_JPRB, 0.29463E+00_JPRB ,&
&  0.29345E+00_JPRB, 0.29419E+00_JPRB, 0.29426E+00_JPRB, 0.29032E+00_JPRB, 0.28266E+00_JPRB, 0.27380E+00_JPRB, 0.26800E+00_JPRB ,&
&  0.26541E+00_JPRB, 0.26728E+00_JPRB, 0.27488E+00_JPRB, 0.28588E+00_JPRB, 0.29622E+00_JPRB, 0.30228E+00_JPRB, 0.30590E+00_JPRB ,&
&  0.31034E+00_JPRB, 0.30898E+00_JPRB, 0.30544E+00_JPRB, 0.30216E+00_JPRB, 0.29911E+00_JPRB, 0.29512E+00_JPRB, 0.29501E+00_JPRB ,&
&  0.29737E+00_JPRB, 0.29959E+00_JPRB, 0.30016E+00_JPRB, 0.29888E+00_JPRB, 0.29672E+00_JPRB, 0.29232E+00_JPRB, 0.28616E+00_JPRB ,&
&  0.27867E+00_JPRB, 0.27004E+00_JPRB, 0.26780E+00_JPRB, 0.26815E+00_JPRB, 0.27193E+00_JPRB, 0.27979E+00_JPRB, 0.29048E+00_JPRB ,&
&  0.30177E+00_JPRB, 0.31120E+00_JPRB, 0.32130E+00_JPRB, 0.33060E+00_JPRB, 0.33881E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.36716E+00_JPRB, 0.36909E+00_JPRB, 0.37142E+00_JPRB, 0.37444E+00_JPRB, 0.37782E+00_JPRB, 0.37998E+00_JPRB, 0.38402E+00_JPRB ,&
&  0.38629E+00_JPRB, 0.39258E+00_JPRB, 0.40304E+00_JPRB, 0.41367E+00_JPRB, 0.42152E+00_JPRB, 0.42828E+00_JPRB, 0.43480E+00_JPRB ,&
&  0.44006E+00_JPRB, 0.44708E+00_JPRB, 0.44791E+00_JPRB, 0.44539E+00_JPRB, 0.44259E+00_JPRB, 0.44040E+00_JPRB, 0.43845E+00_JPRB ,&
&  0.43590E+00_JPRB, 0.43575E+00_JPRB, 0.43955E+00_JPRB, 0.44390E+00_JPRB, 0.44845E+00_JPRB, 0.44996E+00_JPRB, 0.44904E+00_JPRB ,&
&  0.45323E+00_JPRB, 0.44438E+00_JPRB, 0.43883E+00_JPRB, 0.43216E+00_JPRB, 0.42624E+00_JPRB, 0.41512E+00_JPRB, 0.40700E+00_JPRB ,&
&  0.39737E+00_JPRB, 0.38905E+00_JPRB, 0.38559E+00_JPRB, 0.38186E+00_JPRB, 0.37233E+00_JPRB, 0.36114E+00_JPRB, 0.35279E+00_JPRB ,&
&  0.34733E+00_JPRB, 0.34200E+00_JPRB, 0.33582E+00_JPRB, 0.33079E+00_JPRB, 0.32644E+00_JPRB, 0.32485E+00_JPRB, 0.32653E+00_JPRB ,&
&  0.33038E+00_JPRB, 0.33373E+00_JPRB, 0.33546E+00_JPRB, 0.33563E+00_JPRB, 0.33390E+00_JPRB, 0.33114E+00_JPRB, 0.32940E+00_JPRB ,&
&  0.32968E+00_JPRB, 0.33079E+00_JPRB, 0.33290E+00_JPRB, 0.33802E+00_JPRB, 0.33856E+00_JPRB, 0.33701E+00_JPRB, 0.33139E+00_JPRB ,&
&  0.32850E+00_JPRB, 0.32693E+00_JPRB, 0.32837E+00_JPRB, 0.33334E+00_JPRB, 0.34156E+00_JPRB, 0.34951E+00_JPRB, 0.35525E+00_JPRB ,&
&  0.35819E+00_JPRB, 0.36010E+00_JPRB, 0.36325E+00_JPRB, 0.36681E+00_JPRB, 0.36808E+00_JPRB, 0.36719E+00_JPRB, 0.36547E+00_JPRB ,&
&  0.36801E+00_JPRB, 0.37516E+00_JPRB, 0.38273E+00_JPRB, 0.38736E+00_JPRB, 0.38944E+00_JPRB, 0.38816E+00_JPRB, 0.38118E+00_JPRB ,&
&  0.36867E+00_JPRB, 0.35439E+00_JPRB, 0.34053E+00_JPRB, 0.32873E+00_JPRB, 0.32055E+00_JPRB, 0.31674E+00_JPRB, 0.31971E+00_JPRB ,&
&  0.32761E+00_JPRB, 0.33771E+00_JPRB, 0.34712E+00_JPRB, 0.35417E+00_JPRB, 0.36183E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.39182E+00_JPRB, 0.39199E+00_JPRB, 0.39500E+00_JPRB, 0.40150E+00_JPRB, 0.40584E+00_JPRB, 0.40987E+00_JPRB, 0.41764E+00_JPRB ,&
&  0.42460E+00_JPRB, 0.43072E+00_JPRB, 0.43814E+00_JPRB, 0.44641E+00_JPRB, 0.45290E+00_JPRB, 0.46208E+00_JPRB, 0.47066E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.47828E+00_JPRB, 0.47942E+00_JPRB, 0.48039E+00_JPRB, 0.48004E+00_JPRB, 0.47735E+00_JPRB, 0.47677E+00_JPRB ,&
&  0.47491E+00_JPRB, 0.47290E+00_JPRB, 0.47511E+00_JPRB, 0.47557E+00_JPRB, 0.47753E+00_JPRB, 0.48075E+00_JPRB, 0.48145E+00_JPRB ,&
&  0.48006E+00_JPRB, 0.46778E+00_JPRB, 0.46366E+00_JPRB, 0.46126E+00_JPRB, 0.46591E+00_JPRB, 0.46740E+00_JPRB, 0.46635E+00_JPRB ,&
&  0.46040E+00_JPRB, 0.45694E+00_JPRB, 0.45907E+00_JPRB, 0.46162E+00_JPRB, 0.45603E+00_JPRB, 0.44328E+00_JPRB, 0.43302E+00_JPRB ,&
&  0.42500E+00_JPRB, 0.41317E+00_JPRB, 0.39624E+00_JPRB, 0.38637E+00_JPRB, 0.38219E+00_JPRB, 0.37723E+00_JPRB, 0.37237E+00_JPRB ,&
&  0.36968E+00_JPRB, 0.36876E+00_JPRB, 0.37014E+00_JPRB, 0.36912E+00_JPRB, 0.36521E+00_JPRB, 0.36045E+00_JPRB, 0.35846E+00_JPRB ,&
&  0.36080E+00_JPRB, 0.36285E+00_JPRB, 0.36613E+00_JPRB, 0.36856E+00_JPRB, 0.36785E+00_JPRB, 0.36819E+00_JPRB, 0.37168E+00_JPRB ,&
&  0.37684E+00_JPRB, 0.38161E+00_JPRB, 0.38694E+00_JPRB, 0.38913E+00_JPRB, 0.39193E+00_JPRB, 0.39828E+00_JPRB, 0.39859E+00_JPRB ,&
&  0.39717E+00_JPRB, 0.39927E+00_JPRB, 0.40337E+00_JPRB, 0.41097E+00_JPRB, 0.42261E+00_JPRB, 0.43026E+00_JPRB, 0.43132E+00_JPRB ,&
&  0.43130E+00_JPRB, 0.43347E+00_JPRB, 0.43703E+00_JPRB, 0.44582E+00_JPRB, 0.45541E+00_JPRB, 0.46238E+00_JPRB, 0.46535E+00_JPRB ,&
&  0.46238E+00_JPRB, 0.45583E+00_JPRB, 0.44386E+00_JPRB, 0.42978E+00_JPRB, 0.41714E+00_JPRB, 0.40724E+00_JPRB, 0.39913E+00_JPRB ,&
&  0.39123E+00_JPRB, 0.38797E+00_JPRB, 0.38782E+00_JPRB, 0.38983E+00_JPRB, 0.39109E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.42464E+00_JPRB, 0.42549E+00_JPRB, 0.43037E+00_JPRB, 0.43529E+00_JPRB, 0.43884E+00_JPRB, 0.44387E+00_JPRB, 0.45339E+00_JPRB ,&
&  0.46305E+00_JPRB, 0.47239E+00_JPRB, 0.47882E+00_JPRB, 0.48662E+00_JPRB, 0.49751E+00_JPRB, 0.50485E+00_JPRB, 0.51069E+00_JPRB ,&
&  0.51314E+00_JPRB, 0.51338E+00_JPRB, 0.51415E+00_JPRB, 0.51707E+00_JPRB, 0.51652E+00_JPRB, 0.51284E+00_JPRB, 0.51151E+00_JPRB ,&
&  0.51141E+00_JPRB, 0.50981E+00_JPRB, 0.50974E+00_JPRB, 0.51116E+00_JPRB, 0.50935E+00_JPRB, 0.50617E+00_JPRB, 0.50635E+00_JPRB ,&
&  0.50737E+00_JPRB, 0.49437E+00_JPRB, 0.48747E+00_JPRB, 0.48107E+00_JPRB, 0.48524E+00_JPRB, 0.49713E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.51787E+00_JPRB, 0.52322E+00_JPRB, 0.52299E+00_JPRB, 0.52217E+00_JPRB, 0.51775E+00_JPRB, 0.50659E+00_JPRB, 0.49707E+00_JPRB ,&
&  0.49090E+00_JPRB, 0.48221E+00_JPRB, 0.46064E+00_JPRB, 0.44831E+00_JPRB, 0.44525E+00_JPRB, 0.44788E+00_JPRB, 0.45148E+00_JPRB ,&
&  0.45298E+00_JPRB, 0.45056E+00_JPRB, 0.44465E+00_JPRB, 0.43733E+00_JPRB, 0.42725E+00_JPRB, 0.41549E+00_JPRB, 0.40522E+00_JPRB ,&
&  0.39920E+00_JPRB, 0.39462E+00_JPRB, 0.39084E+00_JPRB, 0.39308E+00_JPRB, 0.39228E+00_JPRB, 0.38846E+00_JPRB, 0.39008E+00_JPRB ,&
&  0.40074E+00_JPRB, 0.41414E+00_JPRB, 0.42661E+00_JPRB, 0.43346E+00_JPRB, 0.43614E+00_JPRB, 0.43926E+00_JPRB, 0.44308E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.45480E+00_JPRB, 0.45968E+00_JPRB, 0.46567E+00_JPRB, 0.47323E+00_JPRB, 0.48019E+00_JPRB, 0.48310E+00_JPRB ,&
&  0.48562E+00_JPRB, 0.48698E+00_JPRB, 0.48767E+00_JPRB, 0.49221E+00_JPRB, 0.49960E+00_JPRB, 0.50491E+00_JPRB, 0.50934E+00_JPRB ,&
&  0.51404E+00_JPRB, 0.51629E+00_JPRB, 0.51503E+00_JPRB, 0.51111E+00_JPRB, 0.50558E+00_JPRB, 0.49448E+00_JPRB, 0.48081E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.45102E+00_JPRB, 0.44081E+00_JPRB, 0.43203E+00_JPRB, 0.42604E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.46873E+00_JPRB, 0.46685E+00_JPRB, 0.46743E+00_JPRB, 0.47069E+00_JPRB, 0.47435E+00_JPRB, 0.47726E+00_JPRB, 0.48324E+00_JPRB ,&
&  0.49328E+00_JPRB, 0.50115E+00_JPRB, 0.51069E+00_JPRB, 0.52036E+00_JPRB, 0.52771E+00_JPRB, 0.53267E+00_JPRB, 0.53537E+00_JPRB ,&
&  0.53368E+00_JPRB, 0.53093E+00_JPRB, 0.53354E+00_JPRB, 0.53797E+00_JPRB, 0.53928E+00_JPRB, 0.53880E+00_JPRB, 0.53881E+00_JPRB ,&
&  0.53842E+00_JPRB, 0.53641E+00_JPRB, 0.53395E+00_JPRB, 0.53028E+00_JPRB, 0.52586E+00_JPRB, 0.52421E+00_JPRB, 0.52262E+00_JPRB ,&
&  0.51672E+00_JPRB, 0.51669E+00_JPRB, 0.46719E+00_JPRB, 0.48288E+00_JPRB, 0.49619E+00_JPRB, 0.51635E+00_JPRB, 0.53580E+00_JPRB ,&
&  0.54984E+00_JPRB, 0.55900E+00_JPRB, 0.56374E+00_JPRB, 0.56257E+00_JPRB, 0.55084E+00_JPRB, 0.54354E+00_JPRB, 0.53520E+00_JPRB ,&
&  0.52496E+00_JPRB, 0.51782E+00_JPRB, 0.50554E+00_JPRB, 0.49203E+00_JPRB, 0.48760E+00_JPRB, 0.48814E+00_JPRB, 0.48798E+00_JPRB ,&
&  0.48437E+00_JPRB, 0.48108E+00_JPRB, 0.48250E+00_JPRB, 0.48343E+00_JPRB, 0.48303E+00_JPRB, 0.47746E+00_JPRB, 0.47042E+00_JPRB ,&
&  0.45960E+00_JPRB, 0.44898E+00_JPRB, 0.44161E+00_JPRB, 0.43605E+00_JPRB, 0.43112E+00_JPRB, 0.42569E+00_JPRB, 0.42027E+00_JPRB ,&
&  0.42319E+00_JPRB, 0.43374E+00_JPRB, 0.44362E+00_JPRB, 0.45183E+00_JPRB, 0.45724E+00_JPRB, 0.46094E+00_JPRB, 0.46613E+00_JPRB ,&
&  0.47218E+00_JPRB, 0.48225E+00_JPRB, 0.48899E+00_JPRB, 0.49493E+00_JPRB, 0.49943E+00_JPRB, 0.50369E+00_JPRB, 0.50868E+00_JPRB ,&
&  0.51268E+00_JPRB, 0.51896E+00_JPRB, 0.52328E+00_JPRB, 0.52864E+00_JPRB, 0.53154E+00_JPRB, 0.53605E+00_JPRB, 0.54179E+00_JPRB ,&
&  0.54832E+00_JPRB, 0.55417E+00_JPRB, 0.55648E+00_JPRB, 0.55482E+00_JPRB, 0.54730E+00_JPRB, 0.53709E+00_JPRB, 0.52732E+00_JPRB ,&
&  0.51502E+00_JPRB, 0.50097E+00_JPRB, 0.48708E+00_JPRB, 0.47833E+00_JPRB, 0.47114E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.52793E+00_JPRB, 0.52318E+00_JPRB, 0.51795E+00_JPRB, 0.51737E+00_JPRB, 0.51664E+00_JPRB, 0.51897E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.52942E+00_JPRB, 0.53404E+00_JPRB, 0.54334E+00_JPRB, 0.55077E+00_JPRB, 0.55435E+00_JPRB, 0.55851E+00_JPRB, 0.56117E+00_JPRB ,&
&  0.56223E+00_JPRB, 0.56364E+00_JPRB, 0.56433E+00_JPRB, 0.56855E+00_JPRB, 0.56928E+00_JPRB, 0.56854E+00_JPRB, 0.56766E+00_JPRB ,&
&  0.56610E+00_JPRB, 0.56455E+00_JPRB, 0.56024E+00_JPRB, 0.55400E+00_JPRB, 0.54604E+00_JPRB, 0.54303E+00_JPRB, 0.53758E+00_JPRB ,&
&  0.53929E+00_JPRB, 0.50817E+00_JPRB, 0.51997E+00_JPRB, 0.51864E+00_JPRB, 0.53252E+00_JPRB, 0.55432E+00_JPRB, 0.58523E+00_JPRB ,&
&  0.60696E+00_JPRB, 0.61878E+00_JPRB, 0.62159E+00_JPRB, 0.62126E+00_JPRB, 0.60905E+00_JPRB, 0.59626E+00_JPRB, 0.58146E+00_JPRB ,&
&  0.56418E+00_JPRB, 0.55438E+00_JPRB, 0.54132E+00_JPRB, 0.53163E+00_JPRB, 0.52779E+00_JPRB, 0.52499E+00_JPRB, 0.52027E+00_JPRB ,&
&  0.51312E+00_JPRB, 0.50692E+00_JPRB, 0.50739E+00_JPRB, 0.51068E+00_JPRB, 0.51397E+00_JPRB, 0.51584E+00_JPRB, 0.51436E+00_JPRB ,&
&  0.50771E+00_JPRB, 0.50175E+00_JPRB, 0.49852E+00_JPRB, 0.49765E+00_JPRB, 0.49276E+00_JPRB, 0.48455E+00_JPRB, 0.47608E+00_JPRB ,&
&  0.47386E+00_JPRB, 0.47637E+00_JPRB, 0.47989E+00_JPRB, 0.48449E+00_JPRB, 0.48803E+00_JPRB, 0.49239E+00_JPRB, 0.49452E+00_JPRB ,&
&  0.49932E+00_JPRB, 0.50393E+00_JPRB, 0.51316E+00_JPRB, 0.52199E+00_JPRB, 0.52433E+00_JPRB, 0.53291E+00_JPRB, 0.54167E+00_JPRB ,&
&  0.55008E+00_JPRB, 0.55959E+00_JPRB, 0.56656E+00_JPRB, 0.57078E+00_JPRB, 0.57524E+00_JPRB, 0.57773E+00_JPRB, 0.58278E+00_JPRB ,&
&  0.58987E+00_JPRB, 0.59119E+00_JPRB, 0.59604E+00_JPRB, 0.59580E+00_JPRB, 0.58979E+00_JPRB, 0.58045E+00_JPRB, 0.56941E+00_JPRB ,&
&  0.55782E+00_JPRB, 0.54286E+00_JPRB, 0.53542E+00_JPRB, 0.53124E+00_JPRB, 0.52933E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.57159E+00_JPRB, 0.56512E+00_JPRB, 0.55464E+00_JPRB, 0.54882E+00_JPRB, 0.54689E+00_JPRB, 0.55068E+00_JPRB, 0.55569E+00_JPRB ,&
&  0.56090E+00_JPRB, 0.56307E+00_JPRB, 0.57059E+00_JPRB, 0.57803E+00_JPRB, 0.58235E+00_JPRB, 0.58516E+00_JPRB, 0.58424E+00_JPRB ,&
&  0.58414E+00_JPRB, 0.58405E+00_JPRB, 0.57966E+00_JPRB, 0.57625E+00_JPRB, 0.57128E+00_JPRB, 0.57209E+00_JPRB, 0.57240E+00_JPRB ,&
&  0.57175E+00_JPRB, 0.56946E+00_JPRB, 0.56415E+00_JPRB, 0.55648E+00_JPRB, 0.55343E+00_JPRB, 0.55358E+00_JPRB, 0.55183E+00_JPRB ,&
&  0.55868E+00_JPRB, 0.48083E+00_JPRB, 0.51685E+00_JPRB, 0.53106E+00_JPRB, 0.55333E+00_JPRB, 0.58362E+00_JPRB, 0.61572E+00_JPRB ,&
&  0.64654E+00_JPRB, 0.67183E+00_JPRB, 0.68064E+00_JPRB, 0.68148E+00_JPRB, 0.67161E+00_JPRB, 0.65384E+00_JPRB, 0.63222E+00_JPRB ,&
&  0.61293E+00_JPRB, 0.59795E+00_JPRB, 0.58017E+00_JPRB, 0.56981E+00_JPRB, 0.56158E+00_JPRB, 0.55101E+00_JPRB, 0.54210E+00_JPRB ,&
&  0.53843E+00_JPRB, 0.53624E+00_JPRB, 0.53601E+00_JPRB, 0.53754E+00_JPRB, 0.54009E+00_JPRB, 0.54168E+00_JPRB, 0.54097E+00_JPRB ,&
&  0.53926E+00_JPRB, 0.53627E+00_JPRB, 0.53168E+00_JPRB, 0.52951E+00_JPRB, 0.52842E+00_JPRB, 0.52946E+00_JPRB, 0.52539E+00_JPRB ,&
&  0.51877E+00_JPRB, 0.51993E+00_JPRB, 0.52293E+00_JPRB, 0.52491E+00_JPRB, 0.52793E+00_JPRB, 0.53121E+00_JPRB, 0.53166E+00_JPRB ,&
&  0.53202E+00_JPRB, 0.53599E+00_JPRB, 0.54029E+00_JPRB, 0.54715E+00_JPRB, 0.55202E+00_JPRB, 0.56106E+00_JPRB, 0.56845E+00_JPRB ,&
&  0.57539E+00_JPRB, 0.58595E+00_JPRB, 0.59559E+00_JPRB, 0.60156E+00_JPRB, 0.60502E+00_JPRB, 0.60895E+00_JPRB, 0.61334E+00_JPRB ,&
&  0.61648E+00_JPRB, 0.61898E+00_JPRB, 0.62121E+00_JPRB, 0.62024E+00_JPRB, 0.61417E+00_JPRB, 0.60802E+00_JPRB, 0.60391E+00_JPRB ,&
&  0.59456E+00_JPRB, 0.58304E+00_JPRB, 0.57897E+00_JPRB, 0.57609E+00_JPRB, 0.57302E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.63754E+00_JPRB, 0.62991E+00_JPRB, 0.61700E+00_JPRB, 0.60623E+00_JPRB, 0.59778E+00_JPRB, 0.59554E+00_JPRB, 0.59471E+00_JPRB ,&
&  0.60012E+00_JPRB, 0.60845E+00_JPRB, 0.61580E+00_JPRB, 0.61936E+00_JPRB, 0.62161E+00_JPRB, 0.62115E+00_JPRB, 0.61739E+00_JPRB ,&
&  0.61432E+00_JPRB, 0.60310E+00_JPRB, 0.59490E+00_JPRB, 0.59514E+00_JPRB, 0.59229E+00_JPRB, 0.58921E+00_JPRB, 0.58582E+00_JPRB ,&
&  0.58286E+00_JPRB, 0.58102E+00_JPRB, 0.57365E+00_JPRB, 0.57078E+00_JPRB, 0.57071E+00_JPRB, 0.57347E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.51157E+00_JPRB, 0.43628E+00_JPRB, 0.49296E+00_JPRB, 0.55862E+00_JPRB, 0.59703E+00_JPRB, 0.63903E+00_JPRB, 0.67581E+00_JPRB ,&
&  0.72408E+00_JPRB, 0.75551E+00_JPRB, 0.77302E+00_JPRB, 0.76694E+00_JPRB, 0.74750E+00_JPRB, 0.72547E+00_JPRB, 0.70265E+00_JPRB ,&
&  0.67587E+00_JPRB, 0.64615E+00_JPRB, 0.62704E+00_JPRB, 0.61733E+00_JPRB, 0.60905E+00_JPRB, 0.60009E+00_JPRB, 0.59485E+00_JPRB ,&
&  0.59319E+00_JPRB, 0.59220E+00_JPRB, 0.59057E+00_JPRB, 0.58964E+00_JPRB, 0.59162E+00_JPRB, 0.59365E+00_JPRB, 0.59580E+00_JPRB ,&
&  0.59540E+00_JPRB, 0.59361E+00_JPRB, 0.59008E+00_JPRB, 0.58332E+00_JPRB, 0.57800E+00_JPRB, 0.57805E+00_JPRB, 0.57599E+00_JPRB ,&
&  0.57482E+00_JPRB, 0.57323E+00_JPRB, 0.57308E+00_JPRB, 0.57368E+00_JPRB, 0.57310E+00_JPRB, 0.57283E+00_JPRB, 0.57511E+00_JPRB ,&
&  0.58041E+00_JPRB, 0.58063E+00_JPRB, 0.58363E+00_JPRB, 0.58906E+00_JPRB, 0.59460E+00_JPRB, 0.59969E+00_JPRB, 0.60269E+00_JPRB ,&
&  0.60811E+00_JPRB, 0.61966E+00_JPRB, 0.62781E+00_JPRB, 0.63286E+00_JPRB, 0.63713E+00_JPRB, 0.64216E+00_JPRB, 0.64938E+00_JPRB ,&
&  0.65491E+00_JPRB, 0.65958E+00_JPRB, 0.66548E+00_JPRB, 0.67488E+00_JPRB, 0.67777E+00_JPRB, 0.67509E+00_JPRB, 0.66878E+00_JPRB ,&
&  0.65761E+00_JPRB, 0.64144E+00_JPRB, 0.64252E+00_JPRB, 0.64043E+00_JPRB, 0.63837E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.66462E+00_JPRB, 0.66476E+00_JPRB, 0.65672E+00_JPRB, 0.64588E+00_JPRB, 0.63816E+00_JPRB, 0.63609E+00_JPRB, 0.63637E+00_JPRB ,&
&  0.64125E+00_JPRB, 0.64517E+00_JPRB, 0.64576E+00_JPRB, 0.64212E+00_JPRB, 0.63643E+00_JPRB, 0.62780E+00_JPRB, 0.62096E+00_JPRB ,&
&  0.60827E+00_JPRB, 0.59460E+00_JPRB, 0.59039E+00_JPRB, 0.59058E+00_JPRB, 0.58852E+00_JPRB, 0.58268E+00_JPRB, 0.58037E+00_JPRB ,&
&  0.58071E+00_JPRB, 0.58460E+00_JPRB, 0.58013E+00_JPRB, 0.57785E+00_JPRB, 0.57433E+00_JPRB, 0.57713E+00_JPRB, 0.57921E+00_JPRB ,&
&  0.57976E+00_JPRB, 0.46187E+00_JPRB, 0.53613E+00_JPRB, 0.60033E+00_JPRB, 0.62930E+00_JPRB, 0.68281E+00_JPRB, 0.73919E+00_JPRB ,&
&  0.78121E+00_JPRB, 0.80183E+00_JPRB, 0.79708E+00_JPRB, 0.77977E+00_JPRB, 0.75172E+00_JPRB, 0.72332E+00_JPRB, 0.69780E+00_JPRB ,&
&  0.67384E+00_JPRB, 0.65661E+00_JPRB, 0.64322E+00_JPRB, 0.63149E+00_JPRB, 0.62451E+00_JPRB, 0.61787E+00_JPRB, 0.61206E+00_JPRB ,&
&  0.61285E+00_JPRB, 0.61977E+00_JPRB, 0.62099E+00_JPRB, 0.62395E+00_JPRB, 0.62794E+00_JPRB, 0.63022E+00_JPRB, 0.63832E+00_JPRB ,&
&  0.64275E+00_JPRB, 0.64150E+00_JPRB, 0.63977E+00_JPRB, 0.63509E+00_JPRB, 0.62487E+00_JPRB, 0.62057E+00_JPRB, 0.61522E+00_JPRB ,&
&  0.61053E+00_JPRB, 0.60279E+00_JPRB, 0.59661E+00_JPRB, 0.59454E+00_JPRB, 0.59505E+00_JPRB, 0.59539E+00_JPRB, 0.60156E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.61471E+00_JPRB, 0.61722E+00_JPRB, 0.62206E+00_JPRB, 0.62591E+00_JPRB, 0.62811E+00_JPRB, 0.63164E+00_JPRB ,&
&  0.64032E+00_JPRB, 0.65474E+00_JPRB, 0.66798E+00_JPRB, 0.67344E+00_JPRB, 0.67550E+00_JPRB, 0.68275E+00_JPRB, 0.69052E+00_JPRB ,&
&  0.69863E+00_JPRB, 0.70101E+00_JPRB, 0.70153E+00_JPRB, 0.72585E+00_JPRB, 0.73518E+00_JPRB, 0.73028E+00_JPRB, 0.72092E+00_JPRB ,&
&  0.70186E+00_JPRB, 0.69472E+00_JPRB, 0.56215E+00_JPRB, 0.62321E+00_JPRB, 0.64397E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.70973E+00_JPRB, 0.71279E+00_JPRB, 0.70418E+00_JPRB, 0.69171E+00_JPRB, 0.68723E+00_JPRB, 0.68600E+00_JPRB, 0.68561E+00_JPRB ,&
&  0.67956E+00_JPRB, 0.66996E+00_JPRB, 0.65881E+00_JPRB, 0.64596E+00_JPRB, 0.63266E+00_JPRB, 0.62184E+00_JPRB, 0.61125E+00_JPRB ,&
&  0.59341E+00_JPRB, 0.58366E+00_JPRB, 0.58390E+00_JPRB, 0.58125E+00_JPRB, 0.57877E+00_JPRB, 0.57539E+00_JPRB, 0.57718E+00_JPRB ,&
&  0.58144E+00_JPRB, 0.58341E+00_JPRB, 0.58435E+00_JPRB, 0.57903E+00_JPRB, 0.57524E+00_JPRB, 0.57620E+00_JPRB, 0.57309E+00_JPRB ,&
&  0.57452E+00_JPRB, 0.48121E+00_JPRB, 0.56160E+00_JPRB, 0.65717E+00_JPRB, 0.70240E+00_JPRB, 0.76503E+00_JPRB, 0.84325E+00_JPRB ,&
&  0.88272E+00_JPRB, 0.86501E+00_JPRB, 0.83096E+00_JPRB, 0.79808E+00_JPRB, 0.76246E+00_JPRB, 0.72954E+00_JPRB, 0.70243E+00_JPRB ,&
&  0.68491E+00_JPRB, 0.67407E+00_JPRB, 0.66603E+00_JPRB, 0.65707E+00_JPRB, 0.64772E+00_JPRB, 0.64409E+00_JPRB, 0.64217E+00_JPRB ,&
&  0.64279E+00_JPRB, 0.64718E+00_JPRB, 0.64962E+00_JPRB, 0.65647E+00_JPRB, 0.66242E+00_JPRB, 0.67437E+00_JPRB, 0.69238E+00_JPRB ,&
&  0.69759E+00_JPRB, 0.70647E+00_JPRB, 0.72122E+00_JPRB, 0.72032E+00_JPRB, 0.70496E+00_JPRB, 0.69540E+00_JPRB, 0.68605E+00_JPRB ,&
&  0.67439E+00_JPRB, 0.65512E+00_JPRB, 0.65087E+00_JPRB, 0.64669E+00_JPRB, 0.64244E+00_JPRB, 0.64131E+00_JPRB, 0.65055E+00_JPRB ,&
&  0.65972E+00_JPRB, 0.66284E+00_JPRB, 0.66840E+00_JPRB, 0.67713E+00_JPRB, 0.68144E+00_JPRB, 0.67792E+00_JPRB, 0.67717E+00_JPRB ,&
&  0.68409E+00_JPRB, 0.69549E+00_JPRB, 0.70077E+00_JPRB, 0.70145E+00_JPRB, 0.70902E+00_JPRB, 0.72201E+00_JPRB, 0.73046E+00_JPRB ,&
&  0.73620E+00_JPRB, 0.73714E+00_JPRB, 0.74346E+00_JPRB, 0.72995E+00_JPRB, 0.77668E+00_JPRB, 0.77037E+00_JPRB, 0.75697E+00_JPRB ,&
&  0.74586E+00_JPRB, 0.73224E+00_JPRB, 0.73007E+00_JPRB, 0.63085E+00_JPRB, 0.67676E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.75593E+00_JPRB, 0.76246E+00_JPRB, 0.74704E+00_JPRB, 0.73075E+00_JPRB, 0.72184E+00_JPRB, 0.71152E+00_JPRB, 0.69097E+00_JPRB ,&
&  0.66777E+00_JPRB, 0.64692E+00_JPRB, 0.63492E+00_JPRB, 0.61913E+00_JPRB, 0.60436E+00_JPRB, 0.59594E+00_JPRB, 0.58634E+00_JPRB ,&
&  0.57111E+00_JPRB, 0.56789E+00_JPRB, 0.56859E+00_JPRB, 0.56717E+00_JPRB, 0.56347E+00_JPRB, 0.56204E+00_JPRB, 0.57025E+00_JPRB ,&
&  0.57195E+00_JPRB, 0.56993E+00_JPRB, 0.56980E+00_JPRB, 0.56671E+00_JPRB, 0.56265E+00_JPRB, 0.56306E+00_JPRB, 0.55297E+00_JPRB ,&
&  0.55855E+00_JPRB, 0.53217E+00_JPRB, 0.64322E+00_JPRB, 0.75883E+00_JPRB, 0.85663E+00_JPRB, 0.91277E+00_JPRB, 0.94725E+00_JPRB ,&
&  0.95167E+00_JPRB, 0.91017E+00_JPRB, 0.85735E+00_JPRB, 0.81504E+00_JPRB, 0.76997E+00_JPRB, 0.73619E+00_JPRB, 0.71066E+00_JPRB ,&
&  0.69412E+00_JPRB, 0.68550E+00_JPRB, 0.67911E+00_JPRB, 0.67328E+00_JPRB, 0.66727E+00_JPRB, 0.66258E+00_JPRB, 0.66123E+00_JPRB ,&
&  0.65810E+00_JPRB, 0.65875E+00_JPRB, 0.65980E+00_JPRB, 0.65695E+00_JPRB, 0.66164E+00_JPRB, 0.68641E+00_JPRB, 0.71779E+00_JPRB ,&
&  0.72678E+00_JPRB, 0.74279E+00_JPRB, 0.76423E+00_JPRB, 0.76971E+00_JPRB, 0.75721E+00_JPRB, 0.74936E+00_JPRB, 0.73859E+00_JPRB ,&
&  0.73046E+00_JPRB, 0.71649E+00_JPRB, 0.71323E+00_JPRB, 0.70626E+00_JPRB, 0.69649E+00_JPRB, 0.69104E+00_JPRB, 0.69032E+00_JPRB ,&
&  0.69130E+00_JPRB, 0.69288E+00_JPRB, 0.69998E+00_JPRB, 0.70838E+00_JPRB, 0.70644E+00_JPRB, 0.70020E+00_JPRB, 0.70045E+00_JPRB ,&
&  0.70401E+00_JPRB, 0.69958E+00_JPRB, 0.69017E+00_JPRB, 0.69962E+00_JPRB, 0.71927E+00_JPRB, 0.73085E+00_JPRB, 0.73728E+00_JPRB ,&
&  0.74342E+00_JPRB, 0.75166E+00_JPRB, 0.76472E+00_JPRB, 0.80191E+00_JPRB, 0.80885E+00_JPRB, 0.79297E+00_JPRB, 0.78278E+00_JPRB ,&
&  0.77793E+00_JPRB, 0.76712E+00_JPRB, 0.75547E+00_JPRB, 0.75009E+00_JPRB, 0.70099E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.90179E+00_JPRB, 0.84140E+00_JPRB, 0.79167E+00_JPRB, 0.75608E+00_JPRB, 0.73261E+00_JPRB, 0.70246E+00_JPRB, 0.66635E+00_JPRB ,&
&  0.64040E+00_JPRB, 0.63130E+00_JPRB, 0.62157E+00_JPRB, 0.60696E+00_JPRB, 0.59707E+00_JPRB, 0.58528E+00_JPRB, 0.57495E+00_JPRB ,&
&  0.56086E+00_JPRB, 0.55688E+00_JPRB, 0.55298E+00_JPRB, 0.55750E+00_JPRB, 0.55907E+00_JPRB, 0.56479E+00_JPRB, 0.57694E+00_JPRB ,&
&  0.57934E+00_JPRB, 0.55924E+00_JPRB, 0.55620E+00_JPRB, 0.55904E+00_JPRB, 0.56117E+00_JPRB, 0.56322E+00_JPRB, 0.55225E+00_JPRB ,&
&  0.56949E+00_JPRB, 0.58892E+00_JPRB, 0.80401E+00_JPRB, 0.10120E+01_JPRB, 0.11197E+01_JPRB, 0.11552E+01_JPRB, 0.11052E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10040E+01_JPRB, 0.93400E+00_JPRB, 0.85920E+00_JPRB, 0.78937E+00_JPRB, 0.74535E+00_JPRB, 0.72084E+00_JPRB ,&
&  0.70729E+00_JPRB, 0.70264E+00_JPRB, 0.69696E+00_JPRB, 0.69433E+00_JPRB, 0.69173E+00_JPRB, 0.68719E+00_JPRB, 0.68891E+00_JPRB ,&
&  0.69441E+00_JPRB, 0.68315E+00_JPRB, 0.67690E+00_JPRB, 0.68602E+00_JPRB, 0.70107E+00_JPRB, 0.72078E+00_JPRB, 0.77422E+00_JPRB ,&
&  0.80265E+00_JPRB, 0.82783E+00_JPRB, 0.85381E+00_JPRB, 0.86075E+00_JPRB, 0.85211E+00_JPRB, 0.84366E+00_JPRB, 0.82502E+00_JPRB ,&
&  0.81454E+00_JPRB, 0.80268E+00_JPRB, 0.78823E+00_JPRB, 0.77243E+00_JPRB, 0.75685E+00_JPRB, 0.74750E+00_JPRB, 0.74159E+00_JPRB ,&
&  0.73597E+00_JPRB, 0.74380E+00_JPRB, 0.75021E+00_JPRB, 0.75118E+00_JPRB, 0.74136E+00_JPRB, 0.73394E+00_JPRB, 0.72897E+00_JPRB ,&
&  0.71564E+00_JPRB, 0.69602E+00_JPRB, 0.68944E+00_JPRB, 0.70020E+00_JPRB, 0.72567E+00_JPRB, 0.73619E+00_JPRB, 0.73959E+00_JPRB ,&
&  0.75392E+00_JPRB, 0.77032E+00_JPRB, 0.76268E+00_JPRB, 0.79789E+00_JPRB, 0.86659E+00_JPRB, 0.85647E+00_JPRB, 0.84089E+00_JPRB ,&
&  0.81942E+00_JPRB, 0.81003E+00_JPRB, 0.79186E+00_JPRB, 0.77887E+00_JPRB, 0.88412E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.80516E+00_JPRB, 0.77382E+00_JPRB, 0.74540E+00_JPRB, 0.71284E+00_JPRB, 0.69090E+00_JPRB, 0.66733E+00_JPRB, 0.64460E+00_JPRB ,&
&  0.62485E+00_JPRB, 0.61316E+00_JPRB, 0.60285E+00_JPRB, 0.59297E+00_JPRB, 0.59407E+00_JPRB, 0.58881E+00_JPRB, 0.57673E+00_JPRB ,&
&  0.55928E+00_JPRB, 0.54666E+00_JPRB, 0.54336E+00_JPRB, 0.54695E+00_JPRB, 0.56013E+00_JPRB, 0.57010E+00_JPRB, 0.57910E+00_JPRB ,&
&  0.58307E+00_JPRB, 0.56239E+00_JPRB, 0.55363E+00_JPRB, 0.55695E+00_JPRB, 0.55957E+00_JPRB, 0.56390E+00_JPRB, 0.54876E+00_JPRB ,&
&  0.55994E+00_JPRB, 0.62805E+00_JPRB, 0.98278E+00_JPRB, 0.14494E+01_JPRB, 0.15576E+01_JPRB, 0.14055E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11019E+01_JPRB, 0.10073E+01_JPRB, 0.91478E+00_JPRB, 0.84467E+00_JPRB, 0.79924E+00_JPRB, 0.77235E+00_JPRB ,&
&  0.76442E+00_JPRB, 0.75987E+00_JPRB, 0.74869E+00_JPRB, 0.74103E+00_JPRB, 0.73272E+00_JPRB, 0.72660E+00_JPRB, 0.72479E+00_JPRB ,&
&  0.72104E+00_JPRB, 0.71441E+00_JPRB, 0.70626E+00_JPRB, 0.70404E+00_JPRB, 0.74704E+00_JPRB, 0.80801E+00_JPRB, 0.91059E+00_JPRB ,&
&  0.96758E+00_JPRB, 0.10014E+01_JPRB, 0.10242E+01_JPRB, 0.10196E+01_JPRB, 0.99746E+00_JPRB, 0.97608E+00_JPRB, 0.95774E+00_JPRB ,&
&  0.92934E+00_JPRB, 0.88758E+00_JPRB, 0.85839E+00_JPRB, 0.83613E+00_JPRB, 0.81430E+00_JPRB, 0.79705E+00_JPRB, 0.78294E+00_JPRB ,&
&  0.77902E+00_JPRB, 0.78598E+00_JPRB, 0.78777E+00_JPRB, 0.78300E+00_JPRB, 0.76571E+00_JPRB, 0.74621E+00_JPRB, 0.72801E+00_JPRB ,&
&  0.70249E+00_JPRB, 0.68674E+00_JPRB, 0.69315E+00_JPRB, 0.71349E+00_JPRB, 0.73676E+00_JPRB, 0.74315E+00_JPRB, 0.75169E+00_JPRB ,&
&  0.76981E+00_JPRB, 0.79226E+00_JPRB, 0.79028E+00_JPRB, 0.78902E+00_JPRB, 0.79410E+00_JPRB, 0.85385E+00_JPRB, 0.84387E+00_JPRB ,&
&  0.81864E+00_JPRB, 0.79584E+00_JPRB, 0.78355E+00_JPRB, 0.77234E+00_JPRB, 0.82583E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.76745E+00_JPRB, 0.75671E+00_JPRB, 0.72482E+00_JPRB, 0.69342E+00_JPRB, 0.67182E+00_JPRB, 0.65625E+00_JPRB, 0.64194E+00_JPRB ,&
&  0.62261E+00_JPRB, 0.59853E+00_JPRB, 0.58408E+00_JPRB, 0.58071E+00_JPRB, 0.58173E+00_JPRB, 0.58030E+00_JPRB, 0.56480E+00_JPRB ,&
&  0.54576E+00_JPRB, 0.53302E+00_JPRB, 0.53668E+00_JPRB, 0.54506E+00_JPRB, 0.55997E+00_JPRB, 0.55942E+00_JPRB, 0.57014E+00_JPRB ,&
&  0.57578E+00_JPRB, 0.57324E+00_JPRB, 0.57077E+00_JPRB, 0.56925E+00_JPRB, 0.56504E+00_JPRB, 0.56141E+00_JPRB, 0.54321E+00_JPRB ,&
&  0.56896E+00_JPRB, 0.69104E+00_JPRB, 0.12837E+01_JPRB, 0.17510E+01_JPRB, 0.17462E+01_JPRB, 0.15000E+01_JPRB, 0.13695E+01_JPRB ,&
&  0.12735E+01_JPRB, 0.11905E+01_JPRB, 0.10682E+01_JPRB, 0.99941E+00_JPRB, 0.93793E+00_JPRB, 0.89280E+00_JPRB, 0.86079E+00_JPRB ,&
&  0.84061E+00_JPRB, 0.81921E+00_JPRB, 0.78944E+00_JPRB, 0.76415E+00_JPRB, 0.74982E+00_JPRB, 0.73992E+00_JPRB, 0.73824E+00_JPRB ,&
&  0.74115E+00_JPRB, 0.73894E+00_JPRB, 0.73687E+00_JPRB, 0.74022E+00_JPRB, 0.78216E+00_JPRB, 0.82090E+00_JPRB, 0.10880E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.14122E+01_JPRB, 0.13689E+01_JPRB, 0.12903E+01_JPRB, 0.12037E+01_JPRB, 0.11391E+01_JPRB, 0.10780E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.97300E+00_JPRB, 0.93239E+00_JPRB, 0.90677E+00_JPRB, 0.88800E+00_JPRB, 0.87211E+00_JPRB, 0.86597E+00_JPRB ,&
&  0.85087E+00_JPRB, 0.83662E+00_JPRB, 0.82026E+00_JPRB, 0.79985E+00_JPRB, 0.77028E+00_JPRB, 0.74180E+00_JPRB, 0.71937E+00_JPRB ,&
&  0.69622E+00_JPRB, 0.69258E+00_JPRB, 0.70825E+00_JPRB, 0.71483E+00_JPRB, 0.75217E+00_JPRB, 0.78414E+00_JPRB, 0.78456E+00_JPRB ,&
&  0.76883E+00_JPRB, 0.77662E+00_JPRB, 0.77655E+00_JPRB, 0.76187E+00_JPRB, 0.76538E+00_JPRB, 0.83040E+00_JPRB, 0.81034E+00_JPRB ,&
&  0.78536E+00_JPRB, 0.77328E+00_JPRB, 0.76509E+00_JPRB, 0.75331E+00_JPRB, 0.76982E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.73641E+00_JPRB, 0.70930E+00_JPRB, 0.68385E+00_JPRB, 0.66484E+00_JPRB, 0.64830E+00_JPRB, 0.63605E+00_JPRB, 0.62214E+00_JPRB ,&
&  0.59992E+00_JPRB, 0.58008E+00_JPRB, 0.57521E+00_JPRB, 0.57196E+00_JPRB, 0.56721E+00_JPRB, 0.56296E+00_JPRB, 0.54559E+00_JPRB ,&
&  0.52896E+00_JPRB, 0.51435E+00_JPRB, 0.52529E+00_JPRB, 0.53837E+00_JPRB, 0.55234E+00_JPRB, 0.56355E+00_JPRB, 0.57901E+00_JPRB ,&
&  0.57536E+00_JPRB, 0.57637E+00_JPRB, 0.57894E+00_JPRB, 0.57945E+00_JPRB, 0.56961E+00_JPRB, 0.57068E+00_JPRB, 0.54531E+00_JPRB ,&
&  0.60234E+00_JPRB, 0.65105E+00_JPRB, 0.13660E+01_JPRB, 0.19747E+01_JPRB, 0.18009E+01_JPRB, 0.15774E+01_JPRB, 0.13966E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.12495E+01_JPRB, 0.11133E+01_JPRB, 0.10375E+01_JPRB, 0.97563E+00_JPRB, 0.92639E+00_JPRB, 0.88995E+00_JPRB ,&
&  0.85569E+00_JPRB, 0.81291E+00_JPRB, 0.78271E+00_JPRB, 0.75908E+00_JPRB, 0.75313E+00_JPRB, 0.75095E+00_JPRB, 0.75523E+00_JPRB ,&
&  0.75901E+00_JPRB, 0.75692E+00_JPRB, 0.75711E+00_JPRB, 0.78423E+00_JPRB, 0.85427E+00_JPRB, 0.10616E+01_JPRB, 0.12941E+01_JPRB ,&
&  0.15157E+01_JPRB, 0.17429E+01_JPRB, 0.16165E+01_JPRB, 0.14547E+01_JPRB, 0.13241E+01_JPRB, 0.12279E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.10524E+01_JPRB, 0.10207E+01_JPRB, 0.10047E+01_JPRB, 0.99194E+00_JPRB, 0.97523E+00_JPRB, 0.94440E+00_JPRB ,&
&  0.90278E+00_JPRB, 0.85929E+00_JPRB, 0.81897E+00_JPRB, 0.78336E+00_JPRB, 0.75035E+00_JPRB, 0.73884E+00_JPRB, 0.73352E+00_JPRB ,&
&  0.71999E+00_JPRB, 0.72417E+00_JPRB, 0.72897E+00_JPRB, 0.72304E+00_JPRB, 0.74755E+00_JPRB, 0.75739E+00_JPRB, 0.75962E+00_JPRB ,&
&  0.76196E+00_JPRB, 0.76258E+00_JPRB, 0.75113E+00_JPRB, 0.76971E+00_JPRB, 0.74975E+00_JPRB, 0.81339E+00_JPRB, 0.78809E+00_JPRB ,&
&  0.75906E+00_JPRB, 0.75939E+00_JPRB, 0.75136E+00_JPRB, 0.74947E+00_JPRB, 0.74978E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.70243E+00_JPRB, 0.66793E+00_JPRB, 0.64390E+00_JPRB, 0.63472E+00_JPRB, 0.62785E+00_JPRB, 0.61520E+00_JPRB, 0.60473E+00_JPRB ,&
&  0.58735E+00_JPRB, 0.56950E+00_JPRB, 0.56027E+00_JPRB, 0.55143E+00_JPRB, 0.55464E+00_JPRB, 0.54954E+00_JPRB, 0.53841E+00_JPRB ,&
&  0.52468E+00_JPRB, 0.52179E+00_JPRB, 0.52433E+00_JPRB, 0.53452E+00_JPRB, 0.55499E+00_JPRB, 0.56520E+00_JPRB, 0.57885E+00_JPRB ,&
&  0.58786E+00_JPRB, 0.59667E+00_JPRB, 0.59646E+00_JPRB, 0.59102E+00_JPRB, 0.58417E+00_JPRB, 0.57486E+00_JPRB, 0.56117E+00_JPRB ,&
&  0.63344E+00_JPRB, 0.13380E+01_JPRB, 0.12073E+01_JPRB, 0.22701E+01_JPRB, 0.19310E+01_JPRB, 0.17006E+01_JPRB, 0.13179E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12469E+01_JPRB, 0.11564E+01_JPRB, 0.10514E+01_JPRB, 0.96144E+00_JPRB, 0.88123E+00_JPRB, 0.82428E+00_JPRB ,&
&  0.79992E+00_JPRB, 0.79509E+00_JPRB, 0.78919E+00_JPRB, 0.77519E+00_JPRB, 0.77524E+00_JPRB, 0.77030E+00_JPRB, 0.77285E+00_JPRB ,&
&  0.77585E+00_JPRB, 0.77829E+00_JPRB, 0.79451E+00_JPRB, 0.88027E+00_JPRB, 0.10386E+01_JPRB, 0.13802E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.17838E+01_JPRB, 0.16047E+01_JPRB, 0.14105E+01_JPRB, 0.12878E+01_JPRB, 0.11956E+01_JPRB, 0.11266E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10340E+01_JPRB, 0.10308E+01_JPRB, 0.10270E+01_JPRB, 0.10114E+01_JPRB, 0.96720E+00_JPRB, 0.90330E+00_JPRB ,&
&  0.84350E+00_JPRB, 0.80752E+00_JPRB, 0.77491E+00_JPRB, 0.75685E+00_JPRB, 0.74941E+00_JPRB, 0.75236E+00_JPRB, 0.76120E+00_JPRB ,&
&  0.75838E+00_JPRB, 0.76570E+00_JPRB, 0.74930E+00_JPRB, 0.72196E+00_JPRB, 0.73507E+00_JPRB, 0.72975E+00_JPRB, 0.71500E+00_JPRB ,&
&  0.73932E+00_JPRB, 0.75061E+00_JPRB, 0.75039E+00_JPRB, 0.76651E+00_JPRB, 0.76054E+00_JPRB, 0.83702E+00_JPRB, 0.78173E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.75619E+00_JPRB, 0.76319E+00_JPRB, 0.74623E+00_JPRB, 0.73186E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.67330E+00_JPRB, 0.65297E+00_JPRB, 0.63940E+00_JPRB, 0.64381E+00_JPRB, 0.62209E+00_JPRB, 0.60714E+00_JPRB, 0.60382E+00_JPRB ,&
&  0.57649E+00_JPRB, 0.56015E+00_JPRB, 0.55084E+00_JPRB, 0.55309E+00_JPRB, 0.55758E+00_JPRB, 0.55135E+00_JPRB, 0.55003E+00_JPRB ,&
&  0.53963E+00_JPRB, 0.53121E+00_JPRB, 0.53415E+00_JPRB, 0.55455E+00_JPRB, 0.58161E+00_JPRB, 0.58900E+00_JPRB, 0.60348E+00_JPRB ,&
&  0.61825E+00_JPRB, 0.61266E+00_JPRB, 0.60260E+00_JPRB, 0.60286E+00_JPRB, 0.60881E+00_JPRB, 0.60664E+00_JPRB, 0.62152E+00_JPRB ,&
&  0.66096E+00_JPRB, 0.12416E+01_JPRB, 0.13701E+01_JPRB, 0.24038E+01_JPRB, 0.20696E+01_JPRB, 0.17157E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.12454E+01_JPRB, 0.11144E+01_JPRB, 0.98964E+00_JPRB, 0.86825E+00_JPRB, 0.78108E+00_JPRB, 0.75866E+00_JPRB ,&
&  0.78023E+00_JPRB, 0.80465E+00_JPRB, 0.82433E+00_JPRB, 0.81140E+00_JPRB, 0.79357E+00_JPRB, 0.78743E+00_JPRB, 0.78826E+00_JPRB ,&
&  0.79420E+00_JPRB, 0.81368E+00_JPRB, 0.88880E+00_JPRB, 0.10402E+01_JPRB, 0.14569E+01_JPRB, 0.17026E+01_JPRB, 0.15871E+01_JPRB ,&
&  0.14698E+01_JPRB, 0.15502E+01_JPRB, 0.14052E+01_JPRB, 0.12593E+01_JPRB, 0.11346E+01_JPRB, 0.11133E+01_JPRB, 0.10705E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.10326E+01_JPRB, 0.10055E+01_JPRB, 0.96276E+00_JPRB, 0.91756E+00_JPRB, 0.86741E+00_JPRB, 0.81548E+00_JPRB ,&
&  0.78331E+00_JPRB, 0.76760E+00_JPRB, 0.75663E+00_JPRB, 0.76054E+00_JPRB, 0.77720E+00_JPRB, 0.79352E+00_JPRB, 0.80755E+00_JPRB ,&
&  0.80424E+00_JPRB, 0.80669E+00_JPRB, 0.76920E+00_JPRB, 0.74566E+00_JPRB, 0.75616E+00_JPRB, 0.73813E+00_JPRB, 0.72561E+00_JPRB ,&
&  0.75283E+00_JPRB, 0.77950E+00_JPRB, 0.79400E+00_JPRB, 0.78996E+00_JPRB, 0.79931E+00_JPRB, 0.85029E+00_JPRB, 0.79994E+00_JPRB ,&
&  0.76868E+00_JPRB, 0.79650E+00_JPRB, 0.80705E+00_JPRB, 0.74371E+00_JPRB, 0.70842E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.67795E+00_JPRB, 0.67507E+00_JPRB, 0.66040E+00_JPRB, 0.65497E+00_JPRB, 0.62220E+00_JPRB, 0.59294E+00_JPRB, 0.56874E+00_JPRB ,&
&  0.55746E+00_JPRB, 0.55470E+00_JPRB, 0.56641E+00_JPRB, 0.57014E+00_JPRB, 0.57149E+00_JPRB, 0.56376E+00_JPRB, 0.55777E+00_JPRB ,&
&  0.54437E+00_JPRB, 0.54689E+00_JPRB, 0.55970E+00_JPRB, 0.58105E+00_JPRB, 0.60507E+00_JPRB, 0.61496E+00_JPRB, 0.62583E+00_JPRB ,&
&  0.62661E+00_JPRB, 0.62307E+00_JPRB, 0.62586E+00_JPRB, 0.63650E+00_JPRB, 0.65367E+00_JPRB, 0.67133E+00_JPRB, 0.69232E+00_JPRB ,&
&  0.72241E+00_JPRB, 0.11996E+01_JPRB, 0.17927E+01_JPRB, 0.21129E+01_JPRB, 0.18248E+01_JPRB, 0.14786E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.91728E+00_JPRB, 0.99423E+00_JPRB, 0.90979E+00_JPRB, 0.81322E+00_JPRB, 0.77477E+00_JPRB, 0.78189E+00_JPRB ,&
&  0.81871E+00_JPRB, 0.83198E+00_JPRB, 0.84072E+00_JPRB, 0.84841E+00_JPRB, 0.83889E+00_JPRB, 0.83643E+00_JPRB, 0.84419E+00_JPRB ,&
&  0.88704E+00_JPRB, 0.98538E+00_JPRB, 0.11610E+01_JPRB, 0.14634E+01_JPRB, 0.18575E+01_JPRB, 0.17115E+01_JPRB, 0.14317E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.12663E+01_JPRB, 0.11853E+01_JPRB, 0.11234E+01_JPRB, 0.97198E+00_JPRB, 0.10699E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.99293E+00_JPRB, 0.96361E+00_JPRB, 0.92146E+00_JPRB, 0.88408E+00_JPRB, 0.84970E+00_JPRB, 0.81307E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.77999E+00_JPRB, 0.78154E+00_JPRB, 0.78574E+00_JPRB, 0.79440E+00_JPRB, 0.80890E+00_JPRB, 0.82513E+00_JPRB, 0.83312E+00_JPRB ,&
&  0.83083E+00_JPRB, 0.83665E+00_JPRB, 0.82040E+00_JPRB, 0.82540E+00_JPRB, 0.82496E+00_JPRB, 0.80261E+00_JPRB, 0.78071E+00_JPRB ,&
&  0.79492E+00_JPRB, 0.83133E+00_JPRB, 0.84421E+00_JPRB, 0.84541E+00_JPRB, 0.85118E+00_JPRB, 0.87257E+00_JPRB, 0.81726E+00_JPRB ,&
&  0.80257E+00_JPRB, 0.86347E+00_JPRB, 0.82508E+00_JPRB, 0.74273E+00_JPRB, 0.70309E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.68676E+00_JPRB, 0.68785E+00_JPRB, 0.67767E+00_JPRB, 0.64094E+00_JPRB, 0.60792E+00_JPRB, 0.57365E+00_JPRB, 0.55074E+00_JPRB ,&
&  0.54921E+00_JPRB, 0.56015E+00_JPRB, 0.59165E+00_JPRB, 0.60986E+00_JPRB, 0.59993E+00_JPRB, 0.58240E+00_JPRB, 0.57630E+00_JPRB ,&
&  0.58037E+00_JPRB, 0.59339E+00_JPRB, 0.60123E+00_JPRB, 0.61717E+00_JPRB, 0.63659E+00_JPRB, 0.65499E+00_JPRB, 0.67481E+00_JPRB ,&
&  0.66334E+00_JPRB, 0.66827E+00_JPRB, 0.68265E+00_JPRB, 0.69213E+00_JPRB, 0.70826E+00_JPRB, 0.74347E+00_JPRB, 0.76976E+00_JPRB ,&
&  0.88809E+00_JPRB, 0.13499E+01_JPRB, 0.84509E+00_JPRB, 0.11949E+01_JPRB, 0.13793E+01_JPRB, 0.13615E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.87776E+00_JPRB, 0.88133E+00_JPRB, 0.91826E+00_JPRB, 0.85944E+00_JPRB, 0.86249E+00_JPRB, 0.87199E+00_JPRB ,&
&  0.86579E+00_JPRB, 0.88150E+00_JPRB, 0.88940E+00_JPRB, 0.90101E+00_JPRB, 0.90654E+00_JPRB, 0.92697E+00_JPRB, 0.98981E+00_JPRB ,&
&  0.11162E+01_JPRB, 0.12730E+01_JPRB, 0.15394E+01_JPRB, 0.17212E+01_JPRB, 0.20383E+01_JPRB, 0.16391E+01_JPRB, 0.12821E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.10608E+01_JPRB, 0.10923E+01_JPRB, 0.10770E+01_JPRB, 0.98400E+00_JPRB, 0.92219E+00_JPRB, 0.96173E+00_JPRB ,&
&  0.92010E+00_JPRB, 0.88558E+00_JPRB, 0.87224E+00_JPRB, 0.85480E+00_JPRB, 0.83066E+00_JPRB, 0.81319E+00_JPRB, 0.80343E+00_JPRB ,&
&  0.79707E+00_JPRB, 0.80132E+00_JPRB, 0.81220E+00_JPRB, 0.81864E+00_JPRB, 0.83032E+00_JPRB, 0.83893E+00_JPRB, 0.85205E+00_JPRB ,&
&  0.86665E+00_JPRB, 0.88993E+00_JPRB, 0.91757E+00_JPRB, 0.92524E+00_JPRB, 0.91781E+00_JPRB, 0.89411E+00_JPRB, 0.87869E+00_JPRB ,&
&  0.87883E+00_JPRB, 0.89970E+00_JPRB, 0.88029E+00_JPRB, 0.87180E+00_JPRB, 0.91993E+00_JPRB, 0.89165E+00_JPRB, 0.84898E+00_JPRB ,&
&  0.89583E+00_JPRB, 0.10486E+01_JPRB, 0.10990E+01_JPRB, 0.74167E+00_JPRB, 0.70173E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.65271E+00_JPRB, 0.63254E+00_JPRB, 0.60429E+00_JPRB, 0.56867E+00_JPRB, 0.56431E+00_JPRB, 0.56094E+00_JPRB, 0.57284E+00_JPRB ,&
&  0.60899E+00_JPRB, 0.65804E+00_JPRB, 0.67210E+00_JPRB, 0.66465E+00_JPRB, 0.64785E+00_JPRB, 0.64300E+00_JPRB, 0.64042E+00_JPRB ,&
&  0.64383E+00_JPRB, 0.64395E+00_JPRB, 0.66019E+00_JPRB, 0.68164E+00_JPRB, 0.70573E+00_JPRB, 0.72388E+00_JPRB, 0.72655E+00_JPRB ,&
&  0.73202E+00_JPRB, 0.73704E+00_JPRB, 0.75392E+00_JPRB, 0.77435E+00_JPRB, 0.81796E+00_JPRB, 0.90322E+00_JPRB, 0.10339E+01_JPRB ,&
&  0.13518E+01_JPRB, 0.10543E+01_JPRB, 0.58663E+00_JPRB, 0.11322E+01_JPRB, 0.13078E+01_JPRB, 0.13859E+01_JPRB, 0.11780E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.96551E+00_JPRB, 0.92960E+00_JPRB, 0.10127E+01_JPRB, 0.10017E+01_JPRB, 0.10096E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.98181E+00_JPRB, 0.98313E+00_JPRB, 0.98173E+00_JPRB, 0.99400E+00_JPRB, 0.10213E+01_JPRB, 0.10851E+01_JPRB, 0.12039E+01_JPRB ,&
&  0.13485E+01_JPRB, 0.15325E+01_JPRB, 0.17840E+01_JPRB, 0.18536E+01_JPRB, 0.19887E+01_JPRB, 0.15298E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.97634E+00_JPRB, 0.97993E+00_JPRB, 0.10507E+01_JPRB, 0.10611E+01_JPRB, 0.98769E+00_JPRB, 0.87157E+00_JPRB, 0.91925E+00_JPRB ,&
&  0.89967E+00_JPRB, 0.87582E+00_JPRB, 0.86627E+00_JPRB, 0.84493E+00_JPRB, 0.82650E+00_JPRB, 0.81837E+00_JPRB, 0.81617E+00_JPRB ,&
&  0.81469E+00_JPRB, 0.82325E+00_JPRB, 0.82814E+00_JPRB, 0.83062E+00_JPRB, 0.83708E+00_JPRB, 0.84083E+00_JPRB, 0.86692E+00_JPRB ,&
&  0.89185E+00_JPRB, 0.91241E+00_JPRB, 0.94085E+00_JPRB, 0.96536E+00_JPRB, 0.95343E+00_JPRB, 0.91614E+00_JPRB, 0.91039E+00_JPRB ,&
&  0.89686E+00_JPRB, 0.90226E+00_JPRB, 0.89047E+00_JPRB, 0.91697E+00_JPRB, 0.89086E+00_JPRB, 0.84381E+00_JPRB, 0.79051E+00_JPRB ,&
&  0.79318E+00_JPRB, 0.78995E+00_JPRB, 0.74896E+00_JPRB, 0.68833E+00_JPRB, 0.66176E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.61923E+00_JPRB, 0.59823E+00_JPRB, 0.58474E+00_JPRB, 0.59025E+00_JPRB, 0.59890E+00_JPRB, 0.61787E+00_JPRB, 0.66554E+00_JPRB ,&
&  0.70311E+00_JPRB, 0.72014E+00_JPRB, 0.74103E+00_JPRB, 0.73983E+00_JPRB, 0.74673E+00_JPRB, 0.72698E+00_JPRB, 0.71934E+00_JPRB ,&
&  0.70448E+00_JPRB, 0.71404E+00_JPRB, 0.73208E+00_JPRB, 0.74872E+00_JPRB, 0.76163E+00_JPRB, 0.77273E+00_JPRB, 0.78718E+00_JPRB ,&
&  0.80815E+00_JPRB, 0.83805E+00_JPRB, 0.88434E+00_JPRB, 0.95025E+00_JPRB, 0.10588E+01_JPRB, 0.11920E+01_JPRB, 0.12628E+01_JPRB ,&
&  0.92033E+00_JPRB, 0.45852E+00_JPRB, 0.88338E+00_JPRB, 0.13109E+01_JPRB, 0.13738E+01_JPRB, 0.13989E+01_JPRB, 0.13140E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.10836E+01_JPRB, 0.10286E+01_JPRB, 0.11438E+01_JPRB, 0.11281E+01_JPRB, 0.11224E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10970E+01_JPRB, 0.10996E+01_JPRB, 0.11251E+01_JPRB, 0.11764E+01_JPRB, 0.12616E+01_JPRB, 0.13750E+01_JPRB ,&
&  0.15304E+01_JPRB, 0.17493E+01_JPRB, 0.19394E+01_JPRB, 0.17839E+01_JPRB, 0.18207E+01_JPRB, 0.15986E+01_JPRB, 0.13937E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.90857E+00_JPRB, 0.94329E+00_JPRB, 0.10627E+01_JPRB, 0.10010E+01_JPRB, 0.86030E+00_JPRB, 0.91039E+00_JPRB ,&
&  0.89573E+00_JPRB, 0.87140E+00_JPRB, 0.86053E+00_JPRB, 0.83713E+00_JPRB, 0.82834E+00_JPRB, 0.82638E+00_JPRB, 0.83157E+00_JPRB ,&
&  0.83140E+00_JPRB, 0.83525E+00_JPRB, 0.83732E+00_JPRB, 0.83307E+00_JPRB, 0.81845E+00_JPRB, 0.82235E+00_JPRB, 0.84557E+00_JPRB ,&
&  0.86526E+00_JPRB, 0.87104E+00_JPRB, 0.89539E+00_JPRB, 0.91973E+00_JPRB, 0.94646E+00_JPRB, 0.92610E+00_JPRB, 0.91482E+00_JPRB ,&
&  0.89696E+00_JPRB, 0.89453E+00_JPRB, 0.90319E+00_JPRB, 0.90664E+00_JPRB, 0.90592E+00_JPRB, 0.85361E+00_JPRB, 0.77656E+00_JPRB ,&
&  0.75047E+00_JPRB, 0.72057E+00_JPRB, 0.69610E+00_JPRB, 0.67358E+00_JPRB, 0.64858E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.67732E+00_JPRB, 0.68486E+00_JPRB, 0.67782E+00_JPRB, 0.69869E+00_JPRB, 0.71754E+00_JPRB, 0.74650E+00_JPRB, 0.77266E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.81775E+00_JPRB, 0.84563E+00_JPRB, 0.82090E+00_JPRB, 0.82632E+00_JPRB, 0.81080E+00_JPRB, 0.78464E+00_JPRB ,&
&  0.77836E+00_JPRB, 0.79124E+00_JPRB, 0.80503E+00_JPRB, 0.81750E+00_JPRB, 0.82931E+00_JPRB, 0.85473E+00_JPRB, 0.88947E+00_JPRB ,&
&  0.93419E+00_JPRB, 0.99119E+00_JPRB, 0.10687E+01_JPRB, 0.11329E+01_JPRB, 0.11998E+01_JPRB, 0.12160E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.56764E+00_JPRB, 0.79085E+00_JPRB, 0.12599E+01_JPRB, 0.14338E+01_JPRB, 0.15093E+01_JPRB, 0.14979E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.12961E+01_JPRB, 0.11321E+01_JPRB, 0.11087E+01_JPRB, 0.12466E+01_JPRB, 0.12089E+01_JPRB, 0.12071E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12074E+01_JPRB, 0.12371E+01_JPRB, 0.12681E+01_JPRB, 0.13314E+01_JPRB, 0.14166E+01_JPRB, 0.15380E+01_JPRB ,&
&  0.16945E+01_JPRB, 0.18584E+01_JPRB, 0.20080E+01_JPRB, 0.17778E+01_JPRB, 0.17359E+01_JPRB, 0.16516E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.87826E+00_JPRB, 0.94086E+00_JPRB, 0.10545E+01_JPRB, 0.99151E+00_JPRB, 0.93593E+00_JPRB, 0.90973E+00_JPRB ,&
&  0.90572E+00_JPRB, 0.86618E+00_JPRB, 0.85424E+00_JPRB, 0.83262E+00_JPRB, 0.81964E+00_JPRB, 0.82573E+00_JPRB, 0.84209E+00_JPRB ,&
&  0.83700E+00_JPRB, 0.82253E+00_JPRB, 0.81963E+00_JPRB, 0.82392E+00_JPRB, 0.80756E+00_JPRB, 0.79224E+00_JPRB, 0.78987E+00_JPRB ,&
&  0.79777E+00_JPRB, 0.78593E+00_JPRB, 0.79616E+00_JPRB, 0.82857E+00_JPRB, 0.87043E+00_JPRB, 0.89597E+00_JPRB, 0.90840E+00_JPRB ,&
&  0.89856E+00_JPRB, 0.92098E+00_JPRB, 0.10583E+01_JPRB, 0.11135E+01_JPRB, 0.11178E+01_JPRB, 0.94471E+00_JPRB, 0.77880E+00_JPRB ,&
&  0.75796E+00_JPRB, 0.72846E+00_JPRB, 0.70103E+00_JPRB, 0.69068E+00_JPRB, 0.67954E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.74798E+00_JPRB, 0.76050E+00_JPRB, 0.78049E+00_JPRB, 0.81311E+00_JPRB, 0.84909E+00_JPRB, 0.87372E+00_JPRB, 0.89790E+00_JPRB ,&
&  0.90664E+00_JPRB, 0.91833E+00_JPRB, 0.92754E+00_JPRB, 0.88131E+00_JPRB, 0.88911E+00_JPRB, 0.86926E+00_JPRB, 0.86030E+00_JPRB ,&
&  0.85337E+00_JPRB, 0.86878E+00_JPRB, 0.88775E+00_JPRB, 0.90349E+00_JPRB, 0.92837E+00_JPRB, 0.96153E+00_JPRB, 0.10116E+01_JPRB ,&
&  0.10745E+01_JPRB, 0.11302E+01_JPRB, 0.11997E+01_JPRB, 0.12374E+01_JPRB, 0.12748E+01_JPRB, 0.12897E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.79558E+00_JPRB, 0.13436E+01_JPRB, 0.14799E+01_JPRB, 0.15508E+01_JPRB, 0.15743E+01_JPRB, 0.14528E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.11926E+01_JPRB, 0.11842E+01_JPRB, 0.12466E+01_JPRB, 0.13202E+01_JPRB, 0.13063E+01_JPRB, 0.13054E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.13393E+01_JPRB, 0.13779E+01_JPRB, 0.14213E+01_JPRB, 0.14822E+01_JPRB, 0.15537E+01_JPRB, 0.16589E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.18690E+01_JPRB, 0.19569E+01_JPRB, 0.17634E+01_JPRB, 0.15664E+01_JPRB, 0.15395E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.94173E+00_JPRB, 0.80182E+00_JPRB, 0.88526E+00_JPRB, 0.10162E+01_JPRB, 0.96940E+00_JPRB, 0.88924E+00_JPRB, 0.83088E+00_JPRB ,&
&  0.81371E+00_JPRB, 0.77921E+00_JPRB, 0.79244E+00_JPRB, 0.76147E+00_JPRB, 0.78427E+00_JPRB, 0.78001E+00_JPRB, 0.78925E+00_JPRB ,&
&  0.79691E+00_JPRB, 0.77069E+00_JPRB, 0.75876E+00_JPRB, 0.75441E+00_JPRB, 0.74600E+00_JPRB, 0.73629E+00_JPRB, 0.72959E+00_JPRB ,&
&  0.74016E+00_JPRB, 0.70514E+00_JPRB, 0.68555E+00_JPRB, 0.70218E+00_JPRB, 0.73663E+00_JPRB, 0.78037E+00_JPRB, 0.87520E+00_JPRB ,&
&  0.10333E+01_JPRB, 0.12027E+01_JPRB, 0.14944E+01_JPRB, 0.15878E+01_JPRB, 0.15406E+01_JPRB, 0.11571E+01_JPRB, 0.84533E+00_JPRB ,&
&  0.83735E+00_JPRB, 0.79208E+00_JPRB, 0.75635E+00_JPRB, 0.73567E+00_JPRB, 0.73735E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.89289E+00_JPRB, 0.91331E+00_JPRB, 0.93611E+00_JPRB, 0.95516E+00_JPRB, 0.95804E+00_JPRB, 0.95800E+00_JPRB, 0.95556E+00_JPRB ,&
&  0.97920E+00_JPRB, 0.95575E+00_JPRB, 0.97880E+00_JPRB, 0.95347E+00_JPRB, 0.97256E+00_JPRB, 0.95269E+00_JPRB, 0.95468E+00_JPRB ,&
&  0.96373E+00_JPRB, 0.98844E+00_JPRB, 0.10183E+01_JPRB, 0.10278E+01_JPRB, 0.10231E+01_JPRB, 0.10497E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.11412E+01_JPRB, 0.11652E+01_JPRB, 0.12032E+01_JPRB, 0.12174E+01_JPRB, 0.11895E+01_JPRB, 0.11601E+01_JPRB, 0.83441E+00_JPRB ,&
&  0.12390E+01_JPRB, 0.14789E+01_JPRB, 0.16045E+01_JPRB, 0.15545E+01_JPRB, 0.14272E+01_JPRB, 0.13289E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.13085E+01_JPRB, 0.12760E+01_JPRB, 0.12378E+01_JPRB, 0.12521E+01_JPRB, 0.13993E+01_JPRB, 0.13649E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.14286E+01_JPRB, 0.14636E+01_JPRB, 0.15070E+01_JPRB, 0.15445E+01_JPRB, 0.15839E+01_JPRB, 0.16359E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.16720E+01_JPRB, 0.16974E+01_JPRB, 0.15186E+01_JPRB, 0.14051E+01_JPRB, 0.13681E+01_JPRB, 0.11248E+01_JPRB ,&
&  0.80990E+00_JPRB, 0.75599E+00_JPRB, 0.85577E+00_JPRB, 0.95233E+00_JPRB, 0.92087E+00_JPRB, 0.82685E+00_JPRB, 0.77246E+00_JPRB ,&
&  0.74171E+00_JPRB, 0.72083E+00_JPRB, 0.69740E+00_JPRB, 0.70704E+00_JPRB, 0.70708E+00_JPRB, 0.72054E+00_JPRB, 0.73289E+00_JPRB ,&
&  0.74658E+00_JPRB, 0.72813E+00_JPRB, 0.70652E+00_JPRB, 0.69636E+00_JPRB, 0.70674E+00_JPRB, 0.77194E+00_JPRB, 0.86723E+00_JPRB ,&
&  0.91152E+00_JPRB, 0.57782E+00_JPRB, 0.62010E+00_JPRB, 0.59984E+00_JPRB, 0.61436E+00_JPRB, 0.68467E+00_JPRB, 0.86836E+00_JPRB ,&
&  0.13299E+01_JPRB, 0.20682E+01_JPRB, 0.23382E+01_JPRB, 0.20020E+01_JPRB, 0.24686E+01_JPRB, 0.17598E+01_JPRB, 0.96903E+00_JPRB ,&
&  0.94645E+00_JPRB, 0.91444E+00_JPRB, 0.87522E+00_JPRB, 0.86268E+00_JPRB, 0.86974E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.94876E+00_JPRB, 0.96159E+00_JPRB, 0.95829E+00_JPRB, 0.97098E+00_JPRB, 0.94886E+00_JPRB, 0.96495E+00_JPRB, 0.96436E+00_JPRB ,&
&  0.98142E+00_JPRB, 0.98960E+00_JPRB, 0.10062E+01_JPRB, 0.10198E+01_JPRB, 0.10440E+01_JPRB, 0.10649E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.11019E+01_JPRB, 0.11239E+01_JPRB, 0.11420E+01_JPRB, 0.11526E+01_JPRB, 0.11289E+01_JPRB, 0.11279E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.11371E+01_JPRB, 0.11262E+01_JPRB, 0.11350E+01_JPRB, 0.11348E+01_JPRB, 0.11124E+01_JPRB, 0.11139E+01_JPRB, 0.81852E+00_JPRB ,&
&  0.12791E+01_JPRB, 0.14633E+01_JPRB, 0.14980E+01_JPRB, 0.13973E+01_JPRB, 0.12972E+01_JPRB, 0.12686E+01_JPRB, 0.12684E+01_JPRB ,&
&  0.13009E+01_JPRB, 0.13222E+01_JPRB, 0.12754E+01_JPRB, 0.13213E+01_JPRB, 0.13430E+01_JPRB, 0.13089E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.13356E+01_JPRB, 0.13482E+01_JPRB, 0.13638E+01_JPRB, 0.13668E+01_JPRB, 0.13542E+01_JPRB, 0.13722E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.13538E+01_JPRB, 0.13612E+01_JPRB, 0.12769E+01_JPRB, 0.11993E+01_JPRB, 0.10318E+01_JPRB, 0.88636E+00_JPRB ,&
&  0.69458E+00_JPRB, 0.68154E+00_JPRB, 0.77004E+00_JPRB, 0.92370E+00_JPRB, 0.89021E+00_JPRB, 0.82726E+00_JPRB, 0.76730E+00_JPRB ,&
&  0.70211E+00_JPRB, 0.68522E+00_JPRB, 0.66035E+00_JPRB, 0.65200E+00_JPRB, 0.65039E+00_JPRB, 0.66626E+00_JPRB, 0.67522E+00_JPRB ,&
&  0.68726E+00_JPRB, 0.69685E+00_JPRB, 0.68214E+00_JPRB, 0.66471E+00_JPRB, 0.65691E+00_JPRB, 0.68721E+00_JPRB, 0.63023E+00_JPRB ,&
&  0.71762E+00_JPRB, 0.59698E+00_JPRB, 0.58135E+00_JPRB, 0.55978E+00_JPRB, 0.51822E+00_JPRB, 0.55647E+00_JPRB, 0.73899E+00_JPRB ,&
&  0.12359E+01_JPRB, 0.16081E+01_JPRB, 0.23163E+01_JPRB, 0.39403E+01_JPRB, 0.27689E+01_JPRB, 0.20365E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.96266E+00_JPRB, 0.93665E+00_JPRB, 0.92191E+00_JPRB, 0.91078E+00_JPRB, 0.92790E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.91200E+00_JPRB, 0.92185E+00_JPRB, 0.93103E+00_JPRB, 0.94074E+00_JPRB, 0.94799E+00_JPRB, 0.96483E+00_JPRB, 0.97062E+00_JPRB ,&
&  0.98186E+00_JPRB, 0.98763E+00_JPRB, 0.10028E+01_JPRB, 0.10221E+01_JPRB, 0.10365E+01_JPRB, 0.10588E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.11163E+01_JPRB, 0.11352E+01_JPRB, 0.11588E+01_JPRB, 0.11648E+01_JPRB, 0.11503E+01_JPRB, 0.11502E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11217E+01_JPRB, 0.11307E+01_JPRB, 0.11434E+01_JPRB, 0.11343E+01_JPRB, 0.11256E+01_JPRB, 0.85992E+00_JPRB ,&
&  0.12340E+01_JPRB, 0.13875E+01_JPRB, 0.13879E+01_JPRB, 0.12627E+01_JPRB, 0.11851E+01_JPRB, 0.11808E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.13016E+01_JPRB, 0.12611E+01_JPRB, 0.11871E+01_JPRB, 0.11456E+01_JPRB, 0.11394E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.11409E+01_JPRB, 0.11329E+01_JPRB, 0.11082E+01_JPRB, 0.10800E+01_JPRB, 0.10552E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.97962E+00_JPRB, 0.95563E+00_JPRB, 0.86085E+00_JPRB, 0.89236E+00_JPRB, 0.84328E+00_JPRB, 0.73083E+00_JPRB, 0.61317E+00_JPRB ,&
&  0.52196E+00_JPRB, 0.56110E+00_JPRB, 0.69644E+00_JPRB, 0.90952E+00_JPRB, 0.88086E+00_JPRB, 0.81966E+00_JPRB, 0.78307E+00_JPRB ,&
&  0.73052E+00_JPRB, 0.68033E+00_JPRB, 0.64350E+00_JPRB, 0.62294E+00_JPRB, 0.61416E+00_JPRB, 0.62723E+00_JPRB, 0.63962E+00_JPRB ,&
&  0.64360E+00_JPRB, 0.64749E+00_JPRB, 0.63516E+00_JPRB, 0.62774E+00_JPRB, 0.61388E+00_JPRB, 0.56218E+00_JPRB, 0.65745E+00_JPRB ,&
&  0.61717E+00_JPRB, 0.49046E+00_JPRB, 0.59374E+00_JPRB, 0.38885E+00_JPRB, 0.49430E+00_JPRB, 0.52636E+00_JPRB, 0.63424E+00_JPRB ,&
&  0.76591E+00_JPRB, 0.10882E+01_JPRB, 0.15083E+01_JPRB, 0.13100E+01_JPRB, 0.11720E+01_JPRB, 0.10580E+01_JPRB, 0.96381E+00_JPRB ,&
&  0.92417E+00_JPRB, 0.90879E+00_JPRB, 0.90291E+00_JPRB, 0.89639E+00_JPRB, 0.90170E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.88837E+00_JPRB, 0.89685E+00_JPRB, 0.90476E+00_JPRB, 0.91749E+00_JPRB, 0.92948E+00_JPRB, 0.94533E+00_JPRB, 0.95645E+00_JPRB ,&
&  0.96741E+00_JPRB, 0.97476E+00_JPRB, 0.97457E+00_JPRB, 0.98766E+00_JPRB, 0.99840E+00_JPRB, 0.10196E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10965E+01_JPRB, 0.11266E+01_JPRB, 0.11413E+01_JPRB, 0.11524E+01_JPRB, 0.11678E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11937E+01_JPRB, 0.12090E+01_JPRB, 0.12360E+01_JPRB, 0.12672E+01_JPRB, 0.12995E+01_JPRB, 0.11015E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.12956E+01_JPRB, 0.12321E+01_JPRB, 0.11110E+01_JPRB, 0.10659E+01_JPRB, 0.10364E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.10236E+01_JPRB, 0.99392E+00_JPRB, 0.93989E+00_JPRB, 0.91416E+00_JPRB, 0.93149E+00_JPRB, 0.95795E+00_JPRB ,&
&  0.96727E+00_JPRB, 0.97357E+00_JPRB, 0.96211E+00_JPRB, 0.93875E+00_JPRB, 0.92046E+00_JPRB, 0.90122E+00_JPRB, 0.86328E+00_JPRB ,&
&  0.82013E+00_JPRB, 0.80692E+00_JPRB, 0.69282E+00_JPRB, 0.68802E+00_JPRB, 0.69170E+00_JPRB, 0.64482E+00_JPRB, 0.55894E+00_JPRB ,&
&  0.47994E+00_JPRB, 0.46888E+00_JPRB, 0.53792E+00_JPRB, 0.84242E+00_JPRB, 0.90663E+00_JPRB, 0.85944E+00_JPRB, 0.81146E+00_JPRB ,&
&  0.76329E+00_JPRB, 0.72535E+00_JPRB, 0.66157E+00_JPRB, 0.61897E+00_JPRB, 0.59329E+00_JPRB, 0.60079E+00_JPRB, 0.60526E+00_JPRB ,&
&  0.60146E+00_JPRB, 0.59035E+00_JPRB, 0.57650E+00_JPRB, 0.56076E+00_JPRB, 0.57142E+00_JPRB, 0.52244E+00_JPRB, 0.57038E+00_JPRB ,&
&  0.55986E+00_JPRB, 0.40603E+00_JPRB, 0.43933E+00_JPRB, 0.54737E+00_JPRB, 0.53073E+00_JPRB, 0.59392E+00_JPRB, 0.71002E+00_JPRB ,&
&  0.91346E+00_JPRB, 0.10300E+01_JPRB, 0.10062E+01_JPRB, 0.96809E+00_JPRB, 0.90896E+00_JPRB, 0.90543E+00_JPRB, 0.88311E+00_JPRB ,&
&  0.86551E+00_JPRB, 0.86275E+00_JPRB, 0.86489E+00_JPRB, 0.86869E+00_JPRB, 0.87746E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.84504E+00_JPRB, 0.85995E+00_JPRB, 0.87187E+00_JPRB, 0.87992E+00_JPRB, 0.88377E+00_JPRB, 0.88754E+00_JPRB, 0.89407E+00_JPRB ,&
&  0.89676E+00_JPRB, 0.89456E+00_JPRB, 0.89278E+00_JPRB, 0.90742E+00_JPRB, 0.92574E+00_JPRB, 0.93342E+00_JPRB, 0.95295E+00_JPRB ,&
&  0.94691E+00_JPRB, 0.95725E+00_JPRB, 0.97370E+00_JPRB, 0.98457E+00_JPRB, 0.10059E+01_JPRB, 0.10066E+01_JPRB, 0.99206E+00_JPRB ,&
&  0.98702E+00_JPRB, 0.97962E+00_JPRB, 0.98028E+00_JPRB, 0.98178E+00_JPRB, 0.99283E+00_JPRB, 0.10075E+01_JPRB, 0.11701E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.12364E+01_JPRB, 0.11086E+01_JPRB, 0.97126E+00_JPRB, 0.95926E+00_JPRB, 0.89047E+00_JPRB, 0.85777E+00_JPRB ,&
&  0.81390E+00_JPRB, 0.76470E+00_JPRB, 0.73497E+00_JPRB, 0.71342E+00_JPRB, 0.73033E+00_JPRB, 0.76243E+00_JPRB, 0.78718E+00_JPRB ,&
&  0.82710E+00_JPRB, 0.84091E+00_JPRB, 0.84670E+00_JPRB, 0.84520E+00_JPRB, 0.83589E+00_JPRB, 0.86309E+00_JPRB, 0.83348E+00_JPRB ,&
&  0.84603E+00_JPRB, 0.89742E+00_JPRB, 0.82829E+00_JPRB, 0.85207E+00_JPRB, 0.84436E+00_JPRB, 0.80536E+00_JPRB, 0.72710E+00_JPRB ,&
&  0.62452E+00_JPRB, 0.58855E+00_JPRB, 0.66650E+00_JPRB, 0.79469E+00_JPRB, 0.87651E+00_JPRB, 0.91247E+00_JPRB, 0.84760E+00_JPRB ,&
&  0.79039E+00_JPRB, 0.75541E+00_JPRB, 0.70290E+00_JPRB, 0.64279E+00_JPRB, 0.61708E+00_JPRB, 0.60838E+00_JPRB, 0.59986E+00_JPRB ,&
&  0.58151E+00_JPRB, 0.56550E+00_JPRB, 0.54760E+00_JPRB, 0.53864E+00_JPRB, 0.48698E+00_JPRB, 0.52220E+00_JPRB, 0.55033E+00_JPRB ,&
&  0.51391E+00_JPRB, 0.43400E+00_JPRB, 0.40285E+00_JPRB, 0.55581E+00_JPRB, 0.57865E+00_JPRB, 0.63567E+00_JPRB, 0.70885E+00_JPRB ,&
&  0.80136E+00_JPRB, 0.87023E+00_JPRB, 0.88945E+00_JPRB, 0.90906E+00_JPRB, 0.80101E+00_JPRB, 0.77873E+00_JPRB, 0.77040E+00_JPRB ,&
&  0.77471E+00_JPRB, 0.78572E+00_JPRB, 0.79853E+00_JPRB, 0.81182E+00_JPRB, 0.82876E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.77371E+00_JPRB, 0.79218E+00_JPRB, 0.81373E+00_JPRB, 0.81598E+00_JPRB, 0.82244E+00_JPRB, 0.81888E+00_JPRB, 0.82047E+00_JPRB ,&
&  0.81287E+00_JPRB, 0.81244E+00_JPRB, 0.81510E+00_JPRB, 0.82044E+00_JPRB, 0.82857E+00_JPRB, 0.83905E+00_JPRB, 0.86714E+00_JPRB ,&
&  0.86709E+00_JPRB, 0.87219E+00_JPRB, 0.87941E+00_JPRB, 0.89367E+00_JPRB, 0.91035E+00_JPRB, 0.91338E+00_JPRB, 0.89362E+00_JPRB ,&
&  0.87969E+00_JPRB, 0.88059E+00_JPRB, 0.89410E+00_JPRB, 0.91817E+00_JPRB, 0.10664E+01_JPRB, 0.13613E+01_JPRB, 0.18151E+01_JPRB ,&
&  0.15543E+01_JPRB, 0.11579E+01_JPRB, 0.10326E+01_JPRB, 0.88859E+00_JPRB, 0.87845E+00_JPRB, 0.89762E+00_JPRB, 0.85777E+00_JPRB ,&
&  0.83592E+00_JPRB, 0.79819E+00_JPRB, 0.78201E+00_JPRB, 0.78388E+00_JPRB, 0.78778E+00_JPRB, 0.76896E+00_JPRB, 0.76120E+00_JPRB ,&
&  0.73032E+00_JPRB, 0.72288E+00_JPRB, 0.74305E+00_JPRB, 0.78363E+00_JPRB, 0.89101E+00_JPRB, 0.97647E+00_JPRB, 0.95537E+00_JPRB ,&
&  0.10546E+01_JPRB, 0.11474E+01_JPRB, 0.10933E+01_JPRB, 0.10876E+01_JPRB, 0.10802E+01_JPRB, 0.10309E+01_JPRB, 0.97201E+00_JPRB ,&
&  0.96295E+00_JPRB, 0.84447E+00_JPRB, 0.68338E+00_JPRB, 0.76941E+00_JPRB, 0.83574E+00_JPRB, 0.93724E+00_JPRB, 0.87979E+00_JPRB ,&
&  0.80730E+00_JPRB, 0.76411E+00_JPRB, 0.71824E+00_JPRB, 0.66306E+00_JPRB, 0.63625E+00_JPRB, 0.63294E+00_JPRB, 0.62051E+00_JPRB ,&
&  0.58764E+00_JPRB, 0.58111E+00_JPRB, 0.57249E+00_JPRB, 0.55852E+00_JPRB, 0.57420E+00_JPRB, 0.48413E+00_JPRB, 0.56989E+00_JPRB ,&
&  0.53285E+00_JPRB, 0.52759E+00_JPRB, 0.45779E+00_JPRB, 0.60215E+00_JPRB, 0.62327E+00_JPRB, 0.64237E+00_JPRB, 0.67223E+00_JPRB ,&
&  0.70644E+00_JPRB, 0.72238E+00_JPRB, 0.74336E+00_JPRB, 0.75853E+00_JPRB, 0.74963E+00_JPRB, 0.73732E+00_JPRB, 0.74345E+00_JPRB ,&
&  0.74885E+00_JPRB, 0.74541E+00_JPRB, 0.74495E+00_JPRB, 0.74629E+00_JPRB, 0.75952E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.66900E+00_JPRB, 0.67909E+00_JPRB, 0.69526E+00_JPRB, 0.70727E+00_JPRB, 0.73007E+00_JPRB, 0.72478E+00_JPRB, 0.72127E+00_JPRB ,&
&  0.69775E+00_JPRB, 0.69519E+00_JPRB, 0.69077E+00_JPRB, 0.68944E+00_JPRB, 0.69354E+00_JPRB, 0.71537E+00_JPRB, 0.75651E+00_JPRB ,&
&  0.76853E+00_JPRB, 0.77326E+00_JPRB, 0.76887E+00_JPRB, 0.76111E+00_JPRB, 0.73432E+00_JPRB, 0.71254E+00_JPRB, 0.70144E+00_JPRB ,&
&  0.72898E+00_JPRB, 0.91699E+00_JPRB, 0.11439E+01_JPRB, 0.13340E+01_JPRB, 0.15490E+01_JPRB, 0.14179E+01_JPRB, 0.16090E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.11776E+01_JPRB, 0.11668E+01_JPRB, 0.10321E+01_JPRB, 0.10204E+01_JPRB, 0.10033E+01_JPRB, 0.98733E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.10051E+01_JPRB, 0.98895E+00_JPRB, 0.98258E+00_JPRB, 0.99677E+00_JPRB, 0.10216E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.10231E+01_JPRB, 0.10262E+01_JPRB, 0.10892E+01_JPRB, 0.11502E+01_JPRB, 0.11895E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.13264E+01_JPRB, 0.13469E+01_JPRB, 0.11981E+01_JPRB, 0.11659E+01_JPRB, 0.12131E+01_JPRB, 0.11043E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.10027E+01_JPRB, 0.69184E+00_JPRB, 0.75890E+00_JPRB, 0.82651E+00_JPRB, 0.85739E+00_JPRB, 0.94100E+00_JPRB ,&
&  0.85332E+00_JPRB, 0.81890E+00_JPRB, 0.73100E+00_JPRB, 0.68575E+00_JPRB, 0.67571E+00_JPRB, 0.70381E+00_JPRB, 0.68453E+00_JPRB ,&
&  0.68204E+00_JPRB, 0.60625E+00_JPRB, 0.59788E+00_JPRB, 0.60070E+00_JPRB, 0.59209E+00_JPRB, 0.58275E+00_JPRB, 0.57727E+00_JPRB ,&
&  0.58800E+00_JPRB, 0.57315E+00_JPRB, 0.58605E+00_JPRB, 0.60650E+00_JPRB, 0.55645E+00_JPRB, 0.65797E+00_JPRB, 0.66657E+00_JPRB ,&
&  0.67182E+00_JPRB, 0.66288E+00_JPRB, 0.64849E+00_JPRB, 0.64674E+00_JPRB, 0.63627E+00_JPRB, 0.62744E+00_JPRB, 0.63919E+00_JPRB ,&
&  0.65758E+00_JPRB, 0.65440E+00_JPRB, 0.64992E+00_JPRB, 0.64966E+00_JPRB, 0.65660E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.57101E+00_JPRB, 0.57411E+00_JPRB, 0.57653E+00_JPRB, 0.58397E+00_JPRB, 0.59732E+00_JPRB, 0.60323E+00_JPRB, 0.59955E+00_JPRB ,&
&  0.58670E+00_JPRB, 0.58133E+00_JPRB, 0.56303E+00_JPRB, 0.54030E+00_JPRB, 0.54148E+00_JPRB, 0.58688E+00_JPRB, 0.63528E+00_JPRB ,&
&  0.66831E+00_JPRB, 0.68577E+00_JPRB, 0.69106E+00_JPRB, 0.69960E+00_JPRB, 0.74414E+00_JPRB, 0.78650E+00_JPRB, 0.10011E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.18504E+01_JPRB, 0.21681E+01_JPRB, 0.22888E+01_JPRB, 0.21088E+01_JPRB, 0.16925E+01_JPRB, 0.18136E+01_JPRB ,&
&  0.16389E+01_JPRB, 0.12942E+01_JPRB, 0.11785E+01_JPRB, 0.11331E+01_JPRB, 0.11103E+01_JPRB, 0.10687E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10719E+01_JPRB, 0.10866E+01_JPRB, 0.11068E+01_JPRB, 0.11263E+01_JPRB, 0.11540E+01_JPRB, 0.11830E+01_JPRB ,&
&  0.12092E+01_JPRB, 0.12049E+01_JPRB, 0.12548E+01_JPRB, 0.11511E+01_JPRB, 0.11775E+01_JPRB, 0.12130E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.11798E+01_JPRB, 0.11884E+01_JPRB, 0.11868E+01_JPRB, 0.11887E+01_JPRB, 0.11327E+01_JPRB, 0.10774E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10099E+01_JPRB, 0.77021E+00_JPRB, 0.81472E+00_JPRB, 0.90223E+00_JPRB, 0.80738E+00_JPRB, 0.96789E+00_JPRB ,&
&  0.91777E+00_JPRB, 0.86633E+00_JPRB, 0.80893E+00_JPRB, 0.75903E+00_JPRB, 0.76461E+00_JPRB, 0.77795E+00_JPRB, 0.73206E+00_JPRB ,&
&  0.79144E+00_JPRB, 0.68729E+00_JPRB, 0.65918E+00_JPRB, 0.66155E+00_JPRB, 0.64533E+00_JPRB, 0.62637E+00_JPRB, 0.61870E+00_JPRB ,&
&  0.65330E+00_JPRB, 0.65896E+00_JPRB, 0.63071E+00_JPRB, 0.66258E+00_JPRB, 0.60114E+00_JPRB, 0.64867E+00_JPRB, 0.65798E+00_JPRB ,&
&  0.65634E+00_JPRB, 0.63724E+00_JPRB, 0.63238E+00_JPRB, 0.62676E+00_JPRB, 0.62518E+00_JPRB, 0.61035E+00_JPRB, 0.61390E+00_JPRB ,&
&  0.60969E+00_JPRB, 0.60423E+00_JPRB, 0.59554E+00_JPRB, 0.59329E+00_JPRB, 0.58014E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.60727E+00_JPRB, 0.60114E+00_JPRB, 0.58786E+00_JPRB, 0.57246E+00_JPRB, 0.55705E+00_JPRB, 0.53473E+00_JPRB, 0.54030E+00_JPRB ,&
&  0.53417E+00_JPRB, 0.54524E+00_JPRB, 0.57026E+00_JPRB, 0.59253E+00_JPRB, 0.63175E+00_JPRB, 0.69276E+00_JPRB, 0.75281E+00_JPRB ,&
&  0.79897E+00_JPRB, 0.82506E+00_JPRB, 0.89380E+00_JPRB, 0.99892E+00_JPRB, 0.11737E+01_JPRB, 0.14617E+01_JPRB, 0.19065E+01_JPRB ,&
&  0.21080E+01_JPRB, 0.22124E+01_JPRB, 0.21379E+01_JPRB, 0.18951E+01_JPRB, 0.16076E+01_JPRB, 0.14691E+01_JPRB, 0.16002E+01_JPRB ,&
&  0.16434E+01_JPRB, 0.15056E+01_JPRB, 0.12987E+01_JPRB, 0.12324E+01_JPRB, 0.11773E+01_JPRB, 0.11374E+01_JPRB, 0.11192E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.11275E+01_JPRB, 0.11241E+01_JPRB, 0.11421E+01_JPRB, 0.11524E+01_JPRB, 0.11637E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.11674E+01_JPRB, 0.11625E+01_JPRB, 0.11285E+01_JPRB, 0.11313E+01_JPRB, 0.11383E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.10595E+01_JPRB, 0.10773E+01_JPRB, 0.11080E+01_JPRB, 0.10760E+01_JPRB, 0.10010E+01_JPRB, 0.99116E+00_JPRB ,&
&  0.10102E+01_JPRB, 0.10309E+01_JPRB, 0.81080E+00_JPRB, 0.95882E+00_JPRB, 0.10951E+01_JPRB, 0.10190E+01_JPRB, 0.99545E+00_JPRB ,&
&  0.99420E+00_JPRB, 0.94213E+00_JPRB, 0.91764E+00_JPRB, 0.85749E+00_JPRB, 0.87543E+00_JPRB, 0.86933E+00_JPRB, 0.78611E+00_JPRB ,&
&  0.90637E+00_JPRB, 0.79654E+00_JPRB, 0.77719E+00_JPRB, 0.75582E+00_JPRB, 0.71989E+00_JPRB, 0.67787E+00_JPRB, 0.65094E+00_JPRB ,&
&  0.62878E+00_JPRB, 0.81386E+00_JPRB, 0.91659E+00_JPRB, 0.10998E+01_JPRB, 0.10962E+01_JPRB, 0.66363E+00_JPRB, 0.68558E+00_JPRB ,&
&  0.68118E+00_JPRB, 0.67739E+00_JPRB, 0.67749E+00_JPRB, 0.67189E+00_JPRB, 0.66427E+00_JPRB, 0.65978E+00_JPRB, 0.66089E+00_JPRB ,&
&  0.65236E+00_JPRB, 0.64449E+00_JPRB, 0.63589E+00_JPRB, 0.62959E+00_JPRB, 0.61938E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.70056E+00_JPRB, 0.70403E+00_JPRB, 0.69359E+00_JPRB, 0.66723E+00_JPRB, 0.63192E+00_JPRB, 0.59537E+00_JPRB, 0.57569E+00_JPRB ,&
&  0.55574E+00_JPRB, 0.57924E+00_JPRB, 0.62706E+00_JPRB, 0.69132E+00_JPRB, 0.77015E+00_JPRB, 0.85423E+00_JPRB, 0.93717E+00_JPRB ,&
&  0.99381E+00_JPRB, 0.10524E+01_JPRB, 0.11472E+01_JPRB, 0.13527E+01_JPRB, 0.15771E+01_JPRB, 0.17405E+01_JPRB, 0.18281E+01_JPRB ,&
&  0.18912E+01_JPRB, 0.19228E+01_JPRB, 0.18271E+01_JPRB, 0.13999E+01_JPRB, 0.11758E+01_JPRB, 0.13488E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.12614E+01_JPRB, 0.12192E+01_JPRB, 0.12074E+01_JPRB, 0.12054E+01_JPRB, 0.11601E+01_JPRB, 0.11216E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.11135E+01_JPRB, 0.11254E+01_JPRB, 0.11210E+01_JPRB, 0.11345E+01_JPRB, 0.11574E+01_JPRB, 0.11594E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11252E+01_JPRB, 0.11188E+01_JPRB, 0.11298E+01_JPRB, 0.11058E+01_JPRB, 0.10919E+01_JPRB, 0.10616E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10159E+01_JPRB, 0.10254E+01_JPRB, 0.10677E+01_JPRB, 0.10789E+01_JPRB, 0.97925E+00_JPRB, 0.98719E+00_JPRB ,&
&  0.10273E+01_JPRB, 0.10277E+01_JPRB, 0.94101E+00_JPRB, 0.10996E+01_JPRB, 0.78076E+00_JPRB, 0.90611E+00_JPRB, 0.10572E+01_JPRB ,&
&  0.10653E+01_JPRB, 0.97307E+00_JPRB, 0.91385E+00_JPRB, 0.88710E+00_JPRB, 0.84755E+00_JPRB, 0.81172E+00_JPRB, 0.89333E+00_JPRB ,&
&  0.93787E+00_JPRB, 0.89591E+00_JPRB, 0.85862E+00_JPRB, 0.81865E+00_JPRB, 0.80436E+00_JPRB, 0.76860E+00_JPRB, 0.74187E+00_JPRB ,&
&  0.70303E+00_JPRB, 0.82339E+00_JPRB, 0.84770E+00_JPRB, 0.87338E+00_JPRB, 0.86496E+00_JPRB, 0.65399E+00_JPRB, 0.68058E+00_JPRB ,&
&  0.68475E+00_JPRB, 0.67440E+00_JPRB, 0.67685E+00_JPRB, 0.67508E+00_JPRB, 0.66101E+00_JPRB, 0.65993E+00_JPRB, 0.66985E+00_JPRB ,&
&  0.66965E+00_JPRB, 0.67266E+00_JPRB, 0.67837E+00_JPRB, 0.68144E+00_JPRB, 0.69051E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.69833E+00_JPRB, 0.71618E+00_JPRB, 0.74029E+00_JPRB, 0.75430E+00_JPRB, 0.75735E+00_JPRB, 0.74222E+00_JPRB, 0.69551E+00_JPRB ,&
&  0.64808E+00_JPRB, 0.62976E+00_JPRB, 0.66745E+00_JPRB, 0.71515E+00_JPRB, 0.77112E+00_JPRB, 0.83906E+00_JPRB, 0.93438E+00_JPRB ,&
&  0.10526E+01_JPRB, 0.11582E+01_JPRB, 0.12703E+01_JPRB, 0.14483E+01_JPRB, 0.16429E+01_JPRB, 0.16763E+01_JPRB, 0.17041E+01_JPRB ,&
&  0.15112E+01_JPRB, 0.12493E+01_JPRB, 0.17646E+01_JPRB, 0.19668E+01_JPRB, 0.17393E+01_JPRB, 0.13743E+01_JPRB, 0.11931E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11966E+01_JPRB, 0.12162E+01_JPRB, 0.12191E+01_JPRB, 0.11157E+01_JPRB, 0.11030E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.10586E+01_JPRB, 0.10677E+01_JPRB, 0.10760E+01_JPRB, 0.10681E+01_JPRB, 0.10843E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.10702E+01_JPRB, 0.10784E+01_JPRB, 0.10814E+01_JPRB, 0.10893E+01_JPRB, 0.10827E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10119E+01_JPRB, 0.97756E+00_JPRB, 0.10079E+01_JPRB, 0.10699E+01_JPRB, 0.11173E+01_JPRB, 0.10388E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.10860E+01_JPRB, 0.10720E+01_JPRB, 0.11765E+01_JPRB, 0.91551E+00_JPRB, 0.97564E+00_JPRB, 0.95256E+00_JPRB ,&
&  0.10531E+01_JPRB, 0.10065E+01_JPRB, 0.93120E+00_JPRB, 0.88824E+00_JPRB, 0.87205E+00_JPRB, 0.78134E+00_JPRB, 0.81183E+00_JPRB ,&
&  0.84625E+00_JPRB, 0.85938E+00_JPRB, 0.83173E+00_JPRB, 0.82726E+00_JPRB, 0.78056E+00_JPRB, 0.77437E+00_JPRB, 0.79646E+00_JPRB ,&
&  0.88991E+00_JPRB, 0.86232E+00_JPRB, 0.80808E+00_JPRB, 0.74068E+00_JPRB, 0.71319E+00_JPRB, 0.62947E+00_JPRB, 0.66253E+00_JPRB ,&
&  0.67028E+00_JPRB, 0.66599E+00_JPRB, 0.65546E+00_JPRB, 0.65748E+00_JPRB, 0.65967E+00_JPRB, 0.65635E+00_JPRB, 0.66391E+00_JPRB ,&
&  0.66849E+00_JPRB, 0.67287E+00_JPRB, 0.67607E+00_JPRB, 0.68059E+00_JPRB, 0.68635E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.67587E+00_JPRB, 0.70541E+00_JPRB, 0.75119E+00_JPRB, 0.82283E+00_JPRB, 0.89534E+00_JPRB, 0.99750E+00_JPRB, 0.10824E+01_JPRB ,&
&  0.96190E+00_JPRB, 0.63481E+00_JPRB, 0.66000E+00_JPRB, 0.68495E+00_JPRB, 0.72320E+00_JPRB, 0.77609E+00_JPRB, 0.84730E+00_JPRB ,&
&  0.96214E+00_JPRB, 0.11142E+01_JPRB, 0.12962E+01_JPRB, 0.14544E+01_JPRB, 0.15986E+01_JPRB, 0.16531E+01_JPRB, 0.15598E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.14365E+01_JPRB, 0.22772E+01_JPRB, 0.20976E+01_JPRB, 0.18820E+01_JPRB, 0.15810E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.11159E+01_JPRB, 0.11136E+01_JPRB, 0.10908E+01_JPRB, 0.10298E+01_JPRB, 0.10269E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10030E+01_JPRB, 0.98376E+00_JPRB, 0.99252E+00_JPRB, 0.98391E+00_JPRB, 0.98271E+00_JPRB, 0.99521E+00_JPRB ,&
&  0.10215E+01_JPRB, 0.10307E+01_JPRB, 0.10329E+01_JPRB, 0.10211E+01_JPRB, 0.10506E+01_JPRB, 0.10761E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.99812E+00_JPRB, 0.10097E+01_JPRB, 0.10012E+01_JPRB, 0.96095E+00_JPRB, 0.10603E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.11427E+01_JPRB, 0.11362E+01_JPRB, 0.12488E+01_JPRB, 0.10436E+01_JPRB, 0.10972E+01_JPRB, 0.11346E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10637E+01_JPRB, 0.99412E+00_JPRB, 0.96586E+00_JPRB, 0.93491E+00_JPRB, 0.82317E+00_JPRB, 0.83626E+00_JPRB ,&
&  0.80273E+00_JPRB, 0.85651E+00_JPRB, 0.82447E+00_JPRB, 0.78492E+00_JPRB, 0.69889E+00_JPRB, 0.66842E+00_JPRB, 0.68765E+00_JPRB ,&
&  0.79034E+00_JPRB, 0.87561E+00_JPRB, 0.85742E+00_JPRB, 0.80097E+00_JPRB, 0.71716E+00_JPRB, 0.65247E+00_JPRB, 0.66517E+00_JPRB ,&
&  0.65854E+00_JPRB, 0.65125E+00_JPRB, 0.64875E+00_JPRB, 0.65438E+00_JPRB, 0.65892E+00_JPRB, 0.65664E+00_JPRB, 0.65194E+00_JPRB ,&
&  0.65107E+00_JPRB, 0.65044E+00_JPRB, 0.64903E+00_JPRB, 0.65290E+00_JPRB, 0.66188E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.62759E+00_JPRB, 0.64212E+00_JPRB, 0.67089E+00_JPRB, 0.70602E+00_JPRB, 0.74141E+00_JPRB, 0.78466E+00_JPRB, 0.77646E+00_JPRB ,&
&  0.68676E+00_JPRB, 0.63446E+00_JPRB, 0.65523E+00_JPRB, 0.65758E+00_JPRB, 0.66650E+00_JPRB, 0.69788E+00_JPRB, 0.75200E+00_JPRB ,&
&  0.82242E+00_JPRB, 0.95716E+00_JPRB, 0.11442E+01_JPRB, 0.13924E+01_JPRB, 0.15011E+01_JPRB, 0.15639E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.12125E+01_JPRB, 0.23935E+01_JPRB, 0.21389E+01_JPRB, 0.21926E+01_JPRB, 0.20366E+01_JPRB, 0.19496E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.10887E+01_JPRB, 0.10769E+01_JPRB, 0.10034E+01_JPRB, 0.95706E+00_JPRB, 0.95061E+00_JPRB, 0.96012E+00_JPRB ,&
&  0.95252E+00_JPRB, 0.94429E+00_JPRB, 0.92771E+00_JPRB, 0.93010E+00_JPRB, 0.92934E+00_JPRB, 0.93286E+00_JPRB, 0.94608E+00_JPRB ,&
&  0.97136E+00_JPRB, 0.98819E+00_JPRB, 0.98117E+00_JPRB, 0.98951E+00_JPRB, 0.10075E+01_JPRB, 0.10531E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.94894E+00_JPRB, 0.98893E+00_JPRB, 0.10012E+01_JPRB, 0.10026E+01_JPRB, 0.10693E+01_JPRB, 0.10470E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.11708E+01_JPRB, 0.13074E+01_JPRB, 0.11239E+01_JPRB, 0.11769E+01_JPRB, 0.12984E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.11496E+01_JPRB, 0.10821E+01_JPRB, 0.10451E+01_JPRB, 0.10066E+01_JPRB, 0.90899E+00_JPRB, 0.83796E+00_JPRB ,&
&  0.84229E+00_JPRB, 0.87797E+00_JPRB, 0.86072E+00_JPRB, 0.74900E+00_JPRB, 0.70265E+00_JPRB, 0.64979E+00_JPRB, 0.66413E+00_JPRB ,&
&  0.73946E+00_JPRB, 0.81598E+00_JPRB, 0.86725E+00_JPRB, 0.88040E+00_JPRB, 0.82301E+00_JPRB, 0.76432E+00_JPRB, 0.72418E+00_JPRB ,&
&  0.68932E+00_JPRB, 0.65979E+00_JPRB, 0.64905E+00_JPRB, 0.64816E+00_JPRB, 0.65111E+00_JPRB, 0.65427E+00_JPRB, 0.64717E+00_JPRB ,&
&  0.64293E+00_JPRB, 0.63951E+00_JPRB, 0.63666E+00_JPRB, 0.63280E+00_JPRB, 0.62982E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.62735E+00_JPRB, 0.61534E+00_JPRB, 0.62766E+00_JPRB, 0.65847E+00_JPRB, 0.67167E+00_JPRB, 0.68215E+00_JPRB, 0.66318E+00_JPRB ,&
&  0.60108E+00_JPRB, 0.58761E+00_JPRB, 0.59207E+00_JPRB, 0.59819E+00_JPRB, 0.60983E+00_JPRB, 0.64422E+00_JPRB, 0.69479E+00_JPRB ,&
&  0.75693E+00_JPRB, 0.84824E+00_JPRB, 0.94229E+00_JPRB, 0.11274E+01_JPRB, 0.12934E+01_JPRB, 0.15127E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.22503E+01_JPRB, 0.22779E+01_JPRB, 0.23106E+01_JPRB, 0.22834E+01_JPRB, 0.22167E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.12260E+01_JPRB, 0.10908E+01_JPRB, 0.97835E+00_JPRB, 0.89491E+00_JPRB, 0.90332E+00_JPRB, 0.91057E+00_JPRB ,&
&  0.92200E+00_JPRB, 0.94734E+00_JPRB, 0.90304E+00_JPRB, 0.89170E+00_JPRB, 0.88201E+00_JPRB, 0.87214E+00_JPRB, 0.90031E+00_JPRB ,&
&  0.93314E+00_JPRB, 0.94997E+00_JPRB, 0.96565E+00_JPRB, 0.93899E+00_JPRB, 0.99035E+00_JPRB, 0.10292E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.98756E+00_JPRB, 0.93005E+00_JPRB, 0.99993E+00_JPRB, 0.10892E+01_JPRB, 0.11425E+01_JPRB, 0.11376E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.12115E+01_JPRB, 0.12477E+01_JPRB, 0.11663E+01_JPRB, 0.12647E+01_JPRB, 0.14777E+01_JPRB, 0.14425E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.12862E+01_JPRB, 0.12199E+01_JPRB, 0.11147E+01_JPRB, 0.10725E+01_JPRB, 0.97758E+00_JPRB, 0.97714E+00_JPRB ,&
&  0.10576E+01_JPRB, 0.11341E+01_JPRB, 0.95382E+00_JPRB, 0.79904E+00_JPRB, 0.75724E+00_JPRB, 0.65083E+00_JPRB, 0.63640E+00_JPRB ,&
&  0.68789E+00_JPRB, 0.71277E+00_JPRB, 0.73567E+00_JPRB, 0.84592E+00_JPRB, 0.88920E+00_JPRB, 0.90017E+00_JPRB, 0.83625E+00_JPRB ,&
&  0.72978E+00_JPRB, 0.71043E+00_JPRB, 0.68571E+00_JPRB, 0.66611E+00_JPRB, 0.66597E+00_JPRB, 0.66200E+00_JPRB, 0.65825E+00_JPRB ,&
&  0.64817E+00_JPRB, 0.64436E+00_JPRB, 0.64468E+00_JPRB, 0.64710E+00_JPRB, 0.64336E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.65378E+00_JPRB, 0.63883E+00_JPRB, 0.62176E+00_JPRB, 0.62148E+00_JPRB, 0.64155E+00_JPRB, 0.63312E+00_JPRB, 0.61130E+00_JPRB ,&
&  0.56770E+00_JPRB, 0.54605E+00_JPRB, 0.52336E+00_JPRB, 0.51711E+00_JPRB, 0.52867E+00_JPRB, 0.56202E+00_JPRB, 0.61786E+00_JPRB ,&
&  0.69638E+00_JPRB, 0.76994E+00_JPRB, 0.84442E+00_JPRB, 0.91350E+00_JPRB, 0.10154E+01_JPRB, 0.97890E+00_JPRB, 0.11088E+01_JPRB ,&
&  0.16361E+01_JPRB, 0.20888E+01_JPRB, 0.22364E+01_JPRB, 0.23067E+01_JPRB, 0.24083E+01_JPRB, 0.25463E+01_JPRB, 0.21774E+01_JPRB ,&
&  0.18321E+01_JPRB, 0.15381E+01_JPRB, 0.12449E+01_JPRB, 0.10635E+01_JPRB, 0.93560E+00_JPRB, 0.86790E+00_JPRB, 0.86778E+00_JPRB ,&
&  0.87408E+00_JPRB, 0.89285E+00_JPRB, 0.87389E+00_JPRB, 0.84978E+00_JPRB, 0.83441E+00_JPRB, 0.81907E+00_JPRB, 0.83436E+00_JPRB ,&
&  0.86694E+00_JPRB, 0.89307E+00_JPRB, 0.90988E+00_JPRB, 0.91186E+00_JPRB, 0.95527E+00_JPRB, 0.10030E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10576E+01_JPRB, 0.10862E+01_JPRB, 0.11152E+01_JPRB, 0.12029E+01_JPRB, 0.12561E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.11390E+01_JPRB, 0.10484E+01_JPRB, 0.11377E+01_JPRB, 0.13588E+01_JPRB, 0.16981E+01_JPRB, 0.15349E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.11843E+01_JPRB, 0.12205E+01_JPRB, 0.13118E+01_JPRB, 0.13799E+01_JPRB, 0.12485E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.70541E+00_JPRB, 0.61128E+00_JPRB, 0.68074E+00_JPRB, 0.57093E+00_JPRB, 0.55613E+00_JPRB, 0.65722E+00_JPRB ,&
&  0.67907E+00_JPRB, 0.72901E+00_JPRB, 0.74969E+00_JPRB, 0.72445E+00_JPRB, 0.81753E+00_JPRB, 0.90607E+00_JPRB, 0.91927E+00_JPRB ,&
&  0.80296E+00_JPRB, 0.77628E+00_JPRB, 0.75024E+00_JPRB, 0.73268E+00_JPRB, 0.72054E+00_JPRB, 0.69952E+00_JPRB, 0.68874E+00_JPRB ,&
&  0.67611E+00_JPRB, 0.66599E+00_JPRB, 0.66075E+00_JPRB, 0.65800E+00_JPRB, 0.66087E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.68605E+00_JPRB, 0.67815E+00_JPRB, 0.66514E+00_JPRB, 0.64611E+00_JPRB, 0.64069E+00_JPRB, 0.63307E+00_JPRB, 0.60529E+00_JPRB ,&
&  0.56075E+00_JPRB, 0.53133E+00_JPRB, 0.50286E+00_JPRB, 0.48514E+00_JPRB, 0.49082E+00_JPRB, 0.50720E+00_JPRB, 0.54971E+00_JPRB ,&
&  0.62364E+00_JPRB, 0.73877E+00_JPRB, 0.86976E+00_JPRB, 0.90737E+00_JPRB, 0.90618E+00_JPRB, 0.85758E+00_JPRB, 0.10970E+01_JPRB ,&
&  0.14422E+01_JPRB, 0.17300E+01_JPRB, 0.22385E+01_JPRB, 0.24863E+01_JPRB, 0.25491E+01_JPRB, 0.25501E+01_JPRB, 0.24081E+01_JPRB ,&
&  0.21676E+01_JPRB, 0.18735E+01_JPRB, 0.14834E+01_JPRB, 0.12033E+01_JPRB, 0.10224E+01_JPRB, 0.90470E+00_JPRB, 0.83925E+00_JPRB ,&
&  0.84801E+00_JPRB, 0.86592E+00_JPRB, 0.85522E+00_JPRB, 0.82020E+00_JPRB, 0.79587E+00_JPRB, 0.78894E+00_JPRB, 0.79626E+00_JPRB ,&
&  0.81300E+00_JPRB, 0.83540E+00_JPRB, 0.85537E+00_JPRB, 0.89764E+00_JPRB, 0.79504E+00_JPRB, 0.96212E+00_JPRB, 0.10388E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.11666E+01_JPRB, 0.11174E+01_JPRB, 0.11893E+01_JPRB, 0.13302E+01_JPRB, 0.13574E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.13242E+01_JPRB, 0.11732E+01_JPRB, 0.10330E+01_JPRB, 0.11335E+01_JPRB, 0.13421E+01_JPRB, 0.16976E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.13783E+01_JPRB, 0.13669E+01_JPRB, 0.12630E+01_JPRB, 0.16469E+01_JPRB, 0.17428E+01_JPRB, 0.12047E+01_JPRB ,&
&  0.52157E+00_JPRB, 0.44541E+00_JPRB, 0.46583E+00_JPRB, 0.44520E+00_JPRB, 0.44976E+00_JPRB, 0.48955E+00_JPRB, 0.76230E+00_JPRB ,&
&  0.66481E+00_JPRB, 0.74326E+00_JPRB, 0.74615E+00_JPRB, 0.76226E+00_JPRB, 0.82517E+00_JPRB, 0.92821E+00_JPRB, 0.10816E+01_JPRB ,&
&  0.83960E+00_JPRB, 0.81454E+00_JPRB, 0.74663E+00_JPRB, 0.72227E+00_JPRB, 0.71674E+00_JPRB, 0.71901E+00_JPRB, 0.71055E+00_JPRB ,&
&  0.70903E+00_JPRB, 0.70982E+00_JPRB, 0.70586E+00_JPRB, 0.70047E+00_JPRB, 0.69500E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.69718E+00_JPRB, 0.69699E+00_JPRB, 0.69539E+00_JPRB, 0.68647E+00_JPRB, 0.67208E+00_JPRB, 0.65205E+00_JPRB, 0.62569E+00_JPRB ,&
&  0.58537E+00_JPRB, 0.54302E+00_JPRB, 0.51137E+00_JPRB, 0.49342E+00_JPRB, 0.48886E+00_JPRB, 0.48953E+00_JPRB, 0.51239E+00_JPRB ,&
&  0.57070E+00_JPRB, 0.71249E+00_JPRB, 0.96601E+00_JPRB, 0.91225E+00_JPRB, 0.93903E+00_JPRB, 0.87983E+00_JPRB, 0.98184E+00_JPRB ,&
&  0.12752E+01_JPRB, 0.15252E+01_JPRB, 0.19284E+01_JPRB, 0.23127E+01_JPRB, 0.27421E+01_JPRB, 0.27778E+01_JPRB, 0.27176E+01_JPRB ,&
&  0.24798E+01_JPRB, 0.21980E+01_JPRB, 0.17789E+01_JPRB, 0.14091E+01_JPRB, 0.11253E+01_JPRB, 0.95836E+00_JPRB, 0.86262E+00_JPRB ,&
&  0.84339E+00_JPRB, 0.85666E+00_JPRB, 0.84441E+00_JPRB, 0.79704E+00_JPRB, 0.76824E+00_JPRB, 0.76284E+00_JPRB, 0.78172E+00_JPRB ,&
&  0.78948E+00_JPRB, 0.79900E+00_JPRB, 0.80271E+00_JPRB, 0.84357E+00_JPRB, 0.77364E+00_JPRB, 0.72895E+00_JPRB, 0.92586E+00_JPRB ,&
&  0.11037E+01_JPRB, 0.13159E+01_JPRB, 0.13681E+01_JPRB, 0.14284E+01_JPRB, 0.15381E+01_JPRB, 0.15480E+01_JPRB, 0.16106E+01_JPRB ,&
&  0.14915E+01_JPRB, 0.15088E+01_JPRB, 0.11445E+01_JPRB, 0.11762E+01_JPRB, 0.14199E+01_JPRB, 0.12760E+01_JPRB, 0.12706E+01_JPRB ,&
&  0.14480E+01_JPRB, 0.13057E+01_JPRB, 0.12574E+01_JPRB, 0.10615E+01_JPRB, 0.14941E+01_JPRB, 0.14393E+01_JPRB, 0.60396E+00_JPRB ,&
&  0.49016E+00_JPRB, 0.46338E+00_JPRB, 0.43961E+00_JPRB, 0.42465E+00_JPRB, 0.44234E+00_JPRB, 0.47616E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.64535E+00_JPRB, 0.74520E+00_JPRB, 0.75228E+00_JPRB, 0.77750E+00_JPRB, 0.81118E+00_JPRB, 0.90742E+00_JPRB, 0.11247E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.95941E+00_JPRB, 0.83842E+00_JPRB, 0.76604E+00_JPRB, 0.73601E+00_JPRB, 0.72132E+00_JPRB, 0.70769E+00_JPRB ,&
&  0.69898E+00_JPRB, 0.70184E+00_JPRB, 0.70280E+00_JPRB, 0.69908E+00_JPRB, 0.69743E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.69469E+00_JPRB, 0.69888E+00_JPRB, 0.70149E+00_JPRB, 0.70066E+00_JPRB, 0.69406E+00_JPRB, 0.67695E+00_JPRB, 0.64908E+00_JPRB ,&
&  0.61642E+00_JPRB, 0.58052E+00_JPRB, 0.54560E+00_JPRB, 0.52374E+00_JPRB, 0.50662E+00_JPRB, 0.49535E+00_JPRB, 0.50041E+00_JPRB ,&
&  0.53752E+00_JPRB, 0.69620E+00_JPRB, 0.84748E+00_JPRB, 0.77999E+00_JPRB, 0.75983E+00_JPRB, 0.76320E+00_JPRB, 0.79143E+00_JPRB ,&
&  0.11108E+01_JPRB, 0.13414E+01_JPRB, 0.16491E+01_JPRB, 0.21967E+01_JPRB, 0.26979E+01_JPRB, 0.26997E+01_JPRB, 0.29189E+01_JPRB ,&
&  0.27190E+01_JPRB, 0.24044E+01_JPRB, 0.20271E+01_JPRB, 0.16275E+01_JPRB, 0.13036E+01_JPRB, 0.10589E+01_JPRB, 0.93698E+00_JPRB ,&
&  0.87623E+00_JPRB, 0.86923E+00_JPRB, 0.85039E+00_JPRB, 0.80220E+00_JPRB, 0.77901E+00_JPRB, 0.76085E+00_JPRB, 0.75264E+00_JPRB ,&
&  0.74869E+00_JPRB, 0.76733E+00_JPRB, 0.77805E+00_JPRB, 0.82857E+00_JPRB, 0.91969E+00_JPRB, 0.92782E+00_JPRB, 0.95145E+00_JPRB ,&
&  0.10126E+01_JPRB, 0.11508E+01_JPRB, 0.15236E+01_JPRB, 0.15579E+01_JPRB, 0.17430E+01_JPRB, 0.19096E+01_JPRB, 0.18488E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.16342E+01_JPRB, 0.13189E+01_JPRB, 0.13598E+01_JPRB, 0.12415E+01_JPRB, 0.10517E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.13504E+01_JPRB, 0.11189E+01_JPRB, 0.86901E+00_JPRB, 0.90304E+00_JPRB, 0.68323E+00_JPRB, 0.49629E+00_JPRB ,&
&  0.47042E+00_JPRB, 0.45385E+00_JPRB, 0.43672E+00_JPRB, 0.45558E+00_JPRB, 0.45248E+00_JPRB, 0.48481E+00_JPRB, 0.56694E+00_JPRB ,&
&  0.66818E+00_JPRB, 0.74936E+00_JPRB, 0.76818E+00_JPRB, 0.80038E+00_JPRB, 0.83255E+00_JPRB, 0.85140E+00_JPRB, 0.10407E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.10300E+01_JPRB, 0.91504E+00_JPRB, 0.83597E+00_JPRB, 0.78566E+00_JPRB, 0.75156E+00_JPRB, 0.72698E+00_JPRB ,&
&  0.71577E+00_JPRB, 0.70809E+00_JPRB, 0.70193E+00_JPRB, 0.69806E+00_JPRB, 0.69281E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.69643E+00_JPRB, 0.69705E+00_JPRB, 0.69955E+00_JPRB, 0.69924E+00_JPRB, 0.69468E+00_JPRB, 0.68404E+00_JPRB, 0.66760E+00_JPRB ,&
&  0.63770E+00_JPRB, 0.60679E+00_JPRB, 0.56912E+00_JPRB, 0.54755E+00_JPRB, 0.52574E+00_JPRB, 0.50964E+00_JPRB, 0.50463E+00_JPRB ,&
&  0.52886E+00_JPRB, 0.69535E+00_JPRB, 0.61749E+00_JPRB, 0.61225E+00_JPRB, 0.67688E+00_JPRB, 0.73930E+00_JPRB, 0.66996E+00_JPRB ,&
&  0.94423E+00_JPRB, 0.11878E+01_JPRB, 0.15098E+01_JPRB, 0.21604E+01_JPRB, 0.31157E+01_JPRB, 0.29540E+01_JPRB, 0.30213E+01_JPRB ,&
&  0.29962E+01_JPRB, 0.25333E+01_JPRB, 0.21973E+01_JPRB, 0.18249E+01_JPRB, 0.14627E+01_JPRB, 0.12070E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.92838E+00_JPRB, 0.89774E+00_JPRB, 0.86967E+00_JPRB, 0.83437E+00_JPRB, 0.81049E+00_JPRB, 0.78981E+00_JPRB, 0.75994E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.73804E+00_JPRB, 0.73687E+00_JPRB, 0.81156E+00_JPRB, 0.83645E+00_JPRB, 0.84871E+00_JPRB, 0.11297E+01_JPRB ,&
&  0.12450E+01_JPRB, 0.13815E+01_JPRB, 0.14335E+01_JPRB, 0.17126E+01_JPRB, 0.20339E+01_JPRB, 0.20734E+01_JPRB, 0.21377E+01_JPRB ,&
&  0.16487E+01_JPRB, 0.13429E+01_JPRB, 0.12877E+01_JPRB, 0.12811E+01_JPRB, 0.10731E+01_JPRB, 0.11818E+01_JPRB, 0.18126E+01_JPRB ,&
&  0.13655E+01_JPRB, 0.14570E+01_JPRB, 0.15849E+01_JPRB, 0.16053E+01_JPRB, 0.86466E+00_JPRB, 0.62418E+00_JPRB, 0.91497E+00_JPRB ,&
&  0.93261E+00_JPRB, 0.62305E+00_JPRB, 0.51013E+00_JPRB, 0.62114E+00_JPRB, 0.50491E+00_JPRB, 0.53290E+00_JPRB, 0.62569E+00_JPRB ,&
&  0.67482E+00_JPRB, 0.70885E+00_JPRB, 0.79531E+00_JPRB, 0.85395E+00_JPRB, 0.88488E+00_JPRB, 0.88457E+00_JPRB, 0.95442E+00_JPRB ,&
&  0.98113E+00_JPRB, 0.10043E+01_JPRB, 0.10091E+01_JPRB, 0.93539E+00_JPRB, 0.87457E+00_JPRB, 0.81290E+00_JPRB, 0.76681E+00_JPRB ,&
&  0.73550E+00_JPRB, 0.71411E+00_JPRB, 0.70184E+00_JPRB, 0.69364E+00_JPRB, 0.69581E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.71601E+00_JPRB, 0.70639E+00_JPRB, 0.70689E+00_JPRB, 0.71360E+00_JPRB, 0.71572E+00_JPRB, 0.70476E+00_JPRB, 0.68750E+00_JPRB ,&
&  0.67037E+00_JPRB, 0.63671E+00_JPRB, 0.59892E+00_JPRB, 0.56933E+00_JPRB, 0.55133E+00_JPRB, 0.53192E+00_JPRB, 0.51708E+00_JPRB ,&
&  0.52233E+00_JPRB, 0.55536E+00_JPRB, 0.51995E+00_JPRB, 0.52162E+00_JPRB, 0.66160E+00_JPRB, 0.65523E+00_JPRB, 0.68137E+00_JPRB ,&
&  0.82699E+00_JPRB, 0.10280E+01_JPRB, 0.13864E+01_JPRB, 0.19259E+01_JPRB, 0.24049E+01_JPRB, 0.28438E+01_JPRB, 0.30485E+01_JPRB ,&
&  0.30665E+01_JPRB, 0.26651E+01_JPRB, 0.22563E+01_JPRB, 0.19129E+01_JPRB, 0.15736E+01_JPRB, 0.13142E+01_JPRB, 0.11211E+01_JPRB ,&
&  0.10113E+01_JPRB, 0.95943E+00_JPRB, 0.92399E+00_JPRB, 0.88573E+00_JPRB, 0.86978E+00_JPRB, 0.83086E+00_JPRB, 0.79470E+00_JPRB ,&
&  0.77849E+00_JPRB, 0.77321E+00_JPRB, 0.75217E+00_JPRB, 0.78052E+00_JPRB, 0.78430E+00_JPRB, 0.82863E+00_JPRB, 0.94221E+00_JPRB ,&
&  0.11000E+01_JPRB, 0.12454E+01_JPRB, 0.13590E+01_JPRB, 0.17045E+01_JPRB, 0.21534E+01_JPRB, 0.20872E+01_JPRB, 0.22863E+01_JPRB ,&
&  0.19750E+01_JPRB, 0.16350E+01_JPRB, 0.13417E+01_JPRB, 0.10697E+01_JPRB, 0.12164E+01_JPRB, 0.18519E+01_JPRB, 0.18955E+01_JPRB ,&
&  0.17248E+01_JPRB, 0.16650E+01_JPRB, 0.16525E+01_JPRB, 0.14837E+01_JPRB, 0.12387E+01_JPRB, 0.93796E+00_JPRB, 0.11746E+01_JPRB ,&
&  0.11939E+01_JPRB, 0.11269E+01_JPRB, 0.10157E+01_JPRB, 0.85957E+00_JPRB, 0.69668E+00_JPRB, 0.77959E+00_JPRB, 0.76136E+00_JPRB ,&
&  0.72519E+00_JPRB, 0.69547E+00_JPRB, 0.75537E+00_JPRB, 0.87504E+00_JPRB, 0.88182E+00_JPRB, 0.84665E+00_JPRB, 0.89186E+00_JPRB ,&
&  0.90919E+00_JPRB, 0.95144E+00_JPRB, 0.95785E+00_JPRB, 0.93711E+00_JPRB, 0.91417E+00_JPRB, 0.87511E+00_JPRB, 0.83935E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.79747E+00_JPRB, 0.77431E+00_JPRB, 0.75193E+00_JPRB, 0.73073E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.72168E+00_JPRB, 0.70565E+00_JPRB, 0.69936E+00_JPRB, 0.70506E+00_JPRB, 0.72172E+00_JPRB, 0.72111E+00_JPRB, 0.70838E+00_JPRB ,&
&  0.68964E+00_JPRB, 0.66868E+00_JPRB, 0.63882E+00_JPRB, 0.60358E+00_JPRB, 0.57915E+00_JPRB, 0.54926E+00_JPRB, 0.52911E+00_JPRB ,&
&  0.52660E+00_JPRB, 0.52855E+00_JPRB, 0.47760E+00_JPRB, 0.52621E+00_JPRB, 0.58078E+00_JPRB, 0.56737E+00_JPRB, 0.64203E+00_JPRB ,&
&  0.71395E+00_JPRB, 0.87376E+00_JPRB, 0.11217E+01_JPRB, 0.14410E+01_JPRB, 0.17772E+01_JPRB, 0.22501E+01_JPRB, 0.26095E+01_JPRB ,&
&  0.25795E+01_JPRB, 0.25117E+01_JPRB, 0.23000E+01_JPRB, 0.20648E+01_JPRB, 0.17470E+01_JPRB, 0.14893E+01_JPRB, 0.13066E+01_JPRB ,&
&  0.11595E+01_JPRB, 0.10699E+01_JPRB, 0.10197E+01_JPRB, 0.97944E+00_JPRB, 0.94858E+00_JPRB, 0.91707E+00_JPRB, 0.89197E+00_JPRB ,&
&  0.84665E+00_JPRB, 0.83247E+00_JPRB, 0.80505E+00_JPRB, 0.81445E+00_JPRB, 0.79453E+00_JPRB, 0.84124E+00_JPRB, 0.91842E+00_JPRB ,&
&  0.11708E+01_JPRB, 0.13781E+01_JPRB, 0.14721E+01_JPRB, 0.19612E+01_JPRB, 0.18363E+01_JPRB, 0.18571E+01_JPRB, 0.21903E+01_JPRB ,&
&  0.24927E+01_JPRB, 0.25318E+01_JPRB, 0.24392E+01_JPRB, 0.19177E+01_JPRB, 0.16271E+01_JPRB, 0.21872E+01_JPRB, 0.20966E+01_JPRB ,&
&  0.18653E+01_JPRB, 0.17406E+01_JPRB, 0.15947E+01_JPRB, 0.16775E+01_JPRB, 0.14089E+01_JPRB, 0.12426E+01_JPRB, 0.98447E+00_JPRB ,&
&  0.99248E+00_JPRB, 0.86828E+00_JPRB, 0.98337E+00_JPRB, 0.88273E+00_JPRB, 0.83407E+00_JPRB, 0.80803E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.73529E+00_JPRB, 0.72041E+00_JPRB, 0.71346E+00_JPRB, 0.79406E+00_JPRB, 0.84614E+00_JPRB, 0.78708E+00_JPRB, 0.82560E+00_JPRB ,&
&  0.86232E+00_JPRB, 0.89645E+00_JPRB, 0.90804E+00_JPRB, 0.93372E+00_JPRB, 0.93172E+00_JPRB, 0.91576E+00_JPRB, 0.88827E+00_JPRB ,&
&  0.85697E+00_JPRB, 0.83740E+00_JPRB, 0.81077E+00_JPRB, 0.77756E+00_JPRB, 0.74594E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.70003E+00_JPRB, 0.68664E+00_JPRB, 0.68016E+00_JPRB, 0.69090E+00_JPRB, 0.71478E+00_JPRB, 0.72672E+00_JPRB, 0.71499E+00_JPRB ,&
&  0.70169E+00_JPRB, 0.68496E+00_JPRB, 0.67364E+00_JPRB, 0.64048E+00_JPRB, 0.60745E+00_JPRB, 0.56837E+00_JPRB, 0.54150E+00_JPRB ,&
&  0.54534E+00_JPRB, 0.61111E+00_JPRB, 0.52842E+00_JPRB, 0.48469E+00_JPRB, 0.45618E+00_JPRB, 0.51318E+00_JPRB, 0.59097E+00_JPRB ,&
&  0.68151E+00_JPRB, 0.75571E+00_JPRB, 0.94249E+00_JPRB, 0.10518E+01_JPRB, 0.11871E+01_JPRB, 0.15242E+01_JPRB, 0.19670E+01_JPRB ,&
&  0.24729E+01_JPRB, 0.20800E+01_JPRB, 0.20929E+01_JPRB, 0.19446E+01_JPRB, 0.18000E+01_JPRB, 0.16178E+01_JPRB, 0.14388E+01_JPRB ,&
&  0.12789E+01_JPRB, 0.11443E+01_JPRB, 0.10681E+01_JPRB, 0.10074E+01_JPRB, 0.95508E+00_JPRB, 0.90931E+00_JPRB, 0.87182E+00_JPRB ,&
&  0.85017E+00_JPRB, 0.86112E+00_JPRB, 0.88188E+00_JPRB, 0.87999E+00_JPRB, 0.92691E+00_JPRB, 0.10115E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.11799E+01_JPRB, 0.16538E+01_JPRB, 0.19289E+01_JPRB, 0.21597E+01_JPRB, 0.22900E+01_JPRB, 0.24435E+01_JPRB ,&
&  0.26575E+01_JPRB, 0.27156E+01_JPRB, 0.26601E+01_JPRB, 0.22902E+01_JPRB, 0.25082E+01_JPRB, 0.24610E+01_JPRB, 0.23073E+01_JPRB ,&
&  0.20945E+01_JPRB, 0.20072E+01_JPRB, 0.17826E+01_JPRB, 0.16843E+01_JPRB, 0.16827E+01_JPRB, 0.17546E+01_JPRB, 0.15354E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.13361E+01_JPRB, 0.11551E+01_JPRB, 0.92734E+00_JPRB, 0.75486E+00_JPRB, 0.74831E+00_JPRB, 0.75525E+00_JPRB ,&
&  0.75185E+00_JPRB, 0.75130E+00_JPRB, 0.72859E+00_JPRB, 0.75031E+00_JPRB, 0.79344E+00_JPRB, 0.77540E+00_JPRB, 0.75395E+00_JPRB ,&
&  0.75750E+00_JPRB, 0.81139E+00_JPRB, 0.83147E+00_JPRB, 0.83265E+00_JPRB, 0.86279E+00_JPRB, 0.85993E+00_JPRB, 0.83173E+00_JPRB ,&
&  0.79447E+00_JPRB, 0.76554E+00_JPRB, 0.75018E+00_JPRB, 0.73357E+00_JPRB, 0.71669E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.65492E+00_JPRB, 0.65292E+00_JPRB, 0.65524E+00_JPRB, 0.67143E+00_JPRB, 0.70215E+00_JPRB, 0.72326E+00_JPRB, 0.71628E+00_JPRB ,&
&  0.70862E+00_JPRB, 0.69508E+00_JPRB, 0.68132E+00_JPRB, 0.66118E+00_JPRB, 0.62545E+00_JPRB, 0.58656E+00_JPRB, 0.56346E+00_JPRB ,&
&  0.57687E+00_JPRB, 0.75048E+00_JPRB, 0.62403E+00_JPRB, 0.58634E+00_JPRB, 0.53045E+00_JPRB, 0.57354E+00_JPRB, 0.62119E+00_JPRB ,&
&  0.69645E+00_JPRB, 0.72225E+00_JPRB, 0.77502E+00_JPRB, 0.82896E+00_JPRB, 0.90556E+00_JPRB, 0.11226E+01_JPRB, 0.14764E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.18579E+01_JPRB, 0.17440E+01_JPRB, 0.18503E+01_JPRB, 0.17022E+01_JPRB, 0.14922E+01_JPRB, 0.14892E+01_JPRB ,&
&  0.13539E+01_JPRB, 0.12281E+01_JPRB, 0.11030E+01_JPRB, 0.10230E+01_JPRB, 0.96594E+00_JPRB, 0.91275E+00_JPRB, 0.86918E+00_JPRB ,&
&  0.84623E+00_JPRB, 0.85561E+00_JPRB, 0.89455E+00_JPRB, 0.92057E+00_JPRB, 0.93645E+00_JPRB, 0.10454E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.15026E+01_JPRB, 0.14129E+01_JPRB, 0.19346E+01_JPRB, 0.25911E+01_JPRB, 0.26607E+01_JPRB, 0.25334E+01_JPRB ,&
&  0.26940E+01_JPRB, 0.27493E+01_JPRB, 0.27592E+01_JPRB, 0.27285E+01_JPRB, 0.27082E+01_JPRB, 0.26058E+01_JPRB, 0.24679E+01_JPRB ,&
&  0.23084E+01_JPRB, 0.22066E+01_JPRB, 0.20635E+01_JPRB, 0.19084E+01_JPRB, 0.17304E+01_JPRB, 0.16486E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.15002E+01_JPRB, 0.12583E+01_JPRB, 0.86028E+00_JPRB, 0.86222E+00_JPRB, 0.67935E+00_JPRB, 0.65873E+00_JPRB, 0.72986E+00_JPRB ,&
&  0.72064E+00_JPRB, 0.75901E+00_JPRB, 0.75675E+00_JPRB, 0.70632E+00_JPRB, 0.72435E+00_JPRB, 0.73634E+00_JPRB, 0.70926E+00_JPRB ,&
&  0.73201E+00_JPRB, 0.70372E+00_JPRB, 0.75896E+00_JPRB, 0.74959E+00_JPRB, 0.73845E+00_JPRB, 0.73306E+00_JPRB, 0.72067E+00_JPRB ,&
&  0.69129E+00_JPRB, 0.65920E+00_JPRB, 0.64190E+00_JPRB, 0.64365E+00_JPRB, 0.64977E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.58512E+00_JPRB, 0.59680E+00_JPRB, 0.61190E+00_JPRB, 0.63378E+00_JPRB, 0.66853E+00_JPRB, 0.69631E+00_JPRB, 0.69784E+00_JPRB ,&
&  0.69232E+00_JPRB, 0.68512E+00_JPRB, 0.67261E+00_JPRB, 0.65862E+00_JPRB, 0.63340E+00_JPRB, 0.59496E+00_JPRB, 0.57709E+00_JPRB ,&
&  0.65395E+00_JPRB, 0.73549E+00_JPRB, 0.64968E+00_JPRB, 0.55452E+00_JPRB, 0.63515E+00_JPRB, 0.61197E+00_JPRB, 0.63453E+00_JPRB ,&
&  0.61799E+00_JPRB, 0.68259E+00_JPRB, 0.71497E+00_JPRB, 0.70630E+00_JPRB, 0.72378E+00_JPRB, 0.87631E+00_JPRB, 0.10741E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.13572E+01_JPRB, 0.14039E+01_JPRB, 0.14251E+01_JPRB, 0.14833E+01_JPRB, 0.13344E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.11502E+01_JPRB, 0.10592E+01_JPRB, 0.98867E+00_JPRB, 0.92765E+00_JPRB, 0.87653E+00_JPRB, 0.84521E+00_JPRB ,&
&  0.85385E+00_JPRB, 0.86650E+00_JPRB, 0.89107E+00_JPRB, 0.93095E+00_JPRB, 0.96279E+00_JPRB, 0.11725E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.20177E+01_JPRB, 0.22271E+01_JPRB, 0.25150E+01_JPRB, 0.28580E+01_JPRB, 0.29138E+01_JPRB, 0.29433E+01_JPRB, 0.30102E+01_JPRB ,&
&  0.29889E+01_JPRB, 0.29126E+01_JPRB, 0.28180E+01_JPRB, 0.27228E+01_JPRB, 0.26968E+01_JPRB, 0.26606E+01_JPRB, 0.25712E+01_JPRB ,&
&  0.23425E+01_JPRB, 0.22060E+01_JPRB, 0.21744E+01_JPRB, 0.19898E+01_JPRB, 0.18361E+01_JPRB, 0.16464E+01_JPRB, 0.16511E+01_JPRB ,&
&  0.16462E+01_JPRB, 0.95940E+00_JPRB, 0.74802E+00_JPRB, 0.85401E+00_JPRB, 0.70610E+00_JPRB, 0.69128E+00_JPRB, 0.78194E+00_JPRB ,&
&  0.77381E+00_JPRB, 0.74647E+00_JPRB, 0.74062E+00_JPRB, 0.68836E+00_JPRB, 0.67329E+00_JPRB, 0.73017E+00_JPRB, 0.67046E+00_JPRB ,&
&  0.63021E+00_JPRB, 0.64215E+00_JPRB, 0.66633E+00_JPRB, 0.64221E+00_JPRB, 0.61960E+00_JPRB, 0.61145E+00_JPRB, 0.59695E+00_JPRB ,&
&  0.57412E+00_JPRB, 0.56056E+00_JPRB, 0.55015E+00_JPRB, 0.55425E+00_JPRB, 0.57085E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.54765E+00_JPRB, 0.56862E+00_JPRB, 0.58374E+00_JPRB, 0.60434E+00_JPRB, 0.63536E+00_JPRB, 0.67005E+00_JPRB, 0.68055E+00_JPRB ,&
&  0.67798E+00_JPRB, 0.67251E+00_JPRB, 0.66321E+00_JPRB, 0.64815E+00_JPRB, 0.63168E+00_JPRB, 0.60191E+00_JPRB, 0.59312E+00_JPRB ,&
&  0.51383E+00_JPRB, 0.58467E+00_JPRB, 0.50911E+00_JPRB, 0.52831E+00_JPRB, 0.65648E+00_JPRB, 0.60680E+00_JPRB, 0.59170E+00_JPRB ,&
&  0.57294E+00_JPRB, 0.59945E+00_JPRB, 0.63234E+00_JPRB, 0.65135E+00_JPRB, 0.63344E+00_JPRB, 0.71006E+00_JPRB, 0.85927E+00_JPRB ,&
&  0.92588E+00_JPRB, 0.93775E+00_JPRB, 0.97909E+00_JPRB, 0.98963E+00_JPRB, 0.10274E+01_JPRB, 0.11230E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.10991E+01_JPRB, 0.10299E+01_JPRB, 0.96233E+00_JPRB, 0.91126E+00_JPRB, 0.86588E+00_JPRB, 0.81777E+00_JPRB, 0.78587E+00_JPRB ,&
&  0.77291E+00_JPRB, 0.76864E+00_JPRB, 0.80968E+00_JPRB, 0.88792E+00_JPRB, 0.95333E+00_JPRB, 0.13160E+01_JPRB, 0.20725E+01_JPRB ,&
&  0.23591E+01_JPRB, 0.24692E+01_JPRB, 0.27363E+01_JPRB, 0.31010E+01_JPRB, 0.31331E+01_JPRB, 0.31191E+01_JPRB, 0.30647E+01_JPRB ,&
&  0.30413E+01_JPRB, 0.29804E+01_JPRB, 0.28662E+01_JPRB, 0.28369E+01_JPRB, 0.26976E+01_JPRB, 0.26784E+01_JPRB, 0.25113E+01_JPRB ,&
&  0.22276E+01_JPRB, 0.21346E+01_JPRB, 0.21517E+01_JPRB, 0.21045E+01_JPRB, 0.20173E+01_JPRB, 0.19999E+01_JPRB, 0.19433E+01_JPRB ,&
&  0.18256E+01_JPRB, 0.15438E+01_JPRB, 0.11173E+01_JPRB, 0.98523E+00_JPRB, 0.73301E+00_JPRB, 0.73377E+00_JPRB, 0.88485E+00_JPRB ,&
&  0.87349E+00_JPRB, 0.77011E+00_JPRB, 0.68547E+00_JPRB, 0.64218E+00_JPRB, 0.62931E+00_JPRB, 0.66977E+00_JPRB, 0.60922E+00_JPRB ,&
&  0.54528E+00_JPRB, 0.55114E+00_JPRB, 0.55334E+00_JPRB, 0.55669E+00_JPRB, 0.55992E+00_JPRB, 0.54919E+00_JPRB, 0.52772E+00_JPRB ,&
&  0.52836E+00_JPRB, 0.51388E+00_JPRB, 0.50059E+00_JPRB, 0.50352E+00_JPRB, 0.52175E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.52806E+00_JPRB, 0.55154E+00_JPRB, 0.56860E+00_JPRB, 0.58212E+00_JPRB, 0.60394E+00_JPRB, 0.63709E+00_JPRB, 0.65042E+00_JPRB ,&
&  0.65033E+00_JPRB, 0.64764E+00_JPRB, 0.64318E+00_JPRB, 0.63245E+00_JPRB, 0.61143E+00_JPRB, 0.58458E+00_JPRB, 0.52539E+00_JPRB ,&
&  0.45606E+00_JPRB, 0.51711E+00_JPRB, 0.50138E+00_JPRB, 0.51802E+00_JPRB, 0.52741E+00_JPRB, 0.51810E+00_JPRB, 0.50637E+00_JPRB ,&
&  0.52529E+00_JPRB, 0.54277E+00_JPRB, 0.58405E+00_JPRB, 0.63334E+00_JPRB, 0.63352E+00_JPRB, 0.66458E+00_JPRB, 0.77044E+00_JPRB ,&
&  0.79443E+00_JPRB, 0.80321E+00_JPRB, 0.77338E+00_JPRB, 0.76378E+00_JPRB, 0.80201E+00_JPRB, 0.86862E+00_JPRB, 0.92178E+00_JPRB ,&
&  0.91824E+00_JPRB, 0.86874E+00_JPRB, 0.82010E+00_JPRB, 0.78761E+00_JPRB, 0.76295E+00_JPRB, 0.73619E+00_JPRB, 0.70988E+00_JPRB ,&
&  0.68419E+00_JPRB, 0.67245E+00_JPRB, 0.68837E+00_JPRB, 0.72936E+00_JPRB, 0.80867E+00_JPRB, 0.11180E+01_JPRB, 0.14140E+01_JPRB ,&
&  0.15225E+01_JPRB, 0.16856E+01_JPRB, 0.19156E+01_JPRB, 0.21513E+01_JPRB, 0.23200E+01_JPRB, 0.24553E+01_JPRB, 0.25721E+01_JPRB ,&
&  0.27188E+01_JPRB, 0.26767E+01_JPRB, 0.28950E+01_JPRB, 0.28486E+01_JPRB, 0.26875E+01_JPRB, 0.25923E+01_JPRB, 0.23871E+01_JPRB ,&
&  0.22099E+01_JPRB, 0.20520E+01_JPRB, 0.21090E+01_JPRB, 0.20903E+01_JPRB, 0.20402E+01_JPRB, 0.20103E+01_JPRB, 0.19182E+01_JPRB ,&
&  0.18509E+01_JPRB, 0.17658E+01_JPRB, 0.14790E+01_JPRB, 0.12263E+01_JPRB, 0.97776E+00_JPRB, 0.96738E+00_JPRB, 0.86637E+00_JPRB ,&
&  0.74637E+00_JPRB, 0.61512E+00_JPRB, 0.60463E+00_JPRB, 0.59148E+00_JPRB, 0.59430E+00_JPRB, 0.56300E+00_JPRB, 0.52759E+00_JPRB ,&
&  0.52292E+00_JPRB, 0.52840E+00_JPRB, 0.51121E+00_JPRB, 0.50555E+00_JPRB, 0.51751E+00_JPRB, 0.50553E+00_JPRB, 0.47493E+00_JPRB ,&
&  0.46911E+00_JPRB, 0.47166E+00_JPRB, 0.46414E+00_JPRB, 0.47354E+00_JPRB, 0.49699E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.50929E+00_JPRB, 0.52977E+00_JPRB, 0.54629E+00_JPRB, 0.56112E+00_JPRB, 0.57948E+00_JPRB, 0.59903E+00_JPRB, 0.59586E+00_JPRB ,&
&  0.60887E+00_JPRB, 0.61247E+00_JPRB, 0.60906E+00_JPRB, 0.59695E+00_JPRB, 0.57308E+00_JPRB, 0.50416E+00_JPRB, 0.41703E+00_JPRB ,&
&  0.39988E+00_JPRB, 0.42460E+00_JPRB, 0.45497E+00_JPRB, 0.46841E+00_JPRB, 0.47615E+00_JPRB, 0.48257E+00_JPRB, 0.48114E+00_JPRB ,&
&  0.48710E+00_JPRB, 0.50089E+00_JPRB, 0.53382E+00_JPRB, 0.58591E+00_JPRB, 0.63183E+00_JPRB, 0.64818E+00_JPRB, 0.69020E+00_JPRB ,&
&  0.68870E+00_JPRB, 0.70795E+00_JPRB, 0.70422E+00_JPRB, 0.64321E+00_JPRB, 0.66633E+00_JPRB, 0.67105E+00_JPRB, 0.69827E+00_JPRB ,&
&  0.72239E+00_JPRB, 0.71101E+00_JPRB, 0.68303E+00_JPRB, 0.66488E+00_JPRB, 0.65377E+00_JPRB, 0.63962E+00_JPRB, 0.62618E+00_JPRB ,&
&  0.60937E+00_JPRB, 0.59927E+00_JPRB, 0.61072E+00_JPRB, 0.63447E+00_JPRB, 0.67421E+00_JPRB, 0.88686E+00_JPRB, 0.10102E+01_JPRB ,&
&  0.11155E+01_JPRB, 0.13033E+01_JPRB, 0.16004E+01_JPRB, 0.17038E+01_JPRB, 0.19776E+01_JPRB, 0.21514E+01_JPRB, 0.23493E+01_JPRB ,&
&  0.25109E+01_JPRB, 0.24169E+01_JPRB, 0.25510E+01_JPRB, 0.25409E+01_JPRB, 0.24946E+01_JPRB, 0.23703E+01_JPRB, 0.21885E+01_JPRB ,&
&  0.20726E+01_JPRB, 0.21068E+01_JPRB, 0.21324E+01_JPRB, 0.20552E+01_JPRB, 0.18968E+01_JPRB, 0.17907E+01_JPRB, 0.16855E+01_JPRB ,&
&  0.16319E+01_JPRB, 0.15815E+01_JPRB, 0.14633E+01_JPRB, 0.13113E+01_JPRB, 0.11334E+01_JPRB, 0.97780E+00_JPRB, 0.85064E+00_JPRB ,&
&  0.74520E+00_JPRB, 0.63829E+00_JPRB, 0.59404E+00_JPRB, 0.58192E+00_JPRB, 0.57212E+00_JPRB, 0.52841E+00_JPRB, 0.51004E+00_JPRB ,&
&  0.50371E+00_JPRB, 0.49162E+00_JPRB, 0.49700E+00_JPRB, 0.48709E+00_JPRB, 0.49367E+00_JPRB, 0.49367E+00_JPRB, 0.47241E+00_JPRB ,&
&  0.45642E+00_JPRB, 0.46259E+00_JPRB, 0.46361E+00_JPRB, 0.47017E+00_JPRB, 0.48767E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.49854E+00_JPRB, 0.52008E+00_JPRB, 0.52997E+00_JPRB, 0.53988E+00_JPRB, 0.55143E+00_JPRB, 0.54542E+00_JPRB, 0.54134E+00_JPRB ,&
&  0.56468E+00_JPRB, 0.53066E+00_JPRB, 0.56749E+00_JPRB, 0.45549E+00_JPRB, 0.41225E+00_JPRB, 0.40449E+00_JPRB, 0.39550E+00_JPRB ,&
&  0.40023E+00_JPRB, 0.41618E+00_JPRB, 0.43184E+00_JPRB, 0.44216E+00_JPRB, 0.45981E+00_JPRB, 0.46320E+00_JPRB, 0.46435E+00_JPRB ,&
&  0.46878E+00_JPRB, 0.47864E+00_JPRB, 0.50659E+00_JPRB, 0.52367E+00_JPRB, 0.54286E+00_JPRB, 0.56060E+00_JPRB, 0.57494E+00_JPRB ,&
&  0.56967E+00_JPRB, 0.59032E+00_JPRB, 0.62686E+00_JPRB, 0.61055E+00_JPRB, 0.58864E+00_JPRB, 0.57079E+00_JPRB, 0.57787E+00_JPRB ,&
&  0.57622E+00_JPRB, 0.52311E+00_JPRB, 0.53718E+00_JPRB, 0.53672E+00_JPRB, 0.53689E+00_JPRB, 0.53779E+00_JPRB, 0.53919E+00_JPRB ,&
&  0.54200E+00_JPRB, 0.54430E+00_JPRB, 0.55893E+00_JPRB, 0.57750E+00_JPRB, 0.61609E+00_JPRB, 0.69790E+00_JPRB, 0.75598E+00_JPRB ,&
&  0.89395E+00_JPRB, 0.87110E+00_JPRB, 0.11797E+01_JPRB, 0.14847E+01_JPRB, 0.18895E+01_JPRB, 0.20371E+01_JPRB, 0.22025E+01_JPRB ,&
&  0.24104E+01_JPRB, 0.23215E+01_JPRB, 0.23501E+01_JPRB, 0.22635E+01_JPRB, 0.21626E+01_JPRB, 0.20573E+01_JPRB, 0.19552E+01_JPRB ,&
&  0.18897E+01_JPRB, 0.17920E+01_JPRB, 0.18481E+01_JPRB, 0.17547E+01_JPRB, 0.16133E+01_JPRB, 0.15138E+01_JPRB, 0.14253E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.12650E+01_JPRB, 0.11639E+01_JPRB, 0.97458E+00_JPRB, 0.88511E+00_JPRB, 0.81724E+00_JPRB, 0.72757E+00_JPRB ,&
&  0.68533E+00_JPRB, 0.64231E+00_JPRB, 0.60357E+00_JPRB, 0.59861E+00_JPRB, 0.56806E+00_JPRB, 0.53491E+00_JPRB, 0.52212E+00_JPRB ,&
&  0.49451E+00_JPRB, 0.46922E+00_JPRB, 0.47031E+00_JPRB, 0.46769E+00_JPRB, 0.46424E+00_JPRB, 0.46309E+00_JPRB, 0.48034E+00_JPRB ,&
&  0.46760E+00_JPRB, 0.45921E+00_JPRB, 0.46876E+00_JPRB, 0.46817E+00_JPRB, 0.47902E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.47356E+00_JPRB, 0.50115E+00_JPRB, 0.51327E+00_JPRB, 0.51861E+00_JPRB, 0.52354E+00_JPRB, 0.51804E+00_JPRB, 0.48780E+00_JPRB ,&
&  0.48362E+00_JPRB, 0.43729E+00_JPRB, 0.41494E+00_JPRB, 0.39945E+00_JPRB, 0.39376E+00_JPRB, 0.38802E+00_JPRB, 0.37947E+00_JPRB ,&
&  0.38264E+00_JPRB, 0.39538E+00_JPRB, 0.41377E+00_JPRB, 0.42684E+00_JPRB, 0.43839E+00_JPRB, 0.44603E+00_JPRB, 0.45232E+00_JPRB ,&
&  0.46028E+00_JPRB, 0.46975E+00_JPRB, 0.48286E+00_JPRB, 0.49403E+00_JPRB, 0.49849E+00_JPRB, 0.50047E+00_JPRB, 0.49887E+00_JPRB ,&
&  0.50678E+00_JPRB, 0.52458E+00_JPRB, 0.55319E+00_JPRB, 0.56681E+00_JPRB, 0.56185E+00_JPRB, 0.54150E+00_JPRB, 0.50770E+00_JPRB ,&
&  0.42911E+00_JPRB, 0.42307E+00_JPRB, 0.44857E+00_JPRB, 0.45866E+00_JPRB, 0.46907E+00_JPRB, 0.47918E+00_JPRB, 0.48843E+00_JPRB ,&
&  0.49971E+00_JPRB, 0.50903E+00_JPRB, 0.52813E+00_JPRB, 0.55024E+00_JPRB, 0.58699E+00_JPRB, 0.64608E+00_JPRB, 0.69196E+00_JPRB ,&
&  0.76641E+00_JPRB, 0.76173E+00_JPRB, 0.88935E+00_JPRB, 0.11872E+01_JPRB, 0.17937E+01_JPRB, 0.18745E+01_JPRB, 0.19375E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.20032E+01_JPRB, 0.20207E+01_JPRB, 0.20037E+01_JPRB, 0.19221E+01_JPRB, 0.18264E+01_JPRB, 0.17508E+01_JPRB ,&
&  0.16267E+01_JPRB, 0.15899E+01_JPRB, 0.15161E+01_JPRB, 0.14685E+01_JPRB, 0.13929E+01_JPRB, 0.13349E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.11612E+01_JPRB, 0.10617E+01_JPRB, 0.94487E+00_JPRB, 0.83343E+00_JPRB, 0.74830E+00_JPRB, 0.70458E+00_JPRB, 0.66880E+00_JPRB ,&
&  0.64402E+00_JPRB, 0.62926E+00_JPRB, 0.60982E+00_JPRB, 0.59349E+00_JPRB, 0.56378E+00_JPRB, 0.52587E+00_JPRB, 0.51268E+00_JPRB ,&
&  0.47510E+00_JPRB, 0.45634E+00_JPRB, 0.43855E+00_JPRB, 0.43065E+00_JPRB, 0.43292E+00_JPRB, 0.43506E+00_JPRB, 0.43442E+00_JPRB ,&
&  0.44453E+00_JPRB, 0.43789E+00_JPRB, 0.43127E+00_JPRB, 0.43718E+00_JPRB, 0.44823E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.43881E+00_JPRB, 0.45930E+00_JPRB, 0.47770E+00_JPRB, 0.48973E+00_JPRB, 0.48970E+00_JPRB, 0.48542E+00_JPRB, 0.46921E+00_JPRB ,&
&  0.46701E+00_JPRB, 0.44432E+00_JPRB, 0.42780E+00_JPRB, 0.41034E+00_JPRB, 0.39759E+00_JPRB, 0.38927E+00_JPRB, 0.39045E+00_JPRB ,&
&  0.39660E+00_JPRB, 0.40435E+00_JPRB, 0.41308E+00_JPRB, 0.41985E+00_JPRB, 0.42833E+00_JPRB, 0.43808E+00_JPRB, 0.44988E+00_JPRB ,&
&  0.46091E+00_JPRB, 0.46729E+00_JPRB, 0.47342E+00_JPRB, 0.47871E+00_JPRB, 0.48391E+00_JPRB, 0.48665E+00_JPRB, 0.47930E+00_JPRB ,&
&  0.48192E+00_JPRB, 0.48856E+00_JPRB, 0.51205E+00_JPRB, 0.53334E+00_JPRB, 0.54311E+00_JPRB, 0.52068E+00_JPRB, 0.44448E+00_JPRB ,&
&  0.39512E+00_JPRB, 0.37603E+00_JPRB, 0.38008E+00_JPRB, 0.41030E+00_JPRB, 0.42474E+00_JPRB, 0.43872E+00_JPRB, 0.45501E+00_JPRB ,&
&  0.45991E+00_JPRB, 0.45713E+00_JPRB, 0.47671E+00_JPRB, 0.50852E+00_JPRB, 0.53907E+00_JPRB, 0.59555E+00_JPRB, 0.64975E+00_JPRB ,&
&  0.72932E+00_JPRB, 0.85086E+00_JPRB, 0.10672E+01_JPRB, 0.11177E+01_JPRB, 0.12663E+01_JPRB, 0.16345E+01_JPRB, 0.16181E+01_JPRB ,&
&  0.16313E+01_JPRB, 0.16650E+01_JPRB, 0.17367E+01_JPRB, 0.17867E+01_JPRB, 0.16976E+01_JPRB, 0.15996E+01_JPRB, 0.14911E+01_JPRB ,&
&  0.13443E+01_JPRB, 0.12668E+01_JPRB, 0.12004E+01_JPRB, 0.11983E+01_JPRB, 0.11639E+01_JPRB, 0.11256E+01_JPRB, 0.10885E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.91257E+00_JPRB, 0.81344E+00_JPRB, 0.72435E+00_JPRB, 0.67939E+00_JPRB, 0.65271E+00_JPRB, 0.63500E+00_JPRB ,&
&  0.61571E+00_JPRB, 0.60089E+00_JPRB, 0.58564E+00_JPRB, 0.56682E+00_JPRB, 0.52522E+00_JPRB, 0.47537E+00_JPRB, 0.45070E+00_JPRB ,&
&  0.43576E+00_JPRB, 0.42682E+00_JPRB, 0.41838E+00_JPRB, 0.41365E+00_JPRB, 0.41253E+00_JPRB, 0.40924E+00_JPRB, 0.40031E+00_JPRB ,&
&  0.39945E+00_JPRB, 0.39612E+00_JPRB, 0.39174E+00_JPRB, 0.39755E+00_JPRB, 0.41679E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.41508E+00_JPRB, 0.42356E+00_JPRB, 0.43493E+00_JPRB, 0.44867E+00_JPRB, 0.45411E+00_JPRB, 0.44392E+00_JPRB, 0.42502E+00_JPRB ,&
&  0.41322E+00_JPRB, 0.40812E+00_JPRB, 0.41031E+00_JPRB, 0.41421E+00_JPRB, 0.41632E+00_JPRB, 0.41687E+00_JPRB, 0.41629E+00_JPRB ,&
&  0.41382E+00_JPRB, 0.41398E+00_JPRB, 0.41878E+00_JPRB, 0.42707E+00_JPRB, 0.43700E+00_JPRB, 0.44667E+00_JPRB, 0.45666E+00_JPRB ,&
&  0.46365E+00_JPRB, 0.46582E+00_JPRB, 0.46665E+00_JPRB, 0.46518E+00_JPRB, 0.46640E+00_JPRB, 0.46980E+00_JPRB, 0.46891E+00_JPRB ,&
&  0.46475E+00_JPRB, 0.46553E+00_JPRB, 0.47766E+00_JPRB, 0.49355E+00_JPRB, 0.51117E+00_JPRB, 0.49418E+00_JPRB, 0.43949E+00_JPRB ,&
&  0.39042E+00_JPRB, 0.35520E+00_JPRB, 0.34374E+00_JPRB, 0.34740E+00_JPRB, 0.36322E+00_JPRB, 0.38904E+00_JPRB, 0.40759E+00_JPRB ,&
&  0.42267E+00_JPRB, 0.43553E+00_JPRB, 0.44938E+00_JPRB, 0.46014E+00_JPRB, 0.47784E+00_JPRB, 0.50900E+00_JPRB, 0.55240E+00_JPRB ,&
&  0.62447E+00_JPRB, 0.72170E+00_JPRB, 0.85256E+00_JPRB, 0.84811E+00_JPRB, 0.90508E+00_JPRB, 0.10377E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.12702E+01_JPRB, 0.12905E+01_JPRB, 0.13407E+01_JPRB, 0.13453E+01_JPRB, 0.13250E+01_JPRB, 0.12373E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.10580E+01_JPRB, 0.99404E+00_JPRB, 0.96116E+00_JPRB, 0.95775E+00_JPRB, 0.95398E+00_JPRB, 0.93438E+00_JPRB ,&
&  0.88765E+00_JPRB, 0.82220E+00_JPRB, 0.74564E+00_JPRB, 0.66945E+00_JPRB, 0.64385E+00_JPRB, 0.64138E+00_JPRB, 0.64139E+00_JPRB ,&
&  0.62362E+00_JPRB, 0.60008E+00_JPRB, 0.57370E+00_JPRB, 0.54847E+00_JPRB, 0.50869E+00_JPRB, 0.47062E+00_JPRB, 0.44778E+00_JPRB ,&
&  0.43905E+00_JPRB, 0.43515E+00_JPRB, 0.43473E+00_JPRB, 0.42857E+00_JPRB, 0.42318E+00_JPRB, 0.42060E+00_JPRB, 0.41024E+00_JPRB ,&
&  0.40138E+00_JPRB, 0.39566E+00_JPRB, 0.39007E+00_JPRB, 0.39325E+00_JPRB, 0.40171E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.41736E+00_JPRB, 0.42400E+00_JPRB, 0.42943E+00_JPRB, 0.43232E+00_JPRB, 0.42898E+00_JPRB, 0.42678E+00_JPRB, 0.41970E+00_JPRB ,&
&  0.41581E+00_JPRB, 0.41664E+00_JPRB, 0.41429E+00_JPRB, 0.41385E+00_JPRB, 0.41384E+00_JPRB, 0.41404E+00_JPRB, 0.41563E+00_JPRB ,&
&  0.41909E+00_JPRB, 0.42436E+00_JPRB, 0.43086E+00_JPRB, 0.43777E+00_JPRB, 0.44496E+00_JPRB, 0.45243E+00_JPRB, 0.45925E+00_JPRB ,&
&  0.46309E+00_JPRB, 0.46276E+00_JPRB, 0.46444E+00_JPRB, 0.46398E+00_JPRB, 0.45985E+00_JPRB, 0.45322E+00_JPRB, 0.44588E+00_JPRB ,&
&  0.44620E+00_JPRB, 0.45411E+00_JPRB, 0.45804E+00_JPRB, 0.46462E+00_JPRB, 0.47596E+00_JPRB, 0.46003E+00_JPRB, 0.41901E+00_JPRB ,&
&  0.37420E+00_JPRB, 0.33798E+00_JPRB, 0.32214E+00_JPRB, 0.31697E+00_JPRB, 0.32354E+00_JPRB, 0.33706E+00_JPRB, 0.35378E+00_JPRB ,&
&  0.37040E+00_JPRB, 0.39168E+00_JPRB, 0.41269E+00_JPRB, 0.42688E+00_JPRB, 0.44001E+00_JPRB, 0.45590E+00_JPRB, 0.46959E+00_JPRB ,&
&  0.48951E+00_JPRB, 0.52387E+00_JPRB, 0.58599E+00_JPRB, 0.66349E+00_JPRB, 0.73677E+00_JPRB, 0.77741E+00_JPRB, 0.87769E+00_JPRB ,&
&  0.10653E+01_JPRB, 0.10158E+01_JPRB, 0.98709E+00_JPRB, 0.96099E+00_JPRB, 0.93030E+00_JPRB, 0.90155E+00_JPRB, 0.88414E+00_JPRB ,&
&  0.87814E+00_JPRB, 0.87173E+00_JPRB, 0.84982E+00_JPRB, 0.81770E+00_JPRB, 0.79559E+00_JPRB, 0.78826E+00_JPRB, 0.79751E+00_JPRB ,&
&  0.80117E+00_JPRB, 0.83707E+00_JPRB, 0.97945E+00_JPRB, 0.84652E+00_JPRB, 0.76451E+00_JPRB, 0.72367E+00_JPRB, 0.68215E+00_JPRB ,&
&  0.64426E+00_JPRB, 0.61340E+00_JPRB, 0.58481E+00_JPRB, 0.55407E+00_JPRB, 0.52359E+00_JPRB, 0.49533E+00_JPRB, 0.47658E+00_JPRB ,&
&  0.46826E+00_JPRB, 0.46317E+00_JPRB, 0.46141E+00_JPRB, 0.45690E+00_JPRB, 0.44559E+00_JPRB, 0.43823E+00_JPRB, 0.43331E+00_JPRB ,&
&  0.42475E+00_JPRB, 0.41551E+00_JPRB, 0.41088E+00_JPRB, 0.41236E+00_JPRB, 0.41331E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.43041E+00_JPRB, 0.43325E+00_JPRB, 0.43472E+00_JPRB, 0.43790E+00_JPRB, 0.43590E+00_JPRB, 0.43522E+00_JPRB, 0.42950E+00_JPRB ,&
&  0.42499E+00_JPRB, 0.42191E+00_JPRB, 0.42009E+00_JPRB, 0.42042E+00_JPRB, 0.42212E+00_JPRB, 0.42569E+00_JPRB, 0.43057E+00_JPRB ,&
&  0.43640E+00_JPRB, 0.43726E+00_JPRB, 0.44137E+00_JPRB, 0.44798E+00_JPRB, 0.45196E+00_JPRB, 0.45707E+00_JPRB, 0.46040E+00_JPRB ,&
&  0.46101E+00_JPRB, 0.45763E+00_JPRB, 0.45554E+00_JPRB, 0.45034E+00_JPRB, 0.44595E+00_JPRB, 0.44214E+00_JPRB, 0.44297E+00_JPRB ,&
&  0.43726E+00_JPRB, 0.43436E+00_JPRB, 0.44754E+00_JPRB, 0.45432E+00_JPRB, 0.43492E+00_JPRB, 0.41780E+00_JPRB, 0.38561E+00_JPRB ,&
&  0.35375E+00_JPRB, 0.32408E+00_JPRB, 0.31352E+00_JPRB, 0.30776E+00_JPRB, 0.31140E+00_JPRB, 0.31814E+00_JPRB, 0.33130E+00_JPRB ,&
&  0.34438E+00_JPRB, 0.36297E+00_JPRB, 0.37939E+00_JPRB, 0.39757E+00_JPRB, 0.41032E+00_JPRB, 0.42522E+00_JPRB, 0.43619E+00_JPRB ,&
&  0.44985E+00_JPRB, 0.46856E+00_JPRB, 0.49661E+00_JPRB, 0.53299E+00_JPRB, 0.56592E+00_JPRB, 0.60382E+00_JPRB, 0.65269E+00_JPRB ,&
&  0.72914E+00_JPRB, 0.72936E+00_JPRB, 0.74073E+00_JPRB, 0.73476E+00_JPRB, 0.72555E+00_JPRB, 0.72199E+00_JPRB, 0.71066E+00_JPRB ,&
&  0.71373E+00_JPRB, 0.73319E+00_JPRB, 0.72876E+00_JPRB, 0.71997E+00_JPRB, 0.70884E+00_JPRB, 0.70663E+00_JPRB, 0.71714E+00_JPRB ,&
&  0.72219E+00_JPRB, 0.73850E+00_JPRB, 0.81107E+00_JPRB, 0.79026E+00_JPRB, 0.76191E+00_JPRB, 0.72905E+00_JPRB, 0.69037E+00_JPRB ,&
&  0.65407E+00_JPRB, 0.62208E+00_JPRB, 0.59039E+00_JPRB, 0.56098E+00_JPRB, 0.53867E+00_JPRB, 0.51856E+00_JPRB, 0.50594E+00_JPRB ,&
&  0.49388E+00_JPRB, 0.48399E+00_JPRB, 0.47672E+00_JPRB, 0.46891E+00_JPRB, 0.45549E+00_JPRB, 0.44350E+00_JPRB, 0.43590E+00_JPRB ,&
&  0.42818E+00_JPRB, 0.42559E+00_JPRB, 0.42439E+00_JPRB, 0.42656E+00_JPRB, 0.42792E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.44861E+00_JPRB, 0.45286E+00_JPRB, 0.45671E+00_JPRB, 0.45876E+00_JPRB, 0.45789E+00_JPRB, 0.45676E+00_JPRB, 0.45326E+00_JPRB ,&
&  0.45096E+00_JPRB, 0.44867E+00_JPRB, 0.44717E+00_JPRB, 0.44699E+00_JPRB, 0.44781E+00_JPRB, 0.44945E+00_JPRB, 0.45120E+00_JPRB ,&
&  0.45238E+00_JPRB, 0.45435E+00_JPRB, 0.45483E+00_JPRB, 0.45644E+00_JPRB, 0.45741E+00_JPRB, 0.46055E+00_JPRB, 0.46138E+00_JPRB ,&
&  0.46259E+00_JPRB, 0.45852E+00_JPRB, 0.44844E+00_JPRB, 0.44312E+00_JPRB, 0.44330E+00_JPRB, 0.43302E+00_JPRB, 0.43586E+00_JPRB ,&
&  0.43953E+00_JPRB, 0.43943E+00_JPRB, 0.43436E+00_JPRB, 0.43178E+00_JPRB, 0.41913E+00_JPRB, 0.36037E+00_JPRB, 0.35717E+00_JPRB ,&
&  0.34384E+00_JPRB, 0.32572E+00_JPRB, 0.31681E+00_JPRB, 0.31003E+00_JPRB, 0.30828E+00_JPRB, 0.31575E+00_JPRB, 0.32742E+00_JPRB ,&
&  0.33496E+00_JPRB, 0.36096E+00_JPRB, 0.37218E+00_JPRB, 0.38580E+00_JPRB, 0.39740E+00_JPRB, 0.41306E+00_JPRB, 0.42278E+00_JPRB ,&
&  0.43507E+00_JPRB, 0.44778E+00_JPRB, 0.46872E+00_JPRB, 0.49783E+00_JPRB, 0.52547E+00_JPRB, 0.55155E+00_JPRB, 0.57487E+00_JPRB ,&
&  0.62129E+00_JPRB, 0.62854E+00_JPRB, 0.63980E+00_JPRB, 0.64323E+00_JPRB, 0.63744E+00_JPRB, 0.63420E+00_JPRB, 0.62725E+00_JPRB ,&
&  0.63093E+00_JPRB, 0.64132E+00_JPRB, 0.65290E+00_JPRB, 0.65430E+00_JPRB, 0.65258E+00_JPRB, 0.65264E+00_JPRB, 0.65722E+00_JPRB ,&
&  0.66595E+00_JPRB, 0.67355E+00_JPRB, 0.67983E+00_JPRB, 0.67228E+00_JPRB, 0.66868E+00_JPRB, 0.65157E+00_JPRB, 0.63779E+00_JPRB ,&
&  0.61715E+00_JPRB, 0.60531E+00_JPRB, 0.58087E+00_JPRB, 0.56254E+00_JPRB, 0.54439E+00_JPRB, 0.53039E+00_JPRB, 0.51519E+00_JPRB ,&
&  0.49967E+00_JPRB, 0.48555E+00_JPRB, 0.47465E+00_JPRB, 0.46556E+00_JPRB, 0.45501E+00_JPRB, 0.44426E+00_JPRB, 0.43927E+00_JPRB ,&
&  0.43687E+00_JPRB, 0.43610E+00_JPRB, 0.43652E+00_JPRB, 0.44009E+00_JPRB, 0.44319E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.45394E+00_JPRB, 0.45790E+00_JPRB, 0.46331E+00_JPRB, 0.46757E+00_JPRB, 0.47013E+00_JPRB, 0.47175E+00_JPRB, 0.47167E+00_JPRB ,&
&  0.47076E+00_JPRB, 0.46876E+00_JPRB, 0.46728E+00_JPRB, 0.46550E+00_JPRB, 0.46522E+00_JPRB, 0.46434E+00_JPRB, 0.46460E+00_JPRB ,&
&  0.46401E+00_JPRB, 0.46356E+00_JPRB, 0.46233E+00_JPRB, 0.46133E+00_JPRB, 0.46039E+00_JPRB, 0.46048E+00_JPRB, 0.45921E+00_JPRB ,&
&  0.45916E+00_JPRB, 0.45523E+00_JPRB, 0.44342E+00_JPRB, 0.44182E+00_JPRB, 0.43698E+00_JPRB, 0.43307E+00_JPRB, 0.42777E+00_JPRB ,&
&  0.42043E+00_JPRB, 0.39234E+00_JPRB, 0.40158E+00_JPRB, 0.38913E+00_JPRB, 0.37649E+00_JPRB, 0.36506E+00_JPRB, 0.35215E+00_JPRB ,&
&  0.34469E+00_JPRB, 0.33495E+00_JPRB, 0.32796E+00_JPRB, 0.32559E+00_JPRB, 0.32607E+00_JPRB, 0.33265E+00_JPRB, 0.34146E+00_JPRB ,&
&  0.34871E+00_JPRB, 0.36749E+00_JPRB, 0.37776E+00_JPRB, 0.38969E+00_JPRB, 0.40227E+00_JPRB, 0.41698E+00_JPRB, 0.42434E+00_JPRB ,&
&  0.43650E+00_JPRB, 0.44210E+00_JPRB, 0.45726E+00_JPRB, 0.47521E+00_JPRB, 0.50098E+00_JPRB, 0.52388E+00_JPRB, 0.54619E+00_JPRB ,&
&  0.57622E+00_JPRB, 0.59259E+00_JPRB, 0.60327E+00_JPRB, 0.60704E+00_JPRB, 0.60558E+00_JPRB, 0.60126E+00_JPRB, 0.59582E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.58923E+00_JPRB, 0.59141E+00_JPRB, 0.59534E+00_JPRB, 0.59535E+00_JPRB, 0.59863E+00_JPRB, 0.60280E+00_JPRB ,&
&  0.60809E+00_JPRB, 0.60967E+00_JPRB, 0.61775E+00_JPRB, 0.61235E+00_JPRB, 0.61021E+00_JPRB, 0.59692E+00_JPRB, 0.58715E+00_JPRB ,&
&  0.57946E+00_JPRB, 0.56898E+00_JPRB, 0.55633E+00_JPRB, 0.54303E+00_JPRB, 0.53086E+00_JPRB, 0.52051E+00_JPRB, 0.51163E+00_JPRB ,&
&  0.50219E+00_JPRB, 0.49149E+00_JPRB, 0.48269E+00_JPRB, 0.47101E+00_JPRB, 0.46014E+00_JPRB, 0.45257E+00_JPRB, 0.44962E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.44780E+00_JPRB, 0.44781E+00_JPRB, 0.45081E+00_JPRB, 0.45223E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.47368E+00_JPRB, 0.47275E+00_JPRB, 0.47208E+00_JPRB, 0.47109E+00_JPRB, 0.47018E+00_JPRB, 0.46912E+00_JPRB, 0.46997E+00_JPRB ,&
&  0.47065E+00_JPRB, 0.47112E+00_JPRB, 0.47070E+00_JPRB, 0.46955E+00_JPRB, 0.46912E+00_JPRB, 0.46747E+00_JPRB, 0.46649E+00_JPRB ,&
&  0.46532E+00_JPRB, 0.46413E+00_JPRB, 0.46347E+00_JPRB, 0.46311E+00_JPRB, 0.46227E+00_JPRB, 0.46198E+00_JPRB, 0.45786E+00_JPRB ,&
&  0.45505E+00_JPRB, 0.45261E+00_JPRB, 0.43603E+00_JPRB, 0.43770E+00_JPRB, 0.43148E+00_JPRB, 0.42680E+00_JPRB, 0.42337E+00_JPRB ,&
&  0.41088E+00_JPRB, 0.40519E+00_JPRB, 0.39213E+00_JPRB, 0.38069E+00_JPRB, 0.37859E+00_JPRB, 0.36852E+00_JPRB, 0.36704E+00_JPRB ,&
&  0.36379E+00_JPRB, 0.36061E+00_JPRB, 0.35966E+00_JPRB, 0.36043E+00_JPRB, 0.36472E+00_JPRB, 0.36950E+00_JPRB, 0.37489E+00_JPRB ,&
&  0.38371E+00_JPRB, 0.39263E+00_JPRB, 0.40610E+00_JPRB, 0.41200E+00_JPRB, 0.42525E+00_JPRB, 0.43978E+00_JPRB, 0.45496E+00_JPRB ,&
&  0.46510E+00_JPRB, 0.46461E+00_JPRB, 0.46904E+00_JPRB, 0.47715E+00_JPRB, 0.48610E+00_JPRB, 0.49726E+00_JPRB, 0.51289E+00_JPRB ,&
&  0.53369E+00_JPRB, 0.55146E+00_JPRB, 0.56226E+00_JPRB, 0.56452E+00_JPRB, 0.56424E+00_JPRB, 0.56080E+00_JPRB, 0.55860E+00_JPRB ,&
&  0.55710E+00_JPRB, 0.55660E+00_JPRB, 0.55580E+00_JPRB, 0.55610E+00_JPRB, 0.55481E+00_JPRB, 0.55596E+00_JPRB, 0.55604E+00_JPRB ,&
&  0.55594E+00_JPRB, 0.55520E+00_JPRB, 0.55889E+00_JPRB, 0.55216E+00_JPRB, 0.55123E+00_JPRB, 0.54948E+00_JPRB, 0.54499E+00_JPRB ,&
&  0.53947E+00_JPRB, 0.53485E+00_JPRB, 0.52869E+00_JPRB, 0.52136E+00_JPRB, 0.51277E+00_JPRB, 0.50598E+00_JPRB, 0.50057E+00_JPRB ,&
&  0.49724E+00_JPRB, 0.49411E+00_JPRB, 0.49164E+00_JPRB, 0.48407E+00_JPRB, 0.47887E+00_JPRB, 0.47380E+00_JPRB, 0.47192E+00_JPRB ,&
&  0.47024E+00_JPRB, 0.47118E+00_JPRB, 0.47124E+00_JPRB, 0.47288E+00_JPRB, 0.47387E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.47960E+00_JPRB, 0.47946E+00_JPRB, 0.47927E+00_JPRB, 0.47860E+00_JPRB, 0.47804E+00_JPRB, 0.47736E+00_JPRB, 0.47660E+00_JPRB ,&
&  0.47580E+00_JPRB, 0.47536E+00_JPRB, 0.47473E+00_JPRB, 0.47387E+00_JPRB, 0.47247E+00_JPRB, 0.47111E+00_JPRB, 0.47016E+00_JPRB ,&
&  0.46839E+00_JPRB, 0.46689E+00_JPRB, 0.46568E+00_JPRB, 0.46481E+00_JPRB, 0.46374E+00_JPRB, 0.46282E+00_JPRB, 0.45897E+00_JPRB ,&
&  0.45653E+00_JPRB, 0.45446E+00_JPRB, 0.45171E+00_JPRB, 0.44717E+00_JPRB, 0.44469E+00_JPRB, 0.42771E+00_JPRB, 0.42920E+00_JPRB ,&
&  0.43964E+00_JPRB, 0.43780E+00_JPRB, 0.43502E+00_JPRB, 0.43062E+00_JPRB, 0.41793E+00_JPRB, 0.41029E+00_JPRB, 0.40830E+00_JPRB ,&
&  0.39397E+00_JPRB, 0.40179E+00_JPRB, 0.40367E+00_JPRB, 0.40475E+00_JPRB, 0.40791E+00_JPRB, 0.40737E+00_JPRB, 0.41006E+00_JPRB ,&
&  0.42239E+00_JPRB, 0.42557E+00_JPRB, 0.43327E+00_JPRB, 0.44268E+00_JPRB, 0.45301E+00_JPRB, 0.46285E+00_JPRB, 0.47265E+00_JPRB ,&
&  0.48003E+00_JPRB, 0.48505E+00_JPRB, 0.48835E+00_JPRB, 0.49021E+00_JPRB, 0.49330E+00_JPRB, 0.49803E+00_JPRB, 0.50598E+00_JPRB ,&
&  0.51880E+00_JPRB, 0.53283E+00_JPRB, 0.54332E+00_JPRB, 0.54829E+00_JPRB, 0.55463E+00_JPRB, 0.55917E+00_JPRB, 0.56247E+00_JPRB ,&
&  0.56405E+00_JPRB, 0.56342E+00_JPRB, 0.56057E+00_JPRB, 0.55763E+00_JPRB, 0.55480E+00_JPRB, 0.55038E+00_JPRB, 0.54545E+00_JPRB ,&
&  0.53979E+00_JPRB, 0.53397E+00_JPRB, 0.52833E+00_JPRB, 0.52277E+00_JPRB, 0.51906E+00_JPRB, 0.51539E+00_JPRB, 0.51409E+00_JPRB ,&
&  0.51173E+00_JPRB, 0.51054E+00_JPRB, 0.50767E+00_JPRB, 0.50459E+00_JPRB, 0.49956E+00_JPRB, 0.49515E+00_JPRB, 0.49039E+00_JPRB ,&
&  0.48742E+00_JPRB, 0.48532E+00_JPRB, 0.48332E+00_JPRB, 0.48036E+00_JPRB, 0.47927E+00_JPRB, 0.47867E+00_JPRB, 0.47888E+00_JPRB ,&
&  0.47886E+00_JPRB, 0.47926E+00_JPRB, 0.47924E+00_JPRB, 0.47958E+00_JPRB, 0.47951E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.47209E+00_JPRB, 0.47231E+00_JPRB, 0.47229E+00_JPRB, 0.47209E+00_JPRB, 0.47196E+00_JPRB, 0.47188E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.47134E+00_JPRB, 0.47116E+00_JPRB, 0.47115E+00_JPRB, 0.47103E+00_JPRB, 0.47057E+00_JPRB, 0.47032E+00_JPRB, 0.47020E+00_JPRB ,&
&  0.46986E+00_JPRB, 0.46932E+00_JPRB, 0.46840E+00_JPRB, 0.46784E+00_JPRB, 0.46737E+00_JPRB, 0.46668E+00_JPRB, 0.46668E+00_JPRB ,&
&  0.46664E+00_JPRB, 0.46585E+00_JPRB, 0.46427E+00_JPRB, 0.46000E+00_JPRB, 0.45688E+00_JPRB, 0.45398E+00_JPRB, 0.45136E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44819E+00_JPRB, 0.44817E+00_JPRB, 0.44819E+00_JPRB, 0.45002E+00_JPRB, 0.44964E+00_JPRB, 0.44900E+00_JPRB ,&
&  0.44759E+00_JPRB, 0.44451E+00_JPRB, 0.44535E+00_JPRB, 0.44640E+00_JPRB, 0.44742E+00_JPRB, 0.44999E+00_JPRB, 0.45267E+00_JPRB ,&
&  0.45583E+00_JPRB, 0.45928E+00_JPRB, 0.46448E+00_JPRB, 0.47030E+00_JPRB, 0.47589E+00_JPRB, 0.48117E+00_JPRB, 0.48592E+00_JPRB ,&
&  0.49001E+00_JPRB, 0.49434E+00_JPRB, 0.49903E+00_JPRB, 0.50397E+00_JPRB, 0.50855E+00_JPRB, 0.51392E+00_JPRB, 0.51879E+00_JPRB ,&
&  0.52273E+00_JPRB, 0.52688E+00_JPRB, 0.53340E+00_JPRB, 0.54110E+00_JPRB, 0.54934E+00_JPRB, 0.55437E+00_JPRB, 0.55780E+00_JPRB ,&
&  0.55955E+00_JPRB, 0.56002E+00_JPRB, 0.55846E+00_JPRB, 0.55578E+00_JPRB, 0.55232E+00_JPRB, 0.54699E+00_JPRB, 0.54151E+00_JPRB ,&
&  0.53587E+00_JPRB, 0.53063E+00_JPRB, 0.52545E+00_JPRB, 0.52013E+00_JPRB, 0.51528E+00_JPRB, 0.51055E+00_JPRB, 0.50674E+00_JPRB ,&
&  0.50290E+00_JPRB, 0.49983E+00_JPRB, 0.49689E+00_JPRB, 0.49443E+00_JPRB, 0.49113E+00_JPRB, 0.48802E+00_JPRB, 0.48477E+00_JPRB ,&
&  0.48225E+00_JPRB, 0.47934E+00_JPRB, 0.47644E+00_JPRB, 0.47391E+00_JPRB, 0.47269E+00_JPRB, 0.47202E+00_JPRB, 0.47158E+00_JPRB ,&
&  0.47114E+00_JPRB, 0.47122E+00_JPRB, 0.47160E+00_JPRB, 0.47176E+00_JPRB, 0.47188E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.47611E+00_JPRB, 0.47585E+00_JPRB, 0.47549E+00_JPRB, 0.47514E+00_JPRB, 0.47472E+00_JPRB, 0.47435E+00_JPRB, 0.47394E+00_JPRB ,&
&  0.47347E+00_JPRB, 0.47308E+00_JPRB, 0.47293E+00_JPRB, 0.47281E+00_JPRB, 0.47274E+00_JPRB, 0.47264E+00_JPRB, 0.47266E+00_JPRB ,&
&  0.47258E+00_JPRB, 0.47235E+00_JPRB, 0.47204E+00_JPRB, 0.47214E+00_JPRB, 0.47227E+00_JPRB, 0.47230E+00_JPRB, 0.47244E+00_JPRB ,&
&  0.47237E+00_JPRB, 0.47182E+00_JPRB, 0.47116E+00_JPRB, 0.47098E+00_JPRB, 0.47081E+00_JPRB, 0.47070E+00_JPRB, 0.47056E+00_JPRB ,&
&  0.47038E+00_JPRB, 0.47029E+00_JPRB, 0.47007E+00_JPRB, 0.46985E+00_JPRB, 0.46996E+00_JPRB, 0.47033E+00_JPRB, 0.47079E+00_JPRB ,&
&  0.47131E+00_JPRB, 0.47191E+00_JPRB, 0.47281E+00_JPRB, 0.47370E+00_JPRB, 0.47463E+00_JPRB, 0.47532E+00_JPRB, 0.47628E+00_JPRB ,&
&  0.47713E+00_JPRB, 0.47800E+00_JPRB, 0.47906E+00_JPRB, 0.48036E+00_JPRB, 0.48155E+00_JPRB, 0.48287E+00_JPRB, 0.48413E+00_JPRB ,&
&  0.48607E+00_JPRB, 0.48835E+00_JPRB, 0.49078E+00_JPRB, 0.49322E+00_JPRB, 0.49554E+00_JPRB, 0.49796E+00_JPRB, 0.50064E+00_JPRB ,&
&  0.50317E+00_JPRB, 0.50554E+00_JPRB, 0.50834E+00_JPRB, 0.51145E+00_JPRB, 0.51455E+00_JPRB, 0.51657E+00_JPRB, 0.51857E+00_JPRB ,&
&  0.52053E+00_JPRB, 0.52215E+00_JPRB, 0.52222E+00_JPRB, 0.52217E+00_JPRB, 0.52188E+00_JPRB, 0.52092E+00_JPRB, 0.51958E+00_JPRB ,&
&  0.51797E+00_JPRB, 0.51627E+00_JPRB, 0.51365E+00_JPRB, 0.51059E+00_JPRB, 0.50775E+00_JPRB, 0.50499E+00_JPRB, 0.50221E+00_JPRB ,&
&  0.49958E+00_JPRB, 0.49739E+00_JPRB, 0.49542E+00_JPRB, 0.49350E+00_JPRB, 0.49149E+00_JPRB, 0.48962E+00_JPRB, 0.48778E+00_JPRB ,&
&  0.48619E+00_JPRB, 0.48457E+00_JPRB, 0.48305E+00_JPRB, 0.48162E+00_JPRB, 0.48054E+00_JPRB, 0.47972E+00_JPRB, 0.47901E+00_JPRB ,&
&  0.47833E+00_JPRB, 0.47778E+00_JPRB, 0.47740E+00_JPRB, 0.47695E+00_JPRB, 0.47649E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB ,&
&  0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB, 0.48493E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB, 0.21810E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.24494E+00_JPRB, 0.24498E+00_JPRB, 0.24503E+00_JPRB, 0.24467E+00_JPRB, 0.24320E+00_JPRB, 0.24213E+00_JPRB, 0.24119E+00_JPRB ,&
&  0.24077E+00_JPRB, 0.23999E+00_JPRB, 0.23959E+00_JPRB, 0.23946E+00_JPRB, 0.23913E+00_JPRB, 0.23863E+00_JPRB, 0.23797E+00_JPRB ,&
&  0.23712E+00_JPRB, 0.23599E+00_JPRB, 0.23490E+00_JPRB, 0.23479E+00_JPRB, 0.23503E+00_JPRB, 0.23529E+00_JPRB, 0.23549E+00_JPRB ,&
&  0.23558E+00_JPRB, 0.23563E+00_JPRB, 0.23520E+00_JPRB, 0.23524E+00_JPRB, 0.23527E+00_JPRB, 0.23527E+00_JPRB, 0.23542E+00_JPRB ,&
&  0.23567E+00_JPRB, 0.23586E+00_JPRB, 0.23561E+00_JPRB, 0.23513E+00_JPRB, 0.23422E+00_JPRB, 0.23255E+00_JPRB, 0.22995E+00_JPRB ,&
&  0.22700E+00_JPRB, 0.22404E+00_JPRB, 0.22167E+00_JPRB, 0.21949E+00_JPRB, 0.21749E+00_JPRB, 0.21567E+00_JPRB, 0.21416E+00_JPRB ,&
&  0.21236E+00_JPRB, 0.21059E+00_JPRB, 0.20896E+00_JPRB, 0.20782E+00_JPRB, 0.20659E+00_JPRB, 0.20552E+00_JPRB, 0.20458E+00_JPRB ,&
&  0.20394E+00_JPRB, 0.20338E+00_JPRB, 0.20288E+00_JPRB, 0.20263E+00_JPRB, 0.20243E+00_JPRB, 0.20224E+00_JPRB, 0.20203E+00_JPRB ,&
&  0.20183E+00_JPRB, 0.20168E+00_JPRB, 0.20154E+00_JPRB, 0.20147E+00_JPRB, 0.20152E+00_JPRB, 0.20138E+00_JPRB, 0.20121E+00_JPRB ,&
&  0.20113E+00_JPRB, 0.20134E+00_JPRB, 0.20151E+00_JPRB, 0.20181E+00_JPRB, 0.20227E+00_JPRB, 0.20293E+00_JPRB, 0.20365E+00_JPRB ,&
&  0.20437E+00_JPRB, 0.20521E+00_JPRB, 0.20614E+00_JPRB, 0.20726E+00_JPRB, 0.20846E+00_JPRB, 0.20965E+00_JPRB, 0.21080E+00_JPRB ,&
&  0.21203E+00_JPRB, 0.21316E+00_JPRB, 0.21422E+00_JPRB, 0.21501E+00_JPRB, 0.21605E+00_JPRB, 0.21711E+00_JPRB, 0.21833E+00_JPRB ,&
&  0.21971E+00_JPRB, 0.22151E+00_JPRB, 0.22394E+00_JPRB, 0.22681E+00_JPRB, 0.23030E+00_JPRB, 0.23294E+00_JPRB, 0.23567E+00_JPRB ,&
&  0.23768E+00_JPRB, 0.23919E+00_JPRB, 0.24030E+00_JPRB, 0.24174E+00_JPRB, 0.24334E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.28023E+00_JPRB, 0.28601E+00_JPRB, 0.29015E+00_JPRB, 0.29933E+00_JPRB, 0.31370E+00_JPRB, 0.40943E+00_JPRB, 0.40442E+00_JPRB ,&
&  0.40335E+00_JPRB, 0.36059E+00_JPRB, 0.35233E+00_JPRB, 0.34008E+00_JPRB, 0.32474E+00_JPRB, 0.30574E+00_JPRB, 0.29391E+00_JPRB ,&
&  0.28406E+00_JPRB, 0.27270E+00_JPRB, 0.26196E+00_JPRB, 0.25573E+00_JPRB, 0.25077E+00_JPRB, 0.24647E+00_JPRB, 0.24411E+00_JPRB ,&
&  0.24342E+00_JPRB, 0.24413E+00_JPRB, 0.24517E+00_JPRB, 0.24684E+00_JPRB, 0.25038E+00_JPRB, 0.25418E+00_JPRB, 0.25795E+00_JPRB ,&
&  0.26000E+00_JPRB, 0.26400E+00_JPRB, 0.26464E+00_JPRB, 0.26140E+00_JPRB, 0.25600E+00_JPRB, 0.24893E+00_JPRB, 0.24179E+00_JPRB ,&
&  0.23439E+00_JPRB, 0.22748E+00_JPRB, 0.22177E+00_JPRB, 0.21658E+00_JPRB, 0.21077E+00_JPRB, 0.20542E+00_JPRB, 0.20203E+00_JPRB ,&
&  0.19902E+00_JPRB, 0.19598E+00_JPRB, 0.19361E+00_JPRB, 0.19240E+00_JPRB, 0.19103E+00_JPRB, 0.19013E+00_JPRB, 0.18947E+00_JPRB ,&
&  0.18929E+00_JPRB, 0.18919E+00_JPRB, 0.18926E+00_JPRB, 0.18955E+00_JPRB, 0.18952E+00_JPRB, 0.18944E+00_JPRB, 0.18941E+00_JPRB ,&
&  0.18994E+00_JPRB, 0.18994E+00_JPRB, 0.18983E+00_JPRB, 0.18956E+00_JPRB, 0.19010E+00_JPRB, 0.19020E+00_JPRB, 0.19047E+00_JPRB ,&
&  0.19100E+00_JPRB, 0.19179E+00_JPRB, 0.19241E+00_JPRB, 0.19314E+00_JPRB, 0.19383E+00_JPRB, 0.19466E+00_JPRB, 0.19561E+00_JPRB ,&
&  0.19658E+00_JPRB, 0.19801E+00_JPRB, 0.19947E+00_JPRB, 0.20079E+00_JPRB, 0.20195E+00_JPRB, 0.20354E+00_JPRB, 0.20534E+00_JPRB ,&
&  0.20804E+00_JPRB, 0.21069E+00_JPRB, 0.21342E+00_JPRB, 0.21519E+00_JPRB, 0.21790E+00_JPRB, 0.22017E+00_JPRB, 0.22253E+00_JPRB ,&
&  0.22522E+00_JPRB, 0.22940E+00_JPRB, 0.23529E+00_JPRB, 0.24156E+00_JPRB, 0.24834E+00_JPRB, 0.25468E+00_JPRB, 0.25891E+00_JPRB ,&
&  0.26272E+00_JPRB, 0.26677E+00_JPRB, 0.27280E+00_JPRB, 0.27605E+00_JPRB, 0.27867E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.49787E+00_JPRB, 0.49998E+00_JPRB, 0.50229E+00_JPRB, 0.50613E+00_JPRB, 0.51019E+00_JPRB, 0.51439E+00_JPRB, 0.51819E+00_JPRB ,&
&  0.50010E+00_JPRB, 0.47695E+00_JPRB, 0.45804E+00_JPRB, 0.43560E+00_JPRB, 0.41513E+00_JPRB, 0.39861E+00_JPRB, 0.37854E+00_JPRB ,&
&  0.35861E+00_JPRB, 0.33866E+00_JPRB, 0.31607E+00_JPRB, 0.29756E+00_JPRB, 0.27936E+00_JPRB, 0.26460E+00_JPRB, 0.25536E+00_JPRB ,&
&  0.25051E+00_JPRB, 0.24929E+00_JPRB, 0.25092E+00_JPRB, 0.25442E+00_JPRB, 0.25763E+00_JPRB, 0.26251E+00_JPRB, 0.26916E+00_JPRB ,&
&  0.27833E+00_JPRB, 0.29246E+00_JPRB, 0.30525E+00_JPRB, 0.31005E+00_JPRB, 0.30685E+00_JPRB, 0.29873E+00_JPRB, 0.29175E+00_JPRB ,&
&  0.28319E+00_JPRB, 0.27091E+00_JPRB, 0.25626E+00_JPRB, 0.24183E+00_JPRB, 0.22842E+00_JPRB, 0.21864E+00_JPRB, 0.21063E+00_JPRB ,&
&  0.20394E+00_JPRB, 0.19806E+00_JPRB, 0.19360E+00_JPRB, 0.19057E+00_JPRB, 0.18798E+00_JPRB, 0.18586E+00_JPRB, 0.18389E+00_JPRB ,&
&  0.18285E+00_JPRB, 0.18174E+00_JPRB, 0.18106E+00_JPRB, 0.18066E+00_JPRB, 0.18020E+00_JPRB, 0.18008E+00_JPRB, 0.17998E+00_JPRB ,&
&  0.17965E+00_JPRB, 0.17903E+00_JPRB, 0.17866E+00_JPRB, 0.17840E+00_JPRB, 0.17874E+00_JPRB, 0.17899E+00_JPRB, 0.17944E+00_JPRB ,&
&  0.18017E+00_JPRB, 0.18181E+00_JPRB, 0.18287E+00_JPRB, 0.18386E+00_JPRB, 0.18437E+00_JPRB, 0.18479E+00_JPRB, 0.18535E+00_JPRB ,&
&  0.18630E+00_JPRB, 0.18747E+00_JPRB, 0.18857E+00_JPRB, 0.19001E+00_JPRB, 0.19150E+00_JPRB, 0.19354E+00_JPRB, 0.19572E+00_JPRB ,&
&  0.19830E+00_JPRB, 0.20151E+00_JPRB, 0.20552E+00_JPRB, 0.20925E+00_JPRB, 0.21480E+00_JPRB, 0.22037E+00_JPRB, 0.22654E+00_JPRB ,&
&  0.23280E+00_JPRB, 0.23952E+00_JPRB, 0.24819E+00_JPRB, 0.25841E+00_JPRB, 0.27102E+00_JPRB, 0.27978E+00_JPRB, 0.29163E+00_JPRB ,&
&  0.30963E+00_JPRB, 0.34774E+00_JPRB, 0.48261E+00_JPRB, 0.49604E+00_JPRB, 0.50039E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.51113E+00_JPRB, 0.52004E+00_JPRB, 0.53441E+00_JPRB, 0.54605E+00_JPRB, 0.57056E+00_JPRB, 0.58432E+00_JPRB, 0.63089E+00_JPRB ,&
&  0.58502E+00_JPRB, 0.50575E+00_JPRB, 0.47384E+00_JPRB, 0.43938E+00_JPRB, 0.39983E+00_JPRB, 0.37146E+00_JPRB, 0.35139E+00_JPRB ,&
&  0.33950E+00_JPRB, 0.32938E+00_JPRB, 0.31969E+00_JPRB, 0.30754E+00_JPRB, 0.29501E+00_JPRB, 0.28086E+00_JPRB, 0.27008E+00_JPRB ,&
&  0.26357E+00_JPRB, 0.26160E+00_JPRB, 0.26129E+00_JPRB, 0.26571E+00_JPRB, 0.27185E+00_JPRB, 0.28003E+00_JPRB, 0.28935E+00_JPRB ,&
&  0.31004E+00_JPRB, 0.31578E+00_JPRB, 0.31548E+00_JPRB, 0.31566E+00_JPRB, 0.31698E+00_JPRB, 0.31911E+00_JPRB, 0.30947E+00_JPRB ,&
&  0.31951E+00_JPRB, 0.34215E+00_JPRB, 0.34831E+00_JPRB, 0.33017E+00_JPRB, 0.27573E+00_JPRB, 0.24275E+00_JPRB, 0.22026E+00_JPRB ,&
&  0.21147E+00_JPRB, 0.20198E+00_JPRB, 0.19607E+00_JPRB, 0.19057E+00_JPRB, 0.18719E+00_JPRB, 0.18537E+00_JPRB, 0.18358E+00_JPRB ,&
&  0.18223E+00_JPRB, 0.18109E+00_JPRB, 0.17912E+00_JPRB, 0.17817E+00_JPRB, 0.17716E+00_JPRB, 0.17663E+00_JPRB, 0.17630E+00_JPRB ,&
&  0.17611E+00_JPRB, 0.17575E+00_JPRB, 0.17588E+00_JPRB, 0.17634E+00_JPRB, 0.17720E+00_JPRB, 0.17788E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.17925E+00_JPRB, 0.17991E+00_JPRB, 0.18000E+00_JPRB, 0.18035E+00_JPRB, 0.18068E+00_JPRB, 0.18167E+00_JPRB, 0.18208E+00_JPRB ,&
&  0.18255E+00_JPRB, 0.18289E+00_JPRB, 0.18284E+00_JPRB, 0.18333E+00_JPRB, 0.18416E+00_JPRB, 0.18588E+00_JPRB, 0.18789E+00_JPRB ,&
&  0.19049E+00_JPRB, 0.19350E+00_JPRB, 0.19686E+00_JPRB, 0.20019E+00_JPRB, 0.20459E+00_JPRB, 0.20964E+00_JPRB, 0.21604E+00_JPRB ,&
&  0.22305E+00_JPRB, 0.23155E+00_JPRB, 0.24280E+00_JPRB, 0.25669E+00_JPRB, 0.27352E+00_JPRB, 0.29167E+00_JPRB, 0.31303E+00_JPRB ,&
&  0.34875E+00_JPRB, 0.44976E+00_JPRB, 0.50990E+00_JPRB, 0.49588E+00_JPRB, 0.50560E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.58866E+00_JPRB, 0.60480E+00_JPRB, 0.63017E+00_JPRB, 0.62287E+00_JPRB, 0.64325E+00_JPRB, 0.62974E+00_JPRB, 0.60728E+00_JPRB ,&
&  0.55999E+00_JPRB, 0.50281E+00_JPRB, 0.45647E+00_JPRB, 0.40437E+00_JPRB, 0.36413E+00_JPRB, 0.33453E+00_JPRB, 0.32015E+00_JPRB ,&
&  0.31336E+00_JPRB, 0.31125E+00_JPRB, 0.31035E+00_JPRB, 0.30968E+00_JPRB, 0.30542E+00_JPRB, 0.30235E+00_JPRB, 0.29686E+00_JPRB ,&
&  0.29096E+00_JPRB, 0.28625E+00_JPRB, 0.28298E+00_JPRB, 0.28799E+00_JPRB, 0.29434E+00_JPRB, 0.30105E+00_JPRB, 0.30488E+00_JPRB ,&
&  0.30840E+00_JPRB, 0.30972E+00_JPRB, 0.31234E+00_JPRB, 0.31058E+00_JPRB, 0.32156E+00_JPRB, 0.34305E+00_JPRB, 0.36181E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.39512E+00_JPRB, 0.39132E+00_JPRB, 0.36788E+00_JPRB, 0.30240E+00_JPRB, 0.26817E+00_JPRB, 0.23854E+00_JPRB ,&
&  0.22006E+00_JPRB, 0.20639E+00_JPRB, 0.19735E+00_JPRB, 0.19216E+00_JPRB, 0.18818E+00_JPRB, 0.18585E+00_JPRB, 0.18334E+00_JPRB ,&
&  0.18092E+00_JPRB, 0.17935E+00_JPRB, 0.17767E+00_JPRB, 0.17685E+00_JPRB, 0.17595E+00_JPRB, 0.17478E+00_JPRB, 0.17382E+00_JPRB ,&
&  0.17387E+00_JPRB, 0.17361E+00_JPRB, 0.17461E+00_JPRB, 0.17551E+00_JPRB, 0.17734E+00_JPRB, 0.17890E+00_JPRB, 0.18101E+00_JPRB ,&
&  0.18270E+00_JPRB, 0.18403E+00_JPRB, 0.18384E+00_JPRB, 0.18327E+00_JPRB, 0.18237E+00_JPRB, 0.18179E+00_JPRB, 0.18084E+00_JPRB ,&
&  0.17972E+00_JPRB, 0.17980E+00_JPRB, 0.17992E+00_JPRB, 0.18018E+00_JPRB, 0.18116E+00_JPRB, 0.18262E+00_JPRB, 0.18495E+00_JPRB ,&
&  0.18801E+00_JPRB, 0.19124E+00_JPRB, 0.19394E+00_JPRB, 0.19675E+00_JPRB, 0.20039E+00_JPRB, 0.20460E+00_JPRB, 0.20989E+00_JPRB ,&
&  0.21481E+00_JPRB, 0.22288E+00_JPRB, 0.23386E+00_JPRB, 0.24511E+00_JPRB, 0.26120E+00_JPRB, 0.27802E+00_JPRB, 0.30624E+00_JPRB ,&
&  0.35603E+00_JPRB, 0.53079E+00_JPRB, 0.81527E+00_JPRB, 0.58791E+00_JPRB, 0.57507E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.62502E+00_JPRB, 0.63578E+00_JPRB, 0.64982E+00_JPRB, 0.65265E+00_JPRB, 0.66272E+00_JPRB, 0.66158E+00_JPRB, 0.64462E+00_JPRB ,&
&  0.61643E+00_JPRB, 0.59013E+00_JPRB, 0.57566E+00_JPRB, 0.55884E+00_JPRB, 0.53148E+00_JPRB, 0.47090E+00_JPRB, 0.44761E+00_JPRB ,&
&  0.42927E+00_JPRB, 0.42898E+00_JPRB, 0.42000E+00_JPRB, 0.42028E+00_JPRB, 0.40945E+00_JPRB, 0.39825E+00_JPRB, 0.38000E+00_JPRB ,&
&  0.36304E+00_JPRB, 0.34576E+00_JPRB, 0.33395E+00_JPRB, 0.32798E+00_JPRB, 0.32591E+00_JPRB, 0.32175E+00_JPRB, 0.31014E+00_JPRB ,&
&  0.30175E+00_JPRB, 0.30008E+00_JPRB, 0.30278E+00_JPRB, 0.31328E+00_JPRB, 0.35769E+00_JPRB, 0.41359E+00_JPRB, 0.41386E+00_JPRB ,&
&  0.43696E+00_JPRB, 0.45076E+00_JPRB, 0.46417E+00_JPRB, 0.45356E+00_JPRB, 0.42129E+00_JPRB, 0.38348E+00_JPRB, 0.32929E+00_JPRB ,&
&  0.27758E+00_JPRB, 0.24895E+00_JPRB, 0.22574E+00_JPRB, 0.20979E+00_JPRB, 0.19917E+00_JPRB, 0.19470E+00_JPRB, 0.18901E+00_JPRB ,&
&  0.18520E+00_JPRB, 0.18234E+00_JPRB, 0.18112E+00_JPRB, 0.18039E+00_JPRB, 0.17964E+00_JPRB, 0.17765E+00_JPRB, 0.17705E+00_JPRB ,&
&  0.17750E+00_JPRB, 0.17769E+00_JPRB, 0.17754E+00_JPRB, 0.17827E+00_JPRB, 0.17920E+00_JPRB, 0.18052E+00_JPRB, 0.18166E+00_JPRB ,&
&  0.18337E+00_JPRB, 0.18427E+00_JPRB, 0.18581E+00_JPRB, 0.18747E+00_JPRB, 0.18654E+00_JPRB, 0.18529E+00_JPRB, 0.18248E+00_JPRB ,&
&  0.17943E+00_JPRB, 0.17894E+00_JPRB, 0.17835E+00_JPRB, 0.17887E+00_JPRB, 0.18035E+00_JPRB, 0.18241E+00_JPRB, 0.18516E+00_JPRB ,&
&  0.18781E+00_JPRB, 0.19055E+00_JPRB, 0.19269E+00_JPRB, 0.19480E+00_JPRB, 0.19828E+00_JPRB, 0.20203E+00_JPRB, 0.20678E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.21728E+00_JPRB, 0.22501E+00_JPRB, 0.23501E+00_JPRB, 0.24902E+00_JPRB, 0.26510E+00_JPRB, 0.29003E+00_JPRB ,&
&  0.31410E+00_JPRB, 0.41553E+00_JPRB, 0.65259E+00_JPRB, 0.57938E+00_JPRB, 0.59752E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.70014E+00_JPRB, 0.71129E+00_JPRB, 0.72095E+00_JPRB, 0.72264E+00_JPRB, 0.72189E+00_JPRB, 0.71698E+00_JPRB, 0.70500E+00_JPRB ,&
&  0.68538E+00_JPRB, 0.65916E+00_JPRB, 0.63373E+00_JPRB, 0.61343E+00_JPRB, 0.60075E+00_JPRB, 0.60415E+00_JPRB, 0.61308E+00_JPRB ,&
&  0.63090E+00_JPRB, 0.65673E+00_JPRB, 0.66227E+00_JPRB, 0.65601E+00_JPRB, 0.62051E+00_JPRB, 0.58351E+00_JPRB, 0.54897E+00_JPRB ,&
&  0.51842E+00_JPRB, 0.48993E+00_JPRB, 0.47915E+00_JPRB, 0.46048E+00_JPRB, 0.44728E+00_JPRB, 0.43082E+00_JPRB, 0.40775E+00_JPRB ,&
&  0.36998E+00_JPRB, 0.33927E+00_JPRB, 0.32127E+00_JPRB, 0.33566E+00_JPRB, 0.42557E+00_JPRB, 0.48615E+00_JPRB, 0.49504E+00_JPRB ,&
&  0.52987E+00_JPRB, 0.55488E+00_JPRB, 0.55026E+00_JPRB, 0.53067E+00_JPRB, 0.50941E+00_JPRB, 0.48623E+00_JPRB, 0.46733E+00_JPRB ,&
&  0.44848E+00_JPRB, 0.42221E+00_JPRB, 0.37306E+00_JPRB, 0.30341E+00_JPRB, 0.26774E+00_JPRB, 0.25421E+00_JPRB, 0.23974E+00_JPRB ,&
&  0.22877E+00_JPRB, 0.21961E+00_JPRB, 0.21604E+00_JPRB, 0.21294E+00_JPRB, 0.21219E+00_JPRB, 0.20813E+00_JPRB, 0.20470E+00_JPRB ,&
&  0.20060E+00_JPRB, 0.19761E+00_JPRB, 0.19302E+00_JPRB, 0.18844E+00_JPRB, 0.18516E+00_JPRB, 0.18457E+00_JPRB, 0.18421E+00_JPRB ,&
&  0.18512E+00_JPRB, 0.18903E+00_JPRB, 0.20005E+00_JPRB, 0.20618E+00_JPRB, 0.20651E+00_JPRB, 0.20156E+00_JPRB, 0.19438E+00_JPRB ,&
&  0.18800E+00_JPRB, 0.18596E+00_JPRB, 0.18450E+00_JPRB, 0.18498E+00_JPRB, 0.18757E+00_JPRB, 0.19110E+00_JPRB, 0.19410E+00_JPRB ,&
&  0.19650E+00_JPRB, 0.19880E+00_JPRB, 0.20099E+00_JPRB, 0.20335E+00_JPRB, 0.20618E+00_JPRB, 0.20952E+00_JPRB, 0.21408E+00_JPRB ,&
&  0.21898E+00_JPRB, 0.22583E+00_JPRB, 0.23580E+00_JPRB, 0.25010E+00_JPRB, 0.26265E+00_JPRB, 0.27048E+00_JPRB, 0.28097E+00_JPRB ,&
&  0.29677E+00_JPRB, 0.34335E+00_JPRB, 0.47719E+00_JPRB, 0.70452E+00_JPRB, 0.73667E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.76051E+00_JPRB, 0.77839E+00_JPRB, 0.79929E+00_JPRB, 0.81034E+00_JPRB, 0.81132E+00_JPRB, 0.80840E+00_JPRB, 0.79979E+00_JPRB ,&
&  0.78495E+00_JPRB, 0.76702E+00_JPRB, 0.75385E+00_JPRB, 0.74500E+00_JPRB, 0.74119E+00_JPRB, 0.74429E+00_JPRB, 0.75002E+00_JPRB ,&
&  0.74971E+00_JPRB, 0.74591E+00_JPRB, 0.73409E+00_JPRB, 0.71871E+00_JPRB, 0.70248E+00_JPRB, 0.68985E+00_JPRB, 0.68006E+00_JPRB ,&
&  0.67038E+00_JPRB, 0.65564E+00_JPRB, 0.63265E+00_JPRB, 0.59740E+00_JPRB, 0.55582E+00_JPRB, 0.51775E+00_JPRB, 0.48899E+00_JPRB ,&
&  0.46495E+00_JPRB, 0.39756E+00_JPRB, 0.36016E+00_JPRB, 0.37286E+00_JPRB, 0.46672E+00_JPRB, 0.49678E+00_JPRB, 0.52068E+00_JPRB ,&
&  0.55809E+00_JPRB, 0.59782E+00_JPRB, 0.61673E+00_JPRB, 0.61732E+00_JPRB, 0.61192E+00_JPRB, 0.59956E+00_JPRB, 0.58017E+00_JPRB ,&
&  0.55544E+00_JPRB, 0.52504E+00_JPRB, 0.49920E+00_JPRB, 0.47049E+00_JPRB, 0.44079E+00_JPRB, 0.41881E+00_JPRB, 0.40347E+00_JPRB ,&
&  0.39286E+00_JPRB, 0.38336E+00_JPRB, 0.37590E+00_JPRB, 0.36622E+00_JPRB, 0.35704E+00_JPRB, 0.33600E+00_JPRB, 0.31438E+00_JPRB ,&
&  0.29064E+00_JPRB, 0.27333E+00_JPRB, 0.25466E+00_JPRB, 0.23648E+00_JPRB, 0.22152E+00_JPRB, 0.21181E+00_JPRB, 0.20476E+00_JPRB ,&
&  0.20246E+00_JPRB, 0.20793E+00_JPRB, 0.22612E+00_JPRB, 0.25906E+00_JPRB, 0.29813E+00_JPRB, 0.26720E+00_JPRB, 0.24620E+00_JPRB ,&
&  0.22588E+00_JPRB, 0.21389E+00_JPRB, 0.20940E+00_JPRB, 0.20886E+00_JPRB, 0.21013E+00_JPRB, 0.21247E+00_JPRB, 0.21482E+00_JPRB ,&
&  0.21791E+00_JPRB, 0.22078E+00_JPRB, 0.22384E+00_JPRB, 0.22603E+00_JPRB, 0.22747E+00_JPRB, 0.23012E+00_JPRB, 0.23472E+00_JPRB ,&
&  0.24173E+00_JPRB, 0.25340E+00_JPRB, 0.27296E+00_JPRB, 0.29839E+00_JPRB, 0.32459E+00_JPRB, 0.34848E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.48895E+00_JPRB, 0.60156E+00_JPRB, 0.65756E+00_JPRB, 0.71241E+00_JPRB, 0.73944E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.82066E+00_JPRB, 0.84681E+00_JPRB, 0.87591E+00_JPRB, 0.89258E+00_JPRB, 0.89331E+00_JPRB, 0.89008E+00_JPRB, 0.87509E+00_JPRB ,&
&  0.84414E+00_JPRB, 0.81856E+00_JPRB, 0.81412E+00_JPRB, 0.81788E+00_JPRB, 0.82960E+00_JPRB, 0.84829E+00_JPRB, 0.85024E+00_JPRB ,&
&  0.82433E+00_JPRB, 0.80538E+00_JPRB, 0.79601E+00_JPRB, 0.78944E+00_JPRB, 0.77300E+00_JPRB, 0.74880E+00_JPRB, 0.73311E+00_JPRB ,&
&  0.72879E+00_JPRB, 0.72899E+00_JPRB, 0.72014E+00_JPRB, 0.70616E+00_JPRB, 0.67470E+00_JPRB, 0.63597E+00_JPRB, 0.60583E+00_JPRB ,&
&  0.59598E+00_JPRB, 0.58795E+00_JPRB, 0.54700E+00_JPRB, 0.56462E+00_JPRB, 0.56827E+00_JPRB, 0.56855E+00_JPRB, 0.56640E+00_JPRB ,&
&  0.57929E+00_JPRB, 0.61280E+00_JPRB, 0.64571E+00_JPRB, 0.65549E+00_JPRB, 0.66508E+00_JPRB, 0.66709E+00_JPRB, 0.66889E+00_JPRB ,&
&  0.67368E+00_JPRB, 0.66576E+00_JPRB, 0.63620E+00_JPRB, 0.59236E+00_JPRB, 0.54907E+00_JPRB, 0.51537E+00_JPRB, 0.48666E+00_JPRB ,&
&  0.46678E+00_JPRB, 0.45434E+00_JPRB, 0.44465E+00_JPRB, 0.44121E+00_JPRB, 0.44865E+00_JPRB, 0.46344E+00_JPRB, 0.45832E+00_JPRB ,&
&  0.44031E+00_JPRB, 0.42393E+00_JPRB, 0.40206E+00_JPRB, 0.37240E+00_JPRB, 0.34239E+00_JPRB, 0.30340E+00_JPRB, 0.28356E+00_JPRB ,&
&  0.28787E+00_JPRB, 0.32677E+00_JPRB, 0.37817E+00_JPRB, 0.39698E+00_JPRB, 0.42395E+00_JPRB, 0.45003E+00_JPRB, 0.43946E+00_JPRB ,&
&  0.39313E+00_JPRB, 0.32337E+00_JPRB, 0.30049E+00_JPRB, 0.28838E+00_JPRB, 0.28158E+00_JPRB, 0.27662E+00_JPRB, 0.27591E+00_JPRB ,&
&  0.28006E+00_JPRB, 0.29142E+00_JPRB, 0.30066E+00_JPRB, 0.29736E+00_JPRB, 0.28698E+00_JPRB, 0.28037E+00_JPRB, 0.28080E+00_JPRB ,&
&  0.28948E+00_JPRB, 0.31724E+00_JPRB, 0.38084E+00_JPRB, 0.49749E+00_JPRB, 0.53252E+00_JPRB, 0.56508E+00_JPRB, 0.60288E+00_JPRB ,&
&  0.63910E+00_JPRB, 0.67206E+00_JPRB, 0.70032E+00_JPRB, 0.73522E+00_JPRB, 0.78197E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.81555E+00_JPRB, 0.83683E+00_JPRB, 0.85539E+00_JPRB, 0.86799E+00_JPRB, 0.86964E+00_JPRB, 0.86283E+00_JPRB, 0.84709E+00_JPRB ,&
&  0.81723E+00_JPRB, 0.80028E+00_JPRB, 0.80931E+00_JPRB, 0.82399E+00_JPRB, 0.83299E+00_JPRB, 0.83294E+00_JPRB, 0.80153E+00_JPRB ,&
&  0.74764E+00_JPRB, 0.72729E+00_JPRB, 0.72436E+00_JPRB, 0.73408E+00_JPRB, 0.75257E+00_JPRB, 0.75994E+00_JPRB, 0.74834E+00_JPRB ,&
&  0.73641E+00_JPRB, 0.73833E+00_JPRB, 0.74132E+00_JPRB, 0.74827E+00_JPRB, 0.73746E+00_JPRB, 0.71654E+00_JPRB, 0.70775E+00_JPRB ,&
&  0.70549E+00_JPRB, 0.69856E+00_JPRB, 0.69129E+00_JPRB, 0.64251E+00_JPRB, 0.66770E+00_JPRB, 0.65622E+00_JPRB, 0.63821E+00_JPRB ,&
&  0.62193E+00_JPRB, 0.62480E+00_JPRB, 0.66637E+00_JPRB, 0.71262E+00_JPRB, 0.72405E+00_JPRB, 0.72700E+00_JPRB, 0.72250E+00_JPRB ,&
&  0.71930E+00_JPRB, 0.71970E+00_JPRB, 0.72105E+00_JPRB, 0.71291E+00_JPRB, 0.69250E+00_JPRB, 0.66623E+00_JPRB, 0.63738E+00_JPRB ,&
&  0.61978E+00_JPRB, 0.59459E+00_JPRB, 0.55090E+00_JPRB, 0.53054E+00_JPRB, 0.52846E+00_JPRB, 0.53629E+00_JPRB, 0.54008E+00_JPRB ,&
&  0.53700E+00_JPRB, 0.52969E+00_JPRB, 0.53403E+00_JPRB, 0.53142E+00_JPRB, 0.50976E+00_JPRB, 0.49675E+00_JPRB, 0.49081E+00_JPRB ,&
&  0.47916E+00_JPRB, 0.47762E+00_JPRB, 0.48189E+00_JPRB, 0.49314E+00_JPRB, 0.50307E+00_JPRB, 0.51642E+00_JPRB, 0.52168E+00_JPRB ,&
&  0.52362E+00_JPRB, 0.52028E+00_JPRB, 0.51467E+00_JPRB, 0.50290E+00_JPRB, 0.48470E+00_JPRB, 0.47777E+00_JPRB, 0.47919E+00_JPRB ,&
&  0.48785E+00_JPRB, 0.49347E+00_JPRB, 0.49260E+00_JPRB, 0.49130E+00_JPRB, 0.47973E+00_JPRB, 0.46706E+00_JPRB, 0.45722E+00_JPRB ,&
&  0.45246E+00_JPRB, 0.45484E+00_JPRB, 0.46950E+00_JPRB, 0.49387E+00_JPRB, 0.52535E+00_JPRB, 0.56340E+00_JPRB, 0.60789E+00_JPRB ,&
&  0.63963E+00_JPRB, 0.65854E+00_JPRB, 0.68295E+00_JPRB, 0.73018E+00_JPRB, 0.77547E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.77794E+00_JPRB, 0.78615E+00_JPRB, 0.79589E+00_JPRB, 0.80890E+00_JPRB, 0.82061E+00_JPRB, 0.81730E+00_JPRB, 0.80491E+00_JPRB ,&
&  0.78132E+00_JPRB, 0.77267E+00_JPRB, 0.77749E+00_JPRB, 0.77995E+00_JPRB, 0.78167E+00_JPRB, 0.77932E+00_JPRB, 0.76329E+00_JPRB ,&
&  0.72654E+00_JPRB, 0.69707E+00_JPRB, 0.68293E+00_JPRB, 0.67559E+00_JPRB, 0.67324E+00_JPRB, 0.68040E+00_JPRB, 0.68926E+00_JPRB ,&
&  0.70679E+00_JPRB, 0.71754E+00_JPRB, 0.72999E+00_JPRB, 0.73955E+00_JPRB, 0.74225E+00_JPRB, 0.74618E+00_JPRB, 0.74746E+00_JPRB ,&
&  0.73374E+00_JPRB, 0.73436E+00_JPRB, 0.73640E+00_JPRB, 0.72727E+00_JPRB, 0.70658E+00_JPRB, 0.68574E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.64270E+00_JPRB, 0.63434E+00_JPRB, 0.66008E+00_JPRB, 0.72176E+00_JPRB, 0.76495E+00_JPRB, 0.77309E+00_JPRB, 0.76552E+00_JPRB ,&
&  0.76779E+00_JPRB, 0.77524E+00_JPRB, 0.77870E+00_JPRB, 0.76711E+00_JPRB, 0.74740E+00_JPRB, 0.73011E+00_JPRB, 0.71343E+00_JPRB ,&
&  0.70753E+00_JPRB, 0.70623E+00_JPRB, 0.68830E+00_JPRB, 0.65572E+00_JPRB, 0.62802E+00_JPRB, 0.60849E+00_JPRB, 0.59833E+00_JPRB ,&
&  0.59238E+00_JPRB, 0.58321E+00_JPRB, 0.57778E+00_JPRB, 0.58304E+00_JPRB, 0.56344E+00_JPRB, 0.54550E+00_JPRB, 0.54918E+00_JPRB ,&
&  0.56537E+00_JPRB, 0.57043E+00_JPRB, 0.57915E+00_JPRB, 0.58167E+00_JPRB, 0.57795E+00_JPRB, 0.57361E+00_JPRB, 0.57440E+00_JPRB ,&
&  0.58255E+00_JPRB, 0.59184E+00_JPRB, 0.59864E+00_JPRB, 0.60346E+00_JPRB, 0.60845E+00_JPRB, 0.59688E+00_JPRB, 0.57589E+00_JPRB ,&
&  0.55815E+00_JPRB, 0.55458E+00_JPRB, 0.55885E+00_JPRB, 0.55804E+00_JPRB, 0.55633E+00_JPRB, 0.55634E+00_JPRB, 0.55473E+00_JPRB ,&
&  0.55168E+00_JPRB, 0.54401E+00_JPRB, 0.53500E+00_JPRB, 0.53539E+00_JPRB, 0.54698E+00_JPRB, 0.56930E+00_JPRB, 0.60393E+00_JPRB ,&
&  0.65584E+00_JPRB, 0.69524E+00_JPRB, 0.73281E+00_JPRB, 0.74771E+00_JPRB, 0.75070E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.74794E+00_JPRB, 0.75611E+00_JPRB, 0.76197E+00_JPRB, 0.76611E+00_JPRB, 0.77448E+00_JPRB, 0.78500E+00_JPRB, 0.78470E+00_JPRB ,&
&  0.77003E+00_JPRB, 0.75646E+00_JPRB, 0.74708E+00_JPRB, 0.74446E+00_JPRB, 0.74612E+00_JPRB, 0.74826E+00_JPRB, 0.74252E+00_JPRB ,&
&  0.72210E+00_JPRB, 0.70018E+00_JPRB, 0.68671E+00_JPRB, 0.67328E+00_JPRB, 0.66505E+00_JPRB, 0.65989E+00_JPRB, 0.66702E+00_JPRB ,&
&  0.67753E+00_JPRB, 0.69393E+00_JPRB, 0.70741E+00_JPRB, 0.71725E+00_JPRB, 0.72139E+00_JPRB, 0.71037E+00_JPRB, 0.71205E+00_JPRB ,&
&  0.71956E+00_JPRB, 0.72216E+00_JPRB, 0.73770E+00_JPRB, 0.73760E+00_JPRB, 0.72491E+00_JPRB, 0.71498E+00_JPRB, 0.70485E+00_JPRB ,&
&  0.70221E+00_JPRB, 0.71365E+00_JPRB, 0.73919E+00_JPRB, 0.77580E+00_JPRB, 0.79669E+00_JPRB, 0.80455E+00_JPRB, 0.80181E+00_JPRB ,&
&  0.80503E+00_JPRB, 0.81717E+00_JPRB, 0.82393E+00_JPRB, 0.81568E+00_JPRB, 0.79733E+00_JPRB, 0.78228E+00_JPRB, 0.76885E+00_JPRB ,&
&  0.75867E+00_JPRB, 0.74634E+00_JPRB, 0.73785E+00_JPRB, 0.72755E+00_JPRB, 0.70562E+00_JPRB, 0.67974E+00_JPRB, 0.65738E+00_JPRB ,&
&  0.64093E+00_JPRB, 0.62497E+00_JPRB, 0.60539E+00_JPRB, 0.59959E+00_JPRB, 0.60402E+00_JPRB, 0.58109E+00_JPRB, 0.55598E+00_JPRB ,&
&  0.55920E+00_JPRB, 0.58988E+00_JPRB, 0.61373E+00_JPRB, 0.62293E+00_JPRB, 0.62315E+00_JPRB, 0.62695E+00_JPRB, 0.62990E+00_JPRB ,&
&  0.63589E+00_JPRB, 0.64401E+00_JPRB, 0.65013E+00_JPRB, 0.66089E+00_JPRB, 0.67219E+00_JPRB, 0.67070E+00_JPRB, 0.65684E+00_JPRB ,&
&  0.64144E+00_JPRB, 0.63246E+00_JPRB, 0.62838E+00_JPRB, 0.63026E+00_JPRB, 0.63342E+00_JPRB, 0.63117E+00_JPRB, 0.62528E+00_JPRB ,&
&  0.61369E+00_JPRB, 0.59520E+00_JPRB, 0.57947E+00_JPRB, 0.57109E+00_JPRB, 0.57114E+00_JPRB, 0.57799E+00_JPRB, 0.59416E+00_JPRB ,&
&  0.62543E+00_JPRB, 0.67299E+00_JPRB, 0.70774E+00_JPRB, 0.72673E+00_JPRB, 0.73107E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.72085E+00_JPRB, 0.72714E+00_JPRB, 0.73626E+00_JPRB, 0.74943E+00_JPRB, 0.76297E+00_JPRB, 0.77159E+00_JPRB, 0.77086E+00_JPRB ,&
&  0.75979E+00_JPRB, 0.74657E+00_JPRB, 0.74229E+00_JPRB, 0.74181E+00_JPRB, 0.73896E+00_JPRB, 0.73773E+00_JPRB, 0.73350E+00_JPRB ,&
&  0.72295E+00_JPRB, 0.70661E+00_JPRB, 0.69901E+00_JPRB, 0.68554E+00_JPRB, 0.67277E+00_JPRB, 0.66242E+00_JPRB, 0.66281E+00_JPRB ,&
&  0.67180E+00_JPRB, 0.68208E+00_JPRB, 0.69475E+00_JPRB, 0.70935E+00_JPRB, 0.71764E+00_JPRB, 0.71684E+00_JPRB, 0.71417E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.70976E+00_JPRB, 0.72130E+00_JPRB, 0.73076E+00_JPRB, 0.73712E+00_JPRB, 0.74851E+00_JPRB, 0.75774E+00_JPRB ,&
&  0.76519E+00_JPRB, 0.77627E+00_JPRB, 0.79504E+00_JPRB, 0.80900E+00_JPRB, 0.81898E+00_JPRB, 0.82957E+00_JPRB, 0.83357E+00_JPRB ,&
&  0.84005E+00_JPRB, 0.85131E+00_JPRB, 0.85377E+00_JPRB, 0.84493E+00_JPRB, 0.83056E+00_JPRB, 0.82107E+00_JPRB, 0.81544E+00_JPRB ,&
&  0.80563E+00_JPRB, 0.78591E+00_JPRB, 0.77103E+00_JPRB, 0.75930E+00_JPRB, 0.74040E+00_JPRB, 0.72027E+00_JPRB, 0.69684E+00_JPRB ,&
&  0.67018E+00_JPRB, 0.65042E+00_JPRB, 0.63512E+00_JPRB, 0.62135E+00_JPRB, 0.61820E+00_JPRB, 0.60974E+00_JPRB, 0.60297E+00_JPRB ,&
&  0.60724E+00_JPRB, 0.62492E+00_JPRB, 0.63795E+00_JPRB, 0.64194E+00_JPRB, 0.64383E+00_JPRB, 0.65903E+00_JPRB, 0.67764E+00_JPRB ,&
&  0.69207E+00_JPRB, 0.69786E+00_JPRB, 0.69345E+00_JPRB, 0.69535E+00_JPRB, 0.70013E+00_JPRB, 0.70432E+00_JPRB, 0.69847E+00_JPRB ,&
&  0.69003E+00_JPRB, 0.68446E+00_JPRB, 0.68326E+00_JPRB, 0.67969E+00_JPRB, 0.68118E+00_JPRB, 0.68103E+00_JPRB, 0.67821E+00_JPRB ,&
&  0.67240E+00_JPRB, 0.66281E+00_JPRB, 0.65062E+00_JPRB, 0.64379E+00_JPRB, 0.63384E+00_JPRB, 0.61862E+00_JPRB, 0.61992E+00_JPRB ,&
&  0.63149E+00_JPRB, 0.64993E+00_JPRB, 0.66704E+00_JPRB, 0.68699E+00_JPRB, 0.71012E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.70041E+00_JPRB, 0.71289E+00_JPRB, 0.72839E+00_JPRB, 0.74416E+00_JPRB, 0.75389E+00_JPRB, 0.75974E+00_JPRB, 0.75734E+00_JPRB ,&
&  0.74649E+00_JPRB, 0.73574E+00_JPRB, 0.73728E+00_JPRB, 0.73716E+00_JPRB, 0.73629E+00_JPRB, 0.73479E+00_JPRB, 0.72854E+00_JPRB ,&
&  0.71959E+00_JPRB, 0.71210E+00_JPRB, 0.70910E+00_JPRB, 0.70636E+00_JPRB, 0.69674E+00_JPRB, 0.68941E+00_JPRB, 0.68156E+00_JPRB ,&
&  0.68180E+00_JPRB, 0.68485E+00_JPRB, 0.68830E+00_JPRB, 0.68926E+00_JPRB, 0.69798E+00_JPRB, 0.70890E+00_JPRB, 0.71383E+00_JPRB ,&
&  0.70880E+00_JPRB, 0.71543E+00_JPRB, 0.63420E+00_JPRB, 0.70427E+00_JPRB, 0.72980E+00_JPRB, 0.74484E+00_JPRB, 0.75977E+00_JPRB ,&
&  0.76960E+00_JPRB, 0.78142E+00_JPRB, 0.79056E+00_JPRB, 0.79570E+00_JPRB, 0.81292E+00_JPRB, 0.82724E+00_JPRB, 0.83099E+00_JPRB ,&
&  0.83215E+00_JPRB, 0.83123E+00_JPRB, 0.83260E+00_JPRB, 0.83246E+00_JPRB, 0.83633E+00_JPRB, 0.83549E+00_JPRB, 0.82873E+00_JPRB ,&
&  0.82001E+00_JPRB, 0.80796E+00_JPRB, 0.79055E+00_JPRB, 0.77070E+00_JPRB, 0.73950E+00_JPRB, 0.71233E+00_JPRB, 0.69516E+00_JPRB ,&
&  0.68086E+00_JPRB, 0.67670E+00_JPRB, 0.67144E+00_JPRB, 0.66591E+00_JPRB, 0.66373E+00_JPRB, 0.66594E+00_JPRB, 0.66452E+00_JPRB ,&
&  0.67070E+00_JPRB, 0.67899E+00_JPRB, 0.68291E+00_JPRB, 0.68981E+00_JPRB, 0.69890E+00_JPRB, 0.71244E+00_JPRB, 0.72518E+00_JPRB ,&
&  0.73714E+00_JPRB, 0.74513E+00_JPRB, 0.74599E+00_JPRB, 0.74576E+00_JPRB, 0.74348E+00_JPRB, 0.74248E+00_JPRB, 0.73553E+00_JPRB ,&
&  0.72992E+00_JPRB, 0.72699E+00_JPRB, 0.72080E+00_JPRB, 0.70911E+00_JPRB, 0.70219E+00_JPRB, 0.69636E+00_JPRB, 0.69506E+00_JPRB ,&
&  0.69616E+00_JPRB, 0.69711E+00_JPRB, 0.69437E+00_JPRB, 0.69518E+00_JPRB, 0.68907E+00_JPRB, 0.66921E+00_JPRB, 0.65383E+00_JPRB ,&
&  0.65332E+00_JPRB, 0.65877E+00_JPRB, 0.66385E+00_JPRB, 0.67319E+00_JPRB, 0.68815E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.73981E+00_JPRB, 0.74548E+00_JPRB, 0.75438E+00_JPRB, 0.76448E+00_JPRB, 0.76758E+00_JPRB, 0.76676E+00_JPRB, 0.76419E+00_JPRB ,&
&  0.76478E+00_JPRB, 0.75662E+00_JPRB, 0.75064E+00_JPRB, 0.74724E+00_JPRB, 0.74683E+00_JPRB, 0.74407E+00_JPRB, 0.73571E+00_JPRB ,&
&  0.72885E+00_JPRB, 0.72664E+00_JPRB, 0.73175E+00_JPRB, 0.73373E+00_JPRB, 0.71902E+00_JPRB, 0.71446E+00_JPRB, 0.70700E+00_JPRB ,&
&  0.70607E+00_JPRB, 0.70680E+00_JPRB, 0.71035E+00_JPRB, 0.71720E+00_JPRB, 0.73422E+00_JPRB, 0.74645E+00_JPRB, 0.75320E+00_JPRB ,&
&  0.75238E+00_JPRB, 0.67335E+00_JPRB, 0.70126E+00_JPRB, 0.71413E+00_JPRB, 0.73701E+00_JPRB, 0.75999E+00_JPRB, 0.78092E+00_JPRB ,&
&  0.80894E+00_JPRB, 0.82576E+00_JPRB, 0.83418E+00_JPRB, 0.83655E+00_JPRB, 0.83852E+00_JPRB, 0.83819E+00_JPRB, 0.83008E+00_JPRB ,&
&  0.82375E+00_JPRB, 0.82959E+00_JPRB, 0.84481E+00_JPRB, 0.85189E+00_JPRB, 0.85477E+00_JPRB, 0.85574E+00_JPRB, 0.85116E+00_JPRB ,&
&  0.84303E+00_JPRB, 0.82807E+00_JPRB, 0.80426E+00_JPRB, 0.77622E+00_JPRB, 0.74475E+00_JPRB, 0.72338E+00_JPRB, 0.71156E+00_JPRB ,&
&  0.70364E+00_JPRB, 0.70331E+00_JPRB, 0.70340E+00_JPRB, 0.70418E+00_JPRB, 0.70690E+00_JPRB, 0.71461E+00_JPRB, 0.71722E+00_JPRB ,&
&  0.72429E+00_JPRB, 0.73138E+00_JPRB, 0.74306E+00_JPRB, 0.76212E+00_JPRB, 0.78168E+00_JPRB, 0.79270E+00_JPRB, 0.79336E+00_JPRB ,&
&  0.79883E+00_JPRB, 0.80119E+00_JPRB, 0.79723E+00_JPRB, 0.79698E+00_JPRB, 0.79322E+00_JPRB, 0.79451E+00_JPRB, 0.79172E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.77699E+00_JPRB, 0.76958E+00_JPRB, 0.75488E+00_JPRB, 0.74233E+00_JPRB, 0.73346E+00_JPRB, 0.72608E+00_JPRB ,&
&  0.72545E+00_JPRB, 0.72859E+00_JPRB, 0.73353E+00_JPRB, 0.72851E+00_JPRB, 0.71455E+00_JPRB, 0.70643E+00_JPRB, 0.69672E+00_JPRB ,&
&  0.69022E+00_JPRB, 0.69288E+00_JPRB, 0.69380E+00_JPRB, 0.70240E+00_JPRB, 0.72225E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.77191E+00_JPRB, 0.77567E+00_JPRB, 0.78018E+00_JPRB, 0.78083E+00_JPRB, 0.77706E+00_JPRB, 0.77484E+00_JPRB, 0.77667E+00_JPRB ,&
&  0.77788E+00_JPRB, 0.77005E+00_JPRB, 0.76211E+00_JPRB, 0.75650E+00_JPRB, 0.75485E+00_JPRB, 0.74746E+00_JPRB, 0.73597E+00_JPRB ,&
&  0.72792E+00_JPRB, 0.72784E+00_JPRB, 0.73409E+00_JPRB, 0.73797E+00_JPRB, 0.73031E+00_JPRB, 0.72192E+00_JPRB, 0.71942E+00_JPRB ,&
&  0.72092E+00_JPRB, 0.72754E+00_JPRB, 0.73836E+00_JPRB, 0.74520E+00_JPRB, 0.75469E+00_JPRB, 0.76954E+00_JPRB, 0.77156E+00_JPRB ,&
&  0.77083E+00_JPRB, 0.63714E+00_JPRB, 0.67461E+00_JPRB, 0.68909E+00_JPRB, 0.71935E+00_JPRB, 0.76551E+00_JPRB, 0.80482E+00_JPRB ,&
&  0.84307E+00_JPRB, 0.86715E+00_JPRB, 0.87918E+00_JPRB, 0.87449E+00_JPRB, 0.85987E+00_JPRB, 0.84474E+00_JPRB, 0.83912E+00_JPRB ,&
&  0.83574E+00_JPRB, 0.83945E+00_JPRB, 0.85036E+00_JPRB, 0.85773E+00_JPRB, 0.85911E+00_JPRB, 0.85779E+00_JPRB, 0.85808E+00_JPRB ,&
&  0.85349E+00_JPRB, 0.83696E+00_JPRB, 0.81459E+00_JPRB, 0.78801E+00_JPRB, 0.76781E+00_JPRB, 0.75912E+00_JPRB, 0.74708E+00_JPRB ,&
&  0.73003E+00_JPRB, 0.71931E+00_JPRB, 0.71724E+00_JPRB, 0.71959E+00_JPRB, 0.72762E+00_JPRB, 0.73663E+00_JPRB, 0.74460E+00_JPRB ,&
&  0.75409E+00_JPRB, 0.76920E+00_JPRB, 0.78545E+00_JPRB, 0.80110E+00_JPRB, 0.81758E+00_JPRB, 0.83334E+00_JPRB, 0.84578E+00_JPRB ,&
&  0.85408E+00_JPRB, 0.85234E+00_JPRB, 0.84068E+00_JPRB, 0.83136E+00_JPRB, 0.82743E+00_JPRB, 0.83129E+00_JPRB, 0.83167E+00_JPRB ,&
&  0.82376E+00_JPRB, 0.81749E+00_JPRB, 0.81512E+00_JPRB, 0.80536E+00_JPRB, 0.79441E+00_JPRB, 0.78464E+00_JPRB, 0.77439E+00_JPRB ,&
&  0.76606E+00_JPRB, 0.76250E+00_JPRB, 0.75927E+00_JPRB, 0.75322E+00_JPRB, 0.74460E+00_JPRB, 0.74152E+00_JPRB, 0.73914E+00_JPRB ,&
&  0.73683E+00_JPRB, 0.73346E+00_JPRB, 0.73534E+00_JPRB, 0.74612E+00_JPRB, 0.76143E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.78872E+00_JPRB, 0.79562E+00_JPRB, 0.79991E+00_JPRB, 0.80461E+00_JPRB, 0.80502E+00_JPRB, 0.80335E+00_JPRB, 0.80727E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.80683E+00_JPRB, 0.79888E+00_JPRB, 0.79663E+00_JPRB, 0.78824E+00_JPRB, 0.77555E+00_JPRB, 0.76576E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.74929E+00_JPRB, 0.74718E+00_JPRB, 0.74492E+00_JPRB, 0.73945E+00_JPRB, 0.73223E+00_JPRB, 0.73649E+00_JPRB ,&
&  0.74363E+00_JPRB, 0.75114E+00_JPRB, 0.75371E+00_JPRB, 0.75855E+00_JPRB, 0.76543E+00_JPRB, 0.77314E+00_JPRB, 0.78670E+00_JPRB ,&
&  0.67767E+00_JPRB, 0.55617E+00_JPRB, 0.60726E+00_JPRB, 0.65922E+00_JPRB, 0.70870E+00_JPRB, 0.78188E+00_JPRB, 0.83640E+00_JPRB ,&
&  0.88191E+00_JPRB, 0.92222E+00_JPRB, 0.93158E+00_JPRB, 0.90921E+00_JPRB, 0.88477E+00_JPRB, 0.87513E+00_JPRB, 0.87204E+00_JPRB ,&
&  0.86658E+00_JPRB, 0.86385E+00_JPRB, 0.86576E+00_JPRB, 0.86720E+00_JPRB, 0.86853E+00_JPRB, 0.87181E+00_JPRB, 0.87182E+00_JPRB ,&
&  0.86813E+00_JPRB, 0.85489E+00_JPRB, 0.84219E+00_JPRB, 0.82319E+00_JPRB, 0.80696E+00_JPRB, 0.80237E+00_JPRB, 0.79359E+00_JPRB ,&
&  0.77854E+00_JPRB, 0.76862E+00_JPRB, 0.76639E+00_JPRB, 0.76828E+00_JPRB, 0.77611E+00_JPRB, 0.78588E+00_JPRB, 0.80336E+00_JPRB ,&
&  0.82390E+00_JPRB, 0.83925E+00_JPRB, 0.84997E+00_JPRB, 0.86255E+00_JPRB, 0.86954E+00_JPRB, 0.87882E+00_JPRB, 0.89569E+00_JPRB ,&
&  0.90956E+00_JPRB, 0.90032E+00_JPRB, 0.89131E+00_JPRB, 0.88185E+00_JPRB, 0.87812E+00_JPRB, 0.87515E+00_JPRB, 0.86581E+00_JPRB ,&
&  0.85719E+00_JPRB, 0.85222E+00_JPRB, 0.85002E+00_JPRB, 0.85428E+00_JPRB, 0.85295E+00_JPRB, 0.84799E+00_JPRB, 0.84627E+00_JPRB ,&
&  0.84542E+00_JPRB, 0.83823E+00_JPRB, 0.82926E+00_JPRB, 0.82460E+00_JPRB, 0.81669E+00_JPRB, 0.81236E+00_JPRB, 0.80706E+00_JPRB ,&
&  0.80245E+00_JPRB, 0.78755E+00_JPRB, 0.79213E+00_JPRB, 0.78621E+00_JPRB, 0.78341E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.78827E+00_JPRB, 0.81120E+00_JPRB, 0.81663E+00_JPRB, 0.82901E+00_JPRB, 0.83642E+00_JPRB, 0.84112E+00_JPRB, 0.84926E+00_JPRB ,&
&  0.85473E+00_JPRB, 0.85098E+00_JPRB, 0.84445E+00_JPRB, 0.83121E+00_JPRB, 0.80928E+00_JPRB, 0.78572E+00_JPRB, 0.76694E+00_JPRB ,&
&  0.75448E+00_JPRB, 0.75445E+00_JPRB, 0.74873E+00_JPRB, 0.74057E+00_JPRB, 0.73590E+00_JPRB, 0.73374E+00_JPRB, 0.73681E+00_JPRB ,&
&  0.74550E+00_JPRB, 0.75029E+00_JPRB, 0.74607E+00_JPRB, 0.74578E+00_JPRB, 0.75551E+00_JPRB, 0.77259E+00_JPRB, 0.78115E+00_JPRB ,&
&  0.78426E+00_JPRB, 0.59170E+00_JPRB, 0.63309E+00_JPRB, 0.68647E+00_JPRB, 0.73585E+00_JPRB, 0.80164E+00_JPRB, 0.87722E+00_JPRB ,&
&  0.92989E+00_JPRB, 0.96184E+00_JPRB, 0.96376E+00_JPRB, 0.95295E+00_JPRB, 0.93177E+00_JPRB, 0.91192E+00_JPRB, 0.88945E+00_JPRB ,&
&  0.86993E+00_JPRB, 0.86304E+00_JPRB, 0.86553E+00_JPRB, 0.86771E+00_JPRB, 0.86717E+00_JPRB, 0.86851E+00_JPRB, 0.86083E+00_JPRB ,&
&  0.85728E+00_JPRB, 0.85264E+00_JPRB, 0.84944E+00_JPRB, 0.84814E+00_JPRB, 0.84601E+00_JPRB, 0.84443E+00_JPRB, 0.83843E+00_JPRB ,&
&  0.82307E+00_JPRB, 0.81063E+00_JPRB, 0.80038E+00_JPRB, 0.79176E+00_JPRB, 0.79420E+00_JPRB, 0.80587E+00_JPRB, 0.82538E+00_JPRB ,&
&  0.84701E+00_JPRB, 0.85213E+00_JPRB, 0.85894E+00_JPRB, 0.86703E+00_JPRB, 0.87891E+00_JPRB, 0.89077E+00_JPRB, 0.90537E+00_JPRB ,&
&  0.91013E+00_JPRB, 0.90328E+00_JPRB, 0.89737E+00_JPRB, 0.88646E+00_JPRB, 0.88491E+00_JPRB, 0.87045E+00_JPRB, 0.85309E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.85165E+00_JPRB, 0.85335E+00_JPRB, 0.86345E+00_JPRB, 0.87077E+00_JPRB, 0.87169E+00_JPRB, 0.87961E+00_JPRB ,&
&  0.89157E+00_JPRB, 0.90030E+00_JPRB, 0.89821E+00_JPRB, 0.90030E+00_JPRB, 0.89298E+00_JPRB, 0.88715E+00_JPRB, 0.86872E+00_JPRB ,&
&  0.84438E+00_JPRB, 0.82724E+00_JPRB, 0.64635E+00_JPRB, 0.73227E+00_JPRB, 0.75745E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.82947E+00_JPRB, 0.86604E+00_JPRB, 0.88251E+00_JPRB, 0.88355E+00_JPRB, 0.88673E+00_JPRB, 0.88638E+00_JPRB, 0.88148E+00_JPRB ,&
&  0.87891E+00_JPRB, 0.87380E+00_JPRB, 0.86097E+00_JPRB, 0.84147E+00_JPRB, 0.82182E+00_JPRB, 0.80118E+00_JPRB, 0.78620E+00_JPRB ,&
&  0.78395E+00_JPRB, 0.78410E+00_JPRB, 0.77456E+00_JPRB, 0.76540E+00_JPRB, 0.75616E+00_JPRB, 0.74878E+00_JPRB, 0.74666E+00_JPRB ,&
&  0.74330E+00_JPRB, 0.74257E+00_JPRB, 0.74898E+00_JPRB, 0.75722E+00_JPRB, 0.76756E+00_JPRB, 0.77886E+00_JPRB, 0.79388E+00_JPRB ,&
&  0.79566E+00_JPRB, 0.61147E+00_JPRB, 0.64326E+00_JPRB, 0.74417E+00_JPRB, 0.81487E+00_JPRB, 0.90375E+00_JPRB, 0.98011E+00_JPRB ,&
&  0.10052E+01_JPRB, 0.10286E+01_JPRB, 0.10191E+01_JPRB, 0.10014E+01_JPRB, 0.99566E+00_JPRB, 0.97514E+00_JPRB, 0.93461E+00_JPRB ,&
&  0.90389E+00_JPRB, 0.89979E+00_JPRB, 0.90958E+00_JPRB, 0.91502E+00_JPRB, 0.91055E+00_JPRB, 0.90084E+00_JPRB, 0.89425E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.87829E+00_JPRB, 0.86883E+00_JPRB, 0.85660E+00_JPRB, 0.85898E+00_JPRB, 0.87351E+00_JPRB, 0.88293E+00_JPRB ,&
&  0.87801E+00_JPRB, 0.85973E+00_JPRB, 0.84064E+00_JPRB, 0.83018E+00_JPRB, 0.83219E+00_JPRB, 0.84952E+00_JPRB, 0.86943E+00_JPRB ,&
&  0.88036E+00_JPRB, 0.88552E+00_JPRB, 0.89442E+00_JPRB, 0.89618E+00_JPRB, 0.90182E+00_JPRB, 0.90807E+00_JPRB, 0.91907E+00_JPRB ,&
&  0.92376E+00_JPRB, 0.91638E+00_JPRB, 0.90741E+00_JPRB, 0.89423E+00_JPRB, 0.87658E+00_JPRB, 0.86578E+00_JPRB, 0.86257E+00_JPRB ,&
&  0.85896E+00_JPRB, 0.86472E+00_JPRB, 0.88175E+00_JPRB, 0.90309E+00_JPRB, 0.91192E+00_JPRB, 0.91509E+00_JPRB, 0.91900E+00_JPRB ,&
&  0.92800E+00_JPRB, 0.93328E+00_JPRB, 0.93856E+00_JPRB, 0.90030E+00_JPRB, 0.94216E+00_JPRB, 0.94303E+00_JPRB, 0.92484E+00_JPRB ,&
&  0.90151E+00_JPRB, 0.87377E+00_JPRB, 0.86655E+00_JPRB, 0.72820E+00_JPRB, 0.79734E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.89830E+00_JPRB, 0.93083E+00_JPRB, 0.92701E+00_JPRB, 0.92218E+00_JPRB, 0.91561E+00_JPRB, 0.89794E+00_JPRB, 0.87606E+00_JPRB ,&
&  0.85420E+00_JPRB, 0.84239E+00_JPRB, 0.82578E+00_JPRB, 0.80466E+00_JPRB, 0.79231E+00_JPRB, 0.78635E+00_JPRB, 0.78122E+00_JPRB ,&
&  0.78122E+00_JPRB, 0.78068E+00_JPRB, 0.78084E+00_JPRB, 0.77986E+00_JPRB, 0.77359E+00_JPRB, 0.76799E+00_JPRB, 0.75745E+00_JPRB ,&
&  0.75117E+00_JPRB, 0.75210E+00_JPRB, 0.75402E+00_JPRB, 0.75888E+00_JPRB, 0.76673E+00_JPRB, 0.77536E+00_JPRB, 0.78119E+00_JPRB ,&
&  0.77558E+00_JPRB, 0.67245E+00_JPRB, 0.72712E+00_JPRB, 0.84552E+00_JPRB, 0.97498E+00_JPRB, 0.10760E+01_JPRB, 0.10915E+01_JPRB ,&
&  0.10839E+01_JPRB, 0.10700E+01_JPRB, 0.10515E+01_JPRB, 0.10297E+01_JPRB, 0.10082E+01_JPRB, 0.99511E+00_JPRB, 0.96736E+00_JPRB ,&
&  0.95112E+00_JPRB, 0.94570E+00_JPRB, 0.94863E+00_JPRB, 0.94621E+00_JPRB, 0.93816E+00_JPRB, 0.92720E+00_JPRB, 0.92122E+00_JPRB ,&
&  0.91048E+00_JPRB, 0.88392E+00_JPRB, 0.85450E+00_JPRB, 0.83157E+00_JPRB, 0.82485E+00_JPRB, 0.85365E+00_JPRB, 0.88535E+00_JPRB ,&
&  0.90691E+00_JPRB, 0.88320E+00_JPRB, 0.85744E+00_JPRB, 0.84617E+00_JPRB, 0.84794E+00_JPRB, 0.86543E+00_JPRB, 0.89138E+00_JPRB ,&
&  0.90037E+00_JPRB, 0.90240E+00_JPRB, 0.91450E+00_JPRB, 0.91643E+00_JPRB, 0.91828E+00_JPRB, 0.91608E+00_JPRB, 0.91309E+00_JPRB ,&
&  0.90480E+00_JPRB, 0.89510E+00_JPRB, 0.88953E+00_JPRB, 0.88364E+00_JPRB, 0.88296E+00_JPRB, 0.87575E+00_JPRB, 0.86377E+00_JPRB ,&
&  0.84997E+00_JPRB, 0.85996E+00_JPRB, 0.89285E+00_JPRB, 0.92334E+00_JPRB, 0.92328E+00_JPRB, 0.91433E+00_JPRB, 0.90933E+00_JPRB ,&
&  0.91500E+00_JPRB, 0.90980E+00_JPRB, 0.90303E+00_JPRB, 0.95062E+00_JPRB, 0.95025E+00_JPRB, 0.94433E+00_JPRB, 0.94237E+00_JPRB ,&
&  0.94443E+00_JPRB, 0.93283E+00_JPRB, 0.91980E+00_JPRB, 0.90249E+00_JPRB, 0.83934E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.98443E+00_JPRB, 0.97704E+00_JPRB, 0.96461E+00_JPRB, 0.95635E+00_JPRB, 0.93920E+00_JPRB, 0.90747E+00_JPRB, 0.87548E+00_JPRB ,&
&  0.85428E+00_JPRB, 0.83027E+00_JPRB, 0.79673E+00_JPRB, 0.78128E+00_JPRB, 0.77896E+00_JPRB, 0.78149E+00_JPRB, 0.77946E+00_JPRB ,&
&  0.77660E+00_JPRB, 0.78292E+00_JPRB, 0.78031E+00_JPRB, 0.78406E+00_JPRB, 0.78013E+00_JPRB, 0.78082E+00_JPRB, 0.76570E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.75120E+00_JPRB, 0.75005E+00_JPRB, 0.75688E+00_JPRB, 0.76267E+00_JPRB, 0.76679E+00_JPRB, 0.77086E+00_JPRB ,&
&  0.75585E+00_JPRB, 0.69339E+00_JPRB, 0.83876E+00_JPRB, 0.10340E+01_JPRB, 0.11752E+01_JPRB, 0.12399E+01_JPRB, 0.11816E+01_JPRB ,&
&  0.11338E+01_JPRB, 0.11039E+01_JPRB, 0.11016E+01_JPRB, 0.10830E+01_JPRB, 0.10505E+01_JPRB, 0.10236E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.10006E+01_JPRB, 0.99568E+00_JPRB, 0.98641E+00_JPRB, 0.97381E+00_JPRB, 0.95978E+00_JPRB, 0.95601E+00_JPRB ,&
&  0.93926E+00_JPRB, 0.90096E+00_JPRB, 0.86209E+00_JPRB, 0.83549E+00_JPRB, 0.84519E+00_JPRB, 0.87503E+00_JPRB, 0.91775E+00_JPRB ,&
&  0.93933E+00_JPRB, 0.91905E+00_JPRB, 0.90578E+00_JPRB, 0.89926E+00_JPRB, 0.90021E+00_JPRB, 0.91570E+00_JPRB, 0.92285E+00_JPRB ,&
&  0.92828E+00_JPRB, 0.92363E+00_JPRB, 0.92127E+00_JPRB, 0.91508E+00_JPRB, 0.91021E+00_JPRB, 0.90830E+00_JPRB, 0.90207E+00_JPRB ,&
&  0.89410E+00_JPRB, 0.89363E+00_JPRB, 0.88368E+00_JPRB, 0.88247E+00_JPRB, 0.87350E+00_JPRB, 0.85883E+00_JPRB, 0.84527E+00_JPRB ,&
&  0.85224E+00_JPRB, 0.87922E+00_JPRB, 0.89898E+00_JPRB, 0.92529E+00_JPRB, 0.93119E+00_JPRB, 0.92008E+00_JPRB, 0.92156E+00_JPRB ,&
&  0.91440E+00_JPRB, 0.90558E+00_JPRB, 0.86126E+00_JPRB, 0.91152E+00_JPRB, 0.10131E+01_JPRB, 0.10057E+01_JPRB, 0.99792E+00_JPRB ,&
&  0.99736E+00_JPRB, 0.98132E+00_JPRB, 0.94119E+00_JPRB, 0.91012E+00_JPRB, 0.96417E+00_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.91551E+00_JPRB, 0.90201E+00_JPRB, 0.90981E+00_JPRB, 0.91440E+00_JPRB, 0.90105E+00_JPRB, 0.86978E+00_JPRB, 0.84447E+00_JPRB ,&
&  0.83533E+00_JPRB, 0.80707E+00_JPRB, 0.78140E+00_JPRB, 0.76546E+00_JPRB, 0.77058E+00_JPRB, 0.77483E+00_JPRB, 0.77395E+00_JPRB ,&
&  0.76399E+00_JPRB, 0.76327E+00_JPRB, 0.75310E+00_JPRB, 0.74704E+00_JPRB, 0.74657E+00_JPRB, 0.74953E+00_JPRB, 0.74519E+00_JPRB ,&
&  0.75248E+00_JPRB, 0.75534E+00_JPRB, 0.76058E+00_JPRB, 0.76799E+00_JPRB, 0.76513E+00_JPRB, 0.76855E+00_JPRB, 0.75962E+00_JPRB ,&
&  0.73544E+00_JPRB, 0.78737E+00_JPRB, 0.10433E+01_JPRB, 0.13413E+01_JPRB, 0.14716E+01_JPRB, 0.13746E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11364E+01_JPRB, 0.11416E+01_JPRB, 0.11242E+01_JPRB, 0.11009E+01_JPRB, 0.10824E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10356E+01_JPRB, 0.10166E+01_JPRB, 0.10107E+01_JPRB, 0.99325E+00_JPRB, 0.97017E+00_JPRB, 0.96100E+00_JPRB, 0.96149E+00_JPRB ,&
&  0.91654E+00_JPRB, 0.88085E+00_JPRB, 0.85198E+00_JPRB, 0.83161E+00_JPRB, 0.87308E+00_JPRB, 0.94048E+00_JPRB, 0.10132E+01_JPRB ,&
&  0.98418E+00_JPRB, 0.97231E+00_JPRB, 0.96799E+00_JPRB, 0.96654E+00_JPRB, 0.98205E+00_JPRB, 0.95974E+00_JPRB, 0.94425E+00_JPRB ,&
&  0.91895E+00_JPRB, 0.91935E+00_JPRB, 0.92111E+00_JPRB, 0.92977E+00_JPRB, 0.92813E+00_JPRB, 0.92551E+00_JPRB, 0.89600E+00_JPRB ,&
&  0.88036E+00_JPRB, 0.86703E+00_JPRB, 0.85974E+00_JPRB, 0.85622E+00_JPRB, 0.85709E+00_JPRB, 0.84397E+00_JPRB, 0.83251E+00_JPRB ,&
&  0.84204E+00_JPRB, 0.85672E+00_JPRB, 0.90545E+00_JPRB, 0.96858E+00_JPRB, 0.97013E+00_JPRB, 0.94551E+00_JPRB, 0.92976E+00_JPRB ,&
&  0.91912E+00_JPRB, 0.90535E+00_JPRB, 0.87614E+00_JPRB, 0.94615E+00_JPRB, 0.10001E+01_JPRB, 0.10784E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.10051E+01_JPRB, 0.94560E+00_JPRB, 0.91330E+00_JPRB, 0.89868E+00_JPRB, 0.92325E+00_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.86604E+00_JPRB, 0.86476E+00_JPRB, 0.87791E+00_JPRB, 0.88618E+00_JPRB, 0.87126E+00_JPRB, 0.84175E+00_JPRB, 0.81468E+00_JPRB ,&
&  0.80534E+00_JPRB, 0.80315E+00_JPRB, 0.79911E+00_JPRB, 0.77982E+00_JPRB, 0.76493E+00_JPRB, 0.75749E+00_JPRB, 0.75477E+00_JPRB ,&
&  0.75524E+00_JPRB, 0.74493E+00_JPRB, 0.73144E+00_JPRB, 0.71797E+00_JPRB, 0.72149E+00_JPRB, 0.73272E+00_JPRB, 0.75134E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.75215E+00_JPRB, 0.75892E+00_JPRB, 0.76652E+00_JPRB, 0.77598E+00_JPRB, 0.76775E+00_JPRB, 0.75372E+00_JPRB ,&
&  0.74791E+00_JPRB, 0.91326E+00_JPRB, 0.13787E+01_JPRB, 0.15861E+01_JPRB, 0.15215E+01_JPRB, 0.13335E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.11606E+01_JPRB, 0.11616E+01_JPRB, 0.11740E+01_JPRB, 0.11452E+01_JPRB, 0.11109E+01_JPRB, 0.10835E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.10264E+01_JPRB, 0.99830E+00_JPRB, 0.97144E+00_JPRB, 0.96395E+00_JPRB, 0.96017E+00_JPRB, 0.96395E+00_JPRB, 0.94112E+00_JPRB ,&
&  0.90815E+00_JPRB, 0.87838E+00_JPRB, 0.85248E+00_JPRB, 0.83831E+00_JPRB, 0.87205E+00_JPRB, 0.87815E+00_JPRB, 0.10016E+01_JPRB ,&
&  0.11727E+01_JPRB, 0.10814E+01_JPRB, 0.10473E+01_JPRB, 0.10517E+01_JPRB, 0.10477E+01_JPRB, 0.10224E+01_JPRB, 0.99772E+00_JPRB ,&
&  0.99243E+00_JPRB, 0.10003E+01_JPRB, 0.10304E+01_JPRB, 0.10334E+01_JPRB, 0.99581E+00_JPRB, 0.94477E+00_JPRB, 0.92074E+00_JPRB ,&
&  0.91112E+00_JPRB, 0.90000E+00_JPRB, 0.89217E+00_JPRB, 0.88220E+00_JPRB, 0.87184E+00_JPRB, 0.84959E+00_JPRB, 0.84080E+00_JPRB ,&
&  0.84785E+00_JPRB, 0.88043E+00_JPRB, 0.95307E+00_JPRB, 0.98018E+00_JPRB, 0.10052E+01_JPRB, 0.10119E+01_JPRB, 0.96084E+00_JPRB ,&
&  0.89618E+00_JPRB, 0.87009E+00_JPRB, 0.87396E+00_JPRB, 0.92641E+00_JPRB, 0.10158E+01_JPRB, 0.10951E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.97261E+00_JPRB, 0.90734E+00_JPRB, 0.87308E+00_JPRB, 0.85703E+00_JPRB, 0.87743E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.82616E+00_JPRB, 0.81789E+00_JPRB, 0.82211E+00_JPRB, 0.83624E+00_JPRB, 0.82306E+00_JPRB, 0.80160E+00_JPRB, 0.77859E+00_JPRB ,&
&  0.79983E+00_JPRB, 0.81202E+00_JPRB, 0.81652E+00_JPRB, 0.78519E+00_JPRB, 0.76437E+00_JPRB, 0.74091E+00_JPRB, 0.72702E+00_JPRB ,&
&  0.72623E+00_JPRB, 0.71772E+00_JPRB, 0.70524E+00_JPRB, 0.70024E+00_JPRB, 0.70693E+00_JPRB, 0.71618E+00_JPRB, 0.73573E+00_JPRB ,&
&  0.72997E+00_JPRB, 0.73785E+00_JPRB, 0.76223E+00_JPRB, 0.77606E+00_JPRB, 0.78721E+00_JPRB, 0.76977E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.78000E+00_JPRB, 0.85147E+00_JPRB, 0.14696E+01_JPRB, 0.18391E+01_JPRB, 0.14985E+01_JPRB, 0.12728E+01_JPRB, 0.11427E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.11803E+01_JPRB, 0.11739E+01_JPRB, 0.11287E+01_JPRB, 0.11037E+01_JPRB, 0.10662E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.99853E+00_JPRB, 0.97617E+00_JPRB, 0.96721E+00_JPRB, 0.97323E+00_JPRB, 0.96977E+00_JPRB, 0.95622E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.90697E+00_JPRB, 0.88524E+00_JPRB, 0.85735E+00_JPRB, 0.85850E+00_JPRB, 0.92408E+00_JPRB, 0.10370E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.11841E+01_JPRB, 0.11288E+01_JPRB, 0.11282E+01_JPRB, 0.11042E+01_JPRB, 0.10858E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.10446E+01_JPRB, 0.10375E+01_JPRB, 0.10015E+01_JPRB, 0.96334E+00_JPRB, 0.94666E+00_JPRB, 0.92574E+00_JPRB ,&
&  0.91840E+00_JPRB, 0.92795E+00_JPRB, 0.92746E+00_JPRB, 0.90724E+00_JPRB, 0.88892E+00_JPRB, 0.86962E+00_JPRB, 0.86614E+00_JPRB ,&
&  0.88416E+00_JPRB, 0.90772E+00_JPRB, 0.96468E+00_JPRB, 0.10201E+01_JPRB, 0.10043E+01_JPRB, 0.96950E+00_JPRB, 0.93322E+00_JPRB ,&
&  0.90203E+00_JPRB, 0.87843E+00_JPRB, 0.87318E+00_JPRB, 0.94083E+00_JPRB, 0.97216E+00_JPRB, 0.10393E+01_JPRB, 0.98823E+00_JPRB ,&
&  0.90797E+00_JPRB, 0.85315E+00_JPRB, 0.82801E+00_JPRB, 0.82244E+00_JPRB, 0.83130E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.81123E+00_JPRB, 0.79175E+00_JPRB, 0.79864E+00_JPRB, 0.80456E+00_JPRB, 0.79987E+00_JPRB, 0.77959E+00_JPRB, 0.76771E+00_JPRB ,&
&  0.78667E+00_JPRB, 0.79351E+00_JPRB, 0.79024E+00_JPRB, 0.76237E+00_JPRB, 0.74612E+00_JPRB, 0.72108E+00_JPRB, 0.70272E+00_JPRB ,&
&  0.70288E+00_JPRB, 0.70527E+00_JPRB, 0.69825E+00_JPRB, 0.69905E+00_JPRB, 0.70409E+00_JPRB, 0.72223E+00_JPRB, 0.73523E+00_JPRB ,&
&  0.74837E+00_JPRB, 0.76036E+00_JPRB, 0.77970E+00_JPRB, 0.79160E+00_JPRB, 0.78773E+00_JPRB, 0.77345E+00_JPRB, 0.76789E+00_JPRB ,&
&  0.84233E+00_JPRB, 0.19176E+01_JPRB, 0.12885E+01_JPRB, 0.19784E+01_JPRB, 0.14332E+01_JPRB, 0.11913E+01_JPRB, 0.96461E+00_JPRB ,&
&  0.99797E+00_JPRB, 0.11617E+01_JPRB, 0.11945E+01_JPRB, 0.11342E+01_JPRB, 0.11136E+01_JPRB, 0.10844E+01_JPRB, 0.10316E+01_JPRB ,&
&  0.10016E+01_JPRB, 0.10025E+01_JPRB, 0.98136E+00_JPRB, 0.97645E+00_JPRB, 0.96566E+00_JPRB, 0.94620E+00_JPRB, 0.93688E+00_JPRB ,&
&  0.92062E+00_JPRB, 0.90047E+00_JPRB, 0.87965E+00_JPRB, 0.94703E+00_JPRB, 0.98875E+00_JPRB, 0.11268E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.12412E+01_JPRB, 0.11740E+01_JPRB, 0.11319E+01_JPRB, 0.11211E+01_JPRB, 0.10894E+01_JPRB, 0.10651E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.10324E+01_JPRB, 0.98595E+00_JPRB, 0.96398E+00_JPRB, 0.96600E+00_JPRB, 0.95858E+00_JPRB, 0.92843E+00_JPRB ,&
&  0.89807E+00_JPRB, 0.90102E+00_JPRB, 0.91319E+00_JPRB, 0.90951E+00_JPRB, 0.89510E+00_JPRB, 0.86885E+00_JPRB, 0.86554E+00_JPRB ,&
&  0.88785E+00_JPRB, 0.90180E+00_JPRB, 0.93408E+00_JPRB, 0.10136E+01_JPRB, 0.98730E+00_JPRB, 0.92799E+00_JPRB, 0.88938E+00_JPRB ,&
&  0.90313E+00_JPRB, 0.89023E+00_JPRB, 0.89320E+00_JPRB, 0.93141E+00_JPRB, 0.95844E+00_JPRB, 0.98858E+00_JPRB, 0.89935E+00_JPRB ,&
&  0.85250E+00_JPRB, 0.81747E+00_JPRB, 0.80251E+00_JPRB, 0.80087E+00_JPRB, 0.81391E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.83050E+00_JPRB, 0.80250E+00_JPRB, 0.78078E+00_JPRB, 0.78370E+00_JPRB, 0.78642E+00_JPRB, 0.77559E+00_JPRB, 0.75801E+00_JPRB ,&
&  0.75158E+00_JPRB, 0.74208E+00_JPRB, 0.75376E+00_JPRB, 0.74076E+00_JPRB, 0.73215E+00_JPRB, 0.71819E+00_JPRB, 0.70800E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.71429E+00_JPRB, 0.70139E+00_JPRB, 0.70646E+00_JPRB, 0.73520E+00_JPRB, 0.76094E+00_JPRB, 0.77091E+00_JPRB ,&
&  0.78262E+00_JPRB, 0.79599E+00_JPRB, 0.80000E+00_JPRB, 0.80152E+00_JPRB, 0.79068E+00_JPRB, 0.79099E+00_JPRB, 0.79744E+00_JPRB ,&
&  0.89617E+00_JPRB, 0.20240E+01_JPRB, 0.15295E+01_JPRB, 0.19148E+01_JPRB, 0.14086E+01_JPRB, 0.11330E+01_JPRB, 0.89005E+00_JPRB ,&
&  0.90999E+00_JPRB, 0.11057E+01_JPRB, 0.11597E+01_JPRB, 0.11824E+01_JPRB, 0.11283E+01_JPRB, 0.10795E+01_JPRB, 0.10341E+01_JPRB ,&
&  0.10079E+01_JPRB, 0.10073E+01_JPRB, 0.99173E+00_JPRB, 0.98070E+00_JPRB, 0.95737E+00_JPRB, 0.95627E+00_JPRB, 0.95290E+00_JPRB ,&
&  0.94061E+00_JPRB, 0.93659E+00_JPRB, 0.95347E+00_JPRB, 0.98597E+00_JPRB, 0.11396E+01_JPRB, 0.12282E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.12659E+01_JPRB, 0.12189E+01_JPRB, 0.11368E+01_JPRB, 0.11023E+01_JPRB, 0.10589E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.10061E+01_JPRB, 0.97952E+00_JPRB, 0.95510E+00_JPRB, 0.94465E+00_JPRB, 0.95724E+00_JPRB, 0.95357E+00_JPRB, 0.91135E+00_JPRB ,&
&  0.88475E+00_JPRB, 0.89361E+00_JPRB, 0.90328E+00_JPRB, 0.90767E+00_JPRB, 0.89481E+00_JPRB, 0.87770E+00_JPRB, 0.88459E+00_JPRB ,&
&  0.90632E+00_JPRB, 0.92838E+00_JPRB, 0.93974E+00_JPRB, 0.99525E+00_JPRB, 0.97155E+00_JPRB, 0.92755E+00_JPRB, 0.88074E+00_JPRB ,&
&  0.89152E+00_JPRB, 0.90341E+00_JPRB, 0.90838E+00_JPRB, 0.91353E+00_JPRB, 0.92746E+00_JPRB, 0.91334E+00_JPRB, 0.84781E+00_JPRB ,&
&  0.84166E+00_JPRB, 0.84274E+00_JPRB, 0.84763E+00_JPRB, 0.83166E+00_JPRB, 0.84961E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.81087E+00_JPRB, 0.76770E+00_JPRB, 0.75885E+00_JPRB, 0.77019E+00_JPRB, 0.76104E+00_JPRB, 0.74414E+00_JPRB, 0.71658E+00_JPRB ,&
&  0.69142E+00_JPRB, 0.70993E+00_JPRB, 0.73843E+00_JPRB, 0.75428E+00_JPRB, 0.76194E+00_JPRB, 0.76805E+00_JPRB, 0.74746E+00_JPRB ,&
&  0.71912E+00_JPRB, 0.72085E+00_JPRB, 0.73643E+00_JPRB, 0.76538E+00_JPRB, 0.78620E+00_JPRB, 0.80708E+00_JPRB, 0.83838E+00_JPRB ,&
&  0.84762E+00_JPRB, 0.84439E+00_JPRB, 0.83317E+00_JPRB, 0.82259E+00_JPRB, 0.81939E+00_JPRB, 0.83294E+00_JPRB, 0.86163E+00_JPRB ,&
&  0.10543E+01_JPRB, 0.25674E+01_JPRB, 0.25459E+01_JPRB, 0.18666E+01_JPRB, 0.14034E+01_JPRB, 0.11160E+01_JPRB, 0.93630E+00_JPRB ,&
&  0.89461E+00_JPRB, 0.89115E+00_JPRB, 0.11786E+01_JPRB, 0.11976E+01_JPRB, 0.11244E+01_JPRB, 0.10815E+01_JPRB, 0.10842E+01_JPRB ,&
&  0.10585E+01_JPRB, 0.10555E+01_JPRB, 0.10278E+01_JPRB, 0.10054E+01_JPRB, 0.98692E+00_JPRB, 0.99573E+00_JPRB, 0.10066E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.10258E+01_JPRB, 0.11054E+01_JPRB, 0.11849E+01_JPRB, 0.11860E+01_JPRB, 0.12419E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.11494E+01_JPRB, 0.12283E+01_JPRB, 0.11909E+01_JPRB, 0.11416E+01_JPRB, 0.10399E+01_JPRB, 0.10767E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.99429E+00_JPRB, 0.94918E+00_JPRB, 0.94535E+00_JPRB, 0.95637E+00_JPRB, 0.96514E+00_JPRB, 0.96414E+00_JPRB, 0.93997E+00_JPRB ,&
&  0.92174E+00_JPRB, 0.91891E+00_JPRB, 0.91599E+00_JPRB, 0.91383E+00_JPRB, 0.91025E+00_JPRB, 0.91249E+00_JPRB, 0.94012E+00_JPRB ,&
&  0.95844E+00_JPRB, 0.99144E+00_JPRB, 0.10009E+01_JPRB, 0.10445E+01_JPRB, 0.10261E+01_JPRB, 0.96161E+00_JPRB, 0.90192E+00_JPRB ,&
&  0.90099E+00_JPRB, 0.93429E+00_JPRB, 0.94204E+00_JPRB, 0.93574E+00_JPRB, 0.91252E+00_JPRB, 0.87628E+00_JPRB, 0.86358E+00_JPRB ,&
&  0.87967E+00_JPRB, 0.90518E+00_JPRB, 0.89203E+00_JPRB, 0.84236E+00_JPRB, 0.85494E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.78511E+00_JPRB, 0.74254E+00_JPRB, 0.73283E+00_JPRB, 0.72337E+00_JPRB, 0.70595E+00_JPRB, 0.69832E+00_JPRB, 0.67485E+00_JPRB ,&
&  0.69074E+00_JPRB, 0.74879E+00_JPRB, 0.81722E+00_JPRB, 0.84484E+00_JPRB, 0.85992E+00_JPRB, 0.85705E+00_JPRB, 0.82782E+00_JPRB ,&
&  0.79148E+00_JPRB, 0.79590E+00_JPRB, 0.81137E+00_JPRB, 0.82203E+00_JPRB, 0.84010E+00_JPRB, 0.85443E+00_JPRB, 0.87995E+00_JPRB ,&
&  0.90108E+00_JPRB, 0.88180E+00_JPRB, 0.85586E+00_JPRB, 0.84724E+00_JPRB, 0.88763E+00_JPRB, 0.94506E+00_JPRB, 0.10203E+01_JPRB ,&
&  0.14556E+01_JPRB, 0.29115E+01_JPRB, 0.15695E+01_JPRB, 0.14332E+01_JPRB, 0.13081E+01_JPRB, 0.11431E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.96685E+00_JPRB, 0.87091E+00_JPRB, 0.10034E+01_JPRB, 0.11787E+01_JPRB, 0.11470E+01_JPRB, 0.11171E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.11151E+01_JPRB, 0.11069E+01_JPRB, 0.10709E+01_JPRB, 0.10663E+01_JPRB, 0.10660E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11251E+01_JPRB, 0.12107E+01_JPRB, 0.11919E+01_JPRB, 0.12723E+01_JPRB, 0.12694E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.93824E+00_JPRB, 0.10370E+01_JPRB, 0.11311E+01_JPRB, 0.12010E+01_JPRB, 0.11561E+01_JPRB, 0.99602E+00_JPRB, 0.10470E+01_JPRB ,&
&  0.96992E+00_JPRB, 0.95682E+00_JPRB, 0.96774E+00_JPRB, 0.96991E+00_JPRB, 0.99128E+00_JPRB, 0.98994E+00_JPRB, 0.99250E+00_JPRB ,&
&  0.97547E+00_JPRB, 0.95779E+00_JPRB, 0.94654E+00_JPRB, 0.93862E+00_JPRB, 0.93554E+00_JPRB, 0.95324E+00_JPRB, 0.98569E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.10607E+01_JPRB, 0.11150E+01_JPRB, 0.11330E+01_JPRB, 0.11149E+01_JPRB, 0.10501E+01_JPRB, 0.97825E+00_JPRB ,&
&  0.96704E+00_JPRB, 0.98580E+00_JPRB, 0.94246E+00_JPRB, 0.90599E+00_JPRB, 0.90678E+00_JPRB, 0.89625E+00_JPRB, 0.94541E+00_JPRB ,&
&  0.10310E+01_JPRB, 0.11650E+01_JPRB, 0.12623E+01_JPRB, 0.82249E+00_JPRB, 0.81884E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.71377E+00_JPRB, 0.70226E+00_JPRB, 0.67532E+00_JPRB, 0.65647E+00_JPRB, 0.64995E+00_JPRB, 0.67253E+00_JPRB, 0.71791E+00_JPRB ,&
&  0.78867E+00_JPRB, 0.87791E+00_JPRB, 0.95387E+00_JPRB, 0.97935E+00_JPRB, 0.96656E+00_JPRB, 0.96871E+00_JPRB, 0.91667E+00_JPRB ,&
&  0.86238E+00_JPRB, 0.85299E+00_JPRB, 0.86564E+00_JPRB, 0.87617E+00_JPRB, 0.88945E+00_JPRB, 0.91011E+00_JPRB, 0.93450E+00_JPRB ,&
&  0.94773E+00_JPRB, 0.95582E+00_JPRB, 0.96222E+00_JPRB, 0.10174E+01_JPRB, 0.11074E+01_JPRB, 0.12474E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.20834E+01_JPRB, 0.20187E+01_JPRB, 0.11118E+01_JPRB, 0.12008E+01_JPRB, 0.10413E+01_JPRB, 0.10592E+01_JPRB, 0.10088E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.10147E+01_JPRB, 0.11065E+01_JPRB, 0.12377E+01_JPRB, 0.12253E+01_JPRB, 0.12217E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.11920E+01_JPRB, 0.11678E+01_JPRB, 0.11379E+01_JPRB, 0.11275E+01_JPRB, 0.11391E+01_JPRB, 0.11563E+01_JPRB ,&
&  0.12058E+01_JPRB, 0.12340E+01_JPRB, 0.13278E+01_JPRB, 0.12378E+01_JPRB, 0.13790E+01_JPRB, 0.13301E+01_JPRB, 0.11185E+01_JPRB ,&
&  0.95305E+00_JPRB, 0.94998E+00_JPRB, 0.11013E+01_JPRB, 0.12376E+01_JPRB, 0.12041E+01_JPRB, 0.97767E+00_JPRB, 0.10088E+01_JPRB ,&
&  0.95623E+00_JPRB, 0.95536E+00_JPRB, 0.95703E+00_JPRB, 0.97522E+00_JPRB, 0.10100E+01_JPRB, 0.10145E+01_JPRB, 0.10201E+01_JPRB ,&
&  0.99317E+00_JPRB, 0.96965E+00_JPRB, 0.96035E+00_JPRB, 0.95865E+00_JPRB, 0.96694E+00_JPRB, 0.10014E+01_JPRB, 0.10218E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.10846E+01_JPRB, 0.11333E+01_JPRB, 0.11867E+01_JPRB, 0.11764E+01_JPRB, 0.11130E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10141E+01_JPRB, 0.93987E+00_JPRB, 0.89304E+00_JPRB, 0.85033E+00_JPRB, 0.86360E+00_JPRB, 0.85447E+00_JPRB ,&
&  0.85519E+00_JPRB, 0.85688E+00_JPRB, 0.81744E+00_JPRB, 0.71215E+00_JPRB, 0.71509E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.67893E+00_JPRB, 0.68147E+00_JPRB, 0.67354E+00_JPRB, 0.68972E+00_JPRB, 0.69130E+00_JPRB, 0.75326E+00_JPRB, 0.82213E+00_JPRB ,&
&  0.92652E+00_JPRB, 0.10024E+01_JPRB, 0.10477E+01_JPRB, 0.10542E+01_JPRB, 0.10324E+01_JPRB, 0.97178E+00_JPRB, 0.91525E+00_JPRB ,&
&  0.91154E+00_JPRB, 0.91397E+00_JPRB, 0.92708E+00_JPRB, 0.94764E+00_JPRB, 0.96595E+00_JPRB, 0.99570E+00_JPRB, 0.10248E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.11147E+01_JPRB, 0.11768E+01_JPRB, 0.12671E+01_JPRB, 0.13811E+01_JPRB, 0.15387E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.12846E+01_JPRB, 0.65537E+00_JPRB, 0.93931E+00_JPRB, 0.99419E+00_JPRB, 0.93930E+00_JPRB, 0.10707E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.10980E+01_JPRB, 0.11001E+01_JPRB, 0.12883E+01_JPRB, 0.12865E+01_JPRB, 0.12842E+01_JPRB, 0.12811E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12383E+01_JPRB, 0.12111E+01_JPRB, 0.12091E+01_JPRB, 0.12066E+01_JPRB, 0.12307E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.13036E+01_JPRB, 0.13664E+01_JPRB, 0.14767E+01_JPRB, 0.12565E+01_JPRB, 0.13929E+01_JPRB, 0.14563E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.12849E+01_JPRB, 0.91529E+00_JPRB, 0.92966E+00_JPRB, 0.12158E+01_JPRB, 0.12125E+01_JPRB, 0.97265E+00_JPRB, 0.99675E+00_JPRB ,&
&  0.95526E+00_JPRB, 0.98212E+00_JPRB, 0.96194E+00_JPRB, 0.10160E+01_JPRB, 0.10412E+01_JPRB, 0.10556E+01_JPRB, 0.10254E+01_JPRB ,&
&  0.99428E+00_JPRB, 0.96121E+00_JPRB, 0.94924E+00_JPRB, 0.95755E+00_JPRB, 0.96693E+00_JPRB, 0.99860E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10393E+01_JPRB, 0.10896E+01_JPRB, 0.11406E+01_JPRB, 0.11673E+01_JPRB, 0.11118E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10243E+01_JPRB, 0.98670E+00_JPRB, 0.90304E+00_JPRB, 0.80536E+00_JPRB, 0.78934E+00_JPRB, 0.74279E+00_JPRB, 0.68842E+00_JPRB ,&
&  0.69789E+00_JPRB, 0.69527E+00_JPRB, 0.67981E+00_JPRB, 0.67767E+00_JPRB, 0.68222E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.76271E+00_JPRB, 0.75417E+00_JPRB, 0.74619E+00_JPRB, 0.74864E+00_JPRB, 0.75673E+00_JPRB, 0.80297E+00_JPRB, 0.87786E+00_JPRB ,&
&  0.94844E+00_JPRB, 0.99012E+00_JPRB, 0.10279E+01_JPRB, 0.10431E+01_JPRB, 0.10419E+01_JPRB, 0.99494E+00_JPRB, 0.98557E+00_JPRB ,&
&  0.97391E+00_JPRB, 0.96914E+00_JPRB, 0.98677E+00_JPRB, 0.10161E+01_JPRB, 0.10397E+01_JPRB, 0.10655E+01_JPRB, 0.10907E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11764E+01_JPRB, 0.12266E+01_JPRB, 0.13077E+01_JPRB, 0.13587E+01_JPRB, 0.13715E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.59214E+00_JPRB, 0.70377E+00_JPRB, 0.11149E+01_JPRB, 0.11176E+01_JPRB, 0.11807E+01_JPRB, 0.12519E+01_JPRB, 0.13316E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.10989E+01_JPRB, 0.11234E+01_JPRB, 0.13368E+01_JPRB, 0.13227E+01_JPRB, 0.13466E+01_JPRB, 0.13504E+01_JPRB ,&
&  0.13285E+01_JPRB, 0.13031E+01_JPRB, 0.12835E+01_JPRB, 0.12869E+01_JPRB, 0.13068E+01_JPRB, 0.13328E+01_JPRB, 0.13757E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.14671E+01_JPRB, 0.15467E+01_JPRB, 0.12834E+01_JPRB, 0.13554E+01_JPRB, 0.14834E+01_JPRB, 0.16466E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.86481E+00_JPRB, 0.88607E+00_JPRB, 0.11632E+01_JPRB, 0.11417E+01_JPRB, 0.11631E+01_JPRB, 0.98064E+00_JPRB ,&
&  0.96713E+00_JPRB, 0.91952E+00_JPRB, 0.93488E+00_JPRB, 0.98562E+00_JPRB, 0.10255E+01_JPRB, 0.10478E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.96260E+00_JPRB, 0.92803E+00_JPRB, 0.91165E+00_JPRB, 0.91698E+00_JPRB, 0.93649E+00_JPRB, 0.94927E+00_JPRB, 0.94304E+00_JPRB ,&
&  0.93592E+00_JPRB, 0.94003E+00_JPRB, 0.98143E+00_JPRB, 0.10091E+01_JPRB, 0.10408E+01_JPRB, 0.10769E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.10695E+01_JPRB, 0.11696E+01_JPRB, 0.10886E+01_JPRB, 0.98643E+00_JPRB, 0.84918E+00_JPRB, 0.71471E+00_JPRB ,&
&  0.73095E+00_JPRB, 0.73968E+00_JPRB, 0.73591E+00_JPRB, 0.74263E+00_JPRB, 0.75984E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.85070E+00_JPRB, 0.83355E+00_JPRB, 0.82101E+00_JPRB, 0.80937E+00_JPRB, 0.82082E+00_JPRB, 0.85377E+00_JPRB, 0.89107E+00_JPRB ,&
&  0.92713E+00_JPRB, 0.96856E+00_JPRB, 0.10255E+01_JPRB, 0.10410E+01_JPRB, 0.10750E+01_JPRB, 0.10683E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10290E+01_JPRB, 0.10462E+01_JPRB, 0.10651E+01_JPRB, 0.10661E+01_JPRB, 0.10820E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.11359E+01_JPRB, 0.11662E+01_JPRB, 0.12097E+01_JPRB, 0.12906E+01_JPRB, 0.13015E+01_JPRB, 0.12805E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.68549E+00_JPRB, 0.11373E+01_JPRB, 0.12406E+01_JPRB, 0.12146E+01_JPRB, 0.12674E+01_JPRB, 0.12484E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.12352E+01_JPRB, 0.11989E+01_JPRB, 0.11939E+01_JPRB, 0.12835E+01_JPRB, 0.13649E+01_JPRB, 0.13933E+01_JPRB, 0.14018E+01_JPRB ,&
&  0.13857E+01_JPRB, 0.13657E+01_JPRB, 0.13735E+01_JPRB, 0.14117E+01_JPRB, 0.14382E+01_JPRB, 0.14683E+01_JPRB, 0.15136E+01_JPRB ,&
&  0.15199E+01_JPRB, 0.15241E+01_JPRB, 0.15888E+01_JPRB, 0.14160E+01_JPRB, 0.13284E+01_JPRB, 0.13813E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.94554E+00_JPRB, 0.66411E+00_JPRB, 0.79968E+00_JPRB, 0.10336E+01_JPRB, 0.94625E+00_JPRB, 0.94979E+00_JPRB, 0.89209E+00_JPRB ,&
&  0.79996E+00_JPRB, 0.77184E+00_JPRB, 0.84301E+00_JPRB, 0.90855E+00_JPRB, 0.96413E+00_JPRB, 0.10058E+01_JPRB, 0.98748E+00_JPRB ,&
&  0.93268E+00_JPRB, 0.88896E+00_JPRB, 0.85595E+00_JPRB, 0.85070E+00_JPRB, 0.87425E+00_JPRB, 0.88723E+00_JPRB, 0.87462E+00_JPRB ,&
&  0.86916E+00_JPRB, 0.86027E+00_JPRB, 0.90267E+00_JPRB, 0.93635E+00_JPRB, 0.98260E+00_JPRB, 0.10371E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.16455E+01_JPRB, 0.19181E+01_JPRB, 0.18851E+01_JPRB, 0.17174E+01_JPRB, 0.12300E+01_JPRB, 0.82237E+00_JPRB ,&
&  0.83974E+00_JPRB, 0.85928E+00_JPRB, 0.86139E+00_JPRB, 0.86943E+00_JPRB, 0.85994E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.98366E+00_JPRB, 0.96736E+00_JPRB, 0.96390E+00_JPRB, 0.96000E+00_JPRB, 0.95766E+00_JPRB, 0.96527E+00_JPRB, 0.98738E+00_JPRB ,&
&  0.99686E+00_JPRB, 0.10306E+01_JPRB, 0.10818E+01_JPRB, 0.11127E+01_JPRB, 0.11527E+01_JPRB, 0.11502E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.11294E+01_JPRB, 0.11544E+01_JPRB, 0.11605E+01_JPRB, 0.11179E+01_JPRB, 0.10882E+01_JPRB, 0.10880E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.11294E+01_JPRB, 0.11473E+01_JPRB, 0.11820E+01_JPRB, 0.11733E+01_JPRB, 0.10893E+01_JPRB, 0.73336E+00_JPRB ,&
&  0.10820E+01_JPRB, 0.12980E+01_JPRB, 0.13181E+01_JPRB, 0.12514E+01_JPRB, 0.12190E+01_JPRB, 0.11498E+01_JPRB, 0.11542E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.12579E+01_JPRB, 0.12479E+01_JPRB, 0.12017E+01_JPRB, 0.13314E+01_JPRB, 0.13421E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.13577E+01_JPRB, 0.13631E+01_JPRB, 0.13765E+01_JPRB, 0.13958E+01_JPRB, 0.14077E+01_JPRB, 0.14298E+01_JPRB, 0.14410E+01_JPRB ,&
&  0.14216E+01_JPRB, 0.13938E+01_JPRB, 0.14101E+01_JPRB, 0.13105E+01_JPRB, 0.12725E+01_JPRB, 0.12756E+01_JPRB, 0.10927E+01_JPRB ,&
&  0.69495E+00_JPRB, 0.55377E+00_JPRB, 0.76931E+00_JPRB, 0.98597E+00_JPRB, 0.87695E+00_JPRB, 0.82860E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.78706E+00_JPRB, 0.72771E+00_JPRB, 0.71290E+00_JPRB, 0.76267E+00_JPRB, 0.83584E+00_JPRB, 0.89297E+00_JPRB, 0.90736E+00_JPRB ,&
&  0.87345E+00_JPRB, 0.84609E+00_JPRB, 0.84272E+00_JPRB, 0.85925E+00_JPRB, 0.89656E+00_JPRB, 0.98317E+00_JPRB, 0.11044E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.82183E+00_JPRB, 0.93651E+00_JPRB, 0.10021E+01_JPRB, 0.11226E+01_JPRB, 0.12906E+01_JPRB, 0.15904E+01_JPRB ,&
&  0.20639E+01_JPRB, 0.25754E+01_JPRB, 0.25409E+01_JPRB, 0.21271E+01_JPRB, 0.26918E+01_JPRB, 0.17946E+01_JPRB, 0.94671E+00_JPRB ,&
&  0.97395E+00_JPRB, 0.99187E+00_JPRB, 0.10081E+01_JPRB, 0.10013E+01_JPRB, 0.99510E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10079E+01_JPRB, 0.10091E+01_JPRB, 0.10003E+01_JPRB, 0.10290E+01_JPRB, 0.10252E+01_JPRB, 0.10390E+01_JPRB, 0.10390E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.10786E+01_JPRB, 0.10997E+01_JPRB, 0.11358E+01_JPRB, 0.11430E+01_JPRB, 0.11571E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.12110E+01_JPRB, 0.12451E+01_JPRB, 0.12408E+01_JPRB, 0.12227E+01_JPRB, 0.11837E+01_JPRB, 0.11549E+01_JPRB, 0.11206E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10840E+01_JPRB, 0.10835E+01_JPRB, 0.10910E+01_JPRB, 0.10925E+01_JPRB, 0.10999E+01_JPRB, 0.74350E+00_JPRB ,&
&  0.11952E+01_JPRB, 0.13436E+01_JPRB, 0.12640E+01_JPRB, 0.11923E+01_JPRB, 0.11046E+01_JPRB, 0.11077E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.11851E+01_JPRB, 0.12412E+01_JPRB, 0.12433E+01_JPRB, 0.12225E+01_JPRB, 0.11938E+01_JPRB, 0.12047E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.11972E+01_JPRB, 0.11903E+01_JPRB, 0.11810E+01_JPRB, 0.11672E+01_JPRB, 0.11570E+01_JPRB, 0.11610E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.10782E+01_JPRB, 0.10720E+01_JPRB, 0.10933E+01_JPRB, 0.10736E+01_JPRB, 0.93863E+00_JPRB, 0.74460E+00_JPRB ,&
&  0.50423E+00_JPRB, 0.50341E+00_JPRB, 0.70613E+00_JPRB, 0.98527E+00_JPRB, 0.90432E+00_JPRB, 0.81957E+00_JPRB, 0.75760E+00_JPRB ,&
&  0.73856E+00_JPRB, 0.68884E+00_JPRB, 0.63259E+00_JPRB, 0.62476E+00_JPRB, 0.64669E+00_JPRB, 0.69806E+00_JPRB, 0.70290E+00_JPRB ,&
&  0.71652E+00_JPRB, 0.71227E+00_JPRB, 0.69096E+00_JPRB, 0.67873E+00_JPRB, 0.71826E+00_JPRB, 0.72636E+00_JPRB, 0.64438E+00_JPRB ,&
&  0.77296E+00_JPRB, 0.71557E+00_JPRB, 0.75409E+00_JPRB, 0.84733E+00_JPRB, 0.91714E+00_JPRB, 0.11061E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.18533E+01_JPRB, 0.18301E+01_JPRB, 0.24425E+01_JPRB, 0.37412E+01_JPRB, 0.29304E+01_JPRB, 0.20728E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.10531E+01_JPRB, 0.10581E+01_JPRB, 0.10358E+01_JPRB, 0.10183E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.92337E+00_JPRB, 0.93308E+00_JPRB, 0.95019E+00_JPRB, 0.97532E+00_JPRB, 0.98814E+00_JPRB, 0.10095E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.10567E+01_JPRB, 0.10835E+01_JPRB, 0.11128E+01_JPRB, 0.11383E+01_JPRB, 0.11635E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12244E+01_JPRB, 0.12301E+01_JPRB, 0.12216E+01_JPRB, 0.11995E+01_JPRB, 0.11867E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.11453E+01_JPRB, 0.11512E+01_JPRB, 0.11536E+01_JPRB, 0.11580E+01_JPRB, 0.11957E+01_JPRB, 0.84772E+00_JPRB ,&
&  0.12510E+01_JPRB, 0.13162E+01_JPRB, 0.12496E+01_JPRB, 0.10973E+01_JPRB, 0.10202E+01_JPRB, 0.10105E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.11274E+01_JPRB, 0.11701E+01_JPRB, 0.10897E+01_JPRB, 0.10022E+01_JPRB, 0.10066E+01_JPRB, 0.99629E+00_JPRB ,&
&  0.99627E+00_JPRB, 0.98677E+00_JPRB, 0.97520E+00_JPRB, 0.95594E+00_JPRB, 0.93763E+00_JPRB, 0.91659E+00_JPRB, 0.88747E+00_JPRB ,&
&  0.84499E+00_JPRB, 0.79972E+00_JPRB, 0.72550E+00_JPRB, 0.72599E+00_JPRB, 0.71409E+00_JPRB, 0.62611E+00_JPRB, 0.49605E+00_JPRB ,&
&  0.42686E+00_JPRB, 0.56490E+00_JPRB, 0.72358E+00_JPRB, 0.10021E+01_JPRB, 0.93053E+00_JPRB, 0.86647E+00_JPRB, 0.79395E+00_JPRB ,&
&  0.68441E+00_JPRB, 0.63107E+00_JPRB, 0.58721E+00_JPRB, 0.57162E+00_JPRB, 0.56846E+00_JPRB, 0.58083E+00_JPRB, 0.57481E+00_JPRB ,&
&  0.55597E+00_JPRB, 0.52579E+00_JPRB, 0.51398E+00_JPRB, 0.50762E+00_JPRB, 0.51935E+00_JPRB, 0.46341E+00_JPRB, 0.59822E+00_JPRB ,&
&  0.59494E+00_JPRB, 0.53376E+00_JPRB, 0.70674E+00_JPRB, 0.49650E+00_JPRB, 0.64719E+00_JPRB, 0.73775E+00_JPRB, 0.85972E+00_JPRB ,&
&  0.89938E+00_JPRB, 0.11966E+01_JPRB, 0.15035E+01_JPRB, 0.13258E+01_JPRB, 0.11736E+01_JPRB, 0.10509E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.98554E+00_JPRB, 0.97045E+00_JPRB, 0.96124E+00_JPRB, 0.94686E+00_JPRB, 0.92942E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.87079E+00_JPRB, 0.87779E+00_JPRB, 0.88911E+00_JPRB, 0.90502E+00_JPRB, 0.92136E+00_JPRB, 0.93632E+00_JPRB, 0.94801E+00_JPRB ,&
&  0.95697E+00_JPRB, 0.97117E+00_JPRB, 0.99308E+00_JPRB, 0.10196E+01_JPRB, 0.10301E+01_JPRB, 0.10464E+01_JPRB, 0.10505E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10958E+01_JPRB, 0.11054E+01_JPRB, 0.11160E+01_JPRB, 0.11236E+01_JPRB, 0.11373E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.12092E+01_JPRB, 0.12646E+01_JPRB, 0.13097E+01_JPRB, 0.13702E+01_JPRB, 0.14664E+01_JPRB, 0.12325E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.13399E+01_JPRB, 0.12054E+01_JPRB, 0.10237E+01_JPRB, 0.96224E+00_JPRB, 0.91699E+00_JPRB, 0.88784E+00_JPRB ,&
&  0.87750E+00_JPRB, 0.83790E+00_JPRB, 0.84571E+00_JPRB, 0.84017E+00_JPRB, 0.83336E+00_JPRB, 0.86178E+00_JPRB, 0.85728E+00_JPRB ,&
&  0.85926E+00_JPRB, 0.86455E+00_JPRB, 0.87185E+00_JPRB, 0.87100E+00_JPRB, 0.87769E+00_JPRB, 0.86680E+00_JPRB, 0.83759E+00_JPRB ,&
&  0.79277E+00_JPRB, 0.74666E+00_JPRB, 0.68358E+00_JPRB, 0.66150E+00_JPRB, 0.64950E+00_JPRB, 0.59268E+00_JPRB, 0.53755E+00_JPRB ,&
&  0.56923E+00_JPRB, 0.62203E+00_JPRB, 0.68455E+00_JPRB, 0.95383E+00_JPRB, 0.95068E+00_JPRB, 0.92878E+00_JPRB, 0.78338E+00_JPRB ,&
&  0.71253E+00_JPRB, 0.62096E+00_JPRB, 0.57870E+00_JPRB, 0.57684E+00_JPRB, 0.56916E+00_JPRB, 0.55924E+00_JPRB, 0.53225E+00_JPRB ,&
&  0.48480E+00_JPRB, 0.44592E+00_JPRB, 0.43498E+00_JPRB, 0.42978E+00_JPRB, 0.44282E+00_JPRB, 0.42884E+00_JPRB, 0.51758E+00_JPRB ,&
&  0.50757E+00_JPRB, 0.36687E+00_JPRB, 0.45321E+00_JPRB, 0.55087E+00_JPRB, 0.55826E+00_JPRB, 0.61541E+00_JPRB, 0.74627E+00_JPRB ,&
&  0.95576E+00_JPRB, 0.10198E+01_JPRB, 0.10350E+01_JPRB, 0.96771E+00_JPRB, 0.94901E+00_JPRB, 0.94218E+00_JPRB, 0.92022E+00_JPRB ,&
&  0.90896E+00_JPRB, 0.90172E+00_JPRB, 0.89470E+00_JPRB, 0.88121E+00_JPRB, 0.87149E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.83236E+00_JPRB, 0.83404E+00_JPRB, 0.83900E+00_JPRB, 0.84758E+00_JPRB, 0.85744E+00_JPRB, 0.86815E+00_JPRB, 0.87376E+00_JPRB ,&
&  0.87049E+00_JPRB, 0.87376E+00_JPRB, 0.88322E+00_JPRB, 0.89816E+00_JPRB, 0.91149E+00_JPRB, 0.92301E+00_JPRB, 0.93541E+00_JPRB ,&
&  0.93348E+00_JPRB, 0.93438E+00_JPRB, 0.94923E+00_JPRB, 0.96797E+00_JPRB, 0.97868E+00_JPRB, 0.97880E+00_JPRB, 0.98196E+00_JPRB ,&
&  0.97850E+00_JPRB, 0.98880E+00_JPRB, 0.10152E+01_JPRB, 0.10321E+01_JPRB, 0.10567E+01_JPRB, 0.11172E+01_JPRB, 0.13045E+01_JPRB ,&
&  0.12139E+01_JPRB, 0.13032E+01_JPRB, 0.11158E+01_JPRB, 0.97001E+00_JPRB, 0.94994E+00_JPRB, 0.90589E+00_JPRB, 0.87212E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.80544E+00_JPRB, 0.76183E+00_JPRB, 0.75666E+00_JPRB, 0.73932E+00_JPRB, 0.75992E+00_JPRB, 0.76789E+00_JPRB ,&
&  0.79575E+00_JPRB, 0.81150E+00_JPRB, 0.83887E+00_JPRB, 0.85933E+00_JPRB, 0.90301E+00_JPRB, 0.90700E+00_JPRB, 0.86115E+00_JPRB ,&
&  0.85221E+00_JPRB, 0.88669E+00_JPRB, 0.86370E+00_JPRB, 0.88603E+00_JPRB, 0.85923E+00_JPRB, 0.81258E+00_JPRB, 0.78445E+00_JPRB ,&
&  0.75411E+00_JPRB, 0.74660E+00_JPRB, 0.86166E+00_JPRB, 0.88230E+00_JPRB, 0.94957E+00_JPRB, 0.95871E+00_JPRB, 0.86524E+00_JPRB ,&
&  0.78436E+00_JPRB, 0.69789E+00_JPRB, 0.68452E+00_JPRB, 0.67449E+00_JPRB, 0.69270E+00_JPRB, 0.67636E+00_JPRB, 0.65529E+00_JPRB ,&
&  0.58906E+00_JPRB, 0.52463E+00_JPRB, 0.51381E+00_JPRB, 0.48768E+00_JPRB, 0.45277E+00_JPRB, 0.48125E+00_JPRB, 0.53835E+00_JPRB ,&
&  0.51627E+00_JPRB, 0.43527E+00_JPRB, 0.41139E+00_JPRB, 0.58442E+00_JPRB, 0.61317E+00_JPRB, 0.67407E+00_JPRB, 0.75161E+00_JPRB ,&
&  0.86198E+00_JPRB, 0.95616E+00_JPRB, 0.10117E+01_JPRB, 0.96009E+00_JPRB, 0.88058E+00_JPRB, 0.86088E+00_JPRB, 0.84622E+00_JPRB ,&
&  0.84515E+00_JPRB, 0.84294E+00_JPRB, 0.83956E+00_JPRB, 0.83428E+00_JPRB, 0.82617E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.77361E+00_JPRB, 0.77485E+00_JPRB, 0.78863E+00_JPRB, 0.78973E+00_JPRB, 0.78991E+00_JPRB, 0.79807E+00_JPRB, 0.80191E+00_JPRB ,&
&  0.79779E+00_JPRB, 0.80615E+00_JPRB, 0.81726E+00_JPRB, 0.82912E+00_JPRB, 0.84284E+00_JPRB, 0.85498E+00_JPRB, 0.86142E+00_JPRB ,&
&  0.85227E+00_JPRB, 0.84237E+00_JPRB, 0.86292E+00_JPRB, 0.88139E+00_JPRB, 0.88431E+00_JPRB, 0.88821E+00_JPRB, 0.91847E+00_JPRB ,&
&  0.97688E+00_JPRB, 0.10096E+01_JPRB, 0.10110E+01_JPRB, 0.98691E+00_JPRB, 0.10135E+01_JPRB, 0.12069E+01_JPRB, 0.18740E+01_JPRB ,&
&  0.17334E+01_JPRB, 0.11795E+01_JPRB, 0.10673E+01_JPRB, 0.95940E+00_JPRB, 0.95812E+00_JPRB, 0.97865E+00_JPRB, 0.95889E+00_JPRB ,&
&  0.90834E+00_JPRB, 0.90312E+00_JPRB, 0.89725E+00_JPRB, 0.89056E+00_JPRB, 0.89991E+00_JPRB, 0.91051E+00_JPRB, 0.91240E+00_JPRB ,&
&  0.89222E+00_JPRB, 0.88963E+00_JPRB, 0.90501E+00_JPRB, 0.91752E+00_JPRB, 0.96678E+00_JPRB, 0.10988E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.11012E+01_JPRB, 0.10914E+01_JPRB, 0.10512E+01_JPRB, 0.10457E+01_JPRB, 0.10381E+01_JPRB, 0.98940E+00_JPRB, 0.93763E+00_JPRB ,&
&  0.91421E+00_JPRB, 0.85668E+00_JPRB, 0.74356E+00_JPRB, 0.84767E+00_JPRB, 0.93501E+00_JPRB, 0.10116E+01_JPRB, 0.95956E+00_JPRB ,&
&  0.91334E+00_JPRB, 0.82908E+00_JPRB, 0.80221E+00_JPRB, 0.83574E+00_JPRB, 0.88413E+00_JPRB, 0.87062E+00_JPRB, 0.83370E+00_JPRB ,&
&  0.74644E+00_JPRB, 0.67196E+00_JPRB, 0.65057E+00_JPRB, 0.59409E+00_JPRB, 0.56629E+00_JPRB, 0.50885E+00_JPRB, 0.57776E+00_JPRB ,&
&  0.58864E+00_JPRB, 0.60745E+00_JPRB, 0.49978E+00_JPRB, 0.66530E+00_JPRB, 0.69345E+00_JPRB, 0.70878E+00_JPRB, 0.72925E+00_JPRB ,&
&  0.74948E+00_JPRB, 0.77462E+00_JPRB, 0.77736E+00_JPRB, 0.85152E+00_JPRB, 0.83586E+00_JPRB, 0.82136E+00_JPRB, 0.81981E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.77242E+00_JPRB, 0.76847E+00_JPRB, 0.77430E+00_JPRB, 0.77414E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.70088E+00_JPRB, 0.69776E+00_JPRB, 0.69754E+00_JPRB, 0.71272E+00_JPRB, 0.72653E+00_JPRB, 0.71940E+00_JPRB, 0.72863E+00_JPRB ,&
&  0.71093E+00_JPRB, 0.69708E+00_JPRB, 0.70928E+00_JPRB, 0.71655E+00_JPRB, 0.73385E+00_JPRB, 0.74890E+00_JPRB, 0.75934E+00_JPRB ,&
&  0.76132E+00_JPRB, 0.76107E+00_JPRB, 0.73679E+00_JPRB, 0.75390E+00_JPRB, 0.84695E+00_JPRB, 0.99883E+00_JPRB, 0.12688E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.16749E+01_JPRB, 0.17441E+01_JPRB, 0.17217E+01_JPRB, 0.16019E+01_JPRB, 0.14309E+01_JPRB, 0.18156E+01_JPRB ,&
&  0.16643E+01_JPRB, 0.11906E+01_JPRB, 0.11916E+01_JPRB, 0.11019E+01_JPRB, 0.11012E+01_JPRB, 0.10735E+01_JPRB, 0.10567E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.10192E+01_JPRB, 0.10308E+01_JPRB, 0.10642E+01_JPRB, 0.11002E+01_JPRB, 0.11405E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11713E+01_JPRB, 0.12130E+01_JPRB, 0.11778E+01_JPRB, 0.11507E+01_JPRB, 0.11880E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12601E+01_JPRB, 0.10949E+01_JPRB, 0.10721E+01_JPRB, 0.11028E+01_JPRB, 0.10050E+01_JPRB, 0.98751E+00_JPRB ,&
&  0.10110E+01_JPRB, 0.91582E+00_JPRB, 0.70360E+00_JPRB, 0.75761E+00_JPRB, 0.84913E+00_JPRB, 0.95895E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.94402E+00_JPRB, 0.93169E+00_JPRB, 0.97190E+00_JPRB, 0.99826E+00_JPRB, 0.98983E+00_JPRB, 0.97715E+00_JPRB ,&
&  0.95158E+00_JPRB, 0.81406E+00_JPRB, 0.75014E+00_JPRB, 0.67757E+00_JPRB, 0.64151E+00_JPRB, 0.62931E+00_JPRB, 0.60908E+00_JPRB ,&
&  0.63083E+00_JPRB, 0.64935E+00_JPRB, 0.65771E+00_JPRB, 0.65562E+00_JPRB, 0.62418E+00_JPRB, 0.74862E+00_JPRB, 0.76639E+00_JPRB ,&
&  0.77016E+00_JPRB, 0.77033E+00_JPRB, 0.78098E+00_JPRB, 0.76909E+00_JPRB, 0.74760E+00_JPRB, 0.72676E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.71607E+00_JPRB, 0.72652E+00_JPRB, 0.72968E+00_JPRB, 0.71533E+00_JPRB, 0.70371E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.67796E+00_JPRB, 0.68174E+00_JPRB, 0.68061E+00_JPRB, 0.69420E+00_JPRB, 0.69164E+00_JPRB, 0.70733E+00_JPRB, 0.69777E+00_JPRB ,&
&  0.68834E+00_JPRB, 0.69107E+00_JPRB, 0.70119E+00_JPRB, 0.71160E+00_JPRB, 0.74005E+00_JPRB, 0.79399E+00_JPRB, 0.86240E+00_JPRB ,&
&  0.93479E+00_JPRB, 0.96443E+00_JPRB, 0.10213E+01_JPRB, 0.11430E+01_JPRB, 0.13784E+01_JPRB, 0.16023E+01_JPRB, 0.16466E+01_JPRB ,&
&  0.17227E+01_JPRB, 0.18377E+01_JPRB, 0.19124E+01_JPRB, 0.19571E+01_JPRB, 0.18999E+01_JPRB, 0.18122E+01_JPRB, 0.17276E+01_JPRB ,&
&  0.15034E+01_JPRB, 0.12302E+01_JPRB, 0.11684E+01_JPRB, 0.11469E+01_JPRB, 0.11484E+01_JPRB, 0.11219E+01_JPRB, 0.10969E+01_JPRB ,&
&  0.10785E+01_JPRB, 0.10780E+01_JPRB, 0.10759E+01_JPRB, 0.10917E+01_JPRB, 0.11273E+01_JPRB, 0.11746E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11758E+01_JPRB, 0.11674E+01_JPRB, 0.11733E+01_JPRB, 0.10850E+01_JPRB, 0.11294E+01_JPRB, 0.11481E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.10780E+01_JPRB, 0.10830E+01_JPRB, 0.10658E+01_JPRB, 0.10430E+01_JPRB, 0.98301E+00_JPRB, 0.95338E+00_JPRB ,&
&  0.96215E+00_JPRB, 0.89977E+00_JPRB, 0.73156E+00_JPRB, 0.80265E+00_JPRB, 0.89609E+00_JPRB, 0.89543E+00_JPRB, 0.11236E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10225E+01_JPRB, 0.10129E+01_JPRB, 0.99716E+00_JPRB, 0.10242E+01_JPRB, 0.10203E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.93650E+00_JPRB, 0.84551E+00_JPRB, 0.78064E+00_JPRB, 0.72224E+00_JPRB, 0.67998E+00_JPRB, 0.66509E+00_JPRB ,&
&  0.69050E+00_JPRB, 0.71878E+00_JPRB, 0.70755E+00_JPRB, 0.69648E+00_JPRB, 0.66725E+00_JPRB, 0.75579E+00_JPRB, 0.76310E+00_JPRB ,&
&  0.78512E+00_JPRB, 0.77513E+00_JPRB, 0.75644E+00_JPRB, 0.73184E+00_JPRB, 0.71655E+00_JPRB, 0.70585E+00_JPRB, 0.70193E+00_JPRB ,&
&  0.70708E+00_JPRB, 0.71212E+00_JPRB, 0.70391E+00_JPRB, 0.69451E+00_JPRB, 0.68340E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.65684E+00_JPRB, 0.65357E+00_JPRB, 0.65323E+00_JPRB, 0.68291E+00_JPRB, 0.68209E+00_JPRB, 0.67499E+00_JPRB, 0.68939E+00_JPRB ,&
&  0.70862E+00_JPRB, 0.73283E+00_JPRB, 0.76190E+00_JPRB, 0.79078E+00_JPRB, 0.82908E+00_JPRB, 0.88454E+00_JPRB, 0.94639E+00_JPRB ,&
&  0.10180E+01_JPRB, 0.11033E+01_JPRB, 0.12440E+01_JPRB, 0.14228E+01_JPRB, 0.16323E+01_JPRB, 0.17438E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.16558E+01_JPRB, 0.16907E+01_JPRB, 0.16605E+01_JPRB, 0.13967E+01_JPRB, 0.14182E+01_JPRB, 0.14528E+01_JPRB ,&
&  0.14395E+01_JPRB, 0.12815E+01_JPRB, 0.11601E+01_JPRB, 0.11140E+01_JPRB, 0.10769E+01_JPRB, 0.10820E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10616E+01_JPRB, 0.10587E+01_JPRB, 0.10628E+01_JPRB, 0.10840E+01_JPRB, 0.11270E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11361E+01_JPRB, 0.11138E+01_JPRB, 0.10654E+01_JPRB, 0.10678E+01_JPRB, 0.10653E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10753E+01_JPRB, 0.10342E+01_JPRB, 0.10396E+01_JPRB, 0.10412E+01_JPRB, 0.99114E+00_JPRB, 0.90861E+00_JPRB, 0.89787E+00_JPRB ,&
&  0.92658E+00_JPRB, 0.95856E+00_JPRB, 0.77298E+00_JPRB, 0.97989E+00_JPRB, 0.11280E+01_JPRB, 0.10644E+01_JPRB, 0.10803E+01_JPRB ,&
&  0.10390E+01_JPRB, 0.10249E+01_JPRB, 0.10274E+01_JPRB, 0.10295E+01_JPRB, 0.10194E+01_JPRB, 0.10101E+01_JPRB, 0.96053E+00_JPRB ,&
&  0.10467E+01_JPRB, 0.95118E+00_JPRB, 0.87194E+00_JPRB, 0.83742E+00_JPRB, 0.80070E+00_JPRB, 0.74053E+00_JPRB, 0.69040E+00_JPRB ,&
&  0.67084E+00_JPRB, 0.87698E+00_JPRB, 0.94186E+00_JPRB, 0.10601E+01_JPRB, 0.11184E+01_JPRB, 0.76396E+00_JPRB, 0.80228E+00_JPRB ,&
&  0.79556E+00_JPRB, 0.78244E+00_JPRB, 0.75772E+00_JPRB, 0.72983E+00_JPRB, 0.72516E+00_JPRB, 0.72512E+00_JPRB, 0.72247E+00_JPRB ,&
&  0.71594E+00_JPRB, 0.70161E+00_JPRB, 0.69229E+00_JPRB, 0.67318E+00_JPRB, 0.66386E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.67584E+00_JPRB, 0.67120E+00_JPRB, 0.66981E+00_JPRB, 0.68462E+00_JPRB, 0.69824E+00_JPRB, 0.70203E+00_JPRB, 0.69939E+00_JPRB ,&
&  0.71173E+00_JPRB, 0.74242E+00_JPRB, 0.77811E+00_JPRB, 0.79792E+00_JPRB, 0.80606E+00_JPRB, 0.81887E+00_JPRB, 0.86645E+00_JPRB ,&
&  0.94640E+00_JPRB, 0.10650E+01_JPRB, 0.12152E+01_JPRB, 0.13397E+01_JPRB, 0.14487E+01_JPRB, 0.15328E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.16739E+01_JPRB, 0.17122E+01_JPRB, 0.16411E+01_JPRB, 0.12611E+01_JPRB, 0.11457E+01_JPRB, 0.13452E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.11587E+01_JPRB, 0.11180E+01_JPRB, 0.10727E+01_JPRB, 0.10322E+01_JPRB, 0.10430E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10092E+01_JPRB, 0.10102E+01_JPRB, 0.10178E+01_JPRB, 0.10290E+01_JPRB, 0.10391E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10408E+01_JPRB, 0.10483E+01_JPRB, 0.10478E+01_JPRB, 0.10286E+01_JPRB, 0.10390E+01_JPRB, 0.10345E+01_JPRB ,&
&  0.10402E+01_JPRB, 0.10246E+01_JPRB, 0.10125E+01_JPRB, 0.10228E+01_JPRB, 0.10129E+01_JPRB, 0.89838E+00_JPRB, 0.88546E+00_JPRB ,&
&  0.92343E+00_JPRB, 0.93637E+00_JPRB, 0.86816E+00_JPRB, 0.10157E+01_JPRB, 0.72551E+00_JPRB, 0.89834E+00_JPRB, 0.10647E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.10448E+01_JPRB, 0.10308E+01_JPRB, 0.10241E+01_JPRB, 0.10133E+01_JPRB, 0.92885E+00_JPRB, 0.93240E+00_JPRB ,&
&  0.93392E+00_JPRB, 0.89851E+00_JPRB, 0.86508E+00_JPRB, 0.85488E+00_JPRB, 0.83731E+00_JPRB, 0.80268E+00_JPRB, 0.75839E+00_JPRB ,&
&  0.69545E+00_JPRB, 0.84445E+00_JPRB, 0.85385E+00_JPRB, 0.86340E+00_JPRB, 0.84604E+00_JPRB, 0.73848E+00_JPRB, 0.77421E+00_JPRB ,&
&  0.79467E+00_JPRB, 0.76990E+00_JPRB, 0.73919E+00_JPRB, 0.73480E+00_JPRB, 0.73024E+00_JPRB, 0.72977E+00_JPRB, 0.71607E+00_JPRB ,&
&  0.71389E+00_JPRB, 0.71112E+00_JPRB, 0.70607E+00_JPRB, 0.69512E+00_JPRB, 0.68433E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.67602E+00_JPRB, 0.67521E+00_JPRB, 0.68034E+00_JPRB, 0.69494E+00_JPRB, 0.72280E+00_JPRB, 0.74719E+00_JPRB, 0.71126E+00_JPRB ,&
&  0.70217E+00_JPRB, 0.70015E+00_JPRB, 0.73150E+00_JPRB, 0.74530E+00_JPRB, 0.75566E+00_JPRB, 0.76664E+00_JPRB, 0.81094E+00_JPRB ,&
&  0.89115E+00_JPRB, 0.10083E+01_JPRB, 0.11276E+01_JPRB, 0.12142E+01_JPRB, 0.13065E+01_JPRB, 0.14268E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.13969E+01_JPRB, 0.11906E+01_JPRB, 0.15916E+01_JPRB, 0.16443E+01_JPRB, 0.14787E+01_JPRB, 0.13779E+01_JPRB, 0.13652E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.11745E+01_JPRB, 0.11752E+01_JPRB, 0.11149E+01_JPRB, 0.99886E+00_JPRB, 0.98853E+00_JPRB, 0.98634E+00_JPRB ,&
&  0.97539E+00_JPRB, 0.95643E+00_JPRB, 0.95495E+00_JPRB, 0.96450E+00_JPRB, 0.97523E+00_JPRB, 0.96252E+00_JPRB, 0.94208E+00_JPRB ,&
&  0.93689E+00_JPRB, 0.93602E+00_JPRB, 0.94397E+00_JPRB, 0.94168E+00_JPRB, 0.95195E+00_JPRB, 0.97262E+00_JPRB, 0.98571E+00_JPRB ,&
&  0.99619E+00_JPRB, 0.98617E+00_JPRB, 0.10074E+01_JPRB, 0.10464E+01_JPRB, 0.10763E+01_JPRB, 0.97809E+00_JPRB, 0.99598E+00_JPRB ,&
&  0.97194E+00_JPRB, 0.94570E+00_JPRB, 0.92137E+00_JPRB, 0.10137E+01_JPRB, 0.79585E+00_JPRB, 0.91810E+00_JPRB, 0.95952E+00_JPRB ,&
&  0.10941E+01_JPRB, 0.10646E+01_JPRB, 0.10532E+01_JPRB, 0.10767E+01_JPRB, 0.10876E+01_JPRB, 0.98553E+00_JPRB, 0.94420E+00_JPRB ,&
&  0.91845E+00_JPRB, 0.91153E+00_JPRB, 0.88372E+00_JPRB, 0.86958E+00_JPRB, 0.82359E+00_JPRB, 0.82501E+00_JPRB, 0.84323E+00_JPRB ,&
&  0.88059E+00_JPRB, 0.87243E+00_JPRB, 0.86470E+00_JPRB, 0.83904E+00_JPRB, 0.78337E+00_JPRB, 0.75005E+00_JPRB, 0.75698E+00_JPRB ,&
&  0.76912E+00_JPRB, 0.78229E+00_JPRB, 0.73398E+00_JPRB, 0.70435E+00_JPRB, 0.69951E+00_JPRB, 0.71558E+00_JPRB, 0.70336E+00_JPRB ,&
&  0.70223E+00_JPRB, 0.69403E+00_JPRB, 0.69448E+00_JPRB, 0.68929E+00_JPRB, 0.68486E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.68097E+00_JPRB, 0.69865E+00_JPRB, 0.72953E+00_JPRB, 0.76692E+00_JPRB, 0.80536E+00_JPRB, 0.86436E+00_JPRB, 0.83351E+00_JPRB ,&
&  0.80236E+00_JPRB, 0.63977E+00_JPRB, 0.65807E+00_JPRB, 0.67156E+00_JPRB, 0.68860E+00_JPRB, 0.69058E+00_JPRB, 0.71382E+00_JPRB ,&
&  0.79352E+00_JPRB, 0.91388E+00_JPRB, 0.10236E+01_JPRB, 0.11207E+01_JPRB, 0.12234E+01_JPRB, 0.13600E+01_JPRB, 0.13077E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.12109E+01_JPRB, 0.18114E+01_JPRB, 0.17628E+01_JPRB, 0.16325E+01_JPRB, 0.15081E+01_JPRB, 0.13768E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.11757E+01_JPRB, 0.12050E+01_JPRB, 0.10984E+01_JPRB, 0.96570E+00_JPRB, 0.94467E+00_JPRB, 0.94322E+00_JPRB ,&
&  0.92864E+00_JPRB, 0.91219E+00_JPRB, 0.90042E+00_JPRB, 0.90301E+00_JPRB, 0.90582E+00_JPRB, 0.89497E+00_JPRB, 0.88000E+00_JPRB ,&
&  0.87519E+00_JPRB, 0.87730E+00_JPRB, 0.87684E+00_JPRB, 0.87168E+00_JPRB, 0.89544E+00_JPRB, 0.92179E+00_JPRB, 0.93681E+00_JPRB ,&
&  0.94667E+00_JPRB, 0.97481E+00_JPRB, 0.10006E+01_JPRB, 0.10065E+01_JPRB, 0.96938E+00_JPRB, 0.10510E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10290E+01_JPRB, 0.99213E+00_JPRB, 0.10444E+01_JPRB, 0.85999E+00_JPRB, 0.92699E+00_JPRB, 0.10111E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.11003E+01_JPRB, 0.10756E+01_JPRB, 0.11040E+01_JPRB, 0.11495E+01_JPRB, 0.10896E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.97946E+00_JPRB, 0.10108E+01_JPRB, 0.93564E+00_JPRB, 0.86559E+00_JPRB, 0.78736E+00_JPRB, 0.76866E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.88116E+00_JPRB, 0.90351E+00_JPRB, 0.91139E+00_JPRB, 0.90806E+00_JPRB, 0.86915E+00_JPRB, 0.82824E+00_JPRB, 0.75863E+00_JPRB ,&
&  0.73906E+00_JPRB, 0.69802E+00_JPRB, 0.68475E+00_JPRB, 0.68632E+00_JPRB, 0.67849E+00_JPRB, 0.69201E+00_JPRB, 0.68607E+00_JPRB ,&
&  0.67335E+00_JPRB, 0.67427E+00_JPRB, 0.67420E+00_JPRB, 0.67203E+00_JPRB, 0.67272E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.64202E+00_JPRB, 0.65690E+00_JPRB, 0.68394E+00_JPRB, 0.69857E+00_JPRB, 0.69427E+00_JPRB, 0.71129E+00_JPRB, 0.74478E+00_JPRB ,&
&  0.74338E+00_JPRB, 0.60553E+00_JPRB, 0.58925E+00_JPRB, 0.59758E+00_JPRB, 0.61837E+00_JPRB, 0.63153E+00_JPRB, 0.64158E+00_JPRB ,&
&  0.67879E+00_JPRB, 0.77640E+00_JPRB, 0.93076E+00_JPRB, 0.10081E+01_JPRB, 0.10750E+01_JPRB, 0.12123E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.94317E+00_JPRB, 0.16498E+01_JPRB, 0.15860E+01_JPRB, 0.17731E+01_JPRB, 0.17352E+01_JPRB, 0.16373E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.11017E+01_JPRB, 0.10839E+01_JPRB, 0.10291E+01_JPRB, 0.97547E+00_JPRB, 0.92101E+00_JPRB, 0.90914E+00_JPRB ,&
&  0.89021E+00_JPRB, 0.86295E+00_JPRB, 0.84150E+00_JPRB, 0.84183E+00_JPRB, 0.83876E+00_JPRB, 0.83220E+00_JPRB, 0.82947E+00_JPRB ,&
&  0.83761E+00_JPRB, 0.84456E+00_JPRB, 0.84070E+00_JPRB, 0.85565E+00_JPRB, 0.87422E+00_JPRB, 0.90536E+00_JPRB, 0.92681E+00_JPRB ,&
&  0.90933E+00_JPRB, 0.89338E+00_JPRB, 0.96209E+00_JPRB, 0.99527E+00_JPRB, 0.10124E+01_JPRB, 0.10755E+01_JPRB, 0.10296E+01_JPRB ,&
&  0.10808E+01_JPRB, 0.10138E+01_JPRB, 0.10727E+01_JPRB, 0.87559E+00_JPRB, 0.93581E+00_JPRB, 0.10516E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.11400E+01_JPRB, 0.11248E+01_JPRB, 0.11516E+01_JPRB, 0.11865E+01_JPRB, 0.12282E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11128E+01_JPRB, 0.10931E+01_JPRB, 0.92592E+00_JPRB, 0.83512E+00_JPRB, 0.76184E+00_JPRB, 0.80351E+00_JPRB ,&
&  0.87078E+00_JPRB, 0.88611E+00_JPRB, 0.91164E+00_JPRB, 0.94574E+00_JPRB, 0.93632E+00_JPRB, 0.94651E+00_JPRB, 0.89199E+00_JPRB ,&
&  0.78839E+00_JPRB, 0.73487E+00_JPRB, 0.67388E+00_JPRB, 0.67028E+00_JPRB, 0.67326E+00_JPRB, 0.66506E+00_JPRB, 0.66970E+00_JPRB ,&
&  0.66526E+00_JPRB, 0.65525E+00_JPRB, 0.65006E+00_JPRB, 0.64306E+00_JPRB, 0.63469E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.61194E+00_JPRB, 0.62102E+00_JPRB, 0.63847E+00_JPRB, 0.64834E+00_JPRB, 0.64844E+00_JPRB, 0.63750E+00_JPRB, 0.66468E+00_JPRB ,&
&  0.69229E+00_JPRB, 0.59235E+00_JPRB, 0.54824E+00_JPRB, 0.53273E+00_JPRB, 0.54303E+00_JPRB, 0.57144E+00_JPRB, 0.59616E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.66145E+00_JPRB, 0.81125E+00_JPRB, 0.91434E+00_JPRB, 0.99454E+00_JPRB, 0.11798E+01_JPRB, 0.85726E+00_JPRB ,&
&  0.95686E+00_JPRB, 0.14002E+01_JPRB, 0.15395E+01_JPRB, 0.17224E+01_JPRB, 0.18329E+01_JPRB, 0.17706E+01_JPRB, 0.15409E+01_JPRB ,&
&  0.12776E+01_JPRB, 0.11135E+01_JPRB, 0.10215E+01_JPRB, 0.96266E+00_JPRB, 0.92267E+00_JPRB, 0.89471E+00_JPRB, 0.86177E+00_JPRB ,&
&  0.84713E+00_JPRB, 0.82539E+00_JPRB, 0.79466E+00_JPRB, 0.79068E+00_JPRB, 0.78523E+00_JPRB, 0.77245E+00_JPRB, 0.78463E+00_JPRB ,&
&  0.80814E+00_JPRB, 0.82608E+00_JPRB, 0.84247E+00_JPRB, 0.82919E+00_JPRB, 0.86639E+00_JPRB, 0.89624E+00_JPRB, 0.92472E+00_JPRB ,&
&  0.88747E+00_JPRB, 0.86235E+00_JPRB, 0.98028E+00_JPRB, 0.10846E+01_JPRB, 0.11366E+01_JPRB, 0.11012E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10742E+01_JPRB, 0.10262E+01_JPRB, 0.90378E+00_JPRB, 0.97034E+00_JPRB, 0.11111E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.12269E+01_JPRB, 0.12010E+01_JPRB, 0.11681E+01_JPRB, 0.11828E+01_JPRB, 0.12666E+01_JPRB, 0.12546E+01_JPRB ,&
&  0.13596E+01_JPRB, 0.15121E+01_JPRB, 0.13141E+01_JPRB, 0.11025E+01_JPRB, 0.93849E+00_JPRB, 0.81138E+00_JPRB, 0.78951E+00_JPRB ,&
&  0.84159E+00_JPRB, 0.84085E+00_JPRB, 0.81819E+00_JPRB, 0.86989E+00_JPRB, 0.91294E+00_JPRB, 0.10442E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.92262E+00_JPRB, 0.84411E+00_JPRB, 0.75432E+00_JPRB, 0.70760E+00_JPRB, 0.68243E+00_JPRB, 0.68009E+00_JPRB, 0.67530E+00_JPRB ,&
&  0.67332E+00_JPRB, 0.66364E+00_JPRB, 0.64722E+00_JPRB, 0.63388E+00_JPRB, 0.61857E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.61340E+00_JPRB, 0.60185E+00_JPRB, 0.60074E+00_JPRB, 0.60333E+00_JPRB, 0.61585E+00_JPRB, 0.61181E+00_JPRB, 0.60933E+00_JPRB ,&
&  0.64713E+00_JPRB, 0.56484E+00_JPRB, 0.53138E+00_JPRB, 0.50764E+00_JPRB, 0.51334E+00_JPRB, 0.53985E+00_JPRB, 0.57142E+00_JPRB ,&
&  0.59605E+00_JPRB, 0.62547E+00_JPRB, 0.72157E+00_JPRB, 0.84529E+00_JPRB, 0.86594E+00_JPRB, 0.79329E+00_JPRB, 0.83680E+00_JPRB ,&
&  0.10893E+01_JPRB, 0.13825E+01_JPRB, 0.15343E+01_JPRB, 0.17150E+01_JPRB, 0.19448E+01_JPRB, 0.20545E+01_JPRB, 0.17664E+01_JPRB ,&
&  0.15274E+01_JPRB, 0.12753E+01_JPRB, 0.10780E+01_JPRB, 0.96682E+00_JPRB, 0.91729E+00_JPRB, 0.87062E+00_JPRB, 0.83591E+00_JPRB ,&
&  0.80143E+00_JPRB, 0.77636E+00_JPRB, 0.76161E+00_JPRB, 0.75061E+00_JPRB, 0.75060E+00_JPRB, 0.73313E+00_JPRB, 0.73576E+00_JPRB ,&
&  0.76067E+00_JPRB, 0.78780E+00_JPRB, 0.81844E+00_JPRB, 0.81344E+00_JPRB, 0.85205E+00_JPRB, 0.89087E+00_JPRB, 0.94885E+00_JPRB ,&
&  0.96625E+00_JPRB, 0.10219E+01_JPRB, 0.10750E+01_JPRB, 0.10922E+01_JPRB, 0.11610E+01_JPRB, 0.11839E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.10664E+01_JPRB, 0.95413E+00_JPRB, 0.96502E+00_JPRB, 0.10639E+01_JPRB, 0.12592E+01_JPRB, 0.11651E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.96128E+00_JPRB, 0.10209E+01_JPRB, 0.11388E+01_JPRB, 0.11711E+01_JPRB, 0.12424E+01_JPRB, 0.14262E+01_JPRB ,&
&  0.15048E+01_JPRB, 0.97752E+00_JPRB, 0.82505E+00_JPRB, 0.92898E+00_JPRB, 0.79629E+00_JPRB, 0.66223E+00_JPRB, 0.73629E+00_JPRB ,&
&  0.78789E+00_JPRB, 0.84625E+00_JPRB, 0.85405E+00_JPRB, 0.78143E+00_JPRB, 0.82741E+00_JPRB, 0.10067E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.94924E+00_JPRB, 0.87990E+00_JPRB, 0.80798E+00_JPRB, 0.74885E+00_JPRB, 0.71117E+00_JPRB, 0.69124E+00_JPRB ,&
&  0.68507E+00_JPRB, 0.68035E+00_JPRB, 0.67098E+00_JPRB, 0.65309E+00_JPRB, 0.63194E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.63139E+00_JPRB, 0.61124E+00_JPRB, 0.59831E+00_JPRB, 0.58838E+00_JPRB, 0.58897E+00_JPRB, 0.58486E+00_JPRB, 0.57297E+00_JPRB ,&
&  0.60585E+00_JPRB, 0.56120E+00_JPRB, 0.52322E+00_JPRB, 0.50186E+00_JPRB, 0.50193E+00_JPRB, 0.51861E+00_JPRB, 0.55207E+00_JPRB ,&
&  0.59688E+00_JPRB, 0.63060E+00_JPRB, 0.71920E+00_JPRB, 0.81934E+00_JPRB, 0.80741E+00_JPRB, 0.70861E+00_JPRB, 0.81994E+00_JPRB ,&
&  0.93454E+00_JPRB, 0.12313E+01_JPRB, 0.16899E+01_JPRB, 0.19471E+01_JPRB, 0.20055E+01_JPRB, 0.20969E+01_JPRB, 0.19197E+01_JPRB ,&
&  0.17775E+01_JPRB, 0.15437E+01_JPRB, 0.12609E+01_JPRB, 0.10744E+01_JPRB, 0.94981E+00_JPRB, 0.89413E+00_JPRB, 0.83900E+00_JPRB ,&
&  0.81436E+00_JPRB, 0.77407E+00_JPRB, 0.73851E+00_JPRB, 0.72124E+00_JPRB, 0.71328E+00_JPRB, 0.70804E+00_JPRB, 0.69937E+00_JPRB ,&
&  0.70970E+00_JPRB, 0.73526E+00_JPRB, 0.78104E+00_JPRB, 0.82356E+00_JPRB, 0.73068E+00_JPRB, 0.87566E+00_JPRB, 0.95187E+00_JPRB ,&
&  0.10348E+01_JPRB, 0.11144E+01_JPRB, 0.11006E+01_JPRB, 0.11957E+01_JPRB, 0.12366E+01_JPRB, 0.11859E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.12245E+01_JPRB, 0.11189E+01_JPRB, 0.10271E+01_JPRB, 0.10983E+01_JPRB, 0.11570E+01_JPRB, 0.13435E+01_JPRB, 0.94141E+00_JPRB ,&
&  0.87652E+00_JPRB, 0.10661E+01_JPRB, 0.10288E+01_JPRB, 0.98550E+00_JPRB, 0.10835E+01_JPRB, 0.11804E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.56275E+00_JPRB, 0.48616E+00_JPRB, 0.51276E+00_JPRB, 0.51847E+00_JPRB, 0.50291E+00_JPRB, 0.53276E+00_JPRB, 0.76729E+00_JPRB ,&
&  0.74716E+00_JPRB, 0.82512E+00_JPRB, 0.84667E+00_JPRB, 0.81067E+00_JPRB, 0.78822E+00_JPRB, 0.85106E+00_JPRB, 0.12634E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.10061E+01_JPRB, 0.93115E+00_JPRB, 0.88179E+00_JPRB, 0.82169E+00_JPRB, 0.76837E+00_JPRB, 0.72512E+00_JPRB ,&
&  0.69635E+00_JPRB, 0.68127E+00_JPRB, 0.67332E+00_JPRB, 0.66475E+00_JPRB, 0.65022E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.62728E+00_JPRB, 0.60766E+00_JPRB, 0.59446E+00_JPRB, 0.58491E+00_JPRB, 0.57631E+00_JPRB, 0.57421E+00_JPRB, 0.55892E+00_JPRB ,&
&  0.56846E+00_JPRB, 0.56348E+00_JPRB, 0.52936E+00_JPRB, 0.50676E+00_JPRB, 0.49907E+00_JPRB, 0.51375E+00_JPRB, 0.55588E+00_JPRB ,&
&  0.59107E+00_JPRB, 0.65297E+00_JPRB, 0.79768E+00_JPRB, 0.76657E+00_JPRB, 0.78724E+00_JPRB, 0.69772E+00_JPRB, 0.74595E+00_JPRB ,&
&  0.84405E+00_JPRB, 0.10442E+01_JPRB, 0.14182E+01_JPRB, 0.17145E+01_JPRB, 0.20532E+01_JPRB, 0.21751E+01_JPRB, 0.21265E+01_JPRB ,&
&  0.19774E+01_JPRB, 0.18232E+01_JPRB, 0.15731E+01_JPRB, 0.13014E+01_JPRB, 0.11104E+01_JPRB, 0.98056E+00_JPRB, 0.89831E+00_JPRB ,&
&  0.85715E+00_JPRB, 0.83218E+00_JPRB, 0.76636E+00_JPRB, 0.72367E+00_JPRB, 0.70009E+00_JPRB, 0.68727E+00_JPRB, 0.66500E+00_JPRB ,&
&  0.66431E+00_JPRB, 0.68808E+00_JPRB, 0.73465E+00_JPRB, 0.78584E+00_JPRB, 0.73917E+00_JPRB, 0.70316E+00_JPRB, 0.87078E+00_JPRB ,&
&  0.10427E+01_JPRB, 0.12707E+01_JPRB, 0.13472E+01_JPRB, 0.14820E+01_JPRB, 0.13811E+01_JPRB, 0.12613E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.14044E+01_JPRB, 0.11380E+01_JPRB, 0.12023E+01_JPRB, 0.13523E+01_JPRB, 0.11398E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.12515E+01_JPRB, 0.11325E+01_JPRB, 0.10075E+01_JPRB, 0.87093E+00_JPRB, 0.10290E+01_JPRB, 0.98719E+00_JPRB, 0.56956E+00_JPRB ,&
&  0.48293E+00_JPRB, 0.46572E+00_JPRB, 0.44819E+00_JPRB, 0.43003E+00_JPRB, 0.43192E+00_JPRB, 0.46184E+00_JPRB, 0.60827E+00_JPRB ,&
&  0.65329E+00_JPRB, 0.73991E+00_JPRB, 0.79123E+00_JPRB, 0.82059E+00_JPRB, 0.78740E+00_JPRB, 0.83981E+00_JPRB, 0.99707E+00_JPRB ,&
&  0.11325E+01_JPRB, 0.11066E+01_JPRB, 0.10157E+01_JPRB, 0.94031E+00_JPRB, 0.86844E+00_JPRB, 0.80934E+00_JPRB, 0.76578E+00_JPRB ,&
&  0.73152E+00_JPRB, 0.70872E+00_JPRB, 0.68974E+00_JPRB, 0.67257E+00_JPRB, 0.64935E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.64456E+00_JPRB, 0.62125E+00_JPRB, 0.59554E+00_JPRB, 0.57860E+00_JPRB, 0.57073E+00_JPRB, 0.56179E+00_JPRB, 0.54940E+00_JPRB ,&
&  0.53947E+00_JPRB, 0.55409E+00_JPRB, 0.53340E+00_JPRB, 0.51690E+00_JPRB, 0.50434E+00_JPRB, 0.51061E+00_JPRB, 0.54427E+00_JPRB ,&
&  0.59068E+00_JPRB, 0.68114E+00_JPRB, 0.75023E+00_JPRB, 0.69141E+00_JPRB, 0.65982E+00_JPRB, 0.62362E+00_JPRB, 0.62012E+00_JPRB ,&
&  0.75918E+00_JPRB, 0.89568E+00_JPRB, 0.11283E+01_JPRB, 0.15250E+01_JPRB, 0.19977E+01_JPRB, 0.21561E+01_JPRB, 0.23567E+01_JPRB ,&
&  0.22108E+01_JPRB, 0.20042E+01_JPRB, 0.18330E+01_JPRB, 0.16037E+01_JPRB, 0.13362E+01_JPRB, 0.11578E+01_JPRB, 0.10391E+01_JPRB ,&
&  0.95652E+00_JPRB, 0.90129E+00_JPRB, 0.84464E+00_JPRB, 0.77180E+00_JPRB, 0.71268E+00_JPRB, 0.68156E+00_JPRB, 0.65513E+00_JPRB ,&
&  0.64432E+00_JPRB, 0.66553E+00_JPRB, 0.70814E+00_JPRB, 0.77248E+00_JPRB, 0.84945E+00_JPRB, 0.87769E+00_JPRB, 0.87284E+00_JPRB ,&
&  0.94955E+00_JPRB, 0.10888E+01_JPRB, 0.15082E+01_JPRB, 0.15241E+01_JPRB, 0.15060E+01_JPRB, 0.14851E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.14569E+01_JPRB, 0.16379E+01_JPRB, 0.13720E+01_JPRB, 0.14654E+01_JPRB, 0.13865E+01_JPRB, 0.11670E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.13967E+01_JPRB, 0.12799E+01_JPRB, 0.10449E+01_JPRB, 0.79622E+00_JPRB, 0.76600E+00_JPRB, 0.61299E+00_JPRB, 0.45207E+00_JPRB ,&
&  0.42816E+00_JPRB, 0.41486E+00_JPRB, 0.40145E+00_JPRB, 0.40700E+00_JPRB, 0.40563E+00_JPRB, 0.44970E+00_JPRB, 0.54745E+00_JPRB ,&
&  0.63731E+00_JPRB, 0.69977E+00_JPRB, 0.75437E+00_JPRB, 0.78378E+00_JPRB, 0.78289E+00_JPRB, 0.77508E+00_JPRB, 0.99073E+00_JPRB ,&
&  0.97918E+00_JPRB, 0.10016E+01_JPRB, 0.10104E+01_JPRB, 0.95783E+00_JPRB, 0.89401E+00_JPRB, 0.83910E+00_JPRB, 0.78835E+00_JPRB ,&
&  0.75122E+00_JPRB, 0.73163E+00_JPRB, 0.71493E+00_JPRB, 0.69311E+00_JPRB, 0.66918E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.64970E+00_JPRB, 0.62655E+00_JPRB, 0.60514E+00_JPRB, 0.59067E+00_JPRB, 0.57919E+00_JPRB, 0.56709E+00_JPRB, 0.55081E+00_JPRB ,&
&  0.52951E+00_JPRB, 0.53273E+00_JPRB, 0.52976E+00_JPRB, 0.50672E+00_JPRB, 0.49686E+00_JPRB, 0.50589E+00_JPRB, 0.53708E+00_JPRB ,&
&  0.58713E+00_JPRB, 0.73787E+00_JPRB, 0.63072E+00_JPRB, 0.58602E+00_JPRB, 0.61123E+00_JPRB, 0.61564E+00_JPRB, 0.57053E+00_JPRB ,&
&  0.70528E+00_JPRB, 0.83266E+00_JPRB, 0.10263E+01_JPRB, 0.14813E+01_JPRB, 0.22542E+01_JPRB, 0.23889E+01_JPRB, 0.25312E+01_JPRB ,&
&  0.26090E+01_JPRB, 0.23601E+01_JPRB, 0.21358E+01_JPRB, 0.18856E+01_JPRB, 0.15672E+01_JPRB, 0.13257E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.96385E+00_JPRB, 0.88293E+00_JPRB, 0.81708E+00_JPRB, 0.75636E+00_JPRB, 0.70924E+00_JPRB, 0.68117E+00_JPRB ,&
&  0.66337E+00_JPRB, 0.66710E+00_JPRB, 0.69763E+00_JPRB, 0.76205E+00_JPRB, 0.83188E+00_JPRB, 0.84658E+00_JPRB, 0.11069E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.13980E+01_JPRB, 0.14618E+01_JPRB, 0.17029E+01_JPRB, 0.18691E+01_JPRB, 0.18136E+01_JPRB, 0.19096E+01_JPRB ,&
&  0.16122E+01_JPRB, 0.15384E+01_JPRB, 0.15967E+01_JPRB, 0.16093E+01_JPRB, 0.12725E+01_JPRB, 0.13366E+01_JPRB, 0.18700E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.14446E+01_JPRB, 0.15278E+01_JPRB, 0.15079E+01_JPRB, 0.67094E+00_JPRB, 0.50541E+00_JPRB, 0.68623E+00_JPRB ,&
&  0.69578E+00_JPRB, 0.50182E+00_JPRB, 0.42816E+00_JPRB, 0.48534E+00_JPRB, 0.42557E+00_JPRB, 0.47683E+00_JPRB, 0.59592E+00_JPRB ,&
&  0.60135E+00_JPRB, 0.61292E+00_JPRB, 0.71748E+00_JPRB, 0.75437E+00_JPRB, 0.77993E+00_JPRB, 0.77677E+00_JPRB, 0.87568E+00_JPRB ,&
&  0.92137E+00_JPRB, 0.10258E+01_JPRB, 0.10756E+01_JPRB, 0.10129E+01_JPRB, 0.92932E+00_JPRB, 0.85570E+00_JPRB, 0.79894E+00_JPRB ,&
&  0.75206E+00_JPRB, 0.72392E+00_JPRB, 0.70610E+00_JPRB, 0.68443E+00_JPRB, 0.66618E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.61804E+00_JPRB, 0.59983E+00_JPRB, 0.58368E+00_JPRB, 0.57030E+00_JPRB, 0.56370E+00_JPRB, 0.55751E+00_JPRB, 0.54882E+00_JPRB ,&
&  0.53191E+00_JPRB, 0.51935E+00_JPRB, 0.52493E+00_JPRB, 0.50471E+00_JPRB, 0.48947E+00_JPRB, 0.49340E+00_JPRB, 0.53261E+00_JPRB ,&
&  0.57922E+00_JPRB, 0.62847E+00_JPRB, 0.52868E+00_JPRB, 0.49745E+00_JPRB, 0.58925E+00_JPRB, 0.56759E+00_JPRB, 0.58723E+00_JPRB ,&
&  0.68873E+00_JPRB, 0.79259E+00_JPRB, 0.96485E+00_JPRB, 0.13616E+01_JPRB, 0.17748E+01_JPRB, 0.22171E+01_JPRB, 0.25402E+01_JPRB ,&
&  0.26713E+01_JPRB, 0.25859E+01_JPRB, 0.23869E+01_JPRB, 0.21445E+01_JPRB, 0.17792E+01_JPRB, 0.14905E+01_JPRB, 0.12871E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.10019E+01_JPRB, 0.90475E+00_JPRB, 0.79568E+00_JPRB, 0.74212E+00_JPRB, 0.69835E+00_JPRB, 0.67388E+00_JPRB ,&
&  0.66709E+00_JPRB, 0.67594E+00_JPRB, 0.69997E+00_JPRB, 0.76842E+00_JPRB, 0.75412E+00_JPRB, 0.77924E+00_JPRB, 0.96187E+00_JPRB ,&
&  0.13386E+01_JPRB, 0.14099E+01_JPRB, 0.15136E+01_JPRB, 0.17556E+01_JPRB, 0.19963E+01_JPRB, 0.19233E+01_JPRB, 0.20870E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.13694E+01_JPRB, 0.11973E+01_JPRB, 0.97474E+00_JPRB, 0.11970E+01_JPRB, 0.17762E+01_JPRB, 0.17402E+01_JPRB ,&
&  0.15819E+01_JPRB, 0.15268E+01_JPRB, 0.15063E+01_JPRB, 0.12319E+01_JPRB, 0.96929E+00_JPRB, 0.74187E+00_JPRB, 0.85460E+00_JPRB ,&
&  0.86219E+00_JPRB, 0.81642E+00_JPRB, 0.71942E+00_JPRB, 0.62883E+00_JPRB, 0.54054E+00_JPRB, 0.63205E+00_JPRB, 0.62510E+00_JPRB ,&
&  0.57900E+00_JPRB, 0.55600E+00_JPRB, 0.60917E+00_JPRB, 0.67673E+00_JPRB, 0.72869E+00_JPRB, 0.69658E+00_JPRB, 0.79251E+00_JPRB ,&
&  0.82337E+00_JPRB, 0.87482E+00_JPRB, 0.88914E+00_JPRB, 0.87104E+00_JPRB, 0.83418E+00_JPRB, 0.78702E+00_JPRB, 0.74538E+00_JPRB ,&
&  0.71529E+00_JPRB, 0.69004E+00_JPRB, 0.66740E+00_JPRB, 0.64868E+00_JPRB, 0.63432E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.57963E+00_JPRB, 0.56998E+00_JPRB, 0.55968E+00_JPRB, 0.55332E+00_JPRB, 0.54761E+00_JPRB, 0.54540E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.53280E+00_JPRB, 0.51548E+00_JPRB, 0.51990E+00_JPRB, 0.51105E+00_JPRB, 0.49586E+00_JPRB, 0.49201E+00_JPRB, 0.51376E+00_JPRB ,&
&  0.55775E+00_JPRB, 0.55162E+00_JPRB, 0.45179E+00_JPRB, 0.47202E+00_JPRB, 0.52207E+00_JPRB, 0.50511E+00_JPRB, 0.55845E+00_JPRB ,&
&  0.63476E+00_JPRB, 0.73333E+00_JPRB, 0.83686E+00_JPRB, 0.10644E+01_JPRB, 0.13450E+01_JPRB, 0.17069E+01_JPRB, 0.20535E+01_JPRB ,&
&  0.21554E+01_JPRB, 0.23227E+01_JPRB, 0.23927E+01_JPRB, 0.22109E+01_JPRB, 0.19604E+01_JPRB, 0.16956E+01_JPRB, 0.14711E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.11600E+01_JPRB, 0.99810E+00_JPRB, 0.86829E+00_JPRB, 0.78728E+00_JPRB, 0.74264E+00_JPRB, 0.71369E+00_JPRB ,&
&  0.69934E+00_JPRB, 0.69233E+00_JPRB, 0.69495E+00_JPRB, 0.75726E+00_JPRB, 0.74367E+00_JPRB, 0.82914E+00_JPRB, 0.10046E+01_JPRB ,&
&  0.15367E+01_JPRB, 0.16482E+01_JPRB, 0.16787E+01_JPRB, 0.20296E+01_JPRB, 0.16498E+01_JPRB, 0.16587E+01_JPRB, 0.19357E+01_JPRB ,&
&  0.20777E+01_JPRB, 0.20212E+01_JPRB, 0.19709E+01_JPRB, 0.14944E+01_JPRB, 0.13114E+01_JPRB, 0.17952E+01_JPRB, 0.16808E+01_JPRB ,&
&  0.14913E+01_JPRB, 0.14263E+01_JPRB, 0.13465E+01_JPRB, 0.14611E+01_JPRB, 0.10817E+01_JPRB, 0.90287E+00_JPRB, 0.69858E+00_JPRB ,&
&  0.67531E+00_JPRB, 0.62743E+00_JPRB, 0.69716E+00_JPRB, 0.66295E+00_JPRB, 0.64264E+00_JPRB, 0.63748E+00_JPRB, 0.61849E+00_JPRB ,&
&  0.61746E+00_JPRB, 0.59785E+00_JPRB, 0.58515E+00_JPRB, 0.63325E+00_JPRB, 0.70171E+00_JPRB, 0.67068E+00_JPRB, 0.71607E+00_JPRB ,&
&  0.73167E+00_JPRB, 0.76127E+00_JPRB, 0.76623E+00_JPRB, 0.76952E+00_JPRB, 0.75609E+00_JPRB, 0.73005E+00_JPRB, 0.69166E+00_JPRB ,&
&  0.66667E+00_JPRB, 0.64766E+00_JPRB, 0.62629E+00_JPRB, 0.60851E+00_JPRB, 0.59278E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.52376E+00_JPRB, 0.51871E+00_JPRB, 0.51732E+00_JPRB, 0.51971E+00_JPRB, 0.52328E+00_JPRB, 0.52584E+00_JPRB, 0.52672E+00_JPRB ,&
&  0.51966E+00_JPRB, 0.50286E+00_JPRB, 0.49999E+00_JPRB, 0.50144E+00_JPRB, 0.48856E+00_JPRB, 0.48503E+00_JPRB, 0.50005E+00_JPRB ,&
&  0.53891E+00_JPRB, 0.58007E+00_JPRB, 0.49006E+00_JPRB, 0.43158E+00_JPRB, 0.41553E+00_JPRB, 0.46755E+00_JPRB, 0.52435E+00_JPRB ,&
&  0.61801E+00_JPRB, 0.69408E+00_JPRB, 0.79750E+00_JPRB, 0.92007E+00_JPRB, 0.10970E+01_JPRB, 0.13247E+01_JPRB, 0.16691E+01_JPRB ,&
&  0.21743E+01_JPRB, 0.20060E+01_JPRB, 0.21331E+01_JPRB, 0.20601E+01_JPRB, 0.19808E+01_JPRB, 0.17738E+01_JPRB, 0.15653E+01_JPRB ,&
&  0.13703E+01_JPRB, 0.12248E+01_JPRB, 0.10747E+01_JPRB, 0.94298E+00_JPRB, 0.84626E+00_JPRB, 0.77587E+00_JPRB, 0.73465E+00_JPRB ,&
&  0.71992E+00_JPRB, 0.70664E+00_JPRB, 0.69267E+00_JPRB, 0.71713E+00_JPRB, 0.83573E+00_JPRB, 0.99326E+00_JPRB, 0.12254E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.13553E+01_JPRB, 0.17914E+01_JPRB, 0.19924E+01_JPRB, 0.20940E+01_JPRB, 0.20851E+01_JPRB, 0.21193E+01_JPRB ,&
&  0.22728E+01_JPRB, 0.21775E+01_JPRB, 0.20849E+01_JPRB, 0.17771E+01_JPRB, 0.19828E+01_JPRB, 0.18975E+01_JPRB, 0.17227E+01_JPRB ,&
&  0.15274E+01_JPRB, 0.14522E+01_JPRB, 0.13315E+01_JPRB, 0.13077E+01_JPRB, 0.13012E+01_JPRB, 0.13042E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.80802E+00_JPRB, 0.85529E+00_JPRB, 0.76327E+00_JPRB, 0.67501E+00_JPRB, 0.59873E+00_JPRB, 0.62175E+00_JPRB, 0.65137E+00_JPRB ,&
&  0.67224E+00_JPRB, 0.67635E+00_JPRB, 0.66288E+00_JPRB, 0.68878E+00_JPRB, 0.72992E+00_JPRB, 0.73656E+00_JPRB, 0.72043E+00_JPRB ,&
&  0.68397E+00_JPRB, 0.68599E+00_JPRB, 0.69035E+00_JPRB, 0.67072E+00_JPRB, 0.69064E+00_JPRB, 0.66566E+00_JPRB, 0.63413E+00_JPRB ,&
&  0.60694E+00_JPRB, 0.58645E+00_JPRB, 0.56832E+00_JPRB, 0.55022E+00_JPRB, 0.53333E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.47502E+00_JPRB, 0.47566E+00_JPRB, 0.47683E+00_JPRB, 0.48132E+00_JPRB, 0.48686E+00_JPRB, 0.49631E+00_JPRB, 0.50160E+00_JPRB ,&
&  0.50089E+00_JPRB, 0.48898E+00_JPRB, 0.48092E+00_JPRB, 0.48739E+00_JPRB, 0.48292E+00_JPRB, 0.48007E+00_JPRB, 0.49273E+00_JPRB ,&
&  0.53220E+00_JPRB, 0.65937E+00_JPRB, 0.55442E+00_JPRB, 0.51170E+00_JPRB, 0.46966E+00_JPRB, 0.50552E+00_JPRB, 0.54411E+00_JPRB ,&
&  0.64480E+00_JPRB, 0.69263E+00_JPRB, 0.72227E+00_JPRB, 0.78434E+00_JPRB, 0.92707E+00_JPRB, 0.11389E+01_JPRB, 0.13940E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.17379E+01_JPRB, 0.17557E+01_JPRB, 0.19734E+01_JPRB, 0.18637E+01_JPRB, 0.15952E+01_JPRB, 0.15574E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.12170E+01_JPRB, 0.10619E+01_JPRB, 0.94617E+00_JPRB, 0.85897E+00_JPRB, 0.79322E+00_JPRB, 0.74369E+00_JPRB ,&
&  0.70894E+00_JPRB, 0.68054E+00_JPRB, 0.65101E+00_JPRB, 0.63594E+00_JPRB, 0.70049E+00_JPRB, 0.94837E+00_JPRB, 0.12728E+01_JPRB ,&
&  0.18393E+01_JPRB, 0.16706E+01_JPRB, 0.14362E+01_JPRB, 0.19228E+01_JPRB, 0.24372E+01_JPRB, 0.24050E+01_JPRB, 0.22718E+01_JPRB ,&
&  0.24734E+01_JPRB, 0.24797E+01_JPRB, 0.24033E+01_JPRB, 0.23893E+01_JPRB, 0.22866E+01_JPRB, 0.21228E+01_JPRB, 0.19135E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.14612E+01_JPRB, 0.13605E+01_JPRB, 0.13564E+01_JPRB, 0.12643E+01_JPRB, 0.11945E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.99464E+00_JPRB, 0.85512E+00_JPRB, 0.63402E+00_JPRB, 0.66267E+00_JPRB, 0.55773E+00_JPRB, 0.57437E+00_JPRB, 0.66660E+00_JPRB ,&
&  0.67331E+00_JPRB, 0.72417E+00_JPRB, 0.73994E+00_JPRB, 0.70819E+00_JPRB, 0.76928E+00_JPRB, 0.74290E+00_JPRB, 0.70304E+00_JPRB ,&
&  0.68622E+00_JPRB, 0.61183E+00_JPRB, 0.62536E+00_JPRB, 0.61937E+00_JPRB, 0.59784E+00_JPRB, 0.57110E+00_JPRB, 0.54617E+00_JPRB ,&
&  0.52638E+00_JPRB, 0.51170E+00_JPRB, 0.50255E+00_JPRB, 0.48915E+00_JPRB, 0.47871E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.43570E+00_JPRB, 0.43590E+00_JPRB, 0.43492E+00_JPRB, 0.44000E+00_JPRB, 0.44873E+00_JPRB, 0.46075E+00_JPRB, 0.47304E+00_JPRB ,&
&  0.48104E+00_JPRB, 0.47904E+00_JPRB, 0.47090E+00_JPRB, 0.47193E+00_JPRB, 0.47153E+00_JPRB, 0.47198E+00_JPRB, 0.48594E+00_JPRB ,&
&  0.54197E+00_JPRB, 0.63171E+00_JPRB, 0.57220E+00_JPRB, 0.50378E+00_JPRB, 0.52803E+00_JPRB, 0.52994E+00_JPRB, 0.56974E+00_JPRB ,&
&  0.59735E+00_JPRB, 0.66523E+00_JPRB, 0.69395E+00_JPRB, 0.72289E+00_JPRB, 0.78754E+00_JPRB, 0.94449E+00_JPRB, 0.11384E+01_JPRB ,&
&  0.12430E+01_JPRB, 0.13225E+01_JPRB, 0.14367E+01_JPRB, 0.15470E+01_JPRB, 0.15572E+01_JPRB, 0.13890E+01_JPRB, 0.13724E+01_JPRB ,&
&  0.12588E+01_JPRB, 0.11225E+01_JPRB, 0.97824E+00_JPRB, 0.86279E+00_JPRB, 0.78418E+00_JPRB, 0.73231E+00_JPRB, 0.70020E+00_JPRB ,&
&  0.68085E+00_JPRB, 0.65816E+00_JPRB, 0.62536E+00_JPRB, 0.61822E+00_JPRB, 0.66490E+00_JPRB, 0.10486E+01_JPRB, 0.17418E+01_JPRB ,&
&  0.22795E+01_JPRB, 0.23677E+01_JPRB, 0.24717E+01_JPRB, 0.27198E+01_JPRB, 0.27983E+01_JPRB, 0.28658E+01_JPRB, 0.29070E+01_JPRB ,&
&  0.28246E+01_JPRB, 0.26954E+01_JPRB, 0.25049E+01_JPRB, 0.22767E+01_JPRB, 0.21588E+01_JPRB, 0.20824E+01_JPRB, 0.19224E+01_JPRB ,&
&  0.16668E+01_JPRB, 0.15016E+01_JPRB, 0.14378E+01_JPRB, 0.13549E+01_JPRB, 0.12868E+01_JPRB, 0.11718E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.72848E+00_JPRB, 0.59822E+00_JPRB, 0.68988E+00_JPRB, 0.60301E+00_JPRB, 0.63175E+00_JPRB, 0.75158E+00_JPRB ,&
&  0.76037E+00_JPRB, 0.75347E+00_JPRB, 0.78059E+00_JPRB, 0.73988E+00_JPRB, 0.73297E+00_JPRB, 0.73681E+00_JPRB, 0.67729E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.61560E+00_JPRB, 0.62694E+00_JPRB, 0.59544E+00_JPRB, 0.55434E+00_JPRB, 0.51912E+00_JPRB, 0.49454E+00_JPRB ,&
&  0.47731E+00_JPRB, 0.46654E+00_JPRB, 0.46118E+00_JPRB, 0.44671E+00_JPRB, 0.43614E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.41034E+00_JPRB, 0.40684E+00_JPRB, 0.40699E+00_JPRB, 0.41311E+00_JPRB, 0.42251E+00_JPRB, 0.43661E+00_JPRB, 0.45206E+00_JPRB ,&
&  0.46660E+00_JPRB, 0.47190E+00_JPRB, 0.46444E+00_JPRB, 0.45910E+00_JPRB, 0.45855E+00_JPRB, 0.46144E+00_JPRB, 0.48054E+00_JPRB ,&
&  0.44835E+00_JPRB, 0.49484E+00_JPRB, 0.45163E+00_JPRB, 0.46686E+00_JPRB, 0.52821E+00_JPRB, 0.53345E+00_JPRB, 0.54934E+00_JPRB ,&
&  0.56610E+00_JPRB, 0.60000E+00_JPRB, 0.61886E+00_JPRB, 0.64168E+00_JPRB, 0.68067E+00_JPRB, 0.77706E+00_JPRB, 0.94111E+00_JPRB ,&
&  0.98402E+00_JPRB, 0.97022E+00_JPRB, 0.10325E+01_JPRB, 0.11008E+01_JPRB, 0.10965E+01_JPRB, 0.11449E+01_JPRB, 0.11733E+01_JPRB ,&
&  0.11039E+01_JPRB, 0.10161E+01_JPRB, 0.92283E+00_JPRB, 0.83215E+00_JPRB, 0.75677E+00_JPRB, 0.69386E+00_JPRB, 0.65189E+00_JPRB ,&
&  0.62290E+00_JPRB, 0.60293E+00_JPRB, 0.58964E+00_JPRB, 0.61649E+00_JPRB, 0.66826E+00_JPRB, 0.12361E+01_JPRB, 0.22938E+01_JPRB ,&
&  0.26187E+01_JPRB, 0.26484E+01_JPRB, 0.28102E+01_JPRB, 0.31661E+01_JPRB, 0.31063E+01_JPRB, 0.29997E+01_JPRB, 0.28612E+01_JPRB ,&
&  0.27124E+01_JPRB, 0.24861E+01_JPRB, 0.23249E+01_JPRB, 0.22568E+01_JPRB, 0.21035E+01_JPRB, 0.20878E+01_JPRB, 0.19126E+01_JPRB ,&
&  0.16671E+01_JPRB, 0.15585E+01_JPRB, 0.15090E+01_JPRB, 0.14875E+01_JPRB, 0.14291E+01_JPRB, 0.14420E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.13370E+01_JPRB, 0.11512E+01_JPRB, 0.87073E+00_JPRB, 0.81518E+00_JPRB, 0.66086E+00_JPRB, 0.69782E+00_JPRB, 0.86067E+00_JPRB ,&
&  0.86430E+00_JPRB, 0.80202E+00_JPRB, 0.74020E+00_JPRB, 0.70840E+00_JPRB, 0.69772E+00_JPRB, 0.72583E+00_JPRB, 0.66603E+00_JPRB ,&
&  0.61051E+00_JPRB, 0.62123E+00_JPRB, 0.61432E+00_JPRB, 0.58785E+00_JPRB, 0.55304E+00_JPRB, 0.51351E+00_JPRB, 0.48795E+00_JPRB ,&
&  0.47571E+00_JPRB, 0.46183E+00_JPRB, 0.44279E+00_JPRB, 0.42314E+00_JPRB, 0.41386E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.39881E+00_JPRB, 0.39301E+00_JPRB, 0.39072E+00_JPRB, 0.39652E+00_JPRB, 0.40657E+00_JPRB, 0.41856E+00_JPRB, 0.43444E+00_JPRB ,&
&  0.45154E+00_JPRB, 0.46115E+00_JPRB, 0.45794E+00_JPRB, 0.44973E+00_JPRB, 0.44577E+00_JPRB, 0.44843E+00_JPRB, 0.42891E+00_JPRB ,&
&  0.39456E+00_JPRB, 0.42874E+00_JPRB, 0.42481E+00_JPRB, 0.43641E+00_JPRB, 0.46989E+00_JPRB, 0.47861E+00_JPRB, 0.49167E+00_JPRB ,&
&  0.52044E+00_JPRB, 0.53345E+00_JPRB, 0.55152E+00_JPRB, 0.57299E+00_JPRB, 0.60185E+00_JPRB, 0.67177E+00_JPRB, 0.77865E+00_JPRB ,&
&  0.79618E+00_JPRB, 0.80970E+00_JPRB, 0.82589E+00_JPRB, 0.86699E+00_JPRB, 0.91853E+00_JPRB, 0.94922E+00_JPRB, 0.92944E+00_JPRB ,&
&  0.88529E+00_JPRB, 0.83542E+00_JPRB, 0.79594E+00_JPRB, 0.76012E+00_JPRB, 0.71171E+00_JPRB, 0.66007E+00_JPRB, 0.61996E+00_JPRB ,&
&  0.58854E+00_JPRB, 0.57096E+00_JPRB, 0.57316E+00_JPRB, 0.59375E+00_JPRB, 0.68193E+00_JPRB, 0.11215E+01_JPRB, 0.17486E+01_JPRB ,&
&  0.19821E+01_JPRB, 0.21773E+01_JPRB, 0.22276E+01_JPRB, 0.22814E+01_JPRB, 0.23227E+01_JPRB, 0.23312E+01_JPRB, 0.23111E+01_JPRB ,&
&  0.22913E+01_JPRB, 0.20878E+01_JPRB, 0.21776E+01_JPRB, 0.21880E+01_JPRB, 0.21072E+01_JPRB, 0.20595E+01_JPRB, 0.18478E+01_JPRB ,&
&  0.16685E+01_JPRB, 0.15400E+01_JPRB, 0.15399E+01_JPRB, 0.15207E+01_JPRB, 0.14635E+01_JPRB, 0.14783E+01_JPRB, 0.14188E+01_JPRB ,&
&  0.13917E+01_JPRB, 0.13509E+01_JPRB, 0.11704E+01_JPRB, 0.10204E+01_JPRB, 0.86320E+00_JPRB, 0.89503E+00_JPRB, 0.82847E+00_JPRB ,&
&  0.74337E+00_JPRB, 0.62882E+00_JPRB, 0.62991E+00_JPRB, 0.62626E+00_JPRB, 0.64450E+00_JPRB, 0.65202E+00_JPRB, 0.61200E+00_JPRB ,&
&  0.60343E+00_JPRB, 0.62044E+00_JPRB, 0.59375E+00_JPRB, 0.56555E+00_JPRB, 0.53581E+00_JPRB, 0.49678E+00_JPRB, 0.46630E+00_JPRB ,&
&  0.45264E+00_JPRB, 0.44710E+00_JPRB, 0.42813E+00_JPRB, 0.41006E+00_JPRB, 0.40262E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.40157E+00_JPRB, 0.40074E+00_JPRB, 0.39634E+00_JPRB, 0.39583E+00_JPRB, 0.40181E+00_JPRB, 0.41003E+00_JPRB, 0.41549E+00_JPRB ,&
&  0.43423E+00_JPRB, 0.44118E+00_JPRB, 0.44210E+00_JPRB, 0.43872E+00_JPRB, 0.43142E+00_JPRB, 0.39726E+00_JPRB, 0.34466E+00_JPRB ,&
&  0.33704E+00_JPRB, 0.35671E+00_JPRB, 0.38447E+00_JPRB, 0.40137E+00_JPRB, 0.41641E+00_JPRB, 0.42900E+00_JPRB, 0.43903E+00_JPRB ,&
&  0.44613E+00_JPRB, 0.45295E+00_JPRB, 0.47248E+00_JPRB, 0.50253E+00_JPRB, 0.53479E+00_JPRB, 0.57445E+00_JPRB, 0.63817E+00_JPRB ,&
&  0.64919E+00_JPRB, 0.67419E+00_JPRB, 0.71260E+00_JPRB, 0.68816E+00_JPRB, 0.76776E+00_JPRB, 0.77973E+00_JPRB, 0.77106E+00_JPRB ,&
&  0.75442E+00_JPRB, 0.72222E+00_JPRB, 0.68392E+00_JPRB, 0.65277E+00_JPRB, 0.62487E+00_JPRB, 0.60112E+00_JPRB, 0.56933E+00_JPRB ,&
&  0.54577E+00_JPRB, 0.53199E+00_JPRB, 0.53280E+00_JPRB, 0.54613E+00_JPRB, 0.59444E+00_JPRB, 0.87429E+00_JPRB, 0.11259E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.14643E+01_JPRB, 0.15074E+01_JPRB, 0.14926E+01_JPRB, 0.17123E+01_JPRB, 0.19087E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.19780E+01_JPRB, 0.18347E+01_JPRB, 0.18248E+01_JPRB, 0.18350E+01_JPRB, 0.18657E+01_JPRB, 0.17921E+01_JPRB, 0.16223E+01_JPRB ,&
&  0.15155E+01_JPRB, 0.15528E+01_JPRB, 0.15130E+01_JPRB, 0.14426E+01_JPRB, 0.13322E+01_JPRB, 0.12836E+01_JPRB, 0.12144E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.12070E+01_JPRB, 0.11550E+01_JPRB, 0.10918E+01_JPRB, 0.10091E+01_JPRB, 0.91653E+00_JPRB, 0.82477E+00_JPRB ,&
&  0.74158E+00_JPRB, 0.64556E+00_JPRB, 0.60718E+00_JPRB, 0.60531E+00_JPRB, 0.61514E+00_JPRB, 0.59152E+00_JPRB, 0.58169E+00_JPRB ,&
&  0.57092E+00_JPRB, 0.55693E+00_JPRB, 0.57279E+00_JPRB, 0.55004E+00_JPRB, 0.51981E+00_JPRB, 0.48514E+00_JPRB, 0.46161E+00_JPRB ,&
&  0.43587E+00_JPRB, 0.43991E+00_JPRB, 0.43034E+00_JPRB, 0.41547E+00_JPRB, 0.40459E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.40336E+00_JPRB, 0.40312E+00_JPRB, 0.39913E+00_JPRB, 0.39391E+00_JPRB, 0.39572E+00_JPRB, 0.39501E+00_JPRB, 0.39409E+00_JPRB ,&
&  0.43248E+00_JPRB, 0.40883E+00_JPRB, 0.42018E+00_JPRB, 0.36995E+00_JPRB, 0.35018E+00_JPRB, 0.34461E+00_JPRB, 0.33574E+00_JPRB ,&
&  0.33912E+00_JPRB, 0.35115E+00_JPRB, 0.36282E+00_JPRB, 0.37188E+00_JPRB, 0.38713E+00_JPRB, 0.39554E+00_JPRB, 0.40155E+00_JPRB ,&
&  0.41012E+00_JPRB, 0.42320E+00_JPRB, 0.44538E+00_JPRB, 0.46348E+00_JPRB, 0.49038E+00_JPRB, 0.52760E+00_JPRB, 0.57778E+00_JPRB ,&
&  0.59166E+00_JPRB, 0.62695E+00_JPRB, 0.69563E+00_JPRB, 0.71840E+00_JPRB, 0.73354E+00_JPRB, 0.72263E+00_JPRB, 0.70529E+00_JPRB ,&
&  0.67908E+00_JPRB, 0.58560E+00_JPRB, 0.60198E+00_JPRB, 0.57888E+00_JPRB, 0.55731E+00_JPRB, 0.53560E+00_JPRB, 0.51575E+00_JPRB ,&
&  0.49993E+00_JPRB, 0.49093E+00_JPRB, 0.49022E+00_JPRB, 0.49233E+00_JPRB, 0.50441E+00_JPRB, 0.58620E+00_JPRB, 0.69535E+00_JPRB ,&
&  0.90442E+00_JPRB, 0.82262E+00_JPRB, 0.10454E+01_JPRB, 0.11437E+01_JPRB, 0.13927E+01_JPRB, 0.15566E+01_JPRB, 0.15875E+01_JPRB ,&
&  0.17454E+01_JPRB, 0.16934E+01_JPRB, 0.15966E+01_JPRB, 0.15274E+01_JPRB, 0.15129E+01_JPRB, 0.14722E+01_JPRB, 0.14110E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.12783E+01_JPRB, 0.13180E+01_JPRB, 0.12517E+01_JPRB, 0.11536E+01_JPRB, 0.11093E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.10150E+01_JPRB, 0.98202E+00_JPRB, 0.94084E+00_JPRB, 0.84079E+00_JPRB, 0.80802E+00_JPRB, 0.78084E+00_JPRB, 0.71984E+00_JPRB ,&
&  0.69512E+00_JPRB, 0.66298E+00_JPRB, 0.62774E+00_JPRB, 0.64089E+00_JPRB, 0.61701E+00_JPRB, 0.60394E+00_JPRB, 0.59615E+00_JPRB ,&
&  0.56684E+00_JPRB, 0.52592E+00_JPRB, 0.51899E+00_JPRB, 0.51130E+00_JPRB, 0.49388E+00_JPRB, 0.47654E+00_JPRB, 0.48433E+00_JPRB ,&
&  0.46565E+00_JPRB, 0.45428E+00_JPRB, 0.45489E+00_JPRB, 0.43391E+00_JPRB, 0.41608E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.41319E+00_JPRB, 0.40791E+00_JPRB, 0.40184E+00_JPRB, 0.39269E+00_JPRB, 0.38793E+00_JPRB, 0.38272E+00_JPRB, 0.37153E+00_JPRB ,&
&  0.38299E+00_JPRB, 0.35535E+00_JPRB, 0.33896E+00_JPRB, 0.33264E+00_JPRB, 0.33347E+00_JPRB, 0.33168E+00_JPRB, 0.32663E+00_JPRB ,&
&  0.32906E+00_JPRB, 0.33869E+00_JPRB, 0.35197E+00_JPRB, 0.36193E+00_JPRB, 0.37115E+00_JPRB, 0.37876E+00_JPRB, 0.38613E+00_JPRB ,&
&  0.39638E+00_JPRB, 0.40982E+00_JPRB, 0.42663E+00_JPRB, 0.44956E+00_JPRB, 0.47527E+00_JPRB, 0.50887E+00_JPRB, 0.54070E+00_JPRB ,&
&  0.58759E+00_JPRB, 0.64443E+00_JPRB, 0.68203E+00_JPRB, 0.72137E+00_JPRB, 0.72961E+00_JPRB, 0.70674E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.46418E+00_JPRB, 0.43600E+00_JPRB, 0.50889E+00_JPRB, 0.50346E+00_JPRB, 0.49314E+00_JPRB, 0.47913E+00_JPRB, 0.47096E+00_JPRB ,&
&  0.46722E+00_JPRB, 0.46276E+00_JPRB, 0.46180E+00_JPRB, 0.46043E+00_JPRB, 0.46355E+00_JPRB, 0.48036E+00_JPRB, 0.52212E+00_JPRB ,&
&  0.63053E+00_JPRB, 0.63718E+00_JPRB, 0.67601E+00_JPRB, 0.76296E+00_JPRB, 0.10347E+01_JPRB, 0.11234E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.12496E+01_JPRB, 0.12383E+01_JPRB, 0.12443E+01_JPRB, 0.12555E+01_JPRB, 0.12605E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.11900E+01_JPRB, 0.11746E+01_JPRB, 0.11407E+01_JPRB, 0.10752E+01_JPRB, 0.10316E+01_JPRB, 0.98416E+00_JPRB ,&
&  0.93719E+00_JPRB, 0.88974E+00_JPRB, 0.82201E+00_JPRB, 0.75689E+00_JPRB, 0.70343E+00_JPRB, 0.68102E+00_JPRB, 0.66780E+00_JPRB ,&
&  0.65398E+00_JPRB, 0.64678E+00_JPRB, 0.63836E+00_JPRB, 0.62814E+00_JPRB, 0.61551E+00_JPRB, 0.59881E+00_JPRB, 0.59137E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.51779E+00_JPRB, 0.47856E+00_JPRB, 0.46266E+00_JPRB, 0.46028E+00_JPRB, 0.45990E+00_JPRB, 0.45887E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.45303E+00_JPRB, 0.43987E+00_JPRB, 0.43261E+00_JPRB, 0.42364E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.42500E+00_JPRB, 0.41629E+00_JPRB, 0.40497E+00_JPRB, 0.39006E+00_JPRB, 0.37674E+00_JPRB, 0.36929E+00_JPRB, 0.36430E+00_JPRB ,&
&  0.37038E+00_JPRB, 0.35603E+00_JPRB, 0.34174E+00_JPRB, 0.33099E+00_JPRB, 0.32550E+00_JPRB, 0.32355E+00_JPRB, 0.32688E+00_JPRB ,&
&  0.33351E+00_JPRB, 0.34044E+00_JPRB, 0.34692E+00_JPRB, 0.35324E+00_JPRB, 0.35981E+00_JPRB, 0.36568E+00_JPRB, 0.37283E+00_JPRB ,&
&  0.38300E+00_JPRB, 0.39418E+00_JPRB, 0.40565E+00_JPRB, 0.42199E+00_JPRB, 0.44293E+00_JPRB, 0.47464E+00_JPRB, 0.49829E+00_JPRB ,&
&  0.52656E+00_JPRB, 0.55961E+00_JPRB, 0.60808E+00_JPRB, 0.68017E+00_JPRB, 0.71263E+00_JPRB, 0.68016E+00_JPRB, 0.51465E+00_JPRB ,&
&  0.39445E+00_JPRB, 0.36013E+00_JPRB, 0.37425E+00_JPRB, 0.43178E+00_JPRB, 0.43461E+00_JPRB, 0.43533E+00_JPRB, 0.43831E+00_JPRB ,&
&  0.43033E+00_JPRB, 0.41740E+00_JPRB, 0.42099E+00_JPRB, 0.43516E+00_JPRB, 0.43590E+00_JPRB, 0.43956E+00_JPRB, 0.46090E+00_JPRB ,&
&  0.50435E+00_JPRB, 0.55841E+00_JPRB, 0.61623E+00_JPRB, 0.60225E+00_JPRB, 0.68176E+00_JPRB, 0.95220E+00_JPRB, 0.99322E+00_JPRB ,&
&  0.98483E+00_JPRB, 0.98764E+00_JPRB, 0.10179E+01_JPRB, 0.10465E+01_JPRB, 0.10254E+01_JPRB, 0.10154E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.97935E+00_JPRB, 0.95959E+00_JPRB, 0.94600E+00_JPRB, 0.95463E+00_JPRB, 0.93549E+00_JPRB, 0.91289E+00_JPRB, 0.89470E+00_JPRB ,&
&  0.86337E+00_JPRB, 0.80452E+00_JPRB, 0.74671E+00_JPRB, 0.68595E+00_JPRB, 0.65491E+00_JPRB, 0.63487E+00_JPRB, 0.62279E+00_JPRB ,&
&  0.60838E+00_JPRB, 0.60318E+00_JPRB, 0.60351E+00_JPRB, 0.60377E+00_JPRB, 0.57493E+00_JPRB, 0.53420E+00_JPRB, 0.50473E+00_JPRB ,&
&  0.48728E+00_JPRB, 0.47567E+00_JPRB, 0.46046E+00_JPRB, 0.45376E+00_JPRB, 0.45447E+00_JPRB, 0.45400E+00_JPRB, 0.44541E+00_JPRB ,&
&  0.44001E+00_JPRB, 0.43239E+00_JPRB, 0.42805E+00_JPRB, 0.42810E+00_JPRB, 0.42947E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.42128E+00_JPRB, 0.42078E+00_JPRB, 0.41129E+00_JPRB, 0.39462E+00_JPRB, 0.37299E+00_JPRB, 0.35557E+00_JPRB, 0.34417E+00_JPRB ,&
&  0.34021E+00_JPRB, 0.33627E+00_JPRB, 0.33422E+00_JPRB, 0.33394E+00_JPRB, 0.33486E+00_JPRB, 0.33754E+00_JPRB, 0.34009E+00_JPRB ,&
&  0.34204E+00_JPRB, 0.34476E+00_JPRB, 0.34848E+00_JPRB, 0.35241E+00_JPRB, 0.35674E+00_JPRB, 0.36292E+00_JPRB, 0.37040E+00_JPRB ,&
&  0.37948E+00_JPRB, 0.38622E+00_JPRB, 0.39030E+00_JPRB, 0.39561E+00_JPRB, 0.40559E+00_JPRB, 0.42524E+00_JPRB, 0.44970E+00_JPRB ,&
&  0.47445E+00_JPRB, 0.49385E+00_JPRB, 0.53855E+00_JPRB, 0.61752E+00_JPRB, 0.65751E+00_JPRB, 0.62017E+00_JPRB, 0.47978E+00_JPRB ,&
&  0.38103E+00_JPRB, 0.33106E+00_JPRB, 0.31797E+00_JPRB, 0.32793E+00_JPRB, 0.35242E+00_JPRB, 0.38300E+00_JPRB, 0.39010E+00_JPRB ,&
&  0.39571E+00_JPRB, 0.39777E+00_JPRB, 0.39961E+00_JPRB, 0.40230E+00_JPRB, 0.40361E+00_JPRB, 0.40714E+00_JPRB, 0.42099E+00_JPRB ,&
&  0.44697E+00_JPRB, 0.48040E+00_JPRB, 0.51522E+00_JPRB, 0.52737E+00_JPRB, 0.56392E+00_JPRB, 0.64658E+00_JPRB, 0.72886E+00_JPRB ,&
&  0.74649E+00_JPRB, 0.76528E+00_JPRB, 0.78307E+00_JPRB, 0.83465E+00_JPRB, 0.86239E+00_JPRB, 0.87092E+00_JPRB, 0.86459E+00_JPRB ,&
&  0.84965E+00_JPRB, 0.81794E+00_JPRB, 0.78660E+00_JPRB, 0.76421E+00_JPRB, 0.76709E+00_JPRB, 0.79272E+00_JPRB, 0.80364E+00_JPRB ,&
&  0.80151E+00_JPRB, 0.77138E+00_JPRB, 0.71639E+00_JPRB, 0.63321E+00_JPRB, 0.60289E+00_JPRB, 0.60318E+00_JPRB, 0.61211E+00_JPRB ,&
&  0.60696E+00_JPRB, 0.59797E+00_JPRB, 0.58767E+00_JPRB, 0.58398E+00_JPRB, 0.54784E+00_JPRB, 0.52495E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.49451E+00_JPRB, 0.48656E+00_JPRB, 0.48607E+00_JPRB, 0.48624E+00_JPRB, 0.48059E+00_JPRB, 0.47031E+00_JPRB, 0.45755E+00_JPRB ,&
&  0.44718E+00_JPRB, 0.43693E+00_JPRB, 0.42759E+00_JPRB, 0.42030E+00_JPRB, 0.41905E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.43601E+00_JPRB, 0.43103E+00_JPRB, 0.42328E+00_JPRB, 0.40878E+00_JPRB, 0.38919E+00_JPRB, 0.37055E+00_JPRB, 0.35889E+00_JPRB ,&
&  0.35427E+00_JPRB, 0.35308E+00_JPRB, 0.34979E+00_JPRB, 0.34804E+00_JPRB, 0.34739E+00_JPRB, 0.34789E+00_JPRB, 0.34962E+00_JPRB ,&
&  0.35234E+00_JPRB, 0.35527E+00_JPRB, 0.35752E+00_JPRB, 0.35856E+00_JPRB, 0.36158E+00_JPRB, 0.36734E+00_JPRB, 0.37479E+00_JPRB ,&
&  0.38169E+00_JPRB, 0.38367E+00_JPRB, 0.38466E+00_JPRB, 0.38822E+00_JPRB, 0.39088E+00_JPRB, 0.39817E+00_JPRB, 0.40583E+00_JPRB ,&
&  0.42156E+00_JPRB, 0.44942E+00_JPRB, 0.50431E+00_JPRB, 0.56470E+00_JPRB, 0.59207E+00_JPRB, 0.54736E+00_JPRB, 0.44350E+00_JPRB ,&
&  0.36708E+00_JPRB, 0.32127E+00_JPRB, 0.30251E+00_JPRB, 0.29885E+00_JPRB, 0.30638E+00_JPRB, 0.32339E+00_JPRB, 0.34736E+00_JPRB ,&
&  0.36576E+00_JPRB, 0.37140E+00_JPRB, 0.37890E+00_JPRB, 0.38509E+00_JPRB, 0.38925E+00_JPRB, 0.39166E+00_JPRB, 0.39637E+00_JPRB ,&
&  0.40512E+00_JPRB, 0.42024E+00_JPRB, 0.44619E+00_JPRB, 0.48500E+00_JPRB, 0.53968E+00_JPRB, 0.54592E+00_JPRB, 0.57840E+00_JPRB ,&
&  0.64255E+00_JPRB, 0.66590E+00_JPRB, 0.68525E+00_JPRB, 0.69722E+00_JPRB, 0.70253E+00_JPRB, 0.69583E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.70163E+00_JPRB, 0.69969E+00_JPRB, 0.68387E+00_JPRB, 0.65994E+00_JPRB, 0.64451E+00_JPRB, 0.64123E+00_JPRB, 0.64921E+00_JPRB ,&
&  0.66687E+00_JPRB, 0.71880E+00_JPRB, 0.87652E+00_JPRB, 0.75959E+00_JPRB, 0.69659E+00_JPRB, 0.67425E+00_JPRB, 0.64638E+00_JPRB ,&
&  0.62138E+00_JPRB, 0.60304E+00_JPRB, 0.59238E+00_JPRB, 0.58097E+00_JPRB, 0.55525E+00_JPRB, 0.53804E+00_JPRB, 0.52519E+00_JPRB ,&
&  0.51751E+00_JPRB, 0.50852E+00_JPRB, 0.50125E+00_JPRB, 0.49550E+00_JPRB, 0.48852E+00_JPRB, 0.47948E+00_JPRB, 0.47148E+00_JPRB ,&
&  0.46306E+00_JPRB, 0.45580E+00_JPRB, 0.45154E+00_JPRB, 0.44824E+00_JPRB, 0.44306E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.44000E+00_JPRB, 0.43390E+00_JPRB, 0.42374E+00_JPRB, 0.41665E+00_JPRB, 0.40560E+00_JPRB, 0.39452E+00_JPRB, 0.38234E+00_JPRB ,&
&  0.37330E+00_JPRB, 0.36672E+00_JPRB, 0.36109E+00_JPRB, 0.35816E+00_JPRB, 0.35677E+00_JPRB, 0.35613E+00_JPRB, 0.35725E+00_JPRB ,&
&  0.35877E+00_JPRB, 0.35717E+00_JPRB, 0.35860E+00_JPRB, 0.36198E+00_JPRB, 0.36631E+00_JPRB, 0.37083E+00_JPRB, 0.37602E+00_JPRB ,&
&  0.37752E+00_JPRB, 0.37623E+00_JPRB, 0.37532E+00_JPRB, 0.37569E+00_JPRB, 0.37779E+00_JPRB, 0.38371E+00_JPRB, 0.40068E+00_JPRB ,&
&  0.42549E+00_JPRB, 0.45630E+00_JPRB, 0.48864E+00_JPRB, 0.51954E+00_JPRB, 0.52388E+00_JPRB, 0.48690E+00_JPRB, 0.39859E+00_JPRB ,&
&  0.34984E+00_JPRB, 0.31960E+00_JPRB, 0.30650E+00_JPRB, 0.30117E+00_JPRB, 0.30524E+00_JPRB, 0.31652E+00_JPRB, 0.33599E+00_JPRB ,&
&  0.35018E+00_JPRB, 0.35930E+00_JPRB, 0.36317E+00_JPRB, 0.36673E+00_JPRB, 0.36885E+00_JPRB, 0.37156E+00_JPRB, 0.37401E+00_JPRB ,&
&  0.38015E+00_JPRB, 0.38492E+00_JPRB, 0.39037E+00_JPRB, 0.40056E+00_JPRB, 0.41668E+00_JPRB, 0.43274E+00_JPRB, 0.45324E+00_JPRB ,&
&  0.49408E+00_JPRB, 0.51697E+00_JPRB, 0.53234E+00_JPRB, 0.54259E+00_JPRB, 0.55780E+00_JPRB, 0.56447E+00_JPRB, 0.56282E+00_JPRB ,&
&  0.56278E+00_JPRB, 0.57824E+00_JPRB, 0.58006E+00_JPRB, 0.58025E+00_JPRB, 0.57741E+00_JPRB, 0.57738E+00_JPRB, 0.57845E+00_JPRB ,&
&  0.56700E+00_JPRB, 0.57448E+00_JPRB, 0.64508E+00_JPRB, 0.66594E+00_JPRB, 0.65173E+00_JPRB, 0.62942E+00_JPRB, 0.60221E+00_JPRB ,&
&  0.57900E+00_JPRB, 0.55957E+00_JPRB, 0.54448E+00_JPRB, 0.53153E+00_JPRB, 0.52230E+00_JPRB, 0.51699E+00_JPRB, 0.51592E+00_JPRB ,&
&  0.51028E+00_JPRB, 0.50279E+00_JPRB, 0.49307E+00_JPRB, 0.48608E+00_JPRB, 0.48119E+00_JPRB, 0.47374E+00_JPRB, 0.46583E+00_JPRB ,&
&  0.45789E+00_JPRB, 0.45409E+00_JPRB, 0.45423E+00_JPRB, 0.45314E+00_JPRB, 0.44757E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.43986E+00_JPRB, 0.43624E+00_JPRB, 0.42903E+00_JPRB, 0.42277E+00_JPRB, 0.41427E+00_JPRB, 0.40718E+00_JPRB, 0.39808E+00_JPRB ,&
&  0.39040E+00_JPRB, 0.38314E+00_JPRB, 0.37702E+00_JPRB, 0.37262E+00_JPRB, 0.36943E+00_JPRB, 0.36715E+00_JPRB, 0.36632E+00_JPRB ,&
&  0.36654E+00_JPRB, 0.36749E+00_JPRB, 0.36914E+00_JPRB, 0.37110E+00_JPRB, 0.37373E+00_JPRB, 0.37684E+00_JPRB, 0.38017E+00_JPRB ,&
&  0.38173E+00_JPRB, 0.37982E+00_JPRB, 0.37432E+00_JPRB, 0.37723E+00_JPRB, 0.38407E+00_JPRB, 0.38557E+00_JPRB, 0.40626E+00_JPRB ,&
&  0.42731E+00_JPRB, 0.45525E+00_JPRB, 0.47077E+00_JPRB, 0.47752E+00_JPRB, 0.47177E+00_JPRB, 0.38137E+00_JPRB, 0.36214E+00_JPRB ,&
&  0.34096E+00_JPRB, 0.32464E+00_JPRB, 0.31678E+00_JPRB, 0.31216E+00_JPRB, 0.31154E+00_JPRB, 0.31776E+00_JPRB, 0.32815E+00_JPRB ,&
&  0.33488E+00_JPRB, 0.35828E+00_JPRB, 0.35871E+00_JPRB, 0.35729E+00_JPRB, 0.35692E+00_JPRB, 0.35827E+00_JPRB, 0.35620E+00_JPRB ,&
&  0.35749E+00_JPRB, 0.35876E+00_JPRB, 0.36098E+00_JPRB, 0.36818E+00_JPRB, 0.37474E+00_JPRB, 0.37965E+00_JPRB, 0.38768E+00_JPRB ,&
&  0.40069E+00_JPRB, 0.41192E+00_JPRB, 0.41755E+00_JPRB, 0.42259E+00_JPRB, 0.42937E+00_JPRB, 0.43987E+00_JPRB, 0.45430E+00_JPRB ,&
&  0.46397E+00_JPRB, 0.47405E+00_JPRB, 0.48370E+00_JPRB, 0.49543E+00_JPRB, 0.50626E+00_JPRB, 0.51850E+00_JPRB, 0.53190E+00_JPRB ,&
&  0.54021E+00_JPRB, 0.55343E+00_JPRB, 0.56342E+00_JPRB, 0.57950E+00_JPRB, 0.57259E+00_JPRB, 0.55537E+00_JPRB, 0.54341E+00_JPRB ,&
&  0.53283E+00_JPRB, 0.53224E+00_JPRB, 0.52326E+00_JPRB, 0.51495E+00_JPRB, 0.50712E+00_JPRB, 0.50148E+00_JPRB, 0.49557E+00_JPRB ,&
&  0.49135E+00_JPRB, 0.48645E+00_JPRB, 0.48119E+00_JPRB, 0.47514E+00_JPRB, 0.47226E+00_JPRB, 0.46832E+00_JPRB, 0.46346E+00_JPRB ,&
&  0.45685E+00_JPRB, 0.45304E+00_JPRB, 0.45064E+00_JPRB, 0.44757E+00_JPRB, 0.44283E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.43708E+00_JPRB, 0.43170E+00_JPRB, 0.42708E+00_JPRB, 0.42168E+00_JPRB, 0.41574E+00_JPRB, 0.41019E+00_JPRB, 0.40305E+00_JPRB ,&
&  0.39705E+00_JPRB, 0.39183E+00_JPRB, 0.38760E+00_JPRB, 0.38446E+00_JPRB, 0.38220E+00_JPRB, 0.38067E+00_JPRB, 0.38044E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.38048E+00_JPRB, 0.38150E+00_JPRB, 0.38277E+00_JPRB, 0.38423E+00_JPRB, 0.38524E+00_JPRB, 0.38643E+00_JPRB ,&
&  0.38481E+00_JPRB, 0.37906E+00_JPRB, 0.37336E+00_JPRB, 0.37893E+00_JPRB, 0.37990E+00_JPRB, 0.38503E+00_JPRB, 0.39329E+00_JPRB ,&
&  0.41543E+00_JPRB, 0.40152E+00_JPRB, 0.40612E+00_JPRB, 0.38650E+00_JPRB, 0.37009E+00_JPRB, 0.34918E+00_JPRB, 0.33258E+00_JPRB ,&
&  0.32592E+00_JPRB, 0.31862E+00_JPRB, 0.31489E+00_JPRB, 0.31507E+00_JPRB, 0.31741E+00_JPRB, 0.32435E+00_JPRB, 0.33300E+00_JPRB ,&
&  0.33820E+00_JPRB, 0.35540E+00_JPRB, 0.35679E+00_JPRB, 0.35681E+00_JPRB, 0.35846E+00_JPRB, 0.35982E+00_JPRB, 0.35674E+00_JPRB ,&
&  0.35683E+00_JPRB, 0.35661E+00_JPRB, 0.35694E+00_JPRB, 0.36047E+00_JPRB, 0.36722E+00_JPRB, 0.37164E+00_JPRB, 0.37640E+00_JPRB ,&
&  0.38216E+00_JPRB, 0.38614E+00_JPRB, 0.38868E+00_JPRB, 0.39086E+00_JPRB, 0.39335E+00_JPRB, 0.39610E+00_JPRB, 0.39993E+00_JPRB ,&
&  0.40500E+00_JPRB, 0.41498E+00_JPRB, 0.42416E+00_JPRB, 0.43901E+00_JPRB, 0.45385E+00_JPRB, 0.47214E+00_JPRB, 0.48830E+00_JPRB ,&
&  0.50360E+00_JPRB, 0.51524E+00_JPRB, 0.52704E+00_JPRB, 0.53847E+00_JPRB, 0.54763E+00_JPRB, 0.54257E+00_JPRB, 0.52926E+00_JPRB ,&
&  0.52078E+00_JPRB, 0.50779E+00_JPRB, 0.49884E+00_JPRB, 0.49103E+00_JPRB, 0.48479E+00_JPRB, 0.48106E+00_JPRB, 0.47722E+00_JPRB ,&
&  0.47454E+00_JPRB, 0.47150E+00_JPRB, 0.46848E+00_JPRB, 0.46518E+00_JPRB, 0.46395E+00_JPRB, 0.46183E+00_JPRB, 0.45875E+00_JPRB ,&
&  0.45513E+00_JPRB, 0.45356E+00_JPRB, 0.45028E+00_JPRB, 0.44604E+00_JPRB, 0.44142E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.43613E+00_JPRB, 0.42971E+00_JPRB, 0.42481E+00_JPRB, 0.41932E+00_JPRB, 0.41420E+00_JPRB, 0.40866E+00_JPRB, 0.40401E+00_JPRB ,&
&  0.39955E+00_JPRB, 0.39587E+00_JPRB, 0.39280E+00_JPRB, 0.39051E+00_JPRB, 0.38948E+00_JPRB, 0.38885E+00_JPRB, 0.38877E+00_JPRB ,&
&  0.38870E+00_JPRB, 0.38883E+00_JPRB, 0.38943E+00_JPRB, 0.38994E+00_JPRB, 0.39010E+00_JPRB, 0.38975E+00_JPRB, 0.38868E+00_JPRB ,&
&  0.38482E+00_JPRB, 0.37981E+00_JPRB, 0.36797E+00_JPRB, 0.37101E+00_JPRB, 0.36981E+00_JPRB, 0.36942E+00_JPRB, 0.37244E+00_JPRB ,&
&  0.37281E+00_JPRB, 0.37611E+00_JPRB, 0.37557E+00_JPRB, 0.36996E+00_JPRB, 0.36296E+00_JPRB, 0.34895E+00_JPRB, 0.34093E+00_JPRB ,&
&  0.33340E+00_JPRB, 0.32906E+00_JPRB, 0.32817E+00_JPRB, 0.32842E+00_JPRB, 0.33336E+00_JPRB, 0.33809E+00_JPRB, 0.34363E+00_JPRB ,&
&  0.35154E+00_JPRB, 0.35821E+00_JPRB, 0.36591E+00_JPRB, 0.36448E+00_JPRB, 0.36651E+00_JPRB, 0.36781E+00_JPRB, 0.36967E+00_JPRB ,&
&  0.37145E+00_JPRB, 0.37289E+00_JPRB, 0.37457E+00_JPRB, 0.37610E+00_JPRB, 0.37814E+00_JPRB, 0.38048E+00_JPRB, 0.38277E+00_JPRB ,&
&  0.38536E+00_JPRB, 0.38776E+00_JPRB, 0.39014E+00_JPRB, 0.39241E+00_JPRB, 0.39506E+00_JPRB, 0.39651E+00_JPRB, 0.39813E+00_JPRB ,&
&  0.39999E+00_JPRB, 0.40251E+00_JPRB, 0.40586E+00_JPRB, 0.40993E+00_JPRB, 0.41416E+00_JPRB, 0.41960E+00_JPRB, 0.42581E+00_JPRB ,&
&  0.43421E+00_JPRB, 0.44227E+00_JPRB, 0.45274E+00_JPRB, 0.45931E+00_JPRB, 0.47295E+00_JPRB, 0.48623E+00_JPRB, 0.49345E+00_JPRB ,&
&  0.49205E+00_JPRB, 0.48748E+00_JPRB, 0.48323E+00_JPRB, 0.47893E+00_JPRB, 0.47577E+00_JPRB, 0.47142E+00_JPRB, 0.46841E+00_JPRB ,&
&  0.46449E+00_JPRB, 0.46130E+00_JPRB, 0.45790E+00_JPRB, 0.45572E+00_JPRB, 0.45508E+00_JPRB, 0.45418E+00_JPRB, 0.45270E+00_JPRB ,&
&  0.45121E+00_JPRB, 0.45034E+00_JPRB, 0.44912E+00_JPRB, 0.44623E+00_JPRB, 0.44204E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.43230E+00_JPRB, 0.42909E+00_JPRB, 0.42564E+00_JPRB, 0.42157E+00_JPRB, 0.41844E+00_JPRB, 0.41510E+00_JPRB, 0.41218E+00_JPRB ,&
&  0.40905E+00_JPRB, 0.40587E+00_JPRB, 0.40320E+00_JPRB, 0.40113E+00_JPRB, 0.39945E+00_JPRB, 0.39827E+00_JPRB, 0.39763E+00_JPRB ,&
&  0.39708E+00_JPRB, 0.39648E+00_JPRB, 0.39630E+00_JPRB, 0.39610E+00_JPRB, 0.39567E+00_JPRB, 0.39466E+00_JPRB, 0.39219E+00_JPRB ,&
&  0.38918E+00_JPRB, 0.38582E+00_JPRB, 0.38239E+00_JPRB, 0.37946E+00_JPRB, 0.37884E+00_JPRB, 0.36424E+00_JPRB, 0.36338E+00_JPRB ,&
&  0.36820E+00_JPRB, 0.37151E+00_JPRB, 0.37698E+00_JPRB, 0.38099E+00_JPRB, 0.38257E+00_JPRB, 0.37821E+00_JPRB, 0.37539E+00_JPRB ,&
&  0.36163E+00_JPRB, 0.36381E+00_JPRB, 0.36301E+00_JPRB, 0.36295E+00_JPRB, 0.36497E+00_JPRB, 0.36524E+00_JPRB, 0.36832E+00_JPRB ,&
&  0.37914E+00_JPRB, 0.37970E+00_JPRB, 0.38093E+00_JPRB, 0.38143E+00_JPRB, 0.38170E+00_JPRB, 0.38191E+00_JPRB, 0.38275E+00_JPRB ,&
&  0.38341E+00_JPRB, 0.38469E+00_JPRB, 0.38655E+00_JPRB, 0.38742E+00_JPRB, 0.38826E+00_JPRB, 0.38949E+00_JPRB, 0.39071E+00_JPRB ,&
&  0.39129E+00_JPRB, 0.39194E+00_JPRB, 0.39278E+00_JPRB, 0.39389E+00_JPRB, 0.39556E+00_JPRB, 0.39680E+00_JPRB, 0.39811E+00_JPRB ,&
&  0.39888E+00_JPRB, 0.39866E+00_JPRB, 0.39881E+00_JPRB, 0.39900E+00_JPRB, 0.39903E+00_JPRB, 0.39874E+00_JPRB, 0.39946E+00_JPRB ,&
&  0.40049E+00_JPRB, 0.40162E+00_JPRB, 0.40283E+00_JPRB, 0.40490E+00_JPRB, 0.40852E+00_JPRB, 0.41312E+00_JPRB, 0.42098E+00_JPRB ,&
&  0.42749E+00_JPRB, 0.43260E+00_JPRB, 0.43468E+00_JPRB, 0.43680E+00_JPRB, 0.43934E+00_JPRB, 0.44052E+00_JPRB, 0.44153E+00_JPRB ,&
&  0.44132E+00_JPRB, 0.44051E+00_JPRB, 0.43906E+00_JPRB, 0.43781E+00_JPRB, 0.43632E+00_JPRB, 0.43487E+00_JPRB, 0.43445E+00_JPRB ,&
&  0.43532E+00_JPRB, 0.43653E+00_JPRB, 0.43712E+00_JPRB, 0.43630E+00_JPRB, 0.43469E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.42543E+00_JPRB, 0.42462E+00_JPRB, 0.42345E+00_JPRB, 0.42193E+00_JPRB, 0.42017E+00_JPRB, 0.41808E+00_JPRB, 0.41604E+00_JPRB ,&
&  0.41386E+00_JPRB, 0.41195E+00_JPRB, 0.40994E+00_JPRB, 0.40799E+00_JPRB, 0.40607E+00_JPRB, 0.40481E+00_JPRB, 0.40383E+00_JPRB ,&
&  0.40308E+00_JPRB, 0.40240E+00_JPRB, 0.40172E+00_JPRB, 0.40082E+00_JPRB, 0.39972E+00_JPRB, 0.39824E+00_JPRB, 0.39649E+00_JPRB ,&
&  0.39484E+00_JPRB, 0.39285E+00_JPRB, 0.39076E+00_JPRB, 0.38809E+00_JPRB, 0.38596E+00_JPRB, 0.38413E+00_JPRB, 0.38271E+00_JPRB ,&
&  0.38157E+00_JPRB, 0.38066E+00_JPRB, 0.38021E+00_JPRB, 0.38010E+00_JPRB, 0.38147E+00_JPRB, 0.38244E+00_JPRB, 0.38308E+00_JPRB ,&
&  0.38353E+00_JPRB, 0.38324E+00_JPRB, 0.38327E+00_JPRB, 0.38343E+00_JPRB, 0.38366E+00_JPRB, 0.38444E+00_JPRB, 0.38483E+00_JPRB ,&
&  0.38528E+00_JPRB, 0.38566E+00_JPRB, 0.38642E+00_JPRB, 0.38710E+00_JPRB, 0.38777E+00_JPRB, 0.38862E+00_JPRB, 0.38886E+00_JPRB ,&
&  0.38954E+00_JPRB, 0.39099E+00_JPRB, 0.39267E+00_JPRB, 0.39445E+00_JPRB, 0.39563E+00_JPRB, 0.39660E+00_JPRB, 0.39735E+00_JPRB ,&
&  0.39792E+00_JPRB, 0.39862E+00_JPRB, 0.39926E+00_JPRB, 0.39983E+00_JPRB, 0.40024E+00_JPRB, 0.40086E+00_JPRB, 0.40129E+00_JPRB ,&
&  0.40167E+00_JPRB, 0.40183E+00_JPRB, 0.40228E+00_JPRB, 0.40258E+00_JPRB, 0.40252E+00_JPRB, 0.40225E+00_JPRB, 0.40249E+00_JPRB ,&
&  0.40249E+00_JPRB, 0.40234E+00_JPRB, 0.40217E+00_JPRB, 0.40297E+00_JPRB, 0.40357E+00_JPRB, 0.40390E+00_JPRB, 0.40379E+00_JPRB ,&
&  0.40530E+00_JPRB, 0.40757E+00_JPRB, 0.41061E+00_JPRB, 0.41488E+00_JPRB, 0.41791E+00_JPRB, 0.41974E+00_JPRB, 0.42104E+00_JPRB ,&
&  0.42172E+00_JPRB, 0.42216E+00_JPRB, 0.42248E+00_JPRB, 0.42290E+00_JPRB, 0.42278E+00_JPRB, 0.42303E+00_JPRB, 0.42367E+00_JPRB ,&
&  0.42431E+00_JPRB, 0.42473E+00_JPRB, 0.42525E+00_JPRB, 0.42572E+00_JPRB, 0.42591E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.41600E+00_JPRB, 0.41564E+00_JPRB, 0.41524E+00_JPRB, 0.41470E+00_JPRB, 0.41395E+00_JPRB, 0.41307E+00_JPRB, 0.41226E+00_JPRB ,&
&  0.41140E+00_JPRB, 0.41070E+00_JPRB, 0.40975E+00_JPRB, 0.40879E+00_JPRB, 0.40781E+00_JPRB, 0.40692E+00_JPRB, 0.40603E+00_JPRB ,&
&  0.40509E+00_JPRB, 0.40414E+00_JPRB, 0.40323E+00_JPRB, 0.40222E+00_JPRB, 0.40117E+00_JPRB, 0.40026E+00_JPRB, 0.39937E+00_JPRB ,&
&  0.39867E+00_JPRB, 0.39789E+00_JPRB, 0.39710E+00_JPRB, 0.39617E+00_JPRB, 0.39560E+00_JPRB, 0.39479E+00_JPRB, 0.39419E+00_JPRB ,&
&  0.39348E+00_JPRB, 0.39309E+00_JPRB, 0.39259E+00_JPRB, 0.39223E+00_JPRB, 0.39177E+00_JPRB, 0.39178E+00_JPRB, 0.39158E+00_JPRB ,&
&  0.39152E+00_JPRB, 0.39140E+00_JPRB, 0.39145E+00_JPRB, 0.39145E+00_JPRB, 0.39150E+00_JPRB, 0.39162E+00_JPRB, 0.39197E+00_JPRB ,&
&  0.39229E+00_JPRB, 0.39270E+00_JPRB, 0.39327E+00_JPRB, 0.39381E+00_JPRB, 0.39433E+00_JPRB, 0.39488E+00_JPRB, 0.39541E+00_JPRB ,&
&  0.39646E+00_JPRB, 0.39772E+00_JPRB, 0.39897E+00_JPRB, 0.40020E+00_JPRB, 0.40130E+00_JPRB, 0.40217E+00_JPRB, 0.40293E+00_JPRB ,&
&  0.40353E+00_JPRB, 0.40412E+00_JPRB, 0.40462E+00_JPRB, 0.40517E+00_JPRB, 0.40547E+00_JPRB, 0.40602E+00_JPRB, 0.40636E+00_JPRB ,&
&  0.40664E+00_JPRB, 0.40659E+00_JPRB, 0.40682E+00_JPRB, 0.40695E+00_JPRB, 0.40715E+00_JPRB, 0.40730E+00_JPRB, 0.40773E+00_JPRB ,&
&  0.40814E+00_JPRB, 0.40872E+00_JPRB, 0.40930E+00_JPRB, 0.40989E+00_JPRB, 0.41027E+00_JPRB, 0.41065E+00_JPRB, 0.41091E+00_JPRB ,&
&  0.41118E+00_JPRB, 0.41153E+00_JPRB, 0.41200E+00_JPRB, 0.41234E+00_JPRB, 0.41258E+00_JPRB, 0.41296E+00_JPRB, 0.41338E+00_JPRB ,&
&  0.41400E+00_JPRB, 0.41439E+00_JPRB, 0.41481E+00_JPRB, 0.41521E+00_JPRB, 0.41549E+00_JPRB, 0.41567E+00_JPRB, 0.41574E+00_JPRB ,&
&  0.41580E+00_JPRB, 0.41583E+00_JPRB, 0.41589E+00_JPRB, 0.41603E+00_JPRB, 0.41616E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB ,&
&  0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB, 0.40451E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1950:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1950
