!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1960 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1960* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB, 0.13912E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.17184E+00_JPRB, 0.17424E+00_JPRB, 0.17582E+00_JPRB, 0.17682E+00_JPRB, 0.17659E+00_JPRB, 0.17750E+00_JPRB, 0.17846E+00_JPRB ,&
&  0.17974E+00_JPRB, 0.18130E+00_JPRB, 0.18234E+00_JPRB, 0.18332E+00_JPRB, 0.18380E+00_JPRB, 0.18378E+00_JPRB, 0.18376E+00_JPRB ,&
&  0.18356E+00_JPRB, 0.18331E+00_JPRB, 0.18049E+00_JPRB, 0.17795E+00_JPRB, 0.17617E+00_JPRB, 0.17523E+00_JPRB, 0.17434E+00_JPRB ,&
&  0.17358E+00_JPRB, 0.17246E+00_JPRB, 0.17105E+00_JPRB, 0.16888E+00_JPRB, 0.16570E+00_JPRB, 0.16261E+00_JPRB, 0.15947E+00_JPRB ,&
&  0.15664E+00_JPRB, 0.15429E+00_JPRB, 0.15212E+00_JPRB, 0.14990E+00_JPRB, 0.14850E+00_JPRB, 0.14693E+00_JPRB, 0.14534E+00_JPRB ,&
&  0.14339E+00_JPRB, 0.14132E+00_JPRB, 0.13932E+00_JPRB, 0.13728E+00_JPRB, 0.13532E+00_JPRB, 0.13338E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13046E+00_JPRB, 0.12931E+00_JPRB, 0.12805E+00_JPRB, 0.12689E+00_JPRB, 0.12569E+00_JPRB, 0.12455E+00_JPRB, 0.12331E+00_JPRB ,&
&  0.12246E+00_JPRB, 0.12157E+00_JPRB, 0.12068E+00_JPRB, 0.11984E+00_JPRB, 0.11902E+00_JPRB, 0.11822E+00_JPRB, 0.11731E+00_JPRB ,&
&  0.11648E+00_JPRB, 0.11595E+00_JPRB, 0.11539E+00_JPRB, 0.11477E+00_JPRB, 0.11418E+00_JPRB, 0.11378E+00_JPRB, 0.11343E+00_JPRB ,&
&  0.11310E+00_JPRB, 0.11307E+00_JPRB, 0.11322E+00_JPRB, 0.11337E+00_JPRB, 0.11353E+00_JPRB, 0.11390E+00_JPRB, 0.11444E+00_JPRB ,&
&  0.11509E+00_JPRB, 0.11579E+00_JPRB, 0.11670E+00_JPRB, 0.11798E+00_JPRB, 0.11941E+00_JPRB, 0.12104E+00_JPRB, 0.12283E+00_JPRB ,&
&  0.12475E+00_JPRB, 0.12682E+00_JPRB, 0.12904E+00_JPRB, 0.13126E+00_JPRB, 0.13362E+00_JPRB, 0.13599E+00_JPRB, 0.13828E+00_JPRB ,&
&  0.14065E+00_JPRB, 0.14297E+00_JPRB, 0.14531E+00_JPRB, 0.14743E+00_JPRB, 0.15056E+00_JPRB, 0.15260E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15688E+00_JPRB, 0.15928E+00_JPRB, 0.16171E+00_JPRB, 0.16472E+00_JPRB, 0.16771E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.19851E+00_JPRB, 0.19882E+00_JPRB, 0.19993E+00_JPRB, 0.20682E+00_JPRB, 0.22116E+00_JPRB, 0.29820E+00_JPRB, 0.28993E+00_JPRB ,&
&  0.28313E+00_JPRB, 0.25459E+00_JPRB, 0.25063E+00_JPRB, 0.24327E+00_JPRB, 0.23356E+00_JPRB, 0.22535E+00_JPRB, 0.22033E+00_JPRB ,&
&  0.21499E+00_JPRB, 0.20920E+00_JPRB, 0.20563E+00_JPRB, 0.20219E+00_JPRB, 0.19894E+00_JPRB, 0.19536E+00_JPRB, 0.19266E+00_JPRB ,&
&  0.19035E+00_JPRB, 0.18876E+00_JPRB, 0.18703E+00_JPRB, 0.18376E+00_JPRB, 0.18227E+00_JPRB, 0.17924E+00_JPRB, 0.17468E+00_JPRB ,&
&  0.16876E+00_JPRB, 0.16542E+00_JPRB, 0.16172E+00_JPRB, 0.15837E+00_JPRB, 0.16065E+00_JPRB, 0.15944E+00_JPRB, 0.15716E+00_JPRB ,&
&  0.15371E+00_JPRB, 0.15015E+00_JPRB, 0.14558E+00_JPRB, 0.14124E+00_JPRB, 0.13754E+00_JPRB, 0.13451E+00_JPRB, 0.13184E+00_JPRB ,&
&  0.12969E+00_JPRB, 0.12787E+00_JPRB, 0.12554E+00_JPRB, 0.12342E+00_JPRB, 0.12119E+00_JPRB, 0.11921E+00_JPRB, 0.11726E+00_JPRB ,&
&  0.11572E+00_JPRB, 0.11426E+00_JPRB, 0.11274E+00_JPRB, 0.11116E+00_JPRB, 0.10991E+00_JPRB, 0.10828E+00_JPRB, 0.10706E+00_JPRB ,&
&  0.10596E+00_JPRB, 0.10463E+00_JPRB, 0.10365E+00_JPRB, 0.10245E+00_JPRB, 0.10156E+00_JPRB, 0.10083E+00_JPRB, 0.10012E+00_JPRB ,&
&  0.99161E-01_JPRB, 0.98524E-01_JPRB, 0.98261E-01_JPRB, 0.98047E-01_JPRB, 0.97716E-01_JPRB, 0.97724E-01_JPRB, 0.97996E-01_JPRB ,&
&  0.98387E-01_JPRB, 0.98851E-01_JPRB, 0.99645E-01_JPRB, 0.10060E+00_JPRB, 0.10202E+00_JPRB, 0.10389E+00_JPRB, 0.10686E+00_JPRB ,&
&  0.11009E+00_JPRB, 0.11370E+00_JPRB, 0.11748E+00_JPRB, 0.12192E+00_JPRB, 0.12634E+00_JPRB, 0.13090E+00_JPRB, 0.13612E+00_JPRB ,&
&  0.14177E+00_JPRB, 0.14608E+00_JPRB, 0.14997E+00_JPRB, 0.15410E+00_JPRB, 0.16228E+00_JPRB, 0.16853E+00_JPRB, 0.17348E+00_JPRB ,&
&  0.17896E+00_JPRB, 0.18629E+00_JPRB, 0.19118E+00_JPRB, 0.19463E+00_JPRB, 0.19690E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.45575E+00_JPRB, 0.46717E+00_JPRB, 0.48224E+00_JPRB, 0.49135E+00_JPRB, 0.50103E+00_JPRB, 0.46830E+00_JPRB, 0.44253E+00_JPRB ,&
&  0.39005E+00_JPRB, 0.36013E+00_JPRB, 0.33527E+00_JPRB, 0.31242E+00_JPRB, 0.29214E+00_JPRB, 0.27459E+00_JPRB, 0.25741E+00_JPRB ,&
&  0.24637E+00_JPRB, 0.23961E+00_JPRB, 0.23038E+00_JPRB, 0.22026E+00_JPRB, 0.20952E+00_JPRB, 0.20014E+00_JPRB, 0.19497E+00_JPRB ,&
&  0.19051E+00_JPRB, 0.18728E+00_JPRB, 0.18597E+00_JPRB, 0.18639E+00_JPRB, 0.18839E+00_JPRB, 0.18887E+00_JPRB, 0.18976E+00_JPRB ,&
&  0.18727E+00_JPRB, 0.19432E+00_JPRB, 0.20351E+00_JPRB, 0.20769E+00_JPRB, 0.21521E+00_JPRB, 0.21366E+00_JPRB, 0.20886E+00_JPRB ,&
&  0.20212E+00_JPRB, 0.19388E+00_JPRB, 0.18479E+00_JPRB, 0.17357E+00_JPRB, 0.16307E+00_JPRB, 0.15514E+00_JPRB, 0.14744E+00_JPRB ,&
&  0.14269E+00_JPRB, 0.13729E+00_JPRB, 0.13126E+00_JPRB, 0.12613E+00_JPRB, 0.12169E+00_JPRB, 0.11793E+00_JPRB, 0.11458E+00_JPRB ,&
&  0.11199E+00_JPRB, 0.11037E+00_JPRB, 0.10881E+00_JPRB, 0.10817E+00_JPRB, 0.10700E+00_JPRB, 0.10590E+00_JPRB, 0.10483E+00_JPRB ,&
&  0.10421E+00_JPRB, 0.10306E+00_JPRB, 0.10219E+00_JPRB, 0.10141E+00_JPRB, 0.10025E+00_JPRB, 0.99235E-01_JPRB, 0.98069E-01_JPRB ,&
&  0.97034E-01_JPRB, 0.96447E-01_JPRB, 0.96110E-01_JPRB, 0.95986E-01_JPRB, 0.96173E-01_JPRB, 0.96102E-01_JPRB, 0.96285E-01_JPRB ,&
&  0.96535E-01_JPRB, 0.96914E-01_JPRB, 0.96933E-01_JPRB, 0.97204E-01_JPRB, 0.97987E-01_JPRB, 0.99066E-01_JPRB, 0.10163E+00_JPRB ,&
&  0.10461E+00_JPRB, 0.10846E+00_JPRB, 0.11271E+00_JPRB, 0.11772E+00_JPRB, 0.12279E+00_JPRB, 0.12795E+00_JPRB, 0.13361E+00_JPRB ,&
&  0.14096E+00_JPRB, 0.14756E+00_JPRB, 0.15508E+00_JPRB, 0.16641E+00_JPRB, 0.18978E+00_JPRB, 0.20337E+00_JPRB, 0.21890E+00_JPRB ,&
&  0.24637E+00_JPRB, 0.28503E+00_JPRB, 0.38818E+00_JPRB, 0.42124E+00_JPRB, 0.43845E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.63354E+00_JPRB, 0.66460E+00_JPRB, 0.68918E+00_JPRB, 0.69150E+00_JPRB, 0.69052E+00_JPRB, 0.65334E+00_JPRB, 0.59567E+00_JPRB ,&
&  0.48865E+00_JPRB, 0.41897E+00_JPRB, 0.37529E+00_JPRB, 0.33412E+00_JPRB, 0.29668E+00_JPRB, 0.27167E+00_JPRB, 0.25448E+00_JPRB ,&
&  0.24141E+00_JPRB, 0.23425E+00_JPRB, 0.22879E+00_JPRB, 0.22419E+00_JPRB, 0.21927E+00_JPRB, 0.21452E+00_JPRB, 0.20580E+00_JPRB ,&
&  0.20060E+00_JPRB, 0.19702E+00_JPRB, 0.19572E+00_JPRB, 0.19818E+00_JPRB, 0.20506E+00_JPRB, 0.21240E+00_JPRB, 0.22325E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.25088E+00_JPRB, 0.25744E+00_JPRB, 0.26054E+00_JPRB, 0.26337E+00_JPRB, 0.26133E+00_JPRB, 0.23837E+00_JPRB ,&
&  0.24345E+00_JPRB, 0.26688E+00_JPRB, 0.28525E+00_JPRB, 0.29624E+00_JPRB, 0.25266E+00_JPRB, 0.21142E+00_JPRB, 0.17092E+00_JPRB ,&
&  0.15123E+00_JPRB, 0.13622E+00_JPRB, 0.12852E+00_JPRB, 0.12276E+00_JPRB, 0.11873E+00_JPRB, 0.11620E+00_JPRB, 0.11395E+00_JPRB ,&
&  0.11185E+00_JPRB, 0.11087E+00_JPRB, 0.10943E+00_JPRB, 0.10870E+00_JPRB, 0.10770E+00_JPRB, 0.10747E+00_JPRB, 0.10628E+00_JPRB ,&
&  0.10506E+00_JPRB, 0.10411E+00_JPRB, 0.10350E+00_JPRB, 0.10320E+00_JPRB, 0.10248E+00_JPRB, 0.10175E+00_JPRB, 0.10013E+00_JPRB ,&
&  0.98958E-01_JPRB, 0.97785E-01_JPRB, 0.96875E-01_JPRB, 0.96829E-01_JPRB, 0.97188E-01_JPRB, 0.98659E-01_JPRB, 0.99484E-01_JPRB ,&
&  0.99665E-01_JPRB, 0.99721E-01_JPRB, 0.99788E-01_JPRB, 0.10045E+00_JPRB, 0.10145E+00_JPRB, 0.10320E+00_JPRB, 0.10562E+00_JPRB ,&
&  0.10854E+00_JPRB, 0.11295E+00_JPRB, 0.11748E+00_JPRB, 0.12259E+00_JPRB, 0.12749E+00_JPRB, 0.13246E+00_JPRB, 0.13884E+00_JPRB ,&
&  0.14541E+00_JPRB, 0.15296E+00_JPRB, 0.16395E+00_JPRB, 0.17589E+00_JPRB, 0.19713E+00_JPRB, 0.21663E+00_JPRB, 0.24884E+00_JPRB ,&
&  0.31997E+00_JPRB, 0.46766E+00_JPRB, 0.55289E+00_JPRB, 0.56326E+00_JPRB, 0.59601E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.75414E+00_JPRB, 0.77254E+00_JPRB, 0.78893E+00_JPRB, 0.78633E+00_JPRB, 0.76909E+00_JPRB, 0.71139E+00_JPRB, 0.64800E+00_JPRB ,&
&  0.56929E+00_JPRB, 0.50297E+00_JPRB, 0.43372E+00_JPRB, 0.37358E+00_JPRB, 0.32331E+00_JPRB, 0.28474E+00_JPRB, 0.26211E+00_JPRB ,&
&  0.24624E+00_JPRB, 0.23935E+00_JPRB, 0.23700E+00_JPRB, 0.24161E+00_JPRB, 0.24419E+00_JPRB, 0.24963E+00_JPRB, 0.24407E+00_JPRB ,&
&  0.23717E+00_JPRB, 0.22901E+00_JPRB, 0.22711E+00_JPRB, 0.23592E+00_JPRB, 0.24898E+00_JPRB, 0.26175E+00_JPRB, 0.27831E+00_JPRB ,&
&  0.29667E+00_JPRB, 0.30255E+00_JPRB, 0.31633E+00_JPRB, 0.32010E+00_JPRB, 0.30529E+00_JPRB, 0.30684E+00_JPRB, 0.31260E+00_JPRB ,&
&  0.32561E+00_JPRB, 0.34409E+00_JPRB, 0.35467E+00_JPRB, 0.37491E+00_JPRB, 0.32283E+00_JPRB, 0.26269E+00_JPRB, 0.21130E+00_JPRB ,&
&  0.17318E+00_JPRB, 0.15158E+00_JPRB, 0.13680E+00_JPRB, 0.12924E+00_JPRB, 0.12220E+00_JPRB, 0.11834E+00_JPRB, 0.11419E+00_JPRB ,&
&  0.11233E+00_JPRB, 0.11215E+00_JPRB, 0.11157E+00_JPRB, 0.11115E+00_JPRB, 0.11140E+00_JPRB, 0.11190E+00_JPRB, 0.11130E+00_JPRB ,&
&  0.11052E+00_JPRB, 0.11020E+00_JPRB, 0.10972E+00_JPRB, 0.10939E+00_JPRB, 0.10860E+00_JPRB, 0.10846E+00_JPRB, 0.10711E+00_JPRB ,&
&  0.10642E+00_JPRB, 0.10519E+00_JPRB, 0.10448E+00_JPRB, 0.10433E+00_JPRB, 0.10462E+00_JPRB, 0.10492E+00_JPRB, 0.10543E+00_JPRB ,&
&  0.10648E+00_JPRB, 0.10745E+00_JPRB, 0.10908E+00_JPRB, 0.11073E+00_JPRB, 0.11335E+00_JPRB, 0.11580E+00_JPRB, 0.11925E+00_JPRB ,&
&  0.12291E+00_JPRB, 0.12733E+00_JPRB, 0.13232E+00_JPRB, 0.13788E+00_JPRB, 0.14282E+00_JPRB, 0.14768E+00_JPRB, 0.15302E+00_JPRB ,&
&  0.15729E+00_JPRB, 0.16408E+00_JPRB, 0.17173E+00_JPRB, 0.18443E+00_JPRB, 0.21082E+00_JPRB, 0.23471E+00_JPRB, 0.28390E+00_JPRB ,&
&  0.36832E+00_JPRB, 0.62333E+00_JPRB, 0.85408E+00_JPRB, 0.70482E+00_JPRB, 0.73576E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.76271E+00_JPRB, 0.77527E+00_JPRB, 0.79239E+00_JPRB, 0.76888E+00_JPRB, 0.75351E+00_JPRB, 0.75686E+00_JPRB, 0.75765E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.69973E+00_JPRB, 0.65085E+00_JPRB, 0.61087E+00_JPRB, 0.54502E+00_JPRB, 0.46055E+00_JPRB, 0.42296E+00_JPRB ,&
&  0.38893E+00_JPRB, 0.38547E+00_JPRB, 0.38783E+00_JPRB, 0.40382E+00_JPRB, 0.41267E+00_JPRB, 0.40924E+00_JPRB, 0.38766E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.33337E+00_JPRB, 0.31600E+00_JPRB, 0.30205E+00_JPRB, 0.30184E+00_JPRB, 0.30013E+00_JPRB, 0.29368E+00_JPRB ,&
&  0.28504E+00_JPRB, 0.28920E+00_JPRB, 0.30643E+00_JPRB, 0.33519E+00_JPRB, 0.38531E+00_JPRB, 0.36526E+00_JPRB, 0.37112E+00_JPRB ,&
&  0.40097E+00_JPRB, 0.42242E+00_JPRB, 0.43085E+00_JPRB, 0.43382E+00_JPRB, 0.42721E+00_JPRB, 0.45250E+00_JPRB, 0.39600E+00_JPRB ,&
&  0.28875E+00_JPRB, 0.22991E+00_JPRB, 0.18805E+00_JPRB, 0.16139E+00_JPRB, 0.14396E+00_JPRB, 0.13390E+00_JPRB, 0.12301E+00_JPRB ,&
&  0.11806E+00_JPRB, 0.11448E+00_JPRB, 0.11357E+00_JPRB, 0.11484E+00_JPRB, 0.11714E+00_JPRB, 0.12007E+00_JPRB, 0.12111E+00_JPRB ,&
&  0.12276E+00_JPRB, 0.12282E+00_JPRB, 0.12199E+00_JPRB, 0.12175E+00_JPRB, 0.12078E+00_JPRB, 0.12143E+00_JPRB, 0.12148E+00_JPRB ,&
&  0.12165E+00_JPRB, 0.12177E+00_JPRB, 0.12331E+00_JPRB, 0.12374E+00_JPRB, 0.12460E+00_JPRB, 0.12166E+00_JPRB, 0.11985E+00_JPRB ,&
&  0.11988E+00_JPRB, 0.12177E+00_JPRB, 0.12493E+00_JPRB, 0.12859E+00_JPRB, 0.13362E+00_JPRB, 0.13771E+00_JPRB, 0.14240E+00_JPRB ,&
&  0.14652E+00_JPRB, 0.15053E+00_JPRB, 0.15444E+00_JPRB, 0.15845E+00_JPRB, 0.16269E+00_JPRB, 0.16696E+00_JPRB, 0.17146E+00_JPRB ,&
&  0.17607E+00_JPRB, 0.18409E+00_JPRB, 0.19400E+00_JPRB, 0.21131E+00_JPRB, 0.23472E+00_JPRB, 0.26197E+00_JPRB, 0.30959E+00_JPRB ,&
&  0.37096E+00_JPRB, 0.60072E+00_JPRB, 0.89924E+00_JPRB, 0.72595E+00_JPRB, 0.74392E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.81292E+00_JPRB, 0.81077E+00_JPRB, 0.81884E+00_JPRB, 0.80849E+00_JPRB, 0.78937E+00_JPRB, 0.78024E+00_JPRB, 0.79720E+00_JPRB ,&
&  0.81696E+00_JPRB, 0.78761E+00_JPRB, 0.76609E+00_JPRB, 0.74692E+00_JPRB, 0.73342E+00_JPRB, 0.71972E+00_JPRB, 0.69747E+00_JPRB ,&
&  0.67740E+00_JPRB, 0.67011E+00_JPRB, 0.66831E+00_JPRB, 0.66331E+00_JPRB, 0.65304E+00_JPRB, 0.64295E+00_JPRB, 0.62518E+00_JPRB ,&
&  0.59268E+00_JPRB, 0.55868E+00_JPRB, 0.54074E+00_JPRB, 0.51724E+00_JPRB, 0.50048E+00_JPRB, 0.48117E+00_JPRB, 0.45367E+00_JPRB ,&
&  0.40907E+00_JPRB, 0.35109E+00_JPRB, 0.31425E+00_JPRB, 0.35042E+00_JPRB, 0.44646E+00_JPRB, 0.44067E+00_JPRB, 0.45947E+00_JPRB ,&
&  0.49401E+00_JPRB, 0.51521E+00_JPRB, 0.52549E+00_JPRB, 0.52866E+00_JPRB, 0.53760E+00_JPRB, 0.54285E+00_JPRB, 0.53931E+00_JPRB ,&
&  0.52490E+00_JPRB, 0.47271E+00_JPRB, 0.43054E+00_JPRB, 0.32820E+00_JPRB, 0.25650E+00_JPRB, 0.21876E+00_JPRB, 0.19001E+00_JPRB ,&
&  0.17155E+00_JPRB, 0.16049E+00_JPRB, 0.15800E+00_JPRB, 0.15815E+00_JPRB, 0.16333E+00_JPRB, 0.16846E+00_JPRB, 0.16847E+00_JPRB ,&
&  0.16495E+00_JPRB, 0.15749E+00_JPRB, 0.14653E+00_JPRB, 0.13980E+00_JPRB, 0.13641E+00_JPRB, 0.13661E+00_JPRB, 0.13861E+00_JPRB ,&
&  0.14047E+00_JPRB, 0.14492E+00_JPRB, 0.15734E+00_JPRB, 0.16837E+00_JPRB, 0.18188E+00_JPRB, 0.16343E+00_JPRB, 0.14858E+00_JPRB ,&
&  0.14350E+00_JPRB, 0.14749E+00_JPRB, 0.15377E+00_JPRB, 0.15855E+00_JPRB, 0.16364E+00_JPRB, 0.16810E+00_JPRB, 0.17264E+00_JPRB ,&
&  0.17693E+00_JPRB, 0.17996E+00_JPRB, 0.18159E+00_JPRB, 0.18333E+00_JPRB, 0.18493E+00_JPRB, 0.18729E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.19533E+00_JPRB, 0.20451E+00_JPRB, 0.21649E+00_JPRB, 0.23620E+00_JPRB, 0.25475E+00_JPRB, 0.27330E+00_JPRB, 0.29449E+00_JPRB ,&
&  0.31572E+00_JPRB, 0.39560E+00_JPRB, 0.58958E+00_JPRB, 0.91146E+00_JPRB, 0.90481E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.80683E+00_JPRB, 0.82681E+00_JPRB, 0.83783E+00_JPRB, 0.85184E+00_JPRB, 0.85203E+00_JPRB, 0.82172E+00_JPRB, 0.80696E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.80679E+00_JPRB, 0.79611E+00_JPRB, 0.78961E+00_JPRB, 0.78070E+00_JPRB, 0.77081E+00_JPRB, 0.76230E+00_JPRB ,&
&  0.75399E+00_JPRB, 0.74906E+00_JPRB, 0.75616E+00_JPRB, 0.77025E+00_JPRB, 0.78189E+00_JPRB, 0.78833E+00_JPRB, 0.79335E+00_JPRB ,&
&  0.79279E+00_JPRB, 0.75970E+00_JPRB, 0.71471E+00_JPRB, 0.68195E+00_JPRB, 0.66542E+00_JPRB, 0.65169E+00_JPRB, 0.61894E+00_JPRB ,&
&  0.55225E+00_JPRB, 0.43057E+00_JPRB, 0.37344E+00_JPRB, 0.41432E+00_JPRB, 0.53388E+00_JPRB, 0.49811E+00_JPRB, 0.51833E+00_JPRB ,&
&  0.54857E+00_JPRB, 0.56421E+00_JPRB, 0.55993E+00_JPRB, 0.54938E+00_JPRB, 0.54581E+00_JPRB, 0.54931E+00_JPRB, 0.54583E+00_JPRB ,&
&  0.54219E+00_JPRB, 0.52601E+00_JPRB, 0.51640E+00_JPRB, 0.50581E+00_JPRB, 0.48844E+00_JPRB, 0.46272E+00_JPRB, 0.44730E+00_JPRB ,&
&  0.40953E+00_JPRB, 0.37232E+00_JPRB, 0.36174E+00_JPRB, 0.36291E+00_JPRB, 0.33231E+00_JPRB, 0.32001E+00_JPRB, 0.30566E+00_JPRB ,&
&  0.26546E+00_JPRB, 0.24408E+00_JPRB, 0.21169E+00_JPRB, 0.19122E+00_JPRB, 0.17619E+00_JPRB, 0.16260E+00_JPRB, 0.16078E+00_JPRB ,&
&  0.16533E+00_JPRB, 0.17741E+00_JPRB, 0.19908E+00_JPRB, 0.23405E+00_JPRB, 0.26620E+00_JPRB, 0.22126E+00_JPRB, 0.20266E+00_JPRB ,&
&  0.18883E+00_JPRB, 0.18574E+00_JPRB, 0.19079E+00_JPRB, 0.19684E+00_JPRB, 0.20319E+00_JPRB, 0.21076E+00_JPRB, 0.21792E+00_JPRB ,&
&  0.22311E+00_JPRB, 0.22620E+00_JPRB, 0.22725E+00_JPRB, 0.22665E+00_JPRB, 0.22589E+00_JPRB, 0.22691E+00_JPRB, 0.23047E+00_JPRB ,&
&  0.23645E+00_JPRB, 0.24854E+00_JPRB, 0.27230E+00_JPRB, 0.30743E+00_JPRB, 0.35359E+00_JPRB, 0.41382E+00_JPRB, 0.50633E+00_JPRB ,&
&  0.63144E+00_JPRB, 0.79609E+00_JPRB, 0.83988E+00_JPRB, 0.84883E+00_JPRB, 0.81834E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.84590E+00_JPRB, 0.85656E+00_JPRB, 0.85440E+00_JPRB, 0.85336E+00_JPRB, 0.85517E+00_JPRB, 0.84813E+00_JPRB, 0.83384E+00_JPRB ,&
&  0.81959E+00_JPRB, 0.79307E+00_JPRB, 0.76279E+00_JPRB, 0.74927E+00_JPRB, 0.75692E+00_JPRB, 0.75686E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.73626E+00_JPRB, 0.75010E+00_JPRB, 0.77312E+00_JPRB, 0.79883E+00_JPRB, 0.83354E+00_JPRB, 0.86958E+00_JPRB, 0.89654E+00_JPRB ,&
&  0.90709E+00_JPRB, 0.90503E+00_JPRB, 0.89742E+00_JPRB, 0.87087E+00_JPRB, 0.83539E+00_JPRB, 0.80820E+00_JPRB, 0.78760E+00_JPRB ,&
&  0.76212E+00_JPRB, 0.71489E+00_JPRB, 0.64507E+00_JPRB, 0.63963E+00_JPRB, 0.57525E+00_JPRB, 0.55432E+00_JPRB, 0.57610E+00_JPRB ,&
&  0.59129E+00_JPRB, 0.57321E+00_JPRB, 0.54940E+00_JPRB, 0.52966E+00_JPRB, 0.52585E+00_JPRB, 0.52449E+00_JPRB, 0.51943E+00_JPRB ,&
&  0.51551E+00_JPRB, 0.50869E+00_JPRB, 0.49454E+00_JPRB, 0.48147E+00_JPRB, 0.46910E+00_JPRB, 0.45675E+00_JPRB, 0.44838E+00_JPRB ,&
&  0.42053E+00_JPRB, 0.37797E+00_JPRB, 0.34642E+00_JPRB, 0.33286E+00_JPRB, 0.32543E+00_JPRB, 0.32912E+00_JPRB, 0.34166E+00_JPRB ,&
&  0.35163E+00_JPRB, 0.35193E+00_JPRB, 0.34620E+00_JPRB, 0.33396E+00_JPRB, 0.31576E+00_JPRB, 0.27002E+00_JPRB, 0.24410E+00_JPRB ,&
&  0.25961E+00_JPRB, 0.29389E+00_JPRB, 0.32938E+00_JPRB, 0.33769E+00_JPRB, 0.34719E+00_JPRB, 0.34972E+00_JPRB, 0.33535E+00_JPRB ,&
&  0.30745E+00_JPRB, 0.27591E+00_JPRB, 0.26937E+00_JPRB, 0.27134E+00_JPRB, 0.27545E+00_JPRB, 0.28340E+00_JPRB, 0.29916E+00_JPRB ,&
&  0.32301E+00_JPRB, 0.36189E+00_JPRB, 0.38387E+00_JPRB, 0.38647E+00_JPRB, 0.37916E+00_JPRB, 0.36759E+00_JPRB, 0.36574E+00_JPRB ,&
&  0.37952E+00_JPRB, 0.44053E+00_JPRB, 0.56267E+00_JPRB, 0.71061E+00_JPRB, 0.70915E+00_JPRB, 0.75295E+00_JPRB, 0.78777E+00_JPRB ,&
&  0.81751E+00_JPRB, 0.83082E+00_JPRB, 0.81745E+00_JPRB, 0.81895E+00_JPRB, 0.82578E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.85928E+00_JPRB, 0.86383E+00_JPRB, 0.85429E+00_JPRB, 0.85010E+00_JPRB, 0.83957E+00_JPRB, 0.82612E+00_JPRB, 0.80378E+00_JPRB ,&
&  0.77736E+00_JPRB, 0.74916E+00_JPRB, 0.72263E+00_JPRB, 0.70095E+00_JPRB, 0.70504E+00_JPRB, 0.73773E+00_JPRB, 0.76523E+00_JPRB ,&
&  0.75415E+00_JPRB, 0.75008E+00_JPRB, 0.76711E+00_JPRB, 0.79775E+00_JPRB, 0.83120E+00_JPRB, 0.86212E+00_JPRB, 0.88451E+00_JPRB ,&
&  0.90270E+00_JPRB, 0.88445E+00_JPRB, 0.88797E+00_JPRB, 0.88995E+00_JPRB, 0.88398E+00_JPRB, 0.86626E+00_JPRB, 0.83775E+00_JPRB ,&
&  0.80948E+00_JPRB, 0.78254E+00_JPRB, 0.74906E+00_JPRB, 0.68979E+00_JPRB, 0.76328E+00_JPRB, 0.68583E+00_JPRB, 0.65681E+00_JPRB ,&
&  0.62929E+00_JPRB, 0.61082E+00_JPRB, 0.59021E+00_JPRB, 0.56799E+00_JPRB, 0.56025E+00_JPRB, 0.55562E+00_JPRB, 0.54773E+00_JPRB ,&
&  0.54123E+00_JPRB, 0.53428E+00_JPRB, 0.52369E+00_JPRB, 0.51003E+00_JPRB, 0.49712E+00_JPRB, 0.48326E+00_JPRB, 0.47058E+00_JPRB ,&
&  0.45236E+00_JPRB, 0.42586E+00_JPRB, 0.39806E+00_JPRB, 0.37777E+00_JPRB, 0.36729E+00_JPRB, 0.37640E+00_JPRB, 0.40852E+00_JPRB ,&
&  0.44166E+00_JPRB, 0.44721E+00_JPRB, 0.45678E+00_JPRB, 0.45933E+00_JPRB, 0.45076E+00_JPRB, 0.44097E+00_JPRB, 0.42354E+00_JPRB ,&
&  0.41000E+00_JPRB, 0.40769E+00_JPRB, 0.40861E+00_JPRB, 0.41222E+00_JPRB, 0.41752E+00_JPRB, 0.42494E+00_JPRB, 0.43528E+00_JPRB ,&
&  0.44478E+00_JPRB, 0.44792E+00_JPRB, 0.46001E+00_JPRB, 0.48664E+00_JPRB, 0.51733E+00_JPRB, 0.54846E+00_JPRB, 0.57490E+00_JPRB ,&
&  0.59264E+00_JPRB, 0.60484E+00_JPRB, 0.61295E+00_JPRB, 0.62812E+00_JPRB, 0.64379E+00_JPRB, 0.65967E+00_JPRB, 0.66797E+00_JPRB ,&
&  0.66565E+00_JPRB, 0.66910E+00_JPRB, 0.68588E+00_JPRB, 0.71392E+00_JPRB, 0.73361E+00_JPRB, 0.76742E+00_JPRB, 0.78858E+00_JPRB ,&
&  0.81110E+00_JPRB, 0.81701E+00_JPRB, 0.81171E+00_JPRB, 0.82437E+00_JPRB, 0.84138E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.81896E+00_JPRB, 0.83680E+00_JPRB, 0.84891E+00_JPRB, 0.84997E+00_JPRB, 0.83469E+00_JPRB, 0.80811E+00_JPRB, 0.78530E+00_JPRB ,&
&  0.75920E+00_JPRB, 0.73384E+00_JPRB, 0.70974E+00_JPRB, 0.67705E+00_JPRB, 0.65944E+00_JPRB, 0.66276E+00_JPRB, 0.71164E+00_JPRB ,&
&  0.77680E+00_JPRB, 0.78157E+00_JPRB, 0.77540E+00_JPRB, 0.80065E+00_JPRB, 0.85330E+00_JPRB, 0.88526E+00_JPRB, 0.90644E+00_JPRB ,&
&  0.90984E+00_JPRB, 0.87714E+00_JPRB, 0.84389E+00_JPRB, 0.83020E+00_JPRB, 0.85338E+00_JPRB, 0.87125E+00_JPRB, 0.87663E+00_JPRB ,&
&  0.87242E+00_JPRB, 0.85110E+00_JPRB, 0.83168E+00_JPRB, 0.81528E+00_JPRB, 0.81932E+00_JPRB, 0.79447E+00_JPRB, 0.74210E+00_JPRB ,&
&  0.67571E+00_JPRB, 0.64897E+00_JPRB, 0.63725E+00_JPRB, 0.62251E+00_JPRB, 0.61138E+00_JPRB, 0.61140E+00_JPRB, 0.60208E+00_JPRB ,&
&  0.58859E+00_JPRB, 0.57304E+00_JPRB, 0.55810E+00_JPRB, 0.54035E+00_JPRB, 0.52641E+00_JPRB, 0.50684E+00_JPRB, 0.48859E+00_JPRB ,&
&  0.47170E+00_JPRB, 0.45114E+00_JPRB, 0.43232E+00_JPRB, 0.41935E+00_JPRB, 0.41382E+00_JPRB, 0.42390E+00_JPRB, 0.46333E+00_JPRB ,&
&  0.49443E+00_JPRB, 0.49135E+00_JPRB, 0.48881E+00_JPRB, 0.49249E+00_JPRB, 0.48509E+00_JPRB, 0.47439E+00_JPRB, 0.46860E+00_JPRB ,&
&  0.46354E+00_JPRB, 0.45950E+00_JPRB, 0.45774E+00_JPRB, 0.45965E+00_JPRB, 0.46186E+00_JPRB, 0.46007E+00_JPRB, 0.46268E+00_JPRB ,&
&  0.46756E+00_JPRB, 0.47285E+00_JPRB, 0.48989E+00_JPRB, 0.51435E+00_JPRB, 0.53479E+00_JPRB, 0.54911E+00_JPRB, 0.56236E+00_JPRB ,&
&  0.58304E+00_JPRB, 0.59574E+00_JPRB, 0.58297E+00_JPRB, 0.57079E+00_JPRB, 0.56437E+00_JPRB, 0.56939E+00_JPRB, 0.57989E+00_JPRB ,&
&  0.60061E+00_JPRB, 0.61364E+00_JPRB, 0.61983E+00_JPRB, 0.62144E+00_JPRB, 0.62483E+00_JPRB, 0.62582E+00_JPRB, 0.63843E+00_JPRB ,&
&  0.67218E+00_JPRB, 0.70130E+00_JPRB, 0.70883E+00_JPRB, 0.74168E+00_JPRB, 0.78377E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.75079E+00_JPRB, 0.78260E+00_JPRB, 0.80716E+00_JPRB, 0.81649E+00_JPRB, 0.81485E+00_JPRB, 0.80628E+00_JPRB, 0.78377E+00_JPRB ,&
&  0.75928E+00_JPRB, 0.74185E+00_JPRB, 0.72330E+00_JPRB, 0.69669E+00_JPRB, 0.67459E+00_JPRB, 0.66990E+00_JPRB, 0.68237E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.76960E+00_JPRB, 0.78615E+00_JPRB, 0.79980E+00_JPRB, 0.84976E+00_JPRB, 0.89183E+00_JPRB, 0.90129E+00_JPRB ,&
&  0.89692E+00_JPRB, 0.89896E+00_JPRB, 0.86259E+00_JPRB, 0.82933E+00_JPRB, 0.82924E+00_JPRB, 0.83454E+00_JPRB, 0.81643E+00_JPRB ,&
&  0.84258E+00_JPRB, 0.85956E+00_JPRB, 0.85293E+00_JPRB, 0.82697E+00_JPRB, 0.81904E+00_JPRB, 0.80924E+00_JPRB, 0.79842E+00_JPRB ,&
&  0.77660E+00_JPRB, 0.72788E+00_JPRB, 0.69765E+00_JPRB, 0.69588E+00_JPRB, 0.68849E+00_JPRB, 0.68101E+00_JPRB, 0.66740E+00_JPRB ,&
&  0.62744E+00_JPRB, 0.59155E+00_JPRB, 0.56458E+00_JPRB, 0.54592E+00_JPRB, 0.53420E+00_JPRB, 0.52520E+00_JPRB, 0.51592E+00_JPRB ,&
&  0.50801E+00_JPRB, 0.48142E+00_JPRB, 0.45423E+00_JPRB, 0.44899E+00_JPRB, 0.45534E+00_JPRB, 0.46216E+00_JPRB, 0.48867E+00_JPRB ,&
&  0.51356E+00_JPRB, 0.51185E+00_JPRB, 0.50799E+00_JPRB, 0.50998E+00_JPRB, 0.51837E+00_JPRB, 0.51814E+00_JPRB, 0.50772E+00_JPRB ,&
&  0.50242E+00_JPRB, 0.49977E+00_JPRB, 0.49031E+00_JPRB, 0.48346E+00_JPRB, 0.48286E+00_JPRB, 0.48707E+00_JPRB, 0.50184E+00_JPRB ,&
&  0.51098E+00_JPRB, 0.51246E+00_JPRB, 0.52613E+00_JPRB, 0.54339E+00_JPRB, 0.55367E+00_JPRB, 0.56168E+00_JPRB, 0.56732E+00_JPRB ,&
&  0.58064E+00_JPRB, 0.60010E+00_JPRB, 0.61507E+00_JPRB, 0.61704E+00_JPRB, 0.59402E+00_JPRB, 0.56978E+00_JPRB, 0.55074E+00_JPRB ,&
&  0.54262E+00_JPRB, 0.54721E+00_JPRB, 0.55473E+00_JPRB, 0.56188E+00_JPRB, 0.56847E+00_JPRB, 0.57280E+00_JPRB, 0.57197E+00_JPRB ,&
&  0.59148E+00_JPRB, 0.62271E+00_JPRB, 0.64808E+00_JPRB, 0.67849E+00_JPRB, 0.71023E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.71067E+00_JPRB, 0.72494E+00_JPRB, 0.73652E+00_JPRB, 0.75783E+00_JPRB, 0.77739E+00_JPRB, 0.77494E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.76272E+00_JPRB, 0.75785E+00_JPRB, 0.75035E+00_JPRB, 0.74044E+00_JPRB, 0.73376E+00_JPRB, 0.73315E+00_JPRB, 0.73396E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.75598E+00_JPRB, 0.78920E+00_JPRB, 0.81529E+00_JPRB, 0.84373E+00_JPRB, 0.88735E+00_JPRB, 0.90808E+00_JPRB ,&
&  0.89204E+00_JPRB, 0.89147E+00_JPRB, 0.88072E+00_JPRB, 0.86620E+00_JPRB, 0.86800E+00_JPRB, 0.85869E+00_JPRB, 0.83063E+00_JPRB ,&
&  0.82468E+00_JPRB, 0.81748E+00_JPRB, 0.81394E+00_JPRB, 0.84144E+00_JPRB, 0.86264E+00_JPRB, 0.86262E+00_JPRB, 0.86013E+00_JPRB ,&
&  0.83959E+00_JPRB, 0.80175E+00_JPRB, 0.76610E+00_JPRB, 0.74936E+00_JPRB, 0.74735E+00_JPRB, 0.72715E+00_JPRB, 0.69901E+00_JPRB ,&
&  0.64156E+00_JPRB, 0.59441E+00_JPRB, 0.56826E+00_JPRB, 0.55281E+00_JPRB, 0.54772E+00_JPRB, 0.54507E+00_JPRB, 0.54077E+00_JPRB ,&
&  0.54629E+00_JPRB, 0.54147E+00_JPRB, 0.51447E+00_JPRB, 0.48644E+00_JPRB, 0.48228E+00_JPRB, 0.49107E+00_JPRB, 0.51008E+00_JPRB ,&
&  0.52513E+00_JPRB, 0.52101E+00_JPRB, 0.52772E+00_JPRB, 0.53488E+00_JPRB, 0.55138E+00_JPRB, 0.56711E+00_JPRB, 0.56770E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.56149E+00_JPRB, 0.54812E+00_JPRB, 0.53967E+00_JPRB, 0.52878E+00_JPRB, 0.52491E+00_JPRB, 0.53690E+00_JPRB ,&
&  0.55456E+00_JPRB, 0.56440E+00_JPRB, 0.57544E+00_JPRB, 0.58881E+00_JPRB, 0.59920E+00_JPRB, 0.60740E+00_JPRB, 0.61420E+00_JPRB ,&
&  0.62449E+00_JPRB, 0.63909E+00_JPRB, 0.65374E+00_JPRB, 0.66649E+00_JPRB, 0.66645E+00_JPRB, 0.65247E+00_JPRB, 0.62731E+00_JPRB ,&
&  0.61036E+00_JPRB, 0.59690E+00_JPRB, 0.59339E+00_JPRB, 0.59830E+00_JPRB, 0.59329E+00_JPRB, 0.57405E+00_JPRB, 0.56462E+00_JPRB ,&
&  0.57571E+00_JPRB, 0.59957E+00_JPRB, 0.62242E+00_JPRB, 0.65368E+00_JPRB, 0.68664E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.66722E+00_JPRB, 0.67905E+00_JPRB, 0.69227E+00_JPRB, 0.70512E+00_JPRB, 0.71848E+00_JPRB, 0.71833E+00_JPRB, 0.71965E+00_JPRB ,&
&  0.73857E+00_JPRB, 0.74111E+00_JPRB, 0.73995E+00_JPRB, 0.73101E+00_JPRB, 0.72454E+00_JPRB, 0.72451E+00_JPRB, 0.71584E+00_JPRB ,&
&  0.71174E+00_JPRB, 0.71856E+00_JPRB, 0.73752E+00_JPRB, 0.76630E+00_JPRB, 0.79433E+00_JPRB, 0.83083E+00_JPRB, 0.85776E+00_JPRB ,&
&  0.86224E+00_JPRB, 0.85171E+00_JPRB, 0.85186E+00_JPRB, 0.85507E+00_JPRB, 0.85533E+00_JPRB, 0.83844E+00_JPRB, 0.83402E+00_JPRB ,&
&  0.81446E+00_JPRB, 0.82056E+00_JPRB, 0.69143E+00_JPRB, 0.76099E+00_JPRB, 0.80454E+00_JPRB, 0.83977E+00_JPRB, 0.85209E+00_JPRB ,&
&  0.84621E+00_JPRB, 0.83311E+00_JPRB, 0.80862E+00_JPRB, 0.78187E+00_JPRB, 0.77235E+00_JPRB, 0.75176E+00_JPRB, 0.72811E+00_JPRB ,&
&  0.67898E+00_JPRB, 0.61674E+00_JPRB, 0.58481E+00_JPRB, 0.57323E+00_JPRB, 0.56783E+00_JPRB, 0.56914E+00_JPRB, 0.57440E+00_JPRB ,&
&  0.57836E+00_JPRB, 0.57918E+00_JPRB, 0.56844E+00_JPRB, 0.55246E+00_JPRB, 0.53396E+00_JPRB, 0.52423E+00_JPRB, 0.52136E+00_JPRB ,&
&  0.51766E+00_JPRB, 0.50534E+00_JPRB, 0.50852E+00_JPRB, 0.53775E+00_JPRB, 0.56199E+00_JPRB, 0.57820E+00_JPRB, 0.58429E+00_JPRB ,&
&  0.58835E+00_JPRB, 0.58140E+00_JPRB, 0.57711E+00_JPRB, 0.56824E+00_JPRB, 0.54573E+00_JPRB, 0.54014E+00_JPRB, 0.56141E+00_JPRB ,&
&  0.58516E+00_JPRB, 0.59875E+00_JPRB, 0.60672E+00_JPRB, 0.61847E+00_JPRB, 0.63059E+00_JPRB, 0.63747E+00_JPRB, 0.64383E+00_JPRB ,&
&  0.64884E+00_JPRB, 0.65363E+00_JPRB, 0.66717E+00_JPRB, 0.68186E+00_JPRB, 0.68871E+00_JPRB, 0.68977E+00_JPRB, 0.68932E+00_JPRB ,&
&  0.68593E+00_JPRB, 0.67801E+00_JPRB, 0.66892E+00_JPRB, 0.66021E+00_JPRB, 0.64043E+00_JPRB, 0.61664E+00_JPRB, 0.59949E+00_JPRB ,&
&  0.59122E+00_JPRB, 0.60185E+00_JPRB, 0.60998E+00_JPRB, 0.62279E+00_JPRB, 0.64982E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.67334E+00_JPRB, 0.67028E+00_JPRB, 0.66960E+00_JPRB, 0.68059E+00_JPRB, 0.69777E+00_JPRB, 0.70128E+00_JPRB, 0.70192E+00_JPRB ,&
&  0.71329E+00_JPRB, 0.72164E+00_JPRB, 0.72074E+00_JPRB, 0.71918E+00_JPRB, 0.71345E+00_JPRB, 0.70023E+00_JPRB, 0.69034E+00_JPRB ,&
&  0.69632E+00_JPRB, 0.70276E+00_JPRB, 0.70612E+00_JPRB, 0.73112E+00_JPRB, 0.76376E+00_JPRB, 0.80203E+00_JPRB, 0.82804E+00_JPRB ,&
&  0.83744E+00_JPRB, 0.83028E+00_JPRB, 0.82659E+00_JPRB, 0.82121E+00_JPRB, 0.81582E+00_JPRB, 0.81745E+00_JPRB, 0.81198E+00_JPRB ,&
&  0.80262E+00_JPRB, 0.70867E+00_JPRB, 0.73850E+00_JPRB, 0.74762E+00_JPRB, 0.76403E+00_JPRB, 0.79162E+00_JPRB, 0.82472E+00_JPRB ,&
&  0.82733E+00_JPRB, 0.82211E+00_JPRB, 0.81037E+00_JPRB, 0.78911E+00_JPRB, 0.77117E+00_JPRB, 0.76067E+00_JPRB, 0.74308E+00_JPRB ,&
&  0.71465E+00_JPRB, 0.67670E+00_JPRB, 0.64291E+00_JPRB, 0.62005E+00_JPRB, 0.60996E+00_JPRB, 0.60756E+00_JPRB, 0.60332E+00_JPRB ,&
&  0.60172E+00_JPRB, 0.60066E+00_JPRB, 0.59046E+00_JPRB, 0.58135E+00_JPRB, 0.57230E+00_JPRB, 0.56679E+00_JPRB, 0.55449E+00_JPRB ,&
&  0.54655E+00_JPRB, 0.53597E+00_JPRB, 0.53529E+00_JPRB, 0.55460E+00_JPRB, 0.57407E+00_JPRB, 0.57678E+00_JPRB, 0.58806E+00_JPRB ,&
&  0.60320E+00_JPRB, 0.60888E+00_JPRB, 0.60961E+00_JPRB, 0.60638E+00_JPRB, 0.59889E+00_JPRB, 0.59845E+00_JPRB, 0.60606E+00_JPRB ,&
&  0.61255E+00_JPRB, 0.62207E+00_JPRB, 0.62493E+00_JPRB, 0.62912E+00_JPRB, 0.63546E+00_JPRB, 0.64029E+00_JPRB, 0.65142E+00_JPRB ,&
&  0.65722E+00_JPRB, 0.66145E+00_JPRB, 0.67313E+00_JPRB, 0.68464E+00_JPRB, 0.68916E+00_JPRB, 0.69113E+00_JPRB, 0.69953E+00_JPRB ,&
&  0.70229E+00_JPRB, 0.69983E+00_JPRB, 0.69163E+00_JPRB, 0.68023E+00_JPRB, 0.67236E+00_JPRB, 0.66019E+00_JPRB, 0.64142E+00_JPRB ,&
&  0.62552E+00_JPRB, 0.62422E+00_JPRB, 0.63444E+00_JPRB, 0.64549E+00_JPRB, 0.66340E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.67704E+00_JPRB, 0.66943E+00_JPRB, 0.66348E+00_JPRB, 0.67106E+00_JPRB, 0.68484E+00_JPRB, 0.69482E+00_JPRB, 0.69386E+00_JPRB ,&
&  0.69297E+00_JPRB, 0.69723E+00_JPRB, 0.70541E+00_JPRB, 0.70827E+00_JPRB, 0.70253E+00_JPRB, 0.68749E+00_JPRB, 0.68059E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.68284E+00_JPRB, 0.68410E+00_JPRB, 0.68964E+00_JPRB, 0.71037E+00_JPRB, 0.74190E+00_JPRB, 0.77167E+00_JPRB ,&
&  0.78652E+00_JPRB, 0.78139E+00_JPRB, 0.77532E+00_JPRB, 0.77013E+00_JPRB, 0.76865E+00_JPRB, 0.77326E+00_JPRB, 0.77397E+00_JPRB ,&
&  0.76615E+00_JPRB, 0.62210E+00_JPRB, 0.67612E+00_JPRB, 0.69567E+00_JPRB, 0.72150E+00_JPRB, 0.75904E+00_JPRB, 0.80345E+00_JPRB ,&
&  0.81222E+00_JPRB, 0.80045E+00_JPRB, 0.77880E+00_JPRB, 0.76195E+00_JPRB, 0.74465E+00_JPRB, 0.73873E+00_JPRB, 0.73504E+00_JPRB ,&
&  0.73204E+00_JPRB, 0.72779E+00_JPRB, 0.70469E+00_JPRB, 0.66869E+00_JPRB, 0.64897E+00_JPRB, 0.63711E+00_JPRB, 0.62726E+00_JPRB ,&
&  0.61838E+00_JPRB, 0.60649E+00_JPRB, 0.59475E+00_JPRB, 0.58501E+00_JPRB, 0.58698E+00_JPRB, 0.58587E+00_JPRB, 0.57403E+00_JPRB ,&
&  0.56953E+00_JPRB, 0.56797E+00_JPRB, 0.57546E+00_JPRB, 0.58490E+00_JPRB, 0.58778E+00_JPRB, 0.58126E+00_JPRB, 0.58845E+00_JPRB ,&
&  0.60683E+00_JPRB, 0.61796E+00_JPRB, 0.62786E+00_JPRB, 0.64150E+00_JPRB, 0.64786E+00_JPRB, 0.64613E+00_JPRB, 0.64595E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.64858E+00_JPRB, 0.64090E+00_JPRB, 0.63484E+00_JPRB, 0.63606E+00_JPRB, 0.63697E+00_JPRB, 0.64724E+00_JPRB ,&
&  0.65670E+00_JPRB, 0.65972E+00_JPRB, 0.65569E+00_JPRB, 0.65833E+00_JPRB, 0.66386E+00_JPRB, 0.67288E+00_JPRB, 0.67896E+00_JPRB ,&
&  0.68547E+00_JPRB, 0.68367E+00_JPRB, 0.67190E+00_JPRB, 0.67481E+00_JPRB, 0.68476E+00_JPRB, 0.68446E+00_JPRB, 0.67393E+00_JPRB ,&
&  0.66470E+00_JPRB, 0.65655E+00_JPRB, 0.66014E+00_JPRB, 0.66365E+00_JPRB, 0.67100E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.71331E+00_JPRB, 0.70927E+00_JPRB, 0.70130E+00_JPRB, 0.70033E+00_JPRB, 0.70012E+00_JPRB, 0.69645E+00_JPRB, 0.69415E+00_JPRB ,&
&  0.69141E+00_JPRB, 0.68828E+00_JPRB, 0.69712E+00_JPRB, 0.70569E+00_JPRB, 0.70810E+00_JPRB, 0.70382E+00_JPRB, 0.69999E+00_JPRB ,&
&  0.69767E+00_JPRB, 0.69168E+00_JPRB, 0.68497E+00_JPRB, 0.68299E+00_JPRB, 0.68672E+00_JPRB, 0.70161E+00_JPRB, 0.72375E+00_JPRB ,&
&  0.74035E+00_JPRB, 0.74017E+00_JPRB, 0.74396E+00_JPRB, 0.75485E+00_JPRB, 0.75694E+00_JPRB, 0.75802E+00_JPRB, 0.76456E+00_JPRB ,&
&  0.64676E+00_JPRB, 0.52391E+00_JPRB, 0.58596E+00_JPRB, 0.64297E+00_JPRB, 0.69816E+00_JPRB, 0.76164E+00_JPRB, 0.82954E+00_JPRB ,&
&  0.85008E+00_JPRB, 0.83413E+00_JPRB, 0.81544E+00_JPRB, 0.79342E+00_JPRB, 0.76828E+00_JPRB, 0.75141E+00_JPRB, 0.73814E+00_JPRB ,&
&  0.73413E+00_JPRB, 0.72191E+00_JPRB, 0.70362E+00_JPRB, 0.68855E+00_JPRB, 0.67601E+00_JPRB, 0.66059E+00_JPRB, 0.64588E+00_JPRB ,&
&  0.63101E+00_JPRB, 0.61860E+00_JPRB, 0.61532E+00_JPRB, 0.61404E+00_JPRB, 0.61708E+00_JPRB, 0.61781E+00_JPRB, 0.61395E+00_JPRB ,&
&  0.61098E+00_JPRB, 0.61275E+00_JPRB, 0.62111E+00_JPRB, 0.62772E+00_JPRB, 0.62850E+00_JPRB, 0.62485E+00_JPRB, 0.63117E+00_JPRB ,&
&  0.63422E+00_JPRB, 0.64226E+00_JPRB, 0.65309E+00_JPRB, 0.67405E+00_JPRB, 0.68958E+00_JPRB, 0.69142E+00_JPRB, 0.69120E+00_JPRB ,&
&  0.69429E+00_JPRB, 0.68808E+00_JPRB, 0.67304E+00_JPRB, 0.66286E+00_JPRB, 0.65807E+00_JPRB, 0.65126E+00_JPRB, 0.64931E+00_JPRB ,&
&  0.65413E+00_JPRB, 0.65586E+00_JPRB, 0.65049E+00_JPRB, 0.65251E+00_JPRB, 0.66150E+00_JPRB, 0.67516E+00_JPRB, 0.68808E+00_JPRB ,&
&  0.68088E+00_JPRB, 0.68932E+00_JPRB, 0.69045E+00_JPRB, 0.70762E+00_JPRB, 0.72770E+00_JPRB, 0.72459E+00_JPRB, 0.72003E+00_JPRB ,&
&  0.71452E+00_JPRB, 0.69256E+00_JPRB, 0.69535E+00_JPRB, 0.69789E+00_JPRB, 0.70253E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.71067E+00_JPRB, 0.74013E+00_JPRB, 0.74608E+00_JPRB, 0.73067E+00_JPRB, 0.70538E+00_JPRB, 0.69143E+00_JPRB, 0.68354E+00_JPRB ,&
&  0.67914E+00_JPRB, 0.67576E+00_JPRB, 0.68525E+00_JPRB, 0.69226E+00_JPRB, 0.69488E+00_JPRB, 0.68850E+00_JPRB, 0.68201E+00_JPRB ,&
&  0.67061E+00_JPRB, 0.65601E+00_JPRB, 0.64573E+00_JPRB, 0.63684E+00_JPRB, 0.63690E+00_JPRB, 0.64590E+00_JPRB, 0.66312E+00_JPRB ,&
&  0.68491E+00_JPRB, 0.70015E+00_JPRB, 0.71228E+00_JPRB, 0.72464E+00_JPRB, 0.72661E+00_JPRB, 0.72626E+00_JPRB, 0.72883E+00_JPRB ,&
&  0.73508E+00_JPRB, 0.55185E+00_JPRB, 0.59477E+00_JPRB, 0.66243E+00_JPRB, 0.73833E+00_JPRB, 0.83405E+00_JPRB, 0.91340E+00_JPRB ,&
&  0.93842E+00_JPRB, 0.90514E+00_JPRB, 0.87721E+00_JPRB, 0.84356E+00_JPRB, 0.80636E+00_JPRB, 0.77285E+00_JPRB, 0.73634E+00_JPRB ,&
&  0.70868E+00_JPRB, 0.68863E+00_JPRB, 0.67855E+00_JPRB, 0.66966E+00_JPRB, 0.66003E+00_JPRB, 0.64807E+00_JPRB, 0.63448E+00_JPRB ,&
&  0.62122E+00_JPRB, 0.61458E+00_JPRB, 0.61102E+00_JPRB, 0.60806E+00_JPRB, 0.61026E+00_JPRB, 0.61562E+00_JPRB, 0.62065E+00_JPRB ,&
&  0.62956E+00_JPRB, 0.63587E+00_JPRB, 0.63565E+00_JPRB, 0.63218E+00_JPRB, 0.62845E+00_JPRB, 0.62829E+00_JPRB, 0.64031E+00_JPRB ,&
&  0.64524E+00_JPRB, 0.65530E+00_JPRB, 0.66727E+00_JPRB, 0.69249E+00_JPRB, 0.71515E+00_JPRB, 0.72099E+00_JPRB, 0.72055E+00_JPRB ,&
&  0.71953E+00_JPRB, 0.71123E+00_JPRB, 0.69729E+00_JPRB, 0.68203E+00_JPRB, 0.67171E+00_JPRB, 0.66885E+00_JPRB, 0.66135E+00_JPRB ,&
&  0.65981E+00_JPRB, 0.66254E+00_JPRB, 0.66350E+00_JPRB, 0.67319E+00_JPRB, 0.69031E+00_JPRB, 0.70287E+00_JPRB, 0.69996E+00_JPRB ,&
&  0.69894E+00_JPRB, 0.70383E+00_JPRB, 0.70596E+00_JPRB, 0.74832E+00_JPRB, 0.76662E+00_JPRB, 0.77293E+00_JPRB, 0.76549E+00_JPRB ,&
&  0.75654E+00_JPRB, 0.74763E+00_JPRB, 0.55877E+00_JPRB, 0.65264E+00_JPRB, 0.67773E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.74829E+00_JPRB, 0.77952E+00_JPRB, 0.77674E+00_JPRB, 0.74207E+00_JPRB, 0.70629E+00_JPRB, 0.68304E+00_JPRB, 0.66835E+00_JPRB ,&
&  0.66376E+00_JPRB, 0.65653E+00_JPRB, 0.65748E+00_JPRB, 0.66690E+00_JPRB, 0.67786E+00_JPRB, 0.68096E+00_JPRB, 0.66611E+00_JPRB ,&
&  0.64685E+00_JPRB, 0.63260E+00_JPRB, 0.62693E+00_JPRB, 0.62555E+00_JPRB, 0.62261E+00_JPRB, 0.62139E+00_JPRB, 0.63063E+00_JPRB ,&
&  0.64167E+00_JPRB, 0.65472E+00_JPRB, 0.66828E+00_JPRB, 0.67905E+00_JPRB, 0.68670E+00_JPRB, 0.69351E+00_JPRB, 0.71147E+00_JPRB ,&
&  0.72032E+00_JPRB, 0.54975E+00_JPRB, 0.58199E+00_JPRB, 0.72867E+00_JPRB, 0.86702E+00_JPRB, 0.10086E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.10291E+01_JPRB, 0.97726E+00_JPRB, 0.91883E+00_JPRB, 0.85748E+00_JPRB, 0.80312E+00_JPRB, 0.75298E+00_JPRB ,&
&  0.71803E+00_JPRB, 0.69181E+00_JPRB, 0.67602E+00_JPRB, 0.65870E+00_JPRB, 0.64794E+00_JPRB, 0.64089E+00_JPRB, 0.63437E+00_JPRB ,&
&  0.62916E+00_JPRB, 0.62225E+00_JPRB, 0.62274E+00_JPRB, 0.62760E+00_JPRB, 0.62743E+00_JPRB, 0.63427E+00_JPRB, 0.63525E+00_JPRB ,&
&  0.64457E+00_JPRB, 0.65689E+00_JPRB, 0.65833E+00_JPRB, 0.65375E+00_JPRB, 0.65355E+00_JPRB, 0.66150E+00_JPRB, 0.67417E+00_JPRB ,&
&  0.68368E+00_JPRB, 0.70645E+00_JPRB, 0.72628E+00_JPRB, 0.74849E+00_JPRB, 0.76241E+00_JPRB, 0.76093E+00_JPRB, 0.74865E+00_JPRB ,&
&  0.74959E+00_JPRB, 0.74264E+00_JPRB, 0.73321E+00_JPRB, 0.72107E+00_JPRB, 0.70763E+00_JPRB, 0.69616E+00_JPRB, 0.68847E+00_JPRB ,&
&  0.68715E+00_JPRB, 0.69508E+00_JPRB, 0.70225E+00_JPRB, 0.72284E+00_JPRB, 0.73657E+00_JPRB, 0.73919E+00_JPRB, 0.74218E+00_JPRB ,&
&  0.74635E+00_JPRB, 0.74899E+00_JPRB, 0.75082E+00_JPRB, 0.77036E+00_JPRB, 0.83182E+00_JPRB, 0.83966E+00_JPRB, 0.83023E+00_JPRB ,&
&  0.81397E+00_JPRB, 0.77740E+00_JPRB, 0.76326E+00_JPRB, 0.62150E+00_JPRB, 0.70584E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.85917E+00_JPRB, 0.84540E+00_JPRB, 0.80356E+00_JPRB, 0.75315E+00_JPRB, 0.70825E+00_JPRB, 0.67190E+00_JPRB, 0.65144E+00_JPRB ,&
&  0.63492E+00_JPRB, 0.62267E+00_JPRB, 0.62465E+00_JPRB, 0.63326E+00_JPRB, 0.64378E+00_JPRB, 0.64526E+00_JPRB, 0.63895E+00_JPRB ,&
&  0.62900E+00_JPRB, 0.61496E+00_JPRB, 0.60728E+00_JPRB, 0.60326E+00_JPRB, 0.59595E+00_JPRB, 0.59358E+00_JPRB, 0.59300E+00_JPRB ,&
&  0.59700E+00_JPRB, 0.60278E+00_JPRB, 0.60609E+00_JPRB, 0.61505E+00_JPRB, 0.62960E+00_JPRB, 0.65155E+00_JPRB, 0.67397E+00_JPRB ,&
&  0.67417E+00_JPRB, 0.57313E+00_JPRB, 0.66130E+00_JPRB, 0.87219E+00_JPRB, 0.10787E+01_JPRB, 0.11729E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.11218E+01_JPRB, 0.10615E+01_JPRB, 0.99193E+00_JPRB, 0.91765E+00_JPRB, 0.85082E+00_JPRB, 0.79438E+00_JPRB, 0.74422E+00_JPRB ,&
&  0.70668E+00_JPRB, 0.68050E+00_JPRB, 0.66590E+00_JPRB, 0.65635E+00_JPRB, 0.64932E+00_JPRB, 0.63421E+00_JPRB, 0.62460E+00_JPRB ,&
&  0.62365E+00_JPRB, 0.62420E+00_JPRB, 0.62684E+00_JPRB, 0.62996E+00_JPRB, 0.64308E+00_JPRB, 0.66364E+00_JPRB, 0.67379E+00_JPRB ,&
&  0.67352E+00_JPRB, 0.67986E+00_JPRB, 0.68304E+00_JPRB, 0.67673E+00_JPRB, 0.67404E+00_JPRB, 0.69931E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.74790E+00_JPRB, 0.78358E+00_JPRB, 0.82658E+00_JPRB, 0.85025E+00_JPRB, 0.85028E+00_JPRB, 0.83444E+00_JPRB, 0.80499E+00_JPRB ,&
&  0.78785E+00_JPRB, 0.76647E+00_JPRB, 0.74660E+00_JPRB, 0.73349E+00_JPRB, 0.72369E+00_JPRB, 0.71444E+00_JPRB, 0.70493E+00_JPRB ,&
&  0.69828E+00_JPRB, 0.69051E+00_JPRB, 0.69348E+00_JPRB, 0.71603E+00_JPRB, 0.74008E+00_JPRB, 0.75297E+00_JPRB, 0.75587E+00_JPRB ,&
&  0.75316E+00_JPRB, 0.75564E+00_JPRB, 0.77119E+00_JPRB, 0.85912E+00_JPRB, 0.85756E+00_JPRB, 0.86329E+00_JPRB, 0.85277E+00_JPRB ,&
&  0.84042E+00_JPRB, 0.81738E+00_JPRB, 0.78664E+00_JPRB, 0.76882E+00_JPRB, 0.78750E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.92988E+00_JPRB, 0.86861E+00_JPRB, 0.81210E+00_JPRB, 0.74949E+00_JPRB, 0.70307E+00_JPRB, 0.68158E+00_JPRB, 0.66549E+00_JPRB ,&
&  0.64423E+00_JPRB, 0.62515E+00_JPRB, 0.62154E+00_JPRB, 0.62339E+00_JPRB, 0.62179E+00_JPRB, 0.61619E+00_JPRB, 0.60489E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.60569E+00_JPRB, 0.60158E+00_JPRB, 0.59301E+00_JPRB, 0.58377E+00_JPRB, 0.57605E+00_JPRB, 0.57035E+00_JPRB ,&
&  0.56735E+00_JPRB, 0.57111E+00_JPRB, 0.57357E+00_JPRB, 0.58440E+00_JPRB, 0.60331E+00_JPRB, 0.62564E+00_JPRB, 0.65358E+00_JPRB ,&
&  0.65235E+00_JPRB, 0.58960E+00_JPRB, 0.81876E+00_JPRB, 0.11523E+01_JPRB, 0.14103E+01_JPRB, 0.14095E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.11570E+01_JPRB, 0.10357E+01_JPRB, 0.94562E+00_JPRB, 0.88023E+00_JPRB, 0.82819E+00_JPRB, 0.78662E+00_JPRB, 0.75560E+00_JPRB ,&
&  0.72356E+00_JPRB, 0.69856E+00_JPRB, 0.67531E+00_JPRB, 0.65410E+00_JPRB, 0.64949E+00_JPRB, 0.64409E+00_JPRB, 0.63615E+00_JPRB ,&
&  0.63075E+00_JPRB, 0.63098E+00_JPRB, 0.63323E+00_JPRB, 0.63888E+00_JPRB, 0.65540E+00_JPRB, 0.70497E+00_JPRB, 0.75059E+00_JPRB ,&
&  0.75894E+00_JPRB, 0.76871E+00_JPRB, 0.76703E+00_JPRB, 0.76061E+00_JPRB, 0.77048E+00_JPRB, 0.79158E+00_JPRB, 0.83075E+00_JPRB ,&
&  0.87810E+00_JPRB, 0.90914E+00_JPRB, 0.92691E+00_JPRB, 0.93983E+00_JPRB, 0.93086E+00_JPRB, 0.91466E+00_JPRB, 0.89138E+00_JPRB ,&
&  0.86698E+00_JPRB, 0.83314E+00_JPRB, 0.80442E+00_JPRB, 0.78411E+00_JPRB, 0.76487E+00_JPRB, 0.75065E+00_JPRB, 0.73386E+00_JPRB ,&
&  0.71596E+00_JPRB, 0.69843E+00_JPRB, 0.70845E+00_JPRB, 0.72812E+00_JPRB, 0.74107E+00_JPRB, 0.74324E+00_JPRB, 0.75381E+00_JPRB ,&
&  0.76342E+00_JPRB, 0.79634E+00_JPRB, 0.81097E+00_JPRB, 0.95384E+00_JPRB, 0.10031E+01_JPRB, 0.97541E+00_JPRB, 0.94676E+00_JPRB ,&
&  0.90731E+00_JPRB, 0.86682E+00_JPRB, 0.82405E+00_JPRB, 0.80667E+00_JPRB, 0.93223E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.80774E+00_JPRB, 0.75930E+00_JPRB, 0.70630E+00_JPRB, 0.67097E+00_JPRB, 0.64237E+00_JPRB, 0.61935E+00_JPRB, 0.61413E+00_JPRB ,&
&  0.60943E+00_JPRB, 0.60062E+00_JPRB, 0.60589E+00_JPRB, 0.60318E+00_JPRB, 0.59621E+00_JPRB, 0.58185E+00_JPRB, 0.58595E+00_JPRB ,&
&  0.59172E+00_JPRB, 0.58805E+00_JPRB, 0.58158E+00_JPRB, 0.57400E+00_JPRB, 0.56653E+00_JPRB, 0.55568E+00_JPRB, 0.54880E+00_JPRB ,&
&  0.54349E+00_JPRB, 0.54477E+00_JPRB, 0.55244E+00_JPRB, 0.56605E+00_JPRB, 0.58954E+00_JPRB, 0.61882E+00_JPRB, 0.64766E+00_JPRB ,&
&  0.63947E+00_JPRB, 0.72584E+00_JPRB, 0.10944E+01_JPRB, 0.17189E+01_JPRB, 0.19245E+01_JPRB, 0.15644E+01_JPRB, 0.12843E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.95146E+00_JPRB, 0.87671E+00_JPRB, 0.83240E+00_JPRB, 0.79216E+00_JPRB, 0.77239E+00_JPRB, 0.75495E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.70935E+00_JPRB, 0.67673E+00_JPRB, 0.65459E+00_JPRB, 0.64497E+00_JPRB, 0.64461E+00_JPRB, 0.64009E+00_JPRB ,&
&  0.63648E+00_JPRB, 0.64568E+00_JPRB, 0.65297E+00_JPRB, 0.66331E+00_JPRB, 0.72988E+00_JPRB, 0.82642E+00_JPRB, 0.94370E+00_JPRB ,&
&  0.92954E+00_JPRB, 0.90835E+00_JPRB, 0.90719E+00_JPRB, 0.90472E+00_JPRB, 0.92177E+00_JPRB, 0.93416E+00_JPRB, 0.95713E+00_JPRB ,&
&  0.97890E+00_JPRB, 0.97017E+00_JPRB, 0.94483E+00_JPRB, 0.92533E+00_JPRB, 0.90432E+00_JPRB, 0.89456E+00_JPRB, 0.86167E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.81078E+00_JPRB, 0.79861E+00_JPRB, 0.77392E+00_JPRB, 0.76130E+00_JPRB, 0.74560E+00_JPRB, 0.73038E+00_JPRB ,&
&  0.70676E+00_JPRB, 0.69414E+00_JPRB, 0.72836E+00_JPRB, 0.74710E+00_JPRB, 0.75004E+00_JPRB, 0.75216E+00_JPRB, 0.76947E+00_JPRB ,&
&  0.79897E+00_JPRB, 0.86253E+00_JPRB, 0.89962E+00_JPRB, 0.98770E+00_JPRB, 0.10111E+01_JPRB, 0.10442E+01_JPRB, 0.98892E+00_JPRB ,&
&  0.93167E+00_JPRB, 0.86418E+00_JPRB, 0.79696E+00_JPRB, 0.76886E+00_JPRB, 0.82722E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.74704E+00_JPRB, 0.68232E+00_JPRB, 0.63173E+00_JPRB, 0.60258E+00_JPRB, 0.57651E+00_JPRB, 0.56494E+00_JPRB, 0.55920E+00_JPRB ,&
&  0.56839E+00_JPRB, 0.57695E+00_JPRB, 0.59162E+00_JPRB, 0.58863E+00_JPRB, 0.56742E+00_JPRB, 0.55991E+00_JPRB, 0.56835E+00_JPRB ,&
&  0.56137E+00_JPRB, 0.56325E+00_JPRB, 0.57079E+00_JPRB, 0.56486E+00_JPRB, 0.55652E+00_JPRB, 0.54588E+00_JPRB, 0.53395E+00_JPRB ,&
&  0.53773E+00_JPRB, 0.54444E+00_JPRB, 0.55839E+00_JPRB, 0.58015E+00_JPRB, 0.60896E+00_JPRB, 0.63753E+00_JPRB, 0.65886E+00_JPRB ,&
&  0.67795E+00_JPRB, 0.97100E+00_JPRB, 0.16020E+01_JPRB, 0.20997E+01_JPRB, 0.19244E+01_JPRB, 0.14446E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.89789E+00_JPRB, 0.84901E+00_JPRB, 0.81803E+00_JPRB, 0.78158E+00_JPRB, 0.76597E+00_JPRB, 0.76702E+00_JPRB ,&
&  0.75340E+00_JPRB, 0.72561E+00_JPRB, 0.69363E+00_JPRB, 0.66616E+00_JPRB, 0.65084E+00_JPRB, 0.64953E+00_JPRB, 0.65357E+00_JPRB ,&
&  0.66122E+00_JPRB, 0.67299E+00_JPRB, 0.69288E+00_JPRB, 0.71401E+00_JPRB, 0.77038E+00_JPRB, 0.77929E+00_JPRB, 0.10161E+01_JPRB ,&
&  0.12562E+01_JPRB, 0.12017E+01_JPRB, 0.11613E+01_JPRB, 0.11267E+01_JPRB, 0.10867E+01_JPRB, 0.10448E+01_JPRB, 0.10296E+01_JPRB ,&
&  0.99996E+00_JPRB, 0.95145E+00_JPRB, 0.90829E+00_JPRB, 0.88018E+00_JPRB, 0.84689E+00_JPRB, 0.82198E+00_JPRB, 0.78498E+00_JPRB ,&
&  0.76828E+00_JPRB, 0.75192E+00_JPRB, 0.74234E+00_JPRB, 0.74349E+00_JPRB, 0.74748E+00_JPRB, 0.73064E+00_JPRB, 0.71951E+00_JPRB ,&
&  0.70681E+00_JPRB, 0.72636E+00_JPRB, 0.77277E+00_JPRB, 0.74906E+00_JPRB, 0.75268E+00_JPRB, 0.78119E+00_JPRB, 0.80283E+00_JPRB ,&
&  0.80343E+00_JPRB, 0.86012E+00_JPRB, 0.92558E+00_JPRB, 0.97560E+00_JPRB, 0.99812E+00_JPRB, 0.10490E+01_JPRB, 0.96487E+00_JPRB ,&
&  0.88975E+00_JPRB, 0.81960E+00_JPRB, 0.77575E+00_JPRB, 0.74549E+00_JPRB, 0.77601E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.66396E+00_JPRB, 0.61384E+00_JPRB, 0.58371E+00_JPRB, 0.56073E+00_JPRB, 0.56261E+00_JPRB, 0.55619E+00_JPRB, 0.55136E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.57732E+00_JPRB, 0.58968E+00_JPRB, 0.57839E+00_JPRB, 0.55594E+00_JPRB, 0.54924E+00_JPRB, 0.54062E+00_JPRB ,&
&  0.53128E+00_JPRB, 0.55548E+00_JPRB, 0.57951E+00_JPRB, 0.59006E+00_JPRB, 0.58103E+00_JPRB, 0.55584E+00_JPRB, 0.54289E+00_JPRB ,&
&  0.55012E+00_JPRB, 0.56834E+00_JPRB, 0.59695E+00_JPRB, 0.62857E+00_JPRB, 0.65579E+00_JPRB, 0.68156E+00_JPRB, 0.71851E+00_JPRB ,&
&  0.83244E+00_JPRB, 0.10680E+01_JPRB, 0.17168E+01_JPRB, 0.23351E+01_JPRB, 0.18182E+01_JPRB, 0.13512E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.98371E+00_JPRB, 0.88384E+00_JPRB, 0.85603E+00_JPRB, 0.83419E+00_JPRB, 0.80370E+00_JPRB, 0.78462E+00_JPRB, 0.78384E+00_JPRB ,&
&  0.76358E+00_JPRB, 0.74402E+00_JPRB, 0.72524E+00_JPRB, 0.71108E+00_JPRB, 0.70329E+00_JPRB, 0.70105E+00_JPRB, 0.70329E+00_JPRB ,&
&  0.71749E+00_JPRB, 0.72985E+00_JPRB, 0.75049E+00_JPRB, 0.78859E+00_JPRB, 0.81686E+00_JPRB, 0.91126E+00_JPRB, 0.10919E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.13772E+01_JPRB, 0.12797E+01_JPRB, 0.12106E+01_JPRB, 0.11509E+01_JPRB, 0.11121E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10043E+01_JPRB, 0.94040E+00_JPRB, 0.88406E+00_JPRB, 0.83988E+00_JPRB, 0.78441E+00_JPRB, 0.75413E+00_JPRB, 0.74833E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.73279E+00_JPRB, 0.72626E+00_JPRB, 0.73935E+00_JPRB, 0.72980E+00_JPRB, 0.72240E+00_JPRB, 0.72118E+00_JPRB ,&
&  0.73609E+00_JPRB, 0.79107E+00_JPRB, 0.81968E+00_JPRB, 0.76210E+00_JPRB, 0.74146E+00_JPRB, 0.75679E+00_JPRB, 0.78722E+00_JPRB ,&
&  0.83297E+00_JPRB, 0.89554E+00_JPRB, 0.93015E+00_JPRB, 0.96677E+00_JPRB, 0.92442E+00_JPRB, 0.98665E+00_JPRB, 0.89829E+00_JPRB ,&
&  0.82794E+00_JPRB, 0.78477E+00_JPRB, 0.76244E+00_JPRB, 0.73905E+00_JPRB, 0.72278E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.65329E+00_JPRB, 0.61633E+00_JPRB, 0.57153E+00_JPRB, 0.55207E+00_JPRB, 0.55913E+00_JPRB, 0.54587E+00_JPRB, 0.55125E+00_JPRB ,&
&  0.56732E+00_JPRB, 0.56650E+00_JPRB, 0.56538E+00_JPRB, 0.55647E+00_JPRB, 0.54921E+00_JPRB, 0.53809E+00_JPRB, 0.53187E+00_JPRB ,&
&  0.54545E+00_JPRB, 0.58123E+00_JPRB, 0.59075E+00_JPRB, 0.57921E+00_JPRB, 0.57092E+00_JPRB, 0.55560E+00_JPRB, 0.55121E+00_JPRB ,&
&  0.57644E+00_JPRB, 0.62514E+00_JPRB, 0.67625E+00_JPRB, 0.70535E+00_JPRB, 0.72766E+00_JPRB, 0.75389E+00_JPRB, 0.86296E+00_JPRB ,&
&  0.11557E+01_JPRB, 0.27668E+01_JPRB, 0.12818E+01_JPRB, 0.24244E+01_JPRB, 0.16381E+01_JPRB, 0.12247E+01_JPRB, 0.91839E+00_JPRB ,&
&  0.85973E+00_JPRB, 0.89058E+00_JPRB, 0.87280E+00_JPRB, 0.87452E+00_JPRB, 0.83536E+00_JPRB, 0.80054E+00_JPRB, 0.79333E+00_JPRB ,&
&  0.78033E+00_JPRB, 0.77343E+00_JPRB, 0.78067E+00_JPRB, 0.79036E+00_JPRB, 0.79278E+00_JPRB, 0.78653E+00_JPRB, 0.78328E+00_JPRB ,&
&  0.79026E+00_JPRB, 0.79223E+00_JPRB, 0.82449E+00_JPRB, 0.86281E+00_JPRB, 0.89267E+00_JPRB, 0.10066E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.13558E+01_JPRB, 0.12340E+01_JPRB, 0.11797E+01_JPRB, 0.11189E+01_JPRB, 0.10656E+01_JPRB, 0.98154E+00_JPRB ,&
&  0.90661E+00_JPRB, 0.86157E+00_JPRB, 0.81766E+00_JPRB, 0.78211E+00_JPRB, 0.74395E+00_JPRB, 0.73207E+00_JPRB, 0.71592E+00_JPRB ,&
&  0.72093E+00_JPRB, 0.74637E+00_JPRB, 0.74179E+00_JPRB, 0.73935E+00_JPRB, 0.73238E+00_JPRB, 0.74612E+00_JPRB, 0.76355E+00_JPRB ,&
&  0.79756E+00_JPRB, 0.85617E+00_JPRB, 0.84523E+00_JPRB, 0.74809E+00_JPRB, 0.71705E+00_JPRB, 0.74254E+00_JPRB, 0.77276E+00_JPRB ,&
&  0.84977E+00_JPRB, 0.91039E+00_JPRB, 0.91162E+00_JPRB, 0.91414E+00_JPRB, 0.90914E+00_JPRB, 0.96169E+00_JPRB, 0.85025E+00_JPRB ,&
&  0.78768E+00_JPRB, 0.76220E+00_JPRB, 0.74310E+00_JPRB, 0.71191E+00_JPRB, 0.70057E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.65185E+00_JPRB, 0.63118E+00_JPRB, 0.58838E+00_JPRB, 0.55271E+00_JPRB, 0.55048E+00_JPRB, 0.55587E+00_JPRB, 0.56365E+00_JPRB ,&
&  0.56381E+00_JPRB, 0.56130E+00_JPRB, 0.55667E+00_JPRB, 0.55129E+00_JPRB, 0.55805E+00_JPRB, 0.55266E+00_JPRB, 0.57837E+00_JPRB ,&
&  0.60018E+00_JPRB, 0.59677E+00_JPRB, 0.60181E+00_JPRB, 0.57991E+00_JPRB, 0.57751E+00_JPRB, 0.57364E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.65589E+00_JPRB, 0.72310E+00_JPRB, 0.76402E+00_JPRB, 0.78072E+00_JPRB, 0.80167E+00_JPRB, 0.84617E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.29579E+01_JPRB, 0.14981E+01_JPRB, 0.21904E+01_JPRB, 0.13958E+01_JPRB, 0.10347E+01_JPRB, 0.82421E+00_JPRB ,&
&  0.80953E+00_JPRB, 0.91887E+00_JPRB, 0.91026E+00_JPRB, 0.90160E+00_JPRB, 0.81974E+00_JPRB, 0.78664E+00_JPRB, 0.77815E+00_JPRB ,&
&  0.78889E+00_JPRB, 0.83605E+00_JPRB, 0.88050E+00_JPRB, 0.90599E+00_JPRB, 0.89629E+00_JPRB, 0.87175E+00_JPRB, 0.87099E+00_JPRB ,&
&  0.87104E+00_JPRB, 0.86194E+00_JPRB, 0.91637E+00_JPRB, 0.93480E+00_JPRB, 0.10593E+01_JPRB, 0.11316E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.12148E+01_JPRB, 0.11522E+01_JPRB, 0.11265E+01_JPRB, 0.10477E+01_JPRB, 0.99284E+00_JPRB, 0.90795E+00_JPRB ,&
&  0.85887E+00_JPRB, 0.82395E+00_JPRB, 0.78793E+00_JPRB, 0.74586E+00_JPRB, 0.72796E+00_JPRB, 0.73094E+00_JPRB, 0.71939E+00_JPRB ,&
&  0.74432E+00_JPRB, 0.77382E+00_JPRB, 0.77116E+00_JPRB, 0.77284E+00_JPRB, 0.78207E+00_JPRB, 0.80179E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.85677E+00_JPRB, 0.90337E+00_JPRB, 0.85037E+00_JPRB, 0.74373E+00_JPRB, 0.73974E+00_JPRB, 0.76695E+00_JPRB, 0.78048E+00_JPRB ,&
&  0.84222E+00_JPRB, 0.89049E+00_JPRB, 0.88147E+00_JPRB, 0.84875E+00_JPRB, 0.84911E+00_JPRB, 0.88307E+00_JPRB, 0.81074E+00_JPRB ,&
&  0.77530E+00_JPRB, 0.75928E+00_JPRB, 0.75147E+00_JPRB, 0.73825E+00_JPRB, 0.70357E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.60741E+00_JPRB, 0.59186E+00_JPRB, 0.57488E+00_JPRB, 0.55940E+00_JPRB, 0.55105E+00_JPRB, 0.55773E+00_JPRB, 0.56235E+00_JPRB ,&
&  0.55565E+00_JPRB, 0.55343E+00_JPRB, 0.54725E+00_JPRB, 0.55267E+00_JPRB, 0.57841E+00_JPRB, 0.62673E+00_JPRB, 0.67418E+00_JPRB ,&
&  0.67340E+00_JPRB, 0.64055E+00_JPRB, 0.63505E+00_JPRB, 0.62992E+00_JPRB, 0.64651E+00_JPRB, 0.66849E+00_JPRB, 0.71128E+00_JPRB ,&
&  0.77078E+00_JPRB, 0.81723E+00_JPRB, 0.84239E+00_JPRB, 0.85749E+00_JPRB, 0.88537E+00_JPRB, 0.92439E+00_JPRB, 0.11799E+01_JPRB ,&
&  0.17200E+01_JPRB, 0.34386E+01_JPRB, 0.26536E+01_JPRB, 0.17280E+01_JPRB, 0.11510E+01_JPRB, 0.93980E+00_JPRB, 0.86954E+00_JPRB ,&
&  0.80750E+00_JPRB, 0.73112E+00_JPRB, 0.93936E+00_JPRB, 0.93203E+00_JPRB, 0.85518E+00_JPRB, 0.82004E+00_JPRB, 0.84297E+00_JPRB ,&
&  0.90643E+00_JPRB, 0.96271E+00_JPRB, 0.10022E+01_JPRB, 0.10062E+01_JPRB, 0.97676E+00_JPRB, 0.96076E+00_JPRB, 0.96075E+00_JPRB ,&
&  0.95596E+00_JPRB, 0.98103E+00_JPRB, 0.11159E+01_JPRB, 0.11566E+01_JPRB, 0.11758E+01_JPRB, 0.11399E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10875E+01_JPRB, 0.11058E+01_JPRB, 0.11024E+01_JPRB, 0.95497E+00_JPRB, 0.97788E+00_JPRB, 0.88942E+00_JPRB ,&
&  0.86555E+00_JPRB, 0.82790E+00_JPRB, 0.78470E+00_JPRB, 0.73031E+00_JPRB, 0.74274E+00_JPRB, 0.75252E+00_JPRB, 0.73808E+00_JPRB ,&
&  0.76963E+00_JPRB, 0.80639E+00_JPRB, 0.79470E+00_JPRB, 0.80610E+00_JPRB, 0.81648E+00_JPRB, 0.83840E+00_JPRB, 0.85852E+00_JPRB ,&
&  0.87425E+00_JPRB, 0.91575E+00_JPRB, 0.86733E+00_JPRB, 0.81446E+00_JPRB, 0.80193E+00_JPRB, 0.78542E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.82921E+00_JPRB, 0.89514E+00_JPRB, 0.87047E+00_JPRB, 0.86823E+00_JPRB, 0.83410E+00_JPRB, 0.82829E+00_JPRB, 0.79467E+00_JPRB ,&
&  0.79212E+00_JPRB, 0.78277E+00_JPRB, 0.81255E+00_JPRB, 0.74874E+00_JPRB, 0.66293E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.54955E+00_JPRB, 0.54256E+00_JPRB, 0.53686E+00_JPRB, 0.54151E+00_JPRB, 0.54538E+00_JPRB, 0.53981E+00_JPRB, 0.54264E+00_JPRB ,&
&  0.55190E+00_JPRB, 0.56355E+00_JPRB, 0.56232E+00_JPRB, 0.59412E+00_JPRB, 0.66519E+00_JPRB, 0.74041E+00_JPRB, 0.74339E+00_JPRB ,&
&  0.73176E+00_JPRB, 0.71775E+00_JPRB, 0.71178E+00_JPRB, 0.72476E+00_JPRB, 0.73688E+00_JPRB, 0.77749E+00_JPRB, 0.81975E+00_JPRB ,&
&  0.86221E+00_JPRB, 0.90735E+00_JPRB, 0.94852E+00_JPRB, 0.99054E+00_JPRB, 0.10618E+01_JPRB, 0.11711E+01_JPRB, 0.14554E+01_JPRB ,&
&  0.19620E+01_JPRB, 0.34437E+01_JPRB, 0.10760E+01_JPRB, 0.11077E+01_JPRB, 0.10523E+01_JPRB, 0.89238E+00_JPRB, 0.76101E+00_JPRB ,&
&  0.67620E+00_JPRB, 0.61321E+00_JPRB, 0.82295E+00_JPRB, 0.95604E+00_JPRB, 0.90513E+00_JPRB, 0.92652E+00_JPRB, 0.96908E+00_JPRB ,&
&  0.10465E+01_JPRB, 0.10926E+01_JPRB, 0.11130E+01_JPRB, 0.10858E+01_JPRB, 0.10589E+01_JPRB, 0.10506E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.12182E+01_JPRB, 0.12815E+01_JPRB, 0.11836E+01_JPRB, 0.12129E+01_JPRB, 0.11354E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.94093E+00_JPRB, 0.98624E+00_JPRB, 0.10515E+01_JPRB, 0.10874E+01_JPRB, 0.93523E+00_JPRB, 0.85759E+00_JPRB, 0.88361E+00_JPRB ,&
&  0.85123E+00_JPRB, 0.79518E+00_JPRB, 0.74409E+00_JPRB, 0.70651E+00_JPRB, 0.71463E+00_JPRB, 0.74230E+00_JPRB, 0.73307E+00_JPRB ,&
&  0.76462E+00_JPRB, 0.80216E+00_JPRB, 0.78323E+00_JPRB, 0.81223E+00_JPRB, 0.83440E+00_JPRB, 0.85034E+00_JPRB, 0.86095E+00_JPRB ,&
&  0.87659E+00_JPRB, 0.91881E+00_JPRB, 0.94618E+00_JPRB, 0.92480E+00_JPRB, 0.86685E+00_JPRB, 0.83495E+00_JPRB, 0.85768E+00_JPRB ,&
&  0.88625E+00_JPRB, 0.93636E+00_JPRB, 0.85870E+00_JPRB, 0.80831E+00_JPRB, 0.81444E+00_JPRB, 0.80789E+00_JPRB, 0.82235E+00_JPRB ,&
&  0.85730E+00_JPRB, 0.93479E+00_JPRB, 0.97808E+00_JPRB, 0.69426E+00_JPRB, 0.59690E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.51984E+00_JPRB, 0.51390E+00_JPRB, 0.50402E+00_JPRB, 0.51584E+00_JPRB, 0.51530E+00_JPRB, 0.52376E+00_JPRB, 0.54100E+00_JPRB ,&
&  0.56969E+00_JPRB, 0.60670E+00_JPRB, 0.63846E+00_JPRB, 0.68137E+00_JPRB, 0.74237E+00_JPRB, 0.79768E+00_JPRB, 0.80064E+00_JPRB ,&
&  0.78750E+00_JPRB, 0.78075E+00_JPRB, 0.78402E+00_JPRB, 0.79104E+00_JPRB, 0.81701E+00_JPRB, 0.85173E+00_JPRB, 0.88675E+00_JPRB ,&
&  0.93702E+00_JPRB, 0.10060E+01_JPRB, 0.10766E+01_JPRB, 0.11627E+01_JPRB, 0.12717E+01_JPRB, 0.14414E+01_JPRB, 0.16659E+01_JPRB ,&
&  0.22825E+01_JPRB, 0.21857E+01_JPRB, 0.86599E+00_JPRB, 0.11017E+01_JPRB, 0.95499E+00_JPRB, 0.78074E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.55929E+00_JPRB, 0.67169E+00_JPRB, 0.87202E+00_JPRB, 0.99622E+00_JPRB, 0.98527E+00_JPRB, 0.10453E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.11779E+01_JPRB, 0.11777E+01_JPRB, 0.11642E+01_JPRB, 0.11693E+01_JPRB, 0.11981E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.13307E+01_JPRB, 0.13819E+01_JPRB, 0.13929E+01_JPRB, 0.12289E+01_JPRB, 0.12715E+01_JPRB, 0.11729E+01_JPRB, 0.11254E+01_JPRB ,&
&  0.97586E+00_JPRB, 0.91673E+00_JPRB, 0.95781E+00_JPRB, 0.10212E+01_JPRB, 0.89422E+00_JPRB, 0.79591E+00_JPRB, 0.85524E+00_JPRB ,&
&  0.80303E+00_JPRB, 0.73542E+00_JPRB, 0.71152E+00_JPRB, 0.70191E+00_JPRB, 0.70460E+00_JPRB, 0.71403E+00_JPRB, 0.73176E+00_JPRB ,&
&  0.75537E+00_JPRB, 0.75401E+00_JPRB, 0.73047E+00_JPRB, 0.75259E+00_JPRB, 0.77859E+00_JPRB, 0.80544E+00_JPRB, 0.82786E+00_JPRB ,&
&  0.86025E+00_JPRB, 0.89939E+00_JPRB, 0.92960E+00_JPRB, 0.95042E+00_JPRB, 0.91394E+00_JPRB, 0.86849E+00_JPRB, 0.88150E+00_JPRB ,&
&  0.83335E+00_JPRB, 0.82229E+00_JPRB, 0.80859E+00_JPRB, 0.79309E+00_JPRB, 0.76451E+00_JPRB, 0.79481E+00_JPRB, 0.81706E+00_JPRB ,&
&  0.85786E+00_JPRB, 0.89303E+00_JPRB, 0.81553E+00_JPRB, 0.62108E+00_JPRB, 0.54990E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.52589E+00_JPRB, 0.52738E+00_JPRB, 0.51846E+00_JPRB, 0.51676E+00_JPRB, 0.50972E+00_JPRB, 0.51465E+00_JPRB, 0.52884E+00_JPRB ,&
&  0.55968E+00_JPRB, 0.61236E+00_JPRB, 0.67686E+00_JPRB, 0.73462E+00_JPRB, 0.80382E+00_JPRB, 0.85633E+00_JPRB, 0.85756E+00_JPRB ,&
&  0.83959E+00_JPRB, 0.83444E+00_JPRB, 0.82168E+00_JPRB, 0.84470E+00_JPRB, 0.86637E+00_JPRB, 0.90121E+00_JPRB, 0.94243E+00_JPRB ,&
&  0.99180E+00_JPRB, 0.10549E+01_JPRB, 0.11333E+01_JPRB, 0.12249E+01_JPRB, 0.13509E+01_JPRB, 0.14851E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.57024E+00_JPRB, 0.96280E+00_JPRB, 0.10431E+01_JPRB, 0.79247E+00_JPRB, 0.68069E+00_JPRB, 0.56813E+00_JPRB ,&
&  0.54769E+00_JPRB, 0.76271E+00_JPRB, 0.91331E+00_JPRB, 0.10397E+01_JPRB, 0.10653E+01_JPRB, 0.11348E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.12092E+01_JPRB, 0.12317E+01_JPRB, 0.12545E+01_JPRB, 0.12761E+01_JPRB, 0.13123E+01_JPRB, 0.13549E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.14879E+01_JPRB, 0.14823E+01_JPRB, 0.12577E+01_JPRB, 0.13599E+01_JPRB, 0.13599E+01_JPRB, 0.14300E+01_JPRB ,&
&  0.14639E+01_JPRB, 0.94681E+00_JPRB, 0.82645E+00_JPRB, 0.10467E+01_JPRB, 0.96477E+00_JPRB, 0.75717E+00_JPRB, 0.82274E+00_JPRB ,&
&  0.74877E+00_JPRB, 0.68561E+00_JPRB, 0.67630E+00_JPRB, 0.68436E+00_JPRB, 0.67084E+00_JPRB, 0.69204E+00_JPRB, 0.70769E+00_JPRB ,&
&  0.73172E+00_JPRB, 0.71195E+00_JPRB, 0.68745E+00_JPRB, 0.69991E+00_JPRB, 0.72336E+00_JPRB, 0.75394E+00_JPRB, 0.77674E+00_JPRB ,&
&  0.80855E+00_JPRB, 0.83409E+00_JPRB, 0.86167E+00_JPRB, 0.89551E+00_JPRB, 0.92404E+00_JPRB, 0.87958E+00_JPRB, 0.85948E+00_JPRB ,&
&  0.78215E+00_JPRB, 0.75129E+00_JPRB, 0.72289E+00_JPRB, 0.70664E+00_JPRB, 0.69752E+00_JPRB, 0.72144E+00_JPRB, 0.75431E+00_JPRB ,&
&  0.78445E+00_JPRB, 0.78967E+00_JPRB, 0.72195E+00_JPRB, 0.60554E+00_JPRB, 0.54410E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.51476E+00_JPRB, 0.51693E+00_JPRB, 0.51432E+00_JPRB, 0.52344E+00_JPRB, 0.52177E+00_JPRB, 0.52141E+00_JPRB, 0.51784E+00_JPRB ,&
&  0.54176E+00_JPRB, 0.58849E+00_JPRB, 0.66012E+00_JPRB, 0.74082E+00_JPRB, 0.80043E+00_JPRB, 0.87464E+00_JPRB, 0.90872E+00_JPRB ,&
&  0.90098E+00_JPRB, 0.87886E+00_JPRB, 0.87302E+00_JPRB, 0.89492E+00_JPRB, 0.91158E+00_JPRB, 0.94277E+00_JPRB, 0.97634E+00_JPRB ,&
&  0.10124E+01_JPRB, 0.10650E+01_JPRB, 0.11287E+01_JPRB, 0.12017E+01_JPRB, 0.12890E+01_JPRB, 0.13443E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.53059E+00_JPRB, 0.68881E+00_JPRB, 0.91932E+00_JPRB, 0.95152E+00_JPRB, 0.73135E+00_JPRB, 0.63415E+00_JPRB, 0.56304E+00_JPRB ,&
&  0.64386E+00_JPRB, 0.80294E+00_JPRB, 0.93221E+00_JPRB, 0.10787E+01_JPRB, 0.10909E+01_JPRB, 0.11593E+01_JPRB, 0.12219E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.12603E+01_JPRB, 0.13101E+01_JPRB, 0.13718E+01_JPRB, 0.14201E+01_JPRB, 0.14800E+01_JPRB, 0.15336E+01_JPRB ,&
&  0.15959E+01_JPRB, 0.15991E+01_JPRB, 0.16335E+01_JPRB, 0.13662E+01_JPRB, 0.14452E+01_JPRB, 0.15570E+01_JPRB, 0.19434E+01_JPRB ,&
&  0.18069E+01_JPRB, 0.11484E+01_JPRB, 0.94328E+00_JPRB, 0.10305E+01_JPRB, 0.98061E+00_JPRB, 0.93163E+00_JPRB, 0.82732E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.65796E+00_JPRB, 0.61855E+00_JPRB, 0.60645E+00_JPRB, 0.61639E+00_JPRB, 0.65981E+00_JPRB, 0.69597E+00_JPRB ,&
&  0.68256E+00_JPRB, 0.66478E+00_JPRB, 0.65274E+00_JPRB, 0.64932E+00_JPRB, 0.67864E+00_JPRB, 0.71133E+00_JPRB, 0.72557E+00_JPRB ,&
&  0.72796E+00_JPRB, 0.73297E+00_JPRB, 0.75293E+00_JPRB, 0.77972E+00_JPRB, 0.80720E+00_JPRB, 0.82391E+00_JPRB, 0.83630E+00_JPRB ,&
&  0.74081E+00_JPRB, 0.69194E+00_JPRB, 0.68852E+00_JPRB, 0.68986E+00_JPRB, 0.72298E+00_JPRB, 0.76070E+00_JPRB, 0.71633E+00_JPRB ,&
&  0.68861E+00_JPRB, 0.67608E+00_JPRB, 0.64659E+00_JPRB, 0.58358E+00_JPRB, 0.52481E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.52786E+00_JPRB, 0.54207E+00_JPRB, 0.56125E+00_JPRB, 0.58456E+00_JPRB, 0.58111E+00_JPRB, 0.58129E+00_JPRB, 0.56845E+00_JPRB ,&
&  0.56586E+00_JPRB, 0.58412E+00_JPRB, 0.62967E+00_JPRB, 0.70577E+00_JPRB, 0.76265E+00_JPRB, 0.84180E+00_JPRB, 0.94681E+00_JPRB ,&
&  0.97773E+00_JPRB, 0.95733E+00_JPRB, 0.94473E+00_JPRB, 0.94672E+00_JPRB, 0.96567E+00_JPRB, 0.99412E+00_JPRB, 0.10192E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.11042E+01_JPRB, 0.11530E+01_JPRB, 0.12217E+01_JPRB, 0.12902E+01_JPRB, 0.13278E+01_JPRB, 0.13996E+01_JPRB ,&
&  0.76146E+00_JPRB, 0.97654E+00_JPRB, 0.94121E+00_JPRB, 0.90891E+00_JPRB, 0.77391E+00_JPRB, 0.68078E+00_JPRB, 0.66396E+00_JPRB ,&
&  0.72683E+00_JPRB, 0.79849E+00_JPRB, 0.93948E+00_JPRB, 0.95010E+00_JPRB, 0.10352E+01_JPRB, 0.10859E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.12581E+01_JPRB, 0.13007E+01_JPRB, 0.13487E+01_JPRB, 0.13926E+01_JPRB, 0.14623E+01_JPRB, 0.15301E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.16034E+01_JPRB, 0.16271E+01_JPRB, 0.14432E+01_JPRB, 0.12848E+01_JPRB, 0.13236E+01_JPRB, 0.13127E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.85273E+00_JPRB, 0.84262E+00_JPRB, 0.95839E+00_JPRB, 0.93348E+00_JPRB, 0.88623E+00_JPRB, 0.80939E+00_JPRB ,&
&  0.72195E+00_JPRB, 0.63785E+00_JPRB, 0.59613E+00_JPRB, 0.55997E+00_JPRB, 0.55176E+00_JPRB, 0.58486E+00_JPRB, 0.62492E+00_JPRB ,&
&  0.61912E+00_JPRB, 0.60245E+00_JPRB, 0.59326E+00_JPRB, 0.60582E+00_JPRB, 0.63773E+00_JPRB, 0.66402E+00_JPRB, 0.68196E+00_JPRB ,&
&  0.65461E+00_JPRB, 0.62206E+00_JPRB, 0.61321E+00_JPRB, 0.62573E+00_JPRB, 0.65357E+00_JPRB, 0.68321E+00_JPRB, 0.71855E+00_JPRB ,&
&  0.73828E+00_JPRB, 0.74242E+00_JPRB, 0.83601E+00_JPRB, 0.10301E+01_JPRB, 0.11629E+01_JPRB, 0.12501E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.82336E+00_JPRB, 0.69137E+00_JPRB, 0.61079E+00_JPRB, 0.55747E+00_JPRB, 0.51670E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.52522E+00_JPRB, 0.57166E+00_JPRB, 0.60911E+00_JPRB, 0.66312E+00_JPRB, 0.68150E+00_JPRB, 0.68545E+00_JPRB, 0.68368E+00_JPRB ,&
&  0.67265E+00_JPRB, 0.67975E+00_JPRB, 0.70244E+00_JPRB, 0.75621E+00_JPRB, 0.81344E+00_JPRB, 0.86928E+00_JPRB, 0.92832E+00_JPRB ,&
&  0.10026E+01_JPRB, 0.10475E+01_JPRB, 0.10327E+01_JPRB, 0.10368E+01_JPRB, 0.10635E+01_JPRB, 0.10945E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.11011E+01_JPRB, 0.11183E+01_JPRB, 0.11413E+01_JPRB, 0.11901E+01_JPRB, 0.12552E+01_JPRB, 0.13149E+01_JPRB, 0.92851E+00_JPRB ,&
&  0.10243E+01_JPRB, 0.93670E+00_JPRB, 0.90215E+00_JPRB, 0.90356E+00_JPRB, 0.85858E+00_JPRB, 0.79060E+00_JPRB, 0.75700E+00_JPRB ,&
&  0.75996E+00_JPRB, 0.78329E+00_JPRB, 0.82375E+00_JPRB, 0.85130E+00_JPRB, 0.92563E+00_JPRB, 0.95402E+00_JPRB, 0.10509E+01_JPRB ,&
&  0.11325E+01_JPRB, 0.11617E+01_JPRB, 0.11759E+01_JPRB, 0.11907E+01_JPRB, 0.12095E+01_JPRB, 0.12474E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.13166E+01_JPRB, 0.13213E+01_JPRB, 0.13076E+01_JPRB, 0.12825E+01_JPRB, 0.11552E+01_JPRB, 0.11843E+01_JPRB, 0.96502E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.57206E+00_JPRB, 0.74361E+00_JPRB, 0.94537E+00_JPRB, 0.90490E+00_JPRB, 0.85553E+00_JPRB, 0.77657E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.63185E+00_JPRB, 0.60078E+00_JPRB, 0.57148E+00_JPRB, 0.54629E+00_JPRB, 0.55377E+00_JPRB, 0.56974E+00_JPRB ,&
&  0.57358E+00_JPRB, 0.56669E+00_JPRB, 0.55875E+00_JPRB, 0.56884E+00_JPRB, 0.59650E+00_JPRB, 0.70041E+00_JPRB, 0.89701E+00_JPRB ,&
&  0.10783E+01_JPRB, 0.53636E+00_JPRB, 0.49527E+00_JPRB, 0.51655E+00_JPRB, 0.56576E+00_JPRB, 0.61503E+00_JPRB, 0.67747E+00_JPRB ,&
&  0.83273E+00_JPRB, 0.10814E+01_JPRB, 0.10224E+01_JPRB, 0.12210E+01_JPRB, 0.25075E+01_JPRB, 0.27413E+01_JPRB, 0.15790E+01_JPRB ,&
&  0.99182E+00_JPRB, 0.70105E+00_JPRB, 0.56843E+00_JPRB, 0.52663E+00_JPRB, 0.51630E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.55423E+00_JPRB, 0.61400E+00_JPRB, 0.68216E+00_JPRB, 0.72608E+00_JPRB, 0.70227E+00_JPRB, 0.69827E+00_JPRB, 0.71266E+00_JPRB ,&
&  0.73500E+00_JPRB, 0.76655E+00_JPRB, 0.80034E+00_JPRB, 0.84463E+00_JPRB, 0.87599E+00_JPRB, 0.91595E+00_JPRB, 0.96925E+00_JPRB ,&
&  0.10305E+01_JPRB, 0.10415E+01_JPRB, 0.10618E+01_JPRB, 0.10844E+01_JPRB, 0.11028E+01_JPRB, 0.11240E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.11638E+01_JPRB, 0.11706E+01_JPRB, 0.12105E+01_JPRB, 0.12330E+01_JPRB, 0.13714E+01_JPRB, 0.96317E+00_JPRB ,&
&  0.10691E+01_JPRB, 0.98001E+00_JPRB, 0.90056E+00_JPRB, 0.90093E+00_JPRB, 0.88279E+00_JPRB, 0.85954E+00_JPRB, 0.83459E+00_JPRB ,&
&  0.83673E+00_JPRB, 0.82472E+00_JPRB, 0.78697E+00_JPRB, 0.81680E+00_JPRB, 0.81510E+00_JPRB, 0.82910E+00_JPRB, 0.88369E+00_JPRB ,&
&  0.93588E+00_JPRB, 0.97350E+00_JPRB, 0.97895E+00_JPRB, 0.99654E+00_JPRB, 0.99792E+00_JPRB, 0.98553E+00_JPRB, 0.96958E+00_JPRB ,&
&  0.94712E+00_JPRB, 0.94034E+00_JPRB, 0.93378E+00_JPRB, 0.92914E+00_JPRB, 0.93435E+00_JPRB, 0.83225E+00_JPRB, 0.68250E+00_JPRB ,&
&  0.52283E+00_JPRB, 0.52011E+00_JPRB, 0.79745E+00_JPRB, 0.10243E+01_JPRB, 0.92567E+00_JPRB, 0.87805E+00_JPRB, 0.77616E+00_JPRB ,&
&  0.71190E+00_JPRB, 0.64206E+00_JPRB, 0.61267E+00_JPRB, 0.56879E+00_JPRB, 0.54873E+00_JPRB, 0.55177E+00_JPRB, 0.57824E+00_JPRB ,&
&  0.58800E+00_JPRB, 0.57025E+00_JPRB, 0.55001E+00_JPRB, 0.51939E+00_JPRB, 0.52354E+00_JPRB, 0.55646E+00_JPRB, 0.52766E+00_JPRB ,&
&  0.61041E+00_JPRB, 0.48798E+00_JPRB, 0.49017E+00_JPRB, 0.55229E+00_JPRB, 0.51424E+00_JPRB, 0.56124E+00_JPRB, 0.62633E+00_JPRB ,&
&  0.81078E+00_JPRB, 0.88986E+00_JPRB, 0.12273E+01_JPRB, 0.34283E+01_JPRB, 0.39505E+01_JPRB, 0.49504E+01_JPRB, 0.19745E+01_JPRB ,&
&  0.91924E+00_JPRB, 0.61196E+00_JPRB, 0.54473E+00_JPRB, 0.54437E+00_JPRB, 0.53676E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.57366E+00_JPRB, 0.65178E+00_JPRB, 0.71920E+00_JPRB, 0.64833E+00_JPRB, 0.65580E+00_JPRB, 0.67453E+00_JPRB, 0.70328E+00_JPRB ,&
&  0.73630E+00_JPRB, 0.77626E+00_JPRB, 0.81848E+00_JPRB, 0.86838E+00_JPRB, 0.90803E+00_JPRB, 0.94474E+00_JPRB, 0.98478E+00_JPRB ,&
&  0.10267E+01_JPRB, 0.10410E+01_JPRB, 0.10746E+01_JPRB, 0.10803E+01_JPRB, 0.11044E+01_JPRB, 0.11052E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11898E+01_JPRB, 0.12165E+01_JPRB, 0.12369E+01_JPRB, 0.12270E+01_JPRB, 0.12640E+01_JPRB, 0.93069E+00_JPRB ,&
&  0.11392E+01_JPRB, 0.10287E+01_JPRB, 0.94240E+00_JPRB, 0.92608E+00_JPRB, 0.89197E+00_JPRB, 0.88291E+00_JPRB, 0.87097E+00_JPRB ,&
&  0.85814E+00_JPRB, 0.85400E+00_JPRB, 0.84445E+00_JPRB, 0.80812E+00_JPRB, 0.77320E+00_JPRB, 0.79140E+00_JPRB, 0.81116E+00_JPRB ,&
&  0.84357E+00_JPRB, 0.88352E+00_JPRB, 0.91368E+00_JPRB, 0.93435E+00_JPRB, 0.94652E+00_JPRB, 0.93860E+00_JPRB, 0.92898E+00_JPRB ,&
&  0.91815E+00_JPRB, 0.92409E+00_JPRB, 0.86184E+00_JPRB, 0.86106E+00_JPRB, 0.90603E+00_JPRB, 0.79078E+00_JPRB, 0.63283E+00_JPRB ,&
&  0.52599E+00_JPRB, 0.69141E+00_JPRB, 0.93763E+00_JPRB, 0.11309E+01_JPRB, 0.10034E+01_JPRB, 0.92175E+00_JPRB, 0.85283E+00_JPRB ,&
&  0.76538E+00_JPRB, 0.69774E+00_JPRB, 0.66653E+00_JPRB, 0.61294E+00_JPRB, 0.59411E+00_JPRB, 0.58807E+00_JPRB, 0.57974E+00_JPRB ,&
&  0.55429E+00_JPRB, 0.52249E+00_JPRB, 0.48612E+00_JPRB, 0.45942E+00_JPRB, 0.41751E+00_JPRB, 0.37665E+00_JPRB, 0.55342E+00_JPRB ,&
&  0.47156E+00_JPRB, 0.38842E+00_JPRB, 0.58327E+00_JPRB, 0.41809E+00_JPRB, 0.55008E+00_JPRB, 0.60904E+00_JPRB, 0.65532E+00_JPRB ,&
&  0.61408E+00_JPRB, 0.77572E+00_JPRB, 0.11278E+01_JPRB, 0.11572E+01_JPRB, 0.12159E+01_JPRB, 0.12897E+01_JPRB, 0.76698E+00_JPRB ,&
&  0.61733E+00_JPRB, 0.60512E+00_JPRB, 0.60640E+00_JPRB, 0.55937E+00_JPRB, 0.55438E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.58549E+00_JPRB, 0.63613E+00_JPRB, 0.72588E+00_JPRB, 0.65610E+00_JPRB, 0.65385E+00_JPRB, 0.66844E+00_JPRB, 0.69090E+00_JPRB ,&
&  0.71429E+00_JPRB, 0.74529E+00_JPRB, 0.77864E+00_JPRB, 0.82461E+00_JPRB, 0.86501E+00_JPRB, 0.90096E+00_JPRB, 0.93503E+00_JPRB ,&
&  0.96622E+00_JPRB, 0.99296E+00_JPRB, 0.10186E+01_JPRB, 0.10305E+01_JPRB, 0.10431E+01_JPRB, 0.10652E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11697E+01_JPRB, 0.12002E+01_JPRB, 0.12508E+01_JPRB, 0.12852E+01_JPRB, 0.13110E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.10670E+01_JPRB, 0.99321E+00_JPRB, 0.94586E+00_JPRB, 0.90463E+00_JPRB, 0.87895E+00_JPRB, 0.87579E+00_JPRB ,&
&  0.89972E+00_JPRB, 0.86082E+00_JPRB, 0.86124E+00_JPRB, 0.83644E+00_JPRB, 0.83275E+00_JPRB, 0.86670E+00_JPRB, 0.88693E+00_JPRB ,&
&  0.94321E+00_JPRB, 0.10168E+01_JPRB, 0.10846E+01_JPRB, 0.11304E+01_JPRB, 0.11762E+01_JPRB, 0.11723E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.12420E+01_JPRB, 0.11932E+01_JPRB, 0.11979E+01_JPRB, 0.11792E+01_JPRB, 0.10900E+01_JPRB, 0.89019E+00_JPRB ,&
&  0.76741E+00_JPRB, 0.88533E+00_JPRB, 0.94931E+00_JPRB, 0.11311E+01_JPRB, 0.10576E+01_JPRB, 0.96253E+00_JPRB, 0.92230E+00_JPRB ,&
&  0.85758E+00_JPRB, 0.80583E+00_JPRB, 0.77449E+00_JPRB, 0.71781E+00_JPRB, 0.73368E+00_JPRB, 0.74372E+00_JPRB, 0.73927E+00_JPRB ,&
&  0.68143E+00_JPRB, 0.60130E+00_JPRB, 0.51598E+00_JPRB, 0.45412E+00_JPRB, 0.45206E+00_JPRB, 0.45048E+00_JPRB, 0.52910E+00_JPRB ,&
&  0.52366E+00_JPRB, 0.37069E+00_JPRB, 0.50544E+00_JPRB, 0.65620E+00_JPRB, 0.68689E+00_JPRB, 0.73154E+00_JPRB, 0.79132E+00_JPRB ,&
&  0.86231E+00_JPRB, 0.85742E+00_JPRB, 0.82415E+00_JPRB, 0.81756E+00_JPRB, 0.80727E+00_JPRB, 0.80314E+00_JPRB, 0.72142E+00_JPRB ,&
&  0.69175E+00_JPRB, 0.63499E+00_JPRB, 0.60298E+00_JPRB, 0.58875E+00_JPRB, 0.56096E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.64300E+00_JPRB, 0.65009E+00_JPRB, 0.71223E+00_JPRB, 0.67183E+00_JPRB, 0.67027E+00_JPRB, 0.68465E+00_JPRB, 0.70161E+00_JPRB ,&
&  0.70299E+00_JPRB, 0.71384E+00_JPRB, 0.73527E+00_JPRB, 0.77106E+00_JPRB, 0.80583E+00_JPRB, 0.83280E+00_JPRB, 0.86253E+00_JPRB ,&
&  0.88583E+00_JPRB, 0.90052E+00_JPRB, 0.90943E+00_JPRB, 0.91940E+00_JPRB, 0.94820E+00_JPRB, 0.98089E+00_JPRB, 0.10277E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.11000E+01_JPRB, 0.11000E+01_JPRB, 0.11314E+01_JPRB, 0.11637E+01_JPRB, 0.12649E+01_JPRB, 0.14288E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.11109E+01_JPRB, 0.10325E+01_JPRB, 0.94475E+00_JPRB, 0.94795E+00_JPRB, 0.91326E+00_JPRB, 0.92584E+00_JPRB ,&
&  0.95719E+00_JPRB, 0.95092E+00_JPRB, 0.97208E+00_JPRB, 0.98704E+00_JPRB, 0.10000E+01_JPRB, 0.10302E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.11177E+01_JPRB, 0.11930E+01_JPRB, 0.12578E+01_JPRB, 0.13547E+01_JPRB, 0.14478E+01_JPRB, 0.15338E+01_JPRB, 0.15840E+01_JPRB ,&
&  0.16118E+01_JPRB, 0.16135E+01_JPRB, 0.14920E+01_JPRB, 0.14804E+01_JPRB, 0.14715E+01_JPRB, 0.14071E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.10719E+01_JPRB, 0.10901E+01_JPRB, 0.10542E+01_JPRB, 0.10546E+01_JPRB, 0.10457E+01_JPRB, 0.10018E+01_JPRB ,&
&  0.98391E+00_JPRB, 0.93408E+00_JPRB, 0.89794E+00_JPRB, 0.86734E+00_JPRB, 0.87774E+00_JPRB, 0.90565E+00_JPRB, 0.90303E+00_JPRB ,&
&  0.85114E+00_JPRB, 0.77484E+00_JPRB, 0.68348E+00_JPRB, 0.59004E+00_JPRB, 0.51243E+00_JPRB, 0.60703E+00_JPRB, 0.64245E+00_JPRB ,&
&  0.65373E+00_JPRB, 0.58070E+00_JPRB, 0.59311E+00_JPRB, 0.80874E+00_JPRB, 0.84218E+00_JPRB, 0.83720E+00_JPRB, 0.86679E+00_JPRB ,&
&  0.87570E+00_JPRB, 0.86706E+00_JPRB, 0.89039E+00_JPRB, 0.84604E+00_JPRB, 0.81041E+00_JPRB, 0.79019E+00_JPRB, 0.78034E+00_JPRB ,&
&  0.72705E+00_JPRB, 0.67514E+00_JPRB, 0.62640E+00_JPRB, 0.62942E+00_JPRB, 0.59850E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.65045E+00_JPRB, 0.63969E+00_JPRB, 0.65166E+00_JPRB, 0.68955E+00_JPRB, 0.66708E+00_JPRB, 0.68541E+00_JPRB, 0.68007E+00_JPRB ,&
&  0.67947E+00_JPRB, 0.67485E+00_JPRB, 0.68726E+00_JPRB, 0.70143E+00_JPRB, 0.72300E+00_JPRB, 0.74303E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.76137E+00_JPRB, 0.77396E+00_JPRB, 0.79477E+00_JPRB, 0.83025E+00_JPRB, 0.89240E+00_JPRB, 0.99232E+00_JPRB, 0.11853E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.14822E+01_JPRB, 0.15175E+01_JPRB, 0.15213E+01_JPRB, 0.16891E+01_JPRB, 0.21079E+01_JPRB, 0.23810E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.11742E+01_JPRB, 0.10468E+01_JPRB, 0.93163E+00_JPRB, 0.94377E+00_JPRB, 0.10114E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.10505E+01_JPRB, 0.10999E+01_JPRB, 0.11590E+01_JPRB, 0.12028E+01_JPRB, 0.12359E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.14010E+01_JPRB, 0.14861E+01_JPRB, 0.15822E+01_JPRB, 0.15680E+01_JPRB, 0.17081E+01_JPRB, 0.17390E+01_JPRB ,&
&  0.17534E+01_JPRB, 0.16529E+01_JPRB, 0.14552E+01_JPRB, 0.14579E+01_JPRB, 0.15400E+01_JPRB, 0.15294E+01_JPRB, 0.14221E+01_JPRB ,&
&  0.14285E+01_JPRB, 0.12205E+01_JPRB, 0.97167E+00_JPRB, 0.98665E+00_JPRB, 0.10273E+01_JPRB, 0.11073E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.10755E+01_JPRB, 0.10492E+01_JPRB, 0.10351E+01_JPRB, 0.10346E+01_JPRB, 0.10607E+01_JPRB, 0.10686E+01_JPRB ,&
&  0.98563E+00_JPRB, 0.94725E+00_JPRB, 0.87750E+00_JPRB, 0.82554E+00_JPRB, 0.80204E+00_JPRB, 0.69537E+00_JPRB, 0.81858E+00_JPRB ,&
&  0.81322E+00_JPRB, 0.82751E+00_JPRB, 0.74779E+00_JPRB, 0.94556E+00_JPRB, 0.90211E+00_JPRB, 0.87155E+00_JPRB, 0.87935E+00_JPRB ,&
&  0.90271E+00_JPRB, 0.86075E+00_JPRB, 0.83833E+00_JPRB, 0.81901E+00_JPRB, 0.80864E+00_JPRB, 0.78583E+00_JPRB, 0.73777E+00_JPRB ,&
&  0.70252E+00_JPRB, 0.67945E+00_JPRB, 0.66778E+00_JPRB, 0.65712E+00_JPRB, 0.65515E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.64937E+00_JPRB, 0.65426E+00_JPRB, 0.65291E+00_JPRB, 0.63973E+00_JPRB, 0.64229E+00_JPRB, 0.64724E+00_JPRB, 0.63444E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.62042E+00_JPRB, 0.63440E+00_JPRB, 0.63483E+00_JPRB, 0.63730E+00_JPRB, 0.65050E+00_JPRB, 0.66816E+00_JPRB ,&
&  0.68765E+00_JPRB, 0.70476E+00_JPRB, 0.77246E+00_JPRB, 0.91027E+00_JPRB, 0.11633E+01_JPRB, 0.14915E+01_JPRB, 0.17284E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.19755E+01_JPRB, 0.20680E+01_JPRB, 0.21514E+01_JPRB, 0.20049E+01_JPRB, 0.20408E+01_JPRB, 0.22004E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.11627E+01_JPRB, 0.11652E+01_JPRB, 0.10393E+01_JPRB, 0.10236E+01_JPRB, 0.10410E+01_JPRB, 0.10500E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.11020E+01_JPRB, 0.11537E+01_JPRB, 0.12308E+01_JPRB, 0.13017E+01_JPRB, 0.13461E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.14241E+01_JPRB, 0.14812E+01_JPRB, 0.14638E+01_JPRB, 0.14440E+01_JPRB, 0.14937E+01_JPRB, 0.15108E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.14913E+01_JPRB, 0.13260E+01_JPRB, 0.13227E+01_JPRB, 0.14331E+01_JPRB, 0.13569E+01_JPRB, 0.13352E+01_JPRB ,&
&  0.14079E+01_JPRB, 0.12613E+01_JPRB, 0.88785E+00_JPRB, 0.87717E+00_JPRB, 0.96128E+00_JPRB, 0.10792E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.12132E+01_JPRB, 0.11870E+01_JPRB, 0.11953E+01_JPRB, 0.11827E+01_JPRB, 0.11828E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.10687E+01_JPRB, 0.10396E+01_JPRB, 0.10078E+01_JPRB, 0.98669E+00_JPRB, 0.99981E+00_JPRB, 0.98208E+00_JPRB, 0.94814E+00_JPRB ,&
&  0.92148E+00_JPRB, 0.91008E+00_JPRB, 0.94249E+00_JPRB, 0.90824E+00_JPRB, 0.84916E+00_JPRB, 0.88570E+00_JPRB, 0.86852E+00_JPRB ,&
&  0.88254E+00_JPRB, 0.85806E+00_JPRB, 0.83382E+00_JPRB, 0.81044E+00_JPRB, 0.79216E+00_JPRB, 0.76863E+00_JPRB, 0.73798E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.68663E+00_JPRB, 0.67282E+00_JPRB, 0.65884E+00_JPRB, 0.64501E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.64745E+00_JPRB, 0.65049E+00_JPRB, 0.65467E+00_JPRB, 0.63919E+00_JPRB, 0.61342E+00_JPRB, 0.60106E+00_JPRB, 0.59398E+00_JPRB ,&
&  0.59755E+00_JPRB, 0.61522E+00_JPRB, 0.62554E+00_JPRB, 0.62466E+00_JPRB, 0.62315E+00_JPRB, 0.63754E+00_JPRB, 0.67932E+00_JPRB ,&
&  0.73482E+00_JPRB, 0.80756E+00_JPRB, 0.94106E+00_JPRB, 0.12135E+01_JPRB, 0.15479E+01_JPRB, 0.18092E+01_JPRB, 0.17527E+01_JPRB ,&
&  0.16263E+01_JPRB, 0.16205E+01_JPRB, 0.16098E+01_JPRB, 0.16193E+01_JPRB, 0.16352E+01_JPRB, 0.16908E+01_JPRB, 0.17843E+01_JPRB ,&
&  0.16206E+01_JPRB, 0.12971E+01_JPRB, 0.11482E+01_JPRB, 0.10611E+01_JPRB, 0.10370E+01_JPRB, 0.10283E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.10865E+01_JPRB, 0.11340E+01_JPRB, 0.12100E+01_JPRB, 0.12940E+01_JPRB, 0.13444E+01_JPRB, 0.13618E+01_JPRB ,&
&  0.13641E+01_JPRB, 0.13607E+01_JPRB, 0.13817E+01_JPRB, 0.12741E+01_JPRB, 0.13109E+01_JPRB, 0.13385E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.12641E+01_JPRB, 0.13242E+01_JPRB, 0.13295E+01_JPRB, 0.13324E+01_JPRB, 0.13097E+01_JPRB, 0.12933E+01_JPRB ,&
&  0.13231E+01_JPRB, 0.12407E+01_JPRB, 0.93163E+00_JPRB, 0.94611E+00_JPRB, 0.10143E+01_JPRB, 0.97780E+00_JPRB, 0.12057E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.12437E+01_JPRB, 0.12700E+01_JPRB, 0.13130E+01_JPRB, 0.13696E+01_JPRB, 0.14225E+01_JPRB, 0.12735E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.11325E+01_JPRB, 0.11235E+01_JPRB, 0.11206E+01_JPRB, 0.11568E+01_JPRB, 0.11539E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.98352E+00_JPRB, 0.98103E+00_JPRB, 0.98014E+00_JPRB, 0.91307E+00_JPRB, 0.86217E+00_JPRB, 0.85398E+00_JPRB ,&
&  0.84320E+00_JPRB, 0.80899E+00_JPRB, 0.79909E+00_JPRB, 0.78590E+00_JPRB, 0.77604E+00_JPRB, 0.76499E+00_JPRB, 0.75270E+00_JPRB ,&
&  0.73211E+00_JPRB, 0.71237E+00_JPRB, 0.69188E+00_JPRB, 0.66964E+00_JPRB, 0.65254E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.66017E+00_JPRB, 0.65930E+00_JPRB, 0.66318E+00_JPRB, 0.66165E+00_JPRB, 0.63589E+00_JPRB, 0.62070E+00_JPRB, 0.61780E+00_JPRB ,&
&  0.62905E+00_JPRB, 0.64618E+00_JPRB, 0.65940E+00_JPRB, 0.66960E+00_JPRB, 0.69709E+00_JPRB, 0.71416E+00_JPRB, 0.74401E+00_JPRB ,&
&  0.80023E+00_JPRB, 0.89178E+00_JPRB, 0.10650E+01_JPRB, 0.13386E+01_JPRB, 0.15893E+01_JPRB, 0.18147E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.15637E+01_JPRB, 0.14994E+01_JPRB, 0.14651E+01_JPRB, 0.14120E+01_JPRB, 0.12490E+01_JPRB, 0.13136E+01_JPRB, 0.14038E+01_JPRB ,&
&  0.14787E+01_JPRB, 0.13385E+01_JPRB, 0.11067E+01_JPRB, 0.10361E+01_JPRB, 0.99022E+00_JPRB, 0.98793E+00_JPRB, 0.10054E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.10548E+01_JPRB, 0.10980E+01_JPRB, 0.11541E+01_JPRB, 0.12214E+01_JPRB, 0.12868E+01_JPRB, 0.13175E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.12882E+01_JPRB, 0.12730E+01_JPRB, 0.12228E+01_JPRB, 0.12075E+01_JPRB, 0.12209E+01_JPRB, 0.12374E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.12176E+01_JPRB, 0.12497E+01_JPRB, 0.12928E+01_JPRB, 0.12477E+01_JPRB, 0.11778E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.12237E+01_JPRB, 0.13253E+01_JPRB, 0.10070E+01_JPRB, 0.11659E+01_JPRB, 0.13360E+01_JPRB, 0.11837E+01_JPRB, 0.11728E+01_JPRB ,&
&  0.12307E+01_JPRB, 0.12766E+01_JPRB, 0.13455E+01_JPRB, 0.14225E+01_JPRB, 0.15059E+01_JPRB, 0.16001E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.13228E+01_JPRB, 0.12339E+01_JPRB, 0.12245E+01_JPRB, 0.12416E+01_JPRB, 0.12783E+01_JPRB, 0.12792E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.10759E+01_JPRB, 0.11270E+01_JPRB, 0.12377E+01_JPRB, 0.13949E+01_JPRB, 0.14174E+01_JPRB, 0.85886E+00_JPRB, 0.86950E+00_JPRB ,&
&  0.85050E+00_JPRB, 0.82090E+00_JPRB, 0.80811E+00_JPRB, 0.79743E+00_JPRB, 0.79087E+00_JPRB, 0.78164E+00_JPRB, 0.77484E+00_JPRB ,&
&  0.76074E+00_JPRB, 0.73951E+00_JPRB, 0.71669E+00_JPRB, 0.69424E+00_JPRB, 0.67439E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.68534E+00_JPRB, 0.67723E+00_JPRB, 0.67740E+00_JPRB, 0.67419E+00_JPRB, 0.67104E+00_JPRB, 0.65779E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.65346E+00_JPRB, 0.66502E+00_JPRB, 0.67299E+00_JPRB, 0.69973E+00_JPRB, 0.73323E+00_JPRB, 0.79372E+00_JPRB, 0.85351E+00_JPRB ,&
&  0.92382E+00_JPRB, 0.10174E+01_JPRB, 0.11939E+01_JPRB, 0.14159E+01_JPRB, 0.15869E+01_JPRB, 0.16765E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.14900E+01_JPRB, 0.14840E+01_JPRB, 0.14408E+01_JPRB, 0.11323E+01_JPRB, 0.10331E+01_JPRB, 0.12344E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.10904E+01_JPRB, 0.10113E+01_JPRB, 0.95831E+00_JPRB, 0.91976E+00_JPRB, 0.92667E+00_JPRB, 0.94695E+00_JPRB ,&
&  0.98119E+00_JPRB, 0.10150E+01_JPRB, 0.10567E+01_JPRB, 0.11023E+01_JPRB, 0.11507E+01_JPRB, 0.12230E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.12702E+01_JPRB, 0.12568E+01_JPRB, 0.12394E+01_JPRB, 0.12158E+01_JPRB, 0.11652E+01_JPRB, 0.11580E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.11867E+01_JPRB, 0.11993E+01_JPRB, 0.12379E+01_JPRB, 0.12530E+01_JPRB, 0.11340E+01_JPRB, 0.11189E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.12836E+01_JPRB, 0.11858E+01_JPRB, 0.13718E+01_JPRB, 0.91192E+00_JPRB, 0.10077E+01_JPRB, 0.11705E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.12405E+01_JPRB, 0.13049E+01_JPRB, 0.13891E+01_JPRB, 0.14566E+01_JPRB, 0.13708E+01_JPRB, 0.14505E+01_JPRB ,&
&  0.14357E+01_JPRB, 0.13438E+01_JPRB, 0.13501E+01_JPRB, 0.13522E+01_JPRB, 0.13983E+01_JPRB, 0.14201E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.12107E+01_JPRB, 0.12832E+01_JPRB, 0.13263E+01_JPRB, 0.11851E+01_JPRB, 0.91487E+00_JPRB, 0.88773E+00_JPRB ,&
&  0.87812E+00_JPRB, 0.84174E+00_JPRB, 0.82234E+00_JPRB, 0.81022E+00_JPRB, 0.80355E+00_JPRB, 0.79182E+00_JPRB, 0.78157E+00_JPRB ,&
&  0.77106E+00_JPRB, 0.75256E+00_JPRB, 0.73591E+00_JPRB, 0.71773E+00_JPRB, 0.69969E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.70125E+00_JPRB, 0.69826E+00_JPRB, 0.70350E+00_JPRB, 0.68898E+00_JPRB, 0.68798E+00_JPRB, 0.69620E+00_JPRB, 0.68191E+00_JPRB ,&
&  0.65596E+00_JPRB, 0.62787E+00_JPRB, 0.63483E+00_JPRB, 0.66510E+00_JPRB, 0.70556E+00_JPRB, 0.77272E+00_JPRB, 0.86143E+00_JPRB ,&
&  0.97392E+00_JPRB, 0.10739E+01_JPRB, 0.12316E+01_JPRB, 0.13750E+01_JPRB, 0.14790E+01_JPRB, 0.15482E+01_JPRB, 0.15401E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.10075E+01_JPRB, 0.13828E+01_JPRB, 0.14605E+01_JPRB, 0.13135E+01_JPRB, 0.12378E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10537E+01_JPRB, 0.10299E+01_JPRB, 0.95580E+00_JPRB, 0.86409E+00_JPRB, 0.86443E+00_JPRB, 0.88822E+00_JPRB ,&
&  0.92783E+00_JPRB, 0.96459E+00_JPRB, 0.10111E+01_JPRB, 0.10682E+01_JPRB, 0.11147E+01_JPRB, 0.11759E+01_JPRB, 0.12230E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12400E+01_JPRB, 0.12381E+01_JPRB, 0.12025E+01_JPRB, 0.11785E+01_JPRB, 0.11620E+01_JPRB, 0.11424E+01_JPRB ,&
&  0.11500E+01_JPRB, 0.11467E+01_JPRB, 0.11851E+01_JPRB, 0.12164E+01_JPRB, 0.12683E+01_JPRB, 0.11770E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.12003E+01_JPRB, 0.12461E+01_JPRB, 0.12494E+01_JPRB, 0.13683E+01_JPRB, 0.97657E+00_JPRB, 0.10740E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.12307E+01_JPRB, 0.12526E+01_JPRB, 0.12839E+01_JPRB, 0.13247E+01_JPRB, 0.13896E+01_JPRB, 0.13165E+01_JPRB, 0.14254E+01_JPRB ,&
&  0.14567E+01_JPRB, 0.14734E+01_JPRB, 0.14568E+01_JPRB, 0.15134E+01_JPRB, 0.14580E+01_JPRB, 0.15705E+01_JPRB, 0.15344E+01_JPRB ,&
&  0.15253E+01_JPRB, 0.14068E+01_JPRB, 0.13563E+01_JPRB, 0.13332E+01_JPRB, 0.11983E+01_JPRB, 0.10338E+01_JPRB, 0.96107E+00_JPRB ,&
&  0.92285E+00_JPRB, 0.89167E+00_JPRB, 0.85490E+00_JPRB, 0.82852E+00_JPRB, 0.81341E+00_JPRB, 0.79919E+00_JPRB, 0.78435E+00_JPRB ,&
&  0.76672E+00_JPRB, 0.74926E+00_JPRB, 0.73446E+00_JPRB, 0.71741E+00_JPRB, 0.70698E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.69746E+00_JPRB, 0.70636E+00_JPRB, 0.72418E+00_JPRB, 0.74050E+00_JPRB, 0.83942E+00_JPRB, 0.96215E+00_JPRB, 0.10352E+01_JPRB ,&
&  0.98474E+00_JPRB, 0.61809E+00_JPRB, 0.60153E+00_JPRB, 0.61715E+00_JPRB, 0.64591E+00_JPRB, 0.70192E+00_JPRB, 0.77049E+00_JPRB ,&
&  0.86963E+00_JPRB, 0.98228E+00_JPRB, 0.11387E+01_JPRB, 0.13286E+01_JPRB, 0.14383E+01_JPRB, 0.15411E+01_JPRB, 0.14455E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.10631E+01_JPRB, 0.15458E+01_JPRB, 0.14944E+01_JPRB, 0.13952E+01_JPRB, 0.12955E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.10767E+01_JPRB, 0.97942E+00_JPRB, 0.96481E+00_JPRB, 0.92335E+00_JPRB, 0.82723E+00_JPRB, 0.79945E+00_JPRB, 0.80683E+00_JPRB ,&
&  0.84007E+00_JPRB, 0.88896E+00_JPRB, 0.95017E+00_JPRB, 0.10145E+01_JPRB, 0.10886E+01_JPRB, 0.11374E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.12316E+01_JPRB, 0.12091E+01_JPRB, 0.11719E+01_JPRB, 0.11887E+01_JPRB, 0.11972E+01_JPRB, 0.11875E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.12491E+01_JPRB, 0.12772E+01_JPRB, 0.12059E+01_JPRB, 0.11071E+01_JPRB, 0.12101E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.11818E+01_JPRB, 0.13416E+01_JPRB, 0.11016E+01_JPRB, 0.11044E+01_JPRB, 0.11929E+01_JPRB, 0.12063E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.12506E+01_JPRB, 0.12833E+01_JPRB, 0.13019E+01_JPRB, 0.13213E+01_JPRB, 0.12223E+01_JPRB, 0.13747E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.15276E+01_JPRB, 0.15145E+01_JPRB, 0.15802E+01_JPRB, 0.14510E+01_JPRB, 0.15212E+01_JPRB, 0.16599E+01_JPRB ,&
&  0.17649E+01_JPRB, 0.16705E+01_JPRB, 0.15543E+01_JPRB, 0.14132E+01_JPRB, 0.12527E+01_JPRB, 0.11611E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10106E+01_JPRB, 0.94221E+00_JPRB, 0.90252E+00_JPRB, 0.86011E+00_JPRB, 0.83387E+00_JPRB, 0.81017E+00_JPRB, 0.78667E+00_JPRB ,&
&  0.76545E+00_JPRB, 0.74909E+00_JPRB, 0.73325E+00_JPRB, 0.71127E+00_JPRB, 0.69728E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.68773E+00_JPRB, 0.68759E+00_JPRB, 0.69726E+00_JPRB, 0.71956E+00_JPRB, 0.81698E+00_JPRB, 0.85540E+00_JPRB, 0.85555E+00_JPRB ,&
&  0.83350E+00_JPRB, 0.59224E+00_JPRB, 0.56535E+00_JPRB, 0.57752E+00_JPRB, 0.60900E+00_JPRB, 0.64890E+00_JPRB, 0.71483E+00_JPRB ,&
&  0.77794E+00_JPRB, 0.87717E+00_JPRB, 0.10342E+01_JPRB, 0.12131E+01_JPRB, 0.13424E+01_JPRB, 0.13768E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.88910E+00_JPRB, 0.15786E+01_JPRB, 0.15877E+01_JPRB, 0.16077E+01_JPRB, 0.15778E+01_JPRB, 0.14642E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.96960E+00_JPRB, 0.91175E+00_JPRB, 0.87599E+00_JPRB, 0.82819E+00_JPRB, 0.76424E+00_JPRB, 0.74491E+00_JPRB ,&
&  0.76428E+00_JPRB, 0.81844E+00_JPRB, 0.88501E+00_JPRB, 0.95357E+00_JPRB, 0.10387E+01_JPRB, 0.11177E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.12226E+01_JPRB, 0.12295E+01_JPRB, 0.11817E+01_JPRB, 0.11848E+01_JPRB, 0.12016E+01_JPRB, 0.12430E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.11835E+01_JPRB, 0.13250E+01_JPRB, 0.13424E+01_JPRB, 0.12642E+01_JPRB, 0.12792E+01_JPRB, 0.11596E+01_JPRB ,&
&  0.12296E+01_JPRB, 0.11433E+01_JPRB, 0.12411E+01_JPRB, 0.10733E+01_JPRB, 0.11251E+01_JPRB, 0.12491E+01_JPRB, 0.12397E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.12159E+01_JPRB, 0.12080E+01_JPRB, 0.12425E+01_JPRB, 0.12751E+01_JPRB, 0.12512E+01_JPRB, 0.12307E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.13960E+01_JPRB, 0.15629E+01_JPRB, 0.14034E+01_JPRB, 0.14299E+01_JPRB, 0.15075E+01_JPRB, 0.18239E+01_JPRB ,&
&  0.20560E+01_JPRB, 0.19670E+01_JPRB, 0.19771E+01_JPRB, 0.17356E+01_JPRB, 0.14307E+01_JPRB, 0.12904E+01_JPRB, 0.12004E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.10272E+01_JPRB, 0.96443E+00_JPRB, 0.89338E+00_JPRB, 0.82947E+00_JPRB, 0.80064E+00_JPRB, 0.78605E+00_JPRB ,&
&  0.77065E+00_JPRB, 0.75768E+00_JPRB, 0.73953E+00_JPRB, 0.71394E+00_JPRB, 0.69518E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.69331E+00_JPRB, 0.69541E+00_JPRB, 0.70215E+00_JPRB, 0.71245E+00_JPRB, 0.76268E+00_JPRB, 0.77664E+00_JPRB, 0.71623E+00_JPRB ,&
&  0.64724E+00_JPRB, 0.54127E+00_JPRB, 0.52774E+00_JPRB, 0.53990E+00_JPRB, 0.56543E+00_JPRB, 0.59770E+00_JPRB, 0.65763E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.80673E+00_JPRB, 0.96363E+00_JPRB, 0.11071E+01_JPRB, 0.12440E+01_JPRB, 0.13752E+01_JPRB, 0.94198E+00_JPRB ,&
&  0.97618E+00_JPRB, 0.15503E+01_JPRB, 0.16060E+01_JPRB, 0.16292E+01_JPRB, 0.16398E+01_JPRB, 0.16306E+01_JPRB, 0.13110E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.10494E+01_JPRB, 0.93846E+00_JPRB, 0.84489E+00_JPRB, 0.81055E+00_JPRB, 0.78067E+00_JPRB, 0.72612E+00_JPRB ,&
&  0.71897E+00_JPRB, 0.75892E+00_JPRB, 0.82738E+00_JPRB, 0.90306E+00_JPRB, 0.99005E+00_JPRB, 0.10915E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.12199E+01_JPRB, 0.12361E+01_JPRB, 0.12243E+01_JPRB, 0.11545E+01_JPRB, 0.12062E+01_JPRB, 0.12521E+01_JPRB, 0.13008E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.12204E+01_JPRB, 0.14497E+01_JPRB, 0.16266E+01_JPRB, 0.16043E+01_JPRB, 0.14617E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.12095E+01_JPRB, 0.11113E+01_JPRB, 0.10355E+01_JPRB, 0.11402E+01_JPRB, 0.12632E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12047E+01_JPRB, 0.11578E+01_JPRB, 0.11221E+01_JPRB, 0.11505E+01_JPRB, 0.11374E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.12990E+01_JPRB, 0.14408E+01_JPRB, 0.15097E+01_JPRB, 0.13316E+01_JPRB, 0.13484E+01_JPRB, 0.13729E+01_JPRB, 0.16999E+01_JPRB ,&
&  0.22236E+01_JPRB, 0.22287E+01_JPRB, 0.21226E+01_JPRB, 0.20590E+01_JPRB, 0.16934E+01_JPRB, 0.14893E+01_JPRB, 0.13689E+01_JPRB ,&
&  0.12423E+01_JPRB, 0.11087E+01_JPRB, 0.10243E+01_JPRB, 0.96123E+00_JPRB, 0.88744E+00_JPRB, 0.83649E+00_JPRB, 0.81972E+00_JPRB ,&
&  0.79694E+00_JPRB, 0.78175E+00_JPRB, 0.75851E+00_JPRB, 0.72724E+00_JPRB, 0.70444E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.69756E+00_JPRB, 0.68852E+00_JPRB, 0.69540E+00_JPRB, 0.69714E+00_JPRB, 0.69296E+00_JPRB, 0.68182E+00_JPRB, 0.62143E+00_JPRB ,&
&  0.55588E+00_JPRB, 0.51471E+00_JPRB, 0.49779E+00_JPRB, 0.50043E+00_JPRB, 0.52349E+00_JPRB, 0.55916E+00_JPRB, 0.59580E+00_JPRB ,&
&  0.68251E+00_JPRB, 0.76301E+00_JPRB, 0.88473E+00_JPRB, 0.10108E+01_JPRB, 0.10191E+01_JPRB, 0.88589E+00_JPRB, 0.89761E+00_JPRB ,&
&  0.12728E+01_JPRB, 0.16767E+01_JPRB, 0.16380E+01_JPRB, 0.16338E+01_JPRB, 0.16956E+01_JPRB, 0.17791E+01_JPRB, 0.14691E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.11404E+01_JPRB, 0.99703E+00_JPRB, 0.90000E+00_JPRB, 0.82435E+00_JPRB, 0.80552E+00_JPRB, 0.76026E+00_JPRB ,&
&  0.71264E+00_JPRB, 0.73374E+00_JPRB, 0.81266E+00_JPRB, 0.89514E+00_JPRB, 0.98140E+00_JPRB, 0.10734E+01_JPRB, 0.11687E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.12243E+01_JPRB, 0.12083E+01_JPRB, 0.11616E+01_JPRB, 0.11921E+01_JPRB, 0.12484E+01_JPRB, 0.13814E+01_JPRB ,&
&  0.14530E+01_JPRB, 0.15897E+01_JPRB, 0.16842E+01_JPRB, 0.17887E+01_JPRB, 0.17713E+01_JPRB, 0.17183E+01_JPRB, 0.15635E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.12943E+01_JPRB, 0.10390E+01_JPRB, 0.10568E+01_JPRB, 0.12263E+01_JPRB, 0.13471E+01_JPRB, 0.12236E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.93780E+00_JPRB, 0.92496E+00_JPRB, 0.96988E+00_JPRB, 0.10699E+01_JPRB, 0.10816E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.82090E+00_JPRB, 0.73808E+00_JPRB, 0.90163E+00_JPRB, 0.84915E+00_JPRB, 0.10013E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.23764E+01_JPRB, 0.27678E+01_JPRB, 0.26674E+01_JPRB, 0.21841E+01_JPRB, 0.19915E+01_JPRB, 0.17707E+01_JPRB, 0.16457E+01_JPRB ,&
&  0.14133E+01_JPRB, 0.12351E+01_JPRB, 0.11152E+01_JPRB, 0.10377E+01_JPRB, 0.96998E+00_JPRB, 0.91388E+00_JPRB, 0.88360E+00_JPRB ,&
&  0.85735E+00_JPRB, 0.82484E+00_JPRB, 0.79399E+00_JPRB, 0.75269E+00_JPRB, 0.71681E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.69687E+00_JPRB, 0.67613E+00_JPRB, 0.67990E+00_JPRB, 0.67965E+00_JPRB, 0.65304E+00_JPRB, 0.61214E+00_JPRB, 0.56959E+00_JPRB ,&
&  0.53416E+00_JPRB, 0.50766E+00_JPRB, 0.48746E+00_JPRB, 0.48211E+00_JPRB, 0.49125E+00_JPRB, 0.52662E+00_JPRB, 0.57314E+00_JPRB ,&
&  0.64382E+00_JPRB, 0.75014E+00_JPRB, 0.85283E+00_JPRB, 0.95396E+00_JPRB, 0.88829E+00_JPRB, 0.75031E+00_JPRB, 0.84936E+00_JPRB ,&
&  0.10974E+01_JPRB, 0.15146E+01_JPRB, 0.17168E+01_JPRB, 0.17212E+01_JPRB, 0.17116E+01_JPRB, 0.17139E+01_JPRB, 0.15997E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.12141E+01_JPRB, 0.10584E+01_JPRB, 0.95323E+00_JPRB, 0.86024E+00_JPRB, 0.79810E+00_JPRB, 0.78415E+00_JPRB ,&
&  0.74612E+00_JPRB, 0.71573E+00_JPRB, 0.77795E+00_JPRB, 0.88080E+00_JPRB, 0.96224E+00_JPRB, 0.10469E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.12007E+01_JPRB, 0.12060E+01_JPRB, 0.12137E+01_JPRB, 0.10079E+01_JPRB, 0.12040E+01_JPRB, 0.12866E+01_JPRB ,&
&  0.14468E+01_JPRB, 0.16294E+01_JPRB, 0.17195E+01_JPRB, 0.20456E+01_JPRB, 0.19764E+01_JPRB, 0.18556E+01_JPRB, 0.17316E+01_JPRB ,&
&  0.16688E+01_JPRB, 0.14896E+01_JPRB, 0.12277E+01_JPRB, 0.12298E+01_JPRB, 0.13330E+01_JPRB, 0.13874E+01_JPRB, 0.92785E+00_JPRB ,&
&  0.84768E+00_JPRB, 0.10660E+01_JPRB, 0.96934E+00_JPRB, 0.85338E+00_JPRB, 0.10007E+01_JPRB, 0.10487E+01_JPRB, 0.81969E+00_JPRB ,&
&  0.50308E+00_JPRB, 0.43844E+00_JPRB, 0.44983E+00_JPRB, 0.47036E+00_JPRB, 0.47610E+00_JPRB, 0.75884E+00_JPRB, 0.23171E+01_JPRB ,&
&  0.25472E+01_JPRB, 0.29695E+01_JPRB, 0.27592E+01_JPRB, 0.23916E+01_JPRB, 0.21277E+01_JPRB, 0.19272E+01_JPRB, 0.17987E+01_JPRB ,&
&  0.15345E+01_JPRB, 0.13863E+01_JPRB, 0.12595E+01_JPRB, 0.11717E+01_JPRB, 0.10951E+01_JPRB, 0.10303E+01_JPRB, 0.97081E+00_JPRB ,&
&  0.92797E+00_JPRB, 0.87730E+00_JPRB, 0.83040E+00_JPRB, 0.77528E+00_JPRB, 0.72717E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.68356E+00_JPRB, 0.65153E+00_JPRB, 0.63899E+00_JPRB, 0.62658E+00_JPRB, 0.62121E+00_JPRB, 0.57525E+00_JPRB, 0.56405E+00_JPRB ,&
&  0.52944E+00_JPRB, 0.51578E+00_JPRB, 0.48683E+00_JPRB, 0.47245E+00_JPRB, 0.46868E+00_JPRB, 0.49958E+00_JPRB, 0.55749E+00_JPRB ,&
&  0.62060E+00_JPRB, 0.72727E+00_JPRB, 0.91394E+00_JPRB, 0.88494E+00_JPRB, 0.86557E+00_JPRB, 0.77920E+00_JPRB, 0.73420E+00_JPRB ,&
&  0.91100E+00_JPRB, 0.12800E+01_JPRB, 0.14811E+01_JPRB, 0.15852E+01_JPRB, 0.18019E+01_JPRB, 0.18495E+01_JPRB, 0.17603E+01_JPRB ,&
&  0.16227E+01_JPRB, 0.13867E+01_JPRB, 0.12010E+01_JPRB, 0.10608E+01_JPRB, 0.93802E+00_JPRB, 0.84030E+00_JPRB, 0.78149E+00_JPRB ,&
&  0.77298E+00_JPRB, 0.73534E+00_JPRB, 0.75162E+00_JPRB, 0.84580E+00_JPRB, 0.94516E+00_JPRB, 0.10049E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.11534E+01_JPRB, 0.11670E+01_JPRB, 0.11591E+01_JPRB, 0.97934E+00_JPRB, 0.87578E+00_JPRB, 0.11185E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.16585E+01_JPRB, 0.18959E+01_JPRB, 0.23195E+01_JPRB, 0.20662E+01_JPRB, 0.19535E+01_JPRB, 0.19505E+01_JPRB ,&
&  0.18236E+01_JPRB, 0.19574E+01_JPRB, 0.14234E+01_JPRB, 0.13803E+01_JPRB, 0.14677E+01_JPRB, 0.11189E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.12043E+01_JPRB, 0.11461E+01_JPRB, 0.96403E+00_JPRB, 0.77406E+00_JPRB, 0.94514E+00_JPRB, 0.86012E+00_JPRB, 0.49512E+00_JPRB ,&
&  0.42519E+00_JPRB, 0.40889E+00_JPRB, 0.39380E+00_JPRB, 0.38643E+00_JPRB, 0.40907E+00_JPRB, 0.64979E+00_JPRB, 0.12599E+01_JPRB ,&
&  0.18963E+01_JPRB, 0.27173E+01_JPRB, 0.28053E+01_JPRB, 0.25790E+01_JPRB, 0.22720E+01_JPRB, 0.20970E+01_JPRB, 0.19786E+01_JPRB ,&
&  0.16321E+01_JPRB, 0.14525E+01_JPRB, 0.13122E+01_JPRB, 0.12339E+01_JPRB, 0.11713E+01_JPRB, 0.11172E+01_JPRB, 0.10579E+01_JPRB ,&
&  0.98744E+00_JPRB, 0.92276E+00_JPRB, 0.85908E+00_JPRB, 0.78428E+00_JPRB, 0.72613E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.66370E+00_JPRB, 0.62517E+00_JPRB, 0.58564E+00_JPRB, 0.58143E+00_JPRB, 0.59979E+00_JPRB, 0.56526E+00_JPRB, 0.55041E+00_JPRB ,&
&  0.53069E+00_JPRB, 0.51781E+00_JPRB, 0.49991E+00_JPRB, 0.48066E+00_JPRB, 0.46832E+00_JPRB, 0.48578E+00_JPRB, 0.52851E+00_JPRB ,&
&  0.59362E+00_JPRB, 0.68992E+00_JPRB, 0.82086E+00_JPRB, 0.72143E+00_JPRB, 0.64755E+00_JPRB, 0.58174E+00_JPRB, 0.54154E+00_JPRB ,&
&  0.77066E+00_JPRB, 0.10380E+01_JPRB, 0.12515E+01_JPRB, 0.14873E+01_JPRB, 0.17853E+01_JPRB, 0.18424E+01_JPRB, 0.19774E+01_JPRB ,&
&  0.19273E+01_JPRB, 0.16355E+01_JPRB, 0.13913E+01_JPRB, 0.11890E+01_JPRB, 0.10414E+01_JPRB, 0.90818E+00_JPRB, 0.81653E+00_JPRB ,&
&  0.77023E+00_JPRB, 0.74931E+00_JPRB, 0.73324E+00_JPRB, 0.79414E+00_JPRB, 0.89558E+00_JPRB, 0.96163E+00_JPRB, 0.10188E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10918E+01_JPRB, 0.11087E+01_JPRB, 0.11294E+01_JPRB, 0.11754E+01_JPRB, 0.11384E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.11008E+01_JPRB, 0.12534E+01_JPRB, 0.18573E+01_JPRB, 0.21415E+01_JPRB, 0.20387E+01_JPRB, 0.20866E+01_JPRB, 0.21007E+01_JPRB ,&
&  0.20261E+01_JPRB, 0.21848E+01_JPRB, 0.16918E+01_JPRB, 0.16908E+01_JPRB, 0.14536E+01_JPRB, 0.10432E+01_JPRB, 0.12897E+01_JPRB ,&
&  0.13797E+01_JPRB, 0.13058E+01_JPRB, 0.10182E+01_JPRB, 0.75108E+00_JPRB, 0.72808E+00_JPRB, 0.53100E+00_JPRB, 0.40698E+00_JPRB ,&
&  0.39563E+00_JPRB, 0.38678E+00_JPRB, 0.36295E+00_JPRB, 0.38849E+00_JPRB, 0.40380E+00_JPRB, 0.58408E+00_JPRB, 0.95137E+00_JPRB ,&
&  0.16142E+01_JPRB, 0.23009E+01_JPRB, 0.26568E+01_JPRB, 0.25612E+01_JPRB, 0.23288E+01_JPRB, 0.20754E+01_JPRB, 0.20864E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.15018E+01_JPRB, 0.13537E+01_JPRB, 0.12782E+01_JPRB, 0.12078E+01_JPRB, 0.11426E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.97549E+00_JPRB, 0.91051E+00_JPRB, 0.84844E+00_JPRB, 0.77265E+00_JPRB, 0.71348E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.63830E+00_JPRB, 0.59756E+00_JPRB, 0.54925E+00_JPRB, 0.54878E+00_JPRB, 0.57596E+00_JPRB, 0.56315E+00_JPRB, 0.53742E+00_JPRB ,&
&  0.53176E+00_JPRB, 0.51151E+00_JPRB, 0.50027E+00_JPRB, 0.48504E+00_JPRB, 0.47224E+00_JPRB, 0.47612E+00_JPRB, 0.50900E+00_JPRB ,&
&  0.58621E+00_JPRB, 0.70134E+00_JPRB, 0.61692E+00_JPRB, 0.55037E+00_JPRB, 0.54253E+00_JPRB, 0.53349E+00_JPRB, 0.47223E+00_JPRB ,&
&  0.67424E+00_JPRB, 0.85701E+00_JPRB, 0.10335E+01_JPRB, 0.13824E+01_JPRB, 0.19973E+01_JPRB, 0.21436E+01_JPRB, 0.22097E+01_JPRB ,&
&  0.21706E+01_JPRB, 0.20072E+01_JPRB, 0.16704E+01_JPRB, 0.13597E+01_JPRB, 0.11393E+01_JPRB, 0.99550E+00_JPRB, 0.86240E+00_JPRB ,&
&  0.77100E+00_JPRB, 0.74737E+00_JPRB, 0.74785E+00_JPRB, 0.76243E+00_JPRB, 0.82518E+00_JPRB, 0.88063E+00_JPRB, 0.91704E+00_JPRB ,&
&  0.97572E+00_JPRB, 0.10265E+01_JPRB, 0.10444E+01_JPRB, 0.10490E+01_JPRB, 0.10413E+01_JPRB, 0.98233E+00_JPRB, 0.13745E+01_JPRB ,&
&  0.14431E+01_JPRB, 0.15309E+01_JPRB, 0.15917E+01_JPRB, 0.20952E+01_JPRB, 0.21935E+01_JPRB, 0.20533E+01_JPRB, 0.23741E+01_JPRB ,&
&  0.17601E+01_JPRB, 0.15412E+01_JPRB, 0.15225E+01_JPRB, 0.14425E+01_JPRB, 0.11522E+01_JPRB, 0.12811E+01_JPRB, 0.18732E+01_JPRB ,&
&  0.14303E+01_JPRB, 0.15621E+01_JPRB, 0.16178E+01_JPRB, 0.16433E+01_JPRB, 0.75601E+00_JPRB, 0.49916E+00_JPRB, 0.82576E+00_JPRB ,&
&  0.93693E+00_JPRB, 0.63132E+00_JPRB, 0.48582E+00_JPRB, 0.63267E+00_JPRB, 0.51634E+00_JPRB, 0.63860E+00_JPRB, 0.97245E+00_JPRB ,&
&  0.12389E+01_JPRB, 0.16015E+01_JPRB, 0.23771E+01_JPRB, 0.24696E+01_JPRB, 0.23004E+01_JPRB, 0.21266E+01_JPRB, 0.20661E+01_JPRB ,&
&  0.18922E+01_JPRB, 0.16207E+01_JPRB, 0.15222E+01_JPRB, 0.13291E+01_JPRB, 0.11858E+01_JPRB, 0.10982E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.95289E+00_JPRB, 0.88245E+00_JPRB, 0.81422E+00_JPRB, 0.74373E+00_JPRB, 0.68529E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.59199E+00_JPRB, 0.56162E+00_JPRB, 0.51814E+00_JPRB, 0.51772E+00_JPRB, 0.54249E+00_JPRB, 0.55078E+00_JPRB, 0.53452E+00_JPRB ,&
&  0.52036E+00_JPRB, 0.49995E+00_JPRB, 0.49240E+00_JPRB, 0.48342E+00_JPRB, 0.47525E+00_JPRB, 0.47786E+00_JPRB, 0.49129E+00_JPRB ,&
&  0.54412E+00_JPRB, 0.55806E+00_JPRB, 0.49720E+00_JPRB, 0.46553E+00_JPRB, 0.53824E+00_JPRB, 0.50308E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.63049E+00_JPRB, 0.74706E+00_JPRB, 0.88553E+00_JPRB, 0.11562E+01_JPRB, 0.14937E+01_JPRB, 0.20232E+01_JPRB, 0.22849E+01_JPRB ,&
&  0.23002E+01_JPRB, 0.22537E+01_JPRB, 0.19748E+01_JPRB, 0.16331E+01_JPRB, 0.13097E+01_JPRB, 0.11106E+01_JPRB, 0.96394E+00_JPRB ,&
&  0.84111E+00_JPRB, 0.75538E+00_JPRB, 0.73685E+00_JPRB, 0.75660E+00_JPRB, 0.79431E+00_JPRB, 0.82449E+00_JPRB, 0.82694E+00_JPRB ,&
&  0.85475E+00_JPRB, 0.91687E+00_JPRB, 0.95100E+00_JPRB, 0.94987E+00_JPRB, 0.89930E+00_JPRB, 0.81377E+00_JPRB, 0.98539E+00_JPRB ,&
&  0.13507E+01_JPRB, 0.14208E+01_JPRB, 0.16339E+01_JPRB, 0.20181E+01_JPRB, 0.23255E+01_JPRB, 0.21772E+01_JPRB, 0.27532E+01_JPRB ,&
&  0.22488E+01_JPRB, 0.18238E+01_JPRB, 0.14965E+01_JPRB, 0.10693E+01_JPRB, 0.13413E+01_JPRB, 0.20164E+01_JPRB, 0.20641E+01_JPRB ,&
&  0.19407E+01_JPRB, 0.18812E+01_JPRB, 0.18220E+01_JPRB, 0.15596E+01_JPRB, 0.12489E+01_JPRB, 0.95756E+00_JPRB, 0.11710E+01_JPRB ,&
&  0.12122E+01_JPRB, 0.11814E+01_JPRB, 0.11033E+01_JPRB, 0.95287E+00_JPRB, 0.80421E+00_JPRB, 0.93807E+00_JPRB, 0.98100E+00_JPRB ,&
&  0.10782E+01_JPRB, 0.11776E+01_JPRB, 0.15989E+01_JPRB, 0.20631E+01_JPRB, 0.21956E+01_JPRB, 0.20734E+01_JPRB, 0.21985E+01_JPRB ,&
&  0.19302E+01_JPRB, 0.17763E+01_JPRB, 0.15134E+01_JPRB, 0.12868E+01_JPRB, 0.11465E+01_JPRB, 0.10543E+01_JPRB, 0.98618E+00_JPRB ,&
&  0.90282E+00_JPRB, 0.83720E+00_JPRB, 0.77144E+00_JPRB, 0.69216E+00_JPRB, 0.62903E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.55933E+00_JPRB, 0.53092E+00_JPRB, 0.49018E+00_JPRB, 0.48339E+00_JPRB, 0.50282E+00_JPRB, 0.52683E+00_JPRB, 0.53314E+00_JPRB ,&
&  0.51346E+00_JPRB, 0.49928E+00_JPRB, 0.48812E+00_JPRB, 0.48566E+00_JPRB, 0.48164E+00_JPRB, 0.47455E+00_JPRB, 0.48155E+00_JPRB ,&
&  0.51927E+00_JPRB, 0.50009E+00_JPRB, 0.44234E+00_JPRB, 0.47923E+00_JPRB, 0.51452E+00_JPRB, 0.47804E+00_JPRB, 0.53589E+00_JPRB ,&
&  0.61124E+00_JPRB, 0.69993E+00_JPRB, 0.77182E+00_JPRB, 0.92469E+00_JPRB, 0.11406E+01_JPRB, 0.14752E+01_JPRB, 0.18604E+01_JPRB ,&
&  0.19948E+01_JPRB, 0.21078E+01_JPRB, 0.21417E+01_JPRB, 0.19521E+01_JPRB, 0.15908E+01_JPRB, 0.12995E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.97809E+00_JPRB, 0.83662E+00_JPRB, 0.76134E+00_JPRB, 0.74856E+00_JPRB, 0.77373E+00_JPRB, 0.80990E+00_JPRB, 0.84455E+00_JPRB ,&
&  0.81925E+00_JPRB, 0.81855E+00_JPRB, 0.83663E+00_JPRB, 0.93149E+00_JPRB, 0.77269E+00_JPRB, 0.78140E+00_JPRB, 0.88431E+00_JPRB ,&
&  0.14439E+01_JPRB, 0.16517E+01_JPRB, 0.18012E+01_JPRB, 0.23850E+01_JPRB, 0.21576E+01_JPRB, 0.23574E+01_JPRB, 0.28505E+01_JPRB ,&
&  0.31653E+01_JPRB, 0.32009E+01_JPRB, 0.31595E+01_JPRB, 0.19389E+01_JPRB, 0.16888E+01_JPRB, 0.23108E+01_JPRB, 0.22378E+01_JPRB ,&
&  0.20433E+01_JPRB, 0.20198E+01_JPRB, 0.18946E+01_JPRB, 0.19689E+01_JPRB, 0.15255E+01_JPRB, 0.12912E+01_JPRB, 0.95596E+00_JPRB ,&
&  0.90183E+00_JPRB, 0.80100E+00_JPRB, 0.94546E+00_JPRB, 0.86954E+00_JPRB, 0.87183E+00_JPRB, 0.86272E+00_JPRB, 0.83655E+00_JPRB ,&
&  0.88843E+00_JPRB, 0.95785E+00_JPRB, 0.10875E+01_JPRB, 0.14642E+01_JPRB, 0.18868E+01_JPRB, 0.17819E+01_JPRB, 0.19577E+01_JPRB ,&
&  0.18937E+01_JPRB, 0.16773E+01_JPRB, 0.13879E+01_JPRB, 0.12411E+01_JPRB, 0.11232E+01_JPRB, 0.10280E+01_JPRB, 0.93832E+00_JPRB ,&
&  0.84774E+00_JPRB, 0.77877E+00_JPRB, 0.71353E+00_JPRB, 0.64744E+00_JPRB, 0.59235E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.55124E+00_JPRB, 0.52489E+00_JPRB, 0.47710E+00_JPRB, 0.46579E+00_JPRB, 0.47230E+00_JPRB, 0.49498E+00_JPRB, 0.51043E+00_JPRB ,&
&  0.50736E+00_JPRB, 0.49419E+00_JPRB, 0.48543E+00_JPRB, 0.48267E+00_JPRB, 0.47868E+00_JPRB, 0.47382E+00_JPRB, 0.48816E+00_JPRB ,&
&  0.51626E+00_JPRB, 0.55562E+00_JPRB, 0.48757E+00_JPRB, 0.44723E+00_JPRB, 0.42147E+00_JPRB, 0.47278E+00_JPRB, 0.55332E+00_JPRB ,&
&  0.63853E+00_JPRB, 0.69656E+00_JPRB, 0.77011E+00_JPRB, 0.83768E+00_JPRB, 0.88566E+00_JPRB, 0.10595E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.19674E+01_JPRB, 0.19367E+01_JPRB, 0.20485E+01_JPRB, 0.19213E+01_JPRB, 0.17146E+01_JPRB, 0.14861E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.94160E+00_JPRB, 0.82298E+00_JPRB, 0.75128E+00_JPRB, 0.73634E+00_JPRB, 0.74515E+00_JPRB, 0.79351E+00_JPRB ,&
&  0.81651E+00_JPRB, 0.80196E+00_JPRB, 0.77293E+00_JPRB, 0.79040E+00_JPRB, 0.87392E+00_JPRB, 0.94675E+00_JPRB, 0.11288E+01_JPRB ,&
&  0.12863E+01_JPRB, 0.13600E+01_JPRB, 0.19718E+01_JPRB, 0.23464E+01_JPRB, 0.29618E+01_JPRB, 0.31829E+01_JPRB, 0.30888E+01_JPRB ,&
&  0.31872E+01_JPRB, 0.30741E+01_JPRB, 0.30161E+01_JPRB, 0.24555E+01_JPRB, 0.28084E+01_JPRB, 0.27125E+01_JPRB, 0.24687E+01_JPRB ,&
&  0.22380E+01_JPRB, 0.20737E+01_JPRB, 0.19718E+01_JPRB, 0.19406E+01_JPRB, 0.19738E+01_JPRB, 0.20271E+01_JPRB, 0.15431E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.12323E+01_JPRB, 0.10961E+01_JPRB, 0.90716E+00_JPRB, 0.75426E+00_JPRB, 0.78400E+00_JPRB, 0.81783E+00_JPRB ,&
&  0.84701E+00_JPRB, 0.87061E+00_JPRB, 0.95346E+00_JPRB, 0.12032E+01_JPRB, 0.16063E+01_JPRB, 0.18183E+01_JPRB, 0.18239E+01_JPRB ,&
&  0.16048E+01_JPRB, 0.15769E+01_JPRB, 0.14292E+01_JPRB, 0.12174E+01_JPRB, 0.10852E+01_JPRB, 0.97172E+00_JPRB, 0.86491E+00_JPRB ,&
&  0.77599E+00_JPRB, 0.71715E+00_JPRB, 0.66884E+00_JPRB, 0.62194E+00_JPRB, 0.57553E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.53451E+00_JPRB, 0.51884E+00_JPRB, 0.48186E+00_JPRB, 0.46796E+00_JPRB, 0.46842E+00_JPRB, 0.47844E+00_JPRB, 0.48923E+00_JPRB ,&
&  0.49313E+00_JPRB, 0.47953E+00_JPRB, 0.46998E+00_JPRB, 0.47006E+00_JPRB, 0.47364E+00_JPRB, 0.48011E+00_JPRB, 0.50538E+00_JPRB ,&
&  0.56342E+00_JPRB, 0.68184E+00_JPRB, 0.56302E+00_JPRB, 0.53549E+00_JPRB, 0.48101E+00_JPRB, 0.52434E+00_JPRB, 0.59298E+00_JPRB ,&
&  0.69372E+00_JPRB, 0.70841E+00_JPRB, 0.69089E+00_JPRB, 0.71624E+00_JPRB, 0.74817E+00_JPRB, 0.83882E+00_JPRB, 0.10619E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.16423E+01_JPRB, 0.17254E+01_JPRB, 0.18430E+01_JPRB, 0.16707E+01_JPRB, 0.13958E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.10252E+01_JPRB, 0.88807E+00_JPRB, 0.80933E+00_JPRB, 0.76668E+00_JPRB, 0.73564E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.75936E+00_JPRB, 0.76648E+00_JPRB, 0.75790E+00_JPRB, 0.75117E+00_JPRB, 0.81509E+00_JPRB, 0.10057E+01_JPRB, 0.11950E+01_JPRB ,&
&  0.17341E+01_JPRB, 0.18566E+01_JPRB, 0.16673E+01_JPRB, 0.23955E+01_JPRB, 0.35062E+01_JPRB, 0.34116E+01_JPRB, 0.29148E+01_JPRB ,&
&  0.30214E+01_JPRB, 0.29440E+01_JPRB, 0.29703E+01_JPRB, 0.30392E+01_JPRB, 0.30061E+01_JPRB, 0.28575E+01_JPRB, 0.26334E+01_JPRB ,&
&  0.23481E+01_JPRB, 0.21533E+01_JPRB, 0.20087E+01_JPRB, 0.20323E+01_JPRB, 0.19230E+01_JPRB, 0.18267E+01_JPRB, 0.17418E+01_JPRB ,&
&  0.14390E+01_JPRB, 0.11774E+01_JPRB, 0.79791E+00_JPRB, 0.82685E+00_JPRB, 0.65094E+00_JPRB, 0.66992E+00_JPRB, 0.80105E+00_JPRB ,&
&  0.80912E+00_JPRB, 0.90121E+00_JPRB, 0.10146E+01_JPRB, 0.10945E+01_JPRB, 0.14183E+01_JPRB, 0.15795E+01_JPRB, 0.16348E+01_JPRB ,&
&  0.15695E+01_JPRB, 0.13073E+01_JPRB, 0.12928E+01_JPRB, 0.11633E+01_JPRB, 0.10768E+01_JPRB, 0.92665E+00_JPRB, 0.78618E+00_JPRB ,&
&  0.69286E+00_JPRB, 0.64198E+00_JPRB, 0.60858E+00_JPRB, 0.57663E+00_JPRB, 0.54767E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.49077E+00_JPRB, 0.47843E+00_JPRB, 0.45803E+00_JPRB, 0.45477E+00_JPRB, 0.45339E+00_JPRB, 0.45551E+00_JPRB, 0.46759E+00_JPRB ,&
&  0.48010E+00_JPRB, 0.47214E+00_JPRB, 0.45476E+00_JPRB, 0.44969E+00_JPRB, 0.45482E+00_JPRB, 0.47255E+00_JPRB, 0.52703E+00_JPRB ,&
&  0.64395E+00_JPRB, 0.69001E+00_JPRB, 0.58681E+00_JPRB, 0.49001E+00_JPRB, 0.55533E+00_JPRB, 0.54316E+00_JPRB, 0.59363E+00_JPRB ,&
&  0.57539E+00_JPRB, 0.63304E+00_JPRB, 0.61068E+00_JPRB, 0.59580E+00_JPRB, 0.59436E+00_JPRB, 0.65133E+00_JPRB, 0.76477E+00_JPRB ,&
&  0.90384E+00_JPRB, 0.11247E+01_JPRB, 0.13322E+01_JPRB, 0.13292E+01_JPRB, 0.13689E+01_JPRB, 0.12250E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.96644E+00_JPRB, 0.85304E+00_JPRB, 0.78676E+00_JPRB, 0.74934E+00_JPRB, 0.72387E+00_JPRB, 0.71980E+00_JPRB ,&
&  0.73392E+00_JPRB, 0.74121E+00_JPRB, 0.72924E+00_JPRB, 0.73850E+00_JPRB, 0.84784E+00_JPRB, 0.11461E+01_JPRB, 0.15846E+01_JPRB ,&
&  0.21112E+01_JPRB, 0.24254E+01_JPRB, 0.26166E+01_JPRB, 0.30357E+01_JPRB, 0.31547E+01_JPRB, 0.31386E+01_JPRB, 0.30141E+01_JPRB ,&
&  0.28314E+01_JPRB, 0.26460E+01_JPRB, 0.26786E+01_JPRB, 0.27020E+01_JPRB, 0.27303E+01_JPRB, 0.27678E+01_JPRB, 0.26438E+01_JPRB ,&
&  0.23267E+01_JPRB, 0.21342E+01_JPRB, 0.20399E+01_JPRB, 0.18750E+01_JPRB, 0.17910E+01_JPRB, 0.16531E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.17522E+01_JPRB, 0.99380E+00_JPRB, 0.76683E+00_JPRB, 0.90448E+00_JPRB, 0.75945E+00_JPRB, 0.79662E+00_JPRB, 0.95701E+00_JPRB ,&
&  0.97589E+00_JPRB, 0.99626E+00_JPRB, 0.11215E+01_JPRB, 0.11747E+01_JPRB, 0.12486E+01_JPRB, 0.14454E+01_JPRB, 0.14053E+01_JPRB ,&
&  0.12780E+01_JPRB, 0.11461E+01_JPRB, 0.11389E+01_JPRB, 0.10545E+01_JPRB, 0.95225E+00_JPRB, 0.78922E+00_JPRB, 0.67466E+00_JPRB ,&
&  0.60852E+00_JPRB, 0.57350E+00_JPRB, 0.54849E+00_JPRB, 0.52403E+00_JPRB, 0.49961E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.46335E+00_JPRB, 0.44907E+00_JPRB, 0.43533E+00_JPRB, 0.43735E+00_JPRB, 0.44546E+00_JPRB, 0.44743E+00_JPRB, 0.45921E+00_JPRB ,&
&  0.47089E+00_JPRB, 0.47012E+00_JPRB, 0.45613E+00_JPRB, 0.44508E+00_JPRB, 0.44394E+00_JPRB, 0.46314E+00_JPRB, 0.51999E+00_JPRB ,&
&  0.47940E+00_JPRB, 0.50396E+00_JPRB, 0.44910E+00_JPRB, 0.46839E+00_JPRB, 0.56201E+00_JPRB, 0.50849E+00_JPRB, 0.50010E+00_JPRB ,&
&  0.48380E+00_JPRB, 0.47712E+00_JPRB, 0.45753E+00_JPRB, 0.44916E+00_JPRB, 0.45469E+00_JPRB, 0.48924E+00_JPRB, 0.56145E+00_JPRB ,&
&  0.61220E+00_JPRB, 0.73211E+00_JPRB, 0.85791E+00_JPRB, 0.85408E+00_JPRB, 0.91986E+00_JPRB, 0.97135E+00_JPRB, 0.10095E+01_JPRB ,&
&  0.96632E+00_JPRB, 0.85980E+00_JPRB, 0.76697E+00_JPRB, 0.71538E+00_JPRB, 0.68385E+00_JPRB, 0.66967E+00_JPRB, 0.66021E+00_JPRB ,&
&  0.65326E+00_JPRB, 0.66173E+00_JPRB, 0.68966E+00_JPRB, 0.74824E+00_JPRB, 0.84655E+00_JPRB, 0.12827E+01_JPRB, 0.20049E+01_JPRB ,&
&  0.23608E+01_JPRB, 0.25194E+01_JPRB, 0.27185E+01_JPRB, 0.30440E+01_JPRB, 0.29916E+01_JPRB, 0.27882E+01_JPRB, 0.24957E+01_JPRB ,&
&  0.23265E+01_JPRB, 0.21808E+01_JPRB, 0.23129E+01_JPRB, 0.24876E+01_JPRB, 0.24589E+01_JPRB, 0.26657E+01_JPRB, 0.26015E+01_JPRB ,&
&  0.23543E+01_JPRB, 0.21993E+01_JPRB, 0.20888E+01_JPRB, 0.20549E+01_JPRB, 0.19738E+01_JPRB, 0.20282E+01_JPRB, 0.19886E+01_JPRB ,&
&  0.19740E+01_JPRB, 0.17181E+01_JPRB, 0.12798E+01_JPRB, 0.11770E+01_JPRB, 0.89768E+00_JPRB, 0.92839E+00_JPRB, 0.11894E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.11991E+01_JPRB, 0.11744E+01_JPRB, 0.11660E+01_JPRB, 0.11381E+01_JPRB, 0.12641E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10466E+01_JPRB, 0.98507E+00_JPRB, 0.87601E+00_JPRB, 0.77997E+00_JPRB, 0.65921E+00_JPRB, 0.57890E+00_JPRB ,&
&  0.55233E+00_JPRB, 0.52505E+00_JPRB, 0.51109E+00_JPRB, 0.49298E+00_JPRB, 0.47396E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.44875E+00_JPRB, 0.43628E+00_JPRB, 0.42603E+00_JPRB, 0.42575E+00_JPRB, 0.43772E+00_JPRB, 0.44275E+00_JPRB, 0.45452E+00_JPRB ,&
&  0.46104E+00_JPRB, 0.45970E+00_JPRB, 0.45763E+00_JPRB, 0.44393E+00_JPRB, 0.44137E+00_JPRB, 0.45359E+00_JPRB, 0.44551E+00_JPRB ,&
&  0.40359E+00_JPRB, 0.42366E+00_JPRB, 0.41412E+00_JPRB, 0.42025E+00_JPRB, 0.42573E+00_JPRB, 0.40654E+00_JPRB, 0.39297E+00_JPRB ,&
&  0.40466E+00_JPRB, 0.39724E+00_JPRB, 0.39958E+00_JPRB, 0.40989E+00_JPRB, 0.42510E+00_JPRB, 0.45207E+00_JPRB, 0.49122E+00_JPRB ,&
&  0.50497E+00_JPRB, 0.56249E+00_JPRB, 0.62266E+00_JPRB, 0.61661E+00_JPRB, 0.69685E+00_JPRB, 0.77476E+00_JPRB, 0.79981E+00_JPRB ,&
&  0.80330E+00_JPRB, 0.75697E+00_JPRB, 0.69936E+00_JPRB, 0.65431E+00_JPRB, 0.61290E+00_JPRB, 0.59234E+00_JPRB, 0.58634E+00_JPRB ,&
&  0.58381E+00_JPRB, 0.58921E+00_JPRB, 0.60786E+00_JPRB, 0.64706E+00_JPRB, 0.72519E+00_JPRB, 0.10541E+01_JPRB, 0.14311E+01_JPRB ,&
&  0.16073E+01_JPRB, 0.17281E+01_JPRB, 0.18243E+01_JPRB, 0.18671E+01_JPRB, 0.18854E+01_JPRB, 0.18555E+01_JPRB, 0.17761E+01_JPRB ,&
&  0.18526E+01_JPRB, 0.18179E+01_JPRB, 0.22951E+01_JPRB, 0.23785E+01_JPRB, 0.23889E+01_JPRB, 0.24901E+01_JPRB, 0.24516E+01_JPRB ,&
&  0.24361E+01_JPRB, 0.22713E+01_JPRB, 0.22062E+01_JPRB, 0.21275E+01_JPRB, 0.20608E+01_JPRB, 0.21070E+01_JPRB, 0.20120E+01_JPRB ,&
&  0.20140E+01_JPRB, 0.19985E+01_JPRB, 0.17546E+01_JPRB, 0.15570E+01_JPRB, 0.13199E+01_JPRB, 0.13926E+01_JPRB, 0.13106E+01_JPRB ,&
&  0.12160E+01_JPRB, 0.10025E+01_JPRB, 0.10592E+01_JPRB, 0.10716E+01_JPRB, 0.10803E+01_JPRB, 0.10501E+01_JPRB, 0.98791E+00_JPRB ,&
&  0.96774E+00_JPRB, 0.95892E+00_JPRB, 0.81955E+00_JPRB, 0.70312E+00_JPRB, 0.63300E+00_JPRB, 0.57021E+00_JPRB, 0.48901E+00_JPRB ,&
&  0.46646E+00_JPRB, 0.48340E+00_JPRB, 0.47980E+00_JPRB, 0.47164E+00_JPRB, 0.46082E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.44171E+00_JPRB, 0.43535E+00_JPRB, 0.42893E+00_JPRB, 0.42440E+00_JPRB, 0.43481E+00_JPRB, 0.43886E+00_JPRB, 0.43665E+00_JPRB ,&
&  0.45182E+00_JPRB, 0.44619E+00_JPRB, 0.44448E+00_JPRB, 0.43770E+00_JPRB, 0.43250E+00_JPRB, 0.39765E+00_JPRB, 0.33546E+00_JPRB ,&
&  0.32113E+00_JPRB, 0.33592E+00_JPRB, 0.36315E+00_JPRB, 0.37604E+00_JPRB, 0.37039E+00_JPRB, 0.35805E+00_JPRB, 0.35044E+00_JPRB ,&
&  0.35417E+00_JPRB, 0.36208E+00_JPRB, 0.38168E+00_JPRB, 0.39971E+00_JPRB, 0.40990E+00_JPRB, 0.41959E+00_JPRB, 0.43839E+00_JPRB ,&
&  0.44180E+00_JPRB, 0.46930E+00_JPRB, 0.51885E+00_JPRB, 0.48490E+00_JPRB, 0.54924E+00_JPRB, 0.60135E+00_JPRB, 0.60295E+00_JPRB ,&
&  0.62329E+00_JPRB, 0.61520E+00_JPRB, 0.58979E+00_JPRB, 0.57701E+00_JPRB, 0.54947E+00_JPRB, 0.52358E+00_JPRB, 0.51161E+00_JPRB ,&
&  0.50886E+00_JPRB, 0.51376E+00_JPRB, 0.52990E+00_JPRB, 0.55292E+00_JPRB, 0.60744E+00_JPRB, 0.79210E+00_JPRB, 0.94291E+00_JPRB ,&
&  0.10108E+01_JPRB, 0.10961E+01_JPRB, 0.12222E+01_JPRB, 0.12039E+01_JPRB, 0.12656E+01_JPRB, 0.13692E+01_JPRB, 0.14046E+01_JPRB ,&
&  0.15757E+01_JPRB, 0.15835E+01_JPRB, 0.17675E+01_JPRB, 0.19044E+01_JPRB, 0.20349E+01_JPRB, 0.20975E+01_JPRB, 0.21400E+01_JPRB ,&
&  0.22616E+01_JPRB, 0.23217E+01_JPRB, 0.21049E+01_JPRB, 0.19731E+01_JPRB, 0.18578E+01_JPRB, 0.18435E+01_JPRB, 0.17808E+01_JPRB ,&
&  0.17924E+01_JPRB, 0.18365E+01_JPRB, 0.18216E+01_JPRB, 0.17713E+01_JPRB, 0.16668E+01_JPRB, 0.15242E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.12989E+01_JPRB, 0.11441E+01_JPRB, 0.10575E+01_JPRB, 0.10514E+01_JPRB, 0.10382E+01_JPRB, 0.89974E+00_JPRB, 0.88046E+00_JPRB ,&
&  0.87508E+00_JPRB, 0.76382E+00_JPRB, 0.70946E+00_JPRB, 0.62380E+00_JPRB, 0.58359E+00_JPRB, 0.54483E+00_JPRB, 0.48929E+00_JPRB ,&
&  0.44738E+00_JPRB, 0.47213E+00_JPRB, 0.46515E+00_JPRB, 0.45153E+00_JPRB, 0.44592E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.44223E+00_JPRB, 0.44180E+00_JPRB, 0.43258E+00_JPRB, 0.42482E+00_JPRB, 0.42784E+00_JPRB, 0.41632E+00_JPRB, 0.41374E+00_JPRB ,&
&  0.42450E+00_JPRB, 0.41189E+00_JPRB, 0.43063E+00_JPRB, 0.36950E+00_JPRB, 0.33618E+00_JPRB, 0.32896E+00_JPRB, 0.31805E+00_JPRB ,&
&  0.31841E+00_JPRB, 0.32887E+00_JPRB, 0.33724E+00_JPRB, 0.34390E+00_JPRB, 0.34940E+00_JPRB, 0.34120E+00_JPRB, 0.33726E+00_JPRB ,&
&  0.34206E+00_JPRB, 0.35623E+00_JPRB, 0.37991E+00_JPRB, 0.38960E+00_JPRB, 0.39168E+00_JPRB, 0.39181E+00_JPRB, 0.39657E+00_JPRB ,&
&  0.38981E+00_JPRB, 0.40442E+00_JPRB, 0.44051E+00_JPRB, 0.44446E+00_JPRB, 0.46849E+00_JPRB, 0.50659E+00_JPRB, 0.49949E+00_JPRB ,&
&  0.50040E+00_JPRB, 0.45059E+00_JPRB, 0.47598E+00_JPRB, 0.48310E+00_JPRB, 0.48176E+00_JPRB, 0.46789E+00_JPRB, 0.45368E+00_JPRB ,&
&  0.44662E+00_JPRB, 0.44855E+00_JPRB, 0.45894E+00_JPRB, 0.47587E+00_JPRB, 0.50259E+00_JPRB, 0.55957E+00_JPRB, 0.61118E+00_JPRB ,&
&  0.67950E+00_JPRB, 0.60510E+00_JPRB, 0.73221E+00_JPRB, 0.78581E+00_JPRB, 0.90802E+00_JPRB, 0.10826E+01_JPRB, 0.12110E+01_JPRB ,&
&  0.15208E+01_JPRB, 0.15260E+01_JPRB, 0.16180E+01_JPRB, 0.16331E+01_JPRB, 0.16467E+01_JPRB, 0.16846E+01_JPRB, 0.17541E+01_JPRB ,&
&  0.18551E+01_JPRB, 0.17387E+01_JPRB, 0.17851E+01_JPRB, 0.16586E+01_JPRB, 0.15745E+01_JPRB, 0.15673E+01_JPRB, 0.15602E+01_JPRB ,&
&  0.15784E+01_JPRB, 0.15866E+01_JPRB, 0.15622E+01_JPRB, 0.13718E+01_JPRB, 0.13271E+01_JPRB, 0.13086E+01_JPRB, 0.12109E+01_JPRB ,&
&  0.12231E+01_JPRB, 0.11975E+01_JPRB, 0.11106E+01_JPRB, 0.11560E+01_JPRB, 0.10289E+01_JPRB, 0.94466E+00_JPRB, 0.90451E+00_JPRB ,&
&  0.84162E+00_JPRB, 0.67594E+00_JPRB, 0.63298E+00_JPRB, 0.59855E+00_JPRB, 0.55334E+00_JPRB, 0.51121E+00_JPRB, 0.52365E+00_JPRB ,&
&  0.48359E+00_JPRB, 0.47372E+00_JPRB, 0.47967E+00_JPRB, 0.46188E+00_JPRB, 0.44747E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.45822E+00_JPRB, 0.45103E+00_JPRB, 0.43846E+00_JPRB, 0.42810E+00_JPRB, 0.42462E+00_JPRB, 0.41164E+00_JPRB, 0.38841E+00_JPRB ,&
&  0.37857E+00_JPRB, 0.35611E+00_JPRB, 0.33976E+00_JPRB, 0.32975E+00_JPRB, 0.32591E+00_JPRB, 0.32077E+00_JPRB, 0.30912E+00_JPRB ,&
&  0.30700E+00_JPRB, 0.31348E+00_JPRB, 0.32565E+00_JPRB, 0.32980E+00_JPRB, 0.32931E+00_JPRB, 0.32796E+00_JPRB, 0.33066E+00_JPRB ,&
&  0.34021E+00_JPRB, 0.35693E+00_JPRB, 0.37610E+00_JPRB, 0.38523E+00_JPRB, 0.38270E+00_JPRB, 0.37979E+00_JPRB, 0.38099E+00_JPRB ,&
&  0.37899E+00_JPRB, 0.37959E+00_JPRB, 0.38801E+00_JPRB, 0.40149E+00_JPRB, 0.43422E+00_JPRB, 0.45947E+00_JPRB, 0.44334E+00_JPRB ,&
&  0.34090E+00_JPRB, 0.33104E+00_JPRB, 0.38938E+00_JPRB, 0.40130E+00_JPRB, 0.41049E+00_JPRB, 0.40604E+00_JPRB, 0.40248E+00_JPRB ,&
&  0.39984E+00_JPRB, 0.39968E+00_JPRB, 0.39986E+00_JPRB, 0.40637E+00_JPRB, 0.42296E+00_JPRB, 0.44650E+00_JPRB, 0.46344E+00_JPRB ,&
&  0.50163E+00_JPRB, 0.48281E+00_JPRB, 0.49256E+00_JPRB, 0.54065E+00_JPRB, 0.73373E+00_JPRB, 0.88798E+00_JPRB, 0.10198E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.13139E+01_JPRB, 0.14683E+01_JPRB, 0.15519E+01_JPRB, 0.15632E+01_JPRB, 0.15417E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.16057E+01_JPRB, 0.15968E+01_JPRB, 0.15669E+01_JPRB, 0.15182E+01_JPRB, 0.14254E+01_JPRB, 0.13905E+01_JPRB, 0.13690E+01_JPRB ,&
&  0.13778E+01_JPRB, 0.13773E+01_JPRB, 0.13069E+01_JPRB, 0.12170E+01_JPRB, 0.11442E+01_JPRB, 0.11355E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.11288E+01_JPRB, 0.11329E+01_JPRB, 0.11088E+01_JPRB, 0.10698E+01_JPRB, 0.99298E+00_JPRB, 0.91033E+00_JPRB, 0.85388E+00_JPRB ,&
&  0.74076E+00_JPRB, 0.63901E+00_JPRB, 0.55300E+00_JPRB, 0.54089E+00_JPRB, 0.54734E+00_JPRB, 0.52779E+00_JPRB, 0.49224E+00_JPRB ,&
&  0.48237E+00_JPRB, 0.46431E+00_JPRB, 0.44681E+00_JPRB, 0.45790E+00_JPRB, 0.45821E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.43548E+00_JPRB, 0.42936E+00_JPRB, 0.42054E+00_JPRB, 0.41605E+00_JPRB, 0.40503E+00_JPRB, 0.39155E+00_JPRB, 0.37710E+00_JPRB ,&
&  0.37165E+00_JPRB, 0.36010E+00_JPRB, 0.34767E+00_JPRB, 0.33229E+00_JPRB, 0.32130E+00_JPRB, 0.31259E+00_JPRB, 0.31322E+00_JPRB ,&
&  0.31935E+00_JPRB, 0.32568E+00_JPRB, 0.32651E+00_JPRB, 0.32223E+00_JPRB, 0.31935E+00_JPRB, 0.32191E+00_JPRB, 0.33137E+00_JPRB ,&
&  0.34708E+00_JPRB, 0.36588E+00_JPRB, 0.37857E+00_JPRB, 0.38195E+00_JPRB, 0.38079E+00_JPRB, 0.37942E+00_JPRB, 0.37485E+00_JPRB ,&
&  0.36377E+00_JPRB, 0.35925E+00_JPRB, 0.36462E+00_JPRB, 0.37901E+00_JPRB, 0.40373E+00_JPRB, 0.41052E+00_JPRB, 0.35693E+00_JPRB ,&
&  0.28960E+00_JPRB, 0.26755E+00_JPRB, 0.27397E+00_JPRB, 0.32123E+00_JPRB, 0.33712E+00_JPRB, 0.34831E+00_JPRB, 0.35824E+00_JPRB ,&
&  0.35170E+00_JPRB, 0.34306E+00_JPRB, 0.34817E+00_JPRB, 0.37046E+00_JPRB, 0.38086E+00_JPRB, 0.39456E+00_JPRB, 0.40873E+00_JPRB ,&
&  0.42657E+00_JPRB, 0.44773E+00_JPRB, 0.46933E+00_JPRB, 0.46193E+00_JPRB, 0.55686E+00_JPRB, 0.78174E+00_JPRB, 0.90235E+00_JPRB ,&
&  0.10623E+01_JPRB, 0.12125E+01_JPRB, 0.14211E+01_JPRB, 0.14758E+01_JPRB, 0.14552E+01_JPRB, 0.14513E+01_JPRB, 0.14928E+01_JPRB ,&
&  0.14691E+01_JPRB, 0.14430E+01_JPRB, 0.14048E+01_JPRB, 0.14014E+01_JPRB, 0.13576E+01_JPRB, 0.13058E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.12158E+01_JPRB, 0.11478E+01_JPRB, 0.10589E+01_JPRB, 0.10200E+01_JPRB, 0.10014E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.10092E+01_JPRB, 0.10202E+01_JPRB, 0.10215E+01_JPRB, 0.10314E+01_JPRB, 0.92573E+00_JPRB, 0.79711E+00_JPRB, 0.70461E+00_JPRB ,&
&  0.65646E+00_JPRB, 0.61429E+00_JPRB, 0.56590E+00_JPRB, 0.57178E+00_JPRB, 0.57902E+00_JPRB, 0.56436E+00_JPRB, 0.49981E+00_JPRB ,&
&  0.44957E+00_JPRB, 0.42920E+00_JPRB, 0.41875E+00_JPRB, 0.43009E+00_JPRB, 0.43800E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.38288E+00_JPRB, 0.38986E+00_JPRB, 0.39907E+00_JPRB, 0.39960E+00_JPRB, 0.39716E+00_JPRB, 0.37134E+00_JPRB, 0.34666E+00_JPRB ,&
&  0.33184E+00_JPRB, 0.32577E+00_JPRB, 0.32658E+00_JPRB, 0.32796E+00_JPRB, 0.32920E+00_JPRB, 0.33161E+00_JPRB, 0.33388E+00_JPRB ,&
&  0.33237E+00_JPRB, 0.32858E+00_JPRB, 0.32539E+00_JPRB, 0.32437E+00_JPRB, 0.32755E+00_JPRB, 0.33639E+00_JPRB, 0.35061E+00_JPRB ,&
&  0.36958E+00_JPRB, 0.38445E+00_JPRB, 0.38851E+00_JPRB, 0.38418E+00_JPRB, 0.37702E+00_JPRB, 0.37208E+00_JPRB, 0.36411E+00_JPRB ,&
&  0.35107E+00_JPRB, 0.34694E+00_JPRB, 0.35089E+00_JPRB, 0.36336E+00_JPRB, 0.38007E+00_JPRB, 0.38193E+00_JPRB, 0.32929E+00_JPRB ,&
&  0.27448E+00_JPRB, 0.24184E+00_JPRB, 0.22929E+00_JPRB, 0.23709E+00_JPRB, 0.26332E+00_JPRB, 0.30943E+00_JPRB, 0.32328E+00_JPRB ,&
&  0.33614E+00_JPRB, 0.34981E+00_JPRB, 0.36043E+00_JPRB, 0.37046E+00_JPRB, 0.37872E+00_JPRB, 0.38481E+00_JPRB, 0.38977E+00_JPRB ,&
&  0.39651E+00_JPRB, 0.40784E+00_JPRB, 0.42620E+00_JPRB, 0.41943E+00_JPRB, 0.44586E+00_JPRB, 0.57865E+00_JPRB, 0.75854E+00_JPRB ,&
&  0.93983E+00_JPRB, 0.10302E+01_JPRB, 0.11425E+01_JPRB, 0.13131E+01_JPRB, 0.14011E+01_JPRB, 0.14598E+01_JPRB, 0.14777E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.13822E+01_JPRB, 0.13224E+01_JPRB, 0.13000E+01_JPRB, 0.12976E+01_JPRB, 0.13015E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.11656E+01_JPRB, 0.10695E+01_JPRB, 0.97759E+00_JPRB, 0.95706E+00_JPRB, 0.98270E+00_JPRB, 0.10295E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10231E+01_JPRB, 0.10126E+01_JPRB, 0.10245E+01_JPRB, 0.89870E+00_JPRB, 0.81404E+00_JPRB, 0.75000E+00_JPRB ,&
&  0.72771E+00_JPRB, 0.70992E+00_JPRB, 0.68263E+00_JPRB, 0.69872E+00_JPRB, 0.68166E+00_JPRB, 0.64786E+00_JPRB, 0.58983E+00_JPRB ,&
&  0.52214E+00_JPRB, 0.46954E+00_JPRB, 0.43518E+00_JPRB, 0.40717E+00_JPRB, 0.39271E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.41302E+00_JPRB, 0.39894E+00_JPRB, 0.39217E+00_JPRB, 0.38986E+00_JPRB, 0.38794E+00_JPRB, 0.37597E+00_JPRB, 0.36522E+00_JPRB ,&
&  0.35829E+00_JPRB, 0.35694E+00_JPRB, 0.35576E+00_JPRB, 0.35245E+00_JPRB, 0.35014E+00_JPRB, 0.34804E+00_JPRB, 0.34620E+00_JPRB ,&
&  0.34440E+00_JPRB, 0.34383E+00_JPRB, 0.34521E+00_JPRB, 0.34589E+00_JPRB, 0.35086E+00_JPRB, 0.36274E+00_JPRB, 0.38142E+00_JPRB ,&
&  0.40117E+00_JPRB, 0.40797E+00_JPRB, 0.40367E+00_JPRB, 0.39398E+00_JPRB, 0.37702E+00_JPRB, 0.36290E+00_JPRB, 0.35180E+00_JPRB ,&
&  0.33946E+00_JPRB, 0.33633E+00_JPRB, 0.34471E+00_JPRB, 0.35333E+00_JPRB, 0.36278E+00_JPRB, 0.35426E+00_JPRB, 0.30352E+00_JPRB ,&
&  0.25800E+00_JPRB, 0.22634E+00_JPRB, 0.21113E+00_JPRB, 0.20784E+00_JPRB, 0.21642E+00_JPRB, 0.23714E+00_JPRB, 0.27197E+00_JPRB ,&
&  0.33602E+00_JPRB, 0.37744E+00_JPRB, 0.39345E+00_JPRB, 0.41168E+00_JPRB, 0.41542E+00_JPRB, 0.41445E+00_JPRB, 0.41417E+00_JPRB ,&
&  0.42255E+00_JPRB, 0.43757E+00_JPRB, 0.46016E+00_JPRB, 0.48225E+00_JPRB, 0.51689E+00_JPRB, 0.56663E+00_JPRB, 0.67339E+00_JPRB ,&
&  0.91704E+00_JPRB, 0.99451E+00_JPRB, 0.95458E+00_JPRB, 0.10813E+01_JPRB, 0.12559E+01_JPRB, 0.13945E+01_JPRB, 0.14861E+01_JPRB ,&
&  0.14772E+01_JPRB, 0.13983E+01_JPRB, 0.13360E+01_JPRB, 0.12971E+01_JPRB, 0.12756E+01_JPRB, 0.12640E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.13416E+01_JPRB, 0.14339E+01_JPRB, 0.12249E+01_JPRB, 0.11347E+01_JPRB, 0.11098E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10096E+01_JPRB, 0.10091E+01_JPRB, 0.10134E+01_JPRB, 0.94724E+00_JPRB, 0.88887E+00_JPRB, 0.83963E+00_JPRB ,&
&  0.82203E+00_JPRB, 0.79661E+00_JPRB, 0.77167E+00_JPRB, 0.76258E+00_JPRB, 0.74125E+00_JPRB, 0.69958E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.62717E+00_JPRB, 0.56216E+00_JPRB, 0.51975E+00_JPRB, 0.48206E+00_JPRB, 0.44241E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.42605E+00_JPRB, 0.40984E+00_JPRB, 0.39842E+00_JPRB, 0.39113E+00_JPRB, 0.38453E+00_JPRB, 0.38002E+00_JPRB, 0.37389E+00_JPRB ,&
&  0.36857E+00_JPRB, 0.36375E+00_JPRB, 0.36011E+00_JPRB, 0.35691E+00_JPRB, 0.35537E+00_JPRB, 0.35475E+00_JPRB, 0.35568E+00_JPRB ,&
&  0.35775E+00_JPRB, 0.35685E+00_JPRB, 0.36071E+00_JPRB, 0.37048E+00_JPRB, 0.38547E+00_JPRB, 0.39958E+00_JPRB, 0.41542E+00_JPRB ,&
&  0.41957E+00_JPRB, 0.41428E+00_JPRB, 0.40201E+00_JPRB, 0.38482E+00_JPRB, 0.36865E+00_JPRB, 0.35512E+00_JPRB, 0.34696E+00_JPRB ,&
&  0.34569E+00_JPRB, 0.33943E+00_JPRB, 0.34278E+00_JPRB, 0.34312E+00_JPRB, 0.33342E+00_JPRB, 0.32947E+00_JPRB, 0.27671E+00_JPRB ,&
&  0.23792E+00_JPRB, 0.20930E+00_JPRB, 0.20045E+00_JPRB, 0.19663E+00_JPRB, 0.20202E+00_JPRB, 0.21469E+00_JPRB, 0.24448E+00_JPRB ,&
&  0.30237E+00_JPRB, 0.40949E+00_JPRB, 0.42953E+00_JPRB, 0.43994E+00_JPRB, 0.44593E+00_JPRB, 0.44643E+00_JPRB, 0.44704E+00_JPRB ,&
&  0.46311E+00_JPRB, 0.48072E+00_JPRB, 0.50961E+00_JPRB, 0.54156E+00_JPRB, 0.58148E+00_JPRB, 0.61822E+00_JPRB, 0.65030E+00_JPRB ,&
&  0.70726E+00_JPRB, 0.75790E+00_JPRB, 0.81462E+00_JPRB, 0.92047E+00_JPRB, 0.10970E+01_JPRB, 0.12780E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.13219E+01_JPRB, 0.12769E+01_JPRB, 0.12434E+01_JPRB, 0.12262E+01_JPRB, 0.12176E+01_JPRB, 0.12167E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.12632E+01_JPRB, 0.13543E+01_JPRB, 0.12747E+01_JPRB, 0.12291E+01_JPRB, 0.11806E+01_JPRB, 0.11244E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10517E+01_JPRB, 0.10330E+01_JPRB, 0.10070E+01_JPRB, 0.97664E+00_JPRB, 0.94836E+00_JPRB, 0.92798E+00_JPRB ,&
&  0.89978E+00_JPRB, 0.86554E+00_JPRB, 0.81938E+00_JPRB, 0.78795E+00_JPRB, 0.76158E+00_JPRB, 0.73850E+00_JPRB, 0.71273E+00_JPRB ,&
&  0.66119E+00_JPRB, 0.58942E+00_JPRB, 0.53390E+00_JPRB, 0.48908E+00_JPRB, 0.45239E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.46352E+00_JPRB, 0.44301E+00_JPRB, 0.42532E+00_JPRB, 0.41418E+00_JPRB, 0.40314E+00_JPRB, 0.39721E+00_JPRB, 0.39054E+00_JPRB ,&
&  0.38713E+00_JPRB, 0.38410E+00_JPRB, 0.38296E+00_JPRB, 0.38264E+00_JPRB, 0.38402E+00_JPRB, 0.38656E+00_JPRB, 0.39096E+00_JPRB ,&
&  0.39738E+00_JPRB, 0.40552E+00_JPRB, 0.41637E+00_JPRB, 0.42417E+00_JPRB, 0.43087E+00_JPRB, 0.43428E+00_JPRB, 0.43806E+00_JPRB ,&
&  0.43517E+00_JPRB, 0.43149E+00_JPRB, 0.41000E+00_JPRB, 0.38681E+00_JPRB, 0.36811E+00_JPRB, 0.35059E+00_JPRB, 0.34675E+00_JPRB ,&
&  0.34344E+00_JPRB, 0.33714E+00_JPRB, 0.32742E+00_JPRB, 0.32241E+00_JPRB, 0.31503E+00_JPRB, 0.26758E+00_JPRB, 0.25455E+00_JPRB ,&
&  0.23255E+00_JPRB, 0.20954E+00_JPRB, 0.20125E+00_JPRB, 0.19674E+00_JPRB, 0.19857E+00_JPRB, 0.20897E+00_JPRB, 0.22849E+00_JPRB ,&
&  0.27819E+00_JPRB, 0.41926E+00_JPRB, 0.45195E+00_JPRB, 0.46606E+00_JPRB, 0.45279E+00_JPRB, 0.45846E+00_JPRB, 0.46498E+00_JPRB ,&
&  0.48858E+00_JPRB, 0.51209E+00_JPRB, 0.54356E+00_JPRB, 0.57185E+00_JPRB, 0.60140E+00_JPRB, 0.61917E+00_JPRB, 0.64889E+00_JPRB ,&
&  0.69369E+00_JPRB, 0.75365E+00_JPRB, 0.81078E+00_JPRB, 0.87174E+00_JPRB, 0.93083E+00_JPRB, 0.10459E+01_JPRB, 0.11690E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.11713E+01_JPRB, 0.11723E+01_JPRB, 0.11660E+01_JPRB, 0.11630E+01_JPRB, 0.11580E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.11768E+01_JPRB, 0.11921E+01_JPRB, 0.11614E+01_JPRB, 0.11094E+01_JPRB, 0.10806E+01_JPRB, 0.10398E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.10309E+01_JPRB, 0.10155E+01_JPRB, 0.98792E+00_JPRB, 0.96144E+00_JPRB, 0.93018E+00_JPRB, 0.90453E+00_JPRB ,&
&  0.87690E+00_JPRB, 0.84410E+00_JPRB, 0.80944E+00_JPRB, 0.78169E+00_JPRB, 0.75997E+00_JPRB, 0.74304E+00_JPRB, 0.71794E+00_JPRB ,&
&  0.67276E+00_JPRB, 0.61487E+00_JPRB, 0.56718E+00_JPRB, 0.52356E+00_JPRB, 0.48878E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.51474E+00_JPRB, 0.49291E+00_JPRB, 0.47460E+00_JPRB, 0.46132E+00_JPRB, 0.44917E+00_JPRB, 0.44104E+00_JPRB, 0.43351E+00_JPRB ,&
&  0.42966E+00_JPRB, 0.42592E+00_JPRB, 0.42528E+00_JPRB, 0.42488E+00_JPRB, 0.42673E+00_JPRB, 0.43003E+00_JPRB, 0.43431E+00_JPRB ,&
&  0.44012E+00_JPRB, 0.44477E+00_JPRB, 0.44971E+00_JPRB, 0.45151E+00_JPRB, 0.45449E+00_JPRB, 0.45498E+00_JPRB, 0.45820E+00_JPRB ,&
&  0.45337E+00_JPRB, 0.44625E+00_JPRB, 0.41951E+00_JPRB, 0.39498E+00_JPRB, 0.36615E+00_JPRB, 0.34582E+00_JPRB, 0.33206E+00_JPRB ,&
&  0.33388E+00_JPRB, 0.30674E+00_JPRB, 0.30240E+00_JPRB, 0.28266E+00_JPRB, 0.26349E+00_JPRB, 0.24782E+00_JPRB, 0.23417E+00_JPRB ,&
&  0.22492E+00_JPRB, 0.21182E+00_JPRB, 0.20611E+00_JPRB, 0.20586E+00_JPRB, 0.20943E+00_JPRB, 0.22549E+00_JPRB, 0.25022E+00_JPRB ,&
&  0.30499E+00_JPRB, 0.43343E+00_JPRB, 0.47501E+00_JPRB, 0.49528E+00_JPRB, 0.49306E+00_JPRB, 0.50144E+00_JPRB, 0.50293E+00_JPRB ,&
&  0.53287E+00_JPRB, 0.57168E+00_JPRB, 0.61284E+00_JPRB, 0.64254E+00_JPRB, 0.67721E+00_JPRB, 0.69946E+00_JPRB, 0.72070E+00_JPRB ,&
&  0.74354E+00_JPRB, 0.77708E+00_JPRB, 0.80680E+00_JPRB, 0.83328E+00_JPRB, 0.86403E+00_JPRB, 0.89125E+00_JPRB, 0.92613E+00_JPRB ,&
&  0.94837E+00_JPRB, 0.99160E+00_JPRB, 0.10053E+01_JPRB, 0.10220E+01_JPRB, 0.10230E+01_JPRB, 0.10177E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.10279E+01_JPRB, 0.10300E+01_JPRB, 0.10195E+01_JPRB, 0.10120E+01_JPRB, 0.98862E+00_JPRB, 0.97637E+00_JPRB ,&
&  0.97038E+00_JPRB, 0.96239E+00_JPRB, 0.95130E+00_JPRB, 0.93539E+00_JPRB, 0.91541E+00_JPRB, 0.89275E+00_JPRB, 0.87676E+00_JPRB ,&
&  0.85610E+00_JPRB, 0.83160E+00_JPRB, 0.80450E+00_JPRB, 0.78145E+00_JPRB, 0.76151E+00_JPRB, 0.74066E+00_JPRB, 0.70665E+00_JPRB ,&
&  0.66522E+00_JPRB, 0.62917E+00_JPRB, 0.59601E+00_JPRB, 0.56884E+00_JPRB, 0.54016E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.57283E+00_JPRB, 0.55624E+00_JPRB, 0.53861E+00_JPRB, 0.52334E+00_JPRB, 0.50893E+00_JPRB, 0.49802E+00_JPRB, 0.48777E+00_JPRB ,&
&  0.48037E+00_JPRB, 0.47388E+00_JPRB, 0.46975E+00_JPRB, 0.46724E+00_JPRB, 0.46599E+00_JPRB, 0.46724E+00_JPRB, 0.46821E+00_JPRB ,&
&  0.47075E+00_JPRB, 0.47264E+00_JPRB, 0.47613E+00_JPRB, 0.47905E+00_JPRB, 0.48322E+00_JPRB, 0.48325E+00_JPRB, 0.48298E+00_JPRB ,&
&  0.47679E+00_JPRB, 0.46776E+00_JPRB, 0.43114E+00_JPRB, 0.40775E+00_JPRB, 0.37907E+00_JPRB, 0.35688E+00_JPRB, 0.34585E+00_JPRB ,&
&  0.33321E+00_JPRB, 0.32612E+00_JPRB, 0.31384E+00_JPRB, 0.30132E+00_JPRB, 0.28937E+00_JPRB, 0.27453E+00_JPRB, 0.26356E+00_JPRB ,&
&  0.25404E+00_JPRB, 0.24379E+00_JPRB, 0.23934E+00_JPRB, 0.23999E+00_JPRB, 0.24651E+00_JPRB, 0.26501E+00_JPRB, 0.29440E+00_JPRB ,&
&  0.35023E+00_JPRB, 0.41321E+00_JPRB, 0.49838E+00_JPRB, 0.52629E+00_JPRB, 0.53197E+00_JPRB, 0.55499E+00_JPRB, 0.57572E+00_JPRB ,&
&  0.59051E+00_JPRB, 0.61939E+00_JPRB, 0.64908E+00_JPRB, 0.66500E+00_JPRB, 0.67717E+00_JPRB, 0.68475E+00_JPRB, 0.69299E+00_JPRB ,&
&  0.70054E+00_JPRB, 0.71416E+00_JPRB, 0.72238E+00_JPRB, 0.72855E+00_JPRB, 0.73520E+00_JPRB, 0.74177E+00_JPRB, 0.75584E+00_JPRB ,&
&  0.76736E+00_JPRB, 0.79102E+00_JPRB, 0.81012E+00_JPRB, 0.84151E+00_JPRB, 0.85375E+00_JPRB, 0.86924E+00_JPRB, 0.87262E+00_JPRB ,&
&  0.87876E+00_JPRB, 0.88024E+00_JPRB, 0.87899E+00_JPRB, 0.86481E+00_JPRB, 0.86113E+00_JPRB, 0.86630E+00_JPRB, 0.86317E+00_JPRB ,&
&  0.85651E+00_JPRB, 0.84935E+00_JPRB, 0.84104E+00_JPRB, 0.83123E+00_JPRB, 0.82250E+00_JPRB, 0.81669E+00_JPRB, 0.80852E+00_JPRB ,&
&  0.80289E+00_JPRB, 0.78849E+00_JPRB, 0.77431E+00_JPRB, 0.75825E+00_JPRB, 0.74035E+00_JPRB, 0.71540E+00_JPRB, 0.68418E+00_JPRB ,&
&  0.65733E+00_JPRB, 0.63753E+00_JPRB, 0.61791E+00_JPRB, 0.60328E+00_JPRB, 0.58779E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.57852E+00_JPRB, 0.57271E+00_JPRB, 0.56389E+00_JPRB, 0.55269E+00_JPRB, 0.54249E+00_JPRB, 0.53462E+00_JPRB, 0.52649E+00_JPRB ,&
&  0.51988E+00_JPRB, 0.51515E+00_JPRB, 0.51167E+00_JPRB, 0.50916E+00_JPRB, 0.50683E+00_JPRB, 0.50768E+00_JPRB, 0.50888E+00_JPRB ,&
&  0.51031E+00_JPRB, 0.51050E+00_JPRB, 0.51257E+00_JPRB, 0.51272E+00_JPRB, 0.51168E+00_JPRB, 0.50816E+00_JPRB, 0.50458E+00_JPRB ,&
&  0.49745E+00_JPRB, 0.48902E+00_JPRB, 0.47895E+00_JPRB, 0.46641E+00_JPRB, 0.45497E+00_JPRB, 0.41301E+00_JPRB, 0.39987E+00_JPRB ,&
&  0.39096E+00_JPRB, 0.38623E+00_JPRB, 0.38521E+00_JPRB, 0.38727E+00_JPRB, 0.40000E+00_JPRB, 0.39631E+00_JPRB, 0.39084E+00_JPRB ,&
&  0.36182E+00_JPRB, 0.35244E+00_JPRB, 0.34822E+00_JPRB, 0.34736E+00_JPRB, 0.35447E+00_JPRB, 0.37765E+00_JPRB, 0.40846E+00_JPRB ,&
&  0.46190E+00_JPRB, 0.48787E+00_JPRB, 0.50922E+00_JPRB, 0.53249E+00_JPRB, 0.55266E+00_JPRB, 0.56989E+00_JPRB, 0.58592E+00_JPRB ,&
&  0.60048E+00_JPRB, 0.61505E+00_JPRB, 0.62663E+00_JPRB, 0.63399E+00_JPRB, 0.63742E+00_JPRB, 0.64006E+00_JPRB, 0.64218E+00_JPRB ,&
&  0.64343E+00_JPRB, 0.64350E+00_JPRB, 0.64415E+00_JPRB, 0.64471E+00_JPRB, 0.64350E+00_JPRB, 0.64513E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.65856E+00_JPRB, 0.66993E+00_JPRB, 0.67925E+00_JPRB, 0.69393E+00_JPRB, 0.70835E+00_JPRB, 0.72804E+00_JPRB, 0.73502E+00_JPRB ,&
&  0.74503E+00_JPRB, 0.75115E+00_JPRB, 0.75994E+00_JPRB, 0.75924E+00_JPRB, 0.75899E+00_JPRB, 0.75674E+00_JPRB, 0.75640E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.74274E+00_JPRB, 0.73489E+00_JPRB, 0.73094E+00_JPRB, 0.72087E+00_JPRB, 0.71503E+00_JPRB, 0.70674E+00_JPRB ,&
&  0.70343E+00_JPRB, 0.69094E+00_JPRB, 0.68325E+00_JPRB, 0.67025E+00_JPRB, 0.65886E+00_JPRB, 0.64020E+00_JPRB, 0.62394E+00_JPRB ,&
&  0.60987E+00_JPRB, 0.60045E+00_JPRB, 0.59303E+00_JPRB, 0.58762E+00_JPRB, 0.58256E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.57079E+00_JPRB, 0.56739E+00_JPRB, 0.56425E+00_JPRB, 0.56073E+00_JPRB, 0.55714E+00_JPRB, 0.55417E+00_JPRB, 0.55104E+00_JPRB ,&
&  0.54756E+00_JPRB, 0.54471E+00_JPRB, 0.54202E+00_JPRB, 0.53948E+00_JPRB, 0.53708E+00_JPRB, 0.53607E+00_JPRB, 0.53490E+00_JPRB ,&
&  0.53315E+00_JPRB, 0.53059E+00_JPRB, 0.52852E+00_JPRB, 0.52563E+00_JPRB, 0.52225E+00_JPRB, 0.51776E+00_JPRB, 0.51346E+00_JPRB ,&
&  0.50848E+00_JPRB, 0.50246E+00_JPRB, 0.49734E+00_JPRB, 0.49448E+00_JPRB, 0.49136E+00_JPRB, 0.48700E+00_JPRB, 0.48407E+00_JPRB ,&
&  0.48272E+00_JPRB, 0.48119E+00_JPRB, 0.47984E+00_JPRB, 0.47841E+00_JPRB, 0.47544E+00_JPRB, 0.47459E+00_JPRB, 0.47402E+00_JPRB ,&
&  0.47280E+00_JPRB, 0.47319E+00_JPRB, 0.47623E+00_JPRB, 0.47970E+00_JPRB, 0.48432E+00_JPRB, 0.49110E+00_JPRB, 0.50200E+00_JPRB ,&
&  0.51432E+00_JPRB, 0.52567E+00_JPRB, 0.53706E+00_JPRB, 0.54844E+00_JPRB, 0.55785E+00_JPRB, 0.56563E+00_JPRB, 0.57245E+00_JPRB ,&
&  0.57729E+00_JPRB, 0.58087E+00_JPRB, 0.58281E+00_JPRB, 0.58291E+00_JPRB, 0.58192E+00_JPRB, 0.58154E+00_JPRB, 0.58158E+00_JPRB ,&
&  0.58173E+00_JPRB, 0.57929E+00_JPRB, 0.57875E+00_JPRB, 0.58027E+00_JPRB, 0.58271E+00_JPRB, 0.58527E+00_JPRB, 0.58931E+00_JPRB ,&
&  0.59412E+00_JPRB, 0.59923E+00_JPRB, 0.60362E+00_JPRB, 0.60995E+00_JPRB, 0.61756E+00_JPRB, 0.62648E+00_JPRB, 0.63149E+00_JPRB ,&
&  0.63849E+00_JPRB, 0.64640E+00_JPRB, 0.65501E+00_JPRB, 0.65622E+00_JPRB, 0.65825E+00_JPRB, 0.65983E+00_JPRB, 0.66222E+00_JPRB ,&
&  0.65762E+00_JPRB, 0.65539E+00_JPRB, 0.65312E+00_JPRB, 0.65318E+00_JPRB, 0.64701E+00_JPRB, 0.64318E+00_JPRB, 0.63929E+00_JPRB ,&
&  0.63875E+00_JPRB, 0.63032E+00_JPRB, 0.62478E+00_JPRB, 0.61774E+00_JPRB, 0.61288E+00_JPRB, 0.60364E+00_JPRB, 0.59621E+00_JPRB ,&
&  0.58946E+00_JPRB, 0.58541E+00_JPRB, 0.58090E+00_JPRB, 0.57722E+00_JPRB, 0.57403E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.56291E+00_JPRB, 0.56119E+00_JPRB, 0.55934E+00_JPRB, 0.55770E+00_JPRB, 0.55605E+00_JPRB, 0.55478E+00_JPRB, 0.55357E+00_JPRB ,&
&  0.55187E+00_JPRB, 0.55023E+00_JPRB, 0.54866E+00_JPRB, 0.54707E+00_JPRB, 0.54536E+00_JPRB, 0.54372E+00_JPRB, 0.54209E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.53884E+00_JPRB, 0.53771E+00_JPRB, 0.53631E+00_JPRB, 0.53466E+00_JPRB, 0.53269E+00_JPRB, 0.53078E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.52645E+00_JPRB, 0.52401E+00_JPRB, 0.52173E+00_JPRB, 0.51942E+00_JPRB, 0.51645E+00_JPRB, 0.51383E+00_JPRB ,&
&  0.51221E+00_JPRB, 0.51086E+00_JPRB, 0.50978E+00_JPRB, 0.50853E+00_JPRB, 0.50795E+00_JPRB, 0.50811E+00_JPRB, 0.50835E+00_JPRB ,&
&  0.50877E+00_JPRB, 0.50963E+00_JPRB, 0.51120E+00_JPRB, 0.51298E+00_JPRB, 0.51489E+00_JPRB, 0.51749E+00_JPRB, 0.52109E+00_JPRB ,&
&  0.52439E+00_JPRB, 0.52751E+00_JPRB, 0.53103E+00_JPRB, 0.53475E+00_JPRB, 0.53848E+00_JPRB, 0.54206E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.54778E+00_JPRB, 0.54983E+00_JPRB, 0.55186E+00_JPRB, 0.55352E+00_JPRB, 0.55425E+00_JPRB, 0.55491E+00_JPRB, 0.55555E+00_JPRB ,&
&  0.55601E+00_JPRB, 0.55640E+00_JPRB, 0.55714E+00_JPRB, 0.55840E+00_JPRB, 0.55969E+00_JPRB, 0.56104E+00_JPRB, 0.56271E+00_JPRB ,&
&  0.56482E+00_JPRB, 0.56607E+00_JPRB, 0.56749E+00_JPRB, 0.56924E+00_JPRB, 0.57149E+00_JPRB, 0.57414E+00_JPRB, 0.57625E+00_JPRB ,&
&  0.57839E+00_JPRB, 0.58067E+00_JPRB, 0.58293E+00_JPRB, 0.58416E+00_JPRB, 0.58551E+00_JPRB, 0.58671E+00_JPRB, 0.58785E+00_JPRB ,&
&  0.58748E+00_JPRB, 0.58752E+00_JPRB, 0.58800E+00_JPRB, 0.58873E+00_JPRB, 0.58737E+00_JPRB, 0.58627E+00_JPRB, 0.58535E+00_JPRB ,&
&  0.58557E+00_JPRB, 0.58308E+00_JPRB, 0.58136E+00_JPRB, 0.57911E+00_JPRB, 0.57787E+00_JPRB, 0.57548E+00_JPRB, 0.57360E+00_JPRB ,&
&  0.57153E+00_JPRB, 0.56998E+00_JPRB, 0.56822E+00_JPRB, 0.56641E+00_JPRB, 0.56463E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB ,&
&  0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB, 0.54799E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB ,&
&  0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB, 0.23978E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.26274E+00_JPRB, 0.26536E+00_JPRB, 0.26763E+00_JPRB, 0.26933E+00_JPRB, 0.26979E+00_JPRB, 0.27163E+00_JPRB, 0.27215E+00_JPRB ,&
&  0.27247E+00_JPRB, 0.27245E+00_JPRB, 0.27323E+00_JPRB, 0.27404E+00_JPRB, 0.27458E+00_JPRB, 0.27485E+00_JPRB, 0.27489E+00_JPRB ,&
&  0.27443E+00_JPRB, 0.27350E+00_JPRB, 0.26999E+00_JPRB, 0.26805E+00_JPRB, 0.26695E+00_JPRB, 0.26649E+00_JPRB, 0.26628E+00_JPRB ,&
&  0.26564E+00_JPRB, 0.26484E+00_JPRB, 0.26349E+00_JPRB, 0.26326E+00_JPRB, 0.26245E+00_JPRB, 0.26140E+00_JPRB, 0.25993E+00_JPRB ,&
&  0.25848E+00_JPRB, 0.25732E+00_JPRB, 0.25588E+00_JPRB, 0.25439E+00_JPRB, 0.25256E+00_JPRB, 0.25037E+00_JPRB, 0.24766E+00_JPRB ,&
&  0.24485E+00_JPRB, 0.24198E+00_JPRB, 0.23962E+00_JPRB, 0.23741E+00_JPRB, 0.23551E+00_JPRB, 0.23400E+00_JPRB, 0.23294E+00_JPRB ,&
&  0.23177E+00_JPRB, 0.23045E+00_JPRB, 0.22912E+00_JPRB, 0.22800E+00_JPRB, 0.22690E+00_JPRB, 0.22594E+00_JPRB, 0.22548E+00_JPRB ,&
&  0.22521E+00_JPRB, 0.22488E+00_JPRB, 0.22448E+00_JPRB, 0.22413E+00_JPRB, 0.22368E+00_JPRB, 0.22326E+00_JPRB, 0.22284E+00_JPRB ,&
&  0.22249E+00_JPRB, 0.22227E+00_JPRB, 0.22192E+00_JPRB, 0.22156E+00_JPRB, 0.22133E+00_JPRB, 0.22095E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.22002E+00_JPRB, 0.21971E+00_JPRB, 0.21937E+00_JPRB, 0.21895E+00_JPRB, 0.21856E+00_JPRB, 0.21829E+00_JPRB, 0.21829E+00_JPRB ,&
&  0.21824E+00_JPRB, 0.21813E+00_JPRB, 0.21818E+00_JPRB, 0.21888E+00_JPRB, 0.21961E+00_JPRB, 0.22050E+00_JPRB, 0.22150E+00_JPRB ,&
&  0.22281E+00_JPRB, 0.22418E+00_JPRB, 0.22571E+00_JPRB, 0.22717E+00_JPRB, 0.22910E+00_JPRB, 0.23102E+00_JPRB, 0.23293E+00_JPRB ,&
&  0.23475E+00_JPRB, 0.23663E+00_JPRB, 0.23851E+00_JPRB, 0.24039E+00_JPRB, 0.24283E+00_JPRB, 0.24498E+00_JPRB, 0.24741E+00_JPRB ,&
&  0.24972E+00_JPRB, 0.25224E+00_JPRB, 0.25486E+00_JPRB, 0.25714E+00_JPRB, 0.25952E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.30527E+00_JPRB, 0.31568E+00_JPRB, 0.32457E+00_JPRB, 0.33322E+00_JPRB, 0.33673E+00_JPRB, 0.40149E+00_JPRB, 0.38208E+00_JPRB ,&
&  0.37268E+00_JPRB, 0.34442E+00_JPRB, 0.34523E+00_JPRB, 0.34074E+00_JPRB, 0.33413E+00_JPRB, 0.32653E+00_JPRB, 0.32008E+00_JPRB ,&
&  0.31372E+00_JPRB, 0.30590E+00_JPRB, 0.30086E+00_JPRB, 0.29479E+00_JPRB, 0.28903E+00_JPRB, 0.28333E+00_JPRB, 0.27997E+00_JPRB ,&
&  0.27745E+00_JPRB, 0.27598E+00_JPRB, 0.27488E+00_JPRB, 0.27417E+00_JPRB, 0.27680E+00_JPRB, 0.27822E+00_JPRB, 0.27653E+00_JPRB ,&
&  0.27306E+00_JPRB, 0.26976E+00_JPRB, 0.26557E+00_JPRB, 0.26126E+00_JPRB, 0.25811E+00_JPRB, 0.25533E+00_JPRB, 0.25247E+00_JPRB ,&
&  0.24871E+00_JPRB, 0.24451E+00_JPRB, 0.24032E+00_JPRB, 0.23703E+00_JPRB, 0.23396E+00_JPRB, 0.23085E+00_JPRB, 0.22797E+00_JPRB ,&
&  0.22502E+00_JPRB, 0.22210E+00_JPRB, 0.21917E+00_JPRB, 0.21724E+00_JPRB, 0.21562E+00_JPRB, 0.21410E+00_JPRB, 0.21273E+00_JPRB ,&
&  0.21213E+00_JPRB, 0.21141E+00_JPRB, 0.21052E+00_JPRB, 0.20942E+00_JPRB, 0.20836E+00_JPRB, 0.20721E+00_JPRB, 0.20627E+00_JPRB ,&
&  0.20610E+00_JPRB, 0.20552E+00_JPRB, 0.20491E+00_JPRB, 0.20413E+00_JPRB, 0.20375E+00_JPRB, 0.20299E+00_JPRB, 0.20204E+00_JPRB ,&
&  0.20116E+00_JPRB, 0.20036E+00_JPRB, 0.20016E+00_JPRB, 0.19989E+00_JPRB, 0.19960E+00_JPRB, 0.19940E+00_JPRB, 0.19977E+00_JPRB ,&
&  0.20023E+00_JPRB, 0.20080E+00_JPRB, 0.20175E+00_JPRB, 0.20291E+00_JPRB, 0.20392E+00_JPRB, 0.20492E+00_JPRB, 0.20631E+00_JPRB ,&
&  0.20820E+00_JPRB, 0.21010E+00_JPRB, 0.21221E+00_JPRB, 0.21420E+00_JPRB, 0.21752E+00_JPRB, 0.22109E+00_JPRB, 0.22500E+00_JPRB ,&
&  0.22982E+00_JPRB, 0.23639E+00_JPRB, 0.24386E+00_JPRB, 0.25053E+00_JPRB, 0.25676E+00_JPRB, 0.26391E+00_JPRB, 0.26978E+00_JPRB ,&
&  0.27494E+00_JPRB, 0.28002E+00_JPRB, 0.28584E+00_JPRB, 0.29207E+00_JPRB, 0.29886E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.43174E+00_JPRB, 0.43271E+00_JPRB, 0.43316E+00_JPRB, 0.42967E+00_JPRB, 0.42831E+00_JPRB, 0.42826E+00_JPRB, 0.42549E+00_JPRB ,&
&  0.41759E+00_JPRB, 0.41508E+00_JPRB, 0.41022E+00_JPRB, 0.40325E+00_JPRB, 0.39645E+00_JPRB, 0.38547E+00_JPRB, 0.36922E+00_JPRB ,&
&  0.34742E+00_JPRB, 0.32828E+00_JPRB, 0.31375E+00_JPRB, 0.30047E+00_JPRB, 0.28744E+00_JPRB, 0.27475E+00_JPRB, 0.26945E+00_JPRB ,&
&  0.26493E+00_JPRB, 0.26294E+00_JPRB, 0.26226E+00_JPRB, 0.26581E+00_JPRB, 0.27342E+00_JPRB, 0.28294E+00_JPRB, 0.29331E+00_JPRB ,&
&  0.30524E+00_JPRB, 0.31636E+00_JPRB, 0.31783E+00_JPRB, 0.31080E+00_JPRB, 0.30391E+00_JPRB, 0.29956E+00_JPRB, 0.29622E+00_JPRB ,&
&  0.29231E+00_JPRB, 0.28393E+00_JPRB, 0.27451E+00_JPRB, 0.26466E+00_JPRB, 0.25475E+00_JPRB, 0.24753E+00_JPRB, 0.23905E+00_JPRB ,&
&  0.23288E+00_JPRB, 0.22735E+00_JPRB, 0.22153E+00_JPRB, 0.21654E+00_JPRB, 0.21313E+00_JPRB, 0.21028E+00_JPRB, 0.20743E+00_JPRB ,&
&  0.20592E+00_JPRB, 0.20443E+00_JPRB, 0.20275E+00_JPRB, 0.20108E+00_JPRB, 0.19905E+00_JPRB, 0.19747E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19489E+00_JPRB, 0.19393E+00_JPRB, 0.19334E+00_JPRB, 0.19274E+00_JPRB, 0.19227E+00_JPRB, 0.19135E+00_JPRB, 0.19025E+00_JPRB ,&
&  0.18952E+00_JPRB, 0.18915E+00_JPRB, 0.18839E+00_JPRB, 0.18794E+00_JPRB, 0.18760E+00_JPRB, 0.18688E+00_JPRB, 0.18729E+00_JPRB ,&
&  0.18835E+00_JPRB, 0.18956E+00_JPRB, 0.18989E+00_JPRB, 0.19095E+00_JPRB, 0.19224E+00_JPRB, 0.19385E+00_JPRB, 0.19633E+00_JPRB ,&
&  0.19825E+00_JPRB, 0.20025E+00_JPRB, 0.20272E+00_JPRB, 0.20492E+00_JPRB, 0.20828E+00_JPRB, 0.21240E+00_JPRB, 0.21747E+00_JPRB ,&
&  0.22441E+00_JPRB, 0.23277E+00_JPRB, 0.24432E+00_JPRB, 0.25836E+00_JPRB, 0.27829E+00_JPRB, 0.28668E+00_JPRB, 0.29506E+00_JPRB ,&
&  0.30232E+00_JPRB, 0.31672E+00_JPRB, 0.39888E+00_JPRB, 0.41389E+00_JPRB, 0.42872E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.43160E+00_JPRB, 0.44377E+00_JPRB, 0.46124E+00_JPRB, 0.47201E+00_JPRB, 0.48640E+00_JPRB, 0.48923E+00_JPRB, 0.48417E+00_JPRB ,&
&  0.46516E+00_JPRB, 0.43514E+00_JPRB, 0.41847E+00_JPRB, 0.39810E+00_JPRB, 0.37684E+00_JPRB, 0.35556E+00_JPRB, 0.34614E+00_JPRB ,&
&  0.33486E+00_JPRB, 0.32229E+00_JPRB, 0.30932E+00_JPRB, 0.29822E+00_JPRB, 0.28758E+00_JPRB, 0.27767E+00_JPRB, 0.27182E+00_JPRB ,&
&  0.26583E+00_JPRB, 0.26513E+00_JPRB, 0.26444E+00_JPRB, 0.26857E+00_JPRB, 0.27965E+00_JPRB, 0.29866E+00_JPRB, 0.32451E+00_JPRB ,&
&  0.36355E+00_JPRB, 0.36994E+00_JPRB, 0.35512E+00_JPRB, 0.34876E+00_JPRB, 0.34147E+00_JPRB, 0.33506E+00_JPRB, 0.32090E+00_JPRB ,&
&  0.32811E+00_JPRB, 0.34110E+00_JPRB, 0.35688E+00_JPRB, 0.34668E+00_JPRB, 0.30157E+00_JPRB, 0.28112E+00_JPRB, 0.25763E+00_JPRB ,&
&  0.24500E+00_JPRB, 0.23478E+00_JPRB, 0.22686E+00_JPRB, 0.21947E+00_JPRB, 0.21214E+00_JPRB, 0.20755E+00_JPRB, 0.20249E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19706E+00_JPRB, 0.19463E+00_JPRB, 0.19441E+00_JPRB, 0.19412E+00_JPRB, 0.19327E+00_JPRB, 0.19239E+00_JPRB ,&
&  0.19103E+00_JPRB, 0.18959E+00_JPRB, 0.18885E+00_JPRB, 0.18931E+00_JPRB, 0.19019E+00_JPRB, 0.19105E+00_JPRB, 0.18929E+00_JPRB ,&
&  0.18802E+00_JPRB, 0.18675E+00_JPRB, 0.18658E+00_JPRB, 0.18759E+00_JPRB, 0.18790E+00_JPRB, 0.18857E+00_JPRB, 0.18864E+00_JPRB ,&
&  0.18868E+00_JPRB, 0.18915E+00_JPRB, 0.18884E+00_JPRB, 0.18949E+00_JPRB, 0.18979E+00_JPRB, 0.19088E+00_JPRB, 0.19171E+00_JPRB ,&
&  0.19261E+00_JPRB, 0.19455E+00_JPRB, 0.19660E+00_JPRB, 0.19947E+00_JPRB, 0.20338E+00_JPRB, 0.20614E+00_JPRB, 0.21051E+00_JPRB ,&
&  0.21587E+00_JPRB, 0.22447E+00_JPRB, 0.24031E+00_JPRB, 0.25810E+00_JPRB, 0.27999E+00_JPRB, 0.29169E+00_JPRB, 0.30100E+00_JPRB ,&
&  0.31533E+00_JPRB, 0.35000E+00_JPRB, 0.40045E+00_JPRB, 0.42029E+00_JPRB, 0.42478E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.48874E+00_JPRB, 0.49624E+00_JPRB, 0.50757E+00_JPRB, 0.51230E+00_JPRB, 0.52055E+00_JPRB, 0.50895E+00_JPRB, 0.50210E+00_JPRB ,&
&  0.47197E+00_JPRB, 0.44940E+00_JPRB, 0.41823E+00_JPRB, 0.37885E+00_JPRB, 0.35251E+00_JPRB, 0.33208E+00_JPRB, 0.32523E+00_JPRB ,&
&  0.31828E+00_JPRB, 0.31377E+00_JPRB, 0.30961E+00_JPRB, 0.30801E+00_JPRB, 0.30726E+00_JPRB, 0.30856E+00_JPRB, 0.30857E+00_JPRB ,&
&  0.30515E+00_JPRB, 0.29952E+00_JPRB, 0.29296E+00_JPRB, 0.29608E+00_JPRB, 0.30899E+00_JPRB, 0.32405E+00_JPRB, 0.34164E+00_JPRB ,&
&  0.36187E+00_JPRB, 0.37286E+00_JPRB, 0.38307E+00_JPRB, 0.38727E+00_JPRB, 0.37196E+00_JPRB, 0.37452E+00_JPRB, 0.37704E+00_JPRB ,&
&  0.38507E+00_JPRB, 0.39237E+00_JPRB, 0.38970E+00_JPRB, 0.38585E+00_JPRB, 0.33542E+00_JPRB, 0.30210E+00_JPRB, 0.27357E+00_JPRB ,&
&  0.25214E+00_JPRB, 0.23835E+00_JPRB, 0.22721E+00_JPRB, 0.21781E+00_JPRB, 0.20855E+00_JPRB, 0.20252E+00_JPRB, 0.19798E+00_JPRB ,&
&  0.19426E+00_JPRB, 0.19252E+00_JPRB, 0.19081E+00_JPRB, 0.19018E+00_JPRB, 0.19032E+00_JPRB, 0.18963E+00_JPRB, 0.18937E+00_JPRB ,&
&  0.18896E+00_JPRB, 0.18807E+00_JPRB, 0.18785E+00_JPRB, 0.18876E+00_JPRB, 0.19053E+00_JPRB, 0.19355E+00_JPRB, 0.19471E+00_JPRB ,&
&  0.19554E+00_JPRB, 0.19601E+00_JPRB, 0.19602E+00_JPRB, 0.19777E+00_JPRB, 0.19866E+00_JPRB, 0.19882E+00_JPRB, 0.19721E+00_JPRB ,&
&  0.19556E+00_JPRB, 0.19433E+00_JPRB, 0.19406E+00_JPRB, 0.19442E+00_JPRB, 0.19564E+00_JPRB, 0.19764E+00_JPRB, 0.19813E+00_JPRB ,&
&  0.19895E+00_JPRB, 0.19864E+00_JPRB, 0.19840E+00_JPRB, 0.20002E+00_JPRB, 0.20195E+00_JPRB, 0.20408E+00_JPRB, 0.20699E+00_JPRB ,&
&  0.20882E+00_JPRB, 0.21558E+00_JPRB, 0.22787E+00_JPRB, 0.24661E+00_JPRB, 0.27474E+00_JPRB, 0.29105E+00_JPRB, 0.33032E+00_JPRB ,&
&  0.39480E+00_JPRB, 0.54014E+00_JPRB, 0.60420E+00_JPRB, 0.45919E+00_JPRB, 0.47353E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.51670E+00_JPRB, 0.52815E+00_JPRB, 0.54083E+00_JPRB, 0.53289E+00_JPRB, 0.51920E+00_JPRB, 0.51154E+00_JPRB, 0.51123E+00_JPRB ,&
&  0.51073E+00_JPRB, 0.50793E+00_JPRB, 0.49572E+00_JPRB, 0.48357E+00_JPRB, 0.46581E+00_JPRB, 0.41741E+00_JPRB, 0.39875E+00_JPRB ,&
&  0.38117E+00_JPRB, 0.38359E+00_JPRB, 0.38841E+00_JPRB, 0.39527E+00_JPRB, 0.40971E+00_JPRB, 0.42085E+00_JPRB, 0.42329E+00_JPRB ,&
&  0.41372E+00_JPRB, 0.39556E+00_JPRB, 0.37827E+00_JPRB, 0.36465E+00_JPRB, 0.36432E+00_JPRB, 0.36465E+00_JPRB, 0.35797E+00_JPRB ,&
&  0.34439E+00_JPRB, 0.33815E+00_JPRB, 0.33769E+00_JPRB, 0.35778E+00_JPRB, 0.41102E+00_JPRB, 0.43119E+00_JPRB, 0.40559E+00_JPRB ,&
&  0.40987E+00_JPRB, 0.40519E+00_JPRB, 0.40128E+00_JPRB, 0.40282E+00_JPRB, 0.39303E+00_JPRB, 0.37491E+00_JPRB, 0.34124E+00_JPRB ,&
&  0.29688E+00_JPRB, 0.27385E+00_JPRB, 0.25291E+00_JPRB, 0.23364E+00_JPRB, 0.21918E+00_JPRB, 0.21068E+00_JPRB, 0.20340E+00_JPRB ,&
&  0.19863E+00_JPRB, 0.19465E+00_JPRB, 0.19198E+00_JPRB, 0.19163E+00_JPRB, 0.19276E+00_JPRB, 0.19322E+00_JPRB, 0.19388E+00_JPRB ,&
&  0.19499E+00_JPRB, 0.19549E+00_JPRB, 0.19571E+00_JPRB, 0.19669E+00_JPRB, 0.19794E+00_JPRB, 0.20030E+00_JPRB, 0.20375E+00_JPRB ,&
&  0.20800E+00_JPRB, 0.21172E+00_JPRB, 0.21533E+00_JPRB, 0.22110E+00_JPRB, 0.22158E+00_JPRB, 0.22045E+00_JPRB, 0.21280E+00_JPRB ,&
&  0.20730E+00_JPRB, 0.20479E+00_JPRB, 0.20223E+00_JPRB, 0.20148E+00_JPRB, 0.20243E+00_JPRB, 0.20415E+00_JPRB, 0.20586E+00_JPRB ,&
&  0.20703E+00_JPRB, 0.20706E+00_JPRB, 0.20634E+00_JPRB, 0.20635E+00_JPRB, 0.20698E+00_JPRB, 0.20715E+00_JPRB, 0.21048E+00_JPRB ,&
&  0.21776E+00_JPRB, 0.22858E+00_JPRB, 0.24063E+00_JPRB, 0.25738E+00_JPRB, 0.27301E+00_JPRB, 0.28940E+00_JPRB, 0.31690E+00_JPRB ,&
&  0.35686E+00_JPRB, 0.46579E+00_JPRB, 0.53562E+00_JPRB, 0.47247E+00_JPRB, 0.49622E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.54774E+00_JPRB, 0.55152E+00_JPRB, 0.55595E+00_JPRB, 0.55851E+00_JPRB, 0.55323E+00_JPRB, 0.54608E+00_JPRB, 0.54147E+00_JPRB ,&
&  0.54659E+00_JPRB, 0.54644E+00_JPRB, 0.52293E+00_JPRB, 0.50442E+00_JPRB, 0.49374E+00_JPRB, 0.49015E+00_JPRB, 0.48698E+00_JPRB ,&
&  0.48088E+00_JPRB, 0.47884E+00_JPRB, 0.48021E+00_JPRB, 0.49168E+00_JPRB, 0.50119E+00_JPRB, 0.50951E+00_JPRB, 0.51701E+00_JPRB ,&
&  0.51282E+00_JPRB, 0.52595E+00_JPRB, 0.54772E+00_JPRB, 0.54716E+00_JPRB, 0.54465E+00_JPRB, 0.53771E+00_JPRB, 0.52350E+00_JPRB ,&
&  0.47594E+00_JPRB, 0.41196E+00_JPRB, 0.34949E+00_JPRB, 0.36061E+00_JPRB, 0.43904E+00_JPRB, 0.44917E+00_JPRB, 0.43315E+00_JPRB ,&
&  0.44345E+00_JPRB, 0.43959E+00_JPRB, 0.42180E+00_JPRB, 0.40879E+00_JPRB, 0.39337E+00_JPRB, 0.38331E+00_JPRB, 0.37639E+00_JPRB ,&
&  0.37359E+00_JPRB, 0.37217E+00_JPRB, 0.35789E+00_JPRB, 0.31007E+00_JPRB, 0.28170E+00_JPRB, 0.26460E+00_JPRB, 0.24710E+00_JPRB ,&
&  0.23346E+00_JPRB, 0.22458E+00_JPRB, 0.22281E+00_JPRB, 0.22298E+00_JPRB, 0.22728E+00_JPRB, 0.22887E+00_JPRB, 0.23037E+00_JPRB ,&
&  0.22990E+00_JPRB, 0.22865E+00_JPRB, 0.22457E+00_JPRB, 0.22001E+00_JPRB, 0.21689E+00_JPRB, 0.21631E+00_JPRB, 0.21609E+00_JPRB ,&
&  0.21863E+00_JPRB, 0.22262E+00_JPRB, 0.23875E+00_JPRB, 0.25623E+00_JPRB, 0.25608E+00_JPRB, 0.24085E+00_JPRB, 0.22580E+00_JPRB ,&
&  0.21633E+00_JPRB, 0.21148E+00_JPRB, 0.20842E+00_JPRB, 0.20786E+00_JPRB, 0.21049E+00_JPRB, 0.21419E+00_JPRB, 0.21520E+00_JPRB ,&
&  0.21512E+00_JPRB, 0.21586E+00_JPRB, 0.21600E+00_JPRB, 0.21653E+00_JPRB, 0.21724E+00_JPRB, 0.21970E+00_JPRB, 0.22428E+00_JPRB ,&
&  0.22974E+00_JPRB, 0.23955E+00_JPRB, 0.25075E+00_JPRB, 0.26583E+00_JPRB, 0.27681E+00_JPRB, 0.28474E+00_JPRB, 0.29460E+00_JPRB ,&
&  0.31182E+00_JPRB, 0.34975E+00_JPRB, 0.41875E+00_JPRB, 0.54528E+00_JPRB, 0.54055E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.60446E+00_JPRB, 0.59276E+00_JPRB, 0.58498E+00_JPRB, 0.57958E+00_JPRB, 0.57300E+00_JPRB, 0.57197E+00_JPRB, 0.57142E+00_JPRB ,&
&  0.56572E+00_JPRB, 0.55734E+00_JPRB, 0.55813E+00_JPRB, 0.54069E+00_JPRB, 0.51596E+00_JPRB, 0.49940E+00_JPRB, 0.49714E+00_JPRB ,&
&  0.50111E+00_JPRB, 0.50837E+00_JPRB, 0.51761E+00_JPRB, 0.52669E+00_JPRB, 0.53174E+00_JPRB, 0.54352E+00_JPRB, 0.56457E+00_JPRB ,&
&  0.58911E+00_JPRB, 0.60295E+00_JPRB, 0.60727E+00_JPRB, 0.59944E+00_JPRB, 0.58714E+00_JPRB, 0.57433E+00_JPRB, 0.56359E+00_JPRB ,&
&  0.55587E+00_JPRB, 0.45383E+00_JPRB, 0.38497E+00_JPRB, 0.39105E+00_JPRB, 0.46182E+00_JPRB, 0.44593E+00_JPRB, 0.44366E+00_JPRB ,&
&  0.45617E+00_JPRB, 0.45671E+00_JPRB, 0.44370E+00_JPRB, 0.42678E+00_JPRB, 0.41089E+00_JPRB, 0.39931E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.38652E+00_JPRB, 0.39299E+00_JPRB, 0.40128E+00_JPRB, 0.41416E+00_JPRB, 0.41765E+00_JPRB, 0.40525E+00_JPRB, 0.39031E+00_JPRB ,&
&  0.37584E+00_JPRB, 0.37071E+00_JPRB, 0.37293E+00_JPRB, 0.37897E+00_JPRB, 0.38693E+00_JPRB, 0.39316E+00_JPRB, 0.39120E+00_JPRB ,&
&  0.36202E+00_JPRB, 0.33916E+00_JPRB, 0.31108E+00_JPRB, 0.28465E+00_JPRB, 0.26163E+00_JPRB, 0.24612E+00_JPRB, 0.23824E+00_JPRB ,&
&  0.23616E+00_JPRB, 0.24295E+00_JPRB, 0.26395E+00_JPRB, 0.31284E+00_JPRB, 0.38173E+00_JPRB, 0.30302E+00_JPRB, 0.26377E+00_JPRB ,&
&  0.24196E+00_JPRB, 0.22902E+00_JPRB, 0.22416E+00_JPRB, 0.22476E+00_JPRB, 0.22858E+00_JPRB, 0.23430E+00_JPRB, 0.23877E+00_JPRB ,&
&  0.24234E+00_JPRB, 0.24574E+00_JPRB, 0.24791E+00_JPRB, 0.24901E+00_JPRB, 0.25065E+00_JPRB, 0.25175E+00_JPRB, 0.25070E+00_JPRB ,&
&  0.24905E+00_JPRB, 0.25469E+00_JPRB, 0.27153E+00_JPRB, 0.29830E+00_JPRB, 0.33022E+00_JPRB, 0.36466E+00_JPRB, 0.41639E+00_JPRB ,&
&  0.48207E+00_JPRB, 0.55821E+00_JPRB, 0.57565E+00_JPRB, 0.58762E+00_JPRB, 0.59790E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.64416E+00_JPRB, 0.61868E+00_JPRB, 0.60172E+00_JPRB, 0.58842E+00_JPRB, 0.57019E+00_JPRB, 0.55638E+00_JPRB, 0.55082E+00_JPRB ,&
&  0.55905E+00_JPRB, 0.55778E+00_JPRB, 0.54804E+00_JPRB, 0.53980E+00_JPRB, 0.52379E+00_JPRB, 0.50945E+00_JPRB, 0.50356E+00_JPRB ,&
&  0.51218E+00_JPRB, 0.52856E+00_JPRB, 0.54863E+00_JPRB, 0.55716E+00_JPRB, 0.55673E+00_JPRB, 0.56522E+00_JPRB, 0.58443E+00_JPRB ,&
&  0.61290E+00_JPRB, 0.63277E+00_JPRB, 0.63996E+00_JPRB, 0.62707E+00_JPRB, 0.60974E+00_JPRB, 0.60168E+00_JPRB, 0.58730E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57427E+00_JPRB, 0.50648E+00_JPRB, 0.49911E+00_JPRB, 0.47583E+00_JPRB, 0.46164E+00_JPRB, 0.45813E+00_JPRB ,&
&  0.47149E+00_JPRB, 0.47575E+00_JPRB, 0.46762E+00_JPRB, 0.45574E+00_JPRB, 0.44738E+00_JPRB, 0.43608E+00_JPRB, 0.42549E+00_JPRB ,&
&  0.41795E+00_JPRB, 0.42033E+00_JPRB, 0.42931E+00_JPRB, 0.43860E+00_JPRB, 0.44974E+00_JPRB, 0.46035E+00_JPRB, 0.45680E+00_JPRB ,&
&  0.44511E+00_JPRB, 0.44296E+00_JPRB, 0.44961E+00_JPRB, 0.46380E+00_JPRB, 0.47930E+00_JPRB, 0.48663E+00_JPRB, 0.49061E+00_JPRB ,&
&  0.48317E+00_JPRB, 0.46649E+00_JPRB, 0.45212E+00_JPRB, 0.44025E+00_JPRB, 0.41512E+00_JPRB, 0.35155E+00_JPRB, 0.32861E+00_JPRB ,&
&  0.33913E+00_JPRB, 0.39811E+00_JPRB, 0.46889E+00_JPRB, 0.46001E+00_JPRB, 0.45499E+00_JPRB, 0.44095E+00_JPRB, 0.40526E+00_JPRB ,&
&  0.36490E+00_JPRB, 0.30330E+00_JPRB, 0.28127E+00_JPRB, 0.27746E+00_JPRB, 0.27633E+00_JPRB, 0.28144E+00_JPRB, 0.29340E+00_JPRB ,&
&  0.30925E+00_JPRB, 0.33657E+00_JPRB, 0.35415E+00_JPRB, 0.36097E+00_JPRB, 0.35967E+00_JPRB, 0.34363E+00_JPRB, 0.32989E+00_JPRB ,&
&  0.32048E+00_JPRB, 0.34491E+00_JPRB, 0.40293E+00_JPRB, 0.48574E+00_JPRB, 0.48405E+00_JPRB, 0.50793E+00_JPRB, 0.53237E+00_JPRB ,&
&  0.55575E+00_JPRB, 0.59517E+00_JPRB, 0.62134E+00_JPRB, 0.62727E+00_JPRB, 0.64572E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.66808E+00_JPRB, 0.64057E+00_JPRB, 0.61161E+00_JPRB, 0.58673E+00_JPRB, 0.57455E+00_JPRB, 0.55683E+00_JPRB, 0.54205E+00_JPRB ,&
&  0.54118E+00_JPRB, 0.54911E+00_JPRB, 0.54459E+00_JPRB, 0.53875E+00_JPRB, 0.52939E+00_JPRB, 0.52164E+00_JPRB, 0.51239E+00_JPRB ,&
&  0.50849E+00_JPRB, 0.52028E+00_JPRB, 0.54617E+00_JPRB, 0.55926E+00_JPRB, 0.56570E+00_JPRB, 0.59635E+00_JPRB, 0.62765E+00_JPRB ,&
&  0.63663E+00_JPRB, 0.65040E+00_JPRB, 0.65467E+00_JPRB, 0.62986E+00_JPRB, 0.61364E+00_JPRB, 0.60735E+00_JPRB, 0.61123E+00_JPRB ,&
&  0.60649E+00_JPRB, 0.61078E+00_JPRB, 0.60287E+00_JPRB, 0.53263E+00_JPRB, 0.53213E+00_JPRB, 0.50159E+00_JPRB, 0.49133E+00_JPRB ,&
&  0.48643E+00_JPRB, 0.49208E+00_JPRB, 0.48856E+00_JPRB, 0.47973E+00_JPRB, 0.47614E+00_JPRB, 0.47922E+00_JPRB, 0.48027E+00_JPRB ,&
&  0.47540E+00_JPRB, 0.46451E+00_JPRB, 0.46884E+00_JPRB, 0.47523E+00_JPRB, 0.47902E+00_JPRB, 0.48614E+00_JPRB, 0.49666E+00_JPRB ,&
&  0.48648E+00_JPRB, 0.47750E+00_JPRB, 0.46888E+00_JPRB, 0.46900E+00_JPRB, 0.47531E+00_JPRB, 0.48960E+00_JPRB, 0.49462E+00_JPRB ,&
&  0.49254E+00_JPRB, 0.49121E+00_JPRB, 0.49207E+00_JPRB, 0.49828E+00_JPRB, 0.51322E+00_JPRB, 0.52823E+00_JPRB, 0.52956E+00_JPRB ,&
&  0.53063E+00_JPRB, 0.53505E+00_JPRB, 0.53967E+00_JPRB, 0.54176E+00_JPRB, 0.53797E+00_JPRB, 0.52292E+00_JPRB, 0.50546E+00_JPRB ,&
&  0.48888E+00_JPRB, 0.47242E+00_JPRB, 0.45839E+00_JPRB, 0.45377E+00_JPRB, 0.46667E+00_JPRB, 0.49566E+00_JPRB, 0.51179E+00_JPRB ,&
&  0.51549E+00_JPRB, 0.51747E+00_JPRB, 0.52101E+00_JPRB, 0.52234E+00_JPRB, 0.52461E+00_JPRB, 0.52290E+00_JPRB, 0.51659E+00_JPRB ,&
&  0.50544E+00_JPRB, 0.49406E+00_JPRB, 0.49087E+00_JPRB, 0.49460E+00_JPRB, 0.49923E+00_JPRB, 0.51065E+00_JPRB, 0.52668E+00_JPRB ,&
&  0.54829E+00_JPRB, 0.58638E+00_JPRB, 0.61880E+00_JPRB, 0.64404E+00_JPRB, 0.66230E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.68662E+00_JPRB, 0.68462E+00_JPRB, 0.65569E+00_JPRB, 0.62391E+00_JPRB, 0.60484E+00_JPRB, 0.58373E+00_JPRB, 0.56677E+00_JPRB ,&
&  0.56531E+00_JPRB, 0.56273E+00_JPRB, 0.56079E+00_JPRB, 0.55842E+00_JPRB, 0.55670E+00_JPRB, 0.55350E+00_JPRB, 0.54131E+00_JPRB ,&
&  0.53686E+00_JPRB, 0.54347E+00_JPRB, 0.56596E+00_JPRB, 0.59138E+00_JPRB, 0.61184E+00_JPRB, 0.63831E+00_JPRB, 0.64938E+00_JPRB ,&
&  0.66090E+00_JPRB, 0.67318E+00_JPRB, 0.67549E+00_JPRB, 0.64305E+00_JPRB, 0.61637E+00_JPRB, 0.61754E+00_JPRB, 0.62860E+00_JPRB ,&
&  0.62895E+00_JPRB, 0.63217E+00_JPRB, 0.63590E+00_JPRB, 0.63241E+00_JPRB, 0.61198E+00_JPRB, 0.57400E+00_JPRB, 0.54414E+00_JPRB ,&
&  0.51976E+00_JPRB, 0.50097E+00_JPRB, 0.49538E+00_JPRB, 0.48430E+00_JPRB, 0.48283E+00_JPRB, 0.49279E+00_JPRB, 0.50547E+00_JPRB ,&
&  0.51851E+00_JPRB, 0.52426E+00_JPRB, 0.53117E+00_JPRB, 0.52780E+00_JPRB, 0.52339E+00_JPRB, 0.52594E+00_JPRB, 0.52564E+00_JPRB ,&
&  0.52480E+00_JPRB, 0.51566E+00_JPRB, 0.51296E+00_JPRB, 0.50603E+00_JPRB, 0.50211E+00_JPRB, 0.50255E+00_JPRB, 0.50551E+00_JPRB ,&
&  0.50661E+00_JPRB, 0.51192E+00_JPRB, 0.51348E+00_JPRB, 0.51556E+00_JPRB, 0.52492E+00_JPRB, 0.54151E+00_JPRB, 0.55822E+00_JPRB ,&
&  0.57075E+00_JPRB, 0.58058E+00_JPRB, 0.58842E+00_JPRB, 0.59428E+00_JPRB, 0.59412E+00_JPRB, 0.58953E+00_JPRB, 0.57574E+00_JPRB ,&
&  0.54520E+00_JPRB, 0.51239E+00_JPRB, 0.48755E+00_JPRB, 0.46764E+00_JPRB, 0.46373E+00_JPRB, 0.47444E+00_JPRB, 0.49207E+00_JPRB ,&
&  0.50537E+00_JPRB, 0.50944E+00_JPRB, 0.50362E+00_JPRB, 0.50199E+00_JPRB, 0.50718E+00_JPRB, 0.51684E+00_JPRB, 0.52433E+00_JPRB ,&
&  0.52361E+00_JPRB, 0.52039E+00_JPRB, 0.52397E+00_JPRB, 0.53057E+00_JPRB, 0.54413E+00_JPRB, 0.56146E+00_JPRB, 0.58026E+00_JPRB ,&
&  0.59917E+00_JPRB, 0.61404E+00_JPRB, 0.63342E+00_JPRB, 0.65313E+00_JPRB, 0.67861E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.69274E+00_JPRB, 0.68456E+00_JPRB, 0.66982E+00_JPRB, 0.64317E+00_JPRB, 0.62580E+00_JPRB, 0.59871E+00_JPRB, 0.58336E+00_JPRB ,&
&  0.58554E+00_JPRB, 0.58674E+00_JPRB, 0.59181E+00_JPRB, 0.60119E+00_JPRB, 0.59927E+00_JPRB, 0.59475E+00_JPRB, 0.59031E+00_JPRB ,&
&  0.59348E+00_JPRB, 0.59395E+00_JPRB, 0.60243E+00_JPRB, 0.61805E+00_JPRB, 0.63451E+00_JPRB, 0.64374E+00_JPRB, 0.65327E+00_JPRB ,&
&  0.66059E+00_JPRB, 0.65430E+00_JPRB, 0.64996E+00_JPRB, 0.61974E+00_JPRB, 0.60224E+00_JPRB, 0.60748E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.62492E+00_JPRB, 0.63547E+00_JPRB, 0.64583E+00_JPRB, 0.65167E+00_JPRB, 0.63829E+00_JPRB, 0.63487E+00_JPRB, 0.62451E+00_JPRB ,&
&  0.58270E+00_JPRB, 0.54168E+00_JPRB, 0.51941E+00_JPRB, 0.51206E+00_JPRB, 0.50719E+00_JPRB, 0.51304E+00_JPRB, 0.51993E+00_JPRB ,&
&  0.53234E+00_JPRB, 0.54382E+00_JPRB, 0.54679E+00_JPRB, 0.55015E+00_JPRB, 0.55178E+00_JPRB, 0.55089E+00_JPRB, 0.55107E+00_JPRB ,&
&  0.54794E+00_JPRB, 0.54589E+00_JPRB, 0.55291E+00_JPRB, 0.55351E+00_JPRB, 0.54484E+00_JPRB, 0.54251E+00_JPRB, 0.54066E+00_JPRB ,&
&  0.53812E+00_JPRB, 0.53553E+00_JPRB, 0.53239E+00_JPRB, 0.54237E+00_JPRB, 0.56774E+00_JPRB, 0.59691E+00_JPRB, 0.62218E+00_JPRB ,&
&  0.63651E+00_JPRB, 0.64814E+00_JPRB, 0.65129E+00_JPRB, 0.65347E+00_JPRB, 0.65308E+00_JPRB, 0.64445E+00_JPRB, 0.64135E+00_JPRB ,&
&  0.63364E+00_JPRB, 0.61467E+00_JPRB, 0.58926E+00_JPRB, 0.54965E+00_JPRB, 0.51116E+00_JPRB, 0.48925E+00_JPRB, 0.48884E+00_JPRB ,&
&  0.50097E+00_JPRB, 0.51334E+00_JPRB, 0.51970E+00_JPRB, 0.52783E+00_JPRB, 0.54271E+00_JPRB, 0.56056E+00_JPRB, 0.57929E+00_JPRB ,&
&  0.59103E+00_JPRB, 0.59096E+00_JPRB, 0.58830E+00_JPRB, 0.59613E+00_JPRB, 0.60681E+00_JPRB, 0.62449E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.66111E+00_JPRB, 0.66081E+00_JPRB, 0.66129E+00_JPRB, 0.66267E+00_JPRB, 0.68237E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.69766E+00_JPRB, 0.69868E+00_JPRB, 0.68295E+00_JPRB, 0.67140E+00_JPRB, 0.65479E+00_JPRB, 0.63357E+00_JPRB, 0.61471E+00_JPRB ,&
&  0.61591E+00_JPRB, 0.63069E+00_JPRB, 0.64502E+00_JPRB, 0.65548E+00_JPRB, 0.67094E+00_JPRB, 0.67417E+00_JPRB, 0.68087E+00_JPRB ,&
&  0.67707E+00_JPRB, 0.64024E+00_JPRB, 0.63446E+00_JPRB, 0.64217E+00_JPRB, 0.64400E+00_JPRB, 0.64972E+00_JPRB, 0.66020E+00_JPRB ,&
&  0.66990E+00_JPRB, 0.66510E+00_JPRB, 0.65691E+00_JPRB, 0.62741E+00_JPRB, 0.62070E+00_JPRB, 0.62558E+00_JPRB, 0.63047E+00_JPRB ,&
&  0.64070E+00_JPRB, 0.65724E+00_JPRB, 0.66156E+00_JPRB, 0.67017E+00_JPRB, 0.67971E+00_JPRB, 0.68427E+00_JPRB, 0.68211E+00_JPRB ,&
&  0.65795E+00_JPRB, 0.63299E+00_JPRB, 0.62249E+00_JPRB, 0.60472E+00_JPRB, 0.58250E+00_JPRB, 0.56681E+00_JPRB, 0.55846E+00_JPRB ,&
&  0.55367E+00_JPRB, 0.55828E+00_JPRB, 0.56426E+00_JPRB, 0.57815E+00_JPRB, 0.58334E+00_JPRB, 0.58750E+00_JPRB, 0.59269E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.58750E+00_JPRB, 0.58725E+00_JPRB, 0.59697E+00_JPRB, 0.60295E+00_JPRB, 0.60087E+00_JPRB, 0.59665E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.56985E+00_JPRB, 0.57700E+00_JPRB, 0.59180E+00_JPRB, 0.59852E+00_JPRB, 0.62355E+00_JPRB, 0.65592E+00_JPRB ,&
&  0.68333E+00_JPRB, 0.70128E+00_JPRB, 0.70095E+00_JPRB, 0.69775E+00_JPRB, 0.69672E+00_JPRB, 0.70437E+00_JPRB, 0.71682E+00_JPRB ,&
&  0.70118E+00_JPRB, 0.69204E+00_JPRB, 0.69008E+00_JPRB, 0.67104E+00_JPRB, 0.63498E+00_JPRB, 0.60420E+00_JPRB, 0.58140E+00_JPRB ,&
&  0.56308E+00_JPRB, 0.55262E+00_JPRB, 0.55620E+00_JPRB, 0.55986E+00_JPRB, 0.57283E+00_JPRB, 0.60851E+00_JPRB, 0.64564E+00_JPRB ,&
&  0.68591E+00_JPRB, 0.70667E+00_JPRB, 0.71318E+00_JPRB, 0.72117E+00_JPRB, 0.72710E+00_JPRB, 0.71653E+00_JPRB, 0.70777E+00_JPRB ,&
&  0.70546E+00_JPRB, 0.69729E+00_JPRB, 0.68798E+00_JPRB, 0.67809E+00_JPRB, 0.68510E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.69467E+00_JPRB, 0.71585E+00_JPRB, 0.68239E+00_JPRB, 0.66348E+00_JPRB, 0.64266E+00_JPRB, 0.63791E+00_JPRB, 0.62854E+00_JPRB ,&
&  0.63866E+00_JPRB, 0.65520E+00_JPRB, 0.67871E+00_JPRB, 0.68657E+00_JPRB, 0.69342E+00_JPRB, 0.69627E+00_JPRB, 0.71343E+00_JPRB ,&
&  0.70902E+00_JPRB, 0.70094E+00_JPRB, 0.70422E+00_JPRB, 0.69398E+00_JPRB, 0.68522E+00_JPRB, 0.68984E+00_JPRB, 0.69830E+00_JPRB ,&
&  0.70808E+00_JPRB, 0.69938E+00_JPRB, 0.68207E+00_JPRB, 0.65545E+00_JPRB, 0.64281E+00_JPRB, 0.64015E+00_JPRB, 0.64758E+00_JPRB ,&
&  0.65295E+00_JPRB, 0.67667E+00_JPRB, 0.61498E+00_JPRB, 0.67206E+00_JPRB, 0.68152E+00_JPRB, 0.68124E+00_JPRB, 0.68210E+00_JPRB ,&
&  0.68903E+00_JPRB, 0.67978E+00_JPRB, 0.67122E+00_JPRB, 0.66152E+00_JPRB, 0.63766E+00_JPRB, 0.61833E+00_JPRB, 0.59837E+00_JPRB ,&
&  0.58060E+00_JPRB, 0.56946E+00_JPRB, 0.57419E+00_JPRB, 0.59135E+00_JPRB, 0.60895E+00_JPRB, 0.61876E+00_JPRB, 0.62482E+00_JPRB ,&
&  0.62808E+00_JPRB, 0.62071E+00_JPRB, 0.61061E+00_JPRB, 0.61062E+00_JPRB, 0.61710E+00_JPRB, 0.62210E+00_JPRB, 0.62549E+00_JPRB ,&
&  0.62626E+00_JPRB, 0.60307E+00_JPRB, 0.58987E+00_JPRB, 0.60587E+00_JPRB, 0.62304E+00_JPRB, 0.64167E+00_JPRB, 0.66604E+00_JPRB ,&
&  0.68444E+00_JPRB, 0.69148E+00_JPRB, 0.69051E+00_JPRB, 0.69722E+00_JPRB, 0.70426E+00_JPRB, 0.71318E+00_JPRB, 0.72044E+00_JPRB ,&
&  0.71090E+00_JPRB, 0.70892E+00_JPRB, 0.70979E+00_JPRB, 0.70986E+00_JPRB, 0.70118E+00_JPRB, 0.67868E+00_JPRB, 0.64693E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.61734E+00_JPRB, 0.60615E+00_JPRB, 0.60339E+00_JPRB, 0.60584E+00_JPRB, 0.63190E+00_JPRB, 0.66891E+00_JPRB ,&
&  0.70569E+00_JPRB, 0.73574E+00_JPRB, 0.76168E+00_JPRB, 0.78543E+00_JPRB, 0.78376E+00_JPRB, 0.76268E+00_JPRB, 0.75391E+00_JPRB ,&
&  0.74003E+00_JPRB, 0.72470E+00_JPRB, 0.71374E+00_JPRB, 0.70460E+00_JPRB, 0.69411E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.73671E+00_JPRB, 0.73678E+00_JPRB, 0.72562E+00_JPRB, 0.68599E+00_JPRB, 0.65593E+00_JPRB, 0.66497E+00_JPRB, 0.67382E+00_JPRB ,&
&  0.67292E+00_JPRB, 0.68735E+00_JPRB, 0.71226E+00_JPRB, 0.73760E+00_JPRB, 0.75460E+00_JPRB, 0.75459E+00_JPRB, 0.75882E+00_JPRB ,&
&  0.75654E+00_JPRB, 0.75454E+00_JPRB, 0.76454E+00_JPRB, 0.76944E+00_JPRB, 0.75877E+00_JPRB, 0.73765E+00_JPRB, 0.73492E+00_JPRB ,&
&  0.74073E+00_JPRB, 0.73646E+00_JPRB, 0.71812E+00_JPRB, 0.70560E+00_JPRB, 0.69545E+00_JPRB, 0.70053E+00_JPRB, 0.70689E+00_JPRB ,&
&  0.70159E+00_JPRB, 0.65026E+00_JPRB, 0.68893E+00_JPRB, 0.70266E+00_JPRB, 0.71317E+00_JPRB, 0.70481E+00_JPRB, 0.71202E+00_JPRB ,&
&  0.72473E+00_JPRB, 0.73256E+00_JPRB, 0.71283E+00_JPRB, 0.68919E+00_JPRB, 0.66682E+00_JPRB, 0.64755E+00_JPRB, 0.61833E+00_JPRB ,&
&  0.60699E+00_JPRB, 0.60298E+00_JPRB, 0.61529E+00_JPRB, 0.62877E+00_JPRB, 0.64457E+00_JPRB, 0.65076E+00_JPRB, 0.66082E+00_JPRB ,&
&  0.66288E+00_JPRB, 0.65738E+00_JPRB, 0.65245E+00_JPRB, 0.64884E+00_JPRB, 0.64837E+00_JPRB, 0.64273E+00_JPRB, 0.63453E+00_JPRB ,&
&  0.62831E+00_JPRB, 0.62652E+00_JPRB, 0.63072E+00_JPRB, 0.63994E+00_JPRB, 0.65122E+00_JPRB, 0.66845E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.69822E+00_JPRB, 0.70335E+00_JPRB, 0.70788E+00_JPRB, 0.72076E+00_JPRB, 0.72929E+00_JPRB, 0.72800E+00_JPRB, 0.72545E+00_JPRB ,&
&  0.72311E+00_JPRB, 0.71772E+00_JPRB, 0.72228E+00_JPRB, 0.72132E+00_JPRB, 0.72192E+00_JPRB, 0.71379E+00_JPRB, 0.69910E+00_JPRB ,&
&  0.69355E+00_JPRB, 0.68931E+00_JPRB, 0.68074E+00_JPRB, 0.67799E+00_JPRB, 0.67590E+00_JPRB, 0.69362E+00_JPRB, 0.72470E+00_JPRB ,&
&  0.74168E+00_JPRB, 0.76461E+00_JPRB, 0.79733E+00_JPRB, 0.81924E+00_JPRB, 0.81979E+00_JPRB, 0.82063E+00_JPRB, 0.81079E+00_JPRB ,&
&  0.79278E+00_JPRB, 0.78054E+00_JPRB, 0.76761E+00_JPRB, 0.76234E+00_JPRB, 0.75770E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.77635E+00_JPRB, 0.76858E+00_JPRB, 0.75289E+00_JPRB, 0.72471E+00_JPRB, 0.68958E+00_JPRB, 0.69731E+00_JPRB, 0.70923E+00_JPRB ,&
&  0.70317E+00_JPRB, 0.69351E+00_JPRB, 0.70368E+00_JPRB, 0.72840E+00_JPRB, 0.75297E+00_JPRB, 0.76721E+00_JPRB, 0.76988E+00_JPRB ,&
&  0.76532E+00_JPRB, 0.76776E+00_JPRB, 0.77632E+00_JPRB, 0.79338E+00_JPRB, 0.79234E+00_JPRB, 0.77421E+00_JPRB, 0.76296E+00_JPRB ,&
&  0.75957E+00_JPRB, 0.75753E+00_JPRB, 0.75457E+00_JPRB, 0.73759E+00_JPRB, 0.72625E+00_JPRB, 0.72335E+00_JPRB, 0.72393E+00_JPRB ,&
&  0.72085E+00_JPRB, 0.58476E+00_JPRB, 0.68728E+00_JPRB, 0.70434E+00_JPRB, 0.72205E+00_JPRB, 0.71880E+00_JPRB, 0.72008E+00_JPRB ,&
&  0.72477E+00_JPRB, 0.73990E+00_JPRB, 0.73215E+00_JPRB, 0.70702E+00_JPRB, 0.68065E+00_JPRB, 0.65228E+00_JPRB, 0.63200E+00_JPRB ,&
&  0.62056E+00_JPRB, 0.62148E+00_JPRB, 0.62963E+00_JPRB, 0.63882E+00_JPRB, 0.64911E+00_JPRB, 0.65007E+00_JPRB, 0.65354E+00_JPRB ,&
&  0.65595E+00_JPRB, 0.66111E+00_JPRB, 0.66396E+00_JPRB, 0.67011E+00_JPRB, 0.67273E+00_JPRB, 0.67004E+00_JPRB, 0.66312E+00_JPRB ,&
&  0.65324E+00_JPRB, 0.64696E+00_JPRB, 0.64619E+00_JPRB, 0.64845E+00_JPRB, 0.65536E+00_JPRB, 0.66746E+00_JPRB, 0.67948E+00_JPRB ,&
&  0.69324E+00_JPRB, 0.70221E+00_JPRB, 0.70704E+00_JPRB, 0.70942E+00_JPRB, 0.71866E+00_JPRB, 0.72841E+00_JPRB, 0.72844E+00_JPRB ,&
&  0.72229E+00_JPRB, 0.71531E+00_JPRB, 0.71344E+00_JPRB, 0.71052E+00_JPRB, 0.71125E+00_JPRB, 0.71229E+00_JPRB, 0.70841E+00_JPRB ,&
&  0.70050E+00_JPRB, 0.69883E+00_JPRB, 0.70006E+00_JPRB, 0.68934E+00_JPRB, 0.69095E+00_JPRB, 0.70106E+00_JPRB, 0.71992E+00_JPRB ,&
&  0.74346E+00_JPRB, 0.77454E+00_JPRB, 0.80530E+00_JPRB, 0.83412E+00_JPRB, 0.85358E+00_JPRB, 0.86332E+00_JPRB, 0.86162E+00_JPRB ,&
&  0.84660E+00_JPRB, 0.82302E+00_JPRB, 0.80335E+00_JPRB, 0.79226E+00_JPRB, 0.79192E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.81316E+00_JPRB, 0.80150E+00_JPRB, 0.78644E+00_JPRB, 0.76739E+00_JPRB, 0.75012E+00_JPRB, 0.74031E+00_JPRB, 0.74548E+00_JPRB ,&
&  0.74402E+00_JPRB, 0.73172E+00_JPRB, 0.74105E+00_JPRB, 0.75391E+00_JPRB, 0.76461E+00_JPRB, 0.77490E+00_JPRB, 0.77924E+00_JPRB ,&
&  0.78397E+00_JPRB, 0.78187E+00_JPRB, 0.78543E+00_JPRB, 0.79747E+00_JPRB, 0.81568E+00_JPRB, 0.81697E+00_JPRB, 0.81375E+00_JPRB ,&
&  0.79092E+00_JPRB, 0.77925E+00_JPRB, 0.76862E+00_JPRB, 0.75575E+00_JPRB, 0.74441E+00_JPRB, 0.74087E+00_JPRB, 0.74405E+00_JPRB ,&
&  0.63186E+00_JPRB, 0.53912E+00_JPRB, 0.64976E+00_JPRB, 0.76005E+00_JPRB, 0.77469E+00_JPRB, 0.77312E+00_JPRB, 0.78726E+00_JPRB ,&
&  0.79032E+00_JPRB, 0.78873E+00_JPRB, 0.78532E+00_JPRB, 0.77284E+00_JPRB, 0.74129E+00_JPRB, 0.70771E+00_JPRB, 0.68857E+00_JPRB ,&
&  0.67549E+00_JPRB, 0.67779E+00_JPRB, 0.67650E+00_JPRB, 0.68094E+00_JPRB, 0.67921E+00_JPRB, 0.67100E+00_JPRB, 0.66894E+00_JPRB ,&
&  0.66973E+00_JPRB, 0.68017E+00_JPRB, 0.69018E+00_JPRB, 0.69753E+00_JPRB, 0.70038E+00_JPRB, 0.70148E+00_JPRB, 0.69905E+00_JPRB ,&
&  0.69712E+00_JPRB, 0.69455E+00_JPRB, 0.69023E+00_JPRB, 0.69078E+00_JPRB, 0.69298E+00_JPRB, 0.70007E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.71840E+00_JPRB, 0.71875E+00_JPRB, 0.71965E+00_JPRB, 0.72078E+00_JPRB, 0.73105E+00_JPRB, 0.73916E+00_JPRB, 0.73655E+00_JPRB ,&
&  0.73154E+00_JPRB, 0.72857E+00_JPRB, 0.73458E+00_JPRB, 0.73932E+00_JPRB, 0.74538E+00_JPRB, 0.74795E+00_JPRB, 0.74353E+00_JPRB ,&
&  0.73488E+00_JPRB, 0.72091E+00_JPRB, 0.71045E+00_JPRB, 0.70933E+00_JPRB, 0.71841E+00_JPRB, 0.72541E+00_JPRB, 0.74214E+00_JPRB ,&
&  0.77280E+00_JPRB, 0.81081E+00_JPRB, 0.84305E+00_JPRB, 0.88863E+00_JPRB, 0.91881E+00_JPRB, 0.93636E+00_JPRB, 0.93553E+00_JPRB ,&
&  0.92032E+00_JPRB, 0.88099E+00_JPRB, 0.86773E+00_JPRB, 0.84185E+00_JPRB, 0.82632E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.80654E+00_JPRB, 0.80654E+00_JPRB, 0.80503E+00_JPRB, 0.79122E+00_JPRB, 0.77484E+00_JPRB, 0.76800E+00_JPRB, 0.77325E+00_JPRB ,&
&  0.76808E+00_JPRB, 0.75419E+00_JPRB, 0.75872E+00_JPRB, 0.76251E+00_JPRB, 0.77027E+00_JPRB, 0.77454E+00_JPRB, 0.77967E+00_JPRB ,&
&  0.78496E+00_JPRB, 0.78962E+00_JPRB, 0.78802E+00_JPRB, 0.79637E+00_JPRB, 0.78888E+00_JPRB, 0.78315E+00_JPRB, 0.78001E+00_JPRB ,&
&  0.77115E+00_JPRB, 0.76188E+00_JPRB, 0.75435E+00_JPRB, 0.74423E+00_JPRB, 0.73697E+00_JPRB, 0.72867E+00_JPRB, 0.71364E+00_JPRB ,&
&  0.72612E+00_JPRB, 0.57677E+00_JPRB, 0.66957E+00_JPRB, 0.83364E+00_JPRB, 0.84604E+00_JPRB, 0.83853E+00_JPRB, 0.84318E+00_JPRB ,&
&  0.85591E+00_JPRB, 0.86173E+00_JPRB, 0.85051E+00_JPRB, 0.82409E+00_JPRB, 0.77666E+00_JPRB, 0.75315E+00_JPRB, 0.73847E+00_JPRB ,&
&  0.72149E+00_JPRB, 0.70817E+00_JPRB, 0.69949E+00_JPRB, 0.69947E+00_JPRB, 0.69789E+00_JPRB, 0.69346E+00_JPRB, 0.68919E+00_JPRB ,&
&  0.69215E+00_JPRB, 0.70162E+00_JPRB, 0.70732E+00_JPRB, 0.71269E+00_JPRB, 0.71823E+00_JPRB, 0.71505E+00_JPRB, 0.71414E+00_JPRB ,&
&  0.71198E+00_JPRB, 0.70989E+00_JPRB, 0.70669E+00_JPRB, 0.70532E+00_JPRB, 0.70599E+00_JPRB, 0.72253E+00_JPRB, 0.73601E+00_JPRB ,&
&  0.73654E+00_JPRB, 0.73465E+00_JPRB, 0.73798E+00_JPRB, 0.74378E+00_JPRB, 0.74862E+00_JPRB, 0.75117E+00_JPRB, 0.74670E+00_JPRB ,&
&  0.74220E+00_JPRB, 0.73497E+00_JPRB, 0.73367E+00_JPRB, 0.73305E+00_JPRB, 0.72842E+00_JPRB, 0.72687E+00_JPRB, 0.72981E+00_JPRB ,&
&  0.72435E+00_JPRB, 0.71304E+00_JPRB, 0.71490E+00_JPRB, 0.72802E+00_JPRB, 0.73904E+00_JPRB, 0.74550E+00_JPRB, 0.76063E+00_JPRB ,&
&  0.79158E+00_JPRB, 0.82702E+00_JPRB, 0.85484E+00_JPRB, 0.90553E+00_JPRB, 0.93758E+00_JPRB, 0.95743E+00_JPRB, 0.96200E+00_JPRB ,&
&  0.94841E+00_JPRB, 0.92919E+00_JPRB, 0.66792E+00_JPRB, 0.77836E+00_JPRB, 0.80080E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.82534E+00_JPRB, 0.83107E+00_JPRB, 0.82248E+00_JPRB, 0.80862E+00_JPRB, 0.79818E+00_JPRB, 0.79107E+00_JPRB, 0.79305E+00_JPRB ,&
&  0.79007E+00_JPRB, 0.78345E+00_JPRB, 0.78599E+00_JPRB, 0.78893E+00_JPRB, 0.79244E+00_JPRB, 0.79142E+00_JPRB, 0.79788E+00_JPRB ,&
&  0.80000E+00_JPRB, 0.80250E+00_JPRB, 0.80631E+00_JPRB, 0.80042E+00_JPRB, 0.78101E+00_JPRB, 0.76717E+00_JPRB, 0.75298E+00_JPRB ,&
&  0.75149E+00_JPRB, 0.75106E+00_JPRB, 0.75157E+00_JPRB, 0.74528E+00_JPRB, 0.72599E+00_JPRB, 0.70961E+00_JPRB, 0.69925E+00_JPRB ,&
&  0.70035E+00_JPRB, 0.56296E+00_JPRB, 0.66096E+00_JPRB, 0.89439E+00_JPRB, 0.10039E+01_JPRB, 0.96005E+00_JPRB, 0.93959E+00_JPRB ,&
&  0.93767E+00_JPRB, 0.92850E+00_JPRB, 0.90268E+00_JPRB, 0.86715E+00_JPRB, 0.82263E+00_JPRB, 0.80801E+00_JPRB, 0.78790E+00_JPRB ,&
&  0.76864E+00_JPRB, 0.76089E+00_JPRB, 0.75375E+00_JPRB, 0.75218E+00_JPRB, 0.74112E+00_JPRB, 0.72846E+00_JPRB, 0.72870E+00_JPRB ,&
&  0.73588E+00_JPRB, 0.73461E+00_JPRB, 0.73746E+00_JPRB, 0.74094E+00_JPRB, 0.73608E+00_JPRB, 0.75046E+00_JPRB, 0.75380E+00_JPRB ,&
&  0.75337E+00_JPRB, 0.74880E+00_JPRB, 0.74715E+00_JPRB, 0.74539E+00_JPRB, 0.75124E+00_JPRB, 0.77348E+00_JPRB, 0.78792E+00_JPRB ,&
&  0.78111E+00_JPRB, 0.77100E+00_JPRB, 0.77793E+00_JPRB, 0.78134E+00_JPRB, 0.77466E+00_JPRB, 0.76073E+00_JPRB, 0.74494E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.73556E+00_JPRB, 0.73375E+00_JPRB, 0.72855E+00_JPRB, 0.72070E+00_JPRB, 0.71427E+00_JPRB, 0.71107E+00_JPRB ,&
&  0.70990E+00_JPRB, 0.70984E+00_JPRB, 0.71623E+00_JPRB, 0.73095E+00_JPRB, 0.74994E+00_JPRB, 0.77160E+00_JPRB, 0.79479E+00_JPRB ,&
&  0.82384E+00_JPRB, 0.85801E+00_JPRB, 0.89180E+00_JPRB, 0.90895E+00_JPRB, 0.98905E+00_JPRB, 0.10137E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.97730E+00_JPRB, 0.91651E+00_JPRB, 0.71666E+00_JPRB, 0.79485E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.86307E+00_JPRB, 0.88218E+00_JPRB, 0.86683E+00_JPRB, 0.83469E+00_JPRB, 0.81647E+00_JPRB, 0.80885E+00_JPRB, 0.80654E+00_JPRB ,&
&  0.80156E+00_JPRB, 0.79026E+00_JPRB, 0.79465E+00_JPRB, 0.80124E+00_JPRB, 0.79954E+00_JPRB, 0.78905E+00_JPRB, 0.78190E+00_JPRB ,&
&  0.78043E+00_JPRB, 0.78462E+00_JPRB, 0.78991E+00_JPRB, 0.78010E+00_JPRB, 0.75949E+00_JPRB, 0.74557E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.73174E+00_JPRB, 0.73170E+00_JPRB, 0.72971E+00_JPRB, 0.73156E+00_JPRB, 0.71133E+00_JPRB, 0.68862E+00_JPRB, 0.66960E+00_JPRB ,&
&  0.65616E+00_JPRB, 0.59048E+00_JPRB, 0.79421E+00_JPRB, 0.10815E+01_JPRB, 0.12116E+01_JPRB, 0.11803E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10386E+01_JPRB, 0.98765E+00_JPRB, 0.93944E+00_JPRB, 0.89247E+00_JPRB, 0.86563E+00_JPRB, 0.85080E+00_JPRB, 0.84189E+00_JPRB ,&
&  0.83409E+00_JPRB, 0.82469E+00_JPRB, 0.81738E+00_JPRB, 0.79868E+00_JPRB, 0.78256E+00_JPRB, 0.77837E+00_JPRB, 0.77732E+00_JPRB ,&
&  0.77808E+00_JPRB, 0.77009E+00_JPRB, 0.76754E+00_JPRB, 0.76775E+00_JPRB, 0.74946E+00_JPRB, 0.74326E+00_JPRB, 0.73609E+00_JPRB ,&
&  0.73771E+00_JPRB, 0.75032E+00_JPRB, 0.75401E+00_JPRB, 0.78593E+00_JPRB, 0.80063E+00_JPRB, 0.81316E+00_JPRB, 0.81416E+00_JPRB ,&
&  0.80662E+00_JPRB, 0.80471E+00_JPRB, 0.79754E+00_JPRB, 0.79398E+00_JPRB, 0.78437E+00_JPRB, 0.76220E+00_JPRB, 0.74146E+00_JPRB ,&
&  0.73758E+00_JPRB, 0.73855E+00_JPRB, 0.72968E+00_JPRB, 0.72874E+00_JPRB, 0.72931E+00_JPRB, 0.72252E+00_JPRB, 0.71796E+00_JPRB ,&
&  0.71526E+00_JPRB, 0.71771E+00_JPRB, 0.72083E+00_JPRB, 0.73359E+00_JPRB, 0.75441E+00_JPRB, 0.77965E+00_JPRB, 0.80583E+00_JPRB ,&
&  0.83133E+00_JPRB, 0.85502E+00_JPRB, 0.88981E+00_JPRB, 0.98929E+00_JPRB, 0.10229E+01_JPRB, 0.10383E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.10510E+01_JPRB, 0.10068E+01_JPRB, 0.96054E+00_JPRB, 0.89607E+00_JPRB, 0.81428E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.97449E+00_JPRB, 0.94912E+00_JPRB, 0.91838E+00_JPRB, 0.87735E+00_JPRB, 0.85704E+00_JPRB, 0.83813E+00_JPRB, 0.81929E+00_JPRB ,&
&  0.80948E+00_JPRB, 0.79973E+00_JPRB, 0.79260E+00_JPRB, 0.77796E+00_JPRB, 0.76374E+00_JPRB, 0.75522E+00_JPRB, 0.74492E+00_JPRB ,&
&  0.74559E+00_JPRB, 0.74602E+00_JPRB, 0.73840E+00_JPRB, 0.73057E+00_JPRB, 0.72745E+00_JPRB, 0.72466E+00_JPRB, 0.72885E+00_JPRB ,&
&  0.72085E+00_JPRB, 0.71130E+00_JPRB, 0.71110E+00_JPRB, 0.70420E+00_JPRB, 0.69181E+00_JPRB, 0.68053E+00_JPRB, 0.65071E+00_JPRB ,&
&  0.65131E+00_JPRB, 0.69791E+00_JPRB, 0.10993E+01_JPRB, 0.15397E+01_JPRB, 0.15339E+01_JPRB, 0.14315E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.10871E+01_JPRB, 0.99404E+00_JPRB, 0.93749E+00_JPRB, 0.92306E+00_JPRB, 0.90665E+00_JPRB, 0.88671E+00_JPRB ,&
&  0.86143E+00_JPRB, 0.83749E+00_JPRB, 0.81992E+00_JPRB, 0.82194E+00_JPRB, 0.81407E+00_JPRB, 0.81208E+00_JPRB, 0.81336E+00_JPRB ,&
&  0.80745E+00_JPRB, 0.80155E+00_JPRB, 0.78703E+00_JPRB, 0.77884E+00_JPRB, 0.77216E+00_JPRB, 0.77111E+00_JPRB, 0.78332E+00_JPRB ,&
&  0.81367E+00_JPRB, 0.82051E+00_JPRB, 0.83261E+00_JPRB, 0.84658E+00_JPRB, 0.86090E+00_JPRB, 0.85749E+00_JPRB, 0.83956E+00_JPRB ,&
&  0.82341E+00_JPRB, 0.81583E+00_JPRB, 0.78974E+00_JPRB, 0.79786E+00_JPRB, 0.78873E+00_JPRB, 0.77454E+00_JPRB, 0.76489E+00_JPRB ,&
&  0.75236E+00_JPRB, 0.75409E+00_JPRB, 0.74506E+00_JPRB, 0.74487E+00_JPRB, 0.74724E+00_JPRB, 0.74495E+00_JPRB, 0.73984E+00_JPRB ,&
&  0.74425E+00_JPRB, 0.74204E+00_JPRB, 0.74894E+00_JPRB, 0.77427E+00_JPRB, 0.80646E+00_JPRB, 0.82628E+00_JPRB, 0.84770E+00_JPRB ,&
&  0.86938E+00_JPRB, 0.91283E+00_JPRB, 0.90713E+00_JPRB, 0.10182E+01_JPRB, 0.11741E+01_JPRB, 0.11911E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10296E+01_JPRB, 0.99950E+00_JPRB, 0.95716E+00_JPRB, 0.95802E+00_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.95448E+00_JPRB, 0.91091E+00_JPRB, 0.87880E+00_JPRB, 0.84731E+00_JPRB, 0.83431E+00_JPRB, 0.81459E+00_JPRB, 0.79001E+00_JPRB ,&
&  0.77481E+00_JPRB, 0.77103E+00_JPRB, 0.76190E+00_JPRB, 0.74342E+00_JPRB, 0.72741E+00_JPRB, 0.72121E+00_JPRB, 0.72012E+00_JPRB ,&
&  0.71803E+00_JPRB, 0.71650E+00_JPRB, 0.71415E+00_JPRB, 0.70696E+00_JPRB, 0.71041E+00_JPRB, 0.71979E+00_JPRB, 0.72994E+00_JPRB ,&
&  0.71418E+00_JPRB, 0.70165E+00_JPRB, 0.69075E+00_JPRB, 0.67977E+00_JPRB, 0.67604E+00_JPRB, 0.65744E+00_JPRB, 0.63634E+00_JPRB ,&
&  0.71342E+00_JPRB, 0.92839E+00_JPRB, 0.15018E+01_JPRB, 0.19538E+01_JPRB, 0.17832E+01_JPRB, 0.14454E+01_JPRB, 0.12685E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.10738E+01_JPRB, 0.10063E+01_JPRB, 0.95855E+00_JPRB, 0.93713E+00_JPRB, 0.91087E+00_JPRB, 0.86350E+00_JPRB ,&
&  0.82860E+00_JPRB, 0.80772E+00_JPRB, 0.79713E+00_JPRB, 0.79521E+00_JPRB, 0.78610E+00_JPRB, 0.79322E+00_JPRB, 0.79628E+00_JPRB ,&
&  0.79628E+00_JPRB, 0.79119E+00_JPRB, 0.78379E+00_JPRB, 0.77619E+00_JPRB, 0.79229E+00_JPRB, 0.83663E+00_JPRB, 0.90796E+00_JPRB ,&
&  0.92042E+00_JPRB, 0.93668E+00_JPRB, 0.92863E+00_JPRB, 0.92616E+00_JPRB, 0.91439E+00_JPRB, 0.87012E+00_JPRB, 0.84097E+00_JPRB ,&
&  0.81783E+00_JPRB, 0.81271E+00_JPRB, 0.79055E+00_JPRB, 0.79037E+00_JPRB, 0.79348E+00_JPRB, 0.78895E+00_JPRB, 0.78846E+00_JPRB ,&
&  0.78330E+00_JPRB, 0.77917E+00_JPRB, 0.77448E+00_JPRB, 0.78065E+00_JPRB, 0.77721E+00_JPRB, 0.76403E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.75600E+00_JPRB, 0.74502E+00_JPRB, 0.75834E+00_JPRB, 0.79731E+00_JPRB, 0.83381E+00_JPRB, 0.85504E+00_JPRB, 0.88122E+00_JPRB ,&
&  0.91324E+00_JPRB, 0.96201E+00_JPRB, 0.94711E+00_JPRB, 0.10226E+01_JPRB, 0.11370E+01_JPRB, 0.12443E+01_JPRB, 0.11256E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10167E+01_JPRB, 0.98341E+00_JPRB, 0.94191E+00_JPRB, 0.98454E+00_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.90477E+00_JPRB, 0.87222E+00_JPRB, 0.85274E+00_JPRB, 0.83704E+00_JPRB, 0.81206E+00_JPRB, 0.78538E+00_JPRB, 0.77555E+00_JPRB ,&
&  0.76831E+00_JPRB, 0.74593E+00_JPRB, 0.72057E+00_JPRB, 0.68943E+00_JPRB, 0.68027E+00_JPRB, 0.68355E+00_JPRB, 0.68741E+00_JPRB ,&
&  0.68496E+00_JPRB, 0.68511E+00_JPRB, 0.68375E+00_JPRB, 0.68295E+00_JPRB, 0.68279E+00_JPRB, 0.70623E+00_JPRB, 0.71661E+00_JPRB ,&
&  0.70969E+00_JPRB, 0.71167E+00_JPRB, 0.69151E+00_JPRB, 0.66967E+00_JPRB, 0.65914E+00_JPRB, 0.64749E+00_JPRB, 0.64546E+00_JPRB ,&
&  0.83503E+00_JPRB, 0.12066E+01_JPRB, 0.20704E+01_JPRB, 0.20382E+01_JPRB, 0.16190E+01_JPRB, 0.12649E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11231E+01_JPRB, 0.10546E+01_JPRB, 0.98622E+00_JPRB, 0.93297E+00_JPRB, 0.88990E+00_JPRB, 0.86220E+00_JPRB, 0.83015E+00_JPRB ,&
&  0.81139E+00_JPRB, 0.80111E+00_JPRB, 0.77792E+00_JPRB, 0.76697E+00_JPRB, 0.76087E+00_JPRB, 0.76239E+00_JPRB, 0.76209E+00_JPRB ,&
&  0.76632E+00_JPRB, 0.76862E+00_JPRB, 0.76870E+00_JPRB, 0.75601E+00_JPRB, 0.74645E+00_JPRB, 0.73421E+00_JPRB, 0.95258E+00_JPRB ,&
&  0.12624E+01_JPRB, 0.11644E+01_JPRB, 0.10364E+01_JPRB, 0.97583E+00_JPRB, 0.92976E+00_JPRB, 0.86756E+00_JPRB, 0.84847E+00_JPRB ,&
&  0.83915E+00_JPRB, 0.84186E+00_JPRB, 0.82988E+00_JPRB, 0.84168E+00_JPRB, 0.84067E+00_JPRB, 0.83358E+00_JPRB, 0.82446E+00_JPRB ,&
&  0.80560E+00_JPRB, 0.80315E+00_JPRB, 0.80253E+00_JPRB, 0.80105E+00_JPRB, 0.78662E+00_JPRB, 0.78235E+00_JPRB, 0.77793E+00_JPRB ,&
&  0.77423E+00_JPRB, 0.75875E+00_JPRB, 0.78723E+00_JPRB, 0.80636E+00_JPRB, 0.87769E+00_JPRB, 0.93101E+00_JPRB, 0.93255E+00_JPRB ,&
&  0.90839E+00_JPRB, 0.91236E+00_JPRB, 0.91454E+00_JPRB, 0.96697E+00_JPRB, 0.10625E+01_JPRB, 0.11799E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.98560E+00_JPRB, 0.96641E+00_JPRB, 0.94210E+00_JPRB, 0.91547E+00_JPRB, 0.92243E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.89111E+00_JPRB, 0.86488E+00_JPRB, 0.84081E+00_JPRB, 0.81189E+00_JPRB, 0.77591E+00_JPRB, 0.75327E+00_JPRB, 0.73725E+00_JPRB ,&
&  0.72774E+00_JPRB, 0.69934E+00_JPRB, 0.67736E+00_JPRB, 0.67406E+00_JPRB, 0.67751E+00_JPRB, 0.66729E+00_JPRB, 0.66234E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.66771E+00_JPRB, 0.65876E+00_JPRB, 0.65106E+00_JPRB, 0.65504E+00_JPRB, 0.69076E+00_JPRB, 0.70623E+00_JPRB ,&
&  0.71273E+00_JPRB, 0.70490E+00_JPRB, 0.68396E+00_JPRB, 0.65565E+00_JPRB, 0.64890E+00_JPRB, 0.65713E+00_JPRB, 0.69551E+00_JPRB ,&
&  0.98311E+00_JPRB, 0.11838E+01_JPRB, 0.21380E+01_JPRB, 0.22291E+01_JPRB, 0.15638E+01_JPRB, 0.12106E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.10530E+01_JPRB, 0.10102E+01_JPRB, 0.95173E+00_JPRB, 0.90039E+00_JPRB, 0.85639E+00_JPRB, 0.81790E+00_JPRB, 0.78800E+00_JPRB ,&
&  0.77826E+00_JPRB, 0.76600E+00_JPRB, 0.75507E+00_JPRB, 0.75471E+00_JPRB, 0.74841E+00_JPRB, 0.74785E+00_JPRB, 0.74781E+00_JPRB ,&
&  0.75102E+00_JPRB, 0.74667E+00_JPRB, 0.73979E+00_JPRB, 0.73432E+00_JPRB, 0.76240E+00_JPRB, 0.90691E+00_JPRB, 0.11084E+01_JPRB ,&
&  0.12422E+01_JPRB, 0.13647E+01_JPRB, 0.11618E+01_JPRB, 0.10401E+01_JPRB, 0.93672E+00_JPRB, 0.87377E+00_JPRB, 0.87463E+00_JPRB ,&
&  0.87950E+00_JPRB, 0.89107E+00_JPRB, 0.87852E+00_JPRB, 0.87837E+00_JPRB, 0.87839E+00_JPRB, 0.85479E+00_JPRB, 0.84195E+00_JPRB ,&
&  0.83591E+00_JPRB, 0.83133E+00_JPRB, 0.81987E+00_JPRB, 0.80610E+00_JPRB, 0.78862E+00_JPRB, 0.77801E+00_JPRB, 0.77472E+00_JPRB ,&
&  0.77419E+00_JPRB, 0.76885E+00_JPRB, 0.80360E+00_JPRB, 0.82882E+00_JPRB, 0.87402E+00_JPRB, 0.88979E+00_JPRB, 0.89380E+00_JPRB ,&
&  0.89176E+00_JPRB, 0.88539E+00_JPRB, 0.88339E+00_JPRB, 0.94143E+00_JPRB, 0.94961E+00_JPRB, 0.10529E+01_JPRB, 0.95905E+00_JPRB ,&
&  0.93191E+00_JPRB, 0.91912E+00_JPRB, 0.89487E+00_JPRB, 0.87967E+00_JPRB, 0.88887E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.84034E+00_JPRB, 0.79792E+00_JPRB, 0.76893E+00_JPRB, 0.74257E+00_JPRB, 0.72014E+00_JPRB, 0.70419E+00_JPRB, 0.70985E+00_JPRB ,&
&  0.70458E+00_JPRB, 0.67668E+00_JPRB, 0.65594E+00_JPRB, 0.65677E+00_JPRB, 0.65400E+00_JPRB, 0.65396E+00_JPRB, 0.65311E+00_JPRB ,&
&  0.64552E+00_JPRB, 0.63316E+00_JPRB, 0.62948E+00_JPRB, 0.63602E+00_JPRB, 0.66585E+00_JPRB, 0.70416E+00_JPRB, 0.71286E+00_JPRB ,&
&  0.70538E+00_JPRB, 0.69920E+00_JPRB, 0.67977E+00_JPRB, 0.66161E+00_JPRB, 0.65888E+00_JPRB, 0.67599E+00_JPRB, 0.73045E+00_JPRB ,&
&  0.11187E+01_JPRB, 0.27272E+01_JPRB, 0.13631E+01_JPRB, 0.21576E+01_JPRB, 0.14559E+01_JPRB, 0.11073E+01_JPRB, 0.86637E+00_JPRB ,&
&  0.86384E+00_JPRB, 0.93844E+00_JPRB, 0.91208E+00_JPRB, 0.86747E+00_JPRB, 0.80489E+00_JPRB, 0.76862E+00_JPRB, 0.75505E+00_JPRB ,&
&  0.78276E+00_JPRB, 0.76506E+00_JPRB, 0.75268E+00_JPRB, 0.75228E+00_JPRB, 0.74297E+00_JPRB, 0.74385E+00_JPRB, 0.75372E+00_JPRB ,&
&  0.74840E+00_JPRB, 0.73892E+00_JPRB, 0.73720E+00_JPRB, 0.80378E+00_JPRB, 0.85000E+00_JPRB, 0.10870E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.12059E+01_JPRB, 0.14543E+01_JPRB, 0.12315E+01_JPRB, 0.10674E+01_JPRB, 0.94832E+00_JPRB, 0.90876E+00_JPRB, 0.90532E+00_JPRB ,&
&  0.88322E+00_JPRB, 0.86622E+00_JPRB, 0.84898E+00_JPRB, 0.84924E+00_JPRB, 0.86751E+00_JPRB, 0.85235E+00_JPRB, 0.85440E+00_JPRB ,&
&  0.86242E+00_JPRB, 0.82745E+00_JPRB, 0.81422E+00_JPRB, 0.80407E+00_JPRB, 0.81188E+00_JPRB, 0.81482E+00_JPRB, 0.80738E+00_JPRB ,&
&  0.79975E+00_JPRB, 0.80194E+00_JPRB, 0.82265E+00_JPRB, 0.83632E+00_JPRB, 0.84973E+00_JPRB, 0.85458E+00_JPRB, 0.83710E+00_JPRB ,&
&  0.86406E+00_JPRB, 0.87907E+00_JPRB, 0.87997E+00_JPRB, 0.91977E+00_JPRB, 0.93357E+00_JPRB, 0.10003E+01_JPRB, 0.90637E+00_JPRB ,&
&  0.90650E+00_JPRB, 0.89518E+00_JPRB, 0.86461E+00_JPRB, 0.86289E+00_JPRB, 0.87314E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.78456E+00_JPRB, 0.75250E+00_JPRB, 0.72053E+00_JPRB, 0.68460E+00_JPRB, 0.66145E+00_JPRB, 0.66586E+00_JPRB, 0.68341E+00_JPRB ,&
&  0.66248E+00_JPRB, 0.63668E+00_JPRB, 0.62799E+00_JPRB, 0.64230E+00_JPRB, 0.66149E+00_JPRB, 0.65837E+00_JPRB, 0.65323E+00_JPRB ,&
&  0.63729E+00_JPRB, 0.63082E+00_JPRB, 0.61934E+00_JPRB, 0.62869E+00_JPRB, 0.67951E+00_JPRB, 0.67786E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.68769E+00_JPRB, 0.68443E+00_JPRB, 0.66358E+00_JPRB, 0.66183E+00_JPRB, 0.67190E+00_JPRB, 0.70980E+00_JPRB, 0.78512E+00_JPRB ,&
&  0.12146E+01_JPRB, 0.27030E+01_JPRB, 0.14865E+01_JPRB, 0.19566E+01_JPRB, 0.13259E+01_JPRB, 0.99065E+00_JPRB, 0.75576E+00_JPRB ,&
&  0.74737E+00_JPRB, 0.90073E+00_JPRB, 0.88649E+00_JPRB, 0.83309E+00_JPRB, 0.74587E+00_JPRB, 0.70864E+00_JPRB, 0.73769E+00_JPRB ,&
&  0.76447E+00_JPRB, 0.75497E+00_JPRB, 0.75856E+00_JPRB, 0.77517E+00_JPRB, 0.77749E+00_JPRB, 0.77953E+00_JPRB, 0.78858E+00_JPRB ,&
&  0.78632E+00_JPRB, 0.79750E+00_JPRB, 0.82688E+00_JPRB, 0.86102E+00_JPRB, 0.10220E+01_JPRB, 0.12998E+01_JPRB, 0.14181E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.14531E+01_JPRB, 0.12207E+01_JPRB, 0.10623E+01_JPRB, 0.93726E+00_JPRB, 0.90611E+00_JPRB, 0.86919E+00_JPRB ,&
&  0.83684E+00_JPRB, 0.82290E+00_JPRB, 0.79855E+00_JPRB, 0.80174E+00_JPRB, 0.84706E+00_JPRB, 0.86885E+00_JPRB, 0.85310E+00_JPRB ,&
&  0.86748E+00_JPRB, 0.86490E+00_JPRB, 0.86386E+00_JPRB, 0.84453E+00_JPRB, 0.84956E+00_JPRB, 0.84478E+00_JPRB, 0.83339E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.84691E+00_JPRB, 0.84370E+00_JPRB, 0.84692E+00_JPRB, 0.85331E+00_JPRB, 0.85906E+00_JPRB, 0.83367E+00_JPRB ,&
&  0.85771E+00_JPRB, 0.88233E+00_JPRB, 0.89508E+00_JPRB, 0.89277E+00_JPRB, 0.87792E+00_JPRB, 0.90984E+00_JPRB, 0.89344E+00_JPRB ,&
&  0.90615E+00_JPRB, 0.89512E+00_JPRB, 0.84736E+00_JPRB, 0.83398E+00_JPRB, 0.82990E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.74344E+00_JPRB, 0.70528E+00_JPRB, 0.66773E+00_JPRB, 0.61609E+00_JPRB, 0.62021E+00_JPRB, 0.63011E+00_JPRB, 0.60781E+00_JPRB ,&
&  0.56231E+00_JPRB, 0.55643E+00_JPRB, 0.56854E+00_JPRB, 0.59160E+00_JPRB, 0.61080E+00_JPRB, 0.60778E+00_JPRB, 0.61918E+00_JPRB ,&
&  0.60956E+00_JPRB, 0.60598E+00_JPRB, 0.62670E+00_JPRB, 0.66056E+00_JPRB, 0.66484E+00_JPRB, 0.66406E+00_JPRB, 0.67792E+00_JPRB ,&
&  0.68411E+00_JPRB, 0.67352E+00_JPRB, 0.66788E+00_JPRB, 0.67892E+00_JPRB, 0.71927E+00_JPRB, 0.78559E+00_JPRB, 0.89842E+00_JPRB ,&
&  0.13276E+01_JPRB, 0.29119E+01_JPRB, 0.21322E+01_JPRB, 0.16927E+01_JPRB, 0.11889E+01_JPRB, 0.87199E+00_JPRB, 0.71485E+00_JPRB ,&
&  0.69494E+00_JPRB, 0.71235E+00_JPRB, 0.87186E+00_JPRB, 0.79087E+00_JPRB, 0.70628E+00_JPRB, 0.67267E+00_JPRB, 0.73631E+00_JPRB ,&
&  0.78989E+00_JPRB, 0.80839E+00_JPRB, 0.82881E+00_JPRB, 0.83588E+00_JPRB, 0.84351E+00_JPRB, 0.84839E+00_JPRB, 0.84691E+00_JPRB ,&
&  0.85617E+00_JPRB, 0.90271E+00_JPRB, 0.96577E+00_JPRB, 0.10280E+01_JPRB, 0.11959E+01_JPRB, 0.14782E+01_JPRB, 0.15786E+01_JPRB ,&
&  0.14173E+01_JPRB, 0.13681E+01_JPRB, 0.11837E+01_JPRB, 0.10634E+01_JPRB, 0.88657E+00_JPRB, 0.89648E+00_JPRB, 0.83544E+00_JPRB ,&
&  0.80921E+00_JPRB, 0.77468E+00_JPRB, 0.77166E+00_JPRB, 0.78716E+00_JPRB, 0.80940E+00_JPRB, 0.83446E+00_JPRB, 0.84338E+00_JPRB ,&
&  0.85535E+00_JPRB, 0.86318E+00_JPRB, 0.86470E+00_JPRB, 0.85394E+00_JPRB, 0.85322E+00_JPRB, 0.85626E+00_JPRB, 0.86990E+00_JPRB ,&
&  0.87984E+00_JPRB, 0.90687E+00_JPRB, 0.88137E+00_JPRB, 0.87685E+00_JPRB, 0.88511E+00_JPRB, 0.88596E+00_JPRB, 0.84175E+00_JPRB ,&
&  0.85445E+00_JPRB, 0.90974E+00_JPRB, 0.90146E+00_JPRB, 0.88025E+00_JPRB, 0.86398E+00_JPRB, 0.87276E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.88530E+00_JPRB, 0.88436E+00_JPRB, 0.82660E+00_JPRB, 0.79403E+00_JPRB, 0.79311E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.71814E+00_JPRB, 0.68296E+00_JPRB, 0.62124E+00_JPRB, 0.59455E+00_JPRB, 0.59051E+00_JPRB, 0.58235E+00_JPRB, 0.56817E+00_JPRB ,&
&  0.54575E+00_JPRB, 0.53880E+00_JPRB, 0.56685E+00_JPRB, 0.59633E+00_JPRB, 0.62288E+00_JPRB, 0.62201E+00_JPRB, 0.62405E+00_JPRB ,&
&  0.62665E+00_JPRB, 0.64777E+00_JPRB, 0.68877E+00_JPRB, 0.69762E+00_JPRB, 0.68456E+00_JPRB, 0.67875E+00_JPRB, 0.68065E+00_JPRB ,&
&  0.68882E+00_JPRB, 0.70069E+00_JPRB, 0.72963E+00_JPRB, 0.76728E+00_JPRB, 0.82735E+00_JPRB, 0.92692E+00_JPRB, 0.11367E+01_JPRB ,&
&  0.15907E+01_JPRB, 0.26077E+01_JPRB, 0.74230E+00_JPRB, 0.95408E+00_JPRB, 0.89552E+00_JPRB, 0.76554E+00_JPRB, 0.67621E+00_JPRB ,&
&  0.62108E+00_JPRB, 0.63232E+00_JPRB, 0.73721E+00_JPRB, 0.75677E+00_JPRB, 0.67886E+00_JPRB, 0.73950E+00_JPRB, 0.84291E+00_JPRB ,&
&  0.92348E+00_JPRB, 0.93828E+00_JPRB, 0.93544E+00_JPRB, 0.92216E+00_JPRB, 0.92500E+00_JPRB, 0.93065E+00_JPRB, 0.94600E+00_JPRB ,&
&  0.98524E+00_JPRB, 0.10581E+01_JPRB, 0.11425E+01_JPRB, 0.11508E+01_JPRB, 0.15341E+01_JPRB, 0.18030E+01_JPRB, 0.17236E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.12821E+01_JPRB, 0.11493E+01_JPRB, 0.10669E+01_JPRB, 0.86388E+00_JPRB, 0.75200E+00_JPRB, 0.81699E+00_JPRB ,&
&  0.80620E+00_JPRB, 0.77419E+00_JPRB, 0.75621E+00_JPRB, 0.74344E+00_JPRB, 0.74581E+00_JPRB, 0.79788E+00_JPRB, 0.83584E+00_JPRB ,&
&  0.83649E+00_JPRB, 0.85864E+00_JPRB, 0.86055E+00_JPRB, 0.85275E+00_JPRB, 0.85949E+00_JPRB, 0.87229E+00_JPRB, 0.91035E+00_JPRB ,&
&  0.93182E+00_JPRB, 0.95420E+00_JPRB, 0.96707E+00_JPRB, 0.94997E+00_JPRB, 0.96502E+00_JPRB, 0.93259E+00_JPRB, 0.87890E+00_JPRB ,&
&  0.88691E+00_JPRB, 0.92936E+00_JPRB, 0.85782E+00_JPRB, 0.81280E+00_JPRB, 0.82746E+00_JPRB, 0.82030E+00_JPRB, 0.81425E+00_JPRB ,&
&  0.86242E+00_JPRB, 0.88820E+00_JPRB, 0.87061E+00_JPRB, 0.75062E+00_JPRB, 0.75877E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.69759E+00_JPRB, 0.63362E+00_JPRB, 0.59597E+00_JPRB, 0.59066E+00_JPRB, 0.57856E+00_JPRB, 0.58450E+00_JPRB, 0.59297E+00_JPRB ,&
&  0.58985E+00_JPRB, 0.60487E+00_JPRB, 0.60430E+00_JPRB, 0.61624E+00_JPRB, 0.63602E+00_JPRB, 0.64021E+00_JPRB, 0.66046E+00_JPRB ,&
&  0.68208E+00_JPRB, 0.70897E+00_JPRB, 0.73036E+00_JPRB, 0.69377E+00_JPRB, 0.68579E+00_JPRB, 0.68809E+00_JPRB, 0.70225E+00_JPRB ,&
&  0.73294E+00_JPRB, 0.77704E+00_JPRB, 0.82949E+00_JPRB, 0.89881E+00_JPRB, 0.99876E+00_JPRB, 0.11779E+01_JPRB, 0.15047E+01_JPRB ,&
&  0.23132E+01_JPRB, 0.18291E+01_JPRB, 0.61944E+00_JPRB, 0.75755E+00_JPRB, 0.73376E+00_JPRB, 0.70136E+00_JPRB, 0.59893E+00_JPRB ,&
&  0.58702E+00_JPRB, 0.64015E+00_JPRB, 0.68440E+00_JPRB, 0.73240E+00_JPRB, 0.78729E+00_JPRB, 0.88164E+00_JPRB, 0.96444E+00_JPRB ,&
&  0.10088E+01_JPRB, 0.10275E+01_JPRB, 0.10246E+01_JPRB, 0.10272E+01_JPRB, 0.10389E+01_JPRB, 0.10644E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.12458E+01_JPRB, 0.13469E+01_JPRB, 0.13204E+01_JPRB, 0.17308E+01_JPRB, 0.19198E+01_JPRB, 0.19092E+01_JPRB ,&
&  0.16132E+01_JPRB, 0.12924E+01_JPRB, 0.10925E+01_JPRB, 0.10068E+01_JPRB, 0.78898E+00_JPRB, 0.68424E+00_JPRB, 0.77877E+00_JPRB ,&
&  0.82622E+00_JPRB, 0.77494E+00_JPRB, 0.75644E+00_JPRB, 0.74725E+00_JPRB, 0.70806E+00_JPRB, 0.75768E+00_JPRB, 0.82700E+00_JPRB ,&
&  0.84005E+00_JPRB, 0.84254E+00_JPRB, 0.85932E+00_JPRB, 0.85244E+00_JPRB, 0.85123E+00_JPRB, 0.86778E+00_JPRB, 0.90576E+00_JPRB ,&
&  0.93076E+00_JPRB, 0.95660E+00_JPRB, 0.96624E+00_JPRB, 0.10017E+01_JPRB, 0.10353E+01_JPRB, 0.96566E+00_JPRB, 0.86699E+00_JPRB ,&
&  0.84167E+00_JPRB, 0.85093E+00_JPRB, 0.82524E+00_JPRB, 0.80494E+00_JPRB, 0.77847E+00_JPRB, 0.76810E+00_JPRB, 0.77903E+00_JPRB ,&
&  0.86938E+00_JPRB, 0.81485E+00_JPRB, 0.76179E+00_JPRB, 0.69625E+00_JPRB, 0.71887E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.61734E+00_JPRB, 0.58844E+00_JPRB, 0.58057E+00_JPRB, 0.56583E+00_JPRB, 0.57946E+00_JPRB, 0.60435E+00_JPRB, 0.62341E+00_JPRB ,&
&  0.62677E+00_JPRB, 0.62565E+00_JPRB, 0.62682E+00_JPRB, 0.64465E+00_JPRB, 0.66951E+00_JPRB, 0.69542E+00_JPRB, 0.72814E+00_JPRB ,&
&  0.74922E+00_JPRB, 0.76122E+00_JPRB, 0.73433E+00_JPRB, 0.69703E+00_JPRB, 0.70538E+00_JPRB, 0.72501E+00_JPRB, 0.75683E+00_JPRB ,&
&  0.80142E+00_JPRB, 0.85261E+00_JPRB, 0.92354E+00_JPRB, 0.10453E+01_JPRB, 0.12292E+01_JPRB, 0.14475E+01_JPRB, 0.16639E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.61441E+00_JPRB, 0.61622E+00_JPRB, 0.66244E+00_JPRB, 0.59661E+00_JPRB, 0.60221E+00_JPRB, 0.59835E+00_JPRB ,&
&  0.58447E+00_JPRB, 0.62523E+00_JPRB, 0.65921E+00_JPRB, 0.79257E+00_JPRB, 0.91772E+00_JPRB, 0.98653E+00_JPRB, 0.10399E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.11033E+01_JPRB, 0.11099E+01_JPRB, 0.11173E+01_JPRB, 0.11408E+01_JPRB, 0.11794E+01_JPRB, 0.12317E+01_JPRB ,&
&  0.13148E+01_JPRB, 0.14164E+01_JPRB, 0.15460E+01_JPRB, 0.14537E+01_JPRB, 0.17524E+01_JPRB, 0.20014E+01_JPRB, 0.22652E+01_JPRB ,&
&  0.22068E+01_JPRB, 0.13890E+01_JPRB, 0.10040E+01_JPRB, 0.10123E+01_JPRB, 0.87267E+00_JPRB, 0.70687E+00_JPRB, 0.81749E+00_JPRB ,&
&  0.84969E+00_JPRB, 0.85279E+00_JPRB, 0.84044E+00_JPRB, 0.76510E+00_JPRB, 0.69739E+00_JPRB, 0.68721E+00_JPRB, 0.74389E+00_JPRB ,&
&  0.77694E+00_JPRB, 0.79686E+00_JPRB, 0.79182E+00_JPRB, 0.79418E+00_JPRB, 0.77940E+00_JPRB, 0.79643E+00_JPRB, 0.83470E+00_JPRB ,&
&  0.87059E+00_JPRB, 0.90711E+00_JPRB, 0.92591E+00_JPRB, 0.95663E+00_JPRB, 0.99597E+00_JPRB, 0.93696E+00_JPRB, 0.85033E+00_JPRB ,&
&  0.82513E+00_JPRB, 0.81308E+00_JPRB, 0.77586E+00_JPRB, 0.75715E+00_JPRB, 0.75528E+00_JPRB, 0.80354E+00_JPRB, 0.91520E+00_JPRB ,&
&  0.90526E+00_JPRB, 0.80001E+00_JPRB, 0.68818E+00_JPRB, 0.65699E+00_JPRB, 0.66382E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.51813E+00_JPRB, 0.50996E+00_JPRB, 0.51089E+00_JPRB, 0.50889E+00_JPRB, 0.54093E+00_JPRB, 0.57605E+00_JPRB, 0.58623E+00_JPRB ,&
&  0.58053E+00_JPRB, 0.59782E+00_JPRB, 0.61463E+00_JPRB, 0.64541E+00_JPRB, 0.70846E+00_JPRB, 0.73597E+00_JPRB, 0.76843E+00_JPRB ,&
&  0.80112E+00_JPRB, 0.79775E+00_JPRB, 0.76705E+00_JPRB, 0.75129E+00_JPRB, 0.77255E+00_JPRB, 0.78872E+00_JPRB, 0.82209E+00_JPRB ,&
&  0.87147E+00_JPRB, 0.92876E+00_JPRB, 0.99905E+00_JPRB, 0.10847E+01_JPRB, 0.11826E+01_JPRB, 0.12659E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.54313E+00_JPRB, 0.51671E+00_JPRB, 0.61227E+00_JPRB, 0.59949E+00_JPRB, 0.55592E+00_JPRB, 0.56326E+00_JPRB, 0.58760E+00_JPRB ,&
&  0.60969E+00_JPRB, 0.59731E+00_JPRB, 0.66227E+00_JPRB, 0.89751E+00_JPRB, 0.94744E+00_JPRB, 0.10205E+01_JPRB, 0.10491E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.11200E+01_JPRB, 0.11484E+01_JPRB, 0.11722E+01_JPRB, 0.12076E+01_JPRB, 0.12575E+01_JPRB, 0.13290E+01_JPRB ,&
&  0.14206E+01_JPRB, 0.15806E+01_JPRB, 0.17531E+01_JPRB, 0.16646E+01_JPRB, 0.18204E+01_JPRB, 0.21173E+01_JPRB, 0.26325E+01_JPRB ,&
&  0.25302E+01_JPRB, 0.16141E+01_JPRB, 0.11059E+01_JPRB, 0.10275E+01_JPRB, 0.93252E+00_JPRB, 0.93633E+00_JPRB, 0.91802E+00_JPRB ,&
&  0.87430E+00_JPRB, 0.83892E+00_JPRB, 0.79446E+00_JPRB, 0.73168E+00_JPRB, 0.66347E+00_JPRB, 0.65445E+00_JPRB, 0.65403E+00_JPRB ,&
&  0.68930E+00_JPRB, 0.70349E+00_JPRB, 0.70710E+00_JPRB, 0.70853E+00_JPRB, 0.67149E+00_JPRB, 0.67633E+00_JPRB, 0.70900E+00_JPRB ,&
&  0.74343E+00_JPRB, 0.77229E+00_JPRB, 0.78478E+00_JPRB, 0.79585E+00_JPRB, 0.81259E+00_JPRB, 0.81802E+00_JPRB, 0.78879E+00_JPRB ,&
&  0.74477E+00_JPRB, 0.71444E+00_JPRB, 0.72418E+00_JPRB, 0.72238E+00_JPRB, 0.89085E+00_JPRB, 0.11961E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.10535E+01_JPRB, 0.80452E+00_JPRB, 0.64222E+00_JPRB, 0.59154E+00_JPRB, 0.56980E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.47764E+00_JPRB, 0.46699E+00_JPRB, 0.46785E+00_JPRB, 0.46711E+00_JPRB, 0.50133E+00_JPRB, 0.52427E+00_JPRB, 0.54523E+00_JPRB ,&
&  0.57662E+00_JPRB, 0.59679E+00_JPRB, 0.62920E+00_JPRB, 0.68751E+00_JPRB, 0.74929E+00_JPRB, 0.79751E+00_JPRB, 0.81799E+00_JPRB ,&
&  0.82856E+00_JPRB, 0.82456E+00_JPRB, 0.80979E+00_JPRB, 0.79050E+00_JPRB, 0.80083E+00_JPRB, 0.80229E+00_JPRB, 0.82713E+00_JPRB ,&
&  0.86717E+00_JPRB, 0.90961E+00_JPRB, 0.96149E+00_JPRB, 0.10102E+01_JPRB, 0.10593E+01_JPRB, 0.10940E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.50250E+00_JPRB, 0.67123E+00_JPRB, 0.69142E+00_JPRB, 0.66869E+00_JPRB, 0.61396E+00_JPRB, 0.57961E+00_JPRB, 0.59365E+00_JPRB ,&
&  0.63061E+00_JPRB, 0.63333E+00_JPRB, 0.69505E+00_JPRB, 0.75770E+00_JPRB, 0.89280E+00_JPRB, 0.94083E+00_JPRB, 0.99022E+00_JPRB ,&
&  0.10247E+01_JPRB, 0.10455E+01_JPRB, 0.10806E+01_JPRB, 0.11191E+01_JPRB, 0.11650E+01_JPRB, 0.12149E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.13475E+01_JPRB, 0.14356E+01_JPRB, 0.15596E+01_JPRB, 0.15171E+01_JPRB, 0.15068E+01_JPRB, 0.16849E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.15690E+01_JPRB, 0.10103E+01_JPRB, 0.90371E+00_JPRB, 0.98737E+00_JPRB, 0.98678E+00_JPRB, 0.98107E+00_JPRB, 0.94293E+00_JPRB ,&
&  0.84946E+00_JPRB, 0.79346E+00_JPRB, 0.73211E+00_JPRB, 0.68630E+00_JPRB, 0.64447E+00_JPRB, 0.61983E+00_JPRB, 0.60307E+00_JPRB ,&
&  0.59597E+00_JPRB, 0.61018E+00_JPRB, 0.62440E+00_JPRB, 0.60840E+00_JPRB, 0.57554E+00_JPRB, 0.57549E+00_JPRB, 0.61029E+00_JPRB ,&
&  0.68291E+00_JPRB, 0.62105E+00_JPRB, 0.58234E+00_JPRB, 0.58301E+00_JPRB, 0.58034E+00_JPRB, 0.57462E+00_JPRB, 0.56752E+00_JPRB ,&
&  0.57634E+00_JPRB, 0.57186E+00_JPRB, 0.66918E+00_JPRB, 0.10453E+01_JPRB, 0.14458E+01_JPRB, 0.16976E+01_JPRB, 0.13980E+01_JPRB ,&
&  0.98456E+00_JPRB, 0.72329E+00_JPRB, 0.57497E+00_JPRB, 0.53604E+00_JPRB, 0.51524E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.50455E+00_JPRB, 0.48664E+00_JPRB, 0.46942E+00_JPRB, 0.48067E+00_JPRB, 0.48860E+00_JPRB, 0.49417E+00_JPRB, 0.51995E+00_JPRB ,&
&  0.57397E+00_JPRB, 0.58774E+00_JPRB, 0.61499E+00_JPRB, 0.65834E+00_JPRB, 0.72106E+00_JPRB, 0.76853E+00_JPRB, 0.81184E+00_JPRB ,&
&  0.85609E+00_JPRB, 0.88095E+00_JPRB, 0.87214E+00_JPRB, 0.85543E+00_JPRB, 0.85027E+00_JPRB, 0.85280E+00_JPRB, 0.86325E+00_JPRB ,&
&  0.86333E+00_JPRB, 0.87835E+00_JPRB, 0.91043E+00_JPRB, 0.94940E+00_JPRB, 0.98301E+00_JPRB, 0.97360E+00_JPRB, 0.61129E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.78648E+00_JPRB, 0.79714E+00_JPRB, 0.76054E+00_JPRB, 0.69841E+00_JPRB, 0.63411E+00_JPRB, 0.60896E+00_JPRB ,&
&  0.61903E+00_JPRB, 0.63820E+00_JPRB, 0.65177E+00_JPRB, 0.68569E+00_JPRB, 0.79828E+00_JPRB, 0.82736E+00_JPRB, 0.88569E+00_JPRB ,&
&  0.91354E+00_JPRB, 0.94446E+00_JPRB, 0.97292E+00_JPRB, 0.99347E+00_JPRB, 0.10235E+01_JPRB, 0.10579E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10969E+01_JPRB, 0.10833E+01_JPRB, 0.10918E+01_JPRB, 0.10897E+01_JPRB, 0.10112E+01_JPRB, 0.11581E+01_JPRB, 0.97797E+00_JPRB ,&
&  0.68960E+00_JPRB, 0.55051E+00_JPRB, 0.72440E+00_JPRB, 0.99120E+00_JPRB, 0.98695E+00_JPRB, 0.97953E+00_JPRB, 0.90718E+00_JPRB ,&
&  0.80397E+00_JPRB, 0.72169E+00_JPRB, 0.68137E+00_JPRB, 0.65333E+00_JPRB, 0.62455E+00_JPRB, 0.59236E+00_JPRB, 0.56610E+00_JPRB ,&
&  0.53338E+00_JPRB, 0.50824E+00_JPRB, 0.51896E+00_JPRB, 0.50672E+00_JPRB, 0.50981E+00_JPRB, 0.55415E+00_JPRB, 0.68896E+00_JPRB ,&
&  0.10480E+01_JPRB, 0.57242E+00_JPRB, 0.49784E+00_JPRB, 0.48105E+00_JPRB, 0.45843E+00_JPRB, 0.44004E+00_JPRB, 0.42804E+00_JPRB ,&
&  0.43562E+00_JPRB, 0.49164E+00_JPRB, 0.63948E+00_JPRB, 0.13603E+01_JPRB, 0.25176E+01_JPRB, 0.25829E+01_JPRB, 0.17290E+01_JPRB ,&
&  0.10370E+01_JPRB, 0.71002E+00_JPRB, 0.56883E+00_JPRB, 0.53443E+00_JPRB, 0.50981E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.55430E+00_JPRB, 0.52322E+00_JPRB, 0.51318E+00_JPRB, 0.50865E+00_JPRB, 0.49911E+00_JPRB, 0.51776E+00_JPRB, 0.53221E+00_JPRB ,&
&  0.55287E+00_JPRB, 0.57971E+00_JPRB, 0.61264E+00_JPRB, 0.64148E+00_JPRB, 0.68343E+00_JPRB, 0.72384E+00_JPRB, 0.75551E+00_JPRB ,&
&  0.78033E+00_JPRB, 0.79863E+00_JPRB, 0.81089E+00_JPRB, 0.81542E+00_JPRB, 0.81408E+00_JPRB, 0.81203E+00_JPRB, 0.80699E+00_JPRB ,&
&  0.80148E+00_JPRB, 0.81471E+00_JPRB, 0.84846E+00_JPRB, 0.88797E+00_JPRB, 0.90824E+00_JPRB, 0.93422E+00_JPRB, 0.59357E+00_JPRB ,&
&  0.91022E+00_JPRB, 0.92891E+00_JPRB, 0.88584E+00_JPRB, 0.80486E+00_JPRB, 0.73504E+00_JPRB, 0.68626E+00_JPRB, 0.64534E+00_JPRB ,&
&  0.62095E+00_JPRB, 0.62028E+00_JPRB, 0.58335E+00_JPRB, 0.62545E+00_JPRB, 0.69674E+00_JPRB, 0.70583E+00_JPRB, 0.75212E+00_JPRB ,&
&  0.78819E+00_JPRB, 0.83261E+00_JPRB, 0.86361E+00_JPRB, 0.88986E+00_JPRB, 0.90642E+00_JPRB, 0.92708E+00_JPRB, 0.93022E+00_JPRB ,&
&  0.92577E+00_JPRB, 0.89610E+00_JPRB, 0.84394E+00_JPRB, 0.79607E+00_JPRB, 0.78500E+00_JPRB, 0.70339E+00_JPRB, 0.65202E+00_JPRB ,&
&  0.50203E+00_JPRB, 0.52078E+00_JPRB, 0.81419E+00_JPRB, 0.10812E+01_JPRB, 0.10001E+01_JPRB, 0.96461E+00_JPRB, 0.86728E+00_JPRB ,&
&  0.77619E+00_JPRB, 0.71848E+00_JPRB, 0.68759E+00_JPRB, 0.66301E+00_JPRB, 0.62281E+00_JPRB, 0.59324E+00_JPRB, 0.58159E+00_JPRB ,&
&  0.57077E+00_JPRB, 0.51245E+00_JPRB, 0.47880E+00_JPRB, 0.46558E+00_JPRB, 0.44097E+00_JPRB, 0.46208E+00_JPRB, 0.44088E+00_JPRB ,&
&  0.56879E+00_JPRB, 0.53734E+00_JPRB, 0.47569E+00_JPRB, 0.48694E+00_JPRB, 0.45398E+00_JPRB, 0.44834E+00_JPRB, 0.45370E+00_JPRB ,&
&  0.48135E+00_JPRB, 0.43518E+00_JPRB, 0.92962E+00_JPRB, 0.37082E+01_JPRB, 0.36687E+01_JPRB, 0.46594E+01_JPRB, 0.24178E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.69075E+00_JPRB, 0.59458E+00_JPRB, 0.58518E+00_JPRB, 0.56693E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.57579E+00_JPRB, 0.56745E+00_JPRB, 0.54834E+00_JPRB, 0.51877E+00_JPRB, 0.50747E+00_JPRB, 0.52542E+00_JPRB, 0.55053E+00_JPRB ,&
&  0.56912E+00_JPRB, 0.59210E+00_JPRB, 0.61645E+00_JPRB, 0.64282E+00_JPRB, 0.67185E+00_JPRB, 0.69892E+00_JPRB, 0.72002E+00_JPRB ,&
&  0.73483E+00_JPRB, 0.75470E+00_JPRB, 0.77170E+00_JPRB, 0.78526E+00_JPRB, 0.78893E+00_JPRB, 0.79070E+00_JPRB, 0.79042E+00_JPRB ,&
&  0.82239E+00_JPRB, 0.85358E+00_JPRB, 0.89338E+00_JPRB, 0.93255E+00_JPRB, 0.95282E+00_JPRB, 0.92988E+00_JPRB, 0.70841E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.10398E+01_JPRB, 0.93285E+00_JPRB, 0.83386E+00_JPRB, 0.75870E+00_JPRB, 0.70632E+00_JPRB, 0.65942E+00_JPRB ,&
&  0.62606E+00_JPRB, 0.63945E+00_JPRB, 0.60977E+00_JPRB, 0.59127E+00_JPRB, 0.58486E+00_JPRB, 0.61417E+00_JPRB, 0.64818E+00_JPRB ,&
&  0.70683E+00_JPRB, 0.76791E+00_JPRB, 0.82373E+00_JPRB, 0.86477E+00_JPRB, 0.88760E+00_JPRB, 0.90577E+00_JPRB, 0.87551E+00_JPRB ,&
&  0.87030E+00_JPRB, 0.86523E+00_JPRB, 0.79137E+00_JPRB, 0.76608E+00_JPRB, 0.77674E+00_JPRB, 0.71082E+00_JPRB, 0.65528E+00_JPRB ,&
&  0.56309E+00_JPRB, 0.68386E+00_JPRB, 0.96419E+00_JPRB, 0.11231E+01_JPRB, 0.97572E+00_JPRB, 0.93006E+00_JPRB, 0.86094E+00_JPRB ,&
&  0.78044E+00_JPRB, 0.75316E+00_JPRB, 0.73987E+00_JPRB, 0.71818E+00_JPRB, 0.66885E+00_JPRB, 0.63544E+00_JPRB, 0.63447E+00_JPRB ,&
&  0.63988E+00_JPRB, 0.57621E+00_JPRB, 0.51649E+00_JPRB, 0.49798E+00_JPRB, 0.45888E+00_JPRB, 0.39483E+00_JPRB, 0.51303E+00_JPRB ,&
&  0.47934E+00_JPRB, 0.39898E+00_JPRB, 0.51056E+00_JPRB, 0.38559E+00_JPRB, 0.49309E+00_JPRB, 0.50034E+00_JPRB, 0.52738E+00_JPRB ,&
&  0.45651E+00_JPRB, 0.52618E+00_JPRB, 0.88401E+00_JPRB, 0.12116E+01_JPRB, 0.13939E+01_JPRB, 0.13791E+01_JPRB, 0.77392E+00_JPRB ,&
&  0.67531E+00_JPRB, 0.64613E+00_JPRB, 0.62498E+00_JPRB, 0.60278E+00_JPRB, 0.57894E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.59707E+00_JPRB, 0.59786E+00_JPRB, 0.57834E+00_JPRB, 0.55078E+00_JPRB, 0.53808E+00_JPRB, 0.54779E+00_JPRB, 0.56412E+00_JPRB ,&
&  0.57918E+00_JPRB, 0.59931E+00_JPRB, 0.62256E+00_JPRB, 0.64496E+00_JPRB, 0.66461E+00_JPRB, 0.68582E+00_JPRB, 0.71096E+00_JPRB ,&
&  0.73473E+00_JPRB, 0.76420E+00_JPRB, 0.79386E+00_JPRB, 0.82371E+00_JPRB, 0.84937E+00_JPRB, 0.89440E+00_JPRB, 0.92515E+00_JPRB ,&
&  0.97458E+00_JPRB, 0.10242E+01_JPRB, 0.10851E+01_JPRB, 0.11429E+01_JPRB, 0.12122E+01_JPRB, 0.12511E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.10941E+01_JPRB, 0.95673E+00_JPRB, 0.85754E+00_JPRB, 0.79792E+00_JPRB, 0.75332E+00_JPRB, 0.71368E+00_JPRB ,&
&  0.69752E+00_JPRB, 0.66800E+00_JPRB, 0.63456E+00_JPRB, 0.61297E+00_JPRB, 0.60328E+00_JPRB, 0.63178E+00_JPRB, 0.62337E+00_JPRB ,&
&  0.66655E+00_JPRB, 0.74643E+00_JPRB, 0.80596E+00_JPRB, 0.89768E+00_JPRB, 0.97955E+00_JPRB, 0.10353E+01_JPRB, 0.10216E+01_JPRB ,&
&  0.10246E+01_JPRB, 0.10665E+01_JPRB, 0.10312E+01_JPRB, 0.10330E+01_JPRB, 0.10234E+01_JPRB, 0.10039E+01_JPRB, 0.98664E+00_JPRB ,&
&  0.84621E+00_JPRB, 0.88125E+00_JPRB, 0.10009E+01_JPRB, 0.11058E+01_JPRB, 0.98223E+00_JPRB, 0.90389E+00_JPRB, 0.87822E+00_JPRB ,&
&  0.80720E+00_JPRB, 0.79528E+00_JPRB, 0.79694E+00_JPRB, 0.76675E+00_JPRB, 0.74439E+00_JPRB, 0.72047E+00_JPRB, 0.72082E+00_JPRB ,&
&  0.73001E+00_JPRB, 0.64938E+00_JPRB, 0.58938E+00_JPRB, 0.54195E+00_JPRB, 0.50569E+00_JPRB, 0.46020E+00_JPRB, 0.49940E+00_JPRB ,&
&  0.50103E+00_JPRB, 0.36642E+00_JPRB, 0.45987E+00_JPRB, 0.58701E+00_JPRB, 0.57804E+00_JPRB, 0.60144E+00_JPRB, 0.64584E+00_JPRB ,&
&  0.69162E+00_JPRB, 0.71426E+00_JPRB, 0.70128E+00_JPRB, 0.71876E+00_JPRB, 0.74865E+00_JPRB, 0.73330E+00_JPRB, 0.67523E+00_JPRB ,&
&  0.68010E+00_JPRB, 0.64368E+00_JPRB, 0.62326E+00_JPRB, 0.61405E+00_JPRB, 0.60687E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.61280E+00_JPRB, 0.61920E+00_JPRB, 0.60790E+00_JPRB, 0.58788E+00_JPRB, 0.57290E+00_JPRB, 0.56691E+00_JPRB, 0.57564E+00_JPRB ,&
&  0.58577E+00_JPRB, 0.60674E+00_JPRB, 0.62396E+00_JPRB, 0.64235E+00_JPRB, 0.65160E+00_JPRB, 0.65877E+00_JPRB, 0.67354E+00_JPRB ,&
&  0.69307E+00_JPRB, 0.71414E+00_JPRB, 0.74095E+00_JPRB, 0.77859E+00_JPRB, 0.81369E+00_JPRB, 0.85678E+00_JPRB, 0.87593E+00_JPRB ,&
&  0.87770E+00_JPRB, 0.90430E+00_JPRB, 0.92433E+00_JPRB, 0.96620E+00_JPRB, 0.11584E+01_JPRB, 0.13506E+01_JPRB, 0.15388E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.11016E+01_JPRB, 0.94934E+00_JPRB, 0.85283E+00_JPRB, 0.85653E+00_JPRB, 0.83328E+00_JPRB, 0.83052E+00_JPRB ,&
&  0.83589E+00_JPRB, 0.82882E+00_JPRB, 0.82804E+00_JPRB, 0.83385E+00_JPRB, 0.84581E+00_JPRB, 0.88051E+00_JPRB, 0.84909E+00_JPRB ,&
&  0.83192E+00_JPRB, 0.86488E+00_JPRB, 0.90838E+00_JPRB, 0.10236E+01_JPRB, 0.11837E+01_JPRB, 0.13175E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.14203E+01_JPRB, 0.13622E+01_JPRB, 0.13476E+01_JPRB, 0.13277E+01_JPRB, 0.13116E+01_JPRB, 0.13226E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.11771E+01_JPRB, 0.11664E+01_JPRB, 0.10490E+01_JPRB, 0.98450E+00_JPRB, 0.93877E+00_JPRB, 0.90877E+00_JPRB ,&
&  0.87995E+00_JPRB, 0.87880E+00_JPRB, 0.87935E+00_JPRB, 0.85118E+00_JPRB, 0.83888E+00_JPRB, 0.82182E+00_JPRB, 0.83101E+00_JPRB ,&
&  0.83037E+00_JPRB, 0.76335E+00_JPRB, 0.69617E+00_JPRB, 0.62427E+00_JPRB, 0.52316E+00_JPRB, 0.57342E+00_JPRB, 0.62235E+00_JPRB ,&
&  0.57945E+00_JPRB, 0.48445E+00_JPRB, 0.50596E+00_JPRB, 0.71898E+00_JPRB, 0.68212E+00_JPRB, 0.68359E+00_JPRB, 0.70052E+00_JPRB ,&
&  0.72159E+00_JPRB, 0.73497E+00_JPRB, 0.74275E+00_JPRB, 0.72909E+00_JPRB, 0.72390E+00_JPRB, 0.71965E+00_JPRB, 0.70132E+00_JPRB ,&
&  0.68723E+00_JPRB, 0.66962E+00_JPRB, 0.66456E+00_JPRB, 0.63271E+00_JPRB, 0.61624E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.61903E+00_JPRB, 0.61139E+00_JPRB, 0.61992E+00_JPRB, 0.60265E+00_JPRB, 0.62416E+00_JPRB, 0.61240E+00_JPRB, 0.64139E+00_JPRB ,&
&  0.61784E+00_JPRB, 0.61302E+00_JPRB, 0.61396E+00_JPRB, 0.62621E+00_JPRB, 0.63971E+00_JPRB, 0.65884E+00_JPRB, 0.67441E+00_JPRB ,&
&  0.69827E+00_JPRB, 0.74367E+00_JPRB, 0.80673E+00_JPRB, 0.88157E+00_JPRB, 0.98136E+00_JPRB, 0.10786E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.13723E+01_JPRB, 0.14258E+01_JPRB, 0.15687E+01_JPRB, 0.19713E+01_JPRB, 0.24496E+01_JPRB, 0.24624E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.10489E+01_JPRB, 0.91967E+00_JPRB, 0.83596E+00_JPRB, 0.85689E+00_JPRB, 0.93023E+00_JPRB, 0.94142E+00_JPRB ,&
&  0.95139E+00_JPRB, 0.96790E+00_JPRB, 0.10003E+01_JPRB, 0.10359E+01_JPRB, 0.10886E+01_JPRB, 0.11569E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.11873E+01_JPRB, 0.11960E+01_JPRB, 0.12791E+01_JPRB, 0.13919E+01_JPRB, 0.15290E+01_JPRB, 0.16502E+01_JPRB ,&
&  0.17281E+01_JPRB, 0.16734E+01_JPRB, 0.14140E+01_JPRB, 0.13916E+01_JPRB, 0.14620E+01_JPRB, 0.14617E+01_JPRB, 0.14130E+01_JPRB ,&
&  0.15192E+01_JPRB, 0.14614E+01_JPRB, 0.10839E+01_JPRB, 0.10153E+01_JPRB, 0.99740E+00_JPRB, 0.10012E+01_JPRB, 0.95830E+00_JPRB ,&
&  0.97201E+00_JPRB, 0.98077E+00_JPRB, 0.98927E+00_JPRB, 0.97647E+00_JPRB, 0.95483E+00_JPRB, 0.93248E+00_JPRB, 0.94608E+00_JPRB ,&
&  0.90012E+00_JPRB, 0.88903E+00_JPRB, 0.86293E+00_JPRB, 0.79879E+00_JPRB, 0.79997E+00_JPRB, 0.71792E+00_JPRB, 0.80733E+00_JPRB ,&
&  0.71904E+00_JPRB, 0.70796E+00_JPRB, 0.63634E+00_JPRB, 0.80126E+00_JPRB, 0.79224E+00_JPRB, 0.74561E+00_JPRB, 0.76242E+00_JPRB ,&
&  0.77427E+00_JPRB, 0.79373E+00_JPRB, 0.78310E+00_JPRB, 0.78154E+00_JPRB, 0.78587E+00_JPRB, 0.77532E+00_JPRB, 0.76241E+00_JPRB ,&
&  0.74728E+00_JPRB, 0.72984E+00_JPRB, 0.69872E+00_JPRB, 0.66946E+00_JPRB, 0.64206E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.67427E+00_JPRB, 0.66933E+00_JPRB, 0.67198E+00_JPRB, 0.67824E+00_JPRB, 0.67224E+00_JPRB, 0.66716E+00_JPRB, 0.66192E+00_JPRB ,&
&  0.63408E+00_JPRB, 0.63172E+00_JPRB, 0.62114E+00_JPRB, 0.62966E+00_JPRB, 0.65712E+00_JPRB, 0.68463E+00_JPRB, 0.73039E+00_JPRB ,&
&  0.77639E+00_JPRB, 0.83621E+00_JPRB, 0.92733E+00_JPRB, 0.11189E+01_JPRB, 0.13351E+01_JPRB, 0.14945E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.17485E+01_JPRB, 0.18139E+01_JPRB, 0.19398E+01_JPRB, 0.21103E+01_JPRB, 0.21965E+01_JPRB, 0.22502E+01_JPRB, 0.21800E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.10365E+01_JPRB, 0.10039E+01_JPRB, 0.90982E+00_JPRB, 0.91129E+00_JPRB, 0.92535E+00_JPRB, 0.94761E+00_JPRB ,&
&  0.99087E+00_JPRB, 0.10287E+01_JPRB, 0.10688E+01_JPRB, 0.11061E+01_JPRB, 0.11660E+01_JPRB, 0.12327E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.12678E+01_JPRB, 0.12907E+01_JPRB, 0.13161E+01_JPRB, 0.12849E+01_JPRB, 0.13212E+01_JPRB, 0.13844E+01_JPRB, 0.14484E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.14748E+01_JPRB, 0.12475E+01_JPRB, 0.12547E+01_JPRB, 0.13958E+01_JPRB, 0.13105E+01_JPRB, 0.13361E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.15680E+01_JPRB, 0.10184E+01_JPRB, 0.91450E+00_JPRB, 0.95398E+00_JPRB, 0.97361E+00_JPRB, 0.10247E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.10341E+01_JPRB, 0.10710E+01_JPRB, 0.10906E+01_JPRB, 0.11050E+01_JPRB, 0.10727E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.96934E+00_JPRB, 0.94666E+00_JPRB, 0.95731E+00_JPRB, 0.96416E+00_JPRB, 0.97924E+00_JPRB, 0.10063E+01_JPRB, 0.97496E+00_JPRB ,&
&  0.88056E+00_JPRB, 0.84504E+00_JPRB, 0.90143E+00_JPRB, 0.84669E+00_JPRB, 0.77693E+00_JPRB, 0.81673E+00_JPRB, 0.79372E+00_JPRB ,&
&  0.81990E+00_JPRB, 0.82757E+00_JPRB, 0.82652E+00_JPRB, 0.82120E+00_JPRB, 0.81552E+00_JPRB, 0.80157E+00_JPRB, 0.78627E+00_JPRB ,&
&  0.76811E+00_JPRB, 0.74704E+00_JPRB, 0.72461E+00_JPRB, 0.70473E+00_JPRB, 0.68741E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.68798E+00_JPRB, 0.68954E+00_JPRB, 0.69671E+00_JPRB, 0.69581E+00_JPRB, 0.68981E+00_JPRB, 0.69542E+00_JPRB, 0.69292E+00_JPRB ,&
&  0.68052E+00_JPRB, 0.66895E+00_JPRB, 0.65457E+00_JPRB, 0.66677E+00_JPRB, 0.68956E+00_JPRB, 0.73463E+00_JPRB, 0.79244E+00_JPRB ,&
&  0.87298E+00_JPRB, 0.99230E+00_JPRB, 0.11511E+01_JPRB, 0.12789E+01_JPRB, 0.14477E+01_JPRB, 0.15218E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.16159E+01_JPRB, 0.16857E+01_JPRB, 0.17348E+01_JPRB, 0.17499E+01_JPRB, 0.17189E+01_JPRB, 0.17294E+01_JPRB, 0.18044E+01_JPRB ,&
&  0.14948E+01_JPRB, 0.11083E+01_JPRB, 0.98277E+00_JPRB, 0.90744E+00_JPRB, 0.87974E+00_JPRB, 0.87292E+00_JPRB, 0.89942E+00_JPRB ,&
&  0.94403E+00_JPRB, 0.99157E+00_JPRB, 0.10538E+01_JPRB, 0.11059E+01_JPRB, 0.11567E+01_JPRB, 0.11924E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12410E+01_JPRB, 0.12680E+01_JPRB, 0.13145E+01_JPRB, 0.12442E+01_JPRB, 0.13113E+01_JPRB, 0.13061E+01_JPRB, 0.12977E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.11622E+01_JPRB, 0.11908E+01_JPRB, 0.12035E+01_JPRB, 0.12193E+01_JPRB, 0.12095E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.13269E+01_JPRB, 0.13525E+01_JPRB, 0.10584E+01_JPRB, 0.98194E+00_JPRB, 0.10300E+01_JPRB, 0.91172E+00_JPRB, 0.10575E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10790E+01_JPRB, 0.11042E+01_JPRB, 0.11690E+01_JPRB, 0.12395E+01_JPRB, 0.12930E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.11156E+01_JPRB, 0.11443E+01_JPRB, 0.11776E+01_JPRB, 0.12426E+01_JPRB, 0.13132E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.98860E+00_JPRB, 0.96882E+00_JPRB, 0.90445E+00_JPRB, 0.85603E+00_JPRB, 0.88691E+00_JPRB, 0.83674E+00_JPRB ,&
&  0.82457E+00_JPRB, 0.82836E+00_JPRB, 0.81920E+00_JPRB, 0.80152E+00_JPRB, 0.79094E+00_JPRB, 0.77049E+00_JPRB, 0.75947E+00_JPRB ,&
&  0.75359E+00_JPRB, 0.73821E+00_JPRB, 0.72465E+00_JPRB, 0.71766E+00_JPRB, 0.69871E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.70131E+00_JPRB, 0.69490E+00_JPRB, 0.70225E+00_JPRB, 0.69676E+00_JPRB, 0.68998E+00_JPRB, 0.69384E+00_JPRB, 0.69977E+00_JPRB ,&
&  0.69674E+00_JPRB, 0.69571E+00_JPRB, 0.69032E+00_JPRB, 0.68850E+00_JPRB, 0.69372E+00_JPRB, 0.72083E+00_JPRB, 0.77229E+00_JPRB ,&
&  0.84207E+00_JPRB, 0.95152E+00_JPRB, 0.11025E+01_JPRB, 0.12430E+01_JPRB, 0.14000E+01_JPRB, 0.14686E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.14787E+01_JPRB, 0.15339E+01_JPRB, 0.15194E+01_JPRB, 0.14860E+01_JPRB, 0.12077E+01_JPRB, 0.12617E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.12865E+01_JPRB, 0.10511E+01_JPRB, 0.91516E+00_JPRB, 0.88117E+00_JPRB, 0.84490E+00_JPRB, 0.83306E+00_JPRB, 0.83933E+00_JPRB ,&
&  0.86396E+00_JPRB, 0.90562E+00_JPRB, 0.96624E+00_JPRB, 0.10461E+01_JPRB, 0.11032E+01_JPRB, 0.11266E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.11610E+01_JPRB, 0.11641E+01_JPRB, 0.11685E+01_JPRB, 0.11533E+01_JPRB, 0.11431E+01_JPRB, 0.11296E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.11384E+01_JPRB, 0.10726E+01_JPRB, 0.10739E+01_JPRB, 0.10942E+01_JPRB, 0.10525E+01_JPRB, 0.10067E+01_JPRB, 0.10367E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.13400E+01_JPRB, 0.10992E+01_JPRB, 0.11740E+01_JPRB, 0.13557E+01_JPRB, 0.11311E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10816E+01_JPRB, 0.11124E+01_JPRB, 0.11456E+01_JPRB, 0.12084E+01_JPRB, 0.13074E+01_JPRB, 0.14395E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.12345E+01_JPRB, 0.12983E+01_JPRB, 0.13550E+01_JPRB, 0.13821E+01_JPRB, 0.13490E+01_JPRB, 0.14419E+01_JPRB ,&
&  0.12176E+01_JPRB, 0.11710E+01_JPRB, 0.11921E+01_JPRB, 0.12501E+01_JPRB, 0.13695E+01_JPRB, 0.95240E+00_JPRB, 0.92756E+00_JPRB ,&
&  0.87458E+00_JPRB, 0.85796E+00_JPRB, 0.84306E+00_JPRB, 0.82490E+00_JPRB, 0.80851E+00_JPRB, 0.79142E+00_JPRB, 0.78032E+00_JPRB ,&
&  0.76977E+00_JPRB, 0.75324E+00_JPRB, 0.74155E+00_JPRB, 0.73449E+00_JPRB, 0.71389E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.72909E+00_JPRB, 0.71946E+00_JPRB, 0.71976E+00_JPRB, 0.72131E+00_JPRB, 0.72512E+00_JPRB, 0.72262E+00_JPRB, 0.71325E+00_JPRB ,&
&  0.69081E+00_JPRB, 0.68247E+00_JPRB, 0.68348E+00_JPRB, 0.68396E+00_JPRB, 0.67228E+00_JPRB, 0.66795E+00_JPRB, 0.68786E+00_JPRB ,&
&  0.73882E+00_JPRB, 0.82378E+00_JPRB, 0.94167E+00_JPRB, 0.10831E+01_JPRB, 0.12655E+01_JPRB, 0.13913E+01_JPRB, 0.14228E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.15629E+01_JPRB, 0.14095E+01_JPRB, 0.10192E+01_JPRB, 0.88892E+00_JPRB, 0.10215E+01_JPRB, 0.99090E+00_JPRB ,&
&  0.94325E+00_JPRB, 0.92196E+00_JPRB, 0.90663E+00_JPRB, 0.88784E+00_JPRB, 0.85723E+00_JPRB, 0.83647E+00_JPRB, 0.81748E+00_JPRB ,&
&  0.81386E+00_JPRB, 0.82661E+00_JPRB, 0.86591E+00_JPRB, 0.94230E+00_JPRB, 0.10190E+01_JPRB, 0.10671E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.10988E+01_JPRB, 0.10957E+01_JPRB, 0.10957E+01_JPRB, 0.10812E+01_JPRB, 0.10541E+01_JPRB, 0.10568E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10308E+01_JPRB, 0.10159E+01_JPRB, 0.10194E+01_JPRB, 0.10080E+01_JPRB, 0.91347E+00_JPRB, 0.93284E+00_JPRB ,&
&  0.10345E+01_JPRB, 0.11943E+01_JPRB, 0.11922E+01_JPRB, 0.13968E+01_JPRB, 0.92038E+00_JPRB, 0.96681E+00_JPRB, 0.10437E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.10808E+01_JPRB, 0.11255E+01_JPRB, 0.11826E+01_JPRB, 0.12753E+01_JPRB, 0.12942E+01_JPRB, 0.14125E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.12988E+01_JPRB, 0.13494E+01_JPRB, 0.14541E+01_JPRB, 0.15080E+01_JPRB, 0.14347E+01_JPRB, 0.13928E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.13003E+01_JPRB, 0.12006E+01_JPRB, 0.12015E+01_JPRB, 0.11441E+01_JPRB, 0.93386E+00_JPRB, 0.93222E+00_JPRB ,&
&  0.92002E+00_JPRB, 0.90005E+00_JPRB, 0.89190E+00_JPRB, 0.86870E+00_JPRB, 0.85153E+00_JPRB, 0.83618E+00_JPRB, 0.82145E+00_JPRB ,&
&  0.81293E+00_JPRB, 0.80010E+00_JPRB, 0.78308E+00_JPRB, 0.76612E+00_JPRB, 0.74710E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.76582E+00_JPRB, 0.75722E+00_JPRB, 0.75972E+00_JPRB, 0.75596E+00_JPRB, 0.75739E+00_JPRB, 0.76508E+00_JPRB, 0.75602E+00_JPRB ,&
&  0.70477E+00_JPRB, 0.66241E+00_JPRB, 0.65268E+00_JPRB, 0.65608E+00_JPRB, 0.65468E+00_JPRB, 0.64810E+00_JPRB, 0.65244E+00_JPRB ,&
&  0.67351E+00_JPRB, 0.71509E+00_JPRB, 0.78129E+00_JPRB, 0.92361E+00_JPRB, 0.11114E+01_JPRB, 0.13060E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.14295E+01_JPRB, 0.10786E+01_JPRB, 0.12986E+01_JPRB, 0.12782E+01_JPRB, 0.10913E+01_JPRB, 0.10112E+01_JPRB, 0.98384E+00_JPRB ,&
&  0.96281E+00_JPRB, 0.96512E+00_JPRB, 0.96151E+00_JPRB, 0.93471E+00_JPRB, 0.87604E+00_JPRB, 0.85134E+00_JPRB, 0.82288E+00_JPRB ,&
&  0.80920E+00_JPRB, 0.80303E+00_JPRB, 0.81502E+00_JPRB, 0.86227E+00_JPRB, 0.92205E+00_JPRB, 0.99087E+00_JPRB, 0.10515E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10374E+01_JPRB, 0.10334E+01_JPRB, 0.99984E+00_JPRB, 0.10105E+01_JPRB, 0.10230E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.99702E+00_JPRB, 0.10209E+01_JPRB, 0.10462E+01_JPRB, 0.10557E+01_JPRB, 0.96617E+00_JPRB, 0.10029E+01_JPRB ,&
&  0.10174E+01_JPRB, 0.10878E+01_JPRB, 0.11617E+01_JPRB, 0.12929E+01_JPRB, 0.99085E+00_JPRB, 0.10388E+01_JPRB, 0.93922E+00_JPRB ,&
&  0.97925E+00_JPRB, 0.98672E+00_JPRB, 0.10302E+01_JPRB, 0.10715E+01_JPRB, 0.11696E+01_JPRB, 0.12360E+01_JPRB, 0.14637E+01_JPRB ,&
&  0.14135E+01_JPRB, 0.13972E+01_JPRB, 0.13937E+01_JPRB, 0.15238E+01_JPRB, 0.14957E+01_JPRB, 0.15009E+01_JPRB, 0.15134E+01_JPRB ,&
&  0.16298E+01_JPRB, 0.15162E+01_JPRB, 0.14068E+01_JPRB, 0.13231E+01_JPRB, 0.12491E+01_JPRB, 0.10643E+01_JPRB, 0.99797E+00_JPRB ,&
&  0.95935E+00_JPRB, 0.95745E+00_JPRB, 0.94934E+00_JPRB, 0.93621E+00_JPRB, 0.91790E+00_JPRB, 0.89464E+00_JPRB, 0.87237E+00_JPRB ,&
&  0.85898E+00_JPRB, 0.84240E+00_JPRB, 0.82635E+00_JPRB, 0.81052E+00_JPRB, 0.78938E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.80796E+00_JPRB, 0.78636E+00_JPRB, 0.78417E+00_JPRB, 0.79678E+00_JPRB, 0.80794E+00_JPRB, 0.85748E+00_JPRB, 0.91038E+00_JPRB ,&
&  0.10423E+01_JPRB, 0.74662E+00_JPRB, 0.65858E+00_JPRB, 0.63218E+00_JPRB, 0.63151E+00_JPRB, 0.63547E+00_JPRB, 0.64250E+00_JPRB ,&
&  0.65071E+00_JPRB, 0.67527E+00_JPRB, 0.70878E+00_JPRB, 0.76424E+00_JPRB, 0.92909E+00_JPRB, 0.11422E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.13062E+01_JPRB, 0.15302E+01_JPRB, 0.13389E+01_JPRB, 0.12295E+01_JPRB, 0.12352E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.99694E+00_JPRB, 0.97152E+00_JPRB, 0.91711E+00_JPRB, 0.89328E+00_JPRB, 0.87184E+00_JPRB, 0.85186E+00_JPRB ,&
&  0.82792E+00_JPRB, 0.81126E+00_JPRB, 0.80718E+00_JPRB, 0.83152E+00_JPRB, 0.86700E+00_JPRB, 0.91784E+00_JPRB, 0.99731E+00_JPRB ,&
&  0.10404E+01_JPRB, 0.10149E+01_JPRB, 0.99289E+00_JPRB, 0.95523E+00_JPRB, 0.98247E+00_JPRB, 0.10033E+01_JPRB, 0.10066E+01_JPRB ,&
&  0.10052E+01_JPRB, 0.10432E+01_JPRB, 0.10695E+01_JPRB, 0.10453E+01_JPRB, 0.97844E+00_JPRB, 0.10666E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10462E+01_JPRB, 0.10289E+01_JPRB, 0.11805E+01_JPRB, 0.98628E+00_JPRB, 0.11029E+01_JPRB, 0.11365E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.96964E+00_JPRB, 0.94556E+00_JPRB, 0.92728E+00_JPRB, 0.93784E+00_JPRB, 0.10342E+01_JPRB, 0.11069E+01_JPRB, 0.13459E+01_JPRB ,&
&  0.13516E+01_JPRB, 0.15042E+01_JPRB, 0.14437E+01_JPRB, 0.14538E+01_JPRB, 0.13685E+01_JPRB, 0.13709E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.17938E+01_JPRB, 0.17120E+01_JPRB, 0.15483E+01_JPRB, 0.13827E+01_JPRB, 0.12469E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.10814E+01_JPRB, 0.10364E+01_JPRB, 0.10179E+01_JPRB, 0.98960E+00_JPRB, 0.95415E+00_JPRB, 0.91358E+00_JPRB ,&
&  0.89217E+00_JPRB, 0.87550E+00_JPRB, 0.86265E+00_JPRB, 0.85101E+00_JPRB, 0.83530E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.84454E+00_JPRB, 0.80927E+00_JPRB, 0.77358E+00_JPRB, 0.76413E+00_JPRB, 0.75816E+00_JPRB, 0.77253E+00_JPRB, 0.80919E+00_JPRB ,&
&  0.89384E+00_JPRB, 0.75140E+00_JPRB, 0.67661E+00_JPRB, 0.62583E+00_JPRB, 0.60984E+00_JPRB, 0.61688E+00_JPRB, 0.62342E+00_JPRB ,&
&  0.63210E+00_JPRB, 0.64321E+00_JPRB, 0.66311E+00_JPRB, 0.66958E+00_JPRB, 0.76761E+00_JPRB, 0.93335E+00_JPRB, 0.91556E+00_JPRB ,&
&  0.97409E+00_JPRB, 0.16112E+01_JPRB, 0.15007E+01_JPRB, 0.14152E+01_JPRB, 0.13385E+01_JPRB, 0.14022E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10151E+01_JPRB, 0.10014E+01_JPRB, 0.95078E+00_JPRB, 0.92578E+00_JPRB, 0.88952E+00_JPRB, 0.87310E+00_JPRB ,&
&  0.85524E+00_JPRB, 0.84018E+00_JPRB, 0.82066E+00_JPRB, 0.82620E+00_JPRB, 0.84916E+00_JPRB, 0.88320E+00_JPRB, 0.94295E+00_JPRB ,&
&  0.10047E+01_JPRB, 0.10131E+01_JPRB, 0.97845E+00_JPRB, 0.97785E+00_JPRB, 0.99368E+00_JPRB, 0.10217E+01_JPRB, 0.10374E+01_JPRB ,&
&  0.10125E+01_JPRB, 0.99236E+00_JPRB, 0.11098E+01_JPRB, 0.11532E+01_JPRB, 0.11497E+01_JPRB, 0.11926E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.10487E+01_JPRB, 0.11261E+01_JPRB, 0.97528E+00_JPRB, 0.10975E+01_JPRB, 0.11839E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.10019E+01_JPRB, 0.98789E+00_JPRB, 0.92001E+00_JPRB, 0.89983E+00_JPRB, 0.95248E+00_JPRB, 0.10454E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.12687E+01_JPRB, 0.14277E+01_JPRB, 0.15401E+01_JPRB, 0.13029E+01_JPRB, 0.12546E+01_JPRB, 0.12866E+01_JPRB, 0.15314E+01_JPRB ,&
&  0.19782E+01_JPRB, 0.20491E+01_JPRB, 0.20805E+01_JPRB, 0.18764E+01_JPRB, 0.15627E+01_JPRB, 0.13968E+01_JPRB, 0.13341E+01_JPRB ,&
&  0.12917E+01_JPRB, 0.12578E+01_JPRB, 0.11742E+01_JPRB, 0.10979E+01_JPRB, 0.10450E+01_JPRB, 0.10035E+01_JPRB, 0.94647E+00_JPRB ,&
&  0.91878E+00_JPRB, 0.89367E+00_JPRB, 0.87956E+00_JPRB, 0.87640E+00_JPRB, 0.86529E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.85724E+00_JPRB, 0.81869E+00_JPRB, 0.79288E+00_JPRB, 0.76803E+00_JPRB, 0.73694E+00_JPRB, 0.73453E+00_JPRB, 0.79059E+00_JPRB ,&
&  0.79403E+00_JPRB, 0.70920E+00_JPRB, 0.66287E+00_JPRB, 0.62843E+00_JPRB, 0.60644E+00_JPRB, 0.60057E+00_JPRB, 0.60757E+00_JPRB ,&
&  0.60965E+00_JPRB, 0.61957E+00_JPRB, 0.62243E+00_JPRB, 0.64719E+00_JPRB, 0.70774E+00_JPRB, 0.83267E+00_JPRB, 0.65318E+00_JPRB ,&
&  0.86663E+00_JPRB, 0.13761E+01_JPRB, 0.15072E+01_JPRB, 0.15111E+01_JPRB, 0.14766E+01_JPRB, 0.14636E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.10858E+01_JPRB, 0.10183E+01_JPRB, 0.98995E+00_JPRB, 0.97543E+00_JPRB, 0.94181E+00_JPRB, 0.90378E+00_JPRB ,&
&  0.87889E+00_JPRB, 0.85234E+00_JPRB, 0.82701E+00_JPRB, 0.82036E+00_JPRB, 0.83196E+00_JPRB, 0.85928E+00_JPRB, 0.90574E+00_JPRB ,&
&  0.96238E+00_JPRB, 0.10073E+01_JPRB, 0.10203E+01_JPRB, 0.96679E+00_JPRB, 0.99235E+00_JPRB, 0.10110E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.98423E+00_JPRB, 0.11821E+01_JPRB, 0.13833E+01_JPRB, 0.14692E+01_JPRB, 0.13937E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.11961E+01_JPRB, 0.11108E+01_JPRB, 0.10072E+01_JPRB, 0.10886E+01_JPRB, 0.11994E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.11532E+01_JPRB, 0.10434E+01_JPRB, 0.96189E+00_JPRB, 0.90115E+00_JPRB, 0.91165E+00_JPRB, 0.97085E+00_JPRB, 0.11159E+01_JPRB ,&
&  0.12738E+01_JPRB, 0.14971E+01_JPRB, 0.14838E+01_JPRB, 0.12341E+01_JPRB, 0.11722E+01_JPRB, 0.12417E+01_JPRB, 0.14093E+01_JPRB ,&
&  0.20251E+01_JPRB, 0.21659E+01_JPRB, 0.20007E+01_JPRB, 0.20422E+01_JPRB, 0.17027E+01_JPRB, 0.14981E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.13513E+01_JPRB, 0.12977E+01_JPRB, 0.12431E+01_JPRB, 0.11446E+01_JPRB, 0.10440E+01_JPRB, 0.97696E+00_JPRB ,&
&  0.93989E+00_JPRB, 0.92173E+00_JPRB, 0.90278E+00_JPRB, 0.89171E+00_JPRB, 0.88057E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.84371E+00_JPRB, 0.81385E+00_JPRB, 0.78317E+00_JPRB, 0.76058E+00_JPRB, 0.72794E+00_JPRB, 0.70414E+00_JPRB, 0.72613E+00_JPRB ,&
&  0.71632E+00_JPRB, 0.67705E+00_JPRB, 0.62772E+00_JPRB, 0.61120E+00_JPRB, 0.59969E+00_JPRB, 0.58949E+00_JPRB, 0.59063E+00_JPRB ,&
&  0.59048E+00_JPRB, 0.60183E+00_JPRB, 0.63062E+00_JPRB, 0.65654E+00_JPRB, 0.63147E+00_JPRB, 0.58153E+00_JPRB, 0.63393E+00_JPRB ,&
&  0.10158E+01_JPRB, 0.14232E+01_JPRB, 0.15290E+01_JPRB, 0.16415E+01_JPRB, 0.17148E+01_JPRB, 0.18010E+01_JPRB, 0.16267E+01_JPRB ,&
&  0.14237E+01_JPRB, 0.12934E+01_JPRB, 0.11818E+01_JPRB, 0.11200E+01_JPRB, 0.10604E+01_JPRB, 0.10068E+01_JPRB, 0.97522E+00_JPRB ,&
&  0.93880E+00_JPRB, 0.91184E+00_JPRB, 0.87369E+00_JPRB, 0.82875E+00_JPRB, 0.83265E+00_JPRB, 0.85688E+00_JPRB, 0.88329E+00_JPRB ,&
&  0.91744E+00_JPRB, 0.97884E+00_JPRB, 0.10036E+01_JPRB, 0.95937E+00_JPRB, 0.96556E+00_JPRB, 0.98903E+00_JPRB, 0.10668E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.11962E+01_JPRB, 0.13246E+01_JPRB, 0.14651E+01_JPRB, 0.16189E+01_JPRB, 0.16575E+01_JPRB, 0.15504E+01_JPRB ,&
&  0.14446E+01_JPRB, 0.12848E+01_JPRB, 0.10616E+01_JPRB, 0.10662E+01_JPRB, 0.11579E+01_JPRB, 0.13037E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.89655E+00_JPRB, 0.85440E+00_JPRB, 0.85856E+00_JPRB, 0.93413E+00_JPRB, 0.97998E+00_JPRB, 0.11379E+01_JPRB ,&
&  0.10986E+01_JPRB, 0.80957E+00_JPRB, 0.71565E+00_JPRB, 0.86853E+00_JPRB, 0.82274E+00_JPRB, 0.90567E+00_JPRB, 0.14623E+01_JPRB ,&
&  0.20487E+01_JPRB, 0.24612E+01_JPRB, 0.23718E+01_JPRB, 0.20077E+01_JPRB, 0.17659E+01_JPRB, 0.15402E+01_JPRB, 0.15500E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.13669E+01_JPRB, 0.13395E+01_JPRB, 0.13020E+01_JPRB, 0.12120E+01_JPRB, 0.10953E+01_JPRB, 0.99780E+00_JPRB ,&
&  0.94745E+00_JPRB, 0.91855E+00_JPRB, 0.89326E+00_JPRB, 0.87480E+00_JPRB, 0.85927E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.81020E+00_JPRB, 0.79330E+00_JPRB, 0.76638E+00_JPRB, 0.74308E+00_JPRB, 0.71520E+00_JPRB, 0.68629E+00_JPRB, 0.68194E+00_JPRB ,&
&  0.66277E+00_JPRB, 0.63233E+00_JPRB, 0.60289E+00_JPRB, 0.59233E+00_JPRB, 0.58626E+00_JPRB, 0.58069E+00_JPRB, 0.58132E+00_JPRB ,&
&  0.58205E+00_JPRB, 0.59232E+00_JPRB, 0.64686E+00_JPRB, 0.67900E+00_JPRB, 0.62341E+00_JPRB, 0.55208E+00_JPRB, 0.63559E+00_JPRB ,&
&  0.10313E+01_JPRB, 0.15235E+01_JPRB, 0.17673E+01_JPRB, 0.19644E+01_JPRB, 0.19885E+01_JPRB, 0.20172E+01_JPRB, 0.19172E+01_JPRB ,&
&  0.17543E+01_JPRB, 0.16008E+01_JPRB, 0.14435E+01_JPRB, 0.12724E+01_JPRB, 0.11325E+01_JPRB, 0.10493E+01_JPRB, 0.99212E+00_JPRB ,&
&  0.96849E+00_JPRB, 0.94698E+00_JPRB, 0.91809E+00_JPRB, 0.86928E+00_JPRB, 0.84291E+00_JPRB, 0.85371E+00_JPRB, 0.86895E+00_JPRB ,&
&  0.88085E+00_JPRB, 0.93814E+00_JPRB, 0.97792E+00_JPRB, 0.97961E+00_JPRB, 0.82650E+00_JPRB, 0.95911E+00_JPRB, 0.10257E+01_JPRB ,&
&  0.11290E+01_JPRB, 0.12449E+01_JPRB, 0.12779E+01_JPRB, 0.16204E+01_JPRB, 0.18425E+01_JPRB, 0.18023E+01_JPRB, 0.17145E+01_JPRB ,&
&  0.16593E+01_JPRB, 0.14714E+01_JPRB, 0.12237E+01_JPRB, 0.12488E+01_JPRB, 0.13223E+01_JPRB, 0.13983E+01_JPRB, 0.96907E+00_JPRB ,&
&  0.89672E+00_JPRB, 0.10717E+01_JPRB, 0.98780E+00_JPRB, 0.82034E+00_JPRB, 0.94936E+00_JPRB, 0.99888E+00_JPRB, 0.81135E+00_JPRB ,&
&  0.48964E+00_JPRB, 0.42763E+00_JPRB, 0.44767E+00_JPRB, 0.47496E+00_JPRB, 0.46210E+00_JPRB, 0.73926E+00_JPRB, 0.21463E+01_JPRB ,&
&  0.20855E+01_JPRB, 0.26417E+01_JPRB, 0.25654E+01_JPRB, 0.22795E+01_JPRB, 0.18999E+01_JPRB, 0.16137E+01_JPRB, 0.16756E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.14492E+01_JPRB, 0.13649E+01_JPRB, 0.12972E+01_JPRB, 0.12150E+01_JPRB, 0.11103E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.96725E+00_JPRB, 0.92783E+00_JPRB, 0.88860E+00_JPRB, 0.85835E+00_JPRB, 0.82718E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.78996E+00_JPRB, 0.76096E+00_JPRB, 0.73605E+00_JPRB, 0.71207E+00_JPRB, 0.68444E+00_JPRB, 0.66057E+00_JPRB, 0.64726E+00_JPRB ,&
&  0.62446E+00_JPRB, 0.59475E+00_JPRB, 0.57386E+00_JPRB, 0.56283E+00_JPRB, 0.56451E+00_JPRB, 0.57456E+00_JPRB, 0.57885E+00_JPRB ,&
&  0.58781E+00_JPRB, 0.61258E+00_JPRB, 0.71094E+00_JPRB, 0.68737E+00_JPRB, 0.66623E+00_JPRB, 0.63534E+00_JPRB, 0.67074E+00_JPRB ,&
&  0.92324E+00_JPRB, 0.13920E+01_JPRB, 0.17707E+01_JPRB, 0.20453E+01_JPRB, 0.23499E+01_JPRB, 0.24392E+01_JPRB, 0.23536E+01_JPRB ,&
&  0.21125E+01_JPRB, 0.18932E+01_JPRB, 0.16560E+01_JPRB, 0.13981E+01_JPRB, 0.12114E+01_JPRB, 0.10989E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.96006E+00_JPRB, 0.94475E+00_JPRB, 0.92066E+00_JPRB, 0.89257E+00_JPRB, 0.86475E+00_JPRB, 0.87010E+00_JPRB, 0.88616E+00_JPRB ,&
&  0.88988E+00_JPRB, 0.90476E+00_JPRB, 0.93572E+00_JPRB, 0.93845E+00_JPRB, 0.83505E+00_JPRB, 0.75337E+00_JPRB, 0.89027E+00_JPRB ,&
&  0.10595E+01_JPRB, 0.13361E+01_JPRB, 0.14748E+01_JPRB, 0.19515E+01_JPRB, 0.20325E+01_JPRB, 0.19179E+01_JPRB, 0.19218E+01_JPRB ,&
&  0.18017E+01_JPRB, 0.18623E+01_JPRB, 0.13908E+01_JPRB, 0.14161E+01_JPRB, 0.15290E+01_JPRB, 0.11904E+01_JPRB, 0.11287E+01_JPRB ,&
&  0.12989E+01_JPRB, 0.12238E+01_JPRB, 0.10731E+01_JPRB, 0.84367E+00_JPRB, 0.99839E+00_JPRB, 0.87511E+00_JPRB, 0.50441E+00_JPRB ,&
&  0.43364E+00_JPRB, 0.41321E+00_JPRB, 0.39884E+00_JPRB, 0.39402E+00_JPRB, 0.41795E+00_JPRB, 0.66384E+00_JPRB, 0.11820E+01_JPRB ,&
&  0.14768E+01_JPRB, 0.21872E+01_JPRB, 0.24501E+01_JPRB, 0.24210E+01_JPRB, 0.20563E+01_JPRB, 0.17915E+01_JPRB, 0.18058E+01_JPRB ,&
&  0.16307E+01_JPRB, 0.15902E+01_JPRB, 0.14128E+01_JPRB, 0.12952E+01_JPRB, 0.11915E+01_JPRB, 0.11047E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.96707E+00_JPRB, 0.91787E+00_JPRB, 0.87782E+00_JPRB, 0.85604E+00_JPRB, 0.82314E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.75199E+00_JPRB, 0.72152E+00_JPRB, 0.69395E+00_JPRB, 0.66627E+00_JPRB, 0.63644E+00_JPRB, 0.61073E+00_JPRB, 0.60633E+00_JPRB ,&
&  0.59501E+00_JPRB, 0.56809E+00_JPRB, 0.55076E+00_JPRB, 0.54048E+00_JPRB, 0.55072E+00_JPRB, 0.57094E+00_JPRB, 0.57697E+00_JPRB ,&
&  0.58915E+00_JPRB, 0.64867E+00_JPRB, 0.72911E+00_JPRB, 0.64360E+00_JPRB, 0.58242E+00_JPRB, 0.54776E+00_JPRB, 0.55716E+00_JPRB ,&
&  0.83348E+00_JPRB, 0.11474E+01_JPRB, 0.14619E+01_JPRB, 0.18535E+01_JPRB, 0.22331E+01_JPRB, 0.22652E+01_JPRB, 0.24718E+01_JPRB ,&
&  0.23345E+01_JPRB, 0.20187E+01_JPRB, 0.17230E+01_JPRB, 0.14509E+01_JPRB, 0.12616E+01_JPRB, 0.11244E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.96628E+00_JPRB, 0.94185E+00_JPRB, 0.92202E+00_JPRB, 0.90420E+00_JPRB, 0.89500E+00_JPRB, 0.89475E+00_JPRB, 0.91679E+00_JPRB ,&
&  0.92172E+00_JPRB, 0.90432E+00_JPRB, 0.90542E+00_JPRB, 0.91125E+00_JPRB, 0.93917E+00_JPRB, 0.94492E+00_JPRB, 0.92750E+00_JPRB ,&
&  0.93539E+00_JPRB, 0.10823E+01_JPRB, 0.16077E+01_JPRB, 0.20062E+01_JPRB, 0.20655E+01_JPRB, 0.20803E+01_JPRB, 0.20123E+01_JPRB ,&
&  0.19000E+01_JPRB, 0.20257E+01_JPRB, 0.17321E+01_JPRB, 0.17698E+01_JPRB, 0.15082E+01_JPRB, 0.10642E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.14260E+01_JPRB, 0.11678E+01_JPRB, 0.85528E+00_JPRB, 0.81019E+00_JPRB, 0.57673E+00_JPRB, 0.42833E+00_JPRB ,&
&  0.41142E+00_JPRB, 0.39860E+00_JPRB, 0.37275E+00_JPRB, 0.38804E+00_JPRB, 0.39648E+00_JPRB, 0.56019E+00_JPRB, 0.83890E+00_JPRB ,&
&  0.12433E+01_JPRB, 0.17452E+01_JPRB, 0.21662E+01_JPRB, 0.22556E+01_JPRB, 0.20092E+01_JPRB, 0.17938E+01_JPRB, 0.18663E+01_JPRB ,&
&  0.16380E+01_JPRB, 0.15669E+01_JPRB, 0.14340E+01_JPRB, 0.12769E+01_JPRB, 0.11499E+01_JPRB, 0.10596E+01_JPRB, 0.99237E+00_JPRB ,&
&  0.93649E+00_JPRB, 0.88004E+00_JPRB, 0.83836E+00_JPRB, 0.80940E+00_JPRB, 0.78340E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.70400E+00_JPRB, 0.67835E+00_JPRB, 0.65581E+00_JPRB, 0.62930E+00_JPRB, 0.60325E+00_JPRB, 0.57449E+00_JPRB, 0.56865E+00_JPRB ,&
&  0.57031E+00_JPRB, 0.54394E+00_JPRB, 0.51972E+00_JPRB, 0.51799E+00_JPRB, 0.53604E+00_JPRB, 0.55999E+00_JPRB, 0.58637E+00_JPRB ,&
&  0.62448E+00_JPRB, 0.80276E+00_JPRB, 0.60186E+00_JPRB, 0.49051E+00_JPRB, 0.53125E+00_JPRB, 0.54925E+00_JPRB, 0.51965E+00_JPRB ,&
&  0.73407E+00_JPRB, 0.97040E+00_JPRB, 0.12349E+01_JPRB, 0.17205E+01_JPRB, 0.24415E+01_JPRB, 0.25147E+01_JPRB, 0.26450E+01_JPRB ,&
&  0.26363E+01_JPRB, 0.22333E+01_JPRB, 0.18201E+01_JPRB, 0.14957E+01_JPRB, 0.12769E+01_JPRB, 0.11306E+01_JPRB, 0.10432E+01_JPRB ,&
&  0.98909E+00_JPRB, 0.95250E+00_JPRB, 0.92631E+00_JPRB, 0.92182E+00_JPRB, 0.92001E+00_JPRB, 0.91516E+00_JPRB, 0.93419E+00_JPRB ,&
&  0.93478E+00_JPRB, 0.91457E+00_JPRB, 0.91250E+00_JPRB, 0.92423E+00_JPRB, 0.98259E+00_JPRB, 0.95715E+00_JPRB, 0.13531E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.14543E+01_JPRB, 0.17055E+01_JPRB, 0.22549E+01_JPRB, 0.23730E+01_JPRB, 0.22172E+01_JPRB, 0.23493E+01_JPRB ,&
&  0.17991E+01_JPRB, 0.15799E+01_JPRB, 0.16002E+01_JPRB, 0.15404E+01_JPRB, 0.11882E+01_JPRB, 0.12692E+01_JPRB, 0.19505E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.17194E+01_JPRB, 0.18000E+01_JPRB, 0.18236E+01_JPRB, 0.86801E+00_JPRB, 0.55133E+00_JPRB, 0.80242E+00_JPRB ,&
&  0.87571E+00_JPRB, 0.60919E+00_JPRB, 0.47223E+00_JPRB, 0.59256E+00_JPRB, 0.48798E+00_JPRB, 0.61145E+00_JPRB, 0.85520E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.12456E+01_JPRB, 0.19264E+01_JPRB, 0.21157E+01_JPRB, 0.20163E+01_JPRB, 0.18418E+01_JPRB, 0.18050E+01_JPRB ,&
&  0.17038E+01_JPRB, 0.15660E+01_JPRB, 0.15233E+01_JPRB, 0.13237E+01_JPRB, 0.11572E+01_JPRB, 0.10380E+01_JPRB, 0.94924E+00_JPRB ,&
&  0.88636E+00_JPRB, 0.82413E+00_JPRB, 0.77916E+00_JPRB, 0.75246E+00_JPRB, 0.72969E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.66382E+00_JPRB, 0.64275E+00_JPRB, 0.62141E+00_JPRB, 0.59438E+00_JPRB, 0.56847E+00_JPRB, 0.54773E+00_JPRB, 0.53890E+00_JPRB ,&
&  0.54493E+00_JPRB, 0.51784E+00_JPRB, 0.49727E+00_JPRB, 0.49667E+00_JPRB, 0.50809E+00_JPRB, 0.52727E+00_JPRB, 0.55806E+00_JPRB ,&
&  0.59954E+00_JPRB, 0.60465E+00_JPRB, 0.45665E+00_JPRB, 0.41580E+00_JPRB, 0.53148E+00_JPRB, 0.52250E+00_JPRB, 0.56514E+00_JPRB ,&
&  0.70742E+00_JPRB, 0.86124E+00_JPRB, 0.11005E+01_JPRB, 0.15493E+01_JPRB, 0.19668E+01_JPRB, 0.24141E+01_JPRB, 0.26788E+01_JPRB ,&
&  0.27858E+01_JPRB, 0.25300E+01_JPRB, 0.21230E+01_JPRB, 0.17106E+01_JPRB, 0.13974E+01_JPRB, 0.11978E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.99445E+00_JPRB, 0.93121E+00_JPRB, 0.89633E+00_JPRB, 0.86822E+00_JPRB, 0.88292E+00_JPRB, 0.89048E+00_JPRB, 0.91818E+00_JPRB ,&
&  0.93862E+00_JPRB, 0.92814E+00_JPRB, 0.93643E+00_JPRB, 0.93838E+00_JPRB, 0.94219E+00_JPRB, 0.98175E+00_JPRB, 0.11529E+01_JPRB ,&
&  0.13797E+01_JPRB, 0.16277E+01_JPRB, 0.19293E+01_JPRB, 0.21886E+01_JPRB, 0.23379E+01_JPRB, 0.21996E+01_JPRB, 0.25282E+01_JPRB ,&
&  0.20899E+01_JPRB, 0.16930E+01_JPRB, 0.14002E+01_JPRB, 0.10860E+01_JPRB, 0.13730E+01_JPRB, 0.20898E+01_JPRB, 0.21049E+01_JPRB ,&
&  0.19990E+01_JPRB, 0.20345E+01_JPRB, 0.20149E+01_JPRB, 0.17414E+01_JPRB, 0.15105E+01_JPRB, 0.10809E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.12444E+01_JPRB, 0.11429E+01_JPRB, 0.10287E+01_JPRB, 0.90067E+00_JPRB, 0.75389E+00_JPRB, 0.89140E+00_JPRB, 0.91326E+00_JPRB ,&
&  0.92354E+00_JPRB, 0.10168E+01_JPRB, 0.15481E+01_JPRB, 0.19748E+01_JPRB, 0.19483E+01_JPRB, 0.17108E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.16241E+01_JPRB, 0.15455E+01_JPRB, 0.14113E+01_JPRB, 0.12520E+01_JPRB, 0.11120E+01_JPRB, 0.99252E+00_JPRB, 0.89699E+00_JPRB ,&
&  0.83157E+00_JPRB, 0.77646E+00_JPRB, 0.72449E+00_JPRB, 0.70053E+00_JPRB, 0.68169E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.63913E+00_JPRB, 0.62381E+00_JPRB, 0.60301E+00_JPRB, 0.57876E+00_JPRB, 0.55269E+00_JPRB, 0.53249E+00_JPRB, 0.52283E+00_JPRB ,&
&  0.52278E+00_JPRB, 0.49768E+00_JPRB, 0.48791E+00_JPRB, 0.48619E+00_JPRB, 0.49399E+00_JPRB, 0.50952E+00_JPRB, 0.53689E+00_JPRB ,&
&  0.57384E+00_JPRB, 0.50688E+00_JPRB, 0.39602E+00_JPRB, 0.41613E+00_JPRB, 0.47046E+00_JPRB, 0.47715E+00_JPRB, 0.56312E+00_JPRB ,&
&  0.66784E+00_JPRB, 0.78734E+00_JPRB, 0.94354E+00_JPRB, 0.12451E+01_JPRB, 0.15643E+01_JPRB, 0.18805E+01_JPRB, 0.21618E+01_JPRB ,&
&  0.22256E+01_JPRB, 0.23107E+01_JPRB, 0.22353E+01_JPRB, 0.19913E+01_JPRB, 0.16619E+01_JPRB, 0.13880E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.97691E+00_JPRB, 0.89765E+00_JPRB, 0.85538E+00_JPRB, 0.83932E+00_JPRB, 0.85355E+00_JPRB, 0.89253E+00_JPRB ,&
&  0.93669E+00_JPRB, 0.93365E+00_JPRB, 0.91692E+00_JPRB, 0.94820E+00_JPRB, 0.93295E+00_JPRB, 0.10000E+01_JPRB, 0.11172E+01_JPRB ,&
&  0.16354E+01_JPRB, 0.19940E+01_JPRB, 0.21812E+01_JPRB, 0.24712E+01_JPRB, 0.21139E+01_JPRB, 0.21200E+01_JPRB, 0.25044E+01_JPRB ,&
&  0.27817E+01_JPRB, 0.28287E+01_JPRB, 0.28912E+01_JPRB, 0.19471E+01_JPRB, 0.17180E+01_JPRB, 0.24170E+01_JPRB, 0.22977E+01_JPRB ,&
&  0.20636E+01_JPRB, 0.20880E+01_JPRB, 0.19961E+01_JPRB, 0.21312E+01_JPRB, 0.16766E+01_JPRB, 0.14590E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.93548E+00_JPRB, 0.81523E+00_JPRB, 0.92531E+00_JPRB, 0.85926E+00_JPRB, 0.84679E+00_JPRB, 0.85015E+00_JPRB, 0.83253E+00_JPRB ,&
&  0.85724E+00_JPRB, 0.91108E+00_JPRB, 0.11004E+01_JPRB, 0.14597E+01_JPRB, 0.17500E+01_JPRB, 0.15212E+01_JPRB, 0.15278E+01_JPRB ,&
&  0.14173E+01_JPRB, 0.13432E+01_JPRB, 0.12286E+01_JPRB, 0.11993E+01_JPRB, 0.11342E+01_JPRB, 0.10289E+01_JPRB, 0.91237E+00_JPRB ,&
&  0.81916E+00_JPRB, 0.75570E+00_JPRB, 0.70629E+00_JPRB, 0.67525E+00_JPRB, 0.65510E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.61090E+00_JPRB, 0.60270E+00_JPRB, 0.58605E+00_JPRB, 0.56399E+00_JPRB, 0.54286E+00_JPRB, 0.52730E+00_JPRB, 0.51678E+00_JPRB ,&
&  0.51027E+00_JPRB, 0.49468E+00_JPRB, 0.48207E+00_JPRB, 0.47836E+00_JPRB, 0.48349E+00_JPRB, 0.49186E+00_JPRB, 0.51464E+00_JPRB ,&
&  0.56323E+00_JPRB, 0.55771E+00_JPRB, 0.44301E+00_JPRB, 0.39702E+00_JPRB, 0.38125E+00_JPRB, 0.45693E+00_JPRB, 0.55134E+00_JPRB ,&
&  0.67140E+00_JPRB, 0.74255E+00_JPRB, 0.87472E+00_JPRB, 0.10556E+01_JPRB, 0.12588E+01_JPRB, 0.14410E+01_JPRB, 0.15971E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.18287E+01_JPRB, 0.18723E+01_JPRB, 0.17674E+01_JPRB, 0.16971E+01_JPRB, 0.15065E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.10285E+01_JPRB, 0.92027E+00_JPRB, 0.83888E+00_JPRB, 0.79795E+00_JPRB, 0.79176E+00_JPRB, 0.82883E+00_JPRB ,&
&  0.91400E+00_JPRB, 0.94461E+00_JPRB, 0.93371E+00_JPRB, 0.94858E+00_JPRB, 0.10277E+01_JPRB, 0.11489E+01_JPRB, 0.13462E+01_JPRB ,&
&  0.14691E+01_JPRB, 0.16045E+01_JPRB, 0.24320E+01_JPRB, 0.26134E+01_JPRB, 0.28505E+01_JPRB, 0.27190E+01_JPRB, 0.25938E+01_JPRB ,&
&  0.28374E+01_JPRB, 0.29401E+01_JPRB, 0.30425E+01_JPRB, 0.24388E+01_JPRB, 0.28151E+01_JPRB, 0.27621E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.22431E+01_JPRB, 0.21911E+01_JPRB, 0.20382E+01_JPRB, 0.19936E+01_JPRB, 0.20347E+01_JPRB, 0.21115E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.12729E+01_JPRB, 0.11093E+01_JPRB, 0.90177E+00_JPRB, 0.74305E+00_JPRB, 0.76960E+00_JPRB, 0.80152E+00_JPRB ,&
&  0.82527E+00_JPRB, 0.87087E+00_JPRB, 0.94153E+00_JPRB, 0.11493E+01_JPRB, 0.13776E+01_JPRB, 0.13968E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.10876E+01_JPRB, 0.11216E+01_JPRB, 0.10857E+01_JPRB, 0.10294E+01_JPRB, 0.10230E+01_JPRB, 0.96619E+00_JPRB, 0.88069E+00_JPRB ,&
&  0.78813E+00_JPRB, 0.72017E+00_JPRB, 0.67724E+00_JPRB, 0.64436E+00_JPRB, 0.62154E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.55603E+00_JPRB, 0.56750E+00_JPRB, 0.56897E+00_JPRB, 0.55812E+00_JPRB, 0.54218E+00_JPRB, 0.52621E+00_JPRB, 0.51089E+00_JPRB ,&
&  0.50239E+00_JPRB, 0.48935E+00_JPRB, 0.47202E+00_JPRB, 0.47219E+00_JPRB, 0.47331E+00_JPRB, 0.48036E+00_JPRB, 0.50633E+00_JPRB ,&
&  0.56354E+00_JPRB, 0.66878E+00_JPRB, 0.53305E+00_JPRB, 0.50498E+00_JPRB, 0.48201E+00_JPRB, 0.54986E+00_JPRB, 0.61613E+00_JPRB ,&
&  0.70139E+00_JPRB, 0.72412E+00_JPRB, 0.76697E+00_JPRB, 0.86190E+00_JPRB, 0.10178E+01_JPRB, 0.11264E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.13534E+01_JPRB, 0.14542E+01_JPRB, 0.13510E+01_JPRB, 0.13720E+01_JPRB, 0.13691E+01_JPRB, 0.12473E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.97892E+00_JPRB, 0.90264E+00_JPRB, 0.84261E+00_JPRB, 0.81089E+00_JPRB, 0.79396E+00_JPRB, 0.80942E+00_JPRB ,&
&  0.87742E+00_JPRB, 0.94995E+00_JPRB, 0.95879E+00_JPRB, 0.96440E+00_JPRB, 0.10048E+01_JPRB, 0.11606E+01_JPRB, 0.13475E+01_JPRB ,&
&  0.18007E+01_JPRB, 0.18752E+01_JPRB, 0.16283E+01_JPRB, 0.23736E+01_JPRB, 0.31864E+01_JPRB, 0.29033E+01_JPRB, 0.25542E+01_JPRB ,&
&  0.27958E+01_JPRB, 0.29831E+01_JPRB, 0.31888E+01_JPRB, 0.32574E+01_JPRB, 0.32524E+01_JPRB, 0.31129E+01_JPRB, 0.28075E+01_JPRB ,&
&  0.24271E+01_JPRB, 0.22115E+01_JPRB, 0.21102E+01_JPRB, 0.21508E+01_JPRB, 0.19983E+01_JPRB, 0.18481E+01_JPRB, 0.17381E+01_JPRB ,&
&  0.14479E+01_JPRB, 0.11802E+01_JPRB, 0.77164E+00_JPRB, 0.81728E+00_JPRB, 0.63759E+00_JPRB, 0.65421E+00_JPRB, 0.77903E+00_JPRB ,&
&  0.76439E+00_JPRB, 0.80546E+00_JPRB, 0.87561E+00_JPRB, 0.93316E+00_JPRB, 0.11478E+01_JPRB, 0.11713E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.89170E+00_JPRB, 0.91714E+00_JPRB, 0.88049E+00_JPRB, 0.87320E+00_JPRB, 0.82638E+00_JPRB, 0.76550E+00_JPRB ,&
&  0.69879E+00_JPRB, 0.64726E+00_JPRB, 0.61821E+00_JPRB, 0.58929E+00_JPRB, 0.56263E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.50661E+00_JPRB, 0.53009E+00_JPRB, 0.54152E+00_JPRB, 0.53129E+00_JPRB, 0.52183E+00_JPRB, 0.49802E+00_JPRB, 0.48558E+00_JPRB ,&
&  0.48754E+00_JPRB, 0.47783E+00_JPRB, 0.45926E+00_JPRB, 0.46076E+00_JPRB, 0.46499E+00_JPRB, 0.47317E+00_JPRB, 0.50316E+00_JPRB ,&
&  0.59174E+00_JPRB, 0.65772E+00_JPRB, 0.57557E+00_JPRB, 0.50413E+00_JPRB, 0.62464E+00_JPRB, 0.60447E+00_JPRB, 0.63933E+00_JPRB ,&
&  0.61295E+00_JPRB, 0.65530E+00_JPRB, 0.65908E+00_JPRB, 0.67958E+00_JPRB, 0.76205E+00_JPRB, 0.88174E+00_JPRB, 0.97978E+00_JPRB ,&
&  0.10190E+01_JPRB, 0.10488E+01_JPRB, 0.10192E+01_JPRB, 0.98826E+00_JPRB, 0.97929E+00_JPRB, 0.89228E+00_JPRB, 0.88793E+00_JPRB ,&
&  0.85032E+00_JPRB, 0.79305E+00_JPRB, 0.74825E+00_JPRB, 0.74272E+00_JPRB, 0.74129E+00_JPRB, 0.74412E+00_JPRB, 0.77469E+00_JPRB ,&
&  0.85656E+00_JPRB, 0.94994E+00_JPRB, 0.96284E+00_JPRB, 0.98207E+00_JPRB, 0.10571E+01_JPRB, 0.13265E+01_JPRB, 0.18069E+01_JPRB ,&
&  0.21836E+01_JPRB, 0.23736E+01_JPRB, 0.25574E+01_JPRB, 0.29620E+01_JPRB, 0.30187E+01_JPRB, 0.29159E+01_JPRB, 0.29104E+01_JPRB ,&
&  0.29934E+01_JPRB, 0.30320E+01_JPRB, 0.31657E+01_JPRB, 0.32326E+01_JPRB, 0.32763E+01_JPRB, 0.32593E+01_JPRB, 0.29578E+01_JPRB ,&
&  0.25179E+01_JPRB, 0.22341E+01_JPRB, 0.21066E+01_JPRB, 0.19428E+01_JPRB, 0.18218E+01_JPRB, 0.16415E+01_JPRB, 0.16800E+01_JPRB ,&
&  0.18043E+01_JPRB, 0.95722E+00_JPRB, 0.70168E+00_JPRB, 0.85188E+00_JPRB, 0.71171E+00_JPRB, 0.74741E+00_JPRB, 0.91684E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.83708E+00_JPRB, 0.87742E+00_JPRB, 0.89246E+00_JPRB, 0.96654E+00_JPRB, 0.11093E+01_JPRB, 0.99125E+00_JPRB ,&
&  0.85990E+00_JPRB, 0.80213E+00_JPRB, 0.80049E+00_JPRB, 0.74933E+00_JPRB, 0.73303E+00_JPRB, 0.69703E+00_JPRB, 0.65521E+00_JPRB ,&
&  0.60533E+00_JPRB, 0.57302E+00_JPRB, 0.54263E+00_JPRB, 0.52314E+00_JPRB, 0.51255E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.48522E+00_JPRB, 0.51161E+00_JPRB, 0.51664E+00_JPRB, 0.51059E+00_JPRB, 0.50565E+00_JPRB, 0.47813E+00_JPRB, 0.46860E+00_JPRB ,&
&  0.47105E+00_JPRB, 0.46407E+00_JPRB, 0.44886E+00_JPRB, 0.44906E+00_JPRB, 0.45282E+00_JPRB, 0.45429E+00_JPRB, 0.46235E+00_JPRB ,&
&  0.40374E+00_JPRB, 0.47814E+00_JPRB, 0.44010E+00_JPRB, 0.48253E+00_JPRB, 0.61862E+00_JPRB, 0.58178E+00_JPRB, 0.57077E+00_JPRB ,&
&  0.55485E+00_JPRB, 0.54523E+00_JPRB, 0.52653E+00_JPRB, 0.53654E+00_JPRB, 0.57064E+00_JPRB, 0.66750E+00_JPRB, 0.76743E+00_JPRB ,&
&  0.74569E+00_JPRB, 0.71446E+00_JPRB, 0.69309E+00_JPRB, 0.68047E+00_JPRB, 0.64731E+00_JPRB, 0.63683E+00_JPRB, 0.64180E+00_JPRB ,&
&  0.63280E+00_JPRB, 0.61576E+00_JPRB, 0.61939E+00_JPRB, 0.64302E+00_JPRB, 0.65952E+00_JPRB, 0.66202E+00_JPRB, 0.68270E+00_JPRB ,&
&  0.75990E+00_JPRB, 0.86973E+00_JPRB, 0.91634E+00_JPRB, 0.98840E+00_JPRB, 0.10960E+01_JPRB, 0.16287E+01_JPRB, 0.23513E+01_JPRB ,&
&  0.26135E+01_JPRB, 0.25629E+01_JPRB, 0.27278E+01_JPRB, 0.30226E+01_JPRB, 0.29874E+01_JPRB, 0.28755E+01_JPRB, 0.27894E+01_JPRB ,&
&  0.28145E+01_JPRB, 0.27430E+01_JPRB, 0.28950E+01_JPRB, 0.31168E+01_JPRB, 0.30648E+01_JPRB, 0.31977E+01_JPRB, 0.29893E+01_JPRB ,&
&  0.26245E+01_JPRB, 0.24216E+01_JPRB, 0.23043E+01_JPRB, 0.22025E+01_JPRB, 0.20354E+01_JPRB, 0.20485E+01_JPRB, 0.19338E+01_JPRB ,&
&  0.18963E+01_JPRB, 0.16541E+01_JPRB, 0.11989E+01_JPRB, 0.10794E+01_JPRB, 0.81038E+00_JPRB, 0.84661E+00_JPRB, 0.11136E+01_JPRB ,&
&  0.10723E+01_JPRB, 0.94924E+00_JPRB, 0.87520E+00_JPRB, 0.86054E+00_JPRB, 0.90302E+00_JPRB, 0.10139E+01_JPRB, 0.86361E+00_JPRB ,&
&  0.69421E+00_JPRB, 0.68931E+00_JPRB, 0.67406E+00_JPRB, 0.63712E+00_JPRB, 0.61091E+00_JPRB, 0.59511E+00_JPRB, 0.56340E+00_JPRB ,&
&  0.52972E+00_JPRB, 0.50785E+00_JPRB, 0.49610E+00_JPRB, 0.48173E+00_JPRB, 0.48198E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.47027E+00_JPRB, 0.48804E+00_JPRB, 0.48305E+00_JPRB, 0.48615E+00_JPRB, 0.48785E+00_JPRB, 0.46464E+00_JPRB, 0.45444E+00_JPRB ,&
&  0.45090E+00_JPRB, 0.44617E+00_JPRB, 0.43996E+00_JPRB, 0.43948E+00_JPRB, 0.44073E+00_JPRB, 0.43752E+00_JPRB, 0.39969E+00_JPRB ,&
&  0.36062E+00_JPRB, 0.40672E+00_JPRB, 0.41622E+00_JPRB, 0.45277E+00_JPRB, 0.49803E+00_JPRB, 0.48374E+00_JPRB, 0.46713E+00_JPRB ,&
&  0.47922E+00_JPRB, 0.47354E+00_JPRB, 0.47917E+00_JPRB, 0.49733E+00_JPRB, 0.52417E+00_JPRB, 0.58030E+00_JPRB, 0.64664E+00_JPRB ,&
&  0.60704E+00_JPRB, 0.59330E+00_JPRB, 0.57162E+00_JPRB, 0.56463E+00_JPRB, 0.56264E+00_JPRB, 0.55404E+00_JPRB, 0.53361E+00_JPRB ,&
&  0.51615E+00_JPRB, 0.51860E+00_JPRB, 0.52680E+00_JPRB, 0.54807E+00_JPRB, 0.56795E+00_JPRB, 0.57726E+00_JPRB, 0.58345E+00_JPRB ,&
&  0.62903E+00_JPRB, 0.71386E+00_JPRB, 0.78239E+00_JPRB, 0.86582E+00_JPRB, 0.99778E+00_JPRB, 0.15052E+01_JPRB, 0.19610E+01_JPRB ,&
&  0.20467E+01_JPRB, 0.20577E+01_JPRB, 0.20311E+01_JPRB, 0.20444E+01_JPRB, 0.21213E+01_JPRB, 0.21716E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.23595E+01_JPRB, 0.22976E+01_JPRB, 0.27851E+01_JPRB, 0.30001E+01_JPRB, 0.30089E+01_JPRB, 0.30524E+01_JPRB, 0.28795E+01_JPRB ,&
&  0.27666E+01_JPRB, 0.25224E+01_JPRB, 0.24075E+01_JPRB, 0.22800E+01_JPRB, 0.21350E+01_JPRB, 0.20850E+01_JPRB, 0.19159E+01_JPRB ,&
&  0.18586E+01_JPRB, 0.17806E+01_JPRB, 0.15243E+01_JPRB, 0.13348E+01_JPRB, 0.11189E+01_JPRB, 0.11829E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.10035E+01_JPRB, 0.80686E+00_JPRB, 0.81610E+00_JPRB, 0.82246E+00_JPRB, 0.86310E+00_JPRB, 0.83148E+00_JPRB, 0.68343E+00_JPRB ,&
&  0.62136E+00_JPRB, 0.64212E+00_JPRB, 0.60990E+00_JPRB, 0.58034E+00_JPRB, 0.55623E+00_JPRB, 0.52910E+00_JPRB, 0.46755E+00_JPRB ,&
&  0.44663E+00_JPRB, 0.48057E+00_JPRB, 0.48438E+00_JPRB, 0.47141E+00_JPRB, 0.46538E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.45760E+00_JPRB, 0.46772E+00_JPRB, 0.46355E+00_JPRB, 0.46800E+00_JPRB, 0.46801E+00_JPRB, 0.45061E+00_JPRB, 0.42478E+00_JPRB ,&
&  0.43188E+00_JPRB, 0.42822E+00_JPRB, 0.42676E+00_JPRB, 0.42907E+00_JPRB, 0.43228E+00_JPRB, 0.39132E+00_JPRB, 0.32114E+00_JPRB ,&
&  0.30998E+00_JPRB, 0.33809E+00_JPRB, 0.38905E+00_JPRB, 0.42577E+00_JPRB, 0.43671E+00_JPRB, 0.43096E+00_JPRB, 0.42885E+00_JPRB ,&
&  0.43212E+00_JPRB, 0.43678E+00_JPRB, 0.46338E+00_JPRB, 0.48333E+00_JPRB, 0.49192E+00_JPRB, 0.51009E+00_JPRB, 0.55820E+00_JPRB ,&
&  0.54039E+00_JPRB, 0.53749E+00_JPRB, 0.53543E+00_JPRB, 0.49348E+00_JPRB, 0.50742E+00_JPRB, 0.48591E+00_JPRB, 0.46015E+00_JPRB ,&
&  0.45023E+00_JPRB, 0.46082E+00_JPRB, 0.47725E+00_JPRB, 0.50069E+00_JPRB, 0.51525E+00_JPRB, 0.51975E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.54618E+00_JPRB, 0.59724E+00_JPRB, 0.66079E+00_JPRB, 0.74993E+00_JPRB, 0.87189E+00_JPRB, 0.12044E+01_JPRB, 0.15470E+01_JPRB ,&
&  0.16052E+01_JPRB, 0.16416E+01_JPRB, 0.16491E+01_JPRB, 0.14881E+01_JPRB, 0.15877E+01_JPRB, 0.17994E+01_JPRB, 0.18391E+01_JPRB ,&
&  0.20178E+01_JPRB, 0.20042E+01_JPRB, 0.22146E+01_JPRB, 0.24653E+01_JPRB, 0.26878E+01_JPRB, 0.26922E+01_JPRB, 0.26010E+01_JPRB ,&
&  0.26108E+01_JPRB, 0.25312E+01_JPRB, 0.22821E+01_JPRB, 0.21449E+01_JPRB, 0.19872E+01_JPRB, 0.19046E+01_JPRB, 0.17532E+01_JPRB ,&
&  0.16770E+01_JPRB, 0.16383E+01_JPRB, 0.15682E+01_JPRB, 0.14799E+01_JPRB, 0.13699E+01_JPRB, 0.12583E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.10390E+01_JPRB, 0.88723E+00_JPRB, 0.81537E+00_JPRB, 0.82875E+00_JPRB, 0.86737E+00_JPRB, 0.74153E+00_JPRB, 0.63685E+00_JPRB ,&
&  0.59015E+00_JPRB, 0.55995E+00_JPRB, 0.59113E+00_JPRB, 0.55337E+00_JPRB, 0.51987E+00_JPRB, 0.49287E+00_JPRB, 0.46987E+00_JPRB ,&
&  0.44888E+00_JPRB, 0.49377E+00_JPRB, 0.49189E+00_JPRB, 0.47576E+00_JPRB, 0.45975E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.45818E+00_JPRB, 0.46137E+00_JPRB, 0.45731E+00_JPRB, 0.45679E+00_JPRB, 0.45327E+00_JPRB, 0.42577E+00_JPRB, 0.41304E+00_JPRB ,&
&  0.44452E+00_JPRB, 0.39546E+00_JPRB, 0.41724E+00_JPRB, 0.35023E+00_JPRB, 0.32436E+00_JPRB, 0.32073E+00_JPRB, 0.31285E+00_JPRB ,&
&  0.32087E+00_JPRB, 0.34858E+00_JPRB, 0.38207E+00_JPRB, 0.41006E+00_JPRB, 0.43695E+00_JPRB, 0.42611E+00_JPRB, 0.41573E+00_JPRB ,&
&  0.40278E+00_JPRB, 0.40704E+00_JPRB, 0.44230E+00_JPRB, 0.46831E+00_JPRB, 0.48880E+00_JPRB, 0.50063E+00_JPRB, 0.51719E+00_JPRB ,&
&  0.49003E+00_JPRB, 0.48186E+00_JPRB, 0.48949E+00_JPRB, 0.47742E+00_JPRB, 0.46053E+00_JPRB, 0.44252E+00_JPRB, 0.42046E+00_JPRB ,&
&  0.40343E+00_JPRB, 0.39528E+00_JPRB, 0.44260E+00_JPRB, 0.47039E+00_JPRB, 0.48358E+00_JPRB, 0.48577E+00_JPRB, 0.49667E+00_JPRB ,&
&  0.51491E+00_JPRB, 0.54732E+00_JPRB, 0.60370E+00_JPRB, 0.68650E+00_JPRB, 0.77810E+00_JPRB, 0.93371E+00_JPRB, 0.10922E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.96949E+00_JPRB, 0.11549E+01_JPRB, 0.11947E+01_JPRB, 0.13538E+01_JPRB, 0.16053E+01_JPRB, 0.16674E+01_JPRB ,&
&  0.19644E+01_JPRB, 0.19905E+01_JPRB, 0.20858E+01_JPRB, 0.21643E+01_JPRB, 0.22521E+01_JPRB, 0.22954E+01_JPRB, 0.22639E+01_JPRB ,&
&  0.22766E+01_JPRB, 0.20387E+01_JPRB, 0.20572E+01_JPRB, 0.19556E+01_JPRB, 0.18355E+01_JPRB, 0.17679E+01_JPRB, 0.16597E+01_JPRB ,&
&  0.15844E+01_JPRB, 0.15386E+01_JPRB, 0.14743E+01_JPRB, 0.12831E+01_JPRB, 0.12198E+01_JPRB, 0.11685E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.95853E+00_JPRB, 0.90315E+00_JPRB, 0.97848E+00_JPRB, 0.93975E+00_JPRB, 0.82692E+00_JPRB, 0.72551E+00_JPRB ,&
&  0.62780E+00_JPRB, 0.52865E+00_JPRB, 0.52854E+00_JPRB, 0.51165E+00_JPRB, 0.48034E+00_JPRB, 0.46656E+00_JPRB, 0.51256E+00_JPRB ,&
&  0.49108E+00_JPRB, 0.49071E+00_JPRB, 0.51163E+00_JPRB, 0.49833E+00_JPRB, 0.47432E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.47349E+00_JPRB, 0.46316E+00_JPRB, 0.44916E+00_JPRB, 0.43977E+00_JPRB, 0.43234E+00_JPRB, 0.41282E+00_JPRB, 0.38240E+00_JPRB ,&
&  0.38170E+00_JPRB, 0.33617E+00_JPRB, 0.31593E+00_JPRB, 0.30692E+00_JPRB, 0.30856E+00_JPRB, 0.30733E+00_JPRB, 0.30224E+00_JPRB ,&
&  0.31287E+00_JPRB, 0.34167E+00_JPRB, 0.38960E+00_JPRB, 0.41665E+00_JPRB, 0.42558E+00_JPRB, 0.42337E+00_JPRB, 0.41240E+00_JPRB ,&
&  0.40003E+00_JPRB, 0.40351E+00_JPRB, 0.42773E+00_JPRB, 0.47075E+00_JPRB, 0.48830E+00_JPRB, 0.48519E+00_JPRB, 0.47859E+00_JPRB ,&
&  0.46961E+00_JPRB, 0.46264E+00_JPRB, 0.45630E+00_JPRB, 0.45300E+00_JPRB, 0.43263E+00_JPRB, 0.40595E+00_JPRB, 0.36687E+00_JPRB ,&
&  0.31688E+00_JPRB, 0.32133E+00_JPRB, 0.40979E+00_JPRB, 0.44275E+00_JPRB, 0.46320E+00_JPRB, 0.47212E+00_JPRB, 0.48425E+00_JPRB ,&
&  0.50823E+00_JPRB, 0.54165E+00_JPRB, 0.58551E+00_JPRB, 0.64144E+00_JPRB, 0.69888E+00_JPRB, 0.77608E+00_JPRB, 0.84915E+00_JPRB ,&
&  0.99016E+00_JPRB, 0.82887E+00_JPRB, 0.79934E+00_JPRB, 0.88585E+00_JPRB, 0.12472E+01_JPRB, 0.14562E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.16973E+01_JPRB, 0.17062E+01_JPRB, 0.18031E+01_JPRB, 0.19631E+01_JPRB, 0.20634E+01_JPRB, 0.20802E+01_JPRB, 0.20206E+01_JPRB ,&
&  0.19314E+01_JPRB, 0.18648E+01_JPRB, 0.17756E+01_JPRB, 0.17535E+01_JPRB, 0.16957E+01_JPRB, 0.16718E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.15411E+01_JPRB, 0.14947E+01_JPRB, 0.13866E+01_JPRB, 0.12647E+01_JPRB, 0.11639E+01_JPRB, 0.11000E+01_JPRB, 0.10480E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.10048E+01_JPRB, 0.10138E+01_JPRB, 0.10260E+01_JPRB, 0.96620E+00_JPRB, 0.84363E+00_JPRB, 0.73913E+00_JPRB ,&
&  0.61162E+00_JPRB, 0.52815E+00_JPRB, 0.45584E+00_JPRB, 0.43913E+00_JPRB, 0.44507E+00_JPRB, 0.44873E+00_JPRB, 0.45256E+00_JPRB ,&
&  0.46689E+00_JPRB, 0.47061E+00_JPRB, 0.47203E+00_JPRB, 0.48434E+00_JPRB, 0.48005E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.48431E+00_JPRB, 0.47915E+00_JPRB, 0.45322E+00_JPRB, 0.43297E+00_JPRB, 0.40902E+00_JPRB, 0.38960E+00_JPRB, 0.37051E+00_JPRB ,&
&  0.36802E+00_JPRB, 0.34752E+00_JPRB, 0.33086E+00_JPRB, 0.31688E+00_JPRB, 0.31083E+00_JPRB, 0.30965E+00_JPRB, 0.32350E+00_JPRB ,&
&  0.35341E+00_JPRB, 0.39199E+00_JPRB, 0.41949E+00_JPRB, 0.42454E+00_JPRB, 0.42570E+00_JPRB, 0.42254E+00_JPRB, 0.42005E+00_JPRB ,&
&  0.41156E+00_JPRB, 0.40906E+00_JPRB, 0.41607E+00_JPRB, 0.43372E+00_JPRB, 0.45285E+00_JPRB, 0.45931E+00_JPRB, 0.45266E+00_JPRB ,&
&  0.44040E+00_JPRB, 0.43362E+00_JPRB, 0.43556E+00_JPRB, 0.43694E+00_JPRB, 0.40745E+00_JPRB, 0.36026E+00_JPRB, 0.31370E+00_JPRB ,&
&  0.27906E+00_JPRB, 0.27398E+00_JPRB, 0.30776E+00_JPRB, 0.40494E+00_JPRB, 0.43101E+00_JPRB, 0.45098E+00_JPRB, 0.46899E+00_JPRB ,&
&  0.45887E+00_JPRB, 0.46588E+00_JPRB, 0.50690E+00_JPRB, 0.58910E+00_JPRB, 0.61639E+00_JPRB, 0.66486E+00_JPRB, 0.70750E+00_JPRB ,&
&  0.76174E+00_JPRB, 0.78051E+00_JPRB, 0.83762E+00_JPRB, 0.80814E+00_JPRB, 0.92488E+00_JPRB, 0.13219E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.14533E+01_JPRB, 0.15468E+01_JPRB, 0.17317E+01_JPRB, 0.18328E+01_JPRB, 0.18633E+01_JPRB, 0.18965E+01_JPRB, 0.18521E+01_JPRB ,&
&  0.17324E+01_JPRB, 0.16603E+01_JPRB, 0.15964E+01_JPRB, 0.16338E+01_JPRB, 0.16432E+01_JPRB, 0.16247E+01_JPRB, 0.15831E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.14813E+01_JPRB, 0.13685E+01_JPRB, 0.12171E+01_JPRB, 0.11332E+01_JPRB, 0.10596E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.97542E+00_JPRB, 0.98189E+00_JPRB, 0.10062E+01_JPRB, 0.10158E+01_JPRB, 0.90598E+00_JPRB, 0.74479E+00_JPRB, 0.62172E+00_JPRB ,&
&  0.54520E+00_JPRB, 0.50456E+00_JPRB, 0.45768E+00_JPRB, 0.44283E+00_JPRB, 0.43719E+00_JPRB, 0.43357E+00_JPRB, 0.41672E+00_JPRB ,&
&  0.40176E+00_JPRB, 0.40434E+00_JPRB, 0.43230E+00_JPRB, 0.46650E+00_JPRB, 0.48017E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.44825E+00_JPRB, 0.47413E+00_JPRB, 0.48016E+00_JPRB, 0.45883E+00_JPRB, 0.43816E+00_JPRB, 0.39953E+00_JPRB, 0.37046E+00_JPRB ,&
&  0.35995E+00_JPRB, 0.35735E+00_JPRB, 0.36193E+00_JPRB, 0.36842E+00_JPRB, 0.37917E+00_JPRB, 0.39677E+00_JPRB, 0.41785E+00_JPRB ,&
&  0.43104E+00_JPRB, 0.43864E+00_JPRB, 0.44760E+00_JPRB, 0.45937E+00_JPRB, 0.46503E+00_JPRB, 0.46152E+00_JPRB, 0.45314E+00_JPRB ,&
&  0.43783E+00_JPRB, 0.42621E+00_JPRB, 0.41654E+00_JPRB, 0.41239E+00_JPRB, 0.41379E+00_JPRB, 0.42007E+00_JPRB, 0.42864E+00_JPRB ,&
&  0.43104E+00_JPRB, 0.42217E+00_JPRB, 0.41775E+00_JPRB, 0.41032E+00_JPRB, 0.37389E+00_JPRB, 0.33790E+00_JPRB, 0.29551E+00_JPRB ,&
&  0.26617E+00_JPRB, 0.25248E+00_JPRB, 0.25038E+00_JPRB, 0.26758E+00_JPRB, 0.31655E+00_JPRB, 0.41994E+00_JPRB, 0.44942E+00_JPRB ,&
&  0.47748E+00_JPRB, 0.49777E+00_JPRB, 0.51617E+00_JPRB, 0.53928E+00_JPRB, 0.56201E+00_JPRB, 0.59540E+00_JPRB, 0.62687E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.70376E+00_JPRB, 0.76071E+00_JPRB, 0.68801E+00_JPRB, 0.71384E+00_JPRB, 0.86982E+00_JPRB, 0.10447E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.13038E+01_JPRB, 0.13681E+01_JPRB, 0.15416E+01_JPRB, 0.16378E+01_JPRB, 0.17504E+01_JPRB, 0.17560E+01_JPRB ,&
&  0.17119E+01_JPRB, 0.16443E+01_JPRB, 0.15811E+01_JPRB, 0.15666E+01_JPRB, 0.16051E+01_JPRB, 0.16431E+01_JPRB, 0.15894E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.14685E+01_JPRB, 0.13267E+01_JPRB, 0.11388E+01_JPRB, 0.10643E+01_JPRB, 0.10447E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10354E+01_JPRB, 0.10280E+01_JPRB, 0.10248E+01_JPRB, 0.10228E+01_JPRB, 0.88498E+00_JPRB, 0.75258E+00_JPRB, 0.65047E+00_JPRB ,&
&  0.59666E+00_JPRB, 0.56404E+00_JPRB, 0.52456E+00_JPRB, 0.52331E+00_JPRB, 0.49698E+00_JPRB, 0.46584E+00_JPRB, 0.42932E+00_JPRB ,&
&  0.40765E+00_JPRB, 0.39839E+00_JPRB, 0.40770E+00_JPRB, 0.42356E+00_JPRB, 0.43642E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.48001E+00_JPRB, 0.48930E+00_JPRB, 0.49440E+00_JPRB, 0.49288E+00_JPRB, 0.48465E+00_JPRB, 0.46910E+00_JPRB, 0.46184E+00_JPRB ,&
&  0.46337E+00_JPRB, 0.47146E+00_JPRB, 0.48039E+00_JPRB, 0.48281E+00_JPRB, 0.48768E+00_JPRB, 0.49258E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.50249E+00_JPRB, 0.50942E+00_JPRB, 0.51772E+00_JPRB, 0.51112E+00_JPRB, 0.50086E+00_JPRB, 0.49089E+00_JPRB, 0.48095E+00_JPRB ,&
&  0.46617E+00_JPRB, 0.44897E+00_JPRB, 0.43033E+00_JPRB, 0.41983E+00_JPRB, 0.40754E+00_JPRB, 0.40175E+00_JPRB, 0.39564E+00_JPRB ,&
&  0.38943E+00_JPRB, 0.39009E+00_JPRB, 0.39647E+00_JPRB, 0.37232E+00_JPRB, 0.33793E+00_JPRB, 0.31044E+00_JPRB, 0.27617E+00_JPRB ,&
&  0.25087E+00_JPRB, 0.23430E+00_JPRB, 0.22430E+00_JPRB, 0.22284E+00_JPRB, 0.23238E+00_JPRB, 0.25820E+00_JPRB, 0.31916E+00_JPRB ,&
&  0.45456E+00_JPRB, 0.49181E+00_JPRB, 0.51133E+00_JPRB, 0.53445E+00_JPRB, 0.55919E+00_JPRB, 0.58516E+00_JPRB, 0.61140E+00_JPRB ,&
&  0.63808E+00_JPRB, 0.66045E+00_JPRB, 0.69748E+00_JPRB, 0.72967E+00_JPRB, 0.77862E+00_JPRB, 0.79883E+00_JPRB, 0.91166E+00_JPRB ,&
&  0.11393E+01_JPRB, 0.12217E+01_JPRB, 0.11684E+01_JPRB, 0.12243E+01_JPRB, 0.13244E+01_JPRB, 0.14141E+01_JPRB, 0.14837E+01_JPRB ,&
&  0.15231E+01_JPRB, 0.15320E+01_JPRB, 0.15435E+01_JPRB, 0.15457E+01_JPRB, 0.15656E+01_JPRB, 0.15811E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.15268E+01_JPRB, 0.15652E+01_JPRB, 0.12953E+01_JPRB, 0.11784E+01_JPRB, 0.11487E+01_JPRB, 0.10948E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10365E+01_JPRB, 0.10434E+01_JPRB, 0.10383E+01_JPRB, 0.94800E+00_JPRB, 0.84296E+00_JPRB, 0.73912E+00_JPRB ,&
&  0.68050E+00_JPRB, 0.62869E+00_JPRB, 0.57314E+00_JPRB, 0.54621E+00_JPRB, 0.51721E+00_JPRB, 0.48285E+00_JPRB, 0.45021E+00_JPRB ,&
&  0.43865E+00_JPRB, 0.43530E+00_JPRB, 0.43963E+00_JPRB, 0.46152E+00_JPRB, 0.47416E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.48160E+00_JPRB, 0.49866E+00_JPRB, 0.51209E+00_JPRB, 0.51938E+00_JPRB, 0.52080E+00_JPRB, 0.52342E+00_JPRB, 0.52557E+00_JPRB ,&
&  0.52906E+00_JPRB, 0.53284E+00_JPRB, 0.53590E+00_JPRB, 0.53557E+00_JPRB, 0.53546E+00_JPRB, 0.53466E+00_JPRB, 0.53614E+00_JPRB ,&
&  0.53748E+00_JPRB, 0.53332E+00_JPRB, 0.53096E+00_JPRB, 0.52997E+00_JPRB, 0.52595E+00_JPRB, 0.51472E+00_JPRB, 0.49887E+00_JPRB ,&
&  0.47283E+00_JPRB, 0.44818E+00_JPRB, 0.42748E+00_JPRB, 0.40742E+00_JPRB, 0.39070E+00_JPRB, 0.37759E+00_JPRB, 0.37530E+00_JPRB ,&
&  0.38436E+00_JPRB, 0.37683E+00_JPRB, 0.36704E+00_JPRB, 0.33889E+00_JPRB, 0.31050E+00_JPRB, 0.29424E+00_JPRB, 0.25832E+00_JPRB ,&
&  0.23288E+00_JPRB, 0.21799E+00_JPRB, 0.21314E+00_JPRB, 0.21287E+00_JPRB, 0.21970E+00_JPRB, 0.23784E+00_JPRB, 0.27617E+00_JPRB ,&
&  0.34660E+00_JPRB, 0.47311E+00_JPRB, 0.50345E+00_JPRB, 0.53219E+00_JPRB, 0.56416E+00_JPRB, 0.59484E+00_JPRB, 0.61761E+00_JPRB ,&
&  0.63873E+00_JPRB, 0.65984E+00_JPRB, 0.70179E+00_JPRB, 0.74409E+00_JPRB, 0.80539E+00_JPRB, 0.85842E+00_JPRB, 0.90903E+00_JPRB ,&
&  0.96129E+00_JPRB, 0.96606E+00_JPRB, 0.10071E+01_JPRB, 0.10427E+01_JPRB, 0.11193E+01_JPRB, 0.11911E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.11877E+01_JPRB, 0.12549E+01_JPRB, 0.12685E+01_JPRB, 0.13065E+01_JPRB, 0.13868E+01_JPRB, 0.14834E+01_JPRB, 0.15146E+01_JPRB ,&
&  0.14627E+01_JPRB, 0.14304E+01_JPRB, 0.14835E+01_JPRB, 0.13999E+01_JPRB, 0.13332E+01_JPRB, 0.12516E+01_JPRB, 0.11610E+01_JPRB ,&
&  0.11032E+01_JPRB, 0.10639E+01_JPRB, 0.10388E+01_JPRB, 0.10141E+01_JPRB, 0.96813E+00_JPRB, 0.91362E+00_JPRB, 0.83548E+00_JPRB ,&
&  0.74441E+00_JPRB, 0.66965E+00_JPRB, 0.61061E+00_JPRB, 0.57335E+00_JPRB, 0.54234E+00_JPRB, 0.51005E+00_JPRB, 0.48565E+00_JPRB ,&
&  0.46560E+00_JPRB, 0.45722E+00_JPRB, 0.45529E+00_JPRB, 0.46281E+00_JPRB, 0.47131E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.51851E+00_JPRB, 0.52912E+00_JPRB, 0.53956E+00_JPRB, 0.54645E+00_JPRB, 0.55131E+00_JPRB, 0.55449E+00_JPRB, 0.55487E+00_JPRB ,&
&  0.55568E+00_JPRB, 0.55568E+00_JPRB, 0.55762E+00_JPRB, 0.55913E+00_JPRB, 0.56339E+00_JPRB, 0.56683E+00_JPRB, 0.57197E+00_JPRB ,&
&  0.57963E+00_JPRB, 0.58541E+00_JPRB, 0.58447E+00_JPRB, 0.57353E+00_JPRB, 0.55498E+00_JPRB, 0.53349E+00_JPRB, 0.50325E+00_JPRB ,&
&  0.47488E+00_JPRB, 0.45564E+00_JPRB, 0.42822E+00_JPRB, 0.40249E+00_JPRB, 0.38743E+00_JPRB, 0.37566E+00_JPRB, 0.37233E+00_JPRB ,&
&  0.36725E+00_JPRB, 0.35485E+00_JPRB, 0.33913E+00_JPRB, 0.31698E+00_JPRB, 0.29763E+00_JPRB, 0.25149E+00_JPRB, 0.24049E+00_JPRB ,&
&  0.22747E+00_JPRB, 0.21889E+00_JPRB, 0.21608E+00_JPRB, 0.21724E+00_JPRB, 0.22177E+00_JPRB, 0.23417E+00_JPRB, 0.25378E+00_JPRB ,&
&  0.28831E+00_JPRB, 0.43873E+00_JPRB, 0.48087E+00_JPRB, 0.49838E+00_JPRB, 0.52677E+00_JPRB, 0.55518E+00_JPRB, 0.57980E+00_JPRB ,&
&  0.60562E+00_JPRB, 0.63124E+00_JPRB, 0.67053E+00_JPRB, 0.70835E+00_JPRB, 0.74865E+00_JPRB, 0.78308E+00_JPRB, 0.81091E+00_JPRB ,&
&  0.84087E+00_JPRB, 0.86319E+00_JPRB, 0.91161E+00_JPRB, 0.93321E+00_JPRB, 0.93725E+00_JPRB, 0.99136E+00_JPRB, 0.10274E+01_JPRB ,&
&  0.10303E+01_JPRB, 0.10224E+01_JPRB, 0.10510E+01_JPRB, 0.11191E+01_JPRB, 0.12083E+01_JPRB, 0.12861E+01_JPRB, 0.13068E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.12975E+01_JPRB, 0.12691E+01_JPRB, 0.12448E+01_JPRB, 0.11885E+01_JPRB, 0.11300E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10347E+01_JPRB, 0.10028E+01_JPRB, 0.96955E+00_JPRB, 0.92962E+00_JPRB, 0.89048E+00_JPRB, 0.84288E+00_JPRB ,&
&  0.78390E+00_JPRB, 0.72058E+00_JPRB, 0.66927E+00_JPRB, 0.62483E+00_JPRB, 0.59277E+00_JPRB, 0.55947E+00_JPRB, 0.54116E+00_JPRB ,&
&  0.52174E+00_JPRB, 0.51439E+00_JPRB, 0.50509E+00_JPRB, 0.50570E+00_JPRB, 0.50741E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.58136E+00_JPRB, 0.58386E+00_JPRB, 0.58768E+00_JPRB, 0.59088E+00_JPRB, 0.59358E+00_JPRB, 0.59508E+00_JPRB, 0.59519E+00_JPRB ,&
&  0.59469E+00_JPRB, 0.59448E+00_JPRB, 0.59576E+00_JPRB, 0.59829E+00_JPRB, 0.60186E+00_JPRB, 0.60691E+00_JPRB, 0.61206E+00_JPRB ,&
&  0.61854E+00_JPRB, 0.61624E+00_JPRB, 0.60205E+00_JPRB, 0.58507E+00_JPRB, 0.56602E+00_JPRB, 0.53923E+00_JPRB, 0.50707E+00_JPRB ,&
&  0.47727E+00_JPRB, 0.45432E+00_JPRB, 0.42031E+00_JPRB, 0.39177E+00_JPRB, 0.36856E+00_JPRB, 0.35672E+00_JPRB, 0.34234E+00_JPRB ,&
&  0.34283E+00_JPRB, 0.30483E+00_JPRB, 0.29220E+00_JPRB, 0.26682E+00_JPRB, 0.24934E+00_JPRB, 0.23610E+00_JPRB, 0.22833E+00_JPRB ,&
&  0.22528E+00_JPRB, 0.22316E+00_JPRB, 0.22379E+00_JPRB, 0.22809E+00_JPRB, 0.23431E+00_JPRB, 0.24796E+00_JPRB, 0.26440E+00_JPRB ,&
&  0.30827E+00_JPRB, 0.44104E+00_JPRB, 0.49212E+00_JPRB, 0.50700E+00_JPRB, 0.51298E+00_JPRB, 0.53316E+00_JPRB, 0.55578E+00_JPRB ,&
&  0.58925E+00_JPRB, 0.61698E+00_JPRB, 0.64561E+00_JPRB, 0.66362E+00_JPRB, 0.69797E+00_JPRB, 0.72913E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.78604E+00_JPRB, 0.81311E+00_JPRB, 0.84082E+00_JPRB, 0.86125E+00_JPRB, 0.86615E+00_JPRB, 0.87576E+00_JPRB, 0.88213E+00_JPRB ,&
&  0.89636E+00_JPRB, 0.90963E+00_JPRB, 0.94286E+00_JPRB, 0.98862E+00_JPRB, 0.10376E+01_JPRB, 0.10740E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.11243E+01_JPRB, 0.11390E+01_JPRB, 0.11455E+01_JPRB, 0.11267E+01_JPRB, 0.10934E+01_JPRB, 0.10711E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10054E+01_JPRB, 0.96360E+00_JPRB, 0.92633E+00_JPRB, 0.89271E+00_JPRB, 0.86250E+00_JPRB, 0.83432E+00_JPRB ,&
&  0.80627E+00_JPRB, 0.77493E+00_JPRB, 0.74256E+00_JPRB, 0.70402E+00_JPRB, 0.66892E+00_JPRB, 0.63545E+00_JPRB, 0.61478E+00_JPRB ,&
&  0.59584E+00_JPRB, 0.58850E+00_JPRB, 0.57967E+00_JPRB, 0.57913E+00_JPRB, 0.57722E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.64946E+00_JPRB, 0.64657E+00_JPRB, 0.64593E+00_JPRB, 0.64475E+00_JPRB, 0.64548E+00_JPRB, 0.64487E+00_JPRB, 0.64513E+00_JPRB ,&
&  0.64457E+00_JPRB, 0.64510E+00_JPRB, 0.64488E+00_JPRB, 0.64503E+00_JPRB, 0.64408E+00_JPRB, 0.64403E+00_JPRB, 0.64351E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.63257E+00_JPRB, 0.61518E+00_JPRB, 0.59621E+00_JPRB, 0.57527E+00_JPRB, 0.55098E+00_JPRB, 0.52392E+00_JPRB ,&
&  0.48961E+00_JPRB, 0.45785E+00_JPRB, 0.41422E+00_JPRB, 0.38699E+00_JPRB, 0.36050E+00_JPRB, 0.34132E+00_JPRB, 0.32951E+00_JPRB ,&
&  0.31837E+00_JPRB, 0.30811E+00_JPRB, 0.29791E+00_JPRB, 0.28592E+00_JPRB, 0.27526E+00_JPRB, 0.26219E+00_JPRB, 0.25417E+00_JPRB ,&
&  0.24844E+00_JPRB, 0.24641E+00_JPRB, 0.24660E+00_JPRB, 0.25108E+00_JPRB, 0.25889E+00_JPRB, 0.27136E+00_JPRB, 0.29193E+00_JPRB ,&
&  0.33750E+00_JPRB, 0.40123E+00_JPRB, 0.50179E+00_JPRB, 0.52455E+00_JPRB, 0.52204E+00_JPRB, 0.53567E+00_JPRB, 0.55190E+00_JPRB ,&
&  0.56632E+00_JPRB, 0.58501E+00_JPRB, 0.60054E+00_JPRB, 0.60762E+00_JPRB, 0.61589E+00_JPRB, 0.62322E+00_JPRB, 0.63162E+00_JPRB ,&
&  0.64149E+00_JPRB, 0.66102E+00_JPRB, 0.69155E+00_JPRB, 0.72781E+00_JPRB, 0.76804E+00_JPRB, 0.80486E+00_JPRB, 0.83157E+00_JPRB ,&
&  0.84912E+00_JPRB, 0.86286E+00_JPRB, 0.88594E+00_JPRB, 0.91217E+00_JPRB, 0.94142E+00_JPRB, 0.96250E+00_JPRB, 0.98186E+00_JPRB ,&
&  0.99746E+00_JPRB, 0.10038E+01_JPRB, 0.99821E+00_JPRB, 0.98818E+00_JPRB, 0.98069E+00_JPRB, 0.98328E+00_JPRB, 0.97708E+00_JPRB ,&
&  0.96794E+00_JPRB, 0.95328E+00_JPRB, 0.92944E+00_JPRB, 0.89964E+00_JPRB, 0.87336E+00_JPRB, 0.84899E+00_JPRB, 0.83031E+00_JPRB ,&
&  0.81293E+00_JPRB, 0.79674E+00_JPRB, 0.77901E+00_JPRB, 0.75737E+00_JPRB, 0.73624E+00_JPRB, 0.71493E+00_JPRB, 0.69844E+00_JPRB ,&
&  0.68294E+00_JPRB, 0.67277E+00_JPRB, 0.66354E+00_JPRB, 0.65806E+00_JPRB, 0.65239E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.73684E+00_JPRB, 0.73347E+00_JPRB, 0.73034E+00_JPRB, 0.72711E+00_JPRB, 0.72239E+00_JPRB, 0.71691E+00_JPRB, 0.71025E+00_JPRB ,&
&  0.70165E+00_JPRB, 0.69195E+00_JPRB, 0.68193E+00_JPRB, 0.67178E+00_JPRB, 0.66244E+00_JPRB, 0.65497E+00_JPRB, 0.64876E+00_JPRB ,&
&  0.64355E+00_JPRB, 0.63529E+00_JPRB, 0.62076E+00_JPRB, 0.60296E+00_JPRB, 0.58735E+00_JPRB, 0.57392E+00_JPRB, 0.56099E+00_JPRB ,&
&  0.53741E+00_JPRB, 0.51170E+00_JPRB, 0.48734E+00_JPRB, 0.46073E+00_JPRB, 0.44023E+00_JPRB, 0.39573E+00_JPRB, 0.37966E+00_JPRB ,&
&  0.36431E+00_JPRB, 0.35577E+00_JPRB, 0.35244E+00_JPRB, 0.35130E+00_JPRB, 0.35985E+00_JPRB, 0.35311E+00_JPRB, 0.34829E+00_JPRB ,&
&  0.33016E+00_JPRB, 0.32867E+00_JPRB, 0.33196E+00_JPRB, 0.33784E+00_JPRB, 0.34994E+00_JPRB, 0.36660E+00_JPRB, 0.39066E+00_JPRB ,&
&  0.43226E+00_JPRB, 0.45208E+00_JPRB, 0.48135E+00_JPRB, 0.50402E+00_JPRB, 0.51779E+00_JPRB, 0.52518E+00_JPRB, 0.53402E+00_JPRB ,&
&  0.54358E+00_JPRB, 0.55837E+00_JPRB, 0.56730E+00_JPRB, 0.57354E+00_JPRB, 0.57364E+00_JPRB, 0.57187E+00_JPRB, 0.57219E+00_JPRB ,&
&  0.57117E+00_JPRB, 0.57621E+00_JPRB, 0.58601E+00_JPRB, 0.60540E+00_JPRB, 0.63260E+00_JPRB, 0.66467E+00_JPRB, 0.69648E+00_JPRB ,&
&  0.72478E+00_JPRB, 0.75431E+00_JPRB, 0.77911E+00_JPRB, 0.80375E+00_JPRB, 0.83068E+00_JPRB, 0.85547E+00_JPRB, 0.87783E+00_JPRB ,&
&  0.89491E+00_JPRB, 0.90589E+00_JPRB, 0.90865E+00_JPRB, 0.90929E+00_JPRB, 0.90507E+00_JPRB, 0.90085E+00_JPRB, 0.89909E+00_JPRB ,&
&  0.89567E+00_JPRB, 0.89313E+00_JPRB, 0.89048E+00_JPRB, 0.88269E+00_JPRB, 0.87165E+00_JPRB, 0.85866E+00_JPRB, 0.84531E+00_JPRB ,&
&  0.83161E+00_JPRB, 0.81838E+00_JPRB, 0.80629E+00_JPRB, 0.79543E+00_JPRB, 0.78676E+00_JPRB, 0.77677E+00_JPRB, 0.76853E+00_JPRB ,&
&  0.76003E+00_JPRB, 0.75567E+00_JPRB, 0.74999E+00_JPRB, 0.74534E+00_JPRB, 0.74045E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.72343E+00_JPRB, 0.71681E+00_JPRB, 0.71016E+00_JPRB, 0.70410E+00_JPRB, 0.69649E+00_JPRB, 0.68895E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.66535E+00_JPRB, 0.65707E+00_JPRB, 0.64836E+00_JPRB, 0.63973E+00_JPRB, 0.63313E+00_JPRB, 0.62866E+00_JPRB ,&
&  0.62449E+00_JPRB, 0.61879E+00_JPRB, 0.61220E+00_JPRB, 0.60340E+00_JPRB, 0.59402E+00_JPRB, 0.58505E+00_JPRB, 0.57722E+00_JPRB ,&
&  0.56731E+00_JPRB, 0.55678E+00_JPRB, 0.54582E+00_JPRB, 0.53528E+00_JPRB, 0.52447E+00_JPRB, 0.51364E+00_JPRB, 0.50279E+00_JPRB ,&
&  0.49309E+00_JPRB, 0.48591E+00_JPRB, 0.47882E+00_JPRB, 0.47106E+00_JPRB, 0.46222E+00_JPRB, 0.45930E+00_JPRB, 0.45622E+00_JPRB ,&
&  0.45347E+00_JPRB, 0.45312E+00_JPRB, 0.45754E+00_JPRB, 0.46147E+00_JPRB, 0.46561E+00_JPRB, 0.47032E+00_JPRB, 0.47857E+00_JPRB ,&
&  0.48621E+00_JPRB, 0.49516E+00_JPRB, 0.50668E+00_JPRB, 0.51865E+00_JPRB, 0.52854E+00_JPRB, 0.53574E+00_JPRB, 0.54597E+00_JPRB ,&
&  0.55339E+00_JPRB, 0.56296E+00_JPRB, 0.56980E+00_JPRB, 0.57311E+00_JPRB, 0.57312E+00_JPRB, 0.57309E+00_JPRB, 0.57414E+00_JPRB ,&
&  0.57332E+00_JPRB, 0.57741E+00_JPRB, 0.58123E+00_JPRB, 0.58529E+00_JPRB, 0.58645E+00_JPRB, 0.59183E+00_JPRB, 0.59534E+00_JPRB ,&
&  0.60282E+00_JPRB, 0.61433E+00_JPRB, 0.62687E+00_JPRB, 0.64128E+00_JPRB, 0.65888E+00_JPRB, 0.68088E+00_JPRB, 0.69937E+00_JPRB ,&
&  0.71677E+00_JPRB, 0.73274E+00_JPRB, 0.74734E+00_JPRB, 0.75682E+00_JPRB, 0.76561E+00_JPRB, 0.77380E+00_JPRB, 0.78210E+00_JPRB ,&
&  0.78611E+00_JPRB, 0.78935E+00_JPRB, 0.79203E+00_JPRB, 0.79349E+00_JPRB, 0.79098E+00_JPRB, 0.78764E+00_JPRB, 0.78424E+00_JPRB ,&
&  0.78059E+00_JPRB, 0.77614E+00_JPRB, 0.77230E+00_JPRB, 0.76944E+00_JPRB, 0.76597E+00_JPRB, 0.76147E+00_JPRB, 0.75700E+00_JPRB ,&
&  0.75282E+00_JPRB, 0.74760E+00_JPRB, 0.74184E+00_JPRB, 0.73596E+00_JPRB, 0.73030E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.64043E+00_JPRB, 0.63752E+00_JPRB, 0.63412E+00_JPRB, 0.63116E+00_JPRB, 0.62740E+00_JPRB, 0.62414E+00_JPRB, 0.62068E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.61466E+00_JPRB, 0.61190E+00_JPRB, 0.60868E+00_JPRB, 0.60563E+00_JPRB, 0.60263E+00_JPRB, 0.59973E+00_JPRB ,&
&  0.59667E+00_JPRB, 0.59388E+00_JPRB, 0.59087E+00_JPRB, 0.58751E+00_JPRB, 0.58476E+00_JPRB, 0.58163E+00_JPRB, 0.57700E+00_JPRB ,&
&  0.57011E+00_JPRB, 0.56279E+00_JPRB, 0.55598E+00_JPRB, 0.55064E+00_JPRB, 0.54683E+00_JPRB, 0.54378E+00_JPRB, 0.54115E+00_JPRB ,&
&  0.53955E+00_JPRB, 0.53862E+00_JPRB, 0.53780E+00_JPRB, 0.53628E+00_JPRB, 0.53518E+00_JPRB, 0.53585E+00_JPRB, 0.53608E+00_JPRB ,&
&  0.53568E+00_JPRB, 0.53559E+00_JPRB, 0.53662E+00_JPRB, 0.53758E+00_JPRB, 0.53828E+00_JPRB, 0.53991E+00_JPRB, 0.54296E+00_JPRB ,&
&  0.54615E+00_JPRB, 0.54892E+00_JPRB, 0.55258E+00_JPRB, 0.55639E+00_JPRB, 0.56017E+00_JPRB, 0.56358E+00_JPRB, 0.56861E+00_JPRB ,&
&  0.57385E+00_JPRB, 0.57866E+00_JPRB, 0.58301E+00_JPRB, 0.58625E+00_JPRB, 0.58868E+00_JPRB, 0.58975E+00_JPRB, 0.58997E+00_JPRB ,&
&  0.58937E+00_JPRB, 0.58997E+00_JPRB, 0.58969E+00_JPRB, 0.58931E+00_JPRB, 0.58820E+00_JPRB, 0.58890E+00_JPRB, 0.58937E+00_JPRB ,&
&  0.59123E+00_JPRB, 0.59247E+00_JPRB, 0.59502E+00_JPRB, 0.59748E+00_JPRB, 0.60130E+00_JPRB, 0.60543E+00_JPRB, 0.60982E+00_JPRB ,&
&  0.61477E+00_JPRB, 0.62063E+00_JPRB, 0.62786E+00_JPRB, 0.63252E+00_JPRB, 0.63703E+00_JPRB, 0.64129E+00_JPRB, 0.64604E+00_JPRB ,&
&  0.64993E+00_JPRB, 0.65336E+00_JPRB, 0.65670E+00_JPRB, 0.65910E+00_JPRB, 0.66051E+00_JPRB, 0.66160E+00_JPRB, 0.66255E+00_JPRB ,&
&  0.66264E+00_JPRB, 0.66248E+00_JPRB, 0.66217E+00_JPRB, 0.66193E+00_JPRB, 0.66044E+00_JPRB, 0.65840E+00_JPRB, 0.65608E+00_JPRB ,&
&  0.65414E+00_JPRB, 0.65149E+00_JPRB, 0.64899E+00_JPRB, 0.64620E+00_JPRB, 0.64357E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB, 0.58690E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB ,&
&  0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB, 0.23731E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.25132E+00_JPRB, 0.25159E+00_JPRB, 0.25180E+00_JPRB, 0.25201E+00_JPRB, 0.25118E+00_JPRB, 0.25181E+00_JPRB, 0.25230E+00_JPRB ,&
&  0.25302E+00_JPRB, 0.25418E+00_JPRB, 0.25554E+00_JPRB, 0.25660E+00_JPRB, 0.25742E+00_JPRB, 0.25792E+00_JPRB, 0.25826E+00_JPRB ,&
&  0.25830E+00_JPRB, 0.25847E+00_JPRB, 0.25558E+00_JPRB, 0.25457E+00_JPRB, 0.25429E+00_JPRB, 0.25437E+00_JPRB, 0.25458E+00_JPRB ,&
&  0.25475E+00_JPRB, 0.25479E+00_JPRB, 0.25439E+00_JPRB, 0.25466E+00_JPRB, 0.25495E+00_JPRB, 0.25515E+00_JPRB, 0.25540E+00_JPRB ,&
&  0.25561E+00_JPRB, 0.25577E+00_JPRB, 0.25569E+00_JPRB, 0.25548E+00_JPRB, 0.25530E+00_JPRB, 0.25476E+00_JPRB, 0.25388E+00_JPRB ,&
&  0.25282E+00_JPRB, 0.25151E+00_JPRB, 0.25020E+00_JPRB, 0.24901E+00_JPRB, 0.24797E+00_JPRB, 0.24709E+00_JPRB, 0.24624E+00_JPRB ,&
&  0.24525E+00_JPRB, 0.24397E+00_JPRB, 0.24262E+00_JPRB, 0.24131E+00_JPRB, 0.24006E+00_JPRB, 0.23892E+00_JPRB, 0.23774E+00_JPRB ,&
&  0.23682E+00_JPRB, 0.23604E+00_JPRB, 0.23526E+00_JPRB, 0.23458E+00_JPRB, 0.23393E+00_JPRB, 0.23332E+00_JPRB, 0.23272E+00_JPRB ,&
&  0.23193E+00_JPRB, 0.23147E+00_JPRB, 0.23096E+00_JPRB, 0.23070E+00_JPRB, 0.23070E+00_JPRB, 0.23068E+00_JPRB, 0.23074E+00_JPRB ,&
&  0.23085E+00_JPRB, 0.23104E+00_JPRB, 0.23114E+00_JPRB, 0.23132E+00_JPRB, 0.23161E+00_JPRB, 0.23200E+00_JPRB, 0.23247E+00_JPRB ,&
&  0.23299E+00_JPRB, 0.23355E+00_JPRB, 0.23417E+00_JPRB, 0.23486E+00_JPRB, 0.23560E+00_JPRB, 0.23640E+00_JPRB, 0.23717E+00_JPRB ,&
&  0.23795E+00_JPRB, 0.23867E+00_JPRB, 0.23939E+00_JPRB, 0.23994E+00_JPRB, 0.24072E+00_JPRB, 0.24148E+00_JPRB, 0.24238E+00_JPRB ,&
&  0.24329E+00_JPRB, 0.24425E+00_JPRB, 0.24510E+00_JPRB, 0.24599E+00_JPRB, 0.24696E+00_JPRB, 0.24686E+00_JPRB, 0.24681E+00_JPRB ,&
&  0.24683E+00_JPRB, 0.24712E+00_JPRB, 0.24719E+00_JPRB, 0.24756E+00_JPRB, 0.24839E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.28002E+00_JPRB, 0.28657E+00_JPRB, 0.29152E+00_JPRB, 0.29670E+00_JPRB, 0.29403E+00_JPRB, 0.33112E+00_JPRB, 0.32965E+00_JPRB ,&
&  0.33012E+00_JPRB, 0.31588E+00_JPRB, 0.31583E+00_JPRB, 0.31409E+00_JPRB, 0.31094E+00_JPRB, 0.30641E+00_JPRB, 0.30220E+00_JPRB ,&
&  0.29850E+00_JPRB, 0.29370E+00_JPRB, 0.28817E+00_JPRB, 0.28349E+00_JPRB, 0.27971E+00_JPRB, 0.27649E+00_JPRB, 0.27374E+00_JPRB ,&
&  0.27313E+00_JPRB, 0.27436E+00_JPRB, 0.27675E+00_JPRB, 0.27598E+00_JPRB, 0.27723E+00_JPRB, 0.27934E+00_JPRB, 0.28174E+00_JPRB ,&
&  0.28404E+00_JPRB, 0.28592E+00_JPRB, 0.28599E+00_JPRB, 0.28435E+00_JPRB, 0.28294E+00_JPRB, 0.28048E+00_JPRB, 0.27822E+00_JPRB ,&
&  0.27556E+00_JPRB, 0.27261E+00_JPRB, 0.26916E+00_JPRB, 0.26640E+00_JPRB, 0.26342E+00_JPRB, 0.25979E+00_JPRB, 0.25628E+00_JPRB ,&
&  0.25327E+00_JPRB, 0.25058E+00_JPRB, 0.24772E+00_JPRB, 0.24506E+00_JPRB, 0.24255E+00_JPRB, 0.23998E+00_JPRB, 0.23775E+00_JPRB ,&
&  0.23607E+00_JPRB, 0.23464E+00_JPRB, 0.23320E+00_JPRB, 0.23170E+00_JPRB, 0.23022E+00_JPRB, 0.22878E+00_JPRB, 0.22759E+00_JPRB ,&
&  0.22733E+00_JPRB, 0.22662E+00_JPRB, 0.22592E+00_JPRB, 0.22549E+00_JPRB, 0.22545E+00_JPRB, 0.22495E+00_JPRB, 0.22473E+00_JPRB ,&
&  0.22465E+00_JPRB, 0.22467E+00_JPRB, 0.22490E+00_JPRB, 0.22519E+00_JPRB, 0.22561E+00_JPRB, 0.22615E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22804E+00_JPRB, 0.22928E+00_JPRB, 0.23061E+00_JPRB, 0.23197E+00_JPRB, 0.23337E+00_JPRB, 0.23484E+00_JPRB, 0.23650E+00_JPRB ,&
&  0.23817E+00_JPRB, 0.23980E+00_JPRB, 0.24141E+00_JPRB, 0.24299E+00_JPRB, 0.24549E+00_JPRB, 0.24793E+00_JPRB, 0.25052E+00_JPRB ,&
&  0.25332E+00_JPRB, 0.25661E+00_JPRB, 0.26007E+00_JPRB, 0.26377E+00_JPRB, 0.26785E+00_JPRB, 0.26968E+00_JPRB, 0.27116E+00_JPRB ,&
&  0.27223E+00_JPRB, 0.27510E+00_JPRB, 0.27752E+00_JPRB, 0.27768E+00_JPRB, 0.27766E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.33848E+00_JPRB, 0.33628E+00_JPRB, 0.33660E+00_JPRB, 0.33737E+00_JPRB, 0.34059E+00_JPRB, 0.34556E+00_JPRB, 0.35098E+00_JPRB ,&
&  0.35046E+00_JPRB, 0.35345E+00_JPRB, 0.35271E+00_JPRB, 0.35032E+00_JPRB, 0.34627E+00_JPRB, 0.34261E+00_JPRB, 0.33777E+00_JPRB ,&
&  0.33089E+00_JPRB, 0.32212E+00_JPRB, 0.31355E+00_JPRB, 0.30489E+00_JPRB, 0.29663E+00_JPRB, 0.28804E+00_JPRB, 0.28270E+00_JPRB ,&
&  0.27947E+00_JPRB, 0.27860E+00_JPRB, 0.28059E+00_JPRB, 0.28347E+00_JPRB, 0.28887E+00_JPRB, 0.29620E+00_JPRB, 0.30590E+00_JPRB ,&
&  0.31550E+00_JPRB, 0.32603E+00_JPRB, 0.33148E+00_JPRB, 0.33135E+00_JPRB, 0.33104E+00_JPRB, 0.32890E+00_JPRB, 0.32539E+00_JPRB ,&
&  0.31898E+00_JPRB, 0.31161E+00_JPRB, 0.30320E+00_JPRB, 0.29430E+00_JPRB, 0.28522E+00_JPRB, 0.27647E+00_JPRB, 0.26834E+00_JPRB ,&
&  0.26172E+00_JPRB, 0.25609E+00_JPRB, 0.25098E+00_JPRB, 0.24672E+00_JPRB, 0.24341E+00_JPRB, 0.24092E+00_JPRB, 0.23801E+00_JPRB ,&
&  0.23554E+00_JPRB, 0.23338E+00_JPRB, 0.23083E+00_JPRB, 0.22904E+00_JPRB, 0.22744E+00_JPRB, 0.22639E+00_JPRB, 0.22552E+00_JPRB ,&
&  0.22536E+00_JPRB, 0.22520E+00_JPRB, 0.22566E+00_JPRB, 0.22611E+00_JPRB, 0.22668E+00_JPRB, 0.22655E+00_JPRB, 0.22626E+00_JPRB ,&
&  0.22581E+00_JPRB, 0.22539E+00_JPRB, 0.22415E+00_JPRB, 0.22321E+00_JPRB, 0.22257E+00_JPRB, 0.22186E+00_JPRB, 0.22189E+00_JPRB ,&
&  0.22274E+00_JPRB, 0.22363E+00_JPRB, 0.22439E+00_JPRB, 0.22596E+00_JPRB, 0.22753E+00_JPRB, 0.22978E+00_JPRB, 0.23248E+00_JPRB ,&
&  0.23448E+00_JPRB, 0.23684E+00_JPRB, 0.23947E+00_JPRB, 0.24252E+00_JPRB, 0.24614E+00_JPRB, 0.24992E+00_JPRB, 0.25433E+00_JPRB ,&
&  0.25930E+00_JPRB, 0.26444E+00_JPRB, 0.27025E+00_JPRB, 0.27664E+00_JPRB, 0.28441E+00_JPRB, 0.29169E+00_JPRB, 0.30216E+00_JPRB ,&
&  0.31069E+00_JPRB, 0.31137E+00_JPRB, 0.35174E+00_JPRB, 0.34546E+00_JPRB, 0.34120E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.33550E+00_JPRB, 0.33806E+00_JPRB, 0.34132E+00_JPRB, 0.34528E+00_JPRB, 0.35019E+00_JPRB, 0.35451E+00_JPRB, 0.35969E+00_JPRB ,&
&  0.36263E+00_JPRB, 0.35896E+00_JPRB, 0.35936E+00_JPRB, 0.35528E+00_JPRB, 0.35108E+00_JPRB, 0.34573E+00_JPRB, 0.33976E+00_JPRB ,&
&  0.33335E+00_JPRB, 0.32441E+00_JPRB, 0.31483E+00_JPRB, 0.30630E+00_JPRB, 0.29913E+00_JPRB, 0.29343E+00_JPRB, 0.28879E+00_JPRB ,&
&  0.28560E+00_JPRB, 0.28579E+00_JPRB, 0.28793E+00_JPRB, 0.29391E+00_JPRB, 0.30398E+00_JPRB, 0.31663E+00_JPRB, 0.33569E+00_JPRB ,&
&  0.36275E+00_JPRB, 0.36487E+00_JPRB, 0.36950E+00_JPRB, 0.36849E+00_JPRB, 0.36428E+00_JPRB, 0.36207E+00_JPRB, 0.35772E+00_JPRB ,&
&  0.35817E+00_JPRB, 0.35135E+00_JPRB, 0.36124E+00_JPRB, 0.34550E+00_JPRB, 0.31939E+00_JPRB, 0.30403E+00_JPRB, 0.28582E+00_JPRB ,&
&  0.27218E+00_JPRB, 0.26061E+00_JPRB, 0.25190E+00_JPRB, 0.24801E+00_JPRB, 0.24434E+00_JPRB, 0.24258E+00_JPRB, 0.23906E+00_JPRB ,&
&  0.23536E+00_JPRB, 0.23338E+00_JPRB, 0.23030E+00_JPRB, 0.22992E+00_JPRB, 0.22994E+00_JPRB, 0.22939E+00_JPRB, 0.22948E+00_JPRB ,&
&  0.22985E+00_JPRB, 0.23078E+00_JPRB, 0.23240E+00_JPRB, 0.23391E+00_JPRB, 0.23480E+00_JPRB, 0.23532E+00_JPRB, 0.23518E+00_JPRB ,&
&  0.23557E+00_JPRB, 0.23526E+00_JPRB, 0.23445E+00_JPRB, 0.23318E+00_JPRB, 0.23132E+00_JPRB, 0.22938E+00_JPRB, 0.22755E+00_JPRB ,&
&  0.22610E+00_JPRB, 0.22560E+00_JPRB, 0.22521E+00_JPRB, 0.22549E+00_JPRB, 0.22613E+00_JPRB, 0.22786E+00_JPRB, 0.22949E+00_JPRB ,&
&  0.23136E+00_JPRB, 0.23390E+00_JPRB, 0.23617E+00_JPRB, 0.23893E+00_JPRB, 0.24151E+00_JPRB, 0.24314E+00_JPRB, 0.24712E+00_JPRB ,&
&  0.25220E+00_JPRB, 0.25894E+00_JPRB, 0.26682E+00_JPRB, 0.27455E+00_JPRB, 0.28442E+00_JPRB, 0.30017E+00_JPRB, 0.32170E+00_JPRB ,&
&  0.34391E+00_JPRB, 0.36868E+00_JPRB, 0.36338E+00_JPRB, 0.33477E+00_JPRB, 0.33277E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.34615E+00_JPRB, 0.35277E+00_JPRB, 0.35983E+00_JPRB, 0.36735E+00_JPRB, 0.37671E+00_JPRB, 0.38069E+00_JPRB, 0.38267E+00_JPRB ,&
&  0.37866E+00_JPRB, 0.37386E+00_JPRB, 0.36291E+00_JPRB, 0.35260E+00_JPRB, 0.34303E+00_JPRB, 0.33537E+00_JPRB, 0.32972E+00_JPRB ,&
&  0.32659E+00_JPRB, 0.32416E+00_JPRB, 0.32060E+00_JPRB, 0.31734E+00_JPRB, 0.31460E+00_JPRB, 0.31453E+00_JPRB, 0.31462E+00_JPRB ,&
&  0.31378E+00_JPRB, 0.31383E+00_JPRB, 0.31259E+00_JPRB, 0.31775E+00_JPRB, 0.32684E+00_JPRB, 0.33806E+00_JPRB, 0.35270E+00_JPRB ,&
&  0.37319E+00_JPRB, 0.38446E+00_JPRB, 0.40001E+00_JPRB, 0.40058E+00_JPRB, 0.39869E+00_JPRB, 0.39482E+00_JPRB, 0.39169E+00_JPRB ,&
&  0.38706E+00_JPRB, 0.38043E+00_JPRB, 0.37564E+00_JPRB, 0.36974E+00_JPRB, 0.34083E+00_JPRB, 0.31792E+00_JPRB, 0.30037E+00_JPRB ,&
&  0.28498E+00_JPRB, 0.27046E+00_JPRB, 0.25783E+00_JPRB, 0.25140E+00_JPRB, 0.24639E+00_JPRB, 0.24344E+00_JPRB, 0.24036E+00_JPRB ,&
&  0.23709E+00_JPRB, 0.23578E+00_JPRB, 0.23383E+00_JPRB, 0.23315E+00_JPRB, 0.23410E+00_JPRB, 0.23410E+00_JPRB, 0.23487E+00_JPRB ,&
&  0.23552E+00_JPRB, 0.23648E+00_JPRB, 0.23871E+00_JPRB, 0.24040E+00_JPRB, 0.24219E+00_JPRB, 0.24391E+00_JPRB, 0.24479E+00_JPRB ,&
&  0.24652E+00_JPRB, 0.24795E+00_JPRB, 0.24792E+00_JPRB, 0.24723E+00_JPRB, 0.24555E+00_JPRB, 0.24187E+00_JPRB, 0.23817E+00_JPRB ,&
&  0.23421E+00_JPRB, 0.23134E+00_JPRB, 0.23013E+00_JPRB, 0.22918E+00_JPRB, 0.22995E+00_JPRB, 0.23076E+00_JPRB, 0.23172E+00_JPRB ,&
&  0.23357E+00_JPRB, 0.23530E+00_JPRB, 0.23726E+00_JPRB, 0.24001E+00_JPRB, 0.24217E+00_JPRB, 0.24407E+00_JPRB, 0.24651E+00_JPRB ,&
&  0.24705E+00_JPRB, 0.25126E+00_JPRB, 0.25653E+00_JPRB, 0.26315E+00_JPRB, 0.27423E+00_JPRB, 0.28415E+00_JPRB, 0.30661E+00_JPRB ,&
&  0.34327E+00_JPRB, 0.41274E+00_JPRB, 0.40708E+00_JPRB, 0.33782E+00_JPRB, 0.34035E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.36924E+00_JPRB, 0.37791E+00_JPRB, 0.38559E+00_JPRB, 0.39150E+00_JPRB, 0.39509E+00_JPRB, 0.39571E+00_JPRB, 0.39834E+00_JPRB ,&
&  0.40571E+00_JPRB, 0.40999E+00_JPRB, 0.41296E+00_JPRB, 0.41234E+00_JPRB, 0.41179E+00_JPRB, 0.39290E+00_JPRB, 0.38792E+00_JPRB ,&
&  0.38204E+00_JPRB, 0.37792E+00_JPRB, 0.37233E+00_JPRB, 0.37231E+00_JPRB, 0.37215E+00_JPRB, 0.37410E+00_JPRB, 0.37484E+00_JPRB ,&
&  0.37480E+00_JPRB, 0.37265E+00_JPRB, 0.36938E+00_JPRB, 0.36739E+00_JPRB, 0.36868E+00_JPRB, 0.37022E+00_JPRB, 0.36739E+00_JPRB ,&
&  0.36317E+00_JPRB, 0.36231E+00_JPRB, 0.36562E+00_JPRB, 0.37309E+00_JPRB, 0.41386E+00_JPRB, 0.40883E+00_JPRB, 0.40626E+00_JPRB ,&
&  0.40293E+00_JPRB, 0.39878E+00_JPRB, 0.39826E+00_JPRB, 0.39183E+00_JPRB, 0.38238E+00_JPRB, 0.36170E+00_JPRB, 0.35012E+00_JPRB ,&
&  0.31545E+00_JPRB, 0.29613E+00_JPRB, 0.27909E+00_JPRB, 0.26792E+00_JPRB, 0.25796E+00_JPRB, 0.25220E+00_JPRB, 0.24694E+00_JPRB ,&
&  0.24296E+00_JPRB, 0.23986E+00_JPRB, 0.23710E+00_JPRB, 0.23748E+00_JPRB, 0.23914E+00_JPRB, 0.24012E+00_JPRB, 0.24109E+00_JPRB ,&
&  0.24165E+00_JPRB, 0.24211E+00_JPRB, 0.24274E+00_JPRB, 0.24399E+00_JPRB, 0.24565E+00_JPRB, 0.24785E+00_JPRB, 0.25063E+00_JPRB ,&
&  0.25522E+00_JPRB, 0.25890E+00_JPRB, 0.26045E+00_JPRB, 0.26298E+00_JPRB, 0.26266E+00_JPRB, 0.25737E+00_JPRB, 0.25111E+00_JPRB ,&
&  0.24460E+00_JPRB, 0.24121E+00_JPRB, 0.23790E+00_JPRB, 0.23561E+00_JPRB, 0.23507E+00_JPRB, 0.23509E+00_JPRB, 0.23693E+00_JPRB ,&
&  0.23895E+00_JPRB, 0.24073E+00_JPRB, 0.24196E+00_JPRB, 0.24332E+00_JPRB, 0.24507E+00_JPRB, 0.24645E+00_JPRB, 0.24830E+00_JPRB ,&
&  0.25063E+00_JPRB, 0.25381E+00_JPRB, 0.25854E+00_JPRB, 0.26396E+00_JPRB, 0.27069E+00_JPRB, 0.28095E+00_JPRB, 0.29925E+00_JPRB ,&
&  0.32400E+00_JPRB, 0.38063E+00_JPRB, 0.38668E+00_JPRB, 0.35601E+00_JPRB, 0.36199E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.38184E+00_JPRB, 0.39181E+00_JPRB, 0.39708E+00_JPRB, 0.40227E+00_JPRB, 0.40434E+00_JPRB, 0.40567E+00_JPRB, 0.41352E+00_JPRB ,&
&  0.41828E+00_JPRB, 0.42197E+00_JPRB, 0.42419E+00_JPRB, 0.42490E+00_JPRB, 0.42046E+00_JPRB, 0.41378E+00_JPRB, 0.41153E+00_JPRB ,&
&  0.41031E+00_JPRB, 0.41074E+00_JPRB, 0.40959E+00_JPRB, 0.40958E+00_JPRB, 0.41012E+00_JPRB, 0.41493E+00_JPRB, 0.42168E+00_JPRB ,&
&  0.42326E+00_JPRB, 0.42787E+00_JPRB, 0.44037E+00_JPRB, 0.44810E+00_JPRB, 0.45299E+00_JPRB, 0.45274E+00_JPRB, 0.44712E+00_JPRB ,&
&  0.42016E+00_JPRB, 0.39240E+00_JPRB, 0.36481E+00_JPRB, 0.36636E+00_JPRB, 0.42006E+00_JPRB, 0.42655E+00_JPRB, 0.42565E+00_JPRB ,&
&  0.42404E+00_JPRB, 0.42843E+00_JPRB, 0.42857E+00_JPRB, 0.41405E+00_JPRB, 0.40342E+00_JPRB, 0.38862E+00_JPRB, 0.37597E+00_JPRB ,&
&  0.36535E+00_JPRB, 0.35757E+00_JPRB, 0.34454E+00_JPRB, 0.31744E+00_JPRB, 0.29967E+00_JPRB, 0.29058E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.26663E+00_JPRB, 0.26224E+00_JPRB, 0.26115E+00_JPRB, 0.26082E+00_JPRB, 0.26490E+00_JPRB, 0.26679E+00_JPRB, 0.26570E+00_JPRB ,&
&  0.26272E+00_JPRB, 0.26019E+00_JPRB, 0.25661E+00_JPRB, 0.25152E+00_JPRB, 0.24877E+00_JPRB, 0.24782E+00_JPRB, 0.24814E+00_JPRB ,&
&  0.25173E+00_JPRB, 0.25753E+00_JPRB, 0.27058E+00_JPRB, 0.27993E+00_JPRB, 0.27693E+00_JPRB, 0.26596E+00_JPRB, 0.25558E+00_JPRB ,&
&  0.24900E+00_JPRB, 0.24630E+00_JPRB, 0.24427E+00_JPRB, 0.24221E+00_JPRB, 0.24278E+00_JPRB, 0.24471E+00_JPRB, 0.24618E+00_JPRB ,&
&  0.24649E+00_JPRB, 0.24802E+00_JPRB, 0.24887E+00_JPRB, 0.24994E+00_JPRB, 0.25089E+00_JPRB, 0.25247E+00_JPRB, 0.25469E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.26209E+00_JPRB, 0.26791E+00_JPRB, 0.27437E+00_JPRB, 0.27843E+00_JPRB, 0.28146E+00_JPRB, 0.28729E+00_JPRB ,&
&  0.29367E+00_JPRB, 0.30947E+00_JPRB, 0.34007E+00_JPRB, 0.37768E+00_JPRB, 0.37293E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.39246E+00_JPRB, 0.39948E+00_JPRB, 0.40773E+00_JPRB, 0.41857E+00_JPRB, 0.42093E+00_JPRB, 0.42252E+00_JPRB, 0.43167E+00_JPRB ,&
&  0.43216E+00_JPRB, 0.42926E+00_JPRB, 0.42598E+00_JPRB, 0.42609E+00_JPRB, 0.42799E+00_JPRB, 0.43020E+00_JPRB, 0.42952E+00_JPRB ,&
&  0.42025E+00_JPRB, 0.41487E+00_JPRB, 0.41740E+00_JPRB, 0.42021E+00_JPRB, 0.42469E+00_JPRB, 0.43119E+00_JPRB, 0.43683E+00_JPRB ,&
&  0.43984E+00_JPRB, 0.44334E+00_JPRB, 0.44565E+00_JPRB, 0.45177E+00_JPRB, 0.46135E+00_JPRB, 0.46794E+00_JPRB, 0.47436E+00_JPRB ,&
&  0.46084E+00_JPRB, 0.41333E+00_JPRB, 0.38269E+00_JPRB, 0.38924E+00_JPRB, 0.43952E+00_JPRB, 0.44968E+00_JPRB, 0.45051E+00_JPRB ,&
&  0.45352E+00_JPRB, 0.46112E+00_JPRB, 0.46052E+00_JPRB, 0.45456E+00_JPRB, 0.44282E+00_JPRB, 0.42398E+00_JPRB, 0.40088E+00_JPRB ,&
&  0.38541E+00_JPRB, 0.37501E+00_JPRB, 0.36825E+00_JPRB, 0.36172E+00_JPRB, 0.35824E+00_JPRB, 0.35329E+00_JPRB, 0.34569E+00_JPRB ,&
&  0.34273E+00_JPRB, 0.34125E+00_JPRB, 0.34222E+00_JPRB, 0.34393E+00_JPRB, 0.34598E+00_JPRB, 0.34750E+00_JPRB, 0.34511E+00_JPRB ,&
&  0.33281E+00_JPRB, 0.31737E+00_JPRB, 0.30263E+00_JPRB, 0.28658E+00_JPRB, 0.27278E+00_JPRB, 0.26397E+00_JPRB, 0.25699E+00_JPRB ,&
&  0.25416E+00_JPRB, 0.25953E+00_JPRB, 0.27583E+00_JPRB, 0.29479E+00_JPRB, 0.32748E+00_JPRB, 0.30329E+00_JPRB, 0.27975E+00_JPRB ,&
&  0.26808E+00_JPRB, 0.26249E+00_JPRB, 0.25936E+00_JPRB, 0.25924E+00_JPRB, 0.26076E+00_JPRB, 0.26121E+00_JPRB, 0.26128E+00_JPRB ,&
&  0.26241E+00_JPRB, 0.26463E+00_JPRB, 0.26643E+00_JPRB, 0.26795E+00_JPRB, 0.27038E+00_JPRB, 0.27323E+00_JPRB, 0.27477E+00_JPRB ,&
&  0.27550E+00_JPRB, 0.28003E+00_JPRB, 0.28941E+00_JPRB, 0.30021E+00_JPRB, 0.31112E+00_JPRB, 0.32248E+00_JPRB, 0.34017E+00_JPRB ,&
&  0.36245E+00_JPRB, 0.38974E+00_JPRB, 0.38898E+00_JPRB, 0.38878E+00_JPRB, 0.38917E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.41480E+00_JPRB, 0.42612E+00_JPRB, 0.43939E+00_JPRB, 0.44872E+00_JPRB, 0.44782E+00_JPRB, 0.44783E+00_JPRB, 0.45406E+00_JPRB ,&
&  0.44823E+00_JPRB, 0.44116E+00_JPRB, 0.43522E+00_JPRB, 0.42905E+00_JPRB, 0.42677E+00_JPRB, 0.42532E+00_JPRB, 0.42820E+00_JPRB ,&
&  0.43028E+00_JPRB, 0.42915E+00_JPRB, 0.42742E+00_JPRB, 0.42803E+00_JPRB, 0.43444E+00_JPRB, 0.44258E+00_JPRB, 0.44943E+00_JPRB ,&
&  0.45066E+00_JPRB, 0.45410E+00_JPRB, 0.45968E+00_JPRB, 0.46440E+00_JPRB, 0.46688E+00_JPRB, 0.46848E+00_JPRB, 0.46741E+00_JPRB ,&
&  0.46503E+00_JPRB, 0.46329E+00_JPRB, 0.44640E+00_JPRB, 0.46042E+00_JPRB, 0.46496E+00_JPRB, 0.46596E+00_JPRB, 0.47168E+00_JPRB ,&
&  0.47948E+00_JPRB, 0.48810E+00_JPRB, 0.49200E+00_JPRB, 0.49316E+00_JPRB, 0.48852E+00_JPRB, 0.47928E+00_JPRB, 0.45517E+00_JPRB ,&
&  0.43491E+00_JPRB, 0.41380E+00_JPRB, 0.40063E+00_JPRB, 0.39271E+00_JPRB, 0.38656E+00_JPRB, 0.38117E+00_JPRB, 0.37590E+00_JPRB ,&
&  0.36863E+00_JPRB, 0.36214E+00_JPRB, 0.35819E+00_JPRB, 0.35826E+00_JPRB, 0.36244E+00_JPRB, 0.37334E+00_JPRB, 0.38681E+00_JPRB ,&
&  0.39283E+00_JPRB, 0.38554E+00_JPRB, 0.37759E+00_JPRB, 0.36899E+00_JPRB, 0.35528E+00_JPRB, 0.32752E+00_JPRB, 0.31071E+00_JPRB ,&
&  0.30649E+00_JPRB, 0.31903E+00_JPRB, 0.34040E+00_JPRB, 0.34616E+00_JPRB, 0.35119E+00_JPRB, 0.34893E+00_JPRB, 0.34369E+00_JPRB ,&
&  0.33536E+00_JPRB, 0.31184E+00_JPRB, 0.29891E+00_JPRB, 0.29503E+00_JPRB, 0.29272E+00_JPRB, 0.29277E+00_JPRB, 0.29526E+00_JPRB ,&
&  0.29935E+00_JPRB, 0.30674E+00_JPRB, 0.31190E+00_JPRB, 0.31400E+00_JPRB, 0.31711E+00_JPRB, 0.31723E+00_JPRB, 0.31616E+00_JPRB ,&
&  0.31244E+00_JPRB, 0.31881E+00_JPRB, 0.33690E+00_JPRB, 0.36569E+00_JPRB, 0.37205E+00_JPRB, 0.37951E+00_JPRB, 0.38850E+00_JPRB ,&
&  0.39392E+00_JPRB, 0.39891E+00_JPRB, 0.39930E+00_JPRB, 0.40033E+00_JPRB, 0.40575E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.43363E+00_JPRB, 0.44633E+00_JPRB, 0.46243E+00_JPRB, 0.47390E+00_JPRB, 0.47892E+00_JPRB, 0.48440E+00_JPRB, 0.48685E+00_JPRB ,&
&  0.48190E+00_JPRB, 0.47990E+00_JPRB, 0.47433E+00_JPRB, 0.46619E+00_JPRB, 0.45911E+00_JPRB, 0.45549E+00_JPRB, 0.45578E+00_JPRB ,&
&  0.45792E+00_JPRB, 0.45975E+00_JPRB, 0.45624E+00_JPRB, 0.45443E+00_JPRB, 0.45823E+00_JPRB, 0.46343E+00_JPRB, 0.46681E+00_JPRB ,&
&  0.46813E+00_JPRB, 0.46950E+00_JPRB, 0.47421E+00_JPRB, 0.47725E+00_JPRB, 0.47744E+00_JPRB, 0.47099E+00_JPRB, 0.46631E+00_JPRB ,&
&  0.46153E+00_JPRB, 0.45995E+00_JPRB, 0.46679E+00_JPRB, 0.45874E+00_JPRB, 0.47374E+00_JPRB, 0.47591E+00_JPRB, 0.48160E+00_JPRB ,&
&  0.48980E+00_JPRB, 0.50323E+00_JPRB, 0.51347E+00_JPRB, 0.52133E+00_JPRB, 0.52383E+00_JPRB, 0.52711E+00_JPRB, 0.51743E+00_JPRB ,&
&  0.50378E+00_JPRB, 0.48520E+00_JPRB, 0.46663E+00_JPRB, 0.44668E+00_JPRB, 0.42773E+00_JPRB, 0.41290E+00_JPRB, 0.40554E+00_JPRB ,&
&  0.39662E+00_JPRB, 0.38281E+00_JPRB, 0.36947E+00_JPRB, 0.36417E+00_JPRB, 0.36870E+00_JPRB, 0.37881E+00_JPRB, 0.39434E+00_JPRB ,&
&  0.40378E+00_JPRB, 0.40655E+00_JPRB, 0.40335E+00_JPRB, 0.39897E+00_JPRB, 0.39442E+00_JPRB, 0.38738E+00_JPRB, 0.38333E+00_JPRB ,&
&  0.37998E+00_JPRB, 0.37431E+00_JPRB, 0.37138E+00_JPRB, 0.37314E+00_JPRB, 0.37632E+00_JPRB, 0.37829E+00_JPRB, 0.37663E+00_JPRB ,&
&  0.37354E+00_JPRB, 0.36918E+00_JPRB, 0.36118E+00_JPRB, 0.35659E+00_JPRB, 0.35665E+00_JPRB, 0.36104E+00_JPRB, 0.36869E+00_JPRB ,&
&  0.37120E+00_JPRB, 0.37706E+00_JPRB, 0.38283E+00_JPRB, 0.38499E+00_JPRB, 0.38761E+00_JPRB, 0.38980E+00_JPRB, 0.39001E+00_JPRB ,&
&  0.38833E+00_JPRB, 0.38332E+00_JPRB, 0.37628E+00_JPRB, 0.37307E+00_JPRB, 0.37394E+00_JPRB, 0.38047E+00_JPRB, 0.38955E+00_JPRB ,&
&  0.39864E+00_JPRB, 0.41234E+00_JPRB, 0.42290E+00_JPRB, 0.42535E+00_JPRB, 0.42761E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.46271E+00_JPRB, 0.46789E+00_JPRB, 0.48089E+00_JPRB, 0.49167E+00_JPRB, 0.49519E+00_JPRB, 0.50467E+00_JPRB, 0.51581E+00_JPRB ,&
&  0.52507E+00_JPRB, 0.52698E+00_JPRB, 0.52325E+00_JPRB, 0.51289E+00_JPRB, 0.50059E+00_JPRB, 0.49045E+00_JPRB, 0.48636E+00_JPRB ,&
&  0.48851E+00_JPRB, 0.49159E+00_JPRB, 0.49713E+00_JPRB, 0.50016E+00_JPRB, 0.50376E+00_JPRB, 0.50250E+00_JPRB, 0.50282E+00_JPRB ,&
&  0.50454E+00_JPRB, 0.50554E+00_JPRB, 0.50608E+00_JPRB, 0.50252E+00_JPRB, 0.49973E+00_JPRB, 0.49732E+00_JPRB, 0.48898E+00_JPRB ,&
&  0.47962E+00_JPRB, 0.47332E+00_JPRB, 0.47730E+00_JPRB, 0.48886E+00_JPRB, 0.49630E+00_JPRB, 0.49363E+00_JPRB, 0.49254E+00_JPRB ,&
&  0.49272E+00_JPRB, 0.49730E+00_JPRB, 0.50487E+00_JPRB, 0.51661E+00_JPRB, 0.52503E+00_JPRB, 0.53614E+00_JPRB, 0.53676E+00_JPRB ,&
&  0.53033E+00_JPRB, 0.52114E+00_JPRB, 0.51835E+00_JPRB, 0.51745E+00_JPRB, 0.48913E+00_JPRB, 0.45792E+00_JPRB, 0.44256E+00_JPRB ,&
&  0.43432E+00_JPRB, 0.42406E+00_JPRB, 0.40827E+00_JPRB, 0.39708E+00_JPRB, 0.39573E+00_JPRB, 0.40033E+00_JPRB, 0.41163E+00_JPRB ,&
&  0.41984E+00_JPRB, 0.43201E+00_JPRB, 0.44672E+00_JPRB, 0.45612E+00_JPRB, 0.45617E+00_JPRB, 0.44419E+00_JPRB, 0.43325E+00_JPRB ,&
&  0.42687E+00_JPRB, 0.42519E+00_JPRB, 0.42374E+00_JPRB, 0.42166E+00_JPRB, 0.41971E+00_JPRB, 0.41916E+00_JPRB, 0.41802E+00_JPRB ,&
&  0.41373E+00_JPRB, 0.40637E+00_JPRB, 0.39740E+00_JPRB, 0.39236E+00_JPRB, 0.39117E+00_JPRB, 0.39096E+00_JPRB, 0.39251E+00_JPRB ,&
&  0.40150E+00_JPRB, 0.41271E+00_JPRB, 0.42186E+00_JPRB, 0.42613E+00_JPRB, 0.42332E+00_JPRB, 0.41554E+00_JPRB, 0.41078E+00_JPRB ,&
&  0.40702E+00_JPRB, 0.40434E+00_JPRB, 0.40087E+00_JPRB, 0.39917E+00_JPRB, 0.39875E+00_JPRB, 0.40571E+00_JPRB, 0.42107E+00_JPRB ,&
&  0.44034E+00_JPRB, 0.46571E+00_JPRB, 0.47729E+00_JPRB, 0.47593E+00_JPRB, 0.46885E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.51775E+00_JPRB, 0.50089E+00_JPRB, 0.50111E+00_JPRB, 0.50810E+00_JPRB, 0.51209E+00_JPRB, 0.51989E+00_JPRB, 0.53925E+00_JPRB ,&
&  0.55073E+00_JPRB, 0.55811E+00_JPRB, 0.56099E+00_JPRB, 0.55555E+00_JPRB, 0.55439E+00_JPRB, 0.54708E+00_JPRB, 0.53431E+00_JPRB ,&
&  0.52901E+00_JPRB, 0.52793E+00_JPRB, 0.52954E+00_JPRB, 0.53326E+00_JPRB, 0.53537E+00_JPRB, 0.53803E+00_JPRB, 0.54104E+00_JPRB ,&
&  0.54266E+00_JPRB, 0.54261E+00_JPRB, 0.54178E+00_JPRB, 0.54207E+00_JPRB, 0.54123E+00_JPRB, 0.53382E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.51627E+00_JPRB, 0.50718E+00_JPRB, 0.50473E+00_JPRB, 0.50926E+00_JPRB, 0.51443E+00_JPRB, 0.51783E+00_JPRB, 0.51595E+00_JPRB ,&
&  0.51129E+00_JPRB, 0.50811E+00_JPRB, 0.51008E+00_JPRB, 0.51708E+00_JPRB, 0.52579E+00_JPRB, 0.53551E+00_JPRB, 0.54065E+00_JPRB ,&
&  0.53520E+00_JPRB, 0.51787E+00_JPRB, 0.50744E+00_JPRB, 0.50705E+00_JPRB, 0.50370E+00_JPRB, 0.48957E+00_JPRB, 0.47958E+00_JPRB ,&
&  0.47476E+00_JPRB, 0.46596E+00_JPRB, 0.45663E+00_JPRB, 0.44385E+00_JPRB, 0.43211E+00_JPRB, 0.42768E+00_JPRB, 0.42620E+00_JPRB ,&
&  0.42897E+00_JPRB, 0.44621E+00_JPRB, 0.47837E+00_JPRB, 0.51016E+00_JPRB, 0.52439E+00_JPRB, 0.51827E+00_JPRB, 0.50195E+00_JPRB ,&
&  0.49472E+00_JPRB, 0.49183E+00_JPRB, 0.49330E+00_JPRB, 0.49296E+00_JPRB, 0.48948E+00_JPRB, 0.48077E+00_JPRB, 0.47372E+00_JPRB ,&
&  0.46357E+00_JPRB, 0.45239E+00_JPRB, 0.44785E+00_JPRB, 0.44507E+00_JPRB, 0.44207E+00_JPRB, 0.44202E+00_JPRB, 0.44094E+00_JPRB ,&
&  0.44121E+00_JPRB, 0.44494E+00_JPRB, 0.45653E+00_JPRB, 0.46328E+00_JPRB, 0.46467E+00_JPRB, 0.45453E+00_JPRB, 0.45537E+00_JPRB ,&
&  0.45782E+00_JPRB, 0.45543E+00_JPRB, 0.45083E+00_JPRB, 0.45055E+00_JPRB, 0.45039E+00_JPRB, 0.45216E+00_JPRB, 0.46358E+00_JPRB ,&
&  0.49013E+00_JPRB, 0.51355E+00_JPRB, 0.53287E+00_JPRB, 0.53874E+00_JPRB, 0.53493E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.59131E+00_JPRB, 0.56296E+00_JPRB, 0.55142E+00_JPRB, 0.55303E+00_JPRB, 0.55638E+00_JPRB, 0.55215E+00_JPRB, 0.55933E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.57908E+00_JPRB, 0.58364E+00_JPRB, 0.57981E+00_JPRB, 0.57725E+00_JPRB, 0.57433E+00_JPRB, 0.55969E+00_JPRB ,&
&  0.55487E+00_JPRB, 0.55311E+00_JPRB, 0.55711E+00_JPRB, 0.56395E+00_JPRB, 0.57137E+00_JPRB, 0.58328E+00_JPRB, 0.59207E+00_JPRB ,&
&  0.59787E+00_JPRB, 0.60151E+00_JPRB, 0.60564E+00_JPRB, 0.60587E+00_JPRB, 0.60244E+00_JPRB, 0.59362E+00_JPRB, 0.58686E+00_JPRB ,&
&  0.58294E+00_JPRB, 0.57156E+00_JPRB, 0.56157E+00_JPRB, 0.56036E+00_JPRB, 0.56366E+00_JPRB, 0.56730E+00_JPRB, 0.56741E+00_JPRB ,&
&  0.55433E+00_JPRB, 0.53718E+00_JPRB, 0.53518E+00_JPRB, 0.53517E+00_JPRB, 0.54048E+00_JPRB, 0.55279E+00_JPRB, 0.55646E+00_JPRB ,&
&  0.55360E+00_JPRB, 0.53987E+00_JPRB, 0.52087E+00_JPRB, 0.51716E+00_JPRB, 0.51948E+00_JPRB, 0.51322E+00_JPRB, 0.50693E+00_JPRB ,&
&  0.50565E+00_JPRB, 0.50240E+00_JPRB, 0.49627E+00_JPRB, 0.48804E+00_JPRB, 0.47535E+00_JPRB, 0.45877E+00_JPRB, 0.45006E+00_JPRB ,&
&  0.44899E+00_JPRB, 0.46308E+00_JPRB, 0.48895E+00_JPRB, 0.51860E+00_JPRB, 0.54711E+00_JPRB, 0.56120E+00_JPRB, 0.56092E+00_JPRB ,&
&  0.54960E+00_JPRB, 0.54731E+00_JPRB, 0.55268E+00_JPRB, 0.55699E+00_JPRB, 0.55778E+00_JPRB, 0.54708E+00_JPRB, 0.53027E+00_JPRB ,&
&  0.51817E+00_JPRB, 0.50900E+00_JPRB, 0.50783E+00_JPRB, 0.51269E+00_JPRB, 0.51761E+00_JPRB, 0.51878E+00_JPRB, 0.51253E+00_JPRB ,&
&  0.50579E+00_JPRB, 0.49889E+00_JPRB, 0.49508E+00_JPRB, 0.49813E+00_JPRB, 0.50384E+00_JPRB, 0.50945E+00_JPRB, 0.52202E+00_JPRB ,&
&  0.54000E+00_JPRB, 0.55818E+00_JPRB, 0.56246E+00_JPRB, 0.55851E+00_JPRB, 0.55823E+00_JPRB, 0.55566E+00_JPRB, 0.55330E+00_JPRB ,&
&  0.56332E+00_JPRB, 0.58562E+00_JPRB, 0.60177E+00_JPRB, 0.61115E+00_JPRB, 0.61172E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.64137E+00_JPRB, 0.61667E+00_JPRB, 0.59836E+00_JPRB, 0.59140E+00_JPRB, 0.58764E+00_JPRB, 0.57549E+00_JPRB, 0.57008E+00_JPRB ,&
&  0.57608E+00_JPRB, 0.57796E+00_JPRB, 0.58497E+00_JPRB, 0.58454E+00_JPRB, 0.58374E+00_JPRB, 0.58622E+00_JPRB, 0.57851E+00_JPRB ,&
&  0.57843E+00_JPRB, 0.58732E+00_JPRB, 0.59772E+00_JPRB, 0.61179E+00_JPRB, 0.62360E+00_JPRB, 0.63254E+00_JPRB, 0.64015E+00_JPRB ,&
&  0.64317E+00_JPRB, 0.64969E+00_JPRB, 0.65331E+00_JPRB, 0.64926E+00_JPRB, 0.63868E+00_JPRB, 0.63054E+00_JPRB, 0.62931E+00_JPRB ,&
&  0.62832E+00_JPRB, 0.62254E+00_JPRB, 0.57862E+00_JPRB, 0.60253E+00_JPRB, 0.60996E+00_JPRB, 0.61462E+00_JPRB, 0.61796E+00_JPRB ,&
&  0.61254E+00_JPRB, 0.59708E+00_JPRB, 0.58994E+00_JPRB, 0.57847E+00_JPRB, 0.57754E+00_JPRB, 0.58381E+00_JPRB, 0.58268E+00_JPRB ,&
&  0.57883E+00_JPRB, 0.56843E+00_JPRB, 0.55421E+00_JPRB, 0.54304E+00_JPRB, 0.54008E+00_JPRB, 0.53817E+00_JPRB, 0.53079E+00_JPRB ,&
&  0.52551E+00_JPRB, 0.52444E+00_JPRB, 0.52257E+00_JPRB, 0.51681E+00_JPRB, 0.50226E+00_JPRB, 0.48875E+00_JPRB, 0.48589E+00_JPRB ,&
&  0.48875E+00_JPRB, 0.49511E+00_JPRB, 0.50833E+00_JPRB, 0.52584E+00_JPRB, 0.54497E+00_JPRB, 0.56303E+00_JPRB, 0.57954E+00_JPRB ,&
&  0.58629E+00_JPRB, 0.59016E+00_JPRB, 0.59373E+00_JPRB, 0.59821E+00_JPRB, 0.59884E+00_JPRB, 0.59102E+00_JPRB, 0.57191E+00_JPRB ,&
&  0.55162E+00_JPRB, 0.54640E+00_JPRB, 0.55046E+00_JPRB, 0.55957E+00_JPRB, 0.56431E+00_JPRB, 0.56372E+00_JPRB, 0.55686E+00_JPRB ,&
&  0.55228E+00_JPRB, 0.54370E+00_JPRB, 0.53501E+00_JPRB, 0.53085E+00_JPRB, 0.53075E+00_JPRB, 0.53693E+00_JPRB, 0.55431E+00_JPRB ,&
&  0.57502E+00_JPRB, 0.59816E+00_JPRB, 0.61903E+00_JPRB, 0.62715E+00_JPRB, 0.63177E+00_JPRB, 0.63170E+00_JPRB, 0.62333E+00_JPRB ,&
&  0.62761E+00_JPRB, 0.64652E+00_JPRB, 0.65652E+00_JPRB, 0.65833E+00_JPRB, 0.65636E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.69699E+00_JPRB, 0.67674E+00_JPRB, 0.66414E+00_JPRB, 0.65208E+00_JPRB, 0.64331E+00_JPRB, 0.62496E+00_JPRB, 0.61454E+00_JPRB ,&
&  0.60798E+00_JPRB, 0.60392E+00_JPRB, 0.60315E+00_JPRB, 0.60325E+00_JPRB, 0.61317E+00_JPRB, 0.61940E+00_JPRB, 0.62155E+00_JPRB ,&
&  0.62881E+00_JPRB, 0.64593E+00_JPRB, 0.66161E+00_JPRB, 0.67486E+00_JPRB, 0.68545E+00_JPRB, 0.69328E+00_JPRB, 0.69337E+00_JPRB ,&
&  0.69732E+00_JPRB, 0.70551E+00_JPRB, 0.70960E+00_JPRB, 0.70007E+00_JPRB, 0.68799E+00_JPRB, 0.68176E+00_JPRB, 0.68210E+00_JPRB ,&
&  0.67231E+00_JPRB, 0.62711E+00_JPRB, 0.64160E+00_JPRB, 0.65405E+00_JPRB, 0.66942E+00_JPRB, 0.67813E+00_JPRB, 0.68790E+00_JPRB ,&
&  0.68946E+00_JPRB, 0.67301E+00_JPRB, 0.65264E+00_JPRB, 0.63793E+00_JPRB, 0.64073E+00_JPRB, 0.63794E+00_JPRB, 0.62865E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.62036E+00_JPRB, 0.61245E+00_JPRB, 0.59911E+00_JPRB, 0.58718E+00_JPRB, 0.57913E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.56271E+00_JPRB, 0.55349E+00_JPRB, 0.54852E+00_JPRB, 0.54308E+00_JPRB, 0.53841E+00_JPRB, 0.54231E+00_JPRB, 0.54546E+00_JPRB ,&
&  0.54429E+00_JPRB, 0.54462E+00_JPRB, 0.55210E+00_JPRB, 0.56597E+00_JPRB, 0.57774E+00_JPRB, 0.59128E+00_JPRB, 0.60723E+00_JPRB ,&
&  0.62282E+00_JPRB, 0.63630E+00_JPRB, 0.64047E+00_JPRB, 0.64414E+00_JPRB, 0.63897E+00_JPRB, 0.63072E+00_JPRB, 0.61389E+00_JPRB ,&
&  0.59556E+00_JPRB, 0.59307E+00_JPRB, 0.59498E+00_JPRB, 0.60170E+00_JPRB, 0.60094E+00_JPRB, 0.59846E+00_JPRB, 0.59663E+00_JPRB ,&
&  0.59244E+00_JPRB, 0.58889E+00_JPRB, 0.58402E+00_JPRB, 0.57940E+00_JPRB, 0.58254E+00_JPRB, 0.58834E+00_JPRB, 0.60176E+00_JPRB ,&
&  0.61970E+00_JPRB, 0.63889E+00_JPRB, 0.65897E+00_JPRB, 0.67396E+00_JPRB, 0.68044E+00_JPRB, 0.68491E+00_JPRB, 0.68927E+00_JPRB ,&
&  0.69089E+00_JPRB, 0.69039E+00_JPRB, 0.69755E+00_JPRB, 0.70251E+00_JPRB, 0.70714E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.73743E+00_JPRB, 0.71703E+00_JPRB, 0.70448E+00_JPRB, 0.69837E+00_JPRB, 0.68843E+00_JPRB, 0.66857E+00_JPRB, 0.65232E+00_JPRB ,&
&  0.63731E+00_JPRB, 0.62903E+00_JPRB, 0.62541E+00_JPRB, 0.62423E+00_JPRB, 0.62964E+00_JPRB, 0.63464E+00_JPRB, 0.64089E+00_JPRB ,&
&  0.65168E+00_JPRB, 0.66943E+00_JPRB, 0.68473E+00_JPRB, 0.69910E+00_JPRB, 0.71217E+00_JPRB, 0.72520E+00_JPRB, 0.72893E+00_JPRB ,&
&  0.73196E+00_JPRB, 0.73963E+00_JPRB, 0.74149E+00_JPRB, 0.72974E+00_JPRB, 0.71723E+00_JPRB, 0.71974E+00_JPRB, 0.71659E+00_JPRB ,&
&  0.70853E+00_JPRB, 0.59732E+00_JPRB, 0.65121E+00_JPRB, 0.67901E+00_JPRB, 0.71093E+00_JPRB, 0.73838E+00_JPRB, 0.75693E+00_JPRB ,&
&  0.75190E+00_JPRB, 0.73350E+00_JPRB, 0.71210E+00_JPRB, 0.68460E+00_JPRB, 0.67605E+00_JPRB, 0.66834E+00_JPRB, 0.65843E+00_JPRB ,&
&  0.65102E+00_JPRB, 0.64394E+00_JPRB, 0.63516E+00_JPRB, 0.62365E+00_JPRB, 0.60904E+00_JPRB, 0.59810E+00_JPRB, 0.58834E+00_JPRB ,&
&  0.58561E+00_JPRB, 0.57806E+00_JPRB, 0.57116E+00_JPRB, 0.57173E+00_JPRB, 0.58327E+00_JPRB, 0.58832E+00_JPRB, 0.58812E+00_JPRB ,&
&  0.58767E+00_JPRB, 0.58823E+00_JPRB, 0.59545E+00_JPRB, 0.60826E+00_JPRB, 0.61904E+00_JPRB, 0.62626E+00_JPRB, 0.63665E+00_JPRB ,&
&  0.65374E+00_JPRB, 0.66634E+00_JPRB, 0.66661E+00_JPRB, 0.65637E+00_JPRB, 0.65044E+00_JPRB, 0.65095E+00_JPRB, 0.64217E+00_JPRB ,&
&  0.62768E+00_JPRB, 0.61966E+00_JPRB, 0.61926E+00_JPRB, 0.62001E+00_JPRB, 0.61753E+00_JPRB, 0.61220E+00_JPRB, 0.61160E+00_JPRB ,&
&  0.61299E+00_JPRB, 0.61367E+00_JPRB, 0.61290E+00_JPRB, 0.61550E+00_JPRB, 0.62149E+00_JPRB, 0.62960E+00_JPRB, 0.64086E+00_JPRB ,&
&  0.65732E+00_JPRB, 0.67292E+00_JPRB, 0.68217E+00_JPRB, 0.69703E+00_JPRB, 0.71462E+00_JPRB, 0.72360E+00_JPRB, 0.74002E+00_JPRB ,&
&  0.74333E+00_JPRB, 0.74243E+00_JPRB, 0.74237E+00_JPRB, 0.74841E+00_JPRB, 0.75040E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.78007E+00_JPRB, 0.75794E+00_JPRB, 0.74517E+00_JPRB, 0.73679E+00_JPRB, 0.73027E+00_JPRB, 0.72193E+00_JPRB, 0.70693E+00_JPRB ,&
&  0.69142E+00_JPRB, 0.68319E+00_JPRB, 0.67612E+00_JPRB, 0.66722E+00_JPRB, 0.66781E+00_JPRB, 0.67314E+00_JPRB, 0.68373E+00_JPRB ,&
&  0.69562E+00_JPRB, 0.71103E+00_JPRB, 0.72595E+00_JPRB, 0.74497E+00_JPRB, 0.75772E+00_JPRB, 0.76675E+00_JPRB, 0.77172E+00_JPRB ,&
&  0.77516E+00_JPRB, 0.77575E+00_JPRB, 0.76874E+00_JPRB, 0.75972E+00_JPRB, 0.75730E+00_JPRB, 0.75810E+00_JPRB, 0.75293E+00_JPRB ,&
&  0.65556E+00_JPRB, 0.57168E+00_JPRB, 0.62600E+00_JPRB, 0.69428E+00_JPRB, 0.76648E+00_JPRB, 0.82408E+00_JPRB, 0.85141E+00_JPRB ,&
&  0.83576E+00_JPRB, 0.80140E+00_JPRB, 0.77197E+00_JPRB, 0.73963E+00_JPRB, 0.72570E+00_JPRB, 0.71816E+00_JPRB, 0.70677E+00_JPRB ,&
&  0.69443E+00_JPRB, 0.68094E+00_JPRB, 0.66537E+00_JPRB, 0.64236E+00_JPRB, 0.62596E+00_JPRB, 0.61389E+00_JPRB, 0.61168E+00_JPRB ,&
&  0.61364E+00_JPRB, 0.61707E+00_JPRB, 0.62005E+00_JPRB, 0.62222E+00_JPRB, 0.62875E+00_JPRB, 0.63778E+00_JPRB, 0.64809E+00_JPRB ,&
&  0.66133E+00_JPRB, 0.67194E+00_JPRB, 0.68795E+00_JPRB, 0.69863E+00_JPRB, 0.70789E+00_JPRB, 0.71531E+00_JPRB, 0.72291E+00_JPRB ,&
&  0.72361E+00_JPRB, 0.71923E+00_JPRB, 0.70265E+00_JPRB, 0.69394E+00_JPRB, 0.69396E+00_JPRB, 0.69476E+00_JPRB, 0.69072E+00_JPRB ,&
&  0.68026E+00_JPRB, 0.66771E+00_JPRB, 0.65525E+00_JPRB, 0.64936E+00_JPRB, 0.65092E+00_JPRB, 0.65042E+00_JPRB, 0.64642E+00_JPRB ,&
&  0.64310E+00_JPRB, 0.64323E+00_JPRB, 0.64904E+00_JPRB, 0.66024E+00_JPRB, 0.67524E+00_JPRB, 0.69391E+00_JPRB, 0.70954E+00_JPRB ,&
&  0.72602E+00_JPRB, 0.74286E+00_JPRB, 0.75497E+00_JPRB, 0.78039E+00_JPRB, 0.80447E+00_JPRB, 0.81848E+00_JPRB, 0.82030E+00_JPRB ,&
&  0.82490E+00_JPRB, 0.81805E+00_JPRB, 0.81166E+00_JPRB, 0.80841E+00_JPRB, 0.79812E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.77026E+00_JPRB, 0.77300E+00_JPRB, 0.77525E+00_JPRB, 0.76933E+00_JPRB, 0.76334E+00_JPRB, 0.75264E+00_JPRB, 0.73317E+00_JPRB ,&
&  0.71524E+00_JPRB, 0.70756E+00_JPRB, 0.70621E+00_JPRB, 0.70666E+00_JPRB, 0.70882E+00_JPRB, 0.71464E+00_JPRB, 0.72586E+00_JPRB ,&
&  0.73951E+00_JPRB, 0.75404E+00_JPRB, 0.77118E+00_JPRB, 0.78709E+00_JPRB, 0.79088E+00_JPRB, 0.78795E+00_JPRB, 0.78847E+00_JPRB ,&
&  0.79486E+00_JPRB, 0.79402E+00_JPRB, 0.78793E+00_JPRB, 0.79070E+00_JPRB, 0.79872E+00_JPRB, 0.78973E+00_JPRB, 0.77148E+00_JPRB ,&
&  0.75936E+00_JPRB, 0.61770E+00_JPRB, 0.66016E+00_JPRB, 0.73981E+00_JPRB, 0.83020E+00_JPRB, 0.91221E+00_JPRB, 0.94280E+00_JPRB ,&
&  0.92500E+00_JPRB, 0.88364E+00_JPRB, 0.85069E+00_JPRB, 0.81119E+00_JPRB, 0.78985E+00_JPRB, 0.77310E+00_JPRB, 0.75379E+00_JPRB ,&
&  0.72844E+00_JPRB, 0.70334E+00_JPRB, 0.67689E+00_JPRB, 0.65436E+00_JPRB, 0.63646E+00_JPRB, 0.62845E+00_JPRB, 0.62633E+00_JPRB ,&
&  0.62686E+00_JPRB, 0.63769E+00_JPRB, 0.64353E+00_JPRB, 0.65281E+00_JPRB, 0.66771E+00_JPRB, 0.68867E+00_JPRB, 0.71090E+00_JPRB ,&
&  0.73143E+00_JPRB, 0.74677E+00_JPRB, 0.76233E+00_JPRB, 0.77531E+00_JPRB, 0.78977E+00_JPRB, 0.80127E+00_JPRB, 0.79638E+00_JPRB ,&
&  0.77304E+00_JPRB, 0.74536E+00_JPRB, 0.74241E+00_JPRB, 0.74622E+00_JPRB, 0.74254E+00_JPRB, 0.74065E+00_JPRB, 0.73708E+00_JPRB ,&
&  0.72696E+00_JPRB, 0.71434E+00_JPRB, 0.70671E+00_JPRB, 0.69731E+00_JPRB, 0.69165E+00_JPRB, 0.69076E+00_JPRB, 0.67916E+00_JPRB ,&
&  0.67144E+00_JPRB, 0.67159E+00_JPRB, 0.68036E+00_JPRB, 0.70064E+00_JPRB, 0.72801E+00_JPRB, 0.75421E+00_JPRB, 0.77190E+00_JPRB ,&
&  0.78267E+00_JPRB, 0.79412E+00_JPRB, 0.80101E+00_JPRB, 0.84016E+00_JPRB, 0.85851E+00_JPRB, 0.87176E+00_JPRB, 0.88094E+00_JPRB ,&
&  0.88463E+00_JPRB, 0.88539E+00_JPRB, 0.67108E+00_JPRB, 0.74086E+00_JPRB, 0.75453E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.80156E+00_JPRB, 0.82212E+00_JPRB, 0.82704E+00_JPRB, 0.82420E+00_JPRB, 0.81734E+00_JPRB, 0.79947E+00_JPRB, 0.77917E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.74731E+00_JPRB, 0.74295E+00_JPRB, 0.74643E+00_JPRB, 0.75209E+00_JPRB, 0.76103E+00_JPRB, 0.77381E+00_JPRB ,&
&  0.78597E+00_JPRB, 0.79222E+00_JPRB, 0.79734E+00_JPRB, 0.79592E+00_JPRB, 0.79447E+00_JPRB, 0.79702E+00_JPRB, 0.80152E+00_JPRB ,&
&  0.81067E+00_JPRB, 0.81411E+00_JPRB, 0.81160E+00_JPRB, 0.82032E+00_JPRB, 0.83291E+00_JPRB, 0.82246E+00_JPRB, 0.80277E+00_JPRB ,&
&  0.78870E+00_JPRB, 0.66896E+00_JPRB, 0.71957E+00_JPRB, 0.85940E+00_JPRB, 0.97466E+00_JPRB, 0.10748E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10564E+01_JPRB, 0.99992E+00_JPRB, 0.94525E+00_JPRB, 0.89410E+00_JPRB, 0.85687E+00_JPRB, 0.81951E+00_JPRB, 0.79350E+00_JPRB ,&
&  0.77560E+00_JPRB, 0.75779E+00_JPRB, 0.73351E+00_JPRB, 0.71447E+00_JPRB, 0.69929E+00_JPRB, 0.68462E+00_JPRB, 0.67667E+00_JPRB ,&
&  0.67841E+00_JPRB, 0.68984E+00_JPRB, 0.70263E+00_JPRB, 0.71791E+00_JPRB, 0.74625E+00_JPRB, 0.77396E+00_JPRB, 0.80165E+00_JPRB ,&
&  0.81613E+00_JPRB, 0.83035E+00_JPRB, 0.85212E+00_JPRB, 0.87449E+00_JPRB, 0.90010E+00_JPRB, 0.90968E+00_JPRB, 0.88878E+00_JPRB ,&
&  0.84071E+00_JPRB, 0.80803E+00_JPRB, 0.78627E+00_JPRB, 0.77240E+00_JPRB, 0.77245E+00_JPRB, 0.78221E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.78183E+00_JPRB, 0.77203E+00_JPRB, 0.75915E+00_JPRB, 0.74691E+00_JPRB, 0.74492E+00_JPRB, 0.74611E+00_JPRB, 0.74202E+00_JPRB ,&
&  0.73754E+00_JPRB, 0.74053E+00_JPRB, 0.75143E+00_JPRB, 0.77296E+00_JPRB, 0.79963E+00_JPRB, 0.82931E+00_JPRB, 0.84782E+00_JPRB ,&
&  0.86034E+00_JPRB, 0.87297E+00_JPRB, 0.88817E+00_JPRB, 0.88729E+00_JPRB, 0.93873E+00_JPRB, 0.95886E+00_JPRB, 0.96648E+00_JPRB ,&
&  0.95001E+00_JPRB, 0.91990E+00_JPRB, 0.88237E+00_JPRB, 0.72048E+00_JPRB, 0.77544E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.85808E+00_JPRB, 0.88107E+00_JPRB, 0.88364E+00_JPRB, 0.87803E+00_JPRB, 0.86004E+00_JPRB, 0.83570E+00_JPRB, 0.81709E+00_JPRB ,&
&  0.80772E+00_JPRB, 0.79095E+00_JPRB, 0.78344E+00_JPRB, 0.78104E+00_JPRB, 0.78507E+00_JPRB, 0.79297E+00_JPRB, 0.79412E+00_JPRB ,&
&  0.78602E+00_JPRB, 0.77298E+00_JPRB, 0.76403E+00_JPRB, 0.76367E+00_JPRB, 0.77342E+00_JPRB, 0.78393E+00_JPRB, 0.79093E+00_JPRB ,&
&  0.79403E+00_JPRB, 0.78813E+00_JPRB, 0.78571E+00_JPRB, 0.79545E+00_JPRB, 0.80313E+00_JPRB, 0.79515E+00_JPRB, 0.77476E+00_JPRB ,&
&  0.76943E+00_JPRB, 0.74407E+00_JPRB, 0.90877E+00_JPRB, 0.10648E+01_JPRB, 0.11918E+01_JPRB, 0.12304E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.10623E+01_JPRB, 0.98420E+00_JPRB, 0.92304E+00_JPRB, 0.87969E+00_JPRB, 0.84867E+00_JPRB, 0.82725E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.77232E+00_JPRB, 0.74882E+00_JPRB, 0.73511E+00_JPRB, 0.72952E+00_JPRB, 0.72518E+00_JPRB, 0.72190E+00_JPRB ,&
&  0.71873E+00_JPRB, 0.71596E+00_JPRB, 0.71985E+00_JPRB, 0.73486E+00_JPRB, 0.75686E+00_JPRB, 0.79714E+00_JPRB, 0.84817E+00_JPRB ,&
&  0.87237E+00_JPRB, 0.90697E+00_JPRB, 0.95112E+00_JPRB, 0.99571E+00_JPRB, 0.10071E+01_JPRB, 0.99757E+00_JPRB, 0.96977E+00_JPRB ,&
&  0.93287E+00_JPRB, 0.87848E+00_JPRB, 0.83311E+00_JPRB, 0.80247E+00_JPRB, 0.79009E+00_JPRB, 0.78463E+00_JPRB, 0.79454E+00_JPRB ,&
&  0.80196E+00_JPRB, 0.80692E+00_JPRB, 0.79834E+00_JPRB, 0.78835E+00_JPRB, 0.78503E+00_JPRB, 0.77193E+00_JPRB, 0.76960E+00_JPRB ,&
&  0.77306E+00_JPRB, 0.78826E+00_JPRB, 0.81399E+00_JPRB, 0.83842E+00_JPRB, 0.86979E+00_JPRB, 0.89234E+00_JPRB, 0.90120E+00_JPRB ,&
&  0.90432E+00_JPRB, 0.90603E+00_JPRB, 0.90696E+00_JPRB, 0.97633E+00_JPRB, 0.99724E+00_JPRB, 0.10110E+01_JPRB, 0.10027E+01_JPRB ,&
&  0.98235E+00_JPRB, 0.93745E+00_JPRB, 0.88292E+00_JPRB, 0.85970E+00_JPRB, 0.79888E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.99818E+00_JPRB, 0.98256E+00_JPRB, 0.95285E+00_JPRB, 0.92754E+00_JPRB, 0.89949E+00_JPRB, 0.87912E+00_JPRB, 0.86253E+00_JPRB ,&
&  0.85305E+00_JPRB, 0.83528E+00_JPRB, 0.82602E+00_JPRB, 0.81811E+00_JPRB, 0.81496E+00_JPRB, 0.80707E+00_JPRB, 0.79493E+00_JPRB ,&
&  0.78150E+00_JPRB, 0.77340E+00_JPRB, 0.77083E+00_JPRB, 0.77497E+00_JPRB, 0.78451E+00_JPRB, 0.79037E+00_JPRB, 0.78487E+00_JPRB ,&
&  0.77620E+00_JPRB, 0.76261E+00_JPRB, 0.74648E+00_JPRB, 0.76054E+00_JPRB, 0.77619E+00_JPRB, 0.78040E+00_JPRB, 0.76531E+00_JPRB ,&
&  0.77886E+00_JPRB, 0.86585E+00_JPRB, 0.12562E+01_JPRB, 0.14309E+01_JPRB, 0.14633E+01_JPRB, 0.14606E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.11945E+01_JPRB, 0.10715E+01_JPRB, 0.97887E+00_JPRB, 0.93374E+00_JPRB, 0.90749E+00_JPRB, 0.86934E+00_JPRB, 0.83759E+00_JPRB ,&
&  0.79462E+00_JPRB, 0.77272E+00_JPRB, 0.75797E+00_JPRB, 0.75107E+00_JPRB, 0.74694E+00_JPRB, 0.74693E+00_JPRB, 0.75194E+00_JPRB ,&
&  0.75733E+00_JPRB, 0.75916E+00_JPRB, 0.76512E+00_JPRB, 0.78668E+00_JPRB, 0.80988E+00_JPRB, 0.88246E+00_JPRB, 0.96277E+00_JPRB ,&
&  0.10117E+01_JPRB, 0.10531E+01_JPRB, 0.11025E+01_JPRB, 0.11208E+01_JPRB, 0.10915E+01_JPRB, 0.10450E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.97377E+00_JPRB, 0.93346E+00_JPRB, 0.89206E+00_JPRB, 0.86237E+00_JPRB, 0.83474E+00_JPRB, 0.82357E+00_JPRB, 0.83797E+00_JPRB ,&
&  0.84578E+00_JPRB, 0.85347E+00_JPRB, 0.84532E+00_JPRB, 0.83161E+00_JPRB, 0.80628E+00_JPRB, 0.78809E+00_JPRB, 0.78788E+00_JPRB ,&
&  0.79997E+00_JPRB, 0.82514E+00_JPRB, 0.86070E+00_JPRB, 0.89834E+00_JPRB, 0.92943E+00_JPRB, 0.94605E+00_JPRB, 0.95059E+00_JPRB ,&
&  0.95782E+00_JPRB, 0.96964E+00_JPRB, 0.92554E+00_JPRB, 0.10061E+01_JPRB, 0.11322E+01_JPRB, 0.11216E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.97210E+00_JPRB, 0.91788E+00_JPRB, 0.89526E+00_JPRB, 0.95849E+00_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.95692E+00_JPRB, 0.94077E+00_JPRB, 0.91493E+00_JPRB, 0.89650E+00_JPRB, 0.89147E+00_JPRB, 0.87867E+00_JPRB, 0.86887E+00_JPRB ,&
&  0.86324E+00_JPRB, 0.85103E+00_JPRB, 0.84056E+00_JPRB, 0.82542E+00_JPRB, 0.80711E+00_JPRB, 0.79281E+00_JPRB, 0.78855E+00_JPRB ,&
&  0.78909E+00_JPRB, 0.77983E+00_JPRB, 0.78397E+00_JPRB, 0.78558E+00_JPRB, 0.78214E+00_JPRB, 0.76885E+00_JPRB, 0.75567E+00_JPRB ,&
&  0.74908E+00_JPRB, 0.73688E+00_JPRB, 0.73480E+00_JPRB, 0.75540E+00_JPRB, 0.77358E+00_JPRB, 0.77525E+00_JPRB, 0.75914E+00_JPRB ,&
&  0.80464E+00_JPRB, 0.10356E+01_JPRB, 0.16726E+01_JPRB, 0.19286E+01_JPRB, 0.18667E+01_JPRB, 0.15802E+01_JPRB, 0.13302E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.10476E+01_JPRB, 0.99135E+00_JPRB, 0.94260E+00_JPRB, 0.89322E+00_JPRB, 0.85421E+00_JPRB, 0.82307E+00_JPRB ,&
&  0.78735E+00_JPRB, 0.77190E+00_JPRB, 0.76347E+00_JPRB, 0.76258E+00_JPRB, 0.76142E+00_JPRB, 0.75924E+00_JPRB, 0.76135E+00_JPRB ,&
&  0.76612E+00_JPRB, 0.77243E+00_JPRB, 0.78795E+00_JPRB, 0.80208E+00_JPRB, 0.86096E+00_JPRB, 0.99478E+00_JPRB, 0.11625E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.12323E+01_JPRB, 0.11955E+01_JPRB, 0.11484E+01_JPRB, 0.11067E+01_JPRB, 0.10645E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.99170E+00_JPRB, 0.95952E+00_JPRB, 0.93543E+00_JPRB, 0.91480E+00_JPRB, 0.88746E+00_JPRB, 0.88472E+00_JPRB, 0.89759E+00_JPRB ,&
&  0.91375E+00_JPRB, 0.91699E+00_JPRB, 0.89472E+00_JPRB, 0.87006E+00_JPRB, 0.83942E+00_JPRB, 0.83159E+00_JPRB, 0.83198E+00_JPRB ,&
&  0.83644E+00_JPRB, 0.85601E+00_JPRB, 0.89791E+00_JPRB, 0.93917E+00_JPRB, 0.96827E+00_JPRB, 0.98129E+00_JPRB, 0.98056E+00_JPRB ,&
&  0.98935E+00_JPRB, 0.10189E+01_JPRB, 0.99866E+00_JPRB, 0.10534E+01_JPRB, 0.11113E+01_JPRB, 0.11771E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.10078E+01_JPRB, 0.93780E+00_JPRB, 0.90296E+00_JPRB, 0.89168E+00_JPRB, 0.94673E+00_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.92509E+00_JPRB, 0.90334E+00_JPRB, 0.88615E+00_JPRB, 0.87066E+00_JPRB, 0.85458E+00_JPRB, 0.84611E+00_JPRB, 0.83628E+00_JPRB ,&
&  0.83868E+00_JPRB, 0.83903E+00_JPRB, 0.82795E+00_JPRB, 0.80056E+00_JPRB, 0.77854E+00_JPRB, 0.77507E+00_JPRB, 0.78100E+00_JPRB ,&
&  0.77549E+00_JPRB, 0.76453E+00_JPRB, 0.76762E+00_JPRB, 0.75075E+00_JPRB, 0.72877E+00_JPRB, 0.71501E+00_JPRB, 0.70939E+00_JPRB ,&
&  0.71384E+00_JPRB, 0.72519E+00_JPRB, 0.75253E+00_JPRB, 0.77809E+00_JPRB, 0.78091E+00_JPRB, 0.77313E+00_JPRB, 0.75958E+00_JPRB ,&
&  0.87421E+00_JPRB, 0.11567E+01_JPRB, 0.21585E+01_JPRB, 0.21634E+01_JPRB, 0.18484E+01_JPRB, 0.14935E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.11487E+01_JPRB, 0.10598E+01_JPRB, 0.98942E+00_JPRB, 0.90765E+00_JPRB, 0.84741E+00_JPRB, 0.81637E+00_JPRB, 0.80266E+00_JPRB ,&
&  0.78267E+00_JPRB, 0.76967E+00_JPRB, 0.77192E+00_JPRB, 0.77822E+00_JPRB, 0.78174E+00_JPRB, 0.77883E+00_JPRB, 0.76724E+00_JPRB ,&
&  0.76913E+00_JPRB, 0.77741E+00_JPRB, 0.78916E+00_JPRB, 0.80384E+00_JPRB, 0.88174E+00_JPRB, 0.92248E+00_JPRB, 0.11704E+01_JPRB ,&
&  0.15042E+01_JPRB, 0.14069E+01_JPRB, 0.12734E+01_JPRB, 0.11976E+01_JPRB, 0.11556E+01_JPRB, 0.11071E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10311E+01_JPRB, 0.10036E+01_JPRB, 0.10021E+01_JPRB, 0.99220E+00_JPRB, 0.98963E+00_JPRB, 0.96357E+00_JPRB, 0.94258E+00_JPRB ,&
&  0.94300E+00_JPRB, 0.91708E+00_JPRB, 0.88871E+00_JPRB, 0.85985E+00_JPRB, 0.86331E+00_JPRB, 0.87732E+00_JPRB, 0.88294E+00_JPRB ,&
&  0.89079E+00_JPRB, 0.89741E+00_JPRB, 0.93899E+00_JPRB, 0.94173E+00_JPRB, 0.98562E+00_JPRB, 0.10252E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.97973E+00_JPRB, 0.99870E+00_JPRB, 0.10116E+01_JPRB, 0.10122E+01_JPRB, 0.10680E+01_JPRB, 0.11462E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.92892E+00_JPRB, 0.87952E+00_JPRB, 0.85369E+00_JPRB, 0.86561E+00_JPRB, 0.92942E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.92983E+00_JPRB, 0.90787E+00_JPRB, 0.88579E+00_JPRB, 0.85292E+00_JPRB, 0.84402E+00_JPRB, 0.83212E+00_JPRB, 0.82606E+00_JPRB ,&
&  0.82444E+00_JPRB, 0.81126E+00_JPRB, 0.79642E+00_JPRB, 0.77560E+00_JPRB, 0.76741E+00_JPRB, 0.77793E+00_JPRB, 0.77252E+00_JPRB ,&
&  0.75460E+00_JPRB, 0.73064E+00_JPRB, 0.72273E+00_JPRB, 0.71386E+00_JPRB, 0.70463E+00_JPRB, 0.70570E+00_JPRB, 0.71521E+00_JPRB ,&
&  0.71948E+00_JPRB, 0.73796E+00_JPRB, 0.76828E+00_JPRB, 0.79584E+00_JPRB, 0.79425E+00_JPRB, 0.78793E+00_JPRB, 0.81696E+00_JPRB ,&
&  0.93946E+00_JPRB, 0.92600E+00_JPRB, 0.19582E+01_JPRB, 0.23081E+01_JPRB, 0.18720E+01_JPRB, 0.15376E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.11371E+01_JPRB, 0.10350E+01_JPRB, 0.96449E+00_JPRB, 0.89321E+00_JPRB, 0.85766E+00_JPRB, 0.81129E+00_JPRB, 0.79509E+00_JPRB ,&
&  0.79326E+00_JPRB, 0.78177E+00_JPRB, 0.78167E+00_JPRB, 0.79855E+00_JPRB, 0.80383E+00_JPRB, 0.80860E+00_JPRB, 0.79640E+00_JPRB ,&
&  0.79246E+00_JPRB, 0.79231E+00_JPRB, 0.81367E+00_JPRB, 0.87376E+00_JPRB, 0.97544E+00_JPRB, 0.10807E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.14560E+01_JPRB, 0.12824E+01_JPRB, 0.12031E+01_JPRB, 0.11500E+01_JPRB, 0.10914E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.10187E+01_JPRB, 0.10099E+01_JPRB, 0.10407E+01_JPRB, 0.10137E+01_JPRB, 0.10043E+01_JPRB, 0.97868E+00_JPRB ,&
&  0.94566E+00_JPRB, 0.91339E+00_JPRB, 0.88343E+00_JPRB, 0.86973E+00_JPRB, 0.89010E+00_JPRB, 0.90148E+00_JPRB, 0.89742E+00_JPRB ,&
&  0.89788E+00_JPRB, 0.92522E+00_JPRB, 0.96113E+00_JPRB, 0.96741E+00_JPRB, 0.99861E+00_JPRB, 0.10085E+01_JPRB, 0.10141E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10271E+01_JPRB, 0.98188E+00_JPRB, 0.99358E+00_JPRB, 0.97040E+00_JPRB, 0.10182E+01_JPRB, 0.92348E+00_JPRB ,&
&  0.86731E+00_JPRB, 0.83289E+00_JPRB, 0.84887E+00_JPRB, 0.90337E+00_JPRB, 0.93874E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.91311E+00_JPRB, 0.90972E+00_JPRB, 0.86451E+00_JPRB, 0.85731E+00_JPRB, 0.84125E+00_JPRB, 0.84187E+00_JPRB, 0.85547E+00_JPRB ,&
&  0.84795E+00_JPRB, 0.82259E+00_JPRB, 0.79034E+00_JPRB, 0.77607E+00_JPRB, 0.78660E+00_JPRB, 0.79831E+00_JPRB, 0.76571E+00_JPRB ,&
&  0.72758E+00_JPRB, 0.71510E+00_JPRB, 0.70899E+00_JPRB, 0.71256E+00_JPRB, 0.72327E+00_JPRB, 0.74024E+00_JPRB, 0.74930E+00_JPRB ,&
&  0.74467E+00_JPRB, 0.78273E+00_JPRB, 0.82520E+00_JPRB, 0.84082E+00_JPRB, 0.82555E+00_JPRB, 0.81853E+00_JPRB, 0.83925E+00_JPRB ,&
&  0.93342E+00_JPRB, 0.20973E+01_JPRB, 0.14331E+01_JPRB, 0.23859E+01_JPRB, 0.18062E+01_JPRB, 0.14561E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10385E+01_JPRB, 0.99052E+00_JPRB, 0.91749E+00_JPRB, 0.87093E+00_JPRB, 0.82595E+00_JPRB, 0.78537E+00_JPRB ,&
&  0.76705E+00_JPRB, 0.77060E+00_JPRB, 0.77871E+00_JPRB, 0.82158E+00_JPRB, 0.85425E+00_JPRB, 0.86084E+00_JPRB, 0.84862E+00_JPRB ,&
&  0.83967E+00_JPRB, 0.83818E+00_JPRB, 0.88418E+00_JPRB, 0.10498E+01_JPRB, 0.11894E+01_JPRB, 0.12226E+01_JPRB, 0.12978E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.14056E+01_JPRB, 0.12594E+01_JPRB, 0.11798E+01_JPRB, 0.11160E+01_JPRB, 0.10612E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.99633E+00_JPRB, 0.99017E+00_JPRB, 0.98252E+00_JPRB, 0.99392E+00_JPRB, 0.10052E+01_JPRB, 0.97238E+00_JPRB ,&
&  0.93112E+00_JPRB, 0.89667E+00_JPRB, 0.89904E+00_JPRB, 0.91193E+00_JPRB, 0.92990E+00_JPRB, 0.91885E+00_JPRB, 0.90974E+00_JPRB ,&
&  0.92268E+00_JPRB, 0.97443E+00_JPRB, 0.99366E+00_JPRB, 0.10090E+01_JPRB, 0.10062E+01_JPRB, 0.98653E+00_JPRB, 0.96868E+00_JPRB ,&
&  0.10109E+01_JPRB, 0.10067E+01_JPRB, 0.94463E+00_JPRB, 0.93555E+00_JPRB, 0.92669E+00_JPRB, 0.96363E+00_JPRB, 0.85771E+00_JPRB ,&
&  0.83293E+00_JPRB, 0.83256E+00_JPRB, 0.90191E+00_JPRB, 0.91155E+00_JPRB, 0.92350E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.89948E+00_JPRB, 0.88472E+00_JPRB, 0.84985E+00_JPRB, 0.84920E+00_JPRB, 0.82833E+00_JPRB, 0.85692E+00_JPRB, 0.86139E+00_JPRB ,&
&  0.82674E+00_JPRB, 0.79364E+00_JPRB, 0.77355E+00_JPRB, 0.79238E+00_JPRB, 0.81281E+00_JPRB, 0.79886E+00_JPRB, 0.75882E+00_JPRB ,&
&  0.74570E+00_JPRB, 0.74395E+00_JPRB, 0.74463E+00_JPRB, 0.74843E+00_JPRB, 0.76396E+00_JPRB, 0.78642E+00_JPRB, 0.78373E+00_JPRB ,&
&  0.78052E+00_JPRB, 0.81133E+00_JPRB, 0.83947E+00_JPRB, 0.84126E+00_JPRB, 0.83356E+00_JPRB, 0.82751E+00_JPRB, 0.87185E+00_JPRB ,&
&  0.93552E+00_JPRB, 0.17845E+01_JPRB, 0.13401E+01_JPRB, 0.22021E+01_JPRB, 0.17163E+01_JPRB, 0.14098E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.95471E+00_JPRB, 0.10508E+01_JPRB, 0.98758E+00_JPRB, 0.90459E+00_JPRB, 0.83925E+00_JPRB, 0.80315E+00_JPRB, 0.77297E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.78257E+00_JPRB, 0.82575E+00_JPRB, 0.88507E+00_JPRB, 0.93307E+00_JPRB, 0.94482E+00_JPRB, 0.94892E+00_JPRB ,&
&  0.95201E+00_JPRB, 0.96208E+00_JPRB, 0.10939E+01_JPRB, 0.12592E+01_JPRB, 0.14417E+01_JPRB, 0.14181E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.13466E+01_JPRB, 0.12488E+01_JPRB, 0.11675E+01_JPRB, 0.10876E+01_JPRB, 0.10587E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.97560E+00_JPRB, 0.91965E+00_JPRB, 0.91199E+00_JPRB, 0.92294E+00_JPRB, 0.97787E+00_JPRB, 0.98840E+00_JPRB, 0.94417E+00_JPRB ,&
&  0.90104E+00_JPRB, 0.88625E+00_JPRB, 0.88971E+00_JPRB, 0.90742E+00_JPRB, 0.92639E+00_JPRB, 0.92594E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.96526E+00_JPRB, 0.10305E+01_JPRB, 0.10329E+01_JPRB, 0.10357E+01_JPRB, 0.99668E+00_JPRB, 0.96563E+00_JPRB, 0.94225E+00_JPRB ,&
&  0.96652E+00_JPRB, 0.95396E+00_JPRB, 0.91543E+00_JPRB, 0.87889E+00_JPRB, 0.87063E+00_JPRB, 0.87343E+00_JPRB, 0.83216E+00_JPRB ,&
&  0.83700E+00_JPRB, 0.88261E+00_JPRB, 0.93665E+00_JPRB, 0.90255E+00_JPRB, 0.89768E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.85864E+00_JPRB, 0.82524E+00_JPRB, 0.81090E+00_JPRB, 0.79671E+00_JPRB, 0.78285E+00_JPRB, 0.81402E+00_JPRB, 0.80396E+00_JPRB ,&
&  0.77925E+00_JPRB, 0.75240E+00_JPRB, 0.77038E+00_JPRB, 0.81469E+00_JPRB, 0.81504E+00_JPRB, 0.79373E+00_JPRB, 0.77869E+00_JPRB ,&
&  0.77742E+00_JPRB, 0.79515E+00_JPRB, 0.79919E+00_JPRB, 0.81068E+00_JPRB, 0.81069E+00_JPRB, 0.79997E+00_JPRB, 0.79331E+00_JPRB ,&
&  0.80962E+00_JPRB, 0.82058E+00_JPRB, 0.83288E+00_JPRB, 0.84251E+00_JPRB, 0.86424E+00_JPRB, 0.87942E+00_JPRB, 0.98837E+00_JPRB ,&
&  0.11413E+01_JPRB, 0.17077E+01_JPRB, 0.13948E+01_JPRB, 0.16870E+01_JPRB, 0.15302E+01_JPRB, 0.12811E+01_JPRB, 0.98825E+00_JPRB ,&
&  0.89925E+00_JPRB, 0.85364E+00_JPRB, 0.98281E+00_JPRB, 0.91183E+00_JPRB, 0.86503E+00_JPRB, 0.88082E+00_JPRB, 0.88632E+00_JPRB ,&
&  0.90336E+00_JPRB, 0.91877E+00_JPRB, 0.10211E+01_JPRB, 0.10749E+01_JPRB, 0.11042E+01_JPRB, 0.11146E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.12227E+01_JPRB, 0.14660E+01_JPRB, 0.16359E+01_JPRB, 0.15852E+01_JPRB, 0.14636E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.12554E+01_JPRB, 0.12773E+01_JPRB, 0.12377E+01_JPRB, 0.11696E+01_JPRB, 0.10381E+01_JPRB, 0.10606E+01_JPRB, 0.97973E+00_JPRB ,&
&  0.90342E+00_JPRB, 0.85568E+00_JPRB, 0.85705E+00_JPRB, 0.88734E+00_JPRB, 0.92925E+00_JPRB, 0.92822E+00_JPRB, 0.90515E+00_JPRB ,&
&  0.89393E+00_JPRB, 0.90240E+00_JPRB, 0.89064E+00_JPRB, 0.90822E+00_JPRB, 0.92486E+00_JPRB, 0.95972E+00_JPRB, 0.99321E+00_JPRB ,&
&  0.10420E+01_JPRB, 0.10875E+01_JPRB, 0.10639E+01_JPRB, 0.10316E+01_JPRB, 0.98022E+00_JPRB, 0.94442E+00_JPRB, 0.90050E+00_JPRB ,&
&  0.90476E+00_JPRB, 0.93109E+00_JPRB, 0.91654E+00_JPRB, 0.88284E+00_JPRB, 0.85785E+00_JPRB, 0.84498E+00_JPRB, 0.85756E+00_JPRB ,&
&  0.88538E+00_JPRB, 0.94294E+00_JPRB, 0.94516E+00_JPRB, 0.89406E+00_JPRB, 0.88171E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.78436E+00_JPRB, 0.77395E+00_JPRB, 0.75016E+00_JPRB, 0.73426E+00_JPRB, 0.75697E+00_JPRB, 0.79059E+00_JPRB, 0.78953E+00_JPRB ,&
&  0.76558E+00_JPRB, 0.75267E+00_JPRB, 0.78796E+00_JPRB, 0.82277E+00_JPRB, 0.81941E+00_JPRB, 0.83487E+00_JPRB, 0.84286E+00_JPRB ,&
&  0.87367E+00_JPRB, 0.86728E+00_JPRB, 0.84760E+00_JPRB, 0.82021E+00_JPRB, 0.81769E+00_JPRB, 0.80511E+00_JPRB, 0.80448E+00_JPRB ,&
&  0.81402E+00_JPRB, 0.82727E+00_JPRB, 0.85495E+00_JPRB, 0.90046E+00_JPRB, 0.95412E+00_JPRB, 0.10718E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.16123E+01_JPRB, 0.19889E+01_JPRB, 0.75337E+00_JPRB, 0.96437E+00_JPRB, 0.11048E+01_JPRB, 0.97883E+00_JPRB, 0.83054E+00_JPRB ,&
&  0.74527E+00_JPRB, 0.69670E+00_JPRB, 0.79160E+00_JPRB, 0.93020E+00_JPRB, 0.96871E+00_JPRB, 0.10289E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.11616E+01_JPRB, 0.12436E+01_JPRB, 0.12833E+01_JPRB, 0.13014E+01_JPRB, 0.13068E+01_JPRB, 0.13276E+01_JPRB ,&
&  0.14407E+01_JPRB, 0.16259E+01_JPRB, 0.19033E+01_JPRB, 0.17923E+01_JPRB, 0.17216E+01_JPRB, 0.14784E+01_JPRB, 0.12575E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.11996E+01_JPRB, 0.12297E+01_JPRB, 0.11528E+01_JPRB, 0.10126E+01_JPRB, 0.94016E+00_JPRB, 0.99084E+00_JPRB ,&
&  0.89777E+00_JPRB, 0.84022E+00_JPRB, 0.82272E+00_JPRB, 0.83309E+00_JPRB, 0.88802E+00_JPRB, 0.88573E+00_JPRB, 0.87091E+00_JPRB ,&
&  0.88547E+00_JPRB, 0.87982E+00_JPRB, 0.89953E+00_JPRB, 0.93218E+00_JPRB, 0.96745E+00_JPRB, 0.10287E+01_JPRB, 0.10625E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10927E+01_JPRB, 0.10970E+01_JPRB, 0.10565E+01_JPRB, 0.99509E+00_JPRB, 0.95225E+00_JPRB, 0.93689E+00_JPRB ,&
&  0.95403E+00_JPRB, 0.96592E+00_JPRB, 0.88111E+00_JPRB, 0.83380E+00_JPRB, 0.86113E+00_JPRB, 0.87587E+00_JPRB, 0.97593E+00_JPRB ,&
&  0.10857E+01_JPRB, 0.11602E+01_JPRB, 0.11178E+01_JPRB, 0.84830E+00_JPRB, 0.83737E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.73759E+00_JPRB, 0.69253E+00_JPRB, 0.66066E+00_JPRB, 0.67147E+00_JPRB, 0.72153E+00_JPRB, 0.77285E+00_JPRB, 0.77149E+00_JPRB ,&
&  0.75711E+00_JPRB, 0.77668E+00_JPRB, 0.84794E+00_JPRB, 0.90463E+00_JPRB, 0.91264E+00_JPRB, 0.92207E+00_JPRB, 0.93844E+00_JPRB ,&
&  0.88511E+00_JPRB, 0.85247E+00_JPRB, 0.83646E+00_JPRB, 0.81804E+00_JPRB, 0.81879E+00_JPRB, 0.81417E+00_JPRB, 0.81728E+00_JPRB ,&
&  0.83453E+00_JPRB, 0.85475E+00_JPRB, 0.90733E+00_JPRB, 0.99038E+00_JPRB, 0.11295E+01_JPRB, 0.14157E+01_JPRB, 0.18459E+01_JPRB ,&
&  0.25611E+01_JPRB, 0.16960E+01_JPRB, 0.65526E+00_JPRB, 0.82313E+00_JPRB, 0.84976E+00_JPRB, 0.82295E+00_JPRB, 0.72717E+00_JPRB ,&
&  0.70765E+00_JPRB, 0.73391E+00_JPRB, 0.83216E+00_JPRB, 0.10095E+01_JPRB, 0.10914E+01_JPRB, 0.11724E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.13442E+01_JPRB, 0.13957E+01_JPRB, 0.14439E+01_JPRB, 0.14910E+01_JPRB, 0.15519E+01_JPRB, 0.16485E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.20425E+01_JPRB, 0.22454E+01_JPRB, 0.20257E+01_JPRB, 0.19763E+01_JPRB, 0.15614E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.11319E+01_JPRB, 0.11403E+01_JPRB, 0.11756E+01_JPRB, 0.10858E+01_JPRB, 0.95717E+00_JPRB, 0.87010E+00_JPRB, 0.96790E+00_JPRB ,&
&  0.86794E+00_JPRB, 0.80935E+00_JPRB, 0.77489E+00_JPRB, 0.78934E+00_JPRB, 0.82263E+00_JPRB, 0.83461E+00_JPRB, 0.83879E+00_JPRB ,&
&  0.85506E+00_JPRB, 0.86039E+00_JPRB, 0.90044E+00_JPRB, 0.92703E+00_JPRB, 0.95034E+00_JPRB, 0.99499E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.10026E+01_JPRB, 0.10183E+01_JPRB, 0.10377E+01_JPRB, 0.98568E+00_JPRB, 0.90395E+00_JPRB, 0.88999E+00_JPRB ,&
&  0.88032E+00_JPRB, 0.86655E+00_JPRB, 0.82258E+00_JPRB, 0.83116E+00_JPRB, 0.83796E+00_JPRB, 0.90475E+00_JPRB, 0.98160E+00_JPRB ,&
&  0.10154E+01_JPRB, 0.98989E+00_JPRB, 0.92337E+00_JPRB, 0.79900E+00_JPRB, 0.78538E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.68127E+00_JPRB, 0.65363E+00_JPRB, 0.61420E+00_JPRB, 0.61094E+00_JPRB, 0.66873E+00_JPRB, 0.71611E+00_JPRB, 0.71826E+00_JPRB ,&
&  0.72569E+00_JPRB, 0.78953E+00_JPRB, 0.90349E+00_JPRB, 0.99346E+00_JPRB, 0.10221E+01_JPRB, 0.10130E+01_JPRB, 0.91553E+00_JPRB ,&
&  0.87704E+00_JPRB, 0.85342E+00_JPRB, 0.82746E+00_JPRB, 0.81633E+00_JPRB, 0.81365E+00_JPRB, 0.81997E+00_JPRB, 0.84087E+00_JPRB ,&
&  0.87234E+00_JPRB, 0.92779E+00_JPRB, 0.10235E+01_JPRB, 0.11582E+01_JPRB, 0.13554E+01_JPRB, 0.15785E+01_JPRB, 0.17672E+01_JPRB ,&
&  0.13155E+01_JPRB, 0.57973E+00_JPRB, 0.64354E+00_JPRB, 0.82024E+00_JPRB, 0.75414E+00_JPRB, 0.74200E+00_JPRB, 0.70832E+00_JPRB ,&
&  0.72654E+00_JPRB, 0.79897E+00_JPRB, 0.88095E+00_JPRB, 0.10584E+01_JPRB, 0.11761E+01_JPRB, 0.12898E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.15029E+01_JPRB, 0.15929E+01_JPRB, 0.16758E+01_JPRB, 0.17657E+01_JPRB, 0.18623E+01_JPRB, 0.20207E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.24128E+01_JPRB, 0.25969E+01_JPRB, 0.23242E+01_JPRB, 0.23901E+01_JPRB, 0.19634E+01_JPRB, 0.15725E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.10078E+01_JPRB, 0.97096E+00_JPRB, 0.10600E+01_JPRB, 0.97411E+00_JPRB, 0.81310E+00_JPRB, 0.86111E+00_JPRB ,&
&  0.80469E+00_JPRB, 0.78584E+00_JPRB, 0.77644E+00_JPRB, 0.72335E+00_JPRB, 0.78221E+00_JPRB, 0.80872E+00_JPRB, 0.82749E+00_JPRB ,&
&  0.81345E+00_JPRB, 0.83115E+00_JPRB, 0.85429E+00_JPRB, 0.89402E+00_JPRB, 0.88799E+00_JPRB, 0.89032E+00_JPRB, 0.87569E+00_JPRB ,&
&  0.87958E+00_JPRB, 0.88943E+00_JPRB, 0.91137E+00_JPRB, 0.93955E+00_JPRB, 0.94427E+00_JPRB, 0.87063E+00_JPRB, 0.81498E+00_JPRB ,&
&  0.78618E+00_JPRB, 0.77206E+00_JPRB, 0.73951E+00_JPRB, 0.78432E+00_JPRB, 0.80071E+00_JPRB, 0.81859E+00_JPRB, 0.85049E+00_JPRB ,&
&  0.86499E+00_JPRB, 0.86602E+00_JPRB, 0.81757E+00_JPRB, 0.76222E+00_JPRB, 0.72529E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.57272E+00_JPRB, 0.55956E+00_JPRB, 0.54376E+00_JPRB, 0.56090E+00_JPRB, 0.59461E+00_JPRB, 0.63609E+00_JPRB, 0.65423E+00_JPRB ,&
&  0.67762E+00_JPRB, 0.77643E+00_JPRB, 0.90614E+00_JPRB, 0.10204E+01_JPRB, 0.10386E+01_JPRB, 0.10052E+01_JPRB, 0.90358E+00_JPRB ,&
&  0.87394E+00_JPRB, 0.84640E+00_JPRB, 0.83022E+00_JPRB, 0.81289E+00_JPRB, 0.81463E+00_JPRB, 0.84249E+00_JPRB, 0.89053E+00_JPRB ,&
&  0.95244E+00_JPRB, 0.10194E+01_JPRB, 0.10963E+01_JPRB, 0.11775E+01_JPRB, 0.12447E+01_JPRB, 0.12579E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.49368E+00_JPRB, 0.57203E+00_JPRB, 0.76002E+00_JPRB, 0.73656E+00_JPRB, 0.69356E+00_JPRB, 0.67252E+00_JPRB, 0.70356E+00_JPRB ,&
&  0.74299E+00_JPRB, 0.75714E+00_JPRB, 0.84520E+00_JPRB, 0.10568E+01_JPRB, 0.12036E+01_JPRB, 0.13478E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.15155E+01_JPRB, 0.16152E+01_JPRB, 0.17393E+01_JPRB, 0.18492E+01_JPRB, 0.19524E+01_JPRB, 0.20534E+01_JPRB, 0.21983E+01_JPRB ,&
&  0.24061E+01_JPRB, 0.26469E+01_JPRB, 0.28652E+01_JPRB, 0.26562E+01_JPRB, 0.28208E+01_JPRB, 0.25323E+01_JPRB, 0.22351E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.96129E+00_JPRB, 0.95800E+00_JPRB, 0.10568E+01_JPRB, 0.97967E+00_JPRB, 0.91717E+00_JPRB, 0.86042E+00_JPRB ,&
&  0.80761E+00_JPRB, 0.74208E+00_JPRB, 0.75357E+00_JPRB, 0.76700E+00_JPRB, 0.76480E+00_JPRB, 0.80412E+00_JPRB, 0.77525E+00_JPRB ,&
&  0.78873E+00_JPRB, 0.79651E+00_JPRB, 0.78919E+00_JPRB, 0.79630E+00_JPRB, 0.79845E+00_JPRB, 0.80235E+00_JPRB, 0.78549E+00_JPRB ,&
&  0.77964E+00_JPRB, 0.77786E+00_JPRB, 0.79146E+00_JPRB, 0.81445E+00_JPRB, 0.81905E+00_JPRB, 0.80858E+00_JPRB, 0.76368E+00_JPRB ,&
&  0.71733E+00_JPRB, 0.72852E+00_JPRB, 0.83046E+00_JPRB, 0.96509E+00_JPRB, 0.94041E+00_JPRB, 0.81082E+00_JPRB, 0.63972E+00_JPRB ,&
&  0.62494E+00_JPRB, 0.64589E+00_JPRB, 0.64839E+00_JPRB, 0.62807E+00_JPRB, 0.60248E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.46724E+00_JPRB, 0.45584E+00_JPRB, 0.45512E+00_JPRB, 0.46555E+00_JPRB, 0.47212E+00_JPRB, 0.49628E+00_JPRB, 0.52683E+00_JPRB ,&
&  0.54242E+00_JPRB, 0.61141E+00_JPRB, 0.76426E+00_JPRB, 0.89444E+00_JPRB, 0.94702E+00_JPRB, 0.98490E+00_JPRB, 0.93460E+00_JPRB ,&
&  0.91837E+00_JPRB, 0.88489E+00_JPRB, 0.83331E+00_JPRB, 0.82343E+00_JPRB, 0.83626E+00_JPRB, 0.86548E+00_JPRB, 0.90477E+00_JPRB ,&
&  0.95044E+00_JPRB, 0.99792E+00_JPRB, 0.10372E+01_JPRB, 0.10742E+01_JPRB, 0.11255E+01_JPRB, 0.11081E+01_JPRB, 0.10488E+01_JPRB ,&
&  0.52248E+00_JPRB, 0.79281E+00_JPRB, 0.77037E+00_JPRB, 0.71051E+00_JPRB, 0.64949E+00_JPRB, 0.62595E+00_JPRB, 0.67943E+00_JPRB ,&
&  0.73097E+00_JPRB, 0.72967E+00_JPRB, 0.80308E+00_JPRB, 0.92579E+00_JPRB, 0.11732E+01_JPRB, 0.13064E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.15395E+01_JPRB, 0.16289E+01_JPRB, 0.17315E+01_JPRB, 0.18315E+01_JPRB, 0.19353E+01_JPRB, 0.20200E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.22639E+01_JPRB, 0.24643E+01_JPRB, 0.26476E+01_JPRB, 0.25500E+01_JPRB, 0.22121E+01_JPRB, 0.20759E+01_JPRB, 0.17372E+01_JPRB ,&
&  0.12845E+01_JPRB, 0.90438E+00_JPRB, 0.95621E+00_JPRB, 0.10179E+01_JPRB, 0.99194E+00_JPRB, 0.95638E+00_JPRB, 0.90482E+00_JPRB ,&
&  0.85045E+00_JPRB, 0.78681E+00_JPRB, 0.71226E+00_JPRB, 0.71728E+00_JPRB, 0.74368E+00_JPRB, 0.75075E+00_JPRB, 0.74051E+00_JPRB ,&
&  0.74250E+00_JPRB, 0.72200E+00_JPRB, 0.70063E+00_JPRB, 0.67913E+00_JPRB, 0.67824E+00_JPRB, 0.70619E+00_JPRB, 0.71670E+00_JPRB ,&
&  0.73083E+00_JPRB, 0.68506E+00_JPRB, 0.65143E+00_JPRB, 0.66417E+00_JPRB, 0.67660E+00_JPRB, 0.69448E+00_JPRB, 0.68915E+00_JPRB ,&
&  0.73612E+00_JPRB, 0.88871E+00_JPRB, 0.12837E+01_JPRB, 0.16870E+01_JPRB, 0.13844E+01_JPRB, 0.10890E+01_JPRB, 0.61467E+00_JPRB ,&
&  0.48756E+00_JPRB, 0.47070E+00_JPRB, 0.48834E+00_JPRB, 0.49143E+00_JPRB, 0.48968E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.41672E+00_JPRB, 0.40786E+00_JPRB, 0.41511E+00_JPRB, 0.41350E+00_JPRB, 0.41722E+00_JPRB, 0.42424E+00_JPRB, 0.42874E+00_JPRB ,&
&  0.43236E+00_JPRB, 0.44925E+00_JPRB, 0.53741E+00_JPRB, 0.65803E+00_JPRB, 0.77152E+00_JPRB, 0.85682E+00_JPRB, 0.89247E+00_JPRB ,&
&  0.92441E+00_JPRB, 0.93140E+00_JPRB, 0.91057E+00_JPRB, 0.88475E+00_JPRB, 0.88342E+00_JPRB, 0.91816E+00_JPRB, 0.94488E+00_JPRB ,&
&  0.96792E+00_JPRB, 0.98988E+00_JPRB, 0.10112E+01_JPRB, 0.10488E+01_JPRB, 0.10857E+01_JPRB, 0.10289E+01_JPRB, 0.63616E+00_JPRB ,&
&  0.82698E+00_JPRB, 0.84263E+00_JPRB, 0.82544E+00_JPRB, 0.75019E+00_JPRB, 0.68566E+00_JPRB, 0.63666E+00_JPRB, 0.64520E+00_JPRB ,&
&  0.66534E+00_JPRB, 0.67334E+00_JPRB, 0.69316E+00_JPRB, 0.75478E+00_JPRB, 0.10392E+01_JPRB, 0.11389E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.14471E+01_JPRB, 0.15202E+01_JPRB, 0.16099E+01_JPRB, 0.16911E+01_JPRB, 0.17458E+01_JPRB, 0.18215E+01_JPRB ,&
&  0.18530E+01_JPRB, 0.18992E+01_JPRB, 0.18799E+01_JPRB, 0.17927E+01_JPRB, 0.15380E+01_JPRB, 0.15948E+01_JPRB, 0.11928E+01_JPRB ,&
&  0.92100E+00_JPRB, 0.82269E+00_JPRB, 0.91234E+00_JPRB, 0.10064E+01_JPRB, 0.10118E+01_JPRB, 0.96388E+00_JPRB, 0.87663E+00_JPRB ,&
&  0.81525E+00_JPRB, 0.76784E+00_JPRB, 0.72845E+00_JPRB, 0.69360E+00_JPRB, 0.69162E+00_JPRB, 0.68890E+00_JPRB, 0.67625E+00_JPRB ,&
&  0.66660E+00_JPRB, 0.63273E+00_JPRB, 0.59386E+00_JPRB, 0.58844E+00_JPRB, 0.60153E+00_JPRB, 0.67126E+00_JPRB, 0.82528E+00_JPRB ,&
&  0.91272E+00_JPRB, 0.56484E+00_JPRB, 0.54179E+00_JPRB, 0.52102E+00_JPRB, 0.53454E+00_JPRB, 0.57556E+00_JPRB, 0.66237E+00_JPRB ,&
&  0.87748E+00_JPRB, 0.15880E+01_JPRB, 0.24919E+01_JPRB, 0.19066E+01_JPRB, 0.21861E+01_JPRB, 0.14966E+01_JPRB, 0.59757E+00_JPRB ,&
&  0.46069E+00_JPRB, 0.45047E+00_JPRB, 0.45534E+00_JPRB, 0.44485E+00_JPRB, 0.43184E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.42646E+00_JPRB, 0.41677E+00_JPRB, 0.42384E+00_JPRB, 0.43343E+00_JPRB, 0.42957E+00_JPRB, 0.42904E+00_JPRB, 0.42422E+00_JPRB ,&
&  0.41497E+00_JPRB, 0.40918E+00_JPRB, 0.43094E+00_JPRB, 0.47212E+00_JPRB, 0.52868E+00_JPRB, 0.60151E+00_JPRB, 0.67181E+00_JPRB ,&
&  0.71497E+00_JPRB, 0.76306E+00_JPRB, 0.80279E+00_JPRB, 0.83356E+00_JPRB, 0.83076E+00_JPRB, 0.82595E+00_JPRB, 0.80647E+00_JPRB ,&
&  0.79204E+00_JPRB, 0.79893E+00_JPRB, 0.83294E+00_JPRB, 0.86162E+00_JPRB, 0.91802E+00_JPRB, 0.10013E+01_JPRB, 0.64594E+00_JPRB ,&
&  0.96465E+00_JPRB, 0.95067E+00_JPRB, 0.88456E+00_JPRB, 0.79678E+00_JPRB, 0.74867E+00_JPRB, 0.67934E+00_JPRB, 0.61373E+00_JPRB ,&
&  0.59010E+00_JPRB, 0.59077E+00_JPRB, 0.59051E+00_JPRB, 0.67292E+00_JPRB, 0.79726E+00_JPRB, 0.87870E+00_JPRB, 0.97193E+00_JPRB ,&
&  0.10297E+01_JPRB, 0.11016E+01_JPRB, 0.11663E+01_JPRB, 0.12166E+01_JPRB, 0.12603E+01_JPRB, 0.12731E+01_JPRB, 0.12537E+01_JPRB ,&
&  0.11768E+01_JPRB, 0.11164E+01_JPRB, 0.10908E+01_JPRB, 0.11488E+01_JPRB, 0.10709E+01_JPRB, 0.10102E+01_JPRB, 0.86725E+00_JPRB ,&
&  0.69639E+00_JPRB, 0.67609E+00_JPRB, 0.76870E+00_JPRB, 0.10216E+01_JPRB, 0.10679E+01_JPRB, 0.10236E+01_JPRB, 0.94954E+00_JPRB ,&
&  0.88311E+00_JPRB, 0.82150E+00_JPRB, 0.76967E+00_JPRB, 0.72122E+00_JPRB, 0.68893E+00_JPRB, 0.66337E+00_JPRB, 0.63712E+00_JPRB ,&
&  0.61115E+00_JPRB, 0.57136E+00_JPRB, 0.53033E+00_JPRB, 0.51468E+00_JPRB, 0.50447E+00_JPRB, 0.51038E+00_JPRB, 0.51190E+00_JPRB ,&
&  0.60864E+00_JPRB, 0.49046E+00_JPRB, 0.45653E+00_JPRB, 0.45789E+00_JPRB, 0.42996E+00_JPRB, 0.48348E+00_JPRB, 0.66112E+00_JPRB ,&
&  0.10793E+01_JPRB, 0.16209E+01_JPRB, 0.27667E+01_JPRB, 0.49961E+01_JPRB, 0.27580E+01_JPRB, 0.21221E+01_JPRB, 0.49739E+00_JPRB ,&
&  0.47484E+00_JPRB, 0.47641E+00_JPRB, 0.47276E+00_JPRB, 0.45525E+00_JPRB, 0.45018E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.46809E+00_JPRB, 0.46593E+00_JPRB, 0.47284E+00_JPRB, 0.48043E+00_JPRB, 0.48205E+00_JPRB, 0.48311E+00_JPRB, 0.48357E+00_JPRB ,&
&  0.48561E+00_JPRB, 0.48501E+00_JPRB, 0.48696E+00_JPRB, 0.49430E+00_JPRB, 0.50956E+00_JPRB, 0.54258E+00_JPRB, 0.58036E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.64257E+00_JPRB, 0.67003E+00_JPRB, 0.69337E+00_JPRB, 0.71742E+00_JPRB, 0.73178E+00_JPRB, 0.77779E+00_JPRB ,&
&  0.83565E+00_JPRB, 0.90544E+00_JPRB, 0.97385E+00_JPRB, 0.10192E+01_JPRB, 0.10497E+01_JPRB, 0.98989E+00_JPRB, 0.74005E+00_JPRB ,&
&  0.11660E+01_JPRB, 0.11372E+01_JPRB, 0.10120E+01_JPRB, 0.89901E+00_JPRB, 0.82122E+00_JPRB, 0.75444E+00_JPRB, 0.68867E+00_JPRB ,&
&  0.62693E+00_JPRB, 0.62813E+00_JPRB, 0.58710E+00_JPRB, 0.59236E+00_JPRB, 0.58979E+00_JPRB, 0.63267E+00_JPRB, 0.73784E+00_JPRB ,&
&  0.81705E+00_JPRB, 0.88015E+00_JPRB, 0.94862E+00_JPRB, 0.10251E+01_JPRB, 0.10629E+01_JPRB, 0.10377E+01_JPRB, 0.93368E+00_JPRB ,&
&  0.87122E+00_JPRB, 0.83222E+00_JPRB, 0.77370E+00_JPRB, 0.81434E+00_JPRB, 0.82615E+00_JPRB, 0.80741E+00_JPRB, 0.78803E+00_JPRB ,&
&  0.67540E+00_JPRB, 0.70515E+00_JPRB, 0.83209E+00_JPRB, 0.11237E+01_JPRB, 0.10847E+01_JPRB, 0.10173E+01_JPRB, 0.98688E+00_JPRB ,&
&  0.91749E+00_JPRB, 0.86112E+00_JPRB, 0.81486E+00_JPRB, 0.76094E+00_JPRB, 0.71272E+00_JPRB, 0.67841E+00_JPRB, 0.64271E+00_JPRB ,&
&  0.60135E+00_JPRB, 0.55044E+00_JPRB, 0.50515E+00_JPRB, 0.47346E+00_JPRB, 0.44815E+00_JPRB, 0.38902E+00_JPRB, 0.48453E+00_JPRB ,&
&  0.48255E+00_JPRB, 0.43557E+00_JPRB, 0.47076E+00_JPRB, 0.35144E+00_JPRB, 0.44128E+00_JPRB, 0.49691E+00_JPRB, 0.62232E+00_JPRB ,&
&  0.79135E+00_JPRB, 0.12691E+01_JPRB, 0.19997E+01_JPRB, 0.18040E+01_JPRB, 0.13721E+01_JPRB, 0.87359E+00_JPRB, 0.51322E+00_JPRB ,&
&  0.49173E+00_JPRB, 0.48661E+00_JPRB, 0.49056E+00_JPRB, 0.48039E+00_JPRB, 0.47619E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.49788E+00_JPRB, 0.49681E+00_JPRB, 0.50946E+00_JPRB, 0.52210E+00_JPRB, 0.52914E+00_JPRB, 0.53377E+00_JPRB, 0.54181E+00_JPRB ,&
&  0.54545E+00_JPRB, 0.55169E+00_JPRB, 0.55936E+00_JPRB, 0.57472E+00_JPRB, 0.59825E+00_JPRB, 0.63385E+00_JPRB, 0.67460E+00_JPRB ,&
&  0.72370E+00_JPRB, 0.77641E+00_JPRB, 0.83468E+00_JPRB, 0.89804E+00_JPRB, 0.97732E+00_JPRB, 0.10862E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.14107E+01_JPRB, 0.15250E+01_JPRB, 0.16196E+01_JPRB, 0.16214E+01_JPRB, 0.15559E+01_JPRB, 0.11868E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.12609E+01_JPRB, 0.11281E+01_JPRB, 0.10056E+01_JPRB, 0.92122E+00_JPRB, 0.85174E+00_JPRB, 0.80182E+00_JPRB ,&
&  0.78474E+00_JPRB, 0.75081E+00_JPRB, 0.70226E+00_JPRB, 0.66780E+00_JPRB, 0.64868E+00_JPRB, 0.65789E+00_JPRB, 0.64717E+00_JPRB ,&
&  0.70491E+00_JPRB, 0.82727E+00_JPRB, 0.98217E+00_JPRB, 0.11704E+01_JPRB, 0.12712E+01_JPRB, 0.12489E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.96193E+00_JPRB, 0.87541E+00_JPRB, 0.84452E+00_JPRB, 0.88331E+00_JPRB, 0.91119E+00_JPRB, 0.95552E+00_JPRB, 0.95956E+00_JPRB ,&
&  0.89456E+00_JPRB, 0.84143E+00_JPRB, 0.91334E+00_JPRB, 0.11574E+01_JPRB, 0.10969E+01_JPRB, 0.98246E+00_JPRB, 0.96736E+00_JPRB ,&
&  0.89789E+00_JPRB, 0.84682E+00_JPRB, 0.81800E+00_JPRB, 0.76813E+00_JPRB, 0.71531E+00_JPRB, 0.67710E+00_JPRB, 0.65165E+00_JPRB ,&
&  0.60225E+00_JPRB, 0.54512E+00_JPRB, 0.49535E+00_JPRB, 0.47151E+00_JPRB, 0.46676E+00_JPRB, 0.42327E+00_JPRB, 0.50008E+00_JPRB ,&
&  0.47645E+00_JPRB, 0.37013E+00_JPRB, 0.44307E+00_JPRB, 0.54030E+00_JPRB, 0.53559E+00_JPRB, 0.57073E+00_JPRB, 0.63934E+00_JPRB ,&
&  0.71199E+00_JPRB, 0.77718E+00_JPRB, 0.77068E+00_JPRB, 0.65987E+00_JPRB, 0.61628E+00_JPRB, 0.59310E+00_JPRB, 0.56732E+00_JPRB ,&
&  0.54644E+00_JPRB, 0.50963E+00_JPRB, 0.52118E+00_JPRB, 0.50637E+00_JPRB, 0.50289E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.54442E+00_JPRB, 0.53842E+00_JPRB, 0.53791E+00_JPRB, 0.53623E+00_JPRB, 0.54541E+00_JPRB, 0.54671E+00_JPRB, 0.54631E+00_JPRB ,&
&  0.54454E+00_JPRB, 0.55295E+00_JPRB, 0.55869E+00_JPRB, 0.56794E+00_JPRB, 0.57947E+00_JPRB, 0.60798E+00_JPRB, 0.64136E+00_JPRB ,&
&  0.68223E+00_JPRB, 0.72944E+00_JPRB, 0.78137E+00_JPRB, 0.85541E+00_JPRB, 0.95929E+00_JPRB, 0.10318E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.13735E+01_JPRB, 0.15354E+01_JPRB, 0.17486E+01_JPRB, 0.19939E+01_JPRB, 0.20183E+01_JPRB, 0.18275E+01_JPRB ,&
&  0.13160E+01_JPRB, 0.12670E+01_JPRB, 0.11627E+01_JPRB, 0.10427E+01_JPRB, 0.10428E+01_JPRB, 0.10083E+01_JPRB, 0.10141E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10380E+01_JPRB, 0.10257E+01_JPRB, 0.10306E+01_JPRB, 0.10345E+01_JPRB, 0.10396E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.10559E+01_JPRB, 0.12191E+01_JPRB, 0.14276E+01_JPRB, 0.15700E+01_JPRB, 0.16139E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.14378E+01_JPRB, 0.13288E+01_JPRB, 0.12303E+01_JPRB, 0.12085E+01_JPRB, 0.12463E+01_JPRB, 0.12682E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.10745E+01_JPRB, 0.10810E+01_JPRB, 0.10909E+01_JPRB, 0.10559E+01_JPRB, 0.10027E+01_JPRB, 0.93388E+00_JPRB ,&
&  0.91822E+00_JPRB, 0.86555E+00_JPRB, 0.83591E+00_JPRB, 0.78159E+00_JPRB, 0.72979E+00_JPRB, 0.69476E+00_JPRB, 0.66843E+00_JPRB ,&
&  0.62169E+00_JPRB, 0.57740E+00_JPRB, 0.54318E+00_JPRB, 0.53699E+00_JPRB, 0.49619E+00_JPRB, 0.51806E+00_JPRB, 0.57359E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.47451E+00_JPRB, 0.47700E+00_JPRB, 0.64258E+00_JPRB, 0.63346E+00_JPRB, 0.63889E+00_JPRB, 0.64505E+00_JPRB ,&
&  0.65063E+00_JPRB, 0.64648E+00_JPRB, 0.62390E+00_JPRB, 0.61821E+00_JPRB, 0.62602E+00_JPRB, 0.63225E+00_JPRB, 0.62091E+00_JPRB ,&
&  0.58459E+00_JPRB, 0.56723E+00_JPRB, 0.59017E+00_JPRB, 0.55437E+00_JPRB, 0.54775E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.62617E+00_JPRB, 0.59871E+00_JPRB, 0.57854E+00_JPRB, 0.58293E+00_JPRB, 0.58961E+00_JPRB, 0.57752E+00_JPRB, 0.60550E+00_JPRB ,&
&  0.58084E+00_JPRB, 0.60103E+00_JPRB, 0.61256E+00_JPRB, 0.61071E+00_JPRB, 0.64756E+00_JPRB, 0.68536E+00_JPRB, 0.73344E+00_JPRB ,&
&  0.80120E+00_JPRB, 0.90924E+00_JPRB, 0.10429E+01_JPRB, 0.12084E+01_JPRB, 0.14098E+01_JPRB, 0.15077E+01_JPRB, 0.16096E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.19476E+01_JPRB, 0.22276E+01_JPRB, 0.25426E+01_JPRB, 0.28969E+01_JPRB, 0.27062E+01_JPRB, 0.22469E+01_JPRB ,&
&  0.15953E+01_JPRB, 0.12210E+01_JPRB, 0.11365E+01_JPRB, 0.10401E+01_JPRB, 0.10847E+01_JPRB, 0.11972E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.12651E+01_JPRB, 0.12975E+01_JPRB, 0.13296E+01_JPRB, 0.13684E+01_JPRB, 0.14154E+01_JPRB, 0.14674E+01_JPRB, 0.14906E+01_JPRB ,&
&  0.15073E+01_JPRB, 0.15254E+01_JPRB, 0.15525E+01_JPRB, 0.16399E+01_JPRB, 0.17447E+01_JPRB, 0.18456E+01_JPRB, 0.18771E+01_JPRB ,&
&  0.18995E+01_JPRB, 0.17964E+01_JPRB, 0.15549E+01_JPRB, 0.15303E+01_JPRB, 0.15716E+01_JPRB, 0.15997E+01_JPRB, 0.16275E+01_JPRB ,&
&  0.17372E+01_JPRB, 0.14243E+01_JPRB, 0.10058E+01_JPRB, 0.10326E+01_JPRB, 0.10378E+01_JPRB, 0.10524E+01_JPRB, 0.93974E+00_JPRB ,&
&  0.91674E+00_JPRB, 0.89625E+00_JPRB, 0.89296E+00_JPRB, 0.85008E+00_JPRB, 0.83198E+00_JPRB, 0.80821E+00_JPRB, 0.75874E+00_JPRB ,&
&  0.68616E+00_JPRB, 0.66506E+00_JPRB, 0.62864E+00_JPRB, 0.59817E+00_JPRB, 0.62542E+00_JPRB, 0.59013E+00_JPRB, 0.67775E+00_JPRB ,&
&  0.66957E+00_JPRB, 0.70639E+00_JPRB, 0.63775E+00_JPRB, 0.79879E+00_JPRB, 0.72758E+00_JPRB, 0.70320E+00_JPRB, 0.69542E+00_JPRB ,&
&  0.71152E+00_JPRB, 0.72061E+00_JPRB, 0.73083E+00_JPRB, 0.71953E+00_JPRB, 0.73462E+00_JPRB, 0.75963E+00_JPRB, 0.75771E+00_JPRB ,&
&  0.71423E+00_JPRB, 0.69770E+00_JPRB, 0.67248E+00_JPRB, 0.64221E+00_JPRB, 0.63624E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.68620E+00_JPRB, 0.68978E+00_JPRB, 0.68730E+00_JPRB, 0.68811E+00_JPRB, 0.69002E+00_JPRB, 0.67597E+00_JPRB, 0.69644E+00_JPRB ,&
&  0.69112E+00_JPRB, 0.71074E+00_JPRB, 0.73310E+00_JPRB, 0.75634E+00_JPRB, 0.80111E+00_JPRB, 0.84726E+00_JPRB, 0.91070E+00_JPRB ,&
&  0.10173E+01_JPRB, 0.11863E+01_JPRB, 0.14009E+01_JPRB, 0.16668E+01_JPRB, 0.17438E+01_JPRB, 0.17897E+01_JPRB, 0.18903E+01_JPRB ,&
&  0.20203E+01_JPRB, 0.21254E+01_JPRB, 0.21962E+01_JPRB, 0.22828E+01_JPRB, 0.23486E+01_JPRB, 0.22939E+01_JPRB, 0.22090E+01_JPRB ,&
&  0.17801E+01_JPRB, 0.12840E+01_JPRB, 0.12905E+01_JPRB, 0.11835E+01_JPRB, 0.12102E+01_JPRB, 0.12499E+01_JPRB, 0.12833E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.13327E+01_JPRB, 0.13541E+01_JPRB, 0.13841E+01_JPRB, 0.14303E+01_JPRB, 0.14696E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.15726E+01_JPRB, 0.16014E+01_JPRB, 0.15581E+01_JPRB, 0.16082E+01_JPRB, 0.16972E+01_JPRB, 0.17837E+01_JPRB ,&
&  0.18647E+01_JPRB, 0.18256E+01_JPRB, 0.16106E+01_JPRB, 0.16592E+01_JPRB, 0.17865E+01_JPRB, 0.17160E+01_JPRB, 0.17797E+01_JPRB ,&
&  0.19589E+01_JPRB, 0.17384E+01_JPRB, 0.99691E+00_JPRB, 0.90984E+00_JPRB, 0.96690E+00_JPRB, 0.99933E+00_JPRB, 0.10137E+01_JPRB ,&
&  0.98871E+00_JPRB, 0.96653E+00_JPRB, 0.95801E+00_JPRB, 0.96035E+00_JPRB, 0.94317E+00_JPRB, 0.94275E+00_JPRB, 0.84456E+00_JPRB ,&
&  0.66459E+00_JPRB, 0.65421E+00_JPRB, 0.66960E+00_JPRB, 0.68248E+00_JPRB, 0.73705E+00_JPRB, 0.75827E+00_JPRB, 0.75192E+00_JPRB ,&
&  0.76786E+00_JPRB, 0.82855E+00_JPRB, 0.91488E+00_JPRB, 0.84380E+00_JPRB, 0.71783E+00_JPRB, 0.76869E+00_JPRB, 0.77241E+00_JPRB ,&
&  0.79105E+00_JPRB, 0.79287E+00_JPRB, 0.80364E+00_JPRB, 0.80847E+00_JPRB, 0.81689E+00_JPRB, 0.80193E+00_JPRB, 0.78421E+00_JPRB ,&
&  0.76400E+00_JPRB, 0.74658E+00_JPRB, 0.73236E+00_JPRB, 0.71036E+00_JPRB, 0.69398E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.75583E+00_JPRB, 0.73564E+00_JPRB, 0.72507E+00_JPRB, 0.71180E+00_JPRB, 0.70697E+00_JPRB, 0.70011E+00_JPRB, 0.69738E+00_JPRB ,&
&  0.69705E+00_JPRB, 0.70104E+00_JPRB, 0.71978E+00_JPRB, 0.75278E+00_JPRB, 0.79904E+00_JPRB, 0.86143E+00_JPRB, 0.96854E+00_JPRB ,&
&  0.10737E+01_JPRB, 0.12289E+01_JPRB, 0.14134E+01_JPRB, 0.16238E+01_JPRB, 0.17664E+01_JPRB, 0.18054E+01_JPRB, 0.19023E+01_JPRB ,&
&  0.19640E+01_JPRB, 0.19781E+01_JPRB, 0.19364E+01_JPRB, 0.18757E+01_JPRB, 0.18023E+01_JPRB, 0.18526E+01_JPRB, 0.20061E+01_JPRB ,&
&  0.18433E+01_JPRB, 0.13951E+01_JPRB, 0.12519E+01_JPRB, 0.11961E+01_JPRB, 0.11900E+01_JPRB, 0.12081E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.12804E+01_JPRB, 0.13036E+01_JPRB, 0.13210E+01_JPRB, 0.13405E+01_JPRB, 0.13978E+01_JPRB, 0.14162E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14664E+01_JPRB, 0.14858E+01_JPRB, 0.15359E+01_JPRB, 0.14560E+01_JPRB, 0.15602E+01_JPRB, 0.16000E+01_JPRB, 0.16161E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.15221E+01_JPRB, 0.15895E+01_JPRB, 0.16652E+01_JPRB, 0.17406E+01_JPRB, 0.17373E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.17932E+01_JPRB, 0.16950E+01_JPRB, 0.10464E+01_JPRB, 0.96751E+00_JPRB, 0.10346E+01_JPRB, 0.94448E+00_JPRB, 0.10764E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.10373E+01_JPRB, 0.10196E+01_JPRB, 0.10335E+01_JPRB, 0.10764E+01_JPRB, 0.12123E+01_JPRB, 0.94872E+00_JPRB ,&
&  0.74787E+00_JPRB, 0.70938E+00_JPRB, 0.70674E+00_JPRB, 0.73196E+00_JPRB, 0.79486E+00_JPRB, 0.84062E+00_JPRB, 0.91799E+00_JPRB ,&
&  0.86249E+00_JPRB, 0.87758E+00_JPRB, 0.92504E+00_JPRB, 0.90910E+00_JPRB, 0.81162E+00_JPRB, 0.83611E+00_JPRB, 0.86371E+00_JPRB ,&
&  0.88092E+00_JPRB, 0.88722E+00_JPRB, 0.87775E+00_JPRB, 0.87785E+00_JPRB, 0.86534E+00_JPRB, 0.85888E+00_JPRB, 0.85123E+00_JPRB ,&
&  0.83909E+00_JPRB, 0.83489E+00_JPRB, 0.82305E+00_JPRB, 0.79075E+00_JPRB, 0.76992E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.82100E+00_JPRB, 0.79623E+00_JPRB, 0.77727E+00_JPRB, 0.76195E+00_JPRB, 0.75379E+00_JPRB, 0.73304E+00_JPRB, 0.70478E+00_JPRB ,&
&  0.69047E+00_JPRB, 0.68576E+00_JPRB, 0.69808E+00_JPRB, 0.72639E+00_JPRB, 0.76814E+00_JPRB, 0.81039E+00_JPRB, 0.88729E+00_JPRB ,&
&  0.98062E+00_JPRB, 0.11165E+01_JPRB, 0.12874E+01_JPRB, 0.15266E+01_JPRB, 0.17193E+01_JPRB, 0.18969E+01_JPRB, 0.19344E+01_JPRB ,&
&  0.20172E+01_JPRB, 0.20177E+01_JPRB, 0.18814E+01_JPRB, 0.17023E+01_JPRB, 0.13426E+01_JPRB, 0.13644E+01_JPRB, 0.15720E+01_JPRB ,&
&  0.16568E+01_JPRB, 0.13716E+01_JPRB, 0.11594E+01_JPRB, 0.11315E+01_JPRB, 0.11149E+01_JPRB, 0.11394E+01_JPRB, 0.11697E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.12312E+01_JPRB, 0.12563E+01_JPRB, 0.12960E+01_JPRB, 0.13362E+01_JPRB, 0.13503E+01_JPRB, 0.13779E+01_JPRB ,&
&  0.13748E+01_JPRB, 0.13751E+01_JPRB, 0.13730E+01_JPRB, 0.13588E+01_JPRB, 0.13633E+01_JPRB, 0.13706E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.13234E+01_JPRB, 0.13810E+01_JPRB, 0.14580E+01_JPRB, 0.14505E+01_JPRB, 0.13941E+01_JPRB, 0.14076E+01_JPRB ,&
&  0.15627E+01_JPRB, 0.17061E+01_JPRB, 0.10825E+01_JPRB, 0.11763E+01_JPRB, 0.13460E+01_JPRB, 0.11774E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10684E+01_JPRB, 0.10520E+01_JPRB, 0.10576E+01_JPRB, 0.11318E+01_JPRB, 0.13006E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.87323E+00_JPRB, 0.84200E+00_JPRB, 0.86257E+00_JPRB, 0.89125E+00_JPRB, 0.90474E+00_JPRB, 0.85459E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.79632E+00_JPRB, 0.10551E+01_JPRB, 0.12433E+01_JPRB, 0.14358E+01_JPRB, 0.13470E+01_JPRB, 0.86562E+00_JPRB, 0.94005E+00_JPRB ,&
&  0.96302E+00_JPRB, 0.96506E+00_JPRB, 0.93281E+00_JPRB, 0.92191E+00_JPRB, 0.91296E+00_JPRB, 0.90106E+00_JPRB, 0.89877E+00_JPRB ,&
&  0.89184E+00_JPRB, 0.89384E+00_JPRB, 0.89868E+00_JPRB, 0.87764E+00_JPRB, 0.84593E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.89087E+00_JPRB, 0.87159E+00_JPRB, 0.85064E+00_JPRB, 0.83042E+00_JPRB, 0.81248E+00_JPRB, 0.76823E+00_JPRB, 0.72090E+00_JPRB ,&
&  0.69005E+00_JPRB, 0.67491E+00_JPRB, 0.67751E+00_JPRB, 0.68743E+00_JPRB, 0.71455E+00_JPRB, 0.74412E+00_JPRB, 0.79512E+00_JPRB ,&
&  0.87500E+00_JPRB, 0.98867E+00_JPRB, 0.11426E+01_JPRB, 0.13310E+01_JPRB, 0.15621E+01_JPRB, 0.18305E+01_JPRB, 0.20025E+01_JPRB ,&
&  0.21545E+01_JPRB, 0.21668E+01_JPRB, 0.18979E+01_JPRB, 0.13541E+01_JPRB, 0.11945E+01_JPRB, 0.12031E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11269E+01_JPRB, 0.10671E+01_JPRB, 0.10414E+01_JPRB, 0.10407E+01_JPRB, 0.10622E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.11051E+01_JPRB, 0.11235E+01_JPRB, 0.11472E+01_JPRB, 0.11929E+01_JPRB, 0.12204E+01_JPRB, 0.12560E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.13165E+01_JPRB, 0.12855E+01_JPRB, 0.12601E+01_JPRB, 0.12207E+01_JPRB, 0.12097E+01_JPRB, 0.12030E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.11998E+01_JPRB, 0.12008E+01_JPRB, 0.12492E+01_JPRB, 0.12814E+01_JPRB, 0.11722E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.14019E+01_JPRB, 0.15384E+01_JPRB, 0.12599E+01_JPRB, 0.14365E+01_JPRB, 0.94076E+00_JPRB, 0.10901E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.10929E+01_JPRB, 0.10488E+01_JPRB, 0.10702E+01_JPRB, 0.11394E+01_JPRB, 0.11674E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.97488E+00_JPRB, 0.95400E+00_JPRB, 0.10034E+01_JPRB, 0.10619E+01_JPRB, 0.10835E+01_JPRB, 0.10008E+01_JPRB, 0.90818E+00_JPRB ,&
&  0.84517E+00_JPRB, 0.10182E+01_JPRB, 0.10800E+01_JPRB, 0.11487E+01_JPRB, 0.10060E+01_JPRB, 0.85609E+00_JPRB, 0.90546E+00_JPRB ,&
&  0.94577E+00_JPRB, 0.96549E+00_JPRB, 0.96147E+00_JPRB, 0.94428E+00_JPRB, 0.92065E+00_JPRB, 0.90077E+00_JPRB, 0.89237E+00_JPRB ,&
&  0.89044E+00_JPRB, 0.88617E+00_JPRB, 0.89055E+00_JPRB, 0.89891E+00_JPRB, 0.90251E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.89890E+00_JPRB, 0.89445E+00_JPRB, 0.89333E+00_JPRB, 0.89348E+00_JPRB, 0.88668E+00_JPRB, 0.85830E+00_JPRB, 0.79745E+00_JPRB ,&
&  0.71037E+00_JPRB, 0.64324E+00_JPRB, 0.63202E+00_JPRB, 0.63137E+00_JPRB, 0.64011E+00_JPRB, 0.67061E+00_JPRB, 0.72206E+00_JPRB ,&
&  0.78004E+00_JPRB, 0.87625E+00_JPRB, 0.10110E+01_JPRB, 0.11812E+01_JPRB, 0.14004E+01_JPRB, 0.17356E+01_JPRB, 0.20933E+01_JPRB ,&
&  0.20237E+01_JPRB, 0.15076E+01_JPRB, 0.18210E+01_JPRB, 0.18010E+01_JPRB, 0.16114E+01_JPRB, 0.13747E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.11110E+01_JPRB, 0.10813E+01_JPRB, 0.10378E+01_JPRB, 0.98219E+00_JPRB, 0.99461E+00_JPRB, 0.10104E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.10487E+01_JPRB, 0.10598E+01_JPRB, 0.11071E+01_JPRB, 0.11275E+01_JPRB, 0.11549E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12232E+01_JPRB, 0.12019E+01_JPRB, 0.11314E+01_JPRB, 0.11165E+01_JPRB, 0.11153E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.11177E+01_JPRB, 0.11507E+01_JPRB, 0.11919E+01_JPRB, 0.12419E+01_JPRB, 0.11453E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.14634E+01_JPRB, 0.14194E+01_JPRB, 0.15572E+01_JPRB, 0.11056E+01_JPRB, 0.11996E+01_JPRB, 0.11508E+01_JPRB ,&
&  0.12259E+01_JPRB, 0.11365E+01_JPRB, 0.10694E+01_JPRB, 0.10590E+01_JPRB, 0.11137E+01_JPRB, 0.11827E+01_JPRB, 0.12289E+01_JPRB ,&
&  0.10523E+01_JPRB, 0.10245E+01_JPRB, 0.10713E+01_JPRB, 0.11806E+01_JPRB, 0.11949E+01_JPRB, 0.11505E+01_JPRB, 0.11133E+01_JPRB ,&
&  0.12457E+01_JPRB, 0.11889E+01_JPRB, 0.11659E+01_JPRB, 0.11157E+01_JPRB, 0.98056E+00_JPRB, 0.88095E+00_JPRB, 0.88223E+00_JPRB ,&
&  0.90041E+00_JPRB, 0.91914E+00_JPRB, 0.92105E+00_JPRB, 0.90975E+00_JPRB, 0.88385E+00_JPRB, 0.86056E+00_JPRB, 0.84819E+00_JPRB ,&
&  0.83936E+00_JPRB, 0.84583E+00_JPRB, 0.85456E+00_JPRB, 0.86517E+00_JPRB, 0.88356E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.82568E+00_JPRB, 0.84259E+00_JPRB, 0.85600E+00_JPRB, 0.89556E+00_JPRB, 0.94756E+00_JPRB, 0.10176E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.91016E+00_JPRB, 0.62675E+00_JPRB, 0.61256E+00_JPRB, 0.60343E+00_JPRB, 0.59784E+00_JPRB, 0.60632E+00_JPRB, 0.64970E+00_JPRB ,&
&  0.71556E+00_JPRB, 0.80446E+00_JPRB, 0.90536E+00_JPRB, 0.10324E+01_JPRB, 0.12016E+01_JPRB, 0.15279E+01_JPRB, 0.17742E+01_JPRB ,&
&  0.15736E+01_JPRB, 0.18390E+01_JPRB, 0.22390E+01_JPRB, 0.20205E+01_JPRB, 0.18202E+01_JPRB, 0.16571E+01_JPRB, 0.13290E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.11605E+01_JPRB, 0.11468E+01_JPRB, 0.11287E+01_JPRB, 0.99683E+00_JPRB, 0.97969E+00_JPRB, 0.97825E+00_JPRB ,&
&  0.98360E+00_JPRB, 0.99064E+00_JPRB, 0.99847E+00_JPRB, 0.10507E+01_JPRB, 0.10962E+01_JPRB, 0.11236E+01_JPRB, 0.11440E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.11647E+01_JPRB, 0.11504E+01_JPRB, 0.10750E+01_JPRB, 0.10803E+01_JPRB, 0.10964E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.11508E+01_JPRB, 0.12211E+01_JPRB, 0.12567E+01_JPRB, 0.12153E+01_JPRB, 0.11322E+01_JPRB, 0.12346E+01_JPRB, 0.11819E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.13353E+01_JPRB, 0.16489E+01_JPRB, 0.12579E+01_JPRB, 0.12984E+01_JPRB, 0.13864E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.12824E+01_JPRB, 0.12709E+01_JPRB, 0.11561E+01_JPRB, 0.10733E+01_JPRB, 0.10851E+01_JPRB, 0.11069E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.12458E+01_JPRB, 0.11525E+01_JPRB, 0.11771E+01_JPRB, 0.11508E+01_JPRB, 0.11297E+01_JPRB, 0.12179E+01_JPRB ,&
&  0.15604E+01_JPRB, 0.15031E+01_JPRB, 0.13952E+01_JPRB, 0.12888E+01_JPRB, 0.11539E+01_JPRB, 0.10747E+01_JPRB, 0.98636E+00_JPRB ,&
&  0.93985E+00_JPRB, 0.92295E+00_JPRB, 0.92043E+00_JPRB, 0.89146E+00_JPRB, 0.86706E+00_JPRB, 0.83433E+00_JPRB, 0.81068E+00_JPRB ,&
&  0.79593E+00_JPRB, 0.78993E+00_JPRB, 0.78893E+00_JPRB, 0.79679E+00_JPRB, 0.80790E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.77945E+00_JPRB, 0.78243E+00_JPRB, 0.77795E+00_JPRB, 0.75082E+00_JPRB, 0.73257E+00_JPRB, 0.71850E+00_JPRB, 0.68400E+00_JPRB ,&
&  0.65893E+00_JPRB, 0.64105E+00_JPRB, 0.61981E+00_JPRB, 0.60259E+00_JPRB, 0.59574E+00_JPRB, 0.58708E+00_JPRB, 0.59798E+00_JPRB ,&
&  0.64309E+00_JPRB, 0.71702E+00_JPRB, 0.83825E+00_JPRB, 0.92931E+00_JPRB, 0.10431E+01_JPRB, 0.12570E+01_JPRB, 0.13583E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.24252E+01_JPRB, 0.22642E+01_JPRB, 0.22190E+01_JPRB, 0.21070E+01_JPRB, 0.19693E+01_JPRB, 0.16195E+01_JPRB ,&
&  0.13521E+01_JPRB, 0.12305E+01_JPRB, 0.11599E+01_JPRB, 0.11034E+01_JPRB, 0.10733E+01_JPRB, 0.10084E+01_JPRB, 0.99581E+00_JPRB ,&
&  0.98545E+00_JPRB, 0.97959E+00_JPRB, 0.96832E+00_JPRB, 0.10035E+01_JPRB, 0.10618E+01_JPRB, 0.11034E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.11673E+01_JPRB, 0.11438E+01_JPRB, 0.11302E+01_JPRB, 0.11405E+01_JPRB, 0.11875E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12315E+01_JPRB, 0.13533E+01_JPRB, 0.13770E+01_JPRB, 0.13466E+01_JPRB, 0.13819E+01_JPRB, 0.12443E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.12611E+01_JPRB, 0.15294E+01_JPRB, 0.13341E+01_JPRB, 0.13975E+01_JPRB, 0.15164E+01_JPRB, 0.15029E+01_JPRB ,&
&  0.13572E+01_JPRB, 0.13848E+01_JPRB, 0.12802E+01_JPRB, 0.11877E+01_JPRB, 0.11171E+01_JPRB, 0.11184E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.13020E+01_JPRB, 0.12990E+01_JPRB, 0.13400E+01_JPRB, 0.11874E+01_JPRB, 0.11951E+01_JPRB, 0.12036E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.18702E+01_JPRB, 0.18971E+01_JPRB, 0.17963E+01_JPRB, 0.15922E+01_JPRB, 0.13863E+01_JPRB, 0.13101E+01_JPRB, 0.11928E+01_JPRB ,&
&  0.10989E+01_JPRB, 0.10256E+01_JPRB, 0.97734E+00_JPRB, 0.93351E+00_JPRB, 0.88639E+00_JPRB, 0.83998E+00_JPRB, 0.81019E+00_JPRB ,&
&  0.78734E+00_JPRB, 0.77538E+00_JPRB, 0.77435E+00_JPRB, 0.77283E+00_JPRB, 0.77483E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.77486E+00_JPRB, 0.77008E+00_JPRB, 0.77775E+00_JPRB, 0.75982E+00_JPRB, 0.73546E+00_JPRB, 0.71425E+00_JPRB, 0.70391E+00_JPRB ,&
&  0.68719E+00_JPRB, 0.65980E+00_JPRB, 0.63491E+00_JPRB, 0.61528E+00_JPRB, 0.59794E+00_JPRB, 0.59156E+00_JPRB, 0.59450E+00_JPRB ,&
&  0.61358E+00_JPRB, 0.64240E+00_JPRB, 0.71607E+00_JPRB, 0.81840E+00_JPRB, 0.91493E+00_JPRB, 0.10713E+01_JPRB, 0.95407E+00_JPRB ,&
&  0.12885E+01_JPRB, 0.22239E+01_JPRB, 0.23029E+01_JPRB, 0.23374E+01_JPRB, 0.22768E+01_JPRB, 0.22312E+01_JPRB, 0.18740E+01_JPRB ,&
&  0.15873E+01_JPRB, 0.14143E+01_JPRB, 0.12438E+01_JPRB, 0.11508E+01_JPRB, 0.11107E+01_JPRB, 0.10800E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.98141E+00_JPRB, 0.97283E+00_JPRB, 0.10037E+01_JPRB, 0.10564E+01_JPRB, 0.11257E+01_JPRB, 0.11656E+01_JPRB ,&
&  0.11805E+01_JPRB, 0.11902E+01_JPRB, 0.12142E+01_JPRB, 0.11677E+01_JPRB, 0.12272E+01_JPRB, 0.12762E+01_JPRB, 0.13282E+01_JPRB ,&
&  0.12860E+01_JPRB, 0.12476E+01_JPRB, 0.14474E+01_JPRB, 0.16183E+01_JPRB, 0.16665E+01_JPRB, 0.15660E+01_JPRB, 0.14245E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.13857E+01_JPRB, 0.14648E+01_JPRB, 0.14320E+01_JPRB, 0.15036E+01_JPRB, 0.16168E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.15192E+01_JPRB, 0.13998E+01_JPRB, 0.12686E+01_JPRB, 0.11845E+01_JPRB, 0.11100E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.14330E+01_JPRB, 0.15516E+01_JPRB, 0.15904E+01_JPRB, 0.13934E+01_JPRB, 0.12907E+01_JPRB, 0.12891E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.20156E+01_JPRB, 0.21586E+01_JPRB, 0.19127E+01_JPRB, 0.18348E+01_JPRB, 0.15163E+01_JPRB, 0.14131E+01_JPRB, 0.13073E+01_JPRB ,&
&  0.11830E+01_JPRB, 0.10929E+01_JPRB, 0.10134E+01_JPRB, 0.96069E+00_JPRB, 0.93305E+00_JPRB, 0.88898E+00_JPRB, 0.85457E+00_JPRB ,&
&  0.82803E+00_JPRB, 0.81686E+00_JPRB, 0.80712E+00_JPRB, 0.79748E+00_JPRB, 0.78327E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.80625E+00_JPRB, 0.78746E+00_JPRB, 0.77589E+00_JPRB, 0.77473E+00_JPRB, 0.77158E+00_JPRB, 0.75121E+00_JPRB, 0.73256E+00_JPRB ,&
&  0.71172E+00_JPRB, 0.68016E+00_JPRB, 0.65444E+00_JPRB, 0.62820E+00_JPRB, 0.60900E+00_JPRB, 0.59753E+00_JPRB, 0.59634E+00_JPRB ,&
&  0.60853E+00_JPRB, 0.62528E+00_JPRB, 0.68585E+00_JPRB, 0.75863E+00_JPRB, 0.82485E+00_JPRB, 0.79506E+00_JPRB, 0.90820E+00_JPRB ,&
&  0.15246E+01_JPRB, 0.21790E+01_JPRB, 0.21582E+01_JPRB, 0.22565E+01_JPRB, 0.23682E+01_JPRB, 0.25186E+01_JPRB, 0.21009E+01_JPRB ,&
&  0.17975E+01_JPRB, 0.16047E+01_JPRB, 0.14035E+01_JPRB, 0.12718E+01_JPRB, 0.11810E+01_JPRB, 0.11278E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10108E+01_JPRB, 0.98474E+00_JPRB, 0.10069E+01_JPRB, 0.10822E+01_JPRB, 0.11515E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12001E+01_JPRB, 0.11960E+01_JPRB, 0.12255E+01_JPRB, 0.12239E+01_JPRB, 0.12699E+01_JPRB, 0.13046E+01_JPRB, 0.13680E+01_JPRB ,&
&  0.13940E+01_JPRB, 0.14979E+01_JPRB, 0.16172E+01_JPRB, 0.16805E+01_JPRB, 0.17512E+01_JPRB, 0.17842E+01_JPRB, 0.16886E+01_JPRB ,&
&  0.15788E+01_JPRB, 0.14208E+01_JPRB, 0.12913E+01_JPRB, 0.14274E+01_JPRB, 0.15553E+01_JPRB, 0.17414E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.14558E+01_JPRB, 0.12409E+01_JPRB, 0.12292E+01_JPRB, 0.12875E+01_JPRB, 0.13383E+01_JPRB, 0.12193E+01_JPRB, 0.13997E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.98146E+00_JPRB, 0.87143E+00_JPRB, 0.10380E+01_JPRB, 0.93969E+00_JPRB, 0.99762E+00_JPRB, 0.15302E+01_JPRB ,&
&  0.20133E+01_JPRB, 0.25835E+01_JPRB, 0.23698E+01_JPRB, 0.18329E+01_JPRB, 0.16988E+01_JPRB, 0.15189E+01_JPRB, 0.13846E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.11018E+01_JPRB, 0.10363E+01_JPRB, 0.97134E+00_JPRB, 0.92459E+00_JPRB, 0.90489E+00_JPRB, 0.89431E+00_JPRB ,&
&  0.87494E+00_JPRB, 0.86030E+00_JPRB, 0.85422E+00_JPRB, 0.84696E+00_JPRB, 0.82838E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.84612E+00_JPRB, 0.82065E+00_JPRB, 0.80284E+00_JPRB, 0.78468E+00_JPRB, 0.76813E+00_JPRB, 0.75141E+00_JPRB, 0.73352E+00_JPRB ,&
&  0.71722E+00_JPRB, 0.68737E+00_JPRB, 0.65787E+00_JPRB, 0.63173E+00_JPRB, 0.61141E+00_JPRB, 0.59921E+00_JPRB, 0.59942E+00_JPRB ,&
&  0.60100E+00_JPRB, 0.62205E+00_JPRB, 0.71285E+00_JPRB, 0.76976E+00_JPRB, 0.77909E+00_JPRB, 0.71954E+00_JPRB, 0.87748E+00_JPRB ,&
&  0.12591E+01_JPRB, 0.18126E+01_JPRB, 0.22831E+01_JPRB, 0.24562E+01_JPRB, 0.25502E+01_JPRB, 0.24632E+01_JPRB, 0.21858E+01_JPRB ,&
&  0.19410E+01_JPRB, 0.17858E+01_JPRB, 0.16149E+01_JPRB, 0.14479E+01_JPRB, 0.12968E+01_JPRB, 0.12021E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10375E+01_JPRB, 0.98401E+00_JPRB, 0.99148E+00_JPRB, 0.10577E+01_JPRB, 0.11423E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.11897E+01_JPRB, 0.11791E+01_JPRB, 0.12189E+01_JPRB, 0.12923E+01_JPRB, 0.10949E+01_JPRB, 0.12693E+01_JPRB, 0.12932E+01_JPRB ,&
&  0.13696E+01_JPRB, 0.14759E+01_JPRB, 0.14999E+01_JPRB, 0.17930E+01_JPRB, 0.19085E+01_JPRB, 0.18987E+01_JPRB, 0.18867E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.15935E+01_JPRB, 0.13701E+01_JPRB, 0.14807E+01_JPRB, 0.16047E+01_JPRB, 0.18245E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.14816E+01_JPRB, 0.14171E+01_JPRB, 0.12498E+01_JPRB, 0.14208E+01_JPRB, 0.13670E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.59648E+00_JPRB, 0.49571E+00_JPRB, 0.51779E+00_JPRB, 0.52445E+00_JPRB, 0.51525E+00_JPRB, 0.77654E+00_JPRB, 0.23522E+01_JPRB ,&
&  0.20720E+01_JPRB, 0.27699E+01_JPRB, 0.25773E+01_JPRB, 0.21960E+01_JPRB, 0.18443E+01_JPRB, 0.16952E+01_JPRB, 0.16638E+01_JPRB ,&
&  0.13671E+01_JPRB, 0.12383E+01_JPRB, 0.11097E+01_JPRB, 0.10197E+01_JPRB, 0.97805E+00_JPRB, 0.95043E+00_JPRB, 0.93387E+00_JPRB ,&
&  0.91409E+00_JPRB, 0.89482E+00_JPRB, 0.88619E+00_JPRB, 0.88214E+00_JPRB, 0.87057E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.87502E+00_JPRB, 0.85976E+00_JPRB, 0.84243E+00_JPRB, 0.81901E+00_JPRB, 0.78636E+00_JPRB, 0.75319E+00_JPRB, 0.72680E+00_JPRB ,&
&  0.70929E+00_JPRB, 0.67831E+00_JPRB, 0.64872E+00_JPRB, 0.62495E+00_JPRB, 0.60708E+00_JPRB, 0.59710E+00_JPRB, 0.59701E+00_JPRB ,&
&  0.60294E+00_JPRB, 0.63290E+00_JPRB, 0.81540E+00_JPRB, 0.77111E+00_JPRB, 0.78262E+00_JPRB, 0.74321E+00_JPRB, 0.79652E+00_JPRB ,&
&  0.10769E+01_JPRB, 0.15226E+01_JPRB, 0.20268E+01_JPRB, 0.23379E+01_JPRB, 0.27017E+01_JPRB, 0.27478E+01_JPRB, 0.25679E+01_JPRB ,&
&  0.22098E+01_JPRB, 0.19175E+01_JPRB, 0.16770E+01_JPRB, 0.14819E+01_JPRB, 0.13030E+01_JPRB, 0.11861E+01_JPRB, 0.11221E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10595E+01_JPRB, 0.99556E+00_JPRB, 0.98723E+00_JPRB, 0.10564E+01_JPRB, 0.11413E+01_JPRB, 0.11806E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.11479E+01_JPRB, 0.11694E+01_JPRB, 0.12342E+01_JPRB, 0.11023E+01_JPRB, 0.99434E+00_JPRB, 0.11487E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.15518E+01_JPRB, 0.15913E+01_JPRB, 0.19321E+01_JPRB, 0.20106E+01_JPRB, 0.19900E+01_JPRB, 0.21981E+01_JPRB ,&
&  0.21200E+01_JPRB, 0.21291E+01_JPRB, 0.15412E+01_JPRB, 0.15616E+01_JPRB, 0.17312E+01_JPRB, 0.14499E+01_JPRB, 0.14486E+01_JPRB ,&
&  0.16913E+01_JPRB, 0.15458E+01_JPRB, 0.13942E+01_JPRB, 0.12065E+01_JPRB, 0.14600E+01_JPRB, 0.12030E+01_JPRB, 0.65108E+00_JPRB ,&
&  0.55171E+00_JPRB, 0.50507E+00_JPRB, 0.47366E+00_JPRB, 0.44409E+00_JPRB, 0.45312E+00_JPRB, 0.72571E+00_JPRB, 0.15334E+01_JPRB ,&
&  0.17218E+01_JPRB, 0.25073E+01_JPRB, 0.25221E+01_JPRB, 0.22649E+01_JPRB, 0.18913E+01_JPRB, 0.18303E+01_JPRB, 0.17596E+01_JPRB ,&
&  0.14941E+01_JPRB, 0.14157E+01_JPRB, 0.12705E+01_JPRB, 0.11583E+01_JPRB, 0.10675E+01_JPRB, 0.10103E+01_JPRB, 0.98530E+00_JPRB ,&
&  0.96150E+00_JPRB, 0.94035E+00_JPRB, 0.92492E+00_JPRB, 0.91022E+00_JPRB, 0.89392E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.88197E+00_JPRB, 0.86455E+00_JPRB, 0.85307E+00_JPRB, 0.83566E+00_JPRB, 0.80735E+00_JPRB, 0.76906E+00_JPRB, 0.73239E+00_JPRB ,&
&  0.70560E+00_JPRB, 0.66916E+00_JPRB, 0.64138E+00_JPRB, 0.61972E+00_JPRB, 0.60178E+00_JPRB, 0.59375E+00_JPRB, 0.59339E+00_JPRB ,&
&  0.59623E+00_JPRB, 0.67280E+00_JPRB, 0.80241E+00_JPRB, 0.69457E+00_JPRB, 0.64614E+00_JPRB, 0.60668E+00_JPRB, 0.60786E+00_JPRB ,&
&  0.93497E+00_JPRB, 0.13282E+01_JPRB, 0.17338E+01_JPRB, 0.22889E+01_JPRB, 0.27423E+01_JPRB, 0.26572E+01_JPRB, 0.27517E+01_JPRB ,&
&  0.24577E+01_JPRB, 0.20635E+01_JPRB, 0.17707E+01_JPRB, 0.15475E+01_JPRB, 0.13485E+01_JPRB, 0.12108E+01_JPRB, 0.11226E+01_JPRB ,&
&  0.10831E+01_JPRB, 0.10597E+01_JPRB, 0.10136E+01_JPRB, 0.98932E+00_JPRB, 0.10438E+01_JPRB, 0.11599E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.11240E+01_JPRB, 0.11187E+01_JPRB, 0.11571E+01_JPRB, 0.11973E+01_JPRB, 0.11973E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.12457E+01_JPRB, 0.16569E+01_JPRB, 0.18413E+01_JPRB, 0.19970E+01_JPRB, 0.21588E+01_JPRB, 0.23001E+01_JPRB ,&
&  0.22226E+01_JPRB, 0.23092E+01_JPRB, 0.17966E+01_JPRB, 0.17741E+01_JPRB, 0.15515E+01_JPRB, 0.13343E+01_JPRB, 0.17449E+01_JPRB ,&
&  0.18726E+01_JPRB, 0.17991E+01_JPRB, 0.14889E+01_JPRB, 0.11761E+01_JPRB, 0.10531E+01_JPRB, 0.74704E+00_JPRB, 0.52756E+00_JPRB ,&
&  0.51030E+00_JPRB, 0.49130E+00_JPRB, 0.44666E+00_JPRB, 0.45442E+00_JPRB, 0.45101E+00_JPRB, 0.65959E+00_JPRB, 0.11141E+01_JPRB ,&
&  0.17142E+01_JPRB, 0.20236E+01_JPRB, 0.21345E+01_JPRB, 0.19511E+01_JPRB, 0.17112E+01_JPRB, 0.16669E+01_JPRB, 0.16850E+01_JPRB ,&
&  0.14508E+01_JPRB, 0.14380E+01_JPRB, 0.13513E+01_JPRB, 0.12269E+01_JPRB, 0.11506E+01_JPRB, 0.10913E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.99343E+00_JPRB, 0.96783E+00_JPRB, 0.93427E+00_JPRB, 0.90714E+00_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.88032E+00_JPRB, 0.85545E+00_JPRB, 0.84094E+00_JPRB, 0.82695E+00_JPRB, 0.80874E+00_JPRB, 0.77968E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.70585E+00_JPRB, 0.67062E+00_JPRB, 0.64421E+00_JPRB, 0.62291E+00_JPRB, 0.60176E+00_JPRB, 0.59663E+00_JPRB, 0.59842E+00_JPRB ,&
&  0.61456E+00_JPRB, 0.80718E+00_JPRB, 0.61102E+00_JPRB, 0.51029E+00_JPRB, 0.55585E+00_JPRB, 0.56491E+00_JPRB, 0.53099E+00_JPRB ,&
&  0.82543E+00_JPRB, 0.12384E+01_JPRB, 0.15635E+01_JPRB, 0.22704E+01_JPRB, 0.30605E+01_JPRB, 0.29188E+01_JPRB, 0.28685E+01_JPRB ,&
&  0.27915E+01_JPRB, 0.22641E+01_JPRB, 0.19069E+01_JPRB, 0.16488E+01_JPRB, 0.14002E+01_JPRB, 0.12255E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10395E+01_JPRB, 0.10156E+01_JPRB, 0.10169E+01_JPRB, 0.10408E+01_JPRB, 0.11253E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.11892E+01_JPRB, 0.11316E+01_JPRB, 0.11170E+01_JPRB, 0.11666E+01_JPRB, 0.11589E+01_JPRB, 0.11571E+01_JPRB, 0.13851E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.15461E+01_JPRB, 0.16459E+01_JPRB, 0.20617E+01_JPRB, 0.23033E+01_JPRB, 0.22537E+01_JPRB, 0.24838E+01_JPRB ,&
&  0.19002E+01_JPRB, 0.16889E+01_JPRB, 0.17119E+01_JPRB, 0.16418E+01_JPRB, 0.13859E+01_JPRB, 0.16574E+01_JPRB, 0.25902E+01_JPRB ,&
&  0.19705E+01_JPRB, 0.22053E+01_JPRB, 0.22782E+01_JPRB, 0.23155E+01_JPRB, 0.11186E+01_JPRB, 0.72554E+00_JPRB, 0.11694E+01_JPRB ,&
&  0.12497E+01_JPRB, 0.79681E+00_JPRB, 0.57361E+00_JPRB, 0.70903E+00_JPRB, 0.58280E+00_JPRB, 0.77294E+00_JPRB, 0.11546E+01_JPRB ,&
&  0.13573E+01_JPRB, 0.15051E+01_JPRB, 0.19601E+01_JPRB, 0.17464E+01_JPRB, 0.16128E+01_JPRB, 0.16629E+01_JPRB, 0.16329E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.14686E+01_JPRB, 0.14611E+01_JPRB, 0.12869E+01_JPRB, 0.11745E+01_JPRB, 0.11063E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10301E+01_JPRB, 0.99625E+00_JPRB, 0.95477E+00_JPRB, 0.91167E+00_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.84688E+00_JPRB, 0.81986E+00_JPRB, 0.79991E+00_JPRB, 0.79186E+00_JPRB, 0.79028E+00_JPRB, 0.78051E+00_JPRB, 0.74922E+00_JPRB ,&
&  0.71053E+00_JPRB, 0.67798E+00_JPRB, 0.64668E+00_JPRB, 0.62671E+00_JPRB, 0.60499E+00_JPRB, 0.60096E+00_JPRB, 0.60400E+00_JPRB ,&
&  0.61447E+00_JPRB, 0.63194E+00_JPRB, 0.48357E+00_JPRB, 0.45295E+00_JPRB, 0.56329E+00_JPRB, 0.54395E+00_JPRB, 0.56198E+00_JPRB ,&
&  0.79417E+00_JPRB, 0.10873E+01_JPRB, 0.14327E+01_JPRB, 0.21175E+01_JPRB, 0.24934E+01_JPRB, 0.28390E+01_JPRB, 0.28816E+01_JPRB ,&
&  0.29375E+01_JPRB, 0.24448E+01_JPRB, 0.19982E+01_JPRB, 0.16549E+01_JPRB, 0.13883E+01_JPRB, 0.12227E+01_JPRB, 0.11132E+01_JPRB ,&
&  0.10172E+01_JPRB, 0.97281E+00_JPRB, 0.98850E+00_JPRB, 0.98868E+00_JPRB, 0.10267E+01_JPRB, 0.10732E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.11905E+01_JPRB, 0.11365E+01_JPRB, 0.10851E+01_JPRB, 0.11291E+01_JPRB, 0.10738E+01_JPRB, 0.10508E+01_JPRB, 0.11708E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.16078E+01_JPRB, 0.18087E+01_JPRB, 0.21317E+01_JPRB, 0.24892E+01_JPRB, 0.23602E+01_JPRB, 0.28013E+01_JPRB ,&
&  0.23115E+01_JPRB, 0.20360E+01_JPRB, 0.17826E+01_JPRB, 0.14930E+01_JPRB, 0.17766E+01_JPRB, 0.29007E+01_JPRB, 0.28488E+01_JPRB ,&
&  0.26116E+01_JPRB, 0.26332E+01_JPRB, 0.25692E+01_JPRB, 0.21884E+01_JPRB, 0.17950E+01_JPRB, 0.13001E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.14621E+01_JPRB, 0.14512E+01_JPRB, 0.12687E+01_JPRB, 0.10265E+01_JPRB, 0.87059E+00_JPRB, 0.10571E+01_JPRB, 0.10791E+01_JPRB ,&
&  0.11236E+01_JPRB, 0.11835E+01_JPRB, 0.15291E+01_JPRB, 0.15487E+01_JPRB, 0.15011E+01_JPRB, 0.14807E+01_JPRB, 0.16270E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.14926E+01_JPRB, 0.14470E+01_JPRB, 0.13487E+01_JPRB, 0.12639E+01_JPRB, 0.11854E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10622E+01_JPRB, 0.10139E+01_JPRB, 0.95830E+00_JPRB, 0.91620E+00_JPRB, 0.87779E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.78915E+00_JPRB, 0.76002E+00_JPRB, 0.73954E+00_JPRB, 0.72912E+00_JPRB, 0.73218E+00_JPRB, 0.74149E+00_JPRB, 0.73164E+00_JPRB ,&
&  0.70464E+00_JPRB, 0.67618E+00_JPRB, 0.65117E+00_JPRB, 0.63238E+00_JPRB, 0.61552E+00_JPRB, 0.61030E+00_JPRB, 0.60697E+00_JPRB ,&
&  0.61213E+00_JPRB, 0.54987E+00_JPRB, 0.42341E+00_JPRB, 0.44334E+00_JPRB, 0.50999E+00_JPRB, 0.49949E+00_JPRB, 0.60779E+00_JPRB ,&
&  0.74231E+00_JPRB, 0.94531E+00_JPRB, 0.11875E+01_JPRB, 0.17148E+01_JPRB, 0.20958E+01_JPRB, 0.23619E+01_JPRB, 0.24680E+01_JPRB ,&
&  0.23676E+01_JPRB, 0.22038E+01_JPRB, 0.19684E+01_JPRB, 0.17091E+01_JPRB, 0.14129E+01_JPRB, 0.12818E+01_JPRB, 0.11919E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.10209E+01_JPRB, 0.96857E+00_JPRB, 0.97163E+00_JPRB, 0.99216E+00_JPRB, 0.10357E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.11700E+01_JPRB, 0.11551E+01_JPRB, 0.10609E+01_JPRB, 0.10710E+01_JPRB, 0.94553E+00_JPRB, 0.95938E+00_JPRB, 0.10599E+01_JPRB ,&
&  0.16236E+01_JPRB, 0.18733E+01_JPRB, 0.19648E+01_JPRB, 0.24697E+01_JPRB, 0.22317E+01_JPRB, 0.23635E+01_JPRB, 0.29905E+01_JPRB ,&
&  0.32172E+01_JPRB, 0.32283E+01_JPRB, 0.33506E+01_JPRB, 0.26417E+01_JPRB, 0.22305E+01_JPRB, 0.31872E+01_JPRB, 0.30260E+01_JPRB ,&
&  0.27886E+01_JPRB, 0.26787E+01_JPRB, 0.24107E+01_JPRB, 0.24825E+01_JPRB, 0.19513E+01_JPRB, 0.16135E+01_JPRB, 0.11266E+01_JPRB ,&
&  0.10062E+01_JPRB, 0.90039E+00_JPRB, 0.10045E+01_JPRB, 0.92118E+00_JPRB, 0.93014E+00_JPRB, 0.88783E+00_JPRB, 0.86791E+00_JPRB ,&
&  0.93663E+00_JPRB, 0.96951E+00_JPRB, 0.10315E+01_JPRB, 0.11934E+01_JPRB, 0.13678E+01_JPRB, 0.13073E+01_JPRB, 0.14764E+01_JPRB ,&
&  0.15216E+01_JPRB, 0.14943E+01_JPRB, 0.13526E+01_JPRB, 0.13495E+01_JPRB, 0.12781E+01_JPRB, 0.12142E+01_JPRB, 0.11421E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.99090E+00_JPRB, 0.92995E+00_JPRB, 0.87433E+00_JPRB, 0.82728E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.70166E+00_JPRB, 0.68190E+00_JPRB, 0.67238E+00_JPRB, 0.66846E+00_JPRB, 0.67117E+00_JPRB, 0.68258E+00_JPRB, 0.68158E+00_JPRB ,&
&  0.67160E+00_JPRB, 0.65261E+00_JPRB, 0.62896E+00_JPRB, 0.60975E+00_JPRB, 0.59980E+00_JPRB, 0.60359E+00_JPRB, 0.60666E+00_JPRB ,&
&  0.59705E+00_JPRB, 0.59722E+00_JPRB, 0.47372E+00_JPRB, 0.42065E+00_JPRB, 0.41347E+00_JPRB, 0.49434E+00_JPRB, 0.63260E+00_JPRB ,&
&  0.74184E+00_JPRB, 0.82745E+00_JPRB, 0.10383E+01_JPRB, 0.13828E+01_JPRB, 0.16626E+01_JPRB, 0.18149E+01_JPRB, 0.19690E+01_JPRB ,&
&  0.23379E+01_JPRB, 0.18394E+01_JPRB, 0.17585E+01_JPRB, 0.15618E+01_JPRB, 0.14105E+01_JPRB, 0.13045E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.11093E+01_JPRB, 0.10478E+01_JPRB, 0.96568E+00_JPRB, 0.92557E+00_JPRB, 0.91036E+00_JPRB, 0.91669E+00_JPRB, 0.96687E+00_JPRB ,&
&  0.10338E+01_JPRB, 0.10864E+01_JPRB, 0.10764E+01_JPRB, 0.10736E+01_JPRB, 0.11189E+01_JPRB, 0.11588E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.14593E+01_JPRB, 0.20976E+01_JPRB, 0.24567E+01_JPRB, 0.28526E+01_JPRB, 0.30375E+01_JPRB, 0.31522E+01_JPRB ,&
&  0.33639E+01_JPRB, 0.33227E+01_JPRB, 0.33788E+01_JPRB, 0.30422E+01_JPRB, 0.34597E+01_JPRB, 0.34146E+01_JPRB, 0.31590E+01_JPRB ,&
&  0.28532E+01_JPRB, 0.26548E+01_JPRB, 0.23143E+01_JPRB, 0.21708E+01_JPRB, 0.21582E+01_JPRB, 0.21924E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.12100E+01_JPRB, 0.12908E+01_JPRB, 0.10732E+01_JPRB, 0.92106E+00_JPRB, 0.79351E+00_JPRB, 0.82975E+00_JPRB, 0.87109E+00_JPRB ,&
&  0.90675E+00_JPRB, 0.95349E+00_JPRB, 0.94188E+00_JPRB, 0.10411E+01_JPRB, 0.12202E+01_JPRB, 0.13169E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.13211E+01_JPRB, 0.13523E+01_JPRB, 0.13025E+01_JPRB, 0.12220E+01_JPRB, 0.12077E+01_JPRB, 0.11643E+01_JPRB, 0.10888E+01_JPRB ,&
&  0.99579E+00_JPRB, 0.91507E+00_JPRB, 0.84686E+00_JPRB, 0.78598E+00_JPRB, 0.73646E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.67459E+00_JPRB, 0.65738E+00_JPRB, 0.64585E+00_JPRB, 0.64009E+00_JPRB, 0.64094E+00_JPRB, 0.65333E+00_JPRB, 0.65804E+00_JPRB ,&
&  0.65118E+00_JPRB, 0.64227E+00_JPRB, 0.62303E+00_JPRB, 0.60514E+00_JPRB, 0.59310E+00_JPRB, 0.59435E+00_JPRB, 0.59722E+00_JPRB ,&
&  0.60468E+00_JPRB, 0.74472E+00_JPRB, 0.56298E+00_JPRB, 0.51355E+00_JPRB, 0.48157E+00_JPRB, 0.57696E+00_JPRB, 0.66707E+00_JPRB ,&
&  0.71814E+00_JPRB, 0.75710E+00_JPRB, 0.81846E+00_JPRB, 0.10707E+01_JPRB, 0.13189E+01_JPRB, 0.13546E+01_JPRB, 0.15167E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.15582E+01_JPRB, 0.13828E+01_JPRB, 0.13972E+01_JPRB, 0.12266E+01_JPRB, 0.10638E+01_JPRB, 0.11016E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.99210E+00_JPRB, 0.94104E+00_JPRB, 0.88168E+00_JPRB, 0.85719E+00_JPRB, 0.84216E+00_JPRB, 0.85038E+00_JPRB ,&
&  0.87808E+00_JPRB, 0.91507E+00_JPRB, 0.96877E+00_JPRB, 0.10189E+01_JPRB, 0.10750E+01_JPRB, 0.12195E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.17409E+01_JPRB, 0.18764E+01_JPRB, 0.17067E+01_JPRB, 0.23382E+01_JPRB, 0.35228E+01_JPRB, 0.35097E+01_JPRB, 0.32378E+01_JPRB ,&
&  0.34566E+01_JPRB, 0.34594E+01_JPRB, 0.35349E+01_JPRB, 0.36688E+01_JPRB, 0.36564E+01_JPRB, 0.35098E+01_JPRB, 0.32160E+01_JPRB ,&
&  0.28234E+01_JPRB, 0.25383E+01_JPRB, 0.23070E+01_JPRB, 0.22734E+01_JPRB, 0.20349E+01_JPRB, 0.18785E+01_JPRB, 0.17471E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.11971E+01_JPRB, 0.83674E+00_JPRB, 0.88257E+00_JPRB, 0.69667E+00_JPRB, 0.71902E+00_JPRB, 0.83629E+00_JPRB ,&
&  0.82765E+00_JPRB, 0.92641E+00_JPRB, 0.98318E+00_JPRB, 0.95549E+00_JPRB, 0.11149E+01_JPRB, 0.12156E+01_JPRB, 0.13365E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.10979E+01_JPRB, 0.11245E+01_JPRB, 0.10566E+01_JPRB, 0.10365E+01_JPRB, 0.99036E+00_JPRB, 0.93786E+00_JPRB ,&
&  0.87791E+00_JPRB, 0.81463E+00_JPRB, 0.76240E+00_JPRB, 0.72283E+00_JPRB, 0.69610E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.61324E+00_JPRB, 0.59673E+00_JPRB, 0.57965E+00_JPRB, 0.57881E+00_JPRB, 0.59166E+00_JPRB, 0.60968E+00_JPRB, 0.62111E+00_JPRB ,&
&  0.61675E+00_JPRB, 0.61403E+00_JPRB, 0.60171E+00_JPRB, 0.58801E+00_JPRB, 0.57757E+00_JPRB, 0.56907E+00_JPRB, 0.57652E+00_JPRB ,&
&  0.64164E+00_JPRB, 0.78388E+00_JPRB, 0.62705E+00_JPRB, 0.52365E+00_JPRB, 0.61280E+00_JPRB, 0.62499E+00_JPRB, 0.68914E+00_JPRB ,&
&  0.67306E+00_JPRB, 0.74920E+00_JPRB, 0.76409E+00_JPRB, 0.82978E+00_JPRB, 0.10128E+01_JPRB, 0.10966E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11524E+01_JPRB, 0.11044E+01_JPRB, 0.11047E+01_JPRB, 0.11277E+01_JPRB, 0.11431E+01_JPRB, 0.10092E+01_JPRB, 0.97551E+00_JPRB ,&
&  0.91525E+00_JPRB, 0.87358E+00_JPRB, 0.82473E+00_JPRB, 0.80177E+00_JPRB, 0.79362E+00_JPRB, 0.79589E+00_JPRB, 0.81387E+00_JPRB ,&
&  0.83625E+00_JPRB, 0.85971E+00_JPRB, 0.88331E+00_JPRB, 0.94165E+00_JPRB, 0.10187E+01_JPRB, 0.12827E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.22002E+01_JPRB, 0.25485E+01_JPRB, 0.28873E+01_JPRB, 0.34804E+01_JPRB, 0.35967E+01_JPRB, 0.35222E+01_JPRB, 0.34427E+01_JPRB ,&
&  0.33793E+01_JPRB, 0.32852E+01_JPRB, 0.33238E+01_JPRB, 0.33464E+01_JPRB, 0.33830E+01_JPRB, 0.33631E+01_JPRB, 0.31126E+01_JPRB ,&
&  0.27373E+01_JPRB, 0.24851E+01_JPRB, 0.23522E+01_JPRB, 0.21447E+01_JPRB, 0.19652E+01_JPRB, 0.17737E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.18248E+01_JPRB, 0.10122E+01_JPRB, 0.77807E+00_JPRB, 0.91075E+00_JPRB, 0.76021E+00_JPRB, 0.80256E+00_JPRB, 0.95411E+00_JPRB ,&
&  0.95495E+00_JPRB, 0.94186E+00_JPRB, 0.97375E+00_JPRB, 0.92751E+00_JPRB, 0.92851E+00_JPRB, 0.11054E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.99721E+00_JPRB, 0.93453E+00_JPRB, 0.94856E+00_JPRB, 0.91204E+00_JPRB, 0.88449E+00_JPRB, 0.83852E+00_JPRB, 0.77980E+00_JPRB ,&
&  0.73231E+00_JPRB, 0.70400E+00_JPRB, 0.67189E+00_JPRB, 0.64645E+00_JPRB, 0.62876E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.57061E+00_JPRB, 0.56116E+00_JPRB, 0.54528E+00_JPRB, 0.54568E+00_JPRB, 0.56595E+00_JPRB, 0.58810E+00_JPRB, 0.60113E+00_JPRB ,&
&  0.59927E+00_JPRB, 0.60042E+00_JPRB, 0.58995E+00_JPRB, 0.57969E+00_JPRB, 0.56960E+00_JPRB, 0.54965E+00_JPRB, 0.56640E+00_JPRB ,&
&  0.48124E+00_JPRB, 0.60569E+00_JPRB, 0.50620E+00_JPRB, 0.51962E+00_JPRB, 0.69515E+00_JPRB, 0.64092E+00_JPRB, 0.65099E+00_JPRB ,&
&  0.63668E+00_JPRB, 0.66566E+00_JPRB, 0.67811E+00_JPRB, 0.72838E+00_JPRB, 0.87758E+00_JPRB, 0.97091E+00_JPRB, 0.97259E+00_JPRB ,&
&  0.91262E+00_JPRB, 0.86143E+00_JPRB, 0.89315E+00_JPRB, 0.91339E+00_JPRB, 0.91301E+00_JPRB, 0.93829E+00_JPRB, 0.93793E+00_JPRB ,&
&  0.87247E+00_JPRB, 0.82354E+00_JPRB, 0.76689E+00_JPRB, 0.74092E+00_JPRB, 0.72902E+00_JPRB, 0.72417E+00_JPRB, 0.71883E+00_JPRB ,&
&  0.72690E+00_JPRB, 0.75156E+00_JPRB, 0.78597E+00_JPRB, 0.87560E+00_JPRB, 0.97150E+00_JPRB, 0.15893E+01_JPRB, 0.25396E+01_JPRB ,&
&  0.29519E+01_JPRB, 0.29626E+01_JPRB, 0.32291E+01_JPRB, 0.37241E+01_JPRB, 0.36376E+01_JPRB, 0.34383E+01_JPRB, 0.31825E+01_JPRB ,&
&  0.31008E+01_JPRB, 0.30341E+01_JPRB, 0.30455E+01_JPRB, 0.31312E+01_JPRB, 0.30989E+01_JPRB, 0.31859E+01_JPRB, 0.30312E+01_JPRB ,&
&  0.26760E+01_JPRB, 0.25086E+01_JPRB, 0.24343E+01_JPRB, 0.23809E+01_JPRB, 0.22506E+01_JPRB, 0.22450E+01_JPRB, 0.20946E+01_JPRB ,&
&  0.20221E+01_JPRB, 0.17265E+01_JPRB, 0.12602E+01_JPRB, 0.11273E+01_JPRB, 0.87571E+00_JPRB, 0.90585E+00_JPRB, 0.11865E+01_JPRB ,&
&  0.11653E+01_JPRB, 0.10264E+01_JPRB, 0.91423E+00_JPRB, 0.84579E+00_JPRB, 0.82103E+00_JPRB, 0.93274E+00_JPRB, 0.88833E+00_JPRB ,&
&  0.78237E+00_JPRB, 0.81030E+00_JPRB, 0.81665E+00_JPRB, 0.80438E+00_JPRB, 0.76580E+00_JPRB, 0.71597E+00_JPRB, 0.64238E+00_JPRB ,&
&  0.61704E+00_JPRB, 0.59631E+00_JPRB, 0.58765E+00_JPRB, 0.58369E+00_JPRB, 0.57801E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.55756E+00_JPRB, 0.54620E+00_JPRB, 0.53095E+00_JPRB, 0.53307E+00_JPRB, 0.55431E+00_JPRB, 0.57531E+00_JPRB, 0.58822E+00_JPRB ,&
&  0.58692E+00_JPRB, 0.58604E+00_JPRB, 0.58081E+00_JPRB, 0.57312E+00_JPRB, 0.55760E+00_JPRB, 0.53810E+00_JPRB, 0.48723E+00_JPRB ,&
&  0.42925E+00_JPRB, 0.51672E+00_JPRB, 0.48985E+00_JPRB, 0.51216E+00_JPRB, 0.58957E+00_JPRB, 0.56583E+00_JPRB, 0.55387E+00_JPRB ,&
&  0.56817E+00_JPRB, 0.57950E+00_JPRB, 0.62174E+00_JPRB, 0.66919E+00_JPRB, 0.74399E+00_JPRB, 0.88105E+00_JPRB, 0.89272E+00_JPRB ,&
&  0.79704E+00_JPRB, 0.74751E+00_JPRB, 0.77346E+00_JPRB, 0.83148E+00_JPRB, 0.88995E+00_JPRB, 0.93703E+00_JPRB, 0.91920E+00_JPRB ,&
&  0.87436E+00_JPRB, 0.82203E+00_JPRB, 0.74514E+00_JPRB, 0.69358E+00_JPRB, 0.66464E+00_JPRB, 0.64917E+00_JPRB, 0.64182E+00_JPRB ,&
&  0.64162E+00_JPRB, 0.65918E+00_JPRB, 0.70154E+00_JPRB, 0.77158E+00_JPRB, 0.93305E+00_JPRB, 0.14737E+01_JPRB, 0.19481E+01_JPRB ,&
&  0.20287E+01_JPRB, 0.21325E+01_JPRB, 0.23312E+01_JPRB, 0.25245E+01_JPRB, 0.27193E+01_JPRB, 0.27138E+01_JPRB, 0.27187E+01_JPRB ,&
&  0.27837E+01_JPRB, 0.27431E+01_JPRB, 0.30376E+01_JPRB, 0.30459E+01_JPRB, 0.29968E+01_JPRB, 0.29989E+01_JPRB, 0.28288E+01_JPRB ,&
&  0.26746E+01_JPRB, 0.25428E+01_JPRB, 0.25201E+01_JPRB, 0.24251E+01_JPRB, 0.23286E+01_JPRB, 0.23198E+01_JPRB, 0.21622E+01_JPRB ,&
&  0.21100E+01_JPRB, 0.20281E+01_JPRB, 0.17184E+01_JPRB, 0.14698E+01_JPRB, 0.12147E+01_JPRB, 0.12332E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.85011E+00_JPRB, 0.83074E+00_JPRB, 0.79897E+00_JPRB, 0.79913E+00_JPRB, 0.78649E+00_JPRB, 0.72631E+00_JPRB ,&
&  0.72370E+00_JPRB, 0.74324E+00_JPRB, 0.70282E+00_JPRB, 0.68027E+00_JPRB, 0.64082E+00_JPRB, 0.60567E+00_JPRB, 0.52074E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.52461E+00_JPRB, 0.53999E+00_JPRB, 0.55489E+00_JPRB, 0.56109E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.56252E+00_JPRB, 0.55214E+00_JPRB, 0.54649E+00_JPRB, 0.54320E+00_JPRB, 0.54882E+00_JPRB, 0.56905E+00_JPRB, 0.55594E+00_JPRB ,&
&  0.58216E+00_JPRB, 0.58575E+00_JPRB, 0.57573E+00_JPRB, 0.56579E+00_JPRB, 0.54253E+00_JPRB, 0.46712E+00_JPRB, 0.37879E+00_JPRB ,&
&  0.36763E+00_JPRB, 0.41058E+00_JPRB, 0.45527E+00_JPRB, 0.47837E+00_JPRB, 0.51303E+00_JPRB, 0.51447E+00_JPRB, 0.51251E+00_JPRB ,&
&  0.51327E+00_JPRB, 0.52179E+00_JPRB, 0.55276E+00_JPRB, 0.60682E+00_JPRB, 0.66115E+00_JPRB, 0.74348E+00_JPRB, 0.82452E+00_JPRB ,&
&  0.76297E+00_JPRB, 0.73148E+00_JPRB, 0.75828E+00_JPRB, 0.73566E+00_JPRB, 0.83407E+00_JPRB, 0.85723E+00_JPRB, 0.84847E+00_JPRB ,&
&  0.81949E+00_JPRB, 0.77739E+00_JPRB, 0.72427E+00_JPRB, 0.69132E+00_JPRB, 0.66003E+00_JPRB, 0.63354E+00_JPRB, 0.61892E+00_JPRB ,&
&  0.62481E+00_JPRB, 0.63964E+00_JPRB, 0.67348E+00_JPRB, 0.72937E+00_JPRB, 0.80281E+00_JPRB, 0.11118E+01_JPRB, 0.13544E+01_JPRB ,&
&  0.15093E+01_JPRB, 0.17640E+01_JPRB, 0.20309E+01_JPRB, 0.19668E+01_JPRB, 0.21405E+01_JPRB, 0.22884E+01_JPRB, 0.23162E+01_JPRB ,&
&  0.24785E+01_JPRB, 0.24555E+01_JPRB, 0.25428E+01_JPRB, 0.25689E+01_JPRB, 0.25808E+01_JPRB, 0.25470E+01_JPRB, 0.24593E+01_JPRB ,&
&  0.24128E+01_JPRB, 0.25595E+01_JPRB, 0.25899E+01_JPRB, 0.24740E+01_JPRB, 0.22577E+01_JPRB, 0.21051E+01_JPRB, 0.19342E+01_JPRB ,&
&  0.18531E+01_JPRB, 0.18338E+01_JPRB, 0.17601E+01_JPRB, 0.16778E+01_JPRB, 0.15380E+01_JPRB, 0.13819E+01_JPRB, 0.12500E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.93348E+00_JPRB, 0.83809E+00_JPRB, 0.81053E+00_JPRB, 0.79251E+00_JPRB, 0.71388E+00_JPRB, 0.68894E+00_JPRB ,&
&  0.68189E+00_JPRB, 0.63279E+00_JPRB, 0.64265E+00_JPRB, 0.60623E+00_JPRB, 0.57652E+00_JPRB, 0.55849E+00_JPRB, 0.53617E+00_JPRB ,&
&  0.51024E+00_JPRB, 0.54875E+00_JPRB, 0.56799E+00_JPRB, 0.57494E+00_JPRB, 0.57154E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.56593E+00_JPRB, 0.56708E+00_JPRB, 0.56718E+00_JPRB, 0.55739E+00_JPRB, 0.54989E+00_JPRB, 0.53593E+00_JPRB, 0.53970E+00_JPRB ,&
&  0.57019E+00_JPRB, 0.52682E+00_JPRB, 0.56559E+00_JPRB, 0.45603E+00_JPRB, 0.40375E+00_JPRB, 0.38064E+00_JPRB, 0.36407E+00_JPRB ,&
&  0.36958E+00_JPRB, 0.39783E+00_JPRB, 0.42298E+00_JPRB, 0.44035E+00_JPRB, 0.47633E+00_JPRB, 0.48232E+00_JPRB, 0.48091E+00_JPRB ,&
&  0.48641E+00_JPRB, 0.49299E+00_JPRB, 0.52451E+00_JPRB, 0.53827E+00_JPRB, 0.57643E+00_JPRB, 0.63187E+00_JPRB, 0.71666E+00_JPRB ,&
&  0.68684E+00_JPRB, 0.69565E+00_JPRB, 0.76176E+00_JPRB, 0.78466E+00_JPRB, 0.80091E+00_JPRB, 0.78050E+00_JPRB, 0.74188E+00_JPRB ,&
&  0.71145E+00_JPRB, 0.59033E+00_JPRB, 0.60363E+00_JPRB, 0.58707E+00_JPRB, 0.57124E+00_JPRB, 0.55375E+00_JPRB, 0.52699E+00_JPRB ,&
&  0.51009E+00_JPRB, 0.51310E+00_JPRB, 0.53377E+00_JPRB, 0.56612E+00_JPRB, 0.62185E+00_JPRB, 0.76036E+00_JPRB, 0.92062E+00_JPRB ,&
&  0.11438E+01_JPRB, 0.10645E+01_JPRB, 0.13546E+01_JPRB, 0.14876E+01_JPRB, 0.16662E+01_JPRB, 0.18920E+01_JPRB, 0.20615E+01_JPRB ,&
&  0.23031E+01_JPRB, 0.22302E+01_JPRB, 0.22009E+01_JPRB, 0.21431E+01_JPRB, 0.20724E+01_JPRB, 0.20363E+01_JPRB, 0.20448E+01_JPRB ,&
&  0.20953E+01_JPRB, 0.20680E+01_JPRB, 0.22101E+01_JPRB, 0.21516E+01_JPRB, 0.19909E+01_JPRB, 0.18800E+01_JPRB, 0.17570E+01_JPRB ,&
&  0.16481E+01_JPRB, 0.15724E+01_JPRB, 0.14977E+01_JPRB, 0.13221E+01_JPRB, 0.12742E+01_JPRB, 0.12409E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.10102E+01_JPRB, 0.91336E+00_JPRB, 0.92016E+00_JPRB, 0.84100E+00_JPRB, 0.77574E+00_JPRB, 0.75658E+00_JPRB ,&
&  0.70210E+00_JPRB, 0.59621E+00_JPRB, 0.57858E+00_JPRB, 0.57185E+00_JPRB, 0.54304E+00_JPRB, 0.52851E+00_JPRB, 0.56600E+00_JPRB ,&
&  0.55578E+00_JPRB, 0.56018E+00_JPRB, 0.59132E+00_JPRB, 0.58097E+00_JPRB, 0.57780E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.56260E+00_JPRB, 0.57082E+00_JPRB, 0.57976E+00_JPRB, 0.56802E+00_JPRB, 0.54799E+00_JPRB, 0.52840E+00_JPRB, 0.51513E+00_JPRB ,&
&  0.51432E+00_JPRB, 0.44525E+00_JPRB, 0.41258E+00_JPRB, 0.39572E+00_JPRB, 0.38760E+00_JPRB, 0.37088E+00_JPRB, 0.35556E+00_JPRB ,&
&  0.35595E+00_JPRB, 0.37013E+00_JPRB, 0.39714E+00_JPRB, 0.41807E+00_JPRB, 0.43676E+00_JPRB, 0.45055E+00_JPRB, 0.45733E+00_JPRB ,&
&  0.46530E+00_JPRB, 0.47545E+00_JPRB, 0.48432E+00_JPRB, 0.49619E+00_JPRB, 0.50990E+00_JPRB, 0.53595E+00_JPRB, 0.57162E+00_JPRB ,&
&  0.60195E+00_JPRB, 0.61769E+00_JPRB, 0.59508E+00_JPRB, 0.60102E+00_JPRB, 0.61114E+00_JPRB, 0.61076E+00_JPRB, 0.60689E+00_JPRB ,&
&  0.42032E+00_JPRB, 0.39735E+00_JPRB, 0.46040E+00_JPRB, 0.45649E+00_JPRB, 0.46174E+00_JPRB, 0.45881E+00_JPRB, 0.45447E+00_JPRB ,&
&  0.45759E+00_JPRB, 0.47191E+00_JPRB, 0.50532E+00_JPRB, 0.56128E+00_JPRB, 0.62243E+00_JPRB, 0.69753E+00_JPRB, 0.80483E+00_JPRB ,&
&  0.10176E+01_JPRB, 0.99249E+00_JPRB, 0.10336E+01_JPRB, 0.11734E+01_JPRB, 0.14488E+01_JPRB, 0.16443E+01_JPRB, 0.17323E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.17557E+01_JPRB, 0.17370E+01_JPRB, 0.17675E+01_JPRB, 0.17726E+01_JPRB, 0.17520E+01_JPRB, 0.17672E+01_JPRB ,&
&  0.17483E+01_JPRB, 0.17507E+01_JPRB, 0.17683E+01_JPRB, 0.17916E+01_JPRB, 0.17175E+01_JPRB, 0.16564E+01_JPRB, 0.15872E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.13990E+01_JPRB, 0.12750E+01_JPRB, 0.11752E+01_JPRB, 0.11000E+01_JPRB, 0.10837E+01_JPRB, 0.10714E+01_JPRB ,&
&  0.10398E+01_JPRB, 0.10171E+01_JPRB, 0.98313E+00_JPRB, 0.94711E+00_JPRB, 0.87805E+00_JPRB, 0.80615E+00_JPRB, 0.76921E+00_JPRB ,&
&  0.66741E+00_JPRB, 0.59653E+00_JPRB, 0.51496E+00_JPRB, 0.50292E+00_JPRB, 0.52140E+00_JPRB, 0.52912E+00_JPRB, 0.52656E+00_JPRB ,&
&  0.54549E+00_JPRB, 0.54962E+00_JPRB, 0.54079E+00_JPRB, 0.54723E+00_JPRB, 0.55331E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.54310E+00_JPRB, 0.55129E+00_JPRB, 0.55756E+00_JPRB, 0.55962E+00_JPRB, 0.53992E+00_JPRB, 0.52488E+00_JPRB, 0.51312E+00_JPRB ,&
&  0.51068E+00_JPRB, 0.46357E+00_JPRB, 0.43096E+00_JPRB, 0.40704E+00_JPRB, 0.39131E+00_JPRB, 0.37598E+00_JPRB, 0.37217E+00_JPRB ,&
&  0.37650E+00_JPRB, 0.38495E+00_JPRB, 0.39666E+00_JPRB, 0.40884E+00_JPRB, 0.41905E+00_JPRB, 0.42381E+00_JPRB, 0.43144E+00_JPRB ,&
&  0.44329E+00_JPRB, 0.45331E+00_JPRB, 0.45663E+00_JPRB, 0.46029E+00_JPRB, 0.46428E+00_JPRB, 0.47689E+00_JPRB, 0.48184E+00_JPRB ,&
&  0.48224E+00_JPRB, 0.47556E+00_JPRB, 0.47132E+00_JPRB, 0.48405E+00_JPRB, 0.48937E+00_JPRB, 0.48744E+00_JPRB, 0.43536E+00_JPRB ,&
&  0.35663E+00_JPRB, 0.32653E+00_JPRB, 0.33742E+00_JPRB, 0.40294E+00_JPRB, 0.42037E+00_JPRB, 0.43194E+00_JPRB, 0.44634E+00_JPRB ,&
&  0.43642E+00_JPRB, 0.43456E+00_JPRB, 0.46836E+00_JPRB, 0.55713E+00_JPRB, 0.61965E+00_JPRB, 0.68785E+00_JPRB, 0.76948E+00_JPRB ,&
&  0.90474E+00_JPRB, 0.10163E+01_JPRB, 0.11320E+01_JPRB, 0.10881E+01_JPRB, 0.12135E+01_JPRB, 0.15413E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.14580E+01_JPRB, 0.15394E+01_JPRB, 0.15776E+01_JPRB, 0.15720E+01_JPRB, 0.15319E+01_JPRB, 0.15019E+01_JPRB ,&
&  0.14635E+01_JPRB, 0.14359E+01_JPRB, 0.14276E+01_JPRB, 0.15006E+01_JPRB, 0.15014E+01_JPRB, 0.14677E+01_JPRB, 0.14300E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.12392E+01_JPRB, 0.11259E+01_JPRB, 0.10344E+01_JPRB, 0.10026E+01_JPRB, 0.10006E+01_JPRB, 0.99333E+00_JPRB ,&
&  0.96600E+00_JPRB, 0.95458E+00_JPRB, 0.94706E+00_JPRB, 0.93581E+00_JPRB, 0.84092E+00_JPRB, 0.73110E+00_JPRB, 0.64745E+00_JPRB ,&
&  0.58864E+00_JPRB, 0.55025E+00_JPRB, 0.51636E+00_JPRB, 0.51580E+00_JPRB, 0.52043E+00_JPRB, 0.52728E+00_JPRB, 0.50984E+00_JPRB ,&
&  0.49388E+00_JPRB, 0.48055E+00_JPRB, 0.48300E+00_JPRB, 0.50664E+00_JPRB, 0.52730E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.48584E+00_JPRB, 0.50353E+00_JPRB, 0.53117E+00_JPRB, 0.54352E+00_JPRB, 0.54613E+00_JPRB, 0.51801E+00_JPRB, 0.49006E+00_JPRB ,&
&  0.46480E+00_JPRB, 0.43783E+00_JPRB, 0.42241E+00_JPRB, 0.41378E+00_JPRB, 0.41007E+00_JPRB, 0.40878E+00_JPRB, 0.40945E+00_JPRB ,&
&  0.41150E+00_JPRB, 0.41271E+00_JPRB, 0.41721E+00_JPRB, 0.42760E+00_JPRB, 0.43701E+00_JPRB, 0.43961E+00_JPRB, 0.44055E+00_JPRB ,&
&  0.44295E+00_JPRB, 0.44386E+00_JPRB, 0.43849E+00_JPRB, 0.42824E+00_JPRB, 0.42545E+00_JPRB, 0.43807E+00_JPRB, 0.44366E+00_JPRB ,&
&  0.44283E+00_JPRB, 0.43100E+00_JPRB, 0.43015E+00_JPRB, 0.44124E+00_JPRB, 0.43986E+00_JPRB, 0.43015E+00_JPRB, 0.39038E+00_JPRB ,&
&  0.34094E+00_JPRB, 0.30225E+00_JPRB, 0.28866E+00_JPRB, 0.30347E+00_JPRB, 0.33861E+00_JPRB, 0.39797E+00_JPRB, 0.41347E+00_JPRB ,&
&  0.42525E+00_JPRB, 0.44093E+00_JPRB, 0.46990E+00_JPRB, 0.51808E+00_JPRB, 0.58360E+00_JPRB, 0.65131E+00_JPRB, 0.71428E+00_JPRB ,&
&  0.79563E+00_JPRB, 0.88192E+00_JPRB, 0.10008E+01_JPRB, 0.95130E+00_JPRB, 0.99966E+00_JPRB, 0.11192E+01_JPRB, 0.11935E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.12730E+01_JPRB, 0.13170E+01_JPRB, 0.13679E+01_JPRB, 0.13440E+01_JPRB, 0.13064E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.12265E+01_JPRB, 0.12246E+01_JPRB, 0.12702E+01_JPRB, 0.13360E+01_JPRB, 0.13525E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.12456E+01_JPRB, 0.11411E+01_JPRB, 0.10280E+01_JPRB, 0.94129E+00_JPRB, 0.92521E+00_JPRB, 0.94968E+00_JPRB, 0.96946E+00_JPRB ,&
&  0.97077E+00_JPRB, 0.96076E+00_JPRB, 0.92333E+00_JPRB, 0.89019E+00_JPRB, 0.78995E+00_JPRB, 0.71674E+00_JPRB, 0.66177E+00_JPRB ,&
&  0.63814E+00_JPRB, 0.61661E+00_JPRB, 0.59659E+00_JPRB, 0.60309E+00_JPRB, 0.58660E+00_JPRB, 0.57332E+00_JPRB, 0.55705E+00_JPRB ,&
&  0.53381E+00_JPRB, 0.50870E+00_JPRB, 0.49623E+00_JPRB, 0.48555E+00_JPRB, 0.48231E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.53390E+00_JPRB, 0.53145E+00_JPRB, 0.52913E+00_JPRB, 0.53234E+00_JPRB, 0.53688E+00_JPRB, 0.52974E+00_JPRB, 0.51795E+00_JPRB ,&
&  0.49979E+00_JPRB, 0.48168E+00_JPRB, 0.46784E+00_JPRB, 0.45734E+00_JPRB, 0.45232E+00_JPRB, 0.45069E+00_JPRB, 0.45035E+00_JPRB ,&
&  0.45115E+00_JPRB, 0.45288E+00_JPRB, 0.45250E+00_JPRB, 0.44793E+00_JPRB, 0.44611E+00_JPRB, 0.44701E+00_JPRB, 0.44876E+00_JPRB ,&
&  0.45026E+00_JPRB, 0.44731E+00_JPRB, 0.44154E+00_JPRB, 0.43508E+00_JPRB, 0.42662E+00_JPRB, 0.43273E+00_JPRB, 0.42621E+00_JPRB ,&
&  0.41627E+00_JPRB, 0.41426E+00_JPRB, 0.42246E+00_JPRB, 0.42160E+00_JPRB, 0.41659E+00_JPRB, 0.40384E+00_JPRB, 0.36199E+00_JPRB ,&
&  0.32120E+00_JPRB, 0.28981E+00_JPRB, 0.27297E+00_JPRB, 0.26928E+00_JPRB, 0.27868E+00_JPRB, 0.30361E+00_JPRB, 0.34526E+00_JPRB ,&
&  0.40439E+00_JPRB, 0.42331E+00_JPRB, 0.44579E+00_JPRB, 0.47849E+00_JPRB, 0.52137E+00_JPRB, 0.57281E+00_JPRB, 0.62107E+00_JPRB ,&
&  0.66497E+00_JPRB, 0.71801E+00_JPRB, 0.79131E+00_JPRB, 0.87279E+00_JPRB, 0.97292E+00_JPRB, 0.96992E+00_JPRB, 0.10368E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.12299E+01_JPRB, 0.12004E+01_JPRB, 0.11274E+01_JPRB, 0.10765E+01_JPRB, 0.10403E+01_JPRB, 0.10278E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10660E+01_JPRB, 0.11198E+01_JPRB, 0.11723E+01_JPRB, 0.11947E+01_JPRB, 0.12006E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.11410E+01_JPRB, 0.13671E+01_JPRB, 0.12372E+01_JPRB, 0.11380E+01_JPRB, 0.10986E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.96498E+00_JPRB, 0.91198E+00_JPRB, 0.86401E+00_JPRB, 0.79994E+00_JPRB, 0.74404E+00_JPRB, 0.69841E+00_JPRB ,&
&  0.68362E+00_JPRB, 0.67286E+00_JPRB, 0.65908E+00_JPRB, 0.64827E+00_JPRB, 0.63134E+00_JPRB, 0.61776E+00_JPRB, 0.60245E+00_JPRB ,&
&  0.57928E+00_JPRB, 0.56065E+00_JPRB, 0.55126E+00_JPRB, 0.54333E+00_JPRB, 0.53747E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.53704E+00_JPRB, 0.53194E+00_JPRB, 0.52716E+00_JPRB, 0.52267E+00_JPRB, 0.52083E+00_JPRB, 0.51928E+00_JPRB, 0.51573E+00_JPRB ,&
&  0.50891E+00_JPRB, 0.50356E+00_JPRB, 0.49714E+00_JPRB, 0.49434E+00_JPRB, 0.49148E+00_JPRB, 0.48886E+00_JPRB, 0.48654E+00_JPRB ,&
&  0.48282E+00_JPRB, 0.47121E+00_JPRB, 0.46391E+00_JPRB, 0.46142E+00_JPRB, 0.46080E+00_JPRB, 0.46046E+00_JPRB, 0.46044E+00_JPRB ,&
&  0.45584E+00_JPRB, 0.44780E+00_JPRB, 0.44098E+00_JPRB, 0.43015E+00_JPRB, 0.42278E+00_JPRB, 0.42054E+00_JPRB, 0.42405E+00_JPRB ,&
&  0.42521E+00_JPRB, 0.41133E+00_JPRB, 0.40927E+00_JPRB, 0.40592E+00_JPRB, 0.39381E+00_JPRB, 0.38785E+00_JPRB, 0.33461E+00_JPRB ,&
&  0.30040E+00_JPRB, 0.27857E+00_JPRB, 0.27054E+00_JPRB, 0.26669E+00_JPRB, 0.27468E+00_JPRB, 0.29463E+00_JPRB, 0.33510E+00_JPRB ,&
&  0.38046E+00_JPRB, 0.43166E+00_JPRB, 0.44845E+00_JPRB, 0.47435E+00_JPRB, 0.50062E+00_JPRB, 0.53747E+00_JPRB, 0.56779E+00_JPRB ,&
&  0.58396E+00_JPRB, 0.59571E+00_JPRB, 0.62007E+00_JPRB, 0.64180E+00_JPRB, 0.68288E+00_JPRB, 0.71923E+00_JPRB, 0.73845E+00_JPRB ,&
&  0.76024E+00_JPRB, 0.76896E+00_JPRB, 0.79086E+00_JPRB, 0.80504E+00_JPRB, 0.81579E+00_JPRB, 0.81413E+00_JPRB, 0.81477E+00_JPRB ,&
&  0.82360E+00_JPRB, 0.87367E+00_JPRB, 0.95006E+00_JPRB, 0.10398E+01_JPRB, 0.10751E+01_JPRB, 0.10879E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10434E+01_JPRB, 0.11737E+01_JPRB, 0.11555E+01_JPRB, 0.11419E+01_JPRB, 0.11200E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.98692E+00_JPRB, 0.95448E+00_JPRB, 0.92033E+00_JPRB, 0.85874E+00_JPRB, 0.80229E+00_JPRB, 0.76704E+00_JPRB, 0.74763E+00_JPRB ,&
&  0.73741E+00_JPRB, 0.72982E+00_JPRB, 0.70942E+00_JPRB, 0.69004E+00_JPRB, 0.66784E+00_JPRB, 0.64085E+00_JPRB, 0.61799E+00_JPRB ,&
&  0.59842E+00_JPRB, 0.58346E+00_JPRB, 0.57004E+00_JPRB, 0.55667E+00_JPRB, 0.54608E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.54278E+00_JPRB, 0.53811E+00_JPRB, 0.53325E+00_JPRB, 0.52897E+00_JPRB, 0.52542E+00_JPRB, 0.52395E+00_JPRB, 0.52362E+00_JPRB ,&
&  0.52153E+00_JPRB, 0.51953E+00_JPRB, 0.51681E+00_JPRB, 0.51408E+00_JPRB, 0.51042E+00_JPRB, 0.50711E+00_JPRB, 0.50366E+00_JPRB ,&
&  0.49886E+00_JPRB, 0.49268E+00_JPRB, 0.48536E+00_JPRB, 0.47781E+00_JPRB, 0.47027E+00_JPRB, 0.46511E+00_JPRB, 0.45915E+00_JPRB ,&
&  0.45304E+00_JPRB, 0.44537E+00_JPRB, 0.43331E+00_JPRB, 0.42284E+00_JPRB, 0.41859E+00_JPRB, 0.41184E+00_JPRB, 0.41208E+00_JPRB ,&
&  0.40671E+00_JPRB, 0.39758E+00_JPRB, 0.38531E+00_JPRB, 0.38083E+00_JPRB, 0.37467E+00_JPRB, 0.32332E+00_JPRB, 0.30863E+00_JPRB ,&
&  0.29144E+00_JPRB, 0.27689E+00_JPRB, 0.27267E+00_JPRB, 0.27380E+00_JPRB, 0.28046E+00_JPRB, 0.29896E+00_JPRB, 0.32585E+00_JPRB ,&
&  0.36077E+00_JPRB, 0.45089E+00_JPRB, 0.47749E+00_JPRB, 0.50660E+00_JPRB, 0.52729E+00_JPRB, 0.54850E+00_JPRB, 0.55692E+00_JPRB ,&
&  0.56454E+00_JPRB, 0.57227E+00_JPRB, 0.59395E+00_JPRB, 0.62493E+00_JPRB, 0.65175E+00_JPRB, 0.66105E+00_JPRB, 0.66013E+00_JPRB ,&
&  0.65076E+00_JPRB, 0.64627E+00_JPRB, 0.63999E+00_JPRB, 0.64254E+00_JPRB, 0.66658E+00_JPRB, 0.69006E+00_JPRB, 0.72480E+00_JPRB ,&
&  0.75956E+00_JPRB, 0.78772E+00_JPRB, 0.83514E+00_JPRB, 0.89232E+00_JPRB, 0.93586E+00_JPRB, 0.96910E+00_JPRB, 0.98050E+00_JPRB ,&
&  0.98234E+00_JPRB, 0.99955E+00_JPRB, 0.99938E+00_JPRB, 0.99413E+00_JPRB, 0.98011E+00_JPRB, 0.95082E+00_JPRB, 0.91765E+00_JPRB ,&
&  0.89900E+00_JPRB, 0.90444E+00_JPRB, 0.88969E+00_JPRB, 0.84770E+00_JPRB, 0.80915E+00_JPRB, 0.77733E+00_JPRB, 0.75230E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.72291E+00_JPRB, 0.70182E+00_JPRB, 0.67876E+00_JPRB, 0.65282E+00_JPRB, 0.62658E+00_JPRB, 0.60982E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.58606E+00_JPRB, 0.57320E+00_JPRB, 0.56040E+00_JPRB, 0.55127E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.54768E+00_JPRB, 0.54536E+00_JPRB, 0.54065E+00_JPRB, 0.53665E+00_JPRB, 0.53150E+00_JPRB, 0.52816E+00_JPRB, 0.52443E+00_JPRB ,&
&  0.52073E+00_JPRB, 0.51719E+00_JPRB, 0.51425E+00_JPRB, 0.51165E+00_JPRB, 0.50821E+00_JPRB, 0.50518E+00_JPRB, 0.50224E+00_JPRB ,&
&  0.49936E+00_JPRB, 0.49460E+00_JPRB, 0.48828E+00_JPRB, 0.48077E+00_JPRB, 0.47373E+00_JPRB, 0.46579E+00_JPRB, 0.45694E+00_JPRB ,&
&  0.44800E+00_JPRB, 0.43670E+00_JPRB, 0.41817E+00_JPRB, 0.40589E+00_JPRB, 0.39246E+00_JPRB, 0.38729E+00_JPRB, 0.38110E+00_JPRB ,&
&  0.38736E+00_JPRB, 0.35286E+00_JPRB, 0.34424E+00_JPRB, 0.32719E+00_JPRB, 0.31349E+00_JPRB, 0.30359E+00_JPRB, 0.29316E+00_JPRB ,&
&  0.28910E+00_JPRB, 0.28557E+00_JPRB, 0.28665E+00_JPRB, 0.29388E+00_JPRB, 0.30534E+00_JPRB, 0.32763E+00_JPRB, 0.35552E+00_JPRB ,&
&  0.39489E+00_JPRB, 0.47446E+00_JPRB, 0.50018E+00_JPRB, 0.52642E+00_JPRB, 0.54820E+00_JPRB, 0.57094E+00_JPRB, 0.57113E+00_JPRB ,&
&  0.58162E+00_JPRB, 0.58971E+00_JPRB, 0.60751E+00_JPRB, 0.62428E+00_JPRB, 0.64685E+00_JPRB, 0.64947E+00_JPRB, 0.64858E+00_JPRB ,&
&  0.63220E+00_JPRB, 0.61881E+00_JPRB, 0.61056E+00_JPRB, 0.62056E+00_JPRB, 0.64654E+00_JPRB, 0.68153E+00_JPRB, 0.71511E+00_JPRB ,&
&  0.74540E+00_JPRB, 0.76784E+00_JPRB, 0.78558E+00_JPRB, 0.81015E+00_JPRB, 0.84171E+00_JPRB, 0.88028E+00_JPRB, 0.90278E+00_JPRB ,&
&  0.91780E+00_JPRB, 0.92539E+00_JPRB, 0.92454E+00_JPRB, 0.91644E+00_JPRB, 0.89455E+00_JPRB, 0.87127E+00_JPRB, 0.85769E+00_JPRB ,&
&  0.85898E+00_JPRB, 0.84823E+00_JPRB, 0.83165E+00_JPRB, 0.80840E+00_JPRB, 0.78985E+00_JPRB, 0.76678E+00_JPRB, 0.74235E+00_JPRB ,&
&  0.72049E+00_JPRB, 0.70273E+00_JPRB, 0.67964E+00_JPRB, 0.65906E+00_JPRB, 0.63503E+00_JPRB, 0.61498E+00_JPRB, 0.60122E+00_JPRB ,&
&  0.58791E+00_JPRB, 0.57714E+00_JPRB, 0.56495E+00_JPRB, 0.55659E+00_JPRB, 0.55153E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.54729E+00_JPRB, 0.54585E+00_JPRB, 0.54318E+00_JPRB, 0.54089E+00_JPRB, 0.53597E+00_JPRB, 0.53228E+00_JPRB, 0.52904E+00_JPRB ,&
&  0.52678E+00_JPRB, 0.52541E+00_JPRB, 0.52404E+00_JPRB, 0.52429E+00_JPRB, 0.52316E+00_JPRB, 0.52243E+00_JPRB, 0.51973E+00_JPRB ,&
&  0.51798E+00_JPRB, 0.51448E+00_JPRB, 0.51032E+00_JPRB, 0.50379E+00_JPRB, 0.49674E+00_JPRB, 0.48797E+00_JPRB, 0.47476E+00_JPRB ,&
&  0.45907E+00_JPRB, 0.44490E+00_JPRB, 0.41503E+00_JPRB, 0.40225E+00_JPRB, 0.38565E+00_JPRB, 0.37608E+00_JPRB, 0.37280E+00_JPRB ,&
&  0.36974E+00_JPRB, 0.36800E+00_JPRB, 0.36244E+00_JPRB, 0.35485E+00_JPRB, 0.34555E+00_JPRB, 0.33289E+00_JPRB, 0.32489E+00_JPRB ,&
&  0.31798E+00_JPRB, 0.31676E+00_JPRB, 0.32086E+00_JPRB, 0.33060E+00_JPRB, 0.34710E+00_JPRB, 0.36964E+00_JPRB, 0.39640E+00_JPRB ,&
&  0.43215E+00_JPRB, 0.46741E+00_JPRB, 0.51011E+00_JPRB, 0.52689E+00_JPRB, 0.54118E+00_JPRB, 0.55707E+00_JPRB, 0.57102E+00_JPRB ,&
&  0.57246E+00_JPRB, 0.57618E+00_JPRB, 0.59345E+00_JPRB, 0.59787E+00_JPRB, 0.60156E+00_JPRB, 0.59952E+00_JPRB, 0.59974E+00_JPRB ,&
&  0.59764E+00_JPRB, 0.60210E+00_JPRB, 0.60886E+00_JPRB, 0.62225E+00_JPRB, 0.64340E+00_JPRB, 0.66953E+00_JPRB, 0.69392E+00_JPRB ,&
&  0.71537E+00_JPRB, 0.73810E+00_JPRB, 0.75519E+00_JPRB, 0.77140E+00_JPRB, 0.78751E+00_JPRB, 0.80826E+00_JPRB, 0.82421E+00_JPRB ,&
&  0.83547E+00_JPRB, 0.83345E+00_JPRB, 0.82408E+00_JPRB, 0.80315E+00_JPRB, 0.79422E+00_JPRB, 0.80291E+00_JPRB, 0.79894E+00_JPRB ,&
&  0.79491E+00_JPRB, 0.78586E+00_JPRB, 0.77809E+00_JPRB, 0.76739E+00_JPRB, 0.75539E+00_JPRB, 0.73905E+00_JPRB, 0.71737E+00_JPRB ,&
&  0.69716E+00_JPRB, 0.68050E+00_JPRB, 0.66399E+00_JPRB, 0.64871E+00_JPRB, 0.63085E+00_JPRB, 0.61351E+00_JPRB, 0.59856E+00_JPRB ,&
&  0.58540E+00_JPRB, 0.57387E+00_JPRB, 0.56328E+00_JPRB, 0.55540E+00_JPRB, 0.55088E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.55124E+00_JPRB, 0.55009E+00_JPRB, 0.54823E+00_JPRB, 0.54553E+00_JPRB, 0.54099E+00_JPRB, 0.53709E+00_JPRB, 0.53378E+00_JPRB ,&
&  0.53138E+00_JPRB, 0.52928E+00_JPRB, 0.52817E+00_JPRB, 0.52873E+00_JPRB, 0.52986E+00_JPRB, 0.52965E+00_JPRB, 0.52962E+00_JPRB ,&
&  0.52957E+00_JPRB, 0.52823E+00_JPRB, 0.52612E+00_JPRB, 0.52337E+00_JPRB, 0.51794E+00_JPRB, 0.50937E+00_JPRB, 0.50079E+00_JPRB ,&
&  0.49200E+00_JPRB, 0.48431E+00_JPRB, 0.47655E+00_JPRB, 0.46860E+00_JPRB, 0.46211E+00_JPRB, 0.42518E+00_JPRB, 0.41382E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.41201E+00_JPRB, 0.41793E+00_JPRB, 0.42653E+00_JPRB, 0.44238E+00_JPRB, 0.44005E+00_JPRB, 0.44071E+00_JPRB ,&
&  0.42001E+00_JPRB, 0.41994E+00_JPRB, 0.42189E+00_JPRB, 0.42578E+00_JPRB, 0.43443E+00_JPRB, 0.44539E+00_JPRB, 0.46105E+00_JPRB ,&
&  0.49190E+00_JPRB, 0.49959E+00_JPRB, 0.50660E+00_JPRB, 0.51530E+00_JPRB, 0.52333E+00_JPRB, 0.53011E+00_JPRB, 0.53663E+00_JPRB ,&
&  0.54189E+00_JPRB, 0.54741E+00_JPRB, 0.55242E+00_JPRB, 0.55755E+00_JPRB, 0.56363E+00_JPRB, 0.56979E+00_JPRB, 0.57837E+00_JPRB ,&
&  0.58729E+00_JPRB, 0.59879E+00_JPRB, 0.61121E+00_JPRB, 0.62427E+00_JPRB, 0.63720E+00_JPRB, 0.65092E+00_JPRB, 0.66581E+00_JPRB ,&
&  0.68062E+00_JPRB, 0.69645E+00_JPRB, 0.70763E+00_JPRB, 0.71902E+00_JPRB, 0.72756E+00_JPRB, 0.73793E+00_JPRB, 0.74198E+00_JPRB ,&
&  0.74655E+00_JPRB, 0.74556E+00_JPRB, 0.74688E+00_JPRB, 0.74500E+00_JPRB, 0.74466E+00_JPRB, 0.74448E+00_JPRB, 0.74181E+00_JPRB ,&
&  0.73866E+00_JPRB, 0.73346E+00_JPRB, 0.72833E+00_JPRB, 0.72064E+00_JPRB, 0.70993E+00_JPRB, 0.69593E+00_JPRB, 0.68090E+00_JPRB ,&
&  0.66780E+00_JPRB, 0.65631E+00_JPRB, 0.64481E+00_JPRB, 0.63417E+00_JPRB, 0.62243E+00_JPRB, 0.60892E+00_JPRB, 0.59518E+00_JPRB ,&
&  0.58338E+00_JPRB, 0.57334E+00_JPRB, 0.56504E+00_JPRB, 0.55832E+00_JPRB, 0.55438E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.55797E+00_JPRB, 0.55746E+00_JPRB, 0.55698E+00_JPRB, 0.55583E+00_JPRB, 0.55296E+00_JPRB, 0.55044E+00_JPRB, 0.54814E+00_JPRB ,&
&  0.54673E+00_JPRB, 0.54586E+00_JPRB, 0.54414E+00_JPRB, 0.54190E+00_JPRB, 0.53965E+00_JPRB, 0.53783E+00_JPRB, 0.53614E+00_JPRB ,&
&  0.53455E+00_JPRB, 0.53228E+00_JPRB, 0.52915E+00_JPRB, 0.52439E+00_JPRB, 0.51985E+00_JPRB, 0.51560E+00_JPRB, 0.51290E+00_JPRB ,&
&  0.51099E+00_JPRB, 0.50896E+00_JPRB, 0.50631E+00_JPRB, 0.50393E+00_JPRB, 0.50138E+00_JPRB, 0.49913E+00_JPRB, 0.49656E+00_JPRB ,&
&  0.49518E+00_JPRB, 0.49477E+00_JPRB, 0.49445E+00_JPRB, 0.49386E+00_JPRB, 0.49454E+00_JPRB, 0.49601E+00_JPRB, 0.49733E+00_JPRB ,&
&  0.49832E+00_JPRB, 0.49989E+00_JPRB, 0.50276E+00_JPRB, 0.50515E+00_JPRB, 0.50717E+00_JPRB, 0.50943E+00_JPRB, 0.51083E+00_JPRB ,&
&  0.51089E+00_JPRB, 0.51157E+00_JPRB, 0.51433E+00_JPRB, 0.51723E+00_JPRB, 0.52038E+00_JPRB, 0.52389E+00_JPRB, 0.52910E+00_JPRB ,&
&  0.53334E+00_JPRB, 0.53804E+00_JPRB, 0.54339E+00_JPRB, 0.54993E+00_JPRB, 0.55649E+00_JPRB, 0.56392E+00_JPRB, 0.57179E+00_JPRB ,&
&  0.57965E+00_JPRB, 0.58792E+00_JPRB, 0.59614E+00_JPRB, 0.60436E+00_JPRB, 0.61211E+00_JPRB, 0.61885E+00_JPRB, 0.62608E+00_JPRB ,&
&  0.63371E+00_JPRB, 0.64259E+00_JPRB, 0.64887E+00_JPRB, 0.65567E+00_JPRB, 0.66299E+00_JPRB, 0.67295E+00_JPRB, 0.67739E+00_JPRB ,&
&  0.68147E+00_JPRB, 0.68527E+00_JPRB, 0.68930E+00_JPRB, 0.69082E+00_JPRB, 0.69230E+00_JPRB, 0.69343E+00_JPRB, 0.69332E+00_JPRB ,&
&  0.69129E+00_JPRB, 0.68814E+00_JPRB, 0.68372E+00_JPRB, 0.67722E+00_JPRB, 0.66900E+00_JPRB, 0.66028E+00_JPRB, 0.65034E+00_JPRB ,&
&  0.64094E+00_JPRB, 0.63000E+00_JPRB, 0.61902E+00_JPRB, 0.60954E+00_JPRB, 0.60211E+00_JPRB, 0.59351E+00_JPRB, 0.58473E+00_JPRB ,&
&  0.57647E+00_JPRB, 0.56991E+00_JPRB, 0.56587E+00_JPRB, 0.56232E+00_JPRB, 0.56007E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.56570E+00_JPRB, 0.56466E+00_JPRB, 0.56379E+00_JPRB, 0.56318E+00_JPRB, 0.56210E+00_JPRB, 0.56058E+00_JPRB, 0.55867E+00_JPRB ,&
&  0.55667E+00_JPRB, 0.55504E+00_JPRB, 0.55344E+00_JPRB, 0.55212E+00_JPRB, 0.55097E+00_JPRB, 0.54947E+00_JPRB, 0.54743E+00_JPRB ,&
&  0.54497E+00_JPRB, 0.54201E+00_JPRB, 0.53871E+00_JPRB, 0.53529E+00_JPRB, 0.53293E+00_JPRB, 0.53121E+00_JPRB, 0.52976E+00_JPRB ,&
&  0.52855E+00_JPRB, 0.52726E+00_JPRB, 0.52590E+00_JPRB, 0.52484E+00_JPRB, 0.52466E+00_JPRB, 0.52431E+00_JPRB, 0.52412E+00_JPRB ,&
&  0.52402E+00_JPRB, 0.52431E+00_JPRB, 0.52429E+00_JPRB, 0.52426E+00_JPRB, 0.52415E+00_JPRB, 0.52494E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.52619E+00_JPRB, 0.52664E+00_JPRB, 0.52691E+00_JPRB, 0.52705E+00_JPRB, 0.52738E+00_JPRB, 0.52762E+00_JPRB, 0.52796E+00_JPRB ,&
&  0.52857E+00_JPRB, 0.52966E+00_JPRB, 0.53162E+00_JPRB, 0.53412E+00_JPRB, 0.53665E+00_JPRB, 0.53931E+00_JPRB, 0.54252E+00_JPRB ,&
&  0.54569E+00_JPRB, 0.54896E+00_JPRB, 0.55222E+00_JPRB, 0.55580E+00_JPRB, 0.55948E+00_JPRB, 0.56299E+00_JPRB, 0.56665E+00_JPRB ,&
&  0.57040E+00_JPRB, 0.57432E+00_JPRB, 0.57858E+00_JPRB, 0.58288E+00_JPRB, 0.58682E+00_JPRB, 0.59036E+00_JPRB, 0.59419E+00_JPRB ,&
&  0.59824E+00_JPRB, 0.60221E+00_JPRB, 0.60523E+00_JPRB, 0.60850E+00_JPRB, 0.61184E+00_JPRB, 0.61504E+00_JPRB, 0.61733E+00_JPRB ,&
&  0.61975E+00_JPRB, 0.62246E+00_JPRB, 0.62521E+00_JPRB, 0.62586E+00_JPRB, 0.62626E+00_JPRB, 0.62637E+00_JPRB, 0.62571E+00_JPRB ,&
&  0.62383E+00_JPRB, 0.62179E+00_JPRB, 0.61961E+00_JPRB, 0.61691E+00_JPRB, 0.61299E+00_JPRB, 0.60891E+00_JPRB, 0.60446E+00_JPRB ,&
&  0.59994E+00_JPRB, 0.59474E+00_JPRB, 0.58958E+00_JPRB, 0.58520E+00_JPRB, 0.58188E+00_JPRB, 0.57867E+00_JPRB, 0.57541E+00_JPRB ,&
&  0.57214E+00_JPRB, 0.57008E+00_JPRB, 0.56911E+00_JPRB, 0.56815E+00_JPRB, 0.56712E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB, 0.55856E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB ,&
&  0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB, 0.23209E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.24389E+00_JPRB, 0.24419E+00_JPRB, 0.24432E+00_JPRB, 0.24435E+00_JPRB, 0.24271E+00_JPRB, 0.24318E+00_JPRB, 0.24330E+00_JPRB ,&
&  0.24360E+00_JPRB, 0.24389E+00_JPRB, 0.24447E+00_JPRB, 0.24516E+00_JPRB, 0.24583E+00_JPRB, 0.24644E+00_JPRB, 0.24696E+00_JPRB ,&
&  0.24728E+00_JPRB, 0.24769E+00_JPRB, 0.24645E+00_JPRB, 0.24573E+00_JPRB, 0.24567E+00_JPRB, 0.24579E+00_JPRB, 0.24579E+00_JPRB ,&
&  0.24576E+00_JPRB, 0.24566E+00_JPRB, 0.24524E+00_JPRB, 0.24493E+00_JPRB, 0.24479E+00_JPRB, 0.24461E+00_JPRB, 0.24432E+00_JPRB ,&
&  0.24394E+00_JPRB, 0.24364E+00_JPRB, 0.24328E+00_JPRB, 0.24287E+00_JPRB, 0.24272E+00_JPRB, 0.24250E+00_JPRB, 0.24189E+00_JPRB ,&
&  0.24117E+00_JPRB, 0.24015E+00_JPRB, 0.23921E+00_JPRB, 0.23837E+00_JPRB, 0.23769E+00_JPRB, 0.23733E+00_JPRB, 0.23697E+00_JPRB ,&
&  0.23648E+00_JPRB, 0.23583E+00_JPRB, 0.23520E+00_JPRB, 0.23446E+00_JPRB, 0.23380E+00_JPRB, 0.23321E+00_JPRB, 0.23252E+00_JPRB ,&
&  0.23176E+00_JPRB, 0.23099E+00_JPRB, 0.23020E+00_JPRB, 0.22932E+00_JPRB, 0.22848E+00_JPRB, 0.22763E+00_JPRB, 0.22671E+00_JPRB ,&
&  0.22580E+00_JPRB, 0.22539E+00_JPRB, 0.22492E+00_JPRB, 0.22453E+00_JPRB, 0.22423E+00_JPRB, 0.22400E+00_JPRB, 0.22377E+00_JPRB ,&
&  0.22359E+00_JPRB, 0.22340E+00_JPRB, 0.22330E+00_JPRB, 0.22321E+00_JPRB, 0.22320E+00_JPRB, 0.22326E+00_JPRB, 0.22343E+00_JPRB ,&
&  0.22362E+00_JPRB, 0.22387E+00_JPRB, 0.22420E+00_JPRB, 0.22474E+00_JPRB, 0.22528E+00_JPRB, 0.22590E+00_JPRB, 0.22652E+00_JPRB ,&
&  0.22720E+00_JPRB, 0.22787E+00_JPRB, 0.22855E+00_JPRB, 0.22920E+00_JPRB, 0.23006E+00_JPRB, 0.23090E+00_JPRB, 0.23181E+00_JPRB ,&
&  0.23273E+00_JPRB, 0.23371E+00_JPRB, 0.23468E+00_JPRB, 0.23572E+00_JPRB, 0.23690E+00_JPRB, 0.23726E+00_JPRB, 0.23764E+00_JPRB ,&
&  0.23810E+00_JPRB, 0.23878E+00_JPRB, 0.23941E+00_JPRB, 0.24021E+00_JPRB, 0.24156E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.27066E+00_JPRB, 0.27717E+00_JPRB, 0.28162E+00_JPRB, 0.28644E+00_JPRB, 0.28563E+00_JPRB, 0.32374E+00_JPRB, 0.32274E+00_JPRB ,&
&  0.32149E+00_JPRB, 0.30699E+00_JPRB, 0.30448E+00_JPRB, 0.29965E+00_JPRB, 0.29404E+00_JPRB, 0.28793E+00_JPRB, 0.28264E+00_JPRB ,&
&  0.27783E+00_JPRB, 0.27250E+00_JPRB, 0.26813E+00_JPRB, 0.26410E+00_JPRB, 0.26079E+00_JPRB, 0.25786E+00_JPRB, 0.25579E+00_JPRB ,&
&  0.25526E+00_JPRB, 0.25670E+00_JPRB, 0.25976E+00_JPRB, 0.26154E+00_JPRB, 0.26236E+00_JPRB, 0.26304E+00_JPRB, 0.26371E+00_JPRB ,&
&  0.26379E+00_JPRB, 0.26383E+00_JPRB, 0.26277E+00_JPRB, 0.26076E+00_JPRB, 0.25887E+00_JPRB, 0.25730E+00_JPRB, 0.25606E+00_JPRB ,&
&  0.25487E+00_JPRB, 0.25366E+00_JPRB, 0.25236E+00_JPRB, 0.25153E+00_JPRB, 0.25040E+00_JPRB, 0.24889E+00_JPRB, 0.24682E+00_JPRB ,&
&  0.24476E+00_JPRB, 0.24269E+00_JPRB, 0.24024E+00_JPRB, 0.23829E+00_JPRB, 0.23639E+00_JPRB, 0.23446E+00_JPRB, 0.23255E+00_JPRB ,&
&  0.23147E+00_JPRB, 0.23041E+00_JPRB, 0.22909E+00_JPRB, 0.22742E+00_JPRB, 0.22614E+00_JPRB, 0.22478E+00_JPRB, 0.22359E+00_JPRB ,&
&  0.22280E+00_JPRB, 0.22200E+00_JPRB, 0.22117E+00_JPRB, 0.22031E+00_JPRB, 0.21967E+00_JPRB, 0.21895E+00_JPRB, 0.21829E+00_JPRB ,&
&  0.21771E+00_JPRB, 0.21707E+00_JPRB, 0.21691E+00_JPRB, 0.21684E+00_JPRB, 0.21691E+00_JPRB, 0.21697E+00_JPRB, 0.21727E+00_JPRB ,&
&  0.21773E+00_JPRB, 0.21840E+00_JPRB, 0.21921E+00_JPRB, 0.21998E+00_JPRB, 0.22074E+00_JPRB, 0.22152E+00_JPRB, 0.22266E+00_JPRB ,&
&  0.22391E+00_JPRB, 0.22524E+00_JPRB, 0.22663E+00_JPRB, 0.22772E+00_JPRB, 0.22994E+00_JPRB, 0.23205E+00_JPRB, 0.23430E+00_JPRB ,&
&  0.23687E+00_JPRB, 0.23991E+00_JPRB, 0.24316E+00_JPRB, 0.24662E+00_JPRB, 0.25114E+00_JPRB, 0.25407E+00_JPRB, 0.25624E+00_JPRB ,&
&  0.25862E+00_JPRB, 0.26258E+00_JPRB, 0.26425E+00_JPRB, 0.26569E+00_JPRB, 0.26782E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.34228E+00_JPRB, 0.34694E+00_JPRB, 0.35094E+00_JPRB, 0.35448E+00_JPRB, 0.35758E+00_JPRB, 0.35835E+00_JPRB, 0.35738E+00_JPRB ,&
&  0.34834E+00_JPRB, 0.34163E+00_JPRB, 0.33347E+00_JPRB, 0.32625E+00_JPRB, 0.31957E+00_JPRB, 0.31181E+00_JPRB, 0.30503E+00_JPRB ,&
&  0.29811E+00_JPRB, 0.29120E+00_JPRB, 0.28462E+00_JPRB, 0.27693E+00_JPRB, 0.26891E+00_JPRB, 0.26123E+00_JPRB, 0.25648E+00_JPRB ,&
&  0.25377E+00_JPRB, 0.25362E+00_JPRB, 0.25651E+00_JPRB, 0.26035E+00_JPRB, 0.26532E+00_JPRB, 0.27171E+00_JPRB, 0.27854E+00_JPRB ,&
&  0.28288E+00_JPRB, 0.28528E+00_JPRB, 0.28487E+00_JPRB, 0.28348E+00_JPRB, 0.28151E+00_JPRB, 0.27980E+00_JPRB, 0.27714E+00_JPRB ,&
&  0.27376E+00_JPRB, 0.27130E+00_JPRB, 0.27035E+00_JPRB, 0.27110E+00_JPRB, 0.27069E+00_JPRB, 0.26870E+00_JPRB, 0.26478E+00_JPRB ,&
&  0.26047E+00_JPRB, 0.25614E+00_JPRB, 0.25175E+00_JPRB, 0.24767E+00_JPRB, 0.24433E+00_JPRB, 0.24084E+00_JPRB, 0.23710E+00_JPRB ,&
&  0.23472E+00_JPRB, 0.23219E+00_JPRB, 0.22941E+00_JPRB, 0.22676E+00_JPRB, 0.22457E+00_JPRB, 0.22287E+00_JPRB, 0.22145E+00_JPRB ,&
&  0.22034E+00_JPRB, 0.21949E+00_JPRB, 0.21880E+00_JPRB, 0.21828E+00_JPRB, 0.21778E+00_JPRB, 0.21703E+00_JPRB, 0.21629E+00_JPRB ,&
&  0.21554E+00_JPRB, 0.21526E+00_JPRB, 0.21481E+00_JPRB, 0.21482E+00_JPRB, 0.21507E+00_JPRB, 0.21521E+00_JPRB, 0.21571E+00_JPRB ,&
&  0.21682E+00_JPRB, 0.21797E+00_JPRB, 0.21861E+00_JPRB, 0.21970E+00_JPRB, 0.22076E+00_JPRB, 0.22214E+00_JPRB, 0.22392E+00_JPRB ,&
&  0.22507E+00_JPRB, 0.22635E+00_JPRB, 0.22784E+00_JPRB, 0.22943E+00_JPRB, 0.23203E+00_JPRB, 0.23486E+00_JPRB, 0.23800E+00_JPRB ,&
&  0.24204E+00_JPRB, 0.24601E+00_JPRB, 0.25052E+00_JPRB, 0.25585E+00_JPRB, 0.26518E+00_JPRB, 0.27184E+00_JPRB, 0.28049E+00_JPRB ,&
&  0.28923E+00_JPRB, 0.29468E+00_JPRB, 0.32846E+00_JPRB, 0.33097E+00_JPRB, 0.33619E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.35155E+00_JPRB, 0.35816E+00_JPRB, 0.36502E+00_JPRB, 0.37009E+00_JPRB, 0.37377E+00_JPRB, 0.37681E+00_JPRB, 0.37555E+00_JPRB ,&
&  0.36944E+00_JPRB, 0.35420E+00_JPRB, 0.34276E+00_JPRB, 0.33167E+00_JPRB, 0.32226E+00_JPRB, 0.31334E+00_JPRB, 0.30501E+00_JPRB ,&
&  0.29708E+00_JPRB, 0.29052E+00_JPRB, 0.28514E+00_JPRB, 0.28047E+00_JPRB, 0.27408E+00_JPRB, 0.26848E+00_JPRB, 0.26399E+00_JPRB ,&
&  0.26027E+00_JPRB, 0.26013E+00_JPRB, 0.26237E+00_JPRB, 0.26672E+00_JPRB, 0.27480E+00_JPRB, 0.28311E+00_JPRB, 0.29240E+00_JPRB ,&
&  0.30672E+00_JPRB, 0.31083E+00_JPRB, 0.30982E+00_JPRB, 0.30916E+00_JPRB, 0.30424E+00_JPRB, 0.29818E+00_JPRB, 0.28887E+00_JPRB ,&
&  0.28934E+00_JPRB, 0.28736E+00_JPRB, 0.29429E+00_JPRB, 0.29450E+00_JPRB, 0.29323E+00_JPRB, 0.29301E+00_JPRB, 0.28549E+00_JPRB ,&
&  0.27979E+00_JPRB, 0.27078E+00_JPRB, 0.26385E+00_JPRB, 0.25682E+00_JPRB, 0.25004E+00_JPRB, 0.24598E+00_JPRB, 0.24071E+00_JPRB ,&
&  0.23677E+00_JPRB, 0.23377E+00_JPRB, 0.22992E+00_JPRB, 0.22743E+00_JPRB, 0.22533E+00_JPRB, 0.22317E+00_JPRB, 0.22206E+00_JPRB ,&
&  0.22123E+00_JPRB, 0.22093E+00_JPRB, 0.22104E+00_JPRB, 0.22124E+00_JPRB, 0.22172E+00_JPRB, 0.22261E+00_JPRB, 0.22328E+00_JPRB ,&
&  0.22401E+00_JPRB, 0.22424E+00_JPRB, 0.22386E+00_JPRB, 0.22341E+00_JPRB, 0.22231E+00_JPRB, 0.22156E+00_JPRB, 0.22061E+00_JPRB ,&
&  0.22005E+00_JPRB, 0.22037E+00_JPRB, 0.22067E+00_JPRB, 0.22136E+00_JPRB, 0.22207E+00_JPRB, 0.22359E+00_JPRB, 0.22503E+00_JPRB ,&
&  0.22656E+00_JPRB, 0.22842E+00_JPRB, 0.22997E+00_JPRB, 0.23192E+00_JPRB, 0.23427E+00_JPRB, 0.23590E+00_JPRB, 0.23866E+00_JPRB ,&
&  0.24140E+00_JPRB, 0.24480E+00_JPRB, 0.25050E+00_JPRB, 0.25687E+00_JPRB, 0.26544E+00_JPRB, 0.27626E+00_JPRB, 0.29001E+00_JPRB ,&
&  0.30767E+00_JPRB, 0.33273E+00_JPRB, 0.33466E+00_JPRB, 0.34144E+00_JPRB, 0.34639E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.35668E+00_JPRB, 0.35925E+00_JPRB, 0.36331E+00_JPRB, 0.36870E+00_JPRB, 0.37543E+00_JPRB, 0.38146E+00_JPRB, 0.38281E+00_JPRB ,&
&  0.37357E+00_JPRB, 0.36063E+00_JPRB, 0.34250E+00_JPRB, 0.32802E+00_JPRB, 0.31413E+00_JPRB, 0.30368E+00_JPRB, 0.29515E+00_JPRB ,&
&  0.28984E+00_JPRB, 0.28933E+00_JPRB, 0.28984E+00_JPRB, 0.29108E+00_JPRB, 0.29017E+00_JPRB, 0.29129E+00_JPRB, 0.29173E+00_JPRB ,&
&  0.29062E+00_JPRB, 0.29052E+00_JPRB, 0.29043E+00_JPRB, 0.29397E+00_JPRB, 0.30079E+00_JPRB, 0.30715E+00_JPRB, 0.31346E+00_JPRB ,&
&  0.32082E+00_JPRB, 0.32312E+00_JPRB, 0.32742E+00_JPRB, 0.32630E+00_JPRB, 0.32284E+00_JPRB, 0.31835E+00_JPRB, 0.31206E+00_JPRB ,&
&  0.30739E+00_JPRB, 0.30330E+00_JPRB, 0.30183E+00_JPRB, 0.30224E+00_JPRB, 0.30234E+00_JPRB, 0.30376E+00_JPRB, 0.29744E+00_JPRB ,&
&  0.29015E+00_JPRB, 0.27987E+00_JPRB, 0.26829E+00_JPRB, 0.25979E+00_JPRB, 0.25184E+00_JPRB, 0.24611E+00_JPRB, 0.24099E+00_JPRB ,&
&  0.23624E+00_JPRB, 0.23305E+00_JPRB, 0.22976E+00_JPRB, 0.22736E+00_JPRB, 0.22663E+00_JPRB, 0.22498E+00_JPRB, 0.22409E+00_JPRB ,&
&  0.22374E+00_JPRB, 0.22420E+00_JPRB, 0.22574E+00_JPRB, 0.22717E+00_JPRB, 0.22910E+00_JPRB, 0.23131E+00_JPRB, 0.23320E+00_JPRB ,&
&  0.23571E+00_JPRB, 0.23772E+00_JPRB, 0.23794E+00_JPRB, 0.23733E+00_JPRB, 0.23526E+00_JPRB, 0.23257E+00_JPRB, 0.23014E+00_JPRB ,&
&  0.22774E+00_JPRB, 0.22603E+00_JPRB, 0.22536E+00_JPRB, 0.22454E+00_JPRB, 0.22496E+00_JPRB, 0.22609E+00_JPRB, 0.22698E+00_JPRB ,&
&  0.22865E+00_JPRB, 0.23002E+00_JPRB, 0.23158E+00_JPRB, 0.23417E+00_JPRB, 0.23665E+00_JPRB, 0.23885E+00_JPRB, 0.24114E+00_JPRB ,&
&  0.24138E+00_JPRB, 0.24308E+00_JPRB, 0.24536E+00_JPRB, 0.24931E+00_JPRB, 0.26070E+00_JPRB, 0.26901E+00_JPRB, 0.28820E+00_JPRB ,&
&  0.30548E+00_JPRB, 0.34132E+00_JPRB, 0.34216E+00_JPRB, 0.34717E+00_JPRB, 0.35312E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.35993E+00_JPRB, 0.36434E+00_JPRB, 0.36526E+00_JPRB, 0.36857E+00_JPRB, 0.37311E+00_JPRB, 0.37843E+00_JPRB, 0.38282E+00_JPRB ,&
&  0.38786E+00_JPRB, 0.38759E+00_JPRB, 0.38246E+00_JPRB, 0.37552E+00_JPRB, 0.36965E+00_JPRB, 0.35280E+00_JPRB, 0.34380E+00_JPRB ,&
&  0.33650E+00_JPRB, 0.33597E+00_JPRB, 0.33654E+00_JPRB, 0.33816E+00_JPRB, 0.33926E+00_JPRB, 0.34426E+00_JPRB, 0.34581E+00_JPRB ,&
&  0.34338E+00_JPRB, 0.33833E+00_JPRB, 0.33323E+00_JPRB, 0.33238E+00_JPRB, 0.33498E+00_JPRB, 0.33700E+00_JPRB, 0.33425E+00_JPRB ,&
&  0.33064E+00_JPRB, 0.33038E+00_JPRB, 0.33103E+00_JPRB, 0.33249E+00_JPRB, 0.34221E+00_JPRB, 0.33781E+00_JPRB, 0.33341E+00_JPRB ,&
&  0.32629E+00_JPRB, 0.32008E+00_JPRB, 0.31575E+00_JPRB, 0.31601E+00_JPRB, 0.31876E+00_JPRB, 0.31871E+00_JPRB, 0.32122E+00_JPRB ,&
&  0.30930E+00_JPRB, 0.29942E+00_JPRB, 0.28471E+00_JPRB, 0.27316E+00_JPRB, 0.26208E+00_JPRB, 0.25449E+00_JPRB, 0.24736E+00_JPRB ,&
&  0.24127E+00_JPRB, 0.23625E+00_JPRB, 0.23149E+00_JPRB, 0.22991E+00_JPRB, 0.23078E+00_JPRB, 0.23095E+00_JPRB, 0.23066E+00_JPRB ,&
&  0.23128E+00_JPRB, 0.23226E+00_JPRB, 0.23305E+00_JPRB, 0.23403E+00_JPRB, 0.23556E+00_JPRB, 0.23808E+00_JPRB, 0.24145E+00_JPRB ,&
&  0.24676E+00_JPRB, 0.25102E+00_JPRB, 0.25333E+00_JPRB, 0.25436E+00_JPRB, 0.25125E+00_JPRB, 0.24603E+00_JPRB, 0.24065E+00_JPRB ,&
&  0.23539E+00_JPRB, 0.23291E+00_JPRB, 0.23074E+00_JPRB, 0.22971E+00_JPRB, 0.22955E+00_JPRB, 0.23042E+00_JPRB, 0.23218E+00_JPRB ,&
&  0.23389E+00_JPRB, 0.23603E+00_JPRB, 0.23744E+00_JPRB, 0.23915E+00_JPRB, 0.24124E+00_JPRB, 0.24308E+00_JPRB, 0.24500E+00_JPRB ,&
&  0.24715E+00_JPRB, 0.24883E+00_JPRB, 0.25170E+00_JPRB, 0.25522E+00_JPRB, 0.26187E+00_JPRB, 0.26988E+00_JPRB, 0.28423E+00_JPRB ,&
&  0.30128E+00_JPRB, 0.33404E+00_JPRB, 0.34516E+00_JPRB, 0.35120E+00_JPRB, 0.35790E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.35761E+00_JPRB, 0.36409E+00_JPRB, 0.36917E+00_JPRB, 0.37922E+00_JPRB, 0.38196E+00_JPRB, 0.38460E+00_JPRB, 0.38671E+00_JPRB ,&
&  0.39265E+00_JPRB, 0.39618E+00_JPRB, 0.39393E+00_JPRB, 0.39326E+00_JPRB, 0.39234E+00_JPRB, 0.39124E+00_JPRB, 0.39205E+00_JPRB ,&
&  0.39199E+00_JPRB, 0.39399E+00_JPRB, 0.39171E+00_JPRB, 0.38890E+00_JPRB, 0.39076E+00_JPRB, 0.39549E+00_JPRB, 0.39869E+00_JPRB ,&
&  0.39368E+00_JPRB, 0.39249E+00_JPRB, 0.40111E+00_JPRB, 0.40343E+00_JPRB, 0.40486E+00_JPRB, 0.40701E+00_JPRB, 0.40691E+00_JPRB ,&
&  0.39339E+00_JPRB, 0.36847E+00_JPRB, 0.34091E+00_JPRB, 0.34182E+00_JPRB, 0.35636E+00_JPRB, 0.35147E+00_JPRB, 0.34994E+00_JPRB ,&
&  0.34704E+00_JPRB, 0.34265E+00_JPRB, 0.33842E+00_JPRB, 0.33654E+00_JPRB, 0.33377E+00_JPRB, 0.32930E+00_JPRB, 0.32767E+00_JPRB ,&
&  0.32673E+00_JPRB, 0.33491E+00_JPRB, 0.33080E+00_JPRB, 0.30982E+00_JPRB, 0.29353E+00_JPRB, 0.28472E+00_JPRB, 0.27348E+00_JPRB ,&
&  0.26054E+00_JPRB, 0.25273E+00_JPRB, 0.25138E+00_JPRB, 0.24983E+00_JPRB, 0.25155E+00_JPRB, 0.25263E+00_JPRB, 0.25378E+00_JPRB ,&
&  0.25475E+00_JPRB, 0.25466E+00_JPRB, 0.25021E+00_JPRB, 0.24440E+00_JPRB, 0.24121E+00_JPRB, 0.24115E+00_JPRB, 0.24363E+00_JPRB ,&
&  0.24918E+00_JPRB, 0.25525E+00_JPRB, 0.26922E+00_JPRB, 0.27692E+00_JPRB, 0.26861E+00_JPRB, 0.26271E+00_JPRB, 0.25611E+00_JPRB ,&
&  0.24618E+00_JPRB, 0.24228E+00_JPRB, 0.23907E+00_JPRB, 0.23730E+00_JPRB, 0.23810E+00_JPRB, 0.24123E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.24528E+00_JPRB, 0.24776E+00_JPRB, 0.24875E+00_JPRB, 0.24958E+00_JPRB, 0.25026E+00_JPRB, 0.25203E+00_JPRB, 0.25384E+00_JPRB ,&
&  0.25538E+00_JPRB, 0.25813E+00_JPRB, 0.26150E+00_JPRB, 0.26603E+00_JPRB, 0.27003E+00_JPRB, 0.27243E+00_JPRB, 0.27919E+00_JPRB ,&
&  0.28725E+00_JPRB, 0.30397E+00_JPRB, 0.33149E+00_JPRB, 0.35641E+00_JPRB, 0.35314E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.36538E+00_JPRB, 0.37033E+00_JPRB, 0.37707E+00_JPRB, 0.38457E+00_JPRB, 0.38987E+00_JPRB, 0.39217E+00_JPRB, 0.39457E+00_JPRB ,&
&  0.39775E+00_JPRB, 0.39910E+00_JPRB, 0.40015E+00_JPRB, 0.40294E+00_JPRB, 0.40851E+00_JPRB, 0.41135E+00_JPRB, 0.41595E+00_JPRB ,&
&  0.41770E+00_JPRB, 0.41557E+00_JPRB, 0.40887E+00_JPRB, 0.40684E+00_JPRB, 0.41029E+00_JPRB, 0.41499E+00_JPRB, 0.42025E+00_JPRB ,&
&  0.42578E+00_JPRB, 0.43088E+00_JPRB, 0.43335E+00_JPRB, 0.43415E+00_JPRB, 0.43446E+00_JPRB, 0.43446E+00_JPRB, 0.43197E+00_JPRB ,&
&  0.42736E+00_JPRB, 0.38834E+00_JPRB, 0.36166E+00_JPRB, 0.36410E+00_JPRB, 0.37588E+00_JPRB, 0.37074E+00_JPRB, 0.36921E+00_JPRB ,&
&  0.36762E+00_JPRB, 0.36425E+00_JPRB, 0.35923E+00_JPRB, 0.35522E+00_JPRB, 0.35016E+00_JPRB, 0.34365E+00_JPRB, 0.33834E+00_JPRB ,&
&  0.33725E+00_JPRB, 0.34190E+00_JPRB, 0.34356E+00_JPRB, 0.34615E+00_JPRB, 0.34541E+00_JPRB, 0.34779E+00_JPRB, 0.34698E+00_JPRB ,&
&  0.34201E+00_JPRB, 0.33627E+00_JPRB, 0.33451E+00_JPRB, 0.33426E+00_JPRB, 0.33531E+00_JPRB, 0.33791E+00_JPRB, 0.33802E+00_JPRB ,&
&  0.32485E+00_JPRB, 0.31341E+00_JPRB, 0.29507E+00_JPRB, 0.27729E+00_JPRB, 0.26423E+00_JPRB, 0.25824E+00_JPRB, 0.25385E+00_JPRB ,&
&  0.25281E+00_JPRB, 0.25824E+00_JPRB, 0.27415E+00_JPRB, 0.29375E+00_JPRB, 0.30843E+00_JPRB, 0.29026E+00_JPRB, 0.27442E+00_JPRB ,&
&  0.26464E+00_JPRB, 0.25950E+00_JPRB, 0.25524E+00_JPRB, 0.25365E+00_JPRB, 0.25392E+00_JPRB, 0.25597E+00_JPRB, 0.25901E+00_JPRB ,&
&  0.26343E+00_JPRB, 0.26766E+00_JPRB, 0.27044E+00_JPRB, 0.27177E+00_JPRB, 0.27247E+00_JPRB, 0.27315E+00_JPRB, 0.27251E+00_JPRB ,&
&  0.27217E+00_JPRB, 0.27517E+00_JPRB, 0.28159E+00_JPRB, 0.29009E+00_JPRB, 0.29813E+00_JPRB, 0.30701E+00_JPRB, 0.31955E+00_JPRB ,&
&  0.33357E+00_JPRB, 0.34555E+00_JPRB, 0.34981E+00_JPRB, 0.35562E+00_JPRB, 0.36074E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.38591E+00_JPRB, 0.39026E+00_JPRB, 0.39428E+00_JPRB, 0.39869E+00_JPRB, 0.40493E+00_JPRB, 0.40800E+00_JPRB, 0.40891E+00_JPRB ,&
&  0.41021E+00_JPRB, 0.41086E+00_JPRB, 0.41248E+00_JPRB, 0.41685E+00_JPRB, 0.42274E+00_JPRB, 0.42683E+00_JPRB, 0.43235E+00_JPRB ,&
&  0.43242E+00_JPRB, 0.42396E+00_JPRB, 0.41622E+00_JPRB, 0.41591E+00_JPRB, 0.42131E+00_JPRB, 0.42699E+00_JPRB, 0.43420E+00_JPRB ,&
&  0.44138E+00_JPRB, 0.44667E+00_JPRB, 0.44968E+00_JPRB, 0.45120E+00_JPRB, 0.45323E+00_JPRB, 0.45301E+00_JPRB, 0.45016E+00_JPRB ,&
&  0.44629E+00_JPRB, 0.43774E+00_JPRB, 0.41823E+00_JPRB, 0.41530E+00_JPRB, 0.40661E+00_JPRB, 0.39899E+00_JPRB, 0.39366E+00_JPRB ,&
&  0.38931E+00_JPRB, 0.38585E+00_JPRB, 0.38205E+00_JPRB, 0.37892E+00_JPRB, 0.37535E+00_JPRB, 0.36878E+00_JPRB, 0.36560E+00_JPRB ,&
&  0.36274E+00_JPRB, 0.36035E+00_JPRB, 0.35811E+00_JPRB, 0.35867E+00_JPRB, 0.35940E+00_JPRB, 0.36254E+00_JPRB, 0.36420E+00_JPRB ,&
&  0.36135E+00_JPRB, 0.35791E+00_JPRB, 0.35522E+00_JPRB, 0.35687E+00_JPRB, 0.35909E+00_JPRB, 0.36061E+00_JPRB, 0.36119E+00_JPRB ,&
&  0.35890E+00_JPRB, 0.35461E+00_JPRB, 0.35019E+00_JPRB, 0.34500E+00_JPRB, 0.34022E+00_JPRB, 0.31453E+00_JPRB, 0.30311E+00_JPRB ,&
&  0.29945E+00_JPRB, 0.30822E+00_JPRB, 0.32077E+00_JPRB, 0.31929E+00_JPRB, 0.32095E+00_JPRB, 0.32090E+00_JPRB, 0.31837E+00_JPRB ,&
&  0.30969E+00_JPRB, 0.29601E+00_JPRB, 0.28658E+00_JPRB, 0.28234E+00_JPRB, 0.27923E+00_JPRB, 0.28009E+00_JPRB, 0.28573E+00_JPRB ,&
&  0.29542E+00_JPRB, 0.30696E+00_JPRB, 0.31576E+00_JPRB, 0.31980E+00_JPRB, 0.31881E+00_JPRB, 0.31571E+00_JPRB, 0.31157E+00_JPRB ,&
&  0.30686E+00_JPRB, 0.31211E+00_JPRB, 0.32582E+00_JPRB, 0.34578E+00_JPRB, 0.34831E+00_JPRB, 0.35303E+00_JPRB, 0.35623E+00_JPRB ,&
&  0.36128E+00_JPRB, 0.36763E+00_JPRB, 0.37319E+00_JPRB, 0.37920E+00_JPRB, 0.38296E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.41210E+00_JPRB, 0.41459E+00_JPRB, 0.41700E+00_JPRB, 0.41863E+00_JPRB, 0.42134E+00_JPRB, 0.42612E+00_JPRB, 0.42803E+00_JPRB ,&
&  0.42621E+00_JPRB, 0.42416E+00_JPRB, 0.42401E+00_JPRB, 0.42731E+00_JPRB, 0.43371E+00_JPRB, 0.44047E+00_JPRB, 0.44811E+00_JPRB ,&
&  0.44426E+00_JPRB, 0.43213E+00_JPRB, 0.42771E+00_JPRB, 0.42969E+00_JPRB, 0.43448E+00_JPRB, 0.43983E+00_JPRB, 0.44651E+00_JPRB ,&
&  0.45287E+00_JPRB, 0.45710E+00_JPRB, 0.46150E+00_JPRB, 0.46541E+00_JPRB, 0.46796E+00_JPRB, 0.46633E+00_JPRB, 0.46208E+00_JPRB ,&
&  0.45705E+00_JPRB, 0.45204E+00_JPRB, 0.44767E+00_JPRB, 0.43172E+00_JPRB, 0.42908E+00_JPRB, 0.42012E+00_JPRB, 0.41256E+00_JPRB ,&
&  0.40726E+00_JPRB, 0.40288E+00_JPRB, 0.40014E+00_JPRB, 0.39724E+00_JPRB, 0.39325E+00_JPRB, 0.38643E+00_JPRB, 0.38480E+00_JPRB ,&
&  0.38432E+00_JPRB, 0.38582E+00_JPRB, 0.38171E+00_JPRB, 0.37971E+00_JPRB, 0.37748E+00_JPRB, 0.37579E+00_JPRB, 0.38037E+00_JPRB ,&
&  0.38577E+00_JPRB, 0.38350E+00_JPRB, 0.37969E+00_JPRB, 0.37933E+00_JPRB, 0.38352E+00_JPRB, 0.38513E+00_JPRB, 0.38614E+00_JPRB ,&
&  0.38804E+00_JPRB, 0.38877E+00_JPRB, 0.38447E+00_JPRB, 0.37956E+00_JPRB, 0.37095E+00_JPRB, 0.36041E+00_JPRB, 0.35771E+00_JPRB ,&
&  0.35478E+00_JPRB, 0.34886E+00_JPRB, 0.34511E+00_JPRB, 0.34388E+00_JPRB, 0.34340E+00_JPRB, 0.34403E+00_JPRB, 0.34392E+00_JPRB ,&
&  0.34365E+00_JPRB, 0.34002E+00_JPRB, 0.33471E+00_JPRB, 0.33001E+00_JPRB, 0.32367E+00_JPRB, 0.32584E+00_JPRB, 0.33502E+00_JPRB ,&
&  0.34664E+00_JPRB, 0.35364E+00_JPRB, 0.35822E+00_JPRB, 0.36165E+00_JPRB, 0.36473E+00_JPRB, 0.36621E+00_JPRB, 0.36855E+00_JPRB ,&
&  0.37169E+00_JPRB, 0.37311E+00_JPRB, 0.37161E+00_JPRB, 0.37117E+00_JPRB, 0.37376E+00_JPRB, 0.37846E+00_JPRB, 0.38432E+00_JPRB ,&
&  0.39025E+00_JPRB, 0.39532E+00_JPRB, 0.39962E+00_JPRB, 0.40507E+00_JPRB, 0.40865E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.44505E+00_JPRB, 0.44528E+00_JPRB, 0.44591E+00_JPRB, 0.44736E+00_JPRB, 0.44833E+00_JPRB, 0.44747E+00_JPRB, 0.44722E+00_JPRB ,&
&  0.44407E+00_JPRB, 0.43942E+00_JPRB, 0.43648E+00_JPRB, 0.43774E+00_JPRB, 0.44657E+00_JPRB, 0.45807E+00_JPRB, 0.46512E+00_JPRB ,&
&  0.46106E+00_JPRB, 0.44900E+00_JPRB, 0.44605E+00_JPRB, 0.44836E+00_JPRB, 0.45056E+00_JPRB, 0.45472E+00_JPRB, 0.46087E+00_JPRB ,&
&  0.46649E+00_JPRB, 0.47033E+00_JPRB, 0.47450E+00_JPRB, 0.47621E+00_JPRB, 0.47651E+00_JPRB, 0.46951E+00_JPRB, 0.45845E+00_JPRB ,&
&  0.44795E+00_JPRB, 0.44057E+00_JPRB, 0.43616E+00_JPRB, 0.43263E+00_JPRB, 0.42741E+00_JPRB, 0.42226E+00_JPRB, 0.41556E+00_JPRB ,&
&  0.41221E+00_JPRB, 0.41179E+00_JPRB, 0.41254E+00_JPRB, 0.41065E+00_JPRB, 0.40819E+00_JPRB, 0.40338E+00_JPRB, 0.39951E+00_JPRB ,&
&  0.39827E+00_JPRB, 0.39990E+00_JPRB, 0.40168E+00_JPRB, 0.40131E+00_JPRB, 0.39837E+00_JPRB, 0.39639E+00_JPRB, 0.39811E+00_JPRB ,&
&  0.40417E+00_JPRB, 0.40416E+00_JPRB, 0.40323E+00_JPRB, 0.40387E+00_JPRB, 0.40590E+00_JPRB, 0.40779E+00_JPRB, 0.40727E+00_JPRB ,&
&  0.40673E+00_JPRB, 0.40688E+00_JPRB, 0.40739E+00_JPRB, 0.40952E+00_JPRB, 0.40981E+00_JPRB, 0.40789E+00_JPRB, 0.40255E+00_JPRB ,&
&  0.39426E+00_JPRB, 0.38373E+00_JPRB, 0.37935E+00_JPRB, 0.37620E+00_JPRB, 0.37311E+00_JPRB, 0.37227E+00_JPRB, 0.37342E+00_JPRB ,&
&  0.37191E+00_JPRB, 0.36570E+00_JPRB, 0.36282E+00_JPRB, 0.36306E+00_JPRB, 0.36158E+00_JPRB, 0.36326E+00_JPRB, 0.36649E+00_JPRB ,&
&  0.37607E+00_JPRB, 0.38129E+00_JPRB, 0.38282E+00_JPRB, 0.38070E+00_JPRB, 0.38104E+00_JPRB, 0.38274E+00_JPRB, 0.38692E+00_JPRB ,&
&  0.39331E+00_JPRB, 0.40016E+00_JPRB, 0.40714E+00_JPRB, 0.41482E+00_JPRB, 0.42465E+00_JPRB, 0.43420E+00_JPRB, 0.43780E+00_JPRB ,&
&  0.43787E+00_JPRB, 0.43807E+00_JPRB, 0.43916E+00_JPRB, 0.44058E+00_JPRB, 0.44360E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.47206E+00_JPRB, 0.47276E+00_JPRB, 0.47189E+00_JPRB, 0.47105E+00_JPRB, 0.46996E+00_JPRB, 0.46887E+00_JPRB, 0.46690E+00_JPRB ,&
&  0.46191E+00_JPRB, 0.45548E+00_JPRB, 0.45170E+00_JPRB, 0.44993E+00_JPRB, 0.45544E+00_JPRB, 0.46369E+00_JPRB, 0.47111E+00_JPRB ,&
&  0.47043E+00_JPRB, 0.46164E+00_JPRB, 0.45903E+00_JPRB, 0.46109E+00_JPRB, 0.46371E+00_JPRB, 0.46657E+00_JPRB, 0.47042E+00_JPRB ,&
&  0.47464E+00_JPRB, 0.47896E+00_JPRB, 0.48083E+00_JPRB, 0.47842E+00_JPRB, 0.47208E+00_JPRB, 0.45852E+00_JPRB, 0.44660E+00_JPRB ,&
&  0.43879E+00_JPRB, 0.43291E+00_JPRB, 0.42986E+00_JPRB, 0.42900E+00_JPRB, 0.42672E+00_JPRB, 0.42269E+00_JPRB, 0.41719E+00_JPRB ,&
&  0.41237E+00_JPRB, 0.41006E+00_JPRB, 0.41208E+00_JPRB, 0.41365E+00_JPRB, 0.41452E+00_JPRB, 0.41443E+00_JPRB, 0.41182E+00_JPRB ,&
&  0.41054E+00_JPRB, 0.41048E+00_JPRB, 0.41071E+00_JPRB, 0.41279E+00_JPRB, 0.41095E+00_JPRB, 0.40940E+00_JPRB, 0.41178E+00_JPRB ,&
&  0.41648E+00_JPRB, 0.42044E+00_JPRB, 0.42392E+00_JPRB, 0.42645E+00_JPRB, 0.42741E+00_JPRB, 0.42943E+00_JPRB, 0.43081E+00_JPRB ,&
&  0.43003E+00_JPRB, 0.42668E+00_JPRB, 0.42358E+00_JPRB, 0.42438E+00_JPRB, 0.42700E+00_JPRB, 0.42990E+00_JPRB, 0.43356E+00_JPRB ,&
&  0.43424E+00_JPRB, 0.42915E+00_JPRB, 0.42402E+00_JPRB, 0.41941E+00_JPRB, 0.41511E+00_JPRB, 0.41167E+00_JPRB, 0.40987E+00_JPRB ,&
&  0.40922E+00_JPRB, 0.40610E+00_JPRB, 0.40018E+00_JPRB, 0.39525E+00_JPRB, 0.39429E+00_JPRB, 0.39292E+00_JPRB, 0.38874E+00_JPRB ,&
&  0.38879E+00_JPRB, 0.39096E+00_JPRB, 0.38974E+00_JPRB, 0.38784E+00_JPRB, 0.38986E+00_JPRB, 0.39510E+00_JPRB, 0.40578E+00_JPRB ,&
&  0.42008E+00_JPRB, 0.43657E+00_JPRB, 0.45186E+00_JPRB, 0.46531E+00_JPRB, 0.47346E+00_JPRB, 0.47554E+00_JPRB, 0.47389E+00_JPRB ,&
&  0.47321E+00_JPRB, 0.47379E+00_JPRB, 0.47263E+00_JPRB, 0.47079E+00_JPRB, 0.47088E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.50080E+00_JPRB, 0.49934E+00_JPRB, 0.49782E+00_JPRB, 0.49595E+00_JPRB, 0.49312E+00_JPRB, 0.49053E+00_JPRB, 0.48597E+00_JPRB ,&
&  0.48101E+00_JPRB, 0.47681E+00_JPRB, 0.47497E+00_JPRB, 0.47445E+00_JPRB, 0.47585E+00_JPRB, 0.47983E+00_JPRB, 0.48648E+00_JPRB ,&
&  0.48663E+00_JPRB, 0.48020E+00_JPRB, 0.47734E+00_JPRB, 0.47751E+00_JPRB, 0.48000E+00_JPRB, 0.48185E+00_JPRB, 0.48472E+00_JPRB ,&
&  0.48666E+00_JPRB, 0.48733E+00_JPRB, 0.48676E+00_JPRB, 0.48006E+00_JPRB, 0.46355E+00_JPRB, 0.45348E+00_JPRB, 0.44451E+00_JPRB ,&
&  0.43928E+00_JPRB, 0.43481E+00_JPRB, 0.43268E+00_JPRB, 0.43237E+00_JPRB, 0.43114E+00_JPRB, 0.42904E+00_JPRB, 0.42869E+00_JPRB ,&
&  0.42639E+00_JPRB, 0.42344E+00_JPRB, 0.42181E+00_JPRB, 0.42227E+00_JPRB, 0.42397E+00_JPRB, 0.42568E+00_JPRB, 0.42584E+00_JPRB ,&
&  0.42714E+00_JPRB, 0.43124E+00_JPRB, 0.43281E+00_JPRB, 0.43342E+00_JPRB, 0.43361E+00_JPRB, 0.43339E+00_JPRB, 0.43368E+00_JPRB ,&
&  0.43606E+00_JPRB, 0.44108E+00_JPRB, 0.44523E+00_JPRB, 0.44932E+00_JPRB, 0.45096E+00_JPRB, 0.45176E+00_JPRB, 0.45176E+00_JPRB ,&
&  0.44882E+00_JPRB, 0.44498E+00_JPRB, 0.44383E+00_JPRB, 0.44628E+00_JPRB, 0.45115E+00_JPRB, 0.45731E+00_JPRB, 0.46132E+00_JPRB ,&
&  0.46432E+00_JPRB, 0.46529E+00_JPRB, 0.46295E+00_JPRB, 0.46154E+00_JPRB, 0.45865E+00_JPRB, 0.45393E+00_JPRB, 0.45115E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44624E+00_JPRB, 0.44444E+00_JPRB, 0.43572E+00_JPRB, 0.42803E+00_JPRB, 0.42312E+00_JPRB, 0.41980E+00_JPRB ,&
&  0.41848E+00_JPRB, 0.41752E+00_JPRB, 0.41717E+00_JPRB, 0.41842E+00_JPRB, 0.42040E+00_JPRB, 0.42648E+00_JPRB, 0.43887E+00_JPRB ,&
&  0.45758E+00_JPRB, 0.47523E+00_JPRB, 0.48514E+00_JPRB, 0.49267E+00_JPRB, 0.49799E+00_JPRB, 0.50107E+00_JPRB, 0.50397E+00_JPRB ,&
&  0.50790E+00_JPRB, 0.50926E+00_JPRB, 0.50727E+00_JPRB, 0.50446E+00_JPRB, 0.50294E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.53011E+00_JPRB, 0.52174E+00_JPRB, 0.51537E+00_JPRB, 0.51009E+00_JPRB, 0.50562E+00_JPRB, 0.50376E+00_JPRB, 0.50160E+00_JPRB ,&
&  0.49939E+00_JPRB, 0.49666E+00_JPRB, 0.49526E+00_JPRB, 0.49211E+00_JPRB, 0.48920E+00_JPRB, 0.49188E+00_JPRB, 0.49723E+00_JPRB ,&
&  0.49590E+00_JPRB, 0.48944E+00_JPRB, 0.48434E+00_JPRB, 0.48438E+00_JPRB, 0.48794E+00_JPRB, 0.49064E+00_JPRB, 0.49383E+00_JPRB ,&
&  0.49377E+00_JPRB, 0.49321E+00_JPRB, 0.49193E+00_JPRB, 0.47671E+00_JPRB, 0.46177E+00_JPRB, 0.45442E+00_JPRB, 0.45016E+00_JPRB ,&
&  0.44950E+00_JPRB, 0.45408E+00_JPRB, 0.43904E+00_JPRB, 0.45019E+00_JPRB, 0.44592E+00_JPRB, 0.43941E+00_JPRB, 0.43559E+00_JPRB ,&
&  0.43306E+00_JPRB, 0.43014E+00_JPRB, 0.42809E+00_JPRB, 0.42993E+00_JPRB, 0.43272E+00_JPRB, 0.43379E+00_JPRB, 0.43599E+00_JPRB ,&
&  0.43627E+00_JPRB, 0.43925E+00_JPRB, 0.44189E+00_JPRB, 0.44732E+00_JPRB, 0.45143E+00_JPRB, 0.45390E+00_JPRB, 0.45706E+00_JPRB ,&
&  0.46143E+00_JPRB, 0.46551E+00_JPRB, 0.46934E+00_JPRB, 0.47253E+00_JPRB, 0.47372E+00_JPRB, 0.47341E+00_JPRB, 0.46930E+00_JPRB ,&
&  0.46602E+00_JPRB, 0.46580E+00_JPRB, 0.46815E+00_JPRB, 0.47301E+00_JPRB, 0.47792E+00_JPRB, 0.48008E+00_JPRB, 0.48151E+00_JPRB ,&
&  0.48336E+00_JPRB, 0.48683E+00_JPRB, 0.48720E+00_JPRB, 0.48557E+00_JPRB, 0.48331E+00_JPRB, 0.48050E+00_JPRB, 0.47586E+00_JPRB ,&
&  0.46996E+00_JPRB, 0.46547E+00_JPRB, 0.46097E+00_JPRB, 0.45039E+00_JPRB, 0.44223E+00_JPRB, 0.44148E+00_JPRB, 0.44297E+00_JPRB ,&
&  0.44249E+00_JPRB, 0.44102E+00_JPRB, 0.44222E+00_JPRB, 0.44779E+00_JPRB, 0.44739E+00_JPRB, 0.45342E+00_JPRB, 0.46659E+00_JPRB ,&
&  0.48045E+00_JPRB, 0.49319E+00_JPRB, 0.50428E+00_JPRB, 0.51622E+00_JPRB, 0.52850E+00_JPRB, 0.53856E+00_JPRB, 0.54744E+00_JPRB ,&
&  0.55202E+00_JPRB, 0.55077E+00_JPRB, 0.54527E+00_JPRB, 0.54157E+00_JPRB, 0.53826E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.58115E+00_JPRB, 0.56609E+00_JPRB, 0.55429E+00_JPRB, 0.54530E+00_JPRB, 0.53921E+00_JPRB, 0.53720E+00_JPRB, 0.53427E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.52980E+00_JPRB, 0.52361E+00_JPRB, 0.51711E+00_JPRB, 0.51642E+00_JPRB, 0.52180E+00_JPRB, 0.52624E+00_JPRB ,&
&  0.52357E+00_JPRB, 0.51351E+00_JPRB, 0.50399E+00_JPRB, 0.50214E+00_JPRB, 0.50527E+00_JPRB, 0.50789E+00_JPRB, 0.50772E+00_JPRB ,&
&  0.50943E+00_JPRB, 0.51044E+00_JPRB, 0.50218E+00_JPRB, 0.48553E+00_JPRB, 0.47757E+00_JPRB, 0.47299E+00_JPRB, 0.47728E+00_JPRB ,&
&  0.48451E+00_JPRB, 0.47838E+00_JPRB, 0.49150E+00_JPRB, 0.49210E+00_JPRB, 0.49061E+00_JPRB, 0.48601E+00_JPRB, 0.47865E+00_JPRB ,&
&  0.47271E+00_JPRB, 0.46667E+00_JPRB, 0.46287E+00_JPRB, 0.46135E+00_JPRB, 0.46233E+00_JPRB, 0.46412E+00_JPRB, 0.46947E+00_JPRB ,&
&  0.47144E+00_JPRB, 0.47025E+00_JPRB, 0.47302E+00_JPRB, 0.47998E+00_JPRB, 0.48842E+00_JPRB, 0.49298E+00_JPRB, 0.49714E+00_JPRB ,&
&  0.50061E+00_JPRB, 0.50254E+00_JPRB, 0.50527E+00_JPRB, 0.50601E+00_JPRB, 0.50233E+00_JPRB, 0.50148E+00_JPRB, 0.50134E+00_JPRB ,&
&  0.50330E+00_JPRB, 0.50629E+00_JPRB, 0.50978E+00_JPRB, 0.51314E+00_JPRB, 0.51449E+00_JPRB, 0.51178E+00_JPRB, 0.50927E+00_JPRB ,&
&  0.50981E+00_JPRB, 0.51297E+00_JPRB, 0.51669E+00_JPRB, 0.51865E+00_JPRB, 0.51743E+00_JPRB, 0.51054E+00_JPRB, 0.50256E+00_JPRB ,&
&  0.49427E+00_JPRB, 0.48582E+00_JPRB, 0.47822E+00_JPRB, 0.47374E+00_JPRB, 0.47040E+00_JPRB, 0.46847E+00_JPRB, 0.46747E+00_JPRB ,&
&  0.46731E+00_JPRB, 0.46762E+00_JPRB, 0.47088E+00_JPRB, 0.47697E+00_JPRB, 0.48005E+00_JPRB, 0.48804E+00_JPRB, 0.49854E+00_JPRB ,&
&  0.51000E+00_JPRB, 0.52176E+00_JPRB, 0.53601E+00_JPRB, 0.55590E+00_JPRB, 0.57870E+00_JPRB, 0.59751E+00_JPRB, 0.61227E+00_JPRB ,&
&  0.62193E+00_JPRB, 0.62229E+00_JPRB, 0.61539E+00_JPRB, 0.60687E+00_JPRB, 0.59568E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.63486E+00_JPRB, 0.61174E+00_JPRB, 0.59204E+00_JPRB, 0.57725E+00_JPRB, 0.56793E+00_JPRB, 0.56474E+00_JPRB, 0.56071E+00_JPRB ,&
&  0.55823E+00_JPRB, 0.55473E+00_JPRB, 0.55249E+00_JPRB, 0.55463E+00_JPRB, 0.55567E+00_JPRB, 0.55609E+00_JPRB, 0.55743E+00_JPRB ,&
&  0.55218E+00_JPRB, 0.53976E+00_JPRB, 0.52975E+00_JPRB, 0.52712E+00_JPRB, 0.52833E+00_JPRB, 0.52463E+00_JPRB, 0.52332E+00_JPRB ,&
&  0.52632E+00_JPRB, 0.51843E+00_JPRB, 0.50514E+00_JPRB, 0.49646E+00_JPRB, 0.49678E+00_JPRB, 0.49436E+00_JPRB, 0.49165E+00_JPRB ,&
&  0.51282E+00_JPRB, 0.46649E+00_JPRB, 0.51389E+00_JPRB, 0.52597E+00_JPRB, 0.53253E+00_JPRB, 0.53441E+00_JPRB, 0.53049E+00_JPRB ,&
&  0.52400E+00_JPRB, 0.51747E+00_JPRB, 0.51371E+00_JPRB, 0.51110E+00_JPRB, 0.50951E+00_JPRB, 0.50597E+00_JPRB, 0.50223E+00_JPRB ,&
&  0.50269E+00_JPRB, 0.50358E+00_JPRB, 0.50777E+00_JPRB, 0.51286E+00_JPRB, 0.51967E+00_JPRB, 0.52152E+00_JPRB, 0.52324E+00_JPRB ,&
&  0.52558E+00_JPRB, 0.52696E+00_JPRB, 0.52663E+00_JPRB, 0.52530E+00_JPRB, 0.52363E+00_JPRB, 0.52806E+00_JPRB, 0.53416E+00_JPRB ,&
&  0.53886E+00_JPRB, 0.54339E+00_JPRB, 0.54953E+00_JPRB, 0.55179E+00_JPRB, 0.55186E+00_JPRB, 0.54790E+00_JPRB, 0.54160E+00_JPRB ,&
&  0.53696E+00_JPRB, 0.53572E+00_JPRB, 0.53526E+00_JPRB, 0.53042E+00_JPRB, 0.52332E+00_JPRB, 0.51687E+00_JPRB, 0.50903E+00_JPRB ,&
&  0.49906E+00_JPRB, 0.49117E+00_JPRB, 0.48722E+00_JPRB, 0.48636E+00_JPRB, 0.48778E+00_JPRB, 0.48867E+00_JPRB, 0.48950E+00_JPRB ,&
&  0.49236E+00_JPRB, 0.49335E+00_JPRB, 0.49532E+00_JPRB, 0.50028E+00_JPRB, 0.50467E+00_JPRB, 0.51136E+00_JPRB, 0.52059E+00_JPRB ,&
&  0.53321E+00_JPRB, 0.54807E+00_JPRB, 0.56188E+00_JPRB, 0.58517E+00_JPRB, 0.61785E+00_JPRB, 0.64350E+00_JPRB, 0.65816E+00_JPRB ,&
&  0.67078E+00_JPRB, 0.67446E+00_JPRB, 0.66974E+00_JPRB, 0.66379E+00_JPRB, 0.65134E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.69213E+00_JPRB, 0.67442E+00_JPRB, 0.65131E+00_JPRB, 0.63342E+00_JPRB, 0.62242E+00_JPRB, 0.61654E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.60410E+00_JPRB, 0.60304E+00_JPRB, 0.60296E+00_JPRB, 0.60303E+00_JPRB, 0.60305E+00_JPRB, 0.59990E+00_JPRB, 0.59244E+00_JPRB ,&
&  0.58385E+00_JPRB, 0.57260E+00_JPRB, 0.57071E+00_JPRB, 0.56999E+00_JPRB, 0.56373E+00_JPRB, 0.55940E+00_JPRB, 0.55894E+00_JPRB ,&
&  0.55251E+00_JPRB, 0.54255E+00_JPRB, 0.52460E+00_JPRB, 0.51868E+00_JPRB, 0.52111E+00_JPRB, 0.52108E+00_JPRB, 0.52758E+00_JPRB ,&
&  0.50908E+00_JPRB, 0.48898E+00_JPRB, 0.54242E+00_JPRB, 0.58469E+00_JPRB, 0.60647E+00_JPRB, 0.61848E+00_JPRB, 0.62838E+00_JPRB ,&
&  0.62225E+00_JPRB, 0.62855E+00_JPRB, 0.61827E+00_JPRB, 0.59362E+00_JPRB, 0.57541E+00_JPRB, 0.56547E+00_JPRB, 0.56004E+00_JPRB ,&
&  0.56198E+00_JPRB, 0.56359E+00_JPRB, 0.56276E+00_JPRB, 0.56120E+00_JPRB, 0.56104E+00_JPRB, 0.55682E+00_JPRB, 0.55384E+00_JPRB ,&
&  0.55464E+00_JPRB, 0.55730E+00_JPRB, 0.55828E+00_JPRB, 0.56199E+00_JPRB, 0.57025E+00_JPRB, 0.58312E+00_JPRB, 0.59504E+00_JPRB ,&
&  0.60407E+00_JPRB, 0.61143E+00_JPRB, 0.62175E+00_JPRB, 0.62725E+00_JPRB, 0.62507E+00_JPRB, 0.61418E+00_JPRB, 0.60063E+00_JPRB ,&
&  0.58709E+00_JPRB, 0.57733E+00_JPRB, 0.57049E+00_JPRB, 0.56560E+00_JPRB, 0.55779E+00_JPRB, 0.55013E+00_JPRB, 0.54129E+00_JPRB ,&
&  0.53386E+00_JPRB, 0.52577E+00_JPRB, 0.52158E+00_JPRB, 0.51968E+00_JPRB, 0.52115E+00_JPRB, 0.52432E+00_JPRB, 0.52894E+00_JPRB ,&
&  0.53219E+00_JPRB, 0.53063E+00_JPRB, 0.53059E+00_JPRB, 0.53555E+00_JPRB, 0.54154E+00_JPRB, 0.55168E+00_JPRB, 0.56544E+00_JPRB ,&
&  0.58216E+00_JPRB, 0.59873E+00_JPRB, 0.61550E+00_JPRB, 0.65258E+00_JPRB, 0.69068E+00_JPRB, 0.72044E+00_JPRB, 0.73529E+00_JPRB ,&
&  0.74269E+00_JPRB, 0.73394E+00_JPRB, 0.73222E+00_JPRB, 0.72058E+00_JPRB, 0.70517E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.69214E+00_JPRB, 0.68734E+00_JPRB, 0.67263E+00_JPRB, 0.66300E+00_JPRB, 0.65568E+00_JPRB, 0.64767E+00_JPRB, 0.63614E+00_JPRB ,&
&  0.62803E+00_JPRB, 0.62494E+00_JPRB, 0.62255E+00_JPRB, 0.62226E+00_JPRB, 0.62270E+00_JPRB, 0.61784E+00_JPRB, 0.61012E+00_JPRB ,&
&  0.60197E+00_JPRB, 0.59423E+00_JPRB, 0.59536E+00_JPRB, 0.59463E+00_JPRB, 0.58962E+00_JPRB, 0.58538E+00_JPRB, 0.57978E+00_JPRB ,&
&  0.56541E+00_JPRB, 0.55104E+00_JPRB, 0.53968E+00_JPRB, 0.54232E+00_JPRB, 0.54869E+00_JPRB, 0.54383E+00_JPRB, 0.55822E+00_JPRB ,&
&  0.60916E+00_JPRB, 0.53848E+00_JPRB, 0.57115E+00_JPRB, 0.65203E+00_JPRB, 0.68917E+00_JPRB, 0.72788E+00_JPRB, 0.75851E+00_JPRB ,&
&  0.77852E+00_JPRB, 0.79616E+00_JPRB, 0.76163E+00_JPRB, 0.71552E+00_JPRB, 0.67423E+00_JPRB, 0.64112E+00_JPRB, 0.61693E+00_JPRB ,&
&  0.60439E+00_JPRB, 0.60327E+00_JPRB, 0.60305E+00_JPRB, 0.59818E+00_JPRB, 0.59425E+00_JPRB, 0.58867E+00_JPRB, 0.58482E+00_JPRB ,&
&  0.58241E+00_JPRB, 0.58314E+00_JPRB, 0.58655E+00_JPRB, 0.59644E+00_JPRB, 0.60599E+00_JPRB, 0.61628E+00_JPRB, 0.63010E+00_JPRB ,&
&  0.64946E+00_JPRB, 0.67247E+00_JPRB, 0.69187E+00_JPRB, 0.69746E+00_JPRB, 0.69064E+00_JPRB, 0.67771E+00_JPRB, 0.66044E+00_JPRB ,&
&  0.64070E+00_JPRB, 0.62210E+00_JPRB, 0.61080E+00_JPRB, 0.60496E+00_JPRB, 0.59839E+00_JPRB, 0.59199E+00_JPRB, 0.58644E+00_JPRB ,&
&  0.58156E+00_JPRB, 0.57560E+00_JPRB, 0.57558E+00_JPRB, 0.57562E+00_JPRB, 0.57458E+00_JPRB, 0.56898E+00_JPRB, 0.56313E+00_JPRB ,&
&  0.55797E+00_JPRB, 0.55446E+00_JPRB, 0.55579E+00_JPRB, 0.56202E+00_JPRB, 0.57138E+00_JPRB, 0.58759E+00_JPRB, 0.60835E+00_JPRB ,&
&  0.62474E+00_JPRB, 0.64112E+00_JPRB, 0.65683E+00_JPRB, 0.70378E+00_JPRB, 0.73447E+00_JPRB, 0.75280E+00_JPRB, 0.76117E+00_JPRB ,&
&  0.76118E+00_JPRB, 0.76247E+00_JPRB, 0.59662E+00_JPRB, 0.66525E+00_JPRB, 0.68347E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.70661E+00_JPRB, 0.70098E+00_JPRB, 0.69174E+00_JPRB, 0.68763E+00_JPRB, 0.68565E+00_JPRB, 0.67174E+00_JPRB, 0.65988E+00_JPRB ,&
&  0.65257E+00_JPRB, 0.65194E+00_JPRB, 0.65422E+00_JPRB, 0.65464E+00_JPRB, 0.64933E+00_JPRB, 0.63940E+00_JPRB, 0.63395E+00_JPRB ,&
&  0.62524E+00_JPRB, 0.61731E+00_JPRB, 0.61529E+00_JPRB, 0.61459E+00_JPRB, 0.60904E+00_JPRB, 0.60124E+00_JPRB, 0.59456E+00_JPRB ,&
&  0.58091E+00_JPRB, 0.57211E+00_JPRB, 0.57021E+00_JPRB, 0.57326E+00_JPRB, 0.57660E+00_JPRB, 0.57531E+00_JPRB, 0.59214E+00_JPRB ,&
&  0.63678E+00_JPRB, 0.58446E+00_JPRB, 0.60318E+00_JPRB, 0.75419E+00_JPRB, 0.84237E+00_JPRB, 0.86051E+00_JPRB, 0.88182E+00_JPRB ,&
&  0.90337E+00_JPRB, 0.89674E+00_JPRB, 0.86981E+00_JPRB, 0.81359E+00_JPRB, 0.77010E+00_JPRB, 0.73884E+00_JPRB, 0.71120E+00_JPRB ,&
&  0.69041E+00_JPRB, 0.67423E+00_JPRB, 0.66641E+00_JPRB, 0.65650E+00_JPRB, 0.65198E+00_JPRB, 0.64234E+00_JPRB, 0.63927E+00_JPRB ,&
&  0.64009E+00_JPRB, 0.64595E+00_JPRB, 0.65190E+00_JPRB, 0.65469E+00_JPRB, 0.66188E+00_JPRB, 0.67380E+00_JPRB, 0.69748E+00_JPRB ,&
&  0.73145E+00_JPRB, 0.77064E+00_JPRB, 0.79100E+00_JPRB, 0.78970E+00_JPRB, 0.77776E+00_JPRB, 0.75928E+00_JPRB, 0.73453E+00_JPRB ,&
&  0.70608E+00_JPRB, 0.67936E+00_JPRB, 0.66074E+00_JPRB, 0.65175E+00_JPRB, 0.64731E+00_JPRB, 0.64449E+00_JPRB, 0.64103E+00_JPRB ,&
&  0.63770E+00_JPRB, 0.63694E+00_JPRB, 0.63750E+00_JPRB, 0.63628E+00_JPRB, 0.63424E+00_JPRB, 0.62466E+00_JPRB, 0.60935E+00_JPRB ,&
&  0.59835E+00_JPRB, 0.59519E+00_JPRB, 0.60260E+00_JPRB, 0.60905E+00_JPRB, 0.61757E+00_JPRB, 0.63617E+00_JPRB, 0.65734E+00_JPRB ,&
&  0.67279E+00_JPRB, 0.69316E+00_JPRB, 0.71796E+00_JPRB, 0.74964E+00_JPRB, 0.79401E+00_JPRB, 0.79036E+00_JPRB, 0.79098E+00_JPRB ,&
&  0.78001E+00_JPRB, 0.75901E+00_JPRB, 0.74165E+00_JPRB, 0.62882E+00_JPRB, 0.69069E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.72164E+00_JPRB, 0.71572E+00_JPRB, 0.70760E+00_JPRB, 0.70291E+00_JPRB, 0.69766E+00_JPRB, 0.68550E+00_JPRB, 0.67567E+00_JPRB ,&
&  0.67242E+00_JPRB, 0.66766E+00_JPRB, 0.66301E+00_JPRB, 0.65572E+00_JPRB, 0.64565E+00_JPRB, 0.63869E+00_JPRB, 0.63861E+00_JPRB ,&
&  0.63579E+00_JPRB, 0.63458E+00_JPRB, 0.63383E+00_JPRB, 0.62908E+00_JPRB, 0.62300E+00_JPRB, 0.61389E+00_JPRB, 0.59939E+00_JPRB ,&
&  0.58175E+00_JPRB, 0.57640E+00_JPRB, 0.58495E+00_JPRB, 0.59197E+00_JPRB, 0.59637E+00_JPRB, 0.59866E+00_JPRB, 0.60945E+00_JPRB ,&
&  0.64798E+00_JPRB, 0.66495E+00_JPRB, 0.79927E+00_JPRB, 0.99188E+00_JPRB, 0.10819E+01_JPRB, 0.10512E+01_JPRB, 0.10281E+01_JPRB ,&
&  0.98139E+00_JPRB, 0.93314E+00_JPRB, 0.88302E+00_JPRB, 0.82723E+00_JPRB, 0.78122E+00_JPRB, 0.75133E+00_JPRB, 0.73618E+00_JPRB ,&
&  0.72597E+00_JPRB, 0.71611E+00_JPRB, 0.71047E+00_JPRB, 0.70034E+00_JPRB, 0.69189E+00_JPRB, 0.68625E+00_JPRB, 0.68764E+00_JPRB ,&
&  0.69050E+00_JPRB, 0.69453E+00_JPRB, 0.69610E+00_JPRB, 0.69675E+00_JPRB, 0.69743E+00_JPRB, 0.70862E+00_JPRB, 0.74227E+00_JPRB ,&
&  0.78978E+00_JPRB, 0.83686E+00_JPRB, 0.85216E+00_JPRB, 0.84072E+00_JPRB, 0.83103E+00_JPRB, 0.82333E+00_JPRB, 0.79398E+00_JPRB ,&
&  0.75387E+00_JPRB, 0.72805E+00_JPRB, 0.70850E+00_JPRB, 0.69233E+00_JPRB, 0.68135E+00_JPRB, 0.67550E+00_JPRB, 0.67141E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.66708E+00_JPRB, 0.66429E+00_JPRB, 0.66263E+00_JPRB, 0.66068E+00_JPRB, 0.65226E+00_JPRB, 0.64177E+00_JPRB ,&
&  0.63052E+00_JPRB, 0.63174E+00_JPRB, 0.64416E+00_JPRB, 0.65091E+00_JPRB, 0.65700E+00_JPRB, 0.67055E+00_JPRB, 0.68989E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.72781E+00_JPRB, 0.74477E+00_JPRB, 0.81924E+00_JPRB, 0.83258E+00_JPRB, 0.81511E+00_JPRB, 0.80398E+00_JPRB ,&
&  0.78019E+00_JPRB, 0.74916E+00_JPRB, 0.72414E+00_JPRB, 0.72965E+00_JPRB, 0.68754E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.81058E+00_JPRB, 0.77164E+00_JPRB, 0.73374E+00_JPRB, 0.70983E+00_JPRB, 0.69926E+00_JPRB, 0.69481E+00_JPRB, 0.69176E+00_JPRB ,&
&  0.68691E+00_JPRB, 0.68040E+00_JPRB, 0.66341E+00_JPRB, 0.65107E+00_JPRB, 0.64282E+00_JPRB, 0.64058E+00_JPRB, 0.63856E+00_JPRB ,&
&  0.63903E+00_JPRB, 0.63820E+00_JPRB, 0.63329E+00_JPRB, 0.63172E+00_JPRB, 0.63282E+00_JPRB, 0.62770E+00_JPRB, 0.61129E+00_JPRB ,&
&  0.59486E+00_JPRB, 0.59644E+00_JPRB, 0.60128E+00_JPRB, 0.60439E+00_JPRB, 0.61070E+00_JPRB, 0.62003E+00_JPRB, 0.63615E+00_JPRB ,&
&  0.67110E+00_JPRB, 0.72156E+00_JPRB, 0.10323E+01_JPRB, 0.12269E+01_JPRB, 0.13041E+01_JPRB, 0.13013E+01_JPRB, 0.12060E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.10086E+01_JPRB, 0.92670E+00_JPRB, 0.84833E+00_JPRB, 0.79763E+00_JPRB, 0.76305E+00_JPRB, 0.74024E+00_JPRB ,&
&  0.72850E+00_JPRB, 0.72345E+00_JPRB, 0.71943E+00_JPRB, 0.71420E+00_JPRB, 0.71077E+00_JPRB, 0.70923E+00_JPRB, 0.70959E+00_JPRB ,&
&  0.71246E+00_JPRB, 0.71874E+00_JPRB, 0.72092E+00_JPRB, 0.71924E+00_JPRB, 0.72058E+00_JPRB, 0.76348E+00_JPRB, 0.83916E+00_JPRB ,&
&  0.90642E+00_JPRB, 0.94277E+00_JPRB, 0.93034E+00_JPRB, 0.91865E+00_JPRB, 0.91138E+00_JPRB, 0.88563E+00_JPRB, 0.85460E+00_JPRB ,&
&  0.81988E+00_JPRB, 0.78443E+00_JPRB, 0.74714E+00_JPRB, 0.71806E+00_JPRB, 0.70110E+00_JPRB, 0.69269E+00_JPRB, 0.68868E+00_JPRB ,&
&  0.68973E+00_JPRB, 0.69514E+00_JPRB, 0.69487E+00_JPRB, 0.69581E+00_JPRB, 0.69152E+00_JPRB, 0.68622E+00_JPRB, 0.68384E+00_JPRB ,&
&  0.68145E+00_JPRB, 0.67928E+00_JPRB, 0.69263E+00_JPRB, 0.70123E+00_JPRB, 0.70769E+00_JPRB, 0.71785E+00_JPRB, 0.72944E+00_JPRB ,&
&  0.74475E+00_JPRB, 0.78047E+00_JPRB, 0.77095E+00_JPRB, 0.83901E+00_JPRB, 0.91952E+00_JPRB, 0.86804E+00_JPRB, 0.81233E+00_JPRB ,&
&  0.76430E+00_JPRB, 0.73529E+00_JPRB, 0.72628E+00_JPRB, 0.75290E+00_JPRB, 0.80630E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.77470E+00_JPRB, 0.72671E+00_JPRB, 0.70062E+00_JPRB, 0.69660E+00_JPRB, 0.70118E+00_JPRB, 0.69674E+00_JPRB, 0.69901E+00_JPRB ,&
&  0.69193E+00_JPRB, 0.67810E+00_JPRB, 0.66290E+00_JPRB, 0.64908E+00_JPRB, 0.64269E+00_JPRB, 0.63777E+00_JPRB, 0.63401E+00_JPRB ,&
&  0.63151E+00_JPRB, 0.62927E+00_JPRB, 0.62403E+00_JPRB, 0.62702E+00_JPRB, 0.62867E+00_JPRB, 0.62679E+00_JPRB, 0.60531E+00_JPRB ,&
&  0.59523E+00_JPRB, 0.59667E+00_JPRB, 0.60155E+00_JPRB, 0.60612E+00_JPRB, 0.61370E+00_JPRB, 0.62566E+00_JPRB, 0.63939E+00_JPRB ,&
&  0.67471E+00_JPRB, 0.80446E+00_JPRB, 0.11763E+01_JPRB, 0.14134E+01_JPRB, 0.15550E+01_JPRB, 0.14466E+01_JPRB, 0.12890E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.10999E+01_JPRB, 0.10393E+01_JPRB, 0.93487E+00_JPRB, 0.83408E+00_JPRB, 0.78037E+00_JPRB, 0.75356E+00_JPRB ,&
&  0.74172E+00_JPRB, 0.73862E+00_JPRB, 0.73452E+00_JPRB, 0.73072E+00_JPRB, 0.72625E+00_JPRB, 0.72557E+00_JPRB, 0.72999E+00_JPRB ,&
&  0.73572E+00_JPRB, 0.73353E+00_JPRB, 0.73195E+00_JPRB, 0.72531E+00_JPRB, 0.79953E+00_JPRB, 0.88876E+00_JPRB, 0.10082E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.10763E+01_JPRB, 0.10452E+01_JPRB, 0.99272E+00_JPRB, 0.94218E+00_JPRB, 0.89844E+00_JPRB, 0.86077E+00_JPRB ,&
&  0.82599E+00_JPRB, 0.79447E+00_JPRB, 0.76818E+00_JPRB, 0.75676E+00_JPRB, 0.75491E+00_JPRB, 0.74389E+00_JPRB, 0.73825E+00_JPRB ,&
&  0.73855E+00_JPRB, 0.74182E+00_JPRB, 0.73990E+00_JPRB, 0.72882E+00_JPRB, 0.71365E+00_JPRB, 0.70548E+00_JPRB, 0.70305E+00_JPRB ,&
&  0.71100E+00_JPRB, 0.71525E+00_JPRB, 0.74415E+00_JPRB, 0.74971E+00_JPRB, 0.75295E+00_JPRB, 0.75704E+00_JPRB, 0.77058E+00_JPRB ,&
&  0.79823E+00_JPRB, 0.83026E+00_JPRB, 0.82920E+00_JPRB, 0.85858E+00_JPRB, 0.89224E+00_JPRB, 0.89455E+00_JPRB, 0.80210E+00_JPRB ,&
&  0.73943E+00_JPRB, 0.71382E+00_JPRB, 0.71338E+00_JPRB, 0.73879E+00_JPRB, 0.78759E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.73545E+00_JPRB, 0.70806E+00_JPRB, 0.69692E+00_JPRB, 0.69616E+00_JPRB, 0.68999E+00_JPRB, 0.69152E+00_JPRB, 0.69519E+00_JPRB ,&
&  0.69270E+00_JPRB, 0.67856E+00_JPRB, 0.66545E+00_JPRB, 0.65082E+00_JPRB, 0.64578E+00_JPRB, 0.63833E+00_JPRB, 0.63653E+00_JPRB ,&
&  0.62958E+00_JPRB, 0.62631E+00_JPRB, 0.62007E+00_JPRB, 0.61818E+00_JPRB, 0.61802E+00_JPRB, 0.61309E+00_JPRB, 0.59389E+00_JPRB ,&
&  0.58952E+00_JPRB, 0.58872E+00_JPRB, 0.59635E+00_JPRB, 0.60726E+00_JPRB, 0.62044E+00_JPRB, 0.62780E+00_JPRB, 0.63534E+00_JPRB ,&
&  0.69369E+00_JPRB, 0.81450E+00_JPRB, 0.14242E+01_JPRB, 0.15804E+01_JPRB, 0.15955E+01_JPRB, 0.13892E+01_JPRB, 0.12579E+01_JPRB ,&
&  0.11660E+01_JPRB, 0.11195E+01_JPRB, 0.10961E+01_JPRB, 0.10092E+01_JPRB, 0.89844E+00_JPRB, 0.81466E+00_JPRB, 0.78842E+00_JPRB ,&
&  0.77448E+00_JPRB, 0.76164E+00_JPRB, 0.75624E+00_JPRB, 0.75601E+00_JPRB, 0.75220E+00_JPRB, 0.75078E+00_JPRB, 0.75849E+00_JPRB ,&
&  0.76191E+00_JPRB, 0.76325E+00_JPRB, 0.76610E+00_JPRB, 0.80189E+00_JPRB, 0.89159E+00_JPRB, 0.85557E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.12643E+01_JPRB, 0.11509E+01_JPRB, 0.10087E+01_JPRB, 0.92287E+00_JPRB, 0.88783E+00_JPRB, 0.86080E+00_JPRB ,&
&  0.83779E+00_JPRB, 0.81473E+00_JPRB, 0.79566E+00_JPRB, 0.79597E+00_JPRB, 0.79111E+00_JPRB, 0.77903E+00_JPRB, 0.77070E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.77712E+00_JPRB, 0.76570E+00_JPRB, 0.74207E+00_JPRB, 0.72624E+00_JPRB, 0.72468E+00_JPRB, 0.72305E+00_JPRB ,&
&  0.72519E+00_JPRB, 0.73538E+00_JPRB, 0.76559E+00_JPRB, 0.74271E+00_JPRB, 0.75840E+00_JPRB, 0.79321E+00_JPRB, 0.81072E+00_JPRB ,&
&  0.80891E+00_JPRB, 0.82440E+00_JPRB, 0.85425E+00_JPRB, 0.85335E+00_JPRB, 0.90399E+00_JPRB, 0.88996E+00_JPRB, 0.77506E+00_JPRB ,&
&  0.71359E+00_JPRB, 0.69900E+00_JPRB, 0.71950E+00_JPRB, 0.74856E+00_JPRB, 0.78479E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.71110E+00_JPRB, 0.69410E+00_JPRB, 0.68817E+00_JPRB, 0.68167E+00_JPRB, 0.66999E+00_JPRB, 0.65765E+00_JPRB, 0.65956E+00_JPRB ,&
&  0.66387E+00_JPRB, 0.66694E+00_JPRB, 0.65884E+00_JPRB, 0.64366E+00_JPRB, 0.64126E+00_JPRB, 0.63774E+00_JPRB, 0.63284E+00_JPRB ,&
&  0.62827E+00_JPRB, 0.62756E+00_JPRB, 0.61959E+00_JPRB, 0.61489E+00_JPRB, 0.61679E+00_JPRB, 0.61418E+00_JPRB, 0.60607E+00_JPRB ,&
&  0.59739E+00_JPRB, 0.59220E+00_JPRB, 0.59745E+00_JPRB, 0.60285E+00_JPRB, 0.61585E+00_JPRB, 0.61993E+00_JPRB, 0.62585E+00_JPRB ,&
&  0.69268E+00_JPRB, 0.63580E+00_JPRB, 0.13800E+01_JPRB, 0.18754E+01_JPRB, 0.17696E+01_JPRB, 0.14386E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.11366E+01_JPRB, 0.10843E+01_JPRB, 0.10114E+01_JPRB, 0.94223E+00_JPRB, 0.87097E+00_JPRB, 0.83881E+00_JPRB ,&
&  0.81787E+00_JPRB, 0.79602E+00_JPRB, 0.79001E+00_JPRB, 0.78784E+00_JPRB, 0.79155E+00_JPRB, 0.79305E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.80965E+00_JPRB, 0.82013E+00_JPRB, 0.86468E+00_JPRB, 0.94988E+00_JPRB, 0.10041E+01_JPRB, 0.95650E+00_JPRB, 0.10350E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.12782E+01_JPRB, 0.11385E+01_JPRB, 0.97639E+00_JPRB, 0.89982E+00_JPRB, 0.87462E+00_JPRB, 0.84945E+00_JPRB ,&
&  0.82607E+00_JPRB, 0.80703E+00_JPRB, 0.80946E+00_JPRB, 0.80361E+00_JPRB, 0.78551E+00_JPRB, 0.76502E+00_JPRB, 0.77086E+00_JPRB ,&
&  0.77947E+00_JPRB, 0.77466E+00_JPRB, 0.75715E+00_JPRB, 0.74829E+00_JPRB, 0.74112E+00_JPRB, 0.72717E+00_JPRB, 0.72230E+00_JPRB ,&
&  0.72072E+00_JPRB, 0.74075E+00_JPRB, 0.75557E+00_JPRB, 0.75411E+00_JPRB, 0.76406E+00_JPRB, 0.78773E+00_JPRB, 0.81193E+00_JPRB ,&
&  0.84569E+00_JPRB, 0.86091E+00_JPRB, 0.84273E+00_JPRB, 0.85000E+00_JPRB, 0.81322E+00_JPRB, 0.82453E+00_JPRB, 0.73178E+00_JPRB ,&
&  0.70113E+00_JPRB, 0.69738E+00_JPRB, 0.71013E+00_JPRB, 0.72173E+00_JPRB, 0.73731E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.68571E+00_JPRB, 0.67668E+00_JPRB, 0.67493E+00_JPRB, 0.66553E+00_JPRB, 0.64834E+00_JPRB, 0.63329E+00_JPRB, 0.63093E+00_JPRB ,&
&  0.62875E+00_JPRB, 0.63293E+00_JPRB, 0.64063E+00_JPRB, 0.63215E+00_JPRB, 0.63182E+00_JPRB, 0.63530E+00_JPRB, 0.63491E+00_JPRB ,&
&  0.62700E+00_JPRB, 0.62317E+00_JPRB, 0.61876E+00_JPRB, 0.61525E+00_JPRB, 0.61367E+00_JPRB, 0.60438E+00_JPRB, 0.59970E+00_JPRB ,&
&  0.59911E+00_JPRB, 0.60416E+00_JPRB, 0.60891E+00_JPRB, 0.61023E+00_JPRB, 0.61991E+00_JPRB, 0.62390E+00_JPRB, 0.63051E+00_JPRB ,&
&  0.68368E+00_JPRB, 0.12562E+01_JPRB, 0.11222E+01_JPRB, 0.21522E+01_JPRB, 0.18032E+01_JPRB, 0.14316E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11265E+01_JPRB, 0.11415E+01_JPRB, 0.10682E+01_JPRB, 0.10059E+01_JPRB, 0.95531E+00_JPRB, 0.89905E+00_JPRB, 0.84735E+00_JPRB ,&
&  0.80759E+00_JPRB, 0.79019E+00_JPRB, 0.79270E+00_JPRB, 0.80461E+00_JPRB, 0.82878E+00_JPRB, 0.85619E+00_JPRB, 0.87332E+00_JPRB ,&
&  0.90111E+00_JPRB, 0.96530E+00_JPRB, 0.10634E+01_JPRB, 0.11692E+01_JPRB, 0.10771E+01_JPRB, 0.99502E+00_JPRB, 0.10821E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.12173E+01_JPRB, 0.10754E+01_JPRB, 0.93850E+00_JPRB, 0.88826E+00_JPRB, 0.86587E+00_JPRB, 0.83504E+00_JPRB ,&
&  0.80864E+00_JPRB, 0.79918E+00_JPRB, 0.81295E+00_JPRB, 0.79151E+00_JPRB, 0.76332E+00_JPRB, 0.75787E+00_JPRB, 0.75979E+00_JPRB ,&
&  0.76297E+00_JPRB, 0.75218E+00_JPRB, 0.76800E+00_JPRB, 0.77038E+00_JPRB, 0.76450E+00_JPRB, 0.75160E+00_JPRB, 0.74147E+00_JPRB ,&
&  0.74275E+00_JPRB, 0.76297E+00_JPRB, 0.76044E+00_JPRB, 0.75023E+00_JPRB, 0.75736E+00_JPRB, 0.76650E+00_JPRB, 0.77258E+00_JPRB ,&
&  0.83473E+00_JPRB, 0.85038E+00_JPRB, 0.82933E+00_JPRB, 0.81857E+00_JPRB, 0.80284E+00_JPRB, 0.82300E+00_JPRB, 0.71031E+00_JPRB ,&
&  0.69278E+00_JPRB, 0.68850E+00_JPRB, 0.69468E+00_JPRB, 0.70469E+00_JPRB, 0.70301E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.67557E+00_JPRB, 0.66896E+00_JPRB, 0.66713E+00_JPRB, 0.64896E+00_JPRB, 0.62733E+00_JPRB, 0.61300E+00_JPRB, 0.60895E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.62360E+00_JPRB, 0.63242E+00_JPRB, 0.61935E+00_JPRB, 0.61195E+00_JPRB, 0.61879E+00_JPRB, 0.61993E+00_JPRB ,&
&  0.61854E+00_JPRB, 0.61467E+00_JPRB, 0.61284E+00_JPRB, 0.61267E+00_JPRB, 0.60934E+00_JPRB, 0.59902E+00_JPRB, 0.59943E+00_JPRB ,&
&  0.61158E+00_JPRB, 0.62272E+00_JPRB, 0.62710E+00_JPRB, 0.62818E+00_JPRB, 0.64298E+00_JPRB, 0.64887E+00_JPRB, 0.66231E+00_JPRB ,&
&  0.72196E+00_JPRB, 0.12569E+01_JPRB, 0.12189E+01_JPRB, 0.22240E+01_JPRB, 0.18497E+01_JPRB, 0.14983E+01_JPRB, 0.12242E+01_JPRB ,&
&  0.11261E+01_JPRB, 0.11747E+01_JPRB, 0.10545E+01_JPRB, 0.94262E+00_JPRB, 0.88070E+00_JPRB, 0.86358E+00_JPRB, 0.83683E+00_JPRB ,&
&  0.81048E+00_JPRB, 0.82757E+00_JPRB, 0.85587E+00_JPRB, 0.89494E+00_JPRB, 0.93845E+00_JPRB, 0.97049E+00_JPRB, 0.10241E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.12053E+01_JPRB, 0.13502E+01_JPRB, 0.12729E+01_JPRB, 0.11111E+01_JPRB, 0.10692E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.11325E+01_JPRB, 0.99722E+00_JPRB, 0.90562E+00_JPRB, 0.84921E+00_JPRB, 0.83660E+00_JPRB, 0.80409E+00_JPRB ,&
&  0.77330E+00_JPRB, 0.76977E+00_JPRB, 0.76927E+00_JPRB, 0.75832E+00_JPRB, 0.74884E+00_JPRB, 0.74190E+00_JPRB, 0.73313E+00_JPRB ,&
&  0.73590E+00_JPRB, 0.76070E+00_JPRB, 0.76826E+00_JPRB, 0.77272E+00_JPRB, 0.77916E+00_JPRB, 0.78245E+00_JPRB, 0.77381E+00_JPRB ,&
&  0.77618E+00_JPRB, 0.78807E+00_JPRB, 0.76313E+00_JPRB, 0.75845E+00_JPRB, 0.77026E+00_JPRB, 0.76869E+00_JPRB, 0.77149E+00_JPRB ,&
&  0.80750E+00_JPRB, 0.82084E+00_JPRB, 0.79355E+00_JPRB, 0.75234E+00_JPRB, 0.76437E+00_JPRB, 0.78030E+00_JPRB, 0.70174E+00_JPRB ,&
&  0.69362E+00_JPRB, 0.68945E+00_JPRB, 0.69191E+00_JPRB, 0.69015E+00_JPRB, 0.68309E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.64311E+00_JPRB, 0.63676E+00_JPRB, 0.62949E+00_JPRB, 0.59099E+00_JPRB, 0.55866E+00_JPRB, 0.54667E+00_JPRB, 0.56263E+00_JPRB ,&
&  0.59584E+00_JPRB, 0.61865E+00_JPRB, 0.61567E+00_JPRB, 0.60159E+00_JPRB, 0.59523E+00_JPRB, 0.59792E+00_JPRB, 0.60486E+00_JPRB ,&
&  0.60425E+00_JPRB, 0.59415E+00_JPRB, 0.58702E+00_JPRB, 0.59312E+00_JPRB, 0.59475E+00_JPRB, 0.60333E+00_JPRB, 0.62167E+00_JPRB ,&
&  0.64765E+00_JPRB, 0.66083E+00_JPRB, 0.66189E+00_JPRB, 0.66929E+00_JPRB, 0.70116E+00_JPRB, 0.70933E+00_JPRB, 0.72969E+00_JPRB ,&
&  0.85565E+00_JPRB, 0.14062E+01_JPRB, 0.15566E+01_JPRB, 0.19127E+01_JPRB, 0.16883E+01_JPRB, 0.13386E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.92769E+00_JPRB, 0.94245E+00_JPRB, 0.86323E+00_JPRB, 0.86579E+00_JPRB, 0.87127E+00_JPRB, 0.87704E+00_JPRB ,&
&  0.90808E+00_JPRB, 0.97676E+00_JPRB, 0.10233E+01_JPRB, 0.10637E+01_JPRB, 0.11010E+01_JPRB, 0.11595E+01_JPRB, 0.12540E+01_JPRB ,&
&  0.13508E+01_JPRB, 0.14651E+01_JPRB, 0.15558E+01_JPRB, 0.13338E+01_JPRB, 0.10789E+01_JPRB, 0.10722E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10273E+01_JPRB, 0.93746E+00_JPRB, 0.85776E+00_JPRB, 0.77296E+00_JPRB, 0.80395E+00_JPRB, 0.76734E+00_JPRB ,&
&  0.72837E+00_JPRB, 0.73172E+00_JPRB, 0.72691E+00_JPRB, 0.73218E+00_JPRB, 0.73147E+00_JPRB, 0.72359E+00_JPRB, 0.72575E+00_JPRB ,&
&  0.74036E+00_JPRB, 0.75233E+00_JPRB, 0.76425E+00_JPRB, 0.77359E+00_JPRB, 0.78795E+00_JPRB, 0.79858E+00_JPRB, 0.79818E+00_JPRB ,&
&  0.79473E+00_JPRB, 0.80784E+00_JPRB, 0.78812E+00_JPRB, 0.80225E+00_JPRB, 0.80407E+00_JPRB, 0.79687E+00_JPRB, 0.76767E+00_JPRB ,&
&  0.76948E+00_JPRB, 0.78421E+00_JPRB, 0.75451E+00_JPRB, 0.73790E+00_JPRB, 0.74514E+00_JPRB, 0.73452E+00_JPRB, 0.69657E+00_JPRB ,&
&  0.69912E+00_JPRB, 0.69220E+00_JPRB, 0.68466E+00_JPRB, 0.66625E+00_JPRB, 0.65307E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.62542E+00_JPRB, 0.61468E+00_JPRB, 0.58701E+00_JPRB, 0.52929E+00_JPRB, 0.51562E+00_JPRB, 0.53237E+00_JPRB, 0.59200E+00_JPRB ,&
&  0.61667E+00_JPRB, 0.62165E+00_JPRB, 0.60776E+00_JPRB, 0.59025E+00_JPRB, 0.57909E+00_JPRB, 0.57718E+00_JPRB, 0.59027E+00_JPRB ,&
&  0.60270E+00_JPRB, 0.59450E+00_JPRB, 0.59367E+00_JPRB, 0.60077E+00_JPRB, 0.61471E+00_JPRB, 0.63925E+00_JPRB, 0.67076E+00_JPRB ,&
&  0.69955E+00_JPRB, 0.71372E+00_JPRB, 0.72356E+00_JPRB, 0.76194E+00_JPRB, 0.81330E+00_JPRB, 0.83961E+00_JPRB, 0.91186E+00_JPRB ,&
&  0.12434E+01_JPRB, 0.16856E+01_JPRB, 0.80354E+00_JPRB, 0.12090E+01_JPRB, 0.13516E+01_JPRB, 0.11715E+01_JPRB, 0.10367E+01_JPRB ,&
&  0.94679E+00_JPRB, 0.82783E+00_JPRB, 0.81532E+00_JPRB, 0.89404E+00_JPRB, 0.93433E+00_JPRB, 0.99995E+00_JPRB, 0.10730E+01_JPRB ,&
&  0.11368E+01_JPRB, 0.11808E+01_JPRB, 0.12209E+01_JPRB, 0.12591E+01_JPRB, 0.13146E+01_JPRB, 0.13969E+01_JPRB, 0.14737E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.15615E+01_JPRB, 0.15353E+01_JPRB, 0.11886E+01_JPRB, 0.10354E+01_JPRB, 0.10106E+01_JPRB, 0.96523E+00_JPRB ,&
&  0.89888E+00_JPRB, 0.92111E+00_JPRB, 0.87723E+00_JPRB, 0.79748E+00_JPRB, 0.72426E+00_JPRB, 0.69834E+00_JPRB, 0.72072E+00_JPRB ,&
&  0.70253E+00_JPRB, 0.68927E+00_JPRB, 0.68269E+00_JPRB, 0.68632E+00_JPRB, 0.68707E+00_JPRB, 0.69014E+00_JPRB, 0.69899E+00_JPRB ,&
&  0.71996E+00_JPRB, 0.73937E+00_JPRB, 0.75437E+00_JPRB, 0.76450E+00_JPRB, 0.77934E+00_JPRB, 0.79754E+00_JPRB, 0.80293E+00_JPRB ,&
&  0.81870E+00_JPRB, 0.83046E+00_JPRB, 0.84380E+00_JPRB, 0.84062E+00_JPRB, 0.81894E+00_JPRB, 0.80443E+00_JPRB, 0.79885E+00_JPRB ,&
&  0.79532E+00_JPRB, 0.78451E+00_JPRB, 0.72358E+00_JPRB, 0.71226E+00_JPRB, 0.74610E+00_JPRB, 0.73174E+00_JPRB, 0.77588E+00_JPRB ,&
&  0.84170E+00_JPRB, 0.89403E+00_JPRB, 0.84760E+00_JPRB, 0.65507E+00_JPRB, 0.63297E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.60481E+00_JPRB, 0.57064E+00_JPRB, 0.51804E+00_JPRB, 0.48535E+00_JPRB, 0.50896E+00_JPRB, 0.57271E+00_JPRB, 0.62187E+00_JPRB ,&
&  0.63881E+00_JPRB, 0.63533E+00_JPRB, 0.62932E+00_JPRB, 0.62644E+00_JPRB, 0.63554E+00_JPRB, 0.62730E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.64548E+00_JPRB, 0.65076E+00_JPRB, 0.66044E+00_JPRB, 0.67341E+00_JPRB, 0.68943E+00_JPRB, 0.72125E+00_JPRB, 0.75502E+00_JPRB ,&
&  0.78160E+00_JPRB, 0.80753E+00_JPRB, 0.85041E+00_JPRB, 0.91276E+00_JPRB, 0.10032E+01_JPRB, 0.11591E+01_JPRB, 0.14282E+01_JPRB ,&
&  0.19374E+01_JPRB, 0.13504E+01_JPRB, 0.70651E+00_JPRB, 0.11313E+01_JPRB, 0.11726E+01_JPRB, 0.10813E+01_JPRB, 0.94572E+00_JPRB ,&
&  0.91274E+00_JPRB, 0.87448E+00_JPRB, 0.90188E+00_JPRB, 0.10395E+01_JPRB, 0.11570E+01_JPRB, 0.13155E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.14251E+01_JPRB, 0.14377E+01_JPRB, 0.14566E+01_JPRB, 0.14878E+01_JPRB, 0.15385E+01_JPRB, 0.15956E+01_JPRB, 0.16074E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.16066E+01_JPRB, 0.15330E+01_JPRB, 0.11618E+01_JPRB, 0.10154E+01_JPRB, 0.92899E+00_JPRB, 0.88928E+00_JPRB ,&
&  0.82110E+00_JPRB, 0.82534E+00_JPRB, 0.80827E+00_JPRB, 0.75262E+00_JPRB, 0.69724E+00_JPRB, 0.65995E+00_JPRB, 0.68979E+00_JPRB ,&
&  0.68918E+00_JPRB, 0.65466E+00_JPRB, 0.62717E+00_JPRB, 0.61184E+00_JPRB, 0.63476E+00_JPRB, 0.65731E+00_JPRB, 0.65727E+00_JPRB ,&
&  0.67191E+00_JPRB, 0.69518E+00_JPRB, 0.71300E+00_JPRB, 0.73025E+00_JPRB, 0.75506E+00_JPRB, 0.78235E+00_JPRB, 0.79959E+00_JPRB ,&
&  0.81687E+00_JPRB, 0.82191E+00_JPRB, 0.82805E+00_JPRB, 0.82225E+00_JPRB, 0.78623E+00_JPRB, 0.74827E+00_JPRB, 0.75235E+00_JPRB ,&
&  0.74575E+00_JPRB, 0.74507E+00_JPRB, 0.72781E+00_JPRB, 0.74060E+00_JPRB, 0.74656E+00_JPRB, 0.77229E+00_JPRB, 0.80480E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.81742E+00_JPRB, 0.73506E+00_JPRB, 0.64232E+00_JPRB, 0.62105E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.56316E+00_JPRB, 0.50137E+00_JPRB, 0.45056E+00_JPRB, 0.46709E+00_JPRB, 0.52933E+00_JPRB, 0.58169E+00_JPRB, 0.60826E+00_JPRB ,&
&  0.63842E+00_JPRB, 0.64563E+00_JPRB, 0.65197E+00_JPRB, 0.66095E+00_JPRB, 0.67948E+00_JPRB, 0.68117E+00_JPRB, 0.69587E+00_JPRB ,&
&  0.70025E+00_JPRB, 0.69975E+00_JPRB, 0.70716E+00_JPRB, 0.73575E+00_JPRB, 0.77158E+00_JPRB, 0.80949E+00_JPRB, 0.84770E+00_JPRB ,&
&  0.89210E+00_JPRB, 0.94169E+00_JPRB, 0.10140E+01_JPRB, 0.11003E+01_JPRB, 0.12159E+01_JPRB, 0.13405E+01_JPRB, 0.14323E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.57067E+00_JPRB, 0.98421E+00_JPRB, 0.11577E+01_JPRB, 0.10793E+01_JPRB, 0.10261E+01_JPRB, 0.97939E+00_JPRB ,&
&  0.94505E+00_JPRB, 0.94677E+00_JPRB, 0.99120E+00_JPRB, 0.12749E+01_JPRB, 0.14854E+01_JPRB, 0.15759E+01_JPRB, 0.16129E+01_JPRB ,&
&  0.16419E+01_JPRB, 0.16600E+01_JPRB, 0.16877E+01_JPRB, 0.17340E+01_JPRB, 0.17860E+01_JPRB, 0.18221E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.18437E+01_JPRB, 0.18248E+01_JPRB, 0.17569E+01_JPRB, 0.13252E+01_JPRB, 0.11678E+01_JPRB, 0.10085E+01_JPRB, 0.93260E+00_JPRB ,&
&  0.85151E+00_JPRB, 0.72996E+00_JPRB, 0.71040E+00_JPRB, 0.73408E+00_JPRB, 0.69799E+00_JPRB, 0.62298E+00_JPRB, 0.62404E+00_JPRB ,&
&  0.63581E+00_JPRB, 0.62215E+00_JPRB, 0.59473E+00_JPRB, 0.58113E+00_JPRB, 0.61067E+00_JPRB, 0.61403E+00_JPRB, 0.62141E+00_JPRB ,&
&  0.61808E+00_JPRB, 0.64737E+00_JPRB, 0.67651E+00_JPRB, 0.70071E+00_JPRB, 0.72621E+00_JPRB, 0.75598E+00_JPRB, 0.76116E+00_JPRB ,&
&  0.77288E+00_JPRB, 0.77143E+00_JPRB, 0.77579E+00_JPRB, 0.76154E+00_JPRB, 0.74675E+00_JPRB, 0.71528E+00_JPRB, 0.70764E+00_JPRB ,&
&  0.70973E+00_JPRB, 0.71426E+00_JPRB, 0.71171E+00_JPRB, 0.70577E+00_JPRB, 0.73201E+00_JPRB, 0.73305E+00_JPRB, 0.74255E+00_JPRB ,&
&  0.73982E+00_JPRB, 0.71823E+00_JPRB, 0.66023E+00_JPRB, 0.61621E+00_JPRB, 0.59804E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.47176E+00_JPRB, 0.41783E+00_JPRB, 0.41265E+00_JPRB, 0.45380E+00_JPRB, 0.49550E+00_JPRB, 0.53814E+00_JPRB, 0.57628E+00_JPRB ,&
&  0.61426E+00_JPRB, 0.63516E+00_JPRB, 0.67772E+00_JPRB, 0.69636E+00_JPRB, 0.70886E+00_JPRB, 0.70778E+00_JPRB, 0.70713E+00_JPRB ,&
&  0.72580E+00_JPRB, 0.74676E+00_JPRB, 0.76593E+00_JPRB, 0.80046E+00_JPRB, 0.83634E+00_JPRB, 0.87312E+00_JPRB, 0.91904E+00_JPRB ,&
&  0.96540E+00_JPRB, 0.10189E+01_JPRB, 0.10726E+01_JPRB, 0.11354E+01_JPRB, 0.11816E+01_JPRB, 0.11989E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.63022E+00_JPRB, 0.92779E+00_JPRB, 0.11360E+01_JPRB, 0.11173E+01_JPRB, 0.10797E+01_JPRB, 0.10420E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.99529E+00_JPRB, 0.10731E+01_JPRB, 0.14481E+01_JPRB, 0.15817E+01_JPRB, 0.16498E+01_JPRB, 0.17178E+01_JPRB ,&
&  0.17870E+01_JPRB, 0.18398E+01_JPRB, 0.19020E+01_JPRB, 0.19683E+01_JPRB, 0.20477E+01_JPRB, 0.21203E+01_JPRB, 0.21903E+01_JPRB ,&
&  0.22452E+01_JPRB, 0.22787E+01_JPRB, 0.22810E+01_JPRB, 0.18348E+01_JPRB, 0.17689E+01_JPRB, 0.14980E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.87175E+00_JPRB, 0.69274E+00_JPRB, 0.70312E+00_JPRB, 0.75030E+00_JPRB, 0.69714E+00_JPRB, 0.66092E+00_JPRB, 0.60579E+00_JPRB ,&
&  0.60953E+00_JPRB, 0.61009E+00_JPRB, 0.61049E+00_JPRB, 0.59796E+00_JPRB, 0.59950E+00_JPRB, 0.57343E+00_JPRB, 0.60760E+00_JPRB ,&
&  0.62386E+00_JPRB, 0.61600E+00_JPRB, 0.62150E+00_JPRB, 0.64656E+00_JPRB, 0.67961E+00_JPRB, 0.71086E+00_JPRB, 0.72028E+00_JPRB ,&
&  0.71604E+00_JPRB, 0.70213E+00_JPRB, 0.70230E+00_JPRB, 0.70150E+00_JPRB, 0.69876E+00_JPRB, 0.69583E+00_JPRB, 0.68274E+00_JPRB ,&
&  0.67078E+00_JPRB, 0.68622E+00_JPRB, 0.72533E+00_JPRB, 0.67883E+00_JPRB, 0.72544E+00_JPRB, 0.65971E+00_JPRB, 0.64330E+00_JPRB ,&
&  0.62575E+00_JPRB, 0.60547E+00_JPRB, 0.55930E+00_JPRB, 0.53638E+00_JPRB, 0.50727E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.38245E+00_JPRB, 0.36507E+00_JPRB, 0.37444E+00_JPRB, 0.39800E+00_JPRB, 0.42603E+00_JPRB, 0.46834E+00_JPRB, 0.51864E+00_JPRB ,&
&  0.56156E+00_JPRB, 0.60294E+00_JPRB, 0.67468E+00_JPRB, 0.68934E+00_JPRB, 0.70072E+00_JPRB, 0.73623E+00_JPRB, 0.75707E+00_JPRB ,&
&  0.78012E+00_JPRB, 0.79407E+00_JPRB, 0.81291E+00_JPRB, 0.83217E+00_JPRB, 0.86656E+00_JPRB, 0.90548E+00_JPRB, 0.93627E+00_JPRB ,&
&  0.98100E+00_JPRB, 0.10300E+01_JPRB, 0.11040E+01_JPRB, 0.11799E+01_JPRB, 0.12506E+01_JPRB, 0.12601E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.85177E+00_JPRB, 0.11003E+01_JPRB, 0.11110E+01_JPRB, 0.11016E+01_JPRB, 0.10687E+01_JPRB, 0.10354E+01_JPRB, 0.10428E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10267E+01_JPRB, 0.10728E+01_JPRB, 0.12481E+01_JPRB, 0.15321E+01_JPRB, 0.16058E+01_JPRB, 0.17147E+01_JPRB ,&
&  0.18180E+01_JPRB, 0.19101E+01_JPRB, 0.20093E+01_JPRB, 0.21298E+01_JPRB, 0.22534E+01_JPRB, 0.23703E+01_JPRB, 0.24949E+01_JPRB ,&
&  0.26056E+01_JPRB, 0.27111E+01_JPRB, 0.28440E+01_JPRB, 0.23737E+01_JPRB, 0.19460E+01_JPRB, 0.15572E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.77784E+00_JPRB, 0.66433E+00_JPRB, 0.70323E+00_JPRB, 0.75479E+00_JPRB, 0.70179E+00_JPRB, 0.66267E+00_JPRB, 0.60879E+00_JPRB ,&
&  0.60032E+00_JPRB, 0.60348E+00_JPRB, 0.60951E+00_JPRB, 0.60296E+00_JPRB, 0.61033E+00_JPRB, 0.59240E+00_JPRB, 0.59302E+00_JPRB ,&
&  0.60836E+00_JPRB, 0.60091E+00_JPRB, 0.59846E+00_JPRB, 0.61059E+00_JPRB, 0.63884E+00_JPRB, 0.66919E+00_JPRB, 0.66934E+00_JPRB ,&
&  0.67771E+00_JPRB, 0.61821E+00_JPRB, 0.59627E+00_JPRB, 0.61267E+00_JPRB, 0.61750E+00_JPRB, 0.62112E+00_JPRB, 0.64321E+00_JPRB ,&
&  0.72735E+00_JPRB, 0.83213E+00_JPRB, 0.92556E+00_JPRB, 0.92221E+00_JPRB, 0.97868E+00_JPRB, 0.76664E+00_JPRB, 0.56745E+00_JPRB ,&
&  0.53122E+00_JPRB, 0.51877E+00_JPRB, 0.47543E+00_JPRB, 0.44565E+00_JPRB, 0.41201E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.37693E+00_JPRB, 0.37085E+00_JPRB, 0.38016E+00_JPRB, 0.39661E+00_JPRB, 0.40873E+00_JPRB, 0.43672E+00_JPRB, 0.47033E+00_JPRB ,&
&  0.50709E+00_JPRB, 0.55219E+00_JPRB, 0.58838E+00_JPRB, 0.61177E+00_JPRB, 0.65432E+00_JPRB, 0.71639E+00_JPRB, 0.74042E+00_JPRB ,&
&  0.79038E+00_JPRB, 0.81951E+00_JPRB, 0.82951E+00_JPRB, 0.83402E+00_JPRB, 0.85524E+00_JPRB, 0.89370E+00_JPRB, 0.93647E+00_JPRB ,&
&  0.97307E+00_JPRB, 0.10232E+01_JPRB, 0.10969E+01_JPRB, 0.11439E+01_JPRB, 0.11636E+01_JPRB, 0.11653E+01_JPRB, 0.90438E+00_JPRB ,&
&  0.10545E+01_JPRB, 0.10881E+01_JPRB, 0.10950E+01_JPRB, 0.10635E+01_JPRB, 0.10358E+01_JPRB, 0.10095E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10240E+01_JPRB, 0.10163E+01_JPRB, 0.10732E+01_JPRB, 0.13222E+01_JPRB, 0.14099E+01_JPRB, 0.15599E+01_JPRB ,&
&  0.16637E+01_JPRB, 0.17476E+01_JPRB, 0.18633E+01_JPRB, 0.19900E+01_JPRB, 0.21159E+01_JPRB, 0.22229E+01_JPRB, 0.23170E+01_JPRB ,&
&  0.24047E+01_JPRB, 0.25034E+01_JPRB, 0.25876E+01_JPRB, 0.22015E+01_JPRB, 0.17575E+01_JPRB, 0.14454E+01_JPRB, 0.99918E+00_JPRB ,&
&  0.72856E+00_JPRB, 0.66262E+00_JPRB, 0.70153E+00_JPRB, 0.73747E+00_JPRB, 0.68164E+00_JPRB, 0.64160E+00_JPRB, 0.60506E+00_JPRB ,&
&  0.60400E+00_JPRB, 0.61609E+00_JPRB, 0.63288E+00_JPRB, 0.63620E+00_JPRB, 0.61598E+00_JPRB, 0.59392E+00_JPRB, 0.57775E+00_JPRB ,&
&  0.59481E+00_JPRB, 0.59205E+00_JPRB, 0.58420E+00_JPRB, 0.60692E+00_JPRB, 0.61656E+00_JPRB, 0.68385E+00_JPRB, 0.78551E+00_JPRB ,&
&  0.78286E+00_JPRB, 0.51348E+00_JPRB, 0.52775E+00_JPRB, 0.54703E+00_JPRB, 0.58819E+00_JPRB, 0.66150E+00_JPRB, 0.81419E+00_JPRB ,&
&  0.11612E+01_JPRB, 0.19270E+01_JPRB, 0.22078E+01_JPRB, 0.14614E+01_JPRB, 0.19923E+01_JPRB, 0.12893E+01_JPRB, 0.51232E+00_JPRB ,&
&  0.48449E+00_JPRB, 0.46924E+00_JPRB, 0.44154E+00_JPRB, 0.41192E+00_JPRB, 0.39175E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.45618E+00_JPRB, 0.45779E+00_JPRB, 0.46529E+00_JPRB, 0.47028E+00_JPRB, 0.47547E+00_JPRB, 0.47887E+00_JPRB, 0.48251E+00_JPRB ,&
&  0.49577E+00_JPRB, 0.50347E+00_JPRB, 0.52304E+00_JPRB, 0.53710E+00_JPRB, 0.55010E+00_JPRB, 0.57984E+00_JPRB, 0.60414E+00_JPRB ,&
&  0.64365E+00_JPRB, 0.69404E+00_JPRB, 0.71294E+00_JPRB, 0.74095E+00_JPRB, 0.75679E+00_JPRB, 0.77240E+00_JPRB, 0.78231E+00_JPRB ,&
&  0.84045E+00_JPRB, 0.95206E+00_JPRB, 0.10244E+01_JPRB, 0.11008E+01_JPRB, 0.10973E+01_JPRB, 0.11501E+01_JPRB, 0.86950E+00_JPRB ,&
&  0.10849E+01_JPRB, 0.10813E+01_JPRB, 0.10103E+01_JPRB, 0.96978E+00_JPRB, 0.94369E+00_JPRB, 0.91799E+00_JPRB, 0.89460E+00_JPRB ,&
&  0.90857E+00_JPRB, 0.95726E+00_JPRB, 0.95818E+00_JPRB, 0.10253E+01_JPRB, 0.11181E+01_JPRB, 0.11343E+01_JPRB, 0.12449E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.13874E+01_JPRB, 0.14585E+01_JPRB, 0.15620E+01_JPRB, 0.16353E+01_JPRB, 0.16789E+01_JPRB, 0.16837E+01_JPRB ,&
&  0.16740E+01_JPRB, 0.16940E+01_JPRB, 0.17544E+01_JPRB, 0.17124E+01_JPRB, 0.14153E+01_JPRB, 0.11919E+01_JPRB, 0.95872E+00_JPRB ,&
&  0.72439E+00_JPRB, 0.63941E+00_JPRB, 0.65173E+00_JPRB, 0.72433E+00_JPRB, 0.69262E+00_JPRB, 0.65966E+00_JPRB, 0.63301E+00_JPRB ,&
&  0.61227E+00_JPRB, 0.61678E+00_JPRB, 0.62165E+00_JPRB, 0.61644E+00_JPRB, 0.59911E+00_JPRB, 0.56744E+00_JPRB, 0.53792E+00_JPRB ,&
&  0.53415E+00_JPRB, 0.52553E+00_JPRB, 0.52050E+00_JPRB, 0.52673E+00_JPRB, 0.53051E+00_JPRB, 0.55259E+00_JPRB, 0.51919E+00_JPRB ,&
&  0.56820E+00_JPRB, 0.46712E+00_JPRB, 0.49062E+00_JPRB, 0.53214E+00_JPRB, 0.59308E+00_JPRB, 0.82279E+00_JPRB, 0.12224E+01_JPRB ,&
&  0.18175E+01_JPRB, 0.20854E+01_JPRB, 0.27725E+01_JPRB, 0.40747E+01_JPRB, 0.25654E+01_JPRB, 0.16329E+01_JPRB, 0.52693E+00_JPRB ,&
&  0.51570E+00_JPRB, 0.49020E+00_JPRB, 0.48243E+00_JPRB, 0.46727E+00_JPRB, 0.46229E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.52160E+00_JPRB, 0.53024E+00_JPRB, 0.54077E+00_JPRB, 0.55008E+00_JPRB, 0.55604E+00_JPRB, 0.56683E+00_JPRB, 0.57449E+00_JPRB ,&
&  0.58335E+00_JPRB, 0.58642E+00_JPRB, 0.59300E+00_JPRB, 0.59858E+00_JPRB, 0.60742E+00_JPRB, 0.62067E+00_JPRB, 0.63367E+00_JPRB ,&
&  0.64350E+00_JPRB, 0.65897E+00_JPRB, 0.67775E+00_JPRB, 0.69013E+00_JPRB, 0.72113E+00_JPRB, 0.77917E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.89176E+00_JPRB, 0.95332E+00_JPRB, 0.10085E+01_JPRB, 0.10253E+01_JPRB, 0.10109E+01_JPRB, 0.10071E+01_JPRB, 0.80802E+00_JPRB ,&
&  0.11713E+01_JPRB, 0.11471E+01_JPRB, 0.10300E+01_JPRB, 0.95773E+00_JPRB, 0.89380E+00_JPRB, 0.83650E+00_JPRB, 0.78645E+00_JPRB ,&
&  0.76702E+00_JPRB, 0.85995E+00_JPRB, 0.87006E+00_JPRB, 0.89644E+00_JPRB, 0.86425E+00_JPRB, 0.88254E+00_JPRB, 0.96619E+00_JPRB ,&
&  0.10635E+01_JPRB, 0.10781E+01_JPRB, 0.10483E+01_JPRB, 0.10819E+01_JPRB, 0.11273E+01_JPRB, 0.11392E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.10811E+01_JPRB, 0.10191E+01_JPRB, 0.90314E+00_JPRB, 0.10106E+01_JPRB, 0.94253E+00_JPRB, 0.85543E+00_JPRB, 0.77288E+00_JPRB ,&
&  0.60873E+00_JPRB, 0.60737E+00_JPRB, 0.65788E+00_JPRB, 0.76087E+00_JPRB, 0.72963E+00_JPRB, 0.70461E+00_JPRB, 0.66634E+00_JPRB ,&
&  0.63210E+00_JPRB, 0.61472E+00_JPRB, 0.58653E+00_JPRB, 0.57256E+00_JPRB, 0.56621E+00_JPRB, 0.54564E+00_JPRB, 0.51132E+00_JPRB ,&
&  0.48559E+00_JPRB, 0.48062E+00_JPRB, 0.47201E+00_JPRB, 0.46444E+00_JPRB, 0.45811E+00_JPRB, 0.40497E+00_JPRB, 0.48778E+00_JPRB ,&
&  0.46890E+00_JPRB, 0.41439E+00_JPRB, 0.51664E+00_JPRB, 0.39610E+00_JPRB, 0.51106E+00_JPRB, 0.63052E+00_JPRB, 0.82217E+00_JPRB ,&
&  0.87278E+00_JPRB, 0.11255E+01_JPRB, 0.14764E+01_JPRB, 0.11689E+01_JPRB, 0.88407E+00_JPRB, 0.58875E+00_JPRB, 0.55300E+00_JPRB ,&
&  0.54182E+00_JPRB, 0.53129E+00_JPRB, 0.53347E+00_JPRB, 0.52586E+00_JPRB, 0.52364E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.59251E+00_JPRB, 0.60462E+00_JPRB, 0.61981E+00_JPRB, 0.63958E+00_JPRB, 0.65214E+00_JPRB, 0.67019E+00_JPRB, 0.68417E+00_JPRB ,&
&  0.70352E+00_JPRB, 0.71801E+00_JPRB, 0.72998E+00_JPRB, 0.74250E+00_JPRB, 0.75977E+00_JPRB, 0.78203E+00_JPRB, 0.81068E+00_JPRB ,&
&  0.84155E+00_JPRB, 0.87232E+00_JPRB, 0.90686E+00_JPRB, 0.94641E+00_JPRB, 0.99271E+00_JPRB, 0.10493E+01_JPRB, 0.11019E+01_JPRB ,&
&  0.11499E+01_JPRB, 0.11976E+01_JPRB, 0.12436E+01_JPRB, 0.12783E+01_JPRB, 0.12817E+01_JPRB, 0.12758E+01_JPRB, 0.10809E+01_JPRB ,&
&  0.13502E+01_JPRB, 0.13191E+01_JPRB, 0.12186E+01_JPRB, 0.11031E+01_JPRB, 0.10043E+01_JPRB, 0.91769E+00_JPRB, 0.83743E+00_JPRB ,&
&  0.82990E+00_JPRB, 0.78357E+00_JPRB, 0.76149E+00_JPRB, 0.75660E+00_JPRB, 0.71738E+00_JPRB, 0.72314E+00_JPRB, 0.75819E+00_JPRB ,&
&  0.81116E+00_JPRB, 0.89789E+00_JPRB, 0.94085E+00_JPRB, 0.98690E+00_JPRB, 0.10511E+01_JPRB, 0.10135E+01_JPRB, 0.99287E+00_JPRB ,&
&  0.91122E+00_JPRB, 0.80733E+00_JPRB, 0.64210E+00_JPRB, 0.70620E+00_JPRB, 0.74699E+00_JPRB, 0.76942E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.66104E+00_JPRB, 0.58625E+00_JPRB, 0.63438E+00_JPRB, 0.83555E+00_JPRB, 0.81736E+00_JPRB, 0.75054E+00_JPRB, 0.69583E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.59876E+00_JPRB, 0.55466E+00_JPRB, 0.53529E+00_JPRB, 0.55053E+00_JPRB, 0.54333E+00_JPRB, 0.51316E+00_JPRB ,&
&  0.47308E+00_JPRB, 0.45322E+00_JPRB, 0.44595E+00_JPRB, 0.43725E+00_JPRB, 0.43758E+00_JPRB, 0.37872E+00_JPRB, 0.43693E+00_JPRB ,&
&  0.44419E+00_JPRB, 0.33707E+00_JPRB, 0.42828E+00_JPRB, 0.55259E+00_JPRB, 0.53770E+00_JPRB, 0.59122E+00_JPRB, 0.62475E+00_JPRB ,&
&  0.65719E+00_JPRB, 0.66723E+00_JPRB, 0.62324E+00_JPRB, 0.61993E+00_JPRB, 0.60960E+00_JPRB, 0.59908E+00_JPRB, 0.58742E+00_JPRB ,&
&  0.57585E+00_JPRB, 0.57603E+00_JPRB, 0.58835E+00_JPRB, 0.58699E+00_JPRB, 0.58698E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.60058E+00_JPRB, 0.61172E+00_JPRB, 0.61205E+00_JPRB, 0.61829E+00_JPRB, 0.63171E+00_JPRB, 0.64085E+00_JPRB, 0.65439E+00_JPRB ,&
&  0.66817E+00_JPRB, 0.67338E+00_JPRB, 0.67823E+00_JPRB, 0.68504E+00_JPRB, 0.68911E+00_JPRB, 0.70714E+00_JPRB, 0.72519E+00_JPRB ,&
&  0.74929E+00_JPRB, 0.76841E+00_JPRB, 0.79040E+00_JPRB, 0.82355E+00_JPRB, 0.86533E+00_JPRB, 0.92937E+00_JPRB, 0.99707E+00_JPRB ,&
&  0.10856E+01_JPRB, 0.11887E+01_JPRB, 0.13115E+01_JPRB, 0.14093E+01_JPRB, 0.14544E+01_JPRB, 0.14642E+01_JPRB, 0.15869E+01_JPRB ,&
&  0.13536E+01_JPRB, 0.14182E+01_JPRB, 0.13253E+01_JPRB, 0.11669E+01_JPRB, 0.11392E+01_JPRB, 0.10533E+01_JPRB, 0.10123E+01_JPRB ,&
&  0.10019E+01_JPRB, 0.95012E+00_JPRB, 0.92369E+00_JPRB, 0.89936E+00_JPRB, 0.87523E+00_JPRB, 0.83002E+00_JPRB, 0.79235E+00_JPRB ,&
&  0.78681E+00_JPRB, 0.82665E+00_JPRB, 0.91659E+00_JPRB, 0.11047E+01_JPRB, 0.12860E+01_JPRB, 0.13395E+01_JPRB, 0.12809E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11646E+01_JPRB, 0.10638E+01_JPRB, 0.10204E+01_JPRB, 0.10643E+01_JPRB, 0.10986E+01_JPRB, 0.10900E+01_JPRB ,&
&  0.94817E+00_JPRB, 0.82175E+00_JPRB, 0.79619E+00_JPRB, 0.86011E+00_JPRB, 0.87068E+00_JPRB, 0.84029E+00_JPRB, 0.74714E+00_JPRB ,&
&  0.64476E+00_JPRB, 0.58380E+00_JPRB, 0.55016E+00_JPRB, 0.55769E+00_JPRB, 0.61354E+00_JPRB, 0.62051E+00_JPRB, 0.59104E+00_JPRB ,&
&  0.52760E+00_JPRB, 0.48724E+00_JPRB, 0.45993E+00_JPRB, 0.44038E+00_JPRB, 0.38201E+00_JPRB, 0.39797E+00_JPRB, 0.43573E+00_JPRB ,&
&  0.45357E+00_JPRB, 0.42110E+00_JPRB, 0.45149E+00_JPRB, 0.61334E+00_JPRB, 0.62036E+00_JPRB, 0.63222E+00_JPRB, 0.63186E+00_JPRB ,&
&  0.62339E+00_JPRB, 0.62205E+00_JPRB, 0.61918E+00_JPRB, 0.60274E+00_JPRB, 0.59060E+00_JPRB, 0.58773E+00_JPRB, 0.61227E+00_JPRB ,&
&  0.60765E+00_JPRB, 0.60782E+00_JPRB, 0.60161E+00_JPRB, 0.61100E+00_JPRB, 0.60591E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.62035E+00_JPRB, 0.62231E+00_JPRB, 0.63065E+00_JPRB, 0.64767E+00_JPRB, 0.64523E+00_JPRB, 0.64278E+00_JPRB, 0.66850E+00_JPRB ,&
&  0.65216E+00_JPRB, 0.63941E+00_JPRB, 0.63674E+00_JPRB, 0.61066E+00_JPRB, 0.62685E+00_JPRB, 0.64203E+00_JPRB, 0.67702E+00_JPRB ,&
&  0.71386E+00_JPRB, 0.73715E+00_JPRB, 0.75877E+00_JPRB, 0.79109E+00_JPRB, 0.85250E+00_JPRB, 0.90136E+00_JPRB, 0.94374E+00_JPRB ,&
&  0.10172E+01_JPRB, 0.11307E+01_JPRB, 0.12555E+01_JPRB, 0.15381E+01_JPRB, 0.21853E+01_JPRB, 0.26806E+01_JPRB, 0.26256E+01_JPRB ,&
&  0.19023E+01_JPRB, 0.13596E+01_JPRB, 0.12624E+01_JPRB, 0.11281E+01_JPRB, 0.11609E+01_JPRB, 0.12578E+01_JPRB, 0.12659E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.12391E+01_JPRB, 0.12316E+01_JPRB, 0.12285E+01_JPRB, 0.12374E+01_JPRB, 0.12414E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.11910E+01_JPRB, 0.11417E+01_JPRB, 0.11751E+01_JPRB, 0.13920E+01_JPRB, 0.15491E+01_JPRB, 0.18293E+01_JPRB, 0.18209E+01_JPRB ,&
&  0.17583E+01_JPRB, 0.16079E+01_JPRB, 0.15415E+01_JPRB, 0.15231E+01_JPRB, 0.14922E+01_JPRB, 0.14631E+01_JPRB, 0.14810E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.12703E+01_JPRB, 0.86308E+00_JPRB, 0.92481E+00_JPRB, 0.93522E+00_JPRB, 0.94348E+00_JPRB, 0.78547E+00_JPRB ,&
&  0.65116E+00_JPRB, 0.61682E+00_JPRB, 0.61877E+00_JPRB, 0.65563E+00_JPRB, 0.73349E+00_JPRB, 0.78182E+00_JPRB, 0.72974E+00_JPRB ,&
&  0.63881E+00_JPRB, 0.55783E+00_JPRB, 0.50751E+00_JPRB, 0.48526E+00_JPRB, 0.46941E+00_JPRB, 0.40240E+00_JPRB, 0.50293E+00_JPRB ,&
&  0.54758E+00_JPRB, 0.62524E+00_JPRB, 0.59629E+00_JPRB, 0.78933E+00_JPRB, 0.75725E+00_JPRB, 0.73997E+00_JPRB, 0.72939E+00_JPRB ,&
&  0.72458E+00_JPRB, 0.70855E+00_JPRB, 0.69283E+00_JPRB, 0.67234E+00_JPRB, 0.65108E+00_JPRB, 0.67847E+00_JPRB, 0.71259E+00_JPRB ,&
&  0.68031E+00_JPRB, 0.69772E+00_JPRB, 0.67803E+00_JPRB, 0.64708E+00_JPRB, 0.64182E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.67322E+00_JPRB, 0.65477E+00_JPRB, 0.66217E+00_JPRB, 0.68154E+00_JPRB, 0.69298E+00_JPRB, 0.70429E+00_JPRB, 0.71556E+00_JPRB ,&
&  0.71901E+00_JPRB, 0.72075E+00_JPRB, 0.72532E+00_JPRB, 0.72636E+00_JPRB, 0.75802E+00_JPRB, 0.79316E+00_JPRB, 0.83877E+00_JPRB ,&
&  0.90822E+00_JPRB, 0.97150E+00_JPRB, 0.10303E+01_JPRB, 0.11394E+01_JPRB, 0.12194E+01_JPRB, 0.13000E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.16013E+01_JPRB, 0.17875E+01_JPRB, 0.19761E+01_JPRB, 0.21725E+01_JPRB, 0.24056E+01_JPRB, 0.26164E+01_JPRB, 0.26195E+01_JPRB ,&
&  0.19452E+01_JPRB, 0.14225E+01_JPRB, 0.14628E+01_JPRB, 0.13401E+01_JPRB, 0.13655E+01_JPRB, 0.13848E+01_JPRB, 0.13805E+01_JPRB ,&
&  0.13954E+01_JPRB, 0.14002E+01_JPRB, 0.14037E+01_JPRB, 0.14115E+01_JPRB, 0.14548E+01_JPRB, 0.15171E+01_JPRB, 0.16024E+01_JPRB ,&
&  0.16386E+01_JPRB, 0.16700E+01_JPRB, 0.16888E+01_JPRB, 0.16756E+01_JPRB, 0.17404E+01_JPRB, 0.19038E+01_JPRB, 0.20384E+01_JPRB ,&
&  0.21052E+01_JPRB, 0.21066E+01_JPRB, 0.18821E+01_JPRB, 0.18598E+01_JPRB, 0.18608E+01_JPRB, 0.17337E+01_JPRB, 0.17819E+01_JPRB ,&
&  0.18433E+01_JPRB, 0.16008E+01_JPRB, 0.10222E+01_JPRB, 0.98505E+00_JPRB, 0.97993E+00_JPRB, 0.91565E+00_JPRB, 0.84278E+00_JPRB ,&
&  0.69860E+00_JPRB, 0.68144E+00_JPRB, 0.71956E+00_JPRB, 0.82997E+00_JPRB, 0.89404E+00_JPRB, 0.93142E+00_JPRB, 0.84776E+00_JPRB ,&
&  0.70200E+00_JPRB, 0.61452E+00_JPRB, 0.55536E+00_JPRB, 0.55273E+00_JPRB, 0.52038E+00_JPRB, 0.54871E+00_JPRB, 0.61501E+00_JPRB ,&
&  0.66148E+00_JPRB, 0.74797E+00_JPRB, 0.87524E+00_JPRB, 0.84937E+00_JPRB, 0.77624E+00_JPRB, 0.86685E+00_JPRB, 0.84969E+00_JPRB ,&
&  0.83600E+00_JPRB, 0.83455E+00_JPRB, 0.82535E+00_JPRB, 0.81722E+00_JPRB, 0.80898E+00_JPRB, 0.81015E+00_JPRB, 0.79668E+00_JPRB ,&
&  0.78817E+00_JPRB, 0.77170E+00_JPRB, 0.75985E+00_JPRB, 0.72444E+00_JPRB, 0.69802E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.73434E+00_JPRB, 0.71368E+00_JPRB, 0.70199E+00_JPRB, 0.71089E+00_JPRB, 0.73127E+00_JPRB, 0.74629E+00_JPRB, 0.75257E+00_JPRB ,&
&  0.76909E+00_JPRB, 0.79202E+00_JPRB, 0.82657E+00_JPRB, 0.85737E+00_JPRB, 0.89627E+00_JPRB, 0.93248E+00_JPRB, 0.99126E+00_JPRB ,&
&  0.10489E+01_JPRB, 0.11179E+01_JPRB, 0.12376E+01_JPRB, 0.14171E+01_JPRB, 0.15689E+01_JPRB, 0.17047E+01_JPRB, 0.19140E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.22573E+01_JPRB, 0.24247E+01_JPRB, 0.25119E+01_JPRB, 0.24227E+01_JPRB, 0.23089E+01_JPRB, 0.23765E+01_JPRB ,&
&  0.19586E+01_JPRB, 0.15228E+01_JPRB, 0.14349E+01_JPRB, 0.14086E+01_JPRB, 0.14185E+01_JPRB, 0.14132E+01_JPRB, 0.14154E+01_JPRB ,&
&  0.14306E+01_JPRB, 0.14371E+01_JPRB, 0.14441E+01_JPRB, 0.14584E+01_JPRB, 0.14932E+01_JPRB, 0.15622E+01_JPRB, 0.16630E+01_JPRB ,&
&  0.16782E+01_JPRB, 0.17133E+01_JPRB, 0.17920E+01_JPRB, 0.16529E+01_JPRB, 0.17487E+01_JPRB, 0.18982E+01_JPRB, 0.19851E+01_JPRB ,&
&  0.19762E+01_JPRB, 0.18304E+01_JPRB, 0.18360E+01_JPRB, 0.18508E+01_JPRB, 0.18583E+01_JPRB, 0.17975E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.18479E+01_JPRB, 0.17433E+01_JPRB, 0.12070E+01_JPRB, 0.11058E+01_JPRB, 0.11316E+01_JPRB, 0.92089E+00_JPRB, 0.91122E+00_JPRB ,&
&  0.79287E+00_JPRB, 0.78358E+00_JPRB, 0.87721E+00_JPRB, 0.94907E+00_JPRB, 0.99197E+00_JPRB, 0.10928E+01_JPRB, 0.88196E+00_JPRB ,&
&  0.75560E+00_JPRB, 0.68096E+00_JPRB, 0.65029E+00_JPRB, 0.65509E+00_JPRB, 0.70021E+00_JPRB, 0.79276E+00_JPRB, 0.87561E+00_JPRB ,&
&  0.79020E+00_JPRB, 0.81653E+00_JPRB, 0.86668E+00_JPRB, 0.91671E+00_JPRB, 0.88258E+00_JPRB, 0.92070E+00_JPRB, 0.90927E+00_JPRB ,&
&  0.88818E+00_JPRB, 0.89000E+00_JPRB, 0.88317E+00_JPRB, 0.86442E+00_JPRB, 0.85579E+00_JPRB, 0.84418E+00_JPRB, 0.83981E+00_JPRB ,&
&  0.83794E+00_JPRB, 0.83284E+00_JPRB, 0.81947E+00_JPRB, 0.80185E+00_JPRB, 0.76855E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.81489E+00_JPRB, 0.78357E+00_JPRB, 0.75548E+00_JPRB, 0.74551E+00_JPRB, 0.74949E+00_JPRB, 0.75238E+00_JPRB, 0.76179E+00_JPRB ,&
&  0.78356E+00_JPRB, 0.80986E+00_JPRB, 0.83851E+00_JPRB, 0.87771E+00_JPRB, 0.91691E+00_JPRB, 0.95808E+00_JPRB, 0.10208E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.12024E+01_JPRB, 0.13501E+01_JPRB, 0.15117E+01_JPRB, 0.16559E+01_JPRB, 0.18183E+01_JPRB, 0.19810E+01_JPRB ,&
&  0.21676E+01_JPRB, 0.22387E+01_JPRB, 0.21815E+01_JPRB, 0.20501E+01_JPRB, 0.16345E+01_JPRB, 0.16993E+01_JPRB, 0.18749E+01_JPRB ,&
&  0.19080E+01_JPRB, 0.15527E+01_JPRB, 0.13778E+01_JPRB, 0.13475E+01_JPRB, 0.13489E+01_JPRB, 0.13630E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.14053E+01_JPRB, 0.14225E+01_JPRB, 0.14387E+01_JPRB, 0.14583E+01_JPRB, 0.15039E+01_JPRB, 0.16012E+01_JPRB ,&
&  0.16198E+01_JPRB, 0.16427E+01_JPRB, 0.16561E+01_JPRB, 0.16172E+01_JPRB, 0.16813E+01_JPRB, 0.17650E+01_JPRB, 0.17321E+01_JPRB ,&
&  0.16923E+01_JPRB, 0.15548E+01_JPRB, 0.15487E+01_JPRB, 0.16079E+01_JPRB, 0.15843E+01_JPRB, 0.14700E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.16242E+01_JPRB, 0.17895E+01_JPRB, 0.12187E+01_JPRB, 0.13225E+01_JPRB, 0.15230E+01_JPRB, 0.12635E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.97486E+00_JPRB, 0.97051E+00_JPRB, 0.10658E+01_JPRB, 0.10887E+01_JPRB, 0.10602E+01_JPRB, 0.11586E+01_JPRB, 0.90857E+00_JPRB ,&
&  0.86111E+00_JPRB, 0.78961E+00_JPRB, 0.79259E+00_JPRB, 0.83436E+00_JPRB, 0.91780E+00_JPRB, 0.97654E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.84274E+00_JPRB, 0.10431E+01_JPRB, 0.12425E+01_JPRB, 0.15175E+01_JPRB, 0.15351E+01_JPRB, 0.94793E+00_JPRB, 0.95796E+00_JPRB ,&
&  0.96107E+00_JPRB, 0.95536E+00_JPRB, 0.94012E+00_JPRB, 0.92440E+00_JPRB, 0.91194E+00_JPRB, 0.89897E+00_JPRB, 0.89121E+00_JPRB ,&
&  0.88671E+00_JPRB, 0.88475E+00_JPRB, 0.88131E+00_JPRB, 0.86406E+00_JPRB, 0.84025E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.87611E+00_JPRB, 0.85738E+00_JPRB, 0.82508E+00_JPRB, 0.79769E+00_JPRB, 0.79587E+00_JPRB, 0.79661E+00_JPRB, 0.77795E+00_JPRB ,&
&  0.78070E+00_JPRB, 0.79369E+00_JPRB, 0.81047E+00_JPRB, 0.83413E+00_JPRB, 0.85882E+00_JPRB, 0.87825E+00_JPRB, 0.92992E+00_JPRB ,&
&  0.99210E+00_JPRB, 0.11098E+01_JPRB, 0.12949E+01_JPRB, 0.14858E+01_JPRB, 0.16606E+01_JPRB, 0.18600E+01_JPRB, 0.20539E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.23458E+01_JPRB, 0.21403E+01_JPRB, 0.14463E+01_JPRB, 0.12374E+01_JPRB, 0.13425E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.13507E+01_JPRB, 0.12997E+01_JPRB, 0.12663E+01_JPRB, 0.12533E+01_JPRB, 0.12606E+01_JPRB, 0.12784E+01_JPRB, 0.12764E+01_JPRB ,&
&  0.12849E+01_JPRB, 0.13056E+01_JPRB, 0.13311E+01_JPRB, 0.13647E+01_JPRB, 0.13993E+01_JPRB, 0.14402E+01_JPRB, 0.14855E+01_JPRB ,&
&  0.15041E+01_JPRB, 0.15424E+01_JPRB, 0.15350E+01_JPRB, 0.15056E+01_JPRB, 0.14793E+01_JPRB, 0.14793E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.14368E+01_JPRB, 0.13667E+01_JPRB, 0.13431E+01_JPRB, 0.13909E+01_JPRB, 0.14208E+01_JPRB, 0.12958E+01_JPRB, 0.13200E+01_JPRB ,&
&  0.14736E+01_JPRB, 0.16067E+01_JPRB, 0.13873E+01_JPRB, 0.16526E+01_JPRB, 0.10779E+01_JPRB, 0.12167E+01_JPRB, 0.13208E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.12371E+01_JPRB, 0.12605E+01_JPRB, 0.12933E+01_JPRB, 0.12895E+01_JPRB, 0.11405E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.98666E+00_JPRB, 0.89365E+00_JPRB, 0.85450E+00_JPRB, 0.93287E+00_JPRB, 0.10588E+01_JPRB, 0.10776E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.10440E+01_JPRB, 0.10278E+01_JPRB, 0.11012E+01_JPRB, 0.12696E+01_JPRB, 0.12187E+01_JPRB, 0.96076E+00_JPRB, 0.98160E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.98864E+00_JPRB, 0.97950E+00_JPRB, 0.95441E+00_JPRB, 0.94572E+00_JPRB, 0.93517E+00_JPRB, 0.92521E+00_JPRB ,&
&  0.92412E+00_JPRB, 0.92785E+00_JPRB, 0.92675E+00_JPRB, 0.90855E+00_JPRB, 0.88808E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.92978E+00_JPRB, 0.91223E+00_JPRB, 0.90280E+00_JPRB, 0.88730E+00_JPRB, 0.86042E+00_JPRB, 0.86689E+00_JPRB, 0.86453E+00_JPRB ,&
&  0.82181E+00_JPRB, 0.76873E+00_JPRB, 0.76779E+00_JPRB, 0.77019E+00_JPRB, 0.77754E+00_JPRB, 0.79364E+00_JPRB, 0.82507E+00_JPRB ,&
&  0.85599E+00_JPRB, 0.91240E+00_JPRB, 0.10429E+01_JPRB, 0.12678E+01_JPRB, 0.15264E+01_JPRB, 0.18006E+01_JPRB, 0.20967E+01_JPRB ,&
&  0.19889E+01_JPRB, 0.14165E+01_JPRB, 0.17353E+01_JPRB, 0.16460E+01_JPRB, 0.14186E+01_JPRB, 0.12450E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.12641E+01_JPRB, 0.12421E+01_JPRB, 0.12110E+01_JPRB, 0.11676E+01_JPRB, 0.11748E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.11901E+01_JPRB, 0.12006E+01_JPRB, 0.12255E+01_JPRB, 0.12807E+01_JPRB, 0.13509E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.14104E+01_JPRB, 0.14180E+01_JPRB, 0.13638E+01_JPRB, 0.13201E+01_JPRB, 0.13001E+01_JPRB, 0.13094E+01_JPRB, 0.13038E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.12554E+01_JPRB, 0.12740E+01_JPRB, 0.13304E+01_JPRB, 0.13698E+01_JPRB, 0.12951E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.14442E+01_JPRB, 0.15244E+01_JPRB, 0.15170E+01_JPRB, 0.17437E+01_JPRB, 0.12683E+01_JPRB, 0.13559E+01_JPRB, 0.12485E+01_JPRB ,&
&  0.14078E+01_JPRB, 0.14014E+01_JPRB, 0.13938E+01_JPRB, 0.14062E+01_JPRB, 0.14864E+01_JPRB, 0.13168E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.99257E+00_JPRB, 0.93784E+00_JPRB, 0.10789E+01_JPRB, 0.11892E+01_JPRB, 0.11558E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.15357E+01_JPRB, 0.12522E+01_JPRB, 0.10868E+01_JPRB, 0.10853E+01_JPRB, 0.10352E+01_JPRB, 0.95036E+00_JPRB, 0.95878E+00_JPRB ,&
&  0.97597E+00_JPRB, 0.10023E+01_JPRB, 0.98942E+00_JPRB, 0.97031E+00_JPRB, 0.95601E+00_JPRB, 0.94819E+00_JPRB, 0.95602E+00_JPRB ,&
&  0.95944E+00_JPRB, 0.96421E+00_JPRB, 0.96454E+00_JPRB, 0.95546E+00_JPRB, 0.94252E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.97018E+00_JPRB, 0.94889E+00_JPRB, 0.92643E+00_JPRB, 0.92021E+00_JPRB, 0.94712E+00_JPRB, 0.96903E+00_JPRB, 0.10801E+01_JPRB ,&
&  0.12831E+01_JPRB, 0.85479E+00_JPRB, 0.75275E+00_JPRB, 0.73269E+00_JPRB, 0.72451E+00_JPRB, 0.72503E+00_JPRB, 0.74914E+00_JPRB ,&
&  0.77425E+00_JPRB, 0.80049E+00_JPRB, 0.85322E+00_JPRB, 0.97189E+00_JPRB, 0.12031E+01_JPRB, 0.15630E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.15244E+01_JPRB, 0.16788E+01_JPRB, 0.19268E+01_JPRB, 0.16057E+01_JPRB, 0.14713E+01_JPRB, 0.13854E+01_JPRB, 0.13177E+01_JPRB ,&
&  0.12745E+01_JPRB, 0.12063E+01_JPRB, 0.12086E+01_JPRB, 0.11781E+01_JPRB, 0.11445E+01_JPRB, 0.11347E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11311E+01_JPRB, 0.11269E+01_JPRB, 0.11306E+01_JPRB, 0.11550E+01_JPRB, 0.12157E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.13418E+01_JPRB, 0.12846E+01_JPRB, 0.12285E+01_JPRB, 0.12204E+01_JPRB, 0.12493E+01_JPRB, 0.12626E+01_JPRB ,&
&  0.12722E+01_JPRB, 0.12605E+01_JPRB, 0.12702E+01_JPRB, 0.12676E+01_JPRB, 0.12189E+01_JPRB, 0.13392E+01_JPRB, 0.13500E+01_JPRB ,&
&  0.14450E+01_JPRB, 0.14893E+01_JPRB, 0.17225E+01_JPRB, 0.13953E+01_JPRB, 0.14594E+01_JPRB, 0.15250E+01_JPRB, 0.14981E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14710E+01_JPRB, 0.15383E+01_JPRB, 0.15101E+01_JPRB, 0.15293E+01_JPRB, 0.13865E+01_JPRB, 0.13988E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.11980E+01_JPRB, 0.10612E+01_JPRB, 0.11728E+01_JPRB, 0.12233E+01_JPRB, 0.11886E+01_JPRB, 0.14725E+01_JPRB ,&
&  0.17664E+01_JPRB, 0.16385E+01_JPRB, 0.14447E+01_JPRB, 0.12038E+01_JPRB, 0.99574E+00_JPRB, 0.93029E+00_JPRB, 0.92373E+00_JPRB ,&
&  0.94117E+00_JPRB, 0.99233E+00_JPRB, 0.98689E+00_JPRB, 0.98361E+00_JPRB, 0.97389E+00_JPRB, 0.96948E+00_JPRB, 0.97665E+00_JPRB ,&
&  0.99334E+00_JPRB, 0.10044E+01_JPRB, 0.10082E+01_JPRB, 0.10002E+01_JPRB, 0.99135E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.99996E+00_JPRB, 0.96838E+00_JPRB, 0.92569E+00_JPRB, 0.88515E+00_JPRB, 0.85465E+00_JPRB, 0.85221E+00_JPRB, 0.89127E+00_JPRB ,&
&  0.11514E+01_JPRB, 0.88047E+00_JPRB, 0.77922E+00_JPRB, 0.74562E+00_JPRB, 0.72919E+00_JPRB, 0.70859E+00_JPRB, 0.70499E+00_JPRB ,&
&  0.71577E+00_JPRB, 0.72946E+00_JPRB, 0.76078E+00_JPRB, 0.80647E+00_JPRB, 0.90667E+00_JPRB, 0.11473E+01_JPRB, 0.12561E+01_JPRB ,&
&  0.12343E+01_JPRB, 0.22055E+01_JPRB, 0.18543E+01_JPRB, 0.17215E+01_JPRB, 0.15555E+01_JPRB, 0.15411E+01_JPRB, 0.13417E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.12015E+01_JPRB, 0.12224E+01_JPRB, 0.12229E+01_JPRB, 0.12191E+01_JPRB, 0.11956E+01_JPRB, 0.11772E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11353E+01_JPRB, 0.11270E+01_JPRB, 0.11215E+01_JPRB, 0.11209E+01_JPRB, 0.11505E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.13026E+01_JPRB, 0.12636E+01_JPRB, 0.12234E+01_JPRB, 0.12115E+01_JPRB, 0.12421E+01_JPRB, 0.12638E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.11665E+01_JPRB, 0.12326E+01_JPRB, 0.12718E+01_JPRB, 0.12932E+01_JPRB, 0.13964E+01_JPRB, 0.13824E+01_JPRB ,&
&  0.15054E+01_JPRB, 0.15012E+01_JPRB, 0.17083E+01_JPRB, 0.14622E+01_JPRB, 0.14486E+01_JPRB, 0.16282E+01_JPRB, 0.15967E+01_JPRB ,&
&  0.14293E+01_JPRB, 0.15141E+01_JPRB, 0.15824E+01_JPRB, 0.16069E+01_JPRB, 0.15260E+01_JPRB, 0.14288E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.13184E+01_JPRB, 0.12338E+01_JPRB, 0.11225E+01_JPRB, 0.12166E+01_JPRB, 0.11789E+01_JPRB, 0.15741E+01_JPRB ,&
&  0.19585E+01_JPRB, 0.18925E+01_JPRB, 0.18902E+01_JPRB, 0.16708E+01_JPRB, 0.12722E+01_JPRB, 0.10492E+01_JPRB, 0.95366E+00_JPRB ,&
&  0.95694E+00_JPRB, 0.98037E+00_JPRB, 0.97697E+00_JPRB, 0.95788E+00_JPRB, 0.94977E+00_JPRB, 0.94738E+00_JPRB, 0.94744E+00_JPRB ,&
&  0.96484E+00_JPRB, 0.98900E+00_JPRB, 0.10051E+01_JPRB, 0.10124E+01_JPRB, 0.10196E+01_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.10490E+01_JPRB, 0.10360E+01_JPRB, 0.97417E+00_JPRB, 0.90989E+00_JPRB, 0.87006E+00_JPRB, 0.85086E+00_JPRB, 0.84533E+00_JPRB ,&
&  0.10207E+01_JPRB, 0.89566E+00_JPRB, 0.80352E+00_JPRB, 0.76127E+00_JPRB, 0.73610E+00_JPRB, 0.72043E+00_JPRB, 0.70860E+00_JPRB ,&
&  0.70263E+00_JPRB, 0.70236E+00_JPRB, 0.72480E+00_JPRB, 0.73656E+00_JPRB, 0.77138E+00_JPRB, 0.88993E+00_JPRB, 0.81038E+00_JPRB ,&
&  0.11425E+01_JPRB, 0.20460E+01_JPRB, 0.18918E+01_JPRB, 0.18192E+01_JPRB, 0.16650E+01_JPRB, 0.16187E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.12303E+01_JPRB, 0.12564E+01_JPRB, 0.12878E+01_JPRB, 0.13086E+01_JPRB, 0.13032E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12048E+01_JPRB, 0.11623E+01_JPRB, 0.11485E+01_JPRB, 0.11308E+01_JPRB, 0.11201E+01_JPRB, 0.11332E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.12278E+01_JPRB, 0.12804E+01_JPRB, 0.12009E+01_JPRB, 0.12029E+01_JPRB, 0.12216E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.11185E+01_JPRB, 0.12180E+01_JPRB, 0.13339E+01_JPRB, 0.14308E+01_JPRB, 0.14915E+01_JPRB, 0.15091E+01_JPRB ,&
&  0.15931E+01_JPRB, 0.15884E+01_JPRB, 0.15737E+01_JPRB, 0.14480E+01_JPRB, 0.15108E+01_JPRB, 0.17142E+01_JPRB, 0.17752E+01_JPRB ,&
&  0.17661E+01_JPRB, 0.16316E+01_JPRB, 0.16299E+01_JPRB, 0.16096E+01_JPRB, 0.15482E+01_JPRB, 0.14132E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.15959E+01_JPRB, 0.17810E+01_JPRB, 0.16378E+01_JPRB, 0.14020E+01_JPRB, 0.12449E+01_JPRB, 0.10813E+01_JPRB, 0.13718E+01_JPRB ,&
&  0.18516E+01_JPRB, 0.19215E+01_JPRB, 0.18802E+01_JPRB, 0.20800E+01_JPRB, 0.16650E+01_JPRB, 0.13725E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.98813E+00_JPRB, 0.98800E+00_JPRB, 0.96826E+00_JPRB, 0.94980E+00_JPRB, 0.93590E+00_JPRB, 0.92296E+00_JPRB ,&
&  0.92295E+00_JPRB, 0.94412E+00_JPRB, 0.97837E+00_JPRB, 0.10177E+01_JPRB, 0.10463E+01_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.10655E+01_JPRB, 0.10644E+01_JPRB, 0.99718E+00_JPRB, 0.91484E+00_JPRB, 0.86161E+00_JPRB, 0.83592E+00_JPRB, 0.81526E+00_JPRB ,&
&  0.86517E+00_JPRB, 0.84777E+00_JPRB, 0.80465E+00_JPRB, 0.77030E+00_JPRB, 0.75772E+00_JPRB, 0.73691E+00_JPRB, 0.72426E+00_JPRB ,&
&  0.70334E+00_JPRB, 0.69063E+00_JPRB, 0.71899E+00_JPRB, 0.72403E+00_JPRB, 0.73880E+00_JPRB, 0.64636E+00_JPRB, 0.73796E+00_JPRB ,&
&  0.12632E+01_JPRB, 0.20278E+01_JPRB, 0.19440E+01_JPRB, 0.19665E+01_JPRB, 0.19171E+01_JPRB, 0.19309E+01_JPRB, 0.13985E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.12910E+01_JPRB, 0.13084E+01_JPRB, 0.13717E+01_JPRB, 0.13931E+01_JPRB, 0.13816E+01_JPRB, 0.13542E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.12193E+01_JPRB, 0.11786E+01_JPRB, 0.11453E+01_JPRB, 0.11056E+01_JPRB, 0.10872E+01_JPRB, 0.11116E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11678E+01_JPRB, 0.12278E+01_JPRB, 0.11926E+01_JPRB, 0.11521E+01_JPRB, 0.11547E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.12351E+01_JPRB, 0.12766E+01_JPRB, 0.13129E+01_JPRB, 0.14401E+01_JPRB, 0.15964E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.17778E+01_JPRB, 0.16974E+01_JPRB, 0.15046E+01_JPRB, 0.15250E+01_JPRB, 0.16705E+01_JPRB, 0.19314E+01_JPRB, 0.18316E+01_JPRB ,&
&  0.15966E+01_JPRB, 0.13222E+01_JPRB, 0.12945E+01_JPRB, 0.14665E+01_JPRB, 0.15721E+01_JPRB, 0.15267E+01_JPRB, 0.17682E+01_JPRB ,&
&  0.16906E+01_JPRB, 0.10406E+01_JPRB, 0.88872E+00_JPRB, 0.11188E+01_JPRB, 0.89204E+00_JPRB, 0.82275E+00_JPRB, 0.14175E+01_JPRB ,&
&  0.18546E+01_JPRB, 0.22287E+01_JPRB, 0.22700E+01_JPRB, 0.20096E+01_JPRB, 0.19408E+01_JPRB, 0.17989E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.13627E+01_JPRB, 0.11912E+01_JPRB, 0.10659E+01_JPRB, 0.99719E+00_JPRB, 0.96940E+00_JPRB, 0.95036E+00_JPRB, 0.92635E+00_JPRB ,&
&  0.92032E+00_JPRB, 0.92847E+00_JPRB, 0.95494E+00_JPRB, 0.10073E+01_JPRB, 0.10517E+01_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.10844E+01_JPRB, 0.10595E+01_JPRB, 0.98866E+00_JPRB, 0.90254E+00_JPRB, 0.85059E+00_JPRB, 0.81613E+00_JPRB, 0.79373E+00_JPRB ,&
&  0.79645E+00_JPRB, 0.79688E+00_JPRB, 0.79058E+00_JPRB, 0.77838E+00_JPRB, 0.77151E+00_JPRB, 0.75546E+00_JPRB, 0.73824E+00_JPRB ,&
&  0.71525E+00_JPRB, 0.68744E+00_JPRB, 0.73186E+00_JPRB, 0.76442E+00_JPRB, 0.74584E+00_JPRB, 0.67137E+00_JPRB, 0.81185E+00_JPRB ,&
&  0.12253E+01_JPRB, 0.19278E+01_JPRB, 0.22610E+01_JPRB, 0.23047E+01_JPRB, 0.21273E+01_JPRB, 0.20440E+01_JPRB, 0.16130E+01_JPRB ,&
&  0.14235E+01_JPRB, 0.14165E+01_JPRB, 0.13983E+01_JPRB, 0.14372E+01_JPRB, 0.14759E+01_JPRB, 0.14628E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.13556E+01_JPRB, 0.12775E+01_JPRB, 0.12182E+01_JPRB, 0.11735E+01_JPRB, 0.11264E+01_JPRB, 0.10762E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11275E+01_JPRB, 0.11763E+01_JPRB, 0.12255E+01_JPRB, 0.10221E+01_JPRB, 0.11047E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.12674E+01_JPRB, 0.12670E+01_JPRB, 0.13731E+01_JPRB, 0.15127E+01_JPRB, 0.16092E+01_JPRB, 0.17789E+01_JPRB ,&
&  0.19924E+01_JPRB, 0.19033E+01_JPRB, 0.17067E+01_JPRB, 0.17375E+01_JPRB, 0.18596E+01_JPRB, 0.20888E+01_JPRB, 0.14655E+01_JPRB ,&
&  0.13484E+01_JPRB, 0.15751E+01_JPRB, 0.14493E+01_JPRB, 0.13151E+01_JPRB, 0.15316E+01_JPRB, 0.15971E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.67653E+00_JPRB, 0.52661E+00_JPRB, 0.53237E+00_JPRB, 0.54270E+00_JPRB, 0.54433E+00_JPRB, 0.77124E+00_JPRB, 0.22717E+01_JPRB ,&
&  0.19900E+01_JPRB, 0.24126E+01_JPRB, 0.24456E+01_JPRB, 0.21966E+01_JPRB, 0.20777E+01_JPRB, 0.19642E+01_JPRB, 0.19956E+01_JPRB ,&
&  0.17631E+01_JPRB, 0.15710E+01_JPRB, 0.14009E+01_JPRB, 0.12270E+01_JPRB, 0.10932E+01_JPRB, 0.10043E+01_JPRB, 0.96274E+00_JPRB ,&
&  0.94989E+00_JPRB, 0.95776E+00_JPRB, 0.98091E+00_JPRB, 0.10224E+01_JPRB, 0.10734E+01_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.11093E+01_JPRB, 0.10523E+01_JPRB, 0.96616E+00_JPRB, 0.87320E+00_JPRB, 0.82420E+00_JPRB, 0.78721E+00_JPRB, 0.76135E+00_JPRB ,&
&  0.75768E+00_JPRB, 0.76003E+00_JPRB, 0.76634E+00_JPRB, 0.76682E+00_JPRB, 0.76238E+00_JPRB, 0.75345E+00_JPRB, 0.73826E+00_JPRB ,&
&  0.71490E+00_JPRB, 0.68896E+00_JPRB, 0.77381E+00_JPRB, 0.76139E+00_JPRB, 0.77281E+00_JPRB, 0.72168E+00_JPRB, 0.82316E+00_JPRB ,&
&  0.11425E+01_JPRB, 0.17668E+01_JPRB, 0.22615E+01_JPRB, 0.23453E+01_JPRB, 0.24121E+01_JPRB, 0.23815E+01_JPRB, 0.21684E+01_JPRB ,&
&  0.19215E+01_JPRB, 0.17859E+01_JPRB, 0.16630E+01_JPRB, 0.15959E+01_JPRB, 0.15440E+01_JPRB, 0.15083E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.13227E+01_JPRB, 0.12716E+01_JPRB, 0.12329E+01_JPRB, 0.11808E+01_JPRB, 0.11241E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.11095E+01_JPRB, 0.11387E+01_JPRB, 0.12035E+01_JPRB, 0.11034E+01_JPRB, 0.10138E+01_JPRB, 0.11155E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.13680E+01_JPRB, 0.13966E+01_JPRB, 0.15397E+01_JPRB, 0.15967E+01_JPRB, 0.16655E+01_JPRB, 0.19337E+01_JPRB ,&
&  0.21664E+01_JPRB, 0.24142E+01_JPRB, 0.19751E+01_JPRB, 0.19776E+01_JPRB, 0.21285E+01_JPRB, 0.17312E+01_JPRB, 0.16373E+01_JPRB ,&
&  0.18758E+01_JPRB, 0.16585E+01_JPRB, 0.14707E+01_JPRB, 0.12443E+01_JPRB, 0.14905E+01_JPRB, 0.13162E+01_JPRB, 0.71191E+00_JPRB ,&
&  0.55383E+00_JPRB, 0.50160E+00_JPRB, 0.47495E+00_JPRB, 0.47807E+00_JPRB, 0.51231E+00_JPRB, 0.79096E+00_JPRB, 0.14446E+01_JPRB ,&
&  0.17102E+01_JPRB, 0.21874E+01_JPRB, 0.23884E+01_JPRB, 0.23644E+01_JPRB, 0.21799E+01_JPRB, 0.20607E+01_JPRB, 0.21356E+01_JPRB ,&
&  0.21885E+01_JPRB, 0.20152E+01_JPRB, 0.17585E+01_JPRB, 0.15658E+01_JPRB, 0.14368E+01_JPRB, 0.12953E+01_JPRB, 0.11712E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.10504E+01_JPRB, 0.10485E+01_JPRB, 0.10828E+01_JPRB, 0.11345E+01_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.11414E+01_JPRB, 0.10864E+01_JPRB, 0.98649E+00_JPRB, 0.87714E+00_JPRB, 0.81414E+00_JPRB, 0.76476E+00_JPRB, 0.73557E+00_JPRB ,&
&  0.73116E+00_JPRB, 0.73598E+00_JPRB, 0.73974E+00_JPRB, 0.74000E+00_JPRB, 0.73414E+00_JPRB, 0.72732E+00_JPRB, 0.72100E+00_JPRB ,&
&  0.69873E+00_JPRB, 0.70464E+00_JPRB, 0.73116E+00_JPRB, 0.65896E+00_JPRB, 0.65204E+00_JPRB, 0.64957E+00_JPRB, 0.72232E+00_JPRB ,&
&  0.10764E+01_JPRB, 0.16688E+01_JPRB, 0.21812E+01_JPRB, 0.24585E+01_JPRB, 0.26768E+01_JPRB, 0.25958E+01_JPRB, 0.28164E+01_JPRB ,&
&  0.25615E+01_JPRB, 0.22948E+01_JPRB, 0.20539E+01_JPRB, 0.18769E+01_JPRB, 0.17037E+01_JPRB, 0.15885E+01_JPRB, 0.15178E+01_JPRB ,&
&  0.14119E+01_JPRB, 0.13273E+01_JPRB, 0.12849E+01_JPRB, 0.12473E+01_JPRB, 0.12093E+01_JPRB, 0.11731E+01_JPRB, 0.11424E+01_JPRB ,&
&  0.11368E+01_JPRB, 0.11440E+01_JPRB, 0.11553E+01_JPRB, 0.11696E+01_JPRB, 0.12536E+01_JPRB, 0.12253E+01_JPRB, 0.11986E+01_JPRB ,&
&  0.12045E+01_JPRB, 0.12750E+01_JPRB, 0.15072E+01_JPRB, 0.15603E+01_JPRB, 0.16395E+01_JPRB, 0.18201E+01_JPRB, 0.20865E+01_JPRB ,&
&  0.23470E+01_JPRB, 0.26795E+01_JPRB, 0.22826E+01_JPRB, 0.23237E+01_JPRB, 0.21072E+01_JPRB, 0.17348E+01_JPRB, 0.20133E+01_JPRB ,&
&  0.22026E+01_JPRB, 0.19429E+01_JPRB, 0.15814E+01_JPRB, 0.12599E+01_JPRB, 0.11759E+01_JPRB, 0.84986E+00_JPRB, 0.55749E+00_JPRB ,&
&  0.52301E+00_JPRB, 0.50511E+00_JPRB, 0.47772E+00_JPRB, 0.54390E+00_JPRB, 0.55842E+00_JPRB, 0.78201E+00_JPRB, 0.12618E+01_JPRB ,&
&  0.17731E+01_JPRB, 0.19059E+01_JPRB, 0.21428E+01_JPRB, 0.21950E+01_JPRB, 0.21398E+01_JPRB, 0.20861E+01_JPRB, 0.23654E+01_JPRB ,&
&  0.21524E+01_JPRB, 0.20540E+01_JPRB, 0.19653E+01_JPRB, 0.17302E+01_JPRB, 0.16115E+01_JPRB, 0.15342E+01_JPRB, 0.14329E+01_JPRB ,&
&  0.13069E+01_JPRB, 0.11947E+01_JPRB, 0.11551E+01_JPRB, 0.11639E+01_JPRB, 0.11739E+01_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.11795E+01_JPRB, 0.11306E+01_JPRB, 0.10334E+01_JPRB, 0.91910E+00_JPRB, 0.83570E+00_JPRB, 0.77682E+00_JPRB, 0.74785E+00_JPRB ,&
&  0.74724E+00_JPRB, 0.74568E+00_JPRB, 0.74512E+00_JPRB, 0.73983E+00_JPRB, 0.73100E+00_JPRB, 0.71866E+00_JPRB, 0.71606E+00_JPRB ,&
&  0.69372E+00_JPRB, 0.78881E+00_JPRB, 0.59513E+00_JPRB, 0.50234E+00_JPRB, 0.54843E+00_JPRB, 0.60638E+00_JPRB, 0.61465E+00_JPRB ,&
&  0.99339E+00_JPRB, 0.15915E+01_JPRB, 0.20652E+01_JPRB, 0.26449E+01_JPRB, 0.34658E+01_JPRB, 0.33066E+01_JPRB, 0.33021E+01_JPRB ,&
&  0.31670E+01_JPRB, 0.28404E+01_JPRB, 0.25074E+01_JPRB, 0.21666E+01_JPRB, 0.18553E+01_JPRB, 0.16710E+01_JPRB, 0.15566E+01_JPRB ,&
&  0.14380E+01_JPRB, 0.13394E+01_JPRB, 0.12801E+01_JPRB, 0.12573E+01_JPRB, 0.12213E+01_JPRB, 0.11910E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11827E+01_JPRB, 0.11896E+01_JPRB, 0.11923E+01_JPRB, 0.12267E+01_JPRB, 0.12031E+01_JPRB, 0.11761E+01_JPRB, 0.14300E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.14672E+01_JPRB, 0.14946E+01_JPRB, 0.16735E+01_JPRB, 0.18009E+01_JPRB, 0.19381E+01_JPRB, 0.23129E+01_JPRB ,&
&  0.21835E+01_JPRB, 0.21775E+01_JPRB, 0.22589E+01_JPRB, 0.22869E+01_JPRB, 0.20711E+01_JPRB, 0.23295E+01_JPRB, 0.31509E+01_JPRB ,&
&  0.24323E+01_JPRB, 0.24627E+01_JPRB, 0.24761E+01_JPRB, 0.24392E+01_JPRB, 0.12100E+01_JPRB, 0.82719E+00_JPRB, 0.12536E+01_JPRB ,&
&  0.13414E+01_JPRB, 0.85938E+00_JPRB, 0.68062E+00_JPRB, 0.94601E+00_JPRB, 0.77981E+00_JPRB, 0.96160E+00_JPRB, 0.13946E+01_JPRB ,&
&  0.15508E+01_JPRB, 0.15047E+01_JPRB, 0.19595E+01_JPRB, 0.22226E+01_JPRB, 0.22713E+01_JPRB, 0.22299E+01_JPRB, 0.23792E+01_JPRB ,&
&  0.23684E+01_JPRB, 0.21251E+01_JPRB, 0.21113E+01_JPRB, 0.18829E+01_JPRB, 0.17010E+01_JPRB, 0.15907E+01_JPRB, 0.15064E+01_JPRB ,&
&  0.14134E+01_JPRB, 0.13246E+01_JPRB, 0.12744E+01_JPRB, 0.12564E+01_JPRB, 0.12194E+01_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.11971E+01_JPRB, 0.11534E+01_JPRB, 0.10582E+01_JPRB, 0.94571E+00_JPRB, 0.83535E+00_JPRB, 0.75557E+00_JPRB, 0.73408E+00_JPRB ,&
&  0.74269E+00_JPRB, 0.75237E+00_JPRB, 0.75271E+00_JPRB, 0.74398E+00_JPRB, 0.73045E+00_JPRB, 0.71681E+00_JPRB, 0.70231E+00_JPRB ,&
&  0.67717E+00_JPRB, 0.61908E+00_JPRB, 0.47822E+00_JPRB, 0.42417E+00_JPRB, 0.53283E+00_JPRB, 0.54879E+00_JPRB, 0.60224E+00_JPRB ,&
&  0.88640E+00_JPRB, 0.13744E+01_JPRB, 0.19337E+01_JPRB, 0.25682E+01_JPRB, 0.32248E+01_JPRB, 0.35351E+01_JPRB, 0.35210E+01_JPRB ,&
&  0.35790E+01_JPRB, 0.32338E+01_JPRB, 0.27471E+01_JPRB, 0.23377E+01_JPRB, 0.20532E+01_JPRB, 0.18657E+01_JPRB, 0.16757E+01_JPRB ,&
&  0.14926E+01_JPRB, 0.13666E+01_JPRB, 0.12970E+01_JPRB, 0.12519E+01_JPRB, 0.12274E+01_JPRB, 0.11927E+01_JPRB, 0.11622E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12183E+01_JPRB, 0.12528E+01_JPRB, 0.12675E+01_JPRB, 0.11360E+01_JPRB, 0.11175E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.15009E+01_JPRB, 0.15139E+01_JPRB, 0.15860E+01_JPRB, 0.17368E+01_JPRB, 0.19501E+01_JPRB, 0.20079E+01_JPRB, 0.25200E+01_JPRB ,&
&  0.24098E+01_JPRB, 0.22444E+01_JPRB, 0.21335E+01_JPRB, 0.18764E+01_JPRB, 0.22421E+01_JPRB, 0.33363E+01_JPRB, 0.33197E+01_JPRB ,&
&  0.30401E+01_JPRB, 0.28689E+01_JPRB, 0.26941E+01_JPRB, 0.23814E+01_JPRB, 0.19980E+01_JPRB, 0.14244E+01_JPRB, 0.17185E+01_JPRB ,&
&  0.18222E+01_JPRB, 0.18088E+01_JPRB, 0.16984E+01_JPRB, 0.14518E+01_JPRB, 0.12806E+01_JPRB, 0.15032E+01_JPRB, 0.14205E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.12323E+01_JPRB, 0.16193E+01_JPRB, 0.21124E+01_JPRB, 0.22599E+01_JPRB, 0.21448E+01_JPRB, 0.23754E+01_JPRB ,&
&  0.23022E+01_JPRB, 0.22614E+01_JPRB, 0.21356E+01_JPRB, 0.19251E+01_JPRB, 0.17498E+01_JPRB, 0.16170E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.14382E+01_JPRB, 0.13946E+01_JPRB, 0.13437E+01_JPRB, 0.12960E+01_JPRB, 0.12376E+01_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.12146E+01_JPRB, 0.11620E+01_JPRB, 0.10612E+01_JPRB, 0.94243E+00_JPRB, 0.82402E+00_JPRB, 0.72804E+00_JPRB, 0.70303E+00_JPRB ,&
&  0.71271E+00_JPRB, 0.72230E+00_JPRB, 0.72115E+00_JPRB, 0.71059E+00_JPRB, 0.69782E+00_JPRB, 0.67838E+00_JPRB, 0.66097E+00_JPRB ,&
&  0.64785E+00_JPRB, 0.56668E+00_JPRB, 0.44611E+00_JPRB, 0.43053E+00_JPRB, 0.47620E+00_JPRB, 0.51049E+00_JPRB, 0.61726E+00_JPRB ,&
&  0.80128E+00_JPRB, 0.12485E+01_JPRB, 0.16612E+01_JPRB, 0.21517E+01_JPRB, 0.26673E+01_JPRB, 0.30720E+01_JPRB, 0.31333E+01_JPRB ,&
&  0.29510E+01_JPRB, 0.29226E+01_JPRB, 0.26974E+01_JPRB, 0.23954E+01_JPRB, 0.21252E+01_JPRB, 0.19255E+01_JPRB, 0.17533E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.14226E+01_JPRB, 0.13294E+01_JPRB, 0.12729E+01_JPRB, 0.12373E+01_JPRB, 0.12091E+01_JPRB, 0.11888E+01_JPRB ,&
&  0.11899E+01_JPRB, 0.12219E+01_JPRB, 0.12474E+01_JPRB, 0.12745E+01_JPRB, 0.11206E+01_JPRB, 0.11916E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.17751E+01_JPRB, 0.17496E+01_JPRB, 0.18123E+01_JPRB, 0.21266E+01_JPRB, 0.20075E+01_JPRB, 0.22149E+01_JPRB, 0.28950E+01_JPRB ,&
&  0.33185E+01_JPRB, 0.33798E+01_JPRB, 0.34641E+01_JPRB, 0.28517E+01_JPRB, 0.26021E+01_JPRB, 0.37210E+01_JPRB, 0.34824E+01_JPRB ,&
&  0.31679E+01_JPRB, 0.30194E+01_JPRB, 0.27621E+01_JPRB, 0.27762E+01_JPRB, 0.23584E+01_JPRB, 0.20549E+01_JPRB, 0.15052E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.15079E+01_JPRB, 0.17081E+01_JPRB, 0.15835E+01_JPRB, 0.14931E+01_JPRB, 0.13623E+01_JPRB, 0.11657E+01_JPRB ,&
&  0.10628E+01_JPRB, 0.10155E+01_JPRB, 0.11024E+01_JPRB, 0.14473E+01_JPRB, 0.18144E+01_JPRB, 0.17619E+01_JPRB, 0.20353E+01_JPRB ,&
&  0.21647E+01_JPRB, 0.21923E+01_JPRB, 0.19517E+01_JPRB, 0.19045E+01_JPRB, 0.17964E+01_JPRB, 0.16986E+01_JPRB, 0.15832E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.14163E+01_JPRB, 0.13581E+01_JPRB, 0.13006E+01_JPRB, 0.12495E+01_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.12156E+01_JPRB, 0.11552E+01_JPRB, 0.10481E+01_JPRB, 0.93286E+00_JPRB, 0.80461E+00_JPRB, 0.69686E+00_JPRB, 0.67809E+00_JPRB ,&
&  0.69772E+00_JPRB, 0.69509E+00_JPRB, 0.67527E+00_JPRB, 0.64177E+00_JPRB, 0.62909E+00_JPRB, 0.63177E+00_JPRB, 0.63189E+00_JPRB ,&
&  0.62628E+00_JPRB, 0.68135E+00_JPRB, 0.55225E+00_JPRB, 0.45955E+00_JPRB, 0.44733E+00_JPRB, 0.51058E+00_JPRB, 0.65526E+00_JPRB ,&
&  0.81166E+00_JPRB, 0.10995E+01_JPRB, 0.13801E+01_JPRB, 0.16260E+01_JPRB, 0.19591E+01_JPRB, 0.22259E+01_JPRB, 0.23669E+01_JPRB ,&
&  0.25432E+01_JPRB, 0.21451E+01_JPRB, 0.21242E+01_JPRB, 0.19565E+01_JPRB, 0.17894E+01_JPRB, 0.16538E+01_JPRB, 0.15525E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.13410E+01_JPRB, 0.12366E+01_JPRB, 0.11659E+01_JPRB, 0.11290E+01_JPRB, 0.10967E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10958E+01_JPRB, 0.11496E+01_JPRB, 0.12178E+01_JPRB, 0.12475E+01_JPRB, 0.13232E+01_JPRB, 0.14721E+01_JPRB, 0.16616E+01_JPRB ,&
&  0.16934E+01_JPRB, 0.17286E+01_JPRB, 0.23296E+01_JPRB, 0.25759E+01_JPRB, 0.29516E+01_JPRB, 0.31877E+01_JPRB, 0.34188E+01_JPRB ,&
&  0.38228E+01_JPRB, 0.38070E+01_JPRB, 0.37723E+01_JPRB, 0.34294E+01_JPRB, 0.39132E+01_JPRB, 0.38104E+01_JPRB, 0.35520E+01_JPRB ,&
&  0.32432E+01_JPRB, 0.31818E+01_JPRB, 0.29292E+01_JPRB, 0.27803E+01_JPRB, 0.27398E+01_JPRB, 0.28570E+01_JPRB, 0.25350E+01_JPRB ,&
&  0.20434E+01_JPRB, 0.22234E+01_JPRB, 0.19372E+01_JPRB, 0.15892E+01_JPRB, 0.12238E+01_JPRB, 0.10917E+01_JPRB, 0.10104E+01_JPRB ,&
&  0.94996E+00_JPRB, 0.92008E+00_JPRB, 0.94543E+00_JPRB, 0.10883E+01_JPRB, 0.13120E+01_JPRB, 0.14727E+01_JPRB, 0.15590E+01_JPRB ,&
&  0.15222E+01_JPRB, 0.17448E+01_JPRB, 0.17336E+01_JPRB, 0.16571E+01_JPRB, 0.16357E+01_JPRB, 0.16300E+01_JPRB, 0.15870E+01_JPRB ,&
&  0.14926E+01_JPRB, 0.14164E+01_JPRB, 0.13546E+01_JPRB, 0.13037E+01_JPRB, 0.12544E+01_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.11735E+01_JPRB, 0.11208E+01_JPRB, 0.10296E+01_JPRB, 0.91943E+00_JPRB, 0.78952E+00_JPRB, 0.68909E+00_JPRB, 0.66547E+00_JPRB ,&
&  0.65668E+00_JPRB, 0.64357E+00_JPRB, 0.62083E+00_JPRB, 0.60743E+00_JPRB, 0.60555E+00_JPRB, 0.60915E+00_JPRB, 0.63229E+00_JPRB ,&
&  0.67054E+00_JPRB, 0.94023E+00_JPRB, 0.75489E+00_JPRB, 0.63109E+00_JPRB, 0.56649E+00_JPRB, 0.66716E+00_JPRB, 0.75798E+00_JPRB ,&
&  0.87665E+00_JPRB, 0.10212E+01_JPRB, 0.10873E+01_JPRB, 0.12823E+01_JPRB, 0.14681E+01_JPRB, 0.15432E+01_JPRB, 0.16683E+01_JPRB ,&
&  0.16768E+01_JPRB, 0.16892E+01_JPRB, 0.15159E+01_JPRB, 0.15006E+01_JPRB, 0.14206E+01_JPRB, 0.12734E+01_JPRB, 0.13101E+01_JPRB ,&
&  0.12118E+01_JPRB, 0.11399E+01_JPRB, 0.10706E+01_JPRB, 0.10378E+01_JPRB, 0.10453E+01_JPRB, 0.10467E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.10675E+01_JPRB, 0.11393E+01_JPRB, 0.12191E+01_JPRB, 0.13290E+01_JPRB, 0.15321E+01_JPRB, 0.17145E+01_JPRB ,&
&  0.21388E+01_JPRB, 0.21295E+01_JPRB, 0.19565E+01_JPRB, 0.25618E+01_JPRB, 0.37165E+01_JPRB, 0.38755E+01_JPRB, 0.35992E+01_JPRB ,&
&  0.40282E+01_JPRB, 0.41834E+01_JPRB, 0.41757E+01_JPRB, 0.40204E+01_JPRB, 0.38601E+01_JPRB, 0.37302E+01_JPRB, 0.36294E+01_JPRB ,&
&  0.33085E+01_JPRB, 0.31313E+01_JPRB, 0.29908E+01_JPRB, 0.29245E+01_JPRB, 0.27390E+01_JPRB, 0.26605E+01_JPRB, 0.26456E+01_JPRB ,&
&  0.23922E+01_JPRB, 0.19211E+01_JPRB, 0.12701E+01_JPRB, 0.12549E+01_JPRB, 0.91381E+00_JPRB, 0.78986E+00_JPRB, 0.87992E+00_JPRB ,&
&  0.84116E+00_JPRB, 0.89886E+00_JPRB, 0.96063E+00_JPRB, 0.93343E+00_JPRB, 0.10177E+01_JPRB, 0.10895E+01_JPRB, 0.11868E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11165E+01_JPRB, 0.13235E+01_JPRB, 0.13505E+01_JPRB, 0.13598E+01_JPRB, 0.13678E+01_JPRB, 0.13667E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.13030E+01_JPRB, 0.12691E+01_JPRB, 0.12325E+01_JPRB, 0.12074E+01_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.10436E+01_JPRB, 0.99721E+00_JPRB, 0.93017E+00_JPRB, 0.84754E+00_JPRB, 0.74981E+00_JPRB, 0.66790E+00_JPRB, 0.62943E+00_JPRB ,&
&  0.62012E+00_JPRB, 0.60275E+00_JPRB, 0.58375E+00_JPRB, 0.57761E+00_JPRB, 0.58122E+00_JPRB, 0.59327E+00_JPRB, 0.64405E+00_JPRB ,&
&  0.77209E+00_JPRB, 0.99925E+00_JPRB, 0.85713E+00_JPRB, 0.72770E+00_JPRB, 0.86096E+00_JPRB, 0.82892E+00_JPRB, 0.87473E+00_JPRB ,&
&  0.88299E+00_JPRB, 0.10239E+01_JPRB, 0.98619E+00_JPRB, 0.99437E+00_JPRB, 0.11185E+01_JPRB, 0.11898E+01_JPRB, 0.12230E+01_JPRB ,&
&  0.12033E+01_JPRB, 0.11353E+01_JPRB, 0.10676E+01_JPRB, 0.10716E+01_JPRB, 0.11301E+01_JPRB, 0.10720E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.10150E+01_JPRB, 0.96381E+00_JPRB, 0.89947E+00_JPRB, 0.88444E+00_JPRB, 0.91180E+00_JPRB, 0.96194E+00_JPRB, 0.98977E+00_JPRB ,&
&  0.10407E+01_JPRB, 0.10936E+01_JPRB, 0.11340E+01_JPRB, 0.12271E+01_JPRB, 0.13705E+01_JPRB, 0.17179E+01_JPRB, 0.22052E+01_JPRB ,&
&  0.26986E+01_JPRB, 0.29338E+01_JPRB, 0.31964E+01_JPRB, 0.38170E+01_JPRB, 0.40449E+01_JPRB, 0.39921E+01_JPRB, 0.39685E+01_JPRB ,&
&  0.38570E+01_JPRB, 0.38574E+01_JPRB, 0.39530E+01_JPRB, 0.37806E+01_JPRB, 0.36382E+01_JPRB, 0.36068E+01_JPRB, 0.35295E+01_JPRB ,&
&  0.31724E+01_JPRB, 0.29728E+01_JPRB, 0.29401E+01_JPRB, 0.27697E+01_JPRB, 0.26253E+01_JPRB, 0.24533E+01_JPRB, 0.24365E+01_JPRB ,&
&  0.25066E+01_JPRB, 0.14185E+01_JPRB, 0.10435E+01_JPRB, 0.11421E+01_JPRB, 0.88355E+00_JPRB, 0.80592E+00_JPRB, 0.96013E+00_JPRB ,&
&  0.94784E+00_JPRB, 0.93160E+00_JPRB, 0.95220E+00_JPRB, 0.88949E+00_JPRB, 0.83062E+00_JPRB, 0.87148E+00_JPRB, 0.85796E+00_JPRB ,&
&  0.87486E+00_JPRB, 0.86556E+00_JPRB, 0.10102E+01_JPRB, 0.11074E+01_JPRB, 0.11517E+01_JPRB, 0.11524E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.11110E+01_JPRB, 0.11012E+01_JPRB, 0.10977E+01_JPRB, 0.10868E+01_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.91145E+00_JPRB, 0.88602E+00_JPRB, 0.84926E+00_JPRB, 0.79996E+00_JPRB, 0.73426E+00_JPRB, 0.67465E+00_JPRB, 0.61984E+00_JPRB ,&
&  0.61141E+00_JPRB, 0.59314E+00_JPRB, 0.57418E+00_JPRB, 0.56869E+00_JPRB, 0.57279E+00_JPRB, 0.58021E+00_JPRB, 0.60286E+00_JPRB ,&
&  0.54321E+00_JPRB, 0.67438E+00_JPRB, 0.63312E+00_JPRB, 0.75373E+00_JPRB, 0.10429E+01_JPRB, 0.84528E+00_JPRB, 0.85874E+00_JPRB ,&
&  0.86170E+00_JPRB, 0.91948E+00_JPRB, 0.91245E+00_JPRB, 0.90063E+00_JPRB, 0.93684E+00_JPRB, 0.95989E+00_JPRB, 0.93950E+00_JPRB ,&
&  0.87216E+00_JPRB, 0.77935E+00_JPRB, 0.75185E+00_JPRB, 0.75080E+00_JPRB, 0.82605E+00_JPRB, 0.88494E+00_JPRB, 0.89798E+00_JPRB ,&
&  0.86340E+00_JPRB, 0.82911E+00_JPRB, 0.79882E+00_JPRB, 0.81281E+00_JPRB, 0.84727E+00_JPRB, 0.87659E+00_JPRB, 0.92963E+00_JPRB ,&
&  0.10052E+01_JPRB, 0.10863E+01_JPRB, 0.11807E+01_JPRB, 0.13065E+01_JPRB, 0.13326E+01_JPRB, 0.20619E+01_JPRB, 0.33354E+01_JPRB ,&
&  0.38222E+01_JPRB, 0.36916E+01_JPRB, 0.39559E+01_JPRB, 0.46576E+01_JPRB, 0.46550E+01_JPRB, 0.42920E+01_JPRB, 0.38928E+01_JPRB ,&
&  0.35474E+01_JPRB, 0.33684E+01_JPRB, 0.35319E+01_JPRB, 0.35351E+01_JPRB, 0.33044E+01_JPRB, 0.33509E+01_JPRB, 0.32869E+01_JPRB ,&
&  0.30772E+01_JPRB, 0.29451E+01_JPRB, 0.29201E+01_JPRB, 0.28943E+01_JPRB, 0.27046E+01_JPRB, 0.27146E+01_JPRB, 0.25828E+01_JPRB ,&
&  0.24829E+01_JPRB, 0.20974E+01_JPRB, 0.14776E+01_JPRB, 0.11819E+01_JPRB, 0.87263E+00_JPRB, 0.87122E+00_JPRB, 0.10861E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.96507E+00_JPRB, 0.84210E+00_JPRB, 0.74658E+00_JPRB, 0.68459E+00_JPRB, 0.68409E+00_JPRB, 0.65383E+00_JPRB ,&
&  0.63231E+00_JPRB, 0.69323E+00_JPRB, 0.78097E+00_JPRB, 0.87277E+00_JPRB, 0.94321E+00_JPRB, 0.97666E+00_JPRB, 0.91790E+00_JPRB ,&
&  0.97890E+00_JPRB, 0.97303E+00_JPRB, 0.97760E+00_JPRB, 0.98281E+00_JPRB, 0.95915E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.77765E+00_JPRB, 0.78613E+00_JPRB, 0.76477E+00_JPRB, 0.74698E+00_JPRB, 0.69865E+00_JPRB, 0.66393E+00_JPRB, 0.61442E+00_JPRB ,&
&  0.61630E+00_JPRB, 0.59948E+00_JPRB, 0.57100E+00_JPRB, 0.56102E+00_JPRB, 0.56940E+00_JPRB, 0.56706E+00_JPRB, 0.51314E+00_JPRB ,&
&  0.48103E+00_JPRB, 0.53381E+00_JPRB, 0.53553E+00_JPRB, 0.61972E+00_JPRB, 0.69329E+00_JPRB, 0.65497E+00_JPRB, 0.67927E+00_JPRB ,&
&  0.76391E+00_JPRB, 0.78532E+00_JPRB, 0.80007E+00_JPRB, 0.80514E+00_JPRB, 0.80983E+00_JPRB, 0.81754E+00_JPRB, 0.81185E+00_JPRB ,&
&  0.74295E+00_JPRB, 0.69410E+00_JPRB, 0.65858E+00_JPRB, 0.65491E+00_JPRB, 0.72860E+00_JPRB, 0.76785E+00_JPRB, 0.74340E+00_JPRB ,&
&  0.71328E+00_JPRB, 0.70306E+00_JPRB, 0.71098E+00_JPRB, 0.76166E+00_JPRB, 0.79894E+00_JPRB, 0.81910E+00_JPRB, 0.87067E+00_JPRB ,&
&  0.95047E+00_JPRB, 0.10635E+01_JPRB, 0.11828E+01_JPRB, 0.13116E+01_JPRB, 0.13964E+01_JPRB, 0.21378E+01_JPRB, 0.31014E+01_JPRB ,&
&  0.32302E+01_JPRB, 0.34115E+01_JPRB, 0.36295E+01_JPRB, 0.38450E+01_JPRB, 0.39007E+01_JPRB, 0.35089E+01_JPRB, 0.31393E+01_JPRB ,&
&  0.30300E+01_JPRB, 0.29046E+01_JPRB, 0.35259E+01_JPRB, 0.33885E+01_JPRB, 0.31941E+01_JPRB, 0.30913E+01_JPRB, 0.29361E+01_JPRB ,&
&  0.28742E+01_JPRB, 0.28280E+01_JPRB, 0.27778E+01_JPRB, 0.26899E+01_JPRB, 0.25745E+01_JPRB, 0.25207E+01_JPRB, 0.23376E+01_JPRB ,&
&  0.22284E+01_JPRB, 0.20444E+01_JPRB, 0.16458E+01_JPRB, 0.13246E+01_JPRB, 0.10518E+01_JPRB, 0.10580E+01_JPRB, 0.96489E+00_JPRB ,&
&  0.82916E+00_JPRB, 0.68431E+00_JPRB, 0.65502E+00_JPRB, 0.61910E+00_JPRB, 0.61006E+00_JPRB, 0.58800E+00_JPRB, 0.56186E+00_JPRB ,&
&  0.58751E+00_JPRB, 0.65004E+00_JPRB, 0.66238E+00_JPRB, 0.70427E+00_JPRB, 0.76369E+00_JPRB, 0.82949E+00_JPRB, 0.72310E+00_JPRB ,&
&  0.72995E+00_JPRB, 0.80549E+00_JPRB, 0.82016E+00_JPRB, 0.81709E+00_JPRB, 0.79346E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.71479E+00_JPRB, 0.72888E+00_JPRB, 0.71057E+00_JPRB, 0.71158E+00_JPRB, 0.67830E+00_JPRB, 0.65823E+00_JPRB, 0.58758E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.59247E+00_JPRB, 0.55752E+00_JPRB, 0.54900E+00_JPRB, 0.56402E+00_JPRB, 0.49624E+00_JPRB, 0.43389E+00_JPRB ,&
&  0.41533E+00_JPRB, 0.43804E+00_JPRB, 0.50327E+00_JPRB, 0.53740E+00_JPRB, 0.54747E+00_JPRB, 0.55540E+00_JPRB, 0.55347E+00_JPRB ,&
&  0.61632E+00_JPRB, 0.63687E+00_JPRB, 0.69050E+00_JPRB, 0.71908E+00_JPRB, 0.70817E+00_JPRB, 0.71301E+00_JPRB, 0.73165E+00_JPRB ,&
&  0.68408E+00_JPRB, 0.66262E+00_JPRB, 0.65656E+00_JPRB, 0.60648E+00_JPRB, 0.67202E+00_JPRB, 0.64476E+00_JPRB, 0.60774E+00_JPRB ,&
&  0.60362E+00_JPRB, 0.63087E+00_JPRB, 0.66304E+00_JPRB, 0.69754E+00_JPRB, 0.72140E+00_JPRB, 0.75307E+00_JPRB, 0.80163E+00_JPRB ,&
&  0.87616E+00_JPRB, 0.10214E+01_JPRB, 0.11860E+01_JPRB, 0.13518E+01_JPRB, 0.15686E+01_JPRB, 0.21232E+01_JPRB, 0.27895E+01_JPRB ,&
&  0.29428E+01_JPRB, 0.31107E+01_JPRB, 0.33076E+01_JPRB, 0.29780E+01_JPRB, 0.31942E+01_JPRB, 0.31298E+01_JPRB, 0.28075E+01_JPRB ,&
&  0.27393E+01_JPRB, 0.24580E+01_JPRB, 0.23952E+01_JPRB, 0.24193E+01_JPRB, 0.25519E+01_JPRB, 0.24801E+01_JPRB, 0.24216E+01_JPRB ,&
&  0.24525E+01_JPRB, 0.26495E+01_JPRB, 0.24683E+01_JPRB, 0.23739E+01_JPRB, 0.21695E+01_JPRB, 0.19943E+01_JPRB, 0.17917E+01_JPRB ,&
&  0.16570E+01_JPRB, 0.15115E+01_JPRB, 0.13700E+01_JPRB, 0.12569E+01_JPRB, 0.11076E+01_JPRB, 0.99111E+00_JPRB, 0.89641E+00_JPRB ,&
&  0.79995E+00_JPRB, 0.68925E+00_JPRB, 0.64705E+00_JPRB, 0.61793E+00_JPRB, 0.60283E+00_JPRB, 0.56568E+00_JPRB, 0.56626E+00_JPRB ,&
&  0.60120E+00_JPRB, 0.63615E+00_JPRB, 0.68101E+00_JPRB, 0.67789E+00_JPRB, 0.69852E+00_JPRB, 0.73700E+00_JPRB, 0.69405E+00_JPRB ,&
&  0.64783E+00_JPRB, 0.71390E+00_JPRB, 0.72053E+00_JPRB, 0.71998E+00_JPRB, 0.71423E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.69631E+00_JPRB, 0.69903E+00_JPRB, 0.68775E+00_JPRB, 0.68504E+00_JPRB, 0.67081E+00_JPRB, 0.61367E+00_JPRB, 0.55823E+00_JPRB ,&
&  0.61610E+00_JPRB, 0.54883E+00_JPRB, 0.57011E+00_JPRB, 0.47702E+00_JPRB, 0.44739E+00_JPRB, 0.44206E+00_JPRB, 0.42951E+00_JPRB ,&
&  0.41874E+00_JPRB, 0.44771E+00_JPRB, 0.48208E+00_JPRB, 0.49681E+00_JPRB, 0.51759E+00_JPRB, 0.52635E+00_JPRB, 0.52207E+00_JPRB ,&
&  0.53568E+00_JPRB, 0.56208E+00_JPRB, 0.61257E+00_JPRB, 0.62177E+00_JPRB, 0.60037E+00_JPRB, 0.60025E+00_JPRB, 0.60718E+00_JPRB ,&
&  0.58367E+00_JPRB, 0.57119E+00_JPRB, 0.59229E+00_JPRB, 0.59525E+00_JPRB, 0.58650E+00_JPRB, 0.54689E+00_JPRB, 0.52639E+00_JPRB ,&
&  0.53629E+00_JPRB, 0.51818E+00_JPRB, 0.57896E+00_JPRB, 0.60527E+00_JPRB, 0.63802E+00_JPRB, 0.67798E+00_JPRB, 0.72087E+00_JPRB ,&
&  0.79598E+00_JPRB, 0.92996E+00_JPRB, 0.10941E+01_JPRB, 0.12452E+01_JPRB, 0.14654E+01_JPRB, 0.19952E+01_JPRB, 0.23786E+01_JPRB ,&
&  0.27001E+01_JPRB, 0.19042E+01_JPRB, 0.23108E+01_JPRB, 0.24264E+01_JPRB, 0.26959E+01_JPRB, 0.26646E+01_JPRB, 0.25388E+01_JPRB ,&
&  0.24380E+01_JPRB, 0.20929E+01_JPRB, 0.19008E+01_JPRB, 0.17885E+01_JPRB, 0.18097E+01_JPRB, 0.19442E+01_JPRB, 0.20426E+01_JPRB ,&
&  0.21447E+01_JPRB, 0.20338E+01_JPRB, 0.18698E+01_JPRB, 0.16939E+01_JPRB, 0.15515E+01_JPRB, 0.14550E+01_JPRB, 0.13518E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.11618E+01_JPRB, 0.10702E+01_JPRB, 0.95013E+00_JPRB, 0.88989E+00_JPRB, 0.83991E+00_JPRB, 0.78494E+00_JPRB ,&
&  0.75716E+00_JPRB, 0.71039E+00_JPRB, 0.65806E+00_JPRB, 0.63572E+00_JPRB, 0.60768E+00_JPRB, 0.60112E+00_JPRB, 0.62168E+00_JPRB ,&
&  0.62656E+00_JPRB, 0.59759E+00_JPRB, 0.62838E+00_JPRB, 0.63797E+00_JPRB, 0.63683E+00_JPRB, 0.64311E+00_JPRB, 0.69400E+00_JPRB ,&
&  0.65683E+00_JPRB, 0.66723E+00_JPRB, 0.70984E+00_JPRB, 0.70084E+00_JPRB, 0.69697E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.69502E+00_JPRB, 0.68889E+00_JPRB, 0.67671E+00_JPRB, 0.65950E+00_JPRB, 0.64533E+00_JPRB, 0.61132E+00_JPRB, 0.53498E+00_JPRB ,&
&  0.53150E+00_JPRB, 0.48726E+00_JPRB, 0.45566E+00_JPRB, 0.43951E+00_JPRB, 0.43907E+00_JPRB, 0.43350E+00_JPRB, 0.42995E+00_JPRB ,&
&  0.43346E+00_JPRB, 0.45256E+00_JPRB, 0.47941E+00_JPRB, 0.48974E+00_JPRB, 0.50099E+00_JPRB, 0.50774E+00_JPRB, 0.50371E+00_JPRB ,&
&  0.51211E+00_JPRB, 0.52985E+00_JPRB, 0.55070E+00_JPRB, 0.56726E+00_JPRB, 0.55589E+00_JPRB, 0.54514E+00_JPRB, 0.53561E+00_JPRB ,&
&  0.52819E+00_JPRB, 0.52491E+00_JPRB, 0.52422E+00_JPRB, 0.53957E+00_JPRB, 0.52330E+00_JPRB, 0.50583E+00_JPRB, 0.50301E+00_JPRB ,&
&  0.41645E+00_JPRB, 0.42557E+00_JPRB, 0.51100E+00_JPRB, 0.53969E+00_JPRB, 0.56852E+00_JPRB, 0.60313E+00_JPRB, 0.65959E+00_JPRB ,&
&  0.72925E+00_JPRB, 0.82749E+00_JPRB, 0.94469E+00_JPRB, 0.10640E+01_JPRB, 0.12362E+01_JPRB, 0.15718E+01_JPRB, 0.18852E+01_JPRB ,&
&  0.21521E+01_JPRB, 0.17195E+01_JPRB, 0.16381E+01_JPRB, 0.18263E+01_JPRB, 0.22267E+01_JPRB, 0.19998E+01_JPRB, 0.18275E+01_JPRB ,&
&  0.16739E+01_JPRB, 0.15517E+01_JPRB, 0.15128E+01_JPRB, 0.16188E+01_JPRB, 0.16602E+01_JPRB, 0.17548E+01_JPRB, 0.18103E+01_JPRB ,&
&  0.16283E+01_JPRB, 0.14981E+01_JPRB, 0.13529E+01_JPRB, 0.12760E+01_JPRB, 0.12236E+01_JPRB, 0.11855E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.11177E+01_JPRB, 0.10682E+01_JPRB, 0.97252E+00_JPRB, 0.86794E+00_JPRB, 0.79375E+00_JPRB, 0.75173E+00_JPRB, 0.72546E+00_JPRB ,&
&  0.70157E+00_JPRB, 0.67893E+00_JPRB, 0.66107E+00_JPRB, 0.65019E+00_JPRB, 0.64595E+00_JPRB, 0.65191E+00_JPRB, 0.68261E+00_JPRB ,&
&  0.63967E+00_JPRB, 0.60874E+00_JPRB, 0.56999E+00_JPRB, 0.57325E+00_JPRB, 0.59882E+00_JPRB, 0.62507E+00_JPRB, 0.63678E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.66725E+00_JPRB, 0.64585E+00_JPRB, 0.66045E+00_JPRB, 0.67408E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.68965E+00_JPRB, 0.68083E+00_JPRB, 0.65940E+00_JPRB, 0.64862E+00_JPRB, 0.61941E+00_JPRB, 0.60093E+00_JPRB, 0.56751E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.53031E+00_JPRB, 0.49835E+00_JPRB, 0.46379E+00_JPRB, 0.44704E+00_JPRB, 0.44120E+00_JPRB, 0.44508E+00_JPRB ,&
&  0.45624E+00_JPRB, 0.46810E+00_JPRB, 0.47953E+00_JPRB, 0.48416E+00_JPRB, 0.49476E+00_JPRB, 0.50208E+00_JPRB, 0.50247E+00_JPRB ,&
&  0.50627E+00_JPRB, 0.51455E+00_JPRB, 0.51710E+00_JPRB, 0.51225E+00_JPRB, 0.51047E+00_JPRB, 0.51213E+00_JPRB, 0.50798E+00_JPRB ,&
&  0.49376E+00_JPRB, 0.48213E+00_JPRB, 0.48801E+00_JPRB, 0.51564E+00_JPRB, 0.49677E+00_JPRB, 0.49731E+00_JPRB, 0.44273E+00_JPRB ,&
&  0.38672E+00_JPRB, 0.37354E+00_JPRB, 0.40231E+00_JPRB, 0.49591E+00_JPRB, 0.52342E+00_JPRB, 0.55931E+00_JPRB, 0.59241E+00_JPRB ,&
&  0.60172E+00_JPRB, 0.64190E+00_JPRB, 0.74352E+00_JPRB, 0.94415E+00_JPRB, 0.10600E+01_JPRB, 0.12848E+01_JPRB, 0.14878E+01_JPRB ,&
&  0.16292E+01_JPRB, 0.16939E+01_JPRB, 0.17265E+01_JPRB, 0.14970E+01_JPRB, 0.14550E+01_JPRB, 0.16506E+01_JPRB, 0.14938E+01_JPRB ,&
&  0.14424E+01_JPRB, 0.13728E+01_JPRB, 0.14019E+01_JPRB, 0.14866E+01_JPRB, 0.14066E+01_JPRB, 0.14376E+01_JPRB, 0.14068E+01_JPRB ,&
&  0.12805E+01_JPRB, 0.11959E+01_JPRB, 0.11291E+01_JPRB, 0.10990E+01_JPRB, 0.10713E+01_JPRB, 0.10592E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.11154E+01_JPRB, 0.11142E+01_JPRB, 0.99313E+00_JPRB, 0.81035E+00_JPRB, 0.73938E+00_JPRB, 0.71055E+00_JPRB, 0.69248E+00_JPRB ,&
&  0.66849E+00_JPRB, 0.65294E+00_JPRB, 0.65174E+00_JPRB, 0.65216E+00_JPRB, 0.63693E+00_JPRB, 0.61009E+00_JPRB, 0.58339E+00_JPRB ,&
&  0.56334E+00_JPRB, 0.55234E+00_JPRB, 0.53633E+00_JPRB, 0.55825E+00_JPRB, 0.58764E+00_JPRB, 0.61231E+00_JPRB, 0.60532E+00_JPRB ,&
&  0.59771E+00_JPRB, 0.60540E+00_JPRB, 0.61657E+00_JPRB, 0.64952E+00_JPRB, 0.67069E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.59833E+00_JPRB, 0.62119E+00_JPRB, 0.64624E+00_JPRB, 0.63903E+00_JPRB, 0.63177E+00_JPRB, 0.59247E+00_JPRB, 0.55181E+00_JPRB ,&
&  0.52610E+00_JPRB, 0.51383E+00_JPRB, 0.50973E+00_JPRB, 0.50237E+00_JPRB, 0.49659E+00_JPRB, 0.49459E+00_JPRB, 0.49383E+00_JPRB ,&
&  0.49430E+00_JPRB, 0.49544E+00_JPRB, 0.49665E+00_JPRB, 0.49963E+00_JPRB, 0.50936E+00_JPRB, 0.51771E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.51904E+00_JPRB, 0.51999E+00_JPRB, 0.50986E+00_JPRB, 0.49290E+00_JPRB, 0.48475E+00_JPRB, 0.48681E+00_JPRB, 0.49426E+00_JPRB ,&
&  0.48823E+00_JPRB, 0.47553E+00_JPRB, 0.48030E+00_JPRB, 0.49531E+00_JPRB, 0.48753E+00_JPRB, 0.49042E+00_JPRB, 0.44443E+00_JPRB ,&
&  0.38643E+00_JPRB, 0.34799E+00_JPRB, 0.33376E+00_JPRB, 0.35921E+00_JPRB, 0.41635E+00_JPRB, 0.50786E+00_JPRB, 0.54844E+00_JPRB ,&
&  0.58676E+00_JPRB, 0.64077E+00_JPRB, 0.70507E+00_JPRB, 0.78873E+00_JPRB, 0.89858E+00_JPRB, 0.10763E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.13413E+01_JPRB, 0.13849E+01_JPRB, 0.13827E+01_JPRB, 0.11314E+01_JPRB, 0.10634E+01_JPRB, 0.10714E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.13326E+01_JPRB, 0.12353E+01_JPRB, 0.11464E+01_JPRB, 0.11454E+01_JPRB, 0.11144E+01_JPRB, 0.11192E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.11042E+01_JPRB, 0.10617E+01_JPRB, 0.10111E+01_JPRB, 0.97097E+00_JPRB, 0.96209E+00_JPRB, 0.97887E+00_JPRB, 0.99288E+00_JPRB ,&
&  0.10475E+01_JPRB, 0.11578E+01_JPRB, 0.10908E+01_JPRB, 0.81685E+00_JPRB, 0.76855E+00_JPRB, 0.75328E+00_JPRB, 0.73059E+00_JPRB ,&
&  0.68667E+00_JPRB, 0.65818E+00_JPRB, 0.64882E+00_JPRB, 0.64623E+00_JPRB, 0.62561E+00_JPRB, 0.61082E+00_JPRB, 0.60137E+00_JPRB ,&
&  0.59898E+00_JPRB, 0.60210E+00_JPRB, 0.60492E+00_JPRB, 0.62795E+00_JPRB, 0.63580E+00_JPRB, 0.63574E+00_JPRB, 0.62013E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.60076E+00_JPRB, 0.59530E+00_JPRB, 0.58866E+00_JPRB, 0.59086E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.61528E+00_JPRB, 0.61524E+00_JPRB, 0.62010E+00_JPRB, 0.62397E+00_JPRB, 0.62984E+00_JPRB, 0.61673E+00_JPRB, 0.60339E+00_JPRB ,&
&  0.59854E+00_JPRB, 0.59706E+00_JPRB, 0.58483E+00_JPRB, 0.57252E+00_JPRB, 0.55558E+00_JPRB, 0.54244E+00_JPRB, 0.53891E+00_JPRB ,&
&  0.53991E+00_JPRB, 0.53893E+00_JPRB, 0.53742E+00_JPRB, 0.53350E+00_JPRB, 0.53686E+00_JPRB, 0.54195E+00_JPRB, 0.53809E+00_JPRB ,&
&  0.53329E+00_JPRB, 0.52788E+00_JPRB, 0.51715E+00_JPRB, 0.50422E+00_JPRB, 0.48621E+00_JPRB, 0.47836E+00_JPRB, 0.47253E+00_JPRB ,&
&  0.46719E+00_JPRB, 0.46789E+00_JPRB, 0.48068E+00_JPRB, 0.47559E+00_JPRB, 0.47658E+00_JPRB, 0.47879E+00_JPRB, 0.42626E+00_JPRB ,&
&  0.37279E+00_JPRB, 0.33738E+00_JPRB, 0.31840E+00_JPRB, 0.31795E+00_JPRB, 0.33723E+00_JPRB, 0.38120E+00_JPRB, 0.44045E+00_JPRB ,&
&  0.51463E+00_JPRB, 0.56503E+00_JPRB, 0.61163E+00_JPRB, 0.65667E+00_JPRB, 0.70688E+00_JPRB, 0.80940E+00_JPRB, 0.89634E+00_JPRB ,&
&  0.98629E+00_JPRB, 0.10895E+01_JPRB, 0.11643E+01_JPRB, 0.12045E+01_JPRB, 0.12206E+01_JPRB, 0.11273E+01_JPRB, 0.11884E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.12346E+01_JPRB, 0.99786E+00_JPRB, 0.89791E+00_JPRB, 0.87071E+00_JPRB, 0.86980E+00_JPRB, 0.86931E+00_JPRB ,&
&  0.87597E+00_JPRB, 0.87963E+00_JPRB, 0.87446E+00_JPRB, 0.86700E+00_JPRB, 0.86729E+00_JPRB, 0.86794E+00_JPRB, 0.88574E+00_JPRB ,&
&  0.92654E+00_JPRB, 0.11221E+01_JPRB, 0.14283E+01_JPRB, 0.10573E+01_JPRB, 0.92077E+00_JPRB, 0.82690E+00_JPRB, 0.72723E+00_JPRB ,&
&  0.68370E+00_JPRB, 0.66635E+00_JPRB, 0.66497E+00_JPRB, 0.66483E+00_JPRB, 0.65676E+00_JPRB, 0.65525E+00_JPRB, 0.65725E+00_JPRB ,&
&  0.66710E+00_JPRB, 0.67447E+00_JPRB, 0.67312E+00_JPRB, 0.66924E+00_JPRB, 0.65736E+00_JPRB, 0.64020E+00_JPRB, 0.62483E+00_JPRB ,&
&  0.62362E+00_JPRB, 0.62460E+00_JPRB, 0.62537E+00_JPRB, 0.62468E+00_JPRB, 0.62134E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.61253E+00_JPRB, 0.60787E+00_JPRB, 0.60975E+00_JPRB, 0.61124E+00_JPRB, 0.61647E+00_JPRB, 0.62121E+00_JPRB, 0.62216E+00_JPRB ,&
&  0.62995E+00_JPRB, 0.62716E+00_JPRB, 0.61367E+00_JPRB, 0.59786E+00_JPRB, 0.58074E+00_JPRB, 0.56922E+00_JPRB, 0.56411E+00_JPRB ,&
&  0.56496E+00_JPRB, 0.55924E+00_JPRB, 0.55646E+00_JPRB, 0.55685E+00_JPRB, 0.56100E+00_JPRB, 0.56748E+00_JPRB, 0.56293E+00_JPRB ,&
&  0.54479E+00_JPRB, 0.53143E+00_JPRB, 0.51537E+00_JPRB, 0.49424E+00_JPRB, 0.47354E+00_JPRB, 0.45867E+00_JPRB, 0.46107E+00_JPRB ,&
&  0.46979E+00_JPRB, 0.46365E+00_JPRB, 0.46136E+00_JPRB, 0.46098E+00_JPRB, 0.46114E+00_JPRB, 0.45370E+00_JPRB, 0.37700E+00_JPRB ,&
&  0.34625E+00_JPRB, 0.32511E+00_JPRB, 0.31689E+00_JPRB, 0.31606E+00_JPRB, 0.33104E+00_JPRB, 0.36047E+00_JPRB, 0.41407E+00_JPRB ,&
&  0.46734E+00_JPRB, 0.51952E+00_JPRB, 0.55400E+00_JPRB, 0.59134E+00_JPRB, 0.64554E+00_JPRB, 0.71116E+00_JPRB, 0.76579E+00_JPRB ,&
&  0.82148E+00_JPRB, 0.89758E+00_JPRB, 0.96166E+00_JPRB, 0.10115E+01_JPRB, 0.10277E+01_JPRB, 0.10205E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.99911E+00_JPRB, 0.90407E+00_JPRB, 0.86121E+00_JPRB, 0.83179E+00_JPRB, 0.81164E+00_JPRB, 0.80587E+00_JPRB, 0.78100E+00_JPRB ,&
&  0.77009E+00_JPRB, 0.77581E+00_JPRB, 0.77612E+00_JPRB, 0.77683E+00_JPRB, 0.77914E+00_JPRB, 0.77844E+00_JPRB, 0.78057E+00_JPRB ,&
&  0.79034E+00_JPRB, 0.81100E+00_JPRB, 0.85787E+00_JPRB, 0.82339E+00_JPRB, 0.77928E+00_JPRB, 0.73345E+00_JPRB, 0.70262E+00_JPRB ,&
&  0.69059E+00_JPRB, 0.68745E+00_JPRB, 0.68904E+00_JPRB, 0.68447E+00_JPRB, 0.68207E+00_JPRB, 0.68193E+00_JPRB, 0.69165E+00_JPRB ,&
&  0.69188E+00_JPRB, 0.69118E+00_JPRB, 0.68371E+00_JPRB, 0.67160E+00_JPRB, 0.65600E+00_JPRB, 0.64021E+00_JPRB, 0.62811E+00_JPRB ,&
&  0.62199E+00_JPRB, 0.61874E+00_JPRB, 0.61547E+00_JPRB, 0.61529E+00_JPRB, 0.61510E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.60197E+00_JPRB, 0.59854E+00_JPRB, 0.60023E+00_JPRB, 0.60348E+00_JPRB, 0.60885E+00_JPRB, 0.61536E+00_JPRB, 0.61831E+00_JPRB ,&
&  0.62028E+00_JPRB, 0.61982E+00_JPRB, 0.61658E+00_JPRB, 0.60522E+00_JPRB, 0.59628E+00_JPRB, 0.59181E+00_JPRB, 0.58714E+00_JPRB ,&
&  0.58078E+00_JPRB, 0.57587E+00_JPRB, 0.57341E+00_JPRB, 0.57356E+00_JPRB, 0.57235E+00_JPRB, 0.57585E+00_JPRB, 0.57160E+00_JPRB ,&
&  0.55523E+00_JPRB, 0.54132E+00_JPRB, 0.51591E+00_JPRB, 0.49278E+00_JPRB, 0.47655E+00_JPRB, 0.46450E+00_JPRB, 0.46558E+00_JPRB ,&
&  0.46115E+00_JPRB, 0.45827E+00_JPRB, 0.45712E+00_JPRB, 0.45722E+00_JPRB, 0.45977E+00_JPRB, 0.37788E+00_JPRB, 0.35368E+00_JPRB ,&
&  0.33599E+00_JPRB, 0.32396E+00_JPRB, 0.32051E+00_JPRB, 0.32068E+00_JPRB, 0.32815E+00_JPRB, 0.35197E+00_JPRB, 0.38919E+00_JPRB ,&
&  0.41912E+00_JPRB, 0.50666E+00_JPRB, 0.52725E+00_JPRB, 0.55215E+00_JPRB, 0.59351E+00_JPRB, 0.63662E+00_JPRB, 0.67020E+00_JPRB ,&
&  0.70085E+00_JPRB, 0.74190E+00_JPRB, 0.78438E+00_JPRB, 0.82287E+00_JPRB, 0.83081E+00_JPRB, 0.84537E+00_JPRB, 0.84642E+00_JPRB ,&
&  0.84500E+00_JPRB, 0.82767E+00_JPRB, 0.81631E+00_JPRB, 0.80602E+00_JPRB, 0.79318E+00_JPRB, 0.78509E+00_JPRB, 0.77175E+00_JPRB ,&
&  0.76519E+00_JPRB, 0.75133E+00_JPRB, 0.75428E+00_JPRB, 0.75324E+00_JPRB, 0.75272E+00_JPRB, 0.75313E+00_JPRB, 0.75017E+00_JPRB ,&
&  0.74537E+00_JPRB, 0.73828E+00_JPRB, 0.72321E+00_JPRB, 0.70113E+00_JPRB, 0.68889E+00_JPRB, 0.67501E+00_JPRB, 0.67225E+00_JPRB ,&
&  0.67256E+00_JPRB, 0.68230E+00_JPRB, 0.68212E+00_JPRB, 0.68100E+00_JPRB, 0.68105E+00_JPRB, 0.67737E+00_JPRB, 0.67657E+00_JPRB ,&
&  0.67497E+00_JPRB, 0.67438E+00_JPRB, 0.66841E+00_JPRB, 0.65785E+00_JPRB, 0.64447E+00_JPRB, 0.63135E+00_JPRB, 0.62029E+00_JPRB ,&
&  0.61369E+00_JPRB, 0.61086E+00_JPRB, 0.60446E+00_JPRB, 0.60281E+00_JPRB, 0.60188E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.59142E+00_JPRB, 0.58810E+00_JPRB, 0.58708E+00_JPRB, 0.59010E+00_JPRB, 0.59827E+00_JPRB, 0.60772E+00_JPRB, 0.60782E+00_JPRB ,&
&  0.60749E+00_JPRB, 0.60614E+00_JPRB, 0.60710E+00_JPRB, 0.60565E+00_JPRB, 0.60348E+00_JPRB, 0.59897E+00_JPRB, 0.59505E+00_JPRB ,&
&  0.59164E+00_JPRB, 0.58429E+00_JPRB, 0.57539E+00_JPRB, 0.57119E+00_JPRB, 0.57047E+00_JPRB, 0.57169E+00_JPRB, 0.57123E+00_JPRB ,&
&  0.56729E+00_JPRB, 0.55602E+00_JPRB, 0.52569E+00_JPRB, 0.50294E+00_JPRB, 0.48084E+00_JPRB, 0.47305E+00_JPRB, 0.46270E+00_JPRB ,&
&  0.47396E+00_JPRB, 0.42794E+00_JPRB, 0.42267E+00_JPRB, 0.40093E+00_JPRB, 0.38226E+00_JPRB, 0.36295E+00_JPRB, 0.34781E+00_JPRB ,&
&  0.34187E+00_JPRB, 0.33633E+00_JPRB, 0.33543E+00_JPRB, 0.34180E+00_JPRB, 0.35205E+00_JPRB, 0.37322E+00_JPRB, 0.39887E+00_JPRB ,&
&  0.42330E+00_JPRB, 0.48980E+00_JPRB, 0.51001E+00_JPRB, 0.52485E+00_JPRB, 0.55077E+00_JPRB, 0.57556E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.61626E+00_JPRB, 0.63447E+00_JPRB, 0.65727E+00_JPRB, 0.66894E+00_JPRB, 0.68669E+00_JPRB, 0.69200E+00_JPRB, 0.69553E+00_JPRB ,&
&  0.70149E+00_JPRB, 0.70708E+00_JPRB, 0.70974E+00_JPRB, 0.71025E+00_JPRB, 0.70900E+00_JPRB, 0.70525E+00_JPRB, 0.70104E+00_JPRB ,&
&  0.69509E+00_JPRB, 0.68924E+00_JPRB, 0.68241E+00_JPRB, 0.67541E+00_JPRB, 0.66759E+00_JPRB, 0.66285E+00_JPRB, 0.65650E+00_JPRB ,&
&  0.65092E+00_JPRB, 0.64327E+00_JPRB, 0.63619E+00_JPRB, 0.63135E+00_JPRB, 0.63253E+00_JPRB, 0.63288E+00_JPRB, 0.63833E+00_JPRB ,&
&  0.64717E+00_JPRB, 0.65242E+00_JPRB, 0.65600E+00_JPRB, 0.65884E+00_JPRB, 0.66113E+00_JPRB, 0.65916E+00_JPRB, 0.65549E+00_JPRB ,&
&  0.65314E+00_JPRB, 0.65033E+00_JPRB, 0.64551E+00_JPRB, 0.63610E+00_JPRB, 0.62591E+00_JPRB, 0.61644E+00_JPRB, 0.60949E+00_JPRB ,&
&  0.60214E+00_JPRB, 0.59834E+00_JPRB, 0.59144E+00_JPRB, 0.59105E+00_JPRB, 0.59055E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.58029E+00_JPRB, 0.57911E+00_JPRB, 0.57767E+00_JPRB, 0.57864E+00_JPRB, 0.58137E+00_JPRB, 0.58617E+00_JPRB, 0.58745E+00_JPRB ,&
&  0.59170E+00_JPRB, 0.59561E+00_JPRB, 0.59975E+00_JPRB, 0.60096E+00_JPRB, 0.60173E+00_JPRB, 0.59995E+00_JPRB, 0.59644E+00_JPRB ,&
&  0.59296E+00_JPRB, 0.58808E+00_JPRB, 0.58456E+00_JPRB, 0.58065E+00_JPRB, 0.57633E+00_JPRB, 0.57374E+00_JPRB, 0.57107E+00_JPRB ,&
&  0.56940E+00_JPRB, 0.56688E+00_JPRB, 0.52583E+00_JPRB, 0.50962E+00_JPRB, 0.48811E+00_JPRB, 0.47592E+00_JPRB, 0.47052E+00_JPRB ,&
&  0.46414E+00_JPRB, 0.45793E+00_JPRB, 0.44956E+00_JPRB, 0.43607E+00_JPRB, 0.41818E+00_JPRB, 0.39522E+00_JPRB, 0.38191E+00_JPRB ,&
&  0.37123E+00_JPRB, 0.36811E+00_JPRB, 0.36970E+00_JPRB, 0.37706E+00_JPRB, 0.39071E+00_JPRB, 0.40733E+00_JPRB, 0.42683E+00_JPRB ,&
&  0.45361E+00_JPRB, 0.48286E+00_JPRB, 0.50249E+00_JPRB, 0.50790E+00_JPRB, 0.52888E+00_JPRB, 0.54313E+00_JPRB, 0.55490E+00_JPRB ,&
&  0.56382E+00_JPRB, 0.57109E+00_JPRB, 0.58716E+00_JPRB, 0.59750E+00_JPRB, 0.60445E+00_JPRB, 0.61062E+00_JPRB, 0.61631E+00_JPRB ,&
&  0.62132E+00_JPRB, 0.62579E+00_JPRB, 0.62756E+00_JPRB, 0.62815E+00_JPRB, 0.62674E+00_JPRB, 0.62382E+00_JPRB, 0.62075E+00_JPRB ,&
&  0.61781E+00_JPRB, 0.61493E+00_JPRB, 0.61098E+00_JPRB, 0.60653E+00_JPRB, 0.60284E+00_JPRB, 0.59917E+00_JPRB, 0.59508E+00_JPRB ,&
&  0.59296E+00_JPRB, 0.59180E+00_JPRB, 0.59148E+00_JPRB, 0.58847E+00_JPRB, 0.59093E+00_JPRB, 0.60009E+00_JPRB, 0.60471E+00_JPRB ,&
&  0.61134E+00_JPRB, 0.61633E+00_JPRB, 0.62152E+00_JPRB, 0.62566E+00_JPRB, 0.62827E+00_JPRB, 0.62862E+00_JPRB, 0.62671E+00_JPRB ,&
&  0.62542E+00_JPRB, 0.62293E+00_JPRB, 0.61931E+00_JPRB, 0.61189E+00_JPRB, 0.60461E+00_JPRB, 0.59754E+00_JPRB, 0.59253E+00_JPRB ,&
&  0.58643E+00_JPRB, 0.58263E+00_JPRB, 0.57767E+00_JPRB, 0.57820E+00_JPRB, 0.57870E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.57366E+00_JPRB, 0.57424E+00_JPRB, 0.57453E+00_JPRB, 0.57529E+00_JPRB, 0.57764E+00_JPRB, 0.58033E+00_JPRB, 0.58181E+00_JPRB ,&
&  0.58425E+00_JPRB, 0.58734E+00_JPRB, 0.59105E+00_JPRB, 0.59288E+00_JPRB, 0.59406E+00_JPRB, 0.59399E+00_JPRB, 0.59306E+00_JPRB ,&
&  0.59114E+00_JPRB, 0.58904E+00_JPRB, 0.58698E+00_JPRB, 0.58539E+00_JPRB, 0.58381E+00_JPRB, 0.58245E+00_JPRB, 0.57946E+00_JPRB ,&
&  0.57644E+00_JPRB, 0.57511E+00_JPRB, 0.57008E+00_JPRB, 0.56276E+00_JPRB, 0.55684E+00_JPRB, 0.51193E+00_JPRB, 0.49677E+00_JPRB ,&
&  0.49266E+00_JPRB, 0.48932E+00_JPRB, 0.49100E+00_JPRB, 0.49414E+00_JPRB, 0.50544E+00_JPRB, 0.49977E+00_JPRB, 0.49547E+00_JPRB ,&
&  0.46854E+00_JPRB, 0.46532E+00_JPRB, 0.46093E+00_JPRB, 0.45958E+00_JPRB, 0.46393E+00_JPRB, 0.46899E+00_JPRB, 0.48047E+00_JPRB ,&
&  0.50842E+00_JPRB, 0.51766E+00_JPRB, 0.52179E+00_JPRB, 0.52622E+00_JPRB, 0.53326E+00_JPRB, 0.53906E+00_JPRB, 0.54369E+00_JPRB ,&
&  0.54742E+00_JPRB, 0.55067E+00_JPRB, 0.55436E+00_JPRB, 0.55843E+00_JPRB, 0.56180E+00_JPRB, 0.56312E+00_JPRB, 0.56443E+00_JPRB ,&
&  0.56509E+00_JPRB, 0.56665E+00_JPRB, 0.56897E+00_JPRB, 0.57116E+00_JPRB, 0.57357E+00_JPRB, 0.57464E+00_JPRB, 0.57539E+00_JPRB ,&
&  0.57596E+00_JPRB, 0.57605E+00_JPRB, 0.57557E+00_JPRB, 0.57564E+00_JPRB, 0.57645E+00_JPRB, 0.57645E+00_JPRB, 0.57675E+00_JPRB ,&
&  0.57698E+00_JPRB, 0.57801E+00_JPRB, 0.57893E+00_JPRB, 0.58192E+00_JPRB, 0.58432E+00_JPRB, 0.58825E+00_JPRB, 0.59170E+00_JPRB ,&
&  0.59569E+00_JPRB, 0.59862E+00_JPRB, 0.60078E+00_JPRB, 0.60191E+00_JPRB, 0.60236E+00_JPRB, 0.60238E+00_JPRB, 0.60129E+00_JPRB ,&
&  0.59984E+00_JPRB, 0.59738E+00_JPRB, 0.59397E+00_JPRB, 0.58926E+00_JPRB, 0.58477E+00_JPRB, 0.58036E+00_JPRB, 0.57660E+00_JPRB ,&
&  0.57280E+00_JPRB, 0.57098E+00_JPRB, 0.57070E+00_JPRB, 0.57175E+00_JPRB, 0.57288E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.57880E+00_JPRB, 0.57965E+00_JPRB, 0.57963E+00_JPRB, 0.57962E+00_JPRB, 0.57964E+00_JPRB, 0.58059E+00_JPRB, 0.58141E+00_JPRB ,&
&  0.58225E+00_JPRB, 0.58354E+00_JPRB, 0.58568E+00_JPRB, 0.58678E+00_JPRB, 0.58759E+00_JPRB, 0.58786E+00_JPRB, 0.58819E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.58751E+00_JPRB, 0.58662E+00_JPRB, 0.58607E+00_JPRB, 0.58556E+00_JPRB, 0.58524E+00_JPRB, 0.58474E+00_JPRB ,&
&  0.58358E+00_JPRB, 0.58215E+00_JPRB, 0.58051E+00_JPRB, 0.57765E+00_JPRB, 0.57409E+00_JPRB, 0.57010E+00_JPRB, 0.56567E+00_JPRB ,&
&  0.56094E+00_JPRB, 0.55730E+00_JPRB, 0.55368E+00_JPRB, 0.55024E+00_JPRB, 0.54883E+00_JPRB, 0.54807E+00_JPRB, 0.54722E+00_JPRB ,&
&  0.54646E+00_JPRB, 0.54500E+00_JPRB, 0.54489E+00_JPRB, 0.54526E+00_JPRB, 0.54604E+00_JPRB, 0.54768E+00_JPRB, 0.54918E+00_JPRB ,&
&  0.55025E+00_JPRB, 0.55025E+00_JPRB, 0.54953E+00_JPRB, 0.55017E+00_JPRB, 0.55172E+00_JPRB, 0.55249E+00_JPRB, 0.55206E+00_JPRB ,&
&  0.55176E+00_JPRB, 0.55080E+00_JPRB, 0.54980E+00_JPRB, 0.54874E+00_JPRB, 0.54891E+00_JPRB, 0.54836E+00_JPRB, 0.54749E+00_JPRB ,&
&  0.54692E+00_JPRB, 0.54699E+00_JPRB, 0.54753E+00_JPRB, 0.54827E+00_JPRB, 0.54978E+00_JPRB, 0.55123E+00_JPRB, 0.55279E+00_JPRB ,&
&  0.55451E+00_JPRB, 0.55733E+00_JPRB, 0.55997E+00_JPRB, 0.56251E+00_JPRB, 0.56538E+00_JPRB, 0.56849E+00_JPRB, 0.57142E+00_JPRB ,&
&  0.57401E+00_JPRB, 0.57612E+00_JPRB, 0.57701E+00_JPRB, 0.57837E+00_JPRB, 0.57981E+00_JPRB, 0.58195E+00_JPRB, 0.58339E+00_JPRB ,&
&  0.58513E+00_JPRB, 0.58632E+00_JPRB, 0.58697E+00_JPRB, 0.58743E+00_JPRB, 0.58734E+00_JPRB, 0.58685E+00_JPRB, 0.58582E+00_JPRB ,&
&  0.58450E+00_JPRB, 0.58335E+00_JPRB, 0.58223E+00_JPRB, 0.58093E+00_JPRB, 0.57975E+00_JPRB, 0.57870E+00_JPRB, 0.57768E+00_JPRB ,&
&  0.57705E+00_JPRB, 0.57727E+00_JPRB, 0.57798E+00_JPRB, 0.57828E+00_JPRB, 0.57848E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.58092E+00_JPRB, 0.58069E+00_JPRB, 0.58040E+00_JPRB, 0.58001E+00_JPRB, 0.57961E+00_JPRB, 0.57951E+00_JPRB, 0.57938E+00_JPRB ,&
&  0.57933E+00_JPRB, 0.57927E+00_JPRB, 0.57966E+00_JPRB, 0.57976E+00_JPRB, 0.57994E+00_JPRB, 0.57997E+00_JPRB, 0.58016E+00_JPRB ,&
&  0.58025E+00_JPRB, 0.58036E+00_JPRB, 0.58041E+00_JPRB, 0.58064E+00_JPRB, 0.58085E+00_JPRB, 0.58124E+00_JPRB, 0.58153E+00_JPRB ,&
&  0.58161E+00_JPRB, 0.58124E+00_JPRB, 0.58106E+00_JPRB, 0.58055E+00_JPRB, 0.57996E+00_JPRB, 0.57878E+00_JPRB, 0.57743E+00_JPRB ,&
&  0.57577E+00_JPRB, 0.57434E+00_JPRB, 0.57310E+00_JPRB, 0.57207E+00_JPRB, 0.57122E+00_JPRB, 0.57071E+00_JPRB, 0.57026E+00_JPRB ,&
&  0.56962E+00_JPRB, 0.56907E+00_JPRB, 0.56862E+00_JPRB, 0.56813E+00_JPRB, 0.56729E+00_JPRB, 0.56628E+00_JPRB, 0.56527E+00_JPRB ,&
&  0.56413E+00_JPRB, 0.56282E+00_JPRB, 0.56187E+00_JPRB, 0.56120E+00_JPRB, 0.56035E+00_JPRB, 0.55895E+00_JPRB, 0.55744E+00_JPRB ,&
&  0.55599E+00_JPRB, 0.55429E+00_JPRB, 0.55247E+00_JPRB, 0.55083E+00_JPRB, 0.54976E+00_JPRB, 0.54884E+00_JPRB, 0.54801E+00_JPRB ,&
&  0.54801E+00_JPRB, 0.54888E+00_JPRB, 0.54975E+00_JPRB, 0.55046E+00_JPRB, 0.55160E+00_JPRB, 0.55279E+00_JPRB, 0.55413E+00_JPRB ,&
&  0.55545E+00_JPRB, 0.55706E+00_JPRB, 0.55897E+00_JPRB, 0.56070E+00_JPRB, 0.56268E+00_JPRB, 0.56479E+00_JPRB, 0.56715E+00_JPRB ,&
&  0.56939E+00_JPRB, 0.57180E+00_JPRB, 0.57399E+00_JPRB, 0.57611E+00_JPRB, 0.57737E+00_JPRB, 0.57851E+00_JPRB, 0.57895E+00_JPRB ,&
&  0.57946E+00_JPRB, 0.57974E+00_JPRB, 0.58029E+00_JPRB, 0.58053E+00_JPRB, 0.58080E+00_JPRB, 0.58093E+00_JPRB, 0.58111E+00_JPRB ,&
&  0.58124E+00_JPRB, 0.58151E+00_JPRB, 0.58162E+00_JPRB, 0.58175E+00_JPRB, 0.58162E+00_JPRB, 0.58174E+00_JPRB, 0.58174E+00_JPRB ,&
&  0.58173E+00_JPRB, 0.58178E+00_JPRB, 0.58187E+00_JPRB, 0.58177E+00_JPRB, 0.58142E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB, 0.57129E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB, 0.19929E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.20285E+00_JPRB, 0.20314E+00_JPRB, 0.20324E+00_JPRB, 0.20323E+00_JPRB, 0.20178E+00_JPRB, 0.20126E+00_JPRB, 0.20075E+00_JPRB ,&
&  0.20039E+00_JPRB, 0.20020E+00_JPRB, 0.20059E+00_JPRB, 0.20108E+00_JPRB, 0.20158E+00_JPRB, 0.20207E+00_JPRB, 0.20246E+00_JPRB ,&
&  0.20263E+00_JPRB, 0.20281E+00_JPRB, 0.20186E+00_JPRB, 0.20141E+00_JPRB, 0.20129E+00_JPRB, 0.20137E+00_JPRB, 0.20138E+00_JPRB ,&
&  0.20151E+00_JPRB, 0.20161E+00_JPRB, 0.20161E+00_JPRB, 0.20192E+00_JPRB, 0.20247E+00_JPRB, 0.20293E+00_JPRB, 0.20340E+00_JPRB ,&
&  0.20386E+00_JPRB, 0.20451E+00_JPRB, 0.20510E+00_JPRB, 0.20565E+00_JPRB, 0.20632E+00_JPRB, 0.20695E+00_JPRB, 0.20729E+00_JPRB ,&
&  0.20752E+00_JPRB, 0.20753E+00_JPRB, 0.20760E+00_JPRB, 0.20766E+00_JPRB, 0.20783E+00_JPRB, 0.20805E+00_JPRB, 0.20807E+00_JPRB ,&
&  0.20797E+00_JPRB, 0.20779E+00_JPRB, 0.20754E+00_JPRB, 0.20702E+00_JPRB, 0.20647E+00_JPRB, 0.20595E+00_JPRB, 0.20544E+00_JPRB ,&
&  0.20500E+00_JPRB, 0.20450E+00_JPRB, 0.20395E+00_JPRB, 0.20336E+00_JPRB, 0.20275E+00_JPRB, 0.20216E+00_JPRB, 0.20156E+00_JPRB ,&
&  0.20095E+00_JPRB, 0.20055E+00_JPRB, 0.20012E+00_JPRB, 0.19974E+00_JPRB, 0.19946E+00_JPRB, 0.19917E+00_JPRB, 0.19890E+00_JPRB ,&
&  0.19863E+00_JPRB, 0.19830E+00_JPRB, 0.19800E+00_JPRB, 0.19765E+00_JPRB, 0.19735E+00_JPRB, 0.19702E+00_JPRB, 0.19675E+00_JPRB ,&
&  0.19649E+00_JPRB, 0.19626E+00_JPRB, 0.19607E+00_JPRB, 0.19598E+00_JPRB, 0.19593E+00_JPRB, 0.19597E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19607E+00_JPRB, 0.19611E+00_JPRB, 0.19614E+00_JPRB, 0.19604E+00_JPRB, 0.19622E+00_JPRB, 0.19642E+00_JPRB, 0.19669E+00_JPRB ,&
&  0.19696E+00_JPRB, 0.19739E+00_JPRB, 0.19781E+00_JPRB, 0.19828E+00_JPRB, 0.19900E+00_JPRB, 0.19925E+00_JPRB, 0.19948E+00_JPRB ,&
&  0.19974E+00_JPRB, 0.20026E+00_JPRB, 0.20072E+00_JPRB, 0.20097E+00_JPRB, 0.20151E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.21530E+00_JPRB, 0.22034E+00_JPRB, 0.22424E+00_JPRB, 0.22802E+00_JPRB, 0.22685E+00_JPRB, 0.24631E+00_JPRB, 0.24427E+00_JPRB ,&
&  0.24393E+00_JPRB, 0.23708E+00_JPRB, 0.23792E+00_JPRB, 0.23652E+00_JPRB, 0.23413E+00_JPRB, 0.23109E+00_JPRB, 0.22785E+00_JPRB ,&
&  0.22453E+00_JPRB, 0.22061E+00_JPRB, 0.21698E+00_JPRB, 0.21351E+00_JPRB, 0.21065E+00_JPRB, 0.20817E+00_JPRB, 0.20679E+00_JPRB ,&
&  0.20680E+00_JPRB, 0.20822E+00_JPRB, 0.21066E+00_JPRB, 0.21212E+00_JPRB, 0.21366E+00_JPRB, 0.21526E+00_JPRB, 0.21695E+00_JPRB ,&
&  0.21842E+00_JPRB, 0.22032E+00_JPRB, 0.22134E+00_JPRB, 0.22103E+00_JPRB, 0.21987E+00_JPRB, 0.21925E+00_JPRB, 0.21894E+00_JPRB ,&
&  0.21924E+00_JPRB, 0.21930E+00_JPRB, 0.21925E+00_JPRB, 0.21937E+00_JPRB, 0.21946E+00_JPRB, 0.21887E+00_JPRB, 0.21766E+00_JPRB ,&
&  0.21665E+00_JPRB, 0.21583E+00_JPRB, 0.21431E+00_JPRB, 0.21285E+00_JPRB, 0.21148E+00_JPRB, 0.21006E+00_JPRB, 0.20860E+00_JPRB ,&
&  0.20754E+00_JPRB, 0.20647E+00_JPRB, 0.20514E+00_JPRB, 0.20359E+00_JPRB, 0.20213E+00_JPRB, 0.20082E+00_JPRB, 0.19978E+00_JPRB ,&
&  0.19952E+00_JPRB, 0.19896E+00_JPRB, 0.19847E+00_JPRB, 0.19798E+00_JPRB, 0.19784E+00_JPRB, 0.19745E+00_JPRB, 0.19717E+00_JPRB ,&
&  0.19697E+00_JPRB, 0.19671E+00_JPRB, 0.19645E+00_JPRB, 0.19618E+00_JPRB, 0.19601E+00_JPRB, 0.19575E+00_JPRB, 0.19552E+00_JPRB ,&
&  0.19537E+00_JPRB, 0.19526E+00_JPRB, 0.19511E+00_JPRB, 0.19479E+00_JPRB, 0.19445E+00_JPRB, 0.19403E+00_JPRB, 0.19404E+00_JPRB ,&
&  0.19419E+00_JPRB, 0.19448E+00_JPRB, 0.19497E+00_JPRB, 0.19519E+00_JPRB, 0.19598E+00_JPRB, 0.19675E+00_JPRB, 0.19765E+00_JPRB ,&
&  0.19862E+00_JPRB, 0.20008E+00_JPRB, 0.20168E+00_JPRB, 0.20340E+00_JPRB, 0.20607E+00_JPRB, 0.20711E+00_JPRB, 0.20802E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.21240E+00_JPRB, 0.21361E+00_JPRB, 0.21377E+00_JPRB, 0.21468E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.25803E+00_JPRB, 0.26113E+00_JPRB, 0.26326E+00_JPRB, 0.26468E+00_JPRB, 0.26604E+00_JPRB, 0.26538E+00_JPRB, 0.26393E+00_JPRB ,&
&  0.25990E+00_JPRB, 0.25848E+00_JPRB, 0.25578E+00_JPRB, 0.25295E+00_JPRB, 0.24965E+00_JPRB, 0.24610E+00_JPRB, 0.24209E+00_JPRB ,&
&  0.23765E+00_JPRB, 0.23286E+00_JPRB, 0.22710E+00_JPRB, 0.22091E+00_JPRB, 0.21526E+00_JPRB, 0.20998E+00_JPRB, 0.20672E+00_JPRB ,&
&  0.20519E+00_JPRB, 0.20585E+00_JPRB, 0.20852E+00_JPRB, 0.21193E+00_JPRB, 0.21608E+00_JPRB, 0.22078E+00_JPRB, 0.22665E+00_JPRB ,&
&  0.23172E+00_JPRB, 0.23785E+00_JPRB, 0.24158E+00_JPRB, 0.24322E+00_JPRB, 0.24183E+00_JPRB, 0.24168E+00_JPRB, 0.24068E+00_JPRB ,&
&  0.23933E+00_JPRB, 0.23713E+00_JPRB, 0.23607E+00_JPRB, 0.23590E+00_JPRB, 0.23480E+00_JPRB, 0.23269E+00_JPRB, 0.22993E+00_JPRB ,&
&  0.22744E+00_JPRB, 0.22530E+00_JPRB, 0.22274E+00_JPRB, 0.22056E+00_JPRB, 0.21890E+00_JPRB, 0.21692E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.21200E+00_JPRB, 0.20965E+00_JPRB, 0.20687E+00_JPRB, 0.20447E+00_JPRB, 0.20209E+00_JPRB, 0.20035E+00_JPRB, 0.19879E+00_JPRB ,&
&  0.19810E+00_JPRB, 0.19739E+00_JPRB, 0.19717E+00_JPRB, 0.19704E+00_JPRB, 0.19743E+00_JPRB, 0.19718E+00_JPRB, 0.19682E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19625E+00_JPRB, 0.19577E+00_JPRB, 0.19560E+00_JPRB, 0.19543E+00_JPRB, 0.19484E+00_JPRB, 0.19458E+00_JPRB ,&
&  0.19471E+00_JPRB, 0.19469E+00_JPRB, 0.19425E+00_JPRB, 0.19424E+00_JPRB, 0.19397E+00_JPRB, 0.19355E+00_JPRB, 0.19383E+00_JPRB ,&
&  0.19368E+00_JPRB, 0.19390E+00_JPRB, 0.19422E+00_JPRB, 0.19472E+00_JPRB, 0.19578E+00_JPRB, 0.19714E+00_JPRB, 0.19896E+00_JPRB ,&
&  0.20171E+00_JPRB, 0.20423E+00_JPRB, 0.20677E+00_JPRB, 0.20977E+00_JPRB, 0.21523E+00_JPRB, 0.21834E+00_JPRB, 0.22364E+00_JPRB ,&
&  0.22877E+00_JPRB, 0.22887E+00_JPRB, 0.25149E+00_JPRB, 0.25253E+00_JPRB, 0.25428E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.27166E+00_JPRB, 0.27368E+00_JPRB, 0.27482E+00_JPRB, 0.27685E+00_JPRB, 0.27906E+00_JPRB, 0.27989E+00_JPRB, 0.28069E+00_JPRB ,&
&  0.27807E+00_JPRB, 0.27139E+00_JPRB, 0.26648E+00_JPRB, 0.26009E+00_JPRB, 0.25369E+00_JPRB, 0.24715E+00_JPRB, 0.24113E+00_JPRB ,&
&  0.23503E+00_JPRB, 0.22954E+00_JPRB, 0.22403E+00_JPRB, 0.21942E+00_JPRB, 0.21626E+00_JPRB, 0.21366E+00_JPRB, 0.21076E+00_JPRB ,&
&  0.20825E+00_JPRB, 0.20913E+00_JPRB, 0.21151E+00_JPRB, 0.21624E+00_JPRB, 0.22371E+00_JPRB, 0.23080E+00_JPRB, 0.23851E+00_JPRB ,&
&  0.25228E+00_JPRB, 0.25633E+00_JPRB, 0.25617E+00_JPRB, 0.25798E+00_JPRB, 0.25730E+00_JPRB, 0.25780E+00_JPRB, 0.24769E+00_JPRB ,&
&  0.24928E+00_JPRB, 0.25127E+00_JPRB, 0.25843E+00_JPRB, 0.26168E+00_JPRB, 0.25681E+00_JPRB, 0.25228E+00_JPRB, 0.24432E+00_JPRB ,&
&  0.23752E+00_JPRB, 0.23225E+00_JPRB, 0.22877E+00_JPRB, 0.22664E+00_JPRB, 0.22244E+00_JPRB, 0.21934E+00_JPRB, 0.21416E+00_JPRB ,&
&  0.20977E+00_JPRB, 0.20678E+00_JPRB, 0.20304E+00_JPRB, 0.20068E+00_JPRB, 0.19878E+00_JPRB, 0.19697E+00_JPRB, 0.19560E+00_JPRB ,&
&  0.19485E+00_JPRB, 0.19424E+00_JPRB, 0.19487E+00_JPRB, 0.19562E+00_JPRB, 0.19704E+00_JPRB, 0.19818E+00_JPRB, 0.19847E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.19913E+00_JPRB, 0.19921E+00_JPRB, 0.19973E+00_JPRB, 0.19922E+00_JPRB, 0.19843E+00_JPRB, 0.19743E+00_JPRB ,&
&  0.19639E+00_JPRB, 0.19604E+00_JPRB, 0.19538E+00_JPRB, 0.19498E+00_JPRB, 0.19350E+00_JPRB, 0.19246E+00_JPRB, 0.19189E+00_JPRB ,&
&  0.19136E+00_JPRB, 0.19173E+00_JPRB, 0.19186E+00_JPRB, 0.19264E+00_JPRB, 0.19387E+00_JPRB, 0.19503E+00_JPRB, 0.19703E+00_JPRB ,&
&  0.19957E+00_JPRB, 0.20252E+00_JPRB, 0.20741E+00_JPRB, 0.21271E+00_JPRB, 0.21857E+00_JPRB, 0.22633E+00_JPRB, 0.23394E+00_JPRB ,&
&  0.24446E+00_JPRB, 0.26048E+00_JPRB, 0.26269E+00_JPRB, 0.26511E+00_JPRB, 0.26921E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.28142E+00_JPRB, 0.28251E+00_JPRB, 0.28394E+00_JPRB, 0.28628E+00_JPRB, 0.28917E+00_JPRB, 0.29031E+00_JPRB, 0.29060E+00_JPRB ,&
&  0.28150E+00_JPRB, 0.27379E+00_JPRB, 0.26701E+00_JPRB, 0.25667E+00_JPRB, 0.24847E+00_JPRB, 0.24103E+00_JPRB, 0.23565E+00_JPRB ,&
&  0.23108E+00_JPRB, 0.22861E+00_JPRB, 0.22616E+00_JPRB, 0.22539E+00_JPRB, 0.22548E+00_JPRB, 0.22627E+00_JPRB, 0.22497E+00_JPRB ,&
&  0.22335E+00_JPRB, 0.22429E+00_JPRB, 0.22492E+00_JPRB, 0.22938E+00_JPRB, 0.23584E+00_JPRB, 0.24321E+00_JPRB, 0.25040E+00_JPRB ,&
&  0.25953E+00_JPRB, 0.26355E+00_JPRB, 0.26664E+00_JPRB, 0.26739E+00_JPRB, 0.26594E+00_JPRB, 0.26517E+00_JPRB, 0.26299E+00_JPRB ,&
&  0.26234E+00_JPRB, 0.26171E+00_JPRB, 0.26274E+00_JPRB, 0.26367E+00_JPRB, 0.26237E+00_JPRB, 0.25960E+00_JPRB, 0.25377E+00_JPRB ,&
&  0.24549E+00_JPRB, 0.23873E+00_JPRB, 0.23266E+00_JPRB, 0.22797E+00_JPRB, 0.22180E+00_JPRB, 0.21634E+00_JPRB, 0.21090E+00_JPRB ,&
&  0.20612E+00_JPRB, 0.20246E+00_JPRB, 0.19819E+00_JPRB, 0.19499E+00_JPRB, 0.19352E+00_JPRB, 0.19179E+00_JPRB, 0.19104E+00_JPRB ,&
&  0.19099E+00_JPRB, 0.19134E+00_JPRB, 0.19324E+00_JPRB, 0.19486E+00_JPRB, 0.19738E+00_JPRB, 0.20029E+00_JPRB, 0.20214E+00_JPRB ,&
&  0.20524E+00_JPRB, 0.20791E+00_JPRB, 0.20934E+00_JPRB, 0.21035E+00_JPRB, 0.20941E+00_JPRB, 0.20677E+00_JPRB, 0.20400E+00_JPRB ,&
&  0.20049E+00_JPRB, 0.19797E+00_JPRB, 0.19634E+00_JPRB, 0.19458E+00_JPRB, 0.19370E+00_JPRB, 0.19295E+00_JPRB, 0.19219E+00_JPRB ,&
&  0.19162E+00_JPRB, 0.19138E+00_JPRB, 0.19111E+00_JPRB, 0.19205E+00_JPRB, 0.19308E+00_JPRB, 0.19497E+00_JPRB, 0.19727E+00_JPRB ,&
&  0.19945E+00_JPRB, 0.20276E+00_JPRB, 0.20640E+00_JPRB, 0.21134E+00_JPRB, 0.21999E+00_JPRB, 0.22744E+00_JPRB, 0.24357E+00_JPRB ,&
&  0.25715E+00_JPRB, 0.28541E+00_JPRB, 0.29667E+00_JPRB, 0.28372E+00_JPRB, 0.28192E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.29601E+00_JPRB, 0.29327E+00_JPRB, 0.29075E+00_JPRB, 0.29048E+00_JPRB, 0.29110E+00_JPRB, 0.29408E+00_JPRB, 0.29189E+00_JPRB ,&
&  0.28753E+00_JPRB, 0.28598E+00_JPRB, 0.28740E+00_JPRB, 0.28775E+00_JPRB, 0.28697E+00_JPRB, 0.27630E+00_JPRB, 0.27027E+00_JPRB ,&
&  0.26352E+00_JPRB, 0.26108E+00_JPRB, 0.25918E+00_JPRB, 0.25988E+00_JPRB, 0.26225E+00_JPRB, 0.26374E+00_JPRB, 0.25974E+00_JPRB ,&
&  0.25538E+00_JPRB, 0.25112E+00_JPRB, 0.24681E+00_JPRB, 0.24500E+00_JPRB, 0.24631E+00_JPRB, 0.24781E+00_JPRB, 0.24812E+00_JPRB ,&
&  0.24883E+00_JPRB, 0.25151E+00_JPRB, 0.25393E+00_JPRB, 0.25731E+00_JPRB, 0.27002E+00_JPRB, 0.26860E+00_JPRB, 0.26732E+00_JPRB ,&
&  0.26755E+00_JPRB, 0.26786E+00_JPRB, 0.26875E+00_JPRB, 0.26919E+00_JPRB, 0.27279E+00_JPRB, 0.27588E+00_JPRB, 0.27799E+00_JPRB ,&
&  0.26417E+00_JPRB, 0.25579E+00_JPRB, 0.24783E+00_JPRB, 0.23887E+00_JPRB, 0.22900E+00_JPRB, 0.22090E+00_JPRB, 0.21341E+00_JPRB ,&
&  0.20703E+00_JPRB, 0.20101E+00_JPRB, 0.19568E+00_JPRB, 0.19357E+00_JPRB, 0.19286E+00_JPRB, 0.19179E+00_JPRB, 0.19176E+00_JPRB ,&
&  0.19231E+00_JPRB, 0.19304E+00_JPRB, 0.19355E+00_JPRB, 0.19500E+00_JPRB, 0.19721E+00_JPRB, 0.20066E+00_JPRB, 0.20435E+00_JPRB ,&
&  0.20992E+00_JPRB, 0.21419E+00_JPRB, 0.21912E+00_JPRB, 0.22304E+00_JPRB, 0.22157E+00_JPRB, 0.21675E+00_JPRB, 0.21151E+00_JPRB ,&
&  0.20543E+00_JPRB, 0.20188E+00_JPRB, 0.19835E+00_JPRB, 0.19580E+00_JPRB, 0.19436E+00_JPRB, 0.19356E+00_JPRB, 0.19391E+00_JPRB ,&
&  0.19412E+00_JPRB, 0.19461E+00_JPRB, 0.19424E+00_JPRB, 0.19449E+00_JPRB, 0.19528E+00_JPRB, 0.19699E+00_JPRB, 0.19922E+00_JPRB ,&
&  0.20264E+00_JPRB, 0.20663E+00_JPRB, 0.21143E+00_JPRB, 0.21673E+00_JPRB, 0.22431E+00_JPRB, 0.23453E+00_JPRB, 0.24937E+00_JPRB ,&
&  0.26635E+00_JPRB, 0.29936E+00_JPRB, 0.31265E+00_JPRB, 0.30246E+00_JPRB, 0.29923E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.31300E+00_JPRB, 0.30957E+00_JPRB, 0.30772E+00_JPRB, 0.30805E+00_JPRB, 0.30826E+00_JPRB, 0.31104E+00_JPRB, 0.31059E+00_JPRB ,&
&  0.30640E+00_JPRB, 0.30181E+00_JPRB, 0.30032E+00_JPRB, 0.30004E+00_JPRB, 0.29887E+00_JPRB, 0.29766E+00_JPRB, 0.29822E+00_JPRB ,&
&  0.29668E+00_JPRB, 0.29828E+00_JPRB, 0.30088E+00_JPRB, 0.30220E+00_JPRB, 0.30295E+00_JPRB, 0.30317E+00_JPRB, 0.30329E+00_JPRB ,&
&  0.29727E+00_JPRB, 0.29571E+00_JPRB, 0.30121E+00_JPRB, 0.29864E+00_JPRB, 0.29490E+00_JPRB, 0.29169E+00_JPRB, 0.29063E+00_JPRB ,&
&  0.28266E+00_JPRB, 0.27149E+00_JPRB, 0.25857E+00_JPRB, 0.26326E+00_JPRB, 0.28017E+00_JPRB, 0.27840E+00_JPRB, 0.27619E+00_JPRB ,&
&  0.27589E+00_JPRB, 0.27525E+00_JPRB, 0.27606E+00_JPRB, 0.27651E+00_JPRB, 0.27726E+00_JPRB, 0.27722E+00_JPRB, 0.27814E+00_JPRB ,&
&  0.27939E+00_JPRB, 0.28471E+00_JPRB, 0.28848E+00_JPRB, 0.27027E+00_JPRB, 0.25406E+00_JPRB, 0.24166E+00_JPRB, 0.22992E+00_JPRB ,&
&  0.21949E+00_JPRB, 0.21641E+00_JPRB, 0.21305E+00_JPRB, 0.20757E+00_JPRB, 0.20815E+00_JPRB, 0.20868E+00_JPRB, 0.20799E+00_JPRB ,&
&  0.20624E+00_JPRB, 0.20517E+00_JPRB, 0.20376E+00_JPRB, 0.20146E+00_JPRB, 0.20055E+00_JPRB, 0.20168E+00_JPRB, 0.20489E+00_JPRB ,&
&  0.21007E+00_JPRB, 0.21615E+00_JPRB, 0.22990E+00_JPRB, 0.24184E+00_JPRB, 0.23884E+00_JPRB, 0.23082E+00_JPRB, 0.22335E+00_JPRB ,&
&  0.21361E+00_JPRB, 0.20870E+00_JPRB, 0.20391E+00_JPRB, 0.20007E+00_JPRB, 0.19834E+00_JPRB, 0.19865E+00_JPRB, 0.19957E+00_JPRB ,&
&  0.20083E+00_JPRB, 0.20271E+00_JPRB, 0.20346E+00_JPRB, 0.20303E+00_JPRB, 0.20225E+00_JPRB, 0.20264E+00_JPRB, 0.20408E+00_JPRB ,&
&  0.20682E+00_JPRB, 0.21164E+00_JPRB, 0.21757E+00_JPRB, 0.22485E+00_JPRB, 0.23110E+00_JPRB, 0.23690E+00_JPRB, 0.24576E+00_JPRB ,&
&  0.25354E+00_JPRB, 0.27096E+00_JPRB, 0.29662E+00_JPRB, 0.32363E+00_JPRB, 0.31970E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.31562E+00_JPRB, 0.31254E+00_JPRB, 0.31280E+00_JPRB, 0.31324E+00_JPRB, 0.31225E+00_JPRB, 0.31232E+00_JPRB, 0.31182E+00_JPRB ,&
&  0.31178E+00_JPRB, 0.30999E+00_JPRB, 0.30801E+00_JPRB, 0.30822E+00_JPRB, 0.31011E+00_JPRB, 0.31227E+00_JPRB, 0.31270E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.31345E+00_JPRB, 0.31656E+00_JPRB, 0.32002E+00_JPRB, 0.32081E+00_JPRB, 0.32113E+00_JPRB, 0.32291E+00_JPRB ,&
&  0.32522E+00_JPRB, 0.32603E+00_JPRB, 0.32869E+00_JPRB, 0.32657E+00_JPRB, 0.32224E+00_JPRB, 0.32040E+00_JPRB, 0.31878E+00_JPRB ,&
&  0.31289E+00_JPRB, 0.29014E+00_JPRB, 0.27496E+00_JPRB, 0.27869E+00_JPRB, 0.29484E+00_JPRB, 0.29473E+00_JPRB, 0.29275E+00_JPRB ,&
&  0.29073E+00_JPRB, 0.28808E+00_JPRB, 0.28577E+00_JPRB, 0.28489E+00_JPRB, 0.28484E+00_JPRB, 0.28458E+00_JPRB, 0.28293E+00_JPRB ,&
&  0.28305E+00_JPRB, 0.28720E+00_JPRB, 0.29315E+00_JPRB, 0.29145E+00_JPRB, 0.28813E+00_JPRB, 0.28191E+00_JPRB, 0.27793E+00_JPRB ,&
&  0.27414E+00_JPRB, 0.26995E+00_JPRB, 0.26755E+00_JPRB, 0.26672E+00_JPRB, 0.26641E+00_JPRB, 0.26454E+00_JPRB, 0.26137E+00_JPRB ,&
&  0.25349E+00_JPRB, 0.24470E+00_JPRB, 0.23537E+00_JPRB, 0.22622E+00_JPRB, 0.21922E+00_JPRB, 0.21442E+00_JPRB, 0.21086E+00_JPRB ,&
&  0.21220E+00_JPRB, 0.22239E+00_JPRB, 0.24004E+00_JPRB, 0.25944E+00_JPRB, 0.28076E+00_JPRB, 0.25805E+00_JPRB, 0.24695E+00_JPRB ,&
&  0.23600E+00_JPRB, 0.22665E+00_JPRB, 0.22024E+00_JPRB, 0.21590E+00_JPRB, 0.21283E+00_JPRB, 0.21116E+00_JPRB, 0.21118E+00_JPRB ,&
&  0.21392E+00_JPRB, 0.21773E+00_JPRB, 0.22114E+00_JPRB, 0.22178E+00_JPRB, 0.22090E+00_JPRB, 0.21911E+00_JPRB, 0.21811E+00_JPRB ,&
&  0.21905E+00_JPRB, 0.22394E+00_JPRB, 0.23238E+00_JPRB, 0.24450E+00_JPRB, 0.25570E+00_JPRB, 0.26741E+00_JPRB, 0.28135E+00_JPRB ,&
&  0.29650E+00_JPRB, 0.31143E+00_JPRB, 0.31515E+00_JPRB, 0.31821E+00_JPRB, 0.31827E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.32470E+00_JPRB, 0.32260E+00_JPRB, 0.32290E+00_JPRB, 0.32398E+00_JPRB, 0.32386E+00_JPRB, 0.32343E+00_JPRB, 0.32112E+00_JPRB ,&
&  0.31904E+00_JPRB, 0.31948E+00_JPRB, 0.32082E+00_JPRB, 0.32322E+00_JPRB, 0.32743E+00_JPRB, 0.33186E+00_JPRB, 0.33114E+00_JPRB ,&
&  0.32795E+00_JPRB, 0.32645E+00_JPRB, 0.32845E+00_JPRB, 0.33062E+00_JPRB, 0.33108E+00_JPRB, 0.33171E+00_JPRB, 0.33523E+00_JPRB ,&
&  0.33973E+00_JPRB, 0.34172E+00_JPRB, 0.34456E+00_JPRB, 0.34414E+00_JPRB, 0.34058E+00_JPRB, 0.34078E+00_JPRB, 0.33902E+00_JPRB ,&
&  0.33418E+00_JPRB, 0.32849E+00_JPRB, 0.31960E+00_JPRB, 0.32062E+00_JPRB, 0.31701E+00_JPRB, 0.31411E+00_JPRB, 0.30987E+00_JPRB ,&
&  0.30527E+00_JPRB, 0.30158E+00_JPRB, 0.29742E+00_JPRB, 0.29401E+00_JPRB, 0.29222E+00_JPRB, 0.29214E+00_JPRB, 0.28997E+00_JPRB ,&
&  0.28849E+00_JPRB, 0.28976E+00_JPRB, 0.29122E+00_JPRB, 0.28921E+00_JPRB, 0.28977E+00_JPRB, 0.29647E+00_JPRB, 0.30161E+00_JPRB ,&
&  0.30341E+00_JPRB, 0.30328E+00_JPRB, 0.30077E+00_JPRB, 0.30010E+00_JPRB, 0.29900E+00_JPRB, 0.29699E+00_JPRB, 0.29369E+00_JPRB ,&
&  0.28887E+00_JPRB, 0.28650E+00_JPRB, 0.28488E+00_JPRB, 0.28160E+00_JPRB, 0.27671E+00_JPRB, 0.25957E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.24954E+00_JPRB, 0.25827E+00_JPRB, 0.27344E+00_JPRB, 0.27961E+00_JPRB, 0.28456E+00_JPRB, 0.28936E+00_JPRB, 0.29032E+00_JPRB ,&
&  0.29234E+00_JPRB, 0.27190E+00_JPRB, 0.26084E+00_JPRB, 0.25248E+00_JPRB, 0.24491E+00_JPRB, 0.23986E+00_JPRB, 0.23847E+00_JPRB ,&
&  0.24111E+00_JPRB, 0.24701E+00_JPRB, 0.25190E+00_JPRB, 0.25416E+00_JPRB, 0.25373E+00_JPRB, 0.25041E+00_JPRB, 0.24713E+00_JPRB ,&
&  0.24652E+00_JPRB, 0.25260E+00_JPRB, 0.26493E+00_JPRB, 0.27799E+00_JPRB, 0.28668E+00_JPRB, 0.29590E+00_JPRB, 0.30375E+00_JPRB ,&
&  0.30915E+00_JPRB, 0.31421E+00_JPRB, 0.32054E+00_JPRB, 0.32295E+00_JPRB, 0.32417E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.32545E+00_JPRB, 0.32554E+00_JPRB, 0.32887E+00_JPRB, 0.33249E+00_JPRB, 0.33349E+00_JPRB, 0.33351E+00_JPRB, 0.33125E+00_JPRB ,&
&  0.32973E+00_JPRB, 0.33091E+00_JPRB, 0.33268E+00_JPRB, 0.33390E+00_JPRB, 0.33582E+00_JPRB, 0.33833E+00_JPRB, 0.34026E+00_JPRB ,&
&  0.34025E+00_JPRB, 0.33848E+00_JPRB, 0.33542E+00_JPRB, 0.33617E+00_JPRB, 0.33616E+00_JPRB, 0.33806E+00_JPRB, 0.34528E+00_JPRB ,&
&  0.35045E+00_JPRB, 0.35403E+00_JPRB, 0.35706E+00_JPRB, 0.35748E+00_JPRB, 0.35703E+00_JPRB, 0.35855E+00_JPRB, 0.35535E+00_JPRB ,&
&  0.34930E+00_JPRB, 0.34367E+00_JPRB, 0.33881E+00_JPRB, 0.33180E+00_JPRB, 0.33425E+00_JPRB, 0.33167E+00_JPRB, 0.32857E+00_JPRB ,&
&  0.32453E+00_JPRB, 0.31954E+00_JPRB, 0.31381E+00_JPRB, 0.30848E+00_JPRB, 0.30459E+00_JPRB, 0.30103E+00_JPRB, 0.29616E+00_JPRB ,&
&  0.29455E+00_JPRB, 0.29634E+00_JPRB, 0.30208E+00_JPRB, 0.30514E+00_JPRB, 0.30322E+00_JPRB, 0.30446E+00_JPRB, 0.31116E+00_JPRB ,&
&  0.31780E+00_JPRB, 0.32081E+00_JPRB, 0.32062E+00_JPRB, 0.32113E+00_JPRB, 0.32170E+00_JPRB, 0.32300E+00_JPRB, 0.32360E+00_JPRB ,&
&  0.32257E+00_JPRB, 0.32098E+00_JPRB, 0.31848E+00_JPRB, 0.31248E+00_JPRB, 0.30534E+00_JPRB, 0.29945E+00_JPRB, 0.29705E+00_JPRB ,&
&  0.29958E+00_JPRB, 0.29926E+00_JPRB, 0.29909E+00_JPRB, 0.30046E+00_JPRB, 0.30094E+00_JPRB, 0.30015E+00_JPRB, 0.29914E+00_JPRB ,&
&  0.30031E+00_JPRB, 0.29969E+00_JPRB, 0.30137E+00_JPRB, 0.29861E+00_JPRB, 0.29409E+00_JPRB, 0.28973E+00_JPRB, 0.28562E+00_JPRB ,&
&  0.28315E+00_JPRB, 0.28219E+00_JPRB, 0.28409E+00_JPRB, 0.28468E+00_JPRB, 0.28644E+00_JPRB, 0.28853E+00_JPRB, 0.29029E+00_JPRB ,&
&  0.29126E+00_JPRB, 0.29245E+00_JPRB, 0.29475E+00_JPRB, 0.29959E+00_JPRB, 0.30473E+00_JPRB, 0.30922E+00_JPRB, 0.31392E+00_JPRB ,&
&  0.31835E+00_JPRB, 0.32157E+00_JPRB, 0.32742E+00_JPRB, 0.32880E+00_JPRB, 0.32829E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.33119E+00_JPRB, 0.33344E+00_JPRB, 0.33568E+00_JPRB, 0.33767E+00_JPRB, 0.33897E+00_JPRB, 0.33999E+00_JPRB, 0.33920E+00_JPRB ,&
&  0.33903E+00_JPRB, 0.34059E+00_JPRB, 0.34266E+00_JPRB, 0.34433E+00_JPRB, 0.34526E+00_JPRB, 0.34705E+00_JPRB, 0.34875E+00_JPRB ,&
&  0.34984E+00_JPRB, 0.34894E+00_JPRB, 0.34582E+00_JPRB, 0.34456E+00_JPRB, 0.34348E+00_JPRB, 0.34830E+00_JPRB, 0.35662E+00_JPRB ,&
&  0.36031E+00_JPRB, 0.36265E+00_JPRB, 0.36553E+00_JPRB, 0.36621E+00_JPRB, 0.36663E+00_JPRB, 0.36646E+00_JPRB, 0.36206E+00_JPRB ,&
&  0.35723E+00_JPRB, 0.35275E+00_JPRB, 0.35008E+00_JPRB, 0.34680E+00_JPRB, 0.34647E+00_JPRB, 0.34394E+00_JPRB, 0.34205E+00_JPRB ,&
&  0.34071E+00_JPRB, 0.33830E+00_JPRB, 0.33547E+00_JPRB, 0.33070E+00_JPRB, 0.32472E+00_JPRB, 0.31699E+00_JPRB, 0.31176E+00_JPRB ,&
&  0.30978E+00_JPRB, 0.31332E+00_JPRB, 0.32165E+00_JPRB, 0.32583E+00_JPRB, 0.32323E+00_JPRB, 0.32299E+00_JPRB, 0.32642E+00_JPRB ,&
&  0.33114E+00_JPRB, 0.33622E+00_JPRB, 0.33784E+00_JPRB, 0.33690E+00_JPRB, 0.33499E+00_JPRB, 0.33650E+00_JPRB, 0.34055E+00_JPRB ,&
&  0.34415E+00_JPRB, 0.34612E+00_JPRB, 0.34547E+00_JPRB, 0.34460E+00_JPRB, 0.34200E+00_JPRB, 0.33606E+00_JPRB, 0.32807E+00_JPRB ,&
&  0.32083E+00_JPRB, 0.31636E+00_JPRB, 0.31299E+00_JPRB, 0.30976E+00_JPRB, 0.30864E+00_JPRB, 0.30913E+00_JPRB, 0.31137E+00_JPRB ,&
&  0.31230E+00_JPRB, 0.31418E+00_JPRB, 0.31431E+00_JPRB, 0.31017E+00_JPRB, 0.30684E+00_JPRB, 0.30806E+00_JPRB, 0.30954E+00_JPRB ,&
&  0.31203E+00_JPRB, 0.31441E+00_JPRB, 0.31686E+00_JPRB, 0.31794E+00_JPRB, 0.31951E+00_JPRB, 0.32204E+00_JPRB, 0.32348E+00_JPRB ,&
&  0.32418E+00_JPRB, 0.32475E+00_JPRB, 0.32651E+00_JPRB, 0.32937E+00_JPRB, 0.33186E+00_JPRB, 0.33378E+00_JPRB, 0.33306E+00_JPRB ,&
&  0.33197E+00_JPRB, 0.33088E+00_JPRB, 0.33159E+00_JPRB, 0.33215E+00_JPRB, 0.33094E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.35391E+00_JPRB, 0.35420E+00_JPRB, 0.35374E+00_JPRB, 0.35279E+00_JPRB, 0.35179E+00_JPRB, 0.35095E+00_JPRB, 0.35102E+00_JPRB ,&
&  0.35119E+00_JPRB, 0.35179E+00_JPRB, 0.35257E+00_JPRB, 0.35322E+00_JPRB, 0.35477E+00_JPRB, 0.35656E+00_JPRB, 0.35794E+00_JPRB ,&
&  0.35945E+00_JPRB, 0.35873E+00_JPRB, 0.35636E+00_JPRB, 0.35404E+00_JPRB, 0.35477E+00_JPRB, 0.36005E+00_JPRB, 0.36597E+00_JPRB ,&
&  0.36760E+00_JPRB, 0.36882E+00_JPRB, 0.36942E+00_JPRB, 0.36844E+00_JPRB, 0.36725E+00_JPRB, 0.36695E+00_JPRB, 0.36338E+00_JPRB ,&
&  0.35899E+00_JPRB, 0.35764E+00_JPRB, 0.35731E+00_JPRB, 0.35461E+00_JPRB, 0.35398E+00_JPRB, 0.35588E+00_JPRB, 0.35451E+00_JPRB ,&
&  0.35338E+00_JPRB, 0.35162E+00_JPRB, 0.34870E+00_JPRB, 0.34635E+00_JPRB, 0.34367E+00_JPRB, 0.33823E+00_JPRB, 0.33190E+00_JPRB ,&
&  0.32684E+00_JPRB, 0.32719E+00_JPRB, 0.33261E+00_JPRB, 0.33858E+00_JPRB, 0.33829E+00_JPRB, 0.33783E+00_JPRB, 0.33955E+00_JPRB ,&
&  0.34307E+00_JPRB, 0.34732E+00_JPRB, 0.34992E+00_JPRB, 0.34894E+00_JPRB, 0.34767E+00_JPRB, 0.34938E+00_JPRB, 0.35187E+00_JPRB ,&
&  0.35308E+00_JPRB, 0.35451E+00_JPRB, 0.35552E+00_JPRB, 0.35690E+00_JPRB, 0.35721E+00_JPRB, 0.35529E+00_JPRB, 0.35375E+00_JPRB ,&
&  0.35062E+00_JPRB, 0.34655E+00_JPRB, 0.34165E+00_JPRB, 0.33624E+00_JPRB, 0.33254E+00_JPRB, 0.33052E+00_JPRB, 0.33089E+00_JPRB ,&
&  0.33051E+00_JPRB, 0.33213E+00_JPRB, 0.33254E+00_JPRB, 0.33027E+00_JPRB, 0.32723E+00_JPRB, 0.32718E+00_JPRB, 0.33061E+00_JPRB ,&
&  0.33530E+00_JPRB, 0.33849E+00_JPRB, 0.34396E+00_JPRB, 0.34678E+00_JPRB, 0.34881E+00_JPRB, 0.35121E+00_JPRB, 0.35181E+00_JPRB ,&
&  0.34921E+00_JPRB, 0.34712E+00_JPRB, 0.34670E+00_JPRB, 0.34942E+00_JPRB, 0.35028E+00_JPRB, 0.34987E+00_JPRB, 0.34808E+00_JPRB ,&
&  0.34797E+00_JPRB, 0.35023E+00_JPRB, 0.35129E+00_JPRB, 0.35144E+00_JPRB, 0.35238E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.38231E+00_JPRB, 0.38091E+00_JPRB, 0.37946E+00_JPRB, 0.37707E+00_JPRB, 0.37444E+00_JPRB, 0.37197E+00_JPRB, 0.36967E+00_JPRB ,&
&  0.36805E+00_JPRB, 0.36758E+00_JPRB, 0.36794E+00_JPRB, 0.36793E+00_JPRB, 0.36750E+00_JPRB, 0.36805E+00_JPRB, 0.36817E+00_JPRB ,&
&  0.36949E+00_JPRB, 0.36963E+00_JPRB, 0.36756E+00_JPRB, 0.36731E+00_JPRB, 0.36990E+00_JPRB, 0.37474E+00_JPRB, 0.37681E+00_JPRB ,&
&  0.37481E+00_JPRB, 0.37484E+00_JPRB, 0.37489E+00_JPRB, 0.37308E+00_JPRB, 0.37189E+00_JPRB, 0.37108E+00_JPRB, 0.36724E+00_JPRB ,&
&  0.36261E+00_JPRB, 0.36274E+00_JPRB, 0.36461E+00_JPRB, 0.36739E+00_JPRB, 0.36670E+00_JPRB, 0.36433E+00_JPRB, 0.36446E+00_JPRB ,&
&  0.36250E+00_JPRB, 0.36211E+00_JPRB, 0.36182E+00_JPRB, 0.36255E+00_JPRB, 0.36125E+00_JPRB, 0.35933E+00_JPRB, 0.35557E+00_JPRB ,&
&  0.34731E+00_JPRB, 0.34507E+00_JPRB, 0.34776E+00_JPRB, 0.35260E+00_JPRB, 0.35255E+00_JPRB, 0.35357E+00_JPRB, 0.35667E+00_JPRB ,&
&  0.36041E+00_JPRB, 0.36301E+00_JPRB, 0.36447E+00_JPRB, 0.36540E+00_JPRB, 0.36679E+00_JPRB, 0.36816E+00_JPRB, 0.36889E+00_JPRB ,&
&  0.36996E+00_JPRB, 0.37186E+00_JPRB, 0.37407E+00_JPRB, 0.37626E+00_JPRB, 0.37708E+00_JPRB, 0.37738E+00_JPRB, 0.37697E+00_JPRB ,&
&  0.37534E+00_JPRB, 0.37170E+00_JPRB, 0.36768E+00_JPRB, 0.36222E+00_JPRB, 0.35693E+00_JPRB, 0.35412E+00_JPRB, 0.35284E+00_JPRB ,&
&  0.34965E+00_JPRB, 0.34693E+00_JPRB, 0.34496E+00_JPRB, 0.34466E+00_JPRB, 0.34498E+00_JPRB, 0.34626E+00_JPRB, 0.34853E+00_JPRB ,&
&  0.35041E+00_JPRB, 0.35325E+00_JPRB, 0.35949E+00_JPRB, 0.36386E+00_JPRB, 0.36610E+00_JPRB, 0.36916E+00_JPRB, 0.37059E+00_JPRB ,&
&  0.36895E+00_JPRB, 0.36742E+00_JPRB, 0.36631E+00_JPRB, 0.36835E+00_JPRB, 0.37127E+00_JPRB, 0.37356E+00_JPRB, 0.37547E+00_JPRB ,&
&  0.37903E+00_JPRB, 0.38257E+00_JPRB, 0.38355E+00_JPRB, 0.38365E+00_JPRB, 0.38362E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.40878E+00_JPRB, 0.40470E+00_JPRB, 0.40029E+00_JPRB, 0.39625E+00_JPRB, 0.39261E+00_JPRB, 0.38883E+00_JPRB, 0.38539E+00_JPRB ,&
&  0.38200E+00_JPRB, 0.37980E+00_JPRB, 0.37811E+00_JPRB, 0.37664E+00_JPRB, 0.37323E+00_JPRB, 0.37162E+00_JPRB, 0.37219E+00_JPRB ,&
&  0.37442E+00_JPRB, 0.37665E+00_JPRB, 0.37852E+00_JPRB, 0.37825E+00_JPRB, 0.38067E+00_JPRB, 0.38303E+00_JPRB, 0.38342E+00_JPRB ,&
&  0.38290E+00_JPRB, 0.38241E+00_JPRB, 0.38067E+00_JPRB, 0.37830E+00_JPRB, 0.37851E+00_JPRB, 0.37901E+00_JPRB, 0.37548E+00_JPRB ,&
&  0.36899E+00_JPRB, 0.36994E+00_JPRB, 0.36256E+00_JPRB, 0.37476E+00_JPRB, 0.37724E+00_JPRB, 0.37231E+00_JPRB, 0.36638E+00_JPRB ,&
&  0.36456E+00_JPRB, 0.36556E+00_JPRB, 0.36883E+00_JPRB, 0.37138E+00_JPRB, 0.37272E+00_JPRB, 0.37347E+00_JPRB, 0.37398E+00_JPRB ,&
&  0.36776E+00_JPRB, 0.36462E+00_JPRB, 0.36515E+00_JPRB, 0.36649E+00_JPRB, 0.36678E+00_JPRB, 0.36753E+00_JPRB, 0.36930E+00_JPRB ,&
&  0.37053E+00_JPRB, 0.37123E+00_JPRB, 0.37205E+00_JPRB, 0.37433E+00_JPRB, 0.37590E+00_JPRB, 0.37657E+00_JPRB, 0.37817E+00_JPRB ,&
&  0.38081E+00_JPRB, 0.38412E+00_JPRB, 0.38680E+00_JPRB, 0.38852E+00_JPRB, 0.39187E+00_JPRB, 0.39591E+00_JPRB, 0.39868E+00_JPRB ,&
&  0.39889E+00_JPRB, 0.39715E+00_JPRB, 0.39450E+00_JPRB, 0.38953E+00_JPRB, 0.38288E+00_JPRB, 0.37777E+00_JPRB, 0.37394E+00_JPRB ,&
&  0.36817E+00_JPRB, 0.36561E+00_JPRB, 0.36434E+00_JPRB, 0.36227E+00_JPRB, 0.36053E+00_JPRB, 0.35996E+00_JPRB, 0.36032E+00_JPRB ,&
&  0.36221E+00_JPRB, 0.36575E+00_JPRB, 0.36815E+00_JPRB, 0.37115E+00_JPRB, 0.37514E+00_JPRB, 0.37912E+00_JPRB, 0.37954E+00_JPRB ,&
&  0.37839E+00_JPRB, 0.37803E+00_JPRB, 0.38004E+00_JPRB, 0.38656E+00_JPRB, 0.39491E+00_JPRB, 0.40239E+00_JPRB, 0.40935E+00_JPRB ,&
&  0.41457E+00_JPRB, 0.41710E+00_JPRB, 0.41656E+00_JPRB, 0.41549E+00_JPRB, 0.41265E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.44461E+00_JPRB, 0.43618E+00_JPRB, 0.42729E+00_JPRB, 0.42041E+00_JPRB, 0.41538E+00_JPRB, 0.41028E+00_JPRB, 0.40433E+00_JPRB ,&
&  0.39978E+00_JPRB, 0.39401E+00_JPRB, 0.38949E+00_JPRB, 0.38523E+00_JPRB, 0.38168E+00_JPRB, 0.38039E+00_JPRB, 0.38106E+00_JPRB ,&
&  0.38343E+00_JPRB, 0.38863E+00_JPRB, 0.39170E+00_JPRB, 0.39054E+00_JPRB, 0.39242E+00_JPRB, 0.39470E+00_JPRB, 0.39532E+00_JPRB ,&
&  0.39330E+00_JPRB, 0.39047E+00_JPRB, 0.38734E+00_JPRB, 0.38557E+00_JPRB, 0.38570E+00_JPRB, 0.38687E+00_JPRB, 0.38400E+00_JPRB ,&
&  0.37932E+00_JPRB, 0.37131E+00_JPRB, 0.38268E+00_JPRB, 0.38842E+00_JPRB, 0.38853E+00_JPRB, 0.38501E+00_JPRB, 0.38066E+00_JPRB ,&
&  0.37466E+00_JPRB, 0.37156E+00_JPRB, 0.37319E+00_JPRB, 0.37667E+00_JPRB, 0.38027E+00_JPRB, 0.38258E+00_JPRB, 0.38488E+00_JPRB ,&
&  0.38262E+00_JPRB, 0.38151E+00_JPRB, 0.37992E+00_JPRB, 0.37881E+00_JPRB, 0.37919E+00_JPRB, 0.37978E+00_JPRB, 0.37997E+00_JPRB ,&
&  0.37863E+00_JPRB, 0.37893E+00_JPRB, 0.37951E+00_JPRB, 0.37851E+00_JPRB, 0.37977E+00_JPRB, 0.38180E+00_JPRB, 0.38463E+00_JPRB ,&
&  0.38840E+00_JPRB, 0.39353E+00_JPRB, 0.39906E+00_JPRB, 0.40615E+00_JPRB, 0.41538E+00_JPRB, 0.42381E+00_JPRB, 0.42980E+00_JPRB ,&
&  0.43213E+00_JPRB, 0.43053E+00_JPRB, 0.42594E+00_JPRB, 0.41828E+00_JPRB, 0.41132E+00_JPRB, 0.40565E+00_JPRB, 0.39850E+00_JPRB ,&
&  0.38960E+00_JPRB, 0.38408E+00_JPRB, 0.38082E+00_JPRB, 0.37924E+00_JPRB, 0.37739E+00_JPRB, 0.37656E+00_JPRB, 0.37755E+00_JPRB ,&
&  0.38021E+00_JPRB, 0.38292E+00_JPRB, 0.38442E+00_JPRB, 0.38534E+00_JPRB, 0.38681E+00_JPRB, 0.38622E+00_JPRB, 0.38615E+00_JPRB ,&
&  0.38870E+00_JPRB, 0.39442E+00_JPRB, 0.40497E+00_JPRB, 0.42382E+00_JPRB, 0.43912E+00_JPRB, 0.45038E+00_JPRB, 0.45773E+00_JPRB ,&
&  0.46134E+00_JPRB, 0.46261E+00_JPRB, 0.46053E+00_JPRB, 0.45722E+00_JPRB, 0.45220E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.47182E+00_JPRB, 0.45972E+00_JPRB, 0.44713E+00_JPRB, 0.43856E+00_JPRB, 0.43231E+00_JPRB, 0.42621E+00_JPRB, 0.41946E+00_JPRB ,&
&  0.41208E+00_JPRB, 0.40533E+00_JPRB, 0.39906E+00_JPRB, 0.39289E+00_JPRB, 0.38973E+00_JPRB, 0.38817E+00_JPRB, 0.38870E+00_JPRB ,&
&  0.39132E+00_JPRB, 0.39598E+00_JPRB, 0.39825E+00_JPRB, 0.39824E+00_JPRB, 0.39897E+00_JPRB, 0.39879E+00_JPRB, 0.39721E+00_JPRB ,&
&  0.39330E+00_JPRB, 0.38854E+00_JPRB, 0.38606E+00_JPRB, 0.38702E+00_JPRB, 0.38739E+00_JPRB, 0.38815E+00_JPRB, 0.38807E+00_JPRB ,&
&  0.39151E+00_JPRB, 0.35959E+00_JPRB, 0.38561E+00_JPRB, 0.40335E+00_JPRB, 0.39903E+00_JPRB, 0.39372E+00_JPRB, 0.38800E+00_JPRB ,&
&  0.38191E+00_JPRB, 0.37999E+00_JPRB, 0.38067E+00_JPRB, 0.38426E+00_JPRB, 0.38900E+00_JPRB, 0.39156E+00_JPRB, 0.39181E+00_JPRB ,&
&  0.38878E+00_JPRB, 0.38822E+00_JPRB, 0.38674E+00_JPRB, 0.38579E+00_JPRB, 0.38562E+00_JPRB, 0.38570E+00_JPRB, 0.38503E+00_JPRB ,&
&  0.38312E+00_JPRB, 0.37976E+00_JPRB, 0.37637E+00_JPRB, 0.37584E+00_JPRB, 0.37710E+00_JPRB, 0.38061E+00_JPRB, 0.38706E+00_JPRB ,&
&  0.39621E+00_JPRB, 0.40674E+00_JPRB, 0.41780E+00_JPRB, 0.42947E+00_JPRB, 0.44311E+00_JPRB, 0.45546E+00_JPRB, 0.46468E+00_JPRB ,&
&  0.46537E+00_JPRB, 0.46081E+00_JPRB, 0.45149E+00_JPRB, 0.44002E+00_JPRB, 0.43085E+00_JPRB, 0.42178E+00_JPRB, 0.41315E+00_JPRB ,&
&  0.40669E+00_JPRB, 0.40108E+00_JPRB, 0.39766E+00_JPRB, 0.39704E+00_JPRB, 0.39713E+00_JPRB, 0.39623E+00_JPRB, 0.39584E+00_JPRB ,&
&  0.39567E+00_JPRB, 0.39555E+00_JPRB, 0.39564E+00_JPRB, 0.39560E+00_JPRB, 0.39461E+00_JPRB, 0.39372E+00_JPRB, 0.39517E+00_JPRB ,&
&  0.39991E+00_JPRB, 0.40958E+00_JPRB, 0.42597E+00_JPRB, 0.45024E+00_JPRB, 0.47369E+00_JPRB, 0.49144E+00_JPRB, 0.50062E+00_JPRB ,&
&  0.50126E+00_JPRB, 0.49842E+00_JPRB, 0.49133E+00_JPRB, 0.48582E+00_JPRB, 0.48131E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.49176E+00_JPRB, 0.48422E+00_JPRB, 0.47660E+00_JPRB, 0.46863E+00_JPRB, 0.45982E+00_JPRB, 0.45008E+00_JPRB, 0.43990E+00_JPRB ,&
&  0.43149E+00_JPRB, 0.42456E+00_JPRB, 0.41799E+00_JPRB, 0.41266E+00_JPRB, 0.40785E+00_JPRB, 0.40599E+00_JPRB, 0.40576E+00_JPRB ,&
&  0.40635E+00_JPRB, 0.40596E+00_JPRB, 0.40499E+00_JPRB, 0.40537E+00_JPRB, 0.40513E+00_JPRB, 0.40289E+00_JPRB, 0.39973E+00_JPRB ,&
&  0.39446E+00_JPRB, 0.38996E+00_JPRB, 0.38848E+00_JPRB, 0.39107E+00_JPRB, 0.39036E+00_JPRB, 0.39230E+00_JPRB, 0.39972E+00_JPRB ,&
&  0.38395E+00_JPRB, 0.35902E+00_JPRB, 0.39070E+00_JPRB, 0.42659E+00_JPRB, 0.41741E+00_JPRB, 0.40672E+00_JPRB, 0.40628E+00_JPRB ,&
&  0.40561E+00_JPRB, 0.40719E+00_JPRB, 0.40687E+00_JPRB, 0.40608E+00_JPRB, 0.40636E+00_JPRB, 0.40445E+00_JPRB, 0.40054E+00_JPRB ,&
&  0.39732E+00_JPRB, 0.39764E+00_JPRB, 0.39828E+00_JPRB, 0.39827E+00_JPRB, 0.39692E+00_JPRB, 0.39493E+00_JPRB, 0.39102E+00_JPRB ,&
&  0.38758E+00_JPRB, 0.38567E+00_JPRB, 0.38584E+00_JPRB, 0.38942E+00_JPRB, 0.39541E+00_JPRB, 0.40500E+00_JPRB, 0.41968E+00_JPRB ,&
&  0.43751E+00_JPRB, 0.45442E+00_JPRB, 0.46912E+00_JPRB, 0.48266E+00_JPRB, 0.49637E+00_JPRB, 0.50738E+00_JPRB, 0.51225E+00_JPRB ,&
&  0.50504E+00_JPRB, 0.49019E+00_JPRB, 0.47306E+00_JPRB, 0.46011E+00_JPRB, 0.45112E+00_JPRB, 0.44550E+00_JPRB, 0.44234E+00_JPRB ,&
&  0.44179E+00_JPRB, 0.44133E+00_JPRB, 0.44043E+00_JPRB, 0.44030E+00_JPRB, 0.44040E+00_JPRB, 0.43901E+00_JPRB, 0.43731E+00_JPRB ,&
&  0.43479E+00_JPRB, 0.43063E+00_JPRB, 0.42541E+00_JPRB, 0.42076E+00_JPRB, 0.41748E+00_JPRB, 0.41737E+00_JPRB, 0.42006E+00_JPRB ,&
&  0.42750E+00_JPRB, 0.44263E+00_JPRB, 0.46065E+00_JPRB, 0.49886E+00_JPRB, 0.52626E+00_JPRB, 0.54510E+00_JPRB, 0.55295E+00_JPRB ,&
&  0.54790E+00_JPRB, 0.53029E+00_JPRB, 0.51427E+00_JPRB, 0.50482E+00_JPRB, 0.50153E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.50051E+00_JPRB, 0.49789E+00_JPRB, 0.49158E+00_JPRB, 0.48417E+00_JPRB, 0.47713E+00_JPRB, 0.46811E+00_JPRB, 0.45681E+00_JPRB ,&
&  0.44652E+00_JPRB, 0.43710E+00_JPRB, 0.43119E+00_JPRB, 0.42582E+00_JPRB, 0.41805E+00_JPRB, 0.41369E+00_JPRB, 0.41374E+00_JPRB ,&
&  0.41263E+00_JPRB, 0.40789E+00_JPRB, 0.40227E+00_JPRB, 0.40153E+00_JPRB, 0.39984E+00_JPRB, 0.39747E+00_JPRB, 0.39400E+00_JPRB ,&
&  0.39034E+00_JPRB, 0.38941E+00_JPRB, 0.39031E+00_JPRB, 0.39173E+00_JPRB, 0.38887E+00_JPRB, 0.39025E+00_JPRB, 0.40043E+00_JPRB ,&
&  0.44258E+00_JPRB, 0.38098E+00_JPRB, 0.42310E+00_JPRB, 0.47208E+00_JPRB, 0.46026E+00_JPRB, 0.45239E+00_JPRB, 0.44709E+00_JPRB ,&
&  0.44237E+00_JPRB, 0.44072E+00_JPRB, 0.43748E+00_JPRB, 0.43136E+00_JPRB, 0.42499E+00_JPRB, 0.41770E+00_JPRB, 0.41292E+00_JPRB ,&
&  0.41011E+00_JPRB, 0.41173E+00_JPRB, 0.41174E+00_JPRB, 0.40985E+00_JPRB, 0.40556E+00_JPRB, 0.40176E+00_JPRB, 0.39915E+00_JPRB ,&
&  0.39860E+00_JPRB, 0.40048E+00_JPRB, 0.40318E+00_JPRB, 0.40703E+00_JPRB, 0.41365E+00_JPRB, 0.42757E+00_JPRB, 0.44956E+00_JPRB ,&
&  0.47634E+00_JPRB, 0.49820E+00_JPRB, 0.51586E+00_JPRB, 0.53240E+00_JPRB, 0.54511E+00_JPRB, 0.55061E+00_JPRB, 0.54631E+00_JPRB ,&
&  0.53477E+00_JPRB, 0.51497E+00_JPRB, 0.50070E+00_JPRB, 0.48883E+00_JPRB, 0.47704E+00_JPRB, 0.46824E+00_JPRB, 0.46273E+00_JPRB ,&
&  0.45975E+00_JPRB, 0.45779E+00_JPRB, 0.45592E+00_JPRB, 0.45315E+00_JPRB, 0.45260E+00_JPRB, 0.45272E+00_JPRB, 0.45461E+00_JPRB ,&
&  0.45464E+00_JPRB, 0.45350E+00_JPRB, 0.44850E+00_JPRB, 0.44124E+00_JPRB, 0.43856E+00_JPRB, 0.43899E+00_JPRB, 0.44232E+00_JPRB ,&
&  0.45059E+00_JPRB, 0.46615E+00_JPRB, 0.48381E+00_JPRB, 0.53439E+00_JPRB, 0.56209E+00_JPRB, 0.57666E+00_JPRB, 0.57838E+00_JPRB ,&
&  0.56455E+00_JPRB, 0.54055E+00_JPRB, 0.43303E+00_JPRB, 0.49177E+00_JPRB, 0.50631E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.54256E+00_JPRB, 0.53323E+00_JPRB, 0.52014E+00_JPRB, 0.50184E+00_JPRB, 0.48850E+00_JPRB, 0.47806E+00_JPRB, 0.46699E+00_JPRB ,&
&  0.45671E+00_JPRB, 0.44815E+00_JPRB, 0.44106E+00_JPRB, 0.43300E+00_JPRB, 0.42417E+00_JPRB, 0.42213E+00_JPRB, 0.42199E+00_JPRB ,&
&  0.41905E+00_JPRB, 0.41211E+00_JPRB, 0.40563E+00_JPRB, 0.40435E+00_JPRB, 0.40387E+00_JPRB, 0.40365E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.39596E+00_JPRB, 0.39546E+00_JPRB, 0.39912E+00_JPRB, 0.40026E+00_JPRB, 0.39658E+00_JPRB, 0.39830E+00_JPRB, 0.40854E+00_JPRB ,&
&  0.45903E+00_JPRB, 0.41287E+00_JPRB, 0.45675E+00_JPRB, 0.54782E+00_JPRB, 0.54732E+00_JPRB, 0.55252E+00_JPRB, 0.53649E+00_JPRB ,&
&  0.51382E+00_JPRB, 0.49996E+00_JPRB, 0.48553E+00_JPRB, 0.46590E+00_JPRB, 0.44755E+00_JPRB, 0.43692E+00_JPRB, 0.43422E+00_JPRB ,&
&  0.43589E+00_JPRB, 0.43612E+00_JPRB, 0.43546E+00_JPRB, 0.43035E+00_JPRB, 0.42711E+00_JPRB, 0.42514E+00_JPRB, 0.42466E+00_JPRB ,&
&  0.42479E+00_JPRB, 0.42796E+00_JPRB, 0.43209E+00_JPRB, 0.43493E+00_JPRB, 0.44299E+00_JPRB, 0.46766E+00_JPRB, 0.50276E+00_JPRB ,&
&  0.53507E+00_JPRB, 0.56075E+00_JPRB, 0.57810E+00_JPRB, 0.59124E+00_JPRB, 0.59567E+00_JPRB, 0.59799E+00_JPRB, 0.59376E+00_JPRB ,&
&  0.57702E+00_JPRB, 0.55639E+00_JPRB, 0.54052E+00_JPRB, 0.52305E+00_JPRB, 0.50678E+00_JPRB, 0.49426E+00_JPRB, 0.48411E+00_JPRB ,&
&  0.47614E+00_JPRB, 0.47015E+00_JPRB, 0.46537E+00_JPRB, 0.45874E+00_JPRB, 0.45464E+00_JPRB, 0.45543E+00_JPRB, 0.45765E+00_JPRB ,&
&  0.46047E+00_JPRB, 0.46489E+00_JPRB, 0.46880E+00_JPRB, 0.47144E+00_JPRB, 0.47353E+00_JPRB, 0.47717E+00_JPRB, 0.48180E+00_JPRB ,&
&  0.49415E+00_JPRB, 0.51636E+00_JPRB, 0.53834E+00_JPRB, 0.59829E+00_JPRB, 0.63391E+00_JPRB, 0.62935E+00_JPRB, 0.60699E+00_JPRB ,&
&  0.57761E+00_JPRB, 0.54635E+00_JPRB, 0.52951E+00_JPRB, 0.47328E+00_JPRB, 0.54854E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.60069E+00_JPRB, 0.57720E+00_JPRB, 0.54217E+00_JPRB, 0.50892E+00_JPRB, 0.48596E+00_JPRB, 0.47157E+00_JPRB, 0.46307E+00_JPRB ,&
&  0.45654E+00_JPRB, 0.45026E+00_JPRB, 0.44335E+00_JPRB, 0.43384E+00_JPRB, 0.42804E+00_JPRB, 0.42563E+00_JPRB, 0.42408E+00_JPRB ,&
&  0.42069E+00_JPRB, 0.41221E+00_JPRB, 0.40969E+00_JPRB, 0.40733E+00_JPRB, 0.40625E+00_JPRB, 0.40491E+00_JPRB, 0.40044E+00_JPRB ,&
&  0.39479E+00_JPRB, 0.39215E+00_JPRB, 0.39594E+00_JPRB, 0.39820E+00_JPRB, 0.40050E+00_JPRB, 0.40583E+00_JPRB, 0.42016E+00_JPRB ,&
&  0.47743E+00_JPRB, 0.49009E+00_JPRB, 0.56691E+00_JPRB, 0.64460E+00_JPRB, 0.64685E+00_JPRB, 0.63064E+00_JPRB, 0.61293E+00_JPRB ,&
&  0.58961E+00_JPRB, 0.56963E+00_JPRB, 0.53778E+00_JPRB, 0.50132E+00_JPRB, 0.47240E+00_JPRB, 0.46314E+00_JPRB, 0.45557E+00_JPRB ,&
&  0.44975E+00_JPRB, 0.44721E+00_JPRB, 0.44659E+00_JPRB, 0.44663E+00_JPRB, 0.44715E+00_JPRB, 0.44652E+00_JPRB, 0.44601E+00_JPRB ,&
&  0.44715E+00_JPRB, 0.45064E+00_JPRB, 0.45423E+00_JPRB, 0.46260E+00_JPRB, 0.48512E+00_JPRB, 0.52238E+00_JPRB, 0.56077E+00_JPRB ,&
&  0.58762E+00_JPRB, 0.60662E+00_JPRB, 0.63188E+00_JPRB, 0.65014E+00_JPRB, 0.65761E+00_JPRB, 0.65502E+00_JPRB, 0.64181E+00_JPRB ,&
&  0.62580E+00_JPRB, 0.61269E+00_JPRB, 0.58443E+00_JPRB, 0.55428E+00_JPRB, 0.53582E+00_JPRB, 0.52485E+00_JPRB, 0.51739E+00_JPRB ,&
&  0.51175E+00_JPRB, 0.50348E+00_JPRB, 0.49340E+00_JPRB, 0.48467E+00_JPRB, 0.48016E+00_JPRB, 0.47517E+00_JPRB, 0.46991E+00_JPRB ,&
&  0.47040E+00_JPRB, 0.47724E+00_JPRB, 0.48795E+00_JPRB, 0.49562E+00_JPRB, 0.50242E+00_JPRB, 0.51125E+00_JPRB, 0.51771E+00_JPRB ,&
&  0.52729E+00_JPRB, 0.55181E+00_JPRB, 0.57148E+00_JPRB, 0.63236E+00_JPRB, 0.63895E+00_JPRB, 0.62970E+00_JPRB, 0.60164E+00_JPRB ,&
&  0.56822E+00_JPRB, 0.54046E+00_JPRB, 0.51909E+00_JPRB, 0.53763E+00_JPRB, 0.59595E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.64411E+00_JPRB, 0.58949E+00_JPRB, 0.54439E+00_JPRB, 0.50741E+00_JPRB, 0.48436E+00_JPRB, 0.47079E+00_JPRB, 0.46135E+00_JPRB ,&
&  0.45692E+00_JPRB, 0.45423E+00_JPRB, 0.44772E+00_JPRB, 0.44296E+00_JPRB, 0.43955E+00_JPRB, 0.43819E+00_JPRB, 0.43724E+00_JPRB ,&
&  0.42937E+00_JPRB, 0.42802E+00_JPRB, 0.42821E+00_JPRB, 0.42403E+00_JPRB, 0.41835E+00_JPRB, 0.41249E+00_JPRB, 0.40489E+00_JPRB ,&
&  0.39993E+00_JPRB, 0.39843E+00_JPRB, 0.40066E+00_JPRB, 0.40470E+00_JPRB, 0.40874E+00_JPRB, 0.41989E+00_JPRB, 0.44219E+00_JPRB ,&
&  0.51624E+00_JPRB, 0.57240E+00_JPRB, 0.69158E+00_JPRB, 0.78116E+00_JPRB, 0.73262E+00_JPRB, 0.69623E+00_JPRB, 0.66569E+00_JPRB ,&
&  0.65976E+00_JPRB, 0.65635E+00_JPRB, 0.62296E+00_JPRB, 0.57733E+00_JPRB, 0.53614E+00_JPRB, 0.51241E+00_JPRB, 0.49778E+00_JPRB ,&
&  0.48539E+00_JPRB, 0.47730E+00_JPRB, 0.47084E+00_JPRB, 0.47344E+00_JPRB, 0.47572E+00_JPRB, 0.47419E+00_JPRB, 0.47251E+00_JPRB ,&
&  0.47180E+00_JPRB, 0.47456E+00_JPRB, 0.48203E+00_JPRB, 0.50834E+00_JPRB, 0.56042E+00_JPRB, 0.61236E+00_JPRB, 0.65968E+00_JPRB ,&
&  0.68554E+00_JPRB, 0.71121E+00_JPRB, 0.73808E+00_JPRB, 0.75359E+00_JPRB, 0.75256E+00_JPRB, 0.74377E+00_JPRB, 0.73404E+00_JPRB ,&
&  0.71878E+00_JPRB, 0.68081E+00_JPRB, 0.62995E+00_JPRB, 0.59281E+00_JPRB, 0.57337E+00_JPRB, 0.56315E+00_JPRB, 0.55794E+00_JPRB ,&
&  0.55362E+00_JPRB, 0.54916E+00_JPRB, 0.54141E+00_JPRB, 0.52994E+00_JPRB, 0.51527E+00_JPRB, 0.50519E+00_JPRB, 0.49702E+00_JPRB ,&
&  0.49252E+00_JPRB, 0.49377E+00_JPRB, 0.50239E+00_JPRB, 0.51644E+00_JPRB, 0.52757E+00_JPRB, 0.53737E+00_JPRB, 0.54514E+00_JPRB ,&
&  0.55729E+00_JPRB, 0.59463E+00_JPRB, 0.58767E+00_JPRB, 0.64489E+00_JPRB, 0.70714E+00_JPRB, 0.66953E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.57294E+00_JPRB, 0.54114E+00_JPRB, 0.53203E+00_JPRB, 0.55825E+00_JPRB, 0.67647E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.54965E+00_JPRB, 0.52925E+00_JPRB, 0.51062E+00_JPRB, 0.49089E+00_JPRB, 0.47328E+00_JPRB, 0.46460E+00_JPRB, 0.46231E+00_JPRB ,&
&  0.46410E+00_JPRB, 0.46487E+00_JPRB, 0.46264E+00_JPRB, 0.45654E+00_JPRB, 0.45239E+00_JPRB, 0.44870E+00_JPRB, 0.44573E+00_JPRB ,&
&  0.43944E+00_JPRB, 0.43996E+00_JPRB, 0.43643E+00_JPRB, 0.43047E+00_JPRB, 0.42386E+00_JPRB, 0.41601E+00_JPRB, 0.40842E+00_JPRB ,&
&  0.40394E+00_JPRB, 0.40278E+00_JPRB, 0.40403E+00_JPRB, 0.40852E+00_JPRB, 0.42042E+00_JPRB, 0.43227E+00_JPRB, 0.44729E+00_JPRB ,&
&  0.52944E+00_JPRB, 0.65450E+00_JPRB, 0.85351E+00_JPRB, 0.10331E+01_JPRB, 0.10074E+01_JPRB, 0.87526E+00_JPRB, 0.78919E+00_JPRB ,&
&  0.75646E+00_JPRB, 0.73304E+00_JPRB, 0.69934E+00_JPRB, 0.66100E+00_JPRB, 0.61753E+00_JPRB, 0.59550E+00_JPRB, 0.57399E+00_JPRB ,&
&  0.55278E+00_JPRB, 0.53583E+00_JPRB, 0.51986E+00_JPRB, 0.51462E+00_JPRB, 0.51025E+00_JPRB, 0.50352E+00_JPRB, 0.49793E+00_JPRB ,&
&  0.49858E+00_JPRB, 0.50696E+00_JPRB, 0.52732E+00_JPRB, 0.57629E+00_JPRB, 0.66108E+00_JPRB, 0.73689E+00_JPRB, 0.82225E+00_JPRB ,&
&  0.86194E+00_JPRB, 0.88025E+00_JPRB, 0.89556E+00_JPRB, 0.88634E+00_JPRB, 0.84481E+00_JPRB, 0.81592E+00_JPRB, 0.81214E+00_JPRB ,&
&  0.78705E+00_JPRB, 0.72009E+00_JPRB, 0.65158E+00_JPRB, 0.60836E+00_JPRB, 0.58562E+00_JPRB, 0.57690E+00_JPRB, 0.57126E+00_JPRB ,&
&  0.57037E+00_JPRB, 0.56827E+00_JPRB, 0.55765E+00_JPRB, 0.54144E+00_JPRB, 0.52573E+00_JPRB, 0.51535E+00_JPRB, 0.51026E+00_JPRB ,&
&  0.50528E+00_JPRB, 0.50568E+00_JPRB, 0.51999E+00_JPRB, 0.52880E+00_JPRB, 0.53372E+00_JPRB, 0.54168E+00_JPRB, 0.55501E+00_JPRB ,&
&  0.57408E+00_JPRB, 0.60996E+00_JPRB, 0.60455E+00_JPRB, 0.63482E+00_JPRB, 0.66888E+00_JPRB, 0.67194E+00_JPRB, 0.62242E+00_JPRB ,&
&  0.57117E+00_JPRB, 0.54917E+00_JPRB, 0.55354E+00_JPRB, 0.56386E+00_JPRB, 0.57017E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.52438E+00_JPRB, 0.50895E+00_JPRB, 0.49602E+00_JPRB, 0.47507E+00_JPRB, 0.46221E+00_JPRB, 0.45557E+00_JPRB, 0.45790E+00_JPRB ,&
&  0.46369E+00_JPRB, 0.47061E+00_JPRB, 0.46818E+00_JPRB, 0.46509E+00_JPRB, 0.45947E+00_JPRB, 0.45578E+00_JPRB, 0.44854E+00_JPRB ,&
&  0.44454E+00_JPRB, 0.44052E+00_JPRB, 0.43438E+00_JPRB, 0.42757E+00_JPRB, 0.41763E+00_JPRB, 0.41331E+00_JPRB, 0.40784E+00_JPRB ,&
&  0.40268E+00_JPRB, 0.40080E+00_JPRB, 0.40309E+00_JPRB, 0.41201E+00_JPRB, 0.42318E+00_JPRB, 0.43653E+00_JPRB, 0.45739E+00_JPRB ,&
&  0.56868E+00_JPRB, 0.64990E+00_JPRB, 0.11124E+01_JPRB, 0.12201E+01_JPRB, 0.11376E+01_JPRB, 0.99763E+00_JPRB, 0.96951E+00_JPRB ,&
&  0.94277E+00_JPRB, 0.87784E+00_JPRB, 0.81794E+00_JPRB, 0.76956E+00_JPRB, 0.72295E+00_JPRB, 0.69221E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.62608E+00_JPRB, 0.59557E+00_JPRB, 0.55964E+00_JPRB, 0.53534E+00_JPRB, 0.51855E+00_JPRB, 0.51193E+00_JPRB, 0.51674E+00_JPRB ,&
&  0.52743E+00_JPRB, 0.54596E+00_JPRB, 0.59402E+00_JPRB, 0.66886E+00_JPRB, 0.74895E+00_JPRB, 0.75714E+00_JPRB, 0.87053E+00_JPRB ,&
&  0.10801E+01_JPRB, 0.10837E+01_JPRB, 0.10190E+01_JPRB, 0.95782E+00_JPRB, 0.86736E+00_JPRB, 0.83061E+00_JPRB, 0.81927E+00_JPRB ,&
&  0.78554E+00_JPRB, 0.73232E+00_JPRB, 0.67590E+00_JPRB, 0.63305E+00_JPRB, 0.60304E+00_JPRB, 0.58541E+00_JPRB, 0.58153E+00_JPRB ,&
&  0.57593E+00_JPRB, 0.56153E+00_JPRB, 0.54029E+00_JPRB, 0.52256E+00_JPRB, 0.51323E+00_JPRB, 0.51131E+00_JPRB, 0.51278E+00_JPRB ,&
&  0.51593E+00_JPRB, 0.52209E+00_JPRB, 0.54198E+00_JPRB, 0.52413E+00_JPRB, 0.53042E+00_JPRB, 0.54952E+00_JPRB, 0.55904E+00_JPRB ,&
&  0.55513E+00_JPRB, 0.57216E+00_JPRB, 0.59117E+00_JPRB, 0.61558E+00_JPRB, 0.66507E+00_JPRB, 0.69184E+00_JPRB, 0.63541E+00_JPRB ,&
&  0.57768E+00_JPRB, 0.55718E+00_JPRB, 0.56232E+00_JPRB, 0.56161E+00_JPRB, 0.55406E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.50384E+00_JPRB, 0.48820E+00_JPRB, 0.47820E+00_JPRB, 0.46437E+00_JPRB, 0.45214E+00_JPRB, 0.44586E+00_JPRB, 0.44893E+00_JPRB ,&
&  0.46218E+00_JPRB, 0.47440E+00_JPRB, 0.46977E+00_JPRB, 0.46786E+00_JPRB, 0.46681E+00_JPRB, 0.46040E+00_JPRB, 0.45057E+00_JPRB ,&
&  0.44071E+00_JPRB, 0.43137E+00_JPRB, 0.42047E+00_JPRB, 0.40905E+00_JPRB, 0.40117E+00_JPRB, 0.40170E+00_JPRB, 0.40018E+00_JPRB ,&
&  0.39840E+00_JPRB, 0.40208E+00_JPRB, 0.40696E+00_JPRB, 0.41567E+00_JPRB, 0.42381E+00_JPRB, 0.43693E+00_JPRB, 0.46498E+00_JPRB ,&
&  0.57021E+00_JPRB, 0.46290E+00_JPRB, 0.10525E+01_JPRB, 0.13949E+01_JPRB, 0.14443E+01_JPRB, 0.13230E+01_JPRB, 0.12285E+01_JPRB ,&
&  0.11810E+01_JPRB, 0.10477E+01_JPRB, 0.94288E+00_JPRB, 0.84810E+00_JPRB, 0.74250E+00_JPRB, 0.68030E+00_JPRB, 0.64723E+00_JPRB ,&
&  0.62364E+00_JPRB, 0.60063E+00_JPRB, 0.58119E+00_JPRB, 0.56237E+00_JPRB, 0.54521E+00_JPRB, 0.54060E+00_JPRB, 0.55234E+00_JPRB ,&
&  0.57337E+00_JPRB, 0.61314E+00_JPRB, 0.69366E+00_JPRB, 0.79329E+00_JPRB, 0.83344E+00_JPRB, 0.87053E+00_JPRB, 0.93523E+00_JPRB ,&
&  0.10647E+01_JPRB, 0.11838E+01_JPRB, 0.10399E+01_JPRB, 0.89494E+00_JPRB, 0.80111E+00_JPRB, 0.77514E+00_JPRB, 0.77176E+00_JPRB ,&
&  0.74565E+00_JPRB, 0.71575E+00_JPRB, 0.68152E+00_JPRB, 0.64424E+00_JPRB, 0.61216E+00_JPRB, 0.59246E+00_JPRB, 0.57795E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.53806E+00_JPRB, 0.52375E+00_JPRB, 0.51590E+00_JPRB, 0.51277E+00_JPRB, 0.51293E+00_JPRB, 0.51863E+00_JPRB ,&
&  0.53109E+00_JPRB, 0.53838E+00_JPRB, 0.53605E+00_JPRB, 0.51733E+00_JPRB, 0.51216E+00_JPRB, 0.51843E+00_JPRB, 0.52647E+00_JPRB ,&
&  0.55645E+00_JPRB, 0.57030E+00_JPRB, 0.58196E+00_JPRB, 0.61122E+00_JPRB, 0.60566E+00_JPRB, 0.65762E+00_JPRB, 0.61481E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.55754E+00_JPRB, 0.55412E+00_JPRB, 0.54669E+00_JPRB, 0.52825E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.47940E+00_JPRB, 0.46800E+00_JPRB, 0.45844E+00_JPRB, 0.45401E+00_JPRB, 0.44637E+00_JPRB, 0.44331E+00_JPRB, 0.44829E+00_JPRB ,&
&  0.45933E+00_JPRB, 0.46309E+00_JPRB, 0.46175E+00_JPRB, 0.45948E+00_JPRB, 0.45432E+00_JPRB, 0.44718E+00_JPRB, 0.43782E+00_JPRB ,&
&  0.42972E+00_JPRB, 0.41734E+00_JPRB, 0.40149E+00_JPRB, 0.39016E+00_JPRB, 0.39443E+00_JPRB, 0.39933E+00_JPRB, 0.39913E+00_JPRB ,&
&  0.39817E+00_JPRB, 0.40433E+00_JPRB, 0.40701E+00_JPRB, 0.41700E+00_JPRB, 0.42431E+00_JPRB, 0.44112E+00_JPRB, 0.47360E+00_JPRB ,&
&  0.56577E+00_JPRB, 0.86519E+00_JPRB, 0.88570E+00_JPRB, 0.15895E+01_JPRB, 0.17498E+01_JPRB, 0.16380E+01_JPRB, 0.13737E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.11708E+01_JPRB, 0.10056E+01_JPRB, 0.86815E+00_JPRB, 0.78159E+00_JPRB, 0.71947E+00_JPRB, 0.66574E+00_JPRB ,&
&  0.62289E+00_JPRB, 0.59570E+00_JPRB, 0.57939E+00_JPRB, 0.56549E+00_JPRB, 0.56267E+00_JPRB, 0.57166E+00_JPRB, 0.59169E+00_JPRB ,&
&  0.62815E+00_JPRB, 0.69119E+00_JPRB, 0.80516E+00_JPRB, 0.90812E+00_JPRB, 0.91140E+00_JPRB, 0.96254E+00_JPRB, 0.10952E+01_JPRB ,&
&  0.11429E+01_JPRB, 0.12152E+01_JPRB, 0.97182E+00_JPRB, 0.81288E+00_JPRB, 0.75370E+00_JPRB, 0.75677E+00_JPRB, 0.75016E+00_JPRB ,&
&  0.72024E+00_JPRB, 0.69756E+00_JPRB, 0.66629E+00_JPRB, 0.63325E+00_JPRB, 0.60831E+00_JPRB, 0.58924E+00_JPRB, 0.56576E+00_JPRB ,&
&  0.54725E+00_JPRB, 0.54036E+00_JPRB, 0.53694E+00_JPRB, 0.53412E+00_JPRB, 0.53503E+00_JPRB, 0.54040E+00_JPRB, 0.54672E+00_JPRB ,&
&  0.55388E+00_JPRB, 0.54900E+00_JPRB, 0.52579E+00_JPRB, 0.51163E+00_JPRB, 0.51455E+00_JPRB, 0.52274E+00_JPRB, 0.52420E+00_JPRB ,&
&  0.55890E+00_JPRB, 0.57903E+00_JPRB, 0.58425E+00_JPRB, 0.60610E+00_JPRB, 0.62905E+00_JPRB, 0.72919E+00_JPRB, 0.62550E+00_JPRB ,&
&  0.57974E+00_JPRB, 0.55387E+00_JPRB, 0.54223E+00_JPRB, 0.53405E+00_JPRB, 0.50894E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.45740E+00_JPRB, 0.44374E+00_JPRB, 0.43780E+00_JPRB, 0.43833E+00_JPRB, 0.43819E+00_JPRB, 0.43868E+00_JPRB, 0.44525E+00_JPRB ,&
&  0.44814E+00_JPRB, 0.44855E+00_JPRB, 0.44690E+00_JPRB, 0.44433E+00_JPRB, 0.43739E+00_JPRB, 0.42964E+00_JPRB, 0.42138E+00_JPRB ,&
&  0.41609E+00_JPRB, 0.39981E+00_JPRB, 0.39174E+00_JPRB, 0.39993E+00_JPRB, 0.40395E+00_JPRB, 0.40299E+00_JPRB, 0.40373E+00_JPRB ,&
&  0.40409E+00_JPRB, 0.40808E+00_JPRB, 0.40850E+00_JPRB, 0.41442E+00_JPRB, 0.42574E+00_JPRB, 0.45289E+00_JPRB, 0.49017E+00_JPRB ,&
&  0.60712E+00_JPRB, 0.80468E+00_JPRB, 0.86030E+00_JPRB, 0.16537E+01_JPRB, 0.18023E+01_JPRB, 0.16959E+01_JPRB, 0.14277E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12721E+01_JPRB, 0.10881E+01_JPRB, 0.94402E+00_JPRB, 0.85729E+00_JPRB, 0.78090E+00_JPRB, 0.69408E+00_JPRB ,&
&  0.62352E+00_JPRB, 0.58611E+00_JPRB, 0.57407E+00_JPRB, 0.58183E+00_JPRB, 0.59942E+00_JPRB, 0.61535E+00_JPRB, 0.64018E+00_JPRB ,&
&  0.69916E+00_JPRB, 0.80016E+00_JPRB, 0.93749E+00_JPRB, 0.95590E+00_JPRB, 0.95342E+00_JPRB, 0.98761E+00_JPRB, 0.10310E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10446E+01_JPRB, 0.84119E+00_JPRB, 0.74993E+00_JPRB, 0.71950E+00_JPRB, 0.73402E+00_JPRB, 0.72264E+00_JPRB ,&
&  0.69247E+00_JPRB, 0.66587E+00_JPRB, 0.63340E+00_JPRB, 0.61121E+00_JPRB, 0.59591E+00_JPRB, 0.57516E+00_JPRB, 0.56185E+00_JPRB ,&
&  0.55720E+00_JPRB, 0.55597E+00_JPRB, 0.56001E+00_JPRB, 0.56131E+00_JPRB, 0.56429E+00_JPRB, 0.56595E+00_JPRB, 0.56572E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.54160E+00_JPRB, 0.52533E+00_JPRB, 0.53196E+00_JPRB, 0.55293E+00_JPRB, 0.55811E+00_JPRB, 0.55636E+00_JPRB ,&
&  0.57930E+00_JPRB, 0.59704E+00_JPRB, 0.60439E+00_JPRB, 0.60204E+00_JPRB, 0.62822E+00_JPRB, 0.67308E+00_JPRB, 0.59712E+00_JPRB ,&
&  0.57079E+00_JPRB, 0.54794E+00_JPRB, 0.53172E+00_JPRB, 0.52019E+00_JPRB, 0.48996E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.45902E+00_JPRB, 0.43770E+00_JPRB, 0.42964E+00_JPRB, 0.42521E+00_JPRB, 0.42524E+00_JPRB, 0.43010E+00_JPRB, 0.43497E+00_JPRB ,&
&  0.43397E+00_JPRB, 0.43323E+00_JPRB, 0.43385E+00_JPRB, 0.43040E+00_JPRB, 0.42012E+00_JPRB, 0.41048E+00_JPRB, 0.40218E+00_JPRB ,&
&  0.40178E+00_JPRB, 0.40419E+00_JPRB, 0.41635E+00_JPRB, 0.42414E+00_JPRB, 0.42522E+00_JPRB, 0.42279E+00_JPRB, 0.42366E+00_JPRB ,&
&  0.42715E+00_JPRB, 0.42605E+00_JPRB, 0.42370E+00_JPRB, 0.43372E+00_JPRB, 0.45335E+00_JPRB, 0.49193E+00_JPRB, 0.55355E+00_JPRB ,&
&  0.72660E+00_JPRB, 0.85537E+00_JPRB, 0.91224E+00_JPRB, 0.13999E+01_JPRB, 0.16656E+01_JPRB, 0.15643E+01_JPRB, 0.14166E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.11089E+01_JPRB, 0.11264E+01_JPRB, 0.98812E+00_JPRB, 0.87976E+00_JPRB, 0.79397E+00_JPRB, 0.72213E+00_JPRB ,&
&  0.66462E+00_JPRB, 0.63605E+00_JPRB, 0.64546E+00_JPRB, 0.66439E+00_JPRB, 0.67505E+00_JPRB, 0.68603E+00_JPRB, 0.73241E+00_JPRB ,&
&  0.81372E+00_JPRB, 0.93830E+00_JPRB, 0.10797E+01_JPRB, 0.10648E+01_JPRB, 0.93889E+00_JPRB, 0.93661E+00_JPRB, 0.94180E+00_JPRB ,&
&  0.99139E+00_JPRB, 0.88142E+00_JPRB, 0.77137E+00_JPRB, 0.72517E+00_JPRB, 0.68166E+00_JPRB, 0.71050E+00_JPRB, 0.69171E+00_JPRB ,&
&  0.66778E+00_JPRB, 0.63619E+00_JPRB, 0.60695E+00_JPRB, 0.58976E+00_JPRB, 0.57621E+00_JPRB, 0.56631E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.55578E+00_JPRB, 0.55601E+00_JPRB, 0.56544E+00_JPRB, 0.57055E+00_JPRB, 0.56989E+00_JPRB, 0.55936E+00_JPRB, 0.54381E+00_JPRB ,&
&  0.53423E+00_JPRB, 0.54431E+00_JPRB, 0.55066E+00_JPRB, 0.57129E+00_JPRB, 0.59120E+00_JPRB, 0.60024E+00_JPRB, 0.58696E+00_JPRB ,&
&  0.58599E+00_JPRB, 0.60892E+00_JPRB, 0.62129E+00_JPRB, 0.63421E+00_JPRB, 0.63689E+00_JPRB, 0.63003E+00_JPRB, 0.57773E+00_JPRB ,&
&  0.55353E+00_JPRB, 0.53608E+00_JPRB, 0.52808E+00_JPRB, 0.51534E+00_JPRB, 0.48284E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.46577E+00_JPRB, 0.45458E+00_JPRB, 0.41678E+00_JPRB, 0.41867E+00_JPRB, 0.41909E+00_JPRB, 0.42383E+00_JPRB, 0.41991E+00_JPRB ,&
&  0.41683E+00_JPRB, 0.41789E+00_JPRB, 0.42018E+00_JPRB, 0.41333E+00_JPRB, 0.39285E+00_JPRB, 0.38406E+00_JPRB, 0.38778E+00_JPRB ,&
&  0.39849E+00_JPRB, 0.42041E+00_JPRB, 0.44795E+00_JPRB, 0.45299E+00_JPRB, 0.45500E+00_JPRB, 0.45227E+00_JPRB, 0.44894E+00_JPRB ,&
&  0.45102E+00_JPRB, 0.45626E+00_JPRB, 0.46504E+00_JPRB, 0.48706E+00_JPRB, 0.52612E+00_JPRB, 0.58637E+00_JPRB, 0.68519E+00_JPRB ,&
&  0.93187E+00_JPRB, 0.96522E+00_JPRB, 0.50683E+00_JPRB, 0.90861E+00_JPRB, 0.13093E+01_JPRB, 0.13489E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.10223E+01_JPRB, 0.98770E+00_JPRB, 0.95982E+00_JPRB, 0.86651E+00_JPRB, 0.81496E+00_JPRB, 0.76525E+00_JPRB ,&
&  0.72477E+00_JPRB, 0.72312E+00_JPRB, 0.75186E+00_JPRB, 0.75992E+00_JPRB, 0.76639E+00_JPRB, 0.81151E+00_JPRB, 0.88981E+00_JPRB ,&
&  0.98656E+00_JPRB, 0.11216E+01_JPRB, 0.12311E+01_JPRB, 0.10635E+01_JPRB, 0.95966E+00_JPRB, 0.89759E+00_JPRB, 0.84305E+00_JPRB ,&
&  0.79544E+00_JPRB, 0.78074E+00_JPRB, 0.73987E+00_JPRB, 0.70547E+00_JPRB, 0.66463E+00_JPRB, 0.64919E+00_JPRB, 0.66197E+00_JPRB ,&
&  0.63759E+00_JPRB, 0.59946E+00_JPRB, 0.57510E+00_JPRB, 0.56170E+00_JPRB, 0.55055E+00_JPRB, 0.54467E+00_JPRB, 0.53851E+00_JPRB ,&
&  0.53800E+00_JPRB, 0.54590E+00_JPRB, 0.55476E+00_JPRB, 0.55419E+00_JPRB, 0.54528E+00_JPRB, 0.53188E+00_JPRB, 0.52213E+00_JPRB ,&
&  0.53248E+00_JPRB, 0.55808E+00_JPRB, 0.59330E+00_JPRB, 0.60518E+00_JPRB, 0.61422E+00_JPRB, 0.62897E+00_JPRB, 0.63050E+00_JPRB ,&
&  0.63677E+00_JPRB, 0.65462E+00_JPRB, 0.61784E+00_JPRB, 0.61132E+00_JPRB, 0.62824E+00_JPRB, 0.60471E+00_JPRB, 0.58454E+00_JPRB ,&
&  0.61636E+00_JPRB, 0.69050E+00_JPRB, 0.81282E+00_JPRB, 0.55440E+00_JPRB, 0.49904E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.47707E+00_JPRB, 0.43889E+00_JPRB, 0.40394E+00_JPRB, 0.40315E+00_JPRB, 0.42008E+00_JPRB, 0.42357E+00_JPRB, 0.42003E+00_JPRB ,&
&  0.40925E+00_JPRB, 0.39871E+00_JPRB, 0.38499E+00_JPRB, 0.37757E+00_JPRB, 0.38152E+00_JPRB, 0.39161E+00_JPRB, 0.42296E+00_JPRB ,&
&  0.45422E+00_JPRB, 0.48332E+00_JPRB, 0.50796E+00_JPRB, 0.51204E+00_JPRB, 0.50115E+00_JPRB, 0.50101E+00_JPRB, 0.50755E+00_JPRB ,&
&  0.51304E+00_JPRB, 0.53311E+00_JPRB, 0.56264E+00_JPRB, 0.60484E+00_JPRB, 0.66800E+00_JPRB, 0.77333E+00_JPRB, 0.94086E+00_JPRB ,&
&  0.12344E+01_JPRB, 0.88370E+00_JPRB, 0.60454E+00_JPRB, 0.10820E+01_JPRB, 0.12620E+01_JPRB, 0.12685E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.95110E+00_JPRB, 0.92827E+00_JPRB, 0.95304E+00_JPRB, 0.87469E+00_JPRB, 0.84912E+00_JPRB, 0.81970E+00_JPRB ,&
&  0.80432E+00_JPRB, 0.82629E+00_JPRB, 0.86265E+00_JPRB, 0.88763E+00_JPRB, 0.92782E+00_JPRB, 0.10166E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.13492E+01_JPRB, 0.14162E+01_JPRB, 0.11445E+01_JPRB, 0.99315E+00_JPRB, 0.87237E+00_JPRB, 0.80606E+00_JPRB ,&
&  0.73464E+00_JPRB, 0.72246E+00_JPRB, 0.70494E+00_JPRB, 0.67761E+00_JPRB, 0.64453E+00_JPRB, 0.61770E+00_JPRB, 0.62832E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.55679E+00_JPRB, 0.53328E+00_JPRB, 0.51968E+00_JPRB, 0.51513E+00_JPRB, 0.51450E+00_JPRB, 0.50920E+00_JPRB ,&
&  0.51469E+00_JPRB, 0.51886E+00_JPRB, 0.52670E+00_JPRB, 0.52197E+00_JPRB, 0.51854E+00_JPRB, 0.52003E+00_JPRB, 0.51392E+00_JPRB ,&
&  0.54045E+00_JPRB, 0.57639E+00_JPRB, 0.59384E+00_JPRB, 0.61715E+00_JPRB, 0.61876E+00_JPRB, 0.61052E+00_JPRB, 0.62392E+00_JPRB ,&
&  0.62141E+00_JPRB, 0.63087E+00_JPRB, 0.61110E+00_JPRB, 0.62063E+00_JPRB, 0.60633E+00_JPRB, 0.59848E+00_JPRB, 0.60418E+00_JPRB ,&
&  0.64704E+00_JPRB, 0.69966E+00_JPRB, 0.69437E+00_JPRB, 0.54701E+00_JPRB, 0.50046E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.45154E+00_JPRB, 0.42414E+00_JPRB, 0.38671E+00_JPRB, 0.37257E+00_JPRB, 0.39295E+00_JPRB, 0.40871E+00_JPRB, 0.40176E+00_JPRB ,&
&  0.39478E+00_JPRB, 0.38607E+00_JPRB, 0.38230E+00_JPRB, 0.39026E+00_JPRB, 0.42109E+00_JPRB, 0.46484E+00_JPRB, 0.49650E+00_JPRB ,&
&  0.52804E+00_JPRB, 0.55356E+00_JPRB, 0.57480E+00_JPRB, 0.58122E+00_JPRB, 0.57704E+00_JPRB, 0.58436E+00_JPRB, 0.59672E+00_JPRB ,&
&  0.61984E+00_JPRB, 0.65623E+00_JPRB, 0.70553E+00_JPRB, 0.77715E+00_JPRB, 0.87425E+00_JPRB, 0.99379E+00_JPRB, 0.11124E+01_JPRB ,&
&  0.81541E+00_JPRB, 0.43437E+00_JPRB, 0.10150E+01_JPRB, 0.11524E+01_JPRB, 0.10712E+01_JPRB, 0.10424E+01_JPRB, 0.96356E+00_JPRB ,&
&  0.89764E+00_JPRB, 0.89933E+00_JPRB, 0.86584E+00_JPRB, 0.91884E+00_JPRB, 0.86767E+00_JPRB, 0.87681E+00_JPRB, 0.89653E+00_JPRB ,&
&  0.93101E+00_JPRB, 0.97786E+00_JPRB, 0.10375E+01_JPRB, 0.11031E+01_JPRB, 0.11756E+01_JPRB, 0.12982E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.15410E+01_JPRB, 0.16586E+01_JPRB, 0.17125E+01_JPRB, 0.13115E+01_JPRB, 0.11075E+01_JPRB, 0.98627E+00_JPRB, 0.88286E+00_JPRB ,&
&  0.77449E+00_JPRB, 0.65882E+00_JPRB, 0.63661E+00_JPRB, 0.64925E+00_JPRB, 0.62999E+00_JPRB, 0.58870E+00_JPRB, 0.59268E+00_JPRB ,&
&  0.54592E+00_JPRB, 0.50912E+00_JPRB, 0.48483E+00_JPRB, 0.46733E+00_JPRB, 0.47277E+00_JPRB, 0.47776E+00_JPRB, 0.48007E+00_JPRB ,&
&  0.47293E+00_JPRB, 0.47628E+00_JPRB, 0.49150E+00_JPRB, 0.49681E+00_JPRB, 0.50574E+00_JPRB, 0.51863E+00_JPRB, 0.52571E+00_JPRB ,&
&  0.55659E+00_JPRB, 0.57122E+00_JPRB, 0.58829E+00_JPRB, 0.60872E+00_JPRB, 0.61778E+00_JPRB, 0.59965E+00_JPRB, 0.59404E+00_JPRB ,&
&  0.59571E+00_JPRB, 0.59835E+00_JPRB, 0.59058E+00_JPRB, 0.60035E+00_JPRB, 0.59698E+00_JPRB, 0.59961E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.63334E+00_JPRB, 0.63672E+00_JPRB, 0.60149E+00_JPRB, 0.52330E+00_JPRB, 0.48475E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.43134E+00_JPRB, 0.40723E+00_JPRB, 0.37118E+00_JPRB, 0.33801E+00_JPRB, 0.35813E+00_JPRB, 0.35647E+00_JPRB, 0.36903E+00_JPRB ,&
&  0.37283E+00_JPRB, 0.39548E+00_JPRB, 0.41827E+00_JPRB, 0.45225E+00_JPRB, 0.49237E+00_JPRB, 0.53167E+00_JPRB, 0.56362E+00_JPRB ,&
&  0.59242E+00_JPRB, 0.62119E+00_JPRB, 0.64277E+00_JPRB, 0.65053E+00_JPRB, 0.66404E+00_JPRB, 0.68714E+00_JPRB, 0.71441E+00_JPRB ,&
&  0.75107E+00_JPRB, 0.80063E+00_JPRB, 0.87002E+00_JPRB, 0.95774E+00_JPRB, 0.10614E+01_JPRB, 0.11157E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.67057E+00_JPRB, 0.94829E+00_JPRB, 0.10839E+01_JPRB, 0.10079E+01_JPRB, 0.96783E+00_JPRB, 0.93992E+00_JPRB, 0.91862E+00_JPRB ,&
&  0.87806E+00_JPRB, 0.82492E+00_JPRB, 0.81494E+00_JPRB, 0.89925E+00_JPRB, 0.92002E+00_JPRB, 0.99351E+00_JPRB, 0.10584E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.11973E+01_JPRB, 0.12765E+01_JPRB, 0.13691E+01_JPRB, 0.14753E+01_JPRB, 0.15856E+01_JPRB, 0.17032E+01_JPRB ,&
&  0.18499E+01_JPRB, 0.19678E+01_JPRB, 0.20153E+01_JPRB, 0.15971E+01_JPRB, 0.14610E+01_JPRB, 0.13820E+01_JPRB, 0.12281E+01_JPRB ,&
&  0.79756E+00_JPRB, 0.62029E+00_JPRB, 0.61931E+00_JPRB, 0.63950E+00_JPRB, 0.61218E+00_JPRB, 0.58564E+00_JPRB, 0.55214E+00_JPRB ,&
&  0.51121E+00_JPRB, 0.47611E+00_JPRB, 0.45626E+00_JPRB, 0.44987E+00_JPRB, 0.46496E+00_JPRB, 0.46522E+00_JPRB, 0.45139E+00_JPRB ,&
&  0.43409E+00_JPRB, 0.43765E+00_JPRB, 0.46595E+00_JPRB, 0.48913E+00_JPRB, 0.52229E+00_JPRB, 0.54251E+00_JPRB, 0.55447E+00_JPRB ,&
&  0.56096E+00_JPRB, 0.55653E+00_JPRB, 0.57537E+00_JPRB, 0.59025E+00_JPRB, 0.59162E+00_JPRB, 0.58739E+00_JPRB, 0.57581E+00_JPRB ,&
&  0.56640E+00_JPRB, 0.58000E+00_JPRB, 0.58951E+00_JPRB, 0.60003E+00_JPRB, 0.59601E+00_JPRB, 0.58739E+00_JPRB, 0.58659E+00_JPRB ,&
&  0.57476E+00_JPRB, 0.55301E+00_JPRB, 0.52802E+00_JPRB, 0.49251E+00_JPRB, 0.45949E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.42215E+00_JPRB, 0.40899E+00_JPRB, 0.39248E+00_JPRB, 0.38296E+00_JPRB, 0.37454E+00_JPRB, 0.36528E+00_JPRB, 0.37436E+00_JPRB ,&
&  0.39954E+00_JPRB, 0.42206E+00_JPRB, 0.46038E+00_JPRB, 0.51774E+00_JPRB, 0.56846E+00_JPRB, 0.59469E+00_JPRB, 0.62405E+00_JPRB ,&
&  0.64934E+00_JPRB, 0.67789E+00_JPRB, 0.70050E+00_JPRB, 0.72054E+00_JPRB, 0.74790E+00_JPRB, 0.78287E+00_JPRB, 0.83038E+00_JPRB ,&
&  0.88826E+00_JPRB, 0.94689E+00_JPRB, 0.10219E+01_JPRB, 0.11178E+01_JPRB, 0.12084E+01_JPRB, 0.12400E+01_JPRB, 0.12798E+01_JPRB ,&
&  0.89795E+00_JPRB, 0.10522E+01_JPRB, 0.10293E+01_JPRB, 0.98328E+00_JPRB, 0.93452E+00_JPRB, 0.89418E+00_JPRB, 0.87384E+00_JPRB ,&
&  0.86454E+00_JPRB, 0.82314E+00_JPRB, 0.83393E+00_JPRB, 0.90854E+00_JPRB, 0.10506E+01_JPRB, 0.11379E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.12975E+01_JPRB, 0.13913E+01_JPRB, 0.14873E+01_JPRB, 0.15915E+01_JPRB, 0.16911E+01_JPRB, 0.18123E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.20043E+01_JPRB, 0.20787E+01_JPRB, 0.21071E+01_JPRB, 0.17312E+01_JPRB, 0.14034E+01_JPRB, 0.12240E+01_JPRB, 0.94591E+00_JPRB ,&
&  0.70214E+00_JPRB, 0.60616E+00_JPRB, 0.62566E+00_JPRB, 0.64667E+00_JPRB, 0.61008E+00_JPRB, 0.56951E+00_JPRB, 0.52322E+00_JPRB ,&
&  0.49844E+00_JPRB, 0.45516E+00_JPRB, 0.44009E+00_JPRB, 0.44882E+00_JPRB, 0.47213E+00_JPRB, 0.45395E+00_JPRB, 0.42831E+00_JPRB ,&
&  0.41032E+00_JPRB, 0.40224E+00_JPRB, 0.43617E+00_JPRB, 0.47565E+00_JPRB, 0.52735E+00_JPRB, 0.54252E+00_JPRB, 0.52779E+00_JPRB ,&
&  0.55951E+00_JPRB, 0.52583E+00_JPRB, 0.53345E+00_JPRB, 0.54903E+00_JPRB, 0.55901E+00_JPRB, 0.56774E+00_JPRB, 0.57647E+00_JPRB ,&
&  0.60810E+00_JPRB, 0.66931E+00_JPRB, 0.69025E+00_JPRB, 0.64294E+00_JPRB, 0.60332E+00_JPRB, 0.55956E+00_JPRB, 0.51472E+00_JPRB ,&
&  0.51711E+00_JPRB, 0.50489E+00_JPRB, 0.49209E+00_JPRB, 0.47373E+00_JPRB, 0.44884E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.47722E+00_JPRB, 0.46434E+00_JPRB, 0.45677E+00_JPRB, 0.45059E+00_JPRB, 0.43530E+00_JPRB, 0.43545E+00_JPRB, 0.43224E+00_JPRB ,&
&  0.44822E+00_JPRB, 0.46820E+00_JPRB, 0.50359E+00_JPRB, 0.54628E+00_JPRB, 0.60111E+00_JPRB, 0.63836E+00_JPRB, 0.68166E+00_JPRB ,&
&  0.70595E+00_JPRB, 0.73815E+00_JPRB, 0.77666E+00_JPRB, 0.80680E+00_JPRB, 0.84436E+00_JPRB, 0.86826E+00_JPRB, 0.91341E+00_JPRB ,&
&  0.95462E+00_JPRB, 0.10038E+01_JPRB, 0.10724E+01_JPRB, 0.11338E+01_JPRB, 0.11867E+01_JPRB, 0.12006E+01_JPRB, 0.92871E+00_JPRB ,&
&  0.10402E+01_JPRB, 0.10353E+01_JPRB, 0.10129E+01_JPRB, 0.95988E+00_JPRB, 0.89814E+00_JPRB, 0.84598E+00_JPRB, 0.83473E+00_JPRB ,&
&  0.83819E+00_JPRB, 0.84439E+00_JPRB, 0.87135E+00_JPRB, 0.92363E+00_JPRB, 0.11255E+01_JPRB, 0.12358E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.14039E+01_JPRB, 0.15022E+01_JPRB, 0.16056E+01_JPRB, 0.17204E+01_JPRB, 0.18127E+01_JPRB, 0.19148E+01_JPRB, 0.20181E+01_JPRB ,&
&  0.20830E+01_JPRB, 0.21421E+01_JPRB, 0.20603E+01_JPRB, 0.16088E+01_JPRB, 0.13445E+01_JPRB, 0.11268E+01_JPRB, 0.84058E+00_JPRB ,&
&  0.69259E+00_JPRB, 0.65064E+00_JPRB, 0.67191E+00_JPRB, 0.68345E+00_JPRB, 0.63082E+00_JPRB, 0.57727E+00_JPRB, 0.54168E+00_JPRB ,&
&  0.49439E+00_JPRB, 0.48443E+00_JPRB, 0.47404E+00_JPRB, 0.49559E+00_JPRB, 0.50594E+00_JPRB, 0.48532E+00_JPRB, 0.47204E+00_JPRB ,&
&  0.44920E+00_JPRB, 0.42968E+00_JPRB, 0.44984E+00_JPRB, 0.48658E+00_JPRB, 0.56709E+00_JPRB, 0.63344E+00_JPRB, 0.63444E+00_JPRB ,&
&  0.67223E+00_JPRB, 0.45883E+00_JPRB, 0.48641E+00_JPRB, 0.51633E+00_JPRB, 0.56757E+00_JPRB, 0.60999E+00_JPRB, 0.69761E+00_JPRB ,&
&  0.88853E+00_JPRB, 0.12312E+01_JPRB, 0.10975E+01_JPRB, 0.83612E+00_JPRB, 0.92544E+00_JPRB, 0.84781E+00_JPRB, 0.45981E+00_JPRB ,&
&  0.46156E+00_JPRB, 0.46227E+00_JPRB, 0.47765E+00_JPRB, 0.48217E+00_JPRB, 0.48518E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.55412E+00_JPRB, 0.54864E+00_JPRB, 0.54792E+00_JPRB, 0.53858E+00_JPRB, 0.53355E+00_JPRB, 0.53418E+00_JPRB, 0.52194E+00_JPRB ,&
&  0.53113E+00_JPRB, 0.54728E+00_JPRB, 0.55331E+00_JPRB, 0.57079E+00_JPRB, 0.58111E+00_JPRB, 0.61569E+00_JPRB, 0.66575E+00_JPRB ,&
&  0.69873E+00_JPRB, 0.77310E+00_JPRB, 0.80801E+00_JPRB, 0.85096E+00_JPRB, 0.88657E+00_JPRB, 0.92338E+00_JPRB, 0.95498E+00_JPRB ,&
&  0.10007E+01_JPRB, 0.10549E+01_JPRB, 0.11146E+01_JPRB, 0.11545E+01_JPRB, 0.11551E+01_JPRB, 0.12777E+01_JPRB, 0.97109E+00_JPRB ,&
&  0.10464E+01_JPRB, 0.10245E+01_JPRB, 0.94084E+00_JPRB, 0.89353E+00_JPRB, 0.84892E+00_JPRB, 0.82135E+00_JPRB, 0.80388E+00_JPRB ,&
&  0.81006E+00_JPRB, 0.83004E+00_JPRB, 0.87479E+00_JPRB, 0.98096E+00_JPRB, 0.11106E+01_JPRB, 0.12313E+01_JPRB, 0.13088E+01_JPRB ,&
&  0.13895E+01_JPRB, 0.14606E+01_JPRB, 0.15568E+01_JPRB, 0.16623E+01_JPRB, 0.17753E+01_JPRB, 0.18541E+01_JPRB, 0.19314E+01_JPRB ,&
&  0.19608E+01_JPRB, 0.19840E+01_JPRB, 0.18420E+01_JPRB, 0.16075E+01_JPRB, 0.13688E+01_JPRB, 0.11322E+01_JPRB, 0.89979E+00_JPRB ,&
&  0.72830E+00_JPRB, 0.69116E+00_JPRB, 0.70633E+00_JPRB, 0.74638E+00_JPRB, 0.68404E+00_JPRB, 0.63863E+00_JPRB, 0.61131E+00_JPRB ,&
&  0.58008E+00_JPRB, 0.53307E+00_JPRB, 0.54728E+00_JPRB, 0.58393E+00_JPRB, 0.59408E+00_JPRB, 0.58131E+00_JPRB, 0.55841E+00_JPRB ,&
&  0.53286E+00_JPRB, 0.49902E+00_JPRB, 0.49289E+00_JPRB, 0.49437E+00_JPRB, 0.50251E+00_JPRB, 0.52437E+00_JPRB, 0.45909E+00_JPRB ,&
&  0.50815E+00_JPRB, 0.45279E+00_JPRB, 0.47395E+00_JPRB, 0.52384E+00_JPRB, 0.61352E+00_JPRB, 0.78879E+00_JPRB, 0.11138E+01_JPRB ,&
&  0.16988E+01_JPRB, 0.19592E+01_JPRB, 0.22158E+01_JPRB, 0.27689E+01_JPRB, 0.24194E+01_JPRB, 0.17795E+01_JPRB, 0.51423E+00_JPRB ,&
&  0.51137E+00_JPRB, 0.51552E+00_JPRB, 0.53686E+00_JPRB, 0.54527E+00_JPRB, 0.54999E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.59536E+00_JPRB, 0.60412E+00_JPRB, 0.61603E+00_JPRB, 0.62872E+00_JPRB, 0.64420E+00_JPRB, 0.65977E+00_JPRB, 0.67186E+00_JPRB ,&
&  0.68465E+00_JPRB, 0.70064E+00_JPRB, 0.71601E+00_JPRB, 0.72668E+00_JPRB, 0.73742E+00_JPRB, 0.75841E+00_JPRB, 0.78310E+00_JPRB ,&
&  0.79261E+00_JPRB, 0.83234E+00_JPRB, 0.84919E+00_JPRB, 0.88595E+00_JPRB, 0.91924E+00_JPRB, 0.96074E+00_JPRB, 0.99028E+00_JPRB ,&
&  0.10167E+01_JPRB, 0.10577E+01_JPRB, 0.10680E+01_JPRB, 0.10534E+01_JPRB, 0.10572E+01_JPRB, 0.11162E+01_JPRB, 0.85204E+00_JPRB ,&
&  0.10809E+01_JPRB, 0.10334E+01_JPRB, 0.90689E+00_JPRB, 0.84333E+00_JPRB, 0.79409E+00_JPRB, 0.77144E+00_JPRB, 0.76289E+00_JPRB ,&
&  0.76462E+00_JPRB, 0.80635E+00_JPRB, 0.80889E+00_JPRB, 0.84436E+00_JPRB, 0.96689E+00_JPRB, 0.11144E+01_JPRB, 0.12196E+01_JPRB ,&
&  0.12495E+01_JPRB, 0.13201E+01_JPRB, 0.13420E+01_JPRB, 0.13749E+01_JPRB, 0.14454E+01_JPRB, 0.15123E+01_JPRB, 0.15485E+01_JPRB ,&
&  0.15216E+01_JPRB, 0.14741E+01_JPRB, 0.13386E+01_JPRB, 0.14044E+01_JPRB, 0.13381E+01_JPRB, 0.11388E+01_JPRB, 0.92963E+00_JPRB ,&
&  0.70679E+00_JPRB, 0.73871E+00_JPRB, 0.79299E+00_JPRB, 0.87452E+00_JPRB, 0.83203E+00_JPRB, 0.78974E+00_JPRB, 0.70124E+00_JPRB ,&
&  0.66456E+00_JPRB, 0.65035E+00_JPRB, 0.67560E+00_JPRB, 0.71047E+00_JPRB, 0.73428E+00_JPRB, 0.72076E+00_JPRB, 0.67159E+00_JPRB ,&
&  0.62563E+00_JPRB, 0.57303E+00_JPRB, 0.54281E+00_JPRB, 0.52686E+00_JPRB, 0.49362E+00_JPRB, 0.42127E+00_JPRB, 0.46940E+00_JPRB ,&
&  0.43840E+00_JPRB, 0.41345E+00_JPRB, 0.48207E+00_JPRB, 0.39214E+00_JPRB, 0.57848E+00_JPRB, 0.74796E+00_JPRB, 0.10107E+01_JPRB ,&
&  0.12172E+01_JPRB, 0.17258E+01_JPRB, 0.21824E+01_JPRB, 0.16564E+01_JPRB, 0.11376E+01_JPRB, 0.64298E+00_JPRB, 0.55261E+00_JPRB ,&
&  0.56009E+00_JPRB, 0.56512E+00_JPRB, 0.57361E+00_JPRB, 0.57362E+00_JPRB, 0.58490E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.61174E+00_JPRB, 0.62409E+00_JPRB, 0.64087E+00_JPRB, 0.66345E+00_JPRB, 0.68485E+00_JPRB, 0.70094E+00_JPRB, 0.71135E+00_JPRB ,&
&  0.72952E+00_JPRB, 0.75408E+00_JPRB, 0.76848E+00_JPRB, 0.77567E+00_JPRB, 0.79166E+00_JPRB, 0.80661E+00_JPRB, 0.83314E+00_JPRB ,&
&  0.85027E+00_JPRB, 0.88824E+00_JPRB, 0.91768E+00_JPRB, 0.94783E+00_JPRB, 0.97186E+00_JPRB, 0.10007E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10497E+01_JPRB, 0.10936E+01_JPRB, 0.11322E+01_JPRB, 0.11587E+01_JPRB, 0.11989E+01_JPRB, 0.12285E+01_JPRB, 0.97632E+00_JPRB ,&
&  0.11917E+01_JPRB, 0.11108E+01_JPRB, 0.95917E+00_JPRB, 0.84722E+00_JPRB, 0.77398E+00_JPRB, 0.71702E+00_JPRB, 0.68515E+00_JPRB ,&
&  0.67766E+00_JPRB, 0.69702E+00_JPRB, 0.71433E+00_JPRB, 0.70531E+00_JPRB, 0.70154E+00_JPRB, 0.78022E+00_JPRB, 0.93774E+00_JPRB ,&
&  0.10296E+01_JPRB, 0.10704E+01_JPRB, 0.10966E+01_JPRB, 0.10872E+01_JPRB, 0.11269E+01_JPRB, 0.11840E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.11047E+01_JPRB, 0.96940E+00_JPRB, 0.11510E+01_JPRB, 0.11711E+01_JPRB, 0.11351E+01_JPRB, 0.10247E+01_JPRB ,&
&  0.88666E+00_JPRB, 0.82254E+00_JPRB, 0.87453E+00_JPRB, 0.98944E+00_JPRB, 0.96768E+00_JPRB, 0.94251E+00_JPRB, 0.87753E+00_JPRB ,&
&  0.83012E+00_JPRB, 0.80436E+00_JPRB, 0.81168E+00_JPRB, 0.83666E+00_JPRB, 0.85096E+00_JPRB, 0.83916E+00_JPRB, 0.77191E+00_JPRB ,&
&  0.71037E+00_JPRB, 0.65208E+00_JPRB, 0.61097E+00_JPRB, 0.57145E+00_JPRB, 0.52774E+00_JPRB, 0.43793E+00_JPRB, 0.47048E+00_JPRB ,&
&  0.42510E+00_JPRB, 0.34102E+00_JPRB, 0.39970E+00_JPRB, 0.47596E+00_JPRB, 0.52022E+00_JPRB, 0.59409E+00_JPRB, 0.75485E+00_JPRB ,&
&  0.96433E+00_JPRB, 0.11092E+01_JPRB, 0.10850E+01_JPRB, 0.82186E+00_JPRB, 0.64268E+00_JPRB, 0.58621E+00_JPRB, 0.58344E+00_JPRB ,&
&  0.59159E+00_JPRB, 0.58988E+00_JPRB, 0.59255E+00_JPRB, 0.59723E+00_JPRB, 0.60544E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.59058E+00_JPRB, 0.58977E+00_JPRB, 0.59691E+00_JPRB, 0.61095E+00_JPRB, 0.61745E+00_JPRB, 0.62178E+00_JPRB, 0.62417E+00_JPRB ,&
&  0.64044E+00_JPRB, 0.65927E+00_JPRB, 0.67676E+00_JPRB, 0.69412E+00_JPRB, 0.70935E+00_JPRB, 0.72267E+00_JPRB, 0.75638E+00_JPRB ,&
&  0.77374E+00_JPRB, 0.79839E+00_JPRB, 0.81578E+00_JPRB, 0.83412E+00_JPRB, 0.84967E+00_JPRB, 0.84423E+00_JPRB, 0.84978E+00_JPRB ,&
&  0.86052E+00_JPRB, 0.88904E+00_JPRB, 0.91942E+00_JPRB, 0.97016E+00_JPRB, 0.10362E+01_JPRB, 0.10477E+01_JPRB, 0.11677E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.12423E+01_JPRB, 0.10621E+01_JPRB, 0.89967E+00_JPRB, 0.85538E+00_JPRB, 0.76486E+00_JPRB, 0.71205E+00_JPRB ,&
&  0.70602E+00_JPRB, 0.68355E+00_JPRB, 0.65408E+00_JPRB, 0.62004E+00_JPRB, 0.58450E+00_JPRB, 0.60322E+00_JPRB, 0.64369E+00_JPRB ,&
&  0.76313E+00_JPRB, 0.83727E+00_JPRB, 0.93284E+00_JPRB, 0.10213E+01_JPRB, 0.11211E+01_JPRB, 0.11593E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.11361E+01_JPRB, 0.10852E+01_JPRB, 0.99114E+00_JPRB, 0.11118E+01_JPRB, 0.12138E+01_JPRB, 0.12626E+01_JPRB, 0.12587E+01_JPRB ,&
&  0.12063E+01_JPRB, 0.11010E+01_JPRB, 0.11056E+01_JPRB, 0.11147E+01_JPRB, 0.10832E+01_JPRB, 0.10856E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.97041E+00_JPRB, 0.91988E+00_JPRB, 0.89571E+00_JPRB, 0.89233E+00_JPRB, 0.88706E+00_JPRB, 0.86850E+00_JPRB, 0.78516E+00_JPRB ,&
&  0.72955E+00_JPRB, 0.68595E+00_JPRB, 0.65109E+00_JPRB, 0.60733E+00_JPRB, 0.49397E+00_JPRB, 0.46169E+00_JPRB, 0.47041E+00_JPRB ,&
&  0.43647E+00_JPRB, 0.39637E+00_JPRB, 0.39566E+00_JPRB, 0.50610E+00_JPRB, 0.53409E+00_JPRB, 0.57588E+00_JPRB, 0.64363E+00_JPRB ,&
&  0.65773E+00_JPRB, 0.64498E+00_JPRB, 0.58195E+00_JPRB, 0.52065E+00_JPRB, 0.53503E+00_JPRB, 0.53812E+00_JPRB, 0.54036E+00_JPRB ,&
&  0.55457E+00_JPRB, 0.56246E+00_JPRB, 0.57279E+00_JPRB, 0.59203E+00_JPRB, 0.58660E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.57711E+00_JPRB, 0.56619E+00_JPRB, 0.55578E+00_JPRB, 0.54528E+00_JPRB, 0.55135E+00_JPRB, 0.55214E+00_JPRB, 0.56093E+00_JPRB ,&
&  0.57095E+00_JPRB, 0.56952E+00_JPRB, 0.57347E+00_JPRB, 0.57459E+00_JPRB, 0.58754E+00_JPRB, 0.59921E+00_JPRB, 0.62773E+00_JPRB ,&
&  0.65481E+00_JPRB, 0.66933E+00_JPRB, 0.68916E+00_JPRB, 0.70870E+00_JPRB, 0.72271E+00_JPRB, 0.73798E+00_JPRB, 0.72441E+00_JPRB ,&
&  0.70846E+00_JPRB, 0.71112E+00_JPRB, 0.74925E+00_JPRB, 0.87999E+00_JPRB, 0.11259E+01_JPRB, 0.14417E+01_JPRB, 0.20469E+01_JPRB ,&
&  0.19467E+01_JPRB, 0.12862E+01_JPRB, 0.11105E+01_JPRB, 0.96672E+00_JPRB, 0.10038E+01_JPRB, 0.10746E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10407E+01_JPRB, 0.10290E+01_JPRB, 0.10115E+01_JPRB, 0.98423E+00_JPRB, 0.92785E+00_JPRB, 0.85539E+00_JPRB, 0.80357E+00_JPRB ,&
&  0.78558E+00_JPRB, 0.76309E+00_JPRB, 0.87007E+00_JPRB, 0.11334E+01_JPRB, 0.13137E+01_JPRB, 0.16132E+01_JPRB, 0.15481E+01_JPRB ,&
&  0.15587E+01_JPRB, 0.15183E+01_JPRB, 0.14462E+01_JPRB, 0.15574E+01_JPRB, 0.16027E+01_JPRB, 0.16590E+01_JPRB, 0.16489E+01_JPRB ,&
&  0.17042E+01_JPRB, 0.14965E+01_JPRB, 0.10899E+01_JPRB, 0.12314E+01_JPRB, 0.12054E+01_JPRB, 0.12184E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.10111E+01_JPRB, 0.95495E+00_JPRB, 0.91009E+00_JPRB, 0.87872E+00_JPRB, 0.83767E+00_JPRB, 0.74977E+00_JPRB ,&
&  0.71479E+00_JPRB, 0.69057E+00_JPRB, 0.66901E+00_JPRB, 0.61631E+00_JPRB, 0.57253E+00_JPRB, 0.42561E+00_JPRB, 0.45335E+00_JPRB ,&
&  0.49083E+00_JPRB, 0.51069E+00_JPRB, 0.48944E+00_JPRB, 0.61412E+00_JPRB, 0.58944E+00_JPRB, 0.54944E+00_JPRB, 0.55280E+00_JPRB ,&
&  0.53171E+00_JPRB, 0.51233E+00_JPRB, 0.49712E+00_JPRB, 0.49854E+00_JPRB, 0.51240E+00_JPRB, 0.53670E+00_JPRB, 0.54126E+00_JPRB ,&
&  0.54997E+00_JPRB, 0.56219E+00_JPRB, 0.57943E+00_JPRB, 0.59909E+00_JPRB, 0.59540E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.60881E+00_JPRB, 0.60933E+00_JPRB, 0.61070E+00_JPRB, 0.61835E+00_JPRB, 0.61975E+00_JPRB, 0.61594E+00_JPRB, 0.60341E+00_JPRB ,&
&  0.59470E+00_JPRB, 0.57917E+00_JPRB, 0.56936E+00_JPRB, 0.56268E+00_JPRB, 0.57534E+00_JPRB, 0.60673E+00_JPRB, 0.65453E+00_JPRB ,&
&  0.66869E+00_JPRB, 0.66700E+00_JPRB, 0.64232E+00_JPRB, 0.62849E+00_JPRB, 0.65252E+00_JPRB, 0.65755E+00_JPRB, 0.63663E+00_JPRB ,&
&  0.62099E+00_JPRB, 0.68742E+00_JPRB, 0.86394E+00_JPRB, 0.11266E+01_JPRB, 0.15880E+01_JPRB, 0.18464E+01_JPRB, 0.23267E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.14633E+01_JPRB, 0.14541E+01_JPRB, 0.13528E+01_JPRB, 0.14263E+01_JPRB, 0.14713E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.15224E+01_JPRB, 0.15509E+01_JPRB, 0.15779E+01_JPRB, 0.16123E+01_JPRB, 0.16266E+01_JPRB, 0.16059E+01_JPRB, 0.15851E+01_JPRB ,&
&  0.15650E+01_JPRB, 0.15229E+01_JPRB, 0.15422E+01_JPRB, 0.16077E+01_JPRB, 0.17828E+01_JPRB, 0.19753E+01_JPRB, 0.20002E+01_JPRB ,&
&  0.20187E+01_JPRB, 0.20989E+01_JPRB, 0.20162E+01_JPRB, 0.21364E+01_JPRB, 0.21676E+01_JPRB, 0.21015E+01_JPRB, 0.22389E+01_JPRB ,&
&  0.22850E+01_JPRB, 0.19067E+01_JPRB, 0.12592E+01_JPRB, 0.13502E+01_JPRB, 0.12961E+01_JPRB, 0.12189E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.10789E+01_JPRB, 0.10064E+01_JPRB, 0.95898E+00_JPRB, 0.91628E+00_JPRB, 0.84402E+00_JPRB, 0.80857E+00_JPRB ,&
&  0.74232E+00_JPRB, 0.70631E+00_JPRB, 0.67573E+00_JPRB, 0.63308E+00_JPRB, 0.58257E+00_JPRB, 0.54454E+00_JPRB, 0.54994E+00_JPRB ,&
&  0.59084E+00_JPRB, 0.60235E+00_JPRB, 0.68441E+00_JPRB, 0.74526E+00_JPRB, 0.60675E+00_JPRB, 0.61230E+00_JPRB, 0.59529E+00_JPRB ,&
&  0.59429E+00_JPRB, 0.58883E+00_JPRB, 0.58561E+00_JPRB, 0.57838E+00_JPRB, 0.60229E+00_JPRB, 0.63857E+00_JPRB, 0.63807E+00_JPRB ,&
&  0.64488E+00_JPRB, 0.62310E+00_JPRB, 0.62234E+00_JPRB, 0.62710E+00_JPRB, 0.62912E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.72960E+00_JPRB, 0.72677E+00_JPRB, 0.71465E+00_JPRB, 0.72560E+00_JPRB, 0.73831E+00_JPRB, 0.73556E+00_JPRB, 0.75044E+00_JPRB ,&
&  0.77067E+00_JPRB, 0.79019E+00_JPRB, 0.80295E+00_JPRB, 0.81214E+00_JPRB, 0.86214E+00_JPRB, 0.92298E+00_JPRB, 0.99432E+00_JPRB ,&
&  0.10349E+01_JPRB, 0.10731E+01_JPRB, 0.10832E+01_JPRB, 0.10695E+01_JPRB, 0.10638E+01_JPRB, 0.10290E+01_JPRB, 0.98781E+00_JPRB ,&
&  0.10619E+01_JPRB, 0.13685E+01_JPRB, 0.18671E+01_JPRB, 0.23254E+01_JPRB, 0.27248E+01_JPRB, 0.23746E+01_JPRB, 0.25713E+01_JPRB ,&
&  0.22305E+01_JPRB, 0.16764E+01_JPRB, 0.15679E+01_JPRB, 0.16026E+01_JPRB, 0.16698E+01_JPRB, 0.16744E+01_JPRB, 0.17106E+01_JPRB ,&
&  0.17307E+01_JPRB, 0.17494E+01_JPRB, 0.17827E+01_JPRB, 0.18615E+01_JPRB, 0.19186E+01_JPRB, 0.19591E+01_JPRB, 0.20128E+01_JPRB ,&
&  0.20505E+01_JPRB, 0.20461E+01_JPRB, 0.20364E+01_JPRB, 0.18805E+01_JPRB, 0.19564E+01_JPRB, 0.21144E+01_JPRB, 0.22531E+01_JPRB ,&
&  0.23310E+01_JPRB, 0.23395E+01_JPRB, 0.25091E+01_JPRB, 0.25408E+01_JPRB, 0.25154E+01_JPRB, 0.24752E+01_JPRB, 0.25148E+01_JPRB ,&
&  0.24904E+01_JPRB, 0.22319E+01_JPRB, 0.15584E+01_JPRB, 0.17012E+01_JPRB, 0.15735E+01_JPRB, 0.12950E+01_JPRB, 0.13304E+01_JPRB ,&
&  0.12273E+01_JPRB, 0.11097E+01_JPRB, 0.99914E+00_JPRB, 0.97346E+00_JPRB, 0.97973E+00_JPRB, 0.98287E+00_JPRB, 0.99108E+00_JPRB ,&
&  0.90167E+00_JPRB, 0.79219E+00_JPRB, 0.67808E+00_JPRB, 0.65432E+00_JPRB, 0.66959E+00_JPRB, 0.74052E+00_JPRB, 0.11041E+01_JPRB ,&
&  0.91425E+00_JPRB, 0.74004E+00_JPRB, 0.75413E+00_JPRB, 0.80866E+00_JPRB, 0.70705E+00_JPRB, 0.70192E+00_JPRB, 0.71439E+00_JPRB ,&
&  0.72515E+00_JPRB, 0.73271E+00_JPRB, 0.73856E+00_JPRB, 0.73868E+00_JPRB, 0.74890E+00_JPRB, 0.76836E+00_JPRB, 0.77980E+00_JPRB ,&
&  0.78298E+00_JPRB, 0.76511E+00_JPRB, 0.75024E+00_JPRB, 0.73890E+00_JPRB, 0.72928E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.81926E+00_JPRB, 0.78596E+00_JPRB, 0.75951E+00_JPRB, 0.74732E+00_JPRB, 0.74524E+00_JPRB, 0.75457E+00_JPRB, 0.77461E+00_JPRB ,&
&  0.81032E+00_JPRB, 0.86066E+00_JPRB, 0.91301E+00_JPRB, 0.95927E+00_JPRB, 0.10272E+01_JPRB, 0.11137E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.13416E+01_JPRB, 0.14702E+01_JPRB, 0.15409E+01_JPRB, 0.15707E+01_JPRB, 0.16364E+01_JPRB, 0.17432E+01_JPRB, 0.19889E+01_JPRB ,&
&  0.23390E+01_JPRB, 0.26335E+01_JPRB, 0.28002E+01_JPRB, 0.25701E+01_JPRB, 0.19946E+01_JPRB, 0.19697E+01_JPRB, 0.22234E+01_JPRB ,&
&  0.22628E+01_JPRB, 0.19050E+01_JPRB, 0.16289E+01_JPRB, 0.16106E+01_JPRB, 0.16216E+01_JPRB, 0.16683E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.17094E+01_JPRB, 0.17329E+01_JPRB, 0.17975E+01_JPRB, 0.18690E+01_JPRB, 0.19759E+01_JPRB, 0.20882E+01_JPRB, 0.22100E+01_JPRB ,&
&  0.22467E+01_JPRB, 0.22930E+01_JPRB, 0.22524E+01_JPRB, 0.21616E+01_JPRB, 0.21450E+01_JPRB, 0.22262E+01_JPRB, 0.23618E+01_JPRB ,&
&  0.24949E+01_JPRB, 0.25003E+01_JPRB, 0.26060E+01_JPRB, 0.26740E+01_JPRB, 0.25609E+01_JPRB, 0.23510E+01_JPRB, 0.22683E+01_JPRB ,&
&  0.22660E+01_JPRB, 0.24301E+01_JPRB, 0.17335E+01_JPRB, 0.20905E+01_JPRB, 0.21683E+01_JPRB, 0.18063E+01_JPRB, 0.15206E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.12770E+01_JPRB, 0.11657E+01_JPRB, 0.11939E+01_JPRB, 0.11892E+01_JPRB, 0.11964E+01_JPRB, 0.11220E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.92481E+00_JPRB, 0.73936E+00_JPRB, 0.64031E+00_JPRB, 0.72464E+00_JPRB, 0.86773E+00_JPRB, 0.10774E+01_JPRB ,&
&  0.97851E+00_JPRB, 0.99907E+00_JPRB, 0.10617E+01_JPRB, 0.12757E+01_JPRB, 0.14103E+01_JPRB, 0.74289E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.78452E+00_JPRB, 0.81914E+00_JPRB, 0.82881E+00_JPRB, 0.83675E+00_JPRB, 0.84518E+00_JPRB, 0.86947E+00_JPRB, 0.90066E+00_JPRB ,&
&  0.91313E+00_JPRB, 0.91437E+00_JPRB, 0.91120E+00_JPRB, 0.88838E+00_JPRB, 0.85083E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.91283E+00_JPRB, 0.89576E+00_JPRB, 0.87921E+00_JPRB, 0.85491E+00_JPRB, 0.82102E+00_JPRB, 0.79599E+00_JPRB, 0.79477E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.83744E+00_JPRB, 0.90304E+00_JPRB, 0.98467E+00_JPRB, 0.10917E+01_JPRB, 0.12152E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.15156E+01_JPRB, 0.16891E+01_JPRB, 0.18038E+01_JPRB, 0.18983E+01_JPRB, 0.20103E+01_JPRB, 0.21937E+01_JPRB, 0.24079E+01_JPRB ,&
&  0.25806E+01_JPRB, 0.25729E+01_JPRB, 0.24047E+01_JPRB, 0.17434E+01_JPRB, 0.14624E+01_JPRB, 0.17111E+01_JPRB, 0.17731E+01_JPRB ,&
&  0.17266E+01_JPRB, 0.16231E+01_JPRB, 0.15764E+01_JPRB, 0.15505E+01_JPRB, 0.15583E+01_JPRB, 0.16020E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.16373E+01_JPRB, 0.16514E+01_JPRB, 0.17040E+01_JPRB, 0.18169E+01_JPRB, 0.19777E+01_JPRB, 0.20624E+01_JPRB, 0.21561E+01_JPRB ,&
&  0.22284E+01_JPRB, 0.23405E+01_JPRB, 0.23783E+01_JPRB, 0.23792E+01_JPRB, 0.23195E+01_JPRB, 0.23311E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.25239E+01_JPRB, 0.25635E+01_JPRB, 0.26385E+01_JPRB, 0.27406E+01_JPRB, 0.27442E+01_JPRB, 0.23666E+01_JPRB, 0.22034E+01_JPRB ,&
&  0.21885E+01_JPRB, 0.21697E+01_JPRB, 0.19844E+01_JPRB, 0.25796E+01_JPRB, 0.16965E+01_JPRB, 0.17425E+01_JPRB, 0.16968E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.15643E+01_JPRB, 0.15805E+01_JPRB, 0.15780E+01_JPRB, 0.15085E+01_JPRB, 0.12607E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.97675E+00_JPRB, 0.80056E+00_JPRB, 0.63250E+00_JPRB, 0.75695E+00_JPRB, 0.96733E+00_JPRB, 0.10304E+01_JPRB ,&
&  0.96326E+00_JPRB, 0.10329E+01_JPRB, 0.11630E+01_JPRB, 0.12377E+01_JPRB, 0.11988E+01_JPRB, 0.76869E+00_JPRB, 0.76976E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.81489E+00_JPRB, 0.83338E+00_JPRB, 0.83944E+00_JPRB, 0.86276E+00_JPRB, 0.90213E+00_JPRB, 0.93823E+00_JPRB ,&
&  0.94871E+00_JPRB, 0.94261E+00_JPRB, 0.95214E+00_JPRB, 0.94582E+00_JPRB, 0.93109E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.96734E+00_JPRB, 0.96481E+00_JPRB, 0.97837E+00_JPRB, 0.98010E+00_JPRB, 0.97460E+00_JPRB, 0.91800E+00_JPRB, 0.87349E+00_JPRB ,&
&  0.81900E+00_JPRB, 0.76861E+00_JPRB, 0.82878E+00_JPRB, 0.90512E+00_JPRB, 0.99673E+00_JPRB, 0.10946E+01_JPRB, 0.12000E+01_JPRB ,&
&  0.13388E+01_JPRB, 0.15114E+01_JPRB, 0.17246E+01_JPRB, 0.18891E+01_JPRB, 0.20407E+01_JPRB, 0.22415E+01_JPRB, 0.24053E+01_JPRB ,&
&  0.20136E+01_JPRB, 0.14122E+01_JPRB, 0.17731E+01_JPRB, 0.18736E+01_JPRB, 0.15902E+01_JPRB, 0.14844E+01_JPRB, 0.14715E+01_JPRB ,&
&  0.15229E+01_JPRB, 0.15451E+01_JPRB, 0.15458E+01_JPRB, 0.15404E+01_JPRB, 0.14959E+01_JPRB, 0.15257E+01_JPRB, 0.15675E+01_JPRB ,&
&  0.15924E+01_JPRB, 0.15939E+01_JPRB, 0.16072E+01_JPRB, 0.16958E+01_JPRB, 0.18514E+01_JPRB, 0.19121E+01_JPRB, 0.20440E+01_JPRB ,&
&  0.21700E+01_JPRB, 0.23342E+01_JPRB, 0.23679E+01_JPRB, 0.23025E+01_JPRB, 0.23317E+01_JPRB, 0.24024E+01_JPRB, 0.24851E+01_JPRB ,&
&  0.25760E+01_JPRB, 0.26135E+01_JPRB, 0.26857E+01_JPRB, 0.28175E+01_JPRB, 0.28426E+01_JPRB, 0.24805E+01_JPRB, 0.24355E+01_JPRB ,&
&  0.23450E+01_JPRB, 0.22040E+01_JPRB, 0.21120E+01_JPRB, 0.24993E+01_JPRB, 0.19088E+01_JPRB, 0.18997E+01_JPRB, 0.16600E+01_JPRB ,&
&  0.17611E+01_JPRB, 0.17809E+01_JPRB, 0.17733E+01_JPRB, 0.17105E+01_JPRB, 0.15965E+01_JPRB, 0.13520E+01_JPRB, 0.13005E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.10458E+01_JPRB, 0.83204E+00_JPRB, 0.70206E+00_JPRB, 0.68674E+00_JPRB, 0.85367E+00_JPRB, 0.10185E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.10340E+01_JPRB, 0.10776E+01_JPRB, 0.11340E+01_JPRB, 0.10635E+01_JPRB, 0.83977E+00_JPRB, 0.80386E+00_JPRB ,&
&  0.81518E+00_JPRB, 0.81610E+00_JPRB, 0.78758E+00_JPRB, 0.79449E+00_JPRB, 0.83015E+00_JPRB, 0.88608E+00_JPRB, 0.91829E+00_JPRB ,&
&  0.92694E+00_JPRB, 0.91376E+00_JPRB, 0.91641E+00_JPRB, 0.92915E+00_JPRB, 0.96100E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.94687E+00_JPRB, 0.96596E+00_JPRB, 0.98427E+00_JPRB, 0.10200E+01_JPRB, 0.10728E+01_JPRB, 0.11440E+01_JPRB, 0.11435E+01_JPRB ,&
&  0.98789E+00_JPRB, 0.74214E+00_JPRB, 0.75508E+00_JPRB, 0.78258E+00_JPRB, 0.84031E+00_JPRB, 0.89330E+00_JPRB, 0.95717E+00_JPRB ,&
&  0.10541E+01_JPRB, 0.11811E+01_JPRB, 0.13975E+01_JPRB, 0.16575E+01_JPRB, 0.19051E+01_JPRB, 0.22036E+01_JPRB, 0.21047E+01_JPRB ,&
&  0.16064E+01_JPRB, 0.15270E+01_JPRB, 0.17629E+01_JPRB, 0.17079E+01_JPRB, 0.15565E+01_JPRB, 0.15991E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.15014E+01_JPRB, 0.14520E+01_JPRB, 0.14707E+01_JPRB, 0.14277E+01_JPRB, 0.13881E+01_JPRB, 0.14104E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.15108E+01_JPRB, 0.15508E+01_JPRB, 0.15753E+01_JPRB, 0.16106E+01_JPRB, 0.16866E+01_JPRB, 0.17442E+01_JPRB, 0.18844E+01_JPRB ,&
&  0.20628E+01_JPRB, 0.22198E+01_JPRB, 0.22692E+01_JPRB, 0.22199E+01_JPRB, 0.22936E+01_JPRB, 0.24498E+01_JPRB, 0.26075E+01_JPRB ,&
&  0.27237E+01_JPRB, 0.28825E+01_JPRB, 0.29606E+01_JPRB, 0.29365E+01_JPRB, 0.26945E+01_JPRB, 0.28869E+01_JPRB, 0.25999E+01_JPRB ,&
&  0.25548E+01_JPRB, 0.23338E+01_JPRB, 0.24942E+01_JPRB, 0.20115E+01_JPRB, 0.21744E+01_JPRB, 0.21416E+01_JPRB, 0.19795E+01_JPRB ,&
&  0.18866E+01_JPRB, 0.20376E+01_JPRB, 0.20031E+01_JPRB, 0.18221E+01_JPRB, 0.17236E+01_JPRB, 0.14308E+01_JPRB, 0.13740E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.11222E+01_JPRB, 0.87478E+00_JPRB, 0.71523E+00_JPRB, 0.59713E+00_JPRB, 0.65554E+00_JPRB, 0.88950E+00_JPRB ,&
&  0.12110E+01_JPRB, 0.12577E+01_JPRB, 0.11273E+01_JPRB, 0.10494E+01_JPRB, 0.97727E+00_JPRB, 0.85947E+00_JPRB, 0.85213E+00_JPRB ,&
&  0.86202E+00_JPRB, 0.83232E+00_JPRB, 0.78297E+00_JPRB, 0.78174E+00_JPRB, 0.80753E+00_JPRB, 0.86465E+00_JPRB, 0.88556E+00_JPRB ,&
&  0.91237E+00_JPRB, 0.91446E+00_JPRB, 0.90726E+00_JPRB, 0.90682E+00_JPRB, 0.92111E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.92010E+00_JPRB, 0.92085E+00_JPRB, 0.89877E+00_JPRB, 0.87251E+00_JPRB, 0.85457E+00_JPRB, 0.84374E+00_JPRB, 0.85632E+00_JPRB ,&
&  0.79083E+00_JPRB, 0.76518E+00_JPRB, 0.75260E+00_JPRB, 0.73600E+00_JPRB, 0.74333E+00_JPRB, 0.75731E+00_JPRB, 0.78571E+00_JPRB ,&
&  0.81991E+00_JPRB, 0.85374E+00_JPRB, 0.97773E+00_JPRB, 0.12365E+01_JPRB, 0.15173E+01_JPRB, 0.18534E+01_JPRB, 0.17337E+01_JPRB ,&
&  0.14193E+01_JPRB, 0.21299E+01_JPRB, 0.15865E+01_JPRB, 0.16407E+01_JPRB, 0.16641E+01_JPRB, 0.18132E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.14142E+01_JPRB, 0.13822E+01_JPRB, 0.14132E+01_JPRB, 0.13789E+01_JPRB, 0.13768E+01_JPRB, 0.13764E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.14346E+01_JPRB, 0.14757E+01_JPRB, 0.15007E+01_JPRB, 0.15297E+01_JPRB, 0.15803E+01_JPRB, 0.16249E+01_JPRB, 0.17045E+01_JPRB ,&
&  0.18580E+01_JPRB, 0.20508E+01_JPRB, 0.21309E+01_JPRB, 0.22118E+01_JPRB, 0.23227E+01_JPRB, 0.25863E+01_JPRB, 0.27966E+01_JPRB ,&
&  0.28324E+01_JPRB, 0.28293E+01_JPRB, 0.31295E+01_JPRB, 0.32052E+01_JPRB, 0.31696E+01_JPRB, 0.32867E+01_JPRB, 0.28601E+01_JPRB ,&
&  0.28728E+01_JPRB, 0.25390E+01_JPRB, 0.26821E+01_JPRB, 0.22235E+01_JPRB, 0.22973E+01_JPRB, 0.24437E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.20194E+01_JPRB, 0.22009E+01_JPRB, 0.21510E+01_JPRB, 0.19894E+01_JPRB, 0.17848E+01_JPRB, 0.16095E+01_JPRB, 0.14318E+01_JPRB ,&
&  0.12378E+01_JPRB, 0.11717E+01_JPRB, 0.10956E+01_JPRB, 0.79352E+00_JPRB, 0.70770E+00_JPRB, 0.80798E+00_JPRB, 0.99805E+00_JPRB ,&
&  0.13477E+01_JPRB, 0.15019E+01_JPRB, 0.16728E+01_JPRB, 0.13868E+01_JPRB, 0.10754E+01_JPRB, 0.93052E+00_JPRB, 0.91024E+00_JPRB ,&
&  0.88773E+00_JPRB, 0.87837E+00_JPRB, 0.83384E+00_JPRB, 0.80877E+00_JPRB, 0.83149E+00_JPRB, 0.87725E+00_JPRB, 0.90377E+00_JPRB ,&
&  0.94298E+00_JPRB, 0.96599E+00_JPRB, 0.95840E+00_JPRB, 0.93371E+00_JPRB, 0.92527E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.97500E+00_JPRB, 0.96078E+00_JPRB, 0.94709E+00_JPRB, 0.91026E+00_JPRB, 0.87472E+00_JPRB, 0.83491E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.80733E+00_JPRB, 0.78808E+00_JPRB, 0.77374E+00_JPRB, 0.74388E+00_JPRB, 0.72947E+00_JPRB, 0.71100E+00_JPRB, 0.72028E+00_JPRB ,&
&  0.73150E+00_JPRB, 0.70485E+00_JPRB, 0.73531E+00_JPRB, 0.88725E+00_JPRB, 0.11140E+01_JPRB, 0.15068E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.20262E+01_JPRB, 0.16564E+01_JPRB, 0.16486E+01_JPRB, 0.17129E+01_JPRB, 0.17478E+01_JPRB, 0.14593E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.13024E+01_JPRB, 0.13241E+01_JPRB, 0.13620E+01_JPRB, 0.14162E+01_JPRB, 0.14677E+01_JPRB, 0.14544E+01_JPRB ,&
&  0.14643E+01_JPRB, 0.14895E+01_JPRB, 0.14992E+01_JPRB, 0.14966E+01_JPRB, 0.15138E+01_JPRB, 0.15488E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.16661E+01_JPRB, 0.18160E+01_JPRB, 0.20527E+01_JPRB, 0.20997E+01_JPRB, 0.22944E+01_JPRB, 0.26675E+01_JPRB, 0.29035E+01_JPRB ,&
&  0.28508E+01_JPRB, 0.28188E+01_JPRB, 0.32901E+01_JPRB, 0.37117E+01_JPRB, 0.38294E+01_JPRB, 0.37485E+01_JPRB, 0.33242E+01_JPRB ,&
&  0.31419E+01_JPRB, 0.28645E+01_JPRB, 0.27006E+01_JPRB, 0.22848E+01_JPRB, 0.23254E+01_JPRB, 0.26373E+01_JPRB, 0.25981E+01_JPRB ,&
&  0.26954E+01_JPRB, 0.25057E+01_JPRB, 0.23940E+01_JPRB, 0.20910E+01_JPRB, 0.18231E+01_JPRB, 0.16622E+01_JPRB, 0.16434E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.16261E+01_JPRB, 0.15262E+01_JPRB, 0.10396E+01_JPRB, 0.90102E+00_JPRB, 0.93919E+00_JPRB, 0.12136E+01_JPRB ,&
&  0.17509E+01_JPRB, 0.18982E+01_JPRB, 0.17794E+01_JPRB, 0.20250E+01_JPRB, 0.14824E+01_JPRB, 0.11934E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.98869E+00_JPRB, 0.99344E+00_JPRB, 0.97289E+00_JPRB, 0.92729E+00_JPRB, 0.93061E+00_JPRB, 0.96140E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.10568E+01_JPRB, 0.10706E+01_JPRB, 0.10442E+01_JPRB, 0.10053E+01_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.10140E+01_JPRB, 0.99945E+00_JPRB, 0.96686E+00_JPRB, 0.92816E+00_JPRB, 0.90018E+00_JPRB, 0.85188E+00_JPRB, 0.81197E+00_JPRB ,&
&  0.80633E+00_JPRB, 0.78928E+00_JPRB, 0.77026E+00_JPRB, 0.75259E+00_JPRB, 0.73405E+00_JPRB, 0.71588E+00_JPRB, 0.71136E+00_JPRB ,&
&  0.70745E+00_JPRB, 0.67915E+00_JPRB, 0.70399E+00_JPRB, 0.76900E+00_JPRB, 0.89962E+00_JPRB, 0.97842E+00_JPRB, 0.10894E+01_JPRB ,&
&  0.17181E+01_JPRB, 0.20825E+01_JPRB, 0.17714E+01_JPRB, 0.18782E+01_JPRB, 0.20863E+01_JPRB, 0.23209E+01_JPRB, 0.17305E+01_JPRB ,&
&  0.15081E+01_JPRB, 0.13934E+01_JPRB, 0.13075E+01_JPRB, 0.13694E+01_JPRB, 0.14690E+01_JPRB, 0.15547E+01_JPRB, 0.15692E+01_JPRB ,&
&  0.15120E+01_JPRB, 0.14819E+01_JPRB, 0.14850E+01_JPRB, 0.14788E+01_JPRB, 0.14725E+01_JPRB, 0.15142E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.15776E+01_JPRB, 0.16714E+01_JPRB, 0.18766E+01_JPRB, 0.19899E+01_JPRB, 0.22161E+01_JPRB, 0.25763E+01_JPRB, 0.29884E+01_JPRB ,&
&  0.31058E+01_JPRB, 0.33351E+01_JPRB, 0.36557E+01_JPRB, 0.39732E+01_JPRB, 0.42655E+01_JPRB, 0.44150E+01_JPRB, 0.39412E+01_JPRB ,&
&  0.35771E+01_JPRB, 0.30401E+01_JPRB, 0.25650E+01_JPRB, 0.24004E+01_JPRB, 0.24347E+01_JPRB, 0.28633E+01_JPRB, 0.28054E+01_JPRB ,&
&  0.25712E+01_JPRB, 0.20678E+01_JPRB, 0.19568E+01_JPRB, 0.20739E+01_JPRB, 0.19556E+01_JPRB, 0.18004E+01_JPRB, 0.20145E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.10408E+01_JPRB, 0.88761E+00_JPRB, 0.10572E+01_JPRB, 0.79115E+00_JPRB, 0.80130E+00_JPRB, 0.15149E+01_JPRB ,&
&  0.20135E+01_JPRB, 0.24772E+01_JPRB, 0.23880E+01_JPRB, 0.20141E+01_JPRB, 0.18654E+01_JPRB, 0.15027E+01_JPRB, 0.13498E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.11186E+01_JPRB, 0.11020E+01_JPRB, 0.10493E+01_JPRB, 0.10336E+01_JPRB, 0.10815E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.11591E+01_JPRB, 0.11346E+01_JPRB, 0.10927E+01_JPRB, 0.10438E+01_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.10596E+01_JPRB, 0.10348E+01_JPRB, 0.10165E+01_JPRB, 0.97580E+00_JPRB, 0.93756E+00_JPRB, 0.89034E+00_JPRB, 0.82774E+00_JPRB ,&
&  0.79504E+00_JPRB, 0.77994E+00_JPRB, 0.77418E+00_JPRB, 0.75662E+00_JPRB, 0.74087E+00_JPRB, 0.73229E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.71762E+00_JPRB, 0.69941E+00_JPRB, 0.77210E+00_JPRB, 0.83130E+00_JPRB, 0.85789E+00_JPRB, 0.85445E+00_JPRB, 0.11011E+01_JPRB ,&
&  0.16950E+01_JPRB, 0.22010E+01_JPRB, 0.25119E+01_JPRB, 0.26304E+01_JPRB, 0.27769E+01_JPRB, 0.29941E+01_JPRB, 0.25733E+01_JPRB ,&
&  0.21035E+01_JPRB, 0.17497E+01_JPRB, 0.14993E+01_JPRB, 0.14851E+01_JPRB, 0.15887E+01_JPRB, 0.16699E+01_JPRB, 0.16404E+01_JPRB ,&
&  0.15868E+01_JPRB, 0.15035E+01_JPRB, 0.14569E+01_JPRB, 0.14204E+01_JPRB, 0.14311E+01_JPRB, 0.14804E+01_JPRB, 0.15181E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.15585E+01_JPRB, 0.17019E+01_JPRB, 0.19639E+01_JPRB, 0.19093E+01_JPRB, 0.24772E+01_JPRB, 0.29488E+01_JPRB ,&
&  0.33358E+01_JPRB, 0.35587E+01_JPRB, 0.36323E+01_JPRB, 0.43751E+01_JPRB, 0.48906E+01_JPRB, 0.49038E+01_JPRB, 0.44518E+01_JPRB ,&
&  0.41938E+01_JPRB, 0.35267E+01_JPRB, 0.29726E+01_JPRB, 0.27429E+01_JPRB, 0.27022E+01_JPRB, 0.30157E+01_JPRB, 0.22416E+01_JPRB ,&
&  0.22223E+01_JPRB, 0.26314E+01_JPRB, 0.23195E+01_JPRB, 0.19945E+01_JPRB, 0.21201E+01_JPRB, 0.20128E+01_JPRB, 0.14958E+01_JPRB ,&
&  0.73737E+00_JPRB, 0.59139E+00_JPRB, 0.59612E+00_JPRB, 0.58111E+00_JPRB, 0.54639E+00_JPRB, 0.82763E+00_JPRB, 0.25188E+01_JPRB ,&
&  0.21481E+01_JPRB, 0.28333E+01_JPRB, 0.27067E+01_JPRB, 0.22898E+01_JPRB, 0.19789E+01_JPRB, 0.16719E+01_JPRB, 0.17180E+01_JPRB ,&
&  0.14390E+01_JPRB, 0.13029E+01_JPRB, 0.12299E+01_JPRB, 0.11813E+01_JPRB, 0.11807E+01_JPRB, 0.11925E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.12853E+01_JPRB, 0.12421E+01_JPRB, 0.11716E+01_JPRB, 0.11074E+01_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.11236E+01_JPRB, 0.10650E+01_JPRB, 0.10285E+01_JPRB, 0.10074E+01_JPRB, 0.98132E+00_JPRB, 0.93097E+00_JPRB, 0.85119E+00_JPRB ,&
&  0.79499E+00_JPRB, 0.77101E+00_JPRB, 0.76378E+00_JPRB, 0.75677E+00_JPRB, 0.75480E+00_JPRB, 0.75489E+00_JPRB, 0.75003E+00_JPRB ,&
&  0.72373E+00_JPRB, 0.73344E+00_JPRB, 0.85577E+00_JPRB, 0.87525E+00_JPRB, 0.88143E+00_JPRB, 0.84371E+00_JPRB, 0.10608E+01_JPRB ,&
&  0.16396E+01_JPRB, 0.22072E+01_JPRB, 0.25624E+01_JPRB, 0.26260E+01_JPRB, 0.33977E+01_JPRB, 0.38657E+01_JPRB, 0.38662E+01_JPRB ,&
&  0.31895E+01_JPRB, 0.25896E+01_JPRB, 0.20720E+01_JPRB, 0.18010E+01_JPRB, 0.17827E+01_JPRB, 0.17967E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.15857E+01_JPRB, 0.15093E+01_JPRB, 0.14578E+01_JPRB, 0.14481E+01_JPRB, 0.14806E+01_JPRB, 0.15359E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.15219E+01_JPRB, 0.15486E+01_JPRB, 0.18121E+01_JPRB, 0.20476E+01_JPRB, 0.22296E+01_JPRB, 0.27769E+01_JPRB ,&
&  0.33649E+01_JPRB, 0.38205E+01_JPRB, 0.38733E+01_JPRB, 0.50275E+01_JPRB, 0.53227E+01_JPRB, 0.52189E+01_JPRB, 0.51653E+01_JPRB ,&
&  0.48273E+01_JPRB, 0.46743E+01_JPRB, 0.34271E+01_JPRB, 0.30537E+01_JPRB, 0.30770E+01_JPRB, 0.25859E+01_JPRB, 0.25931E+01_JPRB ,&
&  0.30558E+01_JPRB, 0.26554E+01_JPRB, 0.22422E+01_JPRB, 0.18888E+01_JPRB, 0.22255E+01_JPRB, 0.18269E+01_JPRB, 0.86268E+00_JPRB ,&
&  0.68725E+00_JPRB, 0.64420E+00_JPRB, 0.57221E+00_JPRB, 0.55601E+00_JPRB, 0.58872E+00_JPRB, 0.84815E+00_JPRB, 0.16253E+01_JPRB ,&
&  0.18821E+01_JPRB, 0.25491E+01_JPRB, 0.25954E+01_JPRB, 0.23826E+01_JPRB, 0.19731E+01_JPRB, 0.18754E+01_JPRB, 0.22135E+01_JPRB ,&
&  0.18693E+01_JPRB, 0.16469E+01_JPRB, 0.14328E+01_JPRB, 0.13158E+01_JPRB, 0.12648E+01_JPRB, 0.12723E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.13565E+01_JPRB, 0.13156E+01_JPRB, 0.12484E+01_JPRB, 0.11789E+01_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.11587E+01_JPRB, 0.11108E+01_JPRB, 0.10597E+01_JPRB, 0.10134E+01_JPRB, 0.97761E+00_JPRB, 0.94327E+00_JPRB, 0.89236E+00_JPRB ,&
&  0.83086E+00_JPRB, 0.79457E+00_JPRB, 0.77917E+00_JPRB, 0.76245E+00_JPRB, 0.75736E+00_JPRB, 0.76159E+00_JPRB, 0.75635E+00_JPRB ,&
&  0.73732E+00_JPRB, 0.76365E+00_JPRB, 0.82797E+00_JPRB, 0.79888E+00_JPRB, 0.78568E+00_JPRB, 0.76268E+00_JPRB, 0.92349E+00_JPRB ,&
&  0.14923E+01_JPRB, 0.20979E+01_JPRB, 0.24490E+01_JPRB, 0.28144E+01_JPRB, 0.36818E+01_JPRB, 0.38160E+01_JPRB, 0.43631E+01_JPRB ,&
&  0.39393E+01_JPRB, 0.33654E+01_JPRB, 0.28704E+01_JPRB, 0.25147E+01_JPRB, 0.22723E+01_JPRB, 0.20827E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.18516E+01_JPRB, 0.17134E+01_JPRB, 0.16575E+01_JPRB, 0.15914E+01_JPRB, 0.15559E+01_JPRB, 0.15427E+01_JPRB, 0.16080E+01_JPRB ,&
&  0.16167E+01_JPRB, 0.15655E+01_JPRB, 0.14842E+01_JPRB, 0.16181E+01_JPRB, 0.21098E+01_JPRB, 0.25239E+01_JPRB, 0.27014E+01_JPRB ,&
&  0.29039E+01_JPRB, 0.31164E+01_JPRB, 0.39786E+01_JPRB, 0.49181E+01_JPRB, 0.55419E+01_JPRB, 0.58570E+01_JPRB, 0.55616E+01_JPRB ,&
&  0.52060E+01_JPRB, 0.49418E+01_JPRB, 0.37666E+01_JPRB, 0.33858E+01_JPRB, 0.29608E+01_JPRB, 0.25663E+01_JPRB, 0.31223E+01_JPRB ,&
&  0.34158E+01_JPRB, 0.30446E+01_JPRB, 0.24384E+01_JPRB, 0.19509E+01_JPRB, 0.17747E+01_JPRB, 0.12043E+01_JPRB, 0.81205E+00_JPRB ,&
&  0.76502E+00_JPRB, 0.70852E+00_JPRB, 0.64474E+00_JPRB, 0.69530E+00_JPRB, 0.68291E+00_JPRB, 0.90835E+00_JPRB, 0.14419E+01_JPRB ,&
&  0.20236E+01_JPRB, 0.24377E+01_JPRB, 0.27620E+01_JPRB, 0.24532E+01_JPRB, 0.21095E+01_JPRB, 0.18895E+01_JPRB, 0.23275E+01_JPRB ,&
&  0.17762E+01_JPRB, 0.16699E+01_JPRB, 0.16289E+01_JPRB, 0.15016E+01_JPRB, 0.14277E+01_JPRB, 0.13885E+01_JPRB, 0.13885E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.13762E+01_JPRB, 0.13265E+01_JPRB, 0.12628E+01_JPRB, 0.12050E+01_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.11598E+01_JPRB, 0.11026E+01_JPRB, 0.10521E+01_JPRB, 0.10006E+01_JPRB, 0.95357E+00_JPRB, 0.91798E+00_JPRB, 0.89164E+00_JPRB ,&
&  0.85558E+00_JPRB, 0.82284E+00_JPRB, 0.78808E+00_JPRB, 0.76212E+00_JPRB, 0.74975E+00_JPRB, 0.74561E+00_JPRB, 0.73590E+00_JPRB ,&
&  0.72776E+00_JPRB, 0.78086E+00_JPRB, 0.64488E+00_JPRB, 0.60961E+00_JPRB, 0.67388E+00_JPRB, 0.71880E+00_JPRB, 0.77885E+00_JPRB ,&
&  0.12169E+01_JPRB, 0.18459E+01_JPRB, 0.22454E+01_JPRB, 0.30263E+01_JPRB, 0.46977E+01_JPRB, 0.44957E+01_JPRB, 0.45311E+01_JPRB ,&
&  0.45555E+01_JPRB, 0.38086E+01_JPRB, 0.33119E+01_JPRB, 0.28815E+01_JPRB, 0.25527E+01_JPRB, 0.22883E+01_JPRB, 0.20420E+01_JPRB ,&
&  0.18328E+01_JPRB, 0.16981E+01_JPRB, 0.16317E+01_JPRB, 0.16295E+01_JPRB, 0.16378E+01_JPRB, 0.16542E+01_JPRB, 0.16735E+01_JPRB ,&
&  0.16719E+01_JPRB, 0.16354E+01_JPRB, 0.15460E+01_JPRB, 0.16098E+01_JPRB, 0.18701E+01_JPRB, 0.22895E+01_JPRB, 0.32308E+01_JPRB ,&
&  0.32468E+01_JPRB, 0.35171E+01_JPRB, 0.36065E+01_JPRB, 0.51533E+01_JPRB, 0.60117E+01_JPRB, 0.58564E+01_JPRB, 0.59573E+01_JPRB ,&
&  0.45361E+01_JPRB, 0.38421E+01_JPRB, 0.34645E+01_JPRB, 0.31031E+01_JPRB, 0.28819E+01_JPRB, 0.34251E+01_JPRB, 0.48970E+01_JPRB ,&
&  0.37393E+01_JPRB, 0.39233E+01_JPRB, 0.39907E+01_JPRB, 0.36856E+01_JPRB, 0.18842E+01_JPRB, 0.13271E+01_JPRB, 0.20964E+01_JPRB ,&
&  0.21945E+01_JPRB, 0.13047E+01_JPRB, 0.95316E+00_JPRB, 0.12289E+01_JPRB, 0.94491E+00_JPRB, 0.11073E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.16968E+01_JPRB, 0.20623E+01_JPRB, 0.30431E+01_JPRB, 0.28796E+01_JPRB, 0.25536E+01_JPRB, 0.21990E+01_JPRB, 0.22107E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.16854E+01_JPRB, 0.16539E+01_JPRB, 0.14961E+01_JPRB, 0.14148E+01_JPRB, 0.13647E+01_JPRB, 0.13406E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.13280E+01_JPRB, 0.12945E+01_JPRB, 0.12492E+01_JPRB, 0.12067E+01_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.11608E+01_JPRB, 0.11009E+01_JPRB, 0.10375E+01_JPRB, 0.98795E+00_JPRB, 0.94336E+00_JPRB, 0.90061E+00_JPRB, 0.86494E+00_JPRB ,&
&  0.84111E+00_JPRB, 0.81635E+00_JPRB, 0.77667E+00_JPRB, 0.74879E+00_JPRB, 0.73335E+00_JPRB, 0.73495E+00_JPRB, 0.73066E+00_JPRB ,&
&  0.72600E+00_JPRB, 0.68085E+00_JPRB, 0.56801E+00_JPRB, 0.53414E+00_JPRB, 0.62928E+00_JPRB, 0.62727E+00_JPRB, 0.67219E+00_JPRB ,&
&  0.96615E+00_JPRB, 0.15529E+01_JPRB, 0.22030E+01_JPRB, 0.30406E+01_JPRB, 0.42658E+01_JPRB, 0.49392E+01_JPRB, 0.48057E+01_JPRB ,&
&  0.48623E+01_JPRB, 0.41409E+01_JPRB, 0.35463E+01_JPRB, 0.30912E+01_JPRB, 0.26677E+01_JPRB, 0.22579E+01_JPRB, 0.19154E+01_JPRB ,&
&  0.17175E+01_JPRB, 0.15897E+01_JPRB, 0.15215E+01_JPRB, 0.14558E+01_JPRB, 0.14797E+01_JPRB, 0.15435E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.16413E+01_JPRB, 0.16456E+01_JPRB, 0.16272E+01_JPRB, 0.16266E+01_JPRB, 0.18951E+01_JPRB, 0.23381E+01_JPRB, 0.27592E+01_JPRB ,&
&  0.31366E+01_JPRB, 0.34078E+01_JPRB, 0.37927E+01_JPRB, 0.45939E+01_JPRB, 0.57226E+01_JPRB, 0.55829E+01_JPRB, 0.61704E+01_JPRB ,&
&  0.48842E+01_JPRB, 0.40639E+01_JPRB, 0.33594E+01_JPRB, 0.25712E+01_JPRB, 0.31883E+01_JPRB, 0.52639E+01_JPRB, 0.51311E+01_JPRB ,&
&  0.47270E+01_JPRB, 0.46658E+01_JPRB, 0.43702E+01_JPRB, 0.35512E+01_JPRB, 0.30341E+01_JPRB, 0.23319E+01_JPRB, 0.28012E+01_JPRB ,&
&  0.27155E+01_JPRB, 0.25525E+01_JPRB, 0.22669E+01_JPRB, 0.18775E+01_JPRB, 0.15613E+01_JPRB, 0.16985E+01_JPRB, 0.15663E+01_JPRB ,&
&  0.14714E+01_JPRB, 0.15994E+01_JPRB, 0.26098E+01_JPRB, 0.31769E+01_JPRB, 0.28860E+01_JPRB, 0.23135E+01_JPRB, 0.24626E+01_JPRB ,&
&  0.20901E+01_JPRB, 0.18426E+01_JPRB, 0.16105E+01_JPRB, 0.14452E+01_JPRB, 0.13727E+01_JPRB, 0.13346E+01_JPRB, 0.13151E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.12652E+01_JPRB, 0.12399E+01_JPRB, 0.12159E+01_JPRB, 0.11924E+01_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.11437E+01_JPRB, 0.10882E+01_JPRB, 0.10319E+01_JPRB, 0.98540E+00_JPRB, 0.93814E+00_JPRB, 0.89533E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.81589E+00_JPRB, 0.79719E+00_JPRB, 0.77370E+00_JPRB, 0.75458E+00_JPRB, 0.74441E+00_JPRB, 0.73971E+00_JPRB, 0.74319E+00_JPRB ,&
&  0.74720E+00_JPRB, 0.68687E+00_JPRB, 0.54980E+00_JPRB, 0.52571E+00_JPRB, 0.54480E+00_JPRB, 0.53702E+00_JPRB, 0.62589E+00_JPRB ,&
&  0.82979E+00_JPRB, 0.12496E+01_JPRB, 0.18009E+01_JPRB, 0.25551E+01_JPRB, 0.34005E+01_JPRB, 0.40587E+01_JPRB, 0.42223E+01_JPRB ,&
&  0.37245E+01_JPRB, 0.36165E+01_JPRB, 0.30887E+01_JPRB, 0.26634E+01_JPRB, 0.22396E+01_JPRB, 0.19258E+01_JPRB, 0.17010E+01_JPRB ,&
&  0.15064E+01_JPRB, 0.13873E+01_JPRB, 0.13349E+01_JPRB, 0.13257E+01_JPRB, 0.13531E+01_JPRB, 0.14064E+01_JPRB, 0.14826E+01_JPRB ,&
&  0.15188E+01_JPRB, 0.15426E+01_JPRB, 0.15446E+01_JPRB, 0.16471E+01_JPRB, 0.17444E+01_JPRB, 0.23286E+01_JPRB, 0.25874E+01_JPRB ,&
&  0.33416E+01_JPRB, 0.38823E+01_JPRB, 0.41582E+01_JPRB, 0.54199E+01_JPRB, 0.47157E+01_JPRB, 0.55118E+01_JPRB, 0.63904E+01_JPRB ,&
&  0.59479E+01_JPRB, 0.53834E+01_JPRB, 0.52104E+01_JPRB, 0.42711E+01_JPRB, 0.37384E+01_JPRB, 0.52727E+01_JPRB, 0.48730E+01_JPRB ,&
&  0.43924E+01_JPRB, 0.42861E+01_JPRB, 0.39112E+01_JPRB, 0.37605E+01_JPRB, 0.33427E+01_JPRB, 0.30155E+01_JPRB, 0.22360E+01_JPRB ,&
&  0.22281E+01_JPRB, 0.20302E+01_JPRB, 0.21831E+01_JPRB, 0.19632E+01_JPRB, 0.18219E+01_JPRB, 0.16787E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.13998E+01_JPRB, 0.13843E+01_JPRB, 0.17882E+01_JPRB, 0.22858E+01_JPRB, 0.24669E+01_JPRB, 0.21201E+01_JPRB, 0.22290E+01_JPRB ,&
&  0.20938E+01_JPRB, 0.18773E+01_JPRB, 0.16241E+01_JPRB, 0.15512E+01_JPRB, 0.14796E+01_JPRB, 0.14069E+01_JPRB, 0.13551E+01_JPRB ,&
&  0.13022E+01_JPRB, 0.12520E+01_JPRB, 0.12197E+01_JPRB, 0.12092E+01_JPRB, 0.11845E+01_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.10774E+01_JPRB, 0.10309E+01_JPRB, 0.98406E+00_JPRB, 0.94744E+00_JPRB, 0.91957E+00_JPRB, 0.89110E+00_JPRB, 0.84653E+00_JPRB ,&
&  0.79762E+00_JPRB, 0.77100E+00_JPRB, 0.75753E+00_JPRB, 0.74471E+00_JPRB, 0.73731E+00_JPRB, 0.73956E+00_JPRB, 0.74599E+00_JPRB ,&
&  0.75737E+00_JPRB, 0.81719E+00_JPRB, 0.64283E+00_JPRB, 0.52431E+00_JPRB, 0.46645E+00_JPRB, 0.52171E+00_JPRB, 0.64743E+00_JPRB ,&
&  0.81799E+00_JPRB, 0.11003E+01_JPRB, 0.15684E+01_JPRB, 0.19766E+01_JPRB, 0.24387E+01_JPRB, 0.29067E+01_JPRB, 0.30832E+01_JPRB ,&
&  0.33313E+01_JPRB, 0.26226E+01_JPRB, 0.23239E+01_JPRB, 0.19097E+01_JPRB, 0.16392E+01_JPRB, 0.14722E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11426E+01_JPRB, 0.11072E+01_JPRB, 0.11215E+01_JPRB, 0.11409E+01_JPRB, 0.11822E+01_JPRB, 0.12501E+01_JPRB ,&
&  0.13244E+01_JPRB, 0.13674E+01_JPRB, 0.13723E+01_JPRB, 0.14626E+01_JPRB, 0.17994E+01_JPRB, 0.22435E+01_JPRB, 0.26297E+01_JPRB ,&
&  0.26472E+01_JPRB, 0.28602E+01_JPRB, 0.50715E+01_JPRB, 0.59675E+01_JPRB, 0.64528E+01_JPRB, 0.67401E+01_JPRB, 0.65355E+01_JPRB ,&
&  0.60745E+01_JPRB, 0.56183E+01_JPRB, 0.54034E+01_JPRB, 0.47318E+01_JPRB, 0.53295E+01_JPRB, 0.52514E+01_JPRB, 0.49398E+01_JPRB ,&
&  0.44230E+01_JPRB, 0.41059E+01_JPRB, 0.37612E+01_JPRB, 0.35575E+01_JPRB, 0.34785E+01_JPRB, 0.35381E+01_JPRB, 0.32774E+01_JPRB ,&
&  0.27581E+01_JPRB, 0.29147E+01_JPRB, 0.24651E+01_JPRB, 0.19944E+01_JPRB, 0.16273E+01_JPRB, 0.15163E+01_JPRB, 0.14293E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.12845E+01_JPRB, 0.14117E+01_JPRB, 0.15685E+01_JPRB, 0.17620E+01_JPRB, 0.18518E+01_JPRB, 0.17701E+01_JPRB ,&
&  0.15812E+01_JPRB, 0.16901E+01_JPRB, 0.17007E+01_JPRB, 0.16311E+01_JPRB, 0.15931E+01_JPRB, 0.14918E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.13135E+01_JPRB, 0.12520E+01_JPRB, 0.12089E+01_JPRB, 0.11711E+01_JPRB, 0.11266E+01_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.96634E+00_JPRB, 0.93769E+00_JPRB, 0.91221E+00_JPRB, 0.88272E+00_JPRB, 0.86401E+00_JPRB, 0.84568E+00_JPRB, 0.81751E+00_JPRB ,&
&  0.78386E+00_JPRB, 0.75294E+00_JPRB, 0.73203E+00_JPRB, 0.72113E+00_JPRB, 0.71773E+00_JPRB, 0.72264E+00_JPRB, 0.72402E+00_JPRB ,&
&  0.74419E+00_JPRB, 0.96879E+00_JPRB, 0.75659E+00_JPRB, 0.64071E+00_JPRB, 0.56900E+00_JPRB, 0.63615E+00_JPRB, 0.72905E+00_JPRB ,&
&  0.87635E+00_JPRB, 0.97998E+00_JPRB, 0.11693E+01_JPRB, 0.14822E+01_JPRB, 0.17689E+01_JPRB, 0.20038E+01_JPRB, 0.21569E+01_JPRB ,&
&  0.20020E+01_JPRB, 0.18473E+01_JPRB, 0.14276E+01_JPRB, 0.13252E+01_JPRB, 0.11822E+01_JPRB, 0.10204E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.10186E+01_JPRB, 0.10177E+01_JPRB, 0.10173E+01_JPRB, 0.10427E+01_JPRB, 0.10698E+01_JPRB, 0.10999E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.12182E+01_JPRB, 0.12557E+01_JPRB, 0.13202E+01_JPRB, 0.15265E+01_JPRB, 0.19475E+01_JPRB, 0.22749E+01_JPRB ,&
&  0.28785E+01_JPRB, 0.29519E+01_JPRB, 0.30851E+01_JPRB, 0.46640E+01_JPRB, 0.68284E+01_JPRB, 0.66232E+01_JPRB, 0.57780E+01_JPRB ,&
&  0.55876E+01_JPRB, 0.55212E+01_JPRB, 0.55789E+01_JPRB, 0.53404E+01_JPRB, 0.48561E+01_JPRB, 0.46079E+01_JPRB, 0.43279E+01_JPRB ,&
&  0.38741E+01_JPRB, 0.35439E+01_JPRB, 0.33548E+01_JPRB, 0.33017E+01_JPRB, 0.31290E+01_JPRB, 0.30726E+01_JPRB, 0.30812E+01_JPRB ,&
&  0.29830E+01_JPRB, 0.25353E+01_JPRB, 0.16777E+01_JPRB, 0.16464E+01_JPRB, 0.12842E+01_JPRB, 0.11446E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.10744E+01_JPRB, 0.11587E+01_JPRB, 0.12559E+01_JPRB, 0.12171E+01_JPRB, 0.14549E+01_JPRB, 0.14813E+01_JPRB, 0.14084E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.11778E+01_JPRB, 0.14175E+01_JPRB, 0.14844E+01_JPRB, 0.14764E+01_JPRB, 0.13992E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.10884E+01_JPRB, 0.10541E+01_JPRB, 0.10333E+01_JPRB, 0.99860E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.85341E+00_JPRB, 0.83875E+00_JPRB, 0.81713E+00_JPRB, 0.78266E+00_JPRB, 0.76067E+00_JPRB, 0.75480E+00_JPRB, 0.76209E+00_JPRB ,&
&  0.75915E+00_JPRB, 0.73502E+00_JPRB, 0.71681E+00_JPRB, 0.70068E+00_JPRB, 0.69389E+00_JPRB, 0.69503E+00_JPRB, 0.70276E+00_JPRB ,&
&  0.77579E+00_JPRB, 0.10252E+01_JPRB, 0.82815E+00_JPRB, 0.68007E+00_JPRB, 0.80701E+00_JPRB, 0.76668E+00_JPRB, 0.80971E+00_JPRB ,&
&  0.79578E+00_JPRB, 0.93075E+00_JPRB, 0.94773E+00_JPRB, 0.10436E+01_JPRB, 0.11727E+01_JPRB, 0.13762E+01_JPRB, 0.14406E+01_JPRB ,&
&  0.12983E+01_JPRB, 0.10522E+01_JPRB, 0.86556E+00_JPRB, 0.81797E+00_JPRB, 0.86020E+00_JPRB, 0.78548E+00_JPRB, 0.83914E+00_JPRB ,&
&  0.85783E+00_JPRB, 0.88220E+00_JPRB, 0.90626E+00_JPRB, 0.92104E+00_JPRB, 0.95262E+00_JPRB, 0.99887E+00_JPRB, 0.10531E+01_JPRB ,&
&  0.11178E+01_JPRB, 0.11762E+01_JPRB, 0.12098E+01_JPRB, 0.12654E+01_JPRB, 0.14733E+01_JPRB, 0.20398E+01_JPRB, 0.25885E+01_JPRB ,&
&  0.31480E+01_JPRB, 0.35987E+01_JPRB, 0.43907E+01_JPRB, 0.56699E+01_JPRB, 0.60912E+01_JPRB, 0.61179E+01_JPRB, 0.57247E+01_JPRB ,&
&  0.51120E+01_JPRB, 0.45523E+01_JPRB, 0.43587E+01_JPRB, 0.38838E+01_JPRB, 0.36255E+01_JPRB, 0.35706E+01_JPRB, 0.34476E+01_JPRB ,&
&  0.32064E+01_JPRB, 0.30259E+01_JPRB, 0.29626E+01_JPRB, 0.28138E+01_JPRB, 0.27211E+01_JPRB, 0.25920E+01_JPRB, 0.26055E+01_JPRB ,&
&  0.29114E+01_JPRB, 0.18502E+01_JPRB, 0.13825E+01_JPRB, 0.14763E+01_JPRB, 0.11863E+01_JPRB, 0.10530E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10409E+01_JPRB, 0.11367E+01_JPRB, 0.10515E+01_JPRB, 0.10954E+01_JPRB, 0.12059E+01_JPRB, 0.10443E+01_JPRB ,&
&  0.97697E+00_JPRB, 0.93845E+00_JPRB, 0.10773E+01_JPRB, 0.11793E+01_JPRB, 0.12014E+01_JPRB, 0.11379E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.94707E+00_JPRB, 0.90260E+00_JPRB, 0.88965E+00_JPRB, 0.87720E+00_JPRB, 0.86882E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.74750E+00_JPRB, 0.74113E+00_JPRB, 0.73144E+00_JPRB, 0.72272E+00_JPRB, 0.71722E+00_JPRB, 0.71109E+00_JPRB, 0.71866E+00_JPRB ,&
&  0.72620E+00_JPRB, 0.71396E+00_JPRB, 0.69400E+00_JPRB, 0.67753E+00_JPRB, 0.66101E+00_JPRB, 0.65786E+00_JPRB, 0.68009E+00_JPRB ,&
&  0.59196E+00_JPRB, 0.73501E+00_JPRB, 0.61942E+00_JPRB, 0.66865E+00_JPRB, 0.99939E+00_JPRB, 0.86342E+00_JPRB, 0.82378E+00_JPRB ,&
&  0.74814E+00_JPRB, 0.74027E+00_JPRB, 0.73107E+00_JPRB, 0.80666E+00_JPRB, 0.86495E+00_JPRB, 0.95008E+00_JPRB, 0.98112E+00_JPRB ,&
&  0.85456E+00_JPRB, 0.67672E+00_JPRB, 0.59756E+00_JPRB, 0.57609E+00_JPRB, 0.61613E+00_JPRB, 0.65094E+00_JPRB, 0.72434E+00_JPRB ,&
&  0.76028E+00_JPRB, 0.78741E+00_JPRB, 0.81205E+00_JPRB, 0.85013E+00_JPRB, 0.87269E+00_JPRB, 0.90150E+00_JPRB, 0.94458E+00_JPRB ,&
&  0.99698E+00_JPRB, 0.10663E+01_JPRB, 0.11358E+01_JPRB, 0.12666E+01_JPRB, 0.14055E+01_JPRB, 0.22003E+01_JPRB, 0.35633E+01_JPRB ,&
&  0.40035E+01_JPRB, 0.42029E+01_JPRB, 0.49521E+01_JPRB, 0.62953E+01_JPRB, 0.64293E+01_JPRB, 0.60069E+01_JPRB, 0.52417E+01_JPRB ,&
&  0.45280E+01_JPRB, 0.38473E+01_JPRB, 0.36830E+01_JPRB, 0.34338E+01_JPRB, 0.31336E+01_JPRB, 0.32140E+01_JPRB, 0.31169E+01_JPRB ,&
&  0.29420E+01_JPRB, 0.29009E+01_JPRB, 0.28311E+01_JPRB, 0.28396E+01_JPRB, 0.26610E+01_JPRB, 0.26542E+01_JPRB, 0.25406E+01_JPRB ,&
&  0.25514E+01_JPRB, 0.23082E+01_JPRB, 0.16843E+01_JPRB, 0.13855E+01_JPRB, 0.11095E+01_JPRB, 0.10438E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.10281E+01_JPRB, 0.94047E+00_JPRB, 0.84347E+00_JPRB, 0.81154E+00_JPRB, 0.87554E+00_JPRB, 0.77521E+00_JPRB ,&
&  0.73014E+00_JPRB, 0.79715E+00_JPRB, 0.86798E+00_JPRB, 0.92219E+00_JPRB, 0.94741E+00_JPRB, 0.93466E+00_JPRB, 0.80496E+00_JPRB ,&
&  0.79612E+00_JPRB, 0.76928E+00_JPRB, 0.76142E+00_JPRB, 0.74697E+00_JPRB, 0.74390E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.68094E+00_JPRB, 0.68127E+00_JPRB, 0.67914E+00_JPRB, 0.68587E+00_JPRB, 0.69848E+00_JPRB, 0.69503E+00_JPRB, 0.69368E+00_JPRB ,&
&  0.70218E+00_JPRB, 0.69429E+00_JPRB, 0.67064E+00_JPRB, 0.65561E+00_JPRB, 0.63865E+00_JPRB, 0.63684E+00_JPRB, 0.56612E+00_JPRB ,&
&  0.52098E+00_JPRB, 0.60010E+00_JPRB, 0.58753E+00_JPRB, 0.64998E+00_JPRB, 0.75372E+00_JPRB, 0.73206E+00_JPRB, 0.73120E+00_JPRB ,&
&  0.70575E+00_JPRB, 0.65455E+00_JPRB, 0.63622E+00_JPRB, 0.64527E+00_JPRB, 0.66330E+00_JPRB, 0.71489E+00_JPRB, 0.75878E+00_JPRB ,&
&  0.67408E+00_JPRB, 0.56038E+00_JPRB, 0.50910E+00_JPRB, 0.51110E+00_JPRB, 0.56706E+00_JPRB, 0.61972E+00_JPRB, 0.65237E+00_JPRB ,&
&  0.68028E+00_JPRB, 0.71095E+00_JPRB, 0.75137E+00_JPRB, 0.80197E+00_JPRB, 0.82865E+00_JPRB, 0.85306E+00_JPRB, 0.88128E+00_JPRB ,&
&  0.91454E+00_JPRB, 0.96977E+00_JPRB, 0.10470E+01_JPRB, 0.11864E+01_JPRB, 0.13449E+01_JPRB, 0.21145E+01_JPRB, 0.31672E+01_JPRB ,&
&  0.35442E+01_JPRB, 0.38515E+01_JPRB, 0.43079E+01_JPRB, 0.48565E+01_JPRB, 0.49693E+01_JPRB, 0.45319E+01_JPRB, 0.40682E+01_JPRB ,&
&  0.37975E+01_JPRB, 0.33528E+01_JPRB, 0.36178E+01_JPRB, 0.33142E+01_JPRB, 0.30651E+01_JPRB, 0.30406E+01_JPRB, 0.28879E+01_JPRB ,&
&  0.28319E+01_JPRB, 0.28673E+01_JPRB, 0.27564E+01_JPRB, 0.26492E+01_JPRB, 0.24193E+01_JPRB, 0.23603E+01_JPRB, 0.21789E+01_JPRB ,&
&  0.21917E+01_JPRB, 0.21486E+01_JPRB, 0.18133E+01_JPRB, 0.15076E+01_JPRB, 0.12203E+01_JPRB, 0.11691E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.96684E+00_JPRB, 0.85241E+00_JPRB, 0.77518E+00_JPRB, 0.68299E+00_JPRB, 0.64335E+00_JPRB, 0.62871E+00_JPRB, 0.61886E+00_JPRB ,&
&  0.67423E+00_JPRB, 0.76711E+00_JPRB, 0.77769E+00_JPRB, 0.77253E+00_JPRB, 0.78040E+00_JPRB, 0.80214E+00_JPRB, 0.65186E+00_JPRB ,&
&  0.63709E+00_JPRB, 0.69655E+00_JPRB, 0.68888E+00_JPRB, 0.67825E+00_JPRB, 0.67379E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.63735E+00_JPRB, 0.64103E+00_JPRB, 0.64643E+00_JPRB, 0.66377E+00_JPRB, 0.68207E+00_JPRB, 0.68673E+00_JPRB, 0.64709E+00_JPRB ,&
&  0.68845E+00_JPRB, 0.66385E+00_JPRB, 0.64515E+00_JPRB, 0.63857E+00_JPRB, 0.62707E+00_JPRB, 0.54618E+00_JPRB, 0.46440E+00_JPRB ,&
&  0.45566E+00_JPRB, 0.48908E+00_JPRB, 0.55060E+00_JPRB, 0.59470E+00_JPRB, 0.64289E+00_JPRB, 0.64491E+00_JPRB, 0.63788E+00_JPRB ,&
&  0.62569E+00_JPRB, 0.59823E+00_JPRB, 0.59391E+00_JPRB, 0.58928E+00_JPRB, 0.57960E+00_JPRB, 0.58170E+00_JPRB, 0.59639E+00_JPRB ,&
&  0.54612E+00_JPRB, 0.51272E+00_JPRB, 0.51119E+00_JPRB, 0.49970E+00_JPRB, 0.55980E+00_JPRB, 0.58383E+00_JPRB, 0.61240E+00_JPRB ,&
&  0.64595E+00_JPRB, 0.68555E+00_JPRB, 0.75145E+00_JPRB, 0.80240E+00_JPRB, 0.83732E+00_JPRB, 0.85938E+00_JPRB, 0.89267E+00_JPRB ,&
&  0.91683E+00_JPRB, 0.96210E+00_JPRB, 0.10457E+01_JPRB, 0.11985E+01_JPRB, 0.14552E+01_JPRB, 0.19542E+01_JPRB, 0.29026E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.35801E+01_JPRB, 0.39419E+01_JPRB, 0.35491E+01_JPRB, 0.37801E+01_JPRB, 0.39494E+01_JPRB, 0.37474E+01_JPRB ,&
&  0.36757E+01_JPRB, 0.32264E+01_JPRB, 0.30853E+01_JPRB, 0.29690E+01_JPRB, 0.28994E+01_JPRB, 0.26351E+01_JPRB, 0.23984E+01_JPRB ,&
&  0.24973E+01_JPRB, 0.27420E+01_JPRB, 0.22013E+01_JPRB, 0.19538E+01_JPRB, 0.17503E+01_JPRB, 0.16826E+01_JPRB, 0.15750E+01_JPRB ,&
&  0.15472E+01_JPRB, 0.15391E+01_JPRB, 0.14577E+01_JPRB, 0.13494E+01_JPRB, 0.11611E+01_JPRB, 0.10717E+01_JPRB, 0.97443E+00_JPRB ,&
&  0.92082E+00_JPRB, 0.83877E+00_JPRB, 0.72870E+00_JPRB, 0.66086E+00_JPRB, 0.60898E+00_JPRB, 0.56343E+00_JPRB, 0.56495E+00_JPRB ,&
&  0.60920E+00_JPRB, 0.64359E+00_JPRB, 0.73456E+00_JPRB, 0.73324E+00_JPRB, 0.73501E+00_JPRB, 0.75857E+00_JPRB, 0.66773E+00_JPRB ,&
&  0.62679E+00_JPRB, 0.69719E+00_JPRB, 0.67335E+00_JPRB, 0.65002E+00_JPRB, 0.63731E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.61299E+00_JPRB, 0.62625E+00_JPRB, 0.63620E+00_JPRB, 0.65348E+00_JPRB, 0.67445E+00_JPRB, 0.62841E+00_JPRB, 0.58980E+00_JPRB ,&
&  0.64755E+00_JPRB, 0.56673E+00_JPRB, 0.61424E+00_JPRB, 0.52319E+00_JPRB, 0.47982E+00_JPRB, 0.46196E+00_JPRB, 0.45208E+00_JPRB ,&
&  0.44811E+00_JPRB, 0.47904E+00_JPRB, 0.51355E+00_JPRB, 0.54519E+00_JPRB, 0.59979E+00_JPRB, 0.59827E+00_JPRB, 0.58555E+00_JPRB ,&
&  0.56853E+00_JPRB, 0.55346E+00_JPRB, 0.55906E+00_JPRB, 0.55363E+00_JPRB, 0.55236E+00_JPRB, 0.54981E+00_JPRB, 0.54000E+00_JPRB ,&
&  0.50291E+00_JPRB, 0.49478E+00_JPRB, 0.51558E+00_JPRB, 0.53350E+00_JPRB, 0.54965E+00_JPRB, 0.56001E+00_JPRB, 0.58181E+00_JPRB ,&
&  0.61111E+00_JPRB, 0.63106E+00_JPRB, 0.78307E+00_JPRB, 0.82282E+00_JPRB, 0.86663E+00_JPRB, 0.89414E+00_JPRB, 0.92850E+00_JPRB ,&
&  0.97439E+00_JPRB, 0.10316E+01_JPRB, 0.10899E+01_JPRB, 0.11757E+01_JPRB, 0.13492E+01_JPRB, 0.17666E+01_JPRB, 0.23098E+01_JPRB ,&
&  0.31015E+01_JPRB, 0.22477E+01_JPRB, 0.27768E+01_JPRB, 0.27730E+01_JPRB, 0.31238E+01_JPRB, 0.34230E+01_JPRB, 0.34371E+01_JPRB ,&
&  0.36876E+01_JPRB, 0.32435E+01_JPRB, 0.28591E+01_JPRB, 0.24638E+01_JPRB, 0.21422E+01_JPRB, 0.20712E+01_JPRB, 0.19909E+01_JPRB ,&
&  0.20008E+01_JPRB, 0.18893E+01_JPRB, 0.16218E+01_JPRB, 0.14625E+01_JPRB, 0.13943E+01_JPRB, 0.13541E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11152E+01_JPRB, 0.10418E+01_JPRB, 0.95713E+00_JPRB, 0.89973E+00_JPRB, 0.86773E+00_JPRB, 0.81987E+00_JPRB ,&
&  0.80477E+00_JPRB, 0.77436E+00_JPRB, 0.69326E+00_JPRB, 0.65337E+00_JPRB, 0.61899E+00_JPRB, 0.58306E+00_JPRB, 0.57870E+00_JPRB ,&
&  0.57793E+00_JPRB, 0.56318E+00_JPRB, 0.59545E+00_JPRB, 0.63453E+00_JPRB, 0.64293E+00_JPRB, 0.64697E+00_JPRB, 0.71336E+00_JPRB ,&
&  0.66524E+00_JPRB, 0.67187E+00_JPRB, 0.67292E+00_JPRB, 0.63133E+00_JPRB, 0.61189E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.59560E+00_JPRB, 0.60069E+00_JPRB, 0.62315E+00_JPRB, 0.64273E+00_JPRB, 0.66115E+00_JPRB, 0.60781E+00_JPRB, 0.55268E+00_JPRB ,&
&  0.57269E+00_JPRB, 0.50906E+00_JPRB, 0.47346E+00_JPRB, 0.46193E+00_JPRB, 0.45971E+00_JPRB, 0.44687E+00_JPRB, 0.43652E+00_JPRB ,&
&  0.43522E+00_JPRB, 0.45019E+00_JPRB, 0.47756E+00_JPRB, 0.49842E+00_JPRB, 0.51870E+00_JPRB, 0.53108E+00_JPRB, 0.53058E+00_JPRB ,&
&  0.52799E+00_JPRB, 0.52722E+00_JPRB, 0.52701E+00_JPRB, 0.52785E+00_JPRB, 0.52668E+00_JPRB, 0.52756E+00_JPRB, 0.52307E+00_JPRB ,&
&  0.51169E+00_JPRB, 0.50418E+00_JPRB, 0.51022E+00_JPRB, 0.52960E+00_JPRB, 0.53317E+00_JPRB, 0.53347E+00_JPRB, 0.52816E+00_JPRB ,&
&  0.45460E+00_JPRB, 0.52017E+00_JPRB, 0.76346E+00_JPRB, 0.82955E+00_JPRB, 0.86907E+00_JPRB, 0.92402E+00_JPRB, 0.97557E+00_JPRB ,&
&  0.10299E+01_JPRB, 0.11017E+01_JPRB, 0.11890E+01_JPRB, 0.12853E+01_JPRB, 0.13986E+01_JPRB, 0.15888E+01_JPRB, 0.18973E+01_JPRB ,&
&  0.25551E+01_JPRB, 0.20796E+01_JPRB, 0.19445E+01_JPRB, 0.20743E+01_JPRB, 0.29716E+01_JPRB, 0.31562E+01_JPRB, 0.28449E+01_JPRB ,&
&  0.27087E+01_JPRB, 0.25531E+01_JPRB, 0.23988E+01_JPRB, 0.21556E+01_JPRB, 0.19815E+01_JPRB, 0.19383E+01_JPRB, 0.18601E+01_JPRB ,&
&  0.16869E+01_JPRB, 0.15827E+01_JPRB, 0.13684E+01_JPRB, 0.13070E+01_JPRB, 0.12932E+01_JPRB, 0.12098E+01_JPRB, 0.10959E+01_JPRB ,&
&  0.99684E+00_JPRB, 0.91250E+00_JPRB, 0.84732E+00_JPRB, 0.80248E+00_JPRB, 0.76556E+00_JPRB, 0.74282E+00_JPRB, 0.72515E+00_JPRB ,&
&  0.71120E+00_JPRB, 0.69768E+00_JPRB, 0.66450E+00_JPRB, 0.64668E+00_JPRB, 0.62740E+00_JPRB, 0.61381E+00_JPRB, 0.61302E+00_JPRB ,&
&  0.58455E+00_JPRB, 0.57236E+00_JPRB, 0.54132E+00_JPRB, 0.53843E+00_JPRB, 0.55598E+00_JPRB, 0.59493E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.66929E+00_JPRB, 0.65902E+00_JPRB, 0.61770E+00_JPRB, 0.60558E+00_JPRB, 0.59262E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.54515E+00_JPRB, 0.58015E+00_JPRB, 0.61380E+00_JPRB, 0.63866E+00_JPRB, 0.63470E+00_JPRB, 0.60789E+00_JPRB, 0.54451E+00_JPRB ,&
&  0.52417E+00_JPRB, 0.49303E+00_JPRB, 0.47953E+00_JPRB, 0.46551E+00_JPRB, 0.45554E+00_JPRB, 0.44727E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.45226E+00_JPRB, 0.46253E+00_JPRB, 0.47250E+00_JPRB, 0.47982E+00_JPRB, 0.48542E+00_JPRB, 0.49027E+00_JPRB, 0.49540E+00_JPRB ,&
&  0.49827E+00_JPRB, 0.50053E+00_JPRB, 0.50353E+00_JPRB, 0.50639E+00_JPRB, 0.51099E+00_JPRB, 0.51599E+00_JPRB, 0.51438E+00_JPRB ,&
&  0.50336E+00_JPRB, 0.48974E+00_JPRB, 0.49874E+00_JPRB, 0.51730E+00_JPRB, 0.51053E+00_JPRB, 0.50010E+00_JPRB, 0.45481E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.43906E+00_JPRB, 0.53798E+00_JPRB, 0.75078E+00_JPRB, 0.81674E+00_JPRB, 0.88208E+00_JPRB, 0.92248E+00_JPRB ,&
&  0.87727E+00_JPRB, 0.90428E+00_JPRB, 0.10105E+01_JPRB, 0.12704E+01_JPRB, 0.13484E+01_JPRB, 0.14759E+01_JPRB, 0.16224E+01_JPRB ,&
&  0.19125E+01_JPRB, 0.21036E+01_JPRB, 0.22024E+01_JPRB, 0.18378E+01_JPRB, 0.20763E+01_JPRB, 0.26668E+01_JPRB, 0.23844E+01_JPRB ,&
&  0.22714E+01_JPRB, 0.21849E+01_JPRB, 0.22260E+01_JPRB, 0.20487E+01_JPRB, 0.18084E+01_JPRB, 0.16621E+01_JPRB, 0.16026E+01_JPRB ,&
&  0.15229E+01_JPRB, 0.14466E+01_JPRB, 0.12313E+01_JPRB, 0.11364E+01_JPRB, 0.11240E+01_JPRB, 0.10800E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.99594E+00_JPRB, 0.90448E+00_JPRB, 0.81989E+00_JPRB, 0.73560E+00_JPRB, 0.68007E+00_JPRB, 0.66136E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.65462E+00_JPRB, 0.64603E+00_JPRB, 0.63674E+00_JPRB, 0.62692E+00_JPRB, 0.60478E+00_JPRB, 0.57359E+00_JPRB, 0.55284E+00_JPRB ,&
&  0.53180E+00_JPRB, 0.51059E+00_JPRB, 0.49327E+00_JPRB, 0.49653E+00_JPRB, 0.50561E+00_JPRB, 0.52625E+00_JPRB, 0.54532E+00_JPRB ,&
&  0.55911E+00_JPRB, 0.56439E+00_JPRB, 0.55600E+00_JPRB, 0.55771E+00_JPRB, 0.54261E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.49864E+00_JPRB, 0.53894E+00_JPRB, 0.59963E+00_JPRB, 0.63495E+00_JPRB, 0.63664E+00_JPRB, 0.59442E+00_JPRB, 0.54445E+00_JPRB ,&
&  0.51359E+00_JPRB, 0.49599E+00_JPRB, 0.49125E+00_JPRB, 0.48879E+00_JPRB, 0.48829E+00_JPRB, 0.48872E+00_JPRB, 0.48932E+00_JPRB ,&
&  0.48842E+00_JPRB, 0.48703E+00_JPRB, 0.48626E+00_JPRB, 0.48794E+00_JPRB, 0.49179E+00_JPRB, 0.49383E+00_JPRB, 0.49676E+00_JPRB ,&
&  0.50176E+00_JPRB, 0.50603E+00_JPRB, 0.50673E+00_JPRB, 0.50513E+00_JPRB, 0.50584E+00_JPRB, 0.51059E+00_JPRB, 0.51277E+00_JPRB ,&
&  0.50164E+00_JPRB, 0.48515E+00_JPRB, 0.49068E+00_JPRB, 0.50828E+00_JPRB, 0.50104E+00_JPRB, 0.48323E+00_JPRB, 0.44113E+00_JPRB ,&
&  0.41271E+00_JPRB, 0.39322E+00_JPRB, 0.40566E+00_JPRB, 0.46225E+00_JPRB, 0.56220E+00_JPRB, 0.75378E+00_JPRB, 0.84564E+00_JPRB ,&
&  0.93335E+00_JPRB, 0.99933E+00_JPRB, 0.10586E+01_JPRB, 0.11191E+01_JPRB, 0.12050E+01_JPRB, 0.12924E+01_JPRB, 0.13518E+01_JPRB ,&
&  0.14457E+01_JPRB, 0.15710E+01_JPRB, 0.17581E+01_JPRB, 0.13649E+01_JPRB, 0.13605E+01_JPRB, 0.15643E+01_JPRB, 0.18056E+01_JPRB ,&
&  0.19018E+01_JPRB, 0.17551E+01_JPRB, 0.16757E+01_JPRB, 0.16990E+01_JPRB, 0.15649E+01_JPRB, 0.14485E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.12659E+01_JPRB, 0.11453E+01_JPRB, 0.10055E+01_JPRB, 0.96798E+00_JPRB, 0.96267E+00_JPRB, 0.98250E+00_JPRB ,&
&  0.10382E+01_JPRB, 0.10733E+01_JPRB, 0.95965E+00_JPRB, 0.72906E+00_JPRB, 0.66260E+00_JPRB, 0.62970E+00_JPRB, 0.62519E+00_JPRB ,&
&  0.62719E+00_JPRB, 0.62756E+00_JPRB, 0.62682E+00_JPRB, 0.61751E+00_JPRB, 0.59720E+00_JPRB, 0.57331E+00_JPRB, 0.54997E+00_JPRB ,&
&  0.52984E+00_JPRB, 0.51945E+00_JPRB, 0.51940E+00_JPRB, 0.52182E+00_JPRB, 0.53632E+00_JPRB, 0.55112E+00_JPRB, 0.56666E+00_JPRB ,&
&  0.56337E+00_JPRB, 0.55166E+00_JPRB, 0.53781E+00_JPRB, 0.51022E+00_JPRB, 0.48912E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.50979E+00_JPRB, 0.54347E+00_JPRB, 0.59009E+00_JPRB, 0.62235E+00_JPRB, 0.62516E+00_JPRB, 0.59847E+00_JPRB, 0.57204E+00_JPRB ,&
&  0.55562E+00_JPRB, 0.54945E+00_JPRB, 0.54375E+00_JPRB, 0.53886E+00_JPRB, 0.53454E+00_JPRB, 0.53008E+00_JPRB, 0.52564E+00_JPRB ,&
&  0.52237E+00_JPRB, 0.52001E+00_JPRB, 0.51672E+00_JPRB, 0.51041E+00_JPRB, 0.50689E+00_JPRB, 0.50925E+00_JPRB, 0.51579E+00_JPRB ,&
&  0.52416E+00_JPRB, 0.52713E+00_JPRB, 0.52829E+00_JPRB, 0.52285E+00_JPRB, 0.50987E+00_JPRB, 0.49468E+00_JPRB, 0.48263E+00_JPRB ,&
&  0.47478E+00_JPRB, 0.47831E+00_JPRB, 0.48450E+00_JPRB, 0.50017E+00_JPRB, 0.49123E+00_JPRB, 0.46983E+00_JPRB, 0.42998E+00_JPRB ,&
&  0.40644E+00_JPRB, 0.37874E+00_JPRB, 0.36717E+00_JPRB, 0.37640E+00_JPRB, 0.41288E+00_JPRB, 0.46927E+00_JPRB, 0.56333E+00_JPRB ,&
&  0.73510E+00_JPRB, 0.80493E+00_JPRB, 0.85893E+00_JPRB, 0.91764E+00_JPRB, 0.97352E+00_JPRB, 0.10303E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.11559E+01_JPRB, 0.12497E+01_JPRB, 0.13522E+01_JPRB, 0.13638E+01_JPRB, 0.14115E+01_JPRB, 0.13533E+01_JPRB, 0.15687E+01_JPRB ,&
&  0.20517E+01_JPRB, 0.17922E+01_JPRB, 0.15345E+01_JPRB, 0.13845E+01_JPRB, 0.12926E+01_JPRB, 0.12045E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.10549E+01_JPRB, 0.10127E+01_JPRB, 0.98951E+00_JPRB, 0.92657E+00_JPRB, 0.86166E+00_JPRB, 0.84912E+00_JPRB, 0.90384E+00_JPRB ,&
&  0.10544E+01_JPRB, 0.12789E+01_JPRB, 0.15275E+01_JPRB, 0.10395E+01_JPRB, 0.87989E+00_JPRB, 0.76986E+00_JPRB, 0.70874E+00_JPRB ,&
&  0.68419E+00_JPRB, 0.67113E+00_JPRB, 0.66313E+00_JPRB, 0.64606E+00_JPRB, 0.62320E+00_JPRB, 0.60374E+00_JPRB, 0.58599E+00_JPRB ,&
&  0.57444E+00_JPRB, 0.57301E+00_JPRB, 0.57718E+00_JPRB, 0.58193E+00_JPRB, 0.58549E+00_JPRB, 0.58416E+00_JPRB, 0.58248E+00_JPRB ,&
&  0.57806E+00_JPRB, 0.56409E+00_JPRB, 0.54881E+00_JPRB, 0.52468E+00_JPRB, 0.50738E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.53068E+00_JPRB, 0.55287E+00_JPRB, 0.58327E+00_JPRB, 0.60450E+00_JPRB, 0.61471E+00_JPRB, 0.60267E+00_JPRB, 0.58735E+00_JPRB ,&
&  0.57440E+00_JPRB, 0.56512E+00_JPRB, 0.55784E+00_JPRB, 0.55259E+00_JPRB, 0.54667E+00_JPRB, 0.54114E+00_JPRB, 0.53662E+00_JPRB ,&
&  0.53350E+00_JPRB, 0.52344E+00_JPRB, 0.51930E+00_JPRB, 0.52127E+00_JPRB, 0.52355E+00_JPRB, 0.52823E+00_JPRB, 0.53379E+00_JPRB ,&
&  0.53432E+00_JPRB, 0.52981E+00_JPRB, 0.52589E+00_JPRB, 0.51077E+00_JPRB, 0.49382E+00_JPRB, 0.48506E+00_JPRB, 0.49102E+00_JPRB ,&
&  0.48668E+00_JPRB, 0.47033E+00_JPRB, 0.48881E+00_JPRB, 0.50204E+00_JPRB, 0.48094E+00_JPRB, 0.45751E+00_JPRB, 0.40089E+00_JPRB ,&
&  0.38421E+00_JPRB, 0.36551E+00_JPRB, 0.36308E+00_JPRB, 0.36676E+00_JPRB, 0.38829E+00_JPRB, 0.43204E+00_JPRB, 0.49958E+00_JPRB ,&
&  0.57009E+00_JPRB, 0.69105E+00_JPRB, 0.72662E+00_JPRB, 0.78499E+00_JPRB, 0.84395E+00_JPRB, 0.89844E+00_JPRB, 0.94422E+00_JPRB ,&
&  0.10107E+01_JPRB, 0.11060E+01_JPRB, 0.11855E+01_JPRB, 0.12318E+01_JPRB, 0.12873E+01_JPRB, 0.13363E+01_JPRB, 0.14154E+01_JPRB ,&
&  0.15179E+01_JPRB, 0.12945E+01_JPRB, 0.11595E+01_JPRB, 0.10696E+01_JPRB, 0.10048E+01_JPRB, 0.97970E+00_JPRB, 0.91073E+00_JPRB ,&
&  0.88764E+00_JPRB, 0.87768E+00_JPRB, 0.86035E+00_JPRB, 0.85358E+00_JPRB, 0.84527E+00_JPRB, 0.83274E+00_JPRB, 0.81045E+00_JPRB ,&
&  0.78564E+00_JPRB, 0.79729E+00_JPRB, 0.90347E+00_JPRB, 0.89827E+00_JPRB, 0.87643E+00_JPRB, 0.81145E+00_JPRB, 0.73650E+00_JPRB ,&
&  0.68117E+00_JPRB, 0.65285E+00_JPRB, 0.63781E+00_JPRB, 0.62505E+00_JPRB, 0.61563E+00_JPRB, 0.61037E+00_JPRB, 0.61386E+00_JPRB ,&
&  0.60489E+00_JPRB, 0.60545E+00_JPRB, 0.60741E+00_JPRB, 0.60641E+00_JPRB, 0.59986E+00_JPRB, 0.58824E+00_JPRB, 0.57862E+00_JPRB ,&
&  0.56909E+00_JPRB, 0.55994E+00_JPRB, 0.55032E+00_JPRB, 0.53863E+00_JPRB, 0.53030E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.54668E+00_JPRB, 0.55527E+00_JPRB, 0.57605E+00_JPRB, 0.59317E+00_JPRB, 0.60440E+00_JPRB, 0.60265E+00_JPRB, 0.59267E+00_JPRB ,&
&  0.58362E+00_JPRB, 0.57681E+00_JPRB, 0.56959E+00_JPRB, 0.56262E+00_JPRB, 0.55429E+00_JPRB, 0.54719E+00_JPRB, 0.54192E+00_JPRB ,&
&  0.53875E+00_JPRB, 0.53792E+00_JPRB, 0.53809E+00_JPRB, 0.53810E+00_JPRB, 0.53886E+00_JPRB, 0.54055E+00_JPRB, 0.54134E+00_JPRB ,&
&  0.54263E+00_JPRB, 0.53540E+00_JPRB, 0.51532E+00_JPRB, 0.49703E+00_JPRB, 0.49090E+00_JPRB, 0.48301E+00_JPRB, 0.49170E+00_JPRB ,&
&  0.49323E+00_JPRB, 0.49537E+00_JPRB, 0.49470E+00_JPRB, 0.49455E+00_JPRB, 0.47523E+00_JPRB, 0.39121E+00_JPRB, 0.39068E+00_JPRB ,&
&  0.37972E+00_JPRB, 0.36834E+00_JPRB, 0.36969E+00_JPRB, 0.37445E+00_JPRB, 0.38564E+00_JPRB, 0.41418E+00_JPRB, 0.45918E+00_JPRB ,&
&  0.50160E+00_JPRB, 0.66187E+00_JPRB, 0.70512E+00_JPRB, 0.75184E+00_JPRB, 0.79375E+00_JPRB, 0.84654E+00_JPRB, 0.91009E+00_JPRB ,&
&  0.95857E+00_JPRB, 0.10125E+01_JPRB, 0.10617E+01_JPRB, 0.11047E+01_JPRB, 0.11310E+01_JPRB, 0.11441E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11384E+01_JPRB, 0.10262E+01_JPRB, 0.94764E+00_JPRB, 0.89775E+00_JPRB, 0.84564E+00_JPRB, 0.81035E+00_JPRB, 0.78755E+00_JPRB ,&
&  0.78665E+00_JPRB, 0.76709E+00_JPRB, 0.76580E+00_JPRB, 0.75047E+00_JPRB, 0.73993E+00_JPRB, 0.73097E+00_JPRB, 0.71917E+00_JPRB ,&
&  0.70748E+00_JPRB, 0.70679E+00_JPRB, 0.71924E+00_JPRB, 0.71634E+00_JPRB, 0.72216E+00_JPRB, 0.71331E+00_JPRB, 0.69509E+00_JPRB ,&
&  0.66938E+00_JPRB, 0.66164E+00_JPRB, 0.64940E+00_JPRB, 0.64044E+00_JPRB, 0.63003E+00_JPRB, 0.62773E+00_JPRB, 0.62773E+00_JPRB ,&
&  0.62728E+00_JPRB, 0.62255E+00_JPRB, 0.61685E+00_JPRB, 0.61243E+00_JPRB, 0.60711E+00_JPRB, 0.59938E+00_JPRB, 0.59017E+00_JPRB ,&
&  0.58048E+00_JPRB, 0.57465E+00_JPRB, 0.56463E+00_JPRB, 0.55640E+00_JPRB, 0.55053E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.56389E+00_JPRB, 0.56375E+00_JPRB, 0.56892E+00_JPRB, 0.57401E+00_JPRB, 0.57817E+00_JPRB, 0.57904E+00_JPRB, 0.57322E+00_JPRB ,&
&  0.56815E+00_JPRB, 0.56415E+00_JPRB, 0.56108E+00_JPRB, 0.55937E+00_JPRB, 0.55974E+00_JPRB, 0.55712E+00_JPRB, 0.55218E+00_JPRB ,&
&  0.54717E+00_JPRB, 0.54384E+00_JPRB, 0.54181E+00_JPRB, 0.54019E+00_JPRB, 0.54117E+00_JPRB, 0.54041E+00_JPRB, 0.53965E+00_JPRB ,&
&  0.53830E+00_JPRB, 0.52687E+00_JPRB, 0.50044E+00_JPRB, 0.48942E+00_JPRB, 0.48045E+00_JPRB, 0.48236E+00_JPRB, 0.48544E+00_JPRB ,&
&  0.49070E+00_JPRB, 0.43314E+00_JPRB, 0.43708E+00_JPRB, 0.42146E+00_JPRB, 0.40980E+00_JPRB, 0.40728E+00_JPRB, 0.39795E+00_JPRB ,&
&  0.39235E+00_JPRB, 0.38544E+00_JPRB, 0.38826E+00_JPRB, 0.39709E+00_JPRB, 0.40384E+00_JPRB, 0.42125E+00_JPRB, 0.44875E+00_JPRB ,&
&  0.48947E+00_JPRB, 0.62846E+00_JPRB, 0.69108E+00_JPRB, 0.73419E+00_JPRB, 0.76784E+00_JPRB, 0.80731E+00_JPRB, 0.84872E+00_JPRB ,&
&  0.88339E+00_JPRB, 0.92560E+00_JPRB, 0.95135E+00_JPRB, 0.95283E+00_JPRB, 0.97746E+00_JPRB, 0.97342E+00_JPRB, 0.95176E+00_JPRB ,&
&  0.90738E+00_JPRB, 0.84329E+00_JPRB, 0.79219E+00_JPRB, 0.75934E+00_JPRB, 0.73486E+00_JPRB, 0.71930E+00_JPRB, 0.71115E+00_JPRB ,&
&  0.71756E+00_JPRB, 0.71344E+00_JPRB, 0.70510E+00_JPRB, 0.68437E+00_JPRB, 0.67069E+00_JPRB, 0.66097E+00_JPRB, 0.65712E+00_JPRB ,&
&  0.65712E+00_JPRB, 0.66070E+00_JPRB, 0.67158E+00_JPRB, 0.67841E+00_JPRB, 0.68028E+00_JPRB, 0.67665E+00_JPRB, 0.67840E+00_JPRB ,&
&  0.67611E+00_JPRB, 0.66808E+00_JPRB, 0.65908E+00_JPRB, 0.64818E+00_JPRB, 0.63497E+00_JPRB, 0.62414E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.61390E+00_JPRB, 0.60927E+00_JPRB, 0.60507E+00_JPRB, 0.60232E+00_JPRB, 0.60206E+00_JPRB, 0.60149E+00_JPRB, 0.59974E+00_JPRB ,&
&  0.59096E+00_JPRB, 0.58205E+00_JPRB, 0.57398E+00_JPRB, 0.57127E+00_JPRB, 0.56816E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.55475E+00_JPRB, 0.55183E+00_JPRB, 0.55063E+00_JPRB, 0.54964E+00_JPRB, 0.54951E+00_JPRB, 0.54978E+00_JPRB, 0.55058E+00_JPRB ,&
&  0.55221E+00_JPRB, 0.55493E+00_JPRB, 0.55704E+00_JPRB, 0.55785E+00_JPRB, 0.55986E+00_JPRB, 0.56307E+00_JPRB, 0.56370E+00_JPRB ,&
&  0.55903E+00_JPRB, 0.55290E+00_JPRB, 0.54672E+00_JPRB, 0.54023E+00_JPRB, 0.53453E+00_JPRB, 0.53009E+00_JPRB, 0.52329E+00_JPRB ,&
&  0.51863E+00_JPRB, 0.51035E+00_JPRB, 0.47777E+00_JPRB, 0.47761E+00_JPRB, 0.47078E+00_JPRB, 0.46962E+00_JPRB, 0.47383E+00_JPRB ,&
&  0.46794E+00_JPRB, 0.46800E+00_JPRB, 0.45842E+00_JPRB, 0.44171E+00_JPRB, 0.42995E+00_JPRB, 0.41305E+00_JPRB, 0.41178E+00_JPRB ,&
&  0.40910E+00_JPRB, 0.41340E+00_JPRB, 0.42463E+00_JPRB, 0.43125E+00_JPRB, 0.43486E+00_JPRB, 0.44105E+00_JPRB, 0.45978E+00_JPRB ,&
&  0.49775E+00_JPRB, 0.55164E+00_JPRB, 0.62402E+00_JPRB, 0.66237E+00_JPRB, 0.69538E+00_JPRB, 0.72771E+00_JPRB, 0.75881E+00_JPRB ,&
&  0.78402E+00_JPRB, 0.80553E+00_JPRB, 0.81379E+00_JPRB, 0.81805E+00_JPRB, 0.80987E+00_JPRB, 0.78929E+00_JPRB, 0.76256E+00_JPRB ,&
&  0.73265E+00_JPRB, 0.71097E+00_JPRB, 0.69460E+00_JPRB, 0.68682E+00_JPRB, 0.68243E+00_JPRB, 0.67937E+00_JPRB, 0.67768E+00_JPRB ,&
&  0.67330E+00_JPRB, 0.65681E+00_JPRB, 0.64153E+00_JPRB, 0.62316E+00_JPRB, 0.61329E+00_JPRB, 0.60521E+00_JPRB, 0.60257E+00_JPRB ,&
&  0.60181E+00_JPRB, 0.60619E+00_JPRB, 0.61430E+00_JPRB, 0.61625E+00_JPRB, 0.62285E+00_JPRB, 0.63328E+00_JPRB, 0.63331E+00_JPRB ,&
&  0.63337E+00_JPRB, 0.62953E+00_JPRB, 0.62209E+00_JPRB, 0.61268E+00_JPRB, 0.60152E+00_JPRB, 0.59059E+00_JPRB, 0.58032E+00_JPRB ,&
&  0.57320E+00_JPRB, 0.56856E+00_JPRB, 0.56820E+00_JPRB, 0.56873E+00_JPRB, 0.57149E+00_JPRB, 0.57217E+00_JPRB, 0.56967E+00_JPRB ,&
&  0.56306E+00_JPRB, 0.55862E+00_JPRB, 0.55502E+00_JPRB, 0.55558E+00_JPRB, 0.55582E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.54858E+00_JPRB, 0.54943E+00_JPRB, 0.54958E+00_JPRB, 0.54867E+00_JPRB, 0.54848E+00_JPRB, 0.54903E+00_JPRB, 0.55009E+00_JPRB ,&
&  0.55147E+00_JPRB, 0.55334E+00_JPRB, 0.55532E+00_JPRB, 0.55667E+00_JPRB, 0.55749E+00_JPRB, 0.56379E+00_JPRB, 0.56621E+00_JPRB ,&
&  0.56545E+00_JPRB, 0.56287E+00_JPRB, 0.55956E+00_JPRB, 0.55613E+00_JPRB, 0.55173E+00_JPRB, 0.54594E+00_JPRB, 0.53648E+00_JPRB ,&
&  0.53086E+00_JPRB, 0.52468E+00_JPRB, 0.51763E+00_JPRB, 0.51548E+00_JPRB, 0.51324E+00_JPRB, 0.48758E+00_JPRB, 0.48572E+00_JPRB ,&
&  0.50134E+00_JPRB, 0.50950E+00_JPRB, 0.51343E+00_JPRB, 0.51838E+00_JPRB, 0.50894E+00_JPRB, 0.48434E+00_JPRB, 0.46656E+00_JPRB ,&
&  0.43542E+00_JPRB, 0.45527E+00_JPRB, 0.46949E+00_JPRB, 0.47551E+00_JPRB, 0.47761E+00_JPRB, 0.48120E+00_JPRB, 0.50909E+00_JPRB ,&
&  0.56877E+00_JPRB, 0.58139E+00_JPRB, 0.59596E+00_JPRB, 0.61202E+00_JPRB, 0.62910E+00_JPRB, 0.64241E+00_JPRB, 0.65814E+00_JPRB ,&
&  0.66923E+00_JPRB, 0.67785E+00_JPRB, 0.67994E+00_JPRB, 0.67986E+00_JPRB, 0.67243E+00_JPRB, 0.66601E+00_JPRB, 0.65943E+00_JPRB ,&
&  0.65628E+00_JPRB, 0.65355E+00_JPRB, 0.65035E+00_JPRB, 0.64480E+00_JPRB, 0.63631E+00_JPRB, 0.62726E+00_JPRB, 0.61697E+00_JPRB ,&
&  0.60633E+00_JPRB, 0.59326E+00_JPRB, 0.58307E+00_JPRB, 0.57388E+00_JPRB, 0.57009E+00_JPRB, 0.56599E+00_JPRB, 0.56597E+00_JPRB ,&
&  0.56647E+00_JPRB, 0.56986E+00_JPRB, 0.57414E+00_JPRB, 0.58178E+00_JPRB, 0.58773E+00_JPRB, 0.59266E+00_JPRB, 0.59506E+00_JPRB ,&
&  0.59631E+00_JPRB, 0.59523E+00_JPRB, 0.59305E+00_JPRB, 0.58853E+00_JPRB, 0.58433E+00_JPRB, 0.58041E+00_JPRB, 0.57606E+00_JPRB ,&
&  0.57220E+00_JPRB, 0.56826E+00_JPRB, 0.56502E+00_JPRB, 0.56167E+00_JPRB, 0.55833E+00_JPRB, 0.55392E+00_JPRB, 0.54961E+00_JPRB ,&
&  0.54541E+00_JPRB, 0.54353E+00_JPRB, 0.54269E+00_JPRB, 0.54408E+00_JPRB, 0.54598E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.56183E+00_JPRB, 0.56273E+00_JPRB, 0.56330E+00_JPRB, 0.56325E+00_JPRB, 0.56356E+00_JPRB, 0.56369E+00_JPRB, 0.56332E+00_JPRB ,&
&  0.56222E+00_JPRB, 0.56185E+00_JPRB, 0.56216E+00_JPRB, 0.56222E+00_JPRB, 0.56165E+00_JPRB, 0.56451E+00_JPRB, 0.56992E+00_JPRB ,&
&  0.57098E+00_JPRB, 0.57133E+00_JPRB, 0.57119E+00_JPRB, 0.57108E+00_JPRB, 0.57038E+00_JPRB, 0.56909E+00_JPRB, 0.56771E+00_JPRB ,&
&  0.56771E+00_JPRB, 0.56727E+00_JPRB, 0.56625E+00_JPRB, 0.56444E+00_JPRB, 0.56482E+00_JPRB, 0.56507E+00_JPRB, 0.56481E+00_JPRB ,&
&  0.56426E+00_JPRB, 0.56275E+00_JPRB, 0.56127E+00_JPRB, 0.56085E+00_JPRB, 0.56405E+00_JPRB, 0.56802E+00_JPRB, 0.57287E+00_JPRB ,&
&  0.57818E+00_JPRB, 0.58510E+00_JPRB, 0.59223E+00_JPRB, 0.59773E+00_JPRB, 0.60097E+00_JPRB, 0.60613E+00_JPRB, 0.61024E+00_JPRB ,&
&  0.61151E+00_JPRB, 0.61002E+00_JPRB, 0.60856E+00_JPRB, 0.60930E+00_JPRB, 0.60999E+00_JPRB, 0.61010E+00_JPRB, 0.61149E+00_JPRB ,&
&  0.61339E+00_JPRB, 0.61500E+00_JPRB, 0.61617E+00_JPRB, 0.61679E+00_JPRB, 0.61601E+00_JPRB, 0.61454E+00_JPRB, 0.61214E+00_JPRB ,&
&  0.60982E+00_JPRB, 0.60659E+00_JPRB, 0.60361E+00_JPRB, 0.60068E+00_JPRB, 0.59745E+00_JPRB, 0.59323E+00_JPRB, 0.58808E+00_JPRB ,&
&  0.58389E+00_JPRB, 0.58125E+00_JPRB, 0.57986E+00_JPRB, 0.57697E+00_JPRB, 0.57464E+00_JPRB, 0.57245E+00_JPRB, 0.57292E+00_JPRB ,&
&  0.57365E+00_JPRB, 0.57630E+00_JPRB, 0.57879E+00_JPRB, 0.58293E+00_JPRB, 0.58623E+00_JPRB, 0.58905E+00_JPRB, 0.59021E+00_JPRB ,&
&  0.59131E+00_JPRB, 0.59120E+00_JPRB, 0.59035E+00_JPRB, 0.58859E+00_JPRB, 0.58691E+00_JPRB, 0.58474E+00_JPRB, 0.58191E+00_JPRB ,&
&  0.57868E+00_JPRB, 0.57591E+00_JPRB, 0.57279E+00_JPRB, 0.56930E+00_JPRB, 0.56632E+00_JPRB, 0.56342E+00_JPRB, 0.56139E+00_JPRB ,&
&  0.55969E+00_JPRB, 0.55896E+00_JPRB, 0.55866E+00_JPRB, 0.55951E+00_JPRB, 0.56067E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.57221E+00_JPRB, 0.57316E+00_JPRB, 0.57405E+00_JPRB, 0.57475E+00_JPRB, 0.57547E+00_JPRB, 0.57602E+00_JPRB, 0.57647E+00_JPRB ,&
&  0.57652E+00_JPRB, 0.57728E+00_JPRB, 0.57837E+00_JPRB, 0.57942E+00_JPRB, 0.58036E+00_JPRB, 0.58131E+00_JPRB, 0.58213E+00_JPRB ,&
&  0.58346E+00_JPRB, 0.58764E+00_JPRB, 0.59100E+00_JPRB, 0.59302E+00_JPRB, 0.59482E+00_JPRB, 0.59659E+00_JPRB, 0.59822E+00_JPRB ,&
&  0.59974E+00_JPRB, 0.60095E+00_JPRB, 0.60199E+00_JPRB, 0.60284E+00_JPRB, 0.60380E+00_JPRB, 0.60474E+00_JPRB, 0.60573E+00_JPRB ,&
&  0.60686E+00_JPRB, 0.60855E+00_JPRB, 0.61052E+00_JPRB, 0.61256E+00_JPRB, 0.61481E+00_JPRB, 0.61784E+00_JPRB, 0.62054E+00_JPRB ,&
&  0.62279E+00_JPRB, 0.62430E+00_JPRB, 0.62411E+00_JPRB, 0.62260E+00_JPRB, 0.62027E+00_JPRB, 0.61768E+00_JPRB, 0.61480E+00_JPRB ,&
&  0.61135E+00_JPRB, 0.60814E+00_JPRB, 0.60547E+00_JPRB, 0.60307E+00_JPRB, 0.60088E+00_JPRB, 0.59905E+00_JPRB, 0.59726E+00_JPRB ,&
&  0.59564E+00_JPRB, 0.59389E+00_JPRB, 0.59228E+00_JPRB, 0.59072E+00_JPRB, 0.58912E+00_JPRB, 0.58723E+00_JPRB, 0.58543E+00_JPRB ,&
&  0.58415E+00_JPRB, 0.58295E+00_JPRB, 0.58211E+00_JPRB, 0.58124E+00_JPRB, 0.58056E+00_JPRB, 0.58046E+00_JPRB, 0.58061E+00_JPRB ,&
&  0.58070E+00_JPRB, 0.58093E+00_JPRB, 0.58145E+00_JPRB, 0.58163E+00_JPRB, 0.58156E+00_JPRB, 0.58118E+00_JPRB, 0.58082E+00_JPRB ,&
&  0.58054E+00_JPRB, 0.58082E+00_JPRB, 0.58079E+00_JPRB, 0.58105E+00_JPRB, 0.58127E+00_JPRB, 0.58157E+00_JPRB, 0.58199E+00_JPRB ,&
&  0.58196E+00_JPRB, 0.58196E+00_JPRB, 0.58172E+00_JPRB, 0.58160E+00_JPRB, 0.58133E+00_JPRB, 0.58072E+00_JPRB, 0.57977E+00_JPRB ,&
&  0.57867E+00_JPRB, 0.57744E+00_JPRB, 0.57621E+00_JPRB, 0.57483E+00_JPRB, 0.57387E+00_JPRB, 0.57319E+00_JPRB, 0.57249E+00_JPRB ,&
&  0.57176E+00_JPRB, 0.57138E+00_JPRB, 0.57119E+00_JPRB, 0.57141E+00_JPRB, 0.57155E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB, 0.59195E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB, 0.15480E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.16021E+00_JPRB, 0.16021E+00_JPRB, 0.16008E+00_JPRB, 0.15947E+00_JPRB, 0.15751E+00_JPRB, 0.15717E+00_JPRB, 0.15715E+00_JPRB ,&
&  0.15732E+00_JPRB, 0.15742E+00_JPRB, 0.15820E+00_JPRB, 0.15905E+00_JPRB, 0.15987E+00_JPRB, 0.16051E+00_JPRB, 0.16104E+00_JPRB ,&
&  0.16134E+00_JPRB, 0.16164E+00_JPRB, 0.16066E+00_JPRB, 0.16018E+00_JPRB, 0.16005E+00_JPRB, 0.16006E+00_JPRB, 0.16006E+00_JPRB ,&
&  0.16001E+00_JPRB, 0.15999E+00_JPRB, 0.15987E+00_JPRB, 0.16003E+00_JPRB, 0.16030E+00_JPRB, 0.16052E+00_JPRB, 0.16079E+00_JPRB ,&
&  0.16099E+00_JPRB, 0.16117E+00_JPRB, 0.16128E+00_JPRB, 0.16136E+00_JPRB, 0.16163E+00_JPRB, 0.16175E+00_JPRB, 0.16166E+00_JPRB ,&
&  0.16148E+00_JPRB, 0.16113E+00_JPRB, 0.16075E+00_JPRB, 0.16035E+00_JPRB, 0.16006E+00_JPRB, 0.15978E+00_JPRB, 0.15951E+00_JPRB ,&
&  0.15924E+00_JPRB, 0.15894E+00_JPRB, 0.15862E+00_JPRB, 0.15827E+00_JPRB, 0.15793E+00_JPRB, 0.15768E+00_JPRB, 0.15741E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15698E+00_JPRB, 0.15676E+00_JPRB, 0.15647E+00_JPRB, 0.15617E+00_JPRB, 0.15588E+00_JPRB, 0.15555E+00_JPRB ,&
&  0.15514E+00_JPRB, 0.15485E+00_JPRB, 0.15456E+00_JPRB, 0.15431E+00_JPRB, 0.15410E+00_JPRB, 0.15387E+00_JPRB, 0.15368E+00_JPRB ,&
&  0.15345E+00_JPRB, 0.15325E+00_JPRB, 0.15301E+00_JPRB, 0.15279E+00_JPRB, 0.15259E+00_JPRB, 0.15242E+00_JPRB, 0.15228E+00_JPRB ,&
&  0.15215E+00_JPRB, 0.15202E+00_JPRB, 0.15191E+00_JPRB, 0.15193E+00_JPRB, 0.15192E+00_JPRB, 0.15195E+00_JPRB, 0.15198E+00_JPRB ,&
&  0.15202E+00_JPRB, 0.15205E+00_JPRB, 0.15207E+00_JPRB, 0.15198E+00_JPRB, 0.15212E+00_JPRB, 0.15231E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15294E+00_JPRB, 0.15338E+00_JPRB, 0.15383E+00_JPRB, 0.15431E+00_JPRB, 0.15486E+00_JPRB, 0.15532E+00_JPRB, 0.15583E+00_JPRB ,&
&  0.15641E+00_JPRB, 0.15701E+00_JPRB, 0.15745E+00_JPRB, 0.15796E+00_JPRB, 0.15875E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.17091E+00_JPRB, 0.17376E+00_JPRB, 0.17565E+00_JPRB, 0.17740E+00_JPRB, 0.17257E+00_JPRB, 0.18380E+00_JPRB, 0.18063E+00_JPRB ,&
&  0.18005E+00_JPRB, 0.17836E+00_JPRB, 0.17973E+00_JPRB, 0.17960E+00_JPRB, 0.17887E+00_JPRB, 0.17784E+00_JPRB, 0.17659E+00_JPRB ,&
&  0.17541E+00_JPRB, 0.17396E+00_JPRB, 0.17299E+00_JPRB, 0.17195E+00_JPRB, 0.17113E+00_JPRB, 0.17033E+00_JPRB, 0.16978E+00_JPRB ,&
&  0.16975E+00_JPRB, 0.17047E+00_JPRB, 0.17178E+00_JPRB, 0.17125E+00_JPRB, 0.17130E+00_JPRB, 0.17172E+00_JPRB, 0.17233E+00_JPRB ,&
&  0.17262E+00_JPRB, 0.17277E+00_JPRB, 0.17254E+00_JPRB, 0.17157E+00_JPRB, 0.17149E+00_JPRB, 0.17145E+00_JPRB, 0.17149E+00_JPRB ,&
&  0.17136E+00_JPRB, 0.17103E+00_JPRB, 0.17017E+00_JPRB, 0.16938E+00_JPRB, 0.16855E+00_JPRB, 0.16758E+00_JPRB, 0.16645E+00_JPRB ,&
&  0.16550E+00_JPRB, 0.16476E+00_JPRB, 0.16389E+00_JPRB, 0.16303E+00_JPRB, 0.16229E+00_JPRB, 0.16158E+00_JPRB, 0.16092E+00_JPRB ,&
&  0.16049E+00_JPRB, 0.16018E+00_JPRB, 0.15982E+00_JPRB, 0.15931E+00_JPRB, 0.15873E+00_JPRB, 0.15817E+00_JPRB, 0.15767E+00_JPRB ,&
&  0.15733E+00_JPRB, 0.15680E+00_JPRB, 0.15627E+00_JPRB, 0.15569E+00_JPRB, 0.15525E+00_JPRB, 0.15454E+00_JPRB, 0.15391E+00_JPRB ,&
&  0.15331E+00_JPRB, 0.15277E+00_JPRB, 0.15237E+00_JPRB, 0.15207E+00_JPRB, 0.15181E+00_JPRB, 0.15163E+00_JPRB, 0.15152E+00_JPRB ,&
&  0.15146E+00_JPRB, 0.15143E+00_JPRB, 0.15149E+00_JPRB, 0.15148E+00_JPRB, 0.15144E+00_JPRB, 0.15146E+00_JPRB, 0.15174E+00_JPRB ,&
&  0.15199E+00_JPRB, 0.15217E+00_JPRB, 0.15229E+00_JPRB, 0.15206E+00_JPRB, 0.15244E+00_JPRB, 0.15277E+00_JPRB, 0.15317E+00_JPRB ,&
&  0.15353E+00_JPRB, 0.15427E+00_JPRB, 0.15515E+00_JPRB, 0.15622E+00_JPRB, 0.15822E+00_JPRB, 0.16002E+00_JPRB, 0.16192E+00_JPRB ,&
&  0.16404E+00_JPRB, 0.16711E+00_JPRB, 0.16845E+00_JPRB, 0.16937E+00_JPRB, 0.17015E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.19565E+00_JPRB, 0.19708E+00_JPRB, 0.19847E+00_JPRB, 0.19978E+00_JPRB, 0.20121E+00_JPRB, 0.20176E+00_JPRB, 0.20167E+00_JPRB ,&
&  0.19937E+00_JPRB, 0.19875E+00_JPRB, 0.19722E+00_JPRB, 0.19535E+00_JPRB, 0.19386E+00_JPRB, 0.19118E+00_JPRB, 0.18851E+00_JPRB ,&
&  0.18579E+00_JPRB, 0.18316E+00_JPRB, 0.18080E+00_JPRB, 0.17757E+00_JPRB, 0.17446E+00_JPRB, 0.17141E+00_JPRB, 0.17042E+00_JPRB ,&
&  0.17000E+00_JPRB, 0.17044E+00_JPRB, 0.17203E+00_JPRB, 0.17338E+00_JPRB, 0.17534E+00_JPRB, 0.17745E+00_JPRB, 0.18064E+00_JPRB ,&
&  0.18377E+00_JPRB, 0.18610E+00_JPRB, 0.18648E+00_JPRB, 0.18540E+00_JPRB, 0.18442E+00_JPRB, 0.18408E+00_JPRB, 0.18362E+00_JPRB ,&
&  0.18280E+00_JPRB, 0.18227E+00_JPRB, 0.18160E+00_JPRB, 0.18101E+00_JPRB, 0.17956E+00_JPRB, 0.17708E+00_JPRB, 0.17460E+00_JPRB ,&
&  0.17230E+00_JPRB, 0.17027E+00_JPRB, 0.16837E+00_JPRB, 0.16674E+00_JPRB, 0.16577E+00_JPRB, 0.16480E+00_JPRB, 0.16363E+00_JPRB ,&
&  0.16293E+00_JPRB, 0.16219E+00_JPRB, 0.16133E+00_JPRB, 0.16068E+00_JPRB, 0.15994E+00_JPRB, 0.15941E+00_JPRB, 0.15881E+00_JPRB ,&
&  0.15833E+00_JPRB, 0.15763E+00_JPRB, 0.15702E+00_JPRB, 0.15636E+00_JPRB, 0.15605E+00_JPRB, 0.15533E+00_JPRB, 0.15464E+00_JPRB ,&
&  0.15380E+00_JPRB, 0.15328E+00_JPRB, 0.15233E+00_JPRB, 0.15176E+00_JPRB, 0.15110E+00_JPRB, 0.15052E+00_JPRB, 0.15014E+00_JPRB ,&
&  0.15023E+00_JPRB, 0.15027E+00_JPRB, 0.14994E+00_JPRB, 0.14991E+00_JPRB, 0.14999E+00_JPRB, 0.15043E+00_JPRB, 0.15165E+00_JPRB ,&
&  0.15243E+00_JPRB, 0.15331E+00_JPRB, 0.15416E+00_JPRB, 0.15482E+00_JPRB, 0.15593E+00_JPRB, 0.15700E+00_JPRB, 0.15810E+00_JPRB ,&
&  0.15938E+00_JPRB, 0.16049E+00_JPRB, 0.16180E+00_JPRB, 0.16335E+00_JPRB, 0.16703E+00_JPRB, 0.16971E+00_JPRB, 0.17262E+00_JPRB ,&
&  0.17482E+00_JPRB, 0.17410E+00_JPRB, 0.18777E+00_JPRB, 0.19044E+00_JPRB, 0.19242E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.20434E+00_JPRB, 0.20518E+00_JPRB, 0.20521E+00_JPRB, 0.20616E+00_JPRB, 0.20699E+00_JPRB, 0.20801E+00_JPRB, 0.20826E+00_JPRB ,&
&  0.20762E+00_JPRB, 0.20329E+00_JPRB, 0.20088E+00_JPRB, 0.19762E+00_JPRB, 0.19447E+00_JPRB, 0.19122E+00_JPRB, 0.18853E+00_JPRB ,&
&  0.18677E+00_JPRB, 0.18541E+00_JPRB, 0.18416E+00_JPRB, 0.18246E+00_JPRB, 0.18041E+00_JPRB, 0.17836E+00_JPRB, 0.17648E+00_JPRB ,&
&  0.17479E+00_JPRB, 0.17481E+00_JPRB, 0.17595E+00_JPRB, 0.17806E+00_JPRB, 0.18192E+00_JPRB, 0.18501E+00_JPRB, 0.18898E+00_JPRB ,&
&  0.19531E+00_JPRB, 0.19256E+00_JPRB, 0.19271E+00_JPRB, 0.19289E+00_JPRB, 0.19257E+00_JPRB, 0.19232E+00_JPRB, 0.18919E+00_JPRB ,&
&  0.19223E+00_JPRB, 0.19242E+00_JPRB, 0.19573E+00_JPRB, 0.19503E+00_JPRB, 0.19070E+00_JPRB, 0.18705E+00_JPRB, 0.18217E+00_JPRB ,&
&  0.17854E+00_JPRB, 0.17422E+00_JPRB, 0.17132E+00_JPRB, 0.16874E+00_JPRB, 0.16582E+00_JPRB, 0.16461E+00_JPRB, 0.16259E+00_JPRB ,&
&  0.16134E+00_JPRB, 0.16106E+00_JPRB, 0.16015E+00_JPRB, 0.16008E+00_JPRB, 0.15933E+00_JPRB, 0.15827E+00_JPRB, 0.15755E+00_JPRB ,&
&  0.15691E+00_JPRB, 0.15623E+00_JPRB, 0.15598E+00_JPRB, 0.15573E+00_JPRB, 0.15586E+00_JPRB, 0.15589E+00_JPRB, 0.15598E+00_JPRB ,&
&  0.15620E+00_JPRB, 0.15634E+00_JPRB, 0.15596E+00_JPRB, 0.15579E+00_JPRB, 0.15494E+00_JPRB, 0.15420E+00_JPRB, 0.15307E+00_JPRB ,&
&  0.15239E+00_JPRB, 0.15200E+00_JPRB, 0.15144E+00_JPRB, 0.15106E+00_JPRB, 0.15099E+00_JPRB, 0.15147E+00_JPRB, 0.15207E+00_JPRB ,&
&  0.15276E+00_JPRB, 0.15397E+00_JPRB, 0.15498E+00_JPRB, 0.15633E+00_JPRB, 0.15781E+00_JPRB, 0.15900E+00_JPRB, 0.16068E+00_JPRB ,&
&  0.16236E+00_JPRB, 0.16409E+00_JPRB, 0.16698E+00_JPRB, 0.17022E+00_JPRB, 0.17452E+00_JPRB, 0.17952E+00_JPRB, 0.18252E+00_JPRB ,&
&  0.18595E+00_JPRB, 0.19317E+00_JPRB, 0.19579E+00_JPRB, 0.20249E+00_JPRB, 0.20355E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.20945E+00_JPRB, 0.20934E+00_JPRB, 0.20869E+00_JPRB, 0.20821E+00_JPRB, 0.20865E+00_JPRB, 0.20940E+00_JPRB, 0.21031E+00_JPRB ,&
&  0.20687E+00_JPRB, 0.20416E+00_JPRB, 0.19983E+00_JPRB, 0.19505E+00_JPRB, 0.19088E+00_JPRB, 0.18746E+00_JPRB, 0.18545E+00_JPRB ,&
&  0.18520E+00_JPRB, 0.18707E+00_JPRB, 0.18864E+00_JPRB, 0.19047E+00_JPRB, 0.19062E+00_JPRB, 0.19080E+00_JPRB, 0.18952E+00_JPRB ,&
&  0.18814E+00_JPRB, 0.18757E+00_JPRB, 0.18783E+00_JPRB, 0.18961E+00_JPRB, 0.19231E+00_JPRB, 0.19421E+00_JPRB, 0.19643E+00_JPRB ,&
&  0.19936E+00_JPRB, 0.19884E+00_JPRB, 0.19961E+00_JPRB, 0.19908E+00_JPRB, 0.19927E+00_JPRB, 0.20002E+00_JPRB, 0.20037E+00_JPRB ,&
&  0.20110E+00_JPRB, 0.20185E+00_JPRB, 0.20168E+00_JPRB, 0.20099E+00_JPRB, 0.19689E+00_JPRB, 0.19411E+00_JPRB, 0.18947E+00_JPRB ,&
&  0.18402E+00_JPRB, 0.17925E+00_JPRB, 0.17395E+00_JPRB, 0.17027E+00_JPRB, 0.16605E+00_JPRB, 0.16353E+00_JPRB, 0.16158E+00_JPRB ,&
&  0.15975E+00_JPRB, 0.15903E+00_JPRB, 0.15805E+00_JPRB, 0.15700E+00_JPRB, 0.15679E+00_JPRB, 0.15579E+00_JPRB, 0.15570E+00_JPRB ,&
&  0.15593E+00_JPRB, 0.15614E+00_JPRB, 0.15702E+00_JPRB, 0.15747E+00_JPRB, 0.15825E+00_JPRB, 0.15899E+00_JPRB, 0.15959E+00_JPRB ,&
&  0.16070E+00_JPRB, 0.16204E+00_JPRB, 0.16206E+00_JPRB, 0.16228E+00_JPRB, 0.16145E+00_JPRB, 0.15978E+00_JPRB, 0.15793E+00_JPRB ,&
&  0.15600E+00_JPRB, 0.15449E+00_JPRB, 0.15401E+00_JPRB, 0.15335E+00_JPRB, 0.15350E+00_JPRB, 0.15386E+00_JPRB, 0.15407E+00_JPRB ,&
&  0.15473E+00_JPRB, 0.15532E+00_JPRB, 0.15612E+00_JPRB, 0.15783E+00_JPRB, 0.15906E+00_JPRB, 0.16062E+00_JPRB, 0.16224E+00_JPRB ,&
&  0.16352E+00_JPRB, 0.16584E+00_JPRB, 0.16790E+00_JPRB, 0.17071E+00_JPRB, 0.17713E+00_JPRB, 0.18295E+00_JPRB, 0.19333E+00_JPRB ,&
&  0.19883E+00_JPRB, 0.21129E+00_JPRB, 0.20984E+00_JPRB, 0.20891E+00_JPRB, 0.20982E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.21585E+00_JPRB, 0.21464E+00_JPRB, 0.21353E+00_JPRB, 0.21232E+00_JPRB, 0.21311E+00_JPRB, 0.21420E+00_JPRB, 0.21558E+00_JPRB ,&
&  0.21716E+00_JPRB, 0.21754E+00_JPRB, 0.21729E+00_JPRB, 0.21528E+00_JPRB, 0.21404E+00_JPRB, 0.20772E+00_JPRB, 0.20539E+00_JPRB ,&
&  0.20365E+00_JPRB, 0.20529E+00_JPRB, 0.20764E+00_JPRB, 0.21124E+00_JPRB, 0.21288E+00_JPRB, 0.21459E+00_JPRB, 0.21377E+00_JPRB ,&
&  0.21169E+00_JPRB, 0.20914E+00_JPRB, 0.20675E+00_JPRB, 0.20484E+00_JPRB, 0.20429E+00_JPRB, 0.20286E+00_JPRB, 0.20129E+00_JPRB ,&
&  0.20124E+00_JPRB, 0.20259E+00_JPRB, 0.20340E+00_JPRB, 0.20424E+00_JPRB, 0.21023E+00_JPRB, 0.21099E+00_JPRB, 0.21295E+00_JPRB ,&
&  0.21343E+00_JPRB, 0.21388E+00_JPRB, 0.21314E+00_JPRB, 0.21176E+00_JPRB, 0.21049E+00_JPRB, 0.20563E+00_JPRB, 0.20338E+00_JPRB ,&
&  0.19605E+00_JPRB, 0.19170E+00_JPRB, 0.18510E+00_JPRB, 0.17928E+00_JPRB, 0.17312E+00_JPRB, 0.16852E+00_JPRB, 0.16470E+00_JPRB ,&
&  0.16168E+00_JPRB, 0.15943E+00_JPRB, 0.15755E+00_JPRB, 0.15729E+00_JPRB, 0.15739E+00_JPRB, 0.15709E+00_JPRB, 0.15724E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.15875E+00_JPRB, 0.15869E+00_JPRB, 0.15924E+00_JPRB, 0.15977E+00_JPRB, 0.16053E+00_JPRB, 0.16193E+00_JPRB ,&
&  0.16410E+00_JPRB, 0.16604E+00_JPRB, 0.16687E+00_JPRB, 0.16850E+00_JPRB, 0.16781E+00_JPRB, 0.16524E+00_JPRB, 0.16268E+00_JPRB ,&
&  0.15994E+00_JPRB, 0.15873E+00_JPRB, 0.15744E+00_JPRB, 0.15671E+00_JPRB, 0.15662E+00_JPRB, 0.15682E+00_JPRB, 0.15757E+00_JPRB ,&
&  0.15838E+00_JPRB, 0.15923E+00_JPRB, 0.15959E+00_JPRB, 0.16008E+00_JPRB, 0.16052E+00_JPRB, 0.16131E+00_JPRB, 0.16277E+00_JPRB ,&
&  0.16474E+00_JPRB, 0.16702E+00_JPRB, 0.16923E+00_JPRB, 0.17195E+00_JPRB, 0.17647E+00_JPRB, 0.18261E+00_JPRB, 0.19194E+00_JPRB ,&
&  0.19964E+00_JPRB, 0.21274E+00_JPRB, 0.21364E+00_JPRB, 0.21483E+00_JPRB, 0.21609E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.22463E+00_JPRB, 0.22500E+00_JPRB, 0.22150E+00_JPRB, 0.21835E+00_JPRB, 0.21686E+00_JPRB, 0.21638E+00_JPRB, 0.21748E+00_JPRB ,&
&  0.22017E+00_JPRB, 0.22221E+00_JPRB, 0.22427E+00_JPRB, 0.22607E+00_JPRB, 0.22684E+00_JPRB, 0.22675E+00_JPRB, 0.22766E+00_JPRB ,&
&  0.22853E+00_JPRB, 0.23082E+00_JPRB, 0.23196E+00_JPRB, 0.23422E+00_JPRB, 0.23442E+00_JPRB, 0.23513E+00_JPRB, 0.23632E+00_JPRB ,&
&  0.23592E+00_JPRB, 0.23660E+00_JPRB, 0.23812E+00_JPRB, 0.23610E+00_JPRB, 0.23473E+00_JPRB, 0.23262E+00_JPRB, 0.23087E+00_JPRB ,&
&  0.22496E+00_JPRB, 0.21867E+00_JPRB, 0.21194E+00_JPRB, 0.21500E+00_JPRB, 0.22301E+00_JPRB, 0.22510E+00_JPRB, 0.22880E+00_JPRB ,&
&  0.22978E+00_JPRB, 0.22818E+00_JPRB, 0.22644E+00_JPRB, 0.22336E+00_JPRB, 0.22034E+00_JPRB, 0.21698E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.21149E+00_JPRB, 0.21198E+00_JPRB, 0.20844E+00_JPRB, 0.19565E+00_JPRB, 0.18870E+00_JPRB, 0.18443E+00_JPRB, 0.17802E+00_JPRB ,&
&  0.17172E+00_JPRB, 0.16972E+00_JPRB, 0.16923E+00_JPRB, 0.16721E+00_JPRB, 0.16724E+00_JPRB, 0.16756E+00_JPRB, 0.16751E+00_JPRB ,&
&  0.16633E+00_JPRB, 0.16609E+00_JPRB, 0.16479E+00_JPRB, 0.16292E+00_JPRB, 0.16119E+00_JPRB, 0.16051E+00_JPRB, 0.16090E+00_JPRB ,&
&  0.16233E+00_JPRB, 0.16398E+00_JPRB, 0.16931E+00_JPRB, 0.17284E+00_JPRB, 0.16878E+00_JPRB, 0.16581E+00_JPRB, 0.16557E+00_JPRB ,&
&  0.16197E+00_JPRB, 0.16120E+00_JPRB, 0.16115E+00_JPRB, 0.16108E+00_JPRB, 0.16140E+00_JPRB, 0.16224E+00_JPRB, 0.16226E+00_JPRB ,&
&  0.16288E+00_JPRB, 0.16403E+00_JPRB, 0.16434E+00_JPRB, 0.16438E+00_JPRB, 0.16420E+00_JPRB, 0.16461E+00_JPRB, 0.16573E+00_JPRB ,&
&  0.16707E+00_JPRB, 0.16950E+00_JPRB, 0.17208E+00_JPRB, 0.17580E+00_JPRB, 0.17876E+00_JPRB, 0.18108E+00_JPRB, 0.18521E+00_JPRB ,&
&  0.18898E+00_JPRB, 0.19761E+00_JPRB, 0.21245E+00_JPRB, 0.22342E+00_JPRB, 0.22084E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.23073E+00_JPRB, 0.23066E+00_JPRB, 0.22950E+00_JPRB, 0.22840E+00_JPRB, 0.22689E+00_JPRB, 0.22537E+00_JPRB, 0.22592E+00_JPRB ,&
&  0.22814E+00_JPRB, 0.23107E+00_JPRB, 0.23392E+00_JPRB, 0.23614E+00_JPRB, 0.23790E+00_JPRB, 0.23874E+00_JPRB, 0.24010E+00_JPRB ,&
&  0.24170E+00_JPRB, 0.24438E+00_JPRB, 0.24509E+00_JPRB, 0.24571E+00_JPRB, 0.24504E+00_JPRB, 0.24442E+00_JPRB, 0.24552E+00_JPRB ,&
&  0.24870E+00_JPRB, 0.25029E+00_JPRB, 0.24957E+00_JPRB, 0.24764E+00_JPRB, 0.24818E+00_JPRB, 0.24997E+00_JPRB, 0.24926E+00_JPRB ,&
&  0.24323E+00_JPRB, 0.23029E+00_JPRB, 0.22106E+00_JPRB, 0.22343E+00_JPRB, 0.23226E+00_JPRB, 0.23633E+00_JPRB, 0.23896E+00_JPRB ,&
&  0.24063E+00_JPRB, 0.24090E+00_JPRB, 0.24018E+00_JPRB, 0.23878E+00_JPRB, 0.23635E+00_JPRB, 0.23212E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22240E+00_JPRB, 0.21995E+00_JPRB, 0.21720E+00_JPRB, 0.21242E+00_JPRB, 0.20764E+00_JPRB, 0.20460E+00_JPRB, 0.20125E+00_JPRB ,&
&  0.19787E+00_JPRB, 0.19515E+00_JPRB, 0.19445E+00_JPRB, 0.19412E+00_JPRB, 0.19415E+00_JPRB, 0.19379E+00_JPRB, 0.19192E+00_JPRB ,&
&  0.18842E+00_JPRB, 0.18430E+00_JPRB, 0.17871E+00_JPRB, 0.17370E+00_JPRB, 0.17012E+00_JPRB, 0.16670E+00_JPRB, 0.16273E+00_JPRB ,&
&  0.16071E+00_JPRB, 0.16288E+00_JPRB, 0.17078E+00_JPRB, 0.17991E+00_JPRB, 0.18624E+00_JPRB, 0.17467E+00_JPRB, 0.17015E+00_JPRB ,&
&  0.16882E+00_JPRB, 0.16836E+00_JPRB, 0.16853E+00_JPRB, 0.16973E+00_JPRB, 0.17036E+00_JPRB, 0.17022E+00_JPRB, 0.17075E+00_JPRB ,&
&  0.17250E+00_JPRB, 0.17435E+00_JPRB, 0.17536E+00_JPRB, 0.17640E+00_JPRB, 0.17701E+00_JPRB, 0.17680E+00_JPRB, 0.17648E+00_JPRB ,&
&  0.17704E+00_JPRB, 0.17968E+00_JPRB, 0.18417E+00_JPRB, 0.19060E+00_JPRB, 0.19543E+00_JPRB, 0.19988E+00_JPRB, 0.20570E+00_JPRB ,&
&  0.21306E+00_JPRB, 0.22063E+00_JPRB, 0.22448E+00_JPRB, 0.22792E+00_JPRB, 0.22968E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.23756E+00_JPRB, 0.23892E+00_JPRB, 0.24098E+00_JPRB, 0.24300E+00_JPRB, 0.24372E+00_JPRB, 0.24315E+00_JPRB, 0.24246E+00_JPRB ,&
&  0.24277E+00_JPRB, 0.24456E+00_JPRB, 0.24646E+00_JPRB, 0.24680E+00_JPRB, 0.24694E+00_JPRB, 0.24723E+00_JPRB, 0.24887E+00_JPRB ,&
&  0.25140E+00_JPRB, 0.25430E+00_JPRB, 0.25418E+00_JPRB, 0.25374E+00_JPRB, 0.25343E+00_JPRB, 0.25255E+00_JPRB, 0.25372E+00_JPRB ,&
&  0.25668E+00_JPRB, 0.25695E+00_JPRB, 0.25688E+00_JPRB, 0.25621E+00_JPRB, 0.25775E+00_JPRB, 0.26076E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26088E+00_JPRB, 0.25429E+00_JPRB, 0.24631E+00_JPRB, 0.24604E+00_JPRB, 0.24772E+00_JPRB, 0.24948E+00_JPRB, 0.24977E+00_JPRB ,&
&  0.24938E+00_JPRB, 0.24860E+00_JPRB, 0.24761E+00_JPRB, 0.24666E+00_JPRB, 0.24554E+00_JPRB, 0.24215E+00_JPRB, 0.23818E+00_JPRB ,&
&  0.23503E+00_JPRB, 0.23194E+00_JPRB, 0.22886E+00_JPRB, 0.22483E+00_JPRB, 0.22068E+00_JPRB, 0.21770E+00_JPRB, 0.21439E+00_JPRB ,&
&  0.21029E+00_JPRB, 0.20727E+00_JPRB, 0.20638E+00_JPRB, 0.20643E+00_JPRB, 0.20666E+00_JPRB, 0.20600E+00_JPRB, 0.20519E+00_JPRB ,&
&  0.20370E+00_JPRB, 0.20252E+00_JPRB, 0.20109E+00_JPRB, 0.20039E+00_JPRB, 0.19593E+00_JPRB, 0.18758E+00_JPRB, 0.18180E+00_JPRB ,&
&  0.18093E+00_JPRB, 0.18362E+00_JPRB, 0.18757E+00_JPRB, 0.18812E+00_JPRB, 0.18967E+00_JPRB, 0.19056E+00_JPRB, 0.19024E+00_JPRB ,&
&  0.18800E+00_JPRB, 0.18387E+00_JPRB, 0.18350E+00_JPRB, 0.18433E+00_JPRB, 0.18470E+00_JPRB, 0.18571E+00_JPRB, 0.18723E+00_JPRB ,&
&  0.18997E+00_JPRB, 0.19237E+00_JPRB, 0.19449E+00_JPRB, 0.19658E+00_JPRB, 0.19780E+00_JPRB, 0.19725E+00_JPRB, 0.19523E+00_JPRB ,&
&  0.19497E+00_JPRB, 0.20003E+00_JPRB, 0.20934E+00_JPRB, 0.22003E+00_JPRB, 0.22229E+00_JPRB, 0.22441E+00_JPRB, 0.22642E+00_JPRB ,&
&  0.22742E+00_JPRB, 0.23046E+00_JPRB, 0.23316E+00_JPRB, 0.23479E+00_JPRB, 0.23604E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.24855E+00_JPRB, 0.25034E+00_JPRB, 0.25118E+00_JPRB, 0.25216E+00_JPRB, 0.25419E+00_JPRB, 0.25562E+00_JPRB, 0.25585E+00_JPRB ,&
&  0.25602E+00_JPRB, 0.25712E+00_JPRB, 0.25786E+00_JPRB, 0.25815E+00_JPRB, 0.25582E+00_JPRB, 0.25549E+00_JPRB, 0.25709E+00_JPRB ,&
&  0.25923E+00_JPRB, 0.26150E+00_JPRB, 0.26166E+00_JPRB, 0.26154E+00_JPRB, 0.26111E+00_JPRB, 0.26017E+00_JPRB, 0.26175E+00_JPRB ,&
&  0.26304E+00_JPRB, 0.26363E+00_JPRB, 0.26500E+00_JPRB, 0.26632E+00_JPRB, 0.26913E+00_JPRB, 0.27239E+00_JPRB, 0.27419E+00_JPRB ,&
&  0.27409E+00_JPRB, 0.27241E+00_JPRB, 0.26623E+00_JPRB, 0.25964E+00_JPRB, 0.26021E+00_JPRB, 0.26145E+00_JPRB, 0.26147E+00_JPRB ,&
&  0.25951E+00_JPRB, 0.25682E+00_JPRB, 0.25502E+00_JPRB, 0.25362E+00_JPRB, 0.25191E+00_JPRB, 0.24897E+00_JPRB, 0.24583E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.24213E+00_JPRB, 0.24079E+00_JPRB, 0.23861E+00_JPRB, 0.23543E+00_JPRB, 0.23291E+00_JPRB, 0.23029E+00_JPRB ,&
&  0.22559E+00_JPRB, 0.22097E+00_JPRB, 0.21848E+00_JPRB, 0.21792E+00_JPRB, 0.21837E+00_JPRB, 0.21803E+00_JPRB, 0.21649E+00_JPRB ,&
&  0.21471E+00_JPRB, 0.21235E+00_JPRB, 0.21091E+00_JPRB, 0.21049E+00_JPRB, 0.21154E+00_JPRB, 0.21053E+00_JPRB, 0.20642E+00_JPRB ,&
&  0.20399E+00_JPRB, 0.20230E+00_JPRB, 0.20219E+00_JPRB, 0.20116E+00_JPRB, 0.20019E+00_JPRB, 0.19886E+00_JPRB, 0.19917E+00_JPRB ,&
&  0.20007E+00_JPRB, 0.20041E+00_JPRB, 0.20282E+00_JPRB, 0.20713E+00_JPRB, 0.21109E+00_JPRB, 0.21340E+00_JPRB, 0.21475E+00_JPRB ,&
&  0.21523E+00_JPRB, 0.21453E+00_JPRB, 0.21384E+00_JPRB, 0.21422E+00_JPRB, 0.21599E+00_JPRB, 0.21843E+00_JPRB, 0.22086E+00_JPRB ,&
&  0.22158E+00_JPRB, 0.22262E+00_JPRB, 0.22495E+00_JPRB, 0.22850E+00_JPRB, 0.23066E+00_JPRB, 0.23249E+00_JPRB, 0.23415E+00_JPRB ,&
&  0.23506E+00_JPRB, 0.23659E+00_JPRB, 0.24031E+00_JPRB, 0.24282E+00_JPRB, 0.24539E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.25880E+00_JPRB, 0.25958E+00_JPRB, 0.26039E+00_JPRB, 0.26042E+00_JPRB, 0.26137E+00_JPRB, 0.26218E+00_JPRB, 0.26360E+00_JPRB ,&
&  0.26419E+00_JPRB, 0.26557E+00_JPRB, 0.26755E+00_JPRB, 0.26746E+00_JPRB, 0.26708E+00_JPRB, 0.26686E+00_JPRB, 0.26744E+00_JPRB ,&
&  0.26966E+00_JPRB, 0.27097E+00_JPRB, 0.27053E+00_JPRB, 0.27030E+00_JPRB, 0.26970E+00_JPRB, 0.26923E+00_JPRB, 0.27132E+00_JPRB ,&
&  0.27184E+00_JPRB, 0.27260E+00_JPRB, 0.27443E+00_JPRB, 0.27593E+00_JPRB, 0.27941E+00_JPRB, 0.28358E+00_JPRB, 0.28625E+00_JPRB ,&
&  0.28587E+00_JPRB, 0.28293E+00_JPRB, 0.27818E+00_JPRB, 0.27453E+00_JPRB, 0.27396E+00_JPRB, 0.27498E+00_JPRB, 0.27728E+00_JPRB ,&
&  0.27651E+00_JPRB, 0.27110E+00_JPRB, 0.26430E+00_JPRB, 0.26067E+00_JPRB, 0.25941E+00_JPRB, 0.25779E+00_JPRB, 0.25499E+00_JPRB ,&
&  0.25273E+00_JPRB, 0.25223E+00_JPRB, 0.25236E+00_JPRB, 0.25157E+00_JPRB, 0.24997E+00_JPRB, 0.24725E+00_JPRB, 0.24375E+00_JPRB ,&
&  0.24068E+00_JPRB, 0.23761E+00_JPRB, 0.23568E+00_JPRB, 0.23429E+00_JPRB, 0.23411E+00_JPRB, 0.23434E+00_JPRB, 0.23362E+00_JPRB ,&
&  0.23193E+00_JPRB, 0.23088E+00_JPRB, 0.22940E+00_JPRB, 0.22640E+00_JPRB, 0.22421E+00_JPRB, 0.22350E+00_JPRB, 0.22237E+00_JPRB ,&
&  0.22080E+00_JPRB, 0.22016E+00_JPRB, 0.21982E+00_JPRB, 0.21921E+00_JPRB, 0.21985E+00_JPRB, 0.22136E+00_JPRB, 0.21888E+00_JPRB ,&
&  0.21678E+00_JPRB, 0.21539E+00_JPRB, 0.21543E+00_JPRB, 0.21688E+00_JPRB, 0.21930E+00_JPRB, 0.22193E+00_JPRB, 0.22414E+00_JPRB ,&
&  0.22509E+00_JPRB, 0.22537E+00_JPRB, 0.22571E+00_JPRB, 0.22575E+00_JPRB, 0.22664E+00_JPRB, 0.22863E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.23245E+00_JPRB, 0.23488E+00_JPRB, 0.23744E+00_JPRB, 0.24064E+00_JPRB, 0.24248E+00_JPRB, 0.24423E+00_JPRB, 0.24604E+00_JPRB ,&
&  0.24752E+00_JPRB, 0.24940E+00_JPRB, 0.25206E+00_JPRB, 0.25429E+00_JPRB, 0.25665E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.27221E+00_JPRB, 0.27105E+00_JPRB, 0.27035E+00_JPRB, 0.27084E+00_JPRB, 0.27301E+00_JPRB, 0.27358E+00_JPRB, 0.27350E+00_JPRB ,&
&  0.27320E+00_JPRB, 0.27495E+00_JPRB, 0.27834E+00_JPRB, 0.28039E+00_JPRB, 0.28057E+00_JPRB, 0.28032E+00_JPRB, 0.27991E+00_JPRB ,&
&  0.27979E+00_JPRB, 0.27967E+00_JPRB, 0.27866E+00_JPRB, 0.27871E+00_JPRB, 0.27888E+00_JPRB, 0.27822E+00_JPRB, 0.27919E+00_JPRB ,&
&  0.27940E+00_JPRB, 0.27987E+00_JPRB, 0.28207E+00_JPRB, 0.28380E+00_JPRB, 0.28736E+00_JPRB, 0.29279E+00_JPRB, 0.29515E+00_JPRB ,&
&  0.29429E+00_JPRB, 0.29047E+00_JPRB, 0.28737E+00_JPRB, 0.28805E+00_JPRB, 0.28970E+00_JPRB, 0.29306E+00_JPRB, 0.29665E+00_JPRB ,&
&  0.29748E+00_JPRB, 0.29203E+00_JPRB, 0.28362E+00_JPRB, 0.27779E+00_JPRB, 0.27416E+00_JPRB, 0.27198E+00_JPRB, 0.27010E+00_JPRB ,&
&  0.26758E+00_JPRB, 0.26562E+00_JPRB, 0.26463E+00_JPRB, 0.26442E+00_JPRB, 0.26401E+00_JPRB, 0.26126E+00_JPRB, 0.25823E+00_JPRB ,&
&  0.25427E+00_JPRB, 0.25148E+00_JPRB, 0.24916E+00_JPRB, 0.24833E+00_JPRB, 0.24842E+00_JPRB, 0.24822E+00_JPRB, 0.24730E+00_JPRB ,&
&  0.24612E+00_JPRB, 0.24580E+00_JPRB, 0.24551E+00_JPRB, 0.24443E+00_JPRB, 0.24308E+00_JPRB, 0.24271E+00_JPRB, 0.24164E+00_JPRB ,&
&  0.23941E+00_JPRB, 0.23801E+00_JPRB, 0.23789E+00_JPRB, 0.23868E+00_JPRB, 0.23850E+00_JPRB, 0.23914E+00_JPRB, 0.24053E+00_JPRB ,&
&  0.23956E+00_JPRB, 0.23713E+00_JPRB, 0.23595E+00_JPRB, 0.23655E+00_JPRB, 0.23833E+00_JPRB, 0.24064E+00_JPRB, 0.24181E+00_JPRB ,&
&  0.24287E+00_JPRB, 0.24288E+00_JPRB, 0.24376E+00_JPRB, 0.24502E+00_JPRB, 0.24648E+00_JPRB, 0.24722E+00_JPRB, 0.24751E+00_JPRB ,&
&  0.24793E+00_JPRB, 0.24847E+00_JPRB, 0.25047E+00_JPRB, 0.25363E+00_JPRB, 0.25650E+00_JPRB, 0.25977E+00_JPRB, 0.26240E+00_JPRB ,&
&  0.26426E+00_JPRB, 0.26630E+00_JPRB, 0.26782E+00_JPRB, 0.26898E+00_JPRB, 0.27055E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.29342E+00_JPRB, 0.29061E+00_JPRB, 0.28872E+00_JPRB, 0.28881E+00_JPRB, 0.29083E+00_JPRB, 0.29126E+00_JPRB, 0.28980E+00_JPRB ,&
&  0.28835E+00_JPRB, 0.28955E+00_JPRB, 0.29239E+00_JPRB, 0.29375E+00_JPRB, 0.29411E+00_JPRB, 0.29470E+00_JPRB, 0.29364E+00_JPRB ,&
&  0.29053E+00_JPRB, 0.28730E+00_JPRB, 0.28760E+00_JPRB, 0.28870E+00_JPRB, 0.28971E+00_JPRB, 0.28850E+00_JPRB, 0.28703E+00_JPRB ,&
&  0.28767E+00_JPRB, 0.28775E+00_JPRB, 0.28946E+00_JPRB, 0.29138E+00_JPRB, 0.29477E+00_JPRB, 0.29993E+00_JPRB, 0.30362E+00_JPRB ,&
&  0.30451E+00_JPRB, 0.30300E+00_JPRB, 0.30264E+00_JPRB, 0.30490E+00_JPRB, 0.30727E+00_JPRB, 0.31310E+00_JPRB, 0.31919E+00_JPRB ,&
&  0.32216E+00_JPRB, 0.32081E+00_JPRB, 0.31209E+00_JPRB, 0.30451E+00_JPRB, 0.29979E+00_JPRB, 0.29497E+00_JPRB, 0.29008E+00_JPRB ,&
&  0.28459E+00_JPRB, 0.28037E+00_JPRB, 0.27769E+00_JPRB, 0.27630E+00_JPRB, 0.27561E+00_JPRB, 0.27470E+00_JPRB, 0.27242E+00_JPRB ,&
&  0.26960E+00_JPRB, 0.26742E+00_JPRB, 0.26572E+00_JPRB, 0.26439E+00_JPRB, 0.26316E+00_JPRB, 0.26262E+00_JPRB, 0.26202E+00_JPRB ,&
&  0.26174E+00_JPRB, 0.26216E+00_JPRB, 0.26297E+00_JPRB, 0.26300E+00_JPRB, 0.26282E+00_JPRB, 0.26211E+00_JPRB, 0.26130E+00_JPRB ,&
&  0.25996E+00_JPRB, 0.25752E+00_JPRB, 0.25545E+00_JPRB, 0.25479E+00_JPRB, 0.25439E+00_JPRB, 0.25461E+00_JPRB, 0.25489E+00_JPRB ,&
&  0.25458E+00_JPRB, 0.25449E+00_JPRB, 0.25528E+00_JPRB, 0.25642E+00_JPRB, 0.25733E+00_JPRB, 0.25783E+00_JPRB, 0.25847E+00_JPRB ,&
&  0.26004E+00_JPRB, 0.26185E+00_JPRB, 0.26310E+00_JPRB, 0.26398E+00_JPRB, 0.26449E+00_JPRB, 0.26554E+00_JPRB, 0.26556E+00_JPRB ,&
&  0.26575E+00_JPRB, 0.26692E+00_JPRB, 0.26981E+00_JPRB, 0.27403E+00_JPRB, 0.27844E+00_JPRB, 0.28187E+00_JPRB, 0.28525E+00_JPRB ,&
&  0.28806E+00_JPRB, 0.29034E+00_JPRB, 0.29173E+00_JPRB, 0.29241E+00_JPRB, 0.29333E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.31601E+00_JPRB, 0.31329E+00_JPRB, 0.31131E+00_JPRB, 0.30954E+00_JPRB, 0.30984E+00_JPRB, 0.30893E+00_JPRB, 0.30639E+00_JPRB ,&
&  0.30456E+00_JPRB, 0.30577E+00_JPRB, 0.30810E+00_JPRB, 0.30889E+00_JPRB, 0.30798E+00_JPRB, 0.30650E+00_JPRB, 0.30373E+00_JPRB ,&
&  0.29997E+00_JPRB, 0.29808E+00_JPRB, 0.29771E+00_JPRB, 0.29811E+00_JPRB, 0.29790E+00_JPRB, 0.29637E+00_JPRB, 0.29530E+00_JPRB ,&
&  0.29640E+00_JPRB, 0.29784E+00_JPRB, 0.29814E+00_JPRB, 0.30032E+00_JPRB, 0.30394E+00_JPRB, 0.31026E+00_JPRB, 0.31618E+00_JPRB ,&
&  0.31821E+00_JPRB, 0.31951E+00_JPRB, 0.31383E+00_JPRB, 0.32427E+00_JPRB, 0.32972E+00_JPRB, 0.33745E+00_JPRB, 0.34301E+00_JPRB ,&
&  0.34698E+00_JPRB, 0.34639E+00_JPRB, 0.33800E+00_JPRB, 0.33070E+00_JPRB, 0.32589E+00_JPRB, 0.32115E+00_JPRB, 0.31351E+00_JPRB ,&
&  0.30540E+00_JPRB, 0.29813E+00_JPRB, 0.29354E+00_JPRB, 0.29115E+00_JPRB, 0.29085E+00_JPRB, 0.29098E+00_JPRB, 0.28907E+00_JPRB ,&
&  0.28705E+00_JPRB, 0.28510E+00_JPRB, 0.28424E+00_JPRB, 0.28335E+00_JPRB, 0.28209E+00_JPRB, 0.28113E+00_JPRB, 0.28088E+00_JPRB ,&
&  0.28092E+00_JPRB, 0.28075E+00_JPRB, 0.28082E+00_JPRB, 0.28137E+00_JPRB, 0.28162E+00_JPRB, 0.28087E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.27753E+00_JPRB, 0.27597E+00_JPRB, 0.27456E+00_JPRB, 0.27315E+00_JPRB, 0.27187E+00_JPRB, 0.27093E+00_JPRB, 0.27002E+00_JPRB ,&
&  0.26903E+00_JPRB, 0.26930E+00_JPRB, 0.27059E+00_JPRB, 0.27129E+00_JPRB, 0.27163E+00_JPRB, 0.27261E+00_JPRB, 0.27440E+00_JPRB ,&
&  0.27612E+00_JPRB, 0.27802E+00_JPRB, 0.27938E+00_JPRB, 0.27988E+00_JPRB, 0.27922E+00_JPRB, 0.27910E+00_JPRB, 0.27974E+00_JPRB ,&
&  0.28150E+00_JPRB, 0.28439E+00_JPRB, 0.28870E+00_JPRB, 0.29472E+00_JPRB, 0.30183E+00_JPRB, 0.30731E+00_JPRB, 0.31209E+00_JPRB ,&
&  0.31572E+00_JPRB, 0.31712E+00_JPRB, 0.31723E+00_JPRB, 0.31803E+00_JPRB, 0.31802E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.33811E+00_JPRB, 0.33574E+00_JPRB, 0.33393E+00_JPRB, 0.33067E+00_JPRB, 0.32853E+00_JPRB, 0.32710E+00_JPRB, 0.32528E+00_JPRB ,&
&  0.32393E+00_JPRB, 0.32440E+00_JPRB, 0.32340E+00_JPRB, 0.32292E+00_JPRB, 0.32212E+00_JPRB, 0.31956E+00_JPRB, 0.31627E+00_JPRB ,&
&  0.31438E+00_JPRB, 0.31258E+00_JPRB, 0.31094E+00_JPRB, 0.30917E+00_JPRB, 0.30750E+00_JPRB, 0.30528E+00_JPRB, 0.30548E+00_JPRB ,&
&  0.30681E+00_JPRB, 0.30955E+00_JPRB, 0.31251E+00_JPRB, 0.31599E+00_JPRB, 0.32093E+00_JPRB, 0.32756E+00_JPRB, 0.33392E+00_JPRB ,&
&  0.34084E+00_JPRB, 0.33033E+00_JPRB, 0.34282E+00_JPRB, 0.34900E+00_JPRB, 0.35980E+00_JPRB, 0.36941E+00_JPRB, 0.37432E+00_JPRB ,&
&  0.37469E+00_JPRB, 0.36607E+00_JPRB, 0.35564E+00_JPRB, 0.34823E+00_JPRB, 0.34541E+00_JPRB, 0.34281E+00_JPRB, 0.33671E+00_JPRB ,&
&  0.32935E+00_JPRB, 0.32126E+00_JPRB, 0.31502E+00_JPRB, 0.31135E+00_JPRB, 0.30977E+00_JPRB, 0.30920E+00_JPRB, 0.30760E+00_JPRB ,&
&  0.30602E+00_JPRB, 0.30445E+00_JPRB, 0.30365E+00_JPRB, 0.30325E+00_JPRB, 0.30281E+00_JPRB, 0.30303E+00_JPRB, 0.30352E+00_JPRB ,&
&  0.30361E+00_JPRB, 0.30339E+00_JPRB, 0.30315E+00_JPRB, 0.30348E+00_JPRB, 0.30354E+00_JPRB, 0.30299E+00_JPRB, 0.30203E+00_JPRB ,&
&  0.30050E+00_JPRB, 0.29919E+00_JPRB, 0.29754E+00_JPRB, 0.29640E+00_JPRB, 0.29479E+00_JPRB, 0.29258E+00_JPRB, 0.29045E+00_JPRB ,&
&  0.28928E+00_JPRB, 0.28904E+00_JPRB, 0.28917E+00_JPRB, 0.28949E+00_JPRB, 0.29016E+00_JPRB, 0.29132E+00_JPRB, 0.29261E+00_JPRB ,&
&  0.29389E+00_JPRB, 0.29501E+00_JPRB, 0.29538E+00_JPRB, 0.29540E+00_JPRB, 0.29482E+00_JPRB, 0.29502E+00_JPRB, 0.29646E+00_JPRB ,&
&  0.29996E+00_JPRB, 0.30458E+00_JPRB, 0.31122E+00_JPRB, 0.32220E+00_JPRB, 0.33306E+00_JPRB, 0.33989E+00_JPRB, 0.34456E+00_JPRB ,&
&  0.34523E+00_JPRB, 0.34328E+00_JPRB, 0.34106E+00_JPRB, 0.34064E+00_JPRB, 0.34040E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.35424E+00_JPRB, 0.35278E+00_JPRB, 0.35151E+00_JPRB, 0.34952E+00_JPRB, 0.34775E+00_JPRB, 0.34629E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.34238E+00_JPRB, 0.33929E+00_JPRB, 0.33632E+00_JPRB, 0.33534E+00_JPRB, 0.33396E+00_JPRB, 0.33174E+00_JPRB, 0.32984E+00_JPRB ,&
&  0.32757E+00_JPRB, 0.32510E+00_JPRB, 0.32352E+00_JPRB, 0.32135E+00_JPRB, 0.31814E+00_JPRB, 0.31578E+00_JPRB, 0.31631E+00_JPRB ,&
&  0.31728E+00_JPRB, 0.31985E+00_JPRB, 0.32720E+00_JPRB, 0.33469E+00_JPRB, 0.34231E+00_JPRB, 0.34937E+00_JPRB, 0.35656E+00_JPRB ,&
&  0.36684E+00_JPRB, 0.33052E+00_JPRB, 0.35069E+00_JPRB, 0.36354E+00_JPRB, 0.38354E+00_JPRB, 0.40528E+00_JPRB, 0.40937E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.38617E+00_JPRB, 0.37283E+00_JPRB, 0.36343E+00_JPRB, 0.36093E+00_JPRB, 0.35882E+00_JPRB, 0.35441E+00_JPRB ,&
&  0.34952E+00_JPRB, 0.34244E+00_JPRB, 0.33544E+00_JPRB, 0.33073E+00_JPRB, 0.32799E+00_JPRB, 0.32589E+00_JPRB, 0.32374E+00_JPRB ,&
&  0.32254E+00_JPRB, 0.32230E+00_JPRB, 0.32220E+00_JPRB, 0.32210E+00_JPRB, 0.32262E+00_JPRB, 0.32377E+00_JPRB, 0.32483E+00_JPRB ,&
&  0.32582E+00_JPRB, 0.32620E+00_JPRB, 0.32715E+00_JPRB, 0.32852E+00_JPRB, 0.33037E+00_JPRB, 0.33195E+00_JPRB, 0.33238E+00_JPRB ,&
&  0.33132E+00_JPRB, 0.32962E+00_JPRB, 0.32767E+00_JPRB, 0.32532E+00_JPRB, 0.32236E+00_JPRB, 0.31881E+00_JPRB, 0.31581E+00_JPRB ,&
&  0.31314E+00_JPRB, 0.31097E+00_JPRB, 0.30765E+00_JPRB, 0.30675E+00_JPRB, 0.30736E+00_JPRB, 0.30802E+00_JPRB, 0.30880E+00_JPRB ,&
&  0.30948E+00_JPRB, 0.30935E+00_JPRB, 0.30884E+00_JPRB, 0.30840E+00_JPRB, 0.30839E+00_JPRB, 0.30926E+00_JPRB, 0.31135E+00_JPRB ,&
&  0.31473E+00_JPRB, 0.32097E+00_JPRB, 0.32936E+00_JPRB, 0.34345E+00_JPRB, 0.35589E+00_JPRB, 0.36231E+00_JPRB, 0.36505E+00_JPRB ,&
&  0.36303E+00_JPRB, 0.35934E+00_JPRB, 0.35621E+00_JPRB, 0.35386E+00_JPRB, 0.35455E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.37781E+00_JPRB, 0.37501E+00_JPRB, 0.37348E+00_JPRB, 0.37094E+00_JPRB, 0.36805E+00_JPRB, 0.36501E+00_JPRB, 0.36151E+00_JPRB ,&
&  0.35752E+00_JPRB, 0.35506E+00_JPRB, 0.35274E+00_JPRB, 0.34976E+00_JPRB, 0.34673E+00_JPRB, 0.34422E+00_JPRB, 0.34160E+00_JPRB ,&
&  0.33878E+00_JPRB, 0.33573E+00_JPRB, 0.33371E+00_JPRB, 0.33236E+00_JPRB, 0.33075E+00_JPRB, 0.33021E+00_JPRB, 0.33119E+00_JPRB ,&
&  0.33132E+00_JPRB, 0.33360E+00_JPRB, 0.34290E+00_JPRB, 0.35354E+00_JPRB, 0.36548E+00_JPRB, 0.37553E+00_JPRB, 0.38929E+00_JPRB ,&
&  0.37540E+00_JPRB, 0.33998E+00_JPRB, 0.36266E+00_JPRB, 0.38835E+00_JPRB, 0.41817E+00_JPRB, 0.45018E+00_JPRB, 0.44837E+00_JPRB ,&
&  0.43901E+00_JPRB, 0.42436E+00_JPRB, 0.41032E+00_JPRB, 0.39863E+00_JPRB, 0.39267E+00_JPRB, 0.38632E+00_JPRB, 0.38030E+00_JPRB ,&
&  0.37388E+00_JPRB, 0.36595E+00_JPRB, 0.35865E+00_JPRB, 0.35245E+00_JPRB, 0.34815E+00_JPRB, 0.34473E+00_JPRB, 0.34179E+00_JPRB ,&
&  0.34054E+00_JPRB, 0.34035E+00_JPRB, 0.34034E+00_JPRB, 0.34040E+00_JPRB, 0.34071E+00_JPRB, 0.34219E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34815E+00_JPRB, 0.35327E+00_JPRB, 0.36042E+00_JPRB, 0.36778E+00_JPRB, 0.37424E+00_JPRB, 0.37784E+00_JPRB, 0.37770E+00_JPRB ,&
&  0.37597E+00_JPRB, 0.37246E+00_JPRB, 0.36791E+00_JPRB, 0.36192E+00_JPRB, 0.35454E+00_JPRB, 0.34761E+00_JPRB, 0.34188E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.33263E+00_JPRB, 0.32865E+00_JPRB, 0.32705E+00_JPRB, 0.32699E+00_JPRB, 0.32678E+00_JPRB, 0.32633E+00_JPRB ,&
&  0.32600E+00_JPRB, 0.32552E+00_JPRB, 0.32532E+00_JPRB, 0.32590E+00_JPRB, 0.32751E+00_JPRB, 0.33037E+00_JPRB, 0.33348E+00_JPRB ,&
&  0.33627E+00_JPRB, 0.34272E+00_JPRB, 0.35125E+00_JPRB, 0.36969E+00_JPRB, 0.38242E+00_JPRB, 0.38786E+00_JPRB, 0.38850E+00_JPRB ,&
&  0.38605E+00_JPRB, 0.38234E+00_JPRB, 0.38038E+00_JPRB, 0.37798E+00_JPRB, 0.37800E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.40708E+00_JPRB, 0.40590E+00_JPRB, 0.40090E+00_JPRB, 0.39182E+00_JPRB, 0.38468E+00_JPRB, 0.37939E+00_JPRB, 0.37598E+00_JPRB ,&
&  0.37409E+00_JPRB, 0.37240E+00_JPRB, 0.36998E+00_JPRB, 0.36548E+00_JPRB, 0.36160E+00_JPRB, 0.35675E+00_JPRB, 0.35170E+00_JPRB ,&
&  0.34754E+00_JPRB, 0.34369E+00_JPRB, 0.34158E+00_JPRB, 0.33871E+00_JPRB, 0.33739E+00_JPRB, 0.33858E+00_JPRB, 0.34044E+00_JPRB ,&
&  0.34303E+00_JPRB, 0.34559E+00_JPRB, 0.35396E+00_JPRB, 0.36382E+00_JPRB, 0.38112E+00_JPRB, 0.39288E+00_JPRB, 0.40902E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.37755E+00_JPRB, 0.40621E+00_JPRB, 0.43832E+00_JPRB, 0.47294E+00_JPRB, 0.50596E+00_JPRB, 0.49653E+00_JPRB ,&
&  0.48490E+00_JPRB, 0.47294E+00_JPRB, 0.46128E+00_JPRB, 0.44940E+00_JPRB, 0.43982E+00_JPRB, 0.42516E+00_JPRB, 0.40978E+00_JPRB ,&
&  0.39419E+00_JPRB, 0.38341E+00_JPRB, 0.37540E+00_JPRB, 0.36801E+00_JPRB, 0.36165E+00_JPRB, 0.35767E+00_JPRB, 0.35580E+00_JPRB ,&
&  0.35566E+00_JPRB, 0.35579E+00_JPRB, 0.35545E+00_JPRB, 0.35551E+00_JPRB, 0.35607E+00_JPRB, 0.35859E+00_JPRB, 0.36409E+00_JPRB ,&
&  0.37596E+00_JPRB, 0.39238E+00_JPRB, 0.40812E+00_JPRB, 0.42060E+00_JPRB, 0.42832E+00_JPRB, 0.42851E+00_JPRB, 0.42468E+00_JPRB ,&
&  0.41679E+00_JPRB, 0.40626E+00_JPRB, 0.39623E+00_JPRB, 0.38704E+00_JPRB, 0.37886E+00_JPRB, 0.37174E+00_JPRB, 0.36461E+00_JPRB ,&
&  0.35910E+00_JPRB, 0.35495E+00_JPRB, 0.35272E+00_JPRB, 0.35116E+00_JPRB, 0.34949E+00_JPRB, 0.34807E+00_JPRB, 0.34786E+00_JPRB ,&
&  0.34762E+00_JPRB, 0.34750E+00_JPRB, 0.34791E+00_JPRB, 0.35008E+00_JPRB, 0.35349E+00_JPRB, 0.35631E+00_JPRB, 0.35837E+00_JPRB ,&
&  0.35996E+00_JPRB, 0.36598E+00_JPRB, 0.37231E+00_JPRB, 0.39755E+00_JPRB, 0.40935E+00_JPRB, 0.41291E+00_JPRB, 0.41377E+00_JPRB ,&
&  0.41393E+00_JPRB, 0.40959E+00_JPRB, 0.36434E+00_JPRB, 0.39058E+00_JPRB, 0.40195E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.44420E+00_JPRB, 0.43785E+00_JPRB, 0.42648E+00_JPRB, 0.41473E+00_JPRB, 0.40771E+00_JPRB, 0.40360E+00_JPRB, 0.40091E+00_JPRB ,&
&  0.39751E+00_JPRB, 0.39309E+00_JPRB, 0.38838E+00_JPRB, 0.38547E+00_JPRB, 0.38173E+00_JPRB, 0.37652E+00_JPRB, 0.37019E+00_JPRB ,&
&  0.36434E+00_JPRB, 0.35888E+00_JPRB, 0.35476E+00_JPRB, 0.35007E+00_JPRB, 0.34634E+00_JPRB, 0.34647E+00_JPRB, 0.34806E+00_JPRB ,&
&  0.35344E+00_JPRB, 0.36087E+00_JPRB, 0.36777E+00_JPRB, 0.38151E+00_JPRB, 0.39685E+00_JPRB, 0.41782E+00_JPRB, 0.43780E+00_JPRB ,&
&  0.47446E+00_JPRB, 0.41015E+00_JPRB, 0.43242E+00_JPRB, 0.47648E+00_JPRB, 0.51252E+00_JPRB, 0.54421E+00_JPRB, 0.54614E+00_JPRB ,&
&  0.54096E+00_JPRB, 0.53380E+00_JPRB, 0.52502E+00_JPRB, 0.51379E+00_JPRB, 0.49373E+00_JPRB, 0.46070E+00_JPRB, 0.42953E+00_JPRB ,&
&  0.40791E+00_JPRB, 0.40003E+00_JPRB, 0.39622E+00_JPRB, 0.39169E+00_JPRB, 0.38743E+00_JPRB, 0.38579E+00_JPRB, 0.38666E+00_JPRB ,&
&  0.38712E+00_JPRB, 0.38697E+00_JPRB, 0.38588E+00_JPRB, 0.38419E+00_JPRB, 0.38383E+00_JPRB, 0.39131E+00_JPRB, 0.40980E+00_JPRB ,&
&  0.43964E+00_JPRB, 0.46924E+00_JPRB, 0.49139E+00_JPRB, 0.50524E+00_JPRB, 0.50507E+00_JPRB, 0.49397E+00_JPRB, 0.47434E+00_JPRB ,&
&  0.45220E+00_JPRB, 0.43348E+00_JPRB, 0.42025E+00_JPRB, 0.40979E+00_JPRB, 0.40145E+00_JPRB, 0.39495E+00_JPRB, 0.39095E+00_JPRB ,&
&  0.38922E+00_JPRB, 0.38825E+00_JPRB, 0.38711E+00_JPRB, 0.38478E+00_JPRB, 0.38316E+00_JPRB, 0.38188E+00_JPRB, 0.38142E+00_JPRB ,&
&  0.38091E+00_JPRB, 0.38033E+00_JPRB, 0.38147E+00_JPRB, 0.38407E+00_JPRB, 0.38566E+00_JPRB, 0.38571E+00_JPRB, 0.38603E+00_JPRB ,&
&  0.38890E+00_JPRB, 0.39784E+00_JPRB, 0.40938E+00_JPRB, 0.46183E+00_JPRB, 0.47671E+00_JPRB, 0.46658E+00_JPRB, 0.45588E+00_JPRB ,&
&  0.45050E+00_JPRB, 0.44330E+00_JPRB, 0.44247E+00_JPRB, 0.41520E+00_JPRB, 0.44443E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.46995E+00_JPRB, 0.46676E+00_JPRB, 0.45514E+00_JPRB, 0.44207E+00_JPRB, 0.43041E+00_JPRB, 0.42323E+00_JPRB, 0.41752E+00_JPRB ,&
&  0.41391E+00_JPRB, 0.41135E+00_JPRB, 0.40933E+00_JPRB, 0.40670E+00_JPRB, 0.40272E+00_JPRB, 0.39690E+00_JPRB, 0.38941E+00_JPRB ,&
&  0.38258E+00_JPRB, 0.37583E+00_JPRB, 0.37028E+00_JPRB, 0.36404E+00_JPRB, 0.35634E+00_JPRB, 0.35260E+00_JPRB, 0.35514E+00_JPRB ,&
&  0.36079E+00_JPRB, 0.36781E+00_JPRB, 0.37440E+00_JPRB, 0.38457E+00_JPRB, 0.40645E+00_JPRB, 0.42785E+00_JPRB, 0.44762E+00_JPRB ,&
&  0.48798E+00_JPRB, 0.45676E+00_JPRB, 0.48085E+00_JPRB, 0.51367E+00_JPRB, 0.55376E+00_JPRB, 0.59109E+00_JPRB, 0.61070E+00_JPRB ,&
&  0.62039E+00_JPRB, 0.62057E+00_JPRB, 0.60584E+00_JPRB, 0.57468E+00_JPRB, 0.53017E+00_JPRB, 0.49256E+00_JPRB, 0.47036E+00_JPRB ,&
&  0.45824E+00_JPRB, 0.45213E+00_JPRB, 0.44663E+00_JPRB, 0.44220E+00_JPRB, 0.43962E+00_JPRB, 0.43744E+00_JPRB, 0.43515E+00_JPRB ,&
&  0.43285E+00_JPRB, 0.43100E+00_JPRB, 0.42964E+00_JPRB, 0.42939E+00_JPRB, 0.43514E+00_JPRB, 0.45341E+00_JPRB, 0.48086E+00_JPRB ,&
&  0.52168E+00_JPRB, 0.55249E+00_JPRB, 0.56192E+00_JPRB, 0.56853E+00_JPRB, 0.57268E+00_JPRB, 0.55888E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.49107E+00_JPRB, 0.46422E+00_JPRB, 0.44530E+00_JPRB, 0.43248E+00_JPRB, 0.42452E+00_JPRB, 0.42057E+00_JPRB, 0.42015E+00_JPRB ,&
&  0.42114E+00_JPRB, 0.42202E+00_JPRB, 0.42388E+00_JPRB, 0.42493E+00_JPRB, 0.42570E+00_JPRB, 0.42466E+00_JPRB, 0.42062E+00_JPRB ,&
&  0.41505E+00_JPRB, 0.41121E+00_JPRB, 0.40988E+00_JPRB, 0.40920E+00_JPRB, 0.40841E+00_JPRB, 0.40676E+00_JPRB, 0.40611E+00_JPRB ,&
&  0.40786E+00_JPRB, 0.42616E+00_JPRB, 0.45393E+00_JPRB, 0.51937E+00_JPRB, 0.51532E+00_JPRB, 0.50363E+00_JPRB, 0.48640E+00_JPRB ,&
&  0.47527E+00_JPRB, 0.47084E+00_JPRB, 0.46079E+00_JPRB, 0.46181E+00_JPRB, 0.47132E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.53015E+00_JPRB, 0.51340E+00_JPRB, 0.49553E+00_JPRB, 0.47519E+00_JPRB, 0.45758E+00_JPRB, 0.44508E+00_JPRB, 0.43604E+00_JPRB ,&
&  0.43044E+00_JPRB, 0.42722E+00_JPRB, 0.42478E+00_JPRB, 0.42270E+00_JPRB, 0.41993E+00_JPRB, 0.41694E+00_JPRB, 0.41031E+00_JPRB ,&
&  0.40225E+00_JPRB, 0.39290E+00_JPRB, 0.38569E+00_JPRB, 0.37934E+00_JPRB, 0.36796E+00_JPRB, 0.36155E+00_JPRB, 0.36338E+00_JPRB ,&
&  0.36827E+00_JPRB, 0.37441E+00_JPRB, 0.37935E+00_JPRB, 0.39006E+00_JPRB, 0.40968E+00_JPRB, 0.43031E+00_JPRB, 0.45556E+00_JPRB ,&
&  0.50266E+00_JPRB, 0.50155E+00_JPRB, 0.55553E+00_JPRB, 0.58565E+00_JPRB, 0.62478E+00_JPRB, 0.67752E+00_JPRB, 0.70520E+00_JPRB ,&
&  0.71399E+00_JPRB, 0.74484E+00_JPRB, 0.72625E+00_JPRB, 0.66319E+00_JPRB, 0.60464E+00_JPRB, 0.56329E+00_JPRB, 0.53977E+00_JPRB ,&
&  0.52201E+00_JPRB, 0.50701E+00_JPRB, 0.49375E+00_JPRB, 0.48537E+00_JPRB, 0.47704E+00_JPRB, 0.46870E+00_JPRB, 0.46075E+00_JPRB ,&
&  0.45652E+00_JPRB, 0.45383E+00_JPRB, 0.45313E+00_JPRB, 0.46171E+00_JPRB, 0.48682E+00_JPRB, 0.52832E+00_JPRB, 0.57621E+00_JPRB ,&
&  0.61329E+00_JPRB, 0.63048E+00_JPRB, 0.64480E+00_JPRB, 0.63799E+00_JPRB, 0.61551E+00_JPRB, 0.59353E+00_JPRB, 0.57065E+00_JPRB ,&
&  0.54265E+00_JPRB, 0.51127E+00_JPRB, 0.48864E+00_JPRB, 0.47328E+00_JPRB, 0.46424E+00_JPRB, 0.46005E+00_JPRB, 0.45874E+00_JPRB ,&
&  0.45751E+00_JPRB, 0.45897E+00_JPRB, 0.46585E+00_JPRB, 0.47349E+00_JPRB, 0.47415E+00_JPRB, 0.46743E+00_JPRB, 0.45627E+00_JPRB ,&
&  0.44628E+00_JPRB, 0.43803E+00_JPRB, 0.43722E+00_JPRB, 0.43754E+00_JPRB, 0.43862E+00_JPRB, 0.44113E+00_JPRB, 0.44229E+00_JPRB ,&
&  0.44610E+00_JPRB, 0.48041E+00_JPRB, 0.51008E+00_JPRB, 0.59242E+00_JPRB, 0.63519E+00_JPRB, 0.60165E+00_JPRB, 0.56038E+00_JPRB ,&
&  0.52846E+00_JPRB, 0.50906E+00_JPRB, 0.49009E+00_JPRB, 0.48862E+00_JPRB, 0.54876E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.50941E+00_JPRB, 0.49905E+00_JPRB, 0.48830E+00_JPRB, 0.47904E+00_JPRB, 0.46674E+00_JPRB, 0.45511E+00_JPRB, 0.44383E+00_JPRB ,&
&  0.43567E+00_JPRB, 0.43160E+00_JPRB, 0.43183E+00_JPRB, 0.43246E+00_JPRB, 0.43034E+00_JPRB, 0.42345E+00_JPRB, 0.41094E+00_JPRB ,&
&  0.40260E+00_JPRB, 0.39605E+00_JPRB, 0.38847E+00_JPRB, 0.37915E+00_JPRB, 0.37024E+00_JPRB, 0.36687E+00_JPRB, 0.36772E+00_JPRB ,&
&  0.37243E+00_JPRB, 0.37858E+00_JPRB, 0.38301E+00_JPRB, 0.39477E+00_JPRB, 0.41092E+00_JPRB, 0.43187E+00_JPRB, 0.46115E+00_JPRB ,&
&  0.50437E+00_JPRB, 0.59867E+00_JPRB, 0.65853E+00_JPRB, 0.68348E+00_JPRB, 0.79569E+00_JPRB, 0.80258E+00_JPRB, 0.83453E+00_JPRB ,&
&  0.86866E+00_JPRB, 0.88863E+00_JPRB, 0.85565E+00_JPRB, 0.76936E+00_JPRB, 0.69957E+00_JPRB, 0.65246E+00_JPRB, 0.60714E+00_JPRB ,&
&  0.56385E+00_JPRB, 0.53144E+00_JPRB, 0.50631E+00_JPRB, 0.48845E+00_JPRB, 0.47486E+00_JPRB, 0.46504E+00_JPRB, 0.46010E+00_JPRB ,&
&  0.46023E+00_JPRB, 0.46325E+00_JPRB, 0.47520E+00_JPRB, 0.50047E+00_JPRB, 0.58201E+00_JPRB, 0.66706E+00_JPRB, 0.71919E+00_JPRB ,&
&  0.73322E+00_JPRB, 0.74558E+00_JPRB, 0.75162E+00_JPRB, 0.72138E+00_JPRB, 0.67977E+00_JPRB, 0.63162E+00_JPRB, 0.59464E+00_JPRB ,&
&  0.57061E+00_JPRB, 0.54744E+00_JPRB, 0.52577E+00_JPRB, 0.50562E+00_JPRB, 0.49258E+00_JPRB, 0.48744E+00_JPRB, 0.48627E+00_JPRB ,&
&  0.48427E+00_JPRB, 0.48511E+00_JPRB, 0.48713E+00_JPRB, 0.48782E+00_JPRB, 0.48625E+00_JPRB, 0.48248E+00_JPRB, 0.47635E+00_JPRB ,&
&  0.46887E+00_JPRB, 0.46289E+00_JPRB, 0.47310E+00_JPRB, 0.47699E+00_JPRB, 0.47900E+00_JPRB, 0.47772E+00_JPRB, 0.47439E+00_JPRB ,&
&  0.47938E+00_JPRB, 0.52285E+00_JPRB, 0.53338E+00_JPRB, 0.57563E+00_JPRB, 0.62094E+00_JPRB, 0.62337E+00_JPRB, 0.58034E+00_JPRB ,&
&  0.54865E+00_JPRB, 0.51919E+00_JPRB, 0.49706E+00_JPRB, 0.49218E+00_JPRB, 0.51492E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.49826E+00_JPRB, 0.49188E+00_JPRB, 0.48310E+00_JPRB, 0.47452E+00_JPRB, 0.46428E+00_JPRB, 0.45310E+00_JPRB, 0.44499E+00_JPRB ,&
&  0.44050E+00_JPRB, 0.44015E+00_JPRB, 0.44116E+00_JPRB, 0.43226E+00_JPRB, 0.41979E+00_JPRB, 0.40817E+00_JPRB, 0.40102E+00_JPRB ,&
&  0.39564E+00_JPRB, 0.38984E+00_JPRB, 0.38253E+00_JPRB, 0.37346E+00_JPRB, 0.36933E+00_JPRB, 0.36801E+00_JPRB, 0.36764E+00_JPRB ,&
&  0.37297E+00_JPRB, 0.37805E+00_JPRB, 0.38472E+00_JPRB, 0.39371E+00_JPRB, 0.40807E+00_JPRB, 0.42886E+00_JPRB, 0.46188E+00_JPRB ,&
&  0.52349E+00_JPRB, 0.61362E+00_JPRB, 0.79866E+00_JPRB, 0.78744E+00_JPRB, 0.85093E+00_JPRB, 0.85962E+00_JPRB, 0.96688E+00_JPRB ,&
&  0.10405E+01_JPRB, 0.10397E+01_JPRB, 0.95578E+00_JPRB, 0.84787E+00_JPRB, 0.75335E+00_JPRB, 0.68351E+00_JPRB, 0.63436E+00_JPRB ,&
&  0.58796E+00_JPRB, 0.54710E+00_JPRB, 0.51861E+00_JPRB, 0.49870E+00_JPRB, 0.48512E+00_JPRB, 0.47669E+00_JPRB, 0.47359E+00_JPRB ,&
&  0.47776E+00_JPRB, 0.49326E+00_JPRB, 0.52636E+00_JPRB, 0.59221E+00_JPRB, 0.70845E+00_JPRB, 0.70210E+00_JPRB, 0.74785E+00_JPRB ,&
&  0.85874E+00_JPRB, 0.85714E+00_JPRB, 0.82718E+00_JPRB, 0.76846E+00_JPRB, 0.71298E+00_JPRB, 0.66423E+00_JPRB, 0.61931E+00_JPRB ,&
&  0.58657E+00_JPRB, 0.56139E+00_JPRB, 0.53936E+00_JPRB, 0.52433E+00_JPRB, 0.51650E+00_JPRB, 0.51593E+00_JPRB, 0.51636E+00_JPRB ,&
&  0.51424E+00_JPRB, 0.51086E+00_JPRB, 0.50955E+00_JPRB, 0.51033E+00_JPRB, 0.51097E+00_JPRB, 0.51173E+00_JPRB, 0.50417E+00_JPRB ,&
&  0.49810E+00_JPRB, 0.49837E+00_JPRB, 0.52309E+00_JPRB, 0.50762E+00_JPRB, 0.50458E+00_JPRB, 0.50258E+00_JPRB, 0.49565E+00_JPRB ,&
&  0.48953E+00_JPRB, 0.50581E+00_JPRB, 0.52413E+00_JPRB, 0.55824E+00_JPRB, 0.63329E+00_JPRB, 0.65592E+00_JPRB, 0.58023E+00_JPRB ,&
&  0.55546E+00_JPRB, 0.52911E+00_JPRB, 0.50735E+00_JPRB, 0.49987E+00_JPRB, 0.50734E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.48856E+00_JPRB, 0.48202E+00_JPRB, 0.47484E+00_JPRB, 0.46753E+00_JPRB, 0.45845E+00_JPRB, 0.44976E+00_JPRB, 0.44446E+00_JPRB ,&
&  0.44363E+00_JPRB, 0.44625E+00_JPRB, 0.44306E+00_JPRB, 0.43134E+00_JPRB, 0.41714E+00_JPRB, 0.40407E+00_JPRB, 0.39722E+00_JPRB ,&
&  0.39148E+00_JPRB, 0.38346E+00_JPRB, 0.37307E+00_JPRB, 0.36252E+00_JPRB, 0.35928E+00_JPRB, 0.36145E+00_JPRB, 0.35797E+00_JPRB ,&
&  0.36382E+00_JPRB, 0.37102E+00_JPRB, 0.37827E+00_JPRB, 0.38507E+00_JPRB, 0.39967E+00_JPRB, 0.42196E+00_JPRB, 0.45643E+00_JPRB ,&
&  0.52243E+00_JPRB, 0.45586E+00_JPRB, 0.74026E+00_JPRB, 0.89811E+00_JPRB, 0.98785E+00_JPRB, 0.10302E+01_JPRB, 0.11382E+01_JPRB ,&
&  0.12395E+01_JPRB, 0.11721E+01_JPRB, 0.10648E+01_JPRB, 0.91860E+00_JPRB, 0.78641E+00_JPRB, 0.69502E+00_JPRB, 0.63661E+00_JPRB ,&
&  0.59902E+00_JPRB, 0.57059E+00_JPRB, 0.54469E+00_JPRB, 0.52210E+00_JPRB, 0.50639E+00_JPRB, 0.49717E+00_JPRB, 0.49751E+00_JPRB ,&
&  0.52091E+00_JPRB, 0.56794E+00_JPRB, 0.63167E+00_JPRB, 0.75443E+00_JPRB, 0.80236E+00_JPRB, 0.78540E+00_JPRB, 0.77462E+00_JPRB ,&
&  0.85017E+00_JPRB, 0.94644E+00_JPRB, 0.86536E+00_JPRB, 0.77017E+00_JPRB, 0.72192E+00_JPRB, 0.67594E+00_JPRB, 0.63062E+00_JPRB ,&
&  0.59153E+00_JPRB, 0.56640E+00_JPRB, 0.55077E+00_JPRB, 0.54234E+00_JPRB, 0.53992E+00_JPRB, 0.53558E+00_JPRB, 0.52586E+00_JPRB ,&
&  0.51858E+00_JPRB, 0.51499E+00_JPRB, 0.51533E+00_JPRB, 0.51825E+00_JPRB, 0.51889E+00_JPRB, 0.52082E+00_JPRB, 0.52084E+00_JPRB ,&
&  0.52037E+00_JPRB, 0.52428E+00_JPRB, 0.52400E+00_JPRB, 0.50894E+00_JPRB, 0.50363E+00_JPRB, 0.49694E+00_JPRB, 0.49299E+00_JPRB ,&
&  0.50540E+00_JPRB, 0.51471E+00_JPRB, 0.52990E+00_JPRB, 0.57136E+00_JPRB, 0.59864E+00_JPRB, 0.62897E+00_JPRB, 0.56509E+00_JPRB ,&
&  0.55329E+00_JPRB, 0.53556E+00_JPRB, 0.52147E+00_JPRB, 0.50646E+00_JPRB, 0.50081E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.48561E+00_JPRB, 0.47820E+00_JPRB, 0.47128E+00_JPRB, 0.46271E+00_JPRB, 0.45193E+00_JPRB, 0.44541E+00_JPRB, 0.44728E+00_JPRB ,&
&  0.45248E+00_JPRB, 0.44872E+00_JPRB, 0.44228E+00_JPRB, 0.43125E+00_JPRB, 0.41464E+00_JPRB, 0.40654E+00_JPRB, 0.40127E+00_JPRB ,&
&  0.39259E+00_JPRB, 0.38607E+00_JPRB, 0.37531E+00_JPRB, 0.36973E+00_JPRB, 0.36783E+00_JPRB, 0.36395E+00_JPRB, 0.35655E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.36655E+00_JPRB, 0.37529E+00_JPRB, 0.38364E+00_JPRB, 0.39976E+00_JPRB, 0.42363E+00_JPRB, 0.45956E+00_JPRB ,&
&  0.53174E+00_JPRB, 0.76791E+00_JPRB, 0.67257E+00_JPRB, 0.11827E+01_JPRB, 0.11837E+01_JPRB, 0.12276E+01_JPRB, 0.12041E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.12146E+01_JPRB, 0.11519E+01_JPRB, 0.97723E+00_JPRB, 0.81007E+00_JPRB, 0.70808E+00_JPRB, 0.64282E+00_JPRB ,&
&  0.59783E+00_JPRB, 0.56283E+00_JPRB, 0.53684E+00_JPRB, 0.51892E+00_JPRB, 0.50491E+00_JPRB, 0.50428E+00_JPRB, 0.54016E+00_JPRB ,&
&  0.61441E+00_JPRB, 0.68141E+00_JPRB, 0.79757E+00_JPRB, 0.91062E+00_JPRB, 0.84287E+00_JPRB, 0.80298E+00_JPRB, 0.82463E+00_JPRB ,&
&  0.86693E+00_JPRB, 0.94056E+00_JPRB, 0.80268E+00_JPRB, 0.73064E+00_JPRB, 0.69534E+00_JPRB, 0.67380E+00_JPRB, 0.63555E+00_JPRB ,&
&  0.59953E+00_JPRB, 0.58107E+00_JPRB, 0.57087E+00_JPRB, 0.56489E+00_JPRB, 0.55621E+00_JPRB, 0.54317E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.51745E+00_JPRB, 0.51911E+00_JPRB, 0.52246E+00_JPRB, 0.52492E+00_JPRB, 0.52741E+00_JPRB, 0.52766E+00_JPRB, 0.52952E+00_JPRB ,&
&  0.53515E+00_JPRB, 0.54310E+00_JPRB, 0.52214E+00_JPRB, 0.50286E+00_JPRB, 0.49478E+00_JPRB, 0.49030E+00_JPRB, 0.48831E+00_JPRB ,&
&  0.50663E+00_JPRB, 0.52601E+00_JPRB, 0.54541E+00_JPRB, 0.57880E+00_JPRB, 0.62164E+00_JPRB, 0.66458E+00_JPRB, 0.57223E+00_JPRB ,&
&  0.56680E+00_JPRB, 0.54809E+00_JPRB, 0.53100E+00_JPRB, 0.50876E+00_JPRB, 0.49754E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.48459E+00_JPRB, 0.47450E+00_JPRB, 0.46798E+00_JPRB, 0.46124E+00_JPRB, 0.45477E+00_JPRB, 0.45292E+00_JPRB, 0.45585E+00_JPRB ,&
&  0.45574E+00_JPRB, 0.44893E+00_JPRB, 0.43800E+00_JPRB, 0.42692E+00_JPRB, 0.41007E+00_JPRB, 0.40061E+00_JPRB, 0.39570E+00_JPRB ,&
&  0.39207E+00_JPRB, 0.39231E+00_JPRB, 0.38811E+00_JPRB, 0.38496E+00_JPRB, 0.38214E+00_JPRB, 0.37606E+00_JPRB, 0.37415E+00_JPRB ,&
&  0.37209E+00_JPRB, 0.37531E+00_JPRB, 0.38282E+00_JPRB, 0.39667E+00_JPRB, 0.41696E+00_JPRB, 0.44706E+00_JPRB, 0.48353E+00_JPRB ,&
&  0.57715E+00_JPRB, 0.71121E+00_JPRB, 0.71986E+00_JPRB, 0.14519E+01_JPRB, 0.14241E+01_JPRB, 0.13516E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.11482E+01_JPRB, 0.12266E+01_JPRB, 0.11617E+01_JPRB, 0.10141E+01_JPRB, 0.85616E+00_JPRB, 0.73641E+00_JPRB, 0.64008E+00_JPRB ,&
&  0.57891E+00_JPRB, 0.54078E+00_JPRB, 0.52215E+00_JPRB, 0.51407E+00_JPRB, 0.51453E+00_JPRB, 0.55732E+00_JPRB, 0.64775E+00_JPRB ,&
&  0.72588E+00_JPRB, 0.81200E+00_JPRB, 0.98890E+00_JPRB, 0.93488E+00_JPRB, 0.82809E+00_JPRB, 0.79511E+00_JPRB, 0.80430E+00_JPRB ,&
&  0.83977E+00_JPRB, 0.81882E+00_JPRB, 0.73077E+00_JPRB, 0.68402E+00_JPRB, 0.64746E+00_JPRB, 0.65131E+00_JPRB, 0.63028E+00_JPRB ,&
&  0.60563E+00_JPRB, 0.59165E+00_JPRB, 0.57983E+00_JPRB, 0.56974E+00_JPRB, 0.55610E+00_JPRB, 0.54390E+00_JPRB, 0.52646E+00_JPRB ,&
&  0.52392E+00_JPRB, 0.52502E+00_JPRB, 0.52531E+00_JPRB, 0.53051E+00_JPRB, 0.52709E+00_JPRB, 0.52839E+00_JPRB, 0.53234E+00_JPRB ,&
&  0.54446E+00_JPRB, 0.54938E+00_JPRB, 0.51955E+00_JPRB, 0.50118E+00_JPRB, 0.49782E+00_JPRB, 0.49509E+00_JPRB, 0.49618E+00_JPRB ,&
&  0.51818E+00_JPRB, 0.54529E+00_JPRB, 0.56388E+00_JPRB, 0.57423E+00_JPRB, 0.61579E+00_JPRB, 0.63012E+00_JPRB, 0.58542E+00_JPRB ,&
&  0.56701E+00_JPRB, 0.55327E+00_JPRB, 0.53777E+00_JPRB, 0.51045E+00_JPRB, 0.49772E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.47936E+00_JPRB, 0.46781E+00_JPRB, 0.45862E+00_JPRB, 0.45628E+00_JPRB, 0.45230E+00_JPRB, 0.44952E+00_JPRB, 0.45555E+00_JPRB ,&
&  0.45002E+00_JPRB, 0.43881E+00_JPRB, 0.42178E+00_JPRB, 0.40385E+00_JPRB, 0.38978E+00_JPRB, 0.39154E+00_JPRB, 0.39650E+00_JPRB ,&
&  0.39704E+00_JPRB, 0.39507E+00_JPRB, 0.39363E+00_JPRB, 0.39505E+00_JPRB, 0.39351E+00_JPRB, 0.39219E+00_JPRB, 0.39069E+00_JPRB ,&
&  0.39054E+00_JPRB, 0.39526E+00_JPRB, 0.41045E+00_JPRB, 0.43579E+00_JPRB, 0.46563E+00_JPRB, 0.51084E+00_JPRB, 0.57317E+00_JPRB ,&
&  0.71573E+00_JPRB, 0.78391E+00_JPRB, 0.85168E+00_JPRB, 0.13350E+01_JPRB, 0.14100E+01_JPRB, 0.12799E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.98320E+00_JPRB, 0.11232E+01_JPRB, 0.98357E+00_JPRB, 0.84461E+00_JPRB, 0.71412E+00_JPRB, 0.62159E+00_JPRB ,&
&  0.56858E+00_JPRB, 0.53307E+00_JPRB, 0.52420E+00_JPRB, 0.53560E+00_JPRB, 0.58427E+00_JPRB, 0.68045E+00_JPRB, 0.78234E+00_JPRB ,&
&  0.85211E+00_JPRB, 0.98914E+00_JPRB, 0.11874E+01_JPRB, 0.98238E+00_JPRB, 0.77985E+00_JPRB, 0.73690E+00_JPRB, 0.71638E+00_JPRB ,&
&  0.75205E+00_JPRB, 0.72444E+00_JPRB, 0.68677E+00_JPRB, 0.65473E+00_JPRB, 0.61513E+00_JPRB, 0.64766E+00_JPRB, 0.62032E+00_JPRB ,&
&  0.60458E+00_JPRB, 0.59058E+00_JPRB, 0.57588E+00_JPRB, 0.56352E+00_JPRB, 0.54538E+00_JPRB, 0.52130E+00_JPRB, 0.50799E+00_JPRB ,&
&  0.50960E+00_JPRB, 0.51630E+00_JPRB, 0.51557E+00_JPRB, 0.51762E+00_JPRB, 0.51496E+00_JPRB, 0.52053E+00_JPRB, 0.52985E+00_JPRB ,&
&  0.54661E+00_JPRB, 0.55538E+00_JPRB, 0.53618E+00_JPRB, 0.52933E+00_JPRB, 0.52800E+00_JPRB, 0.52839E+00_JPRB, 0.52656E+00_JPRB ,&
&  0.54056E+00_JPRB, 0.57122E+00_JPRB, 0.58945E+00_JPRB, 0.60585E+00_JPRB, 0.62098E+00_JPRB, 0.61511E+00_JPRB, 0.58894E+00_JPRB ,&
&  0.58532E+00_JPRB, 0.59975E+00_JPRB, 0.59660E+00_JPRB, 0.53031E+00_JPRB, 0.50071E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.45627E+00_JPRB, 0.44736E+00_JPRB, 0.44435E+00_JPRB, 0.44472E+00_JPRB, 0.44284E+00_JPRB, 0.44503E+00_JPRB, 0.44825E+00_JPRB ,&
&  0.42896E+00_JPRB, 0.40887E+00_JPRB, 0.39478E+00_JPRB, 0.38922E+00_JPRB, 0.38837E+00_JPRB, 0.39983E+00_JPRB, 0.40401E+00_JPRB ,&
&  0.40082E+00_JPRB, 0.39952E+00_JPRB, 0.41093E+00_JPRB, 0.41320E+00_JPRB, 0.41377E+00_JPRB, 0.41173E+00_JPRB, 0.41241E+00_JPRB ,&
&  0.42191E+00_JPRB, 0.44046E+00_JPRB, 0.47615E+00_JPRB, 0.52730E+00_JPRB, 0.58113E+00_JPRB, 0.66787E+00_JPRB, 0.76858E+00_JPRB ,&
&  0.95123E+00_JPRB, 0.95493E+00_JPRB, 0.52266E+00_JPRB, 0.84999E+00_JPRB, 0.11057E+01_JPRB, 0.11059E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.92619E+00_JPRB, 0.86144E+00_JPRB, 0.93346E+00_JPRB, 0.91863E+00_JPRB, 0.77431E+00_JPRB, 0.68024E+00_JPRB, 0.60854E+00_JPRB ,&
&  0.56685E+00_JPRB, 0.55395E+00_JPRB, 0.56761E+00_JPRB, 0.61891E+00_JPRB, 0.72282E+00_JPRB, 0.84920E+00_JPRB, 0.93377E+00_JPRB ,&
&  0.10265E+01_JPRB, 0.12375E+01_JPRB, 0.13593E+01_JPRB, 0.97230E+00_JPRB, 0.74511E+00_JPRB, 0.68372E+00_JPRB, 0.64437E+00_JPRB ,&
&  0.62878E+00_JPRB, 0.66843E+00_JPRB, 0.66884E+00_JPRB, 0.64121E+00_JPRB, 0.61008E+00_JPRB, 0.59522E+00_JPRB, 0.60936E+00_JPRB ,&
&  0.59326E+00_JPRB, 0.57871E+00_JPRB, 0.56510E+00_JPRB, 0.54294E+00_JPRB, 0.50925E+00_JPRB, 0.47998E+00_JPRB, 0.48363E+00_JPRB ,&
&  0.48670E+00_JPRB, 0.49654E+00_JPRB, 0.49544E+00_JPRB, 0.50484E+00_JPRB, 0.51348E+00_JPRB, 0.52496E+00_JPRB, 0.53656E+00_JPRB ,&
&  0.55244E+00_JPRB, 0.56778E+00_JPRB, 0.58439E+00_JPRB, 0.57916E+00_JPRB, 0.57445E+00_JPRB, 0.57683E+00_JPRB, 0.58483E+00_JPRB ,&
&  0.60289E+00_JPRB, 0.62073E+00_JPRB, 0.60129E+00_JPRB, 0.61120E+00_JPRB, 0.63703E+00_JPRB, 0.63346E+00_JPRB, 0.62389E+00_JPRB ,&
&  0.67253E+00_JPRB, 0.71469E+00_JPRB, 0.65496E+00_JPRB, 0.50565E+00_JPRB, 0.47473E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.44258E+00_JPRB, 0.43965E+00_JPRB, 0.43811E+00_JPRB, 0.44069E+00_JPRB, 0.43669E+00_JPRB, 0.43940E+00_JPRB, 0.42753E+00_JPRB ,&
&  0.40523E+00_JPRB, 0.39723E+00_JPRB, 0.41140E+00_JPRB, 0.41683E+00_JPRB, 0.41976E+00_JPRB, 0.43411E+00_JPRB, 0.44974E+00_JPRB ,&
&  0.45424E+00_JPRB, 0.45316E+00_JPRB, 0.45050E+00_JPRB, 0.44993E+00_JPRB, 0.44594E+00_JPRB, 0.44847E+00_JPRB, 0.46464E+00_JPRB ,&
&  0.48983E+00_JPRB, 0.53563E+00_JPRB, 0.60377E+00_JPRB, 0.68938E+00_JPRB, 0.78463E+00_JPRB, 0.91301E+00_JPRB, 0.10258E+01_JPRB ,&
&  0.12477E+01_JPRB, 0.89361E+00_JPRB, 0.58175E+00_JPRB, 0.95409E+00_JPRB, 0.10568E+01_JPRB, 0.10330E+01_JPRB, 0.89226E+00_JPRB ,&
&  0.84055E+00_JPRB, 0.81434E+00_JPRB, 0.81071E+00_JPRB, 0.77844E+00_JPRB, 0.69433E+00_JPRB, 0.65528E+00_JPRB, 0.61895E+00_JPRB ,&
&  0.60139E+00_JPRB, 0.61579E+00_JPRB, 0.66673E+00_JPRB, 0.77540E+00_JPRB, 0.92660E+00_JPRB, 0.10405E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.15276E+01_JPRB, 0.15738E+01_JPRB, 0.10910E+01_JPRB, 0.78211E+00_JPRB, 0.68126E+00_JPRB, 0.64003E+00_JPRB ,&
&  0.61592E+00_JPRB, 0.64764E+00_JPRB, 0.66272E+00_JPRB, 0.63840E+00_JPRB, 0.60969E+00_JPRB, 0.58194E+00_JPRB, 0.59718E+00_JPRB ,&
&  0.57871E+00_JPRB, 0.55928E+00_JPRB, 0.53657E+00_JPRB, 0.50412E+00_JPRB, 0.47119E+00_JPRB, 0.47034E+00_JPRB, 0.46458E+00_JPRB ,&
&  0.46806E+00_JPRB, 0.48117E+00_JPRB, 0.48159E+00_JPRB, 0.49623E+00_JPRB, 0.51332E+00_JPRB, 0.53622E+00_JPRB, 0.55832E+00_JPRB ,&
&  0.57240E+00_JPRB, 0.59263E+00_JPRB, 0.61438E+00_JPRB, 0.61865E+00_JPRB, 0.61117E+00_JPRB, 0.59824E+00_JPRB, 0.61535E+00_JPRB ,&
&  0.62310E+00_JPRB, 0.63473E+00_JPRB, 0.62182E+00_JPRB, 0.63514E+00_JPRB, 0.61080E+00_JPRB, 0.59510E+00_JPRB, 0.55845E+00_JPRB ,&
&  0.55645E+00_JPRB, 0.52456E+00_JPRB, 0.48990E+00_JPRB, 0.44298E+00_JPRB, 0.43995E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.44447E+00_JPRB, 0.43893E+00_JPRB, 0.43198E+00_JPRB, 0.42332E+00_JPRB, 0.41391E+00_JPRB, 0.41389E+00_JPRB, 0.40359E+00_JPRB ,&
&  0.40074E+00_JPRB, 0.42018E+00_JPRB, 0.44841E+00_JPRB, 0.47157E+00_JPRB, 0.49131E+00_JPRB, 0.49899E+00_JPRB, 0.51382E+00_JPRB ,&
&  0.51355E+00_JPRB, 0.51106E+00_JPRB, 0.50929E+00_JPRB, 0.50868E+00_JPRB, 0.51511E+00_JPRB, 0.53758E+00_JPRB, 0.57768E+00_JPRB ,&
&  0.63511E+00_JPRB, 0.71126E+00_JPRB, 0.80502E+00_JPRB, 0.90578E+00_JPRB, 0.10012E+01_JPRB, 0.10917E+01_JPRB, 0.11280E+01_JPRB ,&
&  0.78792E+00_JPRB, 0.47232E+00_JPRB, 0.93516E+00_JPRB, 0.10394E+01_JPRB, 0.96529E+00_JPRB, 0.92055E+00_JPRB, 0.84682E+00_JPRB ,&
&  0.77741E+00_JPRB, 0.76429E+00_JPRB, 0.73637E+00_JPRB, 0.72352E+00_JPRB, 0.66484E+00_JPRB, 0.65234E+00_JPRB, 0.65095E+00_JPRB ,&
&  0.66565E+00_JPRB, 0.71023E+00_JPRB, 0.81117E+00_JPRB, 0.98204E+00_JPRB, 0.11198E+01_JPRB, 0.11998E+01_JPRB, 0.13440E+01_JPRB ,&
&  0.16004E+01_JPRB, 0.18301E+01_JPRB, 0.19234E+01_JPRB, 0.13433E+01_JPRB, 0.95725E+00_JPRB, 0.86054E+00_JPRB, 0.78797E+00_JPRB ,&
&  0.73633E+00_JPRB, 0.62988E+00_JPRB, 0.63243E+00_JPRB, 0.63984E+00_JPRB, 0.61436E+00_JPRB, 0.56901E+00_JPRB, 0.57399E+00_JPRB ,&
&  0.54894E+00_JPRB, 0.51212E+00_JPRB, 0.47653E+00_JPRB, 0.45961E+00_JPRB, 0.45370E+00_JPRB, 0.46375E+00_JPRB, 0.46252E+00_JPRB ,&
&  0.46465E+00_JPRB, 0.47768E+00_JPRB, 0.48212E+00_JPRB, 0.49302E+00_JPRB, 0.52613E+00_JPRB, 0.56096E+00_JPRB, 0.58359E+00_JPRB ,&
&  0.59319E+00_JPRB, 0.61599E+00_JPRB, 0.63376E+00_JPRB, 0.64162E+00_JPRB, 0.64276E+00_JPRB, 0.62343E+00_JPRB, 0.62075E+00_JPRB ,&
&  0.62340E+00_JPRB, 0.62195E+00_JPRB, 0.60891E+00_JPRB, 0.60623E+00_JPRB, 0.57723E+00_JPRB, 0.56383E+00_JPRB, 0.50889E+00_JPRB ,&
&  0.48599E+00_JPRB, 0.44889E+00_JPRB, 0.42190E+00_JPRB, 0.41318E+00_JPRB, 0.42092E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.44272E+00_JPRB, 0.42594E+00_JPRB, 0.41595E+00_JPRB, 0.40779E+00_JPRB, 0.39713E+00_JPRB, 0.41794E+00_JPRB, 0.42250E+00_JPRB ,&
&  0.44359E+00_JPRB, 0.47887E+00_JPRB, 0.51959E+00_JPRB, 0.53950E+00_JPRB, 0.54164E+00_JPRB, 0.56058E+00_JPRB, 0.57666E+00_JPRB ,&
&  0.58606E+00_JPRB, 0.59296E+00_JPRB, 0.60397E+00_JPRB, 0.61894E+00_JPRB, 0.64643E+00_JPRB, 0.68891E+00_JPRB, 0.75156E+00_JPRB ,&
&  0.82376E+00_JPRB, 0.90145E+00_JPRB, 0.98494E+00_JPRB, 0.10611E+01_JPRB, 0.11090E+01_JPRB, 0.11490E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.68726E+00_JPRB, 0.89040E+00_JPRB, 0.10169E+01_JPRB, 0.95670E+00_JPRB, 0.89954E+00_JPRB, 0.85488E+00_JPRB, 0.81751E+00_JPRB ,&
&  0.77958E+00_JPRB, 0.73361E+00_JPRB, 0.71546E+00_JPRB, 0.73657E+00_JPRB, 0.68742E+00_JPRB, 0.68238E+00_JPRB, 0.70207E+00_JPRB ,&
&  0.74171E+00_JPRB, 0.82323E+00_JPRB, 0.97619E+00_JPRB, 0.11550E+01_JPRB, 0.12728E+01_JPRB, 0.13935E+01_JPRB, 0.15971E+01_JPRB ,&
&  0.18965E+01_JPRB, 0.21339E+01_JPRB, 0.23051E+01_JPRB, 0.17632E+01_JPRB, 0.15285E+01_JPRB, 0.14950E+01_JPRB, 0.13554E+01_JPRB ,&
&  0.86693E+00_JPRB, 0.64353E+00_JPRB, 0.65049E+00_JPRB, 0.65084E+00_JPRB, 0.59495E+00_JPRB, 0.56218E+00_JPRB, 0.52506E+00_JPRB ,&
&  0.48186E+00_JPRB, 0.44364E+00_JPRB, 0.41961E+00_JPRB, 0.44401E+00_JPRB, 0.46870E+00_JPRB, 0.48260E+00_JPRB, 0.47150E+00_JPRB ,&
&  0.49382E+00_JPRB, 0.48334E+00_JPRB, 0.48285E+00_JPRB, 0.49643E+00_JPRB, 0.55034E+00_JPRB, 0.57438E+00_JPRB, 0.58303E+00_JPRB ,&
&  0.59051E+00_JPRB, 0.60415E+00_JPRB, 0.61442E+00_JPRB, 0.62280E+00_JPRB, 0.62656E+00_JPRB, 0.62470E+00_JPRB, 0.61657E+00_JPRB ,&
&  0.60504E+00_JPRB, 0.60598E+00_JPRB, 0.59586E+00_JPRB, 0.58547E+00_JPRB, 0.56769E+00_JPRB, 0.58849E+00_JPRB, 0.52315E+00_JPRB ,&
&  0.47788E+00_JPRB, 0.44453E+00_JPRB, 0.42033E+00_JPRB, 0.41440E+00_JPRB, 0.42146E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.44445E+00_JPRB, 0.43282E+00_JPRB, 0.41573E+00_JPRB, 0.42354E+00_JPRB, 0.42524E+00_JPRB, 0.44115E+00_JPRB, 0.45951E+00_JPRB ,&
&  0.48366E+00_JPRB, 0.52838E+00_JPRB, 0.56901E+00_JPRB, 0.60095E+00_JPRB, 0.61753E+00_JPRB, 0.64071E+00_JPRB, 0.66316E+00_JPRB ,&
&  0.67626E+00_JPRB, 0.69115E+00_JPRB, 0.71181E+00_JPRB, 0.74414E+00_JPRB, 0.78684E+00_JPRB, 0.83532E+00_JPRB, 0.89503E+00_JPRB ,&
&  0.96088E+00_JPRB, 0.10333E+01_JPRB, 0.11247E+01_JPRB, 0.11526E+01_JPRB, 0.11768E+01_JPRB, 0.12092E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.82650E+00_JPRB, 0.96530E+00_JPRB, 0.93897E+00_JPRB, 0.89867E+00_JPRB, 0.86824E+00_JPRB, 0.82499E+00_JPRB, 0.78803E+00_JPRB ,&
&  0.76152E+00_JPRB, 0.71292E+00_JPRB, 0.69283E+00_JPRB, 0.67974E+00_JPRB, 0.71969E+00_JPRB, 0.72300E+00_JPRB, 0.76645E+00_JPRB ,&
&  0.83952E+00_JPRB, 0.95982E+00_JPRB, 0.11149E+01_JPRB, 0.12756E+01_JPRB, 0.14329E+01_JPRB, 0.16001E+01_JPRB, 0.18322E+01_JPRB ,&
&  0.20834E+01_JPRB, 0.22538E+01_JPRB, 0.23497E+01_JPRB, 0.20352E+01_JPRB, 0.16459E+01_JPRB, 0.14408E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.76597E+00_JPRB, 0.65013E+00_JPRB, 0.66945E+00_JPRB, 0.66485E+00_JPRB, 0.59933E+00_JPRB, 0.54456E+00_JPRB, 0.49570E+00_JPRB ,&
&  0.45798E+00_JPRB, 0.43030E+00_JPRB, 0.44727E+00_JPRB, 0.50287E+00_JPRB, 0.53105E+00_JPRB, 0.55098E+00_JPRB, 0.54828E+00_JPRB ,&
&  0.53932E+00_JPRB, 0.51116E+00_JPRB, 0.49106E+00_JPRB, 0.50272E+00_JPRB, 0.53879E+00_JPRB, 0.57511E+00_JPRB, 0.57500E+00_JPRB ,&
&  0.56038E+00_JPRB, 0.55796E+00_JPRB, 0.57299E+00_JPRB, 0.59616E+00_JPRB, 0.61395E+00_JPRB, 0.63830E+00_JPRB, 0.65729E+00_JPRB ,&
&  0.68376E+00_JPRB, 0.72595E+00_JPRB, 0.71591E+00_JPRB, 0.71214E+00_JPRB, 0.80577E+00_JPRB, 0.85926E+00_JPRB, 0.54853E+00_JPRB ,&
&  0.46551E+00_JPRB, 0.43238E+00_JPRB, 0.42918E+00_JPRB, 0.42271E+00_JPRB, 0.45213E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.47873E+00_JPRB, 0.49768E+00_JPRB, 0.48809E+00_JPRB, 0.49083E+00_JPRB, 0.49768E+00_JPRB, 0.50825E+00_JPRB, 0.53045E+00_JPRB ,&
&  0.53573E+00_JPRB, 0.56379E+00_JPRB, 0.59212E+00_JPRB, 0.61479E+00_JPRB, 0.64731E+00_JPRB, 0.69302E+00_JPRB, 0.74893E+00_JPRB ,&
&  0.80620E+00_JPRB, 0.83906E+00_JPRB, 0.86500E+00_JPRB, 0.88313E+00_JPRB, 0.91938E+00_JPRB, 0.97008E+00_JPRB, 0.99580E+00_JPRB ,&
&  0.10596E+01_JPRB, 0.11053E+01_JPRB, 0.11597E+01_JPRB, 0.12394E+01_JPRB, 0.11835E+01_JPRB, 0.11759E+01_JPRB, 0.83947E+00_JPRB ,&
&  0.92957E+00_JPRB, 0.93242E+00_JPRB, 0.91347E+00_JPRB, 0.89241E+00_JPRB, 0.85833E+00_JPRB, 0.80330E+00_JPRB, 0.76769E+00_JPRB ,&
&  0.74097E+00_JPRB, 0.71588E+00_JPRB, 0.68909E+00_JPRB, 0.68055E+00_JPRB, 0.75255E+00_JPRB, 0.76782E+00_JPRB, 0.84958E+00_JPRB ,&
&  0.95859E+00_JPRB, 0.10844E+01_JPRB, 0.12107E+01_JPRB, 0.13626E+01_JPRB, 0.15485E+01_JPRB, 0.17474E+01_JPRB, 0.19545E+01_JPRB ,&
&  0.21154E+01_JPRB, 0.22425E+01_JPRB, 0.21981E+01_JPRB, 0.17976E+01_JPRB, 0.15996E+01_JPRB, 0.12316E+01_JPRB, 0.87836E+00_JPRB ,&
&  0.72693E+00_JPRB, 0.68444E+00_JPRB, 0.70848E+00_JPRB, 0.70614E+00_JPRB, 0.62199E+00_JPRB, 0.55235E+00_JPRB, 0.50017E+00_JPRB ,&
&  0.46884E+00_JPRB, 0.46601E+00_JPRB, 0.49731E+00_JPRB, 0.53418E+00_JPRB, 0.57667E+00_JPRB, 0.61537E+00_JPRB, 0.62098E+00_JPRB ,&
&  0.60301E+00_JPRB, 0.56236E+00_JPRB, 0.54134E+00_JPRB, 0.55585E+00_JPRB, 0.61567E+00_JPRB, 0.68306E+00_JPRB, 0.73287E+00_JPRB ,&
&  0.60102E+00_JPRB, 0.51690E+00_JPRB, 0.54972E+00_JPRB, 0.59080E+00_JPRB, 0.64427E+00_JPRB, 0.71768E+00_JPRB, 0.82991E+00_JPRB ,&
&  0.99873E+00_JPRB, 0.13534E+01_JPRB, 0.13313E+01_JPRB, 0.11080E+01_JPRB, 0.16273E+01_JPRB, 0.13476E+01_JPRB, 0.49018E+00_JPRB ,&
&  0.47033E+00_JPRB, 0.46202E+00_JPRB, 0.46528E+00_JPRB, 0.46730E+00_JPRB, 0.47694E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.56919E+00_JPRB, 0.57850E+00_JPRB, 0.59560E+00_JPRB, 0.59644E+00_JPRB, 0.60457E+00_JPRB, 0.62267E+00_JPRB, 0.64084E+00_JPRB ,&
&  0.66035E+00_JPRB, 0.66912E+00_JPRB, 0.68140E+00_JPRB, 0.69613E+00_JPRB, 0.71384E+00_JPRB, 0.75533E+00_JPRB, 0.81788E+00_JPRB ,&
&  0.86342E+00_JPRB, 0.90654E+00_JPRB, 0.96221E+00_JPRB, 0.97755E+00_JPRB, 0.10117E+01_JPRB, 0.10625E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.11376E+01_JPRB, 0.11613E+01_JPRB, 0.11824E+01_JPRB, 0.12412E+01_JPRB, 0.12040E+01_JPRB, 0.12357E+01_JPRB, 0.93297E+00_JPRB ,&
&  0.95695E+00_JPRB, 0.93603E+00_JPRB, 0.88619E+00_JPRB, 0.86301E+00_JPRB, 0.83713E+00_JPRB, 0.79872E+00_JPRB, 0.77064E+00_JPRB ,&
&  0.75943E+00_JPRB, 0.72992E+00_JPRB, 0.69966E+00_JPRB, 0.73273E+00_JPRB, 0.76821E+00_JPRB, 0.83441E+00_JPRB, 0.94132E+00_JPRB ,&
&  0.10529E+01_JPRB, 0.11613E+01_JPRB, 0.12634E+01_JPRB, 0.14065E+01_JPRB, 0.15853E+01_JPRB, 0.17794E+01_JPRB, 0.19877E+01_JPRB ,&
&  0.21273E+01_JPRB, 0.22164E+01_JPRB, 0.22201E+01_JPRB, 0.18529E+01_JPRB, 0.16980E+01_JPRB, 0.12818E+01_JPRB, 0.89553E+00_JPRB ,&
&  0.72041E+00_JPRB, 0.66960E+00_JPRB, 0.69397E+00_JPRB, 0.74971E+00_JPRB, 0.67723E+00_JPRB, 0.61757E+00_JPRB, 0.55945E+00_JPRB ,&
&  0.52902E+00_JPRB, 0.52276E+00_JPRB, 0.53286E+00_JPRB, 0.56922E+00_JPRB, 0.61659E+00_JPRB, 0.66557E+00_JPRB, 0.68794E+00_JPRB ,&
&  0.67320E+00_JPRB, 0.62358E+00_JPRB, 0.57631E+00_JPRB, 0.54245E+00_JPRB, 0.54102E+00_JPRB, 0.56438E+00_JPRB, 0.53294E+00_JPRB ,&
&  0.54621E+00_JPRB, 0.52757E+00_JPRB, 0.53837E+00_JPRB, 0.60171E+00_JPRB, 0.70772E+00_JPRB, 0.91899E+00_JPRB, 0.13193E+01_JPRB ,&
&  0.19120E+01_JPRB, 0.20680E+01_JPRB, 0.25610E+01_JPRB, 0.34306E+01_JPRB, 0.25268E+01_JPRB, 0.17712E+01_JPRB, 0.53996E+00_JPRB ,&
&  0.55346E+00_JPRB, 0.54852E+00_JPRB, 0.54699E+00_JPRB, 0.54544E+00_JPRB, 0.55196E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.60272E+00_JPRB, 0.61789E+00_JPRB, 0.63557E+00_JPRB, 0.65058E+00_JPRB, 0.67060E+00_JPRB, 0.69986E+00_JPRB, 0.72907E+00_JPRB ,&
&  0.75345E+00_JPRB, 0.77059E+00_JPRB, 0.78203E+00_JPRB, 0.80063E+00_JPRB, 0.82461E+00_JPRB, 0.85610E+00_JPRB, 0.89775E+00_JPRB ,&
&  0.91160E+00_JPRB, 0.94745E+00_JPRB, 0.96300E+00_JPRB, 0.99735E+00_JPRB, 0.10084E+01_JPRB, 0.10516E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.11016E+01_JPRB, 0.11050E+01_JPRB, 0.11222E+01_JPRB, 0.11009E+01_JPRB, 0.11272E+01_JPRB, 0.83522E+00_JPRB ,&
&  0.92669E+00_JPRB, 0.90068E+00_JPRB, 0.82101E+00_JPRB, 0.78530E+00_JPRB, 0.75344E+00_JPRB, 0.73092E+00_JPRB, 0.71880E+00_JPRB ,&
&  0.70949E+00_JPRB, 0.72977E+00_JPRB, 0.72039E+00_JPRB, 0.73886E+00_JPRB, 0.80133E+00_JPRB, 0.89845E+00_JPRB, 0.98352E+00_JPRB ,&
&  0.10911E+01_JPRB, 0.11954E+01_JPRB, 0.12931E+01_JPRB, 0.14149E+01_JPRB, 0.15682E+01_JPRB, 0.17304E+01_JPRB, 0.18871E+01_JPRB ,&
&  0.19278E+01_JPRB, 0.19358E+01_JPRB, 0.17524E+01_JPRB, 0.17221E+01_JPRB, 0.17046E+01_JPRB, 0.13745E+01_JPRB, 0.98491E+00_JPRB ,&
&  0.66481E+00_JPRB, 0.65805E+00_JPRB, 0.72571E+00_JPRB, 0.84020E+00_JPRB, 0.79920E+00_JPRB, 0.74921E+00_JPRB, 0.68579E+00_JPRB ,&
&  0.64401E+00_JPRB, 0.60977E+00_JPRB, 0.61475E+00_JPRB, 0.65007E+00_JPRB, 0.70169E+00_JPRB, 0.75072E+00_JPRB, 0.76258E+00_JPRB ,&
&  0.73786E+00_JPRB, 0.68429E+00_JPRB, 0.61301E+00_JPRB, 0.54724E+00_JPRB, 0.50356E+00_JPRB, 0.46533E+00_JPRB, 0.52651E+00_JPRB ,&
&  0.51022E+00_JPRB, 0.49212E+00_JPRB, 0.53132E+00_JPRB, 0.48856E+00_JPRB, 0.67395E+00_JPRB, 0.83030E+00_JPRB, 0.10776E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.16511E+01_JPRB, 0.20755E+01_JPRB, 0.14830E+01_JPRB, 0.10965E+01_JPRB, 0.66803E+00_JPRB, 0.59482E+00_JPRB ,&
&  0.59215E+00_JPRB, 0.57616E+00_JPRB, 0.57411E+00_JPRB, 0.58179E+00_JPRB, 0.59057E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.63305E+00_JPRB, 0.65090E+00_JPRB, 0.66892E+00_JPRB, 0.68565E+00_JPRB, 0.70188E+00_JPRB, 0.71646E+00_JPRB, 0.73251E+00_JPRB ,&
&  0.74925E+00_JPRB, 0.76721E+00_JPRB, 0.78452E+00_JPRB, 0.80599E+00_JPRB, 0.82656E+00_JPRB, 0.83776E+00_JPRB, 0.85321E+00_JPRB ,&
&  0.86019E+00_JPRB, 0.89015E+00_JPRB, 0.90195E+00_JPRB, 0.92980E+00_JPRB, 0.95485E+00_JPRB, 0.96964E+00_JPRB, 0.99930E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10221E+01_JPRB, 0.10325E+01_JPRB, 0.10564E+01_JPRB, 0.10629E+01_JPRB, 0.11196E+01_JPRB, 0.86764E+00_JPRB ,&
&  0.87635E+00_JPRB, 0.82489E+00_JPRB, 0.75526E+00_JPRB, 0.70005E+00_JPRB, 0.65571E+00_JPRB, 0.60893E+00_JPRB, 0.58512E+00_JPRB ,&
&  0.59200E+00_JPRB, 0.61146E+00_JPRB, 0.64172E+00_JPRB, 0.68457E+00_JPRB, 0.75976E+00_JPRB, 0.86584E+00_JPRB, 0.94526E+00_JPRB ,&
&  0.10374E+01_JPRB, 0.11469E+01_JPRB, 0.12449E+01_JPRB, 0.13425E+01_JPRB, 0.14551E+01_JPRB, 0.15573E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.15884E+01_JPRB, 0.15120E+01_JPRB, 0.12443E+01_JPRB, 0.13450E+01_JPRB, 0.13985E+01_JPRB, 0.12506E+01_JPRB, 0.97459E+00_JPRB ,&
&  0.74818E+00_JPRB, 0.69360E+00_JPRB, 0.75309E+00_JPRB, 0.94102E+00_JPRB, 0.99912E+00_JPRB, 0.98781E+00_JPRB, 0.93650E+00_JPRB ,&
&  0.88217E+00_JPRB, 0.81494E+00_JPRB, 0.78374E+00_JPRB, 0.79781E+00_JPRB, 0.83368E+00_JPRB, 0.84537E+00_JPRB, 0.79128E+00_JPRB ,&
&  0.74734E+00_JPRB, 0.70852E+00_JPRB, 0.64315E+00_JPRB, 0.54346E+00_JPRB, 0.48301E+00_JPRB, 0.42440E+00_JPRB, 0.47727E+00_JPRB ,&
&  0.45678E+00_JPRB, 0.40285E+00_JPRB, 0.44662E+00_JPRB, 0.53224E+00_JPRB, 0.59823E+00_JPRB, 0.67792E+00_JPRB, 0.84625E+00_JPRB ,&
&  0.98760E+00_JPRB, 0.10347E+01_JPRB, 0.96664E+00_JPRB, 0.78936E+00_JPRB, 0.65053E+00_JPRB, 0.59554E+00_JPRB, 0.59368E+00_JPRB ,&
&  0.59329E+00_JPRB, 0.58824E+00_JPRB, 0.59393E+00_JPRB, 0.60690E+00_JPRB, 0.61792E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.61056E+00_JPRB, 0.62742E+00_JPRB, 0.64455E+00_JPRB, 0.66204E+00_JPRB, 0.67713E+00_JPRB, 0.69125E+00_JPRB, 0.70613E+00_JPRB ,&
&  0.72107E+00_JPRB, 0.73876E+00_JPRB, 0.75762E+00_JPRB, 0.77786E+00_JPRB, 0.79408E+00_JPRB, 0.80101E+00_JPRB, 0.81196E+00_JPRB ,&
&  0.81996E+00_JPRB, 0.82187E+00_JPRB, 0.81511E+00_JPRB, 0.83707E+00_JPRB, 0.85950E+00_JPRB, 0.87568E+00_JPRB, 0.89952E+00_JPRB ,&
&  0.92668E+00_JPRB, 0.91540E+00_JPRB, 0.91340E+00_JPRB, 0.93719E+00_JPRB, 0.95206E+00_JPRB, 0.90335E+00_JPRB, 0.97842E+00_JPRB ,&
&  0.73494E+00_JPRB, 0.86131E+00_JPRB, 0.78549E+00_JPRB, 0.68451E+00_JPRB, 0.65843E+00_JPRB, 0.58108E+00_JPRB, 0.53982E+00_JPRB ,&
&  0.57003E+00_JPRB, 0.57265E+00_JPRB, 0.57725E+00_JPRB, 0.59990E+00_JPRB, 0.66730E+00_JPRB, 0.77050E+00_JPRB, 0.83999E+00_JPRB ,&
&  0.93544E+00_JPRB, 0.10307E+01_JPRB, 0.11405E+01_JPRB, 0.12620E+01_JPRB, 0.14018E+01_JPRB, 0.14623E+01_JPRB, 0.13832E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.11647E+01_JPRB, 0.86809E+00_JPRB, 0.93792E+00_JPRB, 0.10627E+01_JPRB, 0.10282E+01_JPRB, 0.88134E+00_JPRB ,&
&  0.76151E+00_JPRB, 0.78011E+00_JPRB, 0.94683E+00_JPRB, 0.11430E+01_JPRB, 0.12493E+01_JPRB, 0.12723E+01_JPRB, 0.12013E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.10041E+01_JPRB, 0.93440E+00_JPRB, 0.94332E+00_JPRB, 0.92949E+00_JPRB, 0.87236E+00_JPRB, 0.75247E+00_JPRB ,&
&  0.69266E+00_JPRB, 0.67856E+00_JPRB, 0.64497E+00_JPRB, 0.55816E+00_JPRB, 0.44565E+00_JPRB, 0.42546E+00_JPRB, 0.42270E+00_JPRB ,&
&  0.43667E+00_JPRB, 0.40374E+00_JPRB, 0.39672E+00_JPRB, 0.49311E+00_JPRB, 0.53427E+00_JPRB, 0.55839E+00_JPRB, 0.60876E+00_JPRB ,&
&  0.64079E+00_JPRB, 0.60125E+00_JPRB, 0.59192E+00_JPRB, 0.56878E+00_JPRB, 0.56794E+00_JPRB, 0.56290E+00_JPRB, 0.57482E+00_JPRB ,&
&  0.58137E+00_JPRB, 0.58025E+00_JPRB, 0.58597E+00_JPRB, 0.59448E+00_JPRB, 0.60000E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.57758E+00_JPRB, 0.58448E+00_JPRB, 0.58959E+00_JPRB, 0.60550E+00_JPRB, 0.62947E+00_JPRB, 0.63342E+00_JPRB, 0.64631E+00_JPRB ,&
&  0.65345E+00_JPRB, 0.66947E+00_JPRB, 0.68654E+00_JPRB, 0.70750E+00_JPRB, 0.71907E+00_JPRB, 0.72386E+00_JPRB, 0.74195E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.75247E+00_JPRB, 0.76238E+00_JPRB, 0.78133E+00_JPRB, 0.78653E+00_JPRB, 0.81544E+00_JPRB, 0.81393E+00_JPRB ,&
&  0.84292E+00_JPRB, 0.84835E+00_JPRB, 0.87671E+00_JPRB, 0.95644E+00_JPRB, 0.10499E+01_JPRB, 0.12389E+01_JPRB, 0.15792E+01_JPRB ,&
&  0.13006E+01_JPRB, 0.10167E+01_JPRB, 0.90939E+00_JPRB, 0.76828E+00_JPRB, 0.76890E+00_JPRB, 0.78861E+00_JPRB, 0.75829E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.69450E+00_JPRB, 0.67721E+00_JPRB, 0.66787E+00_JPRB, 0.68629E+00_JPRB, 0.74257E+00_JPRB, 0.77875E+00_JPRB ,&
&  0.76143E+00_JPRB, 0.84089E+00_JPRB, 0.94415E+00_JPRB, 0.10846E+01_JPRB, 0.11856E+01_JPRB, 0.14236E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.12220E+01_JPRB, 0.10345E+01_JPRB, 0.10756E+01_JPRB, 0.11675E+01_JPRB, 0.11078E+01_JPRB, 0.95987E+00_JPRB ,&
&  0.91039E+00_JPRB, 0.85366E+00_JPRB, 0.95220E+00_JPRB, 0.13510E+01_JPRB, 0.15270E+01_JPRB, 0.15773E+01_JPRB, 0.14955E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.12344E+01_JPRB, 0.10993E+01_JPRB, 0.10145E+01_JPRB, 0.96758E+00_JPRB, 0.87827E+00_JPRB, 0.76426E+00_JPRB ,&
&  0.67068E+00_JPRB, 0.66138E+00_JPRB, 0.65057E+00_JPRB, 0.56629E+00_JPRB, 0.51475E+00_JPRB, 0.39553E+00_JPRB, 0.41743E+00_JPRB ,&
&  0.40469E+00_JPRB, 0.42003E+00_JPRB, 0.40635E+00_JPRB, 0.51233E+00_JPRB, 0.47368E+00_JPRB, 0.43504E+00_JPRB, 0.44292E+00_JPRB ,&
&  0.45414E+00_JPRB, 0.46317E+00_JPRB, 0.47693E+00_JPRB, 0.48871E+00_JPRB, 0.50311E+00_JPRB, 0.52693E+00_JPRB, 0.55261E+00_JPRB ,&
&  0.57318E+00_JPRB, 0.56072E+00_JPRB, 0.56039E+00_JPRB, 0.56426E+00_JPRB, 0.57695E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.53505E+00_JPRB, 0.53169E+00_JPRB, 0.52387E+00_JPRB, 0.52412E+00_JPRB, 0.52859E+00_JPRB, 0.54152E+00_JPRB, 0.54956E+00_JPRB ,&
&  0.54795E+00_JPRB, 0.55268E+00_JPRB, 0.53654E+00_JPRB, 0.53757E+00_JPRB, 0.55272E+00_JPRB, 0.56171E+00_JPRB, 0.57028E+00_JPRB ,&
&  0.60374E+00_JPRB, 0.61717E+00_JPRB, 0.61655E+00_JPRB, 0.62274E+00_JPRB, 0.62002E+00_JPRB, 0.66450E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.65498E+00_JPRB, 0.68053E+00_JPRB, 0.79789E+00_JPRB, 0.11005E+01_JPRB, 0.13840E+01_JPRB, 0.13810E+01_JPRB, 0.15614E+01_JPRB ,&
&  0.14817E+01_JPRB, 0.12708E+01_JPRB, 0.12807E+01_JPRB, 0.10784E+01_JPRB, 0.10747E+01_JPRB, 0.11123E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.11244E+01_JPRB, 0.11085E+01_JPRB, 0.11110E+01_JPRB, 0.11207E+01_JPRB, 0.11162E+01_JPRB, 0.11109E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.10326E+01_JPRB, 0.11007E+01_JPRB, 0.12307E+01_JPRB, 0.13857E+01_JPRB, 0.15710E+01_JPRB, 0.16315E+01_JPRB ,&
&  0.16831E+01_JPRB, 0.17118E+01_JPRB, 0.16045E+01_JPRB, 0.16373E+01_JPRB, 0.17334E+01_JPRB, 0.15935E+01_JPRB, 0.14603E+01_JPRB ,&
&  0.13221E+01_JPRB, 0.10213E+01_JPRB, 0.10423E+01_JPRB, 0.15720E+01_JPRB, 0.17448E+01_JPRB, 0.17022E+01_JPRB, 0.17199E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.15012E+01_JPRB, 0.12984E+01_JPRB, 0.11309E+01_JPRB, 0.99069E+00_JPRB, 0.85944E+00_JPRB, 0.80047E+00_JPRB ,&
&  0.69332E+00_JPRB, 0.65096E+00_JPRB, 0.65491E+00_JPRB, 0.61093E+00_JPRB, 0.54380E+00_JPRB, 0.48877E+00_JPRB, 0.46568E+00_JPRB ,&
&  0.45653E+00_JPRB, 0.44159E+00_JPRB, 0.54506E+00_JPRB, 0.67599E+00_JPRB, 0.45302E+00_JPRB, 0.41394E+00_JPRB, 0.38207E+00_JPRB ,&
&  0.39670E+00_JPRB, 0.41576E+00_JPRB, 0.42241E+00_JPRB, 0.43148E+00_JPRB, 0.44879E+00_JPRB, 0.46391E+00_JPRB, 0.48671E+00_JPRB ,&
&  0.50067E+00_JPRB, 0.51187E+00_JPRB, 0.52380E+00_JPRB, 0.53728E+00_JPRB, 0.53307E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.59164E+00_JPRB, 0.62014E+00_JPRB, 0.63663E+00_JPRB, 0.65641E+00_JPRB, 0.66379E+00_JPRB, 0.65616E+00_JPRB, 0.62693E+00_JPRB ,&
&  0.60382E+00_JPRB, 0.57988E+00_JPRB, 0.57075E+00_JPRB, 0.55558E+00_JPRB, 0.57283E+00_JPRB, 0.60785E+00_JPRB, 0.63691E+00_JPRB ,&
&  0.71041E+00_JPRB, 0.66997E+00_JPRB, 0.67386E+00_JPRB, 0.70626E+00_JPRB, 0.71780E+00_JPRB, 0.76023E+00_JPRB, 0.88911E+00_JPRB ,&
&  0.10948E+01_JPRB, 0.13268E+01_JPRB, 0.16634E+01_JPRB, 0.22194E+01_JPRB, 0.23607E+01_JPRB, 0.19238E+01_JPRB, 0.20990E+01_JPRB ,&
&  0.19738E+01_JPRB, 0.16436E+01_JPRB, 0.15177E+01_JPRB, 0.14338E+01_JPRB, 0.14229E+01_JPRB, 0.14243E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.15160E+01_JPRB, 0.15420E+01_JPRB, 0.15551E+01_JPRB, 0.15777E+01_JPRB, 0.16108E+01_JPRB, 0.16602E+01_JPRB ,&
&  0.16963E+01_JPRB, 0.17023E+01_JPRB, 0.17294E+01_JPRB, 0.16995E+01_JPRB, 0.18022E+01_JPRB, 0.19137E+01_JPRB, 0.20243E+01_JPRB ,&
&  0.20924E+01_JPRB, 0.21324E+01_JPRB, 0.22818E+01_JPRB, 0.22945E+01_JPRB, 0.22694E+01_JPRB, 0.22216E+01_JPRB, 0.20869E+01_JPRB ,&
&  0.19039E+01_JPRB, 0.15285E+01_JPRB, 0.14253E+01_JPRB, 0.22494E+01_JPRB, 0.22662E+01_JPRB, 0.18794E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.17174E+01_JPRB, 0.15365E+01_JPRB, 0.12872E+01_JPRB, 0.10888E+01_JPRB, 0.93819E+00_JPRB, 0.82519E+00_JPRB ,&
&  0.79143E+00_JPRB, 0.70312E+00_JPRB, 0.67417E+00_JPRB, 0.66053E+00_JPRB, 0.62116E+00_JPRB, 0.59530E+00_JPRB, 0.78202E+00_JPRB ,&
&  0.68546E+00_JPRB, 0.52525E+00_JPRB, 0.54391E+00_JPRB, 0.66512E+00_JPRB, 0.61187E+00_JPRB, 0.42792E+00_JPRB, 0.39900E+00_JPRB ,&
&  0.41298E+00_JPRB, 0.42643E+00_JPRB, 0.43906E+00_JPRB, 0.45102E+00_JPRB, 0.46045E+00_JPRB, 0.47978E+00_JPRB, 0.49704E+00_JPRB ,&
&  0.51561E+00_JPRB, 0.54481E+00_JPRB, 0.55706E+00_JPRB, 0.57145E+00_JPRB, 0.58742E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.84923E+00_JPRB, 0.89252E+00_JPRB, 0.93202E+00_JPRB, 0.95523E+00_JPRB, 0.95823E+00_JPRB, 0.96608E+00_JPRB, 0.96607E+00_JPRB ,&
&  0.97582E+00_JPRB, 0.97517E+00_JPRB, 0.10045E+01_JPRB, 0.10719E+01_JPRB, 0.12030E+01_JPRB, 0.12743E+01_JPRB, 0.13652E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.13962E+01_JPRB, 0.14247E+01_JPRB, 0.14664E+01_JPRB, 0.15469E+01_JPRB, 0.16640E+01_JPRB, 0.19064E+01_JPRB ,&
&  0.23528E+01_JPRB, 0.26432E+01_JPRB, 0.26218E+01_JPRB, 0.23059E+01_JPRB, 0.19191E+01_JPRB, 0.17457E+01_JPRB, 0.20828E+01_JPRB ,&
&  0.21436E+01_JPRB, 0.18525E+01_JPRB, 0.16105E+01_JPRB, 0.15677E+01_JPRB, 0.15638E+01_JPRB, 0.15766E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.15978E+01_JPRB, 0.16222E+01_JPRB, 0.16346E+01_JPRB, 0.16476E+01_JPRB, 0.16934E+01_JPRB, 0.17703E+01_JPRB, 0.18729E+01_JPRB ,&
&  0.20234E+01_JPRB, 0.21249E+01_JPRB, 0.21616E+01_JPRB, 0.21613E+01_JPRB, 0.21584E+01_JPRB, 0.21925E+01_JPRB, 0.22791E+01_JPRB ,&
&  0.23787E+01_JPRB, 0.24126E+01_JPRB, 0.26094E+01_JPRB, 0.27426E+01_JPRB, 0.26700E+01_JPRB, 0.25828E+01_JPRB, 0.25216E+01_JPRB ,&
&  0.24814E+01_JPRB, 0.24081E+01_JPRB, 0.21268E+01_JPRB, 0.29402E+01_JPRB, 0.30304E+01_JPRB, 0.25985E+01_JPRB, 0.21935E+01_JPRB ,&
&  0.20047E+01_JPRB, 0.18723E+01_JPRB, 0.15988E+01_JPRB, 0.12892E+01_JPRB, 0.11070E+01_JPRB, 0.10402E+01_JPRB, 0.87141E+00_JPRB ,&
&  0.90183E+00_JPRB, 0.79079E+00_JPRB, 0.72605E+00_JPRB, 0.70264E+00_JPRB, 0.67817E+00_JPRB, 0.68233E+00_JPRB, 0.79134E+00_JPRB ,&
&  0.73209E+00_JPRB, 0.68530E+00_JPRB, 0.73296E+00_JPRB, 0.11027E+01_JPRB, 0.18633E+01_JPRB, 0.51742E+00_JPRB, 0.43056E+00_JPRB ,&
&  0.45230E+00_JPRB, 0.48237E+00_JPRB, 0.51763E+00_JPRB, 0.54055E+00_JPRB, 0.55497E+00_JPRB, 0.58859E+00_JPRB, 0.64153E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.69718E+00_JPRB, 0.72713E+00_JPRB, 0.78487E+00_JPRB, 0.81413E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.10366E+01_JPRB, 0.10790E+01_JPRB, 0.11062E+01_JPRB, 0.11500E+01_JPRB, 0.11923E+01_JPRB, 0.12368E+01_JPRB, 0.13015E+01_JPRB ,&
&  0.13889E+01_JPRB, 0.14852E+01_JPRB, 0.15754E+01_JPRB, 0.16429E+01_JPRB, 0.17059E+01_JPRB, 0.17510E+01_JPRB, 0.17898E+01_JPRB ,&
&  0.18179E+01_JPRB, 0.18853E+01_JPRB, 0.19615E+01_JPRB, 0.20632E+01_JPRB, 0.21869E+01_JPRB, 0.23289E+01_JPRB, 0.24761E+01_JPRB ,&
&  0.25716E+01_JPRB, 0.24612E+01_JPRB, 0.21355E+01_JPRB, 0.15418E+01_JPRB, 0.13472E+01_JPRB, 0.16120E+01_JPRB, 0.17600E+01_JPRB ,&
&  0.17369E+01_JPRB, 0.16334E+01_JPRB, 0.15629E+01_JPRB, 0.15948E+01_JPRB, 0.16213E+01_JPRB, 0.16455E+01_JPRB, 0.16544E+01_JPRB ,&
&  0.16768E+01_JPRB, 0.17105E+01_JPRB, 0.17321E+01_JPRB, 0.17519E+01_JPRB, 0.18068E+01_JPRB, 0.19267E+01_JPRB, 0.21108E+01_JPRB ,&
&  0.23351E+01_JPRB, 0.24668E+01_JPRB, 0.25544E+01_JPRB, 0.26100E+01_JPRB, 0.25218E+01_JPRB, 0.24542E+01_JPRB, 0.24550E+01_JPRB ,&
&  0.25351E+01_JPRB, 0.26415E+01_JPRB, 0.28431E+01_JPRB, 0.30283E+01_JPRB, 0.30862E+01_JPRB, 0.29068E+01_JPRB, 0.28566E+01_JPRB ,&
&  0.29079E+01_JPRB, 0.28275E+01_JPRB, 0.28146E+01_JPRB, 0.34501E+01_JPRB, 0.23038E+01_JPRB, 0.23109E+01_JPRB, 0.22268E+01_JPRB ,&
&  0.21593E+01_JPRB, 0.21579E+01_JPRB, 0.17108E+01_JPRB, 0.12959E+01_JPRB, 0.11226E+01_JPRB, 0.85064E+00_JPRB, 0.87003E+00_JPRB ,&
&  0.98211E+00_JPRB, 0.91626E+00_JPRB, 0.79343E+00_JPRB, 0.74249E+00_JPRB, 0.77606E+00_JPRB, 0.75050E+00_JPRB, 0.76934E+00_JPRB ,&
&  0.80259E+00_JPRB, 0.77638E+00_JPRB, 0.78082E+00_JPRB, 0.95029E+00_JPRB, 0.10439E+01_JPRB, 0.54672E+00_JPRB, 0.47341E+00_JPRB ,&
&  0.49303E+00_JPRB, 0.53777E+00_JPRB, 0.59086E+00_JPRB, 0.63758E+00_JPRB, 0.67975E+00_JPRB, 0.72669E+00_JPRB, 0.77803E+00_JPRB ,&
&  0.83607E+00_JPRB, 0.90352E+00_JPRB, 0.94841E+00_JPRB, 0.98121E+00_JPRB, 0.10045E+01_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.10572E+01_JPRB, 0.11045E+01_JPRB, 0.11630E+01_JPRB, 0.12390E+01_JPRB, 0.13173E+01_JPRB, 0.13443E+01_JPRB, 0.13401E+01_JPRB ,&
&  0.13770E+01_JPRB, 0.14398E+01_JPRB, 0.15165E+01_JPRB, 0.15745E+01_JPRB, 0.15995E+01_JPRB, 0.16507E+01_JPRB, 0.17129E+01_JPRB ,&
&  0.17741E+01_JPRB, 0.18467E+01_JPRB, 0.19438E+01_JPRB, 0.20621E+01_JPRB, 0.22170E+01_JPRB, 0.24085E+01_JPRB, 0.24897E+01_JPRB ,&
&  0.19683E+01_JPRB, 0.13933E+01_JPRB, 0.17913E+01_JPRB, 0.17371E+01_JPRB, 0.15492E+01_JPRB, 0.14491E+01_JPRB, 0.14590E+01_JPRB ,&
&  0.15673E+01_JPRB, 0.15962E+01_JPRB, 0.16016E+01_JPRB, 0.15927E+01_JPRB, 0.15697E+01_JPRB, 0.16106E+01_JPRB, 0.16521E+01_JPRB ,&
&  0.16907E+01_JPRB, 0.17284E+01_JPRB, 0.17642E+01_JPRB, 0.17981E+01_JPRB, 0.18740E+01_JPRB, 0.20153E+01_JPRB, 0.22477E+01_JPRB ,&
&  0.24803E+01_JPRB, 0.26260E+01_JPRB, 0.27474E+01_JPRB, 0.27889E+01_JPRB, 0.26988E+01_JPRB, 0.26283E+01_JPRB, 0.26459E+01_JPRB ,&
&  0.27470E+01_JPRB, 0.28793E+01_JPRB, 0.30499E+01_JPRB, 0.31805E+01_JPRB, 0.33120E+01_JPRB, 0.30668E+01_JPRB, 0.31539E+01_JPRB ,&
&  0.32570E+01_JPRB, 0.31390E+01_JPRB, 0.32624E+01_JPRB, 0.35868E+01_JPRB, 0.24863E+01_JPRB, 0.23546E+01_JPRB, 0.20645E+01_JPRB ,&
&  0.22385E+01_JPRB, 0.22372E+01_JPRB, 0.19715E+01_JPRB, 0.15967E+01_JPRB, 0.12923E+01_JPRB, 0.93751E+00_JPRB, 0.96762E+00_JPRB ,&
&  0.10871E+01_JPRB, 0.11149E+01_JPRB, 0.88615E+00_JPRB, 0.83242E+00_JPRB, 0.75123E+00_JPRB, 0.71170E+00_JPRB, 0.76342E+00_JPRB ,&
&  0.85897E+00_JPRB, 0.82206E+00_JPRB, 0.80083E+00_JPRB, 0.84361E+00_JPRB, 0.78991E+00_JPRB, 0.58753E+00_JPRB, 0.51264E+00_JPRB ,&
&  0.53144E+00_JPRB, 0.57370E+00_JPRB, 0.63652E+00_JPRB, 0.70986E+00_JPRB, 0.74608E+00_JPRB, 0.78471E+00_JPRB, 0.82239E+00_JPRB ,&
&  0.89719E+00_JPRB, 0.94884E+00_JPRB, 0.98497E+00_JPRB, 0.99380E+00_JPRB, 0.10271E+01_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.99266E+00_JPRB, 0.10466E+01_JPRB, 0.11234E+01_JPRB, 0.12376E+01_JPRB, 0.13950E+01_JPRB, 0.15466E+01_JPRB, 0.14507E+01_JPRB ,&
&  0.13336E+01_JPRB, 0.11942E+01_JPRB, 0.12314E+01_JPRB, 0.12685E+01_JPRB, 0.13047E+01_JPRB, 0.13714E+01_JPRB, 0.14671E+01_JPRB ,&
&  0.15798E+01_JPRB, 0.16819E+01_JPRB, 0.17885E+01_JPRB, 0.18880E+01_JPRB, 0.19657E+01_JPRB, 0.21633E+01_JPRB, 0.20571E+01_JPRB ,&
&  0.15212E+01_JPRB, 0.13244E+01_JPRB, 0.17626E+01_JPRB, 0.15401E+01_JPRB, 0.15648E+01_JPRB, 0.16380E+01_JPRB, 0.15732E+01_JPRB ,&
&  0.15365E+01_JPRB, 0.14904E+01_JPRB, 0.14856E+01_JPRB, 0.14693E+01_JPRB, 0.14417E+01_JPRB, 0.15062E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.16286E+01_JPRB, 0.16605E+01_JPRB, 0.17013E+01_JPRB, 0.17429E+01_JPRB, 0.18152E+01_JPRB, 0.19528E+01_JPRB, 0.22223E+01_JPRB ,&
&  0.24464E+01_JPRB, 0.26644E+01_JPRB, 0.28780E+01_JPRB, 0.29131E+01_JPRB, 0.28425E+01_JPRB, 0.28545E+01_JPRB, 0.29578E+01_JPRB ,&
&  0.30814E+01_JPRB, 0.32741E+01_JPRB, 0.33438E+01_JPRB, 0.33068E+01_JPRB, 0.30843E+01_JPRB, 0.33630E+01_JPRB, 0.31567E+01_JPRB ,&
&  0.33646E+01_JPRB, 0.33206E+01_JPRB, 0.37849E+01_JPRB, 0.30977E+01_JPRB, 0.29438E+01_JPRB, 0.28572E+01_JPRB, 0.26253E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.23888E+01_JPRB, 0.22266E+01_JPRB, 0.18650E+01_JPRB, 0.15278E+01_JPRB, 0.12135E+01_JPRB, 0.13176E+01_JPRB ,&
&  0.13314E+01_JPRB, 0.13511E+01_JPRB, 0.96602E+00_JPRB, 0.80393E+00_JPRB, 0.65373E+00_JPRB, 0.60872E+00_JPRB, 0.75009E+00_JPRB ,&
&  0.91501E+00_JPRB, 0.95447E+00_JPRB, 0.86134E+00_JPRB, 0.80019E+00_JPRB, 0.74613E+00_JPRB, 0.65983E+00_JPRB, 0.58976E+00_JPRB ,&
&  0.61589E+00_JPRB, 0.63313E+00_JPRB, 0.67088E+00_JPRB, 0.73763E+00_JPRB, 0.77359E+00_JPRB, 0.81458E+00_JPRB, 0.81230E+00_JPRB ,&
&  0.84921E+00_JPRB, 0.88311E+00_JPRB, 0.90021E+00_JPRB, 0.92045E+00_JPRB, 0.94665E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.84457E+00_JPRB, 0.86061E+00_JPRB, 0.90098E+00_JPRB, 0.96475E+00_JPRB, 0.10207E+01_JPRB, 0.10716E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10390E+01_JPRB, 0.10289E+01_JPRB, 0.99463E+00_JPRB, 0.97378E+00_JPRB, 0.96716E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.13455E+01_JPRB, 0.15443E+01_JPRB, 0.16828E+01_JPRB, 0.17465E+01_JPRB, 0.18793E+01_JPRB, 0.16465E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.18563E+01_JPRB, 0.14743E+01_JPRB, 0.15310E+01_JPRB, 0.16027E+01_JPRB, 0.17524E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.13704E+01_JPRB, 0.13723E+01_JPRB, 0.13720E+01_JPRB, 0.13746E+01_JPRB, 0.13541E+01_JPRB, 0.13946E+01_JPRB, 0.14638E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.15957E+01_JPRB, 0.16252E+01_JPRB, 0.16433E+01_JPRB, 0.16911E+01_JPRB, 0.17746E+01_JPRB, 0.20140E+01_JPRB ,&
&  0.22592E+01_JPRB, 0.25561E+01_JPRB, 0.28529E+01_JPRB, 0.31154E+01_JPRB, 0.30435E+01_JPRB, 0.31094E+01_JPRB, 0.33420E+01_JPRB ,&
&  0.33925E+01_JPRB, 0.33094E+01_JPRB, 0.35136E+01_JPRB, 0.35310E+01_JPRB, 0.34759E+01_JPRB, 0.36593E+01_JPRB, 0.31965E+01_JPRB ,&
&  0.34783E+01_JPRB, 0.33353E+01_JPRB, 0.38004E+01_JPRB, 0.32958E+01_JPRB, 0.31857E+01_JPRB, 0.33897E+01_JPRB, 0.31764E+01_JPRB ,&
&  0.27553E+01_JPRB, 0.27120E+01_JPRB, 0.24199E+01_JPRB, 0.21066E+01_JPRB, 0.18944E+01_JPRB, 0.17217E+01_JPRB, 0.17816E+01_JPRB ,&
&  0.17371E+01_JPRB, 0.15386E+01_JPRB, 0.11502E+01_JPRB, 0.69511E+00_JPRB, 0.57910E+00_JPRB, 0.60220E+00_JPRB, 0.77672E+00_JPRB ,&
&  0.97471E+00_JPRB, 0.10792E+01_JPRB, 0.11151E+01_JPRB, 0.93675E+00_JPRB, 0.80440E+00_JPRB, 0.72515E+00_JPRB, 0.66967E+00_JPRB ,&
&  0.65796E+00_JPRB, 0.65987E+00_JPRB, 0.66834E+00_JPRB, 0.73233E+00_JPRB, 0.80708E+00_JPRB, 0.84989E+00_JPRB, 0.79442E+00_JPRB ,&
&  0.75257E+00_JPRB, 0.74963E+00_JPRB, 0.78134E+00_JPRB, 0.80722E+00_JPRB, 0.82961E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.74643E+00_JPRB, 0.73547E+00_JPRB, 0.74464E+00_JPRB, 0.77899E+00_JPRB, 0.79450E+00_JPRB, 0.80350E+00_JPRB, 0.85516E+00_JPRB ,&
&  0.87211E+00_JPRB, 0.86142E+00_JPRB, 0.86771E+00_JPRB, 0.86711E+00_JPRB, 0.84848E+00_JPRB, 0.80668E+00_JPRB, 0.77829E+00_JPRB ,&
&  0.78487E+00_JPRB, 0.88853E+00_JPRB, 0.11117E+01_JPRB, 0.14047E+01_JPRB, 0.16033E+01_JPRB, 0.19463E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.15597E+01_JPRB, 0.13940E+01_JPRB, 0.14972E+01_JPRB, 0.15648E+01_JPRB, 0.16548E+01_JPRB, 0.14182E+01_JPRB ,&
&  0.13658E+01_JPRB, 0.13615E+01_JPRB, 0.13237E+01_JPRB, 0.13249E+01_JPRB, 0.13315E+01_JPRB, 0.13470E+01_JPRB, 0.13562E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.14726E+01_JPRB, 0.15232E+01_JPRB, 0.15407E+01_JPRB, 0.15599E+01_JPRB, 0.16074E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.20439E+01_JPRB, 0.23215E+01_JPRB, 0.26986E+01_JPRB, 0.30019E+01_JPRB, 0.32159E+01_JPRB, 0.33196E+01_JPRB, 0.36593E+01_JPRB ,&
&  0.36666E+01_JPRB, 0.35593E+01_JPRB, 0.37943E+01_JPRB, 0.39524E+01_JPRB, 0.40276E+01_JPRB, 0.40775E+01_JPRB, 0.36192E+01_JPRB ,&
&  0.35055E+01_JPRB, 0.35533E+01_JPRB, 0.36544E+01_JPRB, 0.32885E+01_JPRB, 0.32435E+01_JPRB, 0.36356E+01_JPRB, 0.37872E+01_JPRB ,&
&  0.34302E+01_JPRB, 0.31242E+01_JPRB, 0.28722E+01_JPRB, 0.23933E+01_JPRB, 0.21790E+01_JPRB, 0.20917E+01_JPRB, 0.23129E+01_JPRB ,&
&  0.21964E+01_JPRB, 0.18134E+01_JPRB, 0.99242E+00_JPRB, 0.53747E+00_JPRB, 0.48131E+00_JPRB, 0.61399E+00_JPRB, 0.84738E+00_JPRB ,&
&  0.11214E+01_JPRB, 0.12979E+01_JPRB, 0.13939E+01_JPRB, 0.14201E+01_JPRB, 0.12111E+01_JPRB, 0.10030E+01_JPRB, 0.86189E+00_JPRB ,&
&  0.77790E+00_JPRB, 0.70849E+00_JPRB, 0.67696E+00_JPRB, 0.69080E+00_JPRB, 0.76688E+00_JPRB, 0.84809E+00_JPRB, 0.88902E+00_JPRB ,&
&  0.83861E+00_JPRB, 0.75730E+00_JPRB, 0.72031E+00_JPRB, 0.71399E+00_JPRB, 0.72718E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.67644E+00_JPRB, 0.67216E+00_JPRB, 0.68102E+00_JPRB, 0.70573E+00_JPRB, 0.72437E+00_JPRB, 0.71427E+00_JPRB, 0.68908E+00_JPRB ,&
&  0.70225E+00_JPRB, 0.69512E+00_JPRB, 0.71888E+00_JPRB, 0.73310E+00_JPRB, 0.74879E+00_JPRB, 0.73811E+00_JPRB, 0.71929E+00_JPRB ,&
&  0.69060E+00_JPRB, 0.70305E+00_JPRB, 0.85391E+00_JPRB, 0.11386E+01_JPRB, 0.13860E+01_JPRB, 0.15385E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.15019E+01_JPRB, 0.15289E+01_JPRB, 0.15267E+01_JPRB, 0.17914E+01_JPRB, 0.19683E+01_JPRB, 0.22751E+01_JPRB, 0.15234E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.13788E+01_JPRB, 0.13659E+01_JPRB, 0.13738E+01_JPRB, 0.13997E+01_JPRB, 0.13853E+01_JPRB, 0.13750E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.13605E+01_JPRB, 0.13893E+01_JPRB, 0.14159E+01_JPRB, 0.14430E+01_JPRB, 0.14835E+01_JPRB, 0.16347E+01_JPRB ,&
&  0.18939E+01_JPRB, 0.22085E+01_JPRB, 0.25743E+01_JPRB, 0.29899E+01_JPRB, 0.32633E+01_JPRB, 0.35431E+01_JPRB, 0.39992E+01_JPRB ,&
&  0.42079E+01_JPRB, 0.43658E+01_JPRB, 0.43114E+01_JPRB, 0.42073E+01_JPRB, 0.42389E+01_JPRB, 0.46348E+01_JPRB, 0.42623E+01_JPRB ,&
&  0.38282E+01_JPRB, 0.35126E+01_JPRB, 0.32854E+01_JPRB, 0.32014E+01_JPRB, 0.32394E+01_JPRB, 0.38167E+01_JPRB, 0.37310E+01_JPRB ,&
&  0.31762E+01_JPRB, 0.27684E+01_JPRB, 0.26498E+01_JPRB, 0.26121E+01_JPRB, 0.25915E+01_JPRB, 0.25936E+01_JPRB, 0.30277E+01_JPRB ,&
&  0.22825E+01_JPRB, 0.10975E+01_JPRB, 0.63453E+00_JPRB, 0.58443E+00_JPRB, 0.39995E+00_JPRB, 0.46414E+00_JPRB, 0.11436E+01_JPRB ,&
&  0.16201E+01_JPRB, 0.20973E+01_JPRB, 0.21993E+01_JPRB, 0.19613E+01_JPRB, 0.17294E+01_JPRB, 0.14396E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.96764E+00_JPRB, 0.85038E+00_JPRB, 0.79639E+00_JPRB, 0.77701E+00_JPRB, 0.82596E+00_JPRB, 0.89723E+00_JPRB, 0.94871E+00_JPRB ,&
&  0.93673E+00_JPRB, 0.86924E+00_JPRB, 0.76310E+00_JPRB, 0.67877E+00_JPRB, 0.65975E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.67950E+00_JPRB, 0.69479E+00_JPRB, 0.71720E+00_JPRB, 0.71157E+00_JPRB, 0.69534E+00_JPRB, 0.70148E+00_JPRB, 0.67338E+00_JPRB ,&
&  0.65169E+00_JPRB, 0.64095E+00_JPRB, 0.64104E+00_JPRB, 0.65608E+00_JPRB, 0.67982E+00_JPRB, 0.69466E+00_JPRB, 0.67610E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.65522E+00_JPRB, 0.73036E+00_JPRB, 0.98377E+00_JPRB, 0.12991E+01_JPRB, 0.16104E+01_JPRB, 0.16798E+01_JPRB ,&
&  0.15499E+01_JPRB, 0.19501E+01_JPRB, 0.25601E+01_JPRB, 0.28518E+01_JPRB, 0.26307E+01_JPRB, 0.27117E+01_JPRB, 0.19345E+01_JPRB ,&
&  0.16274E+01_JPRB, 0.16116E+01_JPRB, 0.15918E+01_JPRB, 0.15850E+01_JPRB, 0.15487E+01_JPRB, 0.14758E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14156E+01_JPRB, 0.13361E+01_JPRB, 0.13061E+01_JPRB, 0.13066E+01_JPRB, 0.13377E+01_JPRB, 0.13833E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.17656E+01_JPRB, 0.20661E+01_JPRB, 0.24386E+01_JPRB, 0.32647E+01_JPRB, 0.28573E+01_JPRB, 0.38013E+01_JPRB, 0.42412E+01_JPRB ,&
&  0.44414E+01_JPRB, 0.45618E+01_JPRB, 0.42691E+01_JPRB, 0.47155E+01_JPRB, 0.47615E+01_JPRB, 0.50138E+01_JPRB, 0.47361E+01_JPRB ,&
&  0.42685E+01_JPRB, 0.36166E+01_JPRB, 0.34499E+01_JPRB, 0.33547E+01_JPRB, 0.33075E+01_JPRB, 0.38835E+01_JPRB, 0.29494E+01_JPRB ,&
&  0.27323E+01_JPRB, 0.31518E+01_JPRB, 0.30096E+01_JPRB, 0.27166E+01_JPRB, 0.29371E+01_JPRB, 0.30512E+01_JPRB, 0.23245E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.72948E+00_JPRB, 0.62112E+00_JPRB, 0.53411E+00_JPRB, 0.45479E+00_JPRB, 0.68429E+00_JPRB, 0.28872E+01_JPRB ,&
&  0.20644E+01_JPRB, 0.29243E+01_JPRB, 0.29708E+01_JPRB, 0.27394E+01_JPRB, 0.24197E+01_JPRB, 0.20814E+01_JPRB, 0.22585E+01_JPRB ,&
&  0.19304E+01_JPRB, 0.15988E+01_JPRB, 0.13564E+01_JPRB, 0.11835E+01_JPRB, 0.10911E+01_JPRB, 0.10608E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.96599E+00_JPRB, 0.89204E+00_JPRB, 0.78389E+00_JPRB, 0.69097E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.77357E+00_JPRB, 0.76361E+00_JPRB, 0.76529E+00_JPRB, 0.75583E+00_JPRB, 0.73940E+00_JPRB, 0.74242E+00_JPRB, 0.72743E+00_JPRB ,&
&  0.69537E+00_JPRB, 0.66865E+00_JPRB, 0.65630E+00_JPRB, 0.66803E+00_JPRB, 0.67630E+00_JPRB, 0.68139E+00_JPRB, 0.66053E+00_JPRB ,&
&  0.63241E+00_JPRB, 0.64156E+00_JPRB, 0.78232E+00_JPRB, 0.88329E+00_JPRB, 0.12369E+01_JPRB, 0.15302E+01_JPRB, 0.17545E+01_JPRB ,&
&  0.17177E+01_JPRB, 0.21349E+01_JPRB, 0.27808E+01_JPRB, 0.30790E+01_JPRB, 0.39403E+01_JPRB, 0.38173E+01_JPRB, 0.32495E+01_JPRB ,&
&  0.24690E+01_JPRB, 0.23644E+01_JPRB, 0.22785E+01_JPRB, 0.20989E+01_JPRB, 0.18759E+01_JPRB, 0.16673E+01_JPRB, 0.15187E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.13848E+01_JPRB, 0.12931E+01_JPRB, 0.12447E+01_JPRB, 0.12609E+01_JPRB, 0.13018E+01_JPRB, 0.14376E+01_JPRB ,&
&  0.16667E+01_JPRB, 0.19395E+01_JPRB, 0.22691E+01_JPRB, 0.30575E+01_JPRB, 0.31760E+01_JPRB, 0.35090E+01_JPRB, 0.42813E+01_JPRB ,&
&  0.46278E+01_JPRB, 0.49047E+01_JPRB, 0.45469E+01_JPRB, 0.55251E+01_JPRB, 0.51322E+01_JPRB, 0.54081E+01_JPRB, 0.54748E+01_JPRB ,&
&  0.47633E+01_JPRB, 0.44514E+01_JPRB, 0.35404E+01_JPRB, 0.34893E+01_JPRB, 0.36417E+01_JPRB, 0.32457E+01_JPRB, 0.31127E+01_JPRB ,&
&  0.34366E+01_JPRB, 0.30690E+01_JPRB, 0.28194E+01_JPRB, 0.26570E+01_JPRB, 0.32096E+01_JPRB, 0.28622E+01_JPRB, 0.13438E+01_JPRB ,&
&  0.95505E+00_JPRB, 0.87830E+00_JPRB, 0.65952E+00_JPRB, 0.56321E+00_JPRB, 0.55810E+00_JPRB, 0.77629E+00_JPRB, 0.15328E+01_JPRB ,&
&  0.17483E+01_JPRB, 0.28265E+01_JPRB, 0.29631E+01_JPRB, 0.30008E+01_JPRB, 0.25748E+01_JPRB, 0.23061E+01_JPRB, 0.27595E+01_JPRB ,&
&  0.27096E+01_JPRB, 0.23442E+01_JPRB, 0.19470E+01_JPRB, 0.17255E+01_JPRB, 0.15279E+01_JPRB, 0.13560E+01_JPRB, 0.12309E+01_JPRB ,&
&  0.11473E+01_JPRB, 0.10459E+01_JPRB, 0.98504E+00_JPRB, 0.93526E+00_JPRB, 0.83930E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.87491E+00_JPRB, 0.83348E+00_JPRB, 0.81165E+00_JPRB, 0.77209E+00_JPRB, 0.74052E+00_JPRB, 0.73204E+00_JPRB, 0.74903E+00_JPRB ,&
&  0.73868E+00_JPRB, 0.71473E+00_JPRB, 0.69250E+00_JPRB, 0.68912E+00_JPRB, 0.69963E+00_JPRB, 0.70047E+00_JPRB, 0.67256E+00_JPRB ,&
&  0.64833E+00_JPRB, 0.65664E+00_JPRB, 0.77693E+00_JPRB, 0.85274E+00_JPRB, 0.10556E+01_JPRB, 0.13431E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.18138E+01_JPRB, 0.22582E+01_JPRB, 0.28108E+01_JPRB, 0.37324E+01_JPRB, 0.48011E+01_JPRB, 0.52502E+01_JPRB, 0.55225E+01_JPRB ,&
&  0.45647E+01_JPRB, 0.36531E+01_JPRB, 0.31047E+01_JPRB, 0.27093E+01_JPRB, 0.22624E+01_JPRB, 0.19054E+01_JPRB, 0.16362E+01_JPRB ,&
&  0.14707E+01_JPRB, 0.13475E+01_JPRB, 0.13229E+01_JPRB, 0.12549E+01_JPRB, 0.12301E+01_JPRB, 0.12484E+01_JPRB, 0.13832E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.18561E+01_JPRB, 0.22753E+01_JPRB, 0.28281E+01_JPRB, 0.35497E+01_JPRB, 0.43450E+01_JPRB, 0.45282E+01_JPRB ,&
&  0.43006E+01_JPRB, 0.42114E+01_JPRB, 0.49898E+01_JPRB, 0.56551E+01_JPRB, 0.54147E+01_JPRB, 0.60680E+01_JPRB, 0.60454E+01_JPRB ,&
&  0.52151E+01_JPRB, 0.47412E+01_JPRB, 0.38836E+01_JPRB, 0.41694E+01_JPRB, 0.36185E+01_JPRB, 0.31271E+01_JPRB, 0.36112E+01_JPRB ,&
&  0.37766E+01_JPRB, 0.33410E+01_JPRB, 0.28262E+01_JPRB, 0.24723E+01_JPRB, 0.24176E+01_JPRB, 0.18420E+01_JPRB, 0.12985E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.89503E+00_JPRB, 0.73707E+00_JPRB, 0.75399E+00_JPRB, 0.64619E+00_JPRB, 0.84464E+00_JPRB, 0.13873E+01_JPRB ,&
&  0.21424E+01_JPRB, 0.25198E+01_JPRB, 0.30087E+01_JPRB, 0.27295E+01_JPRB, 0.22860E+01_JPRB, 0.20955E+01_JPRB, 0.26245E+01_JPRB ,&
&  0.20829E+01_JPRB, 0.20456E+01_JPRB, 0.20040E+01_JPRB, 0.18367E+01_JPRB, 0.16922E+01_JPRB, 0.15586E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.12431E+01_JPRB, 0.11020E+01_JPRB, 0.10184E+01_JPRB, 0.98522E+00_JPRB, 0.94059E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.92760E+00_JPRB, 0.88945E+00_JPRB, 0.83142E+00_JPRB, 0.74993E+00_JPRB, 0.71005E+00_JPRB, 0.70722E+00_JPRB, 0.72360E+00_JPRB ,&
&  0.73249E+00_JPRB, 0.73665E+00_JPRB, 0.72934E+00_JPRB, 0.72034E+00_JPRB, 0.72601E+00_JPRB, 0.71717E+00_JPRB, 0.68611E+00_JPRB ,&
&  0.66137E+00_JPRB, 0.67999E+00_JPRB, 0.59072E+00_JPRB, 0.62723E+00_JPRB, 0.81507E+00_JPRB, 0.11986E+01_JPRB, 0.11840E+01_JPRB ,&
&  0.16699E+01_JPRB, 0.22081E+01_JPRB, 0.26733E+01_JPRB, 0.37750E+01_JPRB, 0.64368E+01_JPRB, 0.65235E+01_JPRB, 0.62192E+01_JPRB ,&
&  0.57280E+01_JPRB, 0.46747E+01_JPRB, 0.37010E+01_JPRB, 0.29367E+01_JPRB, 0.23647E+01_JPRB, 0.19898E+01_JPRB, 0.17141E+01_JPRB ,&
&  0.15407E+01_JPRB, 0.13972E+01_JPRB, 0.13040E+01_JPRB, 0.12917E+01_JPRB, 0.12615E+01_JPRB, 0.12415E+01_JPRB, 0.13451E+01_JPRB ,&
&  0.15026E+01_JPRB, 0.17550E+01_JPRB, 0.23293E+01_JPRB, 0.30176E+01_JPRB, 0.34066E+01_JPRB, 0.40193E+01_JPRB, 0.52544E+01_JPRB ,&
&  0.48019E+01_JPRB, 0.48150E+01_JPRB, 0.47781E+01_JPRB, 0.59748E+01_JPRB, 0.59977E+01_JPRB, 0.62218E+01_JPRB, 0.66108E+01_JPRB ,&
&  0.47573E+01_JPRB, 0.38893E+01_JPRB, 0.38503E+01_JPRB, 0.42010E+01_JPRB, 0.34137E+01_JPRB, 0.37540E+01_JPRB, 0.50032E+01_JPRB ,&
&  0.38653E+01_JPRB, 0.37832E+01_JPRB, 0.38210E+01_JPRB, 0.40980E+01_JPRB, 0.24081E+01_JPRB, 0.16141E+01_JPRB, 0.24629E+01_JPRB ,&
&  0.23507E+01_JPRB, 0.13346E+01_JPRB, 0.95044E+00_JPRB, 0.11888E+01_JPRB, 0.91441E+00_JPRB, 0.11163E+01_JPRB, 0.17483E+01_JPRB ,&
&  0.20340E+01_JPRB, 0.23608E+01_JPRB, 0.34746E+01_JPRB, 0.30297E+01_JPRB, 0.25167E+01_JPRB, 0.21630E+01_JPRB, 0.22828E+01_JPRB ,&
&  0.22446E+01_JPRB, 0.20074E+01_JPRB, 0.20578E+01_JPRB, 0.18679E+01_JPRB, 0.17212E+01_JPRB, 0.16121E+01_JPRB, 0.14705E+01_JPRB ,&
&  0.13357E+01_JPRB, 0.11946E+01_JPRB, 0.10735E+01_JPRB, 0.99395E+00_JPRB, 0.95852E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.94968E+00_JPRB, 0.90507E+00_JPRB, 0.85584E+00_JPRB, 0.80016E+00_JPRB, 0.74055E+00_JPRB, 0.71364E+00_JPRB, 0.70562E+00_JPRB ,&
&  0.70513E+00_JPRB, 0.71727E+00_JPRB, 0.72779E+00_JPRB, 0.73322E+00_JPRB, 0.73531E+00_JPRB, 0.73219E+00_JPRB, 0.69034E+00_JPRB ,&
&  0.65848E+00_JPRB, 0.62763E+00_JPRB, 0.54185E+00_JPRB, 0.53925E+00_JPRB, 0.73616E+00_JPRB, 0.97261E+00_JPRB, 0.10336E+01_JPRB ,&
&  0.14264E+01_JPRB, 0.19744E+01_JPRB, 0.26113E+01_JPRB, 0.38054E+01_JPRB, 0.50282E+01_JPRB, 0.57515E+01_JPRB, 0.54334E+01_JPRB ,&
&  0.53892E+01_JPRB, 0.48210E+01_JPRB, 0.39861E+01_JPRB, 0.33133E+01_JPRB, 0.27023E+01_JPRB, 0.22258E+01_JPRB, 0.18727E+01_JPRB ,&
&  0.16483E+01_JPRB, 0.15001E+01_JPRB, 0.13812E+01_JPRB, 0.13185E+01_JPRB, 0.13178E+01_JPRB, 0.12860E+01_JPRB, 0.13383E+01_JPRB ,&
&  0.14220E+01_JPRB, 0.16093E+01_JPRB, 0.22680E+01_JPRB, 0.33143E+01_JPRB, 0.39141E+01_JPRB, 0.41947E+01_JPRB, 0.44942E+01_JPRB ,&
&  0.46422E+01_JPRB, 0.45954E+01_JPRB, 0.50326E+01_JPRB, 0.53492E+01_JPRB, 0.60255E+01_JPRB, 0.62578E+01_JPRB, 0.71993E+01_JPRB ,&
&  0.57565E+01_JPRB, 0.46763E+01_JPRB, 0.38384E+01_JPRB, 0.36457E+01_JPRB, 0.42282E+01_JPRB, 0.53699E+01_JPRB, 0.49596E+01_JPRB ,&
&  0.44243E+01_JPRB, 0.41192E+01_JPRB, 0.37767E+01_JPRB, 0.33843E+01_JPRB, 0.35624E+01_JPRB, 0.22190E+01_JPRB, 0.23936E+01_JPRB ,&
&  0.23909E+01_JPRB, 0.23105E+01_JPRB, 0.20557E+01_JPRB, 0.17256E+01_JPRB, 0.15223E+01_JPRB, 0.16479E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.19028E+01_JPRB, 0.20308E+01_JPRB, 0.29425E+01_JPRB, 0.30536E+01_JPRB, 0.29801E+01_JPRB, 0.25366E+01_JPRB, 0.25024E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.21779E+01_JPRB, 0.21333E+01_JPRB, 0.19792E+01_JPRB, 0.17742E+01_JPRB, 0.16040E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.13168E+01_JPRB, 0.12001E+01_JPRB, 0.10886E+01_JPRB, 0.10119E+01_JPRB, 0.98214E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.10044E+01_JPRB, 0.95342E+00_JPRB, 0.88700E+00_JPRB, 0.83482E+00_JPRB, 0.78151E+00_JPRB, 0.75004E+00_JPRB, 0.74014E+00_JPRB ,&
&  0.73587E+00_JPRB, 0.72760E+00_JPRB, 0.73493E+00_JPRB, 0.74721E+00_JPRB, 0.75135E+00_JPRB, 0.74678E+00_JPRB, 0.70131E+00_JPRB ,&
&  0.67899E+00_JPRB, 0.65606E+00_JPRB, 0.54298E+00_JPRB, 0.55284E+00_JPRB, 0.65511E+00_JPRB, 0.77235E+00_JPRB, 0.98237E+00_JPRB ,&
&  0.12028E+01_JPRB, 0.16267E+01_JPRB, 0.21435E+01_JPRB, 0.29689E+01_JPRB, 0.34995E+01_JPRB, 0.37724E+01_JPRB, 0.38414E+01_JPRB ,&
&  0.35263E+01_JPRB, 0.34697E+01_JPRB, 0.32349E+01_JPRB, 0.28850E+01_JPRB, 0.25302E+01_JPRB, 0.22582E+01_JPRB, 0.20327E+01_JPRB ,&
&  0.18364E+01_JPRB, 0.16761E+01_JPRB, 0.15648E+01_JPRB, 0.14791E+01_JPRB, 0.14486E+01_JPRB, 0.14263E+01_JPRB, 0.14303E+01_JPRB ,&
&  0.13983E+01_JPRB, 0.14304E+01_JPRB, 0.18568E+01_JPRB, 0.29843E+01_JPRB, 0.35746E+01_JPRB, 0.41881E+01_JPRB, 0.42628E+01_JPRB ,&
&  0.50675E+01_JPRB, 0.54601E+01_JPRB, 0.56856E+01_JPRB, 0.67281E+01_JPRB, 0.59873E+01_JPRB, 0.67439E+01_JPRB, 0.76387E+01_JPRB ,&
&  0.77418E+01_JPRB, 0.69570E+01_JPRB, 0.63679E+01_JPRB, 0.56674E+01_JPRB, 0.48660E+01_JPRB, 0.56969E+01_JPRB, 0.47165E+01_JPRB ,&
&  0.41705E+01_JPRB, 0.40533E+01_JPRB, 0.37273E+01_JPRB, 0.37708E+01_JPRB, 0.31288E+01_JPRB, 0.27597E+01_JPRB, 0.19118E+01_JPRB ,&
&  0.18924E+01_JPRB, 0.16574E+01_JPRB, 0.16683E+01_JPRB, 0.14771E+01_JPRB, 0.14248E+01_JPRB, 0.13081E+01_JPRB, 0.12533E+01_JPRB ,&
&  0.13795E+01_JPRB, 0.14674E+01_JPRB, 0.18657E+01_JPRB, 0.20585E+01_JPRB, 0.26471E+01_JPRB, 0.24363E+01_JPRB, 0.24684E+01_JPRB ,&
&  0.22446E+01_JPRB, 0.21433E+01_JPRB, 0.18384E+01_JPRB, 0.17914E+01_JPRB, 0.16217E+01_JPRB, 0.14767E+01_JPRB, 0.13639E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.11586E+01_JPRB, 0.10974E+01_JPRB, 0.10407E+01_JPRB, 0.10285E+01_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.89792E+00_JPRB, 0.85894E+00_JPRB, 0.83860E+00_JPRB, 0.82680E+00_JPRB, 0.78952E+00_JPRB, 0.76352E+00_JPRB, 0.77076E+00_JPRB ,&
&  0.76899E+00_JPRB, 0.74217E+00_JPRB, 0.72472E+00_JPRB, 0.72311E+00_JPRB, 0.72686E+00_JPRB, 0.73284E+00_JPRB, 0.71704E+00_JPRB ,&
&  0.70107E+00_JPRB, 0.81112E+00_JPRB, 0.63837E+00_JPRB, 0.53067E+00_JPRB, 0.50194E+00_JPRB, 0.61403E+00_JPRB, 0.79012E+00_JPRB ,&
&  0.10225E+01_JPRB, 0.12627E+01_JPRB, 0.16545E+01_JPRB, 0.19504E+01_JPRB, 0.21984E+01_JPRB, 0.23652E+01_JPRB, 0.25131E+01_JPRB ,&
&  0.27319E+01_JPRB, 0.21084E+01_JPRB, 0.20204E+01_JPRB, 0.19496E+01_JPRB, 0.18873E+01_JPRB, 0.18116E+01_JPRB, 0.17807E+01_JPRB ,&
&  0.17139E+01_JPRB, 0.16624E+01_JPRB, 0.16094E+01_JPRB, 0.15537E+01_JPRB, 0.14981E+01_JPRB, 0.14463E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.13946E+01_JPRB, 0.13848E+01_JPRB, 0.15039E+01_JPRB, 0.20845E+01_JPRB, 0.32055E+01_JPRB, 0.40552E+01_JPRB, 0.47216E+01_JPRB ,&
&  0.45087E+01_JPRB, 0.45197E+01_JPRB, 0.70446E+01_JPRB, 0.76706E+01_JPRB, 0.82246E+01_JPRB, 0.88181E+01_JPRB, 0.86563E+01_JPRB ,&
&  0.84261E+01_JPRB, 0.74872E+01_JPRB, 0.65769E+01_JPRB, 0.57057E+01_JPRB, 0.61226E+01_JPRB, 0.56155E+01_JPRB, 0.48957E+01_JPRB ,&
&  0.44625E+01_JPRB, 0.44923E+01_JPRB, 0.41351E+01_JPRB, 0.37167E+01_JPRB, 0.34051E+01_JPRB, 0.34173E+01_JPRB, 0.26970E+01_JPRB ,&
&  0.20747E+01_JPRB, 0.19251E+01_JPRB, 0.13920E+01_JPRB, 0.10909E+01_JPRB, 0.94899E+00_JPRB, 0.88255E+00_JPRB, 0.89782E+00_JPRB ,&
&  0.95768E+00_JPRB, 0.10874E+01_JPRB, 0.12902E+01_JPRB, 0.14683E+01_JPRB, 0.19044E+01_JPRB, 0.21408E+01_JPRB, 0.21299E+01_JPRB ,&
&  0.17624E+01_JPRB, 0.17308E+01_JPRB, 0.16518E+01_JPRB, 0.15655E+01_JPRB, 0.14978E+01_JPRB, 0.13427E+01_JPRB, 0.12667E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11193E+01_JPRB, 0.10849E+01_JPRB, 0.10107E+01_JPRB, 0.95651E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.82596E+00_JPRB, 0.81042E+00_JPRB, 0.81375E+00_JPRB, 0.81473E+00_JPRB, 0.80636E+00_JPRB, 0.78622E+00_JPRB, 0.78376E+00_JPRB ,&
&  0.76581E+00_JPRB, 0.74359E+00_JPRB, 0.72424E+00_JPRB, 0.71581E+00_JPRB, 0.71274E+00_JPRB, 0.71235E+00_JPRB, 0.71887E+00_JPRB ,&
&  0.73284E+00_JPRB, 0.97849E+00_JPRB, 0.78702E+00_JPRB, 0.67798E+00_JPRB, 0.58276E+00_JPRB, 0.65205E+00_JPRB, 0.75844E+00_JPRB ,&
&  0.92436E+00_JPRB, 0.99974E+00_JPRB, 0.11381E+01_JPRB, 0.13188E+01_JPRB, 0.15018E+01_JPRB, 0.14818E+01_JPRB, 0.15948E+01_JPRB ,&
&  0.15109E+01_JPRB, 0.14152E+01_JPRB, 0.11936E+01_JPRB, 0.13093E+01_JPRB, 0.12812E+01_JPRB, 0.11556E+01_JPRB, 0.13654E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.14693E+01_JPRB, 0.15051E+01_JPRB, 0.15265E+01_JPRB, 0.15348E+01_JPRB, 0.15006E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.13815E+01_JPRB, 0.14806E+01_JPRB, 0.16510E+01_JPRB, 0.21386E+01_JPRB, 0.28446E+01_JPRB, 0.41571E+01_JPRB ,&
&  0.52416E+01_JPRB, 0.50035E+01_JPRB, 0.49710E+01_JPRB, 0.66970E+01_JPRB, 0.84463E+01_JPRB, 0.80401E+01_JPRB, 0.74905E+01_JPRB ,&
&  0.77389E+01_JPRB, 0.76445E+01_JPRB, 0.74290E+01_JPRB, 0.69879E+01_JPRB, 0.65759E+01_JPRB, 0.60701E+01_JPRB, 0.54983E+01_JPRB ,&
&  0.50089E+01_JPRB, 0.48695E+01_JPRB, 0.47720E+01_JPRB, 0.42839E+01_JPRB, 0.34599E+01_JPRB, 0.30466E+01_JPRB, 0.27210E+01_JPRB ,&
&  0.24063E+01_JPRB, 0.14597E+01_JPRB, 0.85072E+00_JPRB, 0.65178E+00_JPRB, 0.58335E+00_JPRB, 0.56842E+00_JPRB, 0.65622E+00_JPRB ,&
&  0.65734E+00_JPRB, 0.80220E+00_JPRB, 0.10523E+01_JPRB, 0.11459E+01_JPRB, 0.15769E+01_JPRB, 0.17909E+01_JPRB, 0.16542E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.11787E+01_JPRB, 0.14677E+01_JPRB, 0.14513E+01_JPRB, 0.13734E+01_JPRB, 0.12532E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.11426E+01_JPRB, 0.10963E+01_JPRB, 0.10376E+01_JPRB, 0.93358E+00_JPRB, 0.86034E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.79186E+00_JPRB, 0.78554E+00_JPRB, 0.78599E+00_JPRB, 0.78459E+00_JPRB, 0.78307E+00_JPRB, 0.75696E+00_JPRB, 0.75410E+00_JPRB ,&
&  0.74493E+00_JPRB, 0.73207E+00_JPRB, 0.71795E+00_JPRB, 0.71352E+00_JPRB, 0.71105E+00_JPRB, 0.70994E+00_JPRB, 0.72527E+00_JPRB ,&
&  0.82734E+00_JPRB, 0.10721E+01_JPRB, 0.94109E+00_JPRB, 0.73697E+00_JPRB, 0.84641E+00_JPRB, 0.82544E+00_JPRB, 0.88581E+00_JPRB ,&
&  0.88483E+00_JPRB, 0.94840E+00_JPRB, 0.92927E+00_JPRB, 0.90664E+00_JPRB, 0.96526E+00_JPRB, 0.10435E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.99893E+00_JPRB, 0.89513E+00_JPRB, 0.83619E+00_JPRB, 0.84216E+00_JPRB, 0.93533E+00_JPRB, 0.87645E+00_JPRB, 0.10065E+01_JPRB ,&
&  0.10770E+01_JPRB, 0.11425E+01_JPRB, 0.12092E+01_JPRB, 0.12872E+01_JPRB, 0.13490E+01_JPRB, 0.14042E+01_JPRB, 0.14285E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.14639E+01_JPRB, 0.15007E+01_JPRB, 0.16271E+01_JPRB, 0.18938E+01_JPRB, 0.26732E+01_JPRB, 0.43770E+01_JPRB ,&
&  0.50436E+01_JPRB, 0.50290E+01_JPRB, 0.59626E+01_JPRB, 0.70803E+01_JPRB, 0.68688E+01_JPRB, 0.65321E+01_JPRB, 0.64506E+01_JPRB ,&
&  0.66032E+01_JPRB, 0.67941E+01_JPRB, 0.71444E+01_JPRB, 0.72409E+01_JPRB, 0.71546E+01_JPRB, 0.65838E+01_JPRB, 0.59714E+01_JPRB ,&
&  0.54070E+01_JPRB, 0.52042E+01_JPRB, 0.50854E+01_JPRB, 0.43967E+01_JPRB, 0.35408E+01_JPRB, 0.28920E+01_JPRB, 0.25714E+01_JPRB ,&
&  0.25008E+01_JPRB, 0.11568E+01_JPRB, 0.73446E+00_JPRB, 0.58467E+00_JPRB, 0.53248E+00_JPRB, 0.56288E+00_JPRB, 0.72222E+00_JPRB ,&
&  0.63161E+00_JPRB, 0.74457E+00_JPRB, 0.96313E+00_JPRB, 0.10305E+01_JPRB, 0.11318E+01_JPRB, 0.12688E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.87249E+00_JPRB, 0.95201E+00_JPRB, 0.12532E+01_JPRB, 0.12968E+01_JPRB, 0.12281E+01_JPRB, 0.11913E+01_JPRB, 0.11874E+01_JPRB ,&
&  0.11086E+01_JPRB, 0.10546E+01_JPRB, 0.95236E+00_JPRB, 0.85794E+00_JPRB, 0.81662E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.80834E+00_JPRB, 0.78464E+00_JPRB, 0.77281E+00_JPRB, 0.76464E+00_JPRB, 0.74960E+00_JPRB, 0.72597E+00_JPRB, 0.72698E+00_JPRB ,&
&  0.71994E+00_JPRB, 0.70670E+00_JPRB, 0.70168E+00_JPRB, 0.69869E+00_JPRB, 0.69112E+00_JPRB, 0.70142E+00_JPRB, 0.74748E+00_JPRB ,&
&  0.67833E+00_JPRB, 0.79699E+00_JPRB, 0.69392E+00_JPRB, 0.73091E+00_JPRB, 0.10349E+01_JPRB, 0.92402E+00_JPRB, 0.91160E+00_JPRB ,&
&  0.85661E+00_JPRB, 0.82967E+00_JPRB, 0.76412E+00_JPRB, 0.74225E+00_JPRB, 0.74685E+00_JPRB, 0.78957E+00_JPRB, 0.81052E+00_JPRB ,&
&  0.72900E+00_JPRB, 0.64932E+00_JPRB, 0.62835E+00_JPRB, 0.63382E+00_JPRB, 0.68847E+00_JPRB, 0.77711E+00_JPRB, 0.89386E+00_JPRB ,&
&  0.93494E+00_JPRB, 0.96848E+00_JPRB, 0.10238E+01_JPRB, 0.10863E+01_JPRB, 0.11471E+01_JPRB, 0.12121E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.13393E+01_JPRB, 0.14266E+01_JPRB, 0.15936E+01_JPRB, 0.17220E+01_JPRB, 0.26477E+01_JPRB, 0.47098E+01_JPRB ,&
&  0.54456E+01_JPRB, 0.47070E+01_JPRB, 0.49284E+01_JPRB, 0.60650E+01_JPRB, 0.60777E+01_JPRB, 0.57107E+01_JPRB, 0.53892E+01_JPRB ,&
&  0.53937E+01_JPRB, 0.54817E+01_JPRB, 0.57184E+01_JPRB, 0.61046E+01_JPRB, 0.63517E+01_JPRB, 0.65133E+01_JPRB, 0.62329E+01_JPRB ,&
&  0.58034E+01_JPRB, 0.55286E+01_JPRB, 0.52678E+01_JPRB, 0.48220E+01_JPRB, 0.39198E+01_JPRB, 0.33237E+01_JPRB, 0.25574E+01_JPRB ,&
&  0.20965E+01_JPRB, 0.15402E+01_JPRB, 0.98544E+00_JPRB, 0.77962E+00_JPRB, 0.68098E+00_JPRB, 0.75964E+00_JPRB, 0.10400E+01_JPRB ,&
&  0.75578E+00_JPRB, 0.78685E+00_JPRB, 0.79219E+00_JPRB, 0.81300E+00_JPRB, 0.82823E+00_JPRB, 0.80489E+00_JPRB, 0.69914E+00_JPRB ,&
&  0.70449E+00_JPRB, 0.87211E+00_JPRB, 0.10902E+01_JPRB, 0.11514E+01_JPRB, 0.10980E+01_JPRB, 0.10762E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.97592E+00_JPRB, 0.90211E+00_JPRB, 0.85579E+00_JPRB, 0.84275E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.78851E+00_JPRB, 0.77488E+00_JPRB, 0.76059E+00_JPRB, 0.74064E+00_JPRB, 0.71896E+00_JPRB, 0.69840E+00_JPRB, 0.68526E+00_JPRB ,&
&  0.68948E+00_JPRB, 0.68662E+00_JPRB, 0.68518E+00_JPRB, 0.68001E+00_JPRB, 0.66762E+00_JPRB, 0.68496E+00_JPRB, 0.63151E+00_JPRB ,&
&  0.58496E+00_JPRB, 0.63235E+00_JPRB, 0.62939E+00_JPRB, 0.68368E+00_JPRB, 0.75548E+00_JPRB, 0.73052E+00_JPRB, 0.71959E+00_JPRB ,&
&  0.71135E+00_JPRB, 0.67670E+00_JPRB, 0.66895E+00_JPRB, 0.67716E+00_JPRB, 0.68142E+00_JPRB, 0.70249E+00_JPRB, 0.74037E+00_JPRB ,&
&  0.68812E+00_JPRB, 0.63395E+00_JPRB, 0.60398E+00_JPRB, 0.61480E+00_JPRB, 0.69224E+00_JPRB, 0.79968E+00_JPRB, 0.86126E+00_JPRB ,&
&  0.87991E+00_JPRB, 0.89809E+00_JPRB, 0.95060E+00_JPRB, 0.98602E+00_JPRB, 0.10125E+01_JPRB, 0.10464E+01_JPRB, 0.10817E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.12019E+01_JPRB, 0.13028E+01_JPRB, 0.14619E+01_JPRB, 0.16003E+01_JPRB, 0.21775E+01_JPRB, 0.30659E+01_JPRB ,&
&  0.33439E+01_JPRB, 0.33390E+01_JPRB, 0.33476E+01_JPRB, 0.35931E+01_JPRB, 0.39611E+01_JPRB, 0.38968E+01_JPRB, 0.38500E+01_JPRB ,&
&  0.41161E+01_JPRB, 0.41223E+01_JPRB, 0.50229E+01_JPRB, 0.54462E+01_JPRB, 0.56522E+01_JPRB, 0.60200E+01_JPRB, 0.59243E+01_JPRB ,&
&  0.58224E+01_JPRB, 0.55684E+01_JPRB, 0.54265E+01_JPRB, 0.49199E+01_JPRB, 0.41053E+01_JPRB, 0.35525E+01_JPRB, 0.27071E+01_JPRB ,&
&  0.21610E+01_JPRB, 0.16863E+01_JPRB, 0.12279E+01_JPRB, 0.10323E+01_JPRB, 0.84811E+00_JPRB, 0.82073E+00_JPRB, 0.76318E+00_JPRB ,&
&  0.70322E+00_JPRB, 0.67814E+00_JPRB, 0.65688E+00_JPRB, 0.67453E+00_JPRB, 0.66292E+00_JPRB, 0.61545E+00_JPRB, 0.65335E+00_JPRB ,&
&  0.73660E+00_JPRB, 0.98586E+00_JPRB, 0.99933E+00_JPRB, 0.10451E+01_JPRB, 0.10050E+01_JPRB, 0.97682E+00_JPRB, 0.79415E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.92017E+00_JPRB, 0.86251E+00_JPRB, 0.82639E+00_JPRB, 0.80395E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.75431E+00_JPRB, 0.75377E+00_JPRB, 0.73185E+00_JPRB, 0.71372E+00_JPRB, 0.70184E+00_JPRB, 0.66668E+00_JPRB, 0.63344E+00_JPRB ,&
&  0.68665E+00_JPRB, 0.67654E+00_JPRB, 0.67751E+00_JPRB, 0.67450E+00_JPRB, 0.66508E+00_JPRB, 0.60215E+00_JPRB, 0.55492E+00_JPRB ,&
&  0.52340E+00_JPRB, 0.52718E+00_JPRB, 0.56899E+00_JPRB, 0.58909E+00_JPRB, 0.59636E+00_JPRB, 0.58218E+00_JPRB, 0.57758E+00_JPRB ,&
&  0.57355E+00_JPRB, 0.58263E+00_JPRB, 0.62202E+00_JPRB, 0.65882E+00_JPRB, 0.66391E+00_JPRB, 0.67274E+00_JPRB, 0.71074E+00_JPRB ,&
&  0.67888E+00_JPRB, 0.67854E+00_JPRB, 0.68392E+00_JPRB, 0.63124E+00_JPRB, 0.74045E+00_JPRB, 0.78119E+00_JPRB, 0.80477E+00_JPRB ,&
&  0.84881E+00_JPRB, 0.88828E+00_JPRB, 0.91124E+00_JPRB, 0.92459E+00_JPRB, 0.95483E+00_JPRB, 0.97272E+00_JPRB, 0.10035E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10983E+01_JPRB, 0.11622E+01_JPRB, 0.12524E+01_JPRB, 0.14503E+01_JPRB, 0.16927E+01_JPRB, 0.20295E+01_JPRB ,&
&  0.22364E+01_JPRB, 0.23246E+01_JPRB, 0.24749E+01_JPRB, 0.23695E+01_JPRB, 0.31158E+01_JPRB, 0.35389E+01_JPRB, 0.33644E+01_JPRB ,&
&  0.35630E+01_JPRB, 0.34213E+01_JPRB, 0.39663E+01_JPRB, 0.44473E+01_JPRB, 0.49112E+01_JPRB, 0.53149E+01_JPRB, 0.54389E+01_JPRB ,&
&  0.53085E+01_JPRB, 0.54588E+01_JPRB, 0.52984E+01_JPRB, 0.47081E+01_JPRB, 0.39257E+01_JPRB, 0.33823E+01_JPRB, 0.26719E+01_JPRB ,&
&  0.20971E+01_JPRB, 0.16436E+01_JPRB, 0.13652E+01_JPRB, 0.12216E+01_JPRB, 0.97649E+00_JPRB, 0.89652E+00_JPRB, 0.81771E+00_JPRB ,&
&  0.74559E+00_JPRB, 0.68790E+00_JPRB, 0.66013E+00_JPRB, 0.63329E+00_JPRB, 0.61659E+00_JPRB, 0.61343E+00_JPRB, 0.61101E+00_JPRB ,&
&  0.66044E+00_JPRB, 0.76643E+00_JPRB, 0.97157E+00_JPRB, 0.95239E+00_JPRB, 0.91620E+00_JPRB, 0.88700E+00_JPRB, 0.79012E+00_JPRB ,&
&  0.74646E+00_JPRB, 0.85133E+00_JPRB, 0.82638E+00_JPRB, 0.78536E+00_JPRB, 0.75013E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.71543E+00_JPRB, 0.71800E+00_JPRB, 0.70514E+00_JPRB, 0.69258E+00_JPRB, 0.67190E+00_JPRB, 0.60548E+00_JPRB, 0.58854E+00_JPRB ,&
&  0.65430E+00_JPRB, 0.59945E+00_JPRB, 0.65103E+00_JPRB, 0.54689E+00_JPRB, 0.52223E+00_JPRB, 0.49664E+00_JPRB, 0.47600E+00_JPRB ,&
&  0.45215E+00_JPRB, 0.45882E+00_JPRB, 0.48501E+00_JPRB, 0.50708E+00_JPRB, 0.51647E+00_JPRB, 0.50908E+00_JPRB, 0.51581E+00_JPRB ,&
&  0.53188E+00_JPRB, 0.56438E+00_JPRB, 0.61230E+00_JPRB, 0.63784E+00_JPRB, 0.65704E+00_JPRB, 0.68119E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.65343E+00_JPRB, 0.64169E+00_JPRB, 0.63860E+00_JPRB, 0.65160E+00_JPRB, 0.69030E+00_JPRB, 0.75184E+00_JPRB, 0.78875E+00_JPRB ,&
&  0.80116E+00_JPRB, 0.67983E+00_JPRB, 0.79753E+00_JPRB, 0.83178E+00_JPRB, 0.89906E+00_JPRB, 0.93879E+00_JPRB, 0.96844E+00_JPRB ,&
&  0.10095E+01_JPRB, 0.10527E+01_JPRB, 0.11003E+01_JPRB, 0.11564E+01_JPRB, 0.12266E+01_JPRB, 0.13670E+01_JPRB, 0.14778E+01_JPRB ,&
&  0.17077E+01_JPRB, 0.12308E+01_JPRB, 0.13832E+01_JPRB, 0.15800E+01_JPRB, 0.22286E+01_JPRB, 0.26389E+01_JPRB, 0.27672E+01_JPRB ,&
&  0.32702E+01_JPRB, 0.31787E+01_JPRB, 0.33882E+01_JPRB, 0.35419E+01_JPRB, 0.38172E+01_JPRB, 0.41982E+01_JPRB, 0.44308E+01_JPRB ,&
&  0.44976E+01_JPRB, 0.45763E+01_JPRB, 0.47697E+01_JPRB, 0.43098E+01_JPRB, 0.37187E+01_JPRB, 0.31609E+01_JPRB, 0.25020E+01_JPRB ,&
&  0.20698E+01_JPRB, 0.16894E+01_JPRB, 0.13667E+01_JPRB, 0.11354E+01_JPRB, 0.97516E+00_JPRB, 0.87761E+00_JPRB, 0.82406E+00_JPRB ,&
&  0.80314E+00_JPRB, 0.76829E+00_JPRB, 0.73132E+00_JPRB, 0.68351E+00_JPRB, 0.65569E+00_JPRB, 0.64252E+00_JPRB, 0.65491E+00_JPRB ,&
&  0.64377E+00_JPRB, 0.63204E+00_JPRB, 0.66541E+00_JPRB, 0.72905E+00_JPRB, 0.73261E+00_JPRB, 0.74290E+00_JPRB, 0.82188E+00_JPRB ,&
&  0.73579E+00_JPRB, 0.75569E+00_JPRB, 0.80522E+00_JPRB, 0.77612E+00_JPRB, 0.72953E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.71728E+00_JPRB, 0.69164E+00_JPRB, 0.67686E+00_JPRB, 0.66185E+00_JPRB, 0.63971E+00_JPRB, 0.58866E+00_JPRB, 0.55272E+00_JPRB ,&
&  0.56022E+00_JPRB, 0.54045E+00_JPRB, 0.52789E+00_JPRB, 0.49763E+00_JPRB, 0.47439E+00_JPRB, 0.45024E+00_JPRB, 0.43524E+00_JPRB ,&
&  0.42529E+00_JPRB, 0.44456E+00_JPRB, 0.48049E+00_JPRB, 0.49099E+00_JPRB, 0.49316E+00_JPRB, 0.49712E+00_JPRB, 0.51623E+00_JPRB ,&
&  0.54212E+00_JPRB, 0.57051E+00_JPRB, 0.58904E+00_JPRB, 0.60837E+00_JPRB, 0.62637E+00_JPRB, 0.64344E+00_JPRB, 0.63187E+00_JPRB ,&
&  0.61555E+00_JPRB, 0.58521E+00_JPRB, 0.55798E+00_JPRB, 0.58559E+00_JPRB, 0.64746E+00_JPRB, 0.70494E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.50160E+00_JPRB, 0.50546E+00_JPRB, 0.68880E+00_JPRB, 0.75158E+00_JPRB, 0.81359E+00_JPRB, 0.85229E+00_JPRB, 0.90299E+00_JPRB ,&
&  0.95160E+00_JPRB, 0.99710E+00_JPRB, 0.10354E+01_JPRB, 0.10762E+01_JPRB, 0.11318E+01_JPRB, 0.12096E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.11327E+01_JPRB, 0.11024E+01_JPRB, 0.12612E+01_JPRB, 0.18111E+01_JPRB, 0.20609E+01_JPRB, 0.21036E+01_JPRB ,&
&  0.23149E+01_JPRB, 0.24301E+01_JPRB, 0.26385E+01_JPRB, 0.28672E+01_JPRB, 0.29531E+01_JPRB, 0.31421E+01_JPRB, 0.34208E+01_JPRB ,&
&  0.38271E+01_JPRB, 0.42058E+01_JPRB, 0.42122E+01_JPRB, 0.40282E+01_JPRB, 0.36942E+01_JPRB, 0.33233E+01_JPRB, 0.27774E+01_JPRB ,&
&  0.21576E+01_JPRB, 0.17193E+01_JPRB, 0.13591E+01_JPRB, 0.11158E+01_JPRB, 0.98231E+00_JPRB, 0.92466E+00_JPRB, 0.88182E+00_JPRB ,&
&  0.88211E+00_JPRB, 0.87150E+00_JPRB, 0.83971E+00_JPRB, 0.76759E+00_JPRB, 0.71096E+00_JPRB, 0.68865E+00_JPRB, 0.67324E+00_JPRB ,&
&  0.64614E+00_JPRB, 0.61835E+00_JPRB, 0.56242E+00_JPRB, 0.56578E+00_JPRB, 0.61925E+00_JPRB, 0.67879E+00_JPRB, 0.70208E+00_JPRB ,&
&  0.71559E+00_JPRB, 0.70117E+00_JPRB, 0.70407E+00_JPRB, 0.72678E+00_JPRB, 0.72472E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.71630E+00_JPRB, 0.65881E+00_JPRB, 0.62515E+00_JPRB, 0.62316E+00_JPRB, 0.57821E+00_JPRB, 0.53957E+00_JPRB, 0.51655E+00_JPRB ,&
&  0.51638E+00_JPRB, 0.51605E+00_JPRB, 0.51806E+00_JPRB, 0.49599E+00_JPRB, 0.47655E+00_JPRB, 0.46610E+00_JPRB, 0.46359E+00_JPRB ,&
&  0.47754E+00_JPRB, 0.49477E+00_JPRB, 0.50902E+00_JPRB, 0.50765E+00_JPRB, 0.50824E+00_JPRB, 0.51745E+00_JPRB, 0.53764E+00_JPRB ,&
&  0.56525E+00_JPRB, 0.58098E+00_JPRB, 0.57928E+00_JPRB, 0.57724E+00_JPRB, 0.59429E+00_JPRB, 0.61081E+00_JPRB, 0.57482E+00_JPRB ,&
&  0.53720E+00_JPRB, 0.50726E+00_JPRB, 0.49854E+00_JPRB, 0.54010E+00_JPRB, 0.60740E+00_JPRB, 0.63389E+00_JPRB, 0.51078E+00_JPRB ,&
&  0.42480E+00_JPRB, 0.40901E+00_JPRB, 0.45229E+00_JPRB, 0.59273E+00_JPRB, 0.64503E+00_JPRB, 0.69810E+00_JPRB, 0.77317E+00_JPRB ,&
&  0.72493E+00_JPRB, 0.72601E+00_JPRB, 0.75631E+00_JPRB, 0.91560E+00_JPRB, 0.97866E+00_JPRB, 0.10297E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.11040E+01_JPRB, 0.11931E+01_JPRB, 0.13959E+01_JPRB, 0.12498E+01_JPRB, 0.14195E+01_JPRB, 0.17761E+01_JPRB, 0.18056E+01_JPRB ,&
&  0.20399E+01_JPRB, 0.22331E+01_JPRB, 0.26508E+01_JPRB, 0.26788E+01_JPRB, 0.25375E+01_JPRB, 0.25459E+01_JPRB, 0.27893E+01_JPRB ,&
&  0.33325E+01_JPRB, 0.37469E+01_JPRB, 0.36174E+01_JPRB, 0.34164E+01_JPRB, 0.32445E+01_JPRB, 0.30298E+01_JPRB, 0.26327E+01_JPRB ,&
&  0.21098E+01_JPRB, 0.17159E+01_JPRB, 0.14245E+01_JPRB, 0.11820E+01_JPRB, 0.10746E+01_JPRB, 0.10362E+01_JPRB, 0.98704E+00_JPRB ,&
&  0.97301E+00_JPRB, 0.95605E+00_JPRB, 0.93166E+00_JPRB, 0.84195E+00_JPRB, 0.75665E+00_JPRB, 0.67646E+00_JPRB, 0.61746E+00_JPRB ,&
&  0.58324E+00_JPRB, 0.54813E+00_JPRB, 0.52745E+00_JPRB, 0.54120E+00_JPRB, 0.60004E+00_JPRB, 0.66971E+00_JPRB, 0.70262E+00_JPRB ,&
&  0.68913E+00_JPRB, 0.66899E+00_JPRB, 0.69365E+00_JPRB, 0.73736E+00_JPRB, 0.75065E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.77155E+00_JPRB, 0.75650E+00_JPRB, 0.72673E+00_JPRB, 0.68309E+00_JPRB, 0.64342E+00_JPRB, 0.59861E+00_JPRB, 0.56527E+00_JPRB ,&
&  0.54736E+00_JPRB, 0.53860E+00_JPRB, 0.53391E+00_JPRB, 0.53350E+00_JPRB, 0.53596E+00_JPRB, 0.54300E+00_JPRB, 0.54497E+00_JPRB ,&
&  0.54910E+00_JPRB, 0.55319E+00_JPRB, 0.56033E+00_JPRB, 0.57183E+00_JPRB, 0.58399E+00_JPRB, 0.59724E+00_JPRB, 0.61078E+00_JPRB ,&
&  0.62269E+00_JPRB, 0.61505E+00_JPRB, 0.58854E+00_JPRB, 0.56939E+00_JPRB, 0.56770E+00_JPRB, 0.57592E+00_JPRB, 0.55212E+00_JPRB ,&
&  0.50641E+00_JPRB, 0.47790E+00_JPRB, 0.47196E+00_JPRB, 0.53007E+00_JPRB, 0.56657E+00_JPRB, 0.58031E+00_JPRB, 0.46099E+00_JPRB ,&
&  0.39561E+00_JPRB, 0.36722E+00_JPRB, 0.36700E+00_JPRB, 0.39699E+00_JPRB, 0.45933E+00_JPRB, 0.54807E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.67319E+00_JPRB, 0.72255E+00_JPRB, 0.74176E+00_JPRB, 0.77292E+00_JPRB, 0.82336E+00_JPRB, 0.87661E+00_JPRB, 0.93496E+00_JPRB ,&
&  0.98401E+00_JPRB, 0.10607E+01_JPRB, 0.12232E+01_JPRB, 0.11364E+01_JPRB, 0.12109E+01_JPRB, 0.13218E+01_JPRB, 0.15183E+01_JPRB ,&
&  0.19213E+01_JPRB, 0.21448E+01_JPRB, 0.23083E+01_JPRB, 0.25987E+01_JPRB, 0.25607E+01_JPRB, 0.25506E+01_JPRB, 0.26784E+01_JPRB ,&
&  0.31063E+01_JPRB, 0.34630E+01_JPRB, 0.33540E+01_JPRB, 0.30055E+01_JPRB, 0.28664E+01_JPRB, 0.27769E+01_JPRB, 0.24259E+01_JPRB ,&
&  0.19884E+01_JPRB, 0.17346E+01_JPRB, 0.15304E+01_JPRB, 0.13011E+01_JPRB, 0.12083E+01_JPRB, 0.11560E+01_JPRB, 0.11205E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.10580E+01_JPRB, 0.10286E+01_JPRB, 0.94118E+00_JPRB, 0.85581E+00_JPRB, 0.76386E+00_JPRB, 0.69745E+00_JPRB ,&
&  0.65315E+00_JPRB, 0.63425E+00_JPRB, 0.63437E+00_JPRB, 0.65071E+00_JPRB, 0.69195E+00_JPRB, 0.72391E+00_JPRB, 0.74136E+00_JPRB ,&
&  0.74152E+00_JPRB, 0.71979E+00_JPRB, 0.71306E+00_JPRB, 0.72155E+00_JPRB, 0.75788E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.84468E+00_JPRB, 0.84707E+00_JPRB, 0.82668E+00_JPRB, 0.79107E+00_JPRB, 0.74885E+00_JPRB, 0.70137E+00_JPRB, 0.67096E+00_JPRB ,&
&  0.65484E+00_JPRB, 0.64965E+00_JPRB, 0.64336E+00_JPRB, 0.63603E+00_JPRB, 0.63267E+00_JPRB, 0.63192E+00_JPRB, 0.63363E+00_JPRB ,&
&  0.63845E+00_JPRB, 0.64543E+00_JPRB, 0.65121E+00_JPRB, 0.64659E+00_JPRB, 0.65092E+00_JPRB, 0.65677E+00_JPRB, 0.66343E+00_JPRB ,&
&  0.65891E+00_JPRB, 0.63151E+00_JPRB, 0.59632E+00_JPRB, 0.57300E+00_JPRB, 0.54604E+00_JPRB, 0.53246E+00_JPRB, 0.51417E+00_JPRB ,&
&  0.48463E+00_JPRB, 0.47484E+00_JPRB, 0.49645E+00_JPRB, 0.52349E+00_JPRB, 0.51619E+00_JPRB, 0.53346E+00_JPRB, 0.41394E+00_JPRB ,&
&  0.37215E+00_JPRB, 0.34928E+00_JPRB, 0.34384E+00_JPRB, 0.35468E+00_JPRB, 0.38582E+00_JPRB, 0.42831E+00_JPRB, 0.47997E+00_JPRB ,&
&  0.55981E+00_JPRB, 0.64112E+00_JPRB, 0.69365E+00_JPRB, 0.73715E+00_JPRB, 0.78261E+00_JPRB, 0.82531E+00_JPRB, 0.85653E+00_JPRB ,&
&  0.88523E+00_JPRB, 0.94106E+00_JPRB, 0.10265E+01_JPRB, 0.11465E+01_JPRB, 0.13781E+01_JPRB, 0.13937E+01_JPRB, 0.14982E+01_JPRB ,&
&  0.20571E+01_JPRB, 0.23914E+01_JPRB, 0.23614E+01_JPRB, 0.23482E+01_JPRB, 0.24054E+01_JPRB, 0.25732E+01_JPRB, 0.27929E+01_JPRB ,&
&  0.30905E+01_JPRB, 0.33105E+01_JPRB, 0.32658E+01_JPRB, 0.29089E+01_JPRB, 0.27266E+01_JPRB, 0.26176E+01_JPRB, 0.23307E+01_JPRB ,&
&  0.20108E+01_JPRB, 0.20190E+01_JPRB, 0.21464E+01_JPRB, 0.17598E+01_JPRB, 0.15012E+01_JPRB, 0.13496E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.12091E+01_JPRB, 0.11542E+01_JPRB, 0.10965E+01_JPRB, 0.10440E+01_JPRB, 0.98674E+00_JPRB, 0.93204E+00_JPRB, 0.88368E+00_JPRB ,&
&  0.84624E+00_JPRB, 0.81467E+00_JPRB, 0.79459E+00_JPRB, 0.77949E+00_JPRB, 0.76104E+00_JPRB, 0.74709E+00_JPRB, 0.74613E+00_JPRB ,&
&  0.75418E+00_JPRB, 0.75742E+00_JPRB, 0.77215E+00_JPRB, 0.79263E+00_JPRB, 0.82062E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.85833E+00_JPRB, 0.86533E+00_JPRB, 0.86553E+00_JPRB, 0.85481E+00_JPRB, 0.83291E+00_JPRB, 0.80813E+00_JPRB, 0.78701E+00_JPRB ,&
&  0.77004E+00_JPRB, 0.75464E+00_JPRB, 0.73933E+00_JPRB, 0.72735E+00_JPRB, 0.71629E+00_JPRB, 0.70888E+00_JPRB, 0.70335E+00_JPRB ,&
&  0.70116E+00_JPRB, 0.68118E+00_JPRB, 0.67565E+00_JPRB, 0.67788E+00_JPRB, 0.68214E+00_JPRB, 0.68060E+00_JPRB, 0.67561E+00_JPRB ,&
&  0.65104E+00_JPRB, 0.61674E+00_JPRB, 0.58489E+00_JPRB, 0.55890E+00_JPRB, 0.53664E+00_JPRB, 0.52162E+00_JPRB, 0.52147E+00_JPRB ,&
&  0.52339E+00_JPRB, 0.51445E+00_JPRB, 0.51156E+00_JPRB, 0.52671E+00_JPRB, 0.53148E+00_JPRB, 0.46586E+00_JPRB, 0.37821E+00_JPRB ,&
&  0.36227E+00_JPRB, 0.34494E+00_JPRB, 0.34463E+00_JPRB, 0.35046E+00_JPRB, 0.36675E+00_JPRB, 0.39613E+00_JPRB, 0.44536E+00_JPRB ,&
&  0.51540E+00_JPRB, 0.65784E+00_JPRB, 0.74089E+00_JPRB, 0.79817E+00_JPRB, 0.84714E+00_JPRB, 0.88438E+00_JPRB, 0.89903E+00_JPRB ,&
&  0.91428E+00_JPRB, 0.94659E+00_JPRB, 0.10023E+01_JPRB, 0.10717E+01_JPRB, 0.11788E+01_JPRB, 0.13819E+01_JPRB, 0.15845E+01_JPRB ,&
&  0.18640E+01_JPRB, 0.20658E+01_JPRB, 0.21436E+01_JPRB, 0.21676E+01_JPRB, 0.23049E+01_JPRB, 0.25027E+01_JPRB, 0.25981E+01_JPRB ,&
&  0.27490E+01_JPRB, 0.29024E+01_JPRB, 0.28547E+01_JPRB, 0.26866E+01_JPRB, 0.25802E+01_JPRB, 0.25096E+01_JPRB, 0.23004E+01_JPRB ,&
&  0.20694E+01_JPRB, 0.19769E+01_JPRB, 0.19907E+01_JPRB, 0.17531E+01_JPRB, 0.15836E+01_JPRB, 0.14282E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.11869E+01_JPRB, 0.11095E+01_JPRB, 0.10472E+01_JPRB, 0.10090E+01_JPRB, 0.97926E+00_JPRB, 0.95179E+00_JPRB ,&
&  0.91982E+00_JPRB, 0.88834E+00_JPRB, 0.85652E+00_JPRB, 0.82907E+00_JPRB, 0.80010E+00_JPRB, 0.78183E+00_JPRB, 0.77425E+00_JPRB ,&
&  0.77479E+00_JPRB, 0.78559E+00_JPRB, 0.80196E+00_JPRB, 0.82355E+00_JPRB, 0.84611E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.88512E+00_JPRB, 0.88696E+00_JPRB, 0.88170E+00_JPRB, 0.86728E+00_JPRB, 0.84489E+00_JPRB, 0.81973E+00_JPRB, 0.79742E+00_JPRB ,&
&  0.77804E+00_JPRB, 0.76284E+00_JPRB, 0.74751E+00_JPRB, 0.73526E+00_JPRB, 0.72103E+00_JPRB, 0.71181E+00_JPRB, 0.70106E+00_JPRB ,&
&  0.69509E+00_JPRB, 0.69243E+00_JPRB, 0.68390E+00_JPRB, 0.67362E+00_JPRB, 0.65515E+00_JPRB, 0.63857E+00_JPRB, 0.62571E+00_JPRB ,&
&  0.61316E+00_JPRB, 0.60282E+00_JPRB, 0.57907E+00_JPRB, 0.56234E+00_JPRB, 0.55293E+00_JPRB, 0.54125E+00_JPRB, 0.54671E+00_JPRB ,&
&  0.54003E+00_JPRB, 0.54243E+00_JPRB, 0.54514E+00_JPRB, 0.53652E+00_JPRB, 0.52533E+00_JPRB, 0.41110E+00_JPRB, 0.38135E+00_JPRB ,&
&  0.37178E+00_JPRB, 0.36400E+00_JPRB, 0.36272E+00_JPRB, 0.36976E+00_JPRB, 0.38308E+00_JPRB, 0.41912E+00_JPRB, 0.45473E+00_JPRB ,&
&  0.48866E+00_JPRB, 0.68115E+00_JPRB, 0.76055E+00_JPRB, 0.83359E+00_JPRB, 0.88975E+00_JPRB, 0.92959E+00_JPRB, 0.95342E+00_JPRB ,&
&  0.98262E+00_JPRB, 0.10219E+01_JPRB, 0.10546E+01_JPRB, 0.11162E+01_JPRB, 0.12234E+01_JPRB, 0.13872E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.17358E+01_JPRB, 0.18755E+01_JPRB, 0.20123E+01_JPRB, 0.21238E+01_JPRB, 0.22374E+01_JPRB, 0.23140E+01_JPRB, 0.23517E+01_JPRB ,&
&  0.23978E+01_JPRB, 0.23761E+01_JPRB, 0.24112E+01_JPRB, 0.23714E+01_JPRB, 0.23335E+01_JPRB, 0.22793E+01_JPRB, 0.21314E+01_JPRB ,&
&  0.19541E+01_JPRB, 0.18329E+01_JPRB, 0.17131E+01_JPRB, 0.15581E+01_JPRB, 0.14420E+01_JPRB, 0.13567E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.12758E+01_JPRB, 0.12466E+01_JPRB, 0.11835E+01_JPRB, 0.11154E+01_JPRB, 0.10692E+01_JPRB, 0.10350E+01_JPRB, 0.99017E+00_JPRB ,&
&  0.94665E+00_JPRB, 0.90456E+00_JPRB, 0.87438E+00_JPRB, 0.84672E+00_JPRB, 0.82302E+00_JPRB, 0.80952E+00_JPRB, 0.80308E+00_JPRB ,&
&  0.80532E+00_JPRB, 0.81345E+00_JPRB, 0.83012E+00_JPRB, 0.85270E+00_JPRB, 0.87683E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.90201E+00_JPRB, 0.88693E+00_JPRB, 0.85585E+00_JPRB, 0.81393E+00_JPRB, 0.78034E+00_JPRB, 0.75471E+00_JPRB, 0.74167E+00_JPRB ,&
&  0.73353E+00_JPRB, 0.73018E+00_JPRB, 0.72526E+00_JPRB, 0.71983E+00_JPRB, 0.71049E+00_JPRB, 0.69986E+00_JPRB, 0.68805E+00_JPRB ,&
&  0.67181E+00_JPRB, 0.65504E+00_JPRB, 0.63924E+00_JPRB, 0.62908E+00_JPRB, 0.62045E+00_JPRB, 0.61444E+00_JPRB, 0.60859E+00_JPRB ,&
&  0.60255E+00_JPRB, 0.59484E+00_JPRB, 0.57059E+00_JPRB, 0.56118E+00_JPRB, 0.55065E+00_JPRB, 0.55031E+00_JPRB, 0.54408E+00_JPRB ,&
&  0.55574E+00_JPRB, 0.48751E+00_JPRB, 0.48050E+00_JPRB, 0.46100E+00_JPRB, 0.44388E+00_JPRB, 0.42553E+00_JPRB, 0.41056E+00_JPRB ,&
&  0.40329E+00_JPRB, 0.39528E+00_JPRB, 0.39257E+00_JPRB, 0.40136E+00_JPRB, 0.41299E+00_JPRB, 0.43767E+00_JPRB, 0.46636E+00_JPRB ,&
&  0.51940E+00_JPRB, 0.69668E+00_JPRB, 0.78700E+00_JPRB, 0.85941E+00_JPRB, 0.93266E+00_JPRB, 0.99680E+00_JPRB, 0.10371E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.11435E+01_JPRB, 0.12036E+01_JPRB, 0.12437E+01_JPRB, 0.13848E+01_JPRB, 0.14889E+01_JPRB, 0.16074E+01_JPRB ,&
&  0.17194E+01_JPRB, 0.18339E+01_JPRB, 0.19363E+01_JPRB, 0.20171E+01_JPRB, 0.20714E+01_JPRB, 0.20800E+01_JPRB, 0.20518E+01_JPRB ,&
&  0.20396E+01_JPRB, 0.20232E+01_JPRB, 0.20223E+01_JPRB, 0.19626E+01_JPRB, 0.19337E+01_JPRB, 0.19040E+01_JPRB, 0.18300E+01_JPRB ,&
&  0.17378E+01_JPRB, 0.16524E+01_JPRB, 0.15611E+01_JPRB, 0.14672E+01_JPRB, 0.13772E+01_JPRB, 0.13203E+01_JPRB, 0.12868E+01_JPRB ,&
&  0.12690E+01_JPRB, 0.12393E+01_JPRB, 0.11978E+01_JPRB, 0.11398E+01_JPRB, 0.10848E+01_JPRB, 0.10448E+01_JPRB, 0.10027E+01_JPRB ,&
&  0.96175E+00_JPRB, 0.91979E+00_JPRB, 0.89031E+00_JPRB, 0.86590E+00_JPRB, 0.85131E+00_JPRB, 0.84389E+00_JPRB, 0.84298E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.85091E+00_JPRB, 0.86523E+00_JPRB, 0.88282E+00_JPRB, 0.89704E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.83711E+00_JPRB, 0.80715E+00_JPRB, 0.77244E+00_JPRB, 0.75163E+00_JPRB, 0.74360E+00_JPRB, 0.73474E+00_JPRB, 0.72717E+00_JPRB ,&
&  0.72129E+00_JPRB, 0.72198E+00_JPRB, 0.72388E+00_JPRB, 0.72356E+00_JPRB, 0.71929E+00_JPRB, 0.71259E+00_JPRB, 0.70322E+00_JPRB ,&
&  0.69111E+00_JPRB, 0.67748E+00_JPRB, 0.66497E+00_JPRB, 0.65332E+00_JPRB, 0.64067E+00_JPRB, 0.62869E+00_JPRB, 0.61194E+00_JPRB ,&
&  0.59674E+00_JPRB, 0.58271E+00_JPRB, 0.53781E+00_JPRB, 0.53480E+00_JPRB, 0.52506E+00_JPRB, 0.52238E+00_JPRB, 0.52453E+00_JPRB ,&
&  0.51904E+00_JPRB, 0.50567E+00_JPRB, 0.49249E+00_JPRB, 0.47480E+00_JPRB, 0.46046E+00_JPRB, 0.44402E+00_JPRB, 0.43604E+00_JPRB ,&
&  0.43312E+00_JPRB, 0.43682E+00_JPRB, 0.44580E+00_JPRB, 0.45912E+00_JPRB, 0.47575E+00_JPRB, 0.49456E+00_JPRB, 0.52305E+00_JPRB ,&
&  0.59025E+00_JPRB, 0.68275E+00_JPRB, 0.79744E+00_JPRB, 0.86518E+00_JPRB, 0.94350E+00_JPRB, 0.10185E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.12049E+01_JPRB, 0.12786E+01_JPRB, 0.13559E+01_JPRB, 0.14359E+01_JPRB, 0.15066E+01_JPRB, 0.15887E+01_JPRB ,&
&  0.16586E+01_JPRB, 0.17185E+01_JPRB, 0.17533E+01_JPRB, 0.17701E+01_JPRB, 0.17734E+01_JPRB, 0.17572E+01_JPRB, 0.17383E+01_JPRB ,&
&  0.17263E+01_JPRB, 0.17114E+01_JPRB, 0.16940E+01_JPRB, 0.16468E+01_JPRB, 0.16284E+01_JPRB, 0.16211E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.15535E+01_JPRB, 0.15067E+01_JPRB, 0.14599E+01_JPRB, 0.13961E+01_JPRB, 0.13385E+01_JPRB, 0.13158E+01_JPRB, 0.12934E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.12459E+01_JPRB, 0.12190E+01_JPRB, 0.11832E+01_JPRB, 0.11340E+01_JPRB, 0.10864E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.98006E+00_JPRB, 0.95165E+00_JPRB, 0.92157E+00_JPRB, 0.90188E+00_JPRB, 0.88407E+00_JPRB, 0.87637E+00_JPRB ,&
&  0.87271E+00_JPRB, 0.87411E+00_JPRB, 0.87342E+00_JPRB, 0.87051E+00_JPRB, 0.85791E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.81713E+00_JPRB, 0.78801E+00_JPRB, 0.77152E+00_JPRB, 0.76559E+00_JPRB, 0.76044E+00_JPRB, 0.75524E+00_JPRB, 0.75069E+00_JPRB ,&
&  0.74228E+00_JPRB, 0.73477E+00_JPRB, 0.73307E+00_JPRB, 0.73277E+00_JPRB, 0.73005E+00_JPRB, 0.72641E+00_JPRB, 0.72114E+00_JPRB ,&
&  0.71499E+00_JPRB, 0.70652E+00_JPRB, 0.69755E+00_JPRB, 0.68622E+00_JPRB, 0.67425E+00_JPRB, 0.66083E+00_JPRB, 0.64401E+00_JPRB ,&
&  0.63038E+00_JPRB, 0.61807E+00_JPRB, 0.60258E+00_JPRB, 0.59649E+00_JPRB, 0.59370E+00_JPRB, 0.56017E+00_JPRB, 0.55208E+00_JPRB ,&
&  0.55712E+00_JPRB, 0.55722E+00_JPRB, 0.55832E+00_JPRB, 0.56069E+00_JPRB, 0.56395E+00_JPRB, 0.55304E+00_JPRB, 0.54989E+00_JPRB ,&
&  0.52643E+00_JPRB, 0.54333E+00_JPRB, 0.55540E+00_JPRB, 0.56815E+00_JPRB, 0.58262E+00_JPRB, 0.60581E+00_JPRB, 0.64824E+00_JPRB ,&
&  0.72862E+00_JPRB, 0.76993E+00_JPRB, 0.81508E+00_JPRB, 0.86092E+00_JPRB, 0.91543E+00_JPRB, 0.97018E+00_JPRB, 0.10190E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.11201E+01_JPRB, 0.11796E+01_JPRB, 0.12492E+01_JPRB, 0.13100E+01_JPRB, 0.13648E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.14768E+01_JPRB, 0.14987E+01_JPRB, 0.15117E+01_JPRB, 0.15265E+01_JPRB, 0.15350E+01_JPRB, 0.15449E+01_JPRB ,&
&  0.15516E+01_JPRB, 0.15536E+01_JPRB, 0.15422E+01_JPRB, 0.15193E+01_JPRB, 0.15049E+01_JPRB, 0.14990E+01_JPRB, 0.14871E+01_JPRB ,&
&  0.14714E+01_JPRB, 0.14539E+01_JPRB, 0.14308E+01_JPRB, 0.14021E+01_JPRB, 0.13643E+01_JPRB, 0.13299E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.12258E+01_JPRB, 0.12012E+01_JPRB, 0.11737E+01_JPRB, 0.11444E+01_JPRB, 0.11155E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.10572E+01_JPRB, 0.10321E+01_JPRB, 0.10128E+01_JPRB, 0.99278E+00_JPRB, 0.97873E+00_JPRB, 0.96030E+00_JPRB, 0.94499E+00_JPRB ,&
&  0.92851E+00_JPRB, 0.91741E+00_JPRB, 0.89776E+00_JPRB, 0.87305E+00_JPRB, 0.84239E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.80525E+00_JPRB, 0.79252E+00_JPRB, 0.78610E+00_JPRB, 0.78128E+00_JPRB, 0.77574E+00_JPRB, 0.76838E+00_JPRB, 0.76064E+00_JPRB ,&
&  0.75428E+00_JPRB, 0.74633E+00_JPRB, 0.73906E+00_JPRB, 0.73557E+00_JPRB, 0.73058E+00_JPRB, 0.72685E+00_JPRB, 0.72375E+00_JPRB ,&
&  0.72151E+00_JPRB, 0.71868E+00_JPRB, 0.71631E+00_JPRB, 0.71297E+00_JPRB, 0.70843E+00_JPRB, 0.70265E+00_JPRB, 0.69759E+00_JPRB ,&
&  0.69471E+00_JPRB, 0.69268E+00_JPRB, 0.68971E+00_JPRB, 0.68738E+00_JPRB, 0.68755E+00_JPRB, 0.68691E+00_JPRB, 0.68520E+00_JPRB ,&
&  0.68784E+00_JPRB, 0.69003E+00_JPRB, 0.69030E+00_JPRB, 0.68890E+00_JPRB, 0.68642E+00_JPRB, 0.68653E+00_JPRB, 0.69036E+00_JPRB ,&
&  0.69573E+00_JPRB, 0.71106E+00_JPRB, 0.72391E+00_JPRB, 0.73628E+00_JPRB, 0.74854E+00_JPRB, 0.76337E+00_JPRB, 0.78134E+00_JPRB ,&
&  0.80058E+00_JPRB, 0.81806E+00_JPRB, 0.83687E+00_JPRB, 0.86039E+00_JPRB, 0.88547E+00_JPRB, 0.91170E+00_JPRB, 0.93961E+00_JPRB ,&
&  0.96437E+00_JPRB, 0.98869E+00_JPRB, 0.10164E+01_JPRB, 0.10466E+01_JPRB, 0.10728E+01_JPRB, 0.11040E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.12046E+01_JPRB, 0.12308E+01_JPRB, 0.12580E+01_JPRB, 0.12885E+01_JPRB, 0.13080E+01_JPRB, 0.13266E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.13497E+01_JPRB, 0.13461E+01_JPRB, 0.13405E+01_JPRB, 0.13374E+01_JPRB, 0.13309E+01_JPRB, 0.13218E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.13091E+01_JPRB, 0.12977E+01_JPRB, 0.12834E+01_JPRB, 0.12657E+01_JPRB, 0.12478E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.11661E+01_JPRB, 0.11495E+01_JPRB, 0.11302E+01_JPRB, 0.11117E+01_JPRB, 0.10945E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10438E+01_JPRB, 0.10257E+01_JPRB, 0.10067E+01_JPRB, 0.99091E+00_JPRB, 0.96963E+00_JPRB, 0.94665E+00_JPRB ,&
&  0.92106E+00_JPRB, 0.90065E+00_JPRB, 0.87624E+00_JPRB, 0.84832E+00_JPRB, 0.82166E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.82973E+00_JPRB, 0.82388E+00_JPRB, 0.81773E+00_JPRB, 0.81129E+00_JPRB, 0.80401E+00_JPRB, 0.79614E+00_JPRB, 0.78746E+00_JPRB ,&
&  0.77891E+00_JPRB, 0.77100E+00_JPRB, 0.76386E+00_JPRB, 0.75706E+00_JPRB, 0.75079E+00_JPRB, 0.74574E+00_JPRB, 0.74333E+00_JPRB ,&
&  0.74255E+00_JPRB, 0.74183E+00_JPRB, 0.74163E+00_JPRB, 0.74153E+00_JPRB, 0.74097E+00_JPRB, 0.74003E+00_JPRB, 0.73901E+00_JPRB ,&
&  0.73841E+00_JPRB, 0.73803E+00_JPRB, 0.73740E+00_JPRB, 0.73749E+00_JPRB, 0.73816E+00_JPRB, 0.73909E+00_JPRB, 0.74002E+00_JPRB ,&
&  0.74197E+00_JPRB, 0.74420E+00_JPRB, 0.74699E+00_JPRB, 0.75008E+00_JPRB, 0.75598E+00_JPRB, 0.76275E+00_JPRB, 0.76983E+00_JPRB ,&
&  0.77679E+00_JPRB, 0.78422E+00_JPRB, 0.79092E+00_JPRB, 0.79707E+00_JPRB, 0.80275E+00_JPRB, 0.80919E+00_JPRB, 0.81638E+00_JPRB ,&
&  0.82443E+00_JPRB, 0.83314E+00_JPRB, 0.84280E+00_JPRB, 0.85256E+00_JPRB, 0.86187E+00_JPRB, 0.87141E+00_JPRB, 0.88131E+00_JPRB ,&
&  0.89124E+00_JPRB, 0.90119E+00_JPRB, 0.91101E+00_JPRB, 0.92066E+00_JPRB, 0.93009E+00_JPRB, 0.94079E+00_JPRB, 0.95312E+00_JPRB ,&
&  0.96621E+00_JPRB, 0.97708E+00_JPRB, 0.98903E+00_JPRB, 0.10019E+01_JPRB, 0.10185E+01_JPRB, 0.10271E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10548E+01_JPRB, 0.10615E+01_JPRB, 0.10687E+01_JPRB, 0.10756E+01_JPRB, 0.10814E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.10897E+01_JPRB, 0.10894E+01_JPRB, 0.10870E+01_JPRB, 0.10833E+01_JPRB, 0.10789E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.10621E+01_JPRB, 0.10544E+01_JPRB, 0.10473E+01_JPRB, 0.10374E+01_JPRB, 0.10276E+01_JPRB, 0.10177E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.99518E+00_JPRB, 0.98267E+00_JPRB, 0.96942E+00_JPRB, 0.95546E+00_JPRB, 0.94151E+00_JPRB, 0.92727E+00_JPRB, 0.91226E+00_JPRB ,&
&  0.89581E+00_JPRB, 0.88048E+00_JPRB, 0.86378E+00_JPRB, 0.84748E+00_JPRB, 0.83528E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB, 0.85743E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB ,&
&  0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB, 0.13196E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.13976E+00_JPRB, 0.13976E+00_JPRB, 0.13968E+00_JPRB, 0.13940E+00_JPRB, 0.13882E+00_JPRB, 0.13869E+00_JPRB, 0.13850E+00_JPRB ,&
&  0.13825E+00_JPRB, 0.13769E+00_JPRB, 0.13767E+00_JPRB, 0.13760E+00_JPRB, 0.13752E+00_JPRB, 0.13741E+00_JPRB, 0.13721E+00_JPRB ,&
&  0.13680E+00_JPRB, 0.13618E+00_JPRB, 0.13429E+00_JPRB, 0.13338E+00_JPRB, 0.13282E+00_JPRB, 0.13252E+00_JPRB, 0.13228E+00_JPRB ,&
&  0.13214E+00_JPRB, 0.13208E+00_JPRB, 0.13201E+00_JPRB, 0.13239E+00_JPRB, 0.13256E+00_JPRB, 0.13263E+00_JPRB, 0.13268E+00_JPRB ,&
&  0.13269E+00_JPRB, 0.13277E+00_JPRB, 0.13278E+00_JPRB, 0.13278E+00_JPRB, 0.13289E+00_JPRB, 0.13289E+00_JPRB, 0.13273E+00_JPRB ,&
&  0.13254E+00_JPRB, 0.13231E+00_JPRB, 0.13209E+00_JPRB, 0.13181E+00_JPRB, 0.13160E+00_JPRB, 0.13144E+00_JPRB, 0.13137E+00_JPRB ,&
&  0.13123E+00_JPRB, 0.13111E+00_JPRB, 0.13099E+00_JPRB, 0.13093E+00_JPRB, 0.13083E+00_JPRB, 0.13078E+00_JPRB, 0.13067E+00_JPRB ,&
&  0.13073E+00_JPRB, 0.13075E+00_JPRB, 0.13078E+00_JPRB, 0.13079E+00_JPRB, 0.13078E+00_JPRB, 0.13074E+00_JPRB, 0.13065E+00_JPRB ,&
&  0.13048E+00_JPRB, 0.13038E+00_JPRB, 0.13027E+00_JPRB, 0.13018E+00_JPRB, 0.13014E+00_JPRB, 0.13011E+00_JPRB, 0.13011E+00_JPRB ,&
&  0.13010E+00_JPRB, 0.13018E+00_JPRB, 0.13020E+00_JPRB, 0.13024E+00_JPRB, 0.13029E+00_JPRB, 0.13035E+00_JPRB, 0.13039E+00_JPRB ,&
&  0.13046E+00_JPRB, 0.13052E+00_JPRB, 0.13061E+00_JPRB, 0.13074E+00_JPRB, 0.13088E+00_JPRB, 0.13107E+00_JPRB, 0.13132E+00_JPRB ,&
&  0.13158E+00_JPRB, 0.13185E+00_JPRB, 0.13211E+00_JPRB, 0.13237E+00_JPRB, 0.13278E+00_JPRB, 0.13318E+00_JPRB, 0.13366E+00_JPRB ,&
&  0.13413E+00_JPRB, 0.13461E+00_JPRB, 0.13503E+00_JPRB, 0.13549E+00_JPRB, 0.13607E+00_JPRB, 0.13637E+00_JPRB, 0.13662E+00_JPRB ,&
&  0.13700E+00_JPRB, 0.13740E+00_JPRB, 0.13780E+00_JPRB, 0.13816E+00_JPRB, 0.13874E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.14457E+00_JPRB, 0.14488E+00_JPRB, 0.14442E+00_JPRB, 0.14395E+00_JPRB, 0.13858E+00_JPRB, 0.14453E+00_JPRB, 0.14382E+00_JPRB ,&
&  0.14414E+00_JPRB, 0.14381E+00_JPRB, 0.14486E+00_JPRB, 0.14488E+00_JPRB, 0.14449E+00_JPRB, 0.14422E+00_JPRB, 0.14390E+00_JPRB ,&
&  0.14354E+00_JPRB, 0.14282E+00_JPRB, 0.14242E+00_JPRB, 0.14188E+00_JPRB, 0.14136E+00_JPRB, 0.14053E+00_JPRB, 0.13967E+00_JPRB ,&
&  0.13929E+00_JPRB, 0.13937E+00_JPRB, 0.13974E+00_JPRB, 0.13841E+00_JPRB, 0.13788E+00_JPRB, 0.13754E+00_JPRB, 0.13740E+00_JPRB ,&
&  0.13713E+00_JPRB, 0.13709E+00_JPRB, 0.13712E+00_JPRB, 0.13708E+00_JPRB, 0.13816E+00_JPRB, 0.13848E+00_JPRB, 0.13873E+00_JPRB ,&
&  0.13875E+00_JPRB, 0.13859E+00_JPRB, 0.13796E+00_JPRB, 0.13738E+00_JPRB, 0.13680E+00_JPRB, 0.13583E+00_JPRB, 0.13480E+00_JPRB ,&
&  0.13392E+00_JPRB, 0.13324E+00_JPRB, 0.13247E+00_JPRB, 0.13189E+00_JPRB, 0.13140E+00_JPRB, 0.13094E+00_JPRB, 0.13063E+00_JPRB ,&
&  0.13054E+00_JPRB, 0.13051E+00_JPRB, 0.13047E+00_JPRB, 0.13042E+00_JPRB, 0.13034E+00_JPRB, 0.13027E+00_JPRB, 0.13020E+00_JPRB ,&
&  0.13039E+00_JPRB, 0.13032E+00_JPRB, 0.13014E+00_JPRB, 0.12986E+00_JPRB, 0.12971E+00_JPRB, 0.12939E+00_JPRB, 0.12917E+00_JPRB ,&
&  0.12898E+00_JPRB, 0.12876E+00_JPRB, 0.12866E+00_JPRB, 0.12862E+00_JPRB, 0.12861E+00_JPRB, 0.12859E+00_JPRB, 0.12866E+00_JPRB ,&
&  0.12878E+00_JPRB, 0.12895E+00_JPRB, 0.12928E+00_JPRB, 0.12956E+00_JPRB, 0.12981E+00_JPRB, 0.13004E+00_JPRB, 0.13048E+00_JPRB ,&
&  0.13096E+00_JPRB, 0.13143E+00_JPRB, 0.13189E+00_JPRB, 0.13221E+00_JPRB, 0.13287E+00_JPRB, 0.13353E+00_JPRB, 0.13426E+00_JPRB ,&
&  0.13503E+00_JPRB, 0.13600E+00_JPRB, 0.13700E+00_JPRB, 0.13799E+00_JPRB, 0.13929E+00_JPRB, 0.14035E+00_JPRB, 0.14137E+00_JPRB ,&
&  0.14250E+00_JPRB, 0.14460E+00_JPRB, 0.14524E+00_JPRB, 0.14548E+00_JPRB, 0.14519E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.15943E+00_JPRB, 0.16013E+00_JPRB, 0.16048E+00_JPRB, 0.16088E+00_JPRB, 0.16183E+00_JPRB, 0.16184E+00_JPRB, 0.16166E+00_JPRB ,&
&  0.15973E+00_JPRB, 0.15894E+00_JPRB, 0.15769E+00_JPRB, 0.15599E+00_JPRB, 0.15427E+00_JPRB, 0.15151E+00_JPRB, 0.14970E+00_JPRB ,&
&  0.14791E+00_JPRB, 0.14603E+00_JPRB, 0.14475E+00_JPRB, 0.14316E+00_JPRB, 0.14179E+00_JPRB, 0.14028E+00_JPRB, 0.13976E+00_JPRB ,&
&  0.13903E+00_JPRB, 0.13898E+00_JPRB, 0.13972E+00_JPRB, 0.14036E+00_JPRB, 0.14149E+00_JPRB, 0.14270E+00_JPRB, 0.14425E+00_JPRB ,&
&  0.14476E+00_JPRB, 0.14559E+00_JPRB, 0.14636E+00_JPRB, 0.14685E+00_JPRB, 0.14842E+00_JPRB, 0.14956E+00_JPRB, 0.14996E+00_JPRB ,&
&  0.14952E+00_JPRB, 0.14874E+00_JPRB, 0.14718E+00_JPRB, 0.14541E+00_JPRB, 0.14334E+00_JPRB, 0.14105E+00_JPRB, 0.13886E+00_JPRB ,&
&  0.13714E+00_JPRB, 0.13556E+00_JPRB, 0.13436E+00_JPRB, 0.13338E+00_JPRB, 0.13276E+00_JPRB, 0.13227E+00_JPRB, 0.13179E+00_JPRB ,&
&  0.13150E+00_JPRB, 0.13125E+00_JPRB, 0.13090E+00_JPRB, 0.13084E+00_JPRB, 0.13071E+00_JPRB, 0.13072E+00_JPRB, 0.13072E+00_JPRB ,&
&  0.13073E+00_JPRB, 0.13065E+00_JPRB, 0.13050E+00_JPRB, 0.13018E+00_JPRB, 0.12986E+00_JPRB, 0.12952E+00_JPRB, 0.12918E+00_JPRB ,&
&  0.12881E+00_JPRB, 0.12874E+00_JPRB, 0.12846E+00_JPRB, 0.12826E+00_JPRB, 0.12800E+00_JPRB, 0.12760E+00_JPRB, 0.12748E+00_JPRB ,&
&  0.12757E+00_JPRB, 0.12770E+00_JPRB, 0.12761E+00_JPRB, 0.12777E+00_JPRB, 0.12798E+00_JPRB, 0.12828E+00_JPRB, 0.12901E+00_JPRB ,&
&  0.12946E+00_JPRB, 0.12994E+00_JPRB, 0.13039E+00_JPRB, 0.13101E+00_JPRB, 0.13206E+00_JPRB, 0.13311E+00_JPRB, 0.13429E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13705E+00_JPRB, 0.13839E+00_JPRB, 0.13971E+00_JPRB, 0.14189E+00_JPRB, 0.14347E+00_JPRB, 0.14556E+00_JPRB ,&
&  0.14741E+00_JPRB, 0.14673E+00_JPRB, 0.15446E+00_JPRB, 0.15534E+00_JPRB, 0.15664E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.16603E+00_JPRB, 0.16632E+00_JPRB, 0.16623E+00_JPRB, 0.16710E+00_JPRB, 0.16790E+00_JPRB, 0.16877E+00_JPRB, 0.16842E+00_JPRB ,&
&  0.16809E+00_JPRB, 0.16652E+00_JPRB, 0.16531E+00_JPRB, 0.16244E+00_JPRB, 0.16014E+00_JPRB, 0.15742E+00_JPRB, 0.15506E+00_JPRB ,&
&  0.15295E+00_JPRB, 0.15096E+00_JPRB, 0.14910E+00_JPRB, 0.14753E+00_JPRB, 0.14609E+00_JPRB, 0.14486E+00_JPRB, 0.14370E+00_JPRB ,&
&  0.14194E+00_JPRB, 0.14148E+00_JPRB, 0.14152E+00_JPRB, 0.14271E+00_JPRB, 0.14510E+00_JPRB, 0.14673E+00_JPRB, 0.14869E+00_JPRB ,&
&  0.15106E+00_JPRB, 0.15065E+00_JPRB, 0.15343E+00_JPRB, 0.15612E+00_JPRB, 0.15832E+00_JPRB, 0.16009E+00_JPRB, 0.16076E+00_JPRB ,&
&  0.16140E+00_JPRB, 0.15727E+00_JPRB, 0.15860E+00_JPRB, 0.15504E+00_JPRB, 0.15071E+00_JPRB, 0.14861E+00_JPRB, 0.14479E+00_JPRB ,&
&  0.14289E+00_JPRB, 0.13993E+00_JPRB, 0.13801E+00_JPRB, 0.13642E+00_JPRB, 0.13460E+00_JPRB, 0.13413E+00_JPRB, 0.13333E+00_JPRB ,&
&  0.13284E+00_JPRB, 0.13284E+00_JPRB, 0.13212E+00_JPRB, 0.13185E+00_JPRB, 0.13138E+00_JPRB, 0.13094E+00_JPRB, 0.13119E+00_JPRB ,&
&  0.13164E+00_JPRB, 0.13221E+00_JPRB, 0.13279E+00_JPRB, 0.13285E+00_JPRB, 0.13279E+00_JPRB, 0.13276E+00_JPRB, 0.13267E+00_JPRB ,&
&  0.13289E+00_JPRB, 0.13290E+00_JPRB, 0.13252E+00_JPRB, 0.13209E+00_JPRB, 0.13113E+00_JPRB, 0.13019E+00_JPRB, 0.12926E+00_JPRB ,&
&  0.12846E+00_JPRB, 0.12802E+00_JPRB, 0.12766E+00_JPRB, 0.12759E+00_JPRB, 0.12764E+00_JPRB, 0.12796E+00_JPRB, 0.12829E+00_JPRB ,&
&  0.12859E+00_JPRB, 0.12924E+00_JPRB, 0.12973E+00_JPRB, 0.13052E+00_JPRB, 0.13147E+00_JPRB, 0.13233E+00_JPRB, 0.13364E+00_JPRB ,&
&  0.13507E+00_JPRB, 0.13664E+00_JPRB, 0.13885E+00_JPRB, 0.14081E+00_JPRB, 0.14327E+00_JPRB, 0.14649E+00_JPRB, 0.14893E+00_JPRB ,&
&  0.15158E+00_JPRB, 0.15437E+00_JPRB, 0.15490E+00_JPRB, 0.16146E+00_JPRB, 0.16511E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.16504E+00_JPRB, 0.16639E+00_JPRB, 0.16781E+00_JPRB, 0.16923E+00_JPRB, 0.17047E+00_JPRB, 0.17169E+00_JPRB, 0.17249E+00_JPRB ,&
&  0.17019E+00_JPRB, 0.16880E+00_JPRB, 0.16624E+00_JPRB, 0.16386E+00_JPRB, 0.16117E+00_JPRB, 0.15825E+00_JPRB, 0.15720E+00_JPRB ,&
&  0.15578E+00_JPRB, 0.15547E+00_JPRB, 0.15434E+00_JPRB, 0.15386E+00_JPRB, 0.15295E+00_JPRB, 0.15245E+00_JPRB, 0.15159E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.14887E+00_JPRB, 0.14797E+00_JPRB, 0.14837E+00_JPRB, 0.14978E+00_JPRB, 0.15133E+00_JPRB, 0.15297E+00_JPRB ,&
&  0.15530E+00_JPRB, 0.15646E+00_JPRB, 0.15874E+00_JPRB, 0.16042E+00_JPRB, 0.16312E+00_JPRB, 0.16540E+00_JPRB, 0.16590E+00_JPRB ,&
&  0.16600E+00_JPRB, 0.16460E+00_JPRB, 0.16432E+00_JPRB, 0.16260E+00_JPRB, 0.15702E+00_JPRB, 0.15378E+00_JPRB, 0.14925E+00_JPRB ,&
&  0.14420E+00_JPRB, 0.14039E+00_JPRB, 0.13777E+00_JPRB, 0.13689E+00_JPRB, 0.13604E+00_JPRB, 0.13574E+00_JPRB, 0.13546E+00_JPRB ,&
&  0.13469E+00_JPRB, 0.13378E+00_JPRB, 0.13274E+00_JPRB, 0.13212E+00_JPRB, 0.13239E+00_JPRB, 0.13301E+00_JPRB, 0.13439E+00_JPRB ,&
&  0.13578E+00_JPRB, 0.13654E+00_JPRB, 0.13740E+00_JPRB, 0.13767E+00_JPRB, 0.13778E+00_JPRB, 0.13799E+00_JPRB, 0.13805E+00_JPRB ,&
&  0.13864E+00_JPRB, 0.13879E+00_JPRB, 0.13832E+00_JPRB, 0.13755E+00_JPRB, 0.13559E+00_JPRB, 0.13303E+00_JPRB, 0.13099E+00_JPRB ,&
&  0.12925E+00_JPRB, 0.12813E+00_JPRB, 0.12773E+00_JPRB, 0.12734E+00_JPRB, 0.12758E+00_JPRB, 0.12793E+00_JPRB, 0.12826E+00_JPRB ,&
&  0.12881E+00_JPRB, 0.12929E+00_JPRB, 0.12976E+00_JPRB, 0.13078E+00_JPRB, 0.13163E+00_JPRB, 0.13288E+00_JPRB, 0.13419E+00_JPRB ,&
&  0.13511E+00_JPRB, 0.13651E+00_JPRB, 0.13770E+00_JPRB, 0.13900E+00_JPRB, 0.14289E+00_JPRB, 0.14602E+00_JPRB, 0.15241E+00_JPRB ,&
&  0.15670E+00_JPRB, 0.15879E+00_JPRB, 0.16011E+00_JPRB, 0.16487E+00_JPRB, 0.16475E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.16519E+00_JPRB, 0.16697E+00_JPRB, 0.16894E+00_JPRB, 0.17045E+00_JPRB, 0.17193E+00_JPRB, 0.17319E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17402E+00_JPRB, 0.17346E+00_JPRB, 0.17353E+00_JPRB, 0.17240E+00_JPRB, 0.17136E+00_JPRB, 0.16720E+00_JPRB, 0.16653E+00_JPRB ,&
&  0.16549E+00_JPRB, 0.16596E+00_JPRB, 0.16552E+00_JPRB, 0.16571E+00_JPRB, 0.16506E+00_JPRB, 0.16473E+00_JPRB, 0.16337E+00_JPRB ,&
&  0.16136E+00_JPRB, 0.15981E+00_JPRB, 0.15826E+00_JPRB, 0.15777E+00_JPRB, 0.15848E+00_JPRB, 0.15904E+00_JPRB, 0.15880E+00_JPRB ,&
&  0.15883E+00_JPRB, 0.15916E+00_JPRB, 0.15964E+00_JPRB, 0.15980E+00_JPRB, 0.16483E+00_JPRB, 0.16693E+00_JPRB, 0.16812E+00_JPRB ,&
&  0.16818E+00_JPRB, 0.16873E+00_JPRB, 0.16901E+00_JPRB, 0.16839E+00_JPRB, 0.16774E+00_JPRB, 0.16418E+00_JPRB, 0.16187E+00_JPRB ,&
&  0.15423E+00_JPRB, 0.14874E+00_JPRB, 0.14454E+00_JPRB, 0.14193E+00_JPRB, 0.14084E+00_JPRB, 0.14069E+00_JPRB, 0.14010E+00_JPRB ,&
&  0.13861E+00_JPRB, 0.13722E+00_JPRB, 0.13632E+00_JPRB, 0.13685E+00_JPRB, 0.13776E+00_JPRB, 0.13883E+00_JPRB, 0.14005E+00_JPRB ,&
&  0.14115E+00_JPRB, 0.14140E+00_JPRB, 0.14137E+00_JPRB, 0.14172E+00_JPRB, 0.14199E+00_JPRB, 0.14243E+00_JPRB, 0.14278E+00_JPRB ,&
&  0.14368E+00_JPRB, 0.14303E+00_JPRB, 0.14357E+00_JPRB, 0.14454E+00_JPRB, 0.14245E+00_JPRB, 0.13908E+00_JPRB, 0.13569E+00_JPRB ,&
&  0.13256E+00_JPRB, 0.13080E+00_JPRB, 0.12939E+00_JPRB, 0.12851E+00_JPRB, 0.12832E+00_JPRB, 0.12845E+00_JPRB, 0.12897E+00_JPRB ,&
&  0.12964E+00_JPRB, 0.13053E+00_JPRB, 0.13112E+00_JPRB, 0.13185E+00_JPRB, 0.13275E+00_JPRB, 0.13370E+00_JPRB, 0.13483E+00_JPRB ,&
&  0.13623E+00_JPRB, 0.13756E+00_JPRB, 0.13849E+00_JPRB, 0.13984E+00_JPRB, 0.14240E+00_JPRB, 0.14587E+00_JPRB, 0.15119E+00_JPRB ,&
&  0.15725E+00_JPRB, 0.16267E+00_JPRB, 0.16085E+00_JPRB, 0.16281E+00_JPRB, 0.16441E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.17047E+00_JPRB, 0.17283E+00_JPRB, 0.17458E+00_JPRB, 0.17620E+00_JPRB, 0.17744E+00_JPRB, 0.17885E+00_JPRB, 0.17950E+00_JPRB ,&
&  0.18009E+00_JPRB, 0.18009E+00_JPRB, 0.18047E+00_JPRB, 0.18038E+00_JPRB, 0.18053E+00_JPRB, 0.17947E+00_JPRB, 0.17885E+00_JPRB ,&
&  0.17839E+00_JPRB, 0.17859E+00_JPRB, 0.17846E+00_JPRB, 0.17821E+00_JPRB, 0.17740E+00_JPRB, 0.17639E+00_JPRB, 0.17492E+00_JPRB ,&
&  0.17204E+00_JPRB, 0.17022E+00_JPRB, 0.17062E+00_JPRB, 0.17027E+00_JPRB, 0.17070E+00_JPRB, 0.17077E+00_JPRB, 0.17120E+00_JPRB ,&
&  0.16738E+00_JPRB, 0.16588E+00_JPRB, 0.16584E+00_JPRB, 0.16501E+00_JPRB, 0.16749E+00_JPRB, 0.16860E+00_JPRB, 0.17044E+00_JPRB ,&
&  0.17112E+00_JPRB, 0.17164E+00_JPRB, 0.17247E+00_JPRB, 0.17226E+00_JPRB, 0.17274E+00_JPRB, 0.17329E+00_JPRB, 0.17233E+00_JPRB ,&
&  0.16966E+00_JPRB, 0.16747E+00_JPRB, 0.16440E+00_JPRB, 0.15666E+00_JPRB, 0.15451E+00_JPRB, 0.15453E+00_JPRB, 0.15303E+00_JPRB ,&
&  0.15036E+00_JPRB, 0.15044E+00_JPRB, 0.14991E+00_JPRB, 0.14848E+00_JPRB, 0.15002E+00_JPRB, 0.15227E+00_JPRB, 0.15235E+00_JPRB ,&
&  0.15125E+00_JPRB, 0.15097E+00_JPRB, 0.15035E+00_JPRB, 0.14919E+00_JPRB, 0.14782E+00_JPRB, 0.14724E+00_JPRB, 0.14673E+00_JPRB ,&
&  0.14650E+00_JPRB, 0.14545E+00_JPRB, 0.14866E+00_JPRB, 0.14960E+00_JPRB, 0.14522E+00_JPRB, 0.14643E+00_JPRB, 0.14411E+00_JPRB ,&
&  0.13815E+00_JPRB, 0.13543E+00_JPRB, 0.13396E+00_JPRB, 0.13277E+00_JPRB, 0.13259E+00_JPRB, 0.13278E+00_JPRB, 0.13272E+00_JPRB ,&
&  0.13356E+00_JPRB, 0.13460E+00_JPRB, 0.13509E+00_JPRB, 0.13570E+00_JPRB, 0.13652E+00_JPRB, 0.13771E+00_JPRB, 0.13926E+00_JPRB ,&
&  0.14060E+00_JPRB, 0.14249E+00_JPRB, 0.14349E+00_JPRB, 0.14513E+00_JPRB, 0.14636E+00_JPRB, 0.14781E+00_JPRB, 0.15124E+00_JPRB ,&
&  0.15417E+00_JPRB, 0.15984E+00_JPRB, 0.16764E+00_JPRB, 0.17044E+00_JPRB, 0.16825E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.17607E+00_JPRB, 0.17831E+00_JPRB, 0.18111E+00_JPRB, 0.18400E+00_JPRB, 0.18620E+00_JPRB, 0.18841E+00_JPRB, 0.19029E+00_JPRB ,&
&  0.19187E+00_JPRB, 0.19288E+00_JPRB, 0.19350E+00_JPRB, 0.19379E+00_JPRB, 0.19397E+00_JPRB, 0.19359E+00_JPRB, 0.19277E+00_JPRB ,&
&  0.19191E+00_JPRB, 0.19051E+00_JPRB, 0.18874E+00_JPRB, 0.18725E+00_JPRB, 0.18621E+00_JPRB, 0.18498E+00_JPRB, 0.18327E+00_JPRB ,&
&  0.18151E+00_JPRB, 0.18001E+00_JPRB, 0.17895E+00_JPRB, 0.17803E+00_JPRB, 0.17810E+00_JPRB, 0.17868E+00_JPRB, 0.17915E+00_JPRB ,&
&  0.17573E+00_JPRB, 0.17070E+00_JPRB, 0.17106E+00_JPRB, 0.17207E+00_JPRB, 0.17117E+00_JPRB, 0.17256E+00_JPRB, 0.17350E+00_JPRB ,&
&  0.17475E+00_JPRB, 0.17627E+00_JPRB, 0.17772E+00_JPRB, 0.17855E+00_JPRB, 0.17917E+00_JPRB, 0.18003E+00_JPRB, 0.17960E+00_JPRB ,&
&  0.17781E+00_JPRB, 0.17615E+00_JPRB, 0.17458E+00_JPRB, 0.17302E+00_JPRB, 0.17258E+00_JPRB, 0.17386E+00_JPRB, 0.17457E+00_JPRB ,&
&  0.17360E+00_JPRB, 0.17219E+00_JPRB, 0.17087E+00_JPRB, 0.17094E+00_JPRB, 0.17152E+00_JPRB, 0.17176E+00_JPRB, 0.16989E+00_JPRB ,&
&  0.16842E+00_JPRB, 0.16575E+00_JPRB, 0.16320E+00_JPRB, 0.16073E+00_JPRB, 0.15935E+00_JPRB, 0.15614E+00_JPRB, 0.15282E+00_JPRB ,&
&  0.15207E+00_JPRB, 0.15554E+00_JPRB, 0.16111E+00_JPRB, 0.16364E+00_JPRB, 0.15580E+00_JPRB, 0.15498E+00_JPRB, 0.15435E+00_JPRB ,&
&  0.14997E+00_JPRB, 0.14533E+00_JPRB, 0.14205E+00_JPRB, 0.14067E+00_JPRB, 0.14019E+00_JPRB, 0.13981E+00_JPRB, 0.14033E+00_JPRB ,&
&  0.14127E+00_JPRB, 0.14187E+00_JPRB, 0.14250E+00_JPRB, 0.14358E+00_JPRB, 0.14461E+00_JPRB, 0.14563E+00_JPRB, 0.14650E+00_JPRB ,&
&  0.14838E+00_JPRB, 0.15069E+00_JPRB, 0.15268E+00_JPRB, 0.15538E+00_JPRB, 0.15827E+00_JPRB, 0.16120E+00_JPRB, 0.16422E+00_JPRB ,&
&  0.16758E+00_JPRB, 0.16985E+00_JPRB, 0.17129E+00_JPRB, 0.17369E+00_JPRB, 0.17484E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.18829E+00_JPRB, 0.19348E+00_JPRB, 0.19814E+00_JPRB, 0.20088E+00_JPRB, 0.20289E+00_JPRB, 0.20557E+00_JPRB, 0.20804E+00_JPRB ,&
&  0.20969E+00_JPRB, 0.21055E+00_JPRB, 0.21086E+00_JPRB, 0.21061E+00_JPRB, 0.20959E+00_JPRB, 0.20799E+00_JPRB, 0.20617E+00_JPRB ,&
&  0.20419E+00_JPRB, 0.20150E+00_JPRB, 0.19904E+00_JPRB, 0.19712E+00_JPRB, 0.19546E+00_JPRB, 0.19352E+00_JPRB, 0.19195E+00_JPRB ,&
&  0.19029E+00_JPRB, 0.18960E+00_JPRB, 0.18889E+00_JPRB, 0.18831E+00_JPRB, 0.18791E+00_JPRB, 0.18714E+00_JPRB, 0.18644E+00_JPRB ,&
&  0.18458E+00_JPRB, 0.18143E+00_JPRB, 0.17978E+00_JPRB, 0.17918E+00_JPRB, 0.17934E+00_JPRB, 0.17895E+00_JPRB, 0.17850E+00_JPRB ,&
&  0.18035E+00_JPRB, 0.18330E+00_JPRB, 0.18552E+00_JPRB, 0.18679E+00_JPRB, 0.18772E+00_JPRB, 0.18891E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.18828E+00_JPRB, 0.18570E+00_JPRB, 0.18372E+00_JPRB, 0.18262E+00_JPRB, 0.18236E+00_JPRB, 0.18297E+00_JPRB, 0.18437E+00_JPRB ,&
&  0.18564E+00_JPRB, 0.18628E+00_JPRB, 0.18582E+00_JPRB, 0.18447E+00_JPRB, 0.18289E+00_JPRB, 0.18207E+00_JPRB, 0.18155E+00_JPRB ,&
&  0.18129E+00_JPRB, 0.18099E+00_JPRB, 0.18058E+00_JPRB, 0.17960E+00_JPRB, 0.17476E+00_JPRB, 0.17079E+00_JPRB, 0.16815E+00_JPRB ,&
&  0.16861E+00_JPRB, 0.16908E+00_JPRB, 0.16874E+00_JPRB, 0.17327E+00_JPRB, 0.17816E+00_JPRB, 0.17415E+00_JPRB, 0.16916E+00_JPRB ,&
&  0.16252E+00_JPRB, 0.16228E+00_JPRB, 0.15963E+00_JPRB, 0.15541E+00_JPRB, 0.15407E+00_JPRB, 0.15407E+00_JPRB, 0.15384E+00_JPRB ,&
&  0.15462E+00_JPRB, 0.15502E+00_JPRB, 0.15623E+00_JPRB, 0.15686E+00_JPRB, 0.15717E+00_JPRB, 0.15782E+00_JPRB, 0.15849E+00_JPRB ,&
&  0.16015E+00_JPRB, 0.16343E+00_JPRB, 0.16754E+00_JPRB, 0.16952E+00_JPRB, 0.17193E+00_JPRB, 0.17424E+00_JPRB, 0.17580E+00_JPRB ,&
&  0.17707E+00_JPRB, 0.17882E+00_JPRB, 0.18049E+00_JPRB, 0.18185E+00_JPRB, 0.18421E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.20597E+00_JPRB, 0.21064E+00_JPRB, 0.21364E+00_JPRB, 0.21653E+00_JPRB, 0.21941E+00_JPRB, 0.22394E+00_JPRB, 0.22700E+00_JPRB ,&
&  0.22755E+00_JPRB, 0.22747E+00_JPRB, 0.22693E+00_JPRB, 0.22572E+00_JPRB, 0.22387E+00_JPRB, 0.22219E+00_JPRB, 0.21944E+00_JPRB ,&
&  0.21604E+00_JPRB, 0.21290E+00_JPRB, 0.21008E+00_JPRB, 0.20742E+00_JPRB, 0.20491E+00_JPRB, 0.20261E+00_JPRB, 0.20075E+00_JPRB ,&
&  0.20005E+00_JPRB, 0.19959E+00_JPRB, 0.19852E+00_JPRB, 0.19817E+00_JPRB, 0.19823E+00_JPRB, 0.19773E+00_JPRB, 0.19657E+00_JPRB ,&
&  0.19512E+00_JPRB, 0.19252E+00_JPRB, 0.18985E+00_JPRB, 0.18876E+00_JPRB, 0.18897E+00_JPRB, 0.18840E+00_JPRB, 0.18767E+00_JPRB ,&
&  0.18913E+00_JPRB, 0.19141E+00_JPRB, 0.19395E+00_JPRB, 0.19544E+00_JPRB, 0.19635E+00_JPRB, 0.19749E+00_JPRB, 0.19889E+00_JPRB ,&
&  0.19908E+00_JPRB, 0.19665E+00_JPRB, 0.19442E+00_JPRB, 0.19384E+00_JPRB, 0.19359E+00_JPRB, 0.19318E+00_JPRB, 0.19344E+00_JPRB ,&
&  0.19401E+00_JPRB, 0.19526E+00_JPRB, 0.19608E+00_JPRB, 0.19511E+00_JPRB, 0.19342E+00_JPRB, 0.19217E+00_JPRB, 0.19134E+00_JPRB ,&
&  0.19087E+00_JPRB, 0.19029E+00_JPRB, 0.18934E+00_JPRB, 0.18787E+00_JPRB, 0.18627E+00_JPRB, 0.18481E+00_JPRB, 0.18340E+00_JPRB ,&
&  0.18221E+00_JPRB, 0.18238E+00_JPRB, 0.18435E+00_JPRB, 0.18578E+00_JPRB, 0.18566E+00_JPRB, 0.18440E+00_JPRB, 0.18242E+00_JPRB ,&
&  0.18101E+00_JPRB, 0.17894E+00_JPRB, 0.17730E+00_JPRB, 0.17600E+00_JPRB, 0.17498E+00_JPRB, 0.17358E+00_JPRB, 0.17304E+00_JPRB ,&
&  0.17327E+00_JPRB, 0.17477E+00_JPRB, 0.17546E+00_JPRB, 0.17579E+00_JPRB, 0.17561E+00_JPRB, 0.17545E+00_JPRB, 0.17633E+00_JPRB ,&
&  0.17751E+00_JPRB, 0.17930E+00_JPRB, 0.18118E+00_JPRB, 0.18367E+00_JPRB, 0.18620E+00_JPRB, 0.18947E+00_JPRB, 0.19222E+00_JPRB ,&
&  0.19395E+00_JPRB, 0.19454E+00_JPRB, 0.19585E+00_JPRB, 0.19772E+00_JPRB, 0.20085E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.22267E+00_JPRB, 0.22622E+00_JPRB, 0.22929E+00_JPRB, 0.23302E+00_JPRB, 0.23599E+00_JPRB, 0.24057E+00_JPRB, 0.24251E+00_JPRB ,&
&  0.24240E+00_JPRB, 0.24190E+00_JPRB, 0.24023E+00_JPRB, 0.23759E+00_JPRB, 0.23368E+00_JPRB, 0.22978E+00_JPRB, 0.22616E+00_JPRB ,&
&  0.22336E+00_JPRB, 0.22101E+00_JPRB, 0.21907E+00_JPRB, 0.21730E+00_JPRB, 0.21486E+00_JPRB, 0.21255E+00_JPRB, 0.21070E+00_JPRB ,&
&  0.20975E+00_JPRB, 0.20885E+00_JPRB, 0.20834E+00_JPRB, 0.20801E+00_JPRB, 0.20773E+00_JPRB, 0.20755E+00_JPRB, 0.20762E+00_JPRB ,&
&  0.20665E+00_JPRB, 0.20350E+00_JPRB, 0.20173E+00_JPRB, 0.20123E+00_JPRB, 0.20087E+00_JPRB, 0.20141E+00_JPRB, 0.20134E+00_JPRB ,&
&  0.20142E+00_JPRB, 0.20255E+00_JPRB, 0.20492E+00_JPRB, 0.20723E+00_JPRB, 0.20734E+00_JPRB, 0.20754E+00_JPRB, 0.20848E+00_JPRB ,&
&  0.20880E+00_JPRB, 0.20807E+00_JPRB, 0.20625E+00_JPRB, 0.20554E+00_JPRB, 0.20595E+00_JPRB, 0.20585E+00_JPRB, 0.20619E+00_JPRB ,&
&  0.20688E+00_JPRB, 0.20800E+00_JPRB, 0.20855E+00_JPRB, 0.20710E+00_JPRB, 0.20556E+00_JPRB, 0.20491E+00_JPRB, 0.20476E+00_JPRB ,&
&  0.20435E+00_JPRB, 0.20329E+00_JPRB, 0.20125E+00_JPRB, 0.19943E+00_JPRB, 0.19675E+00_JPRB, 0.19457E+00_JPRB, 0.19349E+00_JPRB ,&
&  0.19399E+00_JPRB, 0.19446E+00_JPRB, 0.19572E+00_JPRB, 0.19685E+00_JPRB, 0.19750E+00_JPRB, 0.19757E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19391E+00_JPRB, 0.19300E+00_JPRB, 0.19207E+00_JPRB, 0.19123E+00_JPRB, 0.19138E+00_JPRB, 0.19127E+00_JPRB, 0.19087E+00_JPRB ,&
&  0.19038E+00_JPRB, 0.19071E+00_JPRB, 0.19115E+00_JPRB, 0.19130E+00_JPRB, 0.19112E+00_JPRB, 0.19053E+00_JPRB, 0.19110E+00_JPRB ,&
&  0.19244E+00_JPRB, 0.19430E+00_JPRB, 0.19645E+00_JPRB, 0.19880E+00_JPRB, 0.20138E+00_JPRB, 0.20379E+00_JPRB, 0.20617E+00_JPRB ,&
&  0.20919E+00_JPRB, 0.21053E+00_JPRB, 0.21255E+00_JPRB, 0.21558E+00_JPRB, 0.21858E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.23696E+00_JPRB, 0.23986E+00_JPRB, 0.24454E+00_JPRB, 0.24940E+00_JPRB, 0.25268E+00_JPRB, 0.25687E+00_JPRB, 0.25761E+00_JPRB ,&
&  0.25689E+00_JPRB, 0.25500E+00_JPRB, 0.25092E+00_JPRB, 0.24598E+00_JPRB, 0.24145E+00_JPRB, 0.23779E+00_JPRB, 0.23528E+00_JPRB ,&
&  0.23374E+00_JPRB, 0.23236E+00_JPRB, 0.23078E+00_JPRB, 0.22829E+00_JPRB, 0.22516E+00_JPRB, 0.22222E+00_JPRB, 0.21979E+00_JPRB ,&
&  0.21867E+00_JPRB, 0.21829E+00_JPRB, 0.21837E+00_JPRB, 0.21807E+00_JPRB, 0.21791E+00_JPRB, 0.21914E+00_JPRB, 0.22078E+00_JPRB ,&
&  0.21986E+00_JPRB, 0.21584E+00_JPRB, 0.21340E+00_JPRB, 0.21333E+00_JPRB, 0.21370E+00_JPRB, 0.21443E+00_JPRB, 0.21541E+00_JPRB ,&
&  0.21610E+00_JPRB, 0.21795E+00_JPRB, 0.22060E+00_JPRB, 0.22265E+00_JPRB, 0.22299E+00_JPRB, 0.22458E+00_JPRB, 0.22551E+00_JPRB ,&
&  0.22548E+00_JPRB, 0.22465E+00_JPRB, 0.22315E+00_JPRB, 0.22046E+00_JPRB, 0.21849E+00_JPRB, 0.21780E+00_JPRB, 0.21714E+00_JPRB ,&
&  0.21787E+00_JPRB, 0.21866E+00_JPRB, 0.21934E+00_JPRB, 0.21862E+00_JPRB, 0.21695E+00_JPRB, 0.21659E+00_JPRB, 0.21687E+00_JPRB ,&
&  0.21663E+00_JPRB, 0.21534E+00_JPRB, 0.21352E+00_JPRB, 0.21240E+00_JPRB, 0.21132E+00_JPRB, 0.21102E+00_JPRB, 0.21091E+00_JPRB ,&
&  0.21073E+00_JPRB, 0.20985E+00_JPRB, 0.20926E+00_JPRB, 0.20897E+00_JPRB, 0.20881E+00_JPRB, 0.20914E+00_JPRB, 0.20885E+00_JPRB ,&
&  0.20887E+00_JPRB, 0.20909E+00_JPRB, 0.20817E+00_JPRB, 0.20791E+00_JPRB, 0.20787E+00_JPRB, 0.20817E+00_JPRB, 0.20825E+00_JPRB ,&
&  0.20798E+00_JPRB, 0.20757E+00_JPRB, 0.20727E+00_JPRB, 0.20737E+00_JPRB, 0.20706E+00_JPRB, 0.20656E+00_JPRB, 0.20736E+00_JPRB ,&
&  0.21037E+00_JPRB, 0.21354E+00_JPRB, 0.21626E+00_JPRB, 0.21808E+00_JPRB, 0.21876E+00_JPRB, 0.22025E+00_JPRB, 0.22312E+00_JPRB ,&
&  0.22582E+00_JPRB, 0.22728E+00_JPRB, 0.22961E+00_JPRB, 0.23150E+00_JPRB, 0.23364E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.25340E+00_JPRB, 0.25622E+00_JPRB, 0.26043E+00_JPRB, 0.26633E+00_JPRB, 0.27005E+00_JPRB, 0.27325E+00_JPRB, 0.27290E+00_JPRB ,&
&  0.27050E+00_JPRB, 0.26690E+00_JPRB, 0.26257E+00_JPRB, 0.25737E+00_JPRB, 0.25206E+00_JPRB, 0.24809E+00_JPRB, 0.24664E+00_JPRB ,&
&  0.24635E+00_JPRB, 0.24531E+00_JPRB, 0.24277E+00_JPRB, 0.23930E+00_JPRB, 0.23535E+00_JPRB, 0.23204E+00_JPRB, 0.22931E+00_JPRB ,&
&  0.22815E+00_JPRB, 0.22817E+00_JPRB, 0.22875E+00_JPRB, 0.22904E+00_JPRB, 0.22923E+00_JPRB, 0.23157E+00_JPRB, 0.23540E+00_JPRB ,&
&  0.23337E+00_JPRB, 0.23098E+00_JPRB, 0.22608E+00_JPRB, 0.22573E+00_JPRB, 0.22742E+00_JPRB, 0.23014E+00_JPRB, 0.23294E+00_JPRB ,&
&  0.23584E+00_JPRB, 0.24009E+00_JPRB, 0.24388E+00_JPRB, 0.24633E+00_JPRB, 0.24647E+00_JPRB, 0.24681E+00_JPRB, 0.24724E+00_JPRB ,&
&  0.24719E+00_JPRB, 0.24594E+00_JPRB, 0.24390E+00_JPRB, 0.24042E+00_JPRB, 0.23605E+00_JPRB, 0.23319E+00_JPRB, 0.23182E+00_JPRB ,&
&  0.23040E+00_JPRB, 0.23023E+00_JPRB, 0.23016E+00_JPRB, 0.22955E+00_JPRB, 0.22854E+00_JPRB, 0.22813E+00_JPRB, 0.22848E+00_JPRB ,&
&  0.22860E+00_JPRB, 0.22800E+00_JPRB, 0.22698E+00_JPRB, 0.22668E+00_JPRB, 0.22710E+00_JPRB, 0.22824E+00_JPRB, 0.22936E+00_JPRB ,&
&  0.23016E+00_JPRB, 0.22986E+00_JPRB, 0.22908E+00_JPRB, 0.22728E+00_JPRB, 0.22596E+00_JPRB, 0.22604E+00_JPRB, 0.22713E+00_JPRB ,&
&  0.22784E+00_JPRB, 0.22787E+00_JPRB, 0.22813E+00_JPRB, 0.22725E+00_JPRB, 0.22647E+00_JPRB, 0.22660E+00_JPRB, 0.22706E+00_JPRB ,&
&  0.22721E+00_JPRB, 0.22705E+00_JPRB, 0.22671E+00_JPRB, 0.22652E+00_JPRB, 0.22692E+00_JPRB, 0.22705E+00_JPRB, 0.22802E+00_JPRB ,&
&  0.23047E+00_JPRB, 0.23325E+00_JPRB, 0.23605E+00_JPRB, 0.23814E+00_JPRB, 0.23944E+00_JPRB, 0.24016E+00_JPRB, 0.24295E+00_JPRB ,&
&  0.24675E+00_JPRB, 0.24942E+00_JPRB, 0.24968E+00_JPRB, 0.24953E+00_JPRB, 0.25118E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.27356E+00_JPRB, 0.27412E+00_JPRB, 0.27853E+00_JPRB, 0.28364E+00_JPRB, 0.28898E+00_JPRB, 0.28990E+00_JPRB, 0.28722E+00_JPRB ,&
&  0.28425E+00_JPRB, 0.28015E+00_JPRB, 0.27434E+00_JPRB, 0.26734E+00_JPRB, 0.26100E+00_JPRB, 0.25910E+00_JPRB, 0.25908E+00_JPRB ,&
&  0.25925E+00_JPRB, 0.25771E+00_JPRB, 0.25456E+00_JPRB, 0.25096E+00_JPRB, 0.24669E+00_JPRB, 0.24286E+00_JPRB, 0.23988E+00_JPRB ,&
&  0.23873E+00_JPRB, 0.23807E+00_JPRB, 0.23832E+00_JPRB, 0.23841E+00_JPRB, 0.23918E+00_JPRB, 0.24208E+00_JPRB, 0.24743E+00_JPRB ,&
&  0.24520E+00_JPRB, 0.24573E+00_JPRB, 0.23937E+00_JPRB, 0.24193E+00_JPRB, 0.24460E+00_JPRB, 0.25032E+00_JPRB, 0.25727E+00_JPRB ,&
&  0.26450E+00_JPRB, 0.27051E+00_JPRB, 0.27351E+00_JPRB, 0.27369E+00_JPRB, 0.27078E+00_JPRB, 0.26942E+00_JPRB, 0.26854E+00_JPRB ,&
&  0.26746E+00_JPRB, 0.26626E+00_JPRB, 0.26434E+00_JPRB, 0.26077E+00_JPRB, 0.25586E+00_JPRB, 0.25153E+00_JPRB, 0.24834E+00_JPRB ,&
&  0.24744E+00_JPRB, 0.24682E+00_JPRB, 0.24527E+00_JPRB, 0.24373E+00_JPRB, 0.24207E+00_JPRB, 0.24101E+00_JPRB, 0.24134E+00_JPRB ,&
&  0.24238E+00_JPRB, 0.24331E+00_JPRB, 0.24398E+00_JPRB, 0.24514E+00_JPRB, 0.24719E+00_JPRB, 0.24917E+00_JPRB, 0.25065E+00_JPRB ,&
&  0.25117E+00_JPRB, 0.25056E+00_JPRB, 0.24982E+00_JPRB, 0.24895E+00_JPRB, 0.24841E+00_JPRB, 0.24787E+00_JPRB, 0.24786E+00_JPRB ,&
&  0.24801E+00_JPRB, 0.24796E+00_JPRB, 0.24730E+00_JPRB, 0.24611E+00_JPRB, 0.24504E+00_JPRB, 0.24416E+00_JPRB, 0.24389E+00_JPRB ,&
&  0.24421E+00_JPRB, 0.24460E+00_JPRB, 0.24450E+00_JPRB, 0.24410E+00_JPRB, 0.24437E+00_JPRB, 0.24498E+00_JPRB, 0.24638E+00_JPRB ,&
&  0.24843E+00_JPRB, 0.25107E+00_JPRB, 0.25463E+00_JPRB, 0.25843E+00_JPRB, 0.26000E+00_JPRB, 0.26212E+00_JPRB, 0.26617E+00_JPRB ,&
&  0.26992E+00_JPRB, 0.27169E+00_JPRB, 0.27265E+00_JPRB, 0.27347E+00_JPRB, 0.27430E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.29966E+00_JPRB, 0.29651E+00_JPRB, 0.29814E+00_JPRB, 0.30258E+00_JPRB, 0.30840E+00_JPRB, 0.30650E+00_JPRB, 0.30354E+00_JPRB ,&
&  0.29769E+00_JPRB, 0.29028E+00_JPRB, 0.28311E+00_JPRB, 0.27640E+00_JPRB, 0.27305E+00_JPRB, 0.27297E+00_JPRB, 0.27251E+00_JPRB ,&
&  0.27132E+00_JPRB, 0.26832E+00_JPRB, 0.26397E+00_JPRB, 0.25970E+00_JPRB, 0.25657E+00_JPRB, 0.25481E+00_JPRB, 0.25326E+00_JPRB ,&
&  0.25215E+00_JPRB, 0.25082E+00_JPRB, 0.25155E+00_JPRB, 0.25149E+00_JPRB, 0.25231E+00_JPRB, 0.25626E+00_JPRB, 0.25945E+00_JPRB ,&
&  0.26111E+00_JPRB, 0.26324E+00_JPRB, 0.26320E+00_JPRB, 0.26383E+00_JPRB, 0.26728E+00_JPRB, 0.27661E+00_JPRB, 0.28908E+00_JPRB ,&
&  0.29977E+00_JPRB, 0.30554E+00_JPRB, 0.30710E+00_JPRB, 0.30535E+00_JPRB, 0.30104E+00_JPRB, 0.29720E+00_JPRB, 0.29480E+00_JPRB ,&
&  0.29274E+00_JPRB, 0.28928E+00_JPRB, 0.28564E+00_JPRB, 0.28266E+00_JPRB, 0.28137E+00_JPRB, 0.28007E+00_JPRB, 0.27805E+00_JPRB ,&
&  0.27548E+00_JPRB, 0.27285E+00_JPRB, 0.26951E+00_JPRB, 0.26599E+00_JPRB, 0.26295E+00_JPRB, 0.26122E+00_JPRB, 0.26121E+00_JPRB ,&
&  0.26266E+00_JPRB, 0.26511E+00_JPRB, 0.26826E+00_JPRB, 0.27208E+00_JPRB, 0.27503E+00_JPRB, 0.27762E+00_JPRB, 0.27870E+00_JPRB ,&
&  0.27887E+00_JPRB, 0.27838E+00_JPRB, 0.27773E+00_JPRB, 0.27711E+00_JPRB, 0.27624E+00_JPRB, 0.27490E+00_JPRB, 0.27338E+00_JPRB ,&
&  0.27204E+00_JPRB, 0.27098E+00_JPRB, 0.27011E+00_JPRB, 0.26929E+00_JPRB, 0.26752E+00_JPRB, 0.26533E+00_JPRB, 0.26384E+00_JPRB ,&
&  0.26307E+00_JPRB, 0.26271E+00_JPRB, 0.26246E+00_JPRB, 0.26190E+00_JPRB, 0.26174E+00_JPRB, 0.26254E+00_JPRB, 0.26433E+00_JPRB ,&
&  0.26715E+00_JPRB, 0.27109E+00_JPRB, 0.27598E+00_JPRB, 0.28154E+00_JPRB, 0.28716E+00_JPRB, 0.29301E+00_JPRB, 0.29872E+00_JPRB ,&
&  0.30255E+00_JPRB, 0.30307E+00_JPRB, 0.30370E+00_JPRB, 0.30464E+00_JPRB, 0.30327E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.33096E+00_JPRB, 0.32436E+00_JPRB, 0.32195E+00_JPRB, 0.32406E+00_JPRB, 0.32645E+00_JPRB, 0.32434E+00_JPRB, 0.31921E+00_JPRB ,&
&  0.30935E+00_JPRB, 0.30049E+00_JPRB, 0.29309E+00_JPRB, 0.28922E+00_JPRB, 0.28995E+00_JPRB, 0.28967E+00_JPRB, 0.28712E+00_JPRB ,&
&  0.28461E+00_JPRB, 0.28024E+00_JPRB, 0.27664E+00_JPRB, 0.27307E+00_JPRB, 0.27036E+00_JPRB, 0.26916E+00_JPRB, 0.26869E+00_JPRB ,&
&  0.26731E+00_JPRB, 0.26574E+00_JPRB, 0.26570E+00_JPRB, 0.26614E+00_JPRB, 0.26830E+00_JPRB, 0.27293E+00_JPRB, 0.27842E+00_JPRB ,&
&  0.28709E+00_JPRB, 0.27442E+00_JPRB, 0.29101E+00_JPRB, 0.29536E+00_JPRB, 0.29994E+00_JPRB, 0.30991E+00_JPRB, 0.32139E+00_JPRB ,&
&  0.33104E+00_JPRB, 0.33581E+00_JPRB, 0.33723E+00_JPRB, 0.33676E+00_JPRB, 0.33302E+00_JPRB, 0.33003E+00_JPRB, 0.32572E+00_JPRB ,&
&  0.32109E+00_JPRB, 0.31951E+00_JPRB, 0.31882E+00_JPRB, 0.31683E+00_JPRB, 0.31452E+00_JPRB, 0.31075E+00_JPRB, 0.30629E+00_JPRB ,&
&  0.30179E+00_JPRB, 0.29735E+00_JPRB, 0.29380E+00_JPRB, 0.29060E+00_JPRB, 0.28827E+00_JPRB, 0.28731E+00_JPRB, 0.28795E+00_JPRB ,&
&  0.28914E+00_JPRB, 0.29137E+00_JPRB, 0.29680E+00_JPRB, 0.30321E+00_JPRB, 0.30839E+00_JPRB, 0.31172E+00_JPRB, 0.31367E+00_JPRB ,&
&  0.31409E+00_JPRB, 0.31364E+00_JPRB, 0.31268E+00_JPRB, 0.31133E+00_JPRB, 0.30902E+00_JPRB, 0.30655E+00_JPRB, 0.30276E+00_JPRB ,&
&  0.29954E+00_JPRB, 0.29692E+00_JPRB, 0.29413E+00_JPRB, 0.29199E+00_JPRB, 0.28950E+00_JPRB, 0.28627E+00_JPRB, 0.28316E+00_JPRB ,&
&  0.28071E+00_JPRB, 0.27896E+00_JPRB, 0.27767E+00_JPRB, 0.27720E+00_JPRB, 0.27759E+00_JPRB, 0.27974E+00_JPRB, 0.28216E+00_JPRB ,&
&  0.28533E+00_JPRB, 0.29049E+00_JPRB, 0.29731E+00_JPRB, 0.30572E+00_JPRB, 0.31619E+00_JPRB, 0.32358E+00_JPRB, 0.32969E+00_JPRB ,&
&  0.33378E+00_JPRB, 0.33442E+00_JPRB, 0.33580E+00_JPRB, 0.33587E+00_JPRB, 0.33471E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.36392E+00_JPRB, 0.35814E+00_JPRB, 0.35176E+00_JPRB, 0.35093E+00_JPRB, 0.34734E+00_JPRB, 0.34169E+00_JPRB, 0.33405E+00_JPRB ,&
&  0.32299E+00_JPRB, 0.31484E+00_JPRB, 0.30850E+00_JPRB, 0.30623E+00_JPRB, 0.30809E+00_JPRB, 0.30606E+00_JPRB, 0.30194E+00_JPRB ,&
&  0.29891E+00_JPRB, 0.29544E+00_JPRB, 0.29303E+00_JPRB, 0.29024E+00_JPRB, 0.28806E+00_JPRB, 0.28725E+00_JPRB, 0.28673E+00_JPRB ,&
&  0.28525E+00_JPRB, 0.28290E+00_JPRB, 0.28111E+00_JPRB, 0.28064E+00_JPRB, 0.28370E+00_JPRB, 0.29022E+00_JPRB, 0.30047E+00_JPRB ,&
&  0.29763E+00_JPRB, 0.28304E+00_JPRB, 0.30018E+00_JPRB, 0.32098E+00_JPRB, 0.33561E+00_JPRB, 0.35105E+00_JPRB, 0.36247E+00_JPRB ,&
&  0.37304E+00_JPRB, 0.37600E+00_JPRB, 0.37747E+00_JPRB, 0.37784E+00_JPRB, 0.37477E+00_JPRB, 0.36898E+00_JPRB, 0.36387E+00_JPRB ,&
&  0.36051E+00_JPRB, 0.35807E+00_JPRB, 0.35501E+00_JPRB, 0.34912E+00_JPRB, 0.34238E+00_JPRB, 0.33487E+00_JPRB, 0.32745E+00_JPRB ,&
&  0.32123E+00_JPRB, 0.31538E+00_JPRB, 0.31101E+00_JPRB, 0.30895E+00_JPRB, 0.30962E+00_JPRB, 0.31200E+00_JPRB, 0.31664E+00_JPRB ,&
&  0.32216E+00_JPRB, 0.33080E+00_JPRB, 0.34377E+00_JPRB, 0.35418E+00_JPRB, 0.36031E+00_JPRB, 0.36239E+00_JPRB, 0.36319E+00_JPRB ,&
&  0.36085E+00_JPRB, 0.35799E+00_JPRB, 0.35626E+00_JPRB, 0.35345E+00_JPRB, 0.34948E+00_JPRB, 0.34418E+00_JPRB, 0.33735E+00_JPRB ,&
&  0.33144E+00_JPRB, 0.32651E+00_JPRB, 0.32183E+00_JPRB, 0.31752E+00_JPRB, 0.31330E+00_JPRB, 0.30904E+00_JPRB, 0.30530E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.29773E+00_JPRB, 0.29524E+00_JPRB, 0.29527E+00_JPRB, 0.29826E+00_JPRB, 0.30211E+00_JPRB, 0.30683E+00_JPRB ,&
&  0.31206E+00_JPRB, 0.32048E+00_JPRB, 0.32895E+00_JPRB, 0.34282E+00_JPRB, 0.35573E+00_JPRB, 0.36095E+00_JPRB, 0.36547E+00_JPRB ,&
&  0.36923E+00_JPRB, 0.36799E+00_JPRB, 0.36958E+00_JPRB, 0.36806E+00_JPRB, 0.36499E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.39421E+00_JPRB, 0.39582E+00_JPRB, 0.39205E+00_JPRB, 0.38285E+00_JPRB, 0.37082E+00_JPRB, 0.36256E+00_JPRB, 0.35143E+00_JPRB ,&
&  0.33979E+00_JPRB, 0.33272E+00_JPRB, 0.32475E+00_JPRB, 0.32097E+00_JPRB, 0.32048E+00_JPRB, 0.31675E+00_JPRB, 0.31365E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.30663E+00_JPRB, 0.30385E+00_JPRB, 0.30127E+00_JPRB, 0.30102E+00_JPRB, 0.30110E+00_JPRB, 0.30090E+00_JPRB ,&
&  0.29975E+00_JPRB, 0.29756E+00_JPRB, 0.29574E+00_JPRB, 0.29483E+00_JPRB, 0.29734E+00_JPRB, 0.30441E+00_JPRB, 0.31776E+00_JPRB ,&
&  0.34655E+00_JPRB, 0.31359E+00_JPRB, 0.33046E+00_JPRB, 0.34558E+00_JPRB, 0.37401E+00_JPRB, 0.39720E+00_JPRB, 0.40832E+00_JPRB ,&
&  0.42076E+00_JPRB, 0.42995E+00_JPRB, 0.43520E+00_JPRB, 0.43815E+00_JPRB, 0.43061E+00_JPRB, 0.41916E+00_JPRB, 0.41097E+00_JPRB ,&
&  0.40168E+00_JPRB, 0.39184E+00_JPRB, 0.38215E+00_JPRB, 0.37149E+00_JPRB, 0.36447E+00_JPRB, 0.35970E+00_JPRB, 0.35408E+00_JPRB ,&
&  0.34684E+00_JPRB, 0.33940E+00_JPRB, 0.33434E+00_JPRB, 0.33324E+00_JPRB, 0.33709E+00_JPRB, 0.34585E+00_JPRB, 0.36232E+00_JPRB ,&
&  0.37832E+00_JPRB, 0.39559E+00_JPRB, 0.41170E+00_JPRB, 0.42110E+00_JPRB, 0.42527E+00_JPRB, 0.42336E+00_JPRB, 0.41845E+00_JPRB ,&
&  0.41141E+00_JPRB, 0.40449E+00_JPRB, 0.39933E+00_JPRB, 0.39258E+00_JPRB, 0.38406E+00_JPRB, 0.37698E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.36700E+00_JPRB, 0.36158E+00_JPRB, 0.35494E+00_JPRB, 0.34823E+00_JPRB, 0.34133E+00_JPRB, 0.33554E+00_JPRB, 0.33061E+00_JPRB ,&
&  0.32614E+00_JPRB, 0.32184E+00_JPRB, 0.31995E+00_JPRB, 0.32217E+00_JPRB, 0.32653E+00_JPRB, 0.33140E+00_JPRB, 0.33605E+00_JPRB ,&
&  0.34239E+00_JPRB, 0.35251E+00_JPRB, 0.36010E+00_JPRB, 0.38429E+00_JPRB, 0.39624E+00_JPRB, 0.40169E+00_JPRB, 0.40578E+00_JPRB ,&
&  0.40590E+00_JPRB, 0.40241E+00_JPRB, 0.35716E+00_JPRB, 0.38410E+00_JPRB, 0.39140E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.43624E+00_JPRB, 0.43639E+00_JPRB, 0.42980E+00_JPRB, 0.41239E+00_JPRB, 0.39503E+00_JPRB, 0.38446E+00_JPRB, 0.37280E+00_JPRB ,&
&  0.36059E+00_JPRB, 0.35049E+00_JPRB, 0.33854E+00_JPRB, 0.33364E+00_JPRB, 0.33085E+00_JPRB, 0.32945E+00_JPRB, 0.32747E+00_JPRB ,&
&  0.32335E+00_JPRB, 0.31687E+00_JPRB, 0.31182E+00_JPRB, 0.30966E+00_JPRB, 0.31008E+00_JPRB, 0.31124E+00_JPRB, 0.31260E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31172E+00_JPRB, 0.31110E+00_JPRB, 0.31118E+00_JPRB, 0.31310E+00_JPRB, 0.32086E+00_JPRB, 0.33617E+00_JPRB ,&
&  0.37457E+00_JPRB, 0.34281E+00_JPRB, 0.35752E+00_JPRB, 0.37961E+00_JPRB, 0.40495E+00_JPRB, 0.44224E+00_JPRB, 0.48304E+00_JPRB ,&
&  0.50211E+00_JPRB, 0.49935E+00_JPRB, 0.50117E+00_JPRB, 0.50840E+00_JPRB, 0.49879E+00_JPRB, 0.48702E+00_JPRB, 0.47279E+00_JPRB ,&
&  0.45443E+00_JPRB, 0.43471E+00_JPRB, 0.41516E+00_JPRB, 0.40199E+00_JPRB, 0.39615E+00_JPRB, 0.39288E+00_JPRB, 0.39101E+00_JPRB ,&
&  0.38435E+00_JPRB, 0.38073E+00_JPRB, 0.38528E+00_JPRB, 0.39221E+00_JPRB, 0.40048E+00_JPRB, 0.41680E+00_JPRB, 0.44073E+00_JPRB ,&
&  0.45736E+00_JPRB, 0.47440E+00_JPRB, 0.48711E+00_JPRB, 0.49224E+00_JPRB, 0.49581E+00_JPRB, 0.49170E+00_JPRB, 0.47736E+00_JPRB ,&
&  0.46243E+00_JPRB, 0.44678E+00_JPRB, 0.43315E+00_JPRB, 0.42101E+00_JPRB, 0.41211E+00_JPRB, 0.40815E+00_JPRB, 0.40570E+00_JPRB ,&
&  0.40352E+00_JPRB, 0.40169E+00_JPRB, 0.39755E+00_JPRB, 0.39191E+00_JPRB, 0.38684E+00_JPRB, 0.38315E+00_JPRB, 0.37983E+00_JPRB ,&
&  0.37678E+00_JPRB, 0.37343E+00_JPRB, 0.37272E+00_JPRB, 0.37538E+00_JPRB, 0.37800E+00_JPRB, 0.38180E+00_JPRB, 0.38690E+00_JPRB ,&
&  0.39121E+00_JPRB, 0.40135E+00_JPRB, 0.40592E+00_JPRB, 0.46008E+00_JPRB, 0.47022E+00_JPRB, 0.46711E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.45234E+00_JPRB, 0.44288E+00_JPRB, 0.43536E+00_JPRB, 0.40721E+00_JPRB, 0.43267E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.48819E+00_JPRB, 0.48129E+00_JPRB, 0.46105E+00_JPRB, 0.43624E+00_JPRB, 0.41722E+00_JPRB, 0.40078E+00_JPRB, 0.38722E+00_JPRB ,&
&  0.37821E+00_JPRB, 0.36791E+00_JPRB, 0.35638E+00_JPRB, 0.34806E+00_JPRB, 0.34642E+00_JPRB, 0.34359E+00_JPRB, 0.33965E+00_JPRB ,&
&  0.33300E+00_JPRB, 0.32670E+00_JPRB, 0.32252E+00_JPRB, 0.32095E+00_JPRB, 0.32126E+00_JPRB, 0.32296E+00_JPRB, 0.32402E+00_JPRB ,&
&  0.32277E+00_JPRB, 0.31969E+00_JPRB, 0.32014E+00_JPRB, 0.32279E+00_JPRB, 0.32809E+00_JPRB, 0.33601E+00_JPRB, 0.35470E+00_JPRB ,&
&  0.39809E+00_JPRB, 0.38273E+00_JPRB, 0.40498E+00_JPRB, 0.41784E+00_JPRB, 0.43975E+00_JPRB, 0.48096E+00_JPRB, 0.52544E+00_JPRB ,&
&  0.55435E+00_JPRB, 0.56031E+00_JPRB, 0.57417E+00_JPRB, 0.58671E+00_JPRB, 0.57678E+00_JPRB, 0.55550E+00_JPRB, 0.53018E+00_JPRB ,&
&  0.50570E+00_JPRB, 0.48729E+00_JPRB, 0.47153E+00_JPRB, 0.45893E+00_JPRB, 0.45043E+00_JPRB, 0.44662E+00_JPRB, 0.44623E+00_JPRB ,&
&  0.44097E+00_JPRB, 0.44013E+00_JPRB, 0.44836E+00_JPRB, 0.46541E+00_JPRB, 0.48322E+00_JPRB, 0.50086E+00_JPRB, 0.51905E+00_JPRB ,&
&  0.53347E+00_JPRB, 0.54484E+00_JPRB, 0.55151E+00_JPRB, 0.55215E+00_JPRB, 0.54369E+00_JPRB, 0.53211E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.48419E+00_JPRB, 0.46544E+00_JPRB, 0.45518E+00_JPRB, 0.44792E+00_JPRB, 0.44282E+00_JPRB, 0.43979E+00_JPRB, 0.43742E+00_JPRB ,&
&  0.43633E+00_JPRB, 0.43765E+00_JPRB, 0.43853E+00_JPRB, 0.43878E+00_JPRB, 0.43898E+00_JPRB, 0.43938E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.43878E+00_JPRB, 0.43863E+00_JPRB, 0.43951E+00_JPRB, 0.43937E+00_JPRB, 0.43857E+00_JPRB, 0.43794E+00_JPRB, 0.43836E+00_JPRB ,&
&  0.43975E+00_JPRB, 0.45086E+00_JPRB, 0.45695E+00_JPRB, 0.48921E+00_JPRB, 0.49876E+00_JPRB, 0.50095E+00_JPRB, 0.49905E+00_JPRB ,&
&  0.48946E+00_JPRB, 0.47851E+00_JPRB, 0.46180E+00_JPRB, 0.45223E+00_JPRB, 0.46659E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.57426E+00_JPRB, 0.53959E+00_JPRB, 0.49589E+00_JPRB, 0.46185E+00_JPRB, 0.43761E+00_JPRB, 0.41934E+00_JPRB, 0.40831E+00_JPRB ,&
&  0.40371E+00_JPRB, 0.39311E+00_JPRB, 0.37925E+00_JPRB, 0.36784E+00_JPRB, 0.36213E+00_JPRB, 0.35768E+00_JPRB, 0.35163E+00_JPRB ,&
&  0.34636E+00_JPRB, 0.34170E+00_JPRB, 0.33877E+00_JPRB, 0.33758E+00_JPRB, 0.33687E+00_JPRB, 0.33591E+00_JPRB, 0.33294E+00_JPRB ,&
&  0.32839E+00_JPRB, 0.32795E+00_JPRB, 0.32987E+00_JPRB, 0.33655E+00_JPRB, 0.34526E+00_JPRB, 0.35510E+00_JPRB, 0.37338E+00_JPRB ,&
&  0.42638E+00_JPRB, 0.42975E+00_JPRB, 0.47034E+00_JPRB, 0.48659E+00_JPRB, 0.51594E+00_JPRB, 0.56315E+00_JPRB, 0.59316E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.64014E+00_JPRB, 0.67159E+00_JPRB, 0.68053E+00_JPRB, 0.65902E+00_JPRB, 0.62738E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.57180E+00_JPRB, 0.55194E+00_JPRB, 0.53302E+00_JPRB, 0.51594E+00_JPRB, 0.50457E+00_JPRB, 0.50312E+00_JPRB, 0.50823E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.50363E+00_JPRB, 0.51594E+00_JPRB, 0.53491E+00_JPRB, 0.56218E+00_JPRB, 0.58052E+00_JPRB, 0.59545E+00_JPRB ,&
&  0.61193E+00_JPRB, 0.61644E+00_JPRB, 0.61531E+00_JPRB, 0.60271E+00_JPRB, 0.57356E+00_JPRB, 0.54064E+00_JPRB, 0.50964E+00_JPRB ,&
&  0.48077E+00_JPRB, 0.46247E+00_JPRB, 0.45460E+00_JPRB, 0.45109E+00_JPRB, 0.45153E+00_JPRB, 0.45469E+00_JPRB, 0.46160E+00_JPRB ,&
&  0.47252E+00_JPRB, 0.48252E+00_JPRB, 0.48992E+00_JPRB, 0.49286E+00_JPRB, 0.49122E+00_JPRB, 0.48830E+00_JPRB, 0.48386E+00_JPRB ,&
&  0.47850E+00_JPRB, 0.47230E+00_JPRB, 0.46978E+00_JPRB, 0.46868E+00_JPRB, 0.46869E+00_JPRB, 0.46974E+00_JPRB, 0.47187E+00_JPRB ,&
&  0.47523E+00_JPRB, 0.49211E+00_JPRB, 0.49390E+00_JPRB, 0.54695E+00_JPRB, 0.57094E+00_JPRB, 0.55968E+00_JPRB, 0.54482E+00_JPRB ,&
&  0.52584E+00_JPRB, 0.50944E+00_JPRB, 0.49522E+00_JPRB, 0.48956E+00_JPRB, 0.55990E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.52415E+00_JPRB, 0.50947E+00_JPRB, 0.48325E+00_JPRB, 0.45993E+00_JPRB, 0.44467E+00_JPRB, 0.43349E+00_JPRB, 0.42714E+00_JPRB ,&
&  0.42339E+00_JPRB, 0.41497E+00_JPRB, 0.40671E+00_JPRB, 0.39561E+00_JPRB, 0.38626E+00_JPRB, 0.37834E+00_JPRB, 0.37079E+00_JPRB ,&
&  0.36418E+00_JPRB, 0.36011E+00_JPRB, 0.35680E+00_JPRB, 0.35370E+00_JPRB, 0.34892E+00_JPRB, 0.34186E+00_JPRB, 0.33765E+00_JPRB ,&
&  0.33627E+00_JPRB, 0.33756E+00_JPRB, 0.34200E+00_JPRB, 0.35067E+00_JPRB, 0.35951E+00_JPRB, 0.36931E+00_JPRB, 0.38684E+00_JPRB ,&
&  0.43360E+00_JPRB, 0.53849E+00_JPRB, 0.58634E+00_JPRB, 0.60664E+00_JPRB, 0.68191E+00_JPRB, 0.67729E+00_JPRB, 0.68173E+00_JPRB ,&
&  0.71329E+00_JPRB, 0.74219E+00_JPRB, 0.77048E+00_JPRB, 0.77943E+00_JPRB, 0.75890E+00_JPRB, 0.73607E+00_JPRB, 0.70356E+00_JPRB ,&
&  0.66936E+00_JPRB, 0.63197E+00_JPRB, 0.59521E+00_JPRB, 0.56113E+00_JPRB, 0.53203E+00_JPRB, 0.52695E+00_JPRB, 0.53762E+00_JPRB ,&
&  0.55787E+00_JPRB, 0.57175E+00_JPRB, 0.57736E+00_JPRB, 0.59653E+00_JPRB, 0.63184E+00_JPRB, 0.67421E+00_JPRB, 0.71715E+00_JPRB ,&
&  0.73556E+00_JPRB, 0.71449E+00_JPRB, 0.68080E+00_JPRB, 0.63838E+00_JPRB, 0.58531E+00_JPRB, 0.53746E+00_JPRB, 0.50166E+00_JPRB ,&
&  0.47613E+00_JPRB, 0.45878E+00_JPRB, 0.44791E+00_JPRB, 0.44422E+00_JPRB, 0.44909E+00_JPRB, 0.46226E+00_JPRB, 0.47778E+00_JPRB ,&
&  0.49459E+00_JPRB, 0.50945E+00_JPRB, 0.51693E+00_JPRB, 0.51847E+00_JPRB, 0.51287E+00_JPRB, 0.50148E+00_JPRB, 0.48423E+00_JPRB ,&
&  0.46910E+00_JPRB, 0.46042E+00_JPRB, 0.46942E+00_JPRB, 0.47407E+00_JPRB, 0.47719E+00_JPRB, 0.48073E+00_JPRB, 0.48560E+00_JPRB ,&
&  0.49627E+00_JPRB, 0.52264E+00_JPRB, 0.52403E+00_JPRB, 0.53360E+00_JPRB, 0.55961E+00_JPRB, 0.57914E+00_JPRB, 0.55969E+00_JPRB ,&
&  0.54283E+00_JPRB, 0.52584E+00_JPRB, 0.51427E+00_JPRB, 0.50558E+00_JPRB, 0.52530E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.51422E+00_JPRB, 0.50666E+00_JPRB, 0.49012E+00_JPRB, 0.47225E+00_JPRB, 0.45992E+00_JPRB, 0.45026E+00_JPRB, 0.44330E+00_JPRB ,&
&  0.43880E+00_JPRB, 0.43081E+00_JPRB, 0.42367E+00_JPRB, 0.41468E+00_JPRB, 0.40412E+00_JPRB, 0.39528E+00_JPRB, 0.38959E+00_JPRB ,&
&  0.38204E+00_JPRB, 0.37780E+00_JPRB, 0.37327E+00_JPRB, 0.36494E+00_JPRB, 0.35615E+00_JPRB, 0.34890E+00_JPRB, 0.34424E+00_JPRB ,&
&  0.34264E+00_JPRB, 0.34545E+00_JPRB, 0.35151E+00_JPRB, 0.36417E+00_JPRB, 0.37509E+00_JPRB, 0.38577E+00_JPRB, 0.40669E+00_JPRB ,&
&  0.47397E+00_JPRB, 0.62692E+00_JPRB, 0.81287E+00_JPRB, 0.75950E+00_JPRB, 0.75242E+00_JPRB, 0.70802E+00_JPRB, 0.77072E+00_JPRB ,&
&  0.81600E+00_JPRB, 0.86974E+00_JPRB, 0.90270E+00_JPRB, 0.91396E+00_JPRB, 0.87336E+00_JPRB, 0.83935E+00_JPRB, 0.80191E+00_JPRB ,&
&  0.76235E+00_JPRB, 0.71772E+00_JPRB, 0.66088E+00_JPRB, 0.60520E+00_JPRB, 0.57227E+00_JPRB, 0.56967E+00_JPRB, 0.58711E+00_JPRB ,&
&  0.61470E+00_JPRB, 0.63462E+00_JPRB, 0.63616E+00_JPRB, 0.65562E+00_JPRB, 0.67104E+00_JPRB, 0.68113E+00_JPRB, 0.77396E+00_JPRB ,&
&  0.94066E+00_JPRB, 0.88740E+00_JPRB, 0.79728E+00_JPRB, 0.68747E+00_JPRB, 0.60623E+00_JPRB, 0.54440E+00_JPRB, 0.50975E+00_JPRB ,&
&  0.48540E+00_JPRB, 0.46934E+00_JPRB, 0.45955E+00_JPRB, 0.45659E+00_JPRB, 0.46402E+00_JPRB, 0.47684E+00_JPRB, 0.48649E+00_JPRB ,&
&  0.49736E+00_JPRB, 0.50636E+00_JPRB, 0.50984E+00_JPRB, 0.50596E+00_JPRB, 0.49624E+00_JPRB, 0.48242E+00_JPRB, 0.47012E+00_JPRB ,&
&  0.46583E+00_JPRB, 0.47182E+00_JPRB, 0.49842E+00_JPRB, 0.48899E+00_JPRB, 0.49252E+00_JPRB, 0.50386E+00_JPRB, 0.50679E+00_JPRB ,&
&  0.50601E+00_JPRB, 0.51785E+00_JPRB, 0.52990E+00_JPRB, 0.53551E+00_JPRB, 0.57287E+00_JPRB, 0.61328E+00_JPRB, 0.58192E+00_JPRB ,&
&  0.55155E+00_JPRB, 0.52977E+00_JPRB, 0.51962E+00_JPRB, 0.50899E+00_JPRB, 0.51188E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.51635E+00_JPRB, 0.51120E+00_JPRB, 0.49793E+00_JPRB, 0.48188E+00_JPRB, 0.46966E+00_JPRB, 0.45749E+00_JPRB, 0.44840E+00_JPRB ,&
&  0.44168E+00_JPRB, 0.43606E+00_JPRB, 0.43187E+00_JPRB, 0.42689E+00_JPRB, 0.42027E+00_JPRB, 0.41127E+00_JPRB, 0.40257E+00_JPRB ,&
&  0.39316E+00_JPRB, 0.38382E+00_JPRB, 0.37471E+00_JPRB, 0.36536E+00_JPRB, 0.35846E+00_JPRB, 0.35371E+00_JPRB, 0.35055E+00_JPRB ,&
&  0.35059E+00_JPRB, 0.35390E+00_JPRB, 0.36194E+00_JPRB, 0.37810E+00_JPRB, 0.39647E+00_JPRB, 0.40726E+00_JPRB, 0.42116E+00_JPRB ,&
&  0.47729E+00_JPRB, 0.42733E+00_JPRB, 0.73486E+00_JPRB, 0.85761E+00_JPRB, 0.84823E+00_JPRB, 0.84496E+00_JPRB, 0.93642E+00_JPRB ,&
&  0.10105E+01_JPRB, 0.10168E+01_JPRB, 0.10063E+01_JPRB, 0.96757E+00_JPRB, 0.88463E+00_JPRB, 0.81756E+00_JPRB, 0.76331E+00_JPRB ,&
&  0.71885E+00_JPRB, 0.68087E+00_JPRB, 0.63828E+00_JPRB, 0.59580E+00_JPRB, 0.56428E+00_JPRB, 0.55833E+00_JPRB, 0.56937E+00_JPRB ,&
&  0.60922E+00_JPRB, 0.63992E+00_JPRB, 0.66968E+00_JPRB, 0.70532E+00_JPRB, 0.69810E+00_JPRB, 0.76725E+00_JPRB, 0.79684E+00_JPRB ,&
&  0.95887E+00_JPRB, 0.10367E+01_JPRB, 0.87169E+00_JPRB, 0.70758E+00_JPRB, 0.60236E+00_JPRB, 0.55077E+00_JPRB, 0.51270E+00_JPRB ,&
&  0.49366E+00_JPRB, 0.48272E+00_JPRB, 0.47583E+00_JPRB, 0.47380E+00_JPRB, 0.47774E+00_JPRB, 0.48364E+00_JPRB, 0.48584E+00_JPRB ,&
&  0.48871E+00_JPRB, 0.49107E+00_JPRB, 0.49160E+00_JPRB, 0.49048E+00_JPRB, 0.48527E+00_JPRB, 0.47774E+00_JPRB, 0.47599E+00_JPRB ,&
&  0.48345E+00_JPRB, 0.49999E+00_JPRB, 0.51386E+00_JPRB, 0.51225E+00_JPRB, 0.51721E+00_JPRB, 0.52354E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.53710E+00_JPRB, 0.54000E+00_JPRB, 0.54156E+00_JPRB, 0.55808E+00_JPRB, 0.55832E+00_JPRB, 0.60012E+00_JPRB, 0.56765E+00_JPRB ,&
&  0.54293E+00_JPRB, 0.52758E+00_JPRB, 0.51879E+00_JPRB, 0.51366E+00_JPRB, 0.51355E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.52183E+00_JPRB, 0.51685E+00_JPRB, 0.50451E+00_JPRB, 0.48886E+00_JPRB, 0.47290E+00_JPRB, 0.45965E+00_JPRB, 0.45361E+00_JPRB ,&
&  0.44684E+00_JPRB, 0.43973E+00_JPRB, 0.43315E+00_JPRB, 0.42970E+00_JPRB, 0.42828E+00_JPRB, 0.42123E+00_JPRB, 0.40882E+00_JPRB ,&
&  0.39561E+00_JPRB, 0.38302E+00_JPRB, 0.37284E+00_JPRB, 0.36765E+00_JPRB, 0.36201E+00_JPRB, 0.35916E+00_JPRB, 0.35922E+00_JPRB ,&
&  0.36054E+00_JPRB, 0.36664E+00_JPRB, 0.37904E+00_JPRB, 0.39940E+00_JPRB, 0.41629E+00_JPRB, 0.42338E+00_JPRB, 0.43147E+00_JPRB ,&
&  0.48038E+00_JPRB, 0.68615E+00_JPRB, 0.65293E+00_JPRB, 0.10005E+01_JPRB, 0.10464E+01_JPRB, 0.10731E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10960E+01_JPRB, 0.11112E+01_JPRB, 0.10588E+01_JPRB, 0.97602E+00_JPRB, 0.86479E+00_JPRB, 0.77813E+00_JPRB, 0.70296E+00_JPRB ,&
&  0.64316E+00_JPRB, 0.60024E+00_JPRB, 0.56963E+00_JPRB, 0.54647E+00_JPRB, 0.53141E+00_JPRB, 0.53105E+00_JPRB, 0.55279E+00_JPRB ,&
&  0.60044E+00_JPRB, 0.65911E+00_JPRB, 0.72232E+00_JPRB, 0.77784E+00_JPRB, 0.75224E+00_JPRB, 0.82845E+00_JPRB, 0.89282E+00_JPRB ,&
&  0.95681E+00_JPRB, 0.10288E+01_JPRB, 0.82400E+00_JPRB, 0.65583E+00_JPRB, 0.58037E+00_JPRB, 0.54878E+00_JPRB, 0.52639E+00_JPRB ,&
&  0.51299E+00_JPRB, 0.50412E+00_JPRB, 0.49798E+00_JPRB, 0.49320E+00_JPRB, 0.49018E+00_JPRB, 0.48900E+00_JPRB, 0.48778E+00_JPRB ,&
&  0.48791E+00_JPRB, 0.48921E+00_JPRB, 0.49128E+00_JPRB, 0.49289E+00_JPRB, 0.49159E+00_JPRB, 0.49359E+00_JPRB, 0.50231E+00_JPRB ,&
&  0.51666E+00_JPRB, 0.53226E+00_JPRB, 0.53690E+00_JPRB, 0.53446E+00_JPRB, 0.54034E+00_JPRB, 0.54688E+00_JPRB, 0.54463E+00_JPRB ,&
&  0.55695E+00_JPRB, 0.56368E+00_JPRB, 0.56676E+00_JPRB, 0.57554E+00_JPRB, 0.58344E+00_JPRB, 0.63638E+00_JPRB, 0.56815E+00_JPRB ,&
&  0.54328E+00_JPRB, 0.53159E+00_JPRB, 0.52704E+00_JPRB, 0.52070E+00_JPRB, 0.52063E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.52584E+00_JPRB, 0.51411E+00_JPRB, 0.50123E+00_JPRB, 0.48696E+00_JPRB, 0.47149E+00_JPRB, 0.46075E+00_JPRB, 0.45572E+00_JPRB ,&
&  0.44674E+00_JPRB, 0.43871E+00_JPRB, 0.43274E+00_JPRB, 0.43041E+00_JPRB, 0.42836E+00_JPRB, 0.42091E+00_JPRB, 0.40698E+00_JPRB ,&
&  0.39288E+00_JPRB, 0.38350E+00_JPRB, 0.37876E+00_JPRB, 0.37385E+00_JPRB, 0.36832E+00_JPRB, 0.36540E+00_JPRB, 0.36668E+00_JPRB ,&
&  0.37227E+00_JPRB, 0.38134E+00_JPRB, 0.39949E+00_JPRB, 0.42073E+00_JPRB, 0.43888E+00_JPRB, 0.45387E+00_JPRB, 0.46535E+00_JPRB ,&
&  0.54431E+00_JPRB, 0.65296E+00_JPRB, 0.71364E+00_JPRB, 0.12476E+01_JPRB, 0.13031E+01_JPRB, 0.12489E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11948E+01_JPRB, 0.10875E+01_JPRB, 0.95835E+00_JPRB, 0.82686E+00_JPRB, 0.73557E+00_JPRB, 0.66309E+00_JPRB ,&
&  0.60352E+00_JPRB, 0.56563E+00_JPRB, 0.54786E+00_JPRB, 0.54073E+00_JPRB, 0.53713E+00_JPRB, 0.54447E+00_JPRB, 0.57153E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.69803E+00_JPRB, 0.80198E+00_JPRB, 0.85290E+00_JPRB, 0.85127E+00_JPRB, 0.87882E+00_JPRB, 0.87800E+00_JPRB ,&
&  0.94193E+00_JPRB, 0.92250E+00_JPRB, 0.73379E+00_JPRB, 0.62708E+00_JPRB, 0.57469E+00_JPRB, 0.56244E+00_JPRB, 0.54183E+00_JPRB ,&
&  0.53205E+00_JPRB, 0.52335E+00_JPRB, 0.51634E+00_JPRB, 0.50780E+00_JPRB, 0.49916E+00_JPRB, 0.49542E+00_JPRB, 0.49496E+00_JPRB ,&
&  0.49564E+00_JPRB, 0.49875E+00_JPRB, 0.50090E+00_JPRB, 0.50319E+00_JPRB, 0.50676E+00_JPRB, 0.51556E+00_JPRB, 0.52867E+00_JPRB ,&
&  0.54074E+00_JPRB, 0.56050E+00_JPRB, 0.55936E+00_JPRB, 0.55565E+00_JPRB, 0.57011E+00_JPRB, 0.57952E+00_JPRB, 0.56876E+00_JPRB ,&
&  0.57340E+00_JPRB, 0.58950E+00_JPRB, 0.59361E+00_JPRB, 0.58159E+00_JPRB, 0.59841E+00_JPRB, 0.62154E+00_JPRB, 0.56746E+00_JPRB ,&
&  0.55031E+00_JPRB, 0.54268E+00_JPRB, 0.54300E+00_JPRB, 0.53661E+00_JPRB, 0.53104E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.52507E+00_JPRB, 0.50950E+00_JPRB, 0.49710E+00_JPRB, 0.48770E+00_JPRB, 0.47583E+00_JPRB, 0.46358E+00_JPRB, 0.45431E+00_JPRB ,&
&  0.44020E+00_JPRB, 0.43128E+00_JPRB, 0.42502E+00_JPRB, 0.41786E+00_JPRB, 0.41089E+00_JPRB, 0.40447E+00_JPRB, 0.39886E+00_JPRB ,&
&  0.39478E+00_JPRB, 0.39145E+00_JPRB, 0.38919E+00_JPRB, 0.38498E+00_JPRB, 0.38167E+00_JPRB, 0.38142E+00_JPRB, 0.38460E+00_JPRB ,&
&  0.39789E+00_JPRB, 0.41704E+00_JPRB, 0.43997E+00_JPRB, 0.46480E+00_JPRB, 0.48792E+00_JPRB, 0.51783E+00_JPRB, 0.55417E+00_JPRB ,&
&  0.68675E+00_JPRB, 0.73719E+00_JPRB, 0.88463E+00_JPRB, 0.14117E+01_JPRB, 0.14508E+01_JPRB, 0.12770E+01_JPRB, 0.11958E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.10240E+01_JPRB, 0.10334E+01_JPRB, 0.89654E+00_JPRB, 0.77146E+00_JPRB, 0.70435E+00_JPRB, 0.65653E+00_JPRB ,&
&  0.61414E+00_JPRB, 0.58949E+00_JPRB, 0.58298E+00_JPRB, 0.57789E+00_JPRB, 0.57151E+00_JPRB, 0.58759E+00_JPRB, 0.62667E+00_JPRB ,&
&  0.70255E+00_JPRB, 0.80860E+00_JPRB, 0.95407E+00_JPRB, 0.10430E+01_JPRB, 0.92407E+00_JPRB, 0.88468E+00_JPRB, 0.82949E+00_JPRB ,&
&  0.85758E+00_JPRB, 0.79433E+00_JPRB, 0.68129E+00_JPRB, 0.62074E+00_JPRB, 0.57930E+00_JPRB, 0.57756E+00_JPRB, 0.55545E+00_JPRB ,&
&  0.54523E+00_JPRB, 0.53579E+00_JPRB, 0.52564E+00_JPRB, 0.51158E+00_JPRB, 0.50276E+00_JPRB, 0.49605E+00_JPRB, 0.49577E+00_JPRB ,&
&  0.50080E+00_JPRB, 0.50661E+00_JPRB, 0.51264E+00_JPRB, 0.52056E+00_JPRB, 0.53091E+00_JPRB, 0.54052E+00_JPRB, 0.55066E+00_JPRB ,&
&  0.56155E+00_JPRB, 0.58448E+00_JPRB, 0.58272E+00_JPRB, 0.59567E+00_JPRB, 0.61827E+00_JPRB, 0.62903E+00_JPRB, 0.60673E+00_JPRB ,&
&  0.59432E+00_JPRB, 0.60919E+00_JPRB, 0.60799E+00_JPRB, 0.60761E+00_JPRB, 0.61489E+00_JPRB, 0.61469E+00_JPRB, 0.57801E+00_JPRB ,&
&  0.57280E+00_JPRB, 0.57384E+00_JPRB, 0.57575E+00_JPRB, 0.55889E+00_JPRB, 0.54261E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.52597E+00_JPRB, 0.50475E+00_JPRB, 0.48927E+00_JPRB, 0.47419E+00_JPRB, 0.45841E+00_JPRB, 0.44173E+00_JPRB, 0.42866E+00_JPRB ,&
&  0.41293E+00_JPRB, 0.41003E+00_JPRB, 0.40671E+00_JPRB, 0.40173E+00_JPRB, 0.39597E+00_JPRB, 0.39664E+00_JPRB, 0.40313E+00_JPRB ,&
&  0.40874E+00_JPRB, 0.41088E+00_JPRB, 0.41360E+00_JPRB, 0.41567E+00_JPRB, 0.42081E+00_JPRB, 0.43139E+00_JPRB, 0.44725E+00_JPRB ,&
&  0.46831E+00_JPRB, 0.49240E+00_JPRB, 0.52481E+00_JPRB, 0.56473E+00_JPRB, 0.60459E+00_JPRB, 0.66381E+00_JPRB, 0.74040E+00_JPRB ,&
&  0.89934E+00_JPRB, 0.89711E+00_JPRB, 0.58294E+00_JPRB, 0.96809E+00_JPRB, 0.12000E+01_JPRB, 0.11129E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.99664E+00_JPRB, 0.93123E+00_JPRB, 0.90423E+00_JPRB, 0.83290E+00_JPRB, 0.74310E+00_JPRB, 0.71817E+00_JPRB, 0.69959E+00_JPRB ,&
&  0.67422E+00_JPRB, 0.65108E+00_JPRB, 0.63870E+00_JPRB, 0.63486E+00_JPRB, 0.63084E+00_JPRB, 0.65239E+00_JPRB, 0.73219E+00_JPRB ,&
&  0.84168E+00_JPRB, 0.99493E+00_JPRB, 0.11807E+01_JPRB, 0.11423E+01_JPRB, 0.96529E+00_JPRB, 0.86101E+00_JPRB, 0.76569E+00_JPRB ,&
&  0.71989E+00_JPRB, 0.71549E+00_JPRB, 0.66464E+00_JPRB, 0.62546E+00_JPRB, 0.58752E+00_JPRB, 0.56337E+00_JPRB, 0.56327E+00_JPRB ,&
&  0.54852E+00_JPRB, 0.53364E+00_JPRB, 0.52084E+00_JPRB, 0.50234E+00_JPRB, 0.49547E+00_JPRB, 0.49332E+00_JPRB, 0.50053E+00_JPRB ,&
&  0.51032E+00_JPRB, 0.51919E+00_JPRB, 0.52982E+00_JPRB, 0.54226E+00_JPRB, 0.55018E+00_JPRB, 0.55963E+00_JPRB, 0.56888E+00_JPRB ,&
&  0.58304E+00_JPRB, 0.60569E+00_JPRB, 0.63019E+00_JPRB, 0.64664E+00_JPRB, 0.65598E+00_JPRB, 0.65940E+00_JPRB, 0.65334E+00_JPRB ,&
&  0.65520E+00_JPRB, 0.65413E+00_JPRB, 0.61656E+00_JPRB, 0.61148E+00_JPRB, 0.63682E+00_JPRB, 0.63673E+00_JPRB, 0.63813E+00_JPRB ,&
&  0.67322E+00_JPRB, 0.70367E+00_JPRB, 0.70027E+00_JPRB, 0.59193E+00_JPRB, 0.55307E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.49664E+00_JPRB, 0.46367E+00_JPRB, 0.43485E+00_JPRB, 0.41957E+00_JPRB, 0.40895E+00_JPRB, 0.40440E+00_JPRB, 0.40263E+00_JPRB ,&
&  0.40449E+00_JPRB, 0.40522E+00_JPRB, 0.39874E+00_JPRB, 0.40277E+00_JPRB, 0.41391E+00_JPRB, 0.42600E+00_JPRB, 0.44203E+00_JPRB ,&
&  0.45025E+00_JPRB, 0.45650E+00_JPRB, 0.46612E+00_JPRB, 0.48586E+00_JPRB, 0.51170E+00_JPRB, 0.53837E+00_JPRB, 0.56276E+00_JPRB ,&
&  0.58918E+00_JPRB, 0.62008E+00_JPRB, 0.66316E+00_JPRB, 0.72258E+00_JPRB, 0.80480E+00_JPRB, 0.91715E+00_JPRB, 0.10415E+01_JPRB ,&
&  0.11941E+01_JPRB, 0.91296E+00_JPRB, 0.67329E+00_JPRB, 0.10681E+01_JPRB, 0.11243E+01_JPRB, 0.10474E+01_JPRB, 0.90593E+00_JPRB ,&
&  0.88842E+00_JPRB, 0.89331E+00_JPRB, 0.85553E+00_JPRB, 0.82807E+00_JPRB, 0.76293E+00_JPRB, 0.75002E+00_JPRB, 0.74111E+00_JPRB ,&
&  0.72344E+00_JPRB, 0.70332E+00_JPRB, 0.71143E+00_JPRB, 0.72336E+00_JPRB, 0.73859E+00_JPRB, 0.79875E+00_JPRB, 0.91188E+00_JPRB ,&
&  0.10560E+01_JPRB, 0.12481E+01_JPRB, 0.14914E+01_JPRB, 0.13073E+01_JPRB, 0.10647E+01_JPRB, 0.86591E+00_JPRB, 0.76242E+00_JPRB ,&
&  0.69408E+00_JPRB, 0.69300E+00_JPRB, 0.66773E+00_JPRB, 0.63414E+00_JPRB, 0.59707E+00_JPRB, 0.56199E+00_JPRB, 0.55195E+00_JPRB ,&
&  0.53544E+00_JPRB, 0.51590E+00_JPRB, 0.50201E+00_JPRB, 0.48848E+00_JPRB, 0.48863E+00_JPRB, 0.49668E+00_JPRB, 0.50589E+00_JPRB ,&
&  0.51520E+00_JPRB, 0.52508E+00_JPRB, 0.54095E+00_JPRB, 0.56019E+00_JPRB, 0.57085E+00_JPRB, 0.57849E+00_JPRB, 0.59173E+00_JPRB ,&
&  0.60482E+00_JPRB, 0.61854E+00_JPRB, 0.64326E+00_JPRB, 0.65891E+00_JPRB, 0.66193E+00_JPRB, 0.64394E+00_JPRB, 0.65254E+00_JPRB ,&
&  0.65199E+00_JPRB, 0.65709E+00_JPRB, 0.64195E+00_JPRB, 0.65229E+00_JPRB, 0.65054E+00_JPRB, 0.67105E+00_JPRB, 0.69493E+00_JPRB ,&
&  0.71428E+00_JPRB, 0.71028E+00_JPRB, 0.67504E+00_JPRB, 0.56659E+00_JPRB, 0.52718E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.41825E+00_JPRB, 0.39678E+00_JPRB, 0.39905E+00_JPRB, 0.40370E+00_JPRB, 0.41741E+00_JPRB, 0.43004E+00_JPRB, 0.42504E+00_JPRB ,&
&  0.42982E+00_JPRB, 0.43664E+00_JPRB, 0.44801E+00_JPRB, 0.46068E+00_JPRB, 0.47726E+00_JPRB, 0.49430E+00_JPRB, 0.50520E+00_JPRB ,&
&  0.52466E+00_JPRB, 0.54993E+00_JPRB, 0.58180E+00_JPRB, 0.61350E+00_JPRB, 0.64499E+00_JPRB, 0.66989E+00_JPRB, 0.69352E+00_JPRB ,&
&  0.72374E+00_JPRB, 0.77475E+00_JPRB, 0.84451E+00_JPRB, 0.92851E+00_JPRB, 0.10325E+01_JPRB, 0.11565E+01_JPRB, 0.12334E+01_JPRB ,&
&  0.92933E+00_JPRB, 0.55085E+00_JPRB, 0.10540E+01_JPRB, 0.11685E+01_JPRB, 0.10225E+01_JPRB, 0.95108E+00_JPRB, 0.88336E+00_JPRB ,&
&  0.83661E+00_JPRB, 0.83955E+00_JPRB, 0.80413E+00_JPRB, 0.81834E+00_JPRB, 0.78017E+00_JPRB, 0.77573E+00_JPRB, 0.78169E+00_JPRB ,&
&  0.78274E+00_JPRB, 0.79493E+00_JPRB, 0.83509E+00_JPRB, 0.86814E+00_JPRB, 0.91918E+00_JPRB, 0.10113E+01_JPRB, 0.11488E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.15923E+01_JPRB, 0.18545E+01_JPRB, 0.14831E+01_JPRB, 0.12025E+01_JPRB, 0.10271E+01_JPRB, 0.88646E+00_JPRB ,&
&  0.79485E+00_JPRB, 0.67503E+00_JPRB, 0.65468E+00_JPRB, 0.64099E+00_JPRB, 0.60715E+00_JPRB, 0.54819E+00_JPRB, 0.53385E+00_JPRB ,&
&  0.50575E+00_JPRB, 0.49011E+00_JPRB, 0.48210E+00_JPRB, 0.47600E+00_JPRB, 0.47537E+00_JPRB, 0.48254E+00_JPRB, 0.49166E+00_JPRB ,&
&  0.50577E+00_JPRB, 0.51404E+00_JPRB, 0.52983E+00_JPRB, 0.54880E+00_JPRB, 0.56347E+00_JPRB, 0.57913E+00_JPRB, 0.59975E+00_JPRB ,&
&  0.60892E+00_JPRB, 0.62646E+00_JPRB, 0.64281E+00_JPRB, 0.64715E+00_JPRB, 0.65073E+00_JPRB, 0.63795E+00_JPRB, 0.63583E+00_JPRB ,&
&  0.63487E+00_JPRB, 0.63270E+00_JPRB, 0.62319E+00_JPRB, 0.62951E+00_JPRB, 0.63476E+00_JPRB, 0.63804E+00_JPRB, 0.65525E+00_JPRB ,&
&  0.64730E+00_JPRB, 0.61786E+00_JPRB, 0.55594E+00_JPRB, 0.48748E+00_JPRB, 0.45417E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.40563E+00_JPRB, 0.42336E+00_JPRB, 0.44304E+00_JPRB, 0.46102E+00_JPRB, 0.47664E+00_JPRB, 0.48962E+00_JPRB, 0.51083E+00_JPRB ,&
&  0.53386E+00_JPRB, 0.54810E+00_JPRB, 0.55190E+00_JPRB, 0.56470E+00_JPRB, 0.58009E+00_JPRB, 0.60106E+00_JPRB, 0.61714E+00_JPRB ,&
&  0.63849E+00_JPRB, 0.66915E+00_JPRB, 0.69671E+00_JPRB, 0.72623E+00_JPRB, 0.75435E+00_JPRB, 0.78385E+00_JPRB, 0.82586E+00_JPRB ,&
&  0.88042E+00_JPRB, 0.95332E+00_JPRB, 0.10380E+01_JPRB, 0.11168E+01_JPRB, 0.11845E+01_JPRB, 0.12574E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.81555E+00_JPRB, 0.10153E+01_JPRB, 0.11420E+01_JPRB, 0.10424E+01_JPRB, 0.95648E+00_JPRB, 0.90920E+00_JPRB, 0.88667E+00_JPRB ,&
&  0.84933E+00_JPRB, 0.79680E+00_JPRB, 0.78101E+00_JPRB, 0.83010E+00_JPRB, 0.79555E+00_JPRB, 0.80704E+00_JPRB, 0.83388E+00_JPRB ,&
&  0.87545E+00_JPRB, 0.92720E+00_JPRB, 0.98178E+00_JPRB, 0.10536E+01_JPRB, 0.11558E+01_JPRB, 0.12840E+01_JPRB, 0.14495E+01_JPRB ,&
&  0.16583E+01_JPRB, 0.18987E+01_JPRB, 0.21232E+01_JPRB, 0.17314E+01_JPRB, 0.15901E+01_JPRB, 0.15397E+01_JPRB, 0.13622E+01_JPRB ,&
&  0.91287E+00_JPRB, 0.69022E+00_JPRB, 0.68592E+00_JPRB, 0.66833E+00_JPRB, 0.60526E+00_JPRB, 0.55007E+00_JPRB, 0.49936E+00_JPRB ,&
&  0.47187E+00_JPRB, 0.45112E+00_JPRB, 0.44086E+00_JPRB, 0.43417E+00_JPRB, 0.43308E+00_JPRB, 0.44493E+00_JPRB, 0.45987E+00_JPRB ,&
&  0.47457E+00_JPRB, 0.47788E+00_JPRB, 0.49752E+00_JPRB, 0.51054E+00_JPRB, 0.54825E+00_JPRB, 0.58338E+00_JPRB, 0.60211E+00_JPRB ,&
&  0.61250E+00_JPRB, 0.60914E+00_JPRB, 0.60641E+00_JPRB, 0.60348E+00_JPRB, 0.60190E+00_JPRB, 0.59884E+00_JPRB, 0.59434E+00_JPRB ,&
&  0.58711E+00_JPRB, 0.59923E+00_JPRB, 0.59005E+00_JPRB, 0.58516E+00_JPRB, 0.59344E+00_JPRB, 0.57147E+00_JPRB, 0.55703E+00_JPRB ,&
&  0.53468E+00_JPRB, 0.48688E+00_JPRB, 0.45169E+00_JPRB, 0.42677E+00_JPRB, 0.41061E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.48868E+00_JPRB, 0.53100E+00_JPRB, 0.54835E+00_JPRB, 0.56951E+00_JPRB, 0.58349E+00_JPRB, 0.60305E+00_JPRB, 0.61918E+00_JPRB ,&
&  0.63354E+00_JPRB, 0.65118E+00_JPRB, 0.65857E+00_JPRB, 0.67920E+00_JPRB, 0.69595E+00_JPRB, 0.71082E+00_JPRB, 0.72235E+00_JPRB ,&
&  0.73645E+00_JPRB, 0.76201E+00_JPRB, 0.78994E+00_JPRB, 0.82305E+00_JPRB, 0.85616E+00_JPRB, 0.89981E+00_JPRB, 0.96068E+00_JPRB ,&
&  0.10340E+01_JPRB, 0.11039E+01_JPRB, 0.11645E+01_JPRB, 0.12129E+01_JPRB, 0.12344E+01_JPRB, 0.12788E+01_JPRB, 0.12670E+01_JPRB ,&
&  0.90765E+00_JPRB, 0.10663E+01_JPRB, 0.10278E+01_JPRB, 0.97814E+00_JPRB, 0.93944E+00_JPRB, 0.90154E+00_JPRB, 0.87841E+00_JPRB ,&
&  0.85448E+00_JPRB, 0.80060E+00_JPRB, 0.79026E+00_JPRB, 0.80310E+00_JPRB, 0.83075E+00_JPRB, 0.85608E+00_JPRB, 0.90076E+00_JPRB ,&
&  0.96532E+00_JPRB, 0.10484E+01_JPRB, 0.11373E+01_JPRB, 0.12323E+01_JPRB, 0.13595E+01_JPRB, 0.15215E+01_JPRB, 0.16657E+01_JPRB ,&
&  0.18190E+01_JPRB, 0.19801E+01_JPRB, 0.20803E+01_JPRB, 0.17740E+01_JPRB, 0.14901E+01_JPRB, 0.13598E+01_JPRB, 0.10646E+01_JPRB ,&
&  0.80180E+00_JPRB, 0.69164E+00_JPRB, 0.71074E+00_JPRB, 0.69795E+00_JPRB, 0.61103E+00_JPRB, 0.52934E+00_JPRB, 0.46652E+00_JPRB ,&
&  0.44315E+00_JPRB, 0.41312E+00_JPRB, 0.39995E+00_JPRB, 0.39908E+00_JPRB, 0.40309E+00_JPRB, 0.41045E+00_JPRB, 0.43064E+00_JPRB ,&
&  0.44463E+00_JPRB, 0.45971E+00_JPRB, 0.48206E+00_JPRB, 0.50972E+00_JPRB, 0.55657E+00_JPRB, 0.62666E+00_JPRB, 0.63311E+00_JPRB ,&
&  0.61056E+00_JPRB, 0.58108E+00_JPRB, 0.55551E+00_JPRB, 0.55583E+00_JPRB, 0.55435E+00_JPRB, 0.55762E+00_JPRB, 0.57337E+00_JPRB ,&
&  0.59772E+00_JPRB, 0.61674E+00_JPRB, 0.58698E+00_JPRB, 0.56714E+00_JPRB, 0.57418E+00_JPRB, 0.54467E+00_JPRB, 0.48970E+00_JPRB ,&
&  0.46843E+00_JPRB, 0.45059E+00_JPRB, 0.43455E+00_JPRB, 0.44179E+00_JPRB, 0.46249E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.64030E+00_JPRB, 0.66584E+00_JPRB, 0.68332E+00_JPRB, 0.68363E+00_JPRB, 0.70849E+00_JPRB, 0.72517E+00_JPRB, 0.73363E+00_JPRB ,&
&  0.74104E+00_JPRB, 0.75652E+00_JPRB, 0.76459E+00_JPRB, 0.78555E+00_JPRB, 0.80104E+00_JPRB, 0.81744E+00_JPRB, 0.82972E+00_JPRB ,&
&  0.84392E+00_JPRB, 0.87068E+00_JPRB, 0.90200E+00_JPRB, 0.94692E+00_JPRB, 0.99170E+00_JPRB, 0.10481E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11746E+01_JPRB, 0.12278E+01_JPRB, 0.12587E+01_JPRB, 0.12414E+01_JPRB, 0.12175E+01_JPRB, 0.90829E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10151E+01_JPRB, 0.99860E+00_JPRB, 0.97033E+00_JPRB, 0.93666E+00_JPRB, 0.89044E+00_JPRB, 0.86067E+00_JPRB ,&
&  0.84180E+00_JPRB, 0.81882E+00_JPRB, 0.80086E+00_JPRB, 0.79225E+00_JPRB, 0.86404E+00_JPRB, 0.89358E+00_JPRB, 0.94310E+00_JPRB ,&
&  0.10090E+01_JPRB, 0.10931E+01_JPRB, 0.11947E+01_JPRB, 0.13312E+01_JPRB, 0.14778E+01_JPRB, 0.16249E+01_JPRB, 0.17750E+01_JPRB ,&
&  0.18841E+01_JPRB, 0.19401E+01_JPRB, 0.19639E+01_JPRB, 0.15901E+01_JPRB, 0.13594E+01_JPRB, 0.11296E+01_JPRB, 0.89848E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.73350E+00_JPRB, 0.76109E+00_JPRB, 0.73925E+00_JPRB, 0.63550E+00_JPRB, 0.53605E+00_JPRB, 0.46490E+00_JPRB ,&
&  0.43450E+00_JPRB, 0.39821E+00_JPRB, 0.39116E+00_JPRB, 0.38970E+00_JPRB, 0.39706E+00_JPRB, 0.40371E+00_JPRB, 0.41316E+00_JPRB ,&
&  0.42714E+00_JPRB, 0.45173E+00_JPRB, 0.49817E+00_JPRB, 0.56581E+00_JPRB, 0.65293E+00_JPRB, 0.74593E+00_JPRB, 0.71501E+00_JPRB ,&
&  0.60569E+00_JPRB, 0.54526E+00_JPRB, 0.55644E+00_JPRB, 0.56997E+00_JPRB, 0.60541E+00_JPRB, 0.66514E+00_JPRB, 0.73834E+00_JPRB ,&
&  0.86349E+00_JPRB, 0.94155E+00_JPRB, 0.85728E+00_JPRB, 0.71881E+00_JPRB, 0.99800E+00_JPRB, 0.97815E+00_JPRB, 0.51210E+00_JPRB ,&
&  0.55767E+00_JPRB, 0.54827E+00_JPRB, 0.57019E+00_JPRB, 0.59602E+00_JPRB, 0.62809E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.77534E+00_JPRB, 0.79921E+00_JPRB, 0.80973E+00_JPRB, 0.80913E+00_JPRB, 0.82309E+00_JPRB, 0.84825E+00_JPRB, 0.85679E+00_JPRB ,&
&  0.86586E+00_JPRB, 0.86311E+00_JPRB, 0.87452E+00_JPRB, 0.88784E+00_JPRB, 0.90217E+00_JPRB, 0.91848E+00_JPRB, 0.93602E+00_JPRB ,&
&  0.94829E+00_JPRB, 0.98405E+00_JPRB, 0.10031E+01_JPRB, 0.10436E+01_JPRB, 0.10860E+01_JPRB, 0.11449E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.12093E+01_JPRB, 0.12421E+01_JPRB, 0.12837E+01_JPRB, 0.12725E+01_JPRB, 0.12794E+01_JPRB, 0.99512E+00_JPRB ,&
&  0.10588E+01_JPRB, 0.10192E+01_JPRB, 0.96636E+00_JPRB, 0.94050E+00_JPRB, 0.91118E+00_JPRB, 0.88195E+00_JPRB, 0.85500E+00_JPRB ,&
&  0.84744E+00_JPRB, 0.83648E+00_JPRB, 0.81365E+00_JPRB, 0.85171E+00_JPRB, 0.87448E+00_JPRB, 0.90654E+00_JPRB, 0.96805E+00_JPRB ,&
&  0.10311E+01_JPRB, 0.11255E+01_JPRB, 0.12409E+01_JPRB, 0.13816E+01_JPRB, 0.15306E+01_JPRB, 0.16734E+01_JPRB, 0.18104E+01_JPRB ,&
&  0.19386E+01_JPRB, 0.20369E+01_JPRB, 0.20468E+01_JPRB, 0.17019E+01_JPRB, 0.14655E+01_JPRB, 0.11114E+01_JPRB, 0.90052E+00_JPRB ,&
&  0.77956E+00_JPRB, 0.73062E+00_JPRB, 0.74273E+00_JPRB, 0.77671E+00_JPRB, 0.65477E+00_JPRB, 0.56860E+00_JPRB, 0.48769E+00_JPRB ,&
&  0.45448E+00_JPRB, 0.42064E+00_JPRB, 0.41320E+00_JPRB, 0.40553E+00_JPRB, 0.39959E+00_JPRB, 0.40051E+00_JPRB, 0.40760E+00_JPRB ,&
&  0.40990E+00_JPRB, 0.42660E+00_JPRB, 0.46634E+00_JPRB, 0.51059E+00_JPRB, 0.54776E+00_JPRB, 0.56552E+00_JPRB, 0.53881E+00_JPRB ,&
&  0.57139E+00_JPRB, 0.56871E+00_JPRB, 0.57367E+00_JPRB, 0.62976E+00_JPRB, 0.73479E+00_JPRB, 0.89921E+00_JPRB, 0.11064E+01_JPRB ,&
&  0.13610E+01_JPRB, 0.14259E+01_JPRB, 0.17332E+01_JPRB, 0.22872E+01_JPRB, 0.20665E+01_JPRB, 0.17475E+01_JPRB, 0.63413E+00_JPRB ,&
&  0.66619E+00_JPRB, 0.67385E+00_JPRB, 0.70160E+00_JPRB, 0.72844E+00_JPRB, 0.76225E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.78416E+00_JPRB, 0.80451E+00_JPRB, 0.81994E+00_JPRB, 0.83072E+00_JPRB, 0.84773E+00_JPRB, 0.86492E+00_JPRB, 0.88132E+00_JPRB ,&
&  0.89587E+00_JPRB, 0.90712E+00_JPRB, 0.92277E+00_JPRB, 0.93757E+00_JPRB, 0.95523E+00_JPRB, 0.97611E+00_JPRB, 0.10008E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.10452E+01_JPRB, 0.10588E+01_JPRB, 0.10971E+01_JPRB, 0.11457E+01_JPRB, 0.11710E+01_JPRB, 0.11844E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.11996E+01_JPRB, 0.12178E+01_JPRB, 0.12419E+01_JPRB, 0.12459E+01_JPRB, 0.12902E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.11078E+01_JPRB, 0.10379E+01_JPRB, 0.92597E+00_JPRB, 0.89497E+00_JPRB, 0.87336E+00_JPRB, 0.86539E+00_JPRB, 0.85757E+00_JPRB ,&
&  0.85074E+00_JPRB, 0.85309E+00_JPRB, 0.84552E+00_JPRB, 0.84246E+00_JPRB, 0.86073E+00_JPRB, 0.89632E+00_JPRB, 0.94796E+00_JPRB ,&
&  0.10249E+01_JPRB, 0.11291E+01_JPRB, 0.12503E+01_JPRB, 0.13869E+01_JPRB, 0.15383E+01_JPRB, 0.16595E+01_JPRB, 0.17693E+01_JPRB ,&
&  0.18511E+01_JPRB, 0.18972E+01_JPRB, 0.17462E+01_JPRB, 0.17132E+01_JPRB, 0.16373E+01_JPRB, 0.12180E+01_JPRB, 0.93805E+00_JPRB ,&
&  0.71100E+00_JPRB, 0.70878E+00_JPRB, 0.75025E+00_JPRB, 0.82052E+00_JPRB, 0.70733E+00_JPRB, 0.63353E+00_JPRB, 0.54470E+00_JPRB ,&
&  0.47347E+00_JPRB, 0.43982E+00_JPRB, 0.43495E+00_JPRB, 0.46762E+00_JPRB, 0.44378E+00_JPRB, 0.42246E+00_JPRB, 0.42670E+00_JPRB ,&
&  0.41205E+00_JPRB, 0.41271E+00_JPRB, 0.42242E+00_JPRB, 0.44404E+00_JPRB, 0.46398E+00_JPRB, 0.46019E+00_JPRB, 0.51879E+00_JPRB ,&
&  0.52062E+00_JPRB, 0.51788E+00_JPRB, 0.53247E+00_JPRB, 0.48365E+00_JPRB, 0.66518E+00_JPRB, 0.75899E+00_JPRB, 0.89157E+00_JPRB ,&
&  0.10312E+01_JPRB, 0.14279E+01_JPRB, 0.16823E+01_JPRB, 0.13087E+01_JPRB, 0.99584E+00_JPRB, 0.70028E+00_JPRB, 0.66524E+00_JPRB ,&
&  0.68159E+00_JPRB, 0.68625E+00_JPRB, 0.70572E+00_JPRB, 0.73090E+00_JPRB, 0.75843E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.78012E+00_JPRB, 0.80235E+00_JPRB, 0.82073E+00_JPRB, 0.83564E+00_JPRB, 0.85444E+00_JPRB, 0.87434E+00_JPRB, 0.89173E+00_JPRB ,&
&  0.90739E+00_JPRB, 0.92163E+00_JPRB, 0.93914E+00_JPRB, 0.95538E+00_JPRB, 0.97431E+00_JPRB, 0.99724E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.10430E+01_JPRB, 0.10563E+01_JPRB, 0.10868E+01_JPRB, 0.11237E+01_JPRB, 0.11479E+01_JPRB, 0.11539E+01_JPRB, 0.11531E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.11748E+01_JPRB, 0.11903E+01_JPRB, 0.12067E+01_JPRB, 0.12320E+01_JPRB, 0.12915E+01_JPRB, 0.10485E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.10560E+01_JPRB, 0.88941E+00_JPRB, 0.83043E+00_JPRB, 0.81285E+00_JPRB, 0.80097E+00_JPRB, 0.80019E+00_JPRB ,&
&  0.81717E+00_JPRB, 0.79837E+00_JPRB, 0.79634E+00_JPRB, 0.79954E+00_JPRB, 0.82516E+00_JPRB, 0.85907E+00_JPRB, 0.89498E+00_JPRB ,&
&  0.95832E+00_JPRB, 0.10451E+01_JPRB, 0.11504E+01_JPRB, 0.12700E+01_JPRB, 0.14096E+01_JPRB, 0.15110E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.15199E+01_JPRB, 0.13013E+01_JPRB, 0.13739E+01_JPRB, 0.13833E+01_JPRB, 0.11996E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.83342E+00_JPRB, 0.72822E+00_JPRB, 0.77391E+00_JPRB, 0.89061E+00_JPRB, 0.84596E+00_JPRB, 0.73985E+00_JPRB, 0.65054E+00_JPRB ,&
&  0.53657E+00_JPRB, 0.50153E+00_JPRB, 0.49275E+00_JPRB, 0.54832E+00_JPRB, 0.53615E+00_JPRB, 0.48266E+00_JPRB, 0.46571E+00_JPRB ,&
&  0.42799E+00_JPRB, 0.39001E+00_JPRB, 0.37333E+00_JPRB, 0.39264E+00_JPRB, 0.43741E+00_JPRB, 0.42242E+00_JPRB, 0.48212E+00_JPRB ,&
&  0.45831E+00_JPRB, 0.41018E+00_JPRB, 0.44439E+00_JPRB, 0.50675E+00_JPRB, 0.54742E+00_JPRB, 0.59618E+00_JPRB, 0.70075E+00_JPRB ,&
&  0.86509E+00_JPRB, 0.10718E+01_JPRB, 0.10425E+01_JPRB, 0.83183E+00_JPRB, 0.67032E+00_JPRB, 0.63790E+00_JPRB, 0.65270E+00_JPRB ,&
&  0.67330E+00_JPRB, 0.68973E+00_JPRB, 0.70950E+00_JPRB, 0.73140E+00_JPRB, 0.75470E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.76321E+00_JPRB, 0.78491E+00_JPRB, 0.80403E+00_JPRB, 0.82169E+00_JPRB, 0.84038E+00_JPRB, 0.85895E+00_JPRB, 0.87503E+00_JPRB ,&
&  0.88929E+00_JPRB, 0.90595E+00_JPRB, 0.92062E+00_JPRB, 0.93551E+00_JPRB, 0.95747E+00_JPRB, 0.97758E+00_JPRB, 0.99029E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.10366E+01_JPRB, 0.10679E+01_JPRB, 0.10950E+01_JPRB, 0.11081E+01_JPRB, 0.11119E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.10913E+01_JPRB, 0.10975E+01_JPRB, 0.11138E+01_JPRB, 0.11454E+01_JPRB, 0.11281E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.93986E+00_JPRB, 0.11034E+01_JPRB, 0.88594E+00_JPRB, 0.76816E+00_JPRB, 0.76721E+00_JPRB, 0.71952E+00_JPRB, 0.71838E+00_JPRB ,&
&  0.74323E+00_JPRB, 0.75185E+00_JPRB, 0.75840E+00_JPRB, 0.76384E+00_JPRB, 0.78901E+00_JPRB, 0.82935E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.88376E+00_JPRB, 0.93195E+00_JPRB, 0.96019E+00_JPRB, 0.10203E+01_JPRB, 0.11373E+01_JPRB, 0.12349E+01_JPRB, 0.12730E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.11849E+01_JPRB, 0.89822E+00_JPRB, 0.94742E+00_JPRB, 0.10270E+01_JPRB, 0.98224E+00_JPRB, 0.89838E+00_JPRB ,&
&  0.88481E+00_JPRB, 0.82390E+00_JPRB, 0.97471E+00_JPRB, 0.10617E+01_JPRB, 0.10555E+01_JPRB, 0.96721E+00_JPRB, 0.83332E+00_JPRB ,&
&  0.71048E+00_JPRB, 0.61740E+00_JPRB, 0.63234E+00_JPRB, 0.70298E+00_JPRB, 0.68739E+00_JPRB, 0.58631E+00_JPRB, 0.53532E+00_JPRB ,&
&  0.46918E+00_JPRB, 0.40482E+00_JPRB, 0.36282E+00_JPRB, 0.35572E+00_JPRB, 0.33801E+00_JPRB, 0.39231E+00_JPRB, 0.43595E+00_JPRB ,&
&  0.42738E+00_JPRB, 0.38440E+00_JPRB, 0.38281E+00_JPRB, 0.44816E+00_JPRB, 0.45578E+00_JPRB, 0.44845E+00_JPRB, 0.49798E+00_JPRB ,&
&  0.61009E+00_JPRB, 0.70587E+00_JPRB, 0.71314E+00_JPRB, 0.64902E+00_JPRB, 0.62574E+00_JPRB, 0.63235E+00_JPRB, 0.65071E+00_JPRB ,&
&  0.66981E+00_JPRB, 0.68733E+00_JPRB, 0.70453E+00_JPRB, 0.72320E+00_JPRB, 0.74209E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.72737E+00_JPRB, 0.75481E+00_JPRB, 0.77348E+00_JPRB, 0.78855E+00_JPRB, 0.79621E+00_JPRB, 0.80503E+00_JPRB, 0.82270E+00_JPRB ,&
&  0.83706E+00_JPRB, 0.85212E+00_JPRB, 0.86793E+00_JPRB, 0.89056E+00_JPRB, 0.92027E+00_JPRB, 0.94355E+00_JPRB, 0.95300E+00_JPRB ,&
&  0.95916E+00_JPRB, 0.99230E+00_JPRB, 0.10138E+01_JPRB, 0.10555E+01_JPRB, 0.10724E+01_JPRB, 0.10996E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.11047E+01_JPRB, 0.11087E+01_JPRB, 0.11182E+01_JPRB, 0.11425E+01_JPRB, 0.11651E+01_JPRB, 0.12797E+01_JPRB, 0.14453E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.11311E+01_JPRB, 0.88404E+00_JPRB, 0.70632E+00_JPRB, 0.70155E+00_JPRB, 0.70776E+00_JPRB, 0.68479E+00_JPRB ,&
&  0.69150E+00_JPRB, 0.71513E+00_JPRB, 0.73467E+00_JPRB, 0.72778E+00_JPRB, 0.75478E+00_JPRB, 0.81158E+00_JPRB, 0.80982E+00_JPRB ,&
&  0.82648E+00_JPRB, 0.84339E+00_JPRB, 0.83869E+00_JPRB, 0.91691E+00_JPRB, 0.88211E+00_JPRB, 0.96770E+00_JPRB, 0.10256E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.94931E+00_JPRB, 0.71445E+00_JPRB, 0.73086E+00_JPRB, 0.83546E+00_JPRB, 0.84550E+00_JPRB, 0.82128E+00_JPRB ,&
&  0.90890E+00_JPRB, 0.94603E+00_JPRB, 0.88599E+00_JPRB, 0.11800E+01_JPRB, 0.13079E+01_JPRB, 0.13077E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.86383E+00_JPRB, 0.86245E+00_JPRB, 0.86947E+00_JPRB, 0.83944E+00_JPRB, 0.77611E+00_JPRB, 0.67037E+00_JPRB ,&
&  0.56858E+00_JPRB, 0.48368E+00_JPRB, 0.42313E+00_JPRB, 0.39271E+00_JPRB, 0.40788E+00_JPRB, 0.33933E+00_JPRB, 0.40149E+00_JPRB ,&
&  0.38697E+00_JPRB, 0.40512E+00_JPRB, 0.38407E+00_JPRB, 0.46985E+00_JPRB, 0.46185E+00_JPRB, 0.40750E+00_JPRB, 0.42356E+00_JPRB ,&
&  0.47630E+00_JPRB, 0.52080E+00_JPRB, 0.55207E+00_JPRB, 0.57424E+00_JPRB, 0.59357E+00_JPRB, 0.61607E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.64130E+00_JPRB, 0.65811E+00_JPRB, 0.67434E+00_JPRB, 0.69804E+00_JPRB, 0.70946E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.61833E+00_JPRB, 0.62884E+00_JPRB, 0.64472E+00_JPRB, 0.65446E+00_JPRB, 0.66540E+00_JPRB, 0.67357E+00_JPRB, 0.70069E+00_JPRB ,&
&  0.71973E+00_JPRB, 0.73749E+00_JPRB, 0.76267E+00_JPRB, 0.78918E+00_JPRB, 0.79946E+00_JPRB, 0.81058E+00_JPRB, 0.81624E+00_JPRB ,&
&  0.83124E+00_JPRB, 0.84751E+00_JPRB, 0.84553E+00_JPRB, 0.85218E+00_JPRB, 0.85935E+00_JPRB, 0.92304E+00_JPRB, 0.92487E+00_JPRB ,&
&  0.91699E+00_JPRB, 0.92947E+00_JPRB, 0.94479E+00_JPRB, 0.10011E+01_JPRB, 0.11594E+01_JPRB, 0.12876E+01_JPRB, 0.12973E+01_JPRB ,&
&  0.13755E+01_JPRB, 0.12866E+01_JPRB, 0.12249E+01_JPRB, 0.91310E+00_JPRB, 0.84324E+00_JPRB, 0.83775E+00_JPRB, 0.78072E+00_JPRB ,&
&  0.76837E+00_JPRB, 0.78139E+00_JPRB, 0.81923E+00_JPRB, 0.80396E+00_JPRB, 0.81451E+00_JPRB, 0.81164E+00_JPRB, 0.79018E+00_JPRB ,&
&  0.79575E+00_JPRB, 0.81318E+00_JPRB, 0.82748E+00_JPRB, 0.88251E+00_JPRB, 0.89753E+00_JPRB, 0.97499E+00_JPRB, 0.99593E+00_JPRB ,&
&  0.99605E+00_JPRB, 0.10091E+01_JPRB, 0.94831E+00_JPRB, 0.97927E+00_JPRB, 0.10982E+01_JPRB, 0.11113E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.95070E+00_JPRB, 0.76634E+00_JPRB, 0.12111E+01_JPRB, 0.14490E+01_JPRB, 0.14036E+01_JPRB, 0.13983E+01_JPRB ,&
&  0.13100E+01_JPRB, 0.11970E+01_JPRB, 0.11082E+01_JPRB, 0.10768E+01_JPRB, 0.98342E+00_JPRB, 0.89095E+00_JPRB, 0.82667E+00_JPRB ,&
&  0.69761E+00_JPRB, 0.56197E+00_JPRB, 0.46183E+00_JPRB, 0.43134E+00_JPRB, 0.41109E+00_JPRB, 0.40122E+00_JPRB, 0.40928E+00_JPRB ,&
&  0.43104E+00_JPRB, 0.40233E+00_JPRB, 0.50057E+00_JPRB, 0.64985E+00_JPRB, 0.48449E+00_JPRB, 0.42899E+00_JPRB, 0.39138E+00_JPRB ,&
&  0.40462E+00_JPRB, 0.42924E+00_JPRB, 0.45344E+00_JPRB, 0.47203E+00_JPRB, 0.49489E+00_JPRB, 0.51481E+00_JPRB, 0.53035E+00_JPRB ,&
&  0.54578E+00_JPRB, 0.56509E+00_JPRB, 0.58163E+00_JPRB, 0.60033E+00_JPRB, 0.61566E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.61424E+00_JPRB, 0.62770E+00_JPRB, 0.62564E+00_JPRB, 0.62948E+00_JPRB, 0.62307E+00_JPRB, 0.62257E+00_JPRB, 0.60609E+00_JPRB ,&
&  0.59222E+00_JPRB, 0.59968E+00_JPRB, 0.62928E+00_JPRB, 0.68646E+00_JPRB, 0.70027E+00_JPRB, 0.71718E+00_JPRB, 0.72103E+00_JPRB ,&
&  0.73978E+00_JPRB, 0.72849E+00_JPRB, 0.72470E+00_JPRB, 0.70991E+00_JPRB, 0.72143E+00_JPRB, 0.76253E+00_JPRB, 0.85040E+00_JPRB ,&
&  0.97223E+00_JPRB, 0.11213E+01_JPRB, 0.13388E+01_JPRB, 0.15955E+01_JPRB, 0.21220E+01_JPRB, 0.18576E+01_JPRB, 0.16976E+01_JPRB ,&
&  0.20017E+01_JPRB, 0.17422E+01_JPRB, 0.14867E+01_JPRB, 0.13245E+01_JPRB, 0.12164E+01_JPRB, 0.11661E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.12665E+01_JPRB, 0.13128E+01_JPRB, 0.13249E+01_JPRB, 0.13325E+01_JPRB, 0.13017E+01_JPRB, 0.12424E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.11851E+01_JPRB, 0.12226E+01_JPRB, 0.13045E+01_JPRB, 0.14794E+01_JPRB, 0.15591E+01_JPRB, 0.16078E+01_JPRB ,&
&  0.16175E+01_JPRB, 0.17067E+01_JPRB, 0.19105E+01_JPRB, 0.19613E+01_JPRB, 0.19816E+01_JPRB, 0.18493E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.14943E+01_JPRB, 0.11865E+01_JPRB, 0.99485E+00_JPRB, 0.16174E+01_JPRB, 0.18051E+01_JPRB, 0.15150E+01_JPRB, 0.16232E+01_JPRB ,&
&  0.15084E+01_JPRB, 0.14247E+01_JPRB, 0.13793E+01_JPRB, 0.12451E+01_JPRB, 0.10528E+01_JPRB, 0.96837E+00_JPRB, 0.88314E+00_JPRB ,&
&  0.82036E+00_JPRB, 0.66730E+00_JPRB, 0.54012E+00_JPRB, 0.48854E+00_JPRB, 0.47131E+00_JPRB, 0.49525E+00_JPRB, 0.65909E+00_JPRB ,&
&  0.65699E+00_JPRB, 0.45712E+00_JPRB, 0.46750E+00_JPRB, 0.62431E+00_JPRB, 0.65518E+00_JPRB, 0.45519E+00_JPRB, 0.39654E+00_JPRB ,&
&  0.41283E+00_JPRB, 0.43144E+00_JPRB, 0.45095E+00_JPRB, 0.46161E+00_JPRB, 0.46568E+00_JPRB, 0.47250E+00_JPRB, 0.50226E+00_JPRB ,&
&  0.52642E+00_JPRB, 0.54905E+00_JPRB, 0.56728E+00_JPRB, 0.59290E+00_JPRB, 0.60690E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.79611E+00_JPRB, 0.82061E+00_JPRB, 0.83757E+00_JPRB, 0.84208E+00_JPRB, 0.83453E+00_JPRB, 0.82378E+00_JPRB, 0.79918E+00_JPRB ,&
&  0.77106E+00_JPRB, 0.78064E+00_JPRB, 0.81959E+00_JPRB, 0.90930E+00_JPRB, 0.97272E+00_JPRB, 0.10117E+01_JPRB, 0.10674E+01_JPRB ,&
&  0.11227E+01_JPRB, 0.11471E+01_JPRB, 0.11434E+01_JPRB, 0.11738E+01_JPRB, 0.12075E+01_JPRB, 0.12503E+01_JPRB, 0.14398E+01_JPRB ,&
&  0.17119E+01_JPRB, 0.20375E+01_JPRB, 0.23708E+01_JPRB, 0.26218E+01_JPRB, 0.22345E+01_JPRB, 0.16064E+01_JPRB, 0.18252E+01_JPRB ,&
&  0.21350E+01_JPRB, 0.19896E+01_JPRB, 0.16191E+01_JPRB, 0.16083E+01_JPRB, 0.16499E+01_JPRB, 0.17549E+01_JPRB, 0.18857E+01_JPRB ,&
&  0.19791E+01_JPRB, 0.20428E+01_JPRB, 0.21009E+01_JPRB, 0.21414E+01_JPRB, 0.21905E+01_JPRB, 0.22343E+01_JPRB, 0.22757E+01_JPRB ,&
&  0.22752E+01_JPRB, 0.22381E+01_JPRB, 0.22482E+01_JPRB, 0.23528E+01_JPRB, 0.24080E+01_JPRB, 0.24639E+01_JPRB, 0.24929E+01_JPRB ,&
&  0.25058E+01_JPRB, 0.25490E+01_JPRB, 0.28084E+01_JPRB, 0.29309E+01_JPRB, 0.27431E+01_JPRB, 0.25242E+01_JPRB, 0.24243E+01_JPRB ,&
&  0.22653E+01_JPRB, 0.20006E+01_JPRB, 0.16127E+01_JPRB, 0.24072E+01_JPRB, 0.24964E+01_JPRB, 0.21861E+01_JPRB, 0.18828E+01_JPRB ,&
&  0.17068E+01_JPRB, 0.16252E+01_JPRB, 0.14929E+01_JPRB, 0.13432E+01_JPRB, 0.11297E+01_JPRB, 0.10590E+01_JPRB, 0.87570E+00_JPRB ,&
&  0.89420E+00_JPRB, 0.78089E+00_JPRB, 0.63880E+00_JPRB, 0.56045E+00_JPRB, 0.54942E+00_JPRB, 0.58903E+00_JPRB, 0.71021E+00_JPRB ,&
&  0.67591E+00_JPRB, 0.59080E+00_JPRB, 0.60499E+00_JPRB, 0.99033E+00_JPRB, 0.20542E+01_JPRB, 0.56645E+00_JPRB, 0.44590E+00_JPRB ,&
&  0.47086E+00_JPRB, 0.50569E+00_JPRB, 0.53934E+00_JPRB, 0.55548E+00_JPRB, 0.56184E+00_JPRB, 0.58282E+00_JPRB, 0.62177E+00_JPRB ,&
&  0.66608E+00_JPRB, 0.69720E+00_JPRB, 0.71021E+00_JPRB, 0.74481E+00_JPRB, 0.77214E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.94884E+00_JPRB, 0.97664E+00_JPRB, 0.10031E+01_JPRB, 0.10410E+01_JPRB, 0.10773E+01_JPRB, 0.10889E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.11325E+01_JPRB, 0.11746E+01_JPRB, 0.12122E+01_JPRB, 0.12650E+01_JPRB, 0.13116E+01_JPRB, 0.13630E+01_JPRB ,&
&  0.14161E+01_JPRB, 0.14863E+01_JPRB, 0.15892E+01_JPRB, 0.17001E+01_JPRB, 0.17910E+01_JPRB, 0.19345E+01_JPRB, 0.20935E+01_JPRB ,&
&  0.22477E+01_JPRB, 0.20893E+01_JPRB, 0.20143E+01_JPRB, 0.16191E+01_JPRB, 0.11940E+01_JPRB, 0.15576E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.18005E+01_JPRB, 0.18211E+01_JPRB, 0.18424E+01_JPRB, 0.19458E+01_JPRB, 0.21148E+01_JPRB, 0.22564E+01_JPRB, 0.23507E+01_JPRB ,&
&  0.24423E+01_JPRB, 0.25219E+01_JPRB, 0.25951E+01_JPRB, 0.26791E+01_JPRB, 0.27827E+01_JPRB, 0.28778E+01_JPRB, 0.29432E+01_JPRB ,&
&  0.30324E+01_JPRB, 0.30780E+01_JPRB, 0.31269E+01_JPRB, 0.31960E+01_JPRB, 0.31146E+01_JPRB, 0.30960E+01_JPRB, 0.30962E+01_JPRB ,&
&  0.31260E+01_JPRB, 0.31821E+01_JPRB, 0.33427E+01_JPRB, 0.34830E+01_JPRB, 0.34717E+01_JPRB, 0.32012E+01_JPRB, 0.30283E+01_JPRB ,&
&  0.28741E+01_JPRB, 0.25803E+01_JPRB, 0.25285E+01_JPRB, 0.31782E+01_JPRB, 0.20931E+01_JPRB, 0.22364E+01_JPRB, 0.22180E+01_JPRB ,&
&  0.19665E+01_JPRB, 0.18963E+01_JPRB, 0.17070E+01_JPRB, 0.13988E+01_JPRB, 0.11495E+01_JPRB, 0.83152E+00_JPRB, 0.88790E+00_JPRB ,&
&  0.10008E+01_JPRB, 0.90869E+00_JPRB, 0.73849E+00_JPRB, 0.63962E+00_JPRB, 0.67866E+00_JPRB, 0.71909E+00_JPRB, 0.73577E+00_JPRB ,&
&  0.73936E+00_JPRB, 0.71962E+00_JPRB, 0.68935E+00_JPRB, 0.94458E+00_JPRB, 0.10820E+01_JPRB, 0.55680E+00_JPRB, 0.49527E+00_JPRB ,&
&  0.53296E+00_JPRB, 0.59770E+00_JPRB, 0.67350E+00_JPRB, 0.72560E+00_JPRB, 0.74457E+00_JPRB, 0.76450E+00_JPRB, 0.78479E+00_JPRB ,&
&  0.80723E+00_JPRB, 0.84566E+00_JPRB, 0.89680E+00_JPRB, 0.91836E+00_JPRB, 0.93471E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.98098E+00_JPRB, 0.10303E+01_JPRB, 0.11111E+01_JPRB, 0.11751E+01_JPRB, 0.12122E+01_JPRB, 0.12190E+01_JPRB, 0.11977E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.12290E+01_JPRB, 0.13000E+01_JPRB, 0.13633E+01_JPRB, 0.14316E+01_JPRB, 0.14987E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.16301E+01_JPRB, 0.16875E+01_JPRB, 0.17641E+01_JPRB, 0.18567E+01_JPRB, 0.19467E+01_JPRB, 0.21016E+01_JPRB, 0.21146E+01_JPRB ,&
&  0.17194E+01_JPRB, 0.13174E+01_JPRB, 0.16253E+01_JPRB, 0.16902E+01_JPRB, 0.15670E+01_JPRB, 0.16826E+01_JPRB, 0.18137E+01_JPRB ,&
&  0.19671E+01_JPRB, 0.20591E+01_JPRB, 0.21525E+01_JPRB, 0.22303E+01_JPRB, 0.22971E+01_JPRB, 0.24009E+01_JPRB, 0.24909E+01_JPRB ,&
&  0.26116E+01_JPRB, 0.27417E+01_JPRB, 0.28726E+01_JPRB, 0.30273E+01_JPRB, 0.31768E+01_JPRB, 0.33459E+01_JPRB, 0.34831E+01_JPRB ,&
&  0.36085E+01_JPRB, 0.36945E+01_JPRB, 0.36715E+01_JPRB, 0.36129E+01_JPRB, 0.35428E+01_JPRB, 0.35201E+01_JPRB, 0.35132E+01_JPRB ,&
&  0.35945E+01_JPRB, 0.37310E+01_JPRB, 0.39888E+01_JPRB, 0.41401E+01_JPRB, 0.41718E+01_JPRB, 0.38113E+01_JPRB, 0.36486E+01_JPRB ,&
&  0.34560E+01_JPRB, 0.31452E+01_JPRB, 0.32389E+01_JPRB, 0.34284E+01_JPRB, 0.24626E+01_JPRB, 0.24129E+01_JPRB, 0.20559E+01_JPRB ,&
&  0.21731E+01_JPRB, 0.20522E+01_JPRB, 0.18543E+01_JPRB, 0.15959E+01_JPRB, 0.12212E+01_JPRB, 0.92309E+00_JPRB, 0.10217E+01_JPRB ,&
&  0.11966E+01_JPRB, 0.11695E+01_JPRB, 0.90155E+00_JPRB, 0.77920E+00_JPRB, 0.73047E+00_JPRB, 0.76829E+00_JPRB, 0.77229E+00_JPRB ,&
&  0.78498E+00_JPRB, 0.73088E+00_JPRB, 0.72664E+00_JPRB, 0.81956E+00_JPRB, 0.74290E+00_JPRB, 0.58038E+00_JPRB, 0.56122E+00_JPRB ,&
&  0.61695E+00_JPRB, 0.67107E+00_JPRB, 0.71708E+00_JPRB, 0.75086E+00_JPRB, 0.78333E+00_JPRB, 0.82023E+00_JPRB, 0.84237E+00_JPRB ,&
&  0.89235E+00_JPRB, 0.88624E+00_JPRB, 0.92530E+00_JPRB, 0.94045E+00_JPRB, 0.96336E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.98135E+00_JPRB, 0.10405E+01_JPRB, 0.11169E+01_JPRB, 0.11900E+01_JPRB, 0.12277E+01_JPRB, 0.12590E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12421E+01_JPRB, 0.12153E+01_JPRB, 0.12955E+01_JPRB, 0.13758E+01_JPRB, 0.14786E+01_JPRB, 0.15719E+01_JPRB, 0.16620E+01_JPRB ,&
&  0.17414E+01_JPRB, 0.18263E+01_JPRB, 0.19224E+01_JPRB, 0.20426E+01_JPRB, 0.21218E+01_JPRB, 0.22215E+01_JPRB, 0.19738E+01_JPRB ,&
&  0.14208E+01_JPRB, 0.14752E+01_JPRB, 0.20122E+01_JPRB, 0.20132E+01_JPRB, 0.19511E+01_JPRB, 0.20946E+01_JPRB, 0.22179E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.22878E+01_JPRB, 0.23070E+01_JPRB, 0.23355E+01_JPRB, 0.23387E+01_JPRB, 0.23698E+01_JPRB, 0.24130E+01_JPRB ,&
&  0.25170E+01_JPRB, 0.26740E+01_JPRB, 0.28539E+01_JPRB, 0.30849E+01_JPRB, 0.33226E+01_JPRB, 0.35633E+01_JPRB, 0.37066E+01_JPRB ,&
&  0.37988E+01_JPRB, 0.39025E+01_JPRB, 0.39262E+01_JPRB, 0.37454E+01_JPRB, 0.37432E+01_JPRB, 0.38656E+01_JPRB, 0.39964E+01_JPRB ,&
&  0.40996E+01_JPRB, 0.43839E+01_JPRB, 0.45030E+01_JPRB, 0.43570E+01_JPRB, 0.39542E+01_JPRB, 0.41113E+01_JPRB, 0.37506E+01_JPRB ,&
&  0.38154E+01_JPRB, 0.36320E+01_JPRB, 0.39271E+01_JPRB, 0.31742E+01_JPRB, 0.29403E+01_JPRB, 0.28500E+01_JPRB, 0.25950E+01_JPRB ,&
&  0.23251E+01_JPRB, 0.22453E+01_JPRB, 0.20439E+01_JPRB, 0.15411E+01_JPRB, 0.13989E+01_JPRB, 0.11105E+01_JPRB, 0.12861E+01_JPRB ,&
&  0.13806E+01_JPRB, 0.14988E+01_JPRB, 0.10096E+01_JPRB, 0.80262E+00_JPRB, 0.71198E+00_JPRB, 0.74836E+00_JPRB, 0.84700E+00_JPRB ,&
&  0.83789E+00_JPRB, 0.79623E+00_JPRB, 0.76584E+00_JPRB, 0.72192E+00_JPRB, 0.63513E+00_JPRB, 0.59371E+00_JPRB, 0.59646E+00_JPRB ,&
&  0.64687E+00_JPRB, 0.69453E+00_JPRB, 0.70444E+00_JPRB, 0.70953E+00_JPRB, 0.72865E+00_JPRB, 0.77860E+00_JPRB, 0.81190E+00_JPRB ,&
&  0.85434E+00_JPRB, 0.87991E+00_JPRB, 0.90929E+00_JPRB, 0.92167E+00_JPRB, 0.93715E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.82536E+00_JPRB, 0.85741E+00_JPRB, 0.88499E+00_JPRB, 0.88613E+00_JPRB, 0.89489E+00_JPRB, 0.91449E+00_JPRB, 0.95804E+00_JPRB ,&
&  0.10075E+01_JPRB, 0.10656E+01_JPRB, 0.11364E+01_JPRB, 0.12278E+01_JPRB, 0.13481E+01_JPRB, 0.14522E+01_JPRB, 0.15732E+01_JPRB ,&
&  0.16948E+01_JPRB, 0.18163E+01_JPRB, 0.19455E+01_JPRB, 0.20802E+01_JPRB, 0.21468E+01_JPRB, 0.21653E+01_JPRB, 0.17515E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.21304E+01_JPRB, 0.20193E+01_JPRB, 0.22038E+01_JPRB, 0.22875E+01_JPRB, 0.24492E+01_JPRB, 0.24170E+01_JPRB ,&
&  0.24108E+01_JPRB, 0.23673E+01_JPRB, 0.23301E+01_JPRB, 0.23331E+01_JPRB, 0.23434E+01_JPRB, 0.23457E+01_JPRB, 0.23288E+01_JPRB ,&
&  0.23533E+01_JPRB, 0.24420E+01_JPRB, 0.26043E+01_JPRB, 0.28340E+01_JPRB, 0.31118E+01_JPRB, 0.34407E+01_JPRB, 0.36816E+01_JPRB ,&
&  0.38934E+01_JPRB, 0.41067E+01_JPRB, 0.41937E+01_JPRB, 0.40528E+01_JPRB, 0.39892E+01_JPRB, 0.41733E+01_JPRB, 0.45487E+01_JPRB ,&
&  0.45486E+01_JPRB, 0.44265E+01_JPRB, 0.46531E+01_JPRB, 0.46386E+01_JPRB, 0.43659E+01_JPRB, 0.43980E+01_JPRB, 0.38827E+01_JPRB ,&
&  0.41681E+01_JPRB, 0.39023E+01_JPRB, 0.41435E+01_JPRB, 0.34202E+01_JPRB, 0.32835E+01_JPRB, 0.34967E+01_JPRB, 0.33318E+01_JPRB ,&
&  0.28841E+01_JPRB, 0.27264E+01_JPRB, 0.23754E+01_JPRB, 0.18701E+01_JPRB, 0.16327E+01_JPRB, 0.14563E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.14977E+01_JPRB, 0.14210E+01_JPRB, 0.10575E+01_JPRB, 0.72133E+00_JPRB, 0.67788E+00_JPRB, 0.80076E+00_JPRB, 0.98232E+00_JPRB ,&
&  0.96695E+00_JPRB, 0.10155E+01_JPRB, 0.10585E+01_JPRB, 0.90740E+00_JPRB, 0.70076E+00_JPRB, 0.64137E+00_JPRB, 0.62998E+00_JPRB ,&
&  0.66777E+00_JPRB, 0.70988E+00_JPRB, 0.71440E+00_JPRB, 0.70301E+00_JPRB, 0.67218E+00_JPRB, 0.70394E+00_JPRB, 0.71779E+00_JPRB ,&
&  0.75679E+00_JPRB, 0.76330E+00_JPRB, 0.78982E+00_JPRB, 0.80998E+00_JPRB, 0.81775E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.73210E+00_JPRB, 0.76350E+00_JPRB, 0.81057E+00_JPRB, 0.82236E+00_JPRB, 0.80918E+00_JPRB, 0.80570E+00_JPRB, 0.81856E+00_JPRB ,&
&  0.83341E+00_JPRB, 0.85100E+00_JPRB, 0.89009E+00_JPRB, 0.95356E+00_JPRB, 0.10591E+01_JPRB, 0.11662E+01_JPRB, 0.13045E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.16881E+01_JPRB, 0.19262E+01_JPRB, 0.21749E+01_JPRB, 0.23366E+01_JPRB, 0.23521E+01_JPRB, 0.15299E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.20542E+01_JPRB, 0.20596E+01_JPRB, 0.22692E+01_JPRB, 0.24636E+01_JPRB, 0.25832E+01_JPRB, 0.24848E+01_JPRB ,&
&  0.24743E+01_JPRB, 0.24249E+01_JPRB, 0.23982E+01_JPRB, 0.23661E+01_JPRB, 0.23459E+01_JPRB, 0.23313E+01_JPRB, 0.22677E+01_JPRB ,&
&  0.22027E+01_JPRB, 0.21898E+01_JPRB, 0.22612E+01_JPRB, 0.24209E+01_JPRB, 0.26472E+01_JPRB, 0.29150E+01_JPRB, 0.33010E+01_JPRB ,&
&  0.37465E+01_JPRB, 0.42021E+01_JPRB, 0.46095E+01_JPRB, 0.44132E+01_JPRB, 0.44057E+01_JPRB, 0.44244E+01_JPRB, 0.46850E+01_JPRB ,&
&  0.46401E+01_JPRB, 0.43801E+01_JPRB, 0.48269E+01_JPRB, 0.52643E+01_JPRB, 0.51726E+01_JPRB, 0.47884E+01_JPRB, 0.42282E+01_JPRB ,&
&  0.43846E+01_JPRB, 0.43576E+01_JPRB, 0.42115E+01_JPRB, 0.37948E+01_JPRB, 0.37768E+01_JPRB, 0.42222E+01_JPRB, 0.44345E+01_JPRB ,&
&  0.38909E+01_JPRB, 0.34770E+01_JPRB, 0.29676E+01_JPRB, 0.24305E+01_JPRB, 0.20110E+01_JPRB, 0.18727E+01_JPRB, 0.19098E+01_JPRB ,&
&  0.17527E+01_JPRB, 0.16218E+01_JPRB, 0.11155E+01_JPRB, 0.71381E+00_JPRB, 0.63558E+00_JPRB, 0.83597E+00_JPRB, 0.11397E+01_JPRB ,&
&  0.12081E+01_JPRB, 0.13008E+01_JPRB, 0.12899E+01_JPRB, 0.14288E+01_JPRB, 0.99385E+00_JPRB, 0.80824E+00_JPRB, 0.71152E+00_JPRB ,&
&  0.70624E+00_JPRB, 0.72641E+00_JPRB, 0.74041E+00_JPRB, 0.70217E+00_JPRB, 0.65958E+00_JPRB, 0.66556E+00_JPRB, 0.65403E+00_JPRB ,&
&  0.67082E+00_JPRB, 0.68611E+00_JPRB, 0.68875E+00_JPRB, 0.70041E+00_JPRB, 0.72100E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.68032E+00_JPRB, 0.71016E+00_JPRB, 0.74213E+00_JPRB, 0.76596E+00_JPRB, 0.77230E+00_JPRB, 0.76030E+00_JPRB, 0.74539E+00_JPRB ,&
&  0.74043E+00_JPRB, 0.73524E+00_JPRB, 0.75504E+00_JPRB, 0.78642E+00_JPRB, 0.84882E+00_JPRB, 0.91673E+00_JPRB, 0.10199E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.14299E+01_JPRB, 0.17678E+01_JPRB, 0.21410E+01_JPRB, 0.22601E+01_JPRB, 0.19853E+01_JPRB, 0.18084E+01_JPRB ,&
&  0.17323E+01_JPRB, 0.20348E+01_JPRB, 0.20569E+01_JPRB, 0.23844E+01_JPRB, 0.26997E+01_JPRB, 0.28286E+01_JPRB, 0.23988E+01_JPRB ,&
&  0.23706E+01_JPRB, 0.23198E+01_JPRB, 0.22624E+01_JPRB, 0.22432E+01_JPRB, 0.22358E+01_JPRB, 0.22430E+01_JPRB, 0.22132E+01_JPRB ,&
&  0.21285E+01_JPRB, 0.20440E+01_JPRB, 0.20131E+01_JPRB, 0.20586E+01_JPRB, 0.22072E+01_JPRB, 0.23892E+01_JPRB, 0.26550E+01_JPRB ,&
&  0.31243E+01_JPRB, 0.38287E+01_JPRB, 0.46446E+01_JPRB, 0.48786E+01_JPRB, 0.47773E+01_JPRB, 0.47711E+01_JPRB, 0.47984E+01_JPRB ,&
&  0.48968E+01_JPRB, 0.53008E+01_JPRB, 0.54752E+01_JPRB, 0.54954E+01_JPRB, 0.54355E+01_JPRB, 0.53684E+01_JPRB, 0.46604E+01_JPRB ,&
&  0.45647E+01_JPRB, 0.45686E+01_JPRB, 0.42060E+01_JPRB, 0.42599E+01_JPRB, 0.44309E+01_JPRB, 0.49930E+01_JPRB, 0.47061E+01_JPRB ,&
&  0.39607E+01_JPRB, 0.34231E+01_JPRB, 0.31323E+01_JPRB, 0.29834E+01_JPRB, 0.26183E+01_JPRB, 0.25657E+01_JPRB, 0.26169E+01_JPRB ,&
&  0.19020E+01_JPRB, 0.72337E+00_JPRB, 0.45760E+00_JPRB, 0.55402E+00_JPRB, 0.43369E+00_JPRB, 0.65504E+00_JPRB, 0.12822E+01_JPRB ,&
&  0.15704E+01_JPRB, 0.18838E+01_JPRB, 0.21687E+01_JPRB, 0.21617E+01_JPRB, 0.17376E+01_JPRB, 0.12682E+01_JPRB, 0.96805E+00_JPRB ,&
&  0.82330E+00_JPRB, 0.78590E+00_JPRB, 0.80487E+00_JPRB, 0.75799E+00_JPRB, 0.69512E+00_JPRB, 0.67454E+00_JPRB, 0.65069E+00_JPRB ,&
&  0.63535E+00_JPRB, 0.65327E+00_JPRB, 0.66993E+00_JPRB, 0.67014E+00_JPRB, 0.66301E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.70762E+00_JPRB, 0.70992E+00_JPRB, 0.72177E+00_JPRB, 0.71578E+00_JPRB, 0.70747E+00_JPRB, 0.70344E+00_JPRB, 0.69669E+00_JPRB ,&
&  0.68349E+00_JPRB, 0.67210E+00_JPRB, 0.68664E+00_JPRB, 0.71497E+00_JPRB, 0.76093E+00_JPRB, 0.81438E+00_JPRB, 0.86084E+00_JPRB ,&
&  0.96078E+00_JPRB, 0.11617E+01_JPRB, 0.15074E+01_JPRB, 0.20110E+01_JPRB, 0.22741E+01_JPRB, 0.21047E+01_JPRB, 0.20743E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.21053E+01_JPRB, 0.24975E+01_JPRB, 0.31311E+01_JPRB, 0.32879E+01_JPRB, 0.30900E+01_JPRB, 0.25010E+01_JPRB ,&
&  0.24427E+01_JPRB, 0.23261E+01_JPRB, 0.21739E+01_JPRB, 0.20268E+01_JPRB, 0.19535E+01_JPRB, 0.19365E+01_JPRB, 0.19572E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.19192E+01_JPRB, 0.18600E+01_JPRB, 0.18428E+01_JPRB, 0.19075E+01_JPRB, 0.20540E+01_JPRB, 0.22441E+01_JPRB ,&
&  0.25052E+01_JPRB, 0.30023E+01_JPRB, 0.40609E+01_JPRB, 0.52035E+01_JPRB, 0.44898E+01_JPRB, 0.51826E+01_JPRB, 0.50087E+01_JPRB ,&
&  0.49775E+01_JPRB, 0.54543E+01_JPRB, 0.55618E+01_JPRB, 0.64036E+01_JPRB, 0.60685E+01_JPRB, 0.55889E+01_JPRB, 0.48281E+01_JPRB ,&
&  0.46864E+01_JPRB, 0.43645E+01_JPRB, 0.44007E+01_JPRB, 0.46818E+01_JPRB, 0.50613E+01_JPRB, 0.57378E+01_JPRB, 0.41049E+01_JPRB ,&
&  0.37271E+01_JPRB, 0.40426E+01_JPRB, 0.38082E+01_JPRB, 0.32930E+01_JPRB, 0.32513E+01_JPRB, 0.34037E+01_JPRB, 0.22342E+01_JPRB ,&
&  0.75456E+00_JPRB, 0.47623E+00_JPRB, 0.40372E+00_JPRB, 0.38159E+00_JPRB, 0.34779E+00_JPRB, 0.67377E+00_JPRB, 0.25855E+01_JPRB ,&
&  0.21177E+01_JPRB, 0.28176E+01_JPRB, 0.31524E+01_JPRB, 0.31192E+01_JPRB, 0.27525E+01_JPRB, 0.19923E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.10301E+01_JPRB, 0.95411E+00_JPRB, 0.90700E+00_JPRB, 0.81478E+00_JPRB, 0.76375E+00_JPRB, 0.72683E+00_JPRB ,&
&  0.71850E+00_JPRB, 0.71538E+00_JPRB, 0.70851E+00_JPRB, 0.69903E+00_JPRB, 0.70678E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.79022E+00_JPRB, 0.77453E+00_JPRB, 0.75492E+00_JPRB, 0.71159E+00_JPRB, 0.68009E+00_JPRB, 0.67620E+00_JPRB, 0.66705E+00_JPRB ,&
&  0.65128E+00_JPRB, 0.63796E+00_JPRB, 0.64193E+00_JPRB, 0.66276E+00_JPRB, 0.69160E+00_JPRB, 0.73988E+00_JPRB, 0.78616E+00_JPRB ,&
&  0.86165E+00_JPRB, 0.10250E+01_JPRB, 0.13640E+01_JPRB, 0.17290E+01_JPRB, 0.21097E+01_JPRB, 0.20241E+01_JPRB, 0.21993E+01_JPRB ,&
&  0.19805E+01_JPRB, 0.20937E+01_JPRB, 0.26115E+01_JPRB, 0.35118E+01_JPRB, 0.47716E+01_JPRB, 0.47189E+01_JPRB, 0.41124E+01_JPRB ,&
&  0.35888E+01_JPRB, 0.31992E+01_JPRB, 0.28857E+01_JPRB, 0.25408E+01_JPRB, 0.20041E+01_JPRB, 0.17395E+01_JPRB, 0.16739E+01_JPRB ,&
&  0.16378E+01_JPRB, 0.16731E+01_JPRB, 0.16434E+01_JPRB, 0.16320E+01_JPRB, 0.17152E+01_JPRB, 0.18656E+01_JPRB, 0.20757E+01_JPRB ,&
&  0.22347E+01_JPRB, 0.24465E+01_JPRB, 0.31018E+01_JPRB, 0.45793E+01_JPRB, 0.49481E+01_JPRB, 0.47202E+01_JPRB, 0.48137E+01_JPRB ,&
&  0.48791E+01_JPRB, 0.56360E+01_JPRB, 0.63297E+01_JPRB, 0.77130E+01_JPRB, 0.62956E+01_JPRB, 0.56923E+01_JPRB, 0.51865E+01_JPRB ,&
&  0.48674E+01_JPRB, 0.50880E+01_JPRB, 0.42671E+01_JPRB, 0.44241E+01_JPRB, 0.51530E+01_JPRB, 0.47652E+01_JPRB, 0.43234E+01_JPRB ,&
&  0.45780E+01_JPRB, 0.39101E+01_JPRB, 0.36694E+01_JPRB, 0.32825E+01_JPRB, 0.38931E+01_JPRB, 0.31190E+01_JPRB, 0.11502E+01_JPRB ,&
&  0.65962E+00_JPRB, 0.50741E+00_JPRB, 0.36917E+00_JPRB, 0.35791E+00_JPRB, 0.40767E+00_JPRB, 0.68597E+00_JPRB, 0.18412E+01_JPRB ,&
&  0.20503E+01_JPRB, 0.32227E+01_JPRB, 0.35495E+01_JPRB, 0.36971E+01_JPRB, 0.31336E+01_JPRB, 0.28012E+01_JPRB, 0.35413E+01_JPRB ,&
&  0.30462E+01_JPRB, 0.23508E+01_JPRB, 0.17842E+01_JPRB, 0.14131E+01_JPRB, 0.11340E+01_JPRB, 0.98477E+00_JPRB, 0.89759E+00_JPRB ,&
&  0.85920E+00_JPRB, 0.84104E+00_JPRB, 0.82391E+00_JPRB, 0.80909E+00_JPRB, 0.78825E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.87861E+00_JPRB, 0.86254E+00_JPRB, 0.83418E+00_JPRB, 0.75903E+00_JPRB, 0.71157E+00_JPRB, 0.70131E+00_JPRB, 0.68388E+00_JPRB ,&
&  0.66227E+00_JPRB, 0.63542E+00_JPRB, 0.62114E+00_JPRB, 0.62949E+00_JPRB, 0.64234E+00_JPRB, 0.66752E+00_JPRB, 0.71220E+00_JPRB ,&
&  0.80915E+00_JPRB, 0.98833E+00_JPRB, 0.12538E+01_JPRB, 0.14975E+01_JPRB, 0.17251E+01_JPRB, 0.17639E+01_JPRB, 0.21298E+01_JPRB ,&
&  0.22149E+01_JPRB, 0.21702E+01_JPRB, 0.25141E+01_JPRB, 0.35813E+01_JPRB, 0.52631E+01_JPRB, 0.62360E+01_JPRB, 0.65161E+01_JPRB ,&
&  0.56161E+01_JPRB, 0.46294E+01_JPRB, 0.39601E+01_JPRB, 0.35101E+01_JPRB, 0.29006E+01_JPRB, 0.21533E+01_JPRB, 0.17079E+01_JPRB ,&
&  0.15077E+01_JPRB, 0.15170E+01_JPRB, 0.15436E+01_JPRB, 0.15073E+01_JPRB, 0.15759E+01_JPRB, 0.17224E+01_JPRB, 0.19845E+01_JPRB ,&
&  0.21735E+01_JPRB, 0.22999E+01_JPRB, 0.25744E+01_JPRB, 0.35973E+01_JPRB, 0.50972E+01_JPRB, 0.56812E+01_JPRB, 0.48977E+01_JPRB ,&
&  0.42685E+01_JPRB, 0.44810E+01_JPRB, 0.71123E+01_JPRB, 0.78774E+01_JPRB, 0.66295E+01_JPRB, 0.62860E+01_JPRB, 0.55895E+01_JPRB ,&
&  0.54987E+01_JPRB, 0.58562E+01_JPRB, 0.50436E+01_JPRB, 0.50083E+01_JPRB, 0.45525E+01_JPRB, 0.42625E+01_JPRB, 0.48553E+01_JPRB ,&
&  0.48647E+01_JPRB, 0.42076E+01_JPRB, 0.35434E+01_JPRB, 0.31405E+01_JPRB, 0.33072E+01_JPRB, 0.20615E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.91272E+00_JPRB, 0.67432E+00_JPRB, 0.52007E+00_JPRB, 0.53680E+00_JPRB, 0.47507E+00_JPRB, 0.71601E+00_JPRB, 0.12553E+01_JPRB ,&
&  0.23185E+01_JPRB, 0.31370E+01_JPRB, 0.40235E+01_JPRB, 0.40171E+01_JPRB, 0.32079E+01_JPRB, 0.25548E+01_JPRB, 0.39845E+01_JPRB ,&
&  0.28628E+01_JPRB, 0.25217E+01_JPRB, 0.24185E+01_JPRB, 0.19629E+01_JPRB, 0.15098E+01_JPRB, 0.12364E+01_JPRB, 0.10597E+01_JPRB ,&
&  0.95912E+00_JPRB, 0.90606E+00_JPRB, 0.88071E+00_JPRB, 0.88684E+00_JPRB, 0.90040E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.10146E+01_JPRB, 0.95222E+00_JPRB, 0.92446E+00_JPRB, 0.86604E+00_JPRB, 0.77835E+00_JPRB, 0.72203E+00_JPRB, 0.70938E+00_JPRB ,&
&  0.68902E+00_JPRB, 0.66045E+00_JPRB, 0.63051E+00_JPRB, 0.61135E+00_JPRB, 0.61966E+00_JPRB, 0.63569E+00_JPRB, 0.65903E+00_JPRB ,&
&  0.76886E+00_JPRB, 0.10758E+01_JPRB, 0.10566E+01_JPRB, 0.11648E+01_JPRB, 0.13707E+01_JPRB, 0.17074E+01_JPRB, 0.19025E+01_JPRB ,&
&  0.23472E+01_JPRB, 0.23846E+01_JPRB, 0.25370E+01_JPRB, 0.36440E+01_JPRB, 0.64466E+01_JPRB, 0.76195E+01_JPRB, 0.83460E+01_JPRB ,&
&  0.81821E+01_JPRB, 0.67854E+01_JPRB, 0.54180E+01_JPRB, 0.42800E+01_JPRB, 0.35249E+01_JPRB, 0.30168E+01_JPRB, 0.23600E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.16729E+01_JPRB, 0.16226E+01_JPRB, 0.15762E+01_JPRB, 0.15604E+01_JPRB, 0.16423E+01_JPRB, 0.18506E+01_JPRB ,&
&  0.21131E+01_JPRB, 0.23407E+01_JPRB, 0.25340E+01_JPRB, 0.32790E+01_JPRB, 0.41835E+01_JPRB, 0.48488E+01_JPRB, 0.59226E+01_JPRB ,&
&  0.54316E+01_JPRB, 0.58155E+01_JPRB, 0.71698E+01_JPRB, 0.88590E+01_JPRB, 0.74466E+01_JPRB, 0.65269E+01_JPRB, 0.59371E+01_JPRB ,&
&  0.47368E+01_JPRB, 0.47758E+01_JPRB, 0.55118E+01_JPRB, 0.54447E+01_JPRB, 0.43657E+01_JPRB, 0.51044E+01_JPRB, 0.67716E+01_JPRB ,&
&  0.50058E+01_JPRB, 0.47280E+01_JPRB, 0.46452E+01_JPRB, 0.49716E+01_JPRB, 0.30211E+01_JPRB, 0.18152E+01_JPRB, 0.28027E+01_JPRB ,&
&  0.28248E+01_JPRB, 0.14321E+01_JPRB, 0.95913E+00_JPRB, 0.11850E+01_JPRB, 0.81144E+00_JPRB, 0.94625E+00_JPRB, 0.14079E+01_JPRB ,&
&  0.17224E+01_JPRB, 0.20329E+01_JPRB, 0.38503E+01_JPRB, 0.34467E+01_JPRB, 0.30973E+01_JPRB, 0.25412E+01_JPRB, 0.32370E+01_JPRB ,&
&  0.31645E+01_JPRB, 0.23904E+01_JPRB, 0.23876E+01_JPRB, 0.21498E+01_JPRB, 0.18055E+01_JPRB, 0.15429E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.12118E+01_JPRB, 0.11238E+01_JPRB, 0.10838E+01_JPRB, 0.10720E+01_JPRB, 0.10741E+01_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.10456E+01_JPRB, 0.92920E+00_JPRB, 0.87300E+00_JPRB, 0.86005E+00_JPRB, 0.83203E+00_JPRB, 0.75313E+00_JPRB, 0.70614E+00_JPRB ,&
&  0.70756E+00_JPRB, 0.67812E+00_JPRB, 0.63826E+00_JPRB, 0.61756E+00_JPRB, 0.62135E+00_JPRB, 0.62799E+00_JPRB, 0.61951E+00_JPRB ,&
&  0.67917E+00_JPRB, 0.84794E+00_JPRB, 0.90461E+00_JPRB, 0.99182E+00_JPRB, 0.12310E+01_JPRB, 0.15443E+01_JPRB, 0.17243E+01_JPRB ,&
&  0.22133E+01_JPRB, 0.24864E+01_JPRB, 0.28620E+01_JPRB, 0.40945E+01_JPRB, 0.58208E+01_JPRB, 0.74287E+01_JPRB, 0.79773E+01_JPRB ,&
&  0.87130E+01_JPRB, 0.83067E+01_JPRB, 0.66258E+01_JPRB, 0.54029E+01_JPRB, 0.45566E+01_JPRB, 0.40208E+01_JPRB, 0.34477E+01_JPRB ,&
&  0.27673E+01_JPRB, 0.22115E+01_JPRB, 0.19451E+01_JPRB, 0.18024E+01_JPRB, 0.17515E+01_JPRB, 0.17035E+01_JPRB, 0.18167E+01_JPRB ,&
&  0.20612E+01_JPRB, 0.23843E+01_JPRB, 0.25692E+01_JPRB, 0.32270E+01_JPRB, 0.43343E+01_JPRB, 0.51086E+01_JPRB, 0.58061E+01_JPRB ,&
&  0.59944E+01_JPRB, 0.59330E+01_JPRB, 0.75253E+01_JPRB, 0.80701E+01_JPRB, 0.81555E+01_JPRB, 0.72670E+01_JPRB, 0.68280E+01_JPRB ,&
&  0.52758E+01_JPRB, 0.48802E+01_JPRB, 0.51381E+01_JPRB, 0.52704E+01_JPRB, 0.56668E+01_JPRB, 0.71309E+01_JPRB, 0.67330E+01_JPRB ,&
&  0.57735E+01_JPRB, 0.51515E+01_JPRB, 0.46178E+01_JPRB, 0.39817E+01_JPRB, 0.42232E+01_JPRB, 0.29390E+01_JPRB, 0.34059E+01_JPRB ,&
&  0.34587E+01_JPRB, 0.31690E+01_JPRB, 0.25190E+01_JPRB, 0.19893E+01_JPRB, 0.15354E+01_JPRB, 0.16124E+01_JPRB, 0.14901E+01_JPRB ,&
&  0.15514E+01_JPRB, 0.16487E+01_JPRB, 0.26516E+01_JPRB, 0.29499E+01_JPRB, 0.27587E+01_JPRB, 0.23370E+01_JPRB, 0.28548E+01_JPRB ,&
&  0.27454E+01_JPRB, 0.26414E+01_JPRB, 0.23327E+01_JPRB, 0.20294E+01_JPRB, 0.17530E+01_JPRB, 0.15790E+01_JPRB, 0.14820E+01_JPRB ,&
&  0.13504E+01_JPRB, 0.12909E+01_JPRB, 0.12590E+01_JPRB, 0.12197E+01_JPRB, 0.11603E+01_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.10217E+01_JPRB, 0.90276E+00_JPRB, 0.83773E+00_JPRB, 0.79582E+00_JPRB, 0.77903E+00_JPRB, 0.75795E+00_JPRB, 0.71372E+00_JPRB ,&
&  0.68090E+00_JPRB, 0.68241E+00_JPRB, 0.66609E+00_JPRB, 0.65330E+00_JPRB, 0.64647E+00_JPRB, 0.63502E+00_JPRB, 0.61456E+00_JPRB ,&
&  0.63863E+00_JPRB, 0.74342E+00_JPRB, 0.76766E+00_JPRB, 0.90345E+00_JPRB, 0.10979E+01_JPRB, 0.14137E+01_JPRB, 0.17953E+01_JPRB ,&
&  0.20326E+01_JPRB, 0.23090E+01_JPRB, 0.26881E+01_JPRB, 0.36302E+01_JPRB, 0.47484E+01_JPRB, 0.57387E+01_JPRB, 0.60187E+01_JPRB ,&
&  0.56949E+01_JPRB, 0.58174E+01_JPRB, 0.54691E+01_JPRB, 0.49000E+01_JPRB, 0.44053E+01_JPRB, 0.40286E+01_JPRB, 0.35938E+01_JPRB ,&
&  0.31119E+01_JPRB, 0.26350E+01_JPRB, 0.22800E+01_JPRB, 0.20780E+01_JPRB, 0.19713E+01_JPRB, 0.20029E+01_JPRB, 0.20675E+01_JPRB ,&
&  0.21513E+01_JPRB, 0.23942E+01_JPRB, 0.24832E+01_JPRB, 0.28942E+01_JPRB, 0.37556E+01_JPRB, 0.50568E+01_JPRB, 0.56235E+01_JPRB ,&
&  0.66910E+01_JPRB, 0.67007E+01_JPRB, 0.81871E+01_JPRB, 0.92740E+01_JPRB, 0.73378E+01_JPRB, 0.78220E+01_JPRB, 0.83318E+01_JPRB ,&
&  0.77275E+01_JPRB, 0.70847E+01_JPRB, 0.73777E+01_JPRB, 0.70681E+01_JPRB, 0.63265E+01_JPRB, 0.73322E+01_JPRB, 0.65388E+01_JPRB ,&
&  0.56688E+01_JPRB, 0.51366E+01_JPRB, 0.45162E+01_JPRB, 0.45388E+01_JPRB, 0.38851E+01_JPRB, 0.37455E+01_JPRB, 0.29523E+01_JPRB ,&
&  0.30643E+01_JPRB, 0.25558E+01_JPRB, 0.26967E+01_JPRB, 0.22763E+01_JPRB, 0.20291E+01_JPRB, 0.17322E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.13725E+01_JPRB, 0.13656E+01_JPRB, 0.15471E+01_JPRB, 0.20966E+01_JPRB, 0.25728E+01_JPRB, 0.21465E+01_JPRB, 0.23580E+01_JPRB ,&
&  0.26243E+01_JPRB, 0.25847E+01_JPRB, 0.20972E+01_JPRB, 0.19375E+01_JPRB, 0.16753E+01_JPRB, 0.14806E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.12711E+01_JPRB, 0.12282E+01_JPRB, 0.12059E+01_JPRB, 0.11743E+01_JPRB, 0.11283E+01_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.95782E+00_JPRB, 0.85822E+00_JPRB, 0.77316E+00_JPRB, 0.73492E+00_JPRB, 0.72918E+00_JPRB, 0.73056E+00_JPRB, 0.72482E+00_JPRB ,&
&  0.70287E+00_JPRB, 0.69706E+00_JPRB, 0.69594E+00_JPRB, 0.68422E+00_JPRB, 0.67220E+00_JPRB, 0.65986E+00_JPRB, 0.63259E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.74318E+00_JPRB, 0.72903E+00_JPRB, 0.76227E+00_JPRB, 0.90728E+00_JPRB, 0.13026E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.19560E+01_JPRB, 0.21773E+01_JPRB, 0.25107E+01_JPRB, 0.29748E+01_JPRB, 0.34188E+01_JPRB, 0.36559E+01_JPRB, 0.37318E+01_JPRB ,&
&  0.41585E+01_JPRB, 0.35072E+01_JPRB, 0.35396E+01_JPRB, 0.31974E+01_JPRB, 0.29878E+01_JPRB, 0.27767E+01_JPRB, 0.25976E+01_JPRB ,&
&  0.24364E+01_JPRB, 0.22619E+01_JPRB, 0.20835E+01_JPRB, 0.19680E+01_JPRB, 0.19083E+01_JPRB, 0.18855E+01_JPRB, 0.19284E+01_JPRB ,&
&  0.20095E+01_JPRB, 0.21469E+01_JPRB, 0.23613E+01_JPRB, 0.23695E+01_JPRB, 0.29209E+01_JPRB, 0.41111E+01_JPRB, 0.53828E+01_JPRB ,&
&  0.54180E+01_JPRB, 0.52367E+01_JPRB, 0.92284E+01_JPRB, 0.99176E+01_JPRB, 0.92119E+01_JPRB, 0.93550E+01_JPRB, 0.93696E+01_JPRB ,&
&  0.88463E+01_JPRB, 0.77095E+01_JPRB, 0.65990E+01_JPRB, 0.62595E+01_JPRB, 0.74093E+01_JPRB, 0.69833E+01_JPRB, 0.63278E+01_JPRB ,&
&  0.56999E+01_JPRB, 0.53167E+01_JPRB, 0.48380E+01_JPRB, 0.44706E+01_JPRB, 0.42632E+01_JPRB, 0.44296E+01_JPRB, 0.40452E+01_JPRB ,&
&  0.36943E+01_JPRB, 0.38572E+01_JPRB, 0.30746E+01_JPRB, 0.23050E+01_JPRB, 0.18649E+01_JPRB, 0.16376E+01_JPRB, 0.14684E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.11981E+01_JPRB, 0.12198E+01_JPRB, 0.14890E+01_JPRB, 0.20579E+01_JPRB, 0.19996E+01_JPRB, 0.19764E+01_JPRB ,&
&  0.18008E+01_JPRB, 0.20417E+01_JPRB, 0.18942E+01_JPRB, 0.17282E+01_JPRB, 0.15625E+01_JPRB, 0.13824E+01_JPRB, 0.12427E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.11438E+01_JPRB, 0.11376E+01_JPRB, 0.10991E+01_JPRB, 0.10565E+01_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.90801E+00_JPRB, 0.81890E+00_JPRB, 0.73541E+00_JPRB, 0.69911E+00_JPRB, 0.70588E+00_JPRB, 0.71379E+00_JPRB, 0.72058E+00_JPRB ,&
&  0.70872E+00_JPRB, 0.69747E+00_JPRB, 0.70565E+00_JPRB, 0.70795E+00_JPRB, 0.70768E+00_JPRB, 0.69122E+00_JPRB, 0.65608E+00_JPRB ,&
&  0.64832E+00_JPRB, 0.77105E+00_JPRB, 0.72415E+00_JPRB, 0.77127E+00_JPRB, 0.86706E+00_JPRB, 0.11542E+01_JPRB, 0.14073E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.18173E+01_JPRB, 0.19773E+01_JPRB, 0.21994E+01_JPRB, 0.23041E+01_JPRB, 0.22242E+01_JPRB, 0.21233E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.20376E+01_JPRB, 0.17647E+01_JPRB, 0.18613E+01_JPRB, 0.17910E+01_JPRB, 0.15500E+01_JPRB, 0.17071E+01_JPRB ,&
&  0.16721E+01_JPRB, 0.16819E+01_JPRB, 0.16583E+01_JPRB, 0.16463E+01_JPRB, 0.16531E+01_JPRB, 0.16504E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.16366E+01_JPRB, 0.16857E+01_JPRB, 0.19853E+01_JPRB, 0.21250E+01_JPRB, 0.22608E+01_JPRB, 0.28419E+01_JPRB, 0.42506E+01_JPRB ,&
&  0.58580E+01_JPRB, 0.59890E+01_JPRB, 0.61878E+01_JPRB, 0.87964E+01_JPRB, 0.10666E+02_JPRB, 0.99292E+01_JPRB, 0.91910E+01_JPRB ,&
&  0.90265E+01_JPRB, 0.83284E+01_JPRB, 0.76397E+01_JPRB, 0.72181E+01_JPRB, 0.74937E+01_JPRB, 0.72745E+01_JPRB, 0.67290E+01_JPRB ,&
&  0.60473E+01_JPRB, 0.54496E+01_JPRB, 0.51780E+01_JPRB, 0.48720E+01_JPRB, 0.42992E+01_JPRB, 0.40675E+01_JPRB, 0.41279E+01_JPRB ,&
&  0.40301E+01_JPRB, 0.33460E+01_JPRB, 0.23820E+01_JPRB, 0.20663E+01_JPRB, 0.15761E+01_JPRB, 0.13429E+01_JPRB, 0.13302E+01_JPRB ,&
&  0.11027E+01_JPRB, 0.10145E+01_JPRB, 0.10425E+01_JPRB, 0.10930E+01_JPRB, 0.15214E+01_JPRB, 0.15594E+01_JPRB, 0.16701E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.12913E+01_JPRB, 0.15435E+01_JPRB, 0.15091E+01_JPRB, 0.13930E+01_JPRB, 0.12299E+01_JPRB, 0.10938E+01_JPRB ,&
&  0.10441E+01_JPRB, 0.10724E+01_JPRB, 0.10587E+01_JPRB, 0.10329E+01_JPRB, 0.99988E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.84463E+00_JPRB, 0.75824E+00_JPRB, 0.68897E+00_JPRB, 0.64808E+00_JPRB, 0.65093E+00_JPRB, 0.66800E+00_JPRB, 0.67904E+00_JPRB ,&
&  0.69122E+00_JPRB, 0.70006E+00_JPRB, 0.70576E+00_JPRB, 0.71260E+00_JPRB, 0.71713E+00_JPRB, 0.69916E+00_JPRB, 0.68614E+00_JPRB ,&
&  0.69490E+00_JPRB, 0.73808E+00_JPRB, 0.73447E+00_JPRB, 0.73696E+00_JPRB, 0.10070E+01_JPRB, 0.11356E+01_JPRB, 0.13043E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.15431E+01_JPRB, 0.15203E+01_JPRB, 0.14583E+01_JPRB, 0.14028E+01_JPRB, 0.13702E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.10205E+01_JPRB, 0.98765E+00_JPRB, 0.10407E+01_JPRB, 0.11168E+01_JPRB, 0.97856E+00_JPRB, 0.10721E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11756E+01_JPRB, 0.12074E+01_JPRB, 0.12206E+01_JPRB, 0.12509E+01_JPRB, 0.12915E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.13847E+01_JPRB, 0.13991E+01_JPRB, 0.15281E+01_JPRB, 0.18807E+01_JPRB, 0.21034E+01_JPRB, 0.26587E+01_JPRB, 0.42751E+01_JPRB ,&
&  0.57580E+01_JPRB, 0.62542E+01_JPRB, 0.78117E+01_JPRB, 0.10035E+02_JPRB, 0.95989E+01_JPRB, 0.87281E+01_JPRB, 0.82106E+01_JPRB ,&
&  0.77753E+01_JPRB, 0.73446E+01_JPRB, 0.70916E+01_JPRB, 0.68583E+01_JPRB, 0.69721E+01_JPRB, 0.72141E+01_JPRB, 0.70028E+01_JPRB ,&
&  0.63903E+01_JPRB, 0.57911E+01_JPRB, 0.55018E+01_JPRB, 0.50933E+01_JPRB, 0.47217E+01_JPRB, 0.42672E+01_JPRB, 0.42862E+01_JPRB ,&
&  0.43529E+01_JPRB, 0.28490E+01_JPRB, 0.21917E+01_JPRB, 0.20576E+01_JPRB, 0.14412E+01_JPRB, 0.12159E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.89132E+00_JPRB, 0.98887E+00_JPRB, 0.10792E+01_JPRB, 0.11654E+01_JPRB, 0.11987E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.96692E+00_JPRB, 0.94582E+00_JPRB, 0.11778E+01_JPRB, 0.12020E+01_JPRB, 0.11517E+01_JPRB, 0.10764E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.95535E+00_JPRB, 0.97195E+00_JPRB, 0.96410E+00_JPRB, 0.95087E+00_JPRB, 0.92734E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.83147E+00_JPRB, 0.74305E+00_JPRB, 0.66312E+00_JPRB, 0.62688E+00_JPRB, 0.64130E+00_JPRB, 0.65528E+00_JPRB, 0.66283E+00_JPRB ,&
&  0.68059E+00_JPRB, 0.69604E+00_JPRB, 0.70653E+00_JPRB, 0.71405E+00_JPRB, 0.71883E+00_JPRB, 0.71407E+00_JPRB, 0.75723E+00_JPRB ,&
&  0.62378E+00_JPRB, 0.64587E+00_JPRB, 0.62753E+00_JPRB, 0.71280E+00_JPRB, 0.10753E+01_JPRB, 0.10586E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.10572E+01_JPRB, 0.10067E+01_JPRB, 0.95048E+00_JPRB, 0.91158E+00_JPRB, 0.91392E+00_JPRB, 0.87460E+00_JPRB ,&
&  0.76112E+00_JPRB, 0.69021E+00_JPRB, 0.67407E+00_JPRB, 0.67563E+00_JPRB, 0.72375E+00_JPRB, 0.75708E+00_JPRB, 0.83063E+00_JPRB ,&
&  0.86260E+00_JPRB, 0.90925E+00_JPRB, 0.96162E+00_JPRB, 0.10062E+01_JPRB, 0.10281E+01_JPRB, 0.10409E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.11196E+01_JPRB, 0.12123E+01_JPRB, 0.15662E+01_JPRB, 0.18791E+01_JPRB, 0.28577E+01_JPRB, 0.51064E+01_JPRB ,&
&  0.58705E+01_JPRB, 0.55233E+01_JPRB, 0.65187E+01_JPRB, 0.83773E+01_JPRB, 0.79879E+01_JPRB, 0.71339E+01_JPRB, 0.63125E+01_JPRB ,&
&  0.58089E+01_JPRB, 0.54883E+01_JPRB, 0.56322E+01_JPRB, 0.59962E+01_JPRB, 0.62746E+01_JPRB, 0.66929E+01_JPRB, 0.67797E+01_JPRB ,&
&  0.66230E+01_JPRB, 0.64728E+01_JPRB, 0.62529E+01_JPRB, 0.59710E+01_JPRB, 0.55124E+01_JPRB, 0.52185E+01_JPRB, 0.48376E+01_JPRB ,&
&  0.43877E+01_JPRB, 0.37731E+01_JPRB, 0.28125E+01_JPRB, 0.22600E+01_JPRB, 0.15913E+01_JPRB, 0.14235E+01_JPRB, 0.17354E+01_JPRB ,&
&  0.12159E+01_JPRB, 0.10034E+01_JPRB, 0.98342E+00_JPRB, 0.10384E+01_JPRB, 0.10184E+01_JPRB, 0.99227E+00_JPRB, 0.85503E+00_JPRB ,&
&  0.71912E+00_JPRB, 0.79744E+00_JPRB, 0.10334E+01_JPRB, 0.10890E+01_JPRB, 0.10691E+01_JPRB, 0.10176E+01_JPRB, 0.85214E+00_JPRB ,&
&  0.93792E+00_JPRB, 0.89666E+00_JPRB, 0.89456E+00_JPRB, 0.88875E+00_JPRB, 0.87720E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.83243E+00_JPRB, 0.75574E+00_JPRB, 0.66221E+00_JPRB, 0.63564E+00_JPRB, 0.65310E+00_JPRB, 0.65700E+00_JPRB, 0.65725E+00_JPRB ,&
&  0.67180E+00_JPRB, 0.67846E+00_JPRB, 0.68692E+00_JPRB, 0.69978E+00_JPRB, 0.70748E+00_JPRB, 0.73436E+00_JPRB, 0.66263E+00_JPRB ,&
&  0.57958E+00_JPRB, 0.59100E+00_JPRB, 0.57961E+00_JPRB, 0.61796E+00_JPRB, 0.73478E+00_JPRB, 0.74640E+00_JPRB, 0.75367E+00_JPRB ,&
&  0.78124E+00_JPRB, 0.75483E+00_JPRB, 0.75397E+00_JPRB, 0.73483E+00_JPRB, 0.70735E+00_JPRB, 0.71268E+00_JPRB, 0.71858E+00_JPRB ,&
&  0.65046E+00_JPRB, 0.60398E+00_JPRB, 0.57994E+00_JPRB, 0.56943E+00_JPRB, 0.62085E+00_JPRB, 0.68745E+00_JPRB, 0.71819E+00_JPRB ,&
&  0.73515E+00_JPRB, 0.76280E+00_JPRB, 0.80193E+00_JPRB, 0.84910E+00_JPRB, 0.89675E+00_JPRB, 0.93226E+00_JPRB, 0.94876E+00_JPRB ,&
&  0.96499E+00_JPRB, 0.99139E+00_JPRB, 0.10328E+01_JPRB, 0.11956E+01_JPRB, 0.15579E+01_JPRB, 0.23796E+01_JPRB, 0.35161E+01_JPRB ,&
&  0.37583E+01_JPRB, 0.37902E+01_JPRB, 0.40190E+01_JPRB, 0.45194E+01_JPRB, 0.47722E+01_JPRB, 0.44774E+01_JPRB, 0.41266E+01_JPRB ,&
&  0.40694E+01_JPRB, 0.40455E+01_JPRB, 0.46383E+01_JPRB, 0.50803E+01_JPRB, 0.54026E+01_JPRB, 0.58278E+01_JPRB, 0.60102E+01_JPRB ,&
&  0.63785E+01_JPRB, 0.67305E+01_JPRB, 0.67312E+01_JPRB, 0.63951E+01_JPRB, 0.60575E+01_JPRB, 0.57679E+01_JPRB, 0.52353E+01_JPRB ,&
&  0.47051E+01_JPRB, 0.40390E+01_JPRB, 0.32179E+01_JPRB, 0.25628E+01_JPRB, 0.19396E+01_JPRB, 0.17848E+01_JPRB, 0.15399E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.95943E+00_JPRB, 0.92389E+00_JPRB, 0.94576E+00_JPRB, 0.90354E+00_JPRB, 0.81787E+00_JPRB, 0.76716E+00_JPRB ,&
&  0.79073E+00_JPRB, 0.93842E+00_JPRB, 0.10019E+01_JPRB, 0.10485E+01_JPRB, 0.10476E+01_JPRB, 0.10028E+01_JPRB, 0.76213E+00_JPRB ,&
&  0.77448E+00_JPRB, 0.89918E+00_JPRB, 0.89255E+00_JPRB, 0.88510E+00_JPRB, 0.87304E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.82626E+00_JPRB, 0.77014E+00_JPRB, 0.69539E+00_JPRB, 0.65866E+00_JPRB, 0.66000E+00_JPRB, 0.66016E+00_JPRB, 0.63722E+00_JPRB ,&
&  0.66829E+00_JPRB, 0.65300E+00_JPRB, 0.67275E+00_JPRB, 0.70463E+00_JPRB, 0.73564E+00_JPRB, 0.64752E+00_JPRB, 0.52051E+00_JPRB ,&
&  0.48820E+00_JPRB, 0.50607E+00_JPRB, 0.53702E+00_JPRB, 0.55369E+00_JPRB, 0.57430E+00_JPRB, 0.57729E+00_JPRB, 0.58581E+00_JPRB ,&
&  0.59733E+00_JPRB, 0.61283E+00_JPRB, 0.65227E+00_JPRB, 0.66955E+00_JPRB, 0.66858E+00_JPRB, 0.66710E+00_JPRB, 0.67687E+00_JPRB ,&
&  0.63019E+00_JPRB, 0.61159E+00_JPRB, 0.60326E+00_JPRB, 0.57165E+00_JPRB, 0.62482E+00_JPRB, 0.64126E+00_JPRB, 0.65108E+00_JPRB ,&
&  0.66194E+00_JPRB, 0.68676E+00_JPRB, 0.72604E+00_JPRB, 0.76864E+00_JPRB, 0.81245E+00_JPRB, 0.85303E+00_JPRB, 0.88792E+00_JPRB ,&
&  0.91905E+00_JPRB, 0.95435E+00_JPRB, 0.99534E+00_JPRB, 0.10765E+01_JPRB, 0.13691E+01_JPRB, 0.18691E+01_JPRB, 0.25508E+01_JPRB ,&
&  0.27147E+01_JPRB, 0.29053E+01_JPRB, 0.30827E+01_JPRB, 0.29516E+01_JPRB, 0.31397E+01_JPRB, 0.34329E+01_JPRB, 0.33474E+01_JPRB ,&
&  0.34185E+01_JPRB, 0.33741E+01_JPRB, 0.35818E+01_JPRB, 0.39160E+01_JPRB, 0.43886E+01_JPRB, 0.47913E+01_JPRB, 0.51120E+01_JPRB ,&
&  0.56109E+01_JPRB, 0.64678E+01_JPRB, 0.67555E+01_JPRB, 0.64407E+01_JPRB, 0.60885E+01_JPRB, 0.57566E+01_JPRB, 0.52567E+01_JPRB ,&
&  0.47509E+01_JPRB, 0.41816E+01_JPRB, 0.35127E+01_JPRB, 0.28413E+01_JPRB, 0.22347E+01_JPRB, 0.18944E+01_JPRB, 0.15528E+01_JPRB ,&
&  0.12243E+01_JPRB, 0.10022E+01_JPRB, 0.97458E+00_JPRB, 0.94397E+00_JPRB, 0.88547E+00_JPRB, 0.79025E+00_JPRB, 0.71534E+00_JPRB ,&
&  0.72730E+00_JPRB, 0.80852E+00_JPRB, 0.10019E+01_JPRB, 0.10100E+01_JPRB, 0.10174E+01_JPRB, 0.98390E+00_JPRB, 0.82058E+00_JPRB ,&
&  0.76835E+00_JPRB, 0.90876E+00_JPRB, 0.88265E+00_JPRB, 0.88191E+00_JPRB, 0.86742E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.84185E+00_JPRB, 0.80290E+00_JPRB, 0.75786E+00_JPRB, 0.71168E+00_JPRB, 0.67451E+00_JPRB, 0.64080E+00_JPRB, 0.60479E+00_JPRB ,&
&  0.64630E+00_JPRB, 0.57010E+00_JPRB, 0.64133E+00_JPRB, 0.52912E+00_JPRB, 0.49502E+00_JPRB, 0.48836E+00_JPRB, 0.47350E+00_JPRB ,&
&  0.46927E+00_JPRB, 0.48504E+00_JPRB, 0.50768E+00_JPRB, 0.52998E+00_JPRB, 0.53803E+00_JPRB, 0.54256E+00_JPRB, 0.55698E+00_JPRB ,&
&  0.57954E+00_JPRB, 0.61065E+00_JPRB, 0.64770E+00_JPRB, 0.66149E+00_JPRB, 0.66190E+00_JPRB, 0.65535E+00_JPRB, 0.65557E+00_JPRB ,&
&  0.61087E+00_JPRB, 0.59769E+00_JPRB, 0.60201E+00_JPRB, 0.60031E+00_JPRB, 0.59487E+00_JPRB, 0.57961E+00_JPRB, 0.58921E+00_JPRB ,&
&  0.57608E+00_JPRB, 0.54651E+00_JPRB, 0.66956E+00_JPRB, 0.70002E+00_JPRB, 0.74292E+00_JPRB, 0.78740E+00_JPRB, 0.83254E+00_JPRB ,&
&  0.87448E+00_JPRB, 0.91841E+00_JPRB, 0.96822E+00_JPRB, 0.10190E+01_JPRB, 0.11440E+01_JPRB, 0.14233E+01_JPRB, 0.17562E+01_JPRB ,&
&  0.21277E+01_JPRB, 0.16288E+01_JPRB, 0.18317E+01_JPRB, 0.19372E+01_JPRB, 0.23273E+01_JPRB, 0.27931E+01_JPRB, 0.29748E+01_JPRB ,&
&  0.33530E+01_JPRB, 0.33142E+01_JPRB, 0.32431E+01_JPRB, 0.31791E+01_JPRB, 0.34074E+01_JPRB, 0.37624E+01_JPRB, 0.41622E+01_JPRB ,&
&  0.45946E+01_JPRB, 0.50579E+01_JPRB, 0.58725E+01_JPRB, 0.58161E+01_JPRB, 0.56196E+01_JPRB, 0.54047E+01_JPRB, 0.50484E+01_JPRB ,&
&  0.46083E+01_JPRB, 0.40835E+01_JPRB, 0.34453E+01_JPRB, 0.27204E+01_JPRB, 0.22354E+01_JPRB, 0.18360E+01_JPRB, 0.15075E+01_JPRB ,&
&  0.12835E+01_JPRB, 0.11491E+01_JPRB, 0.11100E+01_JPRB, 0.10519E+01_JPRB, 0.98283E+00_JPRB, 0.90405E+00_JPRB, 0.79087E+00_JPRB ,&
&  0.67719E+00_JPRB, 0.67614E+00_JPRB, 0.71646E+00_JPRB, 0.79054E+00_JPRB, 0.83454E+00_JPRB, 0.85411E+00_JPRB, 0.90589E+00_JPRB ,&
&  0.83863E+00_JPRB, 0.87054E+00_JPRB, 0.92116E+00_JPRB, 0.89173E+00_JPRB, 0.87278E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.89787E+00_JPRB, 0.85478E+00_JPRB, 0.81589E+00_JPRB, 0.77543E+00_JPRB, 0.74107E+00_JPRB, 0.69310E+00_JPRB, 0.61396E+00_JPRB ,&
&  0.53759E+00_JPRB, 0.51344E+00_JPRB, 0.52021E+00_JPRB, 0.50512E+00_JPRB, 0.48524E+00_JPRB, 0.47732E+00_JPRB, 0.48408E+00_JPRB ,&
&  0.50243E+00_JPRB, 0.54579E+00_JPRB, 0.58173E+00_JPRB, 0.59783E+00_JPRB, 0.59636E+00_JPRB, 0.59363E+00_JPRB, 0.60234E+00_JPRB ,&
&  0.61350E+00_JPRB, 0.62488E+00_JPRB, 0.63083E+00_JPRB, 0.62212E+00_JPRB, 0.61220E+00_JPRB, 0.60793E+00_JPRB, 0.60470E+00_JPRB ,&
&  0.60444E+00_JPRB, 0.60210E+00_JPRB, 0.58445E+00_JPRB, 0.58620E+00_JPRB, 0.56464E+00_JPRB, 0.55815E+00_JPRB, 0.52466E+00_JPRB ,&
&  0.45376E+00_JPRB, 0.46561E+00_JPRB, 0.60997E+00_JPRB, 0.65960E+00_JPRB, 0.68441E+00_JPRB, 0.73022E+00_JPRB, 0.79090E+00_JPRB ,&
&  0.84133E+00_JPRB, 0.88574E+00_JPRB, 0.93118E+00_JPRB, 0.98972E+00_JPRB, 0.10659E+01_JPRB, 0.12061E+01_JPRB, 0.13807E+01_JPRB ,&
&  0.16766E+01_JPRB, 0.14151E+01_JPRB, 0.12997E+01_JPRB, 0.14114E+01_JPRB, 0.19035E+01_JPRB, 0.22236E+01_JPRB, 0.23236E+01_JPRB ,&
&  0.24039E+01_JPRB, 0.23719E+01_JPRB, 0.23383E+01_JPRB, 0.23638E+01_JPRB, 0.24912E+01_JPRB, 0.27238E+01_JPRB, 0.31070E+01_JPRB ,&
&  0.34245E+01_JPRB, 0.39606E+01_JPRB, 0.47351E+01_JPRB, 0.48994E+01_JPRB, 0.47780E+01_JPRB, 0.46750E+01_JPRB, 0.44735E+01_JPRB ,&
&  0.42427E+01_JPRB, 0.38105E+01_JPRB, 0.31306E+01_JPRB, 0.25644E+01_JPRB, 0.20908E+01_JPRB, 0.17356E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.13165E+01_JPRB, 0.12712E+01_JPRB, 0.12343E+01_JPRB, 0.11696E+01_JPRB, 0.10720E+01_JPRB, 0.97362E+00_JPRB, 0.87834E+00_JPRB ,&
&  0.75077E+00_JPRB, 0.68219E+00_JPRB, 0.63413E+00_JPRB, 0.65156E+00_JPRB, 0.71865E+00_JPRB, 0.79418E+00_JPRB, 0.81226E+00_JPRB ,&
&  0.83068E+00_JPRB, 0.81585E+00_JPRB, 0.80994E+00_JPRB, 0.84402E+00_JPRB, 0.88739E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.88788E+00_JPRB, 0.85909E+00_JPRB, 0.81936E+00_JPRB, 0.80629E+00_JPRB, 0.77165E+00_JPRB, 0.73741E+00_JPRB, 0.65519E+00_JPRB ,&
&  0.55953E+00_JPRB, 0.53922E+00_JPRB, 0.57463E+00_JPRB, 0.59201E+00_JPRB, 0.57812E+00_JPRB, 0.57772E+00_JPRB, 0.59079E+00_JPRB ,&
&  0.60453E+00_JPRB, 0.64155E+00_JPRB, 0.64318E+00_JPRB, 0.62193E+00_JPRB, 0.60628E+00_JPRB, 0.59674E+00_JPRB, 0.59855E+00_JPRB ,&
&  0.59839E+00_JPRB, 0.59764E+00_JPRB, 0.58956E+00_JPRB, 0.57417E+00_JPRB, 0.57216E+00_JPRB, 0.57161E+00_JPRB, 0.56877E+00_JPRB ,&
&  0.56564E+00_JPRB, 0.55681E+00_JPRB, 0.56437E+00_JPRB, 0.57243E+00_JPRB, 0.56148E+00_JPRB, 0.53418E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.42226E+00_JPRB, 0.40968E+00_JPRB, 0.44134E+00_JPRB, 0.54307E+00_JPRB, 0.59725E+00_JPRB, 0.64927E+00_JPRB, 0.70413E+00_JPRB ,&
&  0.67587E+00_JPRB, 0.67612E+00_JPRB, 0.72313E+00_JPRB, 0.87475E+00_JPRB, 0.95780E+00_JPRB, 0.10517E+01_JPRB, 0.11358E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.12842E+01_JPRB, 0.12733E+01_JPRB, 0.10695E+01_JPRB, 0.11481E+01_JPRB, 0.15279E+01_JPRB, 0.15546E+01_JPRB ,&
&  0.17108E+01_JPRB, 0.18674E+01_JPRB, 0.20040E+01_JPRB, 0.19889E+01_JPRB, 0.18784E+01_JPRB, 0.19195E+01_JPRB, 0.21219E+01_JPRB ,&
&  0.24249E+01_JPRB, 0.28675E+01_JPRB, 0.36716E+01_JPRB, 0.41136E+01_JPRB, 0.40258E+01_JPRB, 0.39213E+01_JPRB, 0.38083E+01_JPRB ,&
&  0.37244E+01_JPRB, 0.33092E+01_JPRB, 0.27816E+01_JPRB, 0.23058E+01_JPRB, 0.19027E+01_JPRB, 0.15876E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.12661E+01_JPRB, 0.12498E+01_JPRB, 0.11915E+01_JPRB, 0.10712E+01_JPRB, 0.94637E+00_JPRB, 0.85197E+00_JPRB ,&
&  0.78716E+00_JPRB, 0.72566E+00_JPRB, 0.67780E+00_JPRB, 0.66415E+00_JPRB, 0.70761E+00_JPRB, 0.76578E+00_JPRB, 0.77863E+00_JPRB ,&
&  0.75736E+00_JPRB, 0.74887E+00_JPRB, 0.78832E+00_JPRB, 0.83651E+00_JPRB, 0.85584E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.85240E+00_JPRB, 0.86637E+00_JPRB, 0.86680E+00_JPRB, 0.84863E+00_JPRB, 0.82910E+00_JPRB, 0.75195E+00_JPRB, 0.67437E+00_JPRB ,&
&  0.62243E+00_JPRB, 0.63309E+00_JPRB, 0.66793E+00_JPRB, 0.69820E+00_JPRB, 0.70361E+00_JPRB, 0.68895E+00_JPRB, 0.68398E+00_JPRB ,&
&  0.69481E+00_JPRB, 0.69518E+00_JPRB, 0.66470E+00_JPRB, 0.63936E+00_JPRB, 0.59996E+00_JPRB, 0.59015E+00_JPRB, 0.59196E+00_JPRB ,&
&  0.59602E+00_JPRB, 0.59476E+00_JPRB, 0.59444E+00_JPRB, 0.58890E+00_JPRB, 0.58692E+00_JPRB, 0.57693E+00_JPRB, 0.56101E+00_JPRB ,&
&  0.54179E+00_JPRB, 0.53382E+00_JPRB, 0.53732E+00_JPRB, 0.54860E+00_JPRB, 0.53916E+00_JPRB, 0.52714E+00_JPRB, 0.44936E+00_JPRB ,&
&  0.41908E+00_JPRB, 0.39951E+00_JPRB, 0.39692E+00_JPRB, 0.41057E+00_JPRB, 0.45666E+00_JPRB, 0.49993E+00_JPRB, 0.58457E+00_JPRB ,&
&  0.65368E+00_JPRB, 0.71003E+00_JPRB, 0.73526E+00_JPRB, 0.75638E+00_JPRB, 0.79379E+00_JPRB, 0.86268E+00_JPRB, 0.94505E+00_JPRB ,&
&  0.99782E+00_JPRB, 0.10332E+01_JPRB, 0.10296E+01_JPRB, 0.88478E+00_JPRB, 0.89963E+00_JPRB, 0.10451E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.14770E+01_JPRB, 0.15644E+01_JPRB, 0.15762E+01_JPRB, 0.16913E+01_JPRB, 0.17401E+01_JPRB, 0.17845E+01_JPRB, 0.18302E+01_JPRB ,&
&  0.21160E+01_JPRB, 0.25181E+01_JPRB, 0.31179E+01_JPRB, 0.34265E+01_JPRB, 0.33411E+01_JPRB, 0.33641E+01_JPRB, 0.32059E+01_JPRB ,&
&  0.30756E+01_JPRB, 0.27033E+01_JPRB, 0.23760E+01_JPRB, 0.19810E+01_JPRB, 0.16756E+01_JPRB, 0.15016E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.12383E+01_JPRB, 0.12249E+01_JPRB, 0.11772E+01_JPRB, 0.10767E+01_JPRB, 0.99122E+00_JPRB, 0.92822E+00_JPRB ,&
&  0.88258E+00_JPRB, 0.85346E+00_JPRB, 0.84253E+00_JPRB, 0.81460E+00_JPRB, 0.80869E+00_JPRB, 0.81447E+00_JPRB, 0.80579E+00_JPRB ,&
&  0.79448E+00_JPRB, 0.78655E+00_JPRB, 0.79409E+00_JPRB, 0.80898E+00_JPRB, 0.82448E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.88108E+00_JPRB, 0.89859E+00_JPRB, 0.89504E+00_JPRB, 0.89569E+00_JPRB, 0.90143E+00_JPRB, 0.85013E+00_JPRB, 0.81546E+00_JPRB ,&
&  0.79930E+00_JPRB, 0.81625E+00_JPRB, 0.82423E+00_JPRB, 0.82378E+00_JPRB, 0.81229E+00_JPRB, 0.79933E+00_JPRB, 0.78983E+00_JPRB ,&
&  0.78117E+00_JPRB, 0.75637E+00_JPRB, 0.70783E+00_JPRB, 0.63806E+00_JPRB, 0.61558E+00_JPRB, 0.60817E+00_JPRB, 0.61151E+00_JPRB ,&
&  0.61252E+00_JPRB, 0.60493E+00_JPRB, 0.60317E+00_JPRB, 0.58884E+00_JPRB, 0.57758E+00_JPRB, 0.55004E+00_JPRB, 0.52623E+00_JPRB ,&
&  0.52557E+00_JPRB, 0.52289E+00_JPRB, 0.53979E+00_JPRB, 0.52473E+00_JPRB, 0.50660E+00_JPRB, 0.49619E+00_JPRB, 0.43249E+00_JPRB ,&
&  0.42067E+00_JPRB, 0.40297E+00_JPRB, 0.39666E+00_JPRB, 0.39707E+00_JPRB, 0.41686E+00_JPRB, 0.43550E+00_JPRB, 0.47336E+00_JPRB ,&
&  0.52344E+00_JPRB, 0.63445E+00_JPRB, 0.67855E+00_JPRB, 0.72114E+00_JPRB, 0.75736E+00_JPRB, 0.79974E+00_JPRB, 0.84976E+00_JPRB ,&
&  0.89952E+00_JPRB, 0.95217E+00_JPRB, 0.10046E+01_JPRB, 0.10882E+01_JPRB, 0.11733E+01_JPRB, 0.11704E+01_JPRB, 0.12833E+01_JPRB ,&
&  0.16775E+01_JPRB, 0.16336E+01_JPRB, 0.15566E+01_JPRB, 0.15322E+01_JPRB, 0.15737E+01_JPRB, 0.16435E+01_JPRB, 0.17466E+01_JPRB ,&
&  0.19607E+01_JPRB, 0.22884E+01_JPRB, 0.27502E+01_JPRB, 0.30638E+01_JPRB, 0.30896E+01_JPRB, 0.31274E+01_JPRB, 0.30785E+01_JPRB ,&
&  0.28569E+01_JPRB, 0.27180E+01_JPRB, 0.28632E+01_JPRB, 0.22872E+01_JPRB, 0.19047E+01_JPRB, 0.16924E+01_JPRB, 0.15905E+01_JPRB ,&
&  0.14945E+01_JPRB, 0.13984E+01_JPRB, 0.13402E+01_JPRB, 0.12634E+01_JPRB, 0.11623E+01_JPRB, 0.11023E+01_JPRB, 0.10864E+01_JPRB ,&
&  0.10728E+01_JPRB, 0.10502E+01_JPRB, 0.10294E+01_JPRB, 0.10029E+01_JPRB, 0.96399E+00_JPRB, 0.91997E+00_JPRB, 0.89352E+00_JPRB ,&
&  0.87920E+00_JPRB, 0.86524E+00_JPRB, 0.85480E+00_JPRB, 0.85092E+00_JPRB, 0.85884E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.89388E+00_JPRB, 0.91460E+00_JPRB, 0.91331E+00_JPRB, 0.91197E+00_JPRB, 0.91043E+00_JPRB, 0.89137E+00_JPRB, 0.87624E+00_JPRB ,&
&  0.86880E+00_JPRB, 0.86413E+00_JPRB, 0.85414E+00_JPRB, 0.84051E+00_JPRB, 0.82418E+00_JPRB, 0.81326E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.79206E+00_JPRB, 0.73271E+00_JPRB, 0.70040E+00_JPRB, 0.67439E+00_JPRB, 0.64988E+00_JPRB, 0.63716E+00_JPRB, 0.62894E+00_JPRB ,&
&  0.61563E+00_JPRB, 0.60703E+00_JPRB, 0.60104E+00_JPRB, 0.58920E+00_JPRB, 0.57553E+00_JPRB, 0.55601E+00_JPRB, 0.55368E+00_JPRB ,&
&  0.56733E+00_JPRB, 0.55340E+00_JPRB, 0.53482E+00_JPRB, 0.51354E+00_JPRB, 0.48085E+00_JPRB, 0.45801E+00_JPRB, 0.44530E+00_JPRB ,&
&  0.43419E+00_JPRB, 0.40823E+00_JPRB, 0.40951E+00_JPRB, 0.41241E+00_JPRB, 0.41559E+00_JPRB, 0.43544E+00_JPRB, 0.47333E+00_JPRB ,&
&  0.51514E+00_JPRB, 0.63053E+00_JPRB, 0.67413E+00_JPRB, 0.72623E+00_JPRB, 0.76315E+00_JPRB, 0.79271E+00_JPRB, 0.81993E+00_JPRB ,&
&  0.85002E+00_JPRB, 0.89284E+00_JPRB, 0.94496E+00_JPRB, 0.10150E+01_JPRB, 0.10956E+01_JPRB, 0.11604E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.12991E+01_JPRB, 0.12974E+01_JPRB, 0.13274E+01_JPRB, 0.13480E+01_JPRB, 0.13935E+01_JPRB, 0.14945E+01_JPRB, 0.15458E+01_JPRB ,&
&  0.16684E+01_JPRB, 0.18442E+01_JPRB, 0.21158E+01_JPRB, 0.23845E+01_JPRB, 0.25342E+01_JPRB, 0.26745E+01_JPRB, 0.27201E+01_JPRB ,&
&  0.25804E+01_JPRB, 0.25024E+01_JPRB, 0.25734E+01_JPRB, 0.23528E+01_JPRB, 0.21067E+01_JPRB, 0.18971E+01_JPRB, 0.17433E+01_JPRB ,&
&  0.15963E+01_JPRB, 0.14541E+01_JPRB, 0.13585E+01_JPRB, 0.12789E+01_JPRB, 0.11973E+01_JPRB, 0.11238E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.10839E+01_JPRB, 0.10617E+01_JPRB, 0.10330E+01_JPRB, 0.10006E+01_JPRB, 0.95288E+00_JPRB, 0.90663E+00_JPRB, 0.86920E+00_JPRB ,&
&  0.85622E+00_JPRB, 0.84739E+00_JPRB, 0.85109E+00_JPRB, 0.85517E+00_JPRB, 0.86966E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.89429E+00_JPRB, 0.89881E+00_JPRB, 0.88818E+00_JPRB, 0.87617E+00_JPRB, 0.86981E+00_JPRB, 0.85612E+00_JPRB, 0.85255E+00_JPRB ,&
&  0.85165E+00_JPRB, 0.85011E+00_JPRB, 0.83703E+00_JPRB, 0.82082E+00_JPRB, 0.79979E+00_JPRB, 0.78602E+00_JPRB, 0.76906E+00_JPRB ,&
&  0.75436E+00_JPRB, 0.73443E+00_JPRB, 0.71352E+00_JPRB, 0.69212E+00_JPRB, 0.67533E+00_JPRB, 0.65616E+00_JPRB, 0.64030E+00_JPRB ,&
&  0.62482E+00_JPRB, 0.60572E+00_JPRB, 0.58805E+00_JPRB, 0.58314E+00_JPRB, 0.57297E+00_JPRB, 0.56244E+00_JPRB, 0.56408E+00_JPRB ,&
&  0.56427E+00_JPRB, 0.55678E+00_JPRB, 0.55081E+00_JPRB, 0.53062E+00_JPRB, 0.49248E+00_JPRB, 0.41828E+00_JPRB, 0.43186E+00_JPRB ,&
&  0.42522E+00_JPRB, 0.41303E+00_JPRB, 0.41692E+00_JPRB, 0.42846E+00_JPRB, 0.44017E+00_JPRB, 0.47344E+00_JPRB, 0.48033E+00_JPRB ,&
&  0.47300E+00_JPRB, 0.62788E+00_JPRB, 0.66143E+00_JPRB, 0.69878E+00_JPRB, 0.73105E+00_JPRB, 0.75586E+00_JPRB, 0.78031E+00_JPRB ,&
&  0.80718E+00_JPRB, 0.84260E+00_JPRB, 0.87926E+00_JPRB, 0.93435E+00_JPRB, 0.99907E+00_JPRB, 0.10814E+01_JPRB, 0.11524E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.12230E+01_JPRB, 0.12616E+01_JPRB, 0.12983E+01_JPRB, 0.13444E+01_JPRB, 0.14176E+01_JPRB, 0.15089E+01_JPRB ,&
&  0.16344E+01_JPRB, 0.17176E+01_JPRB, 0.18840E+01_JPRB, 0.20226E+01_JPRB, 0.21142E+01_JPRB, 0.22238E+01_JPRB, 0.23181E+01_JPRB ,&
&  0.22959E+01_JPRB, 0.22953E+01_JPRB, 0.22554E+01_JPRB, 0.20741E+01_JPRB, 0.18698E+01_JPRB, 0.17262E+01_JPRB, 0.15943E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.13824E+01_JPRB, 0.12763E+01_JPRB, 0.11895E+01_JPRB, 0.11201E+01_JPRB, 0.10656E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.99687E+00_JPRB, 0.96755E+00_JPRB, 0.94049E+00_JPRB, 0.92541E+00_JPRB, 0.90399E+00_JPRB, 0.88710E+00_JPRB, 0.87329E+00_JPRB ,&
&  0.86914E+00_JPRB, 0.86686E+00_JPRB, 0.87009E+00_JPRB, 0.87525E+00_JPRB, 0.87940E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.88857E+00_JPRB, 0.89279E+00_JPRB, 0.89356E+00_JPRB, 0.88016E+00_JPRB, 0.87353E+00_JPRB, 0.86680E+00_JPRB, 0.87083E+00_JPRB ,&
&  0.86178E+00_JPRB, 0.85279E+00_JPRB, 0.82557E+00_JPRB, 0.80794E+00_JPRB, 0.78914E+00_JPRB, 0.77971E+00_JPRB, 0.77033E+00_JPRB ,&
&  0.76458E+00_JPRB, 0.75557E+00_JPRB, 0.75079E+00_JPRB, 0.73623E+00_JPRB, 0.72173E+00_JPRB, 0.69645E+00_JPRB, 0.66987E+00_JPRB ,&
&  0.64312E+00_JPRB, 0.61576E+00_JPRB, 0.59047E+00_JPRB, 0.58684E+00_JPRB, 0.57599E+00_JPRB, 0.56918E+00_JPRB, 0.56209E+00_JPRB ,&
&  0.55718E+00_JPRB, 0.50345E+00_JPRB, 0.51322E+00_JPRB, 0.49374E+00_JPRB, 0.47284E+00_JPRB, 0.45026E+00_JPRB, 0.43264E+00_JPRB ,&
&  0.43488E+00_JPRB, 0.43658E+00_JPRB, 0.44995E+00_JPRB, 0.47430E+00_JPRB, 0.48663E+00_JPRB, 0.50901E+00_JPRB, 0.52439E+00_JPRB ,&
&  0.52295E+00_JPRB, 0.63033E+00_JPRB, 0.68540E+00_JPRB, 0.71938E+00_JPRB, 0.75050E+00_JPRB, 0.77744E+00_JPRB, 0.79997E+00_JPRB ,&
&  0.83413E+00_JPRB, 0.87002E+00_JPRB, 0.91044E+00_JPRB, 0.95075E+00_JPRB, 0.10159E+01_JPRB, 0.10648E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.11547E+01_JPRB, 0.11830E+01_JPRB, 0.12321E+01_JPRB, 0.12928E+01_JPRB, 0.13591E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.15080E+01_JPRB, 0.15878E+01_JPRB, 0.16762E+01_JPRB, 0.17597E+01_JPRB, 0.18344E+01_JPRB, 0.18904E+01_JPRB, 0.19409E+01_JPRB ,&
&  0.19292E+01_JPRB, 0.19028E+01_JPRB, 0.18488E+01_JPRB, 0.17633E+01_JPRB, 0.16495E+01_JPRB, 0.15232E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.13215E+01_JPRB, 0.12396E+01_JPRB, 0.11646E+01_JPRB, 0.10917E+01_JPRB, 0.10313E+01_JPRB, 0.98367E+00_JPRB, 0.95541E+00_JPRB ,&
&  0.92852E+00_JPRB, 0.91011E+00_JPRB, 0.89735E+00_JPRB, 0.88932E+00_JPRB, 0.88667E+00_JPRB, 0.88008E+00_JPRB, 0.88071E+00_JPRB ,&
&  0.88231E+00_JPRB, 0.88524E+00_JPRB, 0.88143E+00_JPRB, 0.88059E+00_JPRB, 0.88332E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.88874E+00_JPRB, 0.89536E+00_JPRB, 0.90875E+00_JPRB, 0.91094E+00_JPRB, 0.91150E+00_JPRB, 0.90426E+00_JPRB, 0.90197E+00_JPRB ,&
&  0.89048E+00_JPRB, 0.87746E+00_JPRB, 0.84831E+00_JPRB, 0.82843E+00_JPRB, 0.81002E+00_JPRB, 0.79900E+00_JPRB, 0.79122E+00_JPRB ,&
&  0.78629E+00_JPRB, 0.77934E+00_JPRB, 0.77087E+00_JPRB, 0.75617E+00_JPRB, 0.73652E+00_JPRB, 0.71242E+00_JPRB, 0.68231E+00_JPRB ,&
&  0.65692E+00_JPRB, 0.63789E+00_JPRB, 0.61120E+00_JPRB, 0.61378E+00_JPRB, 0.59909E+00_JPRB, 0.58877E+00_JPRB, 0.58244E+00_JPRB ,&
&  0.56633E+00_JPRB, 0.55064E+00_JPRB, 0.52604E+00_JPRB, 0.50993E+00_JPRB, 0.49671E+00_JPRB, 0.48123E+00_JPRB, 0.49400E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.53925E+00_JPRB, 0.55666E+00_JPRB, 0.56474E+00_JPRB, 0.57231E+00_JPRB, 0.57327E+00_JPRB, 0.58413E+00_JPRB ,&
&  0.59966E+00_JPRB, 0.62283E+00_JPRB, 0.69265E+00_JPRB, 0.73716E+00_JPRB, 0.76404E+00_JPRB, 0.79376E+00_JPRB, 0.81819E+00_JPRB ,&
&  0.85030E+00_JPRB, 0.87380E+00_JPRB, 0.89265E+00_JPRB, 0.91454E+00_JPRB, 0.95148E+00_JPRB, 0.99505E+00_JPRB, 0.10331E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.10866E+01_JPRB, 0.11133E+01_JPRB, 0.11437E+01_JPRB, 0.11845E+01_JPRB, 0.12290E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.13761E+01_JPRB, 0.14160E+01_JPRB, 0.14545E+01_JPRB, 0.14797E+01_JPRB, 0.14940E+01_JPRB, 0.15024E+01_JPRB ,&
&  0.14866E+01_JPRB, 0.14609E+01_JPRB, 0.14199E+01_JPRB, 0.13508E+01_JPRB, 0.12963E+01_JPRB, 0.12637E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.11094E+01_JPRB, 0.10642E+01_JPRB, 0.10178E+01_JPRB, 0.97893E+00_JPRB, 0.94370E+00_JPRB, 0.92072E+00_JPRB ,&
&  0.90275E+00_JPRB, 0.89144E+00_JPRB, 0.88596E+00_JPRB, 0.88341E+00_JPRB, 0.88642E+00_JPRB, 0.88723E+00_JPRB, 0.89473E+00_JPRB ,&
&  0.89603E+00_JPRB, 0.89391E+00_JPRB, 0.88662E+00_JPRB, 0.88374E+00_JPRB, 0.88336E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.89562E+00_JPRB, 0.89638E+00_JPRB, 0.90075E+00_JPRB, 0.90270E+00_JPRB, 0.90449E+00_JPRB, 0.90267E+00_JPRB, 0.90100E+00_JPRB ,&
&  0.89474E+00_JPRB, 0.88237E+00_JPRB, 0.86133E+00_JPRB, 0.84293E+00_JPRB, 0.82606E+00_JPRB, 0.81426E+00_JPRB, 0.80493E+00_JPRB ,&
&  0.79590E+00_JPRB, 0.78889E+00_JPRB, 0.78030E+00_JPRB, 0.77007E+00_JPRB, 0.75647E+00_JPRB, 0.74136E+00_JPRB, 0.72425E+00_JPRB ,&
&  0.71130E+00_JPRB, 0.69840E+00_JPRB, 0.68749E+00_JPRB, 0.67937E+00_JPRB, 0.67560E+00_JPRB, 0.64062E+00_JPRB, 0.63264E+00_JPRB ,&
&  0.64068E+00_JPRB, 0.62544E+00_JPRB, 0.61201E+00_JPRB, 0.59214E+00_JPRB, 0.59231E+00_JPRB, 0.60225E+00_JPRB, 0.60871E+00_JPRB ,&
&  0.58970E+00_JPRB, 0.62390E+00_JPRB, 0.62766E+00_JPRB, 0.62915E+00_JPRB, 0.62640E+00_JPRB, 0.61993E+00_JPRB, 0.63489E+00_JPRB ,&
&  0.67283E+00_JPRB, 0.68895E+00_JPRB, 0.72270E+00_JPRB, 0.76250E+00_JPRB, 0.79282E+00_JPRB, 0.82479E+00_JPRB, 0.85629E+00_JPRB ,&
&  0.88946E+00_JPRB, 0.91415E+00_JPRB, 0.93119E+00_JPRB, 0.94760E+00_JPRB, 0.97287E+00_JPRB, 0.99852E+00_JPRB, 0.10218E+01_JPRB ,&
&  0.10375E+01_JPRB, 0.10465E+01_JPRB, 0.10568E+01_JPRB, 0.10669E+01_JPRB, 0.10805E+01_JPRB, 0.10917E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11182E+01_JPRB, 0.11218E+01_JPRB, 0.11227E+01_JPRB, 0.11213E+01_JPRB, 0.11168E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.11085E+01_JPRB, 0.10964E+01_JPRB, 0.10799E+01_JPRB, 0.10586E+01_JPRB, 0.10409E+01_JPRB, 0.10268E+01_JPRB, 0.10112E+01_JPRB ,&
&  0.99554E+00_JPRB, 0.98003E+00_JPRB, 0.96524E+00_JPRB, 0.94946E+00_JPRB, 0.93451E+00_JPRB, 0.91966E+00_JPRB, 0.90888E+00_JPRB ,&
&  0.90219E+00_JPRB, 0.89721E+00_JPRB, 0.89694E+00_JPRB, 0.89561E+00_JPRB, 0.89921E+00_JPRB, 0.89970E+00_JPRB, 0.90355E+00_JPRB ,&
&  0.90501E+00_JPRB, 0.90305E+00_JPRB, 0.89829E+00_JPRB, 0.89541E+00_JPRB, 0.89364E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.88557E+00_JPRB, 0.88376E+00_JPRB, 0.88271E+00_JPRB, 0.88302E+00_JPRB, 0.88409E+00_JPRB, 0.88439E+00_JPRB, 0.88342E+00_JPRB ,&
&  0.87997E+00_JPRB, 0.87150E+00_JPRB, 0.86153E+00_JPRB, 0.85067E+00_JPRB, 0.83817E+00_JPRB, 0.82631E+00_JPRB, 0.81889E+00_JPRB ,&
&  0.81157E+00_JPRB, 0.80438E+00_JPRB, 0.79684E+00_JPRB, 0.79086E+00_JPRB, 0.78378E+00_JPRB, 0.77622E+00_JPRB, 0.76916E+00_JPRB ,&
&  0.76420E+00_JPRB, 0.75871E+00_JPRB, 0.75261E+00_JPRB, 0.74530E+00_JPRB, 0.73978E+00_JPRB, 0.73370E+00_JPRB, 0.72799E+00_JPRB ,&
&  0.72134E+00_JPRB, 0.71758E+00_JPRB, 0.71497E+00_JPRB, 0.71353E+00_JPRB, 0.71567E+00_JPRB, 0.71824E+00_JPRB, 0.71820E+00_JPRB ,&
&  0.71671E+00_JPRB, 0.71200E+00_JPRB, 0.71264E+00_JPRB, 0.71413E+00_JPRB, 0.71781E+00_JPRB, 0.72509E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.75031E+00_JPRB, 0.76856E+00_JPRB, 0.78875E+00_JPRB, 0.80552E+00_JPRB, 0.82253E+00_JPRB, 0.83798E+00_JPRB, 0.84972E+00_JPRB ,&
&  0.86310E+00_JPRB, 0.87321E+00_JPRB, 0.88341E+00_JPRB, 0.89274E+00_JPRB, 0.90501E+00_JPRB, 0.91480E+00_JPRB, 0.92302E+00_JPRB ,&
&  0.92630E+00_JPRB, 0.92580E+00_JPRB, 0.92457E+00_JPRB, 0.92359E+00_JPRB, 0.92047E+00_JPRB, 0.91855E+00_JPRB, 0.91782E+00_JPRB ,&
&  0.91814E+00_JPRB, 0.91806E+00_JPRB, 0.92032E+00_JPRB, 0.92298E+00_JPRB, 0.92599E+00_JPRB, 0.92646E+00_JPRB, 0.92785E+00_JPRB ,&
&  0.92683E+00_JPRB, 0.92486E+00_JPRB, 0.92116E+00_JPRB, 0.91512E+00_JPRB, 0.91013E+00_JPRB, 0.90694E+00_JPRB, 0.90282E+00_JPRB ,&
&  0.89879E+00_JPRB, 0.89566E+00_JPRB, 0.89348E+00_JPRB, 0.89098E+00_JPRB, 0.88699E+00_JPRB, 0.88359E+00_JPRB, 0.88158E+00_JPRB ,&
&  0.88071E+00_JPRB, 0.87930E+00_JPRB, 0.88032E+00_JPRB, 0.88155E+00_JPRB, 0.88722E+00_JPRB, 0.88964E+00_JPRB, 0.89188E+00_JPRB ,&
&  0.89266E+00_JPRB, 0.89474E+00_JPRB, 0.89416E+00_JPRB, 0.89152E+00_JPRB, 0.88804E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.88314E+00_JPRB, 0.88109E+00_JPRB, 0.87917E+00_JPRB, 0.87772E+00_JPRB, 0.87674E+00_JPRB, 0.87594E+00_JPRB, 0.87359E+00_JPRB ,&
&  0.87062E+00_JPRB, 0.86673E+00_JPRB, 0.86268E+00_JPRB, 0.85782E+00_JPRB, 0.85184E+00_JPRB, 0.84445E+00_JPRB, 0.83801E+00_JPRB ,&
&  0.83220E+00_JPRB, 0.82753E+00_JPRB, 0.82252E+00_JPRB, 0.81885E+00_JPRB, 0.81515E+00_JPRB, 0.81176E+00_JPRB, 0.80769E+00_JPRB ,&
&  0.80425E+00_JPRB, 0.80096E+00_JPRB, 0.79784E+00_JPRB, 0.79512E+00_JPRB, 0.79382E+00_JPRB, 0.79221E+00_JPRB, 0.79045E+00_JPRB ,&
&  0.78883E+00_JPRB, 0.78767E+00_JPRB, 0.78656E+00_JPRB, 0.78551E+00_JPRB, 0.78483E+00_JPRB, 0.78498E+00_JPRB, 0.78511E+00_JPRB ,&
&  0.78560E+00_JPRB, 0.78724E+00_JPRB, 0.78985E+00_JPRB, 0.79225E+00_JPRB, 0.79495E+00_JPRB, 0.79888E+00_JPRB, 0.80401E+00_JPRB ,&
&  0.80863E+00_JPRB, 0.81304E+00_JPRB, 0.81795E+00_JPRB, 0.82360E+00_JPRB, 0.82877E+00_JPRB, 0.83391E+00_JPRB, 0.83841E+00_JPRB ,&
&  0.84290E+00_JPRB, 0.84658E+00_JPRB, 0.85039E+00_JPRB, 0.85222E+00_JPRB, 0.85530E+00_JPRB, 0.85782E+00_JPRB, 0.86089E+00_JPRB ,&
&  0.86181E+00_JPRB, 0.86251E+00_JPRB, 0.86353E+00_JPRB, 0.86498E+00_JPRB, 0.86645E+00_JPRB, 0.86802E+00_JPRB, 0.86969E+00_JPRB ,&
&  0.87191E+00_JPRB, 0.87353E+00_JPRB, 0.87544E+00_JPRB, 0.87717E+00_JPRB, 0.87926E+00_JPRB, 0.88034E+00_JPRB, 0.88176E+00_JPRB ,&
&  0.88267E+00_JPRB, 0.88357E+00_JPRB, 0.88346E+00_JPRB, 0.88328E+00_JPRB, 0.88318E+00_JPRB, 0.88335E+00_JPRB, 0.88369E+00_JPRB ,&
&  0.88417E+00_JPRB, 0.88448E+00_JPRB, 0.88476E+00_JPRB, 0.88451E+00_JPRB, 0.88359E+00_JPRB, 0.88285E+00_JPRB, 0.88239E+00_JPRB ,&
&  0.88265E+00_JPRB, 0.88270E+00_JPRB, 0.88337E+00_JPRB, 0.88423E+00_JPRB, 0.88639E+00_JPRB, 0.88725E+00_JPRB, 0.88813E+00_JPRB ,&
&  0.88835E+00_JPRB, 0.88858E+00_JPRB, 0.88782E+00_JPRB, 0.88660E+00_JPRB, 0.88484E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB ,&
&  0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB, 0.85402E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB ,&
&  0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB, 0.13579E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.13777E+00_JPRB, 0.13846E+00_JPRB, 0.13896E+00_JPRB, 0.13937E+00_JPRB, 0.13924E+00_JPRB, 0.13932E+00_JPRB, 0.13941E+00_JPRB ,&
&  0.13942E+00_JPRB, 0.13942E+00_JPRB, 0.13966E+00_JPRB, 0.13989E+00_JPRB, 0.14012E+00_JPRB, 0.14038E+00_JPRB, 0.14066E+00_JPRB ,&
&  0.14078E+00_JPRB, 0.14103E+00_JPRB, 0.14006E+00_JPRB, 0.13929E+00_JPRB, 0.13882E+00_JPRB, 0.13855E+00_JPRB, 0.13842E+00_JPRB ,&
&  0.13831E+00_JPRB, 0.13825E+00_JPRB, 0.13815E+00_JPRB, 0.13887E+00_JPRB, 0.13935E+00_JPRB, 0.13978E+00_JPRB, 0.14012E+00_JPRB ,&
&  0.14027E+00_JPRB, 0.14047E+00_JPRB, 0.14064E+00_JPRB, 0.14081E+00_JPRB, 0.14101E+00_JPRB, 0.14097E+00_JPRB, 0.14085E+00_JPRB ,&
&  0.14067E+00_JPRB, 0.14037E+00_JPRB, 0.14003E+00_JPRB, 0.13971E+00_JPRB, 0.13945E+00_JPRB, 0.13916E+00_JPRB, 0.13883E+00_JPRB ,&
&  0.13852E+00_JPRB, 0.13815E+00_JPRB, 0.13780E+00_JPRB, 0.13746E+00_JPRB, 0.13714E+00_JPRB, 0.13686E+00_JPRB, 0.13656E+00_JPRB ,&
&  0.13635E+00_JPRB, 0.13615E+00_JPRB, 0.13596E+00_JPRB, 0.13581E+00_JPRB, 0.13560E+00_JPRB, 0.13551E+00_JPRB, 0.13532E+00_JPRB ,&
&  0.13524E+00_JPRB, 0.13516E+00_JPRB, 0.13509E+00_JPRB, 0.13501E+00_JPRB, 0.13499E+00_JPRB, 0.13496E+00_JPRB, 0.13493E+00_JPRB ,&
&  0.13489E+00_JPRB, 0.13498E+00_JPRB, 0.13486E+00_JPRB, 0.13472E+00_JPRB, 0.13455E+00_JPRB, 0.13439E+00_JPRB, 0.13424E+00_JPRB ,&
&  0.13411E+00_JPRB, 0.13397E+00_JPRB, 0.13388E+00_JPRB, 0.13388E+00_JPRB, 0.13390E+00_JPRB, 0.13394E+00_JPRB, 0.13399E+00_JPRB ,&
&  0.13406E+00_JPRB, 0.13415E+00_JPRB, 0.13424E+00_JPRB, 0.13421E+00_JPRB, 0.13423E+00_JPRB, 0.13425E+00_JPRB, 0.13427E+00_JPRB ,&
&  0.13429E+00_JPRB, 0.13434E+00_JPRB, 0.13442E+00_JPRB, 0.13452E+00_JPRB, 0.13473E+00_JPRB, 0.13482E+00_JPRB, 0.13496E+00_JPRB ,&
&  0.13515E+00_JPRB, 0.13550E+00_JPRB, 0.13580E+00_JPRB, 0.13613E+00_JPRB, 0.13652E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.14666E+00_JPRB, 0.14850E+00_JPRB, 0.15000E+00_JPRB, 0.15197E+00_JPRB, 0.14874E+00_JPRB, 0.15577E+00_JPRB, 0.15339E+00_JPRB ,&
&  0.15091E+00_JPRB, 0.14620E+00_JPRB, 0.14613E+00_JPRB, 0.14521E+00_JPRB, 0.14399E+00_JPRB, 0.14324E+00_JPRB, 0.14292E+00_JPRB ,&
&  0.14279E+00_JPRB, 0.14246E+00_JPRB, 0.14270E+00_JPRB, 0.14288E+00_JPRB, 0.14301E+00_JPRB, 0.14310E+00_JPRB, 0.14331E+00_JPRB ,&
&  0.14374E+00_JPRB, 0.14455E+00_JPRB, 0.14539E+00_JPRB, 0.14394E+00_JPRB, 0.14386E+00_JPRB, 0.14421E+00_JPRB, 0.14488E+00_JPRB ,&
&  0.14547E+00_JPRB, 0.14652E+00_JPRB, 0.14728E+00_JPRB, 0.14781E+00_JPRB, 0.14916E+00_JPRB, 0.14916E+00_JPRB, 0.14897E+00_JPRB ,&
&  0.14858E+00_JPRB, 0.14784E+00_JPRB, 0.14675E+00_JPRB, 0.14575E+00_JPRB, 0.14471E+00_JPRB, 0.14337E+00_JPRB, 0.14208E+00_JPRB ,&
&  0.14102E+00_JPRB, 0.14015E+00_JPRB, 0.13905E+00_JPRB, 0.13799E+00_JPRB, 0.13701E+00_JPRB, 0.13607E+00_JPRB, 0.13525E+00_JPRB ,&
&  0.13466E+00_JPRB, 0.13416E+00_JPRB, 0.13365E+00_JPRB, 0.13325E+00_JPRB, 0.13282E+00_JPRB, 0.13242E+00_JPRB, 0.13206E+00_JPRB ,&
&  0.13216E+00_JPRB, 0.13224E+00_JPRB, 0.13232E+00_JPRB, 0.13238E+00_JPRB, 0.13269E+00_JPRB, 0.13273E+00_JPRB, 0.13278E+00_JPRB ,&
&  0.13279E+00_JPRB, 0.13268E+00_JPRB, 0.13258E+00_JPRB, 0.13240E+00_JPRB, 0.13218E+00_JPRB, 0.13193E+00_JPRB, 0.13182E+00_JPRB ,&
&  0.13182E+00_JPRB, 0.13192E+00_JPRB, 0.13221E+00_JPRB, 0.13240E+00_JPRB, 0.13259E+00_JPRB, 0.13275E+00_JPRB, 0.13313E+00_JPRB ,&
&  0.13353E+00_JPRB, 0.13393E+00_JPRB, 0.13431E+00_JPRB, 0.13425E+00_JPRB, 0.13447E+00_JPRB, 0.13469E+00_JPRB, 0.13489E+00_JPRB ,&
&  0.13502E+00_JPRB, 0.13545E+00_JPRB, 0.13610E+00_JPRB, 0.13697E+00_JPRB, 0.13855E+00_JPRB, 0.13993E+00_JPRB, 0.14122E+00_JPRB ,&
&  0.14259E+00_JPRB, 0.14531E+00_JPRB, 0.14650E+00_JPRB, 0.14700E+00_JPRB, 0.14717E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.15634E+00_JPRB, 0.15814E+00_JPRB, 0.15927E+00_JPRB, 0.16007E+00_JPRB, 0.16115E+00_JPRB, 0.16162E+00_JPRB, 0.16205E+00_JPRB ,&
&  0.16027E+00_JPRB, 0.15963E+00_JPRB, 0.15856E+00_JPRB, 0.15724E+00_JPRB, 0.15625E+00_JPRB, 0.15410E+00_JPRB, 0.15190E+00_JPRB ,&
&  0.14943E+00_JPRB, 0.14720E+00_JPRB, 0.14574E+00_JPRB, 0.14432E+00_JPRB, 0.14293E+00_JPRB, 0.14177E+00_JPRB, 0.14320E+00_JPRB ,&
&  0.14404E+00_JPRB, 0.14458E+00_JPRB, 0.14570E+00_JPRB, 0.14578E+00_JPRB, 0.14705E+00_JPRB, 0.14906E+00_JPRB, 0.15172E+00_JPRB ,&
&  0.15366E+00_JPRB, 0.15645E+00_JPRB, 0.15812E+00_JPRB, 0.15897E+00_JPRB, 0.16008E+00_JPRB, 0.16083E+00_JPRB, 0.16051E+00_JPRB ,&
&  0.15930E+00_JPRB, 0.15768E+00_JPRB, 0.15583E+00_JPRB, 0.15347E+00_JPRB, 0.15079E+00_JPRB, 0.14817E+00_JPRB, 0.14557E+00_JPRB ,&
&  0.14361E+00_JPRB, 0.14184E+00_JPRB, 0.14002E+00_JPRB, 0.13838E+00_JPRB, 0.13683E+00_JPRB, 0.13543E+00_JPRB, 0.13403E+00_JPRB ,&
&  0.13290E+00_JPRB, 0.13201E+00_JPRB, 0.13093E+00_JPRB, 0.13030E+00_JPRB, 0.12968E+00_JPRB, 0.12938E+00_JPRB, 0.12921E+00_JPRB ,&
&  0.12948E+00_JPRB, 0.12987E+00_JPRB, 0.13026E+00_JPRB, 0.13047E+00_JPRB, 0.13067E+00_JPRB, 0.13076E+00_JPRB, 0.13093E+00_JPRB ,&
&  0.13110E+00_JPRB, 0.13130E+00_JPRB, 0.13095E+00_JPRB, 0.13063E+00_JPRB, 0.13021E+00_JPRB, 0.12974E+00_JPRB, 0.12961E+00_JPRB ,&
&  0.12987E+00_JPRB, 0.13011E+00_JPRB, 0.13001E+00_JPRB, 0.13028E+00_JPRB, 0.13072E+00_JPRB, 0.13118E+00_JPRB, 0.13231E+00_JPRB ,&
&  0.13292E+00_JPRB, 0.13352E+00_JPRB, 0.13408E+00_JPRB, 0.13453E+00_JPRB, 0.13521E+00_JPRB, 0.13578E+00_JPRB, 0.13648E+00_JPRB ,&
&  0.13731E+00_JPRB, 0.13816E+00_JPRB, 0.13931E+00_JPRB, 0.14075E+00_JPRB, 0.14411E+00_JPRB, 0.14644E+00_JPRB, 0.14859E+00_JPRB ,&
&  0.14983E+00_JPRB, 0.14743E+00_JPRB, 0.15464E+00_JPRB, 0.15417E+00_JPRB, 0.15415E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.16212E+00_JPRB, 0.16455E+00_JPRB, 0.16343E+00_JPRB, 0.16337E+00_JPRB, 0.16295E+00_JPRB, 0.16291E+00_JPRB, 0.16204E+00_JPRB ,&
&  0.16183E+00_JPRB, 0.15974E+00_JPRB, 0.15923E+00_JPRB, 0.15751E+00_JPRB, 0.15570E+00_JPRB, 0.15328E+00_JPRB, 0.15089E+00_JPRB ,&
&  0.14845E+00_JPRB, 0.14629E+00_JPRB, 0.14477E+00_JPRB, 0.14398E+00_JPRB, 0.14273E+00_JPRB, 0.14253E+00_JPRB, 0.14403E+00_JPRB ,&
&  0.14437E+00_JPRB, 0.14536E+00_JPRB, 0.14554E+00_JPRB, 0.14679E+00_JPRB, 0.14841E+00_JPRB, 0.14933E+00_JPRB, 0.15171E+00_JPRB ,&
&  0.15343E+00_JPRB, 0.15490E+00_JPRB, 0.16009E+00_JPRB, 0.16309E+00_JPRB, 0.16490E+00_JPRB, 0.16745E+00_JPRB, 0.16728E+00_JPRB ,&
&  0.16609E+00_JPRB, 0.16286E+00_JPRB, 0.16614E+00_JPRB, 0.16435E+00_JPRB, 0.15820E+00_JPRB, 0.15525E+00_JPRB, 0.15014E+00_JPRB ,&
&  0.14690E+00_JPRB, 0.14326E+00_JPRB, 0.14051E+00_JPRB, 0.13842E+00_JPRB, 0.13576E+00_JPRB, 0.13403E+00_JPRB, 0.13210E+00_JPRB ,&
&  0.13026E+00_JPRB, 0.12939E+00_JPRB, 0.12808E+00_JPRB, 0.12783E+00_JPRB, 0.12787E+00_JPRB, 0.12817E+00_JPRB, 0.12871E+00_JPRB ,&
&  0.12947E+00_JPRB, 0.13018E+00_JPRB, 0.13085E+00_JPRB, 0.13142E+00_JPRB, 0.13190E+00_JPRB, 0.13221E+00_JPRB, 0.13211E+00_JPRB ,&
&  0.13220E+00_JPRB, 0.13208E+00_JPRB, 0.13164E+00_JPRB, 0.13139E+00_JPRB, 0.13055E+00_JPRB, 0.13014E+00_JPRB, 0.12975E+00_JPRB ,&
&  0.12956E+00_JPRB, 0.12958E+00_JPRB, 0.12952E+00_JPRB, 0.12970E+00_JPRB, 0.12997E+00_JPRB, 0.13060E+00_JPRB, 0.13114E+00_JPRB ,&
&  0.13156E+00_JPRB, 0.13234E+00_JPRB, 0.13281E+00_JPRB, 0.13346E+00_JPRB, 0.13438E+00_JPRB, 0.13485E+00_JPRB, 0.13595E+00_JPRB ,&
&  0.13693E+00_JPRB, 0.13806E+00_JPRB, 0.14006E+00_JPRB, 0.14209E+00_JPRB, 0.14496E+00_JPRB, 0.14869E+00_JPRB, 0.15045E+00_JPRB ,&
&  0.15216E+00_JPRB, 0.15399E+00_JPRB, 0.15325E+00_JPRB, 0.15775E+00_JPRB, 0.15984E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.16402E+00_JPRB, 0.16487E+00_JPRB, 0.16323E+00_JPRB, 0.16371E+00_JPRB, 0.16451E+00_JPRB, 0.16496E+00_JPRB, 0.16543E+00_JPRB ,&
&  0.16307E+00_JPRB, 0.16210E+00_JPRB, 0.15977E+00_JPRB, 0.15755E+00_JPRB, 0.15499E+00_JPRB, 0.15227E+00_JPRB, 0.15016E+00_JPRB ,&
&  0.14831E+00_JPRB, 0.14751E+00_JPRB, 0.14665E+00_JPRB, 0.14726E+00_JPRB, 0.14729E+00_JPRB, 0.14882E+00_JPRB, 0.15007E+00_JPRB ,&
&  0.15082E+00_JPRB, 0.15177E+00_JPRB, 0.15067E+00_JPRB, 0.15064E+00_JPRB, 0.15239E+00_JPRB, 0.15409E+00_JPRB, 0.15671E+00_JPRB ,&
&  0.15951E+00_JPRB, 0.16250E+00_JPRB, 0.16730E+00_JPRB, 0.16910E+00_JPRB, 0.17121E+00_JPRB, 0.17260E+00_JPRB, 0.17266E+00_JPRB ,&
&  0.17242E+00_JPRB, 0.17258E+00_JPRB, 0.17353E+00_JPRB, 0.17340E+00_JPRB, 0.16683E+00_JPRB, 0.16170E+00_JPRB, 0.15632E+00_JPRB ,&
&  0.15126E+00_JPRB, 0.14649E+00_JPRB, 0.14227E+00_JPRB, 0.13944E+00_JPRB, 0.13659E+00_JPRB, 0.13453E+00_JPRB, 0.13291E+00_JPRB ,&
&  0.13125E+00_JPRB, 0.13024E+00_JPRB, 0.12898E+00_JPRB, 0.12857E+00_JPRB, 0.12878E+00_JPRB, 0.12944E+00_JPRB, 0.13035E+00_JPRB ,&
&  0.13144E+00_JPRB, 0.13221E+00_JPRB, 0.13319E+00_JPRB, 0.13412E+00_JPRB, 0.13478E+00_JPRB, 0.13522E+00_JPRB, 0.13502E+00_JPRB ,&
&  0.13522E+00_JPRB, 0.13545E+00_JPRB, 0.13504E+00_JPRB, 0.13496E+00_JPRB, 0.13393E+00_JPRB, 0.13274E+00_JPRB, 0.13151E+00_JPRB ,&
&  0.12989E+00_JPRB, 0.12864E+00_JPRB, 0.12817E+00_JPRB, 0.12788E+00_JPRB, 0.12847E+00_JPRB, 0.12939E+00_JPRB, 0.13001E+00_JPRB ,&
&  0.13035E+00_JPRB, 0.13031E+00_JPRB, 0.13038E+00_JPRB, 0.13143E+00_JPRB, 0.13253E+00_JPRB, 0.13391E+00_JPRB, 0.13534E+00_JPRB ,&
&  0.13586E+00_JPRB, 0.13705E+00_JPRB, 0.13811E+00_JPRB, 0.13938E+00_JPRB, 0.14356E+00_JPRB, 0.14698E+00_JPRB, 0.15354E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.15971E+00_JPRB, 0.16007E+00_JPRB, 0.16362E+00_JPRB, 0.16363E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.16825E+00_JPRB, 0.17021E+00_JPRB, 0.17200E+00_JPRB, 0.17114E+00_JPRB, 0.17132E+00_JPRB, 0.17135E+00_JPRB, 0.17143E+00_JPRB ,&
&  0.17142E+00_JPRB, 0.17085E+00_JPRB, 0.17105E+00_JPRB, 0.16947E+00_JPRB, 0.16852E+00_JPRB, 0.16486E+00_JPRB, 0.16343E+00_JPRB ,&
&  0.16147E+00_JPRB, 0.16090E+00_JPRB, 0.16108E+00_JPRB, 0.16214E+00_JPRB, 0.16259E+00_JPRB, 0.16379E+00_JPRB, 0.16314E+00_JPRB ,&
&  0.16328E+00_JPRB, 0.16315E+00_JPRB, 0.16196E+00_JPRB, 0.16165E+00_JPRB, 0.16212E+00_JPRB, 0.16268E+00_JPRB, 0.16301E+00_JPRB ,&
&  0.16474E+00_JPRB, 0.16692E+00_JPRB, 0.16903E+00_JPRB, 0.17047E+00_JPRB, 0.17564E+00_JPRB, 0.17718E+00_JPRB, 0.17902E+00_JPRB ,&
&  0.18015E+00_JPRB, 0.18164E+00_JPRB, 0.18265E+00_JPRB, 0.18212E+00_JPRB, 0.18099E+00_JPRB, 0.17770E+00_JPRB, 0.17645E+00_JPRB ,&
&  0.16640E+00_JPRB, 0.15966E+00_JPRB, 0.15298E+00_JPRB, 0.14712E+00_JPRB, 0.14294E+00_JPRB, 0.14006E+00_JPRB, 0.13766E+00_JPRB ,&
&  0.13568E+00_JPRB, 0.13378E+00_JPRB, 0.13174E+00_JPRB, 0.13172E+00_JPRB, 0.13241E+00_JPRB, 0.13345E+00_JPRB, 0.13449E+00_JPRB ,&
&  0.13567E+00_JPRB, 0.13637E+00_JPRB, 0.13670E+00_JPRB, 0.13738E+00_JPRB, 0.13756E+00_JPRB, 0.13791E+00_JPRB, 0.13810E+00_JPRB ,&
&  0.13925E+00_JPRB, 0.13965E+00_JPRB, 0.13970E+00_JPRB, 0.14090E+00_JPRB, 0.13940E+00_JPRB, 0.13699E+00_JPRB, 0.13428E+00_JPRB ,&
&  0.13163E+00_JPRB, 0.12996E+00_JPRB, 0.12856E+00_JPRB, 0.12799E+00_JPRB, 0.12830E+00_JPRB, 0.12892E+00_JPRB, 0.12979E+00_JPRB ,&
&  0.13000E+00_JPRB, 0.13056E+00_JPRB, 0.13100E+00_JPRB, 0.13183E+00_JPRB, 0.13272E+00_JPRB, 0.13372E+00_JPRB, 0.13478E+00_JPRB ,&
&  0.13636E+00_JPRB, 0.13820E+00_JPRB, 0.14004E+00_JPRB, 0.14192E+00_JPRB, 0.14492E+00_JPRB, 0.14859E+00_JPRB, 0.15448E+00_JPRB ,&
&  0.16002E+00_JPRB, 0.16800E+00_JPRB, 0.16718E+00_JPRB, 0.16909E+00_JPRB, 0.16927E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.17496E+00_JPRB, 0.17859E+00_JPRB, 0.18019E+00_JPRB, 0.18026E+00_JPRB, 0.17927E+00_JPRB, 0.17845E+00_JPRB, 0.17831E+00_JPRB ,&
&  0.17822E+00_JPRB, 0.17717E+00_JPRB, 0.17637E+00_JPRB, 0.17569E+00_JPRB, 0.17495E+00_JPRB, 0.17419E+00_JPRB, 0.17445E+00_JPRB ,&
&  0.17560E+00_JPRB, 0.17693E+00_JPRB, 0.17783E+00_JPRB, 0.17808E+00_JPRB, 0.17760E+00_JPRB, 0.17787E+00_JPRB, 0.17867E+00_JPRB ,&
&  0.17862E+00_JPRB, 0.17897E+00_JPRB, 0.18222E+00_JPRB, 0.18278E+00_JPRB, 0.18376E+00_JPRB, 0.18391E+00_JPRB, 0.18399E+00_JPRB ,&
&  0.17958E+00_JPRB, 0.17451E+00_JPRB, 0.17135E+00_JPRB, 0.17333E+00_JPRB, 0.18165E+00_JPRB, 0.18336E+00_JPRB, 0.18655E+00_JPRB ,&
&  0.18867E+00_JPRB, 0.18990E+00_JPRB, 0.19121E+00_JPRB, 0.19234E+00_JPRB, 0.19369E+00_JPRB, 0.19274E+00_JPRB, 0.19061E+00_JPRB ,&
&  0.18804E+00_JPRB, 0.18708E+00_JPRB, 0.18396E+00_JPRB, 0.17041E+00_JPRB, 0.16198E+00_JPRB, 0.15722E+00_JPRB, 0.15227E+00_JPRB ,&
&  0.14777E+00_JPRB, 0.14513E+00_JPRB, 0.14434E+00_JPRB, 0.14317E+00_JPRB, 0.14401E+00_JPRB, 0.14576E+00_JPRB, 0.14651E+00_JPRB ,&
&  0.14621E+00_JPRB, 0.14656E+00_JPRB, 0.14624E+00_JPRB, 0.14531E+00_JPRB, 0.14373E+00_JPRB, 0.14272E+00_JPRB, 0.14211E+00_JPRB ,&
&  0.14253E+00_JPRB, 0.14236E+00_JPRB, 0.14611E+00_JPRB, 0.14753E+00_JPRB, 0.14331E+00_JPRB, 0.14167E+00_JPRB, 0.13963E+00_JPRB ,&
&  0.13500E+00_JPRB, 0.13240E+00_JPRB, 0.13096E+00_JPRB, 0.13038E+00_JPRB, 0.13121E+00_JPRB, 0.13245E+00_JPRB, 0.13311E+00_JPRB ,&
&  0.13334E+00_JPRB, 0.13426E+00_JPRB, 0.13472E+00_JPRB, 0.13525E+00_JPRB, 0.13561E+00_JPRB, 0.13638E+00_JPRB, 0.13749E+00_JPRB ,&
&  0.13891E+00_JPRB, 0.14143E+00_JPRB, 0.14393E+00_JPRB, 0.14675E+00_JPRB, 0.14845E+00_JPRB, 0.14979E+00_JPRB, 0.15283E+00_JPRB ,&
&  0.15489E+00_JPRB, 0.16153E+00_JPRB, 0.17129E+00_JPRB, 0.17510E+00_JPRB, 0.17078E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.18667E+00_JPRB, 0.18855E+00_JPRB, 0.18922E+00_JPRB, 0.18928E+00_JPRB, 0.18920E+00_JPRB, 0.18973E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.19028E+00_JPRB, 0.18960E+00_JPRB, 0.18847E+00_JPRB, 0.18660E+00_JPRB, 0.18435E+00_JPRB, 0.18307E+00_JPRB, 0.18410E+00_JPRB ,&
&  0.18498E+00_JPRB, 0.18475E+00_JPRB, 0.18464E+00_JPRB, 0.18434E+00_JPRB, 0.18380E+00_JPRB, 0.18392E+00_JPRB, 0.18504E+00_JPRB ,&
&  0.18641E+00_JPRB, 0.18735E+00_JPRB, 0.18876E+00_JPRB, 0.18999E+00_JPRB, 0.19204E+00_JPRB, 0.19389E+00_JPRB, 0.19472E+00_JPRB ,&
&  0.19260E+00_JPRB, 0.18493E+00_JPRB, 0.17946E+00_JPRB, 0.18192E+00_JPRB, 0.19051E+00_JPRB, 0.19535E+00_JPRB, 0.19832E+00_JPRB ,&
&  0.20054E+00_JPRB, 0.20191E+00_JPRB, 0.20236E+00_JPRB, 0.20362E+00_JPRB, 0.20541E+00_JPRB, 0.20709E+00_JPRB, 0.20656E+00_JPRB ,&
&  0.20451E+00_JPRB, 0.20276E+00_JPRB, 0.20024E+00_JPRB, 0.19599E+00_JPRB, 0.19030E+00_JPRB, 0.18471E+00_JPRB, 0.17994E+00_JPRB ,&
&  0.17575E+00_JPRB, 0.17166E+00_JPRB, 0.16817E+00_JPRB, 0.16592E+00_JPRB, 0.16519E+00_JPRB, 0.16630E+00_JPRB, 0.16609E+00_JPRB ,&
&  0.16578E+00_JPRB, 0.16401E+00_JPRB, 0.16184E+00_JPRB, 0.15932E+00_JPRB, 0.15710E+00_JPRB, 0.15329E+00_JPRB, 0.14869E+00_JPRB ,&
&  0.14628E+00_JPRB, 0.14838E+00_JPRB, 0.15458E+00_JPRB, 0.15985E+00_JPRB, 0.16120E+00_JPRB, 0.15391E+00_JPRB, 0.14977E+00_JPRB ,&
&  0.14575E+00_JPRB, 0.14193E+00_JPRB, 0.13953E+00_JPRB, 0.13892E+00_JPRB, 0.13902E+00_JPRB, 0.13918E+00_JPRB, 0.14003E+00_JPRB ,&
&  0.14141E+00_JPRB, 0.14267E+00_JPRB, 0.14340E+00_JPRB, 0.14416E+00_JPRB, 0.14446E+00_JPRB, 0.14452E+00_JPRB, 0.14462E+00_JPRB ,&
&  0.14598E+00_JPRB, 0.14892E+00_JPRB, 0.15296E+00_JPRB, 0.15755E+00_JPRB, 0.16114E+00_JPRB, 0.16444E+00_JPRB, 0.16761E+00_JPRB ,&
&  0.17102E+00_JPRB, 0.17353E+00_JPRB, 0.17648E+00_JPRB, 0.18031E+00_JPRB, 0.18368E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.19701E+00_JPRB, 0.19919E+00_JPRB, 0.20003E+00_JPRB, 0.19978E+00_JPRB, 0.19943E+00_JPRB, 0.20042E+00_JPRB, 0.20173E+00_JPRB ,&
&  0.20241E+00_JPRB, 0.20250E+00_JPRB, 0.20170E+00_JPRB, 0.20011E+00_JPRB, 0.19898E+00_JPRB, 0.19933E+00_JPRB, 0.19779E+00_JPRB ,&
&  0.19621E+00_JPRB, 0.19612E+00_JPRB, 0.19455E+00_JPRB, 0.19286E+00_JPRB, 0.19041E+00_JPRB, 0.18893E+00_JPRB, 0.18942E+00_JPRB ,&
&  0.19148E+00_JPRB, 0.19218E+00_JPRB, 0.19297E+00_JPRB, 0.19405E+00_JPRB, 0.19602E+00_JPRB, 0.19830E+00_JPRB, 0.19993E+00_JPRB ,&
&  0.20119E+00_JPRB, 0.20071E+00_JPRB, 0.19903E+00_JPRB, 0.20195E+00_JPRB, 0.20494E+00_JPRB, 0.20871E+00_JPRB, 0.21150E+00_JPRB ,&
&  0.21439E+00_JPRB, 0.21667E+00_JPRB, 0.21781E+00_JPRB, 0.21780E+00_JPRB, 0.21800E+00_JPRB, 0.21943E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.22027E+00_JPRB, 0.21951E+00_JPRB, 0.21796E+00_JPRB, 0.21649E+00_JPRB, 0.21314E+00_JPRB, 0.20813E+00_JPRB, 0.20074E+00_JPRB ,&
&  0.19310E+00_JPRB, 0.18698E+00_JPRB, 0.18239E+00_JPRB, 0.17866E+00_JPRB, 0.17606E+00_JPRB, 0.17569E+00_JPRB, 0.17559E+00_JPRB ,&
&  0.17514E+00_JPRB, 0.17473E+00_JPRB, 0.17607E+00_JPRB, 0.17689E+00_JPRB, 0.17343E+00_JPRB, 0.16806E+00_JPRB, 0.16367E+00_JPRB ,&
&  0.16268E+00_JPRB, 0.16403E+00_JPRB, 0.16625E+00_JPRB, 0.16807E+00_JPRB, 0.17107E+00_JPRB, 0.17180E+00_JPRB, 0.17091E+00_JPRB ,&
&  0.16598E+00_JPRB, 0.16042E+00_JPRB, 0.15725E+00_JPRB, 0.15489E+00_JPRB, 0.15338E+00_JPRB, 0.15309E+00_JPRB, 0.15364E+00_JPRB ,&
&  0.15558E+00_JPRB, 0.15752E+00_JPRB, 0.15975E+00_JPRB, 0.16080E+00_JPRB, 0.16046E+00_JPRB, 0.15992E+00_JPRB, 0.15913E+00_JPRB ,&
&  0.16012E+00_JPRB, 0.16401E+00_JPRB, 0.17015E+00_JPRB, 0.17380E+00_JPRB, 0.17742E+00_JPRB, 0.17985E+00_JPRB, 0.18112E+00_JPRB ,&
&  0.18205E+00_JPRB, 0.18270E+00_JPRB, 0.18352E+00_JPRB, 0.18785E+00_JPRB, 0.19349E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.20928E+00_JPRB, 0.21164E+00_JPRB, 0.21299E+00_JPRB, 0.21305E+00_JPRB, 0.21272E+00_JPRB, 0.21352E+00_JPRB, 0.21406E+00_JPRB ,&
&  0.21464E+00_JPRB, 0.21488E+00_JPRB, 0.21386E+00_JPRB, 0.21300E+00_JPRB, 0.21468E+00_JPRB, 0.21423E+00_JPRB, 0.21369E+00_JPRB ,&
&  0.21203E+00_JPRB, 0.21005E+00_JPRB, 0.20716E+00_JPRB, 0.20385E+00_JPRB, 0.19981E+00_JPRB, 0.19723E+00_JPRB, 0.19739E+00_JPRB ,&
&  0.19880E+00_JPRB, 0.19936E+00_JPRB, 0.20023E+00_JPRB, 0.20146E+00_JPRB, 0.20394E+00_JPRB, 0.20707E+00_JPRB, 0.20946E+00_JPRB ,&
&  0.21141E+00_JPRB, 0.21462E+00_JPRB, 0.21664E+00_JPRB, 0.21535E+00_JPRB, 0.21962E+00_JPRB, 0.22374E+00_JPRB, 0.22809E+00_JPRB ,&
&  0.23238E+00_JPRB, 0.23509E+00_JPRB, 0.23552E+00_JPRB, 0.23450E+00_JPRB, 0.23342E+00_JPRB, 0.23384E+00_JPRB, 0.23358E+00_JPRB ,&
&  0.23332E+00_JPRB, 0.23486E+00_JPRB, 0.23696E+00_JPRB, 0.23745E+00_JPRB, 0.23565E+00_JPRB, 0.23143E+00_JPRB, 0.22498E+00_JPRB ,&
&  0.21676E+00_JPRB, 0.20812E+00_JPRB, 0.20113E+00_JPRB, 0.19559E+00_JPRB, 0.19130E+00_JPRB, 0.18776E+00_JPRB, 0.18486E+00_JPRB ,&
&  0.18168E+00_JPRB, 0.17945E+00_JPRB, 0.17936E+00_JPRB, 0.17998E+00_JPRB, 0.18025E+00_JPRB, 0.17902E+00_JPRB, 0.17812E+00_JPRB ,&
&  0.17724E+00_JPRB, 0.17766E+00_JPRB, 0.17845E+00_JPRB, 0.17874E+00_JPRB, 0.17884E+00_JPRB, 0.17848E+00_JPRB, 0.17813E+00_JPRB ,&
&  0.17885E+00_JPRB, 0.17909E+00_JPRB, 0.17823E+00_JPRB, 0.17655E+00_JPRB, 0.17511E+00_JPRB, 0.17481E+00_JPRB, 0.17494E+00_JPRB ,&
&  0.17538E+00_JPRB, 0.17636E+00_JPRB, 0.17789E+00_JPRB, 0.17950E+00_JPRB, 0.18075E+00_JPRB, 0.18128E+00_JPRB, 0.18141E+00_JPRB ,&
&  0.18151E+00_JPRB, 0.18127E+00_JPRB, 0.18200E+00_JPRB, 0.18463E+00_JPRB, 0.18754E+00_JPRB, 0.18996E+00_JPRB, 0.19136E+00_JPRB ,&
&  0.19111E+00_JPRB, 0.19195E+00_JPRB, 0.19515E+00_JPRB, 0.20116E+00_JPRB, 0.20571E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.22597E+00_JPRB, 0.22855E+00_JPRB, 0.23129E+00_JPRB, 0.23167E+00_JPRB, 0.23020E+00_JPRB, 0.22951E+00_JPRB, 0.22971E+00_JPRB ,&
&  0.23017E+00_JPRB, 0.23043E+00_JPRB, 0.22937E+00_JPRB, 0.22933E+00_JPRB, 0.22941E+00_JPRB, 0.22851E+00_JPRB, 0.22776E+00_JPRB ,&
&  0.22641E+00_JPRB, 0.22347E+00_JPRB, 0.21938E+00_JPRB, 0.21435E+00_JPRB, 0.21058E+00_JPRB, 0.20853E+00_JPRB, 0.20814E+00_JPRB ,&
&  0.20874E+00_JPRB, 0.20923E+00_JPRB, 0.21173E+00_JPRB, 0.21400E+00_JPRB, 0.21689E+00_JPRB, 0.22068E+00_JPRB, 0.22463E+00_JPRB ,&
&  0.22761E+00_JPRB, 0.23066E+00_JPRB, 0.23361E+00_JPRB, 0.23631E+00_JPRB, 0.24000E+00_JPRB, 0.24630E+00_JPRB, 0.25321E+00_JPRB ,&
&  0.26004E+00_JPRB, 0.26325E+00_JPRB, 0.26075E+00_JPRB, 0.25737E+00_JPRB, 0.25665E+00_JPRB, 0.25749E+00_JPRB, 0.25830E+00_JPRB ,&
&  0.25894E+00_JPRB, 0.25960E+00_JPRB, 0.25999E+00_JPRB, 0.25886E+00_JPRB, 0.25569E+00_JPRB, 0.25079E+00_JPRB, 0.24460E+00_JPRB ,&
&  0.23799E+00_JPRB, 0.23143E+00_JPRB, 0.22507E+00_JPRB, 0.21851E+00_JPRB, 0.21287E+00_JPRB, 0.20744E+00_JPRB, 0.20244E+00_JPRB ,&
&  0.19774E+00_JPRB, 0.19330E+00_JPRB, 0.18993E+00_JPRB, 0.18785E+00_JPRB, 0.18739E+00_JPRB, 0.18642E+00_JPRB, 0.18647E+00_JPRB ,&
&  0.18606E+00_JPRB, 0.18593E+00_JPRB, 0.18616E+00_JPRB, 0.18708E+00_JPRB, 0.18855E+00_JPRB, 0.18879E+00_JPRB, 0.18774E+00_JPRB ,&
&  0.18712E+00_JPRB, 0.18817E+00_JPRB, 0.18789E+00_JPRB, 0.18566E+00_JPRB, 0.18288E+00_JPRB, 0.18218E+00_JPRB, 0.18343E+00_JPRB ,&
&  0.18588E+00_JPRB, 0.18828E+00_JPRB, 0.19033E+00_JPRB, 0.19220E+00_JPRB, 0.19377E+00_JPRB, 0.19575E+00_JPRB, 0.19757E+00_JPRB ,&
&  0.19906E+00_JPRB, 0.19971E+00_JPRB, 0.20105E+00_JPRB, 0.20248E+00_JPRB, 0.20401E+00_JPRB, 0.20514E+00_JPRB, 0.20507E+00_JPRB ,&
&  0.20471E+00_JPRB, 0.20623E+00_JPRB, 0.21094E+00_JPRB, 0.21823E+00_JPRB, 0.22299E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.24640E+00_JPRB, 0.24868E+00_JPRB, 0.25163E+00_JPRB, 0.25410E+00_JPRB, 0.25511E+00_JPRB, 0.25474E+00_JPRB, 0.25404E+00_JPRB ,&
&  0.25398E+00_JPRB, 0.25404E+00_JPRB, 0.25270E+00_JPRB, 0.25030E+00_JPRB, 0.24779E+00_JPRB, 0.24510E+00_JPRB, 0.24303E+00_JPRB ,&
&  0.24103E+00_JPRB, 0.23894E+00_JPRB, 0.23533E+00_JPRB, 0.23006E+00_JPRB, 0.22459E+00_JPRB, 0.22137E+00_JPRB, 0.21980E+00_JPRB ,&
&  0.21967E+00_JPRB, 0.22015E+00_JPRB, 0.22267E+00_JPRB, 0.22448E+00_JPRB, 0.22699E+00_JPRB, 0.23112E+00_JPRB, 0.23676E+00_JPRB ,&
&  0.24211E+00_JPRB, 0.24717E+00_JPRB, 0.25230E+00_JPRB, 0.25665E+00_JPRB, 0.26156E+00_JPRB, 0.26960E+00_JPRB, 0.27978E+00_JPRB ,&
&  0.28819E+00_JPRB, 0.29309E+00_JPRB, 0.29294E+00_JPRB, 0.29295E+00_JPRB, 0.29481E+00_JPRB, 0.29722E+00_JPRB, 0.29865E+00_JPRB ,&
&  0.29940E+00_JPRB, 0.29682E+00_JPRB, 0.29316E+00_JPRB, 0.28833E+00_JPRB, 0.28335E+00_JPRB, 0.27818E+00_JPRB, 0.27195E+00_JPRB ,&
&  0.26468E+00_JPRB, 0.25668E+00_JPRB, 0.24876E+00_JPRB, 0.24108E+00_JPRB, 0.23359E+00_JPRB, 0.22665E+00_JPRB, 0.22041E+00_JPRB ,&
&  0.21446E+00_JPRB, 0.20989E+00_JPRB, 0.20551E+00_JPRB, 0.20290E+00_JPRB, 0.20137E+00_JPRB, 0.19962E+00_JPRB, 0.19935E+00_JPRB ,&
&  0.19855E+00_JPRB, 0.19811E+00_JPRB, 0.19793E+00_JPRB, 0.19949E+00_JPRB, 0.20251E+00_JPRB, 0.20428E+00_JPRB, 0.20310E+00_JPRB ,&
&  0.19984E+00_JPRB, 0.19777E+00_JPRB, 0.19658E+00_JPRB, 0.19565E+00_JPRB, 0.19337E+00_JPRB, 0.19303E+00_JPRB, 0.19481E+00_JPRB ,&
&  0.19866E+00_JPRB, 0.20294E+00_JPRB, 0.20672E+00_JPRB, 0.20896E+00_JPRB, 0.21123E+00_JPRB, 0.21299E+00_JPRB, 0.21455E+00_JPRB ,&
&  0.21630E+00_JPRB, 0.21759E+00_JPRB, 0.21857E+00_JPRB, 0.21995E+00_JPRB, 0.22229E+00_JPRB, 0.22465E+00_JPRB, 0.22701E+00_JPRB ,&
&  0.22956E+00_JPRB, 0.23342E+00_JPRB, 0.23738E+00_JPRB, 0.24039E+00_JPRB, 0.24406E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.27629E+00_JPRB, 0.27820E+00_JPRB, 0.28042E+00_JPRB, 0.28087E+00_JPRB, 0.28034E+00_JPRB, 0.28035E+00_JPRB, 0.27930E+00_JPRB ,&
&  0.27707E+00_JPRB, 0.27583E+00_JPRB, 0.27605E+00_JPRB, 0.27469E+00_JPRB, 0.27188E+00_JPRB, 0.26842E+00_JPRB, 0.26462E+00_JPRB ,&
&  0.26165E+00_JPRB, 0.25866E+00_JPRB, 0.25369E+00_JPRB, 0.24709E+00_JPRB, 0.24139E+00_JPRB, 0.23774E+00_JPRB, 0.23585E+00_JPRB ,&
&  0.23509E+00_JPRB, 0.23515E+00_JPRB, 0.23483E+00_JPRB, 0.23550E+00_JPRB, 0.23759E+00_JPRB, 0.24235E+00_JPRB, 0.24939E+00_JPRB ,&
&  0.25650E+00_JPRB, 0.26302E+00_JPRB, 0.26979E+00_JPRB, 0.27722E+00_JPRB, 0.28731E+00_JPRB, 0.30254E+00_JPRB, 0.31853E+00_JPRB ,&
&  0.32871E+00_JPRB, 0.33112E+00_JPRB, 0.33084E+00_JPRB, 0.33392E+00_JPRB, 0.34138E+00_JPRB, 0.34754E+00_JPRB, 0.35242E+00_JPRB ,&
&  0.35408E+00_JPRB, 0.35151E+00_JPRB, 0.34755E+00_JPRB, 0.34092E+00_JPRB, 0.33232E+00_JPRB, 0.32229E+00_JPRB, 0.31209E+00_JPRB ,&
&  0.30195E+00_JPRB, 0.29135E+00_JPRB, 0.27966E+00_JPRB, 0.26844E+00_JPRB, 0.25883E+00_JPRB, 0.25039E+00_JPRB, 0.24202E+00_JPRB ,&
&  0.23431E+00_JPRB, 0.22811E+00_JPRB, 0.22319E+00_JPRB, 0.21987E+00_JPRB, 0.21719E+00_JPRB, 0.21510E+00_JPRB, 0.21453E+00_JPRB ,&
&  0.21478E+00_JPRB, 0.21603E+00_JPRB, 0.21658E+00_JPRB, 0.21792E+00_JPRB, 0.22026E+00_JPRB, 0.22125E+00_JPRB, 0.22017E+00_JPRB ,&
&  0.21636E+00_JPRB, 0.21238E+00_JPRB, 0.20862E+00_JPRB, 0.20785E+00_JPRB, 0.20770E+00_JPRB, 0.20875E+00_JPRB, 0.21040E+00_JPRB ,&
&  0.21384E+00_JPRB, 0.21897E+00_JPRB, 0.22350E+00_JPRB, 0.22664E+00_JPRB, 0.23016E+00_JPRB, 0.23324E+00_JPRB, 0.23637E+00_JPRB ,&
&  0.23996E+00_JPRB, 0.24390E+00_JPRB, 0.24925E+00_JPRB, 0.25455E+00_JPRB, 0.25938E+00_JPRB, 0.26327E+00_JPRB, 0.26621E+00_JPRB ,&
&  0.26821E+00_JPRB, 0.26892E+00_JPRB, 0.26944E+00_JPRB, 0.27124E+00_JPRB, 0.27402E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.31039E+00_JPRB, 0.31095E+00_JPRB, 0.31200E+00_JPRB, 0.31255E+00_JPRB, 0.31010E+00_JPRB, 0.30721E+00_JPRB, 0.30377E+00_JPRB ,&
&  0.30218E+00_JPRB, 0.30100E+00_JPRB, 0.30044E+00_JPRB, 0.29737E+00_JPRB, 0.29491E+00_JPRB, 0.29337E+00_JPRB, 0.28980E+00_JPRB ,&
&  0.28658E+00_JPRB, 0.27943E+00_JPRB, 0.27177E+00_JPRB, 0.26607E+00_JPRB, 0.26244E+00_JPRB, 0.26001E+00_JPRB, 0.25514E+00_JPRB ,&
&  0.25174E+00_JPRB, 0.24907E+00_JPRB, 0.24784E+00_JPRB, 0.24804E+00_JPRB, 0.25012E+00_JPRB, 0.25586E+00_JPRB, 0.26551E+00_JPRB ,&
&  0.27239E+00_JPRB, 0.28096E+00_JPRB, 0.28147E+00_JPRB, 0.29541E+00_JPRB, 0.31117E+00_JPRB, 0.33659E+00_JPRB, 0.35929E+00_JPRB ,&
&  0.37047E+00_JPRB, 0.37547E+00_JPRB, 0.38368E+00_JPRB, 0.39268E+00_JPRB, 0.39701E+00_JPRB, 0.40003E+00_JPRB, 0.39970E+00_JPRB ,&
&  0.39677E+00_JPRB, 0.39195E+00_JPRB, 0.38583E+00_JPRB, 0.37770E+00_JPRB, 0.36666E+00_JPRB, 0.35417E+00_JPRB, 0.34152E+00_JPRB ,&
&  0.32977E+00_JPRB, 0.31799E+00_JPRB, 0.30650E+00_JPRB, 0.29474E+00_JPRB, 0.28328E+00_JPRB, 0.27285E+00_JPRB, 0.26237E+00_JPRB ,&
&  0.25352E+00_JPRB, 0.24660E+00_JPRB, 0.24147E+00_JPRB, 0.23781E+00_JPRB, 0.23505E+00_JPRB, 0.23233E+00_JPRB, 0.23097E+00_JPRB ,&
&  0.23095E+00_JPRB, 0.23236E+00_JPRB, 0.23446E+00_JPRB, 0.23620E+00_JPRB, 0.23836E+00_JPRB, 0.23873E+00_JPRB, 0.23625E+00_JPRB ,&
&  0.23361E+00_JPRB, 0.23229E+00_JPRB, 0.22805E+00_JPRB, 0.22552E+00_JPRB, 0.22498E+00_JPRB, 0.22540E+00_JPRB, 0.22719E+00_JPRB ,&
&  0.23076E+00_JPRB, 0.23574E+00_JPRB, 0.24030E+00_JPRB, 0.24421E+00_JPRB, 0.24753E+00_JPRB, 0.25103E+00_JPRB, 0.25522E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.26400E+00_JPRB, 0.27083E+00_JPRB, 0.27842E+00_JPRB, 0.28499E+00_JPRB, 0.29213E+00_JPRB, 0.29675E+00_JPRB ,&
&  0.29879E+00_JPRB, 0.29993E+00_JPRB, 0.30263E+00_JPRB, 0.30644E+00_JPRB, 0.30934E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.35445E+00_JPRB, 0.35252E+00_JPRB, 0.34974E+00_JPRB, 0.34733E+00_JPRB, 0.34566E+00_JPRB, 0.34193E+00_JPRB, 0.33713E+00_JPRB ,&
&  0.33372E+00_JPRB, 0.33051E+00_JPRB, 0.32807E+00_JPRB, 0.32446E+00_JPRB, 0.32236E+00_JPRB, 0.32101E+00_JPRB, 0.31694E+00_JPRB ,&
&  0.31222E+00_JPRB, 0.30577E+00_JPRB, 0.29950E+00_JPRB, 0.29390E+00_JPRB, 0.28859E+00_JPRB, 0.28370E+00_JPRB, 0.27822E+00_JPRB ,&
&  0.27302E+00_JPRB, 0.26919E+00_JPRB, 0.26718E+00_JPRB, 0.26540E+00_JPRB, 0.26674E+00_JPRB, 0.27256E+00_JPRB, 0.28204E+00_JPRB ,&
&  0.28787E+00_JPRB, 0.28965E+00_JPRB, 0.30103E+00_JPRB, 0.31229E+00_JPRB, 0.33552E+00_JPRB, 0.37164E+00_JPRB, 0.39880E+00_JPRB ,&
&  0.41001E+00_JPRB, 0.42044E+00_JPRB, 0.43324E+00_JPRB, 0.44333E+00_JPRB, 0.44653E+00_JPRB, 0.44100E+00_JPRB, 0.43489E+00_JPRB ,&
&  0.42771E+00_JPRB, 0.41688E+00_JPRB, 0.40625E+00_JPRB, 0.39314E+00_JPRB, 0.37842E+00_JPRB, 0.36532E+00_JPRB, 0.35403E+00_JPRB ,&
&  0.34350E+00_JPRB, 0.33365E+00_JPRB, 0.32488E+00_JPRB, 0.31626E+00_JPRB, 0.30696E+00_JPRB, 0.29718E+00_JPRB, 0.28694E+00_JPRB ,&
&  0.27704E+00_JPRB, 0.26854E+00_JPRB, 0.26175E+00_JPRB, 0.25806E+00_JPRB, 0.25592E+00_JPRB, 0.25421E+00_JPRB, 0.25305E+00_JPRB ,&
&  0.25331E+00_JPRB, 0.25460E+00_JPRB, 0.25642E+00_JPRB, 0.25814E+00_JPRB, 0.25956E+00_JPRB, 0.25975E+00_JPRB, 0.25807E+00_JPRB ,&
&  0.25634E+00_JPRB, 0.25574E+00_JPRB, 0.25559E+00_JPRB, 0.25316E+00_JPRB, 0.25057E+00_JPRB, 0.25095E+00_JPRB, 0.25285E+00_JPRB ,&
&  0.25656E+00_JPRB, 0.25997E+00_JPRB, 0.26292E+00_JPRB, 0.26536E+00_JPRB, 0.26721E+00_JPRB, 0.27050E+00_JPRB, 0.27454E+00_JPRB ,&
&  0.27956E+00_JPRB, 0.28582E+00_JPRB, 0.29463E+00_JPRB, 0.30557E+00_JPRB, 0.31588E+00_JPRB, 0.32481E+00_JPRB, 0.33097E+00_JPRB ,&
&  0.33521E+00_JPRB, 0.33888E+00_JPRB, 0.34291E+00_JPRB, 0.34787E+00_JPRB, 0.35250E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.40314E+00_JPRB, 0.39789E+00_JPRB, 0.39071E+00_JPRB, 0.38440E+00_JPRB, 0.38108E+00_JPRB, 0.37809E+00_JPRB, 0.37279E+00_JPRB ,&
&  0.36780E+00_JPRB, 0.36322E+00_JPRB, 0.35856E+00_JPRB, 0.35452E+00_JPRB, 0.35119E+00_JPRB, 0.34858E+00_JPRB, 0.34453E+00_JPRB ,&
&  0.33891E+00_JPRB, 0.33214E+00_JPRB, 0.32481E+00_JPRB, 0.31894E+00_JPRB, 0.31171E+00_JPRB, 0.30424E+00_JPRB, 0.29872E+00_JPRB ,&
&  0.29412E+00_JPRB, 0.29097E+00_JPRB, 0.28985E+00_JPRB, 0.28821E+00_JPRB, 0.28906E+00_JPRB, 0.29222E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.31073E+00_JPRB, 0.29386E+00_JPRB, 0.30890E+00_JPRB, 0.32181E+00_JPRB, 0.35191E+00_JPRB, 0.40166E+00_JPRB, 0.43617E+00_JPRB ,&
&  0.45509E+00_JPRB, 0.46805E+00_JPRB, 0.48064E+00_JPRB, 0.48479E+00_JPRB, 0.47605E+00_JPRB, 0.46709E+00_JPRB, 0.45938E+00_JPRB ,&
&  0.44846E+00_JPRB, 0.43535E+00_JPRB, 0.41953E+00_JPRB, 0.40302E+00_JPRB, 0.38760E+00_JPRB, 0.37583E+00_JPRB, 0.36573E+00_JPRB ,&
&  0.35743E+00_JPRB, 0.34985E+00_JPRB, 0.34337E+00_JPRB, 0.33561E+00_JPRB, 0.32672E+00_JPRB, 0.31762E+00_JPRB, 0.30939E+00_JPRB ,&
&  0.30137E+00_JPRB, 0.29350E+00_JPRB, 0.28664E+00_JPRB, 0.28096E+00_JPRB, 0.27787E+00_JPRB, 0.27657E+00_JPRB, 0.27646E+00_JPRB ,&
&  0.27721E+00_JPRB, 0.27921E+00_JPRB, 0.28112E+00_JPRB, 0.28245E+00_JPRB, 0.28317E+00_JPRB, 0.28248E+00_JPRB, 0.28023E+00_JPRB ,&
&  0.28003E+00_JPRB, 0.28006E+00_JPRB, 0.28012E+00_JPRB, 0.28077E+00_JPRB, 0.28057E+00_JPRB, 0.28079E+00_JPRB, 0.28276E+00_JPRB ,&
&  0.28478E+00_JPRB, 0.28606E+00_JPRB, 0.28753E+00_JPRB, 0.28936E+00_JPRB, 0.29170E+00_JPRB, 0.29538E+00_JPRB, 0.30055E+00_JPRB ,&
&  0.30641E+00_JPRB, 0.31305E+00_JPRB, 0.32256E+00_JPRB, 0.33522E+00_JPRB, 0.34836E+00_JPRB, 0.36098E+00_JPRB, 0.37286E+00_JPRB ,&
&  0.37960E+00_JPRB, 0.38606E+00_JPRB, 0.39468E+00_JPRB, 0.39979E+00_JPRB, 0.40395E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.46340E+00_JPRB, 0.45340E+00_JPRB, 0.44073E+00_JPRB, 0.43162E+00_JPRB, 0.42667E+00_JPRB, 0.42126E+00_JPRB, 0.41587E+00_JPRB ,&
&  0.40846E+00_JPRB, 0.40030E+00_JPRB, 0.39248E+00_JPRB, 0.38577E+00_JPRB, 0.38039E+00_JPRB, 0.37634E+00_JPRB, 0.37179E+00_JPRB ,&
&  0.36584E+00_JPRB, 0.35937E+00_JPRB, 0.35296E+00_JPRB, 0.34583E+00_JPRB, 0.33646E+00_JPRB, 0.32900E+00_JPRB, 0.32309E+00_JPRB ,&
&  0.31844E+00_JPRB, 0.31475E+00_JPRB, 0.31352E+00_JPRB, 0.31188E+00_JPRB, 0.31009E+00_JPRB, 0.31024E+00_JPRB, 0.31998E+00_JPRB ,&
&  0.31489E+00_JPRB, 0.29310E+00_JPRB, 0.30936E+00_JPRB, 0.33078E+00_JPRB, 0.36678E+00_JPRB, 0.43329E+00_JPRB, 0.47941E+00_JPRB ,&
&  0.50611E+00_JPRB, 0.51849E+00_JPRB, 0.52284E+00_JPRB, 0.51578E+00_JPRB, 0.50414E+00_JPRB, 0.49733E+00_JPRB, 0.48661E+00_JPRB ,&
&  0.47047E+00_JPRB, 0.45214E+00_JPRB, 0.43515E+00_JPRB, 0.42287E+00_JPRB, 0.41048E+00_JPRB, 0.40066E+00_JPRB, 0.39014E+00_JPRB ,&
&  0.37951E+00_JPRB, 0.37070E+00_JPRB, 0.36534E+00_JPRB, 0.36113E+00_JPRB, 0.35585E+00_JPRB, 0.34815E+00_JPRB, 0.33968E+00_JPRB ,&
&  0.33271E+00_JPRB, 0.32727E+00_JPRB, 0.32261E+00_JPRB, 0.31845E+00_JPRB, 0.31624E+00_JPRB, 0.31511E+00_JPRB, 0.31349E+00_JPRB ,&
&  0.31312E+00_JPRB, 0.31325E+00_JPRB, 0.31371E+00_JPRB, 0.31392E+00_JPRB, 0.31342E+00_JPRB, 0.31227E+00_JPRB, 0.31271E+00_JPRB ,&
&  0.31425E+00_JPRB, 0.31324E+00_JPRB, 0.31276E+00_JPRB, 0.31469E+00_JPRB, 0.31688E+00_JPRB, 0.31825E+00_JPRB, 0.31845E+00_JPRB ,&
&  0.31755E+00_JPRB, 0.31756E+00_JPRB, 0.31907E+00_JPRB, 0.32241E+00_JPRB, 0.32657E+00_JPRB, 0.33263E+00_JPRB, 0.34008E+00_JPRB ,&
&  0.34794E+00_JPRB, 0.35778E+00_JPRB, 0.37061E+00_JPRB, 0.39228E+00_JPRB, 0.41154E+00_JPRB, 0.42865E+00_JPRB, 0.44445E+00_JPRB ,&
&  0.45148E+00_JPRB, 0.45569E+00_JPRB, 0.46355E+00_JPRB, 0.46511E+00_JPRB, 0.46611E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.49369E+00_JPRB, 0.49204E+00_JPRB, 0.48347E+00_JPRB, 0.47294E+00_JPRB, 0.46357E+00_JPRB, 0.45417E+00_JPRB, 0.44652E+00_JPRB ,&
&  0.43654E+00_JPRB, 0.42507E+00_JPRB, 0.41666E+00_JPRB, 0.41270E+00_JPRB, 0.41025E+00_JPRB, 0.40652E+00_JPRB, 0.40076E+00_JPRB ,&
&  0.39378E+00_JPRB, 0.38666E+00_JPRB, 0.38002E+00_JPRB, 0.37198E+00_JPRB, 0.36433E+00_JPRB, 0.35761E+00_JPRB, 0.35252E+00_JPRB ,&
&  0.34749E+00_JPRB, 0.34197E+00_JPRB, 0.33650E+00_JPRB, 0.33291E+00_JPRB, 0.32995E+00_JPRB, 0.32857E+00_JPRB, 0.33885E+00_JPRB ,&
&  0.37157E+00_JPRB, 0.32145E+00_JPRB, 0.34067E+00_JPRB, 0.36560E+00_JPRB, 0.40880E+00_JPRB, 0.47517E+00_JPRB, 0.51888E+00_JPRB ,&
&  0.54109E+00_JPRB, 0.55408E+00_JPRB, 0.55214E+00_JPRB, 0.54651E+00_JPRB, 0.54381E+00_JPRB, 0.53360E+00_JPRB, 0.51247E+00_JPRB ,&
&  0.49008E+00_JPRB, 0.46845E+00_JPRB, 0.45224E+00_JPRB, 0.44053E+00_JPRB, 0.43106E+00_JPRB, 0.42469E+00_JPRB, 0.41603E+00_JPRB ,&
&  0.40238E+00_JPRB, 0.39458E+00_JPRB, 0.39025E+00_JPRB, 0.38619E+00_JPRB, 0.38356E+00_JPRB, 0.37937E+00_JPRB, 0.36944E+00_JPRB ,&
&  0.36011E+00_JPRB, 0.35470E+00_JPRB, 0.35282E+00_JPRB, 0.35265E+00_JPRB, 0.35356E+00_JPRB, 0.35327E+00_JPRB, 0.35095E+00_JPRB ,&
&  0.34814E+00_JPRB, 0.34433E+00_JPRB, 0.34225E+00_JPRB, 0.34073E+00_JPRB, 0.33944E+00_JPRB, 0.34096E+00_JPRB, 0.34307E+00_JPRB ,&
&  0.34389E+00_JPRB, 0.34501E+00_JPRB, 0.34738E+00_JPRB, 0.34917E+00_JPRB, 0.34931E+00_JPRB, 0.34834E+00_JPRB, 0.34694E+00_JPRB ,&
&  0.34543E+00_JPRB, 0.34489E+00_JPRB, 0.34657E+00_JPRB, 0.35191E+00_JPRB, 0.35939E+00_JPRB, 0.36826E+00_JPRB, 0.37818E+00_JPRB ,&
&  0.38942E+00_JPRB, 0.40187E+00_JPRB, 0.41460E+00_JPRB, 0.45132E+00_JPRB, 0.47581E+00_JPRB, 0.49724E+00_JPRB, 0.51603E+00_JPRB ,&
&  0.52275E+00_JPRB, 0.52250E+00_JPRB, 0.43832E+00_JPRB, 0.47595E+00_JPRB, 0.48632E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.53850E+00_JPRB, 0.53899E+00_JPRB, 0.52813E+00_JPRB, 0.51167E+00_JPRB, 0.49679E+00_JPRB, 0.48353E+00_JPRB, 0.47287E+00_JPRB ,&
&  0.45978E+00_JPRB, 0.44894E+00_JPRB, 0.44449E+00_JPRB, 0.44320E+00_JPRB, 0.44135E+00_JPRB, 0.43649E+00_JPRB, 0.43031E+00_JPRB ,&
&  0.42404E+00_JPRB, 0.41706E+00_JPRB, 0.40891E+00_JPRB, 0.40073E+00_JPRB, 0.39395E+00_JPRB, 0.38671E+00_JPRB, 0.38147E+00_JPRB ,&
&  0.37676E+00_JPRB, 0.37279E+00_JPRB, 0.36620E+00_JPRB, 0.35782E+00_JPRB, 0.35444E+00_JPRB, 0.34976E+00_JPRB, 0.36208E+00_JPRB ,&
&  0.40545E+00_JPRB, 0.35474E+00_JPRB, 0.38071E+00_JPRB, 0.42443E+00_JPRB, 0.47416E+00_JPRB, 0.53495E+00_JPRB, 0.56787E+00_JPRB ,&
&  0.58766E+00_JPRB, 0.59713E+00_JPRB, 0.59595E+00_JPRB, 0.59837E+00_JPRB, 0.59250E+00_JPRB, 0.56928E+00_JPRB, 0.53938E+00_JPRB ,&
&  0.51603E+00_JPRB, 0.49261E+00_JPRB, 0.47607E+00_JPRB, 0.46043E+00_JPRB, 0.45260E+00_JPRB, 0.44955E+00_JPRB, 0.44269E+00_JPRB ,&
&  0.43263E+00_JPRB, 0.42815E+00_JPRB, 0.42325E+00_JPRB, 0.42185E+00_JPRB, 0.42203E+00_JPRB, 0.42285E+00_JPRB, 0.41710E+00_JPRB ,&
&  0.41146E+00_JPRB, 0.41353E+00_JPRB, 0.41806E+00_JPRB, 0.41885E+00_JPRB, 0.41837E+00_JPRB, 0.41482E+00_JPRB, 0.40993E+00_JPRB ,&
&  0.40568E+00_JPRB, 0.40257E+00_JPRB, 0.40228E+00_JPRB, 0.40126E+00_JPRB, 0.39999E+00_JPRB, 0.39837E+00_JPRB, 0.39687E+00_JPRB ,&
&  0.39611E+00_JPRB, 0.39558E+00_JPRB, 0.39455E+00_JPRB, 0.39267E+00_JPRB, 0.38996E+00_JPRB, 0.38587E+00_JPRB, 0.38428E+00_JPRB ,&
&  0.38440E+00_JPRB, 0.38674E+00_JPRB, 0.39151E+00_JPRB, 0.39910E+00_JPRB, 0.40941E+00_JPRB, 0.41918E+00_JPRB, 0.43032E+00_JPRB ,&
&  0.44146E+00_JPRB, 0.45485E+00_JPRB, 0.47054E+00_JPRB, 0.53074E+00_JPRB, 0.56361E+00_JPRB, 0.57472E+00_JPRB, 0.58359E+00_JPRB ,&
&  0.58676E+00_JPRB, 0.58486E+00_JPRB, 0.58064E+00_JPRB, 0.51042E+00_JPRB, 0.53322E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.58376E+00_JPRB, 0.58414E+00_JPRB, 0.56612E+00_JPRB, 0.54412E+00_JPRB, 0.52348E+00_JPRB, 0.50679E+00_JPRB, 0.49244E+00_JPRB ,&
&  0.48244E+00_JPRB, 0.47548E+00_JPRB, 0.47078E+00_JPRB, 0.46764E+00_JPRB, 0.46337E+00_JPRB, 0.45676E+00_JPRB, 0.45036E+00_JPRB ,&
&  0.44534E+00_JPRB, 0.43952E+00_JPRB, 0.43065E+00_JPRB, 0.42454E+00_JPRB, 0.42105E+00_JPRB, 0.41694E+00_JPRB, 0.41296E+00_JPRB ,&
&  0.40799E+00_JPRB, 0.39760E+00_JPRB, 0.38714E+00_JPRB, 0.38064E+00_JPRB, 0.37581E+00_JPRB, 0.37382E+00_JPRB, 0.38693E+00_JPRB ,&
&  0.43490E+00_JPRB, 0.41873E+00_JPRB, 0.46009E+00_JPRB, 0.50920E+00_JPRB, 0.56958E+00_JPRB, 0.60699E+00_JPRB, 0.62436E+00_JPRB ,&
&  0.63523E+00_JPRB, 0.64599E+00_JPRB, 0.65245E+00_JPRB, 0.66496E+00_JPRB, 0.64046E+00_JPRB, 0.60326E+00_JPRB, 0.57269E+00_JPRB ,&
&  0.54432E+00_JPRB, 0.51763E+00_JPRB, 0.49483E+00_JPRB, 0.47850E+00_JPRB, 0.47260E+00_JPRB, 0.47009E+00_JPRB, 0.46601E+00_JPRB ,&
&  0.46289E+00_JPRB, 0.45671E+00_JPRB, 0.44952E+00_JPRB, 0.44980E+00_JPRB, 0.44803E+00_JPRB, 0.45889E+00_JPRB, 0.47052E+00_JPRB ,&
&  0.47837E+00_JPRB, 0.48820E+00_JPRB, 0.49188E+00_JPRB, 0.49381E+00_JPRB, 0.48918E+00_JPRB, 0.48263E+00_JPRB, 0.47619E+00_JPRB ,&
&  0.47056E+00_JPRB, 0.46727E+00_JPRB, 0.46668E+00_JPRB, 0.46532E+00_JPRB, 0.46239E+00_JPRB, 0.46085E+00_JPRB, 0.45971E+00_JPRB ,&
&  0.45926E+00_JPRB, 0.45807E+00_JPRB, 0.45668E+00_JPRB, 0.45060E+00_JPRB, 0.44210E+00_JPRB, 0.43564E+00_JPRB, 0.43068E+00_JPRB ,&
&  0.42781E+00_JPRB, 0.42920E+00_JPRB, 0.43364E+00_JPRB, 0.44339E+00_JPRB, 0.45342E+00_JPRB, 0.46486E+00_JPRB, 0.47351E+00_JPRB ,&
&  0.48046E+00_JPRB, 0.49586E+00_JPRB, 0.51761E+00_JPRB, 0.55737E+00_JPRB, 0.58173E+00_JPRB, 0.59831E+00_JPRB, 0.61259E+00_JPRB ,&
&  0.61948E+00_JPRB, 0.62026E+00_JPRB, 0.60508E+00_JPRB, 0.59236E+00_JPRB, 0.57308E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.66987E+00_JPRB, 0.64244E+00_JPRB, 0.60731E+00_JPRB, 0.57370E+00_JPRB, 0.54245E+00_JPRB, 0.51763E+00_JPRB, 0.50201E+00_JPRB ,&
&  0.49190E+00_JPRB, 0.48528E+00_JPRB, 0.48112E+00_JPRB, 0.47788E+00_JPRB, 0.47324E+00_JPRB, 0.46928E+00_JPRB, 0.46370E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.44927E+00_JPRB, 0.44213E+00_JPRB, 0.43748E+00_JPRB, 0.43332E+00_JPRB, 0.43123E+00_JPRB, 0.42872E+00_JPRB ,&
&  0.42281E+00_JPRB, 0.41531E+00_JPRB, 0.40554E+00_JPRB, 0.39556E+00_JPRB, 0.38991E+00_JPRB, 0.38985E+00_JPRB, 0.41144E+00_JPRB ,&
&  0.48019E+00_JPRB, 0.49331E+00_JPRB, 0.54232E+00_JPRB, 0.59815E+00_JPRB, 0.67347E+00_JPRB, 0.71528E+00_JPRB, 0.72184E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.73135E+00_JPRB, 0.75178E+00_JPRB, 0.74330E+00_JPRB, 0.69474E+00_JPRB, 0.64986E+00_JPRB, 0.60630E+00_JPRB ,&
&  0.56559E+00_JPRB, 0.52976E+00_JPRB, 0.49987E+00_JPRB, 0.48716E+00_JPRB, 0.48412E+00_JPRB, 0.48609E+00_JPRB, 0.48676E+00_JPRB ,&
&  0.48728E+00_JPRB, 0.48382E+00_JPRB, 0.48519E+00_JPRB, 0.49907E+00_JPRB, 0.51891E+00_JPRB, 0.54627E+00_JPRB, 0.58987E+00_JPRB ,&
&  0.60961E+00_JPRB, 0.60085E+00_JPRB, 0.60383E+00_JPRB, 0.60222E+00_JPRB, 0.58600E+00_JPRB, 0.57076E+00_JPRB, 0.55842E+00_JPRB ,&
&  0.54908E+00_JPRB, 0.54240E+00_JPRB, 0.53872E+00_JPRB, 0.53221E+00_JPRB, 0.52665E+00_JPRB, 0.52368E+00_JPRB, 0.52324E+00_JPRB ,&
&  0.52269E+00_JPRB, 0.52314E+00_JPRB, 0.52137E+00_JPRB, 0.51489E+00_JPRB, 0.51069E+00_JPRB, 0.50657E+00_JPRB, 0.49804E+00_JPRB ,&
&  0.48770E+00_JPRB, 0.48343E+00_JPRB, 0.48756E+00_JPRB, 0.49536E+00_JPRB, 0.50543E+00_JPRB, 0.51513E+00_JPRB, 0.52218E+00_JPRB ,&
&  0.52948E+00_JPRB, 0.55143E+00_JPRB, 0.55760E+00_JPRB, 0.59912E+00_JPRB, 0.64559E+00_JPRB, 0.65416E+00_JPRB, 0.66460E+00_JPRB ,&
&  0.67000E+00_JPRB, 0.66268E+00_JPRB, 0.64152E+00_JPRB, 0.62115E+00_JPRB, 0.67106E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.64159E+00_JPRB, 0.61398E+00_JPRB, 0.58400E+00_JPRB, 0.55706E+00_JPRB, 0.53259E+00_JPRB, 0.51369E+00_JPRB, 0.50184E+00_JPRB ,&
&  0.49226E+00_JPRB, 0.48591E+00_JPRB, 0.48599E+00_JPRB, 0.48912E+00_JPRB, 0.48776E+00_JPRB, 0.47980E+00_JPRB, 0.46886E+00_JPRB ,&
&  0.45787E+00_JPRB, 0.45014E+00_JPRB, 0.44487E+00_JPRB, 0.44336E+00_JPRB, 0.43969E+00_JPRB, 0.43595E+00_JPRB, 0.43019E+00_JPRB ,&
&  0.42629E+00_JPRB, 0.42246E+00_JPRB, 0.41620E+00_JPRB, 0.40976E+00_JPRB, 0.40561E+00_JPRB, 0.40692E+00_JPRB, 0.43998E+00_JPRB ,&
&  0.52841E+00_JPRB, 0.60504E+00_JPRB, 0.65615E+00_JPRB, 0.72490E+00_JPRB, 0.85006E+00_JPRB, 0.85263E+00_JPRB, 0.87491E+00_JPRB ,&
&  0.88674E+00_JPRB, 0.88989E+00_JPRB, 0.89329E+00_JPRB, 0.84994E+00_JPRB, 0.76742E+00_JPRB, 0.70150E+00_JPRB, 0.63552E+00_JPRB ,&
&  0.57624E+00_JPRB, 0.53089E+00_JPRB, 0.50254E+00_JPRB, 0.49112E+00_JPRB, 0.49296E+00_JPRB, 0.49979E+00_JPRB, 0.50438E+00_JPRB ,&
&  0.50926E+00_JPRB, 0.51421E+00_JPRB, 0.53984E+00_JPRB, 0.59033E+00_JPRB, 0.67568E+00_JPRB, 0.72634E+00_JPRB, 0.80449E+00_JPRB ,&
&  0.84141E+00_JPRB, 0.83651E+00_JPRB, 0.81793E+00_JPRB, 0.78221E+00_JPRB, 0.75378E+00_JPRB, 0.71963E+00_JPRB, 0.69244E+00_JPRB ,&
&  0.66038E+00_JPRB, 0.63204E+00_JPRB, 0.61085E+00_JPRB, 0.59636E+00_JPRB, 0.58873E+00_JPRB, 0.58605E+00_JPRB, 0.58732E+00_JPRB ,&
&  0.58483E+00_JPRB, 0.58498E+00_JPRB, 0.58370E+00_JPRB, 0.58672E+00_JPRB, 0.59102E+00_JPRB, 0.58350E+00_JPRB, 0.56884E+00_JPRB ,&
&  0.55301E+00_JPRB, 0.54021E+00_JPRB, 0.54201E+00_JPRB, 0.54851E+00_JPRB, 0.55387E+00_JPRB, 0.55584E+00_JPRB, 0.55755E+00_JPRB ,&
&  0.57032E+00_JPRB, 0.60298E+00_JPRB, 0.61544E+00_JPRB, 0.62655E+00_JPRB, 0.67459E+00_JPRB, 0.71950E+00_JPRB, 0.72126E+00_JPRB ,&
&  0.71311E+00_JPRB, 0.69340E+00_JPRB, 0.66415E+00_JPRB, 0.63949E+00_JPRB, 0.65374E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.63198E+00_JPRB, 0.61220E+00_JPRB, 0.58475E+00_JPRB, 0.56088E+00_JPRB, 0.54175E+00_JPRB, 0.52254E+00_JPRB, 0.50914E+00_JPRB ,&
&  0.49911E+00_JPRB, 0.49906E+00_JPRB, 0.50190E+00_JPRB, 0.50324E+00_JPRB, 0.49854E+00_JPRB, 0.48674E+00_JPRB, 0.47471E+00_JPRB ,&
&  0.46699E+00_JPRB, 0.46615E+00_JPRB, 0.46655E+00_JPRB, 0.46358E+00_JPRB, 0.45814E+00_JPRB, 0.45132E+00_JPRB, 0.44544E+00_JPRB ,&
&  0.44018E+00_JPRB, 0.43556E+00_JPRB, 0.43077E+00_JPRB, 0.42795E+00_JPRB, 0.42464E+00_JPRB, 0.42916E+00_JPRB, 0.46901E+00_JPRB ,&
&  0.58032E+00_JPRB, 0.67960E+00_JPRB, 0.89297E+00_JPRB, 0.90454E+00_JPRB, 0.94540E+00_JPRB, 0.97244E+00_JPRB, 0.10863E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11477E+01_JPRB, 0.10895E+01_JPRB, 0.97059E+00_JPRB, 0.82821E+00_JPRB, 0.72275E+00_JPRB, 0.64446E+00_JPRB ,&
&  0.58076E+00_JPRB, 0.54040E+00_JPRB, 0.51785E+00_JPRB, 0.51146E+00_JPRB, 0.51512E+00_JPRB, 0.52474E+00_JPRB, 0.53181E+00_JPRB ,&
&  0.54508E+00_JPRB, 0.56603E+00_JPRB, 0.62524E+00_JPRB, 0.72470E+00_JPRB, 0.80561E+00_JPRB, 0.80725E+00_JPRB, 0.94744E+00_JPRB ,&
&  0.11540E+01_JPRB, 0.11010E+01_JPRB, 0.10485E+01_JPRB, 0.99433E+00_JPRB, 0.94558E+00_JPRB, 0.89869E+00_JPRB, 0.82662E+00_JPRB ,&
&  0.76905E+00_JPRB, 0.72016E+00_JPRB, 0.68562E+00_JPRB, 0.66690E+00_JPRB, 0.65660E+00_JPRB, 0.65412E+00_JPRB, 0.65321E+00_JPRB ,&
&  0.65194E+00_JPRB, 0.65095E+00_JPRB, 0.65466E+00_JPRB, 0.65371E+00_JPRB, 0.64602E+00_JPRB, 0.63077E+00_JPRB, 0.61167E+00_JPRB ,&
&  0.59776E+00_JPRB, 0.58755E+00_JPRB, 0.59301E+00_JPRB, 0.58187E+00_JPRB, 0.58710E+00_JPRB, 0.59941E+00_JPRB, 0.60689E+00_JPRB ,&
&  0.61090E+00_JPRB, 0.63038E+00_JPRB, 0.65079E+00_JPRB, 0.66337E+00_JPRB, 0.73701E+00_JPRB, 0.79674E+00_JPRB, 0.78465E+00_JPRB ,&
&  0.76313E+00_JPRB, 0.71654E+00_JPRB, 0.67542E+00_JPRB, 0.64931E+00_JPRB, 0.64284E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.62793E+00_JPRB, 0.60225E+00_JPRB, 0.58159E+00_JPRB, 0.56212E+00_JPRB, 0.54827E+00_JPRB, 0.53465E+00_JPRB, 0.52562E+00_JPRB ,&
&  0.52257E+00_JPRB, 0.52347E+00_JPRB, 0.52244E+00_JPRB, 0.51591E+00_JPRB, 0.50445E+00_JPRB, 0.49526E+00_JPRB, 0.48899E+00_JPRB ,&
&  0.48302E+00_JPRB, 0.48224E+00_JPRB, 0.48099E+00_JPRB, 0.47318E+00_JPRB, 0.46470E+00_JPRB, 0.45889E+00_JPRB, 0.45318E+00_JPRB ,&
&  0.44868E+00_JPRB, 0.44637E+00_JPRB, 0.44454E+00_JPRB, 0.44314E+00_JPRB, 0.44292E+00_JPRB, 0.45319E+00_JPRB, 0.49199E+00_JPRB ,&
&  0.57432E+00_JPRB, 0.50556E+00_JPRB, 0.92859E+00_JPRB, 0.10987E+01_JPRB, 0.11604E+01_JPRB, 0.12215E+01_JPRB, 0.13679E+01_JPRB ,&
&  0.15357E+01_JPRB, 0.14908E+01_JPRB, 0.13042E+01_JPRB, 0.10726E+01_JPRB, 0.86688E+00_JPRB, 0.72817E+00_JPRB, 0.65585E+00_JPRB ,&
&  0.60582E+00_JPRB, 0.57280E+00_JPRB, 0.55810E+00_JPRB, 0.55023E+00_JPRB, 0.55331E+00_JPRB, 0.56227E+00_JPRB, 0.57231E+00_JPRB ,&
&  0.59317E+00_JPRB, 0.62941E+00_JPRB, 0.70078E+00_JPRB, 0.79258E+00_JPRB, 0.82700E+00_JPRB, 0.89040E+00_JPRB, 0.10084E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.13483E+01_JPRB, 0.12250E+01_JPRB, 0.11337E+01_JPRB, 0.10651E+01_JPRB, 0.10014E+01_JPRB, 0.92498E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.79548E+00_JPRB, 0.75477E+00_JPRB, 0.72499E+00_JPRB, 0.70815E+00_JPRB, 0.70138E+00_JPRB, 0.69643E+00_JPRB ,&
&  0.69060E+00_JPRB, 0.68455E+00_JPRB, 0.68144E+00_JPRB, 0.67622E+00_JPRB, 0.66844E+00_JPRB, 0.65152E+00_JPRB, 0.62993E+00_JPRB ,&
&  0.61740E+00_JPRB, 0.61388E+00_JPRB, 0.61079E+00_JPRB, 0.61243E+00_JPRB, 0.62293E+00_JPRB, 0.62971E+00_JPRB, 0.64176E+00_JPRB ,&
&  0.67177E+00_JPRB, 0.68596E+00_JPRB, 0.69717E+00_JPRB, 0.71975E+00_JPRB, 0.74157E+00_JPRB, 0.81241E+00_JPRB, 0.76725E+00_JPRB ,&
&  0.73713E+00_JPRB, 0.70196E+00_JPRB, 0.66927E+00_JPRB, 0.65266E+00_JPRB, 0.64405E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.62237E+00_JPRB, 0.59894E+00_JPRB, 0.58228E+00_JPRB, 0.56971E+00_JPRB, 0.55976E+00_JPRB, 0.55155E+00_JPRB, 0.55022E+00_JPRB ,&
&  0.54919E+00_JPRB, 0.54510E+00_JPRB, 0.53188E+00_JPRB, 0.51613E+00_JPRB, 0.51031E+00_JPRB, 0.50940E+00_JPRB, 0.50056E+00_JPRB ,&
&  0.49813E+00_JPRB, 0.49828E+00_JPRB, 0.49349E+00_JPRB, 0.48473E+00_JPRB, 0.47771E+00_JPRB, 0.47399E+00_JPRB, 0.47333E+00_JPRB ,&
&  0.46664E+00_JPRB, 0.46444E+00_JPRB, 0.46703E+00_JPRB, 0.46912E+00_JPRB, 0.47305E+00_JPRB, 0.49335E+00_JPRB, 0.51538E+00_JPRB ,&
&  0.56651E+00_JPRB, 0.73422E+00_JPRB, 0.78143E+00_JPRB, 0.12116E+01_JPRB, 0.13575E+01_JPRB, 0.15087E+01_JPRB, 0.15534E+01_JPRB ,&
&  0.16214E+01_JPRB, 0.17169E+01_JPRB, 0.14351E+01_JPRB, 0.11427E+01_JPRB, 0.89062E+00_JPRB, 0.75504E+00_JPRB, 0.69069E+00_JPRB ,&
&  0.65828E+00_JPRB, 0.62758E+00_JPRB, 0.60600E+00_JPRB, 0.60036E+00_JPRB, 0.60451E+00_JPRB, 0.61013E+00_JPRB, 0.61993E+00_JPRB ,&
&  0.64879E+00_JPRB, 0.70044E+00_JPRB, 0.78673E+00_JPRB, 0.86045E+00_JPRB, 0.90781E+00_JPRB, 0.10702E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.13928E+01_JPRB, 0.15552E+01_JPRB, 0.14316E+01_JPRB, 0.12972E+01_JPRB, 0.11585E+01_JPRB, 0.10399E+01_JPRB, 0.93951E+00_JPRB ,&
&  0.87064E+00_JPRB, 0.81777E+00_JPRB, 0.77976E+00_JPRB, 0.75211E+00_JPRB, 0.73320E+00_JPRB, 0.72056E+00_JPRB, 0.70372E+00_JPRB ,&
&  0.68439E+00_JPRB, 0.67536E+00_JPRB, 0.67460E+00_JPRB, 0.67353E+00_JPRB, 0.66808E+00_JPRB, 0.65809E+00_JPRB, 0.64619E+00_JPRB ,&
&  0.63996E+00_JPRB, 0.64189E+00_JPRB, 0.63913E+00_JPRB, 0.65762E+00_JPRB, 0.67085E+00_JPRB, 0.66236E+00_JPRB, 0.66639E+00_JPRB ,&
&  0.70796E+00_JPRB, 0.73232E+00_JPRB, 0.73849E+00_JPRB, 0.74450E+00_JPRB, 0.78917E+00_JPRB, 0.87491E+00_JPRB, 0.74502E+00_JPRB ,&
&  0.69613E+00_JPRB, 0.67522E+00_JPRB, 0.65666E+00_JPRB, 0.64616E+00_JPRB, 0.63656E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.62122E+00_JPRB, 0.60589E+00_JPRB, 0.59457E+00_JPRB, 0.58250E+00_JPRB, 0.57259E+00_JPRB, 0.56939E+00_JPRB, 0.56653E+00_JPRB ,&
&  0.55967E+00_JPRB, 0.53978E+00_JPRB, 0.51858E+00_JPRB, 0.51063E+00_JPRB, 0.50941E+00_JPRB, 0.50194E+00_JPRB, 0.50490E+00_JPRB ,&
&  0.50765E+00_JPRB, 0.51104E+00_JPRB, 0.50344E+00_JPRB, 0.49838E+00_JPRB, 0.49543E+00_JPRB, 0.49415E+00_JPRB, 0.49519E+00_JPRB ,&
&  0.49564E+00_JPRB, 0.49647E+00_JPRB, 0.50173E+00_JPRB, 0.50799E+00_JPRB, 0.51857E+00_JPRB, 0.54130E+00_JPRB, 0.56460E+00_JPRB ,&
&  0.66030E+00_JPRB, 0.73781E+00_JPRB, 0.81013E+00_JPRB, 0.14270E+01_JPRB, 0.16743E+01_JPRB, 0.17323E+01_JPRB, 0.15828E+01_JPRB ,&
&  0.15600E+01_JPRB, 0.16965E+01_JPRB, 0.15048E+01_JPRB, 0.12011E+01_JPRB, 0.97112E+00_JPRB, 0.86571E+00_JPRB, 0.79882E+00_JPRB ,&
&  0.74274E+00_JPRB, 0.69069E+00_JPRB, 0.65885E+00_JPRB, 0.64064E+00_JPRB, 0.64567E+00_JPRB, 0.65906E+00_JPRB, 0.68754E+00_JPRB ,&
&  0.73093E+00_JPRB, 0.80509E+00_JPRB, 0.88655E+00_JPRB, 0.94839E+00_JPRB, 0.12120E+01_JPRB, 0.14902E+01_JPRB, 0.15715E+01_JPRB ,&
&  0.15456E+01_JPRB, 0.16439E+01_JPRB, 0.14529E+01_JPRB, 0.13025E+01_JPRB, 0.11192E+01_JPRB, 0.10260E+01_JPRB, 0.92958E+00_JPRB ,&
&  0.86299E+00_JPRB, 0.80929E+00_JPRB, 0.77060E+00_JPRB, 0.74984E+00_JPRB, 0.72989E+00_JPRB, 0.70837E+00_JPRB, 0.68610E+00_JPRB ,&
&  0.66639E+00_JPRB, 0.66708E+00_JPRB, 0.67041E+00_JPRB, 0.66537E+00_JPRB, 0.66278E+00_JPRB, 0.66241E+00_JPRB, 0.66443E+00_JPRB ,&
&  0.66626E+00_JPRB, 0.66864E+00_JPRB, 0.66740E+00_JPRB, 0.69619E+00_JPRB, 0.71760E+00_JPRB, 0.71254E+00_JPRB, 0.71970E+00_JPRB ,&
&  0.75027E+00_JPRB, 0.77239E+00_JPRB, 0.76339E+00_JPRB, 0.73960E+00_JPRB, 0.76018E+00_JPRB, 0.77789E+00_JPRB, 0.70761E+00_JPRB ,&
&  0.69344E+00_JPRB, 0.67777E+00_JPRB, 0.65341E+00_JPRB, 0.63800E+00_JPRB, 0.63064E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.62894E+00_JPRB, 0.61292E+00_JPRB, 0.59600E+00_JPRB, 0.58330E+00_JPRB, 0.57556E+00_JPRB, 0.56793E+00_JPRB, 0.55411E+00_JPRB ,&
&  0.53524E+00_JPRB, 0.52431E+00_JPRB, 0.50429E+00_JPRB, 0.50081E+00_JPRB, 0.50794E+00_JPRB, 0.51313E+00_JPRB, 0.52246E+00_JPRB ,&
&  0.52800E+00_JPRB, 0.52944E+00_JPRB, 0.52604E+00_JPRB, 0.52189E+00_JPRB, 0.52143E+00_JPRB, 0.52329E+00_JPRB, 0.53115E+00_JPRB ,&
&  0.53118E+00_JPRB, 0.53367E+00_JPRB, 0.54138E+00_JPRB, 0.55263E+00_JPRB, 0.58073E+00_JPRB, 0.62953E+00_JPRB, 0.69479E+00_JPRB ,&
&  0.85148E+00_JPRB, 0.89998E+00_JPRB, 0.95594E+00_JPRB, 0.14224E+01_JPRB, 0.16923E+01_JPRB, 0.15941E+01_JPRB, 0.15208E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.14256E+01_JPRB, 0.14207E+01_JPRB, 0.12120E+01_JPRB, 0.10354E+01_JPRB, 0.93912E+00_JPRB, 0.86404E+00_JPRB ,&
&  0.79156E+00_JPRB, 0.73858E+00_JPRB, 0.70074E+00_JPRB, 0.68822E+00_JPRB, 0.70819E+00_JPRB, 0.75280E+00_JPRB, 0.79418E+00_JPRB ,&
&  0.85915E+00_JPRB, 0.95434E+00_JPRB, 0.10691E+01_JPRB, 0.12702E+01_JPRB, 0.16766E+01_JPRB, 0.19286E+01_JPRB, 0.17759E+01_JPRB ,&
&  0.16238E+01_JPRB, 0.15542E+01_JPRB, 0.14016E+01_JPRB, 0.12656E+01_JPRB, 0.10279E+01_JPRB, 0.10027E+01_JPRB, 0.91683E+00_JPRB ,&
&  0.84025E+00_JPRB, 0.78708E+00_JPRB, 0.75961E+00_JPRB, 0.74480E+00_JPRB, 0.72426E+00_JPRB, 0.69579E+00_JPRB, 0.67458E+00_JPRB ,&
&  0.66361E+00_JPRB, 0.66620E+00_JPRB, 0.66905E+00_JPRB, 0.66512E+00_JPRB, 0.66605E+00_JPRB, 0.67185E+00_JPRB, 0.67834E+00_JPRB ,&
&  0.68451E+00_JPRB, 0.69706E+00_JPRB, 0.70903E+00_JPRB, 0.75097E+00_JPRB, 0.77915E+00_JPRB, 0.78833E+00_JPRB, 0.77480E+00_JPRB ,&
&  0.77180E+00_JPRB, 0.78278E+00_JPRB, 0.76990E+00_JPRB, 0.76579E+00_JPRB, 0.75729E+00_JPRB, 0.74779E+00_JPRB, 0.72771E+00_JPRB ,&
&  0.72635E+00_JPRB, 0.70130E+00_JPRB, 0.66538E+00_JPRB, 0.65077E+00_JPRB, 0.64002E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.63045E+00_JPRB, 0.61206E+00_JPRB, 0.58787E+00_JPRB, 0.56324E+00_JPRB, 0.55072E+00_JPRB, 0.53859E+00_JPRB, 0.52334E+00_JPRB ,&
&  0.51563E+00_JPRB, 0.51174E+00_JPRB, 0.50657E+00_JPRB, 0.51857E+00_JPRB, 0.54174E+00_JPRB, 0.55353E+00_JPRB, 0.56011E+00_JPRB ,&
&  0.56745E+00_JPRB, 0.57111E+00_JPRB, 0.55876E+00_JPRB, 0.55639E+00_JPRB, 0.56612E+00_JPRB, 0.57689E+00_JPRB, 0.58157E+00_JPRB ,&
&  0.58647E+00_JPRB, 0.59682E+00_JPRB, 0.61205E+00_JPRB, 0.64128E+00_JPRB, 0.70517E+00_JPRB, 0.80204E+00_JPRB, 0.93126E+00_JPRB ,&
&  0.11094E+01_JPRB, 0.10866E+01_JPRB, 0.64944E+00_JPRB, 0.10746E+01_JPRB, 0.14409E+01_JPRB, 0.14252E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.12988E+01_JPRB, 0.12421E+01_JPRB, 0.12436E+01_JPRB, 0.11594E+01_JPRB, 0.10505E+01_JPRB, 0.95658E+00_JPRB, 0.87345E+00_JPRB ,&
&  0.80263E+00_JPRB, 0.75896E+00_JPRB, 0.74942E+00_JPRB, 0.78011E+00_JPRB, 0.83247E+00_JPRB, 0.86825E+00_JPRB, 0.92363E+00_JPRB ,&
&  0.10216E+01_JPRB, 0.11516E+01_JPRB, 0.13884E+01_JPRB, 0.16664E+01_JPRB, 0.20865E+01_JPRB, 0.20383E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.13251E+01_JPRB, 0.13244E+01_JPRB, 0.12956E+01_JPRB, 0.11587E+01_JPRB, 0.95847E+00_JPRB, 0.88364E+00_JPRB, 0.87516E+00_JPRB ,&
&  0.80908E+00_JPRB, 0.76243E+00_JPRB, 0.74260E+00_JPRB, 0.72216E+00_JPRB, 0.69507E+00_JPRB, 0.67141E+00_JPRB, 0.65860E+00_JPRB ,&
&  0.65965E+00_JPRB, 0.66104E+00_JPRB, 0.66121E+00_JPRB, 0.66078E+00_JPRB, 0.66524E+00_JPRB, 0.67230E+00_JPRB, 0.68426E+00_JPRB ,&
&  0.70587E+00_JPRB, 0.73416E+00_JPRB, 0.77961E+00_JPRB, 0.80533E+00_JPRB, 0.81752E+00_JPRB, 0.82389E+00_JPRB, 0.82554E+00_JPRB ,&
&  0.82815E+00_JPRB, 0.82289E+00_JPRB, 0.77268E+00_JPRB, 0.75343E+00_JPRB, 0.79213E+00_JPRB, 0.79792E+00_JPRB, 0.82743E+00_JPRB ,&
&  0.85957E+00_JPRB, 0.86361E+00_JPRB, 0.80493E+00_JPRB, 0.67521E+00_JPRB, 0.65455E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.58478E+00_JPRB, 0.56256E+00_JPRB, 0.53076E+00_JPRB, 0.52389E+00_JPRB, 0.52356E+00_JPRB, 0.52270E+00_JPRB, 0.53673E+00_JPRB ,&
&  0.54905E+00_JPRB, 0.56798E+00_JPRB, 0.57994E+00_JPRB, 0.60138E+00_JPRB, 0.61338E+00_JPRB, 0.61438E+00_JPRB, 0.62235E+00_JPRB ,&
&  0.62534E+00_JPRB, 0.62734E+00_JPRB, 0.63109E+00_JPRB, 0.64127E+00_JPRB, 0.65558E+00_JPRB, 0.66179E+00_JPRB, 0.66917E+00_JPRB ,&
&  0.68690E+00_JPRB, 0.70846E+00_JPRB, 0.73983E+00_JPRB, 0.81157E+00_JPRB, 0.93387E+00_JPRB, 0.10897E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.13783E+01_JPRB, 0.10297E+01_JPRB, 0.76160E+00_JPRB, 0.12682E+01_JPRB, 0.13994E+01_JPRB, 0.13548E+01_JPRB, 0.11932E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.11656E+01_JPRB, 0.11464E+01_JPRB, 0.11130E+01_JPRB, 0.98726E+00_JPRB, 0.91563E+00_JPRB, 0.86085E+00_JPRB ,&
&  0.83622E+00_JPRB, 0.83270E+00_JPRB, 0.85910E+00_JPRB, 0.90699E+00_JPRB, 0.95394E+00_JPRB, 0.10055E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.14383E+01_JPRB, 0.17627E+01_JPRB, 0.18990E+01_JPRB, 0.21089E+01_JPRB, 0.17996E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.11574E+01_JPRB, 0.11710E+01_JPRB, 0.10638E+01_JPRB, 0.90836E+00_JPRB, 0.82337E+00_JPRB, 0.82385E+00_JPRB ,&
&  0.75754E+00_JPRB, 0.72510E+00_JPRB, 0.69969E+00_JPRB, 0.66910E+00_JPRB, 0.65550E+00_JPRB, 0.65485E+00_JPRB, 0.65752E+00_JPRB ,&
&  0.65968E+00_JPRB, 0.65574E+00_JPRB, 0.65386E+00_JPRB, 0.66059E+00_JPRB, 0.67344E+00_JPRB, 0.68339E+00_JPRB, 0.70194E+00_JPRB ,&
&  0.72716E+00_JPRB, 0.75992E+00_JPRB, 0.79949E+00_JPRB, 0.82842E+00_JPRB, 0.82708E+00_JPRB, 0.80500E+00_JPRB, 0.81812E+00_JPRB ,&
&  0.81859E+00_JPRB, 0.83362E+00_JPRB, 0.82694E+00_JPRB, 0.84008E+00_JPRB, 0.84056E+00_JPRB, 0.85644E+00_JPRB, 0.86037E+00_JPRB ,&
&  0.85776E+00_JPRB, 0.83015E+00_JPRB, 0.74478E+00_JPRB, 0.63712E+00_JPRB, 0.61224E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.52304E+00_JPRB, 0.51719E+00_JPRB, 0.52124E+00_JPRB, 0.52873E+00_JPRB, 0.54083E+00_JPRB, 0.56557E+00_JPRB, 0.59388E+00_JPRB ,&
&  0.61783E+00_JPRB, 0.65837E+00_JPRB, 0.68651E+00_JPRB, 0.68865E+00_JPRB, 0.68355E+00_JPRB, 0.68692E+00_JPRB, 0.69546E+00_JPRB ,&
&  0.70986E+00_JPRB, 0.72116E+00_JPRB, 0.72949E+00_JPRB, 0.73980E+00_JPRB, 0.75437E+00_JPRB, 0.76852E+00_JPRB, 0.78717E+00_JPRB ,&
&  0.81744E+00_JPRB, 0.86824E+00_JPRB, 0.94351E+00_JPRB, 0.10558E+01_JPRB, 0.12081E+01_JPRB, 0.13754E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.10944E+01_JPRB, 0.61345E+00_JPRB, 0.11894E+01_JPRB, 0.14025E+01_JPRB, 0.12764E+01_JPRB, 0.12243E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.11080E+01_JPRB, 0.10724E+01_JPRB, 0.10732E+01_JPRB, 0.96659E+00_JPRB, 0.92509E+00_JPRB, 0.91495E+00_JPRB ,&
&  0.92774E+00_JPRB, 0.94564E+00_JPRB, 0.97547E+00_JPRB, 0.10144E+01_JPRB, 0.10752E+01_JPRB, 0.11665E+01_JPRB, 0.13103E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.16790E+01_JPRB, 0.19693E+01_JPRB, 0.18638E+01_JPRB, 0.18965E+01_JPRB, 0.17362E+01_JPRB, 0.14828E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.10114E+01_JPRB, 0.99789E+00_JPRB, 0.98995E+00_JPRB, 0.87835E+00_JPRB, 0.76723E+00_JPRB, 0.76989E+00_JPRB ,&
&  0.71230E+00_JPRB, 0.68956E+00_JPRB, 0.65730E+00_JPRB, 0.63010E+00_JPRB, 0.62144E+00_JPRB, 0.62907E+00_JPRB, 0.64074E+00_JPRB ,&
&  0.64454E+00_JPRB, 0.64567E+00_JPRB, 0.65209E+00_JPRB, 0.66063E+00_JPRB, 0.67901E+00_JPRB, 0.69572E+00_JPRB, 0.70908E+00_JPRB ,&
&  0.74183E+00_JPRB, 0.77556E+00_JPRB, 0.80832E+00_JPRB, 0.83747E+00_JPRB, 0.85776E+00_JPRB, 0.83928E+00_JPRB, 0.83068E+00_JPRB ,&
&  0.83900E+00_JPRB, 0.84662E+00_JPRB, 0.85588E+00_JPRB, 0.87587E+00_JPRB, 0.87486E+00_JPRB, 0.83968E+00_JPRB, 0.82718E+00_JPRB ,&
&  0.76683E+00_JPRB, 0.69542E+00_JPRB, 0.61240E+00_JPRB, 0.56289E+00_JPRB, 0.54490E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.54616E+00_JPRB, 0.56070E+00_JPRB, 0.57767E+00_JPRB, 0.59862E+00_JPRB, 0.62899E+00_JPRB, 0.66443E+00_JPRB, 0.70953E+00_JPRB ,&
&  0.75575E+00_JPRB, 0.76922E+00_JPRB, 0.77250E+00_JPRB, 0.77421E+00_JPRB, 0.77611E+00_JPRB, 0.78406E+00_JPRB, 0.79616E+00_JPRB ,&
&  0.80857E+00_JPRB, 0.82173E+00_JPRB, 0.83536E+00_JPRB, 0.85120E+00_JPRB, 0.87067E+00_JPRB, 0.89866E+00_JPRB, 0.93667E+00_JPRB ,&
&  0.97986E+00_JPRB, 0.10557E+01_JPRB, 0.11579E+01_JPRB, 0.12739E+01_JPRB, 0.13963E+01_JPRB, 0.14934E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.91911E+00_JPRB, 0.11604E+01_JPRB, 0.13710E+01_JPRB, 0.13008E+01_JPRB, 0.12067E+01_JPRB, 0.11523E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.10282E+01_JPRB, 0.10141E+01_JPRB, 0.10728E+01_JPRB, 0.99565E+00_JPRB, 0.97048E+00_JPRB, 0.98329E+00_JPRB ,&
&  0.10085E+01_JPRB, 0.10404E+01_JPRB, 0.10869E+01_JPRB, 0.11413E+01_JPRB, 0.12196E+01_JPRB, 0.13358E+01_JPRB, 0.14733E+01_JPRB ,&
&  0.16352E+01_JPRB, 0.18060E+01_JPRB, 0.19771E+01_JPRB, 0.17616E+01_JPRB, 0.17224E+01_JPRB, 0.17591E+01_JPRB, 0.16248E+01_JPRB ,&
&  0.11509E+01_JPRB, 0.91932E+00_JPRB, 0.93489E+00_JPRB, 0.93123E+00_JPRB, 0.83582E+00_JPRB, 0.77306E+00_JPRB, 0.71933E+00_JPRB ,&
&  0.68062E+00_JPRB, 0.64523E+00_JPRB, 0.60971E+00_JPRB, 0.59829E+00_JPRB, 0.59611E+00_JPRB, 0.60632E+00_JPRB, 0.62149E+00_JPRB ,&
&  0.63101E+00_JPRB, 0.63416E+00_JPRB, 0.64412E+00_JPRB, 0.65797E+00_JPRB, 0.68952E+00_JPRB, 0.70932E+00_JPRB, 0.72088E+00_JPRB ,&
&  0.74974E+00_JPRB, 0.76883E+00_JPRB, 0.79512E+00_JPRB, 0.82282E+00_JPRB, 0.84704E+00_JPRB, 0.86225E+00_JPRB, 0.85855E+00_JPRB ,&
&  0.85674E+00_JPRB, 0.88120E+00_JPRB, 0.91057E+00_JPRB, 0.91655E+00_JPRB, 0.89121E+00_JPRB, 0.81038E+00_JPRB, 0.73589E+00_JPRB ,&
&  0.65383E+00_JPRB, 0.63151E+00_JPRB, 0.59720E+00_JPRB, 0.57191E+00_JPRB, 0.55368E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.64233E+00_JPRB, 0.65833E+00_JPRB, 0.72707E+00_JPRB, 0.74710E+00_JPRB, 0.81316E+00_JPRB, 0.83882E+00_JPRB, 0.86436E+00_JPRB ,&
&  0.86697E+00_JPRB, 0.86892E+00_JPRB, 0.86950E+00_JPRB, 0.87308E+00_JPRB, 0.87667E+00_JPRB, 0.88591E+00_JPRB, 0.89441E+00_JPRB ,&
&  0.90806E+00_JPRB, 0.92726E+00_JPRB, 0.95078E+00_JPRB, 0.97541E+00_JPRB, 0.10082E+01_JPRB, 0.10468E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.11554E+01_JPRB, 0.12267E+01_JPRB, 0.13043E+01_JPRB, 0.13923E+01_JPRB, 0.14848E+01_JPRB, 0.15530E+01_JPRB, 0.15613E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.13519E+01_JPRB, 0.13327E+01_JPRB, 0.12671E+01_JPRB, 0.11778E+01_JPRB, 0.11142E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.10698E+01_JPRB, 0.10103E+01_JPRB, 0.10052E+01_JPRB, 0.10281E+01_JPRB, 0.10304E+01_JPRB, 0.10081E+01_JPRB, 0.10403E+01_JPRB ,&
&  0.10828E+01_JPRB, 0.11370E+01_JPRB, 0.11927E+01_JPRB, 0.12406E+01_JPRB, 0.13246E+01_JPRB, 0.14337E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.16829E+01_JPRB, 0.18000E+01_JPRB, 0.18831E+01_JPRB, 0.16628E+01_JPRB, 0.14611E+01_JPRB, 0.13869E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.92205E+00_JPRB, 0.82427E+00_JPRB, 0.88064E+00_JPRB, 0.90227E+00_JPRB, 0.81426E+00_JPRB, 0.73252E+00_JPRB, 0.67759E+00_JPRB ,&
&  0.62979E+00_JPRB, 0.59444E+00_JPRB, 0.58229E+00_JPRB, 0.57165E+00_JPRB, 0.56803E+00_JPRB, 0.58360E+00_JPRB, 0.59762E+00_JPRB ,&
&  0.62048E+00_JPRB, 0.64881E+00_JPRB, 0.67430E+00_JPRB, 0.70562E+00_JPRB, 0.72646E+00_JPRB, 0.75009E+00_JPRB, 0.76264E+00_JPRB ,&
&  0.78145E+00_JPRB, 0.81608E+00_JPRB, 0.82165E+00_JPRB, 0.84822E+00_JPRB, 0.85340E+00_JPRB, 0.87836E+00_JPRB, 0.90395E+00_JPRB ,&
&  0.93105E+00_JPRB, 0.95611E+00_JPRB, 0.96031E+00_JPRB, 0.94061E+00_JPRB, 0.88714E+00_JPRB, 0.81116E+00_JPRB, 0.65237E+00_JPRB ,&
&  0.62631E+00_JPRB, 0.60789E+00_JPRB, 0.61302E+00_JPRB, 0.61745E+00_JPRB, 0.63474E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.80618E+00_JPRB, 0.85600E+00_JPRB, 0.90569E+00_JPRB, 0.96688E+00_JPRB, 0.10153E+01_JPRB, 0.10027E+01_JPRB, 0.97470E+00_JPRB ,&
&  0.95751E+00_JPRB, 0.95051E+00_JPRB, 0.95816E+00_JPRB, 0.97349E+00_JPRB, 0.98551E+00_JPRB, 0.99270E+00_JPRB, 0.10053E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.10516E+01_JPRB, 0.10807E+01_JPRB, 0.11135E+01_JPRB, 0.11412E+01_JPRB, 0.11802E+01_JPRB, 0.12264E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.13314E+01_JPRB, 0.13796E+01_JPRB, 0.14281E+01_JPRB, 0.14729E+01_JPRB, 0.14671E+01_JPRB, 0.10787E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.13569E+01_JPRB, 0.13737E+01_JPRB, 0.13049E+01_JPRB, 0.11904E+01_JPRB, 0.11161E+01_JPRB, 0.10878E+01_JPRB ,&
&  0.10644E+01_JPRB, 0.10347E+01_JPRB, 0.10067E+01_JPRB, 0.99440E+00_JPRB, 0.10825E+01_JPRB, 0.10536E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11826E+01_JPRB, 0.12183E+01_JPRB, 0.12723E+01_JPRB, 0.13646E+01_JPRB, 0.14550E+01_JPRB, 0.15600E+01_JPRB ,&
&  0.16590E+01_JPRB, 0.17397E+01_JPRB, 0.17828E+01_JPRB, 0.15509E+01_JPRB, 0.14591E+01_JPRB, 0.13461E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.84584E+00_JPRB, 0.81135E+00_JPRB, 0.87341E+00_JPRB, 0.88960E+00_JPRB, 0.80200E+00_JPRB, 0.70489E+00_JPRB, 0.65767E+00_JPRB ,&
&  0.60670E+00_JPRB, 0.58128E+00_JPRB, 0.57460E+00_JPRB, 0.55590E+00_JPRB, 0.54819E+00_JPRB, 0.57115E+00_JPRB, 0.60284E+00_JPRB ,&
&  0.64237E+00_JPRB, 0.69158E+00_JPRB, 0.73143E+00_JPRB, 0.76693E+00_JPRB, 0.81399E+00_JPRB, 0.89196E+00_JPRB, 0.93608E+00_JPRB ,&
&  0.86252E+00_JPRB, 0.83296E+00_JPRB, 0.89056E+00_JPRB, 0.93735E+00_JPRB, 0.98775E+00_JPRB, 0.10656E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.13439E+01_JPRB, 0.11431E+01_JPRB, 0.96180E+00_JPRB, 0.12870E+01_JPRB, 0.13699E+01_JPRB, 0.70205E+00_JPRB ,&
&  0.67035E+00_JPRB, 0.68826E+00_JPRB, 0.69953E+00_JPRB, 0.73302E+00_JPRB, 0.76811E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.95509E+00_JPRB, 0.98381E+00_JPRB, 0.10151E+01_JPRB, 0.10339E+01_JPRB, 0.10430E+01_JPRB, 0.10282E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10244E+01_JPRB, 0.10404E+01_JPRB, 0.10578E+01_JPRB, 0.10685E+01_JPRB, 0.10788E+01_JPRB, 0.10990E+01_JPRB ,&
&  0.11245E+01_JPRB, 0.11569E+01_JPRB, 0.11972E+01_JPRB, 0.12267E+01_JPRB, 0.12571E+01_JPRB, 0.12812E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.13439E+01_JPRB, 0.13904E+01_JPRB, 0.13979E+01_JPRB, 0.14239E+01_JPRB, 0.14368E+01_JPRB, 0.14795E+01_JPRB, 0.11866E+01_JPRB ,&
&  0.13402E+01_JPRB, 0.14004E+01_JPRB, 0.13792E+01_JPRB, 0.13036E+01_JPRB, 0.11935E+01_JPRB, 0.11414E+01_JPRB, 0.11079E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10741E+01_JPRB, 0.10330E+01_JPRB, 0.10847E+01_JPRB, 0.11207E+01_JPRB, 0.10848E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.11431E+01_JPRB, 0.11777E+01_JPRB, 0.12161E+01_JPRB, 0.12807E+01_JPRB, 0.13713E+01_JPRB, 0.14521E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.17076E+01_JPRB, 0.17740E+01_JPRB, 0.15956E+01_JPRB, 0.15739E+01_JPRB, 0.14276E+01_JPRB, 0.11136E+01_JPRB ,&
&  0.83550E+00_JPRB, 0.76598E+00_JPRB, 0.80796E+00_JPRB, 0.89109E+00_JPRB, 0.80578E+00_JPRB, 0.69643E+00_JPRB, 0.66635E+00_JPRB ,&
&  0.62299E+00_JPRB, 0.59186E+00_JPRB, 0.58272E+00_JPRB, 0.55880E+00_JPRB, 0.54595E+00_JPRB, 0.57332E+00_JPRB, 0.60014E+00_JPRB ,&
&  0.63603E+00_JPRB, 0.68392E+00_JPRB, 0.73687E+00_JPRB, 0.79135E+00_JPRB, 0.85327E+00_JPRB, 0.87253E+00_JPRB, 0.82303E+00_JPRB ,&
&  0.86522E+00_JPRB, 0.90980E+00_JPRB, 0.97401E+00_JPRB, 0.10852E+01_JPRB, 0.11666E+01_JPRB, 0.13869E+01_JPRB, 0.16255E+01_JPRB ,&
&  0.18479E+01_JPRB, 0.17377E+01_JPRB, 0.20675E+01_JPRB, 0.28748E+01_JPRB, 0.26533E+01_JPRB, 0.21403E+01_JPRB, 0.81045E+00_JPRB ,&
&  0.80226E+00_JPRB, 0.80181E+00_JPRB, 0.83540E+00_JPRB, 0.87798E+00_JPRB, 0.92740E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.95760E+00_JPRB, 0.98157E+00_JPRB, 0.10054E+01_JPRB, 0.10280E+01_JPRB, 0.10449E+01_JPRB, 0.10576E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.11028E+01_JPRB, 0.11173E+01_JPRB, 0.11253E+01_JPRB, 0.11329E+01_JPRB, 0.11533E+01_JPRB, 0.11814E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.12346E+01_JPRB, 0.12719E+01_JPRB, 0.12557E+01_JPRB, 0.12378E+01_JPRB, 0.12430E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.12673E+01_JPRB, 0.12597E+01_JPRB, 0.12874E+01_JPRB, 0.13172E+01_JPRB, 0.13596E+01_JPRB, 0.11368E+01_JPRB ,&
&  0.12893E+01_JPRB, 0.13971E+01_JPRB, 0.13477E+01_JPRB, 0.12734E+01_JPRB, 0.11566E+01_JPRB, 0.11219E+01_JPRB, 0.11119E+01_JPRB ,&
&  0.11031E+01_JPRB, 0.10976E+01_JPRB, 0.10851E+01_JPRB, 0.10796E+01_JPRB, 0.10878E+01_JPRB, 0.10912E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.11340E+01_JPRB, 0.11550E+01_JPRB, 0.11967E+01_JPRB, 0.12487E+01_JPRB, 0.13280E+01_JPRB, 0.14074E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.15151E+01_JPRB, 0.15651E+01_JPRB, 0.14651E+01_JPRB, 0.14666E+01_JPRB, 0.15334E+01_JPRB, 0.13825E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.81119E+00_JPRB, 0.77220E+00_JPRB, 0.79819E+00_JPRB, 0.88681E+00_JPRB, 0.80965E+00_JPRB, 0.73056E+00_JPRB, 0.69246E+00_JPRB ,&
&  0.62226E+00_JPRB, 0.55697E+00_JPRB, 0.56682E+00_JPRB, 0.56096E+00_JPRB, 0.54969E+00_JPRB, 0.56839E+00_JPRB, 0.58807E+00_JPRB ,&
&  0.60906E+00_JPRB, 0.65088E+00_JPRB, 0.69183E+00_JPRB, 0.73306E+00_JPRB, 0.75349E+00_JPRB, 0.71714E+00_JPRB, 0.81087E+00_JPRB ,&
&  0.80934E+00_JPRB, 0.80184E+00_JPRB, 0.84393E+00_JPRB, 0.72999E+00_JPRB, 0.10209E+01_JPRB, 0.10891E+01_JPRB, 0.12083E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.15918E+01_JPRB, 0.19725E+01_JPRB, 0.15365E+01_JPRB, 0.12051E+01_JPRB, 0.87565E+00_JPRB, 0.82030E+00_JPRB ,&
&  0.82535E+00_JPRB, 0.83942E+00_JPRB, 0.86651E+00_JPRB, 0.90000E+00_JPRB, 0.93010E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.95812E+00_JPRB, 0.98070E+00_JPRB, 0.10031E+01_JPRB, 0.10279E+01_JPRB, 0.10528E+01_JPRB, 0.10707E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.11315E+01_JPRB, 0.11400E+01_JPRB, 0.11504E+01_JPRB, 0.11670E+01_JPRB, 0.12038E+01_JPRB, 0.12367E+01_JPRB ,&
&  0.12256E+01_JPRB, 0.12421E+01_JPRB, 0.12520E+01_JPRB, 0.12570E+01_JPRB, 0.12456E+01_JPRB, 0.12153E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.12184E+01_JPRB, 0.12181E+01_JPRB, 0.12401E+01_JPRB, 0.12758E+01_JPRB, 0.13545E+01_JPRB, 0.10948E+01_JPRB ,&
&  0.12086E+01_JPRB, 0.13253E+01_JPRB, 0.13067E+01_JPRB, 0.11927E+01_JPRB, 0.10707E+01_JPRB, 0.10363E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10244E+01_JPRB, 0.10276E+01_JPRB, 0.10370E+01_JPRB, 0.10560E+01_JPRB, 0.10893E+01_JPRB, 0.11002E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.11431E+01_JPRB, 0.11746E+01_JPRB, 0.12135E+01_JPRB, 0.12604E+01_JPRB, 0.13011E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.13453E+01_JPRB, 0.13366E+01_JPRB, 0.12091E+01_JPRB, 0.12042E+01_JPRB, 0.12471E+01_JPRB, 0.11359E+01_JPRB, 0.95234E+00_JPRB ,&
&  0.83592E+00_JPRB, 0.75187E+00_JPRB, 0.71699E+00_JPRB, 0.85112E+00_JPRB, 0.81775E+00_JPRB, 0.75844E+00_JPRB, 0.68609E+00_JPRB ,&
&  0.58932E+00_JPRB, 0.53033E+00_JPRB, 0.52924E+00_JPRB, 0.56399E+00_JPRB, 0.55522E+00_JPRB, 0.55141E+00_JPRB, 0.55336E+00_JPRB ,&
&  0.55866E+00_JPRB, 0.56145E+00_JPRB, 0.57667E+00_JPRB, 0.61485E+00_JPRB, 0.64633E+00_JPRB, 0.60101E+00_JPRB, 0.74533E+00_JPRB ,&
&  0.67809E+00_JPRB, 0.55987E+00_JPRB, 0.63887E+00_JPRB, 0.77620E+00_JPRB, 0.80996E+00_JPRB, 0.82436E+00_JPRB, 0.92844E+00_JPRB ,&
&  0.10741E+01_JPRB, 0.12046E+01_JPRB, 0.12118E+01_JPRB, 0.99187E+00_JPRB, 0.87336E+00_JPRB, 0.82169E+00_JPRB, 0.81340E+00_JPRB ,&
&  0.82154E+00_JPRB, 0.83955E+00_JPRB, 0.86940E+00_JPRB, 0.90387E+00_JPRB, 0.93378E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.94296E+00_JPRB, 0.96707E+00_JPRB, 0.98689E+00_JPRB, 0.10034E+01_JPRB, 0.10193E+01_JPRB, 0.10379E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.10854E+01_JPRB, 0.10946E+01_JPRB, 0.11116E+01_JPRB, 0.11291E+01_JPRB, 0.11670E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.12225E+01_JPRB, 0.12364E+01_JPRB, 0.12352E+01_JPRB, 0.12521E+01_JPRB, 0.12438E+01_JPRB, 0.12233E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.12246E+01_JPRB, 0.12213E+01_JPRB, 0.12202E+01_JPRB, 0.12234E+01_JPRB, 0.12279E+01_JPRB, 0.11992E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.91989E+00_JPRB, 0.12489E+01_JPRB, 0.12794E+01_JPRB, 0.11286E+01_JPRB, 0.10640E+01_JPRB, 0.97127E+00_JPRB, 0.96093E+00_JPRB ,&
&  0.97412E+00_JPRB, 0.97462E+00_JPRB, 0.98741E+00_JPRB, 0.10140E+01_JPRB, 0.10420E+01_JPRB, 0.10716E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.11151E+01_JPRB, 0.11284E+01_JPRB, 0.11397E+01_JPRB, 0.11758E+01_JPRB, 0.11771E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11419E+01_JPRB, 0.89116E+00_JPRB, 0.85586E+00_JPRB, 0.90491E+00_JPRB, 0.86778E+00_JPRB, 0.76998E+00_JPRB ,&
&  0.72927E+00_JPRB, 0.72073E+00_JPRB, 0.77413E+00_JPRB, 0.78262E+00_JPRB, 0.79208E+00_JPRB, 0.77423E+00_JPRB, 0.74059E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.56159E+00_JPRB, 0.53596E+00_JPRB, 0.55986E+00_JPRB, 0.57322E+00_JPRB, 0.57382E+00_JPRB, 0.53449E+00_JPRB ,&
&  0.49940E+00_JPRB, 0.46857E+00_JPRB, 0.45723E+00_JPRB, 0.46749E+00_JPRB, 0.40659E+00_JPRB, 0.51994E+00_JPRB, 0.67704E+00_JPRB ,&
&  0.58600E+00_JPRB, 0.48619E+00_JPRB, 0.49567E+00_JPRB, 0.66088E+00_JPRB, 0.66521E+00_JPRB, 0.62804E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.76764E+00_JPRB, 0.81901E+00_JPRB, 0.83686E+00_JPRB, 0.76892E+00_JPRB, 0.78075E+00_JPRB, 0.76204E+00_JPRB, 0.76426E+00_JPRB ,&
&  0.78208E+00_JPRB, 0.81178E+00_JPRB, 0.84846E+00_JPRB, 0.88394E+00_JPRB, 0.91481E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.89129E+00_JPRB, 0.91080E+00_JPRB, 0.92390E+00_JPRB, 0.93744E+00_JPRB, 0.95644E+00_JPRB, 0.97585E+00_JPRB, 0.98595E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.10234E+01_JPRB, 0.10324E+01_JPRB, 0.10533E+01_JPRB, 0.10632E+01_JPRB, 0.11173E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.12053E+01_JPRB, 0.12129E+01_JPRB, 0.12157E+01_JPRB, 0.12286E+01_JPRB, 0.12284E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.12928E+01_JPRB, 0.13471E+01_JPRB, 0.13988E+01_JPRB, 0.14298E+01_JPRB, 0.15263E+01_JPRB, 0.16538E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.11343E+01_JPRB, 0.11381E+01_JPRB, 0.96257E+00_JPRB, 0.96432E+00_JPRB, 0.97139E+00_JPRB, 0.92717E+00_JPRB ,&
&  0.91232E+00_JPRB, 0.94448E+00_JPRB, 0.97557E+00_JPRB, 0.99699E+00_JPRB, 0.10109E+01_JPRB, 0.10300E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10484E+01_JPRB, 0.10759E+01_JPRB, 0.11120E+01_JPRB, 0.10477E+01_JPRB, 0.11340E+01_JPRB, 0.11490E+01_JPRB ,&
&  0.11448E+01_JPRB, 0.10272E+01_JPRB, 0.75265E+00_JPRB, 0.70572E+00_JPRB, 0.72848E+00_JPRB, 0.69815E+00_JPRB, 0.65191E+00_JPRB ,&
&  0.67605E+00_JPRB, 0.66503E+00_JPRB, 0.57933E+00_JPRB, 0.69800E+00_JPRB, 0.77302E+00_JPRB, 0.80617E+00_JPRB, 0.78066E+00_JPRB ,&
&  0.73528E+00_JPRB, 0.65373E+00_JPRB, 0.62584E+00_JPRB, 0.62229E+00_JPRB, 0.62131E+00_JPRB, 0.60167E+00_JPRB, 0.59328E+00_JPRB ,&
&  0.55306E+00_JPRB, 0.49515E+00_JPRB, 0.46655E+00_JPRB, 0.43014E+00_JPRB, 0.46305E+00_JPRB, 0.41127E+00_JPRB, 0.60277E+00_JPRB ,&
&  0.50060E+00_JPRB, 0.48657E+00_JPRB, 0.47814E+00_JPRB, 0.67215E+00_JPRB, 0.58121E+00_JPRB, 0.49357E+00_JPRB, 0.54041E+00_JPRB ,&
&  0.59939E+00_JPRB, 0.62824E+00_JPRB, 0.63781E+00_JPRB, 0.65345E+00_JPRB, 0.69664E+00_JPRB, 0.71946E+00_JPRB, 0.73917E+00_JPRB ,&
&  0.74149E+00_JPRB, 0.77314E+00_JPRB, 0.81693E+00_JPRB, 0.85005E+00_JPRB, 0.86351E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.76158E+00_JPRB, 0.77534E+00_JPRB, 0.79429E+00_JPRB, 0.81455E+00_JPRB, 0.82651E+00_JPRB, 0.84591E+00_JPRB, 0.86522E+00_JPRB ,&
&  0.89128E+00_JPRB, 0.92524E+00_JPRB, 0.94691E+00_JPRB, 0.95695E+00_JPRB, 0.97489E+00_JPRB, 0.99627E+00_JPRB, 0.10073E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.10434E+01_JPRB, 0.10985E+01_JPRB, 0.11257E+01_JPRB, 0.11632E+01_JPRB, 0.12152E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12183E+01_JPRB, 0.12091E+01_JPRB, 0.12250E+01_JPRB, 0.12251E+01_JPRB, 0.13058E+01_JPRB, 0.13168E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10691E+01_JPRB, 0.11903E+01_JPRB, 0.94838E+00_JPRB, 0.93147E+00_JPRB, 0.91757E+00_JPRB, 0.83612E+00_JPRB ,&
&  0.83234E+00_JPRB, 0.87343E+00_JPRB, 0.92196E+00_JPRB, 0.96692E+00_JPRB, 0.10226E+01_JPRB, 0.10364E+01_JPRB, 0.10282E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10480E+01_JPRB, 0.10598E+01_JPRB, 0.98767E+00_JPRB, 0.91265E+00_JPRB, 0.97976E+00_JPRB, 0.10019E+01_JPRB ,&
&  0.99381E+00_JPRB, 0.95869E+00_JPRB, 0.77035E+00_JPRB, 0.70487E+00_JPRB, 0.71715E+00_JPRB, 0.63883E+00_JPRB, 0.60601E+00_JPRB ,&
&  0.64429E+00_JPRB, 0.56866E+00_JPRB, 0.39232E+00_JPRB, 0.54603E+00_JPRB, 0.70174E+00_JPRB, 0.79561E+00_JPRB, 0.83716E+00_JPRB ,&
&  0.81588E+00_JPRB, 0.74307E+00_JPRB, 0.71466E+00_JPRB, 0.73436E+00_JPRB, 0.75548E+00_JPRB, 0.70690E+00_JPRB, 0.68025E+00_JPRB ,&
&  0.61345E+00_JPRB, 0.53647E+00_JPRB, 0.47949E+00_JPRB, 0.48251E+00_JPRB, 0.47541E+00_JPRB, 0.48564E+00_JPRB, 0.58989E+00_JPRB ,&
&  0.58685E+00_JPRB, 0.52683E+00_JPRB, 0.67515E+00_JPRB, 0.84420E+00_JPRB, 0.48716E+00_JPRB, 0.48101E+00_JPRB, 0.48765E+00_JPRB ,&
&  0.51535E+00_JPRB, 0.53231E+00_JPRB, 0.54515E+00_JPRB, 0.56896E+00_JPRB, 0.60326E+00_JPRB, 0.61655E+00_JPRB, 0.62119E+00_JPRB ,&
&  0.63892E+00_JPRB, 0.66768E+00_JPRB, 0.68921E+00_JPRB, 0.71591E+00_JPRB, 0.73979E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.59065E+00_JPRB, 0.58922E+00_JPRB, 0.59204E+00_JPRB, 0.59876E+00_JPRB, 0.60030E+00_JPRB, 0.59979E+00_JPRB, 0.61389E+00_JPRB ,&
&  0.63879E+00_JPRB, 0.68377E+00_JPRB, 0.74883E+00_JPRB, 0.78842E+00_JPRB, 0.77268E+00_JPRB, 0.72308E+00_JPRB, 0.70419E+00_JPRB ,&
&  0.70352E+00_JPRB, 0.70367E+00_JPRB, 0.73586E+00_JPRB, 0.78735E+00_JPRB, 0.85262E+00_JPRB, 0.87617E+00_JPRB, 0.93771E+00_JPRB ,&
&  0.93815E+00_JPRB, 0.96024E+00_JPRB, 0.10507E+01_JPRB, 0.11336E+01_JPRB, 0.17153E+01_JPRB, 0.16461E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.13822E+01_JPRB, 0.14040E+01_JPRB, 0.12888E+01_JPRB, 0.11213E+01_JPRB, 0.11025E+01_JPRB, 0.10689E+01_JPRB, 0.10953E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11429E+01_JPRB, 0.11884E+01_JPRB, 0.12452E+01_JPRB, 0.12834E+01_JPRB, 0.12862E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.13128E+01_JPRB, 0.13129E+01_JPRB, 0.13008E+01_JPRB, 0.11292E+01_JPRB, 0.11610E+01_JPRB, 0.11763E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.10771E+01_JPRB, 0.11186E+01_JPRB, 0.10407E+01_JPRB, 0.96250E+00_JPRB, 0.85255E+00_JPRB, 0.74843E+00_JPRB ,&
&  0.68569E+00_JPRB, 0.53289E+00_JPRB, 0.38821E+00_JPRB, 0.58218E+00_JPRB, 0.78924E+00_JPRB, 0.77579E+00_JPRB, 0.89616E+00_JPRB ,&
&  0.91137E+00_JPRB, 0.87805E+00_JPRB, 0.84673E+00_JPRB, 0.84428E+00_JPRB, 0.81196E+00_JPRB, 0.83134E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.70058E+00_JPRB, 0.58121E+00_JPRB, 0.51934E+00_JPRB, 0.51806E+00_JPRB, 0.53130E+00_JPRB, 0.53486E+00_JPRB, 0.10493E+01_JPRB ,&
&  0.95319E+00_JPRB, 0.65484E+00_JPRB, 0.67767E+00_JPRB, 0.74281E+00_JPRB, 0.55496E+00_JPRB, 0.46012E+00_JPRB, 0.46186E+00_JPRB ,&
&  0.46594E+00_JPRB, 0.47923E+00_JPRB, 0.50396E+00_JPRB, 0.52027E+00_JPRB, 0.53173E+00_JPRB, 0.54095E+00_JPRB, 0.55404E+00_JPRB ,&
&  0.56243E+00_JPRB, 0.57332E+00_JPRB, 0.57779E+00_JPRB, 0.58310E+00_JPRB, 0.58850E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.62160E+00_JPRB, 0.62166E+00_JPRB, 0.60752E+00_JPRB, 0.62314E+00_JPRB, 0.65355E+00_JPRB, 0.72150E+00_JPRB, 0.78708E+00_JPRB ,&
&  0.86579E+00_JPRB, 0.96311E+00_JPRB, 0.10902E+01_JPRB, 0.11978E+01_JPRB, 0.12251E+01_JPRB, 0.12009E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.95858E+00_JPRB, 0.82142E+00_JPRB, 0.74888E+00_JPRB, 0.73390E+00_JPRB, 0.78848E+00_JPRB, 0.82059E+00_JPRB, 0.89241E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.11881E+01_JPRB, 0.14040E+01_JPRB, 0.18775E+01_JPRB, 0.22476E+01_JPRB, 0.15817E+01_JPRB, 0.15088E+01_JPRB ,&
&  0.17839E+01_JPRB, 0.18372E+01_JPRB, 0.14158E+01_JPRB, 0.14044E+01_JPRB, 0.15437E+01_JPRB, 0.16693E+01_JPRB, 0.18198E+01_JPRB ,&
&  0.18978E+01_JPRB, 0.19722E+01_JPRB, 0.20251E+01_JPRB, 0.20525E+01_JPRB, 0.20775E+01_JPRB, 0.20719E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.19695E+01_JPRB, 0.18691E+01_JPRB, 0.18325E+01_JPRB, 0.18464E+01_JPRB, 0.18695E+01_JPRB, 0.18150E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.15911E+01_JPRB, 0.15431E+01_JPRB, 0.16315E+01_JPRB, 0.15933E+01_JPRB, 0.14253E+01_JPRB, 0.13134E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.97909E+00_JPRB, 0.82114E+00_JPRB, 0.67541E+00_JPRB, 0.10434E+01_JPRB, 0.12612E+01_JPRB, 0.11539E+01_JPRB, 0.10573E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10038E+01_JPRB, 0.96210E+00_JPRB, 0.92787E+00_JPRB, 0.89061E+00_JPRB, 0.98880E+00_JPRB, 0.82946E+00_JPRB ,&
&  0.77414E+00_JPRB, 0.62683E+00_JPRB, 0.59085E+00_JPRB, 0.64766E+00_JPRB, 0.71431E+00_JPRB, 0.65645E+00_JPRB, 0.94319E+00_JPRB ,&
&  0.94749E+00_JPRB, 0.96680E+00_JPRB, 0.10890E+01_JPRB, 0.14680E+01_JPRB, 0.18408E+01_JPRB, 0.53609E+00_JPRB, 0.51289E+00_JPRB ,&
&  0.50384E+00_JPRB, 0.50652E+00_JPRB, 0.52457E+00_JPRB, 0.53000E+00_JPRB, 0.54352E+00_JPRB, 0.54643E+00_JPRB, 0.54226E+00_JPRB ,&
&  0.54571E+00_JPRB, 0.56238E+00_JPRB, 0.58062E+00_JPRB, 0.58952E+00_JPRB, 0.60961E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.89996E+00_JPRB, 0.93715E+00_JPRB, 0.93765E+00_JPRB, 0.10094E+01_JPRB, 0.10820E+01_JPRB, 0.11165E+01_JPRB, 0.11434E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.12915E+01_JPRB, 0.13615E+01_JPRB, 0.14043E+01_JPRB, 0.14494E+01_JPRB, 0.15178E+01_JPRB, 0.15308E+01_JPRB ,&
&  0.14975E+01_JPRB, 0.14710E+01_JPRB, 0.14603E+01_JPRB, 0.14551E+01_JPRB, 0.14749E+01_JPRB, 0.15178E+01_JPRB, 0.16771E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.17848E+01_JPRB, 0.17809E+01_JPRB, 0.15327E+01_JPRB, 0.11337E+01_JPRB, 0.14484E+01_JPRB, 0.15217E+01_JPRB ,&
&  0.17046E+01_JPRB, 0.17848E+01_JPRB, 0.17293E+01_JPRB, 0.18276E+01_JPRB, 0.19388E+01_JPRB, 0.20723E+01_JPRB, 0.22108E+01_JPRB ,&
&  0.23110E+01_JPRB, 0.24177E+01_JPRB, 0.25290E+01_JPRB, 0.26130E+01_JPRB, 0.27019E+01_JPRB, 0.28187E+01_JPRB, 0.28423E+01_JPRB ,&
&  0.28284E+01_JPRB, 0.27300E+01_JPRB, 0.26480E+01_JPRB, 0.26354E+01_JPRB, 0.25011E+01_JPRB, 0.23523E+01_JPRB, 0.21416E+01_JPRB ,&
&  0.20464E+01_JPRB, 0.20373E+01_JPRB, 0.21209E+01_JPRB, 0.21569E+01_JPRB, 0.20726E+01_JPRB, 0.19150E+01_JPRB, 0.17349E+01_JPRB ,&
&  0.15813E+01_JPRB, 0.14443E+01_JPRB, 0.14748E+01_JPRB, 0.16039E+01_JPRB, 0.12608E+01_JPRB, 0.12789E+01_JPRB, 0.13626E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.11061E+01_JPRB, 0.10340E+01_JPRB, 0.93074E+00_JPRB, 0.93768E+00_JPRB, 0.83559E+00_JPRB, 0.90014E+00_JPRB ,&
&  0.86043E+00_JPRB, 0.70669E+00_JPRB, 0.70474E+00_JPRB, 0.83915E+00_JPRB, 0.93110E+00_JPRB, 0.83927E+00_JPRB, 0.94240E+00_JPRB ,&
&  0.10813E+01_JPRB, 0.95210E+00_JPRB, 0.96128E+00_JPRB, 0.11069E+01_JPRB, 0.10740E+01_JPRB, 0.61189E+00_JPRB, 0.52501E+00_JPRB ,&
&  0.50632E+00_JPRB, 0.51572E+00_JPRB, 0.53285E+00_JPRB, 0.54977E+00_JPRB, 0.58096E+00_JPRB, 0.61704E+00_JPRB, 0.64661E+00_JPRB ,&
&  0.67108E+00_JPRB, 0.68757E+00_JPRB, 0.72367E+00_JPRB, 0.77579E+00_JPRB, 0.84814E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.12324E+01_JPRB, 0.13066E+01_JPRB, 0.13737E+01_JPRB, 0.13524E+01_JPRB, 0.13402E+01_JPRB, 0.13048E+01_JPRB, 0.12782E+01_JPRB ,&
&  0.13213E+01_JPRB, 0.13678E+01_JPRB, 0.14370E+01_JPRB, 0.14925E+01_JPRB, 0.15329E+01_JPRB, 0.16108E+01_JPRB, 0.17025E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.17641E+01_JPRB, 0.17929E+01_JPRB, 0.18321E+01_JPRB, 0.18349E+01_JPRB, 0.18776E+01_JPRB, 0.18668E+01_JPRB ,&
&  0.14288E+01_JPRB, 0.11208E+01_JPRB, 0.13410E+01_JPRB, 0.15611E+01_JPRB, 0.15324E+01_JPRB, 0.16114E+01_JPRB, 0.17163E+01_JPRB ,&
&  0.19335E+01_JPRB, 0.20204E+01_JPRB, 0.20259E+01_JPRB, 0.20283E+01_JPRB, 0.20350E+01_JPRB, 0.21861E+01_JPRB, 0.23403E+01_JPRB ,&
&  0.24701E+01_JPRB, 0.25875E+01_JPRB, 0.27044E+01_JPRB, 0.28237E+01_JPRB, 0.29243E+01_JPRB, 0.30286E+01_JPRB, 0.30987E+01_JPRB ,&
&  0.31312E+01_JPRB, 0.30701E+01_JPRB, 0.30043E+01_JPRB, 0.28811E+01_JPRB, 0.27449E+01_JPRB, 0.26142E+01_JPRB, 0.24731E+01_JPRB ,&
&  0.24814E+01_JPRB, 0.25848E+01_JPRB, 0.27582E+01_JPRB, 0.28714E+01_JPRB, 0.29002E+01_JPRB, 0.27381E+01_JPRB, 0.25649E+01_JPRB ,&
&  0.23088E+01_JPRB, 0.20429E+01_JPRB, 0.21249E+01_JPRB, 0.21842E+01_JPRB, 0.17012E+01_JPRB, 0.15732E+01_JPRB, 0.14380E+01_JPRB ,&
&  0.14756E+01_JPRB, 0.12941E+01_JPRB, 0.11360E+01_JPRB, 0.10845E+01_JPRB, 0.11781E+01_JPRB, 0.10856E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10019E+01_JPRB, 0.82818E+00_JPRB, 0.78116E+00_JPRB, 0.94531E+00_JPRB, 0.91420E+00_JPRB, 0.88490E+00_JPRB, 0.11007E+01_JPRB ,&
&  0.13237E+01_JPRB, 0.94517E+00_JPRB, 0.92234E+00_JPRB, 0.89700E+00_JPRB, 0.76240E+00_JPRB, 0.59696E+00_JPRB, 0.54432E+00_JPRB ,&
&  0.52161E+00_JPRB, 0.53035E+00_JPRB, 0.56625E+00_JPRB, 0.58460E+00_JPRB, 0.63348E+00_JPRB, 0.68411E+00_JPRB, 0.77984E+00_JPRB ,&
&  0.85812E+00_JPRB, 0.92788E+00_JPRB, 0.98393E+00_JPRB, 0.10553E+01_JPRB, 0.11356E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.12352E+01_JPRB, 0.13526E+01_JPRB, 0.14687E+01_JPRB, 0.15445E+01_JPRB, 0.14978E+01_JPRB, 0.14413E+01_JPRB, 0.14076E+01_JPRB ,&
&  0.13908E+01_JPRB, 0.14137E+01_JPRB, 0.14803E+01_JPRB, 0.15420E+01_JPRB, 0.15914E+01_JPRB, 0.16679E+01_JPRB, 0.17843E+01_JPRB ,&
&  0.18845E+01_JPRB, 0.19642E+01_JPRB, 0.20315E+01_JPRB, 0.21042E+01_JPRB, 0.21246E+01_JPRB, 0.21624E+01_JPRB, 0.18799E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.13064E+01_JPRB, 0.17442E+01_JPRB, 0.17555E+01_JPRB, 0.17795E+01_JPRB, 0.18754E+01_JPRB, 0.20019E+01_JPRB ,&
&  0.21952E+01_JPRB, 0.22068E+01_JPRB, 0.21763E+01_JPRB, 0.21301E+01_JPRB, 0.21228E+01_JPRB, 0.22546E+01_JPRB, 0.24109E+01_JPRB ,&
&  0.25456E+01_JPRB, 0.26372E+01_JPRB, 0.27474E+01_JPRB, 0.28936E+01_JPRB, 0.30718E+01_JPRB, 0.32265E+01_JPRB, 0.33493E+01_JPRB ,&
&  0.33155E+01_JPRB, 0.33235E+01_JPRB, 0.31907E+01_JPRB, 0.29513E+01_JPRB, 0.28889E+01_JPRB, 0.29210E+01_JPRB, 0.29682E+01_JPRB ,&
&  0.31058E+01_JPRB, 0.33701E+01_JPRB, 0.36226E+01_JPRB, 0.36908E+01_JPRB, 0.35050E+01_JPRB, 0.35046E+01_JPRB, 0.32504E+01_JPRB ,&
&  0.32137E+01_JPRB, 0.28773E+01_JPRB, 0.28853E+01_JPRB, 0.24993E+01_JPRB, 0.23725E+01_JPRB, 0.22076E+01_JPRB, 0.21097E+01_JPRB ,&
&  0.19784E+01_JPRB, 0.17865E+01_JPRB, 0.15603E+01_JPRB, 0.14703E+01_JPRB, 0.16007E+01_JPRB, 0.16399E+01_JPRB, 0.16276E+01_JPRB ,&
&  0.13401E+01_JPRB, 0.10988E+01_JPRB, 0.88582E+00_JPRB, 0.98092E+00_JPRB, 0.85263E+00_JPRB, 0.87976E+00_JPRB, 0.14046E+01_JPRB ,&
&  0.17463E+01_JPRB, 0.12919E+01_JPRB, 0.11956E+01_JPRB, 0.10381E+01_JPRB, 0.78341E+00_JPRB, 0.61273E+00_JPRB, 0.54910E+00_JPRB ,&
&  0.53822E+00_JPRB, 0.58300E+00_JPRB, 0.61888E+00_JPRB, 0.64081E+00_JPRB, 0.69984E+00_JPRB, 0.75800E+00_JPRB, 0.82764E+00_JPRB ,&
&  0.86305E+00_JPRB, 0.91507E+00_JPRB, 0.97230E+00_JPRB, 0.10307E+01_JPRB, 0.11216E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.99393E+00_JPRB, 0.10430E+01_JPRB, 0.10838E+01_JPRB, 0.11106E+01_JPRB, 0.11342E+01_JPRB, 0.11136E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.12173E+01_JPRB, 0.13331E+01_JPRB, 0.14406E+01_JPRB, 0.15083E+01_JPRB, 0.15326E+01_JPRB, 0.15926E+01_JPRB, 0.17250E+01_JPRB ,&
&  0.18465E+01_JPRB, 0.19806E+01_JPRB, 0.20963E+01_JPRB, 0.22125E+01_JPRB, 0.22093E+01_JPRB, 0.21536E+01_JPRB, 0.16654E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.18370E+01_JPRB, 0.16182E+01_JPRB, 0.17800E+01_JPRB, 0.19706E+01_JPRB, 0.21829E+01_JPRB, 0.21829E+01_JPRB ,&
&  0.22230E+01_JPRB, 0.22298E+01_JPRB, 0.21702E+01_JPRB, 0.21039E+01_JPRB, 0.20723E+01_JPRB, 0.21623E+01_JPRB, 0.23022E+01_JPRB ,&
&  0.24424E+01_JPRB, 0.25598E+01_JPRB, 0.27044E+01_JPRB, 0.29067E+01_JPRB, 0.31457E+01_JPRB, 0.33950E+01_JPRB, 0.35443E+01_JPRB ,&
&  0.35495E+01_JPRB, 0.35530E+01_JPRB, 0.33416E+01_JPRB, 0.30620E+01_JPRB, 0.30038E+01_JPRB, 0.32340E+01_JPRB, 0.35532E+01_JPRB ,&
&  0.37499E+01_JPRB, 0.39239E+01_JPRB, 0.44385E+01_JPRB, 0.47090E+01_JPRB, 0.45882E+01_JPRB, 0.44348E+01_JPRB, 0.38767E+01_JPRB ,&
&  0.41139E+01_JPRB, 0.37842E+01_JPRB, 0.36627E+01_JPRB, 0.31527E+01_JPRB, 0.29884E+01_JPRB, 0.30026E+01_JPRB, 0.29913E+01_JPRB ,&
&  0.27577E+01_JPRB, 0.26501E+01_JPRB, 0.23221E+01_JPRB, 0.21434E+01_JPRB, 0.21681E+01_JPRB, 0.22780E+01_JPRB, 0.21471E+01_JPRB ,&
&  0.18702E+01_JPRB, 0.13120E+01_JPRB, 0.10259E+01_JPRB, 0.99767E+00_JPRB, 0.92197E+00_JPRB, 0.97100E+00_JPRB, 0.16188E+01_JPRB ,&
&  0.20306E+01_JPRB, 0.18698E+01_JPRB, 0.18024E+01_JPRB, 0.14695E+01_JPRB, 0.10359E+01_JPRB, 0.73919E+00_JPRB, 0.61663E+00_JPRB ,&
&  0.61158E+00_JPRB, 0.66088E+00_JPRB, 0.68674E+00_JPRB, 0.70703E+00_JPRB, 0.75358E+00_JPRB, 0.75453E+00_JPRB, 0.80976E+00_JPRB ,&
&  0.82850E+00_JPRB, 0.86273E+00_JPRB, 0.89270E+00_JPRB, 0.90312E+00_JPRB, 0.93905E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.80788E+00_JPRB, 0.83319E+00_JPRB, 0.87576E+00_JPRB, 0.90256E+00_JPRB, 0.91253E+00_JPRB, 0.93389E+00_JPRB, 0.94713E+00_JPRB ,&
&  0.10070E+01_JPRB, 0.11133E+01_JPRB, 0.12370E+01_JPRB, 0.13633E+01_JPRB, 0.13982E+01_JPRB, 0.14097E+01_JPRB, 0.15126E+01_JPRB ,&
&  0.16865E+01_JPRB, 0.19423E+01_JPRB, 0.22154E+01_JPRB, 0.24518E+01_JPRB, 0.24902E+01_JPRB, 0.23568E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.14025E+01_JPRB, 0.17796E+01_JPRB, 0.16800E+01_JPRB, 0.17863E+01_JPRB, 0.20235E+01_JPRB, 0.22203E+01_JPRB, 0.21613E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.20847E+01_JPRB, 0.19836E+01_JPRB, 0.19394E+01_JPRB, 0.19414E+01_JPRB, 0.19677E+01_JPRB, 0.20399E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.22949E+01_JPRB, 0.24991E+01_JPRB, 0.27899E+01_JPRB, 0.30970E+01_JPRB, 0.33883E+01_JPRB, 0.36192E+01_JPRB ,&
&  0.37330E+01_JPRB, 0.37012E+01_JPRB, 0.36380E+01_JPRB, 0.31621E+01_JPRB, 0.31378E+01_JPRB, 0.34758E+01_JPRB, 0.40330E+01_JPRB ,&
&  0.43474E+01_JPRB, 0.45454E+01_JPRB, 0.52403E+01_JPRB, 0.58620E+01_JPRB, 0.58635E+01_JPRB, 0.53383E+01_JPRB, 0.47418E+01_JPRB ,&
&  0.47771E+01_JPRB, 0.46311E+01_JPRB, 0.42375E+01_JPRB, 0.39514E+01_JPRB, 0.38233E+01_JPRB, 0.39577E+01_JPRB, 0.41246E+01_JPRB ,&
&  0.37115E+01_JPRB, 0.34296E+01_JPRB, 0.30891E+01_JPRB, 0.28421E+01_JPRB, 0.26525E+01_JPRB, 0.25733E+01_JPRB, 0.26952E+01_JPRB ,&
&  0.24635E+01_JPRB, 0.18088E+01_JPRB, 0.13572E+01_JPRB, 0.10655E+01_JPRB, 0.97259E+00_JPRB, 0.10910E+01_JPRB, 0.15839E+01_JPRB ,&
&  0.23543E+01_JPRB, 0.24431E+01_JPRB, 0.22507E+01_JPRB, 0.22282E+01_JPRB, 0.16354E+01_JPRB, 0.10352E+01_JPRB, 0.69594E+00_JPRB ,&
&  0.65133E+00_JPRB, 0.69775E+00_JPRB, 0.71570E+00_JPRB, 0.69842E+00_JPRB, 0.72279E+00_JPRB, 0.75375E+00_JPRB, 0.74632E+00_JPRB ,&
&  0.77406E+00_JPRB, 0.80189E+00_JPRB, 0.79315E+00_JPRB, 0.77669E+00_JPRB, 0.78988E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.71311E+00_JPRB, 0.74212E+00_JPRB, 0.74722E+00_JPRB, 0.74570E+00_JPRB, 0.75777E+00_JPRB, 0.77881E+00_JPRB, 0.78187E+00_JPRB ,&
&  0.83311E+00_JPRB, 0.91443E+00_JPRB, 0.10145E+01_JPRB, 0.11491E+01_JPRB, 0.12274E+01_JPRB, 0.12320E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.16491E+01_JPRB, 0.20649E+01_JPRB, 0.24559E+01_JPRB, 0.25057E+01_JPRB, 0.22602E+01_JPRB, 0.20769E+01_JPRB ,&
&  0.18212E+01_JPRB, 0.19888E+01_JPRB, 0.18694E+01_JPRB, 0.21589E+01_JPRB, 0.25150E+01_JPRB, 0.28334E+01_JPRB, 0.20682E+01_JPRB ,&
&  0.19340E+01_JPRB, 0.19014E+01_JPRB, 0.18117E+01_JPRB, 0.17937E+01_JPRB, 0.17345E+01_JPRB, 0.17285E+01_JPRB, 0.17310E+01_JPRB ,&
&  0.17729E+01_JPRB, 0.19407E+01_JPRB, 0.21752E+01_JPRB, 0.25113E+01_JPRB, 0.29553E+01_JPRB, 0.33162E+01_JPRB, 0.35831E+01_JPRB ,&
&  0.36890E+01_JPRB, 0.38406E+01_JPRB, 0.38769E+01_JPRB, 0.34587E+01_JPRB, 0.32398E+01_JPRB, 0.36411E+01_JPRB, 0.44157E+01_JPRB ,&
&  0.53507E+01_JPRB, 0.60737E+01_JPRB, 0.63472E+01_JPRB, 0.63887E+01_JPRB, 0.63992E+01_JPRB, 0.61786E+01_JPRB, 0.54395E+01_JPRB ,&
&  0.51964E+01_JPRB, 0.50635E+01_JPRB, 0.45079E+01_JPRB, 0.46130E+01_JPRB, 0.47734E+01_JPRB, 0.50834E+01_JPRB, 0.44833E+01_JPRB ,&
&  0.38393E+01_JPRB, 0.34290E+01_JPRB, 0.31897E+01_JPRB, 0.32528E+01_JPRB, 0.32245E+01_JPRB, 0.30083E+01_JPRB, 0.31428E+01_JPRB ,&
&  0.24598E+01_JPRB, 0.96465E+00_JPRB, 0.67671E+00_JPRB, 0.84860E+00_JPRB, 0.62229E+00_JPRB, 0.77722E+00_JPRB, 0.16829E+01_JPRB ,&
&  0.27075E+01_JPRB, 0.35243E+01_JPRB, 0.34736E+01_JPRB, 0.27628E+01_JPRB, 0.24980E+01_JPRB, 0.15412E+01_JPRB, 0.83469E+00_JPRB ,&
&  0.71758E+00_JPRB, 0.77997E+00_JPRB, 0.79149E+00_JPRB, 0.73951E+00_JPRB, 0.72166E+00_JPRB, 0.74892E+00_JPRB, 0.68942E+00_JPRB ,&
&  0.71292E+00_JPRB, 0.72675E+00_JPRB, 0.73289E+00_JPRB, 0.72103E+00_JPRB, 0.70138E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.67156E+00_JPRB, 0.69873E+00_JPRB, 0.71822E+00_JPRB, 0.68276E+00_JPRB, 0.67244E+00_JPRB, 0.68854E+00_JPRB, 0.69146E+00_JPRB ,&
&  0.73847E+00_JPRB, 0.79521E+00_JPRB, 0.85485E+00_JPRB, 0.95544E+00_JPRB, 0.10516E+01_JPRB, 0.10827E+01_JPRB, 0.11040E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.13579E+01_JPRB, 0.17517E+01_JPRB, 0.22973E+01_JPRB, 0.25868E+01_JPRB, 0.24298E+01_JPRB, 0.24980E+01_JPRB ,&
&  0.20953E+01_JPRB, 0.23701E+01_JPRB, 0.30181E+01_JPRB, 0.37763E+01_JPRB, 0.39793E+01_JPRB, 0.36998E+01_JPRB, 0.23952E+01_JPRB ,&
&  0.19433E+01_JPRB, 0.18836E+01_JPRB, 0.17254E+01_JPRB, 0.17554E+01_JPRB, 0.15873E+01_JPRB, 0.15693E+01_JPRB, 0.15459E+01_JPRB ,&
&  0.16144E+01_JPRB, 0.17609E+01_JPRB, 0.19156E+01_JPRB, 0.22307E+01_JPRB, 0.27504E+01_JPRB, 0.32418E+01_JPRB, 0.37191E+01_JPRB ,&
&  0.37745E+01_JPRB, 0.39420E+01_JPRB, 0.40021E+01_JPRB, 0.41313E+01_JPRB, 0.30414E+01_JPRB, 0.40515E+01_JPRB, 0.47666E+01_JPRB ,&
&  0.59570E+01_JPRB, 0.67337E+01_JPRB, 0.65988E+01_JPRB, 0.74180E+01_JPRB, 0.76384E+01_JPRB, 0.68832E+01_JPRB, 0.58462E+01_JPRB ,&
&  0.54860E+01_JPRB, 0.49586E+01_JPRB, 0.47314E+01_JPRB, 0.49194E+01_JPRB, 0.52642E+01_JPRB, 0.61186E+01_JPRB, 0.41657E+01_JPRB ,&
&  0.37693E+01_JPRB, 0.41117E+01_JPRB, 0.37273E+01_JPRB, 0.32026E+01_JPRB, 0.34994E+01_JPRB, 0.35359E+01_JPRB, 0.25937E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.58963E+00_JPRB, 0.50558E+00_JPRB, 0.50689E+00_JPRB, 0.47495E+00_JPRB, 0.83023E+00_JPRB, 0.34724E+01_JPRB ,&
&  0.27386E+01_JPRB, 0.41437E+01_JPRB, 0.42744E+01_JPRB, 0.36595E+01_JPRB, 0.32727E+01_JPRB, 0.22332E+01_JPRB, 0.17047E+01_JPRB ,&
&  0.83800E+00_JPRB, 0.81850E+00_JPRB, 0.89558E+00_JPRB, 0.88769E+00_JPRB, 0.88882E+00_JPRB, 0.83673E+00_JPRB, 0.75761E+00_JPRB ,&
&  0.76036E+00_JPRB, 0.77727E+00_JPRB, 0.73697E+00_JPRB, 0.69544E+00_JPRB, 0.67143E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.66772E+00_JPRB, 0.67114E+00_JPRB, 0.67584E+00_JPRB, 0.66250E+00_JPRB, 0.62563E+00_JPRB, 0.62788E+00_JPRB, 0.64831E+00_JPRB ,&
&  0.67740E+00_JPRB, 0.71332E+00_JPRB, 0.75651E+00_JPRB, 0.81865E+00_JPRB, 0.89855E+00_JPRB, 0.95453E+00_JPRB, 0.98909E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.12223E+01_JPRB, 0.15387E+01_JPRB, 0.19389E+01_JPRB, 0.24468E+01_JPRB, 0.23640E+01_JPRB, 0.27038E+01_JPRB ,&
&  0.23492E+01_JPRB, 0.25716E+01_JPRB, 0.35915E+01_JPRB, 0.49742E+01_JPRB, 0.65418E+01_JPRB, 0.61812E+01_JPRB, 0.45019E+01_JPRB ,&
&  0.30667E+01_JPRB, 0.24806E+01_JPRB, 0.22070E+01_JPRB, 0.19093E+01_JPRB, 0.16764E+01_JPRB, 0.15976E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.15839E+01_JPRB, 0.16353E+01_JPRB, 0.16980E+01_JPRB, 0.19323E+01_JPRB, 0.24053E+01_JPRB, 0.29798E+01_JPRB, 0.36914E+01_JPRB ,&
&  0.41350E+01_JPRB, 0.44328E+01_JPRB, 0.43971E+01_JPRB, 0.43852E+01_JPRB, 0.40721E+01_JPRB, 0.39882E+01_JPRB, 0.48147E+01_JPRB ,&
&  0.63612E+01_JPRB, 0.77518E+01_JPRB, 0.76664E+01_JPRB, 0.90138E+01_JPRB, 0.83230E+01_JPRB, 0.74460E+01_JPRB, 0.64967E+01_JPRB ,&
&  0.56453E+01_JPRB, 0.57710E+01_JPRB, 0.46792E+01_JPRB, 0.47984E+01_JPRB, 0.53874E+01_JPRB, 0.48606E+01_JPRB, 0.44200E+01_JPRB ,&
&  0.48819E+01_JPRB, 0.41142E+01_JPRB, 0.36104E+01_JPRB, 0.30686E+01_JPRB, 0.38146E+01_JPRB, 0.32740E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.95564E+00_JPRB, 0.68676E+00_JPRB, 0.48056E+00_JPRB, 0.49484E+00_JPRB, 0.53703E+00_JPRB, 0.82328E+00_JPRB, 0.22715E+01_JPRB ,&
&  0.27546E+01_JPRB, 0.42590E+01_JPRB, 0.46552E+01_JPRB, 0.41306E+01_JPRB, 0.34760E+01_JPRB, 0.28978E+01_JPRB, 0.30770E+01_JPRB ,&
&  0.19003E+01_JPRB, 0.12018E+01_JPRB, 0.99459E+00_JPRB, 0.10225E+01_JPRB, 0.10351E+01_JPRB, 0.97883E+00_JPRB, 0.89848E+00_JPRB ,&
&  0.85376E+00_JPRB, 0.85231E+00_JPRB, 0.82888E+00_JPRB, 0.79298E+00_JPRB, 0.72960E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.74845E+00_JPRB, 0.71921E+00_JPRB, 0.71499E+00_JPRB, 0.70656E+00_JPRB, 0.67454E+00_JPRB, 0.65440E+00_JPRB, 0.64439E+00_JPRB ,&
&  0.65271E+00_JPRB, 0.67255E+00_JPRB, 0.70033E+00_JPRB, 0.74169E+00_JPRB, 0.79069E+00_JPRB, 0.84153E+00_JPRB, 0.88354E+00_JPRB ,&
&  0.96664E+00_JPRB, 0.11521E+01_JPRB, 0.13627E+01_JPRB, 0.15844E+01_JPRB, 0.19818E+01_JPRB, 0.20925E+01_JPRB, 0.25562E+01_JPRB ,&
&  0.26043E+01_JPRB, 0.27594E+01_JPRB, 0.38306E+01_JPRB, 0.57791E+01_JPRB, 0.87178E+01_JPRB, 0.95475E+01_JPRB, 0.86514E+01_JPRB ,&
&  0.66707E+01_JPRB, 0.45090E+01_JPRB, 0.32172E+01_JPRB, 0.24180E+01_JPRB, 0.19279E+01_JPRB, 0.16616E+01_JPRB, 0.16366E+01_JPRB ,&
&  0.15662E+01_JPRB, 0.15611E+01_JPRB, 0.16111E+01_JPRB, 0.16572E+01_JPRB, 0.19582E+01_JPRB, 0.25168E+01_JPRB, 0.34155E+01_JPRB ,&
&  0.42569E+01_JPRB, 0.49075E+01_JPRB, 0.51146E+01_JPRB, 0.45359E+01_JPRB, 0.53247E+01_JPRB, 0.55718E+01_JPRB, 0.53696E+01_JPRB ,&
&  0.57974E+01_JPRB, 0.66384E+01_JPRB, 0.91172E+01_JPRB, 0.89913E+01_JPRB, 0.82022E+01_JPRB, 0.81696E+01_JPRB, 0.69683E+01_JPRB ,&
&  0.61662E+01_JPRB, 0.60015E+01_JPRB, 0.50342E+01_JPRB, 0.51086E+01_JPRB, 0.48481E+01_JPRB, 0.43595E+01_JPRB, 0.48711E+01_JPRB ,&
&  0.52039E+01_JPRB, 0.46875E+01_JPRB, 0.37911E+01_JPRB, 0.30840E+01_JPRB, 0.29502E+01_JPRB, 0.20333E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.80947E+00_JPRB, 0.57849E+00_JPRB, 0.71469E+00_JPRB, 0.65141E+00_JPRB, 0.92896E+00_JPRB, 0.18456E+01_JPRB ,&
&  0.31882E+01_JPRB, 0.39837E+01_JPRB, 0.46787E+01_JPRB, 0.38839E+01_JPRB, 0.33259E+01_JPRB, 0.29020E+01_JPRB, 0.42630E+01_JPRB ,&
&  0.25294E+01_JPRB, 0.20380E+01_JPRB, 0.15344E+01_JPRB, 0.12255E+01_JPRB, 0.11084E+01_JPRB, 0.10513E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.95353E+00_JPRB, 0.90819E+00_JPRB, 0.89103E+00_JPRB, 0.85861E+00_JPRB, 0.80183E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.91270E+00_JPRB, 0.87380E+00_JPRB, 0.84433E+00_JPRB, 0.81709E+00_JPRB, 0.78536E+00_JPRB, 0.74823E+00_JPRB, 0.70802E+00_JPRB ,&
&  0.68961E+00_JPRB, 0.69074E+00_JPRB, 0.70545E+00_JPRB, 0.73805E+00_JPRB, 0.76293E+00_JPRB, 0.79297E+00_JPRB, 0.82586E+00_JPRB ,&
&  0.93300E+00_JPRB, 0.11744E+01_JPRB, 0.11431E+01_JPRB, 0.12472E+01_JPRB, 0.15777E+01_JPRB, 0.20625E+01_JPRB, 0.23196E+01_JPRB ,&
&  0.26921E+01_JPRB, 0.27729E+01_JPRB, 0.35484E+01_JPRB, 0.53781E+01_JPRB, 0.10785E+02_JPRB, 0.12641E+02_JPRB, 0.11877E+02_JPRB ,&
&  0.11299E+02_JPRB, 0.82276E+01_JPRB, 0.52659E+01_JPRB, 0.36511E+01_JPRB, 0.26331E+01_JPRB, 0.19313E+01_JPRB, 0.16052E+01_JPRB ,&
&  0.14868E+01_JPRB, 0.14551E+01_JPRB, 0.14614E+01_JPRB, 0.15131E+01_JPRB, 0.16690E+01_JPRB, 0.20839E+01_JPRB, 0.29851E+01_JPRB ,&
&  0.39919E+01_JPRB, 0.50242E+01_JPRB, 0.57248E+01_JPRB, 0.56271E+01_JPRB, 0.48112E+01_JPRB, 0.54207E+01_JPRB, 0.79780E+01_JPRB ,&
&  0.79687E+01_JPRB, 0.88646E+01_JPRB, 0.92882E+01_JPRB, 0.96472E+01_JPRB, 0.88830E+01_JPRB, 0.79307E+01_JPRB, 0.72940E+01_JPRB ,&
&  0.51950E+01_JPRB, 0.45085E+01_JPRB, 0.48671E+01_JPRB, 0.50716E+01_JPRB, 0.46121E+01_JPRB, 0.53605E+01_JPRB, 0.71603E+01_JPRB ,&
&  0.54095E+01_JPRB, 0.54465E+01_JPRB, 0.54461E+01_JPRB, 0.53388E+01_JPRB, 0.29642E+01_JPRB, 0.18963E+01_JPRB, 0.28064E+01_JPRB ,&
&  0.27170E+01_JPRB, 0.14537E+01_JPRB, 0.10792E+01_JPRB, 0.15012E+01_JPRB, 0.10112E+01_JPRB, 0.12824E+01_JPRB, 0.21505E+01_JPRB ,&
&  0.25891E+01_JPRB, 0.30317E+01_JPRB, 0.49522E+01_JPRB, 0.41471E+01_JPRB, 0.36554E+01_JPRB, 0.28983E+01_JPRB, 0.33702E+01_JPRB ,&
&  0.33901E+01_JPRB, 0.26164E+01_JPRB, 0.21690E+01_JPRB, 0.16665E+01_JPRB, 0.13484E+01_JPRB, 0.12446E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.11154E+01_JPRB, 0.10677E+01_JPRB, 0.99738E+00_JPRB, 0.95561E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.10467E+01_JPRB, 0.10098E+01_JPRB, 0.97622E+00_JPRB, 0.94085E+00_JPRB, 0.90540E+00_JPRB, 0.85226E+00_JPRB, 0.79095E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.73505E+00_JPRB, 0.74808E+00_JPRB, 0.76399E+00_JPRB, 0.76592E+00_JPRB, 0.78025E+00_JPRB, 0.79760E+00_JPRB ,&
&  0.90083E+00_JPRB, 0.10565E+01_JPRB, 0.10660E+01_JPRB, 0.11491E+01_JPRB, 0.15146E+01_JPRB, 0.20796E+01_JPRB, 0.24542E+01_JPRB ,&
&  0.29543E+01_JPRB, 0.28581E+01_JPRB, 0.33357E+01_JPRB, 0.51258E+01_JPRB, 0.89110E+01_JPRB, 0.11551E+02_JPRB, 0.11340E+02_JPRB ,&
&  0.10956E+02_JPRB, 0.90558E+01_JPRB, 0.68750E+01_JPRB, 0.51608E+01_JPRB, 0.37491E+01_JPRB, 0.27334E+01_JPRB, 0.20186E+01_JPRB ,&
&  0.16417E+01_JPRB, 0.14708E+01_JPRB, 0.14273E+01_JPRB, 0.14279E+01_JPRB, 0.15258E+01_JPRB, 0.16958E+01_JPRB, 0.23514E+01_JPRB ,&
&  0.32336E+01_JPRB, 0.45437E+01_JPRB, 0.60702E+01_JPRB, 0.68224E+01_JPRB, 0.58305E+01_JPRB, 0.64754E+01_JPRB, 0.86969E+01_JPRB ,&
&  0.91407E+01_JPRB, 0.94344E+01_JPRB, 0.10019E+02_JPRB, 0.97311E+01_JPRB, 0.99813E+01_JPRB, 0.88476E+01_JPRB, 0.85467E+01_JPRB ,&
&  0.64104E+01_JPRB, 0.52977E+01_JPRB, 0.48251E+01_JPRB, 0.47624E+01_JPRB, 0.59059E+01_JPRB, 0.78200E+01_JPRB, 0.72920E+01_JPRB ,&
&  0.64479E+01_JPRB, 0.61078E+01_JPRB, 0.55823E+01_JPRB, 0.44908E+01_JPRB, 0.43570E+01_JPRB, 0.34562E+01_JPRB, 0.40967E+01_JPRB ,&
&  0.39688E+01_JPRB, 0.35831E+01_JPRB, 0.29399E+01_JPRB, 0.22267E+01_JPRB, 0.17319E+01_JPRB, 0.21270E+01_JPRB, 0.23146E+01_JPRB ,&
&  0.24355E+01_JPRB, 0.25807E+01_JPRB, 0.39552E+01_JPRB, 0.41294E+01_JPRB, 0.39097E+01_JPRB, 0.27826E+01_JPRB, 0.29390E+01_JPRB ,&
&  0.27899E+01_JPRB, 0.28739E+01_JPRB, 0.26490E+01_JPRB, 0.21667E+01_JPRB, 0.18261E+01_JPRB, 0.15863E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.14152E+01_JPRB, 0.13589E+01_JPRB, 0.12718E+01_JPRB, 0.11776E+01_JPRB, 0.11078E+01_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.10974E+01_JPRB, 0.10676E+01_JPRB, 0.10280E+01_JPRB, 0.98142E+00_JPRB, 0.93184E+00_JPRB, 0.88773E+00_JPRB, 0.84281E+00_JPRB ,&
&  0.80778E+00_JPRB, 0.78391E+00_JPRB, 0.78448E+00_JPRB, 0.79352E+00_JPRB, 0.80072E+00_JPRB, 0.78729E+00_JPRB, 0.78338E+00_JPRB ,&
&  0.84799E+00_JPRB, 0.10040E+01_JPRB, 0.10010E+01_JPRB, 0.11423E+01_JPRB, 0.14998E+01_JPRB, 0.20637E+01_JPRB, 0.28144E+01_JPRB ,&
&  0.31724E+01_JPRB, 0.32044E+01_JPRB, 0.35606E+01_JPRB, 0.49059E+01_JPRB, 0.67083E+01_JPRB, 0.78815E+01_JPRB, 0.81048E+01_JPRB ,&
&  0.69817E+01_JPRB, 0.61877E+01_JPRB, 0.53104E+01_JPRB, 0.45246E+01_JPRB, 0.39227E+01_JPRB, 0.33800E+01_JPRB, 0.27495E+01_JPRB ,&
&  0.21872E+01_JPRB, 0.18180E+01_JPRB, 0.16369E+01_JPRB, 0.15549E+01_JPRB, 0.15400E+01_JPRB, 0.15981E+01_JPRB, 0.18792E+01_JPRB ,&
&  0.24438E+01_JPRB, 0.35459E+01_JPRB, 0.56159E+01_JPRB, 0.78739E+01_JPRB, 0.76575E+01_JPRB, 0.90580E+01_JPRB, 0.92357E+01_JPRB ,&
&  0.10702E+02_JPRB, 0.11412E+02_JPRB, 0.10725E+02_JPRB, 0.10632E+02_JPRB, 0.87511E+01_JPRB, 0.93491E+01_JPRB, 0.97644E+01_JPRB ,&
&  0.94673E+01_JPRB, 0.89756E+01_JPRB, 0.89671E+01_JPRB, 0.77963E+01_JPRB, 0.72243E+01_JPRB, 0.86506E+01_JPRB, 0.74809E+01_JPRB ,&
&  0.64462E+01_JPRB, 0.60678E+01_JPRB, 0.54654E+01_JPRB, 0.53806E+01_JPRB, 0.44377E+01_JPRB, 0.43251E+01_JPRB, 0.36921E+01_JPRB ,&
&  0.34119E+01_JPRB, 0.27763E+01_JPRB, 0.30840E+01_JPRB, 0.26136E+01_JPRB, 0.24236E+01_JPRB, 0.23954E+01_JPRB, 0.23818E+01_JPRB ,&
&  0.23497E+01_JPRB, 0.22589E+01_JPRB, 0.24888E+01_JPRB, 0.31059E+01_JPRB, 0.38168E+01_JPRB, 0.29903E+01_JPRB, 0.28104E+01_JPRB ,&
&  0.26433E+01_JPRB, 0.26349E+01_JPRB, 0.23696E+01_JPRB, 0.23090E+01_JPRB, 0.21317E+01_JPRB, 0.19724E+01_JPRB, 0.17805E+01_JPRB ,&
&  0.15557E+01_JPRB, 0.13623E+01_JPRB, 0.12524E+01_JPRB, 0.11747E+01_JPRB, 0.11215E+01_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.10763E+01_JPRB, 0.10667E+01_JPRB, 0.10250E+01_JPRB, 0.96920E+00_JPRB, 0.92627E+00_JPRB, 0.88741E+00_JPRB, 0.85766E+00_JPRB ,&
&  0.82680E+00_JPRB, 0.80469E+00_JPRB, 0.79713E+00_JPRB, 0.80068E+00_JPRB, 0.80651E+00_JPRB, 0.80592E+00_JPRB, 0.79738E+00_JPRB ,&
&  0.80322E+00_JPRB, 0.98710E+00_JPRB, 0.97978E+00_JPRB, 0.10038E+01_JPRB, 0.13299E+01_JPRB, 0.21270E+01_JPRB, 0.28998E+01_JPRB ,&
&  0.32367E+01_JPRB, 0.33280E+01_JPRB, 0.36384E+01_JPRB, 0.41376E+01_JPRB, 0.47417E+01_JPRB, 0.51384E+01_JPRB, 0.52747E+01_JPRB ,&
&  0.51668E+01_JPRB, 0.39843E+01_JPRB, 0.36073E+01_JPRB, 0.32597E+01_JPRB, 0.30540E+01_JPRB, 0.28654E+01_JPRB, 0.26790E+01_JPRB ,&
&  0.24170E+01_JPRB, 0.20840E+01_JPRB, 0.18289E+01_JPRB, 0.16640E+01_JPRB, 0.15730E+01_JPRB, 0.14921E+01_JPRB, 0.15768E+01_JPRB ,&
&  0.17829E+01_JPRB, 0.21597E+01_JPRB, 0.34605E+01_JPRB, 0.55724E+01_JPRB, 0.86444E+01_JPRB, 0.10606E+02_JPRB, 0.10689E+02_JPRB ,&
&  0.91918E+01_JPRB, 0.93951E+01_JPRB, 0.12942E+02_JPRB, 0.11032E+02_JPRB, 0.10574E+02_JPRB, 0.10925E+02_JPRB, 0.10350E+02_JPRB ,&
&  0.96772E+01_JPRB, 0.89033E+01_JPRB, 0.83122E+01_JPRB, 0.78322E+01_JPRB, 0.87734E+01_JPRB, 0.83362E+01_JPRB, 0.72736E+01_JPRB ,&
&  0.62849E+01_JPRB, 0.60845E+01_JPRB, 0.57081E+01_JPRB, 0.52959E+01_JPRB, 0.49618E+01_JPRB, 0.51524E+01_JPRB, 0.47452E+01_JPRB ,&
&  0.43460E+01_JPRB, 0.47288E+01_JPRB, 0.37316E+01_JPRB, 0.28381E+01_JPRB, 0.23039E+01_JPRB, 0.23766E+01_JPRB, 0.25032E+01_JPRB ,&
&  0.23228E+01_JPRB, 0.20288E+01_JPRB, 0.17843E+01_JPRB, 0.19718E+01_JPRB, 0.25024E+01_JPRB, 0.27235E+01_JPRB, 0.24448E+01_JPRB ,&
&  0.22015E+01_JPRB, 0.22568E+01_JPRB, 0.22033E+01_JPRB, 0.21810E+01_JPRB, 0.22229E+01_JPRB, 0.20095E+01_JPRB, 0.17100E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.12742E+01_JPRB, 0.11847E+01_JPRB, 0.11299E+01_JPRB, 0.10888E+01_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.10771E+01_JPRB, 0.10586E+01_JPRB, 0.10220E+01_JPRB, 0.96840E+00_JPRB, 0.92416E+00_JPRB, 0.89177E+00_JPRB, 0.86082E+00_JPRB ,&
&  0.84428E+00_JPRB, 0.82866E+00_JPRB, 0.82689E+00_JPRB, 0.81968E+00_JPRB, 0.82264E+00_JPRB, 0.83168E+00_JPRB, 0.83662E+00_JPRB ,&
&  0.83510E+00_JPRB, 0.10020E+01_JPRB, 0.96759E+00_JPRB, 0.10219E+01_JPRB, 0.12294E+01_JPRB, 0.19134E+01_JPRB, 0.24602E+01_JPRB ,&
&  0.27509E+01_JPRB, 0.28172E+01_JPRB, 0.29263E+01_JPRB, 0.31955E+01_JPRB, 0.33404E+01_JPRB, 0.32491E+01_JPRB, 0.31877E+01_JPRB ,&
&  0.30568E+01_JPRB, 0.28386E+01_JPRB, 0.23857E+01_JPRB, 0.23547E+01_JPRB, 0.21827E+01_JPRB, 0.19030E+01_JPRB, 0.20507E+01_JPRB ,&
&  0.20605E+01_JPRB, 0.20069E+01_JPRB, 0.18889E+01_JPRB, 0.17786E+01_JPRB, 0.17002E+01_JPRB, 0.15984E+01_JPRB, 0.15540E+01_JPRB ,&
&  0.15752E+01_JPRB, 0.16512E+01_JPRB, 0.19930E+01_JPRB, 0.28773E+01_JPRB, 0.47883E+01_JPRB, 0.71478E+01_JPRB, 0.91545E+01_JPRB ,&
&  0.10951E+02_JPRB, 0.11554E+02_JPRB, 0.10117E+02_JPRB, 0.10593E+02_JPRB, 0.11724E+02_JPRB, 0.10392E+02_JPRB, 0.94418E+01_JPRB ,&
&  0.94880E+01_JPRB, 0.94292E+01_JPRB, 0.90256E+01_JPRB, 0.85778E+01_JPRB, 0.84554E+01_JPRB, 0.80955E+01_JPRB, 0.75027E+01_JPRB ,&
&  0.63927E+01_JPRB, 0.59620E+01_JPRB, 0.58662E+01_JPRB, 0.57375E+01_JPRB, 0.50496E+01_JPRB, 0.47170E+01_JPRB, 0.47297E+01_JPRB ,&
&  0.45798E+01_JPRB, 0.44139E+01_JPRB, 0.33786E+01_JPRB, 0.31820E+01_JPRB, 0.23630E+01_JPRB, 0.22755E+01_JPRB, 0.25481E+01_JPRB ,&
&  0.22271E+01_JPRB, 0.18643E+01_JPRB, 0.15119E+01_JPRB, 0.13704E+01_JPRB, 0.15382E+01_JPRB, 0.17170E+01_JPRB, 0.19498E+01_JPRB ,&
&  0.18658E+01_JPRB, 0.15567E+01_JPRB, 0.17177E+01_JPRB, 0.17215E+01_JPRB, 0.17870E+01_JPRB, 0.17199E+01_JPRB, 0.15766E+01_JPRB ,&
&  0.14581E+01_JPRB, 0.13660E+01_JPRB, 0.12724E+01_JPRB, 0.11808E+01_JPRB, 0.11166E+01_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.10338E+01_JPRB, 0.98231E+00_JPRB, 0.95307E+00_JPRB, 0.91869E+00_JPRB, 0.88604E+00_JPRB, 0.85841E+00_JPRB, 0.84081E+00_JPRB ,&
&  0.85105E+00_JPRB, 0.85762E+00_JPRB, 0.85834E+00_JPRB, 0.84285E+00_JPRB, 0.83084E+00_JPRB, 0.82577E+00_JPRB, 0.84971E+00_JPRB ,&
&  0.86999E+00_JPRB, 0.96932E+00_JPRB, 0.10230E+01_JPRB, 0.10709E+01_JPRB, 0.14497E+01_JPRB, 0.16701E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.19576E+01_JPRB, 0.20292E+01_JPRB, 0.20370E+01_JPRB, 0.20365E+01_JPRB, 0.20046E+01_JPRB, 0.19576E+01_JPRB, 0.18515E+01_JPRB ,&
&  0.16952E+01_JPRB, 0.15130E+01_JPRB, 0.14258E+01_JPRB, 0.14093E+01_JPRB, 0.14556E+01_JPRB, 0.13121E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.15696E+01_JPRB, 0.16080E+01_JPRB, 0.16200E+01_JPRB, 0.16058E+01_JPRB, 0.15978E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.16436E+01_JPRB, 0.16428E+01_JPRB, 0.17000E+01_JPRB, 0.21071E+01_JPRB, 0.31631E+01_JPRB, 0.53877E+01_JPRB, 0.76626E+01_JPRB ,&
&  0.80119E+01_JPRB, 0.96233E+01_JPRB, 0.11584E+02_JPRB, 0.11976E+02_JPRB, 0.10017E+02_JPRB, 0.85628E+01_JPRB, 0.81635E+01_JPRB ,&
&  0.84823E+01_JPRB, 0.89278E+01_JPRB, 0.90100E+01_JPRB, 0.83153E+01_JPRB, 0.81113E+01_JPRB, 0.79183E+01_JPRB, 0.75856E+01_JPRB ,&
&  0.70616E+01_JPRB, 0.64728E+01_JPRB, 0.60904E+01_JPRB, 0.57949E+01_JPRB, 0.55818E+01_JPRB, 0.50744E+01_JPRB, 0.48709E+01_JPRB ,&
&  0.49442E+01_JPRB, 0.39873E+01_JPRB, 0.34747E+01_JPRB, 0.36425E+01_JPRB, 0.28555E+01_JPRB, 0.27815E+01_JPRB, 0.30279E+01_JPRB ,&
&  0.24106E+01_JPRB, 0.18285E+01_JPRB, 0.15754E+01_JPRB, 0.13544E+01_JPRB, 0.12808E+01_JPRB, 0.13575E+01_JPRB, 0.13730E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.12775E+01_JPRB, 0.14205E+01_JPRB, 0.14738E+01_JPRB, 0.14995E+01_JPRB, 0.14585E+01_JPRB, 0.13823E+01_JPRB ,&
&  0.12964E+01_JPRB, 0.12362E+01_JPRB, 0.11941E+01_JPRB, 0.11503E+01_JPRB, 0.11014E+01_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.97262E+00_JPRB, 0.93364E+00_JPRB, 0.90351E+00_JPRB, 0.88437E+00_JPRB, 0.86607E+00_JPRB, 0.84605E+00_JPRB, 0.82995E+00_JPRB ,&
&  0.83148E+00_JPRB, 0.84580E+00_JPRB, 0.85891E+00_JPRB, 0.84586E+00_JPRB, 0.82900E+00_JPRB, 0.82149E+00_JPRB, 0.85561E+00_JPRB ,&
&  0.74565E+00_JPRB, 0.79639E+00_JPRB, 0.85012E+00_JPRB, 0.10305E+01_JPRB, 0.14355E+01_JPRB, 0.13705E+01_JPRB, 0.14107E+01_JPRB ,&
&  0.13800E+01_JPRB, 0.13374E+01_JPRB, 0.13103E+01_JPRB, 0.13063E+01_JPRB, 0.12702E+01_JPRB, 0.12442E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.93630E+00_JPRB, 0.90264E+00_JPRB, 0.88464E+00_JPRB, 0.93084E+00_JPRB, 0.99127E+00_JPRB, 0.10856E+01_JPRB ,&
&  0.11594E+01_JPRB, 0.12320E+01_JPRB, 0.12940E+01_JPRB, 0.13429E+01_JPRB, 0.13706E+01_JPRB, 0.13933E+01_JPRB, 0.14211E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.14972E+01_JPRB, 0.15623E+01_JPRB, 0.18930E+01_JPRB, 0.25989E+01_JPRB, 0.48072E+01_JPRB, 0.67880E+01_JPRB ,&
&  0.58308E+01_JPRB, 0.56397E+01_JPRB, 0.73974E+01_JPRB, 0.89595E+01_JPRB, 0.77727E+01_JPRB, 0.65978E+01_JPRB, 0.60180E+01_JPRB ,&
&  0.62103E+01_JPRB, 0.66791E+01_JPRB, 0.71904E+01_JPRB, 0.72005E+01_JPRB, 0.68570E+01_JPRB, 0.68199E+01_JPRB, 0.67111E+01_JPRB ,&
&  0.64936E+01_JPRB, 0.62884E+01_JPRB, 0.60036E+01_JPRB, 0.57552E+01_JPRB, 0.55552E+01_JPRB, 0.56489E+01_JPRB, 0.55551E+01_JPRB ,&
&  0.53700E+01_JPRB, 0.50116E+01_JPRB, 0.44869E+01_JPRB, 0.41812E+01_JPRB, 0.33305E+01_JPRB, 0.32115E+01_JPRB, 0.37311E+01_JPRB ,&
&  0.28147E+01_JPRB, 0.20899E+01_JPRB, 0.17352E+01_JPRB, 0.14915E+01_JPRB, 0.13730E+01_JPRB, 0.12551E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10742E+01_JPRB, 0.11739E+01_JPRB, 0.12055E+01_JPRB, 0.12148E+01_JPRB, 0.12136E+01_JPRB, 0.11365E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.10597E+01_JPRB, 0.10383E+01_JPRB, 0.10245E+01_JPRB, 0.10038E+01_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.89976E+00_JPRB, 0.87983E+00_JPRB, 0.85911E+00_JPRB, 0.83299E+00_JPRB, 0.81269E+00_JPRB, 0.80620E+00_JPRB, 0.80968E+00_JPRB ,&
&  0.81965E+00_JPRB, 0.83592E+00_JPRB, 0.84278E+00_JPRB, 0.83573E+00_JPRB, 0.82797E+00_JPRB, 0.83700E+00_JPRB, 0.76773E+00_JPRB ,&
&  0.67413E+00_JPRB, 0.69465E+00_JPRB, 0.70575E+00_JPRB, 0.78564E+00_JPRB, 0.87689E+00_JPRB, 0.89471E+00_JPRB, 0.91212E+00_JPRB ,&
&  0.93947E+00_JPRB, 0.91043E+00_JPRB, 0.90923E+00_JPRB, 0.90841E+00_JPRB, 0.89503E+00_JPRB, 0.88084E+00_JPRB, 0.86180E+00_JPRB ,&
&  0.77970E+00_JPRB, 0.72780E+00_JPRB, 0.70050E+00_JPRB, 0.68623E+00_JPRB, 0.72331E+00_JPRB, 0.78899E+00_JPRB, 0.86118E+00_JPRB ,&
&  0.93375E+00_JPRB, 0.99361E+00_JPRB, 0.10531E+01_JPRB, 0.11017E+01_JPRB, 0.11319E+01_JPRB, 0.11574E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.12283E+01_JPRB, 0.12745E+01_JPRB, 0.13125E+01_JPRB, 0.15415E+01_JPRB, 0.18751E+01_JPRB, 0.30106E+01_JPRB, 0.39988E+01_JPRB ,&
&  0.36025E+01_JPRB, 0.34930E+01_JPRB, 0.37942E+01_JPRB, 0.42204E+01_JPRB, 0.43335E+01_JPRB, 0.41374E+01_JPRB, 0.41106E+01_JPRB ,&
&  0.43614E+01_JPRB, 0.46960E+01_JPRB, 0.54800E+01_JPRB, 0.54704E+01_JPRB, 0.54559E+01_JPRB, 0.55632E+01_JPRB, 0.54861E+01_JPRB ,&
&  0.55832E+01_JPRB, 0.55963E+01_JPRB, 0.56931E+01_JPRB, 0.55523E+01_JPRB, 0.53857E+01_JPRB, 0.53964E+01_JPRB, 0.53405E+01_JPRB ,&
&  0.53669E+01_JPRB, 0.53136E+01_JPRB, 0.49181E+01_JPRB, 0.45884E+01_JPRB, 0.40510E+01_JPRB, 0.40230E+01_JPRB, 0.36246E+01_JPRB ,&
&  0.29010E+01_JPRB, 0.20787E+01_JPRB, 0.18805E+01_JPRB, 0.16128E+01_JPRB, 0.14199E+01_JPRB, 0.12070E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.98380E+00_JPRB, 0.10488E+01_JPRB, 0.10675E+01_JPRB, 0.10808E+01_JPRB, 0.10956E+01_JPRB, 0.10803E+01_JPRB, 0.97464E+00_JPRB ,&
&  0.95136E+00_JPRB, 0.98576E+00_JPRB, 0.96832E+00_JPRB, 0.94171E+00_JPRB, 0.91745E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.83937E+00_JPRB, 0.83730E+00_JPRB, 0.83014E+00_JPRB, 0.81242E+00_JPRB, 0.77784E+00_JPRB, 0.78225E+00_JPRB, 0.73552E+00_JPRB ,&
&  0.80706E+00_JPRB, 0.82640E+00_JPRB, 0.81990E+00_JPRB, 0.83234E+00_JPRB, 0.86173E+00_JPRB, 0.79414E+00_JPRB, 0.64537E+00_JPRB ,&
&  0.58487E+00_JPRB, 0.58863E+00_JPRB, 0.61409E+00_JPRB, 0.63724E+00_JPRB, 0.65637E+00_JPRB, 0.67026E+00_JPRB, 0.69207E+00_JPRB ,&
&  0.70684E+00_JPRB, 0.70559E+00_JPRB, 0.71440E+00_JPRB, 0.71425E+00_JPRB, 0.70700E+00_JPRB, 0.70224E+00_JPRB, 0.70472E+00_JPRB ,&
&  0.66298E+00_JPRB, 0.63829E+00_JPRB, 0.62439E+00_JPRB, 0.59757E+00_JPRB, 0.63557E+00_JPRB, 0.66818E+00_JPRB, 0.71528E+00_JPRB ,&
&  0.77677E+00_JPRB, 0.84064E+00_JPRB, 0.88732E+00_JPRB, 0.93088E+00_JPRB, 0.96410E+00_JPRB, 0.99214E+00_JPRB, 0.10193E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10893E+01_JPRB, 0.11295E+01_JPRB, 0.12680E+01_JPRB, 0.15116E+01_JPRB, 0.18765E+01_JPRB, 0.23915E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.27720E+01_JPRB, 0.29897E+01_JPRB, 0.26959E+01_JPRB, 0.29383E+01_JPRB, 0.33156E+01_JPRB, 0.33177E+01_JPRB ,&
&  0.33761E+01_JPRB, 0.32610E+01_JPRB, 0.33256E+01_JPRB, 0.34652E+01_JPRB, 0.38732E+01_JPRB, 0.41758E+01_JPRB, 0.43173E+01_JPRB ,&
&  0.44077E+01_JPRB, 0.47581E+01_JPRB, 0.49715E+01_JPRB, 0.50470E+01_JPRB, 0.50209E+01_JPRB, 0.50402E+01_JPRB, 0.50995E+01_JPRB ,&
&  0.51412E+01_JPRB, 0.51674E+01_JPRB, 0.50799E+01_JPRB, 0.48271E+01_JPRB, 0.44637E+01_JPRB, 0.40322E+01_JPRB, 0.34933E+01_JPRB ,&
&  0.30250E+01_JPRB, 0.25188E+01_JPRB, 0.21052E+01_JPRB, 0.18397E+01_JPRB, 0.15799E+01_JPRB, 0.13176E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.98083E+00_JPRB, 0.95506E+00_JPRB, 0.10325E+01_JPRB, 0.10440E+01_JPRB, 0.10524E+01_JPRB, 0.10290E+01_JPRB, 0.94554E+00_JPRB ,&
&  0.93202E+00_JPRB, 0.95310E+00_JPRB, 0.94142E+00_JPRB, 0.89996E+00_JPRB, 0.86132E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.84161E+00_JPRB, 0.81469E+00_JPRB, 0.80339E+00_JPRB, 0.80655E+00_JPRB, 0.78193E+00_JPRB, 0.72958E+00_JPRB, 0.67846E+00_JPRB ,&
&  0.78780E+00_JPRB, 0.76051E+00_JPRB, 0.80972E+00_JPRB, 0.65624E+00_JPRB, 0.60030E+00_JPRB, 0.58116E+00_JPRB, 0.55295E+00_JPRB ,&
&  0.53586E+00_JPRB, 0.53686E+00_JPRB, 0.54833E+00_JPRB, 0.57259E+00_JPRB, 0.59593E+00_JPRB, 0.60445E+00_JPRB, 0.61064E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.61790E+00_JPRB, 0.63241E+00_JPRB, 0.63774E+00_JPRB, 0.63815E+00_JPRB, 0.63757E+00_JPRB, 0.64155E+00_JPRB ,&
&  0.61312E+00_JPRB, 0.60256E+00_JPRB, 0.60541E+00_JPRB, 0.60284E+00_JPRB, 0.61349E+00_JPRB, 0.62554E+00_JPRB, 0.65050E+00_JPRB ,&
&  0.70378E+00_JPRB, 0.67256E+00_JPRB, 0.77907E+00_JPRB, 0.80788E+00_JPRB, 0.83833E+00_JPRB, 0.86392E+00_JPRB, 0.88547E+00_JPRB ,&
&  0.90746E+00_JPRB, 0.94269E+00_JPRB, 0.98250E+00_JPRB, 0.10490E+01_JPRB, 0.12451E+01_JPRB, 0.14111E+01_JPRB, 0.15533E+01_JPRB ,&
&  0.18714E+01_JPRB, 0.16639E+01_JPRB, 0.19144E+01_JPRB, 0.20465E+01_JPRB, 0.24557E+01_JPRB, 0.28302E+01_JPRB, 0.28542E+01_JPRB ,&
&  0.32637E+01_JPRB, 0.30144E+01_JPRB, 0.27761E+01_JPRB, 0.25799E+01_JPRB, 0.26510E+01_JPRB, 0.28515E+01_JPRB, 0.30883E+01_JPRB ,&
&  0.32842E+01_JPRB, 0.34038E+01_JPRB, 0.36589E+01_JPRB, 0.38268E+01_JPRB, 0.40336E+01_JPRB, 0.43010E+01_JPRB, 0.45522E+01_JPRB ,&
&  0.47608E+01_JPRB, 0.48699E+01_JPRB, 0.49039E+01_JPRB, 0.45368E+01_JPRB, 0.42287E+01_JPRB, 0.39444E+01_JPRB, 0.33932E+01_JPRB ,&
&  0.29655E+01_JPRB, 0.26622E+01_JPRB, 0.24046E+01_JPRB, 0.22227E+01_JPRB, 0.18839E+01_JPRB, 0.15920E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.89389E+00_JPRB, 0.90494E+00_JPRB, 0.92850E+00_JPRB, 0.93691E+00_JPRB, 0.94808E+00_JPRB, 0.96684E+00_JPRB ,&
&  0.94600E+00_JPRB, 0.93197E+00_JPRB, 0.95847E+00_JPRB, 0.92453E+00_JPRB, 0.88222E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.93544E+00_JPRB, 0.88537E+00_JPRB, 0.84050E+00_JPRB, 0.81812E+00_JPRB, 0.79641E+00_JPRB, 0.73959E+00_JPRB, 0.65813E+00_JPRB ,&
&  0.66153E+00_JPRB, 0.61758E+00_JPRB, 0.59585E+00_JPRB, 0.57520E+00_JPRB, 0.55557E+00_JPRB, 0.52205E+00_JPRB, 0.49746E+00_JPRB ,&
&  0.50622E+00_JPRB, 0.53808E+00_JPRB, 0.57854E+00_JPRB, 0.60941E+00_JPRB, 0.62276E+00_JPRB, 0.62144E+00_JPRB, 0.61436E+00_JPRB ,&
&  0.61011E+00_JPRB, 0.61116E+00_JPRB, 0.61560E+00_JPRB, 0.62204E+00_JPRB, 0.61558E+00_JPRB, 0.60652E+00_JPRB, 0.60138E+00_JPRB ,&
&  0.59382E+00_JPRB, 0.58481E+00_JPRB, 0.57397E+00_JPRB, 0.58195E+00_JPRB, 0.59099E+00_JPRB, 0.61085E+00_JPRB, 0.62891E+00_JPRB ,&
&  0.55397E+00_JPRB, 0.56316E+00_JPRB, 0.66851E+00_JPRB, 0.69762E+00_JPRB, 0.72935E+00_JPRB, 0.75812E+00_JPRB, 0.78178E+00_JPRB ,&
&  0.80638E+00_JPRB, 0.84270E+00_JPRB, 0.88577E+00_JPRB, 0.93809E+00_JPRB, 0.10868E+01_JPRB, 0.12329E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.14713E+01_JPRB, 0.14334E+01_JPRB, 0.16089E+01_JPRB, 0.22358E+01_JPRB, 0.23703E+01_JPRB, 0.23460E+01_JPRB ,&
&  0.23500E+01_JPRB, 0.22409E+01_JPRB, 0.21433E+01_JPRB, 0.21497E+01_JPRB, 0.21497E+01_JPRB, 0.22586E+01_JPRB, 0.24483E+01_JPRB ,&
&  0.24619E+01_JPRB, 0.26436E+01_JPRB, 0.28568E+01_JPRB, 0.30256E+01_JPRB, 0.32186E+01_JPRB, 0.35236E+01_JPRB, 0.38879E+01_JPRB ,&
&  0.42590E+01_JPRB, 0.45572E+01_JPRB, 0.46362E+01_JPRB, 0.44856E+01_JPRB, 0.42068E+01_JPRB, 0.39025E+01_JPRB, 0.35234E+01_JPRB ,&
&  0.30250E+01_JPRB, 0.26789E+01_JPRB, 0.24518E+01_JPRB, 0.22402E+01_JPRB, 0.20418E+01_JPRB, 0.17707E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.88089E+00_JPRB, 0.84550E+00_JPRB, 0.84277E+00_JPRB, 0.85907E+00_JPRB, 0.88884E+00_JPRB, 0.90592E+00_JPRB ,&
&  0.91642E+00_JPRB, 0.89176E+00_JPRB, 0.87516E+00_JPRB, 0.90454E+00_JPRB, 0.93809E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.90527E+00_JPRB, 0.88413E+00_JPRB, 0.82983E+00_JPRB, 0.82023E+00_JPRB, 0.77811E+00_JPRB, 0.73672E+00_JPRB, 0.68232E+00_JPRB ,&
&  0.64490E+00_JPRB, 0.60370E+00_JPRB, 0.57444E+00_JPRB, 0.53949E+00_JPRB, 0.52288E+00_JPRB, 0.52189E+00_JPRB, 0.55021E+00_JPRB ,&
&  0.58818E+00_JPRB, 0.61874E+00_JPRB, 0.63831E+00_JPRB, 0.64443E+00_JPRB, 0.64192E+00_JPRB, 0.62899E+00_JPRB, 0.61494E+00_JPRB ,&
&  0.60459E+00_JPRB, 0.60460E+00_JPRB, 0.59766E+00_JPRB, 0.59089E+00_JPRB, 0.58075E+00_JPRB, 0.56624E+00_JPRB, 0.55724E+00_JPRB ,&
&  0.54174E+00_JPRB, 0.53312E+00_JPRB, 0.53845E+00_JPRB, 0.55955E+00_JPRB, 0.58318E+00_JPRB, 0.59427E+00_JPRB, 0.55250E+00_JPRB ,&
&  0.50962E+00_JPRB, 0.49870E+00_JPRB, 0.51209E+00_JPRB, 0.62486E+00_JPRB, 0.66939E+00_JPRB, 0.69230E+00_JPRB, 0.72882E+00_JPRB ,&
&  0.71497E+00_JPRB, 0.71638E+00_JPRB, 0.74879E+00_JPRB, 0.84941E+00_JPRB, 0.92098E+00_JPRB, 0.10587E+01_JPRB, 0.11660E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.14903E+01_JPRB, 0.16301E+01_JPRB, 0.14655E+01_JPRB, 0.15639E+01_JPRB, 0.19242E+01_JPRB, 0.17477E+01_JPRB ,&
&  0.16805E+01_JPRB, 0.16207E+01_JPRB, 0.16817E+01_JPRB, 0.17200E+01_JPRB, 0.16858E+01_JPRB, 0.17184E+01_JPRB, 0.17614E+01_JPRB ,&
&  0.17301E+01_JPRB, 0.18361E+01_JPRB, 0.20659E+01_JPRB, 0.22859E+01_JPRB, 0.24978E+01_JPRB, 0.27757E+01_JPRB, 0.30662E+01_JPRB ,&
&  0.34250E+01_JPRB, 0.37249E+01_JPRB, 0.39106E+01_JPRB, 0.38195E+01_JPRB, 0.36835E+01_JPRB, 0.34608E+01_JPRB, 0.31444E+01_JPRB ,&
&  0.27193E+01_JPRB, 0.24146E+01_JPRB, 0.22114E+01_JPRB, 0.20598E+01_JPRB, 0.18712E+01_JPRB, 0.16150E+01_JPRB, 0.12968E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.92558E+00_JPRB, 0.87030E+00_JPRB, 0.85776E+00_JPRB, 0.86546E+00_JPRB, 0.87191E+00_JPRB, 0.85800E+00_JPRB ,&
&  0.83574E+00_JPRB, 0.81880E+00_JPRB, 0.80748E+00_JPRB, 0.81895E+00_JPRB, 0.88430E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.85604E+00_JPRB, 0.87264E+00_JPRB, 0.88303E+00_JPRB, 0.84802E+00_JPRB, 0.80051E+00_JPRB, 0.72711E+00_JPRB, 0.67219E+00_JPRB ,&
&  0.63306E+00_JPRB, 0.62277E+00_JPRB, 0.62142E+00_JPRB, 0.62882E+00_JPRB, 0.64566E+00_JPRB, 0.67176E+00_JPRB, 0.69180E+00_JPRB ,&
&  0.70915E+00_JPRB, 0.71198E+00_JPRB, 0.71188E+00_JPRB, 0.70012E+00_JPRB, 0.67668E+00_JPRB, 0.64692E+00_JPRB, 0.62468E+00_JPRB ,&
&  0.61232E+00_JPRB, 0.60497E+00_JPRB, 0.59301E+00_JPRB, 0.57341E+00_JPRB, 0.55891E+00_JPRB, 0.54003E+00_JPRB, 0.54078E+00_JPRB ,&
&  0.53010E+00_JPRB, 0.52932E+00_JPRB, 0.54525E+00_JPRB, 0.55786E+00_JPRB, 0.57172E+00_JPRB, 0.57817E+00_JPRB, 0.53030E+00_JPRB ,&
&  0.48199E+00_JPRB, 0.45813E+00_JPRB, 0.44847E+00_JPRB, 0.45228E+00_JPRB, 0.50310E+00_JPRB, 0.62657E+00_JPRB, 0.68304E+00_JPRB ,&
&  0.71426E+00_JPRB, 0.73447E+00_JPRB, 0.75858E+00_JPRB, 0.78769E+00_JPRB, 0.80775E+00_JPRB, 0.88223E+00_JPRB, 0.10026E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.12899E+01_JPRB, 0.14022E+01_JPRB, 0.12178E+01_JPRB, 0.11197E+01_JPRB, 0.11729E+01_JPRB, 0.12705E+01_JPRB ,&
&  0.13907E+01_JPRB, 0.13523E+01_JPRB, 0.13256E+01_JPRB, 0.13907E+01_JPRB, 0.14019E+01_JPRB, 0.14093E+01_JPRB, 0.14129E+01_JPRB ,&
&  0.14262E+01_JPRB, 0.15084E+01_JPRB, 0.16997E+01_JPRB, 0.18020E+01_JPRB, 0.19697E+01_JPRB, 0.22132E+01_JPRB, 0.24220E+01_JPRB ,&
&  0.26741E+01_JPRB, 0.28566E+01_JPRB, 0.29349E+01_JPRB, 0.27839E+01_JPRB, 0.26812E+01_JPRB, 0.26618E+01_JPRB, 0.25829E+01_JPRB ,&
&  0.23327E+01_JPRB, 0.21427E+01_JPRB, 0.19828E+01_JPRB, 0.18433E+01_JPRB, 0.16509E+01_JPRB, 0.14842E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.10596E+01_JPRB, 0.10090E+01_JPRB, 0.97338E+00_JPRB, 0.95691E+00_JPRB, 0.94398E+00_JPRB, 0.91755E+00_JPRB ,&
&  0.88349E+00_JPRB, 0.85070E+00_JPRB, 0.83220E+00_JPRB, 0.83222E+00_JPRB, 0.85491E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.91434E+00_JPRB, 0.90883E+00_JPRB, 0.89783E+00_JPRB, 0.88138E+00_JPRB, 0.84873E+00_JPRB, 0.80514E+00_JPRB, 0.77725E+00_JPRB ,&
&  0.76191E+00_JPRB, 0.76406E+00_JPRB, 0.76486E+00_JPRB, 0.76766E+00_JPRB, 0.77626E+00_JPRB, 0.79001E+00_JPRB, 0.80159E+00_JPRB ,&
&  0.80315E+00_JPRB, 0.78768E+00_JPRB, 0.75870E+00_JPRB, 0.71226E+00_JPRB, 0.67403E+00_JPRB, 0.64627E+00_JPRB, 0.63045E+00_JPRB ,&
&  0.62108E+00_JPRB, 0.60448E+00_JPRB, 0.58133E+00_JPRB, 0.54940E+00_JPRB, 0.52609E+00_JPRB, 0.50822E+00_JPRB, 0.51157E+00_JPRB ,&
&  0.52853E+00_JPRB, 0.53609E+00_JPRB, 0.53750E+00_JPRB, 0.54858E+00_JPRB, 0.54823E+00_JPRB, 0.55270E+00_JPRB, 0.49358E+00_JPRB ,&
&  0.45107E+00_JPRB, 0.43011E+00_JPRB, 0.41532E+00_JPRB, 0.41205E+00_JPRB, 0.42482E+00_JPRB, 0.45364E+00_JPRB, 0.51559E+00_JPRB ,&
&  0.64752E+00_JPRB, 0.68794E+00_JPRB, 0.70266E+00_JPRB, 0.72322E+00_JPRB, 0.74280E+00_JPRB, 0.76767E+00_JPRB, 0.82876E+00_JPRB ,&
&  0.91120E+00_JPRB, 0.10139E+01_JPRB, 0.11031E+01_JPRB, 0.11274E+01_JPRB, 0.11361E+01_JPRB, 0.11301E+01_JPRB, 0.12612E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.14885E+01_JPRB, 0.12916E+01_JPRB, 0.11893E+01_JPRB, 0.11365E+01_JPRB, 0.11146E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.12656E+01_JPRB, 0.13511E+01_JPRB, 0.14659E+01_JPRB, 0.15541E+01_JPRB, 0.16571E+01_JPRB, 0.17999E+01_JPRB, 0.20043E+01_JPRB ,&
&  0.22146E+01_JPRB, 0.24767E+01_JPRB, 0.29627E+01_JPRB, 0.26254E+01_JPRB, 0.24305E+01_JPRB, 0.23486E+01_JPRB, 0.23015E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.21585E+01_JPRB, 0.19625E+01_JPRB, 0.17160E+01_JPRB, 0.15844E+01_JPRB, 0.14400E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.11512E+01_JPRB, 0.11025E+01_JPRB, 0.10633E+01_JPRB, 0.10441E+01_JPRB, 0.10241E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.97880E+00_JPRB, 0.95680E+00_JPRB, 0.94010E+00_JPRB, 0.92645E+00_JPRB, 0.91753E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.96449E+00_JPRB, 0.94649E+00_JPRB, 0.92611E+00_JPRB, 0.90956E+00_JPRB, 0.89075E+00_JPRB, 0.87748E+00_JPRB, 0.86713E+00_JPRB ,&
&  0.86137E+00_JPRB, 0.86125E+00_JPRB, 0.85921E+00_JPRB, 0.85685E+00_JPRB, 0.84885E+00_JPRB, 0.83698E+00_JPRB, 0.81933E+00_JPRB ,&
&  0.79372E+00_JPRB, 0.73446E+00_JPRB, 0.68967E+00_JPRB, 0.66490E+00_JPRB, 0.64272E+00_JPRB, 0.62695E+00_JPRB, 0.61154E+00_JPRB ,&
&  0.59000E+00_JPRB, 0.57002E+00_JPRB, 0.54531E+00_JPRB, 0.51451E+00_JPRB, 0.49908E+00_JPRB, 0.50437E+00_JPRB, 0.51241E+00_JPRB ,&
&  0.52474E+00_JPRB, 0.53685E+00_JPRB, 0.53048E+00_JPRB, 0.52065E+00_JPRB, 0.52294E+00_JPRB, 0.51620E+00_JPRB, 0.46379E+00_JPRB ,&
&  0.42998E+00_JPRB, 0.40516E+00_JPRB, 0.40002E+00_JPRB, 0.39852E+00_JPRB, 0.40617E+00_JPRB, 0.42704E+00_JPRB, 0.47938E+00_JPRB ,&
&  0.56944E+00_JPRB, 0.68165E+00_JPRB, 0.68846E+00_JPRB, 0.69756E+00_JPRB, 0.71334E+00_JPRB, 0.73221E+00_JPRB, 0.75907E+00_JPRB ,&
&  0.79535E+00_JPRB, 0.83635E+00_JPRB, 0.87685E+00_JPRB, 0.91995E+00_JPRB, 0.96016E+00_JPRB, 0.97788E+00_JPRB, 0.99094E+00_JPRB ,&
&  0.10251E+01_JPRB, 0.10130E+01_JPRB, 0.10210E+01_JPRB, 0.10337E+01_JPRB, 0.10517E+01_JPRB, 0.10409E+01_JPRB, 0.10782E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.12683E+01_JPRB, 0.13645E+01_JPRB, 0.14344E+01_JPRB, 0.15144E+01_JPRB, 0.16302E+01_JPRB, 0.17574E+01_JPRB ,&
&  0.18711E+01_JPRB, 0.20258E+01_JPRB, 0.22406E+01_JPRB, 0.22137E+01_JPRB, 0.21651E+01_JPRB, 0.21305E+01_JPRB, 0.21194E+01_JPRB ,&
&  0.20969E+01_JPRB, 0.21011E+01_JPRB, 0.19745E+01_JPRB, 0.17950E+01_JPRB, 0.16306E+01_JPRB, 0.14549E+01_JPRB, 0.13527E+01_JPRB ,&
&  0.12808E+01_JPRB, 0.11776E+01_JPRB, 0.11229E+01_JPRB, 0.10884E+01_JPRB, 0.10632E+01_JPRB, 0.10376E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10063E+01_JPRB, 0.10018E+01_JPRB, 0.99957E+00_JPRB, 0.99108E+00_JPRB, 0.97942E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.96387E+00_JPRB, 0.94484E+00_JPRB, 0.92696E+00_JPRB, 0.91037E+00_JPRB, 0.89463E+00_JPRB, 0.87890E+00_JPRB, 0.86442E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.83978E+00_JPRB, 0.82673E+00_JPRB, 0.80998E+00_JPRB, 0.79155E+00_JPRB, 0.77005E+00_JPRB, 0.75067E+00_JPRB ,&
&  0.72951E+00_JPRB, 0.70702E+00_JPRB, 0.67691E+00_JPRB, 0.64914E+00_JPRB, 0.62831E+00_JPRB, 0.61250E+00_JPRB, 0.59467E+00_JPRB ,&
&  0.58065E+00_JPRB, 0.56047E+00_JPRB, 0.53084E+00_JPRB, 0.50848E+00_JPRB, 0.48477E+00_JPRB, 0.47965E+00_JPRB, 0.51156E+00_JPRB ,&
&  0.52519E+00_JPRB, 0.52467E+00_JPRB, 0.51943E+00_JPRB, 0.50724E+00_JPRB, 0.49514E+00_JPRB, 0.43805E+00_JPRB, 0.44039E+00_JPRB ,&
&  0.42007E+00_JPRB, 0.40382E+00_JPRB, 0.40136E+00_JPRB, 0.40246E+00_JPRB, 0.40919E+00_JPRB, 0.42754E+00_JPRB, 0.45050E+00_JPRB ,&
&  0.49337E+00_JPRB, 0.62680E+00_JPRB, 0.65607E+00_JPRB, 0.68252E+00_JPRB, 0.69382E+00_JPRB, 0.70507E+00_JPRB, 0.71867E+00_JPRB ,&
&  0.73661E+00_JPRB, 0.75764E+00_JPRB, 0.77538E+00_JPRB, 0.79805E+00_JPRB, 0.82155E+00_JPRB, 0.83899E+00_JPRB, 0.87166E+00_JPRB ,&
&  0.89570E+00_JPRB, 0.91743E+00_JPRB, 0.93875E+00_JPRB, 0.95929E+00_JPRB, 0.99151E+00_JPRB, 0.99567E+00_JPRB, 0.10043E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.11865E+01_JPRB, 0.12803E+01_JPRB, 0.13661E+01_JPRB, 0.14350E+01_JPRB, 0.15126E+01_JPRB, 0.16111E+01_JPRB ,&
&  0.16793E+01_JPRB, 0.17849E+01_JPRB, 0.19035E+01_JPRB, 0.19365E+01_JPRB, 0.19245E+01_JPRB, 0.19035E+01_JPRB, 0.18955E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.18862E+01_JPRB, 0.18191E+01_JPRB, 0.17706E+01_JPRB, 0.16779E+01_JPRB, 0.15672E+01_JPRB, 0.14777E+01_JPRB ,&
&  0.13739E+01_JPRB, 0.12815E+01_JPRB, 0.12209E+01_JPRB, 0.11746E+01_JPRB, 0.11387E+01_JPRB, 0.11139E+01_JPRB, 0.10932E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.10534E+01_JPRB, 0.10295E+01_JPRB, 0.10058E+01_JPRB, 0.98277E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.93416E+00_JPRB, 0.91798E+00_JPRB, 0.90271E+00_JPRB, 0.88128E+00_JPRB, 0.86000E+00_JPRB, 0.83979E+00_JPRB, 0.82430E+00_JPRB ,&
&  0.81086E+00_JPRB, 0.79643E+00_JPRB, 0.78235E+00_JPRB, 0.76583E+00_JPRB, 0.74971E+00_JPRB, 0.73397E+00_JPRB, 0.71959E+00_JPRB ,&
&  0.70586E+00_JPRB, 0.68778E+00_JPRB, 0.67001E+00_JPRB, 0.65065E+00_JPRB, 0.63354E+00_JPRB, 0.61731E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.58238E+00_JPRB, 0.55732E+00_JPRB, 0.53087E+00_JPRB, 0.51745E+00_JPRB, 0.49021E+00_JPRB, 0.47582E+00_JPRB, 0.46854E+00_JPRB ,&
&  0.48548E+00_JPRB, 0.47371E+00_JPRB, 0.48480E+00_JPRB, 0.46584E+00_JPRB, 0.45113E+00_JPRB, 0.43763E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.41717E+00_JPRB, 0.40801E+00_JPRB, 0.40832E+00_JPRB, 0.41313E+00_JPRB, 0.41718E+00_JPRB, 0.43184E+00_JPRB, 0.45077E+00_JPRB ,&
&  0.49396E+00_JPRB, 0.61024E+00_JPRB, 0.63666E+00_JPRB, 0.66367E+00_JPRB, 0.67325E+00_JPRB, 0.68365E+00_JPRB, 0.69345E+00_JPRB ,&
&  0.70779E+00_JPRB, 0.71881E+00_JPRB, 0.73166E+00_JPRB, 0.73992E+00_JPRB, 0.76825E+00_JPRB, 0.79083E+00_JPRB, 0.81649E+00_JPRB ,&
&  0.83020E+00_JPRB, 0.84436E+00_JPRB, 0.85763E+00_JPRB, 0.88464E+00_JPRB, 0.93183E+00_JPRB, 0.95718E+00_JPRB, 0.96870E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.10669E+01_JPRB, 0.11540E+01_JPRB, 0.12566E+01_JPRB, 0.13307E+01_JPRB, 0.13905E+01_JPRB, 0.14851E+01_JPRB ,&
&  0.15485E+01_JPRB, 0.16583E+01_JPRB, 0.17702E+01_JPRB, 0.18150E+01_JPRB, 0.18117E+01_JPRB, 0.17809E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.17277E+01_JPRB, 0.16892E+01_JPRB, 0.16614E+01_JPRB, 0.16411E+01_JPRB, 0.15979E+01_JPRB, 0.15477E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.13773E+01_JPRB, 0.13284E+01_JPRB, 0.12727E+01_JPRB, 0.12154E+01_JPRB, 0.11605E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10238E+01_JPRB, 0.98839E+00_JPRB, 0.96622E+00_JPRB, 0.94544E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.89655E+00_JPRB, 0.87392E+00_JPRB, 0.85041E+00_JPRB, 0.82822E+00_JPRB, 0.80984E+00_JPRB, 0.79513E+00_JPRB, 0.78357E+00_JPRB ,&
&  0.77328E+00_JPRB, 0.76155E+00_JPRB, 0.74947E+00_JPRB, 0.73667E+00_JPRB, 0.72444E+00_JPRB, 0.71622E+00_JPRB, 0.70853E+00_JPRB ,&
&  0.70230E+00_JPRB, 0.69196E+00_JPRB, 0.67927E+00_JPRB, 0.66329E+00_JPRB, 0.64545E+00_JPRB, 0.62994E+00_JPRB, 0.60796E+00_JPRB ,&
&  0.59063E+00_JPRB, 0.57113E+00_JPRB, 0.54082E+00_JPRB, 0.52334E+00_JPRB, 0.49282E+00_JPRB, 0.47267E+00_JPRB, 0.46125E+00_JPRB ,&
&  0.45329E+00_JPRB, 0.45393E+00_JPRB, 0.45812E+00_JPRB, 0.46450E+00_JPRB, 0.46346E+00_JPRB, 0.45776E+00_JPRB, 0.45645E+00_JPRB ,&
&  0.44912E+00_JPRB, 0.44500E+00_JPRB, 0.44187E+00_JPRB, 0.44221E+00_JPRB, 0.44601E+00_JPRB, 0.45569E+00_JPRB, 0.47415E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.56724E+00_JPRB, 0.62068E+00_JPRB, 0.65500E+00_JPRB, 0.66411E+00_JPRB, 0.67542E+00_JPRB, 0.68191E+00_JPRB ,&
&  0.69273E+00_JPRB, 0.70236E+00_JPRB, 0.71658E+00_JPRB, 0.73073E+00_JPRB, 0.74759E+00_JPRB, 0.76243E+00_JPRB, 0.77972E+00_JPRB ,&
&  0.78771E+00_JPRB, 0.79652E+00_JPRB, 0.80200E+00_JPRB, 0.82055E+00_JPRB, 0.85335E+00_JPRB, 0.88702E+00_JPRB, 0.90530E+00_JPRB ,&
&  0.93343E+00_JPRB, 0.97114E+00_JPRB, 0.10360E+01_JPRB, 0.11315E+01_JPRB, 0.12333E+01_JPRB, 0.13249E+01_JPRB, 0.14120E+01_JPRB ,&
&  0.14934E+01_JPRB, 0.15916E+01_JPRB, 0.16820E+01_JPRB, 0.17122E+01_JPRB, 0.17041E+01_JPRB, 0.17071E+01_JPRB, 0.16800E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.16288E+01_JPRB, 0.16097E+01_JPRB, 0.15906E+01_JPRB, 0.15548E+01_JPRB, 0.15033E+01_JPRB, 0.14497E+01_JPRB ,&
&  0.13928E+01_JPRB, 0.13327E+01_JPRB, 0.12751E+01_JPRB, 0.12176E+01_JPRB, 0.11626E+01_JPRB, 0.11099E+01_JPRB, 0.10653E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.99128E+00_JPRB, 0.96208E+00_JPRB, 0.93865E+00_JPRB, 0.91763E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.86025E+00_JPRB, 0.83917E+00_JPRB, 0.81985E+00_JPRB, 0.80257E+00_JPRB, 0.78694E+00_JPRB, 0.77390E+00_JPRB, 0.76279E+00_JPRB ,&
&  0.75323E+00_JPRB, 0.74733E+00_JPRB, 0.74151E+00_JPRB, 0.73512E+00_JPRB, 0.72849E+00_JPRB, 0.72529E+00_JPRB, 0.72025E+00_JPRB ,&
&  0.71401E+00_JPRB, 0.70650E+00_JPRB, 0.69982E+00_JPRB, 0.69113E+00_JPRB, 0.67729E+00_JPRB, 0.66059E+00_JPRB, 0.63898E+00_JPRB ,&
&  0.62551E+00_JPRB, 0.60808E+00_JPRB, 0.58853E+00_JPRB, 0.56750E+00_JPRB, 0.55331E+00_JPRB, 0.52265E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.50309E+00_JPRB, 0.48884E+00_JPRB, 0.48070E+00_JPRB, 0.47795E+00_JPRB, 0.49605E+00_JPRB, 0.51322E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.50725E+00_JPRB, 0.51421E+00_JPRB, 0.51069E+00_JPRB, 0.50865E+00_JPRB, 0.51167E+00_JPRB, 0.51960E+00_JPRB, 0.54958E+00_JPRB ,&
&  0.60379E+00_JPRB, 0.62799E+00_JPRB, 0.64519E+00_JPRB, 0.66173E+00_JPRB, 0.67320E+00_JPRB, 0.68380E+00_JPRB, 0.69386E+00_JPRB ,&
&  0.70763E+00_JPRB, 0.72077E+00_JPRB, 0.73340E+00_JPRB, 0.74018E+00_JPRB, 0.74843E+00_JPRB, 0.75656E+00_JPRB, 0.76444E+00_JPRB ,&
&  0.76090E+00_JPRB, 0.76128E+00_JPRB, 0.76645E+00_JPRB, 0.77745E+00_JPRB, 0.79354E+00_JPRB, 0.81981E+00_JPRB, 0.84938E+00_JPRB ,&
&  0.88178E+00_JPRB, 0.90864E+00_JPRB, 0.94860E+00_JPRB, 0.10009E+01_JPRB, 0.10629E+01_JPRB, 0.11369E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.12824E+01_JPRB, 0.13402E+01_JPRB, 0.13928E+01_JPRB, 0.14144E+01_JPRB, 0.14221E+01_JPRB, 0.14257E+01_JPRB, 0.14192E+01_JPRB ,&
&  0.14089E+01_JPRB, 0.13924E+01_JPRB, 0.13761E+01_JPRB, 0.13545E+01_JPRB, 0.13285E+01_JPRB, 0.12964E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.11825E+01_JPRB, 0.11449E+01_JPRB, 0.11089E+01_JPRB, 0.10731E+01_JPRB, 0.10401E+01_JPRB, 0.10105E+01_JPRB ,&
&  0.98315E+00_JPRB, 0.95580E+00_JPRB, 0.93082E+00_JPRB, 0.90721E+00_JPRB, 0.88391E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.81624E+00_JPRB, 0.80256E+00_JPRB, 0.79015E+00_JPRB, 0.77878E+00_JPRB, 0.76936E+00_JPRB, 0.76188E+00_JPRB, 0.75541E+00_JPRB ,&
&  0.74970E+00_JPRB, 0.74628E+00_JPRB, 0.74341E+00_JPRB, 0.74050E+00_JPRB, 0.73765E+00_JPRB, 0.73479E+00_JPRB, 0.73148E+00_JPRB ,&
&  0.72831E+00_JPRB, 0.72614E+00_JPRB, 0.72435E+00_JPRB, 0.72225E+00_JPRB, 0.71799E+00_JPRB, 0.71121E+00_JPRB, 0.70330E+00_JPRB ,&
&  0.69478E+00_JPRB, 0.68385E+00_JPRB, 0.67009E+00_JPRB, 0.65479E+00_JPRB, 0.64590E+00_JPRB, 0.63571E+00_JPRB, 0.62596E+00_JPRB ,&
&  0.61797E+00_JPRB, 0.61584E+00_JPRB, 0.61358E+00_JPRB, 0.61152E+00_JPRB, 0.61000E+00_JPRB, 0.61076E+00_JPRB, 0.61090E+00_JPRB ,&
&  0.61106E+00_JPRB, 0.61341E+00_JPRB, 0.62234E+00_JPRB, 0.63202E+00_JPRB, 0.64328E+00_JPRB, 0.65664E+00_JPRB, 0.67161E+00_JPRB ,&
&  0.68330E+00_JPRB, 0.69105E+00_JPRB, 0.69991E+00_JPRB, 0.70862E+00_JPRB, 0.71543E+00_JPRB, 0.72137E+00_JPRB, 0.72724E+00_JPRB ,&
&  0.73391E+00_JPRB, 0.73886E+00_JPRB, 0.74359E+00_JPRB, 0.74719E+00_JPRB, 0.74784E+00_JPRB, 0.74735E+00_JPRB, 0.74554E+00_JPRB ,&
&  0.74227E+00_JPRB, 0.74291E+00_JPRB, 0.74657E+00_JPRB, 0.75122E+00_JPRB, 0.75932E+00_JPRB, 0.77106E+00_JPRB, 0.78462E+00_JPRB ,&
&  0.80220E+00_JPRB, 0.82388E+00_JPRB, 0.85063E+00_JPRB, 0.87992E+00_JPRB, 0.91344E+00_JPRB, 0.95386E+00_JPRB, 0.98610E+00_JPRB ,&
&  0.10148E+01_JPRB, 0.10390E+01_JPRB, 0.10597E+01_JPRB, 0.10728E+01_JPRB, 0.10809E+01_JPRB, 0.10879E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.10737E+01_JPRB, 0.10679E+01_JPRB, 0.10573E+01_JPRB, 0.10450E+01_JPRB, 0.10327E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.10046E+01_JPRB, 0.98806E+00_JPRB, 0.97229E+00_JPRB, 0.95741E+00_JPRB, 0.94120E+00_JPRB, 0.92563E+00_JPRB, 0.91010E+00_JPRB ,&
&  0.89454E+00_JPRB, 0.87803E+00_JPRB, 0.86225E+00_JPRB, 0.84697E+00_JPRB, 0.83180E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.78201E+00_JPRB, 0.77819E+00_JPRB, 0.77482E+00_JPRB, 0.77123E+00_JPRB, 0.76732E+00_JPRB, 0.76406E+00_JPRB, 0.76115E+00_JPRB ,&
&  0.75843E+00_JPRB, 0.75653E+00_JPRB, 0.75540E+00_JPRB, 0.75467E+00_JPRB, 0.75456E+00_JPRB, 0.75425E+00_JPRB, 0.75416E+00_JPRB ,&
&  0.75414E+00_JPRB, 0.75433E+00_JPRB, 0.75494E+00_JPRB, 0.75514E+00_JPRB, 0.75479E+00_JPRB, 0.75415E+00_JPRB, 0.75321E+00_JPRB ,&
&  0.75147E+00_JPRB, 0.74943E+00_JPRB, 0.74719E+00_JPRB, 0.74478E+00_JPRB, 0.74191E+00_JPRB, 0.73861E+00_JPRB, 0.73496E+00_JPRB ,&
&  0.73166E+00_JPRB, 0.72937E+00_JPRB, 0.72741E+00_JPRB, 0.72526E+00_JPRB, 0.72393E+00_JPRB, 0.72443E+00_JPRB, 0.72486E+00_JPRB ,&
&  0.72531E+00_JPRB, 0.72617E+00_JPRB, 0.72852E+00_JPRB, 0.73088E+00_JPRB, 0.73306E+00_JPRB, 0.73551E+00_JPRB, 0.73848E+00_JPRB ,&
&  0.74052E+00_JPRB, 0.74218E+00_JPRB, 0.74360E+00_JPRB, 0.74493E+00_JPRB, 0.74570E+00_JPRB, 0.74664E+00_JPRB, 0.74738E+00_JPRB ,&
&  0.74781E+00_JPRB, 0.74820E+00_JPRB, 0.74809E+00_JPRB, 0.74695E+00_JPRB, 0.74320E+00_JPRB, 0.74054E+00_JPRB, 0.73967E+00_JPRB ,&
&  0.74044E+00_JPRB, 0.74174E+00_JPRB, 0.74341E+00_JPRB, 0.74567E+00_JPRB, 0.74918E+00_JPRB, 0.75330E+00_JPRB, 0.75802E+00_JPRB ,&
&  0.76310E+00_JPRB, 0.76894E+00_JPRB, 0.77663E+00_JPRB, 0.78466E+00_JPRB, 0.79350E+00_JPRB, 0.80341E+00_JPRB, 0.81223E+00_JPRB ,&
&  0.82140E+00_JPRB, 0.83092E+00_JPRB, 0.83878E+00_JPRB, 0.84577E+00_JPRB, 0.85264E+00_JPRB, 0.85913E+00_JPRB, 0.86364E+00_JPRB ,&
&  0.86522E+00_JPRB, 0.86632E+00_JPRB, 0.86695E+00_JPRB, 0.86513E+00_JPRB, 0.86151E+00_JPRB, 0.85843E+00_JPRB, 0.85513E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.84650E+00_JPRB, 0.84198E+00_JPRB, 0.83709E+00_JPRB, 0.83086E+00_JPRB, 0.82420E+00_JPRB, 0.81797E+00_JPRB ,&
&  0.81173E+00_JPRB, 0.80500E+00_JPRB, 0.79876E+00_JPRB, 0.79282E+00_JPRB, 0.78730E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB, 0.77848E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB ,&
&  0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB, 0.12758E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.13346E+00_JPRB, 0.13331E+00_JPRB, 0.13313E+00_JPRB, 0.13294E+00_JPRB, 0.13231E+00_JPRB, 0.13204E+00_JPRB, 0.13179E+00_JPRB ,&
&  0.13166E+00_JPRB, 0.13157E+00_JPRB, 0.13164E+00_JPRB, 0.13176E+00_JPRB, 0.13189E+00_JPRB, 0.13200E+00_JPRB, 0.13208E+00_JPRB ,&
&  0.13204E+00_JPRB, 0.13203E+00_JPRB, 0.13144E+00_JPRB, 0.13099E+00_JPRB, 0.13072E+00_JPRB, 0.13053E+00_JPRB, 0.13024E+00_JPRB ,&
&  0.13002E+00_JPRB, 0.12974E+00_JPRB, 0.12941E+00_JPRB, 0.12916E+00_JPRB, 0.12916E+00_JPRB, 0.12903E+00_JPRB, 0.12903E+00_JPRB ,&
&  0.12899E+00_JPRB, 0.12912E+00_JPRB, 0.12916E+00_JPRB, 0.12922E+00_JPRB, 0.12950E+00_JPRB, 0.12974E+00_JPRB, 0.12977E+00_JPRB ,&
&  0.12977E+00_JPRB, 0.12959E+00_JPRB, 0.12941E+00_JPRB, 0.12929E+00_JPRB, 0.12922E+00_JPRB, 0.12919E+00_JPRB, 0.12914E+00_JPRB ,&
&  0.12901E+00_JPRB, 0.12881E+00_JPRB, 0.12857E+00_JPRB, 0.12830E+00_JPRB, 0.12805E+00_JPRB, 0.12790E+00_JPRB, 0.12785E+00_JPRB ,&
&  0.12781E+00_JPRB, 0.12767E+00_JPRB, 0.12746E+00_JPRB, 0.12716E+00_JPRB, 0.12689E+00_JPRB, 0.12662E+00_JPRB, 0.12637E+00_JPRB ,&
&  0.12614E+00_JPRB, 0.12602E+00_JPRB, 0.12589E+00_JPRB, 0.12578E+00_JPRB, 0.12571E+00_JPRB, 0.12560E+00_JPRB, 0.12550E+00_JPRB ,&
&  0.12541E+00_JPRB, 0.12533E+00_JPRB, 0.12525E+00_JPRB, 0.12519E+00_JPRB, 0.12514E+00_JPRB, 0.12511E+00_JPRB, 0.12511E+00_JPRB ,&
&  0.12513E+00_JPRB, 0.12515E+00_JPRB, 0.12519E+00_JPRB, 0.12532E+00_JPRB, 0.12547E+00_JPRB, 0.12566E+00_JPRB, 0.12588E+00_JPRB ,&
&  0.12612E+00_JPRB, 0.12639E+00_JPRB, 0.12670E+00_JPRB, 0.12693E+00_JPRB, 0.12727E+00_JPRB, 0.12761E+00_JPRB, 0.12796E+00_JPRB ,&
&  0.12832E+00_JPRB, 0.12876E+00_JPRB, 0.12925E+00_JPRB, 0.12978E+00_JPRB, 0.13035E+00_JPRB, 0.13068E+00_JPRB, 0.13108E+00_JPRB ,&
&  0.13153E+00_JPRB, 0.13210E+00_JPRB, 0.13243E+00_JPRB, 0.13268E+00_JPRB, 0.13296E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.14362E+00_JPRB, 0.14562E+00_JPRB, 0.14709E+00_JPRB, 0.14889E+00_JPRB, 0.14854E+00_JPRB, 0.16008E+00_JPRB, 0.15815E+00_JPRB ,&
&  0.15702E+00_JPRB, 0.15216E+00_JPRB, 0.15170E+00_JPRB, 0.15006E+00_JPRB, 0.14811E+00_JPRB, 0.14596E+00_JPRB, 0.14398E+00_JPRB ,&
&  0.14216E+00_JPRB, 0.14015E+00_JPRB, 0.13868E+00_JPRB, 0.13748E+00_JPRB, 0.13639E+00_JPRB, 0.13539E+00_JPRB, 0.13459E+00_JPRB ,&
&  0.13452E+00_JPRB, 0.13503E+00_JPRB, 0.13598E+00_JPRB, 0.13587E+00_JPRB, 0.13561E+00_JPRB, 0.13526E+00_JPRB, 0.13508E+00_JPRB ,&
&  0.13456E+00_JPRB, 0.13418E+00_JPRB, 0.13385E+00_JPRB, 0.13335E+00_JPRB, 0.13386E+00_JPRB, 0.13434E+00_JPRB, 0.13484E+00_JPRB ,&
&  0.13531E+00_JPRB, 0.13577E+00_JPRB, 0.13571E+00_JPRB, 0.13556E+00_JPRB, 0.13516E+00_JPRB, 0.13461E+00_JPRB, 0.13378E+00_JPRB ,&
&  0.13298E+00_JPRB, 0.13214E+00_JPRB, 0.13129E+00_JPRB, 0.13069E+00_JPRB, 0.13016E+00_JPRB, 0.12966E+00_JPRB, 0.12923E+00_JPRB ,&
&  0.12901E+00_JPRB, 0.12887E+00_JPRB, 0.12869E+00_JPRB, 0.12841E+00_JPRB, 0.12807E+00_JPRB, 0.12771E+00_JPRB, 0.12739E+00_JPRB ,&
&  0.12715E+00_JPRB, 0.12671E+00_JPRB, 0.12631E+00_JPRB, 0.12587E+00_JPRB, 0.12555E+00_JPRB, 0.12515E+00_JPRB, 0.12481E+00_JPRB ,&
&  0.12450E+00_JPRB, 0.12428E+00_JPRB, 0.12421E+00_JPRB, 0.12418E+00_JPRB, 0.12415E+00_JPRB, 0.12420E+00_JPRB, 0.12431E+00_JPRB ,&
&  0.12448E+00_JPRB, 0.12467E+00_JPRB, 0.12491E+00_JPRB, 0.12509E+00_JPRB, 0.12526E+00_JPRB, 0.12541E+00_JPRB, 0.12577E+00_JPRB ,&
&  0.12621E+00_JPRB, 0.12667E+00_JPRB, 0.12712E+00_JPRB, 0.12736E+00_JPRB, 0.12812E+00_JPRB, 0.12889E+00_JPRB, 0.12974E+00_JPRB ,&
&  0.13064E+00_JPRB, 0.13179E+00_JPRB, 0.13304E+00_JPRB, 0.13435E+00_JPRB, 0.13587E+00_JPRB, 0.13689E+00_JPRB, 0.13800E+00_JPRB ,&
&  0.13948E+00_JPRB, 0.14189E+00_JPRB, 0.14282E+00_JPRB, 0.14324E+00_JPRB, 0.14353E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.16813E+00_JPRB, 0.16876E+00_JPRB, 0.16934E+00_JPRB, 0.16956E+00_JPRB, 0.16960E+00_JPRB, 0.16895E+00_JPRB, 0.16815E+00_JPRB ,&
&  0.16457E+00_JPRB, 0.16194E+00_JPRB, 0.15906E+00_JPRB, 0.15622E+00_JPRB, 0.15373E+00_JPRB, 0.15130E+00_JPRB, 0.14907E+00_JPRB ,&
&  0.14704E+00_JPRB, 0.14555E+00_JPRB, 0.14412E+00_JPRB, 0.14213E+00_JPRB, 0.13960E+00_JPRB, 0.13728E+00_JPRB, 0.13575E+00_JPRB ,&
&  0.13526E+00_JPRB, 0.13550E+00_JPRB, 0.13678E+00_JPRB, 0.13815E+00_JPRB, 0.13961E+00_JPRB, 0.14072E+00_JPRB, 0.14183E+00_JPRB ,&
&  0.14127E+00_JPRB, 0.14090E+00_JPRB, 0.14057E+00_JPRB, 0.14001E+00_JPRB, 0.14051E+00_JPRB, 0.14119E+00_JPRB, 0.14156E+00_JPRB ,&
&  0.14200E+00_JPRB, 0.14319E+00_JPRB, 0.14401E+00_JPRB, 0.14479E+00_JPRB, 0.14462E+00_JPRB, 0.14380E+00_JPRB, 0.14235E+00_JPRB ,&
&  0.14060E+00_JPRB, 0.13881E+00_JPRB, 0.13725E+00_JPRB, 0.13591E+00_JPRB, 0.13489E+00_JPRB, 0.13377E+00_JPRB, 0.13286E+00_JPRB ,&
&  0.13240E+00_JPRB, 0.13199E+00_JPRB, 0.13138E+00_JPRB, 0.13077E+00_JPRB, 0.12982E+00_JPRB, 0.12900E+00_JPRB, 0.12815E+00_JPRB ,&
&  0.12737E+00_JPRB, 0.12662E+00_JPRB, 0.12594E+00_JPRB, 0.12524E+00_JPRB, 0.12476E+00_JPRB, 0.12431E+00_JPRB, 0.12388E+00_JPRB ,&
&  0.12343E+00_JPRB, 0.12326E+00_JPRB, 0.12293E+00_JPRB, 0.12283E+00_JPRB, 0.12264E+00_JPRB, 0.12252E+00_JPRB, 0.12269E+00_JPRB ,&
&  0.12317E+00_JPRB, 0.12365E+00_JPRB, 0.12391E+00_JPRB, 0.12421E+00_JPRB, 0.12450E+00_JPRB, 0.12489E+00_JPRB, 0.12551E+00_JPRB ,&
&  0.12598E+00_JPRB, 0.12658E+00_JPRB, 0.12710E+00_JPRB, 0.12772E+00_JPRB, 0.12885E+00_JPRB, 0.13003E+00_JPRB, 0.13137E+00_JPRB ,&
&  0.13304E+00_JPRB, 0.13481E+00_JPRB, 0.13688E+00_JPRB, 0.13892E+00_JPRB, 0.14159E+00_JPRB, 0.14332E+00_JPRB, 0.14649E+00_JPRB ,&
&  0.15001E+00_JPRB, 0.15154E+00_JPRB, 0.16310E+00_JPRB, 0.16422E+00_JPRB, 0.16581E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.17399E+00_JPRB, 0.17511E+00_JPRB, 0.17496E+00_JPRB, 0.17529E+00_JPRB, 0.17440E+00_JPRB, 0.17298E+00_JPRB, 0.17097E+00_JPRB ,&
&  0.16856E+00_JPRB, 0.16306E+00_JPRB, 0.15927E+00_JPRB, 0.15509E+00_JPRB, 0.15155E+00_JPRB, 0.14849E+00_JPRB, 0.14635E+00_JPRB ,&
&  0.14543E+00_JPRB, 0.14517E+00_JPRB, 0.14505E+00_JPRB, 0.14427E+00_JPRB, 0.14282E+00_JPRB, 0.14185E+00_JPRB, 0.14060E+00_JPRB ,&
&  0.13968E+00_JPRB, 0.13958E+00_JPRB, 0.14034E+00_JPRB, 0.14209E+00_JPRB, 0.14514E+00_JPRB, 0.14746E+00_JPRB, 0.14953E+00_JPRB ,&
&  0.15177E+00_JPRB, 0.14860E+00_JPRB, 0.14836E+00_JPRB, 0.14860E+00_JPRB, 0.14842E+00_JPRB, 0.14763E+00_JPRB, 0.14608E+00_JPRB ,&
&  0.14855E+00_JPRB, 0.14917E+00_JPRB, 0.15368E+00_JPRB, 0.15360E+00_JPRB, 0.15257E+00_JPRB, 0.15331E+00_JPRB, 0.15171E+00_JPRB ,&
&  0.15104E+00_JPRB, 0.14808E+00_JPRB, 0.14591E+00_JPRB, 0.14301E+00_JPRB, 0.14024E+00_JPRB, 0.13885E+00_JPRB, 0.13725E+00_JPRB ,&
&  0.13673E+00_JPRB, 0.13661E+00_JPRB, 0.13539E+00_JPRB, 0.13463E+00_JPRB, 0.13326E+00_JPRB, 0.13172E+00_JPRB, 0.13074E+00_JPRB ,&
&  0.12972E+00_JPRB, 0.12891E+00_JPRB, 0.12847E+00_JPRB, 0.12805E+00_JPRB, 0.12789E+00_JPRB, 0.12781E+00_JPRB, 0.12773E+00_JPRB ,&
&  0.12775E+00_JPRB, 0.12766E+00_JPRB, 0.12717E+00_JPRB, 0.12687E+00_JPRB, 0.12598E+00_JPRB, 0.12537E+00_JPRB, 0.12487E+00_JPRB ,&
&  0.12445E+00_JPRB, 0.12430E+00_JPRB, 0.12423E+00_JPRB, 0.12427E+00_JPRB, 0.12439E+00_JPRB, 0.12479E+00_JPRB, 0.12524E+00_JPRB ,&
&  0.12570E+00_JPRB, 0.12650E+00_JPRB, 0.12714E+00_JPRB, 0.12804E+00_JPRB, 0.12910E+00_JPRB, 0.13001E+00_JPRB, 0.13146E+00_JPRB ,&
&  0.13308E+00_JPRB, 0.13497E+00_JPRB, 0.13775E+00_JPRB, 0.14037E+00_JPRB, 0.14340E+00_JPRB, 0.14762E+00_JPRB, 0.15229E+00_JPRB ,&
&  0.15745E+00_JPRB, 0.16370E+00_JPRB, 0.16448E+00_JPRB, 0.16918E+00_JPRB, 0.17229E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.17423E+00_JPRB, 0.17585E+00_JPRB, 0.17656E+00_JPRB, 0.17754E+00_JPRB, 0.17746E+00_JPRB, 0.17530E+00_JPRB, 0.17314E+00_JPRB ,&
&  0.16814E+00_JPRB, 0.16406E+00_JPRB, 0.15911E+00_JPRB, 0.15445E+00_JPRB, 0.15008E+00_JPRB, 0.14725E+00_JPRB, 0.14563E+00_JPRB ,&
&  0.14573E+00_JPRB, 0.14720E+00_JPRB, 0.14831E+00_JPRB, 0.14894E+00_JPRB, 0.14943E+00_JPRB, 0.15047E+00_JPRB, 0.15070E+00_JPRB ,&
&  0.15035E+00_JPRB, 0.14928E+00_JPRB, 0.14849E+00_JPRB, 0.14940E+00_JPRB, 0.15133E+00_JPRB, 0.15335E+00_JPRB, 0.15502E+00_JPRB ,&
&  0.15654E+00_JPRB, 0.15599E+00_JPRB, 0.15602E+00_JPRB, 0.15527E+00_JPRB, 0.15467E+00_JPRB, 0.15392E+00_JPRB, 0.15361E+00_JPRB ,&
&  0.15418E+00_JPRB, 0.15465E+00_JPRB, 0.15619E+00_JPRB, 0.15709E+00_JPRB, 0.15588E+00_JPRB, 0.15734E+00_JPRB, 0.15718E+00_JPRB ,&
&  0.15622E+00_JPRB, 0.15329E+00_JPRB, 0.15021E+00_JPRB, 0.14783E+00_JPRB, 0.14539E+00_JPRB, 0.14346E+00_JPRB, 0.14187E+00_JPRB ,&
&  0.14040E+00_JPRB, 0.13959E+00_JPRB, 0.13792E+00_JPRB, 0.13652E+00_JPRB, 0.13563E+00_JPRB, 0.13470E+00_JPRB, 0.13410E+00_JPRB ,&
&  0.13369E+00_JPRB, 0.13302E+00_JPRB, 0.13286E+00_JPRB, 0.13253E+00_JPRB, 0.13263E+00_JPRB, 0.13278E+00_JPRB, 0.13312E+00_JPRB ,&
&  0.13385E+00_JPRB, 0.13470E+00_JPRB, 0.13461E+00_JPRB, 0.13446E+00_JPRB, 0.13311E+00_JPRB, 0.13162E+00_JPRB, 0.13031E+00_JPRB ,&
&  0.12867E+00_JPRB, 0.12743E+00_JPRB, 0.12665E+00_JPRB, 0.12591E+00_JPRB, 0.12581E+00_JPRB, 0.12587E+00_JPRB, 0.12587E+00_JPRB ,&
&  0.12606E+00_JPRB, 0.12634E+00_JPRB, 0.12676E+00_JPRB, 0.12762E+00_JPRB, 0.12837E+00_JPRB, 0.12946E+00_JPRB, 0.13065E+00_JPRB ,&
&  0.13162E+00_JPRB, 0.13338E+00_JPRB, 0.13527E+00_JPRB, 0.13754E+00_JPRB, 0.14176E+00_JPRB, 0.14551E+00_JPRB, 0.15235E+00_JPRB ,&
&  0.15796E+00_JPRB, 0.16747E+00_JPRB, 0.16579E+00_JPRB, 0.16913E+00_JPRB, 0.17242E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.17639E+00_JPRB, 0.17741E+00_JPRB, 0.17715E+00_JPRB, 0.17730E+00_JPRB, 0.17690E+00_JPRB, 0.17804E+00_JPRB, 0.17737E+00_JPRB ,&
&  0.17745E+00_JPRB, 0.17659E+00_JPRB, 0.17572E+00_JPRB, 0.17345E+00_JPRB, 0.17227E+00_JPRB, 0.16637E+00_JPRB, 0.16488E+00_JPRB ,&
&  0.16372E+00_JPRB, 0.16459E+00_JPRB, 0.16485E+00_JPRB, 0.16536E+00_JPRB, 0.16632E+00_JPRB, 0.16795E+00_JPRB, 0.16805E+00_JPRB ,&
&  0.16703E+00_JPRB, 0.16500E+00_JPRB, 0.16265E+00_JPRB, 0.16050E+00_JPRB, 0.16086E+00_JPRB, 0.16056E+00_JPRB, 0.15967E+00_JPRB ,&
&  0.15868E+00_JPRB, 0.15923E+00_JPRB, 0.15973E+00_JPRB, 0.16104E+00_JPRB, 0.16596E+00_JPRB, 0.16453E+00_JPRB, 0.16330E+00_JPRB ,&
&  0.16307E+00_JPRB, 0.16290E+00_JPRB, 0.16334E+00_JPRB, 0.16364E+00_JPRB, 0.16537E+00_JPRB, 0.16484E+00_JPRB, 0.16728E+00_JPRB ,&
&  0.16455E+00_JPRB, 0.16280E+00_JPRB, 0.16051E+00_JPRB, 0.15771E+00_JPRB, 0.15426E+00_JPRB, 0.15207E+00_JPRB, 0.14998E+00_JPRB ,&
&  0.14771E+00_JPRB, 0.14579E+00_JPRB, 0.14332E+00_JPRB, 0.14241E+00_JPRB, 0.14115E+00_JPRB, 0.14004E+00_JPRB, 0.13903E+00_JPRB ,&
&  0.13825E+00_JPRB, 0.13716E+00_JPRB, 0.13620E+00_JPRB, 0.13555E+00_JPRB, 0.13534E+00_JPRB, 0.13546E+00_JPRB, 0.13641E+00_JPRB ,&
&  0.13828E+00_JPRB, 0.13985E+00_JPRB, 0.14106E+00_JPRB, 0.14201E+00_JPRB, 0.14064E+00_JPRB, 0.13817E+00_JPRB, 0.13567E+00_JPRB ,&
&  0.13247E+00_JPRB, 0.13068E+00_JPRB, 0.12903E+00_JPRB, 0.12805E+00_JPRB, 0.12780E+00_JPRB, 0.12779E+00_JPRB, 0.12807E+00_JPRB ,&
&  0.12837E+00_JPRB, 0.12881E+00_JPRB, 0.12912E+00_JPRB, 0.12952E+00_JPRB, 0.13004E+00_JPRB, 0.13055E+00_JPRB, 0.13130E+00_JPRB ,&
&  0.13249E+00_JPRB, 0.13403E+00_JPRB, 0.13616E+00_JPRB, 0.13870E+00_JPRB, 0.14161E+00_JPRB, 0.14545E+00_JPRB, 0.15113E+00_JPRB ,&
&  0.15712E+00_JPRB, 0.16742E+00_JPRB, 0.16851E+00_JPRB, 0.17298E+00_JPRB, 0.17563E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.17900E+00_JPRB, 0.18057E+00_JPRB, 0.18114E+00_JPRB, 0.18231E+00_JPRB, 0.18276E+00_JPRB, 0.18407E+00_JPRB, 0.18517E+00_JPRB ,&
&  0.18761E+00_JPRB, 0.18981E+00_JPRB, 0.19038E+00_JPRB, 0.18875E+00_JPRB, 0.18788E+00_JPRB, 0.18545E+00_JPRB, 0.18459E+00_JPRB ,&
&  0.18452E+00_JPRB, 0.18563E+00_JPRB, 0.18586E+00_JPRB, 0.18599E+00_JPRB, 0.18600E+00_JPRB, 0.18601E+00_JPRB, 0.18645E+00_JPRB ,&
&  0.18458E+00_JPRB, 0.18402E+00_JPRB, 0.18611E+00_JPRB, 0.18544E+00_JPRB, 0.18488E+00_JPRB, 0.18364E+00_JPRB, 0.18296E+00_JPRB ,&
&  0.17912E+00_JPRB, 0.17355E+00_JPRB, 0.16681E+00_JPRB, 0.16951E+00_JPRB, 0.17666E+00_JPRB, 0.17464E+00_JPRB, 0.17388E+00_JPRB ,&
&  0.17328E+00_JPRB, 0.17213E+00_JPRB, 0.17166E+00_JPRB, 0.17117E+00_JPRB, 0.17166E+00_JPRB, 0.17220E+00_JPRB, 0.17367E+00_JPRB ,&
&  0.17574E+00_JPRB, 0.17877E+00_JPRB, 0.17864E+00_JPRB, 0.17265E+00_JPRB, 0.16946E+00_JPRB, 0.16831E+00_JPRB, 0.16519E+00_JPRB ,&
&  0.16157E+00_JPRB, 0.16000E+00_JPRB, 0.15913E+00_JPRB, 0.15705E+00_JPRB, 0.15651E+00_JPRB, 0.15605E+00_JPRB, 0.15387E+00_JPRB ,&
&  0.15096E+00_JPRB, 0.14880E+00_JPRB, 0.14563E+00_JPRB, 0.14235E+00_JPRB, 0.14003E+00_JPRB, 0.13892E+00_JPRB, 0.13893E+00_JPRB ,&
&  0.14055E+00_JPRB, 0.14234E+00_JPRB, 0.14787E+00_JPRB, 0.15156E+00_JPRB, 0.14877E+00_JPRB, 0.14533E+00_JPRB, 0.14178E+00_JPRB ,&
&  0.13691E+00_JPRB, 0.13456E+00_JPRB, 0.13271E+00_JPRB, 0.13138E+00_JPRB, 0.13124E+00_JPRB, 0.13197E+00_JPRB, 0.13256E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13417E+00_JPRB, 0.13460E+00_JPRB, 0.13478E+00_JPRB, 0.13481E+00_JPRB, 0.13503E+00_JPRB, 0.13558E+00_JPRB ,&
&  0.13629E+00_JPRB, 0.13797E+00_JPRB, 0.14040E+00_JPRB, 0.14301E+00_JPRB, 0.14483E+00_JPRB, 0.14624E+00_JPRB, 0.14919E+00_JPRB ,&
&  0.15267E+00_JPRB, 0.16076E+00_JPRB, 0.17227E+00_JPRB, 0.18126E+00_JPRB, 0.17802E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.19196E+00_JPRB, 0.19288E+00_JPRB, 0.19397E+00_JPRB, 0.19526E+00_JPRB, 0.19676E+00_JPRB, 0.19841E+00_JPRB, 0.19979E+00_JPRB ,&
&  0.20130E+00_JPRB, 0.20227E+00_JPRB, 0.20252E+00_JPRB, 0.20155E+00_JPRB, 0.20081E+00_JPRB, 0.20117E+00_JPRB, 0.20237E+00_JPRB ,&
&  0.20073E+00_JPRB, 0.19803E+00_JPRB, 0.19698E+00_JPRB, 0.19693E+00_JPRB, 0.19645E+00_JPRB, 0.19618E+00_JPRB, 0.19690E+00_JPRB ,&
&  0.19781E+00_JPRB, 0.19757E+00_JPRB, 0.19731E+00_JPRB, 0.19639E+00_JPRB, 0.19562E+00_JPRB, 0.19500E+00_JPRB, 0.19409E+00_JPRB ,&
&  0.19117E+00_JPRB, 0.18136E+00_JPRB, 0.17475E+00_JPRB, 0.17771E+00_JPRB, 0.18485E+00_JPRB, 0.18426E+00_JPRB, 0.18423E+00_JPRB ,&
&  0.18372E+00_JPRB, 0.18246E+00_JPRB, 0.18102E+00_JPRB, 0.17963E+00_JPRB, 0.17905E+00_JPRB, 0.17932E+00_JPRB, 0.18011E+00_JPRB ,&
&  0.18094E+00_JPRB, 0.18283E+00_JPRB, 0.18506E+00_JPRB, 0.18613E+00_JPRB, 0.18718E+00_JPRB, 0.18765E+00_JPRB, 0.18782E+00_JPRB ,&
&  0.18756E+00_JPRB, 0.18670E+00_JPRB, 0.18619E+00_JPRB, 0.18566E+00_JPRB, 0.18514E+00_JPRB, 0.18420E+00_JPRB, 0.18093E+00_JPRB ,&
&  0.17559E+00_JPRB, 0.17055E+00_JPRB, 0.16443E+00_JPRB, 0.15830E+00_JPRB, 0.15266E+00_JPRB, 0.14856E+00_JPRB, 0.14529E+00_JPRB ,&
&  0.14473E+00_JPRB, 0.14812E+00_JPRB, 0.15542E+00_JPRB, 0.16459E+00_JPRB, 0.17114E+00_JPRB, 0.16088E+00_JPRB, 0.15466E+00_JPRB ,&
&  0.14902E+00_JPRB, 0.14463E+00_JPRB, 0.14194E+00_JPRB, 0.14051E+00_JPRB, 0.13958E+00_JPRB, 0.13963E+00_JPRB, 0.14023E+00_JPRB ,&
&  0.14141E+00_JPRB, 0.14274E+00_JPRB, 0.14398E+00_JPRB, 0.14471E+00_JPRB, 0.14497E+00_JPRB, 0.14479E+00_JPRB, 0.14441E+00_JPRB ,&
&  0.14476E+00_JPRB, 0.14682E+00_JPRB, 0.15069E+00_JPRB, 0.15532E+00_JPRB, 0.15922E+00_JPRB, 0.16286E+00_JPRB, 0.16796E+00_JPRB ,&
&  0.17442E+00_JPRB, 0.17990E+00_JPRB, 0.18405E+00_JPRB, 0.18831E+00_JPRB, 0.19056E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.20988E+00_JPRB, 0.21092E+00_JPRB, 0.21193E+00_JPRB, 0.21297E+00_JPRB, 0.21481E+00_JPRB, 0.21621E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.21905E+00_JPRB, 0.21776E+00_JPRB, 0.21605E+00_JPRB, 0.21472E+00_JPRB, 0.21422E+00_JPRB, 0.21369E+00_JPRB, 0.21337E+00_JPRB ,&
&  0.21426E+00_JPRB, 0.21372E+00_JPRB, 0.21126E+00_JPRB, 0.20958E+00_JPRB, 0.20668E+00_JPRB, 0.20473E+00_JPRB, 0.20435E+00_JPRB ,&
&  0.20589E+00_JPRB, 0.20575E+00_JPRB, 0.20536E+00_JPRB, 0.20457E+00_JPRB, 0.20343E+00_JPRB, 0.20288E+00_JPRB, 0.20185E+00_JPRB ,&
&  0.20129E+00_JPRB, 0.20047E+00_JPRB, 0.19574E+00_JPRB, 0.19638E+00_JPRB, 0.19593E+00_JPRB, 0.19577E+00_JPRB, 0.19519E+00_JPRB ,&
&  0.19455E+00_JPRB, 0.19359E+00_JPRB, 0.19185E+00_JPRB, 0.18993E+00_JPRB, 0.18835E+00_JPRB, 0.18753E+00_JPRB, 0.18751E+00_JPRB ,&
&  0.18789E+00_JPRB, 0.18876E+00_JPRB, 0.19025E+00_JPRB, 0.19223E+00_JPRB, 0.19307E+00_JPRB, 0.19366E+00_JPRB, 0.19453E+00_JPRB ,&
&  0.19490E+00_JPRB, 0.19499E+00_JPRB, 0.19505E+00_JPRB, 0.19512E+00_JPRB, 0.19499E+00_JPRB, 0.19434E+00_JPRB, 0.19378E+00_JPRB ,&
&  0.19237E+00_JPRB, 0.19016E+00_JPRB, 0.18871E+00_JPRB, 0.18688E+00_JPRB, 0.18184E+00_JPRB, 0.17164E+00_JPRB, 0.16592E+00_JPRB ,&
&  0.16590E+00_JPRB, 0.17043E+00_JPRB, 0.17635E+00_JPRB, 0.17936E+00_JPRB, 0.18400E+00_JPRB, 0.18418E+00_JPRB, 0.18088E+00_JPRB ,&
&  0.17337E+00_JPRB, 0.16753E+00_JPRB, 0.16341E+00_JPRB, 0.15944E+00_JPRB, 0.15697E+00_JPRB, 0.15637E+00_JPRB, 0.15689E+00_JPRB ,&
&  0.15879E+00_JPRB, 0.16165E+00_JPRB, 0.16435E+00_JPRB, 0.16581E+00_JPRB, 0.16589E+00_JPRB, 0.16448E+00_JPRB, 0.16291E+00_JPRB ,&
&  0.16235E+00_JPRB, 0.16601E+00_JPRB, 0.17388E+00_JPRB, 0.18313E+00_JPRB, 0.18706E+00_JPRB, 0.18849E+00_JPRB, 0.18994E+00_JPRB ,&
&  0.19308E+00_JPRB, 0.19749E+00_JPRB, 0.20202E+00_JPRB, 0.20520E+00_JPRB, 0.20783E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.22814E+00_JPRB, 0.22897E+00_JPRB, 0.22990E+00_JPRB, 0.23116E+00_JPRB, 0.23285E+00_JPRB, 0.23381E+00_JPRB, 0.23456E+00_JPRB ,&
&  0.23392E+00_JPRB, 0.23217E+00_JPRB, 0.23015E+00_JPRB, 0.22801E+00_JPRB, 0.22711E+00_JPRB, 0.22711E+00_JPRB, 0.22735E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22692E+00_JPRB, 0.22532E+00_JPRB, 0.22255E+00_JPRB, 0.21935E+00_JPRB, 0.21722E+00_JPRB, 0.21666E+00_JPRB ,&
&  0.21617E+00_JPRB, 0.21542E+00_JPRB, 0.21516E+00_JPRB, 0.21493E+00_JPRB, 0.21314E+00_JPRB, 0.21220E+00_JPRB, 0.21260E+00_JPRB ,&
&  0.21341E+00_JPRB, 0.21479E+00_JPRB, 0.21505E+00_JPRB, 0.21242E+00_JPRB, 0.21465E+00_JPRB, 0.21435E+00_JPRB, 0.21251E+00_JPRB ,&
&  0.21016E+00_JPRB, 0.20775E+00_JPRB, 0.20561E+00_JPRB, 0.20353E+00_JPRB, 0.20149E+00_JPRB, 0.20000E+00_JPRB, 0.19911E+00_JPRB ,&
&  0.19896E+00_JPRB, 0.19918E+00_JPRB, 0.19956E+00_JPRB, 0.20039E+00_JPRB, 0.20099E+00_JPRB, 0.20131E+00_JPRB, 0.20130E+00_JPRB ,&
&  0.20136E+00_JPRB, 0.20153E+00_JPRB, 0.20188E+00_JPRB, 0.20213E+00_JPRB, 0.20241E+00_JPRB, 0.20268E+00_JPRB, 0.20262E+00_JPRB ,&
&  0.20164E+00_JPRB, 0.19999E+00_JPRB, 0.19777E+00_JPRB, 0.19704E+00_JPRB, 0.19704E+00_JPRB, 0.19573E+00_JPRB, 0.19350E+00_JPRB ,&
&  0.19129E+00_JPRB, 0.19069E+00_JPRB, 0.19163E+00_JPRB, 0.19278E+00_JPRB, 0.19369E+00_JPRB, 0.19400E+00_JPRB, 0.19372E+00_JPRB ,&
&  0.19277E+00_JPRB, 0.19072E+00_JPRB, 0.18807E+00_JPRB, 0.18600E+00_JPRB, 0.18489E+00_JPRB, 0.18482E+00_JPRB, 0.18567E+00_JPRB ,&
&  0.18579E+00_JPRB, 0.18575E+00_JPRB, 0.18670E+00_JPRB, 0.18747E+00_JPRB, 0.18797E+00_JPRB, 0.18866E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.19195E+00_JPRB, 0.19499E+00_JPRB, 0.19869E+00_JPRB, 0.20257E+00_JPRB, 0.20527E+00_JPRB, 0.20443E+00_JPRB, 0.20484E+00_JPRB ,&
&  0.20907E+00_JPRB, 0.21433E+00_JPRB, 0.21880E+00_JPRB, 0.22241E+00_JPRB, 0.22570E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.24954E+00_JPRB, 0.25121E+00_JPRB, 0.25295E+00_JPRB, 0.25491E+00_JPRB, 0.25630E+00_JPRB, 0.25680E+00_JPRB, 0.25544E+00_JPRB ,&
&  0.25280E+00_JPRB, 0.24984E+00_JPRB, 0.24689E+00_JPRB, 0.24434E+00_JPRB, 0.24282E+00_JPRB, 0.24197E+00_JPRB, 0.24150E+00_JPRB ,&
&  0.24115E+00_JPRB, 0.24078E+00_JPRB, 0.23898E+00_JPRB, 0.23632E+00_JPRB, 0.23285E+00_JPRB, 0.22992E+00_JPRB, 0.22922E+00_JPRB ,&
&  0.22779E+00_JPRB, 0.22685E+00_JPRB, 0.22627E+00_JPRB, 0.22560E+00_JPRB, 0.22541E+00_JPRB, 0.22644E+00_JPRB, 0.22821E+00_JPRB ,&
&  0.23015E+00_JPRB, 0.23165E+00_JPRB, 0.23279E+00_JPRB, 0.23365E+00_JPRB, 0.23313E+00_JPRB, 0.23204E+00_JPRB, 0.23079E+00_JPRB ,&
&  0.22936E+00_JPRB, 0.22812E+00_JPRB, 0.22712E+00_JPRB, 0.22564E+00_JPRB, 0.22327E+00_JPRB, 0.22035E+00_JPRB, 0.21831E+00_JPRB ,&
&  0.21669E+00_JPRB, 0.21515E+00_JPRB, 0.21347E+00_JPRB, 0.21258E+00_JPRB, 0.21196E+00_JPRB, 0.21172E+00_JPRB, 0.21180E+00_JPRB ,&
&  0.21221E+00_JPRB, 0.21339E+00_JPRB, 0.21500E+00_JPRB, 0.21673E+00_JPRB, 0.21713E+00_JPRB, 0.21718E+00_JPRB, 0.21732E+00_JPRB ,&
&  0.21661E+00_JPRB, 0.21529E+00_JPRB, 0.21357E+00_JPRB, 0.21174E+00_JPRB, 0.21061E+00_JPRB, 0.20972E+00_JPRB, 0.20872E+00_JPRB ,&
&  0.20693E+00_JPRB, 0.20583E+00_JPRB, 0.20518E+00_JPRB, 0.20503E+00_JPRB, 0.20524E+00_JPRB, 0.20531E+00_JPRB, 0.20466E+00_JPRB ,&
&  0.20384E+00_JPRB, 0.20309E+00_JPRB, 0.20203E+00_JPRB, 0.20088E+00_JPRB, 0.19911E+00_JPRB, 0.19935E+00_JPRB, 0.20020E+00_JPRB ,&
&  0.20065E+00_JPRB, 0.20087E+00_JPRB, 0.20105E+00_JPRB, 0.20205E+00_JPRB, 0.20313E+00_JPRB, 0.20467E+00_JPRB, 0.20700E+00_JPRB ,&
&  0.20892E+00_JPRB, 0.21144E+00_JPRB, 0.21523E+00_JPRB, 0.21871E+00_JPRB, 0.22122E+00_JPRB, 0.22243E+00_JPRB, 0.22486E+00_JPRB ,&
&  0.23043E+00_JPRB, 0.23668E+00_JPRB, 0.24091E+00_JPRB, 0.24488E+00_JPRB, 0.24821E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.27130E+00_JPRB, 0.27629E+00_JPRB, 0.28022E+00_JPRB, 0.28248E+00_JPRB, 0.28259E+00_JPRB, 0.28131E+00_JPRB, 0.27862E+00_JPRB ,&
&  0.27480E+00_JPRB, 0.27096E+00_JPRB, 0.26706E+00_JPRB, 0.26384E+00_JPRB, 0.26183E+00_JPRB, 0.26020E+00_JPRB, 0.25895E+00_JPRB ,&
&  0.25724E+00_JPRB, 0.25515E+00_JPRB, 0.25252E+00_JPRB, 0.25020E+00_JPRB, 0.24749E+00_JPRB, 0.24369E+00_JPRB, 0.24098E+00_JPRB ,&
&  0.23914E+00_JPRB, 0.23735E+00_JPRB, 0.23689E+00_JPRB, 0.23816E+00_JPRB, 0.24049E+00_JPRB, 0.24276E+00_JPRB, 0.24527E+00_JPRB ,&
&  0.24702E+00_JPRB, 0.24780E+00_JPRB, 0.24836E+00_JPRB, 0.24820E+00_JPRB, 0.24826E+00_JPRB, 0.24807E+00_JPRB, 0.24757E+00_JPRB ,&
&  0.24725E+00_JPRB, 0.24727E+00_JPRB, 0.24675E+00_JPRB, 0.24696E+00_JPRB, 0.24696E+00_JPRB, 0.24577E+00_JPRB, 0.24352E+00_JPRB ,&
&  0.23998E+00_JPRB, 0.23527E+00_JPRB, 0.23151E+00_JPRB, 0.22956E+00_JPRB, 0.22887E+00_JPRB, 0.22906E+00_JPRB, 0.22986E+00_JPRB ,&
&  0.23096E+00_JPRB, 0.23247E+00_JPRB, 0.23383E+00_JPRB, 0.23439E+00_JPRB, 0.23405E+00_JPRB, 0.23372E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23108E+00_JPRB, 0.23021E+00_JPRB, 0.22955E+00_JPRB, 0.22886E+00_JPRB, 0.22675E+00_JPRB, 0.22420E+00_JPRB, 0.22368E+00_JPRB ,&
&  0.22231E+00_JPRB, 0.22145E+00_JPRB, 0.22023E+00_JPRB, 0.21970E+00_JPRB, 0.21903E+00_JPRB, 0.21790E+00_JPRB, 0.21665E+00_JPRB ,&
&  0.21558E+00_JPRB, 0.21498E+00_JPRB, 0.21454E+00_JPRB, 0.21354E+00_JPRB, 0.21372E+00_JPRB, 0.21417E+00_JPRB, 0.21544E+00_JPRB ,&
&  0.21662E+00_JPRB, 0.21783E+00_JPRB, 0.21937E+00_JPRB, 0.22074E+00_JPRB, 0.22287E+00_JPRB, 0.22541E+00_JPRB, 0.22833E+00_JPRB ,&
&  0.23078E+00_JPRB, 0.23426E+00_JPRB, 0.23832E+00_JPRB, 0.24211E+00_JPRB, 0.24511E+00_JPRB, 0.24737E+00_JPRB, 0.25064E+00_JPRB ,&
&  0.25600E+00_JPRB, 0.26059E+00_JPRB, 0.26308E+00_JPRB, 0.26548E+00_JPRB, 0.26747E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.30204E+00_JPRB, 0.30698E+00_JPRB, 0.30971E+00_JPRB, 0.31086E+00_JPRB, 0.31108E+00_JPRB, 0.30939E+00_JPRB, 0.30669E+00_JPRB ,&
&  0.30264E+00_JPRB, 0.29866E+00_JPRB, 0.29426E+00_JPRB, 0.28924E+00_JPRB, 0.28476E+00_JPRB, 0.28105E+00_JPRB, 0.27792E+00_JPRB ,&
&  0.27485E+00_JPRB, 0.27206E+00_JPRB, 0.26975E+00_JPRB, 0.26734E+00_JPRB, 0.26463E+00_JPRB, 0.26128E+00_JPRB, 0.25683E+00_JPRB ,&
&  0.25425E+00_JPRB, 0.25228E+00_JPRB, 0.25331E+00_JPRB, 0.25609E+00_JPRB, 0.25821E+00_JPRB, 0.25953E+00_JPRB, 0.26050E+00_JPRB ,&
&  0.26187E+00_JPRB, 0.26299E+00_JPRB, 0.26408E+00_JPRB, 0.26658E+00_JPRB, 0.26906E+00_JPRB, 0.26930E+00_JPRB, 0.26853E+00_JPRB ,&
&  0.26849E+00_JPRB, 0.27027E+00_JPRB, 0.27180E+00_JPRB, 0.27221E+00_JPRB, 0.27125E+00_JPRB, 0.26861E+00_JPRB, 0.26466E+00_JPRB ,&
&  0.25820E+00_JPRB, 0.25347E+00_JPRB, 0.25237E+00_JPRB, 0.25337E+00_JPRB, 0.25509E+00_JPRB, 0.25636E+00_JPRB, 0.25709E+00_JPRB ,&
&  0.25719E+00_JPRB, 0.25642E+00_JPRB, 0.25509E+00_JPRB, 0.25355E+00_JPRB, 0.25161E+00_JPRB, 0.24938E+00_JPRB, 0.24801E+00_JPRB ,&
&  0.24655E+00_JPRB, 0.24526E+00_JPRB, 0.24433E+00_JPRB, 0.24372E+00_JPRB, 0.24253E+00_JPRB, 0.24093E+00_JPRB, 0.24003E+00_JPRB ,&
&  0.23880E+00_JPRB, 0.23814E+00_JPRB, 0.23717E+00_JPRB, 0.23622E+00_JPRB, 0.23497E+00_JPRB, 0.23377E+00_JPRB, 0.23348E+00_JPRB ,&
&  0.23360E+00_JPRB, 0.23246E+00_JPRB, 0.23101E+00_JPRB, 0.23075E+00_JPRB, 0.23109E+00_JPRB, 0.23175E+00_JPRB, 0.23232E+00_JPRB ,&
&  0.23294E+00_JPRB, 0.23375E+00_JPRB, 0.23554E+00_JPRB, 0.23752E+00_JPRB, 0.23973E+00_JPRB, 0.24247E+00_JPRB, 0.24565E+00_JPRB ,&
&  0.24961E+00_JPRB, 0.25441E+00_JPRB, 0.25990E+00_JPRB, 0.26486E+00_JPRB, 0.26915E+00_JPRB, 0.27251E+00_JPRB, 0.27746E+00_JPRB ,&
&  0.28346E+00_JPRB, 0.28793E+00_JPRB, 0.29119E+00_JPRB, 0.29419E+00_JPRB, 0.29736E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.33340E+00_JPRB, 0.33501E+00_JPRB, 0.33499E+00_JPRB, 0.33535E+00_JPRB, 0.33483E+00_JPRB, 0.33293E+00_JPRB, 0.32959E+00_JPRB ,&
&  0.32612E+00_JPRB, 0.32214E+00_JPRB, 0.31799E+00_JPRB, 0.31260E+00_JPRB, 0.30702E+00_JPRB, 0.30260E+00_JPRB, 0.29783E+00_JPRB ,&
&  0.29304E+00_JPRB, 0.28864E+00_JPRB, 0.28568E+00_JPRB, 0.28327E+00_JPRB, 0.28096E+00_JPRB, 0.27884E+00_JPRB, 0.27600E+00_JPRB ,&
&  0.27356E+00_JPRB, 0.27130E+00_JPRB, 0.27169E+00_JPRB, 0.27403E+00_JPRB, 0.27590E+00_JPRB, 0.27727E+00_JPRB, 0.27840E+00_JPRB ,&
&  0.27868E+00_JPRB, 0.28426E+00_JPRB, 0.27868E+00_JPRB, 0.28834E+00_JPRB, 0.29258E+00_JPRB, 0.29525E+00_JPRB, 0.29694E+00_JPRB ,&
&  0.30043E+00_JPRB, 0.30377E+00_JPRB, 0.30505E+00_JPRB, 0.30125E+00_JPRB, 0.29614E+00_JPRB, 0.29219E+00_JPRB, 0.28803E+00_JPRB ,&
&  0.28292E+00_JPRB, 0.28095E+00_JPRB, 0.28101E+00_JPRB, 0.28126E+00_JPRB, 0.28123E+00_JPRB, 0.28055E+00_JPRB, 0.27920E+00_JPRB ,&
&  0.27734E+00_JPRB, 0.27487E+00_JPRB, 0.27215E+00_JPRB, 0.26941E+00_JPRB, 0.26762E+00_JPRB, 0.26550E+00_JPRB, 0.26354E+00_JPRB ,&
&  0.26262E+00_JPRB, 0.26280E+00_JPRB, 0.26288E+00_JPRB, 0.26271E+00_JPRB, 0.26195E+00_JPRB, 0.26142E+00_JPRB, 0.26122E+00_JPRB ,&
&  0.26045E+00_JPRB, 0.25948E+00_JPRB, 0.25832E+00_JPRB, 0.25681E+00_JPRB, 0.25522E+00_JPRB, 0.25442E+00_JPRB, 0.25445E+00_JPRB ,&
&  0.25418E+00_JPRB, 0.25274E+00_JPRB, 0.25050E+00_JPRB, 0.24985E+00_JPRB, 0.24954E+00_JPRB, 0.24961E+00_JPRB, 0.24997E+00_JPRB ,&
&  0.25097E+00_JPRB, 0.25286E+00_JPRB, 0.25491E+00_JPRB, 0.25679E+00_JPRB, 0.25864E+00_JPRB, 0.26152E+00_JPRB, 0.26518E+00_JPRB ,&
&  0.26950E+00_JPRB, 0.27423E+00_JPRB, 0.27885E+00_JPRB, 0.28324E+00_JPRB, 0.28762E+00_JPRB, 0.29393E+00_JPRB, 0.30295E+00_JPRB ,&
&  0.31100E+00_JPRB, 0.31688E+00_JPRB, 0.32086E+00_JPRB, 0.32506E+00_JPRB, 0.32964E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.37005E+00_JPRB, 0.36803E+00_JPRB, 0.36459E+00_JPRB, 0.36218E+00_JPRB, 0.35917E+00_JPRB, 0.35540E+00_JPRB, 0.35152E+00_JPRB ,&
&  0.34846E+00_JPRB, 0.34533E+00_JPRB, 0.34194E+00_JPRB, 0.33719E+00_JPRB, 0.33089E+00_JPRB, 0.32494E+00_JPRB, 0.31817E+00_JPRB ,&
&  0.31195E+00_JPRB, 0.30662E+00_JPRB, 0.30307E+00_JPRB, 0.30113E+00_JPRB, 0.29945E+00_JPRB, 0.29810E+00_JPRB, 0.29652E+00_JPRB ,&
&  0.29572E+00_JPRB, 0.29559E+00_JPRB, 0.29551E+00_JPRB, 0.29577E+00_JPRB, 0.29638E+00_JPRB, 0.29890E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.30241E+00_JPRB, 0.29962E+00_JPRB, 0.30760E+00_JPRB, 0.31778E+00_JPRB, 0.32950E+00_JPRB, 0.33660E+00_JPRB, 0.34168E+00_JPRB ,&
&  0.34727E+00_JPRB, 0.34964E+00_JPRB, 0.34488E+00_JPRB, 0.33783E+00_JPRB, 0.33223E+00_JPRB, 0.32854E+00_JPRB, 0.32287E+00_JPRB ,&
&  0.31748E+00_JPRB, 0.31361E+00_JPRB, 0.31050E+00_JPRB, 0.30826E+00_JPRB, 0.30589E+00_JPRB, 0.30238E+00_JPRB, 0.29844E+00_JPRB ,&
&  0.29372E+00_JPRB, 0.29030E+00_JPRB, 0.28760E+00_JPRB, 0.28653E+00_JPRB, 0.28666E+00_JPRB, 0.28772E+00_JPRB, 0.28935E+00_JPRB ,&
&  0.29184E+00_JPRB, 0.29401E+00_JPRB, 0.29484E+00_JPRB, 0.29495E+00_JPRB, 0.29469E+00_JPRB, 0.29461E+00_JPRB, 0.29397E+00_JPRB ,&
&  0.29250E+00_JPRB, 0.29063E+00_JPRB, 0.28860E+00_JPRB, 0.28633E+00_JPRB, 0.28392E+00_JPRB, 0.28217E+00_JPRB, 0.28064E+00_JPRB ,&
&  0.27895E+00_JPRB, 0.27687E+00_JPRB, 0.27458E+00_JPRB, 0.27328E+00_JPRB, 0.27232E+00_JPRB, 0.27179E+00_JPRB, 0.27248E+00_JPRB ,&
&  0.27439E+00_JPRB, 0.27664E+00_JPRB, 0.27899E+00_JPRB, 0.28097E+00_JPRB, 0.28283E+00_JPRB, 0.28463E+00_JPRB, 0.28730E+00_JPRB ,&
&  0.29199E+00_JPRB, 0.29719E+00_JPRB, 0.30247E+00_JPRB, 0.30803E+00_JPRB, 0.31623E+00_JPRB, 0.32849E+00_JPRB, 0.34218E+00_JPRB ,&
&  0.35148E+00_JPRB, 0.35633E+00_JPRB, 0.36017E+00_JPRB, 0.36439E+00_JPRB, 0.36876E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.39900E+00_JPRB, 0.39575E+00_JPRB, 0.39233E+00_JPRB, 0.38829E+00_JPRB, 0.38320E+00_JPRB, 0.37669E+00_JPRB, 0.37187E+00_JPRB ,&
&  0.36859E+00_JPRB, 0.36453E+00_JPRB, 0.36081E+00_JPRB, 0.35610E+00_JPRB, 0.35053E+00_JPRB, 0.34415E+00_JPRB, 0.33627E+00_JPRB ,&
&  0.32939E+00_JPRB, 0.32318E+00_JPRB, 0.31920E+00_JPRB, 0.31731E+00_JPRB, 0.31727E+00_JPRB, 0.31741E+00_JPRB, 0.31604E+00_JPRB ,&
&  0.31526E+00_JPRB, 0.31515E+00_JPRB, 0.31460E+00_JPRB, 0.31454E+00_JPRB, 0.31502E+00_JPRB, 0.31797E+00_JPRB, 0.32161E+00_JPRB ,&
&  0.32657E+00_JPRB, 0.30623E+00_JPRB, 0.32182E+00_JPRB, 0.33249E+00_JPRB, 0.34508E+00_JPRB, 0.36210E+00_JPRB, 0.38503E+00_JPRB ,&
&  0.40242E+00_JPRB, 0.40168E+00_JPRB, 0.39281E+00_JPRB, 0.38111E+00_JPRB, 0.37229E+00_JPRB, 0.36441E+00_JPRB, 0.35661E+00_JPRB ,&
&  0.34937E+00_JPRB, 0.34463E+00_JPRB, 0.34065E+00_JPRB, 0.33677E+00_JPRB, 0.33123E+00_JPRB, 0.32415E+00_JPRB, 0.31726E+00_JPRB ,&
&  0.31233E+00_JPRB, 0.31024E+00_JPRB, 0.31048E+00_JPRB, 0.31192E+00_JPRB, 0.31393E+00_JPRB, 0.31758E+00_JPRB, 0.32215E+00_JPRB ,&
&  0.32656E+00_JPRB, 0.32829E+00_JPRB, 0.32917E+00_JPRB, 0.33091E+00_JPRB, 0.33243E+00_JPRB, 0.33274E+00_JPRB, 0.33164E+00_JPRB ,&
&  0.32956E+00_JPRB, 0.32621E+00_JPRB, 0.32235E+00_JPRB, 0.31792E+00_JPRB, 0.31354E+00_JPRB, 0.30909E+00_JPRB, 0.30508E+00_JPRB ,&
&  0.30170E+00_JPRB, 0.29826E+00_JPRB, 0.29483E+00_JPRB, 0.29224E+00_JPRB, 0.29147E+00_JPRB, 0.29202E+00_JPRB, 0.29347E+00_JPRB ,&
&  0.29618E+00_JPRB, 0.29902E+00_JPRB, 0.30090E+00_JPRB, 0.30296E+00_JPRB, 0.30513E+00_JPRB, 0.30746E+00_JPRB, 0.31116E+00_JPRB ,&
&  0.31664E+00_JPRB, 0.32128E+00_JPRB, 0.32598E+00_JPRB, 0.33441E+00_JPRB, 0.34849E+00_JPRB, 0.36583E+00_JPRB, 0.38086E+00_JPRB ,&
&  0.38919E+00_JPRB, 0.39078E+00_JPRB, 0.39331E+00_JPRB, 0.39513E+00_JPRB, 0.39965E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.44593E+00_JPRB, 0.44326E+00_JPRB, 0.43656E+00_JPRB, 0.42439E+00_JPRB, 0.41146E+00_JPRB, 0.40065E+00_JPRB, 0.39428E+00_JPRB ,&
&  0.39009E+00_JPRB, 0.38379E+00_JPRB, 0.37832E+00_JPRB, 0.37428E+00_JPRB, 0.36921E+00_JPRB, 0.36369E+00_JPRB, 0.35662E+00_JPRB ,&
&  0.35114E+00_JPRB, 0.34699E+00_JPRB, 0.34389E+00_JPRB, 0.34191E+00_JPRB, 0.34060E+00_JPRB, 0.33926E+00_JPRB, 0.33723E+00_JPRB ,&
&  0.33494E+00_JPRB, 0.33299E+00_JPRB, 0.33138E+00_JPRB, 0.33145E+00_JPRB, 0.33227E+00_JPRB, 0.33589E+00_JPRB, 0.34197E+00_JPRB ,&
&  0.32985E+00_JPRB, 0.30856E+00_JPRB, 0.32912E+00_JPRB, 0.35351E+00_JPRB, 0.37402E+00_JPRB, 0.40455E+00_JPRB, 0.44093E+00_JPRB ,&
&  0.45766E+00_JPRB, 0.46325E+00_JPRB, 0.45173E+00_JPRB, 0.43756E+00_JPRB, 0.42260E+00_JPRB, 0.40814E+00_JPRB, 0.39676E+00_JPRB ,&
&  0.38645E+00_JPRB, 0.37944E+00_JPRB, 0.37345E+00_JPRB, 0.36504E+00_JPRB, 0.35475E+00_JPRB, 0.34626E+00_JPRB, 0.34123E+00_JPRB ,&
&  0.33931E+00_JPRB, 0.33967E+00_JPRB, 0.34042E+00_JPRB, 0.34161E+00_JPRB, 0.34403E+00_JPRB, 0.34910E+00_JPRB, 0.35541E+00_JPRB ,&
&  0.36063E+00_JPRB, 0.36704E+00_JPRB, 0.37528E+00_JPRB, 0.38295E+00_JPRB, 0.38972E+00_JPRB, 0.39253E+00_JPRB, 0.39232E+00_JPRB ,&
&  0.38800E+00_JPRB, 0.38007E+00_JPRB, 0.37138E+00_JPRB, 0.36313E+00_JPRB, 0.35476E+00_JPRB, 0.34579E+00_JPRB, 0.33681E+00_JPRB ,&
&  0.32935E+00_JPRB, 0.32291E+00_JPRB, 0.31808E+00_JPRB, 0.31622E+00_JPRB, 0.31631E+00_JPRB, 0.31847E+00_JPRB, 0.32113E+00_JPRB ,&
&  0.32364E+00_JPRB, 0.32619E+00_JPRB, 0.32801E+00_JPRB, 0.33148E+00_JPRB, 0.33551E+00_JPRB, 0.34059E+00_JPRB, 0.34622E+00_JPRB ,&
&  0.35273E+00_JPRB, 0.35933E+00_JPRB, 0.36696E+00_JPRB, 0.38585E+00_JPRB, 0.40717E+00_JPRB, 0.42585E+00_JPRB, 0.43705E+00_JPRB ,&
&  0.43908E+00_JPRB, 0.43885E+00_JPRB, 0.44117E+00_JPRB, 0.44135E+00_JPRB, 0.44716E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.51166E+00_JPRB, 0.50665E+00_JPRB, 0.49002E+00_JPRB, 0.47279E+00_JPRB, 0.45604E+00_JPRB, 0.44031E+00_JPRB, 0.42923E+00_JPRB ,&
&  0.42167E+00_JPRB, 0.41163E+00_JPRB, 0.40310E+00_JPRB, 0.39732E+00_JPRB, 0.39089E+00_JPRB, 0.38470E+00_JPRB, 0.37765E+00_JPRB ,&
&  0.37211E+00_JPRB, 0.36893E+00_JPRB, 0.36683E+00_JPRB, 0.36436E+00_JPRB, 0.36128E+00_JPRB, 0.35715E+00_JPRB, 0.35269E+00_JPRB ,&
&  0.34818E+00_JPRB, 0.34469E+00_JPRB, 0.34136E+00_JPRB, 0.34005E+00_JPRB, 0.34294E+00_JPRB, 0.34894E+00_JPRB, 0.35484E+00_JPRB ,&
&  0.37561E+00_JPRB, 0.33249E+00_JPRB, 0.35865E+00_JPRB, 0.39665E+00_JPRB, 0.43118E+00_JPRB, 0.47370E+00_JPRB, 0.50795E+00_JPRB ,&
&  0.52027E+00_JPRB, 0.53091E+00_JPRB, 0.51839E+00_JPRB, 0.50076E+00_JPRB, 0.47639E+00_JPRB, 0.45246E+00_JPRB, 0.43295E+00_JPRB ,&
&  0.42028E+00_JPRB, 0.41103E+00_JPRB, 0.40070E+00_JPRB, 0.38916E+00_JPRB, 0.37806E+00_JPRB, 0.37017E+00_JPRB, 0.36705E+00_JPRB ,&
&  0.36746E+00_JPRB, 0.36824E+00_JPRB, 0.36840E+00_JPRB, 0.37104E+00_JPRB, 0.37654E+00_JPRB, 0.38377E+00_JPRB, 0.39268E+00_JPRB ,&
&  0.40586E+00_JPRB, 0.42405E+00_JPRB, 0.43956E+00_JPRB, 0.45273E+00_JPRB, 0.45874E+00_JPRB, 0.45664E+00_JPRB, 0.44191E+00_JPRB ,&
&  0.42336E+00_JPRB, 0.40666E+00_JPRB, 0.39528E+00_JPRB, 0.38856E+00_JPRB, 0.38339E+00_JPRB, 0.37943E+00_JPRB, 0.37486E+00_JPRB ,&
&  0.36891E+00_JPRB, 0.36273E+00_JPRB, 0.35821E+00_JPRB, 0.35542E+00_JPRB, 0.35374E+00_JPRB, 0.35417E+00_JPRB, 0.35588E+00_JPRB ,&
&  0.35798E+00_JPRB, 0.36017E+00_JPRB, 0.36298E+00_JPRB, 0.36764E+00_JPRB, 0.37344E+00_JPRB, 0.37991E+00_JPRB, 0.38546E+00_JPRB ,&
&  0.39200E+00_JPRB, 0.39994E+00_JPRB, 0.40844E+00_JPRB, 0.43872E+00_JPRB, 0.46525E+00_JPRB, 0.48034E+00_JPRB, 0.49080E+00_JPRB ,&
&  0.49613E+00_JPRB, 0.50650E+00_JPRB, 0.43512E+00_JPRB, 0.47946E+00_JPRB, 0.50382E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.59646E+00_JPRB, 0.58137E+00_JPRB, 0.55340E+00_JPRB, 0.52391E+00_JPRB, 0.49850E+00_JPRB, 0.47576E+00_JPRB, 0.45852E+00_JPRB ,&
&  0.44776E+00_JPRB, 0.43981E+00_JPRB, 0.43188E+00_JPRB, 0.42455E+00_JPRB, 0.41623E+00_JPRB, 0.40870E+00_JPRB, 0.40015E+00_JPRB ,&
&  0.39437E+00_JPRB, 0.39078E+00_JPRB, 0.38774E+00_JPRB, 0.38365E+00_JPRB, 0.37902E+00_JPRB, 0.37397E+00_JPRB, 0.36764E+00_JPRB ,&
&  0.36221E+00_JPRB, 0.35932E+00_JPRB, 0.35478E+00_JPRB, 0.35366E+00_JPRB, 0.35988E+00_JPRB, 0.36766E+00_JPRB, 0.37660E+00_JPRB ,&
&  0.40219E+00_JPRB, 0.37273E+00_JPRB, 0.40159E+00_JPRB, 0.47598E+00_JPRB, 0.52464E+00_JPRB, 0.57703E+00_JPRB, 0.59542E+00_JPRB ,&
&  0.59529E+00_JPRB, 0.59379E+00_JPRB, 0.57428E+00_JPRB, 0.55045E+00_JPRB, 0.51707E+00_JPRB, 0.48904E+00_JPRB, 0.46789E+00_JPRB ,&
&  0.44988E+00_JPRB, 0.43444E+00_JPRB, 0.42204E+00_JPRB, 0.41164E+00_JPRB, 0.40224E+00_JPRB, 0.39783E+00_JPRB, 0.39852E+00_JPRB ,&
&  0.40082E+00_JPRB, 0.40376E+00_JPRB, 0.40796E+00_JPRB, 0.41507E+00_JPRB, 0.42227E+00_JPRB, 0.43063E+00_JPRB, 0.45252E+00_JPRB ,&
&  0.48819E+00_JPRB, 0.51339E+00_JPRB, 0.53176E+00_JPRB, 0.53658E+00_JPRB, 0.52852E+00_JPRB, 0.51128E+00_JPRB, 0.48490E+00_JPRB ,&
&  0.45784E+00_JPRB, 0.43702E+00_JPRB, 0.42319E+00_JPRB, 0.41437E+00_JPRB, 0.40899E+00_JPRB, 0.40428E+00_JPRB, 0.40045E+00_JPRB ,&
&  0.39679E+00_JPRB, 0.39424E+00_JPRB, 0.39462E+00_JPRB, 0.39798E+00_JPRB, 0.40272E+00_JPRB, 0.40687E+00_JPRB, 0.40906E+00_JPRB ,&
&  0.40991E+00_JPRB, 0.41038E+00_JPRB, 0.41315E+00_JPRB, 0.41759E+00_JPRB, 0.42258E+00_JPRB, 0.42595E+00_JPRB, 0.43089E+00_JPRB ,&
&  0.43780E+00_JPRB, 0.44752E+00_JPRB, 0.46112E+00_JPRB, 0.52058E+00_JPRB, 0.54866E+00_JPRB, 0.55979E+00_JPRB, 0.56682E+00_JPRB ,&
&  0.57911E+00_JPRB, 0.58260E+00_JPRB, 0.58888E+00_JPRB, 0.52022E+00_JPRB, 0.58883E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.68416E+00_JPRB, 0.64127E+00_JPRB, 0.58672E+00_JPRB, 0.54915E+00_JPRB, 0.52266E+00_JPRB, 0.49509E+00_JPRB, 0.47562E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.45641E+00_JPRB, 0.45041E+00_JPRB, 0.44260E+00_JPRB, 0.43470E+00_JPRB, 0.42917E+00_JPRB, 0.42310E+00_JPRB ,&
&  0.41680E+00_JPRB, 0.41067E+00_JPRB, 0.40467E+00_JPRB, 0.40011E+00_JPRB, 0.39416E+00_JPRB, 0.38920E+00_JPRB, 0.38445E+00_JPRB ,&
&  0.37891E+00_JPRB, 0.37387E+00_JPRB, 0.36832E+00_JPRB, 0.37233E+00_JPRB, 0.38062E+00_JPRB, 0.38749E+00_JPRB, 0.39664E+00_JPRB ,&
&  0.42764E+00_JPRB, 0.43326E+00_JPRB, 0.51045E+00_JPRB, 0.59470E+00_JPRB, 0.63570E+00_JPRB, 0.66299E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.63109E+00_JPRB, 0.63177E+00_JPRB, 0.61624E+00_JPRB, 0.58775E+00_JPRB, 0.56748E+00_JPRB, 0.53211E+00_JPRB, 0.50096E+00_JPRB ,&
&  0.47883E+00_JPRB, 0.46430E+00_JPRB, 0.45171E+00_JPRB, 0.43800E+00_JPRB, 0.42909E+00_JPRB, 0.42653E+00_JPRB, 0.42848E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.43870E+00_JPRB, 0.44508E+00_JPRB, 0.45136E+00_JPRB, 0.45756E+00_JPRB, 0.48612E+00_JPRB, 0.54689E+00_JPRB ,&
&  0.59485E+00_JPRB, 0.62134E+00_JPRB, 0.63219E+00_JPRB, 0.63455E+00_JPRB, 0.61878E+00_JPRB, 0.59549E+00_JPRB, 0.56767E+00_JPRB ,&
&  0.53885E+00_JPRB, 0.50881E+00_JPRB, 0.48124E+00_JPRB, 0.45976E+00_JPRB, 0.44274E+00_JPRB, 0.43107E+00_JPRB, 0.42512E+00_JPRB ,&
&  0.42464E+00_JPRB, 0.42856E+00_JPRB, 0.43495E+00_JPRB, 0.44148E+00_JPRB, 0.44567E+00_JPRB, 0.44650E+00_JPRB, 0.44565E+00_JPRB ,&
&  0.44434E+00_JPRB, 0.44309E+00_JPRB, 0.44418E+00_JPRB, 0.44781E+00_JPRB, 0.45305E+00_JPRB, 0.45882E+00_JPRB, 0.46564E+00_JPRB ,&
&  0.47321E+00_JPRB, 0.48439E+00_JPRB, 0.50016E+00_JPRB, 0.55723E+00_JPRB, 0.58236E+00_JPRB, 0.60970E+00_JPRB, 0.63494E+00_JPRB ,&
&  0.66573E+00_JPRB, 0.67985E+00_JPRB, 0.66931E+00_JPRB, 0.64638E+00_JPRB, 0.66170E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.74665E+00_JPRB, 0.64691E+00_JPRB, 0.58451E+00_JPRB, 0.55398E+00_JPRB, 0.54318E+00_JPRB, 0.52563E+00_JPRB, 0.50700E+00_JPRB ,&
&  0.48979E+00_JPRB, 0.47775E+00_JPRB, 0.47055E+00_JPRB, 0.46160E+00_JPRB, 0.45219E+00_JPRB, 0.44422E+00_JPRB, 0.43577E+00_JPRB ,&
&  0.42681E+00_JPRB, 0.42122E+00_JPRB, 0.41668E+00_JPRB, 0.41346E+00_JPRB, 0.41090E+00_JPRB, 0.40573E+00_JPRB, 0.39992E+00_JPRB ,&
&  0.39548E+00_JPRB, 0.38995E+00_JPRB, 0.38915E+00_JPRB, 0.39549E+00_JPRB, 0.40094E+00_JPRB, 0.40421E+00_JPRB, 0.41786E+00_JPRB ,&
&  0.46071E+00_JPRB, 0.49315E+00_JPRB, 0.59795E+00_JPRB, 0.69433E+00_JPRB, 0.74984E+00_JPRB, 0.79815E+00_JPRB, 0.79913E+00_JPRB ,&
&  0.77440E+00_JPRB, 0.77058E+00_JPRB, 0.74685E+00_JPRB, 0.69214E+00_JPRB, 0.63648E+00_JPRB, 0.59955E+00_JPRB, 0.57270E+00_JPRB ,&
&  0.54972E+00_JPRB, 0.52471E+00_JPRB, 0.49975E+00_JPRB, 0.48015E+00_JPRB, 0.46865E+00_JPRB, 0.46467E+00_JPRB, 0.46402E+00_JPRB ,&
&  0.46565E+00_JPRB, 0.47065E+00_JPRB, 0.47427E+00_JPRB, 0.48113E+00_JPRB, 0.51694E+00_JPRB, 0.61309E+00_JPRB, 0.74771E+00_JPRB ,&
&  0.80968E+00_JPRB, 0.82687E+00_JPRB, 0.83567E+00_JPRB, 0.83097E+00_JPRB, 0.78630E+00_JPRB, 0.73843E+00_JPRB, 0.69166E+00_JPRB ,&
&  0.64733E+00_JPRB, 0.60083E+00_JPRB, 0.56170E+00_JPRB, 0.53045E+00_JPRB, 0.50572E+00_JPRB, 0.49001E+00_JPRB, 0.48292E+00_JPRB ,&
&  0.48070E+00_JPRB, 0.48011E+00_JPRB, 0.47963E+00_JPRB, 0.47690E+00_JPRB, 0.47178E+00_JPRB, 0.46926E+00_JPRB, 0.46789E+00_JPRB ,&
&  0.46696E+00_JPRB, 0.46738E+00_JPRB, 0.47190E+00_JPRB, 0.47782E+00_JPRB, 0.48677E+00_JPRB, 0.49343E+00_JPRB, 0.50249E+00_JPRB ,&
&  0.51355E+00_JPRB, 0.53558E+00_JPRB, 0.54514E+00_JPRB, 0.61641E+00_JPRB, 0.69550E+00_JPRB, 0.71902E+00_JPRB, 0.75064E+00_JPRB ,&
&  0.78211E+00_JPRB, 0.76981E+00_JPRB, 0.72543E+00_JPRB, 0.71196E+00_JPRB, 0.82298E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.61027E+00_JPRB, 0.57919E+00_JPRB, 0.56133E+00_JPRB, 0.55638E+00_JPRB, 0.55549E+00_JPRB, 0.54513E+00_JPRB, 0.52492E+00_JPRB ,&
&  0.50322E+00_JPRB, 0.48480E+00_JPRB, 0.47130E+00_JPRB, 0.46144E+00_JPRB, 0.45442E+00_JPRB, 0.44776E+00_JPRB, 0.44055E+00_JPRB ,&
&  0.43331E+00_JPRB, 0.42968E+00_JPRB, 0.42717E+00_JPRB, 0.42326E+00_JPRB, 0.41473E+00_JPRB, 0.40784E+00_JPRB, 0.40553E+00_JPRB ,&
&  0.40419E+00_JPRB, 0.40497E+00_JPRB, 0.41016E+00_JPRB, 0.41617E+00_JPRB, 0.41855E+00_JPRB, 0.42115E+00_JPRB, 0.43501E+00_JPRB ,&
&  0.48043E+00_JPRB, 0.57786E+00_JPRB, 0.65299E+00_JPRB, 0.78041E+00_JPRB, 0.94172E+00_JPRB, 0.10257E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.98060E+00_JPRB, 0.88070E+00_JPRB, 0.78598E+00_JPRB, 0.71171E+00_JPRB, 0.66806E+00_JPRB, 0.63593E+00_JPRB ,&
&  0.60530E+00_JPRB, 0.57476E+00_JPRB, 0.54879E+00_JPRB, 0.52961E+00_JPRB, 0.51648E+00_JPRB, 0.51169E+00_JPRB, 0.51146E+00_JPRB ,&
&  0.51105E+00_JPRB, 0.50960E+00_JPRB, 0.51121E+00_JPRB, 0.54310E+00_JPRB, 0.69376E+00_JPRB, 0.90785E+00_JPRB, 0.10968E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.12041E+01_JPRB, 0.11865E+01_JPRB, 0.11165E+01_JPRB, 0.10097E+01_JPRB, 0.90481E+00_JPRB, 0.79619E+00_JPRB ,&
&  0.71799E+00_JPRB, 0.66181E+00_JPRB, 0.62158E+00_JPRB, 0.58799E+00_JPRB, 0.55809E+00_JPRB, 0.53583E+00_JPRB, 0.52144E+00_JPRB ,&
&  0.51163E+00_JPRB, 0.50328E+00_JPRB, 0.49517E+00_JPRB, 0.48727E+00_JPRB, 0.48075E+00_JPRB, 0.47662E+00_JPRB, 0.47825E+00_JPRB ,&
&  0.48017E+00_JPRB, 0.48102E+00_JPRB, 0.49577E+00_JPRB, 0.51257E+00_JPRB, 0.52986E+00_JPRB, 0.54139E+00_JPRB, 0.55011E+00_JPRB ,&
&  0.56160E+00_JPRB, 0.59833E+00_JPRB, 0.59577E+00_JPRB, 0.63070E+00_JPRB, 0.74814E+00_JPRB, 0.81357E+00_JPRB, 0.81255E+00_JPRB ,&
&  0.78519E+00_JPRB, 0.74002E+00_JPRB, 0.70033E+00_JPRB, 0.67489E+00_JPRB, 0.65791E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.60657E+00_JPRB, 0.58458E+00_JPRB, 0.57494E+00_JPRB, 0.56679E+00_JPRB, 0.55860E+00_JPRB, 0.54522E+00_JPRB, 0.52518E+00_JPRB ,&
&  0.50989E+00_JPRB, 0.49696E+00_JPRB, 0.48220E+00_JPRB, 0.46970E+00_JPRB, 0.46109E+00_JPRB, 0.45445E+00_JPRB, 0.44673E+00_JPRB ,&
&  0.44147E+00_JPRB, 0.43776E+00_JPRB, 0.42758E+00_JPRB, 0.41765E+00_JPRB, 0.41345E+00_JPRB, 0.41277E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.42039E+00_JPRB, 0.42547E+00_JPRB, 0.42925E+00_JPRB, 0.43230E+00_JPRB, 0.43225E+00_JPRB, 0.43661E+00_JPRB, 0.45697E+00_JPRB ,&
&  0.51657E+00_JPRB, 0.62646E+00_JPRB, 0.83487E+00_JPRB, 0.92635E+00_JPRB, 0.10263E+01_JPRB, 0.11208E+01_JPRB, 0.12294E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.10870E+01_JPRB, 0.94976E+00_JPRB, 0.85955E+00_JPRB, 0.79255E+00_JPRB, 0.74411E+00_JPRB, 0.69360E+00_JPRB ,&
&  0.65158E+00_JPRB, 0.62339E+00_JPRB, 0.59577E+00_JPRB, 0.57106E+00_JPRB, 0.56083E+00_JPRB, 0.55797E+00_JPRB, 0.55504E+00_JPRB ,&
&  0.55066E+00_JPRB, 0.54711E+00_JPRB, 0.56633E+00_JPRB, 0.66654E+00_JPRB, 0.91724E+00_JPRB, 0.11344E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.16838E+01_JPRB, 0.16828E+01_JPRB, 0.15546E+01_JPRB, 0.13418E+01_JPRB, 0.11179E+01_JPRB, 0.92806E+00_JPRB, 0.81028E+00_JPRB ,&
&  0.73587E+00_JPRB, 0.67356E+00_JPRB, 0.63493E+00_JPRB, 0.61042E+00_JPRB, 0.59151E+00_JPRB, 0.57534E+00_JPRB, 0.55579E+00_JPRB ,&
&  0.53434E+00_JPRB, 0.51762E+00_JPRB, 0.50661E+00_JPRB, 0.49791E+00_JPRB, 0.49286E+00_JPRB, 0.49157E+00_JPRB, 0.49474E+00_JPRB ,&
&  0.50260E+00_JPRB, 0.52205E+00_JPRB, 0.56026E+00_JPRB, 0.56530E+00_JPRB, 0.58610E+00_JPRB, 0.61448E+00_JPRB, 0.63656E+00_JPRB ,&
&  0.62830E+00_JPRB, 0.63254E+00_JPRB, 0.61751E+00_JPRB, 0.63690E+00_JPRB, 0.78242E+00_JPRB, 0.86138E+00_JPRB, 0.80396E+00_JPRB ,&
&  0.76313E+00_JPRB, 0.72270E+00_JPRB, 0.68413E+00_JPRB, 0.64855E+00_JPRB, 0.63983E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.61590E+00_JPRB, 0.59482E+00_JPRB, 0.57910E+00_JPRB, 0.56178E+00_JPRB, 0.54946E+00_JPRB, 0.54090E+00_JPRB, 0.52891E+00_JPRB ,&
&  0.51654E+00_JPRB, 0.50414E+00_JPRB, 0.49126E+00_JPRB, 0.47888E+00_JPRB, 0.46856E+00_JPRB, 0.45641E+00_JPRB, 0.44656E+00_JPRB ,&
&  0.43733E+00_JPRB, 0.42585E+00_JPRB, 0.41838E+00_JPRB, 0.41620E+00_JPRB, 0.41995E+00_JPRB, 0.42512E+00_JPRB, 0.43172E+00_JPRB ,&
&  0.43969E+00_JPRB, 0.44495E+00_JPRB, 0.44601E+00_JPRB, 0.44827E+00_JPRB, 0.44654E+00_JPRB, 0.45013E+00_JPRB, 0.47383E+00_JPRB ,&
&  0.53289E+00_JPRB, 0.50925E+00_JPRB, 0.81688E+00_JPRB, 0.10638E+01_JPRB, 0.12005E+01_JPRB, 0.12902E+01_JPRB, 0.13262E+01_JPRB ,&
&  0.13403E+01_JPRB, 0.11977E+01_JPRB, 0.10451E+01_JPRB, 0.94764E+00_JPRB, 0.87054E+00_JPRB, 0.81122E+00_JPRB, 0.76361E+00_JPRB ,&
&  0.71719E+00_JPRB, 0.68229E+00_JPRB, 0.65462E+00_JPRB, 0.61758E+00_JPRB, 0.58952E+00_JPRB, 0.57532E+00_JPRB, 0.57090E+00_JPRB ,&
&  0.56558E+00_JPRB, 0.57703E+00_JPRB, 0.63414E+00_JPRB, 0.79569E+00_JPRB, 0.11346E+01_JPRB, 0.14128E+01_JPRB, 0.15381E+01_JPRB ,&
&  0.16678E+01_JPRB, 0.17808E+01_JPRB, 0.16455E+01_JPRB, 0.13953E+01_JPRB, 0.11486E+01_JPRB, 0.97025E+00_JPRB, 0.82726E+00_JPRB ,&
&  0.74446E+00_JPRB, 0.69743E+00_JPRB, 0.67045E+00_JPRB, 0.64968E+00_JPRB, 0.62721E+00_JPRB, 0.59859E+00_JPRB, 0.56671E+00_JPRB ,&
&  0.54272E+00_JPRB, 0.53001E+00_JPRB, 0.52197E+00_JPRB, 0.51912E+00_JPRB, 0.51852E+00_JPRB, 0.52007E+00_JPRB, 0.52729E+00_JPRB ,&
&  0.54276E+00_JPRB, 0.57117E+00_JPRB, 0.59277E+00_JPRB, 0.60265E+00_JPRB, 0.62174E+00_JPRB, 0.64489E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.68453E+00_JPRB, 0.67319E+00_JPRB, 0.65325E+00_JPRB, 0.67650E+00_JPRB, 0.73073E+00_JPRB, 0.80335E+00_JPRB, 0.77478E+00_JPRB ,&
&  0.75181E+00_JPRB, 0.71608E+00_JPRB, 0.67199E+00_JPRB, 0.65064E+00_JPRB, 0.64105E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.62073E+00_JPRB, 0.60522E+00_JPRB, 0.58644E+00_JPRB, 0.56811E+00_JPRB, 0.56073E+00_JPRB, 0.55555E+00_JPRB, 0.54250E+00_JPRB ,&
&  0.53079E+00_JPRB, 0.52180E+00_JPRB, 0.51019E+00_JPRB, 0.49546E+00_JPRB, 0.47828E+00_JPRB, 0.45652E+00_JPRB, 0.43651E+00_JPRB ,&
&  0.42708E+00_JPRB, 0.41944E+00_JPRB, 0.42239E+00_JPRB, 0.42801E+00_JPRB, 0.43660E+00_JPRB, 0.44349E+00_JPRB, 0.45126E+00_JPRB ,&
&  0.45691E+00_JPRB, 0.45938E+00_JPRB, 0.46319E+00_JPRB, 0.46645E+00_JPRB, 0.46757E+00_JPRB, 0.47424E+00_JPRB, 0.50030E+00_JPRB ,&
&  0.56508E+00_JPRB, 0.83424E+00_JPRB, 0.72139E+00_JPRB, 0.13082E+01_JPRB, 0.14836E+01_JPRB, 0.15454E+01_JPRB, 0.14317E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.13192E+01_JPRB, 0.11532E+01_JPRB, 0.10220E+01_JPRB, 0.92188E+00_JPRB, 0.85555E+00_JPRB, 0.80958E+00_JPRB ,&
&  0.77803E+00_JPRB, 0.74067E+00_JPRB, 0.69575E+00_JPRB, 0.65045E+00_JPRB, 0.62124E+00_JPRB, 0.60915E+00_JPRB, 0.60726E+00_JPRB ,&
&  0.60651E+00_JPRB, 0.63786E+00_JPRB, 0.74799E+00_JPRB, 0.10429E+01_JPRB, 0.13965E+01_JPRB, 0.15821E+01_JPRB, 0.15339E+01_JPRB ,&
&  0.14462E+01_JPRB, 0.16126E+01_JPRB, 0.15079E+01_JPRB, 0.12607E+01_JPRB, 0.10693E+01_JPRB, 0.91691E+00_JPRB, 0.80560E+00_JPRB ,&
&  0.73797E+00_JPRB, 0.69660E+00_JPRB, 0.67039E+00_JPRB, 0.65036E+00_JPRB, 0.62890E+00_JPRB, 0.60527E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.56927E+00_JPRB, 0.55813E+00_JPRB, 0.55091E+00_JPRB, 0.55158E+00_JPRB, 0.55179E+00_JPRB, 0.55501E+00_JPRB, 0.56277E+00_JPRB ,&
&  0.57756E+00_JPRB, 0.60626E+00_JPRB, 0.61603E+00_JPRB, 0.63075E+00_JPRB, 0.63913E+00_JPRB, 0.64728E+00_JPRB, 0.65680E+00_JPRB ,&
&  0.69626E+00_JPRB, 0.70065E+00_JPRB, 0.69296E+00_JPRB, 0.70523E+00_JPRB, 0.74502E+00_JPRB, 0.80237E+00_JPRB, 0.75494E+00_JPRB ,&
&  0.73780E+00_JPRB, 0.70699E+00_JPRB, 0.67601E+00_JPRB, 0.65476E+00_JPRB, 0.63988E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.62463E+00_JPRB, 0.60631E+00_JPRB, 0.59264E+00_JPRB, 0.58661E+00_JPRB, 0.58011E+00_JPRB, 0.57341E+00_JPRB, 0.55904E+00_JPRB ,&
&  0.54347E+00_JPRB, 0.53398E+00_JPRB, 0.52047E+00_JPRB, 0.50766E+00_JPRB, 0.48228E+00_JPRB, 0.45067E+00_JPRB, 0.43795E+00_JPRB ,&
&  0.43078E+00_JPRB, 0.43188E+00_JPRB, 0.44161E+00_JPRB, 0.44581E+00_JPRB, 0.45045E+00_JPRB, 0.45968E+00_JPRB, 0.46893E+00_JPRB ,&
&  0.47552E+00_JPRB, 0.47584E+00_JPRB, 0.48074E+00_JPRB, 0.48737E+00_JPRB, 0.49379E+00_JPRB, 0.51481E+00_JPRB, 0.54390E+00_JPRB ,&
&  0.63221E+00_JPRB, 0.83420E+00_JPRB, 0.73836E+00_JPRB, 0.15286E+01_JPRB, 0.17556E+01_JPRB, 0.17407E+01_JPRB, 0.15897E+01_JPRB ,&
&  0.15257E+01_JPRB, 0.14966E+01_JPRB, 0.12478E+01_JPRB, 0.10727E+01_JPRB, 0.94717E+00_JPRB, 0.88001E+00_JPRB, 0.84014E+00_JPRB ,&
&  0.81135E+00_JPRB, 0.77914E+00_JPRB, 0.73448E+00_JPRB, 0.69288E+00_JPRB, 0.67231E+00_JPRB, 0.66276E+00_JPRB, 0.65652E+00_JPRB ,&
&  0.65765E+00_JPRB, 0.72798E+00_JPRB, 0.91797E+00_JPRB, 0.12858E+01_JPRB, 0.16401E+01_JPRB, 0.16603E+01_JPRB, 0.14805E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.14686E+01_JPRB, 0.13343E+01_JPRB, 0.11041E+01_JPRB, 0.94414E+00_JPRB, 0.85260E+00_JPRB, 0.76324E+00_JPRB ,&
&  0.70734E+00_JPRB, 0.67053E+00_JPRB, 0.65246E+00_JPRB, 0.63883E+00_JPRB, 0.62704E+00_JPRB, 0.61821E+00_JPRB, 0.60804E+00_JPRB ,&
&  0.59849E+00_JPRB, 0.58666E+00_JPRB, 0.57972E+00_JPRB, 0.57210E+00_JPRB, 0.57477E+00_JPRB, 0.57949E+00_JPRB, 0.58639E+00_JPRB ,&
&  0.60926E+00_JPRB, 0.64746E+00_JPRB, 0.64806E+00_JPRB, 0.66296E+00_JPRB, 0.66501E+00_JPRB, 0.65847E+00_JPRB, 0.66998E+00_JPRB ,&
&  0.70023E+00_JPRB, 0.70790E+00_JPRB, 0.72004E+00_JPRB, 0.71328E+00_JPRB, 0.73329E+00_JPRB, 0.76029E+00_JPRB, 0.73739E+00_JPRB ,&
&  0.73336E+00_JPRB, 0.71381E+00_JPRB, 0.69084E+00_JPRB, 0.66225E+00_JPRB, 0.64324E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.61304E+00_JPRB, 0.59372E+00_JPRB, 0.58773E+00_JPRB, 0.58665E+00_JPRB, 0.58257E+00_JPRB, 0.57264E+00_JPRB, 0.55511E+00_JPRB ,&
&  0.53408E+00_JPRB, 0.51912E+00_JPRB, 0.50293E+00_JPRB, 0.48295E+00_JPRB, 0.45341E+00_JPRB, 0.42606E+00_JPRB, 0.42531E+00_JPRB ,&
&  0.44189E+00_JPRB, 0.45123E+00_JPRB, 0.45962E+00_JPRB, 0.46862E+00_JPRB, 0.47895E+00_JPRB, 0.49153E+00_JPRB, 0.50128E+00_JPRB ,&
&  0.50429E+00_JPRB, 0.50917E+00_JPRB, 0.51960E+00_JPRB, 0.53296E+00_JPRB, 0.55099E+00_JPRB, 0.58055E+00_JPRB, 0.62546E+00_JPRB ,&
&  0.74768E+00_JPRB, 0.89889E+00_JPRB, 0.91084E+00_JPRB, 0.15511E+01_JPRB, 0.18450E+01_JPRB, 0.17206E+01_JPRB, 0.16602E+01_JPRB ,&
&  0.15470E+01_JPRB, 0.13294E+01_JPRB, 0.12453E+01_JPRB, 0.10594E+01_JPRB, 0.94882E+00_JPRB, 0.89419E+00_JPRB, 0.86206E+00_JPRB ,&
&  0.84102E+00_JPRB, 0.81442E+00_JPRB, 0.77657E+00_JPRB, 0.75850E+00_JPRB, 0.74891E+00_JPRB, 0.72979E+00_JPRB, 0.71682E+00_JPRB ,&
&  0.74669E+00_JPRB, 0.87425E+00_JPRB, 0.11589E+01_JPRB, 0.16009E+01_JPRB, 0.17765E+01_JPRB, 0.16146E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.12407E+01_JPRB, 0.11231E+01_JPRB, 0.95685E+00_JPRB, 0.81973E+00_JPRB, 0.80740E+00_JPRB, 0.72829E+00_JPRB ,&
&  0.68255E+00_JPRB, 0.65234E+00_JPRB, 0.63832E+00_JPRB, 0.62833E+00_JPRB, 0.62370E+00_JPRB, 0.61891E+00_JPRB, 0.61155E+00_JPRB ,&
&  0.60696E+00_JPRB, 0.60647E+00_JPRB, 0.59974E+00_JPRB, 0.59940E+00_JPRB, 0.60262E+00_JPRB, 0.61010E+00_JPRB, 0.62657E+00_JPRB ,&
&  0.66285E+00_JPRB, 0.70131E+00_JPRB, 0.70698E+00_JPRB, 0.71642E+00_JPRB, 0.71061E+00_JPRB, 0.70232E+00_JPRB, 0.70134E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.73790E+00_JPRB, 0.74749E+00_JPRB, 0.74443E+00_JPRB, 0.74794E+00_JPRB, 0.75108E+00_JPRB, 0.74806E+00_JPRB ,&
&  0.75515E+00_JPRB, 0.74292E+00_JPRB, 0.72550E+00_JPRB, 0.66467E+00_JPRB, 0.64071E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.58123E+00_JPRB, 0.56638E+00_JPRB, 0.55204E+00_JPRB, 0.54802E+00_JPRB, 0.54351E+00_JPRB, 0.52813E+00_JPRB, 0.50861E+00_JPRB ,&
&  0.48615E+00_JPRB, 0.46941E+00_JPRB, 0.44787E+00_JPRB, 0.42979E+00_JPRB, 0.42886E+00_JPRB, 0.44181E+00_JPRB, 0.46761E+00_JPRB ,&
&  0.49327E+00_JPRB, 0.50788E+00_JPRB, 0.51901E+00_JPRB, 0.53017E+00_JPRB, 0.54077E+00_JPRB, 0.54673E+00_JPRB, 0.54745E+00_JPRB ,&
&  0.55375E+00_JPRB, 0.56843E+00_JPRB, 0.59019E+00_JPRB, 0.61538E+00_JPRB, 0.64778E+00_JPRB, 0.69767E+00_JPRB, 0.78396E+00_JPRB ,&
&  0.96377E+00_JPRB, 0.10480E+01_JPRB, 0.59090E+00_JPRB, 0.11278E+01_JPRB, 0.15853E+01_JPRB, 0.15528E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.13156E+01_JPRB, 0.11151E+01_JPRB, 0.10743E+01_JPRB, 0.10234E+01_JPRB, 0.93614E+00_JPRB, 0.90285E+00_JPRB, 0.89194E+00_JPRB ,&
&  0.88935E+00_JPRB, 0.86977E+00_JPRB, 0.85720E+00_JPRB, 0.86019E+00_JPRB, 0.84157E+00_JPRB, 0.80652E+00_JPRB, 0.80967E+00_JPRB ,&
&  0.88454E+00_JPRB, 0.10799E+01_JPRB, 0.14963E+01_JPRB, 0.17582E+01_JPRB, 0.18643E+01_JPRB, 0.15006E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.10035E+01_JPRB, 0.10302E+01_JPRB, 0.96797E+00_JPRB, 0.85961E+00_JPRB, 0.75689E+00_JPRB, 0.71378E+00_JPRB, 0.70965E+00_JPRB ,&
&  0.66836E+00_JPRB, 0.63639E+00_JPRB, 0.61738E+00_JPRB, 0.60370E+00_JPRB, 0.59639E+00_JPRB, 0.59324E+00_JPRB, 0.59459E+00_JPRB ,&
&  0.60415E+00_JPRB, 0.60880E+00_JPRB, 0.61562E+00_JPRB, 0.62665E+00_JPRB, 0.63883E+00_JPRB, 0.65579E+00_JPRB, 0.68644E+00_JPRB ,&
&  0.72859E+00_JPRB, 0.76120E+00_JPRB, 0.78805E+00_JPRB, 0.77842E+00_JPRB, 0.76137E+00_JPRB, 0.75590E+00_JPRB, 0.77090E+00_JPRB ,&
&  0.78074E+00_JPRB, 0.80569E+00_JPRB, 0.74662E+00_JPRB, 0.73665E+00_JPRB, 0.76637E+00_JPRB, 0.76802E+00_JPRB, 0.80110E+00_JPRB ,&
&  0.84899E+00_JPRB, 0.91927E+00_JPRB, 0.92394E+00_JPRB, 0.65473E+00_JPRB, 0.61174E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.53547E+00_JPRB, 0.51330E+00_JPRB, 0.48686E+00_JPRB, 0.47733E+00_JPRB, 0.48258E+00_JPRB, 0.47647E+00_JPRB, 0.47123E+00_JPRB ,&
&  0.46627E+00_JPRB, 0.46213E+00_JPRB, 0.45621E+00_JPRB, 0.46643E+00_JPRB, 0.49427E+00_JPRB, 0.53324E+00_JPRB, 0.56736E+00_JPRB ,&
&  0.59235E+00_JPRB, 0.60362E+00_JPRB, 0.60708E+00_JPRB, 0.61244E+00_JPRB, 0.61661E+00_JPRB, 0.62084E+00_JPRB, 0.62713E+00_JPRB ,&
&  0.64876E+00_JPRB, 0.68333E+00_JPRB, 0.72775E+00_JPRB, 0.77718E+00_JPRB, 0.84024E+00_JPRB, 0.93811E+00_JPRB, 0.10936E+01_JPRB ,&
&  0.13460E+01_JPRB, 0.10219E+01_JPRB, 0.70046E+00_JPRB, 0.14205E+01_JPRB, 0.15774E+01_JPRB, 0.15263E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.10656E+01_JPRB, 0.99360E+00_JPRB, 0.10199E+01_JPRB, 0.94099E+00_JPRB, 0.92553E+00_JPRB, 0.93635E+00_JPRB ,&
&  0.94549E+00_JPRB, 0.95505E+00_JPRB, 0.96513E+00_JPRB, 0.95626E+00_JPRB, 0.92595E+00_JPRB, 0.91478E+00_JPRB, 0.95849E+00_JPRB ,&
&  0.11056E+01_JPRB, 0.14050E+01_JPRB, 0.19132E+01_JPRB, 0.19272E+01_JPRB, 0.19669E+01_JPRB, 0.14363E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.92084E+00_JPRB, 0.90271E+00_JPRB, 0.87595E+00_JPRB, 0.81719E+00_JPRB, 0.73986E+00_JPRB, 0.68099E+00_JPRB, 0.68811E+00_JPRB ,&
&  0.64186E+00_JPRB, 0.60992E+00_JPRB, 0.58396E+00_JPRB, 0.57171E+00_JPRB, 0.57244E+00_JPRB, 0.57858E+00_JPRB, 0.58228E+00_JPRB ,&
&  0.59381E+00_JPRB, 0.60177E+00_JPRB, 0.62125E+00_JPRB, 0.64382E+00_JPRB, 0.66819E+00_JPRB, 0.69857E+00_JPRB, 0.74153E+00_JPRB ,&
&  0.77843E+00_JPRB, 0.80505E+00_JPRB, 0.82186E+00_JPRB, 0.82785E+00_JPRB, 0.80908E+00_JPRB, 0.78547E+00_JPRB, 0.79616E+00_JPRB ,&
&  0.78761E+00_JPRB, 0.78902E+00_JPRB, 0.76924E+00_JPRB, 0.78557E+00_JPRB, 0.79384E+00_JPRB, 0.80367E+00_JPRB, 0.83404E+00_JPRB ,&
&  0.87261E+00_JPRB, 0.87103E+00_JPRB, 0.78688E+00_JPRB, 0.60174E+00_JPRB, 0.55722E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.50878E+00_JPRB, 0.48462E+00_JPRB, 0.47476E+00_JPRB, 0.47438E+00_JPRB, 0.48204E+00_JPRB, 0.48666E+00_JPRB, 0.48655E+00_JPRB ,&
&  0.48956E+00_JPRB, 0.50111E+00_JPRB, 0.51558E+00_JPRB, 0.55914E+00_JPRB, 0.60418E+00_JPRB, 0.63760E+00_JPRB, 0.65852E+00_JPRB ,&
&  0.67743E+00_JPRB, 0.68762E+00_JPRB, 0.69252E+00_JPRB, 0.69959E+00_JPRB, 0.70578E+00_JPRB, 0.72075E+00_JPRB, 0.75119E+00_JPRB ,&
&  0.80024E+00_JPRB, 0.85371E+00_JPRB, 0.92010E+00_JPRB, 0.10042E+01_JPRB, 0.11103E+01_JPRB, 0.12133E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.10028E+01_JPRB, 0.57590E+00_JPRB, 0.13859E+01_JPRB, 0.16341E+01_JPRB, 0.14408E+01_JPRB, 0.13481E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.10238E+01_JPRB, 0.98562E+00_JPRB, 0.10202E+01_JPRB, 0.96630E+00_JPRB, 0.96617E+00_JPRB, 0.97744E+00_JPRB ,&
&  0.99937E+00_JPRB, 0.10326E+01_JPRB, 0.10397E+01_JPRB, 0.10298E+01_JPRB, 0.10306E+01_JPRB, 0.10698E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.18317E+01_JPRB, 0.22006E+01_JPRB, 0.19661E+01_JPRB, 0.19996E+01_JPRB, 0.16116E+01_JPRB, 0.12536E+01_JPRB ,&
&  0.97769E+00_JPRB, 0.81915E+00_JPRB, 0.79962E+00_JPRB, 0.80700E+00_JPRB, 0.74884E+00_JPRB, 0.67434E+00_JPRB, 0.66759E+00_JPRB ,&
&  0.61562E+00_JPRB, 0.57966E+00_JPRB, 0.56316E+00_JPRB, 0.56173E+00_JPRB, 0.56890E+00_JPRB, 0.58008E+00_JPRB, 0.59165E+00_JPRB ,&
&  0.59622E+00_JPRB, 0.60878E+00_JPRB, 0.62486E+00_JPRB, 0.65479E+00_JPRB, 0.68874E+00_JPRB, 0.73687E+00_JPRB, 0.77918E+00_JPRB ,&
&  0.80652E+00_JPRB, 0.83101E+00_JPRB, 0.85525E+00_JPRB, 0.87335E+00_JPRB, 0.88927E+00_JPRB, 0.86158E+00_JPRB, 0.83638E+00_JPRB ,&
&  0.83096E+00_JPRB, 0.83128E+00_JPRB, 0.84372E+00_JPRB, 0.86546E+00_JPRB, 0.83191E+00_JPRB, 0.81280E+00_JPRB, 0.83350E+00_JPRB ,&
&  0.82941E+00_JPRB, 0.78314E+00_JPRB, 0.67849E+00_JPRB, 0.57001E+00_JPRB, 0.53157E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.54970E+00_JPRB, 0.53711E+00_JPRB, 0.52329E+00_JPRB, 0.52667E+00_JPRB, 0.53072E+00_JPRB, 0.54984E+00_JPRB, 0.55124E+00_JPRB ,&
&  0.56572E+00_JPRB, 0.60777E+00_JPRB, 0.65850E+00_JPRB, 0.68589E+00_JPRB, 0.70966E+00_JPRB, 0.73321E+00_JPRB, 0.75250E+00_JPRB ,&
&  0.76716E+00_JPRB, 0.78132E+00_JPRB, 0.79551E+00_JPRB, 0.81198E+00_JPRB, 0.83649E+00_JPRB, 0.87186E+00_JPRB, 0.92616E+00_JPRB ,&
&  0.98066E+00_JPRB, 0.10410E+01_JPRB, 0.11170E+01_JPRB, 0.12130E+01_JPRB, 0.13171E+01_JPRB, 0.13767E+01_JPRB, 0.14223E+01_JPRB ,&
&  0.89623E+00_JPRB, 0.13390E+01_JPRB, 0.15726E+01_JPRB, 0.14687E+01_JPRB, 0.13351E+01_JPRB, 0.12566E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.10226E+01_JPRB, 0.10007E+01_JPRB, 0.10721E+01_JPRB, 0.10164E+01_JPRB, 0.10219E+01_JPRB, 0.10356E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.11255E+01_JPRB, 0.11428E+01_JPRB, 0.11736E+01_JPRB, 0.12262E+01_JPRB, 0.13440E+01_JPRB, 0.15414E+01_JPRB ,&
&  0.18101E+01_JPRB, 0.20876E+01_JPRB, 0.22347E+01_JPRB, 0.19183E+01_JPRB, 0.19508E+01_JPRB, 0.18015E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.99971E+00_JPRB, 0.78980E+00_JPRB, 0.80717E+00_JPRB, 0.82081E+00_JPRB, 0.75351E+00_JPRB, 0.69674E+00_JPRB, 0.65104E+00_JPRB ,&
&  0.60285E+00_JPRB, 0.57384E+00_JPRB, 0.56887E+00_JPRB, 0.56628E+00_JPRB, 0.57371E+00_JPRB, 0.58127E+00_JPRB, 0.59584E+00_JPRB ,&
&  0.60389E+00_JPRB, 0.62195E+00_JPRB, 0.64695E+00_JPRB, 0.67457E+00_JPRB, 0.70912E+00_JPRB, 0.74806E+00_JPRB, 0.79073E+00_JPRB ,&
&  0.82207E+00_JPRB, 0.85141E+00_JPRB, 0.87842E+00_JPRB, 0.90011E+00_JPRB, 0.90986E+00_JPRB, 0.91167E+00_JPRB, 0.88880E+00_JPRB ,&
&  0.86972E+00_JPRB, 0.87851E+00_JPRB, 0.88388E+00_JPRB, 0.87630E+00_JPRB, 0.84556E+00_JPRB, 0.81743E+00_JPRB, 0.79184E+00_JPRB ,&
&  0.74096E+00_JPRB, 0.66027E+00_JPRB, 0.58631E+00_JPRB, 0.56550E+00_JPRB, 0.55320E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.62607E+00_JPRB, 0.63331E+00_JPRB, 0.63399E+00_JPRB, 0.66841E+00_JPRB, 0.67557E+00_JPRB, 0.68933E+00_JPRB, 0.70192E+00_JPRB ,&
&  0.71250E+00_JPRB, 0.73282E+00_JPRB, 0.75707E+00_JPRB, 0.78384E+00_JPRB, 0.80594E+00_JPRB, 0.82327E+00_JPRB, 0.83826E+00_JPRB ,&
&  0.85641E+00_JPRB, 0.87773E+00_JPRB, 0.90415E+00_JPRB, 0.93693E+00_JPRB, 0.98677E+00_JPRB, 0.10419E+01_JPRB, 0.11004E+01_JPRB ,&
&  0.11565E+01_JPRB, 0.12339E+01_JPRB, 0.13291E+01_JPRB, 0.14424E+01_JPRB, 0.15607E+01_JPRB, 0.16463E+01_JPRB, 0.16914E+01_JPRB ,&
&  0.13498E+01_JPRB, 0.16032E+01_JPRB, 0.15543E+01_JPRB, 0.14106E+01_JPRB, 0.12918E+01_JPRB, 0.12260E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.11203E+01_JPRB, 0.10463E+01_JPRB, 0.10253E+01_JPRB, 0.10410E+01_JPRB, 0.10684E+01_JPRB, 0.10827E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.12557E+01_JPRB, 0.13228E+01_JPRB, 0.13891E+01_JPRB, 0.14813E+01_JPRB, 0.16208E+01_JPRB, 0.17805E+01_JPRB ,&
&  0.19479E+01_JPRB, 0.20713E+01_JPRB, 0.21652E+01_JPRB, 0.19015E+01_JPRB, 0.17398E+01_JPRB, 0.15965E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.84999E+00_JPRB, 0.76152E+00_JPRB, 0.81442E+00_JPRB, 0.85545E+00_JPRB, 0.78553E+00_JPRB, 0.71081E+00_JPRB, 0.65069E+00_JPRB ,&
&  0.60341E+00_JPRB, 0.57116E+00_JPRB, 0.57394E+00_JPRB, 0.56337E+00_JPRB, 0.56355E+00_JPRB, 0.58126E+00_JPRB, 0.59946E+00_JPRB ,&
&  0.63795E+00_JPRB, 0.65437E+00_JPRB, 0.69125E+00_JPRB, 0.72267E+00_JPRB, 0.75848E+00_JPRB, 0.79171E+00_JPRB, 0.82341E+00_JPRB ,&
&  0.84853E+00_JPRB, 0.85852E+00_JPRB, 0.86067E+00_JPRB, 0.88578E+00_JPRB, 0.89294E+00_JPRB, 0.90743E+00_JPRB, 0.93145E+00_JPRB ,&
&  0.95896E+00_JPRB, 0.97672E+00_JPRB, 0.96126E+00_JPRB, 0.94361E+00_JPRB, 0.91066E+00_JPRB, 0.87396E+00_JPRB, 0.67104E+00_JPRB ,&
&  0.63928E+00_JPRB, 0.60380E+00_JPRB, 0.61944E+00_JPRB, 0.63904E+00_JPRB, 0.63092E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.76628E+00_JPRB, 0.79814E+00_JPRB, 0.80202E+00_JPRB, 0.82552E+00_JPRB, 0.82163E+00_JPRB, 0.80889E+00_JPRB, 0.81720E+00_JPRB ,&
&  0.83203E+00_JPRB, 0.84287E+00_JPRB, 0.86578E+00_JPRB, 0.88910E+00_JPRB, 0.90751E+00_JPRB, 0.91517E+00_JPRB, 0.93226E+00_JPRB ,&
&  0.96040E+00_JPRB, 0.99718E+00_JPRB, 0.10400E+01_JPRB, 0.10964E+01_JPRB, 0.11366E+01_JPRB, 0.11648E+01_JPRB, 0.12150E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.13358E+01_JPRB, 0.14236E+01_JPRB, 0.15102E+01_JPRB, 0.15662E+01_JPRB, 0.15921E+01_JPRB, 0.12728E+01_JPRB ,&
&  0.17110E+01_JPRB, 0.17418E+01_JPRB, 0.16438E+01_JPRB, 0.14040E+01_JPRB, 0.12867E+01_JPRB, 0.12101E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.10659E+01_JPRB, 0.10254E+01_JPRB, 0.10100E+01_JPRB, 0.11080E+01_JPRB, 0.11365E+01_JPRB, 0.12055E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.13539E+01_JPRB, 0.14245E+01_JPRB, 0.14953E+01_JPRB, 0.15951E+01_JPRB, 0.17083E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.18951E+01_JPRB, 0.19379E+01_JPRB, 0.20222E+01_JPRB, 0.17825E+01_JPRB, 0.17098E+01_JPRB, 0.16045E+01_JPRB, 0.11568E+01_JPRB ,&
&  0.82550E+00_JPRB, 0.77798E+00_JPRB, 0.83981E+00_JPRB, 0.86073E+00_JPRB, 0.81018E+00_JPRB, 0.71999E+00_JPRB, 0.65969E+00_JPRB ,&
&  0.59588E+00_JPRB, 0.58582E+00_JPRB, 0.57973E+00_JPRB, 0.56727E+00_JPRB, 0.55888E+00_JPRB, 0.58572E+00_JPRB, 0.61002E+00_JPRB ,&
&  0.66305E+00_JPRB, 0.70005E+00_JPRB, 0.74751E+00_JPRB, 0.79132E+00_JPRB, 0.85643E+00_JPRB, 0.93664E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.89148E+00_JPRB, 0.83182E+00_JPRB, 0.87032E+00_JPRB, 0.92674E+00_JPRB, 0.96587E+00_JPRB, 0.10531E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.12825E+01_JPRB, 0.13818E+01_JPRB, 0.12652E+01_JPRB, 0.10629E+01_JPRB, 0.14421E+01_JPRB, 0.14908E+01_JPRB, 0.69259E+00_JPRB ,&
&  0.68350E+00_JPRB, 0.71569E+00_JPRB, 0.73396E+00_JPRB, 0.75539E+00_JPRB, 0.77555E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.84480E+00_JPRB, 0.85815E+00_JPRB, 0.87074E+00_JPRB, 0.87934E+00_JPRB, 0.88021E+00_JPRB, 0.89147E+00_JPRB, 0.91293E+00_JPRB ,&
&  0.93330E+00_JPRB, 0.95316E+00_JPRB, 0.97254E+00_JPRB, 0.98821E+00_JPRB, 0.10055E+01_JPRB, 0.10166E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.11091E+01_JPRB, 0.11260E+01_JPRB, 0.11601E+01_JPRB, 0.11805E+01_JPRB, 0.11904E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12773E+01_JPRB, 0.13335E+01_JPRB, 0.13736E+01_JPRB, 0.13971E+01_JPRB, 0.13920E+01_JPRB, 0.14740E+01_JPRB, 0.13146E+01_JPRB ,&
&  0.16266E+01_JPRB, 0.18159E+01_JPRB, 0.16765E+01_JPRB, 0.13652E+01_JPRB, 0.12537E+01_JPRB, 0.11961E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10879E+01_JPRB, 0.10404E+01_JPRB, 0.10855E+01_JPRB, 0.11250E+01_JPRB, 0.11804E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.13198E+01_JPRB, 0.13860E+01_JPRB, 0.14550E+01_JPRB, 0.15223E+01_JPRB, 0.16168E+01_JPRB, 0.16982E+01_JPRB, 0.17493E+01_JPRB ,&
&  0.17864E+01_JPRB, 0.18168E+01_JPRB, 0.18384E+01_JPRB, 0.16958E+01_JPRB, 0.16645E+01_JPRB, 0.14966E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.86936E+00_JPRB, 0.77032E+00_JPRB, 0.80773E+00_JPRB, 0.85441E+00_JPRB, 0.82700E+00_JPRB, 0.72380E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.61246E+00_JPRB, 0.56463E+00_JPRB, 0.56514E+00_JPRB, 0.55601E+00_JPRB, 0.54742E+00_JPRB, 0.57933E+00_JPRB, 0.60519E+00_JPRB ,&
&  0.64650E+00_JPRB, 0.70501E+00_JPRB, 0.74838E+00_JPRB, 0.77585E+00_JPRB, 0.84994E+00_JPRB, 0.92171E+00_JPRB, 0.87018E+00_JPRB ,&
&  0.91536E+00_JPRB, 0.94111E+00_JPRB, 0.97522E+00_JPRB, 0.10899E+01_JPRB, 0.10767E+01_JPRB, 0.12783E+01_JPRB, 0.14989E+01_JPRB ,&
&  0.18007E+01_JPRB, 0.17447E+01_JPRB, 0.20536E+01_JPRB, 0.30853E+01_JPRB, 0.26500E+01_JPRB, 0.22305E+01_JPRB, 0.82661E+00_JPRB ,&
&  0.81563E+00_JPRB, 0.82917E+00_JPRB, 0.85072E+00_JPRB, 0.85119E+00_JPRB, 0.85079E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.87558E+00_JPRB, 0.89118E+00_JPRB, 0.90264E+00_JPRB, 0.92041E+00_JPRB, 0.94262E+00_JPRB, 0.96886E+00_JPRB, 0.98835E+00_JPRB ,&
&  0.10102E+01_JPRB, 0.10275E+01_JPRB, 0.10436E+01_JPRB, 0.10619E+01_JPRB, 0.10752E+01_JPRB, 0.10941E+01_JPRB, 0.10994E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.11153E+01_JPRB, 0.11243E+01_JPRB, 0.11326E+01_JPRB, 0.11402E+01_JPRB, 0.11481E+01_JPRB, 0.11630E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.11688E+01_JPRB, 0.11774E+01_JPRB, 0.11919E+01_JPRB, 0.11957E+01_JPRB, 0.12645E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.15152E+01_JPRB, 0.17126E+01_JPRB, 0.16171E+01_JPRB, 0.12968E+01_JPRB, 0.11771E+01_JPRB, 0.11364E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.10898E+01_JPRB, 0.10743E+01_JPRB, 0.10862E+01_JPRB, 0.11216E+01_JPRB, 0.11763E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.13005E+01_JPRB, 0.13537E+01_JPRB, 0.14121E+01_JPRB, 0.15006E+01_JPRB, 0.15482E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.15393E+01_JPRB, 0.15253E+01_JPRB, 0.14002E+01_JPRB, 0.14037E+01_JPRB, 0.14171E+01_JPRB, 0.11759E+01_JPRB, 0.98237E+00_JPRB ,&
&  0.83211E+00_JPRB, 0.78696E+00_JPRB, 0.81960E+00_JPRB, 0.89507E+00_JPRB, 0.83567E+00_JPRB, 0.76849E+00_JPRB, 0.72770E+00_JPRB ,&
&  0.63578E+00_JPRB, 0.58630E+00_JPRB, 0.57137E+00_JPRB, 0.54637E+00_JPRB, 0.54685E+00_JPRB, 0.56648E+00_JPRB, 0.61329E+00_JPRB ,&
&  0.63704E+00_JPRB, 0.66166E+00_JPRB, 0.69248E+00_JPRB, 0.70531E+00_JPRB, 0.74423E+00_JPRB, 0.71450E+00_JPRB, 0.84546E+00_JPRB ,&
&  0.83780E+00_JPRB, 0.83235E+00_JPRB, 0.88330E+00_JPRB, 0.76381E+00_JPRB, 0.10230E+01_JPRB, 0.11077E+01_JPRB, 0.12593E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.17657E+01_JPRB, 0.21403E+01_JPRB, 0.17956E+01_JPRB, 0.13055E+01_JPRB, 0.89858E+00_JPRB, 0.80511E+00_JPRB ,&
&  0.80212E+00_JPRB, 0.82227E+00_JPRB, 0.84267E+00_JPRB, 0.85248E+00_JPRB, 0.86360E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.87253E+00_JPRB, 0.88769E+00_JPRB, 0.90341E+00_JPRB, 0.92980E+00_JPRB, 0.95860E+00_JPRB, 0.98642E+00_JPRB, 0.10079E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10457E+01_JPRB, 0.10660E+01_JPRB, 0.10844E+01_JPRB, 0.10977E+01_JPRB, 0.11054E+01_JPRB, 0.11062E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.11137E+01_JPRB, 0.11267E+01_JPRB, 0.11365E+01_JPRB, 0.11343E+01_JPRB, 0.11431E+01_JPRB, 0.11621E+01_JPRB ,&
&  0.11750E+01_JPRB, 0.11870E+01_JPRB, 0.12182E+01_JPRB, 0.12632E+01_JPRB, 0.13243E+01_JPRB, 0.14440E+01_JPRB, 0.12762E+01_JPRB ,&
&  0.13918E+01_JPRB, 0.15678E+01_JPRB, 0.15367E+01_JPRB, 0.12103E+01_JPRB, 0.10899E+01_JPRB, 0.10494E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.10185E+01_JPRB, 0.10220E+01_JPRB, 0.10440E+01_JPRB, 0.10810E+01_JPRB, 0.11219E+01_JPRB, 0.11385E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.11856E+01_JPRB, 0.12126E+01_JPRB, 0.12364E+01_JPRB, 0.12813E+01_JPRB, 0.12918E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.12200E+01_JPRB, 0.11779E+01_JPRB, 0.10205E+01_JPRB, 0.95069E+00_JPRB, 0.96015E+00_JPRB, 0.86201E+00_JPRB, 0.77109E+00_JPRB ,&
&  0.78487E+00_JPRB, 0.76694E+00_JPRB, 0.75931E+00_JPRB, 0.92282E+00_JPRB, 0.90546E+00_JPRB, 0.79490E+00_JPRB, 0.76644E+00_JPRB ,&
&  0.67984E+00_JPRB, 0.60122E+00_JPRB, 0.56346E+00_JPRB, 0.56184E+00_JPRB, 0.56904E+00_JPRB, 0.59524E+00_JPRB, 0.59344E+00_JPRB ,&
&  0.60076E+00_JPRB, 0.61573E+00_JPRB, 0.61905E+00_JPRB, 0.61577E+00_JPRB, 0.67033E+00_JPRB, 0.59131E+00_JPRB, 0.75472E+00_JPRB ,&
&  0.70365E+00_JPRB, 0.59809E+00_JPRB, 0.70138E+00_JPRB, 0.86058E+00_JPRB, 0.92448E+00_JPRB, 0.10073E+01_JPRB, 0.11635E+01_JPRB ,&
&  0.13415E+01_JPRB, 0.14772E+01_JPRB, 0.13310E+01_JPRB, 0.11274E+01_JPRB, 0.91778E+00_JPRB, 0.82817E+00_JPRB, 0.79146E+00_JPRB ,&
&  0.78650E+00_JPRB, 0.81168E+00_JPRB, 0.83143E+00_JPRB, 0.84182E+00_JPRB, 0.85646E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.83293E+00_JPRB, 0.85031E+00_JPRB, 0.86958E+00_JPRB, 0.89262E+00_JPRB, 0.91430E+00_JPRB, 0.93410E+00_JPRB, 0.95621E+00_JPRB ,&
&  0.98488E+00_JPRB, 0.10067E+01_JPRB, 0.10291E+01_JPRB, 0.10430E+01_JPRB, 0.10508E+01_JPRB, 0.10654E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.10735E+01_JPRB, 0.10754E+01_JPRB, 0.10915E+01_JPRB, 0.11026E+01_JPRB, 0.11031E+01_JPRB, 0.11013E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.11528E+01_JPRB, 0.11799E+01_JPRB, 0.12138E+01_JPRB, 0.12422E+01_JPRB, 0.12517E+01_JPRB, 0.13288E+01_JPRB ,&
&  0.10186E+01_JPRB, 0.14944E+01_JPRB, 0.14283E+01_JPRB, 0.11176E+01_JPRB, 0.10481E+01_JPRB, 0.98059E+00_JPRB, 0.95637E+00_JPRB ,&
&  0.94693E+00_JPRB, 0.94527E+00_JPRB, 0.95724E+00_JPRB, 0.98401E+00_JPRB, 0.10129E+01_JPRB, 0.10453E+01_JPRB, 0.10575E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10761E+01_JPRB, 0.10787E+01_JPRB, 0.10755E+01_JPRB, 0.10789E+01_JPRB, 0.10639E+01_JPRB, 0.10316E+01_JPRB ,&
&  0.97689E+00_JPRB, 0.92410E+00_JPRB, 0.67824E+00_JPRB, 0.63249E+00_JPRB, 0.66096E+00_JPRB, 0.64060E+00_JPRB, 0.58143E+00_JPRB ,&
&  0.67627E+00_JPRB, 0.78546E+00_JPRB, 0.86538E+00_JPRB, 0.94032E+00_JPRB, 0.94965E+00_JPRB, 0.88661E+00_JPRB, 0.85875E+00_JPRB ,&
&  0.80104E+00_JPRB, 0.71426E+00_JPRB, 0.67260E+00_JPRB, 0.64583E+00_JPRB, 0.64960E+00_JPRB, 0.63397E+00_JPRB, 0.60879E+00_JPRB ,&
&  0.57505E+00_JPRB, 0.55629E+00_JPRB, 0.53665E+00_JPRB, 0.49794E+00_JPRB, 0.41219E+00_JPRB, 0.49761E+00_JPRB, 0.67021E+00_JPRB ,&
&  0.60420E+00_JPRB, 0.52424E+00_JPRB, 0.54420E+00_JPRB, 0.72556E+00_JPRB, 0.75545E+00_JPRB, 0.79386E+00_JPRB, 0.91237E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.10139E+01_JPRB, 0.92076E+00_JPRB, 0.81299E+00_JPRB, 0.77130E+00_JPRB, 0.75578E+00_JPRB, 0.73255E+00_JPRB ,&
&  0.73171E+00_JPRB, 0.75567E+00_JPRB, 0.77837E+00_JPRB, 0.79579E+00_JPRB, 0.81432E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.79327E+00_JPRB, 0.78598E+00_JPRB, 0.79692E+00_JPRB, 0.81363E+00_JPRB, 0.82911E+00_JPRB, 0.84609E+00_JPRB, 0.87412E+00_JPRB ,&
&  0.90846E+00_JPRB, 0.93615E+00_JPRB, 0.96351E+00_JPRB, 0.98314E+00_JPRB, 0.99004E+00_JPRB, 0.10043E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10287E+01_JPRB, 0.10459E+01_JPRB, 0.10755E+01_JPRB, 0.11105E+01_JPRB, 0.11245E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.11540E+01_JPRB, 0.11945E+01_JPRB, 0.12420E+01_JPRB, 0.12986E+01_JPRB, 0.13942E+01_JPRB, 0.15629E+01_JPRB, 0.17382E+01_JPRB ,&
&  0.12534E+01_JPRB, 0.13325E+01_JPRB, 0.12207E+01_JPRB, 0.98218E+00_JPRB, 0.90699E+00_JPRB, 0.92312E+00_JPRB, 0.88174E+00_JPRB ,&
&  0.84970E+00_JPRB, 0.86096E+00_JPRB, 0.86978E+00_JPRB, 0.87789E+00_JPRB, 0.89681E+00_JPRB, 0.93172E+00_JPRB, 0.93353E+00_JPRB ,&
&  0.94826E+00_JPRB, 0.95142E+00_JPRB, 0.94256E+00_JPRB, 0.94865E+00_JPRB, 0.87516E+00_JPRB, 0.97248E+00_JPRB, 0.92829E+00_JPRB ,&
&  0.89241E+00_JPRB, 0.81825E+00_JPRB, 0.59572E+00_JPRB, 0.58165E+00_JPRB, 0.60325E+00_JPRB, 0.55631E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.65023E+00_JPRB, 0.75127E+00_JPRB, 0.69335E+00_JPRB, 0.89334E+00_JPRB, 0.99770E+00_JPRB, 0.10343E+01_JPRB, 0.10411E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.92753E+00_JPRB, 0.84854E+00_JPRB, 0.79455E+00_JPRB, 0.77099E+00_JPRB, 0.73232E+00_JPRB, 0.67317E+00_JPRB ,&
&  0.62146E+00_JPRB, 0.58118E+00_JPRB, 0.52745E+00_JPRB, 0.48033E+00_JPRB, 0.51409E+00_JPRB, 0.41396E+00_JPRB, 0.61262E+00_JPRB ,&
&  0.57468E+00_JPRB, 0.55836E+00_JPRB, 0.50538E+00_JPRB, 0.70352E+00_JPRB, 0.66618E+00_JPRB, 0.60104E+00_JPRB, 0.66826E+00_JPRB ,&
&  0.71660E+00_JPRB, 0.74932E+00_JPRB, 0.73457E+00_JPRB, 0.70605E+00_JPRB, 0.69538E+00_JPRB, 0.69637E+00_JPRB, 0.67573E+00_JPRB ,&
&  0.66727E+00_JPRB, 0.68462E+00_JPRB, 0.71832E+00_JPRB, 0.73048E+00_JPRB, 0.76269E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.70243E+00_JPRB, 0.70643E+00_JPRB, 0.71155E+00_JPRB, 0.71993E+00_JPRB, 0.73721E+00_JPRB, 0.75619E+00_JPRB, 0.76928E+00_JPRB ,&
&  0.79664E+00_JPRB, 0.81868E+00_JPRB, 0.84196E+00_JPRB, 0.85074E+00_JPRB, 0.86690E+00_JPRB, 0.88206E+00_JPRB, 0.88786E+00_JPRB ,&
&  0.90924E+00_JPRB, 0.93481E+00_JPRB, 0.93591E+00_JPRB, 0.94488E+00_JPRB, 0.97325E+00_JPRB, 0.99240E+00_JPRB, 0.97644E+00_JPRB ,&
&  0.95484E+00_JPRB, 0.92911E+00_JPRB, 0.94899E+00_JPRB, 0.98857E+00_JPRB, 0.11756E+01_JPRB, 0.12937E+01_JPRB, 0.12847E+01_JPRB ,&
&  0.12016E+01_JPRB, 0.12922E+01_JPRB, 0.12931E+01_JPRB, 0.98322E+00_JPRB, 0.88692E+00_JPRB, 0.83107E+00_JPRB, 0.77205E+00_JPRB ,&
&  0.78053E+00_JPRB, 0.80350E+00_JPRB, 0.82814E+00_JPRB, 0.85363E+00_JPRB, 0.89175E+00_JPRB, 0.91014E+00_JPRB, 0.94979E+00_JPRB ,&
&  0.95121E+00_JPRB, 0.96307E+00_JPRB, 0.96137E+00_JPRB, 0.93839E+00_JPRB, 0.94948E+00_JPRB, 0.10521E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10672E+01_JPRB, 0.95978E+00_JPRB, 0.92270E+00_JPRB, 0.88164E+00_JPRB, 0.76687E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.67467E+00_JPRB, 0.67122E+00_JPRB, 0.59172E+00_JPRB, 0.81766E+00_JPRB, 0.10117E+01_JPRB, 0.10919E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.11464E+01_JPRB, 0.10034E+01_JPRB, 0.99057E+00_JPRB, 0.98004E+00_JPRB, 0.83912E+00_JPRB, 0.77667E+00_JPRB ,&
&  0.68540E+00_JPRB, 0.62085E+00_JPRB, 0.57249E+00_JPRB, 0.55093E+00_JPRB, 0.51915E+00_JPRB, 0.50338E+00_JPRB, 0.57611E+00_JPRB ,&
&  0.57059E+00_JPRB, 0.55181E+00_JPRB, 0.63472E+00_JPRB, 0.86386E+00_JPRB, 0.59682E+00_JPRB, 0.54604E+00_JPRB, 0.54936E+00_JPRB ,&
&  0.56609E+00_JPRB, 0.56922E+00_JPRB, 0.57236E+00_JPRB, 0.57554E+00_JPRB, 0.58124E+00_JPRB, 0.59907E+00_JPRB, 0.59482E+00_JPRB ,&
&  0.59136E+00_JPRB, 0.58956E+00_JPRB, 0.60712E+00_JPRB, 0.63042E+00_JPRB, 0.67161E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.56328E+00_JPRB, 0.56292E+00_JPRB, 0.55131E+00_JPRB, 0.54422E+00_JPRB, 0.54580E+00_JPRB, 0.55231E+00_JPRB, 0.56256E+00_JPRB ,&
&  0.57998E+00_JPRB, 0.60454E+00_JPRB, 0.63031E+00_JPRB, 0.66327E+00_JPRB, 0.67244E+00_JPRB, 0.64003E+00_JPRB, 0.64549E+00_JPRB ,&
&  0.64281E+00_JPRB, 0.65682E+00_JPRB, 0.65083E+00_JPRB, 0.72768E+00_JPRB, 0.75455E+00_JPRB, 0.76588E+00_JPRB, 0.85696E+00_JPRB ,&
&  0.84706E+00_JPRB, 0.93594E+00_JPRB, 0.10074E+01_JPRB, 0.11328E+01_JPRB, 0.18641E+01_JPRB, 0.17370E+01_JPRB, 0.12572E+01_JPRB ,&
&  0.15362E+01_JPRB, 0.15746E+01_JPRB, 0.12211E+01_JPRB, 0.98437E+00_JPRB, 0.91809E+00_JPRB, 0.83348E+00_JPRB, 0.80352E+00_JPRB ,&
&  0.84645E+00_JPRB, 0.90588E+00_JPRB, 0.10158E+01_JPRB, 0.11184E+01_JPRB, 0.11976E+01_JPRB, 0.12766E+01_JPRB, 0.13724E+01_JPRB ,&
&  0.14327E+01_JPRB, 0.14816E+01_JPRB, 0.14277E+01_JPRB, 0.13407E+01_JPRB, 0.14680E+01_JPRB, 0.15794E+01_JPRB, 0.16459E+01_JPRB ,&
&  0.16989E+01_JPRB, 0.17681E+01_JPRB, 0.18543E+01_JPRB, 0.17420E+01_JPRB, 0.15605E+01_JPRB, 0.14322E+01_JPRB, 0.11837E+01_JPRB ,&
&  0.98842E+00_JPRB, 0.83975E+00_JPRB, 0.78887E+00_JPRB, 0.10718E+01_JPRB, 0.12457E+01_JPRB, 0.11794E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.14731E+01_JPRB, 0.13281E+01_JPRB, 0.12002E+01_JPRB, 0.10439E+01_JPRB, 0.95962E+00_JPRB, 0.85461E+00_JPRB ,&
&  0.78482E+00_JPRB, 0.68967E+00_JPRB, 0.63037E+00_JPRB, 0.59129E+00_JPRB, 0.55886E+00_JPRB, 0.53665E+00_JPRB, 0.93439E+00_JPRB ,&
&  0.89815E+00_JPRB, 0.64961E+00_JPRB, 0.61314E+00_JPRB, 0.87459E+00_JPRB, 0.92212E+00_JPRB, 0.60385E+00_JPRB, 0.50029E+00_JPRB ,&
&  0.49000E+00_JPRB, 0.49786E+00_JPRB, 0.50069E+00_JPRB, 0.50427E+00_JPRB, 0.51841E+00_JPRB, 0.53264E+00_JPRB, 0.52468E+00_JPRB ,&
&  0.52699E+00_JPRB, 0.52148E+00_JPRB, 0.51800E+00_JPRB, 0.52189E+00_JPRB, 0.53589E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.49904E+00_JPRB, 0.50023E+00_JPRB, 0.51393E+00_JPRB, 0.56104E+00_JPRB, 0.58942E+00_JPRB, 0.60463E+00_JPRB, 0.63617E+00_JPRB ,&
&  0.67155E+00_JPRB, 0.73529E+00_JPRB, 0.78293E+00_JPRB, 0.80001E+00_JPRB, 0.77459E+00_JPRB, 0.74432E+00_JPRB, 0.72835E+00_JPRB ,&
&  0.72250E+00_JPRB, 0.67257E+00_JPRB, 0.64354E+00_JPRB, 0.66615E+00_JPRB, 0.72991E+00_JPRB, 0.84012E+00_JPRB, 0.96903E+00_JPRB ,&
&  0.11383E+01_JPRB, 0.13288E+01_JPRB, 0.15773E+01_JPRB, 0.21629E+01_JPRB, 0.24064E+01_JPRB, 0.12261E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.17628E+01_JPRB, 0.17760E+01_JPRB, 0.12415E+01_JPRB, 0.10677E+01_JPRB, 0.99849E+00_JPRB, 0.96491E+00_JPRB, 0.97990E+00_JPRB ,&
&  0.10652E+01_JPRB, 0.11468E+01_JPRB, 0.12437E+01_JPRB, 0.13695E+01_JPRB, 0.15091E+01_JPRB, 0.16374E+01_JPRB, 0.17956E+01_JPRB ,&
&  0.19304E+01_JPRB, 0.19670E+01_JPRB, 0.19761E+01_JPRB, 0.20080E+01_JPRB, 0.21236E+01_JPRB, 0.22705E+01_JPRB, 0.23948E+01_JPRB ,&
&  0.24639E+01_JPRB, 0.24871E+01_JPRB, 0.25511E+01_JPRB, 0.25273E+01_JPRB, 0.22602E+01_JPRB, 0.20462E+01_JPRB, 0.17614E+01_JPRB ,&
&  0.15644E+01_JPRB, 0.13679E+01_JPRB, 0.12832E+01_JPRB, 0.17104E+01_JPRB, 0.18991E+01_JPRB, 0.17435E+01_JPRB, 0.17070E+01_JPRB ,&
&  0.17523E+01_JPRB, 0.17116E+01_JPRB, 0.15738E+01_JPRB, 0.12914E+01_JPRB, 0.10748E+01_JPRB, 0.10705E+01_JPRB, 0.92146E+00_JPRB ,&
&  0.89525E+00_JPRB, 0.78587E+00_JPRB, 0.71373E+00_JPRB, 0.65770E+00_JPRB, 0.61572E+00_JPRB, 0.57570E+00_JPRB, 0.85714E+00_JPRB ,&
&  0.88534E+00_JPRB, 0.84587E+00_JPRB, 0.69795E+00_JPRB, 0.10086E+01_JPRB, 0.19309E+01_JPRB, 0.10036E+01_JPRB, 0.63535E+00_JPRB ,&
&  0.50075E+00_JPRB, 0.49335E+00_JPRB, 0.49853E+00_JPRB, 0.50573E+00_JPRB, 0.51575E+00_JPRB, 0.52872E+00_JPRB, 0.53131E+00_JPRB ,&
&  0.54220E+00_JPRB, 0.53850E+00_JPRB, 0.54190E+00_JPRB, 0.52614E+00_JPRB, 0.49624E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.64600E+00_JPRB, 0.67114E+00_JPRB, 0.71113E+00_JPRB, 0.76473E+00_JPRB, 0.79878E+00_JPRB, 0.82403E+00_JPRB, 0.82091E+00_JPRB ,&
&  0.83969E+00_JPRB, 0.85841E+00_JPRB, 0.89461E+00_JPRB, 0.92732E+00_JPRB, 0.94867E+00_JPRB, 0.99765E+00_JPRB, 0.10313E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.10242E+01_JPRB, 0.10630E+01_JPRB, 0.11320E+01_JPRB, 0.12819E+01_JPRB, 0.15020E+01_JPRB, 0.17208E+01_JPRB ,&
&  0.19269E+01_JPRB, 0.19582E+01_JPRB, 0.19308E+01_JPRB, 0.13056E+01_JPRB, 0.94274E+00_JPRB, 0.11101E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.12908E+01_JPRB, 0.11355E+01_JPRB, 0.10636E+01_JPRB, 0.10249E+01_JPRB, 0.10419E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.13939E+01_JPRB, 0.15338E+01_JPRB, 0.16301E+01_JPRB, 0.17421E+01_JPRB, 0.19056E+01_JPRB, 0.20990E+01_JPRB ,&
&  0.22665E+01_JPRB, 0.23625E+01_JPRB, 0.24024E+01_JPRB, 0.25045E+01_JPRB, 0.25726E+01_JPRB, 0.26821E+01_JPRB, 0.27648E+01_JPRB ,&
&  0.28320E+01_JPRB, 0.28624E+01_JPRB, 0.28933E+01_JPRB, 0.29335E+01_JPRB, 0.28351E+01_JPRB, 0.25217E+01_JPRB, 0.22654E+01_JPRB ,&
&  0.21520E+01_JPRB, 0.19940E+01_JPRB, 0.20538E+01_JPRB, 0.22909E+01_JPRB, 0.16981E+01_JPRB, 0.17838E+01_JPRB, 0.20556E+01_JPRB ,&
&  0.20121E+01_JPRB, 0.19465E+01_JPRB, 0.17919E+01_JPRB, 0.15543E+01_JPRB, 0.14266E+01_JPRB, 0.11987E+01_JPRB, 0.10583E+01_JPRB ,&
&  0.98808E+00_JPRB, 0.86315E+00_JPRB, 0.77000E+00_JPRB, 0.68191E+00_JPRB, 0.70678E+00_JPRB, 0.72706E+00_JPRB, 0.83271E+00_JPRB ,&
&  0.11323E+01_JPRB, 0.91978E+00_JPRB, 0.73925E+00_JPRB, 0.91552E+00_JPRB, 0.10378E+01_JPRB, 0.72434E+00_JPRB, 0.57505E+00_JPRB ,&
&  0.50060E+00_JPRB, 0.49581E+00_JPRB, 0.50169E+00_JPRB, 0.49454E+00_JPRB, 0.50933E+00_JPRB, 0.53368E+00_JPRB, 0.53625E+00_JPRB ,&
&  0.57479E+00_JPRB, 0.59491E+00_JPRB, 0.61900E+00_JPRB, 0.64143E+00_JPRB, 0.63360E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.79382E+00_JPRB, 0.88294E+00_JPRB, 0.95076E+00_JPRB, 0.10186E+01_JPRB, 0.10567E+01_JPRB, 0.11145E+01_JPRB, 0.11022E+01_JPRB ,&
&  0.10225E+01_JPRB, 0.96030E+00_JPRB, 0.10206E+01_JPRB, 0.11053E+01_JPRB, 0.11842E+01_JPRB, 0.12432E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.13881E+01_JPRB, 0.14304E+01_JPRB, 0.14577E+01_JPRB, 0.15091E+01_JPRB, 0.15691E+01_JPRB, 0.15628E+01_JPRB ,&
&  0.12930E+01_JPRB, 0.10648E+01_JPRB, 0.13138E+01_JPRB, 0.14222E+01_JPRB, 0.11900E+01_JPRB, 0.12085E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12610E+01_JPRB, 0.12202E+01_JPRB, 0.12240E+01_JPRB, 0.12217E+01_JPRB, 0.10379E+01_JPRB, 0.11146E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.16257E+01_JPRB, 0.17632E+01_JPRB, 0.18662E+01_JPRB, 0.19786E+01_JPRB, 0.21436E+01_JPRB, 0.23106E+01_JPRB ,&
&  0.24666E+01_JPRB, 0.25737E+01_JPRB, 0.26348E+01_JPRB, 0.27073E+01_JPRB, 0.28081E+01_JPRB, 0.29455E+01_JPRB, 0.30186E+01_JPRB ,&
&  0.30898E+01_JPRB, 0.31741E+01_JPRB, 0.32969E+01_JPRB, 0.34216E+01_JPRB, 0.34464E+01_JPRB, 0.30640E+01_JPRB, 0.28904E+01_JPRB ,&
&  0.27699E+01_JPRB, 0.25646E+01_JPRB, 0.26445E+01_JPRB, 0.27874E+01_JPRB, 0.20917E+01_JPRB, 0.20746E+01_JPRB, 0.21252E+01_JPRB ,&
&  0.23620E+01_JPRB, 0.22404E+01_JPRB, 0.21131E+01_JPRB, 0.21153E+01_JPRB, 0.19373E+01_JPRB, 0.16928E+01_JPRB, 0.14575E+01_JPRB ,&
&  0.10896E+01_JPRB, 0.88475E+00_JPRB, 0.77825E+00_JPRB, 0.80642E+00_JPRB, 0.79482E+00_JPRB, 0.88764E+00_JPRB, 0.10794E+01_JPRB ,&
&  0.14635E+01_JPRB, 0.89600E+00_JPRB, 0.78094E+00_JPRB, 0.86180E+00_JPRB, 0.74515E+00_JPRB, 0.58523E+00_JPRB, 0.52304E+00_JPRB ,&
&  0.49820E+00_JPRB, 0.51622E+00_JPRB, 0.52371E+00_JPRB, 0.51810E+00_JPRB, 0.51773E+00_JPRB, 0.54366E+00_JPRB, 0.57252E+00_JPRB ,&
&  0.59912E+00_JPRB, 0.63262E+00_JPRB, 0.66633E+00_JPRB, 0.70799E+00_JPRB, 0.76297E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.82367E+00_JPRB, 0.88539E+00_JPRB, 0.99352E+00_JPRB, 0.11365E+01_JPRB, 0.13325E+01_JPRB, 0.15213E+01_JPRB, 0.15677E+01_JPRB ,&
&  0.13142E+01_JPRB, 0.10673E+01_JPRB, 0.11510E+01_JPRB, 0.12396E+01_JPRB, 0.13189E+01_JPRB, 0.14055E+01_JPRB, 0.14796E+01_JPRB ,&
&  0.15654E+01_JPRB, 0.16508E+01_JPRB, 0.17260E+01_JPRB, 0.17938E+01_JPRB, 0.18272E+01_JPRB, 0.18275E+01_JPRB, 0.16374E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.11995E+01_JPRB, 0.16343E+01_JPRB, 0.14811E+01_JPRB, 0.13557E+01_JPRB, 0.13878E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.13523E+01_JPRB, 0.12411E+01_JPRB, 0.12807E+01_JPRB, 0.12493E+01_JPRB, 0.11340E+01_JPRB, 0.12507E+01_JPRB, 0.14712E+01_JPRB ,&
&  0.16131E+01_JPRB, 0.17543E+01_JPRB, 0.19047E+01_JPRB, 0.20408E+01_JPRB, 0.21271E+01_JPRB, 0.22554E+01_JPRB, 0.23700E+01_JPRB ,&
&  0.24625E+01_JPRB, 0.25633E+01_JPRB, 0.26268E+01_JPRB, 0.27025E+01_JPRB, 0.29055E+01_JPRB, 0.31499E+01_JPRB, 0.33459E+01_JPRB ,&
&  0.34411E+01_JPRB, 0.37043E+01_JPRB, 0.38050E+01_JPRB, 0.37364E+01_JPRB, 0.34524E+01_JPRB, 0.36423E+01_JPRB, 0.33145E+01_JPRB ,&
&  0.33191E+01_JPRB, 0.31068E+01_JPRB, 0.32976E+01_JPRB, 0.27756E+01_JPRB, 0.27516E+01_JPRB, 0.27715E+01_JPRB, 0.27579E+01_JPRB ,&
&  0.26469E+01_JPRB, 0.26551E+01_JPRB, 0.24094E+01_JPRB, 0.23102E+01_JPRB, 0.23132E+01_JPRB, 0.21270E+01_JPRB, 0.21189E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.10369E+01_JPRB, 0.82292E+00_JPRB, 0.83490E+00_JPRB, 0.80701E+00_JPRB, 0.10117E+01_JPRB, 0.15363E+01_JPRB ,&
&  0.19136E+01_JPRB, 0.13776E+01_JPRB, 0.11546E+01_JPRB, 0.10112E+01_JPRB, 0.74472E+00_JPRB, 0.61081E+00_JPRB, 0.53755E+00_JPRB ,&
&  0.50229E+00_JPRB, 0.51874E+00_JPRB, 0.55365E+00_JPRB, 0.56222E+00_JPRB, 0.55591E+00_JPRB, 0.58013E+00_JPRB, 0.61671E+00_JPRB ,&
&  0.64240E+00_JPRB, 0.66777E+00_JPRB, 0.71474E+00_JPRB, 0.73335E+00_JPRB, 0.78327E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.76458E+00_JPRB, 0.80873E+00_JPRB, 0.87232E+00_JPRB, 0.96616E+00_JPRB, 0.10661E+01_JPRB, 0.11403E+01_JPRB, 0.10930E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.10815E+01_JPRB, 0.11372E+01_JPRB, 0.12062E+01_JPRB, 0.12844E+01_JPRB, 0.13737E+01_JPRB, 0.14570E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.16307E+01_JPRB, 0.17347E+01_JPRB, 0.18340E+01_JPRB, 0.18709E+01_JPRB, 0.17901E+01_JPRB, 0.13133E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.18197E+01_JPRB, 0.15360E+01_JPRB, 0.15405E+01_JPRB, 0.15278E+01_JPRB, 0.16089E+01_JPRB, 0.13982E+01_JPRB ,&
&  0.12386E+01_JPRB, 0.12291E+01_JPRB, 0.12739E+01_JPRB, 0.12701E+01_JPRB, 0.13079E+01_JPRB, 0.14249E+01_JPRB, 0.15644E+01_JPRB ,&
&  0.16569E+01_JPRB, 0.17829E+01_JPRB, 0.19258E+01_JPRB, 0.20727E+01_JPRB, 0.21512E+01_JPRB, 0.22229E+01_JPRB, 0.22801E+01_JPRB ,&
&  0.23362E+01_JPRB, 0.23990E+01_JPRB, 0.24573E+01_JPRB, 0.26845E+01_JPRB, 0.29144E+01_JPRB, 0.32307E+01_JPRB, 0.36080E+01_JPRB ,&
&  0.37541E+01_JPRB, 0.38324E+01_JPRB, 0.42192E+01_JPRB, 0.42293E+01_JPRB, 0.41372E+01_JPRB, 0.41982E+01_JPRB, 0.35311E+01_JPRB ,&
&  0.36865E+01_JPRB, 0.34390E+01_JPRB, 0.37299E+01_JPRB, 0.31368E+01_JPRB, 0.31185E+01_JPRB, 0.33944E+01_JPRB, 0.33855E+01_JPRB ,&
&  0.30866E+01_JPRB, 0.31134E+01_JPRB, 0.28225E+01_JPRB, 0.25360E+01_JPRB, 0.24111E+01_JPRB, 0.24163E+01_JPRB, 0.26023E+01_JPRB ,&
&  0.21664E+01_JPRB, 0.12499E+01_JPRB, 0.94232E+00_JPRB, 0.82635E+00_JPRB, 0.85392E+00_JPRB, 0.10847E+01_JPRB, 0.18105E+01_JPRB ,&
&  0.24698E+01_JPRB, 0.23610E+01_JPRB, 0.23049E+01_JPRB, 0.16455E+01_JPRB, 0.10996E+01_JPRB, 0.90027E+00_JPRB, 0.73275E+00_JPRB ,&
&  0.57472E+00_JPRB, 0.55729E+00_JPRB, 0.60787E+00_JPRB, 0.62745E+00_JPRB, 0.62183E+00_JPRB, 0.64960E+00_JPRB, 0.67098E+00_JPRB ,&
&  0.66137E+00_JPRB, 0.65749E+00_JPRB, 0.69430E+00_JPRB, 0.69600E+00_JPRB, 0.72496E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.71175E+00_JPRB, 0.72612E+00_JPRB, 0.78316E+00_JPRB, 0.83643E+00_JPRB, 0.87032E+00_JPRB, 0.88959E+00_JPRB, 0.90404E+00_JPRB ,&
&  0.96050E+00_JPRB, 0.10253E+01_JPRB, 0.10683E+01_JPRB, 0.10985E+01_JPRB, 0.11473E+01_JPRB, 0.12386E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.14826E+01_JPRB, 0.16387E+01_JPRB, 0.18253E+01_JPRB, 0.20133E+01_JPRB, 0.21653E+01_JPRB, 0.21094E+01_JPRB, 0.13413E+01_JPRB ,&
&  0.12710E+01_JPRB, 0.17366E+01_JPRB, 0.17040E+01_JPRB, 0.16864E+01_JPRB, 0.16062E+01_JPRB, 0.16069E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.13291E+01_JPRB, 0.12991E+01_JPRB, 0.13197E+01_JPRB, 0.13658E+01_JPRB, 0.14159E+01_JPRB, 0.14908E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.15800E+01_JPRB, 0.16586E+01_JPRB, 0.18076E+01_JPRB, 0.19748E+01_JPRB, 0.21057E+01_JPRB, 0.21780E+01_JPRB ,&
&  0.22168E+01_JPRB, 0.22467E+01_JPRB, 0.23352E+01_JPRB, 0.24599E+01_JPRB, 0.28972E+01_JPRB, 0.32147E+01_JPRB, 0.36412E+01_JPRB ,&
&  0.39148E+01_JPRB, 0.40443E+01_JPRB, 0.47861E+01_JPRB, 0.53506E+01_JPRB, 0.55041E+01_JPRB, 0.49064E+01_JPRB, 0.40696E+01_JPRB ,&
&  0.38185E+01_JPRB, 0.36954E+01_JPRB, 0.36894E+01_JPRB, 0.33849E+01_JPRB, 0.33827E+01_JPRB, 0.38467E+01_JPRB, 0.42571E+01_JPRB ,&
&  0.37975E+01_JPRB, 0.34725E+01_JPRB, 0.32422E+01_JPRB, 0.27850E+01_JPRB, 0.25859E+01_JPRB, 0.25496E+01_JPRB, 0.29971E+01_JPRB ,&
&  0.30488E+01_JPRB, 0.21096E+01_JPRB, 0.12952E+01_JPRB, 0.90365E+00_JPRB, 0.95872E+00_JPRB, 0.11928E+01_JPRB, 0.15520E+01_JPRB ,&
&  0.24169E+01_JPRB, 0.27197E+01_JPRB, 0.28598E+01_JPRB, 0.27802E+01_JPRB, 0.18145E+01_JPRB, 0.14262E+01_JPRB, 0.12591E+01_JPRB ,&
&  0.86772E+00_JPRB, 0.72084E+00_JPRB, 0.71195E+00_JPRB, 0.70467E+00_JPRB, 0.68768E+00_JPRB, 0.72569E+00_JPRB, 0.77703E+00_JPRB ,&
&  0.73814E+00_JPRB, 0.65029E+00_JPRB, 0.65238E+00_JPRB, 0.65899E+00_JPRB, 0.67887E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.68368E+00_JPRB, 0.65908E+00_JPRB, 0.68664E+00_JPRB, 0.70909E+00_JPRB, 0.74268E+00_JPRB, 0.76848E+00_JPRB, 0.81504E+00_JPRB ,&
&  0.89200E+00_JPRB, 0.95129E+00_JPRB, 0.99129E+00_JPRB, 0.99044E+00_JPRB, 0.97844E+00_JPRB, 0.10205E+01_JPRB, 0.11064E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.14498E+01_JPRB, 0.17445E+01_JPRB, 0.19824E+01_JPRB, 0.20855E+01_JPRB, 0.18743E+01_JPRB, 0.17510E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.20357E+01_JPRB, 0.21244E+01_JPRB, 0.22194E+01_JPRB, 0.21758E+01_JPRB, 0.21015E+01_JPRB, 0.15447E+01_JPRB ,&
&  0.15324E+01_JPRB, 0.15352E+01_JPRB, 0.15263E+01_JPRB, 0.15643E+01_JPRB, 0.15901E+01_JPRB, 0.16137E+01_JPRB, 0.15793E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.14247E+01_JPRB, 0.14201E+01_JPRB, 0.14942E+01_JPRB, 0.16788E+01_JPRB, 0.18518E+01_JPRB, 0.20300E+01_JPRB ,&
&  0.21780E+01_JPRB, 0.22399E+01_JPRB, 0.23008E+01_JPRB, 0.23260E+01_JPRB, 0.27062E+01_JPRB, 0.33009E+01_JPRB, 0.38282E+01_JPRB ,&
&  0.42451E+01_JPRB, 0.49193E+01_JPRB, 0.54975E+01_JPRB, 0.59196E+01_JPRB, 0.62904E+01_JPRB, 0.59808E+01_JPRB, 0.50559E+01_JPRB ,&
&  0.42241E+01_JPRB, 0.36974E+01_JPRB, 0.33663E+01_JPRB, 0.34509E+01_JPRB, 0.36889E+01_JPRB, 0.42663E+01_JPRB, 0.41730E+01_JPRB ,&
&  0.36080E+01_JPRB, 0.31103E+01_JPRB, 0.29458E+01_JPRB, 0.29700E+01_JPRB, 0.29816E+01_JPRB, 0.28349E+01_JPRB, 0.34068E+01_JPRB ,&
&  0.30252E+01_JPRB, 0.15023E+01_JPRB, 0.10077E+01_JPRB, 0.10603E+01_JPRB, 0.76280E+00_JPRB, 0.90097E+00_JPRB, 0.17146E+01_JPRB ,&
&  0.24405E+01_JPRB, 0.35110E+01_JPRB, 0.38093E+01_JPRB, 0.29565E+01_JPRB, 0.27385E+01_JPRB, 0.22718E+01_JPRB, 0.21211E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.10794E+01_JPRB, 0.84248E+00_JPRB, 0.77763E+00_JPRB, 0.74725E+00_JPRB, 0.76284E+00_JPRB, 0.77394E+00_JPRB ,&
&  0.73530E+00_JPRB, 0.66535E+00_JPRB, 0.65951E+00_JPRB, 0.66871E+00_JPRB, 0.67278E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.75490E+00_JPRB, 0.68644E+00_JPRB, 0.65474E+00_JPRB, 0.63949E+00_JPRB, 0.69983E+00_JPRB, 0.76525E+00_JPRB, 0.78331E+00_JPRB ,&
&  0.85392E+00_JPRB, 0.91995E+00_JPRB, 0.94664E+00_JPRB, 0.93646E+00_JPRB, 0.90880E+00_JPRB, 0.90857E+00_JPRB, 0.93754E+00_JPRB ,&
&  0.10116E+01_JPRB, 0.11715E+01_JPRB, 0.14928E+01_JPRB, 0.18276E+01_JPRB, 0.20005E+01_JPRB, 0.19578E+01_JPRB, 0.20788E+01_JPRB ,&
&  0.20448E+01_JPRB, 0.24263E+01_JPRB, 0.30814E+01_JPRB, 0.33188E+01_JPRB, 0.26569E+01_JPRB, 0.23639E+01_JPRB, 0.18381E+01_JPRB ,&
&  0.17687E+01_JPRB, 0.17984E+01_JPRB, 0.18293E+01_JPRB, 0.18924E+01_JPRB, 0.19071E+01_JPRB, 0.18632E+01_JPRB, 0.18023E+01_JPRB ,&
&  0.16257E+01_JPRB, 0.14992E+01_JPRB, 0.13925E+01_JPRB, 0.13714E+01_JPRB, 0.14885E+01_JPRB, 0.16895E+01_JPRB, 0.19031E+01_JPRB ,&
&  0.21483E+01_JPRB, 0.23096E+01_JPRB, 0.23719E+01_JPRB, 0.26015E+01_JPRB, 0.22092E+01_JPRB, 0.32674E+01_JPRB, 0.41840E+01_JPRB ,&
&  0.48013E+01_JPRB, 0.54262E+01_JPRB, 0.55584E+01_JPRB, 0.66570E+01_JPRB, 0.72938E+01_JPRB, 0.66516E+01_JPRB, 0.57126E+01_JPRB ,&
&  0.48784E+01_JPRB, 0.38627E+01_JPRB, 0.35927E+01_JPRB, 0.37486E+01_JPRB, 0.38435E+01_JPRB, 0.43305E+01_JPRB, 0.32158E+01_JPRB ,&
&  0.30765E+01_JPRB, 0.37048E+01_JPRB, 0.34175E+01_JPRB, 0.28993E+01_JPRB, 0.30970E+01_JPRB, 0.29581E+01_JPRB, 0.26546E+01_JPRB ,&
&  0.12477E+01_JPRB, 0.81553E+00_JPRB, 0.76136E+00_JPRB, 0.68756E+00_JPRB, 0.60645E+00_JPRB, 0.90395E+00_JPRB, 0.30262E+01_JPRB ,&
&  0.24839E+01_JPRB, 0.42033E+01_JPRB, 0.45823E+01_JPRB, 0.37627E+01_JPRB, 0.33216E+01_JPRB, 0.30995E+01_JPRB, 0.33989E+01_JPRB ,&
&  0.20488E+01_JPRB, 0.14817E+01_JPRB, 0.10256E+01_JPRB, 0.85364E+00_JPRB, 0.83038E+00_JPRB, 0.80712E+00_JPRB, 0.76767E+00_JPRB ,&
&  0.73484E+00_JPRB, 0.71151E+00_JPRB, 0.71503E+00_JPRB, 0.70172E+00_JPRB, 0.74701E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.88140E+00_JPRB, 0.79544E+00_JPRB, 0.71946E+00_JPRB, 0.71163E+00_JPRB, 0.76381E+00_JPRB, 0.78984E+00_JPRB, 0.80728E+00_JPRB ,&
&  0.85180E+00_JPRB, 0.91091E+00_JPRB, 0.93897E+00_JPRB, 0.91781E+00_JPRB, 0.89639E+00_JPRB, 0.89124E+00_JPRB, 0.88607E+00_JPRB ,&
&  0.91659E+00_JPRB, 0.10148E+01_JPRB, 0.12736E+01_JPRB, 0.14683E+01_JPRB, 0.17324E+01_JPRB, 0.17549E+01_JPRB, 0.20951E+01_JPRB ,&
&  0.22035E+01_JPRB, 0.25322E+01_JPRB, 0.30971E+01_JPRB, 0.32203E+01_JPRB, 0.33768E+01_JPRB, 0.27742E+01_JPRB, 0.24761E+01_JPRB ,&
&  0.20736E+01_JPRB, 0.19363E+01_JPRB, 0.19086E+01_JPRB, 0.20323E+01_JPRB, 0.21490E+01_JPRB, 0.21528E+01_JPRB, 0.20361E+01_JPRB ,&
&  0.17492E+01_JPRB, 0.15838E+01_JPRB, 0.13834E+01_JPRB, 0.13373E+01_JPRB, 0.13214E+01_JPRB, 0.14370E+01_JPRB, 0.16834E+01_JPRB ,&
&  0.20931E+01_JPRB, 0.24672E+01_JPRB, 0.25246E+01_JPRB, 0.26358E+01_JPRB, 0.25612E+01_JPRB, 0.27831E+01_JPRB, 0.42374E+01_JPRB ,&
&  0.55222E+01_JPRB, 0.64834E+01_JPRB, 0.67941E+01_JPRB, 0.82837E+01_JPRB, 0.80946E+01_JPRB, 0.70942E+01_JPRB, 0.65637E+01_JPRB ,&
&  0.55185E+01_JPRB, 0.48889E+01_JPRB, 0.38923E+01_JPRB, 0.38461E+01_JPRB, 0.39118E+01_JPRB, 0.32334E+01_JPRB, 0.31909E+01_JPRB ,&
&  0.38384E+01_JPRB, 0.37229E+01_JPRB, 0.33337E+01_JPRB, 0.27684E+01_JPRB, 0.32309E+01_JPRB, 0.25821E+01_JPRB, 0.13565E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.94856E+00_JPRB, 0.75313E+00_JPRB, 0.66064E+00_JPRB, 0.65939E+00_JPRB, 0.92254E+00_JPRB, 0.18460E+01_JPRB ,&
&  0.25449E+01_JPRB, 0.48289E+01_JPRB, 0.53505E+01_JPRB, 0.44808E+01_JPRB, 0.35311E+01_JPRB, 0.37388E+01_JPRB, 0.37477E+01_JPRB ,&
&  0.28062E+01_JPRB, 0.21493E+01_JPRB, 0.12100E+01_JPRB, 0.93012E+00_JPRB, 0.87956E+00_JPRB, 0.84453E+00_JPRB, 0.80023E+00_JPRB ,&
&  0.75914E+00_JPRB, 0.75921E+00_JPRB, 0.78049E+00_JPRB, 0.79919E+00_JPRB, 0.81527E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.93022E+00_JPRB, 0.88022E+00_JPRB, 0.80124E+00_JPRB, 0.80407E+00_JPRB, 0.83441E+00_JPRB, 0.87490E+00_JPRB, 0.87193E+00_JPRB ,&
&  0.88365E+00_JPRB, 0.90902E+00_JPRB, 0.93476E+00_JPRB, 0.92063E+00_JPRB, 0.88284E+00_JPRB, 0.88788E+00_JPRB, 0.88037E+00_JPRB ,&
&  0.91016E+00_JPRB, 0.10199E+01_JPRB, 0.11370E+01_JPRB, 0.11844E+01_JPRB, 0.12764E+01_JPRB, 0.13648E+01_JPRB, 0.17213E+01_JPRB ,&
&  0.21792E+01_JPRB, 0.25007E+01_JPRB, 0.28709E+01_JPRB, 0.33150E+01_JPRB, 0.37300E+01_JPRB, 0.32901E+01_JPRB, 0.32535E+01_JPRB ,&
&  0.27454E+01_JPRB, 0.23266E+01_JPRB, 0.21828E+01_JPRB, 0.21564E+01_JPRB, 0.22764E+01_JPRB, 0.23366E+01_JPRB, 0.22459E+01_JPRB ,&
&  0.19411E+01_JPRB, 0.16161E+01_JPRB, 0.14201E+01_JPRB, 0.12942E+01_JPRB, 0.12340E+01_JPRB, 0.12902E+01_JPRB, 0.15355E+01_JPRB ,&
&  0.21087E+01_JPRB, 0.27402E+01_JPRB, 0.29188E+01_JPRB, 0.28067E+01_JPRB, 0.33864E+01_JPRB, 0.37897E+01_JPRB, 0.46470E+01_JPRB ,&
&  0.52469E+01_JPRB, 0.56572E+01_JPRB, 0.80151E+01_JPRB, 0.88241E+01_JPRB, 0.83681E+01_JPRB, 0.76662E+01_JPRB, 0.67718E+01_JPRB ,&
&  0.59293E+01_JPRB, 0.54461E+01_JPRB, 0.43892E+01_JPRB, 0.42612E+01_JPRB, 0.36159E+01_JPRB, 0.31047E+01_JPRB, 0.37543E+01_JPRB ,&
&  0.42197E+01_JPRB, 0.40885E+01_JPRB, 0.34356E+01_JPRB, 0.27973E+01_JPRB, 0.26345E+01_JPRB, 0.17659E+01_JPRB, 0.11501E+01_JPRB ,&
&  0.99723E+00_JPRB, 0.94122E+00_JPRB, 0.79581E+00_JPRB, 0.86811E+00_JPRB, 0.78580E+00_JPRB, 0.98948E+00_JPRB, 0.16075E+01_JPRB ,&
&  0.31594E+01_JPRB, 0.47938E+01_JPRB, 0.51510E+01_JPRB, 0.42788E+01_JPRB, 0.35350E+01_JPRB, 0.32518E+01_JPRB, 0.40974E+01_JPRB ,&
&  0.29141E+01_JPRB, 0.26290E+01_JPRB, 0.17371E+01_JPRB, 0.12063E+01_JPRB, 0.99358E+00_JPRB, 0.89896E+00_JPRB, 0.87576E+00_JPRB ,&
&  0.87280E+00_JPRB, 0.86503E+00_JPRB, 0.87534E+00_JPRB, 0.88872E+00_JPRB, 0.88904E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.93861E+00_JPRB, 0.94019E+00_JPRB, 0.89665E+00_JPRB, 0.89917E+00_JPRB, 0.92916E+00_JPRB, 0.95177E+00_JPRB, 0.94103E+00_JPRB ,&
&  0.92034E+00_JPRB, 0.92602E+00_JPRB, 0.92309E+00_JPRB, 0.89798E+00_JPRB, 0.87238E+00_JPRB, 0.87320E+00_JPRB, 0.87615E+00_JPRB ,&
&  0.92710E+00_JPRB, 0.10908E+01_JPRB, 0.92171E+00_JPRB, 0.90636E+00_JPRB, 0.99936E+00_JPRB, 0.11294E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.19411E+01_JPRB, 0.24597E+01_JPRB, 0.28312E+01_JPRB, 0.35502E+01_JPRB, 0.45905E+01_JPRB, 0.43916E+01_JPRB, 0.41505E+01_JPRB ,&
&  0.35811E+01_JPRB, 0.29918E+01_JPRB, 0.27620E+01_JPRB, 0.26035E+01_JPRB, 0.25687E+01_JPRB, 0.25441E+01_JPRB, 0.23926E+01_JPRB ,&
&  0.20825E+01_JPRB, 0.16733E+01_JPRB, 0.14014E+01_JPRB, 0.13516E+01_JPRB, 0.12945E+01_JPRB, 0.13412E+01_JPRB, 0.15108E+01_JPRB ,&
&  0.21391E+01_JPRB, 0.29405E+01_JPRB, 0.33760E+01_JPRB, 0.35264E+01_JPRB, 0.33757E+01_JPRB, 0.42445E+01_JPRB, 0.70121E+01_JPRB ,&
&  0.71891E+01_JPRB, 0.77197E+01_JPRB, 0.85064E+01_JPRB, 0.10101E+02_JPRB, 0.89389E+01_JPRB, 0.72060E+01_JPRB, 0.69721E+01_JPRB ,&
&  0.50310E+01_JPRB, 0.41365E+01_JPRB, 0.43276E+01_JPRB, 0.42594E+01_JPRB, 0.35911E+01_JPRB, 0.39980E+01_JPRB, 0.58465E+01_JPRB ,&
&  0.46654E+01_JPRB, 0.49381E+01_JPRB, 0.50240E+01_JPRB, 0.49512E+01_JPRB, 0.27723E+01_JPRB, 0.20537E+01_JPRB, 0.30825E+01_JPRB ,&
&  0.33176E+01_JPRB, 0.19699E+01_JPRB, 0.12013E+01_JPRB, 0.17040E+01_JPRB, 0.11948E+01_JPRB, 0.13818E+01_JPRB, 0.20471E+01_JPRB ,&
&  0.26731E+01_JPRB, 0.35319E+01_JPRB, 0.55535E+01_JPRB, 0.47284E+01_JPRB, 0.37556E+01_JPRB, 0.33144E+01_JPRB, 0.35002E+01_JPRB ,&
&  0.33867E+01_JPRB, 0.28315E+01_JPRB, 0.23083E+01_JPRB, 0.16737E+01_JPRB, 0.12752E+01_JPRB, 0.11051E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10561E+01_JPRB, 0.10451E+01_JPRB, 0.10025E+01_JPRB, 0.96234E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.10480E+01_JPRB, 0.10417E+01_JPRB, 0.10038E+01_JPRB, 0.10158E+01_JPRB, 0.10497E+01_JPRB, 0.10172E+01_JPRB, 0.98835E+00_JPRB ,&
&  0.94603E+00_JPRB, 0.92092E+00_JPRB, 0.92309E+00_JPRB, 0.91607E+00_JPRB, 0.89451E+00_JPRB, 0.88563E+00_JPRB, 0.88523E+00_JPRB ,&
&  0.92911E+00_JPRB, 0.95964E+00_JPRB, 0.87251E+00_JPRB, 0.84660E+00_JPRB, 0.94512E+00_JPRB, 0.10141E+01_JPRB, 0.12160E+01_JPRB ,&
&  0.17523E+01_JPRB, 0.23599E+01_JPRB, 0.30113E+01_JPRB, 0.37826E+01_JPRB, 0.43863E+01_JPRB, 0.49191E+01_JPRB, 0.51343E+01_JPRB ,&
&  0.49522E+01_JPRB, 0.41349E+01_JPRB, 0.36163E+01_JPRB, 0.32444E+01_JPRB, 0.29246E+01_JPRB, 0.26693E+01_JPRB, 0.24599E+01_JPRB ,&
&  0.22743E+01_JPRB, 0.19529E+01_JPRB, 0.16059E+01_JPRB, 0.14013E+01_JPRB, 0.13784E+01_JPRB, 0.13907E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.20660E+01_JPRB, 0.28354E+01_JPRB, 0.37493E+01_JPRB, 0.44115E+01_JPRB, 0.40210E+01_JPRB, 0.46322E+01_JPRB, 0.66343E+01_JPRB ,&
&  0.77959E+01_JPRB, 0.80679E+01_JPRB, 0.94984E+01_JPRB, 0.10252E+02_JPRB, 0.97331E+01_JPRB, 0.79177E+01_JPRB, 0.79904E+01_JPRB ,&
&  0.59501E+01_JPRB, 0.46992E+01_JPRB, 0.41575E+01_JPRB, 0.38009E+01_JPRB, 0.44356E+01_JPRB, 0.62366E+01_JPRB, 0.58185E+01_JPRB ,&
&  0.55797E+01_JPRB, 0.53449E+01_JPRB, 0.49191E+01_JPRB, 0.42008E+01_JPRB, 0.39254E+01_JPRB, 0.34895E+01_JPRB, 0.43741E+01_JPRB ,&
&  0.43022E+01_JPRB, 0.40926E+01_JPRB, 0.33949E+01_JPRB, 0.27317E+01_JPRB, 0.21621E+01_JPRB, 0.23904E+01_JPRB, 0.24327E+01_JPRB ,&
&  0.25806E+01_JPRB, 0.28106E+01_JPRB, 0.44606E+01_JPRB, 0.46766E+01_JPRB, 0.38777E+01_JPRB, 0.31284E+01_JPRB, 0.34181E+01_JPRB ,&
&  0.30256E+01_JPRB, 0.29873E+01_JPRB, 0.24966E+01_JPRB, 0.19507E+01_JPRB, 0.16283E+01_JPRB, 0.14578E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13290E+01_JPRB, 0.12762E+01_JPRB, 0.11888E+01_JPRB, 0.11181E+01_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.11880E+01_JPRB, 0.11599E+01_JPRB, 0.11285E+01_JPRB, 0.11191E+01_JPRB, 0.11320E+01_JPRB, 0.10899E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.96139E+00_JPRB, 0.95356E+00_JPRB, 0.93845E+00_JPRB, 0.91460E+00_JPRB, 0.89894E+00_JPRB, 0.90389E+00_JPRB ,&
&  0.94803E+00_JPRB, 0.97328E+00_JPRB, 0.87325E+00_JPRB, 0.90802E+00_JPRB, 0.93773E+00_JPRB, 0.10019E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.16827E+01_JPRB, 0.21351E+01_JPRB, 0.27493E+01_JPRB, 0.34679E+01_JPRB, 0.40474E+01_JPRB, 0.47170E+01_JPRB, 0.54419E+01_JPRB ,&
&  0.51120E+01_JPRB, 0.50165E+01_JPRB, 0.43601E+01_JPRB, 0.36992E+01_JPRB, 0.31477E+01_JPRB, 0.27328E+01_JPRB, 0.25230E+01_JPRB ,&
&  0.23589E+01_JPRB, 0.22501E+01_JPRB, 0.20117E+01_JPRB, 0.17305E+01_JPRB, 0.15882E+01_JPRB, 0.15443E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.17634E+01_JPRB, 0.25105E+01_JPRB, 0.37629E+01_JPRB, 0.52455E+01_JPRB, 0.46968E+01_JPRB, 0.54960E+01_JPRB, 0.64229E+01_JPRB ,&
&  0.95781E+01_JPRB, 0.10322E+02_JPRB, 0.10117E+02_JPRB, 0.10395E+02_JPRB, 0.79810E+01_JPRB, 0.78196E+01_JPRB, 0.79586E+01_JPRB ,&
&  0.73645E+01_JPRB, 0.68036E+01_JPRB, 0.72615E+01_JPRB, 0.58070E+01_JPRB, 0.50788E+01_JPRB, 0.60847E+01_JPRB, 0.51759E+01_JPRB ,&
&  0.47227E+01_JPRB, 0.45141E+01_JPRB, 0.41552E+01_JPRB, 0.44094E+01_JPRB, 0.40541E+01_JPRB, 0.40411E+01_JPRB, 0.33382E+01_JPRB ,&
&  0.32075E+01_JPRB, 0.26629E+01_JPRB, 0.29419E+01_JPRB, 0.25903E+01_JPRB, 0.24304E+01_JPRB, 0.23339E+01_JPRB, 0.22513E+01_JPRB ,&
&  0.24346E+01_JPRB, 0.25401E+01_JPRB, 0.31122E+01_JPRB, 0.36402E+01_JPRB, 0.37407E+01_JPRB, 0.29700E+01_JPRB, 0.28075E+01_JPRB ,&
&  0.28335E+01_JPRB, 0.27879E+01_JPRB, 0.23607E+01_JPRB, 0.19927E+01_JPRB, 0.17812E+01_JPRB, 0.16858E+01_JPRB, 0.16183E+01_JPRB ,&
&  0.15310E+01_JPRB, 0.14583E+01_JPRB, 0.13945E+01_JPRB, 0.13427E+01_JPRB, 0.12825E+01_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.12171E+01_JPRB, 0.11706E+01_JPRB, 0.12161E+01_JPRB, 0.11878E+01_JPRB, 0.11887E+01_JPRB, 0.11443E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.10793E+01_JPRB, 0.10420E+01_JPRB, 0.10040E+01_JPRB, 0.96532E+00_JPRB, 0.93139E+00_JPRB, 0.90734E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.92807E+00_JPRB, 0.10623E+01_JPRB, 0.96825E+00_JPRB, 0.91181E+00_JPRB, 0.91238E+00_JPRB, 0.10799E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.16105E+01_JPRB, 0.18691E+01_JPRB, 0.23542E+01_JPRB, 0.28486E+01_JPRB, 0.33899E+01_JPRB, 0.39615E+01_JPRB, 0.46633E+01_JPRB ,&
&  0.52054E+01_JPRB, 0.44160E+01_JPRB, 0.41367E+01_JPRB, 0.36252E+01_JPRB, 0.31988E+01_JPRB, 0.28346E+01_JPRB, 0.25823E+01_JPRB ,&
&  0.23835E+01_JPRB, 0.22470E+01_JPRB, 0.20742E+01_JPRB, 0.18686E+01_JPRB, 0.16667E+01_JPRB, 0.15591E+01_JPRB, 0.15374E+01_JPRB ,&
&  0.16171E+01_JPRB, 0.19945E+01_JPRB, 0.30129E+01_JPRB, 0.47163E+01_JPRB, 0.73553E+01_JPRB, 0.82814E+01_JPRB, 0.82202E+01_JPRB ,&
&  0.73868E+01_JPRB, 0.76695E+01_JPRB, 0.11407E+02_JPRB, 0.94054E+01_JPRB, 0.90976E+01_JPRB, 0.90323E+01_JPRB, 0.82906E+01_JPRB ,&
&  0.75587E+01_JPRB, 0.62969E+01_JPRB, 0.59773E+01_JPRB, 0.52114E+01_JPRB, 0.58946E+01_JPRB, 0.50593E+01_JPRB, 0.43780E+01_JPRB ,&
&  0.39680E+01_JPRB, 0.39726E+01_JPRB, 0.38202E+01_JPRB, 0.37757E+01_JPRB, 0.39406E+01_JPRB, 0.44964E+01_JPRB, 0.44736E+01_JPRB ,&
&  0.39371E+01_JPRB, 0.40388E+01_JPRB, 0.32908E+01_JPRB, 0.25914E+01_JPRB, 0.20952E+01_JPRB, 0.20711E+01_JPRB, 0.21097E+01_JPRB ,&
&  0.22032E+01_JPRB, 0.23532E+01_JPRB, 0.27251E+01_JPRB, 0.29807E+01_JPRB, 0.30582E+01_JPRB, 0.27983E+01_JPRB, 0.25560E+01_JPRB ,&
&  0.23590E+01_JPRB, 0.24734E+01_JPRB, 0.23855E+01_JPRB, 0.19660E+01_JPRB, 0.18095E+01_JPRB, 0.17468E+01_JPRB, 0.16793E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.14548E+01_JPRB, 0.13831E+01_JPRB, 0.13457E+01_JPRB, 0.13106E+01_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.12621E+01_JPRB, 0.11710E+01_JPRB, 0.12124E+01_JPRB, 0.11987E+01_JPRB, 0.11752E+01_JPRB, 0.11430E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.10780E+01_JPRB, 0.10561E+01_JPRB, 0.10306E+01_JPRB, 0.98697E+00_JPRB, 0.94965E+00_JPRB, 0.92608E+00_JPRB, 0.91701E+00_JPRB ,&
&  0.91925E+00_JPRB, 0.11099E+01_JPRB, 0.10720E+01_JPRB, 0.10935E+01_JPRB, 0.10729E+01_JPRB, 0.11933E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.14766E+01_JPRB, 0.16176E+01_JPRB, 0.18141E+01_JPRB, 0.21185E+01_JPRB, 0.24569E+01_JPRB, 0.28190E+01_JPRB, 0.33562E+01_JPRB ,&
&  0.35375E+01_JPRB, 0.35121E+01_JPRB, 0.33290E+01_JPRB, 0.33916E+01_JPRB, 0.32338E+01_JPRB, 0.27791E+01_JPRB, 0.26017E+01_JPRB ,&
&  0.23246E+01_JPRB, 0.21326E+01_JPRB, 0.19831E+01_JPRB, 0.18905E+01_JPRB, 0.18047E+01_JPRB, 0.16982E+01_JPRB, 0.16310E+01_JPRB ,&
&  0.15755E+01_JPRB, 0.16586E+01_JPRB, 0.21309E+01_JPRB, 0.33753E+01_JPRB, 0.56922E+01_JPRB, 0.77946E+01_JPRB, 0.83561E+01_JPRB ,&
&  0.92243E+01_JPRB, 0.91115E+01_JPRB, 0.72434E+01_JPRB, 0.82218E+01_JPRB, 0.10355E+02_JPRB, 0.92710E+01_JPRB, 0.79625E+01_JPRB ,&
&  0.78603E+01_JPRB, 0.73165E+01_JPRB, 0.70116E+01_JPRB, 0.60960E+01_JPRB, 0.51821E+01_JPRB, 0.46586E+01_JPRB, 0.42680E+01_JPRB ,&
&  0.39663E+01_JPRB, 0.39137E+01_JPRB, 0.39308E+01_JPRB, 0.39267E+01_JPRB, 0.37243E+01_JPRB, 0.37449E+01_JPRB, 0.40651E+01_JPRB ,&
&  0.39937E+01_JPRB, 0.36277E+01_JPRB, 0.26615E+01_JPRB, 0.25715E+01_JPRB, 0.18546E+01_JPRB, 0.17736E+01_JPRB, 0.19901E+01_JPRB ,&
&  0.19348E+01_JPRB, 0.21746E+01_JPRB, 0.24112E+01_JPRB, 0.22346E+01_JPRB, 0.23320E+01_JPRB, 0.22685E+01_JPRB, 0.23035E+01_JPRB ,&
&  0.22461E+01_JPRB, 0.19695E+01_JPRB, 0.21339E+01_JPRB, 0.18515E+01_JPRB, 0.17268E+01_JPRB, 0.16892E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.15861E+01_JPRB, 0.15141E+01_JPRB, 0.14423E+01_JPRB, 0.14043E+01_JPRB, 0.13570E+01_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.12871E+01_JPRB, 0.11926E+01_JPRB, 0.11906E+01_JPRB, 0.11563E+01_JPRB, 0.11201E+01_JPRB, 0.10854E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.10409E+01_JPRB, 0.10225E+01_JPRB, 0.10020E+01_JPRB, 0.97126E+00_JPRB, 0.94437E+00_JPRB, 0.91805E+00_JPRB, 0.89962E+00_JPRB ,&
&  0.90731E+00_JPRB, 0.10096E+01_JPRB, 0.10474E+01_JPRB, 0.10143E+01_JPRB, 0.11718E+01_JPRB, 0.11749E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12872E+01_JPRB, 0.14024E+01_JPRB, 0.14343E+01_JPRB, 0.14857E+01_JPRB, 0.16006E+01_JPRB, 0.18594E+01_JPRB, 0.21951E+01_JPRB ,&
&  0.23154E+01_JPRB, 0.22378E+01_JPRB, 0.22432E+01_JPRB, 0.23664E+01_JPRB, 0.25328E+01_JPRB, 0.22471E+01_JPRB, 0.21581E+01_JPRB ,&
&  0.19772E+01_JPRB, 0.18298E+01_JPRB, 0.17233E+01_JPRB, 0.16542E+01_JPRB, 0.16383E+01_JPRB, 0.16493E+01_JPRB, 0.16740E+01_JPRB ,&
&  0.16673E+01_JPRB, 0.16094E+01_JPRB, 0.17171E+01_JPRB, 0.24950E+01_JPRB, 0.48823E+01_JPRB, 0.79986E+01_JPRB, 0.10371E+02_JPRB ,&
&  0.10236E+02_JPRB, 0.10001E+02_JPRB, 0.10369E+02_JPRB, 0.11039E+02_JPRB, 0.99498E+01_JPRB, 0.85998E+01_JPRB, 0.77770E+01_JPRB ,&
&  0.72623E+01_JPRB, 0.70013E+01_JPRB, 0.70048E+01_JPRB, 0.64563E+01_JPRB, 0.57362E+01_JPRB, 0.53519E+01_JPRB, 0.49971E+01_JPRB ,&
&  0.46876E+01_JPRB, 0.46598E+01_JPRB, 0.46656E+01_JPRB, 0.43978E+01_JPRB, 0.42261E+01_JPRB, 0.39456E+01_JPRB, 0.41165E+01_JPRB ,&
&  0.44734E+01_JPRB, 0.32530E+01_JPRB, 0.26071E+01_JPRB, 0.29252E+01_JPRB, 0.21434E+01_JPRB, 0.20632E+01_JPRB, 0.23795E+01_JPRB ,&
&  0.21584E+01_JPRB, 0.20314E+01_JPRB, 0.20832E+01_JPRB, 0.18908E+01_JPRB, 0.18860E+01_JPRB, 0.20087E+01_JPRB, 0.19785E+01_JPRB ,&
&  0.18933E+01_JPRB, 0.17948E+01_JPRB, 0.19787E+01_JPRB, 0.17167E+01_JPRB, 0.15919E+01_JPRB, 0.15604E+01_JPRB, 0.15158E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.14299E+01_JPRB, 0.13715E+01_JPRB, 0.13389E+01_JPRB, 0.13332E+01_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.12034E+01_JPRB, 0.11749E+01_JPRB, 0.11587E+01_JPRB, 0.11463E+01_JPRB, 0.11174E+01_JPRB, 0.10825E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.99633E+00_JPRB, 0.96579E+00_JPRB, 0.93078E+00_JPRB, 0.90302E+00_JPRB, 0.87655E+00_JPRB, 0.86328E+00_JPRB ,&
&  0.77342E+00_JPRB, 0.79559E+00_JPRB, 0.81080E+00_JPRB, 0.87282E+00_JPRB, 0.10806E+01_JPRB, 0.10566E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.11788E+01_JPRB, 0.11899E+01_JPRB, 0.12185E+01_JPRB, 0.12870E+01_JPRB, 0.14408E+01_JPRB, 0.16337E+01_JPRB ,&
&  0.16060E+01_JPRB, 0.14569E+01_JPRB, 0.14243E+01_JPRB, 0.14153E+01_JPRB, 0.14877E+01_JPRB, 0.15594E+01_JPRB, 0.16514E+01_JPRB ,&
&  0.16111E+01_JPRB, 0.15478E+01_JPRB, 0.15002E+01_JPRB, 0.14663E+01_JPRB, 0.14421E+01_JPRB, 0.14389E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.14862E+01_JPRB, 0.15157E+01_JPRB, 0.19480E+01_JPRB, 0.40189E+01_JPRB, 0.79009E+01_JPRB, 0.12342E+02_JPRB ,&
&  0.11437E+02_JPRB, 0.10097E+02_JPRB, 0.10217E+02_JPRB, 0.10850E+02_JPRB, 0.95376E+01_JPRB, 0.79646E+01_JPRB, 0.67466E+01_JPRB ,&
&  0.61715E+01_JPRB, 0.57837E+01_JPRB, 0.58828E+01_JPRB, 0.59592E+01_JPRB, 0.56753E+01_JPRB, 0.55997E+01_JPRB, 0.54564E+01_JPRB ,&
&  0.52598E+01_JPRB, 0.52095E+01_JPRB, 0.50042E+01_JPRB, 0.47892E+01_JPRB, 0.46589E+01_JPRB, 0.47687E+01_JPRB, 0.47019E+01_JPRB ,&
&  0.47641E+01_JPRB, 0.45019E+01_JPRB, 0.38635E+01_JPRB, 0.37578E+01_JPRB, 0.29026E+01_JPRB, 0.27448E+01_JPRB, 0.32375E+01_JPRB ,&
&  0.26604E+01_JPRB, 0.21751E+01_JPRB, 0.19907E+01_JPRB, 0.19320E+01_JPRB, 0.19121E+01_JPRB, 0.19986E+01_JPRB, 0.18875E+01_JPRB ,&
&  0.16650E+01_JPRB, 0.17368E+01_JPRB, 0.18162E+01_JPRB, 0.15385E+01_JPRB, 0.14016E+01_JPRB, 0.13367E+01_JPRB, 0.12401E+01_JPRB ,&
&  0.12636E+01_JPRB, 0.12343E+01_JPRB, 0.12116E+01_JPRB, 0.11951E+01_JPRB, 0.11984E+01_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.10905E+01_JPRB, 0.10989E+01_JPRB, 0.11009E+01_JPRB, 0.10944E+01_JPRB, 0.10722E+01_JPRB, 0.10417E+01_JPRB, 0.10109E+01_JPRB ,&
&  0.98409E+00_JPRB, 0.94592E+00_JPRB, 0.90770E+00_JPRB, 0.87052E+00_JPRB, 0.84715E+00_JPRB, 0.83277E+00_JPRB, 0.77153E+00_JPRB ,&
&  0.70347E+00_JPRB, 0.71689E+00_JPRB, 0.71792E+00_JPRB, 0.75046E+00_JPRB, 0.81857E+00_JPRB, 0.84717E+00_JPRB, 0.88271E+00_JPRB ,&
&  0.97016E+00_JPRB, 0.10374E+01_JPRB, 0.10812E+01_JPRB, 0.11251E+01_JPRB, 0.11540E+01_JPRB, 0.11889E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.11764E+01_JPRB, 0.10728E+01_JPRB, 0.10031E+01_JPRB, 0.95596E+00_JPRB, 0.10088E+01_JPRB, 0.10957E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.11960E+01_JPRB, 0.12060E+01_JPRB, 0.12165E+01_JPRB, 0.12184E+01_JPRB, 0.12071E+01_JPRB, 0.12025E+01_JPRB, 0.12019E+01_JPRB ,&
&  0.12230E+01_JPRB, 0.12482E+01_JPRB, 0.12852E+01_JPRB, 0.14847E+01_JPRB, 0.30347E+01_JPRB, 0.59862E+01_JPRB, 0.83248E+01_JPRB ,&
&  0.74052E+01_JPRB, 0.71211E+01_JPRB, 0.70787E+01_JPRB, 0.70601E+01_JPRB, 0.66484E+01_JPRB, 0.58440E+01_JPRB, 0.51864E+01_JPRB ,&
&  0.50281E+01_JPRB, 0.47153E+01_JPRB, 0.49892E+01_JPRB, 0.49001E+01_JPRB, 0.48910E+01_JPRB, 0.49334E+01_JPRB, 0.47815E+01_JPRB ,&
&  0.47483E+01_JPRB, 0.45840E+01_JPRB, 0.45668E+01_JPRB, 0.44382E+01_JPRB, 0.44062E+01_JPRB, 0.44775E+01_JPRB, 0.44814E+01_JPRB ,&
&  0.45206E+01_JPRB, 0.45199E+01_JPRB, 0.41808E+01_JPRB, 0.39282E+01_JPRB, 0.36407E+01_JPRB, 0.38074E+01_JPRB, 0.34407E+01_JPRB ,&
&  0.28520E+01_JPRB, 0.21563E+01_JPRB, 0.20764E+01_JPRB, 0.19466E+01_JPRB, 0.18761E+01_JPRB, 0.17918E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.17612E+01_JPRB, 0.16479E+01_JPRB, 0.14005E+01_JPRB, 0.12717E+01_JPRB, 0.11715E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.99450E+00_JPRB, 0.10720E+01_JPRB, 0.10864E+01_JPRB, 0.10808E+01_JPRB, 0.10751E+01_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.10390E+01_JPRB, 0.10436E+01_JPRB, 0.10393E+01_JPRB, 0.10352E+01_JPRB, 0.10239E+01_JPRB, 0.10002E+01_JPRB, 0.94876E+00_JPRB ,&
&  0.95333E+00_JPRB, 0.92047E+00_JPRB, 0.87722E+00_JPRB, 0.84307E+00_JPRB, 0.82606E+00_JPRB, 0.76350E+00_JPRB, 0.66527E+00_JPRB ,&
&  0.63653E+00_JPRB, 0.64830E+00_JPRB, 0.67309E+00_JPRB, 0.69196E+00_JPRB, 0.70984E+00_JPRB, 0.72172E+00_JPRB, 0.75022E+00_JPRB ,&
&  0.78913E+00_JPRB, 0.82632E+00_JPRB, 0.87445E+00_JPRB, 0.92164E+00_JPRB, 0.92851E+00_JPRB, 0.92500E+00_JPRB, 0.93723E+00_JPRB ,&
&  0.89633E+00_JPRB, 0.84347E+00_JPRB, 0.81300E+00_JPRB, 0.74933E+00_JPRB, 0.79927E+00_JPRB, 0.83082E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.89791E+00_JPRB, 0.92285E+00_JPRB, 0.95124E+00_JPRB, 0.96741E+00_JPRB, 0.96909E+00_JPRB, 0.97314E+00_JPRB, 0.98019E+00_JPRB ,&
&  0.99327E+00_JPRB, 0.10258E+01_JPRB, 0.10813E+01_JPRB, 0.11875E+01_JPRB, 0.21215E+01_JPRB, 0.39867E+01_JPRB, 0.51479E+01_JPRB ,&
&  0.48013E+01_JPRB, 0.51138E+01_JPRB, 0.52130E+01_JPRB, 0.46326E+01_JPRB, 0.46715E+01_JPRB, 0.47083E+01_JPRB, 0.44429E+01_JPRB ,&
&  0.45424E+01_JPRB, 0.44383E+01_JPRB, 0.42733E+01_JPRB, 0.41054E+01_JPRB, 0.39610E+01_JPRB, 0.37895E+01_JPRB, 0.36441E+01_JPRB ,&
&  0.36367E+01_JPRB, 0.37851E+01_JPRB, 0.38642E+01_JPRB, 0.40204E+01_JPRB, 0.40816E+01_JPRB, 0.41812E+01_JPRB, 0.42593E+01_JPRB ,&
&  0.42702E+01_JPRB, 0.42815E+01_JPRB, 0.42207E+01_JPRB, 0.40290E+01_JPRB, 0.38137E+01_JPRB, 0.35793E+01_JPRB, 0.33088E+01_JPRB ,&
&  0.30631E+01_JPRB, 0.26410E+01_JPRB, 0.23083E+01_JPRB, 0.21476E+01_JPRB, 0.19739E+01_JPRB, 0.17442E+01_JPRB, 0.16599E+01_JPRB ,&
&  0.15888E+01_JPRB, 0.14340E+01_JPRB, 0.14050E+01_JPRB, 0.12576E+01_JPRB, 0.11484E+01_JPRB, 0.10687E+01_JPRB, 0.95293E+00_JPRB ,&
&  0.91235E+00_JPRB, 0.97233E+00_JPRB, 0.10184E+01_JPRB, 0.10414E+01_JPRB, 0.10407E+01_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.10006E+01_JPRB, 0.10076E+01_JPRB, 0.10074E+01_JPRB, 0.10043E+01_JPRB, 0.97597E+00_JPRB, 0.91690E+00_JPRB, 0.86323E+00_JPRB ,&
&  0.90107E+00_JPRB, 0.85415E+00_JPRB, 0.85249E+00_JPRB, 0.73037E+00_JPRB, 0.68240E+00_JPRB, 0.66678E+00_JPRB, 0.63812E+00_JPRB ,&
&  0.62520E+00_JPRB, 0.62776E+00_JPRB, 0.63027E+00_JPRB, 0.63742E+00_JPRB, 0.64785E+00_JPRB, 0.64978E+00_JPRB, 0.65390E+00_JPRB ,&
&  0.66601E+00_JPRB, 0.68552E+00_JPRB, 0.72113E+00_JPRB, 0.74146E+00_JPRB, 0.75554E+00_JPRB, 0.76171E+00_JPRB, 0.76803E+00_JPRB ,&
&  0.73206E+00_JPRB, 0.71328E+00_JPRB, 0.71361E+00_JPRB, 0.69730E+00_JPRB, 0.69517E+00_JPRB, 0.70519E+00_JPRB, 0.72371E+00_JPRB ,&
&  0.74145E+00_JPRB, 0.71135E+00_JPRB, 0.77127E+00_JPRB, 0.80109E+00_JPRB, 0.82044E+00_JPRB, 0.83402E+00_JPRB, 0.84551E+00_JPRB ,&
&  0.85856E+00_JPRB, 0.88481E+00_JPRB, 0.92917E+00_JPRB, 0.10212E+01_JPRB, 0.15017E+01_JPRB, 0.26787E+01_JPRB, 0.33161E+01_JPRB ,&
&  0.35231E+01_JPRB, 0.27285E+01_JPRB, 0.31196E+01_JPRB, 0.32760E+01_JPRB, 0.35039E+01_JPRB, 0.36671E+01_JPRB, 0.37113E+01_JPRB ,&
&  0.40257E+01_JPRB, 0.40700E+01_JPRB, 0.38131E+01_JPRB, 0.33986E+01_JPRB, 0.30610E+01_JPRB, 0.28867E+01_JPRB, 0.28044E+01_JPRB ,&
&  0.27897E+01_JPRB, 0.26728E+01_JPRB, 0.30072E+01_JPRB, 0.33267E+01_JPRB, 0.35203E+01_JPRB, 0.37054E+01_JPRB, 0.39223E+01_JPRB ,&
&  0.40550E+01_JPRB, 0.40311E+01_JPRB, 0.39700E+01_JPRB, 0.36706E+01_JPRB, 0.35065E+01_JPRB, 0.33782E+01_JPRB, 0.31719E+01_JPRB ,&
&  0.30327E+01_JPRB, 0.29102E+01_JPRB, 0.27355E+01_JPRB, 0.26951E+01_JPRB, 0.23223E+01_JPRB, 0.20662E+01_JPRB, 0.18860E+01_JPRB ,&
&  0.15606E+01_JPRB, 0.12214E+01_JPRB, 0.10853E+01_JPRB, 0.10134E+01_JPRB, 0.95778E+00_JPRB, 0.94223E+00_JPRB, 0.94180E+00_JPRB ,&
&  0.90585E+00_JPRB, 0.91759E+00_JPRB, 0.97196E+00_JPRB, 0.98423E+00_JPRB, 0.98945E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.97230E+00_JPRB, 0.99355E+00_JPRB, 0.10069E+01_JPRB, 0.99646E+00_JPRB, 0.94924E+00_JPRB, 0.87290E+00_JPRB, 0.78676E+00_JPRB ,&
&  0.75166E+00_JPRB, 0.71532E+00_JPRB, 0.68761E+00_JPRB, 0.66359E+00_JPRB, 0.64641E+00_JPRB, 0.62346E+00_JPRB, 0.59870E+00_JPRB ,&
&  0.59447E+00_JPRB, 0.60457E+00_JPRB, 0.61725E+00_JPRB, 0.61753E+00_JPRB, 0.61476E+00_JPRB, 0.61239E+00_JPRB, 0.60900E+00_JPRB ,&
&  0.60460E+00_JPRB, 0.60780E+00_JPRB, 0.62263E+00_JPRB, 0.64530E+00_JPRB, 0.65536E+00_JPRB, 0.65655E+00_JPRB, 0.65963E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.63823E+00_JPRB, 0.62877E+00_JPRB, 0.63061E+00_JPRB, 0.62909E+00_JPRB, 0.64195E+00_JPRB, 0.64158E+00_JPRB ,&
&  0.58825E+00_JPRB, 0.60628E+00_JPRB, 0.67940E+00_JPRB, 0.71118E+00_JPRB, 0.73438E+00_JPRB, 0.75230E+00_JPRB, 0.76872E+00_JPRB ,&
&  0.78474E+00_JPRB, 0.81844E+00_JPRB, 0.86680E+00_JPRB, 0.93720E+00_JPRB, 0.12055E+01_JPRB, 0.18357E+01_JPRB, 0.23120E+01_JPRB ,&
&  0.27665E+01_JPRB, 0.24041E+01_JPRB, 0.22411E+01_JPRB, 0.23516E+01_JPRB, 0.28175E+01_JPRB, 0.29872E+01_JPRB, 0.31374E+01_JPRB ,&
&  0.33121E+01_JPRB, 0.32670E+01_JPRB, 0.31345E+01_JPRB, 0.29177E+01_JPRB, 0.25811E+01_JPRB, 0.23468E+01_JPRB, 0.22366E+01_JPRB ,&
&  0.21137E+01_JPRB, 0.20680E+01_JPRB, 0.21543E+01_JPRB, 0.24197E+01_JPRB, 0.27538E+01_JPRB, 0.30338E+01_JPRB, 0.32491E+01_JPRB ,&
&  0.34687E+01_JPRB, 0.35695E+01_JPRB, 0.34984E+01_JPRB, 0.33731E+01_JPRB, 0.31724E+01_JPRB, 0.30489E+01_JPRB, 0.29984E+01_JPRB ,&
&  0.29315E+01_JPRB, 0.28664E+01_JPRB, 0.27366E+01_JPRB, 0.25636E+01_JPRB, 0.22906E+01_JPRB, 0.20372E+01_JPRB, 0.17421E+01_JPRB ,&
&  0.13186E+01_JPRB, 0.10771E+01_JPRB, 0.91353E+00_JPRB, 0.85753E+00_JPRB, 0.84464E+00_JPRB, 0.84808E+00_JPRB, 0.83748E+00_JPRB ,&
&  0.85964E+00_JPRB, 0.87768E+00_JPRB, 0.87850E+00_JPRB, 0.90755E+00_JPRB, 0.94345E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.92957E+00_JPRB, 0.94831E+00_JPRB, 0.94641E+00_JPRB, 0.93372E+00_JPRB, 0.87099E+00_JPRB, 0.81178E+00_JPRB, 0.75533E+00_JPRB ,&
&  0.72172E+00_JPRB, 0.69037E+00_JPRB, 0.66816E+00_JPRB, 0.64391E+00_JPRB, 0.62462E+00_JPRB, 0.61596E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.62346E+00_JPRB, 0.62062E+00_JPRB, 0.61433E+00_JPRB, 0.60165E+00_JPRB, 0.59155E+00_JPRB, 0.58284E+00_JPRB, 0.57449E+00_JPRB ,&
&  0.56513E+00_JPRB, 0.56481E+00_JPRB, 0.57104E+00_JPRB, 0.58488E+00_JPRB, 0.59256E+00_JPRB, 0.59644E+00_JPRB, 0.59448E+00_JPRB ,&
&  0.58223E+00_JPRB, 0.56613E+00_JPRB, 0.56772E+00_JPRB, 0.57891E+00_JPRB, 0.58458E+00_JPRB, 0.58907E+00_JPRB, 0.56201E+00_JPRB ,&
&  0.54268E+00_JPRB, 0.54113E+00_JPRB, 0.56124E+00_JPRB, 0.64039E+00_JPRB, 0.67786E+00_JPRB, 0.69866E+00_JPRB, 0.71870E+00_JPRB ,&
&  0.71108E+00_JPRB, 0.72046E+00_JPRB, 0.76638E+00_JPRB, 0.87342E+00_JPRB, 0.10220E+01_JPRB, 0.12880E+01_JPRB, 0.16447E+01_JPRB ,&
&  0.19449E+01_JPRB, 0.21638E+01_JPRB, 0.22810E+01_JPRB, 0.20615E+01_JPRB, 0.20180E+01_JPRB, 0.23538E+01_JPRB, 0.25156E+01_JPRB ,&
&  0.25951E+01_JPRB, 0.26039E+01_JPRB, 0.26666E+01_JPRB, 0.24932E+01_JPRB, 0.21681E+01_JPRB, 0.19470E+01_JPRB, 0.17929E+01_JPRB ,&
&  0.16282E+01_JPRB, 0.15253E+01_JPRB, 0.14992E+01_JPRB, 0.15346E+01_JPRB, 0.16343E+01_JPRB, 0.18577E+01_JPRB, 0.21557E+01_JPRB ,&
&  0.24469E+01_JPRB, 0.26180E+01_JPRB, 0.27040E+01_JPRB, 0.26503E+01_JPRB, 0.25825E+01_JPRB, 0.24961E+01_JPRB, 0.24364E+01_JPRB ,&
&  0.23701E+01_JPRB, 0.23448E+01_JPRB, 0.23131E+01_JPRB, 0.22690E+01_JPRB, 0.20376E+01_JPRB, 0.16771E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.95848E+00_JPRB, 0.86386E+00_JPRB, 0.83683E+00_JPRB, 0.83144E+00_JPRB, 0.82458E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.76809E+00_JPRB, 0.78013E+00_JPRB, 0.80594E+00_JPRB, 0.83646E+00_JPRB, 0.88213E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.90383E+00_JPRB, 0.90111E+00_JPRB, 0.87218E+00_JPRB, 0.83112E+00_JPRB, 0.79393E+00_JPRB, 0.73560E+00_JPRB, 0.69213E+00_JPRB ,&
&  0.66888E+00_JPRB, 0.65973E+00_JPRB, 0.65769E+00_JPRB, 0.65444E+00_JPRB, 0.65223E+00_JPRB, 0.65052E+00_JPRB, 0.64402E+00_JPRB ,&
&  0.63285E+00_JPRB, 0.62506E+00_JPRB, 0.61943E+00_JPRB, 0.60919E+00_JPRB, 0.59372E+00_JPRB, 0.57510E+00_JPRB, 0.55876E+00_JPRB ,&
&  0.54755E+00_JPRB, 0.54499E+00_JPRB, 0.55537E+00_JPRB, 0.56599E+00_JPRB, 0.57252E+00_JPRB, 0.57111E+00_JPRB, 0.56343E+00_JPRB ,&
&  0.54613E+00_JPRB, 0.53850E+00_JPRB, 0.54638E+00_JPRB, 0.55055E+00_JPRB, 0.55691E+00_JPRB, 0.55559E+00_JPRB, 0.53631E+00_JPRB ,&
&  0.51876E+00_JPRB, 0.50422E+00_JPRB, 0.50116E+00_JPRB, 0.51269E+00_JPRB, 0.56230E+00_JPRB, 0.63498E+00_JPRB, 0.66357E+00_JPRB ,&
&  0.69094E+00_JPRB, 0.71760E+00_JPRB, 0.74953E+00_JPRB, 0.79312E+00_JPRB, 0.85909E+00_JPRB, 0.95788E+00_JPRB, 0.11437E+01_JPRB ,&
&  0.13698E+01_JPRB, 0.15485E+01_JPRB, 0.17103E+01_JPRB, 0.15459E+01_JPRB, 0.14785E+01_JPRB, 0.15453E+01_JPRB, 0.16753E+01_JPRB ,&
&  0.18034E+01_JPRB, 0.18597E+01_JPRB, 0.18680E+01_JPRB, 0.19093E+01_JPRB, 0.18007E+01_JPRB, 0.16546E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.13496E+01_JPRB, 0.12635E+01_JPRB, 0.12317E+01_JPRB, 0.11816E+01_JPRB, 0.11807E+01_JPRB, 0.12320E+01_JPRB, 0.13275E+01_JPRB ,&
&  0.14857E+01_JPRB, 0.16763E+01_JPRB, 0.17653E+01_JPRB, 0.16904E+01_JPRB, 0.17259E+01_JPRB, 0.18165E+01_JPRB, 0.18942E+01_JPRB ,&
&  0.19007E+01_JPRB, 0.18958E+01_JPRB, 0.19099E+01_JPRB, 0.18859E+01_JPRB, 0.17138E+01_JPRB, 0.15562E+01_JPRB, 0.13771E+01_JPRB ,&
&  0.12092E+01_JPRB, 0.10609E+01_JPRB, 0.97546E+00_JPRB, 0.94483E+00_JPRB, 0.91892E+00_JPRB, 0.88973E+00_JPRB, 0.85110E+00_JPRB ,&
&  0.81822E+00_JPRB, 0.81051E+00_JPRB, 0.82857E+00_JPRB, 0.85490E+00_JPRB, 0.88773E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.88371E+00_JPRB, 0.81857E+00_JPRB, 0.77317E+00_JPRB, 0.75011E+00_JPRB, 0.73387E+00_JPRB, 0.70940E+00_JPRB, 0.69123E+00_JPRB ,&
&  0.67918E+00_JPRB, 0.67888E+00_JPRB, 0.66946E+00_JPRB, 0.65886E+00_JPRB, 0.65264E+00_JPRB, 0.64780E+00_JPRB, 0.64565E+00_JPRB ,&
&  0.64287E+00_JPRB, 0.63527E+00_JPRB, 0.61897E+00_JPRB, 0.59735E+00_JPRB, 0.58147E+00_JPRB, 0.56895E+00_JPRB, 0.55955E+00_JPRB ,&
&  0.55432E+00_JPRB, 0.55531E+00_JPRB, 0.56519E+00_JPRB, 0.56531E+00_JPRB, 0.55953E+00_JPRB, 0.54939E+00_JPRB, 0.53753E+00_JPRB ,&
&  0.53463E+00_JPRB, 0.54631E+00_JPRB, 0.53453E+00_JPRB, 0.54008E+00_JPRB, 0.55116E+00_JPRB, 0.53421E+00_JPRB, 0.51564E+00_JPRB ,&
&  0.49854E+00_JPRB, 0.48259E+00_JPRB, 0.47176E+00_JPRB, 0.46900E+00_JPRB, 0.48638E+00_JPRB, 0.51111E+00_JPRB, 0.55762E+00_JPRB ,&
&  0.62345E+00_JPRB, 0.65992E+00_JPRB, 0.68510E+00_JPRB, 0.71475E+00_JPRB, 0.74455E+00_JPRB, 0.78218E+00_JPRB, 0.84884E+00_JPRB ,&
&  0.95442E+00_JPRB, 0.10688E+01_JPRB, 0.11796E+01_JPRB, 0.12560E+01_JPRB, 0.13222E+01_JPRB, 0.12723E+01_JPRB, 0.12812E+01_JPRB ,&
&  0.14724E+01_JPRB, 0.15718E+01_JPRB, 0.14833E+01_JPRB, 0.13962E+01_JPRB, 0.12947E+01_JPRB, 0.12068E+01_JPRB, 0.11442E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.10895E+01_JPRB, 0.10666E+01_JPRB, 0.10136E+01_JPRB, 0.97085E+00_JPRB, 0.96634E+00_JPRB, 0.99572E+00_JPRB ,&
&  0.10709E+01_JPRB, 0.13162E+01_JPRB, 0.17803E+01_JPRB, 0.15585E+01_JPRB, 0.14764E+01_JPRB, 0.14751E+01_JPRB, 0.14887E+01_JPRB ,&
&  0.15351E+01_JPRB, 0.15725E+01_JPRB, 0.15963E+01_JPRB, 0.15664E+01_JPRB, 0.14952E+01_JPRB, 0.14124E+01_JPRB, 0.13519E+01_JPRB ,&
&  0.13023E+01_JPRB, 0.12518E+01_JPRB, 0.12136E+01_JPRB, 0.11950E+01_JPRB, 0.11626E+01_JPRB, 0.11239E+01_JPRB, 0.10885E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10333E+01_JPRB, 0.10167E+01_JPRB, 0.99055E+00_JPRB, 0.95283E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.77577E+00_JPRB, 0.74621E+00_JPRB, 0.72423E+00_JPRB, 0.71051E+00_JPRB, 0.69943E+00_JPRB, 0.69092E+00_JPRB, 0.68359E+00_JPRB ,&
&  0.67728E+00_JPRB, 0.67205E+00_JPRB, 0.66469E+00_JPRB, 0.65757E+00_JPRB, 0.65201E+00_JPRB, 0.64647E+00_JPRB, 0.64170E+00_JPRB ,&
&  0.63329E+00_JPRB, 0.61143E+00_JPRB, 0.60452E+00_JPRB, 0.59643E+00_JPRB, 0.58345E+00_JPRB, 0.57664E+00_JPRB, 0.57244E+00_JPRB ,&
&  0.56588E+00_JPRB, 0.56180E+00_JPRB, 0.56056E+00_JPRB, 0.55215E+00_JPRB, 0.54407E+00_JPRB, 0.54476E+00_JPRB, 0.54323E+00_JPRB ,&
&  0.52541E+00_JPRB, 0.52563E+00_JPRB, 0.55312E+00_JPRB, 0.55031E+00_JPRB, 0.51761E+00_JPRB, 0.51111E+00_JPRB, 0.50328E+00_JPRB ,&
&  0.49173E+00_JPRB, 0.46406E+00_JPRB, 0.45706E+00_JPRB, 0.45304E+00_JPRB, 0.46293E+00_JPRB, 0.47702E+00_JPRB, 0.50999E+00_JPRB ,&
&  0.54185E+00_JPRB, 0.60444E+00_JPRB, 0.63260E+00_JPRB, 0.65444E+00_JPRB, 0.67290E+00_JPRB, 0.69280E+00_JPRB, 0.71799E+00_JPRB ,&
&  0.75314E+00_JPRB, 0.80143E+00_JPRB, 0.86354E+00_JPRB, 0.92034E+00_JPRB, 0.97328E+00_JPRB, 0.10025E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10592E+01_JPRB, 0.10211E+01_JPRB, 0.98481E+00_JPRB, 0.94139E+00_JPRB, 0.89949E+00_JPRB, 0.85470E+00_JPRB ,&
&  0.84388E+00_JPRB, 0.86233E+00_JPRB, 0.85469E+00_JPRB, 0.84343E+00_JPRB, 0.83254E+00_JPRB, 0.83175E+00_JPRB, 0.83911E+00_JPRB ,&
&  0.83984E+00_JPRB, 0.89201E+00_JPRB, 0.10528E+01_JPRB, 0.10962E+01_JPRB, 0.11272E+01_JPRB, 0.11487E+01_JPRB, 0.11639E+01_JPRB ,&
&  0.11868E+01_JPRB, 0.12155E+01_JPRB, 0.12448E+01_JPRB, 0.12595E+01_JPRB, 0.12711E+01_JPRB, 0.12600E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.12413E+01_JPRB, 0.12152E+01_JPRB, 0.11920E+01_JPRB, 0.11646E+01_JPRB, 0.11350E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.10411E+01_JPRB, 0.98421E+00_JPRB, 0.92774E+00_JPRB, 0.87123E+00_JPRB, 0.81852E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.70487E+00_JPRB, 0.69013E+00_JPRB, 0.67892E+00_JPRB, 0.67003E+00_JPRB, 0.66430E+00_JPRB, 0.65923E+00_JPRB, 0.65520E+00_JPRB ,&
&  0.65213E+00_JPRB, 0.64696E+00_JPRB, 0.64134E+00_JPRB, 0.63455E+00_JPRB, 0.62906E+00_JPRB, 0.62401E+00_JPRB, 0.61861E+00_JPRB ,&
&  0.61136E+00_JPRB, 0.60378E+00_JPRB, 0.59596E+00_JPRB, 0.58959E+00_JPRB, 0.59226E+00_JPRB, 0.59209E+00_JPRB, 0.59029E+00_JPRB ,&
&  0.58190E+00_JPRB, 0.56776E+00_JPRB, 0.55214E+00_JPRB, 0.54950E+00_JPRB, 0.54566E+00_JPRB, 0.53295E+00_JPRB, 0.53233E+00_JPRB ,&
&  0.53970E+00_JPRB, 0.53983E+00_JPRB, 0.53658E+00_JPRB, 0.53256E+00_JPRB, 0.50709E+00_JPRB, 0.46377E+00_JPRB, 0.47926E+00_JPRB ,&
&  0.47440E+00_JPRB, 0.45711E+00_JPRB, 0.45272E+00_JPRB, 0.45061E+00_JPRB, 0.45750E+00_JPRB, 0.47228E+00_JPRB, 0.48724E+00_JPRB ,&
&  0.49482E+00_JPRB, 0.56380E+00_JPRB, 0.59218E+00_JPRB, 0.61144E+00_JPRB, 0.62449E+00_JPRB, 0.63624E+00_JPRB, 0.64778E+00_JPRB ,&
&  0.65921E+00_JPRB, 0.67679E+00_JPRB, 0.70135E+00_JPRB, 0.72528E+00_JPRB, 0.75119E+00_JPRB, 0.76945E+00_JPRB, 0.79026E+00_JPRB ,&
&  0.81174E+00_JPRB, 0.81481E+00_JPRB, 0.79670E+00_JPRB, 0.77769E+00_JPRB, 0.75910E+00_JPRB, 0.74083E+00_JPRB, 0.72498E+00_JPRB ,&
&  0.71906E+00_JPRB, 0.71573E+00_JPRB, 0.72527E+00_JPRB, 0.73147E+00_JPRB, 0.73720E+00_JPRB, 0.74610E+00_JPRB, 0.76253E+00_JPRB ,&
&  0.77754E+00_JPRB, 0.79831E+00_JPRB, 0.81874E+00_JPRB, 0.84615E+00_JPRB, 0.87363E+00_JPRB, 0.89416E+00_JPRB, 0.91265E+00_JPRB ,&
&  0.92505E+00_JPRB, 0.94716E+00_JPRB, 0.95903E+00_JPRB, 0.97305E+00_JPRB, 0.98975E+00_JPRB, 0.99786E+00_JPRB, 0.10054E+01_JPRB ,&
&  0.10087E+01_JPRB, 0.10055E+01_JPRB, 0.99404E+00_JPRB, 0.97450E+00_JPRB, 0.94788E+00_JPRB, 0.91680E+00_JPRB, 0.87922E+00_JPRB ,&
&  0.83842E+00_JPRB, 0.79489E+00_JPRB, 0.76187E+00_JPRB, 0.73912E+00_JPRB, 0.72019E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.67074E+00_JPRB, 0.65975E+00_JPRB, 0.65136E+00_JPRB, 0.64438E+00_JPRB, 0.63910E+00_JPRB, 0.63448E+00_JPRB, 0.63009E+00_JPRB ,&
&  0.62639E+00_JPRB, 0.62285E+00_JPRB, 0.61958E+00_JPRB, 0.61683E+00_JPRB, 0.61390E+00_JPRB, 0.61165E+00_JPRB, 0.60883E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.60588E+00_JPRB, 0.60731E+00_JPRB, 0.60534E+00_JPRB, 0.60740E+00_JPRB, 0.60499E+00_JPRB, 0.59567E+00_JPRB ,&
&  0.58508E+00_JPRB, 0.57082E+00_JPRB, 0.55137E+00_JPRB, 0.55509E+00_JPRB, 0.54741E+00_JPRB, 0.54485E+00_JPRB, 0.53218E+00_JPRB ,&
&  0.52596E+00_JPRB, 0.50675E+00_JPRB, 0.52135E+00_JPRB, 0.51070E+00_JPRB, 0.50330E+00_JPRB, 0.49455E+00_JPRB, 0.47966E+00_JPRB ,&
&  0.47599E+00_JPRB, 0.46656E+00_JPRB, 0.46571E+00_JPRB, 0.46887E+00_JPRB, 0.47458E+00_JPRB, 0.48418E+00_JPRB, 0.49374E+00_JPRB ,&
&  0.49691E+00_JPRB, 0.53964E+00_JPRB, 0.56465E+00_JPRB, 0.58218E+00_JPRB, 0.59920E+00_JPRB, 0.61293E+00_JPRB, 0.62369E+00_JPRB ,&
&  0.63403E+00_JPRB, 0.64278E+00_JPRB, 0.65353E+00_JPRB, 0.65743E+00_JPRB, 0.67154E+00_JPRB, 0.67668E+00_JPRB, 0.68405E+00_JPRB ,&
&  0.69127E+00_JPRB, 0.69160E+00_JPRB, 0.68898E+00_JPRB, 0.68257E+00_JPRB, 0.67792E+00_JPRB, 0.67307E+00_JPRB, 0.67177E+00_JPRB ,&
&  0.67063E+00_JPRB, 0.67121E+00_JPRB, 0.67820E+00_JPRB, 0.68510E+00_JPRB, 0.69136E+00_JPRB, 0.69826E+00_JPRB, 0.70534E+00_JPRB ,&
&  0.71388E+00_JPRB, 0.72640E+00_JPRB, 0.73993E+00_JPRB, 0.76039E+00_JPRB, 0.78008E+00_JPRB, 0.79286E+00_JPRB, 0.80479E+00_JPRB ,&
&  0.81848E+00_JPRB, 0.81693E+00_JPRB, 0.81605E+00_JPRB, 0.81771E+00_JPRB, 0.81921E+00_JPRB, 0.82164E+00_JPRB, 0.82412E+00_JPRB ,&
&  0.82365E+00_JPRB, 0.81662E+00_JPRB, 0.80699E+00_JPRB, 0.79489E+00_JPRB, 0.78266E+00_JPRB, 0.77118E+00_JPRB, 0.75987E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.73210E+00_JPRB, 0.71565E+00_JPRB, 0.69884E+00_JPRB, 0.68351E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.65473E+00_JPRB, 0.64539E+00_JPRB, 0.63736E+00_JPRB, 0.63221E+00_JPRB, 0.62688E+00_JPRB, 0.62283E+00_JPRB, 0.61918E+00_JPRB ,&
&  0.61628E+00_JPRB, 0.61370E+00_JPRB, 0.61222E+00_JPRB, 0.61063E+00_JPRB, 0.60922E+00_JPRB, 0.60807E+00_JPRB, 0.60686E+00_JPRB ,&
&  0.60819E+00_JPRB, 0.60741E+00_JPRB, 0.60945E+00_JPRB, 0.60813E+00_JPRB, 0.60711E+00_JPRB, 0.60412E+00_JPRB, 0.59525E+00_JPRB ,&
&  0.58882E+00_JPRB, 0.58160E+00_JPRB, 0.55661E+00_JPRB, 0.56272E+00_JPRB, 0.55189E+00_JPRB, 0.54590E+00_JPRB, 0.54050E+00_JPRB ,&
&  0.52799E+00_JPRB, 0.52185E+00_JPRB, 0.51440E+00_JPRB, 0.51259E+00_JPRB, 0.51407E+00_JPRB, 0.50828E+00_JPRB, 0.50921E+00_JPRB ,&
&  0.50367E+00_JPRB, 0.50271E+00_JPRB, 0.50388E+00_JPRB, 0.50534E+00_JPRB, 0.51156E+00_JPRB, 0.51412E+00_JPRB, 0.52021E+00_JPRB ,&
&  0.53121E+00_JPRB, 0.54791E+00_JPRB, 0.57537E+00_JPRB, 0.58547E+00_JPRB, 0.60212E+00_JPRB, 0.61553E+00_JPRB, 0.62806E+00_JPRB ,&
&  0.63630E+00_JPRB, 0.64186E+00_JPRB, 0.64654E+00_JPRB, 0.64914E+00_JPRB, 0.65021E+00_JPRB, 0.64736E+00_JPRB, 0.64817E+00_JPRB ,&
&  0.65041E+00_JPRB, 0.65302E+00_JPRB, 0.65575E+00_JPRB, 0.65609E+00_JPRB, 0.65756E+00_JPRB, 0.65793E+00_JPRB, 0.66075E+00_JPRB ,&
&  0.66268E+00_JPRB, 0.66435E+00_JPRB, 0.66444E+00_JPRB, 0.66605E+00_JPRB, 0.66827E+00_JPRB, 0.67116E+00_JPRB, 0.67436E+00_JPRB ,&
&  0.67847E+00_JPRB, 0.68451E+00_JPRB, 0.69222E+00_JPRB, 0.69121E+00_JPRB, 0.69680E+00_JPRB, 0.71470E+00_JPRB, 0.72279E+00_JPRB ,&
&  0.73203E+00_JPRB, 0.73734E+00_JPRB, 0.74023E+00_JPRB, 0.74319E+00_JPRB, 0.74510E+00_JPRB, 0.74878E+00_JPRB, 0.75111E+00_JPRB ,&
&  0.75283E+00_JPRB, 0.74659E+00_JPRB, 0.73915E+00_JPRB, 0.73261E+00_JPRB, 0.72559E+00_JPRB, 0.71970E+00_JPRB, 0.71294E+00_JPRB ,&
&  0.70681E+00_JPRB, 0.69994E+00_JPRB, 0.69261E+00_JPRB, 0.68071E+00_JPRB, 0.66786E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.64349E+00_JPRB, 0.63700E+00_JPRB, 0.63225E+00_JPRB, 0.62869E+00_JPRB, 0.62484E+00_JPRB, 0.62191E+00_JPRB, 0.61976E+00_JPRB ,&
&  0.61831E+00_JPRB, 0.61615E+00_JPRB, 0.61514E+00_JPRB, 0.61403E+00_JPRB, 0.61237E+00_JPRB, 0.61253E+00_JPRB, 0.61207E+00_JPRB ,&
&  0.61222E+00_JPRB, 0.61115E+00_JPRB, 0.61263E+00_JPRB, 0.61108E+00_JPRB, 0.61015E+00_JPRB, 0.60950E+00_JPRB, 0.60575E+00_JPRB ,&
&  0.60327E+00_JPRB, 0.59965E+00_JPRB, 0.59148E+00_JPRB, 0.58304E+00_JPRB, 0.57647E+00_JPRB, 0.55028E+00_JPRB, 0.54779E+00_JPRB ,&
&  0.55301E+00_JPRB, 0.54296E+00_JPRB, 0.53848E+00_JPRB, 0.53476E+00_JPRB, 0.53250E+00_JPRB, 0.53064E+00_JPRB, 0.53304E+00_JPRB ,&
&  0.52138E+00_JPRB, 0.53411E+00_JPRB, 0.53501E+00_JPRB, 0.53565E+00_JPRB, 0.53805E+00_JPRB, 0.53727E+00_JPRB, 0.54455E+00_JPRB ,&
&  0.56952E+00_JPRB, 0.57544E+00_JPRB, 0.58700E+00_JPRB, 0.59712E+00_JPRB, 0.60860E+00_JPRB, 0.61876E+00_JPRB, 0.62874E+00_JPRB ,&
&  0.63650E+00_JPRB, 0.64190E+00_JPRB, 0.64557E+00_JPRB, 0.64635E+00_JPRB, 0.64715E+00_JPRB, 0.64722E+00_JPRB, 0.64871E+00_JPRB ,&
&  0.64970E+00_JPRB, 0.65183E+00_JPRB, 0.65459E+00_JPRB, 0.65647E+00_JPRB, 0.65837E+00_JPRB, 0.65875E+00_JPRB, 0.65940E+00_JPRB ,&
&  0.65941E+00_JPRB, 0.65861E+00_JPRB, 0.65737E+00_JPRB, 0.65631E+00_JPRB, 0.65462E+00_JPRB, 0.65398E+00_JPRB, 0.65472E+00_JPRB ,&
&  0.65693E+00_JPRB, 0.65885E+00_JPRB, 0.66192E+00_JPRB, 0.66548E+00_JPRB, 0.67017E+00_JPRB, 0.67678E+00_JPRB, 0.68166E+00_JPRB ,&
&  0.68940E+00_JPRB, 0.69467E+00_JPRB, 0.69941E+00_JPRB, 0.70331E+00_JPRB, 0.70628E+00_JPRB, 0.71027E+00_JPRB, 0.71327E+00_JPRB ,&
&  0.71720E+00_JPRB, 0.71453E+00_JPRB, 0.71021E+00_JPRB, 0.70529E+00_JPRB, 0.70104E+00_JPRB, 0.69715E+00_JPRB, 0.69271E+00_JPRB ,&
&  0.68806E+00_JPRB, 0.68058E+00_JPRB, 0.67203E+00_JPRB, 0.66169E+00_JPRB, 0.65170E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.64450E+00_JPRB, 0.63997E+00_JPRB, 0.63655E+00_JPRB, 0.63384E+00_JPRB, 0.63185E+00_JPRB, 0.63003E+00_JPRB, 0.62855E+00_JPRB ,&
&  0.62715E+00_JPRB, 0.62709E+00_JPRB, 0.62659E+00_JPRB, 0.62593E+00_JPRB, 0.62493E+00_JPRB, 0.62504E+00_JPRB, 0.62473E+00_JPRB ,&
&  0.62361E+00_JPRB, 0.62078E+00_JPRB, 0.61813E+00_JPRB, 0.61567E+00_JPRB, 0.61501E+00_JPRB, 0.61427E+00_JPRB, 0.61424E+00_JPRB ,&
&  0.61266E+00_JPRB, 0.61076E+00_JPRB, 0.60809E+00_JPRB, 0.60193E+00_JPRB, 0.59542E+00_JPRB, 0.58924E+00_JPRB, 0.58375E+00_JPRB ,&
&  0.57910E+00_JPRB, 0.57573E+00_JPRB, 0.57371E+00_JPRB, 0.57260E+00_JPRB, 0.57449E+00_JPRB, 0.57451E+00_JPRB, 0.57404E+00_JPRB ,&
&  0.57270E+00_JPRB, 0.56931E+00_JPRB, 0.56978E+00_JPRB, 0.57067E+00_JPRB, 0.57219E+00_JPRB, 0.57646E+00_JPRB, 0.58065E+00_JPRB ,&
&  0.58585E+00_JPRB, 0.59178E+00_JPRB, 0.60101E+00_JPRB, 0.60789E+00_JPRB, 0.61476E+00_JPRB, 0.62042E+00_JPRB, 0.62369E+00_JPRB ,&
&  0.62649E+00_JPRB, 0.62895E+00_JPRB, 0.63146E+00_JPRB, 0.63354E+00_JPRB, 0.63522E+00_JPRB, 0.63712E+00_JPRB, 0.63935E+00_JPRB ,&
&  0.64074E+00_JPRB, 0.64180E+00_JPRB, 0.64270E+00_JPRB, 0.64355E+00_JPRB, 0.64438E+00_JPRB, 0.64552E+00_JPRB, 0.64701E+00_JPRB ,&
&  0.64837E+00_JPRB, 0.64868E+00_JPRB, 0.64868E+00_JPRB, 0.64957E+00_JPRB, 0.65057E+00_JPRB, 0.65161E+00_JPRB, 0.65237E+00_JPRB ,&
&  0.65373E+00_JPRB, 0.65512E+00_JPRB, 0.65704E+00_JPRB, 0.66006E+00_JPRB, 0.66243E+00_JPRB, 0.66459E+00_JPRB, 0.66559E+00_JPRB ,&
&  0.66691E+00_JPRB, 0.66694E+00_JPRB, 0.66695E+00_JPRB, 0.66802E+00_JPRB, 0.66922E+00_JPRB, 0.67167E+00_JPRB, 0.67447E+00_JPRB ,&
&  0.67793E+00_JPRB, 0.67883E+00_JPRB, 0.67890E+00_JPRB, 0.67770E+00_JPRB, 0.67641E+00_JPRB, 0.67453E+00_JPRB, 0.67261E+00_JPRB ,&
&  0.67029E+00_JPRB, 0.66661E+00_JPRB, 0.66130E+00_JPRB, 0.65544E+00_JPRB, 0.64974E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.64860E+00_JPRB, 0.64695E+00_JPRB, 0.64537E+00_JPRB, 0.64372E+00_JPRB, 0.64216E+00_JPRB, 0.64089E+00_JPRB, 0.63977E+00_JPRB ,&
&  0.63865E+00_JPRB, 0.63742E+00_JPRB, 0.63597E+00_JPRB, 0.63460E+00_JPRB, 0.63336E+00_JPRB, 0.63210E+00_JPRB, 0.63106E+00_JPRB ,&
&  0.63006E+00_JPRB, 0.62920E+00_JPRB, 0.62835E+00_JPRB, 0.62717E+00_JPRB, 0.62615E+00_JPRB, 0.62510E+00_JPRB, 0.62347E+00_JPRB ,&
&  0.62148E+00_JPRB, 0.61949E+00_JPRB, 0.61726E+00_JPRB, 0.61523E+00_JPRB, 0.61359E+00_JPRB, 0.61153E+00_JPRB, 0.60914E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.60530E+00_JPRB, 0.60397E+00_JPRB, 0.60291E+00_JPRB, 0.60227E+00_JPRB, 0.60171E+00_JPRB, 0.60111E+00_JPRB ,&
&  0.60070E+00_JPRB, 0.60068E+00_JPRB, 0.60105E+00_JPRB, 0.60172E+00_JPRB, 0.60263E+00_JPRB, 0.60406E+00_JPRB, 0.60647E+00_JPRB ,&
&  0.60906E+00_JPRB, 0.61166E+00_JPRB, 0.61441E+00_JPRB, 0.61702E+00_JPRB, 0.61946E+00_JPRB, 0.62141E+00_JPRB, 0.62292E+00_JPRB ,&
&  0.62463E+00_JPRB, 0.62637E+00_JPRB, 0.62822E+00_JPRB, 0.62959E+00_JPRB, 0.63077E+00_JPRB, 0.63179E+00_JPRB, 0.63287E+00_JPRB ,&
&  0.63348E+00_JPRB, 0.63412E+00_JPRB, 0.63492E+00_JPRB, 0.63591E+00_JPRB, 0.63676E+00_JPRB, 0.63751E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.63953E+00_JPRB, 0.64075E+00_JPRB, 0.64177E+00_JPRB, 0.64273E+00_JPRB, 0.64381E+00_JPRB, 0.64516E+00_JPRB, 0.64663E+00_JPRB ,&
&  0.64805E+00_JPRB, 0.64940E+00_JPRB, 0.65069E+00_JPRB, 0.65190E+00_JPRB, 0.65274E+00_JPRB, 0.65315E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.65219E+00_JPRB, 0.65140E+00_JPRB, 0.65108E+00_JPRB, 0.65128E+00_JPRB, 0.65141E+00_JPRB, 0.65178E+00_JPRB, 0.65247E+00_JPRB ,&
&  0.65347E+00_JPRB, 0.65442E+00_JPRB, 0.65531E+00_JPRB, 0.65624E+00_JPRB, 0.65680E+00_JPRB, 0.65680E+00_JPRB, 0.65665E+00_JPRB ,&
&  0.65629E+00_JPRB, 0.65537E+00_JPRB, 0.65390E+00_JPRB, 0.65220E+00_JPRB, 0.65034E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB, 0.63747E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB, 0.13571E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.14130E+00_JPRB, 0.14148E+00_JPRB, 0.14156E+00_JPRB, 0.14160E+00_JPRB, 0.14148E+00_JPRB, 0.14148E+00_JPRB, 0.14147E+00_JPRB ,&
&  0.14147E+00_JPRB, 0.14134E+00_JPRB, 0.14152E+00_JPRB, 0.14162E+00_JPRB, 0.14167E+00_JPRB, 0.14168E+00_JPRB, 0.14157E+00_JPRB ,&
&  0.14132E+00_JPRB, 0.14101E+00_JPRB, 0.14020E+00_JPRB, 0.14006E+00_JPRB, 0.14010E+00_JPRB, 0.14030E+00_JPRB, 0.14044E+00_JPRB ,&
&  0.14066E+00_JPRB, 0.14088E+00_JPRB, 0.14105E+00_JPRB, 0.14155E+00_JPRB, 0.14177E+00_JPRB, 0.14203E+00_JPRB, 0.14228E+00_JPRB ,&
&  0.14253E+00_JPRB, 0.14281E+00_JPRB, 0.14301E+00_JPRB, 0.14323E+00_JPRB, 0.14342E+00_JPRB, 0.14336E+00_JPRB, 0.14317E+00_JPRB ,&
&  0.14288E+00_JPRB, 0.14242E+00_JPRB, 0.14195E+00_JPRB, 0.14151E+00_JPRB, 0.14113E+00_JPRB, 0.14072E+00_JPRB, 0.14034E+00_JPRB ,&
&  0.13992E+00_JPRB, 0.13944E+00_JPRB, 0.13894E+00_JPRB, 0.13848E+00_JPRB, 0.13801E+00_JPRB, 0.13761E+00_JPRB, 0.13715E+00_JPRB ,&
&  0.13679E+00_JPRB, 0.13643E+00_JPRB, 0.13609E+00_JPRB, 0.13573E+00_JPRB, 0.13542E+00_JPRB, 0.13511E+00_JPRB, 0.13480E+00_JPRB ,&
&  0.13448E+00_JPRB, 0.13430E+00_JPRB, 0.13407E+00_JPRB, 0.13386E+00_JPRB, 0.13367E+00_JPRB, 0.13352E+00_JPRB, 0.13333E+00_JPRB ,&
&  0.13316E+00_JPRB, 0.13299E+00_JPRB, 0.13291E+00_JPRB, 0.13278E+00_JPRB, 0.13273E+00_JPRB, 0.13270E+00_JPRB, 0.13276E+00_JPRB ,&
&  0.13281E+00_JPRB, 0.13290E+00_JPRB, 0.13305E+00_JPRB, 0.13329E+00_JPRB, 0.13351E+00_JPRB, 0.13375E+00_JPRB, 0.13400E+00_JPRB ,&
&  0.13428E+00_JPRB, 0.13455E+00_JPRB, 0.13485E+00_JPRB, 0.13513E+00_JPRB, 0.13547E+00_JPRB, 0.13582E+00_JPRB, 0.13621E+00_JPRB ,&
&  0.13657E+00_JPRB, 0.13696E+00_JPRB, 0.13728E+00_JPRB, 0.13766E+00_JPRB, 0.13796E+00_JPRB, 0.13816E+00_JPRB, 0.13833E+00_JPRB ,&
&  0.13865E+00_JPRB, 0.13897E+00_JPRB, 0.13936E+00_JPRB, 0.13971E+00_JPRB, 0.14033E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.15493E+00_JPRB, 0.15727E+00_JPRB, 0.15896E+00_JPRB, 0.16084E+00_JPRB, 0.15979E+00_JPRB, 0.17125E+00_JPRB, 0.16935E+00_JPRB ,&
&  0.16905E+00_JPRB, 0.16380E+00_JPRB, 0.16396E+00_JPRB, 0.16307E+00_JPRB, 0.16161E+00_JPRB, 0.15998E+00_JPRB, 0.15821E+00_JPRB ,&
&  0.15649E+00_JPRB, 0.15435E+00_JPRB, 0.15254E+00_JPRB, 0.15102E+00_JPRB, 0.14965E+00_JPRB, 0.14844E+00_JPRB, 0.14722E+00_JPRB ,&
&  0.14701E+00_JPRB, 0.14759E+00_JPRB, 0.14854E+00_JPRB, 0.14798E+00_JPRB, 0.14839E+00_JPRB, 0.14922E+00_JPRB, 0.15034E+00_JPRB ,&
&  0.15130E+00_JPRB, 0.15260E+00_JPRB, 0.15342E+00_JPRB, 0.15364E+00_JPRB, 0.15383E+00_JPRB, 0.15314E+00_JPRB, 0.15267E+00_JPRB ,&
&  0.15203E+00_JPRB, 0.15115E+00_JPRB, 0.15004E+00_JPRB, 0.14909E+00_JPRB, 0.14802E+00_JPRB, 0.14663E+00_JPRB, 0.14518E+00_JPRB ,&
&  0.14387E+00_JPRB, 0.14272E+00_JPRB, 0.14153E+00_JPRB, 0.14053E+00_JPRB, 0.13958E+00_JPRB, 0.13863E+00_JPRB, 0.13776E+00_JPRB ,&
&  0.13708E+00_JPRB, 0.13647E+00_JPRB, 0.13581E+00_JPRB, 0.13510E+00_JPRB, 0.13431E+00_JPRB, 0.13355E+00_JPRB, 0.13282E+00_JPRB ,&
&  0.13244E+00_JPRB, 0.13211E+00_JPRB, 0.13177E+00_JPRB, 0.13141E+00_JPRB, 0.13126E+00_JPRB, 0.13112E+00_JPRB, 0.13094E+00_JPRB ,&
&  0.13076E+00_JPRB, 0.13051E+00_JPRB, 0.13041E+00_JPRB, 0.13031E+00_JPRB, 0.13023E+00_JPRB, 0.13016E+00_JPRB, 0.13025E+00_JPRB ,&
&  0.13041E+00_JPRB, 0.13063E+00_JPRB, 0.13108E+00_JPRB, 0.13156E+00_JPRB, 0.13207E+00_JPRB, 0.13257E+00_JPRB, 0.13329E+00_JPRB ,&
&  0.13407E+00_JPRB, 0.13482E+00_JPRB, 0.13548E+00_JPRB, 0.13602E+00_JPRB, 0.13686E+00_JPRB, 0.13765E+00_JPRB, 0.13842E+00_JPRB ,&
&  0.13927E+00_JPRB, 0.14036E+00_JPRB, 0.14148E+00_JPRB, 0.14264E+00_JPRB, 0.14397E+00_JPRB, 0.14514E+00_JPRB, 0.14628E+00_JPRB ,&
&  0.14764E+00_JPRB, 0.15010E+00_JPRB, 0.15184E+00_JPRB, 0.15301E+00_JPRB, 0.15432E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.17590E+00_JPRB, 0.17711E+00_JPRB, 0.17814E+00_JPRB, 0.17866E+00_JPRB, 0.17917E+00_JPRB, 0.17919E+00_JPRB, 0.17913E+00_JPRB ,&
&  0.17645E+00_JPRB, 0.17528E+00_JPRB, 0.17360E+00_JPRB, 0.17170E+00_JPRB, 0.17005E+00_JPRB, 0.16806E+00_JPRB, 0.16605E+00_JPRB ,&
&  0.16414E+00_JPRB, 0.16188E+00_JPRB, 0.15945E+00_JPRB, 0.15660E+00_JPRB, 0.15383E+00_JPRB, 0.15122E+00_JPRB, 0.14970E+00_JPRB ,&
&  0.14892E+00_JPRB, 0.14914E+00_JPRB, 0.15055E+00_JPRB, 0.15172E+00_JPRB, 0.15341E+00_JPRB, 0.15558E+00_JPRB, 0.15907E+00_JPRB ,&
&  0.16176E+00_JPRB, 0.16508E+00_JPRB, 0.16731E+00_JPRB, 0.16805E+00_JPRB, 0.16737E+00_JPRB, 0.16702E+00_JPRB, 0.16617E+00_JPRB ,&
&  0.16447E+00_JPRB, 0.16269E+00_JPRB, 0.16113E+00_JPRB, 0.15947E+00_JPRB, 0.15760E+00_JPRB, 0.15528E+00_JPRB, 0.15244E+00_JPRB ,&
&  0.14986E+00_JPRB, 0.14760E+00_JPRB, 0.14550E+00_JPRB, 0.14389E+00_JPRB, 0.14234E+00_JPRB, 0.14093E+00_JPRB, 0.13946E+00_JPRB ,&
&  0.13832E+00_JPRB, 0.13733E+00_JPRB, 0.13607E+00_JPRB, 0.13502E+00_JPRB, 0.13383E+00_JPRB, 0.13284E+00_JPRB, 0.13177E+00_JPRB ,&
&  0.13092E+00_JPRB, 0.13017E+00_JPRB, 0.12955E+00_JPRB, 0.12896E+00_JPRB, 0.12857E+00_JPRB, 0.12820E+00_JPRB, 0.12777E+00_JPRB ,&
&  0.12739E+00_JPRB, 0.12730E+00_JPRB, 0.12727E+00_JPRB, 0.12725E+00_JPRB, 0.12729E+00_JPRB, 0.12699E+00_JPRB, 0.12704E+00_JPRB ,&
&  0.12734E+00_JPRB, 0.12766E+00_JPRB, 0.12780E+00_JPRB, 0.12824E+00_JPRB, 0.12891E+00_JPRB, 0.12971E+00_JPRB, 0.13087E+00_JPRB ,&
&  0.13177E+00_JPRB, 0.13268E+00_JPRB, 0.13354E+00_JPRB, 0.13441E+00_JPRB, 0.13553E+00_JPRB, 0.13674E+00_JPRB, 0.13801E+00_JPRB ,&
&  0.13952E+00_JPRB, 0.14093E+00_JPRB, 0.14241E+00_JPRB, 0.14395E+00_JPRB, 0.14636E+00_JPRB, 0.14879E+00_JPRB, 0.15250E+00_JPRB ,&
&  0.15627E+00_JPRB, 0.15835E+00_JPRB, 0.17105E+00_JPRB, 0.17209E+00_JPRB, 0.17358E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.17640E+00_JPRB, 0.17783E+00_JPRB, 0.17867E+00_JPRB, 0.17978E+00_JPRB, 0.18033E+00_JPRB, 0.18187E+00_JPRB, 0.18215E+00_JPRB ,&
&  0.18227E+00_JPRB, 0.17838E+00_JPRB, 0.17602E+00_JPRB, 0.17341E+00_JPRB, 0.17114E+00_JPRB, 0.16857E+00_JPRB, 0.16605E+00_JPRB ,&
&  0.16408E+00_JPRB, 0.16236E+00_JPRB, 0.16054E+00_JPRB, 0.15800E+00_JPRB, 0.15544E+00_JPRB, 0.15352E+00_JPRB, 0.15209E+00_JPRB ,&
&  0.15067E+00_JPRB, 0.15121E+00_JPRB, 0.15257E+00_JPRB, 0.15547E+00_JPRB, 0.15952E+00_JPRB, 0.16303E+00_JPRB, 0.16764E+00_JPRB ,&
&  0.17374E+00_JPRB, 0.17559E+00_JPRB, 0.17758E+00_JPRB, 0.17854E+00_JPRB, 0.17825E+00_JPRB, 0.17871E+00_JPRB, 0.17630E+00_JPRB ,&
&  0.17652E+00_JPRB, 0.17450E+00_JPRB, 0.17897E+00_JPRB, 0.17644E+00_JPRB, 0.17016E+00_JPRB, 0.16710E+00_JPRB, 0.16119E+00_JPRB ,&
&  0.15665E+00_JPRB, 0.15191E+00_JPRB, 0.14867E+00_JPRB, 0.14683E+00_JPRB, 0.14409E+00_JPRB, 0.14257E+00_JPRB, 0.14030E+00_JPRB ,&
&  0.13822E+00_JPRB, 0.13747E+00_JPRB, 0.13612E+00_JPRB, 0.13553E+00_JPRB, 0.13478E+00_JPRB, 0.13377E+00_JPRB, 0.13268E+00_JPRB ,&
&  0.13193E+00_JPRB, 0.13126E+00_JPRB, 0.13090E+00_JPRB, 0.13040E+00_JPRB, 0.12971E+00_JPRB, 0.12902E+00_JPRB, 0.12825E+00_JPRB ,&
&  0.12773E+00_JPRB, 0.12741E+00_JPRB, 0.12715E+00_JPRB, 0.12702E+00_JPRB, 0.12689E+00_JPRB, 0.12662E+00_JPRB, 0.12668E+00_JPRB ,&
&  0.12690E+00_JPRB, 0.12745E+00_JPRB, 0.12794E+00_JPRB, 0.12860E+00_JPRB, 0.12922E+00_JPRB, 0.13004E+00_JPRB, 0.13068E+00_JPRB ,&
&  0.13136E+00_JPRB, 0.13226E+00_JPRB, 0.13304E+00_JPRB, 0.13396E+00_JPRB, 0.13497E+00_JPRB, 0.13584E+00_JPRB, 0.13719E+00_JPRB ,&
&  0.13856E+00_JPRB, 0.13992E+00_JPRB, 0.14178E+00_JPRB, 0.14373E+00_JPRB, 0.14622E+00_JPRB, 0.15050E+00_JPRB, 0.15545E+00_JPRB ,&
&  0.16164E+00_JPRB, 0.17098E+00_JPRB, 0.17135E+00_JPRB, 0.17376E+00_JPRB, 0.17521E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.18143E+00_JPRB, 0.18334E+00_JPRB, 0.18492E+00_JPRB, 0.18637E+00_JPRB, 0.18664E+00_JPRB, 0.18695E+00_JPRB, 0.18769E+00_JPRB ,&
&  0.18399E+00_JPRB, 0.18127E+00_JPRB, 0.17677E+00_JPRB, 0.17245E+00_JPRB, 0.16836E+00_JPRB, 0.16485E+00_JPRB, 0.16275E+00_JPRB ,&
&  0.16169E+00_JPRB, 0.16174E+00_JPRB, 0.16209E+00_JPRB, 0.16282E+00_JPRB, 0.16231E+00_JPRB, 0.16146E+00_JPRB, 0.16058E+00_JPRB ,&
&  0.15942E+00_JPRB, 0.15953E+00_JPRB, 0.15991E+00_JPRB, 0.16309E+00_JPRB, 0.16759E+00_JPRB, 0.17271E+00_JPRB, 0.17637E+00_JPRB ,&
&  0.17899E+00_JPRB, 0.18096E+00_JPRB, 0.18332E+00_JPRB, 0.18432E+00_JPRB, 0.18519E+00_JPRB, 0.18641E+00_JPRB, 0.18699E+00_JPRB ,&
&  0.18831E+00_JPRB, 0.18853E+00_JPRB, 0.18854E+00_JPRB, 0.18899E+00_JPRB, 0.18097E+00_JPRB, 0.17489E+00_JPRB, 0.16874E+00_JPRB ,&
&  0.16276E+00_JPRB, 0.15675E+00_JPRB, 0.15135E+00_JPRB, 0.14830E+00_JPRB, 0.14527E+00_JPRB, 0.14352E+00_JPRB, 0.14198E+00_JPRB ,&
&  0.14084E+00_JPRB, 0.13965E+00_JPRB, 0.13798E+00_JPRB, 0.13634E+00_JPRB, 0.13539E+00_JPRB, 0.13458E+00_JPRB, 0.13357E+00_JPRB ,&
&  0.13347E+00_JPRB, 0.13287E+00_JPRB, 0.13277E+00_JPRB, 0.13238E+00_JPRB, 0.13210E+00_JPRB, 0.13186E+00_JPRB, 0.13187E+00_JPRB ,&
&  0.13223E+00_JPRB, 0.13291E+00_JPRB, 0.13314E+00_JPRB, 0.13342E+00_JPRB, 0.13325E+00_JPRB, 0.13264E+00_JPRB, 0.13211E+00_JPRB ,&
&  0.13156E+00_JPRB, 0.13111E+00_JPRB, 0.13109E+00_JPRB, 0.13100E+00_JPRB, 0.13148E+00_JPRB, 0.13213E+00_JPRB, 0.13279E+00_JPRB ,&
&  0.13368E+00_JPRB, 0.13438E+00_JPRB, 0.13507E+00_JPRB, 0.13592E+00_JPRB, 0.13666E+00_JPRB, 0.13762E+00_JPRB, 0.13873E+00_JPRB ,&
&  0.13961E+00_JPRB, 0.14125E+00_JPRB, 0.14262E+00_JPRB, 0.14428E+00_JPRB, 0.14736E+00_JPRB, 0.15084E+00_JPRB, 0.15883E+00_JPRB ,&
&  0.16678E+00_JPRB, 0.18369E+00_JPRB, 0.18522E+00_JPRB, 0.17986E+00_JPRB, 0.18031E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.18973E+00_JPRB, 0.19204E+00_JPRB, 0.19440E+00_JPRB, 0.19321E+00_JPRB, 0.19330E+00_JPRB, 0.19359E+00_JPRB, 0.19479E+00_JPRB ,&
&  0.19536E+00_JPRB, 0.19501E+00_JPRB, 0.19467E+00_JPRB, 0.19335E+00_JPRB, 0.19186E+00_JPRB, 0.18493E+00_JPRB, 0.18189E+00_JPRB ,&
&  0.17955E+00_JPRB, 0.17997E+00_JPRB, 0.18047E+00_JPRB, 0.18245E+00_JPRB, 0.18274E+00_JPRB, 0.18198E+00_JPRB, 0.17910E+00_JPRB ,&
&  0.17668E+00_JPRB, 0.17506E+00_JPRB, 0.17344E+00_JPRB, 0.17353E+00_JPRB, 0.17543E+00_JPRB, 0.17756E+00_JPRB, 0.17880E+00_JPRB ,&
&  0.17917E+00_JPRB, 0.18099E+00_JPRB, 0.18255E+00_JPRB, 0.18518E+00_JPRB, 0.19222E+00_JPRB, 0.19363E+00_JPRB, 0.19435E+00_JPRB ,&
&  0.19521E+00_JPRB, 0.19652E+00_JPRB, 0.19737E+00_JPRB, 0.19732E+00_JPRB, 0.19579E+00_JPRB, 0.19132E+00_JPRB, 0.19015E+00_JPRB ,&
&  0.17757E+00_JPRB, 0.17014E+00_JPRB, 0.16302E+00_JPRB, 0.15774E+00_JPRB, 0.15282E+00_JPRB, 0.15014E+00_JPRB, 0.14714E+00_JPRB ,&
&  0.14461E+00_JPRB, 0.14187E+00_JPRB, 0.13981E+00_JPRB, 0.13885E+00_JPRB, 0.13858E+00_JPRB, 0.13832E+00_JPRB, 0.13776E+00_JPRB ,&
&  0.13784E+00_JPRB, 0.13742E+00_JPRB, 0.13710E+00_JPRB, 0.13678E+00_JPRB, 0.13657E+00_JPRB, 0.13665E+00_JPRB, 0.13735E+00_JPRB ,&
&  0.13917E+00_JPRB, 0.14092E+00_JPRB, 0.14267E+00_JPRB, 0.14417E+00_JPRB, 0.14367E+00_JPRB, 0.14208E+00_JPRB, 0.13965E+00_JPRB ,&
&  0.13756E+00_JPRB, 0.13623E+00_JPRB, 0.13479E+00_JPRB, 0.13443E+00_JPRB, 0.13499E+00_JPRB, 0.13570E+00_JPRB, 0.13691E+00_JPRB ,&
&  0.13799E+00_JPRB, 0.13886E+00_JPRB, 0.13944E+00_JPRB, 0.13985E+00_JPRB, 0.14051E+00_JPRB, 0.14115E+00_JPRB, 0.14221E+00_JPRB ,&
&  0.14340E+00_JPRB, 0.14486E+00_JPRB, 0.14621E+00_JPRB, 0.14802E+00_JPRB, 0.15068E+00_JPRB, 0.15476E+00_JPRB, 0.16176E+00_JPRB ,&
&  0.16887E+00_JPRB, 0.18764E+00_JPRB, 0.18983E+00_JPRB, 0.18816E+00_JPRB, 0.18938E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.19727E+00_JPRB, 0.20151E+00_JPRB, 0.20394E+00_JPRB, 0.20349E+00_JPRB, 0.20315E+00_JPRB, 0.20380E+00_JPRB, 0.20462E+00_JPRB ,&
&  0.20569E+00_JPRB, 0.20525E+00_JPRB, 0.20386E+00_JPRB, 0.20247E+00_JPRB, 0.20096E+00_JPRB, 0.19898E+00_JPRB, 0.19761E+00_JPRB ,&
&  0.19654E+00_JPRB, 0.19649E+00_JPRB, 0.19606E+00_JPRB, 0.19685E+00_JPRB, 0.19739E+00_JPRB, 0.19985E+00_JPRB, 0.19857E+00_JPRB ,&
&  0.19522E+00_JPRB, 0.19344E+00_JPRB, 0.19566E+00_JPRB, 0.19606E+00_JPRB, 0.19661E+00_JPRB, 0.19741E+00_JPRB, 0.19873E+00_JPRB ,&
&  0.19638E+00_JPRB, 0.19325E+00_JPRB, 0.18872E+00_JPRB, 0.19360E+00_JPRB, 0.20268E+00_JPRB, 0.20277E+00_JPRB, 0.20200E+00_JPRB ,&
&  0.20335E+00_JPRB, 0.20602E+00_JPRB, 0.20733E+00_JPRB, 0.20772E+00_JPRB, 0.20630E+00_JPRB, 0.20561E+00_JPRB, 0.20453E+00_JPRB ,&
&  0.20184E+00_JPRB, 0.19911E+00_JPRB, 0.19095E+00_JPRB, 0.17544E+00_JPRB, 0.16885E+00_JPRB, 0.16566E+00_JPRB, 0.16172E+00_JPRB ,&
&  0.15676E+00_JPRB, 0.15430E+00_JPRB, 0.15399E+00_JPRB, 0.15259E+00_JPRB, 0.15284E+00_JPRB, 0.15238E+00_JPRB, 0.15148E+00_JPRB ,&
&  0.15029E+00_JPRB, 0.14944E+00_JPRB, 0.14796E+00_JPRB, 0.14562E+00_JPRB, 0.14387E+00_JPRB, 0.14344E+00_JPRB, 0.14413E+00_JPRB ,&
&  0.14595E+00_JPRB, 0.14886E+00_JPRB, 0.15699E+00_JPRB, 0.16175E+00_JPRB, 0.15979E+00_JPRB, 0.15567E+00_JPRB, 0.14972E+00_JPRB ,&
&  0.14461E+00_JPRB, 0.14301E+00_JPRB, 0.14189E+00_JPRB, 0.14096E+00_JPRB, 0.14127E+00_JPRB, 0.14229E+00_JPRB, 0.14331E+00_JPRB ,&
&  0.14446E+00_JPRB, 0.14575E+00_JPRB, 0.14653E+00_JPRB, 0.14696E+00_JPRB, 0.14738E+00_JPRB, 0.14762E+00_JPRB, 0.14810E+00_JPRB ,&
&  0.14827E+00_JPRB, 0.14958E+00_JPRB, 0.15077E+00_JPRB, 0.15305E+00_JPRB, 0.15461E+00_JPRB, 0.15655E+00_JPRB, 0.16050E+00_JPRB ,&
&  0.16399E+00_JPRB, 0.17422E+00_JPRB, 0.18871E+00_JPRB, 0.19677E+00_JPRB, 0.19407E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.20799E+00_JPRB, 0.20914E+00_JPRB, 0.20929E+00_JPRB, 0.20843E+00_JPRB, 0.20723E+00_JPRB, 0.20749E+00_JPRB, 0.20893E+00_JPRB ,&
&  0.21181E+00_JPRB, 0.21395E+00_JPRB, 0.21450E+00_JPRB, 0.21456E+00_JPRB, 0.21363E+00_JPRB, 0.21154E+00_JPRB, 0.20976E+00_JPRB ,&
&  0.20836E+00_JPRB, 0.20749E+00_JPRB, 0.20680E+00_JPRB, 0.20655E+00_JPRB, 0.20654E+00_JPRB, 0.20766E+00_JPRB, 0.20881E+00_JPRB ,&
&  0.21262E+00_JPRB, 0.21278E+00_JPRB, 0.21214E+00_JPRB, 0.21192E+00_JPRB, 0.21168E+00_JPRB, 0.21164E+00_JPRB, 0.21157E+00_JPRB ,&
&  0.20919E+00_JPRB, 0.20141E+00_JPRB, 0.19433E+00_JPRB, 0.19951E+00_JPRB, 0.21195E+00_JPRB, 0.21235E+00_JPRB, 0.21235E+00_JPRB ,&
&  0.21368E+00_JPRB, 0.21657E+00_JPRB, 0.21855E+00_JPRB, 0.21957E+00_JPRB, 0.21842E+00_JPRB, 0.21667E+00_JPRB, 0.21618E+00_JPRB ,&
&  0.21635E+00_JPRB, 0.21451E+00_JPRB, 0.20946E+00_JPRB, 0.20378E+00_JPRB, 0.19807E+00_JPRB, 0.19579E+00_JPRB, 0.19409E+00_JPRB ,&
&  0.19168E+00_JPRB, 0.19031E+00_JPRB, 0.18978E+00_JPRB, 0.18929E+00_JPRB, 0.18969E+00_JPRB, 0.18779E+00_JPRB, 0.18525E+00_JPRB ,&
&  0.18147E+00_JPRB, 0.17752E+00_JPRB, 0.17261E+00_JPRB, 0.16740E+00_JPRB, 0.16284E+00_JPRB, 0.15808E+00_JPRB, 0.15480E+00_JPRB ,&
&  0.15411E+00_JPRB, 0.15771E+00_JPRB, 0.16694E+00_JPRB, 0.17711E+00_JPRB, 0.18545E+00_JPRB, 0.17400E+00_JPRB, 0.16690E+00_JPRB ,&
&  0.15970E+00_JPRB, 0.15412E+00_JPRB, 0.15098E+00_JPRB, 0.15024E+00_JPRB, 0.15042E+00_JPRB, 0.15126E+00_JPRB, 0.15276E+00_JPRB ,&
&  0.15467E+00_JPRB, 0.15709E+00_JPRB, 0.15897E+00_JPRB, 0.16024E+00_JPRB, 0.16112E+00_JPRB, 0.16137E+00_JPRB, 0.16118E+00_JPRB ,&
&  0.16094E+00_JPRB, 0.16212E+00_JPRB, 0.16452E+00_JPRB, 0.16866E+00_JPRB, 0.17172E+00_JPRB, 0.17559E+00_JPRB, 0.18100E+00_JPRB ,&
&  0.18874E+00_JPRB, 0.19794E+00_JPRB, 0.20091E+00_JPRB, 0.20287E+00_JPRB, 0.20568E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.21814E+00_JPRB, 0.21940E+00_JPRB, 0.21954E+00_JPRB, 0.21987E+00_JPRB, 0.22041E+00_JPRB, 0.22223E+00_JPRB, 0.22634E+00_JPRB ,&
&  0.23035E+00_JPRB, 0.23219E+00_JPRB, 0.23072E+00_JPRB, 0.22851E+00_JPRB, 0.22734E+00_JPRB, 0.22667E+00_JPRB, 0.22640E+00_JPRB ,&
&  0.22618E+00_JPRB, 0.22516E+00_JPRB, 0.22233E+00_JPRB, 0.21847E+00_JPRB, 0.21571E+00_JPRB, 0.21514E+00_JPRB, 0.21570E+00_JPRB ,&
&  0.21957E+00_JPRB, 0.22228E+00_JPRB, 0.22482E+00_JPRB, 0.22693E+00_JPRB, 0.22737E+00_JPRB, 0.22698E+00_JPRB, 0.22835E+00_JPRB ,&
&  0.22944E+00_JPRB, 0.22732E+00_JPRB, 0.22207E+00_JPRB, 0.22633E+00_JPRB, 0.22835E+00_JPRB, 0.22959E+00_JPRB, 0.22888E+00_JPRB ,&
&  0.22957E+00_JPRB, 0.23170E+00_JPRB, 0.23367E+00_JPRB, 0.23523E+00_JPRB, 0.23454E+00_JPRB, 0.23143E+00_JPRB, 0.23042E+00_JPRB ,&
&  0.23058E+00_JPRB, 0.22945E+00_JPRB, 0.22679E+00_JPRB, 0.22439E+00_JPRB, 0.22175E+00_JPRB, 0.22061E+00_JPRB, 0.22040E+00_JPRB ,&
&  0.21947E+00_JPRB, 0.21728E+00_JPRB, 0.21294E+00_JPRB, 0.20872E+00_JPRB, 0.20568E+00_JPRB, 0.20276E+00_JPRB, 0.20168E+00_JPRB ,&
&  0.20220E+00_JPRB, 0.20344E+00_JPRB, 0.20353E+00_JPRB, 0.20224E+00_JPRB, 0.19786E+00_JPRB, 0.18745E+00_JPRB, 0.18033E+00_JPRB ,&
&  0.17907E+00_JPRB, 0.18372E+00_JPRB, 0.19198E+00_JPRB, 0.19303E+00_JPRB, 0.19502E+00_JPRB, 0.19648E+00_JPRB, 0.19789E+00_JPRB ,&
&  0.19684E+00_JPRB, 0.18357E+00_JPRB, 0.17769E+00_JPRB, 0.17443E+00_JPRB, 0.17128E+00_JPRB, 0.17072E+00_JPRB, 0.17218E+00_JPRB ,&
&  0.17516E+00_JPRB, 0.17922E+00_JPRB, 0.18193E+00_JPRB, 0.18342E+00_JPRB, 0.18378E+00_JPRB, 0.18331E+00_JPRB, 0.18256E+00_JPRB ,&
&  0.18264E+00_JPRB, 0.18636E+00_JPRB, 0.19404E+00_JPRB, 0.20265E+00_JPRB, 0.20364E+00_JPRB, 0.20529E+00_JPRB, 0.20591E+00_JPRB ,&
&  0.20645E+00_JPRB, 0.20827E+00_JPRB, 0.20941E+00_JPRB, 0.21214E+00_JPRB, 0.21547E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.22945E+00_JPRB, 0.23250E+00_JPRB, 0.23550E+00_JPRB, 0.23725E+00_JPRB, 0.23906E+00_JPRB, 0.24091E+00_JPRB, 0.24419E+00_JPRB ,&
&  0.24803E+00_JPRB, 0.24920E+00_JPRB, 0.24748E+00_JPRB, 0.24471E+00_JPRB, 0.24290E+00_JPRB, 0.24208E+00_JPRB, 0.24179E+00_JPRB ,&
&  0.24158E+00_JPRB, 0.24143E+00_JPRB, 0.24069E+00_JPRB, 0.23896E+00_JPRB, 0.23652E+00_JPRB, 0.23411E+00_JPRB, 0.23235E+00_JPRB ,&
&  0.23222E+00_JPRB, 0.23378E+00_JPRB, 0.23621E+00_JPRB, 0.23661E+00_JPRB, 0.23685E+00_JPRB, 0.23812E+00_JPRB, 0.24011E+00_JPRB ,&
&  0.24400E+00_JPRB, 0.24602E+00_JPRB, 0.24821E+00_JPRB, 0.24414E+00_JPRB, 0.24878E+00_JPRB, 0.25042E+00_JPRB, 0.25097E+00_JPRB ,&
&  0.25227E+00_JPRB, 0.25329E+00_JPRB, 0.25424E+00_JPRB, 0.25353E+00_JPRB, 0.24920E+00_JPRB, 0.24724E+00_JPRB, 0.24777E+00_JPRB ,&
&  0.24830E+00_JPRB, 0.24675E+00_JPRB, 0.24390E+00_JPRB, 0.24124E+00_JPRB, 0.23909E+00_JPRB, 0.23761E+00_JPRB, 0.23638E+00_JPRB ,&
&  0.23574E+00_JPRB, 0.23499E+00_JPRB, 0.23314E+00_JPRB, 0.22807E+00_JPRB, 0.22224E+00_JPRB, 0.21843E+00_JPRB, 0.21583E+00_JPRB ,&
&  0.21594E+00_JPRB, 0.21787E+00_JPRB, 0.21952E+00_JPRB, 0.21967E+00_JPRB, 0.21849E+00_JPRB, 0.21638E+00_JPRB, 0.21316E+00_JPRB ,&
&  0.20833E+00_JPRB, 0.20621E+00_JPRB, 0.20547E+00_JPRB, 0.20454E+00_JPRB, 0.20497E+00_JPRB, 0.20675E+00_JPRB, 0.20848E+00_JPRB ,&
&  0.21054E+00_JPRB, 0.21058E+00_JPRB, 0.20918E+00_JPRB, 0.20681E+00_JPRB, 0.20432E+00_JPRB, 0.20409E+00_JPRB, 0.20462E+00_JPRB ,&
&  0.20507E+00_JPRB, 0.20544E+00_JPRB, 0.20617E+00_JPRB, 0.20721E+00_JPRB, 0.20865E+00_JPRB, 0.21083E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21264E+00_JPRB, 0.21155E+00_JPRB, 0.21257E+00_JPRB, 0.21418E+00_JPRB, 0.21501E+00_JPRB, 0.21560E+00_JPRB, 0.21601E+00_JPRB ,&
&  0.21574E+00_JPRB, 0.21588E+00_JPRB, 0.21782E+00_JPRB, 0.22242E+00_JPRB, 0.22663E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.24223E+00_JPRB, 0.24608E+00_JPRB, 0.25124E+00_JPRB, 0.25260E+00_JPRB, 0.25526E+00_JPRB, 0.25795E+00_JPRB, 0.26138E+00_JPRB ,&
&  0.26351E+00_JPRB, 0.26358E+00_JPRB, 0.26199E+00_JPRB, 0.26041E+00_JPRB, 0.25952E+00_JPRB, 0.25911E+00_JPRB, 0.25848E+00_JPRB ,&
&  0.25820E+00_JPRB, 0.25830E+00_JPRB, 0.25825E+00_JPRB, 0.25811E+00_JPRB, 0.25705E+00_JPRB, 0.25542E+00_JPRB, 0.25362E+00_JPRB ,&
&  0.25052E+00_JPRB, 0.24855E+00_JPRB, 0.24806E+00_JPRB, 0.24804E+00_JPRB, 0.24755E+00_JPRB, 0.24823E+00_JPRB, 0.25005E+00_JPRB ,&
&  0.25386E+00_JPRB, 0.25694E+00_JPRB, 0.26087E+00_JPRB, 0.26493E+00_JPRB, 0.26740E+00_JPRB, 0.26918E+00_JPRB, 0.27124E+00_JPRB ,&
&  0.27256E+00_JPRB, 0.27290E+00_JPRB, 0.27290E+00_JPRB, 0.27280E+00_JPRB, 0.26989E+00_JPRB, 0.26874E+00_JPRB, 0.26803E+00_JPRB ,&
&  0.26677E+00_JPRB, 0.26420E+00_JPRB, 0.26061E+00_JPRB, 0.25746E+00_JPRB, 0.25435E+00_JPRB, 0.25210E+00_JPRB, 0.25026E+00_JPRB ,&
&  0.24894E+00_JPRB, 0.24839E+00_JPRB, 0.24851E+00_JPRB, 0.24721E+00_JPRB, 0.24225E+00_JPRB, 0.23731E+00_JPRB, 0.23452E+00_JPRB ,&
&  0.23321E+00_JPRB, 0.23295E+00_JPRB, 0.23434E+00_JPRB, 0.23322E+00_JPRB, 0.22965E+00_JPRB, 0.22681E+00_JPRB, 0.22562E+00_JPRB ,&
&  0.22404E+00_JPRB, 0.22324E+00_JPRB, 0.22382E+00_JPRB, 0.22440E+00_JPRB, 0.22505E+00_JPRB, 0.22535E+00_JPRB, 0.22647E+00_JPRB ,&
&  0.22740E+00_JPRB, 0.22875E+00_JPRB, 0.22852E+00_JPRB, 0.22583E+00_JPRB, 0.22213E+00_JPRB, 0.21943E+00_JPRB, 0.21886E+00_JPRB ,&
&  0.22023E+00_JPRB, 0.22159E+00_JPRB, 0.22190E+00_JPRB, 0.22195E+00_JPRB, 0.22296E+00_JPRB, 0.22537E+00_JPRB, 0.22850E+00_JPRB ,&
&  0.22982E+00_JPRB, 0.23109E+00_JPRB, 0.23288E+00_JPRB, 0.23473E+00_JPRB, 0.23600E+00_JPRB, 0.23678E+00_JPRB, 0.23708E+00_JPRB ,&
&  0.23653E+00_JPRB, 0.23729E+00_JPRB, 0.23884E+00_JPRB, 0.24064E+00_JPRB, 0.24192E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.26227E+00_JPRB, 0.26485E+00_JPRB, 0.26820E+00_JPRB, 0.26902E+00_JPRB, 0.26953E+00_JPRB, 0.27112E+00_JPRB, 0.27417E+00_JPRB ,&
&  0.27621E+00_JPRB, 0.27672E+00_JPRB, 0.27651E+00_JPRB, 0.27613E+00_JPRB, 0.27630E+00_JPRB, 0.27768E+00_JPRB, 0.27825E+00_JPRB ,&
&  0.27753E+00_JPRB, 0.27688E+00_JPRB, 0.27604E+00_JPRB, 0.27545E+00_JPRB, 0.27411E+00_JPRB, 0.27210E+00_JPRB, 0.27049E+00_JPRB ,&
&  0.26820E+00_JPRB, 0.26613E+00_JPRB, 0.26469E+00_JPRB, 0.26365E+00_JPRB, 0.26322E+00_JPRB, 0.26351E+00_JPRB, 0.26525E+00_JPRB ,&
&  0.26735E+00_JPRB, 0.26998E+00_JPRB, 0.27316E+00_JPRB, 0.27678E+00_JPRB, 0.28042E+00_JPRB, 0.28454E+00_JPRB, 0.28819E+00_JPRB ,&
&  0.29105E+00_JPRB, 0.29339E+00_JPRB, 0.29481E+00_JPRB, 0.29185E+00_JPRB, 0.28817E+00_JPRB, 0.28701E+00_JPRB, 0.28633E+00_JPRB ,&
&  0.28381E+00_JPRB, 0.28032E+00_JPRB, 0.27569E+00_JPRB, 0.27301E+00_JPRB, 0.27220E+00_JPRB, 0.27110E+00_JPRB, 0.26980E+00_JPRB ,&
&  0.26898E+00_JPRB, 0.26845E+00_JPRB, 0.26746E+00_JPRB, 0.26559E+00_JPRB, 0.26238E+00_JPRB, 0.25788E+00_JPRB, 0.25332E+00_JPRB ,&
&  0.25042E+00_JPRB, 0.24843E+00_JPRB, 0.24854E+00_JPRB, 0.24668E+00_JPRB, 0.24271E+00_JPRB, 0.23903E+00_JPRB, 0.23851E+00_JPRB ,&
&  0.23850E+00_JPRB, 0.23887E+00_JPRB, 0.23865E+00_JPRB, 0.23755E+00_JPRB, 0.23786E+00_JPRB, 0.23886E+00_JPRB, 0.24036E+00_JPRB ,&
&  0.24419E+00_JPRB, 0.24685E+00_JPRB, 0.24663E+00_JPRB, 0.24547E+00_JPRB, 0.24352E+00_JPRB, 0.24303E+00_JPRB, 0.24352E+00_JPRB ,&
&  0.24400E+00_JPRB, 0.24412E+00_JPRB, 0.24426E+00_JPRB, 0.24366E+00_JPRB, 0.24377E+00_JPRB, 0.24523E+00_JPRB, 0.24706E+00_JPRB ,&
&  0.24925E+00_JPRB, 0.25197E+00_JPRB, 0.25436E+00_JPRB, 0.25695E+00_JPRB, 0.25943E+00_JPRB, 0.26089E+00_JPRB, 0.26219E+00_JPRB ,&
&  0.26338E+00_JPRB, 0.26366E+00_JPRB, 0.26334E+00_JPRB, 0.26315E+00_JPRB, 0.26296E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.28926E+00_JPRB, 0.28937E+00_JPRB, 0.29031E+00_JPRB, 0.29074E+00_JPRB, 0.29057E+00_JPRB, 0.29106E+00_JPRB, 0.29354E+00_JPRB ,&
&  0.29568E+00_JPRB, 0.29684E+00_JPRB, 0.29769E+00_JPRB, 0.29781E+00_JPRB, 0.29757E+00_JPRB, 0.29882E+00_JPRB, 0.29981E+00_JPRB ,&
&  0.29981E+00_JPRB, 0.29864E+00_JPRB, 0.29739E+00_JPRB, 0.29591E+00_JPRB, 0.29393E+00_JPRB, 0.29190E+00_JPRB, 0.29038E+00_JPRB ,&
&  0.28831E+00_JPRB, 0.28601E+00_JPRB, 0.28379E+00_JPRB, 0.28317E+00_JPRB, 0.28350E+00_JPRB, 0.28398E+00_JPRB, 0.28515E+00_JPRB ,&
&  0.28629E+00_JPRB, 0.28900E+00_JPRB, 0.29215E+00_JPRB, 0.29575E+00_JPRB, 0.29931E+00_JPRB, 0.30405E+00_JPRB, 0.30928E+00_JPRB ,&
&  0.31409E+00_JPRB, 0.31643E+00_JPRB, 0.31803E+00_JPRB, 0.31601E+00_JPRB, 0.31546E+00_JPRB, 0.31476E+00_JPRB, 0.31432E+00_JPRB ,&
&  0.31320E+00_JPRB, 0.30848E+00_JPRB, 0.30462E+00_JPRB, 0.30421E+00_JPRB, 0.30490E+00_JPRB, 0.30535E+00_JPRB, 0.30548E+00_JPRB ,&
&  0.30466E+00_JPRB, 0.30333E+00_JPRB, 0.30057E+00_JPRB, 0.29668E+00_JPRB, 0.29250E+00_JPRB, 0.28738E+00_JPRB, 0.28002E+00_JPRB ,&
&  0.27309E+00_JPRB, 0.26745E+00_JPRB, 0.26317E+00_JPRB, 0.25946E+00_JPRB, 0.25688E+00_JPRB, 0.25399E+00_JPRB, 0.25309E+00_JPRB ,&
&  0.25335E+00_JPRB, 0.25513E+00_JPRB, 0.25617E+00_JPRB, 0.25550E+00_JPRB, 0.25438E+00_JPRB, 0.25457E+00_JPRB, 0.25666E+00_JPRB ,&
&  0.26050E+00_JPRB, 0.26343E+00_JPRB, 0.26431E+00_JPRB, 0.26380E+00_JPRB, 0.26292E+00_JPRB, 0.26363E+00_JPRB, 0.26529E+00_JPRB ,&
&  0.26606E+00_JPRB, 0.26652E+00_JPRB, 0.26766E+00_JPRB, 0.26760E+00_JPRB, 0.26954E+00_JPRB, 0.27277E+00_JPRB, 0.27632E+00_JPRB ,&
&  0.27973E+00_JPRB, 0.28398E+00_JPRB, 0.28841E+00_JPRB, 0.29229E+00_JPRB, 0.29586E+00_JPRB, 0.29663E+00_JPRB, 0.29753E+00_JPRB ,&
&  0.29802E+00_JPRB, 0.29701E+00_JPRB, 0.29563E+00_JPRB, 0.29357E+00_JPRB, 0.29098E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.32122E+00_JPRB, 0.31977E+00_JPRB, 0.31849E+00_JPRB, 0.31785E+00_JPRB, 0.31721E+00_JPRB, 0.31647E+00_JPRB, 0.31613E+00_JPRB ,&
&  0.31647E+00_JPRB, 0.31633E+00_JPRB, 0.31851E+00_JPRB, 0.31907E+00_JPRB, 0.31884E+00_JPRB, 0.31948E+00_JPRB, 0.31992E+00_JPRB ,&
&  0.31849E+00_JPRB, 0.31735E+00_JPRB, 0.31620E+00_JPRB, 0.31410E+00_JPRB, 0.31068E+00_JPRB, 0.30903E+00_JPRB, 0.30771E+00_JPRB ,&
&  0.30542E+00_JPRB, 0.30263E+00_JPRB, 0.30027E+00_JPRB, 0.29936E+00_JPRB, 0.30006E+00_JPRB, 0.30028E+00_JPRB, 0.30141E+00_JPRB ,&
&  0.30205E+00_JPRB, 0.30545E+00_JPRB, 0.29410E+00_JPRB, 0.30377E+00_JPRB, 0.31300E+00_JPRB, 0.32220E+00_JPRB, 0.33180E+00_JPRB ,&
&  0.33839E+00_JPRB, 0.34427E+00_JPRB, 0.35074E+00_JPRB, 0.35228E+00_JPRB, 0.35104E+00_JPRB, 0.34791E+00_JPRB, 0.34688E+00_JPRB ,&
&  0.34628E+00_JPRB, 0.34394E+00_JPRB, 0.34230E+00_JPRB, 0.34221E+00_JPRB, 0.34258E+00_JPRB, 0.34147E+00_JPRB, 0.33958E+00_JPRB ,&
&  0.33803E+00_JPRB, 0.33575E+00_JPRB, 0.33178E+00_JPRB, 0.32724E+00_JPRB, 0.32232E+00_JPRB, 0.31618E+00_JPRB, 0.30786E+00_JPRB ,&
&  0.29832E+00_JPRB, 0.29067E+00_JPRB, 0.28413E+00_JPRB, 0.27993E+00_JPRB, 0.27779E+00_JPRB, 0.27558E+00_JPRB, 0.27353E+00_JPRB ,&
&  0.27298E+00_JPRB, 0.27343E+00_JPRB, 0.27373E+00_JPRB, 0.27369E+00_JPRB, 0.27453E+00_JPRB, 0.27522E+00_JPRB, 0.27767E+00_JPRB ,&
&  0.28032E+00_JPRB, 0.28117E+00_JPRB, 0.28104E+00_JPRB, 0.28108E+00_JPRB, 0.27978E+00_JPRB, 0.28028E+00_JPRB, 0.28194E+00_JPRB ,&
&  0.28385E+00_JPRB, 0.28645E+00_JPRB, 0.28840E+00_JPRB, 0.29015E+00_JPRB, 0.29246E+00_JPRB, 0.29595E+00_JPRB, 0.30121E+00_JPRB ,&
&  0.30624E+00_JPRB, 0.31201E+00_JPRB, 0.31882E+00_JPRB, 0.32525E+00_JPRB, 0.32977E+00_JPRB, 0.33220E+00_JPRB, 0.33427E+00_JPRB ,&
&  0.33449E+00_JPRB, 0.33148E+00_JPRB, 0.32784E+00_JPRB, 0.32529E+00_JPRB, 0.32309E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.36402E+00_JPRB, 0.36058E+00_JPRB, 0.35631E+00_JPRB, 0.35301E+00_JPRB, 0.34972E+00_JPRB, 0.34589E+00_JPRB, 0.34360E+00_JPRB ,&
&  0.34247E+00_JPRB, 0.34213E+00_JPRB, 0.34275E+00_JPRB, 0.34263E+00_JPRB, 0.34232E+00_JPRB, 0.34272E+00_JPRB, 0.34217E+00_JPRB ,&
&  0.34118E+00_JPRB, 0.33973E+00_JPRB, 0.33795E+00_JPRB, 0.33544E+00_JPRB, 0.33228E+00_JPRB, 0.32940E+00_JPRB, 0.32552E+00_JPRB ,&
&  0.32294E+00_JPRB, 0.32163E+00_JPRB, 0.32108E+00_JPRB, 0.32125E+00_JPRB, 0.32277E+00_JPRB, 0.32479E+00_JPRB, 0.32678E+00_JPRB ,&
&  0.32715E+00_JPRB, 0.31755E+00_JPRB, 0.32297E+00_JPRB, 0.32649E+00_JPRB, 0.33613E+00_JPRB, 0.35175E+00_JPRB, 0.36532E+00_JPRB ,&
&  0.37327E+00_JPRB, 0.38122E+00_JPRB, 0.38787E+00_JPRB, 0.39110E+00_JPRB, 0.38910E+00_JPRB, 0.38711E+00_JPRB, 0.38839E+00_JPRB ,&
&  0.38931E+00_JPRB, 0.38830E+00_JPRB, 0.38697E+00_JPRB, 0.38509E+00_JPRB, 0.38228E+00_JPRB, 0.37922E+00_JPRB, 0.37436E+00_JPRB ,&
&  0.36944E+00_JPRB, 0.36588E+00_JPRB, 0.36224E+00_JPRB, 0.35848E+00_JPRB, 0.35285E+00_JPRB, 0.34612E+00_JPRB, 0.33809E+00_JPRB ,&
&  0.32943E+00_JPRB, 0.32302E+00_JPRB, 0.31860E+00_JPRB, 0.31434E+00_JPRB, 0.31015E+00_JPRB, 0.30729E+00_JPRB, 0.30413E+00_JPRB ,&
&  0.30295E+00_JPRB, 0.30365E+00_JPRB, 0.30316E+00_JPRB, 0.30293E+00_JPRB, 0.30356E+00_JPRB, 0.30330E+00_JPRB, 0.30350E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.30064E+00_JPRB, 0.29968E+00_JPRB, 0.29908E+00_JPRB, 0.29861E+00_JPRB, 0.29998E+00_JPRB, 0.30242E+00_JPRB ,&
&  0.30509E+00_JPRB, 0.30894E+00_JPRB, 0.31312E+00_JPRB, 0.31629E+00_JPRB, 0.31906E+00_JPRB, 0.32351E+00_JPRB, 0.32924E+00_JPRB ,&
&  0.33599E+00_JPRB, 0.34414E+00_JPRB, 0.35237E+00_JPRB, 0.36046E+00_JPRB, 0.36582E+00_JPRB, 0.36897E+00_JPRB, 0.37167E+00_JPRB ,&
&  0.37310E+00_JPRB, 0.37002E+00_JPRB, 0.36918E+00_JPRB, 0.36874E+00_JPRB, 0.36671E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.39578E+00_JPRB, 0.39397E+00_JPRB, 0.39046E+00_JPRB, 0.38613E+00_JPRB, 0.38125E+00_JPRB, 0.37851E+00_JPRB, 0.37697E+00_JPRB ,&
&  0.37756E+00_JPRB, 0.37647E+00_JPRB, 0.37681E+00_JPRB, 0.37425E+00_JPRB, 0.37147E+00_JPRB, 0.36937E+00_JPRB, 0.36708E+00_JPRB ,&
&  0.36444E+00_JPRB, 0.36115E+00_JPRB, 0.35798E+00_JPRB, 0.35527E+00_JPRB, 0.35201E+00_JPRB, 0.34762E+00_JPRB, 0.34359E+00_JPRB ,&
&  0.34067E+00_JPRB, 0.33953E+00_JPRB, 0.33906E+00_JPRB, 0.33892E+00_JPRB, 0.33892E+00_JPRB, 0.34043E+00_JPRB, 0.34180E+00_JPRB ,&
&  0.34482E+00_JPRB, 0.31768E+00_JPRB, 0.32936E+00_JPRB, 0.33706E+00_JPRB, 0.34785E+00_JPRB, 0.36614E+00_JPRB, 0.38667E+00_JPRB ,&
&  0.40720E+00_JPRB, 0.42501E+00_JPRB, 0.43648E+00_JPRB, 0.43662E+00_JPRB, 0.42911E+00_JPRB, 0.42363E+00_JPRB, 0.42293E+00_JPRB ,&
&  0.42094E+00_JPRB, 0.41674E+00_JPRB, 0.41308E+00_JPRB, 0.40918E+00_JPRB, 0.40362E+00_JPRB, 0.39537E+00_JPRB, 0.38687E+00_JPRB ,&
&  0.38203E+00_JPRB, 0.37939E+00_JPRB, 0.37703E+00_JPRB, 0.37436E+00_JPRB, 0.36968E+00_JPRB, 0.36531E+00_JPRB, 0.36152E+00_JPRB ,&
&  0.35774E+00_JPRB, 0.35194E+00_JPRB, 0.34811E+00_JPRB, 0.34542E+00_JPRB, 0.34191E+00_JPRB, 0.33961E+00_JPRB, 0.33790E+00_JPRB ,&
&  0.33614E+00_JPRB, 0.33404E+00_JPRB, 0.33267E+00_JPRB, 0.33195E+00_JPRB, 0.32955E+00_JPRB, 0.32705E+00_JPRB, 0.32417E+00_JPRB ,&
&  0.32273E+00_JPRB, 0.32178E+00_JPRB, 0.32068E+00_JPRB, 0.32055E+00_JPRB, 0.32140E+00_JPRB, 0.32374E+00_JPRB, 0.32693E+00_JPRB ,&
&  0.33076E+00_JPRB, 0.33517E+00_JPRB, 0.33917E+00_JPRB, 0.34239E+00_JPRB, 0.34674E+00_JPRB, 0.35043E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.36226E+00_JPRB, 0.36932E+00_JPRB, 0.37667E+00_JPRB, 0.38595E+00_JPRB, 0.39561E+00_JPRB, 0.40146E+00_JPRB, 0.40604E+00_JPRB ,&
&  0.40927E+00_JPRB, 0.40586E+00_JPRB, 0.40312E+00_JPRB, 0.39920E+00_JPRB, 0.39603E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.43704E+00_JPRB, 0.43396E+00_JPRB, 0.42977E+00_JPRB, 0.42411E+00_JPRB, 0.41901E+00_JPRB, 0.41515E+00_JPRB, 0.41296E+00_JPRB ,&
&  0.41284E+00_JPRB, 0.41342E+00_JPRB, 0.41189E+00_JPRB, 0.40702E+00_JPRB, 0.40236E+00_JPRB, 0.39909E+00_JPRB, 0.39717E+00_JPRB ,&
&  0.39453E+00_JPRB, 0.39034E+00_JPRB, 0.38481E+00_JPRB, 0.37948E+00_JPRB, 0.37360E+00_JPRB, 0.36877E+00_JPRB, 0.36486E+00_JPRB ,&
&  0.36177E+00_JPRB, 0.36059E+00_JPRB, 0.35992E+00_JPRB, 0.35968E+00_JPRB, 0.36070E+00_JPRB, 0.36097E+00_JPRB, 0.36639E+00_JPRB ,&
&  0.34180E+00_JPRB, 0.31400E+00_JPRB, 0.33409E+00_JPRB, 0.35330E+00_JPRB, 0.36502E+00_JPRB, 0.38922E+00_JPRB, 0.42234E+00_JPRB ,&
&  0.45882E+00_JPRB, 0.49040E+00_JPRB, 0.51025E+00_JPRB, 0.50701E+00_JPRB, 0.49307E+00_JPRB, 0.48189E+00_JPRB, 0.47460E+00_JPRB ,&
&  0.46355E+00_JPRB, 0.45208E+00_JPRB, 0.44322E+00_JPRB, 0.43332E+00_JPRB, 0.42267E+00_JPRB, 0.41253E+00_JPRB, 0.40411E+00_JPRB ,&
&  0.39724E+00_JPRB, 0.39301E+00_JPRB, 0.39178E+00_JPRB, 0.39022E+00_JPRB, 0.38902E+00_JPRB, 0.38944E+00_JPRB, 0.38846E+00_JPRB ,&
&  0.38675E+00_JPRB, 0.38399E+00_JPRB, 0.38244E+00_JPRB, 0.38093E+00_JPRB, 0.38042E+00_JPRB, 0.37888E+00_JPRB, 0.37666E+00_JPRB ,&
&  0.37407E+00_JPRB, 0.37201E+00_JPRB, 0.37083E+00_JPRB, 0.36789E+00_JPRB, 0.36104E+00_JPRB, 0.35453E+00_JPRB, 0.35086E+00_JPRB ,&
&  0.34969E+00_JPRB, 0.34941E+00_JPRB, 0.34915E+00_JPRB, 0.35008E+00_JPRB, 0.35183E+00_JPRB, 0.35425E+00_JPRB, 0.35783E+00_JPRB ,&
&  0.36264E+00_JPRB, 0.36765E+00_JPRB, 0.37163E+00_JPRB, 0.37527E+00_JPRB, 0.38095E+00_JPRB, 0.38650E+00_JPRB, 0.39248E+00_JPRB ,&
&  0.39907E+00_JPRB, 0.40663E+00_JPRB, 0.41489E+00_JPRB, 0.42817E+00_JPRB, 0.44417E+00_JPRB, 0.45640E+00_JPRB, 0.46014E+00_JPRB ,&
&  0.45741E+00_JPRB, 0.44703E+00_JPRB, 0.44524E+00_JPRB, 0.43989E+00_JPRB, 0.43725E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.46047E+00_JPRB, 0.46185E+00_JPRB, 0.45909E+00_JPRB, 0.45498E+00_JPRB, 0.45177E+00_JPRB, 0.44699E+00_JPRB, 0.44294E+00_JPRB ,&
&  0.43775E+00_JPRB, 0.43424E+00_JPRB, 0.42832E+00_JPRB, 0.42438E+00_JPRB, 0.42175E+00_JPRB, 0.42027E+00_JPRB, 0.41808E+00_JPRB ,&
&  0.41353E+00_JPRB, 0.40641E+00_JPRB, 0.39923E+00_JPRB, 0.39357E+00_JPRB, 0.38965E+00_JPRB, 0.38572E+00_JPRB, 0.38228E+00_JPRB ,&
&  0.37920E+00_JPRB, 0.37642E+00_JPRB, 0.37381E+00_JPRB, 0.37418E+00_JPRB, 0.37489E+00_JPRB, 0.37473E+00_JPRB, 0.37336E+00_JPRB ,&
&  0.38102E+00_JPRB, 0.33787E+00_JPRB, 0.36674E+00_JPRB, 0.38906E+00_JPRB, 0.40102E+00_JPRB, 0.43395E+00_JPRB, 0.48270E+00_JPRB ,&
&  0.52490E+00_JPRB, 0.56054E+00_JPRB, 0.57906E+00_JPRB, 0.56972E+00_JPRB, 0.55172E+00_JPRB, 0.53925E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.50219E+00_JPRB, 0.48358E+00_JPRB, 0.46878E+00_JPRB, 0.45443E+00_JPRB, 0.44071E+00_JPRB, 0.42895E+00_JPRB, 0.41841E+00_JPRB ,&
&  0.41157E+00_JPRB, 0.40855E+00_JPRB, 0.40632E+00_JPRB, 0.40462E+00_JPRB, 0.40498E+00_JPRB, 0.40711E+00_JPRB, 0.40633E+00_JPRB ,&
&  0.40492E+00_JPRB, 0.40598E+00_JPRB, 0.40544E+00_JPRB, 0.40469E+00_JPRB, 0.40591E+00_JPRB, 0.41085E+00_JPRB, 0.41088E+00_JPRB ,&
&  0.40675E+00_JPRB, 0.39986E+00_JPRB, 0.39322E+00_JPRB, 0.38812E+00_JPRB, 0.38384E+00_JPRB, 0.37966E+00_JPRB, 0.37814E+00_JPRB ,&
&  0.37623E+00_JPRB, 0.37614E+00_JPRB, 0.37646E+00_JPRB, 0.37857E+00_JPRB, 0.38108E+00_JPRB, 0.38438E+00_JPRB, 0.38890E+00_JPRB ,&
&  0.39432E+00_JPRB, 0.39981E+00_JPRB, 0.40679E+00_JPRB, 0.41423E+00_JPRB, 0.42073E+00_JPRB, 0.42545E+00_JPRB, 0.42916E+00_JPRB ,&
&  0.43314E+00_JPRB, 0.44007E+00_JPRB, 0.44516E+00_JPRB, 0.46930E+00_JPRB, 0.48598E+00_JPRB, 0.50089E+00_JPRB, 0.49950E+00_JPRB ,&
&  0.49039E+00_JPRB, 0.47957E+00_JPRB, 0.41016E+00_JPRB, 0.44134E+00_JPRB, 0.45318E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.50803E+00_JPRB, 0.50652E+00_JPRB, 0.50231E+00_JPRB, 0.49883E+00_JPRB, 0.49150E+00_JPRB, 0.48359E+00_JPRB, 0.47593E+00_JPRB ,&
&  0.46635E+00_JPRB, 0.45667E+00_JPRB, 0.44815E+00_JPRB, 0.44482E+00_JPRB, 0.44162E+00_JPRB, 0.43798E+00_JPRB, 0.43075E+00_JPRB ,&
&  0.42131E+00_JPRB, 0.41224E+00_JPRB, 0.40423E+00_JPRB, 0.39946E+00_JPRB, 0.39842E+00_JPRB, 0.39675E+00_JPRB, 0.39401E+00_JPRB ,&
&  0.39250E+00_JPRB, 0.39354E+00_JPRB, 0.39307E+00_JPRB, 0.39243E+00_JPRB, 0.39138E+00_JPRB, 0.38661E+00_JPRB, 0.38681E+00_JPRB ,&
&  0.39772E+00_JPRB, 0.36904E+00_JPRB, 0.42415E+00_JPRB, 0.47321E+00_JPRB, 0.49806E+00_JPRB, 0.53629E+00_JPRB, 0.59167E+00_JPRB ,&
&  0.65095E+00_JPRB, 0.67088E+00_JPRB, 0.66641E+00_JPRB, 0.63412E+00_JPRB, 0.60758E+00_JPRB, 0.58652E+00_JPRB, 0.55742E+00_JPRB ,&
&  0.52965E+00_JPRB, 0.51035E+00_JPRB, 0.49516E+00_JPRB, 0.48165E+00_JPRB, 0.47126E+00_JPRB, 0.46281E+00_JPRB, 0.45586E+00_JPRB ,&
&  0.44782E+00_JPRB, 0.43997E+00_JPRB, 0.43293E+00_JPRB, 0.42890E+00_JPRB, 0.42927E+00_JPRB, 0.43286E+00_JPRB, 0.43762E+00_JPRB ,&
&  0.44215E+00_JPRB, 0.44564E+00_JPRB, 0.44596E+00_JPRB, 0.44763E+00_JPRB, 0.45145E+00_JPRB, 0.45429E+00_JPRB, 0.45111E+00_JPRB ,&
&  0.44124E+00_JPRB, 0.43016E+00_JPRB, 0.42249E+00_JPRB, 0.41788E+00_JPRB, 0.41608E+00_JPRB, 0.41652E+00_JPRB, 0.41877E+00_JPRB ,&
&  0.41863E+00_JPRB, 0.41734E+00_JPRB, 0.41598E+00_JPRB, 0.41666E+00_JPRB, 0.41819E+00_JPRB, 0.42248E+00_JPRB, 0.42872E+00_JPRB ,&
&  0.43578E+00_JPRB, 0.44312E+00_JPRB, 0.45162E+00_JPRB, 0.46006E+00_JPRB, 0.46496E+00_JPRB, 0.46986E+00_JPRB, 0.47253E+00_JPRB ,&
&  0.47519E+00_JPRB, 0.48319E+00_JPRB, 0.49311E+00_JPRB, 0.54041E+00_JPRB, 0.56779E+00_JPRB, 0.57074E+00_JPRB, 0.54577E+00_JPRB ,&
&  0.52334E+00_JPRB, 0.50087E+00_JPRB, 0.49810E+00_JPRB, 0.46035E+00_JPRB, 0.49780E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.56369E+00_JPRB, 0.55672E+00_JPRB, 0.54695E+00_JPRB, 0.53817E+00_JPRB, 0.52570E+00_JPRB, 0.51515E+00_JPRB, 0.50183E+00_JPRB ,&
&  0.48968E+00_JPRB, 0.47778E+00_JPRB, 0.46722E+00_JPRB, 0.46328E+00_JPRB, 0.45615E+00_JPRB, 0.44595E+00_JPRB, 0.42988E+00_JPRB ,&
&  0.41654E+00_JPRB, 0.40610E+00_JPRB, 0.40001E+00_JPRB, 0.39793E+00_JPRB, 0.39675E+00_JPRB, 0.39420E+00_JPRB, 0.39310E+00_JPRB ,&
&  0.39395E+00_JPRB, 0.39358E+00_JPRB, 0.39590E+00_JPRB, 0.39561E+00_JPRB, 0.39900E+00_JPRB, 0.39520E+00_JPRB, 0.39266E+00_JPRB ,&
&  0.40987E+00_JPRB, 0.41425E+00_JPRB, 0.50044E+00_JPRB, 0.58927E+00_JPRB, 0.66165E+00_JPRB, 0.70825E+00_JPRB, 0.73818E+00_JPRB ,&
&  0.76348E+00_JPRB, 0.74885E+00_JPRB, 0.71733E+00_JPRB, 0.66947E+00_JPRB, 0.62893E+00_JPRB, 0.58746E+00_JPRB, 0.54826E+00_JPRB ,&
&  0.52839E+00_JPRB, 0.51766E+00_JPRB, 0.50870E+00_JPRB, 0.50050E+00_JPRB, 0.49189E+00_JPRB, 0.48642E+00_JPRB, 0.48354E+00_JPRB ,&
&  0.47713E+00_JPRB, 0.46922E+00_JPRB, 0.45995E+00_JPRB, 0.45589E+00_JPRB, 0.45377E+00_JPRB, 0.46294E+00_JPRB, 0.47813E+00_JPRB ,&
&  0.48588E+00_JPRB, 0.49336E+00_JPRB, 0.49978E+00_JPRB, 0.50882E+00_JPRB, 0.51101E+00_JPRB, 0.50793E+00_JPRB, 0.49267E+00_JPRB ,&
&  0.48016E+00_JPRB, 0.47192E+00_JPRB, 0.46785E+00_JPRB, 0.46597E+00_JPRB, 0.46563E+00_JPRB, 0.46394E+00_JPRB, 0.46393E+00_JPRB ,&
&  0.46092E+00_JPRB, 0.45720E+00_JPRB, 0.45628E+00_JPRB, 0.45703E+00_JPRB, 0.45831E+00_JPRB, 0.45971E+00_JPRB, 0.46196E+00_JPRB ,&
&  0.46554E+00_JPRB, 0.46928E+00_JPRB, 0.47492E+00_JPRB, 0.48029E+00_JPRB, 0.48193E+00_JPRB, 0.48504E+00_JPRB, 0.49203E+00_JPRB ,&
&  0.49902E+00_JPRB, 0.51388E+00_JPRB, 0.52785E+00_JPRB, 0.58324E+00_JPRB, 0.60738E+00_JPRB, 0.59375E+00_JPRB, 0.56100E+00_JPRB ,&
&  0.53480E+00_JPRB, 0.52222E+00_JPRB, 0.52210E+00_JPRB, 0.54169E+00_JPRB, 0.54030E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.67327E+00_JPRB, 0.63113E+00_JPRB, 0.59224E+00_JPRB, 0.56837E+00_JPRB, 0.55249E+00_JPRB, 0.54066E+00_JPRB, 0.52632E+00_JPRB ,&
&  0.51265E+00_JPRB, 0.49867E+00_JPRB, 0.48740E+00_JPRB, 0.47940E+00_JPRB, 0.47071E+00_JPRB, 0.45408E+00_JPRB, 0.43225E+00_JPRB ,&
&  0.41751E+00_JPRB, 0.41464E+00_JPRB, 0.41263E+00_JPRB, 0.40769E+00_JPRB, 0.40280E+00_JPRB, 0.40128E+00_JPRB, 0.39874E+00_JPRB ,&
&  0.39846E+00_JPRB, 0.39844E+00_JPRB, 0.40125E+00_JPRB, 0.40269E+00_JPRB, 0.40100E+00_JPRB, 0.40729E+00_JPRB, 0.40711E+00_JPRB ,&
&  0.42904E+00_JPRB, 0.45909E+00_JPRB, 0.59619E+00_JPRB, 0.74938E+00_JPRB, 0.86982E+00_JPRB, 0.90763E+00_JPRB, 0.86971E+00_JPRB ,&
&  0.83449E+00_JPRB, 0.79631E+00_JPRB, 0.74935E+00_JPRB, 0.70243E+00_JPRB, 0.65185E+00_JPRB, 0.59883E+00_JPRB, 0.56736E+00_JPRB ,&
&  0.55563E+00_JPRB, 0.54253E+00_JPRB, 0.52896E+00_JPRB, 0.51237E+00_JPRB, 0.49675E+00_JPRB, 0.49134E+00_JPRB, 0.49213E+00_JPRB ,&
&  0.48857E+00_JPRB, 0.48463E+00_JPRB, 0.48048E+00_JPRB, 0.48322E+00_JPRB, 0.49958E+00_JPRB, 0.53627E+00_JPRB, 0.56954E+00_JPRB ,&
&  0.58253E+00_JPRB, 0.59607E+00_JPRB, 0.60208E+00_JPRB, 0.60216E+00_JPRB, 0.59109E+00_JPRB, 0.57404E+00_JPRB, 0.55396E+00_JPRB ,&
&  0.53723E+00_JPRB, 0.52528E+00_JPRB, 0.51977E+00_JPRB, 0.51565E+00_JPRB, 0.51187E+00_JPRB, 0.50796E+00_JPRB, 0.50333E+00_JPRB ,&
&  0.50081E+00_JPRB, 0.49971E+00_JPRB, 0.50050E+00_JPRB, 0.49894E+00_JPRB, 0.49688E+00_JPRB, 0.49363E+00_JPRB, 0.49033E+00_JPRB ,&
&  0.48539E+00_JPRB, 0.48252E+00_JPRB, 0.49004E+00_JPRB, 0.50444E+00_JPRB, 0.52080E+00_JPRB, 0.53275E+00_JPRB, 0.54168E+00_JPRB ,&
&  0.54872E+00_JPRB, 0.57140E+00_JPRB, 0.57474E+00_JPRB, 0.65096E+00_JPRB, 0.71598E+00_JPRB, 0.66702E+00_JPRB, 0.62553E+00_JPRB ,&
&  0.58935E+00_JPRB, 0.57204E+00_JPRB, 0.56902E+00_JPRB, 0.59037E+00_JPRB, 0.66848E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.62304E+00_JPRB, 0.58484E+00_JPRB, 0.56521E+00_JPRB, 0.55425E+00_JPRB, 0.54874E+00_JPRB, 0.54423E+00_JPRB, 0.53351E+00_JPRB ,&
&  0.51836E+00_JPRB, 0.50475E+00_JPRB, 0.49476E+00_JPRB, 0.48727E+00_JPRB, 0.47251E+00_JPRB, 0.44996E+00_JPRB, 0.43337E+00_JPRB ,&
&  0.43028E+00_JPRB, 0.42748E+00_JPRB, 0.42535E+00_JPRB, 0.41918E+00_JPRB, 0.41284E+00_JPRB, 0.40992E+00_JPRB, 0.40645E+00_JPRB ,&
&  0.40222E+00_JPRB, 0.39825E+00_JPRB, 0.39941E+00_JPRB, 0.40314E+00_JPRB, 0.40672E+00_JPRB, 0.41029E+00_JPRB, 0.41531E+00_JPRB ,&
&  0.44087E+00_JPRB, 0.56316E+00_JPRB, 0.78801E+00_JPRB, 0.10648E+01_JPRB, 0.12219E+01_JPRB, 0.11330E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.95947E+00_JPRB, 0.86458E+00_JPRB, 0.78796E+00_JPRB, 0.72445E+00_JPRB, 0.65654E+00_JPRB, 0.61801E+00_JPRB, 0.59939E+00_JPRB ,&
&  0.58393E+00_JPRB, 0.56346E+00_JPRB, 0.54233E+00_JPRB, 0.52086E+00_JPRB, 0.50631E+00_JPRB, 0.50836E+00_JPRB, 0.51750E+00_JPRB ,&
&  0.51608E+00_JPRB, 0.51676E+00_JPRB, 0.51900E+00_JPRB, 0.54098E+00_JPRB, 0.60574E+00_JPRB, 0.66587E+00_JPRB, 0.72052E+00_JPRB ,&
&  0.73774E+00_JPRB, 0.77638E+00_JPRB, 0.74636E+00_JPRB, 0.71378E+00_JPRB, 0.67938E+00_JPRB, 0.64937E+00_JPRB, 0.62536E+00_JPRB ,&
&  0.60873E+00_JPRB, 0.59493E+00_JPRB, 0.58329E+00_JPRB, 0.57338E+00_JPRB, 0.56404E+00_JPRB, 0.55434E+00_JPRB, 0.55137E+00_JPRB ,&
&  0.55050E+00_JPRB, 0.54374E+00_JPRB, 0.53724E+00_JPRB, 0.52988E+00_JPRB, 0.51810E+00_JPRB, 0.50710E+00_JPRB, 0.49864E+00_JPRB ,&
&  0.49112E+00_JPRB, 0.49265E+00_JPRB, 0.52051E+00_JPRB, 0.54937E+00_JPRB, 0.56906E+00_JPRB, 0.58601E+00_JPRB, 0.59918E+00_JPRB ,&
&  0.61156E+00_JPRB, 0.64392E+00_JPRB, 0.63677E+00_JPRB, 0.67829E+00_JPRB, 0.75667E+00_JPRB, 0.73398E+00_JPRB, 0.67096E+00_JPRB ,&
&  0.64005E+00_JPRB, 0.62271E+00_JPRB, 0.61336E+00_JPRB, 0.61083E+00_JPRB, 0.65995E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.59417E+00_JPRB, 0.56986E+00_JPRB, 0.55815E+00_JPRB, 0.55084E+00_JPRB, 0.54517E+00_JPRB, 0.54041E+00_JPRB, 0.52996E+00_JPRB ,&
&  0.51493E+00_JPRB, 0.50570E+00_JPRB, 0.49508E+00_JPRB, 0.48034E+00_JPRB, 0.45965E+00_JPRB, 0.44689E+00_JPRB, 0.44558E+00_JPRB ,&
&  0.44429E+00_JPRB, 0.44038E+00_JPRB, 0.43324E+00_JPRB, 0.42969E+00_JPRB, 0.42452E+00_JPRB, 0.41778E+00_JPRB, 0.41399E+00_JPRB ,&
&  0.40519E+00_JPRB, 0.40143E+00_JPRB, 0.40348E+00_JPRB, 0.40870E+00_JPRB, 0.41313E+00_JPRB, 0.41947E+00_JPRB, 0.42881E+00_JPRB ,&
&  0.48050E+00_JPRB, 0.68592E+00_JPRB, 0.11499E+01_JPRB, 0.13994E+01_JPRB, 0.14351E+01_JPRB, 0.13229E+01_JPRB, 0.13042E+01_JPRB ,&
&  0.11601E+01_JPRB, 0.10163E+01_JPRB, 0.89560E+00_JPRB, 0.78952E+00_JPRB, 0.70926E+00_JPRB, 0.66708E+00_JPRB, 0.64269E+00_JPRB ,&
&  0.61377E+00_JPRB, 0.58805E+00_JPRB, 0.56422E+00_JPRB, 0.54327E+00_JPRB, 0.53538E+00_JPRB, 0.54322E+00_JPRB, 0.55398E+00_JPRB ,&
&  0.55785E+00_JPRB, 0.56270E+00_JPRB, 0.57769E+00_JPRB, 0.62342E+00_JPRB, 0.69077E+00_JPRB, 0.73840E+00_JPRB, 0.91537E+00_JPRB ,&
&  0.10910E+01_JPRB, 0.10621E+01_JPRB, 0.96871E+00_JPRB, 0.88990E+00_JPRB, 0.81979E+00_JPRB, 0.76212E+00_JPRB, 0.71882E+00_JPRB ,&
&  0.69504E+00_JPRB, 0.67548E+00_JPRB, 0.66047E+00_JPRB, 0.64388E+00_JPRB, 0.62844E+00_JPRB, 0.61871E+00_JPRB, 0.61021E+00_JPRB ,&
&  0.59882E+00_JPRB, 0.58626E+00_JPRB, 0.57126E+00_JPRB, 0.55078E+00_JPRB, 0.52860E+00_JPRB, 0.51616E+00_JPRB, 0.51201E+00_JPRB ,&
&  0.51460E+00_JPRB, 0.54000E+00_JPRB, 0.58133E+00_JPRB, 0.59152E+00_JPRB, 0.62401E+00_JPRB, 0.65049E+00_JPRB, 0.67507E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.67604E+00_JPRB, 0.68924E+00_JPRB, 0.73454E+00_JPRB, 0.82525E+00_JPRB, 0.80556E+00_JPRB, 0.71821E+00_JPRB ,&
&  0.68412E+00_JPRB, 0.67085E+00_JPRB, 0.65366E+00_JPRB, 0.63054E+00_JPRB, 0.62743E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.58436E+00_JPRB, 0.56893E+00_JPRB, 0.55660E+00_JPRB, 0.55018E+00_JPRB, 0.54462E+00_JPRB, 0.53341E+00_JPRB, 0.52277E+00_JPRB ,&
&  0.51217E+00_JPRB, 0.50046E+00_JPRB, 0.48418E+00_JPRB, 0.46009E+00_JPRB, 0.44361E+00_JPRB, 0.43911E+00_JPRB, 0.44473E+00_JPRB ,&
&  0.44620E+00_JPRB, 0.43917E+00_JPRB, 0.43513E+00_JPRB, 0.43471E+00_JPRB, 0.43401E+00_JPRB, 0.42807E+00_JPRB, 0.42001E+00_JPRB ,&
&  0.41339E+00_JPRB, 0.41417E+00_JPRB, 0.41697E+00_JPRB, 0.42149E+00_JPRB, 0.42504E+00_JPRB, 0.43142E+00_JPRB, 0.44920E+00_JPRB ,&
&  0.51966E+00_JPRB, 0.52523E+00_JPRB, 0.12205E+01_JPRB, 0.16912E+01_JPRB, 0.17078E+01_JPRB, 0.16190E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.12480E+01_JPRB, 0.10776E+01_JPRB, 0.93218E+00_JPRB, 0.82254E+00_JPRB, 0.74843E+00_JPRB, 0.69230E+00_JPRB ,&
&  0.65031E+00_JPRB, 0.61760E+00_JPRB, 0.59028E+00_JPRB, 0.57548E+00_JPRB, 0.57704E+00_JPRB, 0.58042E+00_JPRB, 0.58295E+00_JPRB ,&
&  0.58974E+00_JPRB, 0.60265E+00_JPRB, 0.63623E+00_JPRB, 0.70814E+00_JPRB, 0.79864E+00_JPRB, 0.93408E+00_JPRB, 0.11985E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.14214E+01_JPRB, 0.12337E+01_JPRB, 0.10760E+01_JPRB, 0.96800E+00_JPRB, 0.89756E+00_JPRB, 0.84647E+00_JPRB ,&
&  0.80494E+00_JPRB, 0.77310E+00_JPRB, 0.74766E+00_JPRB, 0.71367E+00_JPRB, 0.68935E+00_JPRB, 0.67310E+00_JPRB, 0.65004E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.59990E+00_JPRB, 0.57546E+00_JPRB, 0.55362E+00_JPRB, 0.53561E+00_JPRB, 0.53405E+00_JPRB, 0.54028E+00_JPRB ,&
&  0.55335E+00_JPRB, 0.58150E+00_JPRB, 0.60328E+00_JPRB, 0.62130E+00_JPRB, 0.65329E+00_JPRB, 0.67721E+00_JPRB, 0.69905E+00_JPRB ,&
&  0.69925E+00_JPRB, 0.71699E+00_JPRB, 0.73930E+00_JPRB, 0.78097E+00_JPRB, 0.75838E+00_JPRB, 0.82456E+00_JPRB, 0.75029E+00_JPRB ,&
&  0.72861E+00_JPRB, 0.70722E+00_JPRB, 0.65969E+00_JPRB, 0.63534E+00_JPRB, 0.61105E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.58535E+00_JPRB, 0.56781E+00_JPRB, 0.55369E+00_JPRB, 0.54126E+00_JPRB, 0.53204E+00_JPRB, 0.52632E+00_JPRB, 0.51870E+00_JPRB ,&
&  0.50812E+00_JPRB, 0.49360E+00_JPRB, 0.47929E+00_JPRB, 0.45956E+00_JPRB, 0.45044E+00_JPRB, 0.45181E+00_JPRB, 0.45946E+00_JPRB ,&
&  0.45400E+00_JPRB, 0.44145E+00_JPRB, 0.44083E+00_JPRB, 0.44204E+00_JPRB, 0.43906E+00_JPRB, 0.43206E+00_JPRB, 0.42502E+00_JPRB ,&
&  0.42793E+00_JPRB, 0.43141E+00_JPRB, 0.43393E+00_JPRB, 0.43669E+00_JPRB, 0.44037E+00_JPRB, 0.45139E+00_JPRB, 0.47839E+00_JPRB ,&
&  0.56254E+00_JPRB, 0.92262E+00_JPRB, 0.10301E+01_JPRB, 0.19613E+01_JPRB, 0.20915E+01_JPRB, 0.20221E+01_JPRB, 0.17989E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.14435E+01_JPRB, 0.12524E+01_JPRB, 0.10983E+01_JPRB, 0.94146E+00_JPRB, 0.82244E+00_JPRB, 0.74634E+00_JPRB ,&
&  0.69520E+00_JPRB, 0.65632E+00_JPRB, 0.63294E+00_JPRB, 0.62447E+00_JPRB, 0.61897E+00_JPRB, 0.61667E+00_JPRB, 0.61954E+00_JPRB ,&
&  0.63005E+00_JPRB, 0.65703E+00_JPRB, 0.71197E+00_JPRB, 0.80799E+00_JPRB, 0.87123E+00_JPRB, 0.11454E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.16692E+01_JPRB, 0.17043E+01_JPRB, 0.14436E+01_JPRB, 0.12172E+01_JPRB, 0.10815E+01_JPRB, 0.99628E+00_JPRB, 0.92660E+00_JPRB ,&
&  0.87634E+00_JPRB, 0.83737E+00_JPRB, 0.79406E+00_JPRB, 0.73949E+00_JPRB, 0.70787E+00_JPRB, 0.68364E+00_JPRB, 0.64924E+00_JPRB ,&
&  0.61281E+00_JPRB, 0.58744E+00_JPRB, 0.57278E+00_JPRB, 0.56228E+00_JPRB, 0.55758E+00_JPRB, 0.56451E+00_JPRB, 0.57669E+00_JPRB ,&
&  0.59360E+00_JPRB, 0.62501E+00_JPRB, 0.62859E+00_JPRB, 0.64051E+00_JPRB, 0.66998E+00_JPRB, 0.70484E+00_JPRB, 0.70669E+00_JPRB ,&
&  0.72228E+00_JPRB, 0.74182E+00_JPRB, 0.76595E+00_JPRB, 0.79680E+00_JPRB, 0.78058E+00_JPRB, 0.86570E+00_JPRB, 0.76474E+00_JPRB ,&
&  0.75016E+00_JPRB, 0.69545E+00_JPRB, 0.64999E+00_JPRB, 0.63245E+00_JPRB, 0.61151E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.58138E+00_JPRB, 0.55407E+00_JPRB, 0.53577E+00_JPRB, 0.52534E+00_JPRB, 0.51999E+00_JPRB, 0.51841E+00_JPRB, 0.51064E+00_JPRB ,&
&  0.50363E+00_JPRB, 0.48944E+00_JPRB, 0.47168E+00_JPRB, 0.45684E+00_JPRB, 0.46208E+00_JPRB, 0.46521E+00_JPRB, 0.46500E+00_JPRB ,&
&  0.45742E+00_JPRB, 0.45334E+00_JPRB, 0.45264E+00_JPRB, 0.45001E+00_JPRB, 0.44505E+00_JPRB, 0.44074E+00_JPRB, 0.43997E+00_JPRB ,&
&  0.44601E+00_JPRB, 0.44684E+00_JPRB, 0.44628E+00_JPRB, 0.45019E+00_JPRB, 0.46078E+00_JPRB, 0.48681E+00_JPRB, 0.52628E+00_JPRB ,&
&  0.64051E+00_JPRB, 0.82594E+00_JPRB, 0.10027E+01_JPRB, 0.20637E+01_JPRB, 0.24280E+01_JPRB, 0.23288E+01_JPRB, 0.19881E+01_JPRB ,&
&  0.16778E+01_JPRB, 0.16242E+01_JPRB, 0.14184E+01_JPRB, 0.11916E+01_JPRB, 0.98677E+00_JPRB, 0.86846E+00_JPRB, 0.78339E+00_JPRB ,&
&  0.72441E+00_JPRB, 0.69274E+00_JPRB, 0.68253E+00_JPRB, 0.67937E+00_JPRB, 0.67475E+00_JPRB, 0.68275E+00_JPRB, 0.69317E+00_JPRB ,&
&  0.70533E+00_JPRB, 0.73800E+00_JPRB, 0.79395E+00_JPRB, 0.82916E+00_JPRB, 0.10354E+01_JPRB, 0.15165E+01_JPRB, 0.18582E+01_JPRB ,&
&  0.19300E+01_JPRB, 0.18419E+01_JPRB, 0.15676E+01_JPRB, 0.13016E+01_JPRB, 0.11248E+01_JPRB, 0.10535E+01_JPRB, 0.97751E+00_JPRB ,&
&  0.89653E+00_JPRB, 0.84427E+00_JPRB, 0.77515E+00_JPRB, 0.73242E+00_JPRB, 0.71535E+00_JPRB, 0.66634E+00_JPRB, 0.62643E+00_JPRB ,&
&  0.60563E+00_JPRB, 0.59674E+00_JPRB, 0.59208E+00_JPRB, 0.58729E+00_JPRB, 0.59014E+00_JPRB, 0.59982E+00_JPRB, 0.61719E+00_JPRB ,&
&  0.64155E+00_JPRB, 0.67669E+00_JPRB, 0.66755E+00_JPRB, 0.67961E+00_JPRB, 0.72084E+00_JPRB, 0.75759E+00_JPRB, 0.75770E+00_JPRB ,&
&  0.75757E+00_JPRB, 0.77037E+00_JPRB, 0.78189E+00_JPRB, 0.76367E+00_JPRB, 0.74603E+00_JPRB, 0.80476E+00_JPRB, 0.76751E+00_JPRB ,&
&  0.76565E+00_JPRB, 0.70207E+00_JPRB, 0.66963E+00_JPRB, 0.63125E+00_JPRB, 0.60824E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.55798E+00_JPRB, 0.53196E+00_JPRB, 0.52634E+00_JPRB, 0.52208E+00_JPRB, 0.51037E+00_JPRB, 0.50073E+00_JPRB, 0.49159E+00_JPRB ,&
&  0.48274E+00_JPRB, 0.46750E+00_JPRB, 0.45627E+00_JPRB, 0.46250E+00_JPRB, 0.47924E+00_JPRB, 0.49047E+00_JPRB, 0.48223E+00_JPRB ,&
&  0.47241E+00_JPRB, 0.46680E+00_JPRB, 0.47179E+00_JPRB, 0.47338E+00_JPRB, 0.46963E+00_JPRB, 0.47170E+00_JPRB, 0.47171E+00_JPRB ,&
&  0.47318E+00_JPRB, 0.47036E+00_JPRB, 0.47304E+00_JPRB, 0.48626E+00_JPRB, 0.51064E+00_JPRB, 0.55433E+00_JPRB, 0.61945E+00_JPRB ,&
&  0.81230E+00_JPRB, 0.91205E+00_JPRB, 0.10988E+01_JPRB, 0.18094E+01_JPRB, 0.22742E+01_JPRB, 0.21286E+01_JPRB, 0.19330E+01_JPRB ,&
&  0.16932E+01_JPRB, 0.13619E+01_JPRB, 0.12816E+01_JPRB, 0.11700E+01_JPRB, 0.10073E+01_JPRB, 0.87985E+00_JPRB, 0.80033E+00_JPRB ,&
&  0.75878E+00_JPRB, 0.73287E+00_JPRB, 0.73173E+00_JPRB, 0.74683E+00_JPRB, 0.77273E+00_JPRB, 0.80261E+00_JPRB, 0.81816E+00_JPRB ,&
&  0.82951E+00_JPRB, 0.85396E+00_JPRB, 0.91059E+00_JPRB, 0.99668E+00_JPRB, 0.13986E+01_JPRB, 0.20060E+01_JPRB, 0.22403E+01_JPRB ,&
&  0.21173E+01_JPRB, 0.18874E+01_JPRB, 0.15409E+01_JPRB, 0.12789E+01_JPRB, 0.10334E+01_JPRB, 0.10353E+01_JPRB, 0.95633E+00_JPRB ,&
&  0.86488E+00_JPRB, 0.79669E+00_JPRB, 0.75620E+00_JPRB, 0.71983E+00_JPRB, 0.67752E+00_JPRB, 0.63863E+00_JPRB, 0.61799E+00_JPRB ,&
&  0.61680E+00_JPRB, 0.62336E+00_JPRB, 0.62240E+00_JPRB, 0.61621E+00_JPRB, 0.62795E+00_JPRB, 0.63817E+00_JPRB, 0.66154E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.71606E+00_JPRB, 0.71730E+00_JPRB, 0.75085E+00_JPRB, 0.79467E+00_JPRB, 0.82703E+00_JPRB, 0.81926E+00_JPRB ,&
&  0.79147E+00_JPRB, 0.82580E+00_JPRB, 0.82460E+00_JPRB, 0.80023E+00_JPRB, 0.78664E+00_JPRB, 0.77976E+00_JPRB, 0.75631E+00_JPRB ,&
&  0.77574E+00_JPRB, 0.76132E+00_JPRB, 0.71492E+00_JPRB, 0.62564E+00_JPRB, 0.58845E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.53999E+00_JPRB, 0.52180E+00_JPRB, 0.52163E+00_JPRB, 0.51438E+00_JPRB, 0.50232E+00_JPRB, 0.49570E+00_JPRB, 0.48821E+00_JPRB ,&
&  0.48077E+00_JPRB, 0.48074E+00_JPRB, 0.48300E+00_JPRB, 0.51327E+00_JPRB, 0.52745E+00_JPRB, 0.51910E+00_JPRB, 0.50913E+00_JPRB ,&
&  0.50243E+00_JPRB, 0.50322E+00_JPRB, 0.50658E+00_JPRB, 0.50824E+00_JPRB, 0.50850E+00_JPRB, 0.51600E+00_JPRB, 0.52489E+00_JPRB ,&
&  0.52424E+00_JPRB, 0.52698E+00_JPRB, 0.53953E+00_JPRB, 0.56564E+00_JPRB, 0.61587E+00_JPRB, 0.70051E+00_JPRB, 0.84505E+00_JPRB ,&
&  0.11571E+01_JPRB, 0.11188E+01_JPRB, 0.56410E+00_JPRB, 0.10842E+01_JPRB, 0.17395E+01_JPRB, 0.18110E+01_JPRB, 0.17089E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.11788E+01_JPRB, 0.10900E+01_JPRB, 0.11190E+01_JPRB, 0.98408E+00_JPRB, 0.88203E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.79170E+00_JPRB, 0.79829E+00_JPRB, 0.84389E+00_JPRB, 0.89036E+00_JPRB, 0.93530E+00_JPRB, 0.95674E+00_JPRB, 0.95859E+00_JPRB ,&
&  0.96922E+00_JPRB, 0.10138E+01_JPRB, 0.11327E+01_JPRB, 0.12974E+01_JPRB, 0.20187E+01_JPRB, 0.23286E+01_JPRB, 0.23419E+01_JPRB ,&
&  0.17224E+01_JPRB, 0.15244E+01_JPRB, 0.13342E+01_JPRB, 0.11624E+01_JPRB, 0.98181E+00_JPRB, 0.89412E+00_JPRB, 0.89574E+00_JPRB ,&
&  0.80195E+00_JPRB, 0.73117E+00_JPRB, 0.70715E+00_JPRB, 0.68182E+00_JPRB, 0.65645E+00_JPRB, 0.64291E+00_JPRB, 0.63832E+00_JPRB ,&
&  0.64981E+00_JPRB, 0.66008E+00_JPRB, 0.66253E+00_JPRB, 0.66289E+00_JPRB, 0.67055E+00_JPRB, 0.68269E+00_JPRB, 0.70540E+00_JPRB ,&
&  0.72228E+00_JPRB, 0.75559E+00_JPRB, 0.79877E+00_JPRB, 0.80661E+00_JPRB, 0.83142E+00_JPRB, 0.88368E+00_JPRB, 0.88326E+00_JPRB ,&
&  0.87817E+00_JPRB, 0.94376E+00_JPRB, 0.85360E+00_JPRB, 0.78303E+00_JPRB, 0.78560E+00_JPRB, 0.76410E+00_JPRB, 0.75976E+00_JPRB ,&
&  0.81384E+00_JPRB, 0.94114E+00_JPRB, 0.90854E+00_JPRB, 0.60677E+00_JPRB, 0.55663E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.49942E+00_JPRB, 0.48601E+00_JPRB, 0.48136E+00_JPRB, 0.48815E+00_JPRB, 0.49290E+00_JPRB, 0.49871E+00_JPRB, 0.48610E+00_JPRB ,&
&  0.49875E+00_JPRB, 0.53269E+00_JPRB, 0.55821E+00_JPRB, 0.57657E+00_JPRB, 0.57140E+00_JPRB, 0.56039E+00_JPRB, 0.55585E+00_JPRB ,&
&  0.55575E+00_JPRB, 0.56049E+00_JPRB, 0.57420E+00_JPRB, 0.58077E+00_JPRB, 0.58593E+00_JPRB, 0.59730E+00_JPRB, 0.60526E+00_JPRB ,&
&  0.61375E+00_JPRB, 0.63004E+00_JPRB, 0.66145E+00_JPRB, 0.72138E+00_JPRB, 0.82617E+00_JPRB, 0.10033E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.16525E+01_JPRB, 0.11937E+01_JPRB, 0.60250E+00_JPRB, 0.13647E+01_JPRB, 0.17229E+01_JPRB, 0.17140E+01_JPRB, 0.14680E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.11408E+01_JPRB, 0.10414E+01_JPRB, 0.11038E+01_JPRB, 0.97705E+00_JPRB, 0.92556E+00_JPRB, 0.91022E+00_JPRB ,&
&  0.92960E+00_JPRB, 0.97380E+00_JPRB, 0.10203E+01_JPRB, 0.10567E+01_JPRB, 0.10702E+01_JPRB, 0.10670E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.11521E+01_JPRB, 0.12590E+01_JPRB, 0.14591E+01_JPRB, 0.17643E+01_JPRB, 0.24325E+01_JPRB, 0.22546E+01_JPRB, 0.21559E+01_JPRB ,&
&  0.14120E+01_JPRB, 0.12270E+01_JPRB, 0.11537E+01_JPRB, 0.10700E+01_JPRB, 0.92075E+00_JPRB, 0.79943E+00_JPRB, 0.80909E+00_JPRB ,&
&  0.73175E+00_JPRB, 0.68541E+00_JPRB, 0.67372E+00_JPRB, 0.65534E+00_JPRB, 0.64069E+00_JPRB, 0.64522E+00_JPRB, 0.66493E+00_JPRB ,&
&  0.69367E+00_JPRB, 0.70251E+00_JPRB, 0.70466E+00_JPRB, 0.70450E+00_JPRB, 0.71039E+00_JPRB, 0.71998E+00_JPRB, 0.72772E+00_JPRB ,&
&  0.74910E+00_JPRB, 0.78382E+00_JPRB, 0.82157E+00_JPRB, 0.83691E+00_JPRB, 0.83842E+00_JPRB, 0.86055E+00_JPRB, 0.91718E+00_JPRB ,&
&  0.90518E+00_JPRB, 0.91545E+00_JPRB, 0.83260E+00_JPRB, 0.80820E+00_JPRB, 0.75729E+00_JPRB, 0.74011E+00_JPRB, 0.72890E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.64863E+00_JPRB, 0.58017E+00_JPRB, 0.50965E+00_JPRB, 0.50045E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.48287E+00_JPRB, 0.47403E+00_JPRB, 0.47145E+00_JPRB, 0.48801E+00_JPRB, 0.48493E+00_JPRB, 0.49691E+00_JPRB, 0.51369E+00_JPRB ,&
&  0.55867E+00_JPRB, 0.60015E+00_JPRB, 0.63544E+00_JPRB, 0.62978E+00_JPRB, 0.62873E+00_JPRB, 0.61685E+00_JPRB, 0.62270E+00_JPRB ,&
&  0.63565E+00_JPRB, 0.65681E+00_JPRB, 0.67989E+00_JPRB, 0.69088E+00_JPRB, 0.70020E+00_JPRB, 0.71165E+00_JPRB, 0.72771E+00_JPRB ,&
&  0.75236E+00_JPRB, 0.78817E+00_JPRB, 0.85367E+00_JPRB, 0.97275E+00_JPRB, 0.11613E+01_JPRB, 0.13517E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.47349E+00_JPRB, 0.12886E+01_JPRB, 0.16842E+01_JPRB, 0.16370E+01_JPRB, 0.15465E+01_JPRB, 0.14531E+01_JPRB ,&
&  0.12350E+01_JPRB, 0.11283E+01_JPRB, 0.10338E+01_JPRB, 0.11315E+01_JPRB, 0.10369E+01_JPRB, 0.10252E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.10872E+01_JPRB, 0.11261E+01_JPRB, 0.11633E+01_JPRB, 0.11752E+01_JPRB, 0.11734E+01_JPRB, 0.11923E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.15300E+01_JPRB, 0.17489E+01_JPRB, 0.18093E+01_JPRB, 0.20681E+01_JPRB, 0.19758E+01_JPRB, 0.20773E+01_JPRB ,&
&  0.14361E+01_JPRB, 0.10232E+01_JPRB, 0.96687E+00_JPRB, 0.99025E+00_JPRB, 0.87917E+00_JPRB, 0.73767E+00_JPRB, 0.73788E+00_JPRB ,&
&  0.68289E+00_JPRB, 0.65751E+00_JPRB, 0.65741E+00_JPRB, 0.64475E+00_JPRB, 0.63144E+00_JPRB, 0.64975E+00_JPRB, 0.67744E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.72219E+00_JPRB, 0.71635E+00_JPRB, 0.71481E+00_JPRB, 0.71939E+00_JPRB, 0.71976E+00_JPRB, 0.72074E+00_JPRB ,&
&  0.74580E+00_JPRB, 0.77488E+00_JPRB, 0.81018E+00_JPRB, 0.84797E+00_JPRB, 0.89320E+00_JPRB, 0.89608E+00_JPRB, 0.88765E+00_JPRB ,&
&  0.86521E+00_JPRB, 0.84729E+00_JPRB, 0.79056E+00_JPRB, 0.77475E+00_JPRB, 0.74874E+00_JPRB, 0.71584E+00_JPRB, 0.68686E+00_JPRB ,&
&  0.63703E+00_JPRB, 0.55778E+00_JPRB, 0.51093E+00_JPRB, 0.50145E+00_JPRB, 0.48946E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.50731E+00_JPRB, 0.49929E+00_JPRB, 0.50385E+00_JPRB, 0.51246E+00_JPRB, 0.52727E+00_JPRB, 0.55267E+00_JPRB, 0.59281E+00_JPRB ,&
&  0.63928E+00_JPRB, 0.66474E+00_JPRB, 0.68630E+00_JPRB, 0.67652E+00_JPRB, 0.67868E+00_JPRB, 0.69477E+00_JPRB, 0.70519E+00_JPRB ,&
&  0.73434E+00_JPRB, 0.76528E+00_JPRB, 0.78347E+00_JPRB, 0.78915E+00_JPRB, 0.80073E+00_JPRB, 0.83081E+00_JPRB, 0.87294E+00_JPRB ,&
&  0.92569E+00_JPRB, 0.99837E+00_JPRB, 0.10980E+01_JPRB, 0.12349E+01_JPRB, 0.13882E+01_JPRB, 0.14461E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.74571E+00_JPRB, 0.12808E+01_JPRB, 0.16877E+01_JPRB, 0.16536E+01_JPRB, 0.15596E+01_JPRB, 0.14786E+01_JPRB, 0.14088E+01_JPRB ,&
&  0.12437E+01_JPRB, 0.11073E+01_JPRB, 0.10852E+01_JPRB, 0.11903E+01_JPRB, 0.11190E+01_JPRB, 0.11362E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.12084E+01_JPRB, 0.12427E+01_JPRB, 0.12588E+01_JPRB, 0.12681E+01_JPRB, 0.13012E+01_JPRB, 0.13629E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.15514E+01_JPRB, 0.16945E+01_JPRB, 0.18025E+01_JPRB, 0.16971E+01_JPRB, 0.16796E+01_JPRB, 0.18049E+01_JPRB, 0.21699E+01_JPRB ,&
&  0.14196E+01_JPRB, 0.91291E+00_JPRB, 0.90229E+00_JPRB, 0.92423E+00_JPRB, 0.81772E+00_JPRB, 0.73883E+00_JPRB, 0.69172E+00_JPRB ,&
&  0.65434E+00_JPRB, 0.62422E+00_JPRB, 0.61127E+00_JPRB, 0.60353E+00_JPRB, 0.61820E+00_JPRB, 0.63781E+00_JPRB, 0.66835E+00_JPRB ,&
&  0.68213E+00_JPRB, 0.68414E+00_JPRB, 0.67555E+00_JPRB, 0.67188E+00_JPRB, 0.67835E+00_JPRB, 0.68851E+00_JPRB, 0.68945E+00_JPRB ,&
&  0.71047E+00_JPRB, 0.71757E+00_JPRB, 0.74204E+00_JPRB, 0.78841E+00_JPRB, 0.84833E+00_JPRB, 0.87154E+00_JPRB, 0.86699E+00_JPRB ,&
&  0.80978E+00_JPRB, 0.80169E+00_JPRB, 0.79609E+00_JPRB, 0.77659E+00_JPRB, 0.75392E+00_JPRB, 0.70337E+00_JPRB, 0.61152E+00_JPRB ,&
&  0.56968E+00_JPRB, 0.52943E+00_JPRB, 0.51320E+00_JPRB, 0.50702E+00_JPRB, 0.51856E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.57772E+00_JPRB, 0.56576E+00_JPRB, 0.57356E+00_JPRB, 0.59120E+00_JPRB, 0.60597E+00_JPRB, 0.63524E+00_JPRB, 0.67177E+00_JPRB ,&
&  0.71845E+00_JPRB, 0.74890E+00_JPRB, 0.76029E+00_JPRB, 0.76163E+00_JPRB, 0.77064E+00_JPRB, 0.77395E+00_JPRB, 0.78842E+00_JPRB ,&
&  0.81533E+00_JPRB, 0.84349E+00_JPRB, 0.87972E+00_JPRB, 0.91004E+00_JPRB, 0.93415E+00_JPRB, 0.98836E+00_JPRB, 0.10490E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.12064E+01_JPRB, 0.13056E+01_JPRB, 0.14136E+01_JPRB, 0.15279E+01_JPRB, 0.15779E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.17297E+01_JPRB, 0.17495E+01_JPRB, 0.15768E+01_JPRB, 0.14226E+01_JPRB, 0.13704E+01_JPRB, 0.13171E+01_JPRB ,&
&  0.12660E+01_JPRB, 0.11666E+01_JPRB, 0.11432E+01_JPRB, 0.11578E+01_JPRB, 0.12023E+01_JPRB, 0.12134E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.13078E+01_JPRB, 0.13418E+01_JPRB, 0.13649E+01_JPRB, 0.13879E+01_JPRB, 0.14278E+01_JPRB, 0.14919E+01_JPRB, 0.15809E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.17229E+01_JPRB, 0.17826E+01_JPRB, 0.16602E+01_JPRB, 0.15550E+01_JPRB, 0.16906E+01_JPRB, 0.16306E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.81852E+00_JPRB, 0.83570E+00_JPRB, 0.86199E+00_JPRB, 0.77964E+00_JPRB, 0.70758E+00_JPRB, 0.65587E+00_JPRB ,&
&  0.61250E+00_JPRB, 0.57524E+00_JPRB, 0.57976E+00_JPRB, 0.58609E+00_JPRB, 0.59257E+00_JPRB, 0.59655E+00_JPRB, 0.61327E+00_JPRB ,&
&  0.63161E+00_JPRB, 0.62435E+00_JPRB, 0.61460E+00_JPRB, 0.60894E+00_JPRB, 0.62248E+00_JPRB, 0.64501E+00_JPRB, 0.67036E+00_JPRB ,&
&  0.72536E+00_JPRB, 0.68536E+00_JPRB, 0.69962E+00_JPRB, 0.74443E+00_JPRB, 0.79097E+00_JPRB, 0.84627E+00_JPRB, 0.89580E+00_JPRB ,&
&  0.93025E+00_JPRB, 0.96606E+00_JPRB, 0.10094E+01_JPRB, 0.10143E+01_JPRB, 0.94102E+00_JPRB, 0.85715E+00_JPRB, 0.61981E+00_JPRB ,&
&  0.60802E+00_JPRB, 0.59670E+00_JPRB, 0.60031E+00_JPRB, 0.59957E+00_JPRB, 0.59796E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.70151E+00_JPRB, 0.69980E+00_JPRB, 0.72488E+00_JPRB, 0.75072E+00_JPRB, 0.77372E+00_JPRB, 0.79583E+00_JPRB, 0.81211E+00_JPRB ,&
&  0.83538E+00_JPRB, 0.85419E+00_JPRB, 0.85818E+00_JPRB, 0.85456E+00_JPRB, 0.86811E+00_JPRB, 0.87029E+00_JPRB, 0.89393E+00_JPRB ,&
&  0.92932E+00_JPRB, 0.97073E+00_JPRB, 0.10245E+01_JPRB, 0.10586E+01_JPRB, 0.10905E+01_JPRB, 0.11418E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.13128E+01_JPRB, 0.13753E+01_JPRB, 0.14450E+01_JPRB, 0.15121E+01_JPRB, 0.15068E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.16420E+01_JPRB, 0.18681E+01_JPRB, 0.17574E+01_JPRB, 0.15070E+01_JPRB, 0.13634E+01_JPRB, 0.13074E+01_JPRB, 0.12975E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12266E+01_JPRB, 0.11681E+01_JPRB, 0.11532E+01_JPRB, 0.12888E+01_JPRB, 0.12690E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.14300E+01_JPRB, 0.14686E+01_JPRB, 0.14999E+01_JPRB, 0.15219E+01_JPRB, 0.15541E+01_JPRB, 0.15887E+01_JPRB ,&
&  0.16218E+01_JPRB, 0.16464E+01_JPRB, 0.17023E+01_JPRB, 0.15403E+01_JPRB, 0.14767E+01_JPRB, 0.15622E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.10393E+01_JPRB, 0.79242E+00_JPRB, 0.80073E+00_JPRB, 0.81325E+00_JPRB, 0.76283E+00_JPRB, 0.68799E+00_JPRB, 0.64735E+00_JPRB ,&
&  0.61452E+00_JPRB, 0.59186E+00_JPRB, 0.58625E+00_JPRB, 0.57398E+00_JPRB, 0.57469E+00_JPRB, 0.58764E+00_JPRB, 0.59236E+00_JPRB ,&
&  0.61553E+00_JPRB, 0.60403E+00_JPRB, 0.59938E+00_JPRB, 0.60225E+00_JPRB, 0.63429E+00_JPRB, 0.69812E+00_JPRB, 0.83192E+00_JPRB ,&
&  0.95618E+00_JPRB, 0.61886E+00_JPRB, 0.67678E+00_JPRB, 0.72203E+00_JPRB, 0.79525E+00_JPRB, 0.92160E+00_JPRB, 0.11193E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.19299E+01_JPRB, 0.18753E+01_JPRB, 0.14885E+01_JPRB, 0.18908E+01_JPRB, 0.15845E+01_JPRB, 0.70396E+00_JPRB ,&
&  0.70886E+00_JPRB, 0.69038E+00_JPRB, 0.69695E+00_JPRB, 0.70391E+00_JPRB, 0.70653E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.79785E+00_JPRB, 0.81762E+00_JPRB, 0.84023E+00_JPRB, 0.86063E+00_JPRB, 0.87346E+00_JPRB, 0.88523E+00_JPRB, 0.89976E+00_JPRB ,&
&  0.90748E+00_JPRB, 0.91005E+00_JPRB, 0.92266E+00_JPRB, 0.94270E+00_JPRB, 0.96753E+00_JPRB, 0.99412E+00_JPRB, 0.10262E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.11077E+01_JPRB, 0.11569E+01_JPRB, 0.11912E+01_JPRB, 0.12112E+01_JPRB, 0.12320E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.13057E+01_JPRB, 0.13277E+01_JPRB, 0.13780E+01_JPRB, 0.14163E+01_JPRB, 0.14643E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.18043E+01_JPRB, 0.16390E+01_JPRB, 0.14080E+01_JPRB, 0.12947E+01_JPRB, 0.12790E+01_JPRB, 0.12743E+01_JPRB ,&
&  0.12807E+01_JPRB, 0.12681E+01_JPRB, 0.11852E+01_JPRB, 0.12279E+01_JPRB, 0.12608E+01_JPRB, 0.12554E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.13641E+01_JPRB, 0.14103E+01_JPRB, 0.14624E+01_JPRB, 0.15117E+01_JPRB, 0.15441E+01_JPRB, 0.15849E+01_JPRB, 0.15750E+01_JPRB ,&
&  0.15639E+01_JPRB, 0.15590E+01_JPRB, 0.15549E+01_JPRB, 0.14081E+01_JPRB, 0.13496E+01_JPRB, 0.12789E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.89359E+00_JPRB, 0.78480E+00_JPRB, 0.75851E+00_JPRB, 0.83403E+00_JPRB, 0.81170E+00_JPRB, 0.74214E+00_JPRB, 0.68921E+00_JPRB ,&
&  0.64569E+00_JPRB, 0.61116E+00_JPRB, 0.62317E+00_JPRB, 0.60836E+00_JPRB, 0.55866E+00_JPRB, 0.57162E+00_JPRB, 0.59063E+00_JPRB ,&
&  0.57899E+00_JPRB, 0.59246E+00_JPRB, 0.61262E+00_JPRB, 0.60539E+00_JPRB, 0.60915E+00_JPRB, 0.63448E+00_JPRB, 0.63205E+00_JPRB ,&
&  0.69597E+00_JPRB, 0.64884E+00_JPRB, 0.66764E+00_JPRB, 0.72356E+00_JPRB, 0.72048E+00_JPRB, 0.93494E+00_JPRB, 0.12851E+01_JPRB ,&
&  0.18751E+01_JPRB, 0.20405E+01_JPRB, 0.26617E+01_JPRB, 0.41523E+01_JPRB, 0.31519E+01_JPRB, 0.20894E+01_JPRB, 0.79764E+00_JPRB ,&
&  0.79776E+00_JPRB, 0.78989E+00_JPRB, 0.80331E+00_JPRB, 0.79885E+00_JPRB, 0.80006E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.80762E+00_JPRB, 0.83329E+00_JPRB, 0.85436E+00_JPRB, 0.87546E+00_JPRB, 0.89473E+00_JPRB, 0.91306E+00_JPRB, 0.93307E+00_JPRB ,&
&  0.94703E+00_JPRB, 0.96753E+00_JPRB, 0.99222E+00_JPRB, 0.10171E+01_JPRB, 0.10382E+01_JPRB, 0.10671E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.11487E+01_JPRB, 0.11705E+01_JPRB, 0.11727E+01_JPRB, 0.11711E+01_JPRB, 0.11810E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11831E+01_JPRB, 0.11949E+01_JPRB, 0.12189E+01_JPRB, 0.12281E+01_JPRB, 0.12749E+01_JPRB, 0.10947E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.15882E+01_JPRB, 0.14777E+01_JPRB, 0.13011E+01_JPRB, 0.12080E+01_JPRB, 0.12011E+01_JPRB, 0.12075E+01_JPRB ,&
&  0.12245E+01_JPRB, 0.12641E+01_JPRB, 0.11794E+01_JPRB, 0.11353E+01_JPRB, 0.11232E+01_JPRB, 0.11306E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12402E+01_JPRB, 0.12853E+01_JPRB, 0.13261E+01_JPRB, 0.13549E+01_JPRB, 0.13654E+01_JPRB, 0.13427E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12458E+01_JPRB, 0.11361E+01_JPRB, 0.11077E+01_JPRB, 0.10531E+01_JPRB, 0.89224E+00_JPRB, 0.73031E+00_JPRB ,&
&  0.67124E+00_JPRB, 0.73059E+00_JPRB, 0.79188E+00_JPRB, 0.94162E+00_JPRB, 0.91604E+00_JPRB, 0.85115E+00_JPRB, 0.77765E+00_JPRB ,&
&  0.70952E+00_JPRB, 0.65710E+00_JPRB, 0.65044E+00_JPRB, 0.62027E+00_JPRB, 0.59841E+00_JPRB, 0.58712E+00_JPRB, 0.57522E+00_JPRB ,&
&  0.58326E+00_JPRB, 0.59157E+00_JPRB, 0.60386E+00_JPRB, 0.60616E+00_JPRB, 0.59300E+00_JPRB, 0.52812E+00_JPRB, 0.66656E+00_JPRB ,&
&  0.64342E+00_JPRB, 0.57972E+00_JPRB, 0.68560E+00_JPRB, 0.52796E+00_JPRB, 0.63837E+00_JPRB, 0.70290E+00_JPRB, 0.83340E+00_JPRB ,&
&  0.92320E+00_JPRB, 0.12954E+01_JPRB, 0.17858E+01_JPRB, 0.14214E+01_JPRB, 0.11377E+01_JPRB, 0.85676E+00_JPRB, 0.76507E+00_JPRB ,&
&  0.76186E+00_JPRB, 0.76395E+00_JPRB, 0.77187E+00_JPRB, 0.77407E+00_JPRB, 0.78467E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.81435E+00_JPRB, 0.83715E+00_JPRB, 0.85456E+00_JPRB, 0.87530E+00_JPRB, 0.89679E+00_JPRB, 0.91676E+00_JPRB, 0.93551E+00_JPRB ,&
&  0.95147E+00_JPRB, 0.96501E+00_JPRB, 0.97670E+00_JPRB, 0.99233E+00_JPRB, 0.10116E+01_JPRB, 0.10268E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.11019E+01_JPRB, 0.11280E+01_JPRB, 0.11245E+01_JPRB, 0.11233E+01_JPRB, 0.11423E+01_JPRB, 0.11612E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.11823E+01_JPRB, 0.12082E+01_JPRB, 0.12394E+01_JPRB, 0.12771E+01_JPRB, 0.13412E+01_JPRB, 0.12462E+01_JPRB ,&
&  0.13344E+01_JPRB, 0.14522E+01_JPRB, 0.13547E+01_JPRB, 0.11813E+01_JPRB, 0.11040E+01_JPRB, 0.10689E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.10227E+01_JPRB, 0.99650E+00_JPRB, 0.96610E+00_JPRB, 0.95476E+00_JPRB, 0.96082E+00_JPRB, 0.96496E+00_JPRB ,&
&  0.97648E+00_JPRB, 0.99070E+00_JPRB, 0.10199E+01_JPRB, 0.10586E+01_JPRB, 0.10972E+01_JPRB, 0.10763E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.97004E+00_JPRB, 0.90182E+00_JPRB, 0.72975E+00_JPRB, 0.67836E+00_JPRB, 0.63938E+00_JPRB, 0.56924E+00_JPRB, 0.50240E+00_JPRB ,&
&  0.55810E+00_JPRB, 0.63853E+00_JPRB, 0.80049E+00_JPRB, 0.10125E+01_JPRB, 0.97895E+00_JPRB, 0.92679E+00_JPRB, 0.84218E+00_JPRB ,&
&  0.75883E+00_JPRB, 0.70149E+00_JPRB, 0.67895E+00_JPRB, 0.64381E+00_JPRB, 0.63195E+00_JPRB, 0.63593E+00_JPRB, 0.59865E+00_JPRB ,&
&  0.58943E+00_JPRB, 0.57990E+00_JPRB, 0.57631E+00_JPRB, 0.56901E+00_JPRB, 0.58034E+00_JPRB, 0.50448E+00_JPRB, 0.61006E+00_JPRB ,&
&  0.61459E+00_JPRB, 0.47152E+00_JPRB, 0.52112E+00_JPRB, 0.60713E+00_JPRB, 0.59696E+00_JPRB, 0.60857E+00_JPRB, 0.69000E+00_JPRB ,&
&  0.84473E+00_JPRB, 0.95831E+00_JPRB, 0.97819E+00_JPRB, 0.92096E+00_JPRB, 0.82697E+00_JPRB, 0.77175E+00_JPRB, 0.74381E+00_JPRB ,&
&  0.75325E+00_JPRB, 0.76221E+00_JPRB, 0.77404E+00_JPRB, 0.77804E+00_JPRB, 0.79001E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.80097E+00_JPRB, 0.82529E+00_JPRB, 0.83976E+00_JPRB, 0.85484E+00_JPRB, 0.87414E+00_JPRB, 0.88457E+00_JPRB, 0.89316E+00_JPRB ,&
&  0.89648E+00_JPRB, 0.89393E+00_JPRB, 0.90138E+00_JPRB, 0.91483E+00_JPRB, 0.92139E+00_JPRB, 0.91897E+00_JPRB, 0.92512E+00_JPRB ,&
&  0.93890E+00_JPRB, 0.96122E+00_JPRB, 0.98477E+00_JPRB, 0.97816E+00_JPRB, 0.97382E+00_JPRB, 0.98612E+00_JPRB, 0.10049E+01_JPRB ,&
&  0.10060E+01_JPRB, 0.10031E+01_JPRB, 0.10131E+01_JPRB, 0.10257E+01_JPRB, 0.10471E+01_JPRB, 0.10524E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.13754E+01_JPRB, 0.12647E+01_JPRB, 0.10724E+01_JPRB, 0.10419E+01_JPRB, 0.95435E+00_JPRB, 0.92644E+00_JPRB ,&
&  0.90836E+00_JPRB, 0.87608E+00_JPRB, 0.84483E+00_JPRB, 0.82052E+00_JPRB, 0.81695E+00_JPRB, 0.82233E+00_JPRB, 0.80094E+00_JPRB ,&
&  0.79966E+00_JPRB, 0.81030E+00_JPRB, 0.83635E+00_JPRB, 0.86820E+00_JPRB, 0.88699E+00_JPRB, 0.87462E+00_JPRB, 0.82180E+00_JPRB ,&
&  0.78317E+00_JPRB, 0.71780E+00_JPRB, 0.55779E+00_JPRB, 0.53422E+00_JPRB, 0.52685E+00_JPRB, 0.46723E+00_JPRB, 0.44169E+00_JPRB ,&
&  0.54769E+00_JPRB, 0.65974E+00_JPRB, 0.90577E+00_JPRB, 0.10490E+01_JPRB, 0.10516E+01_JPRB, 0.10759E+01_JPRB, 0.94878E+00_JPRB ,&
&  0.83835E+00_JPRB, 0.76460E+00_JPRB, 0.74944E+00_JPRB, 0.70773E+00_JPRB, 0.69360E+00_JPRB, 0.67562E+00_JPRB, 0.63581E+00_JPRB ,&
&  0.60749E+00_JPRB, 0.58408E+00_JPRB, 0.56851E+00_JPRB, 0.56408E+00_JPRB, 0.48077E+00_JPRB, 0.47683E+00_JPRB, 0.59303E+00_JPRB ,&
&  0.56596E+00_JPRB, 0.47791E+00_JPRB, 0.44365E+00_JPRB, 0.57363E+00_JPRB, 0.55220E+00_JPRB, 0.54883E+00_JPRB, 0.61107E+00_JPRB ,&
&  0.72625E+00_JPRB, 0.76896E+00_JPRB, 0.77233E+00_JPRB, 0.76103E+00_JPRB, 0.73713E+00_JPRB, 0.71540E+00_JPRB, 0.70651E+00_JPRB ,&
&  0.72155E+00_JPRB, 0.73621E+00_JPRB, 0.75311E+00_JPRB, 0.76248E+00_JPRB, 0.77695E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.77852E+00_JPRB, 0.79395E+00_JPRB, 0.79491E+00_JPRB, 0.81091E+00_JPRB, 0.83131E+00_JPRB, 0.83771E+00_JPRB, 0.84108E+00_JPRB ,&
&  0.83235E+00_JPRB, 0.83512E+00_JPRB, 0.83058E+00_JPRB, 0.84663E+00_JPRB, 0.84726E+00_JPRB, 0.84853E+00_JPRB, 0.85097E+00_JPRB ,&
&  0.86438E+00_JPRB, 0.87387E+00_JPRB, 0.90217E+00_JPRB, 0.90732E+00_JPRB, 0.90589E+00_JPRB, 0.91252E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.95682E+00_JPRB, 0.98535E+00_JPRB, 0.10245E+01_JPRB, 0.10589E+01_JPRB, 0.11707E+01_JPRB, 0.13337E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.13852E+01_JPRB, 0.12765E+01_JPRB, 0.11477E+01_JPRB, 0.97413E+00_JPRB, 0.94507E+00_JPRB, 0.93189E+00_JPRB, 0.87126E+00_JPRB ,&
&  0.81752E+00_JPRB, 0.78711E+00_JPRB, 0.76250E+00_JPRB, 0.75279E+00_JPRB, 0.75542E+00_JPRB, 0.74429E+00_JPRB, 0.73989E+00_JPRB ,&
&  0.71611E+00_JPRB, 0.71509E+00_JPRB, 0.81670E+00_JPRB, 0.92392E+00_JPRB, 0.98637E+00_JPRB, 0.10657E+01_JPRB, 0.97866E+00_JPRB ,&
&  0.95395E+00_JPRB, 0.93460E+00_JPRB, 0.80129E+00_JPRB, 0.75017E+00_JPRB, 0.70471E+00_JPRB, 0.66447E+00_JPRB, 0.64620E+00_JPRB ,&
&  0.71813E+00_JPRB, 0.83806E+00_JPRB, 0.88343E+00_JPRB, 0.10834E+01_JPRB, 0.11760E+01_JPRB, 0.12107E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.90586E+00_JPRB, 0.82932E+00_JPRB, 0.81405E+00_JPRB, 0.78550E+00_JPRB, 0.77508E+00_JPRB, 0.72820E+00_JPRB, 0.66890E+00_JPRB ,&
&  0.64830E+00_JPRB, 0.61071E+00_JPRB, 0.58797E+00_JPRB, 0.54710E+00_JPRB, 0.57587E+00_JPRB, 0.47059E+00_JPRB, 0.57753E+00_JPRB ,&
&  0.54064E+00_JPRB, 0.53712E+00_JPRB, 0.49960E+00_JPRB, 0.67086E+00_JPRB, 0.55118E+00_JPRB, 0.49897E+00_JPRB, 0.53413E+00_JPRB ,&
&  0.60517E+00_JPRB, 0.64298E+00_JPRB, 0.65821E+00_JPRB, 0.68517E+00_JPRB, 0.69259E+00_JPRB, 0.68581E+00_JPRB, 0.69380E+00_JPRB ,&
&  0.72998E+00_JPRB, 0.73590E+00_JPRB, 0.74190E+00_JPRB, 0.74278E+00_JPRB, 0.75996E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.72224E+00_JPRB, 0.73511E+00_JPRB, 0.72933E+00_JPRB, 0.73650E+00_JPRB, 0.74051E+00_JPRB, 0.75506E+00_JPRB, 0.75467E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.75742E+00_JPRB, 0.73768E+00_JPRB, 0.74164E+00_JPRB, 0.73670E+00_JPRB, 0.73980E+00_JPRB, 0.72894E+00_JPRB ,&
&  0.73133E+00_JPRB, 0.74509E+00_JPRB, 0.75059E+00_JPRB, 0.75787E+00_JPRB, 0.75504E+00_JPRB, 0.73802E+00_JPRB, 0.71989E+00_JPRB ,&
&  0.76028E+00_JPRB, 0.86339E+00_JPRB, 0.98399E+00_JPRB, 0.10706E+01_JPRB, 0.12709E+01_JPRB, 0.12813E+01_JPRB, 0.14054E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.12642E+01_JPRB, 0.12357E+01_JPRB, 0.10537E+01_JPRB, 0.10047E+01_JPRB, 0.95937E+00_JPRB, 0.93192E+00_JPRB ,&
&  0.94318E+00_JPRB, 0.97794E+00_JPRB, 0.10235E+01_JPRB, 0.10758E+01_JPRB, 0.11201E+01_JPRB, 0.11235E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.11445E+01_JPRB, 0.13107E+01_JPRB, 0.13739E+01_JPRB, 0.14690E+01_JPRB, 0.15223E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.14849E+01_JPRB, 0.14564E+01_JPRB, 0.12625E+01_JPRB, 0.11352E+01_JPRB, 0.11201E+01_JPRB, 0.10444E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.11446E+01_JPRB, 0.97892E+00_JPRB, 0.10956E+01_JPRB, 0.12041E+01_JPRB, 0.11618E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.10535E+01_JPRB, 0.96480E+00_JPRB, 0.93060E+00_JPRB, 0.91118E+00_JPRB, 0.87414E+00_JPRB, 0.81299E+00_JPRB, 0.78180E+00_JPRB ,&
&  0.69701E+00_JPRB, 0.63024E+00_JPRB, 0.58518E+00_JPRB, 0.56246E+00_JPRB, 0.59348E+00_JPRB, 0.67390E+00_JPRB, 0.66554E+00_JPRB ,&
&  0.64174E+00_JPRB, 0.59820E+00_JPRB, 0.68132E+00_JPRB, 0.81503E+00_JPRB, 0.51405E+00_JPRB, 0.48864E+00_JPRB, 0.47598E+00_JPRB ,&
&  0.49822E+00_JPRB, 0.53060E+00_JPRB, 0.55547E+00_JPRB, 0.58774E+00_JPRB, 0.60416E+00_JPRB, 0.60621E+00_JPRB, 0.61610E+00_JPRB ,&
&  0.64247E+00_JPRB, 0.66773E+00_JPRB, 0.70337E+00_JPRB, 0.70772E+00_JPRB, 0.70652E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.62757E+00_JPRB, 0.62841E+00_JPRB, 0.63162E+00_JPRB, 0.62748E+00_JPRB, 0.62328E+00_JPRB, 0.62276E+00_JPRB, 0.62523E+00_JPRB ,&
&  0.62616E+00_JPRB, 0.61733E+00_JPRB, 0.60812E+00_JPRB, 0.60120E+00_JPRB, 0.58909E+00_JPRB, 0.56161E+00_JPRB, 0.52259E+00_JPRB ,&
&  0.52777E+00_JPRB, 0.53827E+00_JPRB, 0.55298E+00_JPRB, 0.59109E+00_JPRB, 0.67679E+00_JPRB, 0.80554E+00_JPRB, 0.97582E+00_JPRB ,&
&  0.13023E+01_JPRB, 0.16649E+01_JPRB, 0.19231E+01_JPRB, 0.20050E+01_JPRB, 0.22187E+01_JPRB, 0.16650E+01_JPRB, 0.15993E+01_JPRB ,&
&  0.17700E+01_JPRB, 0.14305E+01_JPRB, 0.12483E+01_JPRB, 0.11374E+01_JPRB, 0.10813E+01_JPRB, 0.10889E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.12647E+01_JPRB, 0.13741E+01_JPRB, 0.14671E+01_JPRB, 0.15563E+01_JPRB, 0.16290E+01_JPRB, 0.16994E+01_JPRB, 0.17328E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.17264E+01_JPRB, 0.17609E+01_JPRB, 0.17225E+01_JPRB, 0.18326E+01_JPRB, 0.18874E+01_JPRB, 0.18933E+01_JPRB ,&
&  0.18364E+01_JPRB, 0.17465E+01_JPRB, 0.17068E+01_JPRB, 0.16087E+01_JPRB, 0.15361E+01_JPRB, 0.14791E+01_JPRB, 0.14182E+01_JPRB ,&
&  0.13755E+01_JPRB, 0.13309E+01_JPRB, 0.11395E+01_JPRB, 0.13033E+01_JPRB, 0.13486E+01_JPRB, 0.11520E+01_JPRB, 0.12805E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.11311E+01_JPRB, 0.10746E+01_JPRB, 0.10283E+01_JPRB, 0.99034E+00_JPRB, 0.10627E+01_JPRB, 0.94921E+00_JPRB ,&
&  0.81384E+00_JPRB, 0.67059E+00_JPRB, 0.58637E+00_JPRB, 0.59404E+00_JPRB, 0.65503E+00_JPRB, 0.74615E+00_JPRB, 0.82232E+00_JPRB ,&
&  0.72738E+00_JPRB, 0.67125E+00_JPRB, 0.70506E+00_JPRB, 0.79674E+00_JPRB, 0.60922E+00_JPRB, 0.46556E+00_JPRB, 0.45574E+00_JPRB ,&
&  0.45566E+00_JPRB, 0.46053E+00_JPRB, 0.46896E+00_JPRB, 0.49250E+00_JPRB, 0.51434E+00_JPRB, 0.53292E+00_JPRB, 0.54774E+00_JPRB ,&
&  0.55621E+00_JPRB, 0.57625E+00_JPRB, 0.59512E+00_JPRB, 0.61169E+00_JPRB, 0.62567E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.54652E+00_JPRB, 0.54377E+00_JPRB, 0.55092E+00_JPRB, 0.55007E+00_JPRB, 0.56232E+00_JPRB, 0.56295E+00_JPRB, 0.56718E+00_JPRB ,&
&  0.58035E+00_JPRB, 0.59998E+00_JPRB, 0.59353E+00_JPRB, 0.57355E+00_JPRB, 0.56811E+00_JPRB, 0.57646E+00_JPRB, 0.61236E+00_JPRB ,&
&  0.69732E+00_JPRB, 0.78591E+00_JPRB, 0.86187E+00_JPRB, 0.97564E+00_JPRB, 0.11875E+01_JPRB, 0.15416E+01_JPRB, 0.18854E+01_JPRB ,&
&  0.21359E+01_JPRB, 0.23086E+01_JPRB, 0.23071E+01_JPRB, 0.24048E+01_JPRB, 0.19130E+01_JPRB, 0.13507E+01_JPRB, 0.15528E+01_JPRB ,&
&  0.16634E+01_JPRB, 0.15562E+01_JPRB, 0.12821E+01_JPRB, 0.12190E+01_JPRB, 0.11916E+01_JPRB, 0.12420E+01_JPRB, 0.13500E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.15874E+01_JPRB, 0.16856E+01_JPRB, 0.17593E+01_JPRB, 0.18047E+01_JPRB, 0.18586E+01_JPRB, 0.19012E+01_JPRB ,&
&  0.19596E+01_JPRB, 0.19730E+01_JPRB, 0.19748E+01_JPRB, 0.19971E+01_JPRB, 0.20432E+01_JPRB, 0.21161E+01_JPRB, 0.21488E+01_JPRB ,&
&  0.21467E+01_JPRB, 0.20268E+01_JPRB, 0.19852E+01_JPRB, 0.19378E+01_JPRB, 0.17927E+01_JPRB, 0.16399E+01_JPRB, 0.15988E+01_JPRB ,&
&  0.15686E+01_JPRB, 0.15710E+01_JPRB, 0.13089E+01_JPRB, 0.15939E+01_JPRB, 0.17500E+01_JPRB, 0.14872E+01_JPRB, 0.13950E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.12066E+01_JPRB, 0.11130E+01_JPRB, 0.11072E+01_JPRB, 0.11530E+01_JPRB, 0.13236E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.78153E+00_JPRB, 0.74093E+00_JPRB, 0.79343E+00_JPRB, 0.85768E+00_JPRB, 0.83415E+00_JPRB, 0.76252E+00_JPRB ,&
&  0.69297E+00_JPRB, 0.78058E+00_JPRB, 0.91497E+00_JPRB, 0.13073E+01_JPRB, 0.16824E+01_JPRB, 0.49478E+00_JPRB, 0.46359E+00_JPRB ,&
&  0.47203E+00_JPRB, 0.47437E+00_JPRB, 0.47277E+00_JPRB, 0.48605E+00_JPRB, 0.49418E+00_JPRB, 0.50346E+00_JPRB, 0.52077E+00_JPRB ,&
&  0.53173E+00_JPRB, 0.53675E+00_JPRB, 0.54129E+00_JPRB, 0.53492E+00_JPRB, 0.54280E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.52758E+00_JPRB, 0.53403E+00_JPRB, 0.55701E+00_JPRB, 0.60387E+00_JPRB, 0.67302E+00_JPRB, 0.70996E+00_JPRB, 0.70995E+00_JPRB ,&
&  0.68553E+00_JPRB, 0.69319E+00_JPRB, 0.72445E+00_JPRB, 0.73460E+00_JPRB, 0.78314E+00_JPRB, 0.87824E+00_JPRB, 0.97507E+00_JPRB ,&
&  0.11063E+01_JPRB, 0.12259E+01_JPRB, 0.13336E+01_JPRB, 0.14990E+01_JPRB, 0.16609E+01_JPRB, 0.18580E+01_JPRB, 0.19864E+01_JPRB ,&
&  0.19736E+01_JPRB, 0.17486E+01_JPRB, 0.16467E+01_JPRB, 0.12794E+01_JPRB, 0.10907E+01_JPRB, 0.12707E+01_JPRB, 0.12627E+01_JPRB ,&
&  0.12573E+01_JPRB, 0.12676E+01_JPRB, 0.12517E+01_JPRB, 0.12787E+01_JPRB, 0.13368E+01_JPRB, 0.14044E+01_JPRB, 0.14948E+01_JPRB ,&
&  0.16070E+01_JPRB, 0.17140E+01_JPRB, 0.18074E+01_JPRB, 0.18846E+01_JPRB, 0.19161E+01_JPRB, 0.19657E+01_JPRB, 0.20518E+01_JPRB ,&
&  0.21578E+01_JPRB, 0.21854E+01_JPRB, 0.22095E+01_JPRB, 0.22650E+01_JPRB, 0.22224E+01_JPRB, 0.22486E+01_JPRB, 0.22463E+01_JPRB ,&
&  0.22507E+01_JPRB, 0.22078E+01_JPRB, 0.21799E+01_JPRB, 0.21856E+01_JPRB, 0.21062E+01_JPRB, 0.18277E+01_JPRB, 0.17591E+01_JPRB ,&
&  0.17792E+01_JPRB, 0.17211E+01_JPRB, 0.16155E+01_JPRB, 0.18653E+01_JPRB, 0.12958E+01_JPRB, 0.14149E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.14135E+01_JPRB, 0.12897E+01_JPRB, 0.12541E+01_JPRB, 0.12330E+01_JPRB, 0.10901E+01_JPRB, 0.10965E+01_JPRB ,&
&  0.11615E+01_JPRB, 0.10441E+01_JPRB, 0.10105E+01_JPRB, 0.10720E+01_JPRB, 0.11532E+01_JPRB, 0.10771E+01_JPRB, 0.10081E+01_JPRB ,&
&  0.89862E+00_JPRB, 0.86961E+00_JPRB, 0.81477E+00_JPRB, 0.91288E+00_JPRB, 0.84716E+00_JPRB, 0.48162E+00_JPRB, 0.48548E+00_JPRB ,&
&  0.50075E+00_JPRB, 0.51482E+00_JPRB, 0.51696E+00_JPRB, 0.54385E+00_JPRB, 0.55617E+00_JPRB, 0.55049E+00_JPRB, 0.55576E+00_JPRB ,&
&  0.56484E+00_JPRB, 0.56191E+00_JPRB, 0.55935E+00_JPRB, 0.55222E+00_JPRB, 0.54453E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.61514E+00_JPRB, 0.60077E+00_JPRB, 0.61301E+00_JPRB, 0.70494E+00_JPRB, 0.80353E+00_JPRB, 0.86206E+00_JPRB, 0.84013E+00_JPRB ,&
&  0.79633E+00_JPRB, 0.75741E+00_JPRB, 0.80465E+00_JPRB, 0.86260E+00_JPRB, 0.95028E+00_JPRB, 0.10619E+01_JPRB, 0.11833E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.14322E+01_JPRB, 0.15109E+01_JPRB, 0.16006E+01_JPRB, 0.17186E+01_JPRB, 0.18421E+01_JPRB, 0.19114E+01_JPRB ,&
&  0.15684E+01_JPRB, 0.11899E+01_JPRB, 0.14270E+01_JPRB, 0.13851E+01_JPRB, 0.13407E+01_JPRB, 0.13202E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.13153E+01_JPRB, 0.13316E+01_JPRB, 0.13799E+01_JPRB, 0.14183E+01_JPRB, 0.14056E+01_JPRB, 0.14767E+01_JPRB, 0.15689E+01_JPRB ,&
&  0.16805E+01_JPRB, 0.17703E+01_JPRB, 0.18280E+01_JPRB, 0.18865E+01_JPRB, 0.19257E+01_JPRB, 0.19778E+01_JPRB, 0.20664E+01_JPRB ,&
&  0.21596E+01_JPRB, 0.22343E+01_JPRB, 0.22544E+01_JPRB, 0.22090E+01_JPRB, 0.21917E+01_JPRB, 0.22161E+01_JPRB, 0.21778E+01_JPRB ,&
&  0.21657E+01_JPRB, 0.21392E+01_JPRB, 0.22487E+01_JPRB, 0.23784E+01_JPRB, 0.24104E+01_JPRB, 0.20960E+01_JPRB, 0.21031E+01_JPRB ,&
&  0.20272E+01_JPRB, 0.19320E+01_JPRB, 0.18773E+01_JPRB, 0.20389E+01_JPRB, 0.14912E+01_JPRB, 0.15624E+01_JPRB, 0.15358E+01_JPRB ,&
&  0.16597E+01_JPRB, 0.16082E+01_JPRB, 0.15278E+01_JPRB, 0.14340E+01_JPRB, 0.14475E+01_JPRB, 0.12309E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.12441E+01_JPRB, 0.13703E+01_JPRB, 0.13058E+01_JPRB, 0.13206E+01_JPRB, 0.12638E+01_JPRB, 0.13415E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.10931E+01_JPRB, 0.92170E+00_JPRB, 0.77192E+00_JPRB, 0.58613E+00_JPRB, 0.50874E+00_JPRB, 0.52091E+00_JPRB ,&
&  0.54329E+00_JPRB, 0.56806E+00_JPRB, 0.57630E+00_JPRB, 0.60052E+00_JPRB, 0.62037E+00_JPRB, 0.61591E+00_JPRB, 0.60497E+00_JPRB ,&
&  0.60670E+00_JPRB, 0.62561E+00_JPRB, 0.62024E+00_JPRB, 0.62119E+00_JPRB, 0.62275E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.71222E+00_JPRB, 0.70374E+00_JPRB, 0.70935E+00_JPRB, 0.79191E+00_JPRB, 0.91912E+00_JPRB, 0.10245E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.87647E+00_JPRB, 0.92032E+00_JPRB, 0.95306E+00_JPRB, 0.10537E+01_JPRB, 0.11641E+01_JPRB, 0.12852E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.15214E+01_JPRB, 0.16123E+01_JPRB, 0.16989E+01_JPRB, 0.17985E+01_JPRB, 0.18948E+01_JPRB, 0.16507E+01_JPRB ,&
&  0.12661E+01_JPRB, 0.13562E+01_JPRB, 0.19623E+01_JPRB, 0.16818E+01_JPRB, 0.14799E+01_JPRB, 0.14640E+01_JPRB, 0.14623E+01_JPRB ,&
&  0.14813E+01_JPRB, 0.13962E+01_JPRB, 0.14215E+01_JPRB, 0.14613E+01_JPRB, 0.14473E+01_JPRB, 0.15199E+01_JPRB, 0.16265E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.17795E+01_JPRB, 0.18185E+01_JPRB, 0.18432E+01_JPRB, 0.18776E+01_JPRB, 0.19304E+01_JPRB, 0.20101E+01_JPRB ,&
&  0.20982E+01_JPRB, 0.21786E+01_JPRB, 0.22066E+01_JPRB, 0.21138E+01_JPRB, 0.21025E+01_JPRB, 0.21556E+01_JPRB, 0.21713E+01_JPRB ,&
&  0.21488E+01_JPRB, 0.22331E+01_JPRB, 0.22888E+01_JPRB, 0.22654E+01_JPRB, 0.21265E+01_JPRB, 0.23793E+01_JPRB, 0.22273E+01_JPRB ,&
&  0.22183E+01_JPRB, 0.20995E+01_JPRB, 0.22478E+01_JPRB, 0.17567E+01_JPRB, 0.17933E+01_JPRB, 0.18211E+01_JPRB, 0.17289E+01_JPRB ,&
&  0.16517E+01_JPRB, 0.16857E+01_JPRB, 0.16267E+01_JPRB, 0.15658E+01_JPRB, 0.15449E+01_JPRB, 0.13527E+01_JPRB, 0.13723E+01_JPRB ,&
&  0.13652E+01_JPRB, 0.15851E+01_JPRB, 0.14133E+01_JPRB, 0.13592E+01_JPRB, 0.12456E+01_JPRB, 0.13686E+01_JPRB, 0.15776E+01_JPRB ,&
&  0.17214E+01_JPRB, 0.15658E+01_JPRB, 0.13481E+01_JPRB, 0.10839E+01_JPRB, 0.83229E+00_JPRB, 0.66362E+00_JPRB, 0.58946E+00_JPRB ,&
&  0.58308E+00_JPRB, 0.59933E+00_JPRB, 0.62762E+00_JPRB, 0.64741E+00_JPRB, 0.67060E+00_JPRB, 0.69239E+00_JPRB, 0.70124E+00_JPRB ,&
&  0.70391E+00_JPRB, 0.69740E+00_JPRB, 0.68738E+00_JPRB, 0.71411E+00_JPRB, 0.72770E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.77216E+00_JPRB, 0.74673E+00_JPRB, 0.72494E+00_JPRB, 0.75185E+00_JPRB, 0.81969E+00_JPRB, 0.84722E+00_JPRB, 0.85429E+00_JPRB ,&
&  0.85387E+00_JPRB, 0.84666E+00_JPRB, 0.88872E+00_JPRB, 0.91104E+00_JPRB, 0.10119E+01_JPRB, 0.11239E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.13039E+01_JPRB, 0.13820E+01_JPRB, 0.14795E+01_JPRB, 0.16072E+01_JPRB, 0.16993E+01_JPRB, 0.17759E+01_JPRB, 0.13859E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.20298E+01_JPRB, 0.18637E+01_JPRB, 0.18585E+01_JPRB, 0.16827E+01_JPRB, 0.16807E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.13496E+01_JPRB, 0.13750E+01_JPRB, 0.14045E+01_JPRB, 0.14098E+01_JPRB, 0.14538E+01_JPRB, 0.15234E+01_JPRB ,&
&  0.15934E+01_JPRB, 0.16785E+01_JPRB, 0.17303E+01_JPRB, 0.17698E+01_JPRB, 0.18201E+01_JPRB, 0.18562E+01_JPRB, 0.19350E+01_JPRB ,&
&  0.20387E+01_JPRB, 0.21334E+01_JPRB, 0.21701E+01_JPRB, 0.21983E+01_JPRB, 0.21384E+01_JPRB, 0.22283E+01_JPRB, 0.23549E+01_JPRB ,&
&  0.22950E+01_JPRB, 0.22160E+01_JPRB, 0.24285E+01_JPRB, 0.24386E+01_JPRB, 0.23664E+01_JPRB, 0.25032E+01_JPRB, 0.22695E+01_JPRB ,&
&  0.23958E+01_JPRB, 0.21848E+01_JPRB, 0.24267E+01_JPRB, 0.19043E+01_JPRB, 0.19175E+01_JPRB, 0.20411E+01_JPRB, 0.18838E+01_JPRB ,&
&  0.17006E+01_JPRB, 0.17583E+01_JPRB, 0.16738E+01_JPRB, 0.16551E+01_JPRB, 0.16187E+01_JPRB, 0.14888E+01_JPRB, 0.13836E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.16120E+01_JPRB, 0.15946E+01_JPRB, 0.12899E+01_JPRB, 0.12617E+01_JPRB, 0.13465E+01_JPRB, 0.18681E+01_JPRB ,&
&  0.20995E+01_JPRB, 0.19518E+01_JPRB, 0.18378E+01_JPRB, 0.16495E+01_JPRB, 0.13330E+01_JPRB, 0.10914E+01_JPRB, 0.86682E+00_JPRB ,&
&  0.69269E+00_JPRB, 0.62863E+00_JPRB, 0.67291E+00_JPRB, 0.66565E+00_JPRB, 0.69126E+00_JPRB, 0.72322E+00_JPRB, 0.74276E+00_JPRB ,&
&  0.76267E+00_JPRB, 0.77272E+00_JPRB, 0.78085E+00_JPRB, 0.77625E+00_JPRB, 0.78034E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.77031E+00_JPRB, 0.74508E+00_JPRB, 0.72820E+00_JPRB, 0.73548E+00_JPRB, 0.80625E+00_JPRB, 0.83839E+00_JPRB, 0.82062E+00_JPRB ,&
&  0.80714E+00_JPRB, 0.80917E+00_JPRB, 0.84050E+00_JPRB, 0.82789E+00_JPRB, 0.87910E+00_JPRB, 0.99476E+00_JPRB, 0.10700E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.12213E+01_JPRB, 0.13403E+01_JPRB, 0.14997E+01_JPRB, 0.16624E+01_JPRB, 0.18564E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.12291E+01_JPRB, 0.17685E+01_JPRB, 0.18774E+01_JPRB, 0.19625E+01_JPRB, 0.18143E+01_JPRB, 0.17318E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.14614E+01_JPRB, 0.13448E+01_JPRB, 0.12625E+01_JPRB, 0.12376E+01_JPRB, 0.12277E+01_JPRB, 0.12917E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13121E+01_JPRB, 0.13560E+01_JPRB, 0.14637E+01_JPRB, 0.15648E+01_JPRB, 0.16599E+01_JPRB, 0.17440E+01_JPRB, 0.18422E+01_JPRB ,&
&  0.19725E+01_JPRB, 0.20980E+01_JPRB, 0.22745E+01_JPRB, 0.22473E+01_JPRB, 0.22920E+01_JPRB, 0.23841E+01_JPRB, 0.25714E+01_JPRB ,&
&  0.25211E+01_JPRB, 0.24487E+01_JPRB, 0.27814E+01_JPRB, 0.31125E+01_JPRB, 0.31160E+01_JPRB, 0.27929E+01_JPRB, 0.24653E+01_JPRB ,&
&  0.24277E+01_JPRB, 0.23097E+01_JPRB, 0.22868E+01_JPRB, 0.20755E+01_JPRB, 0.21428E+01_JPRB, 0.23375E+01_JPRB, 0.23363E+01_JPRB ,&
&  0.21347E+01_JPRB, 0.19567E+01_JPRB, 0.18270E+01_JPRB, 0.17646E+01_JPRB, 0.17168E+01_JPRB, 0.15553E+01_JPRB, 0.15279E+01_JPRB ,&
&  0.17776E+01_JPRB, 0.21114E+01_JPRB, 0.19382E+01_JPRB, 0.14747E+01_JPRB, 0.13193E+01_JPRB, 0.12193E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.23210E+01_JPRB, 0.22030E+01_JPRB, 0.20179E+01_JPRB, 0.21540E+01_JPRB, 0.18228E+01_JPRB, 0.16020E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.10287E+01_JPRB, 0.85876E+00_JPRB, 0.78580E+00_JPRB, 0.72478E+00_JPRB, 0.71955E+00_JPRB, 0.73258E+00_JPRB, 0.75851E+00_JPRB ,&
&  0.78415E+00_JPRB, 0.79151E+00_JPRB, 0.78476E+00_JPRB, 0.77604E+00_JPRB, 0.78274E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.79230E+00_JPRB, 0.76214E+00_JPRB, 0.75486E+00_JPRB, 0.74773E+00_JPRB, 0.79288E+00_JPRB, 0.82957E+00_JPRB, 0.78702E+00_JPRB ,&
&  0.78310E+00_JPRB, 0.77402E+00_JPRB, 0.78372E+00_JPRB, 0.77971E+00_JPRB, 0.79823E+00_JPRB, 0.85559E+00_JPRB, 0.90407E+00_JPRB ,&
&  0.96518E+00_JPRB, 0.10356E+01_JPRB, 0.11592E+01_JPRB, 0.13196E+01_JPRB, 0.14100E+01_JPRB, 0.13586E+01_JPRB, 0.14039E+01_JPRB ,&
&  0.16004E+01_JPRB, 0.18672E+01_JPRB, 0.19824E+01_JPRB, 0.22092E+01_JPRB, 0.22265E+01_JPRB, 0.23134E+01_JPRB, 0.19050E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.15651E+01_JPRB, 0.13686E+01_JPRB, 0.12446E+01_JPRB, 0.11412E+01_JPRB, 0.11294E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11582E+01_JPRB, 0.11788E+01_JPRB, 0.12397E+01_JPRB, 0.13110E+01_JPRB, 0.14275E+01_JPRB, 0.15437E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.18727E+01_JPRB, 0.20565E+01_JPRB, 0.22521E+01_JPRB, 0.23520E+01_JPRB, 0.23978E+01_JPRB, 0.25117E+01_JPRB, 0.27919E+01_JPRB ,&
&  0.29415E+01_JPRB, 0.30834E+01_JPRB, 0.31954E+01_JPRB, 0.34353E+01_JPRB, 0.35883E+01_JPRB, 0.33967E+01_JPRB, 0.29238E+01_JPRB ,&
&  0.26224E+01_JPRB, 0.23468E+01_JPRB, 0.20519E+01_JPRB, 0.21400E+01_JPRB, 0.23141E+01_JPRB, 0.27274E+01_JPRB, 0.25273E+01_JPRB ,&
&  0.20223E+01_JPRB, 0.17062E+01_JPRB, 0.16298E+01_JPRB, 0.17672E+01_JPRB, 0.19231E+01_JPRB, 0.18223E+01_JPRB, 0.18595E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.11016E+01_JPRB, 0.88664E+00_JPRB, 0.12128E+01_JPRB, 0.92260E+00_JPRB, 0.86208E+00_JPRB, 0.15422E+01_JPRB ,&
&  0.22973E+01_JPRB, 0.27018E+01_JPRB, 0.25003E+01_JPRB, 0.21609E+01_JPRB, 0.21431E+01_JPRB, 0.20396E+01_JPRB, 0.18040E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.10977E+01_JPRB, 0.96579E+00_JPRB, 0.86965E+00_JPRB, 0.79453E+00_JPRB, 0.77087E+00_JPRB, 0.77081E+00_JPRB ,&
&  0.75813E+00_JPRB, 0.76717E+00_JPRB, 0.77028E+00_JPRB, 0.76964E+00_JPRB, 0.78718E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.79091E+00_JPRB, 0.77835E+00_JPRB, 0.77448E+00_JPRB, 0.77558E+00_JPRB, 0.76025E+00_JPRB, 0.78701E+00_JPRB, 0.79983E+00_JPRB ,&
&  0.77946E+00_JPRB, 0.73903E+00_JPRB, 0.72213E+00_JPRB, 0.73281E+00_JPRB, 0.76006E+00_JPRB, 0.78458E+00_JPRB, 0.80684E+00_JPRB ,&
&  0.84112E+00_JPRB, 0.89293E+00_JPRB, 0.10163E+01_JPRB, 0.11476E+01_JPRB, 0.12079E+01_JPRB, 0.11662E+01_JPRB, 0.14692E+01_JPRB ,&
&  0.16758E+01_JPRB, 0.19427E+01_JPRB, 0.23494E+01_JPRB, 0.25171E+01_JPRB, 0.24741E+01_JPRB, 0.24482E+01_JPRB, 0.21645E+01_JPRB ,&
&  0.19350E+01_JPRB, 0.17561E+01_JPRB, 0.15742E+01_JPRB, 0.14209E+01_JPRB, 0.13217E+01_JPRB, 0.12139E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11155E+01_JPRB, 0.11470E+01_JPRB, 0.11812E+01_JPRB, 0.12810E+01_JPRB, 0.13864E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.17587E+01_JPRB, 0.19932E+01_JPRB, 0.22290E+01_JPRB, 0.25486E+01_JPRB, 0.21237E+01_JPRB, 0.25633E+01_JPRB, 0.27356E+01_JPRB ,&
&  0.29808E+01_JPRB, 0.31104E+01_JPRB, 0.30675E+01_JPRB, 0.38786E+01_JPRB, 0.42503E+01_JPRB, 0.38061E+01_JPRB, 0.32983E+01_JPRB ,&
&  0.29544E+01_JPRB, 0.25251E+01_JPRB, 0.22209E+01_JPRB, 0.23507E+01_JPRB, 0.24635E+01_JPRB, 0.27982E+01_JPRB, 0.20012E+01_JPRB ,&
&  0.18181E+01_JPRB, 0.21164E+01_JPRB, 0.19298E+01_JPRB, 0.15833E+01_JPRB, 0.18671E+01_JPRB, 0.21230E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.68565E+00_JPRB, 0.58560E+00_JPRB, 0.63171E+00_JPRB, 0.63812E+00_JPRB, 0.59483E+00_JPRB, 0.85673E+00_JPRB, 0.30148E+01_JPRB ,&
&  0.23533E+01_JPRB, 0.29886E+01_JPRB, 0.25501E+01_JPRB, 0.22772E+01_JPRB, 0.21278E+01_JPRB, 0.22004E+01_JPRB, 0.22405E+01_JPRB ,&
&  0.15412E+01_JPRB, 0.13003E+01_JPRB, 0.11015E+01_JPRB, 0.96682E+00_JPRB, 0.85976E+00_JPRB, 0.80308E+00_JPRB, 0.78225E+00_JPRB ,&
&  0.76394E+00_JPRB, 0.75474E+00_JPRB, 0.75348E+00_JPRB, 0.76064E+00_JPRB, 0.77610E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.81631E+00_JPRB, 0.80341E+00_JPRB, 0.78963E+00_JPRB, 0.78049E+00_JPRB, 0.75546E+00_JPRB, 0.76168E+00_JPRB, 0.80444E+00_JPRB ,&
&  0.80030E+00_JPRB, 0.74925E+00_JPRB, 0.71958E+00_JPRB, 0.71750E+00_JPRB, 0.74080E+00_JPRB, 0.76429E+00_JPRB, 0.76748E+00_JPRB ,&
&  0.78978E+00_JPRB, 0.82801E+00_JPRB, 0.99562E+00_JPRB, 0.99079E+00_JPRB, 0.10397E+01_JPRB, 0.10035E+01_JPRB, 0.12692E+01_JPRB ,&
&  0.16400E+01_JPRB, 0.20223E+01_JPRB, 0.23855E+01_JPRB, 0.25561E+01_JPRB, 0.28735E+01_JPRB, 0.29308E+01_JPRB, 0.28277E+01_JPRB ,&
&  0.24041E+01_JPRB, 0.20851E+01_JPRB, 0.18325E+01_JPRB, 0.16472E+01_JPRB, 0.15004E+01_JPRB, 0.13365E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11154E+01_JPRB, 0.10858E+01_JPRB, 0.10982E+01_JPRB, 0.11572E+01_JPRB, 0.12494E+01_JPRB, 0.13957E+01_JPRB ,&
&  0.15970E+01_JPRB, 0.18834E+01_JPRB, 0.21589E+01_JPRB, 0.25363E+01_JPRB, 0.22420E+01_JPRB, 0.20132E+01_JPRB, 0.24271E+01_JPRB ,&
&  0.28633E+01_JPRB, 0.32443E+01_JPRB, 0.34004E+01_JPRB, 0.46216E+01_JPRB, 0.46942E+01_JPRB, 0.41139E+01_JPRB, 0.38471E+01_JPRB ,&
&  0.32699E+01_JPRB, 0.31953E+01_JPRB, 0.24983E+01_JPRB, 0.25468E+01_JPRB, 0.27296E+01_JPRB, 0.22292E+01_JPRB, 0.21703E+01_JPRB ,&
&  0.24895E+01_JPRB, 0.23084E+01_JPRB, 0.20086E+01_JPRB, 0.15761E+01_JPRB, 0.19127E+01_JPRB, 0.17259E+01_JPRB, 0.84901E+00_JPRB ,&
&  0.71405E+00_JPRB, 0.68854E+00_JPRB, 0.66903E+00_JPRB, 0.63836E+00_JPRB, 0.66014E+00_JPRB, 0.94625E+00_JPRB, 0.18541E+01_JPRB ,&
&  0.19319E+01_JPRB, 0.28289E+01_JPRB, 0.25945E+01_JPRB, 0.23898E+01_JPRB, 0.20376E+01_JPRB, 0.21933E+01_JPRB, 0.20471E+01_JPRB ,&
&  0.16570E+01_JPRB, 0.14311E+01_JPRB, 0.11752E+01_JPRB, 0.10354E+01_JPRB, 0.91144E+00_JPRB, 0.82369E+00_JPRB, 0.80846E+00_JPRB ,&
&  0.79544E+00_JPRB, 0.77352E+00_JPRB, 0.76985E+00_JPRB, 0.78089E+00_JPRB, 0.79462E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.83850E+00_JPRB, 0.82779E+00_JPRB, 0.81751E+00_JPRB, 0.83524E+00_JPRB, 0.82736E+00_JPRB, 0.80830E+00_JPRB, 0.82771E+00_JPRB ,&
&  0.81554E+00_JPRB, 0.78184E+00_JPRB, 0.75347E+00_JPRB, 0.74470E+00_JPRB, 0.73669E+00_JPRB, 0.75440E+00_JPRB, 0.75936E+00_JPRB ,&
&  0.77518E+00_JPRB, 0.83676E+00_JPRB, 0.92340E+00_JPRB, 0.88556E+00_JPRB, 0.85073E+00_JPRB, 0.84097E+00_JPRB, 0.10057E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.20249E+01_JPRB, 0.23868E+01_JPRB, 0.27559E+01_JPRB, 0.31130E+01_JPRB, 0.30856E+01_JPRB, 0.31665E+01_JPRB ,&
&  0.28251E+01_JPRB, 0.24146E+01_JPRB, 0.21261E+01_JPRB, 0.18771E+01_JPRB, 0.17438E+01_JPRB, 0.16230E+01_JPRB, 0.14949E+01_JPRB ,&
&  0.13210E+01_JPRB, 0.11897E+01_JPRB, 0.11233E+01_JPRB, 0.10683E+01_JPRB, 0.10631E+01_JPRB, 0.11274E+01_JPRB, 0.12379E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.17255E+01_JPRB, 0.19842E+01_JPRB, 0.23731E+01_JPRB, 0.26006E+01_JPRB, 0.25802E+01_JPRB, 0.25063E+01_JPRB ,&
&  0.25563E+01_JPRB, 0.26778E+01_JPRB, 0.38979E+01_JPRB, 0.48042E+01_JPRB, 0.47476E+01_JPRB, 0.44543E+01_JPRB, 0.40914E+01_JPRB ,&
&  0.34543E+01_JPRB, 0.33197E+01_JPRB, 0.27264E+01_JPRB, 0.28469E+01_JPRB, 0.25558E+01_JPRB, 0.21077E+01_JPRB, 0.26224E+01_JPRB ,&
&  0.28728E+01_JPRB, 0.26170E+01_JPRB, 0.21244E+01_JPRB, 0.17150E+01_JPRB, 0.14897E+01_JPRB, 0.11137E+01_JPRB, 0.81489E+00_JPRB ,&
&  0.76086E+00_JPRB, 0.75439E+00_JPRB, 0.69992E+00_JPRB, 0.73321E+00_JPRB, 0.72988E+00_JPRB, 0.97028E+00_JPRB, 0.13591E+01_JPRB ,&
&  0.19885E+01_JPRB, 0.23993E+01_JPRB, 0.25029E+01_JPRB, 0.23276E+01_JPRB, 0.19666E+01_JPRB, 0.18573E+01_JPRB, 0.20049E+01_JPRB ,&
&  0.16321E+01_JPRB, 0.14779E+01_JPRB, 0.12942E+01_JPRB, 0.11057E+01_JPRB, 0.98809E+00_JPRB, 0.88977E+00_JPRB, 0.82574E+00_JPRB ,&
&  0.80104E+00_JPRB, 0.81022E+00_JPRB, 0.82104E+00_JPRB, 0.82355E+00_JPRB, 0.80955E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.85624E+00_JPRB, 0.86679E+00_JPRB, 0.87000E+00_JPRB, 0.88004E+00_JPRB, 0.87905E+00_JPRB, 0.86732E+00_JPRB, 0.86172E+00_JPRB ,&
&  0.83953E+00_JPRB, 0.81663E+00_JPRB, 0.79738E+00_JPRB, 0.78796E+00_JPRB, 0.77322E+00_JPRB, 0.76827E+00_JPRB, 0.75209E+00_JPRB ,&
&  0.75735E+00_JPRB, 0.85918E+00_JPRB, 0.73800E+00_JPRB, 0.71943E+00_JPRB, 0.75508E+00_JPRB, 0.78532E+00_JPRB, 0.87696E+00_JPRB ,&
&  0.13957E+01_JPRB, 0.20539E+01_JPRB, 0.25551E+01_JPRB, 0.32391E+01_JPRB, 0.38984E+01_JPRB, 0.36321E+01_JPRB, 0.34496E+01_JPRB ,&
&  0.32983E+01_JPRB, 0.27813E+01_JPRB, 0.24544E+01_JPRB, 0.21965E+01_JPRB, 0.19885E+01_JPRB, 0.18383E+01_JPRB, 0.16594E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.13120E+01_JPRB, 0.12037E+01_JPRB, 0.11246E+01_JPRB, 0.10704E+01_JPRB, 0.11196E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.15252E+01_JPRB, 0.17997E+01_JPRB, 0.21130E+01_JPRB, 0.23196E+01_JPRB, 0.24684E+01_JPRB, 0.33902E+01_JPRB ,&
&  0.34305E+01_JPRB, 0.35742E+01_JPRB, 0.41187E+01_JPRB, 0.57804E+01_JPRB, 0.50973E+01_JPRB, 0.42863E+01_JPRB, 0.43086E+01_JPRB ,&
&  0.29485E+01_JPRB, 0.25049E+01_JPRB, 0.25927E+01_JPRB, 0.26551E+01_JPRB, 0.23135E+01_JPRB, 0.25821E+01_JPRB, 0.40235E+01_JPRB ,&
&  0.31116E+01_JPRB, 0.29962E+01_JPRB, 0.30605E+01_JPRB, 0.33145E+01_JPRB, 0.15482E+01_JPRB, 0.10678E+01_JPRB, 0.17001E+01_JPRB ,&
&  0.18431E+01_JPRB, 0.11801E+01_JPRB, 0.87400E+00_JPRB, 0.11547E+01_JPRB, 0.89881E+00_JPRB, 0.10691E+01_JPRB, 0.15149E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.17769E+01_JPRB, 0.24662E+01_JPRB, 0.22540E+01_JPRB, 0.19302E+01_JPRB, 0.17615E+01_JPRB, 0.18640E+01_JPRB ,&
&  0.18213E+01_JPRB, 0.16144E+01_JPRB, 0.15071E+01_JPRB, 0.12712E+01_JPRB, 0.11303E+01_JPRB, 0.10250E+01_JPRB, 0.94898E+00_JPRB ,&
&  0.87307E+00_JPRB, 0.83657E+00_JPRB, 0.84387E+00_JPRB, 0.84830E+00_JPRB, 0.83406E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.88678E+00_JPRB, 0.88497E+00_JPRB, 0.89985E+00_JPRB, 0.90414E+00_JPRB, 0.91573E+00_JPRB, 0.91115E+00_JPRB, 0.90473E+00_JPRB ,&
&  0.87219E+00_JPRB, 0.84093E+00_JPRB, 0.83708E+00_JPRB, 0.83199E+00_JPRB, 0.81866E+00_JPRB, 0.79765E+00_JPRB, 0.76505E+00_JPRB ,&
&  0.76372E+00_JPRB, 0.76933E+00_JPRB, 0.67135E+00_JPRB, 0.65790E+00_JPRB, 0.74062E+00_JPRB, 0.76142E+00_JPRB, 0.87344E+00_JPRB ,&
&  0.13526E+01_JPRB, 0.20213E+01_JPRB, 0.26677E+01_JPRB, 0.35048E+01_JPRB, 0.40571E+01_JPRB, 0.39950E+01_JPRB, 0.36885E+01_JPRB ,&
&  0.36430E+01_JPRB, 0.29809E+01_JPRB, 0.26127E+01_JPRB, 0.24018E+01_JPRB, 0.21583E+01_JPRB, 0.19288E+01_JPRB, 0.17317E+01_JPRB ,&
&  0.15448E+01_JPRB, 0.13762E+01_JPRB, 0.12710E+01_JPRB, 0.11844E+01_JPRB, 0.11583E+01_JPRB, 0.11660E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.13067E+01_JPRB, 0.16309E+01_JPRB, 0.19545E+01_JPRB, 0.20881E+01_JPRB, 0.22751E+01_JPRB, 0.29499E+01_JPRB ,&
&  0.35051E+01_JPRB, 0.36291E+01_JPRB, 0.43579E+01_JPRB, 0.52880E+01_JPRB, 0.56393E+01_JPRB, 0.47558E+01_JPRB, 0.50930E+01_JPRB ,&
&  0.36495E+01_JPRB, 0.29727E+01_JPRB, 0.25768E+01_JPRB, 0.22174E+01_JPRB, 0.25086E+01_JPRB, 0.40146E+01_JPRB, 0.36675E+01_JPRB ,&
&  0.33497E+01_JPRB, 0.30707E+01_JPRB, 0.30202E+01_JPRB, 0.26677E+01_JPRB, 0.23351E+01_JPRB, 0.16462E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.22450E+01_JPRB, 0.22129E+01_JPRB, 0.19549E+01_JPRB, 0.16315E+01_JPRB, 0.13231E+01_JPRB, 0.15132E+01_JPRB, 0.15224E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.14970E+01_JPRB, 0.18290E+01_JPRB, 0.19850E+01_JPRB, 0.19562E+01_JPRB, 0.17319E+01_JPRB, 0.18774E+01_JPRB ,&
&  0.17737E+01_JPRB, 0.17929E+01_JPRB, 0.16867E+01_JPRB, 0.14396E+01_JPRB, 0.12593E+01_JPRB, 0.11643E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.92644E+00_JPRB, 0.88668E+00_JPRB, 0.86149E+00_JPRB, 0.85459E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.91675E+00_JPRB, 0.92501E+00_JPRB, 0.92481E+00_JPRB, 0.92017E+00_JPRB, 0.91067E+00_JPRB, 0.89656E+00_JPRB, 0.89222E+00_JPRB ,&
&  0.87179E+00_JPRB, 0.83643E+00_JPRB, 0.82612E+00_JPRB, 0.82115E+00_JPRB, 0.81220E+00_JPRB, 0.78898E+00_JPRB, 0.76641E+00_JPRB ,&
&  0.76444E+00_JPRB, 0.74644E+00_JPRB, 0.64706E+00_JPRB, 0.67734E+00_JPRB, 0.71682E+00_JPRB, 0.73952E+00_JPRB, 0.89746E+00_JPRB ,&
&  0.11537E+01_JPRB, 0.16648E+01_JPRB, 0.22299E+01_JPRB, 0.30204E+01_JPRB, 0.37305E+01_JPRB, 0.40418E+01_JPRB, 0.40812E+01_JPRB ,&
&  0.36244E+01_JPRB, 0.32918E+01_JPRB, 0.28427E+01_JPRB, 0.24806E+01_JPRB, 0.21845E+01_JPRB, 0.19728E+01_JPRB, 0.17608E+01_JPRB ,&
&  0.15847E+01_JPRB, 0.14475E+01_JPRB, 0.13567E+01_JPRB, 0.12684E+01_JPRB, 0.12316E+01_JPRB, 0.12231E+01_JPRB, 0.12113E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11850E+01_JPRB, 0.14297E+01_JPRB, 0.18090E+01_JPRB, 0.19271E+01_JPRB, 0.21565E+01_JPRB, 0.24346E+01_JPRB ,&
&  0.36177E+01_JPRB, 0.41600E+01_JPRB, 0.42724E+01_JPRB, 0.57881E+01_JPRB, 0.40781E+01_JPRB, 0.48840E+01_JPRB, 0.57754E+01_JPRB ,&
&  0.53920E+01_JPRB, 0.47885E+01_JPRB, 0.46110E+01_JPRB, 0.35961E+01_JPRB, 0.28907E+01_JPRB, 0.39286E+01_JPRB, 0.33301E+01_JPRB ,&
&  0.29165E+01_JPRB, 0.29349E+01_JPRB, 0.28505E+01_JPRB, 0.30697E+01_JPRB, 0.23425E+01_JPRB, 0.20126E+01_JPRB, 0.15864E+01_JPRB ,&
&  0.16322E+01_JPRB, 0.14972E+01_JPRB, 0.17522E+01_JPRB, 0.15462E+01_JPRB, 0.15178E+01_JPRB, 0.14347E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.13822E+01_JPRB, 0.13828E+01_JPRB, 0.15826E+01_JPRB, 0.18688E+01_JPRB, 0.16884E+01_JPRB, 0.18075E+01_JPRB ,&
&  0.18190E+01_JPRB, 0.17945E+01_JPRB, 0.16824E+01_JPRB, 0.15756E+01_JPRB, 0.13727E+01_JPRB, 0.12382E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.11271E+01_JPRB, 0.10478E+01_JPRB, 0.96259E+00_JPRB, 0.93021E+00_JPRB, 0.89813E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.94016E+00_JPRB, 0.93270E+00_JPRB, 0.90443E+00_JPRB, 0.90491E+00_JPRB, 0.90106E+00_JPRB, 0.88401E+00_JPRB, 0.85706E+00_JPRB ,&
&  0.82531E+00_JPRB, 0.80773E+00_JPRB, 0.79083E+00_JPRB, 0.78155E+00_JPRB, 0.78047E+00_JPRB, 0.77012E+00_JPRB, 0.75442E+00_JPRB ,&
&  0.74259E+00_JPRB, 0.84247E+00_JPRB, 0.74328E+00_JPRB, 0.67227E+00_JPRB, 0.63443E+00_JPRB, 0.71967E+00_JPRB, 0.87048E+00_JPRB ,&
&  0.10958E+01_JPRB, 0.13681E+01_JPRB, 0.17890E+01_JPRB, 0.21565E+01_JPRB, 0.26598E+01_JPRB, 0.31141E+01_JPRB, 0.34231E+01_JPRB ,&
&  0.35639E+01_JPRB, 0.27283E+01_JPRB, 0.24813E+01_JPRB, 0.21340E+01_JPRB, 0.19115E+01_JPRB, 0.17079E+01_JPRB, 0.15769E+01_JPRB ,&
&  0.14564E+01_JPRB, 0.13785E+01_JPRB, 0.13132E+01_JPRB, 0.12463E+01_JPRB, 0.12069E+01_JPRB, 0.11816E+01_JPRB, 0.11766E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11348E+01_JPRB, 0.12580E+01_JPRB, 0.15548E+01_JPRB, 0.20755E+01_JPRB, 0.25118E+01_JPRB, 0.28044E+01_JPRB ,&
&  0.26443E+01_JPRB, 0.26560E+01_JPRB, 0.46171E+01_JPRB, 0.52574E+01_JPRB, 0.52771E+01_JPRB, 0.61796E+01_JPRB, 0.64782E+01_JPRB ,&
&  0.60531E+01_JPRB, 0.48732E+01_JPRB, 0.41055E+01_JPRB, 0.34885E+01_JPRB, 0.39226E+01_JPRB, 0.35637E+01_JPRB, 0.32072E+01_JPRB ,&
&  0.29042E+01_JPRB, 0.30538E+01_JPRB, 0.29874E+01_JPRB, 0.29573E+01_JPRB, 0.29037E+01_JPRB, 0.28395E+01_JPRB, 0.23945E+01_JPRB ,&
&  0.20789E+01_JPRB, 0.24566E+01_JPRB, 0.19751E+01_JPRB, 0.15053E+01_JPRB, 0.12367E+01_JPRB, 0.12650E+01_JPRB, 0.13487E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.14236E+01_JPRB, 0.14013E+01_JPRB, 0.14732E+01_JPRB, 0.16341E+01_JPRB, 0.16838E+01_JPRB, 0.17235E+01_JPRB ,&
&  0.16565E+01_JPRB, 0.17288E+01_JPRB, 0.16892E+01_JPRB, 0.16132E+01_JPRB, 0.14853E+01_JPRB, 0.13267E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.11561E+01_JPRB, 0.10741E+01_JPRB, 0.99338E+00_JPRB, 0.94290E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.94336E+00_JPRB, 0.91910E+00_JPRB, 0.88841E+00_JPRB, 0.89036E+00_JPRB, 0.89978E+00_JPRB, 0.89175E+00_JPRB, 0.85493E+00_JPRB ,&
&  0.79464E+00_JPRB, 0.76155E+00_JPRB, 0.74935E+00_JPRB, 0.75445E+00_JPRB, 0.76244E+00_JPRB, 0.76694E+00_JPRB, 0.76883E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.98375E+00_JPRB, 0.87548E+00_JPRB, 0.83894E+00_JPRB, 0.75159E+00_JPRB, 0.84657E+00_JPRB, 0.97248E+00_JPRB ,&
&  0.10719E+01_JPRB, 0.11428E+01_JPRB, 0.12913E+01_JPRB, 0.15135E+01_JPRB, 0.17426E+01_JPRB, 0.19545E+01_JPRB, 0.23101E+01_JPRB ,&
&  0.23045E+01_JPRB, 0.21337E+01_JPRB, 0.17667E+01_JPRB, 0.16754E+01_JPRB, 0.15715E+01_JPRB, 0.13707E+01_JPRB, 0.13794E+01_JPRB ,&
&  0.13011E+01_JPRB, 0.12557E+01_JPRB, 0.12152E+01_JPRB, 0.11808E+01_JPRB, 0.11443E+01_JPRB, 0.10944E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10628E+01_JPRB, 0.11509E+01_JPRB, 0.12728E+01_JPRB, 0.16424E+01_JPRB, 0.21182E+01_JPRB, 0.24836E+01_JPRB ,&
&  0.29726E+01_JPRB, 0.27259E+01_JPRB, 0.25673E+01_JPRB, 0.36831E+01_JPRB, 0.57671E+01_JPRB, 0.62485E+01_JPRB, 0.58503E+01_JPRB ,&
&  0.62940E+01_JPRB, 0.58394E+01_JPRB, 0.50944E+01_JPRB, 0.44994E+01_JPRB, 0.41003E+01_JPRB, 0.37284E+01_JPRB, 0.35508E+01_JPRB ,&
&  0.32614E+01_JPRB, 0.31856E+01_JPRB, 0.33164E+01_JPRB, 0.33791E+01_JPRB, 0.31534E+01_JPRB, 0.28929E+01_JPRB, 0.28532E+01_JPRB ,&
&  0.26997E+01_JPRB, 0.22360E+01_JPRB, 0.15084E+01_JPRB, 0.14650E+01_JPRB, 0.11291E+01_JPRB, 0.11224E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.13093E+01_JPRB, 0.14034E+01_JPRB, 0.14304E+01_JPRB, 0.13662E+01_JPRB, 0.14636E+01_JPRB, 0.14782E+01_JPRB, 0.15459E+01_JPRB ,&
&  0.15847E+01_JPRB, 0.13580E+01_JPRB, 0.14082E+01_JPRB, 0.14412E+01_JPRB, 0.14318E+01_JPRB, 0.13479E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.12203E+01_JPRB, 0.11813E+01_JPRB, 0.11246E+01_JPRB, 0.10411E+01_JPRB, 0.97933E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.95435E+00_JPRB, 0.92436E+00_JPRB, 0.89352E+00_JPRB, 0.88070E+00_JPRB, 0.87643E+00_JPRB, 0.86685E+00_JPRB, 0.83380E+00_JPRB ,&
&  0.78637E+00_JPRB, 0.75336E+00_JPRB, 0.74028E+00_JPRB, 0.74468E+00_JPRB, 0.74948E+00_JPRB, 0.74498E+00_JPRB, 0.76937E+00_JPRB ,&
&  0.88147E+00_JPRB, 0.10359E+01_JPRB, 0.94299E+00_JPRB, 0.83140E+00_JPRB, 0.97157E+00_JPRB, 0.93071E+00_JPRB, 0.96707E+00_JPRB ,&
&  0.94380E+00_JPRB, 0.10588E+01_JPRB, 0.10868E+01_JPRB, 0.11373E+01_JPRB, 0.12055E+01_JPRB, 0.13253E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.15549E+01_JPRB, 0.13946E+01_JPRB, 0.12689E+01_JPRB, 0.12500E+01_JPRB, 0.12988E+01_JPRB, 0.11815E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.10824E+01_JPRB, 0.10533E+01_JPRB, 0.10378E+01_JPRB, 0.10210E+01_JPRB, 0.10002E+01_JPRB, 0.98763E+00_JPRB ,&
&  0.98818E+00_JPRB, 0.99529E+00_JPRB, 0.10625E+01_JPRB, 0.11461E+01_JPRB, 0.13664E+01_JPRB, 0.18666E+01_JPRB, 0.26850E+01_JPRB ,&
&  0.31214E+01_JPRB, 0.32626E+01_JPRB, 0.36216E+01_JPRB, 0.43167E+01_JPRB, 0.46892E+01_JPRB, 0.49271E+01_JPRB, 0.52510E+01_JPRB ,&
&  0.53580E+01_JPRB, 0.53539E+01_JPRB, 0.53519E+01_JPRB, 0.50711E+01_JPRB, 0.46171E+01_JPRB, 0.42516E+01_JPRB, 0.40449E+01_JPRB ,&
&  0.37730E+01_JPRB, 0.36382E+01_JPRB, 0.36845E+01_JPRB, 0.35280E+01_JPRB, 0.34183E+01_JPRB, 0.31353E+01_JPRB, 0.32093E+01_JPRB ,&
&  0.31121E+01_JPRB, 0.18265E+01_JPRB, 0.13940E+01_JPRB, 0.15656E+01_JPRB, 0.12375E+01_JPRB, 0.12720E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.13771E+01_JPRB, 0.14048E+01_JPRB, 0.13554E+01_JPRB, 0.13090E+01_JPRB, 0.13667E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.11211E+01_JPRB, 0.11539E+01_JPRB, 0.12037E+01_JPRB, 0.12532E+01_JPRB, 0.12302E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.11341E+01_JPRB, 0.11118E+01_JPRB, 0.10622E+01_JPRB, 0.99711E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.98994E+00_JPRB, 0.96296E+00_JPRB, 0.93676E+00_JPRB, 0.91353E+00_JPRB, 0.88252E+00_JPRB, 0.84780E+00_JPRB, 0.80492E+00_JPRB ,&
&  0.76791E+00_JPRB, 0.73721E+00_JPRB, 0.72312E+00_JPRB, 0.71843E+00_JPRB, 0.71510E+00_JPRB, 0.70669E+00_JPRB, 0.75203E+00_JPRB ,&
&  0.71800E+00_JPRB, 0.83572E+00_JPRB, 0.76974E+00_JPRB, 0.82207E+00_JPRB, 0.10149E+01_JPRB, 0.91417E+00_JPRB, 0.90627E+00_JPRB ,&
&  0.91081E+00_JPRB, 0.94798E+00_JPRB, 0.94024E+00_JPRB, 0.92891E+00_JPRB, 0.92626E+00_JPRB, 0.95086E+00_JPRB, 0.10456E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.98379E+00_JPRB, 0.94825E+00_JPRB, 0.95223E+00_JPRB, 0.99055E+00_JPRB, 0.10398E+01_JPRB, 0.10709E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.99989E+00_JPRB, 0.97579E+00_JPRB, 0.95227E+00_JPRB, 0.92500E+00_JPRB, 0.90086E+00_JPRB, 0.88418E+00_JPRB ,&
&  0.87788E+00_JPRB, 0.88911E+00_JPRB, 0.95326E+00_JPRB, 0.10562E+01_JPRB, 0.11684E+01_JPRB, 0.16720E+01_JPRB, 0.27704E+01_JPRB ,&
&  0.34405E+01_JPRB, 0.34463E+01_JPRB, 0.38113E+01_JPRB, 0.45578E+01_JPRB, 0.46523E+01_JPRB, 0.46236E+01_JPRB, 0.46076E+01_JPRB ,&
&  0.46414E+01_JPRB, 0.46132E+01_JPRB, 0.46136E+01_JPRB, 0.48221E+01_JPRB, 0.48342E+01_JPRB, 0.48773E+01_JPRB, 0.47080E+01_JPRB ,&
&  0.43548E+01_JPRB, 0.41965E+01_JPRB, 0.41076E+01_JPRB, 0.38957E+01_JPRB, 0.36313E+01_JPRB, 0.35449E+01_JPRB, 0.33920E+01_JPRB ,&
&  0.31505E+01_JPRB, 0.27773E+01_JPRB, 0.20552E+01_JPRB, 0.18428E+01_JPRB, 0.13572E+01_JPRB, 0.13686E+01_JPRB, 0.16708E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.13884E+01_JPRB, 0.12624E+01_JPRB, 0.11919E+01_JPRB, 0.11499E+01_JPRB, 0.11700E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.94272E+00_JPRB, 0.96268E+00_JPRB, 0.99753E+00_JPRB, 0.10409E+01_JPRB, 0.10872E+01_JPRB, 0.10977E+01_JPRB, 0.99981E+00_JPRB ,&
&  0.10354E+01_JPRB, 0.10472E+01_JPRB, 0.10488E+01_JPRB, 0.10467E+01_JPRB, 0.10239E+01_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.95544E+00_JPRB, 0.93719E+00_JPRB, 0.91985E+00_JPRB, 0.90008E+00_JPRB, 0.85953E+00_JPRB, 0.80222E+00_JPRB, 0.75699E+00_JPRB ,&
&  0.73928E+00_JPRB, 0.71835E+00_JPRB, 0.70179E+00_JPRB, 0.68981E+00_JPRB, 0.67905E+00_JPRB, 0.66829E+00_JPRB, 0.66231E+00_JPRB ,&
&  0.65188E+00_JPRB, 0.72094E+00_JPRB, 0.71910E+00_JPRB, 0.75044E+00_JPRB, 0.77971E+00_JPRB, 0.75615E+00_JPRB, 0.75471E+00_JPRB ,&
&  0.79799E+00_JPRB, 0.81052E+00_JPRB, 0.81088E+00_JPRB, 0.79907E+00_JPRB, 0.79031E+00_JPRB, 0.80701E+00_JPRB, 0.85370E+00_JPRB ,&
&  0.83431E+00_JPRB, 0.82131E+00_JPRB, 0.80854E+00_JPRB, 0.81900E+00_JPRB, 0.88201E+00_JPRB, 0.95855E+00_JPRB, 0.96948E+00_JPRB ,&
&  0.94132E+00_JPRB, 0.91461E+00_JPRB, 0.88622E+00_JPRB, 0.86364E+00_JPRB, 0.84064E+00_JPRB, 0.81726E+00_JPRB, 0.79944E+00_JPRB ,&
&  0.79669E+00_JPRB, 0.81255E+00_JPRB, 0.86338E+00_JPRB, 0.95641E+00_JPRB, 0.10710E+01_JPRB, 0.14090E+01_JPRB, 0.19484E+01_JPRB ,&
&  0.22640E+01_JPRB, 0.25189E+01_JPRB, 0.28155E+01_JPRB, 0.31729E+01_JPRB, 0.33904E+01_JPRB, 0.34721E+01_JPRB, 0.35167E+01_JPRB ,&
&  0.36870E+01_JPRB, 0.36934E+01_JPRB, 0.39820E+01_JPRB, 0.39761E+01_JPRB, 0.39935E+01_JPRB, 0.41564E+01_JPRB, 0.41355E+01_JPRB ,&
&  0.40227E+01_JPRB, 0.38058E+01_JPRB, 0.38968E+01_JPRB, 0.38095E+01_JPRB, 0.36558E+01_JPRB, 0.34966E+01_JPRB, 0.32625E+01_JPRB ,&
&  0.30335E+01_JPRB, 0.28212E+01_JPRB, 0.24289E+01_JPRB, 0.21470E+01_JPRB, 0.18179E+01_JPRB, 0.18201E+01_JPRB, 0.16260E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.11093E+01_JPRB, 0.10678E+01_JPRB, 0.10326E+01_JPRB, 0.10315E+01_JPRB, 0.96529E+00_JPRB, 0.86206E+00_JPRB ,&
&  0.84669E+00_JPRB, 0.90429E+00_JPRB, 0.91404E+00_JPRB, 0.92762E+00_JPRB, 0.96036E+00_JPRB, 0.98858E+00_JPRB, 0.84945E+00_JPRB ,&
&  0.84325E+00_JPRB, 0.92020E+00_JPRB, 0.94923E+00_JPRB, 0.96479E+00_JPRB, 0.96598E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.88586E+00_JPRB, 0.87328E+00_JPRB, 0.85387E+00_JPRB, 0.83336E+00_JPRB, 0.79664E+00_JPRB, 0.74930E+00_JPRB, 0.70080E+00_JPRB ,&
&  0.70438E+00_JPRB, 0.69269E+00_JPRB, 0.67984E+00_JPRB, 0.66303E+00_JPRB, 0.64408E+00_JPRB, 0.61060E+00_JPRB, 0.56081E+00_JPRB ,&
&  0.56156E+00_JPRB, 0.59629E+00_JPRB, 0.64573E+00_JPRB, 0.67138E+00_JPRB, 0.67027E+00_JPRB, 0.66692E+00_JPRB, 0.67434E+00_JPRB ,&
&  0.68391E+00_JPRB, 0.69297E+00_JPRB, 0.70909E+00_JPRB, 0.72024E+00_JPRB, 0.71570E+00_JPRB, 0.71463E+00_JPRB, 0.73349E+00_JPRB ,&
&  0.70763E+00_JPRB, 0.69705E+00_JPRB, 0.71331E+00_JPRB, 0.68649E+00_JPRB, 0.75558E+00_JPRB, 0.78296E+00_JPRB, 0.79223E+00_JPRB ,&
&  0.77333E+00_JPRB, 0.74311E+00_JPRB, 0.72123E+00_JPRB, 0.71929E+00_JPRB, 0.71886E+00_JPRB, 0.71530E+00_JPRB, 0.71594E+00_JPRB ,&
&  0.72629E+00_JPRB, 0.74694E+00_JPRB, 0.79065E+00_JPRB, 0.86254E+00_JPRB, 0.95923E+00_JPRB, 0.11792E+01_JPRB, 0.14060E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.19681E+01_JPRB, 0.22988E+01_JPRB, 0.23570E+01_JPRB, 0.26210E+01_JPRB, 0.28484E+01_JPRB, 0.28840E+01_JPRB ,&
&  0.30794E+01_JPRB, 0.31159E+01_JPRB, 0.32960E+01_JPRB, 0.33647E+01_JPRB, 0.34148E+01_JPRB, 0.34392E+01_JPRB, 0.34494E+01_JPRB ,&
&  0.34431E+01_JPRB, 0.34407E+01_JPRB, 0.34751E+01_JPRB, 0.33976E+01_JPRB, 0.32284E+01_JPRB, 0.30694E+01_JPRB, 0.28960E+01_JPRB ,&
&  0.27524E+01_JPRB, 0.26008E+01_JPRB, 0.24423E+01_JPRB, 0.23015E+01_JPRB, 0.21397E+01_JPRB, 0.19262E+01_JPRB, 0.17025E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.12038E+01_JPRB, 0.10612E+01_JPRB, 0.10218E+01_JPRB, 0.97874E+00_JPRB, 0.86184E+00_JPRB, 0.79652E+00_JPRB ,&
&  0.77544E+00_JPRB, 0.79422E+00_JPRB, 0.84493E+00_JPRB, 0.85088E+00_JPRB, 0.86877E+00_JPRB, 0.87436E+00_JPRB, 0.80977E+00_JPRB ,&
&  0.76363E+00_JPRB, 0.82418E+00_JPRB, 0.85452E+00_JPRB, 0.87236E+00_JPRB, 0.88530E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.81884E+00_JPRB, 0.81108E+00_JPRB, 0.79195E+00_JPRB, 0.77422E+00_JPRB, 0.75569E+00_JPRB, 0.71150E+00_JPRB, 0.67338E+00_JPRB ,&
&  0.68850E+00_JPRB, 0.65185E+00_JPRB, 0.64248E+00_JPRB, 0.58976E+00_JPRB, 0.56306E+00_JPRB, 0.55134E+00_JPRB, 0.54201E+00_JPRB ,&
&  0.54785E+00_JPRB, 0.56825E+00_JPRB, 0.58692E+00_JPRB, 0.60288E+00_JPRB, 0.61879E+00_JPRB, 0.62114E+00_JPRB, 0.62133E+00_JPRB ,&
&  0.62200E+00_JPRB, 0.62762E+00_JPRB, 0.63944E+00_JPRB, 0.64333E+00_JPRB, 0.64706E+00_JPRB, 0.64700E+00_JPRB, 0.64744E+00_JPRB ,&
&  0.63310E+00_JPRB, 0.63085E+00_JPRB, 0.65247E+00_JPRB, 0.65258E+00_JPRB, 0.65734E+00_JPRB, 0.65507E+00_JPRB, 0.64536E+00_JPRB ,&
&  0.61958E+00_JPRB, 0.59509E+00_JPRB, 0.61065E+00_JPRB, 0.62261E+00_JPRB, 0.63394E+00_JPRB, 0.64076E+00_JPRB, 0.64495E+00_JPRB ,&
&  0.65014E+00_JPRB, 0.66066E+00_JPRB, 0.68185E+00_JPRB, 0.71712E+00_JPRB, 0.77478E+00_JPRB, 0.89314E+00_JPRB, 0.10072E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.11817E+01_JPRB, 0.14737E+01_JPRB, 0.16979E+01_JPRB, 0.19815E+01_JPRB, 0.22165E+01_JPRB, 0.22907E+01_JPRB ,&
&  0.24808E+01_JPRB, 0.25301E+01_JPRB, 0.25775E+01_JPRB, 0.25714E+01_JPRB, 0.25531E+01_JPRB, 0.25603E+01_JPRB, 0.26207E+01_JPRB ,&
&  0.26770E+01_JPRB, 0.25843E+01_JPRB, 0.27496E+01_JPRB, 0.26725E+01_JPRB, 0.25730E+01_JPRB, 0.25243E+01_JPRB, 0.24741E+01_JPRB ,&
&  0.23993E+01_JPRB, 0.22821E+01_JPRB, 0.21256E+01_JPRB, 0.18445E+01_JPRB, 0.17420E+01_JPRB, 0.16498E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.13739E+01_JPRB, 0.12571E+01_JPRB, 0.11347E+01_JPRB, 0.11095E+01_JPRB, 0.98668E+00_JPRB, 0.89677E+00_JPRB, 0.83592E+00_JPRB ,&
&  0.76031E+00_JPRB, 0.72251E+00_JPRB, 0.74299E+00_JPRB, 0.75518E+00_JPRB, 0.75407E+00_JPRB, 0.74284E+00_JPRB, 0.77903E+00_JPRB ,&
&  0.75152E+00_JPRB, 0.76101E+00_JPRB, 0.80618E+00_JPRB, 0.81149E+00_JPRB, 0.81547E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.77862E+00_JPRB, 0.77271E+00_JPRB, 0.75374E+00_JPRB, 0.73755E+00_JPRB, 0.71884E+00_JPRB, 0.68833E+00_JPRB, 0.63879E+00_JPRB ,&
&  0.61756E+00_JPRB, 0.58304E+00_JPRB, 0.55579E+00_JPRB, 0.54639E+00_JPRB, 0.54287E+00_JPRB, 0.53592E+00_JPRB, 0.52532E+00_JPRB ,&
&  0.52857E+00_JPRB, 0.54632E+00_JPRB, 0.56816E+00_JPRB, 0.58074E+00_JPRB, 0.59049E+00_JPRB, 0.59268E+00_JPRB, 0.59545E+00_JPRB ,&
&  0.59840E+00_JPRB, 0.59829E+00_JPRB, 0.60016E+00_JPRB, 0.59819E+00_JPRB, 0.59495E+00_JPRB, 0.59734E+00_JPRB, 0.59949E+00_JPRB ,&
&  0.60631E+00_JPRB, 0.60884E+00_JPRB, 0.61192E+00_JPRB, 0.61642E+00_JPRB, 0.61449E+00_JPRB, 0.61167E+00_JPRB, 0.57464E+00_JPRB ,&
&  0.52596E+00_JPRB, 0.53971E+00_JPRB, 0.55849E+00_JPRB, 0.57155E+00_JPRB, 0.58979E+00_JPRB, 0.60014E+00_JPRB, 0.60605E+00_JPRB ,&
&  0.60814E+00_JPRB, 0.61022E+00_JPRB, 0.61914E+00_JPRB, 0.64090E+00_JPRB, 0.67842E+00_JPRB, 0.73597E+00_JPRB, 0.79664E+00_JPRB ,&
&  0.93028E+00_JPRB, 0.94187E+00_JPRB, 0.99193E+00_JPRB, 0.11445E+01_JPRB, 0.14881E+01_JPRB, 0.16761E+01_JPRB, 0.17592E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.19095E+01_JPRB, 0.19301E+01_JPRB, 0.19666E+01_JPRB, 0.19507E+01_JPRB, 0.19417E+01_JPRB, 0.19773E+01_JPRB ,&
&  0.19746E+01_JPRB, 0.20199E+01_JPRB, 0.20375E+01_JPRB, 0.20543E+01_JPRB, 0.20013E+01_JPRB, 0.19727E+01_JPRB, 0.19445E+01_JPRB ,&
&  0.19049E+01_JPRB, 0.18319E+01_JPRB, 0.16896E+01_JPRB, 0.15463E+01_JPRB, 0.14162E+01_JPRB, 0.13429E+01_JPRB, 0.12838E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11641E+01_JPRB, 0.11118E+01_JPRB, 0.10492E+01_JPRB, 0.97056E+00_JPRB, 0.89796E+00_JPRB, 0.83181E+00_JPRB ,&
&  0.72874E+00_JPRB, 0.69346E+00_JPRB, 0.67655E+00_JPRB, 0.66726E+00_JPRB, 0.67296E+00_JPRB, 0.69353E+00_JPRB, 0.68260E+00_JPRB ,&
&  0.70964E+00_JPRB, 0.71989E+00_JPRB, 0.71586E+00_JPRB, 0.73035E+00_JPRB, 0.75212E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.71542E+00_JPRB, 0.71917E+00_JPRB, 0.70899E+00_JPRB, 0.70583E+00_JPRB, 0.68460E+00_JPRB, 0.66102E+00_JPRB, 0.63341E+00_JPRB ,&
&  0.61613E+00_JPRB, 0.59306E+00_JPRB, 0.56912E+00_JPRB, 0.54723E+00_JPRB, 0.53636E+00_JPRB, 0.53320E+00_JPRB, 0.53885E+00_JPRB ,&
&  0.55034E+00_JPRB, 0.56396E+00_JPRB, 0.57451E+00_JPRB, 0.57889E+00_JPRB, 0.57713E+00_JPRB, 0.57553E+00_JPRB, 0.57740E+00_JPRB ,&
&  0.57773E+00_JPRB, 0.57734E+00_JPRB, 0.57886E+00_JPRB, 0.57793E+00_JPRB, 0.57760E+00_JPRB, 0.57829E+00_JPRB, 0.57954E+00_JPRB ,&
&  0.57958E+00_JPRB, 0.57638E+00_JPRB, 0.58146E+00_JPRB, 0.58879E+00_JPRB, 0.59249E+00_JPRB, 0.57690E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.49849E+00_JPRB, 0.49278E+00_JPRB, 0.53145E+00_JPRB, 0.53620E+00_JPRB, 0.55645E+00_JPRB, 0.56685E+00_JPRB, 0.56651E+00_JPRB ,&
&  0.55921E+00_JPRB, 0.55533E+00_JPRB, 0.56854E+00_JPRB, 0.59325E+00_JPRB, 0.61559E+00_JPRB, 0.64779E+00_JPRB, 0.68322E+00_JPRB ,&
&  0.73597E+00_JPRB, 0.80447E+00_JPRB, 0.87597E+00_JPRB, 0.88177E+00_JPRB, 0.98008E+00_JPRB, 0.13253E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14567E+01_JPRB, 0.14948E+01_JPRB, 0.15704E+01_JPRB, 0.16143E+01_JPRB, 0.15754E+01_JPRB, 0.15460E+01_JPRB, 0.15340E+01_JPRB ,&
&  0.14948E+01_JPRB, 0.14916E+01_JPRB, 0.15058E+01_JPRB, 0.15473E+01_JPRB, 0.15382E+01_JPRB, 0.15139E+01_JPRB, 0.14893E+01_JPRB ,&
&  0.14629E+01_JPRB, 0.13872E+01_JPRB, 0.13035E+01_JPRB, 0.11997E+01_JPRB, 0.11398E+01_JPRB, 0.10966E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10228E+01_JPRB, 0.10103E+01_JPRB, 0.98994E+00_JPRB, 0.90112E+00_JPRB, 0.79679E+00_JPRB, 0.71800E+00_JPRB ,&
&  0.67000E+00_JPRB, 0.64281E+00_JPRB, 0.63348E+00_JPRB, 0.63546E+00_JPRB, 0.64377E+00_JPRB, 0.65214E+00_JPRB, 0.63433E+00_JPRB ,&
&  0.63040E+00_JPRB, 0.63592E+00_JPRB, 0.64987E+00_JPRB, 0.67388E+00_JPRB, 0.69646E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.64100E+00_JPRB, 0.66381E+00_JPRB, 0.68077E+00_JPRB, 0.68298E+00_JPRB, 0.67453E+00_JPRB, 0.63320E+00_JPRB, 0.59534E+00_JPRB ,&
&  0.57339E+00_JPRB, 0.56682E+00_JPRB, 0.56801E+00_JPRB, 0.56972E+00_JPRB, 0.57270E+00_JPRB, 0.57721E+00_JPRB, 0.58036E+00_JPRB ,&
&  0.58114E+00_JPRB, 0.58326E+00_JPRB, 0.58724E+00_JPRB, 0.59104E+00_JPRB, 0.58860E+00_JPRB, 0.58468E+00_JPRB, 0.58386E+00_JPRB ,&
&  0.58278E+00_JPRB, 0.58004E+00_JPRB, 0.58027E+00_JPRB, 0.57639E+00_JPRB, 0.57521E+00_JPRB, 0.57219E+00_JPRB, 0.56593E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.55161E+00_JPRB, 0.55828E+00_JPRB, 0.56251E+00_JPRB, 0.56719E+00_JPRB, 0.54461E+00_JPRB, 0.50538E+00_JPRB ,&
&  0.47712E+00_JPRB, 0.46339E+00_JPRB, 0.47056E+00_JPRB, 0.50053E+00_JPRB, 0.52565E+00_JPRB, 0.51248E+00_JPRB, 0.52420E+00_JPRB ,&
&  0.52546E+00_JPRB, 0.52817E+00_JPRB, 0.53552E+00_JPRB, 0.55276E+00_JPRB, 0.57249E+00_JPRB, 0.59315E+00_JPRB, 0.61283E+00_JPRB ,&
&  0.63673E+00_JPRB, 0.66743E+00_JPRB, 0.70933E+00_JPRB, 0.71835E+00_JPRB, 0.73395E+00_JPRB, 0.81782E+00_JPRB, 0.92688E+00_JPRB ,&
&  0.10381E+01_JPRB, 0.11261E+01_JPRB, 0.11716E+01_JPRB, 0.12376E+01_JPRB, 0.12754E+01_JPRB, 0.12829E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12607E+01_JPRB, 0.12335E+01_JPRB, 0.12166E+01_JPRB, 0.12155E+01_JPRB, 0.12403E+01_JPRB, 0.12575E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11287E+01_JPRB, 0.10650E+01_JPRB, 0.97023E+00_JPRB, 0.94735E+00_JPRB, 0.96327E+00_JPRB, 0.98708E+00_JPRB ,&
&  0.97699E+00_JPRB, 0.95964E+00_JPRB, 0.94659E+00_JPRB, 0.93403E+00_JPRB, 0.84089E+00_JPRB, 0.77441E+00_JPRB, 0.72638E+00_JPRB ,&
&  0.70222E+00_JPRB, 0.68164E+00_JPRB, 0.67535E+00_JPRB, 0.67520E+00_JPRB, 0.67289E+00_JPRB, 0.66692E+00_JPRB, 0.64523E+00_JPRB ,&
&  0.62846E+00_JPRB, 0.61881E+00_JPRB, 0.61791E+00_JPRB, 0.61851E+00_JPRB, 0.62791E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.64013E+00_JPRB, 0.64691E+00_JPRB, 0.65438E+00_JPRB, 0.65842E+00_JPRB, 0.65511E+00_JPRB, 0.63752E+00_JPRB, 0.62145E+00_JPRB ,&
&  0.61083E+00_JPRB, 0.60904E+00_JPRB, 0.60822E+00_JPRB, 0.60620E+00_JPRB, 0.60536E+00_JPRB, 0.60477E+00_JPRB, 0.60489E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.60696E+00_JPRB, 0.60482E+00_JPRB, 0.59783E+00_JPRB, 0.59271E+00_JPRB, 0.59167E+00_JPRB, 0.59300E+00_JPRB ,&
&  0.59280E+00_JPRB, 0.58942E+00_JPRB, 0.58858E+00_JPRB, 0.58001E+00_JPRB, 0.57085E+00_JPRB, 0.55796E+00_JPRB, 0.54838E+00_JPRB ,&
&  0.54741E+00_JPRB, 0.54940E+00_JPRB, 0.54181E+00_JPRB, 0.54183E+00_JPRB, 0.54563E+00_JPRB, 0.51449E+00_JPRB, 0.48396E+00_JPRB ,&
&  0.46241E+00_JPRB, 0.44706E+00_JPRB, 0.43979E+00_JPRB, 0.44079E+00_JPRB, 0.46107E+00_JPRB, 0.49028E+00_JPRB, 0.50350E+00_JPRB ,&
&  0.48654E+00_JPRB, 0.50247E+00_JPRB, 0.52317E+00_JPRB, 0.53578E+00_JPRB, 0.54692E+00_JPRB, 0.56130E+00_JPRB, 0.57562E+00_JPRB ,&
&  0.59001E+00_JPRB, 0.60407E+00_JPRB, 0.61957E+00_JPRB, 0.64990E+00_JPRB, 0.67378E+00_JPRB, 0.67332E+00_JPRB, 0.70028E+00_JPRB ,&
&  0.84666E+00_JPRB, 0.96865E+00_JPRB, 0.10470E+01_JPRB, 0.10876E+01_JPRB, 0.11021E+01_JPRB, 0.10850E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.10508E+01_JPRB, 0.10601E+01_JPRB, 0.10691E+01_JPRB, 0.10810E+01_JPRB, 0.10826E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10865E+01_JPRB, 0.13846E+01_JPRB, 0.12889E+01_JPRB, 0.11961E+01_JPRB, 0.11440E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.98948E+00_JPRB, 0.96733E+00_JPRB, 0.94356E+00_JPRB, 0.87959E+00_JPRB, 0.83093E+00_JPRB, 0.79898E+00_JPRB ,&
&  0.78242E+00_JPRB, 0.76529E+00_JPRB, 0.74898E+00_JPRB, 0.73519E+00_JPRB, 0.71903E+00_JPRB, 0.69928E+00_JPRB, 0.67134E+00_JPRB ,&
&  0.65033E+00_JPRB, 0.64168E+00_JPRB, 0.63735E+00_JPRB, 0.63772E+00_JPRB, 0.63864E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.62276E+00_JPRB, 0.62772E+00_JPRB, 0.63340E+00_JPRB, 0.63754E+00_JPRB, 0.63720E+00_JPRB, 0.63478E+00_JPRB, 0.62965E+00_JPRB ,&
&  0.62491E+00_JPRB, 0.62299E+00_JPRB, 0.62161E+00_JPRB, 0.62112E+00_JPRB, 0.62068E+00_JPRB, 0.61997E+00_JPRB, 0.61915E+00_JPRB ,&
&  0.61961E+00_JPRB, 0.60749E+00_JPRB, 0.60139E+00_JPRB, 0.60212E+00_JPRB, 0.60160E+00_JPRB, 0.60189E+00_JPRB, 0.60264E+00_JPRB ,&
&  0.59734E+00_JPRB, 0.58813E+00_JPRB, 0.58107E+00_JPRB, 0.57210E+00_JPRB, 0.56250E+00_JPRB, 0.56263E+00_JPRB, 0.56082E+00_JPRB ,&
&  0.55047E+00_JPRB, 0.54051E+00_JPRB, 0.54471E+00_JPRB, 0.54539E+00_JPRB, 0.51978E+00_JPRB, 0.49178E+00_JPRB, 0.46101E+00_JPRB ,&
&  0.45400E+00_JPRB, 0.43886E+00_JPRB, 0.43554E+00_JPRB, 0.43592E+00_JPRB, 0.44486E+00_JPRB, 0.45731E+00_JPRB, 0.47905E+00_JPRB ,&
&  0.48934E+00_JPRB, 0.49844E+00_JPRB, 0.51484E+00_JPRB, 0.53036E+00_JPRB, 0.54079E+00_JPRB, 0.55039E+00_JPRB, 0.55449E+00_JPRB ,&
&  0.56367E+00_JPRB, 0.57356E+00_JPRB, 0.58431E+00_JPRB, 0.59640E+00_JPRB, 0.61285E+00_JPRB, 0.62487E+00_JPRB, 0.64099E+00_JPRB ,&
&  0.69790E+00_JPRB, 0.74653E+00_JPRB, 0.78723E+00_JPRB, 0.82421E+00_JPRB, 0.85196E+00_JPRB, 0.86614E+00_JPRB, 0.85238E+00_JPRB ,&
&  0.85056E+00_JPRB, 0.87497E+00_JPRB, 0.89976E+00_JPRB, 0.92439E+00_JPRB, 0.94905E+00_JPRB, 0.97098E+00_JPRB, 0.98061E+00_JPRB ,&
&  0.96874E+00_JPRB, 0.97748E+00_JPRB, 0.10831E+01_JPRB, 0.10976E+01_JPRB, 0.10804E+01_JPRB, 0.10445E+01_JPRB, 0.99621E+00_JPRB ,&
&  0.95617E+00_JPRB, 0.92846E+00_JPRB, 0.90928E+00_JPRB, 0.89166E+00_JPRB, 0.87583E+00_JPRB, 0.86028E+00_JPRB, 0.85380E+00_JPRB ,&
&  0.84026E+00_JPRB, 0.82159E+00_JPRB, 0.79922E+00_JPRB, 0.77726E+00_JPRB, 0.75611E+00_JPRB, 0.73387E+00_JPRB, 0.69796E+00_JPRB ,&
&  0.66031E+00_JPRB, 0.63401E+00_JPRB, 0.62171E+00_JPRB, 0.61722E+00_JPRB, 0.61929E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.61438E+00_JPRB, 0.61439E+00_JPRB, 0.61734E+00_JPRB, 0.61747E+00_JPRB, 0.61835E+00_JPRB, 0.61634E+00_JPRB, 0.61536E+00_JPRB ,&
&  0.61347E+00_JPRB, 0.61328E+00_JPRB, 0.61293E+00_JPRB, 0.61330E+00_JPRB, 0.61342E+00_JPRB, 0.61379E+00_JPRB, 0.61367E+00_JPRB ,&
&  0.61301E+00_JPRB, 0.61197E+00_JPRB, 0.60963E+00_JPRB, 0.60714E+00_JPRB, 0.60500E+00_JPRB, 0.60533E+00_JPRB, 0.60308E+00_JPRB ,&
&  0.59789E+00_JPRB, 0.58809E+00_JPRB, 0.57015E+00_JPRB, 0.56192E+00_JPRB, 0.56071E+00_JPRB, 0.55160E+00_JPRB, 0.55237E+00_JPRB ,&
&  0.54806E+00_JPRB, 0.54659E+00_JPRB, 0.54030E+00_JPRB, 0.53815E+00_JPRB, 0.52186E+00_JPRB, 0.45564E+00_JPRB, 0.44854E+00_JPRB ,&
&  0.44257E+00_JPRB, 0.43425E+00_JPRB, 0.43271E+00_JPRB, 0.43418E+00_JPRB, 0.44040E+00_JPRB, 0.45962E+00_JPRB, 0.47837E+00_JPRB ,&
&  0.48175E+00_JPRB, 0.50939E+00_JPRB, 0.51915E+00_JPRB, 0.52768E+00_JPRB, 0.54263E+00_JPRB, 0.55434E+00_JPRB, 0.55540E+00_JPRB ,&
&  0.55888E+00_JPRB, 0.56285E+00_JPRB, 0.57318E+00_JPRB, 0.58405E+00_JPRB, 0.59357E+00_JPRB, 0.60260E+00_JPRB, 0.61428E+00_JPRB ,&
&  0.62844E+00_JPRB, 0.64275E+00_JPRB, 0.65932E+00_JPRB, 0.67259E+00_JPRB, 0.68550E+00_JPRB, 0.70093E+00_JPRB, 0.71768E+00_JPRB ,&
&  0.73888E+00_JPRB, 0.76007E+00_JPRB, 0.78858E+00_JPRB, 0.80610E+00_JPRB, 0.82875E+00_JPRB, 0.84912E+00_JPRB, 0.87593E+00_JPRB ,&
&  0.89608E+00_JPRB, 0.92636E+00_JPRB, 0.93474E+00_JPRB, 0.93738E+00_JPRB, 0.91965E+00_JPRB, 0.90214E+00_JPRB, 0.89187E+00_JPRB ,&
&  0.88465E+00_JPRB, 0.88904E+00_JPRB, 0.87711E+00_JPRB, 0.86717E+00_JPRB, 0.86056E+00_JPRB, 0.85597E+00_JPRB, 0.85059E+00_JPRB ,&
&  0.84146E+00_JPRB, 0.82754E+00_JPRB, 0.80959E+00_JPRB, 0.78693E+00_JPRB, 0.76631E+00_JPRB, 0.74574E+00_JPRB, 0.71208E+00_JPRB ,&
&  0.67677E+00_JPRB, 0.64303E+00_JPRB, 0.62431E+00_JPRB, 0.61367E+00_JPRB, 0.61215E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.61462E+00_JPRB, 0.61125E+00_JPRB, 0.61196E+00_JPRB, 0.60982E+00_JPRB, 0.61040E+00_JPRB, 0.60899E+00_JPRB, 0.60861E+00_JPRB ,&
&  0.60701E+00_JPRB, 0.60610E+00_JPRB, 0.60461E+00_JPRB, 0.60441E+00_JPRB, 0.60377E+00_JPRB, 0.60421E+00_JPRB, 0.60429E+00_JPRB ,&
&  0.60484E+00_JPRB, 0.60468E+00_JPRB, 0.60491E+00_JPRB, 0.60396E+00_JPRB, 0.60358E+00_JPRB, 0.60242E+00_JPRB, 0.59845E+00_JPRB ,&
&  0.59221E+00_JPRB, 0.58016E+00_JPRB, 0.55548E+00_JPRB, 0.55030E+00_JPRB, 0.54444E+00_JPRB, 0.54368E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.53632E+00_JPRB, 0.49142E+00_JPRB, 0.49452E+00_JPRB, 0.48343E+00_JPRB, 0.47556E+00_JPRB, 0.46780E+00_JPRB, 0.45595E+00_JPRB ,&
&  0.45151E+00_JPRB, 0.44694E+00_JPRB, 0.44620E+00_JPRB, 0.45121E+00_JPRB, 0.45736E+00_JPRB, 0.46806E+00_JPRB, 0.48126E+00_JPRB ,&
&  0.48510E+00_JPRB, 0.50885E+00_JPRB, 0.52003E+00_JPRB, 0.52654E+00_JPRB, 0.54311E+00_JPRB, 0.55575E+00_JPRB, 0.55774E+00_JPRB ,&
&  0.56179E+00_JPRB, 0.56199E+00_JPRB, 0.56849E+00_JPRB, 0.57752E+00_JPRB, 0.58776E+00_JPRB, 0.59241E+00_JPRB, 0.59778E+00_JPRB ,&
&  0.60297E+00_JPRB, 0.60816E+00_JPRB, 0.61385E+00_JPRB, 0.61795E+00_JPRB, 0.62429E+00_JPRB, 0.63557E+00_JPRB, 0.64965E+00_JPRB ,&
&  0.66689E+00_JPRB, 0.68581E+00_JPRB, 0.70633E+00_JPRB, 0.72760E+00_JPRB, 0.74622E+00_JPRB, 0.76364E+00_JPRB, 0.77862E+00_JPRB ,&
&  0.79318E+00_JPRB, 0.80443E+00_JPRB, 0.81078E+00_JPRB, 0.81615E+00_JPRB, 0.81472E+00_JPRB, 0.81015E+00_JPRB, 0.80948E+00_JPRB ,&
&  0.81569E+00_JPRB, 0.81796E+00_JPRB, 0.81854E+00_JPRB, 0.81729E+00_JPRB, 0.81739E+00_JPRB, 0.81554E+00_JPRB, 0.81182E+00_JPRB ,&
&  0.80594E+00_JPRB, 0.79671E+00_JPRB, 0.78451E+00_JPRB, 0.76622E+00_JPRB, 0.74697E+00_JPRB, 0.73191E+00_JPRB, 0.71303E+00_JPRB ,&
&  0.69167E+00_JPRB, 0.66617E+00_JPRB, 0.64339E+00_JPRB, 0.62609E+00_JPRB, 0.61838E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.62295E+00_JPRB, 0.62030E+00_JPRB, 0.62018E+00_JPRB, 0.61913E+00_JPRB, 0.61951E+00_JPRB, 0.61812E+00_JPRB, 0.61794E+00_JPRB ,&
&  0.61594E+00_JPRB, 0.61478E+00_JPRB, 0.61321E+00_JPRB, 0.61193E+00_JPRB, 0.61062E+00_JPRB, 0.60920E+00_JPRB, 0.60806E+00_JPRB ,&
&  0.60758E+00_JPRB, 0.60647E+00_JPRB, 0.60668E+00_JPRB, 0.60585E+00_JPRB, 0.60430E+00_JPRB, 0.60225E+00_JPRB, 0.59496E+00_JPRB ,&
&  0.58680E+00_JPRB, 0.57888E+00_JPRB, 0.54369E+00_JPRB, 0.53937E+00_JPRB, 0.53181E+00_JPRB, 0.52748E+00_JPRB, 0.52804E+00_JPRB ,&
&  0.52370E+00_JPRB, 0.51889E+00_JPRB, 0.50480E+00_JPRB, 0.49035E+00_JPRB, 0.47874E+00_JPRB, 0.46464E+00_JPRB, 0.46127E+00_JPRB ,&
&  0.45781E+00_JPRB, 0.46064E+00_JPRB, 0.46443E+00_JPRB, 0.46860E+00_JPRB, 0.47854E+00_JPRB, 0.48628E+00_JPRB, 0.49775E+00_JPRB ,&
&  0.51175E+00_JPRB, 0.52271E+00_JPRB, 0.52729E+00_JPRB, 0.52617E+00_JPRB, 0.54196E+00_JPRB, 0.55249E+00_JPRB, 0.56125E+00_JPRB ,&
&  0.56615E+00_JPRB, 0.56721E+00_JPRB, 0.57131E+00_JPRB, 0.57319E+00_JPRB, 0.57714E+00_JPRB, 0.58340E+00_JPRB, 0.58811E+00_JPRB ,&
&  0.59310E+00_JPRB, 0.59601E+00_JPRB, 0.59797E+00_JPRB, 0.59876E+00_JPRB, 0.59764E+00_JPRB, 0.59805E+00_JPRB, 0.60111E+00_JPRB ,&
&  0.60717E+00_JPRB, 0.61669E+00_JPRB, 0.62878E+00_JPRB, 0.64282E+00_JPRB, 0.65689E+00_JPRB, 0.67008E+00_JPRB, 0.68071E+00_JPRB ,&
&  0.69029E+00_JPRB, 0.69757E+00_JPRB, 0.70735E+00_JPRB, 0.70920E+00_JPRB, 0.71555E+00_JPRB, 0.73046E+00_JPRB, 0.73370E+00_JPRB ,&
&  0.73879E+00_JPRB, 0.74538E+00_JPRB, 0.75046E+00_JPRB, 0.75406E+00_JPRB, 0.75619E+00_JPRB, 0.75657E+00_JPRB, 0.75592E+00_JPRB ,&
&  0.75378E+00_JPRB, 0.74770E+00_JPRB, 0.73927E+00_JPRB, 0.72857E+00_JPRB, 0.71866E+00_JPRB, 0.70852E+00_JPRB, 0.69697E+00_JPRB ,&
&  0.68414E+00_JPRB, 0.67018E+00_JPRB, 0.65387E+00_JPRB, 0.63807E+00_JPRB, 0.62880E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.63474E+00_JPRB, 0.63052E+00_JPRB, 0.62749E+00_JPRB, 0.62559E+00_JPRB, 0.62430E+00_JPRB, 0.62288E+00_JPRB, 0.62200E+00_JPRB ,&
&  0.62053E+00_JPRB, 0.61875E+00_JPRB, 0.61630E+00_JPRB, 0.61499E+00_JPRB, 0.61353E+00_JPRB, 0.61254E+00_JPRB, 0.61044E+00_JPRB ,&
&  0.60884E+00_JPRB, 0.60628E+00_JPRB, 0.60521E+00_JPRB, 0.60385E+00_JPRB, 0.60237E+00_JPRB, 0.60097E+00_JPRB, 0.59462E+00_JPRB ,&
&  0.58853E+00_JPRB, 0.58344E+00_JPRB, 0.57622E+00_JPRB, 0.56976E+00_JPRB, 0.56514E+00_JPRB, 0.53513E+00_JPRB, 0.52557E+00_JPRB ,&
&  0.53201E+00_JPRB, 0.53689E+00_JPRB, 0.54248E+00_JPRB, 0.54697E+00_JPRB, 0.54142E+00_JPRB, 0.52890E+00_JPRB, 0.52119E+00_JPRB ,&
&  0.49626E+00_JPRB, 0.50012E+00_JPRB, 0.50322E+00_JPRB, 0.50626E+00_JPRB, 0.51219E+00_JPRB, 0.51540E+00_JPRB, 0.52247E+00_JPRB ,&
&  0.54008E+00_JPRB, 0.54406E+00_JPRB, 0.54926E+00_JPRB, 0.55349E+00_JPRB, 0.55501E+00_JPRB, 0.55716E+00_JPRB, 0.56216E+00_JPRB ,&
&  0.56442E+00_JPRB, 0.56661E+00_JPRB, 0.56841E+00_JPRB, 0.56893E+00_JPRB, 0.57103E+00_JPRB, 0.57268E+00_JPRB, 0.57398E+00_JPRB ,&
&  0.57381E+00_JPRB, 0.57449E+00_JPRB, 0.57532E+00_JPRB, 0.57620E+00_JPRB, 0.57751E+00_JPRB, 0.57738E+00_JPRB, 0.57804E+00_JPRB ,&
&  0.57954E+00_JPRB, 0.58265E+00_JPRB, 0.58654E+00_JPRB, 0.59186E+00_JPRB, 0.59864E+00_JPRB, 0.60791E+00_JPRB, 0.61752E+00_JPRB ,&
&  0.62865E+00_JPRB, 0.63915E+00_JPRB, 0.64877E+00_JPRB, 0.65677E+00_JPRB, 0.66366E+00_JPRB, 0.67036E+00_JPRB, 0.67590E+00_JPRB ,&
&  0.68216E+00_JPRB, 0.68833E+00_JPRB, 0.69200E+00_JPRB, 0.69317E+00_JPRB, 0.69648E+00_JPRB, 0.69939E+00_JPRB, 0.70033E+00_JPRB ,&
&  0.70021E+00_JPRB, 0.69930E+00_JPRB, 0.69697E+00_JPRB, 0.69304E+00_JPRB, 0.68930E+00_JPRB, 0.68485E+00_JPRB, 0.67944E+00_JPRB ,&
&  0.67368E+00_JPRB, 0.66782E+00_JPRB, 0.66016E+00_JPRB, 0.65107E+00_JPRB, 0.64232E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.64201E+00_JPRB, 0.63789E+00_JPRB, 0.63402E+00_JPRB, 0.63063E+00_JPRB, 0.62717E+00_JPRB, 0.62466E+00_JPRB, 0.62236E+00_JPRB ,&
&  0.62025E+00_JPRB, 0.61798E+00_JPRB, 0.61572E+00_JPRB, 0.61389E+00_JPRB, 0.61258E+00_JPRB, 0.61155E+00_JPRB, 0.60989E+00_JPRB ,&
&  0.60834E+00_JPRB, 0.60646E+00_JPRB, 0.60486E+00_JPRB, 0.60324E+00_JPRB, 0.60117E+00_JPRB, 0.59872E+00_JPRB, 0.59577E+00_JPRB ,&
&  0.59341E+00_JPRB, 0.59077E+00_JPRB, 0.58798E+00_JPRB, 0.58409E+00_JPRB, 0.58083E+00_JPRB, 0.57751E+00_JPRB, 0.57416E+00_JPRB ,&
&  0.57044E+00_JPRB, 0.56748E+00_JPRB, 0.56489E+00_JPRB, 0.56246E+00_JPRB, 0.56417E+00_JPRB, 0.56518E+00_JPRB, 0.56565E+00_JPRB ,&
&  0.56532E+00_JPRB, 0.56299E+00_JPRB, 0.56212E+00_JPRB, 0.56146E+00_JPRB, 0.56088E+00_JPRB, 0.56317E+00_JPRB, 0.56470E+00_JPRB ,&
&  0.56564E+00_JPRB, 0.56587E+00_JPRB, 0.56734E+00_JPRB, 0.56805E+00_JPRB, 0.56802E+00_JPRB, 0.56713E+00_JPRB, 0.56572E+00_JPRB ,&
&  0.56574E+00_JPRB, 0.56573E+00_JPRB, 0.56550E+00_JPRB, 0.56474E+00_JPRB, 0.56510E+00_JPRB, 0.56565E+00_JPRB, 0.56580E+00_JPRB ,&
&  0.56536E+00_JPRB, 0.56554E+00_JPRB, 0.56566E+00_JPRB, 0.56581E+00_JPRB, 0.56592E+00_JPRB, 0.56675E+00_JPRB, 0.56784E+00_JPRB ,&
&  0.56927E+00_JPRB, 0.57125E+00_JPRB, 0.57375E+00_JPRB, 0.57661E+00_JPRB, 0.57997E+00_JPRB, 0.58444E+00_JPRB, 0.59001E+00_JPRB ,&
&  0.59671E+00_JPRB, 0.60429E+00_JPRB, 0.61230E+00_JPRB, 0.61873E+00_JPRB, 0.62402E+00_JPRB, 0.62871E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.63814E+00_JPRB, 0.64201E+00_JPRB, 0.64591E+00_JPRB, 0.64970E+00_JPRB, 0.65364E+00_JPRB, 0.65661E+00_JPRB, 0.65873E+00_JPRB ,&
&  0.66013E+00_JPRB, 0.66122E+00_JPRB, 0.66232E+00_JPRB, 0.66353E+00_JPRB, 0.66421E+00_JPRB, 0.66352E+00_JPRB, 0.66206E+00_JPRB ,&
&  0.66014E+00_JPRB, 0.65792E+00_JPRB, 0.65501E+00_JPRB, 0.65094E+00_JPRB, 0.64662E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.63105E+00_JPRB, 0.62963E+00_JPRB, 0.62812E+00_JPRB, 0.62661E+00_JPRB, 0.62494E+00_JPRB, 0.62334E+00_JPRB, 0.62167E+00_JPRB ,&
&  0.62023E+00_JPRB, 0.61866E+00_JPRB, 0.61728E+00_JPRB, 0.61591E+00_JPRB, 0.61476E+00_JPRB, 0.61354E+00_JPRB, 0.61240E+00_JPRB ,&
&  0.61118E+00_JPRB, 0.61009E+00_JPRB, 0.60858E+00_JPRB, 0.60729E+00_JPRB, 0.60579E+00_JPRB, 0.60441E+00_JPRB, 0.60267E+00_JPRB ,&
&  0.60132E+00_JPRB, 0.59967E+00_JPRB, 0.59818E+00_JPRB, 0.59616E+00_JPRB, 0.59460E+00_JPRB, 0.59297E+00_JPRB, 0.59153E+00_JPRB ,&
&  0.59027E+00_JPRB, 0.58939E+00_JPRB, 0.58853E+00_JPRB, 0.58758E+00_JPRB, 0.58713E+00_JPRB, 0.58722E+00_JPRB, 0.58735E+00_JPRB ,&
&  0.58742E+00_JPRB, 0.58771E+00_JPRB, 0.58817E+00_JPRB, 0.58839E+00_JPRB, 0.58802E+00_JPRB, 0.58768E+00_JPRB, 0.58743E+00_JPRB ,&
&  0.58697E+00_JPRB, 0.58617E+00_JPRB, 0.58528E+00_JPRB, 0.58452E+00_JPRB, 0.58359E+00_JPRB, 0.58243E+00_JPRB, 0.58146E+00_JPRB ,&
&  0.58117E+00_JPRB, 0.58092E+00_JPRB, 0.58054E+00_JPRB, 0.58011E+00_JPRB, 0.58007E+00_JPRB, 0.58011E+00_JPRB, 0.58014E+00_JPRB ,&
&  0.58023E+00_JPRB, 0.58054E+00_JPRB, 0.58065E+00_JPRB, 0.58075E+00_JPRB, 0.58097E+00_JPRB, 0.58168E+00_JPRB, 0.58260E+00_JPRB ,&
&  0.58372E+00_JPRB, 0.58520E+00_JPRB, 0.58708E+00_JPRB, 0.58896E+00_JPRB, 0.59098E+00_JPRB, 0.59324E+00_JPRB, 0.59569E+00_JPRB ,&
&  0.59824E+00_JPRB, 0.60092E+00_JPRB, 0.60391E+00_JPRB, 0.60690E+00_JPRB, 0.60981E+00_JPRB, 0.61268E+00_JPRB, 0.61582E+00_JPRB ,&
&  0.61845E+00_JPRB, 0.62076E+00_JPRB, 0.62276E+00_JPRB, 0.62478E+00_JPRB, 0.62676E+00_JPRB, 0.62855E+00_JPRB, 0.63004E+00_JPRB ,&
&  0.63125E+00_JPRB, 0.63229E+00_JPRB, 0.63320E+00_JPRB, 0.63419E+00_JPRB, 0.63444E+00_JPRB, 0.63448E+00_JPRB, 0.63447E+00_JPRB ,&
&  0.63436E+00_JPRB, 0.63399E+00_JPRB, 0.63346E+00_JPRB, 0.63289E+00_JPRB, 0.63217E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB, 0.60628E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB ,&
&  0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB, 0.16595E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.17974E+00_JPRB, 0.17998E+00_JPRB, 0.18050E+00_JPRB, 0.18111E+00_JPRB, 0.18173E+00_JPRB, 0.18253E+00_JPRB, 0.18282E+00_JPRB ,&
&  0.18292E+00_JPRB, 0.18288E+00_JPRB, 0.18321E+00_JPRB, 0.18360E+00_JPRB, 0.18385E+00_JPRB, 0.18384E+00_JPRB, 0.18346E+00_JPRB ,&
&  0.18280E+00_JPRB, 0.18204E+00_JPRB, 0.18093E+00_JPRB, 0.18043E+00_JPRB, 0.18005E+00_JPRB, 0.17965E+00_JPRB, 0.17887E+00_JPRB ,&
&  0.17819E+00_JPRB, 0.17698E+00_JPRB, 0.17530E+00_JPRB, 0.17393E+00_JPRB, 0.17361E+00_JPRB, 0.17375E+00_JPRB, 0.17424E+00_JPRB ,&
&  0.17462E+00_JPRB, 0.17485E+00_JPRB, 0.17490E+00_JPRB, 0.17501E+00_JPRB, 0.17519E+00_JPRB, 0.17504E+00_JPRB, 0.17477E+00_JPRB ,&
&  0.17444E+00_JPRB, 0.17392E+00_JPRB, 0.17343E+00_JPRB, 0.17298E+00_JPRB, 0.17256E+00_JPRB, 0.17208E+00_JPRB, 0.17152E+00_JPRB ,&
&  0.17088E+00_JPRB, 0.17008E+00_JPRB, 0.16915E+00_JPRB, 0.16840E+00_JPRB, 0.16762E+00_JPRB, 0.16700E+00_JPRB, 0.16625E+00_JPRB ,&
&  0.16563E+00_JPRB, 0.16506E+00_JPRB, 0.16447E+00_JPRB, 0.16393E+00_JPRB, 0.16341E+00_JPRB, 0.16291E+00_JPRB, 0.16242E+00_JPRB ,&
&  0.16194E+00_JPRB, 0.16155E+00_JPRB, 0.16112E+00_JPRB, 0.16071E+00_JPRB, 0.16043E+00_JPRB, 0.16020E+00_JPRB, 0.16000E+00_JPRB ,&
&  0.15982E+00_JPRB, 0.15976E+00_JPRB, 0.15974E+00_JPRB, 0.15971E+00_JPRB, 0.15970E+00_JPRB, 0.15980E+00_JPRB, 0.15991E+00_JPRB ,&
&  0.16007E+00_JPRB, 0.16018E+00_JPRB, 0.16031E+00_JPRB, 0.16049E+00_JPRB, 0.16070E+00_JPRB, 0.16093E+00_JPRB, 0.16126E+00_JPRB ,&
&  0.16165E+00_JPRB, 0.16212E+00_JPRB, 0.16262E+00_JPRB, 0.16295E+00_JPRB, 0.16361E+00_JPRB, 0.16434E+00_JPRB, 0.16519E+00_JPRB ,&
&  0.16613E+00_JPRB, 0.16722E+00_JPRB, 0.16838E+00_JPRB, 0.16970E+00_JPRB, 0.17104E+00_JPRB, 0.17192E+00_JPRB, 0.17281E+00_JPRB ,&
&  0.17375E+00_JPRB, 0.17483E+00_JPRB, 0.17587E+00_JPRB, 0.17694E+00_JPRB, 0.17824E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.20143E+00_JPRB, 0.20834E+00_JPRB, 0.21535E+00_JPRB, 0.22530E+00_JPRB, 0.23283E+00_JPRB, 0.26815E+00_JPRB, 0.26304E+00_JPRB ,&
&  0.25949E+00_JPRB, 0.24539E+00_JPRB, 0.24369E+00_JPRB, 0.23999E+00_JPRB, 0.23544E+00_JPRB, 0.23047E+00_JPRB, 0.22550E+00_JPRB ,&
&  0.22083E+00_JPRB, 0.21537E+00_JPRB, 0.21062E+00_JPRB, 0.20691E+00_JPRB, 0.20358E+00_JPRB, 0.20033E+00_JPRB, 0.19747E+00_JPRB ,&
&  0.19619E+00_JPRB, 0.19598E+00_JPRB, 0.19516E+00_JPRB, 0.19129E+00_JPRB, 0.18794E+00_JPRB, 0.18585E+00_JPRB, 0.18517E+00_JPRB ,&
&  0.18507E+00_JPRB, 0.18655E+00_JPRB, 0.18739E+00_JPRB, 0.18804E+00_JPRB, 0.18905E+00_JPRB, 0.18822E+00_JPRB, 0.18740E+00_JPRB ,&
&  0.18625E+00_JPRB, 0.18495E+00_JPRB, 0.18358E+00_JPRB, 0.18239E+00_JPRB, 0.18120E+00_JPRB, 0.17945E+00_JPRB, 0.17790E+00_JPRB ,&
&  0.17636E+00_JPRB, 0.17444E+00_JPRB, 0.17196E+00_JPRB, 0.17031E+00_JPRB, 0.16841E+00_JPRB, 0.16679E+00_JPRB, 0.16518E+00_JPRB ,&
&  0.16400E+00_JPRB, 0.16290E+00_JPRB, 0.16159E+00_JPRB, 0.16000E+00_JPRB, 0.15842E+00_JPRB, 0.15701E+00_JPRB, 0.15566E+00_JPRB ,&
&  0.15491E+00_JPRB, 0.15414E+00_JPRB, 0.15338E+00_JPRB, 0.15257E+00_JPRB, 0.15217E+00_JPRB, 0.15176E+00_JPRB, 0.15140E+00_JPRB ,&
&  0.15109E+00_JPRB, 0.15094E+00_JPRB, 0.15099E+00_JPRB, 0.15108E+00_JPRB, 0.15118E+00_JPRB, 0.15154E+00_JPRB, 0.15197E+00_JPRB ,&
&  0.15255E+00_JPRB, 0.15327E+00_JPRB, 0.15423E+00_JPRB, 0.15524E+00_JPRB, 0.15614E+00_JPRB, 0.15689E+00_JPRB, 0.15786E+00_JPRB ,&
&  0.15942E+00_JPRB, 0.16123E+00_JPRB, 0.16321E+00_JPRB, 0.16477E+00_JPRB, 0.16688E+00_JPRB, 0.16880E+00_JPRB, 0.17071E+00_JPRB ,&
&  0.17260E+00_JPRB, 0.17480E+00_JPRB, 0.17720E+00_JPRB, 0.17976E+00_JPRB, 0.18267E+00_JPRB, 0.18535E+00_JPRB, 0.18783E+00_JPRB ,&
&  0.19036E+00_JPRB, 0.19384E+00_JPRB, 0.19592E+00_JPRB, 0.19723E+00_JPRB, 0.19896E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.27457E+00_JPRB, 0.27933E+00_JPRB, 0.28300E+00_JPRB, 0.28533E+00_JPRB, 0.28749E+00_JPRB, 0.28905E+00_JPRB, 0.28960E+00_JPRB ,&
&  0.28317E+00_JPRB, 0.27815E+00_JPRB, 0.27123E+00_JPRB, 0.26296E+00_JPRB, 0.25544E+00_JPRB, 0.24879E+00_JPRB, 0.24095E+00_JPRB ,&
&  0.23454E+00_JPRB, 0.22849E+00_JPRB, 0.22206E+00_JPRB, 0.21535E+00_JPRB, 0.20832E+00_JPRB, 0.20123E+00_JPRB, 0.19757E+00_JPRB ,&
&  0.19639E+00_JPRB, 0.19726E+00_JPRB, 0.19897E+00_JPRB, 0.20051E+00_JPRB, 0.20146E+00_JPRB, 0.20213E+00_JPRB, 0.20102E+00_JPRB ,&
&  0.19742E+00_JPRB, 0.19797E+00_JPRB, 0.19873E+00_JPRB, 0.19920E+00_JPRB, 0.20060E+00_JPRB, 0.20185E+00_JPRB, 0.20161E+00_JPRB ,&
&  0.19988E+00_JPRB, 0.19754E+00_JPRB, 0.19431E+00_JPRB, 0.19131E+00_JPRB, 0.19007E+00_JPRB, 0.18783E+00_JPRB, 0.18530E+00_JPRB ,&
&  0.18313E+00_JPRB, 0.17927E+00_JPRB, 0.17545E+00_JPRB, 0.17275E+00_JPRB, 0.16935E+00_JPRB, 0.16700E+00_JPRB, 0.16395E+00_JPRB ,&
&  0.16147E+00_JPRB, 0.15919E+00_JPRB, 0.15668E+00_JPRB, 0.15471E+00_JPRB, 0.15286E+00_JPRB, 0.15148E+00_JPRB, 0.15011E+00_JPRB ,&
&  0.14931E+00_JPRB, 0.14857E+00_JPRB, 0.14811E+00_JPRB, 0.14760E+00_JPRB, 0.14732E+00_JPRB, 0.14704E+00_JPRB, 0.14682E+00_JPRB ,&
&  0.14670E+00_JPRB, 0.14702E+00_JPRB, 0.14725E+00_JPRB, 0.14765E+00_JPRB, 0.14812E+00_JPRB, 0.14844E+00_JPRB, 0.14898E+00_JPRB ,&
&  0.14998E+00_JPRB, 0.15106E+00_JPRB, 0.15181E+00_JPRB, 0.15276E+00_JPRB, 0.15382E+00_JPRB, 0.15512E+00_JPRB, 0.15687E+00_JPRB ,&
&  0.15870E+00_JPRB, 0.16075E+00_JPRB, 0.16304E+00_JPRB, 0.16532E+00_JPRB, 0.16822E+00_JPRB, 0.17119E+00_JPRB, 0.17431E+00_JPRB ,&
&  0.17801E+00_JPRB, 0.18194E+00_JPRB, 0.18596E+00_JPRB, 0.19038E+00_JPRB, 0.19677E+00_JPRB, 0.20284E+00_JPRB, 0.21039E+00_JPRB ,&
&  0.21783E+00_JPRB, 0.22415E+00_JPRB, 0.25589E+00_JPRB, 0.26072E+00_JPRB, 0.26739E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.28167E+00_JPRB, 0.29241E+00_JPRB, 0.30117E+00_JPRB, 0.30665E+00_JPRB, 0.31233E+00_JPRB, 0.31458E+00_JPRB, 0.32306E+00_JPRB ,&
&  0.31757E+00_JPRB, 0.29939E+00_JPRB, 0.28821E+00_JPRB, 0.27182E+00_JPRB, 0.25153E+00_JPRB, 0.23505E+00_JPRB, 0.22685E+00_JPRB ,&
&  0.22202E+00_JPRB, 0.21826E+00_JPRB, 0.21365E+00_JPRB, 0.20860E+00_JPRB, 0.20347E+00_JPRB, 0.19969E+00_JPRB, 0.19720E+00_JPRB ,&
&  0.19498E+00_JPRB, 0.19547E+00_JPRB, 0.19785E+00_JPRB, 0.20200E+00_JPRB, 0.20786E+00_JPRB, 0.21214E+00_JPRB, 0.21546E+00_JPRB ,&
&  0.21800E+00_JPRB, 0.20957E+00_JPRB, 0.21075E+00_JPRB, 0.21233E+00_JPRB, 0.21089E+00_JPRB, 0.21341E+00_JPRB, 0.21733E+00_JPRB ,&
&  0.21539E+00_JPRB, 0.20953E+00_JPRB, 0.21242E+00_JPRB, 0.20816E+00_JPRB, 0.19989E+00_JPRB, 0.19774E+00_JPRB, 0.19195E+00_JPRB ,&
&  0.18858E+00_JPRB, 0.18140E+00_JPRB, 0.17632E+00_JPRB, 0.17297E+00_JPRB, 0.16789E+00_JPRB, 0.16576E+00_JPRB, 0.16251E+00_JPRB ,&
&  0.15911E+00_JPRB, 0.15666E+00_JPRB, 0.15376E+00_JPRB, 0.15226E+00_JPRB, 0.15098E+00_JPRB, 0.14980E+00_JPRB, 0.14887E+00_JPRB ,&
&  0.14836E+00_JPRB, 0.14788E+00_JPRB, 0.14792E+00_JPRB, 0.14797E+00_JPRB, 0.14804E+00_JPRB, 0.14840E+00_JPRB, 0.14865E+00_JPRB ,&
&  0.14928E+00_JPRB, 0.14998E+00_JPRB, 0.15055E+00_JPRB, 0.15145E+00_JPRB, 0.15188E+00_JPRB, 0.15242E+00_JPRB, 0.15284E+00_JPRB ,&
&  0.15330E+00_JPRB, 0.15398E+00_JPRB, 0.15438E+00_JPRB, 0.15507E+00_JPRB, 0.15587E+00_JPRB, 0.15735E+00_JPRB, 0.15883E+00_JPRB ,&
&  0.16061E+00_JPRB, 0.16265E+00_JPRB, 0.16484E+00_JPRB, 0.16716E+00_JPRB, 0.17001E+00_JPRB, 0.17266E+00_JPRB, 0.17612E+00_JPRB ,&
&  0.17986E+00_JPRB, 0.18340E+00_JPRB, 0.18743E+00_JPRB, 0.19261E+00_JPRB, 0.19941E+00_JPRB, 0.20889E+00_JPRB, 0.21794E+00_JPRB ,&
&  0.22833E+00_JPRB, 0.25491E+00_JPRB, 0.26130E+00_JPRB, 0.26486E+00_JPRB, 0.27322E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.30823E+00_JPRB, 0.31763E+00_JPRB, 0.33167E+00_JPRB, 0.34068E+00_JPRB, 0.34796E+00_JPRB, 0.34382E+00_JPRB, 0.33991E+00_JPRB ,&
&  0.33584E+00_JPRB, 0.32164E+00_JPRB, 0.30451E+00_JPRB, 0.27530E+00_JPRB, 0.25067E+00_JPRB, 0.23183E+00_JPRB, 0.22155E+00_JPRB ,&
&  0.21551E+00_JPRB, 0.21358E+00_JPRB, 0.21183E+00_JPRB, 0.21148E+00_JPRB, 0.21047E+00_JPRB, 0.21220E+00_JPRB, 0.21367E+00_JPRB ,&
&  0.21337E+00_JPRB, 0.21227E+00_JPRB, 0.21160E+00_JPRB, 0.21468E+00_JPRB, 0.22062E+00_JPRB, 0.22529E+00_JPRB, 0.22978E+00_JPRB ,&
&  0.23312E+00_JPRB, 0.23066E+00_JPRB, 0.23171E+00_JPRB, 0.22966E+00_JPRB, 0.22730E+00_JPRB, 0.22726E+00_JPRB, 0.22798E+00_JPRB ,&
&  0.22844E+00_JPRB, 0.22911E+00_JPRB, 0.22850E+00_JPRB, 0.22570E+00_JPRB, 0.21427E+00_JPRB, 0.20830E+00_JPRB, 0.20070E+00_JPRB ,&
&  0.19266E+00_JPRB, 0.18332E+00_JPRB, 0.17589E+00_JPRB, 0.17226E+00_JPRB, 0.16810E+00_JPRB, 0.16640E+00_JPRB, 0.16328E+00_JPRB ,&
&  0.15973E+00_JPRB, 0.15634E+00_JPRB, 0.15398E+00_JPRB, 0.15277E+00_JPRB, 0.15231E+00_JPRB, 0.15135E+00_JPRB, 0.15098E+00_JPRB ,&
&  0.15119E+00_JPRB, 0.15147E+00_JPRB, 0.15224E+00_JPRB, 0.15303E+00_JPRB, 0.15393E+00_JPRB, 0.15477E+00_JPRB, 0.15580E+00_JPRB ,&
&  0.15739E+00_JPRB, 0.15928E+00_JPRB, 0.16014E+00_JPRB, 0.16087E+00_JPRB, 0.16065E+00_JPRB, 0.16026E+00_JPRB, 0.15963E+00_JPRB ,&
&  0.15912E+00_JPRB, 0.15899E+00_JPRB, 0.15899E+00_JPRB, 0.15895E+00_JPRB, 0.15942E+00_JPRB, 0.16031E+00_JPRB, 0.16127E+00_JPRB ,&
&  0.16281E+00_JPRB, 0.16422E+00_JPRB, 0.16580E+00_JPRB, 0.16762E+00_JPRB, 0.16974E+00_JPRB, 0.17222E+00_JPRB, 0.17502E+00_JPRB ,&
&  0.17759E+00_JPRB, 0.18081E+00_JPRB, 0.18353E+00_JPRB, 0.18568E+00_JPRB, 0.19046E+00_JPRB, 0.19738E+00_JPRB, 0.21116E+00_JPRB ,&
&  0.22738E+00_JPRB, 0.26787E+00_JPRB, 0.31055E+00_JPRB, 0.29548E+00_JPRB, 0.30026E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.30737E+00_JPRB, 0.31933E+00_JPRB, 0.33461E+00_JPRB, 0.34401E+00_JPRB, 0.34916E+00_JPRB, 0.35174E+00_JPRB, 0.35050E+00_JPRB ,&
&  0.34709E+00_JPRB, 0.34662E+00_JPRB, 0.35775E+00_JPRB, 0.34012E+00_JPRB, 0.32232E+00_JPRB, 0.28886E+00_JPRB, 0.27243E+00_JPRB ,&
&  0.26024E+00_JPRB, 0.25817E+00_JPRB, 0.25804E+00_JPRB, 0.26282E+00_JPRB, 0.26683E+00_JPRB, 0.27002E+00_JPRB, 0.26652E+00_JPRB ,&
&  0.26140E+00_JPRB, 0.25232E+00_JPRB, 0.24469E+00_JPRB, 0.23748E+00_JPRB, 0.23753E+00_JPRB, 0.23645E+00_JPRB, 0.23388E+00_JPRB ,&
&  0.23087E+00_JPRB, 0.22889E+00_JPRB, 0.22869E+00_JPRB, 0.22871E+00_JPRB, 0.24040E+00_JPRB, 0.23989E+00_JPRB, 0.23966E+00_JPRB ,&
&  0.23919E+00_JPRB, 0.23915E+00_JPRB, 0.23821E+00_JPRB, 0.23649E+00_JPRB, 0.23190E+00_JPRB, 0.21996E+00_JPRB, 0.21968E+00_JPRB ,&
&  0.20996E+00_JPRB, 0.19926E+00_JPRB, 0.18905E+00_JPRB, 0.18193E+00_JPRB, 0.17760E+00_JPRB, 0.17557E+00_JPRB, 0.17090E+00_JPRB ,&
&  0.16596E+00_JPRB, 0.16035E+00_JPRB, 0.15762E+00_JPRB, 0.15723E+00_JPRB, 0.15771E+00_JPRB, 0.15825E+00_JPRB, 0.15872E+00_JPRB ,&
&  0.15986E+00_JPRB, 0.16044E+00_JPRB, 0.16075E+00_JPRB, 0.16137E+00_JPRB, 0.16199E+00_JPRB, 0.16296E+00_JPRB, 0.16463E+00_JPRB ,&
&  0.16762E+00_JPRB, 0.17093E+00_JPRB, 0.17397E+00_JPRB, 0.17632E+00_JPRB, 0.17537E+00_JPRB, 0.17233E+00_JPRB, 0.16822E+00_JPRB ,&
&  0.16528E+00_JPRB, 0.16437E+00_JPRB, 0.16354E+00_JPRB, 0.16355E+00_JPRB, 0.16379E+00_JPRB, 0.16427E+00_JPRB, 0.16492E+00_JPRB ,&
&  0.16575E+00_JPRB, 0.16644E+00_JPRB, 0.16728E+00_JPRB, 0.16836E+00_JPRB, 0.16991E+00_JPRB, 0.17132E+00_JPRB, 0.17331E+00_JPRB ,&
&  0.17628E+00_JPRB, 0.17953E+00_JPRB, 0.18412E+00_JPRB, 0.18819E+00_JPRB, 0.19351E+00_JPRB, 0.20148E+00_JPRB, 0.21464E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.26365E+00_JPRB, 0.27982E+00_JPRB, 0.28998E+00_JPRB, 0.29713E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.30554E+00_JPRB, 0.31437E+00_JPRB, 0.32647E+00_JPRB, 0.33683E+00_JPRB, 0.34573E+00_JPRB, 0.35421E+00_JPRB, 0.36146E+00_JPRB ,&
&  0.36521E+00_JPRB, 0.36737E+00_JPRB, 0.36812E+00_JPRB, 0.37154E+00_JPRB, 0.36858E+00_JPRB, 0.36202E+00_JPRB, 0.35660E+00_JPRB ,&
&  0.35216E+00_JPRB, 0.34863E+00_JPRB, 0.34343E+00_JPRB, 0.34070E+00_JPRB, 0.34049E+00_JPRB, 0.34066E+00_JPRB, 0.33509E+00_JPRB ,&
&  0.32486E+00_JPRB, 0.32424E+00_JPRB, 0.32910E+00_JPRB, 0.32394E+00_JPRB, 0.31842E+00_JPRB, 0.31322E+00_JPRB, 0.30748E+00_JPRB ,&
&  0.29203E+00_JPRB, 0.27232E+00_JPRB, 0.24516E+00_JPRB, 0.24037E+00_JPRB, 0.25630E+00_JPRB, 0.25042E+00_JPRB, 0.25321E+00_JPRB ,&
&  0.25357E+00_JPRB, 0.25399E+00_JPRB, 0.25253E+00_JPRB, 0.25110E+00_JPRB, 0.24763E+00_JPRB, 0.24637E+00_JPRB, 0.24507E+00_JPRB ,&
&  0.23992E+00_JPRB, 0.23517E+00_JPRB, 0.22756E+00_JPRB, 0.21014E+00_JPRB, 0.20456E+00_JPRB, 0.20234E+00_JPRB, 0.19718E+00_JPRB ,&
&  0.18947E+00_JPRB, 0.18333E+00_JPRB, 0.18059E+00_JPRB, 0.17718E+00_JPRB, 0.17923E+00_JPRB, 0.18085E+00_JPRB, 0.18083E+00_JPRB ,&
&  0.18080E+00_JPRB, 0.18041E+00_JPRB, 0.17812E+00_JPRB, 0.17436E+00_JPRB, 0.17141E+00_JPRB, 0.17030E+00_JPRB, 0.17080E+00_JPRB ,&
&  0.17298E+00_JPRB, 0.17744E+00_JPRB, 0.18792E+00_JPRB, 0.19620E+00_JPRB, 0.19542E+00_JPRB, 0.19077E+00_JPRB, 0.18204E+00_JPRB ,&
&  0.17495E+00_JPRB, 0.17221E+00_JPRB, 0.16991E+00_JPRB, 0.16877E+00_JPRB, 0.16918E+00_JPRB, 0.17064E+00_JPRB, 0.17154E+00_JPRB ,&
&  0.17260E+00_JPRB, 0.17359E+00_JPRB, 0.17443E+00_JPRB, 0.17498E+00_JPRB, 0.17586E+00_JPRB, 0.17686E+00_JPRB, 0.17867E+00_JPRB ,&
&  0.18135E+00_JPRB, 0.18567E+00_JPRB, 0.19138E+00_JPRB, 0.19747E+00_JPRB, 0.20184E+00_JPRB, 0.20394E+00_JPRB, 0.20745E+00_JPRB ,&
&  0.21316E+00_JPRB, 0.22755E+00_JPRB, 0.25569E+00_JPRB, 0.30126E+00_JPRB, 0.30552E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.30955E+00_JPRB, 0.31382E+00_JPRB, 0.31658E+00_JPRB, 0.32010E+00_JPRB, 0.32769E+00_JPRB, 0.33847E+00_JPRB, 0.35063E+00_JPRB ,&
&  0.36168E+00_JPRB, 0.37050E+00_JPRB, 0.37478E+00_JPRB, 0.37987E+00_JPRB, 0.38479E+00_JPRB, 0.38719E+00_JPRB, 0.39244E+00_JPRB ,&
&  0.39321E+00_JPRB, 0.38988E+00_JPRB, 0.38385E+00_JPRB, 0.37583E+00_JPRB, 0.37163E+00_JPRB, 0.37300E+00_JPRB, 0.37602E+00_JPRB ,&
&  0.37805E+00_JPRB, 0.37963E+00_JPRB, 0.37980E+00_JPRB, 0.37648E+00_JPRB, 0.37326E+00_JPRB, 0.37137E+00_JPRB, 0.36751E+00_JPRB ,&
&  0.35905E+00_JPRB, 0.30982E+00_JPRB, 0.27208E+00_JPRB, 0.26741E+00_JPRB, 0.28454E+00_JPRB, 0.28696E+00_JPRB, 0.28161E+00_JPRB ,&
&  0.27579E+00_JPRB, 0.27119E+00_JPRB, 0.26859E+00_JPRB, 0.26653E+00_JPRB, 0.26395E+00_JPRB, 0.26177E+00_JPRB, 0.26280E+00_JPRB ,&
&  0.26422E+00_JPRB, 0.26210E+00_JPRB, 0.25911E+00_JPRB, 0.25286E+00_JPRB, 0.24820E+00_JPRB, 0.24589E+00_JPRB, 0.24401E+00_JPRB ,&
&  0.23972E+00_JPRB, 0.23552E+00_JPRB, 0.23512E+00_JPRB, 0.23600E+00_JPRB, 0.23755E+00_JPRB, 0.23863E+00_JPRB, 0.23604E+00_JPRB ,&
&  0.22959E+00_JPRB, 0.22316E+00_JPRB, 0.21486E+00_JPRB, 0.20504E+00_JPRB, 0.19560E+00_JPRB, 0.18904E+00_JPRB, 0.18396E+00_JPRB ,&
&  0.18283E+00_JPRB, 0.18778E+00_JPRB, 0.20011E+00_JPRB, 0.21558E+00_JPRB, 0.23635E+00_JPRB, 0.22460E+00_JPRB, 0.21318E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.19476E+00_JPRB, 0.18877E+00_JPRB, 0.18600E+00_JPRB, 0.18497E+00_JPRB, 0.18509E+00_JPRB, 0.18623E+00_JPRB ,&
&  0.18823E+00_JPRB, 0.19090E+00_JPRB, 0.19334E+00_JPRB, 0.19453E+00_JPRB, 0.19524E+00_JPRB, 0.19586E+00_JPRB, 0.19589E+00_JPRB ,&
&  0.19525E+00_JPRB, 0.19743E+00_JPRB, 0.20400E+00_JPRB, 0.21244E+00_JPRB, 0.22078E+00_JPRB, 0.22845E+00_JPRB, 0.24137E+00_JPRB ,&
&  0.25719E+00_JPRB, 0.27734E+00_JPRB, 0.28920E+00_JPRB, 0.29813E+00_JPRB, 0.30599E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.33500E+00_JPRB, 0.33645E+00_JPRB, 0.33391E+00_JPRB, 0.33104E+00_JPRB, 0.33066E+00_JPRB, 0.33248E+00_JPRB, 0.33838E+00_JPRB ,&
&  0.34815E+00_JPRB, 0.36053E+00_JPRB, 0.37236E+00_JPRB, 0.38224E+00_JPRB, 0.39123E+00_JPRB, 0.39522E+00_JPRB, 0.40129E+00_JPRB ,&
&  0.40776E+00_JPRB, 0.41368E+00_JPRB, 0.41448E+00_JPRB, 0.40427E+00_JPRB, 0.39433E+00_JPRB, 0.38905E+00_JPRB, 0.38907E+00_JPRB ,&
&  0.39093E+00_JPRB, 0.39368E+00_JPRB, 0.39997E+00_JPRB, 0.40241E+00_JPRB, 0.39664E+00_JPRB, 0.39530E+00_JPRB, 0.39322E+00_JPRB ,&
&  0.38780E+00_JPRB, 0.37689E+00_JPRB, 0.34477E+00_JPRB, 0.33952E+00_JPRB, 0.33152E+00_JPRB, 0.32258E+00_JPRB, 0.31352E+00_JPRB ,&
&  0.30329E+00_JPRB, 0.29312E+00_JPRB, 0.28623E+00_JPRB, 0.28227E+00_JPRB, 0.28312E+00_JPRB, 0.28442E+00_JPRB, 0.28402E+00_JPRB ,&
&  0.28622E+00_JPRB, 0.28917E+00_JPRB, 0.29158E+00_JPRB, 0.29186E+00_JPRB, 0.29120E+00_JPRB, 0.28971E+00_JPRB, 0.28477E+00_JPRB ,&
&  0.28032E+00_JPRB, 0.27781E+00_JPRB, 0.27580E+00_JPRB, 0.27400E+00_JPRB, 0.27349E+00_JPRB, 0.27376E+00_JPRB, 0.27376E+00_JPRB ,&
&  0.27156E+00_JPRB, 0.26786E+00_JPRB, 0.26371E+00_JPRB, 0.25683E+00_JPRB, 0.24542E+00_JPRB, 0.22961E+00_JPRB, 0.22016E+00_JPRB ,&
&  0.21850E+00_JPRB, 0.22945E+00_JPRB, 0.25122E+00_JPRB, 0.25934E+00_JPRB, 0.26661E+00_JPRB, 0.27066E+00_JPRB, 0.27535E+00_JPRB ,&
&  0.27164E+00_JPRB, 0.24697E+00_JPRB, 0.23358E+00_JPRB, 0.22679E+00_JPRB, 0.22116E+00_JPRB, 0.21666E+00_JPRB, 0.21648E+00_JPRB ,&
&  0.21959E+00_JPRB, 0.22623E+00_JPRB, 0.23262E+00_JPRB, 0.23573E+00_JPRB, 0.23511E+00_JPRB, 0.23168E+00_JPRB, 0.22830E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22955E+00_JPRB, 0.24420E+00_JPRB, 0.26732E+00_JPRB, 0.27539E+00_JPRB, 0.28155E+00_JPRB, 0.28730E+00_JPRB ,&
&  0.29343E+00_JPRB, 0.30271E+00_JPRB, 0.31409E+00_JPRB, 0.32317E+00_JPRB, 0.33123E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.35119E+00_JPRB, 0.35098E+00_JPRB, 0.35245E+00_JPRB, 0.35137E+00_JPRB, 0.34883E+00_JPRB, 0.34706E+00_JPRB, 0.35086E+00_JPRB ,&
&  0.35785E+00_JPRB, 0.36938E+00_JPRB, 0.38462E+00_JPRB, 0.39799E+00_JPRB, 0.40612E+00_JPRB, 0.41118E+00_JPRB, 0.41890E+00_JPRB ,&
&  0.42605E+00_JPRB, 0.43401E+00_JPRB, 0.43420E+00_JPRB, 0.42535E+00_JPRB, 0.41783E+00_JPRB, 0.41384E+00_JPRB, 0.41326E+00_JPRB ,&
&  0.41259E+00_JPRB, 0.41440E+00_JPRB, 0.41767E+00_JPRB, 0.42120E+00_JPRB, 0.42067E+00_JPRB, 0.41896E+00_JPRB, 0.42274E+00_JPRB ,&
&  0.42103E+00_JPRB, 0.41174E+00_JPRB, 0.40242E+00_JPRB, 0.37768E+00_JPRB, 0.37733E+00_JPRB, 0.36724E+00_JPRB, 0.35940E+00_JPRB ,&
&  0.34886E+00_JPRB, 0.33592E+00_JPRB, 0.32657E+00_JPRB, 0.32060E+00_JPRB, 0.31797E+00_JPRB, 0.31546E+00_JPRB, 0.31354E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.31226E+00_JPRB, 0.31380E+00_JPRB, 0.31180E+00_JPRB, 0.31064E+00_JPRB, 0.31135E+00_JPRB, 0.31280E+00_JPRB ,&
&  0.31224E+00_JPRB, 0.31154E+00_JPRB, 0.31098E+00_JPRB, 0.30972E+00_JPRB, 0.30788E+00_JPRB, 0.30532E+00_JPRB, 0.30287E+00_JPRB ,&
&  0.30173E+00_JPRB, 0.30253E+00_JPRB, 0.30265E+00_JPRB, 0.29873E+00_JPRB, 0.29105E+00_JPRB, 0.28205E+00_JPRB, 0.27613E+00_JPRB ,&
&  0.27346E+00_JPRB, 0.27535E+00_JPRB, 0.28299E+00_JPRB, 0.29402E+00_JPRB, 0.30437E+00_JPRB, 0.31046E+00_JPRB, 0.31415E+00_JPRB ,&
&  0.31863E+00_JPRB, 0.31720E+00_JPRB, 0.31365E+00_JPRB, 0.31040E+00_JPRB, 0.30738E+00_JPRB, 0.30339E+00_JPRB, 0.30330E+00_JPRB ,&
&  0.30575E+00_JPRB, 0.30808E+00_JPRB, 0.30871E+00_JPRB, 0.30747E+00_JPRB, 0.30534E+00_JPRB, 0.30098E+00_JPRB, 0.29488E+00_JPRB ,&
&  0.28743E+00_JPRB, 0.27883E+00_JPRB, 0.27667E+00_JPRB, 0.27710E+00_JPRB, 0.28090E+00_JPRB, 0.28876E+00_JPRB, 0.29944E+00_JPRB ,&
&  0.31069E+00_JPRB, 0.32007E+00_JPRB, 0.33013E+00_JPRB, 0.33942E+00_JPRB, 0.34766E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.37705E+00_JPRB, 0.37903E+00_JPRB, 0.38143E+00_JPRB, 0.38451E+00_JPRB, 0.38793E+00_JPRB, 0.39009E+00_JPRB, 0.39411E+00_JPRB ,&
&  0.39639E+00_JPRB, 0.40277E+00_JPRB, 0.41331E+00_JPRB, 0.42397E+00_JPRB, 0.43183E+00_JPRB, 0.43858E+00_JPRB, 0.44511E+00_JPRB ,&
&  0.45042E+00_JPRB, 0.45746E+00_JPRB, 0.45833E+00_JPRB, 0.45584E+00_JPRB, 0.45307E+00_JPRB, 0.45090E+00_JPRB, 0.44894E+00_JPRB ,&
&  0.44637E+00_JPRB, 0.44618E+00_JPRB, 0.44994E+00_JPRB, 0.45422E+00_JPRB, 0.45868E+00_JPRB, 0.46008E+00_JPRB, 0.45901E+00_JPRB ,&
&  0.46303E+00_JPRB, 0.45398E+00_JPRB, 0.44823E+00_JPRB, 0.44139E+00_JPRB, 0.43533E+00_JPRB, 0.42412E+00_JPRB, 0.41590E+00_JPRB ,&
&  0.40617E+00_JPRB, 0.39771E+00_JPRB, 0.39417E+00_JPRB, 0.39037E+00_JPRB, 0.38079E+00_JPRB, 0.36959E+00_JPRB, 0.36126E+00_JPRB ,&
&  0.35583E+00_JPRB, 0.35049E+00_JPRB, 0.34424E+00_JPRB, 0.33911E+00_JPRB, 0.33471E+00_JPRB, 0.33310E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.33869E+00_JPRB, 0.34216E+00_JPRB, 0.34402E+00_JPRB, 0.34427E+00_JPRB, 0.34256E+00_JPRB, 0.33984E+00_JPRB, 0.33815E+00_JPRB ,&
&  0.33849E+00_JPRB, 0.33965E+00_JPRB, 0.34176E+00_JPRB, 0.34688E+00_JPRB, 0.34746E+00_JPRB, 0.34587E+00_JPRB, 0.34018E+00_JPRB ,&
&  0.33727E+00_JPRB, 0.33569E+00_JPRB, 0.33712E+00_JPRB, 0.34209E+00_JPRB, 0.35037E+00_JPRB, 0.35840E+00_JPRB, 0.36420E+00_JPRB ,&
&  0.36714E+00_JPRB, 0.36904E+00_JPRB, 0.37220E+00_JPRB, 0.37582E+00_JPRB, 0.37719E+00_JPRB, 0.37639E+00_JPRB, 0.37476E+00_JPRB ,&
&  0.37740E+00_JPRB, 0.38466E+00_JPRB, 0.39232E+00_JPRB, 0.39705E+00_JPRB, 0.39925E+00_JPRB, 0.39806E+00_JPRB, 0.39114E+00_JPRB ,&
&  0.37870E+00_JPRB, 0.36446E+00_JPRB, 0.35066E+00_JPRB, 0.33893E+00_JPRB, 0.33079E+00_JPRB, 0.32699E+00_JPRB, 0.32992E+00_JPRB ,&
&  0.33775E+00_JPRB, 0.34778E+00_JPRB, 0.35710E+00_JPRB, 0.36409E+00_JPRB, 0.37172E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.40307E+00_JPRB, 0.40332E+00_JPRB, 0.40643E+00_JPRB, 0.41298E+00_JPRB, 0.41734E+00_JPRB, 0.42133E+00_JPRB, 0.42914E+00_JPRB ,&
&  0.43619E+00_JPRB, 0.44237E+00_JPRB, 0.44982E+00_JPRB, 0.45806E+00_JPRB, 0.46453E+00_JPRB, 0.47370E+00_JPRB, 0.48232E+00_JPRB ,&
&  0.48784E+00_JPRB, 0.49002E+00_JPRB, 0.49120E+00_JPRB, 0.49219E+00_JPRB, 0.49185E+00_JPRB, 0.48916E+00_JPRB, 0.48855E+00_JPRB ,&
&  0.48664E+00_JPRB, 0.48457E+00_JPRB, 0.48671E+00_JPRB, 0.48709E+00_JPRB, 0.48892E+00_JPRB, 0.49198E+00_JPRB, 0.49249E+00_JPRB ,&
&  0.49088E+00_JPRB, 0.47837E+00_JPRB, 0.47405E+00_JPRB, 0.47148E+00_JPRB, 0.47601E+00_JPRB, 0.47737E+00_JPRB, 0.47620E+00_JPRB ,&
&  0.47012E+00_JPRB, 0.46653E+00_JPRB, 0.46858E+00_JPRB, 0.47107E+00_JPRB, 0.46544E+00_JPRB, 0.45262E+00_JPRB, 0.44239E+00_JPRB ,&
&  0.43442E+00_JPRB, 0.42254E+00_JPRB, 0.40545E+00_JPRB, 0.39540E+00_JPRB, 0.39112E+00_JPRB, 0.38617E+00_JPRB, 0.38140E+00_JPRB ,&
&  0.37878E+00_JPRB, 0.37788E+00_JPRB, 0.37924E+00_JPRB, 0.37825E+00_JPRB, 0.37441E+00_JPRB, 0.36972E+00_JPRB, 0.36780E+00_JPRB ,&
&  0.37021E+00_JPRB, 0.37233E+00_JPRB, 0.37565E+00_JPRB, 0.37813E+00_JPRB, 0.37745E+00_JPRB, 0.37777E+00_JPRB, 0.38127E+00_JPRB ,&
&  0.38651E+00_JPRB, 0.39134E+00_JPRB, 0.39660E+00_JPRB, 0.39877E+00_JPRB, 0.40167E+00_JPRB, 0.40814E+00_JPRB, 0.40853E+00_JPRB ,&
&  0.40710E+00_JPRB, 0.40916E+00_JPRB, 0.41328E+00_JPRB, 0.42097E+00_JPRB, 0.43278E+00_JPRB, 0.44059E+00_JPRB, 0.44175E+00_JPRB ,&
&  0.44182E+00_JPRB, 0.44405E+00_JPRB, 0.44773E+00_JPRB, 0.45667E+00_JPRB, 0.46644E+00_JPRB, 0.47352E+00_JPRB, 0.47654E+00_JPRB ,&
&  0.47359E+00_JPRB, 0.46705E+00_JPRB, 0.45512E+00_JPRB, 0.44115E+00_JPRB, 0.42860E+00_JPRB, 0.41880E+00_JPRB, 0.41076E+00_JPRB ,&
&  0.40279E+00_JPRB, 0.39936E+00_JPRB, 0.39908E+00_JPRB, 0.40106E+00_JPRB, 0.40232E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.43761E+00_JPRB, 0.43864E+00_JPRB, 0.44362E+00_JPRB, 0.44859E+00_JPRB, 0.45216E+00_JPRB, 0.45717E+00_JPRB, 0.46677E+00_JPRB ,&
&  0.47645E+00_JPRB, 0.48575E+00_JPRB, 0.49216E+00_JPRB, 0.49993E+00_JPRB, 0.51086E+00_JPRB, 0.51824E+00_JPRB, 0.52410E+00_JPRB ,&
&  0.52655E+00_JPRB, 0.52681E+00_JPRB, 0.52760E+00_JPRB, 0.53051E+00_JPRB, 0.52991E+00_JPRB, 0.52618E+00_JPRB, 0.52478E+00_JPRB ,&
&  0.52459E+00_JPRB, 0.52289E+00_JPRB, 0.52269E+00_JPRB, 0.52394E+00_JPRB, 0.52193E+00_JPRB, 0.51852E+00_JPRB, 0.51847E+00_JPRB ,&
&  0.51922E+00_JPRB, 0.50602E+00_JPRB, 0.49891E+00_JPRB, 0.49233E+00_JPRB, 0.49632E+00_JPRB, 0.50805E+00_JPRB, 0.52084E+00_JPRB ,&
&  0.52859E+00_JPRB, 0.53385E+00_JPRB, 0.53362E+00_JPRB, 0.53278E+00_JPRB, 0.52829E+00_JPRB, 0.51708E+00_JPRB, 0.50757E+00_JPRB ,&
&  0.50139E+00_JPRB, 0.49260E+00_JPRB, 0.47082E+00_JPRB, 0.45829E+00_JPRB, 0.45511E+00_JPRB, 0.45775E+00_JPRB, 0.46146E+00_JPRB ,&
&  0.46299E+00_JPRB, 0.46051E+00_JPRB, 0.45452E+00_JPRB, 0.44720E+00_JPRB, 0.43716E+00_JPRB, 0.42549E+00_JPRB, 0.41529E+00_JPRB ,&
&  0.40934E+00_JPRB, 0.40483E+00_JPRB, 0.40110E+00_JPRB, 0.40339E+00_JPRB, 0.40260E+00_JPRB, 0.39874E+00_JPRB, 0.40040E+00_JPRB ,&
&  0.41118E+00_JPRB, 0.42471E+00_JPRB, 0.43726E+00_JPRB, 0.44415E+00_JPRB, 0.44690E+00_JPRB, 0.45010E+00_JPRB, 0.45397E+00_JPRB ,&
&  0.45978E+00_JPRB, 0.46567E+00_JPRB, 0.47060E+00_JPRB, 0.47674E+00_JPRB, 0.48448E+00_JPRB, 0.49160E+00_JPRB, 0.49463E+00_JPRB ,&
&  0.49724E+00_JPRB, 0.49872E+00_JPRB, 0.49960E+00_JPRB, 0.50434E+00_JPRB, 0.51189E+00_JPRB, 0.51730E+00_JPRB, 0.52181E+00_JPRB ,&
&  0.52657E+00_JPRB, 0.52888E+00_JPRB, 0.52769E+00_JPRB, 0.52393E+00_JPRB, 0.51850E+00_JPRB, 0.50756E+00_JPRB, 0.49407E+00_JPRB ,&
&  0.47849E+00_JPRB, 0.46426E+00_JPRB, 0.45383E+00_JPRB, 0.44491E+00_JPRB, 0.43892E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.48425E+00_JPRB, 0.48241E+00_JPRB, 0.48303E+00_JPRB, 0.48633E+00_JPRB, 0.48998E+00_JPRB, 0.49284E+00_JPRB, 0.49870E+00_JPRB ,&
&  0.50866E+00_JPRB, 0.51647E+00_JPRB, 0.52595E+00_JPRB, 0.53563E+00_JPRB, 0.54298E+00_JPRB, 0.54796E+00_JPRB, 0.55064E+00_JPRB ,&
&  0.54891E+00_JPRB, 0.54613E+00_JPRB, 0.54874E+00_JPRB, 0.55316E+00_JPRB, 0.55442E+00_JPRB, 0.55388E+00_JPRB, 0.55381E+00_JPRB ,&
&  0.55329E+00_JPRB, 0.55108E+00_JPRB, 0.54834E+00_JPRB, 0.54434E+00_JPRB, 0.53961E+00_JPRB, 0.53764E+00_JPRB, 0.53578E+00_JPRB ,&
&  0.52961E+00_JPRB, 0.52959E+00_JPRB, 0.47983E+00_JPRB, 0.49535E+00_JPRB, 0.50850E+00_JPRB, 0.52890E+00_JPRB, 0.54868E+00_JPRB ,&
&  0.56284E+00_JPRB, 0.57189E+00_JPRB, 0.57647E+00_JPRB, 0.57506E+00_JPRB, 0.56308E+00_JPRB, 0.55569E+00_JPRB, 0.54729E+00_JPRB ,&
&  0.53689E+00_JPRB, 0.52959E+00_JPRB, 0.51707E+00_JPRB, 0.50337E+00_JPRB, 0.49882E+00_JPRB, 0.49932E+00_JPRB, 0.49915E+00_JPRB ,&
&  0.49544E+00_JPRB, 0.49200E+00_JPRB, 0.49335E+00_JPRB, 0.49426E+00_JPRB, 0.49388E+00_JPRB, 0.48836E+00_JPRB, 0.48138E+00_JPRB ,&
&  0.47062E+00_JPRB, 0.46006E+00_JPRB, 0.45273E+00_JPRB, 0.44724E+00_JPRB, 0.44237E+00_JPRB, 0.43698E+00_JPRB, 0.43160E+00_JPRB ,&
&  0.43460E+00_JPRB, 0.44522E+00_JPRB, 0.45520E+00_JPRB, 0.46349E+00_JPRB, 0.46896E+00_JPRB, 0.47269E+00_JPRB, 0.47788E+00_JPRB ,&
&  0.48391E+00_JPRB, 0.49401E+00_JPRB, 0.50084E+00_JPRB, 0.50691E+00_JPRB, 0.51152E+00_JPRB, 0.51590E+00_JPRB, 0.52100E+00_JPRB ,&
&  0.52512E+00_JPRB, 0.53163E+00_JPRB, 0.53621E+00_JPRB, 0.54184E+00_JPRB, 0.54497E+00_JPRB, 0.54967E+00_JPRB, 0.55556E+00_JPRB ,&
&  0.56223E+00_JPRB, 0.56822E+00_JPRB, 0.57068E+00_JPRB, 0.56935E+00_JPRB, 0.56211E+00_JPRB, 0.55236E+00_JPRB, 0.54304E+00_JPRB ,&
&  0.53105E+00_JPRB, 0.51688E+00_JPRB, 0.50272E+00_JPRB, 0.49381E+00_JPRB, 0.48655E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.54754E+00_JPRB, 0.54269E+00_JPRB, 0.53741E+00_JPRB, 0.53666E+00_JPRB, 0.53561E+00_JPRB, 0.53757E+00_JPRB, 0.54051E+00_JPRB ,&
&  0.54732E+00_JPRB, 0.55170E+00_JPRB, 0.56086E+00_JPRB, 0.56828E+00_JPRB, 0.57187E+00_JPRB, 0.57610E+00_JPRB, 0.57879E+00_JPRB ,&
&  0.57981E+00_JPRB, 0.58114E+00_JPRB, 0.58170E+00_JPRB, 0.58581E+00_JPRB, 0.58639E+00_JPRB, 0.58552E+00_JPRB, 0.58447E+00_JPRB ,&
&  0.58264E+00_JPRB, 0.58071E+00_JPRB, 0.57598E+00_JPRB, 0.56934E+00_JPRB, 0.56095E+00_JPRB, 0.55761E+00_JPRB, 0.55188E+00_JPRB ,&
&  0.55334E+00_JPRB, 0.52252E+00_JPRB, 0.53439E+00_JPRB, 0.53307E+00_JPRB, 0.54778E+00_JPRB, 0.57080E+00_JPRB, 0.60255E+00_JPRB ,&
&  0.62457E+00_JPRB, 0.63624E+00_JPRB, 0.63851E+00_JPRB, 0.63744E+00_JPRB, 0.62452E+00_JPRB, 0.61112E+00_JPRB, 0.59569E+00_JPRB ,&
&  0.57779E+00_JPRB, 0.56760E+00_JPRB, 0.55427E+00_JPRB, 0.54441E+00_JPRB, 0.54043E+00_JPRB, 0.53751E+00_JPRB, 0.53263E+00_JPRB ,&
&  0.52526E+00_JPRB, 0.51884E+00_JPRB, 0.51923E+00_JPRB, 0.52252E+00_JPRB, 0.52587E+00_JPRB, 0.52780E+00_JPRB, 0.52636E+00_JPRB ,&
&  0.51978E+00_JPRB, 0.51395E+00_JPRB, 0.51096E+00_JPRB, 0.51024E+00_JPRB, 0.50552E+00_JPRB, 0.49744E+00_JPRB, 0.48908E+00_JPRB ,&
&  0.48685E+00_JPRB, 0.48933E+00_JPRB, 0.49286E+00_JPRB, 0.49750E+00_JPRB, 0.50100E+00_JPRB, 0.50531E+00_JPRB, 0.50738E+00_JPRB ,&
&  0.51212E+00_JPRB, 0.51668E+00_JPRB, 0.52592E+00_JPRB, 0.53480E+00_JPRB, 0.53718E+00_JPRB, 0.54585E+00_JPRB, 0.55473E+00_JPRB ,&
&  0.56334E+00_JPRB, 0.57314E+00_JPRB, 0.58045E+00_JPRB, 0.58506E+00_JPRB, 0.58992E+00_JPRB, 0.59271E+00_JPRB, 0.59803E+00_JPRB ,&
&  0.60537E+00_JPRB, 0.60700E+00_JPRB, 0.61219E+00_JPRB, 0.61264E+00_JPRB, 0.60765E+00_JPRB, 0.59948E+00_JPRB, 0.58930E+00_JPRB ,&
&  0.57817E+00_JPRB, 0.56306E+00_JPRB, 0.55539E+00_JPRB, 0.55102E+00_JPRB, 0.54895E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.59744E+00_JPRB, 0.59076E+00_JPRB, 0.57961E+00_JPRB, 0.57305E+00_JPRB, 0.57025E+00_JPRB, 0.57322E+00_JPRB, 0.57744E+00_JPRB ,&
&  0.58211E+00_JPRB, 0.58392E+00_JPRB, 0.59134E+00_JPRB, 0.59888E+00_JPRB, 0.60331E+00_JPRB, 0.60623E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.60500E+00_JPRB, 0.60468E+00_JPRB, 0.59995E+00_JPRB, 0.59604E+00_JPRB, 0.59062E+00_JPRB, 0.59108E+00_JPRB, 0.59099E+00_JPRB ,&
&  0.58985E+00_JPRB, 0.58707E+00_JPRB, 0.58123E+00_JPRB, 0.57311E+00_JPRB, 0.56968E+00_JPRB, 0.56952E+00_JPRB, 0.56752E+00_JPRB ,&
&  0.57409E+00_JPRB, 0.49564E+00_JPRB, 0.53231E+00_JPRB, 0.54811E+00_JPRB, 0.57184E+00_JPRB, 0.60234E+00_JPRB, 0.63449E+00_JPRB ,&
&  0.66573E+00_JPRB, 0.69138E+00_JPRB, 0.70068E+00_JPRB, 0.70167E+00_JPRB, 0.69109E+00_JPRB, 0.67202E+00_JPRB, 0.64916E+00_JPRB ,&
&  0.62896E+00_JPRB, 0.61338E+00_JPRB, 0.59515E+00_JPRB, 0.58443E+00_JPRB, 0.57587E+00_JPRB, 0.56493E+00_JPRB, 0.55564E+00_JPRB ,&
&  0.55159E+00_JPRB, 0.54912E+00_JPRB, 0.54874E+00_JPRB, 0.55025E+00_JPRB, 0.55283E+00_JPRB, 0.55451E+00_JPRB, 0.55390E+00_JPRB ,&
&  0.55247E+00_JPRB, 0.54994E+00_JPRB, 0.54578E+00_JPRB, 0.54392E+00_JPRB, 0.54307E+00_JPRB, 0.54435E+00_JPRB, 0.54045E+00_JPRB ,&
&  0.53389E+00_JPRB, 0.53507E+00_JPRB, 0.53807E+00_JPRB, 0.53991E+00_JPRB, 0.54269E+00_JPRB, 0.54566E+00_JPRB, 0.54584E+00_JPRB ,&
&  0.54599E+00_JPRB, 0.54982E+00_JPRB, 0.55402E+00_JPRB, 0.56088E+00_JPRB, 0.56575E+00_JPRB, 0.57489E+00_JPRB, 0.58245E+00_JPRB ,&
&  0.58967E+00_JPRB, 0.60066E+00_JPRB, 0.61081E+00_JPRB, 0.61736E+00_JPRB, 0.62126E+00_JPRB, 0.62549E+00_JPRB, 0.63023E+00_JPRB ,&
&  0.63380E+00_JPRB, 0.63686E+00_JPRB, 0.63962E+00_JPRB, 0.64060E+00_JPRB, 0.63690E+00_JPRB, 0.63263E+00_JPRB, 0.62990E+00_JPRB ,&
&  0.62153E+00_JPRB, 0.60999E+00_JPRB, 0.60587E+00_JPRB, 0.60264E+00_JPRB, 0.59911E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.67235E+00_JPRB, 0.66380E+00_JPRB, 0.64923E+00_JPRB, 0.63667E+00_JPRB, 0.62631E+00_JPRB, 0.62257E+00_JPRB, 0.62063E+00_JPRB ,&
&  0.62558E+00_JPRB, 0.63390E+00_JPRB, 0.64142E+00_JPRB, 0.64506E+00_JPRB, 0.64729E+00_JPRB, 0.64653E+00_JPRB, 0.64235E+00_JPRB ,&
&  0.63883E+00_JPRB, 0.62707E+00_JPRB, 0.61822E+00_JPRB, 0.61797E+00_JPRB, 0.61465E+00_JPRB, 0.61099E+00_JPRB, 0.60688E+00_JPRB ,&
&  0.60314E+00_JPRB, 0.60050E+00_JPRB, 0.59238E+00_JPRB, 0.58892E+00_JPRB, 0.58844E+00_JPRB, 0.59095E+00_JPRB, 0.60143E+00_JPRB ,&
&  0.52792E+00_JPRB, 0.45196E+00_JPRB, 0.51054E+00_JPRB, 0.57713E+00_JPRB, 0.61451E+00_JPRB, 0.65624E+00_JPRB, 0.69299E+00_JPRB ,&
&  0.74153E+00_JPRB, 0.77325E+00_JPRB, 0.79122E+00_JPRB, 0.78578E+00_JPRB, 0.76682E+00_JPRB, 0.74479E+00_JPRB, 0.72167E+00_JPRB ,&
&  0.69416E+00_JPRB, 0.66345E+00_JPRB, 0.64352E+00_JPRB, 0.63329E+00_JPRB, 0.62461E+00_JPRB, 0.61532E+00_JPRB, 0.60968E+00_JPRB ,&
&  0.60764E+00_JPRB, 0.60636E+00_JPRB, 0.60455E+00_JPRB, 0.60348E+00_JPRB, 0.60539E+00_JPRB, 0.60749E+00_JPRB, 0.60994E+00_JPRB ,&
&  0.61023E+00_JPRB, 0.60911E+00_JPRB, 0.60615E+00_JPRB, 0.59974E+00_JPRB, 0.59465E+00_JPRB, 0.59502E+00_JPRB, 0.59326E+00_JPRB ,&
&  0.59242E+00_JPRB, 0.59092E+00_JPRB, 0.59063E+00_JPRB, 0.59096E+00_JPRB, 0.58996E+00_JPRB, 0.58920E+00_JPRB, 0.59104E+00_JPRB ,&
&  0.59595E+00_JPRB, 0.59593E+00_JPRB, 0.59875E+00_JPRB, 0.60409E+00_JPRB, 0.60966E+00_JPRB, 0.61491E+00_JPRB, 0.61822E+00_JPRB ,&
&  0.62418E+00_JPRB, 0.63637E+00_JPRB, 0.64517E+00_JPRB, 0.65075E+00_JPRB, 0.65533E+00_JPRB, 0.66075E+00_JPRB, 0.66856E+00_JPRB ,&
&  0.67493E+00_JPRB, 0.68067E+00_JPRB, 0.68749E+00_JPRB, 0.70247E+00_JPRB, 0.70941E+00_JPRB, 0.70900E+00_JPRB, 0.70466E+00_JPRB ,&
&  0.69442E+00_JPRB, 0.67815E+00_JPRB, 0.67901E+00_JPRB, 0.67614E+00_JPRB, 0.67362E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.71054E+00_JPRB, 0.70777E+00_JPRB, 0.69626E+00_JPRB, 0.68229E+00_JPRB, 0.67227E+00_JPRB, 0.66881E+00_JPRB, 0.66830E+00_JPRB ,&
&  0.67278E+00_JPRB, 0.67636E+00_JPRB, 0.67650E+00_JPRB, 0.67215E+00_JPRB, 0.66558E+00_JPRB, 0.65587E+00_JPRB, 0.64794E+00_JPRB ,&
&  0.63426E+00_JPRB, 0.61974E+00_JPRB, 0.61490E+00_JPRB, 0.61473E+00_JPRB, 0.61246E+00_JPRB, 0.60622E+00_JPRB, 0.60333E+00_JPRB ,&
&  0.60295E+00_JPRB, 0.60615E+00_JPRB, 0.60098E+00_JPRB, 0.59802E+00_JPRB, 0.59390E+00_JPRB, 0.59623E+00_JPRB, 0.59785E+00_JPRB ,&
&  0.59786E+00_JPRB, 0.47888E+00_JPRB, 0.55471E+00_JPRB, 0.61854E+00_JPRB, 0.64744E+00_JPRB, 0.70213E+00_JPRB, 0.75942E+00_JPRB ,&
&  0.80168E+00_JPRB, 0.82208E+00_JPRB, 0.81689E+00_JPRB, 0.79912E+00_JPRB, 0.77072E+00_JPRB, 0.74243E+00_JPRB, 0.71718E+00_JPRB ,&
&  0.69343E+00_JPRB, 0.67613E+00_JPRB, 0.66222E+00_JPRB, 0.64979E+00_JPRB, 0.64223E+00_JPRB, 0.63498E+00_JPRB, 0.62864E+00_JPRB ,&
&  0.62896E+00_JPRB, 0.63554E+00_JPRB, 0.63650E+00_JPRB, 0.63934E+00_JPRB, 0.64332E+00_JPRB, 0.64567E+00_JPRB, 0.65444E+00_JPRB ,&
&  0.65987E+00_JPRB, 0.65968E+00_JPRB, 0.65909E+00_JPRB, 0.65527E+00_JPRB, 0.64534E+00_JPRB, 0.64108E+00_JPRB, 0.63564E+00_JPRB ,&
&  0.63081E+00_JPRB, 0.62279E+00_JPRB, 0.61631E+00_JPRB, 0.61380E+00_JPRB, 0.61391E+00_JPRB, 0.61397E+00_JPRB, 0.61998E+00_JPRB ,&
&  0.63018E+00_JPRB, 0.63272E+00_JPRB, 0.63485E+00_JPRB, 0.63946E+00_JPRB, 0.64332E+00_JPRB, 0.64585E+00_JPRB, 0.65008E+00_JPRB ,&
&  0.65971E+00_JPRB, 0.67513E+00_JPRB, 0.68924E+00_JPRB, 0.69524E+00_JPRB, 0.69760E+00_JPRB, 0.70549E+00_JPRB, 0.71452E+00_JPRB ,&
&  0.72400E+00_JPRB, 0.72801E+00_JPRB, 0.73002E+00_JPRB, 0.76704E+00_JPRB, 0.78231E+00_JPRB, 0.78047E+00_JPRB, 0.77253E+00_JPRB ,&
&  0.75244E+00_JPRB, 0.74576E+00_JPRB, 0.60318E+00_JPRB, 0.66854E+00_JPRB, 0.69008E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.76680E+00_JPRB, 0.76433E+00_JPRB, 0.75019E+00_JPRB, 0.73358E+00_JPRB, 0.72678E+00_JPRB, 0.72440E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.71576E+00_JPRB, 0.70484E+00_JPRB, 0.69227E+00_JPRB, 0.67786E+00_JPRB, 0.66293E+00_JPRB, 0.65066E+00_JPRB, 0.63884E+00_JPRB ,&
&  0.61984E+00_JPRB, 0.60913E+00_JPRB, 0.60875E+00_JPRB, 0.60586E+00_JPRB, 0.60326E+00_JPRB, 0.59959E+00_JPRB, 0.60099E+00_JPRB ,&
&  0.60479E+00_JPRB, 0.60634E+00_JPRB, 0.60685E+00_JPRB, 0.60093E+00_JPRB, 0.59650E+00_JPRB, 0.59693E+00_JPRB, 0.59333E+00_JPRB ,&
&  0.59440E+00_JPRB, 0.50059E+00_JPRB, 0.58142E+00_JPRB, 0.67753E+00_JPRB, 0.72486E+00_JPRB, 0.79068E+00_JPRB, 0.87134E+00_JPRB ,&
&  0.91044E+00_JPRB, 0.89145E+00_JPRB, 0.85532E+00_JPRB, 0.82099E+00_JPRB, 0.78457E+00_JPRB, 0.75150E+00_JPRB, 0.72448E+00_JPRB ,&
&  0.70712E+00_JPRB, 0.69633E+00_JPRB, 0.68794E+00_JPRB, 0.67828E+00_JPRB, 0.66820E+00_JPRB, 0.66385E+00_JPRB, 0.66132E+00_JPRB ,&
&  0.66144E+00_JPRB, 0.66543E+00_JPRB, 0.66761E+00_JPRB, 0.67433E+00_JPRB, 0.68012E+00_JPRB, 0.69235E+00_JPRB, 0.71182E+00_JPRB ,&
&  0.71873E+00_JPRB, 0.72964E+00_JPRB, 0.74743E+00_JPRB, 0.74808E+00_JPRB, 0.73314E+00_JPRB, 0.72337E+00_JPRB, 0.71330E+00_JPRB ,&
&  0.70063E+00_JPRB, 0.68007E+00_JPRB, 0.67457E+00_JPRB, 0.66927E+00_JPRB, 0.66441E+00_JPRB, 0.66309E+00_JPRB, 0.67233E+00_JPRB ,&
&  0.68164E+00_JPRB, 0.68483E+00_JPRB, 0.69041E+00_JPRB, 0.69923E+00_JPRB, 0.70372E+00_JPRB, 0.70059E+00_JPRB, 0.70069E+00_JPRB ,&
&  0.70872E+00_JPRB, 0.72120E+00_JPRB, 0.72691E+00_JPRB, 0.72752E+00_JPRB, 0.73554E+00_JPRB, 0.75033E+00_JPRB, 0.76092E+00_JPRB ,&
&  0.76864E+00_JPRB, 0.77259E+00_JPRB, 0.78163E+00_JPRB, 0.79924E+00_JPRB, 0.85555E+00_JPRB, 0.84847E+00_JPRB, 0.83114E+00_JPRB ,&
&  0.81724E+00_JPRB, 0.80101E+00_JPRB, 0.79737E+00_JPRB, 0.68680E+00_JPRB, 0.73569E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.81832E+00_JPRB, 0.81923E+00_JPRB, 0.79807E+00_JPRB, 0.77754E+00_JPRB, 0.76603E+00_JPRB, 0.75355E+00_JPRB, 0.73091E+00_JPRB ,&
&  0.70566E+00_JPRB, 0.68295E+00_JPRB, 0.66920E+00_JPRB, 0.65132E+00_JPRB, 0.63454E+00_JPRB, 0.62461E+00_JPRB, 0.61392E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.59405E+00_JPRB, 0.59428E+00_JPRB, 0.59255E+00_JPRB, 0.58857E+00_JPRB, 0.58674E+00_JPRB, 0.59461E+00_JPRB ,&
&  0.59615E+00_JPRB, 0.59410E+00_JPRB, 0.59376E+00_JPRB, 0.58999E+00_JPRB, 0.58512E+00_JPRB, 0.58484E+00_JPRB, 0.57431E+00_JPRB ,&
&  0.58115E+00_JPRB, 0.55716E+00_JPRB, 0.66835E+00_JPRB, 0.78707E+00_JPRB, 0.88862E+00_JPRB, 0.94987E+00_JPRB, 0.98749E+00_JPRB ,&
&  0.98953E+00_JPRB, 0.94411E+00_JPRB, 0.88784E+00_JPRB, 0.84391E+00_JPRB, 0.79758E+00_JPRB, 0.76258E+00_JPRB, 0.73585E+00_JPRB ,&
&  0.71859E+00_JPRB, 0.70937E+00_JPRB, 0.70252E+00_JPRB, 0.69616E+00_JPRB, 0.68957E+00_JPRB, 0.68426E+00_JPRB, 0.68234E+00_JPRB ,&
&  0.67875E+00_JPRB, 0.67917E+00_JPRB, 0.68036E+00_JPRB, 0.67761E+00_JPRB, 0.68251E+00_JPRB, 0.70865E+00_JPRB, 0.74385E+00_JPRB ,&
&  0.75595E+00_JPRB, 0.77571E+00_JPRB, 0.80251E+00_JPRB, 0.81168E+00_JPRB, 0.79966E+00_JPRB, 0.79048E+00_JPRB, 0.77745E+00_JPRB ,&
&  0.76726E+00_JPRB, 0.75110E+00_JPRB, 0.74597E+00_JPRB, 0.73732E+00_JPRB, 0.72599E+00_JPRB, 0.71926E+00_JPRB, 0.71784E+00_JPRB ,&
&  0.71874E+00_JPRB, 0.72039E+00_JPRB, 0.72786E+00_JPRB, 0.73687E+00_JPRB, 0.73581E+00_JPRB, 0.73053E+00_JPRB, 0.73153E+00_JPRB ,&
&  0.73552E+00_JPRB, 0.73074E+00_JPRB, 0.72087E+00_JPRB, 0.73074E+00_JPRB, 0.75255E+00_JPRB, 0.76714E+00_JPRB, 0.77652E+00_JPRB ,&
&  0.78572E+00_JPRB, 0.79899E+00_JPRB, 0.81867E+00_JPRB, 0.88209E+00_JPRB, 0.90046E+00_JPRB, 0.88591E+00_JPRB, 0.87360E+00_JPRB ,&
&  0.86676E+00_JPRB, 0.85304E+00_JPRB, 0.83630E+00_JPRB, 0.82500E+00_JPRB, 0.76516E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.97177E+00_JPRB, 0.90303E+00_JPRB, 0.84698E+00_JPRB, 0.80633E+00_JPRB, 0.77903E+00_JPRB, 0.74536E+00_JPRB, 0.70619E+00_JPRB ,&
&  0.67781E+00_JPRB, 0.66693E+00_JPRB, 0.65556E+00_JPRB, 0.63885E+00_JPRB, 0.62723E+00_JPRB, 0.61412E+00_JPRB, 0.60275E+00_JPRB ,&
&  0.58766E+00_JPRB, 0.58327E+00_JPRB, 0.57915E+00_JPRB, 0.58348E+00_JPRB, 0.58485E+00_JPRB, 0.59024E+00_JPRB, 0.60210E+00_JPRB ,&
&  0.60469E+00_JPRB, 0.58484E+00_JPRB, 0.58159E+00_JPRB, 0.58352E+00_JPRB, 0.58470E+00_JPRB, 0.58599E+00_JPRB, 0.57530E+00_JPRB ,&
&  0.59662E+00_JPRB, 0.62419E+00_JPRB, 0.84303E+00_JPRB, 0.10563E+01_JPRB, 0.11656E+01_JPRB, 0.12050E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.10465E+01_JPRB, 0.97243E+00_JPRB, 0.89419E+00_JPRB, 0.82128E+00_JPRB, 0.77468E+00_JPRB, 0.74855E+00_JPRB ,&
&  0.73403E+00_JPRB, 0.72879E+00_JPRB, 0.72273E+00_JPRB, 0.71962E+00_JPRB, 0.71644E+00_JPRB, 0.71119E+00_JPRB, 0.71240E+00_JPRB ,&
&  0.71778E+00_JPRB, 0.70687E+00_JPRB, 0.70112E+00_JPRB, 0.71072E+00_JPRB, 0.72728E+00_JPRB, 0.75040E+00_JPRB, 0.81429E+00_JPRB ,&
&  0.84982E+00_JPRB, 0.88086E+00_JPRB, 0.91400E+00_JPRB, 0.92456E+00_JPRB, 0.91453E+00_JPRB, 0.90263E+00_JPRB, 0.87896E+00_JPRB ,&
&  0.86394E+00_JPRB, 0.84885E+00_JPRB, 0.83149E+00_JPRB, 0.81306E+00_JPRB, 0.79539E+00_JPRB, 0.78455E+00_JPRB, 0.77777E+00_JPRB ,&
&  0.77172E+00_JPRB, 0.77955E+00_JPRB, 0.78690E+00_JPRB, 0.78911E+00_JPRB, 0.78074E+00_JPRB, 0.77399E+00_JPRB, 0.76875E+00_JPRB ,&
&  0.75403E+00_JPRB, 0.73251E+00_JPRB, 0.72583E+00_JPRB, 0.73778E+00_JPRB, 0.76675E+00_JPRB, 0.78092E+00_JPRB, 0.78786E+00_JPRB ,&
&  0.80613E+00_JPRB, 0.83242E+00_JPRB, 0.83681E+00_JPRB, 0.92461E+00_JPRB, 0.10069E+01_JPRB, 0.98866E+00_JPRB, 0.96157E+00_JPRB ,&
&  0.92795E+00_JPRB, 0.90905E+00_JPRB, 0.88126E+00_JPRB, 0.86007E+00_JPRB, 0.95829E+00_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.87290E+00_JPRB, 0.83526E+00_JPRB, 0.80110E+00_JPRB, 0.76458E+00_JPRB, 0.73954E+00_JPRB, 0.71245E+00_JPRB, 0.68609E+00_JPRB ,&
&  0.66316E+00_JPRB, 0.64887E+00_JPRB, 0.63662E+00_JPRB, 0.62489E+00_JPRB, 0.62484E+00_JPRB, 0.61857E+00_JPRB, 0.60527E+00_JPRB ,&
&  0.58666E+00_JPRB, 0.57353E+00_JPRB, 0.56992E+00_JPRB, 0.57321E+00_JPRB, 0.58612E+00_JPRB, 0.59538E+00_JPRB, 0.60424E+00_JPRB ,&
&  0.60862E+00_JPRB, 0.58855E+00_JPRB, 0.57940E+00_JPRB, 0.58203E+00_JPRB, 0.58389E+00_JPRB, 0.58749E+00_JPRB, 0.57274E+00_JPRB ,&
&  0.58746E+00_JPRB, 0.67879E+00_JPRB, 0.10423E+01_JPRB, 0.15165E+01_JPRB, 0.16200E+01_JPRB, 0.14701E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.11553E+01_JPRB, 0.10569E+01_JPRB, 0.95867E+00_JPRB, 0.88423E+00_JPRB, 0.83604E+00_JPRB, 0.80715E+00_JPRB ,&
&  0.79771E+00_JPRB, 0.79197E+00_JPRB, 0.77981E+00_JPRB, 0.77110E+00_JPRB, 0.76192E+00_JPRB, 0.75508E+00_JPRB, 0.75322E+00_JPRB ,&
&  0.74996E+00_JPRB, 0.74394E+00_JPRB, 0.73638E+00_JPRB, 0.73515E+00_JPRB, 0.78741E+00_JPRB, 0.85665E+00_JPRB, 0.98409E+00_JPRB ,&
&  0.10558E+01_JPRB, 0.10981E+01_JPRB, 0.11283E+01_JPRB, 0.11222E+01_JPRB, 0.10919E+01_JPRB, 0.10621E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.99794E+00_JPRB, 0.94868E+00_JPRB, 0.91451E+00_JPRB, 0.88865E+00_JPRB, 0.86373E+00_JPRB, 0.84424E+00_JPRB, 0.82897E+00_JPRB ,&
&  0.82509E+00_JPRB, 0.83318E+00_JPRB, 0.83656E+00_JPRB, 0.83298E+00_JPRB, 0.81611E+00_JPRB, 0.79591E+00_JPRB, 0.77566E+00_JPRB ,&
&  0.74710E+00_JPRB, 0.72934E+00_JPRB, 0.73829E+00_JPRB, 0.76058E+00_JPRB, 0.78818E+00_JPRB, 0.79834E+00_JPRB, 0.81059E+00_JPRB ,&
&  0.83478E+00_JPRB, 0.88161E+00_JPRB, 0.88003E+00_JPRB, 0.89970E+00_JPRB, 0.94124E+00_JPRB, 0.10025E+01_JPRB, 0.97218E+00_JPRB ,&
&  0.92688E+00_JPRB, 0.88821E+00_JPRB, 0.86626E+00_JPRB, 0.84854E+00_JPRB, 0.89879E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.83043E+00_JPRB, 0.81729E+00_JPRB, 0.78225E+00_JPRB, 0.74735E+00_JPRB, 0.72166E+00_JPRB, 0.70196E+00_JPRB, 0.68385E+00_JPRB ,&
&  0.66168E+00_JPRB, 0.63500E+00_JPRB, 0.61791E+00_JPRB, 0.61234E+00_JPRB, 0.61182E+00_JPRB, 0.60920E+00_JPRB, 0.59281E+00_JPRB ,&
&  0.57315E+00_JPRB, 0.55969E+00_JPRB, 0.56274E+00_JPRB, 0.57074E+00_JPRB, 0.58487E+00_JPRB, 0.58327E+00_JPRB, 0.59427E+00_JPRB ,&
&  0.60030E+00_JPRB, 0.59814E+00_JPRB, 0.59567E+00_JPRB, 0.59378E+00_JPRB, 0.58913E+00_JPRB, 0.58504E+00_JPRB, 0.56673E+00_JPRB ,&
&  0.59804E+00_JPRB, 0.76308E+00_JPRB, 0.13728E+01_JPRB, 0.18398E+01_JPRB, 0.18318E+01_JPRB, 0.15799E+01_JPRB, 0.14424E+01_JPRB ,&
&  0.13396E+01_JPRB, 0.12557E+01_JPRB, 0.11293E+01_JPRB, 0.10557E+01_JPRB, 0.98965E+00_JPRB, 0.94067E+00_JPRB, 0.90488E+00_JPRB ,&
&  0.88195E+00_JPRB, 0.85848E+00_JPRB, 0.82708E+00_JPRB, 0.80071E+00_JPRB, 0.78691E+00_JPRB, 0.77831E+00_JPRB, 0.77803E+00_JPRB ,&
&  0.78137E+00_JPRB, 0.77887E+00_JPRB, 0.77668E+00_JPRB, 0.78367E+00_JPRB, 0.84657E+00_JPRB, 0.90662E+00_JPRB, 0.12351E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.16038E+01_JPRB, 0.15519E+01_JPRB, 0.14556E+01_JPRB, 0.13465E+01_JPRB, 0.12636E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.11158E+01_JPRB, 0.10555E+01_JPRB, 0.10063E+01_JPRB, 0.97503E+00_JPRB, 0.95319E+00_JPRB, 0.93586E+00_JPRB, 0.92866E+00_JPRB ,&
&  0.91349E+00_JPRB, 0.89923E+00_JPRB, 0.88206E+00_JPRB, 0.85986E+00_JPRB, 0.82812E+00_JPRB, 0.79730E+00_JPRB, 0.77257E+00_JPRB ,&
&  0.74783E+00_JPRB, 0.74629E+00_JPRB, 0.77289E+00_JPRB, 0.77363E+00_JPRB, 0.81428E+00_JPRB, 0.84973E+00_JPRB, 0.85261E+00_JPRB ,&
&  0.84339E+00_JPRB, 0.86864E+00_JPRB, 0.86861E+00_JPRB, 0.86850E+00_JPRB, 0.91284E+00_JPRB, 0.97669E+00_JPRB, 0.92299E+00_JPRB ,&
&  0.87729E+00_JPRB, 0.85211E+00_JPRB, 0.83650E+00_JPRB, 0.81998E+00_JPRB, 0.83442E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.79485E+00_JPRB, 0.76626E+00_JPRB, 0.73698E+00_JPRB, 0.71231E+00_JPRB, 0.69179E+00_JPRB, 0.67743E+00_JPRB, 0.66250E+00_JPRB ,&
&  0.63838E+00_JPRB, 0.61596E+00_JPRB, 0.60777E+00_JPRB, 0.60142E+00_JPRB, 0.59496E+00_JPRB, 0.58965E+00_JPRB, 0.57217E+00_JPRB ,&
&  0.55535E+00_JPRB, 0.53964E+00_JPRB, 0.54949E+00_JPRB, 0.56236E+00_JPRB, 0.57544E+00_JPRB, 0.58582E+00_JPRB, 0.60211E+00_JPRB ,&
&  0.59874E+00_JPRB, 0.59981E+00_JPRB, 0.60257E+00_JPRB, 0.60313E+00_JPRB, 0.59303E+00_JPRB, 0.59343E+00_JPRB, 0.56824E+00_JPRB ,&
&  0.63444E+00_JPRB, 0.70159E+00_JPRB, 0.14608E+01_JPRB, 0.20884E+01_JPRB, 0.19032E+01_JPRB, 0.16711E+01_JPRB, 0.14828E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.13333E+01_JPRB, 0.11882E+01_JPRB, 0.11039E+01_JPRB, 0.10340E+01_JPRB, 0.97866E+00_JPRB, 0.93777E+00_JPRB ,&
&  0.90097E+00_JPRB, 0.85667E+00_JPRB, 0.82711E+00_JPRB, 0.80572E+00_JPRB, 0.80240E+00_JPRB, 0.80256E+00_JPRB, 0.80873E+00_JPRB ,&
&  0.81240E+00_JPRB, 0.80938E+00_JPRB, 0.81022E+00_JPRB, 0.84778E+00_JPRB, 0.95427E+00_JPRB, 0.12259E+01_JPRB, 0.15225E+01_JPRB ,&
&  0.17790E+01_JPRB, 0.20151E+01_JPRB, 0.18576E+01_JPRB, 0.16608E+01_JPRB, 0.14972E+01_JPRB, 0.13770E+01_JPRB, 0.12947E+01_JPRB ,&
&  0.12179E+01_JPRB, 0.11581E+01_JPRB, 0.11172E+01_JPRB, 0.10965E+01_JPRB, 0.10807E+01_JPRB, 0.10615E+01_JPRB, 0.10276E+01_JPRB ,&
&  0.98251E+00_JPRB, 0.93335E+00_JPRB, 0.88648E+00_JPRB, 0.84559E+00_JPRB, 0.80990E+00_JPRB, 0.79800E+00_JPRB, 0.79379E+00_JPRB ,&
&  0.78104E+00_JPRB, 0.78865E+00_JPRB, 0.80133E+00_JPRB, 0.79155E+00_JPRB, 0.81658E+00_JPRB, 0.82655E+00_JPRB, 0.83141E+00_JPRB ,&
&  0.84582E+00_JPRB, 0.85356E+00_JPRB, 0.84418E+00_JPRB, 0.88148E+00_JPRB, 0.87379E+00_JPRB, 0.93844E+00_JPRB, 0.88149E+00_JPRB ,&
&  0.83597E+00_JPRB, 0.82659E+00_JPRB, 0.81286E+00_JPRB, 0.80771E+00_JPRB, 0.80782E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.75299E+00_JPRB, 0.71488E+00_JPRB, 0.68627E+00_JPRB, 0.67422E+00_JPRB, 0.66592E+00_JPRB, 0.65280E+00_JPRB, 0.64215E+00_JPRB ,&
&  0.62277E+00_JPRB, 0.60210E+00_JPRB, 0.58931E+00_JPRB, 0.57771E+00_JPRB, 0.57946E+00_JPRB, 0.57377E+00_JPRB, 0.56310E+00_JPRB ,&
&  0.54955E+00_JPRB, 0.54597E+00_JPRB, 0.54781E+00_JPRB, 0.55806E+00_JPRB, 0.57772E+00_JPRB, 0.58749E+00_JPRB, 0.60151E+00_JPRB ,&
&  0.61027E+00_JPRB, 0.61922E+00_JPRB, 0.61935E+00_JPRB, 0.61405E+00_JPRB, 0.60689E+00_JPRB, 0.59732E+00_JPRB, 0.58464E+00_JPRB ,&
&  0.67057E+00_JPRB, 0.14737E+01_JPRB, 0.12971E+01_JPRB, 0.24083E+01_JPRB, 0.20528E+01_JPRB, 0.18110E+01_JPRB, 0.14077E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.13469E+01_JPRB, 0.12397E+01_JPRB, 0.11207E+01_JPRB, 0.10210E+01_JPRB, 0.93198E+00_JPRB, 0.86959E+00_JPRB ,&
&  0.84614E+00_JPRB, 0.84738E+00_JPRB, 0.84880E+00_JPRB, 0.83761E+00_JPRB, 0.83782E+00_JPRB, 0.83320E+00_JPRB, 0.83630E+00_JPRB ,&
&  0.83921E+00_JPRB, 0.84207E+00_JPRB, 0.86493E+00_JPRB, 0.97373E+00_JPRB, 0.12014E+01_JPRB, 0.16445E+01_JPRB, 0.18543E+01_JPRB ,&
&  0.18213E+01_JPRB, 0.20557E+01_JPRB, 0.18363E+01_JPRB, 0.16026E+01_JPRB, 0.14469E+01_JPRB, 0.13357E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.11384E+01_JPRB, 0.11333E+01_JPRB, 0.11289E+01_JPRB, 0.11104E+01_JPRB, 0.10599E+01_JPRB, 0.98739E+00_JPRB ,&
&  0.91821E+00_JPRB, 0.87511E+00_JPRB, 0.83752E+00_JPRB, 0.81685E+00_JPRB, 0.80948E+00_JPRB, 0.81521E+00_JPRB, 0.82733E+00_JPRB ,&
&  0.82661E+00_JPRB, 0.83844E+00_JPRB, 0.82553E+00_JPRB, 0.79427E+00_JPRB, 0.80610E+00_JPRB, 0.80039E+00_JPRB, 0.78649E+00_JPRB ,&
&  0.82114E+00_JPRB, 0.84159E+00_JPRB, 0.84469E+00_JPRB, 0.87306E+00_JPRB, 0.87414E+00_JPRB, 0.95954E+00_JPRB, 0.86049E+00_JPRB ,&
&  0.82048E+00_JPRB, 0.81431E+00_JPRB, 0.81625E+00_JPRB, 0.79799E+00_JPRB, 0.78361E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.71287E+00_JPRB, 0.69029E+00_JPRB, 0.67505E+00_JPRB, 0.67812E+00_JPRB, 0.65545E+00_JPRB, 0.64005E+00_JPRB, 0.63593E+00_JPRB ,&
&  0.60627E+00_JPRB, 0.58781E+00_JPRB, 0.57575E+00_JPRB, 0.57593E+00_JPRB, 0.57934E+00_JPRB, 0.57281E+00_JPRB, 0.57187E+00_JPRB ,&
&  0.56182E+00_JPRB, 0.55362E+00_JPRB, 0.55679E+00_JPRB, 0.57784E+00_JPRB, 0.60549E+00_JPRB, 0.61247E+00_JPRB, 0.62638E+00_JPRB ,&
&  0.64086E+00_JPRB, 0.63545E+00_JPRB, 0.62551E+00_JPRB, 0.62581E+00_JPRB, 0.63251E+00_JPRB, 0.63174E+00_JPRB, 0.64991E+00_JPRB ,&
&  0.70521E+00_JPRB, 0.13552E+01_JPRB, 0.14730E+01_JPRB, 0.25676E+01_JPRB, 0.22093E+01_JPRB, 0.18334E+01_JPRB, 0.13760E+01_JPRB ,&
&  0.12670E+01_JPRB, 0.13588E+01_JPRB, 0.11978E+01_JPRB, 0.10568E+01_JPRB, 0.92337E+00_JPRB, 0.82776E+00_JPRB, 0.80551E+00_JPRB ,&
&  0.83679E+00_JPRB, 0.87461E+00_JPRB, 0.90192E+00_JPRB, 0.88844E+00_JPRB, 0.86698E+00_JPRB, 0.85890E+00_JPRB, 0.85963E+00_JPRB ,&
&  0.86662E+00_JPRB, 0.89318E+00_JPRB, 0.99034E+00_JPRB, 0.11958E+01_JPRB, 0.17499E+01_JPRB, 0.20737E+01_JPRB, 0.18903E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.17709E+01_JPRB, 0.15918E+01_JPRB, 0.14177E+01_JPRB, 0.12594E+01_JPRB, 0.12344E+01_JPRB, 0.11847E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11384E+01_JPRB, 0.11091E+01_JPRB, 0.10604E+01_JPRB, 0.10069E+01_JPRB, 0.94734E+00_JPRB, 0.88665E+00_JPRB ,&
&  0.84769E+00_JPRB, 0.82847E+00_JPRB, 0.81592E+00_JPRB, 0.82105E+00_JPRB, 0.84087E+00_JPRB, 0.86077E+00_JPRB, 0.87808E+00_JPRB ,&
&  0.87782E+00_JPRB, 0.88445E+00_JPRB, 0.84609E+00_JPRB, 0.82063E+00_JPRB, 0.83116E+00_JPRB, 0.81298E+00_JPRB, 0.80311E+00_JPRB ,&
&  0.83943E+00_JPRB, 0.87374E+00_JPRB, 0.88691E+00_JPRB, 0.88304E+00_JPRB, 0.89372E+00_JPRB, 0.94110E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.82366E+00_JPRB, 0.84500E+00_JPRB, 0.85088E+00_JPRB, 0.78630E+00_JPRB, 0.74976E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.70770E+00_JPRB, 0.70458E+00_JPRB, 0.68979E+00_JPRB, 0.68428E+00_JPRB, 0.65071E+00_JPRB, 0.61973E+00_JPRB, 0.59395E+00_JPRB ,&
&  0.58109E+00_JPRB, 0.57690E+00_JPRB, 0.58692E+00_JPRB, 0.58970E+00_JPRB, 0.59074E+00_JPRB, 0.58277E+00_JPRB, 0.57710E+00_JPRB ,&
&  0.56460E+00_JPRB, 0.56807E+00_JPRB, 0.58183E+00_JPRB, 0.60479E+00_JPRB, 0.63053E+00_JPRB, 0.64013E+00_JPRB, 0.65060E+00_JPRB ,&
&  0.65138E+00_JPRB, 0.64796E+00_JPRB, 0.65098E+00_JPRB, 0.66261E+00_JPRB, 0.68231E+00_JPRB, 0.70362E+00_JPRB, 0.73192E+00_JPRB ,&
&  0.78168E+00_JPRB, 0.13167E+01_JPRB, 0.19476E+01_JPRB, 0.22686E+01_JPRB, 0.19541E+01_JPRB, 0.15850E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.10098E+01_JPRB, 0.10718E+01_JPRB, 0.97673E+00_JPRB, 0.87110E+00_JPRB, 0.83180E+00_JPRB, 0.84614E+00_JPRB ,&
&  0.89722E+00_JPRB, 0.91997E+00_JPRB, 0.93206E+00_JPRB, 0.93823E+00_JPRB, 0.92560E+00_JPRB, 0.92125E+00_JPRB, 0.93064E+00_JPRB ,&
&  0.98363E+00_JPRB, 0.11077E+01_JPRB, 0.13403E+01_JPRB, 0.17542E+01_JPRB, 0.22970E+01_JPRB, 0.21137E+01_JPRB, 0.16967E+01_JPRB ,&
&  0.15159E+01_JPRB, 0.14292E+01_JPRB, 0.13222E+01_JPRB, 0.12446E+01_JPRB, 0.10668E+01_JPRB, 0.11803E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.10575E+01_JPRB, 0.10092E+01_JPRB, 0.96459E+00_JPRB, 0.92445E+00_JPRB, 0.88274E+00_JPRB, 0.85255E+00_JPRB ,&
&  0.84347E+00_JPRB, 0.84477E+00_JPRB, 0.84837E+00_JPRB, 0.85759E+00_JPRB, 0.87377E+00_JPRB, 0.89231E+00_JPRB, 0.90307E+00_JPRB ,&
&  0.90348E+00_JPRB, 0.91310E+00_JPRB, 0.89744E+00_JPRB, 0.90671E+00_JPRB, 0.90780E+00_JPRB, 0.88533E+00_JPRB, 0.86432E+00_JPRB ,&
&  0.88441E+00_JPRB, 0.92852E+00_JPRB, 0.92795E+00_JPRB, 0.92738E+00_JPRB, 0.92858E+00_JPRB, 0.94046E+00_JPRB, 0.86554E+00_JPRB ,&
&  0.84431E+00_JPRB, 0.90113E+00_JPRB, 0.85410E+00_JPRB, 0.77154E+00_JPRB, 0.73274E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.70824E+00_JPRB, 0.71059E+00_JPRB, 0.70073E+00_JPRB, 0.66290E+00_JPRB, 0.62844E+00_JPRB, 0.59323E+00_JPRB, 0.56973E+00_JPRB ,&
&  0.56788E+00_JPRB, 0.57845E+00_JPRB, 0.60942E+00_JPRB, 0.62841E+00_JPRB, 0.61883E+00_JPRB, 0.60071E+00_JPRB, 0.59503E+00_JPRB ,&
&  0.60041E+00_JPRB, 0.61540E+00_JPRB, 0.62542E+00_JPRB, 0.64427E+00_JPRB, 0.66587E+00_JPRB, 0.68505E+00_JPRB, 0.70573E+00_JPRB ,&
&  0.69492E+00_JPRB, 0.70003E+00_JPRB, 0.71508E+00_JPRB, 0.72747E+00_JPRB, 0.74772E+00_JPRB, 0.79230E+00_JPRB, 0.83486E+00_JPRB ,&
&  0.98330E+00_JPRB, 0.14957E+01_JPRB, 0.93013E+00_JPRB, 0.12967E+01_JPRB, 0.14796E+01_JPRB, 0.14570E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.94702E+00_JPRB, 0.95189E+00_JPRB, 0.99875E+00_JPRB, 0.94075E+00_JPRB, 0.95090E+00_JPRB, 0.96768E+00_JPRB ,&
&  0.96875E+00_JPRB, 0.98784E+00_JPRB, 0.99890E+00_JPRB, 0.10110E+01_JPRB, 0.10147E+01_JPRB, 0.10366E+01_JPRB, 0.11109E+01_JPRB ,&
&  0.12660E+01_JPRB, 0.14735E+01_JPRB, 0.18299E+01_JPRB, 0.21076E+01_JPRB, 0.25600E+01_JPRB, 0.20389E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.11671E+01_JPRB, 0.11716E+01_JPRB, 0.11944E+01_JPRB, 0.11701E+01_JPRB, 0.10646E+01_JPRB, 0.10066E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.99867E+00_JPRB, 0.95778E+00_JPRB, 0.94201E+00_JPRB, 0.92327E+00_JPRB, 0.89690E+00_JPRB, 0.87818E+00_JPRB, 0.86760E+00_JPRB ,&
&  0.86014E+00_JPRB, 0.86313E+00_JPRB, 0.87307E+00_JPRB, 0.87917E+00_JPRB, 0.89195E+00_JPRB, 0.90241E+00_JPRB, 0.91893E+00_JPRB ,&
&  0.93740E+00_JPRB, 0.96601E+00_JPRB, 0.99876E+00_JPRB, 0.10101E+01_JPRB, 0.10040E+01_JPRB, 0.98194E+00_JPRB, 0.97228E+00_JPRB ,&
&  0.98157E+00_JPRB, 0.10096E+01_JPRB, 0.95135E+00_JPRB, 0.93794E+00_JPRB, 0.97849E+00_JPRB, 0.93804E+00_JPRB, 0.88054E+00_JPRB ,&
&  0.92160E+00_JPRB, 0.10701E+01_JPRB, 0.11160E+01_JPRB, 0.75920E+00_JPRB, 0.72136E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.66874E+00_JPRB, 0.65013E+00_JPRB, 0.62126E+00_JPRB, 0.58439E+00_JPRB, 0.57985E+00_JPRB, 0.57680E+00_JPRB, 0.58898E+00_JPRB ,&
&  0.62686E+00_JPRB, 0.67726E+00_JPRB, 0.69023E+00_JPRB, 0.68393E+00_JPRB, 0.66666E+00_JPRB, 0.66344E+00_JPRB, 0.66330E+00_JPRB ,&
&  0.66976E+00_JPRB, 0.67279E+00_JPRB, 0.69094E+00_JPRB, 0.71489E+00_JPRB, 0.74101E+00_JPRB, 0.76105E+00_JPRB, 0.76645E+00_JPRB ,&
&  0.77472E+00_JPRB, 0.78181E+00_JPRB, 0.80239E+00_JPRB, 0.83161E+00_JPRB, 0.89095E+00_JPRB, 0.10103E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.16676E+01_JPRB, 0.13644E+01_JPRB, 0.69376E+00_JPRB, 0.12429E+01_JPRB, 0.14002E+01_JPRB, 0.14817E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.10478E+01_JPRB, 0.10189E+01_JPRB, 0.11236E+01_JPRB, 0.11242E+01_JPRB, 0.11402E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.11136E+01_JPRB, 0.11165E+01_JPRB, 0.11174E+01_JPRB, 0.11306E+01_JPRB, 0.11617E+01_JPRB, 0.12382E+01_JPRB, 0.13852E+01_JPRB ,&
&  0.15706E+01_JPRB, 0.18112E+01_JPRB, 0.21459E+01_JPRB, 0.22779E+01_JPRB, 0.24980E+01_JPRB, 0.19105E+01_JPRB, 0.15069E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.10639E+01_JPRB, 0.11319E+01_JPRB, 0.11361E+01_JPRB, 0.10535E+01_JPRB, 0.93640E+00_JPRB, 0.98709E+00_JPRB ,&
&  0.96342E+00_JPRB, 0.93664E+00_JPRB, 0.92562E+00_JPRB, 0.90287E+00_JPRB, 0.88485E+00_JPRB, 0.87732E+00_JPRB, 0.87488E+00_JPRB ,&
&  0.87230E+00_JPRB, 0.87996E+00_JPRB, 0.88429E+00_JPRB, 0.88733E+00_JPRB, 0.89480E+00_JPRB, 0.90014E+00_JPRB, 0.92970E+00_JPRB ,&
&  0.95748E+00_JPRB, 0.98198E+00_JPRB, 0.10141E+01_JPRB, 0.10414E+01_JPRB, 0.10287E+01_JPRB, 0.98910E+00_JPRB, 0.98523E+00_JPRB ,&
&  0.96998E+00_JPRB, 0.96944E+00_JPRB, 0.94443E+00_JPRB, 0.96550E+00_JPRB, 0.92868E+00_JPRB, 0.87210E+00_JPRB, 0.81099E+00_JPRB ,&
&  0.81019E+00_JPRB, 0.80426E+00_JPRB, 0.76170E+00_JPRB, 0.70186E+00_JPRB, 0.67649E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.63366E+00_JPRB, 0.61382E+00_JPRB, 0.60019E+00_JPRB, 0.60599E+00_JPRB, 0.61509E+00_JPRB, 0.63431E+00_JPRB, 0.68320E+00_JPRB ,&
&  0.72220E+00_JPRB, 0.73979E+00_JPRB, 0.76137E+00_JPRB, 0.76179E+00_JPRB, 0.77078E+00_JPRB, 0.75438E+00_JPRB, 0.75081E+00_JPRB ,&
&  0.73928E+00_JPRB, 0.75100E+00_JPRB, 0.77148E+00_JPRB, 0.79116E+00_JPRB, 0.80714E+00_JPRB, 0.82282E+00_JPRB, 0.84453E+00_JPRB ,&
&  0.87277E+00_JPRB, 0.91223E+00_JPRB, 0.97374E+00_JPRB, 0.10666E+01_JPRB, 0.12131E+01_JPRB, 0.13932E+01_JPRB, 0.15124E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.53120E+00_JPRB, 0.98994E+00_JPRB, 0.14076E+01_JPRB, 0.14644E+01_JPRB, 0.15010E+01_JPRB, 0.14212E+01_JPRB ,&
&  0.12839E+01_JPRB, 0.11946E+01_JPRB, 0.11481E+01_JPRB, 0.12925E+01_JPRB, 0.12828E+01_JPRB, 0.12802E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.12589E+01_JPRB, 0.12643E+01_JPRB, 0.12955E+01_JPRB, 0.13598E+01_JPRB, 0.14665E+01_JPRB, 0.16107E+01_JPRB ,&
&  0.18099E+01_JPRB, 0.20831E+01_JPRB, 0.23298E+01_JPRB, 0.21692E+01_JPRB, 0.22497E+01_JPRB, 0.19996E+01_JPRB, 0.17503E+01_JPRB ,&
&  0.11920E+01_JPRB, 0.97556E+00_JPRB, 0.10087E+01_JPRB, 0.11288E+01_JPRB, 0.10562E+01_JPRB, 0.91160E+00_JPRB, 0.96439E+00_JPRB ,&
&  0.94621E+00_JPRB, 0.91950E+00_JPRB, 0.90702E+00_JPRB, 0.88327E+00_JPRB, 0.87589E+00_JPRB, 0.87461E+00_JPRB, 0.87937E+00_JPRB ,&
&  0.87851E+00_JPRB, 0.88233E+00_JPRB, 0.88577E+00_JPRB, 0.88368E+00_JPRB, 0.87014E+00_JPRB, 0.87474E+00_JPRB, 0.89958E+00_JPRB ,&
&  0.91994E+00_JPRB, 0.92673E+00_JPRB, 0.95320E+00_JPRB, 0.97953E+00_JPRB, 0.10065E+01_JPRB, 0.98379E+00_JPRB, 0.97064E+00_JPRB ,&
&  0.94811E+00_JPRB, 0.93912E+00_JPRB, 0.93857E+00_JPRB, 0.93593E+00_JPRB, 0.93011E+00_JPRB, 0.87142E+00_JPRB, 0.79062E+00_JPRB ,&
&  0.76352E+00_JPRB, 0.73250E+00_JPRB, 0.70792E+00_JPRB, 0.68598E+00_JPRB, 0.66177E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.69292E+00_JPRB, 0.70222E+00_JPRB, 0.69558E+00_JPRB, 0.71774E+00_JPRB, 0.73737E+00_JPRB, 0.76718E+00_JPRB, 0.79514E+00_JPRB ,&
&  0.81935E+00_JPRB, 0.84273E+00_JPRB, 0.87270E+00_JPRB, 0.85112E+00_JPRB, 0.86013E+00_JPRB, 0.84898E+00_JPRB, 0.82641E+00_JPRB ,&
&  0.82338E+00_JPRB, 0.83915E+00_JPRB, 0.85652E+00_JPRB, 0.87423E+00_JPRB, 0.89390E+00_JPRB, 0.92966E+00_JPRB, 0.97534E+00_JPRB ,&
&  0.10336E+01_JPRB, 0.11074E+01_JPRB, 0.12060E+01_JPRB, 0.12919E+01_JPRB, 0.13777E+01_JPRB, 0.14108E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.67452E+00_JPRB, 0.85816E+00_JPRB, 0.13498E+01_JPRB, 0.15285E+01_JPRB, 0.16205E+01_JPRB, 0.16250E+01_JPRB, 0.15729E+01_JPRB ,&
&  0.14380E+01_JPRB, 0.12681E+01_JPRB, 0.12542E+01_JPRB, 0.14196E+01_JPRB, 0.13785E+01_JPRB, 0.13797E+01_JPRB, 0.13704E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.13912E+01_JPRB, 0.14299E+01_JPRB, 0.14733E+01_JPRB, 0.15553E+01_JPRB, 0.16647E+01_JPRB, 0.18195E+01_JPRB ,&
&  0.20163E+01_JPRB, 0.22182E+01_JPRB, 0.24017E+01_JPRB, 0.21451E+01_JPRB, 0.21137E+01_JPRB, 0.20535E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.12629E+01_JPRB, 0.93196E+00_JPRB, 0.99436E+00_JPRB, 0.11072E+01_JPRB, 0.10352E+01_JPRB, 0.97852E+00_JPRB, 0.95046E+00_JPRB ,&
&  0.94102E+00_JPRB, 0.89901E+00_JPRB, 0.88769E+00_JPRB, 0.86678E+00_JPRB, 0.85531E+00_JPRB, 0.86215E+00_JPRB, 0.87909E+00_JPRB ,&
&  0.87509E+00_JPRB, 0.86232E+00_JPRB, 0.86156E+00_JPRB, 0.86707E+00_JPRB, 0.85129E+00_JPRB, 0.83461E+00_JPRB, 0.83011E+00_JPRB ,&
&  0.83647E+00_JPRB, 0.82511E+00_JPRB, 0.83545E+00_JPRB, 0.86839E+00_JPRB, 0.91156E+00_JPRB, 0.93627E+00_JPRB, 0.94546E+00_JPRB ,&
&  0.92924E+00_JPRB, 0.94656E+00_JPRB, 0.10788E+01_JPRB, 0.11309E+01_JPRB, 0.11320E+01_JPRB, 0.95665E+00_JPRB, 0.79036E+00_JPRB ,&
&  0.77011E+00_JPRB, 0.74094E+00_JPRB, 0.71398E+00_JPRB, 0.70431E+00_JPRB, 0.69399E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.76627E+00_JPRB, 0.78063E+00_JPRB, 0.80194E+00_JPRB, 0.83608E+00_JPRB, 0.87398E+00_JPRB, 0.90047E+00_JPRB, 0.92673E+00_JPRB ,&
&  0.93679E+00_JPRB, 0.95094E+00_JPRB, 0.96333E+00_JPRB, 0.92129E+00_JPRB, 0.93240E+00_JPRB, 0.91662E+00_JPRB, 0.91140E+00_JPRB ,&
&  0.90916E+00_JPRB, 0.92907E+00_JPRB, 0.95419E+00_JPRB, 0.97833E+00_JPRB, 0.10138E+01_JPRB, 0.10589E+01_JPRB, 0.11222E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.12720E+01_JPRB, 0.13537E+01_JPRB, 0.14012E+01_JPRB, 0.14461E+01_JPRB, 0.14701E+01_JPRB, 0.14515E+01_JPRB ,&
&  0.87325E+00_JPRB, 0.14381E+01_JPRB, 0.15861E+01_JPRB, 0.16713E+01_JPRB, 0.17090E+01_JPRB, 0.15913E+01_JPRB, 0.15607E+01_JPRB ,&
&  0.14679E+01_JPRB, 0.13467E+01_JPRB, 0.13465E+01_JPRB, 0.14193E+01_JPRB, 0.15074E+01_JPRB, 0.14950E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.15094E+01_JPRB, 0.15490E+01_JPRB, 0.16010E+01_JPRB, 0.16598E+01_JPRB, 0.17388E+01_JPRB, 0.18317E+01_JPRB, 0.19637E+01_JPRB ,&
&  0.20942E+01_JPRB, 0.22211E+01_JPRB, 0.23264E+01_JPRB, 0.20947E+01_JPRB, 0.18565E+01_JPRB, 0.18354E+01_JPRB, 0.15536E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.84465E+00_JPRB, 0.92563E+00_JPRB, 0.10552E+01_JPRB, 0.10047E+01_JPRB, 0.92316E+00_JPRB, 0.86105E+00_JPRB ,&
&  0.83949E+00_JPRB, 0.80457E+00_JPRB, 0.82063E+00_JPRB, 0.79160E+00_JPRB, 0.81596E+00_JPRB, 0.81094E+00_JPRB, 0.81909E+00_JPRB ,&
&  0.82747E+00_JPRB, 0.80212E+00_JPRB, 0.79066E+00_JPRB, 0.78636E+00_JPRB, 0.77782E+00_JPRB, 0.76681E+00_JPRB, 0.75755E+00_JPRB ,&
&  0.76506E+00_JPRB, 0.72808E+00_JPRB, 0.70380E+00_JPRB, 0.71934E+00_JPRB, 0.75710E+00_JPRB, 0.80201E+00_JPRB, 0.89499E+00_JPRB ,&
&  0.10505E+01_JPRB, 0.12173E+01_JPRB, 0.15054E+01_JPRB, 0.15961E+01_JPRB, 0.15507E+01_JPRB, 0.11683E+01_JPRB, 0.85765E+00_JPRB ,&
&  0.85091E+00_JPRB, 0.80644E+00_JPRB, 0.77150E+00_JPRB, 0.75147E+00_JPRB, 0.75412E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.91621E+00_JPRB, 0.93896E+00_JPRB, 0.96414E+00_JPRB, 0.98597E+00_JPRB, 0.99202E+00_JPRB, 0.99401E+00_JPRB, 0.99328E+00_JPRB ,&
&  0.10185E+01_JPRB, 0.99751E+00_JPRB, 0.10225E+01_JPRB, 0.10002E+01_JPRB, 0.10219E+01_JPRB, 0.10065E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10584E+01_JPRB, 0.10946E+01_JPRB, 0.11117E+01_JPRB, 0.11163E+01_JPRB, 0.11535E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.13086E+01_JPRB, 0.13524E+01_JPRB, 0.13737E+01_JPRB, 0.13481E+01_JPRB, 0.13259E+01_JPRB, 0.95097E+00_JPRB ,&
&  0.13562E+01_JPRB, 0.15880E+01_JPRB, 0.17358E+01_JPRB, 0.16939E+01_JPRB, 0.15649E+01_JPRB, 0.14649E+01_JPRB, 0.14590E+01_JPRB ,&
&  0.14613E+01_JPRB, 0.14349E+01_JPRB, 0.14022E+01_JPRB, 0.14274E+01_JPRB, 0.16010E+01_JPRB, 0.15643E+01_JPRB, 0.15924E+01_JPRB ,&
&  0.16201E+01_JPRB, 0.16563E+01_JPRB, 0.17025E+01_JPRB, 0.17579E+01_JPRB, 0.18076E+01_JPRB, 0.18599E+01_JPRB, 0.19235E+01_JPRB ,&
&  0.19622E+01_JPRB, 0.19687E+01_JPRB, 0.19877E+01_JPRB, 0.17717E+01_JPRB, 0.16201E+01_JPRB, 0.15735E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.85313E+00_JPRB, 0.79169E+00_JPRB, 0.88980E+00_JPRB, 0.98144E+00_JPRB, 0.95209E+00_JPRB, 0.85731E+00_JPRB, 0.80143E+00_JPRB ,&
&  0.76783E+00_JPRB, 0.74738E+00_JPRB, 0.72605E+00_JPRB, 0.73820E+00_JPRB, 0.74012E+00_JPRB, 0.75294E+00_JPRB, 0.76270E+00_JPRB ,&
&  0.77489E+00_JPRB, 0.75483E+00_JPRB, 0.73157E+00_JPRB, 0.72062E+00_JPRB, 0.73342E+00_JPRB, 0.80238E+00_JPRB, 0.87603E+00_JPRB ,&
&  0.89989E+00_JPRB, 0.57712E+00_JPRB, 0.62386E+00_JPRB, 0.60707E+00_JPRB, 0.62577E+00_JPRB, 0.69745E+00_JPRB, 0.88079E+00_JPRB ,&
&  0.13409E+01_JPRB, 0.20769E+01_JPRB, 0.23464E+01_JPRB, 0.20101E+01_JPRB, 0.24794E+01_JPRB, 0.17732E+01_JPRB, 0.98407E+00_JPRB ,&
&  0.96320E+00_JPRB, 0.93240E+00_JPRB, 0.89427E+00_JPRB, 0.88275E+00_JPRB, 0.89103E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.97908E+00_JPRB, 0.99444E+00_JPRB, 0.99367E+00_JPRB, 0.10087E+01_JPRB, 0.98856E+00_JPRB, 0.10063E+01_JPRB, 0.10072E+01_JPRB ,&
&  0.10259E+01_JPRB, 0.10363E+01_JPRB, 0.10553E+01_JPRB, 0.10715E+01_JPRB, 0.10985E+01_JPRB, 0.11225E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.11951E+01_JPRB, 0.12186E+01_JPRB, 0.12372E+01_JPRB, 0.12214E+01_JPRB, 0.12291E+01_JPRB, 0.12322E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12498E+01_JPRB, 0.12607E+01_JPRB, 0.12669E+01_JPRB, 0.12517E+01_JPRB, 0.12860E+01_JPRB, 0.93841E+00_JPRB ,&
&  0.14062E+01_JPRB, 0.15799E+01_JPRB, 0.16316E+01_JPRB, 0.15322E+01_JPRB, 0.14294E+01_JPRB, 0.14034E+01_JPRB, 0.14089E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.14812E+01_JPRB, 0.14412E+01_JPRB, 0.15011E+01_JPRB, 0.15281E+01_JPRB, 0.14896E+01_JPRB, 0.15117E+01_JPRB ,&
&  0.15298E+01_JPRB, 0.15479E+01_JPRB, 0.15665E+01_JPRB, 0.15708E+01_JPRB, 0.15603E+01_JPRB, 0.15850E+01_JPRB, 0.15880E+01_JPRB ,&
&  0.15633E+01_JPRB, 0.15602E+01_JPRB, 0.15541E+01_JPRB, 0.14540E+01_JPRB, 0.13439E+01_JPRB, 0.11343E+01_JPRB, 0.94343E+00_JPRB ,&
&  0.72507E+00_JPRB, 0.71030E+00_JPRB, 0.80233E+00_JPRB, 0.95308E+00_JPRB, 0.92773E+00_JPRB, 0.87018E+00_JPRB, 0.80772E+00_JPRB ,&
&  0.73679E+00_JPRB, 0.71914E+00_JPRB, 0.69538E+00_JPRB, 0.68924E+00_JPRB, 0.69060E+00_JPRB, 0.70514E+00_JPRB, 0.71218E+00_JPRB ,&
&  0.72075E+00_JPRB, 0.72712E+00_JPRB, 0.70910E+00_JPRB, 0.68866E+00_JPRB, 0.68164E+00_JPRB, 0.71136E+00_JPRB, 0.63616E+00_JPRB ,&
&  0.71210E+00_JPRB, 0.59726E+00_JPRB, 0.58491E+00_JPRB, 0.56229E+00_JPRB, 0.52364E+00_JPRB, 0.56279E+00_JPRB, 0.74567E+00_JPRB ,&
&  0.12431E+01_JPRB, 0.16156E+01_JPRB, 0.23244E+01_JPRB, 0.39488E+01_JPRB, 0.27828E+01_JPRB, 0.20529E+01_JPRB, 0.10512E+01_JPRB ,&
&  0.98375E+00_JPRB, 0.95976E+00_JPRB, 0.94674E+00_JPRB, 0.93711E+00_JPRB, 0.95608E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.94529E+00_JPRB, 0.95725E+00_JPRB, 0.96871E+00_JPRB, 0.98080E+00_JPRB, 0.99004E+00_JPRB, 0.10087E+01_JPRB, 0.10157E+01_JPRB ,&
&  0.10279E+01_JPRB, 0.10346E+01_JPRB, 0.10507E+01_JPRB, 0.10709E+01_JPRB, 0.10874E+01_JPRB, 0.11120E+01_JPRB, 0.11414E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11991E+01_JPRB, 0.12275E+01_JPRB, 0.12392E+01_JPRB, 0.12294E+01_JPRB, 0.12340E+01_JPRB, 0.12355E+01_JPRB ,&
&  0.12222E+01_JPRB, 0.12126E+01_JPRB, 0.12242E+01_JPRB, 0.12420E+01_JPRB, 0.12374E+01_JPRB, 0.12474E+01_JPRB, 0.94857E+00_JPRB ,&
&  0.13426E+01_JPRB, 0.15049E+01_JPRB, 0.15159E+01_JPRB, 0.13880E+01_JPRB, 0.13088E+01_JPRB, 0.13100E+01_JPRB, 0.13384E+01_JPRB ,&
&  0.13951E+01_JPRB, 0.14539E+01_JPRB, 0.14140E+01_JPRB, 0.13334E+01_JPRB, 0.12810E+01_JPRB, 0.12742E+01_JPRB, 0.12726E+01_JPRB ,&
&  0.12763E+01_JPRB, 0.12811E+01_JPRB, 0.12713E+01_JPRB, 0.12445E+01_JPRB, 0.12164E+01_JPRB, 0.11905E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11034E+01_JPRB, 0.10727E+01_JPRB, 0.95681E+00_JPRB, 0.99045E+00_JPRB, 0.92280E+00_JPRB, 0.78472E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.54835E+00_JPRB, 0.58489E+00_JPRB, 0.73088E+00_JPRB, 0.94585E+00_JPRB, 0.93875E+00_JPRB, 0.89148E+00_JPRB, 0.85510E+00_JPRB ,&
&  0.79051E+00_JPRB, 0.73097E+00_JPRB, 0.68898E+00_JPRB, 0.66836E+00_JPRB, 0.66329E+00_JPRB, 0.67806E+00_JPRB, 0.68894E+00_JPRB ,&
&  0.68900E+00_JPRB, 0.68704E+00_JPRB, 0.67084E+00_JPRB, 0.65880E+00_JPRB, 0.64451E+00_JPRB, 0.58873E+00_JPRB, 0.67553E+00_JPRB ,&
&  0.63367E+00_JPRB, 0.50367E+00_JPRB, 0.59250E+00_JPRB, 0.39453E+00_JPRB, 0.50197E+00_JPRB, 0.53272E+00_JPRB, 0.64113E+00_JPRB ,&
&  0.77485E+00_JPRB, 0.10965E+01_JPRB, 0.15176E+01_JPRB, 0.13248E+01_JPRB, 0.11915E+01_JPRB, 0.10798E+01_JPRB, 0.98680E+00_JPRB ,&
&  0.94888E+00_JPRB, 0.93520E+00_JPRB, 0.93094E+00_JPRB, 0.92593E+00_JPRB, 0.93295E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.92575E+00_JPRB, 0.93563E+00_JPRB, 0.94509E+00_JPRB, 0.95923E+00_JPRB, 0.97230E+00_JPRB, 0.98941E+00_JPRB, 0.10013E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.10201E+01_JPRB, 0.10203E+01_JPRB, 0.10336E+01_JPRB, 0.10450E+01_JPRB, 0.10675E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.11525E+01_JPRB, 0.11861E+01_JPRB, 0.12049E+01_JPRB, 0.12194E+01_JPRB, 0.12371E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.12620E+01_JPRB, 0.12651E+01_JPRB, 0.12812E+01_JPRB, 0.13117E+01_JPRB, 0.13483E+01_JPRB, 0.13943E+01_JPRB, 0.11785E+01_JPRB ,&
&  0.13318E+01_JPRB, 0.14129E+01_JPRB, 0.13490E+01_JPRB, 0.12222E+01_JPRB, 0.11748E+01_JPRB, 0.11444E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11650E+01_JPRB, 0.11341E+01_JPRB, 0.10996E+01_JPRB, 0.10363E+01_JPRB, 0.10052E+01_JPRB, 0.10237E+01_JPRB, 0.10531E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10737E+01_JPRB, 0.10620E+01_JPRB, 0.10358E+01_JPRB, 0.10185E+01_JPRB, 0.99865E+00_JPRB, 0.95749E+00_JPRB ,&
&  0.91210E+00_JPRB, 0.89858E+00_JPRB, 0.76710E+00_JPRB, 0.75870E+00_JPRB, 0.75677E+00_JPRB, 0.69725E+00_JPRB, 0.59867E+00_JPRB ,&
&  0.51573E+00_JPRB, 0.50490E+00_JPRB, 0.58436E+00_JPRB, 0.90714E+00_JPRB, 0.10065E+01_JPRB, 0.98013E+00_JPRB, 0.93167E+00_JPRB ,&
&  0.86809E+00_JPRB, 0.81211E+00_JPRB, 0.73224E+00_JPRB, 0.68373E+00_JPRB, 0.66073E+00_JPRB, 0.67506E+00_JPRB, 0.68320E+00_JPRB ,&
&  0.67921E+00_JPRB, 0.66337E+00_JPRB, 0.64789E+00_JPRB, 0.62641E+00_JPRB, 0.63025E+00_JPRB, 0.56852E+00_JPRB, 0.63715E+00_JPRB ,&
&  0.59797E+00_JPRB, 0.42701E+00_JPRB, 0.45266E+00_JPRB, 0.57185E+00_JPRB, 0.54479E+00_JPRB, 0.60193E+00_JPRB, 0.71794E+00_JPRB ,&
&  0.92396E+00_JPRB, 0.10434E+01_JPRB, 0.10245E+01_JPRB, 0.99159E+00_JPRB, 0.93537E+00_JPRB, 0.93360E+00_JPRB, 0.91270E+00_JPRB ,&
&  0.89690E+00_JPRB, 0.89569E+00_JPRB, 0.89912E+00_JPRB, 0.90389E+00_JPRB, 0.91376E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.88709E+00_JPRB, 0.90254E+00_JPRB, 0.91518E+00_JPRB, 0.92402E+00_JPRB, 0.92834E+00_JPRB, 0.93215E+00_JPRB, 0.93836E+00_JPRB ,&
&  0.94046E+00_JPRB, 0.93789E+00_JPRB, 0.93584E+00_JPRB, 0.95047E+00_JPRB, 0.96965E+00_JPRB, 0.97829E+00_JPRB, 0.99979E+00_JPRB ,&
&  0.99457E+00_JPRB, 0.10064E+01_JPRB, 0.10246E+01_JPRB, 0.10382E+01_JPRB, 0.10615E+01_JPRB, 0.10623E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.10353E+01_JPRB, 0.10373E+01_JPRB, 0.10412E+01_JPRB, 0.10558E+01_JPRB, 0.10737E+01_JPRB, 0.12432E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.13640E+01_JPRB, 0.12156E+01_JPRB, 0.10658E+01_JPRB, 0.10506E+01_JPRB, 0.96991E+00_JPRB, 0.93031E+00_JPRB ,&
&  0.87756E+00_JPRB, 0.81997E+00_JPRB, 0.78762E+00_JPRB, 0.76673E+00_JPRB, 0.78969E+00_JPRB, 0.82594E+00_JPRB, 0.84790E+00_JPRB ,&
&  0.89109E+00_JPRB, 0.90765E+00_JPRB, 0.91723E+00_JPRB, 0.91702E+00_JPRB, 0.90826E+00_JPRB, 0.94295E+00_JPRB, 0.91296E+00_JPRB ,&
&  0.93432E+00_JPRB, 0.99346E+00_JPRB, 0.92453E+00_JPRB, 0.95484E+00_JPRB, 0.94550E+00_JPRB, 0.90214E+00_JPRB, 0.81646E+00_JPRB ,&
&  0.70165E+00_JPRB, 0.66821E+00_JPRB, 0.77566E+00_JPRB, 0.93375E+00_JPRB, 0.10421E+01_JPRB, 0.10850E+01_JPRB, 0.99961E+00_JPRB ,&
&  0.92021E+00_JPRB, 0.86905E+00_JPRB, 0.80356E+00_JPRB, 0.73750E+00_JPRB, 0.70965E+00_JPRB, 0.70828E+00_JPRB, 0.70211E+00_JPRB ,&
&  0.68497E+00_JPRB, 0.67492E+00_JPRB, 0.66736E+00_JPRB, 0.66658E+00_JPRB, 0.60187E+00_JPRB, 0.64462E+00_JPRB, 0.68382E+00_JPRB ,&
&  0.59068E+00_JPRB, 0.48124E+00_JPRB, 0.42950E+00_JPRB, 0.61719E+00_JPRB, 0.60087E+00_JPRB, 0.64672E+00_JPRB, 0.72052E+00_JPRB ,&
&  0.81573E+00_JPRB, 0.88714E+00_JPRB, 0.91035E+00_JPRB, 0.93490E+00_JPRB, 0.83144E+00_JPRB, 0.81155E+00_JPRB, 0.80544E+00_JPRB ,&
&  0.81199E+00_JPRB, 0.82496E+00_JPRB, 0.83920E+00_JPRB, 0.85307E+00_JPRB, 0.87049E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.81847E+00_JPRB, 0.83694E+00_JPRB, 0.85889E+00_JPRB, 0.86151E+00_JPRB, 0.86800E+00_JPRB, 0.86367E+00_JPRB, 0.86368E+00_JPRB ,&
&  0.85435E+00_JPRB, 0.85245E+00_JPRB, 0.85443E+00_JPRB, 0.85910E+00_JPRB, 0.86767E+00_JPRB, 0.87901E+00_JPRB, 0.90851E+00_JPRB ,&
&  0.90955E+00_JPRB, 0.91543E+00_JPRB, 0.92330E+00_JPRB, 0.93898E+00_JPRB, 0.95595E+00_JPRB, 0.95780E+00_JPRB, 0.93568E+00_JPRB ,&
&  0.92081E+00_JPRB, 0.92250E+00_JPRB, 0.93923E+00_JPRB, 0.97111E+00_JPRB, 0.11359E+01_JPRB, 0.14453E+01_JPRB, 0.19288E+01_JPRB ,&
&  0.17248E+01_JPRB, 0.13052E+01_JPRB, 0.11311E+01_JPRB, 0.96871E+00_JPRB, 0.95511E+00_JPRB, 0.97133E+00_JPRB, 0.92266E+00_JPRB ,&
&  0.89886E+00_JPRB, 0.85755E+00_JPRB, 0.84161E+00_JPRB, 0.84624E+00_JPRB, 0.85223E+00_JPRB, 0.83281E+00_JPRB, 0.82132E+00_JPRB ,&
&  0.78086E+00_JPRB, 0.76973E+00_JPRB, 0.79334E+00_JPRB, 0.84427E+00_JPRB, 0.97390E+00_JPRB, 0.10668E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.12431E+01_JPRB, 0.12228E+01_JPRB, 0.12419E+01_JPRB, 0.12463E+01_JPRB, 0.12038E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11435E+01_JPRB, 0.10236E+01_JPRB, 0.86512E+00_JPRB, 0.10027E+01_JPRB, 0.10890E+01_JPRB, 0.11648E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.95711E+00_JPRB, 0.88808E+00_JPRB, 0.82896E+00_JPRB, 0.76722E+00_JPRB, 0.73993E+00_JPRB, 0.76107E+00_JPRB, 0.76118E+00_JPRB ,&
&  0.72869E+00_JPRB, 0.73720E+00_JPRB, 0.75065E+00_JPRB, 0.76020E+00_JPRB, 0.82618E+00_JPRB, 0.70400E+00_JPRB, 0.78939E+00_JPRB ,&
&  0.65470E+00_JPRB, 0.62429E+00_JPRB, 0.53103E+00_JPRB, 0.73894E+00_JPRB, 0.66549E+00_JPRB, 0.65807E+00_JPRB, 0.68880E+00_JPRB ,&
&  0.72579E+00_JPRB, 0.74336E+00_JPRB, 0.76623E+00_JPRB, 0.78424E+00_JPRB, 0.77799E+00_JPRB, 0.76886E+00_JPRB, 0.77846E+00_JPRB ,&
&  0.78780E+00_JPRB, 0.78773E+00_JPRB, 0.79013E+00_JPRB, 0.79168E+00_JPRB, 0.80454E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.71517E+00_JPRB, 0.72395E+00_JPRB, 0.73950E+00_JPRB, 0.75151E+00_JPRB, 0.77364E+00_JPRB, 0.76683E+00_JPRB, 0.76038E+00_JPRB ,&
&  0.73379E+00_JPRB, 0.72956E+00_JPRB, 0.72338E+00_JPRB, 0.72146E+00_JPRB, 0.72607E+00_JPRB, 0.74833E+00_JPRB, 0.79104E+00_JPRB ,&
&  0.80423E+00_JPRB, 0.80938E+00_JPRB, 0.80415E+00_JPRB, 0.79557E+00_JPRB, 0.76674E+00_JPRB, 0.74205E+00_JPRB, 0.72530E+00_JPRB ,&
&  0.75236E+00_JPRB, 0.95283E+00_JPRB, 0.12097E+01_JPRB, 0.14520E+01_JPRB, 0.17340E+01_JPRB, 0.16320E+01_JPRB, 0.18841E+01_JPRB ,&
&  0.19729E+01_JPRB, 0.13482E+01_JPRB, 0.12831E+01_JPRB, 0.11160E+01_JPRB, 0.10980E+01_JPRB, 0.10805E+01_JPRB, 0.10721E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.11018E+01_JPRB, 0.10887E+01_JPRB, 0.10851E+01_JPRB, 0.11022E+01_JPRB, 0.11290E+01_JPRB, 0.11430E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.11178E+01_JPRB, 0.11172E+01_JPRB, 0.11976E+01_JPRB, 0.12782E+01_JPRB, 0.13282E+01_JPRB, 0.13842E+01_JPRB ,&
&  0.14668E+01_JPRB, 0.14983E+01_JPRB, 0.13749E+01_JPRB, 0.13682E+01_JPRB, 0.14438E+01_JPRB, 0.13378E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.12586E+01_JPRB, 0.89606E+00_JPRB, 0.10292E+01_JPRB, 0.11363E+01_JPRB, 0.10993E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.96497E+00_JPRB, 0.85149E+00_JPRB, 0.80535E+00_JPRB, 0.81341E+00_JPRB, 0.87773E+00_JPRB, 0.88097E+00_JPRB ,&
&  0.86305E+00_JPRB, 0.80334E+00_JPRB, 0.85826E+00_JPRB, 0.91929E+00_JPRB, 0.94394E+00_JPRB, 0.91721E+00_JPRB, 0.86654E+00_JPRB ,&
&  0.75311E+00_JPRB, 0.69222E+00_JPRB, 0.73503E+00_JPRB, 0.75998E+00_JPRB, 0.61511E+00_JPRB, 0.68330E+00_JPRB, 0.69008E+00_JPRB ,&
&  0.69855E+00_JPRB, 0.69131E+00_JPRB, 0.67836E+00_JPRB, 0.67920E+00_JPRB, 0.67155E+00_JPRB, 0.66421E+00_JPRB, 0.67933E+00_JPRB ,&
&  0.70121E+00_JPRB, 0.69895E+00_JPRB, 0.69618E+00_JPRB, 0.69652E+00_JPRB, 0.70336E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.62094E+00_JPRB, 0.62042E+00_JPRB, 0.61943E+00_JPRB, 0.62379E+00_JPRB, 0.63458E+00_JPRB, 0.63689E+00_JPRB, 0.63005E+00_JPRB ,&
&  0.61423E+00_JPRB, 0.60667E+00_JPRB, 0.58635E+00_JPRB, 0.56239E+00_JPRB, 0.56271E+00_JPRB, 0.60745E+00_JPRB, 0.65528E+00_JPRB ,&
&  0.68700E+00_JPRB, 0.70370E+00_JPRB, 0.70892E+00_JPRB, 0.71556E+00_JPRB, 0.75847E+00_JPRB, 0.79875E+00_JPRB, 0.10140E+01_JPRB ,&
&  0.13911E+01_JPRB, 0.19202E+01_JPRB, 0.23102E+01_JPRB, 0.25137E+01_JPRB, 0.23788E+01_JPRB, 0.20150E+01_JPRB, 0.22864E+01_JPRB ,&
&  0.20244E+01_JPRB, 0.15096E+01_JPRB, 0.13155E+01_JPRB, 0.12338E+01_JPRB, 0.11973E+01_JPRB, 0.11634E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11886E+01_JPRB, 0.12077E+01_JPRB, 0.12320E+01_JPRB, 0.12555E+01_JPRB, 0.12869E+01_JPRB, 0.13197E+01_JPRB ,&
&  0.13499E+01_JPRB, 0.13473E+01_JPRB, 0.13977E+01_JPRB, 0.12910E+01_JPRB, 0.13292E+01_JPRB, 0.13768E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.13673E+01_JPRB, 0.14087E+01_JPRB, 0.14289E+01_JPRB, 0.14437E+01_JPRB, 0.14020E+01_JPRB, 0.13561E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.13064E+01_JPRB, 0.10088E+01_JPRB, 0.11101E+01_JPRB, 0.12546E+01_JPRB, 0.10531E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.11324E+01_JPRB, 0.10541E+01_JPRB, 0.98152E+00_JPRB, 0.94045E+00_JPRB, 0.98976E+00_JPRB, 0.10569E+01_JPRB, 0.10007E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.99910E+00_JPRB, 0.10502E+01_JPRB, 0.11130E+01_JPRB, 0.11368E+01_JPRB, 0.11548E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.87460E+00_JPRB, 0.78927E+00_JPRB, 0.72577E+00_JPRB, 0.74420E+00_JPRB, 0.65238E+00_JPRB, 0.67598E+00_JPRB, 0.69115E+00_JPRB ,&
&  0.69377E+00_JPRB, 0.67790E+00_JPRB, 0.67638E+00_JPRB, 0.67388E+00_JPRB, 0.67615E+00_JPRB, 0.66390E+00_JPRB, 0.66953E+00_JPRB ,&
&  0.66740E+00_JPRB, 0.66129E+00_JPRB, 0.65241E+00_JPRB, 0.64992E+00_JPRB, 0.63426E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.66889E+00_JPRB, 0.65608E+00_JPRB, 0.63602E+00_JPRB, 0.61385E+00_JPRB, 0.59239E+00_JPRB, 0.56457E+00_JPRB, 0.56607E+00_JPRB ,&
&  0.55646E+00_JPRB, 0.56546E+00_JPRB, 0.58862E+00_JPRB, 0.60899E+00_JPRB, 0.64538E+00_JPRB, 0.70318E+00_JPRB, 0.76038E+00_JPRB ,&
&  0.80381E+00_JPRB, 0.82707E+00_JPRB, 0.89438E+00_JPRB, 0.99924E+00_JPRB, 0.11754E+01_JPRB, 0.14652E+01_JPRB, 0.19143E+01_JPRB ,&
&  0.21244E+01_JPRB, 0.22542E+01_JPRB, 0.22253E+01_JPRB, 0.20443E+01_JPRB, 0.17704E+01_JPRB, 0.17305E+01_JPRB, 0.19294E+01_JPRB ,&
&  0.19462E+01_JPRB, 0.17092E+01_JPRB, 0.14033E+01_JPRB, 0.13256E+01_JPRB, 0.12744E+01_JPRB, 0.12427E+01_JPRB, 0.12326E+01_JPRB ,&
&  0.12425E+01_JPRB, 0.12518E+01_JPRB, 0.12522E+01_JPRB, 0.12738E+01_JPRB, 0.12866E+01_JPRB, 0.12994E+01_JPRB, 0.13193E+01_JPRB ,&
&  0.13250E+01_JPRB, 0.13145E+01_JPRB, 0.13102E+01_JPRB, 0.12831E+01_JPRB, 0.12954E+01_JPRB, 0.13147E+01_JPRB, 0.13164E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.12552E+01_JPRB, 0.12895E+01_JPRB, 0.13425E+01_JPRB, 0.13174E+01_JPRB, 0.12407E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.13631E+01_JPRB, 0.10628E+01_JPRB, 0.13254E+01_JPRB, 0.15520E+01_JPRB, 0.13787E+01_JPRB, 0.13108E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.11942E+01_JPRB, 0.11915E+01_JPRB, 0.11507E+01_JPRB, 0.12172E+01_JPRB, 0.12395E+01_JPRB, 0.11485E+01_JPRB ,&
&  0.13718E+01_JPRB, 0.13237E+01_JPRB, 0.13947E+01_JPRB, 0.14387E+01_JPRB, 0.14133E+01_JPRB, 0.13236E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.96552E+00_JPRB, 0.10191E+01_JPRB, 0.10342E+01_JPRB, 0.11840E+01_JPRB, 0.11473E+01_JPRB, 0.70230E+00_JPRB, 0.73171E+00_JPRB ,&
&  0.73390E+00_JPRB, 0.73619E+00_JPRB, 0.74227E+00_JPRB, 0.74099E+00_JPRB, 0.73788E+00_JPRB, 0.73619E+00_JPRB, 0.73995E+00_JPRB ,&
&  0.73144E+00_JPRB, 0.72070E+00_JPRB, 0.70936E+00_JPRB, 0.70058E+00_JPRB, 0.68705E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.78138E+00_JPRB, 0.77592E+00_JPRB, 0.75639E+00_JPRB, 0.71958E+00_JPRB, 0.67556E+00_JPRB, 0.63150E+00_JPRB, 0.60568E+00_JPRB ,&
&  0.58207E+00_JPRB, 0.60333E+00_JPRB, 0.64958E+00_JPRB, 0.71158E+00_JPRB, 0.78600E+00_JPRB, 0.86536E+00_JPRB, 0.94405E+00_JPRB ,&
&  0.99672E+00_JPRB, 0.10512E+01_JPRB, 0.11420E+01_JPRB, 0.13447E+01_JPRB, 0.15658E+01_JPRB, 0.17241E+01_JPRB, 0.18035E+01_JPRB ,&
&  0.18593E+01_JPRB, 0.18905E+01_JPRB, 0.18124E+01_JPRB, 0.14384E+01_JPRB, 0.12380E+01_JPRB, 0.14928E+01_JPRB, 0.14555E+01_JPRB ,&
&  0.14102E+01_JPRB, 0.13212E+01_JPRB, 0.12931E+01_JPRB, 0.12928E+01_JPRB, 0.12515E+01_JPRB, 0.12189E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.12118E+01_JPRB, 0.12284E+01_JPRB, 0.12447E+01_JPRB, 0.12437E+01_JPRB, 0.12602E+01_JPRB, 0.12862E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.12711E+01_JPRB, 0.12696E+01_JPRB, 0.12684E+01_JPRB, 0.12898E+01_JPRB, 0.12661E+01_JPRB, 0.12596E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.12133E+01_JPRB, 0.11957E+01_JPRB, 0.12111E+01_JPRB, 0.12705E+01_JPRB, 0.12934E+01_JPRB, 0.11805E+01_JPRB, 0.12076E+01_JPRB ,&
&  0.12992E+01_JPRB, 0.13462E+01_JPRB, 0.12601E+01_JPRB, 0.15481E+01_JPRB, 0.10655E+01_JPRB, 0.12322E+01_JPRB, 0.14310E+01_JPRB ,&
&  0.14127E+01_JPRB, 0.13109E+01_JPRB, 0.12701E+01_JPRB, 0.12759E+01_JPRB, 0.12709E+01_JPRB, 0.12608E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.16342E+01_JPRB, 0.16559E+01_JPRB, 0.16984E+01_JPRB, 0.17017E+01_JPRB, 0.17066E+01_JPRB, 0.16226E+01_JPRB, 0.15438E+01_JPRB ,&
&  0.13171E+01_JPRB, 0.11587E+01_JPRB, 0.10364E+01_JPRB, 0.99607E+00_JPRB, 0.91722E+00_JPRB, 0.70035E+00_JPRB, 0.73411E+00_JPRB ,&
&  0.74694E+00_JPRB, 0.74444E+00_JPRB, 0.75484E+00_JPRB, 0.75959E+00_JPRB, 0.75019E+00_JPRB, 0.75164E+00_JPRB, 0.76375E+00_JPRB ,&
&  0.76340E+00_JPRB, 0.76431E+00_JPRB, 0.76821E+00_JPRB, 0.76919E+00_JPRB, 0.77626E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.79183E+00_JPRB, 0.80346E+00_JPRB, 0.82042E+00_JPRB, 0.82398E+00_JPRB, 0.81713E+00_JPRB, 0.79477E+00_JPRB, 0.74125E+00_JPRB ,&
&  0.69045E+00_JPRB, 0.66918E+00_JPRB, 0.70475E+00_JPRB, 0.74896E+00_JPRB, 0.79950E+00_JPRB, 0.86066E+00_JPRB, 0.94937E+00_JPRB ,&
&  0.10602E+01_JPRB, 0.11575E+01_JPRB, 0.12616E+01_JPRB, 0.14333E+01_JPRB, 0.16199E+01_JPRB, 0.16436E+01_JPRB, 0.16608E+01_JPRB ,&
&  0.14709E+01_JPRB, 0.12285E+01_JPRB, 0.16933E+01_JPRB, 0.19163E+01_JPRB, 0.17326E+01_JPRB, 0.14368E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.12900E+01_JPRB, 0.13059E+01_JPRB, 0.13065E+01_JPRB, 0.11946E+01_JPRB, 0.11882E+01_JPRB, 0.11781E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.11622E+01_JPRB, 0.11755E+01_JPRB, 0.11852E+01_JPRB, 0.11762E+01_JPRB, 0.11946E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.11953E+01_JPRB, 0.12102E+01_JPRB, 0.12169E+01_JPRB, 0.12297E+01_JPRB, 0.12320E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.11690E+01_JPRB, 0.11331E+01_JPRB, 0.11755E+01_JPRB, 0.12619E+01_JPRB, 0.13288E+01_JPRB, 0.12317E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.13628E+01_JPRB, 0.13687E+01_JPRB, 0.15476E+01_JPRB, 0.11940E+01_JPRB, 0.13182E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.14689E+01_JPRB, 0.14364E+01_JPRB, 0.13928E+01_JPRB, 0.13994E+01_JPRB, 0.14383E+01_JPRB, 0.13504E+01_JPRB, 0.14875E+01_JPRB ,&
&  0.16491E+01_JPRB, 0.17849E+01_JPRB, 0.18294E+01_JPRB, 0.18858E+01_JPRB, 0.17694E+01_JPRB, 0.17795E+01_JPRB, 0.18036E+01_JPRB ,&
&  0.18637E+01_JPRB, 0.14337E+01_JPRB, 0.11177E+01_JPRB, 0.90173E+00_JPRB, 0.79276E+00_JPRB, 0.68643E+00_JPRB, 0.72111E+00_JPRB ,&
&  0.73730E+00_JPRB, 0.74444E+00_JPRB, 0.74327E+00_JPRB, 0.75169E+00_JPRB, 0.75707E+00_JPRB, 0.75679E+00_JPRB, 0.76737E+00_JPRB ,&
&  0.77276E+00_JPRB, 0.77683E+00_JPRB, 0.77814E+00_JPRB, 0.78030E+00_JPRB, 0.78385E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.77983E+00_JPRB, 0.80413E+00_JPRB, 0.84302E+00_JPRB, 0.90642E+00_JPRB, 0.96920E+00_JPRB, 0.10659E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.10255E+01_JPRB, 0.69547E+00_JPRB, 0.71758E+00_JPRB, 0.73987E+00_JPRB, 0.77264E+00_JPRB, 0.81959E+00_JPRB, 0.88400E+00_JPRB ,&
&  0.99096E+00_JPRB, 0.11300E+01_JPRB, 0.12986E+01_JPRB, 0.14463E+01_JPRB, 0.15811E+01_JPRB, 0.16234E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.12335E+01_JPRB, 0.13868E+01_JPRB, 0.21246E+01_JPRB, 0.20106E+01_JPRB, 0.18581E+01_JPRB, 0.16560E+01_JPRB, 0.13956E+01_JPRB ,&
&  0.12760E+01_JPRB, 0.11975E+01_JPRB, 0.11906E+01_JPRB, 0.11641E+01_JPRB, 0.10935E+01_JPRB, 0.10973E+01_JPRB, 0.11070E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.10897E+01_JPRB, 0.10766E+01_JPRB, 0.10864E+01_JPRB, 0.10763E+01_JPRB, 0.10759E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11423E+01_JPRB, 0.11498E+01_JPRB, 0.11365E+01_JPRB, 0.11722E+01_JPRB, 0.12092E+01_JPRB, 0.11897E+01_JPRB ,&
&  0.11463E+01_JPRB, 0.11487E+01_JPRB, 0.11671E+01_JPRB, 0.11661E+01_JPRB, 0.11214E+01_JPRB, 0.12523E+01_JPRB, 0.12732E+01_JPRB ,&
&  0.13649E+01_JPRB, 0.13804E+01_JPRB, 0.15348E+01_JPRB, 0.12779E+01_JPRB, 0.13751E+01_JPRB, 0.14823E+01_JPRB, 0.14783E+01_JPRB ,&
&  0.14844E+01_JPRB, 0.15979E+01_JPRB, 0.15971E+01_JPRB, 0.16475E+01_JPRB, 0.16697E+01_JPRB, 0.15265E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.16628E+01_JPRB, 0.19158E+01_JPRB, 0.19323E+01_JPRB, 0.19111E+01_JPRB, 0.16275E+01_JPRB, 0.15875E+01_JPRB, 0.16641E+01_JPRB ,&
&  0.18081E+01_JPRB, 0.17285E+01_JPRB, 0.14415E+01_JPRB, 0.11459E+01_JPRB, 0.87815E+00_JPRB, 0.74408E+00_JPRB, 0.73579E+00_JPRB ,&
&  0.72755E+00_JPRB, 0.72986E+00_JPRB, 0.73846E+00_JPRB, 0.75392E+00_JPRB, 0.76357E+00_JPRB, 0.76350E+00_JPRB, 0.76076E+00_JPRB ,&
&  0.76277E+00_JPRB, 0.76405E+00_JPRB, 0.76211E+00_JPRB, 0.76449E+00_JPRB, 0.77017E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.74774E+00_JPRB, 0.75363E+00_JPRB, 0.77559E+00_JPRB, 0.80278E+00_JPRB, 0.82716E+00_JPRB, 0.86324E+00_JPRB, 0.85179E+00_JPRB ,&
&  0.76397E+00_JPRB, 0.71182E+00_JPRB, 0.73125E+00_JPRB, 0.72971E+00_JPRB, 0.73457E+00_JPRB, 0.76138E+00_JPRB, 0.80868E+00_JPRB ,&
&  0.87048E+00_JPRB, 0.99406E+00_JPRB, 0.11682E+01_JPRB, 0.14035E+01_JPRB, 0.14986E+01_JPRB, 0.15429E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.11891E+01_JPRB, 0.22176E+01_JPRB, 0.19971E+01_JPRB, 0.20735E+01_JPRB, 0.19837E+01_JPRB, 0.19886E+01_JPRB, 0.16219E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.11722E+01_JPRB, 0.11468E+01_JPRB, 0.10637E+01_JPRB, 0.10138E+01_JPRB, 0.10103E+01_JPRB, 0.10250E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.10205E+01_JPRB, 0.10095E+01_JPRB, 0.10137E+01_JPRB, 0.10123E+01_JPRB, 0.10172E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.10907E+01_JPRB, 0.10877E+01_JPRB, 0.11000E+01_JPRB, 0.11215E+01_JPRB, 0.11782E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.10804E+01_JPRB, 0.11305E+01_JPRB, 0.11564E+01_JPRB, 0.11704E+01_JPRB, 0.12606E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.13955E+01_JPRB, 0.13824E+01_JPRB, 0.15494E+01_JPRB, 0.13186E+01_JPRB, 0.13788E+01_JPRB, 0.15775E+01_JPRB, 0.15572E+01_JPRB ,&
&  0.15091E+01_JPRB, 0.17339E+01_JPRB, 0.17654E+01_JPRB, 0.17988E+01_JPRB, 0.17726E+01_JPRB, 0.16697E+01_JPRB, 0.15957E+01_JPRB ,&
&  0.16711E+01_JPRB, 0.18249E+01_JPRB, 0.20105E+01_JPRB, 0.17874E+01_JPRB, 0.17142E+01_JPRB, 0.15731E+01_JPRB, 0.16941E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.18984E+01_JPRB, 0.17354E+01_JPRB, 0.15065E+01_JPRB, 0.11588E+01_JPRB, 0.96006E+00_JPRB, 0.84315E+00_JPRB ,&
&  0.77999E+00_JPRB, 0.74378E+00_JPRB, 0.73800E+00_JPRB, 0.74885E+00_JPRB, 0.76239E+00_JPRB, 0.77006E+00_JPRB, 0.76407E+00_JPRB ,&
&  0.76167E+00_JPRB, 0.76326E+00_JPRB, 0.76403E+00_JPRB, 0.76077E+00_JPRB, 0.75543E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.76783E+00_JPRB, 0.74493E+00_JPRB, 0.74650E+00_JPRB, 0.76576E+00_JPRB, 0.76786E+00_JPRB, 0.76846E+00_JPRB, 0.74551E+00_JPRB ,&
&  0.68367E+00_JPRB, 0.67138E+00_JPRB, 0.67678E+00_JPRB, 0.68170E+00_JPRB, 0.68797E+00_JPRB, 0.71916E+00_JPRB, 0.76395E+00_JPRB ,&
&  0.81670E+00_JPRB, 0.89614E+00_JPRB, 0.97736E+00_JPRB, 0.11544E+01_JPRB, 0.13102E+01_JPRB, 0.15036E+01_JPRB, 0.10830E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.20907E+01_JPRB, 0.21170E+01_JPRB, 0.21631E+01_JPRB, 0.21611E+01_JPRB, 0.21230E+01_JPRB, 0.17194E+01_JPRB ,&
&  0.14720E+01_JPRB, 0.12937E+01_JPRB, 0.11532E+01_JPRB, 0.10312E+01_JPRB, 0.94417E+00_JPRB, 0.95674E+00_JPRB, 0.96892E+00_JPRB ,&
&  0.98538E+00_JPRB, 0.10151E+01_JPRB, 0.97667E+00_JPRB, 0.96870E+00_JPRB, 0.95936E+00_JPRB, 0.95063E+00_JPRB, 0.98493E+00_JPRB ,&
&  0.10256E+01_JPRB, 0.10509E+01_JPRB, 0.10743E+01_JPRB, 0.10478E+01_JPRB, 0.11098E+01_JPRB, 0.11542E+01_JPRB, 0.11900E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.10541E+01_JPRB, 0.11388E+01_JPRB, 0.12554E+01_JPRB, 0.13379E+01_JPRB, 0.13388E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.13687E+01_JPRB, 0.14174E+01_JPRB, 0.14354E+01_JPRB, 0.13144E+01_JPRB, 0.14123E+01_JPRB, 0.16793E+01_JPRB, 0.17746E+01_JPRB ,&
&  0.17973E+01_JPRB, 0.18205E+01_JPRB, 0.18453E+01_JPRB, 0.17798E+01_JPRB, 0.17368E+01_JPRB, 0.16519E+01_JPRB, 0.17366E+01_JPRB ,&
&  0.19631E+01_JPRB, 0.21052E+01_JPRB, 0.18753E+01_JPRB, 0.16650E+01_JPRB, 0.17444E+01_JPRB, 0.15891E+01_JPRB, 0.15229E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.17793E+01_JPRB, 0.16616E+01_JPRB, 0.16653E+01_JPRB, 0.14283E+01_JPRB, 0.12640E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.89871E+00_JPRB, 0.84402E+00_JPRB, 0.80766E+00_JPRB, 0.78630E+00_JPRB, 0.79293E+00_JPRB, 0.79287E+00_JPRB, 0.79054E+00_JPRB ,&
&  0.77918E+00_JPRB, 0.77997E+00_JPRB, 0.78648E+00_JPRB, 0.79365E+00_JPRB, 0.78931E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.81390E+00_JPRB, 0.79201E+00_JPRB, 0.76134E+00_JPRB, 0.74400E+00_JPRB, 0.74877E+00_JPRB, 0.72467E+00_JPRB, 0.69613E+00_JPRB ,&
&  0.65344E+00_JPRB, 0.63398E+00_JPRB, 0.61154E+00_JPRB, 0.60667E+00_JPRB, 0.61619E+00_JPRB, 0.64742E+00_JPRB, 0.69840E+00_JPRB ,&
&  0.76678E+00_JPRB, 0.82798E+00_JPRB, 0.87944E+00_JPRB, 0.94129E+00_JPRB, 0.10426E+01_JPRB, 0.99878E+00_JPRB, 0.11042E+01_JPRB ,&
&  0.15504E+01_JPRB, 0.19282E+01_JPRB, 0.20604E+01_JPRB, 0.21355E+01_JPRB, 0.22475E+01_JPRB, 0.23642E+01_JPRB, 0.20345E+01_JPRB ,&
&  0.17383E+01_JPRB, 0.15110E+01_JPRB, 0.12618E+01_JPRB, 0.10932E+01_JPRB, 0.97058E+00_JPRB, 0.90862E+00_JPRB, 0.91725E+00_JPRB ,&
&  0.92682E+00_JPRB, 0.95010E+00_JPRB, 0.93703E+00_JPRB, 0.91810E+00_JPRB, 0.90664E+00_JPRB, 0.89297E+00_JPRB, 0.91205E+00_JPRB ,&
&  0.95240E+00_JPRB, 0.98938E+00_JPRB, 0.10160E+01_JPRB, 0.10221E+01_JPRB, 0.10716E+01_JPRB, 0.11218E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.11953E+01_JPRB, 0.12389E+01_JPRB, 0.12782E+01_JPRB, 0.14002E+01_JPRB, 0.14823E+01_JPRB, 0.14752E+01_JPRB ,&
&  0.14331E+01_JPRB, 0.13441E+01_JPRB, 0.11881E+01_JPRB, 0.12650E+01_JPRB, 0.14995E+01_JPRB, 0.18627E+01_JPRB, 0.17038E+01_JPRB ,&
&  0.14709E+01_JPRB, 0.14311E+01_JPRB, 0.15401E+01_JPRB, 0.17816E+01_JPRB, 0.19485E+01_JPRB, 0.18798E+01_JPRB, 0.21256E+01_JPRB ,&
&  0.18048E+01_JPRB, 0.97328E+00_JPRB, 0.83941E+00_JPRB, 0.11082E+01_JPRB, 0.92748E+00_JPRB, 0.10519E+01_JPRB, 0.15535E+01_JPRB ,&
&  0.16667E+01_JPRB, 0.18939E+01_JPRB, 0.19198E+01_JPRB, 0.15746E+01_JPRB, 0.14610E+01_JPRB, 0.13829E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.10342E+01_JPRB, 0.96654E+00_JPRB, 0.92524E+00_JPRB, 0.90009E+00_JPRB, 0.87354E+00_JPRB, 0.85921E+00_JPRB ,&
&  0.84299E+00_JPRB, 0.83061E+00_JPRB, 0.82513E+00_JPRB, 0.82238E+00_JPRB, 0.82428E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.86772E+00_JPRB, 0.84986E+00_JPRB, 0.82575E+00_JPRB, 0.79097E+00_JPRB, 0.76803E+00_JPRB, 0.74023E+00_JPRB, 0.69539E+00_JPRB ,&
&  0.64895E+00_JPRB, 0.62392E+00_JPRB, 0.59829E+00_JPRB, 0.58021E+00_JPRB, 0.58490E+00_JPRB, 0.60116E+00_JPRB, 0.64330E+00_JPRB ,&
&  0.71227E+00_JPRB, 0.80398E+00_JPRB, 0.89697E+00_JPRB, 0.92935E+00_JPRB, 0.93035E+00_JPRB, 0.87736E+00_JPRB, 0.10785E+01_JPRB ,&
&  0.13630E+01_JPRB, 0.15990E+01_JPRB, 0.20401E+01_JPRB, 0.22545E+01_JPRB, 0.23319E+01_JPRB, 0.23418E+01_JPRB, 0.22185E+01_JPRB ,&
&  0.20104E+01_JPRB, 0.17654E+01_JPRB, 0.14335E+01_JPRB, 0.11963E+01_JPRB, 0.10360E+01_JPRB, 0.92940E+00_JPRB, 0.87400E+00_JPRB ,&
&  0.88954E+00_JPRB, 0.91322E+00_JPRB, 0.90992E+00_JPRB, 0.88112E+00_JPRB, 0.86310E+00_JPRB, 0.85736E+00_JPRB, 0.86425E+00_JPRB ,&
&  0.88257E+00_JPRB, 0.91589E+00_JPRB, 0.95137E+00_JPRB, 0.10092E+01_JPRB, 0.88682E+00_JPRB, 0.10709E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.12756E+01_JPRB, 0.12370E+01_JPRB, 0.13652E+01_JPRB, 0.15516E+01_JPRB, 0.15944E+01_JPRB, 0.16086E+01_JPRB ,&
&  0.15922E+01_JPRB, 0.14205E+01_JPRB, 0.11975E+01_JPRB, 0.12864E+01_JPRB, 0.15135E+01_JPRB, 0.19618E+01_JPRB, 0.12769E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.15811E+01_JPRB, 0.16263E+01_JPRB, 0.15379E+01_JPRB, 0.20797E+01_JPRB, 0.23779E+01_JPRB, 0.16158E+01_JPRB ,&
&  0.63358E+00_JPRB, 0.52906E+00_JPRB, 0.57083E+00_JPRB, 0.54378E+00_JPRB, 0.55760E+00_JPRB, 0.79533E+00_JPRB, 0.21015E+01_JPRB ,&
&  0.16382E+01_JPRB, 0.18683E+01_JPRB, 0.18594E+01_JPRB, 0.17851E+01_JPRB, 0.15988E+01_JPRB, 0.14915E+01_JPRB, 0.15355E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.11969E+01_JPRB, 0.10701E+01_JPRB, 0.10067E+01_JPRB, 0.98163E+00_JPRB, 0.97053E+00_JPRB, 0.94676E+00_JPRB ,&
&  0.93343E+00_JPRB, 0.92434E+00_JPRB, 0.91175E+00_JPRB, 0.89924E+00_JPRB, 0.88567E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.90313E+00_JPRB, 0.89042E+00_JPRB, 0.87539E+00_JPRB, 0.85082E+00_JPRB, 0.81994E+00_JPRB, 0.78119E+00_JPRB, 0.73232E+00_JPRB ,&
&  0.67932E+00_JPRB, 0.63848E+00_JPRB, 0.61363E+00_JPRB, 0.59602E+00_JPRB, 0.59003E+00_JPRB, 0.59126E+00_JPRB, 0.61971E+00_JPRB ,&
&  0.68048E+00_JPRB, 0.79139E+00_JPRB, 0.98218E+00_JPRB, 0.92925E+00_JPRB, 0.94506E+00_JPRB, 0.88253E+00_JPRB, 0.95191E+00_JPRB ,&
&  0.12018E+01_JPRB, 0.14107E+01_JPRB, 0.17491E+01_JPRB, 0.20823E+01_JPRB, 0.24619E+01_JPRB, 0.25052E+01_JPRB, 0.24732E+01_JPRB ,&
&  0.22749E+01_JPRB, 0.20367E+01_JPRB, 0.16765E+01_JPRB, 0.13621E+01_JPRB, 0.11152E+01_JPRB, 0.96915E+00_JPRB, 0.88523E+00_JPRB ,&
&  0.87451E+00_JPRB, 0.89379E+00_JPRB, 0.88925E+00_JPRB, 0.85162E+00_JPRB, 0.82826E+00_JPRB, 0.82189E+00_JPRB, 0.83212E+00_JPRB ,&
&  0.83708E+00_JPRB, 0.85573E+00_JPRB, 0.87631E+00_JPRB, 0.94167E+00_JPRB, 0.86845E+00_JPRB, 0.81614E+00_JPRB, 0.10317E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.14116E+01_JPRB, 0.14041E+01_JPRB, 0.16174E+01_JPRB, 0.17797E+01_JPRB, 0.17973E+01_JPRB, 0.18945E+01_JPRB ,&
&  0.17841E+01_JPRB, 0.18904E+01_JPRB, 0.13656E+01_JPRB, 0.13729E+01_JPRB, 0.16297E+01_JPRB, 0.14059E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.15763E+01_JPRB, 0.14524E+01_JPRB, 0.14503E+01_JPRB, 0.12446E+01_JPRB, 0.18285E+01_JPRB, 0.18102E+01_JPRB, 0.72092E+00_JPRB ,&
&  0.57821E+00_JPRB, 0.54530E+00_JPRB, 0.51999E+00_JPRB, 0.50317E+00_JPRB, 0.56093E+00_JPRB, 0.77735E+00_JPRB, 0.11783E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.17285E+01_JPRB, 0.17468E+01_JPRB, 0.18136E+01_JPRB, 0.16403E+01_JPRB, 0.15978E+01_JPRB, 0.16446E+01_JPRB ,&
&  0.15386E+01_JPRB, 0.14699E+01_JPRB, 0.12891E+01_JPRB, 0.11595E+01_JPRB, 0.10899E+01_JPRB, 0.10461E+01_JPRB, 0.10064E+01_JPRB ,&
&  0.97514E+00_JPRB, 0.96098E+00_JPRB, 0.94783E+00_JPRB, 0.93071E+00_JPRB, 0.91546E+00_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.91471E+00_JPRB, 0.90728E+00_JPRB, 0.89731E+00_JPRB, 0.88194E+00_JPRB, 0.85955E+00_JPRB, 0.82630E+00_JPRB, 0.77973E+00_JPRB ,&
&  0.72880E+00_JPRB, 0.68519E+00_JPRB, 0.65147E+00_JPRB, 0.63235E+00_JPRB, 0.61434E+00_JPRB, 0.60123E+00_JPRB, 0.61428E+00_JPRB ,&
&  0.66579E+00_JPRB, 0.79514E+00_JPRB, 0.88925E+00_JPRB, 0.81443E+00_JPRB, 0.78331E+00_JPRB, 0.77224E+00_JPRB, 0.77462E+00_JPRB ,&
&  0.10568E+01_JPRB, 0.12590E+01_JPRB, 0.15161E+01_JPRB, 0.19826E+01_JPRB, 0.24249E+01_JPRB, 0.24469E+01_JPRB, 0.26357E+01_JPRB ,&
&  0.24779E+01_JPRB, 0.22149E+01_JPRB, 0.18894E+01_JPRB, 0.15462E+01_JPRB, 0.12666E+01_JPRB, 0.10517E+01_JPRB, 0.94825E+00_JPRB ,&
&  0.89909E+00_JPRB, 0.90160E+00_JPRB, 0.88817E+00_JPRB, 0.84698E+00_JPRB, 0.82613E+00_JPRB, 0.80354E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.78281E+00_JPRB, 0.80565E+00_JPRB, 0.82863E+00_JPRB, 0.90565E+00_JPRB, 0.10298E+01_JPRB, 0.10510E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.13047E+01_JPRB, 0.15716E+01_JPRB, 0.17727E+01_JPRB, 0.20143E+01_JPRB, 0.22010E+01_JPRB, 0.21556E+01_JPRB ,&
&  0.20180E+01_JPRB, 0.20292E+01_JPRB, 0.15964E+01_JPRB, 0.16291E+01_JPRB, 0.14523E+01_JPRB, 0.11816E+01_JPRB, 0.14302E+01_JPRB ,&
&  0.15417E+01_JPRB, 0.15008E+01_JPRB, 0.12756E+01_JPRB, 0.10207E+01_JPRB, 0.10778E+01_JPRB, 0.81410E+00_JPRB, 0.58696E+00_JPRB ,&
&  0.55796E+00_JPRB, 0.54027E+00_JPRB, 0.52180E+00_JPRB, 0.55611E+00_JPRB, 0.56717E+00_JPRB, 0.70104E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.13442E+01_JPRB, 0.15862E+01_JPRB, 0.16960E+01_JPRB, 0.17623E+01_JPRB, 0.17195E+01_JPRB, 0.16480E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.16521E+01_JPRB, 0.16070E+01_JPRB, 0.14480E+01_JPRB, 0.13074E+01_JPRB, 0.12056E+01_JPRB, 0.11260E+01_JPRB, 0.10614E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.98535E+00_JPRB, 0.96078E+00_JPRB, 0.94217E+00_JPRB, 0.92385E+00_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.91541E+00_JPRB, 0.90673E+00_JPRB, 0.89986E+00_JPRB, 0.88958E+00_JPRB, 0.87193E+00_JPRB, 0.84671E+00_JPRB, 0.81407E+00_JPRB ,&
&  0.76556E+00_JPRB, 0.72247E+00_JPRB, 0.67922E+00_JPRB, 0.65859E+00_JPRB, 0.63760E+00_JPRB, 0.61906E+00_JPRB, 0.62021E+00_JPRB ,&
&  0.66339E+00_JPRB, 0.80553E+00_JPRB, 0.70107E+00_JPRB, 0.67921E+00_JPRB, 0.72605E+00_JPRB, 0.75998E+00_JPRB, 0.67126E+00_JPRB ,&
&  0.91575E+00_JPRB, 0.11395E+01_JPRB, 0.14206E+01_JPRB, 0.19657E+01_JPRB, 0.27740E+01_JPRB, 0.26671E+01_JPRB, 0.27313E+01_JPRB ,&
&  0.27104E+01_JPRB, 0.23270E+01_JPRB, 0.20475E+01_JPRB, 0.17249E+01_JPRB, 0.14031E+01_JPRB, 0.11811E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.94190E+00_JPRB, 0.92060E+00_JPRB, 0.89855E+00_JPRB, 0.86779E+00_JPRB, 0.84387E+00_JPRB, 0.81864E+00_JPRB, 0.78387E+00_JPRB ,&
&  0.76873E+00_JPRB, 0.76959E+00_JPRB, 0.77846E+00_JPRB, 0.87151E+00_JPRB, 0.92274E+00_JPRB, 0.94649E+00_JPRB, 0.12720E+01_JPRB ,&
&  0.14334E+01_JPRB, 0.16233E+01_JPRB, 0.16554E+01_JPRB, 0.19899E+01_JPRB, 0.23367E+01_JPRB, 0.23657E+01_JPRB, 0.24880E+01_JPRB ,&
&  0.19110E+01_JPRB, 0.15739E+01_JPRB, 0.15418E+01_JPRB, 0.15247E+01_JPRB, 0.12599E+01_JPRB, 0.13975E+01_JPRB, 0.22052E+01_JPRB ,&
&  0.16130E+01_JPRB, 0.17469E+01_JPRB, 0.19480E+01_JPRB, 0.20670E+01_JPRB, 0.10736E+01_JPRB, 0.76329E+00_JPRB, 0.12266E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.81270E+00_JPRB, 0.64893E+00_JPRB, 0.82383E+00_JPRB, 0.66593E+00_JPRB, 0.76644E+00_JPRB, 0.98695E+00_JPRB ,&
&  0.11284E+01_JPRB, 0.12468E+01_JPRB, 0.16499E+01_JPRB, 0.18767E+01_JPRB, 0.19058E+01_JPRB, 0.18936E+01_JPRB, 0.19136E+01_JPRB ,&
&  0.18322E+01_JPRB, 0.16621E+01_JPRB, 0.16196E+01_JPRB, 0.14573E+01_JPRB, 0.13303E+01_JPRB, 0.12111E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.99278E+00_JPRB, 0.95855E+00_JPRB, 0.93318E+00_JPRB, 0.92477E+00_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.93041E+00_JPRB, 0.90659E+00_JPRB, 0.89873E+00_JPRB, 0.89892E+00_JPRB, 0.89355E+00_JPRB, 0.87159E+00_JPRB, 0.84109E+00_JPRB ,&
&  0.80824E+00_JPRB, 0.76059E+00_JPRB, 0.71472E+00_JPRB, 0.68230E+00_JPRB, 0.66512E+00_JPRB, 0.64432E+00_JPRB, 0.63208E+00_JPRB ,&
&  0.65320E+00_JPRB, 0.67249E+00_JPRB, 0.61901E+00_JPRB, 0.61099E+00_JPRB, 0.73849E+00_JPRB, 0.69829E+00_JPRB, 0.69413E+00_JPRB ,&
&  0.83552E+00_JPRB, 0.10269E+01_JPRB, 0.13515E+01_JPRB, 0.18008E+01_JPRB, 0.21962E+01_JPRB, 0.25813E+01_JPRB, 0.27750E+01_JPRB ,&
&  0.27884E+01_JPRB, 0.24433E+01_JPRB, 0.20985E+01_JPRB, 0.18000E+01_JPRB, 0.15063E+01_JPRB, 0.12814E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.97157E+00_JPRB, 0.93935E+00_JPRB, 0.90058E+00_JPRB, 0.88641E+00_JPRB, 0.84972E+00_JPRB, 0.81570E+00_JPRB ,&
&  0.80366E+00_JPRB, 0.80497E+00_JPRB, 0.79119E+00_JPRB, 0.83041E+00_JPRB, 0.85763E+00_JPRB, 0.93080E+00_JPRB, 0.10685E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.15103E+01_JPRB, 0.16643E+01_JPRB, 0.20217E+01_JPRB, 0.25072E+01_JPRB, 0.24200E+01_JPRB, 0.27164E+01_JPRB ,&
&  0.23300E+01_JPRB, 0.19338E+01_JPRB, 0.15948E+01_JPRB, 0.12745E+01_JPRB, 0.14928E+01_JPRB, 0.23743E+01_JPRB, 0.24063E+01_JPRB ,&
&  0.21493E+01_JPRB, 0.20958E+01_JPRB, 0.21155E+01_JPRB, 0.19105E+01_JPRB, 0.16152E+01_JPRB, 0.12421E+01_JPRB, 0.16225E+01_JPRB ,&
&  0.16747E+01_JPRB, 0.16030E+01_JPRB, 0.14544E+01_JPRB, 0.12139E+01_JPRB, 0.10035E+01_JPRB, 0.11567E+01_JPRB, 0.11245E+01_JPRB ,&
&  0.10856E+01_JPRB, 0.10875E+01_JPRB, 0.13624E+01_JPRB, 0.17726E+01_JPRB, 0.18547E+01_JPRB, 0.18005E+01_JPRB, 0.18340E+01_JPRB ,&
&  0.17220E+01_JPRB, 0.16891E+01_JPRB, 0.16152E+01_JPRB, 0.15123E+01_JPRB, 0.14213E+01_JPRB, 0.13215E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11181E+01_JPRB, 0.10611E+01_JPRB, 0.10091E+01_JPRB, 0.96335E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.92906E+00_JPRB, 0.89497E+00_JPRB, 0.87812E+00_JPRB, 0.87772E+00_JPRB, 0.89120E+00_JPRB, 0.88343E+00_JPRB, 0.86139E+00_JPRB ,&
&  0.83196E+00_JPRB, 0.80094E+00_JPRB, 0.76267E+00_JPRB, 0.72318E+00_JPRB, 0.69661E+00_JPRB, 0.66011E+00_JPRB, 0.64079E+00_JPRB ,&
&  0.65396E+00_JPRB, 0.63186E+00_JPRB, 0.57052E+00_JPRB, 0.62242E+00_JPRB, 0.65939E+00_JPRB, 0.62424E+00_JPRB, 0.69178E+00_JPRB ,&
&  0.77239E+00_JPRB, 0.92604E+00_JPRB, 0.11560E+01_JPRB, 0.14233E+01_JPRB, 0.16873E+01_JPRB, 0.20919E+01_JPRB, 0.24305E+01_JPRB ,&
&  0.24075E+01_JPRB, 0.23350E+01_JPRB, 0.21482E+01_JPRB, 0.19380E+01_JPRB, 0.16552E+01_JPRB, 0.14308E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.10593E+01_JPRB, 0.10144E+01_JPRB, 0.97673E+00_JPRB, 0.94940E+00_JPRB, 0.92481E+00_JPRB, 0.91055E+00_JPRB ,&
&  0.87290E+00_JPRB, 0.86377E+00_JPRB, 0.84093E+00_JPRB, 0.85827E+00_JPRB, 0.87512E+00_JPRB, 0.94820E+00_JPRB, 0.10456E+01_JPRB ,&
&  0.13662E+01_JPRB, 0.16874E+01_JPRB, 0.18360E+01_JPRB, 0.23814E+01_JPRB, 0.21489E+01_JPRB, 0.22117E+01_JPRB, 0.27048E+01_JPRB ,&
&  0.30464E+01_JPRB, 0.30979E+01_JPRB, 0.30059E+01_JPRB, 0.23618E+01_JPRB, 0.20315E+01_JPRB, 0.27961E+01_JPRB, 0.26590E+01_JPRB ,&
&  0.23613E+01_JPRB, 0.22416E+01_JPRB, 0.20756E+01_JPRB, 0.22484E+01_JPRB, 0.18645E+01_JPRB, 0.16714E+01_JPRB, 0.13263E+01_JPRB ,&
&  0.13581E+01_JPRB, 0.11798E+01_JPRB, 0.13763E+01_JPRB, 0.12354E+01_JPRB, 0.11943E+01_JPRB, 0.11568E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.10631E+01_JPRB, 0.11015E+01_JPRB, 0.13412E+01_JPRB, 0.16046E+01_JPRB, 0.15196E+01_JPRB, 0.15747E+01_JPRB ,&
&  0.15896E+01_JPRB, 0.15754E+01_JPRB, 0.14988E+01_JPRB, 0.15043E+01_JPRB, 0.14588E+01_JPRB, 0.13979E+01_JPRB, 0.13249E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.11846E+01_JPRB, 0.11169E+01_JPRB, 0.10444E+01_JPRB, 0.97866E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.90788E+00_JPRB, 0.87591E+00_JPRB, 0.85537E+00_JPRB, 0.85721E+00_JPRB, 0.87577E+00_JPRB, 0.88248E+00_JPRB, 0.86316E+00_JPRB ,&
&  0.84415E+00_JPRB, 0.82121E+00_JPRB, 0.80418E+00_JPRB, 0.76349E+00_JPRB, 0.72486E+00_JPRB, 0.67827E+00_JPRB, 0.65055E+00_JPRB ,&
&  0.66878E+00_JPRB, 0.71017E+00_JPRB, 0.61487E+00_JPRB, 0.57111E+00_JPRB, 0.53552E+00_JPRB, 0.58728E+00_JPRB, 0.67719E+00_JPRB ,&
&  0.77680E+00_JPRB, 0.85063E+00_JPRB, 0.10177E+01_JPRB, 0.10879E+01_JPRB, 0.11851E+01_JPRB, 0.14774E+01_JPRB, 0.18848E+01_JPRB ,&
&  0.23716E+01_JPRB, 0.19935E+01_JPRB, 0.19900E+01_JPRB, 0.18489E+01_JPRB, 0.17101E+01_JPRB, 0.15406E+01_JPRB, 0.13785E+01_JPRB ,&
&  0.12322E+01_JPRB, 0.11106E+01_JPRB, 0.10458E+01_JPRB, 0.99428E+00_JPRB, 0.94967E+00_JPRB, 0.91295E+00_JPRB, 0.88511E+00_JPRB ,&
&  0.87012E+00_JPRB, 0.88859E+00_JPRB, 0.91703E+00_JPRB, 0.92225E+00_JPRB, 0.98798E+00_JPRB, 0.11025E+01_JPRB, 0.12012E+01_JPRB ,&
&  0.12737E+01_JPRB, 0.14456E+01_JPRB, 0.21394E+01_JPRB, 0.24247E+01_JPRB, 0.26934E+01_JPRB, 0.28621E+01_JPRB, 0.30637E+01_JPRB ,&
&  0.33220E+01_JPRB, 0.33873E+01_JPRB, 0.33240E+01_JPRB, 0.28504E+01_JPRB, 0.31887E+01_JPRB, 0.31466E+01_JPRB, 0.29453E+01_JPRB ,&
&  0.26850E+01_JPRB, 0.26189E+01_JPRB, 0.23595E+01_JPRB, 0.22640E+01_JPRB, 0.22939E+01_JPRB, 0.24376E+01_JPRB, 0.21348E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.19367E+01_JPRB, 0.16836E+01_JPRB, 0.13433E+01_JPRB, 0.10769E+01_JPRB, 0.10694E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.10853E+01_JPRB, 0.10973E+01_JPRB, 0.10840E+01_JPRB, 0.11609E+01_JPRB, 0.13301E+01_JPRB, 0.14113E+01_JPRB, 0.13893E+01_JPRB ,&
&  0.13405E+01_JPRB, 0.14147E+01_JPRB, 0.14160E+01_JPRB, 0.13803E+01_JPRB, 0.13685E+01_JPRB, 0.13323E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.11733E+01_JPRB, 0.10947E+01_JPRB, 0.10390E+01_JPRB, 0.99032E+00_JPRB, 0.94581E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.84762E+00_JPRB, 0.83236E+00_JPRB, 0.82258E+00_JPRB, 0.83089E+00_JPRB, 0.85736E+00_JPRB, 0.87250E+00_JPRB, 0.85845E+00_JPRB ,&
&  0.84546E+00_JPRB, 0.82753E+00_JPRB, 0.80915E+00_JPRB, 0.78323E+00_JPRB, 0.74104E+00_JPRB, 0.69597E+00_JPRB, 0.67414E+00_JPRB ,&
&  0.70294E+00_JPRB, 0.83948E+00_JPRB, 0.71157E+00_JPRB, 0.67778E+00_JPRB, 0.62172E+00_JPRB, 0.67354E+00_JPRB, 0.73150E+00_JPRB ,&
&  0.79944E+00_JPRB, 0.83093E+00_JPRB, 0.87403E+00_JPRB, 0.89470E+00_JPRB, 0.93661E+00_JPRB, 0.11333E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.16616E+01_JPRB, 0.18050E+01_JPRB, 0.16792E+01_JPRB, 0.17660E+01_JPRB, 0.16262E+01_JPRB, 0.14299E+01_JPRB, 0.14280E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.11857E+01_JPRB, 0.10742E+01_JPRB, 0.10075E+01_JPRB, 0.96310E+00_JPRB, 0.91875E+00_JPRB, 0.88120E+00_JPRB ,&
&  0.86481E+00_JPRB, 0.88279E+00_JPRB, 0.93314E+00_JPRB, 0.96785E+00_JPRB, 0.99460E+00_JPRB, 0.11321E+01_JPRB, 0.12991E+01_JPRB ,&
&  0.17021E+01_JPRB, 0.17496E+01_JPRB, 0.17015E+01_JPRB, 0.23623E+01_JPRB, 0.33437E+01_JPRB, 0.34393E+01_JPRB, 0.32498E+01_JPRB ,&
&  0.34670E+01_JPRB, 0.35538E+01_JPRB, 0.35702E+01_JPRB, 0.35375E+01_JPRB, 0.35434E+01_JPRB, 0.34175E+01_JPRB, 0.32383E+01_JPRB ,&
&  0.30249E+01_JPRB, 0.29031E+01_JPRB, 0.27496E+01_JPRB, 0.25878E+01_JPRB, 0.23721E+01_JPRB, 0.22940E+01_JPRB, 0.23407E+01_JPRB ,&
&  0.21444E+01_JPRB, 0.18397E+01_JPRB, 0.12270E+01_JPRB, 0.12475E+01_JPRB, 0.95775E+00_JPRB, 0.92609E+00_JPRB, 0.10503E+01_JPRB ,&
&  0.10362E+01_JPRB, 0.11047E+01_JPRB, 0.11132E+01_JPRB, 0.10455E+01_JPRB, 0.11246E+01_JPRB, 0.12335E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12872E+01_JPRB, 0.11856E+01_JPRB, 0.12691E+01_JPRB, 0.12328E+01_JPRB, 0.11830E+01_JPRB, 0.11446E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.10206E+01_JPRB, 0.94070E+00_JPRB, 0.88492E+00_JPRB, 0.86364E+00_JPRB, 0.85427E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.74869E+00_JPRB, 0.75435E+00_JPRB, 0.76298E+00_JPRB, 0.77914E+00_JPRB, 0.81147E+00_JPRB, 0.83567E+00_JPRB, 0.83098E+00_JPRB ,&
&  0.81993E+00_JPRB, 0.80952E+00_JPRB, 0.79441E+00_JPRB, 0.77709E+00_JPRB, 0.74780E+00_JPRB, 0.70487E+00_JPRB, 0.69025E+00_JPRB ,&
&  0.78990E+00_JPRB, 0.84853E+00_JPRB, 0.76148E+00_JPRB, 0.65518E+00_JPRB, 0.75117E+00_JPRB, 0.72284E+00_JPRB, 0.75063E+00_JPRB ,&
&  0.72450E+00_JPRB, 0.79569E+00_JPRB, 0.82755E+00_JPRB, 0.79470E+00_JPRB, 0.78087E+00_JPRB, 0.91139E+00_JPRB, 0.10805E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.13356E+01_JPRB, 0.13647E+01_JPRB, 0.13769E+01_JPRB, 0.14279E+01_JPRB, 0.12908E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11167E+01_JPRB, 0.10371E+01_JPRB, 0.97940E+00_JPRB, 0.92907E+00_JPRB, 0.88700E+00_JPRB, 0.86287E+00_JPRB ,&
&  0.87858E+00_JPRB, 0.90093E+00_JPRB, 0.93388E+00_JPRB, 0.98036E+00_JPRB, 0.10237E+01_JPRB, 0.12593E+01_JPRB, 0.17377E+01_JPRB ,&
&  0.22503E+01_JPRB, 0.25351E+01_JPRB, 0.29398E+01_JPRB, 0.34824E+01_JPRB, 0.36204E+01_JPRB, 0.36988E+01_JPRB, 0.38222E+01_JPRB ,&
&  0.38529E+01_JPRB, 0.38119E+01_JPRB, 0.37254E+01_JPRB, 0.36119E+01_JPRB, 0.35931E+01_JPRB, 0.35445E+01_JPRB, 0.34137E+01_JPRB ,&
&  0.31090E+01_JPRB, 0.29518E+01_JPRB, 0.29452E+01_JPRB, 0.27238E+01_JPRB, 0.25522E+01_JPRB, 0.23183E+01_JPRB, 0.23791E+01_JPRB ,&
&  0.23496E+01_JPRB, 0.13778E+01_JPRB, 0.10619E+01_JPRB, 0.12490E+01_JPRB, 0.10132E+01_JPRB, 0.99350E+00_JPRB, 0.11515E+01_JPRB ,&
&  0.11377E+01_JPRB, 0.10888E+01_JPRB, 0.10794E+01_JPRB, 0.10003E+01_JPRB, 0.97653E+00_JPRB, 0.11222E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.10082E+01_JPRB, 0.99961E+00_JPRB, 0.10404E+01_JPRB, 0.99908E+00_JPRB, 0.95476E+00_JPRB, 0.93589E+00_JPRB, 0.89967E+00_JPRB ,&
&  0.83450E+00_JPRB, 0.78480E+00_JPRB, 0.74685E+00_JPRB, 0.73441E+00_JPRB, 0.74117E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.68879E+00_JPRB, 0.70903E+00_JPRB, 0.72201E+00_JPRB, 0.73952E+00_JPRB, 0.76851E+00_JPRB, 0.80118E+00_JPRB, 0.80740E+00_JPRB ,&
&  0.79965E+00_JPRB, 0.79082E+00_JPRB, 0.78007E+00_JPRB, 0.76306E+00_JPRB, 0.74430E+00_JPRB, 0.71263E+00_JPRB, 0.70820E+00_JPRB ,&
&  0.61660E+00_JPRB, 0.69095E+00_JPRB, 0.60743E+00_JPRB, 0.62974E+00_JPRB, 0.78234E+00_JPRB, 0.72128E+00_JPRB, 0.69813E+00_JPRB ,&
&  0.67142E+00_JPRB, 0.70039E+00_JPRB, 0.73868E+00_JPRB, 0.75095E+00_JPRB, 0.71194E+00_JPRB, 0.76537E+00_JPRB, 0.88192E+00_JPRB ,&
&  0.93086E+00_JPRB, 0.93594E+00_JPRB, 0.97103E+00_JPRB, 0.97981E+00_JPRB, 0.10115E+01_JPRB, 0.10984E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10143E+01_JPRB, 0.95530E+00_JPRB, 0.91138E+00_JPRB, 0.87370E+00_JPRB, 0.83285E+00_JPRB, 0.80659E+00_JPRB ,&
&  0.79845E+00_JPRB, 0.80257E+00_JPRB, 0.85157E+00_JPRB, 0.93847E+00_JPRB, 0.10182E+01_JPRB, 0.14047E+01_JPRB, 0.22022E+01_JPRB ,&
&  0.25977E+01_JPRB, 0.27583E+01_JPRB, 0.31427E+01_JPRB, 0.36925E+01_JPRB, 0.37736E+01_JPRB, 0.38095E+01_JPRB, 0.37992E+01_JPRB ,&
&  0.38340E+01_JPRB, 0.38190E+01_JPRB, 0.37219E+01_JPRB, 0.37209E+01_JPRB, 0.35644E+01_JPRB, 0.35686E+01_JPRB, 0.33617E+01_JPRB ,&
&  0.30053E+01_JPRB, 0.29093E+01_JPRB, 0.29706E+01_JPRB, 0.29208E+01_JPRB, 0.28245E+01_JPRB, 0.28327E+01_JPRB, 0.28061E+01_JPRB ,&
&  0.26536E+01_JPRB, 0.22726E+01_JPRB, 0.16650E+01_JPRB, 0.14814E+01_JPRB, 0.10668E+01_JPRB, 0.10716E+01_JPRB, 0.13156E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.11328E+01_JPRB, 0.99276E+00_JPRB, 0.91946E+00_JPRB, 0.89662E+00_JPRB, 0.97821E+00_JPRB, 0.91603E+00_JPRB ,&
&  0.81683E+00_JPRB, 0.83205E+00_JPRB, 0.84094E+00_JPRB, 0.84476E+00_JPRB, 0.84303E+00_JPRB, 0.81685E+00_JPRB, 0.75492E+00_JPRB ,&
&  0.73389E+00_JPRB, 0.69586E+00_JPRB, 0.66056E+00_JPRB, 0.65087E+00_JPRB, 0.66405E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.65848E+00_JPRB, 0.68216E+00_JPRB, 0.69824E+00_JPRB, 0.70997E+00_JPRB, 0.73021E+00_JPRB, 0.76249E+00_JPRB, 0.77177E+00_JPRB ,&
&  0.76700E+00_JPRB, 0.76063E+00_JPRB, 0.75451E+00_JPRB, 0.74236E+00_JPRB, 0.71909E+00_JPRB, 0.69153E+00_JPRB, 0.63019E+00_JPRB ,&
&  0.55046E+00_JPRB, 0.62005E+00_JPRB, 0.60013E+00_JPRB, 0.62277E+00_JPRB, 0.63098E+00_JPRB, 0.61562E+00_JPRB, 0.59839E+00_JPRB ,&
&  0.61894E+00_JPRB, 0.63884E+00_JPRB, 0.68742E+00_JPRB, 0.73898E+00_JPRB, 0.72657E+00_JPRB, 0.74201E+00_JPRB, 0.81878E+00_JPRB ,&
&  0.82084E+00_JPRB, 0.81457E+00_JPRB, 0.78056E+00_JPRB, 0.77161E+00_JPRB, 0.80690E+00_JPRB, 0.86985E+00_JPRB, 0.92363E+00_JPRB ,&
&  0.92326E+00_JPRB, 0.88133E+00_JPRB, 0.84007E+00_JPRB, 0.81192E+00_JPRB, 0.78978E+00_JPRB, 0.76495E+00_JPRB, 0.74104E+00_JPRB ,&
&  0.71639E+00_JPRB, 0.70805E+00_JPRB, 0.73278E+00_JPRB, 0.78322E+00_JPRB, 0.87718E+00_JPRB, 0.12057E+01_JPRB, 0.15161E+01_JPRB ,&
&  0.16585E+01_JPRB, 0.18660E+01_JPRB, 0.21609E+01_JPRB, 0.24867E+01_JPRB, 0.27360E+01_JPRB, 0.29571E+01_JPRB, 0.31628E+01_JPRB ,&
&  0.34194E+01_JPRB, 0.34242E+01_JPRB, 0.37942E+01_JPRB, 0.37545E+01_JPRB, 0.35814E+01_JPRB, 0.34983E+01_JPRB, 0.32519E+01_JPRB ,&
&  0.30375E+01_JPRB, 0.28278E+01_JPRB, 0.29491E+01_JPRB, 0.29473E+01_JPRB, 0.28997E+01_JPRB, 0.28744E+01_JPRB, 0.27739E+01_JPRB ,&
&  0.26886E+01_JPRB, 0.25796E+01_JPRB, 0.22010E+01_JPRB, 0.18685E+01_JPRB, 0.14884E+01_JPRB, 0.14810E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.11063E+01_JPRB, 0.87743E+00_JPRB, 0.85676E+00_JPRB, 0.83189E+00_JPRB, 0.84018E+00_JPRB, 0.79868E+00_JPRB, 0.74791E+00_JPRB ,&
&  0.74276E+00_JPRB, 0.75490E+00_JPRB, 0.72975E+00_JPRB, 0.72534E+00_JPRB, 0.73946E+00_JPRB, 0.71728E+00_JPRB, 0.64447E+00_JPRB ,&
&  0.62188E+00_JPRB, 0.62078E+00_JPRB, 0.60098E+00_JPRB, 0.60350E+00_JPRB, 0.62579E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.63282E+00_JPRB, 0.65177E+00_JPRB, 0.66687E+00_JPRB, 0.68109E+00_JPRB, 0.69861E+00_JPRB, 0.71707E+00_JPRB, 0.70921E+00_JPRB ,&
&  0.71738E+00_JPRB, 0.71780E+00_JPRB, 0.71343E+00_JPRB, 0.69894E+00_JPRB, 0.67158E+00_JPRB, 0.59681E+00_JPRB, 0.50147E+00_JPRB ,&
&  0.48263E+00_JPRB, 0.51038E+00_JPRB, 0.54442E+00_JPRB, 0.55981E+00_JPRB, 0.56670E+00_JPRB, 0.57375E+00_JPRB, 0.57266E+00_JPRB ,&
&  0.58032E+00_JPRB, 0.59606E+00_JPRB, 0.63306E+00_JPRB, 0.69176E+00_JPRB, 0.74286E+00_JPRB, 0.75133E+00_JPRB, 0.76882E+00_JPRB ,&
&  0.74556E+00_JPRB, 0.74289E+00_JPRB, 0.72246E+00_JPRB, 0.66631E+00_JPRB, 0.68116E+00_JPRB, 0.68825E+00_JPRB, 0.72160E+00_JPRB ,&
&  0.75202E+00_JPRB, 0.74849E+00_JPRB, 0.72853E+00_JPRB, 0.71592E+00_JPRB, 0.70505E+00_JPRB, 0.68925E+00_JPRB, 0.67327E+00_JPRB ,&
&  0.65607E+00_JPRB, 0.64690E+00_JPRB, 0.66021E+00_JPRB, 0.68565E+00_JPRB, 0.72971E+00_JPRB, 0.95296E+00_JPRB, 0.10834E+01_JPRB ,&
&  0.12042E+01_JPRB, 0.14195E+01_JPRB, 0.17629E+01_JPRB, 0.19209E+01_JPRB, 0.22806E+01_JPRB, 0.25396E+01_JPRB, 0.28508E+01_JPRB ,&
&  0.31331E+01_JPRB, 0.30893E+01_JPRB, 0.33324E+01_JPRB, 0.33773E+01_JPRB, 0.33601E+01_JPRB, 0.32349E+01_JPRB, 0.30306E+01_JPRB ,&
&  0.29077E+01_JPRB, 0.29423E+01_JPRB, 0.29969E+01_JPRB, 0.29197E+01_JPRB, 0.27335E+01_JPRB, 0.26071E+01_JPRB, 0.24745E+01_JPRB ,&
&  0.24054E+01_JPRB, 0.23415E+01_JPRB, 0.21972E+01_JPRB, 0.19963E+01_JPRB, 0.17470E+01_JPRB, 0.15027E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.92366E+00_JPRB, 0.84136E+00_JPRB, 0.81952E+00_JPRB, 0.80553E+00_JPRB, 0.73736E+00_JPRB, 0.70932E+00_JPRB ,&
&  0.69734E+00_JPRB, 0.67185E+00_JPRB, 0.68396E+00_JPRB, 0.66871E+00_JPRB, 0.67759E+00_JPRB, 0.67294E+00_JPRB, 0.62863E+00_JPRB ,&
&  0.59751E+00_JPRB, 0.60372E+00_JPRB, 0.59814E+00_JPRB, 0.59742E+00_JPRB, 0.61230E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.61875E+00_JPRB, 0.63701E+00_JPRB, 0.64385E+00_JPRB, 0.65203E+00_JPRB, 0.66270E+00_JPRB, 0.65389E+00_JPRB, 0.64490E+00_JPRB ,&
&  0.66069E+00_JPRB, 0.62465E+00_JPRB, 0.66023E+00_JPRB, 0.54003E+00_JPRB, 0.49283E+00_JPRB, 0.48462E+00_JPRB, 0.47503E+00_JPRB ,&
&  0.48092E+00_JPRB, 0.49887E+00_JPRB, 0.51587E+00_JPRB, 0.52780E+00_JPRB, 0.54689E+00_JPRB, 0.55071E+00_JPRB, 0.55294E+00_JPRB ,&
&  0.55946E+00_JPRB, 0.57204E+00_JPRB, 0.60306E+00_JPRB, 0.62117E+00_JPRB, 0.64341E+00_JPRB, 0.66212E+00_JPRB, 0.66572E+00_JPRB ,&
&  0.64488E+00_JPRB, 0.64631E+00_JPRB, 0.66135E+00_JPRB, 0.63966E+00_JPRB, 0.61855E+00_JPRB, 0.60635E+00_JPRB, 0.61894E+00_JPRB ,&
&  0.62040E+00_JPRB, 0.58142E+00_JPRB, 0.59929E+00_JPRB, 0.60366E+00_JPRB, 0.60422E+00_JPRB, 0.60282E+00_JPRB, 0.60148E+00_JPRB ,&
&  0.60169E+00_JPRB, 0.60335E+00_JPRB, 0.61918E+00_JPRB, 0.64016E+00_JPRB, 0.68541E+00_JPRB, 0.77290E+00_JPRB, 0.83297E+00_JPRB ,&
&  0.97269E+00_JPRB, 0.94297E+00_JPRB, 0.12755E+01_JPRB, 0.16278E+01_JPRB, 0.20979E+01_JPRB, 0.23524E+01_JPRB, 0.26364E+01_JPRB ,&
&  0.30047E+01_JPRB, 0.29607E+01_JPRB, 0.30673E+01_JPRB, 0.30213E+01_JPRB, 0.29472E+01_JPRB, 0.28564E+01_JPRB, 0.27674E+01_JPRB ,&
&  0.27081E+01_JPRB, 0.25737E+01_JPRB, 0.26744E+01_JPRB, 0.25638E+01_JPRB, 0.23847E+01_JPRB, 0.22569E+01_JPRB, 0.21383E+01_JPRB ,&
&  0.20256E+01_JPRB, 0.19086E+01_JPRB, 0.17672E+01_JPRB, 0.14776E+01_JPRB, 0.13445E+01_JPRB, 0.12397E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10136E+01_JPRB, 0.93805E+00_JPRB, 0.86610E+00_JPRB, 0.85971E+00_JPRB, 0.80498E+00_JPRB, 0.75243E+00_JPRB, 0.73153E+00_JPRB ,&
&  0.68519E+00_JPRB, 0.63561E+00_JPRB, 0.63673E+00_JPRB, 0.63137E+00_JPRB, 0.62371E+00_JPRB, 0.61574E+00_JPRB, 0.64207E+00_JPRB ,&
&  0.61479E+00_JPRB, 0.59662E+00_JPRB, 0.60764E+00_JPRB, 0.59943E+00_JPRB, 0.60470E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.58953E+00_JPRB, 0.61352E+00_JPRB, 0.62285E+00_JPRB, 0.62529E+00_JPRB, 0.62860E+00_JPRB, 0.62074E+00_JPRB, 0.58410E+00_JPRB ,&
&  0.57361E+00_JPRB, 0.52179E+00_JPRB, 0.49581E+00_JPRB, 0.47857E+00_JPRB, 0.47220E+00_JPRB, 0.46593E+00_JPRB, 0.45687E+00_JPRB ,&
&  0.46116E+00_JPRB, 0.47621E+00_JPRB, 0.49641E+00_JPRB, 0.51055E+00_JPRB, 0.52256E+00_JPRB, 0.53112E+00_JPRB, 0.53904E+00_JPRB ,&
&  0.54978E+00_JPRB, 0.56218E+00_JPRB, 0.57728E+00_JPRB, 0.58823E+00_JPRB, 0.59237E+00_JPRB, 0.59430E+00_JPRB, 0.59071E+00_JPRB ,&
&  0.59060E+00_JPRB, 0.58937E+00_JPRB, 0.60163E+00_JPRB, 0.60582E+00_JPRB, 0.59729E+00_JPRB, 0.58244E+00_JPRB, 0.55359E+00_JPRB ,&
&  0.49500E+00_JPRB, 0.49687E+00_JPRB, 0.51866E+00_JPRB, 0.53162E+00_JPRB, 0.54286E+00_JPRB, 0.55310E+00_JPRB, 0.56074E+00_JPRB ,&
&  0.56898E+00_JPRB, 0.58015E+00_JPRB, 0.60174E+00_JPRB, 0.62626E+00_JPRB, 0.66734E+00_JPRB, 0.73119E+00_JPRB, 0.78354E+00_JPRB ,&
&  0.86176E+00_JPRB, 0.85342E+00_JPRB, 0.99677E+00_JPRB, 0.13297E+01_JPRB, 0.19967E+01_JPRB, 0.21938E+01_JPRB, 0.23422E+01_JPRB ,&
&  0.24982E+01_JPRB, 0.25502E+01_JPRB, 0.26361E+01_JPRB, 0.26924E+01_JPRB, 0.26562E+01_JPRB, 0.25929E+01_JPRB, 0.25502E+01_JPRB ,&
&  0.24051E+01_JPRB, 0.23737E+01_JPRB, 0.22788E+01_JPRB, 0.22231E+01_JPRB, 0.21138E+01_JPRB, 0.20260E+01_JPRB, 0.19107E+01_JPRB ,&
&  0.17673E+01_JPRB, 0.16183E+01_JPRB, 0.14370E+01_JPRB, 0.12626E+01_JPRB, 0.11242E+01_JPRB, 0.10514E+01_JPRB, 0.99153E+00_JPRB ,&
&  0.94771E+00_JPRB, 0.92126E+00_JPRB, 0.88701E+00_JPRB, 0.85604E+00_JPRB, 0.80505E+00_JPRB, 0.74299E+00_JPRB, 0.71997E+00_JPRB ,&
&  0.65366E+00_JPRB, 0.62057E+00_JPRB, 0.58777E+00_JPRB, 0.57444E+00_JPRB, 0.57670E+00_JPRB, 0.57434E+00_JPRB, 0.56687E+00_JPRB ,&
&  0.57950E+00_JPRB, 0.56710E+00_JPRB, 0.55378E+00_JPRB, 0.55897E+00_JPRB, 0.56664E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.54918E+00_JPRB, 0.56713E+00_JPRB, 0.58402E+00_JPRB, 0.59255E+00_JPRB, 0.59028E+00_JPRB, 0.58391E+00_JPRB, 0.56362E+00_JPRB ,&
&  0.55726E+00_JPRB, 0.53133E+00_JPRB, 0.51195E+00_JPRB, 0.49232E+00_JPRB, 0.47844E+00_JPRB, 0.46985E+00_JPRB, 0.47196E+00_JPRB ,&
&  0.47943E+00_JPRB, 0.48843E+00_JPRB, 0.49769E+00_JPRB, 0.50479E+00_JPRB, 0.51383E+00_JPRB, 0.52524E+00_JPRB, 0.53971E+00_JPRB ,&
&  0.55409E+00_JPRB, 0.56340E+00_JPRB, 0.57151E+00_JPRB, 0.57715E+00_JPRB, 0.58170E+00_JPRB, 0.58346E+00_JPRB, 0.57422E+00_JPRB ,&
&  0.57417E+00_JPRB, 0.57273E+00_JPRB, 0.58163E+00_JPRB, 0.58544E+00_JPRB, 0.58434E+00_JPRB, 0.56109E+00_JPRB, 0.50277E+00_JPRB ,&
&  0.46484E+00_JPRB, 0.44650E+00_JPRB, 0.45041E+00_JPRB, 0.47475E+00_JPRB, 0.49045E+00_JPRB, 0.50576E+00_JPRB, 0.51961E+00_JPRB ,&
&  0.52155E+00_JPRB, 0.52593E+00_JPRB, 0.54959E+00_JPRB, 0.58553E+00_JPRB, 0.61947E+00_JPRB, 0.68342E+00_JPRB, 0.75028E+00_JPRB ,&
&  0.84608E+00_JPRB, 0.98796E+00_JPRB, 0.12343E+01_JPRB, 0.12926E+01_JPRB, 0.14417E+01_JPRB, 0.18796E+01_JPRB, 0.19320E+01_JPRB ,&
&  0.20098E+01_JPRB, 0.21065E+01_JPRB, 0.22628E+01_JPRB, 0.24113E+01_JPRB, 0.23731E+01_JPRB, 0.23092E+01_JPRB, 0.22184E+01_JPRB ,&
&  0.20389E+01_JPRB, 0.19427E+01_JPRB, 0.18559E+01_JPRB, 0.18636E+01_JPRB, 0.18076E+01_JPRB, 0.17411E+01_JPRB, 0.16768E+01_JPRB ,&
&  0.15563E+01_JPRB, 0.13907E+01_JPRB, 0.12291E+01_JPRB, 0.10832E+01_JPRB, 0.10092E+01_JPRB, 0.96482E+00_JPRB, 0.93572E+00_JPRB ,&
&  0.90279E+00_JPRB, 0.87718E+00_JPRB, 0.85003E+00_JPRB, 0.82080E+00_JPRB, 0.74753E+00_JPRB, 0.66108E+00_JPRB, 0.61556E+00_JPRB ,&
&  0.58862E+00_JPRB, 0.57478E+00_JPRB, 0.55863E+00_JPRB, 0.55168E+00_JPRB, 0.54870E+00_JPRB, 0.53970E+00_JPRB, 0.51973E+00_JPRB ,&
&  0.51343E+00_JPRB, 0.50615E+00_JPRB, 0.49760E+00_JPRB, 0.50340E+00_JPRB, 0.52586E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.52010E+00_JPRB, 0.52724E+00_JPRB, 0.53620E+00_JPRB, 0.54820E+00_JPRB, 0.55097E+00_JPRB, 0.53843E+00_JPRB, 0.51577E+00_JPRB ,&
&  0.50091E+00_JPRB, 0.49458E+00_JPRB, 0.49700E+00_JPRB, 0.50112E+00_JPRB, 0.50344E+00_JPRB, 0.50410E+00_JPRB, 0.50328E+00_JPRB ,&
&  0.50027E+00_JPRB, 0.50040E+00_JPRB, 0.50594E+00_JPRB, 0.51572E+00_JPRB, 0.52793E+00_JPRB, 0.54061E+00_JPRB, 0.55388E+00_JPRB ,&
&  0.56374E+00_JPRB, 0.56679E+00_JPRB, 0.56781E+00_JPRB, 0.56561E+00_JPRB, 0.56610E+00_JPRB, 0.56906E+00_JPRB, 0.56657E+00_JPRB ,&
&  0.55996E+00_JPRB, 0.56078E+00_JPRB, 0.56870E+00_JPRB, 0.56735E+00_JPRB, 0.56694E+00_JPRB, 0.54206E+00_JPRB, 0.49589E+00_JPRB ,&
&  0.45287E+00_JPRB, 0.41642E+00_JPRB, 0.40479E+00_JPRB, 0.40842E+00_JPRB, 0.42350E+00_JPRB, 0.44589E+00_JPRB, 0.46651E+00_JPRB ,&
&  0.48402E+00_JPRB, 0.50163E+00_JPRB, 0.51793E+00_JPRB, 0.53007E+00_JPRB, 0.54871E+00_JPRB, 0.58375E+00_JPRB, 0.63922E+00_JPRB ,&
&  0.73253E+00_JPRB, 0.85715E+00_JPRB, 0.10270E+01_JPRB, 0.10083E+01_JPRB, 0.10878E+01_JPRB, 0.12621E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.16481E+01_JPRB, 0.17034E+01_JPRB, 0.18190E+01_JPRB, 0.18782E+01_JPRB, 0.19142E+01_JPRB, 0.18426E+01_JPRB ,&
&  0.17464E+01_JPRB, 0.16356E+01_JPRB, 0.15456E+01_JPRB, 0.15040E+01_JPRB, 0.15012E+01_JPRB, 0.14922E+01_JPRB, 0.14517E+01_JPRB ,&
&  0.13738E+01_JPRB, 0.12573E+01_JPRB, 0.11208E+01_JPRB, 0.99514E+00_JPRB, 0.95282E+00_JPRB, 0.95041E+00_JPRB, 0.95387E+00_JPRB ,&
&  0.92628E+00_JPRB, 0.88600E+00_JPRB, 0.83861E+00_JPRB, 0.79756E+00_JPRB, 0.72240E+00_JPRB, 0.65515E+00_JPRB, 0.61501E+00_JPRB ,&
&  0.60021E+00_JPRB, 0.59370E+00_JPRB, 0.58932E+00_JPRB, 0.58062E+00_JPRB, 0.57070E+00_JPRB, 0.56258E+00_JPRB, 0.54114E+00_JPRB ,&
&  0.52238E+00_JPRB, 0.50910E+00_JPRB, 0.49690E+00_JPRB, 0.49639E+00_JPRB, 0.50512E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.52666E+00_JPRB, 0.53111E+00_JPRB, 0.53398E+00_JPRB, 0.53472E+00_JPRB, 0.52844E+00_JPRB, 0.52492E+00_JPRB, 0.51620E+00_JPRB ,&
&  0.51127E+00_JPRB, 0.51125E+00_JPRB, 0.50798E+00_JPRB, 0.50661E+00_JPRB, 0.50578E+00_JPRB, 0.50500E+00_JPRB, 0.50628E+00_JPRB ,&
&  0.51043E+00_JPRB, 0.51752E+00_JPRB, 0.52671E+00_JPRB, 0.53640E+00_JPRB, 0.54662E+00_JPRB, 0.55732E+00_JPRB, 0.56656E+00_JPRB ,&
&  0.57139E+00_JPRB, 0.57029E+00_JPRB, 0.57148E+00_JPRB, 0.56862E+00_JPRB, 0.56202E+00_JPRB, 0.55313E+00_JPRB, 0.54376E+00_JPRB ,&
&  0.54407E+00_JPRB, 0.55345E+00_JPRB, 0.55456E+00_JPRB, 0.55320E+00_JPRB, 0.55108E+00_JPRB, 0.51998E+00_JPRB, 0.47778E+00_JPRB ,&
&  0.43284E+00_JPRB, 0.39466E+00_JPRB, 0.37831E+00_JPRB, 0.37258E+00_JPRB, 0.37960E+00_JPRB, 0.39409E+00_JPRB, 0.41193E+00_JPRB ,&
&  0.42801E+00_JPRB, 0.45260E+00_JPRB, 0.47739E+00_JPRB, 0.49493E+00_JPRB, 0.51103E+00_JPRB, 0.53008E+00_JPRB, 0.54793E+00_JPRB ,&
&  0.57422E+00_JPRB, 0.62058E+00_JPRB, 0.70564E+00_JPRB, 0.81376E+00_JPRB, 0.91567E+00_JPRB, 0.96315E+00_JPRB, 0.10887E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.13199E+01_JPRB, 0.13077E+01_JPRB, 0.13005E+01_JPRB, 0.12852E+01_JPRB, 0.12769E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.13115E+01_JPRB, 0.13269E+01_JPRB, 0.13077E+01_JPRB, 0.12675E+01_JPRB, 0.12387E+01_JPRB, 0.12276E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.12804E+01_JPRB, 0.14335E+01_JPRB, 0.12257E+01_JPRB, 0.11203E+01_JPRB, 0.10705E+01_JPRB, 0.10134E+01_JPRB ,&
&  0.95710E+00_JPRB, 0.90801E+00_JPRB, 0.86011E+00_JPRB, 0.80813E+00_JPRB, 0.75131E+00_JPRB, 0.69949E+00_JPRB, 0.66367E+00_JPRB ,&
&  0.64813E+00_JPRB, 0.63836E+00_JPRB, 0.63236E+00_JPRB, 0.62265E+00_JPRB, 0.60378E+00_JPRB, 0.58975E+00_JPRB, 0.57898E+00_JPRB ,&
&  0.56256E+00_JPRB, 0.54328E+00_JPRB, 0.53037E+00_JPRB, 0.52758E+00_JPRB, 0.52504E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.54418E+00_JPRB, 0.54551E+00_JPRB, 0.54550E+00_JPRB, 0.54839E+00_JPRB, 0.54473E+00_JPRB, 0.54317E+00_JPRB, 0.53470E+00_JPRB ,&
&  0.52898E+00_JPRB, 0.52469E+00_JPRB, 0.52222E+00_JPRB, 0.52189E+00_JPRB, 0.52303E+00_JPRB, 0.52723E+00_JPRB, 0.53392E+00_JPRB ,&
&  0.54232E+00_JPRB, 0.54442E+00_JPRB, 0.55046E+00_JPRB, 0.55948E+00_JPRB, 0.56504E+00_JPRB, 0.57193E+00_JPRB, 0.57613E+00_JPRB ,&
&  0.57635E+00_JPRB, 0.57117E+00_JPRB, 0.56690E+00_JPRB, 0.55775E+00_JPRB, 0.55079E+00_JPRB, 0.54467E+00_JPRB, 0.54390E+00_JPRB ,&
&  0.53556E+00_JPRB, 0.53131E+00_JPRB, 0.54602E+00_JPRB, 0.55008E+00_JPRB, 0.51845E+00_JPRB, 0.48518E+00_JPRB, 0.45124E+00_JPRB ,&
&  0.41661E+00_JPRB, 0.38219E+00_JPRB, 0.36996E+00_JPRB, 0.36295E+00_JPRB, 0.36675E+00_JPRB, 0.37360E+00_JPRB, 0.38776E+00_JPRB ,&
&  0.40241E+00_JPRB, 0.42395E+00_JPRB, 0.44321E+00_JPRB, 0.46541E+00_JPRB, 0.48183E+00_JPRB, 0.50124E+00_JPRB, 0.51577E+00_JPRB ,&
&  0.53364E+00_JPRB, 0.55806E+00_JPRB, 0.59523E+00_JPRB, 0.64510E+00_JPRB, 0.68856E+00_JPRB, 0.73461E+00_JPRB, 0.78777E+00_JPRB ,&
&  0.88725E+00_JPRB, 0.91017E+00_JPRB, 0.94660E+00_JPRB, 0.96002E+00_JPRB, 0.97067E+00_JPRB, 0.99164E+00_JPRB, 0.99796E+00_JPRB ,&
&  0.10250E+01_JPRB, 0.10797E+01_JPRB, 0.10866E+01_JPRB, 0.10805E+01_JPRB, 0.10686E+01_JPRB, 0.10719E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11171E+01_JPRB, 0.12136E+01_JPRB, 0.11747E+01_JPRB, 0.11319E+01_JPRB, 0.10832E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.96933E+00_JPRB, 0.91793E+00_JPRB, 0.86481E+00_JPRB, 0.81413E+00_JPRB, 0.77299E+00_JPRB, 0.73710E+00_JPRB, 0.71109E+00_JPRB ,&
&  0.68543E+00_JPRB, 0.66349E+00_JPRB, 0.64682E+00_JPRB, 0.63213E+00_JPRB, 0.61222E+00_JPRB, 0.59352E+00_JPRB, 0.57920E+00_JPRB ,&
&  0.56351E+00_JPRB, 0.55370E+00_JPRB, 0.54585E+00_JPRB, 0.54437E+00_JPRB, 0.54334E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.57191E+00_JPRB, 0.57684E+00_JPRB, 0.58217E+00_JPRB, 0.58559E+00_JPRB, 0.58452E+00_JPRB, 0.58306E+00_JPRB, 0.57740E+00_JPRB ,&
&  0.57412E+00_JPRB, 0.57080E+00_JPRB, 0.56871E+00_JPRB, 0.56827E+00_JPRB, 0.56809E+00_JPRB, 0.56970E+00_JPRB, 0.57198E+00_JPRB ,&
&  0.57396E+00_JPRB, 0.57680E+00_JPRB, 0.57738E+00_JPRB, 0.57906E+00_JPRB, 0.57973E+00_JPRB, 0.58300E+00_JPRB, 0.58318E+00_JPRB ,&
&  0.58357E+00_JPRB, 0.57679E+00_JPRB, 0.56198E+00_JPRB, 0.55330E+00_JPRB, 0.55200E+00_JPRB, 0.53700E+00_JPRB, 0.53829E+00_JPRB ,&
&  0.54079E+00_JPRB, 0.53938E+00_JPRB, 0.53194E+00_JPRB, 0.52688E+00_JPRB, 0.50726E+00_JPRB, 0.43376E+00_JPRB, 0.42744E+00_JPRB ,&
&  0.41033E+00_JPRB, 0.38797E+00_JPRB, 0.37647E+00_JPRB, 0.36777E+00_JPRB, 0.36511E+00_JPRB, 0.37377E+00_JPRB, 0.38746E+00_JPRB ,&
&  0.39663E+00_JPRB, 0.42812E+00_JPRB, 0.44237E+00_JPRB, 0.46030E+00_JPRB, 0.47534E+00_JPRB, 0.49593E+00_JPRB, 0.50824E+00_JPRB ,&
&  0.52310E+00_JPRB, 0.53884E+00_JPRB, 0.56585E+00_JPRB, 0.60530E+00_JPRB, 0.63988E+00_JPRB, 0.67143E+00_JPRB, 0.69604E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.77491E+00_JPRB, 0.80616E+00_JPRB, 0.82593E+00_JPRB, 0.83731E+00_JPRB, 0.85072E+00_JPRB, 0.85913E+00_JPRB ,&
&  0.87890E+00_JPRB, 0.91002E+00_JPRB, 0.93928E+00_JPRB, 0.95101E+00_JPRB, 0.95488E+00_JPRB, 0.96179E+00_JPRB, 0.97388E+00_JPRB ,&
&  0.99281E+00_JPRB, 0.10090E+01_JPRB, 0.10148E+01_JPRB, 0.99630E+00_JPRB, 0.98492E+00_JPRB, 0.95354E+00_JPRB, 0.93005E+00_JPRB ,&
&  0.89591E+00_JPRB, 0.87833E+00_JPRB, 0.84058E+00_JPRB, 0.80975E+00_JPRB, 0.77693E+00_JPRB, 0.75068E+00_JPRB, 0.72027E+00_JPRB ,&
&  0.68925E+00_JPRB, 0.65915E+00_JPRB, 0.63608E+00_JPRB, 0.61847E+00_JPRB, 0.60247E+00_JPRB, 0.58541E+00_JPRB, 0.57484E+00_JPRB ,&
&  0.56724E+00_JPRB, 0.56238E+00_JPRB, 0.55977E+00_JPRB, 0.56294E+00_JPRB, 0.56574E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.58271E+00_JPRB, 0.58909E+00_JPRB, 0.59867E+00_JPRB, 0.60652E+00_JPRB, 0.61152E+00_JPRB, 0.61428E+00_JPRB, 0.61311E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.60690E+00_JPRB, 0.60428E+00_JPRB, 0.60119E+00_JPRB, 0.59997E+00_JPRB, 0.59723E+00_JPRB, 0.59599E+00_JPRB ,&
&  0.59426E+00_JPRB, 0.59297E+00_JPRB, 0.59088E+00_JPRB, 0.58900E+00_JPRB, 0.58714E+00_JPRB, 0.58655E+00_JPRB, 0.58387E+00_JPRB ,&
&  0.58246E+00_JPRB, 0.57577E+00_JPRB, 0.55893E+00_JPRB, 0.55623E+00_JPRB, 0.54890E+00_JPRB, 0.54054E+00_JPRB, 0.53040E+00_JPRB ,&
&  0.51845E+00_JPRB, 0.48274E+00_JPRB, 0.49359E+00_JPRB, 0.47749E+00_JPRB, 0.46059E+00_JPRB, 0.44434E+00_JPRB, 0.42706E+00_JPRB ,&
&  0.41669E+00_JPRB, 0.40369E+00_JPRB, 0.39400E+00_JPRB, 0.39036E+00_JPRB, 0.39032E+00_JPRB, 0.39806E+00_JPRB, 0.40892E+00_JPRB ,&
&  0.41995E+00_JPRB, 0.44571E+00_JPRB, 0.45998E+00_JPRB, 0.47671E+00_JPRB, 0.49297E+00_JPRB, 0.51173E+00_JPRB, 0.51954E+00_JPRB ,&
&  0.53394E+00_JPRB, 0.53979E+00_JPRB, 0.55868E+00_JPRB, 0.58386E+00_JPRB, 0.61612E+00_JPRB, 0.64491E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.71285E+00_JPRB, 0.74036E+00_JPRB, 0.76576E+00_JPRB, 0.78057E+00_JPRB, 0.79437E+00_JPRB, 0.80312E+00_JPRB, 0.81136E+00_JPRB ,&
&  0.81559E+00_JPRB, 0.82341E+00_JPRB, 0.83461E+00_JPRB, 0.84726E+00_JPRB, 0.85182E+00_JPRB, 0.86064E+00_JPRB, 0.87088E+00_JPRB ,&
&  0.88329E+00_JPRB, 0.88886E+00_JPRB, 0.90491E+00_JPRB, 0.89651E+00_JPRB, 0.89306E+00_JPRB, 0.86931E+00_JPRB, 0.85097E+00_JPRB ,&
&  0.83672E+00_JPRB, 0.81885E+00_JPRB, 0.79858E+00_JPRB, 0.77703E+00_JPRB, 0.75579E+00_JPRB, 0.73581E+00_JPRB, 0.71579E+00_JPRB ,&
&  0.69342E+00_JPRB, 0.66657E+00_JPRB, 0.64410E+00_JPRB, 0.61988E+00_JPRB, 0.60293E+00_JPRB, 0.59103E+00_JPRB, 0.58590E+00_JPRB ,&
&  0.58172E+00_JPRB, 0.57968E+00_JPRB, 0.57781E+00_JPRB, 0.58027E+00_JPRB, 0.58075E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.62070E+00_JPRB, 0.62044E+00_JPRB, 0.62081E+00_JPRB, 0.62046E+00_JPRB, 0.61993E+00_JPRB, 0.61847E+00_JPRB, 0.61887E+00_JPRB ,&
&  0.61846E+00_JPRB, 0.61700E+00_JPRB, 0.61469E+00_JPRB, 0.61147E+00_JPRB, 0.60963E+00_JPRB, 0.60632E+00_JPRB, 0.60347E+00_JPRB ,&
&  0.59974E+00_JPRB, 0.59600E+00_JPRB, 0.59374E+00_JPRB, 0.59257E+00_JPRB, 0.59085E+00_JPRB, 0.58989E+00_JPRB, 0.58411E+00_JPRB ,&
&  0.58003E+00_JPRB, 0.57652E+00_JPRB, 0.55410E+00_JPRB, 0.55523E+00_JPRB, 0.54558E+00_JPRB, 0.53657E+00_JPRB, 0.52971E+00_JPRB ,&
&  0.51124E+00_JPRB, 0.50236E+00_JPRB, 0.48472E+00_JPRB, 0.46937E+00_JPRB, 0.46510E+00_JPRB, 0.45171E+00_JPRB, 0.44854E+00_JPRB ,&
&  0.44360E+00_JPRB, 0.43932E+00_JPRB, 0.43764E+00_JPRB, 0.43876E+00_JPRB, 0.44414E+00_JPRB, 0.45116E+00_JPRB, 0.45921E+00_JPRB ,&
&  0.47267E+00_JPRB, 0.48612E+00_JPRB, 0.50561E+00_JPRB, 0.51544E+00_JPRB, 0.53273E+00_JPRB, 0.55176E+00_JPRB, 0.57169E+00_JPRB ,&
&  0.58421E+00_JPRB, 0.58202E+00_JPRB, 0.58686E+00_JPRB, 0.59778E+00_JPRB, 0.60948E+00_JPRB, 0.62377E+00_JPRB, 0.64551E+00_JPRB ,&
&  0.67517E+00_JPRB, 0.70282E+00_JPRB, 0.72181E+00_JPRB, 0.72996E+00_JPRB, 0.73702E+00_JPRB, 0.74055E+00_JPRB, 0.74769E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.76291E+00_JPRB, 0.76879E+00_JPRB, 0.77451E+00_JPRB, 0.77582E+00_JPRB, 0.78055E+00_JPRB, 0.78257E+00_JPRB ,&
&  0.78496E+00_JPRB, 0.78620E+00_JPRB, 0.79400E+00_JPRB, 0.78442E+00_JPRB, 0.78527E+00_JPRB, 0.78316E+00_JPRB, 0.77716E+00_JPRB ,&
&  0.76855E+00_JPRB, 0.76151E+00_JPRB, 0.75172E+00_JPRB, 0.74010E+00_JPRB, 0.72536E+00_JPRB, 0.71178E+00_JPRB, 0.69841E+00_JPRB ,&
&  0.68657E+00_JPRB, 0.67361E+00_JPRB, 0.66237E+00_JPRB, 0.64476E+00_JPRB, 0.63367E+00_JPRB, 0.62359E+00_JPRB, 0.61953E+00_JPRB ,&
&  0.61593E+00_JPRB, 0.61667E+00_JPRB, 0.61610E+00_JPRB, 0.61857E+00_JPRB, 0.61967E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.64401E+00_JPRB, 0.64367E+00_JPRB, 0.64305E+00_JPRB, 0.64155E+00_JPRB, 0.64004E+00_JPRB, 0.63811E+00_JPRB, 0.63601E+00_JPRB ,&
&  0.63373E+00_JPRB, 0.63130E+00_JPRB, 0.62849E+00_JPRB, 0.62524E+00_JPRB, 0.62142E+00_JPRB, 0.61685E+00_JPRB, 0.61370E+00_JPRB ,&
&  0.60980E+00_JPRB, 0.60636E+00_JPRB, 0.60305E+00_JPRB, 0.60029E+00_JPRB, 0.59738E+00_JPRB, 0.59477E+00_JPRB, 0.58873E+00_JPRB ,&
&  0.58512E+00_JPRB, 0.58206E+00_JPRB, 0.57788E+00_JPRB, 0.57135E+00_JPRB, 0.56783E+00_JPRB, 0.54409E+00_JPRB, 0.54501E+00_JPRB ,&
&  0.55596E+00_JPRB, 0.55255E+00_JPRB, 0.54827E+00_JPRB, 0.54285E+00_JPRB, 0.52883E+00_JPRB, 0.51893E+00_JPRB, 0.51565E+00_JPRB ,&
&  0.49588E+00_JPRB, 0.50432E+00_JPRB, 0.50589E+00_JPRB, 0.50699E+00_JPRB, 0.51080E+00_JPRB, 0.51114E+00_JPRB, 0.51538E+00_JPRB ,&
&  0.53267E+00_JPRB, 0.53780E+00_JPRB, 0.54856E+00_JPRB, 0.56146E+00_JPRB, 0.57561E+00_JPRB, 0.58944E+00_JPRB, 0.60311E+00_JPRB ,&
&  0.61340E+00_JPRB, 0.62045E+00_JPRB, 0.62529E+00_JPRB, 0.62877E+00_JPRB, 0.63424E+00_JPRB, 0.64199E+00_JPRB, 0.65465E+00_JPRB ,&
&  0.67452E+00_JPRB, 0.69627E+00_JPRB, 0.71235E+00_JPRB, 0.72082E+00_JPRB, 0.73223E+00_JPRB, 0.74252E+00_JPRB, 0.75173E+00_JPRB ,&
&  0.75856E+00_JPRB, 0.76287E+00_JPRB, 0.76443E+00_JPRB, 0.76518E+00_JPRB, 0.76418E+00_JPRB, 0.75994E+00_JPRB, 0.75488E+00_JPRB ,&
&  0.74831E+00_JPRB, 0.74132E+00_JPRB, 0.73478E+00_JPRB, 0.72782E+00_JPRB, 0.72406E+00_JPRB, 0.72011E+00_JPRB, 0.72005E+00_JPRB ,&
&  0.71771E+00_JPRB, 0.71710E+00_JPRB, 0.71317E+00_JPRB, 0.70875E+00_JPRB, 0.70054E+00_JPRB, 0.69243E+00_JPRB, 0.68243E+00_JPRB ,&
&  0.67452E+00_JPRB, 0.66782E+00_JPRB, 0.66198E+00_JPRB, 0.65461E+00_JPRB, 0.65054E+00_JPRB, 0.64671E+00_JPRB, 0.64489E+00_JPRB ,&
&  0.64292E+00_JPRB, 0.64339E+00_JPRB, 0.64338E+00_JPRB, 0.64411E+00_JPRB, 0.64387E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.64162E+00_JPRB, 0.64124E+00_JPRB, 0.64049E+00_JPRB, 0.63942E+00_JPRB, 0.63829E+00_JPRB, 0.63720E+00_JPRB, 0.63584E+00_JPRB ,&
&  0.63421E+00_JPRB, 0.63253E+00_JPRB, 0.63106E+00_JPRB, 0.62942E+00_JPRB, 0.62737E+00_JPRB, 0.62532E+00_JPRB, 0.62374E+00_JPRB ,&
&  0.62198E+00_JPRB, 0.62007E+00_JPRB, 0.61737E+00_JPRB, 0.61528E+00_JPRB, 0.61337E+00_JPRB, 0.61121E+00_JPRB, 0.60985E+00_JPRB ,&
&  0.60847E+00_JPRB, 0.60598E+00_JPRB, 0.60248E+00_JPRB, 0.59573E+00_JPRB, 0.59086E+00_JPRB, 0.58627E+00_JPRB, 0.58205E+00_JPRB ,&
&  0.57773E+00_JPRB, 0.57702E+00_JPRB, 0.57671E+00_JPRB, 0.57645E+00_JPRB, 0.57843E+00_JPRB, 0.57779E+00_JPRB, 0.57674E+00_JPRB ,&
&  0.57450E+00_JPRB, 0.57052E+00_JPRB, 0.57163E+00_JPRB, 0.57295E+00_JPRB, 0.57423E+00_JPRB, 0.57774E+00_JPRB, 0.58177E+00_JPRB ,&
&  0.58645E+00_JPRB, 0.59163E+00_JPRB, 0.59921E+00_JPRB, 0.60790E+00_JPRB, 0.61643E+00_JPRB, 0.62455E+00_JPRB, 0.63247E+00_JPRB ,&
&  0.63915E+00_JPRB, 0.64627E+00_JPRB, 0.65406E+00_JPRB, 0.66275E+00_JPRB, 0.67113E+00_JPRB, 0.68062E+00_JPRB, 0.68887E+00_JPRB ,&
&  0.69483E+00_JPRB, 0.70099E+00_JPRB, 0.71023E+00_JPRB, 0.72112E+00_JPRB, 0.73301E+00_JPRB, 0.74067E+00_JPRB, 0.74681E+00_JPRB ,&
&  0.75110E+00_JPRB, 0.75437E+00_JPRB, 0.75464E+00_JPRB, 0.75345E+00_JPRB, 0.75069E+00_JPRB, 0.74580E+00_JPRB, 0.73977E+00_JPRB ,&
&  0.73321E+00_JPRB, 0.72712E+00_JPRB, 0.72135E+00_JPRB, 0.71543E+00_JPRB, 0.70996E+00_JPRB, 0.70444E+00_JPRB, 0.70017E+00_JPRB ,&
&  0.69559E+00_JPRB, 0.69206E+00_JPRB, 0.68856E+00_JPRB, 0.68557E+00_JPRB, 0.68081E+00_JPRB, 0.67608E+00_JPRB, 0.67074E+00_JPRB ,&
&  0.66640E+00_JPRB, 0.66141E+00_JPRB, 0.65665E+00_JPRB, 0.65225E+00_JPRB, 0.64971E+00_JPRB, 0.64772E+00_JPRB, 0.64607E+00_JPRB ,&
&  0.64440E+00_JPRB, 0.64362E+00_JPRB, 0.64316E+00_JPRB, 0.64255E+00_JPRB, 0.64198E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.64483E+00_JPRB, 0.64387E+00_JPRB, 0.64284E+00_JPRB, 0.64183E+00_JPRB, 0.64068E+00_JPRB, 0.63958E+00_JPRB, 0.63841E+00_JPRB ,&
&  0.63710E+00_JPRB, 0.63581E+00_JPRB, 0.63481E+00_JPRB, 0.63391E+00_JPRB, 0.63304E+00_JPRB, 0.63215E+00_JPRB, 0.63144E+00_JPRB ,&
&  0.63074E+00_JPRB, 0.62986E+00_JPRB, 0.62885E+00_JPRB, 0.62839E+00_JPRB, 0.62803E+00_JPRB, 0.62750E+00_JPRB, 0.62714E+00_JPRB ,&
&  0.62655E+00_JPRB, 0.62527E+00_JPRB, 0.62381E+00_JPRB, 0.62297E+00_JPRB, 0.62229E+00_JPRB, 0.62172E+00_JPRB, 0.62110E+00_JPRB ,&
&  0.62048E+00_JPRB, 0.62007E+00_JPRB, 0.61949E+00_JPRB, 0.61892E+00_JPRB, 0.61882E+00_JPRB, 0.61915E+00_JPRB, 0.61961E+00_JPRB ,&
&  0.62013E+00_JPRB, 0.62081E+00_JPRB, 0.62194E+00_JPRB, 0.62308E+00_JPRB, 0.62423E+00_JPRB, 0.62528E+00_JPRB, 0.62676E+00_JPRB ,&
&  0.62805E+00_JPRB, 0.62938E+00_JPRB, 0.63110E+00_JPRB, 0.63317E+00_JPRB, 0.63531E+00_JPRB, 0.63761E+00_JPRB, 0.63999E+00_JPRB ,&
&  0.64322E+00_JPRB, 0.64691E+00_JPRB, 0.65082E+00_JPRB, 0.65469E+00_JPRB, 0.65837E+00_JPRB, 0.66217E+00_JPRB, 0.66633E+00_JPRB ,&
&  0.67014E+00_JPRB, 0.67361E+00_JPRB, 0.67758E+00_JPRB, 0.68193E+00_JPRB, 0.68606E+00_JPRB, 0.68872E+00_JPRB, 0.69155E+00_JPRB ,&
&  0.69450E+00_JPRB, 0.69721E+00_JPRB, 0.69789E+00_JPRB, 0.69865E+00_JPRB, 0.69920E+00_JPRB, 0.69914E+00_JPRB, 0.69849E+00_JPRB ,&
&  0.69758E+00_JPRB, 0.69661E+00_JPRB, 0.69452E+00_JPRB, 0.69181E+00_JPRB, 0.68920E+00_JPRB, 0.68638E+00_JPRB, 0.68331E+00_JPRB ,&
&  0.68023E+00_JPRB, 0.67755E+00_JPRB, 0.67505E+00_JPRB, 0.67252E+00_JPRB, 0.66985E+00_JPRB, 0.66743E+00_JPRB, 0.66510E+00_JPRB ,&
&  0.66299E+00_JPRB, 0.66071E+00_JPRB, 0.65858E+00_JPRB, 0.65654E+00_JPRB, 0.65482E+00_JPRB, 0.65327E+00_JPRB, 0.65188E+00_JPRB ,&
&  0.65051E+00_JPRB, 0.64925E+00_JPRB, 0.64818E+00_JPRB, 0.64706E+00_JPRB, 0.64594E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB, 0.64683E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB, 0.22523E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.25279E+00_JPRB, 0.25282E+00_JPRB, 0.25287E+00_JPRB, 0.25252E+00_JPRB, 0.25103E+00_JPRB, 0.24993E+00_JPRB, 0.24896E+00_JPRB ,&
&  0.24851E+00_JPRB, 0.24770E+00_JPRB, 0.24729E+00_JPRB, 0.24714E+00_JPRB, 0.24679E+00_JPRB, 0.24627E+00_JPRB, 0.24557E+00_JPRB ,&
&  0.24468E+00_JPRB, 0.24351E+00_JPRB, 0.24235E+00_JPRB, 0.24220E+00_JPRB, 0.24240E+00_JPRB, 0.24261E+00_JPRB, 0.24277E+00_JPRB ,&
&  0.24282E+00_JPRB, 0.24282E+00_JPRB, 0.24237E+00_JPRB, 0.24241E+00_JPRB, 0.24244E+00_JPRB, 0.24244E+00_JPRB, 0.24258E+00_JPRB ,&
&  0.24282E+00_JPRB, 0.24302E+00_JPRB, 0.24277E+00_JPRB, 0.24230E+00_JPRB, 0.24139E+00_JPRB, 0.23972E+00_JPRB, 0.23711E+00_JPRB ,&
&  0.23415E+00_JPRB, 0.23118E+00_JPRB, 0.22879E+00_JPRB, 0.22661E+00_JPRB, 0.22461E+00_JPRB, 0.22278E+00_JPRB, 0.22126E+00_JPRB ,&
&  0.21946E+00_JPRB, 0.21767E+00_JPRB, 0.21604E+00_JPRB, 0.21489E+00_JPRB, 0.21364E+00_JPRB, 0.21256E+00_JPRB, 0.21160E+00_JPRB ,&
&  0.21094E+00_JPRB, 0.21036E+00_JPRB, 0.20986E+00_JPRB, 0.20960E+00_JPRB, 0.20939E+00_JPRB, 0.20920E+00_JPRB, 0.20898E+00_JPRB ,&
&  0.20878E+00_JPRB, 0.20863E+00_JPRB, 0.20850E+00_JPRB, 0.20844E+00_JPRB, 0.20849E+00_JPRB, 0.20836E+00_JPRB, 0.20820E+00_JPRB ,&
&  0.20813E+00_JPRB, 0.20835E+00_JPRB, 0.20852E+00_JPRB, 0.20882E+00_JPRB, 0.20928E+00_JPRB, 0.20995E+00_JPRB, 0.21067E+00_JPRB ,&
&  0.21140E+00_JPRB, 0.21224E+00_JPRB, 0.21318E+00_JPRB, 0.21430E+00_JPRB, 0.21550E+00_JPRB, 0.21670E+00_JPRB, 0.21785E+00_JPRB ,&
&  0.21908E+00_JPRB, 0.22022E+00_JPRB, 0.22128E+00_JPRB, 0.22206E+00_JPRB, 0.22312E+00_JPRB, 0.22418E+00_JPRB, 0.22542E+00_JPRB ,&
&  0.22682E+00_JPRB, 0.22866E+00_JPRB, 0.23115E+00_JPRB, 0.23410E+00_JPRB, 0.23773E+00_JPRB, 0.24044E+00_JPRB, 0.24325E+00_JPRB ,&
&  0.24530E+00_JPRB, 0.24686E+00_JPRB, 0.24801E+00_JPRB, 0.24949E+00_JPRB, 0.25113E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.28806E+00_JPRB, 0.29384E+00_JPRB, 0.29795E+00_JPRB, 0.30711E+00_JPRB, 0.32113E+00_JPRB, 0.41717E+00_JPRB, 0.41217E+00_JPRB ,&
&  0.41114E+00_JPRB, 0.36834E+00_JPRB, 0.36013E+00_JPRB, 0.34791E+00_JPRB, 0.33257E+00_JPRB, 0.31359E+00_JPRB, 0.30178E+00_JPRB ,&
&  0.29195E+00_JPRB, 0.28058E+00_JPRB, 0.26986E+00_JPRB, 0.26362E+00_JPRB, 0.25866E+00_JPRB, 0.25433E+00_JPRB, 0.25192E+00_JPRB ,&
&  0.25115E+00_JPRB, 0.25179E+00_JPRB, 0.25275E+00_JPRB, 0.25428E+00_JPRB, 0.25777E+00_JPRB, 0.26153E+00_JPRB, 0.26527E+00_JPRB ,&
&  0.26725E+00_JPRB, 0.27129E+00_JPRB, 0.27198E+00_JPRB, 0.26876E+00_JPRB, 0.26341E+00_JPRB, 0.25630E+00_JPRB, 0.24912E+00_JPRB ,&
&  0.24170E+00_JPRB, 0.23478E+00_JPRB, 0.22905E+00_JPRB, 0.22386E+00_JPRB, 0.21802E+00_JPRB, 0.21261E+00_JPRB, 0.20918E+00_JPRB ,&
&  0.20610E+00_JPRB, 0.20299E+00_JPRB, 0.20057E+00_JPRB, 0.19934E+00_JPRB, 0.19794E+00_JPRB, 0.19702E+00_JPRB, 0.19634E+00_JPRB ,&
&  0.19616E+00_JPRB, 0.19605E+00_JPRB, 0.19612E+00_JPRB, 0.19641E+00_JPRB, 0.19638E+00_JPRB, 0.19630E+00_JPRB, 0.19627E+00_JPRB ,&
&  0.19680E+00_JPRB, 0.19681E+00_JPRB, 0.19670E+00_JPRB, 0.19645E+00_JPRB, 0.19701E+00_JPRB, 0.19714E+00_JPRB, 0.19742E+00_JPRB ,&
&  0.19797E+00_JPRB, 0.19877E+00_JPRB, 0.19940E+00_JPRB, 0.20013E+00_JPRB, 0.20083E+00_JPRB, 0.20166E+00_JPRB, 0.20263E+00_JPRB ,&
&  0.20361E+00_JPRB, 0.20507E+00_JPRB, 0.20655E+00_JPRB, 0.20788E+00_JPRB, 0.20906E+00_JPRB, 0.21067E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.21520E+00_JPRB, 0.21787E+00_JPRB, 0.22060E+00_JPRB, 0.22235E+00_JPRB, 0.22506E+00_JPRB, 0.22733E+00_JPRB, 0.22970E+00_JPRB ,&
&  0.23241E+00_JPRB, 0.23667E+00_JPRB, 0.24271E+00_JPRB, 0.24919E+00_JPRB, 0.25624E+00_JPRB, 0.26268E+00_JPRB, 0.26697E+00_JPRB ,&
&  0.27079E+00_JPRB, 0.27487E+00_JPRB, 0.28080E+00_JPRB, 0.28398E+00_JPRB, 0.28654E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.50606E+00_JPRB, 0.50826E+00_JPRB, 0.51067E+00_JPRB, 0.51459E+00_JPRB, 0.51876E+00_JPRB, 0.52301E+00_JPRB, 0.52681E+00_JPRB ,&
&  0.50867E+00_JPRB, 0.48552E+00_JPRB, 0.46658E+00_JPRB, 0.44409E+00_JPRB, 0.42359E+00_JPRB, 0.40692E+00_JPRB, 0.38684E+00_JPRB ,&
&  0.36686E+00_JPRB, 0.34686E+00_JPRB, 0.32425E+00_JPRB, 0.30572E+00_JPRB, 0.28749E+00_JPRB, 0.27273E+00_JPRB, 0.26354E+00_JPRB ,&
&  0.25866E+00_JPRB, 0.25740E+00_JPRB, 0.25898E+00_JPRB, 0.26234E+00_JPRB, 0.26544E+00_JPRB, 0.27026E+00_JPRB, 0.27686E+00_JPRB ,&
&  0.28596E+00_JPRB, 0.30020E+00_JPRB, 0.31308E+00_JPRB, 0.31786E+00_JPRB, 0.31454E+00_JPRB, 0.30632E+00_JPRB, 0.29925E+00_JPRB ,&
&  0.29062E+00_JPRB, 0.27837E+00_JPRB, 0.26368E+00_JPRB, 0.24918E+00_JPRB, 0.23571E+00_JPRB, 0.22588E+00_JPRB, 0.21780E+00_JPRB ,&
&  0.21102E+00_JPRB, 0.20506E+00_JPRB, 0.20051E+00_JPRB, 0.19743E+00_JPRB, 0.19479E+00_JPRB, 0.19265E+00_JPRB, 0.19064E+00_JPRB ,&
&  0.18960E+00_JPRB, 0.18850E+00_JPRB, 0.18783E+00_JPRB, 0.18744E+00_JPRB, 0.18697E+00_JPRB, 0.18684E+00_JPRB, 0.18674E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18577E+00_JPRB, 0.18541E+00_JPRB, 0.18515E+00_JPRB, 0.18551E+00_JPRB, 0.18578E+00_JPRB, 0.18625E+00_JPRB ,&
&  0.18702E+00_JPRB, 0.18870E+00_JPRB, 0.18978E+00_JPRB, 0.19079E+00_JPRB, 0.19132E+00_JPRB, 0.19175E+00_JPRB, 0.19233E+00_JPRB ,&
&  0.19331E+00_JPRB, 0.19452E+00_JPRB, 0.19563E+00_JPRB, 0.19710E+00_JPRB, 0.19862E+00_JPRB, 0.20070E+00_JPRB, 0.20291E+00_JPRB ,&
&  0.20553E+00_JPRB, 0.20877E+00_JPRB, 0.21281E+00_JPRB, 0.21655E+00_JPRB, 0.22215E+00_JPRB, 0.22779E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.24040E+00_JPRB, 0.24725E+00_JPRB, 0.25612E+00_JPRB, 0.26661E+00_JPRB, 0.27959E+00_JPRB, 0.28826E+00_JPRB, 0.29996E+00_JPRB ,&
&  0.31779E+00_JPRB, 0.35542E+00_JPRB, 0.49048E+00_JPRB, 0.50393E+00_JPRB, 0.50836E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.52017E+00_JPRB, 0.52919E+00_JPRB, 0.54355E+00_JPRB, 0.55519E+00_JPRB, 0.57963E+00_JPRB, 0.59336E+00_JPRB, 0.63974E+00_JPRB ,&
&  0.59391E+00_JPRB, 0.51475E+00_JPRB, 0.48281E+00_JPRB, 0.44825E+00_JPRB, 0.40867E+00_JPRB, 0.38023E+00_JPRB, 0.36013E+00_JPRB ,&
&  0.34818E+00_JPRB, 0.33800E+00_JPRB, 0.32823E+00_JPRB, 0.31601E+00_JPRB, 0.30341E+00_JPRB, 0.28926E+00_JPRB, 0.27851E+00_JPRB ,&
&  0.27204E+00_JPRB, 0.27015E+00_JPRB, 0.26981E+00_JPRB, 0.27416E+00_JPRB, 0.28020E+00_JPRB, 0.28820E+00_JPRB, 0.29737E+00_JPRB ,&
&  0.31790E+00_JPRB, 0.32383E+00_JPRB, 0.32379E+00_JPRB, 0.32389E+00_JPRB, 0.32512E+00_JPRB, 0.32730E+00_JPRB, 0.31769E+00_JPRB ,&
&  0.32727E+00_JPRB, 0.34948E+00_JPRB, 0.35554E+00_JPRB, 0.33715E+00_JPRB, 0.28267E+00_JPRB, 0.24978E+00_JPRB, 0.22725E+00_JPRB ,&
&  0.21856E+00_JPRB, 0.20898E+00_JPRB, 0.20300E+00_JPRB, 0.19742E+00_JPRB, 0.19391E+00_JPRB, 0.19207E+00_JPRB, 0.19027E+00_JPRB ,&
&  0.18893E+00_JPRB, 0.18784E+00_JPRB, 0.18587E+00_JPRB, 0.18494E+00_JPRB, 0.18390E+00_JPRB, 0.18335E+00_JPRB, 0.18301E+00_JPRB ,&
&  0.18283E+00_JPRB, 0.18247E+00_JPRB, 0.18261E+00_JPRB, 0.18309E+00_JPRB, 0.18396E+00_JPRB, 0.18465E+00_JPRB, 0.18528E+00_JPRB ,&
&  0.18606E+00_JPRB, 0.18675E+00_JPRB, 0.18687E+00_JPRB, 0.18726E+00_JPRB, 0.18761E+00_JPRB, 0.18866E+00_JPRB, 0.18910E+00_JPRB ,&
&  0.18960E+00_JPRB, 0.18998E+00_JPRB, 0.18995E+00_JPRB, 0.19047E+00_JPRB, 0.19134E+00_JPRB, 0.19310E+00_JPRB, 0.19516E+00_JPRB ,&
&  0.19781E+00_JPRB, 0.20087E+00_JPRB, 0.20429E+00_JPRB, 0.20768E+00_JPRB, 0.21214E+00_JPRB, 0.21720E+00_JPRB, 0.22365E+00_JPRB ,&
&  0.23070E+00_JPRB, 0.23936E+00_JPRB, 0.25088E+00_JPRB, 0.26505E+00_JPRB, 0.28226E+00_JPRB, 0.30045E+00_JPRB, 0.32161E+00_JPRB ,&
&  0.35711E+00_JPRB, 0.45768E+00_JPRB, 0.51783E+00_JPRB, 0.50443E+00_JPRB, 0.51444E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.59820E+00_JPRB, 0.61452E+00_JPRB, 0.63985E+00_JPRB, 0.63260E+00_JPRB, 0.65296E+00_JPRB, 0.63948E+00_JPRB, 0.61692E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.51259E+00_JPRB, 0.46619E+00_JPRB, 0.41409E+00_JPRB, 0.37375E+00_JPRB, 0.34416E+00_JPRB, 0.32974E+00_JPRB ,&
&  0.32282E+00_JPRB, 0.32066E+00_JPRB, 0.31960E+00_JPRB, 0.31886E+00_JPRB, 0.31444E+00_JPRB, 0.31127E+00_JPRB, 0.30569E+00_JPRB ,&
&  0.29978E+00_JPRB, 0.29518E+00_JPRB, 0.29190E+00_JPRB, 0.29689E+00_JPRB, 0.30317E+00_JPRB, 0.30978E+00_JPRB, 0.31348E+00_JPRB ,&
&  0.31680E+00_JPRB, 0.31811E+00_JPRB, 0.32074E+00_JPRB, 0.31891E+00_JPRB, 0.32992E+00_JPRB, 0.35132E+00_JPRB, 0.36986E+00_JPRB ,&
&  0.38779E+00_JPRB, 0.40277E+00_JPRB, 0.39886E+00_JPRB, 0.37530E+00_JPRB, 0.30973E+00_JPRB, 0.27550E+00_JPRB, 0.24577E+00_JPRB ,&
&  0.22717E+00_JPRB, 0.21339E+00_JPRB, 0.20425E+00_JPRB, 0.19901E+00_JPRB, 0.19494E+00_JPRB, 0.19260E+00_JPRB, 0.19009E+00_JPRB ,&
&  0.18767E+00_JPRB, 0.18612E+00_JPRB, 0.18445E+00_JPRB, 0.18362E+00_JPRB, 0.18270E+00_JPRB, 0.18149E+00_JPRB, 0.18054E+00_JPRB ,&
&  0.18062E+00_JPRB, 0.18038E+00_JPRB, 0.18141E+00_JPRB, 0.18233E+00_JPRB, 0.18418E+00_JPRB, 0.18576E+00_JPRB, 0.18790E+00_JPRB ,&
&  0.18963E+00_JPRB, 0.19103E+00_JPRB, 0.19085E+00_JPRB, 0.19031E+00_JPRB, 0.18942E+00_JPRB, 0.18887E+00_JPRB, 0.18795E+00_JPRB ,&
&  0.18684E+00_JPRB, 0.18696E+00_JPRB, 0.18713E+00_JPRB, 0.18743E+00_JPRB, 0.18846E+00_JPRB, 0.18997E+00_JPRB, 0.19235E+00_JPRB ,&
&  0.19546E+00_JPRB, 0.19874E+00_JPRB, 0.20150E+00_JPRB, 0.20437E+00_JPRB, 0.20810E+00_JPRB, 0.21237E+00_JPRB, 0.21773E+00_JPRB ,&
&  0.22265E+00_JPRB, 0.23079E+00_JPRB, 0.24192E+00_JPRB, 0.25336E+00_JPRB, 0.27003E+00_JPRB, 0.28708E+00_JPRB, 0.31566E+00_JPRB ,&
&  0.36519E+00_JPRB, 0.53904E+00_JPRB, 0.82327E+00_JPRB, 0.59662E+00_JPRB, 0.58430E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.63482E+00_JPRB, 0.64585E+00_JPRB, 0.65999E+00_JPRB, 0.66296E+00_JPRB, 0.67308E+00_JPRB, 0.67196E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.62676E+00_JPRB, 0.60041E+00_JPRB, 0.58592E+00_JPRB, 0.56882E+00_JPRB, 0.54135E+00_JPRB, 0.48086E+00_JPRB, 0.45764E+00_JPRB ,&
&  0.43925E+00_JPRB, 0.43894E+00_JPRB, 0.42989E+00_JPRB, 0.43014E+00_JPRB, 0.41923E+00_JPRB, 0.40795E+00_JPRB, 0.38955E+00_JPRB ,&
&  0.37248E+00_JPRB, 0.35525E+00_JPRB, 0.34340E+00_JPRB, 0.33750E+00_JPRB, 0.33538E+00_JPRB, 0.33106E+00_JPRB, 0.31925E+00_JPRB ,&
&  0.31074E+00_JPRB, 0.30901E+00_JPRB, 0.31161E+00_JPRB, 0.32174E+00_JPRB, 0.36577E+00_JPRB, 0.42177E+00_JPRB, 0.42213E+00_JPRB ,&
&  0.44505E+00_JPRB, 0.45869E+00_JPRB, 0.47197E+00_JPRB, 0.46124E+00_JPRB, 0.42884E+00_JPRB, 0.39070E+00_JPRB, 0.33662E+00_JPRB ,&
&  0.28506E+00_JPRB, 0.25635E+00_JPRB, 0.23309E+00_JPRB, 0.21701E+00_JPRB, 0.20624E+00_JPRB, 0.20175E+00_JPRB, 0.19603E+00_JPRB ,&
&  0.19222E+00_JPRB, 0.18934E+00_JPRB, 0.18813E+00_JPRB, 0.18743E+00_JPRB, 0.18668E+00_JPRB, 0.18465E+00_JPRB, 0.18405E+00_JPRB ,&
&  0.18452E+00_JPRB, 0.18470E+00_JPRB, 0.18453E+00_JPRB, 0.18528E+00_JPRB, 0.18622E+00_JPRB, 0.18756E+00_JPRB, 0.18875E+00_JPRB ,&
&  0.19052E+00_JPRB, 0.19147E+00_JPRB, 0.19307E+00_JPRB, 0.19481E+00_JPRB, 0.19387E+00_JPRB, 0.19266E+00_JPRB, 0.18986E+00_JPRB ,&
&  0.18681E+00_JPRB, 0.18638E+00_JPRB, 0.18582E+00_JPRB, 0.18638E+00_JPRB, 0.18791E+00_JPRB, 0.19002E+00_JPRB, 0.19282E+00_JPRB ,&
&  0.19552E+00_JPRB, 0.19831E+00_JPRB, 0.20049E+00_JPRB, 0.20264E+00_JPRB, 0.20619E+00_JPRB, 0.21001E+00_JPRB, 0.21484E+00_JPRB ,&
&  0.21986E+00_JPRB, 0.22551E+00_JPRB, 0.23334E+00_JPRB, 0.24347E+00_JPRB, 0.25774E+00_JPRB, 0.27406E+00_JPRB, 0.29934E+00_JPRB ,&
&  0.32345E+00_JPRB, 0.42443E+00_JPRB, 0.66124E+00_JPRB, 0.58861E+00_JPRB, 0.60713E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.71060E+00_JPRB, 0.72208E+00_JPRB, 0.73194E+00_JPRB, 0.73379E+00_JPRB, 0.73309E+00_JPRB, 0.72819E+00_JPRB, 0.71622E+00_JPRB ,&
&  0.69660E+00_JPRB, 0.67038E+00_JPRB, 0.64492E+00_JPRB, 0.62462E+00_JPRB, 0.61192E+00_JPRB, 0.61520E+00_JPRB, 0.62403E+00_JPRB ,&
&  0.64174E+00_JPRB, 0.66748E+00_JPRB, 0.67295E+00_JPRB, 0.66660E+00_JPRB, 0.63103E+00_JPRB, 0.59390E+00_JPRB, 0.55929E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.49995E+00_JPRB, 0.48909E+00_JPRB, 0.47032E+00_JPRB, 0.45708E+00_JPRB, 0.44054E+00_JPRB, 0.41728E+00_JPRB ,&
&  0.37919E+00_JPRB, 0.34854E+00_JPRB, 0.33082E+00_JPRB, 0.34467E+00_JPRB, 0.43400E+00_JPRB, 0.49460E+00_JPRB, 0.50379E+00_JPRB ,&
&  0.53846E+00_JPRB, 0.56329E+00_JPRB, 0.55856E+00_JPRB, 0.53886E+00_JPRB, 0.51750E+00_JPRB, 0.49427E+00_JPRB, 0.47524E+00_JPRB ,&
&  0.45627E+00_JPRB, 0.42999E+00_JPRB, 0.38072E+00_JPRB, 0.31091E+00_JPRB, 0.27530E+00_JPRB, 0.26181E+00_JPRB, 0.24736E+00_JPRB ,&
&  0.23641E+00_JPRB, 0.22739E+00_JPRB, 0.22386E+00_JPRB, 0.22065E+00_JPRB, 0.21992E+00_JPRB, 0.21588E+00_JPRB, 0.21244E+00_JPRB ,&
&  0.20822E+00_JPRB, 0.20513E+00_JPRB, 0.20048E+00_JPRB, 0.19587E+00_JPRB, 0.19250E+00_JPRB, 0.19190E+00_JPRB, 0.19157E+00_JPRB ,&
&  0.19250E+00_JPRB, 0.19646E+00_JPRB, 0.20767E+00_JPRB, 0.21388E+00_JPRB, 0.21425E+00_JPRB, 0.20944E+00_JPRB, 0.20227E+00_JPRB ,&
&  0.19581E+00_JPRB, 0.19383E+00_JPRB, 0.19240E+00_JPRB, 0.19291E+00_JPRB, 0.19557E+00_JPRB, 0.19916E+00_JPRB, 0.20217E+00_JPRB ,&
&  0.20458E+00_JPRB, 0.20693E+00_JPRB, 0.20918E+00_JPRB, 0.21160E+00_JPRB, 0.21452E+00_JPRB, 0.21795E+00_JPRB, 0.22262E+00_JPRB ,&
&  0.22762E+00_JPRB, 0.23460E+00_JPRB, 0.24468E+00_JPRB, 0.25914E+00_JPRB, 0.27192E+00_JPRB, 0.27997E+00_JPRB, 0.29081E+00_JPRB ,&
&  0.30673E+00_JPRB, 0.35355E+00_JPRB, 0.48757E+00_JPRB, 0.71441E+00_JPRB, 0.74660E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.77188E+00_JPRB, 0.78998E+00_JPRB, 0.81107E+00_JPRB, 0.82225E+00_JPRB, 0.82330E+00_JPRB, 0.82042E+00_JPRB, 0.81183E+00_JPRB ,&
&  0.79700E+00_JPRB, 0.77907E+00_JPRB, 0.76588E+00_JPRB, 0.75698E+00_JPRB, 0.75311E+00_JPRB, 0.75613E+00_JPRB, 0.76174E+00_JPRB ,&
&  0.76126E+00_JPRB, 0.75735E+00_JPRB, 0.74546E+00_JPRB, 0.72998E+00_JPRB, 0.71362E+00_JPRB, 0.70086E+00_JPRB, 0.69094E+00_JPRB ,&
&  0.68113E+00_JPRB, 0.66632E+00_JPRB, 0.64328E+00_JPRB, 0.60796E+00_JPRB, 0.56628E+00_JPRB, 0.52816E+00_JPRB, 0.49916E+00_JPRB ,&
&  0.47463E+00_JPRB, 0.40738E+00_JPRB, 0.37007E+00_JPRB, 0.38239E+00_JPRB, 0.47556E+00_JPRB, 0.50593E+00_JPRB, 0.52990E+00_JPRB ,&
&  0.56710E+00_JPRB, 0.60663E+00_JPRB, 0.62548E+00_JPRB, 0.62600E+00_JPRB, 0.62055E+00_JPRB, 0.60812E+00_JPRB, 0.58864E+00_JPRB ,&
&  0.56385E+00_JPRB, 0.53343E+00_JPRB, 0.50755E+00_JPRB, 0.47873E+00_JPRB, 0.44897E+00_JPRB, 0.42698E+00_JPRB, 0.41165E+00_JPRB ,&
&  0.40106E+00_JPRB, 0.39152E+00_JPRB, 0.38400E+00_JPRB, 0.37429E+00_JPRB, 0.36512E+00_JPRB, 0.34405E+00_JPRB, 0.32227E+00_JPRB ,&
&  0.29871E+00_JPRB, 0.28129E+00_JPRB, 0.26263E+00_JPRB, 0.24442E+00_JPRB, 0.22946E+00_JPRB, 0.21972E+00_JPRB, 0.21262E+00_JPRB ,&
&  0.21035E+00_JPRB, 0.21601E+00_JPRB, 0.23443E+00_JPRB, 0.26738E+00_JPRB, 0.30593E+00_JPRB, 0.27575E+00_JPRB, 0.25496E+00_JPRB ,&
&  0.23448E+00_JPRB, 0.22241E+00_JPRB, 0.21790E+00_JPRB, 0.21737E+00_JPRB, 0.21864E+00_JPRB, 0.22097E+00_JPRB, 0.22334E+00_JPRB ,&
&  0.22652E+00_JPRB, 0.22947E+00_JPRB, 0.23261E+00_JPRB, 0.23489E+00_JPRB, 0.23640E+00_JPRB, 0.23912E+00_JPRB, 0.24379E+00_JPRB ,&
&  0.25093E+00_JPRB, 0.26278E+00_JPRB, 0.28246E+00_JPRB, 0.30805E+00_JPRB, 0.33447E+00_JPRB, 0.35856E+00_JPRB, 0.41165E+00_JPRB ,&
&  0.49913E+00_JPRB, 0.61158E+00_JPRB, 0.66799E+00_JPRB, 0.72329E+00_JPRB, 0.75060E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.83269E+00_JPRB, 0.85905E+00_JPRB, 0.88827E+00_JPRB, 0.90502E+00_JPRB, 0.90581E+00_JPRB, 0.90259E+00_JPRB, 0.88759E+00_JPRB ,&
&  0.85663E+00_JPRB, 0.83106E+00_JPRB, 0.82665E+00_JPRB, 0.83042E+00_JPRB, 0.84213E+00_JPRB, 0.86080E+00_JPRB, 0.86262E+00_JPRB ,&
&  0.83650E+00_JPRB, 0.81741E+00_JPRB, 0.80797E+00_JPRB, 0.80128E+00_JPRB, 0.78470E+00_JPRB, 0.76037E+00_JPRB, 0.74452E+00_JPRB ,&
&  0.74006E+00_JPRB, 0.74009E+00_JPRB, 0.73113E+00_JPRB, 0.71710E+00_JPRB, 0.68557E+00_JPRB, 0.64676E+00_JPRB, 0.61649E+00_JPRB ,&
&  0.60641E+00_JPRB, 0.59802E+00_JPRB, 0.55711E+00_JPRB, 0.57429E+00_JPRB, 0.57794E+00_JPRB, 0.57816E+00_JPRB, 0.57584E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.62192E+00_JPRB, 0.65483E+00_JPRB, 0.66459E+00_JPRB, 0.67411E+00_JPRB, 0.67604E+00_JPRB, 0.67774E+00_JPRB ,&
&  0.68246E+00_JPRB, 0.67452E+00_JPRB, 0.64496E+00_JPRB, 0.60110E+00_JPRB, 0.55780E+00_JPRB, 0.52412E+00_JPRB, 0.49542E+00_JPRB ,&
&  0.47561E+00_JPRB, 0.46322E+00_JPRB, 0.45355E+00_JPRB, 0.45015E+00_JPRB, 0.45765E+00_JPRB, 0.47251E+00_JPRB, 0.46738E+00_JPRB ,&
&  0.44928E+00_JPRB, 0.43276E+00_JPRB, 0.41078E+00_JPRB, 0.38099E+00_JPRB, 0.35074E+00_JPRB, 0.31190E+00_JPRB, 0.29209E+00_JPRB ,&
&  0.29647E+00_JPRB, 0.33545E+00_JPRB, 0.38687E+00_JPRB, 0.40589E+00_JPRB, 0.43298E+00_JPRB, 0.45918E+00_JPRB, 0.44859E+00_JPRB ,&
&  0.40190E+00_JPRB, 0.33250E+00_JPRB, 0.30965E+00_JPRB, 0.29742E+00_JPRB, 0.29062E+00_JPRB, 0.28574E+00_JPRB, 0.28501E+00_JPRB ,&
&  0.28924E+00_JPRB, 0.30062E+00_JPRB, 0.30998E+00_JPRB, 0.30680E+00_JPRB, 0.29650E+00_JPRB, 0.29006E+00_JPRB, 0.29063E+00_JPRB ,&
&  0.29960E+00_JPRB, 0.32758E+00_JPRB, 0.39119E+00_JPRB, 0.50752E+00_JPRB, 0.54298E+00_JPRB, 0.57579E+00_JPRB, 0.61378E+00_JPRB ,&
&  0.65021E+00_JPRB, 0.68338E+00_JPRB, 0.71178E+00_JPRB, 0.74683E+00_JPRB, 0.79373E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.82831E+00_JPRB, 0.84977E+00_JPRB, 0.86843E+00_JPRB, 0.88113E+00_JPRB, 0.88287E+00_JPRB, 0.87609E+00_JPRB, 0.86027E+00_JPRB ,&
&  0.83034E+00_JPRB, 0.81333E+00_JPRB, 0.82232E+00_JPRB, 0.83699E+00_JPRB, 0.84597E+00_JPRB, 0.84593E+00_JPRB, 0.81447E+00_JPRB ,&
&  0.76040E+00_JPRB, 0.73991E+00_JPRB, 0.73689E+00_JPRB, 0.74655E+00_JPRB, 0.76492E+00_JPRB, 0.77214E+00_JPRB, 0.76035E+00_JPRB ,&
&  0.74825E+00_JPRB, 0.75002E+00_JPRB, 0.75288E+00_JPRB, 0.75974E+00_JPRB, 0.74889E+00_JPRB, 0.72789E+00_JPRB, 0.71900E+00_JPRB ,&
&  0.71658E+00_JPRB, 0.70941E+00_JPRB, 0.70191E+00_JPRB, 0.65306E+00_JPRB, 0.67781E+00_JPRB, 0.66621E+00_JPRB, 0.64803E+00_JPRB ,&
&  0.63154E+00_JPRB, 0.63428E+00_JPRB, 0.67586E+00_JPRB, 0.72217E+00_JPRB, 0.73361E+00_JPRB, 0.73648E+00_JPRB, 0.73182E+00_JPRB ,&
&  0.72850E+00_JPRB, 0.72889E+00_JPRB, 0.73026E+00_JPRB, 0.72212E+00_JPRB, 0.70173E+00_JPRB, 0.67547E+00_JPRB, 0.64665E+00_JPRB ,&
&  0.62912E+00_JPRB, 0.60404E+00_JPRB, 0.56044E+00_JPRB, 0.54013E+00_JPRB, 0.53812E+00_JPRB, 0.54598E+00_JPRB, 0.54978E+00_JPRB ,&
&  0.54669E+00_JPRB, 0.53932E+00_JPRB, 0.54350E+00_JPRB, 0.54078E+00_JPRB, 0.51908E+00_JPRB, 0.50599E+00_JPRB, 0.49999E+00_JPRB ,&
&  0.48838E+00_JPRB, 0.48693E+00_JPRB, 0.49136E+00_JPRB, 0.50270E+00_JPRB, 0.51267E+00_JPRB, 0.52609E+00_JPRB, 0.53146E+00_JPRB ,&
&  0.53350E+00_JPRB, 0.53013E+00_JPRB, 0.52448E+00_JPRB, 0.51270E+00_JPRB, 0.49448E+00_JPRB, 0.48757E+00_JPRB, 0.48906E+00_JPRB ,&
&  0.49783E+00_JPRB, 0.50360E+00_JPRB, 0.50286E+00_JPRB, 0.50167E+00_JPRB, 0.49018E+00_JPRB, 0.47757E+00_JPRB, 0.46782E+00_JPRB ,&
&  0.46316E+00_JPRB, 0.46568E+00_JPRB, 0.48054E+00_JPRB, 0.50512E+00_JPRB, 0.53679E+00_JPRB, 0.57510E+00_JPRB, 0.61973E+00_JPRB ,&
&  0.65155E+00_JPRB, 0.67045E+00_JPRB, 0.69491E+00_JPRB, 0.74233E+00_JPRB, 0.78792E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.79134E+00_JPRB, 0.79963E+00_JPRB, 0.80945E+00_JPRB, 0.82259E+00_JPRB, 0.83448E+00_JPRB, 0.83134E+00_JPRB, 0.81893E+00_JPRB ,&
&  0.79529E+00_JPRB, 0.78653E+00_JPRB, 0.79126E+00_JPRB, 0.79368E+00_JPRB, 0.79542E+00_JPRB, 0.79308E+00_JPRB, 0.77702E+00_JPRB ,&
&  0.74008E+00_JPRB, 0.71042E+00_JPRB, 0.69614E+00_JPRB, 0.68871E+00_JPRB, 0.68627E+00_JPRB, 0.69332E+00_JPRB, 0.70200E+00_JPRB ,&
&  0.71930E+00_JPRB, 0.72985E+00_JPRB, 0.74218E+00_JPRB, 0.75165E+00_JPRB, 0.75428E+00_JPRB, 0.75811E+00_JPRB, 0.75928E+00_JPRB ,&
&  0.74540E+00_JPRB, 0.74586E+00_JPRB, 0.74771E+00_JPRB, 0.73841E+00_JPRB, 0.71749E+00_JPRB, 0.69643E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.65305E+00_JPRB, 0.64455E+00_JPRB, 0.67022E+00_JPRB, 0.73191E+00_JPRB, 0.77518E+00_JPRB, 0.78330E+00_JPRB, 0.77560E+00_JPRB ,&
&  0.77773E+00_JPRB, 0.78516E+00_JPRB, 0.78864E+00_JPRB, 0.77710E+00_JPRB, 0.75741E+00_JPRB, 0.74015E+00_JPRB, 0.72354E+00_JPRB ,&
&  0.71775E+00_JPRB, 0.71661E+00_JPRB, 0.69883E+00_JPRB, 0.66635E+00_JPRB, 0.63865E+00_JPRB, 0.61910E+00_JPRB, 0.60888E+00_JPRB ,&
&  0.60284E+00_JPRB, 0.59354E+00_JPRB, 0.58796E+00_JPRB, 0.59311E+00_JPRB, 0.57344E+00_JPRB, 0.55550E+00_JPRB, 0.55918E+00_JPRB ,&
&  0.57538E+00_JPRB, 0.58050E+00_JPRB, 0.58929E+00_JPRB, 0.59189E+00_JPRB, 0.58824E+00_JPRB, 0.58395E+00_JPRB, 0.58481E+00_JPRB ,&
&  0.59309E+00_JPRB, 0.60256E+00_JPRB, 0.60951E+00_JPRB, 0.61442E+00_JPRB, 0.61948E+00_JPRB, 0.60793E+00_JPRB, 0.58696E+00_JPRB ,&
&  0.56931E+00_JPRB, 0.56586E+00_JPRB, 0.57023E+00_JPRB, 0.56949E+00_JPRB, 0.56785E+00_JPRB, 0.56793E+00_JPRB, 0.56643E+00_JPRB ,&
&  0.56348E+00_JPRB, 0.55591E+00_JPRB, 0.54696E+00_JPRB, 0.54727E+00_JPRB, 0.55878E+00_JPRB, 0.58131E+00_JPRB, 0.61622E+00_JPRB ,&
&  0.66839E+00_JPRB, 0.70786E+00_JPRB, 0.74556E+00_JPRB, 0.76068E+00_JPRB, 0.76392E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.76213E+00_JPRB, 0.77037E+00_JPRB, 0.77635E+00_JPRB, 0.78066E+00_JPRB, 0.78928E+00_JPRB, 0.80005E+00_JPRB, 0.79983E+00_JPRB ,&
&  0.78512E+00_JPRB, 0.77146E+00_JPRB, 0.76194E+00_JPRB, 0.75921E+00_JPRB, 0.76085E+00_JPRB, 0.76295E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.73652E+00_JPRB, 0.71440E+00_JPRB, 0.70078E+00_JPRB, 0.68721E+00_JPRB, 0.67882E+00_JPRB, 0.67353E+00_JPRB, 0.68051E+00_JPRB ,&
&  0.69086E+00_JPRB, 0.70713E+00_JPRB, 0.72046E+00_JPRB, 0.73015E+00_JPRB, 0.73416E+00_JPRB, 0.72297E+00_JPRB, 0.72449E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.73427E+00_JPRB, 0.74962E+00_JPRB, 0.74937E+00_JPRB, 0.73651E+00_JPRB, 0.72637E+00_JPRB, 0.71611E+00_JPRB ,&
&  0.71343E+00_JPRB, 0.72484E+00_JPRB, 0.75034E+00_JPRB, 0.78696E+00_JPRB, 0.80796E+00_JPRB, 0.81589E+00_JPRB, 0.81302E+00_JPRB ,&
&  0.81608E+00_JPRB, 0.82816E+00_JPRB, 0.83491E+00_JPRB, 0.82667E+00_JPRB, 0.80837E+00_JPRB, 0.79340E+00_JPRB, 0.78008E+00_JPRB ,&
&  0.77006E+00_JPRB, 0.75791E+00_JPRB, 0.74955E+00_JPRB, 0.73931E+00_JPRB, 0.71736E+00_JPRB, 0.69142E+00_JPRB, 0.66895E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.63625E+00_JPRB, 0.61651E+00_JPRB, 0.61056E+00_JPRB, 0.61487E+00_JPRB, 0.59192E+00_JPRB, 0.56684E+00_JPRB ,&
&  0.57009E+00_JPRB, 0.60086E+00_JPRB, 0.62482E+00_JPRB, 0.63411E+00_JPRB, 0.63441E+00_JPRB, 0.63823E+00_JPRB, 0.64123E+00_JPRB ,&
&  0.64732E+00_JPRB, 0.65559E+00_JPRB, 0.66191E+00_JPRB, 0.67283E+00_JPRB, 0.68426E+00_JPRB, 0.68282E+00_JPRB, 0.66895E+00_JPRB ,&
&  0.65357E+00_JPRB, 0.64467E+00_JPRB, 0.64069E+00_JPRB, 0.64265E+00_JPRB, 0.64586E+00_JPRB, 0.64366E+00_JPRB, 0.63780E+00_JPRB ,&
&  0.62622E+00_JPRB, 0.60781E+00_JPRB, 0.59219E+00_JPRB, 0.58371E+00_JPRB, 0.58360E+00_JPRB, 0.59057E+00_JPRB, 0.60694E+00_JPRB ,&
&  0.63851E+00_JPRB, 0.68629E+00_JPRB, 0.72123E+00_JPRB, 0.74043E+00_JPRB, 0.74508E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.73609E+00_JPRB, 0.74253E+00_JPRB, 0.75194E+00_JPRB, 0.76550E+00_JPRB, 0.77939E+00_JPRB, 0.78823E+00_JPRB, 0.78758E+00_JPRB ,&
&  0.77642E+00_JPRB, 0.76305E+00_JPRB, 0.75859E+00_JPRB, 0.75789E+00_JPRB, 0.75487E+00_JPRB, 0.75350E+00_JPRB, 0.74908E+00_JPRB ,&
&  0.73830E+00_JPRB, 0.72175E+00_JPRB, 0.71399E+00_JPRB, 0.70037E+00_JPRB, 0.68742E+00_JPRB, 0.67689E+00_JPRB, 0.67713E+00_JPRB ,&
&  0.68599E+00_JPRB, 0.69619E+00_JPRB, 0.70875E+00_JPRB, 0.72318E+00_JPRB, 0.73123E+00_JPRB, 0.73022E+00_JPRB, 0.72734E+00_JPRB ,&
&  0.72507E+00_JPRB, 0.72256E+00_JPRB, 0.73386E+00_JPRB, 0.74314E+00_JPRB, 0.74937E+00_JPRB, 0.76068E+00_JPRB, 0.76992E+00_JPRB ,&
&  0.77747E+00_JPRB, 0.78866E+00_JPRB, 0.80748E+00_JPRB, 0.82146E+00_JPRB, 0.83153E+00_JPRB, 0.84218E+00_JPRB, 0.84604E+00_JPRB ,&
&  0.85238E+00_JPRB, 0.86362E+00_JPRB, 0.86604E+00_JPRB, 0.85720E+00_JPRB, 0.84287E+00_JPRB, 0.83352E+00_JPRB, 0.82808E+00_JPRB ,&
&  0.81846E+00_JPRB, 0.79884E+00_JPRB, 0.78399E+00_JPRB, 0.77222E+00_JPRB, 0.75322E+00_JPRB, 0.73296E+00_JPRB, 0.70939E+00_JPRB ,&
&  0.68258E+00_JPRB, 0.66268E+00_JPRB, 0.64724E+00_JPRB, 0.63334E+00_JPRB, 0.63005E+00_JPRB, 0.62149E+00_JPRB, 0.61471E+00_JPRB ,&
&  0.61899E+00_JPRB, 0.63678E+00_JPRB, 0.64994E+00_JPRB, 0.65405E+00_JPRB, 0.65602E+00_JPRB, 0.67131E+00_JPRB, 0.68999E+00_JPRB ,&
&  0.70453E+00_JPRB, 0.71046E+00_JPRB, 0.70622E+00_JPRB, 0.70829E+00_JPRB, 0.71322E+00_JPRB, 0.71752E+00_JPRB, 0.71170E+00_JPRB ,&
&  0.70329E+00_JPRB, 0.69777E+00_JPRB, 0.69659E+00_JPRB, 0.69302E+00_JPRB, 0.69452E+00_JPRB, 0.69437E+00_JPRB, 0.69153E+00_JPRB ,&
&  0.68578E+00_JPRB, 0.67653E+00_JPRB, 0.66471E+00_JPRB, 0.65783E+00_JPRB, 0.64778E+00_JPRB, 0.63257E+00_JPRB, 0.63410E+00_JPRB ,&
&  0.64590E+00_JPRB, 0.66446E+00_JPRB, 0.68164E+00_JPRB, 0.70173E+00_JPRB, 0.72514E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.71779E+00_JPRB, 0.73060E+00_JPRB, 0.74657E+00_JPRB, 0.76273E+00_JPRB, 0.77277E+00_JPRB, 0.77880E+00_JPRB, 0.77632E+00_JPRB ,&
&  0.76520E+00_JPRB, 0.75421E+00_JPRB, 0.75552E+00_JPRB, 0.75514E+00_JPRB, 0.75395E+00_JPRB, 0.75212E+00_JPRB, 0.74557E+00_JPRB ,&
&  0.73628E+00_JPRB, 0.72848E+00_JPRB, 0.72527E+00_JPRB, 0.72236E+00_JPRB, 0.71255E+00_JPRB, 0.70505E+00_JPRB, 0.69701E+00_JPRB ,&
&  0.69706E+00_JPRB, 0.69994E+00_JPRB, 0.70320E+00_JPRB, 0.70394E+00_JPRB, 0.71244E+00_JPRB, 0.72315E+00_JPRB, 0.72791E+00_JPRB ,&
&  0.72270E+00_JPRB, 0.72940E+00_JPRB, 0.64768E+00_JPRB, 0.71761E+00_JPRB, 0.74296E+00_JPRB, 0.75796E+00_JPRB, 0.77308E+00_JPRB ,&
&  0.78315E+00_JPRB, 0.79505E+00_JPRB, 0.80418E+00_JPRB, 0.80940E+00_JPRB, 0.82678E+00_JPRB, 0.84106E+00_JPRB, 0.84466E+00_JPRB ,&
&  0.84585E+00_JPRB, 0.84485E+00_JPRB, 0.84620E+00_JPRB, 0.84623E+00_JPRB, 0.85034E+00_JPRB, 0.84978E+00_JPRB, 0.84326E+00_JPRB ,&
&  0.83472E+00_JPRB, 0.82272E+00_JPRB, 0.80522E+00_JPRB, 0.78517E+00_JPRB, 0.75363E+00_JPRB, 0.72615E+00_JPRB, 0.70873E+00_JPRB ,&
&  0.69429E+00_JPRB, 0.69005E+00_JPRB, 0.68472E+00_JPRB, 0.67915E+00_JPRB, 0.67693E+00_JPRB, 0.67909E+00_JPRB, 0.67767E+00_JPRB ,&
&  0.68383E+00_JPRB, 0.69216E+00_JPRB, 0.69609E+00_JPRB, 0.70306E+00_JPRB, 0.71223E+00_JPRB, 0.72583E+00_JPRB, 0.73867E+00_JPRB ,&
&  0.75080E+00_JPRB, 0.75900E+00_JPRB, 0.76006E+00_JPRB, 0.75998E+00_JPRB, 0.75778E+00_JPRB, 0.75684E+00_JPRB, 0.74993E+00_JPRB ,&
&  0.74439E+00_JPRB, 0.74150E+00_JPRB, 0.73532E+00_JPRB, 0.72361E+00_JPRB, 0.71660E+00_JPRB, 0.71068E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.71056E+00_JPRB, 0.71206E+00_JPRB, 0.70980E+00_JPRB, 0.71075E+00_JPRB, 0.70479E+00_JPRB, 0.68528E+00_JPRB, 0.67008E+00_JPRB ,&
&  0.66975E+00_JPRB, 0.67529E+00_JPRB, 0.68041E+00_JPRB, 0.68981E+00_JPRB, 0.70516E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.76116E+00_JPRB, 0.76713E+00_JPRB, 0.77634E+00_JPRB, 0.78664E+00_JPRB, 0.78991E+00_JPRB, 0.78905E+00_JPRB, 0.78621E+00_JPRB ,&
&  0.78648E+00_JPRB, 0.77798E+00_JPRB, 0.77162E+00_JPRB, 0.76782E+00_JPRB, 0.76702E+00_JPRB, 0.76373E+00_JPRB, 0.75480E+00_JPRB ,&
&  0.74745E+00_JPRB, 0.74477E+00_JPRB, 0.74953E+00_JPRB, 0.75121E+00_JPRB, 0.73619E+00_JPRB, 0.73139E+00_JPRB, 0.72366E+00_JPRB ,&
&  0.72254E+00_JPRB, 0.72304E+00_JPRB, 0.72639E+00_JPRB, 0.73304E+00_JPRB, 0.74986E+00_JPRB, 0.76190E+00_JPRB, 0.76848E+00_JPRB ,&
&  0.76735E+00_JPRB, 0.68838E+00_JPRB, 0.71624E+00_JPRB, 0.72890E+00_JPRB, 0.75169E+00_JPRB, 0.77472E+00_JPRB, 0.79577E+00_JPRB ,&
&  0.82385E+00_JPRB, 0.84060E+00_JPRB, 0.84886E+00_JPRB, 0.85113E+00_JPRB, 0.85302E+00_JPRB, 0.85261E+00_JPRB, 0.84455E+00_JPRB ,&
&  0.83850E+00_JPRB, 0.84462E+00_JPRB, 0.86022E+00_JPRB, 0.86770E+00_JPRB, 0.87089E+00_JPRB, 0.87216E+00_JPRB, 0.86787E+00_JPRB ,&
&  0.85993E+00_JPRB, 0.84500E+00_JPRB, 0.82095E+00_JPRB, 0.79249E+00_JPRB, 0.76044E+00_JPRB, 0.73851E+00_JPRB, 0.72630E+00_JPRB ,&
&  0.71819E+00_JPRB, 0.71782E+00_JPRB, 0.71801E+00_JPRB, 0.71886E+00_JPRB, 0.72159E+00_JPRB, 0.72935E+00_JPRB, 0.73198E+00_JPRB ,&
&  0.73904E+00_JPRB, 0.74609E+00_JPRB, 0.75781E+00_JPRB, 0.77697E+00_JPRB, 0.79655E+00_JPRB, 0.80761E+00_JPRB, 0.80839E+00_JPRB ,&
&  0.81408E+00_JPRB, 0.81666E+00_JPRB, 0.81289E+00_JPRB, 0.81276E+00_JPRB, 0.80908E+00_JPRB, 0.81046E+00_JPRB, 0.80779E+00_JPRB ,&
&  0.79931E+00_JPRB, 0.79316E+00_JPRB, 0.78570E+00_JPRB, 0.77093E+00_JPRB, 0.75825E+00_JPRB, 0.74923E+00_JPRB, 0.74174E+00_JPRB ,&
&  0.74140E+00_JPRB, 0.74539E+00_JPRB, 0.75087E+00_JPRB, 0.74627E+00_JPRB, 0.73290E+00_JPRB, 0.72517E+00_JPRB, 0.71557E+00_JPRB ,&
&  0.70933E+00_JPRB, 0.71217E+00_JPRB, 0.71338E+00_JPRB, 0.72245E+00_JPRB, 0.74309E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.79926E+00_JPRB, 0.80297E+00_JPRB, 0.80699E+00_JPRB, 0.80747E+00_JPRB, 0.80351E+00_JPRB, 0.80081E+00_JPRB, 0.80216E+00_JPRB ,&
&  0.80289E+00_JPRB, 0.79464E+00_JPRB, 0.78639E+00_JPRB, 0.78043E+00_JPRB, 0.77833E+00_JPRB, 0.77032E+00_JPRB, 0.75797E+00_JPRB ,&
&  0.74911E+00_JPRB, 0.74835E+00_JPRB, 0.75406E+00_JPRB, 0.75747E+00_JPRB, 0.74937E+00_JPRB, 0.74056E+00_JPRB, 0.73768E+00_JPRB ,&
&  0.73890E+00_JPRB, 0.74529E+00_JPRB, 0.75588E+00_JPRB, 0.76248E+00_JPRB, 0.77172E+00_JPRB, 0.78636E+00_JPRB, 0.78814E+00_JPRB ,&
&  0.78700E+00_JPRB, 0.65243E+00_JPRB, 0.69014E+00_JPRB, 0.70487E+00_JPRB, 0.73541E+00_JPRB, 0.78182E+00_JPRB, 0.82138E+00_JPRB ,&
&  0.85988E+00_JPRB, 0.88371E+00_JPRB, 0.89534E+00_JPRB, 0.89022E+00_JPRB, 0.87523E+00_JPRB, 0.85996E+00_JPRB, 0.85461E+00_JPRB ,&
&  0.85178E+00_JPRB, 0.85613E+00_JPRB, 0.86749E+00_JPRB, 0.87518E+00_JPRB, 0.87689E+00_JPRB, 0.87599E+00_JPRB, 0.87673E+00_JPRB ,&
&  0.87243E+00_JPRB, 0.85590E+00_JPRB, 0.83325E+00_JPRB, 0.80614E+00_JPRB, 0.78527E+00_JPRB, 0.77598E+00_JPRB, 0.76362E+00_JPRB ,&
&  0.74629E+00_JPRB, 0.73550E+00_JPRB, 0.73347E+00_JPRB, 0.73573E+00_JPRB, 0.74368E+00_JPRB, 0.75262E+00_JPRB, 0.76048E+00_JPRB ,&
&  0.76999E+00_JPRB, 0.78525E+00_JPRB, 0.80181E+00_JPRB, 0.81773E+00_JPRB, 0.83443E+00_JPRB, 0.85036E+00_JPRB, 0.86293E+00_JPRB ,&
&  0.87134E+00_JPRB, 0.86973E+00_JPRB, 0.85824E+00_JPRB, 0.84912E+00_JPRB, 0.84542E+00_JPRB, 0.84953E+00_JPRB, 0.85015E+00_JPRB ,&
&  0.84230E+00_JPRB, 0.83597E+00_JPRB, 0.83349E+00_JPRB, 0.82374E+00_JPRB, 0.81284E+00_JPRB, 0.80303E+00_JPRB, 0.79274E+00_JPRB ,&
&  0.78477E+00_JPRB, 0.78238E+00_JPRB, 0.78000E+00_JPRB, 0.77491E+00_JPRB, 0.76690E+00_JPRB, 0.76469E+00_JPRB, 0.76286E+00_JPRB ,&
&  0.76091E+00_JPRB, 0.75794E+00_JPRB, 0.76056E+00_JPRB, 0.77210E+00_JPRB, 0.78828E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.82362E+00_JPRB, 0.83007E+00_JPRB, 0.83368E+00_JPRB, 0.83776E+00_JPRB, 0.83748E+00_JPRB, 0.83507E+00_JPRB, 0.83846E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.83680E+00_JPRB, 0.82809E+00_JPRB, 0.82520E+00_JPRB, 0.81613E+00_JPRB, 0.80271E+00_JPRB, 0.79189E+00_JPRB ,&
&  0.77831E+00_JPRB, 0.77312E+00_JPRB, 0.77006E+00_JPRB, 0.76693E+00_JPRB, 0.76076E+00_JPRB, 0.75292E+00_JPRB, 0.75668E+00_JPRB ,&
&  0.76349E+00_JPRB, 0.77079E+00_JPRB, 0.77306E+00_JPRB, 0.77758E+00_JPRB, 0.78416E+00_JPRB, 0.79159E+00_JPRB, 0.80478E+00_JPRB ,&
&  0.69485E+00_JPRB, 0.57237E+00_JPRB, 0.62396E+00_JPRB, 0.67664E+00_JPRB, 0.72645E+00_JPRB, 0.80049E+00_JPRB, 0.85573E+00_JPRB ,&
&  0.90114E+00_JPRB, 0.94049E+00_JPRB, 0.94934E+00_JPRB, 0.92650E+00_JPRB, 0.90179E+00_JPRB, 0.89230E+00_JPRB, 0.88952E+00_JPRB ,&
&  0.88422E+00_JPRB, 0.88150E+00_JPRB, 0.88352E+00_JPRB, 0.88535E+00_JPRB, 0.88721E+00_JPRB, 0.89102E+00_JPRB, 0.89144E+00_JPRB ,&
&  0.88806E+00_JPRB, 0.87490E+00_JPRB, 0.86205E+00_JPRB, 0.84269E+00_JPRB, 0.82606E+00_JPRB, 0.82102E+00_JPRB, 0.81220E+00_JPRB ,&
&  0.79713E+00_JPRB, 0.78738E+00_JPRB, 0.78500E+00_JPRB, 0.78626E+00_JPRB, 0.79346E+00_JPRB, 0.80278E+00_JPRB, 0.82004E+00_JPRB ,&
&  0.84084E+00_JPRB, 0.85663E+00_JPRB, 0.86776E+00_JPRB, 0.88070E+00_JPRB, 0.88806E+00_JPRB, 0.89781E+00_JPRB, 0.91518E+00_JPRB ,&
&  0.92946E+00_JPRB, 0.92049E+00_JPRB, 0.91171E+00_JPRB, 0.90249E+00_JPRB, 0.89908E+00_JPRB, 0.89639E+00_JPRB, 0.88709E+00_JPRB ,&
&  0.87842E+00_JPRB, 0.87340E+00_JPRB, 0.87140E+00_JPRB, 0.87630E+00_JPRB, 0.87548E+00_JPRB, 0.87075E+00_JPRB, 0.86939E+00_JPRB ,&
&  0.86902E+00_JPRB, 0.86206E+00_JPRB, 0.85282E+00_JPRB, 0.85001E+00_JPRB, 0.84455E+00_JPRB, 0.84323E+00_JPRB, 0.83977E+00_JPRB ,&
&  0.83647E+00_JPRB, 0.82221E+00_JPRB, 0.82756E+00_JPRB, 0.82155E+00_JPRB, 0.81833E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.83563E+00_JPRB, 0.85712E+00_JPRB, 0.86023E+00_JPRB, 0.87096E+00_JPRB, 0.87712E+00_JPRB, 0.88114E+00_JPRB, 0.88889E+00_JPRB ,&
&  0.89389E+00_JPRB, 0.88939E+00_JPRB, 0.88183E+00_JPRB, 0.86731E+00_JPRB, 0.84414E+00_JPRB, 0.81918E+00_JPRB, 0.79878E+00_JPRB ,&
&  0.78452E+00_JPRB, 0.78264E+00_JPRB, 0.77525E+00_JPRB, 0.76554E+00_JPRB, 0.75982E+00_JPRB, 0.75687E+00_JPRB, 0.75936E+00_JPRB ,&
&  0.76753E+00_JPRB, 0.77191E+00_JPRB, 0.76727E+00_JPRB, 0.76655E+00_JPRB, 0.77590E+00_JPRB, 0.79265E+00_JPRB, 0.80089E+00_JPRB ,&
&  0.80354E+00_JPRB, 0.61012E+00_JPRB, 0.65235E+00_JPRB, 0.70620E+00_JPRB, 0.75604E+00_JPRB, 0.82322E+00_JPRB, 0.89910E+00_JPRB ,&
&  0.95121E+00_JPRB, 0.98241E+00_JPRB, 0.98345E+00_JPRB, 0.97193E+00_JPRB, 0.95049E+00_JPRB, 0.93056E+00_JPRB, 0.90777E+00_JPRB ,&
&  0.88790E+00_JPRB, 0.88084E+00_JPRB, 0.88342E+00_JPRB, 0.88594E+00_JPRB, 0.88579E+00_JPRB, 0.88759E+00_JPRB, 0.88027E+00_JPRB ,&
&  0.87703E+00_JPRB, 0.87272E+00_JPRB, 0.86981E+00_JPRB, 0.86879E+00_JPRB, 0.86684E+00_JPRB, 0.86548E+00_JPRB, 0.86022E+00_JPRB ,&
&  0.84526E+00_JPRB, 0.83276E+00_JPRB, 0.82156E+00_JPRB, 0.81144E+00_JPRB, 0.81262E+00_JPRB, 0.82366E+00_JPRB, 0.84313E+00_JPRB ,&
&  0.86527E+00_JPRB, 0.87086E+00_JPRB, 0.87809E+00_JPRB, 0.88662E+00_JPRB, 0.89921E+00_JPRB, 0.91211E+00_JPRB, 0.92760E+00_JPRB ,&
&  0.93323E+00_JPRB, 0.92705E+00_JPRB, 0.92169E+00_JPRB, 0.91109E+00_JPRB, 0.90966E+00_JPRB, 0.89503E+00_JPRB, 0.87746E+00_JPRB ,&
&  0.87543E+00_JPRB, 0.87640E+00_JPRB, 0.87888E+00_JPRB, 0.89014E+00_JPRB, 0.89803E+00_JPRB, 0.89925E+00_JPRB, 0.90781E+00_JPRB ,&
&  0.92023E+00_JPRB, 0.92944E+00_JPRB, 0.92724E+00_JPRB, 0.93558E+00_JPRB, 0.93366E+00_JPRB, 0.93311E+00_JPRB, 0.91758E+00_JPRB ,&
&  0.89511E+00_JPRB, 0.87875E+00_JPRB, 0.68638E+00_JPRB, 0.77842E+00_JPRB, 0.80497E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.89535E+00_JPRB, 0.92746E+00_JPRB, 0.93936E+00_JPRB, 0.93657E+00_JPRB, 0.93717E+00_JPRB, 0.93545E+00_JPRB, 0.92982E+00_JPRB ,&
&  0.92669E+00_JPRB, 0.92088E+00_JPRB, 0.90727E+00_JPRB, 0.88654E+00_JPRB, 0.86518E+00_JPRB, 0.84221E+00_JPRB, 0.82468E+00_JPRB ,&
&  0.81974E+00_JPRB, 0.81738E+00_JPRB, 0.80527E+00_JPRB, 0.79397E+00_JPRB, 0.78314E+00_JPRB, 0.77450E+00_JPRB, 0.77145E+00_JPRB ,&
&  0.76730E+00_JPRB, 0.76596E+00_JPRB, 0.77187E+00_JPRB, 0.77961E+00_JPRB, 0.78956E+00_JPRB, 0.80055E+00_JPRB, 0.81534E+00_JPRB ,&
&  0.81769E+00_JPRB, 0.63393E+00_JPRB, 0.66550E+00_JPRB, 0.76527E+00_JPRB, 0.83735E+00_JPRB, 0.92748E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10295E+01_JPRB, 0.10518E+01_JPRB, 0.10411E+01_JPRB, 0.10224E+01_JPRB, 0.10161E+01_JPRB, 0.99498E+00_JPRB, 0.95371E+00_JPRB ,&
&  0.92238E+00_JPRB, 0.91810E+00_JPRB, 0.92796E+00_JPRB, 0.93346E+00_JPRB, 0.92899E+00_JPRB, 0.91944E+00_JPRB, 0.91329E+00_JPRB ,&
&  0.90643E+00_JPRB, 0.89867E+00_JPRB, 0.88984E+00_JPRB, 0.87827E+00_JPRB, 0.88153E+00_JPRB, 0.89770E+00_JPRB, 0.90943E+00_JPRB ,&
&  0.90524E+00_JPRB, 0.88555E+00_JPRB, 0.86402E+00_JPRB, 0.85163E+00_JPRB, 0.85271E+00_JPRB, 0.86978E+00_JPRB, 0.89008E+00_JPRB ,&
&  0.90164E+00_JPRB, 0.90737E+00_JPRB, 0.91681E+00_JPRB, 0.91937E+00_JPRB, 0.92611E+00_JPRB, 0.93337E+00_JPRB, 0.94541E+00_JPRB ,&
&  0.95095E+00_JPRB, 0.94404E+00_JPRB, 0.93542E+00_JPRB, 0.92230E+00_JPRB, 0.90440E+00_JPRB, 0.89347E+00_JPRB, 0.89037E+00_JPRB ,&
&  0.88723E+00_JPRB, 0.89402E+00_JPRB, 0.91270E+00_JPRB, 0.93524E+00_JPRB, 0.94425E+00_JPRB, 0.94787E+00_JPRB, 0.95258E+00_JPRB ,&
&  0.96240E+00_JPRB, 0.96924E+00_JPRB, 0.97459E+00_JPRB, 0.96615E+00_JPRB, 0.10154E+01_JPRB, 0.10196E+01_JPRB, 0.10034E+01_JPRB ,&
&  0.97978E+00_JPRB, 0.95075E+00_JPRB, 0.94602E+00_JPRB, 0.79146E+00_JPRB, 0.86587E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.97609E+00_JPRB, 0.10036E+01_JPRB, 0.99444E+00_JPRB, 0.98526E+00_JPRB, 0.97608E+00_JPRB, 0.95714E+00_JPRB, 0.93332E+00_JPRB ,&
&  0.90957E+00_JPRB, 0.89604E+00_JPRB, 0.87829E+00_JPRB, 0.85529E+00_JPRB, 0.84041E+00_JPRB, 0.83175E+00_JPRB, 0.82368E+00_JPRB ,&
&  0.82081E+00_JPRB, 0.81771E+00_JPRB, 0.81551E+00_JPRB, 0.81256E+00_JPRB, 0.80442E+00_JPRB, 0.79718E+00_JPRB, 0.78516E+00_JPRB ,&
&  0.77755E+00_JPRB, 0.77744E+00_JPRB, 0.77855E+00_JPRB, 0.78287E+00_JPRB, 0.79045E+00_JPRB, 0.79892E+00_JPRB, 0.80476E+00_JPRB ,&
&  0.80169E+00_JPRB, 0.69932E+00_JPRB, 0.75113E+00_JPRB, 0.86843E+00_JPRB, 0.99930E+00_JPRB, 0.11036E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.11130E+01_JPRB, 0.10976E+01_JPRB, 0.10773E+01_JPRB, 0.10542E+01_JPRB, 0.10316E+01_JPRB, 0.10174E+01_JPRB, 0.98850E+00_JPRB ,&
&  0.97130E+00_JPRB, 0.96530E+00_JPRB, 0.96801E+00_JPRB, 0.96550E+00_JPRB, 0.95754E+00_JPRB, 0.94707E+00_JPRB, 0.94186E+00_JPRB ,&
&  0.93188E+00_JPRB, 0.90575E+00_JPRB, 0.87661E+00_JPRB, 0.85429E+00_JPRB, 0.84898E+00_JPRB, 0.88076E+00_JPRB, 0.91609E+00_JPRB ,&
&  0.93983E+00_JPRB, 0.91246E+00_JPRB, 0.88392E+00_JPRB, 0.87132E+00_JPRB, 0.87281E+00_JPRB, 0.89072E+00_JPRB, 0.91759E+00_JPRB ,&
&  0.92737E+00_JPRB, 0.92997E+00_JPRB, 0.94286E+00_JPRB, 0.94572E+00_JPRB, 0.94831E+00_JPRB, 0.94634E+00_JPRB, 0.94342E+00_JPRB ,&
&  0.93483E+00_JPRB, 0.92463E+00_JPRB, 0.91894E+00_JPRB, 0.91340E+00_JPRB, 0.91344E+00_JPRB, 0.90694E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.88312E+00_JPRB, 0.89486E+00_JPRB, 0.92998E+00_JPRB, 0.96187E+00_JPRB, 0.96237E+00_JPRB, 0.95427E+00_JPRB, 0.95010E+00_JPRB ,&
&  0.95696E+00_JPRB, 0.95415E+00_JPRB, 0.95039E+00_JPRB, 0.10256E+01_JPRB, 0.10361E+01_JPRB, 0.10394E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10418E+01_JPRB, 0.10257E+01_JPRB, 0.99922E+00_JPRB, 0.91938E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.10645E+01_JPRB, 0.10533E+01_JPRB, 0.10378E+01_JPRB, 0.10275E+01_JPRB, 0.10095E+01_JPRB, 0.97604E+00_JPRB, 0.94094E+00_JPRB ,&
&  0.91629E+00_JPRB, 0.88884E+00_JPRB, 0.85178E+00_JPRB, 0.83323E+00_JPRB, 0.82823E+00_JPRB, 0.82827E+00_JPRB, 0.82394E+00_JPRB ,&
&  0.81904E+00_JPRB, 0.82331E+00_JPRB, 0.81867E+00_JPRB, 0.82044E+00_JPRB, 0.81458E+00_JPRB, 0.81287E+00_JPRB, 0.79559E+00_JPRB ,&
&  0.78498E+00_JPRB, 0.77739E+00_JPRB, 0.77545E+00_JPRB, 0.78207E+00_JPRB, 0.78778E+00_JPRB, 0.79180E+00_JPRB, 0.79637E+00_JPRB ,&
&  0.78576E+00_JPRB, 0.72474E+00_JPRB, 0.86635E+00_JPRB, 0.10635E+01_JPRB, 0.12060E+01_JPRB, 0.12754E+01_JPRB, 0.12202E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.11394E+01_JPRB, 0.11358E+01_JPRB, 0.11152E+01_JPRB, 0.10806E+01_JPRB, 0.10510E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.10305E+01_JPRB, 0.10234E+01_JPRB, 0.10175E+01_JPRB, 0.10081E+01_JPRB, 0.99582E+00_JPRB, 0.98253E+00_JPRB, 0.97969E+00_JPRB ,&
&  0.96340E+00_JPRB, 0.92519E+00_JPRB, 0.88645E+00_JPRB, 0.86035E+00_JPRB, 0.87283E+00_JPRB, 0.90748E+00_JPRB, 0.95819E+00_JPRB ,&
&  0.98267E+00_JPRB, 0.95650E+00_JPRB, 0.94058E+00_JPRB, 0.93297E+00_JPRB, 0.93379E+00_JPRB, 0.94977E+00_JPRB, 0.95696E+00_JPRB ,&
&  0.96231E+00_JPRB, 0.95775E+00_JPRB, 0.95572E+00_JPRB, 0.94957E+00_JPRB, 0.94424E+00_JPRB, 0.94156E+00_JPRB, 0.93437E+00_JPRB ,&
&  0.92569E+00_JPRB, 0.92458E+00_JPRB, 0.91445E+00_JPRB, 0.91369E+00_JPRB, 0.90585E+00_JPRB, 0.89268E+00_JPRB, 0.88094E+00_JPRB ,&
&  0.89030E+00_JPRB, 0.91958E+00_JPRB, 0.94272E+00_JPRB, 0.97233E+00_JPRB, 0.97999E+00_JPRB, 0.97024E+00_JPRB, 0.97293E+00_JPRB ,&
&  0.96684E+00_JPRB, 0.96279E+00_JPRB, 0.93051E+00_JPRB, 0.10498E+01_JPRB, 0.11716E+01_JPRB, 0.11644E+01_JPRB, 0.11504E+01_JPRB ,&
&  0.11429E+01_JPRB, 0.11156E+01_JPRB, 0.10560E+01_JPRB, 0.10057E+01_JPRB, 0.10457E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.99345E+00_JPRB, 0.97666E+00_JPRB, 0.98307E+00_JPRB, 0.98741E+00_JPRB, 0.97301E+00_JPRB, 0.93823E+00_JPRB, 0.90860E+00_JPRB ,&
&  0.89571E+00_JPRB, 0.86398E+00_JPRB, 0.83413E+00_JPRB, 0.81433E+00_JPRB, 0.81820E+00_JPRB, 0.82243E+00_JPRB, 0.82097E+00_JPRB ,&
&  0.80947E+00_JPRB, 0.80667E+00_JPRB, 0.79405E+00_JPRB, 0.78521E+00_JPRB, 0.78239E+00_JPRB, 0.78285E+00_JPRB, 0.77580E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.78261E+00_JPRB, 0.78724E+00_JPRB, 0.79462E+00_JPRB, 0.79166E+00_JPRB, 0.79488E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.76628E+00_JPRB, 0.83933E+00_JPRB, 0.10870E+01_JPRB, 0.13867E+01_JPRB, 0.15157E+01_JPRB, 0.14259E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.11853E+01_JPRB, 0.11882E+01_JPRB, 0.11684E+01_JPRB, 0.11412E+01_JPRB, 0.11175E+01_JPRB, 0.10894E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10433E+01_JPRB, 0.10368E+01_JPRB, 0.10193E+01_JPRB, 0.99666E+00_JPRB, 0.98819E+00_JPRB, 0.98895E+00_JPRB ,&
&  0.94383E+00_JPRB, 0.90817E+00_JPRB, 0.87962E+00_JPRB, 0.86165E+00_JPRB, 0.91484E+00_JPRB, 0.99268E+00_JPRB, 0.10832E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.10283E+01_JPRB, 0.10203E+01_JPRB, 0.10177E+01_JPRB, 0.10326E+01_JPRB, 0.10074E+01_JPRB, 0.98821E+00_JPRB ,&
&  0.95999E+00_JPRB, 0.95880E+00_JPRB, 0.95920E+00_JPRB, 0.96684E+00_JPRB, 0.96432E+00_JPRB, 0.96140E+00_JPRB, 0.93097E+00_JPRB ,&
&  0.91460E+00_JPRB, 0.90051E+00_JPRB, 0.89311E+00_JPRB, 0.89052E+00_JPRB, 0.89292E+00_JPRB, 0.88118E+00_JPRB, 0.87170E+00_JPRB ,&
&  0.88346E+00_JPRB, 0.90042E+00_JPRB, 0.95689E+00_JPRB, 0.10277E+01_JPRB, 0.10319E+01_JPRB, 0.10088E+01_JPRB, 0.99569E+00_JPRB ,&
&  0.98933E+00_JPRB, 0.10007E+01_JPRB, 0.97494E+00_JPRB, 0.11100E+01_JPRB, 0.12267E+01_JPRB, 0.13021E+01_JPRB, 0.12324E+01_JPRB ,&
&  0.11502E+01_JPRB, 0.10580E+01_JPRB, 0.10058E+01_JPRB, 0.98178E+00_JPRB, 0.10035E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.93203E+00_JPRB, 0.92967E+00_JPRB, 0.94502E+00_JPRB, 0.95577E+00_JPRB, 0.93948E+00_JPRB, 0.90556E+00_JPRB, 0.87366E+00_JPRB ,&
&  0.86118E+00_JPRB, 0.85466E+00_JPRB, 0.84576E+00_JPRB, 0.82315E+00_JPRB, 0.80907E+00_JPRB, 0.80380E+00_JPRB, 0.80155E+00_JPRB ,&
&  0.80121E+00_JPRB, 0.78877E+00_JPRB, 0.77254E+00_JPRB, 0.75660E+00_JPRB, 0.75802E+00_JPRB, 0.76671E+00_JPRB, 0.78293E+00_JPRB ,&
&  0.78120E+00_JPRB, 0.78113E+00_JPRB, 0.78745E+00_JPRB, 0.79450E+00_JPRB, 0.80341E+00_JPRB, 0.79480E+00_JPRB, 0.78181E+00_JPRB ,&
&  0.78440E+00_JPRB, 0.10028E+01_JPRB, 0.14566E+01_JPRB, 0.16561E+01_JPRB, 0.15888E+01_JPRB, 0.14015E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.12221E+01_JPRB, 0.12299E+01_JPRB, 0.11981E+01_JPRB, 0.11592E+01_JPRB, 0.11252E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.10583E+01_JPRB, 0.10283E+01_JPRB, 0.10013E+01_JPRB, 0.99473E+00_JPRB, 0.99247E+00_JPRB, 0.99709E+00_JPRB, 0.97419E+00_JPRB ,&
&  0.94125E+00_JPRB, 0.91137E+00_JPRB, 0.88535E+00_JPRB, 0.87783E+00_JPRB, 0.93753E+00_JPRB, 0.96213E+00_JPRB, 0.11172E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.11722E+01_JPRB, 0.11239E+01_JPRB, 0.11242E+01_JPRB, 0.11154E+01_JPRB, 0.10838E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10522E+01_JPRB, 0.10803E+01_JPRB, 0.10816E+01_JPRB, 0.10417E+01_JPRB, 0.98891E+00_JPRB, 0.96388E+00_JPRB ,&
&  0.95304E+00_JPRB, 0.94097E+00_JPRB, 0.93335E+00_JPRB, 0.92410E+00_JPRB, 0.91387E+00_JPRB, 0.89111E+00_JPRB, 0.88364E+00_JPRB ,&
&  0.89405E+00_JPRB, 0.93395E+00_JPRB, 0.10286E+01_JPRB, 0.10556E+01_JPRB, 0.10860E+01_JPRB, 0.10956E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.98590E+00_JPRB, 0.98496E+00_JPRB, 0.99132E+00_JPRB, 0.10906E+01_JPRB, 0.12586E+01_JPRB, 0.13260E+01_JPRB, 0.12121E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.99898E+00_JPRB, 0.95057E+00_JPRB, 0.92770E+00_JPRB, 0.94598E+00_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.87875E+00_JPRB, 0.87053E+00_JPRB, 0.87831E+00_JPRB, 0.89582E+00_JPRB, 0.88116E+00_JPRB, 0.85551E+00_JPRB, 0.82959E+00_JPRB ,&
&  0.84797E+00_JPRB, 0.85606E+00_JPRB, 0.85681E+00_JPRB, 0.82494E+00_JPRB, 0.80547E+00_JPRB, 0.78344E+00_JPRB, 0.77026E+00_JPRB ,&
&  0.76927E+00_JPRB, 0.75939E+00_JPRB, 0.74471E+00_JPRB, 0.73841E+00_JPRB, 0.74351E+00_JPRB, 0.75019E+00_JPRB, 0.76787E+00_JPRB ,&
&  0.76129E+00_JPRB, 0.76844E+00_JPRB, 0.79181E+00_JPRB, 0.80423E+00_JPRB, 0.81453E+00_JPRB, 0.79722E+00_JPRB, 0.78802E+00_JPRB ,&
&  0.82246E+00_JPRB, 0.91454E+00_JPRB, 0.15653E+01_JPRB, 0.19352E+01_JPRB, 0.15797E+01_JPRB, 0.13510E+01_JPRB, 0.12178E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.12569E+01_JPRB, 0.12375E+01_JPRB, 0.11831E+01_JPRB, 0.11536E+01_JPRB, 0.11110E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10337E+01_JPRB, 0.10103E+01_JPRB, 0.10034E+01_JPRB, 0.10119E+01_JPRB, 0.10099E+01_JPRB, 0.99732E+00_JPRB, 0.96880E+00_JPRB ,&
&  0.94738E+00_JPRB, 0.92462E+00_JPRB, 0.89840E+00_JPRB, 0.91655E+00_JPRB, 0.10246E+01_JPRB, 0.11782E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.13085E+01_JPRB, 0.12265E+01_JPRB, 0.12163E+01_JPRB, 0.11828E+01_JPRB, 0.11572E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.10852E+01_JPRB, 0.11027E+01_JPRB, 0.10926E+01_JPRB, 0.10548E+01_JPRB, 0.10150E+01_JPRB, 0.99709E+00_JPRB, 0.97474E+00_JPRB ,&
&  0.96583E+00_JPRB, 0.97551E+00_JPRB, 0.97607E+00_JPRB, 0.95573E+00_JPRB, 0.93675E+00_JPRB, 0.91702E+00_JPRB, 0.91507E+00_JPRB ,&
&  0.93746E+00_JPRB, 0.97053E+00_JPRB, 0.10468E+01_JPRB, 0.11122E+01_JPRB, 0.11017E+01_JPRB, 0.10675E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10210E+01_JPRB, 0.10028E+01_JPRB, 0.99842E+00_JPRB, 0.10948E+01_JPRB, 0.11499E+01_JPRB, 0.12190E+01_JPRB, 0.11105E+01_JPRB ,&
&  0.10008E+01_JPRB, 0.92768E+00_JPRB, 0.89331E+00_JPRB, 0.88196E+00_JPRB, 0.88710E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.85371E+00_JPRB, 0.83530E+00_JPRB, 0.84557E+00_JPRB, 0.85352E+00_JPRB, 0.84834E+00_JPRB, 0.82551E+00_JPRB, 0.81123E+00_JPRB ,&
&  0.82666E+00_JPRB, 0.82993E+00_JPRB, 0.82488E+00_JPRB, 0.79793E+00_JPRB, 0.78241E+00_JPRB, 0.75777E+00_JPRB, 0.74094E+00_JPRB ,&
&  0.74192E+00_JPRB, 0.74455E+00_JPRB, 0.73684E+00_JPRB, 0.73735E+00_JPRB, 0.74037E+00_JPRB, 0.75656E+00_JPRB, 0.76834E+00_JPRB ,&
&  0.78052E+00_JPRB, 0.79123E+00_JPRB, 0.80934E+00_JPRB, 0.82010E+00_JPRB, 0.81581E+00_JPRB, 0.80253E+00_JPRB, 0.80140E+00_JPRB ,&
&  0.89148E+00_JPRB, 0.20900E+01_JPRB, 0.13793E+01_JPRB, 0.20893E+01_JPRB, 0.15232E+01_JPRB, 0.12749E+01_JPRB, 0.10360E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.12574E+01_JPRB, 0.12652E+01_JPRB, 0.11886E+01_JPRB, 0.11641E+01_JPRB, 0.11325E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.10443E+01_JPRB, 0.10460E+01_JPRB, 0.10266E+01_JPRB, 0.10233E+01_JPRB, 0.10144E+01_JPRB, 0.99585E+00_JPRB, 0.98794E+00_JPRB ,&
&  0.97098E+00_JPRB, 0.95125E+00_JPRB, 0.93948E+00_JPRB, 0.10382E+01_JPRB, 0.11175E+01_JPRB, 0.12963E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.13802E+01_JPRB, 0.12820E+01_JPRB, 0.12239E+01_JPRB, 0.11970E+01_JPRB, 0.11597E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.10912E+01_JPRB, 0.10405E+01_JPRB, 0.10164E+01_JPRB, 0.10172E+01_JPRB, 0.10092E+01_JPRB, 0.97792E+00_JPRB ,&
&  0.94659E+00_JPRB, 0.94940E+00_JPRB, 0.96198E+00_JPRB, 0.95971E+00_JPRB, 0.94601E+00_JPRB, 0.91972E+00_JPRB, 0.91802E+00_JPRB ,&
&  0.94495E+00_JPRB, 0.96821E+00_JPRB, 0.10144E+01_JPRB, 0.11123E+01_JPRB, 0.10929E+01_JPRB, 0.10366E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.10178E+01_JPRB, 0.10189E+01_JPRB, 0.10703E+01_JPRB, 0.11107E+01_JPRB, 0.11480E+01_JPRB, 0.98700E+00_JPRB ,&
&  0.92358E+00_JPRB, 0.87855E+00_JPRB, 0.85749E+00_JPRB, 0.85044E+00_JPRB, 0.85887E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.86532E+00_JPRB, 0.83801E+00_JPRB, 0.81893E+00_JPRB, 0.82461E+00_JPRB, 0.82862E+00_JPRB, 0.81593E+00_JPRB, 0.79575E+00_JPRB ,&
&  0.78488E+00_JPRB, 0.77196E+00_JPRB, 0.78288E+00_JPRB, 0.77082E+00_JPRB, 0.76303E+00_JPRB, 0.75055E+00_JPRB, 0.74287E+00_JPRB ,&
&  0.74473E+00_JPRB, 0.75238E+00_JPRB, 0.73962E+00_JPRB, 0.74415E+00_JPRB, 0.77130E+00_JPRB, 0.79621E+00_JPRB, 0.80533E+00_JPRB ,&
&  0.81571E+00_JPRB, 0.82792E+00_JPRB, 0.83060E+00_JPRB, 0.83175E+00_JPRB, 0.82093E+00_JPRB, 0.82375E+00_JPRB, 0.83675E+00_JPRB ,&
&  0.95435E+00_JPRB, 0.21827E+01_JPRB, 0.16353E+01_JPRB, 0.20377E+01_JPRB, 0.15089E+01_JPRB, 0.12250E+01_JPRB, 0.97132E+00_JPRB ,&
&  0.10018E+01_JPRB, 0.12179E+01_JPRB, 0.12382E+01_JPRB, 0.12434E+01_JPRB, 0.11839E+01_JPRB, 0.11348E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.10616E+01_JPRB, 0.10474E+01_JPRB, 0.10389E+01_JPRB, 0.10160E+01_JPRB, 0.10173E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10066E+01_JPRB, 0.10096E+01_JPRB, 0.10447E+01_JPRB, 0.11094E+01_JPRB, 0.13189E+01_JPRB, 0.14396E+01_JPRB, 0.13506E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.14220E+01_JPRB, 0.13389E+01_JPRB, 0.12258E+01_JPRB, 0.11705E+01_JPRB, 0.11266E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.10354E+01_JPRB, 0.10090E+01_JPRB, 0.99635E+00_JPRB, 0.10076E+01_JPRB, 0.10041E+01_JPRB, 0.96095E+00_JPRB ,&
&  0.93323E+00_JPRB, 0.94219E+00_JPRB, 0.95363E+00_JPRB, 0.96016E+00_JPRB, 0.94862E+00_JPRB, 0.93234E+00_JPRB, 0.94206E+00_JPRB ,&
&  0.96932E+00_JPRB, 0.10022E+01_JPRB, 0.10257E+01_JPRB, 0.10969E+01_JPRB, 0.10812E+01_JPRB, 0.10427E+01_JPRB, 0.99839E+00_JPRB ,&
&  0.10177E+01_JPRB, 0.10358E+01_JPRB, 0.10291E+01_JPRB, 0.10298E+01_JPRB, 0.10397E+01_JPRB, 0.10152E+01_JPRB, 0.91307E+00_JPRB ,&
&  0.89820E+00_JPRB, 0.89417E+00_JPRB, 0.89231E+00_JPRB, 0.87120E+00_JPRB, 0.88595E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.83647E+00_JPRB, 0.79437E+00_JPRB, 0.78840E+00_JPRB, 0.80384E+00_JPRB, 0.79709E+00_JPRB, 0.77881E+00_JPRB, 0.74830E+00_JPRB ,&
&  0.71845E+00_JPRB, 0.73385E+00_JPRB, 0.76117E+00_JPRB, 0.77900E+00_JPRB, 0.78931E+00_JPRB, 0.79806E+00_JPRB, 0.78086E+00_JPRB ,&
&  0.75509E+00_JPRB, 0.75842E+00_JPRB, 0.77382E+00_JPRB, 0.80254E+00_JPRB, 0.82357E+00_JPRB, 0.84395E+00_JPRB, 0.87438E+00_JPRB ,&
&  0.88288E+00_JPRB, 0.87854E+00_JPRB, 0.86623E+00_JPRB, 0.85546E+00_JPRB, 0.85353E+00_JPRB, 0.87317E+00_JPRB, 0.91485E+00_JPRB ,&
&  0.11386E+01_JPRB, 0.27705E+01_JPRB, 0.27288E+01_JPRB, 0.19962E+01_JPRB, 0.15158E+01_JPRB, 0.12193E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.10097E+01_JPRB, 0.10102E+01_JPRB, 0.12654E+01_JPRB, 0.12657E+01_JPRB, 0.11895E+01_JPRB, 0.11477E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11239E+01_JPRB, 0.10996E+01_JPRB, 0.10782E+01_JPRB, 0.10607E+01_JPRB, 0.10734E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10959E+01_JPRB, 0.11313E+01_JPRB, 0.12403E+01_JPRB, 0.13683E+01_JPRB, 0.13999E+01_JPRB, 0.14955E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.13946E+01_JPRB, 0.13064E+01_JPRB, 0.12260E+01_JPRB, 0.11089E+01_JPRB, 0.11477E+01_JPRB, 0.11021E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10065E+01_JPRB, 0.99964E+00_JPRB, 0.10088E+01_JPRB, 0.10171E+01_JPRB, 0.10164E+01_JPRB, 0.99200E+00_JPRB ,&
&  0.97398E+00_JPRB, 0.97186E+00_JPRB, 0.97089E+00_JPRB, 0.97178E+00_JPRB, 0.97073E+00_JPRB, 0.97561E+00_JPRB, 0.10081E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10772E+01_JPRB, 0.10961E+01_JPRB, 0.11526E+01_JPRB, 0.11445E+01_JPRB, 0.10804E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.10711E+01_JPRB, 0.10520E+01_JPRB, 0.10326E+01_JPRB, 0.99746E+00_JPRB, 0.94749E+00_JPRB, 0.91836E+00_JPRB ,&
&  0.93150E+00_JPRB, 0.95219E+00_JPRB, 0.92493E+00_JPRB, 0.87125E+00_JPRB, 0.88143E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.80432E+00_JPRB, 0.76404E+00_JPRB, 0.75563E+00_JPRB, 0.74785E+00_JPRB, 0.73202E+00_JPRB, 0.72561E+00_JPRB, 0.69972E+00_JPRB ,&
&  0.71258E+00_JPRB, 0.76991E+00_JPRB, 0.83857E+00_JPRB, 0.86948E+00_JPRB, 0.88747E+00_JPRB, 0.88668E+00_JPRB, 0.86036E+00_JPRB ,&
&  0.82788E+00_JPRB, 0.83490E+00_JPRB, 0.85101E+00_JPRB, 0.86151E+00_JPRB, 0.87957E+00_JPRB, 0.89373E+00_JPRB, 0.91942E+00_JPRB ,&
&  0.94069E+00_JPRB, 0.92071E+00_JPRB, 0.89315E+00_JPRB, 0.88497E+00_JPRB, 0.93414E+00_JPRB, 0.10105E+01_JPRB, 0.11129E+01_JPRB ,&
&  0.16026E+01_JPRB, 0.31546E+01_JPRB, 0.16829E+01_JPRB, 0.15487E+01_JPRB, 0.14281E+01_JPRB, 0.12572E+01_JPRB, 0.11084E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.95658E+00_JPRB, 0.10708E+01_JPRB, 0.12512E+01_JPRB, 0.12245E+01_JPRB, 0.11982E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.12083E+01_JPRB, 0.12054E+01_JPRB, 0.12010E+01_JPRB, 0.11655E+01_JPRB, 0.11638E+01_JPRB, 0.11664E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.12637E+01_JPRB, 0.13760E+01_JPRB, 0.13901E+01_JPRB, 0.15285E+01_JPRB, 0.15720E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11760E+01_JPRB, 0.12358E+01_JPRB, 0.12894E+01_JPRB, 0.12292E+01_JPRB, 0.10586E+01_JPRB, 0.11078E+01_JPRB ,&
&  0.10263E+01_JPRB, 0.10100E+01_JPRB, 0.10195E+01_JPRB, 0.10221E+01_JPRB, 0.10444E+01_JPRB, 0.10447E+01_JPRB, 0.10482E+01_JPRB ,&
&  0.10327E+01_JPRB, 0.10160E+01_JPRB, 0.10073E+01_JPRB, 0.10031E+01_JPRB, 0.10041E+01_JPRB, 0.10256E+01_JPRB, 0.10639E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.11560E+01_JPRB, 0.12220E+01_JPRB, 0.12480E+01_JPRB, 0.12385E+01_JPRB, 0.11724E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.11321E+01_JPRB, 0.10314E+01_JPRB, 0.98238E+00_JPRB, 0.97098E+00_JPRB, 0.94655E+00_JPRB, 0.98542E+00_JPRB ,&
&  0.10676E+01_JPRB, 0.11942E+01_JPRB, 0.12854E+01_JPRB, 0.84366E+00_JPRB, 0.83863E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.72757E+00_JPRB, 0.71987E+00_JPRB, 0.69230E+00_JPRB, 0.67398E+00_JPRB, 0.66847E+00_JPRB, 0.69255E+00_JPRB, 0.73859E+00_JPRB ,&
&  0.81007E+00_JPRB, 0.90175E+00_JPRB, 0.97892E+00_JPRB, 0.10086E+01_JPRB, 0.99814E+00_JPRB, 0.10021E+01_JPRB, 0.95397E+00_JPRB ,&
&  0.90373E+00_JPRB, 0.89628E+00_JPRB, 0.90932E+00_JPRB, 0.91953E+00_JPRB, 0.93378E+00_JPRB, 0.95591E+00_JPRB, 0.98252E+00_JPRB ,&
&  0.99710E+00_JPRB, 0.10080E+01_JPRB, 0.10210E+01_JPRB, 0.10947E+01_JPRB, 0.12175E+01_JPRB, 0.14096E+01_JPRB, 0.17521E+01_JPRB ,&
&  0.24935E+01_JPRB, 0.24676E+01_JPRB, 0.12747E+01_JPRB, 0.13406E+01_JPRB, 0.11509E+01_JPRB, 0.11704E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.11450E+01_JPRB, 0.11033E+01_JPRB, 0.11830E+01_JPRB, 0.13284E+01_JPRB, 0.13212E+01_JPRB, 0.13216E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.13089E+01_JPRB, 0.12859E+01_JPRB, 0.12549E+01_JPRB, 0.12462E+01_JPRB, 0.12642E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.13609E+01_JPRB, 0.14044E+01_JPRB, 0.15218E+01_JPRB, 0.14545E+01_JPRB, 0.16907E+01_JPRB, 0.16958E+01_JPRB, 0.14395E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.10800E+01_JPRB, 0.11977E+01_JPRB, 0.13201E+01_JPRB, 0.12707E+01_JPRB, 0.10258E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.10001E+01_JPRB, 0.10024E+01_JPRB, 0.10242E+01_JPRB, 0.10634E+01_JPRB, 0.10710E+01_JPRB, 0.10787E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.10333E+01_JPRB, 0.10268E+01_JPRB, 0.10281E+01_JPRB, 0.10405E+01_JPRB, 0.10794E+01_JPRB, 0.11040E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.11781E+01_JPRB, 0.12353E+01_JPRB, 0.12959E+01_JPRB, 0.12871E+01_JPRB, 0.12198E+01_JPRB, 0.11333E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.11002E+01_JPRB, 0.10054E+01_JPRB, 0.95087E+00_JPRB, 0.89222E+00_JPRB, 0.89389E+00_JPRB, 0.87749E+00_JPRB ,&
&  0.87608E+00_JPRB, 0.87597E+00_JPRB, 0.83500E+00_JPRB, 0.72864E+00_JPRB, 0.72957E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.69345E+00_JPRB, 0.69687E+00_JPRB, 0.68937E+00_JPRB, 0.70617E+00_JPRB, 0.70826E+00_JPRB, 0.77251E+00_JPRB, 0.84282E+00_JPRB ,&
&  0.95023E+00_JPRB, 0.10296E+01_JPRB, 0.10788E+01_JPRB, 0.10893E+01_JPRB, 0.10698E+01_JPRB, 0.10132E+01_JPRB, 0.96018E+00_JPRB ,&
&  0.95931E+00_JPRB, 0.96367E+00_JPRB, 0.97848E+00_JPRB, 0.10014E+01_JPRB, 0.10241E+01_JPRB, 0.10601E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.12157E+01_JPRB, 0.13058E+01_JPRB, 0.14380E+01_JPRB, 0.16069E+01_JPRB, 0.18373E+01_JPRB, 0.20629E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.80477E+00_JPRB, 0.11357E+01_JPRB, 0.11197E+01_JPRB, 0.10302E+01_JPRB, 0.11704E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.12479E+01_JPRB, 0.11919E+01_JPRB, 0.11875E+01_JPRB, 0.13959E+01_JPRB, 0.14005E+01_JPRB, 0.14042E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.13961E+01_JPRB, 0.13748E+01_JPRB, 0.13479E+01_JPRB, 0.13478E+01_JPRB, 0.13525E+01_JPRB, 0.13884E+01_JPRB, 0.14336E+01_JPRB ,&
&  0.14992E+01_JPRB, 0.15817E+01_JPRB, 0.17169E+01_JPRB, 0.14917E+01_JPRB, 0.17236E+01_JPRB, 0.18980E+01_JPRB, 0.18181E+01_JPRB ,&
&  0.17453E+01_JPRB, 0.10732E+01_JPRB, 0.10160E+01_JPRB, 0.12949E+01_JPRB, 0.12661E+01_JPRB, 0.10112E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.99077E+00_JPRB, 0.10166E+01_JPRB, 0.10005E+01_JPRB, 0.10614E+01_JPRB, 0.10921E+01_JPRB, 0.11112E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10564E+01_JPRB, 0.10243E+01_JPRB, 0.10143E+01_JPRB, 0.10249E+01_JPRB, 0.10365E+01_JPRB, 0.10704E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.11190E+01_JPRB, 0.11749E+01_JPRB, 0.12300E+01_JPRB, 0.12556E+01_JPRB, 0.11919E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.10908E+01_JPRB, 0.10443E+01_JPRB, 0.94669E+00_JPRB, 0.84095E+00_JPRB, 0.81344E+00_JPRB, 0.75847E+00_JPRB, 0.70094E+00_JPRB ,&
&  0.71116E+00_JPRB, 0.70881E+00_JPRB, 0.69351E+00_JPRB, 0.69172E+00_JPRB, 0.69615E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.77868E+00_JPRB, 0.77049E+00_JPRB, 0.76293E+00_JPRB, 0.76630E+00_JPRB, 0.77531E+00_JPRB, 0.82308E+00_JPRB, 0.90049E+00_JPRB ,&
&  0.97461E+00_JPRB, 0.10200E+01_JPRB, 0.10613E+01_JPRB, 0.10809E+01_JPRB, 0.10836E+01_JPRB, 0.10417E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.10291E+01_JPRB, 0.10284E+01_JPRB, 0.10514E+01_JPRB, 0.10877E+01_JPRB, 0.11198E+01_JPRB, 0.11557E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.12463E+01_JPRB, 0.13214E+01_JPRB, 0.13994E+01_JPRB, 0.15201E+01_JPRB, 0.16153E+01_JPRB, 0.16758E+01_JPRB, 0.16468E+01_JPRB ,&
&  0.75280E+00_JPRB, 0.85866E+00_JPRB, 0.12485E+01_JPRB, 0.12122E+01_JPRB, 0.12719E+01_JPRB, 0.13534E+01_JPRB, 0.14510E+01_JPRB ,&
&  0.13603E+01_JPRB, 0.12004E+01_JPRB, 0.12251E+01_JPRB, 0.14638E+01_JPRB, 0.14555E+01_JPRB, 0.14893E+01_JPRB, 0.15033E+01_JPRB ,&
&  0.14864E+01_JPRB, 0.14622E+01_JPRB, 0.14469E+01_JPRB, 0.14605E+01_JPRB, 0.14951E+01_JPRB, 0.15376E+01_JPRB, 0.16026E+01_JPRB ,&
&  0.16550E+01_JPRB, 0.17349E+01_JPRB, 0.18340E+01_JPRB, 0.15420E+01_JPRB, 0.16797E+01_JPRB, 0.19278E+01_JPRB, 0.23303E+01_JPRB ,&
&  0.18926E+01_JPRB, 0.10558E+01_JPRB, 0.97763E+00_JPRB, 0.12418E+01_JPRB, 0.11846E+01_JPRB, 0.11907E+01_JPRB, 0.10072E+01_JPRB ,&
&  0.99328E+00_JPRB, 0.94360E+00_JPRB, 0.96464E+00_JPRB, 0.10219E+01_JPRB, 0.10691E+01_JPRB, 0.10980E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.98036E+00_JPRB, 0.96413E+00_JPRB, 0.97127E+00_JPRB, 0.99235E+00_JPRB, 0.10064E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.99594E+00_JPRB, 0.10004E+01_JPRB, 0.10425E+01_JPRB, 0.10698E+01_JPRB, 0.10993E+01_JPRB, 0.11309E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11194E+01_JPRB, 0.11046E+01_JPRB, 0.11982E+01_JPRB, 0.11106E+01_JPRB, 0.99809E+00_JPRB, 0.85806E+00_JPRB, 0.72490E+00_JPRB ,&
&  0.74365E+00_JPRB, 0.75333E+00_JPRB, 0.75023E+00_JPRB, 0.75765E+00_JPRB, 0.77530E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.86884E+00_JPRB, 0.85205E+00_JPRB, 0.84032E+00_JPRB, 0.82952E+00_JPRB, 0.84153E+00_JPRB, 0.87574E+00_JPRB, 0.91498E+00_JPRB ,&
&  0.95407E+00_JPRB, 0.99901E+00_JPRB, 0.10601E+01_JPRB, 0.10809E+01_JPRB, 0.11194E+01_JPRB, 0.11171E+01_JPRB, 0.11042E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.10939E+01_JPRB, 0.11182E+01_JPRB, 0.11450E+01_JPRB, 0.11547E+01_JPRB, 0.11826E+01_JPRB, 0.12197E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.13221E+01_JPRB, 0.13880E+01_JPRB, 0.14998E+01_JPRB, 0.15399E+01_JPRB, 0.15523E+01_JPRB, 0.15845E+01_JPRB ,&
&  0.84057E+00_JPRB, 0.12651E+01_JPRB, 0.13305E+01_JPRB, 0.12964E+01_JPRB, 0.13590E+01_JPRB, 0.13477E+01_JPRB, 0.13254E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.13144E+01_JPRB, 0.13126E+01_JPRB, 0.14138E+01_JPRB, 0.15147E+01_JPRB, 0.15563E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.15689E+01_JPRB, 0.15548E+01_JPRB, 0.15753E+01_JPRB, 0.16290E+01_JPRB, 0.16738E+01_JPRB, 0.17218E+01_JPRB, 0.17895E+01_JPRB ,&
&  0.18091E+01_JPRB, 0.18230E+01_JPRB, 0.19020E+01_JPRB, 0.16909E+01_JPRB, 0.16048E+01_JPRB, 0.17134E+01_JPRB, 0.16082E+01_JPRB ,&
&  0.12265E+01_JPRB, 0.77388E+00_JPRB, 0.86681E+00_JPRB, 0.11064E+01_JPRB, 0.99919E+00_JPRB, 0.98145E+00_JPRB, 0.91946E+00_JPRB ,&
&  0.82390E+00_JPRB, 0.79040E+00_JPRB, 0.86350E+00_JPRB, 0.93507E+00_JPRB, 0.99644E+00_JPRB, 0.10432E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.97115E+00_JPRB, 0.92682E+00_JPRB, 0.89214E+00_JPRB, 0.88770E+00_JPRB, 0.91210E+00_JPRB, 0.92672E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.91316E+00_JPRB, 0.90263E+00_JPRB, 0.94414E+00_JPRB, 0.97694E+00_JPRB, 0.10208E+01_JPRB, 0.10705E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.14542E+01_JPRB, 0.16732E+01_JPRB, 0.19381E+01_JPRB, 0.18954E+01_JPRB, 0.17242E+01_JPRB, 0.12386E+01_JPRB, 0.83412E+00_JPRB ,&
&  0.85361E+00_JPRB, 0.87430E+00_JPRB, 0.87728E+00_JPRB, 0.88676E+00_JPRB, 0.87782E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10057E+01_JPRB, 0.99029E+00_JPRB, 0.98848E+00_JPRB, 0.98620E+00_JPRB, 0.98463E+00_JPRB, 0.99357E+00_JPRB, 0.10170E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10641E+01_JPRB, 0.11189E+01_JPRB, 0.11542E+01_JPRB, 0.11998E+01_JPRB, 0.12028E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.11635E+01_JPRB, 0.11974E+01_JPRB, 0.12284E+01_JPRB, 0.12407E+01_JPRB, 0.12044E+01_JPRB, 0.11845E+01_JPRB, 0.11988E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.12702E+01_JPRB, 0.13044E+01_JPRB, 0.13582E+01_JPRB, 0.13683E+01_JPRB, 0.13021E+01_JPRB, 0.89060E+00_JPRB ,&
&  0.12286E+01_JPRB, 0.13933E+01_JPRB, 0.13993E+01_JPRB, 0.13332E+01_JPRB, 0.13044E+01_JPRB, 0.12362E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.13724E+01_JPRB, 0.13692E+01_JPRB, 0.13293E+01_JPRB, 0.14836E+01_JPRB, 0.15018E+01_JPRB, 0.15270E+01_JPRB ,&
&  0.15386E+01_JPRB, 0.15519E+01_JPRB, 0.15756E+01_JPRB, 0.16071E+01_JPRB, 0.16328E+01_JPRB, 0.16683E+01_JPRB, 0.16908E+01_JPRB ,&
&  0.16762E+01_JPRB, 0.16475E+01_JPRB, 0.16640E+01_JPRB, 0.15472E+01_JPRB, 0.15002E+01_JPRB, 0.15236E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.80663E+00_JPRB, 0.60316E+00_JPRB, 0.82828E+00_JPRB, 0.10701E+01_JPRB, 0.95318E+00_JPRB, 0.89031E+00_JPRB, 0.87891E+00_JPRB ,&
&  0.81974E+00_JPRB, 0.74973E+00_JPRB, 0.73272E+00_JPRB, 0.78635E+00_JPRB, 0.86193E+00_JPRB, 0.92073E+00_JPRB, 0.93516E+00_JPRB ,&
&  0.90082E+00_JPRB, 0.87392E+00_JPRB, 0.87051E+00_JPRB, 0.88871E+00_JPRB, 0.93014E+00_JPRB, 0.10210E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.84834E+00_JPRB, 0.96803E+00_JPRB, 0.10333E+01_JPRB, 0.11521E+01_JPRB, 0.13172E+01_JPRB, 0.16180E+01_JPRB ,&
&  0.20911E+01_JPRB, 0.25963E+01_JPRB, 0.25523E+01_JPRB, 0.21356E+01_JPRB, 0.26991E+01_JPRB, 0.18063E+01_JPRB, 0.96098E+00_JPRB ,&
&  0.99001E+00_JPRB, 0.10088E+01_JPRB, 0.10266E+01_JPRB, 0.10213E+01_JPRB, 0.10162E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10361E+01_JPRB, 0.10386E+01_JPRB, 0.10313E+01_JPRB, 0.10617E+01_JPRB, 0.10596E+01_JPRB, 0.10751E+01_JPRB, 0.10769E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11203E+01_JPRB, 0.11441E+01_JPRB, 0.11835E+01_JPRB, 0.11952E+01_JPRB, 0.12126E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12742E+01_JPRB, 0.13127E+01_JPRB, 0.13123E+01_JPRB, 0.12987E+01_JPRB, 0.12654E+01_JPRB, 0.12431E+01_JPRB, 0.12151E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11904E+01_JPRB, 0.11994E+01_JPRB, 0.12180E+01_JPRB, 0.12351E+01_JPRB, 0.12899E+01_JPRB, 0.87431E+00_JPRB ,&
&  0.13052E+01_JPRB, 0.14286E+01_JPRB, 0.13401E+01_JPRB, 0.12699E+01_JPRB, 0.11810E+01_JPRB, 0.11890E+01_JPRB, 0.12205E+01_JPRB ,&
&  0.12832E+01_JPRB, 0.13495E+01_JPRB, 0.13629E+01_JPRB, 0.13505E+01_JPRB, 0.13209E+01_JPRB, 0.13346E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.13291E+01_JPRB, 0.13208E+01_JPRB, 0.13076E+01_JPRB, 0.13009E+01_JPRB, 0.13118E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.12277E+01_JPRB, 0.12112E+01_JPRB, 0.12504E+01_JPRB, 0.12237E+01_JPRB, 0.10617E+01_JPRB, 0.83090E+00_JPRB ,&
&  0.55233E+00_JPRB, 0.53680E+00_JPRB, 0.77028E+00_JPRB, 0.11001E+01_JPRB, 0.10135E+01_JPRB, 0.91298E+00_JPRB, 0.82846E+00_JPRB ,&
&  0.78991E+00_JPRB, 0.72267E+00_JPRB, 0.65887E+00_JPRB, 0.65633E+00_JPRB, 0.67848E+00_JPRB, 0.72851E+00_JPRB, 0.72746E+00_JPRB ,&
&  0.74046E+00_JPRB, 0.73639E+00_JPRB, 0.71419E+00_JPRB, 0.70299E+00_JPRB, 0.74314E+00_JPRB, 0.74798E+00_JPRB, 0.66348E+00_JPRB ,&
&  0.79303E+00_JPRB, 0.72984E+00_JPRB, 0.77668E+00_JPRB, 0.86622E+00_JPRB, 0.93283E+00_JPRB, 0.11234E+01_JPRB, 0.14209E+01_JPRB ,&
&  0.18711E+01_JPRB, 0.18415E+01_JPRB, 0.24523E+01_JPRB, 0.37489E+01_JPRB, 0.29411E+01_JPRB, 0.20890E+01_JPRB, 0.11201E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.10746E+01_JPRB, 0.10812E+01_JPRB, 0.10609E+01_JPRB, 0.10450E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.95516E+00_JPRB, 0.96650E+00_JPRB, 0.98526E+00_JPRB, 0.10118E+01_JPRB, 0.10261E+01_JPRB, 0.10491E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.11017E+01_JPRB, 0.11307E+01_JPRB, 0.11625E+01_JPRB, 0.11908E+01_JPRB, 0.12184E+01_JPRB, 0.12446E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.12880E+01_JPRB, 0.12969E+01_JPRB, 0.12924E+01_JPRB, 0.12738E+01_JPRB, 0.12631E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.12269E+01_JPRB, 0.12383E+01_JPRB, 0.12457E+01_JPRB, 0.12589E+01_JPRB, 0.13325E+01_JPRB, 0.93400E+00_JPRB ,&
&  0.13480E+01_JPRB, 0.13962E+01_JPRB, 0.13264E+01_JPRB, 0.11727E+01_JPRB, 0.10939E+01_JPRB, 0.10853E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.12149E+01_JPRB, 0.12683E+01_JPRB, 0.11904E+01_JPRB, 0.10938E+01_JPRB, 0.10975E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.10900E+01_JPRB, 0.10783E+01_JPRB, 0.10649E+01_JPRB, 0.10421E+01_JPRB, 0.10224E+01_JPRB, 0.10003E+01_JPRB, 0.96925E+00_JPRB ,&
&  0.92364E+00_JPRB, 0.87180E+00_JPRB, 0.78594E+00_JPRB, 0.79319E+00_JPRB, 0.78126E+00_JPRB, 0.68242E+00_JPRB, 0.53514E+00_JPRB ,&
&  0.46414E+00_JPRB, 0.61699E+00_JPRB, 0.80696E+00_JPRB, 0.11471E+01_JPRB, 0.10671E+01_JPRB, 0.98005E+00_JPRB, 0.89224E+00_JPRB ,&
&  0.75481E+00_JPRB, 0.67951E+00_JPRB, 0.63049E+00_JPRB, 0.62373E+00_JPRB, 0.62268E+00_JPRB, 0.63065E+00_JPRB, 0.61777E+00_JPRB ,&
&  0.59229E+00_JPRB, 0.56044E+00_JPRB, 0.55004E+00_JPRB, 0.54155E+00_JPRB, 0.55032E+00_JPRB, 0.48962E+00_JPRB, 0.63147E+00_JPRB ,&
&  0.61933E+00_JPRB, 0.54821E+00_JPRB, 0.71994E+00_JPRB, 0.50549E+00_JPRB, 0.65738E+00_JPRB, 0.74836E+00_JPRB, 0.87026E+00_JPRB ,&
&  0.90925E+00_JPRB, 0.12055E+01_JPRB, 0.15106E+01_JPRB, 0.13368E+01_JPRB, 0.11913E+01_JPRB, 0.10723E+01_JPRB, 0.10245E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.99603E+00_JPRB, 0.98832E+00_JPRB, 0.97567E+00_JPRB, 0.95976E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.90762E+00_JPRB, 0.91559E+00_JPRB, 0.92798E+00_JPRB, 0.94481E+00_JPRB, 0.96167E+00_JPRB, 0.97723E+00_JPRB, 0.98955E+00_JPRB ,&
&  0.99888E+00_JPRB, 0.10142E+01_JPRB, 0.10378E+01_JPRB, 0.10660E+01_JPRB, 0.10781E+01_JPRB, 0.10956E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11502E+01_JPRB, 0.11620E+01_JPRB, 0.11752E+01_JPRB, 0.11840E+01_JPRB, 0.11983E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.12738E+01_JPRB, 0.13329E+01_JPRB, 0.13812E+01_JPRB, 0.14474E+01_JPRB, 0.15684E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.14383E+01_JPRB, 0.14276E+01_JPRB, 0.12835E+01_JPRB, 0.10970E+01_JPRB, 0.10325E+01_JPRB, 0.98297E+00_JPRB, 0.94980E+00_JPRB ,&
&  0.93527E+00_JPRB, 0.89404E+00_JPRB, 0.90408E+00_JPRB, 0.89929E+00_JPRB, 0.89194E+00_JPRB, 0.92349E+00_JPRB, 0.92023E+00_JPRB ,&
&  0.92408E+00_JPRB, 0.93029E+00_JPRB, 0.93922E+00_JPRB, 0.93820E+00_JPRB, 0.94667E+00_JPRB, 0.93490E+00_JPRB, 0.90035E+00_JPRB ,&
&  0.85155E+00_JPRB, 0.80408E+00_JPRB, 0.74275E+00_JPRB, 0.72674E+00_JPRB, 0.71707E+00_JPRB, 0.65485E+00_JPRB, 0.59156E+00_JPRB ,&
&  0.63409E+00_JPRB, 0.69974E+00_JPRB, 0.78532E+00_JPRB, 0.11171E+01_JPRB, 0.11100E+01_JPRB, 0.10655E+01_JPRB, 0.89995E+00_JPRB ,&
&  0.80743E+00_JPRB, 0.69213E+00_JPRB, 0.65136E+00_JPRB, 0.66132E+00_JPRB, 0.65597E+00_JPRB, 0.64623E+00_JPRB, 0.61240E+00_JPRB ,&
&  0.55116E+00_JPRB, 0.50550E+00_JPRB, 0.49576E+00_JPRB, 0.48825E+00_JPRB, 0.49832E+00_JPRB, 0.47700E+00_JPRB, 0.59210E+00_JPRB ,&
&  0.54599E+00_JPRB, 0.38478E+00_JPRB, 0.46285E+00_JPRB, 0.56464E+00_JPRB, 0.56910E+00_JPRB, 0.62188E+00_JPRB, 0.75322E+00_JPRB ,&
&  0.96404E+00_JPRB, 0.10294E+01_JPRB, 0.10483E+01_JPRB, 0.98768E+00_JPRB, 0.97248E+00_JPRB, 0.96780E+00_JPRB, 0.94801E+00_JPRB ,&
&  0.93833E+00_JPRB, 0.93256E+00_JPRB, 0.92718E+00_JPRB, 0.91528E+00_JPRB, 0.90702E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.87319E+00_JPRB, 0.87540E+00_JPRB, 0.88105E+00_JPRB, 0.89053E+00_JPRB, 0.90032E+00_JPRB, 0.91069E+00_JPRB, 0.91583E+00_JPRB ,&
&  0.91182E+00_JPRB, 0.91486E+00_JPRB, 0.92478E+00_JPRB, 0.94022E+00_JPRB, 0.95471E+00_JPRB, 0.96724E+00_JPRB, 0.98110E+00_JPRB ,&
&  0.97986E+00_JPRB, 0.98168E+00_JPRB, 0.99898E+00_JPRB, 0.10190E+01_JPRB, 0.10300E+01_JPRB, 0.10300E+01_JPRB, 0.10332E+01_JPRB ,&
&  0.10297E+01_JPRB, 0.10416E+01_JPRB, 0.10706E+01_JPRB, 0.10910E+01_JPRB, 0.11194E+01_JPRB, 0.11875E+01_JPRB, 0.13880E+01_JPRB ,&
&  0.13342E+01_JPRB, 0.14111E+01_JPRB, 0.11958E+01_JPRB, 0.10425E+01_JPRB, 0.10201E+01_JPRB, 0.97013E+00_JPRB, 0.93056E+00_JPRB ,&
&  0.89015E+00_JPRB, 0.85866E+00_JPRB, 0.81389E+00_JPRB, 0.81094E+00_JPRB, 0.79520E+00_JPRB, 0.81710E+00_JPRB, 0.81922E+00_JPRB ,&
&  0.84824E+00_JPRB, 0.86673E+00_JPRB, 0.89949E+00_JPRB, 0.92503E+00_JPRB, 0.97425E+00_JPRB, 0.98015E+00_JPRB, 0.92383E+00_JPRB ,&
&  0.91142E+00_JPRB, 0.95684E+00_JPRB, 0.96561E+00_JPRB, 0.10077E+01_JPRB, 0.97954E+00_JPRB, 0.92786E+00_JPRB, 0.89447E+00_JPRB ,&
&  0.85715E+00_JPRB, 0.85596E+00_JPRB, 0.10105E+01_JPRB, 0.10504E+01_JPRB, 0.11195E+01_JPRB, 0.11154E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.91281E+00_JPRB, 0.80738E+00_JPRB, 0.80500E+00_JPRB, 0.81041E+00_JPRB, 0.84356E+00_JPRB, 0.83363E+00_JPRB, 0.81385E+00_JPRB ,&
&  0.73767E+00_JPRB, 0.65740E+00_JPRB, 0.65001E+00_JPRB, 0.62142E+00_JPRB, 0.56274E+00_JPRB, 0.58726E+00_JPRB, 0.65264E+00_JPRB ,&
&  0.58254E+00_JPRB, 0.48424E+00_JPRB, 0.43236E+00_JPRB, 0.61575E+00_JPRB, 0.62315E+00_JPRB, 0.68071E+00_JPRB, 0.75932E+00_JPRB ,&
&  0.87182E+00_JPRB, 0.96930E+00_JPRB, 0.10298E+01_JPRB, 0.98294E+00_JPRB, 0.90562E+00_JPRB, 0.88808E+00_JPRB, 0.87594E+00_JPRB ,&
&  0.87762E+00_JPRB, 0.87741E+00_JPRB, 0.87574E+00_JPRB, 0.87235E+00_JPRB, 0.86580E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.81499E+00_JPRB, 0.81654E+00_JPRB, 0.83183E+00_JPRB, 0.83390E+00_JPRB, 0.83354E+00_JPRB, 0.84124E+00_JPRB, 0.84436E+00_JPRB ,&
&  0.83939E+00_JPRB, 0.84707E+00_JPRB, 0.85741E+00_JPRB, 0.86912E+00_JPRB, 0.88317E+00_JPRB, 0.89573E+00_JPRB, 0.90253E+00_JPRB ,&
&  0.89329E+00_JPRB, 0.88318E+00_JPRB, 0.90485E+00_JPRB, 0.92291E+00_JPRB, 0.92405E+00_JPRB, 0.92451E+00_JPRB, 0.95366E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.10546E+01_JPRB, 0.10632E+01_JPRB, 0.10505E+01_JPRB, 0.10904E+01_JPRB, 0.13010E+01_JPRB, 0.20051E+01_JPRB ,&
&  0.19123E+01_JPRB, 0.13134E+01_JPRB, 0.11523E+01_JPRB, 0.10282E+01_JPRB, 0.10268E+01_JPRB, 0.10509E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.97820E+00_JPRB, 0.97560E+00_JPRB, 0.97309E+00_JPRB, 0.97092E+00_JPRB, 0.98564E+00_JPRB, 0.99979E+00_JPRB, 0.10016E+01_JPRB ,&
&  0.97456E+00_JPRB, 0.96653E+00_JPRB, 0.98069E+00_JPRB, 0.99782E+00_JPRB, 0.10654E+01_JPRB, 0.12116E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.12134E+01_JPRB, 0.12210E+01_JPRB, 0.12342E+01_JPRB, 0.12233E+01_JPRB, 0.11662E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.10709E+01_JPRB, 0.99500E+00_JPRB, 0.87929E+00_JPRB, 0.10154E+01_JPRB, 0.11112E+01_JPRB, 0.12006E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.10963E+01_JPRB, 0.10009E+01_JPRB, 0.98962E+00_JPRB, 0.10567E+01_JPRB, 0.11350E+01_JPRB, 0.11375E+01_JPRB, 0.11011E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.94095E+00_JPRB, 0.91244E+00_JPRB, 0.82900E+00_JPRB, 0.78419E+00_JPRB, 0.67752E+00_JPRB, 0.73671E+00_JPRB ,&
&  0.70217E+00_JPRB, 0.71011E+00_JPRB, 0.57151E+00_JPRB, 0.75566E+00_JPRB, 0.71611E+00_JPRB, 0.71986E+00_JPRB, 0.74010E+00_JPRB ,&
&  0.76339E+00_JPRB, 0.79275E+00_JPRB, 0.79776E+00_JPRB, 0.87540E+00_JPRB, 0.86064E+00_JPRB, 0.84800E+00_JPRB, 0.84953E+00_JPRB ,&
&  0.81696E+00_JPRB, 0.80764E+00_JPRB, 0.80526E+00_JPRB, 0.81292E+00_JPRB, 0.81436E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.74710E+00_JPRB, 0.74324E+00_JPRB, 0.74234E+00_JPRB, 0.75739E+00_JPRB, 0.77052E+00_JPRB, 0.76167E+00_JPRB, 0.77045E+00_JPRB ,&
&  0.75140E+00_JPRB, 0.73604E+00_JPRB, 0.74734E+00_JPRB, 0.75365E+00_JPRB, 0.77020E+00_JPRB, 0.78494E+00_JPRB, 0.79468E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.79108E+00_JPRB, 0.76265E+00_JPRB, 0.77726E+00_JPRB, 0.87066E+00_JPRB, 0.10266E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.17740E+01_JPRB, 0.18685E+01_JPRB, 0.18715E+01_JPRB, 0.17867E+01_JPRB, 0.16647E+01_JPRB, 0.21596E+01_JPRB ,&
&  0.20325E+01_JPRB, 0.13595E+01_JPRB, 0.12993E+01_JPRB, 0.11754E+01_JPRB, 0.11703E+01_JPRB, 0.11466E+01_JPRB, 0.11348E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.11105E+01_JPRB, 0.11312E+01_JPRB, 0.11765E+01_JPRB, 0.12243E+01_JPRB, 0.12763E+01_JPRB, 0.13086E+01_JPRB ,&
&  0.13146E+01_JPRB, 0.13193E+01_JPRB, 0.13629E+01_JPRB, 0.13275E+01_JPRB, 0.13072E+01_JPRB, 0.13549E+01_JPRB, 0.13993E+01_JPRB ,&
&  0.14368E+01_JPRB, 0.14743E+01_JPRB, 0.13151E+01_JPRB, 0.12963E+01_JPRB, 0.13362E+01_JPRB, 0.12207E+01_JPRB, 0.12032E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.10866E+01_JPRB, 0.83338E+00_JPRB, 0.91036E+00_JPRB, 0.10200E+01_JPRB, 0.11458E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.11822E+01_JPRB, 0.11882E+01_JPRB, 0.12652E+01_JPRB, 0.13199E+01_JPRB, 0.13287E+01_JPRB, 0.13286E+01_JPRB ,&
&  0.13317E+01_JPRB, 0.11967E+01_JPRB, 0.11262E+01_JPRB, 0.10262E+01_JPRB, 0.95411E+00_JPRB, 0.91402E+00_JPRB, 0.83839E+00_JPRB ,&
&  0.78218E+00_JPRB, 0.77447E+00_JPRB, 0.80703E+00_JPRB, 0.77286E+00_JPRB, 0.67395E+00_JPRB, 0.76982E+00_JPRB, 0.78290E+00_JPRB ,&
&  0.79170E+00_JPRB, 0.79640E+00_JPRB, 0.80988E+00_JPRB, 0.80034E+00_JPRB, 0.78064E+00_JPRB, 0.76145E+00_JPRB, 0.74369E+00_JPRB ,&
&  0.75532E+00_JPRB, 0.76949E+00_JPRB, 0.77464E+00_JPRB, 0.76115E+00_JPRB, 0.74976E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.73605E+00_JPRB, 0.73774E+00_JPRB, 0.73399E+00_JPRB, 0.74680E+00_JPRB, 0.74170E+00_JPRB, 0.75663E+00_JPRB, 0.74438E+00_JPRB ,&
&  0.73182E+00_JPRB, 0.73183E+00_JPRB, 0.73946E+00_JPRB, 0.74762E+00_JPRB, 0.77393E+00_JPRB, 0.82649E+00_JPRB, 0.89287E+00_JPRB ,&
&  0.96186E+00_JPRB, 0.98737E+00_JPRB, 0.10424E+01_JPRB, 0.11665E+01_JPRB, 0.14132E+01_JPRB, 0.16561E+01_JPRB, 0.17222E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.19580E+01_JPRB, 0.20582E+01_JPRB, 0.21247E+01_JPRB, 0.20959E+01_JPRB, 0.20676E+01_JPRB, 0.21083E+01_JPRB ,&
&  0.18248E+01_JPRB, 0.14296E+01_JPRB, 0.12891E+01_JPRB, 0.12259E+01_JPRB, 0.12151E+01_JPRB, 0.11944E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.11646E+01_JPRB, 0.11735E+01_JPRB, 0.11798E+01_JPRB, 0.12056E+01_JPRB, 0.12540E+01_JPRB, 0.13154E+01_JPRB, 0.13393E+01_JPRB ,&
&  0.13329E+01_JPRB, 0.13312E+01_JPRB, 0.13411E+01_JPRB, 0.12529E+01_JPRB, 0.13208E+01_JPRB, 0.13513E+01_JPRB, 0.13582E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.12934E+01_JPRB, 0.13138E+01_JPRB, 0.13013E+01_JPRB, 0.12816E+01_JPRB, 0.12143E+01_JPRB, 0.11812E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.11023E+01_JPRB, 0.88540E+00_JPRB, 0.98925E+00_JPRB, 0.11080E+01_JPRB, 0.10783E+01_JPRB, 0.13951E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.13218E+01_JPRB, 0.13345E+01_JPRB, 0.13545E+01_JPRB, 0.14346E+01_JPRB, 0.14562E+01_JPRB, 0.14848E+01_JPRB ,&
&  0.16043E+01_JPRB, 0.14868E+01_JPRB, 0.13862E+01_JPRB, 0.12885E+01_JPRB, 0.11768E+01_JPRB, 0.10746E+01_JPRB, 0.98311E+00_JPRB ,&
&  0.88605E+00_JPRB, 0.84545E+00_JPRB, 0.79556E+00_JPRB, 0.76501E+00_JPRB, 0.71352E+00_JPRB, 0.77819E+00_JPRB, 0.78783E+00_JPRB ,&
&  0.81884E+00_JPRB, 0.81405E+00_JPRB, 0.79968E+00_JPRB, 0.77759E+00_JPRB, 0.76479E+00_JPRB, 0.75572E+00_JPRB, 0.75317E+00_JPRB ,&
&  0.76021E+00_JPRB, 0.76872E+00_JPRB, 0.76156E+00_JPRB, 0.75334E+00_JPRB, 0.74301E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.72399E+00_JPRB, 0.71742E+00_JPRB, 0.71294E+00_JPRB, 0.74206E+00_JPRB, 0.73810E+00_JPRB, 0.72773E+00_JPRB, 0.73838E+00_JPRB ,&
&  0.75327E+00_JPRB, 0.77420E+00_JPRB, 0.80080E+00_JPRB, 0.82774E+00_JPRB, 0.86390E+00_JPRB, 0.91837E+00_JPRB, 0.97801E+00_JPRB ,&
&  0.10475E+01_JPRB, 0.11305E+01_JPRB, 0.12701E+01_JPRB, 0.14514E+01_JPRB, 0.16712E+01_JPRB, 0.17910E+01_JPRB, 0.18010E+01_JPRB ,&
&  0.16883E+01_JPRB, 0.17211E+01_JPRB, 0.17700E+01_JPRB, 0.17512E+01_JPRB, 0.14903E+01_JPRB, 0.15489E+01_JPRB, 0.16753E+01_JPRB ,&
&  0.16837E+01_JPRB, 0.14346E+01_JPRB, 0.12313E+01_JPRB, 0.11743E+01_JPRB, 0.11383E+01_JPRB, 0.11476E+01_JPRB, 0.11529E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.11507E+01_JPRB, 0.11559E+01_JPRB, 0.11675E+01_JPRB, 0.11987E+01_JPRB, 0.12548E+01_JPRB, 0.12737E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12976E+01_JPRB, 0.12830E+01_JPRB, 0.12382E+01_JPRB, 0.12478E+01_JPRB, 0.12528E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.12314E+01_JPRB, 0.12471E+01_JPRB, 0.12606E+01_JPRB, 0.12060E+01_JPRB, 0.11098E+01_JPRB, 0.11066E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.12132E+01_JPRB, 0.95899E+00_JPRB, 0.12520E+01_JPRB, 0.14548E+01_JPRB, 0.13191E+01_JPRB, 0.13598E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.13529E+01_JPRB, 0.13864E+01_JPRB, 0.14226E+01_JPRB, 0.14542E+01_JPRB, 0.14914E+01_JPRB, 0.14307E+01_JPRB ,&
&  0.16622E+01_JPRB, 0.15863E+01_JPRB, 0.15027E+01_JPRB, 0.14456E+01_JPRB, 0.13661E+01_JPRB, 0.12330E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.94934E+00_JPRB, 0.10826E+01_JPRB, 0.10550E+01_JPRB, 0.11362E+01_JPRB, 0.11635E+01_JPRB, 0.79678E+00_JPRB, 0.83701E+00_JPRB ,&
&  0.83894E+00_JPRB, 0.83185E+00_JPRB, 0.81262E+00_JPRB, 0.78748E+00_JPRB, 0.78548E+00_JPRB, 0.78748E+00_JPRB, 0.78780E+00_JPRB ,&
&  0.78264E+00_JPRB, 0.76964E+00_JPRB, 0.76177E+00_JPRB, 0.74301E+00_JPRB, 0.73310E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.75135E+00_JPRB, 0.74375E+00_JPRB, 0.73621E+00_JPRB, 0.74732E+00_JPRB, 0.75638E+00_JPRB, 0.75546E+00_JPRB, 0.74748E+00_JPRB ,&
&  0.75612E+00_JPRB, 0.78501E+00_JPRB, 0.81915E+00_JPRB, 0.83764E+00_JPRB, 0.84360E+00_JPRB, 0.85374E+00_JPRB, 0.89996E+00_JPRB ,&
&  0.97830E+00_JPRB, 0.10954E+01_JPRB, 0.12441E+01_JPRB, 0.13649E+01_JPRB, 0.14761E+01_JPRB, 0.15625E+01_JPRB, 0.16919E+01_JPRB ,&
&  0.17038E+01_JPRB, 0.17418E+01_JPRB, 0.16759E+01_JPRB, 0.13093E+01_JPRB, 0.11886E+01_JPRB, 0.13956E+01_JPRB, 0.13222E+01_JPRB ,&
&  0.13107E+01_JPRB, 0.12386E+01_JPRB, 0.11750E+01_JPRB, 0.11225E+01_JPRB, 0.10824E+01_JPRB, 0.10976E+01_JPRB, 0.11006E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10868E+01_JPRB, 0.10972E+01_JPRB, 0.11109E+01_JPRB, 0.11274E+01_JPRB, 0.11473E+01_JPRB, 0.11468E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.11798E+01_JPRB, 0.11991E+01_JPRB, 0.12067E+01_JPRB, 0.11883E+01_JPRB, 0.12064E+01_JPRB, 0.12087E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.12088E+01_JPRB, 0.11979E+01_JPRB, 0.12209E+01_JPRB, 0.12178E+01_JPRB, 0.10791E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11921E+01_JPRB, 0.11241E+01_JPRB, 0.13528E+01_JPRB, 0.91772E+00_JPRB, 0.10882E+01_JPRB, 0.13372E+01_JPRB ,&
&  0.13657E+01_JPRB, 0.13792E+01_JPRB, 0.13891E+01_JPRB, 0.14108E+01_JPRB, 0.14355E+01_JPRB, 0.13812E+01_JPRB, 0.14860E+01_JPRB ,&
&  0.15970E+01_JPRB, 0.15833E+01_JPRB, 0.15701E+01_JPRB, 0.15697E+01_JPRB, 0.15341E+01_JPRB, 0.14444E+01_JPRB, 0.13439E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11693E+01_JPRB, 0.10465E+01_JPRB, 0.98378E+00_JPRB, 0.90598E+00_JPRB, 0.78601E+00_JPRB, 0.81658E+00_JPRB ,&
&  0.84057E+00_JPRB, 0.82364E+00_JPRB, 0.79810E+00_JPRB, 0.79891E+00_JPRB, 0.79855E+00_JPRB, 0.80147E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.78956E+00_JPRB, 0.78912E+00_JPRB, 0.78601E+00_JPRB, 0.77483E+00_JPRB, 0.76262E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.75687E+00_JPRB, 0.75318E+00_JPRB, 0.75372E+00_JPRB, 0.76331E+00_JPRB, 0.78582E+00_JPRB, 0.80354E+00_JPRB, 0.76164E+00_JPRB ,&
&  0.74823E+00_JPRB, 0.74516E+00_JPRB, 0.77534E+00_JPRB, 0.78742E+00_JPRB, 0.79521E+00_JPRB, 0.80324E+00_JPRB, 0.84548E+00_JPRB ,&
&  0.92487E+00_JPRB, 0.10413E+01_JPRB, 0.11598E+01_JPRB, 0.12407E+01_JPRB, 0.13271E+01_JPRB, 0.14446E+01_JPRB, 0.15744E+01_JPRB ,&
&  0.14159E+01_JPRB, 0.12182E+01_JPRB, 0.15980E+01_JPRB, 0.16604E+01_JPRB, 0.14971E+01_JPRB, 0.13983E+01_JPRB, 0.14025E+01_JPRB ,&
&  0.12594E+01_JPRB, 0.12325E+01_JPRB, 0.12341E+01_JPRB, 0.11586E+01_JPRB, 0.10383E+01_JPRB, 0.10324E+01_JPRB, 0.10362E+01_JPRB ,&
&  0.10327E+01_JPRB, 0.10222E+01_JPRB, 0.10308E+01_JPRB, 0.10474E+01_JPRB, 0.10638E+01_JPRB, 0.10551E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10483E+01_JPRB, 0.10645E+01_JPRB, 0.10651E+01_JPRB, 0.10827E+01_JPRB, 0.11135E+01_JPRB, 0.11339E+01_JPRB ,&
&  0.11513E+01_JPRB, 0.11443E+01_JPRB, 0.11808E+01_JPRB, 0.12417E+01_JPRB, 0.12902E+01_JPRB, 0.11683E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.11852E+01_JPRB, 0.11808E+01_JPRB, 0.11693E+01_JPRB, 0.13228E+01_JPRB, 0.98070E+00_JPRB, 0.10955E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.13961E+01_JPRB, 0.14066E+01_JPRB, 0.14305E+01_JPRB, 0.14879E+01_JPRB, 0.15398E+01_JPRB, 0.14443E+01_JPRB, 0.14948E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.16603E+01_JPRB, 0.16898E+01_JPRB, 0.17195E+01_JPRB, 0.16108E+01_JPRB, 0.16300E+01_JPRB, 0.16649E+01_JPRB ,&
&  0.16639E+01_JPRB, 0.13988E+01_JPRB, 0.12180E+01_JPRB, 0.10558E+01_JPRB, 0.90514E+00_JPRB, 0.83232E+00_JPRB, 0.82037E+00_JPRB ,&
&  0.82357E+00_JPRB, 0.83727E+00_JPRB, 0.79379E+00_JPRB, 0.77001E+00_JPRB, 0.77126E+00_JPRB, 0.79393E+00_JPRB, 0.78396E+00_JPRB ,&
&  0.78546E+00_JPRB, 0.77966E+00_JPRB, 0.78296E+00_JPRB, 0.77837E+00_JPRB, 0.77108E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.77237E+00_JPRB, 0.78420E+00_JPRB, 0.81158E+00_JPRB, 0.84316E+00_JPRB, 0.87364E+00_JPRB, 0.92364E+00_JPRB, 0.88831E+00_JPRB ,&
&  0.85425E+00_JPRB, 0.69042E+00_JPRB, 0.70685E+00_JPRB, 0.71833E+00_JPRB, 0.73213E+00_JPRB, 0.72960E+00_JPRB, 0.74936E+00_JPRB ,&
&  0.82773E+00_JPRB, 0.94752E+00_JPRB, 0.10579E+01_JPRB, 0.11544E+01_JPRB, 0.12548E+01_JPRB, 0.13871E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.12219E+01_JPRB, 0.17871E+01_JPRB, 0.17759E+01_JPRB, 0.16633E+01_JPRB, 0.15487E+01_JPRB, 0.14132E+01_JPRB ,&
&  0.12866E+01_JPRB, 0.12000E+01_JPRB, 0.12428E+01_JPRB, 0.11318E+01_JPRB, 0.99896E+00_JPRB, 0.98234E+00_JPRB, 0.98673E+00_JPRB ,&
&  0.97777E+00_JPRB, 0.96986E+00_JPRB, 0.96574E+00_JPRB, 0.97482E+00_JPRB, 0.98354E+00_JPRB, 0.97626E+00_JPRB, 0.96454E+00_JPRB ,&
&  0.96465E+00_JPRB, 0.97379E+00_JPRB, 0.97769E+00_JPRB, 0.97238E+00_JPRB, 0.10051E+01_JPRB, 0.10425E+01_JPRB, 0.10670E+01_JPRB ,&
&  0.10838E+01_JPRB, 0.11247E+01_JPRB, 0.11599E+01_JPRB, 0.11751E+01_JPRB, 0.11351E+01_JPRB, 0.12489E+01_JPRB, 0.12218E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.12145E+01_JPRB, 0.12876E+01_JPRB, 0.10571E+01_JPRB, 0.11164E+01_JPRB, 0.11883E+01_JPRB, 0.12760E+01_JPRB ,&
&  0.13065E+01_JPRB, 0.14114E+01_JPRB, 0.14587E+01_JPRB, 0.15405E+01_JPRB, 0.16357E+01_JPRB, 0.15711E+01_JPRB, 0.16291E+01_JPRB ,&
&  0.15889E+01_JPRB, 0.18095E+01_JPRB, 0.18229E+01_JPRB, 0.18046E+01_JPRB, 0.16085E+01_JPRB, 0.16112E+01_JPRB, 0.17698E+01_JPRB ,&
&  0.18517E+01_JPRB, 0.16747E+01_JPRB, 0.15257E+01_JPRB, 0.13338E+01_JPRB, 0.11295E+01_JPRB, 0.10058E+01_JPRB, 0.87795E+00_JPRB ,&
&  0.82588E+00_JPRB, 0.76672E+00_JPRB, 0.74715E+00_JPRB, 0.75509E+00_JPRB, 0.75267E+00_JPRB, 0.77553E+00_JPRB, 0.77524E+00_JPRB ,&
&  0.76407E+00_JPRB, 0.76918E+00_JPRB, 0.77296E+00_JPRB, 0.77211E+00_JPRB, 0.77014E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.74343E+00_JPRB, 0.75218E+00_JPRB, 0.77483E+00_JPRB, 0.78486E+00_JPRB, 0.77244E+00_JPRB, 0.77705E+00_JPRB, 0.80359E+00_JPRB ,&
&  0.80198E+00_JPRB, 0.66164E+00_JPRB, 0.64263E+00_JPRB, 0.64857E+00_JPRB, 0.66747E+00_JPRB, 0.67700E+00_JPRB, 0.68299E+00_JPRB ,&
&  0.71648E+00_JPRB, 0.81466E+00_JPRB, 0.96852E+00_JPRB, 0.10468E+01_JPRB, 0.11173E+01_JPRB, 0.12504E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.96671E+00_JPRB, 0.16035E+01_JPRB, 0.15633E+01_JPRB, 0.17717E+01_JPRB, 0.17562E+01_JPRB, 0.16981E+01_JPRB, 0.14198E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11001E+01_JPRB, 0.10941E+01_JPRB, 0.10467E+01_JPRB, 0.10023E+01_JPRB, 0.95213E+00_JPRB, 0.94470E+00_JPRB ,&
&  0.93084E+00_JPRB, 0.91248E+00_JPRB, 0.89923E+00_JPRB, 0.90562E+00_JPRB, 0.90813E+00_JPRB, 0.90550E+00_JPRB, 0.90685E+00_JPRB ,&
&  0.92038E+00_JPRB, 0.93198E+00_JPRB, 0.93006E+00_JPRB, 0.94958E+00_JPRB, 0.97246E+00_JPRB, 0.10150E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10133E+01_JPRB, 0.11004E+01_JPRB, 0.11517E+01_JPRB, 0.11800E+01_JPRB, 0.12673E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.12142E+01_JPRB, 0.12946E+01_JPRB, 0.10461E+01_JPRB, 0.10958E+01_JPRB, 0.12076E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.12201E+01_JPRB, 0.13667E+01_JPRB, 0.14432E+01_JPRB, 0.15730E+01_JPRB, 0.16824E+01_JPRB, 0.18015E+01_JPRB, 0.17129E+01_JPRB ,&
&  0.17304E+01_JPRB, 0.18402E+01_JPRB, 0.19641E+01_JPRB, 0.18068E+01_JPRB, 0.18041E+01_JPRB, 0.17234E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.20845E+01_JPRB, 0.18951E+01_JPRB, 0.17801E+01_JPRB, 0.16385E+01_JPRB, 0.13930E+01_JPRB, 0.12851E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.96599E+00_JPRB, 0.86596E+00_JPRB, 0.77323E+00_JPRB, 0.76040E+00_JPRB, 0.76524E+00_JPRB, 0.76264E+00_JPRB, 0.77561E+00_JPRB ,&
&  0.77383E+00_JPRB, 0.76739E+00_JPRB, 0.76404E+00_JPRB, 0.75565E+00_JPRB, 0.74263E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.72812E+00_JPRB, 0.72906E+00_JPRB, 0.74009E+00_JPRB, 0.74384E+00_JPRB, 0.73665E+00_JPRB, 0.71513E+00_JPRB, 0.73198E+00_JPRB ,&
&  0.75638E+00_JPRB, 0.65640E+00_JPRB, 0.60746E+00_JPRB, 0.58997E+00_JPRB, 0.59970E+00_JPRB, 0.62653E+00_JPRB, 0.64497E+00_JPRB ,&
&  0.66275E+00_JPRB, 0.70410E+00_JPRB, 0.84937E+00_JPRB, 0.95857E+00_JPRB, 0.10507E+01_JPRB, 0.12345E+01_JPRB, 0.90871E+00_JPRB ,&
&  0.98677E+00_JPRB, 0.13821E+01_JPRB, 0.15107E+01_JPRB, 0.16877E+01_JPRB, 0.18011E+01_JPRB, 0.17623E+01_JPRB, 0.15030E+01_JPRB ,&
&  0.12462E+01_JPRB, 0.10964E+01_JPRB, 0.10165E+01_JPRB, 0.96697E+00_JPRB, 0.93797E+00_JPRB, 0.91891E+00_JPRB, 0.88926E+00_JPRB ,&
&  0.87838E+00_JPRB, 0.86503E+00_JPRB, 0.84477E+00_JPRB, 0.84729E+00_JPRB, 0.84756E+00_JPRB, 0.83942E+00_JPRB, 0.85708E+00_JPRB ,&
&  0.88613E+00_JPRB, 0.90931E+00_JPRB, 0.93017E+00_JPRB, 0.91366E+00_JPRB, 0.95536E+00_JPRB, 0.99143E+00_JPRB, 0.10315E+01_JPRB ,&
&  0.99524E+00_JPRB, 0.96521E+00_JPRB, 0.11114E+01_JPRB, 0.12561E+01_JPRB, 0.13259E+01_JPRB, 0.12895E+01_JPRB, 0.12569E+01_JPRB ,&
&  0.12953E+01_JPRB, 0.12806E+01_JPRB, 0.12144E+01_JPRB, 0.10587E+01_JPRB, 0.11136E+01_JPRB, 0.12546E+01_JPRB, 0.13568E+01_JPRB ,&
&  0.13743E+01_JPRB, 0.14054E+01_JPRB, 0.14485E+01_JPRB, 0.14918E+01_JPRB, 0.15914E+01_JPRB, 0.18162E+01_JPRB, 0.18490E+01_JPRB ,&
&  0.20884E+01_JPRB, 0.24091E+01_JPRB, 0.22273E+01_JPRB, 0.19384E+01_JPRB, 0.18937E+01_JPRB, 0.18643E+01_JPRB, 0.19637E+01_JPRB ,&
&  0.21971E+01_JPRB, 0.20135E+01_JPRB, 0.17308E+01_JPRB, 0.16802E+01_JPRB, 0.15417E+01_JPRB, 0.15592E+01_JPRB, 0.14966E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.11083E+01_JPRB, 0.95792E+00_JPRB, 0.87307E+00_JPRB, 0.82874E+00_JPRB, 0.81957E+00_JPRB, 0.81344E+00_JPRB ,&
&  0.81214E+00_JPRB, 0.80290E+00_JPRB, 0.78639E+00_JPRB, 0.76901E+00_JPRB, 0.74500E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.75157E+00_JPRB, 0.72924E+00_JPRB, 0.71784E+00_JPRB, 0.71105E+00_JPRB, 0.71502E+00_JPRB, 0.69986E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.71946E+00_JPRB, 0.63823E+00_JPRB, 0.60113E+00_JPRB, 0.57520E+00_JPRB, 0.57935E+00_JPRB, 0.60287E+00_JPRB, 0.62718E+00_JPRB ,&
&  0.64691E+00_JPRB, 0.67348E+00_JPRB, 0.75754E+00_JPRB, 0.88523E+00_JPRB, 0.92357E+00_JPRB, 0.86141E+00_JPRB, 0.89212E+00_JPRB ,&
&  0.11083E+01_JPRB, 0.13605E+01_JPRB, 0.14950E+01_JPRB, 0.16461E+01_JPRB, 0.18602E+01_JPRB, 0.19733E+01_JPRB, 0.17070E+01_JPRB ,&
&  0.14662E+01_JPRB, 0.12361E+01_JPRB, 0.10600E+01_JPRB, 0.96086E+00_JPRB, 0.92065E+00_JPRB, 0.88397E+00_JPRB, 0.85448E+00_JPRB ,&
&  0.82436E+00_JPRB, 0.80497E+00_JPRB, 0.79958E+00_JPRB, 0.79921E+00_JPRB, 0.80669E+00_JPRB, 0.79496E+00_JPRB, 0.80047E+00_JPRB ,&
&  0.82837E+00_JPRB, 0.85901E+00_JPRB, 0.89739E+00_JPRB, 0.89017E+00_JPRB, 0.92576E+00_JPRB, 0.96607E+00_JPRB, 0.10380E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.11334E+01_JPRB, 0.12077E+01_JPRB, 0.12593E+01_JPRB, 0.13453E+01_JPRB, 0.13824E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.13327E+01_JPRB, 0.12611E+01_JPRB, 0.11202E+01_JPRB, 0.11344E+01_JPRB, 0.12217E+01_JPRB, 0.14090E+01_JPRB, 0.12632E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.10473E+01_JPRB, 0.11424E+01_JPRB, 0.13315E+01_JPRB, 0.14625E+01_JPRB, 0.16863E+01_JPRB, 0.20249E+01_JPRB ,&
&  0.21753E+01_JPRB, 0.13213E+01_JPRB, 0.11127E+01_JPRB, 0.13937E+01_JPRB, 0.12088E+01_JPRB, 0.12135E+01_JPRB, 0.17717E+01_JPRB ,&
&  0.20794E+01_JPRB, 0.22463E+01_JPRB, 0.21013E+01_JPRB, 0.16431E+01_JPRB, 0.15628E+01_JPRB, 0.16682E+01_JPRB, 0.17792E+01_JPRB ,&
&  0.15286E+01_JPRB, 0.13939E+01_JPRB, 0.12615E+01_JPRB, 0.11247E+01_JPRB, 0.10098E+01_JPRB, 0.93184E+00_JPRB, 0.88623E+00_JPRB ,&
&  0.86525E+00_JPRB, 0.85222E+00_JPRB, 0.83781E+00_JPRB, 0.81227E+00_JPRB, 0.78136E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.78846E+00_JPRB, 0.75685E+00_JPRB, 0.73268E+00_JPRB, 0.71041E+00_JPRB, 0.69985E+00_JPRB, 0.68401E+00_JPRB, 0.65856E+00_JPRB ,&
&  0.68572E+00_JPRB, 0.64129E+00_JPRB, 0.60028E+00_JPRB, 0.57609E+00_JPRB, 0.57363E+00_JPRB, 0.58773E+00_JPRB, 0.61740E+00_JPRB ,&
&  0.65831E+00_JPRB, 0.68147E+00_JPRB, 0.75381E+00_JPRB, 0.84812E+00_JPRB, 0.85039E+00_JPRB, 0.76398E+00_JPRB, 0.86413E+00_JPRB ,&
&  0.95064E+00_JPRB, 0.12016E+01_JPRB, 0.16046E+01_JPRB, 0.18248E+01_JPRB, 0.18830E+01_JPRB, 0.19850E+01_JPRB, 0.18365E+01_JPRB ,&
&  0.16980E+01_JPRB, 0.14780E+01_JPRB, 0.12233E+01_JPRB, 0.10551E+01_JPRB, 0.94292E+00_JPRB, 0.89642E+00_JPRB, 0.84748E+00_JPRB ,&
&  0.82809E+00_JPRB, 0.79546E+00_JPRB, 0.76630E+00_JPRB, 0.76077E+00_JPRB, 0.76444E+00_JPRB, 0.76446E+00_JPRB, 0.75723E+00_JPRB ,&
&  0.76920E+00_JPRB, 0.79615E+00_JPRB, 0.84856E+00_JPRB, 0.89215E+00_JPRB, 0.78262E+00_JPRB, 0.92871E+00_JPRB, 0.10113E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.11932E+01_JPRB, 0.11858E+01_JPRB, 0.13611E+01_JPRB, 0.14324E+01_JPRB, 0.13823E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.14483E+01_JPRB, 0.13298E+01_JPRB, 0.12096E+01_JPRB, 0.13020E+01_JPRB, 0.13532E+01_JPRB, 0.15840E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.94771E+00_JPRB, 0.11510E+01_JPRB, 0.11330E+01_JPRB, 0.10982E+01_JPRB, 0.12650E+01_JPRB, 0.15152E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.66821E+00_JPRB, 0.56922E+00_JPRB, 0.61954E+00_JPRB, 0.62625E+00_JPRB, 0.60442E+00_JPRB, 0.85418E+00_JPRB, 0.20492E+01_JPRB ,&
&  0.19855E+01_JPRB, 0.23044E+01_JPRB, 0.22528E+01_JPRB, 0.19438E+01_JPRB, 0.16592E+01_JPRB, 0.16032E+01_JPRB, 0.19116E+01_JPRB ,&
&  0.17593E+01_JPRB, 0.15798E+01_JPRB, 0.14452E+01_JPRB, 0.13406E+01_JPRB, 0.12128E+01_JPRB, 0.10926E+01_JPRB, 0.99284E+00_JPRB ,&
&  0.92495E+00_JPRB, 0.88694E+00_JPRB, 0.86529E+00_JPRB, 0.84462E+00_JPRB, 0.81830E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.79481E+00_JPRB, 0.76211E+00_JPRB, 0.73858E+00_JPRB, 0.71958E+00_JPRB, 0.69888E+00_JPRB, 0.68523E+00_JPRB, 0.65451E+00_JPRB ,&
&  0.65449E+00_JPRB, 0.64906E+00_JPRB, 0.61175E+00_JPRB, 0.58653E+00_JPRB, 0.57574E+00_JPRB, 0.58905E+00_JPRB, 0.62887E+00_JPRB ,&
&  0.66124E+00_JPRB, 0.70573E+00_JPRB, 0.82263E+00_JPRB, 0.78443E+00_JPRB, 0.80702E+00_JPRB, 0.72423E+00_JPRB, 0.76610E+00_JPRB ,&
&  0.84603E+00_JPRB, 0.10226E+01_JPRB, 0.13505E+01_JPRB, 0.16040E+01_JPRB, 0.18975E+01_JPRB, 0.20236E+01_JPRB, 0.19998E+01_JPRB ,&
&  0.18750E+01_JPRB, 0.17321E+01_JPRB, 0.15046E+01_JPRB, 0.12603E+01_JPRB, 0.10885E+01_JPRB, 0.97209E+00_JPRB, 0.89622E+00_JPRB ,&
&  0.85941E+00_JPRB, 0.84534E+00_JPRB, 0.78833E+00_JPRB, 0.75403E+00_JPRB, 0.74375E+00_JPRB, 0.73780E+00_JPRB, 0.71829E+00_JPRB ,&
&  0.71969E+00_JPRB, 0.74624E+00_JPRB, 0.79525E+00_JPRB, 0.84423E+00_JPRB, 0.78897E+00_JPRB, 0.74548E+00_JPRB, 0.91521E+00_JPRB ,&
&  0.10957E+01_JPRB, 0.13106E+01_JPRB, 0.13836E+01_JPRB, 0.16624E+01_JPRB, 0.15927E+01_JPRB, 0.14703E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.15636E+01_JPRB, 0.17078E+01_JPRB, 0.13433E+01_JPRB, 0.14113E+01_JPRB, 0.15775E+01_JPRB, 0.12853E+01_JPRB, 0.11874E+01_JPRB ,&
&  0.13353E+01_JPRB, 0.12115E+01_JPRB, 0.11083E+01_JPRB, 0.96909E+00_JPRB, 0.11889E+01_JPRB, 0.11846E+01_JPRB, 0.66817E+00_JPRB ,&
&  0.56253E+00_JPRB, 0.54226E+00_JPRB, 0.52569E+00_JPRB, 0.50363E+00_JPRB, 0.52804E+00_JPRB, 0.73439E+00_JPRB, 0.11996E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.19820E+01_JPRB, 0.21727E+01_JPRB, 0.21637E+01_JPRB, 0.18147E+01_JPRB, 0.17487E+01_JPRB, 0.17394E+01_JPRB ,&
&  0.18742E+01_JPRB, 0.18127E+01_JPRB, 0.16579E+01_JPRB, 0.15042E+01_JPRB, 0.13574E+01_JPRB, 0.12264E+01_JPRB, 0.11202E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.97235E+00_JPRB, 0.92204E+00_JPRB, 0.87967E+00_JPRB, 0.83388E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.83848E+00_JPRB, 0.79672E+00_JPRB, 0.75310E+00_JPRB, 0.72305E+00_JPRB, 0.70546E+00_JPRB, 0.68430E+00_JPRB, 0.65770E+00_JPRB ,&
&  0.63297E+00_JPRB, 0.64647E+00_JPRB, 0.62252E+00_JPRB, 0.60388E+00_JPRB, 0.59077E+00_JPRB, 0.59768E+00_JPRB, 0.63095E+00_JPRB ,&
&  0.66909E+00_JPRB, 0.73865E+00_JPRB, 0.78048E+00_JPRB, 0.70944E+00_JPRB, 0.67818E+00_JPRB, 0.64244E+00_JPRB, 0.63952E+00_JPRB ,&
&  0.75710E+00_JPRB, 0.88185E+00_JPRB, 0.10891E+01_JPRB, 0.14322E+01_JPRB, 0.18497E+01_JPRB, 0.20022E+01_JPRB, 0.21871E+01_JPRB ,&
&  0.20755E+01_JPRB, 0.18900E+01_JPRB, 0.17389E+01_JPRB, 0.15354E+01_JPRB, 0.12910E+01_JPRB, 0.11312E+01_JPRB, 0.10217E+01_JPRB ,&
&  0.94733E+00_JPRB, 0.90365E+00_JPRB, 0.86194E+00_JPRB, 0.79483E+00_JPRB, 0.74498E+00_JPRB, 0.72670E+00_JPRB, 0.70600E+00_JPRB ,&
&  0.69875E+00_JPRB, 0.72501E+00_JPRB, 0.77016E+00_JPRB, 0.83213E+00_JPRB, 0.90662E+00_JPRB, 0.92995E+00_JPRB, 0.94045E+00_JPRB ,&
&  0.10432E+01_JPRB, 0.11947E+01_JPRB, 0.15831E+01_JPRB, 0.17422E+01_JPRB, 0.17463E+01_JPRB, 0.17351E+01_JPRB, 0.16868E+01_JPRB ,&
&  0.17221E+01_JPRB, 0.19923E+01_JPRB, 0.16453E+01_JPRB, 0.17510E+01_JPRB, 0.16052E+01_JPRB, 0.12732E+01_JPRB, 0.14785E+01_JPRB ,&
&  0.15398E+01_JPRB, 0.14093E+01_JPRB, 0.11716E+01_JPRB, 0.91896E+00_JPRB, 0.88807E+00_JPRB, 0.71326E+00_JPRB, 0.52563E+00_JPRB ,&
&  0.49838E+00_JPRB, 0.48281E+00_JPRB, 0.46745E+00_JPRB, 0.48013E+00_JPRB, 0.49293E+00_JPRB, 0.64084E+00_JPRB, 0.91380E+00_JPRB ,&
&  0.13530E+01_JPRB, 0.16793E+01_JPRB, 0.19515E+01_JPRB, 0.20222E+01_JPRB, 0.18319E+01_JPRB, 0.17429E+01_JPRB, 0.19896E+01_JPRB ,&
&  0.18224E+01_JPRB, 0.17580E+01_JPRB, 0.17438E+01_JPRB, 0.16058E+01_JPRB, 0.14553E+01_JPRB, 0.13240E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.10376E+01_JPRB, 0.98556E+00_JPRB, 0.93374E+00_JPRB, 0.88415E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.85958E+00_JPRB, 0.81823E+00_JPRB, 0.77988E+00_JPRB, 0.75322E+00_JPRB, 0.73074E+00_JPRB, 0.70589E+00_JPRB, 0.67534E+00_JPRB ,&
&  0.63653E+00_JPRB, 0.63272E+00_JPRB, 0.62717E+00_JPRB, 0.59970E+00_JPRB, 0.59025E+00_JPRB, 0.59932E+00_JPRB, 0.62721E+00_JPRB ,&
&  0.66925E+00_JPRB, 0.79333E+00_JPRB, 0.67834E+00_JPRB, 0.62070E+00_JPRB, 0.63713E+00_JPRB, 0.63287E+00_JPRB, 0.58806E+00_JPRB ,&
&  0.69854E+00_JPRB, 0.81431E+00_JPRB, 0.98965E+00_JPRB, 0.13886E+01_JPRB, 0.20634E+01_JPRB, 0.22017E+01_JPRB, 0.23356E+01_JPRB ,&
&  0.24140E+01_JPRB, 0.22034E+01_JPRB, 0.20032E+01_JPRB, 0.17797E+01_JPRB, 0.14910E+01_JPRB, 0.12733E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.95909E+00_JPRB, 0.89398E+00_JPRB, 0.84036E+00_JPRB, 0.78241E+00_JPRB, 0.74719E+00_JPRB, 0.72925E+00_JPRB ,&
&  0.71887E+00_JPRB, 0.73001E+00_JPRB, 0.76253E+00_JPRB, 0.82997E+00_JPRB, 0.90766E+00_JPRB, 0.92512E+00_JPRB, 0.12157E+01_JPRB ,&
&  0.14288E+01_JPRB, 0.15952E+01_JPRB, 0.16776E+01_JPRB, 0.20169E+01_JPRB, 0.22012E+01_JPRB, 0.21096E+01_JPRB, 0.22376E+01_JPRB ,&
&  0.18674E+01_JPRB, 0.18051E+01_JPRB, 0.19197E+01_JPRB, 0.19615E+01_JPRB, 0.15439E+01_JPRB, 0.16226E+01_JPRB, 0.23251E+01_JPRB ,&
&  0.16704E+01_JPRB, 0.17625E+01_JPRB, 0.18704E+01_JPRB, 0.18979E+01_JPRB, 0.82031E+00_JPRB, 0.59964E+00_JPRB, 0.88675E+00_JPRB ,&
&  0.91643E+00_JPRB, 0.63075E+00_JPRB, 0.52530E+00_JPRB, 0.62032E+00_JPRB, 0.54391E+00_JPRB, 0.67174E+00_JPRB, 0.97665E+00_JPRB ,&
&  0.10752E+01_JPRB, 0.11583E+01_JPRB, 0.16211E+01_JPRB, 0.18084E+01_JPRB, 0.17978E+01_JPRB, 0.17927E+01_JPRB, 0.19317E+01_JPRB ,&
&  0.19553E+01_JPRB, 0.18860E+01_JPRB, 0.19295E+01_JPRB, 0.17755E+01_JPRB, 0.15851E+01_JPRB, 0.14096E+01_JPRB, 0.12658E+01_JPRB ,&
&  0.11387E+01_JPRB, 0.10508E+01_JPRB, 0.99367E+00_JPRB, 0.93801E+00_JPRB, 0.89470E+00_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.82815E+00_JPRB, 0.79294E+00_JPRB, 0.76173E+00_JPRB, 0.73445E+00_JPRB, 0.71749E+00_JPRB, 0.70151E+00_JPRB, 0.68170E+00_JPRB ,&
&  0.65045E+00_JPRB, 0.62469E+00_JPRB, 0.62753E+00_JPRB, 0.60249E+00_JPRB, 0.58591E+00_JPRB, 0.58834E+00_JPRB, 0.62172E+00_JPRB ,&
&  0.65867E+00_JPRB, 0.68780E+00_JPRB, 0.58331E+00_JPRB, 0.54146E+00_JPRB, 0.61699E+00_JPRB, 0.58504E+00_JPRB, 0.59377E+00_JPRB ,&
&  0.67707E+00_JPRB, 0.77192E+00_JPRB, 0.92887E+00_JPRB, 0.12849E+01_JPRB, 0.16536E+01_JPRB, 0.20436E+01_JPRB, 0.23358E+01_JPRB ,&
&  0.24618E+01_JPRB, 0.24045E+01_JPRB, 0.22293E+01_JPRB, 0.20072E+01_JPRB, 0.16825E+01_JPRB, 0.14257E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.99408E+00_JPRB, 0.91040E+00_JPRB, 0.81647E+00_JPRB, 0.76392E+00_JPRB, 0.72318E+00_JPRB, 0.71291E+00_JPRB ,&
&  0.71848E+00_JPRB, 0.73928E+00_JPRB, 0.76928E+00_JPRB, 0.84033E+00_JPRB, 0.82464E+00_JPRB, 0.86305E+00_JPRB, 0.10681E+01_JPRB ,&
&  0.15175E+01_JPRB, 0.16859E+01_JPRB, 0.18780E+01_JPRB, 0.21451E+01_JPRB, 0.24499E+01_JPRB, 0.23283E+01_JPRB, 0.25589E+01_JPRB ,&
&  0.19937E+01_JPRB, 0.16731E+01_JPRB, 0.14804E+01_JPRB, 0.12225E+01_JPRB, 0.15535E+01_JPRB, 0.23820E+01_JPRB, 0.23182E+01_JPRB ,&
&  0.20687E+01_JPRB, 0.19946E+01_JPRB, 0.19578E+01_JPRB, 0.15734E+01_JPRB, 0.12416E+01_JPRB, 0.96496E+00_JPRB, 0.11556E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11342E+01_JPRB, 0.10009E+01_JPRB, 0.86422E+00_JPRB, 0.77480E+00_JPRB, 0.93999E+00_JPRB, 0.95681E+00_JPRB ,&
&  0.91322E+00_JPRB, 0.91008E+00_JPRB, 0.11373E+01_JPRB, 0.14182E+01_JPRB, 0.15901E+01_JPRB, 0.15258E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.17479E+01_JPRB, 0.18056E+01_JPRB, 0.17580E+01_JPRB, 0.16465E+01_JPRB, 0.15017E+01_JPRB, 0.13495E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.10397E+01_JPRB, 0.96455E+00_JPRB, 0.90606E+00_JPRB, 0.86470E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.78245E+00_JPRB, 0.75618E+00_JPRB, 0.73186E+00_JPRB, 0.71462E+00_JPRB, 0.69938E+00_JPRB, 0.68819E+00_JPRB, 0.67601E+00_JPRB ,&
&  0.65561E+00_JPRB, 0.62494E+00_JPRB, 0.62512E+00_JPRB, 0.61345E+00_JPRB, 0.59523E+00_JPRB, 0.58745E+00_JPRB, 0.60396E+00_JPRB ,&
&  0.63923E+00_JPRB, 0.61101E+00_JPRB, 0.50588E+00_JPRB, 0.51835E+00_JPRB, 0.55492E+00_JPRB, 0.53226E+00_JPRB, 0.57506E+00_JPRB ,&
&  0.64026E+00_JPRB, 0.72892E+00_JPRB, 0.82079E+00_JPRB, 0.10249E+01_JPRB, 0.12737E+01_JPRB, 0.15966E+01_JPRB, 0.19219E+01_JPRB ,&
&  0.20269E+01_JPRB, 0.21774E+01_JPRB, 0.22459E+01_JPRB, 0.20742E+01_JPRB, 0.18435E+01_JPRB, 0.16041E+01_JPRB, 0.14077E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.11390E+01_JPRB, 0.99891E+00_JPRB, 0.88453E+00_JPRB, 0.80976E+00_JPRB, 0.76594E+00_JPRB, 0.74640E+00_JPRB ,&
&  0.74682E+00_JPRB, 0.75227E+00_JPRB, 0.76361E+00_JPRB, 0.83241E+00_JPRB, 0.82518E+00_JPRB, 0.92317E+00_JPRB, 0.11157E+01_JPRB ,&
&  0.17671E+01_JPRB, 0.20000E+01_JPRB, 0.21222E+01_JPRB, 0.25246E+01_JPRB, 0.20198E+01_JPRB, 0.20643E+01_JPRB, 0.24668E+01_JPRB ,&
&  0.26217E+01_JPRB, 0.25504E+01_JPRB, 0.25073E+01_JPRB, 0.19360E+01_JPRB, 0.17293E+01_JPRB, 0.23912E+01_JPRB, 0.22492E+01_JPRB ,&
&  0.19950E+01_JPRB, 0.19114E+01_JPRB, 0.17907E+01_JPRB, 0.19462E+01_JPRB, 0.14081E+01_JPRB, 0.11995E+01_JPRB, 0.92489E+00_JPRB ,&
&  0.90275E+00_JPRB, 0.83020E+00_JPRB, 0.95101E+00_JPRB, 0.90082E+00_JPRB, 0.89796E+00_JPRB, 0.89725E+00_JPRB, 0.86863E+00_JPRB ,&
&  0.88591E+00_JPRB, 0.88278E+00_JPRB, 0.89013E+00_JPRB, 0.10382E+01_JPRB, 0.13029E+01_JPRB, 0.12746E+01_JPRB, 0.13901E+01_JPRB ,&
&  0.14197E+01_JPRB, 0.14874E+01_JPRB, 0.14220E+01_JPRB, 0.14161E+01_JPRB, 0.13600E+01_JPRB, 0.12744E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.10008E+01_JPRB, 0.92806E+00_JPRB, 0.86742E+00_JPRB, 0.81897E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.70910E+00_JPRB, 0.68798E+00_JPRB, 0.67470E+00_JPRB, 0.66939E+00_JPRB, 0.66757E+00_JPRB, 0.66438E+00_JPRB, 0.65766E+00_JPRB ,&
&  0.64263E+00_JPRB, 0.61498E+00_JPRB, 0.60453E+00_JPRB, 0.60174E+00_JPRB, 0.58393E+00_JPRB, 0.57512E+00_JPRB, 0.58657E+00_JPRB ,&
&  0.61922E+00_JPRB, 0.63954E+00_JPRB, 0.53948E+00_JPRB, 0.47976E+00_JPRB, 0.45882E+00_JPRB, 0.50444E+00_JPRB, 0.56194E+00_JPRB ,&
&  0.64736E+00_JPRB, 0.71974E+00_JPRB, 0.81093E+00_JPRB, 0.92270E+00_JPRB, 0.10783E+01_JPRB, 0.12731E+01_JPRB, 0.15861E+01_JPRB ,&
&  0.20751E+01_JPRB, 0.19125E+01_JPRB, 0.20203E+01_JPRB, 0.19532E+01_JPRB, 0.18711E+01_JPRB, 0.16758E+01_JPRB, 0.14853E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.11886E+01_JPRB, 0.10603E+01_JPRB, 0.94850E+00_JPRB, 0.86358E+00_JPRB, 0.79961E+00_JPRB, 0.76149E+00_JPRB ,&
&  0.75896E+00_JPRB, 0.76098E+00_JPRB, 0.75826E+00_JPRB, 0.78697E+00_JPRB, 0.91637E+00_JPRB, 0.10804E+01_JPRB, 0.13391E+01_JPRB ,&
&  0.15445E+01_JPRB, 0.16194E+01_JPRB, 0.22846E+01_JPRB, 0.24463E+01_JPRB, 0.25970E+01_JPRB, 0.26104E+01_JPRB, 0.26564E+01_JPRB ,&
&  0.28405E+01_JPRB, 0.27242E+01_JPRB, 0.26204E+01_JPRB, 0.22666E+01_JPRB, 0.25809E+01_JPRB, 0.25007E+01_JPRB, 0.23071E+01_JPRB ,&
&  0.20726E+01_JPRB, 0.19957E+01_JPRB, 0.18313E+01_JPRB, 0.17898E+01_JPRB, 0.17767E+01_JPRB, 0.18019E+01_JPRB, 0.14324E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.12405E+01_JPRB, 0.10999E+01_JPRB, 0.95594E+00_JPRB, 0.83405E+00_JPRB, 0.87590E+00_JPRB, 0.92828E+00_JPRB ,&
&  0.96436E+00_JPRB, 0.98328E+00_JPRB, 0.96632E+00_JPRB, 0.10248E+01_JPRB, 0.11809E+01_JPRB, 0.12977E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12588E+01_JPRB, 0.12701E+01_JPRB, 0.12292E+01_JPRB, 0.12094E+01_JPRB, 0.11388E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.97620E+00_JPRB, 0.90833E+00_JPRB, 0.85071E+00_JPRB, 0.79464E+00_JPRB, 0.74235E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.64160E+00_JPRB, 0.63004E+00_JPRB, 0.62190E+00_JPRB, 0.62082E+00_JPRB, 0.62239E+00_JPRB, 0.62906E+00_JPRB, 0.62987E+00_JPRB ,&
&  0.62347E+00_JPRB, 0.60259E+00_JPRB, 0.58500E+00_JPRB, 0.58677E+00_JPRB, 0.57718E+00_JPRB, 0.56803E+00_JPRB, 0.57541E+00_JPRB ,&
&  0.60638E+00_JPRB, 0.70768E+00_JPRB, 0.60696E+00_JPRB, 0.56350E+00_JPRB, 0.52368E+00_JPRB, 0.56118E+00_JPRB, 0.60331E+00_JPRB ,&
&  0.69626E+00_JPRB, 0.74816E+00_JPRB, 0.77733E+00_JPRB, 0.83685E+00_JPRB, 0.96883E+00_JPRB, 0.11514E+01_JPRB, 0.13719E+01_JPRB ,&
&  0.14985E+01_JPRB, 0.16804E+01_JPRB, 0.16861E+01_JPRB, 0.18851E+01_JPRB, 0.17742E+01_JPRB, 0.15204E+01_JPRB, 0.14886E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.11757E+01_JPRB, 0.10365E+01_JPRB, 0.93782E+00_JPRB, 0.86622E+00_JPRB, 0.81116E+00_JPRB, 0.76676E+00_JPRB ,&
&  0.73916E+00_JPRB, 0.72327E+00_JPRB, 0.70566E+00_JPRB, 0.69622E+00_JPRB, 0.76677E+00_JPRB, 0.10337E+01_JPRB, 0.13843E+01_JPRB ,&
&  0.20208E+01_JPRB, 0.18911E+01_JPRB, 0.16701E+01_JPRB, 0.22777E+01_JPRB, 0.30395E+01_JPRB, 0.30019E+01_JPRB, 0.28294E+01_JPRB ,&
&  0.30913E+01_JPRB, 0.31189E+01_JPRB, 0.30465E+01_JPRB, 0.30415E+01_JPRB, 0.29486E+01_JPRB, 0.27945E+01_JPRB, 0.25858E+01_JPRB ,&
&  0.22457E+01_JPRB, 0.20704E+01_JPRB, 0.19522E+01_JPRB, 0.19223E+01_JPRB, 0.17835E+01_JPRB, 0.16932E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.14440E+01_JPRB, 0.12649E+01_JPRB, 0.90830E+00_JPRB, 0.95272E+00_JPRB, 0.77795E+00_JPRB, 0.80253E+00_JPRB, 0.95900E+00_JPRB ,&
&  0.97012E+00_JPRB, 0.10580E+01_JPRB, 0.10911E+01_JPRB, 0.10467E+01_JPRB, 0.11956E+01_JPRB, 0.12339E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.10687E+01_JPRB, 0.10936E+01_JPRB, 0.10816E+01_JPRB, 0.10221E+01_JPRB, 0.95093E+00_JPRB, 0.88369E+00_JPRB ,&
&  0.82512E+00_JPRB, 0.77711E+00_JPRB, 0.74194E+00_JPRB, 0.69958E+00_JPRB, 0.66336E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.58414E+00_JPRB, 0.57415E+00_JPRB, 0.56490E+00_JPRB, 0.56574E+00_JPRB, 0.57261E+00_JPRB, 0.58412E+00_JPRB, 0.59536E+00_JPRB ,&
&  0.60036E+00_JPRB, 0.59253E+00_JPRB, 0.57613E+00_JPRB, 0.57122E+00_JPRB, 0.56556E+00_JPRB, 0.55891E+00_JPRB, 0.56716E+00_JPRB ,&
&  0.61285E+00_JPRB, 0.70693E+00_JPRB, 0.64749E+00_JPRB, 0.57736E+00_JPRB, 0.60336E+00_JPRB, 0.61047E+00_JPRB, 0.65620E+00_JPRB ,&
&  0.68211E+00_JPRB, 0.75134E+00_JPRB, 0.77948E+00_JPRB, 0.80600E+00_JPRB, 0.86764E+00_JPRB, 0.10076E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.13070E+01_JPRB, 0.14052E+01_JPRB, 0.15037E+01_JPRB, 0.15054E+01_JPRB, 0.13445E+01_JPRB, 0.13262E+01_JPRB ,&
&  0.12176E+01_JPRB, 0.10881E+01_JPRB, 0.95365E+00_JPRB, 0.85016E+00_JPRB, 0.78223E+00_JPRB, 0.73997E+00_JPRB, 0.71592E+00_JPRB ,&
&  0.70558E+00_JPRB, 0.69445E+00_JPRB, 0.67268E+00_JPRB, 0.67672E+00_JPRB, 0.73348E+00_JPRB, 0.11309E+01_JPRB, 0.18470E+01_JPRB ,&
&  0.24846E+01_JPRB, 0.26311E+01_JPRB, 0.28303E+01_JPRB, 0.32615E+01_JPRB, 0.33863E+01_JPRB, 0.34713E+01_JPRB, 0.35339E+01_JPRB ,&
&  0.34644E+01_JPRB, 0.33366E+01_JPRB, 0.31457E+01_JPRB, 0.29019E+01_JPRB, 0.28190E+01_JPRB, 0.27843E+01_JPRB, 0.26401E+01_JPRB ,&
&  0.23595E+01_JPRB, 0.21777E+01_JPRB, 0.21223E+01_JPRB, 0.19933E+01_JPRB, 0.18963E+01_JPRB, 0.17190E+01_JPRB, 0.17270E+01_JPRB ,&
&  0.17513E+01_JPRB, 0.10570E+01_JPRB, 0.85516E+00_JPRB, 0.10192E+01_JPRB, 0.87160E+00_JPRB, 0.91926E+00_JPRB, 0.11197E+01_JPRB ,&
&  0.11402E+01_JPRB, 0.11254E+01_JPRB, 0.11713E+01_JPRB, 0.11103E+01_JPRB, 0.11168E+01_JPRB, 0.11892E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.10168E+01_JPRB, 0.10241E+01_JPRB, 0.96596E+00_JPRB, 0.88630E+00_JPRB, 0.81534E+00_JPRB, 0.75941E+00_JPRB ,&
&  0.71705E+00_JPRB, 0.68804E+00_JPRB, 0.66746E+00_JPRB, 0.62907E+00_JPRB, 0.59801E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.54996E+00_JPRB, 0.53715E+00_JPRB, 0.53008E+00_JPRB, 0.53205E+00_JPRB, 0.53896E+00_JPRB, 0.55300E+00_JPRB, 0.56920E+00_JPRB ,&
&  0.58257E+00_JPRB, 0.58414E+00_JPRB, 0.57008E+00_JPRB, 0.55855E+00_JPRB, 0.55182E+00_JPRB, 0.54727E+00_JPRB, 0.56136E+00_JPRB ,&
&  0.51775E+00_JPRB, 0.56968E+00_JPRB, 0.52366E+00_JPRB, 0.54560E+00_JPRB, 0.61676E+00_JPRB, 0.62962E+00_JPRB, 0.64782E+00_JPRB ,&
&  0.66093E+00_JPRB, 0.69127E+00_JPRB, 0.70478E+00_JPRB, 0.72475E+00_JPRB, 0.76325E+00_JPRB, 0.85687E+00_JPRB, 0.99335E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.97966E+00_JPRB, 0.10321E+01_JPRB, 0.10931E+01_JPRB, 0.10836E+01_JPRB, 0.11238E+01_JPRB, 0.11445E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.99114E+00_JPRB, 0.90012E+00_JPRB, 0.81704E+00_JPRB, 0.74913E+00_JPRB, 0.69416E+00_JPRB, 0.66006E+00_JPRB ,&
&  0.64028E+00_JPRB, 0.63104E+00_JPRB, 0.62897E+00_JPRB, 0.67729E+00_JPRB, 0.74822E+00_JPRB, 0.13215E+01_JPRB, 0.23778E+01_JPRB ,&
&  0.28193E+01_JPRB, 0.28899E+01_JPRB, 0.31648E+01_JPRB, 0.36895E+01_JPRB, 0.36231E+01_JPRB, 0.35062E+01_JPRB, 0.33791E+01_JPRB ,&
&  0.32637E+01_JPRB, 0.30539E+01_JPRB, 0.29341E+01_JPRB, 0.29265E+01_JPRB, 0.28146E+01_JPRB, 0.28648E+01_JPRB, 0.26867E+01_JPRB ,&
&  0.23945E+01_JPRB, 0.22767E+01_JPRB, 0.22388E+01_JPRB, 0.21926E+01_JPRB, 0.21142E+01_JPRB, 0.21487E+01_JPRB, 0.21027E+01_JPRB ,&
&  0.20027E+01_JPRB, 0.17417E+01_JPRB, 0.13215E+01_JPRB, 0.12443E+01_JPRB, 0.97817E+00_JPRB, 0.10384E+01_JPRB, 0.13104E+01_JPRB ,&
&  0.13235E+01_JPRB, 0.12233E+01_JPRB, 0.11165E+01_JPRB, 0.10637E+01_JPRB, 0.10496E+01_JPRB, 0.11275E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.94973E+00_JPRB, 0.96491E+00_JPRB, 0.94638E+00_JPRB, 0.89752E+00_JPRB, 0.83205E+00_JPRB, 0.76277E+00_JPRB, 0.71376E+00_JPRB ,&
&  0.68962E+00_JPRB, 0.66266E+00_JPRB, 0.62683E+00_JPRB, 0.58691E+00_JPRB, 0.56286E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.53641E+00_JPRB, 0.52083E+00_JPRB, 0.51013E+00_JPRB, 0.51109E+00_JPRB, 0.51813E+00_JPRB, 0.52940E+00_JPRB, 0.54648E+00_JPRB ,&
&  0.56387E+00_JPRB, 0.57151E+00_JPRB, 0.56326E+00_JPRB, 0.54828E+00_JPRB, 0.53683E+00_JPRB, 0.53156E+00_JPRB, 0.50566E+00_JPRB ,&
&  0.46749E+00_JPRB, 0.50746E+00_JPRB, 0.50230E+00_JPRB, 0.51624E+00_JPRB, 0.55573E+00_JPRB, 0.56772E+00_JPRB, 0.58326E+00_JPRB ,&
&  0.61354E+00_JPRB, 0.62434E+00_JPRB, 0.64084E+00_JPRB, 0.66147E+00_JPRB, 0.69033E+00_JPRB, 0.75576E+00_JPRB, 0.83967E+00_JPRB ,&
&  0.82931E+00_JPRB, 0.82613E+00_JPRB, 0.83268E+00_JPRB, 0.86685E+00_JPRB, 0.91428E+00_JPRB, 0.93908E+00_JPRB, 0.91485E+00_JPRB ,&
&  0.87061E+00_JPRB, 0.82284E+00_JPRB, 0.78388E+00_JPRB, 0.74908E+00_JPRB, 0.70668E+00_JPRB, 0.66224E+00_JPRB, 0.62930E+00_JPRB ,&
&  0.60630E+00_JPRB, 0.59808E+00_JPRB, 0.61039E+00_JPRB, 0.64213E+00_JPRB, 0.74242E+00_JPRB, 0.11933E+01_JPRB, 0.18316E+01_JPRB ,&
&  0.21035E+01_JPRB, 0.23328E+01_JPRB, 0.24212E+01_JPRB, 0.25138E+01_JPRB, 0.25920E+01_JPRB, 0.26472E+01_JPRB, 0.27059E+01_JPRB ,&
&  0.27793E+01_JPRB, 0.26202E+01_JPRB, 0.28345E+01_JPRB, 0.28961E+01_JPRB, 0.28714E+01_JPRB, 0.28810E+01_JPRB, 0.26444E+01_JPRB ,&
&  0.24329E+01_JPRB, 0.22462E+01_JPRB, 0.22802E+01_JPRB, 0.22618E+01_JPRB, 0.21966E+01_JPRB, 0.22174E+01_JPRB, 0.21495E+01_JPRB ,&
&  0.21076E+01_JPRB, 0.20478E+01_JPRB, 0.17922E+01_JPRB, 0.15906E+01_JPRB, 0.13404E+01_JPRB, 0.14093E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.11480E+01_JPRB, 0.94059E+00_JPRB, 0.93942E+00_JPRB, 0.93051E+00_JPRB, 0.96197E+00_JPRB, 0.97346E+00_JPRB, 0.90775E+00_JPRB ,&
&  0.89952E+00_JPRB, 0.92963E+00_JPRB, 0.87789E+00_JPRB, 0.82821E+00_JPRB, 0.77561E+00_JPRB, 0.70944E+00_JPRB, 0.65610E+00_JPRB ,&
&  0.63373E+00_JPRB, 0.62544E+00_JPRB, 0.59423E+00_JPRB, 0.56479E+00_JPRB, 0.54879E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.54144E+00_JPRB, 0.53199E+00_JPRB, 0.51724E+00_JPRB, 0.51007E+00_JPRB, 0.51298E+00_JPRB, 0.51907E+00_JPRB, 0.52158E+00_JPRB ,&
&  0.53962E+00_JPRB, 0.54439E+00_JPRB, 0.54180E+00_JPRB, 0.53303E+00_JPRB, 0.51818E+00_JPRB, 0.47331E+00_JPRB, 0.41296E+00_JPRB ,&
&  0.40573E+00_JPRB, 0.42985E+00_JPRB, 0.46035E+00_JPRB, 0.47748E+00_JPRB, 0.49491E+00_JPRB, 0.51090E+00_JPRB, 0.52376E+00_JPRB ,&
&  0.53353E+00_JPRB, 0.54251E+00_JPRB, 0.56535E+00_JPRB, 0.59936E+00_JPRB, 0.63463E+00_JPRB, 0.67162E+00_JPRB, 0.71793E+00_JPRB ,&
&  0.70108E+00_JPRB, 0.70475E+00_JPRB, 0.72719E+00_JPRB, 0.69978E+00_JPRB, 0.77535E+00_JPRB, 0.78446E+00_JPRB, 0.77310E+00_JPRB ,&
&  0.75521E+00_JPRB, 0.72370E+00_JPRB, 0.68605E+00_JPRB, 0.65567E+00_JPRB, 0.62972E+00_JPRB, 0.61020E+00_JPRB, 0.58530E+00_JPRB ,&
&  0.56856E+00_JPRB, 0.56212E+00_JPRB, 0.57111E+00_JPRB, 0.59357E+00_JPRB, 0.65252E+00_JPRB, 0.94163E+00_JPRB, 0.12006E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.15572E+01_JPRB, 0.16108E+01_JPRB, 0.16103E+01_JPRB, 0.18715E+01_JPRB, 0.21331E+01_JPRB, 0.22348E+01_JPRB ,&
&  0.24343E+01_JPRB, 0.23661E+01_JPRB, 0.24439E+01_JPRB, 0.25273E+01_JPRB, 0.26246E+01_JPRB, 0.25760E+01_JPRB, 0.23830E+01_JPRB ,&
&  0.22600E+01_JPRB, 0.22743E+01_JPRB, 0.22381E+01_JPRB, 0.21764E+01_JPRB, 0.20484E+01_JPRB, 0.19814E+01_JPRB, 0.18907E+01_JPRB ,&
&  0.18762E+01_JPRB, 0.18762E+01_JPRB, 0.18100E+01_JPRB, 0.17230E+01_JPRB, 0.16097E+01_JPRB, 0.14623E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.98504E+00_JPRB, 0.90987E+00_JPRB, 0.90253E+00_JPRB, 0.91862E+00_JPRB, 0.87267E+00_JPRB, 0.85573E+00_JPRB ,&
&  0.84192E+00_JPRB, 0.80894E+00_JPRB, 0.83818E+00_JPRB, 0.80014E+00_JPRB, 0.74949E+00_JPRB, 0.69198E+00_JPRB, 0.65196E+00_JPRB ,&
&  0.60429E+00_JPRB, 0.61461E+00_JPRB, 0.59993E+00_JPRB, 0.57492E+00_JPRB, 0.55330E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.54845E+00_JPRB, 0.53766E+00_JPRB, 0.52323E+00_JPRB, 0.50929E+00_JPRB, 0.50707E+00_JPRB, 0.50068E+00_JPRB, 0.49287E+00_JPRB ,&
&  0.52512E+00_JPRB, 0.50004E+00_JPRB, 0.50989E+00_JPRB, 0.45076E+00_JPRB, 0.42603E+00_JPRB, 0.41700E+00_JPRB, 0.40512E+00_JPRB ,&
&  0.40851E+00_JPRB, 0.42277E+00_JPRB, 0.43631E+00_JPRB, 0.44714E+00_JPRB, 0.46545E+00_JPRB, 0.47700E+00_JPRB, 0.48608E+00_JPRB ,&
&  0.49869E+00_JPRB, 0.51724E+00_JPRB, 0.54751E+00_JPRB, 0.57349E+00_JPRB, 0.60716E+00_JPRB, 0.64999E+00_JPRB, 0.69047E+00_JPRB ,&
&  0.68138E+00_JPRB, 0.69053E+00_JPRB, 0.73317E+00_JPRB, 0.74223E+00_JPRB, 0.75425E+00_JPRB, 0.74376E+00_JPRB, 0.72629E+00_JPRB ,&
&  0.69685E+00_JPRB, 0.60960E+00_JPRB, 0.62057E+00_JPRB, 0.59647E+00_JPRB, 0.57603E+00_JPRB, 0.55634E+00_JPRB, 0.53973E+00_JPRB ,&
&  0.52907E+00_JPRB, 0.52579E+00_JPRB, 0.53025E+00_JPRB, 0.53793E+00_JPRB, 0.55733E+00_JPRB, 0.64582E+00_JPRB, 0.76443E+00_JPRB ,&
&  0.98189E+00_JPRB, 0.87983E+00_JPRB, 0.11119E+01_JPRB, 0.12243E+01_JPRB, 0.15028E+01_JPRB, 0.17525E+01_JPRB, 0.19051E+01_JPRB ,&
&  0.22528E+01_JPRB, 0.22776E+01_JPRB, 0.22260E+01_JPRB, 0.21810E+01_JPRB, 0.21962E+01_JPRB, 0.21742E+01_JPRB, 0.21161E+01_JPRB ,&
&  0.20581E+01_JPRB, 0.19258E+01_JPRB, 0.20094E+01_JPRB, 0.19477E+01_JPRB, 0.18340E+01_JPRB, 0.17687E+01_JPRB, 0.16848E+01_JPRB ,&
&  0.16219E+01_JPRB, 0.15633E+01_JPRB, 0.14994E+01_JPRB, 0.13327E+01_JPRB, 0.12817E+01_JPRB, 0.12381E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.10256E+01_JPRB, 0.95790E+00_JPRB, 0.98134E+00_JPRB, 0.93502E+00_JPRB, 0.90889E+00_JPRB, 0.89460E+00_JPRB ,&
&  0.84492E+00_JPRB, 0.76207E+00_JPRB, 0.74941E+00_JPRB, 0.74016E+00_JPRB, 0.71161E+00_JPRB, 0.67949E+00_JPRB, 0.69912E+00_JPRB ,&
&  0.65985E+00_JPRB, 0.64124E+00_JPRB, 0.64422E+00_JPRB, 0.60903E+00_JPRB, 0.57632E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.56748E+00_JPRB, 0.55022E+00_JPRB, 0.53300E+00_JPRB, 0.51286E+00_JPRB, 0.50032E+00_JPRB, 0.48712E+00_JPRB, 0.46633E+00_JPRB ,&
&  0.47046E+00_JPRB, 0.43638E+00_JPRB, 0.41563E+00_JPRB, 0.40722E+00_JPRB, 0.40769E+00_JPRB, 0.40482E+00_JPRB, 0.39814E+00_JPRB ,&
&  0.40070E+00_JPRB, 0.41245E+00_JPRB, 0.42949E+00_JPRB, 0.44278E+00_JPRB, 0.45494E+00_JPRB, 0.46575E+00_JPRB, 0.47681E+00_JPRB ,&
&  0.49281E+00_JPRB, 0.51370E+00_JPRB, 0.53963E+00_JPRB, 0.57265E+00_JPRB, 0.60382E+00_JPRB, 0.64337E+00_JPRB, 0.67362E+00_JPRB ,&
&  0.70445E+00_JPRB, 0.73067E+00_JPRB, 0.73603E+00_JPRB, 0.75387E+00_JPRB, 0.75443E+00_JPRB, 0.73363E+00_JPRB, 0.68498E+00_JPRB ,&
&  0.50615E+00_JPRB, 0.47739E+00_JPRB, 0.53918E+00_JPRB, 0.53419E+00_JPRB, 0.52570E+00_JPRB, 0.51367E+00_JPRB, 0.50729E+00_JPRB ,&
&  0.50480E+00_JPRB, 0.50215E+00_JPRB, 0.50416E+00_JPRB, 0.50557E+00_JPRB, 0.51218E+00_JPRB, 0.53415E+00_JPRB, 0.58501E+00_JPRB ,&
&  0.70750E+00_JPRB, 0.71724E+00_JPRB, 0.76334E+00_JPRB, 0.86450E+00_JPRB, 0.11671E+01_JPRB, 0.13621E+01_JPRB, 0.14900E+01_JPRB ,&
&  0.16606E+01_JPRB, 0.17294E+01_JPRB, 0.17504E+01_JPRB, 0.17889E+01_JPRB, 0.18242E+01_JPRB, 0.18578E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.18556E+01_JPRB, 0.18449E+01_JPRB, 0.18327E+01_JPRB, 0.18235E+01_JPRB, 0.17491E+01_JPRB, 0.16821E+01_JPRB, 0.16051E+01_JPRB ,&
&  0.15269E+01_JPRB, 0.14412E+01_JPRB, 0.13203E+01_JPRB, 0.12060E+01_JPRB, 0.11095E+01_JPRB, 0.10676E+01_JPRB, 0.10417E+01_JPRB ,&
&  0.10145E+01_JPRB, 0.99993E+00_JPRB, 0.98305E+00_JPRB, 0.96214E+00_JPRB, 0.93823E+00_JPRB, 0.90601E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.80692E+00_JPRB, 0.75435E+00_JPRB, 0.68394E+00_JPRB, 0.66240E+00_JPRB, 0.66270E+00_JPRB, 0.66108E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.66055E+00_JPRB, 0.64146E+00_JPRB, 0.62050E+00_JPRB, 0.60587E+00_JPRB, 0.58827E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.58976E+00_JPRB, 0.57006E+00_JPRB, 0.54584E+00_JPRB, 0.51692E+00_JPRB, 0.48987E+00_JPRB, 0.47270E+00_JPRB, 0.46069E+00_JPRB ,&
&  0.46145E+00_JPRB, 0.44131E+00_JPRB, 0.42185E+00_JPRB, 0.40774E+00_JPRB, 0.40064E+00_JPRB, 0.39796E+00_JPRB, 0.40194E+00_JPRB ,&
&  0.41038E+00_JPRB, 0.41968E+00_JPRB, 0.42859E+00_JPRB, 0.43748E+00_JPRB, 0.44685E+00_JPRB, 0.45563E+00_JPRB, 0.46653E+00_JPRB ,&
&  0.48220E+00_JPRB, 0.49990E+00_JPRB, 0.51859E+00_JPRB, 0.54224E+00_JPRB, 0.56655E+00_JPRB, 0.60058E+00_JPRB, 0.62312E+00_JPRB ,&
&  0.63778E+00_JPRB, 0.64529E+00_JPRB, 0.66469E+00_JPRB, 0.71217E+00_JPRB, 0.73374E+00_JPRB, 0.70254E+00_JPRB, 0.55165E+00_JPRB ,&
&  0.43800E+00_JPRB, 0.40688E+00_JPRB, 0.42244E+00_JPRB, 0.47223E+00_JPRB, 0.47489E+00_JPRB, 0.47656E+00_JPRB, 0.47784E+00_JPRB ,&
&  0.46679E+00_JPRB, 0.46024E+00_JPRB, 0.46726E+00_JPRB, 0.48282E+00_JPRB, 0.48592E+00_JPRB, 0.49354E+00_JPRB, 0.52100E+00_JPRB ,&
&  0.57532E+00_JPRB, 0.64385E+00_JPRB, 0.71771E+00_JPRB, 0.70760E+00_JPRB, 0.78865E+00_JPRB, 0.11193E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.12947E+01_JPRB, 0.13480E+01_JPRB, 0.14215E+01_JPRB, 0.14954E+01_JPRB, 0.14886E+01_JPRB, 0.14967E+01_JPRB, 0.15217E+01_JPRB ,&
&  0.14990E+01_JPRB, 0.14955E+01_JPRB, 0.14965E+01_JPRB, 0.15464E+01_JPRB, 0.15379E+01_JPRB, 0.15041E+01_JPRB, 0.14737E+01_JPRB ,&
&  0.14204E+01_JPRB, 0.13093E+01_JPRB, 0.11941E+01_JPRB, 0.10812E+01_JPRB, 0.10230E+01_JPRB, 0.98406E+00_JPRB, 0.96063E+00_JPRB ,&
&  0.93322E+00_JPRB, 0.92271E+00_JPRB, 0.92100E+00_JPRB, 0.92288E+00_JPRB, 0.87254E+00_JPRB, 0.79859E+00_JPRB, 0.74259E+00_JPRB ,&
&  0.70998E+00_JPRB, 0.69094E+00_JPRB, 0.66399E+00_JPRB, 0.65629E+00_JPRB, 0.66080E+00_JPRB, 0.66394E+00_JPRB, 0.64668E+00_JPRB ,&
&  0.63006E+00_JPRB, 0.61498E+00_JPRB, 0.60775E+00_JPRB, 0.60590E+00_JPRB, 0.60322E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.58449E+00_JPRB, 0.57979E+00_JPRB, 0.56265E+00_JPRB, 0.53172E+00_JPRB, 0.49296E+00_JPRB, 0.46050E+00_JPRB, 0.43971E+00_JPRB ,&
&  0.42978E+00_JPRB, 0.42224E+00_JPRB, 0.41818E+00_JPRB, 0.41662E+00_JPRB, 0.41721E+00_JPRB, 0.42038E+00_JPRB, 0.42355E+00_JPRB ,&
&  0.42601E+00_JPRB, 0.42943E+00_JPRB, 0.43436E+00_JPRB, 0.44006E+00_JPRB, 0.44698E+00_JPRB, 0.45610E+00_JPRB, 0.46728E+00_JPRB ,&
&  0.48196E+00_JPRB, 0.49430E+00_JPRB, 0.50129E+00_JPRB, 0.50686E+00_JPRB, 0.51621E+00_JPRB, 0.53550E+00_JPRB, 0.55632E+00_JPRB ,&
&  0.56934E+00_JPRB, 0.57152E+00_JPRB, 0.59708E+00_JPRB, 0.65020E+00_JPRB, 0.67780E+00_JPRB, 0.64457E+00_JPRB, 0.51889E+00_JPRB ,&
&  0.42613E+00_JPRB, 0.37854E+00_JPRB, 0.36670E+00_JPRB, 0.37649E+00_JPRB, 0.40012E+00_JPRB, 0.42926E+00_JPRB, 0.43865E+00_JPRB ,&
&  0.44546E+00_JPRB, 0.44839E+00_JPRB, 0.45094E+00_JPRB, 0.45458E+00_JPRB, 0.45702E+00_JPRB, 0.46260E+00_JPRB, 0.48013E+00_JPRB ,&
&  0.51349E+00_JPRB, 0.55948E+00_JPRB, 0.61120E+00_JPRB, 0.62625E+00_JPRB, 0.68239E+00_JPRB, 0.80457E+00_JPRB, 0.93841E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.10707E+01_JPRB, 0.11093E+01_JPRB, 0.11984E+01_JPRB, 0.12497E+01_JPRB, 0.12841E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.12918E+01_JPRB, 0.12631E+01_JPRB, 0.12287E+01_JPRB, 0.12164E+01_JPRB, 0.12397E+01_JPRB, 0.12944E+01_JPRB, 0.13150E+01_JPRB ,&
&  0.13107E+01_JPRB, 0.12429E+01_JPRB, 0.11148E+01_JPRB, 0.97424E+00_JPRB, 0.92163E+00_JPRB, 0.92234E+00_JPRB, 0.94095E+00_JPRB ,&
&  0.93334E+00_JPRB, 0.91859E+00_JPRB, 0.89975E+00_JPRB, 0.89656E+00_JPRB, 0.82935E+00_JPRB, 0.78675E+00_JPRB, 0.75144E+00_JPRB ,&
&  0.73330E+00_JPRB, 0.72080E+00_JPRB, 0.71836E+00_JPRB, 0.72222E+00_JPRB, 0.71342E+00_JPRB, 0.69445E+00_JPRB, 0.67147E+00_JPRB ,&
&  0.65137E+00_JPRB, 0.62947E+00_JPRB, 0.61101E+00_JPRB, 0.59421E+00_JPRB, 0.58698E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.61879E+00_JPRB, 0.60688E+00_JPRB, 0.58863E+00_JPRB, 0.56159E+00_JPRB, 0.52720E+00_JPRB, 0.49328E+00_JPRB, 0.47141E+00_JPRB ,&
&  0.46072E+00_JPRB, 0.45515E+00_JPRB, 0.44813E+00_JPRB, 0.44339E+00_JPRB, 0.44073E+00_JPRB, 0.44010E+00_JPRB, 0.44143E+00_JPRB ,&
&  0.44435E+00_JPRB, 0.44813E+00_JPRB, 0.45179E+00_JPRB, 0.45368E+00_JPRB, 0.45816E+00_JPRB, 0.46657E+00_JPRB, 0.47802E+00_JPRB ,&
&  0.49053E+00_JPRB, 0.49650E+00_JPRB, 0.49713E+00_JPRB, 0.49808E+00_JPRB, 0.49583E+00_JPRB, 0.50032E+00_JPRB, 0.50107E+00_JPRB ,&
&  0.50620E+00_JPRB, 0.52153E+00_JPRB, 0.55827E+00_JPRB, 0.60011E+00_JPRB, 0.62032E+00_JPRB, 0.57890E+00_JPRB, 0.48830E+00_JPRB ,&
&  0.41602E+00_JPRB, 0.36985E+00_JPRB, 0.35062E+00_JPRB, 0.34642E+00_JPRB, 0.35439E+00_JPRB, 0.37349E+00_JPRB, 0.40074E+00_JPRB ,&
&  0.42212E+00_JPRB, 0.42828E+00_JPRB, 0.43576E+00_JPRB, 0.44226E+00_JPRB, 0.44663E+00_JPRB, 0.44966E+00_JPRB, 0.45618E+00_JPRB ,&
&  0.46935E+00_JPRB, 0.49435E+00_JPRB, 0.53620E+00_JPRB, 0.59701E+00_JPRB, 0.68329E+00_JPRB, 0.70712E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.95377E+00_JPRB, 0.99466E+00_JPRB, 0.10177E+01_JPRB, 0.10313E+01_JPRB, 0.10331E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10673E+01_JPRB, 0.10552E+01_JPRB, 0.10316E+01_JPRB, 0.10186E+01_JPRB, 0.10191E+01_JPRB, 0.10338E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.11276E+01_JPRB, 0.12978E+01_JPRB, 0.11279E+01_JPRB, 0.10487E+01_JPRB, 0.10267E+01_JPRB, 0.98993E+00_JPRB ,&
&  0.95327E+00_JPRB, 0.92430E+00_JPRB, 0.90814E+00_JPRB, 0.89149E+00_JPRB, 0.84532E+00_JPRB, 0.81413E+00_JPRB, 0.79040E+00_JPRB ,&
&  0.77849E+00_JPRB, 0.76388E+00_JPRB, 0.75179E+00_JPRB, 0.74186E+00_JPRB, 0.72877E+00_JPRB, 0.71038E+00_JPRB, 0.69434E+00_JPRB ,&
&  0.67899E+00_JPRB, 0.66428E+00_JPRB, 0.65305E+00_JPRB, 0.64307E+00_JPRB, 0.63222E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.62415E+00_JPRB, 0.61295E+00_JPRB, 0.59468E+00_JPRB, 0.58116E+00_JPRB, 0.56159E+00_JPRB, 0.54096E+00_JPRB, 0.51859E+00_JPRB ,&
&  0.50084E+00_JPRB, 0.48660E+00_JPRB, 0.47463E+00_JPRB, 0.46701E+00_JPRB, 0.46236E+00_JPRB, 0.45960E+00_JPRB, 0.45972E+00_JPRB ,&
&  0.46082E+00_JPRB, 0.45798E+00_JPRB, 0.45945E+00_JPRB, 0.46402E+00_JPRB, 0.47074E+00_JPRB, 0.47789E+00_JPRB, 0.48702E+00_JPRB ,&
&  0.49130E+00_JPRB, 0.49038E+00_JPRB, 0.48516E+00_JPRB, 0.47862E+00_JPRB, 0.47555E+00_JPRB, 0.47999E+00_JPRB, 0.49442E+00_JPRB ,&
&  0.50945E+00_JPRB, 0.52376E+00_JPRB, 0.54141E+00_JPRB, 0.56044E+00_JPRB, 0.56238E+00_JPRB, 0.52579E+00_JPRB, 0.45168E+00_JPRB ,&
&  0.40585E+00_JPRB, 0.37255E+00_JPRB, 0.35720E+00_JPRB, 0.35058E+00_JPRB, 0.35506E+00_JPRB, 0.36820E+00_JPRB, 0.39114E+00_JPRB ,&
&  0.40933E+00_JPRB, 0.42111E+00_JPRB, 0.42441E+00_JPRB, 0.42819E+00_JPRB, 0.43028E+00_JPRB, 0.43439E+00_JPRB, 0.43887E+00_JPRB ,&
&  0.44911E+00_JPRB, 0.45968E+00_JPRB, 0.47378E+00_JPRB, 0.49606E+00_JPRB, 0.52779E+00_JPRB, 0.56214E+00_JPRB, 0.60664E+00_JPRB ,&
&  0.68648E+00_JPRB, 0.73746E+00_JPRB, 0.77171E+00_JPRB, 0.79391E+00_JPRB, 0.82063E+00_JPRB, 0.83843E+00_JPRB, 0.83443E+00_JPRB ,&
&  0.83931E+00_JPRB, 0.87308E+00_JPRB, 0.88544E+00_JPRB, 0.89474E+00_JPRB, 0.89796E+00_JPRB, 0.90480E+00_JPRB, 0.90975E+00_JPRB ,&
&  0.88824E+00_JPRB, 0.89471E+00_JPRB, 0.98900E+00_JPRB, 0.10149E+01_JPRB, 0.99504E+00_JPRB, 0.96554E+00_JPRB, 0.92633E+00_JPRB ,&
&  0.89110E+00_JPRB, 0.85937E+00_JPRB, 0.83306E+00_JPRB, 0.80923E+00_JPRB, 0.79113E+00_JPRB, 0.78115E+00_JPRB, 0.77886E+00_JPRB ,&
&  0.76951E+00_JPRB, 0.75682E+00_JPRB, 0.73932E+00_JPRB, 0.72587E+00_JPRB, 0.71549E+00_JPRB, 0.69956E+00_JPRB, 0.68270E+00_JPRB ,&
&  0.66635E+00_JPRB, 0.65630E+00_JPRB, 0.65214E+00_JPRB, 0.64725E+00_JPRB, 0.63711E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.62521E+00_JPRB, 0.61917E+00_JPRB, 0.60690E+00_JPRB, 0.59620E+00_JPRB, 0.58154E+00_JPRB, 0.56831E+00_JPRB, 0.55129E+00_JPRB ,&
&  0.53576E+00_JPRB, 0.52046E+00_JPRB, 0.50741E+00_JPRB, 0.49757E+00_JPRB, 0.48998E+00_JPRB, 0.48450E+00_JPRB, 0.48155E+00_JPRB ,&
&  0.48052E+00_JPRB, 0.48066E+00_JPRB, 0.48253E+00_JPRB, 0.48536E+00_JPRB, 0.48983E+00_JPRB, 0.49486E+00_JPRB, 0.50110E+00_JPRB ,&
&  0.50362E+00_JPRB, 0.49937E+00_JPRB, 0.48473E+00_JPRB, 0.47922E+00_JPRB, 0.48289E+00_JPRB, 0.48385E+00_JPRB, 0.50194E+00_JPRB ,&
&  0.51245E+00_JPRB, 0.52471E+00_JPRB, 0.52454E+00_JPRB, 0.52233E+00_JPRB, 0.51513E+00_JPRB, 0.43257E+00_JPRB, 0.41762E+00_JPRB ,&
&  0.39772E+00_JPRB, 0.38005E+00_JPRB, 0.37122E+00_JPRB, 0.36569E+00_JPRB, 0.36502E+00_JPRB, 0.37273E+00_JPRB, 0.38506E+00_JPRB ,&
&  0.39384E+00_JPRB, 0.42312E+00_JPRB, 0.42395E+00_JPRB, 0.42386E+00_JPRB, 0.42427E+00_JPRB, 0.42781E+00_JPRB, 0.42735E+00_JPRB ,&
&  0.43211E+00_JPRB, 0.43852E+00_JPRB, 0.44741E+00_JPRB, 0.46309E+00_JPRB, 0.47888E+00_JPRB, 0.49475E+00_JPRB, 0.51434E+00_JPRB ,&
&  0.54358E+00_JPRB, 0.56815E+00_JPRB, 0.58222E+00_JPRB, 0.59360E+00_JPRB, 0.60841E+00_JPRB, 0.62943E+00_JPRB, 0.65634E+00_JPRB ,&
&  0.67789E+00_JPRB, 0.69835E+00_JPRB, 0.72042E+00_JPRB, 0.74693E+00_JPRB, 0.77010E+00_JPRB, 0.79579E+00_JPRB, 0.82091E+00_JPRB ,&
&  0.83620E+00_JPRB, 0.85543E+00_JPRB, 0.86460E+00_JPRB, 0.88808E+00_JPRB, 0.87597E+00_JPRB, 0.84800E+00_JPRB, 0.82840E+00_JPRB ,&
&  0.81055E+00_JPRB, 0.81045E+00_JPRB, 0.79524E+00_JPRB, 0.78000E+00_JPRB, 0.76553E+00_JPRB, 0.75500E+00_JPRB, 0.74403E+00_JPRB ,&
&  0.73565E+00_JPRB, 0.72605E+00_JPRB, 0.71568E+00_JPRB, 0.70345E+00_JPRB, 0.69566E+00_JPRB, 0.68539E+00_JPRB, 0.67468E+00_JPRB ,&
&  0.66072E+00_JPRB, 0.65132E+00_JPRB, 0.64478E+00_JPRB, 0.63814E+00_JPRB, 0.63021E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.62138E+00_JPRB, 0.61290E+00_JPRB, 0.60564E+00_JPRB, 0.59696E+00_JPRB, 0.58708E+00_JPRB, 0.57741E+00_JPRB, 0.56480E+00_JPRB ,&
&  0.55303E+00_JPRB, 0.54156E+00_JPRB, 0.53148E+00_JPRB, 0.52355E+00_JPRB, 0.51736E+00_JPRB, 0.51289E+00_JPRB, 0.51062E+00_JPRB ,&
&  0.50845E+00_JPRB, 0.50813E+00_JPRB, 0.50875E+00_JPRB, 0.51029E+00_JPRB, 0.51273E+00_JPRB, 0.51475E+00_JPRB, 0.51744E+00_JPRB ,&
&  0.51444E+00_JPRB, 0.50242E+00_JPRB, 0.48523E+00_JPRB, 0.48115E+00_JPRB, 0.47848E+00_JPRB, 0.48113E+00_JPRB, 0.48300E+00_JPRB ,&
&  0.49363E+00_JPRB, 0.46895E+00_JPRB, 0.46911E+00_JPRB, 0.44623E+00_JPRB, 0.42743E+00_JPRB, 0.40797E+00_JPRB, 0.38950E+00_JPRB ,&
&  0.38234E+00_JPRB, 0.37433E+00_JPRB, 0.37025E+00_JPRB, 0.37069E+00_JPRB, 0.37357E+00_JPRB, 0.38181E+00_JPRB, 0.39217E+00_JPRB ,&
&  0.39961E+00_JPRB, 0.42317E+00_JPRB, 0.42695E+00_JPRB, 0.42966E+00_JPRB, 0.43426E+00_JPRB, 0.43966E+00_JPRB, 0.43935E+00_JPRB ,&
&  0.44313E+00_JPRB, 0.44717E+00_JPRB, 0.45242E+00_JPRB, 0.46164E+00_JPRB, 0.47462E+00_JPRB, 0.48442E+00_JPRB, 0.49513E+00_JPRB ,&
&  0.50843E+00_JPRB, 0.51936E+00_JPRB, 0.52869E+00_JPRB, 0.53619E+00_JPRB, 0.54418E+00_JPRB, 0.55154E+00_JPRB, 0.56166E+00_JPRB ,&
&  0.57316E+00_JPRB, 0.59389E+00_JPRB, 0.61300E+00_JPRB, 0.64219E+00_JPRB, 0.66982E+00_JPRB, 0.70404E+00_JPRB, 0.73257E+00_JPRB ,&
&  0.76085E+00_JPRB, 0.78171E+00_JPRB, 0.80376E+00_JPRB, 0.82451E+00_JPRB, 0.84063E+00_JPRB, 0.83168E+00_JPRB, 0.80778E+00_JPRB ,&
&  0.79218E+00_JPRB, 0.76875E+00_JPRB, 0.75245E+00_JPRB, 0.73778E+00_JPRB, 0.72555E+00_JPRB, 0.71768E+00_JPRB, 0.71007E+00_JPRB ,&
&  0.70390E+00_JPRB, 0.69743E+00_JPRB, 0.69078E+00_JPRB, 0.68339E+00_JPRB, 0.67805E+00_JPRB, 0.67118E+00_JPRB, 0.66376E+00_JPRB ,&
&  0.65584E+00_JPRB, 0.65107E+00_JPRB, 0.64399E+00_JPRB, 0.63602E+00_JPRB, 0.62825E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.62043E+00_JPRB, 0.61036E+00_JPRB, 0.60288E+00_JPRB, 0.59432E+00_JPRB, 0.58621E+00_JPRB, 0.57724E+00_JPRB, 0.56933E+00_JPRB ,&
&  0.56130E+00_JPRB, 0.55379E+00_JPRB, 0.54669E+00_JPRB, 0.54063E+00_JPRB, 0.53649E+00_JPRB, 0.53331E+00_JPRB, 0.53117E+00_JPRB ,&
&  0.52940E+00_JPRB, 0.52814E+00_JPRB, 0.52776E+00_JPRB, 0.52785E+00_JPRB, 0.52808E+00_JPRB, 0.52794E+00_JPRB, 0.52714E+00_JPRB ,&
&  0.51987E+00_JPRB, 0.50737E+00_JPRB, 0.48302E+00_JPRB, 0.47752E+00_JPRB, 0.46992E+00_JPRB, 0.46529E+00_JPRB, 0.46536E+00_JPRB ,&
&  0.46041E+00_JPRB, 0.45903E+00_JPRB, 0.45312E+00_JPRB, 0.44394E+00_JPRB, 0.43289E+00_JPRB, 0.41667E+00_JPRB, 0.40633E+00_JPRB ,&
&  0.39772E+00_JPRB, 0.39239E+00_JPRB, 0.39119E+00_JPRB, 0.39137E+00_JPRB, 0.39678E+00_JPRB, 0.40277E+00_JPRB, 0.41037E+00_JPRB ,&
&  0.42238E+00_JPRB, 0.43367E+00_JPRB, 0.44708E+00_JPRB, 0.44985E+00_JPRB, 0.45682E+00_JPRB, 0.46340E+00_JPRB, 0.47047E+00_JPRB ,&
&  0.47692E+00_JPRB, 0.48323E+00_JPRB, 0.48910E+00_JPRB, 0.49459E+00_JPRB, 0.50023E+00_JPRB, 0.50551E+00_JPRB, 0.51105E+00_JPRB ,&
&  0.51685E+00_JPRB, 0.52232E+00_JPRB, 0.52800E+00_JPRB, 0.53474E+00_JPRB, 0.54282E+00_JPRB, 0.54888E+00_JPRB, 0.55577E+00_JPRB ,&
&  0.56152E+00_JPRB, 0.56841E+00_JPRB, 0.57531E+00_JPRB, 0.58318E+00_JPRB, 0.59096E+00_JPRB, 0.60128E+00_JPRB, 0.61267E+00_JPRB ,&
&  0.62835E+00_JPRB, 0.64288E+00_JPRB, 0.66234E+00_JPRB, 0.67425E+00_JPRB, 0.69921E+00_JPRB, 0.72308E+00_JPRB, 0.73735E+00_JPRB ,&
&  0.73549E+00_JPRB, 0.72816E+00_JPRB, 0.72073E+00_JPRB, 0.71281E+00_JPRB, 0.70615E+00_JPRB, 0.69759E+00_JPRB, 0.69188E+00_JPRB ,&
&  0.68419E+00_JPRB, 0.67834E+00_JPRB, 0.67187E+00_JPRB, 0.66731E+00_JPRB, 0.66426E+00_JPRB, 0.66028E+00_JPRB, 0.65501E+00_JPRB ,&
&  0.65003E+00_JPRB, 0.64619E+00_JPRB, 0.64220E+00_JPRB, 0.63665E+00_JPRB, 0.62938E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.61658E+00_JPRB, 0.61116E+00_JPRB, 0.60563E+00_JPRB, 0.59923E+00_JPRB, 0.59433E+00_JPRB, 0.58881E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.57858E+00_JPRB, 0.57323E+00_JPRB, 0.56831E+00_JPRB, 0.56376E+00_JPRB, 0.55960E+00_JPRB, 0.55579E+00_JPRB, 0.55292E+00_JPRB ,&
&  0.55005E+00_JPRB, 0.54746E+00_JPRB, 0.54554E+00_JPRB, 0.54425E+00_JPRB, 0.54254E+00_JPRB, 0.54021E+00_JPRB, 0.53628E+00_JPRB ,&
&  0.53090E+00_JPRB, 0.52355E+00_JPRB, 0.51534E+00_JPRB, 0.50662E+00_JPRB, 0.50198E+00_JPRB, 0.47595E+00_JPRB, 0.47073E+00_JPRB ,&
&  0.47118E+00_JPRB, 0.47147E+00_JPRB, 0.47473E+00_JPRB, 0.47844E+00_JPRB, 0.48102E+00_JPRB, 0.47312E+00_JPRB, 0.46736E+00_JPRB ,&
&  0.44841E+00_JPRB, 0.44955E+00_JPRB, 0.44870E+00_JPRB, 0.44917E+00_JPRB, 0.45247E+00_JPRB, 0.45470E+00_JPRB, 0.46063E+00_JPRB ,&
&  0.47724E+00_JPRB, 0.48078E+00_JPRB, 0.48588E+00_JPRB, 0.49054E+00_JPRB, 0.49508E+00_JPRB, 0.49905E+00_JPRB, 0.50381E+00_JPRB ,&
&  0.50816E+00_JPRB, 0.51311E+00_JPRB, 0.51850E+00_JPRB, 0.52241E+00_JPRB, 0.52580E+00_JPRB, 0.52940E+00_JPRB, 0.53264E+00_JPRB ,&
&  0.53459E+00_JPRB, 0.53654E+00_JPRB, 0.53871E+00_JPRB, 0.54185E+00_JPRB, 0.54545E+00_JPRB, 0.54887E+00_JPRB, 0.55221E+00_JPRB ,&
&  0.55495E+00_JPRB, 0.55686E+00_JPRB, 0.55795E+00_JPRB, 0.55897E+00_JPRB, 0.55940E+00_JPRB, 0.55988E+00_JPRB, 0.56121E+00_JPRB ,&
&  0.56348E+00_JPRB, 0.56601E+00_JPRB, 0.56933E+00_JPRB, 0.57371E+00_JPRB, 0.58065E+00_JPRB, 0.58920E+00_JPRB, 0.60368E+00_JPRB ,&
&  0.61555E+00_JPRB, 0.62506E+00_JPRB, 0.62885E+00_JPRB, 0.63281E+00_JPRB, 0.63750E+00_JPRB, 0.64002E+00_JPRB, 0.64229E+00_JPRB ,&
&  0.64237E+00_JPRB, 0.64161E+00_JPRB, 0.63940E+00_JPRB, 0.63720E+00_JPRB, 0.63445E+00_JPRB, 0.63155E+00_JPRB, 0.62969E+00_JPRB ,&
&  0.62867E+00_JPRB, 0.62838E+00_JPRB, 0.62723E+00_JPRB, 0.62461E+00_JPRB, 0.62096E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.60982E+00_JPRB, 0.60811E+00_JPRB, 0.60597E+00_JPRB, 0.60336E+00_JPRB, 0.60057E+00_JPRB, 0.59729E+00_JPRB, 0.59410E+00_JPRB ,&
&  0.59062E+00_JPRB, 0.58760E+00_JPRB, 0.58441E+00_JPRB, 0.58125E+00_JPRB, 0.57797E+00_JPRB, 0.57524E+00_JPRB, 0.57270E+00_JPRB ,&
&  0.57000E+00_JPRB, 0.56699E+00_JPRB, 0.56355E+00_JPRB, 0.56038E+00_JPRB, 0.55689E+00_JPRB, 0.55289E+00_JPRB, 0.54833E+00_JPRB ,&
&  0.54467E+00_JPRB, 0.54058E+00_JPRB, 0.53627E+00_JPRB, 0.53073E+00_JPRB, 0.52603E+00_JPRB, 0.52156E+00_JPRB, 0.51768E+00_JPRB ,&
&  0.51394E+00_JPRB, 0.51103E+00_JPRB, 0.50867E+00_JPRB, 0.50670E+00_JPRB, 0.50612E+00_JPRB, 0.50583E+00_JPRB, 0.50533E+00_JPRB ,&
&  0.50457E+00_JPRB, 0.50369E+00_JPRB, 0.50400E+00_JPRB, 0.50458E+00_JPRB, 0.50519E+00_JPRB, 0.50741E+00_JPRB, 0.50994E+00_JPRB ,&
&  0.51245E+00_JPRB, 0.51472E+00_JPRB, 0.51797E+00_JPRB, 0.52129E+00_JPRB, 0.52431E+00_JPRB, 0.52717E+00_JPRB, 0.52977E+00_JPRB ,&
&  0.53277E+00_JPRB, 0.53658E+00_JPRB, 0.54023E+00_JPRB, 0.54402E+00_JPRB, 0.54674E+00_JPRB, 0.54905E+00_JPRB, 0.55093E+00_JPRB ,&
&  0.55241E+00_JPRB, 0.55384E+00_JPRB, 0.55501E+00_JPRB, 0.55602E+00_JPRB, 0.55662E+00_JPRB, 0.55755E+00_JPRB, 0.55836E+00_JPRB ,&
&  0.55933E+00_JPRB, 0.55996E+00_JPRB, 0.56119E+00_JPRB, 0.56224E+00_JPRB, 0.56280E+00_JPRB, 0.56308E+00_JPRB, 0.56391E+00_JPRB ,&
&  0.56456E+00_JPRB, 0.56496E+00_JPRB, 0.56512E+00_JPRB, 0.56662E+00_JPRB, 0.56782E+00_JPRB, 0.56872E+00_JPRB, 0.56906E+00_JPRB ,&
&  0.57223E+00_JPRB, 0.57680E+00_JPRB, 0.58263E+00_JPRB, 0.59075E+00_JPRB, 0.59694E+00_JPRB, 0.60118E+00_JPRB, 0.60452E+00_JPRB ,&
&  0.60670E+00_JPRB, 0.60835E+00_JPRB, 0.60944E+00_JPRB, 0.61049E+00_JPRB, 0.61028E+00_JPRB, 0.61049E+00_JPRB, 0.61127E+00_JPRB ,&
&  0.61190E+00_JPRB, 0.61199E+00_JPRB, 0.61195E+00_JPRB, 0.61180E+00_JPRB, 0.61113E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.59724E+00_JPRB, 0.59636E+00_JPRB, 0.59547E+00_JPRB, 0.59440E+00_JPRB, 0.59308E+00_JPRB, 0.59157E+00_JPRB, 0.59023E+00_JPRB ,&
&  0.58880E+00_JPRB, 0.58760E+00_JPRB, 0.58596E+00_JPRB, 0.58431E+00_JPRB, 0.58262E+00_JPRB, 0.58109E+00_JPRB, 0.57950E+00_JPRB ,&
&  0.57777E+00_JPRB, 0.57601E+00_JPRB, 0.57418E+00_JPRB, 0.57222E+00_JPRB, 0.57015E+00_JPRB, 0.56825E+00_JPRB, 0.56624E+00_JPRB ,&
&  0.56449E+00_JPRB, 0.56257E+00_JPRB, 0.56064E+00_JPRB, 0.55850E+00_JPRB, 0.55698E+00_JPRB, 0.55506E+00_JPRB, 0.55345E+00_JPRB ,&
&  0.55166E+00_JPRB, 0.55046E+00_JPRB, 0.54909E+00_JPRB, 0.54794E+00_JPRB, 0.54679E+00_JPRB, 0.54656E+00_JPRB, 0.54601E+00_JPRB ,&
&  0.54570E+00_JPRB, 0.54553E+00_JPRB, 0.54576E+00_JPRB, 0.54589E+00_JPRB, 0.54604E+00_JPRB, 0.54648E+00_JPRB, 0.54741E+00_JPRB ,&
&  0.54825E+00_JPRB, 0.54914E+00_JPRB, 0.55036E+00_JPRB, 0.55159E+00_JPRB, 0.55270E+00_JPRB, 0.55379E+00_JPRB, 0.55500E+00_JPRB ,&
&  0.55688E+00_JPRB, 0.55893E+00_JPRB, 0.56089E+00_JPRB, 0.56280E+00_JPRB, 0.56447E+00_JPRB, 0.56572E+00_JPRB, 0.56682E+00_JPRB ,&
&  0.56771E+00_JPRB, 0.56868E+00_JPRB, 0.56937E+00_JPRB, 0.56997E+00_JPRB, 0.57018E+00_JPRB, 0.57087E+00_JPRB, 0.57134E+00_JPRB ,&
&  0.57170E+00_JPRB, 0.57158E+00_JPRB, 0.57206E+00_JPRB, 0.57244E+00_JPRB, 0.57296E+00_JPRB, 0.57341E+00_JPRB, 0.57444E+00_JPRB ,&
&  0.57546E+00_JPRB, 0.57677E+00_JPRB, 0.57820E+00_JPRB, 0.57977E+00_JPRB, 0.58094E+00_JPRB, 0.58210E+00_JPRB, 0.58313E+00_JPRB ,&
&  0.58417E+00_JPRB, 0.58532E+00_JPRB, 0.58670E+00_JPRB, 0.58798E+00_JPRB, 0.58916E+00_JPRB, 0.59056E+00_JPRB, 0.59192E+00_JPRB ,&
&  0.59354E+00_JPRB, 0.59465E+00_JPRB, 0.59570E+00_JPRB, 0.59663E+00_JPRB, 0.59740E+00_JPRB, 0.59783E+00_JPRB, 0.59806E+00_JPRB ,&
&  0.59814E+00_JPRB, 0.59806E+00_JPRB, 0.59794E+00_JPRB, 0.59788E+00_JPRB, 0.59782E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB ,&
&  0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB, 0.57544E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1960:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1960
