!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1970 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1970* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB ,&
&  0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB, 0.14307E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.17612E+00_JPRB, 0.17855E+00_JPRB, 0.18017E+00_JPRB, 0.18119E+00_JPRB, 0.18097E+00_JPRB, 0.18188E+00_JPRB, 0.18285E+00_JPRB ,&
&  0.18412E+00_JPRB, 0.18568E+00_JPRB, 0.18674E+00_JPRB, 0.18772E+00_JPRB, 0.18820E+00_JPRB, 0.18818E+00_JPRB, 0.18816E+00_JPRB ,&
&  0.18796E+00_JPRB, 0.18771E+00_JPRB, 0.18478E+00_JPRB, 0.18215E+00_JPRB, 0.18029E+00_JPRB, 0.17930E+00_JPRB, 0.17838E+00_JPRB ,&
&  0.17760E+00_JPRB, 0.17646E+00_JPRB, 0.17505E+00_JPRB, 0.17288E+00_JPRB, 0.16969E+00_JPRB, 0.16659E+00_JPRB, 0.16342E+00_JPRB ,&
&  0.16058E+00_JPRB, 0.15821E+00_JPRB, 0.15603E+00_JPRB, 0.15382E+00_JPRB, 0.15242E+00_JPRB, 0.15086E+00_JPRB, 0.14927E+00_JPRB ,&
&  0.14731E+00_JPRB, 0.14522E+00_JPRB, 0.14321E+00_JPRB, 0.14116E+00_JPRB, 0.13919E+00_JPRB, 0.13724E+00_JPRB, 0.13581E+00_JPRB ,&
&  0.13430E+00_JPRB, 0.13315E+00_JPRB, 0.13189E+00_JPRB, 0.13074E+00_JPRB, 0.12953E+00_JPRB, 0.12839E+00_JPRB, 0.12714E+00_JPRB ,&
&  0.12628E+00_JPRB, 0.12538E+00_JPRB, 0.12448E+00_JPRB, 0.12364E+00_JPRB, 0.12280E+00_JPRB, 0.12199E+00_JPRB, 0.12107E+00_JPRB ,&
&  0.12024E+00_JPRB, 0.11970E+00_JPRB, 0.11913E+00_JPRB, 0.11850E+00_JPRB, 0.11791E+00_JPRB, 0.11751E+00_JPRB, 0.11716E+00_JPRB ,&
&  0.11683E+00_JPRB, 0.11679E+00_JPRB, 0.11695E+00_JPRB, 0.11710E+00_JPRB, 0.11727E+00_JPRB, 0.11764E+00_JPRB, 0.11819E+00_JPRB ,&
&  0.11885E+00_JPRB, 0.11956E+00_JPRB, 0.12048E+00_JPRB, 0.12177E+00_JPRB, 0.12321E+00_JPRB, 0.12487E+00_JPRB, 0.12667E+00_JPRB ,&
&  0.12862E+00_JPRB, 0.13071E+00_JPRB, 0.13294E+00_JPRB, 0.13517E+00_JPRB, 0.13754E+00_JPRB, 0.13992E+00_JPRB, 0.14221E+00_JPRB ,&
&  0.14459E+00_JPRB, 0.14692E+00_JPRB, 0.14927E+00_JPRB, 0.15141E+00_JPRB, 0.15460E+00_JPRB, 0.15667E+00_JPRB, 0.15889E+00_JPRB ,&
&  0.16099E+00_JPRB, 0.16342E+00_JPRB, 0.16587E+00_JPRB, 0.16891E+00_JPRB, 0.17192E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.20304E+00_JPRB, 0.20330E+00_JPRB, 0.20435E+00_JPRB, 0.21117E+00_JPRB, 0.22507E+00_JPRB, 0.30208E+00_JPRB, 0.29367E+00_JPRB ,&
&  0.28678E+00_JPRB, 0.25832E+00_JPRB, 0.25442E+00_JPRB, 0.24710E+00_JPRB, 0.23742E+00_JPRB, 0.22929E+00_JPRB, 0.22435E+00_JPRB ,&
&  0.21906E+00_JPRB, 0.21333E+00_JPRB, 0.20984E+00_JPRB, 0.20643E+00_JPRB, 0.20319E+00_JPRB, 0.19961E+00_JPRB, 0.19689E+00_JPRB ,&
&  0.19455E+00_JPRB, 0.19293E+00_JPRB, 0.19116E+00_JPRB, 0.18779E+00_JPRB, 0.18626E+00_JPRB, 0.18319E+00_JPRB, 0.17857E+00_JPRB ,&
&  0.17259E+00_JPRB, 0.16923E+00_JPRB, 0.16552E+00_JPRB, 0.16218E+00_JPRB, 0.16454E+00_JPRB, 0.16337E+00_JPRB, 0.16111E+00_JPRB ,&
&  0.15765E+00_JPRB, 0.15407E+00_JPRB, 0.14947E+00_JPRB, 0.14510E+00_JPRB, 0.14139E+00_JPRB, 0.13834E+00_JPRB, 0.13567E+00_JPRB ,&
&  0.13353E+00_JPRB, 0.13171E+00_JPRB, 0.12936E+00_JPRB, 0.12723E+00_JPRB, 0.12498E+00_JPRB, 0.12299E+00_JPRB, 0.12101E+00_JPRB ,&
&  0.11945E+00_JPRB, 0.11798E+00_JPRB, 0.11644E+00_JPRB, 0.11485E+00_JPRB, 0.11360E+00_JPRB, 0.11196E+00_JPRB, 0.11072E+00_JPRB ,&
&  0.10962E+00_JPRB, 0.10828E+00_JPRB, 0.10729E+00_JPRB, 0.10607E+00_JPRB, 0.10517E+00_JPRB, 0.10444E+00_JPRB, 0.10372E+00_JPRB ,&
&  0.10275E+00_JPRB, 0.10210E+00_JPRB, 0.10184E+00_JPRB, 0.10163E+00_JPRB, 0.10131E+00_JPRB, 0.10132E+00_JPRB, 0.10160E+00_JPRB ,&
&  0.10200E+00_JPRB, 0.10248E+00_JPRB, 0.10329E+00_JPRB, 0.10426E+00_JPRB, 0.10570E+00_JPRB, 0.10761E+00_JPRB, 0.11064E+00_JPRB ,&
&  0.11394E+00_JPRB, 0.11760E+00_JPRB, 0.12144E+00_JPRB, 0.12588E+00_JPRB, 0.13033E+00_JPRB, 0.13489E+00_JPRB, 0.14013E+00_JPRB ,&
&  0.14580E+00_JPRB, 0.15012E+00_JPRB, 0.15404E+00_JPRB, 0.15820E+00_JPRB, 0.16654E+00_JPRB, 0.17290E+00_JPRB, 0.17790E+00_JPRB ,&
&  0.18347E+00_JPRB, 0.19094E+00_JPRB, 0.19584E+00_JPRB, 0.19925E+00_JPRB, 0.20147E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.46009E+00_JPRB, 0.47151E+00_JPRB, 0.48657E+00_JPRB, 0.49567E+00_JPRB, 0.50540E+00_JPRB, 0.47269E+00_JPRB, 0.44693E+00_JPRB ,&
&  0.39442E+00_JPRB, 0.36445E+00_JPRB, 0.33955E+00_JPRB, 0.31667E+00_JPRB, 0.29636E+00_JPRB, 0.27872E+00_JPRB, 0.26151E+00_JPRB ,&
&  0.25046E+00_JPRB, 0.24371E+00_JPRB, 0.23452E+00_JPRB, 0.22441E+00_JPRB, 0.21365E+00_JPRB, 0.20427E+00_JPRB, 0.19913E+00_JPRB ,&
&  0.19464E+00_JPRB, 0.19139E+00_JPRB, 0.19007E+00_JPRB, 0.19043E+00_JPRB, 0.19240E+00_JPRB, 0.19282E+00_JPRB, 0.19366E+00_JPRB ,&
&  0.19106E+00_JPRB, 0.19811E+00_JPRB, 0.20732E+00_JPRB, 0.21151E+00_JPRB, 0.21911E+00_JPRB, 0.21760E+00_JPRB, 0.21282E+00_JPRB ,&
&  0.20604E+00_JPRB, 0.19779E+00_JPRB, 0.18865E+00_JPRB, 0.17736E+00_JPRB, 0.16688E+00_JPRB, 0.15899E+00_JPRB, 0.15132E+00_JPRB ,&
&  0.14660E+00_JPRB, 0.14118E+00_JPRB, 0.13508E+00_JPRB, 0.12987E+00_JPRB, 0.12537E+00_JPRB, 0.12158E+00_JPRB, 0.11818E+00_JPRB ,&
&  0.11559E+00_JPRB, 0.11399E+00_JPRB, 0.11245E+00_JPRB, 0.11186E+00_JPRB, 0.11071E+00_JPRB, 0.10963E+00_JPRB, 0.10857E+00_JPRB ,&
&  0.10795E+00_JPRB, 0.10680E+00_JPRB, 0.10593E+00_JPRB, 0.10514E+00_JPRB, 0.10395E+00_JPRB, 0.10291E+00_JPRB, 0.10173E+00_JPRB ,&
&  0.10068E+00_JPRB, 0.10010E+00_JPRB, 0.99779E-01_JPRB, 0.99677E-01_JPRB, 0.99893E-01_JPRB, 0.99835E-01_JPRB, 0.10004E+00_JPRB ,&
&  0.10030E+00_JPRB, 0.10070E+00_JPRB, 0.10071E+00_JPRB, 0.10099E+00_JPRB, 0.10178E+00_JPRB, 0.10288E+00_JPRB, 0.10550E+00_JPRB ,&
&  0.10854E+00_JPRB, 0.11246E+00_JPRB, 0.11678E+00_JPRB, 0.12185E+00_JPRB, 0.12695E+00_JPRB, 0.13212E+00_JPRB, 0.13779E+00_JPRB ,&
&  0.14517E+00_JPRB, 0.15179E+00_JPRB, 0.15938E+00_JPRB, 0.17084E+00_JPRB, 0.19468E+00_JPRB, 0.20841E+00_JPRB, 0.22384E+00_JPRB ,&
&  0.25116E+00_JPRB, 0.28936E+00_JPRB, 0.39251E+00_JPRB, 0.42546E+00_JPRB, 0.44267E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.63839E+00_JPRB, 0.66945E+00_JPRB, 0.69394E+00_JPRB, 0.69620E+00_JPRB, 0.69516E+00_JPRB, 0.65794E+00_JPRB, 0.60019E+00_JPRB ,&
&  0.49324E+00_JPRB, 0.42356E+00_JPRB, 0.37981E+00_JPRB, 0.33855E+00_JPRB, 0.30106E+00_JPRB, 0.27597E+00_JPRB, 0.25874E+00_JPRB ,&
&  0.24565E+00_JPRB, 0.23848E+00_JPRB, 0.23301E+00_JPRB, 0.22841E+00_JPRB, 0.22347E+00_JPRB, 0.21871E+00_JPRB, 0.20999E+00_JPRB ,&
&  0.20479E+00_JPRB, 0.20123E+00_JPRB, 0.19989E+00_JPRB, 0.20234E+00_JPRB, 0.20921E+00_JPRB, 0.21644E+00_JPRB, 0.22720E+00_JPRB ,&
&  0.24089E+00_JPRB, 0.25462E+00_JPRB, 0.26138E+00_JPRB, 0.26453E+00_JPRB, 0.26740E+00_JPRB, 0.26542E+00_JPRB, 0.24261E+00_JPRB ,&
&  0.24769E+00_JPRB, 0.27097E+00_JPRB, 0.28932E+00_JPRB, 0.30010E+00_JPRB, 0.25635E+00_JPRB, 0.21513E+00_JPRB, 0.17462E+00_JPRB ,&
&  0.15500E+00_JPRB, 0.13993E+00_JPRB, 0.13218E+00_JPRB, 0.12637E+00_JPRB, 0.12226E+00_JPRB, 0.11973E+00_JPRB, 0.11749E+00_JPRB ,&
&  0.11542E+00_JPRB, 0.11448E+00_JPRB, 0.11308E+00_JPRB, 0.11240E+00_JPRB, 0.11143E+00_JPRB, 0.11124E+00_JPRB, 0.11007E+00_JPRB ,&
&  0.10887E+00_JPRB, 0.10793E+00_JPRB, 0.10734E+00_JPRB, 0.10705E+00_JPRB, 0.10632E+00_JPRB, 0.10559E+00_JPRB, 0.10393E+00_JPRB ,&
&  0.10274E+00_JPRB, 0.10156E+00_JPRB, 0.10066E+00_JPRB, 0.10065E+00_JPRB, 0.10105E+00_JPRB, 0.10260E+00_JPRB, 0.10347E+00_JPRB ,&
&  0.10366E+00_JPRB, 0.10372E+00_JPRB, 0.10377E+00_JPRB, 0.10444E+00_JPRB, 0.10545E+00_JPRB, 0.10722E+00_JPRB, 0.10969E+00_JPRB ,&
&  0.11265E+00_JPRB, 0.11716E+00_JPRB, 0.12177E+00_JPRB, 0.12695E+00_JPRB, 0.13189E+00_JPRB, 0.13686E+00_JPRB, 0.14329E+00_JPRB ,&
&  0.14990E+00_JPRB, 0.15751E+00_JPRB, 0.16863E+00_JPRB, 0.18066E+00_JPRB, 0.20214E+00_JPRB, 0.22163E+00_JPRB, 0.25368E+00_JPRB ,&
&  0.32464E+00_JPRB, 0.47213E+00_JPRB, 0.55742E+00_JPRB, 0.56802E+00_JPRB, 0.60083E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.75926E+00_JPRB, 0.77763E+00_JPRB, 0.79396E+00_JPRB, 0.79134E+00_JPRB, 0.77408E+00_JPRB, 0.71638E+00_JPRB, 0.65297E+00_JPRB ,&
&  0.57426E+00_JPRB, 0.50797E+00_JPRB, 0.43869E+00_JPRB, 0.37856E+00_JPRB, 0.32825E+00_JPRB, 0.28961E+00_JPRB, 0.26680E+00_JPRB ,&
&  0.25077E+00_JPRB, 0.24379E+00_JPRB, 0.24133E+00_JPRB, 0.24592E+00_JPRB, 0.24845E+00_JPRB, 0.25388E+00_JPRB, 0.24831E+00_JPRB ,&
&  0.24140E+00_JPRB, 0.23327E+00_JPRB, 0.23133E+00_JPRB, 0.24010E+00_JPRB, 0.25319E+00_JPRB, 0.26595E+00_JPRB, 0.28250E+00_JPRB ,&
&  0.30078E+00_JPRB, 0.30662E+00_JPRB, 0.32045E+00_JPRB, 0.32418E+00_JPRB, 0.30939E+00_JPRB, 0.31099E+00_JPRB, 0.31678E+00_JPRB ,&
&  0.32985E+00_JPRB, 0.34837E+00_JPRB, 0.35898E+00_JPRB, 0.37918E+00_JPRB, 0.32694E+00_JPRB, 0.26682E+00_JPRB, 0.21536E+00_JPRB ,&
&  0.17717E+00_JPRB, 0.15551E+00_JPRB, 0.14062E+00_JPRB, 0.13301E+00_JPRB, 0.12587E+00_JPRB, 0.12196E+00_JPRB, 0.11778E+00_JPRB ,&
&  0.11591E+00_JPRB, 0.11577E+00_JPRB, 0.11523E+00_JPRB, 0.11485E+00_JPRB, 0.11516E+00_JPRB, 0.11573E+00_JPRB, 0.11518E+00_JPRB ,&
&  0.11444E+00_JPRB, 0.11416E+00_JPRB, 0.11371E+00_JPRB, 0.11340E+00_JPRB, 0.11262E+00_JPRB, 0.11251E+00_JPRB, 0.11114E+00_JPRB ,&
&  0.11045E+00_JPRB, 0.10921E+00_JPRB, 0.10850E+00_JPRB, 0.10838E+00_JPRB, 0.10870E+00_JPRB, 0.10903E+00_JPRB, 0.10958E+00_JPRB ,&
&  0.11067E+00_JPRB, 0.11166E+00_JPRB, 0.11334E+00_JPRB, 0.11503E+00_JPRB, 0.11770E+00_JPRB, 0.12018E+00_JPRB, 0.12367E+00_JPRB ,&
&  0.12738E+00_JPRB, 0.13186E+00_JPRB, 0.13693E+00_JPRB, 0.14261E+00_JPRB, 0.14761E+00_JPRB, 0.15252E+00_JPRB, 0.15790E+00_JPRB ,&
&  0.16218E+00_JPRB, 0.16903E+00_JPRB, 0.17676E+00_JPRB, 0.18958E+00_JPRB, 0.21629E+00_JPRB, 0.24013E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.37314E+00_JPRB, 0.62787E+00_JPRB, 0.85884E+00_JPRB, 0.70989E+00_JPRB, 0.74087E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.76811E+00_JPRB, 0.78061E+00_JPRB, 0.79769E+00_JPRB, 0.77415E+00_JPRB, 0.75879E+00_JPRB, 0.76212E+00_JPRB, 0.76289E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.70480E+00_JPRB, 0.65582E+00_JPRB, 0.61567E+00_JPRB, 0.54993E+00_JPRB, 0.46559E+00_JPRB, 0.42795E+00_JPRB ,&
&  0.39379E+00_JPRB, 0.39026E+00_JPRB, 0.39253E+00_JPRB, 0.40850E+00_JPRB, 0.41732E+00_JPRB, 0.41386E+00_JPRB, 0.39224E+00_JPRB ,&
&  0.36517E+00_JPRB, 0.33796E+00_JPRB, 0.32052E+00_JPRB, 0.30657E+00_JPRB, 0.30636E+00_JPRB, 0.30458E+00_JPRB, 0.29809E+00_JPRB ,&
&  0.28946E+00_JPRB, 0.29365E+00_JPRB, 0.31090E+00_JPRB, 0.33950E+00_JPRB, 0.38934E+00_JPRB, 0.36929E+00_JPRB, 0.37534E+00_JPRB ,&
&  0.40526E+00_JPRB, 0.42679E+00_JPRB, 0.43530E+00_JPRB, 0.43834E+00_JPRB, 0.43168E+00_JPRB, 0.45678E+00_JPRB, 0.40024E+00_JPRB ,&
&  0.29309E+00_JPRB, 0.23423E+00_JPRB, 0.19235E+00_JPRB, 0.16561E+00_JPRB, 0.14803E+00_JPRB, 0.13787E+00_JPRB, 0.12682E+00_JPRB ,&
&  0.12178E+00_JPRB, 0.11812E+00_JPRB, 0.11722E+00_JPRB, 0.11860E+00_JPRB, 0.12099E+00_JPRB, 0.12403E+00_JPRB, 0.12513E+00_JPRB ,&
&  0.12686E+00_JPRB, 0.12697E+00_JPRB, 0.12619E+00_JPRB, 0.12600E+00_JPRB, 0.12505E+00_JPRB, 0.12573E+00_JPRB, 0.12580E+00_JPRB ,&
&  0.12595E+00_JPRB, 0.12605E+00_JPRB, 0.12762E+00_JPRB, 0.12814E+00_JPRB, 0.12902E+00_JPRB, 0.12609E+00_JPRB, 0.12434E+00_JPRB ,&
&  0.12442E+00_JPRB, 0.12636E+00_JPRB, 0.12958E+00_JPRB, 0.13329E+00_JPRB, 0.13839E+00_JPRB, 0.14251E+00_JPRB, 0.14725E+00_JPRB ,&
&  0.15140E+00_JPRB, 0.15547E+00_JPRB, 0.15944E+00_JPRB, 0.16349E+00_JPRB, 0.16780E+00_JPRB, 0.17212E+00_JPRB, 0.17665E+00_JPRB ,&
&  0.18131E+00_JPRB, 0.18937E+00_JPRB, 0.19932E+00_JPRB, 0.21670E+00_JPRB, 0.24021E+00_JPRB, 0.26736E+00_JPRB, 0.31481E+00_JPRB ,&
&  0.37583E+00_JPRB, 0.60537E+00_JPRB, 0.90424E+00_JPRB, 0.73152E+00_JPRB, 0.74944E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.81870E+00_JPRB, 0.81648E+00_JPRB, 0.82444E+00_JPRB, 0.81408E+00_JPRB, 0.79495E+00_JPRB, 0.78583E+00_JPRB, 0.80276E+00_JPRB ,&
&  0.82250E+00_JPRB, 0.79308E+00_JPRB, 0.77149E+00_JPRB, 0.75229E+00_JPRB, 0.73872E+00_JPRB, 0.72497E+00_JPRB, 0.70273E+00_JPRB ,&
&  0.68262E+00_JPRB, 0.67527E+00_JPRB, 0.67338E+00_JPRB, 0.66831E+00_JPRB, 0.65801E+00_JPRB, 0.64789E+00_JPRB, 0.63012E+00_JPRB ,&
&  0.59756E+00_JPRB, 0.56344E+00_JPRB, 0.54543E+00_JPRB, 0.52187E+00_JPRB, 0.50505E+00_JPRB, 0.48567E+00_JPRB, 0.45806E+00_JPRB ,&
&  0.41333E+00_JPRB, 0.35533E+00_JPRB, 0.31881E+00_JPRB, 0.35508E+00_JPRB, 0.45074E+00_JPRB, 0.44483E+00_JPRB, 0.46379E+00_JPRB ,&
&  0.49840E+00_JPRB, 0.51963E+00_JPRB, 0.52997E+00_JPRB, 0.53318E+00_JPRB, 0.54220E+00_JPRB, 0.54754E+00_JPRB, 0.54397E+00_JPRB ,&
&  0.52948E+00_JPRB, 0.47725E+00_JPRB, 0.43496E+00_JPRB, 0.33248E+00_JPRB, 0.26091E+00_JPRB, 0.22324E+00_JPRB, 0.19440E+00_JPRB ,&
&  0.17584E+00_JPRB, 0.16479E+00_JPRB, 0.16232E+00_JPRB, 0.16235E+00_JPRB, 0.16759E+00_JPRB, 0.17293E+00_JPRB, 0.17300E+00_JPRB ,&
&  0.16945E+00_JPRB, 0.16194E+00_JPRB, 0.15098E+00_JPRB, 0.14435E+00_JPRB, 0.14100E+00_JPRB, 0.14123E+00_JPRB, 0.14328E+00_JPRB ,&
&  0.14509E+00_JPRB, 0.14947E+00_JPRB, 0.16189E+00_JPRB, 0.17288E+00_JPRB, 0.18638E+00_JPRB, 0.16819E+00_JPRB, 0.15347E+00_JPRB ,&
&  0.14834E+00_JPRB, 0.15244E+00_JPRB, 0.15887E+00_JPRB, 0.16374E+00_JPRB, 0.16891E+00_JPRB, 0.17339E+00_JPRB, 0.17796E+00_JPRB ,&
&  0.18234E+00_JPRB, 0.18545E+00_JPRB, 0.18710E+00_JPRB, 0.18885E+00_JPRB, 0.19045E+00_JPRB, 0.19282E+00_JPRB, 0.19580E+00_JPRB ,&
&  0.20083E+00_JPRB, 0.21006E+00_JPRB, 0.22209E+00_JPRB, 0.24192E+00_JPRB, 0.26055E+00_JPRB, 0.27917E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.32164E+00_JPRB, 0.40165E+00_JPRB, 0.59551E+00_JPRB, 0.91706E+00_JPRB, 0.91038E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.81290E+00_JPRB, 0.83285E+00_JPRB, 0.84381E+00_JPRB, 0.85779E+00_JPRB, 0.85796E+00_JPRB, 0.82764E+00_JPRB, 0.81289E+00_JPRB ,&
&  0.81751E+00_JPRB, 0.81263E+00_JPRB, 0.80186E+00_JPRB, 0.79527E+00_JPRB, 0.78633E+00_JPRB, 0.77644E+00_JPRB, 0.76797E+00_JPRB ,&
&  0.75964E+00_JPRB, 0.75465E+00_JPRB, 0.76166E+00_JPRB, 0.77564E+00_JPRB, 0.78716E+00_JPRB, 0.79349E+00_JPRB, 0.79845E+00_JPRB ,&
&  0.79789E+00_JPRB, 0.76479E+00_JPRB, 0.71972E+00_JPRB, 0.68684E+00_JPRB, 0.67024E+00_JPRB, 0.65645E+00_JPRB, 0.62354E+00_JPRB ,&
&  0.55654E+00_JPRB, 0.43492E+00_JPRB, 0.37805E+00_JPRB, 0.41906E+00_JPRB, 0.53819E+00_JPRB, 0.50264E+00_JPRB, 0.52298E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.56899E+00_JPRB, 0.56473E+00_JPRB, 0.55419E+00_JPRB, 0.55063E+00_JPRB, 0.55412E+00_JPRB, 0.55064E+00_JPRB ,&
&  0.54702E+00_JPRB, 0.53088E+00_JPRB, 0.52127E+00_JPRB, 0.51059E+00_JPRB, 0.49315E+00_JPRB, 0.46743E+00_JPRB, 0.45196E+00_JPRB ,&
&  0.41407E+00_JPRB, 0.37675E+00_JPRB, 0.36609E+00_JPRB, 0.36716E+00_JPRB, 0.33653E+00_JPRB, 0.32424E+00_JPRB, 0.30982E+00_JPRB ,&
&  0.26986E+00_JPRB, 0.24855E+00_JPRB, 0.21632E+00_JPRB, 0.19604E+00_JPRB, 0.18124E+00_JPRB, 0.16761E+00_JPRB, 0.16578E+00_JPRB ,&
&  0.17045E+00_JPRB, 0.18271E+00_JPRB, 0.20432E+00_JPRB, 0.23893E+00_JPRB, 0.27060E+00_JPRB, 0.22639E+00_JPRB, 0.20804E+00_JPRB ,&
&  0.19413E+00_JPRB, 0.19102E+00_JPRB, 0.19624E+00_JPRB, 0.20244E+00_JPRB, 0.20890E+00_JPRB, 0.21657E+00_JPRB, 0.22389E+00_JPRB ,&
&  0.22921E+00_JPRB, 0.23233E+00_JPRB, 0.23336E+00_JPRB, 0.23275E+00_JPRB, 0.23194E+00_JPRB, 0.23293E+00_JPRB, 0.23647E+00_JPRB ,&
&  0.24253E+00_JPRB, 0.25471E+00_JPRB, 0.27852E+00_JPRB, 0.31372E+00_JPRB, 0.35985E+00_JPRB, 0.42012E+00_JPRB, 0.51256E+00_JPRB ,&
&  0.63760E+00_JPRB, 0.80207E+00_JPRB, 0.84608E+00_JPRB, 0.85505E+00_JPRB, 0.82446E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.85232E+00_JPRB, 0.86295E+00_JPRB, 0.86076E+00_JPRB, 0.85967E+00_JPRB, 0.86141E+00_JPRB, 0.85433E+00_JPRB, 0.84005E+00_JPRB ,&
&  0.82579E+00_JPRB, 0.79926E+00_JPRB, 0.76893E+00_JPRB, 0.75537E+00_JPRB, 0.76294E+00_JPRB, 0.76282E+00_JPRB, 0.74408E+00_JPRB ,&
&  0.74225E+00_JPRB, 0.75606E+00_JPRB, 0.77902E+00_JPRB, 0.80464E+00_JPRB, 0.83920E+00_JPRB, 0.87508E+00_JPRB, 0.90189E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91033E+00_JPRB, 0.90266E+00_JPRB, 0.87598E+00_JPRB, 0.84035E+00_JPRB, 0.81308E+00_JPRB, 0.79235E+00_JPRB ,&
&  0.76672E+00_JPRB, 0.71928E+00_JPRB, 0.64958E+00_JPRB, 0.64410E+00_JPRB, 0.57990E+00_JPRB, 0.55923E+00_JPRB, 0.58116E+00_JPRB ,&
&  0.59647E+00_JPRB, 0.57847E+00_JPRB, 0.55465E+00_JPRB, 0.53489E+00_JPRB, 0.53105E+00_JPRB, 0.52962E+00_JPRB, 0.52446E+00_JPRB ,&
&  0.52049E+00_JPRB, 0.51365E+00_JPRB, 0.49948E+00_JPRB, 0.48641E+00_JPRB, 0.47405E+00_JPRB, 0.46172E+00_JPRB, 0.45334E+00_JPRB ,&
&  0.42542E+00_JPRB, 0.38274E+00_JPRB, 0.35107E+00_JPRB, 0.33742E+00_JPRB, 0.32993E+00_JPRB, 0.33367E+00_JPRB, 0.34633E+00_JPRB ,&
&  0.35646E+00_JPRB, 0.35691E+00_JPRB, 0.35129E+00_JPRB, 0.33903E+00_JPRB, 0.32058E+00_JPRB, 0.27518E+00_JPRB, 0.24940E+00_JPRB ,&
&  0.26512E+00_JPRB, 0.29915E+00_JPRB, 0.33442E+00_JPRB, 0.34312E+00_JPRB, 0.35265E+00_JPRB, 0.35508E+00_JPRB, 0.34058E+00_JPRB ,&
&  0.31251E+00_JPRB, 0.28139E+00_JPRB, 0.27508E+00_JPRB, 0.27710E+00_JPRB, 0.28137E+00_JPRB, 0.28955E+00_JPRB, 0.30540E+00_JPRB ,&
&  0.32934E+00_JPRB, 0.36814E+00_JPRB, 0.39021E+00_JPRB, 0.39286E+00_JPRB, 0.38558E+00_JPRB, 0.37407E+00_JPRB, 0.37218E+00_JPRB ,&
&  0.38605E+00_JPRB, 0.44701E+00_JPRB, 0.56910E+00_JPRB, 0.71675E+00_JPRB, 0.71563E+00_JPRB, 0.75952E+00_JPRB, 0.79431E+00_JPRB ,&
&  0.82405E+00_JPRB, 0.83738E+00_JPRB, 0.82403E+00_JPRB, 0.82546E+00_JPRB, 0.83222E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.86602E+00_JPRB, 0.87052E+00_JPRB, 0.86089E+00_JPRB, 0.85663E+00_JPRB, 0.84605E+00_JPRB, 0.83254E+00_JPRB, 0.81017E+00_JPRB ,&
&  0.78377E+00_JPRB, 0.75559E+00_JPRB, 0.72905E+00_JPRB, 0.70737E+00_JPRB, 0.71142E+00_JPRB, 0.74407E+00_JPRB, 0.77154E+00_JPRB ,&
&  0.76048E+00_JPRB, 0.75643E+00_JPRB, 0.77344E+00_JPRB, 0.80400E+00_JPRB, 0.83732E+00_JPRB, 0.86809E+00_JPRB, 0.89029E+00_JPRB ,&
&  0.90835E+00_JPRB, 0.89007E+00_JPRB, 0.89361E+00_JPRB, 0.89555E+00_JPRB, 0.88947E+00_JPRB, 0.87164E+00_JPRB, 0.84299E+00_JPRB ,&
&  0.81450E+00_JPRB, 0.78738E+00_JPRB, 0.75376E+00_JPRB, 0.69461E+00_JPRB, 0.76795E+00_JPRB, 0.69074E+00_JPRB, 0.66189E+00_JPRB ,&
&  0.63456E+00_JPRB, 0.61627E+00_JPRB, 0.59575E+00_JPRB, 0.57358E+00_JPRB, 0.56587E+00_JPRB, 0.56119E+00_JPRB, 0.55322E+00_JPRB ,&
&  0.54667E+00_JPRB, 0.53963E+00_JPRB, 0.52894E+00_JPRB, 0.51523E+00_JPRB, 0.50231E+00_JPRB, 0.48848E+00_JPRB, 0.47580E+00_JPRB ,&
&  0.45754E+00_JPRB, 0.43094E+00_JPRB, 0.40299E+00_JPRB, 0.38254E+00_JPRB, 0.37194E+00_JPRB, 0.38105E+00_JPRB, 0.41331E+00_JPRB ,&
&  0.44657E+00_JPRB, 0.45219E+00_JPRB, 0.46186E+00_JPRB, 0.46456E+00_JPRB, 0.45612E+00_JPRB, 0.44636E+00_JPRB, 0.42894E+00_JPRB ,&
&  0.41549E+00_JPRB, 0.41327E+00_JPRB, 0.41433E+00_JPRB, 0.41804E+00_JPRB, 0.42332E+00_JPRB, 0.43070E+00_JPRB, 0.44097E+00_JPRB ,&
&  0.45042E+00_JPRB, 0.45357E+00_JPRB, 0.46572E+00_JPRB, 0.49239E+00_JPRB, 0.52318E+00_JPRB, 0.55441E+00_JPRB, 0.58101E+00_JPRB ,&
&  0.59888E+00_JPRB, 0.61125E+00_JPRB, 0.61946E+00_JPRB, 0.63466E+00_JPRB, 0.65035E+00_JPRB, 0.66623E+00_JPRB, 0.67451E+00_JPRB ,&
&  0.67219E+00_JPRB, 0.67571E+00_JPRB, 0.69264E+00_JPRB, 0.72078E+00_JPRB, 0.74049E+00_JPRB, 0.77431E+00_JPRB, 0.79548E+00_JPRB ,&
&  0.81798E+00_JPRB, 0.82384E+00_JPRB, 0.81853E+00_JPRB, 0.83118E+00_JPRB, 0.84818E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.82594E+00_JPRB, 0.84376E+00_JPRB, 0.85581E+00_JPRB, 0.85676E+00_JPRB, 0.84141E+00_JPRB, 0.81480E+00_JPRB, 0.79199E+00_JPRB ,&
&  0.76589E+00_JPRB, 0.74055E+00_JPRB, 0.71648E+00_JPRB, 0.68380E+00_JPRB, 0.66619E+00_JPRB, 0.66952E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.78361E+00_JPRB, 0.78841E+00_JPRB, 0.78222E+00_JPRB, 0.80742E+00_JPRB, 0.85994E+00_JPRB, 0.89179E+00_JPRB, 0.91279E+00_JPRB ,&
&  0.91597E+00_JPRB, 0.88314E+00_JPRB, 0.84988E+00_JPRB, 0.83621E+00_JPRB, 0.85940E+00_JPRB, 0.87717E+00_JPRB, 0.88241E+00_JPRB ,&
&  0.87799E+00_JPRB, 0.85661E+00_JPRB, 0.83725E+00_JPRB, 0.82084E+00_JPRB, 0.82480E+00_JPRB, 0.79977E+00_JPRB, 0.74731E+00_JPRB ,&
&  0.68095E+00_JPRB, 0.65437E+00_JPRB, 0.64280E+00_JPRB, 0.62818E+00_JPRB, 0.61715E+00_JPRB, 0.61716E+00_JPRB, 0.60785E+00_JPRB ,&
&  0.59436E+00_JPRB, 0.57877E+00_JPRB, 0.56382E+00_JPRB, 0.54609E+00_JPRB, 0.53217E+00_JPRB, 0.51257E+00_JPRB, 0.49427E+00_JPRB ,&
&  0.47732E+00_JPRB, 0.45664E+00_JPRB, 0.43769E+00_JPRB, 0.42454E+00_JPRB, 0.41884E+00_JPRB, 0.42891E+00_JPRB, 0.46839E+00_JPRB ,&
&  0.49953E+00_JPRB, 0.49648E+00_JPRB, 0.49400E+00_JPRB, 0.49779E+00_JPRB, 0.49053E+00_JPRB, 0.48000E+00_JPRB, 0.47434E+00_JPRB ,&
&  0.46938E+00_JPRB, 0.46543E+00_JPRB, 0.46374E+00_JPRB, 0.46573E+00_JPRB, 0.46800E+00_JPRB, 0.46624E+00_JPRB, 0.46880E+00_JPRB ,&
&  0.47361E+00_JPRB, 0.47887E+00_JPRB, 0.49591E+00_JPRB, 0.52038E+00_JPRB, 0.54093E+00_JPRB, 0.55540E+00_JPRB, 0.56878E+00_JPRB ,&
&  0.58957E+00_JPRB, 0.60236E+00_JPRB, 0.58966E+00_JPRB, 0.57752E+00_JPRB, 0.57115E+00_JPRB, 0.57624E+00_JPRB, 0.58678E+00_JPRB ,&
&  0.60754E+00_JPRB, 0.62061E+00_JPRB, 0.62683E+00_JPRB, 0.62846E+00_JPRB, 0.63183E+00_JPRB, 0.63280E+00_JPRB, 0.64543E+00_JPRB ,&
&  0.67921E+00_JPRB, 0.70835E+00_JPRB, 0.71584E+00_JPRB, 0.74866E+00_JPRB, 0.79074E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.75816E+00_JPRB, 0.78999E+00_JPRB, 0.81451E+00_JPRB, 0.82372E+00_JPRB, 0.82200E+00_JPRB, 0.81341E+00_JPRB, 0.79093E+00_JPRB ,&
&  0.76648E+00_JPRB, 0.74912E+00_JPRB, 0.73062E+00_JPRB, 0.70404E+00_JPRB, 0.68196E+00_JPRB, 0.67729E+00_JPRB, 0.68981E+00_JPRB ,&
&  0.73607E+00_JPRB, 0.77708E+00_JPRB, 0.79359E+00_JPRB, 0.80716E+00_JPRB, 0.85701E+00_JPRB, 0.89896E+00_JPRB, 0.90825E+00_JPRB ,&
&  0.90367E+00_JPRB, 0.90556E+00_JPRB, 0.86912E+00_JPRB, 0.83581E+00_JPRB, 0.83570E+00_JPRB, 0.84094E+00_JPRB, 0.82275E+00_JPRB ,&
&  0.84860E+00_JPRB, 0.86553E+00_JPRB, 0.85902E+00_JPRB, 0.83333E+00_JPRB, 0.82542E+00_JPRB, 0.81526E+00_JPRB, 0.80391E+00_JPRB ,&
&  0.78199E+00_JPRB, 0.73357E+00_JPRB, 0.70347E+00_JPRB, 0.70174E+00_JPRB, 0.69449E+00_JPRB, 0.68706E+00_JPRB, 0.67337E+00_JPRB ,&
&  0.63335E+00_JPRB, 0.59750E+00_JPRB, 0.57058E+00_JPRB, 0.55201E+00_JPRB, 0.54041E+00_JPRB, 0.53145E+00_JPRB, 0.52215E+00_JPRB ,&
&  0.51418E+00_JPRB, 0.48745E+00_JPRB, 0.46007E+00_JPRB, 0.45468E+00_JPRB, 0.46092E+00_JPRB, 0.46773E+00_JPRB, 0.49429E+00_JPRB ,&
&  0.51915E+00_JPRB, 0.51743E+00_JPRB, 0.51357E+00_JPRB, 0.51564E+00_JPRB, 0.52413E+00_JPRB, 0.52404E+00_JPRB, 0.51376E+00_JPRB ,&
&  0.50861E+00_JPRB, 0.50610E+00_JPRB, 0.49673E+00_JPRB, 0.48993E+00_JPRB, 0.48939E+00_JPRB, 0.49363E+00_JPRB, 0.50839E+00_JPRB ,&
&  0.51749E+00_JPRB, 0.51889E+00_JPRB, 0.53252E+00_JPRB, 0.54979E+00_JPRB, 0.56015E+00_JPRB, 0.56824E+00_JPRB, 0.57395E+00_JPRB ,&
&  0.58734E+00_JPRB, 0.60688E+00_JPRB, 0.62193E+00_JPRB, 0.62395E+00_JPRB, 0.60096E+00_JPRB, 0.57672E+00_JPRB, 0.55768E+00_JPRB ,&
&  0.54959E+00_JPRB, 0.55424E+00_JPRB, 0.56181E+00_JPRB, 0.56905E+00_JPRB, 0.57569E+00_JPRB, 0.58008E+00_JPRB, 0.57928E+00_JPRB ,&
&  0.59881E+00_JPRB, 0.63006E+00_JPRB, 0.65542E+00_JPRB, 0.68583E+00_JPRB, 0.71758E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.71876E+00_JPRB, 0.73310E+00_JPRB, 0.74474E+00_JPRB, 0.76603E+00_JPRB, 0.78553E+00_JPRB, 0.78301E+00_JPRB, 0.77607E+00_JPRB ,&
&  0.77102E+00_JPRB, 0.76626E+00_JPRB, 0.75878E+00_JPRB, 0.74887E+00_JPRB, 0.74217E+00_JPRB, 0.74153E+00_JPRB, 0.74230E+00_JPRB ,&
&  0.74408E+00_JPRB, 0.76427E+00_JPRB, 0.79739E+00_JPRB, 0.82335E+00_JPRB, 0.85166E+00_JPRB, 0.89514E+00_JPRB, 0.91573E+00_JPRB ,&
&  0.89950E+00_JPRB, 0.89877E+00_JPRB, 0.88795E+00_JPRB, 0.87337E+00_JPRB, 0.87509E+00_JPRB, 0.86571E+00_JPRB, 0.83752E+00_JPRB ,&
&  0.83131E+00_JPRB, 0.82399E+00_JPRB, 0.82026E+00_JPRB, 0.84782E+00_JPRB, 0.86911E+00_JPRB, 0.86873E+00_JPRB, 0.86549E+00_JPRB ,&
&  0.84421E+00_JPRB, 0.80683E+00_JPRB, 0.77179E+00_JPRB, 0.75517E+00_JPRB, 0.75329E+00_JPRB, 0.73316E+00_JPRB, 0.70505E+00_JPRB ,&
&  0.64757E+00_JPRB, 0.60042E+00_JPRB, 0.57433E+00_JPRB, 0.55894E+00_JPRB, 0.55405E+00_JPRB, 0.55157E+00_JPRB, 0.54735E+00_JPRB ,&
&  0.55290E+00_JPRB, 0.54803E+00_JPRB, 0.52087E+00_JPRB, 0.49264E+00_JPRB, 0.48835E+00_JPRB, 0.49715E+00_JPRB, 0.51624E+00_JPRB ,&
&  0.53123E+00_JPRB, 0.52701E+00_JPRB, 0.53371E+00_JPRB, 0.54089E+00_JPRB, 0.55744E+00_JPRB, 0.57336E+00_JPRB, 0.57412E+00_JPRB ,&
&  0.57373E+00_JPRB, 0.56831E+00_JPRB, 0.55507E+00_JPRB, 0.54667E+00_JPRB, 0.53579E+00_JPRB, 0.53187E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.56136E+00_JPRB, 0.57114E+00_JPRB, 0.58221E+00_JPRB, 0.59566E+00_JPRB, 0.60619E+00_JPRB, 0.61444E+00_JPRB, 0.62125E+00_JPRB ,&
&  0.63157E+00_JPRB, 0.64624E+00_JPRB, 0.66097E+00_JPRB, 0.67379E+00_JPRB, 0.67379E+00_JPRB, 0.65981E+00_JPRB, 0.63462E+00_JPRB ,&
&  0.61765E+00_JPRB, 0.60420E+00_JPRB, 0.60075E+00_JPRB, 0.60579E+00_JPRB, 0.60094E+00_JPRB, 0.58179E+00_JPRB, 0.57242E+00_JPRB ,&
&  0.58351E+00_JPRB, 0.60741E+00_JPRB, 0.63027E+00_JPRB, 0.66159E+00_JPRB, 0.69461E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.67657E+00_JPRB, 0.68851E+00_JPRB, 0.70185E+00_JPRB, 0.71474E+00_JPRB, 0.72819E+00_JPRB, 0.72806E+00_JPRB, 0.72949E+00_JPRB ,&
&  0.74854E+00_JPRB, 0.75114E+00_JPRB, 0.74998E+00_JPRB, 0.74093E+00_JPRB, 0.73433E+00_JPRB, 0.73421E+00_JPRB, 0.72545E+00_JPRB ,&
&  0.72127E+00_JPRB, 0.72798E+00_JPRB, 0.74674E+00_JPRB, 0.77528E+00_JPRB, 0.80310E+00_JPRB, 0.83943E+00_JPRB, 0.86622E+00_JPRB ,&
&  0.87056E+00_JPRB, 0.85988E+00_JPRB, 0.85996E+00_JPRB, 0.86310E+00_JPRB, 0.86325E+00_JPRB, 0.84626E+00_JPRB, 0.84169E+00_JPRB ,&
&  0.82180E+00_JPRB, 0.82767E+00_JPRB, 0.69848E+00_JPRB, 0.76794E+00_JPRB, 0.81144E+00_JPRB, 0.84577E+00_JPRB, 0.85638E+00_JPRB ,&
&  0.84928E+00_JPRB, 0.83634E+00_JPRB, 0.81274E+00_JPRB, 0.78651E+00_JPRB, 0.77712E+00_JPRB, 0.75672E+00_JPRB, 0.73345E+00_JPRB ,&
&  0.68463E+00_JPRB, 0.62256E+00_JPRB, 0.59087E+00_JPRB, 0.57943E+00_JPRB, 0.57404E+00_JPRB, 0.57560E+00_JPRB, 0.58117E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.58605E+00_JPRB, 0.57530E+00_JPRB, 0.55925E+00_JPRB, 0.54052E+00_JPRB, 0.53064E+00_JPRB, 0.52778E+00_JPRB ,&
&  0.52407E+00_JPRB, 0.51160E+00_JPRB, 0.51477E+00_JPRB, 0.54409E+00_JPRB, 0.56838E+00_JPRB, 0.58476E+00_JPRB, 0.59107E+00_JPRB ,&
&  0.59540E+00_JPRB, 0.58863E+00_JPRB, 0.58446E+00_JPRB, 0.57566E+00_JPRB, 0.55312E+00_JPRB, 0.54746E+00_JPRB, 0.56866E+00_JPRB ,&
&  0.59236E+00_JPRB, 0.60593E+00_JPRB, 0.61398E+00_JPRB, 0.62588E+00_JPRB, 0.63816E+00_JPRB, 0.64509E+00_JPRB, 0.65147E+00_JPRB ,&
&  0.65649E+00_JPRB, 0.66131E+00_JPRB, 0.67488E+00_JPRB, 0.68963E+00_JPRB, 0.69653E+00_JPRB, 0.69765E+00_JPRB, 0.69726E+00_JPRB ,&
&  0.69392E+00_JPRB, 0.68602E+00_JPRB, 0.67694E+00_JPRB, 0.66831E+00_JPRB, 0.64888E+00_JPRB, 0.62538E+00_JPRB, 0.60837E+00_JPRB ,&
&  0.60002E+00_JPRB, 0.61062E+00_JPRB, 0.61885E+00_JPRB, 0.63180E+00_JPRB, 0.65900E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.68500E+00_JPRB, 0.68209E+00_JPRB, 0.68158E+00_JPRB, 0.69277E+00_JPRB, 0.71014E+00_JPRB, 0.71355E+00_JPRB, 0.71418E+00_JPRB ,&
&  0.72554E+00_JPRB, 0.73381E+00_JPRB, 0.73277E+00_JPRB, 0.73100E+00_JPRB, 0.72511E+00_JPRB, 0.71171E+00_JPRB, 0.70162E+00_JPRB ,&
&  0.70737E+00_JPRB, 0.71356E+00_JPRB, 0.71659E+00_JPRB, 0.74126E+00_JPRB, 0.77361E+00_JPRB, 0.81164E+00_JPRB, 0.83745E+00_JPRB ,&
&  0.84668E+00_JPRB, 0.83938E+00_JPRB, 0.83558E+00_JPRB, 0.83012E+00_JPRB, 0.82466E+00_JPRB, 0.82623E+00_JPRB, 0.82064E+00_JPRB ,&
&  0.81096E+00_JPRB, 0.71678E+00_JPRB, 0.74648E+00_JPRB, 0.75575E+00_JPRB, 0.77159E+00_JPRB, 0.79683E+00_JPRB, 0.82638E+00_JPRB ,&
&  0.82732E+00_JPRB, 0.82226E+00_JPRB, 0.81167E+00_JPRB, 0.79124E+00_JPRB, 0.77376E+00_JPRB, 0.76362E+00_JPRB, 0.74665E+00_JPRB ,&
&  0.71899E+00_JPRB, 0.68175E+00_JPRB, 0.64863E+00_JPRB, 0.62623E+00_JPRB, 0.61629E+00_JPRB, 0.61404E+00_JPRB, 0.61003E+00_JPRB ,&
&  0.60867E+00_JPRB, 0.60773E+00_JPRB, 0.59761E+00_JPRB, 0.58851E+00_JPRB, 0.57932E+00_JPRB, 0.57362E+00_JPRB, 0.56120E+00_JPRB ,&
&  0.55318E+00_JPRB, 0.54247E+00_JPRB, 0.54171E+00_JPRB, 0.56103E+00_JPRB, 0.58068E+00_JPRB, 0.58369E+00_JPRB, 0.59519E+00_JPRB ,&
&  0.61060E+00_JPRB, 0.61648E+00_JPRB, 0.61733E+00_JPRB, 0.61420E+00_JPRB, 0.60680E+00_JPRB, 0.60643E+00_JPRB, 0.61410E+00_JPRB ,&
&  0.62063E+00_JPRB, 0.63020E+00_JPRB, 0.63318E+00_JPRB, 0.63750E+00_JPRB, 0.64394E+00_JPRB, 0.64881E+00_JPRB, 0.65991E+00_JPRB ,&
&  0.66570E+00_JPRB, 0.66996E+00_JPRB, 0.68168E+00_JPRB, 0.69324E+00_JPRB, 0.69783E+00_JPRB, 0.69987E+00_JPRB, 0.70836E+00_JPRB ,&
&  0.71122E+00_JPRB, 0.70886E+00_JPRB, 0.70074E+00_JPRB, 0.68955E+00_JPRB, 0.68241E+00_JPRB, 0.67077E+00_JPRB, 0.65232E+00_JPRB ,&
&  0.63622E+00_JPRB, 0.63491E+00_JPRB, 0.64530E+00_JPRB, 0.65661E+00_JPRB, 0.67481E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.69271E+00_JPRB, 0.68547E+00_JPRB, 0.67965E+00_JPRB, 0.68739E+00_JPRB, 0.70128E+00_JPRB, 0.71101E+00_JPRB, 0.70971E+00_JPRB ,&
&  0.70840E+00_JPRB, 0.71224E+00_JPRB, 0.72007E+00_JPRB, 0.72265E+00_JPRB, 0.71664E+00_JPRB, 0.70127E+00_JPRB, 0.69400E+00_JPRB ,&
&  0.69306E+00_JPRB, 0.69518E+00_JPRB, 0.69599E+00_JPRB, 0.70111E+00_JPRB, 0.72145E+00_JPRB, 0.75264E+00_JPRB, 0.78214E+00_JPRB ,&
&  0.79681E+00_JPRB, 0.79155E+00_JPRB, 0.78537E+00_JPRB, 0.78010E+00_JPRB, 0.77860E+00_JPRB, 0.78317E+00_JPRB, 0.78387E+00_JPRB ,&
&  0.77582E+00_JPRB, 0.63049E+00_JPRB, 0.68482E+00_JPRB, 0.70456E+00_JPRB, 0.72877E+00_JPRB, 0.76053E+00_JPRB, 0.79940E+00_JPRB ,&
&  0.80607E+00_JPRB, 0.79534E+00_JPRB, 0.77538E+00_JPRB, 0.76078E+00_JPRB, 0.74501E+00_JPRB, 0.74014E+00_JPRB, 0.73736E+00_JPRB ,&
&  0.73529E+00_JPRB, 0.73190E+00_JPRB, 0.70953E+00_JPRB, 0.67403E+00_JPRB, 0.65469E+00_JPRB, 0.64319E+00_JPRB, 0.63376E+00_JPRB ,&
&  0.62517E+00_JPRB, 0.61348E+00_JPRB, 0.60194E+00_JPRB, 0.59233E+00_JPRB, 0.59432E+00_JPRB, 0.59313E+00_JPRB, 0.58116E+00_JPRB ,&
&  0.57650E+00_JPRB, 0.57472E+00_JPRB, 0.58213E+00_JPRB, 0.59164E+00_JPRB, 0.59469E+00_JPRB, 0.58846E+00_JPRB, 0.59594E+00_JPRB ,&
&  0.61465E+00_JPRB, 0.62608E+00_JPRB, 0.63627E+00_JPRB, 0.65017E+00_JPRB, 0.65681E+00_JPRB, 0.65541E+00_JPRB, 0.65553E+00_JPRB ,&
&  0.66067E+00_JPRB, 0.65859E+00_JPRB, 0.65099E+00_JPRB, 0.64501E+00_JPRB, 0.64629E+00_JPRB, 0.64719E+00_JPRB, 0.65741E+00_JPRB ,&
&  0.66687E+00_JPRB, 0.66993E+00_JPRB, 0.66594E+00_JPRB, 0.66858E+00_JPRB, 0.67412E+00_JPRB, 0.68326E+00_JPRB, 0.68944E+00_JPRB ,&
&  0.69610E+00_JPRB, 0.69426E+00_JPRB, 0.68259E+00_JPRB, 0.68650E+00_JPRB, 0.69809E+00_JPRB, 0.69881E+00_JPRB, 0.68861E+00_JPRB ,&
&  0.67932E+00_JPRB, 0.67104E+00_JPRB, 0.67475E+00_JPRB, 0.67835E+00_JPRB, 0.68618E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.73634E+00_JPRB, 0.73233E+00_JPRB, 0.72398E+00_JPRB, 0.72271E+00_JPRB, 0.72175E+00_JPRB, 0.71705E+00_JPRB, 0.71376E+00_JPRB ,&
&  0.71022E+00_JPRB, 0.70642E+00_JPRB, 0.71477E+00_JPRB, 0.72299E+00_JPRB, 0.72505E+00_JPRB, 0.72027E+00_JPRB, 0.71569E+00_JPRB ,&
&  0.71267E+00_JPRB, 0.70594E+00_JPRB, 0.69856E+00_JPRB, 0.69597E+00_JPRB, 0.69921E+00_JPRB, 0.71375E+00_JPRB, 0.73559E+00_JPRB ,&
&  0.75196E+00_JPRB, 0.75160E+00_JPRB, 0.75529E+00_JPRB, 0.76614E+00_JPRB, 0.76827E+00_JPRB, 0.76941E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.65790E+00_JPRB, 0.53362E+00_JPRB, 0.59587E+00_JPRB, 0.65218E+00_JPRB, 0.70008E+00_JPRB, 0.75555E+00_JPRB, 0.81737E+00_JPRB ,&
&  0.83682E+00_JPRB, 0.82283E+00_JPRB, 0.80746E+00_JPRB, 0.78845E+00_JPRB, 0.76541E+00_JPRB, 0.75004E+00_JPRB, 0.73784E+00_JPRB ,&
&  0.73495E+00_JPRB, 0.72384E+00_JPRB, 0.70665E+00_JPRB, 0.69266E+00_JPRB, 0.68112E+00_JPRB, 0.66654E+00_JPRB, 0.65245E+00_JPRB ,&
&  0.63801E+00_JPRB, 0.62583E+00_JPRB, 0.62270E+00_JPRB, 0.62163E+00_JPRB, 0.62477E+00_JPRB, 0.62551E+00_JPRB, 0.62146E+00_JPRB ,&
&  0.61830E+00_JPRB, 0.61988E+00_JPRB, 0.62829E+00_JPRB, 0.63513E+00_JPRB, 0.63622E+00_JPRB, 0.63291E+00_JPRB, 0.63957E+00_JPRB ,&
&  0.64308E+00_JPRB, 0.65169E+00_JPRB, 0.66317E+00_JPRB, 0.68482E+00_JPRB, 0.70098E+00_JPRB, 0.70351E+00_JPRB, 0.70378E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.70129E+00_JPRB, 0.68627E+00_JPRB, 0.67605E+00_JPRB, 0.67122E+00_JPRB, 0.66435E+00_JPRB, 0.66230E+00_JPRB ,&
&  0.66716E+00_JPRB, 0.66905E+00_JPRB, 0.66394E+00_JPRB, 0.66616E+00_JPRB, 0.67532E+00_JPRB, 0.68905E+00_JPRB, 0.70203E+00_JPRB ,&
&  0.69458E+00_JPRB, 0.70322E+00_JPRB, 0.70463E+00_JPRB, 0.72400E+00_JPRB, 0.74730E+00_JPRB, 0.74608E+00_JPRB, 0.74226E+00_JPRB ,&
&  0.73700E+00_JPRB, 0.71411E+00_JPRB, 0.71697E+00_JPRB, 0.71980E+00_JPRB, 0.72497E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.74200E+00_JPRB, 0.77087E+00_JPRB, 0.77599E+00_JPRB, 0.75873E+00_JPRB, 0.73099E+00_JPRB, 0.71509E+00_JPRB, 0.70584E+00_JPRB ,&
&  0.70052E+00_JPRB, 0.69659E+00_JPRB, 0.70576E+00_JPRB, 0.71226E+00_JPRB, 0.71422E+00_JPRB, 0.70692E+00_JPRB, 0.69950E+00_JPRB ,&
&  0.68728E+00_JPRB, 0.67197E+00_JPRB, 0.66103E+00_JPRB, 0.65153E+00_JPRB, 0.65120E+00_JPRB, 0.65992E+00_JPRB, 0.67680E+00_JPRB ,&
&  0.69828E+00_JPRB, 0.71325E+00_JPRB, 0.72524E+00_JPRB, 0.73758E+00_JPRB, 0.73962E+00_JPRB, 0.73940E+00_JPRB, 0.74220E+00_JPRB ,&
&  0.74854E+00_JPRB, 0.56438E+00_JPRB, 0.60573E+00_JPRB, 0.66675E+00_JPRB, 0.73010E+00_JPRB, 0.81754E+00_JPRB, 0.89111E+00_JPRB ,&
&  0.91648E+00_JPRB, 0.88687E+00_JPRB, 0.86250E+00_JPRB, 0.83255E+00_JPRB, 0.79842E+00_JPRB, 0.76759E+00_JPRB, 0.73343E+00_JPRB ,&
&  0.70756E+00_JPRB, 0.68901E+00_JPRB, 0.68025E+00_JPRB, 0.67255E+00_JPRB, 0.66409E+00_JPRB, 0.65325E+00_JPRB, 0.64060E+00_JPRB ,&
&  0.62807E+00_JPRB, 0.62189E+00_JPRB, 0.61855E+00_JPRB, 0.61580E+00_JPRB, 0.61818E+00_JPRB, 0.62354E+00_JPRB, 0.62840E+00_JPRB ,&
&  0.63733E+00_JPRB, 0.64386E+00_JPRB, 0.64408E+00_JPRB, 0.64105E+00_JPRB, 0.63763E+00_JPRB, 0.63780E+00_JPRB, 0.65050E+00_JPRB ,&
&  0.65629E+00_JPRB, 0.66750E+00_JPRB, 0.68093E+00_JPRB, 0.70773E+00_JPRB, 0.73155E+00_JPRB, 0.73821E+00_JPRB, 0.73833E+00_JPRB ,&
&  0.73772E+00_JPRB, 0.72960E+00_JPRB, 0.71552E+00_JPRB, 0.70009E+00_JPRB, 0.68954E+00_JPRB, 0.68658E+00_JPRB, 0.67941E+00_JPRB ,&
&  0.67837E+00_JPRB, 0.68167E+00_JPRB, 0.68317E+00_JPRB, 0.69336E+00_JPRB, 0.71105E+00_JPRB, 0.72361E+00_JPRB, 0.72025E+00_JPRB ,&
&  0.71919E+00_JPRB, 0.72433E+00_JPRB, 0.72674E+00_JPRB, 0.77517E+00_JPRB, 0.79719E+00_JPRB, 0.80592E+00_JPRB, 0.79944E+00_JPRB ,&
&  0.79055E+00_JPRB, 0.78099E+00_JPRB, 0.58523E+00_JPRB, 0.68345E+00_JPRB, 0.70920E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.78951E+00_JPRB, 0.81891E+00_JPRB, 0.81339E+00_JPRB, 0.77479E+00_JPRB, 0.73515E+00_JPRB, 0.70896E+00_JPRB, 0.69233E+00_JPRB ,&
&  0.68638E+00_JPRB, 0.67816E+00_JPRB, 0.67859E+00_JPRB, 0.68786E+00_JPRB, 0.69878E+00_JPRB, 0.70159E+00_JPRB, 0.68601E+00_JPRB ,&
&  0.66596E+00_JPRB, 0.65092E+00_JPRB, 0.64456E+00_JPRB, 0.64276E+00_JPRB, 0.63934E+00_JPRB, 0.63772E+00_JPRB, 0.64657E+00_JPRB ,&
&  0.65711E+00_JPRB, 0.66967E+00_JPRB, 0.68291E+00_JPRB, 0.69354E+00_JPRB, 0.70128E+00_JPRB, 0.70833E+00_JPRB, 0.72675E+00_JPRB ,&
&  0.73633E+00_JPRB, 0.56493E+00_JPRB, 0.59397E+00_JPRB, 0.72216E+00_JPRB, 0.84206E+00_JPRB, 0.97425E+00_JPRB, 0.10374E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.10021E+01_JPRB, 0.95616E+00_JPRB, 0.90379E+00_JPRB, 0.84762E+00_JPRB, 0.79720E+00_JPRB, 0.74996E+00_JPRB ,&
&  0.71718E+00_JPRB, 0.69248E+00_JPRB, 0.67780E+00_JPRB, 0.66117E+00_JPRB, 0.65094E+00_JPRB, 0.64423E+00_JPRB, 0.63811E+00_JPRB ,&
&  0.63349E+00_JPRB, 0.62750E+00_JPRB, 0.62897E+00_JPRB, 0.63464E+00_JPRB, 0.63505E+00_JPRB, 0.64217E+00_JPRB, 0.64341E+00_JPRB ,&
&  0.65351E+00_JPRB, 0.66705E+00_JPRB, 0.66946E+00_JPRB, 0.66557E+00_JPRB, 0.66597E+00_JPRB, 0.67480E+00_JPRB, 0.68852E+00_JPRB ,&
&  0.69947E+00_JPRB, 0.72465E+00_JPRB, 0.74745E+00_JPRB, 0.77232E+00_JPRB, 0.78790E+00_JPRB, 0.78709E+00_JPRB, 0.77515E+00_JPRB ,&
&  0.77649E+00_JPRB, 0.76979E+00_JPRB, 0.76027E+00_JPRB, 0.74765E+00_JPRB, 0.73349E+00_JPRB, 0.72196E+00_JPRB, 0.71514E+00_JPRB ,&
&  0.71494E+00_JPRB, 0.72378E+00_JPRB, 0.73133E+00_JPRB, 0.75221E+00_JPRB, 0.76617E+00_JPRB, 0.76857E+00_JPRB, 0.77145E+00_JPRB ,&
&  0.77530E+00_JPRB, 0.77779E+00_JPRB, 0.77908E+00_JPRB, 0.81410E+00_JPRB, 0.88041E+00_JPRB, 0.88865E+00_JPRB, 0.87827E+00_JPRB ,&
&  0.85980E+00_JPRB, 0.82048E+00_JPRB, 0.80419E+00_JPRB, 0.65952E+00_JPRB, 0.74769E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.91396E+00_JPRB, 0.89553E+00_JPRB, 0.84753E+00_JPRB, 0.79099E+00_JPRB, 0.74067E+00_JPRB, 0.70018E+00_JPRB, 0.67682E+00_JPRB ,&
&  0.65806E+00_JPRB, 0.64437E+00_JPRB, 0.64580E+00_JPRB, 0.65461E+00_JPRB, 0.66560E+00_JPRB, 0.66718E+00_JPRB, 0.66048E+00_JPRB ,&
&  0.65011E+00_JPRB, 0.63561E+00_JPRB, 0.62734E+00_JPRB, 0.62292E+00_JPRB, 0.61505E+00_JPRB, 0.61198E+00_JPRB, 0.61072E+00_JPRB ,&
&  0.61412E+00_JPRB, 0.61931E+00_JPRB, 0.62215E+00_JPRB, 0.63086E+00_JPRB, 0.64544E+00_JPRB, 0.66777E+00_JPRB, 0.69099E+00_JPRB ,&
&  0.69258E+00_JPRB, 0.58990E+00_JPRB, 0.66715E+00_JPRB, 0.84774E+00_JPRB, 0.10308E+01_JPRB, 0.11221E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10355E+01_JPRB, 0.97537E+00_JPRB, 0.90909E+00_JPRB, 0.84737E+00_JPRB, 0.79443E+00_JPRB, 0.74640E+00_JPRB ,&
&  0.71040E+00_JPRB, 0.68503E+00_JPRB, 0.67093E+00_JPRB, 0.66127E+00_JPRB, 0.65363E+00_JPRB, 0.63776E+00_JPRB, 0.62781E+00_JPRB ,&
&  0.62701E+00_JPRB, 0.62813E+00_JPRB, 0.63151E+00_JPRB, 0.63557E+00_JPRB, 0.64989E+00_JPRB, 0.67147E+00_JPRB, 0.68361E+00_JPRB ,&
&  0.68661E+00_JPRB, 0.69583E+00_JPRB, 0.70076E+00_JPRB, 0.69603E+00_JPRB, 0.69449E+00_JPRB, 0.72170E+00_JPRB, 0.74600E+00_JPRB ,&
&  0.77536E+00_JPRB, 0.81608E+00_JPRB, 0.86492E+00_JPRB, 0.89276E+00_JPRB, 0.89425E+00_JPRB, 0.87786E+00_JPRB, 0.84722E+00_JPRB ,&
&  0.82907E+00_JPRB, 0.80657E+00_JPRB, 0.78534E+00_JPRB, 0.77093E+00_JPRB, 0.76062E+00_JPRB, 0.75229E+00_JPRB, 0.74368E+00_JPRB ,&
&  0.73723E+00_JPRB, 0.72857E+00_JPRB, 0.73025E+00_JPRB, 0.75256E+00_JPRB, 0.77679E+00_JPRB, 0.78929E+00_JPRB, 0.79160E+00_JPRB ,&
&  0.78862E+00_JPRB, 0.79229E+00_JPRB, 0.81009E+00_JPRB, 0.91095E+00_JPRB, 0.91207E+00_JPRB, 0.92055E+00_JPRB, 0.90983E+00_JPRB ,&
&  0.89689E+00_JPRB, 0.87133E+00_JPRB, 0.83754E+00_JPRB, 0.81945E+00_JPRB, 0.84144E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.99161E+00_JPRB, 0.92395E+00_JPRB, 0.86054E+00_JPRB, 0.79028E+00_JPRB, 0.73800E+00_JPRB, 0.71295E+00_JPRB, 0.69360E+00_JPRB ,&
&  0.66981E+00_JPRB, 0.64866E+00_JPRB, 0.64383E+00_JPRB, 0.64544E+00_JPRB, 0.64430E+00_JPRB, 0.63919E+00_JPRB, 0.62778E+00_JPRB ,&
&  0.62354E+00_JPRB, 0.62836E+00_JPRB, 0.62382E+00_JPRB, 0.61473E+00_JPRB, 0.60475E+00_JPRB, 0.59616E+00_JPRB, 0.58958E+00_JPRB ,&
&  0.58591E+00_JPRB, 0.58909E+00_JPRB, 0.59115E+00_JPRB, 0.60168E+00_JPRB, 0.62056E+00_JPRB, 0.64328E+00_JPRB, 0.67225E+00_JPRB ,&
&  0.67274E+00_JPRB, 0.60893E+00_JPRB, 0.81054E+00_JPRB, 0.10965E+01_JPRB, 0.13234E+01_JPRB, 0.13380E+01_JPRB, 0.12206E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.10245E+01_JPRB, 0.94446E+00_JPRB, 0.88429E+00_JPRB, 0.83367E+00_JPRB, 0.79326E+00_JPRB, 0.76347E+00_JPRB ,&
&  0.73244E+00_JPRB, 0.70770E+00_JPRB, 0.68393E+00_JPRB, 0.66180E+00_JPRB, 0.65611E+00_JPRB, 0.65007E+00_JPRB, 0.64213E+00_JPRB ,&
&  0.63675E+00_JPRB, 0.63709E+00_JPRB, 0.63953E+00_JPRB, 0.64557E+00_JPRB, 0.66310E+00_JPRB, 0.71487E+00_JPRB, 0.76908E+00_JPRB ,&
&  0.78566E+00_JPRB, 0.80040E+00_JPRB, 0.80185E+00_JPRB, 0.79764E+00_JPRB, 0.80966E+00_JPRB, 0.83390E+00_JPRB, 0.87737E+00_JPRB ,&
&  0.92990E+00_JPRB, 0.96633E+00_JPRB, 0.98916E+00_JPRB, 0.10058E+01_JPRB, 0.99914E+00_JPRB, 0.98318E+00_JPRB, 0.95880E+00_JPRB ,&
&  0.93271E+00_JPRB, 0.89587E+00_JPRB, 0.86391E+00_JPRB, 0.84063E+00_JPRB, 0.82020E+00_JPRB, 0.80497E+00_JPRB, 0.78610E+00_JPRB ,&
&  0.76475E+00_JPRB, 0.74343E+00_JPRB, 0.75095E+00_JPRB, 0.76967E+00_JPRB, 0.78212E+00_JPRB, 0.78342E+00_JPRB, 0.79409E+00_JPRB ,&
&  0.80482E+00_JPRB, 0.84308E+00_JPRB, 0.86675E+00_JPRB, 0.10433E+01_JPRB, 0.10910E+01_JPRB, 0.10591E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.97978E+00_JPRB, 0.93255E+00_JPRB, 0.88319E+00_JPRB, 0.86617E+00_JPRB, 0.99758E+00_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.85981E+00_JPRB, 0.80593E+00_JPRB, 0.74544E+00_JPRB, 0.70463E+00_JPRB, 0.67155E+00_JPRB, 0.64533E+00_JPRB, 0.63829E+00_JPRB ,&
&  0.63282E+00_JPRB, 0.62331E+00_JPRB, 0.62796E+00_JPRB, 0.62473E+00_JPRB, 0.61779E+00_JPRB, 0.60430E+00_JPRB, 0.60888E+00_JPRB ,&
&  0.61470E+00_JPRB, 0.61081E+00_JPRB, 0.60446E+00_JPRB, 0.59661E+00_JPRB, 0.58835E+00_JPRB, 0.57646E+00_JPRB, 0.56870E+00_JPRB ,&
&  0.56297E+00_JPRB, 0.56388E+00_JPRB, 0.57111E+00_JPRB, 0.58453E+00_JPRB, 0.60834E+00_JPRB, 0.63836E+00_JPRB, 0.66785E+00_JPRB ,&
&  0.66089E+00_JPRB, 0.76036E+00_JPRB, 0.10752E+01_JPRB, 0.16084E+01_JPRB, 0.17726E+01_JPRB, 0.14928E+01_JPRB, 0.12544E+01_JPRB ,&
&  0.10916E+01_JPRB, 0.97046E+00_JPRB, 0.90015E+00_JPRB, 0.85607E+00_JPRB, 0.81325E+00_JPRB, 0.79118E+00_JPRB, 0.77240E+00_JPRB ,&
&  0.75193E+00_JPRB, 0.72452E+00_JPRB, 0.69005E+00_JPRB, 0.66623E+00_JPRB, 0.65634E+00_JPRB, 0.65649E+00_JPRB, 0.65228E+00_JPRB ,&
&  0.64854E+00_JPRB, 0.65766E+00_JPRB, 0.66493E+00_JPRB, 0.67583E+00_JPRB, 0.75481E+00_JPRB, 0.86390E+00_JPRB, 0.10160E+01_JPRB ,&
&  0.10076E+01_JPRB, 0.98465E+00_JPRB, 0.98583E+00_JPRB, 0.98498E+00_JPRB, 0.10045E+01_JPRB, 0.10187E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10665E+01_JPRB, 0.10576E+01_JPRB, 0.10310E+01_JPRB, 0.10104E+01_JPRB, 0.98824E+00_JPRB, 0.97678E+00_JPRB, 0.93909E+00_JPRB ,&
&  0.90663E+00_JPRB, 0.88043E+00_JPRB, 0.86548E+00_JPRB, 0.83855E+00_JPRB, 0.82559E+00_JPRB, 0.80790E+00_JPRB, 0.78838E+00_JPRB ,&
&  0.75905E+00_JPRB, 0.74164E+00_JPRB, 0.77568E+00_JPRB, 0.79366E+00_JPRB, 0.79570E+00_JPRB, 0.79703E+00_JPRB, 0.81585E+00_JPRB ,&
&  0.85005E+00_JPRB, 0.92851E+00_JPRB, 0.97454E+00_JPRB, 0.10836E+01_JPRB, 0.11182E+01_JPRB, 0.11452E+01_JPRB, 0.10790E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.93153E+00_JPRB, 0.85453E+00_JPRB, 0.82319E+00_JPRB, 0.88288E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.79143E+00_JPRB, 0.71789E+00_JPRB, 0.66033E+00_JPRB, 0.62695E+00_JPRB, 0.59805E+00_JPRB, 0.58466E+00_JPRB, 0.57823E+00_JPRB ,&
&  0.58772E+00_JPRB, 0.59711E+00_JPRB, 0.61196E+00_JPRB, 0.60875E+00_JPRB, 0.58719E+00_JPRB, 0.58029E+00_JPRB, 0.58893E+00_JPRB ,&
&  0.58135E+00_JPRB, 0.58371E+00_JPRB, 0.59252E+00_JPRB, 0.58643E+00_JPRB, 0.57722E+00_JPRB, 0.56558E+00_JPRB, 0.55303E+00_JPRB ,&
&  0.55677E+00_JPRB, 0.56340E+00_JPRB, 0.57732E+00_JPRB, 0.59966E+00_JPRB, 0.62940E+00_JPRB, 0.65831E+00_JPRB, 0.67992E+00_JPRB ,&
&  0.70289E+00_JPRB, 0.10324E+01_JPRB, 0.15744E+01_JPRB, 0.19802E+01_JPRB, 0.18326E+01_JPRB, 0.14243E+01_JPRB, 0.12179E+01_JPRB ,&
&  0.10475E+01_JPRB, 0.94240E+00_JPRB, 0.89338E+00_JPRB, 0.85914E+00_JPRB, 0.81769E+00_JPRB, 0.79764E+00_JPRB, 0.79544E+00_JPRB ,&
&  0.77941E+00_JPRB, 0.74845E+00_JPRB, 0.71280E+00_JPRB, 0.68362E+00_JPRB, 0.66922E+00_JPRB, 0.66902E+00_JPRB, 0.67338E+00_JPRB ,&
&  0.68107E+00_JPRB, 0.69294E+00_JPRB, 0.71451E+00_JPRB, 0.74192E+00_JPRB, 0.83468E+00_JPRB, 0.87162E+00_JPRB, 0.11975E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.13941E+01_JPRB, 0.13333E+01_JPRB, 0.12857E+01_JPRB, 0.12319E+01_JPRB, 0.11770E+01_JPRB, 0.11533E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.10564E+01_JPRB, 0.10044E+01_JPRB, 0.96933E+00_JPRB, 0.93010E+00_JPRB, 0.90056E+00_JPRB, 0.85783E+00_JPRB ,&
&  0.83918E+00_JPRB, 0.81892E+00_JPRB, 0.80754E+00_JPRB, 0.81055E+00_JPRB, 0.81523E+00_JPRB, 0.79344E+00_JPRB, 0.77644E+00_JPRB ,&
&  0.75812E+00_JPRB, 0.77610E+00_JPRB, 0.82729E+00_JPRB, 0.79846E+00_JPRB, 0.80042E+00_JPRB, 0.83054E+00_JPRB, 0.85508E+00_JPRB ,&
&  0.86099E+00_JPRB, 0.93308E+00_JPRB, 0.10096E+01_JPRB, 0.10729E+01_JPRB, 0.11061E+01_JPRB, 0.11555E+01_JPRB, 0.10538E+01_JPRB ,&
&  0.96334E+00_JPRB, 0.88119E+00_JPRB, 0.82897E+00_JPRB, 0.79479E+00_JPRB, 0.82511E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.69273E+00_JPRB, 0.63769E+00_JPRB, 0.60505E+00_JPRB, 0.58035E+00_JPRB, 0.58101E+00_JPRB, 0.57414E+00_JPRB, 0.56971E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.59742E+00_JPRB, 0.61002E+00_JPRB, 0.59822E+00_JPRB, 0.57491E+00_JPRB, 0.56742E+00_JPRB, 0.55717E+00_JPRB ,&
&  0.54724E+00_JPRB, 0.57346E+00_JPRB, 0.59991E+00_JPRB, 0.61064E+00_JPRB, 0.60093E+00_JPRB, 0.57445E+00_JPRB, 0.56122E+00_JPRB ,&
&  0.56881E+00_JPRB, 0.58771E+00_JPRB, 0.61770E+00_JPRB, 0.65089E+00_JPRB, 0.67853E+00_JPRB, 0.70345E+00_JPRB, 0.74135E+00_JPRB ,&
&  0.86509E+00_JPRB, 0.11113E+01_JPRB, 0.17261E+01_JPRB, 0.22509E+01_JPRB, 0.17972E+01_JPRB, 0.13764E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.95264E+00_JPRB, 0.91950E+00_JPRB, 0.89004E+00_JPRB, 0.85262E+00_JPRB, 0.82699E+00_JPRB, 0.82218E+00_JPRB ,&
&  0.79894E+00_JPRB, 0.77645E+00_JPRB, 0.75503E+00_JPRB, 0.74357E+00_JPRB, 0.73768E+00_JPRB, 0.73587E+00_JPRB, 0.73697E+00_JPRB ,&
&  0.75062E+00_JPRB, 0.76440E+00_JPRB, 0.79096E+00_JPRB, 0.85046E+00_JPRB, 0.93218E+00_JPRB, 0.10884E+01_JPRB, 0.13710E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.16769E+01_JPRB, 0.15280E+01_JPRB, 0.14234E+01_JPRB, 0.13381E+01_JPRB, 0.12820E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.11344E+01_JPRB, 0.10502E+01_JPRB, 0.97713E+00_JPRB, 0.92342E+00_JPRB, 0.85733E+00_JPRB, 0.82239E+00_JPRB, 0.81696E+00_JPRB ,&
&  0.80018E+00_JPRB, 0.79792E+00_JPRB, 0.79159E+00_JPRB, 0.80634E+00_JPRB, 0.79113E+00_JPRB, 0.77800E+00_JPRB, 0.77236E+00_JPRB ,&
&  0.78420E+00_JPRB, 0.84165E+00_JPRB, 0.87391E+00_JPRB, 0.81162E+00_JPRB, 0.79105E+00_JPRB, 0.80853E+00_JPRB, 0.84320E+00_JPRB ,&
&  0.89928E+00_JPRB, 0.97441E+00_JPRB, 0.10178E+01_JPRB, 0.10619E+01_JPRB, 0.10152E+01_JPRB, 0.10796E+01_JPRB, 0.97318E+00_JPRB ,&
&  0.88985E+00_JPRB, 0.83859E+00_JPRB, 0.81083E+00_JPRB, 0.78170E+00_JPRB, 0.75858E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.67586E+00_JPRB, 0.63657E+00_JPRB, 0.58955E+00_JPRB, 0.56886E+00_JPRB, 0.57553E+00_JPRB, 0.56258E+00_JPRB, 0.56877E+00_JPRB ,&
&  0.58639E+00_JPRB, 0.58642E+00_JPRB, 0.58540E+00_JPRB, 0.57587E+00_JPRB, 0.56728E+00_JPRB, 0.55392E+00_JPRB, 0.54509E+00_JPRB ,&
&  0.55928E+00_JPRB, 0.59791E+00_JPRB, 0.60980E+00_JPRB, 0.59829E+00_JPRB, 0.58899E+00_JPRB, 0.57272E+00_JPRB, 0.56883E+00_JPRB ,&
&  0.59519E+00_JPRB, 0.64674E+00_JPRB, 0.70110E+00_JPRB, 0.73122E+00_JPRB, 0.75157E+00_JPRB, 0.77642E+00_JPRB, 0.89141E+00_JPRB ,&
&  0.12038E+01_JPRB, 0.29088E+01_JPRB, 0.13131E+01_JPRB, 0.23526E+01_JPRB, 0.16642E+01_JPRB, 0.12780E+01_JPRB, 0.97384E+00_JPRB ,&
&  0.93096E+00_JPRB, 0.99285E+00_JPRB, 0.95827E+00_JPRB, 0.94415E+00_JPRB, 0.89515E+00_JPRB, 0.85772E+00_JPRB, 0.84890E+00_JPRB ,&
&  0.83456E+00_JPRB, 0.82544E+00_JPRB, 0.83679E+00_JPRB, 0.85303E+00_JPRB, 0.85664E+00_JPRB, 0.84640E+00_JPRB, 0.84038E+00_JPRB ,&
&  0.84685E+00_JPRB, 0.85247E+00_JPRB, 0.90183E+00_JPRB, 0.97144E+00_JPRB, 0.10635E+01_JPRB, 0.12478E+01_JPRB, 0.14845E+01_JPRB ,&
&  0.15800E+01_JPRB, 0.16645E+01_JPRB, 0.14805E+01_JPRB, 0.13910E+01_JPRB, 0.12975E+01_JPRB, 0.12232E+01_JPRB, 0.11118E+01_JPRB ,&
&  0.10137E+01_JPRB, 0.95384E+00_JPRB, 0.89810E+00_JPRB, 0.85439E+00_JPRB, 0.80924E+00_JPRB, 0.79510E+00_JPRB, 0.77660E+00_JPRB ,&
&  0.78125E+00_JPRB, 0.80895E+00_JPRB, 0.80287E+00_JPRB, 0.79587E+00_JPRB, 0.78400E+00_JPRB, 0.79616E+00_JPRB, 0.81252E+00_JPRB ,&
&  0.84618E+00_JPRB, 0.90842E+00_JPRB, 0.89902E+00_JPRB, 0.79704E+00_JPRB, 0.76656E+00_JPRB, 0.79662E+00_JPRB, 0.83215E+00_JPRB ,&
&  0.92003E+00_JPRB, 0.99322E+00_JPRB, 0.99712E+00_JPRB, 0.99930E+00_JPRB, 0.99268E+00_JPRB, 0.10492E+01_JPRB, 0.91536E+00_JPRB ,&
&  0.84181E+00_JPRB, 0.80914E+00_JPRB, 0.78166E+00_JPRB, 0.74316E+00_JPRB, 0.72660E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.67265E+00_JPRB, 0.65079E+00_JPRB, 0.60558E+00_JPRB, 0.56826E+00_JPRB, 0.56540E+00_JPRB, 0.57112E+00_JPRB, 0.57979E+00_JPRB ,&
&  0.58117E+00_JPRB, 0.57965E+00_JPRB, 0.57541E+00_JPRB, 0.56939E+00_JPRB, 0.57474E+00_JPRB, 0.56682E+00_JPRB, 0.59182E+00_JPRB ,&
&  0.61481E+00_JPRB, 0.61264E+00_JPRB, 0.61898E+00_JPRB, 0.59671E+00_JPRB, 0.59396E+00_JPRB, 0.59021E+00_JPRB, 0.61606E+00_JPRB ,&
&  0.67738E+00_JPRB, 0.74954E+00_JPRB, 0.79217E+00_JPRB, 0.80673E+00_JPRB, 0.82434E+00_JPRB, 0.86942E+00_JPRB, 0.10441E+01_JPRB ,&
&  0.15086E+01_JPRB, 0.30895E+01_JPRB, 0.15496E+01_JPRB, 0.22016E+01_JPRB, 0.14476E+01_JPRB, 0.11005E+01_JPRB, 0.90030E+00_JPRB ,&
&  0.92364E+00_JPRB, 0.10790E+01_JPRB, 0.10260E+01_JPRB, 0.98668E+00_JPRB, 0.89281E+00_JPRB, 0.85859E+00_JPRB, 0.85112E+00_JPRB ,&
&  0.86666E+00_JPRB, 0.92471E+00_JPRB, 0.98061E+00_JPRB, 0.10099E+01_JPRB, 0.99547E+00_JPRB, 0.96394E+00_JPRB, 0.96048E+00_JPRB ,&
&  0.96139E+00_JPRB, 0.96050E+00_JPRB, 0.10499E+01_JPRB, 0.11129E+01_JPRB, 0.13277E+01_JPRB, 0.14475E+01_JPRB, 0.14808E+01_JPRB ,&
&  0.14571E+01_JPRB, 0.14821E+01_JPRB, 0.13631E+01_JPRB, 0.13102E+01_JPRB, 0.12028E+01_JPRB, 0.11254E+01_JPRB, 0.10186E+01_JPRB ,&
&  0.95582E+00_JPRB, 0.91007E+00_JPRB, 0.86306E+00_JPRB, 0.81091E+00_JPRB, 0.78759E+00_JPRB, 0.78716E+00_JPRB, 0.77232E+00_JPRB ,&
&  0.79766E+00_JPRB, 0.82801E+00_JPRB, 0.82308E+00_JPRB, 0.82230E+00_JPRB, 0.83047E+00_JPRB, 0.85061E+00_JPRB, 0.87302E+00_JPRB ,&
&  0.90677E+00_JPRB, 0.95695E+00_JPRB, 0.90340E+00_JPRB, 0.79328E+00_JPRB, 0.79274E+00_JPRB, 0.82669E+00_JPRB, 0.84563E+00_JPRB ,&
&  0.91614E+00_JPRB, 0.97135E+00_JPRB, 0.95951E+00_JPRB, 0.92283E+00_JPRB, 0.92131E+00_JPRB, 0.95646E+00_JPRB, 0.86957E+00_JPRB ,&
&  0.82376E+00_JPRB, 0.79978E+00_JPRB, 0.78227E+00_JPRB, 0.76389E+00_JPRB, 0.72610E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.62505E+00_JPRB, 0.60829E+00_JPRB, 0.59061E+00_JPRB, 0.57492E+00_JPRB, 0.56609E+00_JPRB, 0.57235E+00_JPRB, 0.57697E+00_JPRB ,&
&  0.57083E+00_JPRB, 0.56963E+00_JPRB, 0.56369E+00_JPRB, 0.56848E+00_JPRB, 0.59308E+00_JPRB, 0.64113E+00_JPRB, 0.68948E+00_JPRB ,&
&  0.68916E+00_JPRB, 0.65687E+00_JPRB, 0.65113E+00_JPRB, 0.64582E+00_JPRB, 0.66295E+00_JPRB, 0.68672E+00_JPRB, 0.73284E+00_JPRB ,&
&  0.79637E+00_JPRB, 0.84605E+00_JPRB, 0.87021E+00_JPRB, 0.88181E+00_JPRB, 0.90794E+00_JPRB, 0.94786E+00_JPRB, 0.12165E+01_JPRB ,&
&  0.17872E+01_JPRB, 0.35957E+01_JPRB, 0.27509E+01_JPRB, 0.17833E+01_JPRB, 0.12064E+01_JPRB, 0.10131E+01_JPRB, 0.97125E+00_JPRB ,&
&  0.97495E+00_JPRB, 0.90735E+00_JPRB, 0.10819E+01_JPRB, 0.10340E+01_JPRB, 0.94531E+00_JPRB, 0.90875E+00_JPRB, 0.94119E+00_JPRB ,&
&  0.10241E+01_JPRB, 0.10982E+01_JPRB, 0.11504E+01_JPRB, 0.11561E+01_JPRB, 0.11191E+01_JPRB, 0.10969E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.11360E+01_JPRB, 0.13374E+01_JPRB, 0.14492E+01_JPRB, 0.15280E+01_JPRB, 0.14841E+01_JPRB, 0.14010E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.13112E+01_JPRB, 0.12947E+01_JPRB, 0.12813E+01_JPRB, 0.10992E+01_JPRB, 0.11069E+01_JPRB, 0.99615E+00_JPRB ,&
&  0.96101E+00_JPRB, 0.91267E+00_JPRB, 0.85637E+00_JPRB, 0.78786E+00_JPRB, 0.79657E+00_JPRB, 0.80334E+00_JPRB, 0.78525E+00_JPRB ,&
&  0.81646E+00_JPRB, 0.85403E+00_JPRB, 0.84123E+00_JPRB, 0.85273E+00_JPRB, 0.86357E+00_JPRB, 0.88661E+00_JPRB, 0.90788E+00_JPRB ,&
&  0.92490E+00_JPRB, 0.97084E+00_JPRB, 0.92280E+00_JPRB, 0.87047E+00_JPRB, 0.86127E+00_JPRB, 0.84923E+00_JPRB, 0.85857E+00_JPRB ,&
&  0.90398E+00_JPRB, 0.97642E+00_JPRB, 0.94392E+00_JPRB, 0.94103E+00_JPRB, 0.90028E+00_JPRB, 0.89436E+00_JPRB, 0.85318E+00_JPRB ,&
&  0.84440E+00_JPRB, 0.82565E+00_JPRB, 0.83843E+00_JPRB, 0.77140E+00_JPRB, 0.68214E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.56077E+00_JPRB, 0.55450E+00_JPRB, 0.54955E+00_JPRB, 0.55512E+00_JPRB, 0.55916E+00_JPRB, 0.55265E+00_JPRB, 0.55526E+00_JPRB ,&
&  0.56489E+00_JPRB, 0.57749E+00_JPRB, 0.57631E+00_JPRB, 0.60806E+00_JPRB, 0.67996E+00_JPRB, 0.75610E+00_JPRB, 0.76050E+00_JPRB ,&
&  0.74901E+00_JPRB, 0.73515E+00_JPRB, 0.72915E+00_JPRB, 0.74258E+00_JPRB, 0.75617E+00_JPRB, 0.79899E+00_JPRB, 0.84350E+00_JPRB ,&
&  0.88833E+00_JPRB, 0.93560E+00_JPRB, 0.97710E+00_JPRB, 0.10180E+01_JPRB, 0.10909E+01_JPRB, 0.12046E+01_JPRB, 0.15040E+01_JPRB ,&
&  0.20422E+01_JPRB, 0.36246E+01_JPRB, 0.11259E+01_JPRB, 0.11590E+01_JPRB, 0.11181E+01_JPRB, 0.98367E+00_JPRB, 0.86577E+00_JPRB ,&
&  0.80485E+00_JPRB, 0.73666E+00_JPRB, 0.93872E+00_JPRB, 0.10734E+01_JPRB, 0.10192E+01_JPRB, 0.10509E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.12092E+01_JPRB, 0.12742E+01_JPRB, 0.13063E+01_JPRB, 0.12754E+01_JPRB, 0.12423E+01_JPRB, 0.12310E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.13328E+01_JPRB, 0.14676E+01_JPRB, 0.15767E+01_JPRB, 0.15068E+01_JPRB, 0.15868E+01_JPRB, 0.14859E+01_JPRB, 0.13781E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11852E+01_JPRB, 0.12176E+01_JPRB, 0.12635E+01_JPRB, 0.10777E+01_JPRB, 0.96939E+00_JPRB, 0.98290E+00_JPRB ,&
&  0.94020E+00_JPRB, 0.87129E+00_JPRB, 0.80399E+00_JPRB, 0.75613E+00_JPRB, 0.76057E+00_JPRB, 0.78499E+00_JPRB, 0.77337E+00_JPRB ,&
&  0.80571E+00_JPRB, 0.84413E+00_JPRB, 0.82488E+00_JPRB, 0.85542E+00_JPRB, 0.87850E+00_JPRB, 0.89579E+00_JPRB, 0.90855E+00_JPRB ,&
&  0.92640E+00_JPRB, 0.97397E+00_JPRB, 0.10065E+01_JPRB, 0.98668E+00_JPRB, 0.92882E+00_JPRB, 0.89965E+00_JPRB, 0.92910E+00_JPRB ,&
&  0.96530E+00_JPRB, 0.10251E+01_JPRB, 0.92868E+00_JPRB, 0.87036E+00_JPRB, 0.87158E+00_JPRB, 0.86053E+00_JPRB, 0.86770E+00_JPRB ,&
&  0.89716E+00_JPRB, 0.96547E+00_JPRB, 0.10001E+01_JPRB, 0.71188E+00_JPRB, 0.61032E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.52709E+00_JPRB, 0.52394E+00_JPRB, 0.51459E+00_JPRB, 0.52753E+00_JPRB, 0.52672E+00_JPRB, 0.53502E+00_JPRB, 0.55273E+00_JPRB ,&
&  0.58200E+00_JPRB, 0.61998E+00_JPRB, 0.65172E+00_JPRB, 0.69534E+00_JPRB, 0.75693E+00_JPRB, 0.81332E+00_JPRB, 0.81765E+00_JPRB ,&
&  0.80486E+00_JPRB, 0.79852E+00_JPRB, 0.80249E+00_JPRB, 0.81068E+00_JPRB, 0.83799E+00_JPRB, 0.87368E+00_JPRB, 0.90995E+00_JPRB ,&
&  0.96222E+00_JPRB, 0.10335E+01_JPRB, 0.11070E+01_JPRB, 0.11957E+01_JPRB, 0.13086E+01_JPRB, 0.14855E+01_JPRB, 0.17232E+01_JPRB ,&
&  0.23804E+01_JPRB, 0.23015E+01_JPRB, 0.91016E+00_JPRB, 0.11711E+01_JPRB, 0.10387E+01_JPRB, 0.87967E+00_JPRB, 0.71405E+00_JPRB ,&
&  0.65993E+00_JPRB, 0.79481E+00_JPRB, 0.99815E+00_JPRB, 0.11360E+01_JPRB, 0.11286E+01_JPRB, 0.12089E+01_JPRB, 0.12770E+01_JPRB ,&
&  0.13517E+01_JPRB, 0.13938E+01_JPRB, 0.14007E+01_JPRB, 0.13882E+01_JPRB, 0.13977E+01_JPRB, 0.14357E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.16167E+01_JPRB, 0.16952E+01_JPRB, 0.17277E+01_JPRB, 0.15572E+01_JPRB, 0.16511E+01_JPRB, 0.15414E+01_JPRB, 0.14868E+01_JPRB ,&
&  0.12676E+01_JPRB, 0.11214E+01_JPRB, 0.11225E+01_JPRB, 0.11973E+01_JPRB, 0.10286E+01_JPRB, 0.89971E+00_JPRB, 0.93581E+00_JPRB ,&
&  0.87147E+00_JPRB, 0.79078E+00_JPRB, 0.75978E+00_JPRB, 0.74683E+00_JPRB, 0.74664E+00_JPRB, 0.74891E+00_JPRB, 0.76436E+00_JPRB ,&
&  0.78880E+00_JPRB, 0.78799E+00_JPRB, 0.76442E+00_JPRB, 0.78814E+00_JPRB, 0.81587E+00_JPRB, 0.84572E+00_JPRB, 0.87152E+00_JPRB ,&
&  0.90709E+00_JPRB, 0.95064E+00_JPRB, 0.98467E+00_JPRB, 0.10075E+01_JPRB, 0.97104E+00_JPRB, 0.92643E+00_JPRB, 0.94319E+00_JPRB ,&
&  0.89176E+00_JPRB, 0.88069E+00_JPRB, 0.86450E+00_JPRB, 0.84377E+00_JPRB, 0.80678E+00_JPRB, 0.83054E+00_JPRB, 0.84676E+00_JPRB ,&
&  0.88400E+00_JPRB, 0.91401E+00_JPRB, 0.83015E+00_JPRB, 0.63165E+00_JPRB, 0.55744E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.53477E+00_JPRB, 0.53699E+00_JPRB, 0.52818E+00_JPRB, 0.52713E+00_JPRB, 0.51980E+00_JPRB, 0.52511E+00_JPRB, 0.53929E+00_JPRB ,&
&  0.57021E+00_JPRB, 0.62355E+00_JPRB, 0.68919E+00_JPRB, 0.74789E+00_JPRB, 0.81790E+00_JPRB, 0.87143E+00_JPRB, 0.87385E+00_JPRB ,&
&  0.85670E+00_JPRB, 0.85225E+00_JPRB, 0.84056E+00_JPRB, 0.86452E+00_JPRB, 0.88720E+00_JPRB, 0.92309E+00_JPRB, 0.96569E+00_JPRB ,&
&  0.10166E+01_JPRB, 0.10815E+01_JPRB, 0.11629E+01_JPRB, 0.12583E+01_JPRB, 0.13895E+01_JPRB, 0.15308E+01_JPRB, 0.16574E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.59811E+00_JPRB, 0.10215E+01_JPRB, 0.11346E+01_JPRB, 0.87791E+00_JPRB, 0.77382E+00_JPRB, 0.65983E+00_JPRB ,&
&  0.64484E+00_JPRB, 0.89597E+00_JPRB, 0.10563E+01_JPRB, 0.11957E+01_JPRB, 0.12335E+01_JPRB, 0.13298E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.14320E+01_JPRB, 0.14671E+01_JPRB, 0.15046E+01_JPRB, 0.15413E+01_JPRB, 0.15945E+01_JPRB, 0.16536E+01_JPRB, 0.17344E+01_JPRB ,&
&  0.18242E+01_JPRB, 0.18441E+01_JPRB, 0.18472E+01_JPRB, 0.15859E+01_JPRB, 0.17612E+01_JPRB, 0.18025E+01_JPRB, 0.19302E+01_JPRB ,&
&  0.19776E+01_JPRB, 0.12048E+01_JPRB, 0.10078E+01_JPRB, 0.12540E+01_JPRB, 0.11013E+01_JPRB, 0.84034E+00_JPRB, 0.88501E+00_JPRB ,&
&  0.79633E+00_JPRB, 0.72368E+00_JPRB, 0.71506E+00_JPRB, 0.72574E+00_JPRB, 0.70852E+00_JPRB, 0.72255E+00_JPRB, 0.73404E+00_JPRB ,&
&  0.75942E+00_JPRB, 0.74094E+00_JPRB, 0.71701E+00_JPRB, 0.73069E+00_JPRB, 0.75617E+00_JPRB, 0.79051E+00_JPRB, 0.81574E+00_JPRB ,&
&  0.84944E+00_JPRB, 0.87723E+00_JPRB, 0.90639E+00_JPRB, 0.94204E+00_JPRB, 0.97147E+00_JPRB, 0.92680E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.82588E+00_JPRB, 0.79400E+00_JPRB, 0.76106E+00_JPRB, 0.73986E+00_JPRB, 0.72220E+00_JPRB, 0.74115E+00_JPRB, 0.77266E+00_JPRB ,&
&  0.80113E+00_JPRB, 0.80397E+00_JPRB, 0.73442E+00_JPRB, 0.61575E+00_JPRB, 0.55250E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.52271E+00_JPRB, 0.52484E+00_JPRB, 0.52244E+00_JPRB, 0.53226E+00_JPRB, 0.53097E+00_JPRB, 0.53096E+00_JPRB, 0.52727E+00_JPRB ,&
&  0.55142E+00_JPRB, 0.59881E+00_JPRB, 0.67161E+00_JPRB, 0.75357E+00_JPRB, 0.81414E+00_JPRB, 0.88952E+00_JPRB, 0.92500E+00_JPRB ,&
&  0.91880E+00_JPRB, 0.89788E+00_JPRB, 0.89283E+00_JPRB, 0.91562E+00_JPRB, 0.93337E+00_JPRB, 0.96570E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.10913E+01_JPRB, 0.11577E+01_JPRB, 0.12344E+01_JPRB, 0.13256E+01_JPRB, 0.13845E+01_JPRB, 0.14163E+01_JPRB ,&
&  0.55290E+00_JPRB, 0.72996E+00_JPRB, 0.99775E+00_JPRB, 0.10405E+01_JPRB, 0.80851E+00_JPRB, 0.71585E+00_JPRB, 0.65185E+00_JPRB ,&
&  0.76203E+00_JPRB, 0.95039E+00_JPRB, 0.10921E+01_JPRB, 0.12527E+01_JPRB, 0.12744E+01_JPRB, 0.13712E+01_JPRB, 0.14545E+01_JPRB ,&
&  0.14782E+01_JPRB, 0.15078E+01_JPRB, 0.15770E+01_JPRB, 0.16633E+01_JPRB, 0.17372E+01_JPRB, 0.18198E+01_JPRB, 0.18996E+01_JPRB ,&
&  0.19883E+01_JPRB, 0.20071E+01_JPRB, 0.20686E+01_JPRB, 0.17344E+01_JPRB, 0.18817E+01_JPRB, 0.20778E+01_JPRB, 0.26627E+01_JPRB ,&
&  0.24697E+01_JPRB, 0.15125E+01_JPRB, 0.11935E+01_JPRB, 0.12340E+01_JPRB, 0.11098E+01_JPRB, 0.10219E+01_JPRB, 0.88301E+00_JPRB ,&
&  0.77258E+00_JPRB, 0.69192E+00_JPRB, 0.65344E+00_JPRB, 0.64099E+00_JPRB, 0.64697E+00_JPRB, 0.68447E+00_JPRB, 0.71771E+00_JPRB ,&
&  0.70618E+00_JPRB, 0.69009E+00_JPRB, 0.67753E+00_JPRB, 0.67462E+00_JPRB, 0.70678E+00_JPRB, 0.74211E+00_JPRB, 0.75775E+00_JPRB ,&
&  0.75997E+00_JPRB, 0.76514E+00_JPRB, 0.78563E+00_JPRB, 0.81217E+00_JPRB, 0.83994E+00_JPRB, 0.85646E+00_JPRB, 0.86962E+00_JPRB ,&
&  0.77118E+00_JPRB, 0.71861E+00_JPRB, 0.70861E+00_JPRB, 0.70630E+00_JPRB, 0.73561E+00_JPRB, 0.77258E+00_JPRB, 0.72810E+00_JPRB ,&
&  0.70000E+00_JPRB, 0.68692E+00_JPRB, 0.65692E+00_JPRB, 0.59278E+00_JPRB, 0.53288E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.53559E+00_JPRB, 0.54981E+00_JPRB, 0.56932E+00_JPRB, 0.59324E+00_JPRB, 0.59020E+00_JPRB, 0.59083E+00_JPRB, 0.57808E+00_JPRB ,&
&  0.57559E+00_JPRB, 0.59399E+00_JPRB, 0.64006E+00_JPRB, 0.71732E+00_JPRB, 0.77500E+00_JPRB, 0.85523E+00_JPRB, 0.96237E+00_JPRB ,&
&  0.99590E+00_JPRB, 0.97746E+00_JPRB, 0.96654E+00_JPRB, 0.97009E+00_JPRB, 0.99028E+00_JPRB, 0.10197E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11345E+01_JPRB, 0.11862E+01_JPRB, 0.12585E+01_JPRB, 0.13308E+01_JPRB, 0.13716E+01_JPRB, 0.14486E+01_JPRB ,&
&  0.80667E+00_JPRB, 0.10497E+01_JPRB, 0.10192E+01_JPRB, 0.98988E+00_JPRB, 0.84995E+00_JPRB, 0.76112E+00_JPRB, 0.76289E+00_JPRB ,&
&  0.85769E+00_JPRB, 0.95629E+00_JPRB, 0.11256E+01_JPRB, 0.11006E+01_JPRB, 0.12084E+01_JPRB, 0.12731E+01_JPRB, 0.14155E+01_JPRB ,&
&  0.14621E+01_JPRB, 0.15029E+01_JPRB, 0.15599E+01_JPRB, 0.16261E+01_JPRB, 0.16924E+01_JPRB, 0.17876E+01_JPRB, 0.18829E+01_JPRB ,&
&  0.19620E+01_JPRB, 0.19904E+01_JPRB, 0.20430E+01_JPRB, 0.18215E+01_JPRB, 0.16377E+01_JPRB, 0.16982E+01_JPRB, 0.16951E+01_JPRB ,&
&  0.15708E+01_JPRB, 0.11081E+01_JPRB, 0.10475E+01_JPRB, 0.11010E+01_JPRB, 0.10236E+01_JPRB, 0.94512E+00_JPRB, 0.85187E+00_JPRB ,&
&  0.75771E+00_JPRB, 0.67209E+00_JPRB, 0.62883E+00_JPRB, 0.58868E+00_JPRB, 0.57869E+00_JPRB, 0.60726E+00_JPRB, 0.64466E+00_JPRB ,&
&  0.63955E+00_JPRB, 0.62323E+00_JPRB, 0.61410E+00_JPRB, 0.62680E+00_JPRB, 0.66002E+00_JPRB, 0.68655E+00_JPRB, 0.70334E+00_JPRB ,&
&  0.67307E+00_JPRB, 0.63973E+00_JPRB, 0.63032E+00_JPRB, 0.64083E+00_JPRB, 0.67007E+00_JPRB, 0.70172E+00_JPRB, 0.73802E+00_JPRB ,&
&  0.75616E+00_JPRB, 0.75506E+00_JPRB, 0.84531E+00_JPRB, 0.10382E+01_JPRB, 0.11710E+01_JPRB, 0.12581E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.83191E+00_JPRB, 0.70019E+00_JPRB, 0.61891E+00_JPRB, 0.56533E+00_JPRB, 0.52433E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.53304E+00_JPRB, 0.57934E+00_JPRB, 0.61716E+00_JPRB, 0.67203E+00_JPRB, 0.69134E+00_JPRB, 0.69614E+00_JPRB, 0.69508E+00_JPRB ,&
&  0.68443E+00_JPRB, 0.69147E+00_JPRB, 0.71421E+00_JPRB, 0.76842E+00_JPRB, 0.82633E+00_JPRB, 0.88319E+00_JPRB, 0.94387E+00_JPRB ,&
&  0.10204E+01_JPRB, 0.10684E+01_JPRB, 0.10556E+01_JPRB, 0.10620E+01_JPRB, 0.10911E+01_JPRB, 0.11234E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11520E+01_JPRB, 0.11779E+01_JPRB, 0.12296E+01_JPRB, 0.12983E+01_JPRB, 0.13612E+01_JPRB, 0.97128E+00_JPRB ,&
&  0.10979E+01_JPRB, 0.10058E+01_JPRB, 0.97347E+00_JPRB, 0.97903E+00_JPRB, 0.93366E+00_JPRB, 0.86761E+00_JPRB, 0.84397E+00_JPRB ,&
&  0.86301E+00_JPRB, 0.90417E+00_JPRB, 0.95425E+00_JPRB, 0.98097E+00_JPRB, 0.10702E+01_JPRB, 0.10981E+01_JPRB, 0.12260E+01_JPRB ,&
&  0.13293E+01_JPRB, 0.13698E+01_JPRB, 0.13882E+01_JPRB, 0.14081E+01_JPRB, 0.14355E+01_JPRB, 0.14866E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.15811E+01_JPRB, 0.15880E+01_JPRB, 0.15806E+01_JPRB, 0.15767E+01_JPRB, 0.14231E+01_JPRB, 0.14395E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.83969E+00_JPRB, 0.68571E+00_JPRB, 0.86201E+00_JPRB, 0.10399E+01_JPRB, 0.96853E+00_JPRB, 0.90416E+00_JPRB, 0.81418E+00_JPRB ,&
&  0.71257E+00_JPRB, 0.66055E+00_JPRB, 0.62633E+00_JPRB, 0.59298E+00_JPRB, 0.57131E+00_JPRB, 0.57875E+00_JPRB, 0.59254E+00_JPRB ,&
&  0.59530E+00_JPRB, 0.58816E+00_JPRB, 0.58073E+00_JPRB, 0.58996E+00_JPRB, 0.61783E+00_JPRB, 0.71713E+00_JPRB, 0.89905E+00_JPRB ,&
&  0.10668E+01_JPRB, 0.53731E+00_JPRB, 0.50127E+00_JPRB, 0.52647E+00_JPRB, 0.58048E+00_JPRB, 0.63115E+00_JPRB, 0.69079E+00_JPRB ,&
&  0.84225E+00_JPRB, 0.10870E+01_JPRB, 0.10282E+01_JPRB, 0.12291E+01_JPRB, 0.25151E+01_JPRB, 0.27482E+01_JPRB, 0.15859E+01_JPRB ,&
&  0.99948E+00_JPRB, 0.70852E+00_JPRB, 0.57585E+00_JPRB, 0.53467E+00_JPRB, 0.52470E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.56270E+00_JPRB, 0.62193E+00_JPRB, 0.69077E+00_JPRB, 0.73614E+00_JPRB, 0.71409E+00_JPRB, 0.71154E+00_JPRB, 0.72718E+00_JPRB ,&
&  0.75034E+00_JPRB, 0.78246E+00_JPRB, 0.81663E+00_JPRB, 0.86142E+00_JPRB, 0.89315E+00_JPRB, 0.93346E+00_JPRB, 0.98776E+00_JPRB ,&
&  0.10509E+01_JPRB, 0.10642E+01_JPRB, 0.10863E+01_JPRB, 0.11109E+01_JPRB, 0.11312E+01_JPRB, 0.11541E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.11997E+01_JPRB, 0.12087E+01_JPRB, 0.12529E+01_JPRB, 0.12796E+01_JPRB, 0.14247E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.10520E+01_JPRB, 0.97196E+00_JPRB, 0.97628E+00_JPRB, 0.95910E+00_JPRB, 0.93640E+00_JPRB, 0.91364E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.91825E+00_JPRB, 0.88837E+00_JPRB, 0.92900E+00_JPRB, 0.92157E+00_JPRB, 0.93121E+00_JPRB, 0.10002E+01_JPRB ,&
&  0.10626E+01_JPRB, 0.11086E+01_JPRB, 0.11155E+01_JPRB, 0.11356E+01_JPRB, 0.11361E+01_JPRB, 0.11223E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10804E+01_JPRB, 0.10764E+01_JPRB, 0.10684E+01_JPRB, 0.10797E+01_JPRB, 0.10851E+01_JPRB, 0.95654E+00_JPRB, 0.77325E+00_JPRB ,&
&  0.59472E+00_JPRB, 0.58491E+00_JPRB, 0.88430E+00_JPRB, 0.11089E+01_JPRB, 0.98719E+00_JPRB, 0.92769E+00_JPRB, 0.81307E+00_JPRB ,&
&  0.74031E+00_JPRB, 0.66323E+00_JPRB, 0.63006E+00_JPRB, 0.58316E+00_JPRB, 0.56706E+00_JPRB, 0.57544E+00_JPRB, 0.60639E+00_JPRB ,&
&  0.61880E+00_JPRB, 0.60151E+00_JPRB, 0.58987E+00_JPRB, 0.56529E+00_JPRB, 0.58272E+00_JPRB, 0.62954E+00_JPRB, 0.58575E+00_JPRB ,&
&  0.61358E+00_JPRB, 0.48839E+00_JPRB, 0.49599E+00_JPRB, 0.56092E+00_JPRB, 0.52259E+00_JPRB, 0.57018E+00_JPRB, 0.63287E+00_JPRB ,&
&  0.81597E+00_JPRB, 0.89501E+00_JPRB, 0.12336E+01_JPRB, 0.34363E+01_JPRB, 0.39586E+01_JPRB, 0.49581E+01_JPRB, 0.19823E+01_JPRB ,&
&  0.92771E+00_JPRB, 0.62105E+00_JPRB, 0.55474E+00_JPRB, 0.55497E+00_JPRB, 0.54674E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.58511E+00_JPRB, 0.66202E+00_JPRB, 0.73014E+00_JPRB, 0.66060E+00_JPRB, 0.66920E+00_JPRB, 0.68939E+00_JPRB, 0.71962E+00_JPRB ,&
&  0.75411E+00_JPRB, 0.79554E+00_JPRB, 0.83921E+00_JPRB, 0.89065E+00_JPRB, 0.93169E+00_JPRB, 0.96958E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10693E+01_JPRB, 0.11041E+01_JPRB, 0.11113E+01_JPRB, 0.11361E+01_JPRB, 0.11386E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.12031E+01_JPRB, 0.12310E+01_JPRB, 0.12612E+01_JPRB, 0.12850E+01_JPRB, 0.12790E+01_JPRB, 0.13255E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.11057E+01_JPRB, 0.10189E+01_JPRB, 0.10064E+01_JPRB, 0.97191E+00_JPRB, 0.96194E+00_JPRB, 0.95048E+00_JPRB ,&
&  0.93784E+00_JPRB, 0.93519E+00_JPRB, 0.93043E+00_JPRB, 0.89904E+00_JPRB, 0.85395E+00_JPRB, 0.87430E+00_JPRB, 0.89619E+00_JPRB ,&
&  0.93449E+00_JPRB, 0.98136E+00_JPRB, 0.10154E+01_JPRB, 0.10373E+01_JPRB, 0.10490E+01_JPRB, 0.10376E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.10279E+01_JPRB, 0.95645E+00_JPRB, 0.96227E+00_JPRB, 0.10160E+01_JPRB, 0.87926E+00_JPRB, 0.69873E+00_JPRB ,&
&  0.60089E+00_JPRB, 0.77797E+00_JPRB, 0.10288E+01_JPRB, 0.12150E+01_JPRB, 0.10715E+01_JPRB, 0.97641E+00_JPRB, 0.89662E+00_JPRB ,&
&  0.79641E+00_JPRB, 0.71843E+00_JPRB, 0.67898E+00_JPRB, 0.61823E+00_JPRB, 0.59401E+00_JPRB, 0.58919E+00_JPRB, 0.59157E+00_JPRB ,&
&  0.58503E+00_JPRB, 0.57792E+00_JPRB, 0.57080E+00_JPRB, 0.56673E+00_JPRB, 0.56537E+00_JPRB, 0.54008E+00_JPRB, 0.84234E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.40263E+00_JPRB, 0.58967E+00_JPRB, 0.42960E+00_JPRB, 0.55641E+00_JPRB, 0.61693E+00_JPRB, 0.66221E+00_JPRB ,&
&  0.62016E+00_JPRB, 0.78155E+00_JPRB, 0.11355E+01_JPRB, 0.11675E+01_JPRB, 0.12268E+01_JPRB, 0.13012E+01_JPRB, 0.77932E+00_JPRB ,&
&  0.63105E+00_JPRB, 0.62004E+00_JPRB, 0.62234E+00_JPRB, 0.57422E+00_JPRB, 0.56800E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.60217E+00_JPRB, 0.64915E+00_JPRB, 0.74002E+00_JPRB, 0.67152E+00_JPRB, 0.66986E+00_JPRB, 0.68534E+00_JPRB, 0.70905E+00_JPRB ,&
&  0.73378E+00_JPRB, 0.76629E+00_JPRB, 0.80133E+00_JPRB, 0.84942E+00_JPRB, 0.89175E+00_JPRB, 0.92914E+00_JPRB, 0.96475E+00_JPRB ,&
&  0.99749E+00_JPRB, 0.10258E+01_JPRB, 0.10530E+01_JPRB, 0.10661E+01_JPRB, 0.10794E+01_JPRB, 0.11035E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.12167E+01_JPRB, 0.12499E+01_JPRB, 0.13029E+01_JPRB, 0.13380E+01_JPRB, 0.13672E+01_JPRB, 0.11487E+01_JPRB ,&
&  0.12748E+01_JPRB, 0.11487E+01_JPRB, 0.10762E+01_JPRB, 0.10330E+01_JPRB, 0.99069E+00_JPRB, 0.96213E+00_JPRB, 0.95816E+00_JPRB ,&
&  0.98676E+00_JPRB, 0.94847E+00_JPRB, 0.95147E+00_JPRB, 0.92763E+00_JPRB, 0.92973E+00_JPRB, 0.96840E+00_JPRB, 0.98767E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.11341E+01_JPRB, 0.12140E+01_JPRB, 0.12656E+01_JPRB, 0.13148E+01_JPRB, 0.13064E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.13386E+01_JPRB, 0.13873E+01_JPRB, 0.13390E+01_JPRB, 0.13510E+01_JPRB, 0.13289E+01_JPRB, 0.12235E+01_JPRB, 0.99685E+00_JPRB ,&
&  0.87294E+00_JPRB, 0.98899E+00_JPRB, 0.10338E+01_JPRB, 0.12184E+01_JPRB, 0.11311E+01_JPRB, 0.10240E+01_JPRB, 0.97580E+00_JPRB ,&
&  0.89789E+00_JPRB, 0.83140E+00_JPRB, 0.78691E+00_JPRB, 0.71743E+00_JPRB, 0.72060E+00_JPRB, 0.72607E+00_JPRB, 0.72747E+00_JPRB ,&
&  0.69119E+00_JPRB, 0.65845E+00_JPRB, 0.63201E+00_JPRB, 0.63701E+00_JPRB, 0.74687E+00_JPRB, 0.77693E+00_JPRB, 0.90127E+00_JPRB ,&
&  0.65111E+00_JPRB, 0.44272E+00_JPRB, 0.53953E+00_JPRB, 0.70400E+00_JPRB, 0.69581E+00_JPRB, 0.74227E+00_JPRB, 0.80186E+00_JPRB ,&
&  0.87398E+00_JPRB, 0.87045E+00_JPRB, 0.83925E+00_JPRB, 0.83473E+00_JPRB, 0.82583E+00_JPRB, 0.82329E+00_JPRB, 0.74223E+00_JPRB ,&
&  0.71362E+00_JPRB, 0.65688E+00_JPRB, 0.62466E+00_JPRB, 0.60968E+00_JPRB, 0.58010E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.66811E+00_JPRB, 0.66852E+00_JPRB, 0.72927E+00_JPRB, 0.68995E+00_JPRB, 0.68860E+00_JPRB, 0.70344E+00_JPRB, 0.72084E+00_JPRB ,&
&  0.72342E+00_JPRB, 0.73549E+00_JPRB, 0.75880E+00_JPRB, 0.79666E+00_JPRB, 0.83324E+00_JPRB, 0.86161E+00_JPRB, 0.89313E+00_JPRB ,&
&  0.91819E+00_JPRB, 0.93437E+00_JPRB, 0.94455E+00_JPRB, 0.95590E+00_JPRB, 0.98608E+00_JPRB, 0.10219E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.11434E+01_JPRB, 0.11667E+01_JPRB, 0.11725E+01_JPRB, 0.12054E+01_JPRB, 0.12345E+01_JPRB, 0.13348E+01_JPRB, 0.15222E+01_JPRB ,&
&  0.11576E+01_JPRB, 0.12024E+01_JPRB, 0.11234E+01_JPRB, 0.10361E+01_JPRB, 0.10452E+01_JPRB, 0.10089E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10687E+01_JPRB, 0.10952E+01_JPRB, 0.11138E+01_JPRB, 0.11329E+01_JPRB, 0.11687E+01_JPRB, 0.12071E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.13497E+01_JPRB, 0.14226E+01_JPRB, 0.15337E+01_JPRB, 0.16377E+01_JPRB, 0.17335E+01_JPRB, 0.17786E+01_JPRB ,&
&  0.18015E+01_JPRB, 0.18029E+01_JPRB, 0.16788E+01_JPRB, 0.16776E+01_JPRB, 0.16645E+01_JPRB, 0.15870E+01_JPRB, 0.14212E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.11939E+01_JPRB, 0.11912E+01_JPRB, 0.11422E+01_JPRB, 0.11352E+01_JPRB, 0.11191E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.10378E+01_JPRB, 0.97104E+00_JPRB, 0.91878E+00_JPRB, 0.86996E+00_JPRB, 0.86122E+00_JPRB, 0.87800E+00_JPRB, 0.87254E+00_JPRB ,&
&  0.83206E+00_JPRB, 0.79482E+00_JPRB, 0.77177E+00_JPRB, 0.77363E+00_JPRB, 0.79848E+00_JPRB, 0.10211E+01_JPRB, 0.94850E+00_JPRB ,&
&  0.88426E+00_JPRB, 0.80051E+00_JPRB, 0.69021E+00_JPRB, 0.92677E+00_JPRB, 0.86620E+00_JPRB, 0.85617E+00_JPRB, 0.88710E+00_JPRB ,&
&  0.89793E+00_JPRB, 0.89003E+00_JPRB, 0.91574E+00_JPRB, 0.87302E+00_JPRB, 0.83797E+00_JPRB, 0.81829E+00_JPRB, 0.81007E+00_JPRB ,&
&  0.75747E+00_JPRB, 0.70544E+00_JPRB, 0.65594E+00_JPRB, 0.65989E+00_JPRB, 0.62671E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.68454E+00_JPRB, 0.66592E+00_JPRB, 0.67151E+00_JPRB, 0.70970E+00_JPRB, 0.68652E+00_JPRB, 0.70566E+00_JPRB, 0.70000E+00_JPRB ,&
&  0.70032E+00_JPRB, 0.69676E+00_JPRB, 0.71023E+00_JPRB, 0.72624E+00_JPRB, 0.74915E+00_JPRB, 0.77007E+00_JPRB, 0.77807E+00_JPRB ,&
&  0.79083E+00_JPRB, 0.80477E+00_JPRB, 0.82792E+00_JPRB, 0.86819E+00_JPRB, 0.93772E+00_JPRB, 0.10504E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.16366E+01_JPRB, 0.16958E+01_JPRB, 0.17027E+01_JPRB, 0.18692E+01_JPRB, 0.22841E+01_JPRB, 0.25430E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.12818E+01_JPRB, 0.11485E+01_JPRB, 0.10266E+01_JPRB, 0.10452E+01_JPRB, 0.11283E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.11947E+01_JPRB, 0.12549E+01_JPRB, 0.13249E+01_JPRB, 0.13777E+01_JPRB, 0.14191E+01_JPRB, 0.14658E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.16038E+01_JPRB, 0.16994E+01_JPRB, 0.18066E+01_JPRB, 0.17907E+01_JPRB, 0.19528E+01_JPRB, 0.19807E+01_JPRB ,&
&  0.19770E+01_JPRB, 0.18663E+01_JPRB, 0.16637E+01_JPRB, 0.16761E+01_JPRB, 0.17583E+01_JPRB, 0.17377E+01_JPRB, 0.16215E+01_JPRB ,&
&  0.16240E+01_JPRB, 0.13675E+01_JPRB, 0.10644E+01_JPRB, 0.10762E+01_JPRB, 0.11156E+01_JPRB, 0.11977E+01_JPRB, 0.11731E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.11338E+01_JPRB, 0.10882E+01_JPRB, 0.10504E+01_JPRB, 0.10262E+01_JPRB, 0.10360E+01_JPRB, 0.10354E+01_JPRB ,&
&  0.94582E+00_JPRB, 0.91208E+00_JPRB, 0.86715E+00_JPRB, 0.86699E+00_JPRB, 0.90371E+00_JPRB, 0.83253E+00_JPRB, 0.96629E+00_JPRB ,&
&  0.10466E+01_JPRB, 0.11176E+01_JPRB, 0.10068E+01_JPRB, 0.11916E+01_JPRB, 0.96551E+00_JPRB, 0.90284E+00_JPRB, 0.91120E+00_JPRB ,&
&  0.93712E+00_JPRB, 0.89604E+00_JPRB, 0.87530E+00_JPRB, 0.85713E+00_JPRB, 0.84807E+00_JPRB, 0.82545E+00_JPRB, 0.77710E+00_JPRB ,&
&  0.74126E+00_JPRB, 0.71875E+00_JPRB, 0.70901E+00_JPRB, 0.69848E+00_JPRB, 0.69475E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.69160E+00_JPRB, 0.68923E+00_JPRB, 0.67991E+00_JPRB, 0.66207E+00_JPRB, 0.66348E+00_JPRB, 0.66826E+00_JPRB, 0.65562E+00_JPRB ,&
&  0.64804E+00_JPRB, 0.64339E+00_JPRB, 0.65847E+00_JPRB, 0.65997E+00_JPRB, 0.66307E+00_JPRB, 0.67768E+00_JPRB, 0.69676E+00_JPRB ,&
&  0.71689E+00_JPRB, 0.73529E+00_JPRB, 0.80810E+00_JPRB, 0.95862E+00_JPRB, 0.12366E+01_JPRB, 0.16037E+01_JPRB, 0.18856E+01_JPRB ,&
&  0.20577E+01_JPRB, 0.22018E+01_JPRB, 0.23236E+01_JPRB, 0.24283E+01_JPRB, 0.22716E+01_JPRB, 0.22920E+01_JPRB, 0.24358E+01_JPRB ,&
&  0.17588E+01_JPRB, 0.12875E+01_JPRB, 0.12968E+01_JPRB, 0.11608E+01_JPRB, 0.11489E+01_JPRB, 0.11752E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12612E+01_JPRB, 0.13251E+01_JPRB, 0.14183E+01_JPRB, 0.15043E+01_JPRB, 0.15586E+01_JPRB, 0.15901E+01_JPRB ,&
&  0.16117E+01_JPRB, 0.16462E+01_JPRB, 0.17115E+01_JPRB, 0.16880E+01_JPRB, 0.16574E+01_JPRB, 0.17159E+01_JPRB, 0.17362E+01_JPRB ,&
&  0.17423E+01_JPRB, 0.17255E+01_JPRB, 0.15448E+01_JPRB, 0.15400E+01_JPRB, 0.16650E+01_JPRB, 0.15662E+01_JPRB, 0.15394E+01_JPRB ,&
&  0.16143E+01_JPRB, 0.14139E+01_JPRB, 0.97545E+00_JPRB, 0.96046E+00_JPRB, 0.10516E+01_JPRB, 0.11789E+01_JPRB, 0.13057E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.13018E+01_JPRB, 0.12597E+01_JPRB, 0.12494E+01_JPRB, 0.12095E+01_JPRB, 0.11816E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.10359E+01_JPRB, 0.99421E+00_JPRB, 0.95664E+00_JPRB, 0.93426E+00_JPRB, 0.94914E+00_JPRB, 0.95954E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.11197E+01_JPRB, 0.11872E+01_JPRB, 0.13241E+01_JPRB, 0.11254E+01_JPRB, 0.95261E+00_JPRB, 0.92778E+00_JPRB, 0.90992E+00_JPRB ,&
&  0.92747E+00_JPRB, 0.90314E+00_JPRB, 0.88044E+00_JPRB, 0.85821E+00_JPRB, 0.84040E+00_JPRB, 0.81666E+00_JPRB, 0.78547E+00_JPRB ,&
&  0.75586E+00_JPRB, 0.73426E+00_JPRB, 0.72096E+00_JPRB, 0.70636E+00_JPRB, 0.69076E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.69495E+00_JPRB, 0.69175E+00_JPRB, 0.68806E+00_JPRB, 0.66752E+00_JPRB, 0.63868E+00_JPRB, 0.62577E+00_JPRB, 0.61906E+00_JPRB ,&
&  0.62445E+00_JPRB, 0.64526E+00_JPRB, 0.65764E+00_JPRB, 0.65781E+00_JPRB, 0.65728E+00_JPRB, 0.67302E+00_JPRB, 0.71626E+00_JPRB ,&
&  0.77415E+00_JPRB, 0.85040E+00_JPRB, 0.99346E+00_JPRB, 0.12929E+01_JPRB, 0.16658E+01_JPRB, 0.19694E+01_JPRB, 0.19400E+01_JPRB ,&
&  0.18290E+01_JPRB, 0.18373E+01_JPRB, 0.18394E+01_JPRB, 0.18682E+01_JPRB, 0.18930E+01_JPRB, 0.19621E+01_JPRB, 0.20267E+01_JPRB ,&
&  0.18009E+01_JPRB, 0.14533E+01_JPRB, 0.12922E+01_JPRB, 0.11977E+01_JPRB, 0.11756E+01_JPRB, 0.11725E+01_JPRB, 0.11902E+01_JPRB ,&
&  0.12101E+01_JPRB, 0.12461E+01_JPRB, 0.13037E+01_JPRB, 0.13980E+01_JPRB, 0.15015E+01_JPRB, 0.15648E+01_JPRB, 0.15873E+01_JPRB ,&
&  0.15886E+01_JPRB, 0.15864E+01_JPRB, 0.16089E+01_JPRB, 0.14716E+01_JPRB, 0.15107E+01_JPRB, 0.15489E+01_JPRB, 0.15694E+01_JPRB ,&
&  0.15395E+01_JPRB, 0.14687E+01_JPRB, 0.15437E+01_JPRB, 0.15503E+01_JPRB, 0.15520E+01_JPRB, 0.15191E+01_JPRB, 0.14883E+01_JPRB ,&
&  0.15147E+01_JPRB, 0.13957E+01_JPRB, 0.10314E+01_JPRB, 0.10441E+01_JPRB, 0.11212E+01_JPRB, 0.10754E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.13564E+01_JPRB, 0.13764E+01_JPRB, 0.14113E+01_JPRB, 0.14502E+01_JPRB, 0.14714E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.10981E+01_JPRB, 0.10802E+01_JPRB, 0.10667E+01_JPRB, 0.10855E+01_JPRB, 0.10762E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.11144E+01_JPRB, 0.11526E+01_JPRB, 0.11696E+01_JPRB, 0.11248E+01_JPRB, 0.10077E+01_JPRB, 0.90470E+00_JPRB, 0.90392E+00_JPRB ,&
&  0.89572E+00_JPRB, 0.86348E+00_JPRB, 0.85503E+00_JPRB, 0.84253E+00_JPRB, 0.83293E+00_JPRB, 0.82167E+00_JPRB, 0.80907E+00_JPRB ,&
&  0.78807E+00_JPRB, 0.76702E+00_JPRB, 0.74553E+00_JPRB, 0.72149E+00_JPRB, 0.70236E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.71082E+00_JPRB, 0.70523E+00_JPRB, 0.70185E+00_JPRB, 0.69482E+00_JPRB, 0.66640E+00_JPRB, 0.65101E+00_JPRB, 0.64958E+00_JPRB ,&
&  0.66406E+00_JPRB, 0.68613E+00_JPRB, 0.70395E+00_JPRB, 0.71574E+00_JPRB, 0.74484E+00_JPRB, 0.76312E+00_JPRB, 0.79518E+00_JPRB ,&
&  0.85592E+00_JPRB, 0.95597E+00_JPRB, 0.11466E+01_JPRB, 0.14540E+01_JPRB, 0.17464E+01_JPRB, 0.20111E+01_JPRB, 0.19034E+01_JPRB ,&
&  0.17992E+01_JPRB, 0.17437E+01_JPRB, 0.17169E+01_JPRB, 0.16784E+01_JPRB, 0.14926E+01_JPRB, 0.16225E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.16476E+01_JPRB, 0.14968E+01_JPRB, 0.12488E+01_JPRB, 0.11755E+01_JPRB, 0.11303E+01_JPRB, 0.11320E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.12151E+01_JPRB, 0.12678E+01_JPRB, 0.13376E+01_JPRB, 0.14210E+01_JPRB, 0.15014E+01_JPRB, 0.15396E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.15067E+01_JPRB, 0.14856E+01_JPRB, 0.14147E+01_JPRB, 0.13952E+01_JPRB, 0.14155E+01_JPRB, 0.14389E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.14141E+01_JPRB, 0.14526E+01_JPRB, 0.15057E+01_JPRB, 0.14477E+01_JPRB, 0.13601E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.15056E+01_JPRB, 0.11159E+01_JPRB, 0.12997E+01_JPRB, 0.15025E+01_JPRB, 0.13280E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.13713E+01_JPRB, 0.14188E+01_JPRB, 0.14931E+01_JPRB, 0.15741E+01_JPRB, 0.16548E+01_JPRB, 0.17126E+01_JPRB, 0.13782E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.12171E+01_JPRB, 0.11958E+01_JPRB, 0.12053E+01_JPRB, 0.12363E+01_JPRB, 0.12287E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.12706E+01_JPRB, 0.14011E+01_JPRB, 0.15437E+01_JPRB, 0.14971E+01_JPRB, 0.90923E+00_JPRB, 0.92319E+00_JPRB ,&
&  0.90849E+00_JPRB, 0.88120E+00_JPRB, 0.86987E+00_JPRB, 0.85989E+00_JPRB, 0.85405E+00_JPRB, 0.84543E+00_JPRB, 0.83904E+00_JPRB ,&
&  0.82398E+00_JPRB, 0.80055E+00_JPRB, 0.77528E+00_JPRB, 0.75036E+00_JPRB, 0.72778E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.73868E+00_JPRB, 0.72714E+00_JPRB, 0.72109E+00_JPRB, 0.71111E+00_JPRB, 0.70610E+00_JPRB, 0.69379E+00_JPRB, 0.69362E+00_JPRB ,&
&  0.69487E+00_JPRB, 0.71135E+00_JPRB, 0.72640E+00_JPRB, 0.75765E+00_JPRB, 0.79437E+00_JPRB, 0.86037E+00_JPRB, 0.92598E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.11126E+01_JPRB, 0.13147E+01_JPRB, 0.15793E+01_JPRB, 0.17979E+01_JPRB, 0.19204E+01_JPRB, 0.18562E+01_JPRB ,&
&  0.17652E+01_JPRB, 0.17666E+01_JPRB, 0.17191E+01_JPRB, 0.13621E+01_JPRB, 0.13223E+01_JPRB, 0.16294E+01_JPRB, 0.15421E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.12373E+01_JPRB, 0.11570E+01_JPRB, 0.10971E+01_JPRB, 0.10552E+01_JPRB, 0.10659E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11732E+01_JPRB, 0.12277E+01_JPRB, 0.12856E+01_JPRB, 0.13465E+01_JPRB, 0.14324E+01_JPRB, 0.14884E+01_JPRB ,&
&  0.14841E+01_JPRB, 0.14687E+01_JPRB, 0.14475E+01_JPRB, 0.14128E+01_JPRB, 0.13526E+01_JPRB, 0.13455E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.13776E+01_JPRB, 0.13851E+01_JPRB, 0.13976E+01_JPRB, 0.14452E+01_JPRB, 0.14625E+01_JPRB, 0.13113E+01_JPRB, 0.12896E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.14700E+01_JPRB, 0.13301E+01_JPRB, 0.15392E+01_JPRB, 0.10050E+01_JPRB, 0.11301E+01_JPRB, 0.13204E+01_JPRB ,&
&  0.13396E+01_JPRB, 0.13977E+01_JPRB, 0.14740E+01_JPRB, 0.15763E+01_JPRB, 0.16636E+01_JPRB, 0.15217E+01_JPRB, 0.15517E+01_JPRB ,&
&  0.15017E+01_JPRB, 0.13642E+01_JPRB, 0.13369E+01_JPRB, 0.13202E+01_JPRB, 0.13633E+01_JPRB, 0.13856E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.13224E+01_JPRB, 0.14538E+01_JPRB, 0.15039E+01_JPRB, 0.12604E+01_JPRB, 0.97417E+00_JPRB, 0.94457E+00_JPRB ,&
&  0.93797E+00_JPRB, 0.90338E+00_JPRB, 0.88578E+00_JPRB, 0.87443E+00_JPRB, 0.86904E+00_JPRB, 0.85842E+00_JPRB, 0.84898E+00_JPRB ,&
&  0.83782E+00_JPRB, 0.81712E+00_JPRB, 0.79781E+00_JPRB, 0.77712E+00_JPRB, 0.75602E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.75688E+00_JPRB, 0.75158E+00_JPRB, 0.75184E+00_JPRB, 0.72986E+00_JPRB, 0.72426E+00_JPRB, 0.73341E+00_JPRB, 0.72157E+00_JPRB ,&
&  0.69894E+00_JPRB, 0.67393E+00_JPRB, 0.68848E+00_JPRB, 0.72724E+00_JPRB, 0.77396E+00_JPRB, 0.84794E+00_JPRB, 0.94350E+00_JPRB ,&
&  0.10662E+01_JPRB, 0.11857E+01_JPRB, 0.13800E+01_JPRB, 0.15704E+01_JPRB, 0.17189E+01_JPRB, 0.18278E+01_JPRB, 0.18549E+01_JPRB ,&
&  0.14951E+01_JPRB, 0.12106E+01_JPRB, 0.17142E+01_JPRB, 0.18668E+01_JPRB, 0.18041E+01_JPRB, 0.17735E+01_JPRB, 0.17944E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.12363E+01_JPRB, 0.12191E+01_JPRB, 0.11184E+01_JPRB, 0.99430E+00_JPRB, 0.99524E+00_JPRB, 0.10228E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.11155E+01_JPRB, 0.11753E+01_JPRB, 0.12492E+01_JPRB, 0.13119E+01_JPRB, 0.13856E+01_JPRB, 0.14392E+01_JPRB ,&
&  0.14512E+01_JPRB, 0.14445E+01_JPRB, 0.14425E+01_JPRB, 0.13964E+01_JPRB, 0.13718E+01_JPRB, 0.13563E+01_JPRB, 0.13342E+01_JPRB ,&
&  0.13434E+01_JPRB, 0.13391E+01_JPRB, 0.13870E+01_JPRB, 0.14274E+01_JPRB, 0.14943E+01_JPRB, 0.13774E+01_JPRB, 0.14259E+01_JPRB ,&
&  0.14008E+01_JPRB, 0.14491E+01_JPRB, 0.14401E+01_JPRB, 0.15583E+01_JPRB, 0.10914E+01_JPRB, 0.12166E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.14019E+01_JPRB, 0.14249E+01_JPRB, 0.14673E+01_JPRB, 0.15260E+01_JPRB, 0.16231E+01_JPRB, 0.15323E+01_JPRB, 0.15896E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.15807E+01_JPRB, 0.14755E+01_JPRB, 0.14673E+01_JPRB, 0.14020E+01_JPRB, 0.15273E+01_JPRB, 0.14849E+01_JPRB ,&
&  0.15285E+01_JPRB, 0.14630E+01_JPRB, 0.14351E+01_JPRB, 0.14264E+01_JPRB, 0.12665E+01_JPRB, 0.10998E+01_JPRB, 0.10225E+01_JPRB ,&
&  0.98361E+00_JPRB, 0.95367E+00_JPRB, 0.91802E+00_JPRB, 0.89263E+00_JPRB, 0.87932E+00_JPRB, 0.86574E+00_JPRB, 0.85096E+00_JPRB ,&
&  0.83308E+00_JPRB, 0.81441E+00_JPRB, 0.79776E+00_JPRB, 0.77832E+00_JPRB, 0.76550E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.75662E+00_JPRB, 0.76277E+00_JPRB, 0.77637E+00_JPRB, 0.78626E+00_JPRB, 0.87914E+00_JPRB, 0.10013E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10279E+01_JPRB, 0.66367E+00_JPRB, 0.65482E+00_JPRB, 0.67974E+00_JPRB, 0.71658E+00_JPRB, 0.78075E+00_JPRB, 0.85948E+00_JPRB ,&
&  0.97107E+00_JPRB, 0.11048E+01_JPRB, 0.13041E+01_JPRB, 0.15404E+01_JPRB, 0.16843E+01_JPRB, 0.18114E+01_JPRB, 0.16870E+01_JPRB ,&
&  0.11837E+01_JPRB, 0.12665E+01_JPRB, 0.18783E+01_JPRB, 0.17990E+01_JPRB, 0.16902E+01_JPRB, 0.16370E+01_JPRB, 0.15577E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.11517E+01_JPRB, 0.11411E+01_JPRB, 0.10871E+01_JPRB, 0.95702E+00_JPRB, 0.92299E+00_JPRB, 0.92949E+00_JPRB ,&
&  0.97040E+00_JPRB, 0.10308E+01_JPRB, 0.11052E+01_JPRB, 0.11851E+01_JPRB, 0.12828E+01_JPRB, 0.13461E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.14351E+01_JPRB, 0.14408E+01_JPRB, 0.14116E+01_JPRB, 0.13645E+01_JPRB, 0.13890E+01_JPRB, 0.14042E+01_JPRB, 0.13964E+01_JPRB ,&
&  0.14006E+01_JPRB, 0.14772E+01_JPRB, 0.15119E+01_JPRB, 0.14238E+01_JPRB, 0.12985E+01_JPRB, 0.14299E+01_JPRB, 0.13879E+01_JPRB ,&
&  0.14138E+01_JPRB, 0.13882E+01_JPRB, 0.15934E+01_JPRB, 0.12546E+01_JPRB, 0.12565E+01_JPRB, 0.13654E+01_JPRB, 0.13770E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.14283E+01_JPRB, 0.14722E+01_JPRB, 0.15043E+01_JPRB, 0.15490E+01_JPRB, 0.14383E+01_JPRB, 0.15854E+01_JPRB ,&
&  0.15228E+01_JPRB, 0.17215E+01_JPRB, 0.16314E+01_JPRB, 0.15699E+01_JPRB, 0.14025E+01_JPRB, 0.14687E+01_JPRB, 0.15727E+01_JPRB ,&
&  0.17482E+01_JPRB, 0.17057E+01_JPRB, 0.16202E+01_JPRB, 0.14921E+01_JPRB, 0.13119E+01_JPRB, 0.12315E+01_JPRB, 0.11659E+01_JPRB ,&
&  0.10830E+01_JPRB, 0.10092E+01_JPRB, 0.96890E+00_JPRB, 0.92466E+00_JPRB, 0.89738E+00_JPRB, 0.87303E+00_JPRB, 0.85052E+00_JPRB ,&
&  0.82973E+00_JPRB, 0.81399E+00_JPRB, 0.79829E+00_JPRB, 0.77486E+00_JPRB, 0.75884E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.75251E+00_JPRB, 0.74835E+00_JPRB, 0.75316E+00_JPRB, 0.76967E+00_JPRB, 0.86096E+00_JPRB, 0.89732E+00_JPRB, 0.89747E+00_JPRB ,&
&  0.87618E+00_JPRB, 0.63764E+00_JPRB, 0.61721E+00_JPRB, 0.63789E+00_JPRB, 0.67860E+00_JPRB, 0.72800E+00_JPRB, 0.80538E+00_JPRB ,&
&  0.88305E+00_JPRB, 0.10046E+01_JPRB, 0.12034E+01_JPRB, 0.14278E+01_JPRB, 0.15799E+01_JPRB, 0.16207E+01_JPRB, 0.14315E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.18974E+01_JPRB, 0.19121E+01_JPRB, 0.19091E+01_JPRB, 0.18462E+01_JPRB, 0.17240E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.11467E+01_JPRB, 0.10726E+01_JPRB, 0.10290E+01_JPRB, 0.97356E+00_JPRB, 0.88701E+00_JPRB, 0.85817E+00_JPRB ,&
&  0.88390E+00_JPRB, 0.95214E+00_JPRB, 0.10334E+01_JPRB, 0.11162E+01_JPRB, 0.12246E+01_JPRB, 0.13272E+01_JPRB, 0.13863E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.14452E+01_JPRB, 0.13851E+01_JPRB, 0.13914E+01_JPRB, 0.14153E+01_JPRB, 0.14706E+01_JPRB, 0.14990E+01_JPRB ,&
&  0.14410E+01_JPRB, 0.14009E+01_JPRB, 0.15754E+01_JPRB, 0.16014E+01_JPRB, 0.15088E+01_JPRB, 0.15291E+01_JPRB, 0.13790E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.13567E+01_JPRB, 0.14772E+01_JPRB, 0.12475E+01_JPRB, 0.12967E+01_JPRB, 0.14420E+01_JPRB, 0.14200E+01_JPRB ,&
&  0.12888E+01_JPRB, 0.13958E+01_JPRB, 0.13876E+01_JPRB, 0.14344E+01_JPRB, 0.14937E+01_JPRB, 0.14816E+01_JPRB, 0.14433E+01_JPRB ,&
&  0.14944E+01_JPRB, 0.16095E+01_JPRB, 0.17705E+01_JPRB, 0.15267E+01_JPRB, 0.14627E+01_JPRB, 0.14807E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.20075E+01_JPRB, 0.19676E+01_JPRB, 0.20249E+01_JPRB, 0.18320E+01_JPRB, 0.14994E+01_JPRB, 0.13758E+01_JPRB, 0.12909E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.11242E+01_JPRB, 0.10479E+01_JPRB, 0.96383E+00_JPRB, 0.89380E+00_JPRB, 0.86287E+00_JPRB, 0.84822E+00_JPRB ,&
&  0.83397E+00_JPRB, 0.82330E+00_JPRB, 0.80782E+00_JPRB, 0.78203E+00_JPRB, 0.76196E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.76420E+00_JPRB, 0.76346E+00_JPRB, 0.76572E+00_JPRB, 0.76823E+00_JPRB, 0.81035E+00_JPRB, 0.82166E+00_JPRB, 0.75840E+00_JPRB ,&
&  0.69019E+00_JPRB, 0.58646E+00_JPRB, 0.57733E+00_JPRB, 0.59622E+00_JPRB, 0.63115E+00_JPRB, 0.67334E+00_JPRB, 0.74351E+00_JPRB ,&
&  0.83262E+00_JPRB, 0.91559E+00_JPRB, 0.11082E+01_JPRB, 0.12879E+01_JPRB, 0.14455E+01_JPRB, 0.16140E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.18478E+01_JPRB, 0.19206E+01_JPRB, 0.19425E+01_JPRB, 0.19292E+01_JPRB, 0.19019E+01_JPRB, 0.15725E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.12551E+01_JPRB, 0.11110E+01_JPRB, 0.99141E+00_JPRB, 0.95157E+00_JPRB, 0.91373E+00_JPRB, 0.83958E+00_JPRB ,&
&  0.83096E+00_JPRB, 0.88375E+00_JPRB, 0.97043E+00_JPRB, 0.10637E+01_JPRB, 0.11744E+01_JPRB, 0.13033E+01_JPRB, 0.13809E+01_JPRB ,&
&  0.14493E+01_JPRB, 0.14614E+01_JPRB, 0.14458E+01_JPRB, 0.13644E+01_JPRB, 0.14354E+01_JPRB, 0.14954E+01_JPRB, 0.15565E+01_JPRB ,&
&  0.14815E+01_JPRB, 0.14501E+01_JPRB, 0.17278E+01_JPRB, 0.19440E+01_JPRB, 0.19326E+01_JPRB, 0.17621E+01_JPRB, 0.15604E+01_JPRB ,&
&  0.15282E+01_JPRB, 0.14601E+01_JPRB, 0.13292E+01_JPRB, 0.12206E+01_JPRB, 0.13420E+01_JPRB, 0.14728E+01_JPRB, 0.14925E+01_JPRB ,&
&  0.14263E+01_JPRB, 0.13909E+01_JPRB, 0.13378E+01_JPRB, 0.12958E+01_JPRB, 0.13384E+01_JPRB, 0.13391E+01_JPRB, 0.14049E+01_JPRB ,&
&  0.15499E+01_JPRB, 0.17202E+01_JPRB, 0.17660E+01_JPRB, 0.15341E+01_JPRB, 0.14843E+01_JPRB, 0.14219E+01_JPRB, 0.17030E+01_JPRB ,&
&  0.21905E+01_JPRB, 0.22078E+01_JPRB, 0.21352E+01_JPRB, 0.21217E+01_JPRB, 0.17609E+01_JPRB, 0.15956E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.13715E+01_JPRB, 0.12411E+01_JPRB, 0.11517E+01_JPRB, 0.10697E+01_JPRB, 0.97678E+00_JPRB, 0.91568E+00_JPRB, 0.89426E+00_JPRB ,&
&  0.86879E+00_JPRB, 0.85462E+00_JPRB, 0.83389E+00_JPRB, 0.80147E+00_JPRB, 0.77676E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.77267E+00_JPRB, 0.76085E+00_JPRB, 0.76407E+00_JPRB, 0.76007E+00_JPRB, 0.74592E+00_JPRB, 0.72753E+00_JPRB, 0.66313E+00_JPRB ,&
&  0.59959E+00_JPRB, 0.56133E+00_JPRB, 0.54656E+00_JPRB, 0.55365E+00_JPRB, 0.58639E+00_JPRB, 0.63388E+00_JPRB, 0.67818E+00_JPRB ,&
&  0.77536E+00_JPRB, 0.86185E+00_JPRB, 0.99910E+00_JPRB, 0.11559E+01_JPRB, 0.11761E+01_JPRB, 0.10243E+01_JPRB, 0.10397E+01_JPRB ,&
&  0.15062E+01_JPRB, 0.20038E+01_JPRB, 0.19635E+01_JPRB, 0.19672E+01_JPRB, 0.20213E+01_JPRB, 0.20996E+01_JPRB, 0.17515E+01_JPRB ,&
&  0.15220E+01_JPRB, 0.13660E+01_JPRB, 0.11895E+01_JPRB, 0.10655E+01_JPRB, 0.96856E+00_JPRB, 0.94196E+00_JPRB, 0.88556E+00_JPRB ,&
&  0.82552E+00_JPRB, 0.85325E+00_JPRB, 0.95484E+00_JPRB, 0.10616E+01_JPRB, 0.11730E+01_JPRB, 0.12915E+01_JPRB, 0.14070E+01_JPRB ,&
&  0.14540E+01_JPRB, 0.14547E+01_JPRB, 0.14355E+01_JPRB, 0.13843E+01_JPRB, 0.14319E+01_JPRB, 0.15069E+01_JPRB, 0.16737E+01_JPRB ,&
&  0.17601E+01_JPRB, 0.19239E+01_JPRB, 0.20238E+01_JPRB, 0.21238E+01_JPRB, 0.21253E+01_JPRB, 0.20760E+01_JPRB, 0.19022E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.15798E+01_JPRB, 0.12423E+01_JPRB, 0.12584E+01_JPRB, 0.14535E+01_JPRB, 0.15834E+01_JPRB, 0.14080E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.10724E+01_JPRB, 0.10607E+01_JPRB, 0.11186E+01_JPRB, 0.12456E+01_JPRB, 0.12804E+01_JPRB, 0.14193E+01_JPRB ,&
&  0.14242E+01_JPRB, 0.96416E+00_JPRB, 0.85461E+00_JPRB, 0.10510E+01_JPRB, 0.96908E+00_JPRB, 0.10733E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.23473E+01_JPRB, 0.27211E+01_JPRB, 0.26442E+01_JPRB, 0.22002E+01_JPRB, 0.20530E+01_JPRB, 0.19014E+01_JPRB, 0.17948E+01_JPRB ,&
&  0.15709E+01_JPRB, 0.13928E+01_JPRB, 0.12737E+01_JPRB, 0.11952E+01_JPRB, 0.11087E+01_JPRB, 0.10292E+01_JPRB, 0.98286E+00_JPRB ,&
&  0.94780E+00_JPRB, 0.91229E+00_JPRB, 0.88166E+00_JPRB, 0.83564E+00_JPRB, 0.79481E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.77295E+00_JPRB, 0.74951E+00_JPRB, 0.75007E+00_JPRB, 0.74509E+00_JPRB, 0.71025E+00_JPRB, 0.66112E+00_JPRB, 0.61477E+00_JPRB ,&
&  0.58007E+00_JPRB, 0.55558E+00_JPRB, 0.53623E+00_JPRB, 0.53385E+00_JPRB, 0.54993E+00_JPRB, 0.59847E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.73755E+00_JPRB, 0.85571E+00_JPRB, 0.96675E+00_JPRB, 0.10813E+01_JPRB, 0.10149E+01_JPRB, 0.85381E+00_JPRB, 0.99277E+00_JPRB ,&
&  0.13098E+01_JPRB, 0.18007E+01_JPRB, 0.20520E+01_JPRB, 0.20678E+01_JPRB, 0.20466E+01_JPRB, 0.20380E+01_JPRB, 0.18953E+01_JPRB ,&
&  0.16791E+01_JPRB, 0.14471E+01_JPRB, 0.12609E+01_JPRB, 0.11370E+01_JPRB, 0.10192E+01_JPRB, 0.93588E+00_JPRB, 0.91467E+00_JPRB ,&
&  0.87073E+00_JPRB, 0.83300E+00_JPRB, 0.91472E+00_JPRB, 0.10490E+01_JPRB, 0.11588E+01_JPRB, 0.12714E+01_JPRB, 0.13774E+01_JPRB ,&
&  0.14202E+01_JPRB, 0.14398E+01_JPRB, 0.14481E+01_JPRB, 0.14646E+01_JPRB, 0.12182E+01_JPRB, 0.14879E+01_JPRB, 0.16002E+01_JPRB ,&
&  0.17941E+01_JPRB, 0.20103E+01_JPRB, 0.20863E+01_JPRB, 0.24086E+01_JPRB, 0.23650E+01_JPRB, 0.22420E+01_JPRB, 0.21094E+01_JPRB ,&
&  0.20420E+01_JPRB, 0.18443E+01_JPRB, 0.15025E+01_JPRB, 0.14823E+01_JPRB, 0.15935E+01_JPRB, 0.16733E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.97706E+00_JPRB, 0.12374E+01_JPRB, 0.11267E+01_JPRB, 0.98863E+00_JPRB, 0.11693E+01_JPRB, 0.12511E+01_JPRB, 0.96473E+00_JPRB ,&
&  0.57394E+00_JPRB, 0.49265E+00_JPRB, 0.50562E+00_JPRB, 0.53296E+00_JPRB, 0.53710E+00_JPRB, 0.80433E+00_JPRB, 0.22929E+01_JPRB ,&
&  0.25155E+01_JPRB, 0.29352E+01_JPRB, 0.27477E+01_JPRB, 0.24053E+01_JPRB, 0.21857E+01_JPRB, 0.20736E+01_JPRB, 0.19798E+01_JPRB ,&
&  0.17297E+01_JPRB, 0.15863E+01_JPRB, 0.14516E+01_JPRB, 0.13624E+01_JPRB, 0.12777E+01_JPRB, 0.11878E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.98398E+00_JPRB, 0.93190E+00_JPRB, 0.86733E+00_JPRB, 0.80949E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.76094E+00_JPRB, 0.72322E+00_JPRB, 0.70726E+00_JPRB, 0.69054E+00_JPRB, 0.68169E+00_JPRB, 0.62831E+00_JPRB, 0.61490E+00_JPRB ,&
&  0.57856E+00_JPRB, 0.56581E+00_JPRB, 0.53584E+00_JPRB, 0.52229E+00_JPRB, 0.52270E+00_JPRB, 0.56641E+00_JPRB, 0.63900E+00_JPRB ,&
&  0.71596E+00_JPRB, 0.83888E+00_JPRB, 0.10598E+01_JPRB, 0.10204E+01_JPRB, 0.10036E+01_JPRB, 0.90557E+00_JPRB, 0.86171E+00_JPRB ,&
&  0.10842E+01_JPRB, 0.15261E+01_JPRB, 0.17602E+01_JPRB, 0.18784E+01_JPRB, 0.21157E+01_JPRB, 0.21607E+01_JPRB, 0.20574E+01_JPRB ,&
&  0.19116E+01_JPRB, 0.16449E+01_JPRB, 0.14275E+01_JPRB, 0.12647E+01_JPRB, 0.11206E+01_JPRB, 0.99348E+00_JPRB, 0.91445E+00_JPRB ,&
&  0.90314E+00_JPRB, 0.86146E+00_JPRB, 0.88450E+00_JPRB, 0.10104E+01_JPRB, 0.11457E+01_JPRB, 0.12310E+01_JPRB, 0.13147E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.14106E+01_JPRB, 0.14209E+01_JPRB, 0.14179E+01_JPRB, 0.12134E+01_JPRB, 0.11106E+01_JPRB, 0.14474E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.21030E+01_JPRB, 0.23520E+01_JPRB, 0.27274E+01_JPRB, 0.24724E+01_JPRB, 0.23691E+01_JPRB, 0.23886E+01_JPRB ,&
&  0.22288E+01_JPRB, 0.24456E+01_JPRB, 0.18002E+01_JPRB, 0.16957E+01_JPRB, 0.17736E+01_JPRB, 0.13337E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.14209E+01_JPRB, 0.13492E+01_JPRB, 0.11312E+01_JPRB, 0.90063E+00_JPRB, 0.11099E+01_JPRB, 0.10164E+01_JPRB, 0.56987E+00_JPRB ,&
&  0.48365E+00_JPRB, 0.46259E+00_JPRB, 0.44421E+00_JPRB, 0.43622E+00_JPRB, 0.45460E+00_JPRB, 0.67873E+00_JPRB, 0.13035E+01_JPRB ,&
&  0.19218E+01_JPRB, 0.27308E+01_JPRB, 0.28266E+01_JPRB, 0.26105E+01_JPRB, 0.23392E+01_JPRB, 0.22682E+01_JPRB, 0.21723E+01_JPRB ,&
&  0.18745E+01_JPRB, 0.17046E+01_JPRB, 0.15448E+01_JPRB, 0.14567E+01_JPRB, 0.13818E+01_JPRB, 0.13084E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.10542E+01_JPRB, 0.97809E+00_JPRB, 0.88662E+00_JPRB, 0.81419E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.74548E+00_JPRB, 0.69666E+00_JPRB, 0.65117E+00_JPRB, 0.64585E+00_JPRB, 0.66470E+00_JPRB, 0.62415E+00_JPRB, 0.60484E+00_JPRB ,&
&  0.58299E+00_JPRB, 0.56918E+00_JPRB, 0.55075E+00_JPRB, 0.53163E+00_JPRB, 0.52178E+00_JPRB, 0.54955E+00_JPRB, 0.60517E+00_JPRB ,&
&  0.68635E+00_JPRB, 0.80117E+00_JPRB, 0.96488E+00_JPRB, 0.84184E+00_JPRB, 0.75455E+00_JPRB, 0.67748E+00_JPRB, 0.63135E+00_JPRB ,&
&  0.91100E+00_JPRB, 0.12318E+01_JPRB, 0.14853E+01_JPRB, 0.17648E+01_JPRB, 0.20839E+01_JPRB, 0.21241E+01_JPRB, 0.22952E+01_JPRB ,&
&  0.22474E+01_JPRB, 0.19183E+01_JPRB, 0.16390E+01_JPRB, 0.14054E+01_JPRB, 0.12387E+01_JPRB, 0.10802E+01_JPRB, 0.96261E+00_JPRB ,&
&  0.90295E+00_JPRB, 0.88067E+00_JPRB, 0.86719E+00_JPRB, 0.95018E+00_JPRB, 0.10894E+01_JPRB, 0.11843E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13643E+01_JPRB, 0.13930E+01_JPRB, 0.14384E+01_JPRB, 0.15264E+01_JPRB, 0.14871E+01_JPRB, 0.13826E+01_JPRB ,&
&  0.13873E+01_JPRB, 0.15853E+01_JPRB, 0.23346E+01_JPRB, 0.25370E+01_JPRB, 0.24628E+01_JPRB, 0.25533E+01_JPRB, 0.25741E+01_JPRB ,&
&  0.24498E+01_JPRB, 0.25077E+01_JPRB, 0.20308E+01_JPRB, 0.20367E+01_JPRB, 0.17564E+01_JPRB, 0.12445E+01_JPRB, 0.15137E+01_JPRB ,&
&  0.16362E+01_JPRB, 0.15456E+01_JPRB, 0.12005E+01_JPRB, 0.87530E+00_JPRB, 0.84898E+00_JPRB, 0.61485E+00_JPRB, 0.46461E+00_JPRB ,&
&  0.45130E+00_JPRB, 0.44135E+00_JPRB, 0.41333E+00_JPRB, 0.44459E+00_JPRB, 0.46076E+00_JPRB, 0.64488E+00_JPRB, 0.10528E+01_JPRB ,&
&  0.17056E+01_JPRB, 0.23684E+01_JPRB, 0.27185E+01_JPRB, 0.26319E+01_JPRB, 0.24155E+01_JPRB, 0.23286E+01_JPRB, 0.24053E+01_JPRB ,&
&  0.20016E+01_JPRB, 0.18472E+01_JPRB, 0.16516E+01_JPRB, 0.15426E+01_JPRB, 0.14436E+01_JPRB, 0.13551E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.10532E+01_JPRB, 0.97593E+00_JPRB, 0.88219E+00_JPRB, 0.80816E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.72464E+00_JPRB, 0.67303E+00_JPRB, 0.61454E+00_JPRB, 0.61486E+00_JPRB, 0.64431E+00_JPRB, 0.62835E+00_JPRB, 0.59553E+00_JPRB ,&
&  0.58763E+00_JPRB, 0.56423E+00_JPRB, 0.55243E+00_JPRB, 0.53724E+00_JPRB, 0.52589E+00_JPRB, 0.53724E+00_JPRB, 0.58218E+00_JPRB ,&
&  0.67995E+00_JPRB, 0.82033E+00_JPRB, 0.72263E+00_JPRB, 0.64208E+00_JPRB, 0.63137E+00_JPRB, 0.62223E+00_JPRB, 0.54907E+00_JPRB ,&
&  0.80009E+00_JPRB, 0.10235E+01_JPRB, 0.12347E+01_JPRB, 0.16561E+01_JPRB, 0.23626E+01_JPRB, 0.24783E+01_JPRB, 0.25487E+01_JPRB ,&
&  0.25180E+01_JPRB, 0.23298E+01_JPRB, 0.19554E+01_JPRB, 0.16028E+01_JPRB, 0.13537E+01_JPRB, 0.11899E+01_JPRB, 0.10257E+01_JPRB ,&
&  0.90820E+00_JPRB, 0.87943E+00_JPRB, 0.88469E+00_JPRB, 0.91164E+00_JPRB, 0.99844E+00_JPRB, 0.10794E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.12278E+01_JPRB, 0.13057E+01_JPRB, 0.13437E+01_JPRB, 0.13633E+01_JPRB, 0.13619E+01_JPRB, 0.12704E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.17989E+01_JPRB, 0.19034E+01_JPRB, 0.19564E+01_JPRB, 0.25140E+01_JPRB, 0.26540E+01_JPRB, 0.25297E+01_JPRB, 0.29275E+01_JPRB ,&
&  0.21419E+01_JPRB, 0.18682E+01_JPRB, 0.18676E+01_JPRB, 0.17876E+01_JPRB, 0.14108E+01_JPRB, 0.15475E+01_JPRB, 0.22533E+01_JPRB ,&
&  0.17243E+01_JPRB, 0.18783E+01_JPRB, 0.19429E+01_JPRB, 0.19481E+01_JPRB, 0.88320E+00_JPRB, 0.57881E+00_JPRB, 0.97512E+00_JPRB ,&
&  0.11143E+01_JPRB, 0.74092E+00_JPRB, 0.56384E+00_JPRB, 0.74379E+00_JPRB, 0.59767E+00_JPRB, 0.73114E+00_JPRB, 0.11154E+01_JPRB ,&
&  0.13821E+01_JPRB, 0.17240E+01_JPRB, 0.24947E+01_JPRB, 0.25928E+01_JPRB, 0.24411E+01_JPRB, 0.24420E+01_JPRB, 0.23883E+01_JPRB ,&
&  0.22699E+01_JPRB, 0.19654E+01_JPRB, 0.18601E+01_JPRB, 0.16248E+01_JPRB, 0.14392E+01_JPRB, 0.13203E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.11225E+01_JPRB, 0.10313E+01_JPRB, 0.94491E+00_JPRB, 0.85642E+00_JPRB, 0.78329E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.67670E+00_JPRB, 0.63664E+00_JPRB, 0.58196E+00_JPRB, 0.58233E+00_JPRB, 0.61070E+00_JPRB, 0.61947E+00_JPRB, 0.59751E+00_JPRB ,&
&  0.57883E+00_JPRB, 0.55419E+00_JPRB, 0.54514E+00_JPRB, 0.53637E+00_JPRB, 0.53012E+00_JPRB, 0.53874E+00_JPRB, 0.56035E+00_JPRB ,&
&  0.62712E+00_JPRB, 0.64660E+00_JPRB, 0.57654E+00_JPRB, 0.53843E+00_JPRB, 0.62829E+00_JPRB, 0.58657E+00_JPRB, 0.58825E+00_JPRB ,&
&  0.74565E+00_JPRB, 0.89007E+00_JPRB, 0.10614E+01_JPRB, 0.13841E+01_JPRB, 0.17571E+01_JPRB, 0.23485E+01_JPRB, 0.26370E+01_JPRB ,&
&  0.26633E+01_JPRB, 0.26038E+01_JPRB, 0.23051E+01_JPRB, 0.19241E+01_JPRB, 0.15582E+01_JPRB, 0.13298E+01_JPRB, 0.11541E+01_JPRB ,&
&  0.10010E+01_JPRB, 0.89342E+00_JPRB, 0.87344E+00_JPRB, 0.90328E+00_JPRB, 0.96295E+00_JPRB, 0.10069E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.11763E+01_JPRB, 0.12320E+01_JPRB, 0.12298E+01_JPRB, 0.11598E+01_JPRB, 0.10379E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.16487E+01_JPRB, 0.17437E+01_JPRB, 0.19926E+01_JPRB, 0.24197E+01_JPRB, 0.28350E+01_JPRB, 0.26592E+01_JPRB, 0.33802E+01_JPRB ,&
&  0.27127E+01_JPRB, 0.22076E+01_JPRB, 0.18355E+01_JPRB, 0.13228E+01_JPRB, 0.16151E+01_JPRB, 0.24189E+01_JPRB, 0.24730E+01_JPRB ,&
&  0.23366E+01_JPRB, 0.22754E+01_JPRB, 0.22023E+01_JPRB, 0.18474E+01_JPRB, 0.14331E+01_JPRB, 0.11090E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.14460E+01_JPRB, 0.14164E+01_JPRB, 0.13256E+01_JPRB, 0.11421E+01_JPRB, 0.93215E+00_JPRB, 0.10936E+01_JPRB, 0.11573E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.13215E+01_JPRB, 0.17327E+01_JPRB, 0.22366E+01_JPRB, 0.23939E+01_JPRB, 0.23868E+01_JPRB, 0.25391E+01_JPRB ,&
&  0.22629E+01_JPRB, 0.21059E+01_JPRB, 0.18330E+01_JPRB, 0.15754E+01_JPRB, 0.13994E+01_JPRB, 0.12782E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.10769E+01_JPRB, 0.99181E+00_JPRB, 0.90755E+00_JPRB, 0.80621E+00_JPRB, 0.72503E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.64188E+00_JPRB, 0.60417E+00_JPRB, 0.55234E+00_JPRB, 0.54347E+00_JPRB, 0.56655E+00_JPRB, 0.59489E+00_JPRB, 0.60100E+00_JPRB ,&
&  0.57474E+00_JPRB, 0.55683E+00_JPRB, 0.54373E+00_JPRB, 0.54177E+00_JPRB, 0.53942E+00_JPRB, 0.53507E+00_JPRB, 0.54875E+00_JPRB ,&
&  0.59722E+00_JPRB, 0.57825E+00_JPRB, 0.50939E+00_JPRB, 0.55631E+00_JPRB, 0.60006E+00_JPRB, 0.55657E+00_JPRB, 0.62884E+00_JPRB ,&
&  0.72101E+00_JPRB, 0.82960E+00_JPRB, 0.91924E+00_JPRB, 0.10995E+01_JPRB, 0.13431E+01_JPRB, 0.17116E+01_JPRB, 0.21375E+01_JPRB ,&
&  0.22906E+01_JPRB, 0.24391E+01_JPRB, 0.24963E+01_JPRB, 0.22947E+01_JPRB, 0.18916E+01_JPRB, 0.15598E+01_JPRB, 0.13668E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.99989E+00_JPRB, 0.90881E+00_JPRB, 0.89763E+00_JPRB, 0.93899E+00_JPRB, 0.99430E+00_JPRB, 0.10534E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.10486E+01_JPRB, 0.10819E+01_JPRB, 0.12118E+01_JPRB, 0.98972E+00_JPRB, 0.99560E+00_JPRB, 0.10933E+01_JPRB ,&
&  0.17408E+01_JPRB, 0.19942E+01_JPRB, 0.21908E+01_JPRB, 0.27771E+01_JPRB, 0.24881E+01_JPRB, 0.27819E+01_JPRB, 0.34926E+01_JPRB ,&
&  0.37774E+01_JPRB, 0.37846E+01_JPRB, 0.37447E+01_JPRB, 0.23292E+01_JPRB, 0.20102E+01_JPRB, 0.27431E+01_JPRB, 0.26647E+01_JPRB ,&
&  0.24524E+01_JPRB, 0.24386E+01_JPRB, 0.22933E+01_JPRB, 0.23590E+01_JPRB, 0.17835E+01_JPRB, 0.15106E+01_JPRB, 0.11334E+01_JPRB ,&
&  0.10786E+01_JPRB, 0.95889E+00_JPRB, 0.11354E+01_JPRB, 0.10441E+01_JPRB, 0.10404E+01_JPRB, 0.10339E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.11288E+01_JPRB, 0.12504E+01_JPRB, 0.16643E+01_JPRB, 0.20873E+01_JPRB, 0.19882E+01_JPRB, 0.22059E+01_JPRB ,&
&  0.21655E+01_JPRB, 0.19398E+01_JPRB, 0.16407E+01_JPRB, 0.14769E+01_JPRB, 0.13400E+01_JPRB, 0.12254E+01_JPRB, 0.11156E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.91981E+00_JPRB, 0.83924E+00_JPRB, 0.75644E+00_JPRB, 0.68601E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.63721E+00_JPRB, 0.60175E+00_JPRB, 0.54138E+00_JPRB, 0.52681E+00_JPRB, 0.53355E+00_JPRB, 0.56030E+00_JPRB, 0.57749E+00_JPRB ,&
&  0.57129E+00_JPRB, 0.55374E+00_JPRB, 0.54326E+00_JPRB, 0.54050E+00_JPRB, 0.53781E+00_JPRB, 0.53676E+00_JPRB, 0.55759E+00_JPRB ,&
&  0.59435E+00_JPRB, 0.64721E+00_JPRB, 0.56599E+00_JPRB, 0.51606E+00_JPRB, 0.48416E+00_JPRB, 0.54719E+00_JPRB, 0.64638E+00_JPRB ,&
&  0.75039E+00_JPRB, 0.82096E+00_JPRB, 0.91214E+00_JPRB, 0.99096E+00_JPRB, 0.10404E+01_JPRB, 0.12325E+01_JPRB, 0.16170E+01_JPRB ,&
&  0.22650E+01_JPRB, 0.22369E+01_JPRB, 0.23891E+01_JPRB, 0.22572E+01_JPRB, 0.20295E+01_JPRB, 0.17770E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.13422E+01_JPRB, 0.11385E+01_JPRB, 0.99017E+00_JPRB, 0.90414E+00_JPRB, 0.88933E+00_JPRB, 0.90574E+00_JPRB, 0.97622E+00_JPRB ,&
&  0.10171E+01_JPRB, 0.10097E+01_JPRB, 0.97828E+00_JPRB, 0.10094E+01_JPRB, 0.11225E+01_JPRB, 0.12101E+01_JPRB, 0.13879E+01_JPRB ,&
&  0.15030E+01_JPRB, 0.16275E+01_JPRB, 0.24198E+01_JPRB, 0.26764E+01_JPRB, 0.33407E+01_JPRB, 0.36304E+01_JPRB, 0.35459E+01_JPRB ,&
&  0.36481E+01_JPRB, 0.35291E+01_JPRB, 0.34805E+01_JPRB, 0.28491E+01_JPRB, 0.32763E+01_JPRB, 0.31773E+01_JPRB, 0.29139E+01_JPRB ,&
&  0.26672E+01_JPRB, 0.24952E+01_JPRB, 0.23959E+01_JPRB, 0.23537E+01_JPRB, 0.23778E+01_JPRB, 0.24316E+01_JPRB, 0.18605E+01_JPRB ,&
&  0.13934E+01_JPRB, 0.14955E+01_JPRB, 0.13283E+01_JPRB, 0.10982E+01_JPRB, 0.91123E+00_JPRB, 0.94893E+00_JPRB, 0.99216E+00_JPRB ,&
&  0.10301E+01_JPRB, 0.10485E+01_JPRB, 0.11362E+01_JPRB, 0.14277E+01_JPRB, 0.18607E+01_JPRB, 0.20369E+01_JPRB, 0.20451E+01_JPRB ,&
&  0.18342E+01_JPRB, 0.18161E+01_JPRB, 0.16618E+01_JPRB, 0.14244E+01_JPRB, 0.12764E+01_JPRB, 0.11490E+01_JPRB, 0.10265E+01_JPRB ,&
&  0.92082E+00_JPRB, 0.84876E+00_JPRB, 0.78848E+00_JPRB, 0.72912E+00_JPRB, 0.67012E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.62090E+00_JPRB, 0.59910E+00_JPRB, 0.55144E+00_JPRB, 0.53321E+00_JPRB, 0.53232E+00_JPRB, 0.54317E+00_JPRB, 0.55449E+00_JPRB ,&
&  0.55705E+00_JPRB, 0.53879E+00_JPRB, 0.52691E+00_JPRB, 0.52719E+00_JPRB, 0.53342E+00_JPRB, 0.54521E+00_JPRB, 0.57997E+00_JPRB ,&
&  0.65172E+00_JPRB, 0.79680E+00_JPRB, 0.65760E+00_JPRB, 0.62587E+00_JPRB, 0.55779E+00_JPRB, 0.60965E+00_JPRB, 0.69469E+00_JPRB ,&
&  0.81700E+00_JPRB, 0.83188E+00_JPRB, 0.81141E+00_JPRB, 0.84321E+00_JPRB, 0.88056E+00_JPRB, 0.98048E+00_JPRB, 0.12344E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.19131E+01_JPRB, 0.20264E+01_JPRB, 0.21850E+01_JPRB, 0.19905E+01_JPRB, 0.16753E+01_JPRB, 0.16213E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.12516E+01_JPRB, 0.10803E+01_JPRB, 0.98146E+00_JPRB, 0.93074E+00_JPRB, 0.89447E+00_JPRB, 0.90696E+00_JPRB ,&
&  0.93337E+00_JPRB, 0.95037E+00_JPRB, 0.94448E+00_JPRB, 0.93498E+00_JPRB, 0.10086E+01_JPRB, 0.12144E+01_JPRB, 0.13888E+01_JPRB ,&
&  0.19549E+01_JPRB, 0.20650E+01_JPRB, 0.18269E+01_JPRB, 0.26170E+01_JPRB, 0.38857E+01_JPRB, 0.37933E+01_JPRB, 0.32619E+01_JPRB ,&
&  0.33991E+01_JPRB, 0.33444E+01_JPRB, 0.33947E+01_JPRB, 0.34766E+01_JPRB, 0.34598E+01_JPRB, 0.33128E+01_JPRB, 0.30740E+01_JPRB ,&
&  0.27703E+01_JPRB, 0.25713E+01_JPRB, 0.24317E+01_JPRB, 0.24393E+01_JPRB, 0.23136E+01_JPRB, 0.22126E+01_JPRB, 0.21316E+01_JPRB ,&
&  0.17723E+01_JPRB, 0.14422E+01_JPRB, 0.96820E+00_JPRB, 0.10048E+01_JPRB, 0.78720E+00_JPRB, 0.81220E+00_JPRB, 0.97668E+00_JPRB ,&
&  0.98879E+00_JPRB, 0.11006E+01_JPRB, 0.12309E+01_JPRB, 0.13154E+01_JPRB, 0.16667E+01_JPRB, 0.18065E+01_JPRB, 0.18505E+01_JPRB ,&
&  0.17876E+01_JPRB, 0.15099E+01_JPRB, 0.15088E+01_JPRB, 0.13685E+01_JPRB, 0.12769E+01_JPRB, 0.11061E+01_JPRB, 0.93954E+00_JPRB ,&
&  0.82514E+00_JPRB, 0.76129E+00_JPRB, 0.71875E+00_JPRB, 0.67700E+00_JPRB, 0.63915E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.56847E+00_JPRB, 0.55199E+00_JPRB, 0.52469E+00_JPRB, 0.51886E+00_JPRB, 0.51601E+00_JPRB, 0.51726E+00_JPRB, 0.53035E+00_JPRB ,&
&  0.54377E+00_JPRB, 0.53273E+00_JPRB, 0.51108E+00_JPRB, 0.50464E+00_JPRB, 0.51172E+00_JPRB, 0.53647E+00_JPRB, 0.60593E+00_JPRB ,&
&  0.74825E+00_JPRB, 0.79940E+00_JPRB, 0.68586E+00_JPRB, 0.56912E+00_JPRB, 0.64745E+00_JPRB, 0.63199E+00_JPRB, 0.69488E+00_JPRB ,&
&  0.67373E+00_JPRB, 0.73965E+00_JPRB, 0.71192E+00_JPRB, 0.69768E+00_JPRB, 0.69928E+00_JPRB, 0.76726E+00_JPRB, 0.89987E+00_JPRB ,&
&  0.10650E+01_JPRB, 0.13295E+01_JPRB, 0.15824E+01_JPRB, 0.15897E+01_JPRB, 0.16532E+01_JPRB, 0.14854E+01_JPRB, 0.14574E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.11879E+01_JPRB, 0.10450E+01_JPRB, 0.95741E+00_JPRB, 0.90769E+00_JPRB, 0.87724E+00_JPRB, 0.87626E+00_JPRB ,&
&  0.89960E+00_JPRB, 0.91171E+00_JPRB, 0.89714E+00_JPRB, 0.89908E+00_JPRB, 0.98835E+00_JPRB, 0.12442E+01_JPRB, 0.16643E+01_JPRB ,&
&  0.22763E+01_JPRB, 0.25961E+01_JPRB, 0.27968E+01_JPRB, 0.33139E+01_JPRB, 0.34884E+01_JPRB, 0.34852E+01_JPRB, 0.33561E+01_JPRB ,&
&  0.31659E+01_JPRB, 0.29832E+01_JPRB, 0.30410E+01_JPRB, 0.30857E+01_JPRB, 0.31464E+01_JPRB, 0.32182E+01_JPRB, 0.30962E+01_JPRB ,&
&  0.27409E+01_JPRB, 0.25328E+01_JPRB, 0.24496E+01_JPRB, 0.22565E+01_JPRB, 0.21691E+01_JPRB, 0.20150E+01_JPRB, 0.20655E+01_JPRB ,&
&  0.21050E+01_JPRB, 0.12043E+01_JPRB, 0.93427E+00_JPRB, 0.11061E+01_JPRB, 0.92856E+00_JPRB, 0.97771E+00_JPRB, 0.11738E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.12168E+01_JPRB, 0.13676E+01_JPRB, 0.14240E+01_JPRB, 0.14962E+01_JPRB, 0.16945E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.14920E+01_JPRB, 0.13509E+01_JPRB, 0.13581E+01_JPRB, 0.12682E+01_JPRB, 0.11454E+01_JPRB, 0.94550E+00_JPRB, 0.80462E+00_JPRB ,&
&  0.72252E+00_JPRB, 0.67802E+00_JPRB, 0.64599E+00_JPRB, 0.61395E+00_JPRB, 0.58137E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.53577E+00_JPRB, 0.51715E+00_JPRB, 0.49856E+00_JPRB, 0.49863E+00_JPRB, 0.50701E+00_JPRB, 0.50852E+00_JPRB, 0.52158E+00_JPRB ,&
&  0.53474E+00_JPRB, 0.53322E+00_JPRB, 0.51487E+00_JPRB, 0.50100E+00_JPRB, 0.49996E+00_JPRB, 0.52512E+00_JPRB, 0.59585E+00_JPRB ,&
&  0.55297E+00_JPRB, 0.58401E+00_JPRB, 0.52161E+00_JPRB, 0.54391E+00_JPRB, 0.65563E+00_JPRB, 0.59084E+00_JPRB, 0.58151E+00_JPRB ,&
&  0.56243E+00_JPRB, 0.55446E+00_JPRB, 0.53189E+00_JPRB, 0.52301E+00_JPRB, 0.53192E+00_JPRB, 0.57635E+00_JPRB, 0.66551E+00_JPRB ,&
&  0.73006E+00_JPRB, 0.87824E+00_JPRB, 0.10303E+01_JPRB, 0.10289E+01_JPRB, 0.11280E+01_JPRB, 0.11965E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.11940E+01_JPRB, 0.10625E+01_JPRB, 0.94248E+00_JPRB, 0.87217E+00_JPRB, 0.82856E+00_JPRB, 0.80909E+00_JPRB, 0.79664E+00_JPRB ,&
&  0.78872E+00_JPRB, 0.79472E+00_JPRB, 0.81151E+00_JPRB, 0.85948E+00_JPRB, 0.93941E+00_JPRB, 0.13046E+01_JPRB, 0.19126E+01_JPRB ,&
&  0.24091E+01_JPRB, 0.25870E+01_JPRB, 0.28326E+01_JPRB, 0.32436E+01_JPRB, 0.32278E+01_JPRB, 0.30444E+01_JPRB, 0.27519E+01_JPRB ,&
&  0.25862E+01_JPRB, 0.24521E+01_JPRB, 0.26150E+01_JPRB, 0.28317E+01_JPRB, 0.28294E+01_JPRB, 0.30949E+01_JPRB, 0.30412E+01_JPRB ,&
&  0.27636E+01_JPRB, 0.25929E+01_JPRB, 0.24824E+01_JPRB, 0.24446E+01_JPRB, 0.23616E+01_JPRB, 0.24411E+01_JPRB, 0.24161E+01_JPRB ,&
&  0.23810E+01_JPRB, 0.20765E+01_JPRB, 0.15610E+01_JPRB, 0.14453E+01_JPRB, 0.11065E+01_JPRB, 0.11482E+01_JPRB, 0.14692E+01_JPRB ,&
&  0.14658E+01_JPRB, 0.14512E+01_JPRB, 0.14335E+01_JPRB, 0.14304E+01_JPRB, 0.13893E+01_JPRB, 0.15165E+01_JPRB, 0.14817E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.12626E+01_JPRB, 0.11993E+01_JPRB, 0.10668E+01_JPRB, 0.94433E+00_JPRB, 0.78981E+00_JPRB, 0.68661E+00_JPRB ,&
&  0.65221E+00_JPRB, 0.61786E+00_JPRB, 0.59985E+00_JPRB, 0.57572E+00_JPRB, 0.55016E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.51926E+00_JPRB, 0.50320E+00_JPRB, 0.48919E+00_JPRB, 0.48676E+00_JPRB, 0.49917E+00_JPRB, 0.50402E+00_JPRB, 0.51685E+00_JPRB ,&
&  0.52393E+00_JPRB, 0.52264E+00_JPRB, 0.51864E+00_JPRB, 0.50145E+00_JPRB, 0.49829E+00_JPRB, 0.51443E+00_JPRB, 0.50909E+00_JPRB ,&
&  0.46323E+00_JPRB, 0.48888E+00_JPRB, 0.47863E+00_JPRB, 0.48654E+00_JPRB, 0.49269E+00_JPRB, 0.46894E+00_JPRB, 0.45333E+00_JPRB ,&
&  0.46781E+00_JPRB, 0.45922E+00_JPRB, 0.46332E+00_JPRB, 0.47751E+00_JPRB, 0.49786E+00_JPRB, 0.53360E+00_JPRB, 0.58506E+00_JPRB ,&
&  0.60693E+00_JPRB, 0.68061E+00_JPRB, 0.75487E+00_JPRB, 0.74418E+00_JPRB, 0.84931E+00_JPRB, 0.95806E+00_JPRB, 0.99323E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.94035E+00_JPRB, 0.86304E+00_JPRB, 0.80026E+00_JPRB, 0.74343E+00_JPRB, 0.71450E+00_JPRB, 0.70470E+00_JPRB ,&
&  0.70054E+00_JPRB, 0.69993E+00_JPRB, 0.70728E+00_JPRB, 0.73527E+00_JPRB, 0.79411E+00_JPRB, 0.10752E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.16151E+01_JPRB, 0.17677E+01_JPRB, 0.19131E+01_JPRB, 0.19942E+01_JPRB, 0.20548E+01_JPRB, 0.20583E+01_JPRB, 0.19904E+01_JPRB ,&
&  0.20876E+01_JPRB, 0.20668E+01_JPRB, 0.26107E+01_JPRB, 0.27110E+01_JPRB, 0.27501E+01_JPRB, 0.28933E+01_JPRB, 0.28661E+01_JPRB ,&
&  0.28584E+01_JPRB, 0.26721E+01_JPRB, 0.26143E+01_JPRB, 0.25242E+01_JPRB, 0.24530E+01_JPRB, 0.25097E+01_JPRB, 0.24093E+01_JPRB ,&
&  0.24103E+01_JPRB, 0.23897E+01_JPRB, 0.21103E+01_JPRB, 0.18916E+01_JPRB, 0.16199E+01_JPRB, 0.17187E+01_JPRB, 0.16156E+01_JPRB ,&
&  0.14952E+01_JPRB, 0.12332E+01_JPRB, 0.13075E+01_JPRB, 0.13278E+01_JPRB, 0.13365E+01_JPRB, 0.12898E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.11775E+01_JPRB, 0.10015E+01_JPRB, 0.85397E+00_JPRB, 0.76374E+00_JPRB, 0.68240E+00_JPRB, 0.57761E+00_JPRB ,&
&  0.54766E+00_JPRB, 0.56715E+00_JPRB, 0.56185E+00_JPRB, 0.55028E+00_JPRB, 0.53502E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.51206E+00_JPRB, 0.50362E+00_JPRB, 0.49415E+00_JPRB, 0.48683E+00_JPRB, 0.49745E+00_JPRB, 0.50074E+00_JPRB, 0.49744E+00_JPRB ,&
&  0.51330E+00_JPRB, 0.50663E+00_JPRB, 0.50409E+00_JPRB, 0.49550E+00_JPRB, 0.48918E+00_JPRB, 0.45126E+00_JPRB, 0.38150E+00_JPRB ,&
&  0.36574E+00_JPRB, 0.38386E+00_JPRB, 0.41689E+00_JPRB, 0.43263E+00_JPRB, 0.42570E+00_JPRB, 0.41113E+00_JPRB, 0.40249E+00_JPRB ,&
&  0.40785E+00_JPRB, 0.41852E+00_JPRB, 0.44377E+00_JPRB, 0.46763E+00_JPRB, 0.48143E+00_JPRB, 0.49479E+00_JPRB, 0.52070E+00_JPRB ,&
&  0.53075E+00_JPRB, 0.57033E+00_JPRB, 0.63396E+00_JPRB, 0.58758E+00_JPRB, 0.66501E+00_JPRB, 0.73889E+00_JPRB, 0.74657E+00_JPRB ,&
&  0.77664E+00_JPRB, 0.76434E+00_JPRB, 0.72594E+00_JPRB, 0.70440E+00_JPRB, 0.66513E+00_JPRB, 0.62911E+00_JPRB, 0.61159E+00_JPRB ,&
&  0.60575E+00_JPRB, 0.60671E+00_JPRB, 0.61671E+00_JPRB, 0.63162E+00_JPRB, 0.67159E+00_JPRB, 0.82205E+00_JPRB, 0.95850E+00_JPRB ,&
&  0.10315E+01_JPRB, 0.11428E+01_JPRB, 0.13041E+01_JPRB, 0.13075E+01_JPRB, 0.13950E+01_JPRB, 0.15347E+01_JPRB, 0.15950E+01_JPRB ,&
&  0.17973E+01_JPRB, 0.18163E+01_JPRB, 0.20393E+01_JPRB, 0.22064E+01_JPRB, 0.23669E+01_JPRB, 0.24569E+01_JPRB, 0.25201E+01_JPRB ,&
&  0.26737E+01_JPRB, 0.27406E+01_JPRB, 0.25030E+01_JPRB, 0.23518E+01_JPRB, 0.22220E+01_JPRB, 0.22080E+01_JPRB, 0.21442E+01_JPRB ,&
&  0.21600E+01_JPRB, 0.22107E+01_JPRB, 0.21983E+01_JPRB, 0.21440E+01_JPRB, 0.20301E+01_JPRB, 0.18656E+01_JPRB, 0.17019E+01_JPRB ,&
&  0.16022E+01_JPRB, 0.14147E+01_JPRB, 0.13083E+01_JPRB, 0.13040E+01_JPRB, 0.12899E+01_JPRB, 0.11133E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.93466E+00_JPRB, 0.86241E+00_JPRB, 0.75529E+00_JPRB, 0.70274E+00_JPRB, 0.65068E+00_JPRB, 0.57841E+00_JPRB ,&
&  0.52427E+00_JPRB, 0.55458E+00_JPRB, 0.54496E+00_JPRB, 0.52654E+00_JPRB, 0.51808E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.51338E+00_JPRB, 0.51213E+00_JPRB, 0.49988E+00_JPRB, 0.48914E+00_JPRB, 0.49104E+00_JPRB, 0.47614E+00_JPRB, 0.47204E+00_JPRB ,&
&  0.47998E+00_JPRB, 0.46699E+00_JPRB, 0.48729E+00_JPRB, 0.41990E+00_JPRB, 0.38200E+00_JPRB, 0.37423E+00_JPRB, 0.36201E+00_JPRB ,&
&  0.36256E+00_JPRB, 0.37502E+00_JPRB, 0.38513E+00_JPRB, 0.39348E+00_JPRB, 0.40054E+00_JPRB, 0.39148E+00_JPRB, 0.38768E+00_JPRB ,&
&  0.39467E+00_JPRB, 0.41329E+00_JPRB, 0.44376E+00_JPRB, 0.45739E+00_JPRB, 0.46130E+00_JPRB, 0.46212E+00_JPRB, 0.46849E+00_JPRB ,&
&  0.46177E+00_JPRB, 0.48429E+00_JPRB, 0.53356E+00_JPRB, 0.53824E+00_JPRB, 0.56491E+00_JPRB, 0.61593E+00_JPRB, 0.61206E+00_JPRB ,&
&  0.61833E+00_JPRB, 0.55058E+00_JPRB, 0.57828E+00_JPRB, 0.58342E+00_JPRB, 0.57838E+00_JPRB, 0.55818E+00_JPRB, 0.53793E+00_JPRB ,&
&  0.52706E+00_JPRB, 0.52675E+00_JPRB, 0.53400E+00_JPRB, 0.54614E+00_JPRB, 0.56352E+00_JPRB, 0.60827E+00_JPRB, 0.65002E+00_JPRB ,&
&  0.71263E+00_JPRB, 0.68184E+00_JPRB, 0.83542E+00_JPRB, 0.89361E+00_JPRB, 0.10328E+01_JPRB, 0.12400E+01_JPRB, 0.14023E+01_JPRB ,&
&  0.17533E+01_JPRB, 0.17611E+01_JPRB, 0.18741E+01_JPRB, 0.19069E+01_JPRB, 0.19422E+01_JPRB, 0.20034E+01_JPRB, 0.20932E+01_JPRB ,&
&  0.22156E+01_JPRB, 0.20771E+01_JPRB, 0.21477E+01_JPRB, 0.20046E+01_JPRB, 0.19070E+01_JPRB, 0.19010E+01_JPRB, 0.19003E+01_JPRB ,&
&  0.19285E+01_JPRB, 0.19418E+01_JPRB, 0.19137E+01_JPRB, 0.16840E+01_JPRB, 0.16365E+01_JPRB, 0.16184E+01_JPRB, 0.15025E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.14934E+01_JPRB, 0.13848E+01_JPRB, 0.14448E+01_JPRB, 0.12831E+01_JPRB, 0.11746E+01_JPRB, 0.11221E+01_JPRB ,&
&  0.10418E+01_JPRB, 0.82657E+00_JPRB, 0.76801E+00_JPRB, 0.72380E+00_JPRB, 0.66338E+00_JPRB, 0.60568E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.56621E+00_JPRB, 0.55455E+00_JPRB, 0.56153E+00_JPRB, 0.53968E+00_JPRB, 0.52109E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.53380E+00_JPRB, 0.52432E+00_JPRB, 0.50823E+00_JPRB, 0.49486E+00_JPRB, 0.48913E+00_JPRB, 0.47247E+00_JPRB, 0.44417E+00_JPRB ,&
&  0.43021E+00_JPRB, 0.40576E+00_JPRB, 0.38627E+00_JPRB, 0.37522E+00_JPRB, 0.37123E+00_JPRB, 0.36560E+00_JPRB, 0.35229E+00_JPRB ,&
&  0.34986E+00_JPRB, 0.35747E+00_JPRB, 0.37205E+00_JPRB, 0.37728E+00_JPRB, 0.37730E+00_JPRB, 0.37673E+00_JPRB, 0.38117E+00_JPRB ,&
&  0.39428E+00_JPRB, 0.41632E+00_JPRB, 0.44149E+00_JPRB, 0.45408E+00_JPRB, 0.45163E+00_JPRB, 0.44800E+00_JPRB, 0.44907E+00_JPRB ,&
&  0.44673E+00_JPRB, 0.44830E+00_JPRB, 0.46160E+00_JPRB, 0.48327E+00_JPRB, 0.52708E+00_JPRB, 0.55776E+00_JPRB, 0.54258E+00_JPRB ,&
&  0.41120E+00_JPRB, 0.39540E+00_JPRB, 0.46697E+00_JPRB, 0.47913E+00_JPRB, 0.48752E+00_JPRB, 0.47934E+00_JPRB, 0.47263E+00_JPRB ,&
&  0.46789E+00_JPRB, 0.46633E+00_JPRB, 0.46459E+00_JPRB, 0.46848E+00_JPRB, 0.47959E+00_JPRB, 0.49625E+00_JPRB, 0.50852E+00_JPRB ,&
&  0.53860E+00_JPRB, 0.51868E+00_JPRB, 0.54202E+00_JPRB, 0.60090E+00_JPRB, 0.83007E+00_JPRB, 0.10238E+01_JPRB, 0.11853E+01_JPRB ,&
&  0.13791E+01_JPRB, 0.15317E+01_JPRB, 0.17098E+01_JPRB, 0.18172E+01_JPRB, 0.18490E+01_JPRB, 0.18440E+01_JPRB, 0.19097E+01_JPRB ,&
&  0.19363E+01_JPRB, 0.19236E+01_JPRB, 0.18954E+01_JPRB, 0.18437E+01_JPRB, 0.17346E+01_JPRB, 0.16940E+01_JPRB, 0.16731E+01_JPRB ,&
&  0.16940E+01_JPRB, 0.17031E+01_JPRB, 0.16205E+01_JPRB, 0.15115E+01_JPRB, 0.14242E+01_JPRB, 0.14158E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.14129E+01_JPRB, 0.14205E+01_JPRB, 0.13899E+01_JPRB, 0.13382E+01_JPRB, 0.12395E+01_JPRB, 0.11331E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.91264E+00_JPRB, 0.78239E+00_JPRB, 0.67119E+00_JPRB, 0.65559E+00_JPRB, 0.65967E+00_JPRB, 0.62766E+00_JPRB, 0.57925E+00_JPRB ,&
&  0.56482E+00_JPRB, 0.54461E+00_JPRB, 0.52335E+00_JPRB, 0.53521E+00_JPRB, 0.53508E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.50852E+00_JPRB, 0.49995E+00_JPRB, 0.48796E+00_JPRB, 0.48124E+00_JPRB, 0.46736E+00_JPRB, 0.45048E+00_JPRB, 0.43251E+00_JPRB ,&
&  0.42450E+00_JPRB, 0.41133E+00_JPRB, 0.39652E+00_JPRB, 0.37888E+00_JPRB, 0.36646E+00_JPRB, 0.35677E+00_JPRB, 0.35782E+00_JPRB ,&
&  0.36528E+00_JPRB, 0.37291E+00_JPRB, 0.37415E+00_JPRB, 0.36945E+00_JPRB, 0.36669E+00_JPRB, 0.37095E+00_JPRB, 0.38382E+00_JPRB ,&
&  0.40463E+00_JPRB, 0.42944E+00_JPRB, 0.44651E+00_JPRB, 0.45149E+00_JPRB, 0.44997E+00_JPRB, 0.44802E+00_JPRB, 0.44211E+00_JPRB ,&
&  0.42833E+00_JPRB, 0.42305E+00_JPRB, 0.43036E+00_JPRB, 0.45033E+00_JPRB, 0.48730E+00_JPRB, 0.49992E+00_JPRB, 0.43416E+00_JPRB ,&
&  0.34612E+00_JPRB, 0.31616E+00_JPRB, 0.32249E+00_JPRB, 0.37970E+00_JPRB, 0.39699E+00_JPRB, 0.40842E+00_JPRB, 0.41825E+00_JPRB ,&
&  0.40709E+00_JPRB, 0.39676E+00_JPRB, 0.40273E+00_JPRB, 0.42892E+00_JPRB, 0.43903E+00_JPRB, 0.45020E+00_JPRB, 0.46119E+00_JPRB ,&
&  0.47582E+00_JPRB, 0.49676E+00_JPRB, 0.52474E+00_JPRB, 0.52626E+00_JPRB, 0.65148E+00_JPRB, 0.92937E+00_JPRB, 0.10839E+01_JPRB ,&
&  0.12714E+01_JPRB, 0.14363E+01_JPRB, 0.16720E+01_JPRB, 0.17390E+01_JPRB, 0.17322E+01_JPRB, 0.17505E+01_JPRB, 0.18192E+01_JPRB ,&
&  0.17939E+01_JPRB, 0.17646E+01_JPRB, 0.17215E+01_JPRB, 0.17222E+01_JPRB, 0.16703E+01_JPRB, 0.16058E+01_JPRB, 0.15457E+01_JPRB ,&
&  0.15404E+01_JPRB, 0.15104E+01_JPRB, 0.14301E+01_JPRB, 0.13211E+01_JPRB, 0.12749E+01_JPRB, 0.12528E+01_JPRB, 0.12677E+01_JPRB ,&
&  0.12664E+01_JPRB, 0.12815E+01_JPRB, 0.12834E+01_JPRB, 0.12959E+01_JPRB, 0.11596E+01_JPRB, 0.99335E+00_JPRB, 0.87311E+00_JPRB ,&
&  0.81086E+00_JPRB, 0.75607E+00_JPRB, 0.69309E+00_JPRB, 0.69959E+00_JPRB, 0.70643E+00_JPRB, 0.68364E+00_JPRB, 0.59773E+00_JPRB ,&
&  0.53256E+00_JPRB, 0.50643E+00_JPRB, 0.49286E+00_JPRB, 0.50546E+00_JPRB, 0.51339E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.44646E+00_JPRB, 0.45424E+00_JPRB, 0.46405E+00_JPRB, 0.46305E+00_JPRB, 0.45919E+00_JPRB, 0.42826E+00_JPRB, 0.39873E+00_JPRB ,&
&  0.38065E+00_JPRB, 0.37336E+00_JPRB, 0.37408E+00_JPRB, 0.37560E+00_JPRB, 0.37711E+00_JPRB, 0.38013E+00_JPRB, 0.38306E+00_JPRB ,&
&  0.38160E+00_JPRB, 0.37749E+00_JPRB, 0.37419E+00_JPRB, 0.37375E+00_JPRB, 0.37867E+00_JPRB, 0.39059E+00_JPRB, 0.40938E+00_JPRB ,&
&  0.43429E+00_JPRB, 0.45443E+00_JPRB, 0.46062E+00_JPRB, 0.45569E+00_JPRB, 0.44630E+00_JPRB, 0.43948E+00_JPRB, 0.42931E+00_JPRB ,&
&  0.41346E+00_JPRB, 0.40837E+00_JPRB, 0.41374E+00_JPRB, 0.43047E+00_JPRB, 0.45455E+00_JPRB, 0.46378E+00_JPRB, 0.39888E+00_JPRB ,&
&  0.32733E+00_JPRB, 0.28498E+00_JPRB, 0.26844E+00_JPRB, 0.27709E+00_JPRB, 0.30831E+00_JPRB, 0.36383E+00_JPRB, 0.37899E+00_JPRB ,&
&  0.39241E+00_JPRB, 0.40785E+00_JPRB, 0.42049E+00_JPRB, 0.43260E+00_JPRB, 0.44236E+00_JPRB, 0.44868E+00_JPRB, 0.45324E+00_JPRB ,&
&  0.45977E+00_JPRB, 0.47386E+00_JPRB, 0.49705E+00_JPRB, 0.50131E+00_JPRB, 0.53871E+00_JPRB, 0.70536E+00_JPRB, 0.92193E+00_JPRB ,&
&  0.11282E+01_JPRB, 0.12230E+01_JPRB, 0.13522E+01_JPRB, 0.15646E+01_JPRB, 0.16979E+01_JPRB, 0.17900E+01_JPRB, 0.18250E+01_JPRB ,&
&  0.18117E+01_JPRB, 0.17146E+01_JPRB, 0.16390E+01_JPRB, 0.16151E+01_JPRB, 0.16100E+01_JPRB, 0.16106E+01_JPRB, 0.15573E+01_JPRB ,&
&  0.15204E+01_JPRB, 0.14515E+01_JPRB, 0.13349E+01_JPRB, 0.12191E+01_JPRB, 0.11928E+01_JPRB, 0.12254E+01_JPRB, 0.12867E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.12830E+01_JPRB, 0.12710E+01_JPRB, 0.12878E+01_JPRB, 0.11258E+01_JPRB, 0.10167E+01_JPRB, 0.93358E+00_JPRB ,&
&  0.90468E+00_JPRB, 0.88057E+00_JPRB, 0.84373E+00_JPRB, 0.86271E+00_JPRB, 0.83803E+00_JPRB, 0.79112E+00_JPRB, 0.71381E+00_JPRB ,&
&  0.62584E+00_JPRB, 0.55804E+00_JPRB, 0.51412E+00_JPRB, 0.47836E+00_JPRB, 0.45967E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.48506E+00_JPRB, 0.46696E+00_JPRB, 0.45769E+00_JPRB, 0.45370E+00_JPRB, 0.45026E+00_JPRB, 0.43550E+00_JPRB, 0.42244E+00_JPRB ,&
&  0.41387E+00_JPRB, 0.41210E+00_JPRB, 0.41041E+00_JPRB, 0.40641E+00_JPRB, 0.40373E+00_JPRB, 0.40144E+00_JPRB, 0.39956E+00_JPRB ,&
&  0.39785E+00_JPRB, 0.39785E+00_JPRB, 0.40054E+00_JPRB, 0.40245E+00_JPRB, 0.40957E+00_JPRB, 0.42522E+00_JPRB, 0.44954E+00_JPRB ,&
&  0.47573E+00_JPRB, 0.48585E+00_JPRB, 0.48097E+00_JPRB, 0.46910E+00_JPRB, 0.44732E+00_JPRB, 0.42907E+00_JPRB, 0.41428E+00_JPRB ,&
&  0.39850E+00_JPRB, 0.39548E+00_JPRB, 0.40756E+00_JPRB, 0.41841E+00_JPRB, 0.43236E+00_JPRB, 0.42799E+00_JPRB, 0.36496E+00_JPRB ,&
&  0.30634E+00_JPRB, 0.26627E+00_JPRB, 0.24689E+00_JPRB, 0.24235E+00_JPRB, 0.25228E+00_JPRB, 0.27712E+00_JPRB, 0.31959E+00_JPRB ,&
&  0.39870E+00_JPRB, 0.44875E+00_JPRB, 0.46687E+00_JPRB, 0.48823E+00_JPRB, 0.49159E+00_JPRB, 0.48885E+00_JPRB, 0.48768E+00_JPRB ,&
&  0.49832E+00_JPRB, 0.51823E+00_JPRB, 0.54815E+00_JPRB, 0.57824E+00_JPRB, 0.62338E+00_JPRB, 0.68484E+00_JPRB, 0.81418E+00_JPRB ,&
&  0.11036E+01_JPRB, 0.11904E+01_JPRB, 0.11435E+01_JPRB, 0.13080E+01_JPRB, 0.15398E+01_JPRB, 0.17254E+01_JPRB, 0.18424E+01_JPRB ,&
&  0.18313E+01_JPRB, 0.17348E+01_JPRB, 0.16569E+01_JPRB, 0.16066E+01_JPRB, 0.15774E+01_JPRB, 0.15591E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.15742E+01_JPRB, 0.16194E+01_JPRB, 0.17115E+01_JPRB, 0.14707E+01_JPRB, 0.13721E+01_JPRB, 0.13524E+01_JPRB, 0.13160E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.12539E+01_JPRB, 0.12571E+01_JPRB, 0.12661E+01_JPRB, 0.11841E+01_JPRB, 0.11105E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.99055E+00_JPRB, 0.95678E+00_JPRB, 0.94280E+00_JPRB, 0.91260E+00_JPRB, 0.85591E+00_JPRB, 0.81472E+00_JPRB ,&
&  0.75834E+00_JPRB, 0.67600E+00_JPRB, 0.62118E+00_JPRB, 0.57279E+00_JPRB, 0.52251E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.50254E+00_JPRB, 0.48172E+00_JPRB, 0.46699E+00_JPRB, 0.45735E+00_JPRB, 0.44874E+00_JPRB, 0.44280E+00_JPRB, 0.43514E+00_JPRB ,&
&  0.42854E+00_JPRB, 0.42262E+00_JPRB, 0.41822E+00_JPRB, 0.41448E+00_JPRB, 0.41282E+00_JPRB, 0.41245E+00_JPRB, 0.41404E+00_JPRB ,&
&  0.41714E+00_JPRB, 0.41690E+00_JPRB, 0.42233E+00_JPRB, 0.43493E+00_JPRB, 0.45434E+00_JPRB, 0.47295E+00_JPRB, 0.49410E+00_JPRB ,&
&  0.50114E+00_JPRB, 0.49591E+00_JPRB, 0.48060E+00_JPRB, 0.45838E+00_JPRB, 0.43728E+00_JPRB, 0.41941E+00_JPRB, 0.40901E+00_JPRB ,&
&  0.40821E+00_JPRB, 0.40106E+00_JPRB, 0.40521E+00_JPRB, 0.40661E+00_JPRB, 0.39759E+00_JPRB, 0.39681E+00_JPRB, 0.32968E+00_JPRB ,&
&  0.28034E+00_JPRB, 0.24551E+00_JPRB, 0.23432E+00_JPRB, 0.22946E+00_JPRB, 0.23583E+00_JPRB, 0.25119E+00_JPRB, 0.28772E+00_JPRB ,&
&  0.35989E+00_JPRB, 0.49376E+00_JPRB, 0.51706E+00_JPRB, 0.52840E+00_JPRB, 0.53496E+00_JPRB, 0.53477E+00_JPRB, 0.53529E+00_JPRB ,&
&  0.55628E+00_JPRB, 0.57982E+00_JPRB, 0.61769E+00_JPRB, 0.65689E+00_JPRB, 0.70417E+00_JPRB, 0.74572E+00_JPRB, 0.78164E+00_JPRB ,&
&  0.85014E+00_JPRB, 0.91301E+00_JPRB, 0.98980E+00_JPRB, 0.11285E+01_JPRB, 0.13537E+01_JPRB, 0.15782E+01_JPRB, 0.16416E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.16278E+01_JPRB, 0.15711E+01_JPRB, 0.15265E+01_JPRB, 0.15051E+01_JPRB, 0.14981E+01_JPRB, 0.14996E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.15439E+01_JPRB, 0.16422E+01_JPRB, 0.15599E+01_JPRB, 0.15102E+01_JPRB, 0.14583E+01_JPRB, 0.13951E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.13127E+01_JPRB, 0.12924E+01_JPRB, 0.12614E+01_JPRB, 0.12230E+01_JPRB, 0.11871E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.10787E+01_JPRB, 0.10175E+01_JPRB, 0.97502E+00_JPRB, 0.93925E+00_JPRB, 0.90672E+00_JPRB, 0.87025E+00_JPRB ,&
&  0.80219E+00_JPRB, 0.71074E+00_JPRB, 0.64017E+00_JPRB, 0.58316E+00_JPRB, 0.53622E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.55142E+00_JPRB, 0.52530E+00_JPRB, 0.50282E+00_JPRB, 0.48866E+00_JPRB, 0.47468E+00_JPRB, 0.46709E+00_JPRB, 0.45863E+00_JPRB ,&
&  0.45433E+00_JPRB, 0.45059E+00_JPRB, 0.44927E+00_JPRB, 0.44913E+00_JPRB, 0.45108E+00_JPRB, 0.45455E+00_JPRB, 0.46037E+00_JPRB ,&
&  0.46880E+00_JPRB, 0.47945E+00_JPRB, 0.49364E+00_JPRB, 0.50415E+00_JPRB, 0.51331E+00_JPRB, 0.51868E+00_JPRB, 0.52484E+00_JPRB ,&
&  0.52271E+00_JPRB, 0.51879E+00_JPRB, 0.49145E+00_JPRB, 0.46145E+00_JPRB, 0.43752E+00_JPRB, 0.41549E+00_JPRB, 0.41035E+00_JPRB ,&
&  0.40604E+00_JPRB, 0.39880E+00_JPRB, 0.38782E+00_JPRB, 0.38276E+00_JPRB, 0.37557E+00_JPRB, 0.31837E+00_JPRB, 0.30198E+00_JPRB ,&
&  0.27398E+00_JPRB, 0.24609E+00_JPRB, 0.23584E+00_JPRB, 0.23028E+00_JPRB, 0.23243E+00_JPRB, 0.24506E+00_JPRB, 0.26916E+00_JPRB ,&
&  0.33174E+00_JPRB, 0.50925E+00_JPRB, 0.54953E+00_JPRB, 0.56599E+00_JPRB, 0.54801E+00_JPRB, 0.55433E+00_JPRB, 0.56229E+00_JPRB ,&
&  0.59213E+00_JPRB, 0.62189E+00_JPRB, 0.66060E+00_JPRB, 0.69445E+00_JPRB, 0.72847E+00_JPRB, 0.74887E+00_JPRB, 0.78559E+00_JPRB ,&
&  0.84314E+00_JPRB, 0.91858E+00_JPRB, 0.99223E+00_JPRB, 0.10698E+01_JPRB, 0.11440E+01_JPRB, 0.12830E+01_JPRB, 0.14298E+01_JPRB ,&
&  0.14474E+01_JPRB, 0.14290E+01_JPRB, 0.14292E+01_JPRB, 0.14246E+01_JPRB, 0.14231E+01_JPRB, 0.14190E+01_JPRB, 0.14252E+01_JPRB ,&
&  0.14451E+01_JPRB, 0.14635E+01_JPRB, 0.14278E+01_JPRB, 0.13717E+01_JPRB, 0.13404E+01_JPRB, 0.12943E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.12911E+01_JPRB, 0.12732E+01_JPRB, 0.12386E+01_JPRB, 0.12040E+01_JPRB, 0.11632E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10926E+01_JPRB, 0.10490E+01_JPRB, 0.10031E+01_JPRB, 0.96567E+00_JPRB, 0.93642E+00_JPRB, 0.91289E+00_JPRB, 0.87802E+00_JPRB ,&
&  0.81798E+00_JPRB, 0.74335E+00_JPRB, 0.68240E+00_JPRB, 0.62730E+00_JPRB, 0.58350E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.61602E+00_JPRB, 0.58859E+00_JPRB, 0.56577E+00_JPRB, 0.54943E+00_JPRB, 0.53438E+00_JPRB, 0.52425E+00_JPRB, 0.51485E+00_JPRB ,&
&  0.51001E+00_JPRB, 0.50539E+00_JPRB, 0.50465E+00_JPRB, 0.50428E+00_JPRB, 0.50676E+00_JPRB, 0.51120E+00_JPRB, 0.51691E+00_JPRB ,&
&  0.52457E+00_JPRB, 0.53091E+00_JPRB, 0.53777E+00_JPRB, 0.54087E+00_JPRB, 0.54529E+00_JPRB, 0.54700E+00_JPRB, 0.55189E+00_JPRB ,&
&  0.54626E+00_JPRB, 0.53732E+00_JPRB, 0.50329E+00_JPRB, 0.47197E+00_JPRB, 0.43603E+00_JPRB, 0.41045E+00_JPRB, 0.39359E+00_JPRB ,&
&  0.39630E+00_JPRB, 0.36327E+00_JPRB, 0.35742E+00_JPRB, 0.33365E+00_JPRB, 0.31076E+00_JPRB, 0.29212E+00_JPRB, 0.27612E+00_JPRB ,&
&  0.26468E+00_JPRB, 0.24916E+00_JPRB, 0.24236E+00_JPRB, 0.24211E+00_JPRB, 0.24660E+00_JPRB, 0.26662E+00_JPRB, 0.29748E+00_JPRB ,&
&  0.36673E+00_JPRB, 0.52916E+00_JPRB, 0.58084E+00_JPRB, 0.60522E+00_JPRB, 0.60115E+00_JPRB, 0.61090E+00_JPRB, 0.61238E+00_JPRB ,&
&  0.64956E+00_JPRB, 0.69798E+00_JPRB, 0.74800E+00_JPRB, 0.78431E+00_JPRB, 0.82660E+00_JPRB, 0.85489E+00_JPRB, 0.88199E+00_JPRB ,&
&  0.91066E+00_JPRB, 0.95174E+00_JPRB, 0.98827E+00_JPRB, 0.10195E+01_JPRB, 0.10548E+01_JPRB, 0.10854E+01_JPRB, 0.11258E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.12055E+01_JPRB, 0.12241E+01_JPRB, 0.12483E+01_JPRB, 0.12530E+01_JPRB, 0.12499E+01_JPRB, 0.12530E+01_JPRB ,&
&  0.12620E+01_JPRB, 0.12706E+01_JPRB, 0.12773E+01_JPRB, 0.12694E+01_JPRB, 0.12631E+01_JPRB, 0.12359E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.12144E+01_JPRB, 0.12038E+01_JPRB, 0.11890E+01_JPRB, 0.11683E+01_JPRB, 0.11419E+01_JPRB, 0.11124E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10638E+01_JPRB, 0.10310E+01_JPRB, 0.99494E+00_JPRB, 0.96427E+00_JPRB, 0.93794E+00_JPRB, 0.91025E+00_JPRB, 0.86515E+00_JPRB ,&
&  0.81059E+00_JPRB, 0.76329E+00_JPRB, 0.72003E+00_JPRB, 0.68494E+00_JPRB, 0.64830E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.69034E+00_JPRB, 0.66896E+00_JPRB, 0.64688E+00_JPRB, 0.62809E+00_JPRB, 0.61026E+00_JPRB, 0.59676E+00_JPRB, 0.58409E+00_JPRB ,&
&  0.57492E+00_JPRB, 0.56698E+00_JPRB, 0.56192E+00_JPRB, 0.55890E+00_JPRB, 0.55748E+00_JPRB, 0.55920E+00_JPRB, 0.56062E+00_JPRB ,&
&  0.56423E+00_JPRB, 0.56716E+00_JPRB, 0.57203E+00_JPRB, 0.57621E+00_JPRB, 0.58205E+00_JPRB, 0.58278E+00_JPRB, 0.58328E+00_JPRB ,&
&  0.57604E+00_JPRB, 0.56479E+00_JPRB, 0.51885E+00_JPRB, 0.48896E+00_JPRB, 0.45337E+00_JPRB, 0.42565E+00_JPRB, 0.41205E+00_JPRB ,&
&  0.39686E+00_JPRB, 0.38841E+00_JPRB, 0.37365E+00_JPRB, 0.35834E+00_JPRB, 0.34344E+00_JPRB, 0.32539E+00_JPRB, 0.31175E+00_JPRB ,&
&  0.30021E+00_JPRB, 0.28809E+00_JPRB, 0.28290E+00_JPRB, 0.28402E+00_JPRB, 0.29230E+00_JPRB, 0.31559E+00_JPRB, 0.35266E+00_JPRB ,&
&  0.42318E+00_JPRB, 0.50273E+00_JPRB, 0.60950E+00_JPRB, 0.64370E+00_JPRB, 0.64949E+00_JPRB, 0.67768E+00_JPRB, 0.70300E+00_JPRB ,&
&  0.72060E+00_JPRB, 0.75592E+00_JPRB, 0.79212E+00_JPRB, 0.81234E+00_JPRB, 0.82791E+00_JPRB, 0.83779E+00_JPRB, 0.84815E+00_JPRB ,&
&  0.85723E+00_JPRB, 0.87385E+00_JPRB, 0.88354E+00_JPRB, 0.89004E+00_JPRB, 0.89682E+00_JPRB, 0.90340E+00_JPRB, 0.91964E+00_JPRB ,&
&  0.93382E+00_JPRB, 0.96514E+00_JPRB, 0.99185E+00_JPRB, 0.10342E+01_JPRB, 0.10518E+01_JPRB, 0.10734E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.10928E+01_JPRB, 0.10932E+01_JPRB, 0.10772E+01_JPRB, 0.10735E+01_JPRB, 0.10805E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.10583E+01_JPRB, 0.10476E+01_JPRB, 0.10351E+01_JPRB, 0.10235E+01_JPRB, 0.10156E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.99607E+00_JPRB, 0.97635E+00_JPRB, 0.95700E+00_JPRB, 0.93553E+00_JPRB, 0.91167E+00_JPRB, 0.87910E+00_JPRB, 0.83831E+00_JPRB ,&
&  0.80290E+00_JPRB, 0.77662E+00_JPRB, 0.75058E+00_JPRB, 0.73100E+00_JPRB, 0.71029E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.70018E+00_JPRB, 0.69251E+00_JPRB, 0.68156E+00_JPRB, 0.66742E+00_JPRB, 0.65431E+00_JPRB, 0.64429E+00_JPRB, 0.63410E+00_JPRB ,&
&  0.62576E+00_JPRB, 0.62004E+00_JPRB, 0.61581E+00_JPRB, 0.61284E+00_JPRB, 0.61000E+00_JPRB, 0.61114E+00_JPRB, 0.61272E+00_JPRB ,&
&  0.61476E+00_JPRB, 0.61536E+00_JPRB, 0.61844E+00_JPRB, 0.61921E+00_JPRB, 0.61848E+00_JPRB, 0.61469E+00_JPRB, 0.61087E+00_JPRB ,&
&  0.60229E+00_JPRB, 0.59191E+00_JPRB, 0.57936E+00_JPRB, 0.56354E+00_JPRB, 0.54907E+00_JPRB, 0.49680E+00_JPRB, 0.48014E+00_JPRB ,&
&  0.46834E+00_JPRB, 0.46265E+00_JPRB, 0.46161E+00_JPRB, 0.46464E+00_JPRB, 0.48159E+00_JPRB, 0.47716E+00_JPRB, 0.47028E+00_JPRB ,&
&  0.43452E+00_JPRB, 0.42232E+00_JPRB, 0.41708E+00_JPRB, 0.41608E+00_JPRB, 0.42511E+00_JPRB, 0.45455E+00_JPRB, 0.49369E+00_JPRB ,&
&  0.56104E+00_JPRB, 0.59415E+00_JPRB, 0.62102E+00_JPRB, 0.64985E+00_JPRB, 0.67466E+00_JPRB, 0.69550E+00_JPRB, 0.71475E+00_JPRB ,&
&  0.73231E+00_JPRB, 0.75011E+00_JPRB, 0.76444E+00_JPRB, 0.77380E+00_JPRB, 0.77834E+00_JPRB, 0.78196E+00_JPRB, 0.78481E+00_JPRB ,&
&  0.78659E+00_JPRB, 0.78671E+00_JPRB, 0.78723E+00_JPRB, 0.78738E+00_JPRB, 0.78537E+00_JPRB, 0.78720E+00_JPRB, 0.79422E+00_JPRB ,&
&  0.80527E+00_JPRB, 0.82113E+00_JPRB, 0.83509E+00_JPRB, 0.85568E+00_JPRB, 0.87541E+00_JPRB, 0.90169E+00_JPRB, 0.91173E+00_JPRB ,&
&  0.92538E+00_JPRB, 0.93377E+00_JPRB, 0.94543E+00_JPRB, 0.94468E+00_JPRB, 0.94446E+00_JPRB, 0.94156E+00_JPRB, 0.94111E+00_JPRB ,&
&  0.93146E+00_JPRB, 0.92338E+00_JPRB, 0.91307E+00_JPRB, 0.90763E+00_JPRB, 0.89417E+00_JPRB, 0.88608E+00_JPRB, 0.87484E+00_JPRB ,&
&  0.87002E+00_JPRB, 0.85333E+00_JPRB, 0.84267E+00_JPRB, 0.82517E+00_JPRB, 0.80962E+00_JPRB, 0.78518E+00_JPRB, 0.76360E+00_JPRB ,&
&  0.74483E+00_JPRB, 0.73196E+00_JPRB, 0.72158E+00_JPRB, 0.71380E+00_JPRB, 0.70635E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.69256E+00_JPRB, 0.68882E+00_JPRB, 0.68468E+00_JPRB, 0.67983E+00_JPRB, 0.67525E+00_JPRB, 0.67150E+00_JPRB, 0.66766E+00_JPRB ,&
&  0.66337E+00_JPRB, 0.65993E+00_JPRB, 0.65664E+00_JPRB, 0.65354E+00_JPRB, 0.65055E+00_JPRB, 0.64929E+00_JPRB, 0.64774E+00_JPRB ,&
&  0.64553E+00_JPRB, 0.64235E+00_JPRB, 0.63996E+00_JPRB, 0.63656E+00_JPRB, 0.63254E+00_JPRB, 0.62710E+00_JPRB, 0.62196E+00_JPRB ,&
&  0.61584E+00_JPRB, 0.60841E+00_JPRB, 0.60211E+00_JPRB, 0.59871E+00_JPRB, 0.59487E+00_JPRB, 0.58943E+00_JPRB, 0.58573E+00_JPRB ,&
&  0.58400E+00_JPRB, 0.58213E+00_JPRB, 0.58049E+00_JPRB, 0.57875E+00_JPRB, 0.57499E+00_JPRB, 0.57394E+00_JPRB, 0.57323E+00_JPRB ,&
&  0.57181E+00_JPRB, 0.57254E+00_JPRB, 0.57649E+00_JPRB, 0.58098E+00_JPRB, 0.58686E+00_JPRB, 0.59532E+00_JPRB, 0.60908E+00_JPRB ,&
&  0.62483E+00_JPRB, 0.63911E+00_JPRB, 0.65307E+00_JPRB, 0.66687E+00_JPRB, 0.67837E+00_JPRB, 0.68781E+00_JPRB, 0.69629E+00_JPRB ,&
&  0.70246E+00_JPRB, 0.70712E+00_JPRB, 0.70974E+00_JPRB, 0.71018E+00_JPRB, 0.70935E+00_JPRB, 0.70930E+00_JPRB, 0.70974E+00_JPRB ,&
&  0.71036E+00_JPRB, 0.70760E+00_JPRB, 0.70713E+00_JPRB, 0.70906E+00_JPRB, 0.71221E+00_JPRB, 0.71559E+00_JPRB, 0.72099E+00_JPRB ,&
&  0.72773E+00_JPRB, 0.73506E+00_JPRB, 0.74172E+00_JPRB, 0.75080E+00_JPRB, 0.76141E+00_JPRB, 0.77340E+00_JPRB, 0.78025E+00_JPRB ,&
&  0.78948E+00_JPRB, 0.79974E+00_JPRB, 0.81080E+00_JPRB, 0.81219E+00_JPRB, 0.81469E+00_JPRB, 0.81663E+00_JPRB, 0.81954E+00_JPRB ,&
&  0.81341E+00_JPRB, 0.81025E+00_JPRB, 0.80703E+00_JPRB, 0.80680E+00_JPRB, 0.79857E+00_JPRB, 0.79337E+00_JPRB, 0.78816E+00_JPRB ,&
&  0.78721E+00_JPRB, 0.77593E+00_JPRB, 0.76818E+00_JPRB, 0.75854E+00_JPRB, 0.75157E+00_JPRB, 0.73910E+00_JPRB, 0.72901E+00_JPRB ,&
&  0.71986E+00_JPRB, 0.71403E+00_JPRB, 0.70747E+00_JPRB, 0.70197E+00_JPRB, 0.69715E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.68371E+00_JPRB, 0.68183E+00_JPRB, 0.67924E+00_JPRB, 0.67699E+00_JPRB, 0.67498E+00_JPRB, 0.67337E+00_JPRB, 0.67187E+00_JPRB ,&
&  0.66975E+00_JPRB, 0.66776E+00_JPRB, 0.66580E+00_JPRB, 0.66379E+00_JPRB, 0.66165E+00_JPRB, 0.65960E+00_JPRB, 0.65752E+00_JPRB ,&
&  0.65538E+00_JPRB, 0.65342E+00_JPRB, 0.65199E+00_JPRB, 0.65022E+00_JPRB, 0.64813E+00_JPRB, 0.64569E+00_JPRB, 0.64332E+00_JPRB ,&
&  0.64065E+00_JPRB, 0.63793E+00_JPRB, 0.63491E+00_JPRB, 0.63210E+00_JPRB, 0.62922E+00_JPRB, 0.62549E+00_JPRB, 0.62222E+00_JPRB ,&
&  0.62019E+00_JPRB, 0.61853E+00_JPRB, 0.61719E+00_JPRB, 0.61568E+00_JPRB, 0.61492E+00_JPRB, 0.61513E+00_JPRB, 0.61545E+00_JPRB ,&
&  0.61600E+00_JPRB, 0.61711E+00_JPRB, 0.61914E+00_JPRB, 0.62140E+00_JPRB, 0.62384E+00_JPRB, 0.62712E+00_JPRB, 0.63169E+00_JPRB ,&
&  0.63588E+00_JPRB, 0.63984E+00_JPRB, 0.64426E+00_JPRB, 0.64891E+00_JPRB, 0.65362E+00_JPRB, 0.65812E+00_JPRB, 0.66235E+00_JPRB ,&
&  0.66544E+00_JPRB, 0.66816E+00_JPRB, 0.67086E+00_JPRB, 0.67313E+00_JPRB, 0.67423E+00_JPRB, 0.67527E+00_JPRB, 0.67631E+00_JPRB ,&
&  0.67713E+00_JPRB, 0.67779E+00_JPRB, 0.67885E+00_JPRB, 0.68054E+00_JPRB, 0.68227E+00_JPRB, 0.68407E+00_JPRB, 0.68625E+00_JPRB ,&
&  0.68903E+00_JPRB, 0.69073E+00_JPRB, 0.69268E+00_JPRB, 0.69508E+00_JPRB, 0.69814E+00_JPRB, 0.70166E+00_JPRB, 0.70445E+00_JPRB ,&
&  0.70728E+00_JPRB, 0.71033E+00_JPRB, 0.71324E+00_JPRB, 0.71474E+00_JPRB, 0.71646E+00_JPRB, 0.71800E+00_JPRB, 0.71945E+00_JPRB ,&
&  0.71888E+00_JPRB, 0.71889E+00_JPRB, 0.71945E+00_JPRB, 0.72034E+00_JPRB, 0.71843E+00_JPRB, 0.71687E+00_JPRB, 0.71549E+00_JPRB ,&
&  0.71553E+00_JPRB, 0.71215E+00_JPRB, 0.70976E+00_JPRB, 0.70668E+00_JPRB, 0.70483E+00_JPRB, 0.70155E+00_JPRB, 0.69893E+00_JPRB ,&
&  0.69609E+00_JPRB, 0.69389E+00_JPRB, 0.69136E+00_JPRB, 0.68874E+00_JPRB, 0.68617E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB, 0.66512E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB ,&
&  0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB, 0.24690E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.27069E+00_JPRB, 0.27336E+00_JPRB, 0.27565E+00_JPRB, 0.27735E+00_JPRB, 0.27773E+00_JPRB, 0.27951E+00_JPRB, 0.27997E+00_JPRB ,&
&  0.28024E+00_JPRB, 0.28017E+00_JPRB, 0.28093E+00_JPRB, 0.28175E+00_JPRB, 0.28229E+00_JPRB, 0.28256E+00_JPRB, 0.28259E+00_JPRB ,&
&  0.28212E+00_JPRB, 0.28117E+00_JPRB, 0.27754E+00_JPRB, 0.27553E+00_JPRB, 0.27437E+00_JPRB, 0.27385E+00_JPRB, 0.27359E+00_JPRB ,&
&  0.27291E+00_JPRB, 0.27206E+00_JPRB, 0.27065E+00_JPRB, 0.27042E+00_JPRB, 0.26961E+00_JPRB, 0.26855E+00_JPRB, 0.26706E+00_JPRB ,&
&  0.26560E+00_JPRB, 0.26443E+00_JPRB, 0.26297E+00_JPRB, 0.26148E+00_JPRB, 0.25965E+00_JPRB, 0.25745E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.25189E+00_JPRB, 0.24901E+00_JPRB, 0.24663E+00_JPRB, 0.24441E+00_JPRB, 0.24250E+00_JPRB, 0.24097E+00_JPRB, 0.23989E+00_JPRB ,&
&  0.23871E+00_JPRB, 0.23736E+00_JPRB, 0.23600E+00_JPRB, 0.23486E+00_JPRB, 0.23373E+00_JPRB, 0.23275E+00_JPRB, 0.23228E+00_JPRB ,&
&  0.23200E+00_JPRB, 0.23166E+00_JPRB, 0.23125E+00_JPRB, 0.23089E+00_JPRB, 0.23043E+00_JPRB, 0.23000E+00_JPRB, 0.22956E+00_JPRB ,&
&  0.22921E+00_JPRB, 0.22899E+00_JPRB, 0.22863E+00_JPRB, 0.22827E+00_JPRB, 0.22805E+00_JPRB, 0.22766E+00_JPRB, 0.22724E+00_JPRB ,&
&  0.22673E+00_JPRB, 0.22644E+00_JPRB, 0.22610E+00_JPRB, 0.22569E+00_JPRB, 0.22531E+00_JPRB, 0.22505E+00_JPRB, 0.22505E+00_JPRB ,&
&  0.22501E+00_JPRB, 0.22491E+00_JPRB, 0.22496E+00_JPRB, 0.22568E+00_JPRB, 0.22643E+00_JPRB, 0.22734E+00_JPRB, 0.22836E+00_JPRB ,&
&  0.22970E+00_JPRB, 0.23109E+00_JPRB, 0.23265E+00_JPRB, 0.23414E+00_JPRB, 0.23612E+00_JPRB, 0.23808E+00_JPRB, 0.24004E+00_JPRB ,&
&  0.24191E+00_JPRB, 0.24383E+00_JPRB, 0.24575E+00_JPRB, 0.24770E+00_JPRB, 0.25022E+00_JPRB, 0.25242E+00_JPRB, 0.25490E+00_JPRB ,&
&  0.25727E+00_JPRB, 0.25987E+00_JPRB, 0.26255E+00_JPRB, 0.26491E+00_JPRB, 0.26733E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.31323E+00_JPRB, 0.32357E+00_JPRB, 0.33235E+00_JPRB, 0.34084E+00_JPRB, 0.34365E+00_JPRB, 0.40833E+00_JPRB, 0.38875E+00_JPRB ,&
&  0.37926E+00_JPRB, 0.35122E+00_JPRB, 0.35218E+00_JPRB, 0.34779E+00_JPRB, 0.34126E+00_JPRB, 0.33380E+00_JPRB, 0.32746E+00_JPRB ,&
&  0.32119E+00_JPRB, 0.31341E+00_JPRB, 0.30845E+00_JPRB, 0.30243E+00_JPRB, 0.29670E+00_JPRB, 0.29101E+00_JPRB, 0.28764E+00_JPRB ,&
&  0.28511E+00_JPRB, 0.28363E+00_JPRB, 0.28252E+00_JPRB, 0.28165E+00_JPRB, 0.28417E+00_JPRB, 0.28550E+00_JPRB, 0.28372E+00_JPRB ,&
&  0.28014E+00_JPRB, 0.27675E+00_JPRB, 0.27249E+00_JPRB, 0.26812E+00_JPRB, 0.26504E+00_JPRB, 0.26229E+00_JPRB, 0.25946E+00_JPRB ,&
&  0.25572E+00_JPRB, 0.25155E+00_JPRB, 0.24735E+00_JPRB, 0.24405E+00_JPRB, 0.24095E+00_JPRB, 0.23779E+00_JPRB, 0.23487E+00_JPRB ,&
&  0.23184E+00_JPRB, 0.22886E+00_JPRB, 0.22586E+00_JPRB, 0.22390E+00_JPRB, 0.22226E+00_JPRB, 0.22072E+00_JPRB, 0.21933E+00_JPRB ,&
&  0.21873E+00_JPRB, 0.21802E+00_JPRB, 0.21713E+00_JPRB, 0.21603E+00_JPRB, 0.21497E+00_JPRB, 0.21383E+00_JPRB, 0.21291E+00_JPRB ,&
&  0.21276E+00_JPRB, 0.21219E+00_JPRB, 0.21159E+00_JPRB, 0.21082E+00_JPRB, 0.21045E+00_JPRB, 0.20967E+00_JPRB, 0.20871E+00_JPRB ,&
&  0.20783E+00_JPRB, 0.20702E+00_JPRB, 0.20683E+00_JPRB, 0.20658E+00_JPRB, 0.20630E+00_JPRB, 0.20611E+00_JPRB, 0.20650E+00_JPRB ,&
&  0.20698E+00_JPRB, 0.20757E+00_JPRB, 0.20855E+00_JPRB, 0.20972E+00_JPRB, 0.21075E+00_JPRB, 0.21177E+00_JPRB, 0.21319E+00_JPRB ,&
&  0.21511E+00_JPRB, 0.21703E+00_JPRB, 0.21917E+00_JPRB, 0.22116E+00_JPRB, 0.22450E+00_JPRB, 0.22811E+00_JPRB, 0.23206E+00_JPRB ,&
&  0.23691E+00_JPRB, 0.24354E+00_JPRB, 0.25113E+00_JPRB, 0.25792E+00_JPRB, 0.26443E+00_JPRB, 0.27174E+00_JPRB, 0.27775E+00_JPRB ,&
&  0.28301E+00_JPRB, 0.28823E+00_JPRB, 0.29398E+00_JPRB, 0.30016E+00_JPRB, 0.30688E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.43928E+00_JPRB, 0.44040E+00_JPRB, 0.44097E+00_JPRB, 0.43757E+00_JPRB, 0.43632E+00_JPRB, 0.43629E+00_JPRB, 0.43354E+00_JPRB ,&
&  0.42563E+00_JPRB, 0.42316E+00_JPRB, 0.41828E+00_JPRB, 0.41123E+00_JPRB, 0.40432E+00_JPRB, 0.39313E+00_JPRB, 0.37684E+00_JPRB ,&
&  0.35501E+00_JPRB, 0.33582E+00_JPRB, 0.32129E+00_JPRB, 0.30795E+00_JPRB, 0.29487E+00_JPRB, 0.28213E+00_JPRB, 0.27689E+00_JPRB ,&
&  0.27237E+00_JPRB, 0.27042E+00_JPRB, 0.26977E+00_JPRB, 0.27328E+00_JPRB, 0.28084E+00_JPRB, 0.29030E+00_JPRB, 0.30064E+00_JPRB ,&
&  0.31248E+00_JPRB, 0.32351E+00_JPRB, 0.32490E+00_JPRB, 0.31777E+00_JPRB, 0.31084E+00_JPRB, 0.30648E+00_JPRB, 0.30313E+00_JPRB ,&
&  0.29921E+00_JPRB, 0.29087E+00_JPRB, 0.28143E+00_JPRB, 0.27155E+00_JPRB, 0.26161E+00_JPRB, 0.25436E+00_JPRB, 0.24586E+00_JPRB ,&
&  0.23966E+00_JPRB, 0.23409E+00_JPRB, 0.22820E+00_JPRB, 0.22315E+00_JPRB, 0.21970E+00_JPRB, 0.21682E+00_JPRB, 0.21392E+00_JPRB ,&
&  0.21243E+00_JPRB, 0.21095E+00_JPRB, 0.20928E+00_JPRB, 0.20764E+00_JPRB, 0.20562E+00_JPRB, 0.20407E+00_JPRB, 0.20265E+00_JPRB ,&
&  0.20156E+00_JPRB, 0.20063E+00_JPRB, 0.20006E+00_JPRB, 0.19947E+00_JPRB, 0.19901E+00_JPRB, 0.19808E+00_JPRB, 0.19698E+00_JPRB ,&
&  0.19626E+00_JPRB, 0.19594E+00_JPRB, 0.19520E+00_JPRB, 0.19478E+00_JPRB, 0.19446E+00_JPRB, 0.19374E+00_JPRB, 0.19417E+00_JPRB ,&
&  0.19527E+00_JPRB, 0.19651E+00_JPRB, 0.19681E+00_JPRB, 0.19787E+00_JPRB, 0.19915E+00_JPRB, 0.20075E+00_JPRB, 0.20325E+00_JPRB ,&
&  0.20518E+00_JPRB, 0.20718E+00_JPRB, 0.20968E+00_JPRB, 0.21187E+00_JPRB, 0.21525E+00_JPRB, 0.21941E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.23154E+00_JPRB, 0.24000E+00_JPRB, 0.25176E+00_JPRB, 0.26608E+00_JPRB, 0.28651E+00_JPRB, 0.29486E+00_JPRB, 0.30312E+00_JPRB ,&
&  0.31016E+00_JPRB, 0.32404E+00_JPRB, 0.40623E+00_JPRB, 0.42116E+00_JPRB, 0.43600E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.43998E+00_JPRB, 0.45218E+00_JPRB, 0.46955E+00_JPRB, 0.48027E+00_JPRB, 0.49459E+00_JPRB, 0.49736E+00_JPRB, 0.49214E+00_JPRB ,&
&  0.47325E+00_JPRB, 0.44344E+00_JPRB, 0.42681E+00_JPRB, 0.40636E+00_JPRB, 0.38511E+00_JPRB, 0.36376E+00_JPRB, 0.35429E+00_JPRB ,&
&  0.34295E+00_JPRB, 0.33024E+00_JPRB, 0.31717E+00_JPRB, 0.30596E+00_JPRB, 0.29520E+00_JPRB, 0.28520E+00_JPRB, 0.27933E+00_JPRB ,&
&  0.27330E+00_JPRB, 0.27263E+00_JPRB, 0.27195E+00_JPRB, 0.27607E+00_JPRB, 0.28717E+00_JPRB, 0.30610E+00_JPRB, 0.33192E+00_JPRB ,&
&  0.37086E+00_JPRB, 0.37697E+00_JPRB, 0.36212E+00_JPRB, 0.35575E+00_JPRB, 0.34842E+00_JPRB, 0.34200E+00_JPRB, 0.32812E+00_JPRB ,&
&  0.33537E+00_JPRB, 0.34804E+00_JPRB, 0.36374E+00_JPRB, 0.35325E+00_JPRB, 0.30803E+00_JPRB, 0.28767E+00_JPRB, 0.26419E+00_JPRB ,&
&  0.25172E+00_JPRB, 0.24154E+00_JPRB, 0.23363E+00_JPRB, 0.22618E+00_JPRB, 0.21865E+00_JPRB, 0.21396E+00_JPRB, 0.20879E+00_JPRB ,&
&  0.20504E+00_JPRB, 0.20340E+00_JPRB, 0.20099E+00_JPRB, 0.20085E+00_JPRB, 0.20061E+00_JPRB, 0.19980E+00_JPRB, 0.19897E+00_JPRB ,&
&  0.19766E+00_JPRB, 0.19625E+00_JPRB, 0.19557E+00_JPRB, 0.19606E+00_JPRB, 0.19695E+00_JPRB, 0.19782E+00_JPRB, 0.19601E+00_JPRB ,&
&  0.19475E+00_JPRB, 0.19350E+00_JPRB, 0.19336E+00_JPRB, 0.19447E+00_JPRB, 0.19483E+00_JPRB, 0.19556E+00_JPRB, 0.19567E+00_JPRB ,&
&  0.19573E+00_JPRB, 0.19623E+00_JPRB, 0.19590E+00_JPRB, 0.19655E+00_JPRB, 0.19683E+00_JPRB, 0.19792E+00_JPRB, 0.19873E+00_JPRB ,&
&  0.19961E+00_JPRB, 0.20158E+00_JPRB, 0.20366E+00_JPRB, 0.20660E+00_JPRB, 0.21060E+00_JPRB, 0.21337E+00_JPRB, 0.21777E+00_JPRB ,&
&  0.22313E+00_JPRB, 0.23181E+00_JPRB, 0.24795E+00_JPRB, 0.26608E+00_JPRB, 0.28843E+00_JPRB, 0.30004E+00_JPRB, 0.30907E+00_JPRB ,&
&  0.32300E+00_JPRB, 0.35724E+00_JPRB, 0.40774E+00_JPRB, 0.42817E+00_JPRB, 0.43299E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.49723E+00_JPRB, 0.50473E+00_JPRB, 0.51598E+00_JPRB, 0.52068E+00_JPRB, 0.52891E+00_JPRB, 0.51731E+00_JPRB, 0.51043E+00_JPRB ,&
&  0.48032E+00_JPRB, 0.45790E+00_JPRB, 0.42683E+00_JPRB, 0.38764E+00_JPRB, 0.36143E+00_JPRB, 0.34110E+00_JPRB, 0.33419E+00_JPRB ,&
&  0.32693E+00_JPRB, 0.32223E+00_JPRB, 0.31777E+00_JPRB, 0.31602E+00_JPRB, 0.31509E+00_JPRB, 0.31632E+00_JPRB, 0.31625E+00_JPRB ,&
&  0.31278E+00_JPRB, 0.30715E+00_JPRB, 0.30058E+00_JPRB, 0.30362E+00_JPRB, 0.31651E+00_JPRB, 0.33148E+00_JPRB, 0.34901E+00_JPRB ,&
&  0.36919E+00_JPRB, 0.38007E+00_JPRB, 0.39024E+00_JPRB, 0.39440E+00_JPRB, 0.37903E+00_JPRB, 0.38157E+00_JPRB, 0.38417E+00_JPRB ,&
&  0.39215E+00_JPRB, 0.39927E+00_JPRB, 0.39651E+00_JPRB, 0.39251E+00_JPRB, 0.34199E+00_JPRB, 0.30879E+00_JPRB, 0.28027E+00_JPRB ,&
&  0.25887E+00_JPRB, 0.24512E+00_JPRB, 0.23389E+00_JPRB, 0.22440E+00_JPRB, 0.21495E+00_JPRB, 0.20883E+00_JPRB, 0.20423E+00_JPRB ,&
&  0.20047E+00_JPRB, 0.19876E+00_JPRB, 0.19709E+00_JPRB, 0.19650E+00_JPRB, 0.19671E+00_JPRB, 0.19604E+00_JPRB, 0.19582E+00_JPRB ,&
&  0.19545E+00_JPRB, 0.19461E+00_JPRB, 0.19446E+00_JPRB, 0.19544E+00_JPRB, 0.19727E+00_JPRB, 0.20034E+00_JPRB, 0.20149E+00_JPRB ,&
&  0.20236E+00_JPRB, 0.20288E+00_JPRB, 0.20293E+00_JPRB, 0.20473E+00_JPRB, 0.20566E+00_JPRB, 0.20582E+00_JPRB, 0.20421E+00_JPRB ,&
&  0.20255E+00_JPRB, 0.20131E+00_JPRB, 0.20107E+00_JPRB, 0.20146E+00_JPRB, 0.20274E+00_JPRB, 0.20479E+00_JPRB, 0.20528E+00_JPRB ,&
&  0.20611E+00_JPRB, 0.20577E+00_JPRB, 0.20551E+00_JPRB, 0.20722E+00_JPRB, 0.20924E+00_JPRB, 0.21147E+00_JPRB, 0.21448E+00_JPRB ,&
&  0.21630E+00_JPRB, 0.22315E+00_JPRB, 0.23560E+00_JPRB, 0.25468E+00_JPRB, 0.28360E+00_JPRB, 0.29999E+00_JPRB, 0.33930E+00_JPRB ,&
&  0.40304E+00_JPRB, 0.54750E+00_JPRB, 0.61160E+00_JPRB, 0.46736E+00_JPRB, 0.48197E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.52510E+00_JPRB, 0.53649E+00_JPRB, 0.54910E+00_JPRB, 0.54108E+00_JPRB, 0.52742E+00_JPRB, 0.51978E+00_JPRB, 0.51952E+00_JPRB ,&
&  0.51909E+00_JPRB, 0.51633E+00_JPRB, 0.50418E+00_JPRB, 0.49191E+00_JPRB, 0.47409E+00_JPRB, 0.42580E+00_JPRB, 0.40732E+00_JPRB ,&
&  0.38976E+00_JPRB, 0.39220E+00_JPRB, 0.39689E+00_JPRB, 0.40365E+00_JPRB, 0.41795E+00_JPRB, 0.42901E+00_JPRB, 0.43135E+00_JPRB ,&
&  0.42170E+00_JPRB, 0.40353E+00_JPRB, 0.38619E+00_JPRB, 0.37257E+00_JPRB, 0.37222E+00_JPRB, 0.37244E+00_JPRB, 0.36565E+00_JPRB ,&
&  0.35202E+00_JPRB, 0.34578E+00_JPRB, 0.34532E+00_JPRB, 0.36534E+00_JPRB, 0.41830E+00_JPRB, 0.43823E+00_JPRB, 0.41273E+00_JPRB ,&
&  0.41698E+00_JPRB, 0.41220E+00_JPRB, 0.40823E+00_JPRB, 0.40972E+00_JPRB, 0.39987E+00_JPRB, 0.38145E+00_JPRB, 0.34772E+00_JPRB ,&
&  0.30364E+00_JPRB, 0.28076E+00_JPRB, 0.25982E+00_JPRB, 0.24043E+00_JPRB, 0.22577E+00_JPRB, 0.21717E+00_JPRB, 0.20983E+00_JPRB ,&
&  0.20503E+00_JPRB, 0.20101E+00_JPRB, 0.19833E+00_JPRB, 0.19806E+00_JPRB, 0.19928E+00_JPRB, 0.19981E+00_JPRB, 0.20053E+00_JPRB ,&
&  0.20171E+00_JPRB, 0.20227E+00_JPRB, 0.20253E+00_JPRB, 0.20356E+00_JPRB, 0.20486E+00_JPRB, 0.20729E+00_JPRB, 0.21078E+00_JPRB ,&
&  0.21509E+00_JPRB, 0.21880E+00_JPRB, 0.22238E+00_JPRB, 0.22822E+00_JPRB, 0.22870E+00_JPRB, 0.22756E+00_JPRB, 0.21988E+00_JPRB ,&
&  0.21434E+00_JPRB, 0.21183E+00_JPRB, 0.20927E+00_JPRB, 0.20853E+00_JPRB, 0.20953E+00_JPRB, 0.21129E+00_JPRB, 0.21306E+00_JPRB ,&
&  0.21427E+00_JPRB, 0.21433E+00_JPRB, 0.21362E+00_JPRB, 0.21363E+00_JPRB, 0.21427E+00_JPRB, 0.21446E+00_JPRB, 0.21786E+00_JPRB ,&
&  0.22530E+00_JPRB, 0.23631E+00_JPRB, 0.24854E+00_JPRB, 0.26549E+00_JPRB, 0.28134E+00_JPRB, 0.29781E+00_JPRB, 0.32536E+00_JPRB ,&
&  0.36521E+00_JPRB, 0.47382E+00_JPRB, 0.54345E+00_JPRB, 0.48079E+00_JPRB, 0.50465E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.55646E+00_JPRB, 0.56018E+00_JPRB, 0.56450E+00_JPRB, 0.56698E+00_JPRB, 0.56164E+00_JPRB, 0.55444E+00_JPRB, 0.54981E+00_JPRB ,&
&  0.55494E+00_JPRB, 0.55485E+00_JPRB, 0.53140E+00_JPRB, 0.51298E+00_JPRB, 0.50237E+00_JPRB, 0.49879E+00_JPRB, 0.49568E+00_JPRB ,&
&  0.48958E+00_JPRB, 0.48751E+00_JPRB, 0.48882E+00_JPRB, 0.50023E+00_JPRB, 0.50968E+00_JPRB, 0.51794E+00_JPRB, 0.52542E+00_JPRB ,&
&  0.52116E+00_JPRB, 0.53420E+00_JPRB, 0.55596E+00_JPRB, 0.55534E+00_JPRB, 0.55277E+00_JPRB, 0.54573E+00_JPRB, 0.53142E+00_JPRB ,&
&  0.48366E+00_JPRB, 0.41959E+00_JPRB, 0.35740E+00_JPRB, 0.36865E+00_JPRB, 0.44665E+00_JPRB, 0.45644E+00_JPRB, 0.44057E+00_JPRB ,&
&  0.45089E+00_JPRB, 0.44686E+00_JPRB, 0.42900E+00_JPRB, 0.41591E+00_JPRB, 0.40047E+00_JPRB, 0.39041E+00_JPRB, 0.38346E+00_JPRB ,&
&  0.38066E+00_JPRB, 0.37933E+00_JPRB, 0.36486E+00_JPRB, 0.31686E+00_JPRB, 0.28856E+00_JPRB, 0.27143E+00_JPRB, 0.25387E+00_JPRB ,&
&  0.24013E+00_JPRB, 0.23131E+00_JPRB, 0.22965E+00_JPRB, 0.22983E+00_JPRB, 0.23424E+00_JPRB, 0.23599E+00_JPRB, 0.23762E+00_JPRB ,&
&  0.23723E+00_JPRB, 0.23599E+00_JPRB, 0.23192E+00_JPRB, 0.22737E+00_JPRB, 0.22425E+00_JPRB, 0.22370E+00_JPRB, 0.22345E+00_JPRB ,&
&  0.22599E+00_JPRB, 0.22994E+00_JPRB, 0.24610E+00_JPRB, 0.26355E+00_JPRB, 0.26308E+00_JPRB, 0.24791E+00_JPRB, 0.23301E+00_JPRB ,&
&  0.22347E+00_JPRB, 0.21858E+00_JPRB, 0.21552E+00_JPRB, 0.21498E+00_JPRB, 0.21770E+00_JPRB, 0.22152E+00_JPRB, 0.22256E+00_JPRB ,&
&  0.22249E+00_JPRB, 0.22324E+00_JPRB, 0.22339E+00_JPRB, 0.22392E+00_JPRB, 0.22464E+00_JPRB, 0.22714E+00_JPRB, 0.23180E+00_JPRB ,&
&  0.23736E+00_JPRB, 0.24732E+00_JPRB, 0.25867E+00_JPRB, 0.27391E+00_JPRB, 0.28501E+00_JPRB, 0.29308E+00_JPRB, 0.30329E+00_JPRB ,&
&  0.32067E+00_JPRB, 0.35881E+00_JPRB, 0.42782E+00_JPRB, 0.55408E+00_JPRB, 0.54913E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.61370E+00_JPRB, 0.60183E+00_JPRB, 0.59392E+00_JPRB, 0.58841E+00_JPRB, 0.58174E+00_JPRB, 0.58063E+00_JPRB, 0.58002E+00_JPRB ,&
&  0.57428E+00_JPRB, 0.56589E+00_JPRB, 0.56672E+00_JPRB, 0.54935E+00_JPRB, 0.52468E+00_JPRB, 0.50815E+00_JPRB, 0.50591E+00_JPRB ,&
&  0.50989E+00_JPRB, 0.51713E+00_JPRB, 0.52634E+00_JPRB, 0.53531E+00_JPRB, 0.54032E+00_JPRB, 0.55207E+00_JPRB, 0.57308E+00_JPRB ,&
&  0.59760E+00_JPRB, 0.61146E+00_JPRB, 0.61580E+00_JPRB, 0.60792E+00_JPRB, 0.59556E+00_JPRB, 0.58271E+00_JPRB, 0.57189E+00_JPRB ,&
&  0.56381E+00_JPRB, 0.46177E+00_JPRB, 0.39313E+00_JPRB, 0.39934E+00_JPRB, 0.46958E+00_JPRB, 0.45354E+00_JPRB, 0.45133E+00_JPRB ,&
&  0.46382E+00_JPRB, 0.46421E+00_JPRB, 0.45112E+00_JPRB, 0.43416E+00_JPRB, 0.41826E+00_JPRB, 0.40667E+00_JPRB, 0.39808E+00_JPRB ,&
&  0.39402E+00_JPRB, 0.40057E+00_JPRB, 0.40882E+00_JPRB, 0.42161E+00_JPRB, 0.42502E+00_JPRB, 0.41254E+00_JPRB, 0.39750E+00_JPRB ,&
&  0.38294E+00_JPRB, 0.37775E+00_JPRB, 0.37998E+00_JPRB, 0.38613E+00_JPRB, 0.39424E+00_JPRB, 0.40057E+00_JPRB, 0.39851E+00_JPRB ,&
&  0.36947E+00_JPRB, 0.34656E+00_JPRB, 0.31847E+00_JPRB, 0.29202E+00_JPRB, 0.26902E+00_JPRB, 0.25353E+00_JPRB, 0.24561E+00_JPRB ,&
&  0.24354E+00_JPRB, 0.25045E+00_JPRB, 0.27155E+00_JPRB, 0.32040E+00_JPRB, 0.38864E+00_JPRB, 0.30992E+00_JPRB, 0.27079E+00_JPRB ,&
&  0.24913E+00_JPRB, 0.23618E+00_JPRB, 0.23136E+00_JPRB, 0.23205E+00_JPRB, 0.23598E+00_JPRB, 0.24181E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.25006E+00_JPRB, 0.25349E+00_JPRB, 0.25559E+00_JPRB, 0.25664E+00_JPRB, 0.25827E+00_JPRB, 0.25940E+00_JPRB, 0.25837E+00_JPRB ,&
&  0.25678E+00_JPRB, 0.26261E+00_JPRB, 0.27969E+00_JPRB, 0.30673E+00_JPRB, 0.33883E+00_JPRB, 0.37355E+00_JPRB, 0.42545E+00_JPRB ,&
&  0.49117E+00_JPRB, 0.56700E+00_JPRB, 0.58468E+00_JPRB, 0.59692E+00_JPRB, 0.60723E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.65372E+00_JPRB, 0.62798E+00_JPRB, 0.61086E+00_JPRB, 0.59748E+00_JPRB, 0.57923E+00_JPRB, 0.56538E+00_JPRB, 0.55979E+00_JPRB ,&
&  0.56799E+00_JPRB, 0.56667E+00_JPRB, 0.55690E+00_JPRB, 0.54867E+00_JPRB, 0.53268E+00_JPRB, 0.51838E+00_JPRB, 0.51252E+00_JPRB ,&
&  0.52112E+00_JPRB, 0.53747E+00_JPRB, 0.55748E+00_JPRB, 0.56588E+00_JPRB, 0.56537E+00_JPRB, 0.57383E+00_JPRB, 0.59305E+00_JPRB ,&
&  0.62157E+00_JPRB, 0.64150E+00_JPRB, 0.64866E+00_JPRB, 0.63567E+00_JPRB, 0.61823E+00_JPRB, 0.61009E+00_JPRB, 0.59569E+00_JPRB ,&
&  0.57896E+00_JPRB, 0.58249E+00_JPRB, 0.51475E+00_JPRB, 0.50725E+00_JPRB, 0.48389E+00_JPRB, 0.46954E+00_JPRB, 0.46590E+00_JPRB ,&
&  0.47918E+00_JPRB, 0.48339E+00_JPRB, 0.47521E+00_JPRB, 0.46325E+00_JPRB, 0.45492E+00_JPRB, 0.44366E+00_JPRB, 0.43315E+00_JPRB ,&
&  0.42574E+00_JPRB, 0.42819E+00_JPRB, 0.43718E+00_JPRB, 0.44647E+00_JPRB, 0.45756E+00_JPRB, 0.46816E+00_JPRB, 0.46464E+00_JPRB ,&
&  0.45294E+00_JPRB, 0.45081E+00_JPRB, 0.45754E+00_JPRB, 0.47182E+00_JPRB, 0.48739E+00_JPRB, 0.49469E+00_JPRB, 0.49853E+00_JPRB ,&
&  0.49090E+00_JPRB, 0.47412E+00_JPRB, 0.45971E+00_JPRB, 0.44780E+00_JPRB, 0.42238E+00_JPRB, 0.35895E+00_JPRB, 0.33610E+00_JPRB ,&
&  0.34676E+00_JPRB, 0.40559E+00_JPRB, 0.47603E+00_JPRB, 0.46726E+00_JPRB, 0.46255E+00_JPRB, 0.44849E+00_JPRB, 0.41265E+00_JPRB ,&
&  0.37184E+00_JPRB, 0.31035E+00_JPRB, 0.28861E+00_JPRB, 0.28493E+00_JPRB, 0.28391E+00_JPRB, 0.28929E+00_JPRB, 0.30140E+00_JPRB ,&
&  0.31739E+00_JPRB, 0.34456E+00_JPRB, 0.36205E+00_JPRB, 0.36880E+00_JPRB, 0.36743E+00_JPRB, 0.35139E+00_JPRB, 0.33763E+00_JPRB ,&
&  0.32832E+00_JPRB, 0.35295E+00_JPRB, 0.41125E+00_JPRB, 0.49405E+00_JPRB, 0.49273E+00_JPRB, 0.51700E+00_JPRB, 0.54168E+00_JPRB ,&
&  0.56527E+00_JPRB, 0.60489E+00_JPRB, 0.63107E+00_JPRB, 0.63696E+00_JPRB, 0.65538E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.67786E+00_JPRB, 0.65009E+00_JPRB, 0.62089E+00_JPRB, 0.59594E+00_JPRB, 0.58379E+00_JPRB, 0.56613E+00_JPRB, 0.55137E+00_JPRB ,&
&  0.55052E+00_JPRB, 0.55843E+00_JPRB, 0.55386E+00_JPRB, 0.54796E+00_JPRB, 0.53859E+00_JPRB, 0.53085E+00_JPRB, 0.52162E+00_JPRB ,&
&  0.51768E+00_JPRB, 0.52941E+00_JPRB, 0.55524E+00_JPRB, 0.56826E+00_JPRB, 0.57464E+00_JPRB, 0.60525E+00_JPRB, 0.63658E+00_JPRB ,&
&  0.64565E+00_JPRB, 0.65952E+00_JPRB, 0.66375E+00_JPRB, 0.63878E+00_JPRB, 0.62244E+00_JPRB, 0.61607E+00_JPRB, 0.61990E+00_JPRB ,&
&  0.61514E+00_JPRB, 0.61935E+00_JPRB, 0.61130E+00_JPRB, 0.54111E+00_JPRB, 0.54038E+00_JPRB, 0.50971E+00_JPRB, 0.49929E+00_JPRB ,&
&  0.49426E+00_JPRB, 0.49982E+00_JPRB, 0.49625E+00_JPRB, 0.48735E+00_JPRB, 0.48377E+00_JPRB, 0.48692E+00_JPRB, 0.48810E+00_JPRB ,&
&  0.48339E+00_JPRB, 0.47262E+00_JPRB, 0.47700E+00_JPRB, 0.48340E+00_JPRB, 0.48715E+00_JPRB, 0.49427E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.49463E+00_JPRB, 0.48558E+00_JPRB, 0.47693E+00_JPRB, 0.47706E+00_JPRB, 0.48338E+00_JPRB, 0.49759E+00_JPRB, 0.50245E+00_JPRB ,&
&  0.50028E+00_JPRB, 0.49891E+00_JPRB, 0.49978E+00_JPRB, 0.50604E+00_JPRB, 0.52099E+00_JPRB, 0.53596E+00_JPRB, 0.53725E+00_JPRB ,&
&  0.53830E+00_JPRB, 0.54272E+00_JPRB, 0.54743E+00_JPRB, 0.54945E+00_JPRB, 0.54560E+00_JPRB, 0.53053E+00_JPRB, 0.51308E+00_JPRB ,&
&  0.49653E+00_JPRB, 0.47996E+00_JPRB, 0.46591E+00_JPRB, 0.46138E+00_JPRB, 0.47442E+00_JPRB, 0.50355E+00_JPRB, 0.51981E+00_JPRB ,&
&  0.52362E+00_JPRB, 0.52567E+00_JPRB, 0.52918E+00_JPRB, 0.53045E+00_JPRB, 0.53267E+00_JPRB, 0.53089E+00_JPRB, 0.52454E+00_JPRB ,&
&  0.51338E+00_JPRB, 0.50211E+00_JPRB, 0.49914E+00_JPRB, 0.50315E+00_JPRB, 0.50807E+00_JPRB, 0.51981E+00_JPRB, 0.53610E+00_JPRB ,&
&  0.55791E+00_JPRB, 0.59615E+00_JPRB, 0.62865E+00_JPRB, 0.65392E+00_JPRB, 0.67214E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.69673E+00_JPRB, 0.69461E+00_JPRB, 0.66536E+00_JPRB, 0.63343E+00_JPRB, 0.61436E+00_JPRB, 0.59332E+00_JPRB, 0.57647E+00_JPRB ,&
&  0.57506E+00_JPRB, 0.57249E+00_JPRB, 0.57051E+00_JPRB, 0.56807E+00_JPRB, 0.56631E+00_JPRB, 0.56312E+00_JPRB, 0.55091E+00_JPRB ,&
&  0.54640E+00_JPRB, 0.55297E+00_JPRB, 0.57544E+00_JPRB, 0.60079E+00_JPRB, 0.62119E+00_JPRB, 0.64761E+00_JPRB, 0.65875E+00_JPRB ,&
&  0.67039E+00_JPRB, 0.68277E+00_JPRB, 0.68505E+00_JPRB, 0.65245E+00_JPRB, 0.62560E+00_JPRB, 0.62670E+00_JPRB, 0.63775E+00_JPRB ,&
&  0.63807E+00_JPRB, 0.64124E+00_JPRB, 0.64476E+00_JPRB, 0.64120E+00_JPRB, 0.62053E+00_JPRB, 0.58234E+00_JPRB, 0.55231E+00_JPRB ,&
&  0.52770E+00_JPRB, 0.50871E+00_JPRB, 0.50304E+00_JPRB, 0.49187E+00_JPRB, 0.49034E+00_JPRB, 0.50036E+00_JPRB, 0.51327E+00_JPRB ,&
&  0.52660E+00_JPRB, 0.53255E+00_JPRB, 0.53950E+00_JPRB, 0.53610E+00_JPRB, 0.53172E+00_JPRB, 0.53434E+00_JPRB, 0.53409E+00_JPRB ,&
&  0.53332E+00_JPRB, 0.52418E+00_JPRB, 0.52144E+00_JPRB, 0.51443E+00_JPRB, 0.51042E+00_JPRB, 0.51072E+00_JPRB, 0.51355E+00_JPRB ,&
&  0.51456E+00_JPRB, 0.51982E+00_JPRB, 0.52135E+00_JPRB, 0.52345E+00_JPRB, 0.53288E+00_JPRB, 0.54953E+00_JPRB, 0.56626E+00_JPRB ,&
&  0.57881E+00_JPRB, 0.58863E+00_JPRB, 0.59641E+00_JPRB, 0.60224E+00_JPRB, 0.60205E+00_JPRB, 0.59744E+00_JPRB, 0.58363E+00_JPRB ,&
&  0.55308E+00_JPRB, 0.52030E+00_JPRB, 0.49548E+00_JPRB, 0.47563E+00_JPRB, 0.47181E+00_JPRB, 0.48261E+00_JPRB, 0.50033E+00_JPRB ,&
&  0.51370E+00_JPRB, 0.51779E+00_JPRB, 0.51192E+00_JPRB, 0.51026E+00_JPRB, 0.51541E+00_JPRB, 0.52506E+00_JPRB, 0.53257E+00_JPRB ,&
&  0.53188E+00_JPRB, 0.52874E+00_JPRB, 0.53247E+00_JPRB, 0.53930E+00_JPRB, 0.55316E+00_JPRB, 0.57079E+00_JPRB, 0.58984E+00_JPRB ,&
&  0.60896E+00_JPRB, 0.62396E+00_JPRB, 0.64348E+00_JPRB, 0.66330E+00_JPRB, 0.68878E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.70335E+00_JPRB, 0.69515E+00_JPRB, 0.68021E+00_JPRB, 0.65324E+00_JPRB, 0.63568E+00_JPRB, 0.60857E+00_JPRB, 0.59328E+00_JPRB ,&
&  0.59553E+00_JPRB, 0.59681E+00_JPRB, 0.60195E+00_JPRB, 0.61130E+00_JPRB, 0.60935E+00_JPRB, 0.60478E+00_JPRB, 0.60024E+00_JPRB ,&
&  0.60332E+00_JPRB, 0.60377E+00_JPRB, 0.61229E+00_JPRB, 0.62788E+00_JPRB, 0.64429E+00_JPRB, 0.65352E+00_JPRB, 0.66315E+00_JPRB ,&
&  0.67056E+00_JPRB, 0.66433E+00_JPRB, 0.65998E+00_JPRB, 0.62964E+00_JPRB, 0.61202E+00_JPRB, 0.61717E+00_JPRB, 0.62453E+00_JPRB ,&
&  0.63458E+00_JPRB, 0.64510E+00_JPRB, 0.65519E+00_JPRB, 0.66099E+00_JPRB, 0.64731E+00_JPRB, 0.64360E+00_JPRB, 0.63294E+00_JPRB ,&
&  0.59067E+00_JPRB, 0.54925E+00_JPRB, 0.52686E+00_JPRB, 0.51949E+00_JPRB, 0.51462E+00_JPRB, 0.52060E+00_JPRB, 0.52779E+00_JPRB ,&
&  0.54055E+00_JPRB, 0.55229E+00_JPRB, 0.55534E+00_JPRB, 0.55864E+00_JPRB, 0.56024E+00_JPRB, 0.55936E+00_JPRB, 0.55962E+00_JPRB ,&
&  0.55664E+00_JPRB, 0.55473E+00_JPRB, 0.56180E+00_JPRB, 0.56232E+00_JPRB, 0.55353E+00_JPRB, 0.55102E+00_JPRB, 0.54899E+00_JPRB ,&
&  0.54631E+00_JPRB, 0.54363E+00_JPRB, 0.54045E+00_JPRB, 0.55047E+00_JPRB, 0.57598E+00_JPRB, 0.60527E+00_JPRB, 0.63063E+00_JPRB ,&
&  0.64509E+00_JPRB, 0.65681E+00_JPRB, 0.65997E+00_JPRB, 0.66205E+00_JPRB, 0.66158E+00_JPRB, 0.65289E+00_JPRB, 0.64973E+00_JPRB ,&
&  0.64200E+00_JPRB, 0.62295E+00_JPRB, 0.59747E+00_JPRB, 0.55785E+00_JPRB, 0.51936E+00_JPRB, 0.49742E+00_JPRB, 0.49701E+00_JPRB ,&
&  0.50917E+00_JPRB, 0.52161E+00_JPRB, 0.52804E+00_JPRB, 0.53619E+00_JPRB, 0.55108E+00_JPRB, 0.56894E+00_JPRB, 0.58773E+00_JPRB ,&
&  0.59956E+00_JPRB, 0.59966E+00_JPRB, 0.59721E+00_JPRB, 0.60533E+00_JPRB, 0.61635E+00_JPRB, 0.63439E+00_JPRB, 0.65408E+00_JPRB ,&
&  0.67148E+00_JPRB, 0.67129E+00_JPRB, 0.67183E+00_JPRB, 0.67326E+00_JPRB, 0.69298E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.70898E+00_JPRB, 0.71011E+00_JPRB, 0.69421E+00_JPRB, 0.68234E+00_JPRB, 0.66542E+00_JPRB, 0.64397E+00_JPRB, 0.62496E+00_JPRB ,&
&  0.62625E+00_JPRB, 0.64114E+00_JPRB, 0.65557E+00_JPRB, 0.66605E+00_JPRB, 0.68151E+00_JPRB, 0.68471E+00_JPRB, 0.69133E+00_JPRB ,&
&  0.68741E+00_JPRB, 0.65053E+00_JPRB, 0.64473E+00_JPRB, 0.65241E+00_JPRB, 0.65423E+00_JPRB, 0.66003E+00_JPRB, 0.67062E+00_JPRB ,&
&  0.68037E+00_JPRB, 0.67561E+00_JPRB, 0.66743E+00_JPRB, 0.63786E+00_JPRB, 0.63111E+00_JPRB, 0.63595E+00_JPRB, 0.64085E+00_JPRB ,&
&  0.65110E+00_JPRB, 0.66771E+00_JPRB, 0.67162E+00_JPRB, 0.68011E+00_JPRB, 0.68914E+00_JPRB, 0.69328E+00_JPRB, 0.69027E+00_JPRB ,&
&  0.66513E+00_JPRB, 0.63959E+00_JPRB, 0.62899E+00_JPRB, 0.61167E+00_JPRB, 0.58978E+00_JPRB, 0.57438E+00_JPRB, 0.56643E+00_JPRB ,&
&  0.56203E+00_JPRB, 0.56694E+00_JPRB, 0.57313E+00_JPRB, 0.58710E+00_JPRB, 0.59229E+00_JPRB, 0.59641E+00_JPRB, 0.60162E+00_JPRB ,&
&  0.60096E+00_JPRB, 0.59658E+00_JPRB, 0.59636E+00_JPRB, 0.60603E+00_JPRB, 0.61192E+00_JPRB, 0.60969E+00_JPRB, 0.60531E+00_JPRB ,&
&  0.59239E+00_JPRB, 0.57824E+00_JPRB, 0.58537E+00_JPRB, 0.60027E+00_JPRB, 0.60717E+00_JPRB, 0.63242E+00_JPRB, 0.66503E+00_JPRB ,&
&  0.69266E+00_JPRB, 0.71075E+00_JPRB, 0.71048E+00_JPRB, 0.70717E+00_JPRB, 0.70598E+00_JPRB, 0.71353E+00_JPRB, 0.72589E+00_JPRB ,&
&  0.71016E+00_JPRB, 0.70093E+00_JPRB, 0.69883E+00_JPRB, 0.67972E+00_JPRB, 0.64360E+00_JPRB, 0.61273E+00_JPRB, 0.58989E+00_JPRB ,&
&  0.57157E+00_JPRB, 0.56115E+00_JPRB, 0.56483E+00_JPRB, 0.56848E+00_JPRB, 0.58143E+00_JPRB, 0.61716E+00_JPRB, 0.65443E+00_JPRB ,&
&  0.69489E+00_JPRB, 0.71592E+00_JPRB, 0.72275E+00_JPRB, 0.73107E+00_JPRB, 0.73740E+00_JPRB, 0.72728E+00_JPRB, 0.71884E+00_JPRB ,&
&  0.71676E+00_JPRB, 0.70863E+00_JPRB, 0.69934E+00_JPRB, 0.68942E+00_JPRB, 0.69639E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.70715E+00_JPRB, 0.72849E+00_JPRB, 0.69492E+00_JPRB, 0.67558E+00_JPRB, 0.65434E+00_JPRB, 0.64921E+00_JPRB, 0.63970E+00_JPRB ,&
&  0.64990E+00_JPRB, 0.66650E+00_JPRB, 0.69005E+00_JPRB, 0.69789E+00_JPRB, 0.70471E+00_JPRB, 0.70749E+00_JPRB, 0.72453E+00_JPRB ,&
&  0.72000E+00_JPRB, 0.71182E+00_JPRB, 0.71504E+00_JPRB, 0.70477E+00_JPRB, 0.69603E+00_JPRB, 0.70073E+00_JPRB, 0.70934E+00_JPRB ,&
&  0.71920E+00_JPRB, 0.71050E+00_JPRB, 0.69318E+00_JPRB, 0.66653E+00_JPRB, 0.65389E+00_JPRB, 0.65122E+00_JPRB, 0.65873E+00_JPRB ,&
&  0.66423E+00_JPRB, 0.68824E+00_JPRB, 0.62587E+00_JPRB, 0.68255E+00_JPRB, 0.69114E+00_JPRB, 0.68975E+00_JPRB, 0.68904E+00_JPRB ,&
&  0.69451E+00_JPRB, 0.68450E+00_JPRB, 0.67581E+00_JPRB, 0.66690E+00_JPRB, 0.64390E+00_JPRB, 0.62532E+00_JPRB, 0.60595E+00_JPRB ,&
&  0.58881E+00_JPRB, 0.57825E+00_JPRB, 0.58334E+00_JPRB, 0.60073E+00_JPRB, 0.61849E+00_JPRB, 0.62840E+00_JPRB, 0.63447E+00_JPRB ,&
&  0.63771E+00_JPRB, 0.63031E+00_JPRB, 0.62017E+00_JPRB, 0.62006E+00_JPRB, 0.62641E+00_JPRB, 0.63127E+00_JPRB, 0.63448E+00_JPRB ,&
&  0.63503E+00_JPRB, 0.61177E+00_JPRB, 0.59858E+00_JPRB, 0.61464E+00_JPRB, 0.63199E+00_JPRB, 0.65093E+00_JPRB, 0.67571E+00_JPRB ,&
&  0.69443E+00_JPRB, 0.70167E+00_JPRB, 0.70086E+00_JPRB, 0.70763E+00_JPRB, 0.71469E+00_JPRB, 0.72357E+00_JPRB, 0.73068E+00_JPRB ,&
&  0.72079E+00_JPRB, 0.71856E+00_JPRB, 0.71933E+00_JPRB, 0.71926E+00_JPRB, 0.71043E+00_JPRB, 0.68780E+00_JPRB, 0.65594E+00_JPRB ,&
&  0.64131E+00_JPRB, 0.62634E+00_JPRB, 0.61525E+00_JPRB, 0.61245E+00_JPRB, 0.61480E+00_JPRB, 0.64095E+00_JPRB, 0.67816E+00_JPRB ,&
&  0.71521E+00_JPRB, 0.74565E+00_JPRB, 0.77203E+00_JPRB, 0.79617E+00_JPRB, 0.79491E+00_JPRB, 0.77441E+00_JPRB, 0.76632E+00_JPRB ,&
&  0.75283E+00_JPRB, 0.73769E+00_JPRB, 0.72666E+00_JPRB, 0.71744E+00_JPRB, 0.70681E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.75156E+00_JPRB, 0.75152E+00_JPRB, 0.74024E+00_JPRB, 0.70009E+00_JPRB, 0.66947E+00_JPRB, 0.67811E+00_JPRB, 0.68668E+00_JPRB ,&
&  0.68569E+00_JPRB, 0.70012E+00_JPRB, 0.72494E+00_JPRB, 0.75014E+00_JPRB, 0.76699E+00_JPRB, 0.76683E+00_JPRB, 0.77089E+00_JPRB ,&
&  0.76839E+00_JPRB, 0.76625E+00_JPRB, 0.77617E+00_JPRB, 0.78105E+00_JPRB, 0.77038E+00_JPRB, 0.74931E+00_JPRB, 0.74669E+00_JPRB ,&
&  0.75259E+00_JPRB, 0.74835E+00_JPRB, 0.73006E+00_JPRB, 0.71759E+00_JPRB, 0.70747E+00_JPRB, 0.71263E+00_JPRB, 0.71920E+00_JPRB ,&
&  0.71388E+00_JPRB, 0.66285E+00_JPRB, 0.70116E+00_JPRB, 0.71333E+00_JPRB, 0.72144E+00_JPRB, 0.71073E+00_JPRB, 0.71619E+00_JPRB ,&
&  0.72778E+00_JPRB, 0.73496E+00_JPRB, 0.71521E+00_JPRB, 0.69199E+00_JPRB, 0.67079E+00_JPRB, 0.65291E+00_JPRB, 0.62480E+00_JPRB ,&
&  0.61433E+00_JPRB, 0.61121E+00_JPRB, 0.62429E+00_JPRB, 0.63826E+00_JPRB, 0.65444E+00_JPRB, 0.66094E+00_JPRB, 0.67120E+00_JPRB ,&
&  0.67332E+00_JPRB, 0.66777E+00_JPRB, 0.66270E+00_JPRB, 0.65886E+00_JPRB, 0.65818E+00_JPRB, 0.65240E+00_JPRB, 0.64400E+00_JPRB ,&
&  0.63752E+00_JPRB, 0.63564E+00_JPRB, 0.63986E+00_JPRB, 0.64912E+00_JPRB, 0.66060E+00_JPRB, 0.67832E+00_JPRB, 0.69212E+00_JPRB ,&
&  0.70918E+00_JPRB, 0.71469E+00_JPRB, 0.71950E+00_JPRB, 0.73252E+00_JPRB, 0.74114E+00_JPRB, 0.73989E+00_JPRB, 0.73718E+00_JPRB ,&
&  0.73443E+00_JPRB, 0.72863E+00_JPRB, 0.73291E+00_JPRB, 0.73165E+00_JPRB, 0.73198E+00_JPRB, 0.72363E+00_JPRB, 0.70877E+00_JPRB ,&
&  0.70318E+00_JPRB, 0.69898E+00_JPRB, 0.69054E+00_JPRB, 0.68787E+00_JPRB, 0.68578E+00_JPRB, 0.70359E+00_JPRB, 0.73488E+00_JPRB ,&
&  0.75221E+00_JPRB, 0.77571E+00_JPRB, 0.80894E+00_JPRB, 0.83138E+00_JPRB, 0.83271E+00_JPRB, 0.83459E+00_JPRB, 0.82563E+00_JPRB ,&
&  0.80825E+00_JPRB, 0.79616E+00_JPRB, 0.78305E+00_JPRB, 0.77766E+00_JPRB, 0.77281E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.79445E+00_JPRB, 0.78668E+00_JPRB, 0.77074E+00_JPRB, 0.74198E+00_JPRB, 0.70611E+00_JPRB, 0.71327E+00_JPRB, 0.72461E+00_JPRB ,&
&  0.71817E+00_JPRB, 0.70814E+00_JPRB, 0.71813E+00_JPRB, 0.74266E+00_JPRB, 0.76698E+00_JPRB, 0.78093E+00_JPRB, 0.78329E+00_JPRB ,&
&  0.77839E+00_JPRB, 0.78055E+00_JPRB, 0.78894E+00_JPRB, 0.80596E+00_JPRB, 0.80494E+00_JPRB, 0.78689E+00_JPRB, 0.77575E+00_JPRB ,&
&  0.77252E+00_JPRB, 0.77058E+00_JPRB, 0.76775E+00_JPRB, 0.75085E+00_JPRB, 0.73959E+00_JPRB, 0.73670E+00_JPRB, 0.73728E+00_JPRB ,&
&  0.73385E+00_JPRB, 0.59642E+00_JPRB, 0.70006E+00_JPRB, 0.71271E+00_JPRB, 0.72539E+00_JPRB, 0.72028E+00_JPRB, 0.72072E+00_JPRB ,&
&  0.72533E+00_JPRB, 0.74072E+00_JPRB, 0.73329E+00_JPRB, 0.70843E+00_JPRB, 0.68229E+00_JPRB, 0.65454E+00_JPRB, 0.63556E+00_JPRB ,&
&  0.62578E+00_JPRB, 0.62826E+00_JPRB, 0.63777E+00_JPRB, 0.64797E+00_JPRB, 0.65895E+00_JPRB, 0.66048E+00_JPRB, 0.66428E+00_JPRB ,&
&  0.66686E+00_JPRB, 0.67205E+00_JPRB, 0.67478E+00_JPRB, 0.68073E+00_JPRB, 0.68316E+00_JPRB, 0.68029E+00_JPRB, 0.67319E+00_JPRB ,&
&  0.66307E+00_JPRB, 0.65665E+00_JPRB, 0.65586E+00_JPRB, 0.65825E+00_JPRB, 0.66551E+00_JPRB, 0.67837E+00_JPRB, 0.69133E+00_JPRB ,&
&  0.70598E+00_JPRB, 0.71560E+00_JPRB, 0.72091E+00_JPRB, 0.72351E+00_JPRB, 0.73281E+00_JPRB, 0.74254E+00_JPRB, 0.74225E+00_JPRB ,&
&  0.73553E+00_JPRB, 0.72800E+00_JPRB, 0.72562E+00_JPRB, 0.72219E+00_JPRB, 0.72249E+00_JPRB, 0.72317E+00_JPRB, 0.71908E+00_JPRB ,&
&  0.71114E+00_JPRB, 0.70957E+00_JPRB, 0.71096E+00_JPRB, 0.70033E+00_JPRB, 0.70208E+00_JPRB, 0.71231E+00_JPRB, 0.73154E+00_JPRB ,&
&  0.75568E+00_JPRB, 0.78748E+00_JPRB, 0.81887E+00_JPRB, 0.84906E+00_JPRB, 0.86994E+00_JPRB, 0.88131E+00_JPRB, 0.88068E+00_JPRB ,&
&  0.86624E+00_JPRB, 0.84271E+00_JPRB, 0.82266E+00_JPRB, 0.81115E+00_JPRB, 0.81028E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.83599E+00_JPRB, 0.82384E+00_JPRB, 0.80819E+00_JPRB, 0.78848E+00_JPRB, 0.77034E+00_JPRB, 0.75940E+00_JPRB, 0.76389E+00_JPRB ,&
&  0.76193E+00_JPRB, 0.74910E+00_JPRB, 0.75821E+00_JPRB, 0.77071E+00_JPRB, 0.78100E+00_JPRB, 0.79092E+00_JPRB, 0.79485E+00_JPRB ,&
&  0.79912E+00_JPRB, 0.79655E+00_JPRB, 0.79978E+00_JPRB, 0.81162E+00_JPRB, 0.82978E+00_JPRB, 0.83110E+00_JPRB, 0.82796E+00_JPRB ,&
&  0.80527E+00_JPRB, 0.79376E+00_JPRB, 0.78326E+00_JPRB, 0.77051E+00_JPRB, 0.75919E+00_JPRB, 0.75550E+00_JPRB, 0.75844E+00_JPRB ,&
&  0.64566E+00_JPRB, 0.55177E+00_JPRB, 0.66219E+00_JPRB, 0.76098E+00_JPRB, 0.76831E+00_JPRB, 0.76764E+00_JPRB, 0.78331E+00_JPRB ,&
&  0.78794E+00_JPRB, 0.78746E+00_JPRB, 0.78463E+00_JPRB, 0.77265E+00_JPRB, 0.74132E+00_JPRB, 0.70799E+00_JPRB, 0.68957E+00_JPRB ,&
&  0.67785E+00_JPRB, 0.68187E+00_JPRB, 0.68277E+00_JPRB, 0.68919E+00_JPRB, 0.68883E+00_JPRB, 0.68141E+00_JPRB, 0.67980E+00_JPRB ,&
&  0.68074E+00_JPRB, 0.69109E+00_JPRB, 0.70098E+00_JPRB, 0.70834E+00_JPRB, 0.71126E+00_JPRB, 0.71238E+00_JPRB, 0.70977E+00_JPRB ,&
&  0.70762E+00_JPRB, 0.70503E+00_JPRB, 0.70085E+00_JPRB, 0.70184E+00_JPRB, 0.70476E+00_JPRB, 0.71314E+00_JPRB, 0.72715E+00_JPRB ,&
&  0.73435E+00_JPRB, 0.73552E+00_JPRB, 0.73688E+00_JPRB, 0.73819E+00_JPRB, 0.74862E+00_JPRB, 0.75661E+00_JPRB, 0.75330E+00_JPRB ,&
&  0.74741E+00_JPRB, 0.74365E+00_JPRB, 0.74892E+00_JPRB, 0.75295E+00_JPRB, 0.75839E+00_JPRB, 0.76048E+00_JPRB, 0.75577E+00_JPRB ,&
&  0.74711E+00_JPRB, 0.73323E+00_JPRB, 0.72287E+00_JPRB, 0.72187E+00_JPRB, 0.73118E+00_JPRB, 0.73856E+00_JPRB, 0.75604E+00_JPRB ,&
&  0.78767E+00_JPRB, 0.82680E+00_JPRB, 0.85990E+00_JPRB, 0.90888E+00_JPRB, 0.94174E+00_JPRB, 0.96118E+00_JPRB, 0.96163E+00_JPRB ,&
&  0.94692E+00_JPRB, 0.90680E+00_JPRB, 0.89324E+00_JPRB, 0.86640E+00_JPRB, 0.84977E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.83623E+00_JPRB, 0.83455E+00_JPRB, 0.83151E+00_JPRB, 0.81626E+00_JPRB, 0.79844E+00_JPRB, 0.79059E+00_JPRB, 0.79513E+00_JPRB ,&
&  0.78930E+00_JPRB, 0.77494E+00_JPRB, 0.77905E+00_JPRB, 0.78222E+00_JPRB, 0.78944E+00_JPRB, 0.79319E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.80252E+00_JPRB, 0.80671E+00_JPRB, 0.80469E+00_JPRB, 0.81263E+00_JPRB, 0.80483E+00_JPRB, 0.79890E+00_JPRB, 0.79568E+00_JPRB ,&
&  0.78692E+00_JPRB, 0.77786E+00_JPRB, 0.77049E+00_JPRB, 0.76041E+00_JPRB, 0.75305E+00_JPRB, 0.74445E+00_JPRB, 0.72921E+00_JPRB ,&
&  0.74119E+00_JPRB, 0.59012E+00_JPRB, 0.67337E+00_JPRB, 0.82077E+00_JPRB, 0.82856E+00_JPRB, 0.82292E+00_JPRB, 0.83219E+00_JPRB ,&
&  0.84885E+00_JPRB, 0.85705E+00_JPRB, 0.84720E+00_JPRB, 0.82216E+00_JPRB, 0.77561E+00_JPRB, 0.75225E+00_JPRB, 0.73792E+00_JPRB ,&
&  0.72195E+00_JPRB, 0.71006E+00_JPRB, 0.70284E+00_JPRB, 0.70401E+00_JPRB, 0.70358E+00_JPRB, 0.70017E+00_JPRB, 0.69686E+00_JPRB ,&
&  0.70061E+00_JPRB, 0.71058E+00_JPRB, 0.71689E+00_JPRB, 0.72304E+00_JPRB, 0.72913E+00_JPRB, 0.72618E+00_JPRB, 0.72535E+00_JPRB ,&
&  0.72353E+00_JPRB, 0.72199E+00_JPRB, 0.71934E+00_JPRB, 0.71900E+00_JPRB, 0.72109E+00_JPRB, 0.73997E+00_JPRB, 0.75566E+00_JPRB ,&
&  0.75751E+00_JPRB, 0.75628E+00_JPRB, 0.75996E+00_JPRB, 0.76582E+00_JPRB, 0.77082E+00_JPRB, 0.77308E+00_JPRB, 0.76784E+00_JPRB ,&
&  0.76219E+00_JPRB, 0.75358E+00_JPRB, 0.75102E+00_JPRB, 0.74937E+00_JPRB, 0.74388E+00_JPRB, 0.74156E+00_JPRB, 0.74406E+00_JPRB ,&
&  0.73844E+00_JPRB, 0.72717E+00_JPRB, 0.72922E+00_JPRB, 0.74253E+00_JPRB, 0.75410E+00_JPRB, 0.76142E+00_JPRB, 0.77796E+00_JPRB ,&
&  0.81043E+00_JPRB, 0.84755E+00_JPRB, 0.87641E+00_JPRB, 0.93368E+00_JPRB, 0.97006E+00_JPRB, 0.99312E+00_JPRB, 0.99903E+00_JPRB ,&
&  0.98534E+00_JPRB, 0.96587E+00_JPRB, 0.69621E+00_JPRB, 0.81048E+00_JPRB, 0.83200E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.86548E+00_JPRB, 0.86695E+00_JPRB, 0.85455E+00_JPRB, 0.83755E+00_JPRB, 0.82498E+00_JPRB, 0.81636E+00_JPRB, 0.81752E+00_JPRB ,&
&  0.81419E+00_JPRB, 0.80738E+00_JPRB, 0.80959E+00_JPRB, 0.81194E+00_JPRB, 0.81472E+00_JPRB, 0.81282E+00_JPRB, 0.81846E+00_JPRB ,&
&  0.81994E+00_JPRB, 0.82190E+00_JPRB, 0.82520E+00_JPRB, 0.81876E+00_JPRB, 0.79872E+00_JPRB, 0.78439E+00_JPRB, 0.76995E+00_JPRB ,&
&  0.76856E+00_JPRB, 0.76826E+00_JPRB, 0.76892E+00_JPRB, 0.76275E+00_JPRB, 0.74326E+00_JPRB, 0.72645E+00_JPRB, 0.71581E+00_JPRB ,&
&  0.71569E+00_JPRB, 0.57347E+00_JPRB, 0.65270E+00_JPRB, 0.86461E+00_JPRB, 0.96589E+00_JPRB, 0.93488E+00_JPRB, 0.92144E+00_JPRB ,&
&  0.92485E+00_JPRB, 0.91983E+00_JPRB, 0.89733E+00_JPRB, 0.86416E+00_JPRB, 0.82084E+00_JPRB, 0.80670E+00_JPRB, 0.78698E+00_JPRB ,&
&  0.76833E+00_JPRB, 0.76114E+00_JPRB, 0.75469E+00_JPRB, 0.75386E+00_JPRB, 0.74376E+00_JPRB, 0.73211E+00_JPRB, 0.73330E+00_JPRB ,&
&  0.74145E+00_JPRB, 0.74122E+00_JPRB, 0.74526E+00_JPRB, 0.74998E+00_JPRB, 0.74588E+00_JPRB, 0.76107E+00_JPRB, 0.76567E+00_JPRB ,&
&  0.76694E+00_JPRB, 0.76414E+00_JPRB, 0.76395E+00_JPRB, 0.76382E+00_JPRB, 0.77213E+00_JPRB, 0.79771E+00_JPRB, 0.81468E+00_JPRB ,&
&  0.80858E+00_JPRB, 0.79869E+00_JPRB, 0.80546E+00_JPRB, 0.80891E+00_JPRB, 0.80197E+00_JPRB, 0.78700E+00_JPRB, 0.77018E+00_JPRB ,&
&  0.76344E+00_JPRB, 0.75844E+00_JPRB, 0.75513E+00_JPRB, 0.74849E+00_JPRB, 0.73938E+00_JPRB, 0.73180E+00_JPRB, 0.72759E+00_JPRB ,&
&  0.72581E+00_JPRB, 0.72557E+00_JPRB, 0.73218E+00_JPRB, 0.74751E+00_JPRB, 0.76779E+00_JPRB, 0.79118E+00_JPRB, 0.81651E+00_JPRB ,&
&  0.84781E+00_JPRB, 0.88441E+00_JPRB, 0.91982E+00_JPRB, 0.95443E+00_JPRB, 0.10425E+01_JPRB, 0.10701E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10765E+01_JPRB, 0.10307E+01_JPRB, 0.96403E+00_JPRB, 0.75611E+00_JPRB, 0.83661E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.91355E+00_JPRB, 0.92709E+00_JPRB, 0.90497E+00_JPRB, 0.86707E+00_JPRB, 0.84554E+00_JPRB, 0.83634E+00_JPRB, 0.83364E+00_JPRB ,&
&  0.82836E+00_JPRB, 0.81675E+00_JPRB, 0.82081E+00_JPRB, 0.82683E+00_JPRB, 0.82417E+00_JPRB, 0.81254E+00_JPRB, 0.80436E+00_JPRB ,&
&  0.80211E+00_JPRB, 0.80578E+00_JPRB, 0.81062E+00_JPRB, 0.80029E+00_JPRB, 0.77892E+00_JPRB, 0.76424E+00_JPRB, 0.75158E+00_JPRB ,&
&  0.74987E+00_JPRB, 0.74992E+00_JPRB, 0.74807E+00_JPRB, 0.75000E+00_JPRB, 0.72951E+00_JPRB, 0.70629E+00_JPRB, 0.68646E+00_JPRB ,&
&  0.67073E+00_JPRB, 0.59919E+00_JPRB, 0.76800E+00_JPRB, 0.10220E+01_JPRB, 0.11461E+01_JPRB, 0.11356E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.97869E+00_JPRB, 0.93498E+00_JPRB, 0.89120E+00_JPRB, 0.86610E+00_JPRB, 0.85151E+00_JPRB, 0.84214E+00_JPRB ,&
&  0.83372E+00_JPRB, 0.82401E+00_JPRB, 0.81704E+00_JPRB, 0.79916E+00_JPRB, 0.78424E+00_JPRB, 0.78135E+00_JPRB, 0.78162E+00_JPRB ,&
&  0.78401E+00_JPRB, 0.77802E+00_JPRB, 0.77737E+00_JPRB, 0.77861E+00_JPRB, 0.76087E+00_JPRB, 0.75556E+00_JPRB, 0.75106E+00_JPRB ,&
&  0.75734E+00_JPRB, 0.77297E+00_JPRB, 0.77899E+00_JPRB, 0.81423E+00_JPRB, 0.83256E+00_JPRB, 0.84804E+00_JPRB, 0.84907E+00_JPRB ,&
&  0.84036E+00_JPRB, 0.83727E+00_JPRB, 0.82900E+00_JPRB, 0.82512E+00_JPRB, 0.81496E+00_JPRB, 0.79188E+00_JPRB, 0.77003E+00_JPRB ,&
&  0.76507E+00_JPRB, 0.76498E+00_JPRB, 0.75511E+00_JPRB, 0.75285E+00_JPRB, 0.75193E+00_JPRB, 0.74377E+00_JPRB, 0.73796E+00_JPRB ,&
&  0.73452E+00_JPRB, 0.73684E+00_JPRB, 0.74027E+00_JPRB, 0.75424E+00_JPRB, 0.77701E+00_JPRB, 0.80501E+00_JPRB, 0.83400E+00_JPRB ,&
&  0.86240E+00_JPRB, 0.89015E+00_JPRB, 0.92892E+00_JPRB, 0.10447E+01_JPRB, 0.10887E+01_JPRB, 0.11109E+01_JPRB, 0.11387E+01_JPRB ,&
&  0.11264E+01_JPRB, 0.10756E+01_JPRB, 0.10227E+01_JPRB, 0.95208E+00_JPRB, 0.86631E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.10362E+01_JPRB, 0.10021E+01_JPRB, 0.96236E+00_JPRB, 0.91402E+00_JPRB, 0.88990E+00_JPRB, 0.86901E+00_JPRB, 0.84891E+00_JPRB ,&
&  0.83815E+00_JPRB, 0.82734E+00_JPRB, 0.81950E+00_JPRB, 0.80393E+00_JPRB, 0.78868E+00_JPRB, 0.77932E+00_JPRB, 0.76828E+00_JPRB ,&
&  0.76848E+00_JPRB, 0.76849E+00_JPRB, 0.76046E+00_JPRB, 0.75210E+00_JPRB, 0.74824E+00_JPRB, 0.74472E+00_JPRB, 0.74828E+00_JPRB ,&
&  0.73995E+00_JPRB, 0.73013E+00_JPRB, 0.73001E+00_JPRB, 0.72305E+00_JPRB, 0.71045E+00_JPRB, 0.69863E+00_JPRB, 0.66651E+00_JPRB ,&
&  0.66543E+00_JPRB, 0.71087E+00_JPRB, 0.10507E+01_JPRB, 0.14307E+01_JPRB, 0.14281E+01_JPRB, 0.13691E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.10854E+01_JPRB, 0.99652E+00_JPRB, 0.94044E+00_JPRB, 0.92512E+00_JPRB, 0.90778E+00_JPRB, 0.88671E+00_JPRB ,&
&  0.86054E+00_JPRB, 0.83604E+00_JPRB, 0.81860E+00_JPRB, 0.82192E+00_JPRB, 0.81630E+00_JPRB, 0.81658E+00_JPRB, 0.81982E+00_JPRB ,&
&  0.81599E+00_JPRB, 0.81186E+00_JPRB, 0.79886E+00_JPRB, 0.79157E+00_JPRB, 0.78604E+00_JPRB, 0.78846E+00_JPRB, 0.81025E+00_JPRB ,&
&  0.85249E+00_JPRB, 0.86559E+00_JPRB, 0.88044E+00_JPRB, 0.89697E+00_JPRB, 0.91239E+00_JPRB, 0.90650E+00_JPRB, 0.88307E+00_JPRB ,&
&  0.86230E+00_JPRB, 0.85207E+00_JPRB, 0.82425E+00_JPRB, 0.83224E+00_JPRB, 0.82272E+00_JPRB, 0.80773E+00_JPRB, 0.79692E+00_JPRB ,&
&  0.78322E+00_JPRB, 0.78435E+00_JPRB, 0.77464E+00_JPRB, 0.77354E+00_JPRB, 0.77463E+00_JPRB, 0.77096E+00_JPRB, 0.76454E+00_JPRB ,&
&  0.76809E+00_JPRB, 0.76569E+00_JPRB, 0.77332E+00_JPRB, 0.80067E+00_JPRB, 0.83620E+00_JPRB, 0.85955E+00_JPRB, 0.88484E+00_JPRB ,&
&  0.91054E+00_JPRB, 0.96142E+00_JPRB, 0.96213E+00_JPRB, 0.11054E+01_JPRB, 0.12813E+01_JPRB, 0.13032E+01_JPRB, 0.12367E+01_JPRB ,&
&  0.11713E+01_JPRB, 0.11119E+01_JPRB, 0.10749E+01_JPRB, 0.10291E+01_JPRB, 0.10258E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.10167E+01_JPRB, 0.96422E+00_JPRB, 0.92395E+00_JPRB, 0.88700E+00_JPRB, 0.87074E+00_JPRB, 0.84802E+00_JPRB, 0.82109E+00_JPRB ,&
&  0.80428E+00_JPRB, 0.79937E+00_JPRB, 0.78923E+00_JPRB, 0.76931E+00_JPRB, 0.75198E+00_JPRB, 0.74525E+00_JPRB, 0.74391E+00_JPRB ,&
&  0.74151E+00_JPRB, 0.73985E+00_JPRB, 0.73733E+00_JPRB, 0.72955E+00_JPRB, 0.73226E+00_JPRB, 0.74104E+00_JPRB, 0.75054E+00_JPRB ,&
&  0.73393E+00_JPRB, 0.72082E+00_JPRB, 0.70987E+00_JPRB, 0.69890E+00_JPRB, 0.69504E+00_JPRB, 0.67494E+00_JPRB, 0.64931E+00_JPRB ,&
&  0.72955E+00_JPRB, 0.96800E+00_JPRB, 0.14452E+01_JPRB, 0.18314E+01_JPRB, 0.16560E+01_JPRB, 0.13934E+01_JPRB, 0.12548E+01_JPRB ,&
&  0.11785E+01_JPRB, 0.10834E+01_JPRB, 0.10165E+01_JPRB, 0.96774E+00_JPRB, 0.94412E+00_JPRB, 0.91636E+00_JPRB, 0.86764E+00_JPRB ,&
&  0.83180E+00_JPRB, 0.81030E+00_JPRB, 0.80009E+00_JPRB, 0.79952E+00_JPRB, 0.79202E+00_JPRB, 0.80038E+00_JPRB, 0.80464E+00_JPRB ,&
&  0.80633E+00_JPRB, 0.80321E+00_JPRB, 0.79771E+00_JPRB, 0.79251E+00_JPRB, 0.82321E+00_JPRB, 0.87927E+00_JPRB, 0.98156E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10347E+01_JPRB, 0.10200E+01_JPRB, 0.10104E+01_JPRB, 0.98861E+00_JPRB, 0.92978E+00_JPRB, 0.88961E+00_JPRB ,&
&  0.86140E+00_JPRB, 0.85431E+00_JPRB, 0.83121E+00_JPRB, 0.83070E+00_JPRB, 0.83284E+00_JPRB, 0.82686E+00_JPRB, 0.82494E+00_JPRB ,&
&  0.81877E+00_JPRB, 0.81382E+00_JPRB, 0.80831E+00_JPRB, 0.81393E+00_JPRB, 0.80928E+00_JPRB, 0.79460E+00_JPRB, 0.78540E+00_JPRB ,&
&  0.78470E+00_JPRB, 0.77333E+00_JPRB, 0.78845E+00_JPRB, 0.83046E+00_JPRB, 0.87167E+00_JPRB, 0.89797E+00_JPRB, 0.92937E+00_JPRB ,&
&  0.96792E+00_JPRB, 0.10330E+01_JPRB, 0.10178E+01_JPRB, 0.11131E+01_JPRB, 0.12601E+01_JPRB, 0.13775E+01_JPRB, 0.12353E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.11028E+01_JPRB, 0.10619E+01_JPRB, 0.10147E+01_JPRB, 0.10556E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.96237E+00_JPRB, 0.92270E+00_JPRB, 0.89750E+00_JPRB, 0.87776E+00_JPRB, 0.84887E+00_JPRB, 0.81896E+00_JPRB, 0.80688E+00_JPRB ,&
&  0.79759E+00_JPRB, 0.77329E+00_JPRB, 0.74605E+00_JPRB, 0.71312E+00_JPRB, 0.70296E+00_JPRB, 0.70652E+00_JPRB, 0.71096E+00_JPRB ,&
&  0.70861E+00_JPRB, 0.70869E+00_JPRB, 0.70691E+00_JPRB, 0.70555E+00_JPRB, 0.70467E+00_JPRB, 0.72757E+00_JPRB, 0.73754E+00_JPRB ,&
&  0.73014E+00_JPRB, 0.73159E+00_JPRB, 0.71113E+00_JPRB, 0.68905E+00_JPRB, 0.67782E+00_JPRB, 0.66255E+00_JPRB, 0.65754E+00_JPRB ,&
&  0.86093E+00_JPRB, 0.12867E+01_JPRB, 0.20210E+01_JPRB, 0.19552E+01_JPRB, 0.15737E+01_JPRB, 0.12645E+01_JPRB, 0.11808E+01_JPRB ,&
&  0.11396E+01_JPRB, 0.10810E+01_JPRB, 0.10122E+01_JPRB, 0.95572E+00_JPRB, 0.90839E+00_JPRB, 0.87733E+00_JPRB, 0.84309E+00_JPRB ,&
&  0.82321E+00_JPRB, 0.81379E+00_JPRB, 0.79166E+00_JPRB, 0.78163E+00_JPRB, 0.77603E+00_JPRB, 0.77759E+00_JPRB, 0.77684E+00_JPRB ,&
&  0.78148E+00_JPRB, 0.78514E+00_JPRB, 0.78691E+00_JPRB, 0.78155E+00_JPRB, 0.80849E+00_JPRB, 0.82699E+00_JPRB, 0.11342E+01_JPRB ,&
&  0.15207E+01_JPRB, 0.13652E+01_JPRB, 0.11817E+01_JPRB, 0.10891E+01_JPRB, 0.10169E+01_JPRB, 0.93265E+00_JPRB, 0.90337E+00_JPRB ,&
&  0.89103E+00_JPRB, 0.89127E+00_JPRB, 0.87760E+00_JPRB, 0.88769E+00_JPRB, 0.88399E+00_JPRB, 0.87503E+00_JPRB, 0.86420E+00_JPRB ,&
&  0.84393E+00_JPRB, 0.84103E+00_JPRB, 0.83982E+00_JPRB, 0.83778E+00_JPRB, 0.82242E+00_JPRB, 0.81699E+00_JPRB, 0.81163E+00_JPRB ,&
&  0.80751E+00_JPRB, 0.79316E+00_JPRB, 0.83141E+00_JPRB, 0.85001E+00_JPRB, 0.92562E+00_JPRB, 0.98486E+00_JPRB, 0.99263E+00_JPRB ,&
&  0.97395E+00_JPRB, 0.98753E+00_JPRB, 0.98874E+00_JPRB, 0.10547E+01_JPRB, 0.11752E+01_JPRB, 0.13038E+01_JPRB, 0.11449E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10502E+01_JPRB, 0.10200E+01_JPRB, 0.98687E+00_JPRB, 0.98793E+00_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.94815E+00_JPRB, 0.91475E+00_JPRB, 0.88467E+00_JPRB, 0.85100E+00_JPRB, 0.81032E+00_JPRB, 0.78464E+00_JPRB, 0.76550E+00_JPRB ,&
&  0.75328E+00_JPRB, 0.72259E+00_JPRB, 0.69900E+00_JPRB, 0.69456E+00_JPRB, 0.69796E+00_JPRB, 0.68887E+00_JPRB, 0.68487E+00_JPRB ,&
&  0.68907E+00_JPRB, 0.69050E+00_JPRB, 0.68106E+00_JPRB, 0.67276E+00_JPRB, 0.67594E+00_JPRB, 0.71168E+00_JPRB, 0.72767E+00_JPRB ,&
&  0.73402E+00_JPRB, 0.72524E+00_JPRB, 0.70358E+00_JPRB, 0.67472E+00_JPRB, 0.66646E+00_JPRB, 0.67190E+00_JPRB, 0.71398E+00_JPRB ,&
&  0.10270E+01_JPRB, 0.12362E+01_JPRB, 0.21313E+01_JPRB, 0.21809E+01_JPRB, 0.15594E+01_JPRB, 0.12466E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.10581E+01_JPRB, 0.99324E+00_JPRB, 0.93445E+00_JPRB, 0.88241E+00_JPRB, 0.83836E+00_JPRB, 0.80625E+00_JPRB ,&
&  0.79677E+00_JPRB, 0.78703E+00_JPRB, 0.77935E+00_JPRB, 0.78017E+00_JPRB, 0.77278E+00_JPRB, 0.77061E+00_JPRB, 0.76952E+00_JPRB ,&
&  0.77258E+00_JPRB, 0.76891E+00_JPRB, 0.76585E+00_JPRB, 0.78102E+00_JPRB, 0.87371E+00_JPRB, 0.11128E+01_JPRB, 0.14273E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.16880E+01_JPRB, 0.13789E+01_JPRB, 0.11876E+01_JPRB, 0.10335E+01_JPRB, 0.94483E+00_JPRB, 0.93752E+00_JPRB ,&
&  0.93858E+00_JPRB, 0.94685E+00_JPRB, 0.93021E+00_JPRB, 0.92755E+00_JPRB, 0.92497E+00_JPRB, 0.90038E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.87848E+00_JPRB, 0.87269E+00_JPRB, 0.86015E+00_JPRB, 0.84540E+00_JPRB, 0.82791E+00_JPRB, 0.81701E+00_JPRB, 0.81386E+00_JPRB ,&
&  0.81440E+00_JPRB, 0.81211E+00_JPRB, 0.85561E+00_JPRB, 0.88191E+00_JPRB, 0.93096E+00_JPRB, 0.95046E+00_JPRB, 0.95818E+00_JPRB ,&
&  0.96303E+00_JPRB, 0.95895E+00_JPRB, 0.95807E+00_JPRB, 0.10261E+01_JPRB, 0.10394E+01_JPRB, 0.11573E+01_JPRB, 0.10479E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.99766E+00_JPRB, 0.96890E+00_JPRB, 0.94696E+00_JPRB, 0.95001E+00_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.89124E+00_JPRB, 0.84194E+00_JPRB, 0.80755E+00_JPRB, 0.77596E+00_JPRB, 0.74981E+00_JPRB, 0.73150E+00_JPRB, 0.73455E+00_JPRB ,&
&  0.72640E+00_JPRB, 0.69612E+00_JPRB, 0.67376E+00_JPRB, 0.67394E+00_JPRB, 0.67195E+00_JPRB, 0.67360E+00_JPRB, 0.67402E+00_JPRB ,&
&  0.66701E+00_JPRB, 0.65482E+00_JPRB, 0.65099E+00_JPRB, 0.65672E+00_JPRB, 0.68625E+00_JPRB, 0.72517E+00_JPRB, 0.73471E+00_JPRB ,&
&  0.72642E+00_JPRB, 0.71966E+00_JPRB, 0.69941E+00_JPRB, 0.68068E+00_JPRB, 0.67681E+00_JPRB, 0.69363E+00_JPRB, 0.75450E+00_JPRB ,&
&  0.11727E+01_JPRB, 0.28707E+01_JPRB, 0.13964E+01_JPRB, 0.21297E+01_JPRB, 0.14821E+01_JPRB, 0.11682E+01_JPRB, 0.92913E+00_JPRB ,&
&  0.93244E+00_JPRB, 0.10225E+01_JPRB, 0.97517E+00_JPRB, 0.91263E+00_JPRB, 0.83434E+00_JPRB, 0.79096E+00_JPRB, 0.77660E+00_JPRB ,&
&  0.80935E+00_JPRB, 0.79861E+00_JPRB, 0.78957E+00_JPRB, 0.79010E+00_JPRB, 0.77664E+00_JPRB, 0.77456E+00_JPRB, 0.78341E+00_JPRB ,&
&  0.77892E+00_JPRB, 0.77444E+00_JPRB, 0.78897E+00_JPRB, 0.90318E+00_JPRB, 0.10285E+01_JPRB, 0.13980E+01_JPRB, 0.17163E+01_JPRB ,&
&  0.15865E+01_JPRB, 0.18317E+01_JPRB, 0.14913E+01_JPRB, 0.12326E+01_JPRB, 0.10501E+01_JPRB, 0.98554E+00_JPRB, 0.96958E+00_JPRB ,&
&  0.93953E+00_JPRB, 0.91870E+00_JPRB, 0.89763E+00_JPRB, 0.89654E+00_JPRB, 0.91668E+00_JPRB, 0.90273E+00_JPRB, 0.90459E+00_JPRB ,&
&  0.90965E+00_JPRB, 0.87085E+00_JPRB, 0.85537E+00_JPRB, 0.84525E+00_JPRB, 0.85418E+00_JPRB, 0.85840E+00_JPRB, 0.85229E+00_JPRB ,&
&  0.84656E+00_JPRB, 0.85314E+00_JPRB, 0.88062E+00_JPRB, 0.89663E+00_JPRB, 0.91209E+00_JPRB, 0.91910E+00_JPRB, 0.90098E+00_JPRB ,&
&  0.93315E+00_JPRB, 0.95255E+00_JPRB, 0.95601E+00_JPRB, 0.10030E+01_JPRB, 0.10217E+01_JPRB, 0.10996E+01_JPRB, 0.98855E+00_JPRB ,&
&  0.98563E+00_JPRB, 0.97087E+00_JPRB, 0.93471E+00_JPRB, 0.92673E+00_JPRB, 0.93022E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.82841E+00_JPRB, 0.79191E+00_JPRB, 0.75454E+00_JPRB, 0.71292E+00_JPRB, 0.68624E+00_JPRB, 0.68908E+00_JPRB, 0.70381E+00_JPRB ,&
&  0.67953E+00_JPRB, 0.65240E+00_JPRB, 0.64209E+00_JPRB, 0.65588E+00_JPRB, 0.67605E+00_JPRB, 0.67522E+00_JPRB, 0.67212E+00_JPRB ,&
&  0.65721E+00_JPRB, 0.65093E+00_JPRB, 0.63921E+00_JPRB, 0.64798E+00_JPRB, 0.69891E+00_JPRB, 0.69840E+00_JPRB, 0.68994E+00_JPRB ,&
&  0.70833E+00_JPRB, 0.70447E+00_JPRB, 0.68318E+00_JPRB, 0.68131E+00_JPRB, 0.69142E+00_JPRB, 0.73124E+00_JPRB, 0.81385E+00_JPRB ,&
&  0.12704E+01_JPRB, 0.28237E+01_JPRB, 0.15368E+01_JPRB, 0.20039E+01_JPRB, 0.13883E+01_JPRB, 0.10850E+01_JPRB, 0.84828E+00_JPRB ,&
&  0.85007E+00_JPRB, 0.10225E+01_JPRB, 0.95433E+00_JPRB, 0.87635E+00_JPRB, 0.77705E+00_JPRB, 0.73576E+00_JPRB, 0.76891E+00_JPRB ,&
&  0.80462E+00_JPRB, 0.80488E+00_JPRB, 0.81266E+00_JPRB, 0.82905E+00_JPRB, 0.82724E+00_JPRB, 0.82912E+00_JPRB, 0.84129E+00_JPRB ,&
&  0.84570E+00_JPRB, 0.87464E+00_JPRB, 0.93696E+00_JPRB, 0.10235E+01_JPRB, 0.13010E+01_JPRB, 0.17415E+01_JPRB, 0.19141E+01_JPRB ,&
&  0.16882E+01_JPRB, 0.18492E+01_JPRB, 0.14784E+01_JPRB, 0.12281E+01_JPRB, 0.10445E+01_JPRB, 0.98307E+00_JPRB, 0.92510E+00_JPRB ,&
&  0.88728E+00_JPRB, 0.87202E+00_JPRB, 0.84591E+00_JPRB, 0.85023E+00_JPRB, 0.89877E+00_JPRB, 0.92363E+00_JPRB, 0.90579E+00_JPRB ,&
&  0.91705E+00_JPRB, 0.91144E+00_JPRB, 0.90885E+00_JPRB, 0.89053E+00_JPRB, 0.89676E+00_JPRB, 0.89344E+00_JPRB, 0.88309E+00_JPRB ,&
&  0.88200E+00_JPRB, 0.90559E+00_JPRB, 0.90708E+00_JPRB, 0.91184E+00_JPRB, 0.91968E+00_JPRB, 0.92822E+00_JPRB, 0.90167E+00_JPRB ,&
&  0.92898E+00_JPRB, 0.95863E+00_JPRB, 0.97349E+00_JPRB, 0.97403E+00_JPRB, 0.95614E+00_JPRB, 0.98999E+00_JPRB, 0.96824E+00_JPRB ,&
&  0.97991E+00_JPRB, 0.96414E+00_JPRB, 0.91024E+00_JPRB, 0.89018E+00_JPRB, 0.88017E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.77894E+00_JPRB, 0.73614E+00_JPRB, 0.69304E+00_JPRB, 0.63758E+00_JPRB, 0.63946E+00_JPRB, 0.64804E+00_JPRB, 0.62310E+00_JPRB ,&
&  0.57526E+00_JPRB, 0.56854E+00_JPRB, 0.57922E+00_JPRB, 0.60204E+00_JPRB, 0.62150E+00_JPRB, 0.62072E+00_JPRB, 0.63511E+00_JPRB ,&
&  0.62612E+00_JPRB, 0.62249E+00_JPRB, 0.64392E+00_JPRB, 0.67861E+00_JPRB, 0.68391E+00_JPRB, 0.68384E+00_JPRB, 0.69768E+00_JPRB ,&
&  0.70368E+00_JPRB, 0.69293E+00_JPRB, 0.68870E+00_JPRB, 0.70090E+00_JPRB, 0.74142E+00_JPRB, 0.81041E+00_JPRB, 0.93118E+00_JPRB ,&
&  0.13829E+01_JPRB, 0.30493E+01_JPRB, 0.22141E+01_JPRB, 0.17632E+01_JPRB, 0.12651E+01_JPRB, 0.96938E+00_JPRB, 0.82474E+00_JPRB ,&
&  0.83497E+00_JPRB, 0.83990E+00_JPRB, 0.95706E+00_JPRB, 0.84186E+00_JPRB, 0.74247E+00_JPRB, 0.70815E+00_JPRB, 0.78427E+00_JPRB ,&
&  0.85181E+00_JPRB, 0.88478E+00_JPRB, 0.91373E+00_JPRB, 0.91695E+00_JPRB, 0.92318E+00_JPRB, 0.93277E+00_JPRB, 0.93992E+00_JPRB ,&
&  0.96632E+00_JPRB, 0.10421E+01_JPRB, 0.11528E+01_JPRB, 0.12888E+01_JPRB, 0.15898E+01_JPRB, 0.20355E+01_JPRB, 0.21691E+01_JPRB ,&
&  0.19053E+01_JPRB, 0.17407E+01_JPRB, 0.14316E+01_JPRB, 0.12281E+01_JPRB, 0.99153E+00_JPRB, 0.97003E+00_JPRB, 0.88686E+00_JPRB ,&
&  0.85899E+00_JPRB, 0.82062E+00_JPRB, 0.81750E+00_JPRB, 0.83383E+00_JPRB, 0.85870E+00_JPRB, 0.88772E+00_JPRB, 0.89787E+00_JPRB ,&
&  0.90903E+00_JPRB, 0.91463E+00_JPRB, 0.91528E+00_JPRB, 0.90639E+00_JPRB, 0.90659E+00_JPRB, 0.91009E+00_JPRB, 0.92476E+00_JPRB ,&
&  0.93932E+00_JPRB, 0.97400E+00_JPRB, 0.94917E+00_JPRB, 0.94411E+00_JPRB, 0.95535E+00_JPRB, 0.95973E+00_JPRB, 0.91206E+00_JPRB ,&
&  0.92723E+00_JPRB, 0.99120E+00_JPRB, 0.97955E+00_JPRB, 0.95490E+00_JPRB, 0.93305E+00_JPRB, 0.94010E+00_JPRB, 0.92967E+00_JPRB ,&
&  0.95125E+00_JPRB, 0.95017E+00_JPRB, 0.87958E+00_JPRB, 0.84126E+00_JPRB, 0.83609E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.73639E+00_JPRB, 0.70152E+00_JPRB, 0.63699E+00_JPRB, 0.60991E+00_JPRB, 0.60503E+00_JPRB, 0.59560E+00_JPRB, 0.58032E+00_JPRB ,&
&  0.55740E+00_JPRB, 0.54963E+00_JPRB, 0.57663E+00_JPRB, 0.60648E+00_JPRB, 0.63348E+00_JPRB, 0.63395E+00_JPRB, 0.63779E+00_JPRB ,&
&  0.64124E+00_JPRB, 0.66295E+00_JPRB, 0.70488E+00_JPRB, 0.71496E+00_JPRB, 0.70282E+00_JPRB, 0.69722E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.70800E+00_JPRB, 0.72187E+00_JPRB, 0.75426E+00_JPRB, 0.79232E+00_JPRB, 0.85266E+00_JPRB, 0.95570E+00_JPRB, 0.11763E+01_JPRB ,&
&  0.16549E+01_JPRB, 0.27416E+01_JPRB, 0.77616E+00_JPRB, 0.10046E+01_JPRB, 0.95883E+00_JPRB, 0.84920E+00_JPRB, 0.76581E+00_JPRB ,&
&  0.72478E+00_JPRB, 0.73738E+00_JPRB, 0.81756E+00_JPRB, 0.81179E+00_JPRB, 0.72187E+00_JPRB, 0.79891E+00_JPRB, 0.92683E+00_JPRB ,&
&  0.10295E+01_JPRB, 0.10578E+01_JPRB, 0.10567E+01_JPRB, 0.10391E+01_JPRB, 0.10446E+01_JPRB, 0.10583E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.12658E+01_JPRB, 0.14101E+01_JPRB, 0.14914E+01_JPRB, 0.21083E+01_JPRB, 0.25280E+01_JPRB, 0.24139E+01_JPRB ,&
&  0.18838E+01_JPRB, 0.16416E+01_JPRB, 0.13761E+01_JPRB, 0.12181E+01_JPRB, 0.96536E+00_JPRB, 0.81587E+00_JPRB, 0.86759E+00_JPRB ,&
&  0.85197E+00_JPRB, 0.81205E+00_JPRB, 0.79543E+00_JPRB, 0.78309E+00_JPRB, 0.78961E+00_JPRB, 0.84839E+00_JPRB, 0.89312E+00_JPRB ,&
&  0.89585E+00_JPRB, 0.91650E+00_JPRB, 0.91639E+00_JPRB, 0.91017E+00_JPRB, 0.91715E+00_JPRB, 0.93049E+00_JPRB, 0.97154E+00_JPRB ,&
&  0.99750E+00_JPRB, 0.10231E+01_JPRB, 0.10368E+01_JPRB, 0.10186E+01_JPRB, 0.10375E+01_JPRB, 0.10053E+01_JPRB, 0.94788E+00_JPRB ,&
&  0.95961E+00_JPRB, 0.10122E+01_JPRB, 0.92448E+00_JPRB, 0.87238E+00_JPRB, 0.88224E+00_JPRB, 0.87152E+00_JPRB, 0.86235E+00_JPRB ,&
&  0.90940E+00_JPRB, 0.93402E+00_JPRB, 0.90806E+00_JPRB, 0.78451E+00_JPRB, 0.78809E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.69350E+00_JPRB, 0.64354E+00_JPRB, 0.60665E+00_JPRB, 0.60218E+00_JPRB, 0.58991E+00_JPRB, 0.59572E+00_JPRB, 0.60422E+00_JPRB ,&
&  0.60097E+00_JPRB, 0.61703E+00_JPRB, 0.61434E+00_JPRB, 0.62684E+00_JPRB, 0.64703E+00_JPRB, 0.65220E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.69620E+00_JPRB, 0.72344E+00_JPRB, 0.74568E+00_JPRB, 0.70993E+00_JPRB, 0.70265E+00_JPRB, 0.70557E+00_JPRB, 0.72117E+00_JPRB ,&
&  0.75440E+00_JPRB, 0.80160E+00_JPRB, 0.85613E+00_JPRB, 0.92621E+00_JPRB, 0.10294E+01_JPRB, 0.12166E+01_JPRB, 0.15607E+01_JPRB ,&
&  0.24130E+01_JPRB, 0.19222E+01_JPRB, 0.65540E+00_JPRB, 0.80584E+00_JPRB, 0.78821E+00_JPRB, 0.76946E+00_JPRB, 0.66771E+00_JPRB ,&
&  0.67542E+00_JPRB, 0.73661E+00_JPRB, 0.75627E+00_JPRB, 0.79080E+00_JPRB, 0.85615E+00_JPRB, 0.98458E+00_JPRB, 0.10919E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.11857E+01_JPRB, 0.11857E+01_JPRB, 0.11916E+01_JPRB, 0.12113E+01_JPRB, 0.12535E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.14067E+01_JPRB, 0.15392E+01_JPRB, 0.17062E+01_JPRB, 0.17473E+01_JPRB, 0.23809E+01_JPRB, 0.26755E+01_JPRB, 0.26585E+01_JPRB ,&
&  0.22227E+01_JPRB, 0.16752E+01_JPRB, 0.13062E+01_JPRB, 0.11549E+01_JPRB, 0.89168E+00_JPRB, 0.74720E+00_JPRB, 0.82446E+00_JPRB ,&
&  0.86649E+00_JPRB, 0.81036E+00_JPRB, 0.79372E+00_JPRB, 0.78622E+00_JPRB, 0.74748E+00_JPRB, 0.80316E+00_JPRB, 0.88089E+00_JPRB ,&
&  0.90116E+00_JPRB, 0.90232E+00_JPRB, 0.91509E+00_JPRB, 0.90914E+00_JPRB, 0.90855E+00_JPRB, 0.92594E+00_JPRB, 0.96607E+00_JPRB ,&
&  0.99178E+00_JPRB, 0.10186E+01_JPRB, 0.10292E+01_JPRB, 0.10657E+01_JPRB, 0.11018E+01_JPRB, 0.10299E+01_JPRB, 0.92512E+00_JPRB ,&
&  0.89853E+00_JPRB, 0.90880E+00_JPRB, 0.87933E+00_JPRB, 0.85330E+00_JPRB, 0.81807E+00_JPRB, 0.80371E+00_JPRB, 0.81113E+00_JPRB ,&
&  0.89948E+00_JPRB, 0.84234E+00_JPRB, 0.78408E+00_JPRB, 0.71403E+00_JPRB, 0.72635E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.62583E+00_JPRB, 0.59840E+00_JPRB, 0.59076E+00_JPRB, 0.57597E+00_JPRB, 0.58954E+00_JPRB, 0.61478E+00_JPRB, 0.63381E+00_JPRB ,&
&  0.63729E+00_JPRB, 0.63590E+00_JPRB, 0.63678E+00_JPRB, 0.65511E+00_JPRB, 0.68071E+00_JPRB, 0.70769E+00_JPRB, 0.74166E+00_JPRB ,&
&  0.76365E+00_JPRB, 0.77605E+00_JPRB, 0.74985E+00_JPRB, 0.71339E+00_JPRB, 0.72280E+00_JPRB, 0.74388E+00_JPRB, 0.77748E+00_JPRB ,&
&  0.82408E+00_JPRB, 0.87725E+00_JPRB, 0.95132E+00_JPRB, 0.10787E+01_JPRB, 0.12720E+01_JPRB, 0.15021E+01_JPRB, 0.17320E+01_JPRB ,&
&  0.13512E+01_JPRB, 0.64493E+00_JPRB, 0.65451E+00_JPRB, 0.70925E+00_JPRB, 0.64628E+00_JPRB, 0.66435E+00_JPRB, 0.67234E+00_JPRB ,&
&  0.66684E+00_JPRB, 0.70593E+00_JPRB, 0.73216E+00_JPRB, 0.87138E+00_JPRB, 0.10238E+01_JPRB, 0.11220E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.12933E+01_JPRB, 0.13093E+01_JPRB, 0.13259E+01_JPRB, 0.13653E+01_JPRB, 0.14265E+01_JPRB, 0.15093E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.17938E+01_JPRB, 0.19876E+01_JPRB, 0.19290E+01_JPRB, 0.23910E+01_JPRB, 0.27475E+01_JPRB, 0.31299E+01_JPRB ,&
&  0.30646E+01_JPRB, 0.18619E+01_JPRB, 0.12366E+01_JPRB, 0.12028E+01_JPRB, 0.10173E+01_JPRB, 0.78731E+00_JPRB, 0.87642E+00_JPRB ,&
&  0.89567E+00_JPRB, 0.89327E+00_JPRB, 0.88135E+00_JPRB, 0.80599E+00_JPRB, 0.73827E+00_JPRB, 0.72778E+00_JPRB, 0.78772E+00_JPRB ,&
&  0.82727E+00_JPRB, 0.84934E+00_JPRB, 0.83898E+00_JPRB, 0.84082E+00_JPRB, 0.82585E+00_JPRB, 0.84507E+00_JPRB, 0.88497E+00_JPRB ,&
&  0.92009E+00_JPRB, 0.95728E+00_JPRB, 0.97705E+00_JPRB, 0.10094E+01_JPRB, 0.10498E+01_JPRB, 0.98836E+00_JPRB, 0.89887E+00_JPRB ,&
&  0.87233E+00_JPRB, 0.85891E+00_JPRB, 0.81709E+00_JPRB, 0.79244E+00_JPRB, 0.78540E+00_JPRB, 0.82948E+00_JPRB, 0.93849E+00_JPRB ,&
&  0.92619E+00_JPRB, 0.81786E+00_JPRB, 0.70225E+00_JPRB, 0.66894E+00_JPRB, 0.67350E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.52705E+00_JPRB, 0.51877E+00_JPRB, 0.52005E+00_JPRB, 0.51797E+00_JPRB, 0.55021E+00_JPRB, 0.58578E+00_JPRB, 0.59597E+00_JPRB ,&
&  0.59039E+00_JPRB, 0.60780E+00_JPRB, 0.62451E+00_JPRB, 0.65563E+00_JPRB, 0.71953E+00_JPRB, 0.74852E+00_JPRB, 0.78268E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.81447E+00_JPRB, 0.78508E+00_JPRB, 0.77077E+00_JPRB, 0.79328E+00_JPRB, 0.81113E+00_JPRB, 0.84641E+00_JPRB ,&
&  0.89788E+00_JPRB, 0.95825E+00_JPRB, 0.10325E+01_JPRB, 0.11228E+01_JPRB, 0.12260E+01_JPRB, 0.13139E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.56613E+00_JPRB, 0.54299E+00_JPRB, 0.65571E+00_JPRB, 0.64831E+00_JPRB, 0.60816E+00_JPRB, 0.62423E+00_JPRB, 0.66569E+00_JPRB ,&
&  0.69881E+00_JPRB, 0.67994E+00_JPRB, 0.75109E+00_JPRB, 0.10072E+01_JPRB, 0.10689E+01_JPRB, 0.11628E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.12667E+01_JPRB, 0.13213E+01_JPRB, 0.13672E+01_JPRB, 0.14107E+01_JPRB, 0.14683E+01_JPRB, 0.15480E+01_JPRB, 0.16583E+01_JPRB ,&
&  0.18027E+01_JPRB, 0.20368E+01_JPRB, 0.22897E+01_JPRB, 0.22317E+01_JPRB, 0.24824E+01_JPRB, 0.28844E+01_JPRB, 0.35735E+01_JPRB ,&
&  0.35489E+01_JPRB, 0.22735E+01_JPRB, 0.14763E+01_JPRB, 0.12896E+01_JPRB, 0.10988E+01_JPRB, 0.10650E+01_JPRB, 0.99479E+00_JPRB ,&
&  0.92930E+00_JPRB, 0.88286E+00_JPRB, 0.83321E+00_JPRB, 0.77184E+00_JPRB, 0.70429E+00_JPRB, 0.69373E+00_JPRB, 0.69280E+00_JPRB ,&
&  0.72960E+00_JPRB, 0.74576E+00_JPRB, 0.74568E+00_JPRB, 0.74133E+00_JPRB, 0.70533E+00_JPRB, 0.71064E+00_JPRB, 0.74382E+00_JPRB ,&
&  0.77787E+00_JPRB, 0.80878E+00_JPRB, 0.82293E+00_JPRB, 0.83534E+00_JPRB, 0.85187E+00_JPRB, 0.85557E+00_JPRB, 0.82693E+00_JPRB ,&
&  0.77975E+00_JPRB, 0.74557E+00_JPRB, 0.75252E+00_JPRB, 0.74745E+00_JPRB, 0.91277E+00_JPRB, 0.12156E+01_JPRB, 0.12557E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.81772E+00_JPRB, 0.65440E+00_JPRB, 0.60318E+00_JPRB, 0.58062E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.48734E+00_JPRB, 0.47655E+00_JPRB, 0.47717E+00_JPRB, 0.47598E+00_JPRB, 0.51034E+00_JPRB, 0.53338E+00_JPRB, 0.55450E+00_JPRB ,&
&  0.58645E+00_JPRB, 0.60688E+00_JPRB, 0.63911E+00_JPRB, 0.69792E+00_JPRB, 0.76063E+00_JPRB, 0.81034E+00_JPRB, 0.83251E+00_JPRB ,&
&  0.84455E+00_JPRB, 0.84195E+00_JPRB, 0.82915E+00_JPRB, 0.81171E+00_JPRB, 0.82373E+00_JPRB, 0.82688E+00_JPRB, 0.85385E+00_JPRB ,&
&  0.89598E+00_JPRB, 0.94046E+00_JPRB, 0.99551E+00_JPRB, 0.10471E+01_JPRB, 0.10991E+01_JPRB, 0.11361E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.52755E+00_JPRB, 0.71708E+00_JPRB, 0.74678E+00_JPRB, 0.72670E+00_JPRB, 0.67147E+00_JPRB, 0.64136E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.72834E+00_JPRB, 0.73885E+00_JPRB, 0.81470E+00_JPRB, 0.85071E+00_JPRB, 0.10062E+01_JPRB, 0.10601E+01_JPRB, 0.11355E+01_JPRB ,&
&  0.11861E+01_JPRB, 0.12200E+01_JPRB, 0.12705E+01_JPRB, 0.13274E+01_JPRB, 0.13958E+01_JPRB, 0.14727E+01_JPRB, 0.15738E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.17890E+01_JPRB, 0.19487E+01_JPRB, 0.19404E+01_JPRB, 0.19716E+01_JPRB, 0.21706E+01_JPRB, 0.21746E+01_JPRB ,&
&  0.21764E+01_JPRB, 0.13829E+01_JPRB, 0.11969E+01_JPRB, 0.12029E+01_JPRB, 0.11340E+01_JPRB, 0.10963E+01_JPRB, 0.10280E+01_JPRB ,&
&  0.91093E+00_JPRB, 0.84154E+00_JPRB, 0.77448E+00_JPRB, 0.72737E+00_JPRB, 0.68453E+00_JPRB, 0.65905E+00_JPRB, 0.63989E+00_JPRB ,&
&  0.63001E+00_JPRB, 0.64521E+00_JPRB, 0.66041E+00_JPRB, 0.63770E+00_JPRB, 0.60156E+00_JPRB, 0.59893E+00_JPRB, 0.63404E+00_JPRB ,&
&  0.70591E+00_JPRB, 0.64459E+00_JPRB, 0.61032E+00_JPRB, 0.61155E+00_JPRB, 0.60883E+00_JPRB, 0.60158E+00_JPRB, 0.59376E+00_JPRB ,&
&  0.60019E+00_JPRB, 0.59123E+00_JPRB, 0.68267E+00_JPRB, 0.10574E+01_JPRB, 0.14597E+01_JPRB, 0.17108E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.99749E+00_JPRB, 0.73556E+00_JPRB, 0.58619E+00_JPRB, 0.54685E+00_JPRB, 0.52597E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.51741E+00_JPRB, 0.49883E+00_JPRB, 0.48048E+00_JPRB, 0.49093E+00_JPRB, 0.49850E+00_JPRB, 0.50432E+00_JPRB, 0.53051E+00_JPRB ,&
&  0.58540E+00_JPRB, 0.59929E+00_JPRB, 0.62647E+00_JPRB, 0.67000E+00_JPRB, 0.73300E+00_JPRB, 0.78132E+00_JPRB, 0.82572E+00_JPRB ,&
&  0.87100E+00_JPRB, 0.89774E+00_JPRB, 0.89145E+00_JPRB, 0.87664E+00_JPRB, 0.87342E+00_JPRB, 0.87758E+00_JPRB, 0.88969E+00_JPRB ,&
&  0.89107E+00_JPRB, 0.90845E+00_JPRB, 0.94443E+00_JPRB, 0.98675E+00_JPRB, 0.10231E+01_JPRB, 0.10138E+01_JPRB, 0.64107E+00_JPRB ,&
&  0.79426E+00_JPRB, 0.84039E+00_JPRB, 0.85786E+00_JPRB, 0.82297E+00_JPRB, 0.75801E+00_JPRB, 0.69187E+00_JPRB, 0.67214E+00_JPRB ,&
&  0.69152E+00_JPRB, 0.71929E+00_JPRB, 0.73531E+00_JPRB, 0.76662E+00_JPRB, 0.89037E+00_JPRB, 0.91093E+00_JPRB, 0.99335E+00_JPRB ,&
&  0.10334E+01_JPRB, 0.10722E+01_JPRB, 0.11113E+01_JPRB, 0.11379E+01_JPRB, 0.11777E+01_JPRB, 0.12289E+01_JPRB, 0.12777E+01_JPRB ,&
&  0.12901E+01_JPRB, 0.12720E+01_JPRB, 0.12799E+01_JPRB, 0.13006E+01_JPRB, 0.12121E+01_JPRB, 0.13635E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.84467E+00_JPRB, 0.68354E+00_JPRB, 0.89634E+00_JPRB, 0.11455E+01_JPRB, 0.11050E+01_JPRB, 0.10795E+01_JPRB, 0.98866E+00_JPRB ,&
&  0.86695E+00_JPRB, 0.77322E+00_JPRB, 0.72878E+00_JPRB, 0.69762E+00_JPRB, 0.66564E+00_JPRB, 0.63095E+00_JPRB, 0.60213E+00_JPRB ,&
&  0.56639E+00_JPRB, 0.54131E+00_JPRB, 0.55515E+00_JPRB, 0.54122E+00_JPRB, 0.53779E+00_JPRB, 0.57660E+00_JPRB, 0.70201E+00_JPRB ,&
&  0.10463E+01_JPRB, 0.57922E+00_JPRB, 0.50885E+00_JPRB, 0.49973E+00_JPRB, 0.47988E+00_JPRB, 0.46005E+00_JPRB, 0.44537E+00_JPRB ,&
&  0.44871E+00_JPRB, 0.49903E+00_JPRB, 0.64683E+00_JPRB, 0.13675E+01_JPRB, 0.25251E+01_JPRB, 0.25922E+01_JPRB, 0.17396E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.72171E+00_JPRB, 0.57965E+00_JPRB, 0.54624E+00_JPRB, 0.52233E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.57166E+00_JPRB, 0.53968E+00_JPRB, 0.52804E+00_JPRB, 0.52183E+00_JPRB, 0.51196E+00_JPRB, 0.53071E+00_JPRB, 0.54541E+00_JPRB ,&
&  0.56670E+00_JPRB, 0.59408E+00_JPRB, 0.62762E+00_JPRB, 0.65684E+00_JPRB, 0.69869E+00_JPRB, 0.73913E+00_JPRB, 0.77087E+00_JPRB ,&
&  0.79615E+00_JPRB, 0.81536E+00_JPRB, 0.82872E+00_JPRB, 0.83397E+00_JPRB, 0.83385E+00_JPRB, 0.83305E+00_JPRB, 0.82992E+00_JPRB ,&
&  0.82658E+00_JPRB, 0.84373E+00_JPRB, 0.88087E+00_JPRB, 0.92268E+00_JPRB, 0.94630E+00_JPRB, 0.97525E+00_JPRB, 0.62531E+00_JPRB ,&
&  0.96666E+00_JPRB, 0.99202E+00_JPRB, 0.95547E+00_JPRB, 0.87330E+00_JPRB, 0.79901E+00_JPRB, 0.74665E+00_JPRB, 0.70444E+00_JPRB ,&
&  0.67960E+00_JPRB, 0.68353E+00_JPRB, 0.64858E+00_JPRB, 0.69788E+00_JPRB, 0.76369E+00_JPRB, 0.76310E+00_JPRB, 0.81890E+00_JPRB ,&
&  0.86338E+00_JPRB, 0.92114E+00_JPRB, 0.95983E+00_JPRB, 0.98962E+00_JPRB, 0.10088E+01_JPRB, 0.10334E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.10078E+01_JPRB, 0.94322E+00_JPRB, 0.89478E+00_JPRB, 0.88160E+00_JPRB, 0.78463E+00_JPRB, 0.72909E+00_JPRB ,&
&  0.57658E+00_JPRB, 0.60374E+00_JPRB, 0.94808E+00_JPRB, 0.12209E+01_JPRB, 0.11055E+01_JPRB, 0.10573E+01_JPRB, 0.94653E+00_JPRB ,&
&  0.84284E+00_JPRB, 0.77740E+00_JPRB, 0.74188E+00_JPRB, 0.71425E+00_JPRB, 0.66964E+00_JPRB, 0.63517E+00_JPRB, 0.62098E+00_JPRB ,&
&  0.60902E+00_JPRB, 0.55189E+00_JPRB, 0.52513E+00_JPRB, 0.52877E+00_JPRB, 0.50869E+00_JPRB, 0.53471E+00_JPRB, 0.50306E+00_JPRB ,&
&  0.59777E+00_JPRB, 0.55471E+00_JPRB, 0.47846E+00_JPRB, 0.49022E+00_JPRB, 0.46344E+00_JPRB, 0.45885E+00_JPRB, 0.46250E+00_JPRB ,&
&  0.48836E+00_JPRB, 0.44126E+00_JPRB, 0.93716E+00_JPRB, 0.37178E+01_JPRB, 0.36783E+01_JPRB, 0.46695E+01_JPRB, 0.24294E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.70485E+00_JPRB, 0.61065E+00_JPRB, 0.60211E+00_JPRB, 0.58418E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.59823E+00_JPRB, 0.58928E+00_JPRB, 0.56779E+00_JPRB, 0.53626E+00_JPRB, 0.52418E+00_JPRB, 0.54174E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.58609E+00_JPRB, 0.60968E+00_JPRB, 0.63451E+00_JPRB, 0.66154E+00_JPRB, 0.69104E+00_JPRB, 0.71854E+00_JPRB, 0.73992E+00_JPRB ,&
&  0.75526E+00_JPRB, 0.77591E+00_JPRB, 0.79369E+00_JPRB, 0.80814E+00_JPRB, 0.81317E+00_JPRB, 0.81603E+00_JPRB, 0.81759E+00_JPRB ,&
&  0.85262E+00_JPRB, 0.88767E+00_JPRB, 0.93152E+00_JPRB, 0.97462E+00_JPRB, 0.99985E+00_JPRB, 0.98329E+00_JPRB, 0.76807E+00_JPRB ,&
&  0.11176E+01_JPRB, 0.11153E+01_JPRB, 0.10111E+01_JPRB, 0.90996E+00_JPRB, 0.82880E+00_JPRB, 0.77077E+00_JPRB, 0.71921E+00_JPRB ,&
&  0.68113E+00_JPRB, 0.69717E+00_JPRB, 0.66620E+00_JPRB, 0.65257E+00_JPRB, 0.63181E+00_JPRB, 0.66363E+00_JPRB, 0.69718E+00_JPRB ,&
&  0.76375E+00_JPRB, 0.83537E+00_JPRB, 0.89996E+00_JPRB, 0.94945E+00_JPRB, 0.97410E+00_JPRB, 0.99221E+00_JPRB, 0.95987E+00_JPRB ,&
&  0.95856E+00_JPRB, 0.95269E+00_JPRB, 0.87024E+00_JPRB, 0.84607E+00_JPRB, 0.85770E+00_JPRB, 0.77994E+00_JPRB, 0.72242E+00_JPRB ,&
&  0.65036E+00_JPRB, 0.79078E+00_JPRB, 0.10968E+01_JPRB, 0.12489E+01_JPRB, 0.10735E+01_JPRB, 0.10169E+01_JPRB, 0.94109E+00_JPRB ,&
&  0.85252E+00_JPRB, 0.82149E+00_JPRB, 0.80632E+00_JPRB, 0.78148E+00_JPRB, 0.72537E+00_JPRB, 0.68545E+00_JPRB, 0.68359E+00_JPRB ,&
&  0.68763E+00_JPRB, 0.62825E+00_JPRB, 0.58863E+00_JPRB, 0.61141E+00_JPRB, 0.60880E+00_JPRB, 0.53196E+00_JPRB, 0.74095E+00_JPRB ,&
&  0.56359E+00_JPRB, 0.42141E+00_JPRB, 0.50748E+00_JPRB, 0.39636E+00_JPRB, 0.50095E+00_JPRB, 0.51104E+00_JPRB, 0.53730E+00_JPRB ,&
&  0.46441E+00_JPRB, 0.53374E+00_JPRB, 0.89342E+00_JPRB, 0.12230E+01_JPRB, 0.14069E+01_JPRB, 0.13943E+01_JPRB, 0.79065E+00_JPRB ,&
&  0.69307E+00_JPRB, 0.66517E+00_JPRB, 0.64538E+00_JPRB, 0.62369E+00_JPRB, 0.60062E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.62298E+00_JPRB, 0.62376E+00_JPRB, 0.60256E+00_JPRB, 0.57249E+00_JPRB, 0.55867E+00_JPRB, 0.56774E+00_JPRB, 0.58443E+00_JPRB ,&
&  0.59988E+00_JPRB, 0.62061E+00_JPRB, 0.64460E+00_JPRB, 0.66772E+00_JPRB, 0.68816E+00_JPRB, 0.71022E+00_JPRB, 0.73643E+00_JPRB ,&
&  0.76135E+00_JPRB, 0.79230E+00_JPRB, 0.82351E+00_JPRB, 0.85515E+00_JPRB, 0.88307E+00_JPRB, 0.93121E+00_JPRB, 0.96470E+00_JPRB ,&
&  0.10176E+01_JPRB, 0.10697E+01_JPRB, 0.11334E+01_JPRB, 0.11946E+01_JPRB, 0.12693E+01_JPRB, 0.13136E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.12620E+01_JPRB, 0.11823E+01_JPRB, 0.10429E+01_JPRB, 0.94069E+00_JPRB, 0.87701E+00_JPRB, 0.82795E+00_JPRB, 0.78339E+00_JPRB ,&
&  0.76274E+00_JPRB, 0.73121E+00_JPRB, 0.69529E+00_JPRB, 0.66996E+00_JPRB, 0.65816E+00_JPRB, 0.69222E+00_JPRB, 0.67827E+00_JPRB ,&
&  0.72232E+00_JPRB, 0.80872E+00_JPRB, 0.87566E+00_JPRB, 0.98069E+00_JPRB, 0.10731E+01_JPRB, 0.11328E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.11116E+01_JPRB, 0.11726E+01_JPRB, 0.11378E+01_JPRB, 0.11418E+01_JPRB, 0.11325E+01_JPRB, 0.11095E+01_JPRB, 0.10984E+01_JPRB ,&
&  0.96823E+00_JPRB, 0.10116E+01_JPRB, 0.11265E+01_JPRB, 0.12301E+01_JPRB, 0.10829E+01_JPRB, 0.98753E+00_JPRB, 0.96004E+00_JPRB ,&
&  0.88522E+00_JPRB, 0.87362E+00_JPRB, 0.87422E+00_JPRB, 0.83966E+00_JPRB, 0.81271E+00_JPRB, 0.78228E+00_JPRB, 0.78131E+00_JPRB ,&
&  0.78790E+00_JPRB, 0.71314E+00_JPRB, 0.68816E+00_JPRB, 0.70171E+00_JPRB, 0.74422E+00_JPRB, 0.76179E+00_JPRB, 0.90136E+00_JPRB ,&
&  0.64327E+00_JPRB, 0.44591E+00_JPRB, 0.48804E+00_JPRB, 0.64317E+00_JPRB, 0.59167E+00_JPRB, 0.61518E+00_JPRB, 0.65906E+00_JPRB ,&
&  0.70431E+00_JPRB, 0.72767E+00_JPRB, 0.71556E+00_JPRB, 0.73420E+00_JPRB, 0.76648E+00_JPRB, 0.75276E+00_JPRB, 0.69618E+00_JPRB ,&
&  0.70220E+00_JPRB, 0.66606E+00_JPRB, 0.64674E+00_JPRB, 0.63858E+00_JPRB, 0.63216E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.64146E+00_JPRB, 0.64734E+00_JPRB, 0.63438E+00_JPRB, 0.61225E+00_JPRB, 0.59609E+00_JPRB, 0.58864E+00_JPRB, 0.59786E+00_JPRB ,&
&  0.60859E+00_JPRB, 0.63087E+00_JPRB, 0.64975E+00_JPRB, 0.66965E+00_JPRB, 0.67966E+00_JPRB, 0.68762E+00_JPRB, 0.70368E+00_JPRB ,&
&  0.72512E+00_JPRB, 0.74832E+00_JPRB, 0.77818E+00_JPRB, 0.81959E+00_JPRB, 0.86040E+00_JPRB, 0.91080E+00_JPRB, 0.93472E+00_JPRB ,&
&  0.93898E+00_JPRB, 0.96642E+00_JPRB, 0.98508E+00_JPRB, 0.10259E+01_JPRB, 0.12285E+01_JPRB, 0.14345E+01_JPRB, 0.16462E+01_JPRB ,&
&  0.12327E+01_JPRB, 0.12010E+01_JPRB, 0.10407E+01_JPRB, 0.93806E+00_JPRB, 0.94551E+00_JPRB, 0.92114E+00_JPRB, 0.91968E+00_JPRB ,&
&  0.92736E+00_JPRB, 0.92172E+00_JPRB, 0.92143E+00_JPRB, 0.92716E+00_JPRB, 0.93919E+00_JPRB, 0.97878E+00_JPRB, 0.94070E+00_JPRB ,&
&  0.91621E+00_JPRB, 0.94803E+00_JPRB, 0.99330E+00_JPRB, 0.11212E+01_JPRB, 0.12995E+01_JPRB, 0.14528E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.14803E+01_JPRB, 0.15478E+01_JPRB, 0.15053E+01_JPRB, 0.14972E+01_JPRB, 0.14753E+01_JPRB, 0.14612E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.14044E+01_JPRB, 0.13422E+01_JPRB, 0.13162E+01_JPRB, 0.11752E+01_JPRB, 0.10957E+01_JPRB, 0.10329E+01_JPRB, 0.99698E+00_JPRB ,&
&  0.96846E+00_JPRB, 0.97132E+00_JPRB, 0.97142E+00_JPRB, 0.93708E+00_JPRB, 0.91895E+00_JPRB, 0.89380E+00_JPRB, 0.90111E+00_JPRB ,&
&  0.89543E+00_JPRB, 0.82934E+00_JPRB, 0.79522E+00_JPRB, 0.78727E+00_JPRB, 0.77952E+00_JPRB, 0.11214E+01_JPRB, 0.11037E+01_JPRB ,&
&  0.86356E+00_JPRB, 0.70781E+00_JPRB, 0.59942E+00_JPRB, 0.86978E+00_JPRB, 0.71213E+00_JPRB, 0.70415E+00_JPRB, 0.72030E+00_JPRB ,&
&  0.74109E+00_JPRB, 0.75482E+00_JPRB, 0.76338E+00_JPRB, 0.75193E+00_JPRB, 0.74792E+00_JPRB, 0.74465E+00_JPRB, 0.72711E+00_JPRB ,&
&  0.71377E+00_JPRB, 0.69671E+00_JPRB, 0.69231E+00_JPRB, 0.66073E+00_JPRB, 0.64445E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.65022E+00_JPRB, 0.64173E+00_JPRB, 0.64825E+00_JPRB, 0.62828E+00_JPRB, 0.64954E+00_JPRB, 0.63565E+00_JPRB, 0.66618E+00_JPRB ,&
&  0.64313E+00_JPRB, 0.64031E+00_JPRB, 0.64324E+00_JPRB, 0.65814E+00_JPRB, 0.67400E+00_JPRB, 0.69648E+00_JPRB, 0.71599E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.79532E+00_JPRB, 0.86715E+00_JPRB, 0.95358E+00_JPRB, 0.10682E+01_JPRB, 0.11808E+01_JPRB, 0.12779E+01_JPRB ,&
&  0.14069E+01_JPRB, 0.15113E+01_JPRB, 0.15607E+01_JPRB, 0.17033E+01_JPRB, 0.21139E+01_JPRB, 0.26033E+01_JPRB, 0.26098E+01_JPRB ,&
&  0.16295E+01_JPRB, 0.11556E+01_JPRB, 0.10128E+01_JPRB, 0.92043E+00_JPRB, 0.94661E+00_JPRB, 0.10318E+01_JPRB, 0.10473E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10829E+01_JPRB, 0.11208E+01_JPRB, 0.11639E+01_JPRB, 0.12251E+01_JPRB, 0.13038E+01_JPRB, 0.13260E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13278E+01_JPRB, 0.13302E+01_JPRB, 0.14126E+01_JPRB, 0.15411E+01_JPRB, 0.17023E+01_JPRB, 0.18250E+01_JPRB ,&
&  0.18835E+01_JPRB, 0.18323E+01_JPRB, 0.15861E+01_JPRB, 0.15766E+01_JPRB, 0.16473E+01_JPRB, 0.16529E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.17432E+01_JPRB, 0.16640E+01_JPRB, 0.12191E+01_JPRB, 0.11453E+01_JPRB, 0.11230E+01_JPRB, 0.11177E+01_JPRB, 0.10616E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.10914E+01_JPRB, 0.11015E+01_JPRB, 0.10833E+01_JPRB, 0.10544E+01_JPRB, 0.10201E+01_JPRB, 0.10273E+01_JPRB ,&
&  0.96182E+00_JPRB, 0.93663E+00_JPRB, 0.93016E+00_JPRB, 0.91203E+00_JPRB, 0.10012E+01_JPRB, 0.97654E+00_JPRB, 0.10673E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.10095E+01_JPRB, 0.87275E+00_JPRB, 0.10675E+01_JPRB, 0.86564E+00_JPRB, 0.77335E+00_JPRB, 0.78753E+00_JPRB ,&
&  0.80034E+00_JPRB, 0.82179E+00_JPRB, 0.81237E+00_JPRB, 0.81295E+00_JPRB, 0.81795E+00_JPRB, 0.80727E+00_JPRB, 0.79549E+00_JPRB ,&
&  0.78038E+00_JPRB, 0.76254E+00_JPRB, 0.73086E+00_JPRB, 0.70181E+00_JPRB, 0.67430E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.71161E+00_JPRB, 0.70598E+00_JPRB, 0.70716E+00_JPRB, 0.71167E+00_JPRB, 0.70469E+00_JPRB, 0.69888E+00_JPRB, 0.69419E+00_JPRB ,&
&  0.66567E+00_JPRB, 0.66489E+00_JPRB, 0.65590E+00_JPRB, 0.66822E+00_JPRB, 0.70227E+00_JPRB, 0.73804E+00_JPRB, 0.79322E+00_JPRB ,&
&  0.84741E+00_JPRB, 0.91623E+00_JPRB, 0.10201E+01_JPRB, 0.12377E+01_JPRB, 0.14848E+01_JPRB, 0.16665E+01_JPRB, 0.18725E+01_JPRB ,&
&  0.19571E+01_JPRB, 0.20264E+01_JPRB, 0.21590E+01_JPRB, 0.23389E+01_JPRB, 0.24325E+01_JPRB, 0.24713E+01_JPRB, 0.23829E+01_JPRB ,&
&  0.17374E+01_JPRB, 0.11531E+01_JPRB, 0.11143E+01_JPRB, 0.10091E+01_JPRB, 0.10134E+01_JPRB, 0.10326E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.11558E+01_JPRB, 0.12051E+01_JPRB, 0.12512E+01_JPRB, 0.13229E+01_JPRB, 0.14026E+01_JPRB, 0.14273E+01_JPRB ,&
&  0.14436E+01_JPRB, 0.14677E+01_JPRB, 0.14936E+01_JPRB, 0.14535E+01_JPRB, 0.14844E+01_JPRB, 0.15625E+01_JPRB, 0.16379E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.16823E+01_JPRB, 0.14542E+01_JPRB, 0.14632E+01_JPRB, 0.16151E+01_JPRB, 0.15204E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.17721E+01_JPRB, 0.17941E+01_JPRB, 0.11441E+01_JPRB, 0.10364E+01_JPRB, 0.10829E+01_JPRB, 0.11009E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.11620E+01_JPRB, 0.12033E+01_JPRB, 0.12256E+01_JPRB, 0.12386E+01_JPRB, 0.11928E+01_JPRB, 0.11583E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.97862E+00_JPRB, 0.98211E+00_JPRB, 0.98878E+00_JPRB, 0.10163E+01_JPRB, 0.10801E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11949E+01_JPRB, 0.13166E+01_JPRB, 0.11259E+01_JPRB, 0.88595E+00_JPRB, 0.85825E+00_JPRB, 0.82507E+00_JPRB ,&
&  0.85287E+00_JPRB, 0.86284E+00_JPRB, 0.86306E+00_JPRB, 0.85898E+00_JPRB, 0.85394E+00_JPRB, 0.84011E+00_JPRB, 0.82476E+00_JPRB ,&
&  0.80625E+00_JPRB, 0.78482E+00_JPRB, 0.76230E+00_JPRB, 0.74226E+00_JPRB, 0.72520E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.72952E+00_JPRB, 0.73146E+00_JPRB, 0.73769E+00_JPRB, 0.73473E+00_JPRB, 0.72798E+00_JPRB, 0.73475E+00_JPRB, 0.73373E+00_JPRB ,&
&  0.72123E+00_JPRB, 0.70947E+00_JPRB, 0.69539E+00_JPRB, 0.71226E+00_JPRB, 0.74430E+00_JPRB, 0.80331E+00_JPRB, 0.87821E+00_JPRB ,&
&  0.97562E+00_JPRB, 0.11144E+01_JPRB, 0.12979E+01_JPRB, 0.14505E+01_JPRB, 0.16552E+01_JPRB, 0.17575E+01_JPRB, 0.18060E+01_JPRB ,&
&  0.18788E+01_JPRB, 0.19591E+01_JPRB, 0.20088E+01_JPRB, 0.20168E+01_JPRB, 0.19712E+01_JPRB, 0.19538E+01_JPRB, 0.20111E+01_JPRB ,&
&  0.16810E+01_JPRB, 0.12428E+01_JPRB, 0.10973E+01_JPRB, 0.10114E+01_JPRB, 0.98261E+00_JPRB, 0.97797E+00_JPRB, 0.10078E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.11153E+01_JPRB, 0.11911E+01_JPRB, 0.12554E+01_JPRB, 0.13176E+01_JPRB, 0.13617E+01_JPRB, 0.13839E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.14562E+01_JPRB, 0.15068E+01_JPRB, 0.14151E+01_JPRB, 0.14877E+01_JPRB, 0.14934E+01_JPRB, 0.14969E+01_JPRB ,&
&  0.14552E+01_JPRB, 0.13609E+01_JPRB, 0.14064E+01_JPRB, 0.14232E+01_JPRB, 0.14406E+01_JPRB, 0.14299E+01_JPRB, 0.14278E+01_JPRB ,&
&  0.15557E+01_JPRB, 0.15663E+01_JPRB, 0.12020E+01_JPRB, 0.11209E+01_JPRB, 0.11772E+01_JPRB, 0.10375E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.12036E+01_JPRB, 0.12277E+01_JPRB, 0.12603E+01_JPRB, 0.13372E+01_JPRB, 0.14237E+01_JPRB, 0.14851E+01_JPRB, 0.13545E+01_JPRB ,&
&  0.12096E+01_JPRB, 0.11508E+01_JPRB, 0.11454E+01_JPRB, 0.11402E+01_JPRB, 0.11592E+01_JPRB, 0.11971E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.12389E+01_JPRB, 0.12503E+01_JPRB, 0.10878E+01_JPRB, 0.97580E+00_JPRB, 0.93730E+00_JPRB, 0.87874E+00_JPRB ,&
&  0.86403E+00_JPRB, 0.86958E+00_JPRB, 0.86145E+00_JPRB, 0.84392E+00_JPRB, 0.83271E+00_JPRB, 0.81151E+00_JPRB, 0.80031E+00_JPRB ,&
&  0.79467E+00_JPRB, 0.77938E+00_JPRB, 0.76593E+00_JPRB, 0.75910E+00_JPRB, 0.73985E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.74303E+00_JPRB, 0.73662E+00_JPRB, 0.74447E+00_JPRB, 0.73824E+00_JPRB, 0.73085E+00_JPRB, 0.73692E+00_JPRB, 0.74733E+00_JPRB ,&
&  0.74645E+00_JPRB, 0.74662E+00_JPRB, 0.74205E+00_JPRB, 0.74247E+00_JPRB, 0.75526E+00_JPRB, 0.79542E+00_JPRB, 0.86319E+00_JPRB ,&
&  0.95405E+00_JPRB, 0.10905E+01_JPRB, 0.12732E+01_JPRB, 0.14457E+01_JPRB, 0.16535E+01_JPRB, 0.17553E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.18147E+01_JPRB, 0.18968E+01_JPRB, 0.18894E+01_JPRB, 0.18341E+01_JPRB, 0.14772E+01_JPRB, 0.15232E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.11812E+01_JPRB, 0.10237E+01_JPRB, 0.98443E+00_JPRB, 0.94557E+00_JPRB, 0.93109E+00_JPRB, 0.93774E+00_JPRB ,&
&  0.96620E+00_JPRB, 0.10152E+01_JPRB, 0.10895E+01_JPRB, 0.11870E+01_JPRB, 0.12581E+01_JPRB, 0.12898E+01_JPRB, 0.13132E+01_JPRB ,&
&  0.13342E+01_JPRB, 0.13442E+01_JPRB, 0.13490E+01_JPRB, 0.13206E+01_JPRB, 0.13062E+01_JPRB, 0.13014E+01_JPRB, 0.13134E+01_JPRB ,&
&  0.13269E+01_JPRB, 0.12532E+01_JPRB, 0.12587E+01_JPRB, 0.12876E+01_JPRB, 0.12391E+01_JPRB, 0.11844E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.15709E+01_JPRB, 0.12601E+01_JPRB, 0.13567E+01_JPRB, 0.15641E+01_JPRB, 0.13038E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12767E+01_JPRB, 0.13196E+01_JPRB, 0.14002E+01_JPRB, 0.15264E+01_JPRB, 0.16942E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.13152E+01_JPRB, 0.13363E+01_JPRB, 0.13542E+01_JPRB, 0.13426E+01_JPRB, 0.12851E+01_JPRB, 0.12521E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.13504E+01_JPRB, 0.14351E+01_JPRB, 0.14727E+01_JPRB, 0.15145E+01_JPRB, 0.10280E+01_JPRB, 0.98730E+00_JPRB ,&
&  0.92635E+00_JPRB, 0.90854E+00_JPRB, 0.89425E+00_JPRB, 0.87489E+00_JPRB, 0.85673E+00_JPRB, 0.83759E+00_JPRB, 0.82556E+00_JPRB ,&
&  0.81573E+00_JPRB, 0.79941E+00_JPRB, 0.78728E+00_JPRB, 0.77909E+00_JPRB, 0.75688E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.77337E+00_JPRB, 0.76165E+00_JPRB, 0.76110E+00_JPRB, 0.76216E+00_JPRB, 0.76586E+00_JPRB, 0.76526E+00_JPRB, 0.76036E+00_JPRB ,&
&  0.74294E+00_JPRB, 0.73936E+00_JPRB, 0.74643E+00_JPRB, 0.75049E+00_JPRB, 0.74252E+00_JPRB, 0.74689E+00_JPRB, 0.78161E+00_JPRB ,&
&  0.85366E+00_JPRB, 0.96651E+00_JPRB, 0.11187E+01_JPRB, 0.12952E+01_JPRB, 0.15322E+01_JPRB, 0.17062E+01_JPRB, 0.17871E+01_JPRB ,&
&  0.19222E+01_JPRB, 0.19812E+01_JPRB, 0.18183E+01_JPRB, 0.13262E+01_JPRB, 0.12062E+01_JPRB, 0.14298E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.10751E+01_JPRB, 0.10387E+01_JPRB, 0.10201E+01_JPRB, 0.99647E+00_JPRB, 0.96078E+00_JPRB, 0.93687E+00_JPRB, 0.91432E+00_JPRB ,&
&  0.91061E+00_JPRB, 0.92541E+00_JPRB, 0.97285E+00_JPRB, 0.10654E+01_JPRB, 0.11584E+01_JPRB, 0.12196E+01_JPRB, 0.12498E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12673E+01_JPRB, 0.12700E+01_JPRB, 0.12429E+01_JPRB, 0.12112E+01_JPRB, 0.12210E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12020E+01_JPRB, 0.11855E+01_JPRB, 0.11942E+01_JPRB, 0.11848E+01_JPRB, 0.10713E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.14090E+01_JPRB, 0.13864E+01_JPRB, 0.16360E+01_JPRB, 0.10407E+01_JPRB, 0.11086E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.11718E+01_JPRB, 0.12483E+01_JPRB, 0.13017E+01_JPRB, 0.13739E+01_JPRB, 0.14975E+01_JPRB, 0.15333E+01_JPRB, 0.16573E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.14498E+01_JPRB, 0.14558E+01_JPRB, 0.15163E+01_JPRB, 0.15179E+01_JPRB, 0.14390E+01_JPRB, 0.13306E+01_JPRB ,&
&  0.12403E+01_JPRB, 0.13960E+01_JPRB, 0.14097E+01_JPRB, 0.15034E+01_JPRB, 0.12742E+01_JPRB, 0.10298E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.99499E+00_JPRB, 0.96850E+00_JPRB, 0.95852E+00_JPRB, 0.93261E+00_JPRB, 0.91104E+00_JPRB, 0.89183E+00_JPRB, 0.87422E+00_JPRB ,&
&  0.86630E+00_JPRB, 0.85376E+00_JPRB, 0.83518E+00_JPRB, 0.81578E+00_JPRB, 0.79389E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.81407E+00_JPRB, 0.80272E+00_JPRB, 0.80303E+00_JPRB, 0.79863E+00_JPRB, 0.79736E+00_JPRB, 0.80447E+00_JPRB, 0.79793E+00_JPRB ,&
&  0.74949E+00_JPRB, 0.71164E+00_JPRB, 0.71017E+00_JPRB, 0.72272E+00_JPRB, 0.72838E+00_JPRB, 0.72947E+00_JPRB, 0.74502E+00_JPRB ,&
&  0.78443E+00_JPRB, 0.84740E+00_JPRB, 0.93874E+00_JPRB, 0.11161E+01_JPRB, 0.13408E+01_JPRB, 0.15731E+01_JPRB, 0.17547E+01_JPRB ,&
&  0.17241E+01_JPRB, 0.13383E+01_JPRB, 0.17391E+01_JPRB, 0.18727E+01_JPRB, 0.16745E+01_JPRB, 0.17314E+01_JPRB, 0.16220E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.10972E+01_JPRB, 0.10994E+01_JPRB, 0.10612E+01_JPRB, 0.98632E+00_JPRB, 0.95985E+00_JPRB, 0.92734E+00_JPRB ,&
&  0.91106E+00_JPRB, 0.90352E+00_JPRB, 0.91841E+00_JPRB, 0.97513E+00_JPRB, 0.10459E+01_JPRB, 0.11304E+01_JPRB, 0.12059E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.11921E+01_JPRB, 0.11923E+01_JPRB, 0.11486E+01_JPRB, 0.11634E+01_JPRB, 0.11832E+01_JPRB, 0.11769E+01_JPRB ,&
&  0.11726E+01_JPRB, 0.11585E+01_JPRB, 0.11899E+01_JPRB, 0.12260E+01_JPRB, 0.12432E+01_JPRB, 0.11364E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.12981E+01_JPRB, 0.13748E+01_JPRB, 0.15276E+01_JPRB, 0.11420E+01_JPRB, 0.12040E+01_JPRB, 0.10810E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11381E+01_JPRB, 0.11914E+01_JPRB, 0.12423E+01_JPRB, 0.13712E+01_JPRB, 0.14732E+01_JPRB, 0.17407E+01_JPRB ,&
&  0.16420E+01_JPRB, 0.16101E+01_JPRB, 0.15609E+01_JPRB, 0.16252E+01_JPRB, 0.15388E+01_JPRB, 0.15314E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.15913E+01_JPRB, 0.15559E+01_JPRB, 0.15277E+01_JPRB, 0.15371E+01_JPRB, 0.13774E+01_JPRB, 0.11762E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.10664E+01_JPRB, 0.10561E+01_JPRB, 0.10412E+01_JPRB, 0.10203E+01_JPRB, 0.99458E+00_JPRB, 0.96371E+00_JPRB, 0.93651E+00_JPRB ,&
&  0.92284E+00_JPRB, 0.90713E+00_JPRB, 0.88854E+00_JPRB, 0.86802E+00_JPRB, 0.84137E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.86385E+00_JPRB, 0.83718E+00_JPRB, 0.83052E+00_JPRB, 0.84269E+00_JPRB, 0.85057E+00_JPRB, 0.89846E+00_JPRB, 0.95225E+00_JPRB ,&
&  0.10855E+01_JPRB, 0.79116E+00_JPRB, 0.70640E+00_JPRB, 0.68389E+00_JPRB, 0.68944E+00_JPRB, 0.70266E+00_JPRB, 0.72327E+00_JPRB ,&
&  0.75003E+00_JPRB, 0.80325E+00_JPRB, 0.86504E+00_JPRB, 0.94457E+00_JPRB, 0.11364E+01_JPRB, 0.13683E+01_JPRB, 0.14406E+01_JPRB ,&
&  0.14085E+01_JPRB, 0.15497E+01_JPRB, 0.19160E+01_JPRB, 0.17180E+01_JPRB, 0.15750E+01_JPRB, 0.16527E+01_JPRB, 0.15434E+01_JPRB ,&
&  0.12784E+01_JPRB, 0.11371E+01_JPRB, 0.11099E+01_JPRB, 0.10427E+01_JPRB, 0.10128E+01_JPRB, 0.98730E+00_JPRB, 0.96498E+00_JPRB ,&
&  0.93767E+00_JPRB, 0.91816E+00_JPRB, 0.91434E+00_JPRB, 0.94499E+00_JPRB, 0.98764E+00_JPRB, 0.10490E+01_JPRB, 0.11465E+01_JPRB ,&
&  0.11982E+01_JPRB, 0.11695E+01_JPRB, 0.11452E+01_JPRB, 0.10994E+01_JPRB, 0.11357E+01_JPRB, 0.11656E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.12192E+01_JPRB, 0.12496E+01_JPRB, 0.12202E+01_JPRB, 0.11402E+01_JPRB, 0.12536E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.12394E+01_JPRB, 0.14312E+01_JPRB, 0.11644E+01_JPRB, 0.12962E+01_JPRB, 0.13356E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.10932E+01_JPRB, 0.10699E+01_JPRB, 0.10806E+01_JPRB, 0.12057E+01_JPRB, 0.13115E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.15894E+01_JPRB, 0.17660E+01_JPRB, 0.16866E+01_JPRB, 0.16157E+01_JPRB, 0.14576E+01_JPRB, 0.14201E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.17614E+01_JPRB, 0.18214E+01_JPRB, 0.17955E+01_JPRB, 0.17014E+01_JPRB, 0.15120E+01_JPRB, 0.13759E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.12178E+01_JPRB, 0.11532E+01_JPRB, 0.11205E+01_JPRB, 0.10844E+01_JPRB, 0.10389E+01_JPRB, 0.98927E+00_JPRB ,&
&  0.96696E+00_JPRB, 0.95293E+00_JPRB, 0.93885E+00_JPRB, 0.92156E+00_JPRB, 0.89800E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.91168E+00_JPRB, 0.86945E+00_JPRB, 0.82705E+00_JPRB, 0.81439E+00_JPRB, 0.80543E+00_JPRB, 0.81931E+00_JPRB, 0.85747E+00_JPRB ,&
&  0.94045E+00_JPRB, 0.79827E+00_JPRB, 0.72466E+00_JPRB, 0.67495E+00_JPRB, 0.66197E+00_JPRB, 0.67432E+00_JPRB, 0.68793E+00_JPRB ,&
&  0.70570E+00_JPRB, 0.73586E+00_JPRB, 0.78956E+00_JPRB, 0.82681E+00_JPRB, 0.96331E+00_JPRB, 0.11475E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.11219E+01_JPRB, 0.19061E+01_JPRB, 0.18000E+01_JPRB, 0.17116E+01_JPRB, 0.16017E+01_JPRB, 0.16833E+01_JPRB, 0.14438E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.11616E+01_JPRB, 0.11432E+01_JPRB, 0.10833E+01_JPRB, 0.10576E+01_JPRB, 0.10117E+01_JPRB, 0.99111E+00_JPRB ,&
&  0.97249E+00_JPRB, 0.95555E+00_JPRB, 0.93366E+00_JPRB, 0.94169E+00_JPRB, 0.97195E+00_JPRB, 0.10133E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.11752E+01_JPRB, 0.11334E+01_JPRB, 0.11331E+01_JPRB, 0.11555E+01_JPRB, 0.11945E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.11871E+01_JPRB, 0.11601E+01_JPRB, 0.12988E+01_JPRB, 0.13479E+01_JPRB, 0.13449E+01_JPRB, 0.13996E+01_JPRB, 0.12747E+01_JPRB ,&
&  0.13761E+01_JPRB, 0.12694E+01_JPRB, 0.13760E+01_JPRB, 0.11730E+01_JPRB, 0.13029E+01_JPRB, 0.14059E+01_JPRB, 0.13432E+01_JPRB ,&
&  0.11720E+01_JPRB, 0.11501E+01_JPRB, 0.10630E+01_JPRB, 0.10362E+01_JPRB, 0.11083E+01_JPRB, 0.12383E+01_JPRB, 0.13653E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.17051E+01_JPRB, 0.18503E+01_JPRB, 0.15295E+01_JPRB, 0.14042E+01_JPRB, 0.13686E+01_JPRB, 0.15062E+01_JPRB ,&
&  0.19375E+01_JPRB, 0.20430E+01_JPRB, 0.21257E+01_JPRB, 0.20038E+01_JPRB, 0.16756E+01_JPRB, 0.15247E+01_JPRB, 0.14785E+01_JPRB ,&
&  0.14401E+01_JPRB, 0.14051E+01_JPRB, 0.13022E+01_JPRB, 0.12106E+01_JPRB, 0.11519E+01_JPRB, 0.11032E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10030E+01_JPRB, 0.97925E+00_JPRB, 0.96483E+00_JPRB, 0.95804E+00_JPRB, 0.93945E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.93224E+00_JPRB, 0.88655E+00_JPRB, 0.85562E+00_JPRB, 0.82602E+00_JPRB, 0.79031E+00_JPRB, 0.78547E+00_JPRB, 0.84169E+00_JPRB ,&
&  0.84348E+00_JPRB, 0.75810E+00_JPRB, 0.71283E+00_JPRB, 0.67916E+00_JPRB, 0.65871E+00_JPRB, 0.65695E+00_JPRB, 0.66982E+00_JPRB ,&
&  0.67589E+00_JPRB, 0.69278E+00_JPRB, 0.71490E+00_JPRB, 0.77689E+00_JPRB, 0.88437E+00_JPRB, 0.10560E+01_JPRB, 0.77536E+00_JPRB ,&
&  0.10092E+01_JPRB, 0.16340E+01_JPRB, 0.17910E+01_JPRB, 0.18162E+01_JPRB, 0.17658E+01_JPRB, 0.17459E+01_JPRB, 0.15805E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.12495E+01_JPRB, 0.11699E+01_JPRB, 0.11321E+01_JPRB, 0.11160E+01_JPRB, 0.10761E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.99977E+00_JPRB, 0.97070E+00_JPRB, 0.94224E+00_JPRB, 0.93592E+00_JPRB, 0.95428E+00_JPRB, 0.98871E+00_JPRB, 0.10473E+01_JPRB ,&
&  0.11181E+01_JPRB, 0.11742E+01_JPRB, 0.11918E+01_JPRB, 0.11271E+01_JPRB, 0.11612E+01_JPRB, 0.11864E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.11797E+01_JPRB, 0.11514E+01_JPRB, 0.13864E+01_JPRB, 0.16213E+01_JPRB, 0.17224E+01_JPRB, 0.16345E+01_JPRB, 0.14888E+01_JPRB ,&
&  0.15006E+01_JPRB, 0.14561E+01_JPRB, 0.13597E+01_JPRB, 0.12156E+01_JPRB, 0.13104E+01_JPRB, 0.14372E+01_JPRB, 0.14878E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.12235E+01_JPRB, 0.11185E+01_JPRB, 0.10414E+01_JPRB, 0.10561E+01_JPRB, 0.11457E+01_JPRB, 0.13402E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.18371E+01_JPRB, 0.18147E+01_JPRB, 0.14851E+01_JPRB, 0.13580E+01_JPRB, 0.13583E+01_JPRB, 0.14430E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.21303E+01_JPRB, 0.19953E+01_JPRB, 0.21401E+01_JPRB, 0.18094E+01_JPRB, 0.16346E+01_JPRB, 0.15852E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.15082E+01_JPRB, 0.14436E+01_JPRB, 0.13753E+01_JPRB, 0.12631E+01_JPRB, 0.11476E+01_JPRB, 0.10702E+01_JPRB ,&
&  0.10296E+01_JPRB, 0.10118E+01_JPRB, 0.99204E+00_JPRB, 0.97880E+00_JPRB, 0.96175E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.92106E+00_JPRB, 0.88722E+00_JPRB, 0.85165E+00_JPRB, 0.82466E+00_JPRB, 0.78717E+00_JPRB, 0.75947E+00_JPRB, 0.77924E+00_JPRB ,&
&  0.76670E+00_JPRB, 0.72572E+00_JPRB, 0.67647E+00_JPRB, 0.66132E+00_JPRB, 0.65220E+00_JPRB, 0.64550E+00_JPRB, 0.65198E+00_JPRB ,&
&  0.65637E+00_JPRB, 0.67392E+00_JPRB, 0.71780E+00_JPRB, 0.76856E+00_JPRB, 0.76410E+00_JPRB, 0.71029E+00_JPRB, 0.75660E+00_JPRB ,&
&  0.12049E+01_JPRB, 0.16812E+01_JPRB, 0.18106E+01_JPRB, 0.19679E+01_JPRB, 0.20506E+01_JPRB, 0.21417E+01_JPRB, 0.19510E+01_JPRB ,&
&  0.16792E+01_JPRB, 0.15077E+01_JPRB, 0.13749E+01_JPRB, 0.12966E+01_JPRB, 0.12220E+01_JPRB, 0.11555E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10413E+01_JPRB, 0.99791E+00_JPRB, 0.94695E+00_JPRB, 0.95516E+00_JPRB, 0.98669E+00_JPRB, 0.10219E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.11438E+01_JPRB, 0.11779E+01_JPRB, 0.11251E+01_JPRB, 0.11350E+01_JPRB, 0.11640E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.14148E+01_JPRB, 0.15644E+01_JPRB, 0.17115E+01_JPRB, 0.18956E+01_JPRB, 0.19459E+01_JPRB, 0.18366E+01_JPRB ,&
&  0.17371E+01_JPRB, 0.15644E+01_JPRB, 0.12930E+01_JPRB, 0.12940E+01_JPRB, 0.13977E+01_JPRB, 0.15657E+01_JPRB, 0.14421E+01_JPRB ,&
&  0.12161E+01_JPRB, 0.10444E+01_JPRB, 0.98789E+00_JPRB, 0.99131E+00_JPRB, 0.10858E+01_JPRB, 0.11632E+01_JPRB, 0.13863E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.96549E+00_JPRB, 0.84428E+00_JPRB, 0.10309E+01_JPRB, 0.95725E+00_JPRB, 0.99908E+00_JPRB, 0.15129E+01_JPRB ,&
&  0.20336E+01_JPRB, 0.24165E+01_JPRB, 0.23472E+01_JPRB, 0.20497E+01_JPRB, 0.18694E+01_JPRB, 0.16816E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.16311E+01_JPRB, 0.15517E+01_JPRB, 0.15194E+01_JPRB, 0.14625E+01_JPRB, 0.13523E+01_JPRB, 0.12147E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10126E+01_JPRB, 0.98134E+00_JPRB, 0.95854E+00_JPRB, 0.94054E+00_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.88409E+00_JPRB, 0.86668E+00_JPRB, 0.83601E+00_JPRB, 0.80875E+00_JPRB, 0.77710E+00_JPRB, 0.74496E+00_JPRB, 0.73673E+00_JPRB ,&
&  0.71409E+00_JPRB, 0.68064E+00_JPRB, 0.64989E+00_JPRB, 0.64016E+00_JPRB, 0.63664E+00_JPRB, 0.63519E+00_JPRB, 0.64201E+00_JPRB ,&
&  0.64888E+00_JPRB, 0.66621E+00_JPRB, 0.73967E+00_JPRB, 0.78463E+00_JPRB, 0.72463E+00_JPRB, 0.63434E+00_JPRB, 0.74025E+00_JPRB ,&
&  0.12310E+01_JPRB, 0.17778E+01_JPRB, 0.20861E+01_JPRB, 0.23464E+01_JPRB, 0.23810E+01_JPRB, 0.24112E+01_JPRB, 0.22928E+01_JPRB ,&
&  0.20910E+01_JPRB, 0.18947E+01_JPRB, 0.17062E+01_JPRB, 0.14951E+01_JPRB, 0.13216E+01_JPRB, 0.12172E+01_JPRB, 0.11440E+01_JPRB ,&
&  0.11136E+01_JPRB, 0.10853E+01_JPRB, 0.10516E+01_JPRB, 0.99516E+00_JPRB, 0.96609E+00_JPRB, 0.98305E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.10931E+01_JPRB, 0.11467E+01_JPRB, 0.11546E+01_JPRB, 0.96825E+00_JPRB, 0.11305E+01_JPRB, 0.12106E+01_JPRB ,&
&  0.13389E+01_JPRB, 0.14817E+01_JPRB, 0.15220E+01_JPRB, 0.18973E+01_JPRB, 0.21631E+01_JPRB, 0.21266E+01_JPRB, 0.20461E+01_JPRB ,&
&  0.20104E+01_JPRB, 0.18025E+01_JPRB, 0.15058E+01_JPRB, 0.15316E+01_JPRB, 0.16065E+01_JPRB, 0.17074E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.12670E+01_JPRB, 0.11593E+01_JPRB, 0.95019E+00_JPRB, 0.11075E+01_JPRB, 0.11916E+01_JPRB, 0.96261E+00_JPRB ,&
&  0.56131E+00_JPRB, 0.48290E+00_JPRB, 0.50689E+00_JPRB, 0.54210E+00_JPRB, 0.52151E+00_JPRB, 0.79057E+00_JPRB, 0.21306E+01_JPRB ,&
&  0.20553E+01_JPRB, 0.25952E+01_JPRB, 0.25469E+01_JPRB, 0.23179E+01_JPRB, 0.20176E+01_JPRB, 0.17824E+01_JPRB, 0.18564E+01_JPRB ,&
&  0.17515E+01_JPRB, 0.16798E+01_JPRB, 0.15752E+01_JPRB, 0.14832E+01_JPRB, 0.13788E+01_JPRB, 0.12508E+01_JPRB, 0.11459E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10266E+01_JPRB, 0.97623E+00_JPRB, 0.93937E+00_JPRB, 0.90317E+00_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.86170E+00_JPRB, 0.83050E+00_JPRB, 0.80294E+00_JPRB, 0.77548E+00_JPRB, 0.74463E+00_JPRB, 0.71872E+00_JPRB, 0.70248E+00_JPRB ,&
&  0.67720E+00_JPRB, 0.64267E+00_JPRB, 0.61999E+00_JPRB, 0.60977E+00_JPRB, 0.61406E+00_JPRB, 0.62910E+00_JPRB, 0.63968E+00_JPRB ,&
&  0.65779E+00_JPRB, 0.69461E+00_JPRB, 0.82593E+00_JPRB, 0.79929E+00_JPRB, 0.77852E+00_JPRB, 0.74275E+00_JPRB, 0.79032E+00_JPRB ,&
&  0.11107E+01_JPRB, 0.16357E+01_JPRB, 0.20832E+01_JPRB, 0.24256E+01_JPRB, 0.27862E+01_JPRB, 0.28962E+01_JPRB, 0.27972E+01_JPRB ,&
&  0.25216E+01_JPRB, 0.22625E+01_JPRB, 0.19790E+01_JPRB, 0.16651E+01_JPRB, 0.14339E+01_JPRB, 0.12925E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.10878E+01_JPRB, 0.10576E+01_JPRB, 0.10255E+01_JPRB, 0.99448E+00_JPRB, 0.10022E+01_JPRB, 0.10238E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10533E+01_JPRB, 0.10964E+01_JPRB, 0.11062E+01_JPRB, 0.98356E+00_JPRB, 0.87582E+00_JPRB, 0.10379E+01_JPRB ,&
&  0.12514E+01_JPRB, 0.16026E+01_JPRB, 0.17809E+01_JPRB, 0.22875E+01_JPRB, 0.23934E+01_JPRB, 0.22841E+01_JPRB, 0.23209E+01_JPRB ,&
&  0.21994E+01_JPRB, 0.23053E+01_JPRB, 0.17371E+01_JPRB, 0.17453E+01_JPRB, 0.18632E+01_JPRB, 0.14306E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.15569E+01_JPRB, 0.14633E+01_JPRB, 0.12729E+01_JPRB, 0.98481E+00_JPRB, 0.11718E+01_JPRB, 0.10334E+01_JPRB, 0.58185E+00_JPRB ,&
&  0.49594E+00_JPRB, 0.46848E+00_JPRB, 0.44955E+00_JPRB, 0.44348E+00_JPRB, 0.46158E+00_JPRB, 0.69287E+00_JPRB, 0.11976E+01_JPRB ,&
&  0.14733E+01_JPRB, 0.21625E+01_JPRB, 0.24497E+01_JPRB, 0.24818E+01_JPRB, 0.22150E+01_JPRB, 0.20335E+01_JPRB, 0.20252E+01_JPRB ,&
&  0.19265E+01_JPRB, 0.19159E+01_JPRB, 0.16865E+01_JPRB, 0.15201E+01_JPRB, 0.13796E+01_JPRB, 0.12629E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.10818E+01_JPRB, 0.10199E+01_JPRB, 0.96854E+00_JPRB, 0.93960E+00_JPRB, 0.89999E+00_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.82344E+00_JPRB, 0.78870E+00_JPRB, 0.75786E+00_JPRB, 0.72677E+00_JPRB, 0.69286E+00_JPRB, 0.66406E+00_JPRB, 0.65871E+00_JPRB ,&
&  0.64707E+00_JPRB, 0.61656E+00_JPRB, 0.59715E+00_JPRB, 0.58699E+00_JPRB, 0.60080E+00_JPRB, 0.62769E+00_JPRB, 0.64029E+00_JPRB ,&
&  0.66225E+00_JPRB, 0.74237E+00_JPRB, 0.85197E+00_JPRB, 0.75201E+00_JPRB, 0.67882E+00_JPRB, 0.63759E+00_JPRB, 0.65309E+00_JPRB ,&
&  0.99140E+00_JPRB, 0.13508E+01_JPRB, 0.17192E+01_JPRB, 0.21941E+01_JPRB, 0.26263E+01_JPRB, 0.26422E+01_JPRB, 0.28884E+01_JPRB ,&
&  0.27434E+01_JPRB, 0.23933E+01_JPRB, 0.20590E+01_JPRB, 0.17391E+01_JPRB, 0.15093E+01_JPRB, 0.13345E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.10871E+01_JPRB, 0.10604E+01_JPRB, 0.10404E+01_JPRB, 0.10312E+01_JPRB, 0.10325E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.10515E+01_JPRB, 0.10571E+01_JPRB, 0.10656E+01_JPRB, 0.11054E+01_JPRB, 0.11115E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.12741E+01_JPRB, 0.19469E+01_JPRB, 0.23397E+01_JPRB, 0.24293E+01_JPRB, 0.24914E+01_JPRB, 0.24348E+01_JPRB ,&
&  0.23267E+01_JPRB, 0.23851E+01_JPRB, 0.20503E+01_JPRB, 0.21223E+01_JPRB, 0.18143E+01_JPRB, 0.12708E+01_JPRB, 0.15761E+01_JPRB ,&
&  0.17411E+01_JPRB, 0.17064E+01_JPRB, 0.13894E+01_JPRB, 0.10002E+01_JPRB, 0.94794E+00_JPRB, 0.66886E+00_JPRB, 0.49041E+00_JPRB ,&
&  0.46958E+00_JPRB, 0.45401E+00_JPRB, 0.42293E+00_JPRB, 0.44110E+00_JPRB, 0.44732E+00_JPRB, 0.60450E+00_JPRB, 0.88168E+00_JPRB ,&
&  0.12599E+01_JPRB, 0.17483E+01_JPRB, 0.21870E+01_JPRB, 0.23488E+01_JPRB, 0.22179E+01_JPRB, 0.21703E+01_JPRB, 0.22662E+01_JPRB ,&
&  0.20630E+01_JPRB, 0.19936E+01_JPRB, 0.17789E+01_JPRB, 0.15394E+01_JPRB, 0.13608E+01_JPRB, 0.12343E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.97971E+00_JPRB, 0.92546E+00_JPRB, 0.89001E+00_JPRB, 0.85937E+00_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.77630E+00_JPRB, 0.74558E+00_JPRB, 0.71866E+00_JPRB, 0.68774E+00_JPRB, 0.65809E+00_JPRB, 0.62539E+00_JPRB, 0.61988E+00_JPRB ,&
&  0.62276E+00_JPRB, 0.59312E+00_JPRB, 0.56622E+00_JPRB, 0.56514E+00_JPRB, 0.58737E+00_JPRB, 0.61858E+00_JPRB, 0.65548E+00_JPRB ,&
&  0.70866E+00_JPRB, 0.93758E+00_JPRB, 0.70161E+00_JPRB, 0.56783E+00_JPRB, 0.62054E+00_JPRB, 0.64481E+00_JPRB, 0.61012E+00_JPRB ,&
&  0.87446E+00_JPRB, 0.11543E+01_JPRB, 0.14694E+01_JPRB, 0.20540E+01_JPRB, 0.28794E+01_JPRB, 0.29026E+01_JPRB, 0.30472E+01_JPRB ,&
&  0.30593E+01_JPRB, 0.26139E+01_JPRB, 0.21575E+01_JPRB, 0.17910E+01_JPRB, 0.15335E+01_JPRB, 0.13515E+01_JPRB, 0.12378E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11065E+01_JPRB, 0.10686E+01_JPRB, 0.10634E+01_JPRB, 0.10632E+01_JPRB, 0.10584E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.10634E+01_JPRB, 0.10621E+01_JPRB, 0.10812E+01_JPRB, 0.11676E+01_JPRB, 0.11294E+01_JPRB, 0.15931E+01_JPRB ,&
&  0.14984E+01_JPRB, 0.17177E+01_JPRB, 0.20332E+01_JPRB, 0.26236E+01_JPRB, 0.27608E+01_JPRB, 0.26351E+01_JPRB, 0.28250E+01_JPRB ,&
&  0.21669E+01_JPRB, 0.18922E+01_JPRB, 0.19415E+01_JPRB, 0.18903E+01_JPRB, 0.14493E+01_JPRB, 0.15500E+01_JPRB, 0.23919E+01_JPRB ,&
&  0.18721E+01_JPRB, 0.20829E+01_JPRB, 0.21724E+01_JPRB, 0.21689E+01_JPRB, 0.10162E+01_JPRB, 0.64137E+00_JPRB, 0.94426E+00_JPRB ,&
&  0.10339E+01_JPRB, 0.71156E+00_JPRB, 0.54499E+00_JPRB, 0.69013E+00_JPRB, 0.55920E+00_JPRB, 0.68919E+00_JPRB, 0.94619E+00_JPRB ,&
&  0.10600E+01_JPRB, 0.12824E+01_JPRB, 0.19645E+01_JPRB, 0.22157E+01_JPRB, 0.22562E+01_JPRB, 0.22270E+01_JPRB, 0.21500E+01_JPRB ,&
&  0.21139E+01_JPRB, 0.19576E+01_JPRB, 0.18850E+01_JPRB, 0.16049E+01_JPRB, 0.13816E+01_JPRB, 0.12195E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.92241E+00_JPRB, 0.86564E+00_JPRB, 0.83350E+00_JPRB, 0.80676E+00_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.74315E+00_JPRB, 0.71540E+00_JPRB, 0.68826E+00_JPRB, 0.65518E+00_JPRB, 0.62311E+00_JPRB, 0.59732E+00_JPRB, 0.58823E+00_JPRB ,&
&  0.59597E+00_JPRB, 0.56608E+00_JPRB, 0.54336E+00_JPRB, 0.54338E+00_JPRB, 0.55774E+00_JPRB, 0.58278E+00_JPRB, 0.62371E+00_JPRB ,&
&  0.68051E+00_JPRB, 0.70101E+00_JPRB, 0.52611E+00_JPRB, 0.47793E+00_JPRB, 0.62254E+00_JPRB, 0.61510E+00_JPRB, 0.66905E+00_JPRB ,&
&  0.84807E+00_JPRB, 0.10343E+01_JPRB, 0.13204E+01_JPRB, 0.18533E+01_JPRB, 0.23205E+01_JPRB, 0.28057E+01_JPRB, 0.30791E+01_JPRB ,&
&  0.32129E+01_JPRB, 0.29242E+01_JPRB, 0.24801E+01_JPRB, 0.20284E+01_JPRB, 0.16735E+01_JPRB, 0.14339E+01_JPRB, 0.12794E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.10851E+01_JPRB, 0.10354E+01_JPRB, 0.99923E+00_JPRB, 0.10191E+01_JPRB, 0.10308E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.10862E+01_JPRB, 0.10711E+01_JPRB, 0.10793E+01_JPRB, 0.10866E+01_JPRB, 0.10989E+01_JPRB, 0.11469E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.15776E+01_JPRB, 0.19122E+01_JPRB, 0.22790E+01_JPRB, 0.25533E+01_JPRB, 0.27720E+01_JPRB, 0.26552E+01_JPRB, 0.30851E+01_JPRB ,&
&  0.25279E+01_JPRB, 0.20451E+01_JPRB, 0.17060E+01_JPRB, 0.13342E+01_JPRB, 0.16541E+01_JPRB, 0.25241E+01_JPRB, 0.25620E+01_JPRB ,&
&  0.24405E+01_JPRB, 0.24844E+01_JPRB, 0.24523E+01_JPRB, 0.20738E+01_JPRB, 0.17362E+01_JPRB, 0.12610E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.14757E+01_JPRB, 0.13582E+01_JPRB, 0.12244E+01_JPRB, 0.10674E+01_JPRB, 0.86573E+00_JPRB, 0.10311E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.10292E+01_JPRB, 0.10915E+01_JPRB, 0.16060E+01_JPRB, 0.20561E+01_JPRB, 0.21491E+01_JPRB, 0.20236E+01_JPRB, 0.20953E+01_JPRB ,&
&  0.19381E+01_JPRB, 0.19085E+01_JPRB, 0.17528E+01_JPRB, 0.15366E+01_JPRB, 0.13462E+01_JPRB, 0.11832E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.95392E+00_JPRB, 0.88120E+00_JPRB, 0.81729E+00_JPRB, 0.78878E+00_JPRB, 0.76579E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.71995E+00_JPRB, 0.69871E+00_JPRB, 0.67237E+00_JPRB, 0.64273E+00_JPRB, 0.60886E+00_JPRB, 0.58286E+00_JPRB, 0.57195E+00_JPRB ,&
&  0.57242E+00_JPRB, 0.54474E+00_JPRB, 0.53432E+00_JPRB, 0.53266E+00_JPRB, 0.54243E+00_JPRB, 0.56276E+00_JPRB, 0.59943E+00_JPRB ,&
&  0.64907E+00_JPRB, 0.58122E+00_JPRB, 0.45190E+00_JPRB, 0.47940E+00_JPRB, 0.54901E+00_JPRB, 0.55998E+00_JPRB, 0.66820E+00_JPRB ,&
&  0.79798E+00_JPRB, 0.94276E+00_JPRB, 0.11281E+01_JPRB, 0.14828E+01_JPRB, 0.18517E+01_JPRB, 0.22001E+01_JPRB, 0.24937E+01_JPRB ,&
&  0.25583E+01_JPRB, 0.26694E+01_JPRB, 0.25993E+01_JPRB, 0.23414E+01_JPRB, 0.19774E+01_JPRB, 0.16615E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.13062E+01_JPRB, 0.11543E+01_JPRB, 0.10500E+01_JPRB, 0.99342E+00_JPRB, 0.97444E+00_JPRB, 0.99342E+00_JPRB, 0.10392E+01_JPRB ,&
&  0.10820E+01_JPRB, 0.10678E+01_JPRB, 0.10428E+01_JPRB, 0.10843E+01_JPRB, 0.10776E+01_JPRB, 0.11549E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.18512E+01_JPRB, 0.23098E+01_JPRB, 0.25788E+01_JPRB, 0.28629E+01_JPRB, 0.24405E+01_JPRB, 0.25238E+01_JPRB, 0.30787E+01_JPRB ,&
&  0.33342E+01_JPRB, 0.33588E+01_JPRB, 0.34445E+01_JPRB, 0.23639E+01_JPRB, 0.20754E+01_JPRB, 0.29124E+01_JPRB, 0.27865E+01_JPRB ,&
&  0.25173E+01_JPRB, 0.25555E+01_JPRB, 0.24472E+01_JPRB, 0.25848E+01_JPRB, 0.19692E+01_JPRB, 0.17184E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.96935E+00_JPRB, 0.11057E+01_JPRB, 0.10260E+01_JPRB, 0.10039E+01_JPRB, 0.10144E+01_JPRB, 0.99486E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.10353E+01_JPRB, 0.11937E+01_JPRB, 0.15440E+01_JPRB, 0.18677E+01_JPRB, 0.16752E+01_JPRB, 0.17117E+01_JPRB ,&
&  0.16416E+01_JPRB, 0.16044E+01_JPRB, 0.14841E+01_JPRB, 0.14403E+01_JPRB, 0.13592E+01_JPRB, 0.12227E+01_JPRB, 0.10643E+01_JPRB ,&
&  0.93839E+00_JPRB, 0.85906E+00_JPRB, 0.79940E+00_JPRB, 0.76336E+00_JPRB, 0.73999E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.68958E+00_JPRB, 0.67678E+00_JPRB, 0.65540E+00_JPRB, 0.62812E+00_JPRB, 0.59925E+00_JPRB, 0.57785E+00_JPRB, 0.56612E+00_JPRB ,&
&  0.55987E+00_JPRB, 0.54291E+00_JPRB, 0.52916E+00_JPRB, 0.52474E+00_JPRB, 0.53125E+00_JPRB, 0.54303E+00_JPRB, 0.57301E+00_JPRB ,&
&  0.63683E+00_JPRB, 0.64189E+00_JPRB, 0.50864E+00_JPRB, 0.45529E+00_JPRB, 0.43779E+00_JPRB, 0.53364E+00_JPRB, 0.64895E+00_JPRB ,&
&  0.79623E+00_JPRB, 0.88337E+00_JPRB, 0.10449E+01_JPRB, 0.12558E+01_JPRB, 0.14945E+01_JPRB, 0.17016E+01_JPRB, 0.18737E+01_JPRB ,&
&  0.23133E+01_JPRB, 0.21270E+01_JPRB, 0.21932E+01_JPRB, 0.20841E+01_JPRB, 0.20211E+01_JPRB, 0.18031E+01_JPRB, 0.15676E+01_JPRB ,&
&  0.13829E+01_JPRB, 0.12275E+01_JPRB, 0.10873E+01_JPRB, 0.98214E+00_JPRB, 0.92788E+00_JPRB, 0.92032E+00_JPRB, 0.96448E+00_JPRB ,&
&  0.10468E+01_JPRB, 0.10674E+01_JPRB, 0.10460E+01_JPRB, 0.10613E+01_JPRB, 0.11529E+01_JPRB, 0.12873E+01_JPRB, 0.14907E+01_JPRB ,&
&  0.16220E+01_JPRB, 0.18330E+01_JPRB, 0.28594E+01_JPRB, 0.29601E+01_JPRB, 0.32092E+01_JPRB, 0.31071E+01_JPRB, 0.29951E+01_JPRB ,&
&  0.32692E+01_JPRB, 0.33904E+01_JPRB, 0.35246E+01_JPRB, 0.28571E+01_JPRB, 0.33255E+01_JPRB, 0.32838E+01_JPRB, 0.29903E+01_JPRB ,&
&  0.27187E+01_JPRB, 0.26777E+01_JPRB, 0.25104E+01_JPRB, 0.24465E+01_JPRB, 0.24669E+01_JPRB, 0.25407E+01_JPRB, 0.19672E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.15459E+01_JPRB, 0.13465E+01_JPRB, 0.10926E+01_JPRB, 0.89665E+00_JPRB, 0.93137E+00_JPRB, 0.97129E+00_JPRB ,&
&  0.99153E+00_JPRB, 0.10262E+01_JPRB, 0.10680E+01_JPRB, 0.12499E+01_JPRB, 0.14847E+01_JPRB, 0.15177E+01_JPRB, 0.14061E+01_JPRB ,&
&  0.12109E+01_JPRB, 0.12837E+01_JPRB, 0.12681E+01_JPRB, 0.12111E+01_JPRB, 0.12035E+01_JPRB, 0.11299E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.90438E+00_JPRB, 0.82303E+00_JPRB, 0.77058E+00_JPRB, 0.73069E+00_JPRB, 0.70324E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.62637E+00_JPRB, 0.63819E+00_JPRB, 0.63756E+00_JPRB, 0.62337E+00_JPRB, 0.60060E+00_JPRB, 0.57817E+00_JPRB, 0.56031E+00_JPRB ,&
&  0.55167E+00_JPRB, 0.53750E+00_JPRB, 0.51842E+00_JPRB, 0.51885E+00_JPRB, 0.52119E+00_JPRB, 0.53112E+00_JPRB, 0.56397E+00_JPRB ,&
&  0.63572E+00_JPRB, 0.77379E+00_JPRB, 0.61842E+00_JPRB, 0.58732E+00_JPRB, 0.56006E+00_JPRB, 0.64386E+00_JPRB, 0.72409E+00_JPRB ,&
&  0.82785E+00_JPRB, 0.85783E+00_JPRB, 0.91242E+00_JPRB, 0.10265E+01_JPRB, 0.12112E+01_JPRB, 0.13389E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.16041E+01_JPRB, 0.17277E+01_JPRB, 0.16138E+01_JPRB, 0.16360E+01_JPRB, 0.16378E+01_JPRB, 0.15035E+01_JPRB, 0.14966E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.11773E+01_JPRB, 0.10756E+01_JPRB, 0.99536E+00_JPRB, 0.95203E+00_JPRB, 0.92804E+00_JPRB, 0.94628E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.10629E+01_JPRB, 0.10690E+01_JPRB, 0.10727E+01_JPRB, 0.11168E+01_JPRB, 0.12799E+01_JPRB, 0.14634E+01_JPRB ,&
&  0.19495E+01_JPRB, 0.20222E+01_JPRB, 0.17517E+01_JPRB, 0.25667E+01_JPRB, 0.35038E+01_JPRB, 0.32157E+01_JPRB, 0.28541E+01_JPRB ,&
&  0.31593E+01_JPRB, 0.34062E+01_JPRB, 0.36572E+01_JPRB, 0.37478E+01_JPRB, 0.37726E+01_JPRB, 0.36504E+01_JPRB, 0.33294E+01_JPRB ,&
&  0.29117E+01_JPRB, 0.26803E+01_JPRB, 0.25887E+01_JPRB, 0.26048E+01_JPRB, 0.24150E+01_JPRB, 0.22425E+01_JPRB, 0.21265E+01_JPRB ,&
&  0.17894E+01_JPRB, 0.14534E+01_JPRB, 0.93773E+00_JPRB, 0.99607E+00_JPRB, 0.77294E+00_JPRB, 0.79512E+00_JPRB, 0.94958E+00_JPRB ,&
&  0.92866E+00_JPRB, 0.96362E+00_JPRB, 0.10090E+01_JPRB, 0.10403E+01_JPRB, 0.12549E+01_JPRB, 0.12763E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.10060E+01_JPRB, 0.10564E+01_JPRB, 0.10274E+01_JPRB, 0.10263E+01_JPRB, 0.96733E+00_JPRB, 0.88893E+00_JPRB ,&
&  0.80416E+00_JPRB, 0.74164E+00_JPRB, 0.70407E+00_JPRB, 0.66792E+00_JPRB, 0.63489E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.56919E+00_JPRB, 0.59600E+00_JPRB, 0.60734E+00_JPRB, 0.59307E+00_JPRB, 0.57866E+00_JPRB, 0.54737E+00_JPRB, 0.53197E+00_JPRB ,&
&  0.53526E+00_JPRB, 0.52554E+00_JPRB, 0.50528E+00_JPRB, 0.50738E+00_JPRB, 0.51339E+00_JPRB, 0.52406E+00_JPRB, 0.56113E+00_JPRB ,&
&  0.66663E+00_JPRB, 0.75890E+00_JPRB, 0.66734E+00_JPRB, 0.58389E+00_JPRB, 0.73067E+00_JPRB, 0.70805E+00_JPRB, 0.74939E+00_JPRB ,&
&  0.71881E+00_JPRB, 0.77032E+00_JPRB, 0.77481E+00_JPRB, 0.80421E+00_JPRB, 0.90697E+00_JPRB, 0.10483E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.12669E+01_JPRB, 0.12327E+01_JPRB, 0.11918E+01_JPRB, 0.11747E+01_JPRB, 0.10744E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.95749E+00_JPRB, 0.89546E+00_JPRB, 0.88116E+00_JPRB, 0.87215E+00_JPRB, 0.87279E+00_JPRB, 0.90949E+00_JPRB ,&
&  0.98653E+00_JPRB, 0.10619E+01_JPRB, 0.10729E+01_JPRB, 0.10935E+01_JPRB, 0.11731E+01_JPRB, 0.14124E+01_JPRB, 0.18622E+01_JPRB ,&
&  0.23277E+01_JPRB, 0.25225E+01_JPRB, 0.27084E+01_JPRB, 0.31982E+01_JPRB, 0.33117E+01_JPRB, 0.32274E+01_JPRB, 0.32378E+01_JPRB ,&
&  0.33617E+01_JPRB, 0.34497E+01_JPRB, 0.36280E+01_JPRB, 0.37360E+01_JPRB, 0.38167E+01_JPRB, 0.38306E+01_JPRB, 0.35106E+01_JPRB ,&
&  0.30141E+01_JPRB, 0.26971E+01_JPRB, 0.25722E+01_JPRB, 0.23639E+01_JPRB, 0.22233E+01_JPRB, 0.20093E+01_JPRB, 0.20753E+01_JPRB ,&
&  0.21752E+01_JPRB, 0.11633E+01_JPRB, 0.85678E+00_JPRB, 0.10452E+01_JPRB, 0.87061E+00_JPRB, 0.91569E+00_JPRB, 0.11166E+01_JPRB ,&
&  0.10777E+01_JPRB, 0.10108E+01_JPRB, 0.10244E+01_JPRB, 0.10156E+01_JPRB, 0.10844E+01_JPRB, 0.12310E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.95978E+00_JPRB, 0.90936E+00_JPRB, 0.91974E+00_JPRB, 0.86875E+00_JPRB, 0.85554E+00_JPRB, 0.81224E+00_JPRB, 0.75976E+00_JPRB ,&
&  0.69666E+00_JPRB, 0.65702E+00_JPRB, 0.61806E+00_JPRB, 0.59231E+00_JPRB, 0.57728E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.54657E+00_JPRB, 0.57689E+00_JPRB, 0.58023E+00_JPRB, 0.57008E+00_JPRB, 0.56183E+00_JPRB, 0.52674E+00_JPRB, 0.51442E+00_JPRB ,&
&  0.51789E+00_JPRB, 0.51122E+00_JPRB, 0.49443E+00_JPRB, 0.49514E+00_JPRB, 0.49981E+00_JPRB, 0.50252E+00_JPRB, 0.51400E+00_JPRB ,&
&  0.45378E+00_JPRB, 0.55025E+00_JPRB, 0.50693E+00_JPRB, 0.55700E+00_JPRB, 0.72293E+00_JPRB, 0.67942E+00_JPRB, 0.66715E+00_JPRB ,&
&  0.65022E+00_JPRB, 0.63968E+00_JPRB, 0.61748E+00_JPRB, 0.63182E+00_JPRB, 0.67665E+00_JPRB, 0.79551E+00_JPRB, 0.91629E+00_JPRB ,&
&  0.90473E+00_JPRB, 0.86924E+00_JPRB, 0.83584E+00_JPRB, 0.81748E+00_JPRB, 0.77556E+00_JPRB, 0.76170E+00_JPRB, 0.76896E+00_JPRB ,&
&  0.75880E+00_JPRB, 0.73616E+00_JPRB, 0.73608E+00_JPRB, 0.75869E+00_JPRB, 0.77499E+00_JPRB, 0.77502E+00_JPRB, 0.79696E+00_JPRB ,&
&  0.87164E+00_JPRB, 0.96648E+00_JPRB, 0.10149E+01_JPRB, 0.11014E+01_JPRB, 0.12210E+01_JPRB, 0.16702E+01_JPRB, 0.22380E+01_JPRB ,&
&  0.26355E+01_JPRB, 0.26126E+01_JPRB, 0.28202E+01_JPRB, 0.31856E+01_JPRB, 0.32041E+01_JPRB, 0.31433E+01_JPRB, 0.30988E+01_JPRB ,&
&  0.31804E+01_JPRB, 0.31505E+01_JPRB, 0.33431E+01_JPRB, 0.36120E+01_JPRB, 0.35723E+01_JPRB, 0.37467E+01_JPRB, 0.35292E+01_JPRB ,&
&  0.31147E+01_JPRB, 0.28890E+01_JPRB, 0.27801E+01_JPRB, 0.26499E+01_JPRB, 0.24663E+01_JPRB, 0.24891E+01_JPRB, 0.23668E+01_JPRB ,&
&  0.22959E+01_JPRB, 0.20078E+01_JPRB, 0.14691E+01_JPRB, 0.13308E+01_JPRB, 0.99736E+00_JPRB, 0.10373E+01_JPRB, 0.13570E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.11519E+01_JPRB, 0.10397E+01_JPRB, 0.99724E+00_JPRB, 0.10381E+01_JPRB, 0.11537E+01_JPRB, 0.98174E+00_JPRB ,&
&  0.79320E+00_JPRB, 0.79610E+00_JPRB, 0.77710E+00_JPRB, 0.73535E+00_JPRB, 0.70849E+00_JPRB, 0.69112E+00_JPRB, 0.65320E+00_JPRB ,&
&  0.60985E+00_JPRB, 0.58245E+00_JPRB, 0.56702E+00_JPRB, 0.54789E+00_JPRB, 0.54444E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.53223E+00_JPRB, 0.55176E+00_JPRB, 0.54234E+00_JPRB, 0.54231E+00_JPRB, 0.54207E+00_JPRB, 0.51262E+00_JPRB, 0.49980E+00_JPRB ,&
&  0.49625E+00_JPRB, 0.49159E+00_JPRB, 0.48506E+00_JPRB, 0.48494E+00_JPRB, 0.48638E+00_JPRB, 0.48344E+00_JPRB, 0.44443E+00_JPRB ,&
&  0.40579E+00_JPRB, 0.46515E+00_JPRB, 0.47798E+00_JPRB, 0.52352E+00_JPRB, 0.57890E+00_JPRB, 0.56220E+00_JPRB, 0.54436E+00_JPRB ,&
&  0.56116E+00_JPRB, 0.55590E+00_JPRB, 0.56414E+00_JPRB, 0.58846E+00_JPRB, 0.62315E+00_JPRB, 0.69462E+00_JPRB, 0.77147E+00_JPRB ,&
&  0.73450E+00_JPRB, 0.72235E+00_JPRB, 0.69115E+00_JPRB, 0.67671E+00_JPRB, 0.67291E+00_JPRB, 0.66073E+00_JPRB, 0.63407E+00_JPRB ,&
&  0.61184E+00_JPRB, 0.61198E+00_JPRB, 0.62013E+00_JPRB, 0.64237E+00_JPRB, 0.66350E+00_JPRB, 0.67248E+00_JPRB, 0.67678E+00_JPRB ,&
&  0.72065E+00_JPRB, 0.79968E+00_JPRB, 0.86799E+00_JPRB, 0.96336E+00_JPRB, 0.11033E+01_JPRB, 0.15431E+01_JPRB, 0.19267E+01_JPRB ,&
&  0.20414E+01_JPRB, 0.20839E+01_JPRB, 0.21112E+01_JPRB, 0.21580E+01_JPRB, 0.23085E+01_JPRB, 0.24364E+01_JPRB, 0.25459E+01_JPRB ,&
&  0.27442E+01_JPRB, 0.27042E+01_JPRB, 0.32432E+01_JPRB, 0.34692E+01_JPRB, 0.34978E+01_JPRB, 0.35663E+01_JPRB, 0.33822E+01_JPRB ,&
&  0.32610E+01_JPRB, 0.29762E+01_JPRB, 0.28660E+01_JPRB, 0.27238E+01_JPRB, 0.25681E+01_JPRB, 0.25080E+01_JPRB, 0.23217E+01_JPRB ,&
&  0.22413E+01_JPRB, 0.21408E+01_JPRB, 0.18443E+01_JPRB, 0.16307E+01_JPRB, 0.13764E+01_JPRB, 0.14599E+01_JPRB, 0.13783E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.98634E+00_JPRB, 0.99052E+00_JPRB, 0.98671E+00_JPRB, 0.10300E+01_JPRB, 0.99201E+00_JPRB, 0.81309E+00_JPRB ,&
&  0.73763E+00_JPRB, 0.75962E+00_JPRB, 0.71313E+00_JPRB, 0.67258E+00_JPRB, 0.64523E+00_JPRB, 0.61421E+00_JPRB, 0.54093E+00_JPRB ,&
&  0.51434E+00_JPRB, 0.55372E+00_JPRB, 0.55730E+00_JPRB, 0.54035E+00_JPRB, 0.52910E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.52005E+00_JPRB, 0.52971E+00_JPRB, 0.52101E+00_JPRB, 0.52257E+00_JPRB, 0.52014E+00_JPRB, 0.49757E+00_JPRB, 0.46778E+00_JPRB ,&
&  0.47472E+00_JPRB, 0.47113E+00_JPRB, 0.47027E+00_JPRB, 0.47262E+00_JPRB, 0.47611E+00_JPRB, 0.43235E+00_JPRB, 0.35686E+00_JPRB ,&
&  0.34693E+00_JPRB, 0.38237E+00_JPRB, 0.44519E+00_JPRB, 0.49082E+00_JPRB, 0.50565E+00_JPRB, 0.50044E+00_JPRB, 0.50024E+00_JPRB ,&
&  0.50577E+00_JPRB, 0.51284E+00_JPRB, 0.54811E+00_JPRB, 0.57525E+00_JPRB, 0.58723E+00_JPRB, 0.61016E+00_JPRB, 0.66989E+00_JPRB ,&
&  0.65257E+00_JPRB, 0.65301E+00_JPRB, 0.64890E+00_JPRB, 0.59126E+00_JPRB, 0.60544E+00_JPRB, 0.57675E+00_JPRB, 0.54287E+00_JPRB ,&
&  0.52873E+00_JPRB, 0.53854E+00_JPRB, 0.55679E+00_JPRB, 0.58338E+00_JPRB, 0.59925E+00_JPRB, 0.60244E+00_JPRB, 0.60644E+00_JPRB ,&
&  0.62740E+00_JPRB, 0.67825E+00_JPRB, 0.74214E+00_JPRB, 0.83880E+00_JPRB, 0.97003E+00_JPRB, 0.12413E+01_JPRB, 0.15670E+01_JPRB ,&
&  0.16291E+01_JPRB, 0.17172E+01_JPRB, 0.17798E+01_JPRB, 0.16447E+01_JPRB, 0.18169E+01_JPRB, 0.21197E+01_JPRB, 0.22001E+01_JPRB ,&
&  0.24135E+01_JPRB, 0.23951E+01_JPRB, 0.26395E+01_JPRB, 0.29175E+01_JPRB, 0.31659E+01_JPRB, 0.31786E+01_JPRB, 0.30889E+01_JPRB ,&
&  0.30987E+01_JPRB, 0.29781E+01_JPRB, 0.26957E+01_JPRB, 0.25464E+01_JPRB, 0.23815E+01_JPRB, 0.22984E+01_JPRB, 0.21368E+01_JPRB ,&
&  0.20411E+01_JPRB, 0.19865E+01_JPRB, 0.19068E+01_JPRB, 0.18037E+01_JPRB, 0.16781E+01_JPRB, 0.15455E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.12761E+01_JPRB, 0.10891E+01_JPRB, 0.99727E+00_JPRB, 0.10124E+01_JPRB, 0.10620E+01_JPRB, 0.90454E+00_JPRB, 0.77330E+00_JPRB ,&
&  0.71317E+00_JPRB, 0.67002E+00_JPRB, 0.69848E+00_JPRB, 0.64662E+00_JPRB, 0.60642E+00_JPRB, 0.57514E+00_JPRB, 0.54733E+00_JPRB ,&
&  0.52026E+00_JPRB, 0.57337E+00_JPRB, 0.56947E+00_JPRB, 0.54838E+00_JPRB, 0.52544E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.52508E+00_JPRB, 0.52599E+00_JPRB, 0.51733E+00_JPRB, 0.51294E+00_JPRB, 0.50580E+00_JPRB, 0.47230E+00_JPRB, 0.45511E+00_JPRB ,&
&  0.48341E+00_JPRB, 0.43521E+00_JPRB, 0.45910E+00_JPRB, 0.38762E+00_JPRB, 0.35973E+00_JPRB, 0.35649E+00_JPRB, 0.34877E+00_JPRB ,&
&  0.35974E+00_JPRB, 0.39468E+00_JPRB, 0.43698E+00_JPRB, 0.47283E+00_JPRB, 0.50812E+00_JPRB, 0.49706E+00_JPRB, 0.48619E+00_JPRB ,&
&  0.47106E+00_JPRB, 0.47788E+00_JPRB, 0.52491E+00_JPRB, 0.55965E+00_JPRB, 0.58717E+00_JPRB, 0.60301E+00_JPRB, 0.62670E+00_JPRB ,&
&  0.59348E+00_JPRB, 0.58304E+00_JPRB, 0.59153E+00_JPRB, 0.57297E+00_JPRB, 0.54820E+00_JPRB, 0.52472E+00_JPRB, 0.49502E+00_JPRB ,&
&  0.47195E+00_JPRB, 0.45990E+00_JPRB, 0.51454E+00_JPRB, 0.54712E+00_JPRB, 0.56189E+00_JPRB, 0.56268E+00_JPRB, 0.57371E+00_JPRB ,&
&  0.59257E+00_JPRB, 0.62668E+00_JPRB, 0.68691E+00_JPRB, 0.77701E+00_JPRB, 0.87276E+00_JPRB, 0.10216E+01_JPRB, 0.11738E+01_JPRB ,&
&  0.13733E+01_JPRB, 0.11348E+01_JPRB, 0.13795E+01_JPRB, 0.14194E+01_JPRB, 0.16249E+01_JPRB, 0.19578E+01_JPRB, 0.20611E+01_JPRB ,&
&  0.23791E+01_JPRB, 0.23871E+01_JPRB, 0.24962E+01_JPRB, 0.25878E+01_JPRB, 0.26975E+01_JPRB, 0.27522E+01_JPRB, 0.27277E+01_JPRB ,&
&  0.27350E+01_JPRB, 0.24334E+01_JPRB, 0.24619E+01_JPRB, 0.23404E+01_JPRB, 0.22103E+01_JPRB, 0.21447E+01_JPRB, 0.20342E+01_JPRB ,&
&  0.19512E+01_JPRB, 0.18945E+01_JPRB, 0.18154E+01_JPRB, 0.15813E+01_JPRB, 0.15051E+01_JPRB, 0.14406E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.12418E+01_JPRB, 0.11805E+01_JPRB, 0.11103E+01_JPRB, 0.12048E+01_JPRB, 0.11569E+01_JPRB, 0.10166E+01_JPRB, 0.89042E+00_JPRB ,&
&  0.76692E+00_JPRB, 0.63743E+00_JPRB, 0.63227E+00_JPRB, 0.60569E+00_JPRB, 0.56502E+00_JPRB, 0.54802E+00_JPRB, 0.60185E+00_JPRB ,&
&  0.57285E+00_JPRB, 0.57133E+00_JPRB, 0.59487E+00_JPRB, 0.57837E+00_JPRB, 0.54687E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.54927E+00_JPRB, 0.53380E+00_JPRB, 0.51293E+00_JPRB, 0.49799E+00_JPRB, 0.48574E+00_JPRB, 0.46076E+00_JPRB, 0.42475E+00_JPRB ,&
&  0.42052E+00_JPRB, 0.37366E+00_JPRB, 0.35085E+00_JPRB, 0.34144E+00_JPRB, 0.34386E+00_JPRB, 0.34330E+00_JPRB, 0.33866E+00_JPRB ,&
&  0.35290E+00_JPRB, 0.38935E+00_JPRB, 0.44961E+00_JPRB, 0.48448E+00_JPRB, 0.49690E+00_JPRB, 0.49595E+00_JPRB, 0.48388E+00_JPRB ,&
&  0.46960E+00_JPRB, 0.47618E+00_JPRB, 0.50911E+00_JPRB, 0.56601E+00_JPRB, 0.59027E+00_JPRB, 0.58799E+00_JPRB, 0.58108E+00_JPRB ,&
&  0.56964E+00_JPRB, 0.56029E+00_JPRB, 0.55133E+00_JPRB, 0.54432E+00_JPRB, 0.51471E+00_JPRB, 0.48058E+00_JPRB, 0.43003E+00_JPRB ,&
&  0.36785E+00_JPRB, 0.37135E+00_JPRB, 0.47528E+00_JPRB, 0.51501E+00_JPRB, 0.53819E+00_JPRB, 0.54762E+00_JPRB, 0.56115E+00_JPRB ,&
&  0.58877E+00_JPRB, 0.62722E+00_JPRB, 0.67559E+00_JPRB, 0.73598E+00_JPRB, 0.79486E+00_JPRB, 0.87060E+00_JPRB, 0.94651E+00_JPRB ,&
&  0.10865E+01_JPRB, 0.91641E+00_JPRB, 0.90959E+00_JPRB, 0.10241E+01_JPRB, 0.14709E+01_JPRB, 0.17659E+01_JPRB, 0.19350E+01_JPRB ,&
&  0.20960E+01_JPRB, 0.20894E+01_JPRB, 0.21900E+01_JPRB, 0.23671E+01_JPRB, 0.24796E+01_JPRB, 0.25013E+01_JPRB, 0.24329E+01_JPRB ,&
&  0.23259E+01_JPRB, 0.22448E+01_JPRB, 0.21467E+01_JPRB, 0.21221E+01_JPRB, 0.20560E+01_JPRB, 0.20349E+01_JPRB, 0.19741E+01_JPRB ,&
&  0.19068E+01_JPRB, 0.18556E+01_JPRB, 0.17219E+01_JPRB, 0.15707E+01_JPRB, 0.14448E+01_JPRB, 0.13628E+01_JPRB, 0.12971E+01_JPRB ,&
&  0.12487E+01_JPRB, 0.12414E+01_JPRB, 0.12519E+01_JPRB, 0.12673E+01_JPRB, 0.11946E+01_JPRB, 0.10419E+01_JPRB, 0.91052E+00_JPRB ,&
&  0.74748E+00_JPRB, 0.63961E+00_JPRB, 0.54680E+00_JPRB, 0.52421E+00_JPRB, 0.52688E+00_JPRB, 0.52816E+00_JPRB, 0.53197E+00_JPRB ,&
&  0.54653E+00_JPRB, 0.54895E+00_JPRB, 0.55002E+00_JPRB, 0.56470E+00_JPRB, 0.55816E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.56673E+00_JPRB, 0.55851E+00_JPRB, 0.52376E+00_JPRB, 0.49617E+00_JPRB, 0.46489E+00_JPRB, 0.44023E+00_JPRB, 0.41722E+00_JPRB ,&
&  0.41252E+00_JPRB, 0.39036E+00_JPRB, 0.37175E+00_JPRB, 0.35624E+00_JPRB, 0.34996E+00_JPRB, 0.34953E+00_JPRB, 0.36744E+00_JPRB ,&
&  0.40552E+00_JPRB, 0.45460E+00_JPRB, 0.48994E+00_JPRB, 0.49719E+00_JPRB, 0.49953E+00_JPRB, 0.49729E+00_JPRB, 0.49581E+00_JPRB ,&
&  0.48687E+00_JPRB, 0.48547E+00_JPRB, 0.49605E+00_JPRB, 0.52024E+00_JPRB, 0.54677E+00_JPRB, 0.55661E+00_JPRB, 0.54881E+00_JPRB ,&
&  0.53378E+00_JPRB, 0.52447E+00_JPRB, 0.52556E+00_JPRB, 0.52424E+00_JPRB, 0.48432E+00_JPRB, 0.42553E+00_JPRB, 0.36648E+00_JPRB ,&
&  0.32299E+00_JPRB, 0.31604E+00_JPRB, 0.35540E+00_JPRB, 0.47160E+00_JPRB, 0.50278E+00_JPRB, 0.52602E+00_JPRB, 0.54711E+00_JPRB ,&
&  0.53193E+00_JPRB, 0.54007E+00_JPRB, 0.58826E+00_JPRB, 0.68346E+00_JPRB, 0.71024E+00_JPRB, 0.76020E+00_JPRB, 0.80820E+00_JPRB ,&
&  0.86817E+00_JPRB, 0.88679E+00_JPRB, 0.96046E+00_JPRB, 0.95407E+00_JPRB, 0.11248E+01_JPRB, 0.16272E+01_JPRB, 0.16787E+01_JPRB ,&
&  0.17961E+01_JPRB, 0.18939E+01_JPRB, 0.21071E+01_JPRB, 0.22097E+01_JPRB, 0.22438E+01_JPRB, 0.22786E+01_JPRB, 0.22214E+01_JPRB ,&
&  0.20800E+01_JPRB, 0.20021E+01_JPRB, 0.19397E+01_JPRB, 0.19904E+01_JPRB, 0.20062E+01_JPRB, 0.19925E+01_JPRB, 0.19531E+01_JPRB ,&
&  0.19124E+01_JPRB, 0.18427E+01_JPRB, 0.17019E+01_JPRB, 0.15133E+01_JPRB, 0.14092E+01_JPRB, 0.13167E+01_JPRB, 0.12548E+01_JPRB ,&
&  0.12099E+01_JPRB, 0.12171E+01_JPRB, 0.12473E+01_JPRB, 0.12601E+01_JPRB, 0.11241E+01_JPRB, 0.92124E+00_JPRB, 0.76489E+00_JPRB ,&
&  0.66655E+00_JPRB, 0.61390E+00_JPRB, 0.55332E+00_JPRB, 0.53361E+00_JPRB, 0.52363E+00_JPRB, 0.51523E+00_JPRB, 0.49103E+00_JPRB ,&
&  0.46986E+00_JPRB, 0.47147E+00_JPRB, 0.50480E+00_JPRB, 0.54582E+00_JPRB, 0.56206E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.52776E+00_JPRB, 0.55800E+00_JPRB, 0.56315E+00_JPRB, 0.53431E+00_JPRB, 0.50672E+00_JPRB, 0.45896E+00_JPRB, 0.42431E+00_JPRB ,&
&  0.41134E+00_JPRB, 0.40875E+00_JPRB, 0.41484E+00_JPRB, 0.42347E+00_JPRB, 0.43751E+00_JPRB, 0.46025E+00_JPRB, 0.48776E+00_JPRB ,&
&  0.50570E+00_JPRB, 0.51624E+00_JPRB, 0.52831E+00_JPRB, 0.54365E+00_JPRB, 0.55156E+00_JPRB, 0.54913E+00_JPRB, 0.54068E+00_JPRB ,&
&  0.52276E+00_JPRB, 0.50878E+00_JPRB, 0.49756E+00_JPRB, 0.49370E+00_JPRB, 0.49738E+00_JPRB, 0.50699E+00_JPRB, 0.51866E+00_JPRB ,&
&  0.52226E+00_JPRB, 0.51020E+00_JPRB, 0.50324E+00_JPRB, 0.49201E+00_JPRB, 0.44421E+00_JPRB, 0.39894E+00_JPRB, 0.34522E+00_JPRB ,&
&  0.30855E+00_JPRB, 0.29131E+00_JPRB, 0.28792E+00_JPRB, 0.30834E+00_JPRB, 0.36712E+00_JPRB, 0.49225E+00_JPRB, 0.52720E+00_JPRB ,&
&  0.56000E+00_JPRB, 0.58417E+00_JPRB, 0.60500E+00_JPRB, 0.62989E+00_JPRB, 0.65379E+00_JPRB, 0.69039E+00_JPRB, 0.72774E+00_JPRB ,&
&  0.77678E+00_JPRB, 0.82548E+00_JPRB, 0.89880E+00_JPRB, 0.84384E+00_JPRB, 0.88363E+00_JPRB, 0.10801E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.14950E+01_JPRB, 0.15861E+01_JPRB, 0.16610E+01_JPRB, 0.18586E+01_JPRB, 0.19651E+01_JPRB, 0.20948E+01_JPRB, 0.21002E+01_JPRB ,&
&  0.20491E+01_JPRB, 0.19770E+01_JPRB, 0.19149E+01_JPRB, 0.19102E+01_JPRB, 0.19683E+01_JPRB, 0.20240E+01_JPRB, 0.19664E+01_JPRB ,&
&  0.19056E+01_JPRB, 0.18284E+01_JPRB, 0.16493E+01_JPRB, 0.14150E+01_JPRB, 0.13215E+01_JPRB, 0.12969E+01_JPRB, 0.13052E+01_JPRB ,&
&  0.12862E+01_JPRB, 0.12790E+01_JPRB, 0.12772E+01_JPRB, 0.12764E+01_JPRB, 0.11027E+01_JPRB, 0.93422E+00_JPRB, 0.80364E+00_JPRB ,&
&  0.73417E+00_JPRB, 0.69197E+00_JPRB, 0.64034E+00_JPRB, 0.63751E+00_JPRB, 0.60114E+00_JPRB, 0.55879E+00_JPRB, 0.50986E+00_JPRB ,&
&  0.48073E+00_JPRB, 0.46767E+00_JPRB, 0.47794E+00_JPRB, 0.49747E+00_JPRB, 0.51382E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.57053E+00_JPRB, 0.58010E+00_JPRB, 0.58501E+00_JPRB, 0.58219E+00_JPRB, 0.57123E+00_JPRB, 0.55196E+00_JPRB, 0.54377E+00_JPRB ,&
&  0.54621E+00_JPRB, 0.55693E+00_JPRB, 0.56835E+00_JPRB, 0.57158E+00_JPRB, 0.57831E+00_JPRB, 0.58521E+00_JPRB, 0.59242E+00_JPRB ,&
&  0.59879E+00_JPRB, 0.60790E+00_JPRB, 0.61882E+00_JPRB, 0.61126E+00_JPRB, 0.59957E+00_JPRB, 0.58938E+00_JPRB, 0.57903E+00_JPRB ,&
&  0.56105E+00_JPRB, 0.53908E+00_JPRB, 0.51557E+00_JPRB, 0.50283E+00_JPRB, 0.48861E+00_JPRB, 0.48279E+00_JPRB, 0.47608E+00_JPRB ,&
&  0.46855E+00_JPRB, 0.46907E+00_JPRB, 0.47709E+00_JPRB, 0.44483E+00_JPRB, 0.39996E+00_JPRB, 0.36496E+00_JPRB, 0.32186E+00_JPRB ,&
&  0.29063E+00_JPRB, 0.27010E+00_JPRB, 0.25755E+00_JPRB, 0.25550E+00_JPRB, 0.26674E+00_JPRB, 0.29764E+00_JPRB, 0.37155E+00_JPRB ,&
&  0.53738E+00_JPRB, 0.58234E+00_JPRB, 0.60500E+00_JPRB, 0.63066E+00_JPRB, 0.65885E+00_JPRB, 0.68840E+00_JPRB, 0.72019E+00_JPRB ,&
&  0.75463E+00_JPRB, 0.78485E+00_JPRB, 0.83468E+00_JPRB, 0.88144E+00_JPRB, 0.95008E+00_JPRB, 0.98582E+00_JPRB, 0.11336E+01_JPRB ,&
&  0.14185E+01_JPRB, 0.15049E+01_JPRB, 0.14280E+01_JPRB, 0.14858E+01_JPRB, 0.15971E+01_JPRB, 0.17006E+01_JPRB, 0.17820E+01_JPRB ,&
&  0.18317E+01_JPRB, 0.18454E+01_JPRB, 0.18631E+01_JPRB, 0.18768E+01_JPRB, 0.19163E+01_JPRB, 0.19453E+01_JPRB, 0.19419E+01_JPRB ,&
&  0.19039E+01_JPRB, 0.18859E+01_JPRB, 0.19087E+01_JPRB, 0.15819E+01_JPRB, 0.14440E+01_JPRB, 0.14123E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.13069E+01_JPRB, 0.12886E+01_JPRB, 0.13007E+01_JPRB, 0.12973E+01_JPRB, 0.11838E+01_JPRB, 0.10503E+01_JPRB, 0.91772E+00_JPRB ,&
&  0.84268E+00_JPRB, 0.77597E+00_JPRB, 0.70427E+00_JPRB, 0.66865E+00_JPRB, 0.62954E+00_JPRB, 0.58340E+00_JPRB, 0.53965E+00_JPRB ,&
&  0.52300E+00_JPRB, 0.51798E+00_JPRB, 0.52279E+00_JPRB, 0.55010E+00_JPRB, 0.56472E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.57647E+00_JPRB, 0.59655E+00_JPRB, 0.61213E+00_JPRB, 0.62050E+00_JPRB, 0.62179E+00_JPRB, 0.62505E+00_JPRB, 0.62802E+00_JPRB ,&
&  0.63270E+00_JPRB, 0.63770E+00_JPRB, 0.64171E+00_JPRB, 0.64129E+00_JPRB, 0.64132E+00_JPRB, 0.64062E+00_JPRB, 0.64282E+00_JPRB ,&
&  0.64501E+00_JPRB, 0.64061E+00_JPRB, 0.63829E+00_JPRB, 0.63862E+00_JPRB, 0.63521E+00_JPRB, 0.62288E+00_JPRB, 0.60427E+00_JPRB ,&
&  0.57125E+00_JPRB, 0.53921E+00_JPRB, 0.51238E+00_JPRB, 0.48691E+00_JPRB, 0.46631E+00_JPRB, 0.45069E+00_JPRB, 0.44848E+00_JPRB ,&
&  0.46068E+00_JPRB, 0.45099E+00_JPRB, 0.43781E+00_JPRB, 0.40141E+00_JPRB, 0.36530E+00_JPRB, 0.34430E+00_JPRB, 0.29997E+00_JPRB ,&
&  0.26905E+00_JPRB, 0.25113E+00_JPRB, 0.24506E+00_JPRB, 0.24457E+00_JPRB, 0.25268E+00_JPRB, 0.27465E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.40584E+00_JPRB, 0.55997E+00_JPRB, 0.59721E+00_JPRB, 0.63126E+00_JPRB, 0.66926E+00_JPRB, 0.70588E+00_JPRB, 0.73354E+00_JPRB ,&
&  0.76103E+00_JPRB, 0.78998E+00_JPRB, 0.84824E+00_JPRB, 0.90720E+00_JPRB, 0.98875E+00_JPRB, 0.10599E+01_JPRB, 0.11246E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.11884E+01_JPRB, 0.12349E+01_JPRB, 0.12737E+01_JPRB, 0.13623E+01_JPRB, 0.14444E+01_JPRB, 0.14282E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.15229E+01_JPRB, 0.15445E+01_JPRB, 0.15959E+01_JPRB, 0.17006E+01_JPRB, 0.18222E+01_JPRB, 0.18645E+01_JPRB ,&
&  0.18032E+01_JPRB, 0.17615E+01_JPRB, 0.18082E+01_JPRB, 0.17074E+01_JPRB, 0.16312E+01_JPRB, 0.15375E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.13622E+01_JPRB, 0.13172E+01_JPRB, 0.12898E+01_JPRB, 0.12625E+01_JPRB, 0.12066E+01_JPRB, 0.11386E+01_JPRB, 0.10397E+01_JPRB ,&
&  0.92373E+00_JPRB, 0.82764E+00_JPRB, 0.75189E+00_JPRB, 0.70372E+00_JPRB, 0.66292E+00_JPRB, 0.62041E+00_JPRB, 0.58875E+00_JPRB ,&
&  0.56260E+00_JPRB, 0.55116E+00_JPRB, 0.54742E+00_JPRB, 0.55541E+00_JPRB, 0.56454E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.62854E+00_JPRB, 0.64089E+00_JPRB, 0.65316E+00_JPRB, 0.66085E+00_JPRB, 0.66611E+00_JPRB, 0.66980E+00_JPRB, 0.66996E+00_JPRB ,&
&  0.67083E+00_JPRB, 0.67077E+00_JPRB, 0.67307E+00_JPRB, 0.67488E+00_JPRB, 0.68023E+00_JPRB, 0.68468E+00_JPRB, 0.69158E+00_JPRB ,&
&  0.70176E+00_JPRB, 0.70965E+00_JPRB, 0.70945E+00_JPRB, 0.69681E+00_JPRB, 0.67417E+00_JPRB, 0.64829E+00_JPRB, 0.61056E+00_JPRB ,&
&  0.57410E+00_JPRB, 0.54889E+00_JPRB, 0.51324E+00_JPRB, 0.48007E+00_JPRB, 0.46101E+00_JPRB, 0.44698E+00_JPRB, 0.44278E+00_JPRB ,&
&  0.43659E+00_JPRB, 0.42080E+00_JPRB, 0.40088E+00_JPRB, 0.37229E+00_JPRB, 0.34809E+00_JPRB, 0.29221E+00_JPRB, 0.27864E+00_JPRB ,&
&  0.26292E+00_JPRB, 0.25275E+00_JPRB, 0.24932E+00_JPRB, 0.25066E+00_JPRB, 0.25612E+00_JPRB, 0.27095E+00_JPRB, 0.29459E+00_JPRB ,&
&  0.33622E+00_JPRB, 0.51681E+00_JPRB, 0.56876E+00_JPRB, 0.58900E+00_JPRB, 0.62366E+00_JPRB, 0.65824E+00_JPRB, 0.68831E+00_JPRB ,&
&  0.72099E+00_JPRB, 0.75452E+00_JPRB, 0.80586E+00_JPRB, 0.85526E+00_JPRB, 0.90717E+00_JPRB, 0.95164E+00_JPRB, 0.98724E+00_JPRB ,&
&  0.10270E+01_JPRB, 0.10559E+01_JPRB, 0.11166E+01_JPRB, 0.11419E+01_JPRB, 0.11435E+01_JPRB, 0.12067E+01_JPRB, 0.12477E+01_JPRB ,&
&  0.12529E+01_JPRB, 0.12461E+01_JPRB, 0.12848E+01_JPRB, 0.13727E+01_JPRB, 0.14893E+01_JPRB, 0.15880E+01_JPRB, 0.16163E+01_JPRB ,&
&  0.16280E+01_JPRB, 0.16023E+01_JPRB, 0.15599E+01_JPRB, 0.15280E+01_JPRB, 0.14611E+01_JPRB, 0.13897E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.12929E+01_JPRB, 0.12781E+01_JPRB, 0.12420E+01_JPRB, 0.12043E+01_JPRB, 0.11566E+01_JPRB, 0.11087E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.97376E+00_JPRB, 0.89249E+00_JPRB, 0.82658E+00_JPRB, 0.76973E+00_JPRB, 0.72866E+00_JPRB, 0.68640E+00_JPRB, 0.66336E+00_JPRB ,&
&  0.63848E+00_JPRB, 0.62859E+00_JPRB, 0.61555E+00_JPRB, 0.61497E+00_JPRB, 0.61571E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.71151E+00_JPRB, 0.71390E+00_JPRB, 0.71767E+00_JPRB, 0.72108E+00_JPRB, 0.72419E+00_JPRB, 0.72580E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72479E+00_JPRB, 0.72406E+00_JPRB, 0.72531E+00_JPRB, 0.72832E+00_JPRB, 0.73270E+00_JPRB, 0.73885E+00_JPRB, 0.74502E+00_JPRB ,&
&  0.75303E+00_JPRB, 0.75064E+00_JPRB, 0.73334E+00_JPRB, 0.71256E+00_JPRB, 0.68894E+00_JPRB, 0.65549E+00_JPRB, 0.61453E+00_JPRB ,&
&  0.57653E+00_JPRB, 0.54718E+00_JPRB, 0.50345E+00_JPRB, 0.46632E+00_JPRB, 0.43667E+00_JPRB, 0.42138E+00_JPRB, 0.40332E+00_JPRB ,&
&  0.40384E+00_JPRB, 0.35699E+00_JPRB, 0.34071E+00_JPRB, 0.30946E+00_JPRB, 0.28829E+00_JPRB, 0.27241E+00_JPRB, 0.26341E+00_JPRB ,&
&  0.25986E+00_JPRB, 0.25750E+00_JPRB, 0.25841E+00_JPRB, 0.26362E+00_JPRB, 0.27119E+00_JPRB, 0.28774E+00_JPRB, 0.30765E+00_JPRB ,&
&  0.36077E+00_JPRB, 0.52012E+00_JPRB, 0.58242E+00_JPRB, 0.60098E+00_JPRB, 0.60860E+00_JPRB, 0.63307E+00_JPRB, 0.66043E+00_JPRB ,&
&  0.70216E+00_JPRB, 0.73807E+00_JPRB, 0.77565E+00_JPRB, 0.79884E+00_JPRB, 0.84150E+00_JPRB, 0.88034E+00_JPRB, 0.91493E+00_JPRB ,&
&  0.95326E+00_JPRB, 0.98761E+00_JPRB, 0.10231E+01_JPRB, 0.10496E+01_JPRB, 0.10571E+01_JPRB, 0.10707E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.11179E+01_JPRB, 0.11620E+01_JPRB, 0.12215E+01_JPRB, 0.12846E+01_JPRB, 0.13308E+01_JPRB, 0.13557E+01_JPRB ,&
&  0.13806E+01_JPRB, 0.13937E+01_JPRB, 0.14095E+01_JPRB, 0.14156E+01_JPRB, 0.13914E+01_JPRB, 0.13497E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12440E+01_JPRB, 0.11942E+01_JPRB, 0.11495E+01_JPRB, 0.11089E+01_JPRB, 0.10719E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.10021E+01_JPRB, 0.96244E+00_JPRB, 0.92111E+00_JPRB, 0.87196E+00_JPRB, 0.82716E+00_JPRB, 0.78462E+00_JPRB, 0.75834E+00_JPRB ,&
&  0.73393E+00_JPRB, 0.72413E+00_JPRB, 0.71218E+00_JPRB, 0.71063E+00_JPRB, 0.70719E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.79983E+00_JPRB, 0.79542E+00_JPRB, 0.79400E+00_JPRB, 0.79200E+00_JPRB, 0.79246E+00_JPRB, 0.79169E+00_JPRB, 0.79169E+00_JPRB ,&
&  0.79058E+00_JPRB, 0.79071E+00_JPRB, 0.78998E+00_JPRB, 0.78985E+00_JPRB, 0.78840E+00_JPRB, 0.78813E+00_JPRB, 0.78742E+00_JPRB ,&
&  0.78439E+00_JPRB, 0.77365E+00_JPRB, 0.75151E+00_JPRB, 0.72743E+00_JPRB, 0.70102E+00_JPRB, 0.67013E+00_JPRB, 0.63566E+00_JPRB ,&
&  0.59203E+00_JPRB, 0.55159E+00_JPRB, 0.49617E+00_JPRB, 0.46033E+00_JPRB, 0.42673E+00_JPRB, 0.40225E+00_JPRB, 0.38722E+00_JPRB ,&
&  0.37334E+00_JPRB, 0.36037E+00_JPRB, 0.34769E+00_JPRB, 0.33270E+00_JPRB, 0.31960E+00_JPRB, 0.30388E+00_JPRB, 0.29431E+00_JPRB ,&
&  0.28747E+00_JPRB, 0.28512E+00_JPRB, 0.28547E+00_JPRB, 0.29112E+00_JPRB, 0.30073E+00_JPRB, 0.31621E+00_JPRB, 0.34120E+00_JPRB ,&
&  0.39652E+00_JPRB, 0.47330E+00_JPRB, 0.59529E+00_JPRB, 0.62383E+00_JPRB, 0.62181E+00_JPRB, 0.63904E+00_JPRB, 0.65960E+00_JPRB ,&
&  0.67767E+00_JPRB, 0.70204E+00_JPRB, 0.72177E+00_JPRB, 0.73013E+00_JPRB, 0.73992E+00_JPRB, 0.74870E+00_JPRB, 0.75933E+00_JPRB ,&
&  0.77191E+00_JPRB, 0.79716E+00_JPRB, 0.83679E+00_JPRB, 0.88426E+00_JPRB, 0.93774E+00_JPRB, 0.98757E+00_JPRB, 0.10238E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.10665E+01_JPRB, 0.10970E+01_JPRB, 0.11308E+01_JPRB, 0.11675E+01_JPRB, 0.11933E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12451E+01_JPRB, 0.12388E+01_JPRB, 0.12251E+01_JPRB, 0.12146E+01_JPRB, 0.12172E+01_JPRB, 0.12092E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11809E+01_JPRB, 0.11514E+01_JPRB, 0.11149E+01_JPRB, 0.10832E+01_JPRB, 0.10537E+01_JPRB, 0.10315E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.99122E+00_JPRB, 0.96916E+00_JPRB, 0.94181E+00_JPRB, 0.91489E+00_JPRB, 0.88752E+00_JPRB, 0.86615E+00_JPRB ,&
&  0.84581E+00_JPRB, 0.83216E+00_JPRB, 0.81980E+00_JPRB, 0.81217E+00_JPRB, 0.80438E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.91239E+00_JPRB, 0.90743E+00_JPRB, 0.90288E+00_JPRB, 0.89822E+00_JPRB, 0.89189E+00_JPRB, 0.88452E+00_JPRB, 0.87602E+00_JPRB ,&
&  0.86494E+00_JPRB, 0.85241E+00_JPRB, 0.83952E+00_JPRB, 0.82640E+00_JPRB, 0.81432E+00_JPRB, 0.80465E+00_JPRB, 0.79658E+00_JPRB ,&
&  0.78985E+00_JPRB, 0.77932E+00_JPRB, 0.76045E+00_JPRB, 0.73743E+00_JPRB, 0.71715E+00_JPRB, 0.69958E+00_JPRB, 0.68278E+00_JPRB ,&
&  0.65246E+00_JPRB, 0.61931E+00_JPRB, 0.58783E+00_JPRB, 0.55354E+00_JPRB, 0.52700E+00_JPRB, 0.47138E+00_JPRB, 0.45068E+00_JPRB ,&
&  0.43052E+00_JPRB, 0.41942E+00_JPRB, 0.41477E+00_JPRB, 0.41288E+00_JPRB, 0.42320E+00_JPRB, 0.41469E+00_JPRB, 0.40855E+00_JPRB ,&
&  0.38684E+00_JPRB, 0.38505E+00_JPRB, 0.38920E+00_JPRB, 0.39658E+00_JPRB, 0.41145E+00_JPRB, 0.43210E+00_JPRB, 0.46135E+00_JPRB ,&
&  0.51160E+00_JPRB, 0.53594E+00_JPRB, 0.57198E+00_JPRB, 0.60012E+00_JPRB, 0.61767E+00_JPRB, 0.62731E+00_JPRB, 0.63891E+00_JPRB ,&
&  0.65102E+00_JPRB, 0.66983E+00_JPRB, 0.68101E+00_JPRB, 0.68884E+00_JPRB, 0.68895E+00_JPRB, 0.68703E+00_JPRB, 0.68783E+00_JPRB ,&
&  0.68723E+00_JPRB, 0.69463E+00_JPRB, 0.70802E+00_JPRB, 0.73385E+00_JPRB, 0.76988E+00_JPRB, 0.81238E+00_JPRB, 0.85413E+00_JPRB ,&
&  0.89124E+00_JPRB, 0.92964E+00_JPRB, 0.96154E+00_JPRB, 0.99276E+00_JPRB, 0.10265E+01_JPRB, 0.10571E+01_JPRB, 0.10848E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.11209E+01_JPRB, 0.11257E+01_JPRB, 0.11272E+01_JPRB, 0.11224E+01_JPRB, 0.11171E+01_JPRB, 0.11144E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.11069E+01_JPRB, 0.11044E+01_JPRB, 0.10953E+01_JPRB, 0.10818E+01_JPRB, 0.10663E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10187E+01_JPRB, 0.10041E+01_JPRB, 0.99062E+00_JPRB, 0.97963E+00_JPRB, 0.96666E+00_JPRB, 0.95578E+00_JPRB ,&
&  0.94458E+00_JPRB, 0.93852E+00_JPRB, 0.93079E+00_JPRB, 0.92435E+00_JPRB, 0.91762E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.89544E+00_JPRB, 0.88664E+00_JPRB, 0.87773E+00_JPRB, 0.86951E+00_JPRB, 0.85947E+00_JPRB, 0.84942E+00_JPRB, 0.83916E+00_JPRB ,&
&  0.82895E+00_JPRB, 0.81793E+00_JPRB, 0.80711E+00_JPRB, 0.79606E+00_JPRB, 0.78497E+00_JPRB, 0.77630E+00_JPRB, 0.77029E+00_JPRB ,&
&  0.76486E+00_JPRB, 0.75756E+00_JPRB, 0.74891E+00_JPRB, 0.73741E+00_JPRB, 0.72517E+00_JPRB, 0.71339E+00_JPRB, 0.70286E+00_JPRB ,&
&  0.68975E+00_JPRB, 0.67591E+00_JPRB, 0.66157E+00_JPRB, 0.64774E+00_JPRB, 0.63362E+00_JPRB, 0.61945E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.59282E+00_JPRB, 0.58348E+00_JPRB, 0.57417E+00_JPRB, 0.56400E+00_JPRB, 0.55247E+00_JPRB, 0.54837E+00_JPRB, 0.54413E+00_JPRB ,&
&  0.54031E+00_JPRB, 0.53957E+00_JPRB, 0.54485E+00_JPRB, 0.54952E+00_JPRB, 0.55450E+00_JPRB, 0.56036E+00_JPRB, 0.57072E+00_JPRB ,&
&  0.58025E+00_JPRB, 0.59144E+00_JPRB, 0.60610E+00_JPRB, 0.62135E+00_JPRB, 0.63399E+00_JPRB, 0.64333E+00_JPRB, 0.65644E+00_JPRB ,&
&  0.66598E+00_JPRB, 0.67828E+00_JPRB, 0.68721E+00_JPRB, 0.69167E+00_JPRB, 0.69202E+00_JPRB, 0.69231E+00_JPRB, 0.69397E+00_JPRB ,&
&  0.69320E+00_JPRB, 0.69873E+00_JPRB, 0.70390E+00_JPRB, 0.70944E+00_JPRB, 0.71131E+00_JPRB, 0.71859E+00_JPRB, 0.72334E+00_JPRB ,&
&  0.73316E+00_JPRB, 0.74786E+00_JPRB, 0.76393E+00_JPRB, 0.78239E+00_JPRB, 0.80495E+00_JPRB, 0.83309E+00_JPRB, 0.85680E+00_JPRB ,&
&  0.87932E+00_JPRB, 0.90047E+00_JPRB, 0.92019E+00_JPRB, 0.93337E+00_JPRB, 0.94547E+00_JPRB, 0.95659E+00_JPRB, 0.96740E+00_JPRB ,&
&  0.97247E+00_JPRB, 0.97671E+00_JPRB, 0.98028E+00_JPRB, 0.98257E+00_JPRB, 0.98001E+00_JPRB, 0.97621E+00_JPRB, 0.97229E+00_JPRB ,&
&  0.96812E+00_JPRB, 0.96274E+00_JPRB, 0.95821E+00_JPRB, 0.95499E+00_JPRB, 0.95093E+00_JPRB, 0.94534E+00_JPRB, 0.93968E+00_JPRB ,&
&  0.93431E+00_JPRB, 0.92750E+00_JPRB, 0.91984E+00_JPRB, 0.91200E+00_JPRB, 0.90448E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.78672E+00_JPRB, 0.78276E+00_JPRB, 0.77823E+00_JPRB, 0.77418E+00_JPRB, 0.76911E+00_JPRB, 0.76464E+00_JPRB, 0.75999E+00_JPRB ,&
&  0.75588E+00_JPRB, 0.75180E+00_JPRB, 0.74802E+00_JPRB, 0.74372E+00_JPRB, 0.73962E+00_JPRB, 0.73562E+00_JPRB, 0.73172E+00_JPRB ,&
&  0.72765E+00_JPRB, 0.72393E+00_JPRB, 0.71993E+00_JPRB, 0.71544E+00_JPRB, 0.71178E+00_JPRB, 0.70765E+00_JPRB, 0.70161E+00_JPRB ,&
&  0.69270E+00_JPRB, 0.68326E+00_JPRB, 0.67439E+00_JPRB, 0.66738E+00_JPRB, 0.66232E+00_JPRB, 0.65824E+00_JPRB, 0.65474E+00_JPRB ,&
&  0.65259E+00_JPRB, 0.65123E+00_JPRB, 0.65002E+00_JPRB, 0.64784E+00_JPRB, 0.64621E+00_JPRB, 0.64686E+00_JPRB, 0.64704E+00_JPRB ,&
&  0.64640E+00_JPRB, 0.64624E+00_JPRB, 0.64742E+00_JPRB, 0.64854E+00_JPRB, 0.64925E+00_JPRB, 0.65138E+00_JPRB, 0.65526E+00_JPRB ,&
&  0.65919E+00_JPRB, 0.66266E+00_JPRB, 0.66740E+00_JPRB, 0.67223E+00_JPRB, 0.67699E+00_JPRB, 0.68127E+00_JPRB, 0.68765E+00_JPRB ,&
&  0.69432E+00_JPRB, 0.70043E+00_JPRB, 0.70599E+00_JPRB, 0.71015E+00_JPRB, 0.71331E+00_JPRB, 0.71471E+00_JPRB, 0.71506E+00_JPRB ,&
&  0.71432E+00_JPRB, 0.71522E+00_JPRB, 0.71495E+00_JPRB, 0.71461E+00_JPRB, 0.71327E+00_JPRB, 0.71429E+00_JPRB, 0.71502E+00_JPRB ,&
&  0.71754E+00_JPRB, 0.71920E+00_JPRB, 0.72259E+00_JPRB, 0.72591E+00_JPRB, 0.73102E+00_JPRB, 0.73654E+00_JPRB, 0.74243E+00_JPRB ,&
&  0.74908E+00_JPRB, 0.75698E+00_JPRB, 0.76670E+00_JPRB, 0.77318E+00_JPRB, 0.77941E+00_JPRB, 0.78531E+00_JPRB, 0.79170E+00_JPRB ,&
&  0.79694E+00_JPRB, 0.80159E+00_JPRB, 0.80609E+00_JPRB, 0.80937E+00_JPRB, 0.81131E+00_JPRB, 0.81290E+00_JPRB, 0.81435E+00_JPRB ,&
&  0.81473E+00_JPRB, 0.81470E+00_JPRB, 0.81444E+00_JPRB, 0.81425E+00_JPRB, 0.81249E+00_JPRB, 0.80995E+00_JPRB, 0.80708E+00_JPRB ,&
&  0.80462E+00_JPRB, 0.80125E+00_JPRB, 0.79797E+00_JPRB, 0.79433E+00_JPRB, 0.79085E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB ,&
&  0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB, 0.71492E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB ,&
&  0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB, 0.24537E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.25980E+00_JPRB, 0.26003E+00_JPRB, 0.26021E+00_JPRB, 0.26038E+00_JPRB, 0.25942E+00_JPRB, 0.26000E+00_JPRB, 0.26048E+00_JPRB ,&
&  0.26121E+00_JPRB, 0.26243E+00_JPRB, 0.26387E+00_JPRB, 0.26498E+00_JPRB, 0.26582E+00_JPRB, 0.26634E+00_JPRB, 0.26670E+00_JPRB ,&
&  0.26675E+00_JPRB, 0.26695E+00_JPRB, 0.26394E+00_JPRB, 0.26287E+00_JPRB, 0.26255E+00_JPRB, 0.26261E+00_JPRB, 0.26282E+00_JPRB ,&
&  0.26297E+00_JPRB, 0.26299E+00_JPRB, 0.26255E+00_JPRB, 0.26283E+00_JPRB, 0.26312E+00_JPRB, 0.26332E+00_JPRB, 0.26356E+00_JPRB ,&
&  0.26377E+00_JPRB, 0.26393E+00_JPRB, 0.26385E+00_JPRB, 0.26363E+00_JPRB, 0.26345E+00_JPRB, 0.26291E+00_JPRB, 0.26204E+00_JPRB ,&
&  0.26097E+00_JPRB, 0.25965E+00_JPRB, 0.25834E+00_JPRB, 0.25714E+00_JPRB, 0.25609E+00_JPRB, 0.25520E+00_JPRB, 0.25434E+00_JPRB ,&
&  0.25333E+00_JPRB, 0.25203E+00_JPRB, 0.25066E+00_JPRB, 0.24933E+00_JPRB, 0.24805E+00_JPRB, 0.24689E+00_JPRB, 0.24568E+00_JPRB ,&
&  0.24474E+00_JPRB, 0.24395E+00_JPRB, 0.24315E+00_JPRB, 0.24246E+00_JPRB, 0.24180E+00_JPRB, 0.24117E+00_JPRB, 0.24056E+00_JPRB ,&
&  0.23975E+00_JPRB, 0.23929E+00_JPRB, 0.23878E+00_JPRB, 0.23852E+00_JPRB, 0.23853E+00_JPRB, 0.23853E+00_JPRB, 0.23860E+00_JPRB ,&
&  0.23874E+00_JPRB, 0.23894E+00_JPRB, 0.23905E+00_JPRB, 0.23925E+00_JPRB, 0.23954E+00_JPRB, 0.23994E+00_JPRB, 0.24043E+00_JPRB ,&
&  0.24096E+00_JPRB, 0.24154E+00_JPRB, 0.24217E+00_JPRB, 0.24288E+00_JPRB, 0.24363E+00_JPRB, 0.24445E+00_JPRB, 0.24524E+00_JPRB ,&
&  0.24603E+00_JPRB, 0.24676E+00_JPRB, 0.24750E+00_JPRB, 0.24807E+00_JPRB, 0.24887E+00_JPRB, 0.24965E+00_JPRB, 0.25057E+00_JPRB ,&
&  0.25151E+00_JPRB, 0.25250E+00_JPRB, 0.25337E+00_JPRB, 0.25430E+00_JPRB, 0.25530E+00_JPRB, 0.25519E+00_JPRB, 0.25513E+00_JPRB ,&
&  0.25516E+00_JPRB, 0.25546E+00_JPRB, 0.25551E+00_JPRB, 0.25588E+00_JPRB, 0.25673E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.28826E+00_JPRB, 0.29480E+00_JPRB, 0.29966E+00_JPRB, 0.30469E+00_JPRB, 0.30130E+00_JPRB, 0.33837E+00_JPRB, 0.33686E+00_JPRB ,&
&  0.33741E+00_JPRB, 0.32359E+00_JPRB, 0.32380E+00_JPRB, 0.32224E+00_JPRB, 0.31921E+00_JPRB, 0.31481E+00_JPRB, 0.31071E+00_JPRB ,&
&  0.30710E+00_JPRB, 0.30235E+00_JPRB, 0.29685E+00_JPRB, 0.29216E+00_JPRB, 0.28840E+00_JPRB, 0.28519E+00_JPRB, 0.28244E+00_JPRB ,&
&  0.28180E+00_JPRB, 0.28303E+00_JPRB, 0.28541E+00_JPRB, 0.28449E+00_JPRB, 0.28563E+00_JPRB, 0.28765E+00_JPRB, 0.28999E+00_JPRB ,&
&  0.29222E+00_JPRB, 0.29407E+00_JPRB, 0.29411E+00_JPRB, 0.29244E+00_JPRB, 0.29111E+00_JPRB, 0.28868E+00_JPRB, 0.28643E+00_JPRB ,&
&  0.28377E+00_JPRB, 0.28082E+00_JPRB, 0.27737E+00_JPRB, 0.27460E+00_JPRB, 0.27160E+00_JPRB, 0.26795E+00_JPRB, 0.26443E+00_JPRB ,&
&  0.26141E+00_JPRB, 0.25871E+00_JPRB, 0.25581E+00_JPRB, 0.25312E+00_JPRB, 0.25057E+00_JPRB, 0.24796E+00_JPRB, 0.24569E+00_JPRB ,&
&  0.24396E+00_JPRB, 0.24250E+00_JPRB, 0.24104E+00_JPRB, 0.23951E+00_JPRB, 0.23801E+00_JPRB, 0.23655E+00_JPRB, 0.23536E+00_JPRB ,&
&  0.23512E+00_JPRB, 0.23442E+00_JPRB, 0.23373E+00_JPRB, 0.23331E+00_JPRB, 0.23330E+00_JPRB, 0.23281E+00_JPRB, 0.23259E+00_JPRB ,&
&  0.23252E+00_JPRB, 0.23254E+00_JPRB, 0.23277E+00_JPRB, 0.23308E+00_JPRB, 0.23350E+00_JPRB, 0.23405E+00_JPRB, 0.23493E+00_JPRB ,&
&  0.23598E+00_JPRB, 0.23724E+00_JPRB, 0.23859E+00_JPRB, 0.23998E+00_JPRB, 0.24140E+00_JPRB, 0.24290E+00_JPRB, 0.24458E+00_JPRB ,&
&  0.24628E+00_JPRB, 0.24793E+00_JPRB, 0.24955E+00_JPRB, 0.25115E+00_JPRB, 0.25371E+00_JPRB, 0.25618E+00_JPRB, 0.25883E+00_JPRB ,&
&  0.26168E+00_JPRB, 0.26504E+00_JPRB, 0.26857E+00_JPRB, 0.27236E+00_JPRB, 0.27657E+00_JPRB, 0.27832E+00_JPRB, 0.27973E+00_JPRB ,&
&  0.28073E+00_JPRB, 0.28364E+00_JPRB, 0.28593E+00_JPRB, 0.28596E+00_JPRB, 0.28584E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.34640E+00_JPRB, 0.34438E+00_JPRB, 0.34482E+00_JPRB, 0.34572E+00_JPRB, 0.34913E+00_JPRB, 0.35427E+00_JPRB, 0.35985E+00_JPRB ,&
&  0.35939E+00_JPRB, 0.36246E+00_JPRB, 0.36167E+00_JPRB, 0.35919E+00_JPRB, 0.35499E+00_JPRB, 0.35108E+00_JPRB, 0.34625E+00_JPRB ,&
&  0.33940E+00_JPRB, 0.33065E+00_JPRB, 0.32211E+00_JPRB, 0.31345E+00_JPRB, 0.30520E+00_JPRB, 0.29663E+00_JPRB, 0.29140E+00_JPRB ,&
&  0.28814E+00_JPRB, 0.28725E+00_JPRB, 0.28922E+00_JPRB, 0.29203E+00_JPRB, 0.29740E+00_JPRB, 0.30471E+00_JPRB, 0.31440E+00_JPRB ,&
&  0.32394E+00_JPRB, 0.33452E+00_JPRB, 0.33999E+00_JPRB, 0.33984E+00_JPRB, 0.33953E+00_JPRB, 0.33738E+00_JPRB, 0.33384E+00_JPRB ,&
&  0.32735E+00_JPRB, 0.31993E+00_JPRB, 0.31143E+00_JPRB, 0.30244E+00_JPRB, 0.29328E+00_JPRB, 0.28453E+00_JPRB, 0.27639E+00_JPRB ,&
&  0.26977E+00_JPRB, 0.26414E+00_JPRB, 0.25903E+00_JPRB, 0.25475E+00_JPRB, 0.25144E+00_JPRB, 0.24895E+00_JPRB, 0.24600E+00_JPRB ,&
&  0.24351E+00_JPRB, 0.24133E+00_JPRB, 0.23875E+00_JPRB, 0.23695E+00_JPRB, 0.23534E+00_JPRB, 0.23428E+00_JPRB, 0.23341E+00_JPRB ,&
&  0.23328E+00_JPRB, 0.23315E+00_JPRB, 0.23367E+00_JPRB, 0.23418E+00_JPRB, 0.23479E+00_JPRB, 0.23469E+00_JPRB, 0.23442E+00_JPRB ,&
&  0.23398E+00_JPRB, 0.23357E+00_JPRB, 0.23229E+00_JPRB, 0.23130E+00_JPRB, 0.23063E+00_JPRB, 0.22989E+00_JPRB, 0.22989E+00_JPRB ,&
&  0.23074E+00_JPRB, 0.23162E+00_JPRB, 0.23237E+00_JPRB, 0.23395E+00_JPRB, 0.23554E+00_JPRB, 0.23783E+00_JPRB, 0.24057E+00_JPRB ,&
&  0.24260E+00_JPRB, 0.24498E+00_JPRB, 0.24765E+00_JPRB, 0.25073E+00_JPRB, 0.25441E+00_JPRB, 0.25824E+00_JPRB, 0.26272E+00_JPRB ,&
&  0.26777E+00_JPRB, 0.27299E+00_JPRB, 0.27888E+00_JPRB, 0.28537E+00_JPRB, 0.29326E+00_JPRB, 0.30036E+00_JPRB, 0.31069E+00_JPRB ,&
&  0.31900E+00_JPRB, 0.31905E+00_JPRB, 0.35936E+00_JPRB, 0.35306E+00_JPRB, 0.34887E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.34418E+00_JPRB, 0.34689E+00_JPRB, 0.35019E+00_JPRB, 0.35419E+00_JPRB, 0.35914E+00_JPRB, 0.36353E+00_JPRB, 0.36857E+00_JPRB ,&
&  0.37151E+00_JPRB, 0.36806E+00_JPRB, 0.36858E+00_JPRB, 0.36441E+00_JPRB, 0.36012E+00_JPRB, 0.35461E+00_JPRB, 0.34859E+00_JPRB ,&
&  0.34219E+00_JPRB, 0.33321E+00_JPRB, 0.32361E+00_JPRB, 0.31508E+00_JPRB, 0.30789E+00_JPRB, 0.30221E+00_JPRB, 0.29762E+00_JPRB ,&
&  0.29446E+00_JPRB, 0.29471E+00_JPRB, 0.29681E+00_JPRB, 0.30281E+00_JPRB, 0.31288E+00_JPRB, 0.32547E+00_JPRB, 0.34451E+00_JPRB ,&
&  0.37147E+00_JPRB, 0.37358E+00_JPRB, 0.37837E+00_JPRB, 0.37736E+00_JPRB, 0.37313E+00_JPRB, 0.37097E+00_JPRB, 0.36676E+00_JPRB ,&
&  0.36720E+00_JPRB, 0.36005E+00_JPRB, 0.36985E+00_JPRB, 0.35378E+00_JPRB, 0.32736E+00_JPRB, 0.31198E+00_JPRB, 0.29372E+00_JPRB ,&
&  0.28018E+00_JPRB, 0.26867E+00_JPRB, 0.26004E+00_JPRB, 0.25626E+00_JPRB, 0.25253E+00_JPRB, 0.25076E+00_JPRB, 0.24716E+00_JPRB ,&
&  0.24342E+00_JPRB, 0.24147E+00_JPRB, 0.23837E+00_JPRB, 0.23805E+00_JPRB, 0.23810E+00_JPRB, 0.23754E+00_JPRB, 0.23763E+00_JPRB ,&
&  0.23801E+00_JPRB, 0.23898E+00_JPRB, 0.24069E+00_JPRB, 0.24226E+00_JPRB, 0.24319E+00_JPRB, 0.24374E+00_JPRB, 0.24359E+00_JPRB ,&
&  0.24400E+00_JPRB, 0.24368E+00_JPRB, 0.24287E+00_JPRB, 0.24159E+00_JPRB, 0.23970E+00_JPRB, 0.23771E+00_JPRB, 0.23583E+00_JPRB ,&
&  0.23432E+00_JPRB, 0.23379E+00_JPRB, 0.23335E+00_JPRB, 0.23361E+00_JPRB, 0.23423E+00_JPRB, 0.23598E+00_JPRB, 0.23762E+00_JPRB ,&
&  0.23950E+00_JPRB, 0.24210E+00_JPRB, 0.24439E+00_JPRB, 0.24719E+00_JPRB, 0.24978E+00_JPRB, 0.25139E+00_JPRB, 0.25542E+00_JPRB ,&
&  0.26057E+00_JPRB, 0.26747E+00_JPRB, 0.27554E+00_JPRB, 0.28340E+00_JPRB, 0.29342E+00_JPRB, 0.30915E+00_JPRB, 0.33055E+00_JPRB ,&
&  0.35254E+00_JPRB, 0.37673E+00_JPRB, 0.37118E+00_JPRB, 0.34298E+00_JPRB, 0.34128E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.35518E+00_JPRB, 0.36192E+00_JPRB, 0.36901E+00_JPRB, 0.37657E+00_JPRB, 0.38599E+00_JPRB, 0.39001E+00_JPRB, 0.39200E+00_JPRB ,&
&  0.38797E+00_JPRB, 0.38327E+00_JPRB, 0.37238E+00_JPRB, 0.36226E+00_JPRB, 0.35274E+00_JPRB, 0.34515E+00_JPRB, 0.33943E+00_JPRB ,&
&  0.33618E+00_JPRB, 0.33366E+00_JPRB, 0.32990E+00_JPRB, 0.32659E+00_JPRB, 0.32376E+00_JPRB, 0.32372E+00_JPRB, 0.32386E+00_JPRB ,&
&  0.32309E+00_JPRB, 0.32320E+00_JPRB, 0.32192E+00_JPRB, 0.32708E+00_JPRB, 0.33619E+00_JPRB, 0.34738E+00_JPRB, 0.36199E+00_JPRB ,&
&  0.38244E+00_JPRB, 0.39364E+00_JPRB, 0.40914E+00_JPRB, 0.40961E+00_JPRB, 0.40771E+00_JPRB, 0.40384E+00_JPRB, 0.40069E+00_JPRB ,&
&  0.39601E+00_JPRB, 0.38930E+00_JPRB, 0.38441E+00_JPRB, 0.37837E+00_JPRB, 0.34925E+00_JPRB, 0.32629E+00_JPRB, 0.30868E+00_JPRB ,&
&  0.29335E+00_JPRB, 0.27887E+00_JPRB, 0.26623E+00_JPRB, 0.25981E+00_JPRB, 0.25475E+00_JPRB, 0.25176E+00_JPRB, 0.24865E+00_JPRB ,&
&  0.24534E+00_JPRB, 0.24408E+00_JPRB, 0.24216E+00_JPRB, 0.24148E+00_JPRB, 0.24247E+00_JPRB, 0.24245E+00_JPRB, 0.24321E+00_JPRB ,&
&  0.24387E+00_JPRB, 0.24486E+00_JPRB, 0.24718E+00_JPRB, 0.24893E+00_JPRB, 0.25078E+00_JPRB, 0.25254E+00_JPRB, 0.25343E+00_JPRB ,&
&  0.25519E+00_JPRB, 0.25667E+00_JPRB, 0.25665E+00_JPRB, 0.25596E+00_JPRB, 0.25426E+00_JPRB, 0.25048E+00_JPRB, 0.24669E+00_JPRB ,&
&  0.24263E+00_JPRB, 0.23968E+00_JPRB, 0.23848E+00_JPRB, 0.23751E+00_JPRB, 0.23831E+00_JPRB, 0.23912E+00_JPRB, 0.24006E+00_JPRB ,&
&  0.24193E+00_JPRB, 0.24368E+00_JPRB, 0.24568E+00_JPRB, 0.24851E+00_JPRB, 0.25071E+00_JPRB, 0.25263E+00_JPRB, 0.25509E+00_JPRB ,&
&  0.25556E+00_JPRB, 0.25982E+00_JPRB, 0.26516E+00_JPRB, 0.27197E+00_JPRB, 0.28355E+00_JPRB, 0.29356E+00_JPRB, 0.31617E+00_JPRB ,&
&  0.35245E+00_JPRB, 0.42097E+00_JPRB, 0.41507E+00_JPRB, 0.34634E+00_JPRB, 0.34923E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.37844E+00_JPRB, 0.38718E+00_JPRB, 0.39490E+00_JPRB, 0.40087E+00_JPRB, 0.40455E+00_JPRB, 0.40523E+00_JPRB, 0.40790E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.41965E+00_JPRB, 0.42272E+00_JPRB, 0.42204E+00_JPRB, 0.42148E+00_JPRB, 0.40270E+00_JPRB, 0.39785E+00_JPRB ,&
&  0.39192E+00_JPRB, 0.38776E+00_JPRB, 0.38207E+00_JPRB, 0.38200E+00_JPRB, 0.38180E+00_JPRB, 0.38376E+00_JPRB, 0.38450E+00_JPRB ,&
&  0.38456E+00_JPRB, 0.38255E+00_JPRB, 0.37935E+00_JPRB, 0.37737E+00_JPRB, 0.37864E+00_JPRB, 0.38009E+00_JPRB, 0.37720E+00_JPRB ,&
&  0.37293E+00_JPRB, 0.37199E+00_JPRB, 0.37522E+00_JPRB, 0.38255E+00_JPRB, 0.42307E+00_JPRB, 0.41790E+00_JPRB, 0.41541E+00_JPRB ,&
&  0.41205E+00_JPRB, 0.40785E+00_JPRB, 0.40725E+00_JPRB, 0.40075E+00_JPRB, 0.39121E+00_JPRB, 0.37020E+00_JPRB, 0.35844E+00_JPRB ,&
&  0.32389E+00_JPRB, 0.30468E+00_JPRB, 0.28778E+00_JPRB, 0.27672E+00_JPRB, 0.26663E+00_JPRB, 0.26079E+00_JPRB, 0.25551E+00_JPRB ,&
&  0.25150E+00_JPRB, 0.24837E+00_JPRB, 0.24557E+00_JPRB, 0.24602E+00_JPRB, 0.24772E+00_JPRB, 0.24868E+00_JPRB, 0.24964E+00_JPRB ,&
&  0.25019E+00_JPRB, 0.25065E+00_JPRB, 0.25126E+00_JPRB, 0.25253E+00_JPRB, 0.25423E+00_JPRB, 0.25648E+00_JPRB, 0.25935E+00_JPRB ,&
&  0.26405E+00_JPRB, 0.26780E+00_JPRB, 0.26935E+00_JPRB, 0.27195E+00_JPRB, 0.27169E+00_JPRB, 0.26627E+00_JPRB, 0.25993E+00_JPRB ,&
&  0.25332E+00_JPRB, 0.24988E+00_JPRB, 0.24650E+00_JPRB, 0.24417E+00_JPRB, 0.24361E+00_JPRB, 0.24361E+00_JPRB, 0.24549E+00_JPRB ,&
&  0.24755E+00_JPRB, 0.24937E+00_JPRB, 0.25063E+00_JPRB, 0.25203E+00_JPRB, 0.25382E+00_JPRB, 0.25523E+00_JPRB, 0.25710E+00_JPRB ,&
&  0.25947E+00_JPRB, 0.26269E+00_JPRB, 0.26747E+00_JPRB, 0.27298E+00_JPRB, 0.27988E+00_JPRB, 0.29027E+00_JPRB, 0.30870E+00_JPRB ,&
&  0.33342E+00_JPRB, 0.38968E+00_JPRB, 0.39546E+00_JPRB, 0.36513E+00_JPRB, 0.37122E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.39109E+00_JPRB, 0.40126E+00_JPRB, 0.40660E+00_JPRB, 0.41188E+00_JPRB, 0.41402E+00_JPRB, 0.41537E+00_JPRB, 0.42324E+00_JPRB ,&
&  0.42803E+00_JPRB, 0.43176E+00_JPRB, 0.43403E+00_JPRB, 0.43487E+00_JPRB, 0.43053E+00_JPRB, 0.42389E+00_JPRB, 0.42171E+00_JPRB ,&
&  0.42050E+00_JPRB, 0.42093E+00_JPRB, 0.41982E+00_JPRB, 0.41980E+00_JPRB, 0.42031E+00_JPRB, 0.42509E+00_JPRB, 0.43187E+00_JPRB ,&
&  0.43347E+00_JPRB, 0.43804E+00_JPRB, 0.45064E+00_JPRB, 0.45837E+00_JPRB, 0.46326E+00_JPRB, 0.46292E+00_JPRB, 0.45722E+00_JPRB ,&
&  0.42994E+00_JPRB, 0.40204E+00_JPRB, 0.37462E+00_JPRB, 0.37612E+00_JPRB, 0.42944E+00_JPRB, 0.43577E+00_JPRB, 0.43496E+00_JPRB ,&
&  0.43336E+00_JPRB, 0.43768E+00_JPRB, 0.43780E+00_JPRB, 0.42321E+00_JPRB, 0.41254E+00_JPRB, 0.39777E+00_JPRB, 0.38508E+00_JPRB ,&
&  0.37431E+00_JPRB, 0.36653E+00_JPRB, 0.35338E+00_JPRB, 0.32616E+00_JPRB, 0.30855E+00_JPRB, 0.29954E+00_JPRB, 0.28584E+00_JPRB ,&
&  0.27555E+00_JPRB, 0.27134E+00_JPRB, 0.27030E+00_JPRB, 0.26983E+00_JPRB, 0.27386E+00_JPRB, 0.27578E+00_JPRB, 0.27458E+00_JPRB ,&
&  0.27143E+00_JPRB, 0.26877E+00_JPRB, 0.26512E+00_JPRB, 0.25998E+00_JPRB, 0.25722E+00_JPRB, 0.25629E+00_JPRB, 0.25668E+00_JPRB ,&
&  0.26040E+00_JPRB, 0.26632E+00_JPRB, 0.27954E+00_JPRB, 0.28897E+00_JPRB, 0.28577E+00_JPRB, 0.27467E+00_JPRB, 0.26435E+00_JPRB ,&
&  0.25774E+00_JPRB, 0.25504E+00_JPRB, 0.25303E+00_JPRB, 0.25096E+00_JPRB, 0.25157E+00_JPRB, 0.25355E+00_JPRB, 0.25501E+00_JPRB ,&
&  0.25530E+00_JPRB, 0.25689E+00_JPRB, 0.25780E+00_JPRB, 0.25893E+00_JPRB, 0.25991E+00_JPRB, 0.26156E+00_JPRB, 0.26384E+00_JPRB ,&
&  0.26609E+00_JPRB, 0.27132E+00_JPRB, 0.27718E+00_JPRB, 0.28372E+00_JPRB, 0.28784E+00_JPRB, 0.29092E+00_JPRB, 0.29692E+00_JPRB ,&
&  0.30329E+00_JPRB, 0.31906E+00_JPRB, 0.34985E+00_JPRB, 0.38713E+00_JPRB, 0.38195E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.40230E+00_JPRB, 0.40937E+00_JPRB, 0.41772E+00_JPRB, 0.42861E+00_JPRB, 0.43095E+00_JPRB, 0.43253E+00_JPRB, 0.44167E+00_JPRB ,&
&  0.44212E+00_JPRB, 0.43918E+00_JPRB, 0.43592E+00_JPRB, 0.43609E+00_JPRB, 0.43808E+00_JPRB, 0.44036E+00_JPRB, 0.43975E+00_JPRB ,&
&  0.43051E+00_JPRB, 0.42516E+00_JPRB, 0.42776E+00_JPRB, 0.43064E+00_JPRB, 0.43514E+00_JPRB, 0.44164E+00_JPRB, 0.44730E+00_JPRB ,&
&  0.45035E+00_JPRB, 0.45392E+00_JPRB, 0.45635E+00_JPRB, 0.46255E+00_JPRB, 0.47214E+00_JPRB, 0.47876E+00_JPRB, 0.48510E+00_JPRB ,&
&  0.47121E+00_JPRB, 0.42360E+00_JPRB, 0.39306E+00_JPRB, 0.39951E+00_JPRB, 0.44922E+00_JPRB, 0.45939E+00_JPRB, 0.46022E+00_JPRB ,&
&  0.46321E+00_JPRB, 0.47077E+00_JPRB, 0.47017E+00_JPRB, 0.46422E+00_JPRB, 0.45241E+00_JPRB, 0.43345E+00_JPRB, 0.41027E+00_JPRB ,&
&  0.39473E+00_JPRB, 0.38435E+00_JPRB, 0.37756E+00_JPRB, 0.37093E+00_JPRB, 0.36743E+00_JPRB, 0.36251E+00_JPRB, 0.35490E+00_JPRB ,&
&  0.35192E+00_JPRB, 0.35052E+00_JPRB, 0.35153E+00_JPRB, 0.35317E+00_JPRB, 0.35515E+00_JPRB, 0.35655E+00_JPRB, 0.35380E+00_JPRB ,&
&  0.34153E+00_JPRB, 0.32600E+00_JPRB, 0.31116E+00_JPRB, 0.29501E+00_JPRB, 0.28119E+00_JPRB, 0.27243E+00_JPRB, 0.26547E+00_JPRB ,&
&  0.26267E+00_JPRB, 0.26821E+00_JPRB, 0.28485E+00_JPRB, 0.30408E+00_JPRB, 0.33632E+00_JPRB, 0.31195E+00_JPRB, 0.28846E+00_JPRB ,&
&  0.27697E+00_JPRB, 0.27145E+00_JPRB, 0.26838E+00_JPRB, 0.26837E+00_JPRB, 0.26996E+00_JPRB, 0.27043E+00_JPRB, 0.27051E+00_JPRB ,&
&  0.27171E+00_JPRB, 0.27400E+00_JPRB, 0.27581E+00_JPRB, 0.27736E+00_JPRB, 0.27981E+00_JPRB, 0.28267E+00_JPRB, 0.28422E+00_JPRB ,&
&  0.28497E+00_JPRB, 0.28964E+00_JPRB, 0.29915E+00_JPRB, 0.31012E+00_JPRB, 0.32114E+00_JPRB, 0.33261E+00_JPRB, 0.35030E+00_JPRB ,&
&  0.37242E+00_JPRB, 0.39927E+00_JPRB, 0.39864E+00_JPRB, 0.39866E+00_JPRB, 0.39906E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.42519E+00_JPRB, 0.43663E+00_JPRB, 0.44999E+00_JPRB, 0.45931E+00_JPRB, 0.45833E+00_JPRB, 0.45828E+00_JPRB, 0.46448E+00_JPRB ,&
&  0.45854E+00_JPRB, 0.45141E+00_JPRB, 0.44546E+00_JPRB, 0.43932E+00_JPRB, 0.43710E+00_JPRB, 0.43573E+00_JPRB, 0.43868E+00_JPRB ,&
&  0.44081E+00_JPRB, 0.43971E+00_JPRB, 0.43799E+00_JPRB, 0.43864E+00_JPRB, 0.44512E+00_JPRB, 0.45331E+00_JPRB, 0.46020E+00_JPRB ,&
&  0.46148E+00_JPRB, 0.46500E+00_JPRB, 0.47069E+00_JPRB, 0.47551E+00_JPRB, 0.47799E+00_JPRB, 0.47954E+00_JPRB, 0.47840E+00_JPRB ,&
&  0.47595E+00_JPRB, 0.47397E+00_JPRB, 0.45705E+00_JPRB, 0.47081E+00_JPRB, 0.47527E+00_JPRB, 0.47615E+00_JPRB, 0.48171E+00_JPRB ,&
&  0.48951E+00_JPRB, 0.49820E+00_JPRB, 0.50215E+00_JPRB, 0.50333E+00_JPRB, 0.49863E+00_JPRB, 0.48928E+00_JPRB, 0.46504E+00_JPRB ,&
&  0.44467E+00_JPRB, 0.42349E+00_JPRB, 0.41026E+00_JPRB, 0.40225E+00_JPRB, 0.39598E+00_JPRB, 0.39054E+00_JPRB, 0.38531E+00_JPRB ,&
&  0.37810E+00_JPRB, 0.37166E+00_JPRB, 0.36773E+00_JPRB, 0.36779E+00_JPRB, 0.37197E+00_JPRB, 0.38287E+00_JPRB, 0.39632E+00_JPRB ,&
&  0.40228E+00_JPRB, 0.39484E+00_JPRB, 0.38681E+00_JPRB, 0.37809E+00_JPRB, 0.36409E+00_JPRB, 0.33647E+00_JPRB, 0.31970E+00_JPRB ,&
&  0.31557E+00_JPRB, 0.32813E+00_JPRB, 0.34937E+00_JPRB, 0.35520E+00_JPRB, 0.36035E+00_JPRB, 0.35808E+00_JPRB, 0.35280E+00_JPRB ,&
&  0.34426E+00_JPRB, 0.32088E+00_JPRB, 0.30821E+00_JPRB, 0.30450E+00_JPRB, 0.30228E+00_JPRB, 0.30247E+00_JPRB, 0.30502E+00_JPRB ,&
&  0.30918E+00_JPRB, 0.31650E+00_JPRB, 0.32162E+00_JPRB, 0.32368E+00_JPRB, 0.32673E+00_JPRB, 0.32686E+00_JPRB, 0.32579E+00_JPRB ,&
&  0.32215E+00_JPRB, 0.32868E+00_JPRB, 0.34693E+00_JPRB, 0.37556E+00_JPRB, 0.38198E+00_JPRB, 0.38957E+00_JPRB, 0.39857E+00_JPRB ,&
&  0.40400E+00_JPRB, 0.40912E+00_JPRB, 0.40961E+00_JPRB, 0.41065E+00_JPRB, 0.41610E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.44448E+00_JPRB, 0.45733E+00_JPRB, 0.47353E+00_JPRB, 0.48502E+00_JPRB, 0.48996E+00_JPRB, 0.49537E+00_JPRB, 0.49777E+00_JPRB ,&
&  0.49275E+00_JPRB, 0.49070E+00_JPRB, 0.48512E+00_JPRB, 0.47700E+00_JPRB, 0.46997E+00_JPRB, 0.46642E+00_JPRB, 0.46677E+00_JPRB ,&
&  0.46894E+00_JPRB, 0.47079E+00_JPRB, 0.46730E+00_JPRB, 0.46549E+00_JPRB, 0.46931E+00_JPRB, 0.47455E+00_JPRB, 0.47796E+00_JPRB ,&
&  0.47930E+00_JPRB, 0.48072E+00_JPRB, 0.48548E+00_JPRB, 0.48856E+00_JPRB, 0.48875E+00_JPRB, 0.48224E+00_JPRB, 0.47745E+00_JPRB ,&
&  0.47256E+00_JPRB, 0.47089E+00_JPRB, 0.47756E+00_JPRB, 0.46953E+00_JPRB, 0.48436E+00_JPRB, 0.48634E+00_JPRB, 0.49192E+00_JPRB ,&
&  0.50011E+00_JPRB, 0.51367E+00_JPRB, 0.52406E+00_JPRB, 0.53202E+00_JPRB, 0.53446E+00_JPRB, 0.53764E+00_JPRB, 0.52784E+00_JPRB ,&
&  0.51413E+00_JPRB, 0.49549E+00_JPRB, 0.47681E+00_JPRB, 0.45666E+00_JPRB, 0.43740E+00_JPRB, 0.42239E+00_JPRB, 0.41501E+00_JPRB ,&
&  0.40608E+00_JPRB, 0.39227E+00_JPRB, 0.37897E+00_JPRB, 0.37376E+00_JPRB, 0.37839E+00_JPRB, 0.38856E+00_JPRB, 0.40411E+00_JPRB ,&
&  0.41357E+00_JPRB, 0.41627E+00_JPRB, 0.41307E+00_JPRB, 0.40873E+00_JPRB, 0.40423E+00_JPRB, 0.39711E+00_JPRB, 0.39297E+00_JPRB ,&
&  0.38956E+00_JPRB, 0.38387E+00_JPRB, 0.38094E+00_JPRB, 0.38270E+00_JPRB, 0.38584E+00_JPRB, 0.38780E+00_JPRB, 0.38619E+00_JPRB ,&
&  0.38317E+00_JPRB, 0.37877E+00_JPRB, 0.37078E+00_JPRB, 0.36632E+00_JPRB, 0.36653E+00_JPRB, 0.37098E+00_JPRB, 0.37863E+00_JPRB ,&
&  0.38112E+00_JPRB, 0.38703E+00_JPRB, 0.39280E+00_JPRB, 0.39491E+00_JPRB, 0.39750E+00_JPRB, 0.39963E+00_JPRB, 0.39980E+00_JPRB ,&
&  0.39809E+00_JPRB, 0.39307E+00_JPRB, 0.38607E+00_JPRB, 0.38299E+00_JPRB, 0.38398E+00_JPRB, 0.39066E+00_JPRB, 0.39985E+00_JPRB ,&
&  0.40899E+00_JPRB, 0.42284E+00_JPRB, 0.43353E+00_JPRB, 0.43602E+00_JPRB, 0.43834E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.47397E+00_JPRB, 0.47926E+00_JPRB, 0.49240E+00_JPRB, 0.50327E+00_JPRB, 0.50676E+00_JPRB, 0.51622E+00_JPRB, 0.52729E+00_JPRB ,&
&  0.53645E+00_JPRB, 0.53828E+00_JPRB, 0.53450E+00_JPRB, 0.52415E+00_JPRB, 0.51193E+00_JPRB, 0.50187E+00_JPRB, 0.49784E+00_JPRB ,&
&  0.50003E+00_JPRB, 0.50312E+00_JPRB, 0.50867E+00_JPRB, 0.51171E+00_JPRB, 0.51533E+00_JPRB, 0.51407E+00_JPRB, 0.51442E+00_JPRB ,&
&  0.51616E+00_JPRB, 0.51716E+00_JPRB, 0.51770E+00_JPRB, 0.51413E+00_JPRB, 0.51133E+00_JPRB, 0.50887E+00_JPRB, 0.50038E+00_JPRB ,&
&  0.49090E+00_JPRB, 0.48452E+00_JPRB, 0.48842E+00_JPRB, 0.49984E+00_JPRB, 0.50714E+00_JPRB, 0.50431E+00_JPRB, 0.50302E+00_JPRB ,&
&  0.50321E+00_JPRB, 0.50810E+00_JPRB, 0.51594E+00_JPRB, 0.52777E+00_JPRB, 0.53611E+00_JPRB, 0.54711E+00_JPRB, 0.54767E+00_JPRB ,&
&  0.54119E+00_JPRB, 0.53187E+00_JPRB, 0.52891E+00_JPRB, 0.52782E+00_JPRB, 0.49919E+00_JPRB, 0.46772E+00_JPRB, 0.45227E+00_JPRB ,&
&  0.44400E+00_JPRB, 0.43371E+00_JPRB, 0.41796E+00_JPRB, 0.40686E+00_JPRB, 0.40564E+00_JPRB, 0.41039E+00_JPRB, 0.42180E+00_JPRB ,&
&  0.43007E+00_JPRB, 0.44233E+00_JPRB, 0.45716E+00_JPRB, 0.46669E+00_JPRB, 0.46676E+00_JPRB, 0.45470E+00_JPRB, 0.44362E+00_JPRB ,&
&  0.43711E+00_JPRB, 0.43533E+00_JPRB, 0.43384E+00_JPRB, 0.43177E+00_JPRB, 0.42979E+00_JPRB, 0.42919E+00_JPRB, 0.42804E+00_JPRB ,&
&  0.42374E+00_JPRB, 0.41639E+00_JPRB, 0.40744E+00_JPRB, 0.40245E+00_JPRB, 0.40141E+00_JPRB, 0.40134E+00_JPRB, 0.40298E+00_JPRB ,&
&  0.41198E+00_JPRB, 0.42321E+00_JPRB, 0.43231E+00_JPRB, 0.43646E+00_JPRB, 0.43354E+00_JPRB, 0.42568E+00_JPRB, 0.42089E+00_JPRB ,&
&  0.41717E+00_JPRB, 0.41452E+00_JPRB, 0.41108E+00_JPRB, 0.40940E+00_JPRB, 0.40904E+00_JPRB, 0.41615E+00_JPRB, 0.43168E+00_JPRB ,&
&  0.45108E+00_JPRB, 0.47658E+00_JPRB, 0.48824E+00_JPRB, 0.48693E+00_JPRB, 0.47998E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.52966E+00_JPRB, 0.51280E+00_JPRB, 0.51309E+00_JPRB, 0.52020E+00_JPRB, 0.52424E+00_JPRB, 0.53200E+00_JPRB, 0.55129E+00_JPRB ,&
&  0.56265E+00_JPRB, 0.56994E+00_JPRB, 0.57275E+00_JPRB, 0.56724E+00_JPRB, 0.56610E+00_JPRB, 0.55882E+00_JPRB, 0.54605E+00_JPRB ,&
&  0.54075E+00_JPRB, 0.53969E+00_JPRB, 0.54132E+00_JPRB, 0.54507E+00_JPRB, 0.54724E+00_JPRB, 0.54996E+00_JPRB, 0.55306E+00_JPRB ,&
&  0.55471E+00_JPRB, 0.55467E+00_JPRB, 0.55382E+00_JPRB, 0.55411E+00_JPRB, 0.55328E+00_JPRB, 0.54581E+00_JPRB, 0.53729E+00_JPRB ,&
&  0.52800E+00_JPRB, 0.51883E+00_JPRB, 0.51624E+00_JPRB, 0.52061E+00_JPRB, 0.52563E+00_JPRB, 0.52880E+00_JPRB, 0.52664E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.51938E+00_JPRB, 0.52170E+00_JPRB, 0.52867E+00_JPRB, 0.53728E+00_JPRB, 0.54688E+00_JPRB, 0.55190E+00_JPRB ,&
&  0.54625E+00_JPRB, 0.52863E+00_JPRB, 0.51791E+00_JPRB, 0.51740E+00_JPRB, 0.51392E+00_JPRB, 0.49963E+00_JPRB, 0.48957E+00_JPRB ,&
&  0.48480E+00_JPRB, 0.47602E+00_JPRB, 0.46673E+00_JPRB, 0.45403E+00_JPRB, 0.44238E+00_JPRB, 0.43804E+00_JPRB, 0.43667E+00_JPRB ,&
&  0.43953E+00_JPRB, 0.45705E+00_JPRB, 0.48962E+00_JPRB, 0.52167E+00_JPRB, 0.53592E+00_JPRB, 0.52970E+00_JPRB, 0.51319E+00_JPRB ,&
&  0.50575E+00_JPRB, 0.50269E+00_JPRB, 0.50411E+00_JPRB, 0.50375E+00_JPRB, 0.50024E+00_JPRB, 0.49145E+00_JPRB, 0.48431E+00_JPRB ,&
&  0.47410E+00_JPRB, 0.46288E+00_JPRB, 0.45834E+00_JPRB, 0.45559E+00_JPRB, 0.45263E+00_JPRB, 0.45269E+00_JPRB, 0.45173E+00_JPRB ,&
&  0.45214E+00_JPRB, 0.45595E+00_JPRB, 0.46753E+00_JPRB, 0.47412E+00_JPRB, 0.47529E+00_JPRB, 0.46511E+00_JPRB, 0.46604E+00_JPRB ,&
&  0.46863E+00_JPRB, 0.46632E+00_JPRB, 0.46175E+00_JPRB, 0.46151E+00_JPRB, 0.46139E+00_JPRB, 0.46323E+00_JPRB, 0.47470E+00_JPRB ,&
&  0.50129E+00_JPRB, 0.52473E+00_JPRB, 0.54417E+00_JPRB, 0.55023E+00_JPRB, 0.54664E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.60392E+00_JPRB, 0.57562E+00_JPRB, 0.56402E+00_JPRB, 0.56568E+00_JPRB, 0.56907E+00_JPRB, 0.56475E+00_JPRB, 0.57184E+00_JPRB ,&
&  0.58303E+00_JPRB, 0.59148E+00_JPRB, 0.59595E+00_JPRB, 0.59199E+00_JPRB, 0.58936E+00_JPRB, 0.58642E+00_JPRB, 0.57172E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.56513E+00_JPRB, 0.56919E+00_JPRB, 0.57613E+00_JPRB, 0.58369E+00_JPRB, 0.59572E+00_JPRB, 0.60462E+00_JPRB ,&
&  0.61048E+00_JPRB, 0.61414E+00_JPRB, 0.61829E+00_JPRB, 0.61852E+00_JPRB, 0.61505E+00_JPRB, 0.60611E+00_JPRB, 0.59923E+00_JPRB ,&
&  0.59518E+00_JPRB, 0.58371E+00_JPRB, 0.57352E+00_JPRB, 0.57223E+00_JPRB, 0.57541E+00_JPRB, 0.57867E+00_JPRB, 0.57841E+00_JPRB ,&
&  0.56564E+00_JPRB, 0.54902E+00_JPRB, 0.54727E+00_JPRB, 0.54716E+00_JPRB, 0.55232E+00_JPRB, 0.56438E+00_JPRB, 0.56773E+00_JPRB ,&
&  0.56448E+00_JPRB, 0.55035E+00_JPRB, 0.53118E+00_JPRB, 0.52751E+00_JPRB, 0.52989E+00_JPRB, 0.52355E+00_JPRB, 0.51721E+00_JPRB ,&
&  0.51604E+00_JPRB, 0.51297E+00_JPRB, 0.50698E+00_JPRB, 0.49884E+00_JPRB, 0.48617E+00_JPRB, 0.46961E+00_JPRB, 0.46098E+00_JPRB ,&
&  0.46006E+00_JPRB, 0.47446E+00_JPRB, 0.50071E+00_JPRB, 0.53065E+00_JPRB, 0.55935E+00_JPRB, 0.57346E+00_JPRB, 0.57304E+00_JPRB ,&
&  0.56145E+00_JPRB, 0.55893E+00_JPRB, 0.56422E+00_JPRB, 0.56854E+00_JPRB, 0.56935E+00_JPRB, 0.55857E+00_JPRB, 0.54160E+00_JPRB ,&
&  0.52942E+00_JPRB, 0.52024E+00_JPRB, 0.51913E+00_JPRB, 0.52407E+00_JPRB, 0.52905E+00_JPRB, 0.53028E+00_JPRB, 0.52410E+00_JPRB ,&
&  0.51748E+00_JPRB, 0.51066E+00_JPRB, 0.50684E+00_JPRB, 0.50980E+00_JPRB, 0.51533E+00_JPRB, 0.52083E+00_JPRB, 0.53354E+00_JPRB ,&
&  0.55174E+00_JPRB, 0.57009E+00_JPRB, 0.57443E+00_JPRB, 0.57054E+00_JPRB, 0.57031E+00_JPRB, 0.56769E+00_JPRB, 0.56519E+00_JPRB ,&
&  0.57508E+00_JPRB, 0.59743E+00_JPRB, 0.61379E+00_JPRB, 0.62341E+00_JPRB, 0.62414E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.65506E+00_JPRB, 0.63040E+00_JPRB, 0.61194E+00_JPRB, 0.60499E+00_JPRB, 0.60122E+00_JPRB, 0.58894E+00_JPRB, 0.58337E+00_JPRB ,&
&  0.58931E+00_JPRB, 0.59113E+00_JPRB, 0.59800E+00_JPRB, 0.59738E+00_JPRB, 0.59650E+00_JPRB, 0.59899E+00_JPRB, 0.59126E+00_JPRB ,&
&  0.59117E+00_JPRB, 0.60011E+00_JPRB, 0.61058E+00_JPRB, 0.62477E+00_JPRB, 0.63668E+00_JPRB, 0.64570E+00_JPRB, 0.65337E+00_JPRB ,&
&  0.65641E+00_JPRB, 0.66293E+00_JPRB, 0.66654E+00_JPRB, 0.66243E+00_JPRB, 0.65170E+00_JPRB, 0.64341E+00_JPRB, 0.64211E+00_JPRB ,&
&  0.64096E+00_JPRB, 0.63515E+00_JPRB, 0.59108E+00_JPRB, 0.61506E+00_JPRB, 0.62230E+00_JPRB, 0.62637E+00_JPRB, 0.62916E+00_JPRB ,&
&  0.62402E+00_JPRB, 0.60899E+00_JPRB, 0.60185E+00_JPRB, 0.58994E+00_JPRB, 0.58870E+00_JPRB, 0.59460E+00_JPRB, 0.59305E+00_JPRB ,&
&  0.58880E+00_JPRB, 0.57839E+00_JPRB, 0.56437E+00_JPRB, 0.55345E+00_JPRB, 0.55073E+00_JPRB, 0.54891E+00_JPRB, 0.54154E+00_JPRB ,&
&  0.53638E+00_JPRB, 0.53549E+00_JPRB, 0.53377E+00_JPRB, 0.52806E+00_JPRB, 0.51349E+00_JPRB, 0.49999E+00_JPRB, 0.49724E+00_JPRB ,&
&  0.50025E+00_JPRB, 0.50683E+00_JPRB, 0.52027E+00_JPRB, 0.53800E+00_JPRB, 0.55731E+00_JPRB, 0.57548E+00_JPRB, 0.59204E+00_JPRB ,&
&  0.59878E+00_JPRB, 0.60262E+00_JPRB, 0.60628E+00_JPRB, 0.61092E+00_JPRB, 0.61162E+00_JPRB, 0.60370E+00_JPRB, 0.58435E+00_JPRB ,&
&  0.56391E+00_JPRB, 0.55865E+00_JPRB, 0.56273E+00_JPRB, 0.57187E+00_JPRB, 0.57663E+00_JPRB, 0.57612E+00_JPRB, 0.56935E+00_JPRB ,&
&  0.56485E+00_JPRB, 0.55630E+00_JPRB, 0.54756E+00_JPRB, 0.54335E+00_JPRB, 0.54320E+00_JPRB, 0.54939E+00_JPRB, 0.56682E+00_JPRB ,&
&  0.58763E+00_JPRB, 0.61090E+00_JPRB, 0.63183E+00_JPRB, 0.64009E+00_JPRB, 0.64496E+00_JPRB, 0.64492E+00_JPRB, 0.63640E+00_JPRB ,&
&  0.64058E+00_JPRB, 0.65956E+00_JPRB, 0.66979E+00_JPRB, 0.67177E+00_JPRB, 0.66994E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.71319E+00_JPRB, 0.69252E+00_JPRB, 0.67952E+00_JPRB, 0.66725E+00_JPRB, 0.65834E+00_JPRB, 0.63982E+00_JPRB, 0.62927E+00_JPRB ,&
&  0.62254E+00_JPRB, 0.61825E+00_JPRB, 0.61705E+00_JPRB, 0.61687E+00_JPRB, 0.62676E+00_JPRB, 0.63301E+00_JPRB, 0.63518E+00_JPRB ,&
&  0.64246E+00_JPRB, 0.65967E+00_JPRB, 0.67550E+00_JPRB, 0.68885E+00_JPRB, 0.69946E+00_JPRB, 0.70726E+00_JPRB, 0.70728E+00_JPRB ,&
&  0.71115E+00_JPRB, 0.71927E+00_JPRB, 0.72329E+00_JPRB, 0.71358E+00_JPRB, 0.70130E+00_JPRB, 0.69497E+00_JPRB, 0.69545E+00_JPRB ,&
&  0.68538E+00_JPRB, 0.64009E+00_JPRB, 0.65465E+00_JPRB, 0.66675E+00_JPRB, 0.68123E+00_JPRB, 0.68919E+00_JPRB, 0.69768E+00_JPRB ,&
&  0.69934E+00_JPRB, 0.68259E+00_JPRB, 0.66143E+00_JPRB, 0.64614E+00_JPRB, 0.64919E+00_JPRB, 0.64642E+00_JPRB, 0.63711E+00_JPRB ,&
&  0.63301E+00_JPRB, 0.62953E+00_JPRB, 0.62199E+00_JPRB, 0.60898E+00_JPRB, 0.59734E+00_JPRB, 0.58957E+00_JPRB, 0.58453E+00_JPRB ,&
&  0.57360E+00_JPRB, 0.56454E+00_JPRB, 0.55968E+00_JPRB, 0.55425E+00_JPRB, 0.54962E+00_JPRB, 0.55363E+00_JPRB, 0.55690E+00_JPRB ,&
&  0.55591E+00_JPRB, 0.55642E+00_JPRB, 0.56403E+00_JPRB, 0.57808E+00_JPRB, 0.59010E+00_JPRB, 0.60392E+00_JPRB, 0.62019E+00_JPRB ,&
&  0.63625E+00_JPRB, 0.65033E+00_JPRB, 0.65499E+00_JPRB, 0.65885E+00_JPRB, 0.65360E+00_JPRB, 0.64514E+00_JPRB, 0.62798E+00_JPRB ,&
&  0.60929E+00_JPRB, 0.60656E+00_JPRB, 0.60836E+00_JPRB, 0.61503E+00_JPRB, 0.61422E+00_JPRB, 0.61176E+00_JPRB, 0.61003E+00_JPRB ,&
&  0.60591E+00_JPRB, 0.60239E+00_JPRB, 0.59750E+00_JPRB, 0.59287E+00_JPRB, 0.59602E+00_JPRB, 0.60187E+00_JPRB, 0.61534E+00_JPRB ,&
&  0.63338E+00_JPRB, 0.65271E+00_JPRB, 0.67287E+00_JPRB, 0.68827E+00_JPRB, 0.69521E+00_JPRB, 0.69984E+00_JPRB, 0.70428E+00_JPRB ,&
&  0.70597E+00_JPRB, 0.70584E+00_JPRB, 0.71348E+00_JPRB, 0.71874E+00_JPRB, 0.72351E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.75804E+00_JPRB, 0.73642E+00_JPRB, 0.72285E+00_JPRB, 0.71603E+00_JPRB, 0.70560E+00_JPRB, 0.68546E+00_JPRB, 0.66901E+00_JPRB ,&
&  0.65351E+00_JPRB, 0.64461E+00_JPRB, 0.64043E+00_JPRB, 0.63889E+00_JPRB, 0.64415E+00_JPRB, 0.64910E+00_JPRB, 0.65539E+00_JPRB ,&
&  0.66625E+00_JPRB, 0.68414E+00_JPRB, 0.69957E+00_JPRB, 0.71397E+00_JPRB, 0.72701E+00_JPRB, 0.73999E+00_JPRB, 0.74360E+00_JPRB ,&
&  0.74651E+00_JPRB, 0.75412E+00_JPRB, 0.75587E+00_JPRB, 0.74390E+00_JPRB, 0.73126E+00_JPRB, 0.73402E+00_JPRB, 0.73100E+00_JPRB ,&
&  0.72219E+00_JPRB, 0.60919E+00_JPRB, 0.66240E+00_JPRB, 0.68832E+00_JPRB, 0.71823E+00_JPRB, 0.74507E+00_JPRB, 0.76058E+00_JPRB ,&
&  0.75568E+00_JPRB, 0.73744E+00_JPRB, 0.71650E+00_JPRB, 0.68913E+00_JPRB, 0.68071E+00_JPRB, 0.67336E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.65783E+00_JPRB, 0.65165E+00_JPRB, 0.64342E+00_JPRB, 0.63223E+00_JPRB, 0.61786E+00_JPRB, 0.60721E+00_JPRB, 0.59787E+00_JPRB ,&
&  0.59557E+00_JPRB, 0.58839E+00_JPRB, 0.58175E+00_JPRB, 0.58251E+00_JPRB, 0.59414E+00_JPRB, 0.59931E+00_JPRB, 0.59926E+00_JPRB ,&
&  0.59911E+00_JPRB, 0.60009E+00_JPRB, 0.60786E+00_JPRB, 0.62117E+00_JPRB, 0.63238E+00_JPRB, 0.64016E+00_JPRB, 0.65146E+00_JPRB ,&
&  0.66987E+00_JPRB, 0.68345E+00_JPRB, 0.68413E+00_JPRB, 0.67373E+00_JPRB, 0.66749E+00_JPRB, 0.66773E+00_JPRB, 0.65862E+00_JPRB ,&
&  0.64353E+00_JPRB, 0.63486E+00_JPRB, 0.63403E+00_JPRB, 0.63459E+00_JPRB, 0.63200E+00_JPRB, 0.62670E+00_JPRB, 0.62617E+00_JPRB ,&
&  0.62761E+00_JPRB, 0.62829E+00_JPRB, 0.62756E+00_JPRB, 0.63019E+00_JPRB, 0.63624E+00_JPRB, 0.64445E+00_JPRB, 0.65586E+00_JPRB ,&
&  0.67252E+00_JPRB, 0.68842E+00_JPRB, 0.69772E+00_JPRB, 0.71358E+00_JPRB, 0.73208E+00_JPRB, 0.74145E+00_JPRB, 0.75841E+00_JPRB ,&
&  0.76267E+00_JPRB, 0.76302E+00_JPRB, 0.76364E+00_JPRB, 0.76992E+00_JPRB, 0.77180E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.80667E+00_JPRB, 0.78234E+00_JPRB, 0.76774E+00_JPRB, 0.75804E+00_JPRB, 0.75058E+00_JPRB, 0.74171E+00_JPRB, 0.72596E+00_JPRB ,&
&  0.70937E+00_JPRB, 0.70032E+00_JPRB, 0.69262E+00_JPRB, 0.68325E+00_JPRB, 0.68360E+00_JPRB, 0.68880E+00_JPRB, 0.69944E+00_JPRB ,&
&  0.71140E+00_JPRB, 0.72692E+00_JPRB, 0.74194E+00_JPRB, 0.76100E+00_JPRB, 0.77367E+00_JPRB, 0.78255E+00_JPRB, 0.78737E+00_JPRB ,&
&  0.79070E+00_JPRB, 0.79125E+00_JPRB, 0.78407E+00_JPRB, 0.77480E+00_JPRB, 0.77245E+00_JPRB, 0.77358E+00_JPRB, 0.76852E+00_JPRB ,&
&  0.67049E+00_JPRB, 0.58533E+00_JPRB, 0.63944E+00_JPRB, 0.70420E+00_JPRB, 0.76997E+00_JPRB, 0.82335E+00_JPRB, 0.84741E+00_JPRB ,&
&  0.83348E+00_JPRB, 0.80139E+00_JPRB, 0.77282E+00_JPRB, 0.74045E+00_JPRB, 0.72696E+00_JPRB, 0.72064E+00_JPRB, 0.71077E+00_JPRB ,&
&  0.69996E+00_JPRB, 0.68764E+00_JPRB, 0.67267E+00_JPRB, 0.64978E+00_JPRB, 0.63371E+00_JPRB, 0.62208E+00_JPRB, 0.62027E+00_JPRB ,&
&  0.62257E+00_JPRB, 0.62636E+00_JPRB, 0.62966E+00_JPRB, 0.63213E+00_JPRB, 0.63890E+00_JPRB, 0.64845E+00_JPRB, 0.65942E+00_JPRB ,&
&  0.67366E+00_JPRB, 0.68582E+00_JPRB, 0.70379E+00_JPRB, 0.71586E+00_JPRB, 0.72601E+00_JPRB, 0.73446E+00_JPRB, 0.74368E+00_JPRB ,&
&  0.74582E+00_JPRB, 0.74184E+00_JPRB, 0.72506E+00_JPRB, 0.71648E+00_JPRB, 0.71665E+00_JPRB, 0.71725E+00_JPRB, 0.71253E+00_JPRB ,&
&  0.70101E+00_JPRB, 0.68699E+00_JPRB, 0.67288E+00_JPRB, 0.66588E+00_JPRB, 0.66710E+00_JPRB, 0.66659E+00_JPRB, 0.66252E+00_JPRB ,&
&  0.65914E+00_JPRB, 0.65932E+00_JPRB, 0.66522E+00_JPRB, 0.67656E+00_JPRB, 0.69180E+00_JPRB, 0.71081E+00_JPRB, 0.72689E+00_JPRB ,&
&  0.74384E+00_JPRB, 0.76108E+00_JPRB, 0.77318E+00_JPRB, 0.80065E+00_JPRB, 0.82629E+00_JPRB, 0.84119E+00_JPRB, 0.84456E+00_JPRB ,&
&  0.85169E+00_JPRB, 0.84683E+00_JPRB, 0.84098E+00_JPRB, 0.83770E+00_JPRB, 0.82637E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.80181E+00_JPRB, 0.80229E+00_JPRB, 0.80275E+00_JPRB, 0.79514E+00_JPRB, 0.78793E+00_JPRB, 0.77589E+00_JPRB, 0.75459E+00_JPRB ,&
&  0.73509E+00_JPRB, 0.72648E+00_JPRB, 0.72455E+00_JPRB, 0.72453E+00_JPRB, 0.72635E+00_JPRB, 0.73193E+00_JPRB, 0.74311E+00_JPRB ,&
&  0.75677E+00_JPRB, 0.77137E+00_JPRB, 0.78852E+00_JPRB, 0.80436E+00_JPRB, 0.80804E+00_JPRB, 0.80503E+00_JPRB, 0.80545E+00_JPRB ,&
&  0.81173E+00_JPRB, 0.81081E+00_JPRB, 0.80449E+00_JPRB, 0.80715E+00_JPRB, 0.81543E+00_JPRB, 0.80660E+00_JPRB, 0.78834E+00_JPRB ,&
&  0.77668E+00_JPRB, 0.63421E+00_JPRB, 0.67289E+00_JPRB, 0.74442E+00_JPRB, 0.82509E+00_JPRB, 0.90060E+00_JPRB, 0.93039E+00_JPRB ,&
&  0.91718E+00_JPRB, 0.87871E+00_JPRB, 0.84709E+00_JPRB, 0.80927E+00_JPRB, 0.79041E+00_JPRB, 0.77609E+00_JPRB, 0.75902E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.71139E+00_JPRB, 0.68550E+00_JPRB, 0.66315E+00_JPRB, 0.64537E+00_JPRB, 0.63733E+00_JPRB, 0.63513E+00_JPRB ,&
&  0.63559E+00_JPRB, 0.64641E+00_JPRB, 0.65233E+00_JPRB, 0.66203E+00_JPRB, 0.67783E+00_JPRB, 0.70025E+00_JPRB, 0.72458E+00_JPRB ,&
&  0.74825E+00_JPRB, 0.76780E+00_JPRB, 0.78781E+00_JPRB, 0.80360E+00_JPRB, 0.81993E+00_JPRB, 0.83294E+00_JPRB, 0.82828E+00_JPRB ,&
&  0.80374E+00_JPRB, 0.77513E+00_JPRB, 0.77221E+00_JPRB, 0.77589E+00_JPRB, 0.77175E+00_JPRB, 0.76928E+00_JPRB, 0.76468E+00_JPRB ,&
&  0.75317E+00_JPRB, 0.73875E+00_JPRB, 0.72921E+00_JPRB, 0.71813E+00_JPRB, 0.71143E+00_JPRB, 0.70991E+00_JPRB, 0.69774E+00_JPRB ,&
&  0.68971E+00_JPRB, 0.68987E+00_JPRB, 0.69885E+00_JPRB, 0.71954E+00_JPRB, 0.74748E+00_JPRB, 0.77461E+00_JPRB, 0.79326E+00_JPRB ,&
&  0.80469E+00_JPRB, 0.81631E+00_JPRB, 0.82301E+00_JPRB, 0.86537E+00_JPRB, 0.88634E+00_JPRB, 0.90221E+00_JPRB, 0.91464E+00_JPRB ,&
&  0.92153E+00_JPRB, 0.92472E+00_JPRB, 0.70228E+00_JPRB, 0.77432E+00_JPRB, 0.78751E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.84088E+00_JPRB, 0.85907E+00_JPRB, 0.86120E+00_JPRB, 0.85596E+00_JPRB, 0.84687E+00_JPRB, 0.82635E+00_JPRB, 0.80326E+00_JPRB ,&
&  0.77938E+00_JPRB, 0.76822E+00_JPRB, 0.76316E+00_JPRB, 0.76622E+00_JPRB, 0.77157E+00_JPRB, 0.78027E+00_JPRB, 0.79291E+00_JPRB ,&
&  0.80493E+00_JPRB, 0.81099E+00_JPRB, 0.81588E+00_JPRB, 0.81423E+00_JPRB, 0.81261E+00_JPRB, 0.81501E+00_JPRB, 0.81942E+00_JPRB ,&
&  0.82846E+00_JPRB, 0.83179E+00_JPRB, 0.82906E+00_JPRB, 0.83790E+00_JPRB, 0.85082E+00_JPRB, 0.84041E+00_JPRB, 0.82093E+00_JPRB ,&
&  0.80809E+00_JPRB, 0.68353E+00_JPRB, 0.71928E+00_JPRB, 0.84487E+00_JPRB, 0.94653E+00_JPRB, 0.10424E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.10350E+01_JPRB, 0.98408E+00_JPRB, 0.93403E+00_JPRB, 0.88758E+00_JPRB, 0.85471E+00_JPRB, 0.82072E+00_JPRB, 0.79753E+00_JPRB ,&
&  0.78202E+00_JPRB, 0.76581E+00_JPRB, 0.74261E+00_JPRB, 0.72404E+00_JPRB, 0.70913E+00_JPRB, 0.69453E+00_JPRB, 0.68653E+00_JPRB ,&
&  0.68778E+00_JPRB, 0.69871E+00_JPRB, 0.71154E+00_JPRB, 0.72786E+00_JPRB, 0.75810E+00_JPRB, 0.78916E+00_JPRB, 0.82220E+00_JPRB ,&
&  0.84358E+00_JPRB, 0.86580E+00_JPRB, 0.89515E+00_JPRB, 0.92378E+00_JPRB, 0.95394E+00_JPRB, 0.96333E+00_JPRB, 0.93824E+00_JPRB ,&
&  0.88403E+00_JPRB, 0.84629E+00_JPRB, 0.82061E+00_JPRB, 0.80398E+00_JPRB, 0.80337E+00_JPRB, 0.81387E+00_JPRB, 0.81962E+00_JPRB ,&
&  0.81437E+00_JPRB, 0.80309E+00_JPRB, 0.78789E+00_JPRB, 0.77352E+00_JPRB, 0.77005E+00_JPRB, 0.77016E+00_JPRB, 0.76525E+00_JPRB ,&
&  0.76022E+00_JPRB, 0.76285E+00_JPRB, 0.77369E+00_JPRB, 0.79549E+00_JPRB, 0.82292E+00_JPRB, 0.85399E+00_JPRB, 0.87370E+00_JPRB ,&
&  0.88691E+00_JPRB, 0.89989E+00_JPRB, 0.91451E+00_JPRB, 0.92502E+00_JPRB, 0.98251E+00_JPRB, 0.10063E+01_JPRB, 0.10160E+01_JPRB ,&
&  0.99943E+00_JPRB, 0.96793E+00_JPRB, 0.92775E+00_JPRB, 0.75858E+00_JPRB, 0.81564E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.90458E+00_JPRB, 0.92536E+00_JPRB, 0.92516E+00_JPRB, 0.91664E+00_JPRB, 0.89508E+00_JPRB, 0.86633E+00_JPRB, 0.84458E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.81478E+00_JPRB, 0.80626E+00_JPRB, 0.80328E+00_JPRB, 0.80698E+00_JPRB, 0.81456E+00_JPRB, 0.81529E+00_JPRB ,&
&  0.80652E+00_JPRB, 0.79270E+00_JPRB, 0.78300E+00_JPRB, 0.78209E+00_JPRB, 0.79156E+00_JPRB, 0.80197E+00_JPRB, 0.80890E+00_JPRB ,&
&  0.81194E+00_JPRB, 0.80586E+00_JPRB, 0.80337E+00_JPRB, 0.81346E+00_JPRB, 0.82145E+00_JPRB, 0.81368E+00_JPRB, 0.79348E+00_JPRB ,&
&  0.79051E+00_JPRB, 0.75285E+00_JPRB, 0.88876E+00_JPRB, 0.10264E+01_JPRB, 0.11379E+01_JPRB, 0.11785E+01_JPRB, 0.11603E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.10443E+01_JPRB, 0.97313E+00_JPRB, 0.91743E+00_JPRB, 0.87723E+00_JPRB, 0.84892E+00_JPRB, 0.83014E+00_JPRB ,&
&  0.80369E+00_JPRB, 0.77855E+00_JPRB, 0.75628E+00_JPRB, 0.74363E+00_JPRB, 0.73915E+00_JPRB, 0.73545E+00_JPRB, 0.73197E+00_JPRB ,&
&  0.72803E+00_JPRB, 0.72490E+00_JPRB, 0.72997E+00_JPRB, 0.74793E+00_JPRB, 0.77389E+00_JPRB, 0.82158E+00_JPRB, 0.88435E+00_JPRB ,&
&  0.92217E+00_JPRB, 0.97069E+00_JPRB, 0.10275E+01_JPRB, 0.10807E+01_JPRB, 0.10921E+01_JPRB, 0.10765E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.99433E+00_JPRB, 0.92959E+00_JPRB, 0.87470E+00_JPRB, 0.83897E+00_JPRB, 0.82547E+00_JPRB, 0.82155E+00_JPRB, 0.83384E+00_JPRB ,&
&  0.84209E+00_JPRB, 0.84591E+00_JPRB, 0.83417E+00_JPRB, 0.82100E+00_JPRB, 0.81526E+00_JPRB, 0.80046E+00_JPRB, 0.79747E+00_JPRB ,&
&  0.80062E+00_JPRB, 0.81535E+00_JPRB, 0.84097E+00_JPRB, 0.86575E+00_JPRB, 0.89841E+00_JPRB, 0.92282E+00_JPRB, 0.93294E+00_JPRB ,&
&  0.93661E+00_JPRB, 0.93938E+00_JPRB, 0.94073E+00_JPRB, 0.10225E+01_JPRB, 0.10519E+01_JPRB, 0.10704E+01_JPRB, 0.10616E+01_JPRB ,&
&  0.10401E+01_JPRB, 0.99135E+00_JPRB, 0.93221E+00_JPRB, 0.90793E+00_JPRB, 0.84429E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.10536E+01_JPRB, 0.10348E+01_JPRB, 0.10006E+01_JPRB, 0.97041E+00_JPRB, 0.93702E+00_JPRB, 0.91295E+00_JPRB, 0.89337E+00_JPRB ,&
&  0.88174E+00_JPRB, 0.86205E+00_JPRB, 0.85157E+00_JPRB, 0.84299E+00_JPRB, 0.83922E+00_JPRB, 0.83041E+00_JPRB, 0.81709E+00_JPRB ,&
&  0.80242E+00_JPRB, 0.79315E+00_JPRB, 0.78967E+00_JPRB, 0.79326E+00_JPRB, 0.80253E+00_JPRB, 0.80835E+00_JPRB, 0.80284E+00_JPRB ,&
&  0.79405E+00_JPRB, 0.78031E+00_JPRB, 0.76425E+00_JPRB, 0.77891E+00_JPRB, 0.79485E+00_JPRB, 0.79940E+00_JPRB, 0.78459E+00_JPRB ,&
&  0.80078E+00_JPRB, 0.88076E+00_JPRB, 0.12198E+01_JPRB, 0.13631E+01_JPRB, 0.13867E+01_JPRB, 0.14033E+01_JPRB, 0.12868E+01_JPRB ,&
&  0.11866E+01_JPRB, 0.10712E+01_JPRB, 0.98155E+00_JPRB, 0.93800E+00_JPRB, 0.91193E+00_JPRB, 0.87357E+00_JPRB, 0.84171E+00_JPRB ,&
&  0.79897E+00_JPRB, 0.77798E+00_JPRB, 0.76426E+00_JPRB, 0.75882E+00_JPRB, 0.75609E+00_JPRB, 0.75685E+00_JPRB, 0.76194E+00_JPRB ,&
&  0.76743E+00_JPRB, 0.77064E+00_JPRB, 0.78001E+00_JPRB, 0.80705E+00_JPRB, 0.83940E+00_JPRB, 0.92988E+00_JPRB, 0.10395E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.11665E+01_JPRB, 0.12224E+01_JPRB, 0.12347E+01_JPRB, 0.11922E+01_JPRB, 0.11315E+01_JPRB, 0.10858E+01_JPRB ,&
&  0.10432E+01_JPRB, 0.99509E+00_JPRB, 0.94719E+00_JPRB, 0.91238E+00_JPRB, 0.88016E+00_JPRB, 0.86782E+00_JPRB, 0.88479E+00_JPRB ,&
&  0.89372E+00_JPRB, 0.89928E+00_JPRB, 0.88710E+00_JPRB, 0.86918E+00_JPRB, 0.83987E+00_JPRB, 0.81953E+00_JPRB, 0.81850E+00_JPRB ,&
&  0.83000E+00_JPRB, 0.85447E+00_JPRB, 0.89051E+00_JPRB, 0.92943E+00_JPRB, 0.96258E+00_JPRB, 0.98148E+00_JPRB, 0.98741E+00_JPRB ,&
&  0.99523E+00_JPRB, 0.10102E+01_JPRB, 0.96881E+00_JPRB, 0.10763E+01_JPRB, 0.12147E+01_JPRB, 0.12019E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.10348E+01_JPRB, 0.97446E+00_JPRB, 0.95157E+00_JPRB, 0.10143E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.10091E+01_JPRB, 0.99047E+00_JPRB, 0.96014E+00_JPRB, 0.93719E+00_JPRB, 0.92861E+00_JPRB, 0.91327E+00_JPRB, 0.90095E+00_JPRB ,&
&  0.89321E+00_JPRB, 0.87928E+00_JPRB, 0.86746E+00_JPRB, 0.85122E+00_JPRB, 0.83154E+00_JPRB, 0.81567E+00_JPRB, 0.81006E+00_JPRB ,&
&  0.80939E+00_JPRB, 0.79919E+00_JPRB, 0.80273E+00_JPRB, 0.80407E+00_JPRB, 0.80040E+00_JPRB, 0.78707E+00_JPRB, 0.77378E+00_JPRB ,&
&  0.76697E+00_JPRB, 0.75465E+00_JPRB, 0.75309E+00_JPRB, 0.77437E+00_JPRB, 0.79262E+00_JPRB, 0.79457E+00_JPRB, 0.77865E+00_JPRB ,&
&  0.82652E+00_JPRB, 0.10749E+01_JPRB, 0.16347E+01_JPRB, 0.18328E+01_JPRB, 0.17571E+01_JPRB, 0.15460E+01_JPRB, 0.13363E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.10691E+01_JPRB, 0.10087E+01_JPRB, 0.95836E+00_JPRB, 0.90685E+00_JPRB, 0.86521E+00_JPRB, 0.83173E+00_JPRB ,&
&  0.79494E+00_JPRB, 0.77985E+00_JPRB, 0.77248E+00_JPRB, 0.77205E+00_JPRB, 0.77126E+00_JPRB, 0.76944E+00_JPRB, 0.77213E+00_JPRB ,&
&  0.77862E+00_JPRB, 0.78864E+00_JPRB, 0.81120E+00_JPRB, 0.83860E+00_JPRB, 0.92973E+00_JPRB, 0.11045E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.14223E+01_JPRB, 0.13578E+01_JPRB, 0.12776E+01_JPRB, 0.12114E+01_JPRB, 0.11523E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.10645E+01_JPRB, 0.10276E+01_JPRB, 0.10004E+01_JPRB, 0.97500E+00_JPRB, 0.94319E+00_JPRB, 0.93690E+00_JPRB, 0.94870E+00_JPRB ,&
&  0.96435E+00_JPRB, 0.96405E+00_JPRB, 0.93790E+00_JPRB, 0.90861E+00_JPRB, 0.87405E+00_JPRB, 0.86406E+00_JPRB, 0.86355E+00_JPRB ,&
&  0.86781E+00_JPRB, 0.88720E+00_JPRB, 0.93115E+00_JPRB, 0.97420E+00_JPRB, 0.10058E+01_JPRB, 0.10210E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.10737E+01_JPRB, 0.10538E+01_JPRB, 0.11273E+01_JPRB, 0.12036E+01_JPRB, 0.12702E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.99501E+00_JPRB, 0.95516E+00_JPRB, 0.94356E+00_JPRB, 0.10001E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.96922E+00_JPRB, 0.94554E+00_JPRB, 0.92525E+00_JPRB, 0.90675E+00_JPRB, 0.88836E+00_JPRB, 0.87777E+00_JPRB, 0.86577E+00_JPRB ,&
&  0.86662E+00_JPRB, 0.86556E+00_JPRB, 0.85294E+00_JPRB, 0.82382E+00_JPRB, 0.80013E+00_JPRB, 0.79558E+00_JPRB, 0.80068E+00_JPRB ,&
&  0.79460E+00_JPRB, 0.78337E+00_JPRB, 0.78635E+00_JPRB, 0.76923E+00_JPRB, 0.74709E+00_JPRB, 0.73294E+00_JPRB, 0.72708E+00_JPRB ,&
&  0.73142E+00_JPRB, 0.74296E+00_JPRB, 0.77084E+00_JPRB, 0.79680E+00_JPRB, 0.79954E+00_JPRB, 0.79111E+00_JPRB, 0.77784E+00_JPRB ,&
&  0.90383E+00_JPRB, 0.12267E+01_JPRB, 0.21264E+01_JPRB, 0.21001E+01_JPRB, 0.18244E+01_JPRB, 0.15229E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.12055E+01_JPRB, 0.11074E+01_JPRB, 0.10209E+01_JPRB, 0.93261E+00_JPRB, 0.86735E+00_JPRB, 0.83179E+00_JPRB, 0.81410E+00_JPRB ,&
&  0.79180E+00_JPRB, 0.77865E+00_JPRB, 0.78199E+00_JPRB, 0.78888E+00_JPRB, 0.79256E+00_JPRB, 0.78958E+00_JPRB, 0.77945E+00_JPRB ,&
&  0.78550E+00_JPRB, 0.80301E+00_JPRB, 0.82984E+00_JPRB, 0.87324E+00_JPRB, 0.10148E+01_JPRB, 0.10929E+01_JPRB, 0.14326E+01_JPRB ,&
&  0.18421E+01_JPRB, 0.16757E+01_JPRB, 0.14682E+01_JPRB, 0.13428E+01_JPRB, 0.12702E+01_JPRB, 0.12035E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.10801E+01_JPRB, 0.10746E+01_JPRB, 0.10610E+01_JPRB, 0.10545E+01_JPRB, 0.10216E+01_JPRB, 0.99310E+00_JPRB ,&
&  0.98935E+00_JPRB, 0.96054E+00_JPRB, 0.92961E+00_JPRB, 0.89766E+00_JPRB, 0.89925E+00_JPRB, 0.91169E+00_JPRB, 0.91671E+00_JPRB ,&
&  0.92546E+00_JPRB, 0.93301E+00_JPRB, 0.98200E+00_JPRB, 0.98252E+00_JPRB, 0.10282E+01_JPRB, 0.10695E+01_JPRB, 0.10681E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10554E+01_JPRB, 0.10720E+01_JPRB, 0.10830E+01_JPRB, 0.11565E+01_JPRB, 0.12365E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.98269E+00_JPRB, 0.92699E+00_JPRB, 0.89709E+00_JPRB, 0.90755E+00_JPRB, 0.97367E+00_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.96930E+00_JPRB, 0.94484E+00_JPRB, 0.91837E+00_JPRB, 0.88142E+00_JPRB, 0.87071E+00_JPRB, 0.85716E+00_JPRB, 0.84952E+00_JPRB ,&
&  0.84681E+00_JPRB, 0.83258E+00_JPRB, 0.81687E+00_JPRB, 0.79493E+00_JPRB, 0.78606E+00_JPRB, 0.79661E+00_JPRB, 0.79126E+00_JPRB ,&
&  0.77332E+00_JPRB, 0.74952E+00_JPRB, 0.74145E+00_JPRB, 0.73232E+00_JPRB, 0.72258E+00_JPRB, 0.72293E+00_JPRB, 0.73220E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.75539E+00_JPRB, 0.78626E+00_JPRB, 0.81427E+00_JPRB, 0.81308E+00_JPRB, 0.80675E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.97927E+00_JPRB, 0.96896E+00_JPRB, 0.19793E+01_JPRB, 0.22899E+01_JPRB, 0.18973E+01_JPRB, 0.16061E+01_JPRB, 0.13358E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.11017E+01_JPRB, 0.10074E+01_JPRB, 0.92660E+00_JPRB, 0.88182E+00_JPRB, 0.82875E+00_JPRB, 0.80783E+00_JPRB ,&
&  0.80331E+00_JPRB, 0.79186E+00_JPRB, 0.79274E+00_JPRB, 0.81049E+00_JPRB, 0.81618E+00_JPRB, 0.82203E+00_JPRB, 0.81341E+00_JPRB ,&
&  0.81971E+00_JPRB, 0.83765E+00_JPRB, 0.88479E+00_JPRB, 0.99900E+00_JPRB, 0.11876E+01_JPRB, 0.13443E+01_JPRB, 0.15611E+01_JPRB ,&
&  0.16627E+01_JPRB, 0.17450E+01_JPRB, 0.14749E+01_JPRB, 0.13404E+01_JPRB, 0.12580E+01_JPRB, 0.11828E+01_JPRB, 0.11561E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.10955E+01_JPRB, 0.10823E+01_JPRB, 0.11106E+01_JPRB, 0.10748E+01_JPRB, 0.10549E+01_JPRB, 0.10216E+01_JPRB ,&
&  0.98606E+00_JPRB, 0.95367E+00_JPRB, 0.92331E+00_JPRB, 0.90842E+00_JPRB, 0.92786E+00_JPRB, 0.93837E+00_JPRB, 0.93422E+00_JPRB ,&
&  0.93501E+00_JPRB, 0.96382E+00_JPRB, 0.10052E+01_JPRB, 0.10127E+01_JPRB, 0.10460E+01_JPRB, 0.10564E+01_JPRB, 0.10631E+01_JPRB ,&
&  0.10719E+01_JPRB, 0.10851E+01_JPRB, 0.10398E+01_JPRB, 0.10602E+01_JPRB, 0.10395E+01_JPRB, 0.10897E+01_JPRB, 0.97812E+00_JPRB ,&
&  0.91314E+00_JPRB, 0.87396E+00_JPRB, 0.88599E+00_JPRB, 0.94022E+00_JPRB, 0.97811E+00_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.94804E+00_JPRB, 0.94160E+00_JPRB, 0.89134E+00_JPRB, 0.88228E+00_JPRB, 0.86417E+00_JPRB, 0.86281E+00_JPRB, 0.87530E+00_JPRB ,&
&  0.86715E+00_JPRB, 0.84132E+00_JPRB, 0.80852E+00_JPRB, 0.79334E+00_JPRB, 0.80389E+00_JPRB, 0.81609E+00_JPRB, 0.78384E+00_JPRB ,&
&  0.74602E+00_JPRB, 0.73325E+00_JPRB, 0.72662E+00_JPRB, 0.72994E+00_JPRB, 0.74067E+00_JPRB, 0.75721E+00_JPRB, 0.76633E+00_JPRB ,&
&  0.76171E+00_JPRB, 0.80028E+00_JPRB, 0.84395E+00_JPRB, 0.86026E+00_JPRB, 0.84591E+00_JPRB, 0.83864E+00_JPRB, 0.86371E+00_JPRB ,&
&  0.96976E+00_JPRB, 0.22209E+01_JPRB, 0.14835E+01_JPRB, 0.23806E+01_JPRB, 0.18571E+01_JPRB, 0.15443E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.11240E+01_JPRB, 0.11263E+01_JPRB, 0.10514E+01_JPRB, 0.96360E+00_JPRB, 0.90078E+00_JPRB, 0.84755E+00_JPRB, 0.80120E+00_JPRB ,&
&  0.78015E+00_JPRB, 0.78327E+00_JPRB, 0.79212E+00_JPRB, 0.83715E+00_JPRB, 0.87263E+00_JPRB, 0.88453E+00_JPRB, 0.88510E+00_JPRB ,&
&  0.89827E+00_JPRB, 0.92448E+00_JPRB, 0.10137E+01_JPRB, 0.12726E+01_JPRB, 0.15226E+01_JPRB, 0.15630E+01_JPRB, 0.16539E+01_JPRB ,&
&  0.15209E+01_JPRB, 0.16512E+01_JPRB, 0.14332E+01_JPRB, 0.13066E+01_JPRB, 0.12148E+01_JPRB, 0.11461E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.10964E+01_JPRB, 0.10628E+01_JPRB, 0.10506E+01_JPRB, 0.10366E+01_JPRB, 0.10402E+01_JPRB, 0.10450E+01_JPRB, 0.10084E+01_JPRB ,&
&  0.96641E+00_JPRB, 0.93210E+00_JPRB, 0.93591E+00_JPRB, 0.94935E+00_JPRB, 0.96789E+00_JPRB, 0.95732E+00_JPRB, 0.94848E+00_JPRB ,&
&  0.96266E+00_JPRB, 0.10185E+01_JPRB, 0.10420E+01_JPRB, 0.10614E+01_JPRB, 0.10583E+01_JPRB, 0.10375E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.10636E+01_JPRB, 0.99952E+00_JPRB, 0.99370E+00_JPRB, 0.98914E+00_JPRB, 0.10309E+01_JPRB, 0.90369E+00_JPRB ,&
&  0.87410E+00_JPRB, 0.86910E+00_JPRB, 0.93563E+00_JPRB, 0.94622E+00_JPRB, 0.95943E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.92942E+00_JPRB, 0.91136E+00_JPRB, 0.87315E+00_JPRB, 0.87084E+00_JPRB, 0.84781E+00_JPRB, 0.87544E+00_JPRB, 0.87967E+00_JPRB ,&
&  0.84496E+00_JPRB, 0.81131E+00_JPRB, 0.78964E+00_JPRB, 0.80809E+00_JPRB, 0.82872E+00_JPRB, 0.81526E+00_JPRB, 0.77603E+00_JPRB ,&
&  0.76316E+00_JPRB, 0.76104E+00_JPRB, 0.76126E+00_JPRB, 0.76531E+00_JPRB, 0.78105E+00_JPRB, 0.80370E+00_JPRB, 0.80112E+00_JPRB ,&
&  0.79778E+00_JPRB, 0.82909E+00_JPRB, 0.85845E+00_JPRB, 0.86044E+00_JPRB, 0.85384E+00_JPRB, 0.84690E+00_JPRB, 0.89599E+00_JPRB ,&
&  0.96707E+00_JPRB, 0.18668E+01_JPRB, 0.13907E+01_JPRB, 0.22626E+01_JPRB, 0.17886E+01_JPRB, 0.15216E+01_JPRB, 0.11985E+01_JPRB ,&
&  0.10878E+01_JPRB, 0.11816E+01_JPRB, 0.10709E+01_JPRB, 0.96016E+00_JPRB, 0.87364E+00_JPRB, 0.83118E+00_JPRB, 0.79624E+00_JPRB ,&
&  0.77052E+00_JPRB, 0.80353E+00_JPRB, 0.85123E+00_JPRB, 0.91803E+00_JPRB, 0.98667E+00_JPRB, 0.10206E+01_JPRB, 0.10520E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.11249E+01_JPRB, 0.13374E+01_JPRB, 0.16103E+01_JPRB, 0.19088E+01_JPRB, 0.18591E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.14742E+01_JPRB, 0.15644E+01_JPRB, 0.14087E+01_JPRB, 0.12860E+01_JPRB, 0.11790E+01_JPRB, 0.11380E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.10370E+01_JPRB, 0.97018E+00_JPRB, 0.95745E+00_JPRB, 0.96285E+00_JPRB, 0.10138E+01_JPRB, 0.10213E+01_JPRB, 0.97485E+00_JPRB ,&
&  0.93127E+00_JPRB, 0.91814E+00_JPRB, 0.92402E+00_JPRB, 0.94281E+00_JPRB, 0.96398E+00_JPRB, 0.96574E+00_JPRB, 0.96970E+00_JPRB ,&
&  0.10112E+01_JPRB, 0.10823E+01_JPRB, 0.10885E+01_JPRB, 0.10935E+01_JPRB, 0.10495E+01_JPRB, 0.10167E+01_JPRB, 0.99207E+00_JPRB ,&
&  0.10187E+01_JPRB, 0.10067E+01_JPRB, 0.96678E+00_JPRB, 0.92992E+00_JPRB, 0.92222E+00_JPRB, 0.92381E+00_JPRB, 0.87189E+00_JPRB ,&
&  0.87342E+00_JPRB, 0.91432E+00_JPRB, 0.96642E+00_JPRB, 0.93333E+00_JPRB, 0.92867E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.88247E+00_JPRB, 0.84610E+00_JPRB, 0.82954E+00_JPRB, 0.81375E+00_JPRB, 0.79891E+00_JPRB, 0.83022E+00_JPRB, 0.82028E+00_JPRB ,&
&  0.79540E+00_JPRB, 0.76683E+00_JPRB, 0.78389E+00_JPRB, 0.82864E+00_JPRB, 0.82932E+00_JPRB, 0.80848E+00_JPRB, 0.79423E+00_JPRB ,&
&  0.79346E+00_JPRB, 0.81114E+00_JPRB, 0.81493E+00_JPRB, 0.82695E+00_JPRB, 0.82731E+00_JPRB, 0.81659E+00_JPRB, 0.80976E+00_JPRB ,&
&  0.82590E+00_JPRB, 0.83776E+00_JPRB, 0.85214E+00_JPRB, 0.86420E+00_JPRB, 0.88663E+00_JPRB, 0.90257E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.11842E+01_JPRB, 0.17890E+01_JPRB, 0.14629E+01_JPRB, 0.17522E+01_JPRB, 0.16121E+01_JPRB, 0.13995E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.10272E+01_JPRB, 0.11097E+01_JPRB, 0.98212E+00_JPRB, 0.91964E+00_JPRB, 0.93338E+00_JPRB, 0.93319E+00_JPRB ,&
&  0.94908E+00_JPRB, 0.97047E+00_JPRB, 0.10992E+01_JPRB, 0.11834E+01_JPRB, 0.12455E+01_JPRB, 0.12877E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.15145E+01_JPRB, 0.18943E+01_JPRB, 0.21827E+01_JPRB, 0.21280E+01_JPRB, 0.19305E+01_JPRB, 0.16458E+01_JPRB ,&
&  0.15194E+01_JPRB, 0.14799E+01_JPRB, 0.13906E+01_JPRB, 0.12821E+01_JPRB, 0.11204E+01_JPRB, 0.11269E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.94777E+00_JPRB, 0.89129E+00_JPRB, 0.89029E+00_JPRB, 0.91869E+00_JPRB, 0.96024E+00_JPRB, 0.95749E+00_JPRB, 0.93374E+00_JPRB ,&
&  0.92412E+00_JPRB, 0.93422E+00_JPRB, 0.92423E+00_JPRB, 0.94472E+00_JPRB, 0.96457E+00_JPRB, 0.10031E+01_JPRB, 0.10401E+01_JPRB ,&
&  0.10931E+01_JPRB, 0.11439E+01_JPRB, 0.11228E+01_JPRB, 0.10900E+01_JPRB, 0.10357E+01_JPRB, 0.99821E+00_JPRB, 0.94946E+00_JPRB ,&
&  0.95320E+00_JPRB, 0.98168E+00_JPRB, 0.96442E+00_JPRB, 0.92941E+00_JPRB, 0.90277E+00_JPRB, 0.88594E+00_JPRB, 0.89293E+00_JPRB ,&
&  0.91776E+00_JPRB, 0.97330E+00_JPRB, 0.97066E+00_JPRB, 0.92001E+00_JPRB, 0.90713E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.80137E+00_JPRB, 0.78906E+00_JPRB, 0.76405E+00_JPRB, 0.74737E+00_JPRB, 0.77038E+00_JPRB, 0.80432E+00_JPRB, 0.80312E+00_JPRB ,&
&  0.77813E+00_JPRB, 0.76420E+00_JPRB, 0.79936E+00_JPRB, 0.83468E+00_JPRB, 0.83128E+00_JPRB, 0.84733E+00_JPRB, 0.85616E+00_JPRB ,&
&  0.88800E+00_JPRB, 0.88196E+00_JPRB, 0.86230E+00_JPRB, 0.83505E+00_JPRB, 0.83233E+00_JPRB, 0.81937E+00_JPRB, 0.81843E+00_JPRB ,&
&  0.82832E+00_JPRB, 0.84347E+00_JPRB, 0.87481E+00_JPRB, 0.92471E+00_JPRB, 0.98183E+00_JPRB, 0.11064E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.16848E+01_JPRB, 0.20951E+01_JPRB, 0.78920E+00_JPRB, 0.10140E+01_JPRB, 0.11825E+01_JPRB, 0.10826E+01_JPRB, 0.94221E+00_JPRB ,&
&  0.87160E+00_JPRB, 0.82719E+00_JPRB, 0.89682E+00_JPRB, 0.10046E+01_JPRB, 0.10537E+01_JPRB, 0.11245E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.12147E+01_JPRB, 0.12986E+01_JPRB, 0.14196E+01_JPRB, 0.14972E+01_JPRB, 0.15478E+01_JPRB, 0.15826E+01_JPRB, 0.16413E+01_JPRB ,&
&  0.18202E+01_JPRB, 0.21127E+01_JPRB, 0.25401E+01_JPRB, 0.24264E+01_JPRB, 0.23311E+01_JPRB, 0.19643E+01_JPRB, 0.16168E+01_JPRB ,&
&  0.13515E+01_JPRB, 0.13830E+01_JPRB, 0.13699E+01_JPRB, 0.12559E+01_JPRB, 0.10861E+01_JPRB, 0.99030E+00_JPRB, 0.10304E+01_JPRB ,&
&  0.92884E+00_JPRB, 0.86583E+00_JPRB, 0.84935E+00_JPRB, 0.86053E+00_JPRB, 0.91536E+00_JPRB, 0.91341E+00_JPRB, 0.89906E+00_JPRB ,&
&  0.91700E+00_JPRB, 0.91237E+00_JPRB, 0.93504E+00_JPRB, 0.97056E+00_JPRB, 0.10089E+01_JPRB, 0.10737E+01_JPRB, 0.11107E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11479E+01_JPRB, 0.11556E+01_JPRB, 0.11152E+01_JPRB, 0.10538E+01_JPRB, 0.10101E+01_JPRB, 0.99293E+00_JPRB ,&
&  0.10119E+01_JPRB, 0.10242E+01_JPRB, 0.92349E+00_JPRB, 0.87218E+00_JPRB, 0.89812E+00_JPRB, 0.90877E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.11117E+01_JPRB, 0.11837E+01_JPRB, 0.11379E+01_JPRB, 0.86794E+00_JPRB, 0.85597E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.74724E+00_JPRB, 0.70451E+00_JPRB, 0.67190E+00_JPRB, 0.68247E+00_JPRB, 0.73261E+00_JPRB, 0.78398E+00_JPRB, 0.78213E+00_JPRB ,&
&  0.76744E+00_JPRB, 0.78729E+00_JPRB, 0.85788E+00_JPRB, 0.91538E+00_JPRB, 0.92317E+00_JPRB, 0.93238E+00_JPRB, 0.95009E+00_JPRB ,&
&  0.89820E+00_JPRB, 0.86612E+00_JPRB, 0.85041E+00_JPRB, 0.83171E+00_JPRB, 0.83209E+00_JPRB, 0.82712E+00_JPRB, 0.83036E+00_JPRB ,&
&  0.84898E+00_JPRB, 0.87251E+00_JPRB, 0.93011E+00_JPRB, 0.10199E+01_JPRB, 0.11675E+01_JPRB, 0.14702E+01_JPRB, 0.19284E+01_JPRB ,&
&  0.26853E+01_JPRB, 0.17788E+01_JPRB, 0.68906E+00_JPRB, 0.88311E+00_JPRB, 0.92685E+00_JPRB, 0.91602E+00_JPRB, 0.81358E+00_JPRB ,&
&  0.81390E+00_JPRB, 0.84846E+00_JPRB, 0.92307E+00_JPRB, 0.11113E+01_JPRB, 0.12169E+01_JPRB, 0.13207E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.15706E+01_JPRB, 0.16596E+01_JPRB, 0.17530E+01_JPRB, 0.18409E+01_JPRB, 0.19519E+01_JPRB, 0.21117E+01_JPRB ,&
&  0.23973E+01_JPRB, 0.27265E+01_JPRB, 0.30385E+01_JPRB, 0.27821E+01_JPRB, 0.27149E+01_JPRB, 0.20764E+01_JPRB, 0.17143E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13040E+01_JPRB, 0.12973E+01_JPRB, 0.11791E+01_JPRB, 0.10332E+01_JPRB, 0.91398E+00_JPRB, 0.99512E+00_JPRB ,&
&  0.88820E+00_JPRB, 0.82782E+00_JPRB, 0.79607E+00_JPRB, 0.81373E+00_JPRB, 0.84600E+00_JPRB, 0.85928E+00_JPRB, 0.86567E+00_JPRB ,&
&  0.88635E+00_JPRB, 0.89334E+00_JPRB, 0.93479E+00_JPRB, 0.96305E+00_JPRB, 0.98793E+00_JPRB, 0.10348E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.10445E+01_JPRB, 0.10491E+01_JPRB, 0.10679E+01_JPRB, 0.10879E+01_JPRB, 0.10345E+01_JPRB, 0.95020E+00_JPRB, 0.93551E+00_JPRB ,&
&  0.92463E+00_JPRB, 0.90767E+00_JPRB, 0.85559E+00_JPRB, 0.86109E+00_JPRB, 0.86532E+00_JPRB, 0.92839E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10063E+01_JPRB, 0.93648E+00_JPRB, 0.81023E+00_JPRB, 0.79479E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.68839E+00_JPRB, 0.66335E+00_JPRB, 0.62390E+00_JPRB, 0.61970E+00_JPRB, 0.67666E+00_JPRB, 0.72428E+00_JPRB, 0.72640E+00_JPRB ,&
&  0.73404E+00_JPRB, 0.79809E+00_JPRB, 0.91288E+00_JPRB, 0.10033E+01_JPRB, 0.10319E+01_JPRB, 0.10229E+01_JPRB, 0.92699E+00_JPRB ,&
&  0.88972E+00_JPRB, 0.86666E+00_JPRB, 0.84102E+00_JPRB, 0.83001E+00_JPRB, 0.82788E+00_JPRB, 0.83522E+00_JPRB, 0.85786E+00_JPRB ,&
&  0.89213E+00_JPRB, 0.95209E+00_JPRB, 0.10535E+01_JPRB, 0.11960E+01_JPRB, 0.14049E+01_JPRB, 0.16427E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.13774E+01_JPRB, 0.60618E+00_JPRB, 0.68034E+00_JPRB, 0.89092E+00_JPRB, 0.82762E+00_JPRB, 0.82462E+00_JPRB, 0.79589E+00_JPRB ,&
&  0.82394E+00_JPRB, 0.89938E+00_JPRB, 0.98721E+00_JPRB, 0.11868E+01_JPRB, 0.13321E+01_JPRB, 0.14881E+01_JPRB, 0.15837E+01_JPRB ,&
&  0.16867E+01_JPRB, 0.18081E+01_JPRB, 0.19544E+01_JPRB, 0.20949E+01_JPRB, 0.22434E+01_JPRB, 0.23969E+01_JPRB, 0.26379E+01_JPRB ,&
&  0.29399E+01_JPRB, 0.32481E+01_JPRB, 0.35364E+01_JPRB, 0.32031E+01_JPRB, 0.33137E+01_JPRB, 0.26601E+01_JPRB, 0.20445E+01_JPRB ,&
&  0.15713E+01_JPRB, 0.11538E+01_JPRB, 0.10746E+01_JPRB, 0.11587E+01_JPRB, 0.10563E+01_JPRB, 0.85518E+00_JPRB, 0.88277E+00_JPRB ,&
&  0.82220E+00_JPRB, 0.80436E+00_JPRB, 0.79984E+00_JPRB, 0.74899E+00_JPRB, 0.80708E+00_JPRB, 0.83404E+00_JPRB, 0.85392E+00_JPRB ,&
&  0.84171E+00_JPRB, 0.86030E+00_JPRB, 0.88299E+00_JPRB, 0.92327E+00_JPRB, 0.91790E+00_JPRB, 0.92085E+00_JPRB, 0.90536E+00_JPRB ,&
&  0.91006E+00_JPRB, 0.92329E+00_JPRB, 0.94963E+00_JPRB, 0.97896E+00_JPRB, 0.98269E+00_JPRB, 0.90626E+00_JPRB, 0.84855E+00_JPRB ,&
&  0.81763E+00_JPRB, 0.80089E+00_JPRB, 0.76348E+00_JPRB, 0.80632E+00_JPRB, 0.81942E+00_JPRB, 0.83322E+00_JPRB, 0.86380E+00_JPRB ,&
&  0.87643E+00_JPRB, 0.87556E+00_JPRB, 0.82507E+00_JPRB, 0.76823E+00_JPRB, 0.73121E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.57976E+00_JPRB, 0.56707E+00_JPRB, 0.55112E+00_JPRB, 0.56825E+00_JPRB, 0.60197E+00_JPRB, 0.64371E+00_JPRB, 0.66191E+00_JPRB ,&
&  0.68527E+00_JPRB, 0.78451E+00_JPRB, 0.91506E+00_JPRB, 0.10301E+01_JPRB, 0.10484E+01_JPRB, 0.10154E+01_JPRB, 0.91493E+00_JPRB ,&
&  0.88682E+00_JPRB, 0.86013E+00_JPRB, 0.84482E+00_JPRB, 0.82864E+00_JPRB, 0.83174E+00_JPRB, 0.86189E+00_JPRB, 0.91312E+00_JPRB ,&
&  0.97904E+00_JPRB, 0.10500E+01_JPRB, 0.11311E+01_JPRB, 0.12169E+01_JPRB, 0.12886E+01_JPRB, 0.13048E+01_JPRB, 0.12002E+01_JPRB ,&
&  0.51400E+00_JPRB, 0.60330E+00_JPRB, 0.82347E+00_JPRB, 0.80393E+00_JPRB, 0.76126E+00_JPRB, 0.74594E+00_JPRB, 0.79112E+00_JPRB ,&
&  0.84540E+00_JPRB, 0.86478E+00_JPRB, 0.96976E+00_JPRB, 0.12115E+01_JPRB, 0.13824E+01_JPRB, 0.15673E+01_JPRB, 0.16908E+01_JPRB ,&
&  0.18176E+01_JPRB, 0.19696E+01_JPRB, 0.21614E+01_JPRB, 0.23336E+01_JPRB, 0.24992E+01_JPRB, 0.26636E+01_JPRB, 0.28875E+01_JPRB ,&
&  0.31959E+01_JPRB, 0.35598E+01_JPRB, 0.38973E+01_JPRB, 0.36665E+01_JPRB, 0.39408E+01_JPRB, 0.34873E+01_JPRB, 0.29971E+01_JPRB ,&
&  0.18326E+01_JPRB, 0.11073E+01_JPRB, 0.10719E+01_JPRB, 0.11637E+01_JPRB, 0.10600E+01_JPRB, 0.97879E+00_JPRB, 0.88908E+00_JPRB ,&
&  0.83203E+00_JPRB, 0.76712E+00_JPRB, 0.78219E+00_JPRB, 0.79920E+00_JPRB, 0.79461E+00_JPRB, 0.83285E+00_JPRB, 0.80075E+00_JPRB ,&
&  0.81263E+00_JPRB, 0.82048E+00_JPRB, 0.81239E+00_JPRB, 0.81837E+00_JPRB, 0.82127E+00_JPRB, 0.82629E+00_JPRB, 0.80805E+00_JPRB ,&
&  0.80016E+00_JPRB, 0.80149E+00_JPRB, 0.81966E+00_JPRB, 0.84412E+00_JPRB, 0.84803E+00_JPRB, 0.83578E+00_JPRB, 0.78822E+00_JPRB ,&
&  0.73820E+00_JPRB, 0.74793E+00_JPRB, 0.84763E+00_JPRB, 0.97985E+00_JPRB, 0.95148E+00_JPRB, 0.81986E+00_JPRB, 0.64805E+00_JPRB ,&
&  0.63261E+00_JPRB, 0.65278E+00_JPRB, 0.65470E+00_JPRB, 0.63418E+00_JPRB, 0.60920E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.47459E+00_JPRB, 0.46333E+00_JPRB, 0.46260E+00_JPRB, 0.47309E+00_JPRB, 0.47960E+00_JPRB, 0.50392E+00_JPRB, 0.53437E+00_JPRB ,&
&  0.54970E+00_JPRB, 0.61861E+00_JPRB, 0.77226E+00_JPRB, 0.90324E+00_JPRB, 0.95629E+00_JPRB, 0.99465E+00_JPRB, 0.94581E+00_JPRB ,&
&  0.93107E+00_JPRB, 0.89910E+00_JPRB, 0.84935E+00_JPRB, 0.84152E+00_JPRB, 0.85640E+00_JPRB, 0.88814E+00_JPRB, 0.93055E+00_JPRB ,&
&  0.97903E+00_JPRB, 0.10292E+01_JPRB, 0.10709E+01_JPRB, 0.11095E+01_JPRB, 0.11632E+01_JPRB, 0.11460E+01_JPRB, 0.10850E+01_JPRB ,&
&  0.54760E+00_JPRB, 0.85186E+00_JPRB, 0.83537E+00_JPRB, 0.77472E+00_JPRB, 0.71334E+00_JPRB, 0.69707E+00_JPRB, 0.77131E+00_JPRB ,&
&  0.84088E+00_JPRB, 0.84687E+00_JPRB, 0.94744E+00_JPRB, 0.10562E+01_JPRB, 0.13510E+01_JPRB, 0.15157E+01_JPRB, 0.16745E+01_JPRB ,&
&  0.18301E+01_JPRB, 0.19627E+01_JPRB, 0.21248E+01_JPRB, 0.22823E+01_JPRB, 0.24535E+01_JPRB, 0.25935E+01_JPRB, 0.27530E+01_JPRB ,&
&  0.29609E+01_JPRB, 0.32415E+01_JPRB, 0.35098E+01_JPRB, 0.34150E+01_JPRB, 0.29754E+01_JPRB, 0.27558E+01_JPRB, 0.22885E+01_JPRB ,&
&  0.16592E+01_JPRB, 0.10635E+01_JPRB, 0.10876E+01_JPRB, 0.11269E+01_JPRB, 0.10755E+01_JPRB, 0.10157E+01_JPRB, 0.94395E+00_JPRB ,&
&  0.88652E+00_JPRB, 0.82741E+00_JPRB, 0.75363E+00_JPRB, 0.75564E+00_JPRB, 0.77940E+00_JPRB, 0.78110E+00_JPRB, 0.76620E+00_JPRB ,&
&  0.76532E+00_JPRB, 0.74342E+00_JPRB, 0.72090E+00_JPRB, 0.69864E+00_JPRB, 0.69887E+00_JPRB, 0.72630E+00_JPRB, 0.73275E+00_JPRB ,&
&  0.74251E+00_JPRB, 0.69929E+00_JPRB, 0.66954E+00_JPRB, 0.68285E+00_JPRB, 0.69569E+00_JPRB, 0.71647E+00_JPRB, 0.70798E+00_JPRB ,&
&  0.75116E+00_JPRB, 0.90181E+00_JPRB, 0.12943E+01_JPRB, 0.16954E+01_JPRB, 0.13928E+01_JPRB, 0.10960E+01_JPRB, 0.62165E+00_JPRB ,&
&  0.49452E+00_JPRB, 0.47742E+00_JPRB, 0.49499E+00_JPRB, 0.49824E+00_JPRB, 0.49693E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.42458E+00_JPRB, 0.41585E+00_JPRB, 0.42348E+00_JPRB, 0.42188E+00_JPRB, 0.42560E+00_JPRB, 0.43262E+00_JPRB, 0.43694E+00_JPRB ,&
&  0.44032E+00_JPRB, 0.45669E+00_JPRB, 0.54475E+00_JPRB, 0.66575E+00_JPRB, 0.77981E+00_JPRB, 0.86576E+00_JPRB, 0.90224E+00_JPRB ,&
&  0.93574E+00_JPRB, 0.94492E+00_JPRB, 0.92684E+00_JPRB, 0.90353E+00_JPRB, 0.90478E+00_JPRB, 0.94159E+00_JPRB, 0.96996E+00_JPRB ,&
&  0.99452E+00_JPRB, 0.10182E+01_JPRB, 0.10423E+01_JPRB, 0.10828E+01_JPRB, 0.11221E+01_JPRB, 0.10648E+01_JPRB, 0.66307E+00_JPRB ,&
&  0.87627E+00_JPRB, 0.90441E+00_JPRB, 0.89150E+00_JPRB, 0.81458E+00_JPRB, 0.74676E+00_JPRB, 0.69915E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.75269E+00_JPRB, 0.76845E+00_JPRB, 0.78911E+00_JPRB, 0.85365E+00_JPRB, 0.11790E+01_JPRB, 0.12800E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.17037E+01_JPRB, 0.18123E+01_JPRB, 0.19363E+01_JPRB, 0.20619E+01_JPRB, 0.21644E+01_JPRB, 0.22940E+01_JPRB ,&
&  0.23480E+01_JPRB, 0.23921E+01_JPRB, 0.23522E+01_JPRB, 0.22372E+01_JPRB, 0.19032E+01_JPRB, 0.19598E+01_JPRB, 0.15144E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.98887E+00_JPRB, 0.10781E+01_JPRB, 0.11379E+01_JPRB, 0.11154E+01_JPRB, 0.10442E+01_JPRB, 0.93886E+00_JPRB ,&
&  0.87655E+00_JPRB, 0.82911E+00_JPRB, 0.78515E+00_JPRB, 0.73993E+00_JPRB, 0.73257E+00_JPRB, 0.72194E+00_JPRB, 0.70465E+00_JPRB ,&
&  0.69144E+00_JPRB, 0.65661E+00_JPRB, 0.61812E+00_JPRB, 0.61404E+00_JPRB, 0.62844E+00_JPRB, 0.70075E+00_JPRB, 0.83683E+00_JPRB ,&
&  0.90574E+00_JPRB, 0.56883E+00_JPRB, 0.55441E+00_JPRB, 0.53601E+00_JPRB, 0.55122E+00_JPRB, 0.59580E+00_JPRB, 0.68011E+00_JPRB ,&
&  0.88886E+00_JPRB, 0.15949E+01_JPRB, 0.24989E+01_JPRB, 0.19132E+01_JPRB, 0.21941E+01_JPRB, 0.15039E+01_JPRB, 0.60520E+00_JPRB ,&
&  0.46874E+00_JPRB, 0.45797E+00_JPRB, 0.46283E+00_JPRB, 0.45266E+00_JPRB, 0.43981E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.43559E+00_JPRB, 0.42599E+00_JPRB, 0.43352E+00_JPRB, 0.44325E+00_JPRB, 0.43941E+00_JPRB, 0.43895E+00_JPRB, 0.43403E+00_JPRB ,&
&  0.42465E+00_JPRB, 0.41848E+00_JPRB, 0.44002E+00_JPRB, 0.48089E+00_JPRB, 0.53740E+00_JPRB, 0.61047E+00_JPRB, 0.68144E+00_JPRB ,&
&  0.72569E+00_JPRB, 0.77544E+00_JPRB, 0.81704E+00_JPRB, 0.84953E+00_JPRB, 0.84759E+00_JPRB, 0.84351E+00_JPRB, 0.82448E+00_JPRB ,&
&  0.81038E+00_JPRB, 0.81845E+00_JPRB, 0.85503E+00_JPRB, 0.88708E+00_JPRB, 0.94937E+00_JPRB, 0.10381E+01_JPRB, 0.67695E+00_JPRB ,&
&  0.10258E+01_JPRB, 0.10181E+01_JPRB, 0.95326E+00_JPRB, 0.86165E+00_JPRB, 0.81107E+00_JPRB, 0.73735E+00_JPRB, 0.67126E+00_JPRB ,&
&  0.65343E+00_JPRB, 0.66466E+00_JPRB, 0.66807E+00_JPRB, 0.75677E+00_JPRB, 0.88493E+00_JPRB, 0.96079E+00_JPRB, 0.10715E+01_JPRB ,&
&  0.11462E+01_JPRB, 0.12399E+01_JPRB, 0.13309E+01_JPRB, 0.13968E+01_JPRB, 0.14574E+01_JPRB, 0.14891E+01_JPRB, 0.14834E+01_JPRB ,&
&  0.13957E+01_JPRB, 0.13130E+01_JPRB, 0.12636E+01_JPRB, 0.13170E+01_JPRB, 0.12181E+01_JPRB, 0.11468E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.82841E+00_JPRB, 0.79572E+00_JPRB, 0.89801E+00_JPRB, 0.11483E+01_JPRB, 0.11810E+01_JPRB, 0.11297E+01_JPRB, 0.10465E+01_JPRB ,&
&  0.97335E+00_JPRB, 0.90220E+00_JPRB, 0.83919E+00_JPRB, 0.77916E+00_JPRB, 0.73798E+00_JPRB, 0.70536E+00_JPRB, 0.67326E+00_JPRB ,&
&  0.64406E+00_JPRB, 0.60459E+00_JPRB, 0.56500E+00_JPRB, 0.55258E+00_JPRB, 0.54743E+00_JPRB, 0.56242E+00_JPRB, 0.56106E+00_JPRB ,&
&  0.64710E+00_JPRB, 0.51460E+00_JPRB, 0.47427E+00_JPRB, 0.47162E+00_JPRB, 0.44130E+00_JPRB, 0.49808E+00_JPRB, 0.67455E+00_JPRB ,&
&  0.10874E+01_JPRB, 0.16272E+01_JPRB, 0.27734E+01_JPRB, 0.50024E+01_JPRB, 0.27658E+01_JPRB, 0.21300E+01_JPRB, 0.50567E+00_JPRB ,&
&  0.48332E+00_JPRB, 0.48488E+00_JPRB, 0.48142E+00_JPRB, 0.46424E+00_JPRB, 0.45930E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.47883E+00_JPRB, 0.47696E+00_JPRB, 0.48441E+00_JPRB, 0.49239E+00_JPRB, 0.49431E+00_JPRB, 0.49580E+00_JPRB, 0.49674E+00_JPRB ,&
&  0.49920E+00_JPRB, 0.49882E+00_JPRB, 0.50075E+00_JPRB, 0.50782E+00_JPRB, 0.52299E+00_JPRB, 0.55635E+00_JPRB, 0.59488E+00_JPRB ,&
&  0.62846E+00_JPRB, 0.65928E+00_JPRB, 0.68827E+00_JPRB, 0.71250E+00_JPRB, 0.73774E+00_JPRB, 0.75310E+00_JPRB, 0.80160E+00_JPRB ,&
&  0.86275E+00_JPRB, 0.93701E+00_JPRB, 0.10102E+01_JPRB, 0.10599E+01_JPRB, 0.10969E+01_JPRB, 0.10420E+01_JPRB, 0.80261E+00_JPRB ,&
&  0.12504E+01_JPRB, 0.12206E+01_JPRB, 0.10934E+01_JPRB, 0.97453E+00_JPRB, 0.89122E+00_JPRB, 0.81804E+00_JPRB, 0.74702E+00_JPRB ,&
&  0.68188E+00_JPRB, 0.69020E+00_JPRB, 0.64924E+00_JPRB, 0.66440E+00_JPRB, 0.64152E+00_JPRB, 0.68388E+00_JPRB, 0.79600E+00_JPRB ,&
&  0.88817E+00_JPRB, 0.96786E+00_JPRB, 0.10508E+01_JPRB, 0.11365E+01_JPRB, 0.11798E+01_JPRB, 0.11504E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.96684E+00_JPRB, 0.91997E+00_JPRB, 0.85433E+00_JPRB, 0.89905E+00_JPRB, 0.90922E+00_JPRB, 0.89393E+00_JPRB, 0.89735E+00_JPRB ,&
&  0.79645E+00_JPRB, 0.82011E+00_JPRB, 0.95009E+00_JPRB, 0.12454E+01_JPRB, 0.11997E+01_JPRB, 0.11313E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.10165E+01_JPRB, 0.95229E+00_JPRB, 0.89517E+00_JPRB, 0.82901E+00_JPRB, 0.76979E+00_JPRB, 0.72830E+00_JPRB, 0.68903E+00_JPRB ,&
&  0.64613E+00_JPRB, 0.59641E+00_JPRB, 0.55616E+00_JPRB, 0.52962E+00_JPRB, 0.52278E+00_JPRB, 0.48747E+00_JPRB, 0.67799E+00_JPRB ,&
&  0.58064E+00_JPRB, 0.48623E+00_JPRB, 0.49735E+00_JPRB, 0.36624E+00_JPRB, 0.45739E+00_JPRB, 0.51312E+00_JPRB, 0.63413E+00_JPRB ,&
&  0.79794E+00_JPRB, 0.12741E+01_JPRB, 0.20052E+01_JPRB, 0.18107E+01_JPRB, 0.13796E+01_JPRB, 0.88177E+00_JPRB, 0.52215E+00_JPRB ,&
&  0.50086E+00_JPRB, 0.49584E+00_JPRB, 0.50007E+00_JPRB, 0.49043E+00_JPRB, 0.48668E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.50944E+00_JPRB, 0.50871E+00_JPRB, 0.52217E+00_JPRB, 0.53555E+00_JPRB, 0.54340E+00_JPRB, 0.54917E+00_JPRB, 0.55855E+00_JPRB ,&
&  0.56374E+00_JPRB, 0.57143E+00_JPRB, 0.58038E+00_JPRB, 0.59691E+00_JPRB, 0.62149E+00_JPRB, 0.65871E+00_JPRB, 0.70178E+00_JPRB ,&
&  0.75429E+00_JPRB, 0.81087E+00_JPRB, 0.87364E+00_JPRB, 0.94123E+00_JPRB, 0.10251E+01_JPRB, 0.11397E+01_JPRB, 0.12954E+01_JPRB ,&
&  0.13768E+01_JPRB, 0.14800E+01_JPRB, 0.16025E+01_JPRB, 0.17055E+01_JPRB, 0.17116E+01_JPRB, 0.16485E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.14411E+01_JPRB, 0.13562E+01_JPRB, 0.12198E+01_JPRB, 0.10931E+01_JPRB, 0.10037E+01_JPRB, 0.92831E+00_JPRB, 0.87280E+00_JPRB ,&
&  0.85261E+00_JPRB, 0.81483E+00_JPRB, 0.76263E+00_JPRB, 0.72473E+00_JPRB, 0.70358E+00_JPRB, 0.71507E+00_JPRB, 0.69945E+00_JPRB ,&
&  0.76113E+00_JPRB, 0.89929E+00_JPRB, 0.10720E+01_JPRB, 0.12775E+01_JPRB, 0.13871E+01_JPRB, 0.13590E+01_JPRB, 0.11642E+01_JPRB ,&
&  0.10375E+01_JPRB, 0.95327E+00_JPRB, 0.92008E+00_JPRB, 0.96312E+00_JPRB, 0.99605E+00_JPRB, 0.10524E+01_JPRB, 0.10739E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.96315E+00_JPRB, 0.10220E+01_JPRB, 0.12806E+01_JPRB, 0.12167E+01_JPRB, 0.10922E+01_JPRB, 0.10716E+01_JPRB ,&
&  0.99363E+00_JPRB, 0.93698E+00_JPRB, 0.89976E+00_JPRB, 0.83890E+00_JPRB, 0.77387E+00_JPRB, 0.72787E+00_JPRB, 0.70190E+00_JPRB ,&
&  0.65289E+00_JPRB, 0.59877E+00_JPRB, 0.55788E+00_JPRB, 0.55360E+00_JPRB, 0.60206E+00_JPRB, 0.64085E+00_JPRB, 0.85863E+00_JPRB ,&
&  0.61406E+00_JPRB, 0.45083E+00_JPRB, 0.48387E+00_JPRB, 0.59474E+00_JPRB, 0.56152E+00_JPRB, 0.58702E+00_JPRB, 0.64800E+00_JPRB ,&
&  0.71813E+00_JPRB, 0.78354E+00_JPRB, 0.77804E+00_JPRB, 0.66810E+00_JPRB, 0.62529E+00_JPRB, 0.60307E+00_JPRB, 0.57755E+00_JPRB ,&
&  0.55674E+00_JPRB, 0.51992E+00_JPRB, 0.53206E+00_JPRB, 0.51685E+00_JPRB, 0.51381E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.55662E+00_JPRB, 0.55010E+00_JPRB, 0.54952E+00_JPRB, 0.54805E+00_JPRB, 0.55805E+00_JPRB, 0.56056E+00_JPRB, 0.56165E+00_JPRB ,&
&  0.56142E+00_JPRB, 0.57197E+00_JPRB, 0.58007E+00_JPRB, 0.59189E+00_JPRB, 0.60589E+00_JPRB, 0.63773E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.72145E+00_JPRB, 0.77427E+00_JPRB, 0.83301E+00_JPRB, 0.91671E+00_JPRB, 0.10338E+01_JPRB, 0.11190E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.14830E+01_JPRB, 0.16507E+01_JPRB, 0.18752E+01_JPRB, 0.21411E+01_JPRB, 0.21721E+01_JPRB, 0.19677E+01_JPRB ,&
&  0.14108E+01_JPRB, 0.13664E+01_JPRB, 0.12622E+01_JPRB, 0.11368E+01_JPRB, 0.11412E+01_JPRB, 0.11051E+01_JPRB, 0.11123E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11380E+01_JPRB, 0.11226E+01_JPRB, 0.11265E+01_JPRB, 0.11299E+01_JPRB, 0.11347E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11109E+01_JPRB, 0.11506E+01_JPRB, 0.13243E+01_JPRB, 0.15480E+01_JPRB, 0.17005E+01_JPRB, 0.17523E+01_JPRB, 0.16677E+01_JPRB ,&
&  0.15388E+01_JPRB, 0.14305E+01_JPRB, 0.13293E+01_JPRB, 0.13121E+01_JPRB, 0.13608E+01_JPRB, 0.13923E+01_JPRB, 0.14916E+01_JPRB ,&
&  0.14029E+01_JPRB, 0.12165E+01_JPRB, 0.12043E+01_JPRB, 0.12096E+01_JPRB, 0.11753E+01_JPRB, 0.11155E+01_JPRB, 0.10342E+01_JPRB ,&
&  0.10161E+01_JPRB, 0.95655E+00_JPRB, 0.91968E+00_JPRB, 0.85316E+00_JPRB, 0.79016E+00_JPRB, 0.74618E+00_JPRB, 0.71926E+00_JPRB ,&
&  0.67203E+00_JPRB, 0.63108E+00_JPRB, 0.60917E+00_JPRB, 0.62608E+00_JPRB, 0.66424E+00_JPRB, 0.91929E+00_JPRB, 0.90989E+00_JPRB ,&
&  0.71964E+00_JPRB, 0.64763E+00_JPRB, 0.55939E+00_JPRB, 0.76735E+00_JPRB, 0.66659E+00_JPRB, 0.65338E+00_JPRB, 0.65497E+00_JPRB ,&
&  0.65984E+00_JPRB, 0.65590E+00_JPRB, 0.63407E+00_JPRB, 0.62904E+00_JPRB, 0.63743E+00_JPRB, 0.64444E+00_JPRB, 0.63361E+00_JPRB ,&
&  0.59737E+00_JPRB, 0.57996E+00_JPRB, 0.60371E+00_JPRB, 0.56676E+00_JPRB, 0.55953E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.64238E+00_JPRB, 0.61416E+00_JPRB, 0.59218E+00_JPRB, 0.59640E+00_JPRB, 0.60371E+00_JPRB, 0.59253E+00_JPRB, 0.62306E+00_JPRB ,&
&  0.59957E+00_JPRB, 0.62240E+00_JPRB, 0.63626E+00_JPRB, 0.63697E+00_JPRB, 0.67811E+00_JPRB, 0.72055E+00_JPRB, 0.77483E+00_JPRB ,&
&  0.85099E+00_JPRB, 0.97214E+00_JPRB, 0.11246E+01_JPRB, 0.13159E+01_JPRB, 0.15495E+01_JPRB, 0.16692E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.19534E+01_JPRB, 0.21681E+01_JPRB, 0.24662E+01_JPRB, 0.27870E+01_JPRB, 0.31422E+01_JPRB, 0.29076E+01_JPRB, 0.23948E+01_JPRB ,&
&  0.16977E+01_JPRB, 0.13223E+01_JPRB, 0.12356E+01_JPRB, 0.11353E+01_JPRB, 0.11896E+01_JPRB, 0.13166E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.13907E+01_JPRB, 0.14247E+01_JPRB, 0.14580E+01_JPRB, 0.15000E+01_JPRB, 0.15517E+01_JPRB, 0.16090E+01_JPRB, 0.16320E+01_JPRB ,&
&  0.16487E+01_JPRB, 0.16669E+01_JPRB, 0.16893E+01_JPRB, 0.17769E+01_JPRB, 0.18910E+01_JPRB, 0.20151E+01_JPRB, 0.20379E+01_JPRB ,&
&  0.20364E+01_JPRB, 0.19263E+01_JPRB, 0.16884E+01_JPRB, 0.16714E+01_JPRB, 0.17236E+01_JPRB, 0.17577E+01_JPRB, 0.18012E+01_JPRB ,&
&  0.19336E+01_JPRB, 0.15909E+01_JPRB, 0.11137E+01_JPRB, 0.11485E+01_JPRB, 0.11621E+01_JPRB, 0.11777E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.98901E+00_JPRB, 0.98354E+00_JPRB, 0.93229E+00_JPRB, 0.90696E+00_JPRB, 0.87841E+00_JPRB, 0.82483E+00_JPRB ,&
&  0.72999E+00_JPRB, 0.69578E+00_JPRB, 0.66785E+00_JPRB, 0.65812E+00_JPRB, 0.74427E+00_JPRB, 0.74543E+00_JPRB, 0.80969E+00_JPRB ,&
&  0.83576E+00_JPRB, 0.93519E+00_JPRB, 0.86069E+00_JPRB, 0.10912E+01_JPRB, 0.80204E+00_JPRB, 0.72601E+00_JPRB, 0.71008E+00_JPRB ,&
&  0.72755E+00_JPRB, 0.73641E+00_JPRB, 0.74754E+00_JPRB, 0.73642E+00_JPRB, 0.75164E+00_JPRB, 0.77780E+00_JPRB, 0.77671E+00_JPRB ,&
&  0.73314E+00_JPRB, 0.71662E+00_JPRB, 0.69060E+00_JPRB, 0.65912E+00_JPRB, 0.65271E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.70958E+00_JPRB, 0.71347E+00_JPRB, 0.71035E+00_JPRB, 0.70988E+00_JPRB, 0.71171E+00_JPRB, 0.69887E+00_JPRB, 0.72189E+00_JPRB ,&
&  0.71919E+00_JPRB, 0.74166E+00_JPRB, 0.76695E+00_JPRB, 0.79428E+00_JPRB, 0.84384E+00_JPRB, 0.89540E+00_JPRB, 0.96712E+00_JPRB ,&
&  0.10861E+01_JPRB, 0.12732E+01_JPRB, 0.15155E+01_JPRB, 0.18197E+01_JPRB, 0.19301E+01_JPRB, 0.20036E+01_JPRB, 0.21257E+01_JPRB ,&
&  0.22808E+01_JPRB, 0.24103E+01_JPRB, 0.24998E+01_JPRB, 0.25881E+01_JPRB, 0.26402E+01_JPRB, 0.25357E+01_JPRB, 0.23995E+01_JPRB ,&
&  0.19420E+01_JPRB, 0.14050E+01_JPRB, 0.14125E+01_JPRB, 0.12980E+01_JPRB, 0.13317E+01_JPRB, 0.13790E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.14691E+01_JPRB, 0.14935E+01_JPRB, 0.15297E+01_JPRB, 0.15839E+01_JPRB, 0.16294E+01_JPRB, 0.16748E+01_JPRB ,&
&  0.17095E+01_JPRB, 0.17401E+01_JPRB, 0.17663E+01_JPRB, 0.17149E+01_JPRB, 0.17662E+01_JPRB, 0.18754E+01_JPRB, 0.19775E+01_JPRB ,&
&  0.20536E+01_JPRB, 0.20173E+01_JPRB, 0.18009E+01_JPRB, 0.18470E+01_JPRB, 0.19843E+01_JPRB, 0.19111E+01_JPRB, 0.19898E+01_JPRB ,&
&  0.21798E+01_JPRB, 0.19187E+01_JPRB, 0.11003E+01_JPRB, 0.10158E+01_JPRB, 0.10897E+01_JPRB, 0.11272E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.10730E+01_JPRB, 0.10597E+01_JPRB, 0.10593E+01_JPRB, 0.10376E+01_JPRB, 0.10376E+01_JPRB, 0.92425E+00_JPRB ,&
&  0.68953E+00_JPRB, 0.66468E+00_JPRB, 0.68267E+00_JPRB, 0.70659E+00_JPRB, 0.76641E+00_JPRB, 0.78764E+00_JPRB, 0.79031E+00_JPRB ,&
&  0.90269E+00_JPRB, 0.10594E+01_JPRB, 0.12770E+01_JPRB, 0.11581E+01_JPRB, 0.84769E+00_JPRB, 0.80312E+00_JPRB, 0.79351E+00_JPRB ,&
&  0.81467E+00_JPRB, 0.81603E+00_JPRB, 0.82762E+00_JPRB, 0.83351E+00_JPRB, 0.84306E+00_JPRB, 0.82811E+00_JPRB, 0.81057E+00_JPRB ,&
&  0.79055E+00_JPRB, 0.77327E+00_JPRB, 0.75838E+00_JPRB, 0.73515E+00_JPRB, 0.71771E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.78587E+00_JPRB, 0.76580E+00_JPRB, 0.75587E+00_JPRB, 0.74281E+00_JPRB, 0.73786E+00_JPRB, 0.73124E+00_JPRB, 0.72984E+00_JPRB ,&
&  0.73220E+00_JPRB, 0.73928E+00_JPRB, 0.76255E+00_JPRB, 0.80094E+00_JPRB, 0.85221E+00_JPRB, 0.92098E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.13254E+01_JPRB, 0.15314E+01_JPRB, 0.17711E+01_JPRB, 0.19464E+01_JPRB, 0.20127E+01_JPRB, 0.21349E+01_JPRB ,&
&  0.22219E+01_JPRB, 0.22585E+01_JPRB, 0.22413E+01_JPRB, 0.21848E+01_JPRB, 0.20821E+01_JPRB, 0.20865E+01_JPRB, 0.22053E+01_JPRB ,&
&  0.20307E+01_JPRB, 0.15395E+01_JPRB, 0.13790E+01_JPRB, 0.13194E+01_JPRB, 0.13159E+01_JPRB, 0.13383E+01_JPRB, 0.13823E+01_JPRB ,&
&  0.14176E+01_JPRB, 0.14435E+01_JPRB, 0.14648E+01_JPRB, 0.14900E+01_JPRB, 0.15572E+01_JPRB, 0.15809E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.16384E+01_JPRB, 0.16616E+01_JPRB, 0.17144E+01_JPRB, 0.16185E+01_JPRB, 0.17257E+01_JPRB, 0.17827E+01_JPRB, 0.18229E+01_JPRB ,&
&  0.18080E+01_JPRB, 0.17386E+01_JPRB, 0.18351E+01_JPRB, 0.19064E+01_JPRB, 0.19740E+01_JPRB, 0.19750E+01_JPRB, 0.19298E+01_JPRB ,&
&  0.20251E+01_JPRB, 0.18835E+01_JPRB, 0.11603E+01_JPRB, 0.10865E+01_JPRB, 0.11736E+01_JPRB, 0.10688E+01_JPRB, 0.12181E+01_JPRB ,&
&  0.11842E+01_JPRB, 0.11612E+01_JPRB, 0.11345E+01_JPRB, 0.11461E+01_JPRB, 0.11937E+01_JPRB, 0.13540E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.77158E+00_JPRB, 0.72428E+00_JPRB, 0.72016E+00_JPRB, 0.73678E+00_JPRB, 0.77685E+00_JPRB, 0.79325E+00_JPRB, 0.81381E+00_JPRB ,&
&  0.89564E+00_JPRB, 0.99149E+00_JPRB, 0.10881E+01_JPRB, 0.10621E+01_JPRB, 0.91435E+00_JPRB, 0.85829E+00_JPRB, 0.89049E+00_JPRB ,&
&  0.91040E+00_JPRB, 0.91827E+00_JPRB, 0.90928E+00_JPRB, 0.91008E+00_JPRB, 0.89808E+00_JPRB, 0.89132E+00_JPRB, 0.88360E+00_JPRB ,&
&  0.87162E+00_JPRB, 0.86790E+00_JPRB, 0.85604E+00_JPRB, 0.82296E+00_JPRB, 0.80082E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.85481E+00_JPRB, 0.82976E+00_JPRB, 0.81149E+00_JPRB, 0.79703E+00_JPRB, 0.79006E+00_JPRB, 0.76997E+00_JPRB, 0.74193E+00_JPRB ,&
&  0.72863E+00_JPRB, 0.72624E+00_JPRB, 0.74314E+00_JPRB, 0.77773E+00_JPRB, 0.82632E+00_JPRB, 0.87465E+00_JPRB, 0.96003E+00_JPRB ,&
&  0.10646E+01_JPRB, 0.12148E+01_JPRB, 0.14016E+01_JPRB, 0.16666E+01_JPRB, 0.18848E+01_JPRB, 0.20953E+01_JPRB, 0.21617E+01_JPRB ,&
&  0.22808E+01_JPRB, 0.23180E+01_JPRB, 0.22108E+01_JPRB, 0.20734E+01_JPRB, 0.16321E+01_JPRB, 0.16085E+01_JPRB, 0.17472E+01_JPRB ,&
&  0.18211E+01_JPRB, 0.15076E+01_JPRB, 0.12739E+01_JPRB, 0.12503E+01_JPRB, 0.12376E+01_JPRB, 0.12674E+01_JPRB, 0.13011E+01_JPRB ,&
&  0.13360E+01_JPRB, 0.13679E+01_JPRB, 0.13984E+01_JPRB, 0.14464E+01_JPRB, 0.14972E+01_JPRB, 0.15170E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.15486E+01_JPRB, 0.15529E+01_JPRB, 0.15514E+01_JPRB, 0.15272E+01_JPRB, 0.15299E+01_JPRB, 0.15516E+01_JPRB, 0.15853E+01_JPRB ,&
&  0.16196E+01_JPRB, 0.15503E+01_JPRB, 0.16216E+01_JPRB, 0.17115E+01_JPRB, 0.16891E+01_JPRB, 0.16205E+01_JPRB, 0.16320E+01_JPRB ,&
&  0.17910E+01_JPRB, 0.19265E+01_JPRB, 0.12072E+01_JPRB, 0.13355E+01_JPRB, 0.15470E+01_JPRB, 0.13542E+01_JPRB, 0.12614E+01_JPRB ,&
&  0.12321E+01_JPRB, 0.12009E+01_JPRB, 0.11781E+01_JPRB, 0.11780E+01_JPRB, 0.12626E+01_JPRB, 0.14528E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.90877E+00_JPRB, 0.86596E+00_JPRB, 0.88177E+00_JPRB, 0.90348E+00_JPRB, 0.90211E+00_JPRB, 0.83320E+00_JPRB, 0.77013E+00_JPRB ,&
&  0.78415E+00_JPRB, 0.11347E+01_JPRB, 0.13484E+01_JPRB, 0.15344E+01_JPRB, 0.14080E+01_JPRB, 0.89084E+00_JPRB, 0.96992E+00_JPRB ,&
&  0.99635E+00_JPRB, 0.10001E+01_JPRB, 0.96850E+00_JPRB, 0.95770E+00_JPRB, 0.94861E+00_JPRB, 0.93633E+00_JPRB, 0.93337E+00_JPRB ,&
&  0.92647E+00_JPRB, 0.92864E+00_JPRB, 0.93409E+00_JPRB, 0.91315E+00_JPRB, 0.88060E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.92744E+00_JPRB, 0.90799E+00_JPRB, 0.88754E+00_JPRB, 0.86786E+00_JPRB, 0.85075E+00_JPRB, 0.80736E+00_JPRB, 0.76017E+00_JPRB ,&
&  0.73038E+00_JPRB, 0.71702E+00_JPRB, 0.72341E+00_JPRB, 0.73843E+00_JPRB, 0.77190E+00_JPRB, 0.80817E+00_JPRB, 0.86728E+00_JPRB ,&
&  0.95659E+00_JPRB, 0.10852E+01_JPRB, 0.12582E+01_JPRB, 0.14672E+01_JPRB, 0.17249E+01_JPRB, 0.20268E+01_JPRB, 0.22415E+01_JPRB ,&
&  0.24486E+01_JPRB, 0.25015E+01_JPRB, 0.22041E+01_JPRB, 0.16143E+01_JPRB, 0.15124E+01_JPRB, 0.15865E+01_JPRB, 0.13673E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.12348E+01_JPRB, 0.11759E+01_JPRB, 0.11528E+01_JPRB, 0.11567E+01_JPRB, 0.11858E+01_JPRB, 0.12138E+01_JPRB ,&
&  0.12377E+01_JPRB, 0.12558E+01_JPRB, 0.12802E+01_JPRB, 0.13342E+01_JPRB, 0.13731E+01_JPRB, 0.14199E+01_JPRB, 0.14660E+01_JPRB ,&
&  0.14898E+01_JPRB, 0.15006E+01_JPRB, 0.14706E+01_JPRB, 0.14347E+01_JPRB, 0.13917E+01_JPRB, 0.13962E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.14148E+01_JPRB, 0.14149E+01_JPRB, 0.14776E+01_JPRB, 0.15177E+01_JPRB, 0.13769E+01_JPRB, 0.14217E+01_JPRB ,&
&  0.16226E+01_JPRB, 0.17598E+01_JPRB, 0.14224E+01_JPRB, 0.16424E+01_JPRB, 0.10561E+01_JPRB, 0.12560E+01_JPRB, 0.13888E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12360E+01_JPRB, 0.11768E+01_JPRB, 0.11962E+01_JPRB, 0.12847E+01_JPRB, 0.13140E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.98868E+00_JPRB, 0.10315E+01_JPRB, 0.10847E+01_JPRB, 0.10916E+01_JPRB, 0.10083E+01_JPRB, 0.88005E+00_JPRB ,&
&  0.82216E+00_JPRB, 0.10701E+01_JPRB, 0.11660E+01_JPRB, 0.12458E+01_JPRB, 0.10433E+01_JPRB, 0.88217E+00_JPRB, 0.93590E+00_JPRB ,&
&  0.98064E+00_JPRB, 0.10018E+01_JPRB, 0.99871E+00_JPRB, 0.98201E+00_JPRB, 0.95828E+00_JPRB, 0.93786E+00_JPRB, 0.92911E+00_JPRB ,&
&  0.92649E+00_JPRB, 0.92163E+00_JPRB, 0.92613E+00_JPRB, 0.93517E+00_JPRB, 0.93907E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.93864E+00_JPRB, 0.93449E+00_JPRB, 0.93360E+00_JPRB, 0.93403E+00_JPRB, 0.92752E+00_JPRB, 0.89922E+00_JPRB, 0.83798E+00_JPRB ,&
&  0.75096E+00_JPRB, 0.68457E+00_JPRB, 0.67484E+00_JPRB, 0.67640E+00_JPRB, 0.68836E+00_JPRB, 0.72416E+00_JPRB, 0.78423E+00_JPRB ,&
&  0.85038E+00_JPRB, 0.95745E+00_JPRB, 0.11094E+01_JPRB, 0.13091E+01_JPRB, 0.15578E+01_JPRB, 0.19333E+01_JPRB, 0.23544E+01_JPRB ,&
&  0.22969E+01_JPRB, 0.17340E+01_JPRB, 0.21579E+01_JPRB, 0.21944E+01_JPRB, 0.20372E+01_JPRB, 0.20078E+01_JPRB, 0.17560E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.12481E+01_JPRB, 0.12186E+01_JPRB, 0.11612E+01_JPRB, 0.10928E+01_JPRB, 0.11126E+01_JPRB, 0.11345E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.11833E+01_JPRB, 0.11932E+01_JPRB, 0.12442E+01_JPRB, 0.12720E+01_JPRB, 0.13096E+01_JPRB, 0.13672E+01_JPRB ,&
&  0.13961E+01_JPRB, 0.13960E+01_JPRB, 0.13775E+01_JPRB, 0.12975E+01_JPRB, 0.12878E+01_JPRB, 0.13028E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.13278E+01_JPRB, 0.13224E+01_JPRB, 0.13673E+01_JPRB, 0.14264E+01_JPRB, 0.14946E+01_JPRB, 0.13671E+01_JPRB, 0.14656E+01_JPRB ,&
&  0.15366E+01_JPRB, 0.16986E+01_JPRB, 0.16356E+01_JPRB, 0.18103E+01_JPRB, 0.12704E+01_JPRB, 0.13930E+01_JPRB, 0.13340E+01_JPRB ,&
&  0.14274E+01_JPRB, 0.13055E+01_JPRB, 0.12113E+01_JPRB, 0.11878E+01_JPRB, 0.12575E+01_JPRB, 0.13613E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.10782E+01_JPRB, 0.11122E+01_JPRB, 0.12025E+01_JPRB, 0.12086E+01_JPRB, 0.11733E+01_JPRB, 0.11113E+01_JPRB ,&
&  0.12311E+01_JPRB, 0.12185E+01_JPRB, 0.12210E+01_JPRB, 0.11774E+01_JPRB, 0.10128E+01_JPRB, 0.91119E+00_JPRB, 0.91802E+00_JPRB ,&
&  0.94144E+00_JPRB, 0.96038E+00_JPRB, 0.96114E+00_JPRB, 0.95038E+00_JPRB, 0.92477E+00_JPRB, 0.90110E+00_JPRB, 0.88767E+00_JPRB ,&
&  0.87835E+00_JPRB, 0.88428E+00_JPRB, 0.89272E+00_JPRB, 0.90345E+00_JPRB, 0.92241E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.86830E+00_JPRB, 0.88578E+00_JPRB, 0.89951E+00_JPRB, 0.93877E+00_JPRB, 0.99064E+00_JPRB, 0.10604E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.95181E+00_JPRB, 0.66870E+00_JPRB, 0.65499E+00_JPRB, 0.64613E+00_JPRB, 0.64152E+00_JPRB, 0.65299E+00_JPRB, 0.70227E+00_JPRB ,&
&  0.77723E+00_JPRB, 0.87732E+00_JPRB, 0.98788E+00_JPRB, 0.11334E+01_JPRB, 0.13271E+01_JPRB, 0.16932E+01_JPRB, 0.19815E+01_JPRB ,&
&  0.17717E+01_JPRB, 0.21096E+01_JPRB, 0.26498E+01_JPRB, 0.23978E+01_JPRB, 0.21559E+01_JPRB, 0.20671E+01_JPRB, 0.21755E+01_JPRB ,&
&  0.15670E+01_JPRB, 0.13411E+01_JPRB, 0.13178E+01_JPRB, 0.12793E+01_JPRB, 0.11169E+01_JPRB, 0.10967E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.11173E+01_JPRB, 0.11302E+01_JPRB, 0.11883E+01_JPRB, 0.12405E+01_JPRB, 0.12753E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.13343E+01_JPRB, 0.13202E+01_JPRB, 0.12370E+01_JPRB, 0.12555E+01_JPRB, 0.12877E+01_JPRB, 0.13244E+01_JPRB ,&
&  0.13654E+01_JPRB, 0.14535E+01_JPRB, 0.14996E+01_JPRB, 0.14558E+01_JPRB, 0.13553E+01_JPRB, 0.14926E+01_JPRB, 0.14286E+01_JPRB ,&
&  0.14911E+01_JPRB, 0.15742E+01_JPRB, 0.19513E+01_JPRB, 0.14647E+01_JPRB, 0.15230E+01_JPRB, 0.16304E+01_JPRB, 0.16214E+01_JPRB ,&
&  0.15035E+01_JPRB, 0.14863E+01_JPRB, 0.13353E+01_JPRB, 0.12193E+01_JPRB, 0.12261E+01_JPRB, 0.12714E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.13514E+01_JPRB, 0.12261E+01_JPRB, 0.12195E+01_JPRB, 0.11917E+01_JPRB, 0.11579E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.15437E+01_JPRB, 0.15235E+01_JPRB, 0.14438E+01_JPRB, 0.13529E+01_JPRB, 0.11958E+01_JPRB, 0.11155E+01_JPRB, 0.10319E+01_JPRB ,&
&  0.99396E+00_JPRB, 0.97978E+00_JPRB, 0.97453E+00_JPRB, 0.94128E+00_JPRB, 0.91461E+00_JPRB, 0.88049E+00_JPRB, 0.85576E+00_JPRB ,&
&  0.83979E+00_JPRB, 0.83303E+00_JPRB, 0.83140E+00_JPRB, 0.83916E+00_JPRB, 0.84995E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.82579E+00_JPRB, 0.82854E+00_JPRB, 0.82450E+00_JPRB, 0.79787E+00_JPRB, 0.77920E+00_JPRB, 0.76498E+00_JPRB, 0.73053E+00_JPRB ,&
&  0.70443E+00_JPRB, 0.68726E+00_JPRB, 0.66617E+00_JPRB, 0.64839E+00_JPRB, 0.64150E+00_JPRB, 0.63416E+00_JPRB, 0.64833E+00_JPRB ,&
&  0.69960E+00_JPRB, 0.78318E+00_JPRB, 0.91511E+00_JPRB, 0.10140E+01_JPRB, 0.11445E+01_JPRB, 0.13758E+01_JPRB, 0.14925E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.28170E+01_JPRB, 0.26551E+01_JPRB, 0.26139E+01_JPRB, 0.24991E+01_JPRB, 0.23191E+01_JPRB, 0.21814E+01_JPRB ,&
&  0.16855E+01_JPRB, 0.14313E+01_JPRB, 0.13389E+01_JPRB, 0.12612E+01_JPRB, 0.12238E+01_JPRB, 0.11426E+01_JPRB, 0.11232E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11057E+01_JPRB, 0.10973E+01_JPRB, 0.11397E+01_JPRB, 0.12077E+01_JPRB, 0.12559E+01_JPRB, 0.13049E+01_JPRB ,&
&  0.13313E+01_JPRB, 0.13420E+01_JPRB, 0.13148E+01_JPRB, 0.13039E+01_JPRB, 0.13277E+01_JPRB, 0.13946E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.14533E+01_JPRB, 0.16057E+01_JPRB, 0.16462E+01_JPRB, 0.16216E+01_JPRB, 0.16787E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.15977E+01_JPRB, 0.15115E+01_JPRB, 0.18322E+01_JPRB, 0.15842E+01_JPRB, 0.16560E+01_JPRB, 0.17988E+01_JPRB, 0.17779E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.16362E+01_JPRB, 0.15032E+01_JPRB, 0.13782E+01_JPRB, 0.12879E+01_JPRB, 0.12951E+01_JPRB, 0.14412E+01_JPRB ,&
&  0.14846E+01_JPRB, 0.14736E+01_JPRB, 0.15159E+01_JPRB, 0.13241E+01_JPRB, 0.12849E+01_JPRB, 0.12356E+01_JPRB, 0.13554E+01_JPRB ,&
&  0.18333E+01_JPRB, 0.18892E+01_JPRB, 0.18317E+01_JPRB, 0.16665E+01_JPRB, 0.14427E+01_JPRB, 0.13704E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.11653E+01_JPRB, 0.11002E+01_JPRB, 0.10540E+01_JPRB, 0.10059E+01_JPRB, 0.94907E+00_JPRB, 0.89566E+00_JPRB, 0.86260E+00_JPRB ,&
&  0.83794E+00_JPRB, 0.82530E+00_JPRB, 0.82369E+00_JPRB, 0.82134E+00_JPRB, 0.82209E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.82858E+00_JPRB, 0.82176E+00_JPRB, 0.82859E+00_JPRB, 0.81055E+00_JPRB, 0.78612E+00_JPRB, 0.76565E+00_JPRB, 0.75621E+00_JPRB ,&
&  0.73992E+00_JPRB, 0.71208E+00_JPRB, 0.68675E+00_JPRB, 0.66628E+00_JPRB, 0.64791E+00_JPRB, 0.64141E+00_JPRB, 0.64659E+00_JPRB ,&
&  0.67063E+00_JPRB, 0.70565E+00_JPRB, 0.78887E+00_JPRB, 0.89939E+00_JPRB, 0.10049E+01_JPRB, 0.11766E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.25837E+01_JPRB, 0.27013E+01_JPRB, 0.27517E+01_JPRB, 0.27245E+01_JPRB, 0.26651E+01_JPRB, 0.24015E+01_JPRB ,&
&  0.19805E+01_JPRB, 0.16699E+01_JPRB, 0.14530E+01_JPRB, 0.13397E+01_JPRB, 0.12891E+01_JPRB, 0.12484E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11136E+01_JPRB, 0.11058E+01_JPRB, 0.11445E+01_JPRB, 0.12071E+01_JPRB, 0.12873E+01_JPRB, 0.13367E+01_JPRB ,&
&  0.13583E+01_JPRB, 0.13737E+01_JPRB, 0.14049E+01_JPRB, 0.13508E+01_JPRB, 0.14308E+01_JPRB, 0.14975E+01_JPRB, 0.15640E+01_JPRB ,&
&  0.15139E+01_JPRB, 0.14665E+01_JPRB, 0.17124E+01_JPRB, 0.19274E+01_JPRB, 0.20096E+01_JPRB, 0.19116E+01_JPRB, 0.17510E+01_JPRB ,&
&  0.17302E+01_JPRB, 0.16967E+01_JPRB, 0.17795E+01_JPRB, 0.17204E+01_JPRB, 0.18097E+01_JPRB, 0.19349E+01_JPRB, 0.20185E+01_JPRB ,&
&  0.19769E+01_JPRB, 0.18116E+01_JPRB, 0.16616E+01_JPRB, 0.14944E+01_JPRB, 0.13871E+01_JPRB, 0.13010E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.16861E+01_JPRB, 0.18413E+01_JPRB, 0.18845E+01_JPRB, 0.16451E+01_JPRB, 0.14688E+01_JPRB, 0.13508E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.19587E+01_JPRB, 0.21088E+01_JPRB, 0.19039E+01_JPRB, 0.18873E+01_JPRB, 0.15770E+01_JPRB, 0.15000E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.11844E+01_JPRB, 0.11024E+01_JPRB, 0.10424E+01_JPRB, 0.10071E+01_JPRB, 0.95547E+00_JPRB, 0.91559E+00_JPRB ,&
&  0.88595E+00_JPRB, 0.87433E+00_JPRB, 0.86493E+00_JPRB, 0.85532E+00_JPRB, 0.83966E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.87297E+00_JPRB, 0.85161E+00_JPRB, 0.83589E+00_JPRB, 0.83188E+00_JPRB, 0.82722E+00_JPRB, 0.80767E+00_JPRB, 0.79079E+00_JPRB ,&
&  0.77196E+00_JPRB, 0.73975E+00_JPRB, 0.71244E+00_JPRB, 0.68368E+00_JPRB, 0.66261E+00_JPRB, 0.65065E+00_JPRB, 0.65077E+00_JPRB ,&
&  0.66585E+00_JPRB, 0.68772E+00_JPRB, 0.76490E+00_JPRB, 0.84661E+00_JPRB, 0.91558E+00_JPRB, 0.87455E+00_JPRB, 0.10092E+01_JPRB ,&
&  0.17669E+01_JPRB, 0.25458E+01_JPRB, 0.25300E+01_JPRB, 0.26590E+01_JPRB, 0.28215E+01_JPRB, 0.30059E+01_JPRB, 0.26298E+01_JPRB ,&
&  0.22476E+01_JPRB, 0.19261E+01_JPRB, 0.16633E+01_JPRB, 0.15002E+01_JPRB, 0.13863E+01_JPRB, 0.13169E+01_JPRB, 0.12676E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.11561E+01_JPRB, 0.11265E+01_JPRB, 0.11547E+01_JPRB, 0.12425E+01_JPRB, 0.13245E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.13881E+01_JPRB, 0.13875E+01_JPRB, 0.14262E+01_JPRB, 0.14244E+01_JPRB, 0.14843E+01_JPRB, 0.15329E+01_JPRB, 0.16173E+01_JPRB ,&
&  0.16528E+01_JPRB, 0.17818E+01_JPRB, 0.19238E+01_JPRB, 0.20008E+01_JPRB, 0.21146E+01_JPRB, 0.21944E+01_JPRB, 0.21012E+01_JPRB ,&
&  0.19694E+01_JPRB, 0.17520E+01_JPRB, 0.15656E+01_JPRB, 0.17320E+01_JPRB, 0.18863E+01_JPRB, 0.20877E+01_JPRB, 0.19902E+01_JPRB ,&
&  0.17416E+01_JPRB, 0.14705E+01_JPRB, 0.14553E+01_JPRB, 0.15235E+01_JPRB, 0.15910E+01_JPRB, 0.14570E+01_JPRB, 0.17014E+01_JPRB ,&
&  0.15954E+01_JPRB, 0.11635E+01_JPRB, 0.10275E+01_JPRB, 0.12357E+01_JPRB, 0.10924E+01_JPRB, 0.10558E+01_JPRB, 0.15149E+01_JPRB ,&
&  0.19599E+01_JPRB, 0.25227E+01_JPRB, 0.23358E+01_JPRB, 0.18445E+01_JPRB, 0.17516E+01_JPRB, 0.16125E+01_JPRB, 0.14960E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.12218E+01_JPRB, 0.11528E+01_JPRB, 0.10751E+01_JPRB, 0.10146E+01_JPRB, 0.98727E+00_JPRB, 0.97223E+00_JPRB ,&
&  0.94770E+00_JPRB, 0.92991E+00_JPRB, 0.92226E+00_JPRB, 0.91475E+00_JPRB, 0.89605E+00_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.92526E+00_JPRB, 0.89865E+00_JPRB, 0.87757E+00_JPRB, 0.85432E+00_JPRB, 0.83272E+00_JPRB, 0.81355E+00_JPRB, 0.79522E+00_JPRB ,&
&  0.78137E+00_JPRB, 0.75144E+00_JPRB, 0.72008E+00_JPRB, 0.69087E+00_JPRB, 0.66781E+00_JPRB, 0.65471E+00_JPRB, 0.65579E+00_JPRB ,&
&  0.65897E+00_JPRB, 0.68699E+00_JPRB, 0.80432E+00_JPRB, 0.87310E+00_JPRB, 0.87721E+00_JPRB, 0.80214E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.14779E+01_JPRB, 0.21236E+01_JPRB, 0.26781E+01_JPRB, 0.28872E+01_JPRB, 0.30337E+01_JPRB, 0.29747E+01_JPRB, 0.27180E+01_JPRB ,&
&  0.24293E+01_JPRB, 0.21844E+01_JPRB, 0.19479E+01_JPRB, 0.17294E+01_JPRB, 0.15364E+01_JPRB, 0.14133E+01_JPRB, 0.13279E+01_JPRB ,&
&  0.12811E+01_JPRB, 0.11958E+01_JPRB, 0.11317E+01_JPRB, 0.11405E+01_JPRB, 0.12174E+01_JPRB, 0.13163E+01_JPRB, 0.13703E+01_JPRB ,&
&  0.13796E+01_JPRB, 0.13726E+01_JPRB, 0.14258E+01_JPRB, 0.15151E+01_JPRB, 0.12784E+01_JPRB, 0.15028E+01_JPRB, 0.15429E+01_JPRB ,&
&  0.16463E+01_JPRB, 0.17811E+01_JPRB, 0.18093E+01_JPRB, 0.21574E+01_JPRB, 0.23170E+01_JPRB, 0.23451E+01_JPRB, 0.23686E+01_JPRB ,&
&  0.23271E+01_JPRB, 0.19946E+01_JPRB, 0.16894E+01_JPRB, 0.18224E+01_JPRB, 0.19747E+01_JPRB, 0.22667E+01_JPRB, 0.15632E+01_JPRB ,&
&  0.14657E+01_JPRB, 0.17774E+01_JPRB, 0.16990E+01_JPRB, 0.14861E+01_JPRB, 0.17043E+01_JPRB, 0.16620E+01_JPRB, 0.12982E+01_JPRB ,&
&  0.69211E+00_JPRB, 0.56327E+00_JPRB, 0.58748E+00_JPRB, 0.59739E+00_JPRB, 0.58255E+00_JPRB, 0.80860E+00_JPRB, 0.23043E+01_JPRB ,&
&  0.20408E+01_JPRB, 0.27309E+01_JPRB, 0.25631E+01_JPRB, 0.22045E+01_JPRB, 0.18852E+01_JPRB, 0.17988E+01_JPRB, 0.18026E+01_JPRB ,&
&  0.15307E+01_JPRB, 0.14039E+01_JPRB, 0.12648E+01_JPRB, 0.11585E+01_JPRB, 0.11032E+01_JPRB, 0.10631E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.10089E+01_JPRB, 0.98382E+00_JPRB, 0.96990E+00_JPRB, 0.96293E+00_JPRB, 0.95052E+00_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.97216E+00_JPRB, 0.95537E+00_JPRB, 0.93426E+00_JPRB, 0.90506E+00_JPRB, 0.86495E+00_JPRB, 0.82529E+00_JPRB, 0.79445E+00_JPRB ,&
&  0.77647E+00_JPRB, 0.74402E+00_JPRB, 0.71178E+00_JPRB, 0.68546E+00_JPRB, 0.66481E+00_JPRB, 0.65348E+00_JPRB, 0.65353E+00_JPRB ,&
&  0.66206E+00_JPRB, 0.70522E+00_JPRB, 0.94058E+00_JPRB, 0.89455E+00_JPRB, 0.90491E+00_JPRB, 0.85785E+00_JPRB, 0.92947E+00_JPRB ,&
&  0.12764E+01_JPRB, 0.18002E+01_JPRB, 0.24011E+01_JPRB, 0.27758E+01_JPRB, 0.32161E+01_JPRB, 0.33231E+01_JPRB, 0.31427E+01_JPRB ,&
&  0.27250E+01_JPRB, 0.23470E+01_JPRB, 0.20305E+01_JPRB, 0.17846E+01_JPRB, 0.15562E+01_JPRB, 0.14022E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.12537E+01_JPRB, 0.12260E+01_JPRB, 0.11496E+01_JPRB, 0.11397E+01_JPRB, 0.12195E+01_JPRB, 0.13161E+01_JPRB, 0.13631E+01_JPRB ,&
&  0.13538E+01_JPRB, 0.13368E+01_JPRB, 0.13705E+01_JPRB, 0.14556E+01_JPRB, 0.12951E+01_JPRB, 0.11722E+01_JPRB, 0.13815E+01_JPRB ,&
&  0.16146E+01_JPRB, 0.19146E+01_JPRB, 0.19578E+01_JPRB, 0.23585E+01_JPRB, 0.24593E+01_JPRB, 0.24743E+01_JPRB, 0.27776E+01_JPRB ,&
&  0.26917E+01_JPRB, 0.26774E+01_JPRB, 0.19157E+01_JPRB, 0.19350E+01_JPRB, 0.21457E+01_JPRB, 0.17641E+01_JPRB, 0.17327E+01_JPRB ,&
&  0.20373E+01_JPRB, 0.18589E+01_JPRB, 0.16742E+01_JPRB, 0.14364E+01_JPRB, 0.17566E+01_JPRB, 0.14575E+01_JPRB, 0.76550E+00_JPRB ,&
&  0.64346E+00_JPRB, 0.58374E+00_JPRB, 0.54295E+00_JPRB, 0.50608E+00_JPRB, 0.50329E+00_JPRB, 0.75083E+00_JPRB, 0.15456E+01_JPRB ,&
&  0.17484E+01_JPRB, 0.25366E+01_JPRB, 0.25665E+01_JPRB, 0.23127E+01_JPRB, 0.19522E+01_JPRB, 0.19565E+01_JPRB, 0.19261E+01_JPRB ,&
&  0.17025E+01_JPRB, 0.16505E+01_JPRB, 0.14942E+01_JPRB, 0.13589E+01_JPRB, 0.12407E+01_JPRB, 0.11617E+01_JPRB, 0.11207E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10561E+01_JPRB, 0.10340E+01_JPRB, 0.10121E+01_JPRB, 0.99245E+00_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.99463E+00_JPRB, 0.97179E+00_JPRB, 0.95579E+00_JPRB, 0.93255E+00_JPRB, 0.89627E+00_JPRB, 0.84962E+00_JPRB, 0.80539E+00_JPRB ,&
&  0.77483E+00_JPRB, 0.73476E+00_JPRB, 0.70383E+00_JPRB, 0.67920E+00_JPRB, 0.65872E+00_JPRB, 0.64949E+00_JPRB, 0.64917E+00_JPRB ,&
&  0.65517E+00_JPRB, 0.75784E+00_JPRB, 0.93939E+00_JPRB, 0.81291E+00_JPRB, 0.75320E+00_JPRB, 0.70612E+00_JPRB, 0.71297E+00_JPRB ,&
&  0.11104E+01_JPRB, 0.15774E+01_JPRB, 0.20717E+01_JPRB, 0.27319E+01_JPRB, 0.32567E+01_JPRB, 0.31554E+01_JPRB, 0.33073E+01_JPRB ,&
&  0.29870E+01_JPRB, 0.25176E+01_JPRB, 0.21377E+01_JPRB, 0.18505E+01_JPRB, 0.16066E+01_JPRB, 0.14337E+01_JPRB, 0.13141E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.12225E+01_JPRB, 0.11704E+01_JPRB, 0.11431E+01_JPRB, 0.12084E+01_JPRB, 0.13420E+01_JPRB, 0.14106E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13099E+01_JPRB, 0.13112E+01_JPRB, 0.13697E+01_JPRB, 0.14284E+01_JPRB, 0.14385E+01_JPRB, 0.13453E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.15265E+01_JPRB, 0.20827E+01_JPRB, 0.22570E+01_JPRB, 0.24733E+01_JPRB, 0.27172E+01_JPRB, 0.29182E+01_JPRB ,&
&  0.28142E+01_JPRB, 0.27615E+01_JPRB, 0.22177E+01_JPRB, 0.22166E+01_JPRB, 0.19257E+01_JPRB, 0.16358E+01_JPRB, 0.21095E+01_JPRB ,&
&  0.22690E+01_JPRB, 0.21734E+01_JPRB, 0.17895E+01_JPRB, 0.13936E+01_JPRB, 0.12494E+01_JPRB, 0.88324E+00_JPRB, 0.61538E+00_JPRB ,&
&  0.59623E+00_JPRB, 0.57379E+00_JPRB, 0.51900E+00_JPRB, 0.52679E+00_JPRB, 0.51969E+00_JPRB, 0.72405E+00_JPRB, 0.11874E+01_JPRB ,&
&  0.17889E+01_JPRB, 0.21071E+01_JPRB, 0.22263E+01_JPRB, 0.20501E+01_JPRB, 0.18223E+01_JPRB, 0.18826E+01_JPRB, 0.19294E+01_JPRB ,&
&  0.17540E+01_JPRB, 0.17904E+01_JPRB, 0.16750E+01_JPRB, 0.15077E+01_JPRB, 0.13983E+01_JPRB, 0.13074E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.11418E+01_JPRB, 0.11046E+01_JPRB, 0.10611E+01_JPRB, 0.10264E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.99858E+00_JPRB, 0.96648E+00_JPRB, 0.94734E+00_JPRB, 0.92849E+00_JPRB, 0.90390E+00_JPRB, 0.86658E+00_JPRB, 0.81978E+00_JPRB ,&
&  0.77609E+00_JPRB, 0.73606E+00_JPRB, 0.70589E+00_JPRB, 0.68143E+00_JPRB, 0.65787E+00_JPRB, 0.65219E+00_JPRB, 0.65425E+00_JPRB ,&
&  0.67741E+00_JPRB, 0.92663E+00_JPRB, 0.70474E+00_JPRB, 0.58739E+00_JPRB, 0.64920E+00_JPRB, 0.66400E+00_JPRB, 0.62561E+00_JPRB ,&
&  0.98290E+00_JPRB, 0.14670E+01_JPRB, 0.18675E+01_JPRB, 0.27298E+01_JPRB, 0.36491E+01_JPRB, 0.34478E+01_JPRB, 0.34042E+01_JPRB ,&
&  0.33419E+01_JPRB, 0.27314E+01_JPRB, 0.23027E+01_JPRB, 0.19821E+01_JPRB, 0.16770E+01_JPRB, 0.14614E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.11958E+01_JPRB, 0.11686E+01_JPRB, 0.11770E+01_JPRB, 0.12075E+01_JPRB, 0.13069E+01_JPRB, 0.14130E+01_JPRB ,&
&  0.13823E+01_JPRB, 0.13211E+01_JPRB, 0.13131E+01_JPRB, 0.13905E+01_JPRB, 0.14029E+01_JPRB, 0.14085E+01_JPRB, 0.17016E+01_JPRB ,&
&  0.16976E+01_JPRB, 0.19175E+01_JPRB, 0.20652E+01_JPRB, 0.25409E+01_JPRB, 0.28499E+01_JPRB, 0.28172E+01_JPRB, 0.31771E+01_JPRB ,&
&  0.23932E+01_JPRB, 0.21230E+01_JPRB, 0.21506E+01_JPRB, 0.20599E+01_JPRB, 0.17180E+01_JPRB, 0.20369E+01_JPRB, 0.31735E+01_JPRB ,&
&  0.24073E+01_JPRB, 0.26784E+01_JPRB, 0.27566E+01_JPRB, 0.27680E+01_JPRB, 0.13293E+01_JPRB, 0.86480E+00_JPRB, 0.14113E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.95533E+00_JPRB, 0.67850E+00_JPRB, 0.84019E+00_JPRB, 0.67973E+00_JPRB, 0.87960E+00_JPRB, 0.12798E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.16409E+01_JPRB, 0.21010E+01_JPRB, 0.18734E+01_JPRB, 0.17644E+01_JPRB, 0.19190E+01_JPRB, 0.18559E+01_JPRB ,&
&  0.18025E+01_JPRB, 0.17585E+01_JPRB, 0.17669E+01_JPRB, 0.15621E+01_JPRB, 0.14185E+01_JPRB, 0.13260E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.12412E+01_JPRB, 0.12006E+01_JPRB, 0.11513E+01_JPRB, 0.10956E+01_JPRB, 0.10394E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.96353E+00_JPRB, 0.92908E+00_JPRB, 0.90365E+00_JPRB, 0.89127E+00_JPRB, 0.88661E+00_JPRB, 0.87217E+00_JPRB, 0.83161E+00_JPRB ,&
&  0.78342E+00_JPRB, 0.74474E+00_JPRB, 0.70805E+00_JPRB, 0.68478E+00_JPRB, 0.66039E+00_JPRB, 0.65660E+00_JPRB, 0.66057E+00_JPRB ,&
&  0.67639E+00_JPRB, 0.71537E+00_JPRB, 0.54986E+00_JPRB, 0.51825E+00_JPRB, 0.66165E+00_JPRB, 0.64257E+00_JPRB, 0.66830E+00_JPRB ,&
&  0.95004E+00_JPRB, 0.12953E+01_JPRB, 0.17301E+01_JPRB, 0.25520E+01_JPRB, 0.29712E+01_JPRB, 0.33608E+01_JPRB, 0.33955E+01_JPRB ,&
&  0.34799E+01_JPRB, 0.29183E+01_JPRB, 0.23948E+01_JPRB, 0.19876E+01_JPRB, 0.16628E+01_JPRB, 0.14584E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.11886E+01_JPRB, 0.11203E+01_JPRB, 0.11328E+01_JPRB, 0.11380E+01_JPRB, 0.11888E+01_JPRB, 0.12457E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.13839E+01_JPRB, 0.13275E+01_JPRB, 0.12731E+01_JPRB, 0.13412E+01_JPRB, 0.12967E+01_JPRB, 0.12974E+01_JPRB, 0.14546E+01_JPRB ,&
&  0.17615E+01_JPRB, 0.20127E+01_JPRB, 0.22682E+01_JPRB, 0.26579E+01_JPRB, 0.31759E+01_JPRB, 0.29808E+01_JPRB, 0.36311E+01_JPRB ,&
&  0.29092E+01_JPRB, 0.25397E+01_JPRB, 0.22077E+01_JPRB, 0.18430E+01_JPRB, 0.21539E+01_JPRB, 0.35185E+01_JPRB, 0.34472E+01_JPRB ,&
&  0.31738E+01_JPRB, 0.32035E+01_JPRB, 0.31204E+01_JPRB, 0.26242E+01_JPRB, 0.21009E+01_JPRB, 0.15467E+01_JPRB, 0.18369E+01_JPRB ,&
&  0.17738E+01_JPRB, 0.17625E+01_JPRB, 0.15329E+01_JPRB, 0.12264E+01_JPRB, 0.10084E+01_JPRB, 0.12241E+01_JPRB, 0.12330E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.13215E+01_JPRB, 0.16752E+01_JPRB, 0.16815E+01_JPRB, 0.16414E+01_JPRB, 0.16941E+01_JPRB, 0.18365E+01_JPRB ,&
&  0.17206E+01_JPRB, 0.17347E+01_JPRB, 0.17090E+01_JPRB, 0.16077E+01_JPRB, 0.15122E+01_JPRB, 0.14139E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.11865E+01_JPRB, 0.11111E+01_JPRB, 0.10538E+01_JPRB, 0.10034E+01_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.89412E+00_JPRB, 0.85778E+00_JPRB, 0.83189E+00_JPRB, 0.81787E+00_JPRB, 0.81951E+00_JPRB, 0.82747E+00_JPRB, 0.81209E+00_JPRB ,&
&  0.77684E+00_JPRB, 0.74206E+00_JPRB, 0.71174E+00_JPRB, 0.69025E+00_JPRB, 0.67179E+00_JPRB, 0.66668E+00_JPRB, 0.66425E+00_JPRB ,&
&  0.67338E+00_JPRB, 0.61790E+00_JPRB, 0.47665E+00_JPRB, 0.50804E+00_JPRB, 0.59794E+00_JPRB, 0.58778E+00_JPRB, 0.72678E+00_JPRB ,&
&  0.89168E+00_JPRB, 0.11370E+01_JPRB, 0.14435E+01_JPRB, 0.20771E+01_JPRB, 0.25213E+01_JPRB, 0.28197E+01_JPRB, 0.29189E+01_JPRB ,&
&  0.27924E+01_JPRB, 0.26075E+01_JPRB, 0.23376E+01_JPRB, 0.20305E+01_JPRB, 0.16750E+01_JPRB, 0.15200E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.13127E+01_JPRB, 0.11911E+01_JPRB, 0.11197E+01_JPRB, 0.11216E+01_JPRB, 0.11496E+01_JPRB, 0.12053E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.13661E+01_JPRB, 0.13499E+01_JPRB, 0.12411E+01_JPRB, 0.12656E+01_JPRB, 0.11551E+01_JPRB, 0.11987E+01_JPRB, 0.13200E+01_JPRB ,&
&  0.20284E+01_JPRB, 0.23847E+01_JPRB, 0.25500E+01_JPRB, 0.31763E+01_JPRB, 0.28308E+01_JPRB, 0.30161E+01_JPRB, 0.39814E+01_JPRB ,&
&  0.41382E+01_JPRB, 0.41036E+01_JPRB, 0.42191E+01_JPRB, 0.32713E+01_JPRB, 0.26940E+01_JPRB, 0.38213E+01_JPRB, 0.36382E+01_JPRB ,&
&  0.33727E+01_JPRB, 0.32662E+01_JPRB, 0.29608E+01_JPRB, 0.30289E+01_JPRB, 0.23354E+01_JPRB, 0.19222E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.12135E+01_JPRB, 0.10823E+01_JPRB, 0.12089E+01_JPRB, 0.11017E+01_JPRB, 0.11037E+01_JPRB, 0.10480E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.11131E+01_JPRB, 0.11799E+01_JPRB, 0.13480E+01_JPRB, 0.15054E+01_JPRB, 0.14344E+01_JPRB, 0.16398E+01_JPRB ,&
&  0.17076E+01_JPRB, 0.16994E+01_JPRB, 0.15743E+01_JPRB, 0.15889E+01_JPRB, 0.15218E+01_JPRB, 0.14464E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.12564E+01_JPRB, 0.11608E+01_JPRB, 0.10778E+01_JPRB, 0.10041E+01_JPRB, 0.94289E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.79381E+00_JPRB, 0.76789E+00_JPRB, 0.75440E+00_JPRB, 0.74813E+00_JPRB, 0.74965E+00_JPRB, 0.76049E+00_JPRB, 0.75621E+00_JPRB ,&
&  0.74094E+00_JPRB, 0.71683E+00_JPRB, 0.68732E+00_JPRB, 0.66400E+00_JPRB, 0.65323E+00_JPRB, 0.65876E+00_JPRB, 0.66476E+00_JPRB ,&
&  0.65666E+00_JPRB, 0.67424E+00_JPRB, 0.53704E+00_JPRB, 0.47995E+00_JPRB, 0.47601E+00_JPRB, 0.58020E+00_JPRB, 0.75811E+00_JPRB ,&
&  0.89232E+00_JPRB, 0.99217E+00_JPRB, 0.12549E+01_JPRB, 0.16722E+01_JPRB, 0.19990E+01_JPRB, 0.21661E+01_JPRB, 0.23371E+01_JPRB ,&
&  0.27566E+01_JPRB, 0.21644E+01_JPRB, 0.20739E+01_JPRB, 0.18438E+01_JPRB, 0.16648E+01_JPRB, 0.15376E+01_JPRB, 0.14223E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.12332E+01_JPRB, 0.11259E+01_JPRB, 0.10700E+01_JPRB, 0.10505E+01_JPRB, 0.10605E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.12702E+01_JPRB, 0.12591E+01_JPRB, 0.12584E+01_JPRB, 0.13357E+01_JPRB, 0.14068E+01_JPRB, 0.15528E+01_JPRB ,&
&  0.16857E+01_JPRB, 0.18375E+01_JPRB, 0.28073E+01_JPRB, 0.32216E+01_JPRB, 0.36234E+01_JPRB, 0.38041E+01_JPRB, 0.39132E+01_JPRB ,&
&  0.41350E+01_JPRB, 0.40550E+01_JPRB, 0.40911E+01_JPRB, 0.36453E+01_JPRB, 0.41153E+01_JPRB, 0.40626E+01_JPRB, 0.37755E+01_JPRB ,&
&  0.34348E+01_JPRB, 0.32320E+01_JPRB, 0.28534E+01_JPRB, 0.26801E+01_JPRB, 0.26515E+01_JPRB, 0.26701E+01_JPRB, 0.20127E+01_JPRB ,&
&  0.14667E+01_JPRB, 0.15687E+01_JPRB, 0.12993E+01_JPRB, 0.11080E+01_JPRB, 0.94924E+00_JPRB, 0.99632E+00_JPRB, 0.10493E+01_JPRB ,&
&  0.10850E+01_JPRB, 0.11286E+01_JPRB, 0.11094E+01_JPRB, 0.12215E+01_JPRB, 0.13998E+01_JPRB, 0.14607E+01_JPRB, 0.15782E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.15301E+01_JPRB, 0.15012E+01_JPRB, 0.14300E+01_JPRB, 0.14240E+01_JPRB, 0.13780E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.10683E+01_JPRB, 0.97969E+00_JPRB, 0.90179E+00_JPRB, 0.83844E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.76470E+00_JPRB, 0.74080E+00_JPRB, 0.72425E+00_JPRB, 0.71536E+00_JPRB, 0.71477E+00_JPRB, 0.72685E+00_JPRB, 0.72885E+00_JPRB ,&
&  0.71710E+00_JPRB, 0.70388E+00_JPRB, 0.67879E+00_JPRB, 0.65694E+00_JPRB, 0.64345E+00_JPRB, 0.64687E+00_JPRB, 0.65274E+00_JPRB ,&
&  0.66499E+00_JPRB, 0.85716E+00_JPRB, 0.64626E+00_JPRB, 0.59468E+00_JPRB, 0.56028E+00_JPRB, 0.68350E+00_JPRB, 0.79654E+00_JPRB ,&
&  0.85800E+00_JPRB, 0.90260E+00_JPRB, 0.97817E+00_JPRB, 0.12877E+01_JPRB, 0.15856E+01_JPRB, 0.16171E+01_JPRB, 0.17979E+01_JPRB ,&
&  0.18481E+01_JPRB, 0.18289E+01_JPRB, 0.16212E+01_JPRB, 0.16414E+01_JPRB, 0.14401E+01_JPRB, 0.12440E+01_JPRB, 0.12877E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.11589E+01_JPRB, 0.10985E+01_JPRB, 0.10262E+01_JPRB, 0.99641E+00_JPRB, 0.97857E+00_JPRB, 0.98898E+00_JPRB ,&
&  0.10225E+01_JPRB, 0.10674E+01_JPRB, 0.11305E+01_JPRB, 0.11895E+01_JPRB, 0.12633E+01_JPRB, 0.14352E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.20426E+01_JPRB, 0.22212E+01_JPRB, 0.20549E+01_JPRB, 0.28162E+01_JPRB, 0.42076E+01_JPRB, 0.41218E+01_JPRB, 0.37637E+01_JPRB ,&
&  0.40237E+01_JPRB, 0.40428E+01_JPRB, 0.41300E+01_JPRB, 0.42602E+01_JPRB, 0.42465E+01_JPRB, 0.41031E+01_JPRB, 0.37940E+01_JPRB ,&
&  0.33547E+01_JPRB, 0.30446E+01_JPRB, 0.28090E+01_JPRB, 0.27522E+01_JPRB, 0.24643E+01_JPRB, 0.22804E+01_JPRB, 0.21313E+01_JPRB ,&
&  0.17942E+01_JPRB, 0.14598E+01_JPRB, 0.10075E+01_JPRB, 0.10652E+01_JPRB, 0.83599E+00_JPRB, 0.86849E+00_JPRB, 0.10181E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.11279E+01_JPRB, 0.11885E+01_JPRB, 0.11432E+01_JPRB, 0.13111E+01_JPRB, 0.13896E+01_JPRB, 0.14968E+01_JPRB ,&
&  0.14963E+01_JPRB, 0.12624E+01_JPRB, 0.13095E+01_JPRB, 0.12387E+01_JPRB, 0.12192E+01_JPRB, 0.11673E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.94890E+00_JPRB, 0.88254E+00_JPRB, 0.83086E+00_JPRB, 0.79449E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.69340E+00_JPRB, 0.67058E+00_JPRB, 0.64733E+00_JPRB, 0.64364E+00_JPRB, 0.65630E+00_JPRB, 0.67505E+00_JPRB, 0.68539E+00_JPRB ,&
&  0.67746E+00_JPRB, 0.67197E+00_JPRB, 0.65491E+00_JPRB, 0.63802E+00_JPRB, 0.62562E+00_JPRB, 0.61703E+00_JPRB, 0.62757E+00_JPRB ,&
&  0.70739E+00_JPRB, 0.90804E+00_JPRB, 0.72841E+00_JPRB, 0.60510E+00_JPRB, 0.71696E+00_JPRB, 0.73556E+00_JPRB, 0.81700E+00_JPRB ,&
&  0.79593E+00_JPRB, 0.88698E+00_JPRB, 0.90336E+00_JPRB, 0.98573E+00_JPRB, 0.12123E+01_JPRB, 0.13084E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.13550E+01_JPRB, 0.12944E+01_JPRB, 0.12978E+01_JPRB, 0.13286E+01_JPRB, 0.13457E+01_JPRB, 0.11861E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10236E+01_JPRB, 0.96490E+00_JPRB, 0.93774E+00_JPRB, 0.92690E+00_JPRB, 0.92853E+00_JPRB, 0.94893E+00_JPRB ,&
&  0.97448E+00_JPRB, 0.10004E+01_JPRB, 0.10248E+01_JPRB, 0.10922E+01_JPRB, 0.11849E+01_JPRB, 0.14386E+01_JPRB, 0.18741E+01_JPRB ,&
&  0.24459E+01_JPRB, 0.28495E+01_JPRB, 0.32405E+01_JPRB, 0.39531E+01_JPRB, 0.41038E+01_JPRB, 0.39934E+01_JPRB, 0.38789E+01_JPRB ,&
&  0.38105E+01_JPRB, 0.37155E+01_JPRB, 0.37693E+01_JPRB, 0.38036E+01_JPRB, 0.38678E+01_JPRB, 0.38797E+01_JPRB, 0.36316E+01_JPRB ,&
&  0.32295E+01_JPRB, 0.29583E+01_JPRB, 0.28331E+01_JPRB, 0.25891E+01_JPRB, 0.23824E+01_JPRB, 0.21556E+01_JPRB, 0.21060E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.12150E+01_JPRB, 0.93698E+00_JPRB, 0.11060E+01_JPRB, 0.92097E+00_JPRB, 0.97795E+00_JPRB, 0.11653E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11497E+01_JPRB, 0.11859E+01_JPRB, 0.11228E+01_JPRB, 0.11175E+01_JPRB, 0.13104E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.10949E+01_JPRB, 0.11182E+01_JPRB, 0.10750E+01_JPRB, 0.10402E+01_JPRB, 0.98363E+00_JPRB, 0.91206E+00_JPRB ,&
&  0.85180E+00_JPRB, 0.81475E+00_JPRB, 0.77364E+00_JPRB, 0.73927E+00_JPRB, 0.71510E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.64023E+00_JPRB, 0.62730E+00_JPRB, 0.60694E+00_JPRB, 0.60457E+00_JPRB, 0.62493E+00_JPRB, 0.64784E+00_JPRB, 0.66054E+00_JPRB ,&
&  0.65616E+00_JPRB, 0.65545E+00_JPRB, 0.64106E+00_JPRB, 0.62856E+00_JPRB, 0.61648E+00_JPRB, 0.59444E+00_JPRB, 0.61443E+00_JPRB ,&
&  0.52954E+00_JPRB, 0.69499E+00_JPRB, 0.58078E+00_JPRB, 0.59641E+00_JPRB, 0.81014E+00_JPRB, 0.74697E+00_JPRB, 0.76160E+00_JPRB ,&
&  0.74543E+00_JPRB, 0.78025E+00_JPRB, 0.79612E+00_JPRB, 0.85754E+00_JPRB, 0.10422E+01_JPRB, 0.11681E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10153E+01_JPRB, 0.10573E+01_JPRB, 0.10877E+01_JPRB, 0.10892E+01_JPRB, 0.11194E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.97527E+00_JPRB, 0.90255E+00_JPRB, 0.86825E+00_JPRB, 0.85136E+00_JPRB, 0.84308E+00_JPRB, 0.83437E+00_JPRB ,&
&  0.84127E+00_JPRB, 0.86770E+00_JPRB, 0.90547E+00_JPRB, 0.10076E+01_JPRB, 0.11177E+01_JPRB, 0.16379E+01_JPRB, 0.24097E+01_JPRB ,&
&  0.29935E+01_JPRB, 0.30638E+01_JPRB, 0.33900E+01_JPRB, 0.39904E+01_JPRB, 0.39416E+01_JPRB, 0.37438E+01_JPRB, 0.34780E+01_JPRB ,&
&  0.34103E+01_JPRB, 0.33609E+01_JPRB, 0.34008E+01_JPRB, 0.35244E+01_JPRB, 0.35217E+01_JPRB, 0.36564E+01_JPRB, 0.35161E+01_JPRB ,&
&  0.31357E+01_JPRB, 0.29604E+01_JPRB, 0.29050E+01_JPRB, 0.28443E+01_JPRB, 0.27034E+01_JPRB, 0.27076E+01_JPRB, 0.25439E+01_JPRB ,&
&  0.24427E+01_JPRB, 0.20860E+01_JPRB, 0.15364E+01_JPRB, 0.13827E+01_JPRB, 0.10733E+01_JPRB, 0.11135E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.12404E+01_JPRB, 0.11081E+01_JPRB, 0.10259E+01_JPRB, 0.99223E+00_JPRB, 0.11160E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.92021E+00_JPRB, 0.95822E+00_JPRB, 0.96525E+00_JPRB, 0.94759E+00_JPRB, 0.89678E+00_JPRB, 0.83405E+00_JPRB, 0.74441E+00_JPRB ,&
&  0.71012E+00_JPRB, 0.68286E+00_JPRB, 0.66967E+00_JPRB, 0.66118E+00_JPRB, 0.65161E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.62270E+00_JPRB, 0.60817E+00_JPRB, 0.58911E+00_JPRB, 0.58872E+00_JPRB, 0.61034E+00_JPRB, 0.63254E+00_JPRB, 0.64531E+00_JPRB ,&
&  0.64152E+00_JPRB, 0.63921E+00_JPRB, 0.63119E+00_JPRB, 0.62140E+00_JPRB, 0.60326E+00_JPRB, 0.58163E+00_JPRB, 0.52975E+00_JPRB ,&
&  0.47341E+00_JPRB, 0.58626E+00_JPRB, 0.55699E+00_JPRB, 0.58443E+00_JPRB, 0.67876E+00_JPRB, 0.65145E+00_JPRB, 0.63794E+00_JPRB ,&
&  0.65608E+00_JPRB, 0.67019E+00_JPRB, 0.72239E+00_JPRB, 0.78147E+00_JPRB, 0.87501E+00_JPRB, 0.10479E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.95902E+00_JPRB, 0.89873E+00_JPRB, 0.93138E+00_JPRB, 0.10037E+01_JPRB, 0.10752E+01_JPRB, 0.11333E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.98493E+00_JPRB, 0.88537E+00_JPRB, 0.81743E+00_JPRB, 0.77795E+00_JPRB, 0.75639E+00_JPRB, 0.74574E+00_JPRB ,&
&  0.74372E+00_JPRB, 0.76152E+00_JPRB, 0.80879E+00_JPRB, 0.88763E+00_JPRB, 0.10323E+01_JPRB, 0.14846E+01_JPRB, 0.18788E+01_JPRB ,&
&  0.20102E+01_JPRB, 0.21482E+01_JPRB, 0.23872E+01_JPRB, 0.26266E+01_JPRB, 0.28721E+01_JPRB, 0.29027E+01_JPRB, 0.29427E+01_JPRB ,&
&  0.30431E+01_JPRB, 0.30253E+01_JPRB, 0.33844E+01_JPRB, 0.34111E+01_JPRB, 0.33902E+01_JPRB, 0.34326E+01_JPRB, 0.32768E+01_JPRB ,&
&  0.31314E+01_JPRB, 0.29828E+01_JPRB, 0.29813E+01_JPRB, 0.28930E+01_JPRB, 0.27951E+01_JPRB, 0.27857E+01_JPRB, 0.26188E+01_JPRB ,&
&  0.25526E+01_JPRB, 0.24473E+01_JPRB, 0.20879E+01_JPRB, 0.18061E+01_JPRB, 0.15041E+01_JPRB, 0.15290E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.10393E+01_JPRB, 0.10130E+01_JPRB, 0.97134E+00_JPRB, 0.96746E+00_JPRB, 0.94540E+00_JPRB, 0.86619E+00_JPRB ,&
&  0.86129E+00_JPRB, 0.88306E+00_JPRB, 0.83017E+00_JPRB, 0.79840E+00_JPRB, 0.74544E+00_JPRB, 0.70085E+00_JPRB, 0.59799E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.59482E+00_JPRB, 0.61066E+00_JPRB, 0.62524E+00_JPRB, 0.62981E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.62651E+00_JPRB, 0.61270E+00_JPRB, 0.60395E+00_JPRB, 0.59871E+00_JPRB, 0.60407E+00_JPRB, 0.62495E+00_JPRB, 0.61011E+00_JPRB ,&
&  0.63380E+00_JPRB, 0.63702E+00_JPRB, 0.62531E+00_JPRB, 0.61269E+00_JPRB, 0.58581E+00_JPRB, 0.50552E+00_JPRB, 0.41289E+00_JPRB ,&
&  0.40439E+00_JPRB, 0.45824E+00_JPRB, 0.51254E+00_JPRB, 0.54101E+00_JPRB, 0.58369E+00_JPRB, 0.58625E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.58557E+00_JPRB, 0.59599E+00_JPRB, 0.63461E+00_JPRB, 0.70450E+00_JPRB, 0.77694E+00_JPRB, 0.88270E+00_JPRB, 0.98508E+00_JPRB ,&
&  0.91405E+00_JPRB, 0.88331E+00_JPRB, 0.92603E+00_JPRB, 0.90391E+00_JPRB, 0.10308E+01_JPRB, 0.10567E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10007E+01_JPRB, 0.94569E+00_JPRB, 0.87691E+00_JPRB, 0.83302E+00_JPRB, 0.78918E+00_JPRB, 0.74992E+00_JPRB, 0.72683E+00_JPRB ,&
&  0.72956E+00_JPRB, 0.74140E+00_JPRB, 0.77180E+00_JPRB, 0.82677E+00_JPRB, 0.89668E+00_JPRB, 0.11294E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.14893E+01_JPRB, 0.17580E+01_JPRB, 0.20550E+01_JPRB, 0.20149E+01_JPRB, 0.22223E+01_JPRB, 0.24122E+01_JPRB, 0.24743E+01_JPRB ,&
&  0.26817E+01_JPRB, 0.26868E+01_JPRB, 0.28158E+01_JPRB, 0.28776E+01_JPRB, 0.29218E+01_JPRB, 0.29176E+01_JPRB, 0.28562E+01_JPRB ,&
&  0.28362E+01_JPRB, 0.29987E+01_JPRB, 0.30521E+01_JPRB, 0.29403E+01_JPRB, 0.27139E+01_JPRB, 0.25518E+01_JPRB, 0.23687E+01_JPRB ,&
&  0.22786E+01_JPRB, 0.22508E+01_JPRB, 0.21617E+01_JPRB, 0.20601E+01_JPRB, 0.18978E+01_JPRB, 0.17042E+01_JPRB, 0.15410E+01_JPRB ,&
&  0.13706E+01_JPRB, 0.11429E+01_JPRB, 0.10197E+01_JPRB, 0.98356E+00_JPRB, 0.95894E+00_JPRB, 0.85739E+00_JPRB, 0.82396E+00_JPRB ,&
&  0.81374E+00_JPRB, 0.74985E+00_JPRB, 0.75640E+00_JPRB, 0.70733E+00_JPRB, 0.66755E+00_JPRB, 0.64376E+00_JPRB, 0.61597E+00_JPRB ,&
&  0.58234E+00_JPRB, 0.62382E+00_JPRB, 0.64354E+00_JPRB, 0.64806E+00_JPRB, 0.64067E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.63073E+00_JPRB, 0.62846E+00_JPRB, 0.62591E+00_JPRB, 0.61341E+00_JPRB, 0.60432E+00_JPRB, 0.58932E+00_JPRB, 0.59110E+00_JPRB ,&
&  0.61690E+00_JPRB, 0.57227E+00_JPRB, 0.61124E+00_JPRB, 0.49647E+00_JPRB, 0.43989E+00_JPRB, 0.41439E+00_JPRB, 0.39738E+00_JPRB ,&
&  0.40567E+00_JPRB, 0.44049E+00_JPRB, 0.47194E+00_JPRB, 0.49387E+00_JPRB, 0.53819E+00_JPRB, 0.54649E+00_JPRB, 0.54544E+00_JPRB ,&
&  0.55199E+00_JPRB, 0.56026E+00_JPRB, 0.60038E+00_JPRB, 0.62193E+00_JPRB, 0.67447E+00_JPRB, 0.74841E+00_JPRB, 0.85725E+00_JPRB ,&
&  0.82108E+00_JPRB, 0.83372E+00_JPRB, 0.92149E+00_JPRB, 0.95514E+00_JPRB, 0.98065E+00_JPRB, 0.95676E+00_JPRB, 0.90723E+00_JPRB ,&
&  0.86491E+00_JPRB, 0.70832E+00_JPRB, 0.72373E+00_JPRB, 0.69941E+00_JPRB, 0.67583E+00_JPRB, 0.64929E+00_JPRB, 0.61105E+00_JPRB ,&
&  0.58488E+00_JPRB, 0.58330E+00_JPRB, 0.60125E+00_JPRB, 0.63186E+00_JPRB, 0.68342E+00_JPRB, 0.79593E+00_JPRB, 0.92305E+00_JPRB ,&
&  0.11334E+01_JPRB, 0.11065E+01_JPRB, 0.14217E+01_JPRB, 0.15602E+01_JPRB, 0.17534E+01_JPRB, 0.20143E+01_JPRB, 0.22293E+01_JPRB ,&
&  0.25203E+01_JPRB, 0.24584E+01_JPRB, 0.24540E+01_JPRB, 0.24198E+01_JPRB, 0.23663E+01_JPRB, 0.23484E+01_JPRB, 0.23882E+01_JPRB ,&
&  0.24720E+01_JPRB, 0.24461E+01_JPRB, 0.26290E+01_JPRB, 0.25764E+01_JPRB, 0.24104E+01_JPRB, 0.22947E+01_JPRB, 0.21614E+01_JPRB ,&
&  0.20420E+01_JPRB, 0.19547E+01_JPRB, 0.18622E+01_JPRB, 0.16408E+01_JPRB, 0.15808E+01_JPRB, 0.15371E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.13378E+01_JPRB, 0.12400E+01_JPRB, 0.11154E+01_JPRB, 0.11214E+01_JPRB, 0.10201E+01_JPRB, 0.93467E+00_JPRB, 0.90771E+00_JPRB ,&
&  0.83798E+00_JPRB, 0.70252E+00_JPRB, 0.67659E+00_JPRB, 0.66595E+00_JPRB, 0.62882E+00_JPRB, 0.60894E+00_JPRB, 0.65130E+00_JPRB ,&
&  0.63714E+00_JPRB, 0.63923E+00_JPRB, 0.67388E+00_JPRB, 0.65786E+00_JPRB, 0.64953E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.62881E+00_JPRB, 0.63413E+00_JPRB, 0.64086E+00_JPRB, 0.62611E+00_JPRB, 0.60347E+00_JPRB, 0.58196E+00_JPRB, 0.56536E+00_JPRB ,&
&  0.56004E+00_JPRB, 0.48744E+00_JPRB, 0.45003E+00_JPRB, 0.43197E+00_JPRB, 0.42300E+00_JPRB, 0.40456E+00_JPRB, 0.38834E+00_JPRB ,&
&  0.38974E+00_JPRB, 0.40715E+00_JPRB, 0.44022E+00_JPRB, 0.46644E+00_JPRB, 0.48982E+00_JPRB, 0.50724E+00_JPRB, 0.51592E+00_JPRB ,&
&  0.52601E+00_JPRB, 0.53924E+00_JPRB, 0.55337E+00_JPRB, 0.57163E+00_JPRB, 0.59140E+00_JPRB, 0.62606E+00_JPRB, 0.67258E+00_JPRB ,&
&  0.71172E+00_JPRB, 0.73134E+00_JPRB, 0.70249E+00_JPRB, 0.71006E+00_JPRB, 0.72376E+00_JPRB, 0.72430E+00_JPRB, 0.72363E+00_JPRB ,&
&  0.48932E+00_JPRB, 0.45972E+00_JPRB, 0.53240E+00_JPRB, 0.52559E+00_JPRB, 0.52948E+00_JPRB, 0.52323E+00_JPRB, 0.51502E+00_JPRB ,&
&  0.51551E+00_JPRB, 0.52866E+00_JPRB, 0.55961E+00_JPRB, 0.61005E+00_JPRB, 0.66242E+00_JPRB, 0.72734E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.10218E+01_JPRB, 0.10137E+01_JPRB, 0.10777E+01_JPRB, 0.12281E+01_JPRB, 0.15238E+01_JPRB, 0.17541E+01_JPRB, 0.18721E+01_JPRB ,&
&  0.19626E+01_JPRB, 0.19554E+01_JPRB, 0.19562E+01_JPRB, 0.20085E+01_JPRB, 0.20309E+01_JPRB, 0.20260E+01_JPRB, 0.20642E+01_JPRB ,&
&  0.20654E+01_JPRB, 0.20856E+01_JPRB, 0.21202E+01_JPRB, 0.21646E+01_JPRB, 0.20897E+01_JPRB, 0.20229E+01_JPRB, 0.19467E+01_JPRB ,&
&  0.18515E+01_JPRB, 0.17313E+01_JPRB, 0.15795E+01_JPRB, 0.14545E+01_JPRB, 0.13590E+01_JPRB, 0.13368E+01_JPRB, 0.13190E+01_JPRB ,&
&  0.12771E+01_JPRB, 0.12463E+01_JPRB, 0.12012E+01_JPRB, 0.11538E+01_JPRB, 0.10651E+01_JPRB, 0.97153E+00_JPRB, 0.92128E+00_JPRB ,&
&  0.79370E+00_JPRB, 0.70160E+00_JPRB, 0.59870E+00_JPRB, 0.58330E+00_JPRB, 0.60470E+00_JPRB, 0.61180E+00_JPRB, 0.60649E+00_JPRB ,&
&  0.62707E+00_JPRB, 0.62998E+00_JPRB, 0.61775E+00_JPRB, 0.62202E+00_JPRB, 0.62460E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.60770E+00_JPRB, 0.61464E+00_JPRB, 0.61971E+00_JPRB, 0.62063E+00_JPRB, 0.59830E+00_JPRB, 0.58040E+00_JPRB, 0.56564E+00_JPRB ,&
&  0.55991E+00_JPRB, 0.50938E+00_JPRB, 0.47172E+00_JPRB, 0.44531E+00_JPRB, 0.42797E+00_JPRB, 0.41133E+00_JPRB, 0.40780E+00_JPRB ,&
&  0.41359E+00_JPRB, 0.42450E+00_JPRB, 0.43957E+00_JPRB, 0.45516E+00_JPRB, 0.46819E+00_JPRB, 0.47452E+00_JPRB, 0.48436E+00_JPRB ,&
&  0.49961E+00_JPRB, 0.51339E+00_JPRB, 0.52041E+00_JPRB, 0.52734E+00_JPRB, 0.53384E+00_JPRB, 0.55053E+00_JPRB, 0.55793E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.55050E+00_JPRB, 0.54548E+00_JPRB, 0.56198E+00_JPRB, 0.56948E+00_JPRB, 0.56771E+00_JPRB, 0.50487E+00_JPRB ,&
&  0.40837E+00_JPRB, 0.37125E+00_JPRB, 0.38246E+00_JPRB, 0.45584E+00_JPRB, 0.47538E+00_JPRB, 0.48825E+00_JPRB, 0.50261E+00_JPRB ,&
&  0.48842E+00_JPRB, 0.48673E+00_JPRB, 0.51694E+00_JPRB, 0.60129E+00_JPRB, 0.65741E+00_JPRB, 0.72060E+00_JPRB, 0.79926E+00_JPRB ,&
&  0.93801E+00_JPRB, 0.10604E+01_JPRB, 0.11962E+01_JPRB, 0.11593E+01_JPRB, 0.13088E+01_JPRB, 0.16762E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.16304E+01_JPRB, 0.16490E+01_JPRB, 0.17531E+01_JPRB, 0.18037E+01_JPRB, 0.18123E+01_JPRB, 0.17845E+01_JPRB, 0.17652E+01_JPRB ,&
&  0.17361E+01_JPRB, 0.17178E+01_JPRB, 0.17193E+01_JPRB, 0.18188E+01_JPRB, 0.18330E+01_JPRB, 0.17952E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.16618E+01_JPRB, 0.15200E+01_JPRB, 0.13812E+01_JPRB, 0.12677E+01_JPRB, 0.12273E+01_JPRB, 0.12242E+01_JPRB, 0.12151E+01_JPRB ,&
&  0.11804E+01_JPRB, 0.11647E+01_JPRB, 0.11533E+01_JPRB, 0.11376E+01_JPRB, 0.10172E+01_JPRB, 0.87802E+00_JPRB, 0.77236E+00_JPRB ,&
&  0.69795E+00_JPRB, 0.64856E+00_JPRB, 0.60482E+00_JPRB, 0.60227E+00_JPRB, 0.60570E+00_JPRB, 0.61193E+00_JPRB, 0.58842E+00_JPRB ,&
&  0.56721E+00_JPRB, 0.54937E+00_JPRB, 0.54936E+00_JPRB, 0.57364E+00_JPRB, 0.59347E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.54557E+00_JPRB, 0.56351E+00_JPRB, 0.59309E+00_JPRB, 0.60595E+00_JPRB, 0.60784E+00_JPRB, 0.57592E+00_JPRB, 0.54352E+00_JPRB ,&
&  0.51423E+00_JPRB, 0.48469E+00_JPRB, 0.46687E+00_JPRB, 0.45690E+00_JPRB, 0.45288E+00_JPRB, 0.45180E+00_JPRB, 0.45312E+00_JPRB ,&
&  0.45605E+00_JPRB, 0.45827E+00_JPRB, 0.46466E+00_JPRB, 0.47818E+00_JPRB, 0.49036E+00_JPRB, 0.49450E+00_JPRB, 0.49728E+00_JPRB ,&
&  0.50197E+00_JPRB, 0.50481E+00_JPRB, 0.50012E+00_JPRB, 0.48998E+00_JPRB, 0.48809E+00_JPRB, 0.50451E+00_JPRB, 0.51265E+00_JPRB ,&
&  0.51219E+00_JPRB, 0.49770E+00_JPRB, 0.49673E+00_JPRB, 0.51085E+00_JPRB, 0.50935E+00_JPRB, 0.49725E+00_JPRB, 0.44824E+00_JPRB ,&
&  0.38851E+00_JPRB, 0.34200E+00_JPRB, 0.32609E+00_JPRB, 0.34327E+00_JPRB, 0.38420E+00_JPRB, 0.45320E+00_JPRB, 0.47037E+00_JPRB ,&
&  0.48194E+00_JPRB, 0.49463E+00_JPRB, 0.51900E+00_JPRB, 0.56371E+00_JPRB, 0.62541E+00_JPRB, 0.68987E+00_JPRB, 0.75218E+00_JPRB ,&
&  0.84013E+00_JPRB, 0.93555E+00_JPRB, 0.10649E+01_JPRB, 0.10458E+01_JPRB, 0.11053E+01_JPRB, 0.12519E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.13921E+01_JPRB, 0.14556E+01_JPRB, 0.15194E+01_JPRB, 0.15886E+01_JPRB, 0.15709E+01_JPRB, 0.15371E+01_JPRB, 0.14927E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.14826E+01_JPRB, 0.14853E+01_JPRB, 0.15474E+01_JPRB, 0.16353E+01_JPRB, 0.16571E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.15243E+01_JPRB, 0.13913E+01_JPRB, 0.12482E+01_JPRB, 0.11399E+01_JPRB, 0.11195E+01_JPRB, 0.11502E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.11790E+01_JPRB, 0.11677E+01_JPRB, 0.11204E+01_JPRB, 0.10783E+01_JPRB, 0.95275E+00_JPRB, 0.86037E+00_JPRB, 0.79062E+00_JPRB ,&
&  0.75990E+00_JPRB, 0.73156E+00_JPRB, 0.70505E+00_JPRB, 0.71139E+00_JPRB, 0.68853E+00_JPRB, 0.66917E+00_JPRB, 0.64672E+00_JPRB ,&
&  0.61612E+00_JPRB, 0.58340E+00_JPRB, 0.56574E+00_JPRB, 0.55035E+00_JPRB, 0.54392E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.60630E+00_JPRB, 0.60136E+00_JPRB, 0.59675E+00_JPRB, 0.59844E+00_JPRB, 0.60246E+00_JPRB, 0.59435E+00_JPRB, 0.58137E+00_JPRB ,&
&  0.56148E+00_JPRB, 0.54206E+00_JPRB, 0.52630E+00_JPRB, 0.51395E+00_JPRB, 0.50772E+00_JPRB, 0.50545E+00_JPRB, 0.50499E+00_JPRB ,&
&  0.50614E+00_JPRB, 0.50892E+00_JPRB, 0.50999E+00_JPRB, 0.50618E+00_JPRB, 0.50556E+00_JPRB, 0.50782E+00_JPRB, 0.51086E+00_JPRB ,&
&  0.51366E+00_JPRB, 0.51095E+00_JPRB, 0.50501E+00_JPRB, 0.49984E+00_JPRB, 0.49163E+00_JPRB, 0.50053E+00_JPRB, 0.49370E+00_JPRB ,&
&  0.48108E+00_JPRB, 0.47877E+00_JPRB, 0.48946E+00_JPRB, 0.48816E+00_JPRB, 0.48170E+00_JPRB, 0.46585E+00_JPRB, 0.41381E+00_JPRB ,&
&  0.36463E+00_JPRB, 0.32760E+00_JPRB, 0.30812E+00_JPRB, 0.30398E+00_JPRB, 0.31492E+00_JPRB, 0.34382E+00_JPRB, 0.39200E+00_JPRB ,&
&  0.45868E+00_JPRB, 0.47647E+00_JPRB, 0.49569E+00_JPRB, 0.52592E+00_JPRB, 0.56501E+00_JPRB, 0.61361E+00_JPRB, 0.66045E+00_JPRB ,&
&  0.70895E+00_JPRB, 0.77006E+00_JPRB, 0.85473E+00_JPRB, 0.95981E+00_JPRB, 0.10781E+01_JPRB, 0.10859E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.14122E+01_JPRB, 0.14293E+01_JPRB, 0.14011E+01_JPRB, 0.13193E+01_JPRB, 0.12674E+01_JPRB, 0.12391E+01_JPRB, 0.12396E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.12992E+01_JPRB, 0.13681E+01_JPRB, 0.14350E+01_JPRB, 0.14632E+01_JPRB, 0.14695E+01_JPRB, 0.14173E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.13789E+01_JPRB, 0.16307E+01_JPRB, 0.14729E+01_JPRB, 0.13594E+01_JPRB, 0.13177E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.11674E+01_JPRB, 0.11021E+01_JPRB, 0.10411E+01_JPRB, 0.96103E+00_JPRB, 0.89168E+00_JPRB, 0.83442E+00_JPRB ,&
&  0.81522E+00_JPRB, 0.80092E+00_JPRB, 0.78280E+00_JPRB, 0.76780E+00_JPRB, 0.74462E+00_JPRB, 0.72547E+00_JPRB, 0.70396E+00_JPRB ,&
&  0.67250E+00_JPRB, 0.64687E+00_JPRB, 0.63296E+00_JPRB, 0.62129E+00_JPRB, 0.61238E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.61614E+00_JPRB, 0.60894E+00_JPRB, 0.60183E+00_JPRB, 0.59556E+00_JPRB, 0.59260E+00_JPRB, 0.59058E+00_JPRB, 0.58668E+00_JPRB ,&
&  0.57962E+00_JPRB, 0.57412E+00_JPRB, 0.56684E+00_JPRB, 0.56348E+00_JPRB, 0.56006E+00_JPRB, 0.55674E+00_JPRB, 0.55362E+00_JPRB ,&
&  0.54925E+00_JPRB, 0.53621E+00_JPRB, 0.52812E+00_JPRB, 0.52577E+00_JPRB, 0.52565E+00_JPRB, 0.52556E+00_JPRB, 0.52603E+00_JPRB ,&
&  0.52141E+00_JPRB, 0.51300E+00_JPRB, 0.50638E+00_JPRB, 0.49566E+00_JPRB, 0.48877E+00_JPRB, 0.48716E+00_JPRB, 0.49135E+00_JPRB ,&
&  0.49331E+00_JPRB, 0.47659E+00_JPRB, 0.47346E+00_JPRB, 0.46882E+00_JPRB, 0.45461E+00_JPRB, 0.44590E+00_JPRB, 0.38139E+00_JPRB ,&
&  0.34091E+00_JPRB, 0.31563E+00_JPRB, 0.30628E+00_JPRB, 0.30178E+00_JPRB, 0.31114E+00_JPRB, 0.33457E+00_JPRB, 0.38144E+00_JPRB ,&
&  0.43283E+00_JPRB, 0.48786E+00_JPRB, 0.50367E+00_JPRB, 0.52855E+00_JPRB, 0.55177E+00_JPRB, 0.58700E+00_JPRB, 0.61435E+00_JPRB ,&
&  0.63295E+00_JPRB, 0.64836E+00_JPRB, 0.67911E+00_JPRB, 0.70821E+00_JPRB, 0.76150E+00_JPRB, 0.81102E+00_JPRB, 0.84109E+00_JPRB ,&
&  0.87011E+00_JPRB, 0.88196E+00_JPRB, 0.90984E+00_JPRB, 0.93329E+00_JPRB, 0.95805E+00_JPRB, 0.96440E+00_JPRB, 0.96922E+00_JPRB ,&
&  0.98192E+00_JPRB, 0.10466E+01_JPRB, 0.11443E+01_JPRB, 0.12587E+01_JPRB, 0.13020E+01_JPRB, 0.13166E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.12530E+01_JPRB, 0.14039E+01_JPRB, 0.13880E+01_JPRB, 0.13774E+01_JPRB, 0.13578E+01_JPRB, 0.12910E+01_JPRB ,&
&  0.11992E+01_JPRB, 0.11596E+01_JPRB, 0.11160E+01_JPRB, 0.10366E+01_JPRB, 0.96321E+00_JPRB, 0.91838E+00_JPRB, 0.89376E+00_JPRB ,&
&  0.88086E+00_JPRB, 0.87091E+00_JPRB, 0.84470E+00_JPRB, 0.81926E+00_JPRB, 0.78947E+00_JPRB, 0.75384E+00_JPRB, 0.72337E+00_JPRB ,&
&  0.69722E+00_JPRB, 0.67650E+00_JPRB, 0.65823E+00_JPRB, 0.64090E+00_JPRB, 0.62757E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.62963E+00_JPRB, 0.62365E+00_JPRB, 0.61745E+00_JPRB, 0.61205E+00_JPRB, 0.60752E+00_JPRB, 0.60528E+00_JPRB, 0.60454E+00_JPRB ,&
&  0.60218E+00_JPRB, 0.59989E+00_JPRB, 0.59654E+00_JPRB, 0.59314E+00_JPRB, 0.58835E+00_JPRB, 0.58395E+00_JPRB, 0.57931E+00_JPRB ,&
&  0.57317E+00_JPRB, 0.56558E+00_JPRB, 0.55673E+00_JPRB, 0.54747E+00_JPRB, 0.53832E+00_JPRB, 0.53238E+00_JPRB, 0.52603E+00_JPRB ,&
&  0.51990E+00_JPRB, 0.51211E+00_JPRB, 0.49894E+00_JPRB, 0.48757E+00_JPRB, 0.48334E+00_JPRB, 0.47617E+00_JPRB, 0.47672E+00_JPRB ,&
&  0.47063E+00_JPRB, 0.45923E+00_JPRB, 0.44427E+00_JPRB, 0.43816E+00_JPRB, 0.43018E+00_JPRB, 0.36861E+00_JPRB, 0.35093E+00_JPRB ,&
&  0.33080E+00_JPRB, 0.31398E+00_JPRB, 0.30905E+00_JPRB, 0.31053E+00_JPRB, 0.31833E+00_JPRB, 0.33989E+00_JPRB, 0.37123E+00_JPRB ,&
&  0.41119E+00_JPRB, 0.51272E+00_JPRB, 0.54225E+00_JPRB, 0.57381E+00_JPRB, 0.59549E+00_JPRB, 0.61470E+00_JPRB, 0.61951E+00_JPRB ,&
&  0.62833E+00_JPRB, 0.63931E+00_JPRB, 0.66736E+00_JPRB, 0.70631E+00_JPRB, 0.74073E+00_JPRB, 0.75337E+00_JPRB, 0.75275E+00_JPRB ,&
&  0.74316E+00_JPRB, 0.74191E+00_JPRB, 0.73961E+00_JPRB, 0.74616E+00_JPRB, 0.77888E+00_JPRB, 0.81046E+00_JPRB, 0.85666E+00_JPRB ,&
&  0.90190E+00_JPRB, 0.93769E+00_JPRB, 0.99776E+00_JPRB, 0.10704E+01_JPRB, 0.11252E+01_JPRB, 0.11669E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.12054E+01_JPRB, 0.12067E+01_JPRB, 0.12053E+01_JPRB, 0.11904E+01_JPRB, 0.11559E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10965E+01_JPRB, 0.10777E+01_JPRB, 0.10241E+01_JPRB, 0.97399E+00_JPRB, 0.93296E+00_JPRB, 0.90033E+00_JPRB ,&
&  0.87918E+00_JPRB, 0.86246E+00_JPRB, 0.83523E+00_JPRB, 0.80494E+00_JPRB, 0.77122E+00_JPRB, 0.73716E+00_JPRB, 0.71518E+00_JPRB ,&
&  0.69739E+00_JPRB, 0.68369E+00_JPRB, 0.66720E+00_JPRB, 0.65155E+00_JPRB, 0.64018E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.64190E+00_JPRB, 0.63940E+00_JPRB, 0.63368E+00_JPRB, 0.62870E+00_JPRB, 0.62217E+00_JPRB, 0.61755E+00_JPRB, 0.61247E+00_JPRB ,&
&  0.60770E+00_JPRB, 0.60273E+00_JPRB, 0.59836E+00_JPRB, 0.59445E+00_JPRB, 0.58962E+00_JPRB, 0.58544E+00_JPRB, 0.58128E+00_JPRB ,&
&  0.57716E+00_JPRB, 0.57082E+00_JPRB, 0.56275E+00_JPRB, 0.55343E+00_JPRB, 0.54476E+00_JPRB, 0.53531E+00_JPRB, 0.52495E+00_JPRB ,&
&  0.51465E+00_JPRB, 0.50152E+00_JPRB, 0.47965E+00_JPRB, 0.46512E+00_JPRB, 0.44967E+00_JPRB, 0.44442E+00_JPRB, 0.43779E+00_JPRB ,&
&  0.44588E+00_JPRB, 0.40488E+00_JPRB, 0.39421E+00_JPRB, 0.37366E+00_JPRB, 0.35709E+00_JPRB, 0.34510E+00_JPRB, 0.33299E+00_JPRB ,&
&  0.32824E+00_JPRB, 0.32432E+00_JPRB, 0.32571E+00_JPRB, 0.33430E+00_JPRB, 0.34777E+00_JPRB, 0.37400E+00_JPRB, 0.40677E+00_JPRB ,&
&  0.45281E+00_JPRB, 0.54391E+00_JPRB, 0.57217E+00_JPRB, 0.60085E+00_JPRB, 0.62409E+00_JPRB, 0.64792E+00_JPRB, 0.64631E+00_JPRB ,&
&  0.65914E+00_JPRB, 0.67120E+00_JPRB, 0.69490E+00_JPRB, 0.71820E+00_JPRB, 0.74520E+00_JPRB, 0.75012E+00_JPRB, 0.74990E+00_JPRB ,&
&  0.73425E+00_JPRB, 0.72250E+00_JPRB, 0.71626E+00_JPRB, 0.73121E+00_JPRB, 0.76640E+00_JPRB, 0.81215E+00_JPRB, 0.85558E+00_JPRB ,&
&  0.89487E+00_JPRB, 0.92380E+00_JPRB, 0.94707E+00_JPRB, 0.97814E+00_JPRB, 0.10185E+01_JPRB, 0.10682E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.11171E+01_JPRB, 0.11273E+01_JPRB, 0.11260E+01_JPRB, 0.11156E+01_JPRB, 0.10873E+01_JPRB, 0.10573E+01_JPRB, 0.10395E+01_JPRB ,&
&  0.10406E+01_JPRB, 0.10272E+01_JPRB, 0.10068E+01_JPRB, 0.97753E+00_JPRB, 0.95322E+00_JPRB, 0.92335E+00_JPRB, 0.89130E+00_JPRB ,&
&  0.86301E+00_JPRB, 0.83946E+00_JPRB, 0.80930E+00_JPRB, 0.78195E+00_JPRB, 0.75102E+00_JPRB, 0.72491E+00_JPRB, 0.70694E+00_JPRB ,&
&  0.68990E+00_JPRB, 0.67633E+00_JPRB, 0.66163E+00_JPRB, 0.65205E+00_JPRB, 0.64613E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.64655E+00_JPRB, 0.64502E+00_JPRB, 0.64173E+00_JPRB, 0.63890E+00_JPRB, 0.63230E+00_JPRB, 0.62682E+00_JPRB, 0.62194E+00_JPRB ,&
&  0.61852E+00_JPRB, 0.61642E+00_JPRB, 0.61412E+00_JPRB, 0.61364E+00_JPRB, 0.61143E+00_JPRB, 0.60991E+00_JPRB, 0.60585E+00_JPRB ,&
&  0.60304E+00_JPRB, 0.59809E+00_JPRB, 0.59250E+00_JPRB, 0.58425E+00_JPRB, 0.57524E+00_JPRB, 0.56425E+00_JPRB, 0.54762E+00_JPRB ,&
&  0.52777E+00_JPRB, 0.50987E+00_JPRB, 0.47441E+00_JPRB, 0.45920E+00_JPRB, 0.43997E+00_JPRB, 0.42893E+00_JPRB, 0.42577E+00_JPRB ,&
&  0.42246E+00_JPRB, 0.42040E+00_JPRB, 0.41404E+00_JPRB, 0.40497E+00_JPRB, 0.39388E+00_JPRB, 0.37895E+00_JPRB, 0.36968E+00_JPRB ,&
&  0.36185E+00_JPRB, 0.36084E+00_JPRB, 0.36603E+00_JPRB, 0.37792E+00_JPRB, 0.39760E+00_JPRB, 0.42465E+00_JPRB, 0.45642E+00_JPRB ,&
&  0.49861E+00_JPRB, 0.54029E+00_JPRB, 0.59010E+00_JPRB, 0.60887E+00_JPRB, 0.62432E+00_JPRB, 0.64243E+00_JPRB, 0.65844E+00_JPRB ,&
&  0.66070E+00_JPRB, 0.66628E+00_JPRB, 0.68869E+00_JPRB, 0.69610E+00_JPRB, 0.70215E+00_JPRB, 0.70248E+00_JPRB, 0.70521E+00_JPRB ,&
&  0.70626E+00_JPRB, 0.71446E+00_JPRB, 0.72527E+00_JPRB, 0.74323E+00_JPRB, 0.77096E+00_JPRB, 0.80479E+00_JPRB, 0.83600E+00_JPRB ,&
&  0.86321E+00_JPRB, 0.89162E+00_JPRB, 0.91322E+00_JPRB, 0.93397E+00_JPRB, 0.95477E+00_JPRB, 0.98151E+00_JPRB, 0.10020E+01_JPRB ,&
&  0.10165E+01_JPRB, 0.10140E+01_JPRB, 0.10018E+01_JPRB, 0.97507E+00_JPRB, 0.96324E+00_JPRB, 0.97330E+00_JPRB, 0.96816E+00_JPRB ,&
&  0.96314E+00_JPRB, 0.95172E+00_JPRB, 0.94201E+00_JPRB, 0.92843E+00_JPRB, 0.91293E+00_JPRB, 0.89183E+00_JPRB, 0.86361E+00_JPRB ,&
&  0.83703E+00_JPRB, 0.81449E+00_JPRB, 0.79243E+00_JPRB, 0.77178E+00_JPRB, 0.74836E+00_JPRB, 0.72589E+00_JPRB, 0.70685E+00_JPRB ,&
&  0.69042E+00_JPRB, 0.67663E+00_JPRB, 0.66417E+00_JPRB, 0.65533E+00_JPRB, 0.65056E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.65398E+00_JPRB, 0.65266E+00_JPRB, 0.65029E+00_JPRB, 0.64672E+00_JPRB, 0.64066E+00_JPRB, 0.63514E+00_JPRB, 0.63032E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.62370E+00_JPRB, 0.62196E+00_JPRB, 0.62211E+00_JPRB, 0.62294E+00_JPRB, 0.62216E+00_JPRB, 0.62130E+00_JPRB ,&
&  0.62048E+00_JPRB, 0.61811E+00_JPRB, 0.61491E+00_JPRB, 0.61117E+00_JPRB, 0.60418E+00_JPRB, 0.59310E+00_JPRB, 0.58165E+00_JPRB ,&
&  0.57005E+00_JPRB, 0.56004E+00_JPRB, 0.55020E+00_JPRB, 0.54027E+00_JPRB, 0.53246E+00_JPRB, 0.48925E+00_JPRB, 0.47604E+00_JPRB ,&
&  0.47221E+00_JPRB, 0.47438E+00_JPRB, 0.48164E+00_JPRB, 0.49211E+00_JPRB, 0.51135E+00_JPRB, 0.50875E+00_JPRB, 0.50956E+00_JPRB ,&
&  0.48489E+00_JPRB, 0.48436E+00_JPRB, 0.48687E+00_JPRB, 0.49161E+00_JPRB, 0.50181E+00_JPRB, 0.51504E+00_JPRB, 0.53341E+00_JPRB ,&
&  0.56955E+00_JPRB, 0.57853E+00_JPRB, 0.58688E+00_JPRB, 0.59752E+00_JPRB, 0.60743E+00_JPRB, 0.61606E+00_JPRB, 0.62476E+00_JPRB ,&
&  0.63235E+00_JPRB, 0.64053E+00_JPRB, 0.64820E+00_JPRB, 0.65654E+00_JPRB, 0.66582E+00_JPRB, 0.67534E+00_JPRB, 0.68732E+00_JPRB ,&
&  0.70003E+00_JPRB, 0.71544E+00_JPRB, 0.73182E+00_JPRB, 0.74912E+00_JPRB, 0.76612E+00_JPRB, 0.78369E+00_JPRB, 0.80235E+00_JPRB ,&
&  0.82103E+00_JPRB, 0.84097E+00_JPRB, 0.85500E+00_JPRB, 0.86930E+00_JPRB, 0.88011E+00_JPRB, 0.89319E+00_JPRB, 0.89837E+00_JPRB ,&
&  0.90418E+00_JPRB, 0.90296E+00_JPRB, 0.90461E+00_JPRB, 0.90212E+00_JPRB, 0.90148E+00_JPRB, 0.90096E+00_JPRB, 0.89748E+00_JPRB ,&
&  0.89352E+00_JPRB, 0.88693E+00_JPRB, 0.88043E+00_JPRB, 0.87047E+00_JPRB, 0.85677E+00_JPRB, 0.83866E+00_JPRB, 0.81905E+00_JPRB ,&
&  0.80167E+00_JPRB, 0.78628E+00_JPRB, 0.77092E+00_JPRB, 0.75648E+00_JPRB, 0.74089E+00_JPRB, 0.72360E+00_JPRB, 0.70644E+00_JPRB ,&
&  0.69191E+00_JPRB, 0.67986E+00_JPRB, 0.67007E+00_JPRB, 0.66214E+00_JPRB, 0.65765E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.66462E+00_JPRB, 0.66410E+00_JPRB, 0.66356E+00_JPRB, 0.66212E+00_JPRB, 0.65843E+00_JPRB, 0.65494E+00_JPRB, 0.65143E+00_JPRB ,&
&  0.64889E+00_JPRB, 0.64704E+00_JPRB, 0.64436E+00_JPRB, 0.64114E+00_JPRB, 0.63779E+00_JPRB, 0.63526E+00_JPRB, 0.63270E+00_JPRB ,&
&  0.63014E+00_JPRB, 0.62674E+00_JPRB, 0.62259E+00_JPRB, 0.61615E+00_JPRB, 0.60986E+00_JPRB, 0.60385E+00_JPRB, 0.59970E+00_JPRB ,&
&  0.59667E+00_JPRB, 0.59384E+00_JPRB, 0.59041E+00_JPRB, 0.58732E+00_JPRB, 0.58415E+00_JPRB, 0.58139E+00_JPRB, 0.57830E+00_JPRB ,&
&  0.57671E+00_JPRB, 0.57637E+00_JPRB, 0.57605E+00_JPRB, 0.57534E+00_JPRB, 0.57609E+00_JPRB, 0.57790E+00_JPRB, 0.57946E+00_JPRB ,&
&  0.58056E+00_JPRB, 0.58246E+00_JPRB, 0.58588E+00_JPRB, 0.58863E+00_JPRB, 0.59085E+00_JPRB, 0.59365E+00_JPRB, 0.59561E+00_JPRB ,&
&  0.59606E+00_JPRB, 0.59744E+00_JPRB, 0.60143E+00_JPRB, 0.60575E+00_JPRB, 0.61044E+00_JPRB, 0.61562E+00_JPRB, 0.62300E+00_JPRB ,&
&  0.62915E+00_JPRB, 0.63594E+00_JPRB, 0.64351E+00_JPRB, 0.65249E+00_JPRB, 0.66154E+00_JPRB, 0.67159E+00_JPRB, 0.68208E+00_JPRB ,&
&  0.69260E+00_JPRB, 0.70357E+00_JPRB, 0.71441E+00_JPRB, 0.72515E+00_JPRB, 0.73518E+00_JPRB, 0.74379E+00_JPRB, 0.75285E+00_JPRB ,&
&  0.76236E+00_JPRB, 0.77352E+00_JPRB, 0.78139E+00_JPRB, 0.78993E+00_JPRB, 0.79911E+00_JPRB, 0.81158E+00_JPRB, 0.81714E+00_JPRB ,&
&  0.82226E+00_JPRB, 0.82699E+00_JPRB, 0.83204E+00_JPRB, 0.83387E+00_JPRB, 0.83565E+00_JPRB, 0.83703E+00_JPRB, 0.83688E+00_JPRB ,&
&  0.83434E+00_JPRB, 0.83041E+00_JPRB, 0.82481E+00_JPRB, 0.81647E+00_JPRB, 0.80597E+00_JPRB, 0.79468E+00_JPRB, 0.78185E+00_JPRB ,&
&  0.76967E+00_JPRB, 0.75554E+00_JPRB, 0.74134E+00_JPRB, 0.72905E+00_JPRB, 0.71941E+00_JPRB, 0.70846E+00_JPRB, 0.69744E+00_JPRB ,&
&  0.68709E+00_JPRB, 0.67896E+00_JPRB, 0.67406E+00_JPRB, 0.66978E+00_JPRB, 0.66714E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.67667E+00_JPRB, 0.67537E+00_JPRB, 0.67425E+00_JPRB, 0.67345E+00_JPRB, 0.67204E+00_JPRB, 0.67006E+00_JPRB, 0.66753E+00_JPRB ,&
&  0.66479E+00_JPRB, 0.66221E+00_JPRB, 0.65955E+00_JPRB, 0.65731E+00_JPRB, 0.65532E+00_JPRB, 0.65306E+00_JPRB, 0.65023E+00_JPRB ,&
&  0.64694E+00_JPRB, 0.64299E+00_JPRB, 0.63870E+00_JPRB, 0.63414E+00_JPRB, 0.63068E+00_JPRB, 0.62810E+00_JPRB, 0.62602E+00_JPRB ,&
&  0.62423E+00_JPRB, 0.62229E+00_JPRB, 0.62023E+00_JPRB, 0.61860E+00_JPRB, 0.61821E+00_JPRB, 0.61765E+00_JPRB, 0.61730E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.61742E+00_JPRB, 0.61741E+00_JPRB, 0.61732E+00_JPRB, 0.61721E+00_JPRB, 0.61821E+00_JPRB, 0.61890E+00_JPRB ,&
&  0.61986E+00_JPRB, 0.62048E+00_JPRB, 0.62087E+00_JPRB, 0.62116E+00_JPRB, 0.62174E+00_JPRB, 0.62246E+00_JPRB, 0.62331E+00_JPRB ,&
&  0.62446E+00_JPRB, 0.62620E+00_JPRB, 0.62905E+00_JPRB, 0.63251E+00_JPRB, 0.63599E+00_JPRB, 0.63960E+00_JPRB, 0.64404E+00_JPRB ,&
&  0.64848E+00_JPRB, 0.65303E+00_JPRB, 0.65749E+00_JPRB, 0.66240E+00_JPRB, 0.66746E+00_JPRB, 0.67234E+00_JPRB, 0.67737E+00_JPRB ,&
&  0.68256E+00_JPRB, 0.68788E+00_JPRB, 0.69349E+00_JPRB, 0.69913E+00_JPRB, 0.70424E+00_JPRB, 0.70877E+00_JPRB, 0.71363E+00_JPRB ,&
&  0.71876E+00_JPRB, 0.72380E+00_JPRB, 0.72761E+00_JPRB, 0.73172E+00_JPRB, 0.73590E+00_JPRB, 0.73990E+00_JPRB, 0.74275E+00_JPRB ,&
&  0.74578E+00_JPRB, 0.74916E+00_JPRB, 0.75261E+00_JPRB, 0.75341E+00_JPRB, 0.75391E+00_JPRB, 0.75402E+00_JPRB, 0.75318E+00_JPRB ,&
&  0.75081E+00_JPRB, 0.74823E+00_JPRB, 0.74543E+00_JPRB, 0.74200E+00_JPRB, 0.73708E+00_JPRB, 0.73191E+00_JPRB, 0.72625E+00_JPRB ,&
&  0.72050E+00_JPRB, 0.71393E+00_JPRB, 0.70738E+00_JPRB, 0.70178E+00_JPRB, 0.69752E+00_JPRB, 0.69337E+00_JPRB, 0.68916E+00_JPRB ,&
&  0.68492E+00_JPRB, 0.68226E+00_JPRB, 0.68102E+00_JPRB, 0.67979E+00_JPRB, 0.67844E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB ,&
&  0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB, 0.66800E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB, 0.24060E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.25287E+00_JPRB, 0.25316E+00_JPRB, 0.25328E+00_JPRB, 0.25329E+00_JPRB, 0.25150E+00_JPRB, 0.25190E+00_JPRB, 0.25198E+00_JPRB ,&
&  0.25224E+00_JPRB, 0.25251E+00_JPRB, 0.25312E+00_JPRB, 0.25383E+00_JPRB, 0.25452E+00_JPRB, 0.25516E+00_JPRB, 0.25572E+00_JPRB ,&
&  0.25606E+00_JPRB, 0.25649E+00_JPRB, 0.25522E+00_JPRB, 0.25447E+00_JPRB, 0.25439E+00_JPRB, 0.25450E+00_JPRB, 0.25448E+00_JPRB ,&
&  0.25444E+00_JPRB, 0.25433E+00_JPRB, 0.25388E+00_JPRB, 0.25356E+00_JPRB, 0.25341E+00_JPRB, 0.25322E+00_JPRB, 0.25291E+00_JPRB ,&
&  0.25252E+00_JPRB, 0.25221E+00_JPRB, 0.25185E+00_JPRB, 0.25143E+00_JPRB, 0.25128E+00_JPRB, 0.25108E+00_JPRB, 0.25047E+00_JPRB ,&
&  0.24976E+00_JPRB, 0.24873E+00_JPRB, 0.24779E+00_JPRB, 0.24694E+00_JPRB, 0.24626E+00_JPRB, 0.24590E+00_JPRB, 0.24554E+00_JPRB ,&
&  0.24505E+00_JPRB, 0.24439E+00_JPRB, 0.24376E+00_JPRB, 0.24300E+00_JPRB, 0.24233E+00_JPRB, 0.24173E+00_JPRB, 0.24102E+00_JPRB ,&
&  0.24024E+00_JPRB, 0.23945E+00_JPRB, 0.23863E+00_JPRB, 0.23773E+00_JPRB, 0.23686E+00_JPRB, 0.23597E+00_JPRB, 0.23502E+00_JPRB ,&
&  0.23407E+00_JPRB, 0.23365E+00_JPRB, 0.23318E+00_JPRB, 0.23277E+00_JPRB, 0.23246E+00_JPRB, 0.23223E+00_JPRB, 0.23199E+00_JPRB ,&
&  0.23181E+00_JPRB, 0.23161E+00_JPRB, 0.23151E+00_JPRB, 0.23142E+00_JPRB, 0.23142E+00_JPRB, 0.23148E+00_JPRB, 0.23165E+00_JPRB ,&
&  0.23185E+00_JPRB, 0.23211E+00_JPRB, 0.23245E+00_JPRB, 0.23301E+00_JPRB, 0.23357E+00_JPRB, 0.23421E+00_JPRB, 0.23485E+00_JPRB ,&
&  0.23555E+00_JPRB, 0.23624E+00_JPRB, 0.23695E+00_JPRB, 0.23762E+00_JPRB, 0.23852E+00_JPRB, 0.23939E+00_JPRB, 0.24034E+00_JPRB ,&
&  0.24129E+00_JPRB, 0.24231E+00_JPRB, 0.24333E+00_JPRB, 0.24441E+00_JPRB, 0.24564E+00_JPRB, 0.24601E+00_JPRB, 0.24640E+00_JPRB ,&
&  0.24688E+00_JPRB, 0.24758E+00_JPRB, 0.24822E+00_JPRB, 0.24905E+00_JPRB, 0.25045E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.27971E+00_JPRB, 0.28625E+00_JPRB, 0.29066E+00_JPRB, 0.29541E+00_JPRB, 0.29408E+00_JPRB, 0.33230E+00_JPRB, 0.33132E+00_JPRB ,&
&  0.33015E+00_JPRB, 0.31587E+00_JPRB, 0.31351E+00_JPRB, 0.30875E+00_JPRB, 0.30317E+00_JPRB, 0.29707E+00_JPRB, 0.29179E+00_JPRB ,&
&  0.28697E+00_JPRB, 0.28161E+00_JPRB, 0.27723E+00_JPRB, 0.27319E+00_JPRB, 0.26987E+00_JPRB, 0.26693E+00_JPRB, 0.26485E+00_JPRB ,&
&  0.26432E+00_JPRB, 0.26578E+00_JPRB, 0.26888E+00_JPRB, 0.27064E+00_JPRB, 0.27140E+00_JPRB, 0.27204E+00_JPRB, 0.27266E+00_JPRB ,&
&  0.27270E+00_JPRB, 0.27268E+00_JPRB, 0.27154E+00_JPRB, 0.26944E+00_JPRB, 0.26752E+00_JPRB, 0.26598E+00_JPRB, 0.26477E+00_JPRB ,&
&  0.26361E+00_JPRB, 0.26245E+00_JPRB, 0.26121E+00_JPRB, 0.26043E+00_JPRB, 0.25934E+00_JPRB, 0.25784E+00_JPRB, 0.25577E+00_JPRB ,&
&  0.25368E+00_JPRB, 0.25157E+00_JPRB, 0.24906E+00_JPRB, 0.24704E+00_JPRB, 0.24509E+00_JPRB, 0.24311E+00_JPRB, 0.24114E+00_JPRB ,&
&  0.24002E+00_JPRB, 0.23893E+00_JPRB, 0.23756E+00_JPRB, 0.23583E+00_JPRB, 0.23451E+00_JPRB, 0.23312E+00_JPRB, 0.23190E+00_JPRB ,&
&  0.23109E+00_JPRB, 0.23026E+00_JPRB, 0.22941E+00_JPRB, 0.22852E+00_JPRB, 0.22787E+00_JPRB, 0.22713E+00_JPRB, 0.22645E+00_JPRB ,&
&  0.22585E+00_JPRB, 0.22518E+00_JPRB, 0.22502E+00_JPRB, 0.22495E+00_JPRB, 0.22502E+00_JPRB, 0.22508E+00_JPRB, 0.22539E+00_JPRB ,&
&  0.22586E+00_JPRB, 0.22654E+00_JPRB, 0.22738E+00_JPRB, 0.22816E+00_JPRB, 0.22893E+00_JPRB, 0.22973E+00_JPRB, 0.23089E+00_JPRB ,&
&  0.23217E+00_JPRB, 0.23352E+00_JPRB, 0.23493E+00_JPRB, 0.23603E+00_JPRB, 0.23831E+00_JPRB, 0.24048E+00_JPRB, 0.24279E+00_JPRB ,&
&  0.24543E+00_JPRB, 0.24856E+00_JPRB, 0.25192E+00_JPRB, 0.25549E+00_JPRB, 0.26018E+00_JPRB, 0.26319E+00_JPRB, 0.26538E+00_JPRB ,&
&  0.26780E+00_JPRB, 0.27183E+00_JPRB, 0.27344E+00_JPRB, 0.27483E+00_JPRB, 0.27693E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.35170E+00_JPRB, 0.35649E+00_JPRB, 0.36058E+00_JPRB, 0.36420E+00_JPRB, 0.36741E+00_JPRB, 0.36825E+00_JPRB, 0.36731E+00_JPRB ,&
&  0.35826E+00_JPRB, 0.35156E+00_JPRB, 0.34338E+00_JPRB, 0.33610E+00_JPRB, 0.32938E+00_JPRB, 0.32145E+00_JPRB, 0.31461E+00_JPRB ,&
&  0.30762E+00_JPRB, 0.30063E+00_JPRB, 0.29400E+00_JPRB, 0.28623E+00_JPRB, 0.27813E+00_JPRB, 0.27037E+00_JPRB, 0.26561E+00_JPRB ,&
&  0.26288E+00_JPRB, 0.26274E+00_JPRB, 0.26567E+00_JPRB, 0.26951E+00_JPRB, 0.27448E+00_JPRB, 0.28088E+00_JPRB, 0.28773E+00_JPRB ,&
&  0.29206E+00_JPRB, 0.29443E+00_JPRB, 0.29397E+00_JPRB, 0.29249E+00_JPRB, 0.29046E+00_JPRB, 0.28873E+00_JPRB, 0.28605E+00_JPRB ,&
&  0.28264E+00_JPRB, 0.28022E+00_JPRB, 0.27939E+00_JPRB, 0.28031E+00_JPRB, 0.28002E+00_JPRB, 0.27811E+00_JPRB, 0.27419E+00_JPRB ,&
&  0.26984E+00_JPRB, 0.26544E+00_JPRB, 0.26095E+00_JPRB, 0.25678E+00_JPRB, 0.25335E+00_JPRB, 0.24977E+00_JPRB, 0.24590E+00_JPRB ,&
&  0.24346E+00_JPRB, 0.24086E+00_JPRB, 0.23801E+00_JPRB, 0.23530E+00_JPRB, 0.23304E+00_JPRB, 0.23130E+00_JPRB, 0.22984E+00_JPRB ,&
&  0.22871E+00_JPRB, 0.22783E+00_JPRB, 0.22713E+00_JPRB, 0.22660E+00_JPRB, 0.22609E+00_JPRB, 0.22532E+00_JPRB, 0.22456E+00_JPRB ,&
&  0.22380E+00_JPRB, 0.22353E+00_JPRB, 0.22306E+00_JPRB, 0.22308E+00_JPRB, 0.22333E+00_JPRB, 0.22349E+00_JPRB, 0.22400E+00_JPRB ,&
&  0.22512E+00_JPRB, 0.22629E+00_JPRB, 0.22692E+00_JPRB, 0.22802E+00_JPRB, 0.22910E+00_JPRB, 0.23050E+00_JPRB, 0.23231E+00_JPRB ,&
&  0.23347E+00_JPRB, 0.23476E+00_JPRB, 0.23627E+00_JPRB, 0.23788E+00_JPRB, 0.24053E+00_JPRB, 0.24342E+00_JPRB, 0.24664E+00_JPRB ,&
&  0.25077E+00_JPRB, 0.25481E+00_JPRB, 0.25942E+00_JPRB, 0.26487E+00_JPRB, 0.27448E+00_JPRB, 0.28117E+00_JPRB, 0.28979E+00_JPRB ,&
&  0.29842E+00_JPRB, 0.30346E+00_JPRB, 0.33744E+00_JPRB, 0.34003E+00_JPRB, 0.34537E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.36147E+00_JPRB, 0.36815E+00_JPRB, 0.37498E+00_JPRB, 0.38010E+00_JPRB, 0.38377E+00_JPRB, 0.38686E+00_JPRB, 0.38545E+00_JPRB ,&
&  0.37937E+00_JPRB, 0.36427E+00_JPRB, 0.35287E+00_JPRB, 0.34174E+00_JPRB, 0.33233E+00_JPRB, 0.32339E+00_JPRB, 0.31504E+00_JPRB ,&
&  0.30707E+00_JPRB, 0.30047E+00_JPRB, 0.29503E+00_JPRB, 0.29028E+00_JPRB, 0.28380E+00_JPRB, 0.27811E+00_JPRB, 0.27353E+00_JPRB ,&
&  0.26974E+00_JPRB, 0.26959E+00_JPRB, 0.27185E+00_JPRB, 0.27622E+00_JPRB, 0.28435E+00_JPRB, 0.29264E+00_JPRB, 0.30192E+00_JPRB ,&
&  0.31618E+00_JPRB, 0.32008E+00_JPRB, 0.31903E+00_JPRB, 0.31835E+00_JPRB, 0.31341E+00_JPRB, 0.30728E+00_JPRB, 0.29800E+00_JPRB ,&
&  0.29871E+00_JPRB, 0.29673E+00_JPRB, 0.30379E+00_JPRB, 0.30395E+00_JPRB, 0.30268E+00_JPRB, 0.30263E+00_JPRB, 0.29516E+00_JPRB ,&
&  0.28963E+00_JPRB, 0.28056E+00_JPRB, 0.27357E+00_JPRB, 0.26639E+00_JPRB, 0.25935E+00_JPRB, 0.25515E+00_JPRB, 0.24969E+00_JPRB ,&
&  0.24563E+00_JPRB, 0.24257E+00_JPRB, 0.23862E+00_JPRB, 0.23609E+00_JPRB, 0.23394E+00_JPRB, 0.23171E+00_JPRB, 0.23056E+00_JPRB ,&
&  0.22968E+00_JPRB, 0.22936E+00_JPRB, 0.22947E+00_JPRB, 0.22966E+00_JPRB, 0.23014E+00_JPRB, 0.23104E+00_JPRB, 0.23171E+00_JPRB ,&
&  0.23246E+00_JPRB, 0.23270E+00_JPRB, 0.23233E+00_JPRB, 0.23189E+00_JPRB, 0.23078E+00_JPRB, 0.23003E+00_JPRB, 0.22906E+00_JPRB ,&
&  0.22849E+00_JPRB, 0.22881E+00_JPRB, 0.22911E+00_JPRB, 0.22980E+00_JPRB, 0.23052E+00_JPRB, 0.23206E+00_JPRB, 0.23352E+00_JPRB ,&
&  0.23507E+00_JPRB, 0.23698E+00_JPRB, 0.23856E+00_JPRB, 0.24055E+00_JPRB, 0.24294E+00_JPRB, 0.24459E+00_JPRB, 0.24741E+00_JPRB ,&
&  0.25019E+00_JPRB, 0.25365E+00_JPRB, 0.25950E+00_JPRB, 0.26602E+00_JPRB, 0.27479E+00_JPRB, 0.28570E+00_JPRB, 0.29951E+00_JPRB ,&
&  0.31721E+00_JPRB, 0.34214E+00_JPRB, 0.34393E+00_JPRB, 0.35109E+00_JPRB, 0.35621E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.36677E+00_JPRB, 0.36936E+00_JPRB, 0.37339E+00_JPRB, 0.37882E+00_JPRB, 0.38562E+00_JPRB, 0.39177E+00_JPRB, 0.39319E+00_JPRB ,&
&  0.38391E+00_JPRB, 0.37100E+00_JPRB, 0.35283E+00_JPRB, 0.33837E+00_JPRB, 0.32445E+00_JPRB, 0.31400E+00_JPRB, 0.30549E+00_JPRB ,&
&  0.30012E+00_JPRB, 0.29964E+00_JPRB, 0.30009E+00_JPRB, 0.30132E+00_JPRB, 0.30034E+00_JPRB, 0.30142E+00_JPRB, 0.30180E+00_JPRB ,&
&  0.30062E+00_JPRB, 0.30051E+00_JPRB, 0.30041E+00_JPRB, 0.30391E+00_JPRB, 0.31071E+00_JPRB, 0.31703E+00_JPRB, 0.32330E+00_JPRB ,&
&  0.33063E+00_JPRB, 0.33284E+00_JPRB, 0.33708E+00_JPRB, 0.33587E+00_JPRB, 0.33238E+00_JPRB, 0.32788E+00_JPRB, 0.32157E+00_JPRB ,&
&  0.31697E+00_JPRB, 0.31294E+00_JPRB, 0.31151E+00_JPRB, 0.31190E+00_JPRB, 0.31197E+00_JPRB, 0.31367E+00_JPRB, 0.30752E+00_JPRB ,&
&  0.30027E+00_JPRB, 0.28993E+00_JPRB, 0.27815E+00_JPRB, 0.26946E+00_JPRB, 0.26124E+00_JPRB, 0.25532E+00_JPRB, 0.25004E+00_JPRB ,&
&  0.24515E+00_JPRB, 0.24187E+00_JPRB, 0.23848E+00_JPRB, 0.23599E+00_JPRB, 0.23522E+00_JPRB, 0.23350E+00_JPRB, 0.23255E+00_JPRB ,&
&  0.23217E+00_JPRB, 0.23261E+00_JPRB, 0.23419E+00_JPRB, 0.23563E+00_JPRB, 0.23761E+00_JPRB, 0.23986E+00_JPRB, 0.24178E+00_JPRB ,&
&  0.24435E+00_JPRB, 0.24640E+00_JPRB, 0.24664E+00_JPRB, 0.24604E+00_JPRB, 0.24395E+00_JPRB, 0.24122E+00_JPRB, 0.23878E+00_JPRB ,&
&  0.23634E+00_JPRB, 0.23461E+00_JPRB, 0.23396E+00_JPRB, 0.23313E+00_JPRB, 0.23356E+00_JPRB, 0.23471E+00_JPRB, 0.23559E+00_JPRB ,&
&  0.23730E+00_JPRB, 0.23869E+00_JPRB, 0.24029E+00_JPRB, 0.24299E+00_JPRB, 0.24555E+00_JPRB, 0.24781E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.25035E+00_JPRB, 0.25208E+00_JPRB, 0.25439E+00_JPRB, 0.25846E+00_JPRB, 0.27030E+00_JPRB, 0.27880E+00_JPRB, 0.29837E+00_JPRB ,&
&  0.31569E+00_JPRB, 0.35111E+00_JPRB, 0.35158E+00_JPRB, 0.35695E+00_JPRB, 0.36313E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.37030E+00_JPRB, 0.37470E+00_JPRB, 0.37556E+00_JPRB, 0.37888E+00_JPRB, 0.38355E+00_JPRB, 0.38901E+00_JPRB, 0.39356E+00_JPRB ,&
&  0.39868E+00_JPRB, 0.39841E+00_JPRB, 0.39329E+00_JPRB, 0.38622E+00_JPRB, 0.38028E+00_JPRB, 0.36346E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.34726E+00_JPRB, 0.34679E+00_JPRB, 0.34734E+00_JPRB, 0.34897E+00_JPRB, 0.35004E+00_JPRB, 0.35504E+00_JPRB, 0.35655E+00_JPRB ,&
&  0.35405E+00_JPRB, 0.34894E+00_JPRB, 0.34377E+00_JPRB, 0.34288E+00_JPRB, 0.34548E+00_JPRB, 0.34744E+00_JPRB, 0.34460E+00_JPRB ,&
&  0.34096E+00_JPRB, 0.34071E+00_JPRB, 0.34131E+00_JPRB, 0.34274E+00_JPRB, 0.35238E+00_JPRB, 0.34774E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.33618E+00_JPRB, 0.33001E+00_JPRB, 0.32571E+00_JPRB, 0.32599E+00_JPRB, 0.32882E+00_JPRB, 0.32855E+00_JPRB, 0.33106E+00_JPRB ,&
&  0.31931E+00_JPRB, 0.30959E+00_JPRB, 0.29481E+00_JPRB, 0.28315E+00_JPRB, 0.27172E+00_JPRB, 0.26389E+00_JPRB, 0.25657E+00_JPRB ,&
&  0.25033E+00_JPRB, 0.24514E+00_JPRB, 0.24024E+00_JPRB, 0.23865E+00_JPRB, 0.23954E+00_JPRB, 0.23966E+00_JPRB, 0.23931E+00_JPRB ,&
&  0.23988E+00_JPRB, 0.24084E+00_JPRB, 0.24160E+00_JPRB, 0.24257E+00_JPRB, 0.24413E+00_JPRB, 0.24669E+00_JPRB, 0.25014E+00_JPRB ,&
&  0.25557E+00_JPRB, 0.25989E+00_JPRB, 0.26223E+00_JPRB, 0.26334E+00_JPRB, 0.26024E+00_JPRB, 0.25498E+00_JPRB, 0.24960E+00_JPRB ,&
&  0.24427E+00_JPRB, 0.24177E+00_JPRB, 0.23956E+00_JPRB, 0.23852E+00_JPRB, 0.23834E+00_JPRB, 0.23922E+00_JPRB, 0.24100E+00_JPRB ,&
&  0.24275E+00_JPRB, 0.24495E+00_JPRB, 0.24641E+00_JPRB, 0.24818E+00_JPRB, 0.25033E+00_JPRB, 0.25221E+00_JPRB, 0.25418E+00_JPRB ,&
&  0.25638E+00_JPRB, 0.25811E+00_JPRB, 0.26105E+00_JPRB, 0.26467E+00_JPRB, 0.27153E+00_JPRB, 0.27970E+00_JPRB, 0.29429E+00_JPRB ,&
&  0.31153E+00_JPRB, 0.34404E+00_JPRB, 0.35482E+00_JPRB, 0.36138E+00_JPRB, 0.36835E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.36825E+00_JPRB, 0.37486E+00_JPRB, 0.37993E+00_JPRB, 0.38998E+00_JPRB, 0.39279E+00_JPRB, 0.39553E+00_JPRB, 0.39780E+00_JPRB ,&
&  0.40387E+00_JPRB, 0.40744E+00_JPRB, 0.40521E+00_JPRB, 0.40457E+00_JPRB, 0.40368E+00_JPRB, 0.40250E+00_JPRB, 0.40338E+00_JPRB ,&
&  0.40335E+00_JPRB, 0.40538E+00_JPRB, 0.40306E+00_JPRB, 0.40026E+00_JPRB, 0.40214E+00_JPRB, 0.40689E+00_JPRB, 0.41010E+00_JPRB ,&
&  0.40503E+00_JPRB, 0.40374E+00_JPRB, 0.41233E+00_JPRB, 0.41456E+00_JPRB, 0.41590E+00_JPRB, 0.41797E+00_JPRB, 0.41779E+00_JPRB ,&
&  0.40411E+00_JPRB, 0.37897E+00_JPRB, 0.35142E+00_JPRB, 0.35261E+00_JPRB, 0.36716E+00_JPRB, 0.36194E+00_JPRB, 0.36035E+00_JPRB ,&
&  0.35745E+00_JPRB, 0.35302E+00_JPRB, 0.34876E+00_JPRB, 0.34680E+00_JPRB, 0.34401E+00_JPRB, 0.33943E+00_JPRB, 0.33779E+00_JPRB ,&
&  0.33676E+00_JPRB, 0.34510E+00_JPRB, 0.34074E+00_JPRB, 0.31938E+00_JPRB, 0.30301E+00_JPRB, 0.29412E+00_JPRB, 0.28273E+00_JPRB ,&
&  0.26958E+00_JPRB, 0.26177E+00_JPRB, 0.26049E+00_JPRB, 0.25882E+00_JPRB, 0.26044E+00_JPRB, 0.26153E+00_JPRB, 0.26266E+00_JPRB ,&
&  0.26357E+00_JPRB, 0.26342E+00_JPRB, 0.25889E+00_JPRB, 0.25301E+00_JPRB, 0.24979E+00_JPRB, 0.24974E+00_JPRB, 0.25231E+00_JPRB ,&
&  0.25799E+00_JPRB, 0.26414E+00_JPRB, 0.27826E+00_JPRB, 0.28597E+00_JPRB, 0.27754E+00_JPRB, 0.27186E+00_JPRB, 0.26551E+00_JPRB ,&
&  0.25539E+00_JPRB, 0.25145E+00_JPRB, 0.24821E+00_JPRB, 0.24643E+00_JPRB, 0.24726E+00_JPRB, 0.25046E+00_JPRB, 0.25215E+00_JPRB ,&
&  0.25454E+00_JPRB, 0.25710E+00_JPRB, 0.25813E+00_JPRB, 0.25899E+00_JPRB, 0.25971E+00_JPRB, 0.26156E+00_JPRB, 0.26344E+00_JPRB ,&
&  0.26503E+00_JPRB, 0.26785E+00_JPRB, 0.27130E+00_JPRB, 0.27591E+00_JPRB, 0.27998E+00_JPRB, 0.28242E+00_JPRB, 0.28938E+00_JPRB ,&
&  0.29763E+00_JPRB, 0.31466E+00_JPRB, 0.34259E+00_JPRB, 0.36734E+00_JPRB, 0.36364E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.37649E+00_JPRB, 0.38151E+00_JPRB, 0.38832E+00_JPRB, 0.39590E+00_JPRB, 0.40126E+00_JPRB, 0.40363E+00_JPRB, 0.40611E+00_JPRB ,&
&  0.40938E+00_JPRB, 0.41077E+00_JPRB, 0.41182E+00_JPRB, 0.41461E+00_JPRB, 0.42019E+00_JPRB, 0.42309E+00_JPRB, 0.42778E+00_JPRB ,&
&  0.42957E+00_JPRB, 0.42740E+00_JPRB, 0.42065E+00_JPRB, 0.41863E+00_JPRB, 0.42213E+00_JPRB, 0.42686E+00_JPRB, 0.43212E+00_JPRB ,&
&  0.43761E+00_JPRB, 0.44266E+00_JPRB, 0.44508E+00_JPRB, 0.44583E+00_JPRB, 0.44608E+00_JPRB, 0.44601E+00_JPRB, 0.44345E+00_JPRB ,&
&  0.43858E+00_JPRB, 0.39935E+00_JPRB, 0.37275E+00_JPRB, 0.37557E+00_JPRB, 0.38722E+00_JPRB, 0.38184E+00_JPRB, 0.38029E+00_JPRB ,&
&  0.37870E+00_JPRB, 0.37525E+00_JPRB, 0.37012E+00_JPRB, 0.36599E+00_JPRB, 0.36081E+00_JPRB, 0.35415E+00_JPRB, 0.34872E+00_JPRB ,&
&  0.34751E+00_JPRB, 0.35223E+00_JPRB, 0.35383E+00_JPRB, 0.35624E+00_JPRB, 0.35527E+00_JPRB, 0.35745E+00_JPRB, 0.35646E+00_JPRB ,&
&  0.35129E+00_JPRB, 0.34536E+00_JPRB, 0.34347E+00_JPRB, 0.34315E+00_JPRB, 0.34420E+00_JPRB, 0.34681E+00_JPRB, 0.34678E+00_JPRB ,&
&  0.33363E+00_JPRB, 0.32215E+00_JPRB, 0.30372E+00_JPRB, 0.28587E+00_JPRB, 0.27281E+00_JPRB, 0.26686E+00_JPRB, 0.26250E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.26713E+00_JPRB, 0.28339E+00_JPRB, 0.30329E+00_JPRB, 0.31753E+00_JPRB, 0.29933E+00_JPRB, 0.28365E+00_JPRB ,&
&  0.27404E+00_JPRB, 0.26895E+00_JPRB, 0.26470E+00_JPRB, 0.26316E+00_JPRB, 0.26350E+00_JPRB, 0.26561E+00_JPRB, 0.26871E+00_JPRB ,&
&  0.27323E+00_JPRB, 0.27756E+00_JPRB, 0.28034E+00_JPRB, 0.28168E+00_JPRB, 0.28240E+00_JPRB, 0.28313E+00_JPRB, 0.28251E+00_JPRB ,&
&  0.28222E+00_JPRB, 0.28537E+00_JPRB, 0.29194E+00_JPRB, 0.30056E+00_JPRB, 0.30867E+00_JPRB, 0.31766E+00_JPRB, 0.33036E+00_JPRB ,&
&  0.34449E+00_JPRB, 0.35637E+00_JPRB, 0.36063E+00_JPRB, 0.36658E+00_JPRB, 0.37179E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.39750E+00_JPRB, 0.40194E+00_JPRB, 0.40604E+00_JPRB, 0.41054E+00_JPRB, 0.41686E+00_JPRB, 0.42001E+00_JPRB, 0.42096E+00_JPRB ,&
&  0.42228E+00_JPRB, 0.42291E+00_JPRB, 0.42448E+00_JPRB, 0.42884E+00_JPRB, 0.43480E+00_JPRB, 0.43897E+00_JPRB, 0.44456E+00_JPRB ,&
&  0.44464E+00_JPRB, 0.43608E+00_JPRB, 0.42828E+00_JPRB, 0.42797E+00_JPRB, 0.43341E+00_JPRB, 0.43911E+00_JPRB, 0.44633E+00_JPRB ,&
&  0.45351E+00_JPRB, 0.45877E+00_JPRB, 0.46175E+00_JPRB, 0.46326E+00_JPRB, 0.46525E+00_JPRB, 0.46494E+00_JPRB, 0.46200E+00_JPRB ,&
&  0.45801E+00_JPRB, 0.44927E+00_JPRB, 0.42979E+00_JPRB, 0.42694E+00_JPRB, 0.41820E+00_JPRB, 0.41058E+00_JPRB, 0.40525E+00_JPRB ,&
&  0.40091E+00_JPRB, 0.39743E+00_JPRB, 0.39356E+00_JPRB, 0.39031E+00_JPRB, 0.38661E+00_JPRB, 0.37986E+00_JPRB, 0.37650E+00_JPRB ,&
&  0.37346E+00_JPRB, 0.37093E+00_JPRB, 0.36852E+00_JPRB, 0.36893E+00_JPRB, 0.36952E+00_JPRB, 0.37253E+00_JPRB, 0.37409E+00_JPRB ,&
&  0.37113E+00_JPRB, 0.36759E+00_JPRB, 0.36484E+00_JPRB, 0.36648E+00_JPRB, 0.36869E+00_JPRB, 0.37022E+00_JPRB, 0.37079E+00_JPRB ,&
&  0.36840E+00_JPRB, 0.36396E+00_JPRB, 0.35940E+00_JPRB, 0.35409E+00_JPRB, 0.34903E+00_JPRB, 0.32334E+00_JPRB, 0.31202E+00_JPRB ,&
&  0.30854E+00_JPRB, 0.31739E+00_JPRB, 0.32979E+00_JPRB, 0.32841E+00_JPRB, 0.33033E+00_JPRB, 0.33040E+00_JPRB, 0.32788E+00_JPRB ,&
&  0.31905E+00_JPRB, 0.30545E+00_JPRB, 0.29622E+00_JPRB, 0.29207E+00_JPRB, 0.28902E+00_JPRB, 0.29005E+00_JPRB, 0.29584E+00_JPRB ,&
&  0.30570E+00_JPRB, 0.31730E+00_JPRB, 0.32612E+00_JPRB, 0.33016E+00_JPRB, 0.32913E+00_JPRB, 0.32605E+00_JPRB, 0.32192E+00_JPRB ,&
&  0.31730E+00_JPRB, 0.32273E+00_JPRB, 0.33663E+00_JPRB, 0.35655E+00_JPRB, 0.35913E+00_JPRB, 0.36399E+00_JPRB, 0.36729E+00_JPRB ,&
&  0.37245E+00_JPRB, 0.37894E+00_JPRB, 0.38461E+00_JPRB, 0.39069E+00_JPRB, 0.39449E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.42424E+00_JPRB, 0.42678E+00_JPRB, 0.42927E+00_JPRB, 0.43097E+00_JPRB, 0.43374E+00_JPRB, 0.43856E+00_JPRB, 0.44048E+00_JPRB ,&
&  0.43862E+00_JPRB, 0.43653E+00_JPRB, 0.43635E+00_JPRB, 0.43968E+00_JPRB, 0.44617E+00_JPRB, 0.45298E+00_JPRB, 0.46062E+00_JPRB ,&
&  0.45670E+00_JPRB, 0.44446E+00_JPRB, 0.44002E+00_JPRB, 0.44201E+00_JPRB, 0.44681E+00_JPRB, 0.45216E+00_JPRB, 0.45885E+00_JPRB ,&
&  0.46521E+00_JPRB, 0.46943E+00_JPRB, 0.47382E+00_JPRB, 0.47774E+00_JPRB, 0.48030E+00_JPRB, 0.47862E+00_JPRB, 0.47426E+00_JPRB ,&
&  0.46907E+00_JPRB, 0.46388E+00_JPRB, 0.45934E+00_JPRB, 0.44354E+00_JPRB, 0.44095E+00_JPRB, 0.43197E+00_JPRB, 0.42443E+00_JPRB ,&
&  0.41916E+00_JPRB, 0.41481E+00_JPRB, 0.41207E+00_JPRB, 0.40911E+00_JPRB, 0.40502E+00_JPRB, 0.39805E+00_JPRB, 0.39623E+00_JPRB ,&
&  0.39555E+00_JPRB, 0.39686E+00_JPRB, 0.39254E+00_JPRB, 0.39036E+00_JPRB, 0.38796E+00_JPRB, 0.38612E+00_JPRB, 0.39057E+00_JPRB ,&
&  0.39587E+00_JPRB, 0.39350E+00_JPRB, 0.38963E+00_JPRB, 0.38929E+00_JPRB, 0.39352E+00_JPRB, 0.39515E+00_JPRB, 0.39612E+00_JPRB ,&
&  0.39794E+00_JPRB, 0.39859E+00_JPRB, 0.39419E+00_JPRB, 0.38921E+00_JPRB, 0.38062E+00_JPRB, 0.36999E+00_JPRB, 0.36721E+00_JPRB ,&
&  0.36423E+00_JPRB, 0.35833E+00_JPRB, 0.35471E+00_JPRB, 0.35354E+00_JPRB, 0.35310E+00_JPRB, 0.35380E+00_JPRB, 0.35376E+00_JPRB ,&
&  0.35358E+00_JPRB, 0.34999E+00_JPRB, 0.34469E+00_JPRB, 0.34005E+00_JPRB, 0.33377E+00_JPRB, 0.33606E+00_JPRB, 0.34542E+00_JPRB ,&
&  0.35720E+00_JPRB, 0.36435E+00_JPRB, 0.36902E+00_JPRB, 0.37247E+00_JPRB, 0.37556E+00_JPRB, 0.37702E+00_JPRB, 0.37937E+00_JPRB ,&
&  0.38251E+00_JPRB, 0.38395E+00_JPRB, 0.38250E+00_JPRB, 0.38225E+00_JPRB, 0.38504E+00_JPRB, 0.38990E+00_JPRB, 0.39590E+00_JPRB ,&
&  0.40195E+00_JPRB, 0.40713E+00_JPRB, 0.41152E+00_JPRB, 0.41706E+00_JPRB, 0.42073E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.45775E+00_JPRB, 0.45804E+00_JPRB, 0.45873E+00_JPRB, 0.46020E+00_JPRB, 0.46120E+00_JPRB, 0.46033E+00_JPRB, 0.46002E+00_JPRB ,&
&  0.45680E+00_JPRB, 0.45209E+00_JPRB, 0.44911E+00_JPRB, 0.45040E+00_JPRB, 0.45928E+00_JPRB, 0.47080E+00_JPRB, 0.47781E+00_JPRB ,&
&  0.47366E+00_JPRB, 0.46152E+00_JPRB, 0.45856E+00_JPRB, 0.46088E+00_JPRB, 0.46310E+00_JPRB, 0.46726E+00_JPRB, 0.47343E+00_JPRB ,&
&  0.47906E+00_JPRB, 0.48291E+00_JPRB, 0.48709E+00_JPRB, 0.48880E+00_JPRB, 0.48906E+00_JPRB, 0.48200E+00_JPRB, 0.47086E+00_JPRB ,&
&  0.46015E+00_JPRB, 0.45251E+00_JPRB, 0.44790E+00_JPRB, 0.44441E+00_JPRB, 0.43930E+00_JPRB, 0.43420E+00_JPRB, 0.42756E+00_JPRB ,&
&  0.42425E+00_JPRB, 0.42388E+00_JPRB, 0.42469E+00_JPRB, 0.42281E+00_JPRB, 0.42031E+00_JPRB, 0.41538E+00_JPRB, 0.41134E+00_JPRB ,&
&  0.40992E+00_JPRB, 0.41137E+00_JPRB, 0.41293E+00_JPRB, 0.41235E+00_JPRB, 0.40924E+00_JPRB, 0.40711E+00_JPRB, 0.40870E+00_JPRB ,&
&  0.41468E+00_JPRB, 0.41459E+00_JPRB, 0.41364E+00_JPRB, 0.41436E+00_JPRB, 0.41647E+00_JPRB, 0.41841E+00_JPRB, 0.41786E+00_JPRB ,&
&  0.41725E+00_JPRB, 0.41732E+00_JPRB, 0.41775E+00_JPRB, 0.41981E+00_JPRB, 0.42008E+00_JPRB, 0.41813E+00_JPRB, 0.41273E+00_JPRB ,&
&  0.40440E+00_JPRB, 0.39384E+00_JPRB, 0.38946E+00_JPRB, 0.38634E+00_JPRB, 0.38329E+00_JPRB, 0.38251E+00_JPRB, 0.38371E+00_JPRB ,&
&  0.38222E+00_JPRB, 0.37601E+00_JPRB, 0.37316E+00_JPRB, 0.37346E+00_JPRB, 0.37211E+00_JPRB, 0.37392E+00_JPRB, 0.37730E+00_JPRB ,&
&  0.38706E+00_JPRB, 0.39241E+00_JPRB, 0.39399E+00_JPRB, 0.39186E+00_JPRB, 0.39217E+00_JPRB, 0.39390E+00_JPRB, 0.39816E+00_JPRB ,&
&  0.40466E+00_JPRB, 0.41165E+00_JPRB, 0.41880E+00_JPRB, 0.42667E+00_JPRB, 0.43666E+00_JPRB, 0.44631E+00_JPRB, 0.45000E+00_JPRB ,&
&  0.45016E+00_JPRB, 0.45047E+00_JPRB, 0.45165E+00_JPRB, 0.45313E+00_JPRB, 0.45624E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.48560E+00_JPRB, 0.48639E+00_JPRB, 0.48553E+00_JPRB, 0.48466E+00_JPRB, 0.48350E+00_JPRB, 0.48231E+00_JPRB, 0.48023E+00_JPRB ,&
&  0.47514E+00_JPRB, 0.46859E+00_JPRB, 0.46470E+00_JPRB, 0.46286E+00_JPRB, 0.46835E+00_JPRB, 0.47658E+00_JPRB, 0.48400E+00_JPRB ,&
&  0.48325E+00_JPRB, 0.47437E+00_JPRB, 0.47174E+00_JPRB, 0.47378E+00_JPRB, 0.47641E+00_JPRB, 0.47930E+00_JPRB, 0.48317E+00_JPRB ,&
&  0.48740E+00_JPRB, 0.49171E+00_JPRB, 0.49359E+00_JPRB, 0.49111E+00_JPRB, 0.48465E+00_JPRB, 0.47096E+00_JPRB, 0.45896E+00_JPRB ,&
&  0.45101E+00_JPRB, 0.44488E+00_JPRB, 0.44148E+00_JPRB, 0.44060E+00_JPRB, 0.43843E+00_JPRB, 0.43457E+00_JPRB, 0.42920E+00_JPRB ,&
&  0.42449E+00_JPRB, 0.42226E+00_JPRB, 0.42434E+00_JPRB, 0.42597E+00_JPRB, 0.42684E+00_JPRB, 0.42666E+00_JPRB, 0.42392E+00_JPRB ,&
&  0.42250E+00_JPRB, 0.42226E+00_JPRB, 0.42226E+00_JPRB, 0.42414E+00_JPRB, 0.42214E+00_JPRB, 0.42045E+00_JPRB, 0.42272E+00_JPRB ,&
&  0.42735E+00_JPRB, 0.43132E+00_JPRB, 0.43487E+00_JPRB, 0.43751E+00_JPRB, 0.43857E+00_JPRB, 0.44065E+00_JPRB, 0.44202E+00_JPRB ,&
&  0.44119E+00_JPRB, 0.43775E+00_JPRB, 0.43457E+00_JPRB, 0.43528E+00_JPRB, 0.43785E+00_JPRB, 0.44069E+00_JPRB, 0.44433E+00_JPRB ,&
&  0.44497E+00_JPRB, 0.43987E+00_JPRB, 0.43477E+00_JPRB, 0.43020E+00_JPRB, 0.42595E+00_JPRB, 0.42258E+00_JPRB, 0.42082E+00_JPRB ,&
&  0.42020E+00_JPRB, 0.41706E+00_JPRB, 0.41112E+00_JPRB, 0.40620E+00_JPRB, 0.40532E+00_JPRB, 0.40408E+00_JPRB, 0.39996E+00_JPRB ,&
&  0.40007E+00_JPRB, 0.40233E+00_JPRB, 0.40113E+00_JPRB, 0.39924E+00_JPRB, 0.40128E+00_JPRB, 0.40664E+00_JPRB, 0.41754E+00_JPRB ,&
&  0.43212E+00_JPRB, 0.44888E+00_JPRB, 0.46443E+00_JPRB, 0.47808E+00_JPRB, 0.48638E+00_JPRB, 0.48855E+00_JPRB, 0.48697E+00_JPRB ,&
&  0.48636E+00_JPRB, 0.48700E+00_JPRB, 0.48588E+00_JPRB, 0.48409E+00_JPRB, 0.48431E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.51572E+00_JPRB, 0.51429E+00_JPRB, 0.51271E+00_JPRB, 0.51070E+00_JPRB, 0.50764E+00_JPRB, 0.50477E+00_JPRB, 0.49994E+00_JPRB ,&
&  0.49477E+00_JPRB, 0.49041E+00_JPRB, 0.48841E+00_JPRB, 0.48771E+00_JPRB, 0.48898E+00_JPRB, 0.49293E+00_JPRB, 0.49961E+00_JPRB ,&
&  0.49971E+00_JPRB, 0.49319E+00_JPRB, 0.49024E+00_JPRB, 0.49038E+00_JPRB, 0.49288E+00_JPRB, 0.49475E+00_JPRB, 0.49764E+00_JPRB ,&
&  0.49959E+00_JPRB, 0.50025E+00_JPRB, 0.49963E+00_JPRB, 0.49277E+00_JPRB, 0.47609E+00_JPRB, 0.46592E+00_JPRB, 0.45691E+00_JPRB ,&
&  0.45159E+00_JPRB, 0.44707E+00_JPRB, 0.44441E+00_JPRB, 0.44365E+00_JPRB, 0.44262E+00_JPRB, 0.44071E+00_JPRB, 0.44056E+00_JPRB ,&
&  0.43851E+00_JPRB, 0.43582E+00_JPRB, 0.43430E+00_JPRB, 0.43476E+00_JPRB, 0.43641E+00_JPRB, 0.43804E+00_JPRB, 0.43807E+00_JPRB ,&
&  0.43923E+00_JPRB, 0.44317E+00_JPRB, 0.44450E+00_JPRB, 0.44494E+00_JPRB, 0.44497E+00_JPRB, 0.44466E+00_JPRB, 0.44488E+00_JPRB ,&
&  0.44726E+00_JPRB, 0.45232E+00_JPRB, 0.45655E+00_JPRB, 0.46076E+00_JPRB, 0.46253E+00_JPRB, 0.46338E+00_JPRB, 0.46339E+00_JPRB ,&
&  0.46042E+00_JPRB, 0.45652E+00_JPRB, 0.45533E+00_JPRB, 0.45780E+00_JPRB, 0.46275E+00_JPRB, 0.46886E+00_JPRB, 0.47277E+00_JPRB ,&
&  0.47568E+00_JPRB, 0.47667E+00_JPRB, 0.47438E+00_JPRB, 0.47305E+00_JPRB, 0.47024E+00_JPRB, 0.46560E+00_JPRB, 0.46289E+00_JPRB ,&
&  0.46028E+00_JPRB, 0.45803E+00_JPRB, 0.45616E+00_JPRB, 0.44738E+00_JPRB, 0.43972E+00_JPRB, 0.43489E+00_JPRB, 0.43166E+00_JPRB ,&
&  0.43038E+00_JPRB, 0.42942E+00_JPRB, 0.42909E+00_JPRB, 0.43036E+00_JPRB, 0.43243E+00_JPRB, 0.43875E+00_JPRB, 0.45147E+00_JPRB ,&
&  0.47056E+00_JPRB, 0.48851E+00_JPRB, 0.49867E+00_JPRB, 0.50646E+00_JPRB, 0.51213E+00_JPRB, 0.51549E+00_JPRB, 0.51856E+00_JPRB ,&
&  0.52257E+00_JPRB, 0.52394E+00_JPRB, 0.52193E+00_JPRB, 0.51916E+00_JPRB, 0.51778E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.54740E+00_JPRB, 0.53869E+00_JPRB, 0.53194E+00_JPRB, 0.52626E+00_JPRB, 0.52130E+00_JPRB, 0.51899E+00_JPRB, 0.51648E+00_JPRB ,&
&  0.51400E+00_JPRB, 0.51097E+00_JPRB, 0.50926E+00_JPRB, 0.50573E+00_JPRB, 0.50260E+00_JPRB, 0.50523E+00_JPRB, 0.51059E+00_JPRB ,&
&  0.50922E+00_JPRB, 0.50261E+00_JPRB, 0.49737E+00_JPRB, 0.49737E+00_JPRB, 0.50094E+00_JPRB, 0.50365E+00_JPRB, 0.50687E+00_JPRB ,&
&  0.50684E+00_JPRB, 0.50631E+00_JPRB, 0.50495E+00_JPRB, 0.48954E+00_JPRB, 0.47450E+00_JPRB, 0.46706E+00_JPRB, 0.46278E+00_JPRB ,&
&  0.46229E+00_JPRB, 0.46714E+00_JPRB, 0.45123E+00_JPRB, 0.46124E+00_JPRB, 0.45727E+00_JPRB, 0.45107E+00_JPRB, 0.44755E+00_JPRB ,&
&  0.44535E+00_JPRB, 0.44281E+00_JPRB, 0.44097E+00_JPRB, 0.44284E+00_JPRB, 0.44551E+00_JPRB, 0.44638E+00_JPRB, 0.44827E+00_JPRB ,&
&  0.44825E+00_JPRB, 0.45095E+00_JPRB, 0.45331E+00_JPRB, 0.45853E+00_JPRB, 0.46254E+00_JPRB, 0.46506E+00_JPRB, 0.46828E+00_JPRB ,&
&  0.47275E+00_JPRB, 0.47691E+00_JPRB, 0.48083E+00_JPRB, 0.48411E+00_JPRB, 0.48540E+00_JPRB, 0.48514E+00_JPRB, 0.48105E+00_JPRB ,&
&  0.47778E+00_JPRB, 0.47759E+00_JPRB, 0.48013E+00_JPRB, 0.48540E+00_JPRB, 0.49061E+00_JPRB, 0.49263E+00_JPRB, 0.49379E+00_JPRB ,&
&  0.49553E+00_JPRB, 0.49909E+00_JPRB, 0.49960E+00_JPRB, 0.49811E+00_JPRB, 0.49598E+00_JPRB, 0.49328E+00_JPRB, 0.48868E+00_JPRB ,&
&  0.48277E+00_JPRB, 0.47820E+00_JPRB, 0.47360E+00_JPRB, 0.46292E+00_JPRB, 0.45470E+00_JPRB, 0.45396E+00_JPRB, 0.45552E+00_JPRB ,&
&  0.45508E+00_JPRB, 0.45361E+00_JPRB, 0.45484E+00_JPRB, 0.46048E+00_JPRB, 0.46030E+00_JPRB, 0.46670E+00_JPRB, 0.48020E+00_JPRB ,&
&  0.49439E+00_JPRB, 0.50753E+00_JPRB, 0.51935E+00_JPRB, 0.53218E+00_JPRB, 0.54548E+00_JPRB, 0.55654E+00_JPRB, 0.56611E+00_JPRB ,&
&  0.57080E+00_JPRB, 0.56929E+00_JPRB, 0.56338E+00_JPRB, 0.55939E+00_JPRB, 0.55586E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.60222E+00_JPRB, 0.58608E+00_JPRB, 0.57333E+00_JPRB, 0.56339E+00_JPRB, 0.55645E+00_JPRB, 0.55363E+00_JPRB, 0.55013E+00_JPRB ,&
&  0.54708E+00_JPRB, 0.54471E+00_JPRB, 0.53806E+00_JPRB, 0.53123E+00_JPRB, 0.53039E+00_JPRB, 0.53574E+00_JPRB, 0.54011E+00_JPRB ,&
&  0.53728E+00_JPRB, 0.52697E+00_JPRB, 0.51726E+00_JPRB, 0.51531E+00_JPRB, 0.51841E+00_JPRB, 0.52102E+00_JPRB, 0.52087E+00_JPRB ,&
&  0.52263E+00_JPRB, 0.52368E+00_JPRB, 0.51537E+00_JPRB, 0.49851E+00_JPRB, 0.49048E+00_JPRB, 0.48583E+00_JPRB, 0.49021E+00_JPRB ,&
&  0.49812E+00_JPRB, 0.49240E+00_JPRB, 0.50429E+00_JPRB, 0.50285E+00_JPRB, 0.50192E+00_JPRB, 0.49811E+00_JPRB, 0.49133E+00_JPRB ,&
&  0.48582E+00_JPRB, 0.48012E+00_JPRB, 0.47651E+00_JPRB, 0.47487E+00_JPRB, 0.47536E+00_JPRB, 0.47660E+00_JPRB, 0.48118E+00_JPRB ,&
&  0.48235E+00_JPRB, 0.48059E+00_JPRB, 0.48309E+00_JPRB, 0.49001E+00_JPRB, 0.49866E+00_JPRB, 0.50355E+00_JPRB, 0.50801E+00_JPRB ,&
&  0.51170E+00_JPRB, 0.51378E+00_JPRB, 0.51662E+00_JPRB, 0.51745E+00_JPRB, 0.51385E+00_JPRB, 0.51310E+00_JPRB, 0.51304E+00_JPRB ,&
&  0.51513E+00_JPRB, 0.51839E+00_JPRB, 0.52258E+00_JPRB, 0.52683E+00_JPRB, 0.52851E+00_JPRB, 0.52559E+00_JPRB, 0.52275E+00_JPRB ,&
&  0.52327E+00_JPRB, 0.52664E+00_JPRB, 0.53063E+00_JPRB, 0.53281E+00_JPRB, 0.53176E+00_JPRB, 0.52487E+00_JPRB, 0.51674E+00_JPRB ,&
&  0.50833E+00_JPRB, 0.49974E+00_JPRB, 0.49199E+00_JPRB, 0.48739E+00_JPRB, 0.48395E+00_JPRB, 0.48189E+00_JPRB, 0.48079E+00_JPRB ,&
&  0.48061E+00_JPRB, 0.48094E+00_JPRB, 0.48431E+00_JPRB, 0.49060E+00_JPRB, 0.49402E+00_JPRB, 0.50233E+00_JPRB, 0.51320E+00_JPRB ,&
&  0.52524E+00_JPRB, 0.53798E+00_JPRB, 0.55344E+00_JPRB, 0.57472E+00_JPRB, 0.59915E+00_JPRB, 0.61962E+00_JPRB, 0.63549E+00_JPRB ,&
&  0.64574E+00_JPRB, 0.64619E+00_JPRB, 0.63913E+00_JPRB, 0.63019E+00_JPRB, 0.61798E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.66137E+00_JPRB, 0.63655E+00_JPRB, 0.61483E+00_JPRB, 0.59806E+00_JPRB, 0.58718E+00_JPRB, 0.58279E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.57464E+00_JPRB, 0.57063E+00_JPRB, 0.56805E+00_JPRB, 0.57000E+00_JPRB, 0.57083E+00_JPRB, 0.57103E+00_JPRB, 0.57211E+00_JPRB ,&
&  0.56655E+00_JPRB, 0.55382E+00_JPRB, 0.54362E+00_JPRB, 0.54087E+00_JPRB, 0.54200E+00_JPRB, 0.53823E+00_JPRB, 0.53688E+00_JPRB ,&
&  0.53989E+00_JPRB, 0.53192E+00_JPRB, 0.51841E+00_JPRB, 0.50953E+00_JPRB, 0.50977E+00_JPRB, 0.50722E+00_JPRB, 0.50454E+00_JPRB ,&
&  0.52678E+00_JPRB, 0.47919E+00_JPRB, 0.52613E+00_JPRB, 0.53514E+00_JPRB, 0.54148E+00_JPRB, 0.54462E+00_JPRB, 0.54319E+00_JPRB ,&
&  0.53837E+00_JPRB, 0.53206E+00_JPRB, 0.52686E+00_JPRB, 0.52201E+00_JPRB, 0.51869E+00_JPRB, 0.51401E+00_JPRB, 0.50969E+00_JPRB ,&
&  0.51013E+00_JPRB, 0.51117E+00_JPRB, 0.51566E+00_JPRB, 0.52116E+00_JPRB, 0.52855E+00_JPRB, 0.53100E+00_JPRB, 0.53319E+00_JPRB ,&
&  0.53583E+00_JPRB, 0.53744E+00_JPRB, 0.53734E+00_JPRB, 0.53626E+00_JPRB, 0.53483E+00_JPRB, 0.53942E+00_JPRB, 0.54572E+00_JPRB ,&
&  0.55076E+00_JPRB, 0.55625E+00_JPRB, 0.56383E+00_JPRB, 0.56756E+00_JPRB, 0.56837E+00_JPRB, 0.56453E+00_JPRB, 0.55778E+00_JPRB ,&
&  0.55295E+00_JPRB, 0.55182E+00_JPRB, 0.55156E+00_JPRB, 0.54674E+00_JPRB, 0.53959E+00_JPRB, 0.53306E+00_JPRB, 0.52496E+00_JPRB ,&
&  0.51460E+00_JPRB, 0.50640E+00_JPRB, 0.50222E+00_JPRB, 0.50119E+00_JPRB, 0.50247E+00_JPRB, 0.50322E+00_JPRB, 0.50390E+00_JPRB ,&
&  0.50667E+00_JPRB, 0.50764E+00_JPRB, 0.50970E+00_JPRB, 0.51491E+00_JPRB, 0.51964E+00_JPRB, 0.52668E+00_JPRB, 0.53657E+00_JPRB ,&
&  0.55019E+00_JPRB, 0.56649E+00_JPRB, 0.58153E+00_JPRB, 0.60690E+00_JPRB, 0.64190E+00_JPRB, 0.66949E+00_JPRB, 0.68473E+00_JPRB ,&
&  0.69779E+00_JPRB, 0.70188E+00_JPRB, 0.69745E+00_JPRB, 0.69191E+00_JPRB, 0.67893E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.72168E+00_JPRB, 0.70276E+00_JPRB, 0.67794E+00_JPRB, 0.65822E+00_JPRB, 0.64530E+00_JPRB, 0.63777E+00_JPRB, 0.63054E+00_JPRB ,&
&  0.62269E+00_JPRB, 0.62091E+00_JPRB, 0.62040E+00_JPRB, 0.62014E+00_JPRB, 0.61979E+00_JPRB, 0.61609E+00_JPRB, 0.60806E+00_JPRB ,&
&  0.59896E+00_JPRB, 0.58737E+00_JPRB, 0.58532E+00_JPRB, 0.58453E+00_JPRB, 0.57820E+00_JPRB, 0.57378E+00_JPRB, 0.57323E+00_JPRB ,&
&  0.56670E+00_JPRB, 0.55646E+00_JPRB, 0.53808E+00_JPRB, 0.53199E+00_JPRB, 0.53437E+00_JPRB, 0.53428E+00_JPRB, 0.54115E+00_JPRB ,&
&  0.52526E+00_JPRB, 0.50441E+00_JPRB, 0.55675E+00_JPRB, 0.59359E+00_JPRB, 0.61177E+00_JPRB, 0.62545E+00_JPRB, 0.63948E+00_JPRB ,&
&  0.63438E+00_JPRB, 0.63813E+00_JPRB, 0.62358E+00_JPRB, 0.59666E+00_JPRB, 0.57766E+00_JPRB, 0.56798E+00_JPRB, 0.56373E+00_JPRB ,&
&  0.56696E+00_JPRB, 0.56960E+00_JPRB, 0.56958E+00_JPRB, 0.56872E+00_JPRB, 0.56925E+00_JPRB, 0.56568E+00_JPRB, 0.56329E+00_JPRB ,&
&  0.56459E+00_JPRB, 0.56789E+00_JPRB, 0.56959E+00_JPRB, 0.57379E+00_JPRB, 0.58245E+00_JPRB, 0.59558E+00_JPRB, 0.60793E+00_JPRB ,&
&  0.61803E+00_JPRB, 0.62769E+00_JPRB, 0.64168E+00_JPRB, 0.65050E+00_JPRB, 0.64992E+00_JPRB, 0.63860E+00_JPRB, 0.62362E+00_JPRB ,&
&  0.60881E+00_JPRB, 0.59822E+00_JPRB, 0.59072E+00_JPRB, 0.58521E+00_JPRB, 0.57678E+00_JPRB, 0.56865E+00_JPRB, 0.55956E+00_JPRB ,&
&  0.55223E+00_JPRB, 0.54411E+00_JPRB, 0.53970E+00_JPRB, 0.53731E+00_JPRB, 0.53819E+00_JPRB, 0.54096E+00_JPRB, 0.54516E+00_JPRB ,&
&  0.54804E+00_JPRB, 0.54621E+00_JPRB, 0.54607E+00_JPRB, 0.55120E+00_JPRB, 0.55751E+00_JPRB, 0.56825E+00_JPRB, 0.58307E+00_JPRB ,&
&  0.60114E+00_JPRB, 0.61929E+00_JPRB, 0.63726E+00_JPRB, 0.67761E+00_JPRB, 0.71874E+00_JPRB, 0.75027E+00_JPRB, 0.76568E+00_JPRB ,&
&  0.77343E+00_JPRB, 0.76446E+00_JPRB, 0.76298E+00_JPRB, 0.75115E+00_JPRB, 0.73531E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.72189E+00_JPRB, 0.71619E+00_JPRB, 0.70042E+00_JPRB, 0.68986E+00_JPRB, 0.68148E+00_JPRB, 0.67205E+00_JPRB, 0.65895E+00_JPRB ,&
&  0.64936E+00_JPRB, 0.64506E+00_JPRB, 0.64181E+00_JPRB, 0.64098E+00_JPRB, 0.64089E+00_JPRB, 0.63540E+00_JPRB, 0.62700E+00_JPRB ,&
&  0.61824E+00_JPRB, 0.61012E+00_JPRB, 0.61109E+00_JPRB, 0.61020E+00_JPRB, 0.60502E+00_JPRB, 0.60060E+00_JPRB, 0.59481E+00_JPRB ,&
&  0.58015E+00_JPRB, 0.56519E+00_JPRB, 0.55343E+00_JPRB, 0.55598E+00_JPRB, 0.56231E+00_JPRB, 0.55743E+00_JPRB, 0.57288E+00_JPRB ,&
&  0.62944E+00_JPRB, 0.55806E+00_JPRB, 0.58661E+00_JPRB, 0.65909E+00_JPRB, 0.69342E+00_JPRB, 0.72803E+00_JPRB, 0.75495E+00_JPRB ,&
&  0.77389E+00_JPRB, 0.78799E+00_JPRB, 0.75414E+00_JPRB, 0.71152E+00_JPRB, 0.67355E+00_JPRB, 0.64318E+00_JPRB, 0.62167E+00_JPRB ,&
&  0.61086E+00_JPRB, 0.61065E+00_JPRB, 0.61111E+00_JPRB, 0.60668E+00_JPRB, 0.60291E+00_JPRB, 0.59750E+00_JPRB, 0.59412E+00_JPRB ,&
&  0.59253E+00_JPRB, 0.59434E+00_JPRB, 0.59879E+00_JPRB, 0.60958E+00_JPRB, 0.61979E+00_JPRB, 0.63091E+00_JPRB, 0.64657E+00_JPRB ,&
&  0.66992E+00_JPRB, 0.69990E+00_JPRB, 0.72723E+00_JPRB, 0.73760E+00_JPRB, 0.73115E+00_JPRB, 0.71620E+00_JPRB, 0.69561E+00_JPRB ,&
&  0.67259E+00_JPRB, 0.65143E+00_JPRB, 0.63822E+00_JPRB, 0.63082E+00_JPRB, 0.62319E+00_JPRB, 0.61616E+00_JPRB, 0.61032E+00_JPRB ,&
&  0.60524E+00_JPRB, 0.59906E+00_JPRB, 0.59883E+00_JPRB, 0.59835E+00_JPRB, 0.59639E+00_JPRB, 0.58950E+00_JPRB, 0.58242E+00_JPRB ,&
&  0.57613E+00_JPRB, 0.57176E+00_JPRB, 0.57258E+00_JPRB, 0.57899E+00_JPRB, 0.58869E+00_JPRB, 0.60592E+00_JPRB, 0.62804E+00_JPRB ,&
&  0.64585E+00_JPRB, 0.66434E+00_JPRB, 0.68096E+00_JPRB, 0.73368E+00_JPRB, 0.76760E+00_JPRB, 0.78671E+00_JPRB, 0.79554E+00_JPRB ,&
&  0.79614E+00_JPRB, 0.79752E+00_JPRB, 0.62434E+00_JPRB, 0.69498E+00_JPRB, 0.71329E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.73722E+00_JPRB, 0.73053E+00_JPRB, 0.72025E+00_JPRB, 0.71546E+00_JPRB, 0.71287E+00_JPRB, 0.69784E+00_JPRB, 0.68481E+00_JPRB ,&
&  0.67618E+00_JPRB, 0.67442E+00_JPRB, 0.67578E+00_JPRB, 0.67537E+00_JPRB, 0.66927E+00_JPRB, 0.65846E+00_JPRB, 0.65234E+00_JPRB ,&
&  0.64296E+00_JPRB, 0.63452E+00_JPRB, 0.63210E+00_JPRB, 0.63113E+00_JPRB, 0.62531E+00_JPRB, 0.61731E+00_JPRB, 0.61032E+00_JPRB ,&
&  0.59600E+00_JPRB, 0.58648E+00_JPRB, 0.58429E+00_JPRB, 0.58736E+00_JPRB, 0.59088E+00_JPRB, 0.58973E+00_JPRB, 0.60764E+00_JPRB ,&
&  0.65940E+00_JPRB, 0.60450E+00_JPRB, 0.61613E+00_JPRB, 0.74849E+00_JPRB, 0.83198E+00_JPRB, 0.84895E+00_JPRB, 0.86901E+00_JPRB ,&
&  0.89119E+00_JPRB, 0.88436E+00_JPRB, 0.85990E+00_JPRB, 0.80883E+00_JPRB, 0.76889E+00_JPRB, 0.74108E+00_JPRB, 0.71713E+00_JPRB ,&
&  0.69945E+00_JPRB, 0.68515E+00_JPRB, 0.67859E+00_JPRB, 0.66940E+00_JPRB, 0.66533E+00_JPRB, 0.65572E+00_JPRB, 0.65254E+00_JPRB ,&
&  0.65330E+00_JPRB, 0.65937E+00_JPRB, 0.66573E+00_JPRB, 0.66915E+00_JPRB, 0.67783E+00_JPRB, 0.69317E+00_JPRB, 0.72335E+00_JPRB ,&
&  0.76724E+00_JPRB, 0.81983E+00_JPRB, 0.84954E+00_JPRB, 0.85137E+00_JPRB, 0.83925E+00_JPRB, 0.81672E+00_JPRB, 0.78607E+00_JPRB ,&
&  0.75125E+00_JPRB, 0.71904E+00_JPRB, 0.69645E+00_JPRB, 0.68534E+00_JPRB, 0.67955E+00_JPRB, 0.67575E+00_JPRB, 0.67124E+00_JPRB ,&
&  0.66699E+00_JPRB, 0.66569E+00_JPRB, 0.66595E+00_JPRB, 0.66433E+00_JPRB, 0.66139E+00_JPRB, 0.65027E+00_JPRB, 0.63268E+00_JPRB ,&
&  0.61989E+00_JPRB, 0.61557E+00_JPRB, 0.62271E+00_JPRB, 0.62916E+00_JPRB, 0.63779E+00_JPRB, 0.65734E+00_JPRB, 0.67969E+00_JPRB ,&
&  0.69707E+00_JPRB, 0.72003E+00_JPRB, 0.74550E+00_JPRB, 0.78863E+00_JPRB, 0.83597E+00_JPRB, 0.83053E+00_JPRB, 0.83069E+00_JPRB ,&
&  0.81966E+00_JPRB, 0.79640E+00_JPRB, 0.77560E+00_JPRB, 0.65858E+00_JPRB, 0.72197E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.75362E+00_JPRB, 0.74619E+00_JPRB, 0.73697E+00_JPRB, 0.73166E+00_JPRB, 0.72589E+00_JPRB, 0.71297E+00_JPRB, 0.70217E+00_JPRB ,&
&  0.69797E+00_JPRB, 0.69218E+00_JPRB, 0.68635E+00_JPRB, 0.67806E+00_JPRB, 0.66711E+00_JPRB, 0.65938E+00_JPRB, 0.65862E+00_JPRB ,&
&  0.65511E+00_JPRB, 0.65326E+00_JPRB, 0.65204E+00_JPRB, 0.64683E+00_JPRB, 0.64039E+00_JPRB, 0.63095E+00_JPRB, 0.61583E+00_JPRB ,&
&  0.59720E+00_JPRB, 0.59124E+00_JPRB, 0.59974E+00_JPRB, 0.60702E+00_JPRB, 0.61185E+00_JPRB, 0.61432E+00_JPRB, 0.62598E+00_JPRB ,&
&  0.66728E+00_JPRB, 0.67746E+00_JPRB, 0.79192E+00_JPRB, 0.95902E+00_JPRB, 0.10470E+01_JPRB, 0.10263E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.97126E+00_JPRB, 0.92840E+00_JPRB, 0.88263E+00_JPRB, 0.83037E+00_JPRB, 0.78630E+00_JPRB, 0.75723E+00_JPRB, 0.74371E+00_JPRB ,&
&  0.73534E+00_JPRB, 0.72762E+00_JPRB, 0.72405E+00_JPRB, 0.71525E+00_JPRB, 0.70746E+00_JPRB, 0.70148E+00_JPRB, 0.70225E+00_JPRB ,&
&  0.70464E+00_JPRB, 0.70822E+00_JPRB, 0.71016E+00_JPRB, 0.71291E+00_JPRB, 0.71968E+00_JPRB, 0.73916E+00_JPRB, 0.78780E+00_JPRB ,&
&  0.85355E+00_JPRB, 0.91711E+00_JPRB, 0.93915E+00_JPRB, 0.92546E+00_JPRB, 0.91351E+00_JPRB, 0.90182E+00_JPRB, 0.86452E+00_JPRB ,&
&  0.81334E+00_JPRB, 0.77959E+00_JPRB, 0.75415E+00_JPRB, 0.73333E+00_JPRB, 0.71905E+00_JPRB, 0.71017E+00_JPRB, 0.70369E+00_JPRB ,&
&  0.69984E+00_JPRB, 0.69783E+00_JPRB, 0.69521E+00_JPRB, 0.69362E+00_JPRB, 0.69121E+00_JPRB, 0.68164E+00_JPRB, 0.66920E+00_JPRB ,&
&  0.65588E+00_JPRB, 0.65623E+00_JPRB, 0.66871E+00_JPRB, 0.67543E+00_JPRB, 0.68139E+00_JPRB, 0.69534E+00_JPRB, 0.71623E+00_JPRB ,&
&  0.73194E+00_JPRB, 0.75927E+00_JPRB, 0.77718E+00_JPRB, 0.86248E+00_JPRB, 0.87966E+00_JPRB, 0.86077E+00_JPRB, 0.84900E+00_JPRB ,&
&  0.82323E+00_JPRB, 0.78758E+00_JPRB, 0.75787E+00_JPRB, 0.76253E+00_JPRB, 0.71996E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.84615E+00_JPRB, 0.80441E+00_JPRB, 0.76416E+00_JPRB, 0.73892E+00_JPRB, 0.72757E+00_JPRB, 0.72260E+00_JPRB, 0.71894E+00_JPRB ,&
&  0.71334E+00_JPRB, 0.70588E+00_JPRB, 0.68783E+00_JPRB, 0.67456E+00_JPRB, 0.66552E+00_JPRB, 0.66263E+00_JPRB, 0.66006E+00_JPRB ,&
&  0.65978E+00_JPRB, 0.65819E+00_JPRB, 0.65251E+00_JPRB, 0.65039E+00_JPRB, 0.65110E+00_JPRB, 0.64552E+00_JPRB, 0.62835E+00_JPRB ,&
&  0.61098E+00_JPRB, 0.61210E+00_JPRB, 0.61721E+00_JPRB, 0.62083E+00_JPRB, 0.62783E+00_JPRB, 0.63738E+00_JPRB, 0.65350E+00_JPRB ,&
&  0.68386E+00_JPRB, 0.72973E+00_JPRB, 0.10008E+01_JPRB, 0.11715E+01_JPRB, 0.12472E+01_JPRB, 0.12602E+01_JPRB, 0.11867E+01_JPRB ,&
&  0.11059E+01_JPRB, 0.10119E+01_JPRB, 0.93655E+00_JPRB, 0.86075E+00_JPRB, 0.81061E+00_JPRB, 0.77628E+00_JPRB, 0.75370E+00_JPRB ,&
&  0.74272E+00_JPRB, 0.73847E+00_JPRB, 0.73462E+00_JPRB, 0.72894E+00_JPRB, 0.72510E+00_JPRB, 0.72320E+00_JPRB, 0.72329E+00_JPRB ,&
&  0.72594E+00_JPRB, 0.73247E+00_JPRB, 0.73711E+00_JPRB, 0.74351E+00_JPRB, 0.75667E+00_JPRB, 0.81737E+00_JPRB, 0.92546E+00_JPRB ,&
&  0.10209E+01_JPRB, 0.10716E+01_JPRB, 0.10549E+01_JPRB, 0.10376E+01_JPRB, 0.10248E+01_JPRB, 0.98625E+00_JPRB, 0.94149E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.84760E+00_JPRB, 0.79968E+00_JPRB, 0.76185E+00_JPRB, 0.73852E+00_JPRB, 0.72571E+00_JPRB, 0.71959E+00_JPRB ,&
&  0.72073E+00_JPRB, 0.72734E+00_JPRB, 0.72798E+00_JPRB, 0.72974E+00_JPRB, 0.72523E+00_JPRB, 0.71879E+00_JPRB, 0.71472E+00_JPRB ,&
&  0.71123E+00_JPRB, 0.70841E+00_JPRB, 0.72220E+00_JPRB, 0.73053E+00_JPRB, 0.73621E+00_JPRB, 0.74665E+00_JPRB, 0.75983E+00_JPRB ,&
&  0.77739E+00_JPRB, 0.81834E+00_JPRB, 0.81201E+00_JPRB, 0.89932E+00_JPRB, 0.98444E+00_JPRB, 0.92546E+00_JPRB, 0.86270E+00_JPRB ,&
&  0.80800E+00_JPRB, 0.77242E+00_JPRB, 0.75986E+00_JPRB, 0.78803E+00_JPRB, 0.84305E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.81009E+00_JPRB, 0.75823E+00_JPRB, 0.72980E+00_JPRB, 0.72498E+00_JPRB, 0.72909E+00_JPRB, 0.72406E+00_JPRB, 0.72590E+00_JPRB ,&
&  0.71815E+00_JPRB, 0.70317E+00_JPRB, 0.68689E+00_JPRB, 0.67242E+00_JPRB, 0.66566E+00_JPRB, 0.66025E+00_JPRB, 0.65595E+00_JPRB ,&
&  0.65280E+00_JPRB, 0.64986E+00_JPRB, 0.64384E+00_JPRB, 0.64625E+00_JPRB, 0.64747E+00_JPRB, 0.64507E+00_JPRB, 0.62279E+00_JPRB ,&
&  0.61186E+00_JPRB, 0.61316E+00_JPRB, 0.61853E+00_JPRB, 0.62347E+00_JPRB, 0.63160E+00_JPRB, 0.64358E+00_JPRB, 0.65575E+00_JPRB ,&
&  0.68348E+00_JPRB, 0.82112E+00_JPRB, 0.11419E+01_JPRB, 0.13508E+01_JPRB, 0.14499E+01_JPRB, 0.13861E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11092E+01_JPRB, 0.10584E+01_JPRB, 0.95428E+00_JPRB, 0.85140E+00_JPRB, 0.79687E+00_JPRB, 0.77010E+00_JPRB ,&
&  0.75858E+00_JPRB, 0.75517E+00_JPRB, 0.75030E+00_JPRB, 0.74575E+00_JPRB, 0.74087E+00_JPRB, 0.73996E+00_JPRB, 0.74397E+00_JPRB ,&
&  0.74980E+00_JPRB, 0.75005E+00_JPRB, 0.75662E+00_JPRB, 0.76681E+00_JPRB, 0.87950E+00_JPRB, 0.10085E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12784E+01_JPRB, 0.12295E+01_JPRB, 0.11511E+01_JPRB, 0.10753E+01_JPRB, 0.10086E+01_JPRB, 0.95116E+00_JPRB ,&
&  0.90045E+00_JPRB, 0.85618E+00_JPRB, 0.81996E+00_JPRB, 0.80004E+00_JPRB, 0.79145E+00_JPRB, 0.77667E+00_JPRB, 0.77036E+00_JPRB ,&
&  0.77195E+00_JPRB, 0.77649E+00_JPRB, 0.77475E+00_JPRB, 0.76410E+00_JPRB, 0.74880E+00_JPRB, 0.73963E+00_JPRB, 0.73634E+00_JPRB ,&
&  0.74438E+00_JPRB, 0.74862E+00_JPRB, 0.77956E+00_JPRB, 0.78370E+00_JPRB, 0.78557E+00_JPRB, 0.79023E+00_JPRB, 0.80632E+00_JPRB ,&
&  0.83823E+00_JPRB, 0.88230E+00_JPRB, 0.88156E+00_JPRB, 0.91821E+00_JPRB, 0.96461E+00_JPRB, 0.96001E+00_JPRB, 0.85409E+00_JPRB ,&
&  0.78013E+00_JPRB, 0.74817E+00_JPRB, 0.74602E+00_JPRB, 0.77421E+00_JPRB, 0.82550E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.76924E+00_JPRB, 0.73824E+00_JPRB, 0.72515E+00_JPRB, 0.72297E+00_JPRB, 0.71566E+00_JPRB, 0.71675E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.71672E+00_JPRB, 0.70173E+00_JPRB, 0.68805E+00_JPRB, 0.67299E+00_JPRB, 0.66780E+00_JPRB, 0.66030E+00_JPRB, 0.65833E+00_JPRB ,&
&  0.65088E+00_JPRB, 0.64707E+00_JPRB, 0.64007E+00_JPRB, 0.63754E+00_JPRB, 0.63692E+00_JPRB, 0.63145E+00_JPRB, 0.61148E+00_JPRB ,&
&  0.60640E+00_JPRB, 0.60559E+00_JPRB, 0.61378E+00_JPRB, 0.62516E+00_JPRB, 0.63911E+00_JPRB, 0.64582E+00_JPRB, 0.64854E+00_JPRB ,&
&  0.70375E+00_JPRB, 0.84219E+00_JPRB, 0.13875E+01_JPRB, 0.15201E+01_JPRB, 0.15402E+01_JPRB, 0.13841E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11427E+01_JPRB, 0.11275E+01_JPRB, 0.10409E+01_JPRB, 0.92466E+00_JPRB, 0.83637E+00_JPRB, 0.80732E+00_JPRB ,&
&  0.79133E+00_JPRB, 0.77665E+00_JPRB, 0.77009E+00_JPRB, 0.76962E+00_JPRB, 0.76602E+00_JPRB, 0.76463E+00_JPRB, 0.77240E+00_JPRB ,&
&  0.77865E+00_JPRB, 0.78901E+00_JPRB, 0.81414E+00_JPRB, 0.88654E+00_JPRB, 0.10488E+01_JPRB, 0.10224E+01_JPRB, 0.12509E+01_JPRB ,&
&  0.16233E+01_JPRB, 0.15627E+01_JPRB, 0.13864E+01_JPRB, 0.11745E+01_JPRB, 0.10474E+01_JPRB, 0.99206E+00_JPRB, 0.94790E+00_JPRB ,&
&  0.91138E+00_JPRB, 0.87666E+00_JPRB, 0.84505E+00_JPRB, 0.83652E+00_JPRB, 0.82608E+00_JPRB, 0.81132E+00_JPRB, 0.80355E+00_JPRB ,&
&  0.81032E+00_JPRB, 0.81462E+00_JPRB, 0.80255E+00_JPRB, 0.77838E+00_JPRB, 0.76173E+00_JPRB, 0.75969E+00_JPRB, 0.75774E+00_JPRB ,&
&  0.76062E+00_JPRB, 0.77226E+00_JPRB, 0.80861E+00_JPRB, 0.78063E+00_JPRB, 0.79517E+00_JPRB, 0.83203E+00_JPRB, 0.85318E+00_JPRB ,&
&  0.85710E+00_JPRB, 0.88124E+00_JPRB, 0.91387E+00_JPRB, 0.91504E+00_JPRB, 0.98289E+00_JPRB, 0.95798E+00_JPRB, 0.82289E+00_JPRB ,&
&  0.75027E+00_JPRB, 0.73179E+00_JPRB, 0.75345E+00_JPRB, 0.78584E+00_JPRB, 0.82298E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.74192E+00_JPRB, 0.72186E+00_JPRB, 0.71432E+00_JPRB, 0.70641E+00_JPRB, 0.69333E+00_JPRB, 0.67980E+00_JPRB, 0.68059E+00_JPRB ,&
&  0.68414E+00_JPRB, 0.68679E+00_JPRB, 0.67849E+00_JPRB, 0.66316E+00_JPRB, 0.66101E+00_JPRB, 0.65811E+00_JPRB, 0.65349E+00_JPRB ,&
&  0.64883E+00_JPRB, 0.64791E+00_JPRB, 0.63917E+00_JPRB, 0.63392E+00_JPRB, 0.63543E+00_JPRB, 0.63237E+00_JPRB, 0.62371E+00_JPRB ,&
&  0.61453E+00_JPRB, 0.60948E+00_JPRB, 0.61521E+00_JPRB, 0.62121E+00_JPRB, 0.63522E+00_JPRB, 0.63763E+00_JPRB, 0.63888E+00_JPRB ,&
&  0.70544E+00_JPRB, 0.65627E+00_JPRB, 0.13913E+01_JPRB, 0.18386E+01_JPRB, 0.17721E+01_JPRB, 0.14870E+01_JPRB, 0.12987E+01_JPRB ,&
&  0.12631E+01_JPRB, 0.11887E+01_JPRB, 0.11342E+01_JPRB, 0.10554E+01_JPRB, 0.97942E+00_JPRB, 0.89955E+00_JPRB, 0.86122E+00_JPRB ,&
&  0.83605E+00_JPRB, 0.81167E+00_JPRB, 0.80498E+00_JPRB, 0.80334E+00_JPRB, 0.80815E+00_JPRB, 0.81093E+00_JPRB, 0.82687E+00_JPRB ,&
&  0.84173E+00_JPRB, 0.87521E+00_JPRB, 0.96694E+00_JPRB, 0.11176E+01_JPRB, 0.12410E+01_JPRB, 0.11827E+01_JPRB, 0.13103E+01_JPRB ,&
&  0.15044E+01_JPRB, 0.15845E+01_JPRB, 0.13614E+01_JPRB, 0.11210E+01_JPRB, 0.10088E+01_JPRB, 0.96809E+00_JPRB, 0.92947E+00_JPRB ,&
&  0.89517E+00_JPRB, 0.86148E+00_JPRB, 0.85299E+00_JPRB, 0.84096E+00_JPRB, 0.81967E+00_JPRB, 0.79840E+00_JPRB, 0.80692E+00_JPRB ,&
&  0.81845E+00_JPRB, 0.81470E+00_JPRB, 0.79703E+00_JPRB, 0.78699E+00_JPRB, 0.77816E+00_JPRB, 0.76271E+00_JPRB, 0.75737E+00_JPRB ,&
&  0.75696E+00_JPRB, 0.77928E+00_JPRB, 0.79698E+00_JPRB, 0.79463E+00_JPRB, 0.80437E+00_JPRB, 0.83045E+00_JPRB, 0.86037E+00_JPRB ,&
&  0.90414E+00_JPRB, 0.92303E+00_JPRB, 0.90296E+00_JPRB, 0.91170E+00_JPRB, 0.87356E+00_JPRB, 0.88018E+00_JPRB, 0.76997E+00_JPRB ,&
&  0.73370E+00_JPRB, 0.72888E+00_JPRB, 0.74390E+00_JPRB, 0.75781E+00_JPRB, 0.77188E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.71313E+00_JPRB, 0.70198E+00_JPRB, 0.69885E+00_JPRB, 0.68772E+00_JPRB, 0.66867E+00_JPRB, 0.65176E+00_JPRB, 0.64759E+00_JPRB ,&
&  0.64453E+00_JPRB, 0.64866E+00_JPRB, 0.65680E+00_JPRB, 0.64849E+00_JPRB, 0.64871E+00_JPRB, 0.65338E+00_JPRB, 0.65398E+00_JPRB ,&
&  0.64657E+00_JPRB, 0.64292E+00_JPRB, 0.63800E+00_JPRB, 0.63419E+00_JPRB, 0.63226E+00_JPRB, 0.62240E+00_JPRB, 0.61726E+00_JPRB ,&
&  0.61673E+00_JPRB, 0.62233E+00_JPRB, 0.62758E+00_JPRB, 0.62930E+00_JPRB, 0.64007E+00_JPRB, 0.64292E+00_JPRB, 0.64640E+00_JPRB ,&
&  0.70166E+00_JPRB, 0.13244E+01_JPRB, 0.11675E+01_JPRB, 0.21296E+01_JPRB, 0.18458E+01_JPRB, 0.14995E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.12095E+01_JPRB, 0.12282E+01_JPRB, 0.11380E+01_JPRB, 0.10597E+01_JPRB, 0.99663E+00_JPRB, 0.93168E+00_JPRB, 0.87377E+00_JPRB ,&
&  0.82901E+00_JPRB, 0.80963E+00_JPRB, 0.81253E+00_JPRB, 0.82584E+00_JPRB, 0.85314E+00_JPRB, 0.88782E+00_JPRB, 0.91762E+00_JPRB ,&
&  0.97687E+00_JPRB, 0.11006E+01_JPRB, 0.12753E+01_JPRB, 0.14609E+01_JPRB, 0.13632E+01_JPRB, 0.12360E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.14865E+01_JPRB, 0.12634E+01_JPRB, 0.10624E+01_JPRB, 0.98606E+00_JPRB, 0.95250E+00_JPRB, 0.90896E+00_JPRB ,&
&  0.86874E+00_JPRB, 0.84718E+00_JPRB, 0.85573E+00_JPRB, 0.83032E+00_JPRB, 0.80033E+00_JPRB, 0.79497E+00_JPRB, 0.79883E+00_JPRB ,&
&  0.80378E+00_JPRB, 0.79338E+00_JPRB, 0.80934E+00_JPRB, 0.81058E+00_JPRB, 0.80299E+00_JPRB, 0.78834E+00_JPRB, 0.77769E+00_JPRB ,&
&  0.78032E+00_JPRB, 0.80323E+00_JPRB, 0.80269E+00_JPRB, 0.79339E+00_JPRB, 0.80110E+00_JPRB, 0.81250E+00_JPRB, 0.82176E+00_JPRB ,&
&  0.89270E+00_JPRB, 0.91129E+00_JPRB, 0.88859E+00_JPRB, 0.87652E+00_JPRB, 0.85938E+00_JPRB, 0.87843E+00_JPRB, 0.74376E+00_JPRB ,&
&  0.72347E+00_JPRB, 0.71848E+00_JPRB, 0.72593E+00_JPRB, 0.73664E+00_JPRB, 0.73302E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.69991E+00_JPRB, 0.69177E+00_JPRB, 0.68849E+00_JPRB, 0.66816E+00_JPRB, 0.64425E+00_JPRB, 0.62716E+00_JPRB, 0.62127E+00_JPRB ,&
&  0.62396E+00_JPRB, 0.63608E+00_JPRB, 0.64552E+00_JPRB, 0.63315E+00_JPRB, 0.62650E+00_JPRB, 0.63453E+00_JPRB, 0.63696E+00_JPRB ,&
&  0.63670E+00_JPRB, 0.63328E+00_JPRB, 0.63119E+00_JPRB, 0.63081E+00_JPRB, 0.62725E+00_JPRB, 0.61646E+00_JPRB, 0.61677E+00_JPRB ,&
&  0.63024E+00_JPRB, 0.64272E+00_JPRB, 0.64759E+00_JPRB, 0.64837E+00_JPRB, 0.66518E+00_JPRB, 0.67079E+00_JPRB, 0.68225E+00_JPRB ,&
&  0.74338E+00_JPRB, 0.13202E+01_JPRB, 0.12731E+01_JPRB, 0.22902E+01_JPRB, 0.19369E+01_JPRB, 0.15927E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.13000E+01_JPRB, 0.11291E+01_JPRB, 0.99094E+00_JPRB, 0.91998E+00_JPRB, 0.90047E+00_JPRB, 0.86786E+00_JPRB ,&
&  0.83834E+00_JPRB, 0.85807E+00_JPRB, 0.89253E+00_JPRB, 0.93872E+00_JPRB, 0.99313E+00_JPRB, 0.10455E+01_JPRB, 0.11401E+01_JPRB ,&
&  0.12829E+01_JPRB, 0.14770E+01_JPRB, 0.17249E+01_JPRB, 0.16523E+01_JPRB, 0.14231E+01_JPRB, 0.13392E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.13205E+01_JPRB, 0.13612E+01_JPRB, 0.11478E+01_JPRB, 0.10131E+01_JPRB, 0.93630E+00_JPRB, 0.91425E+00_JPRB, 0.86827E+00_JPRB ,&
&  0.82406E+00_JPRB, 0.81293E+00_JPRB, 0.80935E+00_JPRB, 0.79694E+00_JPRB, 0.78726E+00_JPRB, 0.77963E+00_JPRB, 0.77132E+00_JPRB ,&
&  0.77529E+00_JPRB, 0.80132E+00_JPRB, 0.80889E+00_JPRB, 0.81224E+00_JPRB, 0.81768E+00_JPRB, 0.82022E+00_JPRB, 0.81124E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.83020E+00_JPRB, 0.80651E+00_JPRB, 0.80362E+00_JPRB, 0.81699E+00_JPRB, 0.81670E+00_JPRB, 0.82134E+00_JPRB ,&
&  0.86184E+00_JPRB, 0.87701E+00_JPRB, 0.84547E+00_JPRB, 0.79900E+00_JPRB, 0.81050E+00_JPRB, 0.82477E+00_JPRB, 0.73231E+00_JPRB ,&
&  0.72237E+00_JPRB, 0.71736E+00_JPRB, 0.71999E+00_JPRB, 0.71772E+00_JPRB, 0.70917E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.66376E+00_JPRB, 0.65626E+00_JPRB, 0.64735E+00_JPRB, 0.60620E+00_JPRB, 0.57057E+00_JPRB, 0.55628E+00_JPRB, 0.57187E+00_JPRB ,&
&  0.60571E+00_JPRB, 0.62906E+00_JPRB, 0.62675E+00_JPRB, 0.61351E+00_JPRB, 0.60811E+00_JPRB, 0.61208E+00_JPRB, 0.62042E+00_JPRB ,&
&  0.62098E+00_JPRB, 0.61137E+00_JPRB, 0.60349E+00_JPRB, 0.60928E+00_JPRB, 0.61127E+00_JPRB, 0.62051E+00_JPRB, 0.64154E+00_JPRB ,&
&  0.67111E+00_JPRB, 0.68638E+00_JPRB, 0.68767E+00_JPRB, 0.69525E+00_JPRB, 0.73001E+00_JPRB, 0.73653E+00_JPRB, 0.75305E+00_JPRB ,&
&  0.88400E+00_JPRB, 0.14803E+01_JPRB, 0.16394E+01_JPRB, 0.20009E+01_JPRB, 0.17917E+01_JPRB, 0.14405E+01_JPRB, 0.12953E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.10502E+01_JPRB, 0.10134E+01_JPRB, 0.91137E+00_JPRB, 0.91695E+00_JPRB, 0.92731E+00_JPRB, 0.93484E+00_JPRB ,&
&  0.97029E+00_JPRB, 0.10568E+01_JPRB, 0.11196E+01_JPRB, 0.11771E+01_JPRB, 0.12365E+01_JPRB, 0.13423E+01_JPRB, 0.15140E+01_JPRB ,&
&  0.16907E+01_JPRB, 0.18882E+01_JPRB, 0.20588E+01_JPRB, 0.17665E+01_JPRB, 0.13808E+01_JPRB, 0.13451E+01_JPRB, 0.13061E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.12072E+01_JPRB, 0.10627E+01_JPRB, 0.94958E+00_JPRB, 0.84539E+00_JPRB, 0.87050E+00_JPRB, 0.81963E+00_JPRB ,&
&  0.77040E+00_JPRB, 0.76922E+00_JPRB, 0.76193E+00_JPRB, 0.76650E+00_JPRB, 0.76577E+00_JPRB, 0.75713E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.77607E+00_JPRB, 0.78888E+00_JPRB, 0.80091E+00_JPRB, 0.80963E+00_JPRB, 0.82384E+00_JPRB, 0.83499E+00_JPRB, 0.83562E+00_JPRB ,&
&  0.83463E+00_JPRB, 0.85114E+00_JPRB, 0.83323E+00_JPRB, 0.85039E+00_JPRB, 0.85387E+00_JPRB, 0.84630E+00_JPRB, 0.81482E+00_JPRB ,&
&  0.81715E+00_JPRB, 0.83279E+00_JPRB, 0.79700E+00_JPRB, 0.77734E+00_JPRB, 0.78332E+00_JPRB, 0.76975E+00_JPRB, 0.72689E+00_JPRB ,&
&  0.73073E+00_JPRB, 0.72350E+00_JPRB, 0.70924E+00_JPRB, 0.68948E+00_JPRB, 0.67493E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.64331E+00_JPRB, 0.63122E+00_JPRB, 0.60135E+00_JPRB, 0.54037E+00_JPRB, 0.52409E+00_JPRB, 0.54026E+00_JPRB, 0.60063E+00_JPRB ,&
&  0.62605E+00_JPRB, 0.63175E+00_JPRB, 0.61768E+00_JPRB, 0.60069E+00_JPRB, 0.59045E+00_JPRB, 0.58979E+00_JPRB, 0.60423E+00_JPRB ,&
&  0.61844E+00_JPRB, 0.61090E+00_JPRB, 0.60942E+00_JPRB, 0.61680E+00_JPRB, 0.63364E+00_JPRB, 0.66185E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.73128E+00_JPRB, 0.74711E+00_JPRB, 0.75803E+00_JPRB, 0.79728E+00_JPRB, 0.84844E+00_JPRB, 0.87127E+00_JPRB, 0.94517E+00_JPRB ,&
&  0.12985E+01_JPRB, 0.17826E+01_JPRB, 0.84172E+00_JPRB, 0.12893E+01_JPRB, 0.14650E+01_JPRB, 0.12797E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.92342E+00_JPRB, 0.88096E+00_JPRB, 0.96391E+00_JPRB, 0.10292E+01_JPRB, 0.11155E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.12975E+01_JPRB, 0.13630E+01_JPRB, 0.14288E+01_JPRB, 0.14932E+01_JPRB, 0.15921E+01_JPRB, 0.17368E+01_JPRB, 0.18732E+01_JPRB ,&
&  0.19606E+01_JPRB, 0.20497E+01_JPRB, 0.20412E+01_JPRB, 0.15490E+01_JPRB, 0.13125E+01_JPRB, 0.12625E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10613E+01_JPRB, 0.97924E+00_JPRB, 0.87308E+00_JPRB, 0.78361E+00_JPRB, 0.74667E+00_JPRB, 0.76094E+00_JPRB ,&
&  0.73756E+00_JPRB, 0.71954E+00_JPRB, 0.71131E+00_JPRB, 0.71296E+00_JPRB, 0.71328E+00_JPRB, 0.71666E+00_JPRB, 0.72685E+00_JPRB ,&
&  0.74922E+00_JPRB, 0.76960E+00_JPRB, 0.78510E+00_JPRB, 0.79539E+00_JPRB, 0.81109E+00_JPRB, 0.83148E+00_JPRB, 0.83941E+00_JPRB ,&
&  0.85884E+00_JPRB, 0.87413E+00_JPRB, 0.89100E+00_JPRB, 0.89021E+00_JPRB, 0.86862E+00_JPRB, 0.85287E+00_JPRB, 0.84766E+00_JPRB ,&
&  0.84424E+00_JPRB, 0.83167E+00_JPRB, 0.75829E+00_JPRB, 0.74518E+00_JPRB, 0.77851E+00_JPRB, 0.76140E+00_JPRB, 0.80470E+00_JPRB ,&
&  0.87152E+00_JPRB, 0.92068E+00_JPRB, 0.86848E+00_JPRB, 0.67512E+00_JPRB, 0.65194E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.61238E+00_JPRB, 0.58315E+00_JPRB, 0.52799E+00_JPRB, 0.49336E+00_JPRB, 0.51630E+00_JPRB, 0.58061E+00_JPRB, 0.63079E+00_JPRB ,&
&  0.64861E+00_JPRB, 0.64589E+00_JPRB, 0.63894E+00_JPRB, 0.63623E+00_JPRB, 0.64619E+00_JPRB, 0.63905E+00_JPRB, 0.64692E+00_JPRB ,&
&  0.66094E+00_JPRB, 0.66785E+00_JPRB, 0.67904E+00_JPRB, 0.69561E+00_JPRB, 0.71551E+00_JPRB, 0.75234E+00_JPRB, 0.78919E+00_JPRB ,&
&  0.81811E+00_JPRB, 0.84616E+00_JPRB, 0.89023E+00_JPRB, 0.95361E+00_JPRB, 0.10459E+01_JPRB, 0.12096E+01_JPRB, 0.14948E+01_JPRB ,&
&  0.20325E+01_JPRB, 0.14135E+01_JPRB, 0.74234E+00_JPRB, 0.12374E+01_JPRB, 0.12989E+01_JPRB, 0.12051E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.10233E+01_JPRB, 0.97357E+00_JPRB, 0.99869E+00_JPRB, 0.11759E+01_JPRB, 0.13505E+01_JPRB, 0.15786E+01_JPRB, 0.17086E+01_JPRB ,&
&  0.17386E+01_JPRB, 0.17710E+01_JPRB, 0.18125E+01_JPRB, 0.18732E+01_JPRB, 0.19646E+01_JPRB, 0.20602E+01_JPRB, 0.20908E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.21236E+01_JPRB, 0.20355E+01_JPRB, 0.15076E+01_JPRB, 0.12790E+01_JPRB, 0.11397E+01_JPRB, 0.10752E+01_JPRB ,&
&  0.96725E+00_JPRB, 0.93508E+00_JPRB, 0.89350E+00_JPRB, 0.81695E+00_JPRB, 0.74910E+00_JPRB, 0.70101E+00_JPRB, 0.72070E+00_JPRB ,&
&  0.71642E+00_JPRB, 0.67802E+00_JPRB, 0.64922E+00_JPRB, 0.63194E+00_JPRB, 0.65438E+00_JPRB, 0.67861E+00_JPRB, 0.67985E+00_JPRB ,&
&  0.69454E+00_JPRB, 0.71826E+00_JPRB, 0.73719E+00_JPRB, 0.75598E+00_JPRB, 0.78312E+00_JPRB, 0.81370E+00_JPRB, 0.83404E+00_JPRB ,&
&  0.85491E+00_JPRB, 0.86225E+00_JPRB, 0.86941E+00_JPRB, 0.86383E+00_JPRB, 0.82598E+00_JPRB, 0.78608E+00_JPRB, 0.79036E+00_JPRB ,&
&  0.78107E+00_JPRB, 0.77746E+00_JPRB, 0.75632E+00_JPRB, 0.76893E+00_JPRB, 0.77312E+00_JPRB, 0.79739E+00_JPRB, 0.82920E+00_JPRB ,&
&  0.85557E+00_JPRB, 0.83828E+00_JPRB, 0.75208E+00_JPRB, 0.65786E+00_JPRB, 0.63297E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.56955E+00_JPRB, 0.51036E+00_JPRB, 0.45836E+00_JPRB, 0.47444E+00_JPRB, 0.53694E+00_JPRB, 0.58969E+00_JPRB, 0.61680E+00_JPRB ,&
&  0.64792E+00_JPRB, 0.65587E+00_JPRB, 0.66245E+00_JPRB, 0.67148E+00_JPRB, 0.69101E+00_JPRB, 0.69413E+00_JPRB, 0.71132E+00_JPRB ,&
&  0.71845E+00_JPRB, 0.72111E+00_JPRB, 0.73118E+00_JPRB, 0.76303E+00_JPRB, 0.80273E+00_JPRB, 0.84387E+00_JPRB, 0.88418E+00_JPRB ,&
&  0.93113E+00_JPRB, 0.98360E+00_JPRB, 0.10595E+01_JPRB, 0.11494E+01_JPRB, 0.12705E+01_JPRB, 0.14021E+01_JPRB, 0.14995E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.59597E+00_JPRB, 0.10842E+01_JPRB, 0.13014E+01_JPRB, 0.12220E+01_JPRB, 0.11657E+01_JPRB, 0.11104E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.10761E+01_JPRB, 0.11415E+01_JPRB, 0.15168E+01_JPRB, 0.18402E+01_JPRB, 0.19827E+01_JPRB, 0.20424E+01_JPRB ,&
&  0.20941E+01_JPRB, 0.21295E+01_JPRB, 0.21808E+01_JPRB, 0.22570E+01_JPRB, 0.23435E+01_JPRB, 0.24046E+01_JPRB, 0.24330E+01_JPRB ,&
&  0.24632E+01_JPRB, 0.24505E+01_JPRB, 0.23607E+01_JPRB, 0.17528E+01_JPRB, 0.15098E+01_JPRB, 0.12611E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.10115E+01_JPRB, 0.81654E+00_JPRB, 0.77986E+00_JPRB, 0.79169E+00_JPRB, 0.74228E+00_JPRB, 0.65519E+00_JPRB, 0.64609E+00_JPRB ,&
&  0.65494E+00_JPRB, 0.63982E+00_JPRB, 0.61210E+00_JPRB, 0.59754E+00_JPRB, 0.62783E+00_JPRB, 0.63115E+00_JPRB, 0.63743E+00_JPRB ,&
&  0.63347E+00_JPRB, 0.66393E+00_JPRB, 0.69524E+00_JPRB, 0.72192E+00_JPRB, 0.74973E+00_JPRB, 0.78257E+00_JPRB, 0.79038E+00_JPRB ,&
&  0.80516E+00_JPRB, 0.80478E+00_JPRB, 0.80883E+00_JPRB, 0.79333E+00_JPRB, 0.77698E+00_JPRB, 0.74392E+00_JPRB, 0.73517E+00_JPRB ,&
&  0.73565E+00_JPRB, 0.73919E+00_JPRB, 0.73460E+00_JPRB, 0.72768E+00_JPRB, 0.75279E+00_JPRB, 0.75136E+00_JPRB, 0.75996E+00_JPRB ,&
&  0.75607E+00_JPRB, 0.73210E+00_JPRB, 0.67164E+00_JPRB, 0.62528E+00_JPRB, 0.60468E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.47797E+00_JPRB, 0.42457E+00_JPRB, 0.41944E+00_JPRB, 0.46064E+00_JPRB, 0.50243E+00_JPRB, 0.54559E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.62382E+00_JPRB, 0.64574E+00_JPRB, 0.68949E+00_JPRB, 0.70915E+00_JPRB, 0.72276E+00_JPRB, 0.72358E+00_JPRB, 0.72558E+00_JPRB ,&
&  0.74754E+00_JPRB, 0.77198E+00_JPRB, 0.79431E+00_JPRB, 0.83172E+00_JPRB, 0.87117E+00_JPRB, 0.91123E+00_JPRB, 0.96057E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.10654E+01_JPRB, 0.11212E+01_JPRB, 0.11867E+01_JPRB, 0.12360E+01_JPRB, 0.12554E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.66706E+00_JPRB, 0.10405E+01_JPRB, 0.12983E+01_JPRB, 0.12826E+01_JPRB, 0.12448E+01_JPRB, 0.12059E+01_JPRB, 0.12318E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.11676E+01_JPRB, 0.12826E+01_JPRB, 0.17828E+01_JPRB, 0.20034E+01_JPRB, 0.21115E+01_JPRB, 0.22178E+01_JPRB ,&
&  0.23255E+01_JPRB, 0.24099E+01_JPRB, 0.25103E+01_JPRB, 0.26177E+01_JPRB, 0.27376E+01_JPRB, 0.28514E+01_JPRB, 0.29650E+01_JPRB ,&
&  0.30625E+01_JPRB, 0.31249E+01_JPRB, 0.31298E+01_JPRB, 0.25083E+01_JPRB, 0.24201E+01_JPRB, 0.20220E+01_JPRB, 0.17251E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.76712E+00_JPRB, 0.76919E+00_JPRB, 0.80672E+00_JPRB, 0.73363E+00_JPRB, 0.68828E+00_JPRB, 0.62500E+00_JPRB ,&
&  0.62730E+00_JPRB, 0.62718E+00_JPRB, 0.62789E+00_JPRB, 0.61598E+00_JPRB, 0.61796E+00_JPRB, 0.58848E+00_JPRB, 0.62089E+00_JPRB ,&
&  0.63756E+00_JPRB, 0.63033E+00_JPRB, 0.63769E+00_JPRB, 0.66428E+00_JPRB, 0.69884E+00_JPRB, 0.73112E+00_JPRB, 0.74247E+00_JPRB ,&
&  0.74117E+00_JPRB, 0.72823E+00_JPRB, 0.72793E+00_JPRB, 0.72613E+00_JPRB, 0.72191E+00_JPRB, 0.71781E+00_JPRB, 0.70337E+00_JPRB ,&
&  0.68910E+00_JPRB, 0.70359E+00_JPRB, 0.74168E+00_JPRB, 0.69325E+00_JPRB, 0.73899E+00_JPRB, 0.67200E+00_JPRB, 0.65525E+00_JPRB ,&
&  0.63589E+00_JPRB, 0.61362E+00_JPRB, 0.56616E+00_JPRB, 0.54237E+00_JPRB, 0.51310E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.38860E+00_JPRB, 0.37122E+00_JPRB, 0.38076E+00_JPRB, 0.40441E+00_JPRB, 0.43258E+00_JPRB, 0.47543E+00_JPRB, 0.52674E+00_JPRB ,&
&  0.57087E+00_JPRB, 0.61365E+00_JPRB, 0.68758E+00_JPRB, 0.70327E+00_JPRB, 0.71592E+00_JPRB, 0.75350E+00_JPRB, 0.77689E+00_JPRB ,&
&  0.80332E+00_JPRB, 0.82146E+00_JPRB, 0.84438E+00_JPRB, 0.86751E+00_JPRB, 0.90619E+00_JPRB, 0.94934E+00_JPRB, 0.98376E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.10837E+01_JPRB, 0.11616E+01_JPRB, 0.12426E+01_JPRB, 0.13208E+01_JPRB, 0.13360E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.94305E+00_JPRB, 0.12643E+01_JPRB, 0.12866E+01_JPRB, 0.12828E+01_JPRB, 0.12508E+01_JPRB, 0.12174E+01_JPRB, 0.12352E+01_JPRB ,&
&  0.12686E+01_JPRB, 0.12365E+01_JPRB, 0.13042E+01_JPRB, 0.15323E+01_JPRB, 0.19363E+01_JPRB, 0.20478E+01_JPRB, 0.22171E+01_JPRB ,&
&  0.23720E+01_JPRB, 0.25118E+01_JPRB, 0.26597E+01_JPRB, 0.28427E+01_JPRB, 0.30284E+01_JPRB, 0.32047E+01_JPRB, 0.33928E+01_JPRB ,&
&  0.35592E+01_JPRB, 0.37219E+01_JPRB, 0.39042E+01_JPRB, 0.32321E+01_JPRB, 0.26078E+01_JPRB, 0.20441E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.90966E+00_JPRB, 0.73350E+00_JPRB, 0.76398E+00_JPRB, 0.80663E+00_JPRB, 0.73389E+00_JPRB, 0.68814E+00_JPRB, 0.62867E+00_JPRB ,&
&  0.62032E+00_JPRB, 0.62547E+00_JPRB, 0.63194E+00_JPRB, 0.62586E+00_JPRB, 0.63275E+00_JPRB, 0.61008E+00_JPRB, 0.60712E+00_JPRB ,&
&  0.62269E+00_JPRB, 0.61474E+00_JPRB, 0.61333E+00_JPRB, 0.62635E+00_JPRB, 0.65546E+00_JPRB, 0.68390E+00_JPRB, 0.68209E+00_JPRB ,&
&  0.69276E+00_JPRB, 0.63459E+00_JPRB, 0.61285E+00_JPRB, 0.62887E+00_JPRB, 0.63343E+00_JPRB, 0.63635E+00_JPRB, 0.65671E+00_JPRB ,&
&  0.73954E+00_JPRB, 0.84304E+00_JPRB, 0.93513E+00_JPRB, 0.93068E+00_JPRB, 0.98718E+00_JPRB, 0.77477E+00_JPRB, 0.57545E+00_JPRB ,&
&  0.53800E+00_JPRB, 0.52527E+00_JPRB, 0.48152E+00_JPRB, 0.45157E+00_JPRB, 0.41804E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.38350E+00_JPRB, 0.37742E+00_JPRB, 0.38705E+00_JPRB, 0.40379E+00_JPRB, 0.41622E+00_JPRB, 0.44483E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.51689E+00_JPRB, 0.56296E+00_JPRB, 0.60042E+00_JPRB, 0.62504E+00_JPRB, 0.66914E+00_JPRB, 0.73293E+00_JPRB, 0.75865E+00_JPRB ,&
&  0.81150E+00_JPRB, 0.84341E+00_JPRB, 0.85671E+00_JPRB, 0.86393E+00_JPRB, 0.88818E+00_JPRB, 0.92971E+00_JPRB, 0.97641E+00_JPRB ,&
&  0.10173E+01_JPRB, 0.10714E+01_JPRB, 0.11504E+01_JPRB, 0.12016E+01_JPRB, 0.12240E+01_JPRB, 0.12253E+01_JPRB, 0.96279E+00_JPRB ,&
&  0.11830E+01_JPRB, 0.12382E+01_JPRB, 0.12552E+01_JPRB, 0.12299E+01_JPRB, 0.12058E+01_JPRB, 0.11826E+01_JPRB, 0.11970E+01_JPRB ,&
&  0.12316E+01_JPRB, 0.12354E+01_JPRB, 0.12262E+01_JPRB, 0.13030E+01_JPRB, 0.16365E+01_JPRB, 0.17579E+01_JPRB, 0.19771E+01_JPRB ,&
&  0.21402E+01_JPRB, 0.22665E+01_JPRB, 0.24366E+01_JPRB, 0.26222E+01_JPRB, 0.28067E+01_JPRB, 0.29653E+01_JPRB, 0.30993E+01_JPRB ,&
&  0.32190E+01_JPRB, 0.33556E+01_JPRB, 0.34679E+01_JPRB, 0.29226E+01_JPRB, 0.22605E+01_JPRB, 0.18038E+01_JPRB, 0.12106E+01_JPRB ,&
&  0.82963E+00_JPRB, 0.73545E+00_JPRB, 0.76447E+00_JPRB, 0.78691E+00_JPRB, 0.71719E+00_JPRB, 0.67377E+00_JPRB, 0.63523E+00_JPRB ,&
&  0.63546E+00_JPRB, 0.65179E+00_JPRB, 0.66891E+00_JPRB, 0.67041E+00_JPRB, 0.64713E+00_JPRB, 0.61838E+00_JPRB, 0.59621E+00_JPRB ,&
&  0.61204E+00_JPRB, 0.60753E+00_JPRB, 0.59875E+00_JPRB, 0.62110E+00_JPRB, 0.63039E+00_JPRB, 0.69378E+00_JPRB, 0.78695E+00_JPRB ,&
&  0.78338E+00_JPRB, 0.52125E+00_JPRB, 0.53866E+00_JPRB, 0.56078E+00_JPRB, 0.60186E+00_JPRB, 0.67425E+00_JPRB, 0.82460E+00_JPRB ,&
&  0.11690E+01_JPRB, 0.19328E+01_JPRB, 0.22139E+01_JPRB, 0.14677E+01_JPRB, 0.19996E+01_JPRB, 0.12969E+01_JPRB, 0.51988E+00_JPRB ,&
&  0.49212E+00_JPRB, 0.47671E+00_JPRB, 0.44884E+00_JPRB, 0.41910E+00_JPRB, 0.39865E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.46430E+00_JPRB, 0.46639E+00_JPRB, 0.47451E+00_JPRB, 0.48010E+00_JPRB, 0.48571E+00_JPRB, 0.48955E+00_JPRB, 0.49358E+00_JPRB ,&
&  0.50724E+00_JPRB, 0.51507E+00_JPRB, 0.53511E+00_JPRB, 0.54992E+00_JPRB, 0.56383E+00_JPRB, 0.59433E+00_JPRB, 0.61915E+00_JPRB ,&
&  0.65995E+00_JPRB, 0.71273E+00_JPRB, 0.73309E+00_JPRB, 0.76241E+00_JPRB, 0.77894E+00_JPRB, 0.79593E+00_JPRB, 0.80729E+00_JPRB ,&
&  0.86941E+00_JPRB, 0.98926E+00_JPRB, 0.10688E+01_JPRB, 0.11530E+01_JPRB, 0.11521E+01_JPRB, 0.12065E+01_JPRB, 0.93485E+00_JPRB ,&
&  0.11923E+01_JPRB, 0.11964E+01_JPRB, 0.11252E+01_JPRB, 0.10865E+01_JPRB, 0.10623E+01_JPRB, 0.10422E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.10646E+01_JPRB, 0.11330E+01_JPRB, 0.11364E+01_JPRB, 0.12181E+01_JPRB, 0.13304E+01_JPRB, 0.13499E+01_JPRB, 0.15033E+01_JPRB ,&
&  0.16428E+01_JPRB, 0.17391E+01_JPRB, 0.18491E+01_JPRB, 0.19997E+01_JPRB, 0.21055E+01_JPRB, 0.21684E+01_JPRB, 0.21680E+01_JPRB ,&
&  0.21483E+01_JPRB, 0.21617E+01_JPRB, 0.22208E+01_JPRB, 0.21717E+01_JPRB, 0.17448E+01_JPRB, 0.14410E+01_JPRB, 0.11416E+01_JPRB ,&
&  0.83547E+00_JPRB, 0.71616E+00_JPRB, 0.71749E+00_JPRB, 0.78013E+00_JPRB, 0.74105E+00_JPRB, 0.70967E+00_JPRB, 0.68240E+00_JPRB ,&
&  0.66270E+00_JPRB, 0.66904E+00_JPRB, 0.67189E+00_JPRB, 0.66462E+00_JPRB, 0.64363E+00_JPRB, 0.60417E+00_JPRB, 0.56663E+00_JPRB ,&
&  0.55988E+00_JPRB, 0.54939E+00_JPRB, 0.54286E+00_JPRB, 0.54605E+00_JPRB, 0.55062E+00_JPRB, 0.57472E+00_JPRB, 0.53074E+00_JPRB ,&
&  0.57201E+00_JPRB, 0.47358E+00_JPRB, 0.49660E+00_JPRB, 0.53796E+00_JPRB, 0.60164E+00_JPRB, 0.83159E+00_JPRB, 0.12297E+01_JPRB ,&
&  0.18242E+01_JPRB, 0.20915E+01_JPRB, 0.27783E+01_JPRB, 0.40803E+01_JPRB, 0.25733E+01_JPRB, 0.16412E+01_JPRB, 0.53548E+00_JPRB ,&
&  0.52447E+00_JPRB, 0.49916E+00_JPRB, 0.49126E+00_JPRB, 0.47596E+00_JPRB, 0.47066E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.53201E+00_JPRB, 0.54111E+00_JPRB, 0.55270E+00_JPRB, 0.56310E+00_JPRB, 0.56997E+00_JPRB, 0.58165E+00_JPRB, 0.59005E+00_JPRB ,&
&  0.59963E+00_JPRB, 0.60325E+00_JPRB, 0.61042E+00_JPRB, 0.61675E+00_JPRB, 0.62632E+00_JPRB, 0.64039E+00_JPRB, 0.65428E+00_JPRB ,&
&  0.66471E+00_JPRB, 0.68145E+00_JPRB, 0.70111E+00_JPRB, 0.71405E+00_JPRB, 0.74609E+00_JPRB, 0.80671E+00_JPRB, 0.87417E+00_JPRB ,&
&  0.92663E+00_JPRB, 0.99417E+00_JPRB, 0.10560E+01_JPRB, 0.10776E+01_JPRB, 0.10657E+01_JPRB, 0.10687E+01_JPRB, 0.88063E+00_JPRB ,&
&  0.12548E+01_JPRB, 0.12294E+01_JPRB, 0.11073E+01_JPRB, 0.10317E+01_JPRB, 0.96397E+00_JPRB, 0.90400E+00_JPRB, 0.85882E+00_JPRB ,&
&  0.85363E+00_JPRB, 0.97986E+00_JPRB, 0.99471E+00_JPRB, 0.10322E+01_JPRB, 0.98368E+00_JPRB, 0.10093E+01_JPRB, 0.11098E+01_JPRB ,&
&  0.12349E+01_JPRB, 0.12746E+01_JPRB, 0.12633E+01_JPRB, 0.13189E+01_JPRB, 0.13872E+01_JPRB, 0.14085E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.13147E+01_JPRB, 0.12213E+01_JPRB, 0.10676E+01_JPRB, 0.12024E+01_JPRB, 0.11013E+01_JPRB, 0.99024E+00_JPRB, 0.90296E+00_JPRB ,&
&  0.71592E+00_JPRB, 0.70025E+00_JPRB, 0.74216E+00_JPRB, 0.82763E+00_JPRB, 0.78967E+00_JPRB, 0.76738E+00_JPRB, 0.72909E+00_JPRB ,&
&  0.69205E+00_JPRB, 0.67266E+00_JPRB, 0.64020E+00_JPRB, 0.62558E+00_JPRB, 0.62061E+00_JPRB, 0.59705E+00_JPRB, 0.55569E+00_JPRB ,&
&  0.52577E+00_JPRB, 0.52057E+00_JPRB, 0.51087E+00_JPRB, 0.50154E+00_JPRB, 0.50118E+00_JPRB, 0.45099E+00_JPRB, 0.54118E+00_JPRB ,&
&  0.49178E+00_JPRB, 0.42741E+00_JPRB, 0.51648E+00_JPRB, 0.40431E+00_JPRB, 0.51630E+00_JPRB, 0.63764E+00_JPRB, 0.82890E+00_JPRB ,&
&  0.87882E+00_JPRB, 0.11306E+01_JPRB, 0.14822E+01_JPRB, 0.11766E+01_JPRB, 0.89318E+00_JPRB, 0.59896E+00_JPRB, 0.56377E+00_JPRB ,&
&  0.55290E+00_JPRB, 0.54248E+00_JPRB, 0.54447E+00_JPRB, 0.53667E+00_JPRB, 0.53408E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.60464E+00_JPRB, 0.61753E+00_JPRB, 0.63392E+00_JPRB, 0.65525E+00_JPRB, 0.66955E+00_JPRB, 0.68949E+00_JPRB, 0.70525E+00_JPRB ,&
&  0.72629E+00_JPRB, 0.74250E+00_JPRB, 0.75616E+00_JPRB, 0.77072E+00_JPRB, 0.79026E+00_JPRB, 0.81459E+00_JPRB, 0.84553E+00_JPRB ,&
&  0.87856E+00_JPRB, 0.91112E+00_JPRB, 0.94765E+00_JPRB, 0.98919E+00_JPRB, 0.10377E+01_JPRB, 0.10966E+01_JPRB, 0.11511E+01_JPRB ,&
&  0.12013E+01_JPRB, 0.12519E+01_JPRB, 0.13019E+01_JPRB, 0.13390E+01_JPRB, 0.13434E+01_JPRB, 0.13433E+01_JPRB, 0.11571E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.13976E+01_JPRB, 0.12947E+01_JPRB, 0.11742E+01_JPRB, 0.10701E+01_JPRB, 0.97658E+00_JPRB, 0.89213E+00_JPRB ,&
&  0.88718E+00_JPRB, 0.84048E+00_JPRB, 0.82085E+00_JPRB, 0.81573E+00_JPRB, 0.77238E+00_JPRB, 0.79526E+00_JPRB, 0.84660E+00_JPRB ,&
&  0.91647E+00_JPRB, 0.10267E+01_JPRB, 0.10889E+01_JPRB, 0.11515E+01_JPRB, 0.12227E+01_JPRB, 0.11840E+01_JPRB, 0.11390E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.93775E+00_JPRB, 0.72615E+00_JPRB, 0.78780E+00_JPRB, 0.83360E+00_JPRB, 0.86912E+00_JPRB, 0.89645E+00_JPRB ,&
&  0.80408E+00_JPRB, 0.69656E+00_JPRB, 0.72956E+00_JPRB, 0.92508E+00_JPRB, 0.89484E+00_JPRB, 0.82525E+00_JPRB, 0.76598E+00_JPRB ,&
&  0.70276E+00_JPRB, 0.65600E+00_JPRB, 0.60856E+00_JPRB, 0.58924E+00_JPRB, 0.61219E+00_JPRB, 0.60744E+00_JPRB, 0.57377E+00_JPRB ,&
&  0.52859E+00_JPRB, 0.50814E+00_JPRB, 0.50423E+00_JPRB, 0.50147E+00_JPRB, 0.52689E+00_JPRB, 0.52376E+00_JPRB, 0.70732E+00_JPRB ,&
&  0.52164E+00_JPRB, 0.38571E+00_JPRB, 0.44823E+00_JPRB, 0.59229E+00_JPRB, 0.54216E+00_JPRB, 0.59747E+00_JPRB, 0.63070E+00_JPRB ,&
&  0.66333E+00_JPRB, 0.67392E+00_JPRB, 0.63031E+00_JPRB, 0.62760E+00_JPRB, 0.61829E+00_JPRB, 0.60908E+00_JPRB, 0.59924E+00_JPRB ,&
&  0.58879E+00_JPRB, 0.58911E+00_JPRB, 0.60099E+00_JPRB, 0.59920E+00_JPRB, 0.59881E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.61238E+00_JPRB, 0.62369E+00_JPRB, 0.62451E+00_JPRB, 0.63157E+00_JPRB, 0.64633E+00_JPRB, 0.65722E+00_JPRB, 0.67261E+00_JPRB ,&
&  0.68851E+00_JPRB, 0.69556E+00_JPRB, 0.70252E+00_JPRB, 0.71224E+00_JPRB, 0.71940E+00_JPRB, 0.74069E+00_JPRB, 0.76186E+00_JPRB ,&
&  0.78900E+00_JPRB, 0.80999E+00_JPRB, 0.83375E+00_JPRB, 0.87006E+00_JPRB, 0.91608E+00_JPRB, 0.98460E+00_JPRB, 0.10562E+01_JPRB ,&
&  0.11498E+01_JPRB, 0.12576E+01_JPRB, 0.13859E+01_JPRB, 0.14881E+01_JPRB, 0.15326E+01_JPRB, 0.15434E+01_JPRB, 0.16769E+01_JPRB ,&
&  0.14306E+01_JPRB, 0.15042E+01_JPRB, 0.14115E+01_JPRB, 0.12484E+01_JPRB, 0.12227E+01_JPRB, 0.11302E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.10739E+01_JPRB, 0.10199E+01_JPRB, 0.99168E+00_JPRB, 0.96626E+00_JPRB, 0.94003E+00_JPRB, 0.89373E+00_JPRB, 0.85813E+00_JPRB ,&
&  0.85772E+00_JPRB, 0.90863E+00_JPRB, 0.10135E+01_JPRB, 0.12246E+01_JPRB, 0.14200E+01_JPRB, 0.14741E+01_JPRB, 0.13961E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.12805E+01_JPRB, 0.11588E+01_JPRB, 0.11146E+01_JPRB, 0.11724E+01_JPRB, 0.12228E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.10003E+01_JPRB, 0.94451E+00_JPRB, 0.96970E+00_JPRB, 0.96682E+00_JPRB, 0.92752E+00_JPRB, 0.82614E+00_JPRB ,&
&  0.70932E+00_JPRB, 0.64038E+00_JPRB, 0.60681E+00_JPRB, 0.62148E+00_JPRB, 0.69484E+00_JPRB, 0.70796E+00_JPRB, 0.67601E+00_JPRB ,&
&  0.60137E+00_JPRB, 0.55612E+00_JPRB, 0.53455E+00_JPRB, 0.52800E+00_JPRB, 0.51410E+00_JPRB, 0.83495E+00_JPRB, 0.10471E+01_JPRB ,&
&  0.62520E+00_JPRB, 0.58984E+00_JPRB, 0.50931E+00_JPRB, 0.70774E+00_JPRB, 0.63342E+00_JPRB, 0.63903E+00_JPRB, 0.63888E+00_JPRB ,&
&  0.63112E+00_JPRB, 0.62999E+00_JPRB, 0.62732E+00_JPRB, 0.61131E+00_JPRB, 0.59957E+00_JPRB, 0.59815E+00_JPRB, 0.62485E+00_JPRB ,&
&  0.62072E+00_JPRB, 0.62214E+00_JPRB, 0.61464E+00_JPRB, 0.62319E+00_JPRB, 0.61746E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.63443E+00_JPRB, 0.63582E+00_JPRB, 0.64425E+00_JPRB, 0.66186E+00_JPRB, 0.66028E+00_JPRB, 0.65854E+00_JPRB, 0.68642E+00_JPRB ,&
&  0.67201E+00_JPRB, 0.66086E+00_JPRB, 0.66063E+00_JPRB, 0.63660E+00_JPRB, 0.65690E+00_JPRB, 0.67704E+00_JPRB, 0.71916E+00_JPRB ,&
&  0.76331E+00_JPRB, 0.79284E+00_JPRB, 0.82016E+00_JPRB, 0.85756E+00_JPRB, 0.92793E+00_JPRB, 0.98328E+00_JPRB, 0.10292E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.12274E+01_JPRB, 0.13546E+01_JPRB, 0.16423E+01_JPRB, 0.23123E+01_JPRB, 0.28143E+01_JPRB, 0.27428E+01_JPRB ,&
&  0.19949E+01_JPRB, 0.14559E+01_JPRB, 0.13579E+01_JPRB, 0.12158E+01_JPRB, 0.12549E+01_JPRB, 0.13629E+01_JPRB, 0.13736E+01_JPRB ,&
&  0.13494E+01_JPRB, 0.13460E+01_JPRB, 0.13387E+01_JPRB, 0.13362E+01_JPRB, 0.13457E+01_JPRB, 0.13507E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.12933E+01_JPRB, 0.12367E+01_JPRB, 0.12703E+01_JPRB, 0.15042E+01_JPRB, 0.16774E+01_JPRB, 0.19848E+01_JPRB, 0.19653E+01_JPRB ,&
&  0.18762E+01_JPRB, 0.17113E+01_JPRB, 0.16488E+01_JPRB, 0.16506E+01_JPRB, 0.16342E+01_JPRB, 0.16085E+01_JPRB, 0.16513E+01_JPRB ,&
&  0.16873E+01_JPRB, 0.14963E+01_JPRB, 0.10028E+01_JPRB, 0.10632E+01_JPRB, 0.10515E+01_JPRB, 0.10441E+01_JPRB, 0.87284E+00_JPRB ,&
&  0.71999E+00_JPRB, 0.68211E+00_JPRB, 0.69036E+00_JPRB, 0.74033E+00_JPRB, 0.84195E+00_JPRB, 0.90270E+00_JPRB, 0.84149E+00_JPRB ,&
&  0.72890E+00_JPRB, 0.63513E+00_JPRB, 0.58213E+00_JPRB, 0.56641E+00_JPRB, 0.59953E+00_JPRB, 0.65895E+00_JPRB, 0.76833E+00_JPRB ,&
&  0.72597E+00_JPRB, 0.84867E+00_JPRB, 0.78822E+00_JPRB, 0.10257E+01_JPRB, 0.79459E+00_JPRB, 0.75141E+00_JPRB, 0.74054E+00_JPRB ,&
&  0.73763E+00_JPRB, 0.72264E+00_JPRB, 0.70755E+00_JPRB, 0.68730E+00_JPRB, 0.66624E+00_JPRB, 0.69647E+00_JPRB, 0.73314E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.71915E+00_JPRB, 0.69841E+00_JPRB, 0.66500E+00_JPRB, 0.65736E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.69654E+00_JPRB, 0.67475E+00_JPRB, 0.68083E+00_JPRB, 0.70040E+00_JPRB, 0.71314E+00_JPRB, 0.72524E+00_JPRB, 0.73813E+00_JPRB ,&
&  0.74441E+00_JPRB, 0.74961E+00_JPRB, 0.75812E+00_JPRB, 0.76310E+00_JPRB, 0.80070E+00_JPRB, 0.84337E+00_JPRB, 0.89904E+00_JPRB ,&
&  0.98229E+00_JPRB, 0.10599E+01_JPRB, 0.11339E+01_JPRB, 0.12624E+01_JPRB, 0.13582E+01_JPRB, 0.14564E+01_JPRB, 0.16025E+01_JPRB ,&
&  0.17908E+01_JPRB, 0.19949E+01_JPRB, 0.21876E+01_JPRB, 0.23836E+01_JPRB, 0.26212E+01_JPRB, 0.28189E+01_JPRB, 0.28080E+01_JPRB ,&
&  0.21049E+01_JPRB, 0.15519E+01_JPRB, 0.15956E+01_JPRB, 0.14601E+01_JPRB, 0.14905E+01_JPRB, 0.15151E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.15283E+01_JPRB, 0.15322E+01_JPRB, 0.15360E+01_JPRB, 0.15446E+01_JPRB, 0.15925E+01_JPRB, 0.16610E+01_JPRB, 0.17533E+01_JPRB ,&
&  0.17932E+01_JPRB, 0.18265E+01_JPRB, 0.18421E+01_JPRB, 0.18268E+01_JPRB, 0.18937E+01_JPRB, 0.20728E+01_JPRB, 0.22139E+01_JPRB ,&
&  0.22639E+01_JPRB, 0.22495E+01_JPRB, 0.20235E+01_JPRB, 0.20254E+01_JPRB, 0.20474E+01_JPRB, 0.19196E+01_JPRB, 0.19907E+01_JPRB ,&
&  0.20799E+01_JPRB, 0.18226E+01_JPRB, 0.11657E+01_JPRB, 0.11397E+01_JPRB, 0.11232E+01_JPRB, 0.10328E+01_JPRB, 0.95273E+00_JPRB ,&
&  0.78427E+00_JPRB, 0.76792E+00_JPRB, 0.82106E+00_JPRB, 0.96239E+00_JPRB, 0.10422E+01_JPRB, 0.10855E+01_JPRB, 0.99444E+00_JPRB ,&
&  0.80608E+00_JPRB, 0.69233E+00_JPRB, 0.61459E+00_JPRB, 0.59763E+00_JPRB, 0.59144E+00_JPRB, 0.68254E+00_JPRB, 0.73408E+00_JPRB ,&
&  0.84318E+00_JPRB, 0.98461E+00_JPRB, 0.12129E+01_JPRB, 0.10705E+01_JPRB, 0.85100E+00_JPRB, 0.88676E+00_JPRB, 0.87066E+00_JPRB ,&
&  0.85991E+00_JPRB, 0.86089E+00_JPRB, 0.85377E+00_JPRB, 0.84704E+00_JPRB, 0.83987E+00_JPRB, 0.84261E+00_JPRB, 0.83102E+00_JPRB ,&
&  0.82334E+00_JPRB, 0.80734E+00_JPRB, 0.79520E+00_JPRB, 0.75704E+00_JPRB, 0.72556E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.77279E+00_JPRB, 0.74570E+00_JPRB, 0.72919E+00_JPRB, 0.73603E+00_JPRB, 0.75740E+00_JPRB, 0.77451E+00_JPRB, 0.78329E+00_JPRB ,&
&  0.80437E+00_JPRB, 0.83354E+00_JPRB, 0.87553E+00_JPRB, 0.91338E+00_JPRB, 0.96022E+00_JPRB, 0.10050E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.12317E+01_JPRB, 0.13730E+01_JPRB, 0.15854E+01_JPRB, 0.17638E+01_JPRB, 0.19285E+01_JPRB, 0.21642E+01_JPRB ,&
&  0.23993E+01_JPRB, 0.25491E+01_JPRB, 0.27221E+01_JPRB, 0.28058E+01_JPRB, 0.26886E+01_JPRB, 0.25335E+01_JPRB, 0.25908E+01_JPRB ,&
&  0.21622E+01_JPRB, 0.16877E+01_JPRB, 0.15918E+01_JPRB, 0.15581E+01_JPRB, 0.15682E+01_JPRB, 0.15617E+01_JPRB, 0.15647E+01_JPRB ,&
&  0.15790E+01_JPRB, 0.15844E+01_JPRB, 0.15900E+01_JPRB, 0.16051E+01_JPRB, 0.16444E+01_JPRB, 0.17225E+01_JPRB, 0.18357E+01_JPRB ,&
&  0.18580E+01_JPRB, 0.18990E+01_JPRB, 0.19806E+01_JPRB, 0.18231E+01_JPRB, 0.19188E+01_JPRB, 0.20791E+01_JPRB, 0.21862E+01_JPRB ,&
&  0.21715E+01_JPRB, 0.20108E+01_JPRB, 0.20319E+01_JPRB, 0.20554E+01_JPRB, 0.20699E+01_JPRB, 0.20261E+01_JPRB, 0.20093E+01_JPRB ,&
&  0.21124E+01_JPRB, 0.19820E+01_JPRB, 0.13904E+01_JPRB, 0.12940E+01_JPRB, 0.13265E+01_JPRB, 0.10695E+01_JPRB, 0.10518E+01_JPRB ,&
&  0.90913E+00_JPRB, 0.90548E+00_JPRB, 0.10257E+01_JPRB, 0.11180E+01_JPRB, 0.11703E+01_JPRB, 0.12956E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.87889E+00_JPRB, 0.76253E+00_JPRB, 0.68436E+00_JPRB, 0.65044E+00_JPRB, 0.67767E+00_JPRB, 0.74585E+00_JPRB, 0.78560E+00_JPRB ,&
&  0.85508E+00_JPRB, 0.91960E+00_JPRB, 0.99018E+00_JPRB, 0.10257E+01_JPRB, 0.94860E+00_JPRB, 0.94448E+00_JPRB, 0.94098E+00_JPRB ,&
&  0.92349E+00_JPRB, 0.92856E+00_JPRB, 0.92464E+00_JPRB, 0.90806E+00_JPRB, 0.90053E+00_JPRB, 0.88974E+00_JPRB, 0.88681E+00_JPRB ,&
&  0.88738E+00_JPRB, 0.88296E+00_JPRB, 0.86920E+00_JPRB, 0.84995E+00_JPRB, 0.81302E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.86710E+00_JPRB, 0.83047E+00_JPRB, 0.79544E+00_JPRB, 0.77916E+00_JPRB, 0.78011E+00_JPRB, 0.78469E+00_JPRB, 0.79845E+00_JPRB ,&
&  0.82613E+00_JPRB, 0.86016E+00_JPRB, 0.89875E+00_JPRB, 0.94849E+00_JPRB, 0.99718E+00_JPRB, 0.10486E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.13318E+01_JPRB, 0.15034E+01_JPRB, 0.16968E+01_JPRB, 0.18739E+01_JPRB, 0.20696E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.24840E+01_JPRB, 0.25822E+01_JPRB, 0.25513E+01_JPRB, 0.24222E+01_JPRB, 0.19052E+01_JPRB, 0.19390E+01_JPRB, 0.20566E+01_JPRB ,&
&  0.21170E+01_JPRB, 0.17313E+01_JPRB, 0.15423E+01_JPRB, 0.15096E+01_JPRB, 0.15111E+01_JPRB, 0.15228E+01_JPRB, 0.15238E+01_JPRB ,&
&  0.15405E+01_JPRB, 0.15592E+01_JPRB, 0.15761E+01_JPRB, 0.15920E+01_JPRB, 0.16139E+01_JPRB, 0.16674E+01_JPRB, 0.17776E+01_JPRB ,&
&  0.18084E+01_JPRB, 0.18399E+01_JPRB, 0.18566E+01_JPRB, 0.18175E+01_JPRB, 0.18765E+01_JPRB, 0.19512E+01_JPRB, 0.19341E+01_JPRB ,&
&  0.19013E+01_JPRB, 0.17616E+01_JPRB, 0.17742E+01_JPRB, 0.18468E+01_JPRB, 0.18165E+01_JPRB, 0.16954E+01_JPRB, 0.17019E+01_JPRB ,&
&  0.18841E+01_JPRB, 0.20731E+01_JPRB, 0.14250E+01_JPRB, 0.15721E+01_JPRB, 0.18091E+01_JPRB, 0.14986E+01_JPRB, 0.12851E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.11406E+01_JPRB, 0.12612E+01_JPRB, 0.12938E+01_JPRB, 0.12609E+01_JPRB, 0.13873E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.86795E+00_JPRB, 0.79812E+00_JPRB, 0.78600E+00_JPRB, 0.83968E+00_JPRB, 0.87226E+00_JPRB, 0.83138E+00_JPRB ,&
&  0.80731E+00_JPRB, 0.11280E+01_JPRB, 0.13369E+01_JPRB, 0.16154E+01_JPRB, 0.15935E+01_JPRB, 0.97964E+00_JPRB, 0.99530E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.10017E+01_JPRB, 0.98923E+00_JPRB, 0.97595E+00_JPRB, 0.96503E+00_JPRB, 0.95261E+00_JPRB, 0.94553E+00_JPRB ,&
&  0.94305E+00_JPRB, 0.94344E+00_JPRB, 0.94096E+00_JPRB, 0.92201E+00_JPRB, 0.89557E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.93625E+00_JPRB, 0.91542E+00_JPRB, 0.87966E+00_JPRB, 0.84503E+00_JPRB, 0.83487E+00_JPRB, 0.83128E+00_JPRB, 0.81491E+00_JPRB ,&
&  0.82405E+00_JPRB, 0.84465E+00_JPRB, 0.86999E+00_JPRB, 0.90324E+00_JPRB, 0.93626E+00_JPRB, 0.96318E+00_JPRB, 0.10255E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.12283E+01_JPRB, 0.14337E+01_JPRB, 0.16530E+01_JPRB, 0.18607E+01_JPRB, 0.20974E+01_JPRB, 0.23385E+01_JPRB ,&
&  0.26267E+01_JPRB, 0.27045E+01_JPRB, 0.24974E+01_JPRB, 0.17647E+01_JPRB, 0.15711E+01_JPRB, 0.16921E+01_JPRB, 0.15619E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.14648E+01_JPRB, 0.14333E+01_JPRB, 0.14218E+01_JPRB, 0.14267E+01_JPRB, 0.14443E+01_JPRB, 0.14385E+01_JPRB ,&
&  0.14438E+01_JPRB, 0.14626E+01_JPRB, 0.14876E+01_JPRB, 0.15219E+01_JPRB, 0.15575E+01_JPRB, 0.16051E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.16938E+01_JPRB, 0.17494E+01_JPRB, 0.17536E+01_JPRB, 0.17306E+01_JPRB, 0.16830E+01_JPRB, 0.16714E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.16266E+01_JPRB, 0.15742E+01_JPRB, 0.15643E+01_JPRB, 0.16275E+01_JPRB, 0.16614E+01_JPRB, 0.15081E+01_JPRB, 0.15371E+01_JPRB ,&
&  0.17262E+01_JPRB, 0.18897E+01_JPRB, 0.16393E+01_JPRB, 0.19876E+01_JPRB, 0.12641E+01_JPRB, 0.14490E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.14800E+01_JPRB, 0.14776E+01_JPRB, 0.15059E+01_JPRB, 0.15469E+01_JPRB, 0.15407E+01_JPRB, 0.13622E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.95502E+00_JPRB, 0.85145E+00_JPRB, 0.87767E+00_JPRB, 0.95775E+00_JPRB, 0.98652E+00_JPRB, 0.98904E+00_JPRB ,&
&  0.89412E+00_JPRB, 0.10882E+01_JPRB, 0.12219E+01_JPRB, 0.14228E+01_JPRB, 0.12610E+01_JPRB, 0.99431E+00_JPRB, 0.10194E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10361E+01_JPRB, 0.10312E+01_JPRB, 0.10083E+01_JPRB, 0.10019E+01_JPRB, 0.99288E+00_JPRB, 0.98472E+00_JPRB ,&
&  0.98545E+00_JPRB, 0.99285E+00_JPRB, 0.99366E+00_JPRB, 0.97408E+00_JPRB, 0.95059E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.10000E+01_JPRB, 0.97808E+00_JPRB, 0.96632E+00_JPRB, 0.94754E+00_JPRB, 0.91151E+00_JPRB, 0.90785E+00_JPRB, 0.90171E+00_JPRB ,&
&  0.86146E+00_JPRB, 0.81587E+00_JPRB, 0.82475E+00_JPRB, 0.83494E+00_JPRB, 0.84955E+00_JPRB, 0.87302E+00_JPRB, 0.91231E+00_JPRB ,&
&  0.94905E+00_JPRB, 0.10128E+01_JPRB, 0.11557E+01_JPRB, 0.13969E+01_JPRB, 0.16791E+01_JPRB, 0.19967E+01_JPRB, 0.23599E+01_JPRB ,&
&  0.22734E+01_JPRB, 0.16196E+01_JPRB, 0.20319E+01_JPRB, 0.21203E+01_JPRB, 0.20379E+01_JPRB, 0.20470E+01_JPRB, 0.19095E+01_JPRB ,&
&  0.15752E+01_JPRB, 0.14551E+01_JPRB, 0.14447E+01_JPRB, 0.14014E+01_JPRB, 0.13409E+01_JPRB, 0.13475E+01_JPRB, 0.13549E+01_JPRB ,&
&  0.13483E+01_JPRB, 0.13480E+01_JPRB, 0.13560E+01_JPRB, 0.13797E+01_JPRB, 0.14351E+01_JPRB, 0.15085E+01_JPRB, 0.15618E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.16194E+01_JPRB, 0.15779E+01_JPRB, 0.15327E+01_JPRB, 0.15016E+01_JPRB, 0.15135E+01_JPRB, 0.14963E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.14486E+01_JPRB, 0.14910E+01_JPRB, 0.15759E+01_JPRB, 0.16308E+01_JPRB, 0.15347E+01_JPRB, 0.16423E+01_JPRB ,&
&  0.17107E+01_JPRB, 0.18079E+01_JPRB, 0.18052E+01_JPRB, 0.21046E+01_JPRB, 0.15188E+01_JPRB, 0.16246E+01_JPRB, 0.14891E+01_JPRB ,&
&  0.16911E+01_JPRB, 0.16845E+01_JPRB, 0.16738E+01_JPRB, 0.16857E+01_JPRB, 0.17885E+01_JPRB, 0.15791E+01_JPRB, 0.14517E+01_JPRB ,&
&  0.12104E+01_JPRB, 0.10594E+01_JPRB, 0.93852E+00_JPRB, 0.10220E+01_JPRB, 0.10631E+01_JPRB, 0.10786E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.13665E+01_JPRB, 0.12492E+01_JPRB, 0.11453E+01_JPRB, 0.11693E+01_JPRB, 0.10603E+01_JPRB, 0.97847E+00_JPRB, 0.99462E+00_JPRB ,&
&  0.10181E+01_JPRB, 0.10502E+01_JPRB, 0.10418E+01_JPRB, 0.10251E+01_JPRB, 0.10137E+01_JPRB, 0.10090E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10284E+01_JPRB, 0.10377E+01_JPRB, 0.10406E+01_JPRB, 0.10312E+01_JPRB, 0.10161E+01_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.10567E+01_JPRB, 0.10288E+01_JPRB, 0.99977E+00_JPRB, 0.98784E+00_JPRB, 0.10096E+01_JPRB, 0.10213E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.13215E+01_JPRB, 0.89446E+00_JPRB, 0.80183E+00_JPRB, 0.79379E+00_JPRB, 0.79585E+00_JPRB, 0.80140E+00_JPRB, 0.83009E+00_JPRB ,&
&  0.85773E+00_JPRB, 0.88628E+00_JPRB, 0.94733E+00_JPRB, 0.10800E+01_JPRB, 0.13278E+01_JPRB, 0.17160E+01_JPRB, 0.18416E+01_JPRB ,&
&  0.17121E+01_JPRB, 0.19166E+01_JPRB, 0.22356E+01_JPRB, 0.19520E+01_JPRB, 0.18862E+01_JPRB, 0.20277E+01_JPRB, 0.21647E+01_JPRB ,&
&  0.15848E+01_JPRB, 0.14191E+01_JPRB, 0.14270E+01_JPRB, 0.13827E+01_JPRB, 0.13395E+01_JPRB, 0.13271E+01_JPRB, 0.13245E+01_JPRB ,&
&  0.13142E+01_JPRB, 0.12948E+01_JPRB, 0.12807E+01_JPRB, 0.12783E+01_JPRB, 0.13001E+01_JPRB, 0.13629E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.15326E+01_JPRB, 0.14907E+01_JPRB, 0.14269E+01_JPRB, 0.14158E+01_JPRB, 0.14566E+01_JPRB, 0.14700E+01_JPRB ,&
&  0.14591E+01_JPRB, 0.14630E+01_JPRB, 0.14936E+01_JPRB, 0.15023E+01_JPRB, 0.14470E+01_JPRB, 0.16096E+01_JPRB, 0.16162E+01_JPRB ,&
&  0.17281E+01_JPRB, 0.17792E+01_JPRB, 0.20808E+01_JPRB, 0.16687E+01_JPRB, 0.17652E+01_JPRB, 0.18356E+01_JPRB, 0.17977E+01_JPRB ,&
&  0.16391E+01_JPRB, 0.17691E+01_JPRB, 0.18546E+01_JPRB, 0.18139E+01_JPRB, 0.18396E+01_JPRB, 0.16624E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.13061E+01_JPRB, 0.13006E+01_JPRB, 0.10761E+01_JPRB, 0.11378E+01_JPRB, 0.11676E+01_JPRB, 0.11310E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.16106E+01_JPRB, 0.16112E+01_JPRB, 0.14758E+01_JPRB, 0.12577E+01_JPRB, 0.10178E+01_JPRB, 0.95049E+00_JPRB, 0.94895E+00_JPRB ,&
&  0.97588E+00_JPRB, 0.10385E+01_JPRB, 0.10398E+01_JPRB, 0.10379E+01_JPRB, 0.10281E+01_JPRB, 0.10273E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.10656E+01_JPRB, 0.10848E+01_JPRB, 0.10948E+01_JPRB, 0.10896E+01_JPRB, 0.10809E+01_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.11027E+01_JPRB, 0.10660E+01_JPRB, 0.10138E+01_JPRB, 0.96561E+00_JPRB, 0.92798E+00_JPRB, 0.91635E+00_JPRB, 0.94653E+00_JPRB ,&
&  0.11972E+01_JPRB, 0.92221E+00_JPRB, 0.82209E+00_JPRB, 0.80015E+00_JPRB, 0.80000E+00_JPRB, 0.78777E+00_JPRB, 0.78652E+00_JPRB ,&
&  0.79751E+00_JPRB, 0.81196E+00_JPRB, 0.84868E+00_JPRB, 0.89894E+00_JPRB, 0.10067E+01_JPRB, 0.12623E+01_JPRB, 0.13732E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.25406E+01_JPRB, 0.21470E+01_JPRB, 0.20402E+01_JPRB, 0.18691E+01_JPRB, 0.18998E+01_JPRB, 0.17201E+01_JPRB ,&
&  0.14608E+01_JPRB, 0.14052E+01_JPRB, 0.14447E+01_JPRB, 0.14513E+01_JPRB, 0.14521E+01_JPRB, 0.14227E+01_JPRB, 0.13900E+01_JPRB ,&
&  0.13485E+01_JPRB, 0.13162E+01_JPRB, 0.12958E+01_JPRB, 0.12793E+01_JPRB, 0.12714E+01_JPRB, 0.12981E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.13811E+01_JPRB, 0.14800E+01_JPRB, 0.14581E+01_JPRB, 0.14255E+01_JPRB, 0.14123E+01_JPRB, 0.14563E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.14210E+01_JPRB, 0.13569E+01_JPRB, 0.14521E+01_JPRB, 0.15098E+01_JPRB, 0.15462E+01_JPRB, 0.16887E+01_JPRB, 0.16707E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.18048E+01_JPRB, 0.20634E+01_JPRB, 0.17717E+01_JPRB, 0.17410E+01_JPRB, 0.19581E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.17035E+01_JPRB, 0.18123E+01_JPRB, 0.19052E+01_JPRB, 0.19322E+01_JPRB, 0.18325E+01_JPRB, 0.17204E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.15124E+01_JPRB, 0.13474E+01_JPRB, 0.11839E+01_JPRB, 0.12612E+01_JPRB, 0.11836E+01_JPRB, 0.14655E+01_JPRB ,&
&  0.18159E+01_JPRB, 0.18335E+01_JPRB, 0.18983E+01_JPRB, 0.17654E+01_JPRB, 0.13243E+01_JPRB, 0.10906E+01_JPRB, 0.98822E+00_JPRB ,&
&  0.99306E+00_JPRB, 0.10242E+01_JPRB, 0.10284E+01_JPRB, 0.10126E+01_JPRB, 0.10040E+01_JPRB, 0.10022E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10688E+01_JPRB, 0.10948E+01_JPRB, 0.11100E+01_JPRB, 0.11224E+01_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.11620E+01_JPRB, 0.11475E+01_JPRB, 0.10761E+01_JPRB, 0.10010E+01_JPRB, 0.95423E+00_JPRB, 0.92615E+00_JPRB, 0.90944E+00_JPRB ,&
&  0.10744E+01_JPRB, 0.94283E+00_JPRB, 0.84801E+00_JPRB, 0.80980E+00_JPRB, 0.79845E+00_JPRB, 0.79532E+00_JPRB, 0.78954E+00_JPRB ,&
&  0.78396E+00_JPRB, 0.78624E+00_JPRB, 0.81777E+00_JPRB, 0.83240E+00_JPRB, 0.86846E+00_JPRB, 0.99373E+00_JPRB, 0.88652E+00_JPRB ,&
&  0.12779E+01_JPRB, 0.23646E+01_JPRB, 0.22048E+01_JPRB, 0.21444E+01_JPRB, 0.19568E+01_JPRB, 0.19241E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.14281E+01_JPRB, 0.14732E+01_JPRB, 0.15271E+01_JPRB, 0.15617E+01_JPRB, 0.15595E+01_JPRB, 0.14931E+01_JPRB ,&
&  0.14242E+01_JPRB, 0.13625E+01_JPRB, 0.13362E+01_JPRB, 0.13054E+01_JPRB, 0.12838E+01_JPRB, 0.12906E+01_JPRB, 0.13023E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.13939E+01_JPRB, 0.14753E+01_JPRB, 0.13965E+01_JPRB, 0.14065E+01_JPRB, 0.14395E+01_JPRB, 0.14716E+01_JPRB ,&
&  0.13939E+01_JPRB, 0.13143E+01_JPRB, 0.14414E+01_JPRB, 0.15889E+01_JPRB, 0.17195E+01_JPRB, 0.18134E+01_JPRB, 0.18446E+01_JPRB ,&
&  0.19492E+01_JPRB, 0.19494E+01_JPRB, 0.19193E+01_JPRB, 0.17474E+01_JPRB, 0.18306E+01_JPRB, 0.20701E+01_JPRB, 0.21280E+01_JPRB ,&
&  0.21222E+01_JPRB, 0.19508E+01_JPRB, 0.19542E+01_JPRB, 0.19288E+01_JPRB, 0.18542E+01_JPRB, 0.16987E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.19127E+01_JPRB, 0.21419E+01_JPRB, 0.19261E+01_JPRB, 0.15996E+01_JPRB, 0.13619E+01_JPRB, 0.11255E+01_JPRB, 0.13731E+01_JPRB ,&
&  0.17820E+01_JPRB, 0.18688E+01_JPRB, 0.18775E+01_JPRB, 0.21922E+01_JPRB, 0.17643E+01_JPRB, 0.14656E+01_JPRB, 0.12194E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10445E+01_JPRB, 0.10462E+01_JPRB, 0.10251E+01_JPRB, 0.10068E+01_JPRB, 0.99234E+00_JPRB, 0.98078E+00_JPRB ,&
&  0.98664E+00_JPRB, 0.10190E+01_JPRB, 0.10661E+01_JPRB, 0.11181E+01_JPRB, 0.11558E+01_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.11855E+01_JPRB, 0.11852E+01_JPRB, 0.11082E+01_JPRB, 0.10138E+01_JPRB, 0.95088E+00_JPRB, 0.91790E+00_JPRB, 0.88787E+00_JPRB ,&
&  0.92662E+00_JPRB, 0.90125E+00_JPRB, 0.85397E+00_JPRB, 0.82023E+00_JPRB, 0.81563E+00_JPRB, 0.80658E+00_JPRB, 0.80133E+00_JPRB ,&
&  0.78325E+00_JPRB, 0.77668E+00_JPRB, 0.82083E+00_JPRB, 0.83125E+00_JPRB, 0.84740E+00_JPRB, 0.72682E+00_JPRB, 0.82883E+00_JPRB ,&
&  0.14532E+01_JPRB, 0.23481E+01_JPRB, 0.22530E+01_JPRB, 0.23121E+01_JPRB, 0.22525E+01_JPRB, 0.22820E+01_JPRB, 0.16224E+01_JPRB ,&
&  0.14740E+01_JPRB, 0.15034E+01_JPRB, 0.15378E+01_JPRB, 0.16275E+01_JPRB, 0.16635E+01_JPRB, 0.16532E+01_JPRB, 0.16208E+01_JPRB ,&
&  0.15365E+01_JPRB, 0.14381E+01_JPRB, 0.13799E+01_JPRB, 0.13321E+01_JPRB, 0.12776E+01_JPRB, 0.12493E+01_JPRB, 0.12716E+01_JPRB ,&
&  0.13005E+01_JPRB, 0.13308E+01_JPRB, 0.14139E+01_JPRB, 0.13863E+01_JPRB, 0.13521E+01_JPRB, 0.13695E+01_JPRB, 0.14287E+01_JPRB ,&
&  0.14384E+01_JPRB, 0.14829E+01_JPRB, 0.15308E+01_JPRB, 0.15679E+01_JPRB, 0.17314E+01_JPRB, 0.19479E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.22027E+01_JPRB, 0.21144E+01_JPRB, 0.18501E+01_JPRB, 0.18544E+01_JPRB, 0.20309E+01_JPRB, 0.23376E+01_JPRB, 0.21931E+01_JPRB ,&
&  0.19112E+01_JPRB, 0.15618E+01_JPRB, 0.15317E+01_JPRB, 0.17490E+01_JPRB, 0.18814E+01_JPRB, 0.18442E+01_JPRB, 0.21676E+01_JPRB ,&
&  0.20705E+01_JPRB, 0.12541E+01_JPRB, 0.10580E+01_JPRB, 0.13047E+01_JPRB, 0.10152E+01_JPRB, 0.88328E+00_JPRB, 0.14482E+01_JPRB ,&
&  0.18328E+01_JPRB, 0.21840E+01_JPRB, 0.22582E+01_JPRB, 0.20664E+01_JPRB, 0.20694E+01_JPRB, 0.19656E+01_JPRB, 0.17303E+01_JPRB ,&
&  0.14988E+01_JPRB, 0.13056E+01_JPRB, 0.11609E+01_JPRB, 0.10771E+01_JPRB, 0.10395E+01_JPRB, 0.10146E+01_JPRB, 0.99056E+00_JPRB ,&
&  0.98811E+00_JPRB, 0.10051E+01_JPRB, 0.10430E+01_JPRB, 0.11092E+01_JPRB, 0.11658E+01_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.12125E+01_JPRB, 0.11856E+01_JPRB, 0.11043E+01_JPRB, 0.10047E+01_JPRB, 0.94315E+00_JPRB, 0.89987E+00_JPRB, 0.87123E+00_JPRB ,&
&  0.86600E+00_JPRB, 0.85880E+00_JPRB, 0.84822E+00_JPRB, 0.83479E+00_JPRB, 0.83102E+00_JPRB, 0.82274E+00_JPRB, 0.81284E+00_JPRB ,&
&  0.79500E+00_JPRB, 0.77496E+00_JPRB, 0.84151E+00_JPRB, 0.88725E+00_JPRB, 0.86583E+00_JPRB, 0.77288E+00_JPRB, 0.94480E+00_JPRB ,&
&  0.14434E+01_JPRB, 0.22637E+01_JPRB, 0.26520E+01_JPRB, 0.27227E+01_JPRB, 0.25063E+01_JPRB, 0.24027E+01_JPRB, 0.18722E+01_JPRB ,&
&  0.16494E+01_JPRB, 0.16557E+01_JPRB, 0.16459E+01_JPRB, 0.17057E+01_JPRB, 0.17659E+01_JPRB, 0.17554E+01_JPRB, 0.16870E+01_JPRB ,&
&  0.16193E+01_JPRB, 0.15140E+01_JPRB, 0.14331E+01_JPRB, 0.13739E+01_JPRB, 0.13133E+01_JPRB, 0.12488E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.12944E+01_JPRB, 0.13555E+01_JPRB, 0.14288E+01_JPRB, 0.11955E+01_JPRB, 0.13176E+01_JPRB, 0.13878E+01_JPRB ,&
&  0.14587E+01_JPRB, 0.15457E+01_JPRB, 0.15410E+01_JPRB, 0.16605E+01_JPRB, 0.18313E+01_JPRB, 0.19690E+01_JPRB, 0.22086E+01_JPRB ,&
&  0.24976E+01_JPRB, 0.24031E+01_JPRB, 0.21429E+01_JPRB, 0.21473E+01_JPRB, 0.22841E+01_JPRB, 0.25862E+01_JPRB, 0.17679E+01_JPRB ,&
&  0.16115E+01_JPRB, 0.18793E+01_JPRB, 0.17231E+01_JPRB, 0.15620E+01_JPRB, 0.18314E+01_JPRB, 0.19408E+01_JPRB, 0.16382E+01_JPRB ,&
&  0.79457E+00_JPRB, 0.60258E+00_JPRB, 0.60682E+00_JPRB, 0.61659E+00_JPRB, 0.61529E+00_JPRB, 0.82742E+00_JPRB, 0.22622E+01_JPRB ,&
&  0.19935E+01_JPRB, 0.23953E+01_JPRB, 0.24533E+01_JPRB, 0.22591E+01_JPRB, 0.22214E+01_JPRB, 0.21668E+01_JPRB, 0.22003E+01_JPRB ,&
&  0.19787E+01_JPRB, 0.17845E+01_JPRB, 0.15939E+01_JPRB, 0.13820E+01_JPRB, 0.12065E+01_JPRB, 0.10910E+01_JPRB, 0.10410E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10487E+01_JPRB, 0.10808E+01_JPRB, 0.11341E+01_JPRB, 0.11974E+01_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.12523E+01_JPRB, 0.11873E+01_JPRB, 0.10879E+01_JPRB, 0.97928E+00_JPRB, 0.92031E+00_JPRB, 0.87460E+00_JPRB, 0.84166E+00_JPRB ,&
&  0.83228E+00_JPRB, 0.82997E+00_JPRB, 0.83278E+00_JPRB, 0.83049E+00_JPRB, 0.82557E+00_JPRB, 0.82019E+00_JPRB, 0.81068E+00_JPRB ,&
&  0.79222E+00_JPRB, 0.77635E+00_JPRB, 0.89668E+00_JPRB, 0.88900E+00_JPRB, 0.90939E+00_JPRB, 0.84996E+00_JPRB, 0.97562E+00_JPRB ,&
&  0.13637E+01_JPRB, 0.20979E+01_JPRB, 0.26807E+01_JPRB, 0.28008E+01_JPRB, 0.28880E+01_JPRB, 0.28383E+01_JPRB, 0.25614E+01_JPRB ,&
&  0.22652E+01_JPRB, 0.21180E+01_JPRB, 0.19767E+01_JPRB, 0.19038E+01_JPRB, 0.18480E+01_JPRB, 0.18119E+01_JPRB, 0.17284E+01_JPRB ,&
&  0.16367E+01_JPRB, 0.15739E+01_JPRB, 0.15043E+01_JPRB, 0.14521E+01_JPRB, 0.13875E+01_JPRB, 0.13182E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.12842E+01_JPRB, 0.13156E+01_JPRB, 0.14032E+01_JPRB, 0.12972E+01_JPRB, 0.12044E+01_JPRB, 0.13536E+01_JPRB ,&
&  0.15139E+01_JPRB, 0.17186E+01_JPRB, 0.17458E+01_JPRB, 0.18934E+01_JPRB, 0.19494E+01_JPRB, 0.20507E+01_JPRB, 0.24290E+01_JPRB ,&
&  0.27367E+01_JPRB, 0.30952E+01_JPRB, 0.24937E+01_JPRB, 0.24550E+01_JPRB, 0.26213E+01_JPRB, 0.21056E+01_JPRB, 0.19737E+01_JPRB ,&
&  0.22614E+01_JPRB, 0.19817E+01_JPRB, 0.17478E+01_JPRB, 0.14696E+01_JPRB, 0.17763E+01_JPRB, 0.15795E+01_JPRB, 0.83351E+00_JPRB ,&
&  0.63776E+00_JPRB, 0.57152E+00_JPRB, 0.53690E+00_JPRB, 0.54077E+00_JPRB, 0.57227E+00_JPRB, 0.83910E+00_JPRB, 0.15009E+01_JPRB ,&
&  0.17632E+01_JPRB, 0.22320E+01_JPRB, 0.24542E+01_JPRB, 0.24662E+01_JPRB, 0.23487E+01_JPRB, 0.23127E+01_JPRB, 0.23953E+01_JPRB ,&
&  0.25016E+01_JPRB, 0.23524E+01_JPRB, 0.20568E+01_JPRB, 0.18138E+01_JPRB, 0.16334E+01_JPRB, 0.14492E+01_JPRB, 0.13005E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.11698E+01_JPRB, 0.11731E+01_JPRB, 0.12179E+01_JPRB, 0.12804E+01_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.13022E+01_JPRB, 0.12362E+01_JPRB, 0.11187E+01_JPRB, 0.99060E+00_JPRB, 0.91561E+00_JPRB, 0.85590E+00_JPRB, 0.81967E+00_JPRB ,&
&  0.81172E+00_JPRB, 0.81422E+00_JPRB, 0.81502E+00_JPRB, 0.81148E+00_JPRB, 0.80338E+00_JPRB, 0.79698E+00_JPRB, 0.79384E+00_JPRB ,&
&  0.77445E+00_JPRB, 0.79541E+00_JPRB, 0.85010E+00_JPRB, 0.76811E+00_JPRB, 0.76324E+00_JPRB, 0.76492E+00_JPRB, 0.85789E+00_JPRB ,&
&  0.12920E+01_JPRB, 0.19949E+01_JPRB, 0.25913E+01_JPRB, 0.29334E+01_JPRB, 0.31852E+01_JPRB, 0.30852E+01_JPRB, 0.33642E+01_JPRB ,&
&  0.30726E+01_JPRB, 0.27607E+01_JPRB, 0.24791E+01_JPRB, 0.22675E+01_JPRB, 0.20564E+01_JPRB, 0.19124E+01_JPRB, 0.18214E+01_JPRB ,&
&  0.16869E+01_JPRB, 0.15759E+01_JPRB, 0.15188E+01_JPRB, 0.14714E+01_JPRB, 0.14245E+01_JPRB, 0.13805E+01_JPRB, 0.13426E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.13286E+01_JPRB, 0.13379E+01_JPRB, 0.13647E+01_JPRB, 0.14868E+01_JPRB, 0.14772E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.14796E+01_JPRB, 0.15983E+01_JPRB, 0.19048E+01_JPRB, 0.19286E+01_JPRB, 0.20242E+01_JPRB, 0.22685E+01_JPRB, 0.26628E+01_JPRB ,&
&  0.29876E+01_JPRB, 0.32655E+01_JPRB, 0.28332E+01_JPRB, 0.28732E+01_JPRB, 0.25761E+01_JPRB, 0.20911E+01_JPRB, 0.24074E+01_JPRB ,&
&  0.26569E+01_JPRB, 0.23394E+01_JPRB, 0.18932E+01_JPRB, 0.14897E+01_JPRB, 0.13874E+01_JPRB, 0.99747E+00_JPRB, 0.64178E+00_JPRB ,&
&  0.59962E+00_JPRB, 0.57775E+00_JPRB, 0.54427E+00_JPRB, 0.62565E+00_JPRB, 0.63775E+00_JPRB, 0.86146E+00_JPRB, 0.13618E+01_JPRB ,&
&  0.18625E+01_JPRB, 0.19812E+01_JPRB, 0.22331E+01_JPRB, 0.23396E+01_JPRB, 0.23444E+01_JPRB, 0.24634E+01_JPRB, 0.27936E+01_JPRB ,&
&  0.26261E+01_JPRB, 0.25834E+01_JPRB, 0.24329E+01_JPRB, 0.20956E+01_JPRB, 0.19058E+01_JPRB, 0.17818E+01_JPRB, 0.16477E+01_JPRB ,&
&  0.14935E+01_JPRB, 0.13604E+01_JPRB, 0.13165E+01_JPRB, 0.13320E+01_JPRB, 0.13436E+01_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.13545E+01_JPRB, 0.12925E+01_JPRB, 0.11757E+01_JPRB, 0.10404E+01_JPRB, 0.94192E+00_JPRB, 0.87231E+00_JPRB, 0.83862E+00_JPRB ,&
&  0.83711E+00_JPRB, 0.83390E+00_JPRB, 0.83142E+00_JPRB, 0.82299E+00_JPRB, 0.81135E+00_JPRB, 0.79730E+00_JPRB, 0.79510E+00_JPRB ,&
&  0.77249E+00_JPRB, 0.90007E+00_JPRB, 0.68564E+00_JPRB, 0.57797E+00_JPRB, 0.64023E+00_JPRB, 0.71780E+00_JPRB, 0.73058E+00_JPRB ,&
&  0.11961E+01_JPRB, 0.19103E+01_JPRB, 0.24651E+01_JPRB, 0.31540E+01_JPRB, 0.41010E+01_JPRB, 0.38930E+01_JPRB, 0.39247E+01_JPRB ,&
&  0.38016E+01_JPRB, 0.34446E+01_JPRB, 0.30485E+01_JPRB, 0.26403E+01_JPRB, 0.22606E+01_JPRB, 0.20282E+01_JPRB, 0.18785E+01_JPRB ,&
&  0.17239E+01_JPRB, 0.15917E+01_JPRB, 0.15117E+01_JPRB, 0.14802E+01_JPRB, 0.14364E+01_JPRB, 0.13997E+01_JPRB, 0.13829E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.13818E+01_JPRB, 0.13814E+01_JPRB, 0.14346E+01_JPRB, 0.14379E+01_JPRB, 0.14323E+01_JPRB, 0.17834E+01_JPRB ,&
&  0.17888E+01_JPRB, 0.18379E+01_JPRB, 0.18722E+01_JPRB, 0.20723E+01_JPRB, 0.22313E+01_JPRB, 0.24343E+01_JPRB, 0.30133E+01_JPRB ,&
&  0.28148E+01_JPRB, 0.27532E+01_JPRB, 0.28306E+01_JPRB, 0.28195E+01_JPRB, 0.25193E+01_JPRB, 0.28195E+01_JPRB, 0.38213E+01_JPRB ,&
&  0.29556E+01_JPRB, 0.29988E+01_JPRB, 0.30209E+01_JPRB, 0.29566E+01_JPRB, 0.14371E+01_JPRB, 0.97083E+00_JPRB, 0.14973E+01_JPRB ,&
&  0.16034E+01_JPRB, 0.10121E+01_JPRB, 0.79342E+00_JPRB, 0.11115E+01_JPRB, 0.90496E+00_JPRB, 0.10893E+01_JPRB, 0.15559E+01_JPRB ,&
&  0.16852E+01_JPRB, 0.16046E+01_JPRB, 0.20660E+01_JPRB, 0.23842E+01_JPRB, 0.25293E+01_JPRB, 0.26381E+01_JPRB, 0.27733E+01_JPRB ,&
&  0.28260E+01_JPRB, 0.26055E+01_JPRB, 0.25962E+01_JPRB, 0.22885E+01_JPRB, 0.20345E+01_JPRB, 0.18763E+01_JPRB, 0.17601E+01_JPRB ,&
&  0.16391E+01_JPRB, 0.15291E+01_JPRB, 0.14716E+01_JPRB, 0.14523E+01_JPRB, 0.14051E+01_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.13785E+01_JPRB, 0.13220E+01_JPRB, 0.12072E+01_JPRB, 0.10724E+01_JPRB, 0.94325E+00_JPRB, 0.85068E+00_JPRB, 0.82718E+00_JPRB ,&
&  0.83774E+00_JPRB, 0.84785E+00_JPRB, 0.84682E+00_JPRB, 0.83498E+00_JPRB, 0.81801E+00_JPRB, 0.80164E+00_JPRB, 0.78394E+00_JPRB ,&
&  0.75446E+00_JPRB, 0.70179E+00_JPRB, 0.54457E+00_JPRB, 0.48405E+00_JPRB, 0.62486E+00_JPRB, 0.64895E+00_JPRB, 0.71722E+00_JPRB ,&
&  0.10684E+01_JPRB, 0.16522E+01_JPRB, 0.23188E+01_JPRB, 0.30750E+01_JPRB, 0.38122E+01_JPRB, 0.41670E+01_JPRB, 0.41489E+01_JPRB ,&
&  0.42487E+01_JPRB, 0.38818E+01_JPRB, 0.33308E+01_JPRB, 0.28561E+01_JPRB, 0.25146E+01_JPRB, 0.22823E+01_JPRB, 0.20371E+01_JPRB ,&
&  0.17971E+01_JPRB, 0.16284E+01_JPRB, 0.15295E+01_JPRB, 0.14671E+01_JPRB, 0.14356E+01_JPRB, 0.13925E+01_JPRB, 0.13526E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.14059E+01_JPRB, 0.14429E+01_JPRB, 0.14644E+01_JPRB, 0.13436E+01_JPRB, 0.13684E+01_JPRB, 0.16425E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.18964E+01_JPRB, 0.19851E+01_JPRB, 0.21670E+01_JPRB, 0.24894E+01_JPRB, 0.25912E+01_JPRB, 0.34085E+01_JPRB ,&
&  0.31776E+01_JPRB, 0.29201E+01_JPRB, 0.27083E+01_JPRB, 0.23152E+01_JPRB, 0.26971E+01_JPRB, 0.40459E+01_JPRB, 0.40402E+01_JPRB ,&
&  0.37058E+01_JPRB, 0.35037E+01_JPRB, 0.32886E+01_JPRB, 0.28855E+01_JPRB, 0.23565E+01_JPRB, 0.16989E+01_JPRB, 0.20613E+01_JPRB ,&
&  0.21798E+01_JPRB, 0.21576E+01_JPRB, 0.20270E+01_JPRB, 0.17251E+01_JPRB, 0.14877E+01_JPRB, 0.17531E+01_JPRB, 0.16392E+01_JPRB ,&
&  0.14059E+01_JPRB, 0.13494E+01_JPRB, 0.17386E+01_JPRB, 0.22613E+01_JPRB, 0.25004E+01_JPRB, 0.24820E+01_JPRB, 0.27398E+01_JPRB ,&
&  0.26735E+01_JPRB, 0.26721E+01_JPRB, 0.25563E+01_JPRB, 0.23078E+01_JPRB, 0.20909E+01_JPRB, 0.19120E+01_JPRB, 0.17623E+01_JPRB ,&
&  0.16779E+01_JPRB, 0.16272E+01_JPRB, 0.15684E+01_JPRB, 0.15086E+01_JPRB, 0.14326E+01_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.13951E+01_JPRB, 0.13301E+01_JPRB, 0.12107E+01_JPRB, 0.10696E+01_JPRB, 0.93129E+00_JPRB, 0.82157E+00_JPRB, 0.79398E+00_JPRB ,&
&  0.80585E+00_JPRB, 0.81630E+00_JPRB, 0.81364E+00_JPRB, 0.79991E+00_JPRB, 0.78289E+00_JPRB, 0.75850E+00_JPRB, 0.73709E+00_JPRB ,&
&  0.72137E+00_JPRB, 0.64088E+00_JPRB, 0.50579E+00_JPRB, 0.49178E+00_JPRB, 0.55500E+00_JPRB, 0.60146E+00_JPRB, 0.73654E+00_JPRB ,&
&  0.96612E+00_JPRB, 0.15082E+01_JPRB, 0.20025E+01_JPRB, 0.25935E+01_JPRB, 0.31881E+01_JPRB, 0.36429E+01_JPRB, 0.36884E+01_JPRB ,&
&  0.34781E+01_JPRB, 0.34764E+01_JPRB, 0.32400E+01_JPRB, 0.29036E+01_JPRB, 0.25936E+01_JPRB, 0.23537E+01_JPRB, 0.21341E+01_JPRB ,&
&  0.19102E+01_JPRB, 0.16961E+01_JPRB, 0.15657E+01_JPRB, 0.14897E+01_JPRB, 0.14435E+01_JPRB, 0.14074E+01_JPRB, 0.13812E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.14062E+01_JPRB, 0.14248E+01_JPRB, 0.14581E+01_JPRB, 0.13315E+01_JPRB, 0.14637E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.21921E+01_JPRB, 0.22134E+01_JPRB, 0.23038E+01_JPRB, 0.26623E+01_JPRB, 0.25427E+01_JPRB, 0.29149E+01_JPRB, 0.40372E+01_JPRB ,&
&  0.44299E+01_JPRB, 0.44211E+01_JPRB, 0.44405E+01_JPRB, 0.35935E+01_JPRB, 0.31750E+01_JPRB, 0.45324E+01_JPRB, 0.42568E+01_JPRB ,&
&  0.38732E+01_JPRB, 0.36930E+01_JPRB, 0.33777E+01_JPRB, 0.33878E+01_JPRB, 0.28220E+01_JPRB, 0.24555E+01_JPRB, 0.18035E+01_JPRB ,&
&  0.18872E+01_JPRB, 0.17981E+01_JPRB, 0.20482E+01_JPRB, 0.19011E+01_JPRB, 0.17860E+01_JPRB, 0.16250E+01_JPRB, 0.13718E+01_JPRB ,&
&  0.12284E+01_JPRB, 0.11453E+01_JPRB, 0.12181E+01_JPRB, 0.15819E+01_JPRB, 0.19731E+01_JPRB, 0.19335E+01_JPRB, 0.22635E+01_JPRB ,&
&  0.24373E+01_JPRB, 0.25153E+01_JPRB, 0.22955E+01_JPRB, 0.22557E+01_JPRB, 0.21292E+01_JPRB, 0.19997E+01_JPRB, 0.18484E+01_JPRB ,&
&  0.17287E+01_JPRB, 0.16471E+01_JPRB, 0.15785E+01_JPRB, 0.15077E+01_JPRB, 0.14413E+01_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.13903E+01_JPRB, 0.13181E+01_JPRB, 0.11925E+01_JPRB, 0.10566E+01_JPRB, 0.90885E+00_JPRB, 0.78612E+00_JPRB, 0.76647E+00_JPRB ,&
&  0.79055E+00_JPRB, 0.78588E+00_JPRB, 0.76029E+00_JPRB, 0.71848E+00_JPRB, 0.70216E+00_JPRB, 0.70445E+00_JPRB, 0.70559E+00_JPRB ,&
&  0.69882E+00_JPRB, 0.78365E+00_JPRB, 0.63716E+00_JPRB, 0.52901E+00_JPRB, 0.51680E+00_JPRB, 0.59801E+00_JPRB, 0.77921E+00_JPRB ,&
&  0.97629E+00_JPRB, 0.13261E+01_JPRB, 0.16701E+01_JPRB, 0.19645E+01_JPRB, 0.23512E+01_JPRB, 0.26595E+01_JPRB, 0.28130E+01_JPRB ,&
&  0.30033E+01_JPRB, 0.25483E+01_JPRB, 0.25467E+01_JPRB, 0.23571E+01_JPRB, 0.21661E+01_JPRB, 0.20086E+01_JPRB, 0.18826E+01_JPRB ,&
&  0.17245E+01_JPRB, 0.16030E+01_JPRB, 0.14603E+01_JPRB, 0.13639E+01_JPRB, 0.13140E+01_JPRB, 0.12699E+01_JPRB, 0.12356E+01_JPRB ,&
&  0.12622E+01_JPRB, 0.13186E+01_JPRB, 0.13793E+01_JPRB, 0.14068E+01_JPRB, 0.15145E+01_JPRB, 0.17205E+01_JPRB, 0.19557E+01_JPRB ,&
&  0.20159E+01_JPRB, 0.21798E+01_JPRB, 0.30379E+01_JPRB, 0.32451E+01_JPRB, 0.36632E+01_JPRB, 0.40494E+01_JPRB, 0.44130E+01_JPRB ,&
&  0.48231E+01_JPRB, 0.47597E+01_JPRB, 0.46960E+01_JPRB, 0.42219E+01_JPRB, 0.47634E+01_JPRB, 0.46319E+01_JPRB, 0.43316E+01_JPRB ,&
&  0.39674E+01_JPRB, 0.38942E+01_JPRB, 0.35907E+01_JPRB, 0.34052E+01_JPRB, 0.33441E+01_JPRB, 0.34767E+01_JPRB, 0.30748E+01_JPRB ,&
&  0.24676E+01_JPRB, 0.26878E+01_JPRB, 0.23430E+01_JPRB, 0.19224E+01_JPRB, 0.14774E+01_JPRB, 0.13170E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.11269E+01_JPRB, 0.10726E+01_JPRB, 0.10777E+01_JPRB, 0.12280E+01_JPRB, 0.14578E+01_JPRB, 0.16057E+01_JPRB, 0.16957E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.19924E+01_JPRB, 0.20043E+01_JPRB, 0.19291E+01_JPRB, 0.19095E+01_JPRB, 0.18975E+01_JPRB, 0.18388E+01_JPRB ,&
&  0.17274E+01_JPRB, 0.16411E+01_JPRB, 0.15666E+01_JPRB, 0.15023E+01_JPRB, 0.14390E+01_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.13321E+01_JPRB, 0.12701E+01_JPRB, 0.11641E+01_JPRB, 0.10379E+01_JPRB, 0.88954E+00_JPRB, 0.77655E+00_JPRB, 0.75085E+00_JPRB ,&
&  0.73958E+00_JPRB, 0.72210E+00_JPRB, 0.69367E+00_JPRB, 0.67710E+00_JPRB, 0.67449E+00_JPRB, 0.68001E+00_JPRB, 0.70874E+00_JPRB ,&
&  0.75611E+00_JPRB, 0.11074E+01_JPRB, 0.88797E+00_JPRB, 0.74068E+00_JPRB, 0.66208E+00_JPRB, 0.78738E+00_JPRB, 0.90036E+00_JPRB ,&
&  0.10529E+01_JPRB, 0.12296E+01_JPRB, 0.13082E+01_JPRB, 0.15490E+01_JPRB, 0.17706E+01_JPRB, 0.18485E+01_JPRB, 0.19925E+01_JPRB ,&
&  0.20007E+01_JPRB, 0.20265E+01_JPRB, 0.18257E+01_JPRB, 0.18066E+01_JPRB, 0.17105E+01_JPRB, 0.15376E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.14518E+01_JPRB, 0.13551E+01_JPRB, 0.12579E+01_JPRB, 0.12119E+01_JPRB, 0.12147E+01_JPRB, 0.12114E+01_JPRB, 0.11686E+01_JPRB ,&
&  0.11775E+01_JPRB, 0.12188E+01_JPRB, 0.12946E+01_JPRB, 0.13717E+01_JPRB, 0.14949E+01_JPRB, 0.17327E+01_JPRB, 0.19439E+01_JPRB ,&
&  0.24208E+01_JPRB, 0.24496E+01_JPRB, 0.22899E+01_JPRB, 0.30123E+01_JPRB, 0.44416E+01_JPRB, 0.46925E+01_JPRB, 0.43809E+01_JPRB ,&
&  0.48663E+01_JPRB, 0.50572E+01_JPRB, 0.50334E+01_JPRB, 0.48230E+01_JPRB, 0.46382E+01_JPRB, 0.44989E+01_JPRB, 0.43882E+01_JPRB ,&
&  0.40297E+01_JPRB, 0.38383E+01_JPRB, 0.36815E+01_JPRB, 0.35911E+01_JPRB, 0.33591E+01_JPRB, 0.32612E+01_JPRB, 0.32408E+01_JPRB ,&
&  0.29182E+01_JPRB, 0.23369E+01_JPRB, 0.15362E+01_JPRB, 0.15233E+01_JPRB, 0.11054E+01_JPRB, 0.95426E+00_JPRB, 0.10667E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.10833E+01_JPRB, 0.11502E+01_JPRB, 0.11016E+01_JPRB, 0.11641E+01_JPRB, 0.11961E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.13574E+01_JPRB, 0.12777E+01_JPRB, 0.15266E+01_JPRB, 0.15630E+01_JPRB, 0.15727E+01_JPRB, 0.15775E+01_JPRB, 0.15728E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.14985E+01_JPRB, 0.14549E+01_JPRB, 0.14078E+01_JPRB, 0.13740E+01_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.11777E+01_JPRB, 0.11219E+01_JPRB, 0.10447E+01_JPRB, 0.95355E+00_JPRB, 0.84364E+00_JPRB, 0.75151E+00_JPRB, 0.70734E+00_JPRB ,&
&  0.69464E+00_JPRB, 0.67245E+00_JPRB, 0.64878E+00_JPRB, 0.64041E+00_JPRB, 0.64389E+00_JPRB, 0.65904E+00_JPRB, 0.72112E+00_JPRB ,&
&  0.87470E+00_JPRB, 0.11653E+01_JPRB, 0.10100E+01_JPRB, 0.85584E+00_JPRB, 0.10175E+01_JPRB, 0.98251E+00_JPRB, 0.10421E+01_JPRB ,&
&  0.10580E+01_JPRB, 0.12324E+01_JPRB, 0.11827E+01_JPRB, 0.11954E+01_JPRB, 0.13525E+01_JPRB, 0.14365E+01_JPRB, 0.14740E+01_JPRB ,&
&  0.14460E+01_JPRB, 0.13690E+01_JPRB, 0.12868E+01_JPRB, 0.12883E+01_JPRB, 0.13544E+01_JPRB, 0.12836E+01_JPRB, 0.12876E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.11363E+01_JPRB, 0.10494E+01_JPRB, 0.10243E+01_JPRB, 0.10505E+01_JPRB, 0.11044E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.12512E+01_JPRB, 0.12885E+01_JPRB, 0.13898E+01_JPRB, 0.15426E+01_JPRB, 0.18692E+01_JPRB, 0.23249E+01_JPRB ,&
&  0.29363E+01_JPRB, 0.32234E+01_JPRB, 0.35280E+01_JPRB, 0.43005E+01_JPRB, 0.46841E+01_JPRB, 0.46754E+01_JPRB, 0.46653E+01_JPRB ,&
&  0.45755E+01_JPRB, 0.45563E+01_JPRB, 0.46390E+01_JPRB, 0.44352E+01_JPRB, 0.42741E+01_JPRB, 0.42510E+01_JPRB, 0.41852E+01_JPRB ,&
&  0.37927E+01_JPRB, 0.35923E+01_JPRB, 0.35823E+01_JPRB, 0.33891E+01_JPRB, 0.32181E+01_JPRB, 0.30041E+01_JPRB, 0.29925E+01_JPRB ,&
&  0.29986E+01_JPRB, 0.17154E+01_JPRB, 0.12660E+01_JPRB, 0.13944E+01_JPRB, 0.10747E+01_JPRB, 0.97867E+00_JPRB, 0.11622E+01_JPRB ,&
&  0.11475E+01_JPRB, 0.11261E+01_JPRB, 0.11460E+01_JPRB, 0.10594E+01_JPRB, 0.96952E+00_JPRB, 0.98747E+00_JPRB, 0.95549E+00_JPRB ,&
&  0.98951E+00_JPRB, 0.99216E+00_JPRB, 0.11673E+01_JPRB, 0.12781E+01_JPRB, 0.13255E+01_JPRB, 0.13220E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.12918E+01_JPRB, 0.12715E+01_JPRB, 0.12553E+01_JPRB, 0.12467E+01_JPRB, 0.12293E+01_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.10229E+01_JPRB, 0.99016E+00_JPRB, 0.94887E+00_JPRB, 0.89698E+00_JPRB, 0.82427E+00_JPRB, 0.75709E+00_JPRB, 0.69396E+00_JPRB ,&
&  0.68259E+00_JPRB, 0.66029E+00_JPRB, 0.63626E+00_JPRB, 0.62800E+00_JPRB, 0.63225E+00_JPRB, 0.64155E+00_JPRB, 0.67009E+00_JPRB ,&
&  0.61289E+00_JPRB, 0.78205E+00_JPRB, 0.73807E+00_JPRB, 0.88731E+00_JPRB, 0.12387E+01_JPRB, 0.10027E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.10316E+01_JPRB, 0.11049E+01_JPRB, 0.10952E+01_JPRB, 0.10817E+01_JPRB, 0.11290E+01_JPRB, 0.11572E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.10569E+01_JPRB, 0.94629E+00_JPRB, 0.91134E+00_JPRB, 0.90852E+00_JPRB, 0.99160E+00_JPRB, 0.10564E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.97004E+00_JPRB, 0.92581E+00_JPRB, 0.93668E+00_JPRB, 0.97030E+00_JPRB, 0.99732E+00_JPRB, 0.10520E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.12306E+01_JPRB, 0.13373E+01_JPRB, 0.14827E+01_JPRB, 0.15171E+01_JPRB, 0.21416E+01_JPRB, 0.32248E+01_JPRB ,&
&  0.39012E+01_JPRB, 0.38261E+01_JPRB, 0.41805E+01_JPRB, 0.50375E+01_JPRB, 0.51589E+01_JPRB, 0.48450E+01_JPRB, 0.44803E+01_JPRB ,&
&  0.41619E+01_JPRB, 0.39647E+01_JPRB, 0.41166E+01_JPRB, 0.41171E+01_JPRB, 0.38619E+01_JPRB, 0.39260E+01_JPRB, 0.38521E+01_JPRB ,&
&  0.36189E+01_JPRB, 0.34895E+01_JPRB, 0.34981E+01_JPRB, 0.34709E+01_JPRB, 0.32634E+01_JPRB, 0.32915E+01_JPRB, 0.31511E+01_JPRB ,&
&  0.30128E+01_JPRB, 0.25443E+01_JPRB, 0.18045E+01_JPRB, 0.14473E+01_JPRB, 0.10629E+01_JPRB, 0.10596E+01_JPRB, 0.13187E+01_JPRB ,&
&  0.12729E+01_JPRB, 0.11614E+01_JPRB, 0.10095E+01_JPRB, 0.88932E+00_JPRB, 0.80595E+00_JPRB, 0.79217E+00_JPRB, 0.74420E+00_JPRB ,&
&  0.72328E+00_JPRB, 0.79459E+00_JPRB, 0.89719E+00_JPRB, 0.10010E+01_JPRB, 0.10806E+01_JPRB, 0.11143E+01_JPRB, 0.10485E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.11078E+01_JPRB, 0.11085E+01_JPRB, 0.11087E+01_JPRB, 0.10791E+01_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.86962E+00_JPRB, 0.87657E+00_JPRB, 0.85287E+00_JPRB, 0.83485E+00_JPRB, 0.78169E+00_JPRB, 0.74297E+00_JPRB, 0.68636E+00_JPRB ,&
&  0.68732E+00_JPRB, 0.66688E+00_JPRB, 0.63110E+00_JPRB, 0.61796E+00_JPRB, 0.62744E+00_JPRB, 0.62697E+00_JPRB, 0.57293E+00_JPRB ,&
&  0.54354E+00_JPRB, 0.61287E+00_JPRB, 0.61780E+00_JPRB, 0.72285E+00_JPRB, 0.81299E+00_JPRB, 0.76627E+00_JPRB, 0.79784E+00_JPRB ,&
&  0.90507E+00_JPRB, 0.93349E+00_JPRB, 0.95364E+00_JPRB, 0.96118E+00_JPRB, 0.96889E+00_JPRB, 0.97857E+00_JPRB, 0.96990E+00_JPRB ,&
&  0.89552E+00_JPRB, 0.84359E+00_JPRB, 0.80493E+00_JPRB, 0.80131E+00_JPRB, 0.88487E+00_JPRB, 0.92688E+00_JPRB, 0.88762E+00_JPRB ,&
&  0.83970E+00_JPRB, 0.81863E+00_JPRB, 0.82292E+00_JPRB, 0.88048E+00_JPRB, 0.91568E+00_JPRB, 0.92583E+00_JPRB, 0.97968E+00_JPRB ,&
&  0.10739E+01_JPRB, 0.12040E+01_JPRB, 0.13416E+01_JPRB, 0.15005E+01_JPRB, 0.15903E+01_JPRB, 0.21896E+01_JPRB, 0.30140E+01_JPRB ,&
&  0.32385E+01_JPRB, 0.34919E+01_JPRB, 0.38137E+01_JPRB, 0.41435E+01_JPRB, 0.43118E+01_JPRB, 0.39567E+01_JPRB, 0.36344E+01_JPRB ,&
&  0.35628E+01_JPRB, 0.34496E+01_JPRB, 0.41169E+01_JPRB, 0.39338E+01_JPRB, 0.37075E+01_JPRB, 0.35977E+01_JPRB, 0.34352E+01_JPRB ,&
&  0.33786E+01_JPRB, 0.33227E+01_JPRB, 0.32764E+01_JPRB, 0.31923E+01_JPRB, 0.30839E+01_JPRB, 0.30337E+01_JPRB, 0.28360E+01_JPRB ,&
&  0.27019E+01_JPRB, 0.24673E+01_JPRB, 0.19969E+01_JPRB, 0.16154E+01_JPRB, 0.12826E+01_JPRB, 0.12898E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.99606E+00_JPRB, 0.81594E+00_JPRB, 0.77914E+00_JPRB, 0.73358E+00_JPRB, 0.71822E+00_JPRB, 0.68768E+00_JPRB, 0.65130E+00_JPRB ,&
&  0.68029E+00_JPRB, 0.75022E+00_JPRB, 0.75968E+00_JPRB, 0.80338E+00_JPRB, 0.86997E+00_JPRB, 0.94203E+00_JPRB, 0.82208E+00_JPRB ,&
&  0.82875E+00_JPRB, 0.91164E+00_JPRB, 0.92482E+00_JPRB, 0.91804E+00_JPRB, 0.89023E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.79784E+00_JPRB, 0.81139E+00_JPRB, 0.79062E+00_JPRB, 0.79150E+00_JPRB, 0.75371E+00_JPRB, 0.73116E+00_JPRB, 0.65494E+00_JPRB ,&
&  0.67971E+00_JPRB, 0.65844E+00_JPRB, 0.61526E+00_JPRB, 0.60402E+00_JPRB, 0.61991E+00_JPRB, 0.54955E+00_JPRB, 0.48468E+00_JPRB ,&
&  0.46669E+00_JPRB, 0.49612E+00_JPRB, 0.57471E+00_JPRB, 0.61596E+00_JPRB, 0.62853E+00_JPRB, 0.63818E+00_JPRB, 0.63689E+00_JPRB ,&
&  0.71465E+00_JPRB, 0.74031E+00_JPRB, 0.80826E+00_JPRB, 0.84656E+00_JPRB, 0.83544E+00_JPRB, 0.84438E+00_JPRB, 0.86832E+00_JPRB ,&
&  0.81341E+00_JPRB, 0.79239E+00_JPRB, 0.79309E+00_JPRB, 0.72751E+00_JPRB, 0.80237E+00_JPRB, 0.76694E+00_JPRB, 0.71690E+00_JPRB ,&
&  0.70717E+00_JPRB, 0.73678E+00_JPRB, 0.77099E+00_JPRB, 0.80599E+00_JPRB, 0.82337E+00_JPRB, 0.85004E+00_JPRB, 0.90027E+00_JPRB ,&
&  0.98398E+00_JPRB, 0.11374E+01_JPRB, 0.13151E+01_JPRB, 0.15277E+01_JPRB, 0.17683E+01_JPRB, 0.21446E+01_JPRB, 0.27283E+01_JPRB ,&
&  0.29346E+01_JPRB, 0.32007E+01_JPRB, 0.35155E+01_JPRB, 0.32399E+01_JPRB, 0.35542E+01_JPRB, 0.35302E+01_JPRB, 0.32279E+01_JPRB ,&
&  0.32040E+01_JPRB, 0.28951E+01_JPRB, 0.27995E+01_JPRB, 0.28227E+01_JPRB, 0.29778E+01_JPRB, 0.29042E+01_JPRB, 0.28585E+01_JPRB ,&
&  0.29105E+01_JPRB, 0.31198E+01_JPRB, 0.29164E+01_JPRB, 0.28255E+01_JPRB, 0.26061E+01_JPRB, 0.24086E+01_JPRB, 0.21776E+01_JPRB ,&
&  0.20148E+01_JPRB, 0.18314E+01_JPRB, 0.16582E+01_JPRB, 0.15198E+01_JPRB, 0.13410E+01_JPRB, 0.11970E+01_JPRB, 0.10771E+01_JPRB ,&
&  0.95609E+00_JPRB, 0.81918E+00_JPRB, 0.76590E+00_JPRB, 0.72883E+00_JPRB, 0.70723E+00_JPRB, 0.66028E+00_JPRB, 0.65912E+00_JPRB ,&
&  0.69928E+00_JPRB, 0.73527E+00_JPRB, 0.78045E+00_JPRB, 0.77276E+00_JPRB, 0.79315E+00_JPRB, 0.83490E+00_JPRB, 0.78423E+00_JPRB ,&
&  0.73190E+00_JPRB, 0.80331E+00_JPRB, 0.80939E+00_JPRB, 0.80638E+00_JPRB, 0.79944E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.77613E+00_JPRB, 0.77720E+00_JPRB, 0.76364E+00_JPRB, 0.76017E+00_JPRB, 0.74166E+00_JPRB, 0.67965E+00_JPRB, 0.61843E+00_JPRB ,&
&  0.67624E+00_JPRB, 0.61023E+00_JPRB, 0.62703E+00_JPRB, 0.52784E+00_JPRB, 0.49650E+00_JPRB, 0.49091E+00_JPRB, 0.47907E+00_JPRB ,&
&  0.46900E+00_JPRB, 0.50327E+00_JPRB, 0.54416E+00_JPRB, 0.56277E+00_JPRB, 0.58759E+00_JPRB, 0.59750E+00_JPRB, 0.59192E+00_JPRB ,&
&  0.60832E+00_JPRB, 0.64017E+00_JPRB, 0.70227E+00_JPRB, 0.71698E+00_JPRB, 0.69336E+00_JPRB, 0.69634E+00_JPRB, 0.70903E+00_JPRB ,&
&  0.68417E+00_JPRB, 0.66995E+00_JPRB, 0.69485E+00_JPRB, 0.69361E+00_JPRB, 0.67971E+00_JPRB, 0.63478E+00_JPRB, 0.60995E+00_JPRB ,&
&  0.62257E+00_JPRB, 0.59774E+00_JPRB, 0.66267E+00_JPRB, 0.69104E+00_JPRB, 0.72418E+00_JPRB, 0.76256E+00_JPRB, 0.80415E+00_JPRB ,&
&  0.87729E+00_JPRB, 0.10050E+01_JPRB, 0.11682E+01_JPRB, 0.13406E+01_JPRB, 0.15710E+01_JPRB, 0.20304E+01_JPRB, 0.23743E+01_JPRB ,&
&  0.27269E+01_JPRB, 0.20554E+01_JPRB, 0.25525E+01_JPRB, 0.27107E+01_JPRB, 0.30472E+01_JPRB, 0.30350E+01_JPRB, 0.29242E+01_JPRB ,&
&  0.28319E+01_JPRB, 0.24450E+01_JPRB, 0.22126E+01_JPRB, 0.20892E+01_JPRB, 0.21251E+01_JPRB, 0.23013E+01_JPRB, 0.24389E+01_JPRB ,&
&  0.25683E+01_JPRB, 0.24268E+01_JPRB, 0.22382E+01_JPRB, 0.20370E+01_JPRB, 0.18727E+01_JPRB, 0.17569E+01_JPRB, 0.16353E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.14061E+01_JPRB, 0.12914E+01_JPRB, 0.11421E+01_JPRB, 0.10668E+01_JPRB, 0.10026E+01_JPRB, 0.93236E+00_JPRB ,&
&  0.89681E+00_JPRB, 0.83899E+00_JPRB, 0.77444E+00_JPRB, 0.74619E+00_JPRB, 0.71029E+00_JPRB, 0.70110E+00_JPRB, 0.72495E+00_JPRB ,&
&  0.73012E+00_JPRB, 0.69211E+00_JPRB, 0.72388E+00_JPRB, 0.73116E+00_JPRB, 0.72656E+00_JPRB, 0.73114E+00_JPRB, 0.78363E+00_JPRB ,&
&  0.74110E+00_JPRB, 0.75034E+00_JPRB, 0.79580E+00_JPRB, 0.78399E+00_JPRB, 0.77850E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.77329E+00_JPRB, 0.76546E+00_JPRB, 0.75151E+00_JPRB, 0.73230E+00_JPRB, 0.71320E+00_JPRB, 0.67516E+00_JPRB, 0.59304E+00_JPRB ,&
&  0.58757E+00_JPRB, 0.54456E+00_JPRB, 0.50823E+00_JPRB, 0.48924E+00_JPRB, 0.48818E+00_JPRB, 0.48176E+00_JPRB, 0.47893E+00_JPRB ,&
&  0.48361E+00_JPRB, 0.50593E+00_JPRB, 0.53701E+00_JPRB, 0.54994E+00_JPRB, 0.56346E+00_JPRB, 0.57108E+00_JPRB, 0.56595E+00_JPRB ,&
&  0.57589E+00_JPRB, 0.59783E+00_JPRB, 0.62452E+00_JPRB, 0.64705E+00_JPRB, 0.63551E+00_JPRB, 0.62487E+00_JPRB, 0.61627E+00_JPRB ,&
&  0.60933E+00_JPRB, 0.60422E+00_JPRB, 0.59983E+00_JPRB, 0.61271E+00_JPRB, 0.59454E+00_JPRB, 0.57533E+00_JPRB, 0.57726E+00_JPRB ,&
&  0.47623E+00_JPRB, 0.48329E+00_JPRB, 0.57707E+00_JPRB, 0.60948E+00_JPRB, 0.64109E+00_JPRB, 0.67658E+00_JPRB, 0.73427E+00_JPRB ,&
&  0.80004E+00_JPRB, 0.88753E+00_JPRB, 0.99524E+00_JPRB, 0.11172E+01_JPRB, 0.12965E+01_JPRB, 0.16068E+01_JPRB, 0.19089E+01_JPRB ,&
&  0.21979E+01_JPRB, 0.18063E+01_JPRB, 0.17766E+01_JPRB, 0.20266E+01_JPRB, 0.25227E+01_JPRB, 0.22938E+01_JPRB, 0.21168E+01_JPRB ,&
&  0.19520E+01_JPRB, 0.18178E+01_JPRB, 0.17732E+01_JPRB, 0.19105E+01_JPRB, 0.19682E+01_JPRB, 0.20964E+01_JPRB, 0.21730E+01_JPRB ,&
&  0.19552E+01_JPRB, 0.17950E+01_JPRB, 0.16216E+01_JPRB, 0.15287E+01_JPRB, 0.14662E+01_JPRB, 0.14216E+01_JPRB, 0.13795E+01_JPRB ,&
&  0.13454E+01_JPRB, 0.12860E+01_JPRB, 0.11674E+01_JPRB, 0.10358E+01_JPRB, 0.94139E+00_JPRB, 0.88753E+00_JPRB, 0.85460E+00_JPRB ,&
&  0.82464E+00_JPRB, 0.79633E+00_JPRB, 0.77362E+00_JPRB, 0.75973E+00_JPRB, 0.75414E+00_JPRB, 0.76034E+00_JPRB, 0.79820E+00_JPRB ,&
&  0.74577E+00_JPRB, 0.70656E+00_JPRB, 0.65797E+00_JPRB, 0.65932E+00_JPRB, 0.68545E+00_JPRB, 0.71332E+00_JPRB, 0.72408E+00_JPRB ,&
&  0.75675E+00_JPRB, 0.75420E+00_JPRB, 0.72818E+00_JPRB, 0.74329E+00_JPRB, 0.75508E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.76679E+00_JPRB, 0.75612E+00_JPRB, 0.73274E+00_JPRB, 0.71978E+00_JPRB, 0.68537E+00_JPRB, 0.66382E+00_JPRB, 0.62712E+00_JPRB ,&
&  0.61505E+00_JPRB, 0.58910E+00_JPRB, 0.55441E+00_JPRB, 0.51651E+00_JPRB, 0.49784E+00_JPRB, 0.49138E+00_JPRB, 0.49599E+00_JPRB ,&
&  0.50903E+00_JPRB, 0.52238E+00_JPRB, 0.53522E+00_JPRB, 0.54233E+00_JPRB, 0.55530E+00_JPRB, 0.56360E+00_JPRB, 0.56378E+00_JPRB ,&
&  0.56906E+00_JPRB, 0.57984E+00_JPRB, 0.58446E+00_JPRB, 0.58054E+00_JPRB, 0.57964E+00_JPRB, 0.58251E+00_JPRB, 0.57849E+00_JPRB ,&
&  0.56208E+00_JPRB, 0.54636E+00_JPRB, 0.55020E+00_JPRB, 0.57909E+00_JPRB, 0.56039E+00_JPRB, 0.56220E+00_JPRB, 0.50333E+00_JPRB ,&
&  0.43903E+00_JPRB, 0.42272E+00_JPRB, 0.45401E+00_JPRB, 0.55714E+00_JPRB, 0.58780E+00_JPRB, 0.62403E+00_JPRB, 0.65630E+00_JPRB ,&
&  0.65901E+00_JPRB, 0.70049E+00_JPRB, 0.80220E+00_JPRB, 0.10046E+01_JPRB, 0.11249E+01_JPRB, 0.13375E+01_JPRB, 0.15483E+01_JPRB ,&
&  0.17125E+01_JPRB, 0.18121E+01_JPRB, 0.18927E+01_JPRB, 0.16989E+01_JPRB, 0.16897E+01_JPRB, 0.19313E+01_JPRB, 0.17557E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.16282E+01_JPRB, 0.16680E+01_JPRB, 0.17766E+01_JPRB, 0.16868E+01_JPRB, 0.17336E+01_JPRB, 0.16991E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.14396E+01_JPRB, 0.13547E+01_JPRB, 0.13135E+01_JPRB, 0.12772E+01_JPRB, 0.12622E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.13329E+01_JPRB, 0.11830E+01_JPRB, 0.95797E+00_JPRB, 0.87013E+00_JPRB, 0.83457E+00_JPRB, 0.81323E+00_JPRB ,&
&  0.78407E+00_JPRB, 0.76439E+00_JPRB, 0.76178E+00_JPRB, 0.76159E+00_JPRB, 0.74307E+00_JPRB, 0.71051E+00_JPRB, 0.67833E+00_JPRB ,&
&  0.65253E+00_JPRB, 0.63775E+00_JPRB, 0.61742E+00_JPRB, 0.64058E+00_JPRB, 0.67159E+00_JPRB, 0.69714E+00_JPRB, 0.68720E+00_JPRB ,&
&  0.67669E+00_JPRB, 0.68409E+00_JPRB, 0.69580E+00_JPRB, 0.73137E+00_JPRB, 0.75108E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.66931E+00_JPRB, 0.69231E+00_JPRB, 0.71762E+00_JPRB, 0.70865E+00_JPRB, 0.69882E+00_JPRB, 0.65541E+00_JPRB, 0.61121E+00_JPRB ,&
&  0.58394E+00_JPRB, 0.57201E+00_JPRB, 0.56796E+00_JPRB, 0.56049E+00_JPRB, 0.55436E+00_JPRB, 0.55222E+00_JPRB, 0.55136E+00_JPRB ,&
&  0.55203E+00_JPRB, 0.55375E+00_JPRB, 0.55557E+00_JPRB, 0.55992E+00_JPRB, 0.57220E+00_JPRB, 0.58155E+00_JPRB, 0.58395E+00_JPRB ,&
&  0.58534E+00_JPRB, 0.58765E+00_JPRB, 0.57701E+00_JPRB, 0.55823E+00_JPRB, 0.54931E+00_JPRB, 0.55121E+00_JPRB, 0.55877E+00_JPRB ,&
&  0.55114E+00_JPRB, 0.53600E+00_JPRB, 0.54055E+00_JPRB, 0.55733E+00_JPRB, 0.55045E+00_JPRB, 0.55376E+00_JPRB, 0.50386E+00_JPRB ,&
&  0.43795E+00_JPRB, 0.39312E+00_JPRB, 0.37602E+00_JPRB, 0.40362E+00_JPRB, 0.46603E+00_JPRB, 0.56565E+00_JPRB, 0.60958E+00_JPRB ,&
&  0.64816E+00_JPRB, 0.70531E+00_JPRB, 0.77235E+00_JPRB, 0.85881E+00_JPRB, 0.97222E+00_JPRB, 0.11415E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.14371E+01_JPRB, 0.15002E+01_JPRB, 0.15232E+01_JPRB, 0.13016E+01_JPRB, 0.12497E+01_JPRB, 0.12756E+01_JPRB, 0.14479E+01_JPRB ,&
&  0.16066E+01_JPRB, 0.14778E+01_JPRB, 0.13716E+01_JPRB, 0.13731E+01_JPRB, 0.13408E+01_JPRB, 0.13498E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.13286E+01_JPRB, 0.12713E+01_JPRB, 0.12028E+01_JPRB, 0.11502E+01_JPRB, 0.11374E+01_JPRB, 0.11569E+01_JPRB, 0.11749E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.13769E+01_JPRB, 0.12853E+01_JPRB, 0.95795E+00_JPRB, 0.90151E+00_JPRB, 0.88493E+00_JPRB, 0.85923E+00_JPRB ,&
&  0.80601E+00_JPRB, 0.77053E+00_JPRB, 0.75802E+00_JPRB, 0.75373E+00_JPRB, 0.72819E+00_JPRB, 0.70976E+00_JPRB, 0.69756E+00_JPRB ,&
&  0.69302E+00_JPRB, 0.69495E+00_JPRB, 0.69669E+00_JPRB, 0.72045E+00_JPRB, 0.72680E+00_JPRB, 0.72431E+00_JPRB, 0.70452E+00_JPRB ,&
&  0.68750E+00_JPRB, 0.67892E+00_JPRB, 0.67176E+00_JPRB, 0.66366E+00_JPRB, 0.66426E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.69086E+00_JPRB, 0.68797E+00_JPRB, 0.69028E+00_JPRB, 0.69274E+00_JPRB, 0.69777E+00_JPRB, 0.68303E+00_JPRB, 0.66808E+00_JPRB ,&
&  0.66294E+00_JPRB, 0.66258E+00_JPRB, 0.65002E+00_JPRB, 0.63834E+00_JPRB, 0.62139E+00_JPRB, 0.60797E+00_JPRB, 0.60464E+00_JPRB ,&
&  0.60579E+00_JPRB, 0.60444E+00_JPRB, 0.60240E+00_JPRB, 0.59830E+00_JPRB, 0.60385E+00_JPRB, 0.61033E+00_JPRB, 0.60600E+00_JPRB ,&
&  0.60184E+00_JPRB, 0.59711E+00_JPRB, 0.58559E+00_JPRB, 0.57074E+00_JPRB, 0.54986E+00_JPRB, 0.54013E+00_JPRB, 0.53288E+00_JPRB ,&
&  0.52635E+00_JPRB, 0.52696E+00_JPRB, 0.54217E+00_JPRB, 0.53811E+00_JPRB, 0.53976E+00_JPRB, 0.54133E+00_JPRB, 0.48314E+00_JPRB ,&
&  0.42193E+00_JPRB, 0.38073E+00_JPRB, 0.35822E+00_JPRB, 0.35712E+00_JPRB, 0.37839E+00_JPRB, 0.42719E+00_JPRB, 0.49236E+00_JPRB ,&
&  0.57217E+00_JPRB, 0.62591E+00_JPRB, 0.67474E+00_JPRB, 0.72142E+00_JPRB, 0.77277E+00_JPRB, 0.86505E+00_JPRB, 0.95454E+00_JPRB ,&
&  0.10578E+01_JPRB, 0.11744E+01_JPRB, 0.12729E+01_JPRB, 0.13511E+01_JPRB, 0.14048E+01_JPRB, 0.13255E+01_JPRB, 0.14180E+01_JPRB ,&
&  0.17828E+01_JPRB, 0.14736E+01_JPRB, 0.11849E+01_JPRB, 0.10638E+01_JPRB, 0.10317E+01_JPRB, 0.10300E+01_JPRB, 0.10269E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10337E+01_JPRB, 0.10248E+01_JPRB, 0.10147E+01_JPRB, 0.10150E+01_JPRB, 0.10154E+01_JPRB, 0.10374E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.13182E+01_JPRB, 0.16495E+01_JPRB, 0.12176E+01_JPRB, 0.10687E+01_JPRB, 0.96496E+00_JPRB, 0.84792E+00_JPRB ,&
&  0.79675E+00_JPRB, 0.77631E+00_JPRB, 0.77407E+00_JPRB, 0.77372E+00_JPRB, 0.76379E+00_JPRB, 0.76128E+00_JPRB, 0.76238E+00_JPRB ,&
&  0.77166E+00_JPRB, 0.77795E+00_JPRB, 0.77457E+00_JPRB, 0.76808E+00_JPRB, 0.75282E+00_JPRB, 0.73124E+00_JPRB, 0.71100E+00_JPRB ,&
&  0.70787E+00_JPRB, 0.70773E+00_JPRB, 0.70739E+00_JPRB, 0.70548E+00_JPRB, 0.70018E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.68742E+00_JPRB, 0.67972E+00_JPRB, 0.67968E+00_JPRB, 0.67976E+00_JPRB, 0.68427E+00_JPRB, 0.68920E+00_JPRB, 0.69013E+00_JPRB ,&
&  0.69852E+00_JPRB, 0.69660E+00_JPRB, 0.68337E+00_JPRB, 0.66831E+00_JPRB, 0.65116E+00_JPRB, 0.63924E+00_JPRB, 0.63401E+00_JPRB ,&
&  0.63465E+00_JPRB, 0.62776E+00_JPRB, 0.62462E+00_JPRB, 0.62498E+00_JPRB, 0.63024E+00_JPRB, 0.63867E+00_JPRB, 0.63365E+00_JPRB ,&
&  0.61385E+00_JPRB, 0.59960E+00_JPRB, 0.58219E+00_JPRB, 0.55869E+00_JPRB, 0.53517E+00_JPRB, 0.51789E+00_JPRB, 0.51992E+00_JPRB ,&
&  0.52944E+00_JPRB, 0.52298E+00_JPRB, 0.52121E+00_JPRB, 0.52146E+00_JPRB, 0.52124E+00_JPRB, 0.51310E+00_JPRB, 0.42712E+00_JPRB ,&
&  0.39194E+00_JPRB, 0.36713E+00_JPRB, 0.35718E+00_JPRB, 0.35568E+00_JPRB, 0.37220E+00_JPRB, 0.40507E+00_JPRB, 0.46471E+00_JPRB ,&
&  0.52254E+00_JPRB, 0.57916E+00_JPRB, 0.61493E+00_JPRB, 0.65077E+00_JPRB, 0.70019E+00_JPRB, 0.76155E+00_JPRB, 0.81518E+00_JPRB ,&
&  0.87977E+00_JPRB, 0.96234E+00_JPRB, 0.10403E+01_JPRB, 0.11089E+01_JPRB, 0.11461E+01_JPRB, 0.11580E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.10461E+01_JPRB, 0.99838E+00_JPRB, 0.96541E+00_JPRB, 0.94277E+00_JPRB, 0.93739E+00_JPRB, 0.90728E+00_JPRB ,&
&  0.89367E+00_JPRB, 0.90134E+00_JPRB, 0.90267E+00_JPRB, 0.90300E+00_JPRB, 0.90575E+00_JPRB, 0.90520E+00_JPRB, 0.90797E+00_JPRB ,&
&  0.91852E+00_JPRB, 0.93889E+00_JPRB, 0.98776E+00_JPRB, 0.95040E+00_JPRB, 0.90047E+00_JPRB, 0.84771E+00_JPRB, 0.81250E+00_JPRB ,&
&  0.80005E+00_JPRB, 0.79743E+00_JPRB, 0.79926E+00_JPRB, 0.79320E+00_JPRB, 0.78969E+00_JPRB, 0.78859E+00_JPRB, 0.79915E+00_JPRB ,&
&  0.79855E+00_JPRB, 0.79641E+00_JPRB, 0.78599E+00_JPRB, 0.76998E+00_JPRB, 0.75075E+00_JPRB, 0.73099E+00_JPRB, 0.71561E+00_JPRB ,&
&  0.70719E+00_JPRB, 0.70207E+00_JPRB, 0.69667E+00_JPRB, 0.69488E+00_JPRB, 0.69264E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.67564E+00_JPRB, 0.67016E+00_JPRB, 0.67055E+00_JPRB, 0.67282E+00_JPRB, 0.67808E+00_JPRB, 0.68496E+00_JPRB, 0.68827E+00_JPRB ,&
&  0.69073E+00_JPRB, 0.69153E+00_JPRB, 0.68912E+00_JPRB, 0.67818E+00_JPRB, 0.66918E+00_JPRB, 0.66465E+00_JPRB, 0.66006E+00_JPRB ,&
&  0.65303E+00_JPRB, 0.64689E+00_JPRB, 0.64364E+00_JPRB, 0.64343E+00_JPRB, 0.64226E+00_JPRB, 0.64705E+00_JPRB, 0.64290E+00_JPRB ,&
&  0.62507E+00_JPRB, 0.60971E+00_JPRB, 0.58149E+00_JPRB, 0.55633E+00_JPRB, 0.53856E+00_JPRB, 0.52507E+00_JPRB, 0.52636E+00_JPRB ,&
&  0.52102E+00_JPRB, 0.51743E+00_JPRB, 0.51588E+00_JPRB, 0.51643E+00_JPRB, 0.51932E+00_JPRB, 0.42727E+00_JPRB, 0.39995E+00_JPRB ,&
&  0.37978E+00_JPRB, 0.36562E+00_JPRB, 0.36124E+00_JPRB, 0.36092E+00_JPRB, 0.36889E+00_JPRB, 0.39568E+00_JPRB, 0.43763E+00_JPRB ,&
&  0.47001E+00_JPRB, 0.56691E+00_JPRB, 0.58895E+00_JPRB, 0.61394E+00_JPRB, 0.65413E+00_JPRB, 0.69486E+00_JPRB, 0.72362E+00_JPRB ,&
&  0.75524E+00_JPRB, 0.79827E+00_JPRB, 0.84676E+00_JPRB, 0.89415E+00_JPRB, 0.91025E+00_JPRB, 0.93580E+00_JPRB, 0.94625E+00_JPRB ,&
&  0.95230E+00_JPRB, 0.93987E+00_JPRB, 0.93086E+00_JPRB, 0.92346E+00_JPRB, 0.91068E+00_JPRB, 0.90458E+00_JPRB, 0.88995E+00_JPRB ,&
&  0.88408E+00_JPRB, 0.86924E+00_JPRB, 0.87502E+00_JPRB, 0.87558E+00_JPRB, 0.87529E+00_JPRB, 0.87517E+00_JPRB, 0.86988E+00_JPRB ,&
&  0.86284E+00_JPRB, 0.85288E+00_JPRB, 0.83386E+00_JPRB, 0.80740E+00_JPRB, 0.79265E+00_JPRB, 0.77591E+00_JPRB, 0.77320E+00_JPRB ,&
&  0.77463E+00_JPRB, 0.78732E+00_JPRB, 0.78729E+00_JPRB, 0.78547E+00_JPRB, 0.78483E+00_JPRB, 0.78013E+00_JPRB, 0.77924E+00_JPRB ,&
&  0.77712E+00_JPRB, 0.77568E+00_JPRB, 0.76762E+00_JPRB, 0.75389E+00_JPRB, 0.73725E+00_JPRB, 0.72080E+00_JPRB, 0.70684E+00_JPRB ,&
&  0.69776E+00_JPRB, 0.69302E+00_JPRB, 0.68409E+00_JPRB, 0.68051E+00_JPRB, 0.67741E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.66394E+00_JPRB, 0.65948E+00_JPRB, 0.65797E+00_JPRB, 0.66071E+00_JPRB, 0.66939E+00_JPRB, 0.67946E+00_JPRB, 0.68003E+00_JPRB ,&
&  0.68001E+00_JPRB, 0.67957E+00_JPRB, 0.68110E+00_JPRB, 0.67987E+00_JPRB, 0.67797E+00_JPRB, 0.67330E+00_JPRB, 0.66937E+00_JPRB ,&
&  0.66581E+00_JPRB, 0.65794E+00_JPRB, 0.64827E+00_JPRB, 0.64310E+00_JPRB, 0.64158E+00_JPRB, 0.64280E+00_JPRB, 0.64205E+00_JPRB ,&
&  0.63766E+00_JPRB, 0.62506E+00_JPRB, 0.59168E+00_JPRB, 0.56722E+00_JPRB, 0.54288E+00_JPRB, 0.53429E+00_JPRB, 0.52281E+00_JPRB ,&
&  0.53528E+00_JPRB, 0.48311E+00_JPRB, 0.47707E+00_JPRB, 0.45245E+00_JPRB, 0.43156E+00_JPRB, 0.40985E+00_JPRB, 0.39254E+00_JPRB ,&
&  0.38578E+00_JPRB, 0.37918E+00_JPRB, 0.37776E+00_JPRB, 0.38451E+00_JPRB, 0.39573E+00_JPRB, 0.41974E+00_JPRB, 0.44836E+00_JPRB ,&
&  0.47495E+00_JPRB, 0.54927E+00_JPRB, 0.57164E+00_JPRB, 0.58835E+00_JPRB, 0.61724E+00_JPRB, 0.64326E+00_JPRB, 0.66415E+00_JPRB ,&
&  0.68292E+00_JPRB, 0.70046E+00_JPRB, 0.72644E+00_JPRB, 0.74139E+00_JPRB, 0.76357E+00_JPRB, 0.77248E+00_JPRB, 0.77856E+00_JPRB ,&
&  0.78751E+00_JPRB, 0.79718E+00_JPRB, 0.80304E+00_JPRB, 0.80761E+00_JPRB, 0.80904E+00_JPRB, 0.80735E+00_JPRB, 0.80401E+00_JPRB ,&
&  0.79862E+00_JPRB, 0.79306E+00_JPRB, 0.78654E+00_JPRB, 0.77882E+00_JPRB, 0.76977E+00_JPRB, 0.76411E+00_JPRB, 0.75628E+00_JPRB ,&
&  0.74976E+00_JPRB, 0.74041E+00_JPRB, 0.73200E+00_JPRB, 0.72534E+00_JPRB, 0.72614E+00_JPRB, 0.72620E+00_JPRB, 0.73278E+00_JPRB ,&
&  0.74353E+00_JPRB, 0.75014E+00_JPRB, 0.75408E+00_JPRB, 0.75701E+00_JPRB, 0.75922E+00_JPRB, 0.75683E+00_JPRB, 0.75280E+00_JPRB ,&
&  0.74984E+00_JPRB, 0.74586E+00_JPRB, 0.73949E+00_JPRB, 0.72747E+00_JPRB, 0.71473E+00_JPRB, 0.70272E+00_JPRB, 0.69353E+00_JPRB ,&
&  0.68347E+00_JPRB, 0.67762E+00_JPRB, 0.66830E+00_JPRB, 0.66643E+00_JPRB, 0.66421E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.65380E+00_JPRB, 0.65212E+00_JPRB, 0.65030E+00_JPRB, 0.65108E+00_JPRB, 0.65403E+00_JPRB, 0.65915E+00_JPRB, 0.66069E+00_JPRB ,&
&  0.66526E+00_JPRB, 0.66956E+00_JPRB, 0.67417E+00_JPRB, 0.67559E+00_JPRB, 0.67669E+00_JPRB, 0.67479E+00_JPRB, 0.67116E+00_JPRB ,&
&  0.66752E+00_JPRB, 0.66233E+00_JPRB, 0.65875E+00_JPRB, 0.65485E+00_JPRB, 0.65021E+00_JPRB, 0.64725E+00_JPRB, 0.64355E+00_JPRB ,&
&  0.64110E+00_JPRB, 0.63782E+00_JPRB, 0.59183E+00_JPRB, 0.57405E+00_JPRB, 0.55029E+00_JPRB, 0.53672E+00_JPRB, 0.53087E+00_JPRB ,&
&  0.52340E+00_JPRB, 0.51634E+00_JPRB, 0.50658E+00_JPRB, 0.49129E+00_JPRB, 0.47134E+00_JPRB, 0.44544E+00_JPRB, 0.43042E+00_JPRB ,&
&  0.41836E+00_JPRB, 0.41476E+00_JPRB, 0.41641E+00_JPRB, 0.42457E+00_JPRB, 0.43980E+00_JPRB, 0.45845E+00_JPRB, 0.48024E+00_JPRB ,&
&  0.51008E+00_JPRB, 0.54263E+00_JPRB, 0.56431E+00_JPRB, 0.57053E+00_JPRB, 0.59510E+00_JPRB, 0.61214E+00_JPRB, 0.62626E+00_JPRB ,&
&  0.63704E+00_JPRB, 0.64592E+00_JPRB, 0.66567E+00_JPRB, 0.67852E+00_JPRB, 0.68713E+00_JPRB, 0.69354E+00_JPRB, 0.70032E+00_JPRB ,&
&  0.70689E+00_JPRB, 0.71296E+00_JPRB, 0.71607E+00_JPRB, 0.71817E+00_JPRB, 0.71803E+00_JPRB, 0.71606E+00_JPRB, 0.71354E+00_JPRB ,&
&  0.71084E+00_JPRB, 0.70765E+00_JPRB, 0.70328E+00_JPRB, 0.69771E+00_JPRB, 0.69261E+00_JPRB, 0.68784E+00_JPRB, 0.68257E+00_JPRB ,&
&  0.68001E+00_JPRB, 0.67845E+00_JPRB, 0.67817E+00_JPRB, 0.67443E+00_JPRB, 0.67732E+00_JPRB, 0.68803E+00_JPRB, 0.69371E+00_JPRB ,&
&  0.70131E+00_JPRB, 0.70712E+00_JPRB, 0.71284E+00_JPRB, 0.71753E+00_JPRB, 0.72025E+00_JPRB, 0.72059E+00_JPRB, 0.71841E+00_JPRB ,&
&  0.71685E+00_JPRB, 0.71361E+00_JPRB, 0.70872E+00_JPRB, 0.69929E+00_JPRB, 0.68998E+00_JPRB, 0.68093E+00_JPRB, 0.67406E+00_JPRB ,&
&  0.66578E+00_JPRB, 0.66034E+00_JPRB, 0.65362E+00_JPRB, 0.65313E+00_JPRB, 0.65275E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.64862E+00_JPRB, 0.64903E+00_JPRB, 0.64913E+00_JPRB, 0.64976E+00_JPRB, 0.65214E+00_JPRB, 0.65482E+00_JPRB, 0.65622E+00_JPRB ,&
&  0.65864E+00_JPRB, 0.66176E+00_JPRB, 0.66566E+00_JPRB, 0.66753E+00_JPRB, 0.66880E+00_JPRB, 0.66870E+00_JPRB, 0.66769E+00_JPRB ,&
&  0.66569E+00_JPRB, 0.66346E+00_JPRB, 0.66135E+00_JPRB, 0.65973E+00_JPRB, 0.65806E+00_JPRB, 0.65665E+00_JPRB, 0.65320E+00_JPRB ,&
&  0.64975E+00_JPRB, 0.64807E+00_JPRB, 0.64233E+00_JPRB, 0.63405E+00_JPRB, 0.62741E+00_JPRB, 0.57702E+00_JPRB, 0.56025E+00_JPRB ,&
&  0.55586E+00_JPRB, 0.55220E+00_JPRB, 0.55420E+00_JPRB, 0.55783E+00_JPRB, 0.57043E+00_JPRB, 0.56367E+00_JPRB, 0.55850E+00_JPRB ,&
&  0.52783E+00_JPRB, 0.52445E+00_JPRB, 0.51960E+00_JPRB, 0.51813E+00_JPRB, 0.52301E+00_JPRB, 0.52842E+00_JPRB, 0.54117E+00_JPRB ,&
&  0.57281E+00_JPRB, 0.58327E+00_JPRB, 0.58770E+00_JPRB, 0.59246E+00_JPRB, 0.60104E+00_JPRB, 0.60824E+00_JPRB, 0.61417E+00_JPRB ,&
&  0.61902E+00_JPRB, 0.62345E+00_JPRB, 0.62861E+00_JPRB, 0.63453E+00_JPRB, 0.63943E+00_JPRB, 0.64108E+00_JPRB, 0.64294E+00_JPRB ,&
&  0.64445E+00_JPRB, 0.64689E+00_JPRB, 0.64993E+00_JPRB, 0.65278E+00_JPRB, 0.65595E+00_JPRB, 0.65729E+00_JPRB, 0.65819E+00_JPRB ,&
&  0.65894E+00_JPRB, 0.65901E+00_JPRB, 0.65806E+00_JPRB, 0.65782E+00_JPRB, 0.65842E+00_JPRB, 0.65835E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.65909E+00_JPRB, 0.66044E+00_JPRB, 0.66165E+00_JPRB, 0.66530E+00_JPRB, 0.66850E+00_JPRB, 0.67337E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.68220E+00_JPRB, 0.68565E+00_JPRB, 0.68810E+00_JPRB, 0.68929E+00_JPRB, 0.68959E+00_JPRB, 0.68945E+00_JPRB, 0.68814E+00_JPRB ,&
&  0.68648E+00_JPRB, 0.68345E+00_JPRB, 0.67900E+00_JPRB, 0.67295E+00_JPRB, 0.66712E+00_JPRB, 0.66124E+00_JPRB, 0.65608E+00_JPRB ,&
&  0.65087E+00_JPRB, 0.64800E+00_JPRB, 0.64688E+00_JPRB, 0.64741E+00_JPRB, 0.64818E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.65403E+00_JPRB, 0.65477E+00_JPRB, 0.65456E+00_JPRB, 0.65441E+00_JPRB, 0.65419E+00_JPRB, 0.65515E+00_JPRB, 0.65598E+00_JPRB ,&
&  0.65686E+00_JPRB, 0.65817E+00_JPRB, 0.66041E+00_JPRB, 0.66152E+00_JPRB, 0.66231E+00_JPRB, 0.66249E+00_JPRB, 0.66277E+00_JPRB ,&
&  0.66257E+00_JPRB, 0.66186E+00_JPRB, 0.66091E+00_JPRB, 0.66032E+00_JPRB, 0.65977E+00_JPRB, 0.65941E+00_JPRB, 0.65884E+00_JPRB ,&
&  0.65754E+00_JPRB, 0.65599E+00_JPRB, 0.65419E+00_JPRB, 0.65100E+00_JPRB, 0.64704E+00_JPRB, 0.64257E+00_JPRB, 0.63763E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.62826E+00_JPRB, 0.62423E+00_JPRB, 0.62045E+00_JPRB, 0.61904E+00_JPRB, 0.61832E+00_JPRB, 0.61749E+00_JPRB ,&
&  0.61680E+00_JPRB, 0.61533E+00_JPRB, 0.61531E+00_JPRB, 0.61576E+00_JPRB, 0.61663E+00_JPRB, 0.61838E+00_JPRB, 0.61991E+00_JPRB ,&
&  0.62095E+00_JPRB, 0.62082E+00_JPRB, 0.61988E+00_JPRB, 0.62065E+00_JPRB, 0.62264E+00_JPRB, 0.62367E+00_JPRB, 0.62319E+00_JPRB ,&
&  0.62287E+00_JPRB, 0.62181E+00_JPRB, 0.62072E+00_JPRB, 0.61960E+00_JPRB, 0.61988E+00_JPRB, 0.61941E+00_JPRB, 0.61865E+00_JPRB ,&
&  0.61836E+00_JPRB, 0.61877E+00_JPRB, 0.61966E+00_JPRB, 0.62081E+00_JPRB, 0.62278E+00_JPRB, 0.62447E+00_JPRB, 0.62632E+00_JPRB ,&
&  0.62840E+00_JPRB, 0.63174E+00_JPRB, 0.63477E+00_JPRB, 0.63780E+00_JPRB, 0.64131E+00_JPRB, 0.64516E+00_JPRB, 0.64880E+00_JPRB ,&
&  0.65211E+00_JPRB, 0.65486E+00_JPRB, 0.65623E+00_JPRB, 0.65810E+00_JPRB, 0.66011E+00_JPRB, 0.66288E+00_JPRB, 0.66474E+00_JPRB ,&
&  0.66680E+00_JPRB, 0.66824E+00_JPRB, 0.66904E+00_JPRB, 0.66951E+00_JPRB, 0.66917E+00_JPRB, 0.66837E+00_JPRB, 0.66700E+00_JPRB ,&
&  0.66526E+00_JPRB, 0.66360E+00_JPRB, 0.66198E+00_JPRB, 0.66015E+00_JPRB, 0.65837E+00_JPRB, 0.65668E+00_JPRB, 0.65501E+00_JPRB ,&
&  0.65383E+00_JPRB, 0.65362E+00_JPRB, 0.65407E+00_JPRB, 0.65405E+00_JPRB, 0.65398E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.65531E+00_JPRB, 0.65498E+00_JPRB, 0.65462E+00_JPRB, 0.65415E+00_JPRB, 0.65365E+00_JPRB, 0.65349E+00_JPRB, 0.65329E+00_JPRB ,&
&  0.65319E+00_JPRB, 0.65306E+00_JPRB, 0.65343E+00_JPRB, 0.65348E+00_JPRB, 0.65364E+00_JPRB, 0.65361E+00_JPRB, 0.65376E+00_JPRB ,&
&  0.65380E+00_JPRB, 0.65388E+00_JPRB, 0.65391E+00_JPRB, 0.65415E+00_JPRB, 0.65436E+00_JPRB, 0.65478E+00_JPRB, 0.65505E+00_JPRB ,&
&  0.65516E+00_JPRB, 0.65476E+00_JPRB, 0.65460E+00_JPRB, 0.65399E+00_JPRB, 0.65331E+00_JPRB, 0.65199E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.64859E+00_JPRB, 0.64698E+00_JPRB, 0.64555E+00_JPRB, 0.64439E+00_JPRB, 0.64341E+00_JPRB, 0.64283E+00_JPRB, 0.64228E+00_JPRB ,&
&  0.64153E+00_JPRB, 0.64086E+00_JPRB, 0.64032E+00_JPRB, 0.63971E+00_JPRB, 0.63869E+00_JPRB, 0.63748E+00_JPRB, 0.63630E+00_JPRB ,&
&  0.63493E+00_JPRB, 0.63341E+00_JPRB, 0.63231E+00_JPRB, 0.63160E+00_JPRB, 0.63070E+00_JPRB, 0.62916E+00_JPRB, 0.62749E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.62405E+00_JPRB, 0.62204E+00_JPRB, 0.62021E+00_JPRB, 0.61903E+00_JPRB, 0.61802E+00_JPRB, 0.61715E+00_JPRB ,&
&  0.61719E+00_JPRB, 0.61828E+00_JPRB, 0.61939E+00_JPRB, 0.62034E+00_JPRB, 0.62175E+00_JPRB, 0.62322E+00_JPRB, 0.62486E+00_JPRB ,&
&  0.62651E+00_JPRB, 0.62843E+00_JPRB, 0.63072E+00_JPRB, 0.63279E+00_JPRB, 0.63519E+00_JPRB, 0.63773E+00_JPRB, 0.64055E+00_JPRB ,&
&  0.64322E+00_JPRB, 0.64611E+00_JPRB, 0.64869E+00_JPRB, 0.65122E+00_JPRB, 0.65273E+00_JPRB, 0.65412E+00_JPRB, 0.65466E+00_JPRB ,&
&  0.65527E+00_JPRB, 0.65563E+00_JPRB, 0.65631E+00_JPRB, 0.65658E+00_JPRB, 0.65681E+00_JPRB, 0.65688E+00_JPRB, 0.65703E+00_JPRB ,&
&  0.65705E+00_JPRB, 0.65721E+00_JPRB, 0.65719E+00_JPRB, 0.65722E+00_JPRB, 0.65692E+00_JPRB, 0.65692E+00_JPRB, 0.65678E+00_JPRB ,&
&  0.65667E+00_JPRB, 0.65662E+00_JPRB, 0.65663E+00_JPRB, 0.65643E+00_JPRB, 0.65598E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB ,&
&  0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB, 0.64349E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB ,&
&  0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB, 0.20714E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.21083E+00_JPRB, 0.21113E+00_JPRB, 0.21120E+00_JPRB, 0.21117E+00_JPRB, 0.20960E+00_JPRB, 0.20901E+00_JPRB, 0.20846E+00_JPRB ,&
&  0.20805E+00_JPRB, 0.20784E+00_JPRB, 0.20824E+00_JPRB, 0.20875E+00_JPRB, 0.20927E+00_JPRB, 0.20978E+00_JPRB, 0.21020E+00_JPRB ,&
&  0.21039E+00_JPRB, 0.21060E+00_JPRB, 0.20961E+00_JPRB, 0.20914E+00_JPRB, 0.20901E+00_JPRB, 0.20909E+00_JPRB, 0.20909E+00_JPRB ,&
&  0.20921E+00_JPRB, 0.20931E+00_JPRB, 0.20930E+00_JPRB, 0.20963E+00_JPRB, 0.21019E+00_JPRB, 0.21066E+00_JPRB, 0.21115E+00_JPRB ,&
&  0.21162E+00_JPRB, 0.21229E+00_JPRB, 0.21290E+00_JPRB, 0.21348E+00_JPRB, 0.21419E+00_JPRB, 0.21487E+00_JPRB, 0.21524E+00_JPRB ,&
&  0.21551E+00_JPRB, 0.21554E+00_JPRB, 0.21564E+00_JPRB, 0.21572E+00_JPRB, 0.21591E+00_JPRB, 0.21616E+00_JPRB, 0.21619E+00_JPRB ,&
&  0.21609E+00_JPRB, 0.21592E+00_JPRB, 0.21567E+00_JPRB, 0.21514E+00_JPRB, 0.21458E+00_JPRB, 0.21405E+00_JPRB, 0.21352E+00_JPRB ,&
&  0.21306E+00_JPRB, 0.21254E+00_JPRB, 0.21198E+00_JPRB, 0.21138E+00_JPRB, 0.21074E+00_JPRB, 0.21013E+00_JPRB, 0.20951E+00_JPRB ,&
&  0.20888E+00_JPRB, 0.20847E+00_JPRB, 0.20802E+00_JPRB, 0.20764E+00_JPRB, 0.20734E+00_JPRB, 0.20704E+00_JPRB, 0.20676E+00_JPRB ,&
&  0.20647E+00_JPRB, 0.20613E+00_JPRB, 0.20581E+00_JPRB, 0.20546E+00_JPRB, 0.20515E+00_JPRB, 0.20479E+00_JPRB, 0.20451E+00_JPRB ,&
&  0.20424E+00_JPRB, 0.20400E+00_JPRB, 0.20380E+00_JPRB, 0.20371E+00_JPRB, 0.20366E+00_JPRB, 0.20369E+00_JPRB, 0.20373E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20383E+00_JPRB, 0.20386E+00_JPRB, 0.20374E+00_JPRB, 0.20393E+00_JPRB, 0.20413E+00_JPRB, 0.20442E+00_JPRB ,&
&  0.20469E+00_JPRB, 0.20513E+00_JPRB, 0.20557E+00_JPRB, 0.20606E+00_JPRB, 0.20681E+00_JPRB, 0.20708E+00_JPRB, 0.20732E+00_JPRB ,&
&  0.20759E+00_JPRB, 0.20814E+00_JPRB, 0.20862E+00_JPRB, 0.20888E+00_JPRB, 0.20944E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.22304E+00_JPRB, 0.22809E+00_JPRB, 0.23194E+00_JPRB, 0.23557E+00_JPRB, 0.23390E+00_JPRB, 0.25345E+00_JPRB, 0.25145E+00_JPRB ,&
&  0.25121E+00_JPRB, 0.24466E+00_JPRB, 0.24566E+00_JPRB, 0.24435E+00_JPRB, 0.24199E+00_JPRB, 0.23897E+00_JPRB, 0.23576E+00_JPRB ,&
&  0.23245E+00_JPRB, 0.22854E+00_JPRB, 0.22491E+00_JPRB, 0.22145E+00_JPRB, 0.21858E+00_JPRB, 0.21609E+00_JPRB, 0.21471E+00_JPRB ,&
&  0.21473E+00_JPRB, 0.21618E+00_JPRB, 0.21868E+00_JPRB, 0.22012E+00_JPRB, 0.22165E+00_JPRB, 0.22325E+00_JPRB, 0.22495E+00_JPRB ,&
&  0.22640E+00_JPRB, 0.22830E+00_JPRB, 0.22931E+00_JPRB, 0.22897E+00_JPRB, 0.22786E+00_JPRB, 0.22730E+00_JPRB, 0.22705E+00_JPRB ,&
&  0.22743E+00_JPRB, 0.22756E+00_JPRB, 0.22757E+00_JPRB, 0.22774E+00_JPRB, 0.22788E+00_JPRB, 0.22732E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22511E+00_JPRB, 0.22429E+00_JPRB, 0.22273E+00_JPRB, 0.22123E+00_JPRB, 0.21983E+00_JPRB, 0.21837E+00_JPRB, 0.21686E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21466E+00_JPRB, 0.21329E+00_JPRB, 0.21171E+00_JPRB, 0.21020E+00_JPRB, 0.20887E+00_JPRB, 0.20781E+00_JPRB ,&
&  0.20756E+00_JPRB, 0.20699E+00_JPRB, 0.20649E+00_JPRB, 0.20599E+00_JPRB, 0.20585E+00_JPRB, 0.20543E+00_JPRB, 0.20515E+00_JPRB ,&
&  0.20493E+00_JPRB, 0.20465E+00_JPRB, 0.20436E+00_JPRB, 0.20407E+00_JPRB, 0.20388E+00_JPRB, 0.20360E+00_JPRB, 0.20334E+00_JPRB ,&
&  0.20318E+00_JPRB, 0.20305E+00_JPRB, 0.20289E+00_JPRB, 0.20254E+00_JPRB, 0.20218E+00_JPRB, 0.20175E+00_JPRB, 0.20175E+00_JPRB ,&
&  0.20190E+00_JPRB, 0.20218E+00_JPRB, 0.20267E+00_JPRB, 0.20287E+00_JPRB, 0.20368E+00_JPRB, 0.20446E+00_JPRB, 0.20539E+00_JPRB ,&
&  0.20637E+00_JPRB, 0.20787E+00_JPRB, 0.20951E+00_JPRB, 0.21128E+00_JPRB, 0.21406E+00_JPRB, 0.21509E+00_JPRB, 0.21598E+00_JPRB ,&
&  0.21706E+00_JPRB, 0.22048E+00_JPRB, 0.22164E+00_JPRB, 0.22172E+00_JPRB, 0.22256E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.26594E+00_JPRB, 0.26923E+00_JPRB, 0.27147E+00_JPRB, 0.27296E+00_JPRB, 0.27441E+00_JPRB, 0.27375E+00_JPRB, 0.27227E+00_JPRB ,&
&  0.26820E+00_JPRB, 0.26677E+00_JPRB, 0.26406E+00_JPRB, 0.26120E+00_JPRB, 0.25786E+00_JPRB, 0.25417E+00_JPRB, 0.25017E+00_JPRB ,&
&  0.24572E+00_JPRB, 0.24090E+00_JPRB, 0.23512E+00_JPRB, 0.22887E+00_JPRB, 0.22316E+00_JPRB, 0.21783E+00_JPRB, 0.21460E+00_JPRB ,&
&  0.21310E+00_JPRB, 0.21381E+00_JPRB, 0.21656E+00_JPRB, 0.22001E+00_JPRB, 0.22422E+00_JPRB, 0.22898E+00_JPRB, 0.23491E+00_JPRB ,&
&  0.24001E+00_JPRB, 0.24616E+00_JPRB, 0.24989E+00_JPRB, 0.25151E+00_JPRB, 0.25010E+00_JPRB, 0.24996E+00_JPRB, 0.24897E+00_JPRB ,&
&  0.24764E+00_JPRB, 0.24551E+00_JPRB, 0.24454E+00_JPRB, 0.24447E+00_JPRB, 0.24347E+00_JPRB, 0.24144E+00_JPRB, 0.23873E+00_JPRB ,&
&  0.23625E+00_JPRB, 0.23412E+00_JPRB, 0.23151E+00_JPRB, 0.22930E+00_JPRB, 0.22761E+00_JPRB, 0.22558E+00_JPRB, 0.22290E+00_JPRB ,&
&  0.22048E+00_JPRB, 0.21806E+00_JPRB, 0.21520E+00_JPRB, 0.21274E+00_JPRB, 0.21031E+00_JPRB, 0.20854E+00_JPRB, 0.20694E+00_JPRB ,&
&  0.20625E+00_JPRB, 0.20552E+00_JPRB, 0.20530E+00_JPRB, 0.20517E+00_JPRB, 0.20557E+00_JPRB, 0.20531E+00_JPRB, 0.20492E+00_JPRB ,&
&  0.20444E+00_JPRB, 0.20432E+00_JPRB, 0.20381E+00_JPRB, 0.20363E+00_JPRB, 0.20345E+00_JPRB, 0.20283E+00_JPRB, 0.20254E+00_JPRB ,&
&  0.20265E+00_JPRB, 0.20259E+00_JPRB, 0.20210E+00_JPRB, 0.20207E+00_JPRB, 0.20177E+00_JPRB, 0.20131E+00_JPRB, 0.20159E+00_JPRB ,&
&  0.20143E+00_JPRB, 0.20164E+00_JPRB, 0.20196E+00_JPRB, 0.20247E+00_JPRB, 0.20354E+00_JPRB, 0.20491E+00_JPRB, 0.20677E+00_JPRB ,&
&  0.20958E+00_JPRB, 0.21214E+00_JPRB, 0.21473E+00_JPRB, 0.21776E+00_JPRB, 0.22335E+00_JPRB, 0.22638E+00_JPRB, 0.23158E+00_JPRB ,&
&  0.23655E+00_JPRB, 0.23612E+00_JPRB, 0.25883E+00_JPRB, 0.25990E+00_JPRB, 0.26181E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.28007E+00_JPRB, 0.28213E+00_JPRB, 0.28322E+00_JPRB, 0.28524E+00_JPRB, 0.28739E+00_JPRB, 0.28820E+00_JPRB, 0.28886E+00_JPRB ,&
&  0.28631E+00_JPRB, 0.27979E+00_JPRB, 0.27495E+00_JPRB, 0.26855E+00_JPRB, 0.26220E+00_JPRB, 0.25565E+00_JPRB, 0.24966E+00_JPRB ,&
&  0.24353E+00_JPRB, 0.23799E+00_JPRB, 0.23241E+00_JPRB, 0.22772E+00_JPRB, 0.22447E+00_JPRB, 0.22179E+00_JPRB, 0.21886E+00_JPRB ,&
&  0.21633E+00_JPRB, 0.21728E+00_JPRB, 0.21973E+00_JPRB, 0.22454E+00_JPRB, 0.23212E+00_JPRB, 0.23924E+00_JPRB, 0.24699E+00_JPRB ,&
&  0.26075E+00_JPRB, 0.26462E+00_JPRB, 0.26447E+00_JPRB, 0.26634E+00_JPRB, 0.26568E+00_JPRB, 0.26617E+00_JPRB, 0.25612E+00_JPRB ,&
&  0.25798E+00_JPRB, 0.26001E+00_JPRB, 0.26733E+00_JPRB, 0.27054E+00_JPRB, 0.26554E+00_JPRB, 0.26107E+00_JPRB, 0.25314E+00_JPRB ,&
&  0.24647E+00_JPRB, 0.24128E+00_JPRB, 0.23784E+00_JPRB, 0.23576E+00_JPRB, 0.23139E+00_JPRB, 0.22817E+00_JPRB, 0.22279E+00_JPRB ,&
&  0.21826E+00_JPRB, 0.21520E+00_JPRB, 0.21138E+00_JPRB, 0.20897E+00_JPRB, 0.20703E+00_JPRB, 0.20517E+00_JPRB, 0.20376E+00_JPRB ,&
&  0.20298E+00_JPRB, 0.20235E+00_JPRB, 0.20299E+00_JPRB, 0.20375E+00_JPRB, 0.20519E+00_JPRB, 0.20635E+00_JPRB, 0.20663E+00_JPRB ,&
&  0.20691E+00_JPRB, 0.20730E+00_JPRB, 0.20740E+00_JPRB, 0.20796E+00_JPRB, 0.20745E+00_JPRB, 0.20662E+00_JPRB, 0.20559E+00_JPRB ,&
&  0.20450E+00_JPRB, 0.20410E+00_JPRB, 0.20339E+00_JPRB, 0.20295E+00_JPRB, 0.20138E+00_JPRB, 0.20028E+00_JPRB, 0.19968E+00_JPRB ,&
&  0.19912E+00_JPRB, 0.19951E+00_JPRB, 0.19964E+00_JPRB, 0.20044E+00_JPRB, 0.20170E+00_JPRB, 0.20288E+00_JPRB, 0.20492E+00_JPRB ,&
&  0.20751E+00_JPRB, 0.21051E+00_JPRB, 0.21553E+00_JPRB, 0.22092E+00_JPRB, 0.22687E+00_JPRB, 0.23463E+00_JPRB, 0.24205E+00_JPRB ,&
&  0.25237E+00_JPRB, 0.26809E+00_JPRB, 0.27026E+00_JPRB, 0.27314E+00_JPRB, 0.27747E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.29008E+00_JPRB, 0.29123E+00_JPRB, 0.29260E+00_JPRB, 0.29494E+00_JPRB, 0.29781E+00_JPRB, 0.29891E+00_JPRB, 0.29919E+00_JPRB ,&
&  0.29007E+00_JPRB, 0.28243E+00_JPRB, 0.27567E+00_JPRB, 0.26537E+00_JPRB, 0.25721E+00_JPRB, 0.24985E+00_JPRB, 0.24452E+00_JPRB ,&
&  0.23986E+00_JPRB, 0.23739E+00_JPRB, 0.23484E+00_JPRB, 0.23405E+00_JPRB, 0.23406E+00_JPRB, 0.23479E+00_JPRB, 0.23344E+00_JPRB ,&
&  0.23180E+00_JPRB, 0.23281E+00_JPRB, 0.23347E+00_JPRB, 0.23792E+00_JPRB, 0.24441E+00_JPRB, 0.25179E+00_JPRB, 0.25898E+00_JPRB ,&
&  0.26815E+00_JPRB, 0.27214E+00_JPRB, 0.27522E+00_JPRB, 0.27595E+00_JPRB, 0.27453E+00_JPRB, 0.27380E+00_JPRB, 0.27164E+00_JPRB ,&
&  0.27109E+00_JPRB, 0.27052E+00_JPRB, 0.27165E+00_JPRB, 0.27264E+00_JPRB, 0.27130E+00_JPRB, 0.26865E+00_JPRB, 0.26293E+00_JPRB ,&
&  0.25470E+00_JPRB, 0.24800E+00_JPRB, 0.24188E+00_JPRB, 0.23713E+00_JPRB, 0.23073E+00_JPRB, 0.22509E+00_JPRB, 0.21949E+00_JPRB ,&
&  0.21459E+00_JPRB, 0.21086E+00_JPRB, 0.20648E+00_JPRB, 0.20317E+00_JPRB, 0.20164E+00_JPRB, 0.19982E+00_JPRB, 0.19904E+00_JPRB ,&
&  0.19899E+00_JPRB, 0.19934E+00_JPRB, 0.20131E+00_JPRB, 0.20296E+00_JPRB, 0.20555E+00_JPRB, 0.20854E+00_JPRB, 0.21040E+00_JPRB ,&
&  0.21358E+00_JPRB, 0.21633E+00_JPRB, 0.21780E+00_JPRB, 0.21886E+00_JPRB, 0.21793E+00_JPRB, 0.21520E+00_JPRB, 0.21238E+00_JPRB ,&
&  0.20875E+00_JPRB, 0.20616E+00_JPRB, 0.20448E+00_JPRB, 0.20266E+00_JPRB, 0.20174E+00_JPRB, 0.20095E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.19949E+00_JPRB, 0.19921E+00_JPRB, 0.19893E+00_JPRB, 0.19990E+00_JPRB, 0.20097E+00_JPRB, 0.20293E+00_JPRB, 0.20530E+00_JPRB ,&
&  0.20754E+00_JPRB, 0.21094E+00_JPRB, 0.21467E+00_JPRB, 0.21974E+00_JPRB, 0.22870E+00_JPRB, 0.23630E+00_JPRB, 0.25277E+00_JPRB ,&
&  0.26612E+00_JPRB, 0.29355E+00_JPRB, 0.30457E+00_JPRB, 0.29209E+00_JPRB, 0.29048E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.30503E+00_JPRB, 0.30235E+00_JPRB, 0.29981E+00_JPRB, 0.29953E+00_JPRB, 0.30017E+00_JPRB, 0.30309E+00_JPRB, 0.30086E+00_JPRB ,&
&  0.29645E+00_JPRB, 0.29487E+00_JPRB, 0.29628E+00_JPRB, 0.29653E+00_JPRB, 0.29585E+00_JPRB, 0.28533E+00_JPRB, 0.27939E+00_JPRB ,&
&  0.27263E+00_JPRB, 0.27025E+00_JPRB, 0.26838E+00_JPRB, 0.26911E+00_JPRB, 0.27144E+00_JPRB, 0.27289E+00_JPRB, 0.26885E+00_JPRB ,&
&  0.26447E+00_JPRB, 0.26022E+00_JPRB, 0.25590E+00_JPRB, 0.25406E+00_JPRB, 0.25538E+00_JPRB, 0.25683E+00_JPRB, 0.25709E+00_JPRB ,&
&  0.25779E+00_JPRB, 0.26051E+00_JPRB, 0.26298E+00_JPRB, 0.26641E+00_JPRB, 0.27911E+00_JPRB, 0.27750E+00_JPRB, 0.27619E+00_JPRB ,&
&  0.27645E+00_JPRB, 0.27679E+00_JPRB, 0.27774E+00_JPRB, 0.27827E+00_JPRB, 0.28204E+00_JPRB, 0.28508E+00_JPRB, 0.28716E+00_JPRB ,&
&  0.27340E+00_JPRB, 0.26522E+00_JPRB, 0.25735E+00_JPRB, 0.24834E+00_JPRB, 0.23816E+00_JPRB, 0.22984E+00_JPRB, 0.22217E+00_JPRB ,&
&  0.21564E+00_JPRB, 0.20947E+00_JPRB, 0.20396E+00_JPRB, 0.20180E+00_JPRB, 0.20105E+00_JPRB, 0.19991E+00_JPRB, 0.19985E+00_JPRB ,&
&  0.20040E+00_JPRB, 0.20114E+00_JPRB, 0.20163E+00_JPRB, 0.20310E+00_JPRB, 0.20538E+00_JPRB, 0.20892E+00_JPRB, 0.21272E+00_JPRB ,&
&  0.21843E+00_JPRB, 0.22274E+00_JPRB, 0.22776E+00_JPRB, 0.23184E+00_JPRB, 0.23038E+00_JPRB, 0.22548E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21395E+00_JPRB, 0.21032E+00_JPRB, 0.20667E+00_JPRB, 0.20403E+00_JPRB, 0.20252E+00_JPRB, 0.20167E+00_JPRB, 0.20198E+00_JPRB ,&
&  0.20214E+00_JPRB, 0.20262E+00_JPRB, 0.20222E+00_JPRB, 0.20247E+00_JPRB, 0.20329E+00_JPRB, 0.20505E+00_JPRB, 0.20735E+00_JPRB ,&
&  0.21090E+00_JPRB, 0.21502E+00_JPRB, 0.21996E+00_JPRB, 0.22539E+00_JPRB, 0.23321E+00_JPRB, 0.24366E+00_JPRB, 0.25874E+00_JPRB ,&
&  0.27566E+00_JPRB, 0.30799E+00_JPRB, 0.32098E+00_JPRB, 0.31138E+00_JPRB, 0.30832E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.32251E+00_JPRB, 0.31919E+00_JPRB, 0.31733E+00_JPRB, 0.31766E+00_JPRB, 0.31786E+00_JPRB, 0.32058E+00_JPRB, 0.32004E+00_JPRB ,&
&  0.31581E+00_JPRB, 0.31123E+00_JPRB, 0.30979E+00_JPRB, 0.30959E+00_JPRB, 0.30846E+00_JPRB, 0.30725E+00_JPRB, 0.30787E+00_JPRB ,&
&  0.30633E+00_JPRB, 0.30802E+00_JPRB, 0.31069E+00_JPRB, 0.31207E+00_JPRB, 0.31281E+00_JPRB, 0.31303E+00_JPRB, 0.31319E+00_JPRB ,&
&  0.30714E+00_JPRB, 0.30551E+00_JPRB, 0.31105E+00_JPRB, 0.30843E+00_JPRB, 0.30463E+00_JPRB, 0.30137E+00_JPRB, 0.30028E+00_JPRB ,&
&  0.29215E+00_JPRB, 0.28078E+00_JPRB, 0.26792E+00_JPRB, 0.27291E+00_JPRB, 0.28978E+00_JPRB, 0.28774E+00_JPRB, 0.28548E+00_JPRB ,&
&  0.28516E+00_JPRB, 0.28450E+00_JPRB, 0.28531E+00_JPRB, 0.28577E+00_JPRB, 0.28659E+00_JPRB, 0.28661E+00_JPRB, 0.28764E+00_JPRB ,&
&  0.28895E+00_JPRB, 0.29447E+00_JPRB, 0.29816E+00_JPRB, 0.27958E+00_JPRB, 0.26320E+00_JPRB, 0.25068E+00_JPRB, 0.23878E+00_JPRB ,&
&  0.22819E+00_JPRB, 0.22530E+00_JPRB, 0.22188E+00_JPRB, 0.21613E+00_JPRB, 0.21667E+00_JPRB, 0.21723E+00_JPRB, 0.21650E+00_JPRB ,&
&  0.21466E+00_JPRB, 0.21353E+00_JPRB, 0.21212E+00_JPRB, 0.20980E+00_JPRB, 0.20890E+00_JPRB, 0.21008E+00_JPRB, 0.21341E+00_JPRB ,&
&  0.21873E+00_JPRB, 0.22487E+00_JPRB, 0.23881E+00_JPRB, 0.25085E+00_JPRB, 0.24764E+00_JPRB, 0.23984E+00_JPRB, 0.23247E+00_JPRB ,&
&  0.22245E+00_JPRB, 0.21742E+00_JPRB, 0.21249E+00_JPRB, 0.20852E+00_JPRB, 0.20672E+00_JPRB, 0.20698E+00_JPRB, 0.20786E+00_JPRB ,&
&  0.20910E+00_JPRB, 0.21102E+00_JPRB, 0.21180E+00_JPRB, 0.21139E+00_JPRB, 0.21062E+00_JPRB, 0.21107E+00_JPRB, 0.21258E+00_JPRB ,&
&  0.21543E+00_JPRB, 0.22040E+00_JPRB, 0.22649E+00_JPRB, 0.23398E+00_JPRB, 0.24043E+00_JPRB, 0.24643E+00_JPRB, 0.25562E+00_JPRB ,&
&  0.26348E+00_JPRB, 0.28122E+00_JPRB, 0.30702E+00_JPRB, 0.33342E+00_JPRB, 0.32902E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.32593E+00_JPRB, 0.32273E+00_JPRB, 0.32296E+00_JPRB, 0.32337E+00_JPRB, 0.32235E+00_JPRB, 0.32238E+00_JPRB, 0.32184E+00_JPRB ,&
&  0.32175E+00_JPRB, 0.31995E+00_JPRB, 0.31800E+00_JPRB, 0.31829E+00_JPRB, 0.32026E+00_JPRB, 0.32249E+00_JPRB, 0.32298E+00_JPRB ,&
&  0.32266E+00_JPRB, 0.32383E+00_JPRB, 0.32699E+00_JPRB, 0.33046E+00_JPRB, 0.33122E+00_JPRB, 0.33155E+00_JPRB, 0.33336E+00_JPRB ,&
&  0.33568E+00_JPRB, 0.33650E+00_JPRB, 0.33920E+00_JPRB, 0.33706E+00_JPRB, 0.33269E+00_JPRB, 0.33087E+00_JPRB, 0.32927E+00_JPRB ,&
&  0.32321E+00_JPRB, 0.30025E+00_JPRB, 0.28505E+00_JPRB, 0.28900E+00_JPRB, 0.30500E+00_JPRB, 0.30471E+00_JPRB, 0.30264E+00_JPRB ,&
&  0.30053E+00_JPRB, 0.29775E+00_JPRB, 0.29535E+00_JPRB, 0.29443E+00_JPRB, 0.29442E+00_JPRB, 0.29421E+00_JPRB, 0.29262E+00_JPRB ,&
&  0.29283E+00_JPRB, 0.29717E+00_JPRB, 0.30316E+00_JPRB, 0.30115E+00_JPRB, 0.29753E+00_JPRB, 0.29108E+00_JPRB, 0.28697E+00_JPRB ,&
&  0.28307E+00_JPRB, 0.27884E+00_JPRB, 0.27637E+00_JPRB, 0.27552E+00_JPRB, 0.27522E+00_JPRB, 0.27328E+00_JPRB, 0.27001E+00_JPRB ,&
&  0.26229E+00_JPRB, 0.25341E+00_JPRB, 0.24405E+00_JPRB, 0.23487E+00_JPRB, 0.22796E+00_JPRB, 0.22317E+00_JPRB, 0.21958E+00_JPRB ,&
&  0.22102E+00_JPRB, 0.23156E+00_JPRB, 0.24954E+00_JPRB, 0.26908E+00_JPRB, 0.28974E+00_JPRB, 0.26725E+00_JPRB, 0.25632E+00_JPRB ,&
&  0.24529E+00_JPRB, 0.23582E+00_JPRB, 0.22928E+00_JPRB, 0.22483E+00_JPRB, 0.22164E+00_JPRB, 0.21987E+00_JPRB, 0.21983E+00_JPRB ,&
&  0.22261E+00_JPRB, 0.22649E+00_JPRB, 0.22998E+00_JPRB, 0.23072E+00_JPRB, 0.22992E+00_JPRB, 0.22815E+00_JPRB, 0.22716E+00_JPRB ,&
&  0.22820E+00_JPRB, 0.23329E+00_JPRB, 0.24195E+00_JPRB, 0.25443E+00_JPRB, 0.26587E+00_JPRB, 0.27777E+00_JPRB, 0.29184E+00_JPRB ,&
&  0.30702E+00_JPRB, 0.32171E+00_JPRB, 0.32542E+00_JPRB, 0.32854E+00_JPRB, 0.32864E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.33562E+00_JPRB, 0.33345E+00_JPRB, 0.33373E+00_JPRB, 0.33479E+00_JPRB, 0.33462E+00_JPRB, 0.33413E+00_JPRB, 0.33178E+00_JPRB ,&
&  0.32966E+00_JPRB, 0.33011E+00_JPRB, 0.33147E+00_JPRB, 0.33392E+00_JPRB, 0.33820E+00_JPRB, 0.34271E+00_JPRB, 0.34203E+00_JPRB ,&
&  0.33884E+00_JPRB, 0.33736E+00_JPRB, 0.33938E+00_JPRB, 0.34153E+00_JPRB, 0.34195E+00_JPRB, 0.34261E+00_JPRB, 0.34619E+00_JPRB ,&
&  0.35076E+00_JPRB, 0.35279E+00_JPRB, 0.35567E+00_JPRB, 0.35527E+00_JPRB, 0.35168E+00_JPRB, 0.35190E+00_JPRB, 0.35012E+00_JPRB ,&
&  0.34525E+00_JPRB, 0.33946E+00_JPRB, 0.33046E+00_JPRB, 0.33141E+00_JPRB, 0.32779E+00_JPRB, 0.32483E+00_JPRB, 0.32047E+00_JPRB ,&
&  0.31574E+00_JPRB, 0.31188E+00_JPRB, 0.30755E+00_JPRB, 0.30404E+00_JPRB, 0.30220E+00_JPRB, 0.30209E+00_JPRB, 0.29991E+00_JPRB ,&
&  0.29846E+00_JPRB, 0.29988E+00_JPRB, 0.30132E+00_JPRB, 0.29905E+00_JPRB, 0.29940E+00_JPRB, 0.30599E+00_JPRB, 0.31111E+00_JPRB ,&
&  0.31296E+00_JPRB, 0.31285E+00_JPRB, 0.31029E+00_JPRB, 0.30962E+00_JPRB, 0.30855E+00_JPRB, 0.30656E+00_JPRB, 0.30323E+00_JPRB ,&
&  0.29832E+00_JPRB, 0.29589E+00_JPRB, 0.29424E+00_JPRB, 0.29089E+00_JPRB, 0.28571E+00_JPRB, 0.26869E+00_JPRB, 0.25932E+00_JPRB ,&
&  0.25892E+00_JPRB, 0.26772E+00_JPRB, 0.28275E+00_JPRB, 0.28911E+00_JPRB, 0.29439E+00_JPRB, 0.29930E+00_JPRB, 0.30018E+00_JPRB ,&
&  0.30190E+00_JPRB, 0.28157E+00_JPRB, 0.27054E+00_JPRB, 0.26203E+00_JPRB, 0.25432E+00_JPRB, 0.24925E+00_JPRB, 0.24778E+00_JPRB ,&
&  0.25044E+00_JPRB, 0.25629E+00_JPRB, 0.26126E+00_JPRB, 0.26363E+00_JPRB, 0.26327E+00_JPRB, 0.26008E+00_JPRB, 0.25684E+00_JPRB ,&
&  0.25639E+00_JPRB, 0.26270E+00_JPRB, 0.27529E+00_JPRB, 0.28831E+00_JPRB, 0.29716E+00_JPRB, 0.30657E+00_JPRB, 0.31453E+00_JPRB ,&
&  0.32003E+00_JPRB, 0.32517E+00_JPRB, 0.33156E+00_JPRB, 0.33397E+00_JPRB, 0.33515E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.33666E+00_JPRB, 0.33677E+00_JPRB, 0.34016E+00_JPRB, 0.34383E+00_JPRB, 0.34485E+00_JPRB, 0.34486E+00_JPRB, 0.34258E+00_JPRB ,&
&  0.34104E+00_JPRB, 0.34224E+00_JPRB, 0.34403E+00_JPRB, 0.34526E+00_JPRB, 0.34723E+00_JPRB, 0.34980E+00_JPRB, 0.35179E+00_JPRB ,&
&  0.35178E+00_JPRB, 0.34996E+00_JPRB, 0.34683E+00_JPRB, 0.34750E+00_JPRB, 0.34748E+00_JPRB, 0.34946E+00_JPRB, 0.35679E+00_JPRB ,&
&  0.36206E+00_JPRB, 0.36569E+00_JPRB, 0.36877E+00_JPRB, 0.36920E+00_JPRB, 0.36874E+00_JPRB, 0.37028E+00_JPRB, 0.36705E+00_JPRB ,&
&  0.36095E+00_JPRB, 0.35530E+00_JPRB, 0.35028E+00_JPRB, 0.34323E+00_JPRB, 0.34571E+00_JPRB, 0.34314E+00_JPRB, 0.34003E+00_JPRB ,&
&  0.33592E+00_JPRB, 0.33077E+00_JPRB, 0.32485E+00_JPRB, 0.31932E+00_JPRB, 0.31526E+00_JPRB, 0.31151E+00_JPRB, 0.30651E+00_JPRB ,&
&  0.30484E+00_JPRB, 0.30671E+00_JPRB, 0.31245E+00_JPRB, 0.31535E+00_JPRB, 0.31323E+00_JPRB, 0.31433E+00_JPRB, 0.32102E+00_JPRB ,&
&  0.32773E+00_JPRB, 0.33080E+00_JPRB, 0.33061E+00_JPRB, 0.33111E+00_JPRB, 0.33171E+00_JPRB, 0.33305E+00_JPRB, 0.33369E+00_JPRB ,&
&  0.33266E+00_JPRB, 0.33104E+00_JPRB, 0.32852E+00_JPRB, 0.32252E+00_JPRB, 0.31537E+00_JPRB, 0.30940E+00_JPRB, 0.30698E+00_JPRB ,&
&  0.30952E+00_JPRB, 0.30931E+00_JPRB, 0.30932E+00_JPRB, 0.31082E+00_JPRB, 0.31135E+00_JPRB, 0.31059E+00_JPRB, 0.30958E+00_JPRB ,&
&  0.31079E+00_JPRB, 0.31007E+00_JPRB, 0.31163E+00_JPRB, 0.30885E+00_JPRB, 0.30430E+00_JPRB, 0.29987E+00_JPRB, 0.29568E+00_JPRB ,&
&  0.29319E+00_JPRB, 0.29227E+00_JPRB, 0.29420E+00_JPRB, 0.29483E+00_JPRB, 0.29665E+00_JPRB, 0.29880E+00_JPRB, 0.30062E+00_JPRB ,&
&  0.30164E+00_JPRB, 0.30290E+00_JPRB, 0.30532E+00_JPRB, 0.31033E+00_JPRB, 0.31562E+00_JPRB, 0.32024E+00_JPRB, 0.32505E+00_JPRB ,&
&  0.32957E+00_JPRB, 0.33282E+00_JPRB, 0.33870E+00_JPRB, 0.34006E+00_JPRB, 0.33953E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.34300E+00_JPRB, 0.34531E+00_JPRB, 0.34761E+00_JPRB, 0.34965E+00_JPRB, 0.35101E+00_JPRB, 0.35206E+00_JPRB, 0.35127E+00_JPRB ,&
&  0.35110E+00_JPRB, 0.35267E+00_JPRB, 0.35474E+00_JPRB, 0.35641E+00_JPRB, 0.35734E+00_JPRB, 0.35915E+00_JPRB, 0.36089E+00_JPRB ,&
&  0.36201E+00_JPRB, 0.36107E+00_JPRB, 0.35781E+00_JPRB, 0.35644E+00_JPRB, 0.35536E+00_JPRB, 0.36026E+00_JPRB, 0.36867E+00_JPRB ,&
&  0.37242E+00_JPRB, 0.37477E+00_JPRB, 0.37765E+00_JPRB, 0.37829E+00_JPRB, 0.37867E+00_JPRB, 0.37849E+00_JPRB, 0.37409E+00_JPRB ,&
&  0.36923E+00_JPRB, 0.36476E+00_JPRB, 0.36200E+00_JPRB, 0.35870E+00_JPRB, 0.35848E+00_JPRB, 0.35608E+00_JPRB, 0.35433E+00_JPRB ,&
&  0.35306E+00_JPRB, 0.35060E+00_JPRB, 0.34758E+00_JPRB, 0.34255E+00_JPRB, 0.33628E+00_JPRB, 0.32819E+00_JPRB, 0.32269E+00_JPRB ,&
&  0.32055E+00_JPRB, 0.32405E+00_JPRB, 0.33241E+00_JPRB, 0.33644E+00_JPRB, 0.33360E+00_JPRB, 0.33316E+00_JPRB, 0.33649E+00_JPRB ,&
&  0.34123E+00_JPRB, 0.34639E+00_JPRB, 0.34812E+00_JPRB, 0.34723E+00_JPRB, 0.34533E+00_JPRB, 0.34689E+00_JPRB, 0.35103E+00_JPRB ,&
&  0.35468E+00_JPRB, 0.35667E+00_JPRB, 0.35602E+00_JPRB, 0.35515E+00_JPRB, 0.35257E+00_JPRB, 0.34670E+00_JPRB, 0.33880E+00_JPRB ,&
&  0.33165E+00_JPRB, 0.32722E+00_JPRB, 0.32387E+00_JPRB, 0.32068E+00_JPRB, 0.31955E+00_JPRB, 0.32000E+00_JPRB, 0.32224E+00_JPRB ,&
&  0.32317E+00_JPRB, 0.32504E+00_JPRB, 0.32518E+00_JPRB, 0.32103E+00_JPRB, 0.31771E+00_JPRB, 0.31896E+00_JPRB, 0.32045E+00_JPRB ,&
&  0.32295E+00_JPRB, 0.32540E+00_JPRB, 0.32791E+00_JPRB, 0.32899E+00_JPRB, 0.33059E+00_JPRB, 0.33318E+00_JPRB, 0.33468E+00_JPRB ,&
&  0.33542E+00_JPRB, 0.33600E+00_JPRB, 0.33777E+00_JPRB, 0.34064E+00_JPRB, 0.34318E+00_JPRB, 0.34519E+00_JPRB, 0.34457E+00_JPRB ,&
&  0.34357E+00_JPRB, 0.34251E+00_JPRB, 0.34326E+00_JPRB, 0.34387E+00_JPRB, 0.34272E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.36728E+00_JPRB, 0.36756E+00_JPRB, 0.36705E+00_JPRB, 0.36602E+00_JPRB, 0.36491E+00_JPRB, 0.36398E+00_JPRB, 0.36396E+00_JPRB ,&
&  0.36405E+00_JPRB, 0.36460E+00_JPRB, 0.36535E+00_JPRB, 0.36596E+00_JPRB, 0.36749E+00_JPRB, 0.36927E+00_JPRB, 0.37067E+00_JPRB ,&
&  0.37219E+00_JPRB, 0.37142E+00_JPRB, 0.36890E+00_JPRB, 0.36648E+00_JPRB, 0.36720E+00_JPRB, 0.37253E+00_JPRB, 0.37849E+00_JPRB ,&
&  0.38010E+00_JPRB, 0.38127E+00_JPRB, 0.38181E+00_JPRB, 0.38074E+00_JPRB, 0.37941E+00_JPRB, 0.37904E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.37102E+00_JPRB, 0.36971E+00_JPRB, 0.36933E+00_JPRB, 0.36682E+00_JPRB, 0.36646E+00_JPRB, 0.36871E+00_JPRB, 0.36762E+00_JPRB ,&
&  0.36659E+00_JPRB, 0.36479E+00_JPRB, 0.36175E+00_JPRB, 0.35915E+00_JPRB, 0.35610E+00_JPRB, 0.35027E+00_JPRB, 0.34350E+00_JPRB ,&
&  0.33803E+00_JPRB, 0.33819E+00_JPRB, 0.34351E+00_JPRB, 0.34924E+00_JPRB, 0.34858E+00_JPRB, 0.34782E+00_JPRB, 0.34942E+00_JPRB ,&
&  0.35301E+00_JPRB, 0.35743E+00_JPRB, 0.36022E+00_JPRB, 0.35941E+00_JPRB, 0.35827E+00_JPRB, 0.36010E+00_JPRB, 0.36271E+00_JPRB ,&
&  0.36401E+00_JPRB, 0.36550E+00_JPRB, 0.36656E+00_JPRB, 0.36801E+00_JPRB, 0.36845E+00_JPRB, 0.36679E+00_JPRB, 0.36559E+00_JPRB ,&
&  0.36278E+00_JPRB, 0.35893E+00_JPRB, 0.35410E+00_JPRB, 0.34865E+00_JPRB, 0.34480E+00_JPRB, 0.34257E+00_JPRB, 0.34279E+00_JPRB ,&
&  0.34231E+00_JPRB, 0.34384E+00_JPRB, 0.34422E+00_JPRB, 0.34197E+00_JPRB, 0.33895E+00_JPRB, 0.33892E+00_JPRB, 0.34238E+00_JPRB ,&
&  0.34710E+00_JPRB, 0.35039E+00_JPRB, 0.35596E+00_JPRB, 0.35882E+00_JPRB, 0.36087E+00_JPRB, 0.36330E+00_JPRB, 0.36391E+00_JPRB ,&
&  0.36130E+00_JPRB, 0.35918E+00_JPRB, 0.35873E+00_JPRB, 0.36143E+00_JPRB, 0.36238E+00_JPRB, 0.36220E+00_JPRB, 0.36058E+00_JPRB ,&
&  0.36060E+00_JPRB, 0.36303E+00_JPRB, 0.36427E+00_JPRB, 0.36459E+00_JPRB, 0.36566E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.39780E+00_JPRB, 0.39627E+00_JPRB, 0.39468E+00_JPRB, 0.39210E+00_JPRB, 0.38922E+00_JPRB, 0.38647E+00_JPRB, 0.38387E+00_JPRB ,&
&  0.38199E+00_JPRB, 0.38133E+00_JPRB, 0.38155E+00_JPRB, 0.38144E+00_JPRB, 0.38095E+00_JPRB, 0.38145E+00_JPRB, 0.38151E+00_JPRB ,&
&  0.38279E+00_JPRB, 0.38289E+00_JPRB, 0.38069E+00_JPRB, 0.38033E+00_JPRB, 0.38288E+00_JPRB, 0.38771E+00_JPRB, 0.38971E+00_JPRB ,&
&  0.38764E+00_JPRB, 0.38757E+00_JPRB, 0.38751E+00_JPRB, 0.38555E+00_JPRB, 0.38420E+00_JPRB, 0.38329E+00_JPRB, 0.37940E+00_JPRB ,&
&  0.37467E+00_JPRB, 0.37495E+00_JPRB, 0.37673E+00_JPRB, 0.37994E+00_JPRB, 0.37967E+00_JPRB, 0.37766E+00_JPRB, 0.37790E+00_JPRB ,&
&  0.37594E+00_JPRB, 0.37549E+00_JPRB, 0.37512E+00_JPRB, 0.37564E+00_JPRB, 0.37406E+00_JPRB, 0.37176E+00_JPRB, 0.36744E+00_JPRB ,&
&  0.35869E+00_JPRB, 0.35596E+00_JPRB, 0.35812E+00_JPRB, 0.36247E+00_JPRB, 0.36209E+00_JPRB, 0.36293E+00_JPRB, 0.36603E+00_JPRB ,&
&  0.36991E+00_JPRB, 0.37276E+00_JPRB, 0.37452E+00_JPRB, 0.37573E+00_JPRB, 0.37736E+00_JPRB, 0.37891E+00_JPRB, 0.37982E+00_JPRB ,&
&  0.38104E+00_JPRB, 0.38309E+00_JPRB, 0.38549E+00_JPRB, 0.38792E+00_JPRB, 0.38919E+00_JPRB, 0.39023E+00_JPRB, 0.39063E+00_JPRB ,&
&  0.38960E+00_JPRB, 0.38631E+00_JPRB, 0.38248E+00_JPRB, 0.37707E+00_JPRB, 0.37158E+00_JPRB, 0.36845E+00_JPRB, 0.36684E+00_JPRB ,&
&  0.36333E+00_JPRB, 0.36032E+00_JPRB, 0.35815E+00_JPRB, 0.35773E+00_JPRB, 0.35799E+00_JPRB, 0.35921E+00_JPRB, 0.36142E+00_JPRB ,&
&  0.36327E+00_JPRB, 0.36615E+00_JPRB, 0.37246E+00_JPRB, 0.37684E+00_JPRB, 0.37908E+00_JPRB, 0.38215E+00_JPRB, 0.38357E+00_JPRB ,&
&  0.38187E+00_JPRB, 0.38032E+00_JPRB, 0.37922E+00_JPRB, 0.38133E+00_JPRB, 0.38473E+00_JPRB, 0.38763E+00_JPRB, 0.38996E+00_JPRB ,&
&  0.39385E+00_JPRB, 0.39769E+00_JPRB, 0.39886E+00_JPRB, 0.39910E+00_JPRB, 0.39916E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.42604E+00_JPRB, 0.42165E+00_JPRB, 0.41686E+00_JPRB, 0.41248E+00_JPRB, 0.40852E+00_JPRB, 0.40446E+00_JPRB, 0.40075E+00_JPRB ,&
&  0.39708E+00_JPRB, 0.39456E+00_JPRB, 0.39258E+00_JPRB, 0.39088E+00_JPRB, 0.38729E+00_JPRB, 0.38555E+00_JPRB, 0.38602E+00_JPRB ,&
&  0.38823E+00_JPRB, 0.39046E+00_JPRB, 0.39225E+00_JPRB, 0.39183E+00_JPRB, 0.39416E+00_JPRB, 0.39644E+00_JPRB, 0.39675E+00_JPRB ,&
&  0.39610E+00_JPRB, 0.39546E+00_JPRB, 0.39356E+00_JPRB, 0.39103E+00_JPRB, 0.39109E+00_JPRB, 0.39148E+00_JPRB, 0.38785E+00_JPRB ,&
&  0.38121E+00_JPRB, 0.38238E+00_JPRB, 0.37488E+00_JPRB, 0.38789E+00_JPRB, 0.39090E+00_JPRB, 0.38596E+00_JPRB, 0.37983E+00_JPRB ,&
&  0.37775E+00_JPRB, 0.37857E+00_JPRB, 0.38166E+00_JPRB, 0.38383E+00_JPRB, 0.38472E+00_JPRB, 0.38501E+00_JPRB, 0.38512E+00_JPRB ,&
&  0.37813E+00_JPRB, 0.37410E+00_JPRB, 0.37383E+00_JPRB, 0.37471E+00_JPRB, 0.37482E+00_JPRB, 0.37556E+00_JPRB, 0.37750E+00_JPRB ,&
&  0.37897E+00_JPRB, 0.37997E+00_JPRB, 0.38112E+00_JPRB, 0.38372E+00_JPRB, 0.38560E+00_JPRB, 0.38661E+00_JPRB, 0.38857E+00_JPRB ,&
&  0.39159E+00_JPRB, 0.39528E+00_JPRB, 0.39839E+00_JPRB, 0.40066E+00_JPRB, 0.40502E+00_JPRB, 0.41060E+00_JPRB, 0.41480E+00_JPRB ,&
&  0.41596E+00_JPRB, 0.41474E+00_JPRB, 0.41224E+00_JPRB, 0.40712E+00_JPRB, 0.40015E+00_JPRB, 0.39471E+00_JPRB, 0.39053E+00_JPRB ,&
&  0.38444E+00_JPRB, 0.38155E+00_JPRB, 0.37993E+00_JPRB, 0.37751E+00_JPRB, 0.37548E+00_JPRB, 0.37469E+00_JPRB, 0.37492E+00_JPRB ,&
&  0.37674E+00_JPRB, 0.38022E+00_JPRB, 0.38253E+00_JPRB, 0.38541E+00_JPRB, 0.38930E+00_JPRB, 0.39318E+00_JPRB, 0.39348E+00_JPRB ,&
&  0.39224E+00_JPRB, 0.39187E+00_JPRB, 0.39397E+00_JPRB, 0.40125E+00_JPRB, 0.41089E+00_JPRB, 0.41920E+00_JPRB, 0.42674E+00_JPRB ,&
&  0.43229E+00_JPRB, 0.43498E+00_JPRB, 0.43447E+00_JPRB, 0.43332E+00_JPRB, 0.43024E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.46454E+00_JPRB, 0.45545E+00_JPRB, 0.44589E+00_JPRB, 0.43849E+00_JPRB, 0.43301E+00_JPRB, 0.42750E+00_JPRB, 0.42113E+00_JPRB ,&
&  0.41617E+00_JPRB, 0.40993E+00_JPRB, 0.40497E+00_JPRB, 0.40029E+00_JPRB, 0.39642E+00_JPRB, 0.39490E+00_JPRB, 0.39539E+00_JPRB ,&
&  0.39769E+00_JPRB, 0.40290E+00_JPRB, 0.40589E+00_JPRB, 0.40454E+00_JPRB, 0.40631E+00_JPRB, 0.40852E+00_JPRB, 0.40904E+00_JPRB ,&
&  0.40687E+00_JPRB, 0.40389E+00_JPRB, 0.40059E+00_JPRB, 0.39862E+00_JPRB, 0.39860E+00_JPRB, 0.39973E+00_JPRB, 0.39676E+00_JPRB ,&
&  0.39186E+00_JPRB, 0.38371E+00_JPRB, 0.39525E+00_JPRB, 0.40202E+00_JPRB, 0.40232E+00_JPRB, 0.39877E+00_JPRB, 0.39408E+00_JPRB ,&
&  0.38762E+00_JPRB, 0.38401E+00_JPRB, 0.38476E+00_JPRB, 0.38718E+00_JPRB, 0.38997E+00_JPRB, 0.39143E+00_JPRB, 0.39289E+00_JPRB ,&
&  0.38988E+00_JPRB, 0.38824E+00_JPRB, 0.38633E+00_JPRB, 0.38512E+00_JPRB, 0.38571E+00_JPRB, 0.38664E+00_JPRB, 0.38719E+00_JPRB ,&
&  0.38618E+00_JPRB, 0.38679E+00_JPRB, 0.38763E+00_JPRB, 0.38691E+00_JPRB, 0.38855E+00_JPRB, 0.39105E+00_JPRB, 0.39439E+00_JPRB ,&
&  0.39879E+00_JPRB, 0.40470E+00_JPRB, 0.41109E+00_JPRB, 0.41934E+00_JPRB, 0.43058E+00_JPRB, 0.44150E+00_JPRB, 0.44962E+00_JPRB ,&
&  0.45332E+00_JPRB, 0.45221E+00_JPRB, 0.44756E+00_JPRB, 0.43949E+00_JPRB, 0.43197E+00_JPRB, 0.42565E+00_JPRB, 0.41785E+00_JPRB ,&
&  0.40842E+00_JPRB, 0.40241E+00_JPRB, 0.39873E+00_JPRB, 0.39680E+00_JPRB, 0.39471E+00_JPRB, 0.39368E+00_JPRB, 0.39452E+00_JPRB ,&
&  0.39709E+00_JPRB, 0.39967E+00_JPRB, 0.40097E+00_JPRB, 0.40161E+00_JPRB, 0.40274E+00_JPRB, 0.40183E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.40396E+00_JPRB, 0.40982E+00_JPRB, 0.42114E+00_JPRB, 0.44233E+00_JPRB, 0.45899E+00_JPRB, 0.47129E+00_JPRB, 0.47927E+00_JPRB ,&
&  0.48310E+00_JPRB, 0.48446E+00_JPRB, 0.48224E+00_JPRB, 0.47849E+00_JPRB, 0.47285E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.49534E+00_JPRB, 0.48245E+00_JPRB, 0.46889E+00_JPRB, 0.45945E+00_JPRB, 0.45244E+00_JPRB, 0.44569E+00_JPRB, 0.43838E+00_JPRB ,&
&  0.43028E+00_JPRB, 0.42285E+00_JPRB, 0.41599E+00_JPRB, 0.40921E+00_JPRB, 0.40553E+00_JPRB, 0.40356E+00_JPRB, 0.40379E+00_JPRB ,&
&  0.40622E+00_JPRB, 0.41078E+00_JPRB, 0.41285E+00_JPRB, 0.41261E+00_JPRB, 0.41324E+00_JPRB, 0.41296E+00_JPRB, 0.41124E+00_JPRB ,&
&  0.40716E+00_JPRB, 0.40221E+00_JPRB, 0.39956E+00_JPRB, 0.40035E+00_JPRB, 0.40058E+00_JPRB, 0.40137E+00_JPRB, 0.40148E+00_JPRB ,&
&  0.40484E+00_JPRB, 0.37131E+00_JPRB, 0.39733E+00_JPRB, 0.41613E+00_JPRB, 0.41112E+00_JPRB, 0.40570E+00_JPRB, 0.39958E+00_JPRB ,&
&  0.39249E+00_JPRB, 0.38917E+00_JPRB, 0.38798E+00_JPRB, 0.39010E+00_JPRB, 0.39374E+00_JPRB, 0.39546E+00_JPRB, 0.39548E+00_JPRB ,&
&  0.39242E+00_JPRB, 0.39209E+00_JPRB, 0.39110E+00_JPRB, 0.39075E+00_JPRB, 0.39118E+00_JPRB, 0.39178E+00_JPRB, 0.39154E+00_JPRB ,&
&  0.38989E+00_JPRB, 0.38672E+00_JPRB, 0.38358E+00_JPRB, 0.38333E+00_JPRB, 0.38502E+00_JPRB, 0.38908E+00_JPRB, 0.39630E+00_JPRB ,&
&  0.40667E+00_JPRB, 0.41884E+00_JPRB, 0.43187E+00_JPRB, 0.44600E+00_JPRB, 0.46308E+00_JPRB, 0.47914E+00_JPRB, 0.49143E+00_JPRB ,&
&  0.49338E+00_JPRB, 0.48862E+00_JPRB, 0.47819E+00_JPRB, 0.46537E+00_JPRB, 0.45493E+00_JPRB, 0.44478E+00_JPRB, 0.43534E+00_JPRB ,&
&  0.42837E+00_JPRB, 0.42252E+00_JPRB, 0.41904E+00_JPRB, 0.41845E+00_JPRB, 0.41857E+00_JPRB, 0.41749E+00_JPRB, 0.41673E+00_JPRB ,&
&  0.41604E+00_JPRB, 0.41535E+00_JPRB, 0.41494E+00_JPRB, 0.41439E+00_JPRB, 0.41295E+00_JPRB, 0.41166E+00_JPRB, 0.41281E+00_JPRB ,&
&  0.41764E+00_JPRB, 0.42819E+00_JPRB, 0.44626E+00_JPRB, 0.47362E+00_JPRB, 0.49927E+00_JPRB, 0.51834E+00_JPRB, 0.52819E+00_JPRB ,&
&  0.52876E+00_JPRB, 0.52535E+00_JPRB, 0.51735E+00_JPRB, 0.51080E+00_JPRB, 0.50552E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.51862E+00_JPRB, 0.51033E+00_JPRB, 0.50185E+00_JPRB, 0.49279E+00_JPRB, 0.48279E+00_JPRB, 0.47191E+00_JPRB, 0.46060E+00_JPRB ,&
&  0.45133E+00_JPRB, 0.44379E+00_JPRB, 0.43668E+00_JPRB, 0.43063E+00_JPRB, 0.42497E+00_JPRB, 0.42243E+00_JPRB, 0.42172E+00_JPRB ,&
&  0.42205E+00_JPRB, 0.42141E+00_JPRB, 0.42005E+00_JPRB, 0.42017E+00_JPRB, 0.41975E+00_JPRB, 0.41733E+00_JPRB, 0.41396E+00_JPRB ,&
&  0.40850E+00_JPRB, 0.40382E+00_JPRB, 0.40219E+00_JPRB, 0.40462E+00_JPRB, 0.40377E+00_JPRB, 0.40579E+00_JPRB, 0.41372E+00_JPRB ,&
&  0.39843E+00_JPRB, 0.37256E+00_JPRB, 0.40508E+00_JPRB, 0.44137E+00_JPRB, 0.42830E+00_JPRB, 0.41499E+00_JPRB, 0.41096E+00_JPRB ,&
&  0.40719E+00_JPRB, 0.40627E+00_JPRB, 0.40490E+00_JPRB, 0.40415E+00_JPRB, 0.40486E+00_JPRB, 0.40359E+00_JPRB, 0.40080E+00_JPRB ,&
&  0.39878E+00_JPRB, 0.40040E+00_JPRB, 0.40231E+00_JPRB, 0.40328E+00_JPRB, 0.40263E+00_JPRB, 0.40105E+00_JPRB, 0.39731E+00_JPRB ,&
&  0.39391E+00_JPRB, 0.39202E+00_JPRB, 0.39229E+00_JPRB, 0.39614E+00_JPRB, 0.40265E+00_JPRB, 0.41315E+00_JPRB, 0.42942E+00_JPRB ,&
&  0.45007E+00_JPRB, 0.47059E+00_JPRB, 0.48894E+00_JPRB, 0.50669E+00_JPRB, 0.52524E+00_JPRB, 0.54089E+00_JPRB, 0.54862E+00_JPRB ,&
&  0.54137E+00_JPRB, 0.52457E+00_JPRB, 0.50508E+00_JPRB, 0.48996E+00_JPRB, 0.47941E+00_JPRB, 0.47294E+00_JPRB, 0.46951E+00_JPRB ,&
&  0.46916E+00_JPRB, 0.46925E+00_JPRB, 0.46913E+00_JPRB, 0.46969E+00_JPRB, 0.47030E+00_JPRB, 0.46897E+00_JPRB, 0.46681E+00_JPRB ,&
&  0.46319E+00_JPRB, 0.45730E+00_JPRB, 0.45017E+00_JPRB, 0.44392E+00_JPRB, 0.43952E+00_JPRB, 0.43875E+00_JPRB, 0.44126E+00_JPRB ,&
&  0.44930E+00_JPRB, 0.46607E+00_JPRB, 0.48543E+00_JPRB, 0.52874E+00_JPRB, 0.55911E+00_JPRB, 0.57926E+00_JPRB, 0.58722E+00_JPRB ,&
&  0.58158E+00_JPRB, 0.56158E+00_JPRB, 0.54362E+00_JPRB, 0.53312E+00_JPRB, 0.52917E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.52877E+00_JPRB, 0.52528E+00_JPRB, 0.51799E+00_JPRB, 0.50974E+00_JPRB, 0.50182E+00_JPRB, 0.49170E+00_JPRB, 0.47913E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.45779E+00_JPRB, 0.45129E+00_JPRB, 0.44515E+00_JPRB, 0.43632E+00_JPRB, 0.43105E+00_JPRB, 0.43045E+00_JPRB ,&
&  0.42886E+00_JPRB, 0.42356E+00_JPRB, 0.41745E+00_JPRB, 0.41647E+00_JPRB, 0.41458E+00_JPRB, 0.41205E+00_JPRB, 0.40841E+00_JPRB ,&
&  0.40464E+00_JPRB, 0.40366E+00_JPRB, 0.40443E+00_JPRB, 0.40562E+00_JPRB, 0.40257E+00_JPRB, 0.40391E+00_JPRB, 0.41513E+00_JPRB ,&
&  0.45992E+00_JPRB, 0.39642E+00_JPRB, 0.43951E+00_JPRB, 0.48742E+00_JPRB, 0.46956E+00_JPRB, 0.45567E+00_JPRB, 0.44492E+00_JPRB ,&
&  0.43733E+00_JPRB, 0.43456E+00_JPRB, 0.43214E+00_JPRB, 0.42714E+00_JPRB, 0.42195E+00_JPRB, 0.41582E+00_JPRB, 0.41263E+00_JPRB ,&
&  0.41160E+00_JPRB, 0.41506E+00_JPRB, 0.41679E+00_JPRB, 0.41622E+00_JPRB, 0.41281E+00_JPRB, 0.40949E+00_JPRB, 0.40703E+00_JPRB ,&
&  0.40640E+00_JPRB, 0.40826E+00_JPRB, 0.41127E+00_JPRB, 0.41590E+00_JPRB, 0.42381E+00_JPRB, 0.43964E+00_JPRB, 0.46468E+00_JPRB ,&
&  0.49593E+00_JPRB, 0.52334E+00_JPRB, 0.54624E+00_JPRB, 0.56889E+00_JPRB, 0.58683E+00_JPRB, 0.59607E+00_JPRB, 0.59330E+00_JPRB ,&
&  0.58114E+00_JPRB, 0.55899E+00_JPRB, 0.54232E+00_JPRB, 0.52773E+00_JPRB, 0.51345E+00_JPRB, 0.50240E+00_JPRB, 0.49539E+00_JPRB ,&
&  0.49160E+00_JPRB, 0.48930E+00_JPRB, 0.48743E+00_JPRB, 0.48474E+00_JPRB, 0.48440E+00_JPRB, 0.48470E+00_JPRB, 0.48683E+00_JPRB ,&
&  0.48679E+00_JPRB, 0.48538E+00_JPRB, 0.47907E+00_JPRB, 0.46978E+00_JPRB, 0.46546E+00_JPRB, 0.46513E+00_JPRB, 0.46846E+00_JPRB ,&
&  0.47739E+00_JPRB, 0.49449E+00_JPRB, 0.51307E+00_JPRB, 0.57246E+00_JPRB, 0.60445E+00_JPRB, 0.61940E+00_JPRB, 0.62003E+00_JPRB ,&
&  0.60326E+00_JPRB, 0.57441E+00_JPRB, 0.45969E+00_JPRB, 0.52120E+00_JPRB, 0.53561E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.57389E+00_JPRB, 0.56275E+00_JPRB, 0.54790E+00_JPRB, 0.52811E+00_JPRB, 0.51367E+00_JPRB, 0.50227E+00_JPRB, 0.49033E+00_JPRB ,&
&  0.47914E+00_JPRB, 0.46969E+00_JPRB, 0.46163E+00_JPRB, 0.45255E+00_JPRB, 0.44272E+00_JPRB, 0.43996E+00_JPRB, 0.43922E+00_JPRB ,&
&  0.43565E+00_JPRB, 0.42803E+00_JPRB, 0.42111E+00_JPRB, 0.41970E+00_JPRB, 0.41918E+00_JPRB, 0.41891E+00_JPRB, 0.41590E+00_JPRB ,&
&  0.41093E+00_JPRB, 0.41033E+00_JPRB, 0.41379E+00_JPRB, 0.41461E+00_JPRB, 0.41068E+00_JPRB, 0.41216E+00_JPRB, 0.42326E+00_JPRB ,&
&  0.47783E+00_JPRB, 0.43160E+00_JPRB, 0.47643E+00_JPRB, 0.56201E+00_JPRB, 0.55125E+00_JPRB, 0.54855E+00_JPRB, 0.52896E+00_JPRB ,&
&  0.50673E+00_JPRB, 0.49445E+00_JPRB, 0.48296E+00_JPRB, 0.46578E+00_JPRB, 0.44861E+00_JPRB, 0.43872E+00_JPRB, 0.43668E+00_JPRB ,&
&  0.43931E+00_JPRB, 0.44077E+00_JPRB, 0.44131E+00_JPRB, 0.43717E+00_JPRB, 0.43475E+00_JPRB, 0.43328E+00_JPRB, 0.43311E+00_JPRB ,&
&  0.43356E+00_JPRB, 0.43751E+00_JPRB, 0.44322E+00_JPRB, 0.44863E+00_JPRB, 0.45951E+00_JPRB, 0.48835E+00_JPRB, 0.52906E+00_JPRB ,&
&  0.56775E+00_JPRB, 0.60057E+00_JPRB, 0.62406E+00_JPRB, 0.64295E+00_JPRB, 0.65146E+00_JPRB, 0.65729E+00_JPRB, 0.65576E+00_JPRB ,&
&  0.63844E+00_JPRB, 0.61444E+00_JPRB, 0.59354E+00_JPRB, 0.57082E+00_JPRB, 0.54983E+00_JPRB, 0.53347E+00_JPRB, 0.52033E+00_JPRB ,&
&  0.51017E+00_JPRB, 0.50268E+00_JPRB, 0.49655E+00_JPRB, 0.48862E+00_JPRB, 0.48370E+00_JPRB, 0.48420E+00_JPRB, 0.48695E+00_JPRB ,&
&  0.49141E+00_JPRB, 0.49735E+00_JPRB, 0.50183E+00_JPRB, 0.50408E+00_JPRB, 0.50546E+00_JPRB, 0.50862E+00_JPRB, 0.51303E+00_JPRB ,&
&  0.52559E+00_JPRB, 0.54968E+00_JPRB, 0.57276E+00_JPRB, 0.64659E+00_JPRB, 0.68924E+00_JPRB, 0.68211E+00_JPRB, 0.65456E+00_JPRB ,&
&  0.61903E+00_JPRB, 0.58169E+00_JPRB, 0.56125E+00_JPRB, 0.50410E+00_JPRB, 0.58199E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.63353E+00_JPRB, 0.60814E+00_JPRB, 0.57065E+00_JPRB, 0.53507E+00_JPRB, 0.51049E+00_JPRB, 0.49521E+00_JPRB, 0.48617E+00_JPRB ,&
&  0.47906E+00_JPRB, 0.47162E+00_JPRB, 0.46360E+00_JPRB, 0.45318E+00_JPRB, 0.44676E+00_JPRB, 0.44385E+00_JPRB, 0.44179E+00_JPRB ,&
&  0.43777E+00_JPRB, 0.42878E+00_JPRB, 0.42604E+00_JPRB, 0.42357E+00_JPRB, 0.42242E+00_JPRB, 0.42088E+00_JPRB, 0.41607E+00_JPRB ,&
&  0.41005E+00_JPRB, 0.40716E+00_JPRB, 0.41064E+00_JPRB, 0.41261E+00_JPRB, 0.41475E+00_JPRB, 0.41995E+00_JPRB, 0.43490E+00_JPRB ,&
&  0.49875E+00_JPRB, 0.51582E+00_JPRB, 0.58786E+00_JPRB, 0.65327E+00_JPRB, 0.64124E+00_JPRB, 0.62090E+00_JPRB, 0.60539E+00_JPRB ,&
&  0.58480E+00_JPRB, 0.56844E+00_JPRB, 0.54025E+00_JPRB, 0.50558E+00_JPRB, 0.47739E+00_JPRB, 0.46892E+00_JPRB, 0.46207E+00_JPRB ,&
&  0.45664E+00_JPRB, 0.45430E+00_JPRB, 0.45369E+00_JPRB, 0.45409E+00_JPRB, 0.45497E+00_JPRB, 0.45466E+00_JPRB, 0.45457E+00_JPRB ,&
&  0.45688E+00_JPRB, 0.46293E+00_JPRB, 0.47073E+00_JPRB, 0.48407E+00_JPRB, 0.51328E+00_JPRB, 0.55977E+00_JPRB, 0.60722E+00_JPRB ,&
&  0.64109E+00_JPRB, 0.66683E+00_JPRB, 0.70101E+00_JPRB, 0.72671E+00_JPRB, 0.73919E+00_JPRB, 0.73921E+00_JPRB, 0.72509E+00_JPRB ,&
&  0.70696E+00_JPRB, 0.68878E+00_JPRB, 0.65112E+00_JPRB, 0.61118E+00_JPRB, 0.58656E+00_JPRB, 0.57191E+00_JPRB, 0.56191E+00_JPRB ,&
&  0.55437E+00_JPRB, 0.54353E+00_JPRB, 0.53061E+00_JPRB, 0.51945E+00_JPRB, 0.51313E+00_JPRB, 0.50699E+00_JPRB, 0.50115E+00_JPRB ,&
&  0.50232E+00_JPRB, 0.51073E+00_JPRB, 0.52291E+00_JPRB, 0.53108E+00_JPRB, 0.53831E+00_JPRB, 0.54764E+00_JPRB, 0.55422E+00_JPRB ,&
&  0.56373E+00_JPRB, 0.59134E+00_JPRB, 0.61304E+00_JPRB, 0.68509E+00_JPRB, 0.69566E+00_JPRB, 0.68361E+00_JPRB, 0.64982E+00_JPRB ,&
&  0.60858E+00_JPRB, 0.57496E+00_JPRB, 0.55023E+00_JPRB, 0.56908E+00_JPRB, 0.62953E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.67839E+00_JPRB, 0.62110E+00_JPRB, 0.57355E+00_JPRB, 0.53398E+00_JPRB, 0.50881E+00_JPRB, 0.49393E+00_JPRB, 0.48382E+00_JPRB ,&
&  0.47878E+00_JPRB, 0.47509E+00_JPRB, 0.46780E+00_JPRB, 0.46272E+00_JPRB, 0.45889E+00_JPRB, 0.45713E+00_JPRB, 0.45566E+00_JPRB ,&
&  0.44718E+00_JPRB, 0.44551E+00_JPRB, 0.44549E+00_JPRB, 0.44110E+00_JPRB, 0.43515E+00_JPRB, 0.42887E+00_JPRB, 0.42075E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.41356E+00_JPRB, 0.41554E+00_JPRB, 0.41944E+00_JPRB, 0.42346E+00_JPRB, 0.43464E+00_JPRB, 0.45759E+00_JPRB ,&
&  0.54410E+00_JPRB, 0.60807E+00_JPRB, 0.71238E+00_JPRB, 0.77738E+00_JPRB, 0.71316E+00_JPRB, 0.68077E+00_JPRB, 0.66005E+00_JPRB ,&
&  0.65932E+00_JPRB, 0.66089E+00_JPRB, 0.63104E+00_JPRB, 0.58693E+00_JPRB, 0.54675E+00_JPRB, 0.52419E+00_JPRB, 0.51023E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.48948E+00_JPRB, 0.48238E+00_JPRB, 0.48452E+00_JPRB, 0.48651E+00_JPRB, 0.48483E+00_JPRB, 0.48392E+00_JPRB ,&
&  0.48591E+00_JPRB, 0.49380E+00_JPRB, 0.50819E+00_JPRB, 0.54461E+00_JPRB, 0.61486E+00_JPRB, 0.68373E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.78078E+00_JPRB, 0.81584E+00_JPRB, 0.85489E+00_JPRB, 0.87979E+00_JPRB, 0.88066E+00_JPRB, 0.87029E+00_JPRB, 0.85845E+00_JPRB ,&
&  0.83699E+00_JPRB, 0.78374E+00_JPRB, 0.71351E+00_JPRB, 0.66212E+00_JPRB, 0.63438E+00_JPRB, 0.62008E+00_JPRB, 0.61209E+00_JPRB ,&
&  0.60492E+00_JPRB, 0.59762E+00_JPRB, 0.58670E+00_JPRB, 0.57167E+00_JPRB, 0.55390E+00_JPRB, 0.54189E+00_JPRB, 0.53251E+00_JPRB ,&
&  0.52698E+00_JPRB, 0.52775E+00_JPRB, 0.53689E+00_JPRB, 0.55281E+00_JPRB, 0.56583E+00_JPRB, 0.57717E+00_JPRB, 0.58587E+00_JPRB ,&
&  0.59872E+00_JPRB, 0.64097E+00_JPRB, 0.63479E+00_JPRB, 0.70828E+00_JPRB, 0.77973E+00_JPRB, 0.73460E+00_JPRB, 0.67394E+00_JPRB ,&
&  0.61514E+00_JPRB, 0.57622E+00_JPRB, 0.56419E+00_JPRB, 0.59284E+00_JPRB, 0.71229E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.58051E+00_JPRB, 0.55838E+00_JPRB, 0.53839E+00_JPRB, 0.51664E+00_JPRB, 0.49717E+00_JPRB, 0.48746E+00_JPRB, 0.48482E+00_JPRB ,&
&  0.48616E+00_JPRB, 0.48603E+00_JPRB, 0.48345E+00_JPRB, 0.47721E+00_JPRB, 0.47269E+00_JPRB, 0.46859E+00_JPRB, 0.46511E+00_JPRB ,&
&  0.45826E+00_JPRB, 0.45854E+00_JPRB, 0.45473E+00_JPRB, 0.44833E+00_JPRB, 0.44123E+00_JPRB, 0.43286E+00_JPRB, 0.42473E+00_JPRB ,&
&  0.41980E+00_JPRB, 0.41834E+00_JPRB, 0.41926E+00_JPRB, 0.42358E+00_JPRB, 0.43556E+00_JPRB, 0.44780E+00_JPRB, 0.46408E+00_JPRB ,&
&  0.56213E+00_JPRB, 0.70707E+00_JPRB, 0.88341E+00_JPRB, 0.10190E+01_JPRB, 0.95449E+00_JPRB, 0.84893E+00_JPRB, 0.78414E+00_JPRB ,&
&  0.75877E+00_JPRB, 0.74223E+00_JPRB, 0.71119E+00_JPRB, 0.67630E+00_JPRB, 0.63598E+00_JPRB, 0.61650E+00_JPRB, 0.59602E+00_JPRB ,&
&  0.57437E+00_JPRB, 0.55605E+00_JPRB, 0.53808E+00_JPRB, 0.53094E+00_JPRB, 0.52503E+00_JPRB, 0.51726E+00_JPRB, 0.51340E+00_JPRB ,&
&  0.51915E+00_JPRB, 0.53586E+00_JPRB, 0.56869E+00_JPRB, 0.63964E+00_JPRB, 0.76220E+00_JPRB, 0.86970E+00_JPRB, 0.98966E+00_JPRB ,&
&  0.10451E+01_JPRB, 0.10726E+01_JPRB, 0.10960E+01_JPRB, 0.10849E+01_JPRB, 0.10255E+01_JPRB, 0.98262E+00_JPRB, 0.97419E+00_JPRB ,&
&  0.93423E+00_JPRB, 0.83739E+00_JPRB, 0.74037E+00_JPRB, 0.67992E+00_JPRB, 0.64828E+00_JPRB, 0.63544E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.62241E+00_JPRB, 0.61746E+00_JPRB, 0.60358E+00_JPRB, 0.58453E+00_JPRB, 0.56640E+00_JPRB, 0.55419E+00_JPRB, 0.54763E+00_JPRB ,&
&  0.54116E+00_JPRB, 0.54014E+00_JPRB, 0.55596E+00_JPRB, 0.56635E+00_JPRB, 0.57289E+00_JPRB, 0.58272E+00_JPRB, 0.59821E+00_JPRB ,&
&  0.61929E+00_JPRB, 0.66370E+00_JPRB, 0.65711E+00_JPRB, 0.69546E+00_JPRB, 0.74085E+00_JPRB, 0.73926E+00_JPRB, 0.67751E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.58572E+00_JPRB, 0.58784E+00_JPRB, 0.59822E+00_JPRB, 0.60335E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.55208E+00_JPRB, 0.53545E+00_JPRB, 0.52192E+00_JPRB, 0.49944E+00_JPRB, 0.48535E+00_JPRB, 0.47820E+00_JPRB, 0.48039E+00_JPRB ,&
&  0.48595E+00_JPRB, 0.49251E+00_JPRB, 0.48995E+00_JPRB, 0.48679E+00_JPRB, 0.48077E+00_JPRB, 0.47653E+00_JPRB, 0.46878E+00_JPRB ,&
&  0.46431E+00_JPRB, 0.45983E+00_JPRB, 0.45320E+00_JPRB, 0.44547E+00_JPRB, 0.43467E+00_JPRB, 0.42982E+00_JPRB, 0.42393E+00_JPRB ,&
&  0.41849E+00_JPRB, 0.41642E+00_JPRB, 0.41844E+00_JPRB, 0.42718E+00_JPRB, 0.43835E+00_JPRB, 0.45213E+00_JPRB, 0.47667E+00_JPRB ,&
&  0.60470E+00_JPRB, 0.69670E+00_JPRB, 0.11396E+01_JPRB, 0.12060E+01_JPRB, 0.11157E+01_JPRB, 0.99336E+00_JPRB, 0.97942E+00_JPRB ,&
&  0.95662E+00_JPRB, 0.89873E+00_JPRB, 0.84153E+00_JPRB, 0.79863E+00_JPRB, 0.75577E+00_JPRB, 0.72730E+00_JPRB, 0.68933E+00_JPRB ,&
&  0.65642E+00_JPRB, 0.62172E+00_JPRB, 0.58189E+00_JPRB, 0.55369E+00_JPRB, 0.53476E+00_JPRB, 0.52755E+00_JPRB, 0.53566E+00_JPRB ,&
&  0.55341E+00_JPRB, 0.58558E+00_JPRB, 0.65970E+00_JPRB, 0.77376E+00_JPRB, 0.90570E+00_JPRB, 0.93184E+00_JPRB, 0.10986E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.13905E+01_JPRB, 0.12941E+01_JPRB, 0.12005E+01_JPRB, 0.10662E+01_JPRB, 0.10095E+01_JPRB, 0.98912E+00_JPRB ,&
&  0.93546E+00_JPRB, 0.85564E+00_JPRB, 0.77324E+00_JPRB, 0.71223E+00_JPRB, 0.66992E+00_JPRB, 0.64459E+00_JPRB, 0.63573E+00_JPRB ,&
&  0.62562E+00_JPRB, 0.60707E+00_JPRB, 0.58251E+00_JPRB, 0.56222E+00_JPRB, 0.55126E+00_JPRB, 0.54814E+00_JPRB, 0.54826E+00_JPRB ,&
&  0.55058E+00_JPRB, 0.55676E+00_JPRB, 0.58358E+00_JPRB, 0.56436E+00_JPRB, 0.57091E+00_JPRB, 0.59131E+00_JPRB, 0.60291E+00_JPRB ,&
&  0.60192E+00_JPRB, 0.62374E+00_JPRB, 0.64485E+00_JPRB, 0.67645E+00_JPRB, 0.74029E+00_JPRB, 0.76553E+00_JPRB, 0.69548E+00_JPRB ,&
&  0.62306E+00_JPRB, 0.59476E+00_JPRB, 0.59652E+00_JPRB, 0.59412E+00_JPRB, 0.58461E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.52906E+00_JPRB, 0.51238E+00_JPRB, 0.50191E+00_JPRB, 0.48728E+00_JPRB, 0.47433E+00_JPRB, 0.46781E+00_JPRB, 0.47086E+00_JPRB ,&
&  0.48431E+00_JPRB, 0.49676E+00_JPRB, 0.49225E+00_JPRB, 0.49025E+00_JPRB, 0.48871E+00_JPRB, 0.48152E+00_JPRB, 0.47094E+00_JPRB ,&
&  0.46031E+00_JPRB, 0.45024E+00_JPRB, 0.43835E+00_JPRB, 0.42562E+00_JPRB, 0.41687E+00_JPRB, 0.41727E+00_JPRB, 0.41562E+00_JPRB ,&
&  0.41381E+00_JPRB, 0.41759E+00_JPRB, 0.42230E+00_JPRB, 0.43072E+00_JPRB, 0.43883E+00_JPRB, 0.45258E+00_JPRB, 0.48560E+00_JPRB ,&
&  0.60200E+00_JPRB, 0.48176E+00_JPRB, 0.10877E+01_JPRB, 0.13965E+01_JPRB, 0.14485E+01_JPRB, 0.13438E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.12191E+01_JPRB, 0.10908E+01_JPRB, 0.98596E+00_JPRB, 0.89306E+00_JPRB, 0.78349E+00_JPRB, 0.71679E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.65237E+00_JPRB, 0.62636E+00_JPRB, 0.60320E+00_JPRB, 0.58108E+00_JPRB, 0.56262E+00_JPRB, 0.55976E+00_JPRB, 0.57617E+00_JPRB ,&
&  0.60837E+00_JPRB, 0.67484E+00_JPRB, 0.79914E+00_JPRB, 0.95596E+00_JPRB, 0.10389E+01_JPRB, 0.11075E+01_JPRB, 0.12165E+01_JPRB ,&
&  0.14038E+01_JPRB, 0.15561E+01_JPRB, 0.13327E+01_JPRB, 0.11128E+01_JPRB, 0.97037E+00_JPRB, 0.92775E+00_JPRB, 0.91705E+00_JPRB ,&
&  0.87509E+00_JPRB, 0.82881E+00_JPRB, 0.77787E+00_JPRB, 0.72485E+00_JPRB, 0.67990E+00_JPRB, 0.65120E+00_JPRB, 0.62993E+00_JPRB ,&
&  0.60615E+00_JPRB, 0.58007E+00_JPRB, 0.56286E+00_JPRB, 0.55315E+00_JPRB, 0.54915E+00_JPRB, 0.54829E+00_JPRB, 0.55315E+00_JPRB ,&
&  0.56591E+00_JPRB, 0.57489E+00_JPRB, 0.57621E+00_JPRB, 0.55700E+00_JPRB, 0.55137E+00_JPRB, 0.55825E+00_JPRB, 0.56822E+00_JPRB ,&
&  0.60640E+00_JPRB, 0.62256E+00_JPRB, 0.63657E+00_JPRB, 0.67264E+00_JPRB, 0.66719E+00_JPRB, 0.72356E+00_JPRB, 0.67075E+00_JPRB ,&
&  0.61950E+00_JPRB, 0.59498E+00_JPRB, 0.58759E+00_JPRB, 0.57715E+00_JPRB, 0.55558E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.50160E+00_JPRB, 0.48918E+00_JPRB, 0.47911E+00_JPRB, 0.47452E+00_JPRB, 0.46675E+00_JPRB, 0.46396E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.48089E+00_JPRB, 0.48486E+00_JPRB, 0.48358E+00_JPRB, 0.48123E+00_JPRB, 0.47545E+00_JPRB, 0.46732E+00_JPRB, 0.45707E+00_JPRB ,&
&  0.44813E+00_JPRB, 0.43471E+00_JPRB, 0.41744E+00_JPRB, 0.40493E+00_JPRB, 0.40876E+00_JPRB, 0.41376E+00_JPRB, 0.41384E+00_JPRB ,&
&  0.41293E+00_JPRB, 0.41912E+00_JPRB, 0.42185E+00_JPRB, 0.43193E+00_JPRB, 0.43935E+00_JPRB, 0.45754E+00_JPRB, 0.49451E+00_JPRB ,&
&  0.59612E+00_JPRB, 0.92114E+00_JPRB, 0.92772E+00_JPRB, 0.15852E+01_JPRB, 0.17739E+01_JPRB, 0.16875E+01_JPRB, 0.14444E+01_JPRB ,&
&  0.13309E+01_JPRB, 0.12471E+01_JPRB, 0.10705E+01_JPRB, 0.92404E+00_JPRB, 0.82970E+00_JPRB, 0.76037E+00_JPRB, 0.69979E+00_JPRB ,&
&  0.65214E+00_JPRB, 0.62125E+00_JPRB, 0.60103E+00_JPRB, 0.58491E+00_JPRB, 0.58266E+00_JPRB, 0.59645E+00_JPRB, 0.62372E+00_JPRB ,&
&  0.68049E+00_JPRB, 0.78702E+00_JPRB, 0.96369E+00_JPRB, 0.11283E+01_JPRB, 0.11592E+01_JPRB, 0.12476E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.15331E+01_JPRB, 0.16014E+01_JPRB, 0.12291E+01_JPRB, 0.98887E+00_JPRB, 0.89833E+00_JPRB, 0.89687E+00_JPRB, 0.88211E+00_JPRB ,&
&  0.83578E+00_JPRB, 0.80100E+00_JPRB, 0.75523E+00_JPRB, 0.70808E+00_JPRB, 0.67262E+00_JPRB, 0.64531E+00_JPRB, 0.61495E+00_JPRB ,&
&  0.59154E+00_JPRB, 0.58166E+00_JPRB, 0.57602E+00_JPRB, 0.57128E+00_JPRB, 0.57126E+00_JPRB, 0.57636E+00_JPRB, 0.58279E+00_JPRB ,&
&  0.59075E+00_JPRB, 0.58684E+00_JPRB, 0.56448E+00_JPRB, 0.55089E+00_JPRB, 0.55558E+00_JPRB, 0.56605E+00_JPRB, 0.56835E+00_JPRB ,&
&  0.60811E+00_JPRB, 0.63228E+00_JPRB, 0.63952E+00_JPRB, 0.66720E+00_JPRB, 0.69485E+00_JPRB, 0.81310E+00_JPRB, 0.68558E+00_JPRB ,&
&  0.62552E+00_JPRB, 0.59142E+00_JPRB, 0.57431E+00_JPRB, 0.56162E+00_JPRB, 0.53331E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.47576E+00_JPRB, 0.46063E+00_JPRB, 0.45466E+00_JPRB, 0.45571E+00_JPRB, 0.45625E+00_JPRB, 0.45766E+00_JPRB, 0.46522E+00_JPRB ,&
&  0.46854E+00_JPRB, 0.46906E+00_JPRB, 0.46731E+00_JPRB, 0.46456E+00_JPRB, 0.45695E+00_JPRB, 0.44824E+00_JPRB, 0.43892E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.41478E+00_JPRB, 0.40590E+00_JPRB, 0.41376E+00_JPRB, 0.41767E+00_JPRB, 0.41704E+00_JPRB, 0.41785E+00_JPRB ,&
&  0.41821E+00_JPRB, 0.42246E+00_JPRB, 0.42300E+00_JPRB, 0.42938E+00_JPRB, 0.44147E+00_JPRB, 0.47097E+00_JPRB, 0.51244E+00_JPRB ,&
&  0.64198E+00_JPRB, 0.85219E+00_JPRB, 0.89919E+00_JPRB, 0.17012E+01_JPRB, 0.18669E+01_JPRB, 0.17839E+01_JPRB, 0.15405E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.14069E+01_JPRB, 0.11790E+01_JPRB, 0.10131E+01_JPRB, 0.91525E+00_JPRB, 0.82837E+00_JPRB, 0.73090E+00_JPRB ,&
&  0.65348E+00_JPRB, 0.61185E+00_JPRB, 0.59721E+00_JPRB, 0.60535E+00_JPRB, 0.62681E+00_JPRB, 0.65062E+00_JPRB, 0.68889E+00_JPRB ,&
&  0.78617E+00_JPRB, 0.94740E+00_JPRB, 0.11629E+01_JPRB, 0.12111E+01_JPRB, 0.12252E+01_JPRB, 0.12817E+01_JPRB, 0.13509E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.13357E+01_JPRB, 0.10273E+01_JPRB, 0.89282E+00_JPRB, 0.84883E+00_JPRB, 0.85971E+00_JPRB, 0.83890E+00_JPRB ,&
&  0.79391E+00_JPRB, 0.75419E+00_JPRB, 0.70814E+00_JPRB, 0.67602E+00_JPRB, 0.65343E+00_JPRB, 0.62541E+00_JPRB, 0.60727E+00_JPRB ,&
&  0.59934E+00_JPRB, 0.59579E+00_JPRB, 0.59833E+00_JPRB, 0.59847E+00_JPRB, 0.60142E+00_JPRB, 0.60344E+00_JPRB, 0.60318E+00_JPRB ,&
&  0.59626E+00_JPRB, 0.57894E+00_JPRB, 0.56509E+00_JPRB, 0.57506E+00_JPRB, 0.60077E+00_JPRB, 0.60801E+00_JPRB, 0.60628E+00_JPRB ,&
&  0.63161E+00_JPRB, 0.65251E+00_JPRB, 0.66218E+00_JPRB, 0.66179E+00_JPRB, 0.69220E+00_JPRB, 0.74355E+00_JPRB, 0.65000E+00_JPRB ,&
&  0.61309E+00_JPRB, 0.58301E+00_JPRB, 0.56028E+00_JPRB, 0.54422E+00_JPRB, 0.51100E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.47487E+00_JPRB, 0.45170E+00_JPRB, 0.44398E+00_JPRB, 0.44031E+00_JPRB, 0.44118E+00_JPRB, 0.44714E+00_JPRB, 0.45330E+00_JPRB ,&
&  0.45295E+00_JPRB, 0.45224E+00_JPRB, 0.45240E+00_JPRB, 0.44851E+00_JPRB, 0.43756E+00_JPRB, 0.42689E+00_JPRB, 0.41732E+00_JPRB ,&
&  0.41589E+00_JPRB, 0.41814E+00_JPRB, 0.43068E+00_JPRB, 0.43842E+00_JPRB, 0.43948E+00_JPRB, 0.43708E+00_JPRB, 0.43793E+00_JPRB ,&
&  0.44172E+00_JPRB, 0.44112E+00_JPRB, 0.43935E+00_JPRB, 0.45026E+00_JPRB, 0.47164E+00_JPRB, 0.51335E+00_JPRB, 0.58174E+00_JPRB ,&
&  0.77023E+00_JPRB, 0.90779E+00_JPRB, 0.95894E+00_JPRB, 0.14636E+01_JPRB, 0.17533E+01_JPRB, 0.16718E+01_JPRB, 0.15594E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.12617E+01_JPRB, 0.12455E+01_JPRB, 0.10707E+01_JPRB, 0.94163E+00_JPRB, 0.84311E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.70095E+00_JPRB, 0.67085E+00_JPRB, 0.68253E+00_JPRB, 0.70329E+00_JPRB, 0.71956E+00_JPRB, 0.74503E+00_JPRB, 0.82405E+00_JPRB ,&
&  0.95511E+00_JPRB, 0.11493E+01_JPRB, 0.13716E+01_JPRB, 0.13712E+01_JPRB, 0.12030E+01_JPRB, 0.11974E+01_JPRB, 0.12001E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.10829E+01_JPRB, 0.91779E+00_JPRB, 0.85299E+00_JPRB, 0.79509E+00_JPRB, 0.82271E+00_JPRB, 0.79280E+00_JPRB ,&
&  0.75557E+00_JPRB, 0.71044E+00_JPRB, 0.66940E+00_JPRB, 0.64399E+00_JPRB, 0.62398E+00_JPRB, 0.60897E+00_JPRB, 0.59823E+00_JPRB ,&
&  0.59330E+00_JPRB, 0.59259E+00_JPRB, 0.60259E+00_JPRB, 0.60837E+00_JPRB, 0.60778E+00_JPRB, 0.59601E+00_JPRB, 0.57923E+00_JPRB ,&
&  0.56964E+00_JPRB, 0.58297E+00_JPRB, 0.59489E+00_JPRB, 0.62043E+00_JPRB, 0.64572E+00_JPRB, 0.65764E+00_JPRB, 0.64247E+00_JPRB ,&
&  0.63967E+00_JPRB, 0.66554E+00_JPRB, 0.68051E+00_JPRB, 0.69689E+00_JPRB, 0.69897E+00_JPRB, 0.68897E+00_JPRB, 0.62563E+00_JPRB ,&
&  0.59385E+00_JPRB, 0.57223E+00_JPRB, 0.55307E+00_JPRB, 0.53664E+00_JPRB, 0.50138E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.48009E+00_JPRB, 0.46742E+00_JPRB, 0.42929E+00_JPRB, 0.43203E+00_JPRB, 0.43333E+00_JPRB, 0.43913E+00_JPRB, 0.43650E+00_JPRB ,&
&  0.43404E+00_JPRB, 0.43481E+00_JPRB, 0.43647E+00_JPRB, 0.42868E+00_JPRB, 0.40707E+00_JPRB, 0.39779E+00_JPRB, 0.40116E+00_JPRB ,&
&  0.41209E+00_JPRB, 0.43550E+00_JPRB, 0.46424E+00_JPRB, 0.46944E+00_JPRB, 0.47100E+00_JPRB, 0.46768E+00_JPRB, 0.46437E+00_JPRB ,&
&  0.46736E+00_JPRB, 0.47386E+00_JPRB, 0.48377E+00_JPRB, 0.50745E+00_JPRB, 0.54959E+00_JPRB, 0.61459E+00_JPRB, 0.72170E+00_JPRB ,&
&  0.98677E+00_JPRB, 0.10235E+01_JPRB, 0.53241E+00_JPRB, 0.96413E+00_JPRB, 0.13982E+01_JPRB, 0.14612E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.11443E+01_JPRB, 0.10875E+01_JPRB, 0.10358E+01_JPRB, 0.92875E+00_JPRB, 0.87050E+00_JPRB, 0.81732E+00_JPRB ,&
&  0.77709E+00_JPRB, 0.78145E+00_JPRB, 0.82149E+00_JPRB, 0.82528E+00_JPRB, 0.84173E+00_JPRB, 0.92136E+00_JPRB, 0.10484E+01_JPRB ,&
&  0.12017E+01_JPRB, 0.14147E+01_JPRB, 0.15936E+01_JPRB, 0.13719E+01_JPRB, 0.12281E+01_JPRB, 0.11316E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.97390E+00_JPRB, 0.93049E+00_JPRB, 0.86433E+00_JPRB, 0.82266E+00_JPRB, 0.76980E+00_JPRB, 0.74250E+00_JPRB, 0.75009E+00_JPRB ,&
&  0.71313E+00_JPRB, 0.66011E+00_JPRB, 0.62595E+00_JPRB, 0.60606E+00_JPRB, 0.59002E+00_JPRB, 0.58060E+00_JPRB, 0.57222E+00_JPRB ,&
&  0.57092E+00_JPRB, 0.57961E+00_JPRB, 0.59021E+00_JPRB, 0.59021E+00_JPRB, 0.58083E+00_JPRB, 0.56631E+00_JPRB, 0.55596E+00_JPRB ,&
&  0.56890E+00_JPRB, 0.59944E+00_JPRB, 0.64202E+00_JPRB, 0.65783E+00_JPRB, 0.67112E+00_JPRB, 0.68986E+00_JPRB, 0.69203E+00_JPRB ,&
&  0.69895E+00_JPRB, 0.72003E+00_JPRB, 0.67394E+00_JPRB, 0.66651E+00_JPRB, 0.68382E+00_JPRB, 0.65441E+00_JPRB, 0.62665E+00_JPRB ,&
&  0.65414E+00_JPRB, 0.72339E+00_JPRB, 0.83617E+00_JPRB, 0.57399E+00_JPRB, 0.51589E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.48798E+00_JPRB, 0.45015E+00_JPRB, 0.41459E+00_JPRB, 0.41477E+00_JPRB, 0.43340E+00_JPRB, 0.43824E+00_JPRB, 0.43568E+00_JPRB ,&
&  0.42478E+00_JPRB, 0.41316E+00_JPRB, 0.39747E+00_JPRB, 0.38957E+00_JPRB, 0.39403E+00_JPRB, 0.40526E+00_JPRB, 0.43866E+00_JPRB ,&
&  0.47186E+00_JPRB, 0.50279E+00_JPRB, 0.52855E+00_JPRB, 0.53258E+00_JPRB, 0.52023E+00_JPRB, 0.51954E+00_JPRB, 0.52723E+00_JPRB ,&
&  0.53499E+00_JPRB, 0.55739E+00_JPRB, 0.58872E+00_JPRB, 0.63318E+00_JPRB, 0.70059E+00_JPRB, 0.81314E+00_JPRB, 0.99169E+00_JPRB ,&
&  0.13024E+01_JPRB, 0.92802E+00_JPRB, 0.63856E+00_JPRB, 0.11586E+01_JPRB, 0.13636E+01_JPRB, 0.13904E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.10488E+01_JPRB, 0.10098E+01_JPRB, 0.10304E+01_JPRB, 0.94609E+00_JPRB, 0.92206E+00_JPRB, 0.89398E+00_JPRB ,&
&  0.88245E+00_JPRB, 0.91841E+00_JPRB, 0.97050E+00_JPRB, 0.10045E+01_JPRB, 0.10710E+01_JPRB, 0.12120E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.15587E+01_JPRB, 0.17492E+01_JPRB, 0.18686E+01_JPRB, 0.14942E+01_JPRB, 0.12722E+01_JPRB, 0.10889E+01_JPRB, 0.98787E+00_JPRB ,&
&  0.88050E+00_JPRB, 0.84674E+00_JPRB, 0.81721E+00_JPRB, 0.78389E+00_JPRB, 0.74044E+00_JPRB, 0.69914E+00_JPRB, 0.70301E+00_JPRB ,&
&  0.65243E+00_JPRB, 0.60536E+00_JPRB, 0.57409E+00_JPRB, 0.55576E+00_JPRB, 0.54798E+00_JPRB, 0.54537E+00_JPRB, 0.53893E+00_JPRB ,&
&  0.54448E+00_JPRB, 0.54961E+00_JPRB, 0.55863E+00_JPRB, 0.55400E+00_JPRB, 0.55038E+00_JPRB, 0.55291E+00_JPRB, 0.54730E+00_JPRB ,&
&  0.57736E+00_JPRB, 0.61835E+00_JPRB, 0.64045E+00_JPRB, 0.66845E+00_JPRB, 0.67209E+00_JPRB, 0.66466E+00_JPRB, 0.68025E+00_JPRB ,&
&  0.67670E+00_JPRB, 0.68567E+00_JPRB, 0.66070E+00_JPRB, 0.67074E+00_JPRB, 0.65294E+00_JPRB, 0.64063E+00_JPRB, 0.64174E+00_JPRB ,&
&  0.68080E+00_JPRB, 0.72871E+00_JPRB, 0.71519E+00_JPRB, 0.56353E+00_JPRB, 0.51343E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.46107E+00_JPRB, 0.43459E+00_JPRB, 0.39631E+00_JPRB, 0.38207E+00_JPRB, 0.40388E+00_JPRB, 0.42122E+00_JPRB, 0.41494E+00_JPRB ,&
&  0.40760E+00_JPRB, 0.39784E+00_JPRB, 0.39329E+00_JPRB, 0.40176E+00_JPRB, 0.43469E+00_JPRB, 0.48163E+00_JPRB, 0.51631E+00_JPRB ,&
&  0.55009E+00_JPRB, 0.57708E+00_JPRB, 0.59918E+00_JPRB, 0.60622E+00_JPRB, 0.60181E+00_JPRB, 0.61010E+00_JPRB, 0.62405E+00_JPRB ,&
&  0.64955E+00_JPRB, 0.68799E+00_JPRB, 0.73970E+00_JPRB, 0.81542E+00_JPRB, 0.91980E+00_JPRB, 0.10477E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.86370E+00_JPRB, 0.45886E+00_JPRB, 0.10869E+01_JPRB, 0.12509E+01_JPRB, 0.11719E+01_JPRB, 0.11503E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.98592E+00_JPRB, 0.98167E+00_JPRB, 0.94173E+00_JPRB, 0.10029E+01_JPRB, 0.95428E+00_JPRB, 0.97312E+00_JPRB, 0.10050E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.11294E+01_JPRB, 0.12210E+01_JPRB, 0.13218E+01_JPRB, 0.14331E+01_JPRB, 0.16203E+01_JPRB, 0.17900E+01_JPRB ,&
&  0.20080E+01_JPRB, 0.22031E+01_JPRB, 0.23074E+01_JPRB, 0.17452E+01_JPRB, 0.14423E+01_JPRB, 0.12614E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.93895E+00_JPRB, 0.76253E+00_JPRB, 0.73215E+00_JPRB, 0.74516E+00_JPRB, 0.71543E+00_JPRB, 0.65839E+00_JPRB, 0.65464E+00_JPRB ,&
&  0.59337E+00_JPRB, 0.54773E+00_JPRB, 0.51704E+00_JPRB, 0.49516E+00_JPRB, 0.49941E+00_JPRB, 0.50431E+00_JPRB, 0.50609E+00_JPRB ,&
&  0.49788E+00_JPRB, 0.50214E+00_JPRB, 0.51963E+00_JPRB, 0.52504E+00_JPRB, 0.53432E+00_JPRB, 0.54859E+00_JPRB, 0.55638E+00_JPRB ,&
&  0.58887E+00_JPRB, 0.60752E+00_JPRB, 0.62853E+00_JPRB, 0.65249E+00_JPRB, 0.66392E+00_JPRB, 0.64493E+00_JPRB, 0.63962E+00_JPRB ,&
&  0.64106E+00_JPRB, 0.64346E+00_JPRB, 0.63256E+00_JPRB, 0.64219E+00_JPRB, 0.63597E+00_JPRB, 0.63494E+00_JPRB, 0.64367E+00_JPRB ,&
&  0.66018E+00_JPRB, 0.65899E+00_JPRB, 0.61859E+00_JPRB, 0.53723E+00_JPRB, 0.49569E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.44120E+00_JPRB, 0.41670E+00_JPRB, 0.37945E+00_JPRB, 0.34593E+00_JPRB, 0.36696E+00_JPRB, 0.36611E+00_JPRB, 0.37963E+00_JPRB ,&
&  0.38376E+00_JPRB, 0.40681E+00_JPRB, 0.43018E+00_JPRB, 0.46531E+00_JPRB, 0.50828E+00_JPRB, 0.55072E+00_JPRB, 0.58560E+00_JPRB ,&
&  0.61673E+00_JPRB, 0.64720E+00_JPRB, 0.66977E+00_JPRB, 0.67811E+00_JPRB, 0.69256E+00_JPRB, 0.71729E+00_JPRB, 0.74673E+00_JPRB ,&
&  0.78658E+00_JPRB, 0.84028E+00_JPRB, 0.91523E+00_JPRB, 0.10102E+01_JPRB, 0.11229E+01_JPRB, 0.11827E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.71459E+00_JPRB, 0.10280E+01_JPRB, 0.11851E+01_JPRB, 0.11090E+01_JPRB, 0.10720E+01_JPRB, 0.10441E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.97035E+00_JPRB, 0.90880E+00_JPRB, 0.90521E+00_JPRB, 0.10084E+01_JPRB, 0.10367E+01_JPRB, 0.11414E+01_JPRB, 0.12360E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.14429E+01_JPRB, 0.15645E+01_JPRB, 0.17044E+01_JPRB, 0.18679E+01_JPRB, 0.20389E+01_JPRB, 0.22255E+01_JPRB ,&
&  0.24547E+01_JPRB, 0.26448E+01_JPRB, 0.27412E+01_JPRB, 0.21603E+01_JPRB, 0.19746E+01_JPRB, 0.18760E+01_JPRB, 0.16481E+01_JPRB ,&
&  0.98734E+00_JPRB, 0.71058E+00_JPRB, 0.70919E+00_JPRB, 0.72740E+00_JPRB, 0.68484E+00_JPRB, 0.64729E+00_JPRB, 0.60157E+00_JPRB ,&
&  0.54850E+00_JPRB, 0.50640E+00_JPRB, 0.48184E+00_JPRB, 0.47333E+00_JPRB, 0.48918E+00_JPRB, 0.48848E+00_JPRB, 0.47325E+00_JPRB ,&
&  0.45512E+00_JPRB, 0.45970E+00_JPRB, 0.49123E+00_JPRB, 0.51622E+00_JPRB, 0.55008E+00_JPRB, 0.56903E+00_JPRB, 0.57911E+00_JPRB ,&
&  0.58052E+00_JPRB, 0.58041E+00_JPRB, 0.60528E+00_JPRB, 0.62355E+00_JPRB, 0.62707E+00_JPRB, 0.62374E+00_JPRB, 0.61273E+00_JPRB ,&
&  0.60207E+00_JPRB, 0.61532E+00_JPRB, 0.62373E+00_JPRB, 0.63369E+00_JPRB, 0.62689E+00_JPRB, 0.61399E+00_JPRB, 0.60967E+00_JPRB ,&
&  0.59475E+00_JPRB, 0.56972E+00_JPRB, 0.54253E+00_JPRB, 0.50489E+00_JPRB, 0.47001E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.43199E+00_JPRB, 0.41790E+00_JPRB, 0.40090E+00_JPRB, 0.39140E+00_JPRB, 0.38318E+00_JPRB, 0.37410E+00_JPRB, 0.38368E+00_JPRB ,&
&  0.40978E+00_JPRB, 0.43315E+00_JPRB, 0.47268E+00_JPRB, 0.53185E+00_JPRB, 0.58575E+00_JPRB, 0.61491E+00_JPRB, 0.64721E+00_JPRB ,&
&  0.67483E+00_JPRB, 0.70559E+00_JPRB, 0.73011E+00_JPRB, 0.75242E+00_JPRB, 0.78229E+00_JPRB, 0.82075E+00_JPRB, 0.87314E+00_JPRB ,&
&  0.93684E+00_JPRB, 0.10011E+01_JPRB, 0.10825E+01_JPRB, 0.11855E+01_JPRB, 0.12845E+01_JPRB, 0.13216E+01_JPRB, 0.13722E+01_JPRB ,&
&  0.97720E+00_JPRB, 0.11585E+01_JPRB, 0.11422E+01_JPRB, 0.10984E+01_JPRB, 0.10477E+01_JPRB, 0.10045E+01_JPRB, 0.98465E+00_JPRB ,&
&  0.97853E+00_JPRB, 0.93390E+00_JPRB, 0.94984E+00_JPRB, 0.10358E+01_JPRB, 0.12201E+01_JPRB, 0.13477E+01_JPRB, 0.14601E+01_JPRB ,&
&  0.15783E+01_JPRB, 0.17172E+01_JPRB, 0.18620E+01_JPRB, 0.20199E+01_JPRB, 0.21724E+01_JPRB, 0.23532E+01_JPRB, 0.24985E+01_JPRB ,&
&  0.26478E+01_JPRB, 0.27662E+01_JPRB, 0.28115E+01_JPRB, 0.22949E+01_JPRB, 0.18236E+01_JPRB, 0.15795E+01_JPRB, 0.11895E+01_JPRB ,&
&  0.83281E+00_JPRB, 0.69161E+00_JPRB, 0.71155E+00_JPRB, 0.72773E+00_JPRB, 0.67194E+00_JPRB, 0.62007E+00_JPRB, 0.56223E+00_JPRB ,&
&  0.52801E+00_JPRB, 0.47836E+00_JPRB, 0.46061E+00_JPRB, 0.47055E+00_JPRB, 0.49668E+00_JPRB, 0.47644E+00_JPRB, 0.44860E+00_JPRB ,&
&  0.42972E+00_JPRB, 0.42217E+00_JPRB, 0.45989E+00_JPRB, 0.50370E+00_JPRB, 0.55783E+00_JPRB, 0.56392E+00_JPRB, 0.53599E+00_JPRB ,&
&  0.56206E+00_JPRB, 0.53325E+00_JPRB, 0.54616E+00_JPRB, 0.56649E+00_JPRB, 0.58137E+00_JPRB, 0.59332E+00_JPRB, 0.60258E+00_JPRB ,&
&  0.63388E+00_JPRB, 0.69368E+00_JPRB, 0.71260E+00_JPRB, 0.66403E+00_JPRB, 0.62384E+00_JPRB, 0.57643E+00_JPRB, 0.53055E+00_JPRB ,&
&  0.53211E+00_JPRB, 0.51884E+00_JPRB, 0.50513E+00_JPRB, 0.48547E+00_JPRB, 0.45973E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.48818E+00_JPRB, 0.47462E+00_JPRB, 0.46687E+00_JPRB, 0.46082E+00_JPRB, 0.44543E+00_JPRB, 0.44575E+00_JPRB, 0.44274E+00_JPRB ,&
&  0.45911E+00_JPRB, 0.47964E+00_JPRB, 0.51593E+00_JPRB, 0.55970E+00_JPRB, 0.61637E+00_JPRB, 0.65572E+00_JPRB, 0.70206E+00_JPRB ,&
&  0.72885E+00_JPRB, 0.76425E+00_JPRB, 0.80564E+00_JPRB, 0.83951E+00_JPRB, 0.88094E+00_JPRB, 0.90913E+00_JPRB, 0.95932E+00_JPRB ,&
&  0.10048E+01_JPRB, 0.10588E+01_JPRB, 0.11330E+01_JPRB, 0.11999E+01_JPRB, 0.12560E+01_JPRB, 0.12749E+01_JPRB, 0.99613E+00_JPRB ,&
&  0.11439E+01_JPRB, 0.11484E+01_JPRB, 0.11318E+01_JPRB, 0.10774E+01_JPRB, 0.10101E+01_JPRB, 0.95317E+00_JPRB, 0.94445E+00_JPRB ,&
&  0.95594E+00_JPRB, 0.96846E+00_JPRB, 0.10034E+01_JPRB, 0.10719E+01_JPRB, 0.13391E+01_JPRB, 0.14853E+01_JPRB, 0.15961E+01_JPRB ,&
&  0.17257E+01_JPRB, 0.18676E+01_JPRB, 0.20192E+01_JPRB, 0.21875E+01_JPRB, 0.23205E+01_JPRB, 0.24652E+01_JPRB, 0.26076E+01_JPRB ,&
&  0.27002E+01_JPRB, 0.27876E+01_JPRB, 0.26774E+01_JPRB, 0.20572E+01_JPRB, 0.16787E+01_JPRB, 0.13755E+01_JPRB, 0.99885E+00_JPRB ,&
&  0.80333E+00_JPRB, 0.74783E+00_JPRB, 0.76761E+00_JPRB, 0.76520E+00_JPRB, 0.68997E+00_JPRB, 0.62485E+00_JPRB, 0.58035E+00_JPRB ,&
&  0.52382E+00_JPRB, 0.50959E+00_JPRB, 0.49901E+00_JPRB, 0.52444E+00_JPRB, 0.53762E+00_JPRB, 0.51475E+00_JPRB, 0.50103E+00_JPRB ,&
&  0.47662E+00_JPRB, 0.45659E+00_JPRB, 0.47988E+00_JPRB, 0.52126E+00_JPRB, 0.60543E+00_JPRB, 0.66143E+00_JPRB, 0.63321E+00_JPRB ,&
&  0.66278E+00_JPRB, 0.45841E+00_JPRB, 0.48987E+00_JPRB, 0.52738E+00_JPRB, 0.58679E+00_JPRB, 0.62799E+00_JPRB, 0.71472E+00_JPRB ,&
&  0.90420E+00_JPRB, 0.12462E+01_JPRB, 0.11120E+01_JPRB, 0.84902E+00_JPRB, 0.94035E+00_JPRB, 0.86048E+00_JPRB, 0.47159E+00_JPRB ,&
&  0.47322E+00_JPRB, 0.47344E+00_JPRB, 0.48897E+00_JPRB, 0.49373E+00_JPRB, 0.49682E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.56819E+00_JPRB, 0.56271E+00_JPRB, 0.56202E+00_JPRB, 0.55260E+00_JPRB, 0.54769E+00_JPRB, 0.54844E+00_JPRB, 0.53642E+00_JPRB ,&
&  0.54547E+00_JPRB, 0.56194E+00_JPRB, 0.56819E+00_JPRB, 0.58637E+00_JPRB, 0.59717E+00_JPRB, 0.63283E+00_JPRB, 0.68490E+00_JPRB ,&
&  0.71979E+00_JPRB, 0.79792E+00_JPRB, 0.83593E+00_JPRB, 0.88220E+00_JPRB, 0.92096E+00_JPRB, 0.96177E+00_JPRB, 0.99664E+00_JPRB ,&
&  0.10468E+01_JPRB, 0.11064E+01_JPRB, 0.11721E+01_JPRB, 0.12173E+01_JPRB, 0.12210E+01_JPRB, 0.13490E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.11486E+01_JPRB, 0.11333E+01_JPRB, 0.10481E+01_JPRB, 0.99880E+00_JPRB, 0.95243E+00_JPRB, 0.92669E+00_JPRB, 0.91325E+00_JPRB ,&
&  0.93009E+00_JPRB, 0.96219E+00_JPRB, 0.10230E+01_JPRB, 0.11643E+01_JPRB, 0.13312E+01_JPRB, 0.14870E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.17038E+01_JPRB, 0.18074E+01_JPRB, 0.19393E+01_JPRB, 0.20841E+01_JPRB, 0.22382E+01_JPRB, 0.23476E+01_JPRB, 0.24489E+01_JPRB ,&
&  0.24897E+01_JPRB, 0.25208E+01_JPRB, 0.23325E+01_JPRB, 0.20061E+01_JPRB, 0.16632E+01_JPRB, 0.13476E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.85799E+00_JPRB, 0.81325E+00_JPRB, 0.82564E+00_JPRB, 0.85136E+00_JPRB, 0.76232E+00_JPRB, 0.70615E+00_JPRB, 0.67020E+00_JPRB ,&
&  0.63223E+00_JPRB, 0.57788E+00_JPRB, 0.59314E+00_JPRB, 0.63344E+00_JPRB, 0.64490E+00_JPRB, 0.63079E+00_JPRB, 0.60722E+00_JPRB ,&
&  0.57925E+00_JPRB, 0.54244E+00_JPRB, 0.53591E+00_JPRB, 0.53661E+00_JPRB, 0.54249E+00_JPRB, 0.56050E+00_JPRB, 0.47030E+00_JPRB ,&
&  0.51543E+00_JPRB, 0.45382E+00_JPRB, 0.47292E+00_JPRB, 0.52783E+00_JPRB, 0.62474E+00_JPRB, 0.80089E+00_JPRB, 0.11244E+01_JPRB ,&
&  0.17087E+01_JPRB, 0.19695E+01_JPRB, 0.22261E+01_JPRB, 0.27769E+01_JPRB, 0.24314E+01_JPRB, 0.17909E+01_JPRB, 0.52555E+00_JPRB ,&
&  0.52313E+00_JPRB, 0.52759E+00_JPRB, 0.54997E+00_JPRB, 0.55858E+00_JPRB, 0.56334E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.61241E+00_JPRB, 0.62166E+00_JPRB, 0.63412E+00_JPRB, 0.64738E+00_JPRB, 0.66365E+00_JPRB, 0.68014E+00_JPRB, 0.69292E+00_JPRB ,&
&  0.70663E+00_JPRB, 0.72373E+00_JPRB, 0.74022E+00_JPRB, 0.75158E+00_JPRB, 0.76313E+00_JPRB, 0.78512E+00_JPRB, 0.81081E+00_JPRB ,&
&  0.82106E+00_JPRB, 0.86277E+00_JPRB, 0.88145E+00_JPRB, 0.92057E+00_JPRB, 0.95654E+00_JPRB, 0.10008E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.11072E+01_JPRB, 0.11216E+01_JPRB, 0.11096E+01_JPRB, 0.11174E+01_JPRB, 0.11858E+01_JPRB, 0.93174E+00_JPRB ,&
&  0.11737E+01_JPRB, 0.11277E+01_JPRB, 0.99669E+00_JPRB, 0.93063E+00_JPRB, 0.87942E+00_JPRB, 0.85976E+00_JPRB, 0.85860E+00_JPRB ,&
&  0.87314E+00_JPRB, 0.93779E+00_JPRB, 0.94804E+00_JPRB, 0.10004E+01_JPRB, 0.11456E+01_JPRB, 0.13304E+01_JPRB, 0.14661E+01_JPRB ,&
&  0.15117E+01_JPRB, 0.16081E+01_JPRB, 0.16401E+01_JPRB, 0.16866E+01_JPRB, 0.17782E+01_JPRB, 0.18682E+01_JPRB, 0.19140E+01_JPRB ,&
&  0.18742E+01_JPRB, 0.17993E+01_JPRB, 0.16185E+01_JPRB, 0.17006E+01_JPRB, 0.15916E+01_JPRB, 0.13323E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.85342E+00_JPRB, 0.88961E+00_JPRB, 0.94188E+00_JPRB, 0.10164E+01_JPRB, 0.94858E+00_JPRB, 0.89642E+00_JPRB, 0.78830E+00_JPRB ,&
&  0.74331E+00_JPRB, 0.72769E+00_JPRB, 0.75783E+00_JPRB, 0.80056E+00_JPRB, 0.82638E+00_JPRB, 0.80997E+00_JPRB, 0.75138E+00_JPRB ,&
&  0.69640E+00_JPRB, 0.63585E+00_JPRB, 0.60169E+00_JPRB, 0.58432E+00_JPRB, 0.54652E+00_JPRB, 0.46719E+00_JPRB, 0.50171E+00_JPRB ,&
&  0.46637E+00_JPRB, 0.42684E+00_JPRB, 0.47619E+00_JPRB, 0.39873E+00_JPRB, 0.58448E+00_JPRB, 0.75807E+00_JPRB, 0.10196E+01_JPRB ,&
&  0.12249E+01_JPRB, 0.17330E+01_JPRB, 0.21910E+01_JPRB, 0.16668E+01_JPRB, 0.11503E+01_JPRB, 0.65702E+00_JPRB, 0.56654E+00_JPRB ,&
&  0.57477E+00_JPRB, 0.58008E+00_JPRB, 0.58893E+00_JPRB, 0.58924E+00_JPRB, 0.60108E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.62970E+00_JPRB, 0.64281E+00_JPRB, 0.66045E+00_JPRB, 0.68406E+00_JPRB, 0.70655E+00_JPRB, 0.72374E+00_JPRB, 0.73513E+00_JPRB ,&
&  0.75467E+00_JPRB, 0.78096E+00_JPRB, 0.79661E+00_JPRB, 0.80413E+00_JPRB, 0.82105E+00_JPRB, 0.83659E+00_JPRB, 0.86418E+00_JPRB ,&
&  0.88208E+00_JPRB, 0.92187E+00_JPRB, 0.95305E+00_JPRB, 0.98439E+00_JPRB, 0.10095E+01_JPRB, 0.10392E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.10901E+01_JPRB, 0.11365E+01_JPRB, 0.11774E+01_JPRB, 0.12060E+01_JPRB, 0.12492E+01_JPRB, 0.12860E+01_JPRB, 0.10430E+01_JPRB ,&
&  0.12779E+01_JPRB, 0.11977E+01_JPRB, 0.10402E+01_JPRB, 0.92028E+00_JPRB, 0.84106E+00_JPRB, 0.78028E+00_JPRB, 0.74753E+00_JPRB ,&
&  0.74379E+00_JPRB, 0.77382E+00_JPRB, 0.80330E+00_JPRB, 0.79815E+00_JPRB, 0.79746E+00_JPRB, 0.90589E+00_JPRB, 0.11029E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12702E+01_JPRB, 0.13008E+01_JPRB, 0.12934E+01_JPRB, 0.13471E+01_JPRB, 0.14201E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.14125E+01_JPRB, 0.13168E+01_JPRB, 0.11408E+01_JPRB, 0.13499E+01_JPRB, 0.13653E+01_JPRB, 0.13189E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.10134E+01_JPRB, 0.10499E+01_JPRB, 0.11587E+01_JPRB, 0.11154E+01_JPRB, 0.10840E+01_JPRB, 0.10083E+01_JPRB ,&
&  0.94832E+00_JPRB, 0.92014E+00_JPRB, 0.93118E+00_JPRB, 0.96357E+00_JPRB, 0.98266E+00_JPRB, 0.96676E+00_JPRB, 0.88444E+00_JPRB ,&
&  0.80961E+00_JPRB, 0.74219E+00_JPRB, 0.69589E+00_JPRB, 0.65475E+00_JPRB, 0.61471E+00_JPRB, 0.56951E+00_JPRB, 0.72438E+00_JPRB ,&
&  0.50218E+00_JPRB, 0.38633E+00_JPRB, 0.41237E+00_JPRB, 0.49411E+00_JPRB, 0.52848E+00_JPRB, 0.60231E+00_JPRB, 0.76237E+00_JPRB ,&
&  0.97210E+00_JPRB, 0.11180E+01_JPRB, 0.10947E+01_JPRB, 0.83260E+00_JPRB, 0.65610E+00_JPRB, 0.60140E+00_JPRB, 0.59817E+00_JPRB ,&
&  0.60702E+00_JPRB, 0.60556E+00_JPRB, 0.60854E+00_JPRB, 0.61373E+00_JPRB, 0.62258E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.60632E+00_JPRB, 0.60608E+00_JPRB, 0.61398E+00_JPRB, 0.62891E+00_JPRB, 0.63629E+00_JPRB, 0.64164E+00_JPRB, 0.64530E+00_JPRB ,&
&  0.66287E+00_JPRB, 0.68290E+00_JPRB, 0.70135E+00_JPRB, 0.71958E+00_JPRB, 0.73589E+00_JPRB, 0.74997E+00_JPRB, 0.78496E+00_JPRB ,&
&  0.80335E+00_JPRB, 0.82944E+00_JPRB, 0.84738E+00_JPRB, 0.86642E+00_JPRB, 0.88257E+00_JPRB, 0.87650E+00_JPRB, 0.88232E+00_JPRB ,&
&  0.89372E+00_JPRB, 0.92392E+00_JPRB, 0.95621E+00_JPRB, 0.10114E+01_JPRB, 0.10817E+01_JPRB, 0.10955E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.13401E+01_JPRB, 0.11519E+01_JPRB, 0.97580E+00_JPRB, 0.92809E+00_JPRB, 0.82742E+00_JPRB, 0.76831E+00_JPRB ,&
&  0.76235E+00_JPRB, 0.74107E+00_JPRB, 0.70967E+00_JPRB, 0.67388E+00_JPRB, 0.63666E+00_JPRB, 0.66588E+00_JPRB, 0.72082E+00_JPRB ,&
&  0.87432E+00_JPRB, 0.97056E+00_JPRB, 0.10866E+01_JPRB, 0.11931E+01_JPRB, 0.13079E+01_JPRB, 0.13530E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.13031E+01_JPRB, 0.12432E+01_JPRB, 0.11273E+01_JPRB, 0.12732E+01_JPRB, 0.14007E+01_JPRB, 0.14680E+01_JPRB, 0.14958E+01_JPRB ,&
&  0.14859E+01_JPRB, 0.13645E+01_JPRB, 0.13390E+01_JPRB, 0.13217E+01_JPRB, 0.12652E+01_JPRB, 0.12664E+01_JPRB, 0.11948E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.10601E+01_JPRB, 0.10341E+01_JPRB, 0.10305E+01_JPRB, 0.10286E+01_JPRB, 0.10024E+01_JPRB, 0.90407E+00_JPRB ,&
&  0.84178E+00_JPRB, 0.79790E+00_JPRB, 0.76674E+00_JPRB, 0.73066E+00_JPRB, 0.64583E+00_JPRB, 0.97245E+00_JPRB, 0.11836E+01_JPRB ,&
&  0.62317E+00_JPRB, 0.54581E+00_JPRB, 0.43462E+00_JPRB, 0.55999E+00_JPRB, 0.54739E+00_JPRB, 0.58418E+00_JPRB, 0.65225E+00_JPRB ,&
&  0.66655E+00_JPRB, 0.65409E+00_JPRB, 0.59141E+00_JPRB, 0.53071E+00_JPRB, 0.54686E+00_JPRB, 0.55205E+00_JPRB, 0.55441E+00_JPRB ,&
&  0.56911E+00_JPRB, 0.57682E+00_JPRB, 0.58724E+00_JPRB, 0.60698E+00_JPRB, 0.60183E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.59117E+00_JPRB, 0.58059E+00_JPRB, 0.57056E+00_JPRB, 0.56022E+00_JPRB, 0.56684E+00_JPRB, 0.56827E+00_JPRB, 0.57786E+00_JPRB ,&
&  0.58881E+00_JPRB, 0.58825E+00_JPRB, 0.59343E+00_JPRB, 0.59532E+00_JPRB, 0.60966E+00_JPRB, 0.62231E+00_JPRB, 0.65137E+00_JPRB ,&
&  0.67950E+00_JPRB, 0.69461E+00_JPRB, 0.71455E+00_JPRB, 0.73446E+00_JPRB, 0.74810E+00_JPRB, 0.76331E+00_JPRB, 0.74991E+00_JPRB ,&
&  0.73404E+00_JPRB, 0.73706E+00_JPRB, 0.77607E+00_JPRB, 0.91450E+00_JPRB, 0.11730E+01_JPRB, 0.14983E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.20513E+01_JPRB, 0.14047E+01_JPRB, 0.12186E+01_JPRB, 0.10571E+01_JPRB, 0.10995E+01_JPRB, 0.11773E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11442E+01_JPRB, 0.11318E+01_JPRB, 0.11145E+01_JPRB, 0.10862E+01_JPRB, 0.10237E+01_JPRB, 0.94404E+00_JPRB, 0.88957E+00_JPRB ,&
&  0.87105E+00_JPRB, 0.84987E+00_JPRB, 0.97566E+00_JPRB, 0.12841E+01_JPRB, 0.14988E+01_JPRB, 0.18551E+01_JPRB, 0.17740E+01_JPRB ,&
&  0.17522E+01_JPRB, 0.16882E+01_JPRB, 0.16230E+01_JPRB, 0.17824E+01_JPRB, 0.18585E+01_JPRB, 0.19301E+01_JPRB, 0.19387E+01_JPRB ,&
&  0.20527E+01_JPRB, 0.18193E+01_JPRB, 0.13033E+01_JPRB, 0.14693E+01_JPRB, 0.14207E+01_JPRB, 0.14233E+01_JPRB, 0.13365E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.11641E+01_JPRB, 0.10925E+01_JPRB, 0.10322E+01_JPRB, 0.99539E+00_JPRB, 0.94284E+00_JPRB, 0.84600E+00_JPRB ,&
&  0.81448E+00_JPRB, 0.79733E+00_JPRB, 0.78286E+00_JPRB, 0.73691E+00_JPRB, 0.72648E+00_JPRB, 0.72006E+00_JPRB, 0.85084E+00_JPRB ,&
&  0.70408E+00_JPRB, 0.74586E+00_JPRB, 0.66040E+00_JPRB, 0.86234E+00_JPRB, 0.65255E+00_JPRB, 0.56304E+00_JPRB, 0.56380E+00_JPRB ,&
&  0.54341E+00_JPRB, 0.52391E+00_JPRB, 0.50880E+00_JPRB, 0.51096E+00_JPRB, 0.52636E+00_JPRB, 0.55358E+00_JPRB, 0.55834E+00_JPRB ,&
&  0.56670E+00_JPRB, 0.57771E+00_JPRB, 0.59429E+00_JPRB, 0.61375E+00_JPRB, 0.60947E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.62886E+00_JPRB, 0.62981E+00_JPRB, 0.63189E+00_JPRB, 0.64014E+00_JPRB, 0.64187E+00_JPRB, 0.63903E+00_JPRB, 0.62639E+00_JPRB ,&
&  0.61722E+00_JPRB, 0.60173E+00_JPRB, 0.59284E+00_JPRB, 0.58779E+00_JPRB, 0.60344E+00_JPRB, 0.63893E+00_JPRB, 0.69095E+00_JPRB ,&
&  0.70747E+00_JPRB, 0.70636E+00_JPRB, 0.67856E+00_JPRB, 0.66090E+00_JPRB, 0.68345E+00_JPRB, 0.68523E+00_JPRB, 0.66188E+00_JPRB ,&
&  0.64634E+00_JPRB, 0.71796E+00_JPRB, 0.90741E+00_JPRB, 0.11962E+01_JPRB, 0.17104E+01_JPRB, 0.19833E+01_JPRB, 0.24911E+01_JPRB ,&
&  0.23269E+01_JPRB, 0.16202E+01_JPRB, 0.16163E+01_JPRB, 0.15000E+01_JPRB, 0.15832E+01_JPRB, 0.16349E+01_JPRB, 0.16528E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.17298E+01_JPRB, 0.17619E+01_JPRB, 0.18025E+01_JPRB, 0.18204E+01_JPRB, 0.17995E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.17539E+01_JPRB, 0.17054E+01_JPRB, 0.17285E+01_JPRB, 0.18077E+01_JPRB, 0.20175E+01_JPRB, 0.22515E+01_JPRB, 0.22890E+01_JPRB ,&
&  0.22919E+01_JPRB, 0.23665E+01_JPRB, 0.22896E+01_JPRB, 0.24628E+01_JPRB, 0.25272E+01_JPRB, 0.24687E+01_JPRB, 0.26425E+01_JPRB ,&
&  0.27179E+01_JPRB, 0.22820E+01_JPRB, 0.14985E+01_JPRB, 0.16254E+01_JPRB, 0.15489E+01_JPRB, 0.14286E+01_JPRB, 0.14537E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.12308E+01_JPRB, 0.11307E+01_JPRB, 0.10599E+01_JPRB, 0.10145E+01_JPRB, 0.93621E+00_JPRB, 0.91305E+00_JPRB ,&
&  0.84239E+00_JPRB, 0.80058E+00_JPRB, 0.76637E+00_JPRB, 0.72554E+00_JPRB, 0.67765E+00_JPRB, 0.70157E+00_JPRB, 0.73697E+00_JPRB ,&
&  0.79612E+00_JPRB, 0.86120E+00_JPRB, 0.10514E+01_JPRB, 0.11746E+01_JPRB, 0.75539E+00_JPRB, 0.65345E+00_JPRB, 0.61349E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.60896E+00_JPRB, 0.60562E+00_JPRB, 0.59813E+00_JPRB, 0.62485E+00_JPRB, 0.66643E+00_JPRB, 0.66868E+00_JPRB ,&
&  0.67650E+00_JPRB, 0.65007E+00_JPRB, 0.64612E+00_JPRB, 0.64917E+00_JPRB, 0.64969E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.76415E+00_JPRB, 0.76073E+00_JPRB, 0.74922E+00_JPRB, 0.76174E+00_JPRB, 0.77564E+00_JPRB, 0.77431E+00_JPRB, 0.79076E+00_JPRB ,&
&  0.81247E+00_JPRB, 0.83433E+00_JPRB, 0.85072E+00_JPRB, 0.86450E+00_JPRB, 0.92321E+00_JPRB, 0.99173E+00_JPRB, 0.10731E+01_JPRB ,&
&  0.11212E+01_JPRB, 0.11663E+01_JPRB, 0.11775E+01_JPRB, 0.11612E+01_JPRB, 0.11496E+01_JPRB, 0.11051E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.11347E+01_JPRB, 0.14663E+01_JPRB, 0.20074E+01_JPRB, 0.25218E+01_JPRB, 0.30044E+01_JPRB, 0.26033E+01_JPRB, 0.28134E+01_JPRB ,&
&  0.24762E+01_JPRB, 0.18783E+01_JPRB, 0.17623E+01_JPRB, 0.17956E+01_JPRB, 0.18686E+01_JPRB, 0.18763E+01_JPRB, 0.19171E+01_JPRB ,&
&  0.19409E+01_JPRB, 0.19641E+01_JPRB, 0.20034E+01_JPRB, 0.20942E+01_JPRB, 0.21583E+01_JPRB, 0.22039E+01_JPRB, 0.22637E+01_JPRB ,&
&  0.23049E+01_JPRB, 0.22967E+01_JPRB, 0.22845E+01_JPRB, 0.21118E+01_JPRB, 0.22065E+01_JPRB, 0.24078E+01_JPRB, 0.25893E+01_JPRB ,&
&  0.26870E+01_JPRB, 0.27024E+01_JPRB, 0.29116E+01_JPRB, 0.29654E+01_JPRB, 0.29503E+01_JPRB, 0.29375E+01_JPRB, 0.30236E+01_JPRB ,&
&  0.29944E+01_JPRB, 0.26842E+01_JPRB, 0.18799E+01_JPRB, 0.20726E+01_JPRB, 0.19042E+01_JPRB, 0.15440E+01_JPRB, 0.15618E+01_JPRB ,&
&  0.14132E+01_JPRB, 0.12622E+01_JPRB, 0.11020E+01_JPRB, 0.10663E+01_JPRB, 0.10899E+01_JPRB, 0.11108E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.89283E+00_JPRB, 0.73974E+00_JPRB, 0.69862E+00_JPRB, 0.66791E+00_JPRB, 0.71975E+00_JPRB, 0.79782E+00_JPRB ,&
&  0.86490E+00_JPRB, 0.87032E+00_JPRB, 0.97879E+00_JPRB, 0.11332E+01_JPRB, 0.89413E+00_JPRB, 0.76949E+00_JPRB, 0.75100E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.76595E+00_JPRB, 0.77298E+00_JPRB, 0.77271E+00_JPRB, 0.78438E+00_JPRB, 0.80709E+00_JPRB, 0.82424E+00_JPRB ,&
&  0.83194E+00_JPRB, 0.81208E+00_JPRB, 0.79219E+00_JPRB, 0.77760E+00_JPRB, 0.76559E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.87459E+00_JPRB, 0.83816E+00_JPRB, 0.80990E+00_JPRB, 0.79852E+00_JPRB, 0.79768E+00_JPRB, 0.80783E+00_JPRB, 0.83068E+00_JPRB ,&
&  0.87168E+00_JPRB, 0.92848E+00_JPRB, 0.98893E+00_JPRB, 0.10442E+01_JPRB, 0.11223E+01_JPRB, 0.12188E+01_JPRB, 0.13383E+01_JPRB ,&
&  0.14699E+01_JPRB, 0.16114E+01_JPRB, 0.16902E+01_JPRB, 0.17235E+01_JPRB, 0.17943E+01_JPRB, 0.19056E+01_JPRB, 0.21677E+01_JPRB ,&
&  0.25611E+01_JPRB, 0.28991E+01_JPRB, 0.30980E+01_JPRB, 0.28861E+01_JPRB, 0.22637E+01_JPRB, 0.22416E+01_JPRB, 0.24563E+01_JPRB ,&
&  0.25307E+01_JPRB, 0.21483E+01_JPRB, 0.18478E+01_JPRB, 0.18197E+01_JPRB, 0.18298E+01_JPRB, 0.18814E+01_JPRB, 0.19139E+01_JPRB ,&
&  0.19289E+01_JPRB, 0.19584E+01_JPRB, 0.20332E+01_JPRB, 0.21148E+01_JPRB, 0.22335E+01_JPRB, 0.23590E+01_JPRB, 0.24918E+01_JPRB ,&
&  0.25370E+01_JPRB, 0.25874E+01_JPRB, 0.25386E+01_JPRB, 0.24309E+01_JPRB, 0.24094E+01_JPRB, 0.25159E+01_JPRB, 0.27032E+01_JPRB ,&
&  0.28858E+01_JPRB, 0.29397E+01_JPRB, 0.31228E+01_JPRB, 0.32446E+01_JPRB, 0.31248E+01_JPRB, 0.29192E+01_JPRB, 0.28358E+01_JPRB ,&
&  0.28263E+01_JPRB, 0.30139E+01_JPRB, 0.21341E+01_JPRB, 0.25844E+01_JPRB, 0.26441E+01_JPRB, 0.21923E+01_JPRB, 0.18148E+01_JPRB ,&
&  0.16484E+01_JPRB, 0.14737E+01_JPRB, 0.13141E+01_JPRB, 0.13485E+01_JPRB, 0.13566E+01_JPRB, 0.13773E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.10612E+01_JPRB, 0.82094E+00_JPRB, 0.67315E+00_JPRB, 0.70631E+00_JPRB, 0.82944E+00_JPRB, 0.85549E+00_JPRB ,&
&  0.83670E+00_JPRB, 0.99804E+00_JPRB, 0.11694E+01_JPRB, 0.14867E+01_JPRB, 0.16161E+01_JPRB, 0.85912E+00_JPRB, 0.82650E+00_JPRB ,&
&  0.82566E+00_JPRB, 0.85994E+00_JPRB, 0.87125E+00_JPRB, 0.87906E+00_JPRB, 0.88914E+00_JPRB, 0.91592E+00_JPRB, 0.95068E+00_JPRB ,&
&  0.96831E+00_JPRB, 0.97537E+00_JPRB, 0.97466E+00_JPRB, 0.95064E+00_JPRB, 0.90967E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.97975E+00_JPRB, 0.96067E+00_JPRB, 0.94276E+00_JPRB, 0.91798E+00_JPRB, 0.88408E+00_JPRB, 0.85794E+00_JPRB, 0.85651E+00_JPRB ,&
&  0.86339E+00_JPRB, 0.90611E+00_JPRB, 0.97890E+00_JPRB, 0.10697E+01_JPRB, 0.11871E+01_JPRB, 0.13245E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.16592E+01_JPRB, 0.18521E+01_JPRB, 0.19806E+01_JPRB, 0.20905E+01_JPRB, 0.22246E+01_JPRB, 0.24379E+01_JPRB, 0.26918E+01_JPRB ,&
&  0.29150E+01_JPRB, 0.29341E+01_JPRB, 0.27567E+01_JPRB, 0.20333E+01_JPRB, 0.17949E+01_JPRB, 0.21244E+01_JPRB, 0.20111E+01_JPRB ,&
&  0.19701E+01_JPRB, 0.18631E+01_JPRB, 0.18044E+01_JPRB, 0.17678E+01_JPRB, 0.17701E+01_JPRB, 0.18190E+01_JPRB, 0.18553E+01_JPRB ,&
&  0.18609E+01_JPRB, 0.18792E+01_JPRB, 0.19415E+01_JPRB, 0.20701E+01_JPRB, 0.22493E+01_JPRB, 0.23458E+01_JPRB, 0.24495E+01_JPRB ,&
&  0.25295E+01_JPRB, 0.26572E+01_JPRB, 0.26956E+01_JPRB, 0.26953E+01_JPRB, 0.26256E+01_JPRB, 0.26456E+01_JPRB, 0.27986E+01_JPRB ,&
&  0.29718E+01_JPRB, 0.30975E+01_JPRB, 0.32490E+01_JPRB, 0.34208E+01_JPRB, 0.34567E+01_JPRB, 0.29874E+01_JPRB, 0.27788E+01_JPRB ,&
&  0.27731E+01_JPRB, 0.27487E+01_JPRB, 0.25056E+01_JPRB, 0.32625E+01_JPRB, 0.20683E+01_JPRB, 0.21276E+01_JPRB, 0.20399E+01_JPRB ,&
&  0.18728E+01_JPRB, 0.18466E+01_JPRB, 0.18648E+01_JPRB, 0.18651E+01_JPRB, 0.17693E+01_JPRB, 0.14862E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.11391E+01_JPRB, 0.90188E+00_JPRB, 0.65744E+00_JPRB, 0.73504E+00_JPRB, 0.95331E+00_JPRB, 0.93543E+00_JPRB ,&
&  0.84101E+00_JPRB, 0.99190E+00_JPRB, 0.11976E+01_JPRB, 0.14379E+01_JPRB, 0.13591E+01_JPRB, 0.87807E+00_JPRB, 0.84280E+00_JPRB ,&
&  0.85299E+00_JPRB, 0.86051E+00_JPRB, 0.87318E+00_JPRB, 0.87765E+00_JPRB, 0.90464E+00_JPRB, 0.94767E+00_JPRB, 0.98852E+00_JPRB ,&
&  0.10020E+01_JPRB, 0.10004E+01_JPRB, 0.10169E+01_JPRB, 0.10144E+01_JPRB, 0.10002E+01_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.10297E+01_JPRB, 0.10276E+01_JPRB, 0.10426E+01_JPRB, 0.10468E+01_JPRB, 0.10430E+01_JPRB, 0.98717E+00_JPRB, 0.94170E+00_JPRB ,&
&  0.88697E+00_JPRB, 0.83747E+00_JPRB, 0.90162E+00_JPRB, 0.98406E+00_JPRB, 0.10835E+01_JPRB, 0.11897E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.14596E+01_JPRB, 0.16515E+01_JPRB, 0.18891E+01_JPRB, 0.20789E+01_JPRB, 0.22612E+01_JPRB, 0.25086E+01_JPRB, 0.27306E+01_JPRB ,&
&  0.23206E+01_JPRB, 0.16493E+01_JPRB, 0.21642E+01_JPRB, 0.25240E+01_JPRB, 0.24767E+01_JPRB, 0.27300E+01_JPRB, 0.25360E+01_JPRB ,&
&  0.20262E+01_JPRB, 0.18203E+01_JPRB, 0.18194E+01_JPRB, 0.17913E+01_JPRB, 0.17217E+01_JPRB, 0.17500E+01_JPRB, 0.17961E+01_JPRB ,&
&  0.18250E+01_JPRB, 0.18263E+01_JPRB, 0.18439E+01_JPRB, 0.19463E+01_JPRB, 0.21211E+01_JPRB, 0.21931E+01_JPRB, 0.23365E+01_JPRB ,&
&  0.24744E+01_JPRB, 0.26554E+01_JPRB, 0.26994E+01_JPRB, 0.26462E+01_JPRB, 0.26903E+01_JPRB, 0.27741E+01_JPRB, 0.28998E+01_JPRB ,&
&  0.30581E+01_JPRB, 0.31505E+01_JPRB, 0.32829E+01_JPRB, 0.34893E+01_JPRB, 0.35525E+01_JPRB, 0.31029E+01_JPRB, 0.30660E+01_JPRB ,&
&  0.29687E+01_JPRB, 0.28102E+01_JPRB, 0.26973E+01_JPRB, 0.32243E+01_JPRB, 0.24059E+01_JPRB, 0.23494E+01_JPRB, 0.20223E+01_JPRB ,&
&  0.21197E+01_JPRB, 0.21360E+01_JPRB, 0.21334E+01_JPRB, 0.20607E+01_JPRB, 0.19131E+01_JPRB, 0.16201E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.12278E+01_JPRB, 0.92415E+00_JPRB, 0.70004E+00_JPRB, 0.64254E+00_JPRB, 0.82909E+00_JPRB, 0.95511E+00_JPRB ,&
&  0.10830E+01_JPRB, 0.99615E+00_JPRB, 0.10972E+01_JPRB, 0.12768E+01_JPRB, 0.11705E+01_JPRB, 0.91416E+00_JPRB, 0.85718E+00_JPRB ,&
&  0.86262E+00_JPRB, 0.85701E+00_JPRB, 0.81951E+00_JPRB, 0.82448E+00_JPRB, 0.86442E+00_JPRB, 0.92575E+00_JPRB, 0.96442E+00_JPRB ,&
&  0.97889E+00_JPRB, 0.96728E+00_JPRB, 0.97305E+00_JPRB, 0.98825E+00_JPRB, 0.10228E+01_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.10122E+01_JPRB, 0.10296E+01_JPRB, 0.10461E+01_JPRB, 0.10813E+01_JPRB, 0.11350E+01_JPRB, 0.12085E+01_JPRB, 0.12116E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.81404E+00_JPRB, 0.82822E+00_JPRB, 0.85614E+00_JPRB, 0.91678E+00_JPRB, 0.97199E+00_JPRB, 0.10401E+01_JPRB ,&
&  0.11455E+01_JPRB, 0.12838E+01_JPRB, 0.15180E+01_JPRB, 0.18108E+01_JPRB, 0.21007E+01_JPRB, 0.24474E+01_JPRB, 0.23458E+01_JPRB ,&
&  0.18302E+01_JPRB, 0.17578E+01_JPRB, 0.21369E+01_JPRB, 0.22131E+01_JPRB, 0.22117E+01_JPRB, 0.25929E+01_JPRB, 0.24968E+01_JPRB ,&
&  0.19182E+01_JPRB, 0.17173E+01_JPRB, 0.17481E+01_JPRB, 0.16846E+01_JPRB, 0.16261E+01_JPRB, 0.16407E+01_JPRB, 0.16897E+01_JPRB ,&
&  0.17472E+01_JPRB, 0.17927E+01_JPRB, 0.18203E+01_JPRB, 0.18619E+01_JPRB, 0.19490E+01_JPRB, 0.20201E+01_JPRB, 0.21737E+01_JPRB ,&
&  0.23685E+01_JPRB, 0.25482E+01_JPRB, 0.26203E+01_JPRB, 0.25871E+01_JPRB, 0.26815E+01_JPRB, 0.28648E+01_JPRB, 0.30699E+01_JPRB ,&
&  0.32401E+01_JPRB, 0.34652E+01_JPRB, 0.35972E+01_JPRB, 0.36077E+01_JPRB, 0.33350E+01_JPRB, 0.36194E+01_JPRB, 0.32747E+01_JPRB ,&
&  0.32361E+01_JPRB, 0.29794E+01_JPRB, 0.32274E+01_JPRB, 0.25831E+01_JPRB, 0.27684E+01_JPRB, 0.26728E+01_JPRB, 0.24387E+01_JPRB ,&
&  0.23022E+01_JPRB, 0.24891E+01_JPRB, 0.24384E+01_JPRB, 0.22132E+01_JPRB, 0.20824E+01_JPRB, 0.17250E+01_JPRB, 0.16650E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.13061E+01_JPRB, 0.95810E+00_JPRB, 0.70543E+00_JPRB, 0.57468E+00_JPRB, 0.64098E+00_JPRB, 0.82458E+00_JPRB ,&
&  0.11348E+01_JPRB, 0.12041E+01_JPRB, 0.11258E+01_JPRB, 0.11081E+01_JPRB, 0.10338E+01_JPRB, 0.91314E+00_JPRB, 0.89608E+00_JPRB ,&
&  0.90109E+00_JPRB, 0.86744E+00_JPRB, 0.81299E+00_JPRB, 0.81085E+00_JPRB, 0.83919E+00_JPRB, 0.90294E+00_JPRB, 0.93217E+00_JPRB ,&
&  0.96786E+00_JPRB, 0.97358E+00_JPRB, 0.96913E+00_JPRB, 0.97141E+00_JPRB, 0.98635E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.99626E+00_JPRB, 0.99509E+00_JPRB, 0.96806E+00_JPRB, 0.93618E+00_JPRB, 0.91459E+00_JPRB, 0.90125E+00_JPRB, 0.91481E+00_JPRB ,&
&  0.85314E+00_JPRB, 0.83445E+00_JPRB, 0.82653E+00_JPRB, 0.81058E+00_JPRB, 0.81690E+00_JPRB, 0.82940E+00_JPRB, 0.85761E+00_JPRB ,&
&  0.89186E+00_JPRB, 0.92880E+00_JPRB, 0.10643E+01_JPRB, 0.13505E+01_JPRB, 0.16686E+01_JPRB, 0.20528E+01_JPRB, 0.19343E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.24824E+01_JPRB, 0.18968E+01_JPRB, 0.20338E+01_JPRB, 0.20988E+01_JPRB, 0.23078E+01_JPRB, 0.20071E+01_JPRB ,&
&  0.16649E+01_JPRB, 0.16027E+01_JPRB, 0.16588E+01_JPRB, 0.16230E+01_JPRB, 0.16295E+01_JPRB, 0.16236E+01_JPRB, 0.16352E+01_JPRB ,&
&  0.16752E+01_JPRB, 0.17187E+01_JPRB, 0.17452E+01_JPRB, 0.17778E+01_JPRB, 0.18367E+01_JPRB, 0.18944E+01_JPRB, 0.19896E+01_JPRB ,&
&  0.21619E+01_JPRB, 0.23764E+01_JPRB, 0.24779E+01_JPRB, 0.25977E+01_JPRB, 0.27501E+01_JPRB, 0.30707E+01_JPRB, 0.33293E+01_JPRB ,&
&  0.33979E+01_JPRB, 0.34019E+01_JPRB, 0.37871E+01_JPRB, 0.39086E+01_JPRB, 0.39061E+01_JPRB, 0.41107E+01_JPRB, 0.36061E+01_JPRB ,&
&  0.36487E+01_JPRB, 0.32517E+01_JPRB, 0.34846E+01_JPRB, 0.28892E+01_JPRB, 0.29445E+01_JPRB, 0.30770E+01_JPRB, 0.26728E+01_JPRB ,&
&  0.24998E+01_JPRB, 0.27209E+01_JPRB, 0.26446E+01_JPRB, 0.24277E+01_JPRB, 0.21752E+01_JPRB, 0.19599E+01_JPRB, 0.17393E+01_JPRB ,&
&  0.14882E+01_JPRB, 0.13909E+01_JPRB, 0.12406E+01_JPRB, 0.83719E+00_JPRB, 0.71440E+00_JPRB, 0.80022E+00_JPRB, 0.94044E+00_JPRB ,&
&  0.12804E+01_JPRB, 0.14352E+01_JPRB, 0.16451E+01_JPRB, 0.14532E+01_JPRB, 0.11200E+01_JPRB, 0.97293E+00_JPRB, 0.94803E+00_JPRB ,&
&  0.92401E+00_JPRB, 0.91462E+00_JPRB, 0.86827E+00_JPRB, 0.84043E+00_JPRB, 0.86583E+00_JPRB, 0.91794E+00_JPRB, 0.95519E+00_JPRB ,&
&  0.10061E+01_JPRB, 0.10387E+01_JPRB, 0.10345E+01_JPRB, 0.10103E+01_JPRB, 0.10021E+01_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.10647E+01_JPRB, 0.10473E+01_JPRB, 0.10278E+01_JPRB, 0.98160E+00_JPRB, 0.93876E+00_JPRB, 0.89250E+00_JPRB, 0.88525E+00_JPRB ,&
&  0.86297E+00_JPRB, 0.84783E+00_JPRB, 0.84043E+00_JPRB, 0.81600E+00_JPRB, 0.80376E+00_JPRB, 0.78378E+00_JPRB, 0.79173E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.77117E+00_JPRB, 0.80678E+00_JPRB, 0.97572E+00_JPRB, 0.12299E+01_JPRB, 0.16741E+01_JPRB, 0.13150E+01_JPRB ,&
&  0.17045E+01_JPRB, 0.23622E+01_JPRB, 0.19756E+01_JPRB, 0.20114E+01_JPRB, 0.20986E+01_JPRB, 0.21378E+01_JPRB, 0.17119E+01_JPRB ,&
&  0.15292E+01_JPRB, 0.15020E+01_JPRB, 0.15377E+01_JPRB, 0.15921E+01_JPRB, 0.16806E+01_JPRB, 0.17515E+01_JPRB, 0.17313E+01_JPRB ,&
&  0.17331E+01_JPRB, 0.17514E+01_JPRB, 0.17543E+01_JPRB, 0.17432E+01_JPRB, 0.17594E+01_JPRB, 0.18033E+01_JPRB, 0.18559E+01_JPRB ,&
&  0.19503E+01_JPRB, 0.21244E+01_JPRB, 0.24027E+01_JPRB, 0.24765E+01_JPRB, 0.27292E+01_JPRB, 0.32098E+01_JPRB, 0.35110E+01_JPRB ,&
&  0.34834E+01_JPRB, 0.34431E+01_JPRB, 0.40005E+01_JPRB, 0.45107E+01_JPRB, 0.46954E+01_JPRB, 0.46794E+01_JPRB, 0.42061E+01_JPRB ,&
&  0.39980E+01_JPRB, 0.36839E+01_JPRB, 0.35127E+01_JPRB, 0.29650E+01_JPRB, 0.30036E+01_JPRB, 0.33414E+01_JPRB, 0.32236E+01_JPRB ,&
&  0.33701E+01_JPRB, 0.31218E+01_JPRB, 0.29603E+01_JPRB, 0.25553E+01_JPRB, 0.22247E+01_JPRB, 0.20389E+01_JPRB, 0.20194E+01_JPRB ,&
&  0.19695E+01_JPRB, 0.19987E+01_JPRB, 0.18181E+01_JPRB, 0.11738E+01_JPRB, 0.93841E+00_JPRB, 0.93989E+00_JPRB, 0.11747E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.18108E+01_JPRB, 0.17271E+01_JPRB, 0.20908E+01_JPRB, 0.15507E+01_JPRB, 0.12496E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.10367E+01_JPRB, 0.10140E+01_JPRB, 0.96672E+00_JPRB, 0.97299E+00_JPRB, 0.10106E+01_JPRB, 0.10604E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11439E+01_JPRB, 0.11666E+01_JPRB, 0.11404E+01_JPRB, 0.10983E+01_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.11152E+01_JPRB, 0.10964E+01_JPRB, 0.10579E+01_JPRB, 0.10095E+01_JPRB, 0.97267E+00_JPRB, 0.91642E+00_JPRB, 0.87051E+00_JPRB ,&
&  0.86344E+00_JPRB, 0.84691E+00_JPRB, 0.83063E+00_JPRB, 0.81684E+00_JPRB, 0.80247E+00_JPRB, 0.78869E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.77970E+00_JPRB, 0.74905E+00_JPRB, 0.78264E+00_JPRB, 0.86181E+00_JPRB, 0.10133E+01_JPRB, 0.10904E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.19756E+01_JPRB, 0.24317E+01_JPRB, 0.21061E+01_JPRB, 0.22886E+01_JPRB, 0.25629E+01_JPRB, 0.28429E+01_JPRB, 0.20755E+01_JPRB ,&
&  0.17753E+01_JPRB, 0.16245E+01_JPRB, 0.15241E+01_JPRB, 0.16065E+01_JPRB, 0.17439E+01_JPRB, 0.18573E+01_JPRB, 0.18813E+01_JPRB ,&
&  0.18072E+01_JPRB, 0.17560E+01_JPRB, 0.17458E+01_JPRB, 0.17235E+01_JPRB, 0.17088E+01_JPRB, 0.17550E+01_JPRB, 0.17961E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.19534E+01_JPRB, 0.22025E+01_JPRB, 0.23547E+01_JPRB, 0.26363E+01_JPRB, 0.30889E+01_JPRB, 0.36089E+01_JPRB ,&
&  0.37912E+01_JPRB, 0.41000E+01_JPRB, 0.44687E+01_JPRB, 0.48059E+01_JPRB, 0.51838E+01_JPRB, 0.55137E+01_JPRB, 0.50225E+01_JPRB ,&
&  0.45820E+01_JPRB, 0.39131E+01_JPRB, 0.32935E+01_JPRB, 0.30770E+01_JPRB, 0.31331E+01_JPRB, 0.36252E+01_JPRB, 0.34631E+01_JPRB ,&
&  0.31979E+01_JPRB, 0.25815E+01_JPRB, 0.24285E+01_JPRB, 0.25562E+01_JPRB, 0.24000E+01_JPRB, 0.22257E+01_JPRB, 0.25188E+01_JPRB ,&
&  0.21827E+01_JPRB, 0.12592E+01_JPRB, 0.10626E+01_JPRB, 0.12393E+01_JPRB, 0.87669E+00_JPRB, 0.82679E+00_JPRB, 0.14944E+01_JPRB ,&
&  0.19592E+01_JPRB, 0.23927E+01_JPRB, 0.23163E+01_JPRB, 0.20202E+01_JPRB, 0.19349E+01_JPRB, 0.15923E+01_JPRB, 0.14292E+01_JPRB ,&
&  0.12234E+01_JPRB, 0.11802E+01_JPRB, 0.11566E+01_JPRB, 0.10997E+01_JPRB, 0.10868E+01_JPRB, 0.11410E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12399E+01_JPRB, 0.12524E+01_JPRB, 0.12355E+01_JPRB, 0.11991E+01_JPRB, 0.11483E+01_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.11628E+01_JPRB, 0.11366E+01_JPRB, 0.11129E+01_JPRB, 0.10644E+01_JPRB, 0.10185E+01_JPRB, 0.96462E+00_JPRB, 0.89426E+00_JPRB ,&
&  0.85827E+00_JPRB, 0.84267E+00_JPRB, 0.83812E+00_JPRB, 0.81994E+00_JPRB, 0.80491E+00_JPRB, 0.79992E+00_JPRB, 0.80439E+00_JPRB ,&
&  0.79002E+00_JPRB, 0.77436E+00_JPRB, 0.87090E+00_JPRB, 0.95004E+00_JPRB, 0.98568E+00_JPRB, 0.97591E+00_JPRB, 0.12747E+01_JPRB ,&
&  0.19748E+01_JPRB, 0.25949E+01_JPRB, 0.30069E+01_JPRB, 0.32006E+01_JPRB, 0.34295E+01_JPRB, 0.37168E+01_JPRB, 0.31780E+01_JPRB ,&
&  0.25703E+01_JPRB, 0.21006E+01_JPRB, 0.17735E+01_JPRB, 0.17582E+01_JPRB, 0.18998E+01_JPRB, 0.20143E+01_JPRB, 0.19820E+01_JPRB ,&
&  0.19118E+01_JPRB, 0.17966E+01_JPRB, 0.17238E+01_JPRB, 0.16639E+01_JPRB, 0.16640E+01_JPRB, 0.17139E+01_JPRB, 0.17581E+01_JPRB ,&
&  0.17815E+01_JPRB, 0.18121E+01_JPRB, 0.19944E+01_JPRB, 0.23222E+01_JPRB, 0.22806E+01_JPRB, 0.29608E+01_JPRB, 0.35363E+01_JPRB ,&
&  0.40440E+01_JPRB, 0.43428E+01_JPRB, 0.44237E+01_JPRB, 0.52550E+01_JPRB, 0.59082E+01_JPRB, 0.61031E+01_JPRB, 0.56897E+01_JPRB ,&
&  0.54189E+01_JPRB, 0.45690E+01_JPRB, 0.38304E+01_JPRB, 0.34895E+01_JPRB, 0.34495E+01_JPRB, 0.38844E+01_JPRB, 0.28006E+01_JPRB ,&
&  0.27711E+01_JPRB, 0.33208E+01_JPRB, 0.29130E+01_JPRB, 0.24765E+01_JPRB, 0.26221E+01_JPRB, 0.25114E+01_JPRB, 0.18550E+01_JPRB ,&
&  0.87387E+00_JPRB, 0.68483E+00_JPRB, 0.68669E+00_JPRB, 0.66819E+00_JPRB, 0.61531E+00_JPRB, 0.85991E+00_JPRB, 0.24974E+01_JPRB ,&
&  0.21488E+01_JPRB, 0.27967E+01_JPRB, 0.26646E+01_JPRB, 0.23072E+01_JPRB, 0.20690E+01_JPRB, 0.17995E+01_JPRB, 0.18670E+01_JPRB ,&
&  0.15856E+01_JPRB, 0.14138E+01_JPRB, 0.13267E+01_JPRB, 0.12626E+01_JPRB, 0.12533E+01_JPRB, 0.12667E+01_JPRB, 0.13284E+01_JPRB ,&
&  0.13691E+01_JPRB, 0.13798E+01_JPRB, 0.13389E+01_JPRB, 0.12716E+01_JPRB, 0.12091E+01_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.12230E+01_JPRB, 0.11644E+01_JPRB, 0.11238E+01_JPRB, 0.10958E+01_JPRB, 0.10656E+01_JPRB, 0.10117E+01_JPRB, 0.92454E+00_JPRB ,&
&  0.86266E+00_JPRB, 0.83780E+00_JPRB, 0.83218E+00_JPRB, 0.82499E+00_JPRB, 0.82231E+00_JPRB, 0.82288E+00_JPRB, 0.82029E+00_JPRB ,&
&  0.79551E+00_JPRB, 0.81580E+00_JPRB, 0.97953E+00_JPRB, 0.10169E+01_JPRB, 0.10320E+01_JPRB, 0.98868E+00_JPRB, 0.12513E+01_JPRB ,&
&  0.19373E+01_JPRB, 0.26291E+01_JPRB, 0.31035E+01_JPRB, 0.32361E+01_JPRB, 0.42156E+01_JPRB, 0.48129E+01_JPRB, 0.47931E+01_JPRB ,&
&  0.39550E+01_JPRB, 0.31956E+01_JPRB, 0.25296E+01_JPRB, 0.21743E+01_JPRB, 0.21597E+01_JPRB, 0.21910E+01_JPRB, 0.21569E+01_JPRB ,&
&  0.20269E+01_JPRB, 0.19107E+01_JPRB, 0.17999E+01_JPRB, 0.17162E+01_JPRB, 0.16878E+01_JPRB, 0.17145E+01_JPRB, 0.17732E+01_JPRB ,&
&  0.17888E+01_JPRB, 0.17483E+01_JPRB, 0.17974E+01_JPRB, 0.21307E+01_JPRB, 0.24680E+01_JPRB, 0.26863E+01_JPRB, 0.33496E+01_JPRB ,&
&  0.40795E+01_JPRB, 0.46599E+01_JPRB, 0.47150E+01_JPRB, 0.59864E+01_JPRB, 0.63306E+01_JPRB, 0.64360E+01_JPRB, 0.66042E+01_JPRB ,&
&  0.62612E+01_JPRB, 0.60970E+01_JPRB, 0.43819E+01_JPRB, 0.38643E+01_JPRB, 0.39187E+01_JPRB, 0.32925E+01_JPRB, 0.32433E+01_JPRB ,&
&  0.38490E+01_JPRB, 0.33538E+01_JPRB, 0.28157E+01_JPRB, 0.23477E+01_JPRB, 0.27682E+01_JPRB, 0.22652E+01_JPRB, 0.10346E+01_JPRB ,&
&  0.81338E+00_JPRB, 0.75949E+00_JPRB, 0.66834E+00_JPRB, 0.64772E+00_JPRB, 0.66506E+00_JPRB, 0.90205E+00_JPRB, 0.16979E+01_JPRB ,&
&  0.19705E+01_JPRB, 0.26108E+01_JPRB, 0.26353E+01_JPRB, 0.24417E+01_JPRB, 0.21188E+01_JPRB, 0.20961E+01_JPRB, 0.24749E+01_JPRB ,&
&  0.22009E+01_JPRB, 0.19528E+01_JPRB, 0.16524E+01_JPRB, 0.14737E+01_JPRB, 0.13830E+01_JPRB, 0.13736E+01_JPRB, 0.14340E+01_JPRB ,&
&  0.14602E+01_JPRB, 0.14598E+01_JPRB, 0.14184E+01_JPRB, 0.13486E+01_JPRB, 0.12775E+01_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.12649E+01_JPRB, 0.12155E+01_JPRB, 0.11588E+01_JPRB, 0.11052E+01_JPRB, 0.10645E+01_JPRB, 0.10275E+01_JPRB, 0.97294E+00_JPRB ,&
&  0.90611E+00_JPRB, 0.86669E+00_JPRB, 0.85129E+00_JPRB, 0.83435E+00_JPRB, 0.82836E+00_JPRB, 0.83223E+00_JPRB, 0.82723E+00_JPRB ,&
&  0.81023E+00_JPRB, 0.85626E+00_JPRB, 0.95543E+00_JPRB, 0.93134E+00_JPRB, 0.92101E+00_JPRB, 0.89689E+00_JPRB, 0.10939E+01_JPRB ,&
&  0.17848E+01_JPRB, 0.25220E+01_JPRB, 0.29784E+01_JPRB, 0.34730E+01_JPRB, 0.45286E+01_JPRB, 0.47085E+01_JPRB, 0.54046E+01_JPRB ,&
&  0.48928E+01_JPRB, 0.41938E+01_JPRB, 0.35756E+01_JPRB, 0.31171E+01_JPRB, 0.28052E+01_JPRB, 0.25635E+01_JPRB, 0.24196E+01_JPRB ,&
&  0.22570E+01_JPRB, 0.20717E+01_JPRB, 0.19867E+01_JPRB, 0.18840E+01_JPRB, 0.18219E+01_JPRB, 0.17957E+01_JPRB, 0.18557E+01_JPRB ,&
&  0.18589E+01_JPRB, 0.17937E+01_JPRB, 0.17042E+01_JPRB, 0.18836E+01_JPRB, 0.25147E+01_JPRB, 0.30436E+01_JPRB, 0.32577E+01_JPRB ,&
&  0.35067E+01_JPRB, 0.37910E+01_JPRB, 0.48945E+01_JPRB, 0.58479E+01_JPRB, 0.65650E+01_JPRB, 0.71703E+01_JPRB, 0.71566E+01_JPRB ,&
&  0.68063E+01_JPRB, 0.62292E+01_JPRB, 0.47582E+01_JPRB, 0.42858E+01_JPRB, 0.37881E+01_JPRB, 0.32796E+01_JPRB, 0.39454E+01_JPRB ,&
&  0.43257E+01_JPRB, 0.38525E+01_JPRB, 0.30655E+01_JPRB, 0.24231E+01_JPRB, 0.22019E+01_JPRB, 0.14694E+01_JPRB, 0.97694E+00_JPRB ,&
&  0.91964E+00_JPRB, 0.84851E+00_JPRB, 0.76898E+00_JPRB, 0.83005E+00_JPRB, 0.80541E+00_JPRB, 0.10277E+01_JPRB, 0.16041E+01_JPRB ,&
&  0.21961E+01_JPRB, 0.25805E+01_JPRB, 0.28642E+01_JPRB, 0.25665E+01_JPRB, 0.23113E+01_JPRB, 0.22973E+01_JPRB, 0.28071E+01_JPRB ,&
&  0.23077E+01_JPRB, 0.22618E+01_JPRB, 0.21268E+01_JPRB, 0.18237E+01_JPRB, 0.16380E+01_JPRB, 0.15474E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.15383E+01_JPRB, 0.15066E+01_JPRB, 0.14482E+01_JPRB, 0.13741E+01_JPRB, 0.13115E+01_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.12669E+01_JPRB, 0.12037E+01_JPRB, 0.11464E+01_JPRB, 0.10901E+01_JPRB, 0.10393E+01_JPRB, 0.10017E+01_JPRB, 0.97465E+00_JPRB ,&
&  0.93611E+00_JPRB, 0.90049E+00_JPRB, 0.86261E+00_JPRB, 0.83522E+00_JPRB, 0.82234E+00_JPRB, 0.81724E+00_JPRB, 0.80734E+00_JPRB ,&
&  0.80278E+00_JPRB, 0.87862E+00_JPRB, 0.73442E+00_JPRB, 0.69962E+00_JPRB, 0.78583E+00_JPRB, 0.84724E+00_JPRB, 0.92381E+00_JPRB ,&
&  0.14590E+01_JPRB, 0.22295E+01_JPRB, 0.27336E+01_JPRB, 0.37099E+01_JPRB, 0.56965E+01_JPRB, 0.54364E+01_JPRB, 0.55480E+01_JPRB ,&
&  0.56110E+01_JPRB, 0.47336E+01_JPRB, 0.41483E+01_JPRB, 0.36141E+01_JPRB, 0.31887E+01_JPRB, 0.28365E+01_JPRB, 0.25065E+01_JPRB ,&
&  0.22284E+01_JPRB, 0.20449E+01_JPRB, 0.19485E+01_JPRB, 0.19319E+01_JPRB, 0.19263E+01_JPRB, 0.19370E+01_JPRB, 0.19456E+01_JPRB ,&
&  0.19333E+01_JPRB, 0.18801E+01_JPRB, 0.17617E+01_JPRB, 0.18499E+01_JPRB, 0.22123E+01_JPRB, 0.27440E+01_JPRB, 0.38714E+01_JPRB ,&
&  0.38807E+01_JPRB, 0.42567E+01_JPRB, 0.44257E+01_JPRB, 0.61624E+01_JPRB, 0.72137E+01_JPRB, 0.72743E+01_JPRB, 0.77949E+01_JPRB ,&
&  0.59799E+01_JPRB, 0.50153E+01_JPRB, 0.44701E+01_JPRB, 0.39895E+01_JPRB, 0.37297E+01_JPRB, 0.44069E+01_JPRB, 0.62513E+01_JPRB ,&
&  0.47800E+01_JPRB, 0.49876E+01_JPRB, 0.50507E+01_JPRB, 0.46272E+01_JPRB, 0.23401E+01_JPRB, 0.16355E+01_JPRB, 0.26000E+01_JPRB ,&
&  0.27224E+01_JPRB, 0.16004E+01_JPRB, 0.11583E+01_JPRB, 0.14978E+01_JPRB, 0.11361E+01_JPRB, 0.13010E+01_JPRB, 0.17547E+01_JPRB ,&
&  0.19215E+01_JPRB, 0.22605E+01_JPRB, 0.31838E+01_JPRB, 0.30192E+01_JPRB, 0.27494E+01_JPRB, 0.26079E+01_JPRB, 0.25887E+01_JPRB ,&
&  0.23982E+01_JPRB, 0.21504E+01_JPRB, 0.20822E+01_JPRB, 0.18179E+01_JPRB, 0.16583E+01_JPRB, 0.15522E+01_JPRB, 0.14967E+01_JPRB ,&
&  0.14846E+01_JPRB, 0.14629E+01_JPRB, 0.14220E+01_JPRB, 0.13648E+01_JPRB, 0.13163E+01_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.12698E+01_JPRB, 0.12032E+01_JPRB, 0.11341E+01_JPRB, 0.10809E+01_JPRB, 0.10333E+01_JPRB, 0.98822E+00_JPRB, 0.95045E+00_JPRB ,&
&  0.92583E+00_JPRB, 0.89924E+00_JPRB, 0.85518E+00_JPRB, 0.82397E+00_JPRB, 0.80749E+00_JPRB, 0.80933E+00_JPRB, 0.80450E+00_JPRB ,&
&  0.80386E+00_JPRB, 0.76917E+00_JPRB, 0.64589E+00_JPRB, 0.61032E+00_JPRB, 0.73522E+00_JPRB, 0.73831E+00_JPRB, 0.79808E+00_JPRB ,&
&  0.11602E+01_JPRB, 0.18880E+01_JPRB, 0.26920E+01_JPRB, 0.37063E+01_JPRB, 0.51417E+01_JPRB, 0.59606E+01_JPRB, 0.57943E+01_JPRB ,&
&  0.58975E+01_JPRB, 0.50786E+01_JPRB, 0.44266E+01_JPRB, 0.38823E+01_JPRB, 0.33422E+01_JPRB, 0.28059E+01_JPRB, 0.23516E+01_JPRB ,&
&  0.20829E+01_JPRB, 0.19058E+01_JPRB, 0.18047E+01_JPRB, 0.17167E+01_JPRB, 0.17392E+01_JPRB, 0.18095E+01_JPRB, 0.18898E+01_JPRB ,&
&  0.19141E+01_JPRB, 0.18954E+01_JPRB, 0.18473E+01_JPRB, 0.18457E+01_JPRB, 0.22043E+01_JPRB, 0.27852E+01_JPRB, 0.32983E+01_JPRB ,&
&  0.37463E+01_JPRB, 0.41667E+01_JPRB, 0.46982E+01_JPRB, 0.56709E+01_JPRB, 0.72084E+01_JPRB, 0.72092E+01_JPRB, 0.83791E+01_JPRB ,&
&  0.65653E+01_JPRB, 0.53911E+01_JPRB, 0.43896E+01_JPRB, 0.33266E+01_JPRB, 0.41106E+01_JPRB, 0.67483E+01_JPRB, 0.65765E+01_JPRB ,&
&  0.60532E+01_JPRB, 0.59445E+01_JPRB, 0.55364E+01_JPRB, 0.44634E+01_JPRB, 0.37345E+01_JPRB, 0.28899E+01_JPRB, 0.34845E+01_JPRB ,&
&  0.33707E+01_JPRB, 0.31593E+01_JPRB, 0.28032E+01_JPRB, 0.23110E+01_JPRB, 0.18912E+01_JPRB, 0.20477E+01_JPRB, 0.18502E+01_JPRB ,&
&  0.17141E+01_JPRB, 0.18222E+01_JPRB, 0.28015E+01_JPRB, 0.32962E+01_JPRB, 0.30815E+01_JPRB, 0.26204E+01_JPRB, 0.28072E+01_JPRB ,&
&  0.24619E+01_JPRB, 0.22430E+01_JPRB, 0.19628E+01_JPRB, 0.17195E+01_JPRB, 0.15966E+01_JPRB, 0.15222E+01_JPRB, 0.14751E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.13959E+01_JPRB, 0.13643E+01_JPRB, 0.13316E+01_JPRB, 0.13043E+01_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.12553E+01_JPRB, 0.11958E+01_JPRB, 0.11360E+01_JPRB, 0.10866E+01_JPRB, 0.10358E+01_JPRB, 0.98932E+00_JPRB, 0.93999E+00_JPRB ,&
&  0.90402E+00_JPRB, 0.88447E+00_JPRB, 0.85919E+00_JPRB, 0.83792E+00_JPRB, 0.82590E+00_JPRB, 0.81870E+00_JPRB, 0.82190E+00_JPRB ,&
&  0.83054E+00_JPRB, 0.78098E+00_JPRB, 0.62610E+00_JPRB, 0.60241E+00_JPRB, 0.63436E+00_JPRB, 0.62924E+00_JPRB, 0.74289E+00_JPRB ,&
&  0.99657E+00_JPRB, 0.15243E+01_JPRB, 0.22058E+01_JPRB, 0.30984E+01_JPRB, 0.41046E+01_JPRB, 0.49069E+01_JPRB, 0.50844E+01_JPRB ,&
&  0.45059E+01_JPRB, 0.44298E+01_JPRB, 0.38538E+01_JPRB, 0.33370E+01_JPRB, 0.27909E+01_JPRB, 0.23806E+01_JPRB, 0.20818E+01_JPRB ,&
&  0.18245E+01_JPRB, 0.16567E+01_JPRB, 0.15788E+01_JPRB, 0.15579E+01_JPRB, 0.15853E+01_JPRB, 0.16458E+01_JPRB, 0.17357E+01_JPRB ,&
&  0.17721E+01_JPRB, 0.17803E+01_JPRB, 0.17474E+01_JPRB, 0.18445E+01_JPRB, 0.20141E+01_JPRB, 0.27309E+01_JPRB, 0.30679E+01_JPRB ,&
&  0.40107E+01_JPRB, 0.48304E+01_JPRB, 0.53267E+01_JPRB, 0.69072E+01_JPRB, 0.60000E+01_JPRB, 0.72988E+01_JPRB, 0.90484E+01_JPRB ,&
&  0.80617E+01_JPRB, 0.71152E+01_JPRB, 0.67781E+01_JPRB, 0.55492E+01_JPRB, 0.47872E+01_JPRB, 0.67174E+01_JPRB, 0.62231E+01_JPRB ,&
&  0.56141E+01_JPRB, 0.54670E+01_JPRB, 0.49814E+01_JPRB, 0.47635E+01_JPRB, 0.41999E+01_JPRB, 0.37547E+01_JPRB, 0.27948E+01_JPRB ,&
&  0.27735E+01_JPRB, 0.25148E+01_JPRB, 0.27027E+01_JPRB, 0.24260E+01_JPRB, 0.22443E+01_JPRB, 0.20621E+01_JPRB, 0.18295E+01_JPRB ,&
&  0.16742E+01_JPRB, 0.16345E+01_JPRB, 0.20165E+01_JPRB, 0.24395E+01_JPRB, 0.26275E+01_JPRB, 0.22935E+01_JPRB, 0.24773E+01_JPRB ,&
&  0.23847E+01_JPRB, 0.22279E+01_JPRB, 0.19495E+01_JPRB, 0.18336E+01_JPRB, 0.17172E+01_JPRB, 0.16120E+01_JPRB, 0.15302E+01_JPRB ,&
&  0.14497E+01_JPRB, 0.13844E+01_JPRB, 0.13431E+01_JPRB, 0.13263E+01_JPRB, 0.12986E+01_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.11854E+01_JPRB, 0.11358E+01_JPRB, 0.10867E+01_JPRB, 0.10490E+01_JPRB, 0.10207E+01_JPRB, 0.99197E+00_JPRB, 0.94390E+00_JPRB ,&
&  0.89006E+00_JPRB, 0.86157E+00_JPRB, 0.84696E+00_JPRB, 0.83179E+00_JPRB, 0.82211E+00_JPRB, 0.82293E+00_JPRB, 0.82881E+00_JPRB ,&
&  0.84390E+00_JPRB, 0.94280E+00_JPRB, 0.74155E+00_JPRB, 0.60423E+00_JPRB, 0.53811E+00_JPRB, 0.61024E+00_JPRB, 0.76793E+00_JPRB ,&
&  0.98324E+00_JPRB, 0.13432E+01_JPRB, 0.19275E+01_JPRB, 0.24146E+01_JPRB, 0.29555E+01_JPRB, 0.35061E+01_JPRB, 0.37002E+01_JPRB ,&
&  0.39961E+01_JPRB, 0.31766E+01_JPRB, 0.28625E+01_JPRB, 0.23617E+01_JPRB, 0.20206E+01_JPRB, 0.17980E+01_JPRB, 0.16049E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.13517E+01_JPRB, 0.12999E+01_JPRB, 0.13071E+01_JPRB, 0.13237E+01_JPRB, 0.13656E+01_JPRB, 0.14395E+01_JPRB ,&
&  0.15244E+01_JPRB, 0.15704E+01_JPRB, 0.15602E+01_JPRB, 0.16297E+01_JPRB, 0.19941E+01_JPRB, 0.25083E+01_JPRB, 0.29818E+01_JPRB ,&
&  0.31071E+01_JPRB, 0.36218E+01_JPRB, 0.69062E+01_JPRB, 0.77802E+01_JPRB, 0.81844E+01_JPRB, 0.86749E+01_JPRB, 0.86582E+01_JPRB ,&
&  0.79938E+01_JPRB, 0.73190E+01_JPRB, 0.69542E+01_JPRB, 0.60742E+01_JPRB, 0.67655E+01_JPRB, 0.66527E+01_JPRB, 0.62508E+01_JPRB ,&
&  0.55995E+01_JPRB, 0.52192E+01_JPRB, 0.47977E+01_JPRB, 0.45324E+01_JPRB, 0.44125E+01_JPRB, 0.44613E+01_JPRB, 0.41325E+01_JPRB ,&
&  0.34644E+01_JPRB, 0.36398E+01_JPRB, 0.30745E+01_JPRB, 0.24805E+01_JPRB, 0.20162E+01_JPRB, 0.18736E+01_JPRB, 0.17545E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.15340E+01_JPRB, 0.16472E+01_JPRB, 0.17890E+01_JPRB, 0.19465E+01_JPRB, 0.20179E+01_JPRB, 0.19422E+01_JPRB ,&
&  0.17822E+01_JPRB, 0.19733E+01_JPRB, 0.20118E+01_JPRB, 0.19223E+01_JPRB, 0.18561E+01_JPRB, 0.17174E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.13914E+01_JPRB, 0.13353E+01_JPRB, 0.12881E+01_JPRB, 0.12381E+01_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.10631E+01_JPRB, 0.10323E+01_JPRB, 0.10055E+01_JPRB, 0.97307E+00_JPRB, 0.95394E+00_JPRB, 0.93828E+00_JPRB, 0.91191E+00_JPRB ,&
&  0.87557E+00_JPRB, 0.84121E+00_JPRB, 0.81814E+00_JPRB, 0.80559E+00_JPRB, 0.80055E+00_JPRB, 0.80517E+00_JPRB, 0.80642E+00_JPRB ,&
&  0.83156E+00_JPRB, 0.11356E+01_JPRB, 0.88499E+00_JPRB, 0.75013E+00_JPRB, 0.66506E+00_JPRB, 0.74853E+00_JPRB, 0.86561E+00_JPRB ,&
&  0.10559E+01_JPRB, 0.11913E+01_JPRB, 0.14309E+01_JPRB, 0.18130E+01_JPRB, 0.21483E+01_JPRB, 0.24285E+01_JPRB, 0.26063E+01_JPRB ,&
&  0.24240E+01_JPRB, 0.22556E+01_JPRB, 0.17532E+01_JPRB, 0.16268E+01_JPRB, 0.14478E+01_JPRB, 0.12372E+01_JPRB, 0.12731E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12018E+01_JPRB, 0.11875E+01_JPRB, 0.12040E+01_JPRB, 0.12280E+01_JPRB, 0.12569E+01_JPRB, 0.12952E+01_JPRB ,&
&  0.13334E+01_JPRB, 0.13785E+01_JPRB, 0.14207E+01_JPRB, 0.14833E+01_JPRB, 0.16807E+01_JPRB, 0.21177E+01_JPRB, 0.24798E+01_JPRB ,&
&  0.32356E+01_JPRB, 0.34196E+01_JPRB, 0.37089E+01_JPRB, 0.56583E+01_JPRB, 0.83271E+01_JPRB, 0.81307E+01_JPRB, 0.72303E+01_JPRB ,&
&  0.70578E+01_JPRB, 0.70079E+01_JPRB, 0.70799E+01_JPRB, 0.67635E+01_JPRB, 0.61389E+01_JPRB, 0.58129E+01_JPRB, 0.54548E+01_JPRB ,&
&  0.48922E+01_JPRB, 0.44853E+01_JPRB, 0.42584E+01_JPRB, 0.41903E+01_JPRB, 0.39708E+01_JPRB, 0.38924E+01_JPRB, 0.38938E+01_JPRB ,&
&  0.37571E+01_JPRB, 0.31831E+01_JPRB, 0.20887E+01_JPRB, 0.20422E+01_JPRB, 0.15833E+01_JPRB, 0.14056E+01_JPRB, 0.14619E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.13956E+01_JPRB, 0.14968E+01_JPRB, 0.14409E+01_JPRB, 0.16680E+01_JPRB, 0.16357E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.14678E+01_JPRB, 0.13535E+01_JPRB, 0.16524E+01_JPRB, 0.17256E+01_JPRB, 0.17092E+01_JPRB, 0.16061E+01_JPRB, 0.14616E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.12062E+01_JPRB, 0.11615E+01_JPRB, 0.11359E+01_JPRB, 0.10983E+01_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.94134E+00_JPRB, 0.92244E+00_JPRB, 0.89593E+00_JPRB, 0.85693E+00_JPRB, 0.83498E+00_JPRB, 0.83341E+00_JPRB, 0.84699E+00_JPRB ,&
&  0.84753E+00_JPRB, 0.82111E+00_JPRB, 0.80094E+00_JPRB, 0.78258E+00_JPRB, 0.77389E+00_JPRB, 0.77503E+00_JPRB, 0.78560E+00_JPRB ,&
&  0.87328E+00_JPRB, 0.12007E+01_JPRB, 0.97573E+00_JPRB, 0.80043E+00_JPRB, 0.95536E+00_JPRB, 0.90579E+00_JPRB, 0.96015E+00_JPRB ,&
&  0.94792E+00_JPRB, 0.11203E+01_JPRB, 0.11472E+01_JPRB, 0.12699E+01_JPRB, 0.14317E+01_JPRB, 0.16857E+01_JPRB, 0.17574E+01_JPRB ,&
&  0.15894E+01_JPRB, 0.12949E+01_JPRB, 0.10588E+01_JPRB, 0.99701E+00_JPRB, 0.10418E+01_JPRB, 0.94100E+00_JPRB, 0.99417E+00_JPRB ,&
&  0.10051E+01_JPRB, 0.10215E+01_JPRB, 0.10358E+01_JPRB, 0.10417E+01_JPRB, 0.10694E+01_JPRB, 0.11184E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.13176E+01_JPRB, 0.13525E+01_JPRB, 0.14111E+01_JPRB, 0.16119E+01_JPRB, 0.21157E+01_JPRB, 0.26403E+01_JPRB ,&
&  0.33833E+01_JPRB, 0.39308E+01_JPRB, 0.48735E+01_JPRB, 0.64649E+01_JPRB, 0.70658E+01_JPRB, 0.71480E+01_JPRB, 0.67813E+01_JPRB ,&
&  0.61489E+01_JPRB, 0.55435E+01_JPRB, 0.53342E+01_JPRB, 0.47877E+01_JPRB, 0.44789E+01_JPRB, 0.44288E+01_JPRB, 0.42971E+01_JPRB ,&
&  0.40016E+01_JPRB, 0.37727E+01_JPRB, 0.36982E+01_JPRB, 0.35221E+01_JPRB, 0.34108E+01_JPRB, 0.32482E+01_JPRB, 0.32690E+01_JPRB ,&
&  0.35864E+01_JPRB, 0.22814E+01_JPRB, 0.17020E+01_JPRB, 0.18149E+01_JPRB, 0.14514E+01_JPRB, 0.12834E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.12547E+01_JPRB, 0.12574E+01_JPRB, 0.13668E+01_JPRB, 0.12460E+01_JPRB, 0.12717E+01_JPRB, 0.13608E+01_JPRB, 0.11566E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.10686E+01_JPRB, 0.12457E+01_JPRB, 0.13631E+01_JPRB, 0.13814E+01_JPRB, 0.12985E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.10044E+01_JPRB, 0.98584E+00_JPRB, 0.97009E+00_JPRB, 0.96021E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.82401E+00_JPRB, 0.81392E+00_JPRB, 0.80128E+00_JPRB, 0.79167E+00_JPRB, 0.78762E+00_JPRB, 0.78467E+00_JPRB, 0.79751E+00_JPRB ,&
&  0.80906E+00_JPRB, 0.79643E+00_JPRB, 0.77450E+00_JPRB, 0.75581E+00_JPRB, 0.73592E+00_JPRB, 0.73236E+00_JPRB, 0.75847E+00_JPRB ,&
&  0.67236E+00_JPRB, 0.85971E+00_JPRB, 0.72370E+00_JPRB, 0.78499E+00_JPRB, 0.11878E+01_JPRB, 0.10213E+01_JPRB, 0.97419E+00_JPRB ,&
&  0.88294E+00_JPRB, 0.87564E+00_JPRB, 0.86814E+00_JPRB, 0.96577E+00_JPRB, 0.10433E+01_JPRB, 0.11538E+01_JPRB, 0.11974E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.82659E+00_JPRB, 0.72110E+00_JPRB, 0.68852E+00_JPRB, 0.73023E+00_JPRB, 0.76415E+00_JPRB, 0.84080E+00_JPRB ,&
&  0.87341E+00_JPRB, 0.89447E+00_JPRB, 0.91333E+00_JPRB, 0.94704E+00_JPRB, 0.96459E+00_JPRB, 0.99081E+00_JPRB, 0.10349E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.11685E+01_JPRB, 0.12465E+01_JPRB, 0.13909E+01_JPRB, 0.15212E+01_JPRB, 0.21906E+01_JPRB, 0.33859E+01_JPRB ,&
&  0.40411E+01_JPRB, 0.42953E+01_JPRB, 0.51872E+01_JPRB, 0.68143E+01_JPRB, 0.70953E+01_JPRB, 0.66973E+01_JPRB, 0.58965E+01_JPRB ,&
&  0.51519E+01_JPRB, 0.44482E+01_JPRB, 0.42963E+01_JPRB, 0.40481E+01_JPRB, 0.37273E+01_JPRB, 0.38552E+01_JPRB, 0.37614E+01_JPRB ,&
&  0.35550E+01_JPRB, 0.35092E+01_JPRB, 0.34436E+01_JPRB, 0.34538E+01_JPRB, 0.32505E+01_JPRB, 0.32506E+01_JPRB, 0.31246E+01_JPRB ,&
&  0.31293E+01_JPRB, 0.28217E+01_JPRB, 0.20593E+01_JPRB, 0.16922E+01_JPRB, 0.13491E+01_JPRB, 0.12643E+01_JPRB, 0.15696E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.12373E+01_JPRB, 0.11284E+01_JPRB, 0.10020E+01_JPRB, 0.94874E+00_JPRB, 0.10034E+01_JPRB, 0.87580E+00_JPRB ,&
&  0.82382E+00_JPRB, 0.90264E+00_JPRB, 0.99534E+00_JPRB, 0.10613E+01_JPRB, 0.10887E+01_JPRB, 0.10688E+01_JPRB, 0.91592E+00_JPRB ,&
&  0.89605E+00_JPRB, 0.86128E+00_JPRB, 0.84855E+00_JPRB, 0.82982E+00_JPRB, 0.82324E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.75401E+00_JPRB, 0.75224E+00_JPRB, 0.74792E+00_JPRB, 0.75506E+00_JPRB, 0.77090E+00_JPRB, 0.77097E+00_JPRB, 0.77135E+00_JPRB ,&
&  0.78229E+00_JPRB, 0.77368E+00_JPRB, 0.74730E+00_JPRB, 0.73030E+00_JPRB, 0.71142E+00_JPRB, 0.70903E+00_JPRB, 0.63389E+00_JPRB ,&
&  0.59414E+00_JPRB, 0.69583E+00_JPRB, 0.68197E+00_JPRB, 0.75883E+00_JPRB, 0.88509E+00_JPRB, 0.85805E+00_JPRB, 0.85852E+00_JPRB ,&
&  0.82861E+00_JPRB, 0.76681E+00_JPRB, 0.74590E+00_JPRB, 0.75978E+00_JPRB, 0.78559E+00_JPRB, 0.85303E+00_JPRB, 0.91293E+00_JPRB ,&
&  0.81876E+00_JPRB, 0.67857E+00_JPRB, 0.60848E+00_JPRB, 0.60410E+00_JPRB, 0.66326E+00_JPRB, 0.71720E+00_JPRB, 0.74645E+00_JPRB ,&
&  0.76988E+00_JPRB, 0.79657E+00_JPRB, 0.83481E+00_JPRB, 0.88395E+00_JPRB, 0.90642E+00_JPRB, 0.92764E+00_JPRB, 0.95531E+00_JPRB ,&
&  0.98933E+00_JPRB, 0.10476E+01_JPRB, 0.11295E+01_JPRB, 0.12765E+01_JPRB, 0.14089E+01_JPRB, 0.20784E+01_JPRB, 0.30586E+01_JPRB ,&
&  0.35118E+01_JPRB, 0.38771E+01_JPRB, 0.44334E+01_JPRB, 0.51269E+01_JPRB, 0.53395E+01_JPRB, 0.49224E+01_JPRB, 0.44788E+01_JPRB ,&
&  0.42443E+01_JPRB, 0.38004E+01_JPRB, 0.41484E+01_JPRB, 0.38381E+01_JPRB, 0.35878E+01_JPRB, 0.35826E+01_JPRB, 0.34272E+01_JPRB ,&
&  0.33732E+01_JPRB, 0.34054E+01_JPRB, 0.33017E+01_JPRB, 0.31896E+01_JPRB, 0.29260E+01_JPRB, 0.28567E+01_JPRB, 0.26464E+01_JPRB ,&
&  0.26565E+01_JPRB, 0.25980E+01_JPRB, 0.21957E+01_JPRB, 0.18295E+01_JPRB, 0.14797E+01_JPRB, 0.14168E+01_JPRB, 0.12741E+01_JPRB ,&
&  0.11643E+01_JPRB, 0.10210E+01_JPRB, 0.92472E+00_JPRB, 0.80912E+00_JPRB, 0.75571E+00_JPRB, 0.73157E+00_JPRB, 0.71421E+00_JPRB ,&
&  0.77684E+00_JPRB, 0.88453E+00_JPRB, 0.89826E+00_JPRB, 0.89273E+00_JPRB, 0.89938E+00_JPRB, 0.92058E+00_JPRB, 0.74369E+00_JPRB ,&
&  0.72188E+00_JPRB, 0.78260E+00_JPRB, 0.77167E+00_JPRB, 0.75705E+00_JPRB, 0.74909E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.70740E+00_JPRB, 0.70910E+00_JPRB, 0.71361E+00_JPRB, 0.73156E+00_JPRB, 0.75342E+00_JPRB, 0.76118E+00_JPRB, 0.72171E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.73708E+00_JPRB, 0.71817E+00_JPRB, 0.70994E+00_JPRB, 0.69634E+00_JPRB, 0.60997E+00_JPRB, 0.52384E+00_JPRB ,&
&  0.51778E+00_JPRB, 0.55971E+00_JPRB, 0.63372E+00_JPRB, 0.68833E+00_JPRB, 0.74795E+00_JPRB, 0.75034E+00_JPRB, 0.74279E+00_JPRB ,&
&  0.72929E+00_JPRB, 0.69680E+00_JPRB, 0.69220E+00_JPRB, 0.68845E+00_JPRB, 0.67961E+00_JPRB, 0.68462E+00_JPRB, 0.70391E+00_JPRB ,&
&  0.65161E+00_JPRB, 0.61276E+00_JPRB, 0.60890E+00_JPRB, 0.59058E+00_JPRB, 0.65331E+00_JPRB, 0.67411E+00_JPRB, 0.69593E+00_JPRB ,&
&  0.72501E+00_JPRB, 0.76237E+00_JPRB, 0.82518E+00_JPRB, 0.87424E+00_JPRB, 0.90746E+00_JPRB, 0.92743E+00_JPRB, 0.95986E+00_JPRB ,&
&  0.98426E+00_JPRB, 0.10304E+01_JPRB, 0.11090E+01_JPRB, 0.12529E+01_JPRB, 0.14911E+01_JPRB, 0.19228E+01_JPRB, 0.28450E+01_JPRB ,&
&  0.31487E+01_JPRB, 0.35849E+01_JPRB, 0.40465E+01_JPRB, 0.37191E+01_JPRB, 0.40420E+01_JPRB, 0.42641E+01_JPRB, 0.40981E+01_JPRB ,&
&  0.40774E+01_JPRB, 0.36323E+01_JPRB, 0.35085E+01_JPRB, 0.34142E+01_JPRB, 0.33677E+01_JPRB, 0.30916E+01_JPRB, 0.28357E+01_JPRB ,&
&  0.29620E+01_JPRB, 0.32349E+01_JPRB, 0.26236E+01_JPRB, 0.23453E+01_JPRB, 0.21114E+01_JPRB, 0.20328E+01_JPRB, 0.19071E+01_JPRB ,&
&  0.18735E+01_JPRB, 0.18616E+01_JPRB, 0.17638E+01_JPRB, 0.16317E+01_JPRB, 0.14019E+01_JPRB, 0.12916E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.11020E+01_JPRB, 0.99985E+00_JPRB, 0.86401E+00_JPRB, 0.78026E+00_JPRB, 0.71532E+00_JPRB, 0.65688E+00_JPRB, 0.65538E+00_JPRB ,&
&  0.70724E+00_JPRB, 0.74763E+00_JPRB, 0.85021E+00_JPRB, 0.84738E+00_JPRB, 0.84713E+00_JPRB, 0.86745E+00_JPRB, 0.75926E+00_JPRB ,&
&  0.71251E+00_JPRB, 0.78586E+00_JPRB, 0.75798E+00_JPRB, 0.72895E+00_JPRB, 0.71076E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.68214E+00_JPRB, 0.69369E+00_JPRB, 0.70275E+00_JPRB, 0.72064E+00_JPRB, 0.74456E+00_JPRB, 0.70020E+00_JPRB, 0.65867E+00_JPRB ,&
&  0.71420E+00_JPRB, 0.63128E+00_JPRB, 0.67755E+00_JPRB, 0.58532E+00_JPRB, 0.54019E+00_JPRB, 0.52023E+00_JPRB, 0.50992E+00_JPRB ,&
&  0.50738E+00_JPRB, 0.54401E+00_JPRB, 0.58585E+00_JPRB, 0.62577E+00_JPRB, 0.69340E+00_JPRB, 0.69175E+00_JPRB, 0.67700E+00_JPRB ,&
&  0.65741E+00_JPRB, 0.63988E+00_JPRB, 0.64727E+00_JPRB, 0.64225E+00_JPRB, 0.64304E+00_JPRB, 0.64207E+00_JPRB, 0.63213E+00_JPRB ,&
&  0.59126E+00_JPRB, 0.58313E+00_JPRB, 0.60720E+00_JPRB, 0.62585E+00_JPRB, 0.64016E+00_JPRB, 0.64381E+00_JPRB, 0.65649E+00_JPRB ,&
&  0.68124E+00_JPRB, 0.69866E+00_JPRB, 0.84951E+00_JPRB, 0.88883E+00_JPRB, 0.93230E+00_JPRB, 0.95883E+00_JPRB, 0.99261E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.10942E+01_JPRB, 0.11453E+01_JPRB, 0.12243E+01_JPRB, 0.13822E+01_JPRB, 0.17669E+01_JPRB, 0.23132E+01_JPRB ,&
&  0.30760E+01_JPRB, 0.23890E+01_JPRB, 0.29965E+01_JPRB, 0.30083E+01_JPRB, 0.34122E+01_JPRB, 0.37754E+01_JPRB, 0.38475E+01_JPRB ,&
&  0.41685E+01_JPRB, 0.36985E+01_JPRB, 0.32848E+01_JPRB, 0.28565E+01_JPRB, 0.25021E+01_JPRB, 0.24422E+01_JPRB, 0.23716E+01_JPRB ,&
&  0.23925E+01_JPRB, 0.22491E+01_JPRB, 0.19372E+01_JPRB, 0.17500E+01_JPRB, 0.16739E+01_JPRB, 0.16280E+01_JPRB, 0.14905E+01_JPRB ,&
&  0.14113E+01_JPRB, 0.13428E+01_JPRB, 0.12522E+01_JPRB, 0.11471E+01_JPRB, 0.10761E+01_JPRB, 0.10361E+01_JPRB, 0.97497E+00_JPRB ,&
&  0.95525E+00_JPRB, 0.91775E+00_JPRB, 0.81799E+00_JPRB, 0.76930E+00_JPRB, 0.72661E+00_JPRB, 0.68090E+00_JPRB, 0.67293E+00_JPRB ,&
&  0.67156E+00_JPRB, 0.65347E+00_JPRB, 0.68930E+00_JPRB, 0.73184E+00_JPRB, 0.74032E+00_JPRB, 0.74232E+00_JPRB, 0.81377E+00_JPRB ,&
&  0.75866E+00_JPRB, 0.76222E+00_JPRB, 0.76216E+00_JPRB, 0.71262E+00_JPRB, 0.68660E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.66556E+00_JPRB, 0.66548E+00_JPRB, 0.68757E+00_JPRB, 0.70844E+00_JPRB, 0.72916E+00_JPRB, 0.67638E+00_JPRB, 0.61830E+00_JPRB ,&
&  0.63560E+00_JPRB, 0.57069E+00_JPRB, 0.53059E+00_JPRB, 0.51925E+00_JPRB, 0.51759E+00_JPRB, 0.50291E+00_JPRB, 0.49211E+00_JPRB ,&
&  0.49144E+00_JPRB, 0.50923E+00_JPRB, 0.54081E+00_JPRB, 0.56552E+00_JPRB, 0.59091E+00_JPRB, 0.60722E+00_JPRB, 0.60800E+00_JPRB ,&
&  0.60617E+00_JPRB, 0.60601E+00_JPRB, 0.60609E+00_JPRB, 0.60693E+00_JPRB, 0.60595E+00_JPRB, 0.60815E+00_JPRB, 0.60453E+00_JPRB ,&
&  0.59331E+00_JPRB, 0.58685E+00_JPRB, 0.59290E+00_JPRB, 0.61416E+00_JPRB, 0.61651E+00_JPRB, 0.60996E+00_JPRB, 0.59507E+00_JPRB ,&
&  0.51267E+00_JPRB, 0.57790E+00_JPRB, 0.82456E+00_JPRB, 0.89094E+00_JPRB, 0.93005E+00_JPRB, 0.98342E+00_JPRB, 0.10318E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.11437E+01_JPRB, 0.12144E+01_JPRB, 0.12983E+01_JPRB, 0.14039E+01_JPRB, 0.15857E+01_JPRB, 0.19025E+01_JPRB ,&
&  0.25579E+01_JPRB, 0.21584E+01_JPRB, 0.20858E+01_JPRB, 0.22575E+01_JPRB, 0.32748E+01_JPRB, 0.35787E+01_JPRB, 0.32863E+01_JPRB ,&
&  0.31614E+01_JPRB, 0.30056E+01_JPRB, 0.28289E+01_JPRB, 0.25431E+01_JPRB, 0.23441E+01_JPRB, 0.23055E+01_JPRB, 0.22264E+01_JPRB ,&
&  0.20265E+01_JPRB, 0.18973E+01_JPRB, 0.16397E+01_JPRB, 0.15671E+01_JPRB, 0.15504E+01_JPRB, 0.14495E+01_JPRB, 0.13128E+01_JPRB ,&
&  0.11935E+01_JPRB, 0.10914E+01_JPRB, 0.10115E+01_JPRB, 0.95659E+00_JPRB, 0.91058E+00_JPRB, 0.88144E+00_JPRB, 0.85823E+00_JPRB ,&
&  0.83983E+00_JPRB, 0.82289E+00_JPRB, 0.78194E+00_JPRB, 0.75962E+00_JPRB, 0.73503E+00_JPRB, 0.71740E+00_JPRB, 0.71515E+00_JPRB ,&
&  0.67988E+00_JPRB, 0.66417E+00_JPRB, 0.62635E+00_JPRB, 0.62179E+00_JPRB, 0.64104E+00_JPRB, 0.68530E+00_JPRB, 0.71601E+00_JPRB ,&
&  0.77030E+00_JPRB, 0.75463E+00_JPRB, 0.70650E+00_JPRB, 0.69192E+00_JPRB, 0.67146E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.60786E+00_JPRB, 0.64234E+00_JPRB, 0.67742E+00_JPRB, 0.70297E+00_JPRB, 0.70153E+00_JPRB, 0.67535E+00_JPRB, 0.60831E+00_JPRB ,&
&  0.58443E+00_JPRB, 0.55196E+00_JPRB, 0.53598E+00_JPRB, 0.52148E+00_JPRB, 0.51132E+00_JPRB, 0.50278E+00_JPRB, 0.50182E+00_JPRB ,&
&  0.50903E+00_JPRB, 0.52026E+00_JPRB, 0.53140E+00_JPRB, 0.54043E+00_JPRB, 0.54833E+00_JPRB, 0.55587E+00_JPRB, 0.56359E+00_JPRB ,&
&  0.56794E+00_JPRB, 0.57106E+00_JPRB, 0.57492E+00_JPRB, 0.57827E+00_JPRB, 0.58359E+00_JPRB, 0.58970E+00_JPRB, 0.58851E+00_JPRB ,&
&  0.57720E+00_JPRB, 0.56287E+00_JPRB, 0.57199E+00_JPRB, 0.59286E+00_JPRB, 0.58611E+00_JPRB, 0.56837E+00_JPRB, 0.51219E+00_JPRB ,&
&  0.47196E+00_JPRB, 0.48987E+00_JPRB, 0.59322E+00_JPRB, 0.80765E+00_JPRB, 0.87290E+00_JPRB, 0.93575E+00_JPRB, 0.96969E+00_JPRB ,&
&  0.91433E+00_JPRB, 0.93436E+00_JPRB, 0.10277E+01_JPRB, 0.12730E+01_JPRB, 0.13457E+01_JPRB, 0.14731E+01_JPRB, 0.16369E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.21832E+01_JPRB, 0.23398E+01_JPRB, 0.20324E+01_JPRB, 0.23666E+01_JPRB, 0.30945E+01_JPRB, 0.28257E+01_JPRB ,&
&  0.27147E+01_JPRB, 0.26213E+01_JPRB, 0.26726E+01_JPRB, 0.24457E+01_JPRB, 0.21586E+01_JPRB, 0.19831E+01_JPRB, 0.19170E+01_JPRB ,&
&  0.18280E+01_JPRB, 0.17371E+01_JPRB, 0.14744E+01_JPRB, 0.13589E+01_JPRB, 0.13425E+01_JPRB, 0.12862E+01_JPRB, 0.12388E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.10740E+01_JPRB, 0.97079E+00_JPRB, 0.86936E+00_JPRB, 0.80210E+00_JPRB, 0.77935E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.76928E+00_JPRB, 0.75864E+00_JPRB, 0.74730E+00_JPRB, 0.73487E+00_JPRB, 0.70766E+00_JPRB, 0.66909E+00_JPRB, 0.64268E+00_JPRB ,&
&  0.61619E+00_JPRB, 0.58929E+00_JPRB, 0.56751E+00_JPRB, 0.57005E+00_JPRB, 0.57887E+00_JPRB, 0.60145E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.63851E+00_JPRB, 0.64265E+00_JPRB, 0.63200E+00_JPRB, 0.63371E+00_JPRB, 0.61229E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.55899E+00_JPRB, 0.60003E+00_JPRB, 0.66284E+00_JPRB, 0.69996E+00_JPRB, 0.70281E+00_JPRB, 0.66120E+00_JPRB, 0.60824E+00_JPRB ,&
&  0.57400E+00_JPRB, 0.55505E+00_JPRB, 0.54916E+00_JPRB, 0.54659E+00_JPRB, 0.54636E+00_JPRB, 0.54708E+00_JPRB, 0.54767E+00_JPRB ,&
&  0.54683E+00_JPRB, 0.54592E+00_JPRB, 0.54583E+00_JPRB, 0.54863E+00_JPRB, 0.55406E+00_JPRB, 0.55784E+00_JPRB, 0.56272E+00_JPRB ,&
&  0.56950E+00_JPRB, 0.57493E+00_JPRB, 0.57620E+00_JPRB, 0.57464E+00_JPRB, 0.57570E+00_JPRB, 0.58166E+00_JPRB, 0.58488E+00_JPRB ,&
&  0.57261E+00_JPRB, 0.55479E+00_JPRB, 0.56038E+00_JPRB, 0.57892E+00_JPRB, 0.57164E+00_JPRB, 0.54576E+00_JPRB, 0.49423E+00_JPRB ,&
&  0.46096E+00_JPRB, 0.43863E+00_JPRB, 0.45265E+00_JPRB, 0.51121E+00_JPRB, 0.61247E+00_JPRB, 0.80838E+00_JPRB, 0.90063E+00_JPRB ,&
&  0.98572E+00_JPRB, 0.10450E+01_JPRB, 0.10930E+01_JPRB, 0.11430E+01_JPRB, 0.12215E+01_JPRB, 0.13129E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.15091E+01_JPRB, 0.16671E+01_JPRB, 0.19020E+01_JPRB, 0.15452E+01_JPRB, 0.15576E+01_JPRB, 0.18345E+01_JPRB, 0.21475E+01_JPRB ,&
&  0.22708E+01_JPRB, 0.20892E+01_JPRB, 0.19982E+01_JPRB, 0.20331E+01_JPRB, 0.18729E+01_JPRB, 0.17293E+01_JPRB, 0.16222E+01_JPRB ,&
&  0.15432E+01_JPRB, 0.15114E+01_JPRB, 0.13648E+01_JPRB, 0.11953E+01_JPRB, 0.11483E+01_JPRB, 0.11377E+01_JPRB, 0.11588E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.12682E+01_JPRB, 0.11237E+01_JPRB, 0.84991E+00_JPRB, 0.77432E+00_JPRB, 0.73752E+00_JPRB, 0.73246E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.73589E+00_JPRB, 0.73562E+00_JPRB, 0.72452E+00_JPRB, 0.70004E+00_JPRB, 0.66983E+00_JPRB, 0.63937E+00_JPRB ,&
&  0.61312E+00_JPRB, 0.59870E+00_JPRB, 0.59656E+00_JPRB, 0.59750E+00_JPRB, 0.61243E+00_JPRB, 0.62809E+00_JPRB, 0.64505E+00_JPRB ,&
&  0.64097E+00_JPRB, 0.62595E+00_JPRB, 0.60884E+00_JPRB, 0.57601E+00_JPRB, 0.55026E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.57170E+00_JPRB, 0.60656E+00_JPRB, 0.65536E+00_JPRB, 0.68873E+00_JPRB, 0.69222E+00_JPRB, 0.66580E+00_JPRB, 0.63803E+00_JPRB ,&
&  0.62037E+00_JPRB, 0.61413E+00_JPRB, 0.60676E+00_JPRB, 0.60146E+00_JPRB, 0.59705E+00_JPRB, 0.59258E+00_JPRB, 0.58818E+00_JPRB ,&
&  0.58501E+00_JPRB, 0.58276E+00_JPRB, 0.57960E+00_JPRB, 0.57313E+00_JPRB, 0.57016E+00_JPRB, 0.57418E+00_JPRB, 0.58282E+00_JPRB ,&
&  0.59294E+00_JPRB, 0.59679E+00_JPRB, 0.59879E+00_JPRB, 0.59301E+00_JPRB, 0.57913E+00_JPRB, 0.56288E+00_JPRB, 0.55015E+00_JPRB ,&
&  0.54182E+00_JPRB, 0.54625E+00_JPRB, 0.55114E+00_JPRB, 0.56864E+00_JPRB, 0.55805E+00_JPRB, 0.52710E+00_JPRB, 0.47957E+00_JPRB ,&
&  0.45233E+00_JPRB, 0.42100E+00_JPRB, 0.40742E+00_JPRB, 0.41586E+00_JPRB, 0.45408E+00_JPRB, 0.51237E+00_JPRB, 0.61140E+00_JPRB ,&
&  0.79222E+00_JPRB, 0.86561E+00_JPRB, 0.92134E+00_JPRB, 0.97686E+00_JPRB, 0.10263E+01_JPRB, 0.10793E+01_JPRB, 0.11367E+01_JPRB ,&
&  0.12228E+01_JPRB, 0.13345E+01_JPRB, 0.14632E+01_JPRB, 0.15068E+01_JPRB, 0.15849E+01_JPRB, 0.15560E+01_JPRB, 0.18320E+01_JPRB ,&
&  0.24181E+01_JPRB, 0.20996E+01_JPRB, 0.18006E+01_JPRB, 0.16250E+01_JPRB, 0.15212E+01_JPRB, 0.14200E+01_JPRB, 0.13289E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.11887E+01_JPRB, 0.11600E+01_JPRB, 0.10850E+01_JPRB, 0.10075E+01_JPRB, 0.99284E+00_JPRB, 0.10578E+01_JPRB ,&
&  0.12373E+01_JPRB, 0.14946E+01_JPRB, 0.17648E+01_JPRB, 0.12034E+01_JPRB, 0.10294E+01_JPRB, 0.90643E+00_JPRB, 0.83460E+00_JPRB ,&
&  0.80502E+00_JPRB, 0.79033E+00_JPRB, 0.78054E+00_JPRB, 0.75925E+00_JPRB, 0.73114E+00_JPRB, 0.70568E+00_JPRB, 0.68188E+00_JPRB ,&
&  0.66540E+00_JPRB, 0.66057E+00_JPRB, 0.66225E+00_JPRB, 0.66534E+00_JPRB, 0.66805E+00_JPRB, 0.66499E+00_JPRB, 0.66155E+00_JPRB ,&
&  0.65547E+00_JPRB, 0.63805E+00_JPRB, 0.61933E+00_JPRB, 0.59066E+00_JPRB, 0.57031E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.59711E+00_JPRB, 0.61899E+00_JPRB, 0.65070E+00_JPRB, 0.67289E+00_JPRB, 0.68340E+00_JPRB, 0.67141E+00_JPRB, 0.65516E+00_JPRB ,&
&  0.64130E+00_JPRB, 0.63165E+00_JPRB, 0.62367E+00_JPRB, 0.61802E+00_JPRB, 0.61183E+00_JPRB, 0.60599E+00_JPRB, 0.60146E+00_JPRB ,&
&  0.59845E+00_JPRB, 0.58760E+00_JPRB, 0.58363E+00_JPRB, 0.58656E+00_JPRB, 0.58960E+00_JPRB, 0.59532E+00_JPRB, 0.60187E+00_JPRB ,&
&  0.60234E+00_JPRB, 0.59771E+00_JPRB, 0.59388E+00_JPRB, 0.57758E+00_JPRB, 0.55952E+00_JPRB, 0.55103E+00_JPRB, 0.55886E+00_JPRB ,&
&  0.55269E+00_JPRB, 0.53327E+00_JPRB, 0.55482E+00_JPRB, 0.56894E+00_JPRB, 0.54126E+00_JPRB, 0.51043E+00_JPRB, 0.44815E+00_JPRB ,&
&  0.42939E+00_JPRB, 0.40693E+00_JPRB, 0.40275E+00_JPRB, 0.40502E+00_JPRB, 0.42592E+00_JPRB, 0.47042E+00_JPRB, 0.54316E+00_JPRB ,&
&  0.62018E+00_JPRB, 0.75083E+00_JPRB, 0.79148E+00_JPRB, 0.85469E+00_JPRB, 0.91641E+00_JPRB, 0.97215E+00_JPRB, 0.10182E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.11941E+01_JPRB, 0.12920E+01_JPRB, 0.13611E+01_JPRB, 0.14386E+01_JPRB, 0.15152E+01_JPRB, 0.16252E+01_JPRB ,&
&  0.17589E+01_JPRB, 0.14946E+01_JPRB, 0.13388E+01_JPRB, 0.12364E+01_JPRB, 0.11615E+01_JPRB, 0.11323E+01_JPRB, 0.10515E+01_JPRB ,&
&  0.10244E+01_JPRB, 0.10128E+01_JPRB, 0.99315E+00_JPRB, 0.98516E+00_JPRB, 0.97592E+00_JPRB, 0.96236E+00_JPRB, 0.93703E+00_JPRB ,&
&  0.90779E+00_JPRB, 0.92013E+00_JPRB, 0.10490E+01_JPRB, 0.10570E+01_JPRB, 0.10334E+01_JPRB, 0.95610E+00_JPRB, 0.86510E+00_JPRB ,&
&  0.79849E+00_JPRB, 0.76474E+00_JPRB, 0.74540E+00_JPRB, 0.72866E+00_JPRB, 0.71552E+00_JPRB, 0.70692E+00_JPRB, 0.70865E+00_JPRB ,&
&  0.69560E+00_JPRB, 0.69369E+00_JPRB, 0.69380E+00_JPRB, 0.69112E+00_JPRB, 0.68262E+00_JPRB, 0.66820E+00_JPRB, 0.65651E+00_JPRB ,&
&  0.64491E+00_JPRB, 0.63368E+00_JPRB, 0.62183E+00_JPRB, 0.60812E+00_JPRB, 0.59856E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.61647E+00_JPRB, 0.62349E+00_JPRB, 0.64510E+00_JPRB, 0.66314E+00_JPRB, 0.67422E+00_JPRB, 0.67264E+00_JPRB, 0.66204E+00_JPRB ,&
&  0.65287E+00_JPRB, 0.64615E+00_JPRB, 0.63866E+00_JPRB, 0.63149E+00_JPRB, 0.62285E+00_JPRB, 0.61531E+00_JPRB, 0.60989E+00_JPRB ,&
&  0.60654E+00_JPRB, 0.60603E+00_JPRB, 0.60657E+00_JPRB, 0.60696E+00_JPRB, 0.60815E+00_JPRB, 0.61034E+00_JPRB, 0.61105E+00_JPRB ,&
&  0.61197E+00_JPRB, 0.60340E+00_JPRB, 0.58100E+00_JPRB, 0.56189E+00_JPRB, 0.55596E+00_JPRB, 0.54714E+00_JPRB, 0.55661E+00_JPRB ,&
&  0.55815E+00_JPRB, 0.55951E+00_JPRB, 0.55803E+00_JPRB, 0.55711E+00_JPRB, 0.53372E+00_JPRB, 0.43847E+00_JPRB, 0.43758E+00_JPRB ,&
&  0.42476E+00_JPRB, 0.41070E+00_JPRB, 0.41069E+00_JPRB, 0.41448E+00_JPRB, 0.42471E+00_JPRB, 0.45484E+00_JPRB, 0.50156E+00_JPRB ,&
&  0.54706E+00_JPRB, 0.71964E+00_JPRB, 0.76782E+00_JPRB, 0.82123E+00_JPRB, 0.86900E+00_JPRB, 0.92955E+00_JPRB, 0.10018E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.11148E+01_JPRB, 0.11731E+01_JPRB, 0.12246E+01_JPRB, 0.12644E+01_JPRB, 0.12904E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.12998E+01_JPRB, 0.11709E+01_JPRB, 0.10807E+01_JPRB, 0.10227E+01_JPRB, 0.96123E+00_JPRB, 0.92009E+00_JPRB, 0.89359E+00_JPRB ,&
&  0.89414E+00_JPRB, 0.87434E+00_JPRB, 0.87439E+00_JPRB, 0.85729E+00_JPRB, 0.84576E+00_JPRB, 0.83620E+00_JPRB, 0.82340E+00_JPRB ,&
&  0.80975E+00_JPRB, 0.81043E+00_JPRB, 0.82995E+00_JPRB, 0.83107E+00_JPRB, 0.83996E+00_JPRB, 0.83041E+00_JPRB, 0.80859E+00_JPRB ,&
&  0.77791E+00_JPRB, 0.76739E+00_JPRB, 0.75090E+00_JPRB, 0.73881E+00_JPRB, 0.72460E+00_JPRB, 0.72027E+00_JPRB, 0.71873E+00_JPRB ,&
&  0.71675E+00_JPRB, 0.70999E+00_JPRB, 0.70242E+00_JPRB, 0.69654E+00_JPRB, 0.68985E+00_JPRB, 0.68031E+00_JPRB, 0.66944E+00_JPRB ,&
&  0.65790E+00_JPRB, 0.65097E+00_JPRB, 0.63926E+00_JPRB, 0.62975E+00_JPRB, 0.62285E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.63544E+00_JPRB, 0.63364E+00_JPRB, 0.63808E+00_JPRB, 0.64296E+00_JPRB, 0.64717E+00_JPRB, 0.64853E+00_JPRB, 0.64257E+00_JPRB ,&
&  0.63761E+00_JPRB, 0.63410E+00_JPRB, 0.63150E+00_JPRB, 0.63057E+00_JPRB, 0.63198E+00_JPRB, 0.62965E+00_JPRB, 0.62427E+00_JPRB ,&
&  0.61862E+00_JPRB, 0.61489E+00_JPRB, 0.61256E+00_JPRB, 0.61073E+00_JPRB, 0.61186E+00_JPRB, 0.61103E+00_JPRB, 0.60968E+00_JPRB ,&
&  0.60767E+00_JPRB, 0.59457E+00_JPRB, 0.56522E+00_JPRB, 0.55395E+00_JPRB, 0.54420E+00_JPRB, 0.54616E+00_JPRB, 0.54890E+00_JPRB ,&
&  0.55376E+00_JPRB, 0.48874E+00_JPRB, 0.49375E+00_JPRB, 0.47580E+00_JPRB, 0.46253E+00_JPRB, 0.45953E+00_JPRB, 0.44809E+00_JPRB ,&
&  0.44101E+00_JPRB, 0.43201E+00_JPRB, 0.43378E+00_JPRB, 0.44238E+00_JPRB, 0.44810E+00_JPRB, 0.46595E+00_JPRB, 0.49391E+00_JPRB ,&
&  0.53509E+00_JPRB, 0.68403E+00_JPRB, 0.75305E+00_JPRB, 0.80091E+00_JPRB, 0.83860E+00_JPRB, 0.88291E+00_JPRB, 0.93129E+00_JPRB ,&
&  0.97282E+00_JPRB, 0.10229E+01_JPRB, 0.10548E+01_JPRB, 0.10593E+01_JPRB, 0.10912E+01_JPRB, 0.10907E+01_JPRB, 0.10700E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.95119E+00_JPRB, 0.89258E+00_JPRB, 0.85487E+00_JPRB, 0.82686E+00_JPRB, 0.81049E+00_JPRB, 0.80270E+00_JPRB ,&
&  0.81168E+00_JPRB, 0.80927E+00_JPRB, 0.80191E+00_JPRB, 0.77963E+00_JPRB, 0.76524E+00_JPRB, 0.75460E+00_JPRB, 0.75082E+00_JPRB ,&
&  0.75152E+00_JPRB, 0.75768E+00_JPRB, 0.77307E+00_JPRB, 0.78281E+00_JPRB, 0.78609E+00_JPRB, 0.78276E+00_JPRB, 0.78493E+00_JPRB ,&
&  0.78129E+00_JPRB, 0.77030E+00_JPRB, 0.75786E+00_JPRB, 0.74373E+00_JPRB, 0.72713E+00_JPRB, 0.71389E+00_JPRB, 0.70579E+00_JPRB ,&
&  0.70081E+00_JPRB, 0.69465E+00_JPRB, 0.68891E+00_JPRB, 0.68467E+00_JPRB, 0.68312E+00_JPRB, 0.68128E+00_JPRB, 0.67868E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.65888E+00_JPRB, 0.64971E+00_JPRB, 0.64626E+00_JPRB, 0.64200E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.62722E+00_JPRB, 0.62271E+00_JPRB, 0.62047E+00_JPRB, 0.61872E+00_JPRB, 0.61848E+00_JPRB, 0.61887E+00_JPRB, 0.62019E+00_JPRB ,&
&  0.62269E+00_JPRB, 0.62648E+00_JPRB, 0.62958E+00_JPRB, 0.63119E+00_JPRB, 0.63428E+00_JPRB, 0.63901E+00_JPRB, 0.64032E+00_JPRB ,&
&  0.63495E+00_JPRB, 0.62769E+00_JPRB, 0.62015E+00_JPRB, 0.61239E+00_JPRB, 0.60552E+00_JPRB, 0.60024E+00_JPRB, 0.59201E+00_JPRB ,&
&  0.58644E+00_JPRB, 0.57711E+00_JPRB, 0.54075E+00_JPRB, 0.54129E+00_JPRB, 0.53376E+00_JPRB, 0.53241E+00_JPRB, 0.53669E+00_JPRB ,&
&  0.52938E+00_JPRB, 0.52909E+00_JPRB, 0.51795E+00_JPRB, 0.49855E+00_JPRB, 0.48486E+00_JPRB, 0.46520E+00_JPRB, 0.46357E+00_JPRB ,&
&  0.46025E+00_JPRB, 0.46491E+00_JPRB, 0.47700E+00_JPRB, 0.48379E+00_JPRB, 0.48645E+00_JPRB, 0.49212E+00_JPRB, 0.51149E+00_JPRB ,&
&  0.55109E+00_JPRB, 0.60748E+00_JPRB, 0.68568E+00_JPRB, 0.72922E+00_JPRB, 0.76697E+00_JPRB, 0.80311E+00_JPRB, 0.83838E+00_JPRB ,&
&  0.86751E+00_JPRB, 0.89274E+00_JPRB, 0.90410E+00_JPRB, 0.91062E+00_JPRB, 0.90357E+00_JPRB, 0.88212E+00_JPRB, 0.85328E+00_JPRB ,&
&  0.82093E+00_JPRB, 0.79742E+00_JPRB, 0.77957E+00_JPRB, 0.77224E+00_JPRB, 0.76886E+00_JPRB, 0.76737E+00_JPRB, 0.76729E+00_JPRB ,&
&  0.76478E+00_JPRB, 0.74792E+00_JPRB, 0.73271E+00_JPRB, 0.71287E+00_JPRB, 0.70247E+00_JPRB, 0.69351E+00_JPRB, 0.69102E+00_JPRB ,&
&  0.69070E+00_JPRB, 0.69661E+00_JPRB, 0.70665E+00_JPRB, 0.70939E+00_JPRB, 0.71728E+00_JPRB, 0.72938E+00_JPRB, 0.72933E+00_JPRB ,&
&  0.72884E+00_JPRB, 0.72354E+00_JPRB, 0.71407E+00_JPRB, 0.70238E+00_JPRB, 0.68889E+00_JPRB, 0.67594E+00_JPRB, 0.66370E+00_JPRB ,&
&  0.65504E+00_JPRB, 0.64898E+00_JPRB, 0.64777E+00_JPRB, 0.64749E+00_JPRB, 0.64991E+00_JPRB, 0.65013E+00_JPRB, 0.64713E+00_JPRB ,&
&  0.63976E+00_JPRB, 0.63442E+00_JPRB, 0.62997E+00_JPRB, 0.62995E+00_JPRB, 0.62946E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.62390E+00_JPRB, 0.62428E+00_JPRB, 0.62402E+00_JPRB, 0.62249E+00_JPRB, 0.62207E+00_JPRB, 0.62257E+00_JPRB, 0.62386E+00_JPRB ,&
&  0.62568E+00_JPRB, 0.62831E+00_JPRB, 0.63105E+00_JPRB, 0.63300E+00_JPRB, 0.63439E+00_JPRB, 0.64256E+00_JPRB, 0.64592E+00_JPRB ,&
&  0.64526E+00_JPRB, 0.64228E+00_JPRB, 0.63822E+00_JPRB, 0.63413E+00_JPRB, 0.62886E+00_JPRB, 0.62204E+00_JPRB, 0.61089E+00_JPRB ,&
&  0.60426E+00_JPRB, 0.59711E+00_JPRB, 0.58897E+00_JPRB, 0.58636E+00_JPRB, 0.58375E+00_JPRB, 0.55400E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.56969E+00_JPRB, 0.57903E+00_JPRB, 0.58312E+00_JPRB, 0.58851E+00_JPRB, 0.57766E+00_JPRB, 0.54901E+00_JPRB, 0.52821E+00_JPRB ,&
&  0.49186E+00_JPRB, 0.51400E+00_JPRB, 0.52998E+00_JPRB, 0.53648E+00_JPRB, 0.53814E+00_JPRB, 0.54109E+00_JPRB, 0.57074E+00_JPRB ,&
&  0.63568E+00_JPRB, 0.64919E+00_JPRB, 0.66521E+00_JPRB, 0.68317E+00_JPRB, 0.70278E+00_JPRB, 0.71827E+00_JPRB, 0.73669E+00_JPRB ,&
&  0.75006E+00_JPRB, 0.76066E+00_JPRB, 0.76406E+00_JPRB, 0.76525E+00_JPRB, 0.75815E+00_JPRB, 0.75184E+00_JPRB, 0.74501E+00_JPRB ,&
&  0.74226E+00_JPRB, 0.73996E+00_JPRB, 0.73734E+00_JPRB, 0.73197E+00_JPRB, 0.72294E+00_JPRB, 0.71336E+00_JPRB, 0.70249E+00_JPRB ,&
&  0.69140E+00_JPRB, 0.67714E+00_JPRB, 0.66631E+00_JPRB, 0.65639E+00_JPRB, 0.65260E+00_JPRB, 0.64788E+00_JPRB, 0.64806E+00_JPRB ,&
&  0.64879E+00_JPRB, 0.65290E+00_JPRB, 0.65796E+00_JPRB, 0.66700E+00_JPRB, 0.67411E+00_JPRB, 0.68012E+00_JPRB, 0.68303E+00_JPRB ,&
&  0.68437E+00_JPRB, 0.68293E+00_JPRB, 0.68015E+00_JPRB, 0.67463E+00_JPRB, 0.66956E+00_JPRB, 0.66482E+00_JPRB, 0.65956E+00_JPRB ,&
&  0.65478E+00_JPRB, 0.64983E+00_JPRB, 0.64570E+00_JPRB, 0.64147E+00_JPRB, 0.63734E+00_JPRB, 0.63229E+00_JPRB, 0.62725E+00_JPRB ,&
&  0.62222E+00_JPRB, 0.61970E+00_JPRB, 0.61841E+00_JPRB, 0.61961E+00_JPRB, 0.62130E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.64189E+00_JPRB, 0.64269E+00_JPRB, 0.64318E+00_JPRB, 0.64297E+00_JPRB, 0.64333E+00_JPRB, 0.64354E+00_JPRB, 0.64323E+00_JPRB ,&
&  0.64211E+00_JPRB, 0.64194E+00_JPRB, 0.64255E+00_JPRB, 0.64289E+00_JPRB, 0.64248E+00_JPRB, 0.64618E+00_JPRB, 0.65304E+00_JPRB ,&
&  0.65452E+00_JPRB, 0.65514E+00_JPRB, 0.65499E+00_JPRB, 0.65481E+00_JPRB, 0.65391E+00_JPRB, 0.65229E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.65047E+00_JPRB, 0.64992E+00_JPRB, 0.64856E+00_JPRB, 0.64624E+00_JPRB, 0.64655E+00_JPRB, 0.64671E+00_JPRB, 0.64629E+00_JPRB ,&
&  0.64580E+00_JPRB, 0.64405E+00_JPRB, 0.64231E+00_JPRB, 0.64175E+00_JPRB, 0.64531E+00_JPRB, 0.64960E+00_JPRB, 0.65463E+00_JPRB ,&
&  0.65984E+00_JPRB, 0.66690E+00_JPRB, 0.67427E+00_JPRB, 0.67958E+00_JPRB, 0.68212E+00_JPRB, 0.68713E+00_JPRB, 0.69125E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.69055E+00_JPRB, 0.68904E+00_JPRB, 0.69022E+00_JPRB, 0.69134E+00_JPRB, 0.69179E+00_JPRB, 0.69406E+00_JPRB ,&
&  0.69696E+00_JPRB, 0.69945E+00_JPRB, 0.70126E+00_JPRB, 0.70279E+00_JPRB, 0.70259E+00_JPRB, 0.70127E+00_JPRB, 0.69869E+00_JPRB ,&
&  0.69613E+00_JPRB, 0.69250E+00_JPRB, 0.68875E+00_JPRB, 0.68506E+00_JPRB, 0.68123E+00_JPRB, 0.67645E+00_JPRB, 0.67052E+00_JPRB ,&
&  0.66571E+00_JPRB, 0.66261E+00_JPRB, 0.66117E+00_JPRB, 0.65806E+00_JPRB, 0.65552E+00_JPRB, 0.65300E+00_JPRB, 0.65367E+00_JPRB ,&
&  0.65451E+00_JPRB, 0.65764E+00_JPRB, 0.66054E+00_JPRB, 0.66560E+00_JPRB, 0.66973E+00_JPRB, 0.67348E+00_JPRB, 0.67517E+00_JPRB ,&
&  0.67699E+00_JPRB, 0.67723E+00_JPRB, 0.67638E+00_JPRB, 0.67444E+00_JPRB, 0.67265E+00_JPRB, 0.67021E+00_JPRB, 0.66695E+00_JPRB ,&
&  0.66313E+00_JPRB, 0.66004E+00_JPRB, 0.65649E+00_JPRB, 0.65246E+00_JPRB, 0.64898E+00_JPRB, 0.64566E+00_JPRB, 0.64316E+00_JPRB ,&
&  0.64096E+00_JPRB, 0.63982E+00_JPRB, 0.63917E+00_JPRB, 0.63979E+00_JPRB, 0.64081E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.65492E+00_JPRB, 0.65593E+00_JPRB, 0.65685E+00_JPRB, 0.65756E+00_JPRB, 0.65833E+00_JPRB, 0.65894E+00_JPRB, 0.65943E+00_JPRB ,&
&  0.65947E+00_JPRB, 0.66037E+00_JPRB, 0.66166E+00_JPRB, 0.66289E+00_JPRB, 0.66400E+00_JPRB, 0.66514E+00_JPRB, 0.66612E+00_JPRB ,&
&  0.66768E+00_JPRB, 0.67278E+00_JPRB, 0.67684E+00_JPRB, 0.67920E+00_JPRB, 0.68123E+00_JPRB, 0.68318E+00_JPRB, 0.68495E+00_JPRB ,&
&  0.68662E+00_JPRB, 0.68793E+00_JPRB, 0.68904E+00_JPRB, 0.68990E+00_JPRB, 0.69088E+00_JPRB, 0.69188E+00_JPRB, 0.69291E+00_JPRB ,&
&  0.69410E+00_JPRB, 0.69595E+00_JPRB, 0.69811E+00_JPRB, 0.70034E+00_JPRB, 0.70283E+00_JPRB, 0.70621E+00_JPRB, 0.70921E+00_JPRB ,&
&  0.71168E+00_JPRB, 0.71332E+00_JPRB, 0.71305E+00_JPRB, 0.71132E+00_JPRB, 0.70871E+00_JPRB, 0.70596E+00_JPRB, 0.70295E+00_JPRB ,&
&  0.69932E+00_JPRB, 0.69594E+00_JPRB, 0.69317E+00_JPRB, 0.69070E+00_JPRB, 0.68844E+00_JPRB, 0.68660E+00_JPRB, 0.68483E+00_JPRB ,&
&  0.68320E+00_JPRB, 0.68142E+00_JPRB, 0.67975E+00_JPRB, 0.67814E+00_JPRB, 0.67639E+00_JPRB, 0.67424E+00_JPRB, 0.67216E+00_JPRB ,&
&  0.67059E+00_JPRB, 0.66910E+00_JPRB, 0.66797E+00_JPRB, 0.66675E+00_JPRB, 0.66569E+00_JPRB, 0.66528E+00_JPRB, 0.66518E+00_JPRB ,&
&  0.66504E+00_JPRB, 0.66511E+00_JPRB, 0.66558E+00_JPRB, 0.66566E+00_JPRB, 0.66544E+00_JPRB, 0.66489E+00_JPRB, 0.66441E+00_JPRB ,&
&  0.66401E+00_JPRB, 0.66432E+00_JPRB, 0.66429E+00_JPRB, 0.66477E+00_JPRB, 0.66518E+00_JPRB, 0.66570E+00_JPRB, 0.66629E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.66656E+00_JPRB, 0.66641E+00_JPRB, 0.66640E+00_JPRB, 0.66617E+00_JPRB, 0.66551E+00_JPRB, 0.66443E+00_JPRB ,&
&  0.66321E+00_JPRB, 0.66181E+00_JPRB, 0.66039E+00_JPRB, 0.65876E+00_JPRB, 0.65760E+00_JPRB, 0.65676E+00_JPRB, 0.65586E+00_JPRB ,&
&  0.65492E+00_JPRB, 0.65437E+00_JPRB, 0.65405E+00_JPRB, 0.65420E+00_JPRB, 0.65426E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB ,&
&  0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB, 0.67866E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB ,&
&  0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB, 0.16177E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.16739E+00_JPRB, 0.16738E+00_JPRB, 0.16723E+00_JPRB, 0.16657E+00_JPRB, 0.16448E+00_JPRB, 0.16411E+00_JPRB, 0.16409E+00_JPRB ,&
&  0.16425E+00_JPRB, 0.16436E+00_JPRB, 0.16517E+00_JPRB, 0.16605E+00_JPRB, 0.16692E+00_JPRB, 0.16759E+00_JPRB, 0.16816E+00_JPRB ,&
&  0.16848E+00_JPRB, 0.16879E+00_JPRB, 0.16777E+00_JPRB, 0.16726E+00_JPRB, 0.16712E+00_JPRB, 0.16712E+00_JPRB, 0.16712E+00_JPRB ,&
&  0.16706E+00_JPRB, 0.16704E+00_JPRB, 0.16691E+00_JPRB, 0.16709E+00_JPRB, 0.16737E+00_JPRB, 0.16760E+00_JPRB, 0.16788E+00_JPRB ,&
&  0.16809E+00_JPRB, 0.16828E+00_JPRB, 0.16840E+00_JPRB, 0.16848E+00_JPRB, 0.16877E+00_JPRB, 0.16891E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.16864E+00_JPRB, 0.16830E+00_JPRB, 0.16791E+00_JPRB, 0.16751E+00_JPRB, 0.16722E+00_JPRB, 0.16694E+00_JPRB, 0.16666E+00_JPRB ,&
&  0.16638E+00_JPRB, 0.16609E+00_JPRB, 0.16576E+00_JPRB, 0.16540E+00_JPRB, 0.16506E+00_JPRB, 0.16481E+00_JPRB, 0.16454E+00_JPRB ,&
&  0.16431E+00_JPRB, 0.16410E+00_JPRB, 0.16386E+00_JPRB, 0.16357E+00_JPRB, 0.16326E+00_JPRB, 0.16296E+00_JPRB, 0.16261E+00_JPRB ,&
&  0.16219E+00_JPRB, 0.16188E+00_JPRB, 0.16159E+00_JPRB, 0.16132E+00_JPRB, 0.16110E+00_JPRB, 0.16086E+00_JPRB, 0.16066E+00_JPRB ,&
&  0.16042E+00_JPRB, 0.16021E+00_JPRB, 0.15996E+00_JPRB, 0.15973E+00_JPRB, 0.15952E+00_JPRB, 0.15933E+00_JPRB, 0.15918E+00_JPRB ,&
&  0.15905E+00_JPRB, 0.15890E+00_JPRB, 0.15878E+00_JPRB, 0.15879E+00_JPRB, 0.15877E+00_JPRB, 0.15880E+00_JPRB, 0.15883E+00_JPRB ,&
&  0.15886E+00_JPRB, 0.15888E+00_JPRB, 0.15890E+00_JPRB, 0.15880E+00_JPRB, 0.15894E+00_JPRB, 0.15914E+00_JPRB, 0.15944E+00_JPRB ,&
&  0.15979E+00_JPRB, 0.16024E+00_JPRB, 0.16072E+00_JPRB, 0.16121E+00_JPRB, 0.16179E+00_JPRB, 0.16227E+00_JPRB, 0.16281E+00_JPRB ,&
&  0.16342E+00_JPRB, 0.16405E+00_JPRB, 0.16451E+00_JPRB, 0.16505E+00_JPRB, 0.16587E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.17822E+00_JPRB, 0.18110E+00_JPRB, 0.18297E+00_JPRB, 0.18463E+00_JPRB, 0.17932E+00_JPRB, 0.19058E+00_JPRB, 0.18738E+00_JPRB ,&
&  0.18681E+00_JPRB, 0.18532E+00_JPRB, 0.18682E+00_JPRB, 0.18677E+00_JPRB, 0.18610E+00_JPRB, 0.18512E+00_JPRB, 0.18390E+00_JPRB ,&
&  0.18274E+00_JPRB, 0.18131E+00_JPRB, 0.18036E+00_JPRB, 0.17935E+00_JPRB, 0.17854E+00_JPRB, 0.17775E+00_JPRB, 0.17720E+00_JPRB ,&
&  0.17719E+00_JPRB, 0.17795E+00_JPRB, 0.17931E+00_JPRB, 0.17875E+00_JPRB, 0.17877E+00_JPRB, 0.17918E+00_JPRB, 0.17978E+00_JPRB ,&
&  0.18005E+00_JPRB, 0.18018E+00_JPRB, 0.17992E+00_JPRB, 0.17890E+00_JPRB, 0.17885E+00_JPRB, 0.17885E+00_JPRB, 0.17893E+00_JPRB ,&
&  0.17883E+00_JPRB, 0.17851E+00_JPRB, 0.17766E+00_JPRB, 0.17686E+00_JPRB, 0.17604E+00_JPRB, 0.17506E+00_JPRB, 0.17390E+00_JPRB ,&
&  0.17294E+00_JPRB, 0.17219E+00_JPRB, 0.17130E+00_JPRB, 0.17042E+00_JPRB, 0.16966E+00_JPRB, 0.16893E+00_JPRB, 0.16825E+00_JPRB ,&
&  0.16781E+00_JPRB, 0.16748E+00_JPRB, 0.16712E+00_JPRB, 0.16659E+00_JPRB, 0.16599E+00_JPRB, 0.16542E+00_JPRB, 0.16490E+00_JPRB ,&
&  0.16456E+00_JPRB, 0.16400E+00_JPRB, 0.16345E+00_JPRB, 0.16285E+00_JPRB, 0.16239E+00_JPRB, 0.16165E+00_JPRB, 0.16100E+00_JPRB ,&
&  0.16037E+00_JPRB, 0.15980E+00_JPRB, 0.15938E+00_JPRB, 0.15906E+00_JPRB, 0.15878E+00_JPRB, 0.15859E+00_JPRB, 0.15846E+00_JPRB ,&
&  0.15839E+00_JPRB, 0.15834E+00_JPRB, 0.15838E+00_JPRB, 0.15836E+00_JPRB, 0.15830E+00_JPRB, 0.15831E+00_JPRB, 0.15859E+00_JPRB ,&
&  0.15884E+00_JPRB, 0.15901E+00_JPRB, 0.15912E+00_JPRB, 0.15886E+00_JPRB, 0.15924E+00_JPRB, 0.15958E+00_JPRB, 0.15999E+00_JPRB ,&
&  0.16035E+00_JPRB, 0.16111E+00_JPRB, 0.16202E+00_JPRB, 0.16312E+00_JPRB, 0.16521E+00_JPRB, 0.16707E+00_JPRB, 0.16904E+00_JPRB ,&
&  0.17126E+00_JPRB, 0.17445E+00_JPRB, 0.17582E+00_JPRB, 0.17674E+00_JPRB, 0.17750E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.20305E+00_JPRB, 0.20454E+00_JPRB, 0.20598E+00_JPRB, 0.20734E+00_JPRB, 0.20886E+00_JPRB, 0.20946E+00_JPRB, 0.20939E+00_JPRB ,&
&  0.20709E+00_JPRB, 0.20650E+00_JPRB, 0.20498E+00_JPRB, 0.20311E+00_JPRB, 0.20160E+00_JPRB, 0.19881E+00_JPRB, 0.19608E+00_JPRB ,&
&  0.19331E+00_JPRB, 0.19065E+00_JPRB, 0.18827E+00_JPRB, 0.18500E+00_JPRB, 0.18186E+00_JPRB, 0.17878E+00_JPRB, 0.17783E+00_JPRB ,&
&  0.17743E+00_JPRB, 0.17791E+00_JPRB, 0.17955E+00_JPRB, 0.18093E+00_JPRB, 0.18292E+00_JPRB, 0.18505E+00_JPRB, 0.18828E+00_JPRB ,&
&  0.19142E+00_JPRB, 0.19374E+00_JPRB, 0.19408E+00_JPRB, 0.19296E+00_JPRB, 0.19199E+00_JPRB, 0.19168E+00_JPRB, 0.19125E+00_JPRB ,&
&  0.19046E+00_JPRB, 0.18997E+00_JPRB, 0.18933E+00_JPRB, 0.18878E+00_JPRB, 0.18734E+00_JPRB, 0.18484E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.17999E+00_JPRB, 0.17792E+00_JPRB, 0.17598E+00_JPRB, 0.17430E+00_JPRB, 0.17331E+00_JPRB, 0.17231E+00_JPRB, 0.17109E+00_JPRB ,&
&  0.17035E+00_JPRB, 0.16958E+00_JPRB, 0.16869E+00_JPRB, 0.16803E+00_JPRB, 0.16727E+00_JPRB, 0.16673E+00_JPRB, 0.16611E+00_JPRB ,&
&  0.16563E+00_JPRB, 0.16491E+00_JPRB, 0.16429E+00_JPRB, 0.16361E+00_JPRB, 0.16329E+00_JPRB, 0.16254E+00_JPRB, 0.16183E+00_JPRB ,&
&  0.16095E+00_JPRB, 0.16041E+00_JPRB, 0.15941E+00_JPRB, 0.15881E+00_JPRB, 0.15812E+00_JPRB, 0.15750E+00_JPRB, 0.15708E+00_JPRB ,&
&  0.15716E+00_JPRB, 0.15718E+00_JPRB, 0.15680E+00_JPRB, 0.15675E+00_JPRB, 0.15681E+00_JPRB, 0.15724E+00_JPRB, 0.15851E+00_JPRB ,&
&  0.15933E+00_JPRB, 0.16023E+00_JPRB, 0.16111E+00_JPRB, 0.16179E+00_JPRB, 0.16292E+00_JPRB, 0.16402E+00_JPRB, 0.16514E+00_JPRB ,&
&  0.16645E+00_JPRB, 0.16759E+00_JPRB, 0.16893E+00_JPRB, 0.17051E+00_JPRB, 0.17431E+00_JPRB, 0.17702E+00_JPRB, 0.17992E+00_JPRB ,&
&  0.18206E+00_JPRB, 0.18108E+00_JPRB, 0.19494E+00_JPRB, 0.19763E+00_JPRB, 0.19965E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.21211E+00_JPRB, 0.21297E+00_JPRB, 0.21297E+00_JPRB, 0.21393E+00_JPRB, 0.21476E+00_JPRB, 0.21580E+00_JPRB, 0.21599E+00_JPRB ,&
&  0.21539E+00_JPRB, 0.21114E+00_JPRB, 0.20879E+00_JPRB, 0.20551E+00_JPRB, 0.20236E+00_JPRB, 0.19910E+00_JPRB, 0.19640E+00_JPRB ,&
&  0.19464E+00_JPRB, 0.19328E+00_JPRB, 0.19202E+00_JPRB, 0.19031E+00_JPRB, 0.18823E+00_JPRB, 0.18614E+00_JPRB, 0.18422E+00_JPRB ,&
&  0.18250E+00_JPRB, 0.18253E+00_JPRB, 0.18370E+00_JPRB, 0.18583E+00_JPRB, 0.18975E+00_JPRB, 0.19283E+00_JPRB, 0.19678E+00_JPRB ,&
&  0.20304E+00_JPRB, 0.20009E+00_JPRB, 0.20024E+00_JPRB, 0.20047E+00_JPRB, 0.20018E+00_JPRB, 0.19997E+00_JPRB, 0.19696E+00_JPRB ,&
&  0.20024E+00_JPRB, 0.20044E+00_JPRB, 0.20381E+00_JPRB, 0.20301E+00_JPRB, 0.19856E+00_JPRB, 0.19491E+00_JPRB, 0.18999E+00_JPRB ,&
&  0.18640E+00_JPRB, 0.18203E+00_JPRB, 0.17909E+00_JPRB, 0.17645E+00_JPRB, 0.17339E+00_JPRB, 0.17213E+00_JPRB, 0.17001E+00_JPRB ,&
&  0.16872E+00_JPRB, 0.16843E+00_JPRB, 0.16749E+00_JPRB, 0.16743E+00_JPRB, 0.16665E+00_JPRB, 0.16554E+00_JPRB, 0.16479E+00_JPRB ,&
&  0.16413E+00_JPRB, 0.16343E+00_JPRB, 0.16318E+00_JPRB, 0.16292E+00_JPRB, 0.16304E+00_JPRB, 0.16306E+00_JPRB, 0.16312E+00_JPRB ,&
&  0.16333E+00_JPRB, 0.16346E+00_JPRB, 0.16306E+00_JPRB, 0.16289E+00_JPRB, 0.16201E+00_JPRB, 0.16126E+00_JPRB, 0.16008E+00_JPRB ,&
&  0.15937E+00_JPRB, 0.15895E+00_JPRB, 0.15833E+00_JPRB, 0.15793E+00_JPRB, 0.15784E+00_JPRB, 0.15832E+00_JPRB, 0.15894E+00_JPRB ,&
&  0.15965E+00_JPRB, 0.16092E+00_JPRB, 0.16197E+00_JPRB, 0.16338E+00_JPRB, 0.16492E+00_JPRB, 0.16614E+00_JPRB, 0.16787E+00_JPRB ,&
&  0.16960E+00_JPRB, 0.17137E+00_JPRB, 0.17437E+00_JPRB, 0.17772E+00_JPRB, 0.18216E+00_JPRB, 0.18720E+00_JPRB, 0.19009E+00_JPRB ,&
&  0.19337E+00_JPRB, 0.20039E+00_JPRB, 0.20304E+00_JPRB, 0.21014E+00_JPRB, 0.21128E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.21738E+00_JPRB, 0.21729E+00_JPRB, 0.21659E+00_JPRB, 0.21611E+00_JPRB, 0.21656E+00_JPRB, 0.21733E+00_JPRB, 0.21828E+00_JPRB ,&
&  0.21484E+00_JPRB, 0.21218E+00_JPRB, 0.20787E+00_JPRB, 0.20309E+00_JPRB, 0.19891E+00_JPRB, 0.19553E+00_JPRB, 0.19356E+00_JPRB ,&
&  0.19332E+00_JPRB, 0.19527E+00_JPRB, 0.19685E+00_JPRB, 0.19873E+00_JPRB, 0.19885E+00_JPRB, 0.19901E+00_JPRB, 0.19767E+00_JPRB ,&
&  0.19622E+00_JPRB, 0.19563E+00_JPRB, 0.19588E+00_JPRB, 0.19762E+00_JPRB, 0.20030E+00_JPRB, 0.20215E+00_JPRB, 0.20434E+00_JPRB ,&
&  0.20725E+00_JPRB, 0.20666E+00_JPRB, 0.20741E+00_JPRB, 0.20686E+00_JPRB, 0.20711E+00_JPRB, 0.20794E+00_JPRB, 0.20833E+00_JPRB ,&
&  0.20916E+00_JPRB, 0.20999E+00_JPRB, 0.20985E+00_JPRB, 0.20912E+00_JPRB, 0.20495E+00_JPRB, 0.20223E+00_JPRB, 0.19758E+00_JPRB ,&
&  0.19210E+00_JPRB, 0.18727E+00_JPRB, 0.18184E+00_JPRB, 0.17805E+00_JPRB, 0.17364E+00_JPRB, 0.17102E+00_JPRB, 0.16899E+00_JPRB ,&
&  0.16708E+00_JPRB, 0.16634E+00_JPRB, 0.16532E+00_JPRB, 0.16422E+00_JPRB, 0.16398E+00_JPRB, 0.16290E+00_JPRB, 0.16282E+00_JPRB ,&
&  0.16307E+00_JPRB, 0.16330E+00_JPRB, 0.16422E+00_JPRB, 0.16467E+00_JPRB, 0.16545E+00_JPRB, 0.16619E+00_JPRB, 0.16677E+00_JPRB ,&
&  0.16788E+00_JPRB, 0.16925E+00_JPRB, 0.16925E+00_JPRB, 0.16949E+00_JPRB, 0.16865E+00_JPRB, 0.16694E+00_JPRB, 0.16505E+00_JPRB ,&
&  0.16305E+00_JPRB, 0.16149E+00_JPRB, 0.16100E+00_JPRB, 0.16032E+00_JPRB, 0.16047E+00_JPRB, 0.16083E+00_JPRB, 0.16103E+00_JPRB ,&
&  0.16170E+00_JPRB, 0.16230E+00_JPRB, 0.16313E+00_JPRB, 0.16493E+00_JPRB, 0.16622E+00_JPRB, 0.16784E+00_JPRB, 0.16952E+00_JPRB ,&
&  0.17084E+00_JPRB, 0.17325E+00_JPRB, 0.17538E+00_JPRB, 0.17830E+00_JPRB, 0.18500E+00_JPRB, 0.19098E+00_JPRB, 0.20164E+00_JPRB ,&
&  0.20700E+00_JPRB, 0.21896E+00_JPRB, 0.21728E+00_JPRB, 0.21673E+00_JPRB, 0.21774E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.22408E+00_JPRB, 0.22290E+00_JPRB, 0.22173E+00_JPRB, 0.22050E+00_JPRB, 0.22133E+00_JPRB, 0.22245E+00_JPRB, 0.22387E+00_JPRB ,&
&  0.22550E+00_JPRB, 0.22591E+00_JPRB, 0.22570E+00_JPRB, 0.22360E+00_JPRB, 0.22234E+00_JPRB, 0.21608E+00_JPRB, 0.21379E+00_JPRB ,&
&  0.21208E+00_JPRB, 0.21380E+00_JPRB, 0.21623E+00_JPRB, 0.21994E+00_JPRB, 0.22157E+00_JPRB, 0.22328E+00_JPRB, 0.22240E+00_JPRB ,&
&  0.22027E+00_JPRB, 0.21769E+00_JPRB, 0.21523E+00_JPRB, 0.21328E+00_JPRB, 0.21272E+00_JPRB, 0.21122E+00_JPRB, 0.20959E+00_JPRB ,&
&  0.20956E+00_JPRB, 0.21095E+00_JPRB, 0.21176E+00_JPRB, 0.21260E+00_JPRB, 0.21858E+00_JPRB, 0.21928E+00_JPRB, 0.22130E+00_JPRB ,&
&  0.22183E+00_JPRB, 0.22234E+00_JPRB, 0.22163E+00_JPRB, 0.22024E+00_JPRB, 0.21896E+00_JPRB, 0.21386E+00_JPRB, 0.21153E+00_JPRB ,&
&  0.20425E+00_JPRB, 0.19994E+00_JPRB, 0.19329E+00_JPRB, 0.18736E+00_JPRB, 0.18098E+00_JPRB, 0.17620E+00_JPRB, 0.17222E+00_JPRB ,&
&  0.16907E+00_JPRB, 0.16673E+00_JPRB, 0.16477E+00_JPRB, 0.16450E+00_JPRB, 0.16457E+00_JPRB, 0.16423E+00_JPRB, 0.16438E+00_JPRB ,&
&  0.16515E+00_JPRB, 0.16599E+00_JPRB, 0.16593E+00_JPRB, 0.16651E+00_JPRB, 0.16704E+00_JPRB, 0.16781E+00_JPRB, 0.16923E+00_JPRB ,&
&  0.17143E+00_JPRB, 0.17335E+00_JPRB, 0.17413E+00_JPRB, 0.17583E+00_JPRB, 0.17514E+00_JPRB, 0.17254E+00_JPRB, 0.16995E+00_JPRB ,&
&  0.16714E+00_JPRB, 0.16591E+00_JPRB, 0.16458E+00_JPRB, 0.16381E+00_JPRB, 0.16371E+00_JPRB, 0.16390E+00_JPRB, 0.16466E+00_JPRB ,&
&  0.16549E+00_JPRB, 0.16636E+00_JPRB, 0.16674E+00_JPRB, 0.16724E+00_JPRB, 0.16770E+00_JPRB, 0.16853E+00_JPRB, 0.17005E+00_JPRB ,&
&  0.17210E+00_JPRB, 0.17447E+00_JPRB, 0.17677E+00_JPRB, 0.17960E+00_JPRB, 0.18428E+00_JPRB, 0.19060E+00_JPRB, 0.20019E+00_JPRB ,&
&  0.20799E+00_JPRB, 0.22085E+00_JPRB, 0.22145E+00_JPRB, 0.22300E+00_JPRB, 0.22434E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.23318E+00_JPRB, 0.23367E+00_JPRB, 0.23015E+00_JPRB, 0.22702E+00_JPRB, 0.22551E+00_JPRB, 0.22504E+00_JPRB, 0.22619E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.23111E+00_JPRB, 0.23327E+00_JPRB, 0.23516E+00_JPRB, 0.23595E+00_JPRB, 0.23580E+00_JPRB, 0.23671E+00_JPRB ,&
&  0.23757E+00_JPRB, 0.23991E+00_JPRB, 0.24112E+00_JPRB, 0.24347E+00_JPRB, 0.24363E+00_JPRB, 0.24430E+00_JPRB, 0.24547E+00_JPRB ,&
&  0.24505E+00_JPRB, 0.24565E+00_JPRB, 0.24712E+00_JPRB, 0.24501E+00_JPRB, 0.24360E+00_JPRB, 0.24146E+00_JPRB, 0.23973E+00_JPRB ,&
&  0.23372E+00_JPRB, 0.22733E+00_JPRB, 0.22066E+00_JPRB, 0.22393E+00_JPRB, 0.23191E+00_JPRB, 0.23390E+00_JPRB, 0.23771E+00_JPRB ,&
&  0.23876E+00_JPRB, 0.23713E+00_JPRB, 0.23537E+00_JPRB, 0.23219E+00_JPRB, 0.22909E+00_JPRB, 0.22563E+00_JPRB, 0.22290E+00_JPRB ,&
&  0.21993E+00_JPRB, 0.22043E+00_JPRB, 0.21671E+00_JPRB, 0.20369E+00_JPRB, 0.19672E+00_JPRB, 0.19240E+00_JPRB, 0.18580E+00_JPRB ,&
&  0.17926E+00_JPRB, 0.17728E+00_JPRB, 0.17680E+00_JPRB, 0.17465E+00_JPRB, 0.17460E+00_JPRB, 0.17492E+00_JPRB, 0.17486E+00_JPRB ,&
&  0.17363E+00_JPRB, 0.17338E+00_JPRB, 0.17210E+00_JPRB, 0.17023E+00_JPRB, 0.16847E+00_JPRB, 0.16777E+00_JPRB, 0.16818E+00_JPRB ,&
&  0.16962E+00_JPRB, 0.17124E+00_JPRB, 0.17662E+00_JPRB, 0.18011E+00_JPRB, 0.17581E+00_JPRB, 0.17296E+00_JPRB, 0.17295E+00_JPRB ,&
&  0.16925E+00_JPRB, 0.16849E+00_JPRB, 0.16846E+00_JPRB, 0.16839E+00_JPRB, 0.16872E+00_JPRB, 0.16957E+00_JPRB, 0.16955E+00_JPRB ,&
&  0.17017E+00_JPRB, 0.17139E+00_JPRB, 0.17174E+00_JPRB, 0.17180E+00_JPRB, 0.17163E+00_JPRB, 0.17208E+00_JPRB, 0.17325E+00_JPRB ,&
&  0.17461E+00_JPRB, 0.17713E+00_JPRB, 0.17979E+00_JPRB, 0.18365E+00_JPRB, 0.18672E+00_JPRB, 0.18912E+00_JPRB, 0.19344E+00_JPRB ,&
&  0.19733E+00_JPRB, 0.20620E+00_JPRB, 0.22136E+00_JPRB, 0.23212E+00_JPRB, 0.22920E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.23982E+00_JPRB, 0.23980E+00_JPRB, 0.23871E+00_JPRB, 0.23768E+00_JPRB, 0.23619E+00_JPRB, 0.23469E+00_JPRB, 0.23526E+00_JPRB ,&
&  0.23755E+00_JPRB, 0.24059E+00_JPRB, 0.24353E+00_JPRB, 0.24582E+00_JPRB, 0.24761E+00_JPRB, 0.24846E+00_JPRB, 0.24983E+00_JPRB ,&
&  0.25142E+00_JPRB, 0.25411E+00_JPRB, 0.25483E+00_JPRB, 0.25544E+00_JPRB, 0.25467E+00_JPRB, 0.25396E+00_JPRB, 0.25501E+00_JPRB ,&
&  0.25816E+00_JPRB, 0.25970E+00_JPRB, 0.25892E+00_JPRB, 0.25692E+00_JPRB, 0.25745E+00_JPRB, 0.25932E+00_JPRB, 0.25871E+00_JPRB ,&
&  0.25259E+00_JPRB, 0.23951E+00_JPRB, 0.23030E+00_JPRB, 0.23291E+00_JPRB, 0.24171E+00_JPRB, 0.24576E+00_JPRB, 0.24846E+00_JPRB ,&
&  0.25018E+00_JPRB, 0.25044E+00_JPRB, 0.24965E+00_JPRB, 0.24816E+00_JPRB, 0.24559E+00_JPRB, 0.24117E+00_JPRB, 0.23590E+00_JPRB ,&
&  0.23114E+00_JPRB, 0.22865E+00_JPRB, 0.22581E+00_JPRB, 0.22084E+00_JPRB, 0.21589E+00_JPRB, 0.21274E+00_JPRB, 0.20919E+00_JPRB ,&
&  0.20559E+00_JPRB, 0.20275E+00_JPRB, 0.20202E+00_JPRB, 0.20165E+00_JPRB, 0.20161E+00_JPRB, 0.20116E+00_JPRB, 0.19913E+00_JPRB ,&
&  0.19569E+00_JPRB, 0.19154E+00_JPRB, 0.18595E+00_JPRB, 0.18095E+00_JPRB, 0.17742E+00_JPRB, 0.17399E+00_JPRB, 0.16993E+00_JPRB ,&
&  0.16787E+00_JPRB, 0.17012E+00_JPRB, 0.17827E+00_JPRB, 0.18752E+00_JPRB, 0.19329E+00_JPRB, 0.18173E+00_JPRB, 0.17740E+00_JPRB ,&
&  0.17626E+00_JPRB, 0.17588E+00_JPRB, 0.17611E+00_JPRB, 0.17739E+00_JPRB, 0.17806E+00_JPRB, 0.17788E+00_JPRB, 0.17841E+00_JPRB ,&
&  0.18024E+00_JPRB, 0.18215E+00_JPRB, 0.18319E+00_JPRB, 0.18431E+00_JPRB, 0.18498E+00_JPRB, 0.18480E+00_JPRB, 0.18448E+00_JPRB ,&
&  0.18507E+00_JPRB, 0.18782E+00_JPRB, 0.19244E+00_JPRB, 0.19904E+00_JPRB, 0.20394E+00_JPRB, 0.20845E+00_JPRB, 0.21431E+00_JPRB ,&
&  0.22175E+00_JPRB, 0.22929E+00_JPRB, 0.23331E+00_JPRB, 0.23693E+00_JPRB, 0.23874E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.24722E+00_JPRB, 0.24868E+00_JPRB, 0.25084E+00_JPRB, 0.25293E+00_JPRB, 0.25373E+00_JPRB, 0.25318E+00_JPRB, 0.25248E+00_JPRB ,&
&  0.25283E+00_JPRB, 0.25470E+00_JPRB, 0.25671E+00_JPRB, 0.25711E+00_JPRB, 0.25729E+00_JPRB, 0.25757E+00_JPRB, 0.25921E+00_JPRB ,&
&  0.26171E+00_JPRB, 0.26457E+00_JPRB, 0.26435E+00_JPRB, 0.26384E+00_JPRB, 0.26340E+00_JPRB, 0.26239E+00_JPRB, 0.26351E+00_JPRB ,&
&  0.26643E+00_JPRB, 0.26663E+00_JPRB, 0.26650E+00_JPRB, 0.26580E+00_JPRB, 0.26740E+00_JPRB, 0.27050E+00_JPRB, 0.27227E+00_JPRB ,&
&  0.27091E+00_JPRB, 0.26430E+00_JPRB, 0.25627E+00_JPRB, 0.25597E+00_JPRB, 0.25768E+00_JPRB, 0.25943E+00_JPRB, 0.25971E+00_JPRB ,&
&  0.25931E+00_JPRB, 0.25851E+00_JPRB, 0.25749E+00_JPRB, 0.25646E+00_JPRB, 0.25523E+00_JPRB, 0.25163E+00_JPRB, 0.24745E+00_JPRB ,&
&  0.24409E+00_JPRB, 0.24089E+00_JPRB, 0.23772E+00_JPRB, 0.23354E+00_JPRB, 0.22925E+00_JPRB, 0.22614E+00_JPRB, 0.22267E+00_JPRB ,&
&  0.21840E+00_JPRB, 0.21530E+00_JPRB, 0.21436E+00_JPRB, 0.21437E+00_JPRB, 0.21454E+00_JPRB, 0.21382E+00_JPRB, 0.21292E+00_JPRB ,&
&  0.21133E+00_JPRB, 0.21009E+00_JPRB, 0.20864E+00_JPRB, 0.20791E+00_JPRB, 0.20322E+00_JPRB, 0.19497E+00_JPRB, 0.18922E+00_JPRB ,&
&  0.18843E+00_JPRB, 0.19111E+00_JPRB, 0.19490E+00_JPRB, 0.19549E+00_JPRB, 0.19717E+00_JPRB, 0.19809E+00_JPRB, 0.19775E+00_JPRB ,&
&  0.19536E+00_JPRB, 0.19145E+00_JPRB, 0.19134E+00_JPRB, 0.19229E+00_JPRB, 0.19276E+00_JPRB, 0.19388E+00_JPRB, 0.19542E+00_JPRB ,&
&  0.19820E+00_JPRB, 0.20055E+00_JPRB, 0.20269E+00_JPRB, 0.20486E+00_JPRB, 0.20617E+00_JPRB, 0.20575E+00_JPRB, 0.20376E+00_JPRB ,&
&  0.20357E+00_JPRB, 0.20878E+00_JPRB, 0.21823E+00_JPRB, 0.22881E+00_JPRB, 0.23116E+00_JPRB, 0.23335E+00_JPRB, 0.23540E+00_JPRB ,&
&  0.23648E+00_JPRB, 0.23968E+00_JPRB, 0.24255E+00_JPRB, 0.24431E+00_JPRB, 0.24561E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.25876E+00_JPRB, 0.26067E+00_JPRB, 0.26155E+00_JPRB, 0.26253E+00_JPRB, 0.26464E+00_JPRB, 0.26613E+00_JPRB, 0.26638E+00_JPRB ,&
&  0.26661E+00_JPRB, 0.26781E+00_JPRB, 0.26865E+00_JPRB, 0.26898E+00_JPRB, 0.26663E+00_JPRB, 0.26624E+00_JPRB, 0.26782E+00_JPRB ,&
&  0.26993E+00_JPRB, 0.27209E+00_JPRB, 0.27213E+00_JPRB, 0.27193E+00_JPRB, 0.27136E+00_JPRB, 0.27028E+00_JPRB, 0.27178E+00_JPRB ,&
&  0.27301E+00_JPRB, 0.27354E+00_JPRB, 0.27488E+00_JPRB, 0.27624E+00_JPRB, 0.27915E+00_JPRB, 0.28256E+00_JPRB, 0.28459E+00_JPRB ,&
&  0.28477E+00_JPRB, 0.28314E+00_JPRB, 0.27676E+00_JPRB, 0.27006E+00_JPRB, 0.27052E+00_JPRB, 0.27170E+00_JPRB, 0.27171E+00_JPRB ,&
&  0.26970E+00_JPRB, 0.26696E+00_JPRB, 0.26511E+00_JPRB, 0.26364E+00_JPRB, 0.26181E+00_JPRB, 0.25865E+00_JPRB, 0.25524E+00_JPRB ,&
&  0.25225E+00_JPRB, 0.25119E+00_JPRB, 0.24978E+00_JPRB, 0.24751E+00_JPRB, 0.24421E+00_JPRB, 0.24156E+00_JPRB, 0.23884E+00_JPRB ,&
&  0.23401E+00_JPRB, 0.22930E+00_JPRB, 0.22675E+00_JPRB, 0.22617E+00_JPRB, 0.22659E+00_JPRB, 0.22620E+00_JPRB, 0.22456E+00_JPRB ,&
&  0.22269E+00_JPRB, 0.22025E+00_JPRB, 0.21876E+00_JPRB, 0.21831E+00_JPRB, 0.21937E+00_JPRB, 0.21828E+00_JPRB, 0.21409E+00_JPRB ,&
&  0.21165E+00_JPRB, 0.21003E+00_JPRB, 0.20999E+00_JPRB, 0.20894E+00_JPRB, 0.20800E+00_JPRB, 0.20672E+00_JPRB, 0.20709E+00_JPRB ,&
&  0.20808E+00_JPRB, 0.20843E+00_JPRB, 0.21095E+00_JPRB, 0.21544E+00_JPRB, 0.21957E+00_JPRB, 0.22197E+00_JPRB, 0.22337E+00_JPRB ,&
&  0.22386E+00_JPRB, 0.22316E+00_JPRB, 0.22249E+00_JPRB, 0.22291E+00_JPRB, 0.22476E+00_JPRB, 0.22732E+00_JPRB, 0.22985E+00_JPRB ,&
&  0.23059E+00_JPRB, 0.23167E+00_JPRB, 0.23413E+00_JPRB, 0.23785E+00_JPRB, 0.24011E+00_JPRB, 0.24199E+00_JPRB, 0.24369E+00_JPRB ,&
&  0.24463E+00_JPRB, 0.24623E+00_JPRB, 0.25010E+00_JPRB, 0.25275E+00_JPRB, 0.25544E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.26962E+00_JPRB, 0.27041E+00_JPRB, 0.27113E+00_JPRB, 0.27106E+00_JPRB, 0.27205E+00_JPRB, 0.27291E+00_JPRB, 0.27440E+00_JPRB ,&
&  0.27511E+00_JPRB, 0.27661E+00_JPRB, 0.27868E+00_JPRB, 0.27861E+00_JPRB, 0.27820E+00_JPRB, 0.27794E+00_JPRB, 0.27847E+00_JPRB ,&
&  0.28065E+00_JPRB, 0.28183E+00_JPRB, 0.28127E+00_JPRB, 0.28096E+00_JPRB, 0.28022E+00_JPRB, 0.27961E+00_JPRB, 0.28162E+00_JPRB ,&
&  0.28208E+00_JPRB, 0.28280E+00_JPRB, 0.28464E+00_JPRB, 0.28618E+00_JPRB, 0.28981E+00_JPRB, 0.29423E+00_JPRB, 0.29729E+00_JPRB ,&
&  0.29721E+00_JPRB, 0.29418E+00_JPRB, 0.28916E+00_JPRB, 0.28537E+00_JPRB, 0.28475E+00_JPRB, 0.28579E+00_JPRB, 0.28814E+00_JPRB ,&
&  0.28739E+00_JPRB, 0.28188E+00_JPRB, 0.27481E+00_JPRB, 0.27093E+00_JPRB, 0.26947E+00_JPRB, 0.26755E+00_JPRB, 0.26440E+00_JPRB ,&
&  0.26184E+00_JPRB, 0.26115E+00_JPRB, 0.26121E+00_JPRB, 0.26040E+00_JPRB, 0.25877E+00_JPRB, 0.25599E+00_JPRB, 0.25242E+00_JPRB ,&
&  0.24931E+00_JPRB, 0.24619E+00_JPRB, 0.24425E+00_JPRB, 0.24287E+00_JPRB, 0.24273E+00_JPRB, 0.24293E+00_JPRB, 0.24216E+00_JPRB ,&
&  0.24037E+00_JPRB, 0.23924E+00_JPRB, 0.23768E+00_JPRB, 0.23459E+00_JPRB, 0.23236E+00_JPRB, 0.23164E+00_JPRB, 0.23051E+00_JPRB ,&
&  0.22893E+00_JPRB, 0.22831E+00_JPRB, 0.22803E+00_JPRB, 0.22749E+00_JPRB, 0.22823E+00_JPRB, 0.22982E+00_JPRB, 0.22738E+00_JPRB ,&
&  0.22529E+00_JPRB, 0.22394E+00_JPRB, 0.22408E+00_JPRB, 0.22568E+00_JPRB, 0.22826E+00_JPRB, 0.23104E+00_JPRB, 0.23334E+00_JPRB ,&
&  0.23432E+00_JPRB, 0.23459E+00_JPRB, 0.23491E+00_JPRB, 0.23498E+00_JPRB, 0.23592E+00_JPRB, 0.23801E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.24195E+00_JPRB, 0.24442E+00_JPRB, 0.24705E+00_JPRB, 0.25035E+00_JPRB, 0.25228E+00_JPRB, 0.25414E+00_JPRB, 0.25605E+00_JPRB ,&
&  0.25761E+00_JPRB, 0.25961E+00_JPRB, 0.26246E+00_JPRB, 0.26481E+00_JPRB, 0.26730E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.28371E+00_JPRB, 0.28234E+00_JPRB, 0.28140E+00_JPRB, 0.28183E+00_JPRB, 0.28406E+00_JPRB, 0.28466E+00_JPRB, 0.28462E+00_JPRB ,&
&  0.28437E+00_JPRB, 0.28622E+00_JPRB, 0.28972E+00_JPRB, 0.29185E+00_JPRB, 0.29208E+00_JPRB, 0.29183E+00_JPRB, 0.29135E+00_JPRB ,&
&  0.29109E+00_JPRB, 0.29085E+00_JPRB, 0.28972E+00_JPRB, 0.28974E+00_JPRB, 0.28980E+00_JPRB, 0.28900E+00_JPRB, 0.28987E+00_JPRB ,&
&  0.28999E+00_JPRB, 0.29043E+00_JPRB, 0.29263E+00_JPRB, 0.29439E+00_JPRB, 0.29809E+00_JPRB, 0.30392E+00_JPRB, 0.30673E+00_JPRB ,&
&  0.30611E+00_JPRB, 0.30203E+00_JPRB, 0.29874E+00_JPRB, 0.29941E+00_JPRB, 0.30124E+00_JPRB, 0.30493E+00_JPRB, 0.30894E+00_JPRB ,&
&  0.30995E+00_JPRB, 0.30416E+00_JPRB, 0.29515E+00_JPRB, 0.28876E+00_JPRB, 0.28459E+00_JPRB, 0.28180E+00_JPRB, 0.27940E+00_JPRB ,&
&  0.27652E+00_JPRB, 0.27438E+00_JPRB, 0.27338E+00_JPRB, 0.27322E+00_JPRB, 0.27288E+00_JPRB, 0.27017E+00_JPRB, 0.26712E+00_JPRB ,&
&  0.26313E+00_JPRB, 0.26037E+00_JPRB, 0.25808E+00_JPRB, 0.25730E+00_JPRB, 0.25743E+00_JPRB, 0.25722E+00_JPRB, 0.25626E+00_JPRB ,&
&  0.25504E+00_JPRB, 0.25467E+00_JPRB, 0.25432E+00_JPRB, 0.25317E+00_JPRB, 0.25176E+00_JPRB, 0.25136E+00_JPRB, 0.25028E+00_JPRB ,&
&  0.24810E+00_JPRB, 0.24677E+00_JPRB, 0.24676E+00_JPRB, 0.24769E+00_JPRB, 0.24758E+00_JPRB, 0.24829E+00_JPRB, 0.24975E+00_JPRB ,&
&  0.24882E+00_JPRB, 0.24643E+00_JPRB, 0.24529E+00_JPRB, 0.24597E+00_JPRB, 0.24784E+00_JPRB, 0.25028E+00_JPRB, 0.25155E+00_JPRB ,&
&  0.25267E+00_JPRB, 0.25271E+00_JPRB, 0.25361E+00_JPRB, 0.25490E+00_JPRB, 0.25642E+00_JPRB, 0.25724E+00_JPRB, 0.25756E+00_JPRB ,&
&  0.25800E+00_JPRB, 0.25858E+00_JPRB, 0.26066E+00_JPRB, 0.26392E+00_JPRB, 0.26692E+00_JPRB, 0.27033E+00_JPRB, 0.27310E+00_JPRB ,&
&  0.27514E+00_JPRB, 0.27739E+00_JPRB, 0.27912E+00_JPRB, 0.28038E+00_JPRB, 0.28202E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.30589E+00_JPRB, 0.30266E+00_JPRB, 0.30048E+00_JPRB, 0.30049E+00_JPRB, 0.30252E+00_JPRB, 0.30292E+00_JPRB, 0.30140E+00_JPRB ,&
&  0.29991E+00_JPRB, 0.30115E+00_JPRB, 0.30410E+00_JPRB, 0.30558E+00_JPRB, 0.30601E+00_JPRB, 0.30662E+00_JPRB, 0.30548E+00_JPRB ,&
&  0.30219E+00_JPRB, 0.29884E+00_JPRB, 0.29908E+00_JPRB, 0.30017E+00_JPRB, 0.30109E+00_JPRB, 0.29975E+00_JPRB, 0.29815E+00_JPRB ,&
&  0.29868E+00_JPRB, 0.29874E+00_JPRB, 0.30044E+00_JPRB, 0.30234E+00_JPRB, 0.30591E+00_JPRB, 0.31153E+00_JPRB, 0.31563E+00_JPRB ,&
&  0.31677E+00_JPRB, 0.31500E+00_JPRB, 0.31456E+00_JPRB, 0.31709E+00_JPRB, 0.31991E+00_JPRB, 0.32651E+00_JPRB, 0.33334E+00_JPRB ,&
&  0.33608E+00_JPRB, 0.33368E+00_JPRB, 0.32388E+00_JPRB, 0.31537E+00_JPRB, 0.30982E+00_JPRB, 0.30425E+00_JPRB, 0.29877E+00_JPRB ,&
&  0.29294E+00_JPRB, 0.28860E+00_JPRB, 0.28597E+00_JPRB, 0.28472E+00_JPRB, 0.28422E+00_JPRB, 0.28351E+00_JPRB, 0.28136E+00_JPRB ,&
&  0.27862E+00_JPRB, 0.27652E+00_JPRB, 0.27492E+00_JPRB, 0.27369E+00_JPRB, 0.27253E+00_JPRB, 0.27201E+00_JPRB, 0.27140E+00_JPRB ,&
&  0.27111E+00_JPRB, 0.27149E+00_JPRB, 0.27225E+00_JPRB, 0.27222E+00_JPRB, 0.27200E+00_JPRB, 0.27126E+00_JPRB, 0.27048E+00_JPRB ,&
&  0.26924E+00_JPRB, 0.26691E+00_JPRB, 0.26497E+00_JPRB, 0.26447E+00_JPRB, 0.26420E+00_JPRB, 0.26452E+00_JPRB, 0.26484E+00_JPRB ,&
&  0.26456E+00_JPRB, 0.26456E+00_JPRB, 0.26539E+00_JPRB, 0.26658E+00_JPRB, 0.26755E+00_JPRB, 0.26816E+00_JPRB, 0.26887E+00_JPRB ,&
&  0.27052E+00_JPRB, 0.27240E+00_JPRB, 0.27368E+00_JPRB, 0.27462E+00_JPRB, 0.27519E+00_JPRB, 0.27629E+00_JPRB, 0.27635E+00_JPRB ,&
&  0.27655E+00_JPRB, 0.27775E+00_JPRB, 0.28071E+00_JPRB, 0.28506E+00_JPRB, 0.28974E+00_JPRB, 0.29351E+00_JPRB, 0.29723E+00_JPRB ,&
&  0.30040E+00_JPRB, 0.30300E+00_JPRB, 0.30460E+00_JPRB, 0.30528E+00_JPRB, 0.30608E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.32995E+00_JPRB, 0.32673E+00_JPRB, 0.32436E+00_JPRB, 0.32233E+00_JPRB, 0.32245E+00_JPRB, 0.32141E+00_JPRB, 0.31870E+00_JPRB ,&
&  0.31675E+00_JPRB, 0.31795E+00_JPRB, 0.32033E+00_JPRB, 0.32117E+00_JPRB, 0.32025E+00_JPRB, 0.31871E+00_JPRB, 0.31585E+00_JPRB ,&
&  0.31199E+00_JPRB, 0.31008E+00_JPRB, 0.30971E+00_JPRB, 0.31011E+00_JPRB, 0.30981E+00_JPRB, 0.30816E+00_JPRB, 0.30697E+00_JPRB ,&
&  0.30797E+00_JPRB, 0.30941E+00_JPRB, 0.30976E+00_JPRB, 0.31192E+00_JPRB, 0.31577E+00_JPRB, 0.32263E+00_JPRB, 0.32895E+00_JPRB ,&
&  0.33124E+00_JPRB, 0.33187E+00_JPRB, 0.32626E+00_JPRB, 0.33752E+00_JPRB, 0.34413E+00_JPRB, 0.35291E+00_JPRB, 0.35831E+00_JPRB ,&
&  0.36038E+00_JPRB, 0.35726E+00_JPRB, 0.34720E+00_JPRB, 0.33901E+00_JPRB, 0.33382E+00_JPRB, 0.32876E+00_JPRB, 0.32092E+00_JPRB ,&
&  0.31277E+00_JPRB, 0.30563E+00_JPRB, 0.30130E+00_JPRB, 0.29920E+00_JPRB, 0.29921E+00_JPRB, 0.29967E+00_JPRB, 0.29802E+00_JPRB ,&
&  0.29620E+00_JPRB, 0.29444E+00_JPRB, 0.29376E+00_JPRB, 0.29302E+00_JPRB, 0.29184E+00_JPRB, 0.29091E+00_JPRB, 0.29063E+00_JPRB ,&
&  0.29064E+00_JPRB, 0.29038E+00_JPRB, 0.29044E+00_JPRB, 0.29101E+00_JPRB, 0.29126E+00_JPRB, 0.29059E+00_JPRB, 0.28920E+00_JPRB ,&
&  0.28766E+00_JPRB, 0.28636E+00_JPRB, 0.28523E+00_JPRB, 0.28407E+00_JPRB, 0.28297E+00_JPRB, 0.28211E+00_JPRB, 0.28117E+00_JPRB ,&
&  0.28008E+00_JPRB, 0.28030E+00_JPRB, 0.28157E+00_JPRB, 0.28227E+00_JPRB, 0.28270E+00_JPRB, 0.28377E+00_JPRB, 0.28562E+00_JPRB ,&
&  0.28739E+00_JPRB, 0.28932E+00_JPRB, 0.29071E+00_JPRB, 0.29127E+00_JPRB, 0.29065E+00_JPRB, 0.29057E+00_JPRB, 0.29125E+00_JPRB ,&
&  0.29306E+00_JPRB, 0.29600E+00_JPRB, 0.30042E+00_JPRB, 0.30676E+00_JPRB, 0.31449E+00_JPRB, 0.32058E+00_JPRB, 0.32588E+00_JPRB ,&
&  0.32993E+00_JPRB, 0.33162E+00_JPRB, 0.33184E+00_JPRB, 0.33258E+00_JPRB, 0.33237E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.35329E+00_JPRB, 0.35065E+00_JPRB, 0.34858E+00_JPRB, 0.34500E+00_JPRB, 0.34251E+00_JPRB, 0.34075E+00_JPRB, 0.33867E+00_JPRB ,&
&  0.33713E+00_JPRB, 0.33748E+00_JPRB, 0.33640E+00_JPRB, 0.33590E+00_JPRB, 0.33507E+00_JPRB, 0.33243E+00_JPRB, 0.32904E+00_JPRB ,&
&  0.32708E+00_JPRB, 0.32524E+00_JPRB, 0.32358E+00_JPRB, 0.32175E+00_JPRB, 0.32000E+00_JPRB, 0.31767E+00_JPRB, 0.31781E+00_JPRB ,&
&  0.31904E+00_JPRB, 0.32176E+00_JPRB, 0.32484E+00_JPRB, 0.32844E+00_JPRB, 0.33372E+00_JPRB, 0.34087E+00_JPRB, 0.34764E+00_JPRB ,&
&  0.35493E+00_JPRB, 0.34301E+00_JPRB, 0.35599E+00_JPRB, 0.36341E+00_JPRB, 0.37592E+00_JPRB, 0.38599E+00_JPRB, 0.38732E+00_JPRB ,&
&  0.38323E+00_JPRB, 0.37164E+00_JPRB, 0.36021E+00_JPRB, 0.35239E+00_JPRB, 0.34968E+00_JPRB, 0.34761E+00_JPRB, 0.34226E+00_JPRB ,&
&  0.33574E+00_JPRB, 0.32830E+00_JPRB, 0.32256E+00_JPRB, 0.31934E+00_JPRB, 0.31817E+00_JPRB, 0.31797E+00_JPRB, 0.31673E+00_JPRB ,&
&  0.31546E+00_JPRB, 0.31414E+00_JPRB, 0.31350E+00_JPRB, 0.31317E+00_JPRB, 0.31274E+00_JPRB, 0.31291E+00_JPRB, 0.31336E+00_JPRB ,&
&  0.31348E+00_JPRB, 0.31338E+00_JPRB, 0.31324E+00_JPRB, 0.31369E+00_JPRB, 0.31405E+00_JPRB, 0.31399E+00_JPRB, 0.31364E+00_JPRB ,&
&  0.31267E+00_JPRB, 0.31184E+00_JPRB, 0.31068E+00_JPRB, 0.30987E+00_JPRB, 0.30838E+00_JPRB, 0.30603E+00_JPRB, 0.30361E+00_JPRB ,&
&  0.30214E+00_JPRB, 0.30162E+00_JPRB, 0.30152E+00_JPRB, 0.30177E+00_JPRB, 0.30245E+00_JPRB, 0.30363E+00_JPRB, 0.30492E+00_JPRB ,&
&  0.30615E+00_JPRB, 0.30719E+00_JPRB, 0.30756E+00_JPRB, 0.30762E+00_JPRB, 0.30706E+00_JPRB, 0.30732E+00_JPRB, 0.30878E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.31706E+00_JPRB, 0.32400E+00_JPRB, 0.33576E+00_JPRB, 0.34762E+00_JPRB, 0.35511E+00_JPRB, 0.36026E+00_JPRB ,&
&  0.36100E+00_JPRB, 0.35895E+00_JPRB, 0.35664E+00_JPRB, 0.35609E+00_JPRB, 0.35581E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.37055E+00_JPRB, 0.36902E+00_JPRB, 0.36757E+00_JPRB, 0.36534E+00_JPRB, 0.36319E+00_JPRB, 0.36137E+00_JPRB, 0.35802E+00_JPRB ,&
&  0.35681E+00_JPRB, 0.35342E+00_JPRB, 0.35027E+00_JPRB, 0.34917E+00_JPRB, 0.34770E+00_JPRB, 0.34542E+00_JPRB, 0.34348E+00_JPRB ,&
&  0.34116E+00_JPRB, 0.33859E+00_JPRB, 0.33686E+00_JPRB, 0.33457E+00_JPRB, 0.33122E+00_JPRB, 0.32878E+00_JPRB, 0.32926E+00_JPRB ,&
&  0.33016E+00_JPRB, 0.33272E+00_JPRB, 0.34032E+00_JPRB, 0.34805E+00_JPRB, 0.35618E+00_JPRB, 0.36381E+00_JPRB, 0.37128E+00_JPRB ,&
&  0.38214E+00_JPRB, 0.34248E+00_JPRB, 0.36330E+00_JPRB, 0.37747E+00_JPRB, 0.39891E+00_JPRB, 0.41823E+00_JPRB, 0.41623E+00_JPRB ,&
&  0.40380E+00_JPRB, 0.38683E+00_JPRB, 0.37341E+00_JPRB, 0.36439E+00_JPRB, 0.36270E+00_JPRB, 0.36201E+00_JPRB, 0.35914E+00_JPRB ,&
&  0.35566E+00_JPRB, 0.34955E+00_JPRB, 0.34309E+00_JPRB, 0.33886E+00_JPRB, 0.33657E+00_JPRB, 0.33489E+00_JPRB, 0.33311E+00_JPRB ,&
&  0.33218E+00_JPRB, 0.33213E+00_JPRB, 0.33212E+00_JPRB, 0.33202E+00_JPRB, 0.33250E+00_JPRB, 0.33365E+00_JPRB, 0.33487E+00_JPRB ,&
&  0.33608E+00_JPRB, 0.33665E+00_JPRB, 0.33800E+00_JPRB, 0.34034E+00_JPRB, 0.34356E+00_JPRB, 0.34661E+00_JPRB, 0.34826E+00_JPRB ,&
&  0.34822E+00_JPRB, 0.34734E+00_JPRB, 0.34591E+00_JPRB, 0.34366E+00_JPRB, 0.34037E+00_JPRB, 0.33623E+00_JPRB, 0.33256E+00_JPRB ,&
&  0.32919E+00_JPRB, 0.32636E+00_JPRB, 0.32241E+00_JPRB, 0.32108E+00_JPRB, 0.32141E+00_JPRB, 0.32184E+00_JPRB, 0.32240E+00_JPRB ,&
&  0.32290E+00_JPRB, 0.32268E+00_JPRB, 0.32217E+00_JPRB, 0.32172E+00_JPRB, 0.32171E+00_JPRB, 0.32267E+00_JPRB, 0.32489E+00_JPRB ,&
&  0.32831E+00_JPRB, 0.33480E+00_JPRB, 0.34373E+00_JPRB, 0.35919E+00_JPRB, 0.37277E+00_JPRB, 0.37951E+00_JPRB, 0.38215E+00_JPRB ,&
&  0.37966E+00_JPRB, 0.37560E+00_JPRB, 0.37225E+00_JPRB, 0.36988E+00_JPRB, 0.37080E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.39613E+00_JPRB, 0.39315E+00_JPRB, 0.39144E+00_JPRB, 0.38853E+00_JPRB, 0.38506E+00_JPRB, 0.38142E+00_JPRB, 0.37734E+00_JPRB ,&
&  0.37291E+00_JPRB, 0.37019E+00_JPRB, 0.36768E+00_JPRB, 0.36452E+00_JPRB, 0.36131E+00_JPRB, 0.35871E+00_JPRB, 0.35605E+00_JPRB ,&
&  0.35315E+00_JPRB, 0.34995E+00_JPRB, 0.34775E+00_JPRB, 0.34624E+00_JPRB, 0.34451E+00_JPRB, 0.34387E+00_JPRB, 0.34481E+00_JPRB ,&
&  0.34484E+00_JPRB, 0.34713E+00_JPRB, 0.35664E+00_JPRB, 0.36772E+00_JPRB, 0.38051E+00_JPRB, 0.39146E+00_JPRB, 0.40572E+00_JPRB ,&
&  0.39240E+00_JPRB, 0.35331E+00_JPRB, 0.37639E+00_JPRB, 0.40239E+00_JPRB, 0.42969E+00_JPRB, 0.45522E+00_JPRB, 0.44846E+00_JPRB ,&
&  0.43688E+00_JPRB, 0.42229E+00_JPRB, 0.40928E+00_JPRB, 0.39893E+00_JPRB, 0.39461E+00_JPRB, 0.39026E+00_JPRB, 0.38597E+00_JPRB ,&
&  0.38096E+00_JPRB, 0.37391E+00_JPRB, 0.36708E+00_JPRB, 0.36112E+00_JPRB, 0.35711E+00_JPRB, 0.35397E+00_JPRB, 0.35122E+00_JPRB ,&
&  0.35011E+00_JPRB, 0.35004E+00_JPRB, 0.35010E+00_JPRB, 0.35022E+00_JPRB, 0.35067E+00_JPRB, 0.35250E+00_JPRB, 0.35527E+00_JPRB ,&
&  0.35915E+00_JPRB, 0.36512E+00_JPRB, 0.37419E+00_JPRB, 0.38436E+00_JPRB, 0.39382E+00_JPRB, 0.39984E+00_JPRB, 0.40155E+00_JPRB ,&
&  0.40113E+00_JPRB, 0.39832E+00_JPRB, 0.39369E+00_JPRB, 0.38697E+00_JPRB, 0.37855E+00_JPRB, 0.37063E+00_JPRB, 0.36386E+00_JPRB ,&
&  0.35814E+00_JPRB, 0.35226E+00_JPRB, 0.34691E+00_JPRB, 0.34425E+00_JPRB, 0.34341E+00_JPRB, 0.34249E+00_JPRB, 0.34156E+00_JPRB ,&
&  0.34105E+00_JPRB, 0.34056E+00_JPRB, 0.34031E+00_JPRB, 0.34082E+00_JPRB, 0.34257E+00_JPRB, 0.34567E+00_JPRB, 0.34898E+00_JPRB ,&
&  0.35172E+00_JPRB, 0.35851E+00_JPRB, 0.36753E+00_JPRB, 0.38786E+00_JPRB, 0.40156E+00_JPRB, 0.40713E+00_JPRB, 0.40725E+00_JPRB ,&
&  0.40419E+00_JPRB, 0.39996E+00_JPRB, 0.39789E+00_JPRB, 0.39578E+00_JPRB, 0.39620E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.42956E+00_JPRB, 0.42761E+00_JPRB, 0.42164E+00_JPRB, 0.41139E+00_JPRB, 0.40325E+00_JPRB, 0.39713E+00_JPRB, 0.39311E+00_JPRB ,&
&  0.39090E+00_JPRB, 0.38896E+00_JPRB, 0.38627E+00_JPRB, 0.38143E+00_JPRB, 0.37725E+00_JPRB, 0.37214E+00_JPRB, 0.36688E+00_JPRB ,&
&  0.36247E+00_JPRB, 0.35839E+00_JPRB, 0.35608E+00_JPRB, 0.35301E+00_JPRB, 0.35159E+00_JPRB, 0.35275E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.35714E+00_JPRB, 0.35969E+00_JPRB, 0.36809E+00_JPRB, 0.37834E+00_JPRB, 0.39645E+00_JPRB, 0.40871E+00_JPRB, 0.42620E+00_JPRB ,&
&  0.46470E+00_JPRB, 0.39384E+00_JPRB, 0.42093E+00_JPRB, 0.45261E+00_JPRB, 0.47741E+00_JPRB, 0.50172E+00_JPRB, 0.48973E+00_JPRB ,&
&  0.47894E+00_JPRB, 0.46930E+00_JPRB, 0.45967E+00_JPRB, 0.45062E+00_JPRB, 0.44369E+00_JPRB, 0.43147E+00_JPRB, 0.41768E+00_JPRB ,&
&  0.40299E+00_JPRB, 0.39252E+00_JPRB, 0.38456E+00_JPRB, 0.37727E+00_JPRB, 0.37104E+00_JPRB, 0.36716E+00_JPRB, 0.36539E+00_JPRB ,&
&  0.36531E+00_JPRB, 0.36554E+00_JPRB, 0.36528E+00_JPRB, 0.36552E+00_JPRB, 0.36652E+00_JPRB, 0.36960E+00_JPRB, 0.37579E+00_JPRB ,&
&  0.38951E+00_JPRB, 0.40954E+00_JPRB, 0.43009E+00_JPRB, 0.44785E+00_JPRB, 0.45978E+00_JPRB, 0.46213E+00_JPRB, 0.45915E+00_JPRB ,&
&  0.45065E+00_JPRB, 0.43875E+00_JPRB, 0.42706E+00_JPRB, 0.41625E+00_JPRB, 0.40677E+00_JPRB, 0.39871E+00_JPRB, 0.39074E+00_JPRB ,&
&  0.38435E+00_JPRB, 0.37893E+00_JPRB, 0.37526E+00_JPRB, 0.37241E+00_JPRB, 0.36949E+00_JPRB, 0.36703E+00_JPRB, 0.36615E+00_JPRB ,&
&  0.36550E+00_JPRB, 0.36507E+00_JPRB, 0.36534E+00_JPRB, 0.36761E+00_JPRB, 0.37138E+00_JPRB, 0.37449E+00_JPRB, 0.37656E+00_JPRB ,&
&  0.37781E+00_JPRB, 0.38428E+00_JPRB, 0.39075E+00_JPRB, 0.41893E+00_JPRB, 0.43259E+00_JPRB, 0.43607E+00_JPRB, 0.43612E+00_JPRB ,&
&  0.43582E+00_JPRB, 0.43008E+00_JPRB, 0.38393E+00_JPRB, 0.41200E+00_JPRB, 0.42438E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.47126E+00_JPRB, 0.46268E+00_JPRB, 0.44935E+00_JPRB, 0.43617E+00_JPRB, 0.42820E+00_JPRB, 0.42337E+00_JPRB, 0.42004E+00_JPRB ,&
&  0.41607E+00_JPRB, 0.41109E+00_JPRB, 0.40595E+00_JPRB, 0.40269E+00_JPRB, 0.39863E+00_JPRB, 0.39306E+00_JPRB, 0.38631E+00_JPRB ,&
&  0.38007E+00_JPRB, 0.37428E+00_JPRB, 0.36985E+00_JPRB, 0.36487E+00_JPRB, 0.36094E+00_JPRB, 0.36091E+00_JPRB, 0.36240E+00_JPRB ,&
&  0.36783E+00_JPRB, 0.37533E+00_JPRB, 0.38226E+00_JPRB, 0.39623E+00_JPRB, 0.41207E+00_JPRB, 0.43418E+00_JPRB, 0.45568E+00_JPRB ,&
&  0.49486E+00_JPRB, 0.42856E+00_JPRB, 0.44759E+00_JPRB, 0.48971E+00_JPRB, 0.51782E+00_JPRB, 0.54138E+00_JPRB, 0.54129E+00_JPRB ,&
&  0.53758E+00_JPRB, 0.53416E+00_JPRB, 0.52900E+00_JPRB, 0.51970E+00_JPRB, 0.50096E+00_JPRB, 0.46899E+00_JPRB, 0.43844E+00_JPRB ,&
&  0.41722E+00_JPRB, 0.40993E+00_JPRB, 0.40682E+00_JPRB, 0.40255E+00_JPRB, 0.39832E+00_JPRB, 0.39671E+00_JPRB, 0.39764E+00_JPRB ,&
&  0.39813E+00_JPRB, 0.39797E+00_JPRB, 0.39690E+00_JPRB, 0.39547E+00_JPRB, 0.39564E+00_JPRB, 0.40438E+00_JPRB, 0.42625E+00_JPRB ,&
&  0.46303E+00_JPRB, 0.50188E+00_JPRB, 0.53341E+00_JPRB, 0.55509E+00_JPRB, 0.55744E+00_JPRB, 0.54510E+00_JPRB, 0.52170E+00_JPRB ,&
&  0.49474E+00_JPRB, 0.47150E+00_JPRB, 0.45475E+00_JPRB, 0.44161E+00_JPRB, 0.43131E+00_JPRB, 0.42340E+00_JPRB, 0.41856E+00_JPRB ,&
&  0.41639E+00_JPRB, 0.41500E+00_JPRB, 0.41337E+00_JPRB, 0.41050E+00_JPRB, 0.40814E+00_JPRB, 0.40609E+00_JPRB, 0.40479E+00_JPRB ,&
&  0.40328E+00_JPRB, 0.40196E+00_JPRB, 0.40258E+00_JPRB, 0.40504E+00_JPRB, 0.40666E+00_JPRB, 0.40660E+00_JPRB, 0.40661E+00_JPRB ,&
&  0.40900E+00_JPRB, 0.41859E+00_JPRB, 0.43006E+00_JPRB, 0.49310E+00_JPRB, 0.50972E+00_JPRB, 0.49784E+00_JPRB, 0.48484E+00_JPRB ,&
&  0.47797E+00_JPRB, 0.46888E+00_JPRB, 0.46760E+00_JPRB, 0.44122E+00_JPRB, 0.47264E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.49752E+00_JPRB, 0.49246E+00_JPRB, 0.47906E+00_JPRB, 0.46482E+00_JPRB, 0.45229E+00_JPRB, 0.44442E+00_JPRB, 0.43798E+00_JPRB ,&
&  0.43365E+00_JPRB, 0.43051E+00_JPRB, 0.42822E+00_JPRB, 0.42526E+00_JPRB, 0.42097E+00_JPRB, 0.41474E+00_JPRB, 0.40678E+00_JPRB ,&
&  0.39949E+00_JPRB, 0.39233E+00_JPRB, 0.38639E+00_JPRB, 0.37978E+00_JPRB, 0.37164E+00_JPRB, 0.36754E+00_JPRB, 0.36992E+00_JPRB ,&
&  0.37559E+00_JPRB, 0.38262E+00_JPRB, 0.38921E+00_JPRB, 0.39934E+00_JPRB, 0.42195E+00_JPRB, 0.44517E+00_JPRB, 0.46676E+00_JPRB ,&
&  0.50987E+00_JPRB, 0.47716E+00_JPRB, 0.49876E+00_JPRB, 0.52894E+00_JPRB, 0.56048E+00_JPRB, 0.58594E+00_JPRB, 0.60475E+00_JPRB ,&
&  0.61846E+00_JPRB, 0.62577E+00_JPRB, 0.61814E+00_JPRB, 0.59026E+00_JPRB, 0.54496E+00_JPRB, 0.50528E+00_JPRB, 0.48213E+00_JPRB ,&
&  0.47028E+00_JPRB, 0.46505E+00_JPRB, 0.46023E+00_JPRB, 0.45611E+00_JPRB, 0.45354E+00_JPRB, 0.45124E+00_JPRB, 0.44868E+00_JPRB ,&
&  0.44616E+00_JPRB, 0.44424E+00_JPRB, 0.44305E+00_JPRB, 0.44334E+00_JPRB, 0.45084E+00_JPRB, 0.47460E+00_JPRB, 0.51321E+00_JPRB ,&
&  0.56937E+00_JPRB, 0.61530E+00_JPRB, 0.63293E+00_JPRB, 0.64429E+00_JPRB, 0.65206E+00_JPRB, 0.63468E+00_JPRB, 0.58771E+00_JPRB ,&
&  0.54437E+00_JPRB, 0.50860E+00_JPRB, 0.48338E+00_JPRB, 0.46619E+00_JPRB, 0.45538E+00_JPRB, 0.44960E+00_JPRB, 0.44824E+00_JPRB ,&
&  0.44906E+00_JPRB, 0.45066E+00_JPRB, 0.45403E+00_JPRB, 0.45629E+00_JPRB, 0.45748E+00_JPRB, 0.45602E+00_JPRB, 0.45071E+00_JPRB ,&
&  0.44337E+00_JPRB, 0.43793E+00_JPRB, 0.43536E+00_JPRB, 0.43390E+00_JPRB, 0.43252E+00_JPRB, 0.43049E+00_JPRB, 0.42929E+00_JPRB ,&
&  0.43026E+00_JPRB, 0.45030E+00_JPRB, 0.48213E+00_JPRB, 0.55890E+00_JPRB, 0.55368E+00_JPRB, 0.54072E+00_JPRB, 0.52091E+00_JPRB ,&
&  0.50766E+00_JPRB, 0.50140E+00_JPRB, 0.48963E+00_JPRB, 0.49072E+00_JPRB, 0.50043E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.56030E+00_JPRB, 0.54121E+00_JPRB, 0.52139E+00_JPRB, 0.49959E+00_JPRB, 0.48088E+00_JPRB, 0.46756E+00_JPRB, 0.45773E+00_JPRB ,&
&  0.45138E+00_JPRB, 0.44755E+00_JPRB, 0.44476E+00_JPRB, 0.44243E+00_JPRB, 0.43951E+00_JPRB, 0.43633E+00_JPRB, 0.42935E+00_JPRB ,&
&  0.42082E+00_JPRB, 0.41089E+00_JPRB, 0.40318E+00_JPRB, 0.39642E+00_JPRB, 0.38438E+00_JPRB, 0.37737E+00_JPRB, 0.37892E+00_JPRB ,&
&  0.38368E+00_JPRB, 0.38970E+00_JPRB, 0.39460E+00_JPRB, 0.40531E+00_JPRB, 0.42544E+00_JPRB, 0.44828E+00_JPRB, 0.47613E+00_JPRB ,&
&  0.52544E+00_JPRB, 0.52566E+00_JPRB, 0.57626E+00_JPRB, 0.59747E+00_JPRB, 0.61850E+00_JPRB, 0.66232E+00_JPRB, 0.69893E+00_JPRB ,&
&  0.71810E+00_JPRB, 0.75852E+00_JPRB, 0.74528E+00_JPRB, 0.68422E+00_JPRB, 0.62551E+00_JPRB, 0.58379E+00_JPRB, 0.55883E+00_JPRB ,&
&  0.54000E+00_JPRB, 0.52395E+00_JPRB, 0.51002E+00_JPRB, 0.50130E+00_JPRB, 0.49261E+00_JPRB, 0.48382E+00_JPRB, 0.47527E+00_JPRB ,&
&  0.47071E+00_JPRB, 0.46808E+00_JPRB, 0.46797E+00_JPRB, 0.47853E+00_JPRB, 0.51281E+00_JPRB, 0.57410E+00_JPRB, 0.64562E+00_JPRB ,&
&  0.70059E+00_JPRB, 0.72860E+00_JPRB, 0.75139E+00_JPRB, 0.74380E+00_JPRB, 0.71324E+00_JPRB, 0.68353E+00_JPRB, 0.65212E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.57022E+00_JPRB, 0.53849E+00_JPRB, 0.51638E+00_JPRB, 0.50265E+00_JPRB, 0.49556E+00_JPRB, 0.49258E+00_JPRB ,&
&  0.49058E+00_JPRB, 0.49250E+00_JPRB, 0.50151E+00_JPRB, 0.51144E+00_JPRB, 0.51301E+00_JPRB, 0.50513E+00_JPRB, 0.49187E+00_JPRB ,&
&  0.47938E+00_JPRB, 0.46870E+00_JPRB, 0.46649E+00_JPRB, 0.46593E+00_JPRB, 0.46662E+00_JPRB, 0.46896E+00_JPRB, 0.46942E+00_JPRB ,&
&  0.47216E+00_JPRB, 0.51145E+00_JPRB, 0.54891E+00_JPRB, 0.65026E+00_JPRB, 0.69812E+00_JPRB, 0.65661E+00_JPRB, 0.60773E+00_JPRB ,&
&  0.56971E+00_JPRB, 0.54573E+00_JPRB, 0.52264E+00_JPRB, 0.52041E+00_JPRB, 0.58092E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.53969E+00_JPRB, 0.52794E+00_JPRB, 0.51570E+00_JPRB, 0.50502E+00_JPRB, 0.49139E+00_JPRB, 0.47875E+00_JPRB, 0.46665E+00_JPRB ,&
&  0.45769E+00_JPRB, 0.45291E+00_JPRB, 0.45287E+00_JPRB, 0.45341E+00_JPRB, 0.45116E+00_JPRB, 0.44392E+00_JPRB, 0.43096E+00_JPRB ,&
&  0.42220E+00_JPRB, 0.41512E+00_JPRB, 0.40680E+00_JPRB, 0.39662E+00_JPRB, 0.38695E+00_JPRB, 0.38309E+00_JPRB, 0.38367E+00_JPRB ,&
&  0.38829E+00_JPRB, 0.39432E+00_JPRB, 0.39873E+00_JPRB, 0.41054E+00_JPRB, 0.42717E+00_JPRB, 0.45025E+00_JPRB, 0.48265E+00_JPRB ,&
&  0.52845E+00_JPRB, 0.63884E+00_JPRB, 0.68851E+00_JPRB, 0.69798E+00_JPRB, 0.75736E+00_JPRB, 0.77716E+00_JPRB, 0.82960E+00_JPRB ,&
&  0.88088E+00_JPRB, 0.91347E+00_JPRB, 0.88474E+00_JPRB, 0.79831E+00_JPRB, 0.72741E+00_JPRB, 0.67824E+00_JPRB, 0.63051E+00_JPRB ,&
&  0.58478E+00_JPRB, 0.54982E+00_JPRB, 0.52259E+00_JPRB, 0.50384E+00_JPRB, 0.48968E+00_JPRB, 0.47939E+00_JPRB, 0.47434E+00_JPRB ,&
&  0.47477E+00_JPRB, 0.47866E+00_JPRB, 0.49397E+00_JPRB, 0.52987E+00_JPRB, 0.65233E+00_JPRB, 0.78271E+00_JPRB, 0.86274E+00_JPRB ,&
&  0.88308E+00_JPRB, 0.89957E+00_JPRB, 0.90390E+00_JPRB, 0.85755E+00_JPRB, 0.79759E+00_JPRB, 0.72997E+00_JPRB, 0.67947E+00_JPRB ,&
&  0.64723E+00_JPRB, 0.61529E+00_JPRB, 0.58513E+00_JPRB, 0.55722E+00_JPRB, 0.53859E+00_JPRB, 0.53010E+00_JPRB, 0.52686E+00_JPRB ,&
&  0.52326E+00_JPRB, 0.52339E+00_JPRB, 0.52516E+00_JPRB, 0.52586E+00_JPRB, 0.52469E+00_JPRB, 0.52107E+00_JPRB, 0.51421E+00_JPRB ,&
&  0.50487E+00_JPRB, 0.49644E+00_JPRB, 0.50778E+00_JPRB, 0.51136E+00_JPRB, 0.51249E+00_JPRB, 0.51002E+00_JPRB, 0.50495E+00_JPRB ,&
&  0.50985E+00_JPRB, 0.56343E+00_JPRB, 0.57546E+00_JPRB, 0.62709E+00_JPRB, 0.68279E+00_JPRB, 0.68084E+00_JPRB, 0.62903E+00_JPRB ,&
&  0.59033E+00_JPRB, 0.55566E+00_JPRB, 0.52940E+00_JPRB, 0.52359E+00_JPRB, 0.54641E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.52794E+00_JPRB, 0.52075E+00_JPRB, 0.51090E+00_JPRB, 0.50106E+00_JPRB, 0.48959E+00_JPRB, 0.47746E+00_JPRB, 0.46871E+00_JPRB ,&
&  0.46353E+00_JPRB, 0.46247E+00_JPRB, 0.46300E+00_JPRB, 0.45350E+00_JPRB, 0.44047E+00_JPRB, 0.42840E+00_JPRB, 0.42099E+00_JPRB ,&
&  0.41525E+00_JPRB, 0.40878E+00_JPRB, 0.40064E+00_JPRB, 0.39076E+00_JPRB, 0.38612E+00_JPRB, 0.38436E+00_JPRB, 0.38366E+00_JPRB ,&
&  0.38899E+00_JPRB, 0.39401E+00_JPRB, 0.40069E+00_JPRB, 0.40975E+00_JPRB, 0.42477E+00_JPRB, 0.44766E+00_JPRB, 0.48322E+00_JPRB ,&
&  0.54604E+00_JPRB, 0.65418E+00_JPRB, 0.82501E+00_JPRB, 0.79225E+00_JPRB, 0.83246E+00_JPRB, 0.85876E+00_JPRB, 0.97999E+00_JPRB ,&
&  0.10682E+01_JPRB, 0.10825E+01_JPRB, 0.99993E+00_JPRB, 0.88773E+00_JPRB, 0.78803E+00_JPRB, 0.71230E+00_JPRB, 0.65870E+00_JPRB ,&
&  0.60911E+00_JPRB, 0.56552E+00_JPRB, 0.53500E+00_JPRB, 0.51398E+00_JPRB, 0.50012E+00_JPRB, 0.49170E+00_JPRB, 0.48885E+00_JPRB ,&
&  0.49418E+00_JPRB, 0.51407E+00_JPRB, 0.56229E+00_JPRB, 0.66303E+00_JPRB, 0.85604E+00_JPRB, 0.86418E+00_JPRB, 0.93646E+00_JPRB ,&
&  0.10905E+01_JPRB, 0.10802E+01_JPRB, 0.10221E+01_JPRB, 0.92554E+00_JPRB, 0.84272E+00_JPRB, 0.77239E+00_JPRB, 0.70976E+00_JPRB ,&
&  0.66461E+00_JPRB, 0.63000E+00_JPRB, 0.59989E+00_JPRB, 0.57866E+00_JPRB, 0.56623E+00_JPRB, 0.56291E+00_JPRB, 0.56128E+00_JPRB ,&
&  0.55742E+00_JPRB, 0.55243E+00_JPRB, 0.55004E+00_JPRB, 0.55095E+00_JPRB, 0.55195E+00_JPRB, 0.55323E+00_JPRB, 0.54478E+00_JPRB ,&
&  0.53669E+00_JPRB, 0.53559E+00_JPRB, 0.56707E+00_JPRB, 0.54861E+00_JPRB, 0.54313E+00_JPRB, 0.53830E+00_JPRB, 0.52861E+00_JPRB ,&
&  0.52183E+00_JPRB, 0.54257E+00_JPRB, 0.56304E+00_JPRB, 0.60543E+00_JPRB, 0.69726E+00_JPRB, 0.72041E+00_JPRB, 0.62920E+00_JPRB ,&
&  0.59749E+00_JPRB, 0.56629E+00_JPRB, 0.54100E+00_JPRB, 0.53136E+00_JPRB, 0.53784E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.51732E+00_JPRB, 0.51009E+00_JPRB, 0.50217E+00_JPRB, 0.49376E+00_JPRB, 0.48358E+00_JPRB, 0.47405E+00_JPRB, 0.46830E+00_JPRB ,&
&  0.46686E+00_JPRB, 0.46879E+00_JPRB, 0.46499E+00_JPRB, 0.45261E+00_JPRB, 0.43779E+00_JPRB, 0.42409E+00_JPRB, 0.41669E+00_JPRB ,&
&  0.41027E+00_JPRB, 0.40142E+00_JPRB, 0.39007E+00_JPRB, 0.37883E+00_JPRB, 0.37531E+00_JPRB, 0.37722E+00_JPRB, 0.37325E+00_JPRB ,&
&  0.37908E+00_JPRB, 0.38650E+00_JPRB, 0.39389E+00_JPRB, 0.40094E+00_JPRB, 0.41652E+00_JPRB, 0.44086E+00_JPRB, 0.47655E+00_JPRB ,&
&  0.54389E+00_JPRB, 0.47205E+00_JPRB, 0.76588E+00_JPRB, 0.90610E+00_JPRB, 0.99124E+00_JPRB, 0.10513E+01_JPRB, 0.11768E+01_JPRB ,&
&  0.12853E+01_JPRB, 0.12324E+01_JPRB, 0.11243E+01_JPRB, 0.96829E+00_JPRB, 0.82632E+00_JPRB, 0.72593E+00_JPRB, 0.66151E+00_JPRB ,&
&  0.62117E+00_JPRB, 0.59064E+00_JPRB, 0.56331E+00_JPRB, 0.53994E+00_JPRB, 0.52397E+00_JPRB, 0.51446E+00_JPRB, 0.51582E+00_JPRB ,&
&  0.54691E+00_JPRB, 0.61500E+00_JPRB, 0.71618E+00_JPRB, 0.91053E+00_JPRB, 0.10092E+01_JPRB, 0.99687E+00_JPRB, 0.99008E+00_JPRB ,&
&  0.11016E+01_JPRB, 0.12237E+01_JPRB, 0.10816E+01_JPRB, 0.93050E+00_JPRB, 0.85466E+00_JPRB, 0.78303E+00_JPRB, 0.71843E+00_JPRB ,&
&  0.66512E+00_JPRB, 0.63089E+00_JPRB, 0.60926E+00_JPRB, 0.59735E+00_JPRB, 0.59211E+00_JPRB, 0.58409E+00_JPRB, 0.57062E+00_JPRB ,&
&  0.55987E+00_JPRB, 0.55400E+00_JPRB, 0.55342E+00_JPRB, 0.55640E+00_JPRB, 0.55756E+00_JPRB, 0.56045E+00_JPRB, 0.56083E+00_JPRB ,&
&  0.55970E+00_JPRB, 0.56378E+00_JPRB, 0.56523E+00_JPRB, 0.54802E+00_JPRB, 0.54069E+00_JPRB, 0.53143E+00_JPRB, 0.52565E+00_JPRB ,&
&  0.54050E+00_JPRB, 0.55104E+00_JPRB, 0.56991E+00_JPRB, 0.62066E+00_JPRB, 0.65523E+00_JPRB, 0.68796E+00_JPRB, 0.61013E+00_JPRB ,&
&  0.59388E+00_JPRB, 0.57269E+00_JPRB, 0.55627E+00_JPRB, 0.53824E+00_JPRB, 0.53080E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.51303E+00_JPRB, 0.50487E+00_JPRB, 0.49730E+00_JPRB, 0.48772E+00_JPRB, 0.47583E+00_JPRB, 0.46870E+00_JPRB, 0.47047E+00_JPRB ,&
&  0.47538E+00_JPRB, 0.47084E+00_JPRB, 0.46357E+00_JPRB, 0.45190E+00_JPRB, 0.43448E+00_JPRB, 0.42560E+00_JPRB, 0.41960E+00_JPRB ,&
&  0.41009E+00_JPRB, 0.40289E+00_JPRB, 0.39144E+00_JPRB, 0.38558E+00_JPRB, 0.38365E+00_JPRB, 0.37937E+00_JPRB, 0.37130E+00_JPRB ,&
&  0.37406E+00_JPRB, 0.38155E+00_JPRB, 0.39074E+00_JPRB, 0.39984E+00_JPRB, 0.41741E+00_JPRB, 0.44378E+00_JPRB, 0.48149E+00_JPRB ,&
&  0.55670E+00_JPRB, 0.81056E+00_JPRB, 0.70298E+00_JPRB, 0.11795E+01_JPRB, 0.12006E+01_JPRB, 0.12648E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.12790E+01_JPRB, 0.12970E+01_JPRB, 0.12300E+01_JPRB, 0.10392E+01_JPRB, 0.85492E+00_JPRB, 0.74105E+00_JPRB, 0.66893E+00_JPRB ,&
&  0.62051E+00_JPRB, 0.58356E+00_JPRB, 0.55670E+00_JPRB, 0.53829E+00_JPRB, 0.52396E+00_JPRB, 0.52492E+00_JPRB, 0.57250E+00_JPRB ,&
&  0.68040E+00_JPRB, 0.78462E+00_JPRB, 0.96787E+00_JPRB, 0.11549E+01_JPRB, 0.10766E+01_JPRB, 0.10220E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.12095E+01_JPRB, 0.98794E+00_JPRB, 0.87116E+00_JPRB, 0.81285E+00_JPRB, 0.77431E+00_JPRB, 0.71931E+00_JPRB ,&
&  0.66974E+00_JPRB, 0.64423E+00_JPRB, 0.62972E+00_JPRB, 0.62041E+00_JPRB, 0.60802E+00_JPRB, 0.58993E+00_JPRB, 0.57100E+00_JPRB ,&
&  0.55489E+00_JPRB, 0.55471E+00_JPRB, 0.55742E+00_JPRB, 0.56029E+00_JPRB, 0.56361E+00_JPRB, 0.56483E+00_JPRB, 0.56814E+00_JPRB ,&
&  0.57512E+00_JPRB, 0.58404E+00_JPRB, 0.56131E+00_JPRB, 0.53921E+00_JPRB, 0.52915E+00_JPRB, 0.52326E+00_JPRB, 0.52080E+00_JPRB ,&
&  0.54130E+00_JPRB, 0.56486E+00_JPRB, 0.58989E+00_JPRB, 0.63068E+00_JPRB, 0.68158E+00_JPRB, 0.73108E+00_JPRB, 0.61585E+00_JPRB ,&
&  0.60630E+00_JPRB, 0.58416E+00_JPRB, 0.56459E+00_JPRB, 0.53916E+00_JPRB, 0.52621E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.50904E+00_JPRB, 0.49835E+00_JPRB, 0.49135E+00_JPRB, 0.48403E+00_JPRB, 0.47692E+00_JPRB, 0.47483E+00_JPRB, 0.47783E+00_JPRB ,&
&  0.47719E+00_JPRB, 0.46963E+00_JPRB, 0.45772E+00_JPRB, 0.44564E+00_JPRB, 0.42784E+00_JPRB, 0.41784E+00_JPRB, 0.41241E+00_JPRB ,&
&  0.40841E+00_JPRB, 0.40833E+00_JPRB, 0.40391E+00_JPRB, 0.40074E+00_JPRB, 0.39791E+00_JPRB, 0.39137E+00_JPRB, 0.38904E+00_JPRB ,&
&  0.38697E+00_JPRB, 0.39052E+00_JPRB, 0.39885E+00_JPRB, 0.41436E+00_JPRB, 0.43709E+00_JPRB, 0.47073E+00_JPRB, 0.51041E+00_JPRB ,&
&  0.61245E+00_JPRB, 0.75325E+00_JPRB, 0.76023E+00_JPRB, 0.14907E+01_JPRB, 0.14714E+01_JPRB, 0.14142E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.12508E+01_JPRB, 0.13515E+01_JPRB, 0.12619E+01_JPRB, 0.10919E+01_JPRB, 0.90951E+00_JPRB, 0.77479E+00_JPRB, 0.66959E+00_JPRB ,&
&  0.60351E+00_JPRB, 0.56270E+00_JPRB, 0.54324E+00_JPRB, 0.53536E+00_JPRB, 0.53792E+00_JPRB, 0.59479E+00_JPRB, 0.72428E+00_JPRB ,&
&  0.84723E+00_JPRB, 0.98265E+00_JPRB, 0.12644E+01_JPRB, 0.12058E+01_JPRB, 0.10559E+01_JPRB, 0.10061E+01_JPRB, 0.10224E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.10229E+01_JPRB, 0.87616E+00_JPRB, 0.79981E+00_JPRB, 0.74729E+00_JPRB, 0.74025E+00_JPRB, 0.70868E+00_JPRB ,&
&  0.67391E+00_JPRB, 0.65377E+00_JPRB, 0.63672E+00_JPRB, 0.62194E+00_JPRB, 0.60363E+00_JPRB, 0.58674E+00_JPRB, 0.56499E+00_JPRB ,&
&  0.55977E+00_JPRB, 0.55945E+00_JPRB, 0.55897E+00_JPRB, 0.56447E+00_JPRB, 0.56164E+00_JPRB, 0.56450E+00_JPRB, 0.57002E+00_JPRB ,&
&  0.58399E+00_JPRB, 0.58982E+00_JPRB, 0.55697E+00_JPRB, 0.53600E+00_JPRB, 0.53186E+00_JPRB, 0.52871E+00_JPRB, 0.53050E+00_JPRB ,&
&  0.55623E+00_JPRB, 0.58875E+00_JPRB, 0.61149E+00_JPRB, 0.62360E+00_JPRB, 0.67093E+00_JPRB, 0.68442E+00_JPRB, 0.62444E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.58458E+00_JPRB, 0.56639E+00_JPRB, 0.53727E+00_JPRB, 0.52324E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.49997E+00_JPRB, 0.48821E+00_JPRB, 0.47879E+00_JPRB, 0.47638E+00_JPRB, 0.47241E+00_JPRB, 0.46968E+00_JPRB, 0.47571E+00_JPRB ,&
&  0.46941E+00_JPRB, 0.45716E+00_JPRB, 0.43903E+00_JPRB, 0.41999E+00_JPRB, 0.40511E+00_JPRB, 0.40693E+00_JPRB, 0.41208E+00_JPRB ,&
&  0.41259E+00_JPRB, 0.41033E+00_JPRB, 0.40901E+00_JPRB, 0.41082E+00_JPRB, 0.40938E+00_JPRB, 0.40791E+00_JPRB, 0.40631E+00_JPRB ,&
&  0.40652E+00_JPRB, 0.41204E+00_JPRB, 0.42924E+00_JPRB, 0.45804E+00_JPRB, 0.49212E+00_JPRB, 0.54353E+00_JPRB, 0.61358E+00_JPRB ,&
&  0.77091E+00_JPRB, 0.84445E+00_JPRB, 0.91075E+00_JPRB, 0.13991E+01_JPRB, 0.14788E+01_JPRB, 0.13592E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.11822E+01_JPRB, 0.11140E+01_JPRB, 0.12393E+01_JPRB, 0.10652E+01_JPRB, 0.90262E+00_JPRB, 0.75635E+00_JPRB, 0.65431E+00_JPRB ,&
&  0.59596E+00_JPRB, 0.55759E+00_JPRB, 0.54880E+00_JPRB, 0.56403E+00_JPRB, 0.62687E+00_JPRB, 0.76163E+00_JPRB, 0.91888E+00_JPRB ,&
&  0.10316E+01_JPRB, 0.12503E+01_JPRB, 0.15669E+01_JPRB, 0.12834E+01_JPRB, 0.98330E+00_JPRB, 0.91743E+00_JPRB, 0.88708E+00_JPRB ,&
&  0.94078E+00_JPRB, 0.87370E+00_JPRB, 0.80464E+00_JPRB, 0.75613E+00_JPRB, 0.70099E+00_JPRB, 0.73020E+00_JPRB, 0.69265E+00_JPRB ,&
&  0.66905E+00_JPRB, 0.64893E+00_JPRB, 0.62811E+00_JPRB, 0.61003E+00_JPRB, 0.58612E+00_JPRB, 0.55647E+00_JPRB, 0.54036E+00_JPRB ,&
&  0.54122E+00_JPRB, 0.54752E+00_JPRB, 0.54677E+00_JPRB, 0.54965E+00_JPRB, 0.54774E+00_JPRB, 0.55472E+00_JPRB, 0.56539E+00_JPRB ,&
&  0.58453E+00_JPRB, 0.59513E+00_JPRB, 0.57524E+00_JPRB, 0.56779E+00_JPRB, 0.56620E+00_JPRB, 0.56686E+00_JPRB, 0.56530E+00_JPRB ,&
&  0.58177E+00_JPRB, 0.61783E+00_JPRB, 0.63835E+00_JPRB, 0.65633E+00_JPRB, 0.67068E+00_JPRB, 0.65881E+00_JPRB, 0.62333E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.63139E+00_JPRB, 0.62066E+00_JPRB, 0.55349E+00_JPRB, 0.52229E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.47275E+00_JPRB, 0.46465E+00_JPRB, 0.46195E+00_JPRB, 0.46251E+00_JPRB, 0.46079E+00_JPRB, 0.46299E+00_JPRB, 0.46599E+00_JPRB ,&
&  0.44553E+00_JPRB, 0.42460E+00_JPRB, 0.40975E+00_JPRB, 0.40359E+00_JPRB, 0.40233E+00_JPRB, 0.41428E+00_JPRB, 0.41883E+00_JPRB ,&
&  0.41578E+00_JPRB, 0.41472E+00_JPRB, 0.42719E+00_JPRB, 0.42988E+00_JPRB, 0.43056E+00_JPRB, 0.42858E+00_JPRB, 0.42976E+00_JPRB ,&
&  0.44058E+00_JPRB, 0.46189E+00_JPRB, 0.50285E+00_JPRB, 0.56120E+00_JPRB, 0.62270E+00_JPRB, 0.72097E+00_JPRB, 0.83299E+00_JPRB ,&
&  0.10261E+01_JPRB, 0.10310E+01_JPRB, 0.55794E+00_JPRB, 0.90480E+00_JPRB, 0.11779E+01_JPRB, 0.11933E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10221E+01_JPRB, 0.95505E+00_JPRB, 0.10203E+01_JPRB, 0.98889E+00_JPRB, 0.82856E+00_JPRB, 0.72356E+00_JPRB, 0.64447E+00_JPRB ,&
&  0.59878E+00_JPRB, 0.58562E+00_JPRB, 0.60371E+00_JPRB, 0.66951E+00_JPRB, 0.81202E+00_JPRB, 0.99987E+00_JPRB, 0.11387E+01_JPRB ,&
&  0.12917E+01_JPRB, 0.16258E+01_JPRB, 0.18331E+01_JPRB, 0.12737E+01_JPRB, 0.93386E+00_JPRB, 0.84190E+00_JPRB, 0.78119E+00_JPRB ,&
&  0.75498E+00_JPRB, 0.78525E+00_JPRB, 0.77054E+00_JPRB, 0.73448E+00_JPRB, 0.69128E+00_JPRB, 0.66439E+00_JPRB, 0.67537E+00_JPRB ,&
&  0.65221E+00_JPRB, 0.63061E+00_JPRB, 0.61058E+00_JPRB, 0.58204E+00_JPRB, 0.54212E+00_JPRB, 0.50806E+00_JPRB, 0.51059E+00_JPRB ,&
&  0.51349E+00_JPRB, 0.52441E+00_JPRB, 0.52445E+00_JPRB, 0.53521E+00_JPRB, 0.54502E+00_JPRB, 0.55774E+00_JPRB, 0.57126E+00_JPRB ,&
&  0.58995E+00_JPRB, 0.60859E+00_JPRB, 0.62916E+00_JPRB, 0.62419E+00_JPRB, 0.61931E+00_JPRB, 0.62288E+00_JPRB, 0.63240E+00_JPRB ,&
&  0.65334E+00_JPRB, 0.67365E+00_JPRB, 0.64689E+00_JPRB, 0.65599E+00_JPRB, 0.68073E+00_JPRB, 0.66948E+00_JPRB, 0.65290E+00_JPRB ,&
&  0.69913E+00_JPRB, 0.73884E+00_JPRB, 0.67275E+00_JPRB, 0.52304E+00_JPRB, 0.49123E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.45410E+00_JPRB, 0.45581E+00_JPRB, 0.45395E+00_JPRB, 0.45655E+00_JPRB, 0.45248E+00_JPRB, 0.45519E+00_JPRB, 0.44267E+00_JPRB ,&
&  0.41971E+00_JPRB, 0.41164E+00_JPRB, 0.42582E+00_JPRB, 0.43120E+00_JPRB, 0.43409E+00_JPRB, 0.44953E+00_JPRB, 0.46638E+00_JPRB ,&
&  0.47205E+00_JPRB, 0.47191E+00_JPRB, 0.46943E+00_JPRB, 0.46919E+00_JPRB, 0.46523E+00_JPRB, 0.46859E+00_JPRB, 0.48708E+00_JPRB ,&
&  0.51618E+00_JPRB, 0.56860E+00_JPRB, 0.64672E+00_JPRB, 0.74371E+00_JPRB, 0.84890E+00_JPRB, 0.98733E+00_JPRB, 0.11058E+01_JPRB ,&
&  0.13378E+01_JPRB, 0.95478E+00_JPRB, 0.62210E+00_JPRB, 0.10265E+01_JPRB, 0.11411E+01_JPRB, 0.11253E+01_JPRB, 0.97437E+00_JPRB ,&
&  0.91935E+00_JPRB, 0.88895E+00_JPRB, 0.87654E+00_JPRB, 0.83544E+00_JPRB, 0.74239E+00_JPRB, 0.69910E+00_JPRB, 0.65993E+00_JPRB ,&
&  0.64220E+00_JPRB, 0.66235E+00_JPRB, 0.73121E+00_JPRB, 0.88178E+00_JPRB, 0.11039E+01_JPRB, 0.12887E+01_JPRB, 0.14272E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.20685E+01_JPRB, 0.21663E+01_JPRB, 0.14614E+01_JPRB, 0.99557E+00_JPRB, 0.84502E+00_JPRB, 0.78169E+00_JPRB ,&
&  0.73666E+00_JPRB, 0.75314E+00_JPRB, 0.76121E+00_JPRB, 0.72742E+00_JPRB, 0.68764E+00_JPRB, 0.64652E+00_JPRB, 0.65468E+00_JPRB ,&
&  0.62904E+00_JPRB, 0.60209E+00_JPRB, 0.57268E+00_JPRB, 0.53442E+00_JPRB, 0.49682E+00_JPRB, 0.49412E+00_JPRB, 0.48807E+00_JPRB ,&
&  0.49218E+00_JPRB, 0.50647E+00_JPRB, 0.50750E+00_JPRB, 0.52354E+00_JPRB, 0.54257E+00_JPRB, 0.56780E+00_JPRB, 0.59333E+00_JPRB ,&
&  0.60982E+00_JPRB, 0.63497E+00_JPRB, 0.66124E+00_JPRB, 0.66693E+00_JPRB, 0.65861E+00_JPRB, 0.64418E+00_JPRB, 0.66332E+00_JPRB ,&
&  0.67086E+00_JPRB, 0.68149E+00_JPRB, 0.66282E+00_JPRB, 0.67514E+00_JPRB, 0.64416E+00_JPRB, 0.62111E+00_JPRB, 0.57939E+00_JPRB ,&
&  0.57487E+00_JPRB, 0.54000E+00_JPRB, 0.50197E+00_JPRB, 0.45431E+00_JPRB, 0.45035E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.45716E+00_JPRB, 0.45313E+00_JPRB, 0.44578E+00_JPRB, 0.43682E+00_JPRB, 0.42698E+00_JPRB, 0.42695E+00_JPRB, 0.41635E+00_JPRB ,&
&  0.41369E+00_JPRB, 0.43364E+00_JPRB, 0.46323E+00_JPRB, 0.48762E+00_JPRB, 0.50869E+00_JPRB, 0.51808E+00_JPRB, 0.53464E+00_JPRB ,&
&  0.53576E+00_JPRB, 0.53418E+00_JPRB, 0.53283E+00_JPRB, 0.53294E+00_JPRB, 0.54073E+00_JPRB, 0.56662E+00_JPRB, 0.61261E+00_JPRB ,&
&  0.67785E+00_JPRB, 0.76411E+00_JPRB, 0.86847E+00_JPRB, 0.97854E+00_JPRB, 0.10810E+01_JPRB, 0.11779E+01_JPRB, 0.12172E+01_JPRB ,&
&  0.85221E+00_JPRB, 0.50747E+00_JPRB, 0.10206E+01_JPRB, 0.11319E+01_JPRB, 0.10523E+01_JPRB, 0.10059E+01_JPRB, 0.92323E+00_JPRB ,&
&  0.84423E+00_JPRB, 0.82620E+00_JPRB, 0.79423E+00_JPRB, 0.77569E+00_JPRB, 0.71252E+00_JPRB, 0.70018E+00_JPRB, 0.70122E+00_JPRB ,&
&  0.72390E+00_JPRB, 0.78877E+00_JPRB, 0.93466E+00_JPRB, 0.11859E+01_JPRB, 0.14009E+01_JPRB, 0.15347E+01_JPRB, 0.17615E+01_JPRB ,&
&  0.21600E+01_JPRB, 0.25305E+01_JPRB, 0.27005E+01_JPRB, 0.18475E+01_JPRB, 0.12641E+01_JPRB, 0.11190E+01_JPRB, 0.10057E+01_JPRB ,&
&  0.91418E+00_JPRB, 0.73059E+00_JPRB, 0.72738E+00_JPRB, 0.72754E+00_JPRB, 0.68328E+00_JPRB, 0.62275E+00_JPRB, 0.62143E+00_JPRB ,&
&  0.58848E+00_JPRB, 0.54488E+00_JPRB, 0.50374E+00_JPRB, 0.48428E+00_JPRB, 0.47623E+00_JPRB, 0.48557E+00_JPRB, 0.48378E+00_JPRB ,&
&  0.48537E+00_JPRB, 0.49908E+00_JPRB, 0.50428E+00_JPRB, 0.51690E+00_JPRB, 0.55300E+00_JPRB, 0.59119E+00_JPRB, 0.61663E+00_JPRB ,&
&  0.62822E+00_JPRB, 0.65563E+00_JPRB, 0.67688E+00_JPRB, 0.68593E+00_JPRB, 0.68728E+00_JPRB, 0.66646E+00_JPRB, 0.66310E+00_JPRB ,&
&  0.66433E+00_JPRB, 0.65989E+00_JPRB, 0.64186E+00_JPRB, 0.63601E+00_JPRB, 0.60132E+00_JPRB, 0.58123E+00_JPRB, 0.52329E+00_JPRB ,&
&  0.49894E+00_JPRB, 0.46001E+00_JPRB, 0.43190E+00_JPRB, 0.42363E+00_JPRB, 0.43196E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.45427E+00_JPRB, 0.43760E+00_JPRB, 0.42748E+00_JPRB, 0.41943E+00_JPRB, 0.40826E+00_JPRB, 0.42985E+00_JPRB, 0.43472E+00_JPRB ,&
&  0.45657E+00_JPRB, 0.49305E+00_JPRB, 0.53596E+00_JPRB, 0.55857E+00_JPRB, 0.56302E+00_JPRB, 0.58446E+00_JPRB, 0.60288E+00_JPRB ,&
&  0.61430E+00_JPRB, 0.62310E+00_JPRB, 0.63635E+00_JPRB, 0.65388E+00_JPRB, 0.68532E+00_JPRB, 0.73323E+00_JPRB, 0.80397E+00_JPRB ,&
&  0.88499E+00_JPRB, 0.97142E+00_JPRB, 0.10629E+01_JPRB, 0.11460E+01_JPRB, 0.11983E+01_JPRB, 0.12418E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.74741E+00_JPRB, 0.97831E+00_JPRB, 0.11156E+01_JPRB, 0.10486E+01_JPRB, 0.98808E+00_JPRB, 0.93811E+00_JPRB, 0.89357E+00_JPRB ,&
&  0.85025E+00_JPRB, 0.79990E+00_JPRB, 0.78029E+00_JPRB, 0.79776E+00_JPRB, 0.74071E+00_JPRB, 0.74110E+00_JPRB, 0.77127E+00_JPRB ,&
&  0.83129E+00_JPRB, 0.95261E+00_JPRB, 0.11786E+01_JPRB, 0.14458E+01_JPRB, 0.16332E+01_JPRB, 0.18279E+01_JPRB, 0.21450E+01_JPRB ,&
&  0.26113E+01_JPRB, 0.29908E+01_JPRB, 0.32736E+01_JPRB, 0.24870E+01_JPRB, 0.21466E+01_JPRB, 0.20997E+01_JPRB, 0.18833E+01_JPRB ,&
&  0.11174E+01_JPRB, 0.75064E+00_JPRB, 0.75202E+00_JPRB, 0.73571E+00_JPRB, 0.65070E+00_JPRB, 0.60770E+00_JPRB, 0.56140E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.46837E+00_JPRB, 0.44089E+00_JPRB, 0.46675E+00_JPRB, 0.49223E+00_JPRB, 0.50622E+00_JPRB, 0.49196E+00_JPRB ,&
&  0.51447E+00_JPRB, 0.50335E+00_JPRB, 0.50251E+00_JPRB, 0.51694E+00_JPRB, 0.57347E+00_JPRB, 0.59927E+00_JPRB, 0.60998E+00_JPRB ,&
&  0.61784E+00_JPRB, 0.63517E+00_JPRB, 0.64925E+00_JPRB, 0.65856E+00_JPRB, 0.66193E+00_JPRB, 0.65902E+00_JPRB, 0.64923E+00_JPRB ,&
&  0.63467E+00_JPRB, 0.63278E+00_JPRB, 0.61922E+00_JPRB, 0.60559E+00_JPRB, 0.58257E+00_JPRB, 0.59984E+00_JPRB, 0.53386E+00_JPRB ,&
&  0.48814E+00_JPRB, 0.45452E+00_JPRB, 0.43077E+00_JPRB, 0.42504E+00_JPRB, 0.43258E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.45487E+00_JPRB, 0.44352E+00_JPRB, 0.42636E+00_JPRB, 0.43456E+00_JPRB, 0.43638E+00_JPRB, 0.45281E+00_JPRB, 0.47166E+00_JPRB ,&
&  0.49670E+00_JPRB, 0.54342E+00_JPRB, 0.58704E+00_JPRB, 0.62236E+00_JPRB, 0.64213E+00_JPRB, 0.66868E+00_JPRB, 0.69434E+00_JPRB ,&
&  0.71086E+00_JPRB, 0.72871E+00_JPRB, 0.75326E+00_JPRB, 0.79045E+00_JPRB, 0.83878E+00_JPRB, 0.89326E+00_JPRB, 0.96004E+00_JPRB ,&
&  0.10331E+01_JPRB, 0.11130E+01_JPRB, 0.12140E+01_JPRB, 0.12468E+01_JPRB, 0.12741E+01_JPRB, 0.13086E+01_JPRB, 0.12863E+01_JPRB ,&
&  0.91071E+00_JPRB, 0.10683E+01_JPRB, 0.10399E+01_JPRB, 0.99822E+00_JPRB, 0.96642E+00_JPRB, 0.91802E+00_JPRB, 0.87667E+00_JPRB ,&
&  0.84769E+00_JPRB, 0.79384E+00_JPRB, 0.77043E+00_JPRB, 0.74596E+00_JPRB, 0.78679E+00_JPRB, 0.80305E+00_JPRB, 0.86907E+00_JPRB ,&
&  0.97825E+00_JPRB, 0.11549E+01_JPRB, 0.13858E+01_JPRB, 0.16294E+01_JPRB, 0.18726E+01_JPRB, 0.21350E+01_JPRB, 0.24902E+01_JPRB ,&
&  0.28719E+01_JPRB, 0.31371E+01_JPRB, 0.32915E+01_JPRB, 0.28448E+01_JPRB, 0.22798E+01_JPRB, 0.19738E+01_JPRB, 0.14435E+01_JPRB ,&
&  0.94453E+00_JPRB, 0.76047E+00_JPRB, 0.77215E+00_JPRB, 0.74226E+00_JPRB, 0.64694E+00_JPRB, 0.58304E+00_JPRB, 0.52613E+00_JPRB ,&
&  0.48269E+00_JPRB, 0.45260E+00_JPRB, 0.47087E+00_JPRB, 0.53181E+00_JPRB, 0.56279E+00_JPRB, 0.58249E+00_JPRB, 0.57663E+00_JPRB ,&
&  0.56465E+00_JPRB, 0.53353E+00_JPRB, 0.51044E+00_JPRB, 0.52142E+00_JPRB, 0.55712E+00_JPRB, 0.59232E+00_JPRB, 0.59286E+00_JPRB ,&
&  0.57860E+00_JPRB, 0.57673E+00_JPRB, 0.59641E+00_JPRB, 0.62088E+00_JPRB, 0.63881E+00_JPRB, 0.66227E+00_JPRB, 0.67984E+00_JPRB ,&
&  0.70451E+00_JPRB, 0.74496E+00_JPRB, 0.73270E+00_JPRB, 0.72540E+00_JPRB, 0.81594E+00_JPRB, 0.86790E+00_JPRB, 0.55695E+00_JPRB ,&
&  0.47374E+00_JPRB, 0.44051E+00_JPRB, 0.43820E+00_JPRB, 0.43195E+00_JPRB, 0.46268E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.48959E+00_JPRB, 0.50973E+00_JPRB, 0.50046E+00_JPRB, 0.50359E+00_JPRB, 0.51051E+00_JPRB, 0.52135E+00_JPRB, 0.54415E+00_JPRB ,&
&  0.54975E+00_JPRB, 0.57929E+00_JPRB, 0.60980E+00_JPRB, 0.63466E+00_JPRB, 0.66970E+00_JPRB, 0.71853E+00_JPRB, 0.77889E+00_JPRB ,&
&  0.84201E+00_JPRB, 0.87957E+00_JPRB, 0.91050E+00_JPRB, 0.93432E+00_JPRB, 0.97761E+00_JPRB, 0.10359E+01_JPRB, 0.10684E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.11911E+01_JPRB, 0.12520E+01_JPRB, 0.13399E+01_JPRB, 0.12789E+01_JPRB, 0.12700E+01_JPRB, 0.92088E+00_JPRB ,&
&  0.10337E+01_JPRB, 0.10431E+01_JPRB, 0.10252E+01_JPRB, 0.10051E+01_JPRB, 0.96717E+00_JPRB, 0.90503E+00_JPRB, 0.86559E+00_JPRB ,&
&  0.83530E+00_JPRB, 0.80536E+00_JPRB, 0.77090E+00_JPRB, 0.75892E+00_JPRB, 0.84406E+00_JPRB, 0.87504E+00_JPRB, 0.99683E+00_JPRB ,&
&  0.11550E+01_JPRB, 0.13389E+01_JPRB, 0.15288E+01_JPRB, 0.17593E+01_JPRB, 0.20398E+01_JPRB, 0.23365E+01_JPRB, 0.26459E+01_JPRB ,&
&  0.28865E+01_JPRB, 0.30665E+01_JPRB, 0.29933E+01_JPRB, 0.24089E+01_JPRB, 0.21145E+01_JPRB, 0.15934E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.86592E+00_JPRB, 0.80218E+00_JPRB, 0.81446E+00_JPRB, 0.78089E+00_JPRB, 0.66900E+00_JPRB, 0.59020E+00_JPRB, 0.53078E+00_JPRB ,&
&  0.49425E+00_JPRB, 0.49157E+00_JPRB, 0.52826E+00_JPRB, 0.57162E+00_JPRB, 0.61872E+00_JPRB, 0.66004E+00_JPRB, 0.66300E+00_JPRB ,&
&  0.63828E+00_JPRB, 0.59025E+00_JPRB, 0.56292E+00_JPRB, 0.57330E+00_JPRB, 0.63065E+00_JPRB, 0.69140E+00_JPRB, 0.73913E+00_JPRB ,&
&  0.60847E+00_JPRB, 0.52705E+00_JPRB, 0.56462E+00_JPRB, 0.60849E+00_JPRB, 0.66279E+00_JPRB, 0.73523E+00_JPRB, 0.84621E+00_JPRB ,&
&  0.10140E+01_JPRB, 0.13699E+01_JPRB, 0.13457E+01_JPRB, 0.11180E+01_JPRB, 0.16363E+01_JPRB, 0.13565E+01_JPRB, 0.49872E+00_JPRB ,&
&  0.47884E+00_JPRB, 0.47027E+00_JPRB, 0.47421E+00_JPRB, 0.47682E+00_JPRB, 0.48722E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.58343E+00_JPRB, 0.59401E+00_JPRB, 0.61223E+00_JPRB, 0.61397E+00_JPRB, 0.62259E+00_JPRB, 0.64109E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.67947E+00_JPRB, 0.68952E+00_JPRB, 0.70252E+00_JPRB, 0.71816E+00_JPRB, 0.73744E+00_JPRB, 0.78256E+00_JPRB, 0.84948E+00_JPRB ,&
&  0.90162E+00_JPRB, 0.95090E+00_JPRB, 0.10141E+01_JPRB, 0.10356E+01_JPRB, 0.10769E+01_JPRB, 0.11345E+01_JPRB, 0.11692E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12513E+01_JPRB, 0.12771E+01_JPRB, 0.13413E+01_JPRB, 0.13005E+01_JPRB, 0.13285E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10691E+01_JPRB, 0.10564E+01_JPRB, 0.10052E+01_JPRB, 0.98319E+00_JPRB, 0.95525E+00_JPRB, 0.91210E+00_JPRB, 0.88089E+00_JPRB ,&
&  0.86783E+00_JPRB, 0.83084E+00_JPRB, 0.79205E+00_JPRB, 0.83272E+00_JPRB, 0.88478E+00_JPRB, 0.98097E+00_JPRB, 0.11351E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.14570E+01_JPRB, 0.16110E+01_JPRB, 0.18218E+01_JPRB, 0.20809E+01_JPRB, 0.23635E+01_JPRB, 0.26665E+01_JPRB ,&
&  0.28688E+01_JPRB, 0.29862E+01_JPRB, 0.29806E+01_JPRB, 0.24378E+01_JPRB, 0.21657E+01_JPRB, 0.15922E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.85698E+00_JPRB, 0.78517E+00_JPRB, 0.80338E+00_JPRB, 0.84149E+00_JPRB, 0.73761E+00_JPRB, 0.66803E+00_JPRB, 0.60038E+00_JPRB ,&
&  0.56386E+00_JPRB, 0.55716E+00_JPRB, 0.57275E+00_JPRB, 0.61871E+00_JPRB, 0.67387E+00_JPRB, 0.73069E+00_JPRB, 0.75141E+00_JPRB ,&
&  0.72614E+00_JPRB, 0.66420E+00_JPRB, 0.60727E+00_JPRB, 0.56543E+00_JPRB, 0.55784E+00_JPRB, 0.57589E+00_JPRB, 0.54441E+00_JPRB ,&
&  0.55089E+00_JPRB, 0.53393E+00_JPRB, 0.54667E+00_JPRB, 0.61219E+00_JPRB, 0.72193E+00_JPRB, 0.93234E+00_JPRB, 0.13327E+01_JPRB ,&
&  0.19263E+01_JPRB, 0.20806E+01_JPRB, 0.25714E+01_JPRB, 0.34395E+01_JPRB, 0.25357E+01_JPRB, 0.17809E+01_JPRB, 0.54954E+00_JPRB ,&
&  0.56355E+00_JPRB, 0.55919E+00_JPRB, 0.55833E+00_JPRB, 0.55760E+00_JPRB, 0.56505E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.62124E+00_JPRB, 0.63791E+00_JPRB, 0.65697E+00_JPRB, 0.67298E+00_JPRB, 0.69419E+00_JPRB, 0.72513E+00_JPRB, 0.75615E+00_JPRB ,&
&  0.78224E+00_JPRB, 0.80103E+00_JPRB, 0.81409E+00_JPRB, 0.83442E+00_JPRB, 0.86044E+00_JPRB, 0.89504E+00_JPRB, 0.94025E+00_JPRB ,&
&  0.95802E+00_JPRB, 0.99753E+00_JPRB, 0.10173E+01_JPRB, 0.10568E+01_JPRB, 0.10730E+01_JPRB, 0.11217E+01_JPRB, 0.11568E+01_JPRB ,&
&  0.11770E+01_JPRB, 0.11877E+01_JPRB, 0.11955E+01_JPRB, 0.12166E+01_JPRB, 0.11948E+01_JPRB, 0.12234E+01_JPRB, 0.93024E+00_JPRB ,&
&  0.10322E+01_JPRB, 0.10130E+01_JPRB, 0.93000E+00_JPRB, 0.89324E+00_JPRB, 0.85947E+00_JPRB, 0.83617E+00_JPRB, 0.82524E+00_JPRB ,&
&  0.81948E+00_JPRB, 0.84461E+00_JPRB, 0.83076E+00_JPRB, 0.85764E+00_JPRB, 0.94162E+00_JPRB, 0.10798E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.13579E+01_JPRB, 0.15098E+01_JPRB, 0.16545E+01_JPRB, 0.18318E+01_JPRB, 0.20510E+01_JPRB, 0.22852E+01_JPRB, 0.25100E+01_JPRB ,&
&  0.25640E+01_JPRB, 0.25587E+01_JPRB, 0.22983E+01_JPRB, 0.22304E+01_JPRB, 0.21572E+01_JPRB, 0.16886E+01_JPRB, 0.12042E+01_JPRB ,&
&  0.80411E+00_JPRB, 0.77878E+00_JPRB, 0.85586E+00_JPRB, 0.96266E+00_JPRB, 0.89470E+00_JPRB, 0.83511E+00_JPRB, 0.75951E+00_JPRB ,&
&  0.70790E+00_JPRB, 0.66840E+00_JPRB, 0.67804E+00_JPRB, 0.72357E+00_JPRB, 0.78482E+00_JPRB, 0.84486E+00_JPRB, 0.85668E+00_JPRB ,&
&  0.82379E+00_JPRB, 0.75640E+00_JPRB, 0.67041E+00_JPRB, 0.59281E+00_JPRB, 0.53739E+00_JPRB, 0.49315E+00_JPRB, 0.54447E+00_JPRB ,&
&  0.51778E+00_JPRB, 0.49636E+00_JPRB, 0.53494E+00_JPRB, 0.49866E+00_JPRB, 0.68398E+00_JPRB, 0.84073E+00_JPRB, 0.10881E+01_JPRB ,&
&  0.12196E+01_JPRB, 0.16602E+01_JPRB, 0.20860E+01_JPRB, 0.14932E+01_JPRB, 0.11071E+01_JPRB, 0.67939E+00_JPRB, 0.60674E+00_JPRB ,&
&  0.60493E+00_JPRB, 0.59000E+00_JPRB, 0.58889E+00_JPRB, 0.59777E+00_JPRB, 0.60777E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.65292E+00_JPRB, 0.67221E+00_JPRB, 0.69169E+00_JPRB, 0.70960E+00_JPRB, 0.72708E+00_JPRB, 0.74301E+00_JPRB, 0.76034E+00_JPRB ,&
&  0.77839E+00_JPRB, 0.79775E+00_JPRB, 0.81635E+00_JPRB, 0.83948E+00_JPRB, 0.86155E+00_JPRB, 0.87372E+00_JPRB, 0.89190E+00_JPRB ,&
&  0.90037E+00_JPRB, 0.93424E+00_JPRB, 0.94884E+00_JPRB, 0.98055E+00_JPRB, 0.10090E+01_JPRB, 0.10266E+01_JPRB, 0.10618E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.10926E+01_JPRB, 0.11055E+01_JPRB, 0.11325E+01_JPRB, 0.11404E+01_JPRB, 0.12034E+01_JPRB, 0.95533E+00_JPRB ,&
&  0.96690E+00_JPRB, 0.91871E+00_JPRB, 0.84673E+00_JPRB, 0.78828E+00_JPRB, 0.74021E+00_JPRB, 0.68965E+00_JPRB, 0.66509E+00_JPRB ,&
&  0.67960E+00_JPRB, 0.70695E+00_JPRB, 0.74780E+00_JPRB, 0.80523E+00_JPRB, 0.90771E+00_JPRB, 0.10562E+01_JPRB, 0.11656E+01_JPRB ,&
&  0.12948E+01_JPRB, 0.14484E+01_JPRB, 0.15878E+01_JPRB, 0.17254E+01_JPRB, 0.18854E+01_JPRB, 0.20325E+01_JPRB, 0.21117E+01_JPRB ,&
&  0.20741E+01_JPRB, 0.19625E+01_JPRB, 0.15851E+01_JPRB, 0.16949E+01_JPRB, 0.17324E+01_JPRB, 0.15193E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.94930E+00_JPRB, 0.84540E+00_JPRB, 0.90256E+00_JPRB, 0.10977E+01_JPRB, 0.11532E+01_JPRB, 0.11444E+01_JPRB, 0.10805E+01_JPRB ,&
&  0.10117E+01_JPRB, 0.92640E+00_JPRB, 0.89077E+00_JPRB, 0.90925E+00_JPRB, 0.95253E+00_JPRB, 0.96870E+00_JPRB, 0.90598E+00_JPRB ,&
&  0.86060E+00_JPRB, 0.81862E+00_JPRB, 0.75039E+00_JPRB, 0.64722E+00_JPRB, 0.59868E+00_JPRB, 0.54911E+00_JPRB, 0.61053E+00_JPRB ,&
&  0.47380E+00_JPRB, 0.42297E+00_JPRB, 0.44826E+00_JPRB, 0.54171E+00_JPRB, 0.60721E+00_JPRB, 0.68710E+00_JPRB, 0.85497E+00_JPRB ,&
&  0.99626E+00_JPRB, 0.10451E+01_JPRB, 0.97841E+00_JPRB, 0.80187E+00_JPRB, 0.66355E+00_JPRB, 0.60873E+00_JPRB, 0.60732E+00_JPRB ,&
&  0.60753E+00_JPRB, 0.60330E+00_JPRB, 0.61006E+00_JPRB, 0.62424E+00_JPRB, 0.63652E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.62927E+00_JPRB, 0.64733E+00_JPRB, 0.66576E+00_JPRB, 0.68461E+00_JPRB, 0.70092E+00_JPRB, 0.71625E+00_JPRB, 0.73229E+00_JPRB ,&
&  0.74837E+00_JPRB, 0.76738E+00_JPRB, 0.78752E+00_JPRB, 0.80937E+00_JPRB, 0.82725E+00_JPRB, 0.83573E+00_JPRB, 0.84859E+00_JPRB ,&
&  0.85903E+00_JPRB, 0.86328E+00_JPRB, 0.85742E+00_JPRB, 0.88138E+00_JPRB, 0.90627E+00_JPRB, 0.92363E+00_JPRB, 0.95042E+00_JPRB ,&
&  0.98121E+00_JPRB, 0.97147E+00_JPRB, 0.97121E+00_JPRB, 0.99865E+00_JPRB, 0.10147E+01_JPRB, 0.96385E+00_JPRB, 0.10482E+01_JPRB ,&
&  0.80716E+00_JPRB, 0.95406E+00_JPRB, 0.87340E+00_JPRB, 0.76236E+00_JPRB, 0.73416E+00_JPRB, 0.64774E+00_JPRB, 0.60366E+00_JPRB ,&
&  0.64394E+00_JPRB, 0.65635E+00_JPRB, 0.66970E+00_JPRB, 0.70836E+00_JPRB, 0.80468E+00_JPRB, 0.94803E+00_JPRB, 0.10389E+01_JPRB ,&
&  0.11654E+01_JPRB, 0.12972E+01_JPRB, 0.14470E+01_JPRB, 0.16060E+01_JPRB, 0.17918E+01_JPRB, 0.18703E+01_JPRB, 0.17647E+01_JPRB ,&
&  0.16080E+01_JPRB, 0.14670E+01_JPRB, 0.10546E+01_JPRB, 0.11273E+01_JPRB, 0.12766E+01_JPRB, 0.12311E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.96081E+00_JPRB, 0.97515E+00_JPRB, 0.11570E+01_JPRB, 0.13663E+01_JPRB, 0.14822E+01_JPRB, 0.15062E+01_JPRB, 0.14150E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.11612E+01_JPRB, 0.10767E+01_JPRB, 0.10871E+01_JPRB, 0.10708E+01_JPRB, 0.10035E+01_JPRB, 0.86444E+00_JPRB ,&
&  0.80543E+00_JPRB, 0.80956E+00_JPRB, 0.80532E+00_JPRB, 0.75604E+00_JPRB, 0.71050E+00_JPRB, 0.92617E+00_JPRB, 0.92790E+00_JPRB ,&
&  0.56490E+00_JPRB, 0.51678E+00_JPRB, 0.40538E+00_JPRB, 0.50576E+00_JPRB, 0.54345E+00_JPRB, 0.56796E+00_JPRB, 0.61726E+00_JPRB ,&
&  0.65062E+00_JPRB, 0.61279E+00_JPRB, 0.60400E+00_JPRB, 0.58136E+00_JPRB, 0.58140E+00_JPRB, 0.57684E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.59599E+00_JPRB, 0.59537E+00_JPRB, 0.60185E+00_JPRB, 0.61124E+00_JPRB, 0.61771E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.59292E+00_JPRB, 0.60037E+00_JPRB, 0.60612E+00_JPRB, 0.62285E+00_JPRB, 0.64804E+00_JPRB, 0.65328E+00_JPRB, 0.66748E+00_JPRB ,&
&  0.67594E+00_JPRB, 0.69336E+00_JPRB, 0.71164E+00_JPRB, 0.73411E+00_JPRB, 0.74773E+00_JPRB, 0.75411E+00_JPRB, 0.77455E+00_JPRB ,&
&  0.79241E+00_JPRB, 0.78887E+00_JPRB, 0.79960E+00_JPRB, 0.82037E+00_JPRB, 0.82563E+00_JPRB, 0.85604E+00_JPRB, 0.85394E+00_JPRB ,&
&  0.88427E+00_JPRB, 0.89094E+00_JPRB, 0.92244E+00_JPRB, 0.10113E+01_JPRB, 0.11139E+01_JPRB, 0.13115E+01_JPRB, 0.16717E+01_JPRB ,&
&  0.13999E+01_JPRB, 0.11374E+01_JPRB, 0.10140E+01_JPRB, 0.85684E+00_JPRB, 0.85771E+00_JPRB, 0.88031E+00_JPRB, 0.84905E+00_JPRB ,&
&  0.82157E+00_JPRB, 0.78251E+00_JPRB, 0.77055E+00_JPRB, 0.77075E+00_JPRB, 0.80765E+00_JPRB, 0.89388E+00_JPRB, 0.95024E+00_JPRB ,&
&  0.93334E+00_JPRB, 0.10386E+01_JPRB, 0.11722E+01_JPRB, 0.13462E+01_JPRB, 0.14670E+01_JPRB, 0.17766E+01_JPRB, 0.17629E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.14573E+01_JPRB, 0.12109E+01_JPRB, 0.12770E+01_JPRB, 0.14034E+01_JPRB, 0.13333E+01_JPRB, 0.11599E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.10820E+01_JPRB, 0.11852E+01_JPRB, 0.16541E+01_JPRB, 0.18437E+01_JPRB, 0.18853E+01_JPRB, 0.17782E+01_JPRB ,&
&  0.16290E+01_JPRB, 0.14400E+01_JPRB, 0.12676E+01_JPRB, 0.11643E+01_JPRB, 0.11085E+01_JPRB, 0.10029E+01_JPRB, 0.87767E+00_JPRB ,&
&  0.77769E+00_JPRB, 0.79121E+00_JPRB, 0.81747E+00_JPRB, 0.77598E+00_JPRB, 0.79454E+00_JPRB, 0.73786E+00_JPRB, 0.79410E+00_JPRB ,&
&  0.56528E+00_JPRB, 0.60681E+00_JPRB, 0.54060E+00_JPRB, 0.72230E+00_JPRB, 0.54588E+00_JPRB, 0.45178E+00_JPRB, 0.45172E+00_JPRB ,&
&  0.46455E+00_JPRB, 0.47407E+00_JPRB, 0.48829E+00_JPRB, 0.50041E+00_JPRB, 0.51506E+00_JPRB, 0.53909E+00_JPRB, 0.56479E+00_JPRB ,&
&  0.58590E+00_JPRB, 0.57394E+00_JPRB, 0.57396E+00_JPRB, 0.57838E+00_JPRB, 0.59173E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.54819E+00_JPRB, 0.54537E+00_JPRB, 0.53762E+00_JPRB, 0.53802E+00_JPRB, 0.54288E+00_JPRB, 0.55647E+00_JPRB, 0.56505E+00_JPRB ,&
&  0.56386E+00_JPRB, 0.56925E+00_JPRB, 0.55340E+00_JPRB, 0.55530E+00_JPRB, 0.57220E+00_JPRB, 0.58274E+00_JPRB, 0.59309E+00_JPRB ,&
&  0.63033E+00_JPRB, 0.64481E+00_JPRB, 0.64391E+00_JPRB, 0.65016E+00_JPRB, 0.64708E+00_JPRB, 0.69423E+00_JPRB, 0.69950E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.71656E+00_JPRB, 0.84723E+00_JPRB, 0.11949E+01_JPRB, 0.15667E+01_JPRB, 0.15633E+01_JPRB, 0.17484E+01_JPRB ,&
&  0.16672E+01_JPRB, 0.14472E+01_JPRB, 0.14437E+01_JPRB, 0.12134E+01_JPRB, 0.12099E+01_JPRB, 0.12554E+01_JPRB, 0.12588E+01_JPRB ,&
&  0.12760E+01_JPRB, 0.12597E+01_JPRB, 0.12667E+01_JPRB, 0.12836E+01_JPRB, 0.12847E+01_JPRB, 0.12863E+01_JPRB, 0.12823E+01_JPRB ,&
&  0.12087E+01_JPRB, 0.12215E+01_JPRB, 0.13140E+01_JPRB, 0.14709E+01_JPRB, 0.16504E+01_JPRB, 0.18906E+01_JPRB, 0.19856E+01_JPRB ,&
&  0.20330E+01_JPRB, 0.20395E+01_JPRB, 0.19037E+01_JPRB, 0.19628E+01_JPRB, 0.21037E+01_JPRB, 0.19422E+01_JPRB, 0.17875E+01_JPRB ,&
&  0.16448E+01_JPRB, 0.12907E+01_JPRB, 0.13033E+01_JPRB, 0.19637E+01_JPRB, 0.21419E+01_JPRB, 0.20528E+01_JPRB, 0.20593E+01_JPRB ,&
&  0.19324E+01_JPRB, 0.17652E+01_JPRB, 0.14972E+01_JPRB, 0.12871E+01_JPRB, 0.11165E+01_JPRB, 0.96287E+00_JPRB, 0.91491E+00_JPRB ,&
&  0.79452E+00_JPRB, 0.75432E+00_JPRB, 0.78833E+00_JPRB, 0.77213E+00_JPRB, 0.71915E+00_JPRB, 0.72269E+00_JPRB, 0.66185E+00_JPRB ,&
&  0.61075E+00_JPRB, 0.65751E+00_JPRB, 0.95571E+00_JPRB, 0.12989E+01_JPRB, 0.66100E+00_JPRB, 0.46409E+00_JPRB, 0.39173E+00_JPRB ,&
&  0.40809E+00_JPRB, 0.42773E+00_JPRB, 0.43490E+00_JPRB, 0.44422E+00_JPRB, 0.46164E+00_JPRB, 0.47684E+00_JPRB, 0.49991E+00_JPRB ,&
&  0.51369E+00_JPRB, 0.52456E+00_JPRB, 0.53600E+00_JPRB, 0.54982E+00_JPRB, 0.54580E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.61017E+00_JPRB, 0.64035E+00_JPRB, 0.65861E+00_JPRB, 0.67969E+00_JPRB, 0.68783E+00_JPRB, 0.68028E+00_JPRB, 0.64984E+00_JPRB ,&
&  0.62594E+00_JPRB, 0.60196E+00_JPRB, 0.59405E+00_JPRB, 0.58015E+00_JPRB, 0.60062E+00_JPRB, 0.63992E+00_JPRB, 0.67170E+00_JPRB ,&
&  0.75390E+00_JPRB, 0.71133E+00_JPRB, 0.71600E+00_JPRB, 0.75112E+00_JPRB, 0.76410E+00_JPRB, 0.81083E+00_JPRB, 0.95447E+00_JPRB ,&
&  0.11857E+01_JPRB, 0.14483E+01_JPRB, 0.18436E+01_JPRB, 0.25218E+01_JPRB, 0.27980E+01_JPRB, 0.22642E+01_JPRB, 0.23937E+01_JPRB ,&
&  0.22535E+01_JPRB, 0.18948E+01_JPRB, 0.17414E+01_JPRB, 0.16343E+01_JPRB, 0.16218E+01_JPRB, 0.16280E+01_JPRB, 0.16640E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.17450E+01_JPRB, 0.17767E+01_JPRB, 0.17961E+01_JPRB, 0.18259E+01_JPRB, 0.18671E+01_JPRB, 0.19273E+01_JPRB ,&
&  0.19715E+01_JPRB, 0.19805E+01_JPRB, 0.20219E+01_JPRB, 0.19920E+01_JPRB, 0.21269E+01_JPRB, 0.22823E+01_JPRB, 0.24438E+01_JPRB ,&
&  0.25320E+01_JPRB, 0.25689E+01_JPRB, 0.27506E+01_JPRB, 0.27754E+01_JPRB, 0.27649E+01_JPRB, 0.27236E+01_JPRB, 0.25691E+01_JPRB ,&
&  0.23634E+01_JPRB, 0.19169E+01_JPRB, 0.17899E+01_JPRB, 0.28520E+01_JPRB, 0.28343E+01_JPRB, 0.23000E+01_JPRB, 0.23512E+01_JPRB ,&
&  0.21544E+01_JPRB, 0.20248E+01_JPRB, 0.17676E+01_JPRB, 0.14472E+01_JPRB, 0.12044E+01_JPRB, 0.10400E+01_JPRB, 0.94353E+00_JPRB ,&
&  0.89572E+00_JPRB, 0.78448E+00_JPRB, 0.75638E+00_JPRB, 0.74196E+00_JPRB, 0.68275E+00_JPRB, 0.66165E+00_JPRB, 0.63881E+00_JPRB ,&
&  0.63298E+00_JPRB, 0.65088E+00_JPRB, 0.79737E+00_JPRB, 0.10439E+01_JPRB, 0.82407E+00_JPRB, 0.46364E+00_JPRB, 0.41383E+00_JPRB ,&
&  0.42800E+00_JPRB, 0.44192E+00_JPRB, 0.45519E+00_JPRB, 0.46735E+00_JPRB, 0.47705E+00_JPRB, 0.49710E+00_JPRB, 0.51449E+00_JPRB ,&
&  0.53275E+00_JPRB, 0.56198E+00_JPRB, 0.57367E+00_JPRB, 0.58799E+00_JPRB, 0.60491E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.88671E+00_JPRB, 0.93505E+00_JPRB, 0.97930E+00_JPRB, 0.10068E+01_JPRB, 0.10127E+01_JPRB, 0.10226E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.10340E+01_JPRB, 0.10668E+01_JPRB, 0.11413E+01_JPRB, 0.12830E+01_JPRB, 0.13618E+01_JPRB, 0.14616E+01_JPRB ,&
&  0.14735E+01_JPRB, 0.15017E+01_JPRB, 0.15355E+01_JPRB, 0.15887E+01_JPRB, 0.16834E+01_JPRB, 0.18163E+01_JPRB, 0.20916E+01_JPRB ,&
&  0.25916E+01_JPRB, 0.29340E+01_JPRB, 0.29797E+01_JPRB, 0.27238E+01_JPRB, 0.22806E+01_JPRB, 0.21331E+01_JPRB, 0.23943E+01_JPRB ,&
&  0.24650E+01_JPRB, 0.21456E+01_JPRB, 0.18686E+01_JPRB, 0.18036E+01_JPRB, 0.17985E+01_JPRB, 0.18150E+01_JPRB, 0.18242E+01_JPRB ,&
&  0.18466E+01_JPRB, 0.18757E+01_JPRB, 0.18912E+01_JPRB, 0.19093E+01_JPRB, 0.19636E+01_JPRB, 0.20559E+01_JPRB, 0.21783E+01_JPRB ,&
&  0.23588E+01_JPRB, 0.24776E+01_JPRB, 0.25241E+01_JPRB, 0.25271E+01_JPRB, 0.25321E+01_JPRB, 0.25925E+01_JPRB, 0.27250E+01_JPRB ,&
&  0.28658E+01_JPRB, 0.29134E+01_JPRB, 0.31682E+01_JPRB, 0.33405E+01_JPRB, 0.32670E+01_JPRB, 0.31852E+01_JPRB, 0.31200E+01_JPRB ,&
&  0.30753E+01_JPRB, 0.30092E+01_JPRB, 0.26718E+01_JPRB, 0.37485E+01_JPRB, 0.38351E+01_JPRB, 0.32324E+01_JPRB, 0.26665E+01_JPRB ,&
&  0.23952E+01_JPRB, 0.22008E+01_JPRB, 0.18188E+01_JPRB, 0.14209E+01_JPRB, 0.12011E+01_JPRB, 0.11341E+01_JPRB, 0.98107E+00_JPRB ,&
&  0.10139E+01_JPRB, 0.87043E+00_JPRB, 0.78504E+00_JPRB, 0.74517E+00_JPRB, 0.68594E+00_JPRB, 0.69507E+00_JPRB, 0.65422E+00_JPRB ,&
&  0.59175E+00_JPRB, 0.69498E+00_JPRB, 0.89362E+00_JPRB, 0.13543E+01_JPRB, 0.21001E+01_JPRB, 0.57405E+00_JPRB, 0.45374E+00_JPRB ,&
&  0.47223E+00_JPRB, 0.50358E+00_JPRB, 0.54026E+00_JPRB, 0.56394E+00_JPRB, 0.57736E+00_JPRB, 0.61255E+00_JPRB, 0.66732E+00_JPRB ,&
&  0.70044E+00_JPRB, 0.72496E+00_JPRB, 0.75463E+00_JPRB, 0.81543E+00_JPRB, 0.84738E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.10966E+01_JPRB, 0.11455E+01_JPRB, 0.11789E+01_JPRB, 0.12277E+01_JPRB, 0.12743E+01_JPRB, 0.13230E+01_JPRB, 0.13932E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.15921E+01_JPRB, 0.16916E+01_JPRB, 0.17687E+01_JPRB, 0.18419E+01_JPRB, 0.18948E+01_JPRB, 0.19398E+01_JPRB ,&
&  0.19718E+01_JPRB, 0.20464E+01_JPRB, 0.21326E+01_JPRB, 0.22501E+01_JPRB, 0.23922E+01_JPRB, 0.25557E+01_JPRB, 0.27374E+01_JPRB ,&
&  0.28805E+01_JPRB, 0.28214E+01_JPRB, 0.25111E+01_JPRB, 0.18851E+01_JPRB, 0.17698E+01_JPRB, 0.21801E+01_JPRB, 0.20780E+01_JPRB ,&
&  0.20380E+01_JPRB, 0.19226E+01_JPRB, 0.18323E+01_JPRB, 0.18517E+01_JPRB, 0.18770E+01_JPRB, 0.19029E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.19427E+01_JPRB, 0.19844E+01_JPRB, 0.20111E+01_JPRB, 0.20359E+01_JPRB, 0.21022E+01_JPRB, 0.22441E+01_JPRB, 0.24606E+01_JPRB ,&
&  0.27295E+01_JPRB, 0.28869E+01_JPRB, 0.29854E+01_JPRB, 0.30504E+01_JPRB, 0.29522E+01_JPRB, 0.28859E+01_JPRB, 0.29107E+01_JPRB ,&
&  0.30332E+01_JPRB, 0.31881E+01_JPRB, 0.34786E+01_JPRB, 0.37388E+01_JPRB, 0.38318E+01_JPRB, 0.36420E+01_JPRB, 0.35869E+01_JPRB ,&
&  0.36296E+01_JPRB, 0.35283E+01_JPRB, 0.35464E+01_JPRB, 0.44233E+01_JPRB, 0.28758E+01_JPRB, 0.28437E+01_JPRB, 0.26847E+01_JPRB ,&
&  0.25343E+01_JPRB, 0.24950E+01_JPRB, 0.19106E+01_JPRB, 0.14040E+01_JPRB, 0.12007E+01_JPRB, 0.91647E+00_JPRB, 0.96618E+00_JPRB ,&
&  0.11079E+01_JPRB, 0.10137E+01_JPRB, 0.85035E+00_JPRB, 0.77171E+00_JPRB, 0.75439E+00_JPRB, 0.73862E+00_JPRB, 0.73345E+00_JPRB ,&
&  0.66336E+00_JPRB, 0.71301E+00_JPRB, 0.85760E+00_JPRB, 0.11943E+01_JPRB, 0.12174E+01_JPRB, 0.61847E+00_JPRB, 0.50903E+00_JPRB ,&
&  0.51708E+00_JPRB, 0.56429E+00_JPRB, 0.61988E+00_JPRB, 0.66843E+00_JPRB, 0.71152E+00_JPRB, 0.76077E+00_JPRB, 0.81470E+00_JPRB ,&
&  0.87596E+00_JPRB, 0.94858E+00_JPRB, 0.99714E+00_JPRB, 0.10337E+01_JPRB, 0.10602E+01_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.11285E+01_JPRB, 0.11813E+01_JPRB, 0.12469E+01_JPRB, 0.13301E+01_JPRB, 0.14160E+01_JPRB, 0.14520E+01_JPRB, 0.14565E+01_JPRB ,&
&  0.15047E+01_JPRB, 0.15783E+01_JPRB, 0.16662E+01_JPRB, 0.17352E+01_JPRB, 0.17710E+01_JPRB, 0.18323E+01_JPRB, 0.19057E+01_JPRB ,&
&  0.19757E+01_JPRB, 0.20585E+01_JPRB, 0.21684E+01_JPRB, 0.23052E+01_JPRB, 0.24811E+01_JPRB, 0.27019E+01_JPRB, 0.28085E+01_JPRB ,&
&  0.22576E+01_JPRB, 0.16457E+01_JPRB, 0.22173E+01_JPRB, 0.23801E+01_JPRB, 0.24913E+01_JPRB, 0.27611E+01_JPRB, 0.26688E+01_JPRB ,&
&  0.20921E+01_JPRB, 0.18931E+01_JPRB, 0.19095E+01_JPRB, 0.18704E+01_JPRB, 0.18256E+01_JPRB, 0.18709E+01_JPRB, 0.19183E+01_JPRB ,&
&  0.19643E+01_JPRB, 0.20092E+01_JPRB, 0.20529E+01_JPRB, 0.20949E+01_JPRB, 0.21863E+01_JPRB, 0.23558E+01_JPRB, 0.26321E+01_JPRB ,&
&  0.29027E+01_JPRB, 0.30638E+01_JPRB, 0.31986E+01_JPRB, 0.32567E+01_JPRB, 0.31688E+01_JPRB, 0.30962E+01_JPRB, 0.31369E+01_JPRB ,&
&  0.32857E+01_JPRB, 0.34857E+01_JPRB, 0.37535E+01_JPRB, 0.39522E+01_JPRB, 0.41405E+01_JPRB, 0.38375E+01_JPRB, 0.39804E+01_JPRB ,&
&  0.41159E+01_JPRB, 0.39567E+01_JPRB, 0.41483E+01_JPRB, 0.46043E+01_JPRB, 0.31028E+01_JPRB, 0.28739E+01_JPRB, 0.24615E+01_JPRB ,&
&  0.26121E+01_JPRB, 0.25482E+01_JPRB, 0.21943E+01_JPRB, 0.17398E+01_JPRB, 0.13988E+01_JPRB, 0.10343E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.12784E+01_JPRB, 0.97694E+00_JPRB, 0.85795E+00_JPRB, 0.71028E+00_JPRB, 0.67112E+00_JPRB, 0.70575E+00_JPRB ,&
&  0.73898E+00_JPRB, 0.72038E+00_JPRB, 0.79374E+00_JPRB, 0.95873E+00_JPRB, 0.89700E+00_JPRB, 0.66418E+00_JPRB, 0.56291E+00_JPRB ,&
&  0.56037E+00_JPRB, 0.60265E+00_JPRB, 0.66994E+00_JPRB, 0.74807E+00_JPRB, 0.78641E+00_JPRB, 0.82876E+00_JPRB, 0.86667E+00_JPRB ,&
&  0.94762E+00_JPRB, 0.10042E+01_JPRB, 0.10445E+01_JPRB, 0.10559E+01_JPRB, 0.10936E+01_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.10617E+01_JPRB, 0.11196E+01_JPRB, 0.12015E+01_JPRB, 0.13220E+01_JPRB, 0.14866E+01_JPRB, 0.16478E+01_JPRB, 0.15604E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.13173E+01_JPRB, 0.13616E+01_JPRB, 0.14059E+01_JPRB, 0.14515E+01_JPRB, 0.15301E+01_JPRB, 0.16426E+01_JPRB ,&
&  0.17767E+01_JPRB, 0.18993E+01_JPRB, 0.20267E+01_JPRB, 0.21497E+01_JPRB, 0.22443E+01_JPRB, 0.24664E+01_JPRB, 0.23477E+01_JPRB ,&
&  0.17870E+01_JPRB, 0.15946E+01_JPRB, 0.21781E+01_JPRB, 0.20158E+01_JPRB, 0.21840E+01_JPRB, 0.25027E+01_JPRB, 0.25215E+01_JPRB ,&
&  0.19425E+01_JPRB, 0.17663E+01_JPRB, 0.17701E+01_JPRB, 0.17318E+01_JPRB, 0.16886E+01_JPRB, 0.17582E+01_JPRB, 0.18450E+01_JPRB ,&
&  0.19022E+01_JPRB, 0.19390E+01_JPRB, 0.19843E+01_JPRB, 0.20327E+01_JPRB, 0.21183E+01_JPRB, 0.22869E+01_JPRB, 0.26110E+01_JPRB ,&
&  0.28703E+01_JPRB, 0.31217E+01_JPRB, 0.33696E+01_JPRB, 0.34181E+01_JPRB, 0.33509E+01_JPRB, 0.33757E+01_JPRB, 0.35158E+01_JPRB ,&
&  0.36930E+01_JPRB, 0.39615E+01_JPRB, 0.40770E+01_JPRB, 0.40634E+01_JPRB, 0.38023E+01_JPRB, 0.41745E+01_JPRB, 0.39556E+01_JPRB ,&
&  0.42512E+01_JPRB, 0.42098E+01_JPRB, 0.48368E+01_JPRB, 0.39365E+01_JPRB, 0.36574E+01_JPRB, 0.34742E+01_JPRB, 0.31492E+01_JPRB ,&
&  0.27263E+01_JPRB, 0.27571E+01_JPRB, 0.25266E+01_JPRB, 0.20823E+01_JPRB, 0.16981E+01_JPRB, 0.13817E+01_JPRB, 0.15426E+01_JPRB ,&
&  0.15858E+01_JPRB, 0.15905E+01_JPRB, 0.10915E+01_JPRB, 0.79932E+00_JPRB, 0.61855E+00_JPRB, 0.57176E+00_JPRB, 0.65120E+00_JPRB ,&
&  0.78078E+00_JPRB, 0.80443E+00_JPRB, 0.78671E+00_JPRB, 0.82236E+00_JPRB, 0.79883E+00_JPRB, 0.71015E+00_JPRB, 0.64694E+00_JPRB ,&
&  0.65878E+00_JPRB, 0.66870E+00_JPRB, 0.71024E+00_JPRB, 0.78278E+00_JPRB, 0.82103E+00_JPRB, 0.86462E+00_JPRB, 0.86025E+00_JPRB ,&
&  0.89998E+00_JPRB, 0.93859E+00_JPRB, 0.96022E+00_JPRB, 0.98449E+00_JPRB, 0.10126E+01_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.90574E+00_JPRB, 0.92406E+00_JPRB, 0.96845E+00_JPRB, 0.10379E+01_JPRB, 0.10982E+01_JPRB, 0.11557E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.11412E+01_JPRB, 0.11341E+01_JPRB, 0.11019E+01_JPRB, 0.10834E+01_JPRB, 0.10792E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.15161E+01_JPRB, 0.17501E+01_JPRB, 0.19253E+01_JPRB, 0.20174E+01_JPRB, 0.21832E+01_JPRB, 0.19152E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.22381E+01_JPRB, 0.17844E+01_JPRB, 0.18960E+01_JPRB, 0.20040E+01_JPRB, 0.22087E+01_JPRB, 0.19005E+01_JPRB ,&
&  0.16543E+01_JPRB, 0.16131E+01_JPRB, 0.16293E+01_JPRB, 0.16321E+01_JPRB, 0.16014E+01_JPRB, 0.16411E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.18130E+01_JPRB, 0.18700E+01_JPRB, 0.18981E+01_JPRB, 0.19150E+01_JPRB, 0.19694E+01_JPRB, 0.20748E+01_JPRB, 0.23697E+01_JPRB ,&
&  0.26552E+01_JPRB, 0.29965E+01_JPRB, 0.33386E+01_JPRB, 0.36467E+01_JPRB, 0.35759E+01_JPRB, 0.36723E+01_JPRB, 0.39658E+01_JPRB ,&
&  0.40521E+01_JPRB, 0.39822E+01_JPRB, 0.42524E+01_JPRB, 0.43052E+01_JPRB, 0.42748E+01_JPRB, 0.45373E+01_JPRB, 0.39869E+01_JPRB ,&
&  0.43886E+01_JPRB, 0.42306E+01_JPRB, 0.48525E+01_JPRB, 0.41867E+01_JPRB, 0.39545E+01_JPRB, 0.41312E+01_JPRB, 0.38287E+01_JPRB ,&
&  0.32916E+01_JPRB, 0.32038E+01_JPRB, 0.28304E+01_JPRB, 0.24307E+01_JPRB, 0.21696E+01_JPRB, 0.20071E+01_JPRB, 0.21198E+01_JPRB ,&
&  0.20905E+01_JPRB, 0.18566E+01_JPRB, 0.13428E+01_JPRB, 0.72829E+00_JPRB, 0.56785E+00_JPRB, 0.58426E+00_JPRB, 0.69278E+00_JPRB ,&
&  0.84720E+00_JPRB, 0.91959E+00_JPRB, 0.97286E+00_JPRB, 0.92335E+00_JPRB, 0.82430E+00_JPRB, 0.75608E+00_JPRB, 0.70802E+00_JPRB ,&
&  0.69767E+00_JPRB, 0.69484E+00_JPRB, 0.70583E+00_JPRB, 0.77794E+00_JPRB, 0.86352E+00_JPRB, 0.91165E+00_JPRB, 0.84706E+00_JPRB ,&
&  0.80023E+00_JPRB, 0.79848E+00_JPRB, 0.83427E+00_JPRB, 0.86280E+00_JPRB, 0.88840E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.79712E+00_JPRB, 0.78733E+00_JPRB, 0.79939E+00_JPRB, 0.83900E+00_JPRB, 0.85878E+00_JPRB, 0.87087E+00_JPRB, 0.92861E+00_JPRB ,&
&  0.95114E+00_JPRB, 0.94554E+00_JPRB, 0.95578E+00_JPRB, 0.95960E+00_JPRB, 0.94196E+00_JPRB, 0.89869E+00_JPRB, 0.87029E+00_JPRB ,&
&  0.88198E+00_JPRB, 0.10013E+01_JPRB, 0.12579E+01_JPRB, 0.16084E+01_JPRB, 0.18574E+01_JPRB, 0.22749E+01_JPRB, 0.15955E+01_JPRB ,&
&  0.14769E+01_JPRB, 0.18637E+01_JPRB, 0.16894E+01_JPRB, 0.18404E+01_JPRB, 0.19223E+01_JPRB, 0.20160E+01_JPRB, 0.16895E+01_JPRB ,&
&  0.16005E+01_JPRB, 0.15965E+01_JPRB, 0.15714E+01_JPRB, 0.15798E+01_JPRB, 0.15882E+01_JPRB, 0.16009E+01_JPRB, 0.16056E+01_JPRB ,&
&  0.16529E+01_JPRB, 0.17276E+01_JPRB, 0.17784E+01_JPRB, 0.17924E+01_JPRB, 0.18125E+01_JPRB, 0.18726E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.23993E+01_JPRB, 0.27222E+01_JPRB, 0.31555E+01_JPRB, 0.34963E+01_JPRB, 0.37518E+01_JPRB, 0.38971E+01_JPRB, 0.43279E+01_JPRB ,&
&  0.43564E+01_JPRB, 0.42577E+01_JPRB, 0.45761E+01_JPRB, 0.48117E+01_JPRB, 0.49700E+01_JPRB, 0.50733E+01_JPRB, 0.45422E+01_JPRB ,&
&  0.44331E+01_JPRB, 0.45360E+01_JPRB, 0.46697E+01_JPRB, 0.41470E+01_JPRB, 0.40378E+01_JPRB, 0.44576E+01_JPRB, 0.45775E+01_JPRB ,&
&  0.41080E+01_JPRB, 0.37147E+01_JPRB, 0.33946E+01_JPRB, 0.27902E+01_JPRB, 0.25347E+01_JPRB, 0.24702E+01_JPRB, 0.27817E+01_JPRB ,&
&  0.26742E+01_JPRB, 0.22605E+01_JPRB, 0.11867E+01_JPRB, 0.60714E+00_JPRB, 0.49707E+00_JPRB, 0.60879E+00_JPRB, 0.79458E+00_JPRB ,&
&  0.10044E+01_JPRB, 0.11478E+01_JPRB, 0.12339E+01_JPRB, 0.13917E+01_JPRB, 0.12295E+01_JPRB, 0.10402E+01_JPRB, 0.91260E+00_JPRB ,&
&  0.82316E+00_JPRB, 0.74796E+00_JPRB, 0.71354E+00_JPRB, 0.72995E+00_JPRB, 0.81883E+00_JPRB, 0.91328E+00_JPRB, 0.96231E+00_JPRB ,&
&  0.90785E+00_JPRB, 0.81445E+00_JPRB, 0.77024E+00_JPRB, 0.76049E+00_JPRB, 0.77471E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.71695E+00_JPRB, 0.71529E+00_JPRB, 0.72814E+00_JPRB, 0.75792E+00_JPRB, 0.78133E+00_JPRB, 0.77452E+00_JPRB, 0.75126E+00_JPRB ,&
&  0.76757E+00_JPRB, 0.76416E+00_JPRB, 0.79281E+00_JPRB, 0.80973E+00_JPRB, 0.82886E+00_JPRB, 0.81884E+00_JPRB, 0.79979E+00_JPRB ,&
&  0.77237E+00_JPRB, 0.79061E+00_JPRB, 0.96725E+00_JPRB, 0.13060E+01_JPRB, 0.16106E+01_JPRB, 0.18138E+01_JPRB, 0.18419E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.18270E+01_JPRB, 0.18854E+01_JPRB, 0.22399E+01_JPRB, 0.24293E+01_JPRB, 0.27696E+01_JPRB, 0.18068E+01_JPRB ,&
&  0.16450E+01_JPRB, 0.16248E+01_JPRB, 0.16203E+01_JPRB, 0.16389E+01_JPRB, 0.16766E+01_JPRB, 0.16578E+01_JPRB, 0.16421E+01_JPRB ,&
&  0.16028E+01_JPRB, 0.16018E+01_JPRB, 0.16239E+01_JPRB, 0.16498E+01_JPRB, 0.16781E+01_JPRB, 0.17261E+01_JPRB, 0.19042E+01_JPRB ,&
&  0.22078E+01_JPRB, 0.25738E+01_JPRB, 0.30015E+01_JPRB, 0.34747E+01_JPRB, 0.37946E+01_JPRB, 0.41502E+01_JPRB, 0.47155E+01_JPRB ,&
&  0.49836E+01_JPRB, 0.52284E+01_JPRB, 0.52184E+01_JPRB, 0.51233E+01_JPRB, 0.52518E+01_JPRB, 0.58063E+01_JPRB, 0.53919E+01_JPRB ,&
&  0.48780E+01_JPRB, 0.44854E+01_JPRB, 0.41812E+01_JPRB, 0.40344E+01_JPRB, 0.40228E+01_JPRB, 0.47076E+01_JPRB, 0.45052E+01_JPRB ,&
&  0.38071E+01_JPRB, 0.33255E+01_JPRB, 0.31905E+01_JPRB, 0.31171E+01_JPRB, 0.30715E+01_JPRB, 0.31020E+01_JPRB, 0.37049E+01_JPRB ,&
&  0.28089E+01_JPRB, 0.13213E+01_JPRB, 0.74529E+00_JPRB, 0.68830E+00_JPRB, 0.44650E+00_JPRB, 0.47638E+00_JPRB, 0.11036E+01_JPRB ,&
&  0.15102E+01_JPRB, 0.19435E+01_JPRB, 0.20411E+01_JPRB, 0.19072E+01_JPRB, 0.17947E+01_JPRB, 0.15281E+01_JPRB, 0.12584E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.90147E+00_JPRB, 0.84457E+00_JPRB, 0.82444E+00_JPRB, 0.88154E+00_JPRB, 0.96887E+00_JPRB, 0.10329E+01_JPRB ,&
&  0.10240E+01_JPRB, 0.95107E+00_JPRB, 0.82486E+00_JPRB, 0.72299E+00_JPRB, 0.69893E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.72000E+00_JPRB, 0.73742E+00_JPRB, 0.76510E+00_JPRB, 0.76184E+00_JPRB, 0.74669E+00_JPRB, 0.75701E+00_JPRB, 0.73128E+00_JPRB ,&
&  0.71081E+00_JPRB, 0.70203E+00_JPRB, 0.70579E+00_JPRB, 0.72394E+00_JPRB, 0.75056E+00_JPRB, 0.76777E+00_JPRB, 0.74880E+00_JPRB ,&
&  0.72061E+00_JPRB, 0.73377E+00_JPRB, 0.83029E+00_JPRB, 0.11317E+01_JPRB, 0.15192E+01_JPRB, 0.19222E+01_JPRB, 0.20042E+01_JPRB ,&
&  0.18430E+01_JPRB, 0.23799E+01_JPRB, 0.31877E+01_JPRB, 0.35060E+01_JPRB, 0.32703E+01_JPRB, 0.33697E+01_JPRB, 0.23624E+01_JPRB ,&
&  0.19590E+01_JPRB, 0.19381E+01_JPRB, 0.19222E+01_JPRB, 0.19218E+01_JPRB, 0.18754E+01_JPRB, 0.17793E+01_JPRB, 0.17351E+01_JPRB ,&
&  0.16939E+01_JPRB, 0.15833E+01_JPRB, 0.15337E+01_JPRB, 0.15281E+01_JPRB, 0.15626E+01_JPRB, 0.16142E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.20420E+01_JPRB, 0.23910E+01_JPRB, 0.28318E+01_JPRB, 0.37963E+01_JPRB, 0.33220E+01_JPRB, 0.44863E+01_JPRB, 0.50360E+01_JPRB ,&
&  0.52902E+01_JPRB, 0.54986E+01_JPRB, 0.52131E+01_JPRB, 0.57535E+01_JPRB, 0.59306E+01_JPRB, 0.63474E+01_JPRB, 0.60586E+01_JPRB ,&
&  0.54947E+01_JPRB, 0.46433E+01_JPRB, 0.44055E+01_JPRB, 0.42275E+01_JPRB, 0.41045E+01_JPRB, 0.48686E+01_JPRB, 0.35966E+01_JPRB ,&
&  0.32922E+01_JPRB, 0.38331E+01_JPRB, 0.36773E+01_JPRB, 0.33000E+01_JPRB, 0.35542E+01_JPRB, 0.37082E+01_JPRB, 0.28392E+01_JPRB ,&
&  0.12142E+01_JPRB, 0.83708E+00_JPRB, 0.70092E+00_JPRB, 0.60287E+00_JPRB, 0.50399E+00_JPRB, 0.69537E+00_JPRB, 0.27675E+01_JPRB ,&
&  0.19918E+01_JPRB, 0.28348E+01_JPRB, 0.29153E+01_JPRB, 0.27817E+01_JPRB, 0.26681E+01_JPRB, 0.23791E+01_JPRB, 0.25028E+01_JPRB ,&
&  0.21158E+01_JPRB, 0.17298E+01_JPRB, 0.14568E+01_JPRB, 0.12741E+01_JPRB, 0.11678E+01_JPRB, 0.11453E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.10633E+01_JPRB, 0.97605E+00_JPRB, 0.84598E+00_JPRB, 0.73561E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.83041E+00_JPRB, 0.81797E+00_JPRB, 0.82120E+00_JPRB, 0.81115E+00_JPRB, 0.79284E+00_JPRB, 0.79809E+00_JPRB, 0.78648E+00_JPRB ,&
&  0.75589E+00_JPRB, 0.72878E+00_JPRB, 0.71961E+00_JPRB, 0.73519E+00_JPRB, 0.74468E+00_JPRB, 0.75048E+00_JPRB, 0.72970E+00_JPRB ,&
&  0.70160E+00_JPRB, 0.71779E+00_JPRB, 0.89823E+00_JPRB, 0.10260E+01_JPRB, 0.14638E+01_JPRB, 0.18465E+01_JPRB, 0.21215E+01_JPRB ,&
&  0.20666E+01_JPRB, 0.26275E+01_JPRB, 0.34842E+01_JPRB, 0.38428E+01_JPRB, 0.49868E+01_JPRB, 0.48341E+01_JPRB, 0.40927E+01_JPRB ,&
&  0.30739E+01_JPRB, 0.29305E+01_JPRB, 0.28227E+01_JPRB, 0.25938E+01_JPRB, 0.23066E+01_JPRB, 0.20346E+01_JPRB, 0.18363E+01_JPRB ,&
&  0.17232E+01_JPRB, 0.16547E+01_JPRB, 0.15261E+01_JPRB, 0.14609E+01_JPRB, 0.14796E+01_JPRB, 0.15272E+01_JPRB, 0.16754E+01_JPRB ,&
&  0.19261E+01_JPRB, 0.22333E+01_JPRB, 0.26149E+01_JPRB, 0.35556E+01_JPRB, 0.37312E+01_JPRB, 0.41750E+01_JPRB, 0.51447E+01_JPRB ,&
&  0.56178E+01_JPRB, 0.60343E+01_JPRB, 0.56562E+01_JPRB, 0.67801E+01_JPRB, 0.63857E+01_JPRB, 0.69264E+01_JPRB, 0.71357E+01_JPRB ,&
&  0.62133E+01_JPRB, 0.57923E+01_JPRB, 0.45171E+01_JPRB, 0.43949E+01_JPRB, 0.45149E+01_JPRB, 0.39915E+01_JPRB, 0.37656E+01_JPRB ,&
&  0.41996E+01_JPRB, 0.37657E+01_JPRB, 0.34641E+01_JPRB, 0.32541E+01_JPRB, 0.39485E+01_JPRB, 0.35168E+01_JPRB, 0.16138E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.10293E+01_JPRB, 0.75811E+00_JPRB, 0.64252E+00_JPRB, 0.61077E+00_JPRB, 0.79391E+00_JPRB, 0.15360E+01_JPRB ,&
&  0.17503E+01_JPRB, 0.28376E+01_JPRB, 0.30442E+01_JPRB, 0.32135E+01_JPRB, 0.30457E+01_JPRB, 0.28871E+01_JPRB, 0.32676E+01_JPRB ,&
&  0.32643E+01_JPRB, 0.28040E+01_JPRB, 0.22156E+01_JPRB, 0.18923E+01_JPRB, 0.16542E+01_JPRB, 0.14775E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.12690E+01_JPRB, 0.11554E+01_JPRB, 0.10846E+01_JPRB, 0.10247E+01_JPRB, 0.90933E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.95946E+00_JPRB, 0.90894E+00_JPRB, 0.88233E+00_JPRB, 0.83640E+00_JPRB, 0.80007E+00_JPRB, 0.78945E+00_JPRB, 0.80850E+00_JPRB ,&
&  0.80019E+00_JPRB, 0.77755E+00_JPRB, 0.75674E+00_JPRB, 0.75652E+00_JPRB, 0.76944E+00_JPRB, 0.77169E+00_JPRB, 0.74375E+00_JPRB ,&
&  0.71894E+00_JPRB, 0.73803E+00_JPRB, 0.90181E+00_JPRB, 0.10034E+01_JPRB, 0.12571E+01_JPRB, 0.16242E+01_JPRB, 0.18938E+01_JPRB ,&
&  0.22104E+01_JPRB, 0.27894E+01_JPRB, 0.35287E+01_JPRB, 0.47200E+01_JPRB, 0.61040E+01_JPRB, 0.66749E+01_JPRB, 0.70242E+01_JPRB ,&
&  0.57869E+01_JPRB, 0.46057E+01_JPRB, 0.39002E+01_JPRB, 0.33913E+01_JPRB, 0.28137E+01_JPRB, 0.23493E+01_JPRB, 0.19935E+01_JPRB ,&
&  0.17776E+01_JPRB, 0.16147E+01_JPRB, 0.15734E+01_JPRB, 0.14787E+01_JPRB, 0.14478E+01_JPRB, 0.14718E+01_JPRB, 0.16230E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.21448E+01_JPRB, 0.26160E+01_JPRB, 0.32924E+01_JPRB, 0.42166E+01_JPRB, 0.52365E+01_JPRB, 0.54825E+01_JPRB ,&
&  0.52642E+01_JPRB, 0.52666E+01_JPRB, 0.63335E+01_JPRB, 0.69741E+01_JPRB, 0.67578E+01_JPRB, 0.78796E+01_JPRB, 0.80541E+01_JPRB ,&
&  0.69066E+01_JPRB, 0.60230E+01_JPRB, 0.49447E+01_JPRB, 0.52890E+01_JPRB, 0.44934E+01_JPRB, 0.38084E+01_JPRB, 0.43913E+01_JPRB ,&
&  0.46528E+01_JPRB, 0.41371E+01_JPRB, 0.34987E+01_JPRB, 0.30459E+01_JPRB, 0.29756E+01_JPRB, 0.22513E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.13134E+01_JPRB, 0.10607E+01_JPRB, 0.86456E+00_JPRB, 0.88444E+00_JPRB, 0.74326E+00_JPRB, 0.92101E+00_JPRB, 0.14631E+01_JPRB ,&
&  0.21941E+01_JPRB, 0.25792E+01_JPRB, 0.31660E+01_JPRB, 0.30771E+01_JPRB, 0.28802E+01_JPRB, 0.29306E+01_JPRB, 0.35743E+01_JPRB ,&
&  0.29841E+01_JPRB, 0.29116E+01_JPRB, 0.27355E+01_JPRB, 0.23295E+01_JPRB, 0.20237E+01_JPRB, 0.18010E+01_JPRB, 0.15927E+01_JPRB ,&
&  0.13971E+01_JPRB, 0.12297E+01_JPRB, 0.11329E+01_JPRB, 0.10956E+01_JPRB, 0.10402E+01_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.10353E+01_JPRB, 0.99053E+00_JPRB, 0.92171E+00_JPRB, 0.82600E+00_JPRB, 0.77800E+00_JPRB, 0.76983E+00_JPRB, 0.78335E+00_JPRB ,&
&  0.79315E+00_JPRB, 0.79918E+00_JPRB, 0.79383E+00_JPRB, 0.78765E+00_JPRB, 0.79660E+00_JPRB, 0.79027E+00_JPRB, 0.76063E+00_JPRB ,&
&  0.73604E+00_JPRB, 0.76866E+00_JPRB, 0.67530E+00_JPRB, 0.72678E+00_JPRB, 0.96356E+00_JPRB, 0.14503E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.20436E+01_JPRB, 0.27297E+01_JPRB, 0.33440E+01_JPRB, 0.47751E+01_JPRB, 0.81620E+01_JPRB, 0.82171E+01_JPRB, 0.78551E+01_JPRB ,&
&  0.72528E+01_JPRB, 0.58997E+01_JPRB, 0.46559E+01_JPRB, 0.36834E+01_JPRB, 0.29492E+01_JPRB, 0.24603E+01_JPRB, 0.20992E+01_JPRB ,&
&  0.18710E+01_JPRB, 0.16807E+01_JPRB, 0.15540E+01_JPRB, 0.15306E+01_JPRB, 0.14889E+01_JPRB, 0.14666E+01_JPRB, 0.15884E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.20401E+01_JPRB, 0.26876E+01_JPRB, 0.35194E+01_JPRB, 0.40999E+01_JPRB, 0.48810E+01_JPRB, 0.63393E+01_JPRB ,&
&  0.58285E+01_JPRB, 0.60237E+01_JPRB, 0.60650E+01_JPRB, 0.74304E+01_JPRB, 0.75410E+01_JPRB, 0.81798E+01_JPRB, 0.89736E+01_JPRB ,&
&  0.63443E+01_JPRB, 0.50754E+01_JPRB, 0.49956E+01_JPRB, 0.53997E+01_JPRB, 0.42731E+01_JPRB, 0.46312E+01_JPRB, 0.62236E+01_JPRB ,&
&  0.48057E+01_JPRB, 0.47304E+01_JPRB, 0.47954E+01_JPRB, 0.51341E+01_JPRB, 0.29918E+01_JPRB, 0.19871E+01_JPRB, 0.30438E+01_JPRB ,&
&  0.29054E+01_JPRB, 0.16281E+01_JPRB, 0.11463E+01_JPRB, 0.14379E+01_JPRB, 0.10869E+01_JPRB, 0.12868E+01_JPRB, 0.19368E+01_JPRB ,&
&  0.21686E+01_JPRB, 0.24653E+01_JPRB, 0.36618E+01_JPRB, 0.34474E+01_JPRB, 0.32087E+01_JPRB, 0.29411E+01_JPRB, 0.29721E+01_JPRB ,&
&  0.30891E+01_JPRB, 0.27707E+01_JPRB, 0.28030E+01_JPRB, 0.24188E+01_JPRB, 0.21220E+01_JPRB, 0.19177E+01_JPRB, 0.17046E+01_JPRB ,&
&  0.15262E+01_JPRB, 0.13510E+01_JPRB, 0.12084E+01_JPRB, 0.11168E+01_JPRB, 0.10733E+01_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.10682E+01_JPRB, 0.10178E+01_JPRB, 0.96043E+00_JPRB, 0.89376E+00_JPRB, 0.82127E+00_JPRB, 0.78586E+00_JPRB, 0.77244E+00_JPRB ,&
&  0.76897E+00_JPRB, 0.78116E+00_JPRB, 0.79393E+00_JPRB, 0.80162E+00_JPRB, 0.80668E+00_JPRB, 0.80803E+00_JPRB, 0.76706E+00_JPRB ,&
&  0.73617E+00_JPRB, 0.71184E+00_JPRB, 0.61691E+00_JPRB, 0.61809E+00_JPRB, 0.86808E+00_JPRB, 0.11704E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.17508E+01_JPRB, 0.24454E+01_JPRB, 0.32602E+01_JPRB, 0.47860E+01_JPRB, 0.63036E+01_JPRB, 0.72075E+01_JPRB, 0.67310E+01_JPRB ,&
&  0.66378E+01_JPRB, 0.59717E+01_JPRB, 0.49783E+01_JPRB, 0.41522E+01_JPRB, 0.33834E+01_JPRB, 0.27707E+01_JPRB, 0.23094E+01_JPRB ,&
&  0.20131E+01_JPRB, 0.18120E+01_JPRB, 0.16527E+01_JPRB, 0.15684E+01_JPRB, 0.15624E+01_JPRB, 0.15249E+01_JPRB, 0.15896E+01_JPRB ,&
&  0.16870E+01_JPRB, 0.18821E+01_JPRB, 0.26143E+01_JPRB, 0.38479E+01_JPRB, 0.46819E+01_JPRB, 0.50680E+01_JPRB, 0.53548E+01_JPRB ,&
&  0.55717E+01_JPRB, 0.57385E+01_JPRB, 0.64215E+01_JPRB, 0.68300E+01_JPRB, 0.78007E+01_JPRB, 0.82920E+01_JPRB, 0.98154E+01_JPRB ,&
&  0.76455E+01_JPRB, 0.61424E+01_JPRB, 0.50062E+01_JPRB, 0.47112E+01_JPRB, 0.53415E+01_JPRB, 0.67438E+01_JPRB, 0.62644E+01_JPRB ,&
&  0.55816E+01_JPRB, 0.52170E+01_JPRB, 0.47827E+01_JPRB, 0.42532E+01_JPRB, 0.44498E+01_JPRB, 0.27411E+01_JPRB, 0.29611E+01_JPRB ,&
&  0.29618E+01_JPRB, 0.28625E+01_JPRB, 0.25428E+01_JPRB, 0.21146E+01_JPRB, 0.18310E+01_JPRB, 0.19438E+01_JPRB, 0.19935E+01_JPRB ,&
&  0.21302E+01_JPRB, 0.21770E+01_JPRB, 0.30826E+01_JPRB, 0.33710E+01_JPRB, 0.35475E+01_JPRB, 0.31623E+01_JPRB, 0.31272E+01_JPRB ,&
&  0.28489E+01_JPRB, 0.28524E+01_JPRB, 0.28101E+01_JPRB, 0.25250E+01_JPRB, 0.21860E+01_JPRB, 0.19221E+01_JPRB, 0.16964E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.13695E+01_JPRB, 0.12333E+01_JPRB, 0.11443E+01_JPRB, 0.11082E+01_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.11269E+01_JPRB, 0.10680E+01_JPRB, 0.99292E+00_JPRB, 0.93180E+00_JPRB, 0.86683E+00_JPRB, 0.82764E+00_JPRB, 0.81467E+00_JPRB ,&
&  0.80785E+00_JPRB, 0.79705E+00_JPRB, 0.80505E+00_JPRB, 0.82041E+00_JPRB, 0.82753E+00_JPRB, 0.82626E+00_JPRB, 0.78050E+00_JPRB ,&
&  0.76227E+00_JPRB, 0.75112E+00_JPRB, 0.61945E+00_JPRB, 0.63461E+00_JPRB, 0.76724E+00_JPRB, 0.91940E+00_JPRB, 0.11899E+01_JPRB ,&
&  0.14703E+01_JPRB, 0.20067E+01_JPRB, 0.26600E+01_JPRB, 0.37005E+01_JPRB, 0.43765E+01_JPRB, 0.46925E+01_JPRB, 0.46784E+01_JPRB ,&
&  0.42982E+01_JPRB, 0.42922E+01_JPRB, 0.40458E+01_JPRB, 0.36155E+01_JPRB, 0.31711E+01_JPRB, 0.28178E+01_JPRB, 0.25149E+01_JPRB ,&
&  0.22537E+01_JPRB, 0.20437E+01_JPRB, 0.18891E+01_JPRB, 0.17718E+01_JPRB, 0.17307E+01_JPRB, 0.17055E+01_JPRB, 0.17161E+01_JPRB ,&
&  0.16739E+01_JPRB, 0.16838E+01_JPRB, 0.21225E+01_JPRB, 0.33938E+01_JPRB, 0.41615E+01_JPRB, 0.48949E+01_JPRB, 0.49383E+01_JPRB ,&
&  0.60116E+01_JPRB, 0.68208E+01_JPRB, 0.74016E+01_JPRB, 0.87810E+01_JPRB, 0.77051E+01_JPRB, 0.89658E+01_JPRB, 0.10723E+02_JPRB ,&
&  0.10321E+02_JPRB, 0.91687E+01_JPRB, 0.83345E+01_JPRB, 0.73895E+01_JPRB, 0.62517E+01_JPRB, 0.72722E+01_JPRB, 0.60363E+01_JPRB ,&
&  0.53135E+01_JPRB, 0.51585E+01_JPRB, 0.47245E+01_JPRB, 0.47424E+01_JPRB, 0.39110E+01_JPRB, 0.34397E+01_JPRB, 0.23727E+01_JPRB ,&
&  0.23404E+01_JPRB, 0.20418E+01_JPRB, 0.20501E+01_JPRB, 0.17991E+01_JPRB, 0.17160E+01_JPRB, 0.15548E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.15837E+01_JPRB, 0.16154E+01_JPRB, 0.19819E+01_JPRB, 0.22325E+01_JPRB, 0.29478E+01_JPRB, 0.27781E+01_JPRB, 0.29058E+01_JPRB ,&
&  0.27354E+01_JPRB, 0.26624E+01_JPRB, 0.23082E+01_JPRB, 0.22066E+01_JPRB, 0.19577E+01_JPRB, 0.17500E+01_JPRB, 0.15930E+01_JPRB ,&
&  0.14326E+01_JPRB, 0.13277E+01_JPRB, 0.12453E+01_JPRB, 0.11722E+01_JPRB, 0.11569E+01_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.98921E+00_JPRB, 0.94634E+00_JPRB, 0.92493E+00_JPRB, 0.91322E+00_JPRB, 0.87174E+00_JPRB, 0.84304E+00_JPRB, 0.85053E+00_JPRB ,&
&  0.84857E+00_JPRB, 0.81831E+00_JPRB, 0.79870E+00_JPRB, 0.79740E+00_JPRB, 0.80261E+00_JPRB, 0.81251E+00_JPRB, 0.79932E+00_JPRB ,&
&  0.78877E+00_JPRB, 0.93956E+00_JPRB, 0.73678E+00_JPRB, 0.60910E+00_JPRB, 0.57776E+00_JPRB, 0.72125E+00_JPRB, 0.94411E+00_JPRB ,&
&  0.12396E+01_JPRB, 0.15475E+01_JPRB, 0.20377E+01_JPRB, 0.24019E+01_JPRB, 0.27079E+01_JPRB, 0.28815E+01_JPRB, 0.30444E+01_JPRB ,&
&  0.33039E+01_JPRB, 0.25913E+01_JPRB, 0.25002E+01_JPRB, 0.24109E+01_JPRB, 0.23383E+01_JPRB, 0.22397E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.20971E+01_JPRB, 0.20222E+01_JPRB, 0.19495E+01_JPRB, 0.18696E+01_JPRB, 0.17930E+01_JPRB, 0.17269E+01_JPRB, 0.16759E+01_JPRB ,&
&  0.16752E+01_JPRB, 0.16425E+01_JPRB, 0.17229E+01_JPRB, 0.23118E+01_JPRB, 0.35194E+01_JPRB, 0.44558E+01_JPRB, 0.52108E+01_JPRB ,&
&  0.51799E+01_JPRB, 0.56275E+01_JPRB, 0.95060E+01_JPRB, 0.99786E+01_JPRB, 0.10463E+02_JPRB, 0.11365E+02_JPRB, 0.11517E+02_JPRB ,&
&  0.11047E+02_JPRB, 0.98482E+01_JPRB, 0.86140E+01_JPRB, 0.74133E+01_JPRB, 0.78783E+01_JPRB, 0.72122E+01_JPRB, 0.62873E+01_JPRB ,&
&  0.57000E+01_JPRB, 0.57122E+01_JPRB, 0.52234E+01_JPRB, 0.46644E+01_JPRB, 0.42562E+01_JPRB, 0.42610E+01_JPRB, 0.33536E+01_JPRB ,&
&  0.25617E+01_JPRB, 0.23593E+01_JPRB, 0.16982E+01_JPRB, 0.13189E+01_JPRB, 0.11390E+01_JPRB, 0.10409E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.12031E+01_JPRB, 0.14069E+01_JPRB, 0.15909E+01_JPRB, 0.20759E+01_JPRB, 0.23587E+01_JPRB, 0.23801E+01_JPRB ,&
&  0.20323E+01_JPRB, 0.20575E+01_JPRB, 0.19763E+01_JPRB, 0.18503E+01_JPRB, 0.17394E+01_JPRB, 0.15523E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.12703E+01_JPRB, 0.12201E+01_JPRB, 0.11248E+01_JPRB, 0.10567E+01_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.89563E+00_JPRB, 0.87966E+00_JPRB, 0.88628E+00_JPRB, 0.89167E+00_JPRB, 0.88590E+00_JPRB, 0.86501E+00_JPRB, 0.86309E+00_JPRB ,&
&  0.84383E+00_JPRB, 0.81934E+00_JPRB, 0.79831E+00_JPRB, 0.78987E+00_JPRB, 0.78726E+00_JPRB, 0.78884E+00_JPRB, 0.80137E+00_JPRB ,&
&  0.82435E+00_JPRB, 0.11420E+01_JPRB, 0.91792E+00_JPRB, 0.79034E+00_JPRB, 0.67758E+00_JPRB, 0.76450E+00_JPRB, 0.89974E+00_JPRB ,&
&  0.11144E+01_JPRB, 0.12148E+01_JPRB, 0.13932E+01_JPRB, 0.16182E+01_JPRB, 0.18336E+01_JPRB, 0.17979E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.18439E+01_JPRB, 0.17410E+01_JPRB, 0.14707E+01_JPRB, 0.16125E+01_JPRB, 0.15757E+01_JPRB, 0.14065E+01_JPRB, 0.16483E+01_JPRB ,&
&  0.17182E+01_JPRB, 0.17723E+01_JPRB, 0.18143E+01_JPRB, 0.18390E+01_JPRB, 0.18445E+01_JPRB, 0.17984E+01_JPRB, 0.17348E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.16340E+01_JPRB, 0.17040E+01_JPRB, 0.18470E+01_JPRB, 0.23327E+01_JPRB, 0.30342E+01_JPRB, 0.43888E+01_JPRB ,&
&  0.57274E+01_JPRB, 0.55867E+01_JPRB, 0.58184E+01_JPRB, 0.79430E+01_JPRB, 0.10187E+02_JPRB, 0.98159E+01_JPRB, 0.94766E+01_JPRB ,&
&  0.99181E+01_JPRB, 0.99588E+01_JPRB, 0.96939E+01_JPRB, 0.90887E+01_JPRB, 0.84921E+01_JPRB, 0.78211E+01_JPRB, 0.70632E+01_JPRB ,&
&  0.63876E+01_JPRB, 0.61687E+01_JPRB, 0.60017E+01_JPRB, 0.53503E+01_JPRB, 0.43106E+01_JPRB, 0.37833E+01_JPRB, 0.33689E+01_JPRB ,&
&  0.29645E+01_JPRB, 0.17844E+01_JPRB, 0.10259E+01_JPRB, 0.78083E+00_JPRB, 0.69408E+00_JPRB, 0.66563E+00_JPRB, 0.75329E+00_JPRB ,&
&  0.74542E+00_JPRB, 0.90170E+00_JPRB, 0.11876E+01_JPRB, 0.12817E+01_JPRB, 0.17405E+01_JPRB, 0.19799E+01_JPRB, 0.18457E+01_JPRB ,&
&  0.15675E+01_JPRB, 0.13627E+01_JPRB, 0.17170E+01_JPRB, 0.16971E+01_JPRB, 0.15846E+01_JPRB, 0.14397E+01_JPRB, 0.13928E+01_JPRB ,&
&  0.12977E+01_JPRB, 0.12325E+01_JPRB, 0.11535E+01_JPRB, 0.10236E+01_JPRB, 0.93579E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.85680E+00_JPRB, 0.84946E+00_JPRB, 0.85221E+00_JPRB, 0.85585E+00_JPRB, 0.85957E+00_JPRB, 0.83302E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.82239E+00_JPRB, 0.80783E+00_JPRB, 0.79183E+00_JPRB, 0.78739E+00_JPRB, 0.78512E+00_JPRB, 0.78524E+00_JPRB, 0.80842E+00_JPRB ,&
&  0.93250E+00_JPRB, 0.12523E+01_JPRB, 0.11073E+01_JPRB, 0.86452E+00_JPRB, 0.99856E+00_JPRB, 0.97371E+00_JPRB, 0.10507E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.11391E+01_JPRB, 0.11210E+01_JPRB, 0.10979E+01_JPRB, 0.11725E+01_JPRB, 0.12698E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.12177E+01_JPRB, 0.10935E+01_JPRB, 0.10220E+01_JPRB, 0.10244E+01_JPRB, 0.11316E+01_JPRB, 0.10436E+01_JPRB, 0.11888E+01_JPRB ,&
&  0.12715E+01_JPRB, 0.13510E+01_JPRB, 0.14335E+01_JPRB, 0.15294E+01_JPRB, 0.16056E+01_JPRB, 0.16732E+01_JPRB, 0.17021E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.17127E+01_JPRB, 0.17070E+01_JPRB, 0.18109E+01_JPRB, 0.20429E+01_JPRB, 0.27151E+01_JPRB, 0.43429E+01_JPRB ,&
&  0.53058E+01_JPRB, 0.53119E+01_JPRB, 0.64464E+01_JPRB, 0.80008E+01_JPRB, 0.79256E+01_JPRB, 0.76279E+01_JPRB, 0.77996E+01_JPRB ,&
&  0.82094E+01_JPRB, 0.86348E+01_JPRB, 0.91467E+01_JPRB, 0.93074E+01_JPRB, 0.91881E+01_JPRB, 0.84451E+01_JPRB, 0.76577E+01_JPRB ,&
&  0.68941E+01_JPRB, 0.65977E+01_JPRB, 0.64002E+01_JPRB, 0.54846E+01_JPRB, 0.43999E+01_JPRB, 0.35766E+01_JPRB, 0.31658E+01_JPRB ,&
&  0.30449E+01_JPRB, 0.13991E+01_JPRB, 0.88371E+00_JPRB, 0.69908E+00_JPRB, 0.63050E+00_JPRB, 0.65910E+00_JPRB, 0.84435E+00_JPRB ,&
&  0.73443E+00_JPRB, 0.87654E+00_JPRB, 0.11456E+01_JPRB, 0.12171E+01_JPRB, 0.13040E+01_JPRB, 0.14326E+01_JPRB, 0.11184E+01_JPRB ,&
&  0.97657E+00_JPRB, 0.10751E+01_JPRB, 0.14298E+01_JPRB, 0.14972E+01_JPRB, 0.14045E+01_JPRB, 0.13582E+01_JPRB, 0.13502E+01_JPRB ,&
&  0.12503E+01_JPRB, 0.11808E+01_JPRB, 0.10553E+01_JPRB, 0.93935E+00_JPRB, 0.88725E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.87860E+00_JPRB, 0.85296E+00_JPRB, 0.84222E+00_JPRB, 0.83731E+00_JPRB, 0.82623E+00_JPRB, 0.80247E+00_JPRB, 0.80485E+00_JPRB ,&
&  0.79812E+00_JPRB, 0.78288E+00_JPRB, 0.77595E+00_JPRB, 0.77214E+00_JPRB, 0.76360E+00_JPRB, 0.77642E+00_JPRB, 0.83056E+00_JPRB ,&
&  0.77267E+00_JPRB, 0.92836E+00_JPRB, 0.80887E+00_JPRB, 0.85706E+00_JPRB, 0.12273E+01_JPRB, 0.10917E+01_JPRB, 0.10785E+01_JPRB ,&
&  0.10145E+01_JPRB, 0.98438E+00_JPRB, 0.90695E+00_JPRB, 0.87962E+00_JPRB, 0.88577E+00_JPRB, 0.94160E+00_JPRB, 0.96761E+00_JPRB ,&
&  0.87263E+00_JPRB, 0.77509E+00_JPRB, 0.74802E+00_JPRB, 0.74828E+00_JPRB, 0.80339E+00_JPRB, 0.89460E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.10700E+01_JPRB, 0.11117E+01_JPRB, 0.11804E+01_JPRB, 0.12601E+01_JPRB, 0.13368E+01_JPRB, 0.14170E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.15258E+01_JPRB, 0.15876E+01_JPRB, 0.17362E+01_JPRB, 0.18128E+01_JPRB, 0.25773E+01_JPRB, 0.44112E+01_JPRB ,&
&  0.53416E+01_JPRB, 0.46869E+01_JPRB, 0.50660E+01_JPRB, 0.64836E+01_JPRB, 0.66732E+01_JPRB, 0.63813E+01_JPRB, 0.61606E+01_JPRB ,&
&  0.63994E+01_JPRB, 0.67122E+01_JPRB, 0.71066E+01_JPRB, 0.76905E+01_JPRB, 0.80649E+01_JPRB, 0.82880E+01_JPRB, 0.79456E+01_JPRB ,&
&  0.73762E+01_JPRB, 0.69962E+01_JPRB, 0.66192E+01_JPRB, 0.59915E+01_JPRB, 0.48569E+01_JPRB, 0.41107E+01_JPRB, 0.31563E+01_JPRB ,&
&  0.25739E+01_JPRB, 0.18816E+01_JPRB, 0.11964E+01_JPRB, 0.93923E+00_JPRB, 0.81469E+00_JPRB, 0.90731E+00_JPRB, 0.12521E+01_JPRB ,&
&  0.90508E+00_JPRB, 0.95206E+00_JPRB, 0.96032E+00_JPRB, 0.97145E+00_JPRB, 0.96779E+00_JPRB, 0.91363E+00_JPRB, 0.77898E+00_JPRB ,&
&  0.78354E+00_JPRB, 0.96161E+00_JPRB, 0.12141E+01_JPRB, 0.13001E+01_JPRB, 0.12341E+01_JPRB, 0.12138E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.12064E+01_JPRB, 0.10949E+01_JPRB, 0.10047E+01_JPRB, 0.94240E+00_JPRB, 0.91998E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.86182E+00_JPRB, 0.84808E+00_JPRB, 0.83455E+00_JPRB, 0.81598E+00_JPRB, 0.79623E+00_JPRB, 0.77620E+00_JPRB, 0.76207E+00_JPRB ,&
&  0.76813E+00_JPRB, 0.76448E+00_JPRB, 0.76174E+00_JPRB, 0.75497E+00_JPRB, 0.74134E+00_JPRB, 0.75803E+00_JPRB, 0.70652E+00_JPRB ,&
&  0.66749E+00_JPRB, 0.72985E+00_JPRB, 0.72886E+00_JPRB, 0.79558E+00_JPRB, 0.88387E+00_JPRB, 0.85354E+00_JPRB, 0.84138E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.79015E+00_JPRB, 0.78052E+00_JPRB, 0.78926E+00_JPRB, 0.79426E+00_JPRB, 0.82033E+00_JPRB, 0.86496E+00_JPRB ,&
&  0.79992E+00_JPRB, 0.73481E+00_JPRB, 0.69691E+00_JPRB, 0.70585E+00_JPRB, 0.78385E+00_JPRB, 0.89685E+00_JPRB, 0.96308E+00_JPRB ,&
&  0.98398E+00_JPRB, 0.10054E+01_JPRB, 0.10664E+01_JPRB, 0.11124E+01_JPRB, 0.11488E+01_JPRB, 0.11924E+01_JPRB, 0.12337E+01_JPRB ,&
&  0.12712E+01_JPRB, 0.13309E+01_JPRB, 0.14085E+01_JPRB, 0.15385E+01_JPRB, 0.16353E+01_JPRB, 0.21186E+01_JPRB, 0.29177E+01_JPRB ,&
&  0.32435E+01_JPRB, 0.33224E+01_JPRB, 0.34361E+01_JPRB, 0.37945E+01_JPRB, 0.42819E+01_JPRB, 0.42616E+01_JPRB, 0.42989E+01_JPRB ,&
&  0.47253E+01_JPRB, 0.49141E+01_JPRB, 0.60827E+01_JPRB, 0.66960E+01_JPRB, 0.70559E+01_JPRB, 0.75696E+01_JPRB, 0.74846E+01_JPRB ,&
&  0.73473E+01_JPRB, 0.69925E+01_JPRB, 0.68091E+01_JPRB, 0.61300E+01_JPRB, 0.51075E+01_JPRB, 0.44205E+01_JPRB, 0.33745E+01_JPRB ,&
&  0.26824E+01_JPRB, 0.20797E+01_JPRB, 0.15047E+01_JPRB, 0.12611E+01_JPRB, 0.10299E+01_JPRB, 0.99552E+00_JPRB, 0.92515E+00_JPRB ,&
&  0.85201E+00_JPRB, 0.82151E+00_JPRB, 0.79280E+00_JPRB, 0.81031E+00_JPRB, 0.78489E+00_JPRB, 0.71654E+00_JPRB, 0.75246E+00_JPRB ,&
&  0.83196E+00_JPRB, 0.10894E+01_JPRB, 0.11153E+01_JPRB, 0.11715E+01_JPRB, 0.11268E+01_JPRB, 0.11031E+01_JPRB, 0.90619E+00_JPRB ,&
&  0.93113E+00_JPRB, 0.10315E+01_JPRB, 0.96227E+00_JPRB, 0.91374E+00_JPRB, 0.88244E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.83106E+00_JPRB, 0.83070E+00_JPRB, 0.80902E+00_JPRB, 0.79290E+00_JPRB, 0.78215E+00_JPRB, 0.74378E+00_JPRB, 0.70951E+00_JPRB ,&
&  0.76618E+00_JPRB, 0.75649E+00_JPRB, 0.75656E+00_JPRB, 0.74942E+00_JPRB, 0.73837E+00_JPRB, 0.67381E+00_JPRB, 0.62916E+00_JPRB ,&
&  0.59755E+00_JPRB, 0.60533E+00_JPRB, 0.65525E+00_JPRB, 0.67952E+00_JPRB, 0.68834E+00_JPRB, 0.67152E+00_JPRB, 0.66594E+00_JPRB ,&
&  0.66062E+00_JPRB, 0.67039E+00_JPRB, 0.71618E+00_JPRB, 0.75910E+00_JPRB, 0.76400E+00_JPRB, 0.77174E+00_JPRB, 0.81218E+00_JPRB ,&
&  0.77318E+00_JPRB, 0.77119E+00_JPRB, 0.77119E+00_JPRB, 0.71084E+00_JPRB, 0.82338E+00_JPRB, 0.86487E+00_JPRB, 0.88953E+00_JPRB ,&
&  0.93375E+00_JPRB, 0.97476E+00_JPRB, 0.10022E+01_JPRB, 0.10208E+01_JPRB, 0.10566E+01_JPRB, 0.10785E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.11449E+01_JPRB, 0.11900E+01_JPRB, 0.12460E+01_JPRB, 0.13260E+01_JPRB, 0.14978E+01_JPRB, 0.16966E+01_JPRB, 0.20270E+01_JPRB ,&
&  0.22566E+01_JPRB, 0.24098E+01_JPRB, 0.26635E+01_JPRB, 0.25689E+01_JPRB, 0.33925E+01_JPRB, 0.38723E+01_JPRB, 0.37117E+01_JPRB ,&
&  0.40005E+01_JPRB, 0.39744E+01_JPRB, 0.47186E+01_JPRB, 0.54052E+01_JPRB, 0.60420E+01_JPRB, 0.66014E+01_JPRB, 0.68154E+01_JPRB ,&
&  0.66729E+01_JPRB, 0.68122E+01_JPRB, 0.66480E+01_JPRB, 0.59127E+01_JPRB, 0.49491E+01_JPRB, 0.42760E+01_JPRB, 0.33806E+01_JPRB ,&
&  0.26436E+01_JPRB, 0.20600E+01_JPRB, 0.17019E+01_JPRB, 0.15172E+01_JPRB, 0.12038E+01_JPRB, 0.10998E+01_JPRB, 0.99912E+00_JPRB ,&
&  0.90607E+00_JPRB, 0.83108E+00_JPRB, 0.79211E+00_JPRB, 0.75688E+00_JPRB, 0.73263E+00_JPRB, 0.72400E+00_JPRB, 0.71303E+00_JPRB ,&
&  0.76230E+00_JPRB, 0.87549E+00_JPRB, 0.10843E+01_JPRB, 0.10712E+01_JPRB, 0.10317E+01_JPRB, 0.10011E+01_JPRB, 0.89538E+00_JPRB ,&
&  0.84726E+00_JPRB, 0.95827E+00_JPRB, 0.92662E+00_JPRB, 0.87333E+00_JPRB, 0.82884E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.79443E+00_JPRB, 0.79722E+00_JPRB, 0.78474E+00_JPRB, 0.77270E+00_JPRB, 0.75072E+00_JPRB, 0.68048E+00_JPRB, 0.66228E+00_JPRB ,&
&  0.73046E+00_JPRB, 0.67493E+00_JPRB, 0.72410E+00_JPRB, 0.61957E+00_JPRB, 0.59373E+00_JPRB, 0.56438E+00_JPRB, 0.54232E+00_JPRB ,&
&  0.51637E+00_JPRB, 0.52361E+00_JPRB, 0.55378E+00_JPRB, 0.58045E+00_JPRB, 0.59090E+00_JPRB, 0.58161E+00_JPRB, 0.58942E+00_JPRB ,&
&  0.60846E+00_JPRB, 0.64585E+00_JPRB, 0.70064E+00_JPRB, 0.73036E+00_JPRB, 0.75028E+00_JPRB, 0.77192E+00_JPRB, 0.78566E+00_JPRB ,&
&  0.73497E+00_JPRB, 0.72034E+00_JPRB, 0.71320E+00_JPRB, 0.72269E+00_JPRB, 0.76088E+00_JPRB, 0.82330E+00_JPRB, 0.86014E+00_JPRB ,&
&  0.87062E+00_JPRB, 0.74832E+00_JPRB, 0.86961E+00_JPRB, 0.90873E+00_JPRB, 0.97920E+00_JPRB, 0.10201E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.11278E+01_JPRB, 0.11742E+01_JPRB, 0.12281E+01_JPRB, 0.12960E+01_JPRB, 0.14238E+01_JPRB, 0.15250E+01_JPRB ,&
&  0.17783E+01_JPRB, 0.14006E+01_JPRB, 0.16072E+01_JPRB, 0.18255E+01_JPRB, 0.25425E+01_JPRB, 0.29793E+01_JPRB, 0.31475E+01_JPRB ,&
&  0.37274E+01_JPRB, 0.36688E+01_JPRB, 0.39875E+01_JPRB, 0.42762E+01_JPRB, 0.46896E+01_JPRB, 0.52163E+01_JPRB, 0.55522E+01_JPRB ,&
&  0.56434E+01_JPRB, 0.57138E+01_JPRB, 0.60049E+01_JPRB, 0.54561E+01_JPRB, 0.47282E+01_JPRB, 0.40250E+01_JPRB, 0.31863E+01_JPRB ,&
&  0.26291E+01_JPRB, 0.21372E+01_JPRB, 0.17213E+01_JPRB, 0.14222E+01_JPRB, 0.12109E+01_JPRB, 0.10798E+01_JPRB, 0.10069E+01_JPRB ,&
&  0.97462E+00_JPRB, 0.92505E+00_JPRB, 0.87435E+00_JPRB, 0.81407E+00_JPRB, 0.77767E+00_JPRB, 0.75780E+00_JPRB, 0.76466E+00_JPRB ,&
&  0.74681E+00_JPRB, 0.73528E+00_JPRB, 0.76566E+00_JPRB, 0.83778E+00_JPRB, 0.84135E+00_JPRB, 0.85172E+00_JPRB, 0.93513E+00_JPRB ,&
&  0.84376E+00_JPRB, 0.86123E+00_JPRB, 0.90974E+00_JPRB, 0.87004E+00_JPRB, 0.81268E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.80549E+00_JPRB, 0.77504E+00_JPRB, 0.75719E+00_JPRB, 0.74061E+00_JPRB, 0.71617E+00_JPRB, 0.66314E+00_JPRB, 0.62740E+00_JPRB ,&
&  0.63535E+00_JPRB, 0.61679E+00_JPRB, 0.60171E+00_JPRB, 0.56856E+00_JPRB, 0.54144E+00_JPRB, 0.51307E+00_JPRB, 0.49709E+00_JPRB ,&
&  0.48564E+00_JPRB, 0.50670E+00_JPRB, 0.54611E+00_JPRB, 0.55753E+00_JPRB, 0.55968E+00_JPRB, 0.56414E+00_JPRB, 0.58685E+00_JPRB ,&
&  0.61717E+00_JPRB, 0.64914E+00_JPRB, 0.66992E+00_JPRB, 0.69180E+00_JPRB, 0.70844E+00_JPRB, 0.72169E+00_JPRB, 0.70648E+00_JPRB ,&
&  0.68621E+00_JPRB, 0.65268E+00_JPRB, 0.62363E+00_JPRB, 0.64963E+00_JPRB, 0.71168E+00_JPRB, 0.76889E+00_JPRB, 0.73560E+00_JPRB ,&
&  0.55906E+00_JPRB, 0.56316E+00_JPRB, 0.74885E+00_JPRB, 0.81826E+00_JPRB, 0.88230E+00_JPRB, 0.92052E+00_JPRB, 0.96986E+00_JPRB ,&
&  0.10160E+01_JPRB, 0.10630E+01_JPRB, 0.11013E+01_JPRB, 0.11390E+01_JPRB, 0.11894E+01_JPRB, 0.12629E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.14562E+01_JPRB, 0.12283E+01_JPRB, 0.12249E+01_JPRB, 0.14285E+01_JPRB, 0.20627E+01_JPRB, 0.24077E+01_JPRB, 0.24808E+01_JPRB ,&
&  0.27103E+01_JPRB, 0.28362E+01_JPRB, 0.31033E+01_JPRB, 0.34411E+01_JPRB, 0.36097E+01_JPRB, 0.38908E+01_JPRB, 0.42807E+01_JPRB ,&
&  0.48117E+01_JPRB, 0.53006E+01_JPRB, 0.53237E+01_JPRB, 0.51082E+01_JPRB, 0.47085E+01_JPRB, 0.42478E+01_JPRB, 0.35490E+01_JPRB ,&
&  0.27440E+01_JPRB, 0.21758E+01_JPRB, 0.17086E+01_JPRB, 0.13939E+01_JPRB, 0.12173E+01_JPRB, 0.11361E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.10672E+01_JPRB, 0.10464E+01_JPRB, 0.10020E+01_JPRB, 0.91290E+00_JPRB, 0.84283E+00_JPRB, 0.81250E+00_JPRB, 0.79092E+00_JPRB ,&
&  0.75700E+00_JPRB, 0.72658E+00_JPRB, 0.65935E+00_JPRB, 0.66168E+00_JPRB, 0.72282E+00_JPRB, 0.79233E+00_JPRB, 0.81675E+00_JPRB ,&
&  0.83274E+00_JPRB, 0.81037E+00_JPRB, 0.80875E+00_JPRB, 0.83032E+00_JPRB, 0.82214E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.80992E+00_JPRB, 0.74329E+00_JPRB, 0.70462E+00_JPRB, 0.70024E+00_JPRB, 0.65252E+00_JPRB, 0.61257E+00_JPRB, 0.58881E+00_JPRB ,&
&  0.58854E+00_JPRB, 0.58916E+00_JPRB, 0.59053E+00_JPRB, 0.56613E+00_JPRB, 0.54417E+00_JPRB, 0.53218E+00_JPRB, 0.52902E+00_JPRB ,&
&  0.54343E+00_JPRB, 0.56132E+00_JPRB, 0.57686E+00_JPRB, 0.57588E+00_JPRB, 0.57675E+00_JPRB, 0.58710E+00_JPRB, 0.60997E+00_JPRB ,&
&  0.64099E+00_JPRB, 0.65783E+00_JPRB, 0.65580E+00_JPRB, 0.65336E+00_JPRB, 0.66839E+00_JPRB, 0.68106E+00_JPRB, 0.64142E+00_JPRB ,&
&  0.60148E+00_JPRB, 0.56897E+00_JPRB, 0.55872E+00_JPRB, 0.59956E+00_JPRB, 0.66836E+00_JPRB, 0.69315E+00_JPRB, 0.56729E+00_JPRB ,&
&  0.47705E+00_JPRB, 0.45881E+00_JPRB, 0.50411E+00_JPRB, 0.64732E+00_JPRB, 0.70575E+00_JPRB, 0.76087E+00_JPRB, 0.83340E+00_JPRB ,&
&  0.78246E+00_JPRB, 0.78772E+00_JPRB, 0.82035E+00_JPRB, 0.98064E+00_JPRB, 0.10460E+01_JPRB, 0.11004E+01_JPRB, 0.11500E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.13071E+01_JPRB, 0.15294E+01_JPRB, 0.13989E+01_JPRB, 0.16146E+01_JPRB, 0.20291E+01_JPRB, 0.21045E+01_JPRB ,&
&  0.24031E+01_JPRB, 0.26254E+01_JPRB, 0.31164E+01_JPRB, 0.31870E+01_JPRB, 0.30751E+01_JPRB, 0.31344E+01_JPRB, 0.34791E+01_JPRB ,&
&  0.41908E+01_JPRB, 0.47306E+01_JPRB, 0.45722E+01_JPRB, 0.43167E+01_JPRB, 0.41082E+01_JPRB, 0.38474E+01_JPRB, 0.33476E+01_JPRB ,&
&  0.26745E+01_JPRB, 0.21655E+01_JPRB, 0.17842E+01_JPRB, 0.14693E+01_JPRB, 0.13259E+01_JPRB, 0.12696E+01_JPRB, 0.12004E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.11466E+01_JPRB, 0.11126E+01_JPRB, 0.10038E+01_JPRB, 0.90041E+00_JPRB, 0.80055E+00_JPRB, 0.72740E+00_JPRB ,&
&  0.68530E+00_JPRB, 0.64340E+00_JPRB, 0.61904E+00_JPRB, 0.63440E+00_JPRB, 0.70207E+00_JPRB, 0.78207E+00_JPRB, 0.81980E+00_JPRB ,&
&  0.80360E+00_JPRB, 0.77578E+00_JPRB, 0.79814E+00_JPRB, 0.84224E+00_JPRB, 0.85359E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.88188E+00_JPRB, 0.85990E+00_JPRB, 0.82308E+00_JPRB, 0.77459E+00_JPRB, 0.73085E+00_JPRB, 0.68407E+00_JPRB, 0.64928E+00_JPRB ,&
&  0.63008E+00_JPRB, 0.62011E+00_JPRB, 0.61327E+00_JPRB, 0.61136E+00_JPRB, 0.61234E+00_JPRB, 0.61827E+00_JPRB, 0.61934E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.62801E+00_JPRB, 0.63583E+00_JPRB, 0.64872E+00_JPRB, 0.66189E+00_JPRB, 0.67612E+00_JPRB, 0.69076E+00_JPRB ,&
&  0.70364E+00_JPRB, 0.69418E+00_JPRB, 0.66461E+00_JPRB, 0.64369E+00_JPRB, 0.64010E+00_JPRB, 0.64452E+00_JPRB, 0.61599E+00_JPRB ,&
&  0.56749E+00_JPRB, 0.53850E+00_JPRB, 0.52988E+00_JPRB, 0.58653E+00_JPRB, 0.62352E+00_JPRB, 0.63549E+00_JPRB, 0.51275E+00_JPRB ,&
&  0.44423E+00_JPRB, 0.41409E+00_JPRB, 0.41481E+00_JPRB, 0.44802E+00_JPRB, 0.51658E+00_JPRB, 0.60765E+00_JPRB, 0.68921E+00_JPRB ,&
&  0.73962E+00_JPRB, 0.79016E+00_JPRB, 0.80938E+00_JPRB, 0.84111E+00_JPRB, 0.89471E+00_JPRB, 0.95308E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.10793E+01_JPRB, 0.11664E+01_JPRB, 0.13383E+01_JPRB, 0.12765E+01_JPRB, 0.13529E+01_JPRB, 0.14923E+01_JPRB, 0.17358E+01_JPRB ,&
&  0.22460E+01_JPRB, 0.25287E+01_JPRB, 0.27341E+01_JPRB, 0.30780E+01_JPRB, 0.30667E+01_JPRB, 0.30972E+01_JPRB, 0.33049E+01_JPRB ,&
&  0.38768E+01_JPRB, 0.43483E+01_JPRB, 0.42210E+01_JPRB, 0.37806E+01_JPRB, 0.36054E+01_JPRB, 0.34964E+01_JPRB, 0.30526E+01_JPRB ,&
&  0.24946E+01_JPRB, 0.21689E+01_JPRB, 0.18906E+01_JPRB, 0.15989E+01_JPRB, 0.14788E+01_JPRB, 0.14091E+01_JPRB, 0.13590E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.12702E+01_JPRB, 0.12312E+01_JPRB, 0.11244E+01_JPRB, 0.10226E+01_JPRB, 0.91022E+00_JPRB, 0.82874E+00_JPRB ,&
&  0.77366E+00_JPRB, 0.74917E+00_JPRB, 0.74798E+00_JPRB, 0.76412E+00_JPRB, 0.80947E+00_JPRB, 0.84345E+00_JPRB, 0.86016E+00_JPRB ,&
&  0.85903E+00_JPRB, 0.83264E+00_JPRB, 0.82233E+00_JPRB, 0.83033E+00_JPRB, 0.86957E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.95828E+00_JPRB, 0.95630E+00_JPRB, 0.93066E+00_JPRB, 0.89114E+00_JPRB, 0.84640E+00_JPRB, 0.79738E+00_JPRB, 0.76593E+00_JPRB ,&
&  0.74912E+00_JPRB, 0.74406E+00_JPRB, 0.73581E+00_JPRB, 0.72728E+00_JPRB, 0.72261E+00_JPRB, 0.72063E+00_JPRB, 0.72145E+00_JPRB ,&
&  0.72571E+00_JPRB, 0.73197E+00_JPRB, 0.73713E+00_JPRB, 0.73062E+00_JPRB, 0.73462E+00_JPRB, 0.74096E+00_JPRB, 0.74855E+00_JPRB ,&
&  0.74370E+00_JPRB, 0.71322E+00_JPRB, 0.67442E+00_JPRB, 0.64825E+00_JPRB, 0.61763E+00_JPRB, 0.59891E+00_JPRB, 0.57714E+00_JPRB ,&
&  0.54657E+00_JPRB, 0.53458E+00_JPRB, 0.55217E+00_JPRB, 0.57852E+00_JPRB, 0.56897E+00_JPRB, 0.58369E+00_JPRB, 0.46189E+00_JPRB ,&
&  0.41876E+00_JPRB, 0.39442E+00_JPRB, 0.38890E+00_JPRB, 0.40143E+00_JPRB, 0.43757E+00_JPRB, 0.48592E+00_JPRB, 0.54337E+00_JPRB ,&
&  0.62328E+00_JPRB, 0.71197E+00_JPRB, 0.76875E+00_JPRB, 0.81366E+00_JPRB, 0.86218E+00_JPRB, 0.90936E+00_JPRB, 0.94635E+00_JPRB ,&
&  0.98155E+00_JPRB, 0.10458E+01_JPRB, 0.11397E+01_JPRB, 0.12710E+01_JPRB, 0.15186E+01_JPRB, 0.15586E+01_JPRB, 0.17017E+01_JPRB ,&
&  0.23869E+01_JPRB, 0.27872E+01_JPRB, 0.27884E+01_JPRB, 0.28034E+01_JPRB, 0.28895E+01_JPRB, 0.31071E+01_JPRB, 0.34119E+01_JPRB ,&
&  0.38131E+01_JPRB, 0.41150E+01_JPRB, 0.40755E+01_JPRB, 0.36309E+01_JPRB, 0.34053E+01_JPRB, 0.32718E+01_JPRB, 0.29112E+01_JPRB ,&
&  0.25030E+01_JPRB, 0.24947E+01_JPRB, 0.25772E+01_JPRB, 0.21215E+01_JPRB, 0.18212E+01_JPRB, 0.16398E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.13887E+01_JPRB, 0.13155E+01_JPRB, 0.12505E+01_JPRB, 0.11816E+01_JPRB, 0.11148E+01_JPRB, 0.10552E+01_JPRB ,&
&  0.10076E+01_JPRB, 0.96674E+00_JPRB, 0.93933E+00_JPRB, 0.91775E+00_JPRB, 0.89260E+00_JPRB, 0.87308E+00_JPRB, 0.86796E+00_JPRB ,&
&  0.87257E+00_JPRB, 0.87204E+00_JPRB, 0.88501E+00_JPRB, 0.90537E+00_JPRB, 0.93473E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.97649E+00_JPRB, 0.97948E+00_JPRB, 0.97611E+00_JPRB, 0.96214E+00_JPRB, 0.93781E+00_JPRB, 0.91156E+00_JPRB, 0.88955E+00_JPRB ,&
&  0.87213E+00_JPRB, 0.85651E+00_JPRB, 0.83991E+00_JPRB, 0.82662E+00_JPRB, 0.81390E+00_JPRB, 0.80487E+00_JPRB, 0.79791E+00_JPRB ,&
&  0.79466E+00_JPRB, 0.77007E+00_JPRB, 0.76339E+00_JPRB, 0.76568E+00_JPRB, 0.76995E+00_JPRB, 0.76824E+00_JPRB, 0.76313E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.69888E+00_JPRB, 0.66388E+00_JPRB, 0.63536E+00_JPRB, 0.60997E+00_JPRB, 0.59126E+00_JPRB, 0.58854E+00_JPRB ,&
&  0.58693E+00_JPRB, 0.57470E+00_JPRB, 0.56964E+00_JPRB, 0.58354E+00_JPRB, 0.58389E+00_JPRB, 0.51523E+00_JPRB, 0.42506E+00_JPRB ,&
&  0.40908E+00_JPRB, 0.39019E+00_JPRB, 0.39040E+00_JPRB, 0.39694E+00_JPRB, 0.41581E+00_JPRB, 0.45006E+00_JPRB, 0.50654E+00_JPRB ,&
&  0.58118E+00_JPRB, 0.73290E+00_JPRB, 0.82463E+00_JPRB, 0.88769E+00_JPRB, 0.94260E+00_JPRB, 0.98440E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10622E+01_JPRB, 0.11255E+01_JPRB, 0.12057E+01_JPRB, 0.13281E+01_JPRB, 0.15592E+01_JPRB, 0.18009E+01_JPRB ,&
&  0.21329E+01_JPRB, 0.23976E+01_JPRB, 0.25267E+01_JPRB, 0.25992E+01_JPRB, 0.27886E+01_JPRB, 0.30400E+01_JPRB, 0.31712E+01_JPRB ,&
&  0.33707E+01_JPRB, 0.35774E+01_JPRB, 0.35344E+01_JPRB, 0.33328E+01_JPRB, 0.32038E+01_JPRB, 0.31188E+01_JPRB, 0.28543E+01_JPRB ,&
&  0.25542E+01_JPRB, 0.24235E+01_JPRB, 0.24086E+01_JPRB, 0.21299E+01_JPRB, 0.19249E+01_JPRB, 0.17319E+01_JPRB, 0.16001E+01_JPRB ,&
&  0.15042E+01_JPRB, 0.14261E+01_JPRB, 0.13296E+01_JPRB, 0.12522E+01_JPRB, 0.12044E+01_JPRB, 0.11663E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.10899E+01_JPRB, 0.10489E+01_JPRB, 0.10074E+01_JPRB, 0.97153E+00_JPRB, 0.93458E+00_JPRB, 0.91023E+00_JPRB, 0.89839E+00_JPRB ,&
&  0.89501E+00_JPRB, 0.90373E+00_JPRB, 0.91887E+00_JPRB, 0.94195E+00_JPRB, 0.96595E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.10064E+01_JPRB, 0.10047E+01_JPRB, 0.99587E+00_JPRB, 0.97806E+00_JPRB, 0.95270E+00_JPRB, 0.92498E+00_JPRB, 0.90149E+00_JPRB ,&
&  0.88105E+00_JPRB, 0.86476E+00_JPRB, 0.84775E+00_JPRB, 0.83382E+00_JPRB, 0.81758E+00_JPRB, 0.80692E+00_JPRB, 0.79454E+00_JPRB ,&
&  0.78716E+00_JPRB, 0.78356E+00_JPRB, 0.77384E+00_JPRB, 0.76198E+00_JPRB, 0.74193E+00_JPRB, 0.72429E+00_JPRB, 0.71054E+00_JPRB ,&
&  0.69686E+00_JPRB, 0.68543E+00_JPRB, 0.65862E+00_JPRB, 0.64045E+00_JPRB, 0.62963E+00_JPRB, 0.61385E+00_JPRB, 0.61765E+00_JPRB ,&
&  0.60698E+00_JPRB, 0.60701E+00_JPRB, 0.60777E+00_JPRB, 0.59799E+00_JPRB, 0.58502E+00_JPRB, 0.46062E+00_JPRB, 0.43048E+00_JPRB ,&
&  0.42113E+00_JPRB, 0.41301E+00_JPRB, 0.41206E+00_JPRB, 0.42020E+00_JPRB, 0.43559E+00_JPRB, 0.47717E+00_JPRB, 0.51815E+00_JPRB ,&
&  0.55367E+00_JPRB, 0.76279E+00_JPRB, 0.84915E+00_JPRB, 0.92923E+00_JPRB, 0.99386E+00_JPRB, 0.10391E+01_JPRB, 0.10667E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.11476E+01_JPRB, 0.11878E+01_JPRB, 0.12626E+01_JPRB, 0.13907E+01_JPRB, 0.15800E+01_JPRB, 0.17835E+01_JPRB ,&
&  0.20043E+01_JPRB, 0.21938E+01_JPRB, 0.23827E+01_JPRB, 0.25460E+01_JPRB, 0.27039E+01_JPRB, 0.28102E+01_JPRB, 0.28666E+01_JPRB ,&
&  0.29317E+01_JPRB, 0.29156E+01_JPRB, 0.29661E+01_JPRB, 0.29185E+01_JPRB, 0.28693E+01_JPRB, 0.27969E+01_JPRB, 0.26107E+01_JPRB ,&
&  0.23865E+01_JPRB, 0.22336E+01_JPRB, 0.20823E+01_JPRB, 0.18945E+01_JPRB, 0.17500E+01_JPRB, 0.16430E+01_JPRB, 0.15861E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.14946E+01_JPRB, 0.14135E+01_JPRB, 0.13274E+01_JPRB, 0.12686E+01_JPRB, 0.12237E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.10617E+01_JPRB, 0.10228E+01_JPRB, 0.98766E+00_JPRB, 0.95837E+00_JPRB, 0.94012E+00_JPRB, 0.92968E+00_JPRB ,&
&  0.92871E+00_JPRB, 0.93460E+00_JPRB, 0.95028E+00_JPRB, 0.97388E+00_JPRB, 0.99903E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.10225E+01_JPRB, 0.10062E+01_JPRB, 0.97246E+00_JPRB, 0.92783E+00_JPRB, 0.89142E+00_JPRB, 0.86268E+00_JPRB, 0.84792E+00_JPRB ,&
&  0.83852E+00_JPRB, 0.83465E+00_JPRB, 0.82852E+00_JPRB, 0.82211E+00_JPRB, 0.81092E+00_JPRB, 0.79878E+00_JPRB, 0.78474E+00_JPRB ,&
&  0.76609E+00_JPRB, 0.74636E+00_JPRB, 0.72853E+00_JPRB, 0.71663E+00_JPRB, 0.70678E+00_JPRB, 0.70007E+00_JPRB, 0.69333E+00_JPRB ,&
&  0.68650E+00_JPRB, 0.67796E+00_JPRB, 0.65048E+00_JPRB, 0.64057E+00_JPRB, 0.62871E+00_JPRB, 0.62775E+00_JPRB, 0.61921E+00_JPRB ,&
&  0.62877E+00_JPRB, 0.55002E+00_JPRB, 0.54338E+00_JPRB, 0.52233E+00_JPRB, 0.50366E+00_JPRB, 0.48341E+00_JPRB, 0.46689E+00_JPRB ,&
&  0.45929E+00_JPRB, 0.45024E+00_JPRB, 0.44718E+00_JPRB, 0.45746E+00_JPRB, 0.47089E+00_JPRB, 0.49927E+00_JPRB, 0.53193E+00_JPRB ,&
&  0.58857E+00_JPRB, 0.78298E+00_JPRB, 0.88314E+00_JPRB, 0.96422E+00_JPRB, 0.10481E+01_JPRB, 0.11210E+01_JPRB, 0.11682E+01_JPRB ,&
&  0.12275E+01_JPRB, 0.12961E+01_JPRB, 0.13698E+01_JPRB, 0.14229E+01_JPRB, 0.15944E+01_JPRB, 0.17222E+01_JPRB, 0.18726E+01_JPRB ,&
&  0.20178E+01_JPRB, 0.21698E+01_JPRB, 0.23084E+01_JPRB, 0.24193E+01_JPRB, 0.24937E+01_JPRB, 0.25128E+01_JPRB, 0.24834E+01_JPRB ,&
&  0.24729E+01_JPRB, 0.24545E+01_JPRB, 0.24560E+01_JPRB, 0.23838E+01_JPRB, 0.23495E+01_JPRB, 0.23135E+01_JPRB, 0.22225E+01_JPRB ,&
&  0.21086E+01_JPRB, 0.20043E+01_JPRB, 0.18932E+01_JPRB, 0.17761E+01_JPRB, 0.16626E+01_JPRB, 0.15898E+01_JPRB, 0.15440E+01_JPRB ,&
&  0.15174E+01_JPRB, 0.14759E+01_JPRB, 0.14219E+01_JPRB, 0.13476E+01_JPRB, 0.12772E+01_JPRB, 0.12250E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.10711E+01_JPRB, 0.10355E+01_JPRB, 0.10060E+01_JPRB, 0.98775E+00_JPRB, 0.97668E+00_JPRB, 0.97274E+00_JPRB ,&
&  0.97015E+00_JPRB, 0.97406E+00_JPRB, 0.98642E+00_JPRB, 0.10033E+01_JPRB, 0.10174E+01_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.95884E+00_JPRB, 0.92978E+00_JPRB, 0.89407E+00_JPRB, 0.87171E+00_JPRB, 0.86136E+00_JPRB, 0.84953E+00_JPRB, 0.83935E+00_JPRB ,&
&  0.83138E+00_JPRB, 0.83133E+00_JPRB, 0.83294E+00_JPRB, 0.83195E+00_JPRB, 0.82657E+00_JPRB, 0.81847E+00_JPRB, 0.80713E+00_JPRB ,&
&  0.79286E+00_JPRB, 0.77656E+00_JPRB, 0.76193E+00_JPRB, 0.74827E+00_JPRB, 0.73361E+00_JPRB, 0.71979E+00_JPRB, 0.69982E+00_JPRB ,&
&  0.68221E+00_JPRB, 0.66613E+00_JPRB, 0.61515E+00_JPRB, 0.61231E+00_JPRB, 0.60161E+00_JPRB, 0.59869E+00_JPRB, 0.60093E+00_JPRB ,&
&  0.59294E+00_JPRB, 0.57636E+00_JPRB, 0.56013E+00_JPRB, 0.54024E+00_JPRB, 0.52461E+00_JPRB, 0.50645E+00_JPRB, 0.49826E+00_JPRB ,&
&  0.49546E+00_JPRB, 0.50028E+00_JPRB, 0.51097E+00_JPRB, 0.52689E+00_JPRB, 0.54611E+00_JPRB, 0.56699E+00_JPRB, 0.59906E+00_JPRB ,&
&  0.67348E+00_JPRB, 0.77433E+00_JPRB, 0.90231E+00_JPRB, 0.97957E+00_JPRB, 0.10700E+01_JPRB, 0.11569E+01_JPRB, 0.12265E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.13850E+01_JPRB, 0.14751E+01_JPRB, 0.15678E+01_JPRB, 0.16666E+01_JPRB, 0.17566E+01_JPRB, 0.18595E+01_JPRB ,&
&  0.19483E+01_JPRB, 0.20272E+01_JPRB, 0.20767E+01_JPRB, 0.21023E+01_JPRB, 0.21103E+01_JPRB, 0.20937E+01_JPRB, 0.20738E+01_JPRB ,&
&  0.20618E+01_JPRB, 0.20450E+01_JPRB, 0.20243E+01_JPRB, 0.19676E+01_JPRB, 0.19474E+01_JPRB, 0.19413E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.18617E+01_JPRB, 0.18051E+01_JPRB, 0.17475E+01_JPRB, 0.16685E+01_JPRB, 0.15949E+01_JPRB, 0.15651E+01_JPRB, 0.15345E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.14717E+01_JPRB, 0.14378E+01_JPRB, 0.13916E+01_JPRB, 0.13288E+01_JPRB, 0.12679E+01_JPRB, 0.12156E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.11341E+01_JPRB, 0.10994E+01_JPRB, 0.10630E+01_JPRB, 0.10387E+01_JPRB, 0.10162E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.99832E+00_JPRB, 0.99775E+00_JPRB, 0.99553E+00_JPRB, 0.99198E+00_JPRB, 0.97925E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.94317E+00_JPRB, 0.91365E+00_JPRB, 0.89637E+00_JPRB, 0.88910E+00_JPRB, 0.88230E+00_JPRB, 0.87549E+00_JPRB, 0.86956E+00_JPRB ,&
&  0.85895E+00_JPRB, 0.84933E+00_JPRB, 0.84703E+00_JPRB, 0.84673E+00_JPRB, 0.84365E+00_JPRB, 0.83942E+00_JPRB, 0.83324E+00_JPRB ,&
&  0.82599E+00_JPRB, 0.81612E+00_JPRB, 0.80537E+00_JPRB, 0.79179E+00_JPRB, 0.77741E+00_JPRB, 0.76140E+00_JPRB, 0.74113E+00_JPRB ,&
&  0.72511E+00_JPRB, 0.71075E+00_JPRB, 0.69288E+00_JPRB, 0.68573E+00_JPRB, 0.68282E+00_JPRB, 0.64409E+00_JPRB, 0.63488E+00_JPRB ,&
&  0.64107E+00_JPRB, 0.64127E+00_JPRB, 0.64274E+00_JPRB, 0.64556E+00_JPRB, 0.64840E+00_JPRB, 0.63471E+00_JPRB, 0.63045E+00_JPRB ,&
&  0.60320E+00_JPRB, 0.62314E+00_JPRB, 0.63709E+00_JPRB, 0.65160E+00_JPRB, 0.66800E+00_JPRB, 0.69411E+00_JPRB, 0.74344E+00_JPRB ,&
&  0.83675E+00_JPRB, 0.88392E+00_JPRB, 0.93593E+00_JPRB, 0.98916E+00_JPRB, 0.10525E+01_JPRB, 0.11171E+01_JPRB, 0.11753E+01_JPRB ,&
&  0.12347E+01_JPRB, 0.12996E+01_JPRB, 0.13718E+01_JPRB, 0.14537E+01_JPRB, 0.15267E+01_JPRB, 0.15924E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.16906E+01_JPRB, 0.17250E+01_JPRB, 0.17511E+01_JPRB, 0.17660E+01_JPRB, 0.17835E+01_JPRB, 0.17942E+01_JPRB, 0.18070E+01_JPRB ,&
&  0.18163E+01_JPRB, 0.18191E+01_JPRB, 0.18057E+01_JPRB, 0.17782E+01_JPRB, 0.17618E+01_JPRB, 0.17566E+01_JPRB, 0.17442E+01_JPRB ,&
&  0.17271E+01_JPRB, 0.17073E+01_JPRB, 0.16808E+01_JPRB, 0.16471E+01_JPRB, 0.16013E+01_JPRB, 0.15595E+01_JPRB, 0.15108E+01_JPRB ,&
&  0.14672E+01_JPRB, 0.14306E+01_JPRB, 0.13996E+01_JPRB, 0.13656E+01_JPRB, 0.13290E+01_JPRB, 0.12924E+01_JPRB, 0.12525E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.11876E+01_JPRB, 0.11638E+01_JPRB, 0.11394E+01_JPRB, 0.11221E+01_JPRB, 0.10997E+01_JPRB, 0.10814E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.10494E+01_JPRB, 0.10274E+01_JPRB, 0.10008E+01_JPRB, 0.96888E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.93360E+00_JPRB, 0.92062E+00_JPRB, 0.91385E+00_JPRB, 0.90822E+00_JPRB, 0.90128E+00_JPRB, 0.89216E+00_JPRB, 0.88267E+00_JPRB ,&
&  0.87493E+00_JPRB, 0.86510E+00_JPRB, 0.85630E+00_JPRB, 0.85215E+00_JPRB, 0.84606E+00_JPRB, 0.84144E+00_JPRB, 0.83782E+00_JPRB ,&
&  0.83535E+00_JPRB, 0.83216E+00_JPRB, 0.82950E+00_JPRB, 0.82554E+00_JPRB, 0.82013E+00_JPRB, 0.81322E+00_JPRB, 0.80719E+00_JPRB ,&
&  0.80385E+00_JPRB, 0.80144E+00_JPRB, 0.79795E+00_JPRB, 0.79502E+00_JPRB, 0.79511E+00_JPRB, 0.79441E+00_JPRB, 0.79243E+00_JPRB ,&
&  0.79548E+00_JPRB, 0.79831E+00_JPRB, 0.79892E+00_JPRB, 0.79747E+00_JPRB, 0.79465E+00_JPRB, 0.79470E+00_JPRB, 0.79902E+00_JPRB ,&
&  0.80502E+00_JPRB, 0.82231E+00_JPRB, 0.83734E+00_JPRB, 0.85168E+00_JPRB, 0.86582E+00_JPRB, 0.88287E+00_JPRB, 0.90348E+00_JPRB ,&
&  0.92595E+00_JPRB, 0.94702E+00_JPRB, 0.96962E+00_JPRB, 0.99743E+00_JPRB, 0.10272E+01_JPRB, 0.10586E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.11510E+01_JPRB, 0.11844E+01_JPRB, 0.12208E+01_JPRB, 0.12524E+01_JPRB, 0.12888E+01_JPRB, 0.13288E+01_JPRB ,&
&  0.13744E+01_JPRB, 0.14018E+01_JPRB, 0.14318E+01_JPRB, 0.14631E+01_JPRB, 0.14974E+01_JPRB, 0.15197E+01_JPRB, 0.15412E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.15673E+01_JPRB, 0.15626E+01_JPRB, 0.15557E+01_JPRB, 0.15521E+01_JPRB, 0.15446E+01_JPRB, 0.15339E+01_JPRB ,&
&  0.15252E+01_JPRB, 0.15201E+01_JPRB, 0.15076E+01_JPRB, 0.14908E+01_JPRB, 0.14692E+01_JPRB, 0.14470E+01_JPRB, 0.14104E+01_JPRB ,&
&  0.13754E+01_JPRB, 0.13479E+01_JPRB, 0.13271E+01_JPRB, 0.13036E+01_JPRB, 0.12810E+01_JPRB, 0.12597E+01_JPRB, 0.12389E+01_JPRB ,&
&  0.12190E+01_JPRB, 0.11978E+01_JPRB, 0.11764E+01_JPRB, 0.11541E+01_JPRB, 0.11358E+01_JPRB, 0.11113E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.10345E+01_JPRB, 0.10082E+01_JPRB, 0.97855E+00_JPRB, 0.95062E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.96347E+00_JPRB, 0.95667E+00_JPRB, 0.94933E+00_JPRB, 0.94156E+00_JPRB, 0.93279E+00_JPRB, 0.92338E+00_JPRB, 0.91292E+00_JPRB ,&
&  0.90250E+00_JPRB, 0.89271E+00_JPRB, 0.88387E+00_JPRB, 0.87548E+00_JPRB, 0.86776E+00_JPRB, 0.86157E+00_JPRB, 0.85867E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.85710E+00_JPRB, 0.85700E+00_JPRB, 0.85697E+00_JPRB, 0.85638E+00_JPRB, 0.85532E+00_JPRB, 0.85418E+00_JPRB ,&
&  0.85351E+00_JPRB, 0.85310E+00_JPRB, 0.85235E+00_JPRB, 0.85243E+00_JPRB, 0.85318E+00_JPRB, 0.85428E+00_JPRB, 0.85538E+00_JPRB ,&
&  0.85765E+00_JPRB, 0.86032E+00_JPRB, 0.86363E+00_JPRB, 0.86721E+00_JPRB, 0.87401E+00_JPRB, 0.88180E+00_JPRB, 0.89001E+00_JPRB ,&
&  0.89815E+00_JPRB, 0.90690E+00_JPRB, 0.91494E+00_JPRB, 0.92243E+00_JPRB, 0.92936E+00_JPRB, 0.93718E+00_JPRB, 0.94595E+00_JPRB ,&
&  0.95580E+00_JPRB, 0.96638E+00_JPRB, 0.97800E+00_JPRB, 0.98982E+00_JPRB, 0.10011E+01_JPRB, 0.10125E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.10479E+01_JPRB, 0.10594E+01_JPRB, 0.10708E+01_JPRB, 0.10820E+01_JPRB, 0.10946E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.11240E+01_JPRB, 0.11366E+01_JPRB, 0.11504E+01_JPRB, 0.11651E+01_JPRB, 0.11832E+01_JPRB, 0.11927E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.12233E+01_JPRB, 0.12304E+01_JPRB, 0.12383E+01_JPRB, 0.12459E+01_JPRB, 0.12522E+01_JPRB, 0.12555E+01_JPRB ,&
&  0.12584E+01_JPRB, 0.12608E+01_JPRB, 0.12601E+01_JPRB, 0.12568E+01_JPRB, 0.12521E+01_JPRB, 0.12466E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.12176E+01_JPRB, 0.12091E+01_JPRB, 0.11976E+01_JPRB, 0.11860E+01_JPRB, 0.11745E+01_JPRB, 0.11628E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.11343E+01_JPRB, 0.11193E+01_JPRB, 0.11036E+01_JPRB, 0.10877E+01_JPRB, 0.10716E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.10192E+01_JPRB, 0.10008E+01_JPRB, 0.98293E+00_JPRB, 0.96955E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB ,&
&  0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB, 0.99510E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB ,&
&  0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB, 0.13826E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.14650E+00_JPRB, 0.14649E+00_JPRB, 0.14640E+00_JPRB, 0.14609E+00_JPRB, 0.14548E+00_JPRB, 0.14534E+00_JPRB, 0.14515E+00_JPRB ,&
&  0.14488E+00_JPRB, 0.14428E+00_JPRB, 0.14427E+00_JPRB, 0.14420E+00_JPRB, 0.14410E+00_JPRB, 0.14400E+00_JPRB, 0.14378E+00_JPRB ,&
&  0.14335E+00_JPRB, 0.14269E+00_JPRB, 0.14069E+00_JPRB, 0.13974E+00_JPRB, 0.13913E+00_JPRB, 0.13881E+00_JPRB, 0.13855E+00_JPRB ,&
&  0.13840E+00_JPRB, 0.13833E+00_JPRB, 0.13825E+00_JPRB, 0.13864E+00_JPRB, 0.13882E+00_JPRB, 0.13889E+00_JPRB, 0.13893E+00_JPRB ,&
&  0.13893E+00_JPRB, 0.13902E+00_JPRB, 0.13902E+00_JPRB, 0.13902E+00_JPRB, 0.13913E+00_JPRB, 0.13913E+00_JPRB, 0.13896E+00_JPRB ,&
&  0.13876E+00_JPRB, 0.13852E+00_JPRB, 0.13828E+00_JPRB, 0.13799E+00_JPRB, 0.13778E+00_JPRB, 0.13761E+00_JPRB, 0.13754E+00_JPRB ,&
&  0.13739E+00_JPRB, 0.13728E+00_JPRB, 0.13715E+00_JPRB, 0.13709E+00_JPRB, 0.13699E+00_JPRB, 0.13694E+00_JPRB, 0.13683E+00_JPRB ,&
&  0.13690E+00_JPRB, 0.13691E+00_JPRB, 0.13695E+00_JPRB, 0.13696E+00_JPRB, 0.13697E+00_JPRB, 0.13692E+00_JPRB, 0.13684E+00_JPRB ,&
&  0.13667E+00_JPRB, 0.13657E+00_JPRB, 0.13645E+00_JPRB, 0.13637E+00_JPRB, 0.13633E+00_JPRB, 0.13631E+00_JPRB, 0.13630E+00_JPRB ,&
&  0.13630E+00_JPRB, 0.13638E+00_JPRB, 0.13641E+00_JPRB, 0.13645E+00_JPRB, 0.13651E+00_JPRB, 0.13657E+00_JPRB, 0.13662E+00_JPRB ,&
&  0.13670E+00_JPRB, 0.13677E+00_JPRB, 0.13686E+00_JPRB, 0.13699E+00_JPRB, 0.13714E+00_JPRB, 0.13735E+00_JPRB, 0.13761E+00_JPRB ,&
&  0.13788E+00_JPRB, 0.13817E+00_JPRB, 0.13844E+00_JPRB, 0.13872E+00_JPRB, 0.13915E+00_JPRB, 0.13958E+00_JPRB, 0.14008E+00_JPRB ,&
&  0.14058E+00_JPRB, 0.14108E+00_JPRB, 0.14153E+00_JPRB, 0.14201E+00_JPRB, 0.14262E+00_JPRB, 0.14293E+00_JPRB, 0.14320E+00_JPRB ,&
&  0.14361E+00_JPRB, 0.14403E+00_JPRB, 0.14445E+00_JPRB, 0.14483E+00_JPRB, 0.14543E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.15130E+00_JPRB, 0.15155E+00_JPRB, 0.15100E+00_JPRB, 0.15043E+00_JPRB, 0.14472E+00_JPRB, 0.15074E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15042E+00_JPRB, 0.15023E+00_JPRB, 0.15137E+00_JPRB, 0.15145E+00_JPRB, 0.15111E+00_JPRB, 0.15089E+00_JPRB, 0.15059E+00_JPRB ,&
&  0.15026E+00_JPRB, 0.14954E+00_JPRB, 0.14915E+00_JPRB, 0.14861E+00_JPRB, 0.14808E+00_JPRB, 0.14722E+00_JPRB, 0.14633E+00_JPRB ,&
&  0.14592E+00_JPRB, 0.14600E+00_JPRB, 0.14637E+00_JPRB, 0.14495E+00_JPRB, 0.14434E+00_JPRB, 0.14396E+00_JPRB, 0.14378E+00_JPRB ,&
&  0.14346E+00_JPRB, 0.14339E+00_JPRB, 0.14342E+00_JPRB, 0.14336E+00_JPRB, 0.14450E+00_JPRB, 0.14483E+00_JPRB, 0.14509E+00_JPRB ,&
&  0.14511E+00_JPRB, 0.14494E+00_JPRB, 0.14429E+00_JPRB, 0.14369E+00_JPRB, 0.14310E+00_JPRB, 0.14209E+00_JPRB, 0.14103E+00_JPRB ,&
&  0.14012E+00_JPRB, 0.13942E+00_JPRB, 0.13863E+00_JPRB, 0.13802E+00_JPRB, 0.13752E+00_JPRB, 0.13704E+00_JPRB, 0.13672E+00_JPRB ,&
&  0.13663E+00_JPRB, 0.13661E+00_JPRB, 0.13657E+00_JPRB, 0.13653E+00_JPRB, 0.13646E+00_JPRB, 0.13641E+00_JPRB, 0.13636E+00_JPRB ,&
&  0.13658E+00_JPRB, 0.13652E+00_JPRB, 0.13636E+00_JPRB, 0.13608E+00_JPRB, 0.13594E+00_JPRB, 0.13562E+00_JPRB, 0.13540E+00_JPRB ,&
&  0.13522E+00_JPRB, 0.13500E+00_JPRB, 0.13489E+00_JPRB, 0.13486E+00_JPRB, 0.13484E+00_JPRB, 0.13482E+00_JPRB, 0.13489E+00_JPRB ,&
&  0.13502E+00_JPRB, 0.13520E+00_JPRB, 0.13554E+00_JPRB, 0.13582E+00_JPRB, 0.13608E+00_JPRB, 0.13632E+00_JPRB, 0.13678E+00_JPRB ,&
&  0.13727E+00_JPRB, 0.13776E+00_JPRB, 0.13825E+00_JPRB, 0.13858E+00_JPRB, 0.13927E+00_JPRB, 0.13998E+00_JPRB, 0.14074E+00_JPRB ,&
&  0.14154E+00_JPRB, 0.14255E+00_JPRB, 0.14359E+00_JPRB, 0.14463E+00_JPRB, 0.14598E+00_JPRB, 0.14708E+00_JPRB, 0.14815E+00_JPRB ,&
&  0.14933E+00_JPRB, 0.15152E+00_JPRB, 0.15216E+00_JPRB, 0.15237E+00_JPRB, 0.15202E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.16637E+00_JPRB, 0.16710E+00_JPRB, 0.16747E+00_JPRB, 0.16787E+00_JPRB, 0.16886E+00_JPRB, 0.16887E+00_JPRB, 0.16870E+00_JPRB ,&
&  0.16673E+00_JPRB, 0.16595E+00_JPRB, 0.16470E+00_JPRB, 0.16297E+00_JPRB, 0.16121E+00_JPRB, 0.15835E+00_JPRB, 0.15651E+00_JPRB ,&
&  0.15469E+00_JPRB, 0.15278E+00_JPRB, 0.15150E+00_JPRB, 0.14987E+00_JPRB, 0.14848E+00_JPRB, 0.14693E+00_JPRB, 0.14640E+00_JPRB ,&
&  0.14564E+00_JPRB, 0.14559E+00_JPRB, 0.14634E+00_JPRB, 0.14697E+00_JPRB, 0.14812E+00_JPRB, 0.14934E+00_JPRB, 0.15091E+00_JPRB ,&
&  0.15137E+00_JPRB, 0.15217E+00_JPRB, 0.15292E+00_JPRB, 0.15340E+00_JPRB, 0.15501E+00_JPRB, 0.15616E+00_JPRB, 0.15655E+00_JPRB ,&
&  0.15609E+00_JPRB, 0.15527E+00_JPRB, 0.15364E+00_JPRB, 0.15180E+00_JPRB, 0.14967E+00_JPRB, 0.14732E+00_JPRB, 0.14508E+00_JPRB ,&
&  0.14334E+00_JPRB, 0.14172E+00_JPRB, 0.14049E+00_JPRB, 0.13949E+00_JPRB, 0.13887E+00_JPRB, 0.13837E+00_JPRB, 0.13788E+00_JPRB ,&
&  0.13759E+00_JPRB, 0.13735E+00_JPRB, 0.13700E+00_JPRB, 0.13696E+00_JPRB, 0.13685E+00_JPRB, 0.13688E+00_JPRB, 0.13691E+00_JPRB ,&
&  0.13696E+00_JPRB, 0.13690E+00_JPRB, 0.13677E+00_JPRB, 0.13647E+00_JPRB, 0.13616E+00_JPRB, 0.13583E+00_JPRB, 0.13550E+00_JPRB ,&
&  0.13513E+00_JPRB, 0.13507E+00_JPRB, 0.13478E+00_JPRB, 0.13457E+00_JPRB, 0.13430E+00_JPRB, 0.13388E+00_JPRB, 0.13375E+00_JPRB ,&
&  0.13382E+00_JPRB, 0.13395E+00_JPRB, 0.13384E+00_JPRB, 0.13398E+00_JPRB, 0.13418E+00_JPRB, 0.13448E+00_JPRB, 0.13524E+00_JPRB ,&
&  0.13571E+00_JPRB, 0.13621E+00_JPRB, 0.13668E+00_JPRB, 0.13733E+00_JPRB, 0.13844E+00_JPRB, 0.13955E+00_JPRB, 0.14078E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14365E+00_JPRB, 0.14504E+00_JPRB, 0.14641E+00_JPRB, 0.14866E+00_JPRB, 0.15029E+00_JPRB, 0.15243E+00_JPRB ,&
&  0.15431E+00_JPRB, 0.15347E+00_JPRB, 0.16132E+00_JPRB, 0.16217E+00_JPRB, 0.16346E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.17328E+00_JPRB, 0.17364E+00_JPRB, 0.17357E+00_JPRB, 0.17448E+00_JPRB, 0.17528E+00_JPRB, 0.17615E+00_JPRB, 0.17573E+00_JPRB ,&
&  0.17533E+00_JPRB, 0.17382E+00_JPRB, 0.17265E+00_JPRB, 0.16975E+00_JPRB, 0.16744E+00_JPRB, 0.16467E+00_JPRB, 0.16227E+00_JPRB ,&
&  0.16013E+00_JPRB, 0.15809E+00_JPRB, 0.15618E+00_JPRB, 0.15457E+00_JPRB, 0.15308E+00_JPRB, 0.15180E+00_JPRB, 0.15058E+00_JPRB ,&
&  0.14872E+00_JPRB, 0.14823E+00_JPRB, 0.14826E+00_JPRB, 0.14947E+00_JPRB, 0.15193E+00_JPRB, 0.15357E+00_JPRB, 0.15556E+00_JPRB ,&
&  0.15788E+00_JPRB, 0.15736E+00_JPRB, 0.16021E+00_JPRB, 0.16297E+00_JPRB, 0.16521E+00_JPRB, 0.16701E+00_JPRB, 0.16783E+00_JPRB ,&
&  0.16845E+00_JPRB, 0.16401E+00_JPRB, 0.16526E+00_JPRB, 0.16143E+00_JPRB, 0.15697E+00_JPRB, 0.15487E+00_JPRB, 0.15099E+00_JPRB ,&
&  0.14913E+00_JPRB, 0.14613E+00_JPRB, 0.14417E+00_JPRB, 0.14254E+00_JPRB, 0.14064E+00_JPRB, 0.14016E+00_JPRB, 0.13936E+00_JPRB ,&
&  0.13889E+00_JPRB, 0.13894E+00_JPRB, 0.13823E+00_JPRB, 0.13799E+00_JPRB, 0.13754E+00_JPRB, 0.13711E+00_JPRB, 0.13742E+00_JPRB ,&
&  0.13793E+00_JPRB, 0.13857E+00_JPRB, 0.13921E+00_JPRB, 0.13930E+00_JPRB, 0.13925E+00_JPRB, 0.13923E+00_JPRB, 0.13915E+00_JPRB ,&
&  0.13938E+00_JPRB, 0.13939E+00_JPRB, 0.13900E+00_JPRB, 0.13855E+00_JPRB, 0.13754E+00_JPRB, 0.13656E+00_JPRB, 0.13558E+00_JPRB ,&
&  0.13474E+00_JPRB, 0.13428E+00_JPRB, 0.13390E+00_JPRB, 0.13380E+00_JPRB, 0.13384E+00_JPRB, 0.13416E+00_JPRB, 0.13448E+00_JPRB ,&
&  0.13478E+00_JPRB, 0.13546E+00_JPRB, 0.13597E+00_JPRB, 0.13680E+00_JPRB, 0.13780E+00_JPRB, 0.13871E+00_JPRB, 0.14008E+00_JPRB ,&
&  0.14158E+00_JPRB, 0.14321E+00_JPRB, 0.14553E+00_JPRB, 0.14756E+00_JPRB, 0.15011E+00_JPRB, 0.15343E+00_JPRB, 0.15591E+00_JPRB ,&
&  0.15860E+00_JPRB, 0.16135E+00_JPRB, 0.16176E+00_JPRB, 0.16850E+00_JPRB, 0.17227E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.17239E+00_JPRB, 0.17381E+00_JPRB, 0.17526E+00_JPRB, 0.17672E+00_JPRB, 0.17799E+00_JPRB, 0.17924E+00_JPRB, 0.18006E+00_JPRB ,&
&  0.17772E+00_JPRB, 0.17633E+00_JPRB, 0.17377E+00_JPRB, 0.17143E+00_JPRB, 0.16874E+00_JPRB, 0.16579E+00_JPRB, 0.16476E+00_JPRB ,&
&  0.16330E+00_JPRB, 0.16299E+00_JPRB, 0.16180E+00_JPRB, 0.16128E+00_JPRB, 0.16030E+00_JPRB, 0.15974E+00_JPRB, 0.15879E+00_JPRB ,&
&  0.15688E+00_JPRB, 0.15589E+00_JPRB, 0.15495E+00_JPRB, 0.15533E+00_JPRB, 0.15677E+00_JPRB, 0.15834E+00_JPRB, 0.16000E+00_JPRB ,&
&  0.16237E+00_JPRB, 0.16349E+00_JPRB, 0.16578E+00_JPRB, 0.16748E+00_JPRB, 0.17026E+00_JPRB, 0.17260E+00_JPRB, 0.17308E+00_JPRB ,&
&  0.17313E+00_JPRB, 0.17155E+00_JPRB, 0.17113E+00_JPRB, 0.16920E+00_JPRB, 0.16343E+00_JPRB, 0.16011E+00_JPRB, 0.15545E+00_JPRB ,&
&  0.15027E+00_JPRB, 0.14638E+00_JPRB, 0.14371E+00_JPRB, 0.14282E+00_JPRB, 0.14195E+00_JPRB, 0.14166E+00_JPRB, 0.14142E+00_JPRB ,&
&  0.14067E+00_JPRB, 0.13981E+00_JPRB, 0.13880E+00_JPRB, 0.13822E+00_JPRB, 0.13854E+00_JPRB, 0.13924E+00_JPRB, 0.14073E+00_JPRB ,&
&  0.14223E+00_JPRB, 0.14308E+00_JPRB, 0.14401E+00_JPRB, 0.14432E+00_JPRB, 0.14445E+00_JPRB, 0.14467E+00_JPRB, 0.14472E+00_JPRB ,&
&  0.14533E+00_JPRB, 0.14548E+00_JPRB, 0.14500E+00_JPRB, 0.14419E+00_JPRB, 0.14216E+00_JPRB, 0.13948E+00_JPRB, 0.13736E+00_JPRB ,&
&  0.13556E+00_JPRB, 0.13440E+00_JPRB, 0.13399E+00_JPRB, 0.13358E+00_JPRB, 0.13381E+00_JPRB, 0.13417E+00_JPRB, 0.13449E+00_JPRB ,&
&  0.13504E+00_JPRB, 0.13552E+00_JPRB, 0.13599E+00_JPRB, 0.13706E+00_JPRB, 0.13794E+00_JPRB, 0.13926E+00_JPRB, 0.14064E+00_JPRB ,&
&  0.14160E+00_JPRB, 0.14308E+00_JPRB, 0.14432E+00_JPRB, 0.14570E+00_JPRB, 0.14979E+00_JPRB, 0.15307E+00_JPRB, 0.15971E+00_JPRB ,&
&  0.16410E+00_JPRB, 0.16595E+00_JPRB, 0.16696E+00_JPRB, 0.17195E+00_JPRB, 0.17199E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.17254E+00_JPRB, 0.17440E+00_JPRB, 0.17639E+00_JPRB, 0.17794E+00_JPRB, 0.17948E+00_JPRB, 0.18081E+00_JPRB, 0.18153E+00_JPRB ,&
&  0.18173E+00_JPRB, 0.18120E+00_JPRB, 0.18131E+00_JPRB, 0.18015E+00_JPRB, 0.17907E+00_JPRB, 0.17491E+00_JPRB, 0.17431E+00_JPRB ,&
&  0.17328E+00_JPRB, 0.17380E+00_JPRB, 0.17334E+00_JPRB, 0.17353E+00_JPRB, 0.17282E+00_JPRB, 0.17246E+00_JPRB, 0.17100E+00_JPRB ,&
&  0.16886E+00_JPRB, 0.16724E+00_JPRB, 0.16564E+00_JPRB, 0.16517E+00_JPRB, 0.16591E+00_JPRB, 0.16647E+00_JPRB, 0.16620E+00_JPRB ,&
&  0.16623E+00_JPRB, 0.16652E+00_JPRB, 0.16697E+00_JPRB, 0.16705E+00_JPRB, 0.17212E+00_JPRB, 0.17421E+00_JPRB, 0.17534E+00_JPRB ,&
&  0.17531E+00_JPRB, 0.17576E+00_JPRB, 0.17595E+00_JPRB, 0.17519E+00_JPRB, 0.17440E+00_JPRB, 0.17059E+00_JPRB, 0.16807E+00_JPRB ,&
&  0.16034E+00_JPRB, 0.15479E+00_JPRB, 0.15055E+00_JPRB, 0.14795E+00_JPRB, 0.14686E+00_JPRB, 0.14674E+00_JPRB, 0.14619E+00_JPRB ,&
&  0.14470E+00_JPRB, 0.14333E+00_JPRB, 0.14248E+00_JPRB, 0.14310E+00_JPRB, 0.14412E+00_JPRB, 0.14529E+00_JPRB, 0.14661E+00_JPRB ,&
&  0.14781E+00_JPRB, 0.14812E+00_JPRB, 0.14812E+00_JPRB, 0.14851E+00_JPRB, 0.14881E+00_JPRB, 0.14926E+00_JPRB, 0.14961E+00_JPRB ,&
&  0.15054E+00_JPRB, 0.14982E+00_JPRB, 0.15038E+00_JPRB, 0.15143E+00_JPRB, 0.14926E+00_JPRB, 0.14577E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.13905E+00_JPRB, 0.13723E+00_JPRB, 0.13576E+00_JPRB, 0.13483E+00_JPRB, 0.13461E+00_JPRB, 0.13472E+00_JPRB, 0.13524E+00_JPRB ,&
&  0.13590E+00_JPRB, 0.13681E+00_JPRB, 0.13740E+00_JPRB, 0.13815E+00_JPRB, 0.13907E+00_JPRB, 0.14006E+00_JPRB, 0.14125E+00_JPRB ,&
&  0.14272E+00_JPRB, 0.14414E+00_JPRB, 0.14513E+00_JPRB, 0.14656E+00_JPRB, 0.14926E+00_JPRB, 0.15289E+00_JPRB, 0.15841E+00_JPRB ,&
&  0.16465E+00_JPRB, 0.16998E+00_JPRB, 0.16785E+00_JPRB, 0.16998E+00_JPRB, 0.17172E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.17789E+00_JPRB, 0.18035E+00_JPRB, 0.18214E+00_JPRB, 0.18382E+00_JPRB, 0.18513E+00_JPRB, 0.18662E+00_JPRB, 0.18737E+00_JPRB ,&
&  0.18806E+00_JPRB, 0.18811E+00_JPRB, 0.18856E+00_JPRB, 0.18852E+00_JPRB, 0.18871E+00_JPRB, 0.18764E+00_JPRB, 0.18705E+00_JPRB ,&
&  0.18659E+00_JPRB, 0.18681E+00_JPRB, 0.18668E+00_JPRB, 0.18642E+00_JPRB, 0.18558E+00_JPRB, 0.18452E+00_JPRB, 0.18300E+00_JPRB ,&
&  0.18003E+00_JPRB, 0.17811E+00_JPRB, 0.17849E+00_JPRB, 0.17810E+00_JPRB, 0.17850E+00_JPRB, 0.17853E+00_JPRB, 0.17895E+00_JPRB ,&
&  0.17496E+00_JPRB, 0.17343E+00_JPRB, 0.17352E+00_JPRB, 0.17261E+00_JPRB, 0.17490E+00_JPRB, 0.17590E+00_JPRB, 0.17770E+00_JPRB ,&
&  0.17828E+00_JPRB, 0.17872E+00_JPRB, 0.17951E+00_JPRB, 0.17917E+00_JPRB, 0.17954E+00_JPRB, 0.18002E+00_JPRB, 0.17892E+00_JPRB ,&
&  0.17606E+00_JPRB, 0.17375E+00_JPRB, 0.17056E+00_JPRB, 0.16274E+00_JPRB, 0.16070E+00_JPRB, 0.16085E+00_JPRB, 0.15940E+00_JPRB ,&
&  0.15673E+00_JPRB, 0.15695E+00_JPRB, 0.15648E+00_JPRB, 0.15502E+00_JPRB, 0.15666E+00_JPRB, 0.15909E+00_JPRB, 0.15925E+00_JPRB ,&
&  0.15816E+00_JPRB, 0.15791E+00_JPRB, 0.15734E+00_JPRB, 0.15618E+00_JPRB, 0.15477E+00_JPRB, 0.15417E+00_JPRB, 0.15364E+00_JPRB ,&
&  0.15339E+00_JPRB, 0.15223E+00_JPRB, 0.15545E+00_JPRB, 0.15631E+00_JPRB, 0.15175E+00_JPRB, 0.15328E+00_JPRB, 0.15102E+00_JPRB ,&
&  0.14483E+00_JPRB, 0.14203E+00_JPRB, 0.14052E+00_JPRB, 0.13927E+00_JPRB, 0.13908E+00_JPRB, 0.13924E+00_JPRB, 0.13915E+00_JPRB ,&
&  0.14000E+00_JPRB, 0.14105E+00_JPRB, 0.14154E+00_JPRB, 0.14215E+00_JPRB, 0.14300E+00_JPRB, 0.14424E+00_JPRB, 0.14587E+00_JPRB ,&
&  0.14728E+00_JPRB, 0.14928E+00_JPRB, 0.15033E+00_JPRB, 0.15205E+00_JPRB, 0.15334E+00_JPRB, 0.15487E+00_JPRB, 0.15845E+00_JPRB ,&
&  0.16147E+00_JPRB, 0.16729E+00_JPRB, 0.17524E+00_JPRB, 0.17794E+00_JPRB, 0.17558E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.18375E+00_JPRB, 0.18607E+00_JPRB, 0.18896E+00_JPRB, 0.19193E+00_JPRB, 0.19424E+00_JPRB, 0.19657E+00_JPRB, 0.19858E+00_JPRB ,&
&  0.20025E+00_JPRB, 0.20134E+00_JPRB, 0.20201E+00_JPRB, 0.20235E+00_JPRB, 0.20256E+00_JPRB, 0.20220E+00_JPRB, 0.20138E+00_JPRB ,&
&  0.20053E+00_JPRB, 0.19912E+00_JPRB, 0.19732E+00_JPRB, 0.19580E+00_JPRB, 0.19472E+00_JPRB, 0.19343E+00_JPRB, 0.19166E+00_JPRB ,&
&  0.18988E+00_JPRB, 0.18836E+00_JPRB, 0.18727E+00_JPRB, 0.18627E+00_JPRB, 0.18628E+00_JPRB, 0.18683E+00_JPRB, 0.18728E+00_JPRB ,&
&  0.18355E+00_JPRB, 0.17843E+00_JPRB, 0.17898E+00_JPRB, 0.17999E+00_JPRB, 0.17872E+00_JPRB, 0.18005E+00_JPRB, 0.18088E+00_JPRB ,&
&  0.18204E+00_JPRB, 0.18351E+00_JPRB, 0.18490E+00_JPRB, 0.18561E+00_JPRB, 0.18613E+00_JPRB, 0.18686E+00_JPRB, 0.18628E+00_JPRB ,&
&  0.18431E+00_JPRB, 0.18256E+00_JPRB, 0.18097E+00_JPRB, 0.17939E+00_JPRB, 0.17896E+00_JPRB, 0.18034E+00_JPRB, 0.18114E+00_JPRB ,&
&  0.18022E+00_JPRB, 0.17884E+00_JPRB, 0.17753E+00_JPRB, 0.17768E+00_JPRB, 0.17832E+00_JPRB, 0.17863E+00_JPRB, 0.17674E+00_JPRB ,&
&  0.17540E+00_JPRB, 0.17278E+00_JPRB, 0.17028E+00_JPRB, 0.16785E+00_JPRB, 0.16654E+00_JPRB, 0.16326E+00_JPRB, 0.15982E+00_JPRB ,&
&  0.15907E+00_JPRB, 0.16268E+00_JPRB, 0.16838E+00_JPRB, 0.17074E+00_JPRB, 0.16200E+00_JPRB, 0.16165E+00_JPRB, 0.16135E+00_JPRB ,&
&  0.15698E+00_JPRB, 0.15220E+00_JPRB, 0.14881E+00_JPRB, 0.14738E+00_JPRB, 0.14690E+00_JPRB, 0.14651E+00_JPRB, 0.14707E+00_JPRB ,&
&  0.14805E+00_JPRB, 0.14865E+00_JPRB, 0.14926E+00_JPRB, 0.15037E+00_JPRB, 0.15142E+00_JPRB, 0.15248E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15539E+00_JPRB, 0.15784E+00_JPRB, 0.15988E+00_JPRB, 0.16265E+00_JPRB, 0.16562E+00_JPRB, 0.16862E+00_JPRB, 0.17167E+00_JPRB ,&
&  0.17504E+00_JPRB, 0.17719E+00_JPRB, 0.17869E+00_JPRB, 0.18123E+00_JPRB, 0.18245E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.19646E+00_JPRB, 0.20184E+00_JPRB, 0.20667E+00_JPRB, 0.20956E+00_JPRB, 0.21169E+00_JPRB, 0.21446E+00_JPRB, 0.21699E+00_JPRB ,&
&  0.21872E+00_JPRB, 0.21969E+00_JPRB, 0.22008E+00_JPRB, 0.21987E+00_JPRB, 0.21884E+00_JPRB, 0.21720E+00_JPRB, 0.21532E+00_JPRB ,&
&  0.21327E+00_JPRB, 0.21049E+00_JPRB, 0.20795E+00_JPRB, 0.20596E+00_JPRB, 0.20425E+00_JPRB, 0.20226E+00_JPRB, 0.20065E+00_JPRB ,&
&  0.19899E+00_JPRB, 0.19835E+00_JPRB, 0.19764E+00_JPRB, 0.19701E+00_JPRB, 0.19653E+00_JPRB, 0.19564E+00_JPRB, 0.19484E+00_JPRB ,&
&  0.19281E+00_JPRB, 0.18944E+00_JPRB, 0.18777E+00_JPRB, 0.18708E+00_JPRB, 0.18722E+00_JPRB, 0.18675E+00_JPRB, 0.18614E+00_JPRB ,&
&  0.18786E+00_JPRB, 0.19073E+00_JPRB, 0.19282E+00_JPRB, 0.19398E+00_JPRB, 0.19478E+00_JPRB, 0.19584E+00_JPRB, 0.19708E+00_JPRB ,&
&  0.19491E+00_JPRB, 0.19222E+00_JPRB, 0.19024E+00_JPRB, 0.18921E+00_JPRB, 0.18903E+00_JPRB, 0.18972E+00_JPRB, 0.19121E+00_JPRB ,&
&  0.19256E+00_JPRB, 0.19330E+00_JPRB, 0.19291E+00_JPRB, 0.19158E+00_JPRB, 0.19001E+00_JPRB, 0.18925E+00_JPRB, 0.18877E+00_JPRB ,&
&  0.18854E+00_JPRB, 0.18826E+00_JPRB, 0.18789E+00_JPRB, 0.18691E+00_JPRB, 0.18186E+00_JPRB, 0.17800E+00_JPRB, 0.17538E+00_JPRB ,&
&  0.17590E+00_JPRB, 0.17620E+00_JPRB, 0.17550E+00_JPRB, 0.18026E+00_JPRB, 0.18543E+00_JPRB, 0.18128E+00_JPRB, 0.17617E+00_JPRB ,&
&  0.16931E+00_JPRB, 0.16944E+00_JPRB, 0.16682E+00_JPRB, 0.16246E+00_JPRB, 0.16112E+00_JPRB, 0.16119E+00_JPRB, 0.16094E+00_JPRB ,&
&  0.16176E+00_JPRB, 0.16211E+00_JPRB, 0.16334E+00_JPRB, 0.16397E+00_JPRB, 0.16430E+00_JPRB, 0.16501E+00_JPRB, 0.16572E+00_JPRB ,&
&  0.16751E+00_JPRB, 0.17093E+00_JPRB, 0.17511E+00_JPRB, 0.17691E+00_JPRB, 0.17943E+00_JPRB, 0.18182E+00_JPRB, 0.18344E+00_JPRB ,&
&  0.18476E+00_JPRB, 0.18657E+00_JPRB, 0.18831E+00_JPRB, 0.18975E+00_JPRB, 0.19223E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.21480E+00_JPRB, 0.21974E+00_JPRB, 0.22299E+00_JPRB, 0.22611E+00_JPRB, 0.22902E+00_JPRB, 0.23361E+00_JPRB, 0.23676E+00_JPRB ,&
&  0.23741E+00_JPRB, 0.23745E+00_JPRB, 0.23699E+00_JPRB, 0.23576E+00_JPRB, 0.23385E+00_JPRB, 0.23208E+00_JPRB, 0.22923E+00_JPRB ,&
&  0.22570E+00_JPRB, 0.22243E+00_JPRB, 0.21949E+00_JPRB, 0.21673E+00_JPRB, 0.21412E+00_JPRB, 0.21172E+00_JPRB, 0.20980E+00_JPRB ,&
&  0.20911E+00_JPRB, 0.20871E+00_JPRB, 0.20765E+00_JPRB, 0.20727E+00_JPRB, 0.20725E+00_JPRB, 0.20660E+00_JPRB, 0.20530E+00_JPRB ,&
&  0.20370E+00_JPRB, 0.20093E+00_JPRB, 0.19812E+00_JPRB, 0.19706E+00_JPRB, 0.19720E+00_JPRB, 0.19655E+00_JPRB, 0.19567E+00_JPRB ,&
&  0.19698E+00_JPRB, 0.19909E+00_JPRB, 0.20141E+00_JPRB, 0.20260E+00_JPRB, 0.20335E+00_JPRB, 0.20447E+00_JPRB, 0.20583E+00_JPRB ,&
&  0.20597E+00_JPRB, 0.20352E+00_JPRB, 0.20134E+00_JPRB, 0.20085E+00_JPRB, 0.20072E+00_JPRB, 0.20036E+00_JPRB, 0.20066E+00_JPRB ,&
&  0.20125E+00_JPRB, 0.20258E+00_JPRB, 0.20348E+00_JPRB, 0.20255E+00_JPRB, 0.20088E+00_JPRB, 0.19966E+00_JPRB, 0.19887E+00_JPRB ,&
&  0.19841E+00_JPRB, 0.19785E+00_JPRB, 0.19690E+00_JPRB, 0.19544E+00_JPRB, 0.19383E+00_JPRB, 0.19233E+00_JPRB, 0.19083E+00_JPRB ,&
&  0.18958E+00_JPRB, 0.18977E+00_JPRB, 0.19185E+00_JPRB, 0.19329E+00_JPRB, 0.19313E+00_JPRB, 0.19185E+00_JPRB, 0.18992E+00_JPRB ,&
&  0.18858E+00_JPRB, 0.18647E+00_JPRB, 0.18477E+00_JPRB, 0.18344E+00_JPRB, 0.18241E+00_JPRB, 0.18099E+00_JPRB, 0.18045E+00_JPRB ,&
&  0.18074E+00_JPRB, 0.18233E+00_JPRB, 0.18303E+00_JPRB, 0.18332E+00_JPRB, 0.18311E+00_JPRB, 0.18293E+00_JPRB, 0.18384E+00_JPRB ,&
&  0.18506E+00_JPRB, 0.18692E+00_JPRB, 0.18891E+00_JPRB, 0.19153E+00_JPRB, 0.19411E+00_JPRB, 0.19745E+00_JPRB, 0.20028E+00_JPRB ,&
&  0.20208E+00_JPRB, 0.20275E+00_JPRB, 0.20416E+00_JPRB, 0.20616E+00_JPRB, 0.20947E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.23226E+00_JPRB, 0.23610E+00_JPRB, 0.23947E+00_JPRB, 0.24336E+00_JPRB, 0.24621E+00_JPRB, 0.25091E+00_JPRB, 0.25307E+00_JPRB ,&
&  0.25316E+00_JPRB, 0.25282E+00_JPRB, 0.25119E+00_JPRB, 0.24847E+00_JPRB, 0.24441E+00_JPRB, 0.24037E+00_JPRB, 0.23660E+00_JPRB ,&
&  0.23365E+00_JPRB, 0.23115E+00_JPRB, 0.22908E+00_JPRB, 0.22720E+00_JPRB, 0.22462E+00_JPRB, 0.22216E+00_JPRB, 0.22020E+00_JPRB ,&
&  0.21921E+00_JPRB, 0.21834E+00_JPRB, 0.21782E+00_JPRB, 0.21748E+00_JPRB, 0.21709E+00_JPRB, 0.21676E+00_JPRB, 0.21673E+00_JPRB ,&
&  0.21563E+00_JPRB, 0.21230E+00_JPRB, 0.21044E+00_JPRB, 0.20988E+00_JPRB, 0.20945E+00_JPRB, 0.20997E+00_JPRB, 0.20983E+00_JPRB ,&
&  0.20976E+00_JPRB, 0.21055E+00_JPRB, 0.21224E+00_JPRB, 0.21400E+00_JPRB, 0.21415E+00_JPRB, 0.21463E+00_JPRB, 0.21573E+00_JPRB ,&
&  0.21611E+00_JPRB, 0.21546E+00_JPRB, 0.21375E+00_JPRB, 0.21318E+00_JPRB, 0.21371E+00_JPRB, 0.21367E+00_JPRB, 0.21401E+00_JPRB ,&
&  0.21465E+00_JPRB, 0.21574E+00_JPRB, 0.21629E+00_JPRB, 0.21482E+00_JPRB, 0.21333E+00_JPRB, 0.21278E+00_JPRB, 0.21272E+00_JPRB ,&
&  0.21237E+00_JPRB, 0.21133E+00_JPRB, 0.20925E+00_JPRB, 0.20738E+00_JPRB, 0.20466E+00_JPRB, 0.20245E+00_JPRB, 0.20132E+00_JPRB ,&
&  0.20182E+00_JPRB, 0.20232E+00_JPRB, 0.20362E+00_JPRB, 0.20474E+00_JPRB, 0.20539E+00_JPRB, 0.20546E+00_JPRB, 0.20388E+00_JPRB ,&
&  0.20184E+00_JPRB, 0.20097E+00_JPRB, 0.20007E+00_JPRB, 0.19922E+00_JPRB, 0.19941E+00_JPRB, 0.19931E+00_JPRB, 0.19891E+00_JPRB ,&
&  0.19844E+00_JPRB, 0.19880E+00_JPRB, 0.19927E+00_JPRB, 0.19936E+00_JPRB, 0.19909E+00_JPRB, 0.19845E+00_JPRB, 0.19904E+00_JPRB ,&
&  0.20045E+00_JPRB, 0.20238E+00_JPRB, 0.20463E+00_JPRB, 0.20705E+00_JPRB, 0.20968E+00_JPRB, 0.21215E+00_JPRB, 0.21466E+00_JPRB ,&
&  0.21784E+00_JPRB, 0.21930E+00_JPRB, 0.22152E+00_JPRB, 0.22477E+00_JPRB, 0.22795E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.24750E+00_JPRB, 0.25059E+00_JPRB, 0.25547E+00_JPRB, 0.26033E+00_JPRB, 0.26361E+00_JPRB, 0.26814E+00_JPRB, 0.26913E+00_JPRB ,&
&  0.26859E+00_JPRB, 0.26679E+00_JPRB, 0.26263E+00_JPRB, 0.25754E+00_JPRB, 0.25285E+00_JPRB, 0.24905E+00_JPRB, 0.24643E+00_JPRB ,&
&  0.24477E+00_JPRB, 0.24327E+00_JPRB, 0.24155E+00_JPRB, 0.23888E+00_JPRB, 0.23556E+00_JPRB, 0.23242E+00_JPRB, 0.22980E+00_JPRB ,&
&  0.22860E+00_JPRB, 0.22822E+00_JPRB, 0.22830E+00_JPRB, 0.22796E+00_JPRB, 0.22768E+00_JPRB, 0.22880E+00_JPRB, 0.23037E+00_JPRB ,&
&  0.22933E+00_JPRB, 0.22516E+00_JPRB, 0.22253E+00_JPRB, 0.22234E+00_JPRB, 0.22260E+00_JPRB, 0.22326E+00_JPRB, 0.22416E+00_JPRB ,&
&  0.22468E+00_JPRB, 0.22586E+00_JPRB, 0.22757E+00_JPRB, 0.22903E+00_JPRB, 0.22989E+00_JPRB, 0.23212E+00_JPRB, 0.23328E+00_JPRB ,&
&  0.23335E+00_JPRB, 0.23264E+00_JPRB, 0.23130E+00_JPRB, 0.22875E+00_JPRB, 0.22688E+00_JPRB, 0.22623E+00_JPRB, 0.22554E+00_JPRB ,&
&  0.22620E+00_JPRB, 0.22690E+00_JPRB, 0.22753E+00_JPRB, 0.22676E+00_JPRB, 0.22511E+00_JPRB, 0.22484E+00_JPRB, 0.22524E+00_JPRB ,&
&  0.22513E+00_JPRB, 0.22392E+00_JPRB, 0.22208E+00_JPRB, 0.22090E+00_JPRB, 0.21979E+00_JPRB, 0.21948E+00_JPRB, 0.21936E+00_JPRB ,&
&  0.21920E+00_JPRB, 0.21830E+00_JPRB, 0.21770E+00_JPRB, 0.21741E+00_JPRB, 0.21725E+00_JPRB, 0.21760E+00_JPRB, 0.21734E+00_JPRB ,&
&  0.21745E+00_JPRB, 0.21772E+00_JPRB, 0.21683E+00_JPRB, 0.21658E+00_JPRB, 0.21655E+00_JPRB, 0.21686E+00_JPRB, 0.21695E+00_JPRB ,&
&  0.21668E+00_JPRB, 0.21626E+00_JPRB, 0.21597E+00_JPRB, 0.21609E+00_JPRB, 0.21577E+00_JPRB, 0.21522E+00_JPRB, 0.21604E+00_JPRB ,&
&  0.21913E+00_JPRB, 0.22238E+00_JPRB, 0.22518E+00_JPRB, 0.22703E+00_JPRB, 0.22773E+00_JPRB, 0.22928E+00_JPRB, 0.23232E+00_JPRB ,&
&  0.23525E+00_JPRB, 0.23698E+00_JPRB, 0.23963E+00_JPRB, 0.24171E+00_JPRB, 0.24398E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.26501E+00_JPRB, 0.26785E+00_JPRB, 0.27212E+00_JPRB, 0.27804E+00_JPRB, 0.28206E+00_JPRB, 0.28562E+00_JPRB, 0.28544E+00_JPRB ,&
&  0.28312E+00_JPRB, 0.27949E+00_JPRB, 0.27504E+00_JPRB, 0.26966E+00_JPRB, 0.26417E+00_JPRB, 0.26010E+00_JPRB, 0.25864E+00_JPRB ,&
&  0.25833E+00_JPRB, 0.25715E+00_JPRB, 0.25441E+00_JPRB, 0.25070E+00_JPRB, 0.24649E+00_JPRB, 0.24291E+00_JPRB, 0.23991E+00_JPRB ,&
&  0.23862E+00_JPRB, 0.23860E+00_JPRB, 0.23919E+00_JPRB, 0.23945E+00_JPRB, 0.23955E+00_JPRB, 0.24177E+00_JPRB, 0.24555E+00_JPRB ,&
&  0.24346E+00_JPRB, 0.24099E+00_JPRB, 0.23572E+00_JPRB, 0.23519E+00_JPRB, 0.23666E+00_JPRB, 0.23915E+00_JPRB, 0.24176E+00_JPRB ,&
&  0.24426E+00_JPRB, 0.24756E+00_JPRB, 0.25060E+00_JPRB, 0.25259E+00_JPRB, 0.25360E+00_JPRB, 0.25476E+00_JPRB, 0.25552E+00_JPRB ,&
&  0.25568E+00_JPRB, 0.25461E+00_JPRB, 0.25274E+00_JPRB, 0.24935E+00_JPRB, 0.24498E+00_JPRB, 0.24204E+00_JPRB, 0.24060E+00_JPRB ,&
&  0.23909E+00_JPRB, 0.23884E+00_JPRB, 0.23874E+00_JPRB, 0.23812E+00_JPRB, 0.23716E+00_JPRB, 0.23686E+00_JPRB, 0.23734E+00_JPRB ,&
&  0.23762E+00_JPRB, 0.23712E+00_JPRB, 0.23611E+00_JPRB, 0.23581E+00_JPRB, 0.23629E+00_JPRB, 0.23749E+00_JPRB, 0.23868E+00_JPRB ,&
&  0.23957E+00_JPRB, 0.23932E+00_JPRB, 0.23853E+00_JPRB, 0.23669E+00_JPRB, 0.23537E+00_JPRB, 0.23548E+00_JPRB, 0.23666E+00_JPRB ,&
&  0.23745E+00_JPRB, 0.23753E+00_JPRB, 0.23779E+00_JPRB, 0.23689E+00_JPRB, 0.23609E+00_JPRB, 0.23622E+00_JPRB, 0.23668E+00_JPRB ,&
&  0.23683E+00_JPRB, 0.23667E+00_JPRB, 0.23633E+00_JPRB, 0.23616E+00_JPRB, 0.23659E+00_JPRB, 0.23674E+00_JPRB, 0.23775E+00_JPRB ,&
&  0.24027E+00_JPRB, 0.24311E+00_JPRB, 0.24592E+00_JPRB, 0.24798E+00_JPRB, 0.24927E+00_JPRB, 0.25017E+00_JPRB, 0.25335E+00_JPRB ,&
&  0.25757E+00_JPRB, 0.26061E+00_JPRB, 0.26114E+00_JPRB, 0.26115E+00_JPRB, 0.26282E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.28686E+00_JPRB, 0.28700E+00_JPRB, 0.29134E+00_JPRB, 0.29655E+00_JPRB, 0.30227E+00_JPRB, 0.30342E+00_JPRB, 0.30081E+00_JPRB ,&
&  0.29780E+00_JPRB, 0.29360E+00_JPRB, 0.28760E+00_JPRB, 0.28034E+00_JPRB, 0.27376E+00_JPRB, 0.27178E+00_JPRB, 0.27180E+00_JPRB ,&
&  0.27202E+00_JPRB, 0.27032E+00_JPRB, 0.26693E+00_JPRB, 0.26308E+00_JPRB, 0.25856E+00_JPRB, 0.25446E+00_JPRB, 0.25124E+00_JPRB ,&
&  0.24991E+00_JPRB, 0.24911E+00_JPRB, 0.24929E+00_JPRB, 0.24935E+00_JPRB, 0.25005E+00_JPRB, 0.25280E+00_JPRB, 0.25809E+00_JPRB ,&
&  0.25588E+00_JPRB, 0.25648E+00_JPRB, 0.24967E+00_JPRB, 0.25217E+00_JPRB, 0.25475E+00_JPRB, 0.25988E+00_JPRB, 0.26596E+00_JPRB ,&
&  0.27219E+00_JPRB, 0.27734E+00_JPRB, 0.28007E+00_JPRB, 0.27993E+00_JPRB, 0.27794E+00_JPRB, 0.27752E+00_JPRB, 0.27723E+00_JPRB ,&
&  0.27647E+00_JPRB, 0.27546E+00_JPRB, 0.27361E+00_JPRB, 0.26995E+00_JPRB, 0.26487E+00_JPRB, 0.26042E+00_JPRB, 0.25719E+00_JPRB ,&
&  0.25634E+00_JPRB, 0.25579E+00_JPRB, 0.25428E+00_JPRB, 0.25274E+00_JPRB, 0.25114E+00_JPRB, 0.25020E+00_JPRB, 0.25067E+00_JPRB ,&
&  0.25188E+00_JPRB, 0.25295E+00_JPRB, 0.25375E+00_JPRB, 0.25508E+00_JPRB, 0.25735E+00_JPRB, 0.25956E+00_JPRB, 0.26130E+00_JPRB ,&
&  0.26206E+00_JPRB, 0.26156E+00_JPRB, 0.26082E+00_JPRB, 0.25990E+00_JPRB, 0.25939E+00_JPRB, 0.25891E+00_JPRB, 0.25897E+00_JPRB ,&
&  0.25912E+00_JPRB, 0.25903E+00_JPRB, 0.25830E+00_JPRB, 0.25702E+00_JPRB, 0.25586E+00_JPRB, 0.25491E+00_JPRB, 0.25459E+00_JPRB ,&
&  0.25489E+00_JPRB, 0.25527E+00_JPRB, 0.25517E+00_JPRB, 0.25479E+00_JPRB, 0.25510E+00_JPRB, 0.25579E+00_JPRB, 0.25729E+00_JPRB ,&
&  0.25940E+00_JPRB, 0.26208E+00_JPRB, 0.26570E+00_JPRB, 0.26954E+00_JPRB, 0.27127E+00_JPRB, 0.27381E+00_JPRB, 0.27844E+00_JPRB ,&
&  0.28276E+00_JPRB, 0.28493E+00_JPRB, 0.28624E+00_JPRB, 0.28733E+00_JPRB, 0.28806E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.31563E+00_JPRB, 0.31137E+00_JPRB, 0.31253E+00_JPRB, 0.31700E+00_JPRB, 0.32310E+00_JPRB, 0.32125E+00_JPRB, 0.31826E+00_JPRB ,&
&  0.31220E+00_JPRB, 0.30448E+00_JPRB, 0.29699E+00_JPRB, 0.28998E+00_JPRB, 0.28652E+00_JPRB, 0.28641E+00_JPRB, 0.28596E+00_JPRB ,&
&  0.28475E+00_JPRB, 0.28157E+00_JPRB, 0.27693E+00_JPRB, 0.27239E+00_JPRB, 0.26905E+00_JPRB, 0.26716E+00_JPRB, 0.26548E+00_JPRB ,&
&  0.26424E+00_JPRB, 0.26267E+00_JPRB, 0.26326E+00_JPRB, 0.26308E+00_JPRB, 0.26378E+00_JPRB, 0.26756E+00_JPRB, 0.27068E+00_JPRB ,&
&  0.27229E+00_JPRB, 0.27447E+00_JPRB, 0.27391E+00_JPRB, 0.27454E+00_JPRB, 0.27803E+00_JPRB, 0.28638E+00_JPRB, 0.29706E+00_JPRB ,&
&  0.30631E+00_JPRB, 0.31176E+00_JPRB, 0.31350E+00_JPRB, 0.31165E+00_JPRB, 0.30822E+00_JPRB, 0.30538E+00_JPRB, 0.30362E+00_JPRB ,&
&  0.30172E+00_JPRB, 0.29814E+00_JPRB, 0.29430E+00_JPRB, 0.29110E+00_JPRB, 0.28971E+00_JPRB, 0.28856E+00_JPRB, 0.28687E+00_JPRB ,&
&  0.28467E+00_JPRB, 0.28234E+00_JPRB, 0.27915E+00_JPRB, 0.27573E+00_JPRB, 0.27272E+00_JPRB, 0.27110E+00_JPRB, 0.27125E+00_JPRB ,&
&  0.27287E+00_JPRB, 0.27552E+00_JPRB, 0.27898E+00_JPRB, 0.28320E+00_JPRB, 0.28666E+00_JPRB, 0.28993E+00_JPRB, 0.29167E+00_JPRB ,&
&  0.29232E+00_JPRB, 0.29210E+00_JPRB, 0.29153E+00_JPRB, 0.29092E+00_JPRB, 0.29002E+00_JPRB, 0.28862E+00_JPRB, 0.28698E+00_JPRB ,&
&  0.28549E+00_JPRB, 0.28427E+00_JPRB, 0.28323E+00_JPRB, 0.28220E+00_JPRB, 0.28016E+00_JPRB, 0.27770E+00_JPRB, 0.27598E+00_JPRB ,&
&  0.27508E+00_JPRB, 0.27465E+00_JPRB, 0.27435E+00_JPRB, 0.27379E+00_JPRB, 0.27371E+00_JPRB, 0.27467E+00_JPRB, 0.27663E+00_JPRB ,&
&  0.27957E+00_JPRB, 0.28364E+00_JPRB, 0.28876E+00_JPRB, 0.29452E+00_JPRB, 0.30077E+00_JPRB, 0.30738E+00_JPRB, 0.31381E+00_JPRB ,&
&  0.31836E+00_JPRB, 0.31938E+00_JPRB, 0.32046E+00_JPRB, 0.32168E+00_JPRB, 0.32008E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.35023E+00_JPRB, 0.34210E+00_JPRB, 0.33871E+00_JPRB, 0.34042E+00_JPRB, 0.34275E+00_JPRB, 0.34047E+00_JPRB, 0.33503E+00_JPRB ,&
&  0.32470E+00_JPRB, 0.31539E+00_JPRB, 0.30767E+00_JPRB, 0.30364E+00_JPRB, 0.30441E+00_JPRB, 0.30412E+00_JPRB, 0.30145E+00_JPRB ,&
&  0.29882E+00_JPRB, 0.29427E+00_JPRB, 0.29047E+00_JPRB, 0.28664E+00_JPRB, 0.28373E+00_JPRB, 0.28243E+00_JPRB, 0.28190E+00_JPRB ,&
&  0.28038E+00_JPRB, 0.27859E+00_JPRB, 0.27828E+00_JPRB, 0.27847E+00_JPRB, 0.28041E+00_JPRB, 0.28483E+00_JPRB, 0.29022E+00_JPRB ,&
&  0.29889E+00_JPRB, 0.28461E+00_JPRB, 0.30128E+00_JPRB, 0.30600E+00_JPRB, 0.31053E+00_JPRB, 0.31899E+00_JPRB, 0.32880E+00_JPRB ,&
&  0.33742E+00_JPRB, 0.34200E+00_JPRB, 0.34363E+00_JPRB, 0.34314E+00_JPRB, 0.34025E+00_JPRB, 0.33812E+00_JPRB, 0.33406E+00_JPRB ,&
&  0.32917E+00_JPRB, 0.32732E+00_JPRB, 0.32659E+00_JPRB, 0.32471E+00_JPRB, 0.32250E+00_JPRB, 0.31897E+00_JPRB, 0.31498E+00_JPRB ,&
&  0.31111E+00_JPRB, 0.30726E+00_JPRB, 0.30416E+00_JPRB, 0.30130E+00_JPRB, 0.29920E+00_JPRB, 0.29847E+00_JPRB, 0.29932E+00_JPRB ,&
&  0.30070E+00_JPRB, 0.30328E+00_JPRB, 0.30941E+00_JPRB, 0.31696E+00_JPRB, 0.32363E+00_JPRB, 0.32836E+00_JPRB, 0.33151E+00_JPRB ,&
&  0.33265E+00_JPRB, 0.33263E+00_JPRB, 0.33183E+00_JPRB, 0.33046E+00_JPRB, 0.32784E+00_JPRB, 0.32495E+00_JPRB, 0.32062E+00_JPRB ,&
&  0.31685E+00_JPRB, 0.31373E+00_JPRB, 0.31042E+00_JPRB, 0.30773E+00_JPRB, 0.30468E+00_JPRB, 0.30091E+00_JPRB, 0.29738E+00_JPRB ,&
&  0.29461E+00_JPRB, 0.29269E+00_JPRB, 0.29129E+00_JPRB, 0.29083E+00_JPRB, 0.29138E+00_JPRB, 0.29376E+00_JPRB, 0.29639E+00_JPRB ,&
&  0.29977E+00_JPRB, 0.30541E+00_JPRB, 0.31255E+00_JPRB, 0.32155E+00_JPRB, 0.33315E+00_JPRB, 0.34152E+00_JPRB, 0.34837E+00_JPRB ,&
&  0.35327E+00_JPRB, 0.35440E+00_JPRB, 0.35632E+00_JPRB, 0.35652E+00_JPRB, 0.35498E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.38644E+00_JPRB, 0.37901E+00_JPRB, 0.37119E+00_JPRB, 0.36959E+00_JPRB, 0.36531E+00_JPRB, 0.35903E+00_JPRB, 0.35085E+00_JPRB ,&
&  0.33921E+00_JPRB, 0.33066E+00_JPRB, 0.32411E+00_JPRB, 0.32174E+00_JPRB, 0.32362E+00_JPRB, 0.32148E+00_JPRB, 0.31712E+00_JPRB ,&
&  0.31393E+00_JPRB, 0.31032E+00_JPRB, 0.30783E+00_JPRB, 0.30489E+00_JPRB, 0.30254E+00_JPRB, 0.30160E+00_JPRB, 0.30100E+00_JPRB ,&
&  0.29937E+00_JPRB, 0.29669E+00_JPRB, 0.29455E+00_JPRB, 0.29374E+00_JPRB, 0.29653E+00_JPRB, 0.30286E+00_JPRB, 0.31289E+00_JPRB ,&
&  0.31009E+00_JPRB, 0.29348E+00_JPRB, 0.31049E+00_JPRB, 0.33158E+00_JPRB, 0.34555E+00_JPRB, 0.35955E+00_JPRB, 0.37000E+00_JPRB ,&
&  0.37963E+00_JPRB, 0.38228E+00_JPRB, 0.38371E+00_JPRB, 0.38384E+00_JPRB, 0.38143E+00_JPRB, 0.37619E+00_JPRB, 0.37104E+00_JPRB ,&
&  0.36767E+00_JPRB, 0.36538E+00_JPRB, 0.36267E+00_JPRB, 0.35716E+00_JPRB, 0.35089E+00_JPRB, 0.34393E+00_JPRB, 0.33720E+00_JPRB ,&
&  0.33166E+00_JPRB, 0.32639E+00_JPRB, 0.32253E+00_JPRB, 0.32090E+00_JPRB, 0.32203E+00_JPRB, 0.32490E+00_JPRB, 0.33020E+00_JPRB ,&
&  0.33650E+00_JPRB, 0.34643E+00_JPRB, 0.36192E+00_JPRB, 0.37523E+00_JPRB, 0.38378E+00_JPRB, 0.38758E+00_JPRB, 0.38979E+00_JPRB ,&
&  0.38805E+00_JPRB, 0.38540E+00_JPRB, 0.38358E+00_JPRB, 0.38026E+00_JPRB, 0.37546E+00_JPRB, 0.36903E+00_JPRB, 0.36089E+00_JPRB ,&
&  0.35380E+00_JPRB, 0.34778E+00_JPRB, 0.34208E+00_JPRB, 0.33682E+00_JPRB, 0.33172E+00_JPRB, 0.32672E+00_JPRB, 0.32242E+00_JPRB ,&
&  0.31822E+00_JPRB, 0.31401E+00_JPRB, 0.31121E+00_JPRB, 0.31124E+00_JPRB, 0.31447E+00_JPRB, 0.31867E+00_JPRB, 0.32374E+00_JPRB ,&
&  0.32929E+00_JPRB, 0.33845E+00_JPRB, 0.34721E+00_JPRB, 0.36252E+00_JPRB, 0.37709E+00_JPRB, 0.38297E+00_JPRB, 0.38808E+00_JPRB ,&
&  0.39252E+00_JPRB, 0.39124E+00_JPRB, 0.39331E+00_JPRB, 0.39189E+00_JPRB, 0.38836E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.41984E+00_JPRB, 0.41986E+00_JPRB, 0.41438E+00_JPRB, 0.40368E+00_JPRB, 0.39043E+00_JPRB, 0.38128E+00_JPRB, 0.36942E+00_JPRB ,&
&  0.35717E+00_JPRB, 0.34981E+00_JPRB, 0.34161E+00_JPRB, 0.33745E+00_JPRB, 0.33689E+00_JPRB, 0.33303E+00_JPRB, 0.32970E+00_JPRB ,&
&  0.32695E+00_JPRB, 0.32215E+00_JPRB, 0.31919E+00_JPRB, 0.31655E+00_JPRB, 0.31625E+00_JPRB, 0.31630E+00_JPRB, 0.31602E+00_JPRB ,&
&  0.31464E+00_JPRB, 0.31215E+00_JPRB, 0.31003E+00_JPRB, 0.30883E+00_JPRB, 0.31109E+00_JPRB, 0.31800E+00_JPRB, 0.33136E+00_JPRB ,&
&  0.36042E+00_JPRB, 0.32511E+00_JPRB, 0.33978E+00_JPRB, 0.35535E+00_JPRB, 0.38247E+00_JPRB, 0.40372E+00_JPRB, 0.41363E+00_JPRB ,&
&  0.42458E+00_JPRB, 0.43385E+00_JPRB, 0.43892E+00_JPRB, 0.44259E+00_JPRB, 0.43625E+00_JPRB, 0.42655E+00_JPRB, 0.41958E+00_JPRB ,&
&  0.41122E+00_JPRB, 0.40151E+00_JPRB, 0.39177E+00_JPRB, 0.38125E+00_JPRB, 0.37478E+00_JPRB, 0.37079E+00_JPRB, 0.36598E+00_JPRB ,&
&  0.35931E+00_JPRB, 0.35252E+00_JPRB, 0.34810E+00_JPRB, 0.34780E+00_JPRB, 0.35277E+00_JPRB, 0.36304E+00_JPRB, 0.38220E+00_JPRB ,&
&  0.40105E+00_JPRB, 0.42163E+00_JPRB, 0.44189E+00_JPRB, 0.45516E+00_JPRB, 0.46229E+00_JPRB, 0.46180E+00_JPRB, 0.45739E+00_JPRB ,&
&  0.44988E+00_JPRB, 0.44216E+00_JPRB, 0.43609E+00_JPRB, 0.42794E+00_JPRB, 0.41763E+00_JPRB, 0.40874E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.39600E+00_JPRB, 0.38931E+00_JPRB, 0.38137E+00_JPRB, 0.37329E+00_JPRB, 0.36510E+00_JPRB, 0.35826E+00_JPRB, 0.35246E+00_JPRB ,&
&  0.34719E+00_JPRB, 0.34219E+00_JPRB, 0.33990E+00_JPRB, 0.34220E+00_JPRB, 0.34686E+00_JPRB, 0.35201E+00_JPRB, 0.35691E+00_JPRB ,&
&  0.36342E+00_JPRB, 0.37424E+00_JPRB, 0.38165E+00_JPRB, 0.40886E+00_JPRB, 0.42264E+00_JPRB, 0.42884E+00_JPRB, 0.43320E+00_JPRB ,&
&  0.43329E+00_JPRB, 0.42893E+00_JPRB, 0.38122E+00_JPRB, 0.41058E+00_JPRB, 0.41816E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.46564E+00_JPRB, 0.46337E+00_JPRB, 0.45462E+00_JPRB, 0.43514E+00_JPRB, 0.41612E+00_JPRB, 0.40453E+00_JPRB, 0.39214E+00_JPRB ,&
&  0.37939E+00_JPRB, 0.36896E+00_JPRB, 0.35633E+00_JPRB, 0.35088E+00_JPRB, 0.34806E+00_JPRB, 0.34666E+00_JPRB, 0.34443E+00_JPRB ,&
&  0.33991E+00_JPRB, 0.33294E+00_JPRB, 0.32765E+00_JPRB, 0.32545E+00_JPRB, 0.32591E+00_JPRB, 0.32711E+00_JPRB, 0.32843E+00_JPRB ,&
&  0.32759E+00_JPRB, 0.32708E+00_JPRB, 0.32623E+00_JPRB, 0.32605E+00_JPRB, 0.32771E+00_JPRB, 0.33544E+00_JPRB, 0.35094E+00_JPRB ,&
&  0.38995E+00_JPRB, 0.35656E+00_JPRB, 0.36891E+00_JPRB, 0.39034E+00_JPRB, 0.41417E+00_JPRB, 0.44829E+00_JPRB, 0.48316E+00_JPRB ,&
&  0.49997E+00_JPRB, 0.49723E+00_JPRB, 0.50129E+00_JPRB, 0.51318E+00_JPRB, 0.50817E+00_JPRB, 0.49952E+00_JPRB, 0.48670E+00_JPRB ,&
&  0.46871E+00_JPRB, 0.44838E+00_JPRB, 0.42823E+00_JPRB, 0.41483E+00_JPRB, 0.40951E+00_JPRB, 0.40720E+00_JPRB, 0.40618E+00_JPRB ,&
&  0.40011E+00_JPRB, 0.39750E+00_JPRB, 0.40404E+00_JPRB, 0.41336E+00_JPRB, 0.42458E+00_JPRB, 0.44460E+00_JPRB, 0.47358E+00_JPRB ,&
&  0.49429E+00_JPRB, 0.51663E+00_JPRB, 0.53489E+00_JPRB, 0.54472E+00_JPRB, 0.55173E+00_JPRB, 0.54839E+00_JPRB, 0.53211E+00_JPRB ,&
&  0.51451E+00_JPRB, 0.49560E+00_JPRB, 0.47862E+00_JPRB, 0.46356E+00_JPRB, 0.45224E+00_JPRB, 0.44680E+00_JPRB, 0.44326E+00_JPRB ,&
&  0.44024E+00_JPRB, 0.43769E+00_JPRB, 0.43254E+00_JPRB, 0.42581E+00_JPRB, 0.41967E+00_JPRB, 0.41499E+00_JPRB, 0.41068E+00_JPRB ,&
&  0.40680E+00_JPRB, 0.40276E+00_JPRB, 0.40171E+00_JPRB, 0.40425E+00_JPRB, 0.40638E+00_JPRB, 0.40972E+00_JPRB, 0.41449E+00_JPRB ,&
&  0.41813E+00_JPRB, 0.42849E+00_JPRB, 0.43186E+00_JPRB, 0.49676E+00_JPRB, 0.50839E+00_JPRB, 0.50490E+00_JPRB, 0.49855E+00_JPRB ,&
&  0.48636E+00_JPRB, 0.47452E+00_JPRB, 0.46495E+00_JPRB, 0.43628E+00_JPRB, 0.46384E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.52003E+00_JPRB, 0.51034E+00_JPRB, 0.48744E+00_JPRB, 0.46025E+00_JPRB, 0.43961E+00_JPRB, 0.42213E+00_JPRB, 0.40790E+00_JPRB ,&
&  0.39852E+00_JPRB, 0.38766E+00_JPRB, 0.37524E+00_JPRB, 0.36634E+00_JPRB, 0.36461E+00_JPRB, 0.36171E+00_JPRB, 0.35750E+00_JPRB ,&
&  0.35044E+00_JPRB, 0.34377E+00_JPRB, 0.33932E+00_JPRB, 0.33770E+00_JPRB, 0.33803E+00_JPRB, 0.33972E+00_JPRB, 0.34058E+00_JPRB ,&
&  0.33896E+00_JPRB, 0.33549E+00_JPRB, 0.33567E+00_JPRB, 0.33819E+00_JPRB, 0.34347E+00_JPRB, 0.35164E+00_JPRB, 0.37081E+00_JPRB ,&
&  0.41537E+00_JPRB, 0.39695E+00_JPRB, 0.41742E+00_JPRB, 0.42941E+00_JPRB, 0.44743E+00_JPRB, 0.48473E+00_JPRB, 0.52411E+00_JPRB ,&
&  0.54834E+00_JPRB, 0.55935E+00_JPRB, 0.58104E+00_JPRB, 0.60034E+00_JPRB, 0.59456E+00_JPRB, 0.57466E+00_JPRB, 0.54955E+00_JPRB ,&
&  0.52468E+00_JPRB, 0.50591E+00_JPRB, 0.48989E+00_JPRB, 0.47723E+00_JPRB, 0.46916E+00_JPRB, 0.46599E+00_JPRB, 0.46629E+00_JPRB ,&
&  0.46138E+00_JPRB, 0.46175E+00_JPRB, 0.47305E+00_JPRB, 0.49418E+00_JPRB, 0.51732E+00_JPRB, 0.54111E+00_JPRB, 0.56660E+00_JPRB ,&
&  0.58967E+00_JPRB, 0.60965E+00_JPRB, 0.62292E+00_JPRB, 0.62701E+00_JPRB, 0.61829E+00_JPRB, 0.60465E+00_JPRB, 0.57448E+00_JPRB ,&
&  0.54431E+00_JPRB, 0.52017E+00_JPRB, 0.50633E+00_JPRB, 0.49625E+00_JPRB, 0.48908E+00_JPRB, 0.48456E+00_JPRB, 0.48141E+00_JPRB ,&
&  0.48023E+00_JPRB, 0.48155E+00_JPRB, 0.48208E+00_JPRB, 0.48167E+00_JPRB, 0.48129E+00_JPRB, 0.48145E+00_JPRB, 0.48100E+00_JPRB ,&
&  0.48081E+00_JPRB, 0.48124E+00_JPRB, 0.48207E+00_JPRB, 0.48073E+00_JPRB, 0.47816E+00_JPRB, 0.47561E+00_JPRB, 0.47432E+00_JPRB ,&
&  0.47409E+00_JPRB, 0.48520E+00_JPRB, 0.49128E+00_JPRB, 0.52745E+00_JPRB, 0.53951E+00_JPRB, 0.54272E+00_JPRB, 0.54060E+00_JPRB ,&
&  0.52881E+00_JPRB, 0.51499E+00_JPRB, 0.49459E+00_JPRB, 0.48344E+00_JPRB, 0.49929E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.60955E+00_JPRB, 0.57101E+00_JPRB, 0.52392E+00_JPRB, 0.48757E+00_JPRB, 0.46172E+00_JPRB, 0.44252E+00_JPRB, 0.43108E+00_JPRB ,&
&  0.42633E+00_JPRB, 0.41505E+00_JPRB, 0.40014E+00_JPRB, 0.38789E+00_JPRB, 0.38177E+00_JPRB, 0.37713E+00_JPRB, 0.37071E+00_JPRB ,&
&  0.36522E+00_JPRB, 0.36023E+00_JPRB, 0.35694E+00_JPRB, 0.35555E+00_JPRB, 0.35471E+00_JPRB, 0.35350E+00_JPRB, 0.35012E+00_JPRB ,&
&  0.34501E+00_JPRB, 0.34416E+00_JPRB, 0.34591E+00_JPRB, 0.35263E+00_JPRB, 0.36161E+00_JPRB, 0.37209E+00_JPRB, 0.39082E+00_JPRB ,&
&  0.44507E+00_JPRB, 0.44568E+00_JPRB, 0.48405E+00_JPRB, 0.49636E+00_JPRB, 0.51573E+00_JPRB, 0.55649E+00_JPRB, 0.58754E+00_JPRB ,&
&  0.61759E+00_JPRB, 0.64898E+00_JPRB, 0.68882E+00_JPRB, 0.70362E+00_JPRB, 0.68376E+00_JPRB, 0.65227E+00_JPRB, 0.62482E+00_JPRB ,&
&  0.59780E+00_JPRB, 0.57815E+00_JPRB, 0.55912E+00_JPRB, 0.54154E+00_JPRB, 0.52978E+00_JPRB, 0.52857E+00_JPRB, 0.53477E+00_JPRB ,&
&  0.53853E+00_JPRB, 0.53176E+00_JPRB, 0.54745E+00_JPRB, 0.57106E+00_JPRB, 0.60788E+00_JPRB, 0.64026E+00_JPRB, 0.66867E+00_JPRB ,&
&  0.69751E+00_JPRB, 0.71016E+00_JPRB, 0.71418E+00_JPRB, 0.70051E+00_JPRB, 0.66257E+00_JPRB, 0.61854E+00_JPRB, 0.57736E+00_JPRB ,&
&  0.53935E+00_JPRB, 0.51518E+00_JPRB, 0.50450E+00_JPRB, 0.49964E+00_JPRB, 0.49955E+00_JPRB, 0.50294E+00_JPRB, 0.51125E+00_JPRB ,&
&  0.52458E+00_JPRB, 0.53656E+00_JPRB, 0.54488E+00_JPRB, 0.54754E+00_JPRB, 0.54490E+00_JPRB, 0.54114E+00_JPRB, 0.53560E+00_JPRB ,&
&  0.52944E+00_JPRB, 0.52213E+00_JPRB, 0.51789E+00_JPRB, 0.51431E+00_JPRB, 0.51198E+00_JPRB, 0.51135E+00_JPRB, 0.51240E+00_JPRB ,&
&  0.51513E+00_JPRB, 0.53364E+00_JPRB, 0.53650E+00_JPRB, 0.60134E+00_JPRB, 0.62808E+00_JPRB, 0.61406E+00_JPRB, 0.59504E+00_JPRB ,&
&  0.57168E+00_JPRB, 0.55097E+00_JPRB, 0.53309E+00_JPRB, 0.52628E+00_JPRB, 0.59741E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.55883E+00_JPRB, 0.54141E+00_JPRB, 0.51254E+00_JPRB, 0.48734E+00_JPRB, 0.47098E+00_JPRB, 0.45919E+00_JPRB, 0.45259E+00_JPRB ,&
&  0.44872E+00_JPRB, 0.43975E+00_JPRB, 0.43084E+00_JPRB, 0.41886E+00_JPRB, 0.40882E+00_JPRB, 0.40024E+00_JPRB, 0.39209E+00_JPRB ,&
&  0.38496E+00_JPRB, 0.38040E+00_JPRB, 0.37650E+00_JPRB, 0.37281E+00_JPRB, 0.36746E+00_JPRB, 0.35981E+00_JPRB, 0.35516E+00_JPRB ,&
&  0.35337E+00_JPRB, 0.35447E+00_JPRB, 0.35893E+00_JPRB, 0.36770E+00_JPRB, 0.37696E+00_JPRB, 0.38783E+00_JPRB, 0.40558E+00_JPRB ,&
&  0.45113E+00_JPRB, 0.56959E+00_JPRB, 0.61064E+00_JPRB, 0.61726E+00_JPRB, 0.65222E+00_JPRB, 0.65203E+00_JPRB, 0.67090E+00_JPRB ,&
&  0.71871E+00_JPRB, 0.76215E+00_JPRB, 0.80041E+00_JPRB, 0.81399E+00_JPRB, 0.79536E+00_JPRB, 0.77482E+00_JPRB, 0.74383E+00_JPRB ,&
&  0.70896E+00_JPRB, 0.66964E+00_JPRB, 0.62961E+00_JPRB, 0.59217E+00_JPRB, 0.56067E+00_JPRB, 0.55556E+00_JPRB, 0.56836E+00_JPRB ,&
&  0.59249E+00_JPRB, 0.60998E+00_JPRB, 0.61937E+00_JPRB, 0.65040E+00_JPRB, 0.71393E+00_JPRB, 0.78415E+00_JPRB, 0.84899E+00_JPRB ,&
&  0.87522E+00_JPRB, 0.84990E+00_JPRB, 0.80784E+00_JPRB, 0.75191E+00_JPRB, 0.68008E+00_JPRB, 0.61332E+00_JPRB, 0.56465E+00_JPRB ,&
&  0.53067E+00_JPRB, 0.50751E+00_JPRB, 0.49283E+00_JPRB, 0.48715E+00_JPRB, 0.49207E+00_JPRB, 0.50699E+00_JPRB, 0.52522E+00_JPRB ,&
&  0.54554E+00_JPRB, 0.56370E+00_JPRB, 0.57270E+00_JPRB, 0.57446E+00_JPRB, 0.56791E+00_JPRB, 0.55441E+00_JPRB, 0.53341E+00_JPRB ,&
&  0.51459E+00_JPRB, 0.50278E+00_JPRB, 0.51065E+00_JPRB, 0.51386E+00_JPRB, 0.51615E+00_JPRB, 0.51991E+00_JPRB, 0.52585E+00_JPRB ,&
&  0.53793E+00_JPRB, 0.57167E+00_JPRB, 0.57331E+00_JPRB, 0.58432E+00_JPRB, 0.61834E+00_JPRB, 0.63843E+00_JPRB, 0.61451E+00_JPRB ,&
&  0.59303E+00_JPRB, 0.57095E+00_JPRB, 0.55544E+00_JPRB, 0.54452E+00_JPRB, 0.56205E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.54920E+00_JPRB, 0.54001E+00_JPRB, 0.52172E+00_JPRB, 0.50239E+00_JPRB, 0.48926E+00_JPRB, 0.47911E+00_JPRB, 0.47188E+00_JPRB ,&
&  0.46714E+00_JPRB, 0.45850E+00_JPRB, 0.45059E+00_JPRB, 0.44073E+00_JPRB, 0.42912E+00_JPRB, 0.41932E+00_JPRB, 0.41291E+00_JPRB ,&
&  0.40450E+00_JPRB, 0.39953E+00_JPRB, 0.39427E+00_JPRB, 0.38504E+00_JPRB, 0.37535E+00_JPRB, 0.36745E+00_JPRB, 0.36240E+00_JPRB ,&
&  0.36048E+00_JPRB, 0.36326E+00_JPRB, 0.36956E+00_JPRB, 0.38239E+00_JPRB, 0.39389E+00_JPRB, 0.40556E+00_JPRB, 0.42552E+00_JPRB ,&
&  0.49120E+00_JPRB, 0.66363E+00_JPRB, 0.82528E+00_JPRB, 0.75829E+00_JPRB, 0.72968E+00_JPRB, 0.69388E+00_JPRB, 0.76978E+00_JPRB ,&
&  0.83486E+00_JPRB, 0.90516E+00_JPRB, 0.94679E+00_JPRB, 0.96682E+00_JPRB, 0.92933E+00_JPRB, 0.89611E+00_JPRB, 0.85760E+00_JPRB ,&
&  0.81562E+00_JPRB, 0.76716E+00_JPRB, 0.70349E+00_JPRB, 0.64160E+00_JPRB, 0.60564E+00_JPRB, 0.60332E+00_JPRB, 0.62495E+00_JPRB ,&
&  0.65910E+00_JPRB, 0.68703E+00_JPRB, 0.70101E+00_JPRB, 0.74491E+00_JPRB, 0.79135E+00_JPRB, 0.82233E+00_JPRB, 0.95690E+00_JPRB ,&
&  0.11792E+01_JPRB, 0.10997E+01_JPRB, 0.97343E+00_JPRB, 0.82165E+00_JPRB, 0.70678E+00_JPRB, 0.62044E+00_JPRB, 0.57268E+00_JPRB ,&
&  0.53967E+00_JPRB, 0.51783E+00_JPRB, 0.50362E+00_JPRB, 0.49800E+00_JPRB, 0.50553E+00_JPRB, 0.51977E+00_JPRB, 0.53057E+00_JPRB ,&
&  0.54338E+00_JPRB, 0.55467E+00_JPRB, 0.55924E+00_JPRB, 0.55509E+00_JPRB, 0.54365E+00_JPRB, 0.52705E+00_JPRB, 0.51173E+00_JPRB ,&
&  0.50571E+00_JPRB, 0.51039E+00_JPRB, 0.54026E+00_JPRB, 0.52792E+00_JPRB, 0.53089E+00_JPRB, 0.54345E+00_JPRB, 0.54777E+00_JPRB ,&
&  0.54909E+00_JPRB, 0.56524E+00_JPRB, 0.57880E+00_JPRB, 0.58614E+00_JPRB, 0.63477E+00_JPRB, 0.68023E+00_JPRB, 0.64214E+00_JPRB ,&
&  0.60447E+00_JPRB, 0.57620E+00_JPRB, 0.56209E+00_JPRB, 0.54804E+00_JPRB, 0.54833E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.55307E+00_JPRB, 0.54661E+00_JPRB, 0.53204E+00_JPRB, 0.51463E+00_JPRB, 0.50146E+00_JPRB, 0.48843E+00_JPRB, 0.47878E+00_JPRB ,&
&  0.47144E+00_JPRB, 0.46508E+00_JPRB, 0.46020E+00_JPRB, 0.45445E+00_JPRB, 0.44699E+00_JPRB, 0.43688E+00_JPRB, 0.42710E+00_JPRB ,&
&  0.41660E+00_JPRB, 0.40623E+00_JPRB, 0.39605E+00_JPRB, 0.38564E+00_JPRB, 0.37790E+00_JPRB, 0.37269E+00_JPRB, 0.36928E+00_JPRB ,&
&  0.36913E+00_JPRB, 0.37256E+00_JPRB, 0.38095E+00_JPRB, 0.39750E+00_JPRB, 0.41625E+00_JPRB, 0.42752E+00_JPRB, 0.43944E+00_JPRB ,&
&  0.49288E+00_JPRB, 0.44159E+00_JPRB, 0.74768E+00_JPRB, 0.84972E+00_JPRB, 0.83632E+00_JPRB, 0.84683E+00_JPRB, 0.96267E+00_JPRB ,&
&  0.10500E+01_JPRB, 0.10711E+01_JPRB, 0.10711E+01_JPRB, 0.10370E+01_JPRB, 0.94794E+00_JPRB, 0.87548E+00_JPRB, 0.81747E+00_JPRB ,&
&  0.77032E+00_JPRB, 0.72901E+00_JPRB, 0.68098E+00_JPRB, 0.63326E+00_JPRB, 0.59888E+00_JPRB, 0.59315E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.65952E+00_JPRB, 0.70931E+00_JPRB, 0.76667E+00_JPRB, 0.82989E+00_JPRB, 0.83946E+00_JPRB, 0.95303E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.13189E+01_JPRB, 0.10777E+01_JPRB, 0.84453E+00_JPRB, 0.69570E+00_JPRB, 0.62270E+00_JPRB, 0.57103E+00_JPRB ,&
&  0.54480E+00_JPRB, 0.52991E+00_JPRB, 0.51989E+00_JPRB, 0.51629E+00_JPRB, 0.51998E+00_JPRB, 0.52611E+00_JPRB, 0.52835E+00_JPRB ,&
&  0.53164E+00_JPRB, 0.53465E+00_JPRB, 0.53506E+00_JPRB, 0.53388E+00_JPRB, 0.52783E+00_JPRB, 0.51887E+00_JPRB, 0.51600E+00_JPRB ,&
&  0.52306E+00_JPRB, 0.53953E+00_JPRB, 0.55518E+00_JPRB, 0.55265E+00_JPRB, 0.55779E+00_JPRB, 0.56532E+00_JPRB, 0.57110E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.58741E+00_JPRB, 0.59036E+00_JPRB, 0.61094E+00_JPRB, 0.61407E+00_JPRB, 0.66269E+00_JPRB, 0.62430E+00_JPRB ,&
&  0.59375E+00_JPRB, 0.57357E+00_JPRB, 0.56141E+00_JPRB, 0.55361E+00_JPRB, 0.55158E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.56044E+00_JPRB, 0.55401E+00_JPRB, 0.54016E+00_JPRB, 0.52289E+00_JPRB, 0.50540E+00_JPRB, 0.49095E+00_JPRB, 0.48417E+00_JPRB ,&
&  0.47654E+00_JPRB, 0.46859E+00_JPRB, 0.46123E+00_JPRB, 0.45719E+00_JPRB, 0.45525E+00_JPRB, 0.44730E+00_JPRB, 0.43357E+00_JPRB ,&
&  0.41902E+00_JPRB, 0.40535E+00_JPRB, 0.39422E+00_JPRB, 0.38816E+00_JPRB, 0.38163E+00_JPRB, 0.37847E+00_JPRB, 0.37854E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.38632E+00_JPRB, 0.39928E+00_JPRB, 0.42008E+00_JPRB, 0.43703E+00_JPRB, 0.44433E+00_JPRB, 0.45040E+00_JPRB ,&
&  0.49810E+00_JPRB, 0.71842E+00_JPRB, 0.68076E+00_JPRB, 0.98994E+00_JPRB, 0.10533E+01_JPRB, 0.10978E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.11636E+01_JPRB, 0.11905E+01_JPRB, 0.11429E+01_JPRB, 0.10513E+01_JPRB, 0.92871E+00_JPRB, 0.83550E+00_JPRB, 0.75558E+00_JPRB ,&
&  0.69070E+00_JPRB, 0.64399E+00_JPRB, 0.61018E+00_JPRB, 0.58504E+00_JPRB, 0.56870E+00_JPRB, 0.56939E+00_JPRB, 0.59853E+00_JPRB ,&
&  0.66562E+00_JPRB, 0.75713E+00_JPRB, 0.85335E+00_JPRB, 0.93621E+00_JPRB, 0.91321E+00_JPRB, 0.10372E+01_JPRB, 0.11436E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.13064E+01_JPRB, 0.10050E+01_JPRB, 0.76515E+00_JPRB, 0.66012E+00_JPRB, 0.61317E+00_JPRB, 0.58194E+00_JPRB ,&
&  0.56356E+00_JPRB, 0.55160E+00_JPRB, 0.54310E+00_JPRB, 0.53744E+00_JPRB, 0.53366E+00_JPRB, 0.53202E+00_JPRB, 0.53011E+00_JPRB ,&
&  0.52978E+00_JPRB, 0.53090E+00_JPRB, 0.53277E+00_JPRB, 0.53442E+00_JPRB, 0.53312E+00_JPRB, 0.53486E+00_JPRB, 0.54353E+00_JPRB ,&
&  0.55803E+00_JPRB, 0.57398E+00_JPRB, 0.58010E+00_JPRB, 0.57756E+00_JPRB, 0.58420E+00_JPRB, 0.59159E+00_JPRB, 0.58919E+00_JPRB ,&
&  0.60335E+00_JPRB, 0.61251E+00_JPRB, 0.61807E+00_JPRB, 0.63057E+00_JPRB, 0.64116E+00_JPRB, 0.70481E+00_JPRB, 0.62380E+00_JPRB ,&
&  0.59272E+00_JPRB, 0.57700E+00_JPRB, 0.57043E+00_JPRB, 0.56202E+00_JPRB, 0.56057E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.56415E+00_JPRB, 0.55047E+00_JPRB, 0.53556E+00_JPRB, 0.51961E+00_JPRB, 0.50241E+00_JPRB, 0.49030E+00_JPRB, 0.48443E+00_JPRB ,&
&  0.47459E+00_JPRB, 0.46589E+00_JPRB, 0.45945E+00_JPRB, 0.45679E+00_JPRB, 0.45439E+00_JPRB, 0.44612E+00_JPRB, 0.43089E+00_JPRB ,&
&  0.41560E+00_JPRB, 0.40536E+00_JPRB, 0.40001E+00_JPRB, 0.39442E+00_JPRB, 0.38838E+00_JPRB, 0.38514E+00_JPRB, 0.38643E+00_JPRB ,&
&  0.39258E+00_JPRB, 0.40231E+00_JPRB, 0.42167E+00_JPRB, 0.44360E+00_JPRB, 0.46162E+00_JPRB, 0.47634E+00_JPRB, 0.48647E+00_JPRB ,&
&  0.56848E+00_JPRB, 0.68465E+00_JPRB, 0.75098E+00_JPRB, 0.12629E+01_JPRB, 0.13354E+01_JPRB, 0.12983E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.13095E+01_JPRB, 0.11825E+01_JPRB, 0.10356E+01_JPRB, 0.89239E+00_JPRB, 0.79555E+00_JPRB, 0.71574E+00_JPRB ,&
&  0.65034E+00_JPRB, 0.61058E+00_JPRB, 0.59209E+00_JPRB, 0.58470E+00_JPRB, 0.58031E+00_JPRB, 0.59060E+00_JPRB, 0.62983E+00_JPRB ,&
&  0.70705E+00_JPRB, 0.81741E+00_JPRB, 0.96738E+00_JPRB, 0.10435E+01_JPRB, 0.10508E+01_JPRB, 0.11035E+01_JPRB, 0.11102E+01_JPRB ,&
&  0.12041E+01_JPRB, 0.11525E+01_JPRB, 0.87076E+00_JPRB, 0.71829E+00_JPRB, 0.64725E+00_JPRB, 0.62340E+00_JPRB, 0.59633E+00_JPRB ,&
&  0.58329E+00_JPRB, 0.57179E+00_JPRB, 0.56266E+00_JPRB, 0.55275E+00_JPRB, 0.54298E+00_JPRB, 0.53830E+00_JPRB, 0.53716E+00_JPRB ,&
&  0.53725E+00_JPRB, 0.54015E+00_JPRB, 0.54218E+00_JPRB, 0.54482E+00_JPRB, 0.54892E+00_JPRB, 0.55806E+00_JPRB, 0.57160E+00_JPRB ,&
&  0.58419E+00_JPRB, 0.60497E+00_JPRB, 0.60536E+00_JPRB, 0.60157E+00_JPRB, 0.61768E+00_JPRB, 0.62807E+00_JPRB, 0.61716E+00_JPRB ,&
&  0.62380E+00_JPRB, 0.64339E+00_JPRB, 0.64980E+00_JPRB, 0.63743E+00_JPRB, 0.65692E+00_JPRB, 0.68338E+00_JPRB, 0.61913E+00_JPRB ,&
&  0.59707E+00_JPRB, 0.58656E+00_JPRB, 0.58571E+00_JPRB, 0.57788E+00_JPRB, 0.57085E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.55965E+00_JPRB, 0.54211E+00_JPRB, 0.52772E+00_JPRB, 0.51699E+00_JPRB, 0.50370E+00_JPRB, 0.49019E+00_JPRB, 0.47988E+00_JPRB ,&
&  0.46483E+00_JPRB, 0.45562E+00_JPRB, 0.44923E+00_JPRB, 0.44187E+00_JPRB, 0.43442E+00_JPRB, 0.42731E+00_JPRB, 0.42103E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.41257E+00_JPRB, 0.41014E+00_JPRB, 0.40575E+00_JPRB, 0.40225E+00_JPRB, 0.40185E+00_JPRB, 0.40560E+00_JPRB ,&
&  0.42035E+00_JPRB, 0.44126E+00_JPRB, 0.46607E+00_JPRB, 0.49213E+00_JPRB, 0.51549E+00_JPRB, 0.54631E+00_JPRB, 0.58295E+00_JPRB ,&
&  0.72407E+00_JPRB, 0.78081E+00_JPRB, 0.93434E+00_JPRB, 0.14635E+01_JPRB, 0.15055E+01_JPRB, 0.13430E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.11475E+01_JPRB, 0.11297E+01_JPRB, 0.97170E+00_JPRB, 0.83787E+00_JPRB, 0.76660E+00_JPRB, 0.71417E+00_JPRB ,&
&  0.67019E+00_JPRB, 0.64457E+00_JPRB, 0.63740E+00_JPRB, 0.63020E+00_JPRB, 0.62333E+00_JPRB, 0.64739E+00_JPRB, 0.70634E+00_JPRB ,&
&  0.81756E+00_JPRB, 0.97334E+00_JPRB, 0.11809E+01_JPRB, 0.13117E+01_JPRB, 0.11573E+01_JPRB, 0.11103E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.96040E+00_JPRB, 0.78824E+00_JPRB, 0.70401E+00_JPRB, 0.64857E+00_JPRB, 0.63590E+00_JPRB, 0.60927E+00_JPRB ,&
&  0.59580E+00_JPRB, 0.58399E+00_JPRB, 0.57212E+00_JPRB, 0.55630E+00_JPRB, 0.54626E+00_JPRB, 0.53844E+00_JPRB, 0.53741E+00_JPRB ,&
&  0.54194E+00_JPRB, 0.54800E+00_JPRB, 0.55442E+00_JPRB, 0.56338E+00_JPRB, 0.57513E+00_JPRB, 0.58535E+00_JPRB, 0.59527E+00_JPRB ,&
&  0.60602E+00_JPRB, 0.63112E+00_JPRB, 0.63045E+00_JPRB, 0.64422E+00_JPRB, 0.67036E+00_JPRB, 0.68420E+00_JPRB, 0.66304E+00_JPRB ,&
&  0.65058E+00_JPRB, 0.66761E+00_JPRB, 0.66575E+00_JPRB, 0.66564E+00_JPRB, 0.67251E+00_JPRB, 0.67110E+00_JPRB, 0.62834E+00_JPRB ,&
&  0.62123E+00_JPRB, 0.62248E+00_JPRB, 0.61738E+00_JPRB, 0.59800E+00_JPRB, 0.57954E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.55476E+00_JPRB, 0.53181E+00_JPRB, 0.51466E+00_JPRB, 0.49830E+00_JPRB, 0.48122E+00_JPRB, 0.46358E+00_JPRB, 0.44980E+00_JPRB ,&
&  0.43349E+00_JPRB, 0.43097E+00_JPRB, 0.42774E+00_JPRB, 0.42273E+00_JPRB, 0.41673E+00_JPRB, 0.41738E+00_JPRB, 0.42413E+00_JPRB ,&
&  0.42995E+00_JPRB, 0.43264E+00_JPRB, 0.43634E+00_JPRB, 0.43920E+00_JPRB, 0.44498E+00_JPRB, 0.45662E+00_JPRB, 0.47445E+00_JPRB ,&
&  0.49872E+00_JPRB, 0.52525E+00_JPRB, 0.56024E+00_JPRB, 0.60263E+00_JPRB, 0.64450E+00_JPRB, 0.70706E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.95680E+00_JPRB, 0.95849E+00_JPRB, 0.62228E+00_JPRB, 0.10201E+01_JPRB, 0.12601E+01_JPRB, 0.11863E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.10301E+01_JPRB, 0.98811E+00_JPRB, 0.90604E+00_JPRB, 0.81260E+00_JPRB, 0.78826E+00_JPRB, 0.77320E+00_JPRB ,&
&  0.74818E+00_JPRB, 0.72126E+00_JPRB, 0.70859E+00_JPRB, 0.70477E+00_JPRB, 0.70318E+00_JPRB, 0.73802E+00_JPRB, 0.85354E+00_JPRB ,&
&  0.10147E+01_JPRB, 0.12367E+01_JPRB, 0.15037E+01_JPRB, 0.14644E+01_JPRB, 0.12213E+01_JPRB, 0.10785E+01_JPRB, 0.94033E+00_JPRB ,&
&  0.87315E+00_JPRB, 0.84209E+00_JPRB, 0.75697E+00_JPRB, 0.70622E+00_JPRB, 0.65560E+00_JPRB, 0.61906E+00_JPRB, 0.61596E+00_JPRB ,&
&  0.59942E+00_JPRB, 0.58087E+00_JPRB, 0.56624E+00_JPRB, 0.54499E+00_JPRB, 0.53648E+00_JPRB, 0.53353E+00_JPRB, 0.54025E+00_JPRB ,&
&  0.55042E+00_JPRB, 0.56027E+00_JPRB, 0.57234E+00_JPRB, 0.58666E+00_JPRB, 0.59593E+00_JPRB, 0.60530E+00_JPRB, 0.61351E+00_JPRB ,&
&  0.62835E+00_JPRB, 0.65278E+00_JPRB, 0.67955E+00_JPRB, 0.69820E+00_JPRB, 0.71155E+00_JPRB, 0.71885E+00_JPRB, 0.71688E+00_JPRB ,&
&  0.72125E+00_JPRB, 0.71919E+00_JPRB, 0.67107E+00_JPRB, 0.66507E+00_JPRB, 0.69151E+00_JPRB, 0.68952E+00_JPRB, 0.68731E+00_JPRB ,&
&  0.72199E+00_JPRB, 0.75070E+00_JPRB, 0.73822E+00_JPRB, 0.62679E+00_JPRB, 0.58465E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.51561E+00_JPRB, 0.48511E+00_JPRB, 0.45434E+00_JPRB, 0.43821E+00_JPRB, 0.42700E+00_JPRB, 0.42236E+00_JPRB, 0.42096E+00_JPRB ,&
&  0.42362E+00_JPRB, 0.42562E+00_JPRB, 0.41781E+00_JPRB, 0.42247E+00_JPRB, 0.43441E+00_JPRB, 0.44756E+00_JPRB, 0.46491E+00_JPRB ,&
&  0.47456E+00_JPRB, 0.48286E+00_JPRB, 0.49490E+00_JPRB, 0.51730E+00_JPRB, 0.54611E+00_JPRB, 0.57618E+00_JPRB, 0.60411E+00_JPRB ,&
&  0.63336E+00_JPRB, 0.66660E+00_JPRB, 0.71283E+00_JPRB, 0.77590E+00_JPRB, 0.86237E+00_JPRB, 0.98175E+00_JPRB, 0.11132E+01_JPRB ,&
&  0.12754E+01_JPRB, 0.97597E+00_JPRB, 0.72193E+00_JPRB, 0.11366E+01_JPRB, 0.11967E+01_JPRB, 0.11314E+01_JPRB, 0.99037E+00_JPRB ,&
&  0.98002E+00_JPRB, 0.98766E+00_JPRB, 0.93851E+00_JPRB, 0.90950E+00_JPRB, 0.84516E+00_JPRB, 0.83746E+00_JPRB, 0.83419E+00_JPRB ,&
&  0.81662E+00_JPRB, 0.79567E+00_JPRB, 0.81159E+00_JPRB, 0.83019E+00_JPRB, 0.85536E+00_JPRB, 0.94541E+00_JPRB, 0.11119E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.16052E+01_JPRB, 0.19680E+01_JPRB, 0.17244E+01_JPRB, 0.13694E+01_JPRB, 0.10858E+01_JPRB, 0.93413E+00_JPRB ,&
&  0.83210E+00_JPRB, 0.80424E+00_JPRB, 0.75813E+00_JPRB, 0.71344E+00_JPRB, 0.66459E+00_JPRB, 0.61683E+00_JPRB, 0.60029E+00_JPRB ,&
&  0.58241E+00_JPRB, 0.55947E+00_JPRB, 0.54331E+00_JPRB, 0.52707E+00_JPRB, 0.52598E+00_JPRB, 0.53328E+00_JPRB, 0.54233E+00_JPRB ,&
&  0.55247E+00_JPRB, 0.56443E+00_JPRB, 0.58272E+00_JPRB, 0.60456E+00_JPRB, 0.61560E+00_JPRB, 0.62237E+00_JPRB, 0.63545E+00_JPRB ,&
&  0.64993E+00_JPRB, 0.66429E+00_JPRB, 0.69132E+00_JPRB, 0.70918E+00_JPRB, 0.71409E+00_JPRB, 0.69688E+00_JPRB, 0.70868E+00_JPRB ,&
&  0.70823E+00_JPRB, 0.71192E+00_JPRB, 0.69188E+00_JPRB, 0.70233E+00_JPRB, 0.69810E+00_JPRB, 0.71596E+00_JPRB, 0.73759E+00_JPRB ,&
&  0.75536E+00_JPRB, 0.74725E+00_JPRB, 0.70482E+00_JPRB, 0.59227E+00_JPRB, 0.54865E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.43405E+00_JPRB, 0.41321E+00_JPRB, 0.41512E+00_JPRB, 0.41959E+00_JPRB, 0.43386E+00_JPRB, 0.44756E+00_JPRB, 0.44296E+00_JPRB ,&
&  0.44873E+00_JPRB, 0.45683E+00_JPRB, 0.46947E+00_JPRB, 0.48380E+00_JPRB, 0.50219E+00_JPRB, 0.52111E+00_JPRB, 0.53475E+00_JPRB ,&
&  0.55746E+00_JPRB, 0.58693E+00_JPRB, 0.62326E+00_JPRB, 0.65947E+00_JPRB, 0.69500E+00_JPRB, 0.72202E+00_JPRB, 0.74739E+00_JPRB ,&
&  0.77971E+00_JPRB, 0.83371E+00_JPRB, 0.90858E+00_JPRB, 0.99791E+00_JPRB, 0.11086E+01_JPRB, 0.12406E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.99893E+00_JPRB, 0.59314E+00_JPRB, 0.11362E+01_JPRB, 0.12531E+01_JPRB, 0.11040E+01_JPRB, 0.10387E+01_JPRB, 0.97294E+00_JPRB ,&
&  0.92529E+00_JPRB, 0.92835E+00_JPRB, 0.88881E+00_JPRB, 0.90988E+00_JPRB, 0.87786E+00_JPRB, 0.88265E+00_JPRB, 0.89786E+00_JPRB ,&
&  0.90655E+00_JPRB, 0.93083E+00_JPRB, 0.98953E+00_JPRB, 0.10377E+01_JPRB, 0.11137E+01_JPRB, 0.12512E+01_JPRB, 0.14544E+01_JPRB ,&
&  0.17446E+01_JPRB, 0.21136E+01_JPRB, 0.25166E+01_JPRB, 0.19944E+01_JPRB, 0.15739E+01_JPRB, 0.13269E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.97277E+00_JPRB, 0.77895E+00_JPRB, 0.74472E+00_JPRB, 0.71732E+00_JPRB, 0.66863E+00_JPRB, 0.59592E+00_JPRB, 0.57704E+00_JPRB ,&
&  0.54555E+00_JPRB, 0.52625E+00_JPRB, 0.51545E+00_JPRB, 0.50743E+00_JPRB, 0.50577E+00_JPRB, 0.51322E+00_JPRB, 0.52332E+00_JPRB ,&
&  0.53884E+00_JPRB, 0.54885E+00_JPRB, 0.56685E+00_JPRB, 0.58804E+00_JPRB, 0.60346E+00_JPRB, 0.61891E+00_JPRB, 0.64094E+00_JPRB ,&
&  0.65074E+00_JPRB, 0.66873E+00_JPRB, 0.68606E+00_JPRB, 0.69126E+00_JPRB, 0.69528E+00_JPRB, 0.68231E+00_JPRB, 0.68139E+00_JPRB ,&
&  0.67994E+00_JPRB, 0.67586E+00_JPRB, 0.66231E+00_JPRB, 0.66721E+00_JPRB, 0.67022E+00_JPRB, 0.67034E+00_JPRB, 0.68609E+00_JPRB ,&
&  0.67583E+00_JPRB, 0.64245E+00_JPRB, 0.57613E+00_JPRB, 0.50546E+00_JPRB, 0.47052E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.41951E+00_JPRB, 0.43818E+00_JPRB, 0.45824E+00_JPRB, 0.47716E+00_JPRB, 0.49382E+00_JPRB, 0.50838E+00_JPRB, 0.53181E+00_JPRB ,&
&  0.55772E+00_JPRB, 0.57458E+00_JPRB, 0.58046E+00_JPRB, 0.59589E+00_JPRB, 0.61417E+00_JPRB, 0.63837E+00_JPRB, 0.65814E+00_JPRB ,&
&  0.68332E+00_JPRB, 0.71896E+00_JPRB, 0.75094E+00_JPRB, 0.78453E+00_JPRB, 0.81541E+00_JPRB, 0.84618E+00_JPRB, 0.89113E+00_JPRB ,&
&  0.94926E+00_JPRB, 0.10266E+01_JPRB, 0.11166E+01_JPRB, 0.12001E+01_JPRB, 0.12715E+01_JPRB, 0.13489E+01_JPRB, 0.13647E+01_JPRB ,&
&  0.88186E+00_JPRB, 0.11003E+01_JPRB, 0.12356E+01_JPRB, 0.11307E+01_JPRB, 0.10480E+01_JPRB, 0.10024E+01_JPRB, 0.98540E+00_JPRB ,&
&  0.94788E+00_JPRB, 0.88943E+00_JPRB, 0.87556E+00_JPRB, 0.93900E+00_JPRB, 0.90861E+00_JPRB, 0.93274E+00_JPRB, 0.97626E+00_JPRB ,&
&  0.10413E+01_JPRB, 0.11202E+01_JPRB, 0.12004E+01_JPRB, 0.13064E+01_JPRB, 0.14570E+01_JPRB, 0.16491E+01_JPRB, 0.18939E+01_JPRB ,&
&  0.22020E+01_JPRB, 0.25660E+01_JPRB, 0.29173E+01_JPRB, 0.23628E+01_JPRB, 0.21661E+01_JPRB, 0.21096E+01_JPRB, 0.18418E+01_JPRB ,&
&  0.11453E+01_JPRB, 0.79031E+00_JPRB, 0.78007E+00_JPRB, 0.73993E+00_JPRB, 0.65659E+00_JPRB, 0.58998E+00_JPRB, 0.53311E+00_JPRB ,&
&  0.50123E+00_JPRB, 0.47664E+00_JPRB, 0.46416E+00_JPRB, 0.45708E+00_JPRB, 0.45626E+00_JPRB, 0.46930E+00_JPRB, 0.48619E+00_JPRB ,&
&  0.50231E+00_JPRB, 0.50622E+00_JPRB, 0.52790E+00_JPRB, 0.54186E+00_JPRB, 0.58088E+00_JPRB, 0.61670E+00_JPRB, 0.63641E+00_JPRB ,&
&  0.64754E+00_JPRB, 0.64421E+00_JPRB, 0.64187E+00_JPRB, 0.63841E+00_JPRB, 0.63660E+00_JPRB, 0.63353E+00_JPRB, 0.62820E+00_JPRB ,&
&  0.61924E+00_JPRB, 0.62933E+00_JPRB, 0.61763E+00_JPRB, 0.61066E+00_JPRB, 0.61692E+00_JPRB, 0.59210E+00_JPRB, 0.57616E+00_JPRB ,&
&  0.55212E+00_JPRB, 0.50182E+00_JPRB, 0.46579E+00_JPRB, 0.44045E+00_JPRB, 0.42405E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.50321E+00_JPRB, 0.54721E+00_JPRB, 0.56555E+00_JPRB, 0.58820E+00_JPRB, 0.60387E+00_JPRB, 0.62621E+00_JPRB, 0.64554E+00_JPRB ,&
&  0.66337E+00_JPRB, 0.68466E+00_JPRB, 0.69543E+00_JPRB, 0.71931E+00_JPRB, 0.73893E+00_JPRB, 0.75639E+00_JPRB, 0.77171E+00_JPRB ,&
&  0.78964E+00_JPRB, 0.82013E+00_JPRB, 0.85281E+00_JPRB, 0.89021E+00_JPRB, 0.92625E+00_JPRB, 0.97276E+00_JPRB, 0.10376E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.11892E+01_JPRB, 0.12536E+01_JPRB, 0.13043E+01_JPRB, 0.13255E+01_JPRB, 0.13732E+01_JPRB, 0.13625E+01_JPRB ,&
&  0.99084E+00_JPRB, 0.11678E+01_JPRB, 0.11300E+01_JPRB, 0.10814E+01_JPRB, 0.10451E+01_JPRB, 0.10101E+01_JPRB, 0.99155E+00_JPRB ,&
&  0.97070E+00_JPRB, 0.91115E+00_JPRB, 0.90206E+00_JPRB, 0.91605E+00_JPRB, 0.95940E+00_JPRB, 0.10017E+01_JPRB, 0.10705E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.12919E+01_JPRB, 0.14246E+01_JPRB, 0.15648E+01_JPRB, 0.17473E+01_JPRB, 0.19794E+01_JPRB, 0.21889E+01_JPRB ,&
&  0.24097E+01_JPRB, 0.26449E+01_JPRB, 0.27936E+01_JPRB, 0.23668E+01_JPRB, 0.19653E+01_JPRB, 0.17853E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.95674E+00_JPRB, 0.78713E+00_JPRB, 0.79890E+00_JPRB, 0.76246E+00_JPRB, 0.65160E+00_JPRB, 0.55869E+00_JPRB, 0.48976E+00_JPRB ,&
&  0.46326E+00_JPRB, 0.43132E+00_JPRB, 0.41792E+00_JPRB, 0.41752E+00_JPRB, 0.42226E+00_JPRB, 0.43054E+00_JPRB, 0.45275E+00_JPRB ,&
&  0.46761E+00_JPRB, 0.48340E+00_JPRB, 0.50643E+00_JPRB, 0.53311E+00_JPRB, 0.58083E+00_JPRB, 0.65488E+00_JPRB, 0.66085E+00_JPRB ,&
&  0.63801E+00_JPRB, 0.60781E+00_JPRB, 0.58195E+00_JPRB, 0.58136E+00_JPRB, 0.57869E+00_JPRB, 0.58130E+00_JPRB, 0.59650E+00_JPRB ,&
&  0.61971E+00_JPRB, 0.63698E+00_JPRB, 0.60527E+00_JPRB, 0.58284E+00_JPRB, 0.58876E+00_JPRB, 0.55801E+00_JPRB, 0.50249E+00_JPRB ,&
&  0.48065E+00_JPRB, 0.46250E+00_JPRB, 0.44621E+00_JPRB, 0.45390E+00_JPRB, 0.47575E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.65899E+00_JPRB, 0.68579E+00_JPRB, 0.70500E+00_JPRB, 0.70664E+00_JPRB, 0.73335E+00_JPRB, 0.75347E+00_JPRB, 0.76531E+00_JPRB ,&
&  0.77556E+00_JPRB, 0.79441E+00_JPRB, 0.80585E+00_JPRB, 0.83022E+00_JPRB, 0.84900E+00_JPRB, 0.86887E+00_JPRB, 0.88621E+00_JPRB ,&
&  0.90507E+00_JPRB, 0.93787E+00_JPRB, 0.97440E+00_JPRB, 0.10246E+01_JPRB, 0.10733E+01_JPRB, 0.11331E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.12198E+01_JPRB, 0.12643E+01_JPRB, 0.13205E+01_JPRB, 0.13533E+01_JPRB, 0.13342E+01_JPRB, 0.13101E+01_JPRB, 0.99272E+00_JPRB ,&
&  0.11089E+01_JPRB, 0.11227E+01_JPRB, 0.11097E+01_JPRB, 0.10844E+01_JPRB, 0.10530E+01_JPRB, 0.10075E+01_JPRB, 0.98024E+00_JPRB ,&
&  0.96331E+00_JPRB, 0.93977E+00_JPRB, 0.92034E+00_JPRB, 0.91334E+00_JPRB, 0.10109E+01_JPRB, 0.10575E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.13453E+01_JPRB, 0.14921E+01_JPRB, 0.16859E+01_JPRB, 0.18912E+01_JPRB, 0.20992E+01_JPRB, 0.23065E+01_JPRB ,&
&  0.24582E+01_JPRB, 0.25278E+01_JPRB, 0.25500E+01_JPRB, 0.20337E+01_JPRB, 0.17034E+01_JPRB, 0.13894E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.89053E+00_JPRB, 0.83399E+00_JPRB, 0.85075E+00_JPRB, 0.79761E+00_JPRB, 0.67049E+00_JPRB, 0.56046E+00_JPRB, 0.48296E+00_JPRB ,&
&  0.45036E+00_JPRB, 0.41354E+00_JPRB, 0.40684E+00_JPRB, 0.40578E+00_JPRB, 0.41351E+00_JPRB, 0.42032E+00_JPRB, 0.43019E+00_JPRB ,&
&  0.44418E+00_JPRB, 0.46841E+00_JPRB, 0.51351E+00_JPRB, 0.58020E+00_JPRB, 0.66891E+00_JPRB, 0.76342E+00_JPRB, 0.73048E+00_JPRB ,&
&  0.62585E+00_JPRB, 0.56619E+00_JPRB, 0.57762E+00_JPRB, 0.58960E+00_JPRB, 0.62412E+00_JPRB, 0.68352E+00_JPRB, 0.75600E+00_JPRB ,&
&  0.88050E+00_JPRB, 0.95679E+00_JPRB, 0.87048E+00_JPRB, 0.72927E+00_JPRB, 0.10096E+01_JPRB, 0.99008E+00_JPRB, 0.52361E+00_JPRB ,&
&  0.57053E+00_JPRB, 0.56106E+00_JPRB, 0.58419E+00_JPRB, 0.61151E+00_JPRB, 0.64546E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.80145E+00_JPRB, 0.82693E+00_JPRB, 0.83925E+00_JPRB, 0.83965E+00_JPRB, 0.85513E+00_JPRB, 0.88272E+00_JPRB, 0.89385E+00_JPRB ,&
&  0.90634E+00_JPRB, 0.90718E+00_JPRB, 0.92202E+00_JPRB, 0.93837E+00_JPRB, 0.95664E+00_JPRB, 0.97695E+00_JPRB, 0.99984E+00_JPRB ,&
&  0.10171E+01_JPRB, 0.10582E+01_JPRB, 0.10814E+01_JPRB, 0.11262E+01_JPRB, 0.11729E+01_JPRB, 0.12350E+01_JPRB, 0.12675E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.13026E+01_JPRB, 0.13382E+01_JPRB, 0.13826E+01_JPRB, 0.13717E+01_JPRB, 0.13785E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.11710E+01_JPRB, 0.11331E+01_JPRB, 0.10810E+01_JPRB, 0.10586E+01_JPRB, 0.10348E+01_JPRB, 0.10077E+01_JPRB, 0.98183E+00_JPRB ,&
&  0.97817E+00_JPRB, 0.96955E+00_JPRB, 0.94503E+00_JPRB, 0.99698E+00_JPRB, 0.10309E+01_JPRB, 0.10803E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.12555E+01_JPRB, 0.13849E+01_JPRB, 0.15447E+01_JPRB, 0.17365E+01_JPRB, 0.19403E+01_JPRB, 0.21320E+01_JPRB, 0.23121E+01_JPRB ,&
&  0.24761E+01_JPRB, 0.25942E+01_JPRB, 0.25969E+01_JPRB, 0.21238E+01_JPRB, 0.17748E+01_JPRB, 0.13108E+01_JPRB, 0.10488E+01_JPRB ,&
&  0.89852E+00_JPRB, 0.83643E+00_JPRB, 0.83751E+00_JPRB, 0.84125E+00_JPRB, 0.68836E+00_JPRB, 0.59361E+00_JPRB, 0.50543E+00_JPRB ,&
&  0.46957E+00_JPRB, 0.43529E+00_JPRB, 0.42835E+00_JPRB, 0.42136E+00_JPRB, 0.41582E+00_JPRB, 0.41506E+00_JPRB, 0.42226E+00_JPRB ,&
&  0.42338E+00_JPRB, 0.43911E+00_JPRB, 0.47773E+00_JPRB, 0.52122E+00_JPRB, 0.55861E+00_JPRB, 0.57830E+00_JPRB, 0.55366E+00_JPRB ,&
&  0.58571E+00_JPRB, 0.58322E+00_JPRB, 0.58719E+00_JPRB, 0.64227E+00_JPRB, 0.74968E+00_JPRB, 0.91296E+00_JPRB, 0.11195E+01_JPRB ,&
&  0.13736E+01_JPRB, 0.14365E+01_JPRB, 0.17430E+01_JPRB, 0.22967E+01_JPRB, 0.20773E+01_JPRB, 0.17606E+01_JPRB, 0.64826E+00_JPRB ,&
&  0.68257E+00_JPRB, 0.69177E+00_JPRB, 0.72153E+00_JPRB, 0.75045E+00_JPRB, 0.78665E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.81611E+00_JPRB, 0.83836E+00_JPRB, 0.85532E+00_JPRB, 0.86771E+00_JPRB, 0.88666E+00_JPRB, 0.90620E+00_JPRB, 0.92509E+00_JPRB ,&
&  0.94237E+00_JPRB, 0.95669E+00_JPRB, 0.97514E+00_JPRB, 0.99325E+00_JPRB, 0.10146E+01_JPRB, 0.10386E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.10910E+01_JPRB, 0.11184E+01_JPRB, 0.11349E+01_JPRB, 0.11770E+01_JPRB, 0.12298E+01_JPRB, 0.12589E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.12736E+01_JPRB, 0.12944E+01_JPRB, 0.13157E+01_JPRB, 0.13416E+01_JPRB, 0.13470E+01_JPRB, 0.13978E+01_JPRB, 0.11892E+01_JPRB ,&
&  0.12318E+01_JPRB, 0.11561E+01_JPRB, 0.10382E+01_JPRB, 0.10118E+01_JPRB, 0.99876E+00_JPRB, 0.99464E+00_JPRB, 0.99040E+00_JPRB ,&
&  0.98679E+00_JPRB, 0.99399E+00_JPRB, 0.98961E+00_JPRB, 0.99079E+00_JPRB, 0.10179E+01_JPRB, 0.10725E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.13919E+01_JPRB, 0.15560E+01_JPRB, 0.17394E+01_JPRB, 0.19386E+01_JPRB, 0.20927E+01_JPRB, 0.22291E+01_JPRB ,&
&  0.23231E+01_JPRB, 0.23633E+01_JPRB, 0.21577E+01_JPRB, 0.20931E+01_JPRB, 0.19457E+01_JPRB, 0.14047E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.82758E+00_JPRB, 0.81794E+00_JPRB, 0.86411E+00_JPRB, 0.90007E+00_JPRB, 0.75272E+00_JPRB, 0.66893E+00_JPRB, 0.56881E+00_JPRB ,&
&  0.49072E+00_JPRB, 0.45523E+00_JPRB, 0.45125E+00_JPRB, 0.48748E+00_JPRB, 0.46458E+00_JPRB, 0.43959E+00_JPRB, 0.44393E+00_JPRB ,&
&  0.42738E+00_JPRB, 0.42657E+00_JPRB, 0.43474E+00_JPRB, 0.45478E+00_JPRB, 0.47480E+00_JPRB, 0.47018E+00_JPRB, 0.52337E+00_JPRB ,&
&  0.53069E+00_JPRB, 0.52740E+00_JPRB, 0.54054E+00_JPRB, 0.49468E+00_JPRB, 0.67670E+00_JPRB, 0.76967E+00_JPRB, 0.90166E+00_JPRB ,&
&  0.10402E+01_JPRB, 0.14375E+01_JPRB, 0.16932E+01_JPRB, 0.13209E+01_JPRB, 0.10115E+01_JPRB, 0.71855E+00_JPRB, 0.68504E+00_JPRB ,&
&  0.70366E+00_JPRB, 0.70977E+00_JPRB, 0.73113E+00_JPRB, 0.75850E+00_JPRB, 0.78830E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.81469E+00_JPRB, 0.83892E+00_JPRB, 0.85907E+00_JPRB, 0.87566E+00_JPRB, 0.89637E+00_JPRB, 0.91843E+00_JPRB, 0.93805E+00_JPRB ,&
&  0.95575E+00_JPRB, 0.97184E+00_JPRB, 0.99132E+00_JPRB, 0.10103E+01_JPRB, 0.10317E+01_JPRB, 0.10570E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.11097E+01_JPRB, 0.11259E+01_JPRB, 0.11593E+01_JPRB, 0.11993E+01_JPRB, 0.12260E+01_JPRB, 0.12345E+01_JPRB, 0.12357E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.12606E+01_JPRB, 0.12770E+01_JPRB, 0.12936E+01_JPRB, 0.13200E+01_JPRB, 0.13857E+01_JPRB, 0.11551E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.11752E+01_JPRB, 0.99750E+00_JPRB, 0.94090E+00_JPRB, 0.93140E+00_JPRB, 0.92187E+00_JPRB, 0.92516E+00_JPRB ,&
&  0.94899E+00_JPRB, 0.93066E+00_JPRB, 0.93251E+00_JPRB, 0.94077E+00_JPRB, 0.97851E+00_JPRB, 0.10298E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.12818E+01_JPRB, 0.14206E+01_JPRB, 0.15779E+01_JPRB, 0.17552E+01_JPRB, 0.18782E+01_JPRB, 0.19420E+01_JPRB ,&
&  0.19303E+01_JPRB, 0.18655E+01_JPRB, 0.15729E+01_JPRB, 0.16389E+01_JPRB, 0.16093E+01_JPRB, 0.13650E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.86267E+00_JPRB, 0.91941E+00_JPRB, 0.10138E+01_JPRB, 0.93540E+00_JPRB, 0.81090E+00_JPRB, 0.70109E+00_JPRB ,&
&  0.56963E+00_JPRB, 0.52678E+00_JPRB, 0.51869E+00_JPRB, 0.58318E+00_JPRB, 0.57261E+00_JPRB, 0.51180E+00_JPRB, 0.49454E+00_JPRB ,&
&  0.45663E+00_JPRB, 0.41634E+00_JPRB, 0.40086E+00_JPRB, 0.42944E+00_JPRB, 0.49354E+00_JPRB, 0.48814E+00_JPRB, 0.54375E+00_JPRB ,&
&  0.47470E+00_JPRB, 0.42968E+00_JPRB, 0.44914E+00_JPRB, 0.51835E+00_JPRB, 0.55698E+00_JPRB, 0.60665E+00_JPRB, 0.71043E+00_JPRB ,&
&  0.87516E+00_JPRB, 0.10846E+01_JPRB, 0.10573E+01_JPRB, 0.84905E+00_JPRB, 0.68999E+00_JPRB, 0.65897E+00_JPRB, 0.67551E+00_JPRB ,&
&  0.69765E+00_JPRB, 0.71579E+00_JPRB, 0.73756E+00_JPRB, 0.76162E+00_JPRB, 0.78717E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.79611E+00_JPRB, 0.81992E+00_JPRB, 0.84075E+00_JPRB, 0.85998E+00_JPRB, 0.88046E+00_JPRB, 0.90123E+00_JPRB, 0.91930E+00_JPRB ,&
&  0.93529E+00_JPRB, 0.95372E+00_JPRB, 0.97015E+00_JPRB, 0.98707E+00_JPRB, 0.10114E+01_JPRB, 0.10345E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.11008E+01_JPRB, 0.11339E+01_JPRB, 0.11635E+01_JPRB, 0.11782E+01_JPRB, 0.11838E+01_JPRB, 0.11821E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.11655E+01_JPRB, 0.11735E+01_JPRB, 0.11900E+01_JPRB, 0.12202E+01_JPRB, 0.12012E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.12308E+01_JPRB, 0.99264E+00_JPRB, 0.86897E+00_JPRB, 0.87602E+00_JPRB, 0.82444E+00_JPRB, 0.82658E+00_JPRB ,&
&  0.85813E+00_JPRB, 0.87075E+00_JPRB, 0.88335E+00_JPRB, 0.89457E+00_JPRB, 0.93407E+00_JPRB, 0.99334E+00_JPRB, 0.10144E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.11349E+01_JPRB, 0.11725E+01_JPRB, 0.12478E+01_JPRB, 0.13944E+01_JPRB, 0.15121E+01_JPRB, 0.15544E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.14304E+01_JPRB, 0.10571E+01_JPRB, 0.10973E+01_JPRB, 0.11738E+01_JPRB, 0.11104E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.10196E+01_JPRB, 0.11917E+01_JPRB, 0.12567E+01_JPRB, 0.12138E+01_JPRB, 0.10988E+01_JPRB, 0.93496E+00_JPRB ,&
&  0.78528E+00_JPRB, 0.67184E+00_JPRB, 0.68660E+00_JPRB, 0.76940E+00_JPRB, 0.75089E+00_JPRB, 0.63689E+00_JPRB, 0.58529E+00_JPRB ,&
&  0.52050E+00_JPRB, 0.45986E+00_JPRB, 0.42961E+00_JPRB, 0.45080E+00_JPRB, 0.51563E+00_JPRB, 0.79474E+00_JPRB, 0.80223E+00_JPRB ,&
&  0.52645E+00_JPRB, 0.51019E+00_JPRB, 0.39038E+00_JPRB, 0.46024E+00_JPRB, 0.46568E+00_JPRB, 0.46230E+00_JPRB, 0.50835E+00_JPRB ,&
&  0.62156E+00_JPRB, 0.71884E+00_JPRB, 0.72839E+00_JPRB, 0.66645E+00_JPRB, 0.64507E+00_JPRB, 0.65322E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.69348E+00_JPRB, 0.71254E+00_JPRB, 0.73145E+00_JPRB, 0.75188E+00_JPRB, 0.77281E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.75321E+00_JPRB, 0.78275E+00_JPRB, 0.80310E+00_JPRB, 0.81965E+00_JPRB, 0.82963E+00_JPRB, 0.84109E+00_JPRB, 0.86109E+00_JPRB ,&
&  0.87796E+00_JPRB, 0.89544E+00_JPRB, 0.91319E+00_JPRB, 0.93791E+00_JPRB, 0.97012E+00_JPRB, 0.99628E+00_JPRB, 0.10079E+01_JPRB ,&
&  0.10156E+01_JPRB, 0.10511E+01_JPRB, 0.10736E+01_JPRB, 0.11171E+01_JPRB, 0.11349E+01_JPRB, 0.11651E+01_JPRB, 0.11736E+01_JPRB ,&
&  0.11722E+01_JPRB, 0.11770E+01_JPRB, 0.11873E+01_JPRB, 0.12097E+01_JPRB, 0.12290E+01_JPRB, 0.13449E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.12875E+01_JPRB, 0.12745E+01_JPRB, 0.99223E+00_JPRB, 0.79528E+00_JPRB, 0.79377E+00_JPRB, 0.80273E+00_JPRB, 0.78092E+00_JPRB ,&
&  0.78828E+00_JPRB, 0.81873E+00_JPRB, 0.84739E+00_JPRB, 0.84368E+00_JPRB, 0.88248E+00_JPRB, 0.96013E+00_JPRB, 0.96100E+00_JPRB ,&
&  0.98826E+00_JPRB, 0.10139E+01_JPRB, 0.10104E+01_JPRB, 0.11080E+01_JPRB, 0.10637E+01_JPRB, 0.11777E+01_JPRB, 0.12563E+01_JPRB ,&
&  0.12582E+01_JPRB, 0.11049E+01_JPRB, 0.80738E+00_JPRB, 0.83411E+00_JPRB, 0.96043E+00_JPRB, 0.97035E+00_JPRB, 0.95778E+00_JPRB ,&
&  0.11345E+01_JPRB, 0.12247E+01_JPRB, 0.11028E+01_JPRB, 0.14256E+01_JPRB, 0.15452E+01_JPRB, 0.15251E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.97173E+00_JPRB, 0.96406E+00_JPRB, 0.97039E+00_JPRB, 0.93056E+00_JPRB, 0.86011E+00_JPRB, 0.75300E+00_JPRB ,&
&  0.64721E+00_JPRB, 0.56573E+00_JPRB, 0.51750E+00_JPRB, 0.50942E+00_JPRB, 0.61399E+00_JPRB, 0.72585E+00_JPRB, 0.83196E+00_JPRB ,&
&  0.53927E+00_JPRB, 0.58887E+00_JPRB, 0.52397E+00_JPRB, 0.67234E+00_JPRB, 0.56461E+00_JPRB, 0.43595E+00_JPRB, 0.43657E+00_JPRB ,&
&  0.48871E+00_JPRB, 0.53383E+00_JPRB, 0.56620E+00_JPRB, 0.58942E+00_JPRB, 0.61003E+00_JPRB, 0.63351E+00_JPRB, 0.64428E+00_JPRB ,&
&  0.66014E+00_JPRB, 0.67795E+00_JPRB, 0.69516E+00_JPRB, 0.72040E+00_JPRB, 0.73355E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.63808E+00_JPRB, 0.64915E+00_JPRB, 0.66579E+00_JPRB, 0.67616E+00_JPRB, 0.68859E+00_JPRB, 0.69875E+00_JPRB, 0.72879E+00_JPRB ,&
&  0.75144E+00_JPRB, 0.77247E+00_JPRB, 0.80114E+00_JPRB, 0.83141E+00_JPRB, 0.84303E+00_JPRB, 0.85495E+00_JPRB, 0.86160E+00_JPRB ,&
&  0.87872E+00_JPRB, 0.89571E+00_JPRB, 0.89349E+00_JPRB, 0.89956E+00_JPRB, 0.90641E+00_JPRB, 0.97356E+00_JPRB, 0.97604E+00_JPRB ,&
&  0.97067E+00_JPRB, 0.98805E+00_JPRB, 0.10091E+01_JPRB, 0.10752E+01_JPRB, 0.12608E+01_JPRB, 0.14004E+01_JPRB, 0.14056E+01_JPRB ,&
&  0.15275E+01_JPRB, 0.14629E+01_JPRB, 0.13809E+01_JPRB, 0.10273E+01_JPRB, 0.94676E+00_JPRB, 0.94325E+00_JPRB, 0.88168E+00_JPRB ,&
&  0.86851E+00_JPRB, 0.88809E+00_JPRB, 0.93750E+00_JPRB, 0.92298E+00_JPRB, 0.93878E+00_JPRB, 0.94064E+00_JPRB, 0.92236E+00_JPRB ,&
&  0.93419E+00_JPRB, 0.96121E+00_JPRB, 0.98509E+00_JPRB, 0.10540E+01_JPRB, 0.10727E+01_JPRB, 0.11787E+01_JPRB, 0.12142E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11888E+01_JPRB, 0.11085E+01_JPRB, 0.11581E+01_JPRB, 0.13087E+01_JPRB, 0.13297E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12817E+01_JPRB, 0.12306E+01_JPRB, 0.95498E+00_JPRB, 0.14839E+01_JPRB, 0.17383E+01_JPRB, 0.16532E+01_JPRB, 0.16343E+01_JPRB ,&
&  0.15243E+01_JPRB, 0.13767E+01_JPRB, 0.12586E+01_JPRB, 0.12095E+01_JPRB, 0.10905E+01_JPRB, 0.98255E+00_JPRB, 0.93587E+00_JPRB ,&
&  0.80145E+00_JPRB, 0.64661E+00_JPRB, 0.54480E+00_JPRB, 0.52222E+00_JPRB, 0.53465E+00_JPRB, 0.68090E+00_JPRB, 0.68692E+00_JPRB ,&
&  0.59477E+00_JPRB, 0.60606E+00_JPRB, 0.88355E+00_JPRB, 0.13069E+01_JPRB, 0.74069E+00_JPRB, 0.51570E+00_JPRB, 0.40801E+00_JPRB ,&
&  0.41767E+00_JPRB, 0.44194E+00_JPRB, 0.46630E+00_JPRB, 0.48526E+00_JPRB, 0.50866E+00_JPRB, 0.52919E+00_JPRB, 0.54513E+00_JPRB ,&
&  0.56101E+00_JPRB, 0.58119E+00_JPRB, 0.59866E+00_JPRB, 0.61849E+00_JPRB, 0.63490E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.63884E+00_JPRB, 0.65402E+00_JPRB, 0.65259E+00_JPRB, 0.65751E+00_JPRB, 0.65164E+00_JPRB, 0.65212E+00_JPRB, 0.63531E+00_JPRB ,&
&  0.62169E+00_JPRB, 0.63192E+00_JPRB, 0.66723E+00_JPRB, 0.73417E+00_JPRB, 0.75230E+00_JPRB, 0.77311E+00_JPRB, 0.78031E+00_JPRB ,&
&  0.80275E+00_JPRB, 0.78633E+00_JPRB, 0.77884E+00_JPRB, 0.75859E+00_JPRB, 0.76748E+00_JPRB, 0.81008E+00_JPRB, 0.90576E+00_JPRB ,&
&  0.10468E+01_JPRB, 0.12266E+01_JPRB, 0.14837E+01_JPRB, 0.17877E+01_JPRB, 0.24293E+01_JPRB, 0.21298E+01_JPRB, 0.19318E+01_JPRB ,&
&  0.22900E+01_JPRB, 0.19894E+01_JPRB, 0.16906E+01_JPRB, 0.15041E+01_JPRB, 0.13778E+01_JPRB, 0.13201E+01_JPRB, 0.13691E+01_JPRB ,&
&  0.13998E+01_JPRB, 0.14516E+01_JPRB, 0.15122E+01_JPRB, 0.15319E+01_JPRB, 0.15442E+01_JPRB, 0.15099E+01_JPRB, 0.14427E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.13784E+01_JPRB, 0.14327E+01_JPRB, 0.15419E+01_JPRB, 0.17651E+01_JPRB, 0.18814E+01_JPRB, 0.19605E+01_JPRB ,&
&  0.19735E+01_JPRB, 0.20754E+01_JPRB, 0.23195E+01_JPRB, 0.23863E+01_JPRB, 0.24220E+01_JPRB, 0.22715E+01_JPRB, 0.20676E+01_JPRB ,&
&  0.18330E+01_JPRB, 0.14827E+01_JPRB, 0.12431E+01_JPRB, 0.20053E+01_JPRB, 0.21966E+01_JPRB, 0.18090E+01_JPRB, 0.19165E+01_JPRB ,&
&  0.17670E+01_JPRB, 0.16573E+01_JPRB, 0.15831E+01_JPRB, 0.13908E+01_JPRB, 0.11465E+01_JPRB, 0.10653E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.93635E+00_JPRB, 0.74608E+00_JPRB, 0.59880E+00_JPRB, 0.52991E+00_JPRB, 0.50025E+00_JPRB, 0.54556E+00_JPRB, 0.57167E+00_JPRB ,&
&  0.58302E+00_JPRB, 0.55888E+00_JPRB, 0.69384E+00_JPRB, 0.10679E+01_JPRB, 0.90721E+00_JPRB, 0.51857E+00_JPRB, 0.41879E+00_JPRB ,&
&  0.42988E+00_JPRB, 0.44765E+00_JPRB, 0.46683E+00_JPRB, 0.47670E+00_JPRB, 0.48034E+00_JPRB, 0.48744E+00_JPRB, 0.51824E+00_JPRB ,&
&  0.54361E+00_JPRB, 0.56790E+00_JPRB, 0.58747E+00_JPRB, 0.61493E+00_JPRB, 0.63025E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.83994E+00_JPRB, 0.86892E+00_JPRB, 0.88970E+00_JPRB, 0.89684E+00_JPRB, 0.89230E+00_JPRB, 0.88321E+00_JPRB, 0.85950E+00_JPRB ,&
&  0.83179E+00_JPRB, 0.84504E+00_JPRB, 0.89160E+00_JPRB, 0.99775E+00_JPRB, 0.10775E+01_JPRB, 0.11280E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.12968E+01_JPRB, 0.12893E+01_JPRB, 0.13196E+01_JPRB, 0.13476E+01_JPRB, 0.13889E+01_JPRB, 0.16087E+01_JPRB ,&
&  0.19340E+01_JPRB, 0.23155E+01_JPRB, 0.26971E+01_JPRB, 0.29971E+01_JPRB, 0.25586E+01_JPRB, 0.19856E+01_JPRB, 0.21288E+01_JPRB ,&
&  0.24477E+01_JPRB, 0.22672E+01_JPRB, 0.18451E+01_JPRB, 0.18353E+01_JPRB, 0.18868E+01_JPRB, 0.20096E+01_JPRB, 0.21694E+01_JPRB ,&
&  0.22850E+01_JPRB, 0.23657E+01_JPRB, 0.24381E+01_JPRB, 0.24917E+01_JPRB, 0.25524E+01_JPRB, 0.26059E+01_JPRB, 0.26577E+01_JPRB ,&
&  0.26623E+01_JPRB, 0.26253E+01_JPRB, 0.26523E+01_JPRB, 0.27924E+01_JPRB, 0.28859E+01_JPRB, 0.29876E+01_JPRB, 0.30520E+01_JPRB ,&
&  0.30825E+01_JPRB, 0.31383E+01_JPRB, 0.34608E+01_JPRB, 0.36105E+01_JPRB, 0.33818E+01_JPRB, 0.31243E+01_JPRB, 0.29986E+01_JPRB ,&
&  0.27944E+01_JPRB, 0.24706E+01_JPRB, 0.19876E+01_JPRB, 0.29836E+01_JPRB, 0.30617E+01_JPRB, 0.26480E+01_JPRB, 0.22405E+01_JPRB ,&
&  0.20072E+01_JPRB, 0.18890E+01_JPRB, 0.17006E+01_JPRB, 0.14773E+01_JPRB, 0.12165E+01_JPRB, 0.11606E+01_JPRB, 0.99343E+00_JPRB ,&
&  0.10118E+01_JPRB, 0.86000E+00_JPRB, 0.68214E+00_JPRB, 0.57520E+00_JPRB, 0.54343E+00_JPRB, 0.59083E+00_JPRB, 0.59394E+00_JPRB ,&
&  0.53415E+00_JPRB, 0.59452E+00_JPRB, 0.69383E+00_JPRB, 0.12013E+01_JPRB, 0.23176E+01_JPRB, 0.64573E+00_JPRB, 0.48431E+00_JPRB ,&
&  0.49872E+00_JPRB, 0.53254E+00_JPRB, 0.56663E+00_JPRB, 0.58095E+00_JPRB, 0.58543E+00_JPRB, 0.60778E+00_JPRB, 0.64829E+00_JPRB ,&
&  0.69516E+00_JPRB, 0.72967E+00_JPRB, 0.74542E+00_JPRB, 0.78286E+00_JPRB, 0.81255E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.10167E+01_JPRB, 0.10510E+01_JPRB, 0.10837E+01_JPRB, 0.11276E+01_JPRB, 0.11714E+01_JPRB, 0.11899E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.12545E+01_JPRB, 0.13054E+01_JPRB, 0.13516E+01_JPRB, 0.14175E+01_JPRB, 0.14791E+01_JPRB, 0.15478E+01_JPRB ,&
&  0.16200E+01_JPRB, 0.17097E+01_JPRB, 0.18332E+01_JPRB, 0.19599E+01_JPRB, 0.20623E+01_JPRB, 0.22326E+01_JPRB, 0.24334E+01_JPRB ,&
&  0.26260E+01_JPRB, 0.24517E+01_JPRB, 0.23592E+01_JPRB, 0.19662E+01_JPRB, 0.16079E+01_JPRB, 0.21101E+01_JPRB, 0.20617E+01_JPRB ,&
&  0.20842E+01_JPRB, 0.20894E+01_JPRB, 0.21149E+01_JPRB, 0.22352E+01_JPRB, 0.24347E+01_JPRB, 0.26054E+01_JPRB, 0.27208E+01_JPRB ,&
&  0.28323E+01_JPRB, 0.29273E+01_JPRB, 0.30155E+01_JPRB, 0.31154E+01_JPRB, 0.32389E+01_JPRB, 0.33490E+01_JPRB, 0.34349E+01_JPRB ,&
&  0.35478E+01_JPRB, 0.36303E+01_JPRB, 0.36962E+01_JPRB, 0.37925E+01_JPRB, 0.37334E+01_JPRB, 0.37526E+01_JPRB, 0.37899E+01_JPRB ,&
&  0.38540E+01_JPRB, 0.39412E+01_JPRB, 0.41491E+01_JPRB, 0.43207E+01_JPRB, 0.43057E+01_JPRB, 0.39826E+01_JPRB, 0.37551E+01_JPRB ,&
&  0.35416E+01_JPRB, 0.31682E+01_JPRB, 0.31073E+01_JPRB, 0.39312E+01_JPRB, 0.25120E+01_JPRB, 0.26698E+01_JPRB, 0.26293E+01_JPRB ,&
&  0.22902E+01_JPRB, 0.21759E+01_JPRB, 0.19127E+01_JPRB, 0.15142E+01_JPRB, 0.12294E+01_JPRB, 0.91265E+00_JPRB, 0.99930E+00_JPRB ,&
&  0.11235E+01_JPRB, 0.99423E+00_JPRB, 0.76586E+00_JPRB, 0.62870E+00_JPRB, 0.62915E+00_JPRB, 0.67225E+00_JPRB, 0.66619E+00_JPRB ,&
&  0.59846E+00_JPRB, 0.64814E+00_JPRB, 0.71836E+00_JPRB, 0.11624E+01_JPRB, 0.12242E+01_JPRB, 0.62354E+00_JPRB, 0.54530E+00_JPRB ,&
&  0.57112E+00_JPRB, 0.63351E+00_JPRB, 0.71352E+00_JPRB, 0.76788E+00_JPRB, 0.78744E+00_JPRB, 0.80973E+00_JPRB, 0.83163E+00_JPRB ,&
&  0.85480E+00_JPRB, 0.89614E+00_JPRB, 0.95083E+00_JPRB, 0.97609E+00_JPRB, 0.99711E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.10551E+01_JPRB, 0.11106E+01_JPRB, 0.12011E+01_JPRB, 0.12744E+01_JPRB, 0.13206E+01_JPRB, 0.13341E+01_JPRB, 0.13185E+01_JPRB ,&
&  0.13267E+01_JPRB, 0.13681E+01_JPRB, 0.14518E+01_JPRB, 0.15287E+01_JPRB, 0.16130E+01_JPRB, 0.16988E+01_JPRB, 0.17869E+01_JPRB ,&
&  0.18706E+01_JPRB, 0.19485E+01_JPRB, 0.20466E+01_JPRB, 0.21614E+01_JPRB, 0.22735E+01_JPRB, 0.24641E+01_JPRB, 0.24909E+01_JPRB ,&
&  0.20376E+01_JPRB, 0.15699E+01_JPRB, 0.20066E+01_JPRB, 0.22944E+01_JPRB, 0.24726E+01_JPRB, 0.29008E+01_JPRB, 0.29838E+01_JPRB ,&
&  0.24681E+01_JPRB, 0.23718E+01_JPRB, 0.24964E+01_JPRB, 0.25809E+01_JPRB, 0.26600E+01_JPRB, 0.27855E+01_JPRB, 0.28921E+01_JPRB ,&
&  0.30349E+01_JPRB, 0.31850E+01_JPRB, 0.33396E+01_JPRB, 0.35209E+01_JPRB, 0.36916E+01_JPRB, 0.38847E+01_JPRB, 0.40574E+01_JPRB ,&
&  0.42212E+01_JPRB, 0.43459E+01_JPRB, 0.43244E+01_JPRB, 0.42879E+01_JPRB, 0.42616E+01_JPRB, 0.42827E+01_JPRB, 0.43130E+01_JPRB ,&
&  0.44523E+01_JPRB, 0.46592E+01_JPRB, 0.50026E+01_JPRB, 0.51918E+01_JPRB, 0.52286E+01_JPRB, 0.47725E+01_JPRB, 0.45359E+01_JPRB ,&
&  0.42616E+01_JPRB, 0.38543E+01_JPRB, 0.39716E+01_JPRB, 0.42151E+01_JPRB, 0.29681E+01_JPRB, 0.28733E+01_JPRB, 0.24191E+01_JPRB ,&
&  0.25207E+01_JPRB, 0.23349E+01_JPRB, 0.20542E+01_JPRB, 0.17232E+01_JPRB, 0.13251E+01_JPRB, 0.10291E+01_JPRB, 0.11653E+01_JPRB ,&
&  0.13777E+01_JPRB, 0.13164E+01_JPRB, 0.96029E+00_JPRB, 0.76588E+00_JPRB, 0.67955E+00_JPRB, 0.70466E+00_JPRB, 0.68399E+00_JPRB ,&
&  0.65600E+00_JPRB, 0.65084E+00_JPRB, 0.70720E+00_JPRB, 0.92123E+00_JPRB, 0.81350E+00_JPRB, 0.63529E+00_JPRB, 0.61559E+00_JPRB ,&
&  0.66520E+00_JPRB, 0.71657E+00_JPRB, 0.76435E+00_JPRB, 0.79956E+00_JPRB, 0.83297E+00_JPRB, 0.87378E+00_JPRB, 0.89664E+00_JPRB ,&
&  0.94972E+00_JPRB, 0.94570E+00_JPRB, 0.98889E+00_JPRB, 0.10066E+01_JPRB, 0.10326E+01_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.10578E+01_JPRB, 0.11204E+01_JPRB, 0.12054E+01_JPRB, 0.12862E+01_JPRB, 0.13323E+01_JPRB, 0.13695E+01_JPRB, 0.13879E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.13540E+01_JPRB, 0.14483E+01_JPRB, 0.15434E+01_JPRB, 0.16648E+01_JPRB, 0.17820E+01_JPRB, 0.18966E+01_JPRB ,&
&  0.20004E+01_JPRB, 0.21106E+01_JPRB, 0.22301E+01_JPRB, 0.23754E+01_JPRB, 0.24701E+01_JPRB, 0.25881E+01_JPRB, 0.23025E+01_JPRB ,&
&  0.16767E+01_JPRB, 0.17602E+01_JPRB, 0.24656E+01_JPRB, 0.25542E+01_JPRB, 0.26052E+01_JPRB, 0.28836E+01_JPRB, 0.29661E+01_JPRB ,&
&  0.26808E+01_JPRB, 0.26473E+01_JPRB, 0.26805E+01_JPRB, 0.27135E+01_JPRB, 0.27213E+01_JPRB, 0.27579E+01_JPRB, 0.28049E+01_JPRB ,&
&  0.29234E+01_JPRB, 0.31034E+01_JPRB, 0.33074E+01_JPRB, 0.35663E+01_JPRB, 0.38347E+01_JPRB, 0.41123E+01_JPRB, 0.43032E+01_JPRB ,&
&  0.44433E+01_JPRB, 0.45895E+01_JPRB, 0.46313E+01_JPRB, 0.44783E+01_JPRB, 0.45309E+01_JPRB, 0.47287E+01_JPRB, 0.49291E+01_JPRB ,&
&  0.50971E+01_JPRB, 0.54825E+01_JPRB, 0.56497E+01_JPRB, 0.54908E+01_JPRB, 0.49673E+01_JPRB, 0.51451E+01_JPRB, 0.46639E+01_JPRB ,&
&  0.47196E+01_JPRB, 0.44644E+01_JPRB, 0.48271E+01_JPRB, 0.38737E+01_JPRB, 0.35287E+01_JPRB, 0.33908E+01_JPRB, 0.30691E+01_JPRB ,&
&  0.27075E+01_JPRB, 0.25759E+01_JPRB, 0.22841E+01_JPRB, 0.16783E+01_JPRB, 0.15383E+01_JPRB, 0.12499E+01_JPRB, 0.14769E+01_JPRB ,&
&  0.16065E+01_JPRB, 0.17332E+01_JPRB, 0.11284E+01_JPRB, 0.83129E+00_JPRB, 0.71024E+00_JPRB, 0.72603E+00_JPRB, 0.72086E+00_JPRB ,&
&  0.73494E+00_JPRB, 0.72655E+00_JPRB, 0.73240E+00_JPRB, 0.74408E+00_JPRB, 0.66901E+00_JPRB, 0.63841E+00_JPRB, 0.64490E+00_JPRB ,&
&  0.69918E+00_JPRB, 0.74574E+00_JPRB, 0.75407E+00_JPRB, 0.75685E+00_JPRB, 0.77466E+00_JPRB, 0.82947E+00_JPRB, 0.86640E+00_JPRB ,&
&  0.91320E+00_JPRB, 0.94314E+00_JPRB, 0.97682E+00_JPRB, 0.99171E+00_JPRB, 0.10101E+01_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.90001E+00_JPRB, 0.93502E+00_JPRB, 0.96865E+00_JPRB, 0.97438E+00_JPRB, 0.98970E+00_JPRB, 0.10137E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.11876E+01_JPRB, 0.12691E+01_JPRB, 0.13730E+01_JPRB, 0.15106E+01_JPRB, 0.16360E+01_JPRB, 0.17854E+01_JPRB ,&
&  0.19410E+01_JPRB, 0.20990E+01_JPRB, 0.22630E+01_JPRB, 0.24350E+01_JPRB, 0.25235E+01_JPRB, 0.25469E+01_JPRB, 0.20645E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.25408E+01_JPRB, 0.24220E+01_JPRB, 0.26496E+01_JPRB, 0.27714E+01_JPRB, 0.29636E+01_JPRB, 0.28787E+01_JPRB ,&
&  0.28042E+01_JPRB, 0.27604E+01_JPRB, 0.27205E+01_JPRB, 0.27247E+01_JPRB, 0.27369E+01_JPRB, 0.27343E+01_JPRB, 0.27071E+01_JPRB ,&
&  0.27295E+01_JPRB, 0.28253E+01_JPRB, 0.30008E+01_JPRB, 0.32481E+01_JPRB, 0.35531E+01_JPRB, 0.39257E+01_JPRB, 0.42254E+01_JPRB ,&
&  0.45079E+01_JPRB, 0.47992E+01_JPRB, 0.49420E+01_JPRB, 0.48537E+01_JPRB, 0.48292E+01_JPRB, 0.50869E+01_JPRB, 0.55825E+01_JPRB ,&
&  0.56103E+01_JPRB, 0.54750E+01_JPRB, 0.57716E+01_JPRB, 0.58082E+01_JPRB, 0.55028E+01_JPRB, 0.55200E+01_JPRB, 0.48285E+01_JPRB ,&
&  0.51804E+01_JPRB, 0.48133E+01_JPRB, 0.50904E+01_JPRB, 0.41666E+01_JPRB, 0.39272E+01_JPRB, 0.41792E+01_JPRB, 0.39792E+01_JPRB ,&
&  0.34250E+01_JPRB, 0.31940E+01_JPRB, 0.27157E+01_JPRB, 0.20975E+01_JPRB, 0.18317E+01_JPRB, 0.16581E+01_JPRB, 0.17269E+01_JPRB ,&
&  0.17580E+01_JPRB, 0.16801E+01_JPRB, 0.12316E+01_JPRB, 0.80240E+00_JPRB, 0.72649E+00_JPRB, 0.81639E+00_JPRB, 0.86443E+00_JPRB ,&
&  0.88328E+00_JPRB, 0.93989E+00_JPRB, 0.10087E+01_JPRB, 0.93518E+00_JPRB, 0.73368E+00_JPRB, 0.68200E+00_JPRB, 0.68091E+00_JPRB ,&
&  0.71777E+00_JPRB, 0.75976E+00_JPRB, 0.76763E+00_JPRB, 0.75506E+00_JPRB, 0.71687E+00_JPRB, 0.75093E+00_JPRB, 0.76496E+00_JPRB ,&
&  0.80917E+00_JPRB, 0.82087E+00_JPRB, 0.85199E+00_JPRB, 0.87680E+00_JPRB, 0.88857E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.79637E+00_JPRB, 0.83468E+00_JPRB, 0.88926E+00_JPRB, 0.90661E+00_JPRB, 0.89658E+00_JPRB, 0.89466E+00_JPRB, 0.91082E+00_JPRB ,&
&  0.92837E+00_JPRB, 0.94946E+00_JPRB, 0.99419E+00_JPRB, 0.10650E+01_JPRB, 0.11830E+01_JPRB, 0.13086E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.16910E+01_JPRB, 0.19474E+01_JPRB, 0.22469E+01_JPRB, 0.25592E+01_JPRB, 0.27694E+01_JPRB, 0.27964E+01_JPRB, 0.18238E+01_JPRB ,&
&  0.16595E+01_JPRB, 0.24544E+01_JPRB, 0.24518E+01_JPRB, 0.26957E+01_JPRB, 0.29335E+01_JPRB, 0.30570E+01_JPRB, 0.29160E+01_JPRB ,&
&  0.28975E+01_JPRB, 0.28437E+01_JPRB, 0.28064E+01_JPRB, 0.27667E+01_JPRB, 0.27399E+01_JPRB, 0.27134E+01_JPRB, 0.26274E+01_JPRB ,&
&  0.25403E+01_JPRB, 0.25167E+01_JPRB, 0.25873E+01_JPRB, 0.27495E+01_JPRB, 0.29859E+01_JPRB, 0.32788E+01_JPRB, 0.37117E+01_JPRB ,&
&  0.42470E+01_JPRB, 0.48351E+01_JPRB, 0.53765E+01_JPRB, 0.52406E+01_JPRB, 0.53059E+01_JPRB, 0.53576E+01_JPRB, 0.56973E+01_JPRB ,&
&  0.56605E+01_JPRB, 0.53507E+01_JPRB, 0.59327E+01_JPRB, 0.65397E+01_JPRB, 0.65607E+01_JPRB, 0.60682E+01_JPRB, 0.52796E+01_JPRB ,&
&  0.54524E+01_JPRB, 0.54035E+01_JPRB, 0.51700E+01_JPRB, 0.45887E+01_JPRB, 0.45440E+01_JPRB, 0.50619E+01_JPRB, 0.52959E+01_JPRB ,&
&  0.46300E+01_JPRB, 0.40956E+01_JPRB, 0.34431E+01_JPRB, 0.27890E+01_JPRB, 0.23048E+01_JPRB, 0.21670E+01_JPRB, 0.22358E+01_JPRB ,&
&  0.20818E+01_JPRB, 0.19741E+01_JPRB, 0.13607E+01_JPRB, 0.83596E+00_JPRB, 0.68845E+00_JPRB, 0.84090E+00_JPRB, 0.10460E+01_JPRB ,&
&  0.11091E+01_JPRB, 0.12080E+01_JPRB, 0.12327E+01_JPRB, 0.14633E+01_JPRB, 0.10401E+01_JPRB, 0.85228E+00_JPRB, 0.75531E+00_JPRB ,&
&  0.75380E+00_JPRB, 0.77447E+00_JPRB, 0.79517E+00_JPRB, 0.75416E+00_JPRB, 0.70604E+00_JPRB, 0.71330E+00_JPRB, 0.69727E+00_JPRB ,&
&  0.71525E+00_JPRB, 0.73537E+00_JPRB, 0.74183E+00_JPRB, 0.75566E+00_JPRB, 0.78065E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.73583E+00_JPRB, 0.77284E+00_JPRB, 0.81160E+00_JPRB, 0.84081E+00_JPRB, 0.85051E+00_JPRB, 0.84027E+00_JPRB, 0.82736E+00_JPRB ,&
&  0.82370E+00_JPRB, 0.81834E+00_JPRB, 0.83971E+00_JPRB, 0.87381E+00_JPRB, 0.94264E+00_JPRB, 0.10221E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.13548E+01_JPRB, 0.16480E+01_JPRB, 0.20686E+01_JPRB, 0.25330E+01_JPRB, 0.26929E+01_JPRB, 0.23842E+01_JPRB, 0.21777E+01_JPRB ,&
&  0.20752E+01_JPRB, 0.24326E+01_JPRB, 0.24509E+01_JPRB, 0.28517E+01_JPRB, 0.32462E+01_JPRB, 0.33767E+01_JPRB, 0.28173E+01_JPRB ,&
&  0.27843E+01_JPRB, 0.27259E+01_JPRB, 0.26516E+01_JPRB, 0.26233E+01_JPRB, 0.26061E+01_JPRB, 0.26035E+01_JPRB, 0.25575E+01_JPRB ,&
&  0.24465E+01_JPRB, 0.23385E+01_JPRB, 0.22938E+01_JPRB, 0.23327E+01_JPRB, 0.24775E+01_JPRB, 0.26626E+01_JPRB, 0.29469E+01_JPRB ,&
&  0.34642E+01_JPRB, 0.42973E+01_JPRB, 0.53151E+01_JPRB, 0.56986E+01_JPRB, 0.57038E+01_JPRB, 0.57346E+01_JPRB, 0.57792E+01_JPRB ,&
&  0.59126E+01_JPRB, 0.64261E+01_JPRB, 0.66920E+01_JPRB, 0.67790E+01_JPRB, 0.69168E+01_JPRB, 0.68978E+01_JPRB, 0.58888E+01_JPRB ,&
&  0.57032E+01_JPRB, 0.56687E+01_JPRB, 0.51487E+01_JPRB, 0.51404E+01_JPRB, 0.53247E+01_JPRB, 0.59793E+01_JPRB, 0.55886E+01_JPRB ,&
&  0.47418E+01_JPRB, 0.41208E+01_JPRB, 0.37526E+01_JPRB, 0.35307E+01_JPRB, 0.30633E+01_JPRB, 0.30184E+01_JPRB, 0.31223E+01_JPRB ,&
&  0.22984E+01_JPRB, 0.86574E+00_JPRB, 0.53156E+00_JPRB, 0.65277E+00_JPRB, 0.48099E+00_JPRB, 0.65771E+00_JPRB, 0.12291E+01_JPRB ,&
&  0.14806E+01_JPRB, 0.17885E+01_JPRB, 0.20963E+01_JPRB, 0.21702E+01_JPRB, 0.18652E+01_JPRB, 0.13749E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.87336E+00_JPRB, 0.83482E+00_JPRB, 0.86515E+00_JPRB, 0.81459E+00_JPRB, 0.74492E+00_JPRB, 0.72207E+00_JPRB, 0.69871E+00_JPRB ,&
&  0.68110E+00_JPRB, 0.69818E+00_JPRB, 0.71657E+00_JPRB, 0.71813E+00_JPRB, 0.71290E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.76355E+00_JPRB, 0.76852E+00_JPRB, 0.78325E+00_JPRB, 0.77877E+00_JPRB, 0.77216E+00_JPRB, 0.77193E+00_JPRB, 0.76835E+00_JPRB ,&
&  0.75686E+00_JPRB, 0.74606E+00_JPRB, 0.76215E+00_JPRB, 0.79276E+00_JPRB, 0.84317E+00_JPRB, 0.90385E+00_JPRB, 0.96069E+00_JPRB ,&
&  0.10839E+01_JPRB, 0.13277E+01_JPRB, 0.17595E+01_JPRB, 0.23874E+01_JPRB, 0.27254E+01_JPRB, 0.25421E+01_JPRB, 0.25147E+01_JPRB ,&
&  0.22298E+01_JPRB, 0.25364E+01_JPRB, 0.30325E+01_JPRB, 0.38341E+01_JPRB, 0.40362E+01_JPRB, 0.37739E+01_JPRB, 0.30044E+01_JPRB ,&
&  0.29247E+01_JPRB, 0.27783E+01_JPRB, 0.25834E+01_JPRB, 0.23920E+01_JPRB, 0.22872E+01_JPRB, 0.22546E+01_JPRB, 0.22684E+01_JPRB ,&
&  0.22695E+01_JPRB, 0.22055E+01_JPRB, 0.21290E+01_JPRB, 0.20986E+01_JPRB, 0.21503E+01_JPRB, 0.22903E+01_JPRB, 0.24830E+01_JPRB ,&
&  0.27528E+01_JPRB, 0.33152E+01_JPRB, 0.45490E+01_JPRB, 0.59561E+01_JPRB, 0.53055E+01_JPRB, 0.61882E+01_JPRB, 0.60319E+01_JPRB ,&
&  0.60122E+01_JPRB, 0.66141E+01_JPRB, 0.67850E+01_JPRB, 0.78606E+01_JPRB, 0.77597E+01_JPRB, 0.72488E+01_JPRB, 0.61744E+01_JPRB ,&
&  0.59061E+01_JPRB, 0.54316E+01_JPRB, 0.54257E+01_JPRB, 0.56860E+01_JPRB, 0.61008E+01_JPRB, 0.69943E+01_JPRB, 0.49254E+01_JPRB ,&
&  0.45010E+01_JPRB, 0.49262E+01_JPRB, 0.46425E+01_JPRB, 0.39836E+01_JPRB, 0.38781E+01_JPRB, 0.40719E+01_JPRB, 0.26781E+01_JPRB ,&
&  0.86562E+00_JPRB, 0.53136E+00_JPRB, 0.44358E+00_JPRB, 0.41731E+00_JPRB, 0.37271E+00_JPRB, 0.66801E+00_JPRB, 0.24875E+01_JPRB ,&
&  0.20745E+01_JPRB, 0.27738E+01_JPRB, 0.31670E+01_JPRB, 0.32288E+01_JPRB, 0.30710E+01_JPRB, 0.22800E+01_JPRB, 0.19497E+01_JPRB ,&
&  0.13683E+01_JPRB, 0.10978E+01_JPRB, 0.10258E+01_JPRB, 0.98381E+00_JPRB, 0.88109E+00_JPRB, 0.82181E+00_JPRB, 0.78487E+00_JPRB ,&
&  0.78357E+00_JPRB, 0.77832E+00_JPRB, 0.76837E+00_JPRB, 0.75418E+00_JPRB, 0.76061E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.85997E+00_JPRB, 0.84214E+00_JPRB, 0.82183E+00_JPRB, 0.77468E+00_JPRB, 0.73972E+00_JPRB, 0.73731E+00_JPRB, 0.73054E+00_JPRB ,&
&  0.71594E+00_JPRB, 0.70300E+00_JPRB, 0.70824E+00_JPRB, 0.73205E+00_JPRB, 0.76500E+00_JPRB, 0.82056E+00_JPRB, 0.87626E+00_JPRB ,&
&  0.96854E+00_JPRB, 0.11626E+01_JPRB, 0.15863E+01_JPRB, 0.20553E+01_JPRB, 0.25404E+01_JPRB, 0.24581E+01_JPRB, 0.26790E+01_JPRB ,&
&  0.24064E+01_JPRB, 0.25401E+01_JPRB, 0.31975E+01_JPRB, 0.43494E+01_JPRB, 0.60013E+01_JPRB, 0.59462E+01_JPRB, 0.51374E+01_JPRB ,&
&  0.44490E+01_JPRB, 0.39470E+01_JPRB, 0.35451E+01_JPRB, 0.30976E+01_JPRB, 0.24067E+01_JPRB, 0.20575E+01_JPRB, 0.19605E+01_JPRB ,&
&  0.19065E+01_JPRB, 0.19428E+01_JPRB, 0.19048E+01_JPRB, 0.18857E+01_JPRB, 0.19598E+01_JPRB, 0.20994E+01_JPRB, 0.22999E+01_JPRB ,&
&  0.24613E+01_JPRB, 0.26965E+01_JPRB, 0.34344E+01_JPRB, 0.51565E+01_JPRB, 0.57238E+01_JPRB, 0.55420E+01_JPRB, 0.57357E+01_JPRB ,&
&  0.58955E+01_JPRB, 0.69070E+01_JPRB, 0.77702E+01_JPRB, 0.94814E+01_JPRB, 0.81179E+01_JPRB, 0.74559E+01_JPRB, 0.67051E+01_JPRB ,&
&  0.61826E+01_JPRB, 0.63868E+01_JPRB, 0.52275E+01_JPRB, 0.53620E+01_JPRB, 0.62136E+01_JPRB, 0.57224E+01_JPRB, 0.51751E+01_JPRB ,&
&  0.55879E+01_JPRB, 0.47751E+01_JPRB, 0.44797E+01_JPRB, 0.39879E+01_JPRB, 0.47065E+01_JPRB, 0.37664E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.74911E+00_JPRB, 0.56632E+00_JPRB, 0.40433E+00_JPRB, 0.39066E+00_JPRB, 0.42877E+00_JPRB, 0.68921E+00_JPRB, 0.18302E+01_JPRB ,&
&  0.20676E+01_JPRB, 0.32745E+01_JPRB, 0.37161E+01_JPRB, 0.40179E+01_JPRB, 0.36687E+01_JPRB, 0.35612E+01_JPRB, 0.40818E+01_JPRB ,&
&  0.35183E+01_JPRB, 0.26353E+01_JPRB, 0.19518E+01_JPRB, 0.15386E+01_JPRB, 0.12406E+01_JPRB, 0.10769E+01_JPRB, 0.98688E+00_JPRB ,&
&  0.95446E+00_JPRB, 0.93919E+00_JPRB, 0.91352E+00_JPRB, 0.88909E+00_JPRB, 0.85996E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.96911E+00_JPRB, 0.94705E+00_JPRB, 0.91287E+00_JPRB, 0.82923E+00_JPRB, 0.77553E+00_JPRB, 0.76265E+00_JPRB, 0.74470E+00_JPRB ,&
&  0.72309E+00_JPRB, 0.69367E+00_JPRB, 0.67970E+00_JPRB, 0.69075E+00_JPRB, 0.70740E+00_JPRB, 0.73852E+00_JPRB, 0.79313E+00_JPRB ,&
&  0.90932E+00_JPRB, 0.11257E+01_JPRB, 0.14633E+01_JPRB, 0.17760E+01_JPRB, 0.20712E+01_JPRB, 0.21328E+01_JPRB, 0.25942E+01_JPRB ,&
&  0.27003E+01_JPRB, 0.26489E+01_JPRB, 0.30897E+01_JPRB, 0.44514E+01_JPRB, 0.66478E+01_JPRB, 0.79312E+01_JPRB, 0.82943E+01_JPRB ,&
&  0.71043E+01_JPRB, 0.58185E+01_JPRB, 0.49552E+01_JPRB, 0.43707E+01_JPRB, 0.35817E+01_JPRB, 0.26137E+01_JPRB, 0.20324E+01_JPRB ,&
&  0.17683E+01_JPRB, 0.17772E+01_JPRB, 0.18103E+01_JPRB, 0.17661E+01_JPRB, 0.18314E+01_JPRB, 0.19629E+01_JPRB, 0.22136E+01_JPRB ,&
&  0.24095E+01_JPRB, 0.25463E+01_JPRB, 0.28504E+01_JPRB, 0.40192E+01_JPRB, 0.57808E+01_JPRB, 0.66496E+01_JPRB, 0.57998E+01_JPRB ,&
&  0.51039E+01_JPRB, 0.54889E+01_JPRB, 0.88348E+01_JPRB, 0.97155E+01_JPRB, 0.86647E+01_JPRB, 0.83118E+01_JPRB, 0.72543E+01_JPRB ,&
&  0.70184E+01_JPRB, 0.71246E+01_JPRB, 0.61251E+01_JPRB, 0.60806E+01_JPRB, 0.54951E+01_JPRB, 0.50883E+01_JPRB, 0.58619E+01_JPRB ,&
&  0.59687E+01_JPRB, 0.51685E+01_JPRB, 0.43397E+01_JPRB, 0.38298E+01_JPRB, 0.40269E+01_JPRB, 0.24775E+01_JPRB, 0.14025E+01_JPRB ,&
&  0.10578E+01_JPRB, 0.76633E+00_JPRB, 0.58168E+00_JPRB, 0.60144E+00_JPRB, 0.52224E+00_JPRB, 0.74866E+00_JPRB, 0.12867E+01_JPRB ,&
&  0.23632E+01_JPRB, 0.32295E+01_JPRB, 0.42802E+01_JPRB, 0.45685E+01_JPRB, 0.40080E+01_JPRB, 0.36173E+01_JPRB, 0.52052E+01_JPRB ,&
&  0.38961E+01_JPRB, 0.36101E+01_JPRB, 0.33491E+01_JPRB, 0.24729E+01_JPRB, 0.17656E+01_JPRB, 0.13842E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.10191E+01_JPRB, 0.98405E+00_JPRB, 0.98529E+00_JPRB, 0.99723E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.11298E+01_JPRB, 0.10542E+01_JPRB, 0.10214E+01_JPRB, 0.95551E+00_JPRB, 0.85658E+00_JPRB, 0.78985E+00_JPRB, 0.77220E+00_JPRB ,&
&  0.74953E+00_JPRB, 0.71907E+00_JPRB, 0.68664E+00_JPRB, 0.66738E+00_JPRB, 0.67977E+00_JPRB, 0.70201E+00_JPRB, 0.73382E+00_JPRB ,&
&  0.86388E+00_JPRB, 0.12357E+01_JPRB, 0.12253E+01_JPRB, 0.13650E+01_JPRB, 0.16300E+01_JPRB, 0.20604E+01_JPRB, 0.23109E+01_JPRB ,&
&  0.28629E+01_JPRB, 0.29193E+01_JPRB, 0.31240E+01_JPRB, 0.45487E+01_JPRB, 0.81887E+01_JPRB, 0.97198E+01_JPRB, 0.10668E+02_JPRB ,&
&  0.10435E+02_JPRB, 0.86096E+01_JPRB, 0.68465E+01_JPRB, 0.53776E+01_JPRB, 0.44005E+01_JPRB, 0.37356E+01_JPRB, 0.28776E+01_JPRB ,&
&  0.22615E+01_JPRB, 0.19834E+01_JPRB, 0.19239E+01_JPRB, 0.18659E+01_JPRB, 0.18329E+01_JPRB, 0.18940E+01_JPRB, 0.20941E+01_JPRB ,&
&  0.23670E+01_JPRB, 0.26042E+01_JPRB, 0.28100E+01_JPRB, 0.36508E+01_JPRB, 0.47674E+01_JPRB, 0.57842E+01_JPRB, 0.71408E+01_JPRB ,&
&  0.65231E+01_JPRB, 0.70832E+01_JPRB, 0.89118E+01_JPRB, 0.11061E+02_JPRB, 0.98419E+01_JPRB, 0.86769E+01_JPRB, 0.77240E+01_JPRB ,&
&  0.60241E+01_JPRB, 0.59388E+01_JPRB, 0.68188E+01_JPRB, 0.66957E+01_JPRB, 0.52937E+01_JPRB, 0.61968E+01_JPRB, 0.83624E+01_JPRB ,&
&  0.61761E+01_JPRB, 0.58589E+01_JPRB, 0.57485E+01_JPRB, 0.61376E+01_JPRB, 0.36930E+01_JPRB, 0.21866E+01_JPRB, 0.33836E+01_JPRB ,&
&  0.34013E+01_JPRB, 0.16949E+01_JPRB, 0.11130E+01_JPRB, 0.13732E+01_JPRB, 0.92044E+00_JPRB, 0.10308E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.18023E+01_JPRB, 0.21201E+01_JPRB, 0.41193E+01_JPRB, 0.39560E+01_JPRB, 0.39135E+01_JPRB, 0.34391E+01_JPRB, 0.41008E+01_JPRB ,&
&  0.42869E+01_JPRB, 0.33937E+01_JPRB, 0.33003E+01_JPRB, 0.27520E+01_JPRB, 0.21988E+01_JPRB, 0.17992E+01_JPRB, 0.15539E+01_JPRB ,&
&  0.13717E+01_JPRB, 0.12620E+01_JPRB, 0.12123E+01_JPRB, 0.11974E+01_JPRB, 0.12001E+01_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.11731E+01_JPRB, 0.10348E+01_JPRB, 0.96838E+00_JPRB, 0.95321E+00_JPRB, 0.91943E+00_JPRB, 0.82864E+00_JPRB, 0.77203E+00_JPRB ,&
&  0.76941E+00_JPRB, 0.73831E+00_JPRB, 0.69646E+00_JPRB, 0.67500E+00_JPRB, 0.68121E+00_JPRB, 0.69252E+00_JPRB, 0.68787E+00_JPRB ,&
&  0.76461E+00_JPRB, 0.97929E+00_JPRB, 0.10527E+01_JPRB, 0.11631E+01_JPRB, 0.14646E+01_JPRB, 0.18623E+01_JPRB, 0.20916E+01_JPRB ,&
&  0.26981E+01_JPRB, 0.30490E+01_JPRB, 0.35441E+01_JPRB, 0.51410E+01_JPRB, 0.73856E+01_JPRB, 0.94798E+01_JPRB, 0.10166E+02_JPRB ,&
&  0.11046E+02_JPRB, 0.10527E+02_JPRB, 0.83936E+01_JPRB, 0.68280E+01_JPRB, 0.57349E+01_JPRB, 0.50354E+01_JPRB, 0.42815E+01_JPRB ,&
&  0.33976E+01_JPRB, 0.26746E+01_JPRB, 0.23326E+01_JPRB, 0.21515E+01_JPRB, 0.20800E+01_JPRB, 0.19921E+01_JPRB, 0.20890E+01_JPRB ,&
&  0.23375E+01_JPRB, 0.26582E+01_JPRB, 0.28518E+01_JPRB, 0.35985E+01_JPRB, 0.49523E+01_JPRB, 0.61295E+01_JPRB, 0.69501E+01_JPRB ,&
&  0.71521E+01_JPRB, 0.72978E+01_JPRB, 0.96099E+01_JPRB, 0.10411E+02_JPRB, 0.10798E+02_JPRB, 0.97168E+01_JPRB, 0.89843E+01_JPRB ,&
&  0.67414E+01_JPRB, 0.61344E+01_JPRB, 0.63875E+01_JPRB, 0.65063E+01_JPRB, 0.69536E+01_JPRB, 0.88121E+01_JPRB, 0.83854E+01_JPRB ,&
&  0.72200E+01_JPRB, 0.64469E+01_JPRB, 0.57446E+01_JPRB, 0.49198E+01_JPRB, 0.52005E+01_JPRB, 0.35911E+01_JPRB, 0.41510E+01_JPRB ,&
&  0.42102E+01_JPRB, 0.38343E+01_JPRB, 0.30255E+01_JPRB, 0.23694E+01_JPRB, 0.17859E+01_JPRB, 0.18092E+01_JPRB, 0.16198E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.17380E+01_JPRB, 0.27966E+01_JPRB, 0.32607E+01_JPRB, 0.33097E+01_JPRB, 0.29712E+01_JPRB, 0.36096E+01_JPRB ,&
&  0.35262E+01_JPRB, 0.36415E+01_JPRB, 0.31474E+01_JPRB, 0.26022E+01_JPRB, 0.21758E+01_JPRB, 0.19003E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.15487E+01_JPRB, 0.14652E+01_JPRB, 0.14234E+01_JPRB, 0.13778E+01_JPRB, 0.13088E+01_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.11528E+01_JPRB, 0.10109E+01_JPRB, 0.93221E+00_JPRB, 0.88110E+00_JPRB, 0.85978E+00_JPRB, 0.83539E+00_JPRB, 0.78481E+00_JPRB ,&
&  0.74583E+00_JPRB, 0.74544E+00_JPRB, 0.72948E+00_JPRB, 0.71793E+00_JPRB, 0.71175E+00_JPRB, 0.70031E+00_JPRB, 0.68123E+00_JPRB ,&
&  0.71743E+00_JPRB, 0.85991E+00_JPRB, 0.89580E+00_JPRB, 0.10655E+01_JPRB, 0.13085E+01_JPRB, 0.17061E+01_JPRB, 0.21845E+01_JPRB ,&
&  0.24795E+01_JPRB, 0.28331E+01_JPRB, 0.33272E+01_JPRB, 0.45504E+01_JPRB, 0.60045E+01_JPRB, 0.72897E+01_JPRB, 0.76110E+01_JPRB ,&
&  0.71513E+01_JPRB, 0.73008E+01_JPRB, 0.68820E+01_JPRB, 0.61713E+01_JPRB, 0.55454E+01_JPRB, 0.50649E+01_JPRB, 0.44996E+01_JPRB ,&
&  0.38667E+01_JPRB, 0.32402E+01_JPRB, 0.27718E+01_JPRB, 0.25081E+01_JPRB, 0.23681E+01_JPRB, 0.23867E+01_JPRB, 0.24275E+01_JPRB ,&
&  0.24810E+01_JPRB, 0.26762E+01_JPRB, 0.27476E+01_JPRB, 0.32331E+01_JPRB, 0.43139E+01_JPRB, 0.59425E+01_JPRB, 0.66224E+01_JPRB ,&
&  0.79313E+01_JPRB, 0.82925E+01_JPRB, 0.10706E+02_JPRB, 0.12098E+02_JPRB, 0.95432E+01_JPRB, 0.10522E+02_JPRB, 0.11288E+02_JPRB ,&
&  0.99285E+01_JPRB, 0.89706E+01_JPRB, 0.92602E+01_JPRB, 0.88273E+01_JPRB, 0.78423E+01_JPRB, 0.91281E+01_JPRB, 0.81988E+01_JPRB ,&
&  0.71374E+01_JPRB, 0.64759E+01_JPRB, 0.56682E+01_JPRB, 0.56588E+01_JPRB, 0.48175E+01_JPRB, 0.46268E+01_JPRB, 0.36261E+01_JPRB ,&
&  0.37541E+01_JPRB, 0.31221E+01_JPRB, 0.32905E+01_JPRB, 0.27592E+01_JPRB, 0.24343E+01_JPRB, 0.20314E+01_JPRB, 0.16536E+01_JPRB ,&
&  0.15058E+01_JPRB, 0.14842E+01_JPRB, 0.16889E+01_JPRB, 0.23107E+01_JPRB, 0.29299E+01_JPRB, 0.25389E+01_JPRB, 0.29253E+01_JPRB ,&
&  0.33312E+01_JPRB, 0.34229E+01_JPRB, 0.28166E+01_JPRB, 0.24817E+01_JPRB, 0.20780E+01_JPRB, 0.17868E+01_JPRB, 0.16306E+01_JPRB ,&
&  0.14690E+01_JPRB, 0.14005E+01_JPRB, 0.13690E+01_JPRB, 0.13335E+01_JPRB, 0.12806E+01_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.10795E+01_JPRB, 0.96182E+00_JPRB, 0.86274E+00_JPRB, 0.81597E+00_JPRB, 0.80595E+00_JPRB, 0.80678E+00_JPRB, 0.80055E+00_JPRB ,&
&  0.77674E+00_JPRB, 0.76934E+00_JPRB, 0.76808E+00_JPRB, 0.75511E+00_JPRB, 0.74112E+00_JPRB, 0.72821E+00_JPRB, 0.70114E+00_JPRB ,&
&  0.69679E+00_JPRB, 0.85686E+00_JPRB, 0.84907E+00_JPRB, 0.89535E+00_JPRB, 0.10773E+01_JPRB, 0.15719E+01_JPRB, 0.20610E+01_JPRB ,&
&  0.23938E+01_JPRB, 0.26745E+01_JPRB, 0.31069E+01_JPRB, 0.37138E+01_JPRB, 0.42904E+01_JPRB, 0.45983E+01_JPRB, 0.46724E+01_JPRB ,&
&  0.51728E+01_JPRB, 0.43210E+01_JPRB, 0.43803E+01_JPRB, 0.39678E+01_JPRB, 0.37141E+01_JPRB, 0.34541E+01_JPRB, 0.32322E+01_JPRB ,&
&  0.30224E+01_JPRB, 0.27897E+01_JPRB, 0.25488E+01_JPRB, 0.23862E+01_JPRB, 0.22981E+01_JPRB, 0.22661E+01_JPRB, 0.22982E+01_JPRB ,&
&  0.23654E+01_JPRB, 0.24367E+01_JPRB, 0.25878E+01_JPRB, 0.26196E+01_JPRB, 0.32653E+01_JPRB, 0.46700E+01_JPRB, 0.61433E+01_JPRB ,&
&  0.63027E+01_JPRB, 0.65186E+01_JPRB, 0.12461E+02_JPRB, 0.12837E+02_JPRB, 0.11817E+02_JPRB, 0.12308E+02_JPRB, 0.12437E+02_JPRB ,&
&  0.11433E+02_JPRB, 0.98040E+01_JPRB, 0.82845E+01_JPRB, 0.78158E+01_JPRB, 0.92239E+01_JPRB, 0.87239E+01_JPRB, 0.79626E+01_JPRB ,&
&  0.71966E+01_JPRB, 0.67296E+01_JPRB, 0.61158E+01_JPRB, 0.56189E+01_JPRB, 0.53249E+01_JPRB, 0.55104E+01_JPRB, 0.50157E+01_JPRB ,&
&  0.45721E+01_JPRB, 0.47551E+01_JPRB, 0.37918E+01_JPRB, 0.28287E+01_JPRB, 0.22764E+01_JPRB, 0.19717E+01_JPRB, 0.17140E+01_JPRB ,&
&  0.14241E+01_JPRB, 0.13410E+01_JPRB, 0.13656E+01_JPRB, 0.16620E+01_JPRB, 0.23164E+01_JPRB, 0.22837E+01_JPRB, 0.23595E+01_JPRB ,&
&  0.22307E+01_JPRB, 0.26436E+01_JPRB, 0.24814E+01_JPRB, 0.21921E+01_JPRB, 0.19105E+01_JPRB, 0.16427E+01_JPRB, 0.14490E+01_JPRB ,&
&  0.13201E+01_JPRB, 0.13083E+01_JPRB, 0.12924E+01_JPRB, 0.12460E+01_JPRB, 0.11967E+01_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.10260E+01_JPRB, 0.92329E+00_JPRB, 0.82564E+00_JPRB, 0.78040E+00_JPRB, 0.78302E+00_JPRB, 0.79027E+00_JPRB, 0.79849E+00_JPRB ,&
&  0.78513E+00_JPRB, 0.77273E+00_JPRB, 0.78189E+00_JPRB, 0.78430E+00_JPRB, 0.78264E+00_JPRB, 0.76417E+00_JPRB, 0.72710E+00_JPRB ,&
&  0.72110E+00_JPRB, 0.88700E+00_JPRB, 0.84070E+00_JPRB, 0.90459E+00_JPRB, 0.10267E+01_JPRB, 0.13829E+01_JPRB, 0.17024E+01_JPRB ,&
&  0.20189E+01_JPRB, 0.22177E+01_JPRB, 0.24248E+01_JPRB, 0.27173E+01_JPRB, 0.28613E+01_JPRB, 0.27597E+01_JPRB, 0.26245E+01_JPRB ,&
&  0.24928E+01_JPRB, 0.24874E+01_JPRB, 0.21490E+01_JPRB, 0.22750E+01_JPRB, 0.21973E+01_JPRB, 0.19004E+01_JPRB, 0.20920E+01_JPRB ,&
&  0.20490E+01_JPRB, 0.20575E+01_JPRB, 0.20217E+01_JPRB, 0.19962E+01_JPRB, 0.19937E+01_JPRB, 0.19818E+01_JPRB, 0.19609E+01_JPRB ,&
&  0.19508E+01_JPRB, 0.19709E+01_JPRB, 0.21831E+01_JPRB, 0.23086E+01_JPRB, 0.24887E+01_JPRB, 0.31732E+01_JPRB, 0.46907E+01_JPRB ,&
&  0.66093E+01_JPRB, 0.69245E+01_JPRB, 0.73682E+01_JPRB, 0.10659E+02_JPRB, 0.13078E+02_JPRB, 0.12312E+02_JPRB, 0.11574E+02_JPRB ,&
&  0.11410E+02_JPRB, 0.10508E+02_JPRB, 0.95815E+01_JPRB, 0.89986E+01_JPRB, 0.93301E+01_JPRB, 0.90827E+01_JPRB, 0.84442E+01_JPRB ,&
&  0.76161E+01_JPRB, 0.68759E+01_JPRB, 0.65471E+01_JPRB, 0.61577E+01_JPRB, 0.54242E+01_JPRB, 0.51018E+01_JPRB, 0.51556E+01_JPRB ,&
&  0.50168E+01_JPRB, 0.41607E+01_JPRB, 0.29456E+01_JPRB, 0.25490E+01_JPRB, 0.19288E+01_JPRB, 0.16309E+01_JPRB, 0.15977E+01_JPRB ,&
&  0.12967E+01_JPRB, 0.11641E+01_JPRB, 0.11924E+01_JPRB, 0.12441E+01_JPRB, 0.17097E+01_JPRB, 0.17535E+01_JPRB, 0.19427E+01_JPRB ,&
&  0.18262E+01_JPRB, 0.15838E+01_JPRB, 0.19141E+01_JPRB, 0.18721E+01_JPRB, 0.16812E+01_JPRB, 0.14503E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12323E+01_JPRB, 0.12076E+01_JPRB, 0.11722E+01_JPRB, 0.11321E+01_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.95689E+00_JPRB, 0.85920E+00_JPRB, 0.77834E+00_JPRB, 0.72737E+00_JPRB, 0.72561E+00_JPRB, 0.74179E+00_JPRB, 0.75291E+00_JPRB ,&
&  0.76634E+00_JPRB, 0.77697E+00_JPRB, 0.78413E+00_JPRB, 0.79142E+00_JPRB, 0.79527E+00_JPRB, 0.77553E+00_JPRB, 0.76134E+00_JPRB ,&
&  0.77287E+00_JPRB, 0.85157E+00_JPRB, 0.85585E+00_JPRB, 0.86407E+00_JPRB, 0.11963E+01_JPRB, 0.13553E+01_JPRB, 0.15653E+01_JPRB ,&
&  0.16904E+01_JPRB, 0.18639E+01_JPRB, 0.18392E+01_JPRB, 0.17699E+01_JPRB, 0.17077E+01_JPRB, 0.16688E+01_JPRB, 0.15059E+01_JPRB ,&
&  0.12966E+01_JPRB, 0.12211E+01_JPRB, 0.11788E+01_JPRB, 0.12508E+01_JPRB, 0.13488E+01_JPRB, 0.11776E+01_JPRB, 0.12880E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.14135E+01_JPRB, 0.14517E+01_JPRB, 0.14649E+01_JPRB, 0.14957E+01_JPRB, 0.15398E+01_JPRB, 0.15900E+01_JPRB ,&
&  0.16479E+01_JPRB, 0.16615E+01_JPRB, 0.17389E+01_JPRB, 0.20269E+01_JPRB, 0.22533E+01_JPRB, 0.28217E+01_JPRB, 0.44446E+01_JPRB ,&
&  0.62546E+01_JPRB, 0.69272E+01_JPRB, 0.88862E+01_JPRB, 0.11790E+02_JPRB, 0.11480E+02_JPRB, 0.10516E+02_JPRB, 0.10043E+02_JPRB ,&
&  0.96757E+01_JPRB, 0.92398E+01_JPRB, 0.89438E+01_JPRB, 0.86330E+01_JPRB, 0.87478E+01_JPRB, 0.90340E+01_JPRB, 0.87649E+01_JPRB ,&
&  0.80078E+01_JPRB, 0.72713E+01_JPRB, 0.69179E+01_JPRB, 0.64176E+01_JPRB, 0.59730E+01_JPRB, 0.53979E+01_JPRB, 0.54051E+01_JPRB ,&
&  0.54472E+01_JPRB, 0.35537E+01_JPRB, 0.27195E+01_JPRB, 0.25528E+01_JPRB, 0.17693E+01_JPRB, 0.14782E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.12334E+01_JPRB, 0.10444E+01_JPRB, 0.11551E+01_JPRB, 0.12552E+01_JPRB, 0.13317E+01_JPRB, 0.13576E+01_JPRB, 0.12883E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.11063E+01_JPRB, 0.13862E+01_JPRB, 0.14223E+01_JPRB, 0.13539E+01_JPRB, 0.12472E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.10956E+01_JPRB, 0.11111E+01_JPRB, 0.10970E+01_JPRB, 0.10793E+01_JPRB, 0.10508E+01_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.94756E+00_JPRB, 0.84838E+00_JPRB, 0.75600E+00_JPRB, 0.71089E+00_JPRB, 0.72302E+00_JPRB, 0.73461E+00_JPRB, 0.74020E+00_JPRB ,&
&  0.75856E+00_JPRB, 0.77558E+00_JPRB, 0.78704E+00_JPRB, 0.79459E+00_JPRB, 0.79911E+00_JPRB, 0.79472E+00_JPRB, 0.83453E+00_JPRB ,&
&  0.70219E+00_JPRB, 0.74047E+00_JPRB, 0.72412E+00_JPRB, 0.83260E+00_JPRB, 0.12761E+01_JPRB, 0.12549E+01_JPRB, 0.13161E+01_JPRB ,&
&  0.12738E+01_JPRB, 0.12525E+01_JPRB, 0.11903E+01_JPRB, 0.11236E+01_JPRB, 0.10789E+01_JPRB, 0.10839E+01_JPRB, 0.10364E+01_JPRB ,&
&  0.89748E+00_JPRB, 0.81234E+00_JPRB, 0.79359E+00_JPRB, 0.79814E+00_JPRB, 0.85695E+00_JPRB, 0.89465E+00_JPRB, 0.97944E+00_JPRB ,&
&  0.10162E+01_JPRB, 0.10728E+01_JPRB, 0.11361E+01_JPRB, 0.11890E+01_JPRB, 0.12122E+01_JPRB, 0.12230E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.13028E+01_JPRB, 0.13880E+01_JPRB, 0.16795E+01_JPRB, 0.19332E+01_JPRB, 0.28327E+01_JPRB, 0.48653E+01_JPRB ,&
&  0.59144E+01_JPRB, 0.58041E+01_JPRB, 0.71363E+01_JPRB, 0.94929E+01_JPRB, 0.92815E+01_JPRB, 0.84070E+01_JPRB, 0.75717E+01_JPRB ,&
&  0.71104E+01_JPRB, 0.68451E+01_JPRB, 0.71016E+01_JPRB, 0.75903E+01_JPRB, 0.79244E+01_JPRB, 0.84333E+01_JPRB, 0.85258E+01_JPRB ,&
&  0.83112E+01_JPRB, 0.81137E+01_JPRB, 0.78416E+01_JPRB, 0.74980E+01_JPRB, 0.69472E+01_JPRB, 0.65975E+01_JPRB, 0.61262E+01_JPRB ,&
&  0.55385E+01_JPRB, 0.47420E+01_JPRB, 0.35132E+01_JPRB, 0.28115E+01_JPRB, 0.19605E+01_JPRB, 0.17371E+01_JPRB, 0.21094E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.11988E+01_JPRB, 0.11665E+01_JPRB, 0.12208E+01_JPRB, 0.11776E+01_JPRB, 0.11423E+01_JPRB, 0.98208E+00_JPRB ,&
&  0.82682E+00_JPRB, 0.91451E+00_JPRB, 0.11900E+01_JPRB, 0.12641E+01_JPRB, 0.12428E+01_JPRB, 0.11667E+01_JPRB, 0.97741E+00_JPRB ,&
&  0.10577E+01_JPRB, 0.10139E+01_JPRB, 0.10115E+01_JPRB, 0.10073E+01_JPRB, 0.99641E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.95434E+00_JPRB, 0.86983E+00_JPRB, 0.76076E+00_JPRB, 0.72760E+00_JPRB, 0.74392E+00_JPRB, 0.74384E+00_JPRB, 0.73994E+00_JPRB ,&
&  0.75286E+00_JPRB, 0.75876E+00_JPRB, 0.76593E+00_JPRB, 0.77821E+00_JPRB, 0.78728E+00_JPRB, 0.81261E+00_JPRB, 0.73512E+00_JPRB ,&
&  0.65345E+00_JPRB, 0.67232E+00_JPRB, 0.66209E+00_JPRB, 0.71058E+00_JPRB, 0.85501E+00_JPRB, 0.87025E+00_JPRB, 0.87954E+00_JPRB ,&
&  0.91101E+00_JPRB, 0.87692E+00_JPRB, 0.87358E+00_JPRB, 0.85109E+00_JPRB, 0.81799E+00_JPRB, 0.82477E+00_JPRB, 0.83087E+00_JPRB ,&
&  0.75085E+00_JPRB, 0.69770E+00_JPRB, 0.67159E+00_JPRB, 0.66165E+00_JPRB, 0.72248E+00_JPRB, 0.79934E+00_JPRB, 0.83372E+00_JPRB ,&
&  0.85285E+00_JPRB, 0.88440E+00_JPRB, 0.93028E+00_JPRB, 0.98534E+00_JPRB, 0.10399E+01_JPRB, 0.10788E+01_JPRB, 0.10941E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.11313E+01_JPRB, 0.11702E+01_JPRB, 0.13227E+01_JPRB, 0.16409E+01_JPRB, 0.23711E+01_JPRB, 0.33957E+01_JPRB ,&
&  0.37525E+01_JPRB, 0.39560E+01_JPRB, 0.43948E+01_JPRB, 0.50931E+01_JPRB, 0.54897E+01_JPRB, 0.52229E+01_JPRB, 0.49136E+01_JPRB ,&
&  0.49337E+01_JPRB, 0.49860E+01_JPRB, 0.57899E+01_JPRB, 0.64118E+01_JPRB, 0.68491E+01_JPRB, 0.73782E+01_JPRB, 0.75948E+01_JPRB ,&
&  0.80373E+01_JPRB, 0.84505E+01_JPRB, 0.84668E+01_JPRB, 0.80491E+01_JPRB, 0.76364E+01_JPRB, 0.72824E+01_JPRB, 0.66235E+01_JPRB ,&
&  0.59549E+01_JPRB, 0.50996E+01_JPRB, 0.40395E+01_JPRB, 0.31973E+01_JPRB, 0.24009E+01_JPRB, 0.21997E+01_JPRB, 0.18816E+01_JPRB ,&
&  0.13864E+01_JPRB, 0.11570E+01_JPRB, 0.11056E+01_JPRB, 0.11204E+01_JPRB, 0.10552E+01_JPRB, 0.95041E+00_JPRB, 0.89332E+00_JPRB ,&
&  0.91693E+00_JPRB, 0.10732E+01_JPRB, 0.11554E+01_JPRB, 0.12188E+01_JPRB, 0.12148E+01_JPRB, 0.11455E+01_JPRB, 0.87783E+00_JPRB ,&
&  0.87940E+00_JPRB, 0.10103E+01_JPRB, 0.10097E+01_JPRB, 0.10063E+01_JPRB, 0.99695E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.94816E+00_JPRB, 0.88720E+00_JPRB, 0.80006E+00_JPRB, 0.75484E+00_JPRB, 0.75218E+00_JPRB, 0.74654E+00_JPRB, 0.71799E+00_JPRB ,&
&  0.74594E+00_JPRB, 0.72961E+00_JPRB, 0.74663E+00_JPRB, 0.77647E+00_JPRB, 0.80448E+00_JPRB, 0.71331E+00_JPRB, 0.58625E+00_JPRB ,&
&  0.55483E+00_JPRB, 0.57780E+00_JPRB, 0.61412E+00_JPRB, 0.63517E+00_JPRB, 0.66131E+00_JPRB, 0.66554E+00_JPRB, 0.67520E+00_JPRB ,&
&  0.68682E+00_JPRB, 0.70280E+00_JPRB, 0.74540E+00_JPRB, 0.76642E+00_JPRB, 0.76554E+00_JPRB, 0.76452E+00_JPRB, 0.77339E+00_JPRB ,&
&  0.71974E+00_JPRB, 0.69805E+00_JPRB, 0.68722E+00_JPRB, 0.65635E+00_JPRB, 0.71562E+00_JPRB, 0.73482E+00_JPRB, 0.74578E+00_JPRB ,&
&  0.75713E+00_JPRB, 0.78197E+00_JPRB, 0.82455E+00_JPRB, 0.87268E+00_JPRB, 0.92338E+00_JPRB, 0.96935E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.10698E+01_JPRB, 0.11088E+01_JPRB, 0.11953E+01_JPRB, 0.14912E+01_JPRB, 0.19213E+01_JPRB, 0.25530E+01_JPRB ,&
&  0.27761E+01_JPRB, 0.31061E+01_JPRB, 0.34465E+01_JPRB, 0.33720E+01_JPRB, 0.36721E+01_JPRB, 0.40608E+01_JPRB, 0.39953E+01_JPRB ,&
&  0.41217E+01_JPRB, 0.41283E+01_JPRB, 0.44337E+01_JPRB, 0.48971E+01_JPRB, 0.55251E+01_JPRB, 0.60553E+01_JPRB, 0.64669E+01_JPRB ,&
&  0.70788E+01_JPRB, 0.80998E+01_JPRB, 0.84904E+01_JPRB, 0.81022E+01_JPRB, 0.76628E+01_JPRB, 0.72482E+01_JPRB, 0.66227E+01_JPRB ,&
&  0.59899E+01_JPRB, 0.52729E+01_JPRB, 0.44226E+01_JPRB, 0.35623E+01_JPRB, 0.27881E+01_JPRB, 0.23526E+01_JPRB, 0.19170E+01_JPRB ,&
&  0.14986E+01_JPRB, 0.12173E+01_JPRB, 0.11774E+01_JPRB, 0.11316E+01_JPRB, 0.10480E+01_JPRB, 0.92459E+00_JPRB, 0.83317E+00_JPRB ,&
&  0.84252E+00_JPRB, 0.93429E+00_JPRB, 0.11365E+01_JPRB, 0.11555E+01_JPRB, 0.11633E+01_JPRB, 0.11200E+01_JPRB, 0.93546E+00_JPRB ,&
&  0.87472E+00_JPRB, 0.10185E+01_JPRB, 0.99952E+00_JPRB, 0.10026E+01_JPRB, 0.99055E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.96764E+00_JPRB, 0.92747E+00_JPRB, 0.87692E+00_JPRB, 0.82156E+00_JPRB, 0.77220E+00_JPRB, 0.72884E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.71989E+00_JPRB, 0.64528E+00_JPRB, 0.71201E+00_JPRB, 0.59631E+00_JPRB, 0.56239E+00_JPRB, 0.55425E+00_JPRB, 0.53861E+00_JPRB ,&
&  0.53502E+00_JPRB, 0.55417E+00_JPRB, 0.58126E+00_JPRB, 0.60893E+00_JPRB, 0.61856E+00_JPRB, 0.62360E+00_JPRB, 0.63921E+00_JPRB ,&
&  0.66419E+00_JPRB, 0.69927E+00_JPRB, 0.74137E+00_JPRB, 0.75778E+00_JPRB, 0.75859E+00_JPRB, 0.75049E+00_JPRB, 0.74770E+00_JPRB ,&
&  0.69691E+00_JPRB, 0.68128E+00_JPRB, 0.68360E+00_JPRB, 0.68159E+00_JPRB, 0.67654E+00_JPRB, 0.66022E+00_JPRB, 0.66736E+00_JPRB ,&
&  0.64851E+00_JPRB, 0.61784E+00_JPRB, 0.74697E+00_JPRB, 0.78483E+00_JPRB, 0.83338E+00_JPRB, 0.88201E+00_JPRB, 0.92959E+00_JPRB ,&
&  0.97212E+00_JPRB, 0.10154E+01_JPRB, 0.10647E+01_JPRB, 0.11190E+01_JPRB, 0.12589E+01_JPRB, 0.15480E+01_JPRB, 0.18763E+01_JPRB ,&
&  0.22750E+01_JPRB, 0.18705E+01_JPRB, 0.21935E+01_JPRB, 0.23579E+01_JPRB, 0.28957E+01_JPRB, 0.35023E+01_JPRB, 0.37438E+01_JPRB ,&
&  0.41236E+01_JPRB, 0.40995E+01_JPRB, 0.40231E+01_JPRB, 0.39579E+01_JPRB, 0.42674E+01_JPRB, 0.47307E+01_JPRB, 0.52437E+01_JPRB ,&
&  0.57825E+01_JPRB, 0.63435E+01_JPRB, 0.73622E+01_JPRB, 0.72778E+01_JPRB, 0.70334E+01_JPRB, 0.67687E+01_JPRB, 0.63288E+01_JPRB ,&
&  0.57808E+01_JPRB, 0.51225E+01_JPRB, 0.43199E+01_JPRB, 0.34024E+01_JPRB, 0.27874E+01_JPRB, 0.22791E+01_JPRB, 0.18598E+01_JPRB ,&
&  0.15749E+01_JPRB, 0.14024E+01_JPRB, 0.13495E+01_JPRB, 0.12711E+01_JPRB, 0.11778E+01_JPRB, 0.10736E+01_JPRB, 0.92829E+00_JPRB ,&
&  0.78071E+00_JPRB, 0.77994E+00_JPRB, 0.81988E+00_JPRB, 0.90517E+00_JPRB, 0.95800E+00_JPRB, 0.98067E+00_JPRB, 0.10323E+01_JPRB ,&
&  0.96069E+00_JPRB, 0.98991E+00_JPRB, 0.10451E+01_JPRB, 0.10147E+01_JPRB, 0.99742E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.10408E+01_JPRB, 0.99320E+00_JPRB, 0.94875E+00_JPRB, 0.90226E+00_JPRB, 0.85843E+00_JPRB, 0.79686E+00_JPRB, 0.70348E+00_JPRB ,&
&  0.61499E+00_JPRB, 0.59321E+00_JPRB, 0.60024E+00_JPRB, 0.58293E+00_JPRB, 0.55935E+00_JPRB, 0.54952E+00_JPRB, 0.55879E+00_JPRB ,&
&  0.58107E+00_JPRB, 0.63180E+00_JPRB, 0.67300E+00_JPRB, 0.69114E+00_JPRB, 0.68884E+00_JPRB, 0.68338E+00_JPRB, 0.69195E+00_JPRB ,&
&  0.70412E+00_JPRB, 0.71689E+00_JPRB, 0.72433E+00_JPRB, 0.71417E+00_JPRB, 0.70327E+00_JPRB, 0.69794E+00_JPRB, 0.69225E+00_JPRB ,&
&  0.68807E+00_JPRB, 0.68246E+00_JPRB, 0.66287E+00_JPRB, 0.66371E+00_JPRB, 0.64145E+00_JPRB, 0.63221E+00_JPRB, 0.58880E+00_JPRB ,&
&  0.51679E+00_JPRB, 0.52774E+00_JPRB, 0.67837E+00_JPRB, 0.73642E+00_JPRB, 0.76347E+00_JPRB, 0.81168E+00_JPRB, 0.87531E+00_JPRB ,&
&  0.92637E+00_JPRB, 0.97003E+00_JPRB, 0.10148E+01_JPRB, 0.10740E+01_JPRB, 0.11569E+01_JPRB, 0.13138E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.18056E+01_JPRB, 0.15762E+01_JPRB, 0.14850E+01_JPRB, 0.16596E+01_JPRB, 0.23401E+01_JPRB, 0.28216E+01_JPRB, 0.29778E+01_JPRB ,&
&  0.30656E+01_JPRB, 0.29804E+01_JPRB, 0.29053E+01_JPRB, 0.29263E+01_JPRB, 0.30858E+01_JPRB, 0.33819E+01_JPRB, 0.38611E+01_JPRB ,&
&  0.42614E+01_JPRB, 0.49349E+01_JPRB, 0.58963E+01_JPRB, 0.60990E+01_JPRB, 0.59623E+01_JPRB, 0.58385E+01_JPRB, 0.55907E+01_JPRB ,&
&  0.53063E+01_JPRB, 0.47659E+01_JPRB, 0.39102E+01_JPRB, 0.31961E+01_JPRB, 0.25966E+01_JPRB, 0.21458E+01_JPRB, 0.17908E+01_JPRB ,&
&  0.16125E+01_JPRB, 0.15529E+01_JPRB, 0.15038E+01_JPRB, 0.14192E+01_JPRB, 0.12922E+01_JPRB, 0.11658E+01_JPRB, 0.10428E+01_JPRB ,&
&  0.88094E+00_JPRB, 0.79850E+00_JPRB, 0.74187E+00_JPRB, 0.76188E+00_JPRB, 0.84018E+00_JPRB, 0.92924E+00_JPRB, 0.94814E+00_JPRB ,&
&  0.96873E+00_JPRB, 0.94651E+00_JPRB, 0.93959E+00_JPRB, 0.98311E+00_JPRB, 0.10359E+01_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.10303E+01_JPRB, 0.99917E+00_JPRB, 0.95611E+00_JPRB, 0.93871E+00_JPRB, 0.89911E+00_JPRB, 0.85696E+00_JPRB, 0.75810E+00_JPRB ,&
&  0.64496E+00_JPRB, 0.62347E+00_JPRB, 0.66590E+00_JPRB, 0.68876E+00_JPRB, 0.67470E+00_JPRB, 0.67417E+00_JPRB, 0.68870E+00_JPRB ,&
&  0.70304E+00_JPRB, 0.74365E+00_JPRB, 0.74439E+00_JPRB, 0.71976E+00_JPRB, 0.70062E+00_JPRB, 0.68827E+00_JPRB, 0.68882E+00_JPRB ,&
&  0.68790E+00_JPRB, 0.68750E+00_JPRB, 0.67917E+00_JPRB, 0.66280E+00_JPRB, 0.66111E+00_JPRB, 0.65975E+00_JPRB, 0.65512E+00_JPRB ,&
&  0.65001E+00_JPRB, 0.63873E+00_JPRB, 0.64491E+00_JPRB, 0.65055E+00_JPRB, 0.63764E+00_JPRB, 0.60292E+00_JPRB, 0.52315E+00_JPRB ,&
&  0.48365E+00_JPRB, 0.46945E+00_JPRB, 0.50283E+00_JPRB, 0.61073E+00_JPRB, 0.67212E+00_JPRB, 0.72730E+00_JPRB, 0.78597E+00_JPRB ,&
&  0.75452E+00_JPRB, 0.75328E+00_JPRB, 0.79936E+00_JPRB, 0.94654E+00_JPRB, 0.10283E+01_JPRB, 0.11261E+01_JPRB, 0.12262E+01_JPRB ,&
&  0.13477E+01_JPRB, 0.14288E+01_JPRB, 0.14435E+01_JPRB, 0.12432E+01_JPRB, 0.13903E+01_JPRB, 0.19027E+01_JPRB, 0.19437E+01_JPRB ,&
&  0.21294E+01_JPRB, 0.23007E+01_JPRB, 0.24624E+01_JPRB, 0.24313E+01_JPRB, 0.23017E+01_JPRB, 0.23611E+01_JPRB, 0.26208E+01_JPRB ,&
&  0.30011E+01_JPRB, 0.35534E+01_JPRB, 0.45496E+01_JPRB, 0.50924E+01_JPRB, 0.49935E+01_JPRB, 0.48681E+01_JPRB, 0.47316E+01_JPRB ,&
&  0.46318E+01_JPRB, 0.41122E+01_JPRB, 0.34499E+01_JPRB, 0.28537E+01_JPRB, 0.23477E+01_JPRB, 0.19523E+01_JPRB, 0.17064E+01_JPRB ,&
&  0.16011E+01_JPRB, 0.15422E+01_JPRB, 0.15193E+01_JPRB, 0.14443E+01_JPRB, 0.12929E+01_JPRB, 0.11370E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.93784E+00_JPRB, 0.86191E+00_JPRB, 0.80370E+00_JPRB, 0.78586E+00_JPRB, 0.83594E+00_JPRB, 0.90241E+00_JPRB, 0.91593E+00_JPRB ,&
&  0.89095E+00_JPRB, 0.87857E+00_JPRB, 0.92303E+00_JPRB, 0.97796E+00_JPRB, 0.99897E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.10082E+01_JPRB, 0.10228E+01_JPRB, 0.10168E+01_JPRB, 0.99250E+00_JPRB, 0.96421E+00_JPRB, 0.87659E+00_JPRB, 0.78917E+00_JPRB ,&
&  0.73028E+00_JPRB, 0.74322E+00_JPRB, 0.78491E+00_JPRB, 0.82071E+00_JPRB, 0.82633E+00_JPRB, 0.80570E+00_JPRB, 0.79616E+00_JPRB ,&
&  0.80737E+00_JPRB, 0.80796E+00_JPRB, 0.77161E+00_JPRB, 0.74111E+00_JPRB, 0.69257E+00_JPRB, 0.67994E+00_JPRB, 0.68146E+00_JPRB ,&
&  0.68648E+00_JPRB, 0.68600E+00_JPRB, 0.68733E+00_JPRB, 0.68263E+00_JPRB, 0.68146E+00_JPRB, 0.66825E+00_JPRB, 0.64727E+00_JPRB ,&
&  0.62394E+00_JPRB, 0.61554E+00_JPRB, 0.61801E+00_JPRB, 0.62571E+00_JPRB, 0.61338E+00_JPRB, 0.59581E+00_JPRB, 0.51338E+00_JPRB ,&
&  0.48165E+00_JPRB, 0.46089E+00_JPRB, 0.45769E+00_JPRB, 0.47199E+00_JPRB, 0.52437E+00_JPRB, 0.56925E+00_JPRB, 0.66474E+00_JPRB ,&
&  0.73646E+00_JPRB, 0.79744E+00_JPRB, 0.82147E+00_JPRB, 0.84050E+00_JPRB, 0.87558E+00_JPRB, 0.94954E+00_JPRB, 0.10459E+01_JPRB ,&
&  0.11170E+01_JPRB, 0.11735E+01_JPRB, 0.11794E+01_JPRB, 0.10502E+01_JPRB, 0.10774E+01_JPRB, 0.12610E+01_JPRB, 0.15145E+01_JPRB ,&
&  0.17865E+01_JPRB, 0.18791E+01_JPRB, 0.18941E+01_JPRB, 0.20385E+01_JPRB, 0.21060E+01_JPRB, 0.21675E+01_JPRB, 0.22301E+01_JPRB ,&
&  0.25917E+01_JPRB, 0.30972E+01_JPRB, 0.38457E+01_JPRB, 0.42290E+01_JPRB, 0.41247E+01_JPRB, 0.41549E+01_JPRB, 0.39583E+01_JPRB ,&
&  0.37957E+01_JPRB, 0.33293E+01_JPRB, 0.29143E+01_JPRB, 0.24340E+01_JPRB, 0.20549E+01_JPRB, 0.18387E+01_JPRB, 0.17096E+01_JPRB ,&
&  0.15955E+01_JPRB, 0.15041E+01_JPRB, 0.14852E+01_JPRB, 0.14236E+01_JPRB, 0.12988E+01_JPRB, 0.11930E+01_JPRB, 0.11156E+01_JPRB ,&
&  0.10590E+01_JPRB, 0.10221E+01_JPRB, 0.10079E+01_JPRB, 0.97148E+00_JPRB, 0.96233E+00_JPRB, 0.96718E+00_JPRB, 0.95366E+00_JPRB ,&
&  0.93803E+00_JPRB, 0.92795E+00_JPRB, 0.93699E+00_JPRB, 0.95609E+00_JPRB, 0.97542E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.10392E+01_JPRB, 0.10602E+01_JPRB, 0.10531E+01_JPRB, 0.10503E+01_JPRB, 0.10505E+01_JPRB, 0.99245E+00_JPRB, 0.95302E+00_JPRB ,&
&  0.93479E+00_JPRB, 0.95652E+00_JPRB, 0.96566E+00_JPRB, 0.96557E+00_JPRB, 0.95265E+00_JPRB, 0.93690E+00_JPRB, 0.92534E+00_JPRB ,&
&  0.91509E+00_JPRB, 0.88545E+00_JPRB, 0.82644E+00_JPRB, 0.74371E+00_JPRB, 0.71649E+00_JPRB, 0.70697E+00_JPRB, 0.71010E+00_JPRB ,&
&  0.71034E+00_JPRB, 0.70193E+00_JPRB, 0.70061E+00_JPRB, 0.68310E+00_JPRB, 0.66977E+00_JPRB, 0.63653E+00_JPRB, 0.60822E+00_JPRB ,&
&  0.60701E+00_JPRB, 0.60248E+00_JPRB, 0.61868E+00_JPRB, 0.60142E+00_JPRB, 0.57758E+00_JPRB, 0.56198E+00_JPRB, 0.49749E+00_JPRB ,&
&  0.48568E+00_JPRB, 0.46602E+00_JPRB, 0.45883E+00_JPRB, 0.45938E+00_JPRB, 0.48269E+00_JPRB, 0.50394E+00_JPRB, 0.54817E+00_JPRB ,&
&  0.60155E+00_JPRB, 0.73325E+00_JPRB, 0.78405E+00_JPRB, 0.82819E+00_JPRB, 0.86414E+00_JPRB, 0.91050E+00_JPRB, 0.97113E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.11047E+01_JPRB, 0.11711E+01_JPRB, 0.12878E+01_JPRB, 0.14069E+01_JPRB, 0.14140E+01_JPRB, 0.15554E+01_JPRB ,&
&  0.20439E+01_JPRB, 0.19654E+01_JPRB, 0.18702E+01_JPRB, 0.18414E+01_JPRB, 0.18948E+01_JPRB, 0.19877E+01_JPRB, 0.21259E+01_JPRB ,&
&  0.23943E+01_JPRB, 0.28030E+01_JPRB, 0.33796E+01_JPRB, 0.37703E+01_JPRB, 0.38024E+01_JPRB, 0.38516E+01_JPRB, 0.37851E+01_JPRB ,&
&  0.35017E+01_JPRB, 0.33076E+01_JPRB, 0.34286E+01_JPRB, 0.27601E+01_JPRB, 0.23101E+01_JPRB, 0.20635E+01_JPRB, 0.19426E+01_JPRB ,&
&  0.18237E+01_JPRB, 0.17023E+01_JPRB, 0.16271E+01_JPRB, 0.15297E+01_JPRB, 0.14044E+01_JPRB, 0.13294E+01_JPRB, 0.13095E+01_JPRB ,&
&  0.12916E+01_JPRB, 0.12626E+01_JPRB, 0.12357E+01_JPRB, 0.12022E+01_JPRB, 0.11535E+01_JPRB, 0.10989E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10238E+01_JPRB, 0.10089E+01_JPRB, 0.10030E+01_JPRB, 0.10119E+01_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.10551E+01_JPRB, 0.10779E+01_JPRB, 0.10738E+01_JPRB, 0.10713E+01_JPRB, 0.10679E+01_JPRB, 0.10466E+01_JPRB, 0.10299E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.10199E+01_JPRB, 0.10098E+01_JPRB, 0.99461E+00_JPRB, 0.97540E+00_JPRB, 0.96239E+00_JPRB, 0.95018E+00_JPRB ,&
&  0.93693E+00_JPRB, 0.86600E+00_JPRB, 0.82693E+00_JPRB, 0.79422E+00_JPRB, 0.76296E+00_JPRB, 0.74615E+00_JPRB, 0.73482E+00_JPRB ,&
&  0.71792E+00_JPRB, 0.70753E+00_JPRB, 0.69925E+00_JPRB, 0.68414E+00_JPRB, 0.66754E+00_JPRB, 0.64384E+00_JPRB, 0.63925E+00_JPRB ,&
&  0.65348E+00_JPRB, 0.63734E+00_JPRB, 0.61591E+00_JPRB, 0.58955E+00_JPRB, 0.54790E+00_JPRB, 0.52585E+00_JPRB, 0.51673E+00_JPRB ,&
&  0.50383E+00_JPRB, 0.47322E+00_JPRB, 0.47493E+00_JPRB, 0.47809E+00_JPRB, 0.48177E+00_JPRB, 0.50582E+00_JPRB, 0.55080E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.73306E+00_JPRB, 0.78553E+00_JPRB, 0.84579E+00_JPRB, 0.88547E+00_JPRB, 0.91641E+00_JPRB, 0.94514E+00_JPRB ,&
&  0.98207E+00_JPRB, 0.10354E+01_JPRB, 0.11000E+01_JPRB, 0.11885E+01_JPRB, 0.12922E+01_JPRB, 0.13762E+01_JPRB, 0.14471E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.15401E+01_JPRB, 0.15806E+01_JPRB, 0.16122E+01_JPRB, 0.16718E+01_JPRB, 0.17966E+01_JPRB, 0.18677E+01_JPRB ,&
&  0.20225E+01_JPRB, 0.22422E+01_JPRB, 0.25836E+01_JPRB, 0.29175E+01_JPRB, 0.31027E+01_JPRB, 0.32790E+01_JPRB, 0.33359E+01_JPRB ,&
&  0.31644E+01_JPRB, 0.30676E+01_JPRB, 0.31448E+01_JPRB, 0.28771E+01_JPRB, 0.25775E+01_JPRB, 0.23201E+01_JPRB, 0.21301E+01_JPRB ,&
&  0.19466E+01_JPRB, 0.17683E+01_JPRB, 0.16482E+01_JPRB, 0.15483E+01_JPRB, 0.14462E+01_JPRB, 0.13538E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.12748E+01_JPRB, 0.12389E+01_JPRB, 0.11984E+01_JPRB, 0.11394E+01_JPRB, 0.10828E+01_JPRB, 0.10372E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10076E+01_JPRB, 0.10087E+01_JPRB, 0.10106E+01_JPRB, 0.10266E+01_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.10581E+01_JPRB, 0.10633E+01_JPRB, 0.10496E+01_JPRB, 0.10350E+01_JPRB, 0.10281E+01_JPRB, 0.10124E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.10095E+01_JPRB, 0.99424E+00_JPRB, 0.97484E+00_JPRB, 0.94984E+00_JPRB, 0.93372E+00_JPRB, 0.91361E+00_JPRB ,&
&  0.89590E+00_JPRB, 0.87132E+00_JPRB, 0.84546E+00_JPRB, 0.81852E+00_JPRB, 0.79721E+00_JPRB, 0.77241E+00_JPRB, 0.75204E+00_JPRB ,&
&  0.73191E+00_JPRB, 0.70776E+00_JPRB, 0.68606E+00_JPRB, 0.67978E+00_JPRB, 0.66712E+00_JPRB, 0.65289E+00_JPRB, 0.65406E+00_JPRB ,&
&  0.65219E+00_JPRB, 0.64317E+00_JPRB, 0.63630E+00_JPRB, 0.61356E+00_JPRB, 0.56725E+00_JPRB, 0.48383E+00_JPRB, 0.50185E+00_JPRB ,&
&  0.49411E+00_JPRB, 0.47942E+00_JPRB, 0.48415E+00_JPRB, 0.49809E+00_JPRB, 0.51212E+00_JPRB, 0.55231E+00_JPRB, 0.56020E+00_JPRB ,&
&  0.54964E+00_JPRB, 0.73402E+00_JPRB, 0.77382E+00_JPRB, 0.81823E+00_JPRB, 0.85530E+00_JPRB, 0.88058E+00_JPRB, 0.90442E+00_JPRB ,&
&  0.93675E+00_JPRB, 0.97808E+00_JPRB, 0.10230E+01_JPRB, 0.10884E+01_JPRB, 0.11700E+01_JPRB, 0.12734E+01_JPRB, 0.13628E+01_JPRB ,&
&  0.14164E+01_JPRB, 0.14559E+01_JPRB, 0.15074E+01_JPRB, 0.15559E+01_JPRB, 0.16120E+01_JPRB, 0.17027E+01_JPRB, 0.18159E+01_JPRB ,&
&  0.19712E+01_JPRB, 0.20790E+01_JPRB, 0.22878E+01_JPRB, 0.24643E+01_JPRB, 0.25803E+01_JPRB, 0.27200E+01_JPRB, 0.28391E+01_JPRB ,&
&  0.28129E+01_JPRB, 0.28133E+01_JPRB, 0.27623E+01_JPRB, 0.25373E+01_JPRB, 0.22843E+01_JPRB, 0.21054E+01_JPRB, 0.19412E+01_JPRB ,&
&  0.18078E+01_JPRB, 0.16750E+01_JPRB, 0.15425E+01_JPRB, 0.14344E+01_JPRB, 0.13480E+01_JPRB, 0.12805E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.11606E+01_JPRB, 0.11280E+01_JPRB, 0.11096E+01_JPRB, 0.10834E+01_JPRB, 0.10621E+01_JPRB, 0.10444E+01_JPRB ,&
&  0.10378E+01_JPRB, 0.10330E+01_JPRB, 0.10340E+01_JPRB, 0.10377E+01_JPRB, 0.10408E+01_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.10559E+01_JPRB, 0.10623E+01_JPRB, 0.10641E+01_JPRB, 0.10485E+01_JPRB, 0.10407E+01_JPRB, 0.10333E+01_JPRB, 0.10389E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10175E+01_JPRB, 0.98409E+00_JPRB, 0.96234E+00_JPRB, 0.93977E+00_JPRB, 0.92870E+00_JPRB, 0.91771E+00_JPRB ,&
&  0.91094E+00_JPRB, 0.90016E+00_JPRB, 0.89452E+00_JPRB, 0.87636E+00_JPRB, 0.85773E+00_JPRB, 0.82546E+00_JPRB, 0.79143E+00_JPRB ,&
&  0.75726E+00_JPRB, 0.72298E+00_JPRB, 0.69200E+00_JPRB, 0.68739E+00_JPRB, 0.67410E+00_JPRB, 0.66572E+00_JPRB, 0.65618E+00_JPRB ,&
&  0.64591E+00_JPRB, 0.58463E+00_JPRB, 0.59924E+00_JPRB, 0.57635E+00_JPRB, 0.55181E+00_JPRB, 0.52538E+00_JPRB, 0.50435E+00_JPRB ,&
&  0.50690E+00_JPRB, 0.50882E+00_JPRB, 0.52490E+00_JPRB, 0.55478E+00_JPRB, 0.56993E+00_JPRB, 0.59700E+00_JPRB, 0.61481E+00_JPRB ,&
&  0.61122E+00_JPRB, 0.73856E+00_JPRB, 0.80443E+00_JPRB, 0.84626E+00_JPRB, 0.88393E+00_JPRB, 0.91552E+00_JPRB, 0.94069E+00_JPRB ,&
&  0.98114E+00_JPRB, 0.10230E+01_JPRB, 0.10725E+01_JPRB, 0.11226E+01_JPRB, 0.12035E+01_JPRB, 0.12646E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.13809E+01_JPRB, 0.14170E+01_JPRB, 0.14788E+01_JPRB, 0.15544E+01_JPRB, 0.16378E+01_JPRB, 0.17276E+01_JPRB ,&
&  0.18260E+01_JPRB, 0.19259E+01_JPRB, 0.20349E+01_JPRB, 0.21386E+01_JPRB, 0.22321E+01_JPRB, 0.23025E+01_JPRB, 0.23663E+01_JPRB ,&
&  0.23523E+01_JPRB, 0.23198E+01_JPRB, 0.22543E+01_JPRB, 0.21495E+01_JPRB, 0.20089E+01_JPRB, 0.18521E+01_JPRB, 0.16933E+01_JPRB ,&
&  0.16006E+01_JPRB, 0.14981E+01_JPRB, 0.14048E+01_JPRB, 0.13154E+01_JPRB, 0.12416E+01_JPRB, 0.11831E+01_JPRB, 0.11485E+01_JPRB ,&
&  0.11160E+01_JPRB, 0.10936E+01_JPRB, 0.10778E+01_JPRB, 0.10672E+01_JPRB, 0.10629E+01_JPRB, 0.10529E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10514E+01_JPRB, 0.10534E+01_JPRB, 0.10471E+01_JPRB, 0.10454E+01_JPRB, 0.10488E+01_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.10602E+01_JPRB, 0.10695E+01_JPRB, 0.10870E+01_JPRB, 0.10905E+01_JPRB, 0.10914E+01_JPRB, 0.10826E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10655E+01_JPRB, 0.10492E+01_JPRB, 0.10130E+01_JPRB, 0.98860E+00_JPRB, 0.96651E+00_JPRB, 0.95320E+00_JPRB, 0.94415E+00_JPRB ,&
&  0.93850E+00_JPRB, 0.93045E+00_JPRB, 0.92033E+00_JPRB, 0.90208E+00_JPRB, 0.87716E+00_JPRB, 0.84651E+00_JPRB, 0.80904E+00_JPRB ,&
&  0.77784E+00_JPRB, 0.75441E+00_JPRB, 0.72227E+00_JPRB, 0.72570E+00_JPRB, 0.70746E+00_JPRB, 0.69466E+00_JPRB, 0.68655E+00_JPRB ,&
&  0.66576E+00_JPRB, 0.64672E+00_JPRB, 0.61631E+00_JPRB, 0.59716E+00_JPRB, 0.58154E+00_JPRB, 0.56289E+00_JPRB, 0.57874E+00_JPRB ,&
&  0.59201E+00_JPRB, 0.63477E+00_JPRB, 0.65656E+00_JPRB, 0.66684E+00_JPRB, 0.67614E+00_JPRB, 0.67656E+00_JPRB, 0.68881E+00_JPRB ,&
&  0.70610E+00_JPRB, 0.73304E+00_JPRB, 0.81545E+00_JPRB, 0.86986E+00_JPRB, 0.90371E+00_JPRB, 0.93998E+00_JPRB, 0.96975E+00_JPRB ,&
&  0.10092E+01_JPRB, 0.10381E+01_JPRB, 0.10611E+01_JPRB, 0.10878E+01_JPRB, 0.11345E+01_JPRB, 0.11898E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12785E+01_JPRB, 0.13059E+01_JPRB, 0.13398E+01_JPRB, 0.13786E+01_JPRB, 0.14300E+01_JPRB, 0.14869E+01_JPRB, 0.15478E+01_JPRB ,&
&  0.16094E+01_JPRB, 0.16727E+01_JPRB, 0.17250E+01_JPRB, 0.17731E+01_JPRB, 0.18047E+01_JPRB, 0.18214E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.18100E+01_JPRB, 0.17778E+01_JPRB, 0.17268E+01_JPRB, 0.16414E+01_JPRB, 0.15737E+01_JPRB, 0.15332E+01_JPRB, 0.14631E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.13425E+01_JPRB, 0.12866E+01_JPRB, 0.12292E+01_JPRB, 0.11804E+01_JPRB, 0.11360E+01_JPRB, 0.11071E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.10693E+01_JPRB, 0.10615E+01_JPRB, 0.10569E+01_JPRB, 0.10591E+01_JPRB, 0.10584E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.10669E+01_JPRB, 0.10638E+01_JPRB, 0.10547E+01_JPRB, 0.10519E+01_JPRB, 0.10526E+01_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.10701E+01_JPRB, 0.10715E+01_JPRB, 0.10776E+01_JPRB, 0.10806E+01_JPRB, 0.10827E+01_JPRB, 0.10803E+01_JPRB, 0.10782E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.10550E+01_JPRB, 0.10292E+01_JPRB, 0.10069E+01_JPRB, 0.98640E+00_JPRB, 0.97199E+00_JPRB, 0.96079E+00_JPRB ,&
&  0.94995E+00_JPRB, 0.94160E+00_JPRB, 0.93100E+00_JPRB, 0.91804E+00_JPRB, 0.90074E+00_JPRB, 0.88165E+00_JPRB, 0.86065E+00_JPRB ,&
&  0.84482E+00_JPRB, 0.82906E+00_JPRB, 0.81595E+00_JPRB, 0.80645E+00_JPRB, 0.80205E+00_JPRB, 0.75951E+00_JPRB, 0.74988E+00_JPRB ,&
&  0.75972E+00_JPRB, 0.74131E+00_JPRB, 0.72489E+00_JPRB, 0.70058E+00_JPRB, 0.70021E+00_JPRB, 0.71167E+00_JPRB, 0.71919E+00_JPRB ,&
&  0.69566E+00_JPRB, 0.73753E+00_JPRB, 0.74232E+00_JPRB, 0.74409E+00_JPRB, 0.74069E+00_JPRB, 0.73205E+00_JPRB, 0.74951E+00_JPRB ,&
&  0.79508E+00_JPRB, 0.81415E+00_JPRB, 0.85477E+00_JPRB, 0.90313E+00_JPRB, 0.94130E+00_JPRB, 0.98102E+00_JPRB, 0.10202E+01_JPRB ,&
&  0.10616E+01_JPRB, 0.10924E+01_JPRB, 0.11135E+01_JPRB, 0.11338E+01_JPRB, 0.11658E+01_JPRB, 0.11983E+01_JPRB, 0.12279E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.12599E+01_JPRB, 0.12734E+01_JPRB, 0.12865E+01_JPRB, 0.13040E+01_JPRB, 0.13185E+01_JPRB, 0.13333E+01_JPRB ,&
&  0.13440E+01_JPRB, 0.13535E+01_JPRB, 0.13585E+01_JPRB, 0.13599E+01_JPRB, 0.13586E+01_JPRB, 0.13529E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.13273E+01_JPRB, 0.13062E+01_JPRB, 0.12798E+01_JPRB, 0.12580E+01_JPRB, 0.12403E+01_JPRB, 0.12209E+01_JPRB ,&
&  0.12012E+01_JPRB, 0.11817E+01_JPRB, 0.11629E+01_JPRB, 0.11427E+01_JPRB, 0.11235E+01_JPRB, 0.11043E+01_JPRB, 0.10903E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10742E+01_JPRB, 0.10729E+01_JPRB, 0.10703E+01_JPRB, 0.10739E+01_JPRB, 0.10738E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.10772E+01_JPRB, 0.10716E+01_JPRB, 0.10686E+01_JPRB, 0.10672E+01_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.10580E+01_JPRB, 0.10560E+01_JPRB, 0.10551E+01_JPRB, 0.10558E+01_JPRB, 0.10572E+01_JPRB, 0.10575E+01_JPRB, 0.10562E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10414E+01_JPRB, 0.10291E+01_JPRB, 0.10157E+01_JPRB, 0.10003E+01_JPRB, 0.98565E+00_JPRB, 0.97651E+00_JPRB ,&
&  0.96753E+00_JPRB, 0.95884E+00_JPRB, 0.94965E+00_JPRB, 0.94219E+00_JPRB, 0.93332E+00_JPRB, 0.92368E+00_JPRB, 0.91467E+00_JPRB ,&
&  0.90848E+00_JPRB, 0.90165E+00_JPRB, 0.89420E+00_JPRB, 0.88529E+00_JPRB, 0.87852E+00_JPRB, 0.87111E+00_JPRB, 0.86418E+00_JPRB ,&
&  0.85611E+00_JPRB, 0.85143E+00_JPRB, 0.84817E+00_JPRB, 0.84631E+00_JPRB, 0.84881E+00_JPRB, 0.85183E+00_JPRB, 0.85169E+00_JPRB ,&
&  0.84983E+00_JPRB, 0.84393E+00_JPRB, 0.84462E+00_JPRB, 0.84633E+00_JPRB, 0.85071E+00_JPRB, 0.85948E+00_JPRB, 0.87203E+00_JPRB ,&
&  0.89015E+00_JPRB, 0.91247E+00_JPRB, 0.93722E+00_JPRB, 0.95815E+00_JPRB, 0.97937E+00_JPRB, 0.99865E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.10426E+01_JPRB, 0.10553E+01_JPRB, 0.10670E+01_JPRB, 0.10825E+01_JPRB, 0.10950E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11095E+01_JPRB, 0.11084E+01_JPRB, 0.11075E+01_JPRB, 0.11040E+01_JPRB, 0.11020E+01_JPRB, 0.11014E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11025E+01_JPRB, 0.11055E+01_JPRB, 0.11091E+01_JPRB, 0.11130E+01_JPRB, 0.11134E+01_JPRB, 0.11146E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.11110E+01_JPRB, 0.11060E+01_JPRB, 0.10983E+01_JPRB, 0.10920E+01_JPRB, 0.10879E+01_JPRB, 0.10827E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10735E+01_JPRB, 0.10707E+01_JPRB, 0.10672E+01_JPRB, 0.10618E+01_JPRB, 0.10572E+01_JPRB, 0.10543E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10504E+01_JPRB, 0.10512E+01_JPRB, 0.10522E+01_JPRB, 0.10589E+01_JPRB, 0.10617E+01_JPRB, 0.10644E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.10681E+01_JPRB, 0.10677E+01_JPRB, 0.10648E+01_JPRB, 0.10609E+01_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.10552E+01_JPRB, 0.10527E+01_JPRB, 0.10505E+01_JPRB, 0.10488E+01_JPRB, 0.10477E+01_JPRB, 0.10468E+01_JPRB, 0.10439E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10354E+01_JPRB, 0.10304E+01_JPRB, 0.10244E+01_JPRB, 0.10169E+01_JPRB, 0.10078E+01_JPRB, 0.99986E+00_JPRB ,&
&  0.99262E+00_JPRB, 0.98678E+00_JPRB, 0.98056E+00_JPRB, 0.97603E+00_JPRB, 0.97147E+00_JPRB, 0.96726E+00_JPRB, 0.96208E+00_JPRB ,&
&  0.95770E+00_JPRB, 0.95348E+00_JPRB, 0.94952E+00_JPRB, 0.94608E+00_JPRB, 0.94445E+00_JPRB, 0.94242E+00_JPRB, 0.94020E+00_JPRB ,&
&  0.93819E+00_JPRB, 0.93674E+00_JPRB, 0.93536E+00_JPRB, 0.93403E+00_JPRB, 0.93315E+00_JPRB, 0.93331E+00_JPRB, 0.93347E+00_JPRB ,&
&  0.93409E+00_JPRB, 0.93608E+00_JPRB, 0.93930E+00_JPRB, 0.94225E+00_JPRB, 0.94558E+00_JPRB, 0.95040E+00_JPRB, 0.95680E+00_JPRB ,&
&  0.96257E+00_JPRB, 0.96816E+00_JPRB, 0.97441E+00_JPRB, 0.98157E+00_JPRB, 0.98812E+00_JPRB, 0.99458E+00_JPRB, 0.10002E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.10105E+01_JPRB, 0.10152E+01_JPRB, 0.10175E+01_JPRB, 0.10214E+01_JPRB, 0.10246E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10298E+01_JPRB, 0.10307E+01_JPRB, 0.10320E+01_JPRB, 0.10339E+01_JPRB, 0.10359E+01_JPRB, 0.10380E+01_JPRB, 0.10402E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.10452E+01_JPRB, 0.10476E+01_JPRB, 0.10499E+01_JPRB, 0.10524E+01_JPRB, 0.10535E+01_JPRB, 0.10550E+01_JPRB ,&
&  0.10563E+01_JPRB, 0.10575E+01_JPRB, 0.10572E+01_JPRB, 0.10569E+01_JPRB, 0.10567E+01_JPRB, 0.10568E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10578E+01_JPRB, 0.10580E+01_JPRB, 0.10576E+01_JPRB, 0.10563E+01_JPRB, 0.10553E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10547E+01_JPRB, 0.10546E+01_JPRB, 0.10554E+01_JPRB, 0.10564E+01_JPRB, 0.10590E+01_JPRB, 0.10600E+01_JPRB, 0.10612E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10618E+01_JPRB, 0.10610E+01_JPRB, 0.10595E+01_JPRB, 0.10573E+01_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB, 0.10200E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB ,&
&  0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB, 0.14226E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.14436E+00_JPRB, 0.14509E+00_JPRB, 0.14561E+00_JPRB, 0.14603E+00_JPRB, 0.14588E+00_JPRB, 0.14596E+00_JPRB, 0.14603E+00_JPRB ,&
&  0.14604E+00_JPRB, 0.14602E+00_JPRB, 0.14627E+00_JPRB, 0.14650E+00_JPRB, 0.14675E+00_JPRB, 0.14701E+00_JPRB, 0.14731E+00_JPRB ,&
&  0.14743E+00_JPRB, 0.14771E+00_JPRB, 0.14668E+00_JPRB, 0.14585E+00_JPRB, 0.14534E+00_JPRB, 0.14504E+00_JPRB, 0.14489E+00_JPRB ,&
&  0.14476E+00_JPRB, 0.14469E+00_JPRB, 0.14458E+00_JPRB, 0.14534E+00_JPRB, 0.14584E+00_JPRB, 0.14629E+00_JPRB, 0.14664E+00_JPRB ,&
&  0.14678E+00_JPRB, 0.14699E+00_JPRB, 0.14717E+00_JPRB, 0.14735E+00_JPRB, 0.14755E+00_JPRB, 0.14752E+00_JPRB, 0.14740E+00_JPRB ,&
&  0.14721E+00_JPRB, 0.14692E+00_JPRB, 0.14657E+00_JPRB, 0.14624E+00_JPRB, 0.14598E+00_JPRB, 0.14567E+00_JPRB, 0.14534E+00_JPRB ,&
&  0.14501E+00_JPRB, 0.14464E+00_JPRB, 0.14427E+00_JPRB, 0.14393E+00_JPRB, 0.14360E+00_JPRB, 0.14332E+00_JPRB, 0.14301E+00_JPRB ,&
&  0.14279E+00_JPRB, 0.14259E+00_JPRB, 0.14239E+00_JPRB, 0.14224E+00_JPRB, 0.14202E+00_JPRB, 0.14193E+00_JPRB, 0.14174E+00_JPRB ,&
&  0.14166E+00_JPRB, 0.14157E+00_JPRB, 0.14151E+00_JPRB, 0.14143E+00_JPRB, 0.14142E+00_JPRB, 0.14138E+00_JPRB, 0.14136E+00_JPRB ,&
&  0.14132E+00_JPRB, 0.14142E+00_JPRB, 0.14130E+00_JPRB, 0.14115E+00_JPRB, 0.14098E+00_JPRB, 0.14081E+00_JPRB, 0.14066E+00_JPRB ,&
&  0.14052E+00_JPRB, 0.14038E+00_JPRB, 0.14029E+00_JPRB, 0.14030E+00_JPRB, 0.14032E+00_JPRB, 0.14036E+00_JPRB, 0.14042E+00_JPRB ,&
&  0.14049E+00_JPRB, 0.14059E+00_JPRB, 0.14068E+00_JPRB, 0.14065E+00_JPRB, 0.14068E+00_JPRB, 0.14070E+00_JPRB, 0.14072E+00_JPRB ,&
&  0.14074E+00_JPRB, 0.14078E+00_JPRB, 0.14086E+00_JPRB, 0.14096E+00_JPRB, 0.14119E+00_JPRB, 0.14129E+00_JPRB, 0.14143E+00_JPRB ,&
&  0.14163E+00_JPRB, 0.14199E+00_JPRB, 0.14230E+00_JPRB, 0.14264E+00_JPRB, 0.14305E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.15339E+00_JPRB, 0.15522E+00_JPRB, 0.15669E+00_JPRB, 0.15865E+00_JPRB, 0.15513E+00_JPRB, 0.16211E+00_JPRB, 0.15960E+00_JPRB ,&
&  0.15699E+00_JPRB, 0.15226E+00_JPRB, 0.15225E+00_JPRB, 0.15137E+00_JPRB, 0.15018E+00_JPRB, 0.14950E+00_JPRB, 0.14924E+00_JPRB ,&
&  0.14917E+00_JPRB, 0.14889E+00_JPRB, 0.14921E+00_JPRB, 0.14944E+00_JPRB, 0.14962E+00_JPRB, 0.14975E+00_JPRB, 0.14999E+00_JPRB ,&
&  0.15046E+00_JPRB, 0.15131E+00_JPRB, 0.15219E+00_JPRB, 0.15063E+00_JPRB, 0.15051E+00_JPRB, 0.15083E+00_JPRB, 0.15149E+00_JPRB ,&
&  0.15203E+00_JPRB, 0.15309E+00_JPRB, 0.15387E+00_JPRB, 0.15441E+00_JPRB, 0.15586E+00_JPRB, 0.15588E+00_JPRB, 0.15570E+00_JPRB ,&
&  0.15530E+00_JPRB, 0.15454E+00_JPRB, 0.15343E+00_JPRB, 0.15240E+00_JPRB, 0.15134E+00_JPRB, 0.14996E+00_JPRB, 0.14864E+00_JPRB ,&
&  0.14756E+00_JPRB, 0.14667E+00_JPRB, 0.14554E+00_JPRB, 0.14445E+00_JPRB, 0.14345E+00_JPRB, 0.14248E+00_JPRB, 0.14164E+00_JPRB ,&
&  0.14104E+00_JPRB, 0.14053E+00_JPRB, 0.14001E+00_JPRB, 0.13959E+00_JPRB, 0.13916E+00_JPRB, 0.13875E+00_JPRB, 0.13838E+00_JPRB ,&
&  0.13849E+00_JPRB, 0.13858E+00_JPRB, 0.13867E+00_JPRB, 0.13875E+00_JPRB, 0.13908E+00_JPRB, 0.13913E+00_JPRB, 0.13919E+00_JPRB ,&
&  0.13921E+00_JPRB, 0.13910E+00_JPRB, 0.13899E+00_JPRB, 0.13879E+00_JPRB, 0.13857E+00_JPRB, 0.13829E+00_JPRB, 0.13817E+00_JPRB ,&
&  0.13817E+00_JPRB, 0.13827E+00_JPRB, 0.13857E+00_JPRB, 0.13878E+00_JPRB, 0.13898E+00_JPRB, 0.13914E+00_JPRB, 0.13955E+00_JPRB ,&
&  0.13996E+00_JPRB, 0.14038E+00_JPRB, 0.14078E+00_JPRB, 0.14071E+00_JPRB, 0.14094E+00_JPRB, 0.14116E+00_JPRB, 0.14136E+00_JPRB ,&
&  0.14149E+00_JPRB, 0.14193E+00_JPRB, 0.14260E+00_JPRB, 0.14351E+00_JPRB, 0.14516E+00_JPRB, 0.14660E+00_JPRB, 0.14793E+00_JPRB ,&
&  0.14936E+00_JPRB, 0.15220E+00_JPRB, 0.15341E+00_JPRB, 0.15388E+00_JPRB, 0.15400E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.16269E+00_JPRB, 0.16452E+00_JPRB, 0.16568E+00_JPRB, 0.16649E+00_JPRB, 0.16763E+00_JPRB, 0.16813E+00_JPRB, 0.16859E+00_JPRB ,&
&  0.16679E+00_JPRB, 0.16617E+00_JPRB, 0.16511E+00_JPRB, 0.16381E+00_JPRB, 0.16284E+00_JPRB, 0.16063E+00_JPRB, 0.15840E+00_JPRB ,&
&  0.15590E+00_JPRB, 0.15365E+00_JPRB, 0.15222E+00_JPRB, 0.15083E+00_JPRB, 0.14946E+00_JPRB, 0.14831E+00_JPRB, 0.14987E+00_JPRB ,&
&  0.15080E+00_JPRB, 0.15139E+00_JPRB, 0.15255E+00_JPRB, 0.15257E+00_JPRB, 0.15385E+00_JPRB, 0.15587E+00_JPRB, 0.15856E+00_JPRB ,&
&  0.16044E+00_JPRB, 0.16324E+00_JPRB, 0.16491E+00_JPRB, 0.16576E+00_JPRB, 0.16689E+00_JPRB, 0.16765E+00_JPRB, 0.16734E+00_JPRB ,&
&  0.16610E+00_JPRB, 0.16444E+00_JPRB, 0.16252E+00_JPRB, 0.16008E+00_JPRB, 0.15734E+00_JPRB, 0.15467E+00_JPRB, 0.15202E+00_JPRB ,&
&  0.15006E+00_JPRB, 0.14827E+00_JPRB, 0.14642E+00_JPRB, 0.14475E+00_JPRB, 0.14317E+00_JPRB, 0.14173E+00_JPRB, 0.14029E+00_JPRB ,&
&  0.13915E+00_JPRB, 0.13824E+00_JPRB, 0.13714E+00_JPRB, 0.13651E+00_JPRB, 0.13589E+00_JPRB, 0.13560E+00_JPRB, 0.13544E+00_JPRB ,&
&  0.13575E+00_JPRB, 0.13616E+00_JPRB, 0.13659E+00_JPRB, 0.13681E+00_JPRB, 0.13703E+00_JPRB, 0.13712E+00_JPRB, 0.13732E+00_JPRB ,&
&  0.13749E+00_JPRB, 0.13771E+00_JPRB, 0.13734E+00_JPRB, 0.13700E+00_JPRB, 0.13655E+00_JPRB, 0.13605E+00_JPRB, 0.13591E+00_JPRB ,&
&  0.13617E+00_JPRB, 0.13641E+00_JPRB, 0.13629E+00_JPRB, 0.13656E+00_JPRB, 0.13702E+00_JPRB, 0.13750E+00_JPRB, 0.13868E+00_JPRB ,&
&  0.13932E+00_JPRB, 0.13995E+00_JPRB, 0.14054E+00_JPRB, 0.14102E+00_JPRB, 0.14172E+00_JPRB, 0.14230E+00_JPRB, 0.14302E+00_JPRB ,&
&  0.14388E+00_JPRB, 0.14475E+00_JPRB, 0.14593E+00_JPRB, 0.14742E+00_JPRB, 0.15091E+00_JPRB, 0.15330E+00_JPRB, 0.15546E+00_JPRB ,&
&  0.15663E+00_JPRB, 0.15393E+00_JPRB, 0.16111E+00_JPRB, 0.16052E+00_JPRB, 0.16043E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.16867E+00_JPRB, 0.17113E+00_JPRB, 0.16994E+00_JPRB, 0.16985E+00_JPRB, 0.16938E+00_JPRB, 0.16930E+00_JPRB, 0.16838E+00_JPRB ,&
&  0.16817E+00_JPRB, 0.16613E+00_JPRB, 0.16567E+00_JPRB, 0.16396E+00_JPRB, 0.16217E+00_JPRB, 0.15974E+00_JPRB, 0.15734E+00_JPRB ,&
&  0.15490E+00_JPRB, 0.15275E+00_JPRB, 0.15126E+00_JPRB, 0.15052E+00_JPRB, 0.14929E+00_JPRB, 0.14915E+00_JPRB, 0.15080E+00_JPRB ,&
&  0.15118E+00_JPRB, 0.15221E+00_JPRB, 0.15237E+00_JPRB, 0.15362E+00_JPRB, 0.15524E+00_JPRB, 0.15608E+00_JPRB, 0.15842E+00_JPRB ,&
&  0.15998E+00_JPRB, 0.16148E+00_JPRB, 0.16688E+00_JPRB, 0.16996E+00_JPRB, 0.17177E+00_JPRB, 0.17437E+00_JPRB, 0.17433E+00_JPRB ,&
&  0.17304E+00_JPRB, 0.16949E+00_JPRB, 0.17270E+00_JPRB, 0.17071E+00_JPRB, 0.16446E+00_JPRB, 0.16155E+00_JPRB, 0.15639E+00_JPRB ,&
&  0.15322E+00_JPRB, 0.14959E+00_JPRB, 0.14685E+00_JPRB, 0.14476E+00_JPRB, 0.14200E+00_JPRB, 0.14022E+00_JPRB, 0.13823E+00_JPRB ,&
&  0.13634E+00_JPRB, 0.13549E+00_JPRB, 0.13416E+00_JPRB, 0.13393E+00_JPRB, 0.13400E+00_JPRB, 0.13433E+00_JPRB, 0.13492E+00_JPRB ,&
&  0.13572E+00_JPRB, 0.13648E+00_JPRB, 0.13718E+00_JPRB, 0.13778E+00_JPRB, 0.13828E+00_JPRB, 0.13859E+00_JPRB, 0.13848E+00_JPRB ,&
&  0.13856E+00_JPRB, 0.13844E+00_JPRB, 0.13798E+00_JPRB, 0.13772E+00_JPRB, 0.13684E+00_JPRB, 0.13643E+00_JPRB, 0.13602E+00_JPRB ,&
&  0.13583E+00_JPRB, 0.13584E+00_JPRB, 0.13577E+00_JPRB, 0.13596E+00_JPRB, 0.13624E+00_JPRB, 0.13689E+00_JPRB, 0.13744E+00_JPRB ,&
&  0.13789E+00_JPRB, 0.13872E+00_JPRB, 0.13922E+00_JPRB, 0.13991E+00_JPRB, 0.14086E+00_JPRB, 0.14133E+00_JPRB, 0.14247E+00_JPRB ,&
&  0.14346E+00_JPRB, 0.14462E+00_JPRB, 0.14669E+00_JPRB, 0.14879E+00_JPRB, 0.15176E+00_JPRB, 0.15558E+00_JPRB, 0.15728E+00_JPRB ,&
&  0.15890E+00_JPRB, 0.16049E+00_JPRB, 0.15954E+00_JPRB, 0.16417E+00_JPRB, 0.16634E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.17060E+00_JPRB, 0.17145E+00_JPRB, 0.16972E+00_JPRB, 0.17016E+00_JPRB, 0.17094E+00_JPRB, 0.17138E+00_JPRB, 0.17186E+00_JPRB ,&
&  0.16947E+00_JPRB, 0.16856E+00_JPRB, 0.16624E+00_JPRB, 0.16409E+00_JPRB, 0.16157E+00_JPRB, 0.15888E+00_JPRB, 0.15681E+00_JPRB ,&
&  0.15493E+00_JPRB, 0.15415E+00_JPRB, 0.15327E+00_JPRB, 0.15395E+00_JPRB, 0.15398E+00_JPRB, 0.15557E+00_JPRB, 0.15687E+00_JPRB ,&
&  0.15762E+00_JPRB, 0.15862E+00_JPRB, 0.15746E+00_JPRB, 0.15735E+00_JPRB, 0.15909E+00_JPRB, 0.16075E+00_JPRB, 0.16337E+00_JPRB ,&
&  0.16616E+00_JPRB, 0.16920E+00_JPRB, 0.17410E+00_JPRB, 0.17594E+00_JPRB, 0.17811E+00_JPRB, 0.17949E+00_JPRB, 0.17950E+00_JPRB ,&
&  0.17923E+00_JPRB, 0.17929E+00_JPRB, 0.18019E+00_JPRB, 0.17999E+00_JPRB, 0.17334E+00_JPRB, 0.16818E+00_JPRB, 0.16275E+00_JPRB ,&
&  0.15768E+00_JPRB, 0.15288E+00_JPRB, 0.14863E+00_JPRB, 0.14577E+00_JPRB, 0.14285E+00_JPRB, 0.14075E+00_JPRB, 0.13910E+00_JPRB ,&
&  0.13741E+00_JPRB, 0.13640E+00_JPRB, 0.13512E+00_JPRB, 0.13471E+00_JPRB, 0.13493E+00_JPRB, 0.13563E+00_JPRB, 0.13659E+00_JPRB ,&
&  0.13773E+00_JPRB, 0.13855E+00_JPRB, 0.13957E+00_JPRB, 0.14053E+00_JPRB, 0.14121E+00_JPRB, 0.14164E+00_JPRB, 0.14140E+00_JPRB ,&
&  0.14160E+00_JPRB, 0.14183E+00_JPRB, 0.14140E+00_JPRB, 0.14132E+00_JPRB, 0.14026E+00_JPRB, 0.13902E+00_JPRB, 0.13773E+00_JPRB ,&
&  0.13604E+00_JPRB, 0.13475E+00_JPRB, 0.13427E+00_JPRB, 0.13400E+00_JPRB, 0.13464E+00_JPRB, 0.13561E+00_JPRB, 0.13627E+00_JPRB ,&
&  0.13662E+00_JPRB, 0.13657E+00_JPRB, 0.13664E+00_JPRB, 0.13773E+00_JPRB, 0.13887E+00_JPRB, 0.14030E+00_JPRB, 0.14179E+00_JPRB ,&
&  0.14231E+00_JPRB, 0.14355E+00_JPRB, 0.14464E+00_JPRB, 0.14596E+00_JPRB, 0.15034E+00_JPRB, 0.15388E+00_JPRB, 0.16065E+00_JPRB ,&
&  0.16424E+00_JPRB, 0.16624E+00_JPRB, 0.16625E+00_JPRB, 0.17005E+00_JPRB, 0.17021E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.17508E+00_JPRB, 0.17701E+00_JPRB, 0.17872E+00_JPRB, 0.17784E+00_JPRB, 0.17801E+00_JPRB, 0.17803E+00_JPRB, 0.17811E+00_JPRB ,&
&  0.17807E+00_JPRB, 0.17749E+00_JPRB, 0.17773E+00_JPRB, 0.17606E+00_JPRB, 0.17509E+00_JPRB, 0.17148E+00_JPRB, 0.17014E+00_JPRB ,&
&  0.16822E+00_JPRB, 0.16771E+00_JPRB, 0.16793E+00_JPRB, 0.16906E+00_JPRB, 0.16951E+00_JPRB, 0.17075E+00_JPRB, 0.17007E+00_JPRB ,&
&  0.17024E+00_JPRB, 0.17018E+00_JPRB, 0.16898E+00_JPRB, 0.16866E+00_JPRB, 0.16912E+00_JPRB, 0.16963E+00_JPRB, 0.16996E+00_JPRB ,&
&  0.17178E+00_JPRB, 0.17400E+00_JPRB, 0.17614E+00_JPRB, 0.17752E+00_JPRB, 0.18260E+00_JPRB, 0.18409E+00_JPRB, 0.18598E+00_JPRB ,&
&  0.18708E+00_JPRB, 0.18852E+00_JPRB, 0.18952E+00_JPRB, 0.18894E+00_JPRB, 0.18776E+00_JPRB, 0.18432E+00_JPRB, 0.18303E+00_JPRB ,&
&  0.17302E+00_JPRB, 0.16626E+00_JPRB, 0.15957E+00_JPRB, 0.15366E+00_JPRB, 0.14939E+00_JPRB, 0.14645E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14202E+00_JPRB, 0.14008E+00_JPRB, 0.13800E+00_JPRB, 0.13800E+00_JPRB, 0.13872E+00_JPRB, 0.13981E+00_JPRB, 0.14088E+00_JPRB ,&
&  0.14213E+00_JPRB, 0.14285E+00_JPRB, 0.14318E+00_JPRB, 0.14387E+00_JPRB, 0.14402E+00_JPRB, 0.14435E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14571E+00_JPRB, 0.14608E+00_JPRB, 0.14611E+00_JPRB, 0.14737E+00_JPRB, 0.14579E+00_JPRB, 0.14329E+00_JPRB, 0.14050E+00_JPRB ,&
&  0.13777E+00_JPRB, 0.13605E+00_JPRB, 0.13463E+00_JPRB, 0.13407E+00_JPRB, 0.13441E+00_JPRB, 0.13509E+00_JPRB, 0.13599E+00_JPRB ,&
&  0.13621E+00_JPRB, 0.13679E+00_JPRB, 0.13724E+00_JPRB, 0.13810E+00_JPRB, 0.13902E+00_JPRB, 0.14005E+00_JPRB, 0.14115E+00_JPRB ,&
&  0.14280E+00_JPRB, 0.14473E+00_JPRB, 0.14665E+00_JPRB, 0.14862E+00_JPRB, 0.15175E+00_JPRB, 0.15553E+00_JPRB, 0.16156E+00_JPRB ,&
&  0.16712E+00_JPRB, 0.17479E+00_JPRB, 0.17375E+00_JPRB, 0.17595E+00_JPRB, 0.17618E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.18201E+00_JPRB, 0.18568E+00_JPRB, 0.18725E+00_JPRB, 0.18729E+00_JPRB, 0.18628E+00_JPRB, 0.18542E+00_JPRB, 0.18526E+00_JPRB ,&
&  0.18516E+00_JPRB, 0.18409E+00_JPRB, 0.18328E+00_JPRB, 0.18264E+00_JPRB, 0.18194E+00_JPRB, 0.18116E+00_JPRB, 0.18143E+00_JPRB ,&
&  0.18260E+00_JPRB, 0.18400E+00_JPRB, 0.18498E+00_JPRB, 0.18532E+00_JPRB, 0.18487E+00_JPRB, 0.18517E+00_JPRB, 0.18599E+00_JPRB ,&
&  0.18595E+00_JPRB, 0.18625E+00_JPRB, 0.18949E+00_JPRB, 0.18998E+00_JPRB, 0.19094E+00_JPRB, 0.19106E+00_JPRB, 0.19114E+00_JPRB ,&
&  0.18660E+00_JPRB, 0.18149E+00_JPRB, 0.17849E+00_JPRB, 0.18054E+00_JPRB, 0.18875E+00_JPRB, 0.19035E+00_JPRB, 0.19358E+00_JPRB ,&
&  0.19566E+00_JPRB, 0.19677E+00_JPRB, 0.19810E+00_JPRB, 0.19926E+00_JPRB, 0.20067E+00_JPRB, 0.19979E+00_JPRB, 0.19766E+00_JPRB ,&
&  0.19503E+00_JPRB, 0.19406E+00_JPRB, 0.19089E+00_JPRB, 0.17718E+00_JPRB, 0.16871E+00_JPRB, 0.16393E+00_JPRB, 0.15892E+00_JPRB ,&
&  0.15437E+00_JPRB, 0.15176E+00_JPRB, 0.15103E+00_JPRB, 0.14979E+00_JPRB, 0.15063E+00_JPRB, 0.15249E+00_JPRB, 0.15327E+00_JPRB ,&
&  0.15295E+00_JPRB, 0.15331E+00_JPRB, 0.15299E+00_JPRB, 0.15202E+00_JPRB, 0.15035E+00_JPRB, 0.14927E+00_JPRB, 0.14863E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14885E+00_JPRB, 0.15264E+00_JPRB, 0.15398E+00_JPRB, 0.14948E+00_JPRB, 0.14794E+00_JPRB, 0.14602E+00_JPRB ,&
&  0.14126E+00_JPRB, 0.13858E+00_JPRB, 0.13714E+00_JPRB, 0.13658E+00_JPRB, 0.13749E+00_JPRB, 0.13880E+00_JPRB, 0.13947E+00_JPRB ,&
&  0.13970E+00_JPRB, 0.14065E+00_JPRB, 0.14113E+00_JPRB, 0.14168E+00_JPRB, 0.14206E+00_JPRB, 0.14286E+00_JPRB, 0.14402E+00_JPRB ,&
&  0.14549E+00_JPRB, 0.14812E+00_JPRB, 0.15075E+00_JPRB, 0.15369E+00_JPRB, 0.15546E+00_JPRB, 0.15685E+00_JPRB, 0.16000E+00_JPRB ,&
&  0.16203E+00_JPRB, 0.16881E+00_JPRB, 0.17875E+00_JPRB, 0.18225E+00_JPRB, 0.17769E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.19414E+00_JPRB, 0.19599E+00_JPRB, 0.19664E+00_JPRB, 0.19667E+00_JPRB, 0.19656E+00_JPRB, 0.19708E+00_JPRB, 0.19762E+00_JPRB ,&
&  0.19762E+00_JPRB, 0.19694E+00_JPRB, 0.19582E+00_JPRB, 0.19393E+00_JPRB, 0.19166E+00_JPRB, 0.19035E+00_JPRB, 0.19138E+00_JPRB ,&
&  0.19231E+00_JPRB, 0.19215E+00_JPRB, 0.19211E+00_JPRB, 0.19187E+00_JPRB, 0.19139E+00_JPRB, 0.19157E+00_JPRB, 0.19272E+00_JPRB ,&
&  0.19407E+00_JPRB, 0.19497E+00_JPRB, 0.19637E+00_JPRB, 0.19757E+00_JPRB, 0.19963E+00_JPRB, 0.20149E+00_JPRB, 0.20225E+00_JPRB ,&
&  0.19993E+00_JPRB, 0.19225E+00_JPRB, 0.18688E+00_JPRB, 0.18935E+00_JPRB, 0.19770E+00_JPRB, 0.20264E+00_JPRB, 0.20567E+00_JPRB ,&
&  0.20775E+00_JPRB, 0.20888E+00_JPRB, 0.20917E+00_JPRB, 0.21044E+00_JPRB, 0.21234E+00_JPRB, 0.21417E+00_JPRB, 0.21378E+00_JPRB ,&
&  0.21180E+00_JPRB, 0.21009E+00_JPRB, 0.20762E+00_JPRB, 0.20338E+00_JPRB, 0.19763E+00_JPRB, 0.19198E+00_JPRB, 0.18715E+00_JPRB ,&
&  0.18286E+00_JPRB, 0.17867E+00_JPRB, 0.17512E+00_JPRB, 0.17283E+00_JPRB, 0.17211E+00_JPRB, 0.17328E+00_JPRB, 0.17301E+00_JPRB ,&
&  0.17281E+00_JPRB, 0.17101E+00_JPRB, 0.16882E+00_JPRB, 0.16625E+00_JPRB, 0.16400E+00_JPRB, 0.16006E+00_JPRB, 0.15527E+00_JPRB ,&
&  0.15278E+00_JPRB, 0.15498E+00_JPRB, 0.16136E+00_JPRB, 0.16665E+00_JPRB, 0.16734E+00_JPRB, 0.16021E+00_JPRB, 0.15627E+00_JPRB ,&
&  0.15228E+00_JPRB, 0.14840E+00_JPRB, 0.14597E+00_JPRB, 0.14540E+00_JPRB, 0.14556E+00_JPRB, 0.14574E+00_JPRB, 0.14664E+00_JPRB ,&
&  0.14808E+00_JPRB, 0.14938E+00_JPRB, 0.15012E+00_JPRB, 0.15093E+00_JPRB, 0.15127E+00_JPRB, 0.15136E+00_JPRB, 0.15149E+00_JPRB ,&
&  0.15296E+00_JPRB, 0.15607E+00_JPRB, 0.16030E+00_JPRB, 0.16507E+00_JPRB, 0.16875E+00_JPRB, 0.17211E+00_JPRB, 0.17525E+00_JPRB ,&
&  0.17857E+00_JPRB, 0.18083E+00_JPRB, 0.18383E+00_JPRB, 0.18777E+00_JPRB, 0.19118E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.20486E+00_JPRB, 0.20705E+00_JPRB, 0.20786E+00_JPRB, 0.20757E+00_JPRB, 0.20718E+00_JPRB, 0.20816E+00_JPRB, 0.20950E+00_JPRB ,&
&  0.21020E+00_JPRB, 0.21032E+00_JPRB, 0.20950E+00_JPRB, 0.20787E+00_JPRB, 0.20669E+00_JPRB, 0.20706E+00_JPRB, 0.20555E+00_JPRB ,&
&  0.20400E+00_JPRB, 0.20394E+00_JPRB, 0.20239E+00_JPRB, 0.20069E+00_JPRB, 0.19826E+00_JPRB, 0.19683E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.19945E+00_JPRB, 0.20014E+00_JPRB, 0.20091E+00_JPRB, 0.20195E+00_JPRB, 0.20385E+00_JPRB, 0.20607E+00_JPRB, 0.20760E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.20815E+00_JPRB, 0.20652E+00_JPRB, 0.20934E+00_JPRB, 0.21240E+00_JPRB, 0.21629E+00_JPRB, 0.21900E+00_JPRB ,&
&  0.22149E+00_JPRB, 0.22339E+00_JPRB, 0.22426E+00_JPRB, 0.22418E+00_JPRB, 0.22450E+00_JPRB, 0.22616E+00_JPRB, 0.22731E+00_JPRB ,&
&  0.22759E+00_JPRB, 0.22710E+00_JPRB, 0.22568E+00_JPRB, 0.22431E+00_JPRB, 0.22101E+00_JPRB, 0.21602E+00_JPRB, 0.20856E+00_JPRB ,&
&  0.20079E+00_JPRB, 0.19451E+00_JPRB, 0.18977E+00_JPRB, 0.18594E+00_JPRB, 0.18325E+00_JPRB, 0.18288E+00_JPRB, 0.18276E+00_JPRB ,&
&  0.18228E+00_JPRB, 0.18184E+00_JPRB, 0.18317E+00_JPRB, 0.18392E+00_JPRB, 0.18020E+00_JPRB, 0.17490E+00_JPRB, 0.17042E+00_JPRB ,&
&  0.16939E+00_JPRB, 0.17057E+00_JPRB, 0.17250E+00_JPRB, 0.17446E+00_JPRB, 0.17765E+00_JPRB, 0.17843E+00_JPRB, 0.17749E+00_JPRB ,&
&  0.17239E+00_JPRB, 0.16696E+00_JPRB, 0.16386E+00_JPRB, 0.16151E+00_JPRB, 0.16005E+00_JPRB, 0.15988E+00_JPRB, 0.16050E+00_JPRB ,&
&  0.16251E+00_JPRB, 0.16442E+00_JPRB, 0.16671E+00_JPRB, 0.16785E+00_JPRB, 0.16757E+00_JPRB, 0.16714E+00_JPRB, 0.16640E+00_JPRB ,&
&  0.16755E+00_JPRB, 0.17166E+00_JPRB, 0.17800E+00_JPRB, 0.18149E+00_JPRB, 0.18527E+00_JPRB, 0.18780E+00_JPRB, 0.18907E+00_JPRB ,&
&  0.18998E+00_JPRB, 0.19059E+00_JPRB, 0.19130E+00_JPRB, 0.19561E+00_JPRB, 0.20129E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.21764E+00_JPRB, 0.22000E+00_JPRB, 0.22134E+00_JPRB, 0.22133E+00_JPRB, 0.22092E+00_JPRB, 0.22172E+00_JPRB, 0.22230E+00_JPRB ,&
&  0.22295E+00_JPRB, 0.22320E+00_JPRB, 0.22212E+00_JPRB, 0.22121E+00_JPRB, 0.22289E+00_JPRB, 0.22248E+00_JPRB, 0.22199E+00_JPRB ,&
&  0.22035E+00_JPRB, 0.21834E+00_JPRB, 0.21541E+00_JPRB, 0.21206E+00_JPRB, 0.20797E+00_JPRB, 0.20541E+00_JPRB, 0.20560E+00_JPRB ,&
&  0.20705E+00_JPRB, 0.20762E+00_JPRB, 0.20848E+00_JPRB, 0.20968E+00_JPRB, 0.21208E+00_JPRB, 0.21512E+00_JPRB, 0.21741E+00_JPRB ,&
&  0.21927E+00_JPRB, 0.22245E+00_JPRB, 0.22445E+00_JPRB, 0.22326E+00_JPRB, 0.22747E+00_JPRB, 0.23143E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23876E+00_JPRB, 0.24086E+00_JPRB, 0.24093E+00_JPRB, 0.23985E+00_JPRB, 0.23894E+00_JPRB, 0.23974E+00_JPRB, 0.23999E+00_JPRB ,&
&  0.24027E+00_JPRB, 0.24234E+00_JPRB, 0.24489E+00_JPRB, 0.24570E+00_JPRB, 0.24409E+00_JPRB, 0.23993E+00_JPRB, 0.23341E+00_JPRB ,&
&  0.22502E+00_JPRB, 0.21617E+00_JPRB, 0.20895E+00_JPRB, 0.20320E+00_JPRB, 0.19875E+00_JPRB, 0.19512E+00_JPRB, 0.19216E+00_JPRB ,&
&  0.18890E+00_JPRB, 0.18661E+00_JPRB, 0.18652E+00_JPRB, 0.18712E+00_JPRB, 0.18729E+00_JPRB, 0.18588E+00_JPRB, 0.18484E+00_JPRB ,&
&  0.18389E+00_JPRB, 0.18431E+00_JPRB, 0.18513E+00_JPRB, 0.18544E+00_JPRB, 0.18557E+00_JPRB, 0.18526E+00_JPRB, 0.18496E+00_JPRB ,&
&  0.18576E+00_JPRB, 0.18599E+00_JPRB, 0.18510E+00_JPRB, 0.18342E+00_JPRB, 0.18198E+00_JPRB, 0.18173E+00_JPRB, 0.18195E+00_JPRB ,&
&  0.18250E+00_JPRB, 0.18361E+00_JPRB, 0.18526E+00_JPRB, 0.18698E+00_JPRB, 0.18833E+00_JPRB, 0.18893E+00_JPRB, 0.18914E+00_JPRB ,&
&  0.18932E+00_JPRB, 0.18916E+00_JPRB, 0.19001E+00_JPRB, 0.19284E+00_JPRB, 0.19586E+00_JPRB, 0.19832E+00_JPRB, 0.19970E+00_JPRB ,&
&  0.19938E+00_JPRB, 0.20020E+00_JPRB, 0.20344E+00_JPRB, 0.20952E+00_JPRB, 0.21407E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.23485E+00_JPRB, 0.23744E+00_JPRB, 0.24018E+00_JPRB, 0.24054E+00_JPRB, 0.23902E+00_JPRB, 0.23831E+00_JPRB, 0.23856E+00_JPRB ,&
&  0.23906E+00_JPRB, 0.23932E+00_JPRB, 0.23820E+00_JPRB, 0.23813E+00_JPRB, 0.23820E+00_JPRB, 0.23729E+00_JPRB, 0.23656E+00_JPRB ,&
&  0.23523E+00_JPRB, 0.23228E+00_JPRB, 0.22811E+00_JPRB, 0.22299E+00_JPRB, 0.21915E+00_JPRB, 0.21709E+00_JPRB, 0.21672E+00_JPRB ,&
&  0.21735E+00_JPRB, 0.21785E+00_JPRB, 0.22035E+00_JPRB, 0.22260E+00_JPRB, 0.22544E+00_JPRB, 0.22917E+00_JPRB, 0.23306E+00_JPRB ,&
&  0.23597E+00_JPRB, 0.23898E+00_JPRB, 0.24192E+00_JPRB, 0.24463E+00_JPRB, 0.24816E+00_JPRB, 0.25373E+00_JPRB, 0.25909E+00_JPRB ,&
&  0.26472E+00_JPRB, 0.26749E+00_JPRB, 0.26475E+00_JPRB, 0.26149E+00_JPRB, 0.26113E+00_JPRB, 0.26256E+00_JPRB, 0.26408E+00_JPRB ,&
&  0.26539E+00_JPRB, 0.26675E+00_JPRB, 0.26780E+00_JPRB, 0.26720E+00_JPRB, 0.26436E+00_JPRB, 0.25957E+00_JPRB, 0.25330E+00_JPRB ,&
&  0.24648E+00_JPRB, 0.23964E+00_JPRB, 0.23305E+00_JPRB, 0.22638E+00_JPRB, 0.22064E+00_JPRB, 0.21511E+00_JPRB, 0.21002E+00_JPRB ,&
&  0.20526E+00_JPRB, 0.20075E+00_JPRB, 0.19733E+00_JPRB, 0.19520E+00_JPRB, 0.19464E+00_JPRB, 0.19357E+00_JPRB, 0.19355E+00_JPRB ,&
&  0.19305E+00_JPRB, 0.19288E+00_JPRB, 0.19304E+00_JPRB, 0.19392E+00_JPRB, 0.19540E+00_JPRB, 0.19574E+00_JPRB, 0.19476E+00_JPRB ,&
&  0.19420E+00_JPRB, 0.19536E+00_JPRB, 0.19516E+00_JPRB, 0.19294E+00_JPRB, 0.19018E+00_JPRB, 0.18953E+00_JPRB, 0.19089E+00_JPRB ,&
&  0.19348E+00_JPRB, 0.19604E+00_JPRB, 0.19822E+00_JPRB, 0.20019E+00_JPRB, 0.20185E+00_JPRB, 0.20393E+00_JPRB, 0.20584E+00_JPRB ,&
&  0.20741E+00_JPRB, 0.20812E+00_JPRB, 0.20952E+00_JPRB, 0.21103E+00_JPRB, 0.21263E+00_JPRB, 0.21379E+00_JPRB, 0.21371E+00_JPRB ,&
&  0.21333E+00_JPRB, 0.21490E+00_JPRB, 0.21969E+00_JPRB, 0.22704E+00_JPRB, 0.23185E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.25578E+00_JPRB, 0.25815E+00_JPRB, 0.26121E+00_JPRB, 0.26374E+00_JPRB, 0.26477E+00_JPRB, 0.26441E+00_JPRB, 0.26372E+00_JPRB ,&
&  0.26365E+00_JPRB, 0.26369E+00_JPRB, 0.26233E+00_JPRB, 0.25987E+00_JPRB, 0.25729E+00_JPRB, 0.25454E+00_JPRB, 0.25244E+00_JPRB ,&
&  0.25043E+00_JPRB, 0.24835E+00_JPRB, 0.24470E+00_JPRB, 0.23933E+00_JPRB, 0.23375E+00_JPRB, 0.23048E+00_JPRB, 0.22889E+00_JPRB ,&
&  0.22874E+00_JPRB, 0.22922E+00_JPRB, 0.23172E+00_JPRB, 0.23351E+00_JPRB, 0.23599E+00_JPRB, 0.24011E+00_JPRB, 0.24575E+00_JPRB ,&
&  0.25108E+00_JPRB, 0.25612E+00_JPRB, 0.26127E+00_JPRB, 0.26561E+00_JPRB, 0.27005E+00_JPRB, 0.27635E+00_JPRB, 0.28398E+00_JPRB ,&
&  0.29139E+00_JPRB, 0.29612E+00_JPRB, 0.29629E+00_JPRB, 0.29677E+00_JPRB, 0.29928E+00_JPRB, 0.30244E+00_JPRB, 0.30456E+00_JPRB ,&
&  0.30605E+00_JPRB, 0.30433E+00_JPRB, 0.30148E+00_JPRB, 0.29732E+00_JPRB, 0.29281E+00_JPRB, 0.28783E+00_JPRB, 0.28137E+00_JPRB ,&
&  0.27370E+00_JPRB, 0.26532E+00_JPRB, 0.25718E+00_JPRB, 0.24934E+00_JPRB, 0.24174E+00_JPRB, 0.23472E+00_JPRB, 0.22839E+00_JPRB ,&
&  0.22235E+00_JPRB, 0.21770E+00_JPRB, 0.21326E+00_JPRB, 0.21060E+00_JPRB, 0.20902E+00_JPRB, 0.20719E+00_JPRB, 0.20685E+00_JPRB ,&
&  0.20597E+00_JPRB, 0.20544E+00_JPRB, 0.20524E+00_JPRB, 0.20680E+00_JPRB, 0.20985E+00_JPRB, 0.21169E+00_JPRB, 0.21053E+00_JPRB ,&
&  0.20730E+00_JPRB, 0.20528E+00_JPRB, 0.20413E+00_JPRB, 0.20326E+00_JPRB, 0.20105E+00_JPRB, 0.20081E+00_JPRB, 0.20273E+00_JPRB ,&
&  0.20677E+00_JPRB, 0.21122E+00_JPRB, 0.21511E+00_JPRB, 0.21746E+00_JPRB, 0.21984E+00_JPRB, 0.22172E+00_JPRB, 0.22338E+00_JPRB ,&
&  0.22520E+00_JPRB, 0.22652E+00_JPRB, 0.22752E+00_JPRB, 0.22891E+00_JPRB, 0.23127E+00_JPRB, 0.23365E+00_JPRB, 0.23603E+00_JPRB ,&
&  0.23863E+00_JPRB, 0.24257E+00_JPRB, 0.24657E+00_JPRB, 0.24961E+00_JPRB, 0.25335E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.28671E+00_JPRB, 0.28880E+00_JPRB, 0.29116E+00_JPRB, 0.29164E+00_JPRB, 0.29109E+00_JPRB, 0.29107E+00_JPRB, 0.28998E+00_JPRB ,&
&  0.28770E+00_JPRB, 0.28643E+00_JPRB, 0.28665E+00_JPRB, 0.28527E+00_JPRB, 0.28240E+00_JPRB, 0.27882E+00_JPRB, 0.27492E+00_JPRB ,&
&  0.27188E+00_JPRB, 0.26883E+00_JPRB, 0.26378E+00_JPRB, 0.25711E+00_JPRB, 0.25136E+00_JPRB, 0.24765E+00_JPRB, 0.24570E+00_JPRB ,&
&  0.24486E+00_JPRB, 0.24483E+00_JPRB, 0.24444E+00_JPRB, 0.24504E+00_JPRB, 0.24711E+00_JPRB, 0.25188E+00_JPRB, 0.25895E+00_JPRB ,&
&  0.26605E+00_JPRB, 0.27261E+00_JPRB, 0.27943E+00_JPRB, 0.28686E+00_JPRB, 0.29610E+00_JPRB, 0.30869E+00_JPRB, 0.32142E+00_JPRB ,&
&  0.33122E+00_JPRB, 0.33415E+00_JPRB, 0.33489E+00_JPRB, 0.33913E+00_JPRB, 0.34767E+00_JPRB, 0.35422E+00_JPRB, 0.35887E+00_JPRB ,&
&  0.36095E+00_JPRB, 0.35940E+00_JPRB, 0.35653E+00_JPRB, 0.35086E+00_JPRB, 0.34286E+00_JPRB, 0.33289E+00_JPRB, 0.32235E+00_JPRB ,&
&  0.31188E+00_JPRB, 0.30104E+00_JPRB, 0.28923E+00_JPRB, 0.27788E+00_JPRB, 0.26809E+00_JPRB, 0.25938E+00_JPRB, 0.25075E+00_JPRB ,&
&  0.24283E+00_JPRB, 0.23650E+00_JPRB, 0.23149E+00_JPRB, 0.22806E+00_JPRB, 0.22532E+00_JPRB, 0.22314E+00_JPRB, 0.22250E+00_JPRB ,&
&  0.22268E+00_JPRB, 0.22389E+00_JPRB, 0.22446E+00_JPRB, 0.22587E+00_JPRB, 0.22832E+00_JPRB, 0.22938E+00_JPRB, 0.22833E+00_JPRB ,&
&  0.22452E+00_JPRB, 0.22058E+00_JPRB, 0.21681E+00_JPRB, 0.21610E+00_JPRB, 0.21602E+00_JPRB, 0.21718E+00_JPRB, 0.21895E+00_JPRB ,&
&  0.22254E+00_JPRB, 0.22781E+00_JPRB, 0.23249E+00_JPRB, 0.23578E+00_JPRB, 0.23945E+00_JPRB, 0.24268E+00_JPRB, 0.24591E+00_JPRB ,&
&  0.24959E+00_JPRB, 0.25357E+00_JPRB, 0.25895E+00_JPRB, 0.26429E+00_JPRB, 0.26913E+00_JPRB, 0.27304E+00_JPRB, 0.27604E+00_JPRB ,&
&  0.27813E+00_JPRB, 0.27887E+00_JPRB, 0.27940E+00_JPRB, 0.28125E+00_JPRB, 0.28421E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.32255E+00_JPRB, 0.32330E+00_JPRB, 0.32448E+00_JPRB, 0.32506E+00_JPRB, 0.32253E+00_JPRB, 0.31945E+00_JPRB, 0.31585E+00_JPRB ,&
&  0.31415E+00_JPRB, 0.31291E+00_JPRB, 0.31233E+00_JPRB, 0.30918E+00_JPRB, 0.30664E+00_JPRB, 0.30501E+00_JPRB, 0.30134E+00_JPRB ,&
&  0.29798E+00_JPRB, 0.29066E+00_JPRB, 0.28287E+00_JPRB, 0.27713E+00_JPRB, 0.27348E+00_JPRB, 0.27101E+00_JPRB, 0.26599E+00_JPRB ,&
&  0.26242E+00_JPRB, 0.25959E+00_JPRB, 0.25822E+00_JPRB, 0.25834E+00_JPRB, 0.26036E+00_JPRB, 0.26608E+00_JPRB, 0.27572E+00_JPRB ,&
&  0.28268E+00_JPRB, 0.29133E+00_JPRB, 0.29193E+00_JPRB, 0.30610E+00_JPRB, 0.32093E+00_JPRB, 0.34313E+00_JPRB, 0.36273E+00_JPRB ,&
&  0.37409E+00_JPRB, 0.38133E+00_JPRB, 0.39208E+00_JPRB, 0.40357E+00_JPRB, 0.40909E+00_JPRB, 0.41169E+00_JPRB, 0.41078E+00_JPRB ,&
&  0.40740E+00_JPRB, 0.40221E+00_JPRB, 0.39604E+00_JPRB, 0.38818E+00_JPRB, 0.37730E+00_JPRB, 0.36479E+00_JPRB, 0.35224E+00_JPRB ,&
&  0.34069E+00_JPRB, 0.32923E+00_JPRB, 0.31793E+00_JPRB, 0.30598E+00_JPRB, 0.29413E+00_JPRB, 0.28322E+00_JPRB, 0.27226E+00_JPRB ,&
&  0.26306E+00_JPRB, 0.25589E+00_JPRB, 0.25055E+00_JPRB, 0.24670E+00_JPRB, 0.24379E+00_JPRB, 0.24093E+00_JPRB, 0.23946E+00_JPRB ,&
&  0.23935E+00_JPRB, 0.24076E+00_JPRB, 0.24297E+00_JPRB, 0.24486E+00_JPRB, 0.24721E+00_JPRB, 0.24771E+00_JPRB, 0.24523E+00_JPRB ,&
&  0.24260E+00_JPRB, 0.24137E+00_JPRB, 0.23718E+00_JPRB, 0.23468E+00_JPRB, 0.23419E+00_JPRB, 0.23468E+00_JPRB, 0.23656E+00_JPRB ,&
&  0.24025E+00_JPRB, 0.24538E+00_JPRB, 0.25009E+00_JPRB, 0.25415E+00_JPRB, 0.25762E+00_JPRB, 0.26126E+00_JPRB, 0.26557E+00_JPRB ,&
&  0.27013E+00_JPRB, 0.27455E+00_JPRB, 0.28148E+00_JPRB, 0.28915E+00_JPRB, 0.29576E+00_JPRB, 0.30295E+00_JPRB, 0.30770E+00_JPRB ,&
&  0.30982E+00_JPRB, 0.31098E+00_JPRB, 0.31379E+00_JPRB, 0.31787E+00_JPRB, 0.32115E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.36959E+00_JPRB, 0.36787E+00_JPRB, 0.36514E+00_JPRB, 0.36261E+00_JPRB, 0.36076E+00_JPRB, 0.35668E+00_JPRB, 0.35148E+00_JPRB ,&
&  0.34778E+00_JPRB, 0.34434E+00_JPRB, 0.34172E+00_JPRB, 0.33792E+00_JPRB, 0.33569E+00_JPRB, 0.33421E+00_JPRB, 0.32997E+00_JPRB ,&
&  0.32509E+00_JPRB, 0.31849E+00_JPRB, 0.31211E+00_JPRB, 0.30639E+00_JPRB, 0.30096E+00_JPRB, 0.29591E+00_JPRB, 0.29021E+00_JPRB ,&
&  0.28478E+00_JPRB, 0.28073E+00_JPRB, 0.27852E+00_JPRB, 0.27657E+00_JPRB, 0.27780E+00_JPRB, 0.28356E+00_JPRB, 0.29306E+00_JPRB ,&
&  0.29926E+00_JPRB, 0.30126E+00_JPRB, 0.31271E+00_JPRB, 0.32441E+00_JPRB, 0.34705E+00_JPRB, 0.37958E+00_JPRB, 0.40341E+00_JPRB ,&
&  0.41579E+00_JPRB, 0.42916E+00_JPRB, 0.44498E+00_JPRB, 0.45601E+00_JPRB, 0.45832E+00_JPRB, 0.45254E+00_JPRB, 0.44708E+00_JPRB ,&
&  0.44043E+00_JPRB, 0.42926E+00_JPRB, 0.41772E+00_JPRB, 0.40362E+00_JPRB, 0.38835E+00_JPRB, 0.37539E+00_JPRB, 0.36473E+00_JPRB ,&
&  0.35487E+00_JPRB, 0.34545E+00_JPRB, 0.33679E+00_JPRB, 0.32808E+00_JPRB, 0.31858E+00_JPRB, 0.30851E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.28758E+00_JPRB, 0.27874E+00_JPRB, 0.27168E+00_JPRB, 0.26777E+00_JPRB, 0.26544E+00_JPRB, 0.26354E+00_JPRB, 0.26221E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26377E+00_JPRB, 0.26584E+00_JPRB, 0.26782E+00_JPRB, 0.26950E+00_JPRB, 0.26987E+00_JPRB, 0.26821E+00_JPRB ,&
&  0.26648E+00_JPRB, 0.26596E+00_JPRB, 0.26591E+00_JPRB, 0.26352E+00_JPRB, 0.26094E+00_JPRB, 0.26138E+00_JPRB, 0.26341E+00_JPRB ,&
&  0.26725E+00_JPRB, 0.27079E+00_JPRB, 0.27383E+00_JPRB, 0.27634E+00_JPRB, 0.27827E+00_JPRB, 0.28166E+00_JPRB, 0.28583E+00_JPRB ,&
&  0.29099E+00_JPRB, 0.29738E+00_JPRB, 0.30634E+00_JPRB, 0.31736E+00_JPRB, 0.32767E+00_JPRB, 0.33672E+00_JPRB, 0.34316E+00_JPRB ,&
&  0.34770E+00_JPRB, 0.35180E+00_JPRB, 0.35643E+00_JPRB, 0.36200E+00_JPRB, 0.36717E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.42337E+00_JPRB, 0.41803E+00_JPRB, 0.41044E+00_JPRB, 0.40357E+00_JPRB, 0.39976E+00_JPRB, 0.39619E+00_JPRB, 0.39023E+00_JPRB ,&
&  0.38463E+00_JPRB, 0.37954E+00_JPRB, 0.37446E+00_JPRB, 0.37006E+00_JPRB, 0.36648E+00_JPRB, 0.36366E+00_JPRB, 0.35943E+00_JPRB ,&
&  0.35362E+00_JPRB, 0.34667E+00_JPRB, 0.33911E+00_JPRB, 0.33299E+00_JPRB, 0.32547E+00_JPRB, 0.31768E+00_JPRB, 0.31189E+00_JPRB ,&
&  0.30705E+00_JPRB, 0.30367E+00_JPRB, 0.30237E+00_JPRB, 0.30052E+00_JPRB, 0.30120E+00_JPRB, 0.30422E+00_JPRB, 0.31268E+00_JPRB ,&
&  0.32387E+00_JPRB, 0.30560E+00_JPRB, 0.32132E+00_JPRB, 0.33521E+00_JPRB, 0.36465E+00_JPRB, 0.40931E+00_JPRB, 0.43994E+00_JPRB ,&
&  0.45868E+00_JPRB, 0.47309E+00_JPRB, 0.48553E+00_JPRB, 0.48920E+00_JPRB, 0.48154E+00_JPRB, 0.47507E+00_JPRB, 0.46987E+00_JPRB ,&
&  0.46042E+00_JPRB, 0.44761E+00_JPRB, 0.43112E+00_JPRB, 0.41376E+00_JPRB, 0.39787E+00_JPRB, 0.38601E+00_JPRB, 0.37631E+00_JPRB ,&
&  0.36851E+00_JPRB, 0.36140E+00_JPRB, 0.35531E+00_JPRB, 0.34773E+00_JPRB, 0.33892E+00_JPRB, 0.32982E+00_JPRB, 0.32142E+00_JPRB ,&
&  0.31306E+00_JPRB, 0.30480E+00_JPRB, 0.29759E+00_JPRB, 0.29163E+00_JPRB, 0.28833E+00_JPRB, 0.28687E+00_JPRB, 0.28670E+00_JPRB ,&
&  0.28767E+00_JPRB, 0.28995E+00_JPRB, 0.29220E+00_JPRB, 0.29384E+00_JPRB, 0.29481E+00_JPRB, 0.29431E+00_JPRB, 0.29210E+00_JPRB ,&
&  0.29186E+00_JPRB, 0.29185E+00_JPRB, 0.29195E+00_JPRB, 0.29274E+00_JPRB, 0.29268E+00_JPRB, 0.29298E+00_JPRB, 0.29503E+00_JPRB ,&
&  0.29710E+00_JPRB, 0.29838E+00_JPRB, 0.29983E+00_JPRB, 0.30169E+00_JPRB, 0.30412E+00_JPRB, 0.30794E+00_JPRB, 0.31327E+00_JPRB ,&
&  0.31928E+00_JPRB, 0.32603E+00_JPRB, 0.33576E+00_JPRB, 0.34847E+00_JPRB, 0.36169E+00_JPRB, 0.37484E+00_JPRB, 0.38762E+00_JPRB ,&
&  0.39544E+00_JPRB, 0.40317E+00_JPRB, 0.41320E+00_JPRB, 0.41912E+00_JPRB, 0.42385E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.49048E+00_JPRB, 0.47962E+00_JPRB, 0.46576E+00_JPRB, 0.45551E+00_JPRB, 0.44958E+00_JPRB, 0.44321E+00_JPRB, 0.43694E+00_JPRB ,&
&  0.42863E+00_JPRB, 0.41964E+00_JPRB, 0.41106E+00_JPRB, 0.40371E+00_JPRB, 0.39788E+00_JPRB, 0.39358E+00_JPRB, 0.38882E+00_JPRB ,&
&  0.38266E+00_JPRB, 0.37593E+00_JPRB, 0.36918E+00_JPRB, 0.36161E+00_JPRB, 0.35170E+00_JPRB, 0.34379E+00_JPRB, 0.33753E+00_JPRB ,&
&  0.33261E+00_JPRB, 0.32869E+00_JPRB, 0.32724E+00_JPRB, 0.32538E+00_JPRB, 0.32338E+00_JPRB, 0.32342E+00_JPRB, 0.33373E+00_JPRB ,&
&  0.32993E+00_JPRB, 0.30590E+00_JPRB, 0.32284E+00_JPRB, 0.34551E+00_JPRB, 0.38072E+00_JPRB, 0.43933E+00_JPRB, 0.47949E+00_JPRB ,&
&  0.50266E+00_JPRB, 0.51408E+00_JPRB, 0.51852E+00_JPRB, 0.51357E+00_JPRB, 0.50569E+00_JPRB, 0.50250E+00_JPRB, 0.49475E+00_JPRB ,&
&  0.48030E+00_JPRB, 0.46293E+00_JPRB, 0.44635E+00_JPRB, 0.43419E+00_JPRB, 0.42157E+00_JPRB, 0.41162E+00_JPRB, 0.40115E+00_JPRB ,&
&  0.39075E+00_JPRB, 0.38231E+00_JPRB, 0.37755E+00_JPRB, 0.37406E+00_JPRB, 0.36942E+00_JPRB, 0.36199E+00_JPRB, 0.35341E+00_JPRB ,&
&  0.34614E+00_JPRB, 0.34033E+00_JPRB, 0.33529E+00_JPRB, 0.33090E+00_JPRB, 0.32860E+00_JPRB, 0.32762E+00_JPRB, 0.32632E+00_JPRB ,&
&  0.32627E+00_JPRB, 0.32671E+00_JPRB, 0.32753E+00_JPRB, 0.32802E+00_JPRB, 0.32772E+00_JPRB, 0.32664E+00_JPRB, 0.32708E+00_JPRB ,&
&  0.32855E+00_JPRB, 0.32741E+00_JPRB, 0.32689E+00_JPRB, 0.32892E+00_JPRB, 0.33127E+00_JPRB, 0.33265E+00_JPRB, 0.33279E+00_JPRB ,&
&  0.33176E+00_JPRB, 0.33165E+00_JPRB, 0.33312E+00_JPRB, 0.33654E+00_JPRB, 0.34090E+00_JPRB, 0.34721E+00_JPRB, 0.35490E+00_JPRB ,&
&  0.36294E+00_JPRB, 0.37296E+00_JPRB, 0.38587E+00_JPRB, 0.40807E+00_JPRB, 0.42837E+00_JPRB, 0.44720E+00_JPRB, 0.46537E+00_JPRB ,&
&  0.47447E+00_JPRB, 0.48054E+00_JPRB, 0.48980E+00_JPRB, 0.49194E+00_JPRB, 0.49327E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.52615E+00_JPRB, 0.52309E+00_JPRB, 0.51261E+00_JPRB, 0.50017E+00_JPRB, 0.48926E+00_JPRB, 0.47858E+00_JPRB, 0.46997E+00_JPRB ,&
&  0.45905E+00_JPRB, 0.44663E+00_JPRB, 0.43741E+00_JPRB, 0.43296E+00_JPRB, 0.43017E+00_JPRB, 0.42617E+00_JPRB, 0.42017E+00_JPRB ,&
&  0.41295E+00_JPRB, 0.40546E+00_JPRB, 0.39837E+00_JPRB, 0.38977E+00_JPRB, 0.38157E+00_JPRB, 0.37438E+00_JPRB, 0.36887E+00_JPRB ,&
&  0.36342E+00_JPRB, 0.35747E+00_JPRB, 0.35154E+00_JPRB, 0.34758E+00_JPRB, 0.34437E+00_JPRB, 0.34287E+00_JPRB, 0.35428E+00_JPRB ,&
&  0.38996E+00_JPRB, 0.33568E+00_JPRB, 0.35533E+00_JPRB, 0.38194E+00_JPRB, 0.42323E+00_JPRB, 0.48067E+00_JPRB, 0.51573E+00_JPRB ,&
&  0.53414E+00_JPRB, 0.54534E+00_JPRB, 0.54591E+00_JPRB, 0.54410E+00_JPRB, 0.54503E+00_JPRB, 0.53714E+00_JPRB, 0.51810E+00_JPRB ,&
&  0.49780E+00_JPRB, 0.47807E+00_JPRB, 0.46305E+00_JPRB, 0.45199E+00_JPRB, 0.44290E+00_JPRB, 0.43661E+00_JPRB, 0.42782E+00_JPRB ,&
&  0.41407E+00_JPRB, 0.40663E+00_JPRB, 0.40317E+00_JPRB, 0.40018E+00_JPRB, 0.39855E+00_JPRB, 0.39500E+00_JPRB, 0.38486E+00_JPRB ,&
&  0.37525E+00_JPRB, 0.36974E+00_JPRB, 0.36805E+00_JPRB, 0.36858E+00_JPRB, 0.37035E+00_JPRB, 0.37128E+00_JPRB, 0.36932E+00_JPRB ,&
&  0.36661E+00_JPRB, 0.36280E+00_JPRB, 0.36086E+00_JPRB, 0.35955E+00_JPRB, 0.35837E+00_JPRB, 0.35980E+00_JPRB, 0.36159E+00_JPRB ,&
&  0.36202E+00_JPRB, 0.36285E+00_JPRB, 0.36501E+00_JPRB, 0.36663E+00_JPRB, 0.36649E+00_JPRB, 0.36520E+00_JPRB, 0.36354E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.36116E+00_JPRB, 0.36294E+00_JPRB, 0.36855E+00_JPRB, 0.37646E+00_JPRB, 0.38581E+00_JPRB, 0.39621E+00_JPRB ,&
&  0.40787E+00_JPRB, 0.42060E+00_JPRB, 0.43275E+00_JPRB, 0.47302E+00_JPRB, 0.50074E+00_JPRB, 0.52521E+00_JPRB, 0.54703E+00_JPRB ,&
&  0.55582E+00_JPRB, 0.55641E+00_JPRB, 0.46757E+00_JPRB, 0.50828E+00_JPRB, 0.51907E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.57608E+00_JPRB, 0.57436E+00_JPRB, 0.56074E+00_JPRB, 0.54182E+00_JPRB, 0.52503E+00_JPRB, 0.51039E+00_JPRB, 0.49868E+00_JPRB ,&
&  0.48454E+00_JPRB, 0.47280E+00_JPRB, 0.46776E+00_JPRB, 0.46619E+00_JPRB, 0.46404E+00_JPRB, 0.45882E+00_JPRB, 0.45229E+00_JPRB ,&
&  0.44562E+00_JPRB, 0.43815E+00_JPRB, 0.42943E+00_JPRB, 0.42065E+00_JPRB, 0.41339E+00_JPRB, 0.40574E+00_JPRB, 0.40008E+00_JPRB ,&
&  0.39489E+00_JPRB, 0.39040E+00_JPRB, 0.38316E+00_JPRB, 0.37413E+00_JPRB, 0.37031E+00_JPRB, 0.36535E+00_JPRB, 0.37907E+00_JPRB ,&
&  0.42590E+00_JPRB, 0.37018E+00_JPRB, 0.39615E+00_JPRB, 0.44022E+00_JPRB, 0.48518E+00_JPRB, 0.53786E+00_JPRB, 0.56451E+00_JPRB ,&
&  0.58144E+00_JPRB, 0.59150E+00_JPRB, 0.59452E+00_JPRB, 0.60145E+00_JPRB, 0.59853E+00_JPRB, 0.57608E+00_JPRB, 0.54623E+00_JPRB ,&
&  0.52357E+00_JPRB, 0.50117E+00_JPRB, 0.48591E+00_JPRB, 0.47144E+00_JPRB, 0.46458E+00_JPRB, 0.46203E+00_JPRB, 0.45517E+00_JPRB ,&
&  0.44529E+00_JPRB, 0.44155E+00_JPRB, 0.43787E+00_JPRB, 0.43772E+00_JPRB, 0.43920E+00_JPRB, 0.44118E+00_JPRB, 0.43576E+00_JPRB ,&
&  0.43053E+00_JPRB, 0.43460E+00_JPRB, 0.44204E+00_JPRB, 0.44545E+00_JPRB, 0.44704E+00_JPRB, 0.44455E+00_JPRB, 0.43953E+00_JPRB ,&
&  0.43522E+00_JPRB, 0.43211E+00_JPRB, 0.43213E+00_JPRB, 0.43112E+00_JPRB, 0.42939E+00_JPRB, 0.42668E+00_JPRB, 0.42394E+00_JPRB ,&
&  0.42205E+00_JPRB, 0.42055E+00_JPRB, 0.41857E+00_JPRB, 0.41574E+00_JPRB, 0.41200E+00_JPRB, 0.40693E+00_JPRB, 0.40469E+00_JPRB ,&
&  0.40450E+00_JPRB, 0.40683E+00_JPRB, 0.41186E+00_JPRB, 0.41996E+00_JPRB, 0.43101E+00_JPRB, 0.44159E+00_JPRB, 0.45347E+00_JPRB ,&
&  0.46494E+00_JPRB, 0.47830E+00_JPRB, 0.49293E+00_JPRB, 0.56696E+00_JPRB, 0.60365E+00_JPRB, 0.61642E+00_JPRB, 0.62677E+00_JPRB ,&
&  0.63017E+00_JPRB, 0.62792E+00_JPRB, 0.62155E+00_JPRB, 0.54735E+00_JPRB, 0.57159E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.62381E+00_JPRB, 0.62206E+00_JPRB, 0.60149E+00_JPRB, 0.57711E+00_JPRB, 0.55474E+00_JPRB, 0.53663E+00_JPRB, 0.52108E+00_JPRB ,&
&  0.51014E+00_JPRB, 0.50240E+00_JPRB, 0.49711E+00_JPRB, 0.49357E+00_JPRB, 0.48878E+00_JPRB, 0.48160E+00_JPRB, 0.47475E+00_JPRB ,&
&  0.46934E+00_JPRB, 0.46302E+00_JPRB, 0.45345E+00_JPRB, 0.44677E+00_JPRB, 0.44291E+00_JPRB, 0.43846E+00_JPRB, 0.43407E+00_JPRB ,&
&  0.42853E+00_JPRB, 0.41731E+00_JPRB, 0.40602E+00_JPRB, 0.39879E+00_JPRB, 0.39334E+00_JPRB, 0.39106E+00_JPRB, 0.40599E+00_JPRB ,&
&  0.45785E+00_JPRB, 0.43610E+00_JPRB, 0.47425E+00_JPRB, 0.51737E+00_JPRB, 0.56902E+00_JPRB, 0.60139E+00_JPRB, 0.61933E+00_JPRB ,&
&  0.63116E+00_JPRB, 0.64594E+00_JPRB, 0.65712E+00_JPRB, 0.67343E+00_JPRB, 0.65024E+00_JPRB, 0.61369E+00_JPRB, 0.58314E+00_JPRB ,&
&  0.55472E+00_JPRB, 0.52831E+00_JPRB, 0.50626E+00_JPRB, 0.49105E+00_JPRB, 0.48601E+00_JPRB, 0.48406E+00_JPRB, 0.48022E+00_JPRB ,&
&  0.47758E+00_JPRB, 0.47206E+00_JPRB, 0.46556E+00_JPRB, 0.46719E+00_JPRB, 0.46728E+00_JPRB, 0.48123E+00_JPRB, 0.49662E+00_JPRB ,&
&  0.50847E+00_JPRB, 0.52374E+00_JPRB, 0.53188E+00_JPRB, 0.53781E+00_JPRB, 0.53504E+00_JPRB, 0.52883E+00_JPRB, 0.52253E+00_JPRB ,&
&  0.51674E+00_JPRB, 0.51311E+00_JPRB, 0.51173E+00_JPRB, 0.50883E+00_JPRB, 0.50379E+00_JPRB, 0.50000E+00_JPRB, 0.49699E+00_JPRB ,&
&  0.49501E+00_JPRB, 0.49279E+00_JPRB, 0.49065E+00_JPRB, 0.48321E+00_JPRB, 0.47279E+00_JPRB, 0.46465E+00_JPRB, 0.45840E+00_JPRB ,&
&  0.45454E+00_JPRB, 0.45560E+00_JPRB, 0.46029E+00_JPRB, 0.47085E+00_JPRB, 0.48187E+00_JPRB, 0.49391E+00_JPRB, 0.50272E+00_JPRB ,&
&  0.50918E+00_JPRB, 0.52457E+00_JPRB, 0.54783E+00_JPRB, 0.59326E+00_JPRB, 0.62211E+00_JPRB, 0.64245E+00_JPRB, 0.65979E+00_JPRB ,&
&  0.66816E+00_JPRB, 0.66908E+00_JPRB, 0.65076E+00_JPRB, 0.63483E+00_JPRB, 0.61389E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.71416E+00_JPRB, 0.68343E+00_JPRB, 0.64511E+00_JPRB, 0.60902E+00_JPRB, 0.57583E+00_JPRB, 0.54923E+00_JPRB, 0.53236E+00_JPRB ,&
&  0.52129E+00_JPRB, 0.51391E+00_JPRB, 0.50930E+00_JPRB, 0.50578E+00_JPRB, 0.50068E+00_JPRB, 0.49643E+00_JPRB, 0.49053E+00_JPRB ,&
&  0.48349E+00_JPRB, 0.47463E+00_JPRB, 0.46668E+00_JPRB, 0.46142E+00_JPRB, 0.45694E+00_JPRB, 0.45457E+00_JPRB, 0.45149E+00_JPRB ,&
&  0.44479E+00_JPRB, 0.43655E+00_JPRB, 0.42598E+00_JPRB, 0.41520E+00_JPRB, 0.40886E+00_JPRB, 0.40866E+00_JPRB, 0.43303E+00_JPRB ,&
&  0.50886E+00_JPRB, 0.51654E+00_JPRB, 0.56072E+00_JPRB, 0.60539E+00_JPRB, 0.66211E+00_JPRB, 0.70059E+00_JPRB, 0.71668E+00_JPRB ,&
&  0.72833E+00_JPRB, 0.74051E+00_JPRB, 0.76605E+00_JPRB, 0.76014E+00_JPRB, 0.71179E+00_JPRB, 0.66644E+00_JPRB, 0.62271E+00_JPRB ,&
&  0.58153E+00_JPRB, 0.54503E+00_JPRB, 0.51465E+00_JPRB, 0.50204E+00_JPRB, 0.49940E+00_JPRB, 0.50191E+00_JPRB, 0.50299E+00_JPRB ,&
&  0.50424E+00_JPRB, 0.50124E+00_JPRB, 0.50369E+00_JPRB, 0.52048E+00_JPRB, 0.54576E+00_JPRB, 0.58155E+00_JPRB, 0.63936E+00_JPRB ,&
&  0.66946E+00_JPRB, 0.66467E+00_JPRB, 0.67434E+00_JPRB, 0.67777E+00_JPRB, 0.66165E+00_JPRB, 0.64557E+00_JPRB, 0.63149E+00_JPRB ,&
&  0.62000E+00_JPRB, 0.61018E+00_JPRB, 0.60308E+00_JPRB, 0.59197E+00_JPRB, 0.58184E+00_JPRB, 0.57556E+00_JPRB, 0.57272E+00_JPRB ,&
&  0.57044E+00_JPRB, 0.56999E+00_JPRB, 0.56728E+00_JPRB, 0.55938E+00_JPRB, 0.55398E+00_JPRB, 0.54843E+00_JPRB, 0.53811E+00_JPRB ,&
&  0.52542E+00_JPRB, 0.51979E+00_JPRB, 0.52325E+00_JPRB, 0.53121E+00_JPRB, 0.54132E+00_JPRB, 0.55064E+00_JPRB, 0.55683E+00_JPRB ,&
&  0.56291E+00_JPRB, 0.58634E+00_JPRB, 0.59529E+00_JPRB, 0.64925E+00_JPRB, 0.70204E+00_JPRB, 0.71264E+00_JPRB, 0.72506E+00_JPRB ,&
&  0.73053E+00_JPRB, 0.71995E+00_JPRB, 0.69295E+00_JPRB, 0.66875E+00_JPRB, 0.71700E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.68694E+00_JPRB, 0.65584E+00_JPRB, 0.62300E+00_JPRB, 0.59400E+00_JPRB, 0.56728E+00_JPRB, 0.54656E+00_JPRB, 0.53340E+00_JPRB ,&
&  0.52264E+00_JPRB, 0.51557E+00_JPRB, 0.51568E+00_JPRB, 0.51903E+00_JPRB, 0.51742E+00_JPRB, 0.50896E+00_JPRB, 0.49729E+00_JPRB ,&
&  0.48527E+00_JPRB, 0.47641E+00_JPRB, 0.47037E+00_JPRB, 0.46839E+00_JPRB, 0.46433E+00_JPRB, 0.46015E+00_JPRB, 0.45361E+00_JPRB ,&
&  0.44895E+00_JPRB, 0.44461E+00_JPRB, 0.43779E+00_JPRB, 0.43070E+00_JPRB, 0.42590E+00_JPRB, 0.42733E+00_JPRB, 0.46470E+00_JPRB ,&
&  0.56214E+00_JPRB, 0.64362E+00_JPRB, 0.67710E+00_JPRB, 0.72756E+00_JPRB, 0.81434E+00_JPRB, 0.83411E+00_JPRB, 0.87509E+00_JPRB ,&
&  0.90029E+00_JPRB, 0.91301E+00_JPRB, 0.91987E+00_JPRB, 0.87620E+00_JPRB, 0.79167E+00_JPRB, 0.72436E+00_JPRB, 0.65731E+00_JPRB ,&
&  0.59616E+00_JPRB, 0.54869E+00_JPRB, 0.51903E+00_JPRB, 0.50738E+00_JPRB, 0.50972E+00_JPRB, 0.51727E+00_JPRB, 0.52276E+00_JPRB ,&
&  0.52863E+00_JPRB, 0.53491E+00_JPRB, 0.56583E+00_JPRB, 0.62876E+00_JPRB, 0.74060E+00_JPRB, 0.81011E+00_JPRB, 0.92166E+00_JPRB ,&
&  0.97668E+00_JPRB, 0.97933E+00_JPRB, 0.96120E+00_JPRB, 0.91939E+00_JPRB, 0.88437E+00_JPRB, 0.83949E+00_JPRB, 0.80265E+00_JPRB ,&
&  0.75955E+00_JPRB, 0.72063E+00_JPRB, 0.69082E+00_JPRB, 0.66915E+00_JPRB, 0.65680E+00_JPRB, 0.65112E+00_JPRB, 0.65089E+00_JPRB ,&
&  0.64679E+00_JPRB, 0.64632E+00_JPRB, 0.64389E+00_JPRB, 0.64605E+00_JPRB, 0.64972E+00_JPRB, 0.64026E+00_JPRB, 0.62280E+00_JPRB ,&
&  0.60394E+00_JPRB, 0.58813E+00_JPRB, 0.58867E+00_JPRB, 0.59377E+00_JPRB, 0.59707E+00_JPRB, 0.59665E+00_JPRB, 0.59670E+00_JPRB ,&
&  0.60921E+00_JPRB, 0.64888E+00_JPRB, 0.66335E+00_JPRB, 0.67741E+00_JPRB, 0.73908E+00_JPRB, 0.79001E+00_JPRB, 0.79269E+00_JPRB ,&
&  0.78146E+00_JPRB, 0.75548E+00_JPRB, 0.71940E+00_JPRB, 0.69042E+00_JPRB, 0.70175E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.67762E+00_JPRB, 0.65550E+00_JPRB, 0.62597E+00_JPRB, 0.60002E+00_JPRB, 0.57857E+00_JPRB, 0.55696E+00_JPRB, 0.54206E+00_JPRB ,&
&  0.53087E+00_JPRB, 0.53055E+00_JPRB, 0.53357E+00_JPRB, 0.53488E+00_JPRB, 0.52968E+00_JPRB, 0.51680E+00_JPRB, 0.50363E+00_JPRB ,&
&  0.49472E+00_JPRB, 0.49302E+00_JPRB, 0.49303E+00_JPRB, 0.48958E+00_JPRB, 0.48350E+00_JPRB, 0.47597E+00_JPRB, 0.46945E+00_JPRB ,&
&  0.46356E+00_JPRB, 0.45855E+00_JPRB, 0.45341E+00_JPRB, 0.45014E+00_JPRB, 0.44622E+00_JPRB, 0.45106E+00_JPRB, 0.49572E+00_JPRB ,&
&  0.61361E+00_JPRB, 0.72120E+00_JPRB, 0.90866E+00_JPRB, 0.90196E+00_JPRB, 0.92936E+00_JPRB, 0.97253E+00_JPRB, 0.11051E+01_JPRB ,&
&  0.11782E+01_JPRB, 0.11972E+01_JPRB, 0.11392E+01_JPRB, 0.10145E+01_JPRB, 0.86381E+00_JPRB, 0.75242E+00_JPRB, 0.66975E+00_JPRB ,&
&  0.60222E+00_JPRB, 0.55959E+00_JPRB, 0.53577E+00_JPRB, 0.52934E+00_JPRB, 0.53364E+00_JPRB, 0.54436E+00_JPRB, 0.55320E+00_JPRB ,&
&  0.56892E+00_JPRB, 0.59530E+00_JPRB, 0.67128E+00_JPRB, 0.80060E+00_JPRB, 0.92492E+00_JPRB, 0.94173E+00_JPRB, 0.11407E+01_JPRB ,&
&  0.14106E+01_JPRB, 0.13419E+01_JPRB, 0.12720E+01_JPRB, 0.12013E+01_JPRB, 0.11348E+01_JPRB, 0.10704E+01_JPRB, 0.97060E+00_JPRB ,&
&  0.89199E+00_JPRB, 0.82634E+00_JPRB, 0.77931E+00_JPRB, 0.75196E+00_JPRB, 0.73694E+00_JPRB, 0.73251E+00_JPRB, 0.73097E+00_JPRB ,&
&  0.72851E+00_JPRB, 0.72607E+00_JPRB, 0.72890E+00_JPRB, 0.72628E+00_JPRB, 0.71596E+00_JPRB, 0.69780E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.65842E+00_JPRB, 0.64420E+00_JPRB, 0.64964E+00_JPRB, 0.63371E+00_JPRB, 0.63558E+00_JPRB, 0.64531E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.65614E+00_JPRB, 0.68082E+00_JPRB, 0.70360E+00_JPRB, 0.71885E+00_JPRB, 0.81085E+00_JPRB, 0.87963E+00_JPRB, 0.86590E+00_JPRB ,&
&  0.83832E+00_JPRB, 0.78131E+00_JPRB, 0.73142E+00_JPRB, 0.70011E+00_JPRB, 0.69087E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.67331E+00_JPRB, 0.64542E+00_JPRB, 0.62261E+00_JPRB, 0.60119E+00_JPRB, 0.58563E+00_JPRB, 0.57015E+00_JPRB, 0.55991E+00_JPRB ,&
&  0.55628E+00_JPRB, 0.55698E+00_JPRB, 0.55578E+00_JPRB, 0.54836E+00_JPRB, 0.53550E+00_JPRB, 0.52493E+00_JPRB, 0.51731E+00_JPRB ,&
&  0.50999E+00_JPRB, 0.50845E+00_JPRB, 0.50703E+00_JPRB, 0.49880E+00_JPRB, 0.48987E+00_JPRB, 0.48357E+00_JPRB, 0.47736E+00_JPRB ,&
&  0.47230E+00_JPRB, 0.46974E+00_JPRB, 0.46780E+00_JPRB, 0.46619E+00_JPRB, 0.46561E+00_JPRB, 0.47607E+00_JPRB, 0.51854E+00_JPRB ,&
&  0.60427E+00_JPRB, 0.52843E+00_JPRB, 0.96216E+00_JPRB, 0.11057E+01_JPRB, 0.11642E+01_JPRB, 0.12400E+01_JPRB, 0.14137E+01_JPRB ,&
&  0.16033E+01_JPRB, 0.15701E+01_JPRB, 0.13761E+01_JPRB, 0.11300E+01_JPRB, 0.90977E+00_JPRB, 0.76173E+00_JPRB, 0.68444E+00_JPRB ,&
&  0.63095E+00_JPRB, 0.59621E+00_JPRB, 0.58081E+00_JPRB, 0.57269E+00_JPRB, 0.57609E+00_JPRB, 0.58641E+00_JPRB, 0.59938E+00_JPRB ,&
&  0.62592E+00_JPRB, 0.67699E+00_JPRB, 0.77135E+00_JPRB, 0.89178E+00_JPRB, 0.95929E+00_JPRB, 0.10565E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.15451E+01_JPRB, 0.16828E+01_JPRB, 0.15081E+01_JPRB, 0.13794E+01_JPRB, 0.12822E+01_JPRB, 0.11907E+01_JPRB, 0.10857E+01_JPRB ,&
&  0.99539E+00_JPRB, 0.91508E+00_JPRB, 0.86074E+00_JPRB, 0.82059E+00_JPRB, 0.79876E+00_JPRB, 0.79031E+00_JPRB, 0.78348E+00_JPRB ,&
&  0.77489E+00_JPRB, 0.76565E+00_JPRB, 0.75960E+00_JPRB, 0.75166E+00_JPRB, 0.74113E+00_JPRB, 0.72138E+00_JPRB, 0.69609E+00_JPRB ,&
&  0.67953E+00_JPRB, 0.67210E+00_JPRB, 0.66573E+00_JPRB, 0.66285E+00_JPRB, 0.67059E+00_JPRB, 0.67603E+00_JPRB, 0.68887E+00_JPRB ,&
&  0.72504E+00_JPRB, 0.74211E+00_JPRB, 0.75628E+00_JPRB, 0.78294E+00_JPRB, 0.81232E+00_JPRB, 0.89650E+00_JPRB, 0.84318E+00_JPRB ,&
&  0.80429E+00_JPRB, 0.76053E+00_JPRB, 0.72170E+00_JPRB, 0.70182E+00_JPRB, 0.69125E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.66585E+00_JPRB, 0.64014E+00_JPRB, 0.62184E+00_JPRB, 0.60800E+00_JPRB, 0.59682E+00_JPRB, 0.58733E+00_JPRB, 0.58523E+00_JPRB ,&
&  0.58359E+00_JPRB, 0.57863E+00_JPRB, 0.56389E+00_JPRB, 0.54633E+00_JPRB, 0.53908E+00_JPRB, 0.53698E+00_JPRB, 0.52682E+00_JPRB ,&
&  0.52311E+00_JPRB, 0.52291E+00_JPRB, 0.51864E+00_JPRB, 0.50992E+00_JPRB, 0.50292E+00_JPRB, 0.49918E+00_JPRB, 0.49817E+00_JPRB ,&
&  0.49063E+00_JPRB, 0.48801E+00_JPRB, 0.49061E+00_JPRB, 0.49265E+00_JPRB, 0.49645E+00_JPRB, 0.51736E+00_JPRB, 0.54166E+00_JPRB ,&
&  0.59458E+00_JPRB, 0.77566E+00_JPRB, 0.82334E+00_JPRB, 0.12161E+01_JPRB, 0.13772E+01_JPRB, 0.15502E+01_JPRB, 0.16252E+01_JPRB ,&
&  0.17172E+01_JPRB, 0.18317E+01_JPRB, 0.15274E+01_JPRB, 0.12096E+01_JPRB, 0.93708E+00_JPRB, 0.79095E+00_JPRB, 0.72259E+00_JPRB ,&
&  0.68854E+00_JPRB, 0.65673E+00_JPRB, 0.63390E+00_JPRB, 0.62750E+00_JPRB, 0.63200E+00_JPRB, 0.64016E+00_JPRB, 0.65617E+00_JPRB ,&
&  0.69825E+00_JPRB, 0.77224E+00_JPRB, 0.88508E+00_JPRB, 0.98247E+00_JPRB, 0.10638E+01_JPRB, 0.12940E+01_JPRB, 0.15174E+01_JPRB ,&
&  0.17698E+01_JPRB, 0.19630E+01_JPRB, 0.17770E+01_JPRB, 0.15876E+01_JPRB, 0.13935E+01_JPRB, 0.12258E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.93487E+00_JPRB, 0.88747E+00_JPRB, 0.85314E+00_JPRB, 0.82946E+00_JPRB, 0.81243E+00_JPRB, 0.78950E+00_JPRB ,&
&  0.76365E+00_JPRB, 0.75061E+00_JPRB, 0.74706E+00_JPRB, 0.74334E+00_JPRB, 0.73628E+00_JPRB, 0.72451E+00_JPRB, 0.70929E+00_JPRB ,&
&  0.69876E+00_JPRB, 0.69638E+00_JPRB, 0.68983E+00_JPRB, 0.70541E+00_JPRB, 0.71751E+00_JPRB, 0.70931E+00_JPRB, 0.71608E+00_JPRB ,&
&  0.76413E+00_JPRB, 0.79378E+00_JPRB, 0.80273E+00_JPRB, 0.81053E+00_JPRB, 0.86479E+00_JPRB, 0.96782E+00_JPRB, 0.81241E+00_JPRB ,&
&  0.75264E+00_JPRB, 0.72657E+00_JPRB, 0.70501E+00_JPRB, 0.69274E+00_JPRB, 0.68148E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.66201E+00_JPRB, 0.64518E+00_JPRB, 0.63287E+00_JPRB, 0.61982E+00_JPRB, 0.60871E+00_JPRB, 0.60436E+00_JPRB, 0.60050E+00_JPRB ,&
&  0.59223E+00_JPRB, 0.57018E+00_JPRB, 0.54683E+00_JPRB, 0.53727E+00_JPRB, 0.53503E+00_JPRB, 0.52643E+00_JPRB, 0.52863E+00_JPRB ,&
&  0.53070E+00_JPRB, 0.53437E+00_JPRB, 0.52783E+00_JPRB, 0.52321E+00_JPRB, 0.52070E+00_JPRB, 0.51982E+00_JPRB, 0.52000E+00_JPRB ,&
&  0.51889E+00_JPRB, 0.51912E+00_JPRB, 0.52471E+00_JPRB, 0.53138E+00_JPRB, 0.54273E+00_JPRB, 0.56691E+00_JPRB, 0.59237E+00_JPRB ,&
&  0.69471E+00_JPRB, 0.78029E+00_JPRB, 0.85742E+00_JPRB, 0.14628E+01_JPRB, 0.17234E+01_JPRB, 0.18056E+01_JPRB, 0.16799E+01_JPRB ,&
&  0.16793E+01_JPRB, 0.18422E+01_JPRB, 0.16177E+01_JPRB, 0.12773E+01_JPRB, 0.10267E+01_JPRB, 0.91244E+00_JPRB, 0.84152E+00_JPRB ,&
&  0.78204E+00_JPRB, 0.72661E+00_JPRB, 0.69280E+00_JPRB, 0.67343E+00_JPRB, 0.68093E+00_JPRB, 0.70351E+00_JPRB, 0.74916E+00_JPRB ,&
&  0.81249E+00_JPRB, 0.91536E+00_JPRB, 0.10204E+01_JPRB, 0.11078E+01_JPRB, 0.14811E+01_JPRB, 0.18764E+01_JPRB, 0.20023E+01_JPRB ,&
&  0.19711E+01_JPRB, 0.20767E+01_JPRB, 0.17927E+01_JPRB, 0.15844E+01_JPRB, 0.13323E+01_JPRB, 0.11928E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.98543E+00_JPRB, 0.91962E+00_JPRB, 0.87302E+00_JPRB, 0.84766E+00_JPRB, 0.82287E+00_JPRB, 0.79378E+00_JPRB, 0.76238E+00_JPRB ,&
&  0.73614E+00_JPRB, 0.73367E+00_JPRB, 0.73456E+00_JPRB, 0.72783E+00_JPRB, 0.72400E+00_JPRB, 0.72194E+00_JPRB, 0.72153E+00_JPRB ,&
&  0.72003E+00_JPRB, 0.72011E+00_JPRB, 0.71550E+00_JPRB, 0.74239E+00_JPRB, 0.76560E+00_JPRB, 0.76338E+00_JPRB, 0.77438E+00_JPRB ,&
&  0.81160E+00_JPRB, 0.83953E+00_JPRB, 0.83050E+00_JPRB, 0.80461E+00_JPRB, 0.82797E+00_JPRB, 0.84827E+00_JPRB, 0.76340E+00_JPRB ,&
&  0.74435E+00_JPRB, 0.72528E+00_JPRB, 0.69855E+00_JPRB, 0.68158E+00_JPRB, 0.67277E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.66650E+00_JPRB, 0.64909E+00_JPRB, 0.63104E+00_JPRB, 0.61729E+00_JPRB, 0.60852E+00_JPRB, 0.59958E+00_JPRB, 0.58390E+00_JPRB ,&
&  0.56297E+00_JPRB, 0.55052E+00_JPRB, 0.52849E+00_JPRB, 0.52374E+00_JPRB, 0.53013E+00_JPRB, 0.53500E+00_JPRB, 0.54419E+00_JPRB ,&
&  0.54943E+00_JPRB, 0.55185E+00_JPRB, 0.55001E+00_JPRB, 0.54677E+00_JPRB, 0.54672E+00_JPRB, 0.54792E+00_JPRB, 0.55312E+00_JPRB ,&
&  0.55256E+00_JPRB, 0.55583E+00_JPRB, 0.56459E+00_JPRB, 0.57692E+00_JPRB, 0.60733E+00_JPRB, 0.66032E+00_JPRB, 0.73211E+00_JPRB ,&
&  0.89970E+00_JPRB, 0.95527E+00_JPRB, 0.10147E+01_JPRB, 0.14803E+01_JPRB, 0.17595E+01_JPRB, 0.16771E+01_JPRB, 0.16300E+01_JPRB ,&
&  0.16238E+01_JPRB, 0.15770E+01_JPRB, 0.15398E+01_JPRB, 0.12908E+01_JPRB, 0.10978E+01_JPRB, 0.99625E+00_JPRB, 0.91614E+00_JPRB ,&
&  0.83821E+00_JPRB, 0.78189E+00_JPRB, 0.74348E+00_JPRB, 0.73431E+00_JPRB, 0.76683E+00_JPRB, 0.83322E+00_JPRB, 0.89679E+00_JPRB ,&
&  0.98642E+00_JPRB, 0.11133E+01_JPRB, 0.12596E+01_JPRB, 0.15461E+01_JPRB, 0.21420E+01_JPRB, 0.25132E+01_JPRB, 0.23002E+01_JPRB ,&
&  0.20804E+01_JPRB, 0.19397E+01_JPRB, 0.17099E+01_JPRB, 0.15263E+01_JPRB, 0.12087E+01_JPRB, 0.11547E+01_JPRB, 0.10497E+01_JPRB ,&
&  0.95311E+00_JPRB, 0.88794E+00_JPRB, 0.85563E+00_JPRB, 0.83822E+00_JPRB, 0.80927E+00_JPRB, 0.76973E+00_JPRB, 0.73996E+00_JPRB ,&
&  0.72500E+00_JPRB, 0.72529E+00_JPRB, 0.72688E+00_JPRB, 0.72190E+00_JPRB, 0.72136E+00_JPRB, 0.72593E+00_JPRB, 0.73127E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.74693E+00_JPRB, 0.75721E+00_JPRB, 0.79977E+00_JPRB, 0.83186E+00_JPRB, 0.84503E+00_JPRB, 0.83426E+00_JPRB ,&
&  0.83613E+00_JPRB, 0.85101E+00_JPRB, 0.83458E+00_JPRB, 0.83028E+00_JPRB, 0.82084E+00_JPRB, 0.80880E+00_JPRB, 0.78170E+00_JPRB ,&
&  0.77945E+00_JPRB, 0.75158E+00_JPRB, 0.70759E+00_JPRB, 0.69143E+00_JPRB, 0.67919E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.66284E+00_JPRB, 0.64265E+00_JPRB, 0.61677E+00_JPRB, 0.59075E+00_JPRB, 0.57738E+00_JPRB, 0.56421E+00_JPRB, 0.54756E+00_JPRB ,&
&  0.53896E+00_JPRB, 0.53454E+00_JPRB, 0.52763E+00_JPRB, 0.53886E+00_JPRB, 0.56210E+00_JPRB, 0.57389E+00_JPRB, 0.58065E+00_JPRB ,&
&  0.58842E+00_JPRB, 0.59356E+00_JPRB, 0.58302E+00_JPRB, 0.58140E+00_JPRB, 0.59090E+00_JPRB, 0.60043E+00_JPRB, 0.60453E+00_JPRB ,&
&  0.61042E+00_JPRB, 0.62262E+00_JPRB, 0.63959E+00_JPRB, 0.67173E+00_JPRB, 0.74088E+00_JPRB, 0.84582E+00_JPRB, 0.98516E+00_JPRB ,&
&  0.11736E+01_JPRB, 0.11500E+01_JPRB, 0.68455E+00_JPRB, 0.11257E+01_JPRB, 0.15078E+01_JPRB, 0.15083E+01_JPRB, 0.14189E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.13486E+01_JPRB, 0.13371E+01_JPRB, 0.12307E+01_JPRB, 0.11166E+01_JPRB, 0.10202E+01_JPRB, 0.93293E+00_JPRB ,&
&  0.85749E+00_JPRB, 0.81307E+00_JPRB, 0.80980E+00_JPRB, 0.85899E+00_JPRB, 0.93731E+00_JPRB, 0.99346E+00_JPRB, 0.10715E+01_JPRB ,&
&  0.12029E+01_JPRB, 0.13796E+01_JPRB, 0.16986E+01_JPRB, 0.21199E+01_JPRB, 0.27443E+01_JPRB, 0.26925E+01_JPRB, 0.21252E+01_JPRB ,&
&  0.16687E+01_JPRB, 0.16188E+01_JPRB, 0.15569E+01_JPRB, 0.13778E+01_JPRB, 0.11148E+01_JPRB, 0.10090E+01_JPRB, 0.99233E+00_JPRB ,&
&  0.91044E+00_JPRB, 0.85323E+00_JPRB, 0.82975E+00_JPRB, 0.80293E+00_JPRB, 0.76485E+00_JPRB, 0.73212E+00_JPRB, 0.71578E+00_JPRB ,&
&  0.71501E+00_JPRB, 0.71432E+00_JPRB, 0.71304E+00_JPRB, 0.71171E+00_JPRB, 0.71592E+00_JPRB, 0.72262E+00_JPRB, 0.73421E+00_JPRB ,&
&  0.75549E+00_JPRB, 0.78364E+00_JPRB, 0.82953E+00_JPRB, 0.85728E+00_JPRB, 0.87249E+00_JPRB, 0.88273E+00_JPRB, 0.89136E+00_JPRB ,&
&  0.90023E+00_JPRB, 0.89524E+00_JPRB, 0.83119E+00_JPRB, 0.81091E+00_JPRB, 0.84936E+00_JPRB, 0.85122E+00_JPRB, 0.87677E+00_JPRB ,&
&  0.90720E+00_JPRB, 0.90710E+00_JPRB, 0.84266E+00_JPRB, 0.71179E+00_JPRB, 0.68916E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.60627E+00_JPRB, 0.58558E+00_JPRB, 0.55248E+00_JPRB, 0.54556E+00_JPRB, 0.54537E+00_JPRB, 0.54432E+00_JPRB, 0.55885E+00_JPRB ,&
&  0.57174E+00_JPRB, 0.59145E+00_JPRB, 0.60098E+00_JPRB, 0.62236E+00_JPRB, 0.63406E+00_JPRB, 0.63524E+00_JPRB, 0.64395E+00_JPRB ,&
&  0.64846E+00_JPRB, 0.65282E+00_JPRB, 0.65854E+00_JPRB, 0.66952E+00_JPRB, 0.68353E+00_JPRB, 0.69014E+00_JPRB, 0.69920E+00_JPRB ,&
&  0.71869E+00_JPRB, 0.74259E+00_JPRB, 0.77737E+00_JPRB, 0.85515E+00_JPRB, 0.98642E+00_JPRB, 0.11531E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.14587E+01_JPRB, 0.10860E+01_JPRB, 0.80296E+00_JPRB, 0.13320E+01_JPRB, 0.14711E+01_JPRB, 0.14368E+01_JPRB, 0.12718E+01_JPRB ,&
&  0.12790E+01_JPRB, 0.12523E+01_JPRB, 0.12248E+01_JPRB, 0.11832E+01_JPRB, 0.10534E+01_JPRB, 0.98119E+00_JPRB, 0.92789E+00_JPRB ,&
&  0.90876E+00_JPRB, 0.91641E+00_JPRB, 0.96222E+00_JPRB, 0.10329E+01_JPRB, 0.11018E+01_JPRB, 0.11764E+01_JPRB, 0.13215E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.17754E+01_JPRB, 0.22374E+01_JPRB, 0.24778E+01_JPRB, 0.27946E+01_JPRB, 0.23653E+01_JPRB, 0.18922E+01_JPRB ,&
&  0.14581E+01_JPRB, 0.13895E+01_JPRB, 0.13900E+01_JPRB, 0.12446E+01_JPRB, 0.10438E+01_JPRB, 0.93057E+00_JPRB, 0.92077E+00_JPRB ,&
&  0.84055E+00_JPRB, 0.80100E+00_JPRB, 0.76948E+00_JPRB, 0.72876E+00_JPRB, 0.70802E+00_JPRB, 0.70564E+00_JPRB, 0.70865E+00_JPRB ,&
&  0.70995E+00_JPRB, 0.70407E+00_JPRB, 0.70076E+00_JPRB, 0.70725E+00_JPRB, 0.72073E+00_JPRB, 0.73103E+00_JPRB, 0.74906E+00_JPRB ,&
&  0.77365E+00_JPRB, 0.80601E+00_JPRB, 0.84639E+00_JPRB, 0.87645E+00_JPRB, 0.87564E+00_JPRB, 0.85474E+00_JPRB, 0.87262E+00_JPRB ,&
&  0.87400E+00_JPRB, 0.88812E+00_JPRB, 0.87805E+00_JPRB, 0.89085E+00_JPRB, 0.88769E+00_JPRB, 0.89837E+00_JPRB, 0.89846E+00_JPRB ,&
&  0.89297E+00_JPRB, 0.86157E+00_JPRB, 0.77257E+00_JPRB, 0.66277E+00_JPRB, 0.63515E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.54052E+00_JPRB, 0.53669E+00_JPRB, 0.54092E+00_JPRB, 0.54827E+00_JPRB, 0.56071E+00_JPRB, 0.58601E+00_JPRB, 0.61509E+00_JPRB ,&
&  0.63943E+00_JPRB, 0.68088E+00_JPRB, 0.70986E+00_JPRB, 0.71201E+00_JPRB, 0.70687E+00_JPRB, 0.71119E+00_JPRB, 0.72179E+00_JPRB ,&
&  0.73902E+00_JPRB, 0.75291E+00_JPRB, 0.76290E+00_JPRB, 0.77353E+00_JPRB, 0.78875E+00_JPRB, 0.80494E+00_JPRB, 0.82608E+00_JPRB ,&
&  0.85890E+00_JPRB, 0.91377E+00_JPRB, 0.99543E+00_JPRB, 0.11164E+01_JPRB, 0.12779E+01_JPRB, 0.14548E+01_JPRB, 0.15540E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.64908E+00_JPRB, 0.12531E+01_JPRB, 0.14775E+01_JPRB, 0.13517E+01_JPRB, 0.13056E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.11935E+01_JPRB, 0.11875E+01_JPRB, 0.11469E+01_JPRB, 0.11473E+01_JPRB, 0.10420E+01_JPRB, 0.10077E+01_JPRB, 0.10107E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.10740E+01_JPRB, 0.11202E+01_JPRB, 0.11786E+01_JPRB, 0.12631E+01_JPRB, 0.13907E+01_JPRB, 0.15924E+01_JPRB ,&
&  0.18303E+01_JPRB, 0.21227E+01_JPRB, 0.25393E+01_JPRB, 0.24436E+01_JPRB, 0.24911E+01_JPRB, 0.22872E+01_JPRB, 0.19309E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.11960E+01_JPRB, 0.11666E+01_JPRB, 0.11376E+01_JPRB, 0.98618E+00_JPRB, 0.84873E+00_JPRB, 0.84530E+00_JPRB ,&
&  0.77644E+00_JPRB, 0.74851E+00_JPRB, 0.70556E+00_JPRB, 0.67138E+00_JPRB, 0.66162E+00_JPRB, 0.67202E+00_JPRB, 0.68540E+00_JPRB ,&
&  0.68834E+00_JPRB, 0.68811E+00_JPRB, 0.69333E+00_JPRB, 0.70227E+00_JPRB, 0.72167E+00_JPRB, 0.73855E+00_JPRB, 0.75085E+00_JPRB ,&
&  0.78275E+00_JPRB, 0.81662E+00_JPRB, 0.84956E+00_JPRB, 0.87879E+00_JPRB, 0.89880E+00_JPRB, 0.88065E+00_JPRB, 0.87537E+00_JPRB ,&
&  0.88370E+00_JPRB, 0.89019E+00_JPRB, 0.89656E+00_JPRB, 0.91504E+00_JPRB, 0.91005E+00_JPRB, 0.86977E+00_JPRB, 0.85413E+00_JPRB ,&
&  0.79071E+00_JPRB, 0.71709E+00_JPRB, 0.63174E+00_JPRB, 0.58055E+00_JPRB, 0.56175E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.56234E+00_JPRB, 0.57824E+00_JPRB, 0.59552E+00_JPRB, 0.61709E+00_JPRB, 0.64872E+00_JPRB, 0.68564E+00_JPRB, 0.73292E+00_JPRB ,&
&  0.78142E+00_JPRB, 0.79605E+00_JPRB, 0.79977E+00_JPRB, 0.80208E+00_JPRB, 0.80520E+00_JPRB, 0.81539E+00_JPRB, 0.83014E+00_JPRB ,&
&  0.84503E+00_JPRB, 0.86027E+00_JPRB, 0.87576E+00_JPRB, 0.89292E+00_JPRB, 0.91396E+00_JPRB, 0.94493E+00_JPRB, 0.98670E+00_JPRB ,&
&  0.10343E+01_JPRB, 0.11163E+01_JPRB, 0.12252E+01_JPRB, 0.13483E+01_JPRB, 0.14774E+01_JPRB, 0.15799E+01_JPRB, 0.16079E+01_JPRB ,&
&  0.97452E+00_JPRB, 0.12295E+01_JPRB, 0.14539E+01_JPRB, 0.13849E+01_JPRB, 0.12922E+01_JPRB, 0.12420E+01_JPRB, 0.12314E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11129E+01_JPRB, 0.11002E+01_JPRB, 0.11683E+01_JPRB, 0.10922E+01_JPRB, 0.10789E+01_JPRB, 0.11072E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.11960E+01_JPRB, 0.12626E+01_JPRB, 0.13432E+01_JPRB, 0.14537E+01_JPRB, 0.16183E+01_JPRB, 0.18149E+01_JPRB ,&
&  0.20454E+01_JPRB, 0.22931E+01_JPRB, 0.25374E+01_JPRB, 0.22800E+01_JPRB, 0.22404E+01_JPRB, 0.23400E+01_JPRB, 0.21769E+01_JPRB ,&
&  0.14413E+01_JPRB, 0.10648E+01_JPRB, 0.10757E+01_JPRB, 0.10494E+01_JPRB, 0.91629E+00_JPRB, 0.84071E+00_JPRB, 0.77670E+00_JPRB ,&
&  0.73090E+00_JPRB, 0.68680E+00_JPRB, 0.64229E+00_JPRB, 0.62939E+00_JPRB, 0.62897E+00_JPRB, 0.64196E+00_JPRB, 0.65802E+00_JPRB ,&
&  0.66730E+00_JPRB, 0.67035E+00_JPRB, 0.67985E+00_JPRB, 0.69430E+00_JPRB, 0.72691E+00_JPRB, 0.74706E+00_JPRB, 0.75800E+00_JPRB ,&
&  0.78696E+00_JPRB, 0.80552E+00_JPRB, 0.83141E+00_JPRB, 0.85878E+00_JPRB, 0.88300E+00_JPRB, 0.89770E+00_JPRB, 0.89408E+00_JPRB ,&
&  0.89005E+00_JPRB, 0.91315E+00_JPRB, 0.94142E+00_JPRB, 0.94487E+00_JPRB, 0.91560E+00_JPRB, 0.83086E+00_JPRB, 0.75395E+00_JPRB ,&
&  0.66982E+00_JPRB, 0.64707E+00_JPRB, 0.61268E+00_JPRB, 0.58728E+00_JPRB, 0.56909E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.65921E+00_JPRB, 0.67579E+00_JPRB, 0.74673E+00_JPRB, 0.76799E+00_JPRB, 0.83684E+00_JPRB, 0.86451E+00_JPRB, 0.89304E+00_JPRB ,&
&  0.89803E+00_JPRB, 0.90108E+00_JPRB, 0.90292E+00_JPRB, 0.90832E+00_JPRB, 0.91414E+00_JPRB, 0.92579E+00_JPRB, 0.93689E+00_JPRB ,&
&  0.95300E+00_JPRB, 0.97472E+00_JPRB, 0.10006E+01_JPRB, 0.10272E+01_JPRB, 0.10620E+01_JPRB, 0.11031E+01_JPRB, 0.11564E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.12990E+01_JPRB, 0.13820E+01_JPRB, 0.14745E+01_JPRB, 0.15717E+01_JPRB, 0.16440E+01_JPRB, 0.16551E+01_JPRB ,&
&  0.11927E+01_JPRB, 0.14448E+01_JPRB, 0.14306E+01_JPRB, 0.13671E+01_JPRB, 0.12787E+01_JPRB, 0.12179E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11155E+01_JPRB, 0.11118E+01_JPRB, 0.11326E+01_JPRB, 0.11443E+01_JPRB, 0.11317E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.13126E+01_JPRB, 0.13926E+01_JPRB, 0.14631E+01_JPRB, 0.15783E+01_JPRB, 0.17315E+01_JPRB, 0.19004E+01_JPRB ,&
&  0.20713E+01_JPRB, 0.22322E+01_JPRB, 0.23378E+01_JPRB, 0.20651E+01_JPRB, 0.17949E+01_JPRB, 0.17199E+01_JPRB, 0.13420E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.93210E+00_JPRB, 0.98587E+00_JPRB, 0.99149E+00_JPRB, 0.87420E+00_JPRB, 0.78406E+00_JPRB, 0.72008E+00_JPRB ,&
&  0.66323E+00_JPRB, 0.62002E+00_JPRB, 0.60595E+00_JPRB, 0.59634E+00_JPRB, 0.59462E+00_JPRB, 0.61202E+00_JPRB, 0.62640E+00_JPRB ,&
&  0.65012E+00_JPRB, 0.67986E+00_JPRB, 0.70652E+00_JPRB, 0.73906E+00_JPRB, 0.75998E+00_JPRB, 0.78351E+00_JPRB, 0.79527E+00_JPRB ,&
&  0.81287E+00_JPRB, 0.84742E+00_JPRB, 0.85204E+00_JPRB, 0.87741E+00_JPRB, 0.88158E+00_JPRB, 0.90561E+00_JPRB, 0.93067E+00_JPRB ,&
&  0.95636E+00_JPRB, 0.97971E+00_JPRB, 0.98052E+00_JPRB, 0.95733E+00_JPRB, 0.90242E+00_JPRB, 0.82477E+00_JPRB, 0.66583E+00_JPRB ,&
&  0.63973E+00_JPRB, 0.62114E+00_JPRB, 0.62732E+00_JPRB, 0.63269E+00_JPRB, 0.65092E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.82717E+00_JPRB, 0.87858E+00_JPRB, 0.93023E+00_JPRB, 0.99468E+00_JPRB, 0.10469E+01_JPRB, 0.10377E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.99500E+00_JPRB, 0.98908E+00_JPRB, 0.99834E+00_JPRB, 0.10157E+01_JPRB, 0.10300E+01_JPRB, 0.10396E+01_JPRB, 0.10547E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.11064E+01_JPRB, 0.11380E+01_JPRB, 0.11731E+01_JPRB, 0.12029E+01_JPRB, 0.12457E+01_JPRB, 0.12957E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.14097E+01_JPRB, 0.14612E+01_JPRB, 0.15126E+01_JPRB, 0.15601E+01_JPRB, 0.15557E+01_JPRB, 0.11540E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.14615E+01_JPRB, 0.14889E+01_JPRB, 0.14221E+01_JPRB, 0.13025E+01_JPRB, 0.12281E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.11509E+01_JPRB, 0.11192E+01_JPRB, 0.11057E+01_JPRB, 0.12110E+01_JPRB, 0.11859E+01_JPRB, 0.12394E+01_JPRB ,&
&  0.13041E+01_JPRB, 0.13646E+01_JPRB, 0.14190E+01_JPRB, 0.14971E+01_JPRB, 0.16180E+01_JPRB, 0.17393E+01_JPRB, 0.18761E+01_JPRB ,&
&  0.20024E+01_JPRB, 0.21015E+01_JPRB, 0.21424E+01_JPRB, 0.18547E+01_JPRB, 0.17173E+01_JPRB, 0.15610E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.95333E+00_JPRB, 0.90336E+00_JPRB, 0.96069E+00_JPRB, 0.95822E+00_JPRB, 0.84829E+00_JPRB, 0.74442E+00_JPRB, 0.68862E+00_JPRB ,&
&  0.62871E+00_JPRB, 0.59966E+00_JPRB, 0.59296E+00_JPRB, 0.57510E+00_JPRB, 0.56869E+00_JPRB, 0.59358E+00_JPRB, 0.62623E+00_JPRB ,&
&  0.66679E+00_JPRB, 0.71756E+00_JPRB, 0.76006E+00_JPRB, 0.79715E+00_JPRB, 0.84321E+00_JPRB, 0.91940E+00_JPRB, 0.96108E+00_JPRB ,&
&  0.88883E+00_JPRB, 0.85918E+00_JPRB, 0.91560E+00_JPRB, 0.96186E+00_JPRB, 0.10114E+01_JPRB, 0.10880E+01_JPRB, 0.11662E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.13601E+01_JPRB, 0.11564E+01_JPRB, 0.97230E+00_JPRB, 0.12982E+01_JPRB, 0.13827E+01_JPRB, 0.71654E+00_JPRB ,&
&  0.68545E+00_JPRB, 0.70395E+00_JPRB, 0.71643E+00_JPRB, 0.75133E+00_JPRB, 0.78782E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.98760E+00_JPRB, 0.10185E+01_JPRB, 0.10519E+01_JPRB, 0.10731E+01_JPRB, 0.10846E+01_JPRB, 0.10713E+01_JPRB, 0.10593E+01_JPRB ,&
&  0.10605E+01_JPRB, 0.10703E+01_JPRB, 0.10873E+01_JPRB, 0.11062E+01_JPRB, 0.11190E+01_JPRB, 0.11316E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.12191E+01_JPRB, 0.12626E+01_JPRB, 0.12949E+01_JPRB, 0.13283E+01_JPRB, 0.13545E+01_JPRB, 0.13916E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.14720E+01_JPRB, 0.14814E+01_JPRB, 0.15095E+01_JPRB, 0.15241E+01_JPRB, 0.15703E+01_JPRB, 0.12695E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.15148E+01_JPRB, 0.15038E+01_JPRB, 0.14287E+01_JPRB, 0.13151E+01_JPRB, 0.12629E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.12010E+01_JPRB, 0.11540E+01_JPRB, 0.12128E+01_JPRB, 0.12579E+01_JPRB, 0.12268E+01_JPRB, 0.12674E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.13630E+01_JPRB, 0.14160E+01_JPRB, 0.15008E+01_JPRB, 0.16154E+01_JPRB, 0.17178E+01_JPRB, 0.18175E+01_JPRB ,&
&  0.19208E+01_JPRB, 0.20226E+01_JPRB, 0.20897E+01_JPRB, 0.18679E+01_JPRB, 0.18120E+01_JPRB, 0.16124E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.93384E+00_JPRB, 0.84724E+00_JPRB, 0.88526E+00_JPRB, 0.95686E+00_JPRB, 0.84583E+00_JPRB, 0.72936E+00_JPRB, 0.69081E+00_JPRB ,&
&  0.64053E+00_JPRB, 0.60723E+00_JPRB, 0.59829E+00_JPRB, 0.57437E+00_JPRB, 0.56212E+00_JPRB, 0.59049E+00_JPRB, 0.61752E+00_JPRB ,&
&  0.65501E+00_JPRB, 0.70521E+00_JPRB, 0.76163E+00_JPRB, 0.81700E+00_JPRB, 0.87608E+00_JPRB, 0.89397E+00_JPRB, 0.84248E+00_JPRB ,&
&  0.88192E+00_JPRB, 0.93129E+00_JPRB, 0.99431E+00_JPRB, 0.11046E+01_JPRB, 0.11865E+01_JPRB, 0.14041E+01_JPRB, 0.16401E+01_JPRB ,&
&  0.18612E+01_JPRB, 0.17483E+01_JPRB, 0.20774E+01_JPRB, 0.28841E+01_JPRB, 0.26660E+01_JPRB, 0.21566E+01_JPRB, 0.82901E+00_JPRB ,&
&  0.82295E+00_JPRB, 0.82386E+00_JPRB, 0.85957E+00_JPRB, 0.90486E+00_JPRB, 0.95722E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.99805E+00_JPRB, 0.10244E+01_JPRB, 0.10505E+01_JPRB, 0.10755E+01_JPRB, 0.10947E+01_JPRB, 0.11093E+01_JPRB, 0.11191E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11581E+01_JPRB, 0.11730E+01_JPRB, 0.11819E+01_JPRB, 0.11908E+01_JPRB, 0.12138E+01_JPRB, 0.12453E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.13047E+01_JPRB, 0.13432E+01_JPRB, 0.13274E+01_JPRB, 0.13101E+01_JPRB, 0.13153E+01_JPRB, 0.13433E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.13437E+01_JPRB, 0.13374E+01_JPRB, 0.13664E+01_JPRB, 0.13990E+01_JPRB, 0.14476E+01_JPRB, 0.12304E+01_JPRB ,&
&  0.13930E+01_JPRB, 0.15141E+01_JPRB, 0.14723E+01_JPRB, 0.13992E+01_JPRB, 0.12770E+01_JPRB, 0.12430E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.12309E+01_JPRB, 0.12276E+01_JPRB, 0.12158E+01_JPRB, 0.12140E+01_JPRB, 0.12297E+01_JPRB, 0.12426E+01_JPRB, 0.12754E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.13397E+01_JPRB, 0.13963E+01_JPRB, 0.14626E+01_JPRB, 0.15606E+01_JPRB, 0.16590E+01_JPRB, 0.17167E+01_JPRB ,&
&  0.17822E+01_JPRB, 0.18277E+01_JPRB, 0.16940E+01_JPRB, 0.16890E+01_JPRB, 0.17427E+01_JPRB, 0.15381E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.91613E+00_JPRB, 0.85880E+00_JPRB, 0.88144E+00_JPRB, 0.95469E+00_JPRB, 0.85053E+00_JPRB, 0.76475E+00_JPRB, 0.71681E+00_JPRB ,&
&  0.63914E+00_JPRB, 0.57088E+00_JPRB, 0.58120E+00_JPRB, 0.57516E+00_JPRB, 0.56373E+00_JPRB, 0.58305E+00_JPRB, 0.60328E+00_JPRB ,&
&  0.62537E+00_JPRB, 0.66924E+00_JPRB, 0.71231E+00_JPRB, 0.75526E+00_JPRB, 0.77493E+00_JPRB, 0.73564E+00_JPRB, 0.82217E+00_JPRB ,&
&  0.82483E+00_JPRB, 0.81943E+00_JPRB, 0.86027E+00_JPRB, 0.74918E+00_JPRB, 0.10359E+01_JPRB, 0.11005E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.16005E+01_JPRB, 0.19829E+01_JPRB, 0.15500E+01_JPRB, 0.12241E+01_JPRB, 0.89873E+00_JPRB, 0.84543E+00_JPRB ,&
&  0.85272E+00_JPRB, 0.86913E+00_JPRB, 0.89890E+00_JPRB, 0.93516E+00_JPRB, 0.96797E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.10021E+01_JPRB, 0.10275E+01_JPRB, 0.10528E+01_JPRB, 0.10806E+01_JPRB, 0.11086E+01_JPRB, 0.11287E+01_JPRB, 0.11507E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11932E+01_JPRB, 0.12020E+01_JPRB, 0.12117E+01_JPRB, 0.12286E+01_JPRB, 0.12684E+01_JPRB, 0.13040E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.13129E+01_JPRB, 0.13222E+01_JPRB, 0.13278E+01_JPRB, 0.13154E+01_JPRB, 0.12831E+01_JPRB, 0.12917E+01_JPRB ,&
&  0.12856E+01_JPRB, 0.12864E+01_JPRB, 0.12862E+01_JPRB, 0.13085E+01_JPRB, 0.13449E+01_JPRB, 0.14288E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.13014E+01_JPRB, 0.14400E+01_JPRB, 0.14283E+01_JPRB, 0.13103E+01_JPRB, 0.11812E+01_JPRB, 0.11480E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.11461E+01_JPRB, 0.11541E+01_JPRB, 0.11698E+01_JPRB, 0.11999E+01_JPRB, 0.12480E+01_JPRB, 0.12683E+01_JPRB ,&
&  0.12934E+01_JPRB, 0.13298E+01_JPRB, 0.13717E+01_JPRB, 0.14187E+01_JPRB, 0.14756E+01_JPRB, 0.15242E+01_JPRB, 0.15526E+01_JPRB ,&
&  0.15660E+01_JPRB, 0.15474E+01_JPRB, 0.13861E+01_JPRB, 0.13692E+01_JPRB, 0.14029E+01_JPRB, 0.12577E+01_JPRB, 0.10530E+01_JPRB ,&
&  0.96615E+00_JPRB, 0.86009E+00_JPRB, 0.79822E+00_JPRB, 0.91821E+00_JPRB, 0.86262E+00_JPRB, 0.79689E+00_JPRB, 0.71227E+00_JPRB ,&
&  0.60602E+00_JPRB, 0.54492E+00_JPRB, 0.54474E+00_JPRB, 0.58050E+00_JPRB, 0.57096E+00_JPRB, 0.56689E+00_JPRB, 0.56939E+00_JPRB ,&
&  0.57720E+00_JPRB, 0.58318E+00_JPRB, 0.60189E+00_JPRB, 0.64747E+00_JPRB, 0.69394E+00_JPRB, 0.65968E+00_JPRB, 0.79914E+00_JPRB ,&
&  0.69822E+00_JPRB, 0.58664E+00_JPRB, 0.65302E+00_JPRB, 0.79426E+00_JPRB, 0.82340E+00_JPRB, 0.83570E+00_JPRB, 0.93833E+00_JPRB ,&
&  0.10849E+01_JPRB, 0.12179E+01_JPRB, 0.12274E+01_JPRB, 0.10115E+01_JPRB, 0.89627E+00_JPRB, 0.84670E+00_JPRB, 0.84040E+00_JPRB ,&
&  0.85083E+00_JPRB, 0.87138E+00_JPRB, 0.90434E+00_JPRB, 0.94212E+00_JPRB, 0.97509E+00_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.98659E+00_JPRB, 0.10135E+01_JPRB, 0.10361E+01_JPRB, 0.10555E+01_JPRB, 0.10742E+01_JPRB, 0.10955E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11464E+01_JPRB, 0.11562E+01_JPRB, 0.11730E+01_JPRB, 0.11907E+01_JPRB, 0.12322E+01_JPRB, 0.12527E+01_JPRB ,&
&  0.12911E+01_JPRB, 0.13058E+01_JPRB, 0.13038E+01_JPRB, 0.13222E+01_JPRB, 0.13116E+01_JPRB, 0.12887E+01_JPRB, 0.12843E+01_JPRB ,&
&  0.12901E+01_JPRB, 0.12867E+01_JPRB, 0.12848E+01_JPRB, 0.12868E+01_JPRB, 0.12895E+01_JPRB, 0.12585E+01_JPRB, 0.13276E+01_JPRB ,&
&  0.98738E+00_JPRB, 0.13640E+01_JPRB, 0.13975E+01_JPRB, 0.12367E+01_JPRB, 0.11716E+01_JPRB, 0.10754E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.10871E+01_JPRB, 0.10926E+01_JPRB, 0.11098E+01_JPRB, 0.11430E+01_JPRB, 0.11829E+01_JPRB, 0.12254E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.12796E+01_JPRB, 0.12929E+01_JPRB, 0.13102E+01_JPRB, 0.13229E+01_JPRB, 0.13654E+01_JPRB, 0.13672E+01_JPRB, 0.13478E+01_JPRB ,&
&  0.13348E+01_JPRB, 0.13038E+01_JPRB, 0.10049E+01_JPRB, 0.95838E+00_JPRB, 0.10077E+01_JPRB, 0.95752E+00_JPRB, 0.85040E+00_JPRB ,&
&  0.84349E+00_JPRB, 0.84851E+00_JPRB, 0.86956E+00_JPRB, 0.84935E+00_JPRB, 0.83804E+00_JPRB, 0.81557E+00_JPRB, 0.77338E+00_JPRB ,&
&  0.67424E+00_JPRB, 0.58177E+00_JPRB, 0.55657E+00_JPRB, 0.58211E+00_JPRB, 0.59517E+00_JPRB, 0.59412E+00_JPRB, 0.55485E+00_JPRB ,&
&  0.52495E+00_JPRB, 0.50347E+00_JPRB, 0.50586E+00_JPRB, 0.54020E+00_JPRB, 0.53693E+00_JPRB, 0.87759E+00_JPRB, 0.10368E+01_JPRB ,&
&  0.67832E+00_JPRB, 0.62252E+00_JPRB, 0.52155E+00_JPRB, 0.68685E+00_JPRB, 0.68282E+00_JPRB, 0.64192E+00_JPRB, 0.67476E+00_JPRB ,&
&  0.77904E+00_JPRB, 0.83338E+00_JPRB, 0.85315E+00_JPRB, 0.78777E+00_JPRB, 0.80205E+00_JPRB, 0.78559E+00_JPRB, 0.78974E+00_JPRB ,&
&  0.81008E+00_JPRB, 0.84266E+00_JPRB, 0.88273E+00_JPRB, 0.92157E+00_JPRB, 0.95549E+00_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.93065E+00_JPRB, 0.95238E+00_JPRB, 0.96766E+00_JPRB, 0.98322E+00_JPRB, 0.10046E+01_JPRB, 0.10269E+01_JPRB, 0.10400E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10816E+01_JPRB, 0.10914E+01_JPRB, 0.11130E+01_JPRB, 0.11229E+01_JPRB, 0.11788E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.12446E+01_JPRB, 0.12698E+01_JPRB, 0.12784E+01_JPRB, 0.12816E+01_JPRB, 0.12952E+01_JPRB, 0.12919E+01_JPRB, 0.12875E+01_JPRB ,&
&  0.13104E+01_JPRB, 0.13551E+01_JPRB, 0.14082E+01_JPRB, 0.14580E+01_JPRB, 0.14869E+01_JPRB, 0.15819E+01_JPRB, 0.17074E+01_JPRB ,&
&  0.12376E+01_JPRB, 0.12540E+01_JPRB, 0.12472E+01_JPRB, 0.10520E+01_JPRB, 0.10596E+01_JPRB, 0.10736E+01_JPRB, 0.10298E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.10561E+01_JPRB, 0.10892E+01_JPRB, 0.11169E+01_JPRB, 0.11414E+01_JPRB, 0.11694E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.11857E+01_JPRB, 0.12040E+01_JPRB, 0.12372E+01_JPRB, 0.12788E+01_JPRB, 0.12082E+01_JPRB, 0.13191E+01_JPRB, 0.13408E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.11637E+01_JPRB, 0.83640E+00_JPRB, 0.78490E+00_JPRB, 0.81224E+00_JPRB, 0.77591E+00_JPRB, 0.72529E+00_JPRB ,&
&  0.78174E+00_JPRB, 0.79642E+00_JPRB, 0.65586E+00_JPRB, 0.76326E+00_JPRB, 0.82501E+00_JPRB, 0.85285E+00_JPRB, 0.82055E+00_JPRB ,&
&  0.76938E+00_JPRB, 0.68281E+00_JPRB, 0.65387E+00_JPRB, 0.65403E+00_JPRB, 0.65541E+00_JPRB, 0.63629E+00_JPRB, 0.63517E+00_JPRB ,&
&  0.60269E+00_JPRB, 0.55037E+00_JPRB, 0.53360E+00_JPRB, 0.51177E+00_JPRB, 0.62184E+00_JPRB, 0.73088E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.64249E+00_JPRB, 0.67304E+00_JPRB, 0.65445E+00_JPRB, 0.95782E+00_JPRB, 0.68385E+00_JPRB, 0.52126E+00_JPRB, 0.55030E+00_JPRB ,&
&  0.60993E+00_JPRB, 0.64045E+00_JPRB, 0.65095E+00_JPRB, 0.66821E+00_JPRB, 0.71327E+00_JPRB, 0.73782E+00_JPRB, 0.75959E+00_JPRB ,&
&  0.76454E+00_JPRB, 0.79965E+00_JPRB, 0.84740E+00_JPRB, 0.88384E+00_JPRB, 0.90003E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.79021E+00_JPRB, 0.80509E+00_JPRB, 0.82582E+00_JPRB, 0.84725E+00_JPRB, 0.86067E+00_JPRB, 0.88288E+00_JPRB, 0.90660E+00_JPRB ,&
&  0.93797E+00_JPRB, 0.97830E+00_JPRB, 0.10041E+01_JPRB, 0.10152E+01_JPRB, 0.10316E+01_JPRB, 0.10506E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.10825E+01_JPRB, 0.10938E+01_JPRB, 0.11518E+01_JPRB, 0.11804E+01_JPRB, 0.12194E+01_JPRB, 0.12719E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.12740E+01_JPRB, 0.12639E+01_JPRB, 0.12796E+01_JPRB, 0.12810E+01_JPRB, 0.13781E+01_JPRB, 0.13889E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.11966E+01_JPRB, 0.13196E+01_JPRB, 0.10393E+01_JPRB, 0.10210E+01_JPRB, 0.10109E+01_JPRB, 0.92147E+00_JPRB ,&
&  0.91534E+00_JPRB, 0.96285E+00_JPRB, 0.10195E+01_JPRB, 0.10756E+01_JPRB, 0.11448E+01_JPRB, 0.11647E+01_JPRB, 0.11590E+01_JPRB ,&
&  0.11645E+01_JPRB, 0.11867E+01_JPRB, 0.12052E+01_JPRB, 0.11335E+01_JPRB, 0.10564E+01_JPRB, 0.11452E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.11561E+01_JPRB, 0.10983E+01_JPRB, 0.87027E+00_JPRB, 0.79499E+00_JPRB, 0.81087E+00_JPRB, 0.72040E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.74105E+00_JPRB, 0.67026E+00_JPRB, 0.44067E+00_JPRB, 0.60520E+00_JPRB, 0.76175E+00_JPRB, 0.84771E+00_JPRB, 0.88607E+00_JPRB ,&
&  0.85924E+00_JPRB, 0.77946E+00_JPRB, 0.75070E+00_JPRB, 0.77717E+00_JPRB, 0.80300E+00_JPRB, 0.75296E+00_JPRB, 0.74258E+00_JPRB ,&
&  0.67727E+00_JPRB, 0.58270E+00_JPRB, 0.52072E+00_JPRB, 0.54243E+00_JPRB, 0.57442E+00_JPRB, 0.72975E+00_JPRB, 0.81073E+00_JPRB ,&
&  0.73925E+00_JPRB, 0.73942E+00_JPRB, 0.11376E+01_JPRB, 0.16128E+01_JPRB, 0.70474E+00_JPRB, 0.55411E+00_JPRB, 0.50295E+00_JPRB ,&
&  0.52693E+00_JPRB, 0.54424E+00_JPRB, 0.55757E+00_JPRB, 0.58242E+00_JPRB, 0.61819E+00_JPRB, 0.63283E+00_JPRB, 0.63828E+00_JPRB ,&
&  0.65760E+00_JPRB, 0.68845E+00_JPRB, 0.71149E+00_JPRB, 0.74030E+00_JPRB, 0.76613E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.61389E+00_JPRB, 0.61304E+00_JPRB, 0.61605E+00_JPRB, 0.62302E+00_JPRB, 0.62583E+00_JPRB, 0.62757E+00_JPRB, 0.64539E+00_JPRB ,&
&  0.67624E+00_JPRB, 0.72891E+00_JPRB, 0.80223E+00_JPRB, 0.84758E+00_JPRB, 0.82964E+00_JPRB, 0.77138E+00_JPRB, 0.74588E+00_JPRB ,&
&  0.74065E+00_JPRB, 0.73814E+00_JPRB, 0.77126E+00_JPRB, 0.82479E+00_JPRB, 0.89287E+00_JPRB, 0.91794E+00_JPRB, 0.98342E+00_JPRB ,&
&  0.98684E+00_JPRB, 0.10174E+01_JPRB, 0.11227E+01_JPRB, 0.12256E+01_JPRB, 0.18764E+01_JPRB, 0.17941E+01_JPRB, 0.14750E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.16072E+01_JPRB, 0.14587E+01_JPRB, 0.12527E+01_JPRB, 0.12285E+01_JPRB, 0.11943E+01_JPRB, 0.12289E+01_JPRB ,&
&  0.12132E+01_JPRB, 0.12883E+01_JPRB, 0.13431E+01_JPRB, 0.14090E+01_JPRB, 0.14533E+01_JPRB, 0.14552E+01_JPRB, 0.14785E+01_JPRB ,&
&  0.14878E+01_JPRB, 0.14898E+01_JPRB, 0.14860E+01_JPRB, 0.13078E+01_JPRB, 0.13582E+01_JPRB, 0.13885E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.12753E+01_JPRB, 0.13203E+01_JPRB, 0.12241E+01_JPRB, 0.11307E+01_JPRB, 0.10006E+01_JPRB, 0.87797E+00_JPRB ,&
&  0.80105E+00_JPRB, 0.61900E+00_JPRB, 0.44686E+00_JPRB, 0.67598E+00_JPRB, 0.89765E+00_JPRB, 0.85229E+00_JPRB, 0.96657E+00_JPRB ,&
&  0.97012E+00_JPRB, 0.93468E+00_JPRB, 0.90000E+00_JPRB, 0.89641E+00_JPRB, 0.86260E+00_JPRB, 0.90577E+00_JPRB, 0.86908E+00_JPRB ,&
&  0.78632E+00_JPRB, 0.63166E+00_JPRB, 0.55173E+00_JPRB, 0.53970E+00_JPRB, 0.53970E+00_JPRB, 0.55410E+00_JPRB, 0.64584E+00_JPRB ,&
&  0.74087E+00_JPRB, 0.75360E+00_JPRB, 0.90942E+00_JPRB, 0.11364E+01_JPRB, 0.73395E+00_JPRB, 0.51181E+00_JPRB, 0.48346E+00_JPRB ,&
&  0.48208E+00_JPRB, 0.49525E+00_JPRB, 0.52004E+00_JPRB, 0.53663E+00_JPRB, 0.54853E+00_JPRB, 0.55782E+00_JPRB, 0.57165E+00_JPRB ,&
&  0.58101E+00_JPRB, 0.59275E+00_JPRB, 0.59782E+00_JPRB, 0.60416E+00_JPRB, 0.61080E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.65329E+00_JPRB, 0.65486E+00_JPRB, 0.64147E+00_JPRB, 0.66108E+00_JPRB, 0.69929E+00_JPRB, 0.78143E+00_JPRB, 0.86167E+00_JPRB ,&
&  0.95771E+00_JPRB, 0.10752E+01_JPRB, 0.12270E+01_JPRB, 0.13526E+01_JPRB, 0.13858E+01_JPRB, 0.13561E+01_JPRB, 0.12320E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.90964E+00_JPRB, 0.82349E+00_JPRB, 0.80530E+00_JPRB, 0.86712E+00_JPRB, 0.90681E+00_JPRB, 0.99135E+00_JPRB ,&
&  0.11228E+01_JPRB, 0.13296E+01_JPRB, 0.15751E+01_JPRB, 0.20899E+01_JPRB, 0.24350E+01_JPRB, 0.18124E+01_JPRB, 0.17096E+01_JPRB ,&
&  0.20573E+01_JPRB, 0.21071E+01_JPRB, 0.16011E+01_JPRB, 0.15882E+01_JPRB, 0.17560E+01_JPRB, 0.19060E+01_JPRB, 0.20893E+01_JPRB ,&
&  0.21870E+01_JPRB, 0.22797E+01_JPRB, 0.23457E+01_JPRB, 0.23790E+01_JPRB, 0.24051E+01_JPRB, 0.23927E+01_JPRB, 0.23425E+01_JPRB ,&
&  0.22612E+01_JPRB, 0.21443E+01_JPRB, 0.21200E+01_JPRB, 0.21594E+01_JPRB, 0.22100E+01_JPRB, 0.21670E+01_JPRB, 0.20360E+01_JPRB ,&
&  0.19236E+01_JPRB, 0.18627E+01_JPRB, 0.19686E+01_JPRB, 0.19245E+01_JPRB, 0.17240E+01_JPRB, 0.15961E+01_JPRB, 0.13727E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.98441E+00_JPRB, 0.80547E+00_JPRB, 0.12580E+01_JPRB, 0.14933E+01_JPRB, 0.13313E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.10903E+01_JPRB, 0.10341E+01_JPRB, 0.99580E+00_JPRB, 0.97039E+00_JPRB, 0.11168E+01_JPRB, 0.97406E+00_JPRB ,&
&  0.89127E+00_JPRB, 0.69283E+00_JPRB, 0.63047E+00_JPRB, 0.65893E+00_JPRB, 0.67667E+00_JPRB, 0.65288E+00_JPRB, 0.65369E+00_JPRB ,&
&  0.70401E+00_JPRB, 0.10458E+01_JPRB, 0.12886E+01_JPRB, 0.17325E+01_JPRB, 0.20221E+01_JPRB, 0.59678E+00_JPRB, 0.54512E+00_JPRB ,&
&  0.53089E+00_JPRB, 0.53260E+00_JPRB, 0.55182E+00_JPRB, 0.55603E+00_JPRB, 0.56848E+00_JPRB, 0.57051E+00_JPRB, 0.56527E+00_JPRB ,&
&  0.56866E+00_JPRB, 0.58589E+00_JPRB, 0.60514E+00_JPRB, 0.61563E+00_JPRB, 0.63871E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.96171E+00_JPRB, 0.10067E+01_JPRB, 0.10118E+01_JPRB, 0.10989E+01_JPRB, 0.11908E+01_JPRB, 0.12443E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.13691E+01_JPRB, 0.14816E+01_JPRB, 0.15710E+01_JPRB, 0.16231E+01_JPRB, 0.16752E+01_JPRB, 0.17526E+01_JPRB, 0.17665E+01_JPRB ,&
&  0.17305E+01_JPRB, 0.17008E+01_JPRB, 0.16884E+01_JPRB, 0.16831E+01_JPRB, 0.17118E+01_JPRB, 0.17650E+01_JPRB, 0.19503E+01_JPRB ,&
&  0.20594E+01_JPRB, 0.20437E+01_JPRB, 0.20294E+01_JPRB, 0.17908E+01_JPRB, 0.14179E+01_JPRB, 0.18884E+01_JPRB, 0.18635E+01_JPRB ,&
&  0.19812E+01_JPRB, 0.20475E+01_JPRB, 0.19807E+01_JPRB, 0.21006E+01_JPRB, 0.22349E+01_JPRB, 0.23937E+01_JPRB, 0.25616E+01_JPRB ,&
&  0.26854E+01_JPRB, 0.28175E+01_JPRB, 0.29527E+01_JPRB, 0.30565E+01_JPRB, 0.31609E+01_JPRB, 0.32920E+01_JPRB, 0.33135E+01_JPRB ,&
&  0.32922E+01_JPRB, 0.31792E+01_JPRB, 0.30899E+01_JPRB, 0.30981E+01_JPRB, 0.29693E+01_JPRB, 0.28204E+01_JPRB, 0.25908E+01_JPRB ,&
&  0.24900E+01_JPRB, 0.24845E+01_JPRB, 0.25901E+01_JPRB, 0.26380E+01_JPRB, 0.25408E+01_JPRB, 0.23529E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.19384E+01_JPRB, 0.17633E+01_JPRB, 0.18062E+01_JPRB, 0.19555E+01_JPRB, 0.14834E+01_JPRB, 0.14948E+01_JPRB, 0.15620E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.12313E+01_JPRB, 0.11409E+01_JPRB, 0.10286E+01_JPRB, 0.10585E+01_JPRB, 0.98518E+00_JPRB, 0.10832E+01_JPRB ,&
&  0.10079E+01_JPRB, 0.79211E+00_JPRB, 0.74225E+00_JPRB, 0.82132E+00_JPRB, 0.83164E+00_JPRB, 0.83159E+00_JPRB, 0.76536E+00_JPRB ,&
&  0.80390E+00_JPRB, 0.10348E+01_JPRB, 0.12282E+01_JPRB, 0.14561E+01_JPRB, 0.12162E+01_JPRB, 0.66899E+00_JPRB, 0.55954E+00_JPRB ,&
&  0.53745E+00_JPRB, 0.54643E+00_JPRB, 0.56459E+00_JPRB, 0.58039E+00_JPRB, 0.61297E+00_JPRB, 0.65176E+00_JPRB, 0.68147E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.72484E+00_JPRB, 0.76352E+00_JPRB, 0.82063E+00_JPRB, 0.90171E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.13322E+01_JPRB, 0.14197E+01_JPRB, 0.15021E+01_JPRB, 0.14901E+01_JPRB, 0.14847E+01_JPRB, 0.14618E+01_JPRB, 0.14519E+01_JPRB ,&
&  0.15152E+01_JPRB, 0.15806E+01_JPRB, 0.16707E+01_JPRB, 0.17409E+01_JPRB, 0.17887E+01_JPRB, 0.18772E+01_JPRB, 0.19820E+01_JPRB ,&
&  0.20317E+01_JPRB, 0.20648E+01_JPRB, 0.21002E+01_JPRB, 0.21495E+01_JPRB, 0.21583E+01_JPRB, 0.22156E+01_JPRB, 0.21767E+01_JPRB ,&
&  0.16664E+01_JPRB, 0.13760E+01_JPRB, 0.16641E+01_JPRB, 0.21436E+01_JPRB, 0.23745E+01_JPRB, 0.28330E+01_JPRB, 0.29763E+01_JPRB ,&
&  0.25241E+01_JPRB, 0.23350E+01_JPRB, 0.23597E+01_JPRB, 0.23540E+01_JPRB, 0.23579E+01_JPRB, 0.25405E+01_JPRB, 0.27243E+01_JPRB ,&
&  0.28814E+01_JPRB, 0.30224E+01_JPRB, 0.31632E+01_JPRB, 0.33090E+01_JPRB, 0.34303E+01_JPRB, 0.35504E+01_JPRB, 0.36386E+01_JPRB ,&
&  0.36798E+01_JPRB, 0.36043E+01_JPRB, 0.35347E+01_JPRB, 0.34305E+01_JPRB, 0.33024E+01_JPRB, 0.31645E+01_JPRB, 0.30133E+01_JPRB ,&
&  0.30382E+01_JPRB, 0.31763E+01_JPRB, 0.33973E+01_JPRB, 0.35427E+01_JPRB, 0.35903E+01_JPRB, 0.33957E+01_JPRB, 0.31763E+01_JPRB ,&
&  0.28542E+01_JPRB, 0.25140E+01_JPRB, 0.26103E+01_JPRB, 0.26812E+01_JPRB, 0.20534E+01_JPRB, 0.18699E+01_JPRB, 0.16861E+01_JPRB ,&
&  0.17007E+01_JPRB, 0.14794E+01_JPRB, 0.12958E+01_JPRB, 0.12511E+01_JPRB, 0.13970E+01_JPRB, 0.13168E+01_JPRB, 0.13461E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.93641E+00_JPRB, 0.82146E+00_JPRB, 0.86431E+00_JPRB, 0.76046E+00_JPRB, 0.83469E+00_JPRB, 0.88386E+00_JPRB ,&
&  0.99851E+00_JPRB, 0.10390E+01_JPRB, 0.12118E+01_JPRB, 0.12167E+01_JPRB, 0.86912E+00_JPRB, 0.64689E+00_JPRB, 0.57998E+00_JPRB ,&
&  0.55490E+00_JPRB, 0.56231E+00_JPRB, 0.60202E+00_JPRB, 0.61903E+00_JPRB, 0.67098E+00_JPRB, 0.72412E+00_JPRB, 0.82470E+00_JPRB ,&
&  0.90956E+00_JPRB, 0.98665E+00_JPRB, 0.10491E+01_JPRB, 0.11298E+01_JPRB, 0.12206E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.13369E+01_JPRB, 0.14667E+01_JPRB, 0.15974E+01_JPRB, 0.16857E+01_JPRB, 0.16474E+01_JPRB, 0.16017E+01_JPRB, 0.15861E+01_JPRB ,&
&  0.15918E+01_JPRB, 0.16423E+01_JPRB, 0.17318E+01_JPRB, 0.18133E+01_JPRB, 0.18753E+01_JPRB, 0.19643E+01_JPRB, 0.21003E+01_JPRB ,&
&  0.22231E+01_JPRB, 0.23215E+01_JPRB, 0.24036E+01_JPRB, 0.24910E+01_JPRB, 0.25074E+01_JPRB, 0.25407E+01_JPRB, 0.21984E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.15684E+01_JPRB, 0.22309E+01_JPRB, 0.24656E+01_JPRB, 0.26101E+01_JPRB, 0.29025E+01_JPRB, 0.30027E+01_JPRB ,&
&  0.27264E+01_JPRB, 0.25698E+01_JPRB, 0.25405E+01_JPRB, 0.24768E+01_JPRB, 0.24687E+01_JPRB, 0.26273E+01_JPRB, 0.28107E+01_JPRB ,&
&  0.29680E+01_JPRB, 0.30750E+01_JPRB, 0.32060E+01_JPRB, 0.33822E+01_JPRB, 0.35925E+01_JPRB, 0.37752E+01_JPRB, 0.39351E+01_JPRB ,&
&  0.39017E+01_JPRB, 0.39099E+01_JPRB, 0.37857E+01_JPRB, 0.35554E+01_JPRB, 0.35211E+01_JPRB, 0.35769E+01_JPRB, 0.36488E+01_JPRB ,&
&  0.38313E+01_JPRB, 0.41741E+01_JPRB, 0.44963E+01_JPRB, 0.45863E+01_JPRB, 0.43668E+01_JPRB, 0.43755E+01_JPRB, 0.40434E+01_JPRB ,&
&  0.39924E+01_JPRB, 0.35681E+01_JPRB, 0.35707E+01_JPRB, 0.30734E+01_JPRB, 0.28753E+01_JPRB, 0.26423E+01_JPRB, 0.25074E+01_JPRB ,&
&  0.23242E+01_JPRB, 0.20923E+01_JPRB, 0.18265E+01_JPRB, 0.17431E+01_JPRB, 0.19234E+01_JPRB, 0.19887E+01_JPRB, 0.19737E+01_JPRB ,&
&  0.15943E+01_JPRB, 0.12602E+01_JPRB, 0.94856E+00_JPRB, 0.88139E+00_JPRB, 0.72743E+00_JPRB, 0.75588E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.13300E+01_JPRB, 0.14327E+01_JPRB, 0.13082E+01_JPRB, 0.90555E+00_JPRB, 0.66602E+00_JPRB, 0.58763E+00_JPRB ,&
&  0.57355E+00_JPRB, 0.62134E+00_JPRB, 0.65820E+00_JPRB, 0.68080E+00_JPRB, 0.74367E+00_JPRB, 0.80443E+00_JPRB, 0.87855E+00_JPRB ,&
&  0.91808E+00_JPRB, 0.97772E+00_JPRB, 0.10424E+01_JPRB, 0.11075E+01_JPRB, 0.12092E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.10853E+01_JPRB, 0.11424E+01_JPRB, 0.11916E+01_JPRB, 0.12248E+01_JPRB, 0.12608E+01_JPRB, 0.12505E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.15437E+01_JPRB, 0.16865E+01_JPRB, 0.17787E+01_JPRB, 0.18133E+01_JPRB, 0.18832E+01_JPRB, 0.20385E+01_JPRB ,&
&  0.21884E+01_JPRB, 0.23556E+01_JPRB, 0.25038E+01_JPRB, 0.26515E+01_JPRB, 0.26457E+01_JPRB, 0.25669E+01_JPRB, 0.19911E+01_JPRB ,&
&  0.13878E+01_JPRB, 0.22023E+01_JPRB, 0.20817E+01_JPRB, 0.24376E+01_JPRB, 0.26644E+01_JPRB, 0.28747E+01_JPRB, 0.28605E+01_JPRB ,&
&  0.27035E+01_JPRB, 0.26120E+01_JPRB, 0.25365E+01_JPRB, 0.24509E+01_JPRB, 0.24118E+01_JPRB, 0.25147E+01_JPRB, 0.26781E+01_JPRB ,&
&  0.28375E+01_JPRB, 0.29695E+01_JPRB, 0.31346E+01_JPRB, 0.33736E+01_JPRB, 0.36513E+01_JPRB, 0.39432E+01_JPRB, 0.41408E+01_JPRB ,&
&  0.41612E+01_JPRB, 0.41798E+01_JPRB, 0.39718E+01_JPRB, 0.36969E+01_JPRB, 0.36740E+01_JPRB, 0.39851E+01_JPRB, 0.44039E+01_JPRB ,&
&  0.46601E+01_JPRB, 0.48952E+01_JPRB, 0.55575E+01_JPRB, 0.59002E+01_JPRB, 0.57499E+01_JPRB, 0.55595E+01_JPRB, 0.48413E+01_JPRB ,&
&  0.51363E+01_JPRB, 0.47114E+01_JPRB, 0.45525E+01_JPRB, 0.38960E+01_JPRB, 0.36251E+01_JPRB, 0.36078E+01_JPRB, 0.35843E+01_JPRB ,&
&  0.32980E+01_JPRB, 0.31606E+01_JPRB, 0.27703E+01_JPRB, 0.25646E+01_JPRB, 0.26155E+01_JPRB, 0.27614E+01_JPRB, 0.26006E+01_JPRB ,&
&  0.22359E+01_JPRB, 0.15376E+01_JPRB, 0.11384E+01_JPRB, 0.97859E+00_JPRB, 0.83789E+00_JPRB, 0.87149E+00_JPRB, 0.12537E+01_JPRB ,&
&  0.17818E+01_JPRB, 0.18344E+01_JPRB, 0.19467E+01_JPRB, 0.17387E+01_JPRB, 0.11770E+01_JPRB, 0.80327E+00_JPRB, 0.65766E+00_JPRB ,&
&  0.65206E+00_JPRB, 0.70655E+00_JPRB, 0.73515E+00_JPRB, 0.75550E+00_JPRB, 0.80499E+00_JPRB, 0.80111E+00_JPRB, 0.86036E+00_JPRB ,&
&  0.88410E+00_JPRB, 0.92606E+00_JPRB, 0.96125E+00_JPRB, 0.97540E+00_JPRB, 0.10194E+01_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.88238E+00_JPRB, 0.91282E+00_JPRB, 0.96255E+00_JPRB, 0.99469E+00_JPRB, 0.10105E+01_JPRB, 0.10398E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.12765E+01_JPRB, 0.14359E+01_JPRB, 0.16006E+01_JPRB, 0.16481E+01_JPRB, 0.16607E+01_JPRB, 0.17810E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.23105E+01_JPRB, 0.26545E+01_JPRB, 0.29565E+01_JPRB, 0.30079E+01_JPRB, 0.28351E+01_JPRB, 0.20429E+01_JPRB ,&
&  0.16934E+01_JPRB, 0.21486E+01_JPRB, 0.21279E+01_JPRB, 0.23601E+01_JPRB, 0.26324E+01_JPRB, 0.28190E+01_JPRB, 0.27431E+01_JPRB ,&
&  0.25882E+01_JPRB, 0.24643E+01_JPRB, 0.23282E+01_JPRB, 0.22714E+01_JPRB, 0.22690E+01_JPRB, 0.22895E+01_JPRB, 0.23626E+01_JPRB ,&
&  0.24990E+01_JPRB, 0.26410E+01_JPRB, 0.28718E+01_JPRB, 0.32047E+01_JPRB, 0.35570E+01_JPRB, 0.38957E+01_JPRB, 0.41877E+01_JPRB ,&
&  0.43400E+01_JPRB, 0.43266E+01_JPRB, 0.43003E+01_JPRB, 0.38038E+01_JPRB, 0.38301E+01_JPRB, 0.43045E+01_JPRB, 0.50347E+01_JPRB ,&
&  0.54463E+01_JPRB, 0.57137E+01_JPRB, 0.66257E+01_JPRB, 0.74388E+01_JPRB, 0.74463E+01_JPRB, 0.67651E+01_JPRB, 0.59671E+01_JPRB ,&
&  0.59978E+01_JPRB, 0.58014E+01_JPRB, 0.52928E+01_JPRB, 0.48801E+01_JPRB, 0.46732E+01_JPRB, 0.47879E+01_JPRB, 0.49396E+01_JPRB ,&
&  0.44329E+01_JPRB, 0.40863E+01_JPRB, 0.36800E+01_JPRB, 0.33993E+01_JPRB, 0.32033E+01_JPRB, 0.31298E+01_JPRB, 0.33050E+01_JPRB ,&
&  0.30262E+01_JPRB, 0.22425E+01_JPRB, 0.15891E+01_JPRB, 0.11124E+01_JPRB, 0.95260E+00_JPRB, 0.10743E+01_JPRB, 0.14872E+01_JPRB ,&
&  0.22049E+01_JPRB, 0.23456E+01_JPRB, 0.22414E+01_JPRB, 0.24505E+01_JPRB, 0.18143E+01_JPRB, 0.11262E+01_JPRB, 0.74374E+00_JPRB ,&
&  0.69491E+00_JPRB, 0.74416E+00_JPRB, 0.76608E+00_JPRB, 0.74945E+00_JPRB, 0.77506E+00_JPRB, 0.80630E+00_JPRB, 0.79872E+00_JPRB ,&
&  0.83317E+00_JPRB, 0.86978E+00_JPRB, 0.85946E+00_JPRB, 0.84168E+00_JPRB, 0.85914E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.77727E+00_JPRB, 0.81116E+00_JPRB, 0.81905E+00_JPRB, 0.81885E+00_JPRB, 0.83521E+00_JPRB, 0.86258E+00_JPRB, 0.87066E+00_JPRB ,&
&  0.93504E+00_JPRB, 0.10380E+01_JPRB, 0.11644E+01_JPRB, 0.13356E+01_JPRB, 0.14366E+01_JPRB, 0.14396E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.19463E+01_JPRB, 0.24689E+01_JPRB, 0.29701E+01_JPRB, 0.30458E+01_JPRB, 0.27552E+01_JPRB, 0.25374E+01_JPRB ,&
&  0.22119E+01_JPRB, 0.24111E+01_JPRB, 0.23210E+01_JPRB, 0.27672E+01_JPRB, 0.32061E+01_JPRB, 0.35579E+01_JPRB, 0.25738E+01_JPRB ,&
&  0.23427E+01_JPRB, 0.22586E+01_JPRB, 0.21423E+01_JPRB, 0.21190E+01_JPRB, 0.20378E+01_JPRB, 0.20134E+01_JPRB, 0.20019E+01_JPRB ,&
&  0.20376E+01_JPRB, 0.22256E+01_JPRB, 0.24886E+01_JPRB, 0.28709E+01_JPRB, 0.33739E+01_JPRB, 0.37857E+01_JPRB, 0.40951E+01_JPRB ,&
&  0.42490E+01_JPRB, 0.44534E+01_JPRB, 0.45475E+01_JPRB, 0.41213E+01_JPRB, 0.39369E+01_JPRB, 0.44931E+01_JPRB, 0.55233E+01_JPRB ,&
&  0.67336E+01_JPRB, 0.76701E+01_JPRB, 0.80715E+01_JPRB, 0.81376E+01_JPRB, 0.81964E+01_JPRB, 0.79109E+01_JPRB, 0.69002E+01_JPRB ,&
&  0.65469E+01_JPRB, 0.63496E+01_JPRB, 0.56240E+01_JPRB, 0.57139E+01_JPRB, 0.58513E+01_JPRB, 0.61804E+01_JPRB, 0.53759E+01_JPRB ,&
&  0.46106E+01_JPRB, 0.41442E+01_JPRB, 0.38674E+01_JPRB, 0.39439E+01_JPRB, 0.39257E+01_JPRB, 0.36882E+01_JPRB, 0.39215E+01_JPRB ,&
&  0.30816E+01_JPRB, 0.11734E+01_JPRB, 0.79107E+00_JPRB, 0.94178E+00_JPRB, 0.65387E+00_JPRB, 0.80514E+00_JPRB, 0.16824E+01_JPRB ,&
&  0.26498E+01_JPRB, 0.33814E+01_JPRB, 0.33189E+01_JPRB, 0.27791E+01_JPRB, 0.26383E+01_JPRB, 0.16676E+01_JPRB, 0.91988E+00_JPRB ,&
&  0.78505E+00_JPRB, 0.85094E+00_JPRB, 0.85761E+00_JPRB, 0.79927E+00_JPRB, 0.78051E+00_JPRB, 0.81155E+00_JPRB, 0.74610E+00_JPRB ,&
&  0.77438E+00_JPRB, 0.79256E+00_JPRB, 0.80210E+00_JPRB, 0.78704E+00_JPRB, 0.76337E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.73256E+00_JPRB, 0.76256E+00_JPRB, 0.78626E+00_JPRB, 0.74735E+00_JPRB, 0.73875E+00_JPRB, 0.75997E+00_JPRB, 0.76565E+00_JPRB ,&
&  0.82278E+00_JPRB, 0.89399E+00_JPRB, 0.97067E+00_JPRB, 0.10945E+01_JPRB, 0.12155E+01_JPRB, 0.12528E+01_JPRB, 0.12717E+01_JPRB ,&
&  0.13454E+01_JPRB, 0.15792E+01_JPRB, 0.20732E+01_JPRB, 0.27689E+01_JPRB, 0.31473E+01_JPRB, 0.29683E+01_JPRB, 0.30754E+01_JPRB ,&
&  0.25639E+01_JPRB, 0.29012E+01_JPRB, 0.37427E+01_JPRB, 0.47704E+01_JPRB, 0.50704E+01_JPRB, 0.46711E+01_JPRB, 0.29571E+01_JPRB ,&
&  0.23505E+01_JPRB, 0.22630E+01_JPRB, 0.20698E+01_JPRB, 0.21025E+01_JPRB, 0.18835E+01_JPRB, 0.18439E+01_JPRB, 0.18037E+01_JPRB ,&
&  0.18673E+01_JPRB, 0.20226E+01_JPRB, 0.21904E+01_JPRB, 0.25452E+01_JPRB, 0.31331E+01_JPRB, 0.36989E+01_JPRB, 0.42371E+01_JPRB ,&
&  0.43204E+01_JPRB, 0.45231E+01_JPRB, 0.46375E+01_JPRB, 0.48558E+01_JPRB, 0.36705E+01_JPRB, 0.49480E+01_JPRB, 0.58987E+01_JPRB ,&
&  0.74548E+01_JPRB, 0.85138E+01_JPRB, 0.84188E+01_JPRB, 0.93753E+01_JPRB, 0.97838E+01_JPRB, 0.88994E+01_JPRB, 0.74813E+01_JPRB ,&
&  0.69293E+01_JPRB, 0.62320E+01_JPRB, 0.59403E+01_JPRB, 0.61263E+01_JPRB, 0.65102E+01_JPRB, 0.75849E+01_JPRB, 0.50726E+01_JPRB ,&
&  0.45853E+01_JPRB, 0.50146E+01_JPRB, 0.45678E+01_JPRB, 0.39201E+01_JPRB, 0.42962E+01_JPRB, 0.43643E+01_JPRB, 0.32259E+01_JPRB ,&
&  0.13045E+01_JPRB, 0.68664E+00_JPRB, 0.57549E+00_JPRB, 0.57215E+00_JPRB, 0.52300E+00_JPRB, 0.86994E+00_JPRB, 0.34818E+01_JPRB ,&
&  0.27373E+01_JPRB, 0.40321E+01_JPRB, 0.40832E+01_JPRB, 0.35498E+01_JPRB, 0.33230E+01_JPRB, 0.24122E+01_JPRB, 0.18619E+01_JPRB ,&
&  0.94700E+00_JPRB, 0.93284E+00_JPRB, 0.10104E+01_JPRB, 0.99276E+00_JPRB, 0.98272E+00_JPRB, 0.92026E+00_JPRB, 0.82757E+00_JPRB ,&
&  0.83120E+00_JPRB, 0.85408E+00_JPRB, 0.81043E+00_JPRB, 0.76316E+00_JPRB, 0.73492E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.73243E+00_JPRB, 0.73420E+00_JPRB, 0.73969E+00_JPRB, 0.72513E+00_JPRB, 0.68558E+00_JPRB, 0.69084E+00_JPRB, 0.71454E+00_JPRB ,&
&  0.74981E+00_JPRB, 0.79536E+00_JPRB, 0.85087E+00_JPRB, 0.92657E+00_JPRB, 0.10231E+01_JPRB, 0.10919E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.14035E+01_JPRB, 0.18027E+01_JPRB, 0.23207E+01_JPRB, 0.29770E+01_JPRB, 0.28924E+01_JPRB, 0.33362E+01_JPRB ,&
&  0.28914E+01_JPRB, 0.31744E+01_JPRB, 0.45034E+01_JPRB, 0.63185E+01_JPRB, 0.83674E+01_JPRB, 0.78777E+01_JPRB, 0.56673E+01_JPRB ,&
&  0.38126E+01_JPRB, 0.30621E+01_JPRB, 0.27086E+01_JPRB, 0.23166E+01_JPRB, 0.20165E+01_JPRB, 0.19063E+01_JPRB, 0.19167E+01_JPRB ,&
&  0.18655E+01_JPRB, 0.19014E+01_JPRB, 0.19581E+01_JPRB, 0.22196E+01_JPRB, 0.27506E+01_JPRB, 0.34096E+01_JPRB, 0.42152E+01_JPRB ,&
&  0.47223E+01_JPRB, 0.50456E+01_JPRB, 0.50263E+01_JPRB, 0.50824E+01_JPRB, 0.47819E+01_JPRB, 0.47758E+01_JPRB, 0.58841E+01_JPRB ,&
&  0.79466E+01_JPRB, 0.99369E+01_JPRB, 0.99433E+01_JPRB, 0.11461E+02_JPRB, 0.10648E+02_JPRB, 0.96776E+01_JPRB, 0.84012E+01_JPRB ,&
&  0.71827E+01_JPRB, 0.72806E+01_JPRB, 0.58780E+01_JPRB, 0.60075E+01_JPRB, 0.67041E+01_JPRB, 0.59803E+01_JPRB, 0.53656E+01_JPRB ,&
&  0.59840E+01_JPRB, 0.50434E+01_JPRB, 0.44366E+01_JPRB, 0.37674E+01_JPRB, 0.47069E+01_JPRB, 0.40436E+01_JPRB, 0.18574E+01_JPRB ,&
&  0.11399E+01_JPRB, 0.80709E+00_JPRB, 0.55039E+00_JPRB, 0.56574E+00_JPRB, 0.59193E+00_JPRB, 0.88534E+00_JPRB, 0.24210E+01_JPRB ,&
&  0.28812E+01_JPRB, 0.42247E+01_JPRB, 0.44922E+01_JPRB, 0.39511E+01_JPRB, 0.34663E+01_JPRB, 0.30998E+01_JPRB, 0.32680E+01_JPRB ,&
&  0.21096E+01_JPRB, 0.14142E+01_JPRB, 0.11539E+01_JPRB, 0.11791E+01_JPRB, 0.11788E+01_JPRB, 0.10978E+01_JPRB, 0.99972E+00_JPRB ,&
&  0.94735E+00_JPRB, 0.94576E+00_JPRB, 0.92033E+00_JPRB, 0.88113E+00_JPRB, 0.80673E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.83416E+00_JPRB, 0.79731E+00_JPRB, 0.78960E+00_JPRB, 0.77856E+00_JPRB, 0.74129E+00_JPRB, 0.71794E+00_JPRB, 0.70673E+00_JPRB ,&
&  0.71778E+00_JPRB, 0.74565E+00_JPRB, 0.78305E+00_JPRB, 0.83354E+00_JPRB, 0.89188E+00_JPRB, 0.95312E+00_JPRB, 0.10042E+01_JPRB ,&
&  0.11010E+01_JPRB, 0.13192E+01_JPRB, 0.15893E+01_JPRB, 0.18776E+01_JPRB, 0.23910E+01_JPRB, 0.25465E+01_JPRB, 0.31418E+01_JPRB ,&
&  0.32114E+01_JPRB, 0.34211E+01_JPRB, 0.48165E+01_JPRB, 0.73471E+01_JPRB, 0.11158E+02_JPRB, 0.12177E+02_JPRB, 0.10962E+02_JPRB ,&
&  0.84285E+01_JPRB, 0.56761E+01_JPRB, 0.40152E+01_JPRB, 0.29786E+01_JPRB, 0.23447E+01_JPRB, 0.20017E+01_JPRB, 0.19642E+01_JPRB ,&
&  0.18676E+01_JPRB, 0.18471E+01_JPRB, 0.18929E+01_JPRB, 0.19324E+01_JPRB, 0.22676E+01_JPRB, 0.29134E+01_JPRB, 0.39411E+01_JPRB ,&
&  0.48846E+01_JPRB, 0.55839E+01_JPRB, 0.58059E+01_JPRB, 0.51962E+01_JPRB, 0.61315E+01_JPRB, 0.66003E+01_JPRB, 0.65258E+01_JPRB ,&
&  0.72030E+01_JPRB, 0.85381E+01_JPRB, 0.12015E+02_JPRB, 0.11527E+02_JPRB, 0.10686E+02_JPRB, 0.10713E+02_JPRB, 0.90626E+01_JPRB ,&
&  0.79113E+01_JPRB, 0.74534E+01_JPRB, 0.63084E+01_JPRB, 0.64301E+01_JPRB, 0.60442E+01_JPRB, 0.53230E+01_JPRB, 0.59391E+01_JPRB ,&
&  0.64164E+01_JPRB, 0.57893E+01_JPRB, 0.46861E+01_JPRB, 0.38045E+01_JPRB, 0.36370E+01_JPRB, 0.24865E+01_JPRB, 0.14914E+01_JPRB ,&
&  0.11949E+01_JPRB, 0.95959E+00_JPRB, 0.67591E+00_JPRB, 0.84347E+00_JPRB, 0.76124E+00_JPRB, 0.10716E+01_JPRB, 0.20947E+01_JPRB ,&
&  0.34488E+01_JPRB, 0.40828E+01_JPRB, 0.46276E+01_JPRB, 0.37976E+01_JPRB, 0.33347E+01_JPRB, 0.33187E+01_JPRB, 0.48068E+01_JPRB ,&
&  0.30705E+01_JPRB, 0.27154E+01_JPRB, 0.19746E+01_JPRB, 0.15213E+01_JPRB, 0.13136E+01_JPRB, 0.12029E+01_JPRB, 0.11393E+01_JPRB ,&
&  0.10712E+01_JPRB, 0.10201E+01_JPRB, 0.10025E+01_JPRB, 0.96732E+00_JPRB, 0.90010E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.10378E+01_JPRB, 0.98749E+00_JPRB, 0.94594E+00_JPRB, 0.90844E+00_JPRB, 0.86689E+00_JPRB, 0.82226E+00_JPRB, 0.77793E+00_JPRB ,&
&  0.75810E+00_JPRB, 0.76280E+00_JPRB, 0.78532E+00_JPRB, 0.82617E+00_JPRB, 0.85879E+00_JPRB, 0.89630E+00_JPRB, 0.93566E+00_JPRB ,&
&  0.10606E+01_JPRB, 0.13491E+01_JPRB, 0.13199E+01_JPRB, 0.14548E+01_JPRB, 0.18783E+01_JPRB, 0.25012E+01_JPRB, 0.28384E+01_JPRB ,&
&  0.33105E+01_JPRB, 0.34303E+01_JPRB, 0.44450E+01_JPRB, 0.68165E+01_JPRB, 0.13775E+02_JPRB, 0.15987E+02_JPRB, 0.14988E+02_JPRB ,&
&  0.14316E+02_JPRB, 0.10435E+02_JPRB, 0.66390E+01_JPRB, 0.45667E+01_JPRB, 0.32556E+01_JPRB, 0.23536E+01_JPRB, 0.19350E+01_JPRB ,&
&  0.17813E+01_JPRB, 0.17343E+01_JPRB, 0.17351E+01_JPRB, 0.17890E+01_JPRB, 0.19611E+01_JPRB, 0.24550E+01_JPRB, 0.34917E+01_JPRB ,&
&  0.46302E+01_JPRB, 0.57513E+01_JPRB, 0.64885E+01_JPRB, 0.63763E+01_JPRB, 0.55555E+01_JPRB, 0.65605E+01_JPRB, 0.99496E+01_JPRB ,&
&  0.10026E+02_JPRB, 0.11418E+02_JPRB, 0.12200E+02_JPRB, 0.12505E+02_JPRB, 0.11933E+02_JPRB, 0.10627E+02_JPRB, 0.96384E+01_JPRB ,&
&  0.67312E+01_JPRB, 0.57794E+01_JPRB, 0.62207E+01_JPRB, 0.64359E+01_JPRB, 0.57516E+01_JPRB, 0.66210E+01_JPRB, 0.89058E+01_JPRB ,&
&  0.67099E+01_JPRB, 0.67769E+01_JPRB, 0.67962E+01_JPRB, 0.66651E+01_JPRB, 0.36746E+01_JPRB, 0.23238E+01_JPRB, 0.34398E+01_JPRB ,&
&  0.33356E+01_JPRB, 0.17687E+01_JPRB, 0.13003E+01_JPRB, 0.18211E+01_JPRB, 0.12132E+01_JPRB, 0.15371E+01_JPRB, 0.25236E+01_JPRB ,&
&  0.29199E+01_JPRB, 0.32541E+01_JPRB, 0.49802E+01_JPRB, 0.41881E+01_JPRB, 0.37956E+01_JPRB, 0.33843E+01_JPRB, 0.38516E+01_JPRB ,&
&  0.40798E+01_JPRB, 0.32600E+01_JPRB, 0.27150E+01_JPRB, 0.20358E+01_JPRB, 0.16105E+01_JPRB, 0.14571E+01_JPRB, 0.13733E+01_JPRB ,&
&  0.13241E+01_JPRB, 0.12656E+01_JPRB, 0.12175E+01_JPRB, 0.11376E+01_JPRB, 0.10885E+01_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.12021E+01_JPRB, 0.11551E+01_JPRB, 0.11088E+01_JPRB, 0.10589E+01_JPRB, 0.10117E+01_JPRB, 0.94754E+00_JPRB, 0.87728E+00_JPRB ,&
&  0.83027E+00_JPRB, 0.81683E+00_JPRB, 0.83553E+00_JPRB, 0.85793E+00_JPRB, 0.86548E+00_JPRB, 0.88423E+00_JPRB, 0.90445E+00_JPRB ,&
&  0.10241E+01_JPRB, 0.12163E+01_JPRB, 0.12335E+01_JPRB, 0.13378E+01_JPRB, 0.17979E+01_JPRB, 0.25141E+01_JPRB, 0.29963E+01_JPRB ,&
&  0.36290E+01_JPRB, 0.35271E+01_JPRB, 0.41578E+01_JPRB, 0.64752E+01_JPRB, 0.11349E+02_JPRB, 0.14679E+02_JPRB, 0.14326E+02_JPRB ,&
&  0.13927E+02_JPRB, 0.11511E+02_JPRB, 0.87086E+01_JPRB, 0.65139E+01_JPRB, 0.46964E+01_JPRB, 0.33841E+01_JPRB, 0.24657E+01_JPRB ,&
&  0.19778E+01_JPRB, 0.17573E+01_JPRB, 0.17004E+01_JPRB, 0.17000E+01_JPRB, 0.18153E+01_JPRB, 0.20225E+01_JPRB, 0.28021E+01_JPRB ,&
&  0.38091E+01_JPRB, 0.52303E+01_JPRB, 0.68716E+01_JPRB, 0.77174E+01_JPRB, 0.67715E+01_JPRB, 0.78992E+01_JPRB, 0.10777E+02_JPRB ,&
&  0.11449E+02_JPRB, 0.12218E+02_JPRB, 0.13523E+02_JPRB, 0.13152E+02_JPRB, 0.13776E+02_JPRB, 0.12120E+02_JPRB, 0.11543E+02_JPRB ,&
&  0.84352E+01_JPRB, 0.68738E+01_JPRB, 0.61758E+01_JPRB, 0.60215E+01_JPRB, 0.73732E+01_JPRB, 0.97617E+01_JPRB, 0.91417E+01_JPRB ,&
&  0.80737E+01_JPRB, 0.76558E+01_JPRB, 0.70037E+01_JPRB, 0.56242E+01_JPRB, 0.54356E+01_JPRB, 0.42825E+01_JPRB, 0.50648E+01_JPRB ,&
&  0.49043E+01_JPRB, 0.44242E+01_JPRB, 0.36220E+01_JPRB, 0.27362E+01_JPRB, 0.21124E+01_JPRB, 0.25963E+01_JPRB, 0.28189E+01_JPRB ,&
&  0.28511E+01_JPRB, 0.28620E+01_JPRB, 0.40701E+01_JPRB, 0.42129E+01_JPRB, 0.41755E+01_JPRB, 0.32858E+01_JPRB, 0.34500E+01_JPRB ,&
&  0.33211E+01_JPRB, 0.35048E+01_JPRB, 0.32273E+01_JPRB, 0.25650E+01_JPRB, 0.21288E+01_JPRB, 0.18302E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.16104E+01_JPRB, 0.15488E+01_JPRB, 0.14566E+01_JPRB, 0.13504E+01_JPRB, 0.12708E+01_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.12637E+01_JPRB, 0.12279E+01_JPRB, 0.11783E+01_JPRB, 0.11175E+01_JPRB, 0.10549E+01_JPRB, 0.99799E+00_JPRB, 0.94490E+00_JPRB ,&
&  0.90537E+00_JPRB, 0.87865E+00_JPRB, 0.88157E+00_JPRB, 0.89669E+00_JPRB, 0.90837E+00_JPRB, 0.89378E+00_JPRB, 0.89006E+00_JPRB ,&
&  0.96542E+00_JPRB, 0.11617E+01_JPRB, 0.11619E+01_JPRB, 0.13334E+01_JPRB, 0.17776E+01_JPRB, 0.24857E+01_JPRB, 0.34288E+01_JPRB ,&
&  0.38859E+01_JPRB, 0.39460E+01_JPRB, 0.44201E+01_JPRB, 0.61572E+01_JPRB, 0.84783E+01_JPRB, 0.99839E+01_JPRB, 0.10256E+02_JPRB ,&
&  0.88304E+01_JPRB, 0.78324E+01_JPRB, 0.67026E+01_JPRB, 0.56980E+01_JPRB, 0.49240E+01_JPRB, 0.42226E+01_JPRB, 0.34087E+01_JPRB ,&
&  0.26770E+01_JPRB, 0.21934E+01_JPRB, 0.19603E+01_JPRB, 0.18613E+01_JPRB, 0.18496E+01_JPRB, 0.19238E+01_JPRB, 0.22691E+01_JPRB ,&
&  0.29305E+01_JPRB, 0.41351E+01_JPRB, 0.63541E+01_JPRB, 0.88803E+01_JPRB, 0.88367E+01_JPRB, 0.10795E+02_JPRB, 0.11357E+02_JPRB ,&
&  0.13531E+02_JPRB, 0.15089E+02_JPRB, 0.14790E+02_JPRB, 0.14703E+02_JPRB, 0.12127E+02_JPRB, 0.13063E+02_JPRB, 0.13458E+02_JPRB ,&
&  0.12496E+02_JPRB, 0.11675E+02_JPRB, 0.11528E+02_JPRB, 0.99259E+01_JPRB, 0.91020E+01_JPRB, 0.10900E+02_JPRB, 0.94547E+01_JPRB ,&
&  0.81290E+01_JPRB, 0.76502E+01_JPRB, 0.68969E+01_JPRB, 0.67815E+01_JPRB, 0.55738E+01_JPRB, 0.54070E+01_JPRB, 0.45905E+01_JPRB ,&
&  0.42189E+01_JPRB, 0.34277E+01_JPRB, 0.38189E+01_JPRB, 0.32393E+01_JPRB, 0.30009E+01_JPRB, 0.29721E+01_JPRB, 0.29507E+01_JPRB ,&
&  0.28302E+01_JPRB, 0.25908E+01_JPRB, 0.26824E+01_JPRB, 0.32774E+01_JPRB, 0.40715E+01_JPRB, 0.33490E+01_JPRB, 0.33142E+01_JPRB ,&
&  0.32236E+01_JPRB, 0.32243E+01_JPRB, 0.29080E+01_JPRB, 0.27459E+01_JPRB, 0.24929E+01_JPRB, 0.22819E+01_JPRB, 0.20398E+01_JPRB ,&
&  0.17723E+01_JPRB, 0.15509E+01_JPRB, 0.14309E+01_JPRB, 0.13460E+01_JPRB, 0.12870E+01_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.12410E+01_JPRB, 0.12314E+01_JPRB, 0.11811E+01_JPRB, 0.11120E+01_JPRB, 0.10571E+01_JPRB, 0.10085E+01_JPRB, 0.97267E+00_JPRB ,&
&  0.93816E+00_JPRB, 0.91184E+00_JPRB, 0.90294E+00_JPRB, 0.90779E+00_JPRB, 0.91600E+00_JPRB, 0.91700E+00_JPRB, 0.90915E+00_JPRB ,&
&  0.91762E+00_JPRB, 0.11450E+01_JPRB, 0.11395E+01_JPRB, 0.11683E+01_JPRB, 0.15669E+01_JPRB, 0.25520E+01_JPRB, 0.35177E+01_JPRB ,&
&  0.39511E+01_JPRB, 0.40877E+01_JPRB, 0.45046E+01_JPRB, 0.51633E+01_JPRB, 0.59521E+01_JPRB, 0.64708E+01_JPRB, 0.66465E+01_JPRB ,&
&  0.65035E+01_JPRB, 0.49986E+01_JPRB, 0.45177E+01_JPRB, 0.40712E+01_JPRB, 0.38049E+01_JPRB, 0.35675E+01_JPRB, 0.33239E+01_JPRB ,&
&  0.29754E+01_JPRB, 0.25371E+01_JPRB, 0.22027E+01_JPRB, 0.19936E+01_JPRB, 0.18864E+01_JPRB, 0.17916E+01_JPRB, 0.19046E+01_JPRB ,&
&  0.21601E+01_JPRB, 0.25792E+01_JPRB, 0.39610E+01_JPRB, 0.62680E+01_JPRB, 0.97756E+01_JPRB, 0.12228E+02_JPRB, 0.12687E+02_JPRB ,&
&  0.11375E+02_JPRB, 0.12340E+02_JPRB, 0.17866E+02_JPRB, 0.15022E+02_JPRB, 0.14493E+02_JPRB, 0.15232E+02_JPRB, 0.14245E+02_JPRB ,&
&  0.12786E+02_JPRB, 0.11529E+02_JPRB, 0.10615E+02_JPRB, 0.99223E+01_JPRB, 0.11070E+02_JPRB, 0.10530E+02_JPRB, 0.91991E+01_JPRB ,&
&  0.79398E+01_JPRB, 0.77036E+01_JPRB, 0.72265E+01_JPRB, 0.66865E+01_JPRB, 0.62455E+01_JPRB, 0.64688E+01_JPRB, 0.59389E+01_JPRB ,&
&  0.54199E+01_JPRB, 0.58848E+01_JPRB, 0.46447E+01_JPRB, 0.35291E+01_JPRB, 0.28589E+01_JPRB, 0.29520E+01_JPRB, 0.31030E+01_JPRB ,&
&  0.28101E+01_JPRB, 0.23614E+01_JPRB, 0.19854E+01_JPRB, 0.21397E+01_JPRB, 0.27050E+01_JPRB, 0.30214E+01_JPRB, 0.28505E+01_JPRB ,&
&  0.26515E+01_JPRB, 0.27638E+01_JPRB, 0.26749E+01_JPRB, 0.25957E+01_JPRB, 0.26181E+01_JPRB, 0.23538E+01_JPRB, 0.19877E+01_JPRB ,&
&  0.16629E+01_JPRB, 0.14604E+01_JPRB, 0.13579E+01_JPRB, 0.12985E+01_JPRB, 0.12526E+01_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.12448E+01_JPRB, 0.12229E+01_JPRB, 0.11791E+01_JPRB, 0.11146E+01_JPRB, 0.10599E+01_JPRB, 0.10183E+01_JPRB, 0.98033E+00_JPRB ,&
&  0.96152E+00_JPRB, 0.94308E+00_JPRB, 0.93963E+00_JPRB, 0.93194E+00_JPRB, 0.93616E+00_JPRB, 0.94681E+00_JPRB, 0.95368E+00_JPRB ,&
&  0.95413E+00_JPRB, 0.11672E+01_JPRB, 0.11284E+01_JPRB, 0.11941E+01_JPRB, 0.14454E+01_JPRB, 0.22825E+01_JPRB, 0.29691E+01_JPRB ,&
&  0.33506E+01_JPRB, 0.34470E+01_JPRB, 0.36031E+01_JPRB, 0.39620E+01_JPRB, 0.41651E+01_JPRB, 0.40667E+01_JPRB, 0.39914E+01_JPRB ,&
&  0.38168E+01_JPRB, 0.35341E+01_JPRB, 0.29579E+01_JPRB, 0.29212E+01_JPRB, 0.27003E+01_JPRB, 0.23458E+01_JPRB, 0.25195E+01_JPRB ,&
&  0.25250E+01_JPRB, 0.24466E+01_JPRB, 0.22850E+01_JPRB, 0.21391E+01_JPRB, 0.20421E+01_JPRB, 0.19197E+01_JPRB, 0.18717E+01_JPRB ,&
&  0.19030E+01_JPRB, 0.19898E+01_JPRB, 0.23170E+01_JPRB, 0.32545E+01_JPRB, 0.53691E+01_JPRB, 0.81505E+01_JPRB, 0.10605E+02_JPRB ,&
&  0.12991E+02_JPRB, 0.14130E+02_JPRB, 0.12838E+02_JPRB, 0.13516E+02_JPRB, 0.15029E+02_JPRB, 0.13645E+02_JPRB, 0.12512E+02_JPRB ,&
&  0.12385E+02_JPRB, 0.12189E+02_JPRB, 0.11537E+02_JPRB, 0.10881E+02_JPRB, 0.10694E+02_JPRB, 0.10243E+02_JPRB, 0.94846E+01_JPRB ,&
&  0.80638E+01_JPRB, 0.75349E+01_JPRB, 0.74226E+01_JPRB, 0.72473E+01_JPRB, 0.63571E+01_JPRB, 0.59262E+01_JPRB, 0.59251E+01_JPRB ,&
&  0.57228E+01_JPRB, 0.55107E+01_JPRB, 0.42092E+01_JPRB, 0.39569E+01_JPRB, 0.29278E+01_JPRB, 0.28126E+01_JPRB, 0.31531E+01_JPRB ,&
&  0.27093E+01_JPRB, 0.21765E+01_JPRB, 0.16919E+01_JPRB, 0.15007E+01_JPRB, 0.16491E+01_JPRB, 0.18906E+01_JPRB, 0.22261E+01_JPRB ,&
&  0.22240E+01_JPRB, 0.18661E+01_JPRB, 0.20498E+01_JPRB, 0.20524E+01_JPRB, 0.21267E+01_JPRB, 0.20414E+01_JPRB, 0.18617E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.16002E+01_JPRB, 0.14823E+01_JPRB, 0.13706E+01_JPRB, 0.12929E+01_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.12002E+01_JPRB, 0.11363E+01_JPRB, 0.11000E+01_JPRB, 0.10581E+01_JPRB, 0.10169E+01_JPRB, 0.98183E+00_JPRB, 0.95923E+00_JPRB ,&
&  0.97172E+00_JPRB, 0.97965E+00_JPRB, 0.97981E+00_JPRB, 0.96259E+00_JPRB, 0.94923E+00_JPRB, 0.94514E+00_JPRB, 0.97070E+00_JPRB ,&
&  0.99528E+00_JPRB, 0.11359E+01_JPRB, 0.12027E+01_JPRB, 0.12605E+01_JPRB, 0.17177E+01_JPRB, 0.19888E+01_JPRB, 0.22650E+01_JPRB ,&
&  0.23674E+01_JPRB, 0.24690E+01_JPRB, 0.24903E+01_JPRB, 0.25020E+01_JPRB, 0.24723E+01_JPRB, 0.24200E+01_JPRB, 0.22876E+01_JPRB ,&
&  0.20908E+01_JPRB, 0.18633E+01_JPRB, 0.17507E+01_JPRB, 0.17290E+01_JPRB, 0.17797E+01_JPRB, 0.15973E+01_JPRB, 0.17063E+01_JPRB ,&
&  0.18050E+01_JPRB, 0.19011E+01_JPRB, 0.19406E+01_JPRB, 0.19470E+01_JPRB, 0.19279E+01_JPRB, 0.19211E+01_JPRB, 0.19332E+01_JPRB ,&
&  0.19895E+01_JPRB, 0.19921E+01_JPRB, 0.20096E+01_JPRB, 0.23835E+01_JPRB, 0.35024E+01_JPRB, 0.59339E+01_JPRB, 0.85300E+01_JPRB ,&
&  0.92913E+01_JPRB, 0.11378E+02_JPRB, 0.14127E+02_JPRB, 0.14722E+02_JPRB, 0.12324E+02_JPRB, 0.10605E+02_JPRB, 0.10240E+02_JPRB ,&
&  0.10719E+02_JPRB, 0.11364E+02_JPRB, 0.11476E+02_JPRB, 0.10594E+02_JPRB, 0.10308E+02_JPRB, 0.10046E+02_JPRB, 0.95964E+01_JPRB ,&
&  0.88938E+01_JPRB, 0.81558E+01_JPRB, 0.76986E+01_JPRB, 0.73306E+01_JPRB, 0.70284E+01_JPRB, 0.63543E+01_JPRB, 0.60774E+01_JPRB ,&
&  0.61538E+01_JPRB, 0.49631E+01_JPRB, 0.43229E+01_JPRB, 0.45168E+01_JPRB, 0.35280E+01_JPRB, 0.34187E+01_JPRB, 0.36982E+01_JPRB ,&
&  0.29154E+01_JPRB, 0.21300E+01_JPRB, 0.17781E+01_JPRB, 0.14963E+01_JPRB, 0.14020E+01_JPRB, 0.15055E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.15928E+01_JPRB, 0.15233E+01_JPRB, 0.16894E+01_JPRB, 0.17553E+01_JPRB, 0.17909E+01_JPRB, 0.17398E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.15337E+01_JPRB, 0.14600E+01_JPRB, 0.14061E+01_JPRB, 0.13488E+01_JPRB, 0.12854E+01_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.11331E+01_JPRB, 0.10828E+01_JPRB, 0.10434E+01_JPRB, 0.10194E+01_JPRB, 0.99626E+00_JPRB, 0.97093E+00_JPRB, 0.95075E+00_JPRB ,&
&  0.95421E+00_JPRB, 0.97189E+00_JPRB, 0.98696E+00_JPRB, 0.97274E+00_JPRB, 0.95341E+00_JPRB, 0.94346E+00_JPRB, 0.97158E+00_JPRB ,&
&  0.86655E+00_JPRB, 0.93044E+00_JPRB, 0.99568E+00_JPRB, 0.12141E+01_JPRB, 0.17068E+01_JPRB, 0.16311E+01_JPRB, 0.16880E+01_JPRB ,&
&  0.16605E+01_JPRB, 0.16159E+01_JPRB, 0.15882E+01_JPRB, 0.15880E+01_JPRB, 0.15469E+01_JPRB, 0.15161E+01_JPRB, 0.14478E+01_JPRB ,&
&  0.12603E+01_JPRB, 0.11253E+01_JPRB, 0.10805E+01_JPRB, 0.10585E+01_JPRB, 0.11153E+01_JPRB, 0.11896E+01_JPRB, 0.13019E+01_JPRB ,&
&  0.13898E+01_JPRB, 0.14769E+01_JPRB, 0.15492E+01_JPRB, 0.16038E+01_JPRB, 0.16342E+01_JPRB, 0.16643E+01_JPRB, 0.17029E+01_JPRB ,&
&  0.17578E+01_JPRB, 0.18044E+01_JPRB, 0.18730E+01_JPRB, 0.21260E+01_JPRB, 0.28016E+01_JPRB, 0.49679E+01_JPRB, 0.70427E+01_JPRB ,&
&  0.64412E+01_JPRB, 0.64003E+01_JPRB, 0.85449E+01_JPRB, 0.10431E+02_JPRB, 0.91070E+01_JPRB, 0.77903E+01_JPRB, 0.72199E+01_JPRB ,&
&  0.75796E+01_JPRB, 0.82845E+01_JPRB, 0.90124E+01_JPRB, 0.91158E+01_JPRB, 0.87038E+01_JPRB, 0.86278E+01_JPRB, 0.84716E+01_JPRB ,&
&  0.81543E+01_JPRB, 0.78776E+01_JPRB, 0.75431E+01_JPRB, 0.72505E+01_JPRB, 0.69862E+01_JPRB, 0.70624E+01_JPRB, 0.69168E+01_JPRB ,&
&  0.66755E+01_JPRB, 0.62217E+01_JPRB, 0.55587E+01_JPRB, 0.51654E+01_JPRB, 0.40992E+01_JPRB, 0.39279E+01_JPRB, 0.45193E+01_JPRB ,&
&  0.33882E+01_JPRB, 0.24652E+01_JPRB, 0.20276E+01_JPRB, 0.17183E+01_JPRB, 0.15763E+01_JPRB, 0.14345E+01_JPRB, 0.13208E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.12738E+01_JPRB, 0.13925E+01_JPRB, 0.14323E+01_JPRB, 0.14478E+01_JPRB, 0.14442E+01_JPRB, 0.13518E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.12462E+01_JPRB, 0.12218E+01_JPRB, 0.12027E+01_JPRB, 0.11748E+01_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.10473E+01_JPRB, 0.10209E+01_JPRB, 0.99106E+00_JPRB, 0.95727E+00_JPRB, 0.93301E+00_JPRB, 0.92449E+00_JPRB, 0.92830E+00_JPRB ,&
&  0.94337E+00_JPRB, 0.96508E+00_JPRB, 0.97434E+00_JPRB, 0.96646E+00_JPRB, 0.95516E+00_JPRB, 0.95611E+00_JPRB, 0.87691E+00_JPRB ,&
&  0.77788E+00_JPRB, 0.80414E+00_JPRB, 0.81830E+00_JPRB, 0.91516E+00_JPRB, 0.10273E+01_JPRB, 0.10514E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.11154E+01_JPRB, 0.10846E+01_JPRB, 0.10864E+01_JPRB, 0.10878E+01_JPRB, 0.10725E+01_JPRB, 0.10536E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.92339E+00_JPRB, 0.85795E+00_JPRB, 0.82397E+00_JPRB, 0.80750E+00_JPRB, 0.85277E+00_JPRB, 0.93300E+00_JPRB, 0.10179E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11747E+01_JPRB, 0.12473E+01_JPRB, 0.13063E+01_JPRB, 0.13431E+01_JPRB, 0.13751E+01_JPRB, 0.14142E+01_JPRB ,&
&  0.14648E+01_JPRB, 0.15216E+01_JPRB, 0.15638E+01_JPRB, 0.17388E+01_JPRB, 0.20278E+01_JPRB, 0.31562E+01_JPRB, 0.42822E+01_JPRB ,&
&  0.39876E+01_JPRB, 0.39256E+01_JPRB, 0.42957E+01_JPRB, 0.48152E+01_JPRB, 0.49690E+01_JPRB, 0.47873E+01_JPRB, 0.48198E+01_JPRB ,&
&  0.52261E+01_JPRB, 0.57270E+01_JPRB, 0.67614E+01_JPRB, 0.68220E+01_JPRB, 0.68727E+01_JPRB, 0.70182E+01_JPRB, 0.69015E+01_JPRB ,&
&  0.69857E+01_JPRB, 0.69480E+01_JPRB, 0.70919E+01_JPRB, 0.69368E+01_JPRB, 0.67296E+01_JPRB, 0.67209E+01_JPRB, 0.66380E+01_JPRB ,&
&  0.66657E+01_JPRB, 0.65939E+01_JPRB, 0.60886E+01_JPRB, 0.56583E+01_JPRB, 0.49726E+01_JPRB, 0.49186E+01_JPRB, 0.44114E+01_JPRB ,&
&  0.35053E+01_JPRB, 0.25011E+01_JPRB, 0.22597E+01_JPRB, 0.19271E+01_JPRB, 0.16864E+01_JPRB, 0.14285E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.11726E+01_JPRB, 0.12485E+01_JPRB, 0.12717E+01_JPRB, 0.12876E+01_JPRB, 0.13066E+01_JPRB, 0.12841E+01_JPRB, 0.11616E+01_JPRB ,&
&  0.11271E+01_JPRB, 0.11593E+01_JPRB, 0.11395E+01_JPRB, 0.11037E+01_JPRB, 0.10713E+01_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.97671E+00_JPRB, 0.97211E+00_JPRB, 0.96014E+00_JPRB, 0.93283E+00_JPRB, 0.89027E+00_JPRB, 0.88900E+00_JPRB, 0.84478E+00_JPRB ,&
&  0.92306E+00_JPRB, 0.95059E+00_JPRB, 0.94798E+00_JPRB, 0.95821E+00_JPRB, 0.98006E+00_JPRB, 0.89806E+00_JPRB, 0.74131E+00_JPRB ,&
&  0.67528E+00_JPRB, 0.67956E+00_JPRB, 0.70836E+00_JPRB, 0.73570E+00_JPRB, 0.75868E+00_JPRB, 0.77646E+00_JPRB, 0.80437E+00_JPRB ,&
&  0.82574E+00_JPRB, 0.82707E+00_JPRB, 0.83974E+00_JPRB, 0.84042E+00_JPRB, 0.83157E+00_JPRB, 0.82484E+00_JPRB, 0.82524E+00_JPRB ,&
&  0.77466E+00_JPRB, 0.74534E+00_JPRB, 0.72766E+00_JPRB, 0.69695E+00_JPRB, 0.74116E+00_JPRB, 0.78142E+00_JPRB, 0.83670E+00_JPRB ,&
&  0.90716E+00_JPRB, 0.98082E+00_JPRB, 0.10379E+01_JPRB, 0.10925E+01_JPRB, 0.11345E+01_JPRB, 0.11697E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.12448E+01_JPRB, 0.12877E+01_JPRB, 0.13333E+01_JPRB, 0.14365E+01_JPRB, 0.16208E+01_JPRB, 0.19943E+01_JPRB, 0.25949E+01_JPRB ,&
&  0.27922E+01_JPRB, 0.30730E+01_JPRB, 0.33165E+01_JPRB, 0.30299E+01_JPRB, 0.33278E+01_JPRB, 0.37625E+01_JPRB, 0.38068E+01_JPRB ,&
&  0.39349E+01_JPRB, 0.38954E+01_JPRB, 0.40381E+01_JPRB, 0.42490E+01_JPRB, 0.48030E+01_JPRB, 0.52031E+01_JPRB, 0.53786E+01_JPRB ,&
&  0.54708E+01_JPRB, 0.58290E+01_JPRB, 0.61336E+01_JPRB, 0.62508E+01_JPRB, 0.62271E+01_JPRB, 0.62522E+01_JPRB, 0.63241E+01_JPRB ,&
&  0.63765E+01_JPRB, 0.64037E+01_JPRB, 0.62826E+01_JPRB, 0.59619E+01_JPRB, 0.55006E+01_JPRB, 0.49497E+01_JPRB, 0.42706E+01_JPRB ,&
&  0.36820E+01_JPRB, 0.30633E+01_JPRB, 0.25595E+01_JPRB, 0.22304E+01_JPRB, 0.19075E+01_JPRB, 0.15860E+01_JPRB, 0.13585E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.11414E+01_JPRB, 0.12273E+01_JPRB, 0.12449E+01_JPRB, 0.12560E+01_JPRB, 0.12239E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.11285E+01_JPRB, 0.11124E+01_JPRB, 0.10582E+01_JPRB, 0.10067E+01_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.98251E+00_JPRB, 0.94888E+00_JPRB, 0.93434E+00_JPRB, 0.93216E+00_JPRB, 0.89655E+00_JPRB, 0.83453E+00_JPRB, 0.77737E+00_JPRB ,&
&  0.88791E+00_JPRB, 0.86897E+00_JPRB, 0.91244E+00_JPRB, 0.75154E+00_JPRB, 0.69542E+00_JPRB, 0.67225E+00_JPRB, 0.63846E+00_JPRB ,&
&  0.61767E+00_JPRB, 0.61802E+00_JPRB, 0.63042E+00_JPRB, 0.65867E+00_JPRB, 0.68506E+00_JPRB, 0.69536E+00_JPRB, 0.70374E+00_JPRB ,&
&  0.70940E+00_JPRB, 0.71687E+00_JPRB, 0.73550E+00_JPRB, 0.74318E+00_JPRB, 0.74382E+00_JPRB, 0.74236E+00_JPRB, 0.74505E+00_JPRB ,&
&  0.71181E+00_JPRB, 0.69893E+00_JPRB, 0.70067E+00_JPRB, 0.69666E+00_JPRB, 0.70895E+00_JPRB, 0.72480E+00_JPRB, 0.75378E+00_JPRB ,&
&  0.80998E+00_JPRB, 0.77948E+00_JPRB, 0.89882E+00_JPRB, 0.93658E+00_JPRB, 0.97528E+00_JPRB, 0.10076E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10612E+01_JPRB, 0.11034E+01_JPRB, 0.11491E+01_JPRB, 0.12063E+01_JPRB, 0.13396E+01_JPRB, 0.15054E+01_JPRB, 0.16896E+01_JPRB ,&
&  0.20601E+01_JPRB, 0.18977E+01_JPRB, 0.21812E+01_JPRB, 0.23381E+01_JPRB, 0.28026E+01_JPRB, 0.32337E+01_JPRB, 0.32808E+01_JPRB ,&
&  0.37665E+01_JPRB, 0.35285E+01_JPRB, 0.33132E+01_JPRB, 0.31133E+01_JPRB, 0.32311E+01_JPRB, 0.34946E+01_JPRB, 0.37935E+01_JPRB ,&
&  0.40321E+01_JPRB, 0.41613E+01_JPRB, 0.44878E+01_JPRB, 0.47128E+01_JPRB, 0.49783E+01_JPRB, 0.53145E+01_JPRB, 0.56276E+01_JPRB ,&
&  0.58908E+01_JPRB, 0.60244E+01_JPRB, 0.60629E+01_JPRB, 0.56112E+01_JPRB, 0.52234E+01_JPRB, 0.48582E+01_JPRB, 0.41668E+01_JPRB ,&
&  0.36327E+01_JPRB, 0.32565E+01_JPRB, 0.29382E+01_JPRB, 0.27099E+01_JPRB, 0.22915E+01_JPRB, 0.19291E+01_JPRB, 0.15902E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.10689E+01_JPRB, 0.10785E+01_JPRB, 0.11085E+01_JPRB, 0.11214E+01_JPRB, 0.11351E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.11100E+01_JPRB, 0.11383E+01_JPRB, 0.10924E+01_JPRB, 0.10362E+01_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.10962E+01_JPRB, 0.10341E+01_JPRB, 0.97942E+00_JPRB, 0.95073E+00_JPRB, 0.91772E+00_JPRB, 0.84907E+00_JPRB, 0.75719E+00_JPRB ,&
&  0.75547E+00_JPRB, 0.71293E+00_JPRB, 0.68841E+00_JPRB, 0.66446E+00_JPRB, 0.64052E+00_JPRB, 0.60046E+00_JPRB, 0.57374E+00_JPRB ,&
&  0.58551E+00_JPRB, 0.62312E+00_JPRB, 0.66827E+00_JPRB, 0.70367E+00_JPRB, 0.71911E+00_JPRB, 0.71749E+00_JPRB, 0.70985E+00_JPRB ,&
&  0.70601E+00_JPRB, 0.70847E+00_JPRB, 0.71438E+00_JPRB, 0.72357E+00_JPRB, 0.71581E+00_JPRB, 0.70419E+00_JPRB, 0.69669E+00_JPRB ,&
&  0.68588E+00_JPRB, 0.67353E+00_JPRB, 0.66240E+00_JPRB, 0.67037E+00_JPRB, 0.68093E+00_JPRB, 0.70405E+00_JPRB, 0.71775E+00_JPRB ,&
&  0.63996E+00_JPRB, 0.65169E+00_JPRB, 0.76213E+00_JPRB, 0.80144E+00_JPRB, 0.84027E+00_JPRB, 0.87465E+00_JPRB, 0.90332E+00_JPRB ,&
&  0.93105E+00_JPRB, 0.97205E+00_JPRB, 0.10193E+01_JPRB, 0.10724E+01_JPRB, 0.11802E+01_JPRB, 0.13025E+01_JPRB, 0.14240E+01_JPRB ,&
&  0.16743E+01_JPRB, 0.16406E+01_JPRB, 0.16216E+01_JPRB, 0.18373E+01_JPRB, 0.25545E+01_JPRB, 0.27363E+01_JPRB, 0.27156E+01_JPRB ,&
&  0.27073E+01_JPRB, 0.25903E+01_JPRB, 0.25153E+01_JPRB, 0.25618E+01_JPRB, 0.25864E+01_JPRB, 0.27329E+01_JPRB, 0.29727E+01_JPRB ,&
&  0.29926E+01_JPRB, 0.32160E+01_JPRB, 0.34841E+01_JPRB, 0.37051E+01_JPRB, 0.39540E+01_JPRB, 0.43421E+01_JPRB, 0.48007E+01_JPRB ,&
&  0.52654E+01_JPRB, 0.56353E+01_JPRB, 0.57329E+01_JPRB, 0.55493E+01_JPRB, 0.51988E+01_JPRB, 0.48104E+01_JPRB, 0.43286E+01_JPRB ,&
&  0.37078E+01_JPRB, 0.32766E+01_JPRB, 0.29959E+01_JPRB, 0.27355E+01_JPRB, 0.24919E+01_JPRB, 0.21542E+01_JPRB, 0.16985E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.10532E+01_JPRB, 0.10101E+01_JPRB, 0.10082E+01_JPRB, 0.10290E+01_JPRB, 0.10652E+01_JPRB, 0.10848E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.10646E+01_JPRB, 0.10424E+01_JPRB, 0.10748E+01_JPRB, 0.11121E+01_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.10587E+01_JPRB, 0.10310E+01_JPRB, 0.96865E+00_JPRB, 0.95105E+00_JPRB, 0.90239E+00_JPRB, 0.85190E+00_JPRB, 0.78747E+00_JPRB ,&
&  0.74040E+00_JPRB, 0.69319E+00_JPRB, 0.65862E+00_JPRB, 0.61961E+00_JPRB, 0.60198E+00_JPRB, 0.60270E+00_JPRB, 0.63700E+00_JPRB ,&
&  0.68251E+00_JPRB, 0.71860E+00_JPRB, 0.74200E+00_JPRB, 0.74956E+00_JPRB, 0.74659E+00_JPRB, 0.73118E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.70210E+00_JPRB, 0.70169E+00_JPRB, 0.69345E+00_JPRB, 0.68585E+00_JPRB, 0.67385E+00_JPRB, 0.65539E+00_JPRB, 0.64348E+00_JPRB ,&
&  0.62573E+00_JPRB, 0.61682E+00_JPRB, 0.62287E+00_JPRB, 0.64403E+00_JPRB, 0.67096E+00_JPRB, 0.67875E+00_JPRB, 0.63173E+00_JPRB ,&
&  0.58681E+00_JPRB, 0.57490E+00_JPRB, 0.58870E+00_JPRB, 0.70552E+00_JPRB, 0.76011E+00_JPRB, 0.78900E+00_JPRB, 0.82882E+00_JPRB ,&
&  0.81272E+00_JPRB, 0.81815E+00_JPRB, 0.85363E+00_JPRB, 0.95677E+00_JPRB, 0.10193E+01_JPRB, 0.11290E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.14251E+01_JPRB, 0.16519E+01_JPRB, 0.18195E+01_JPRB, 0.16585E+01_JPRB, 0.17905E+01_JPRB, 0.22341E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.19618E+01_JPRB, 0.18802E+01_JPRB, 0.19515E+01_JPRB, 0.20174E+01_JPRB, 0.19984E+01_JPRB, 0.20529E+01_JPRB, 0.21137E+01_JPRB ,&
&  0.20822E+01_JPRB, 0.22195E+01_JPRB, 0.25107E+01_JPRB, 0.27898E+01_JPRB, 0.30596E+01_JPRB, 0.34137E+01_JPRB, 0.37824E+01_JPRB ,&
&  0.42323E+01_JPRB, 0.46056E+01_JPRB, 0.48287E+01_JPRB, 0.47136E+01_JPRB, 0.45392E+01_JPRB, 0.42557E+01_JPRB, 0.38561E+01_JPRB ,&
&  0.33276E+01_JPRB, 0.29497E+01_JPRB, 0.27005E+01_JPRB, 0.25158E+01_JPRB, 0.22858E+01_JPRB, 0.19677E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.11117E+01_JPRB, 0.10439E+01_JPRB, 0.10279E+01_JPRB, 0.10363E+01_JPRB, 0.10433E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.99830E+00_JPRB, 0.97718E+00_JPRB, 0.96091E+00_JPRB, 0.96914E+00_JPRB, 0.10414E+01_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.10128E+01_JPRB, 0.10257E+01_JPRB, 0.10296E+01_JPRB, 0.98645E+00_JPRB, 0.92811E+00_JPRB, 0.84594E+00_JPRB, 0.78399E+00_JPRB ,&
&  0.73851E+00_JPRB, 0.72556E+00_JPRB, 0.72329E+00_JPRB, 0.73215E+00_JPRB, 0.75274E+00_JPRB, 0.78340E+00_JPRB, 0.80763E+00_JPRB ,&
&  0.82955E+00_JPRB, 0.83443E+00_JPRB, 0.83479E+00_JPRB, 0.81922E+00_JPRB, 0.78919E+00_JPRB, 0.75316E+00_JPRB, 0.72692E+00_JPRB ,&
&  0.71235E+00_JPRB, 0.70325E+00_JPRB, 0.68822E+00_JPRB, 0.66421E+00_JPRB, 0.64651E+00_JPRB, 0.62344E+00_JPRB, 0.62393E+00_JPRB ,&
&  0.61188E+00_JPRB, 0.61209E+00_JPRB, 0.62960E+00_JPRB, 0.64068E+00_JPRB, 0.65600E+00_JPRB, 0.65636E+00_JPRB, 0.60384E+00_JPRB ,&
&  0.55206E+00_JPRB, 0.52628E+00_JPRB, 0.51617E+00_JPRB, 0.51933E+00_JPRB, 0.57465E+00_JPRB, 0.70419E+00_JPRB, 0.77075E+00_JPRB ,&
&  0.80664E+00_JPRB, 0.82999E+00_JPRB, 0.85797E+00_JPRB, 0.89058E+00_JPRB, 0.91282E+00_JPRB, 0.97544E+00_JPRB, 0.10949E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.14469E+01_JPRB, 0.15775E+01_JPRB, 0.13850E+01_JPRB, 0.12830E+01_JPRB, 0.13633E+01_JPRB, 0.14968E+01_JPRB ,&
&  0.16514E+01_JPRB, 0.15952E+01_JPRB, 0.15427E+01_JPRB, 0.16116E+01_JPRB, 0.16377E+01_JPRB, 0.16599E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.18135E+01_JPRB, 0.20595E+01_JPRB, 0.21922E+01_JPRB, 0.24061E+01_JPRB, 0.27140E+01_JPRB, 0.29780E+01_JPRB ,&
&  0.32936E+01_JPRB, 0.35152E+01_JPRB, 0.35983E+01_JPRB, 0.34176E+01_JPRB, 0.32884E+01_JPRB, 0.32620E+01_JPRB, 0.31614E+01_JPRB ,&
&  0.28494E+01_JPRB, 0.26147E+01_JPRB, 0.24206E+01_JPRB, 0.22521E+01_JPRB, 0.20174E+01_JPRB, 0.18103E+01_JPRB, 0.15953E+01_JPRB ,&
&  0.14045E+01_JPRB, 0.12771E+01_JPRB, 0.12133E+01_JPRB, 0.11663E+01_JPRB, 0.11438E+01_JPRB, 0.11272E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10141E+01_JPRB, 0.99057E+00_JPRB, 0.98863E+00_JPRB, 0.10137E+01_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.10819E+01_JPRB, 0.10734E+01_JPRB, 0.10575E+01_JPRB, 0.10357E+01_JPRB, 0.99514E+00_JPRB, 0.94508E+00_JPRB, 0.91194E+00_JPRB ,&
&  0.89364E+00_JPRB, 0.89601E+00_JPRB, 0.89710E+00_JPRB, 0.90255E+00_JPRB, 0.91487E+00_JPRB, 0.93218E+00_JPRB, 0.94605E+00_JPRB ,&
&  0.94753E+00_JPRB, 0.92809E+00_JPRB, 0.89141E+00_JPRB, 0.83398E+00_JPRB, 0.78708E+00_JPRB, 0.75384E+00_JPRB, 0.73472E+00_JPRB ,&
&  0.72227E+00_JPRB, 0.70161E+00_JPRB, 0.67373E+00_JPRB, 0.63477E+00_JPRB, 0.60675E+00_JPRB, 0.58605E+00_JPRB, 0.59077E+00_JPRB ,&
&  0.61125E+00_JPRB, 0.61915E+00_JPRB, 0.61671E+00_JPRB, 0.63017E+00_JPRB, 0.62679E+00_JPRB, 0.62378E+00_JPRB, 0.56132E+00_JPRB ,&
&  0.51555E+00_JPRB, 0.49241E+00_JPRB, 0.47629E+00_JPRB, 0.47297E+00_JPRB, 0.48845E+00_JPRB, 0.52071E+00_JPRB, 0.58922E+00_JPRB ,&
&  0.72788E+00_JPRB, 0.77909E+00_JPRB, 0.79728E+00_JPRB, 0.82064E+00_JPRB, 0.84404E+00_JPRB, 0.87349E+00_JPRB, 0.93358E+00_JPRB ,&
&  0.10228E+01_JPRB, 0.11382E+01_JPRB, 0.12418E+01_JPRB, 0.12755E+01_JPRB, 0.12938E+01_JPRB, 0.13063E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.19909E+01_JPRB, 0.17711E+01_JPRB, 0.15169E+01_JPRB, 0.13814E+01_JPRB, 0.13206E+01_JPRB, 0.13001E+01_JPRB, 0.13813E+01_JPRB ,&
&  0.15016E+01_JPRB, 0.16154E+01_JPRB, 0.17651E+01_JPRB, 0.18809E+01_JPRB, 0.20146E+01_JPRB, 0.21946E+01_JPRB, 0.24484E+01_JPRB ,&
&  0.27033E+01_JPRB, 0.30101E+01_JPRB, 0.35652E+01_JPRB, 0.31785E+01_JPRB, 0.29567E+01_JPRB, 0.28659E+01_JPRB, 0.28110E+01_JPRB ,&
&  0.26725E+01_JPRB, 0.26384E+01_JPRB, 0.23988E+01_JPRB, 0.20970E+01_JPRB, 0.19368E+01_JPRB, 0.17566E+01_JPRB, 0.16172E+01_JPRB ,&
&  0.14939E+01_JPRB, 0.13871E+01_JPRB, 0.13234E+01_JPRB, 0.12727E+01_JPRB, 0.12475E+01_JPRB, 0.12219E+01_JPRB, 0.11946E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11375E+01_JPRB, 0.11166E+01_JPRB, 0.10992E+01_JPRB, 0.10873E+01_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.11485E+01_JPRB, 0.11256E+01_JPRB, 0.10996E+01_JPRB, 0.10784E+01_JPRB, 0.10553E+01_JPRB, 0.10392E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.10211E+01_JPRB, 0.10189E+01_JPRB, 0.10167E+01_JPRB, 0.10070E+01_JPRB, 0.99180E+00_JPRB, 0.96922E+00_JPRB ,&
&  0.93696E+00_JPRB, 0.86504E+00_JPRB, 0.81044E+00_JPRB, 0.77913E+00_JPRB, 0.75059E+00_JPRB, 0.72990E+00_JPRB, 0.70952E+00_JPRB ,&
&  0.68259E+00_JPRB, 0.65849E+00_JPRB, 0.62938E+00_JPRB, 0.59418E+00_JPRB, 0.57623E+00_JPRB, 0.58275E+00_JPRB, 0.59028E+00_JPRB ,&
&  0.60055E+00_JPRB, 0.61430E+00_JPRB, 0.60799E+00_JPRB, 0.59339E+00_JPRB, 0.58966E+00_JPRB, 0.58354E+00_JPRB, 0.53056E+00_JPRB ,&
&  0.49249E+00_JPRB, 0.46349E+00_JPRB, 0.45838E+00_JPRB, 0.45694E+00_JPRB, 0.46596E+00_JPRB, 0.48934E+00_JPRB, 0.54761E+00_JPRB ,&
&  0.64470E+00_JPRB, 0.76959E+00_JPRB, 0.78274E+00_JPRB, 0.79602E+00_JPRB, 0.81543E+00_JPRB, 0.84005E+00_JPRB, 0.87038E+00_JPRB ,&
&  0.90841E+00_JPRB, 0.95244E+00_JPRB, 0.10002E+01_JPRB, 0.10514E+01_JPRB, 0.11013E+01_JPRB, 0.11246E+01_JPRB, 0.11451E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.11828E+01_JPRB, 0.11946E+01_JPRB, 0.12073E+01_JPRB, 0.12240E+01_JPRB, 0.12092E+01_JPRB, 0.12618E+01_JPRB ,&
&  0.13841E+01_JPRB, 0.15114E+01_JPRB, 0.16358E+01_JPRB, 0.17266E+01_JPRB, 0.18283E+01_JPRB, 0.19735E+01_JPRB, 0.21306E+01_JPRB ,&
&  0.22686E+01_JPRB, 0.24582E+01_JPRB, 0.27173E+01_JPRB, 0.26905E+01_JPRB, 0.26345E+01_JPRB, 0.25937E+01_JPRB, 0.25816E+01_JPRB ,&
&  0.25563E+01_JPRB, 0.25658E+01_JPRB, 0.24130E+01_JPRB, 0.21928E+01_JPRB, 0.19878E+01_JPRB, 0.17677E+01_JPRB, 0.16387E+01_JPRB ,&
&  0.15461E+01_JPRB, 0.14151E+01_JPRB, 0.13455E+01_JPRB, 0.13016E+01_JPRB, 0.12697E+01_JPRB, 0.12382E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.12004E+01_JPRB, 0.11949E+01_JPRB, 0.11923E+01_JPRB, 0.11817E+01_JPRB, 0.11672E+01_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.11531E+01_JPRB, 0.11295E+01_JPRB, 0.11077E+01_JPRB, 0.10869E+01_JPRB, 0.10672E+01_JPRB, 0.10473E+01_JPRB, 0.10293E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.99826E+00_JPRB, 0.98173E+00_JPRB, 0.96094E+00_JPRB, 0.93765E+00_JPRB, 0.91052E+00_JPRB, 0.88542E+00_JPRB ,&
&  0.85784E+00_JPRB, 0.82901E+00_JPRB, 0.79112E+00_JPRB, 0.75613E+00_JPRB, 0.72999E+00_JPRB, 0.70985E+00_JPRB, 0.68742E+00_JPRB ,&
&  0.67010E+00_JPRB, 0.64568E+00_JPRB, 0.61192E+00_JPRB, 0.58784E+00_JPRB, 0.55952E+00_JPRB, 0.55228E+00_JPRB, 0.58697E+00_JPRB ,&
&  0.60138E+00_JPRB, 0.59920E+00_JPRB, 0.59014E+00_JPRB, 0.57444E+00_JPRB, 0.55941E+00_JPRB, 0.49885E+00_JPRB, 0.50445E+00_JPRB ,&
&  0.48121E+00_JPRB, 0.46214E+00_JPRB, 0.45992E+00_JPRB, 0.46172E+00_JPRB, 0.46967E+00_JPRB, 0.49150E+00_JPRB, 0.51761E+00_JPRB ,&
&  0.56461E+00_JPRB, 0.71350E+00_JPRB, 0.74821E+00_JPRB, 0.77966E+00_JPRB, 0.79546E+00_JPRB, 0.81106E+00_JPRB, 0.82732E+00_JPRB ,&
&  0.84708E+00_JPRB, 0.87140E+00_JPRB, 0.89344E+00_JPRB, 0.92213E+00_JPRB, 0.95216E+00_JPRB, 0.97499E+00_JPRB, 0.10143E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10681E+01_JPRB, 0.10945E+01_JPRB, 0.11196E+01_JPRB, 0.11575E+01_JPRB, 0.11614E+01_JPRB, 0.11739E+01_JPRB ,&
&  0.12730E+01_JPRB, 0.14078E+01_JPRB, 0.15283E+01_JPRB, 0.16360E+01_JPRB, 0.17228E+01_JPRB, 0.18205E+01_JPRB, 0.19435E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.21616E+01_JPRB, 0.23087E+01_JPRB, 0.23515E+01_JPRB, 0.23398E+01_JPRB, 0.23160E+01_JPRB, 0.23088E+01_JPRB ,&
&  0.22984E+01_JPRB, 0.23020E+01_JPRB, 0.22214E+01_JPRB, 0.21608E+01_JPRB, 0.20439E+01_JPRB, 0.19051E+01_JPRB, 0.17921E+01_JPRB ,&
&  0.16602E+01_JPRB, 0.15429E+01_JPRB, 0.14664E+01_JPRB, 0.14082E+01_JPRB, 0.13639E+01_JPRB, 0.13338E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.12616E+01_JPRB, 0.12328E+01_JPRB, 0.12042E+01_JPRB, 0.11762E+01_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.11179E+01_JPRB, 0.10979E+01_JPRB, 0.10791E+01_JPRB, 0.10524E+01_JPRB, 0.10256E+01_JPRB, 0.99997E+00_JPRB, 0.98006E+00_JPRB ,&
&  0.96261E+00_JPRB, 0.94404E+00_JPRB, 0.92583E+00_JPRB, 0.90462E+00_JPRB, 0.88385E+00_JPRB, 0.86333E+00_JPRB, 0.84440E+00_JPRB ,&
&  0.82585E+00_JPRB, 0.80264E+00_JPRB, 0.78020E+00_JPRB, 0.75621E+00_JPRB, 0.73525E+00_JPRB, 0.71552E+00_JPRB, 0.69102E+00_JPRB ,&
&  0.67304E+00_JPRB, 0.64339E+00_JPRB, 0.61326E+00_JPRB, 0.59886E+00_JPRB, 0.56620E+00_JPRB, 0.54816E+00_JPRB, 0.53780E+00_JPRB ,&
&  0.55438E+00_JPRB, 0.54149E+00_JPRB, 0.55653E+00_JPRB, 0.53458E+00_JPRB, 0.51837E+00_JPRB, 0.50348E+00_JPRB, 0.48940E+00_JPRB ,&
&  0.47930E+00_JPRB, 0.46809E+00_JPRB, 0.46893E+00_JPRB, 0.47536E+00_JPRB, 0.48063E+00_JPRB, 0.49843E+00_JPRB, 0.52034E+00_JPRB ,&
&  0.56719E+00_JPRB, 0.69569E+00_JPRB, 0.72863E+00_JPRB, 0.76106E+00_JPRB, 0.77483E+00_JPRB, 0.78912E+00_JPRB, 0.80135E+00_JPRB ,&
&  0.81901E+00_JPRB, 0.83282E+00_JPRB, 0.85095E+00_JPRB, 0.86505E+00_JPRB, 0.90016E+00_JPRB, 0.92705E+00_JPRB, 0.95560E+00_JPRB ,&
&  0.96850E+00_JPRB, 0.98318E+00_JPRB, 0.99823E+00_JPRB, 0.10309E+01_JPRB, 0.10885E+01_JPRB, 0.11201E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.11906E+01_JPRB, 0.12628E+01_JPRB, 0.13718E+01_JPRB, 0.14973E+01_JPRB, 0.15882E+01_JPRB, 0.16623E+01_JPRB, 0.17808E+01_JPRB ,&
&  0.18612E+01_JPRB, 0.19987E+01_JPRB, 0.21387E+01_JPRB, 0.21973E+01_JPRB, 0.21958E+01_JPRB, 0.21600E+01_JPRB, 0.21181E+01_JPRB ,&
&  0.20994E+01_JPRB, 0.20545E+01_JPRB, 0.20210E+01_JPRB, 0.19950E+01_JPRB, 0.19403E+01_JPRB, 0.18777E+01_JPRB, 0.18146E+01_JPRB ,&
&  0.17354E+01_JPRB, 0.16636E+01_JPRB, 0.16020E+01_JPRB, 0.15324E+01_JPRB, 0.14614E+01_JPRB, 0.13934E+01_JPRB, 0.13305E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12264E+01_JPRB, 0.11833E+01_JPRB, 0.11567E+01_JPRB, 0.11315E+01_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.10695E+01_JPRB, 0.10413E+01_JPRB, 0.10120E+01_JPRB, 0.98422E+00_JPRB, 0.96112E+00_JPRB, 0.94235E+00_JPRB, 0.92742E+00_JPRB ,&
&  0.91380E+00_JPRB, 0.89835E+00_JPRB, 0.88252E+00_JPRB, 0.86574E+00_JPRB, 0.84994E+00_JPRB, 0.83882E+00_JPRB, 0.82856E+00_JPRB ,&
&  0.82031E+00_JPRB, 0.80736E+00_JPRB, 0.79198E+00_JPRB, 0.77277E+00_JPRB, 0.75141E+00_JPRB, 0.73258E+00_JPRB, 0.70495E+00_JPRB ,&
&  0.68458E+00_JPRB, 0.66159E+00_JPRB, 0.62640E+00_JPRB, 0.60685E+00_JPRB, 0.57003E+00_JPRB, 0.54545E+00_JPRB, 0.53110E+00_JPRB ,&
&  0.52074E+00_JPRB, 0.52090E+00_JPRB, 0.52507E+00_JPRB, 0.53279E+00_JPRB, 0.53308E+00_JPRB, 0.52742E+00_JPRB, 0.52707E+00_JPRB ,&
&  0.51903E+00_JPRB, 0.51465E+00_JPRB, 0.51131E+00_JPRB, 0.51208E+00_JPRB, 0.51674E+00_JPRB, 0.52783E+00_JPRB, 0.54858E+00_JPRB ,&
&  0.59114E+00_JPRB, 0.65012E+00_JPRB, 0.70995E+00_JPRB, 0.75240E+00_JPRB, 0.76809E+00_JPRB, 0.78237E+00_JPRB, 0.79141E+00_JPRB ,&
&  0.80534E+00_JPRB, 0.81828E+00_JPRB, 0.83798E+00_JPRB, 0.85678E+00_JPRB, 0.87700E+00_JPRB, 0.89424E+00_JPRB, 0.91366E+00_JPRB ,&
&  0.92145E+00_JPRB, 0.93044E+00_JPRB, 0.93690E+00_JPRB, 0.95887E+00_JPRB, 0.99838E+00_JPRB, 0.10400E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10987E+01_JPRB, 0.11457E+01_JPRB, 0.12254E+01_JPRB, 0.13421E+01_JPRB, 0.14668E+01_JPRB, 0.15795E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.17902E+01_JPRB, 0.19131E+01_JPRB, 0.20268E+01_JPRB, 0.20665E+01_JPRB, 0.20582E+01_JPRB, 0.20639E+01_JPRB, 0.20320E+01_JPRB ,&
&  0.20059E+01_JPRB, 0.19737E+01_JPRB, 0.19518E+01_JPRB, 0.19291E+01_JPRB, 0.18848E+01_JPRB, 0.18205E+01_JPRB, 0.17544E+01_JPRB ,&
&  0.16836E+01_JPRB, 0.16089E+01_JPRB, 0.15373E+01_JPRB, 0.14659E+01_JPRB, 0.13978E+01_JPRB, 0.13326E+01_JPRB, 0.12777E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.11863E+01_JPRB, 0.11504E+01_JPRB, 0.11216E+01_JPRB, 0.10957E+01_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.10215E+01_JPRB, 0.99512E+00_JPRB, 0.97092E+00_JPRB, 0.94924E+00_JPRB, 0.92959E+00_JPRB, 0.91307E+00_JPRB, 0.89877E+00_JPRB ,&
&  0.88637E+00_JPRB, 0.87841E+00_JPRB, 0.87048E+00_JPRB, 0.86189E+00_JPRB, 0.85324E+00_JPRB, 0.84909E+00_JPRB, 0.84280E+00_JPRB ,&
&  0.83533E+00_JPRB, 0.82653E+00_JPRB, 0.81890E+00_JPRB, 0.80856E+00_JPRB, 0.79168E+00_JPRB, 0.77094E+00_JPRB, 0.74398E+00_JPRB ,&
&  0.72755E+00_JPRB, 0.70639E+00_JPRB, 0.68292E+00_JPRB, 0.65787E+00_JPRB, 0.64093E+00_JPRB, 0.60438E+00_JPRB, 0.59084E+00_JPRB ,&
&  0.58148E+00_JPRB, 0.56377E+00_JPRB, 0.55319E+00_JPRB, 0.54893E+00_JPRB, 0.56837E+00_JPRB, 0.58879E+00_JPRB, 0.60008E+00_JPRB ,&
&  0.58384E+00_JPRB, 0.59417E+00_JPRB, 0.59043E+00_JPRB, 0.58800E+00_JPRB, 0.59121E+00_JPRB, 0.59950E+00_JPRB, 0.63300E+00_JPRB ,&
&  0.69424E+00_JPRB, 0.72292E+00_JPRB, 0.74458E+00_JPRB, 0.76560E+00_JPRB, 0.78128E+00_JPRB, 0.79539E+00_JPRB, 0.80892E+00_JPRB ,&
&  0.82605E+00_JPRB, 0.84292E+00_JPRB, 0.85960E+00_JPRB, 0.86790E+00_JPRB, 0.87799E+00_JPRB, 0.88770E+00_JPRB, 0.89655E+00_JPRB ,&
&  0.89143E+00_JPRB, 0.89095E+00_JPRB, 0.89687E+00_JPRB, 0.90949E+00_JPRB, 0.92850E+00_JPRB, 0.96054E+00_JPRB, 0.99746E+00_JPRB ,&
&  0.10372E+01_JPRB, 0.10701E+01_JPRB, 0.11193E+01_JPRB, 0.11836E+01_JPRB, 0.12602E+01_JPRB, 0.13521E+01_JPRB, 0.14442E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.16050E+01_JPRB, 0.16706E+01_JPRB, 0.16981E+01_JPRB, 0.17080E+01_JPRB, 0.17132E+01_JPRB, 0.17061E+01_JPRB ,&
&  0.16949E+01_JPRB, 0.16760E+01_JPRB, 0.16573E+01_JPRB, 0.16321E+01_JPRB, 0.16016E+01_JPRB, 0.15627E+01_JPRB, 0.15195E+01_JPRB ,&
&  0.14699E+01_JPRB, 0.14223E+01_JPRB, 0.13759E+01_JPRB, 0.13315E+01_JPRB, 0.12873E+01_JPRB, 0.12465E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11415E+01_JPRB, 0.11103E+01_JPRB, 0.10805E+01_JPRB, 0.10513E+01_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.96480E+00_JPRB, 0.94759E+00_JPRB, 0.93187E+00_JPRB, 0.91742E+00_JPRB, 0.90534E+00_JPRB, 0.89568E+00_JPRB, 0.88720E+00_JPRB ,&
&  0.87961E+00_JPRB, 0.87486E+00_JPRB, 0.87083E+00_JPRB, 0.86688E+00_JPRB, 0.86312E+00_JPRB, 0.85958E+00_JPRB, 0.85571E+00_JPRB ,&
&  0.85217E+00_JPRB, 0.85001E+00_JPRB, 0.84849E+00_JPRB, 0.84639E+00_JPRB, 0.84141E+00_JPRB, 0.83326E+00_JPRB, 0.82354E+00_JPRB ,&
&  0.81308E+00_JPRB, 0.79975E+00_JPRB, 0.78288E+00_JPRB, 0.76389E+00_JPRB, 0.75296E+00_JPRB, 0.74039E+00_JPRB, 0.72825E+00_JPRB ,&
&  0.71813E+00_JPRB, 0.71538E+00_JPRB, 0.71251E+00_JPRB, 0.70999E+00_JPRB, 0.70817E+00_JPRB, 0.70883E+00_JPRB, 0.70874E+00_JPRB ,&
&  0.70861E+00_JPRB, 0.71104E+00_JPRB, 0.72183E+00_JPRB, 0.73353E+00_JPRB, 0.74709E+00_JPRB, 0.76315E+00_JPRB, 0.78094E+00_JPRB ,&
&  0.79516E+00_JPRB, 0.80524E+00_JPRB, 0.81707E+00_JPRB, 0.82869E+00_JPRB, 0.83803E+00_JPRB, 0.84601E+00_JPRB, 0.85378E+00_JPRB ,&
&  0.86244E+00_JPRB, 0.86893E+00_JPRB, 0.87483E+00_JPRB, 0.87862E+00_JPRB, 0.87882E+00_JPRB, 0.87758E+00_JPRB, 0.87450E+00_JPRB ,&
&  0.86972E+00_JPRB, 0.86973E+00_JPRB, 0.87382E+00_JPRB, 0.87913E+00_JPRB, 0.88885E+00_JPRB, 0.90296E+00_JPRB, 0.91962E+00_JPRB ,&
&  0.94112E+00_JPRB, 0.96780E+00_JPRB, 0.10008E+01_JPRB, 0.10370E+01_JPRB, 0.10785E+01_JPRB, 0.11289E+01_JPRB, 0.11693E+01_JPRB ,&
&  0.12054E+01_JPRB, 0.12356E+01_JPRB, 0.12617E+01_JPRB, 0.12781E+01_JPRB, 0.12882E+01_JPRB, 0.12968E+01_JPRB, 0.12934E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.12796E+01_JPRB, 0.12728E+01_JPRB, 0.12604E+01_JPRB, 0.12459E+01_JPRB, 0.12313E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.11779E+01_JPRB, 0.11588E+01_JPRB, 0.11407E+01_JPRB, 0.11206E+01_JPRB, 0.11014E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.10424E+01_JPRB, 0.10226E+01_JPRB, 0.10035E+01_JPRB, 0.98444E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.91873E+00_JPRB, 0.91373E+00_JPRB, 0.90926E+00_JPRB, 0.90462E+00_JPRB, 0.89973E+00_JPRB, 0.89565E+00_JPRB, 0.89195E+00_JPRB ,&
&  0.88849E+00_JPRB, 0.88613E+00_JPRB, 0.88477E+00_JPRB, 0.88391E+00_JPRB, 0.88380E+00_JPRB, 0.88344E+00_JPRB, 0.88342E+00_JPRB ,&
&  0.88355E+00_JPRB, 0.88411E+00_JPRB, 0.88531E+00_JPRB, 0.88597E+00_JPRB, 0.88588E+00_JPRB, 0.88541E+00_JPRB, 0.88460E+00_JPRB ,&
&  0.88286E+00_JPRB, 0.88071E+00_JPRB, 0.87810E+00_JPRB, 0.87524E+00_JPRB, 0.87183E+00_JPRB, 0.86787E+00_JPRB, 0.86340E+00_JPRB ,&
&  0.85937E+00_JPRB, 0.85657E+00_JPRB, 0.85417E+00_JPRB, 0.85153E+00_JPRB, 0.84988E+00_JPRB, 0.85039E+00_JPRB, 0.85082E+00_JPRB ,&
&  0.85123E+00_JPRB, 0.85212E+00_JPRB, 0.85482E+00_JPRB, 0.85755E+00_JPRB, 0.86005E+00_JPRB, 0.86296E+00_JPRB, 0.86665E+00_JPRB ,&
&  0.86932E+00_JPRB, 0.87150E+00_JPRB, 0.87343E+00_JPRB, 0.87533E+00_JPRB, 0.87663E+00_JPRB, 0.87816E+00_JPRB, 0.87931E+00_JPRB ,&
&  0.88006E+00_JPRB, 0.88061E+00_JPRB, 0.88039E+00_JPRB, 0.87874E+00_JPRB, 0.87378E+00_JPRB, 0.87010E+00_JPRB, 0.86854E+00_JPRB ,&
&  0.86902E+00_JPRB, 0.87017E+00_JPRB, 0.87189E+00_JPRB, 0.87442E+00_JPRB, 0.87853E+00_JPRB, 0.88336E+00_JPRB, 0.88896E+00_JPRB ,&
&  0.89507E+00_JPRB, 0.90223E+00_JPRB, 0.91154E+00_JPRB, 0.92134E+00_JPRB, 0.93213E+00_JPRB, 0.94449E+00_JPRB, 0.95560E+00_JPRB ,&
&  0.96720E+00_JPRB, 0.97922E+00_JPRB, 0.98923E+00_JPRB, 0.99803E+00_JPRB, 0.10066E+01_JPRB, 0.10147E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.10236E+01_JPRB, 0.10243E+01_JPRB, 0.10221E+01_JPRB, 0.10176E+01_JPRB, 0.10138E+01_JPRB, 0.10098E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.99914E+00_JPRB, 0.99359E+00_JPRB, 0.98761E+00_JPRB, 0.97998E+00_JPRB, 0.97174E+00_JPRB, 0.96405E+00_JPRB ,&
&  0.95630E+00_JPRB, 0.94788E+00_JPRB, 0.94003E+00_JPRB, 0.93254E+00_JPRB, 0.92549E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB ,&
&  0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB, 0.91496E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB ,&
&  0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB, 0.13327E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.13937E+00_JPRB, 0.13919E+00_JPRB, 0.13900E+00_JPRB, 0.13878E+00_JPRB, 0.13810E+00_JPRB, 0.13779E+00_JPRB, 0.13751E+00_JPRB ,&
&  0.13736E+00_JPRB, 0.13725E+00_JPRB, 0.13732E+00_JPRB, 0.13744E+00_JPRB, 0.13757E+00_JPRB, 0.13768E+00_JPRB, 0.13777E+00_JPRB ,&
&  0.13774E+00_JPRB, 0.13773E+00_JPRB, 0.13711E+00_JPRB, 0.13664E+00_JPRB, 0.13635E+00_JPRB, 0.13614E+00_JPRB, 0.13584E+00_JPRB ,&
&  0.13561E+00_JPRB, 0.13531E+00_JPRB, 0.13496E+00_JPRB, 0.13471E+00_JPRB, 0.13472E+00_JPRB, 0.13458E+00_JPRB, 0.13459E+00_JPRB ,&
&  0.13456E+00_JPRB, 0.13471E+00_JPRB, 0.13475E+00_JPRB, 0.13483E+00_JPRB, 0.13514E+00_JPRB, 0.13541E+00_JPRB, 0.13546E+00_JPRB ,&
&  0.13547E+00_JPRB, 0.13530E+00_JPRB, 0.13513E+00_JPRB, 0.13501E+00_JPRB, 0.13495E+00_JPRB, 0.13493E+00_JPRB, 0.13488E+00_JPRB ,&
&  0.13476E+00_JPRB, 0.13456E+00_JPRB, 0.13432E+00_JPRB, 0.13404E+00_JPRB, 0.13379E+00_JPRB, 0.13365E+00_JPRB, 0.13361E+00_JPRB ,&
&  0.13357E+00_JPRB, 0.13343E+00_JPRB, 0.13323E+00_JPRB, 0.13292E+00_JPRB, 0.13264E+00_JPRB, 0.13237E+00_JPRB, 0.13211E+00_JPRB ,&
&  0.13187E+00_JPRB, 0.13175E+00_JPRB, 0.13162E+00_JPRB, 0.13151E+00_JPRB, 0.13143E+00_JPRB, 0.13133E+00_JPRB, 0.13123E+00_JPRB ,&
&  0.13113E+00_JPRB, 0.13105E+00_JPRB, 0.13097E+00_JPRB, 0.13090E+00_JPRB, 0.13085E+00_JPRB, 0.13081E+00_JPRB, 0.13081E+00_JPRB ,&
&  0.13083E+00_JPRB, 0.13085E+00_JPRB, 0.13088E+00_JPRB, 0.13102E+00_JPRB, 0.13117E+00_JPRB, 0.13136E+00_JPRB, 0.13158E+00_JPRB ,&
&  0.13183E+00_JPRB, 0.13211E+00_JPRB, 0.13241E+00_JPRB, 0.13264E+00_JPRB, 0.13299E+00_JPRB, 0.13334E+00_JPRB, 0.13370E+00_JPRB ,&
&  0.13407E+00_JPRB, 0.13452E+00_JPRB, 0.13502E+00_JPRB, 0.13557E+00_JPRB, 0.13615E+00_JPRB, 0.13649E+00_JPRB, 0.13690E+00_JPRB ,&
&  0.13737E+00_JPRB, 0.13797E+00_JPRB, 0.13831E+00_JPRB, 0.13856E+00_JPRB, 0.13885E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.14969E+00_JPRB, 0.15168E+00_JPRB, 0.15311E+00_JPRB, 0.15487E+00_JPRB, 0.15428E+00_JPRB, 0.16588E+00_JPRB, 0.16391E+00_JPRB ,&
&  0.16276E+00_JPRB, 0.15796E+00_JPRB, 0.15755E+00_JPRB, 0.15591E+00_JPRB, 0.15394E+00_JPRB, 0.15177E+00_JPRB, 0.14977E+00_JPRB ,&
&  0.14792E+00_JPRB, 0.14588E+00_JPRB, 0.14439E+00_JPRB, 0.14319E+00_JPRB, 0.14209E+00_JPRB, 0.14108E+00_JPRB, 0.14028E+00_JPRB ,&
&  0.14022E+00_JPRB, 0.14074E+00_JPRB, 0.14172E+00_JPRB, 0.14160E+00_JPRB, 0.14132E+00_JPRB, 0.14095E+00_JPRB, 0.14075E+00_JPRB ,&
&  0.14022E+00_JPRB, 0.13983E+00_JPRB, 0.13950E+00_JPRB, 0.13898E+00_JPRB, 0.13954E+00_JPRB, 0.14008E+00_JPRB, 0.14063E+00_JPRB ,&
&  0.14115E+00_JPRB, 0.14166E+00_JPRB, 0.14163E+00_JPRB, 0.14150E+00_JPRB, 0.14111E+00_JPRB, 0.14057E+00_JPRB, 0.13974E+00_JPRB ,&
&  0.13893E+00_JPRB, 0.13809E+00_JPRB, 0.13722E+00_JPRB, 0.13661E+00_JPRB, 0.13607E+00_JPRB, 0.13556E+00_JPRB, 0.13513E+00_JPRB ,&
&  0.13491E+00_JPRB, 0.13476E+00_JPRB, 0.13458E+00_JPRB, 0.13429E+00_JPRB, 0.13394E+00_JPRB, 0.13358E+00_JPRB, 0.13326E+00_JPRB ,&
&  0.13301E+00_JPRB, 0.13256E+00_JPRB, 0.13215E+00_JPRB, 0.13170E+00_JPRB, 0.13137E+00_JPRB, 0.13095E+00_JPRB, 0.13060E+00_JPRB ,&
&  0.13028E+00_JPRB, 0.13004E+00_JPRB, 0.12997E+00_JPRB, 0.12992E+00_JPRB, 0.12989E+00_JPRB, 0.12994E+00_JPRB, 0.13006E+00_JPRB ,&
&  0.13022E+00_JPRB, 0.13041E+00_JPRB, 0.13066E+00_JPRB, 0.13084E+00_JPRB, 0.13102E+00_JPRB, 0.13117E+00_JPRB, 0.13154E+00_JPRB ,&
&  0.13198E+00_JPRB, 0.13246E+00_JPRB, 0.13291E+00_JPRB, 0.13314E+00_JPRB, 0.13391E+00_JPRB, 0.13470E+00_JPRB, 0.13557E+00_JPRB ,&
&  0.13649E+00_JPRB, 0.13767E+00_JPRB, 0.13895E+00_JPRB, 0.14031E+00_JPRB, 0.14188E+00_JPRB, 0.14292E+00_JPRB, 0.14406E+00_JPRB ,&
&  0.14559E+00_JPRB, 0.14810E+00_JPRB, 0.14903E+00_JPRB, 0.14942E+00_JPRB, 0.14967E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.17461E+00_JPRB, 0.17526E+00_JPRB, 0.17585E+00_JPRB, 0.17608E+00_JPRB, 0.17612E+00_JPRB, 0.17544E+00_JPRB, 0.17458E+00_JPRB ,&
&  0.17091E+00_JPRB, 0.16824E+00_JPRB, 0.16530E+00_JPRB, 0.16239E+00_JPRB, 0.15985E+00_JPRB, 0.15732E+00_JPRB, 0.15501E+00_JPRB ,&
&  0.15292E+00_JPRB, 0.15136E+00_JPRB, 0.14990E+00_JPRB, 0.14788E+00_JPRB, 0.14530E+00_JPRB, 0.14295E+00_JPRB, 0.14140E+00_JPRB ,&
&  0.14093E+00_JPRB, 0.14119E+00_JPRB, 0.14250E+00_JPRB, 0.14388E+00_JPRB, 0.14536E+00_JPRB, 0.14648E+00_JPRB, 0.14760E+00_JPRB ,&
&  0.14703E+00_JPRB, 0.14667E+00_JPRB, 0.14634E+00_JPRB, 0.14579E+00_JPRB, 0.14633E+00_JPRB, 0.14706E+00_JPRB, 0.14747E+00_JPRB ,&
&  0.14795E+00_JPRB, 0.14926E+00_JPRB, 0.15018E+00_JPRB, 0.15104E+00_JPRB, 0.15091E+00_JPRB, 0.15011E+00_JPRB, 0.14866E+00_JPRB ,&
&  0.14688E+00_JPRB, 0.14505E+00_JPRB, 0.14345E+00_JPRB, 0.14207E+00_JPRB, 0.14101E+00_JPRB, 0.13986E+00_JPRB, 0.13891E+00_JPRB ,&
&  0.13844E+00_JPRB, 0.13801E+00_JPRB, 0.13738E+00_JPRB, 0.13674E+00_JPRB, 0.13577E+00_JPRB, 0.13492E+00_JPRB, 0.13404E+00_JPRB ,&
&  0.13323E+00_JPRB, 0.13246E+00_JPRB, 0.13176E+00_JPRB, 0.13103E+00_JPRB, 0.13053E+00_JPRB, 0.13008E+00_JPRB, 0.12963E+00_JPRB ,&
&  0.12917E+00_JPRB, 0.12900E+00_JPRB, 0.12866E+00_JPRB, 0.12855E+00_JPRB, 0.12835E+00_JPRB, 0.12822E+00_JPRB, 0.12840E+00_JPRB ,&
&  0.12889E+00_JPRB, 0.12939E+00_JPRB, 0.12965E+00_JPRB, 0.12995E+00_JPRB, 0.13026E+00_JPRB, 0.13065E+00_JPRB, 0.13129E+00_JPRB ,&
&  0.13177E+00_JPRB, 0.13240E+00_JPRB, 0.13292E+00_JPRB, 0.13355E+00_JPRB, 0.13472E+00_JPRB, 0.13593E+00_JPRB, 0.13731E+00_JPRB ,&
&  0.13903E+00_JPRB, 0.14085E+00_JPRB, 0.14299E+00_JPRB, 0.14508E+00_JPRB, 0.14783E+00_JPRB, 0.14956E+00_JPRB, 0.15276E+00_JPRB ,&
&  0.15631E+00_JPRB, 0.15772E+00_JPRB, 0.16944E+00_JPRB, 0.17059E+00_JPRB, 0.17220E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.18094E+00_JPRB, 0.18208E+00_JPRB, 0.18188E+00_JPRB, 0.18218E+00_JPRB, 0.18120E+00_JPRB, 0.17968E+00_JPRB, 0.17752E+00_JPRB ,&
&  0.17501E+00_JPRB, 0.16947E+00_JPRB, 0.16562E+00_JPRB, 0.16135E+00_JPRB, 0.15776E+00_JPRB, 0.15463E+00_JPRB, 0.15243E+00_JPRB ,&
&  0.15147E+00_JPRB, 0.15118E+00_JPRB, 0.15103E+00_JPRB, 0.15022E+00_JPRB, 0.14872E+00_JPRB, 0.14772E+00_JPRB, 0.14644E+00_JPRB ,&
&  0.14549E+00_JPRB, 0.14540E+00_JPRB, 0.14618E+00_JPRB, 0.14794E+00_JPRB, 0.15105E+00_JPRB, 0.15337E+00_JPRB, 0.15544E+00_JPRB ,&
&  0.15762E+00_JPRB, 0.15433E+00_JPRB, 0.15412E+00_JPRB, 0.15441E+00_JPRB, 0.15428E+00_JPRB, 0.15353E+00_JPRB, 0.15210E+00_JPRB ,&
&  0.15476E+00_JPRB, 0.15544E+00_JPRB, 0.16011E+00_JPRB, 0.15998E+00_JPRB, 0.15896E+00_JPRB, 0.15983E+00_JPRB, 0.15829E+00_JPRB ,&
&  0.15772E+00_JPRB, 0.15472E+00_JPRB, 0.15251E+00_JPRB, 0.14950E+00_JPRB, 0.14660E+00_JPRB, 0.14514E+00_JPRB, 0.14348E+00_JPRB ,&
&  0.14294E+00_JPRB, 0.14283E+00_JPRB, 0.14157E+00_JPRB, 0.14079E+00_JPRB, 0.13936E+00_JPRB, 0.13774E+00_JPRB, 0.13672E+00_JPRB ,&
&  0.13566E+00_JPRB, 0.13482E+00_JPRB, 0.13437E+00_JPRB, 0.13394E+00_JPRB, 0.13376E+00_JPRB, 0.13368E+00_JPRB, 0.13359E+00_JPRB ,&
&  0.13361E+00_JPRB, 0.13353E+00_JPRB, 0.13302E+00_JPRB, 0.13273E+00_JPRB, 0.13181E+00_JPRB, 0.13118E+00_JPRB, 0.13066E+00_JPRB ,&
&  0.13022E+00_JPRB, 0.13007E+00_JPRB, 0.12998E+00_JPRB, 0.13003E+00_JPRB, 0.13014E+00_JPRB, 0.13055E+00_JPRB, 0.13101E+00_JPRB ,&
&  0.13148E+00_JPRB, 0.13231E+00_JPRB, 0.13297E+00_JPRB, 0.13389E+00_JPRB, 0.13500E+00_JPRB, 0.13594E+00_JPRB, 0.13745E+00_JPRB ,&
&  0.13911E+00_JPRB, 0.14107E+00_JPRB, 0.14395E+00_JPRB, 0.14663E+00_JPRB, 0.14975E+00_JPRB, 0.15407E+00_JPRB, 0.15881E+00_JPRB ,&
&  0.16403E+00_JPRB, 0.17023E+00_JPRB, 0.17095E+00_JPRB, 0.17591E+00_JPRB, 0.17916E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.18133E+00_JPRB, 0.18303E+00_JPRB, 0.18372E+00_JPRB, 0.18468E+00_JPRB, 0.18451E+00_JPRB, 0.18226E+00_JPRB, 0.17999E+00_JPRB ,&
&  0.17485E+00_JPRB, 0.17069E+00_JPRB, 0.16564E+00_JPRB, 0.16091E+00_JPRB, 0.15646E+00_JPRB, 0.15358E+00_JPRB, 0.15192E+00_JPRB ,&
&  0.15197E+00_JPRB, 0.15341E+00_JPRB, 0.15450E+00_JPRB, 0.15512E+00_JPRB, 0.15557E+00_JPRB, 0.15658E+00_JPRB, 0.15675E+00_JPRB ,&
&  0.15637E+00_JPRB, 0.15529E+00_JPRB, 0.15450E+00_JPRB, 0.15539E+00_JPRB, 0.15734E+00_JPRB, 0.15937E+00_JPRB, 0.16104E+00_JPRB ,&
&  0.16255E+00_JPRB, 0.16195E+00_JPRB, 0.16198E+00_JPRB, 0.16123E+00_JPRB, 0.16067E+00_JPRB, 0.15999E+00_JPRB, 0.15974E+00_JPRB ,&
&  0.16042E+00_JPRB, 0.16098E+00_JPRB, 0.16263E+00_JPRB, 0.16355E+00_JPRB, 0.16236E+00_JPRB, 0.16402E+00_JPRB, 0.16399E+00_JPRB ,&
&  0.16311E+00_JPRB, 0.16013E+00_JPRB, 0.15696E+00_JPRB, 0.15450E+00_JPRB, 0.15196E+00_JPRB, 0.14993E+00_JPRB, 0.14827E+00_JPRB ,&
&  0.14673E+00_JPRB, 0.14590E+00_JPRB, 0.14417E+00_JPRB, 0.14271E+00_JPRB, 0.14178E+00_JPRB, 0.14080E+00_JPRB, 0.14018E+00_JPRB ,&
&  0.13976E+00_JPRB, 0.13906E+00_JPRB, 0.13891E+00_JPRB, 0.13855E+00_JPRB, 0.13866E+00_JPRB, 0.13880E+00_JPRB, 0.13915E+00_JPRB ,&
&  0.13989E+00_JPRB, 0.14078E+00_JPRB, 0.14070E+00_JPRB, 0.14056E+00_JPRB, 0.13917E+00_JPRB, 0.13764E+00_JPRB, 0.13630E+00_JPRB ,&
&  0.13460E+00_JPRB, 0.13332E+00_JPRB, 0.13253E+00_JPRB, 0.13176E+00_JPRB, 0.13165E+00_JPRB, 0.13169E+00_JPRB, 0.13167E+00_JPRB ,&
&  0.13185E+00_JPRB, 0.13212E+00_JPRB, 0.13255E+00_JPRB, 0.13345E+00_JPRB, 0.13423E+00_JPRB, 0.13536E+00_JPRB, 0.13660E+00_JPRB ,&
&  0.13760E+00_JPRB, 0.13943E+00_JPRB, 0.14139E+00_JPRB, 0.14374E+00_JPRB, 0.14816E+00_JPRB, 0.15205E+00_JPRB, 0.15910E+00_JPRB ,&
&  0.16477E+00_JPRB, 0.17413E+00_JPRB, 0.17235E+00_JPRB, 0.17598E+00_JPRB, 0.17944E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.18362E+00_JPRB, 0.18472E+00_JPRB, 0.18446E+00_JPRB, 0.18462E+00_JPRB, 0.18420E+00_JPRB, 0.18528E+00_JPRB, 0.18455E+00_JPRB ,&
&  0.18458E+00_JPRB, 0.18365E+00_JPRB, 0.18272E+00_JPRB, 0.18032E+00_JPRB, 0.17907E+00_JPRB, 0.17312E+00_JPRB, 0.17157E+00_JPRB ,&
&  0.17031E+00_JPRB, 0.17116E+00_JPRB, 0.17141E+00_JPRB, 0.17190E+00_JPRB, 0.17282E+00_JPRB, 0.17444E+00_JPRB, 0.17448E+00_JPRB ,&
&  0.17342E+00_JPRB, 0.17139E+00_JPRB, 0.16902E+00_JPRB, 0.16686E+00_JPRB, 0.16722E+00_JPRB, 0.16690E+00_JPRB, 0.16597E+00_JPRB ,&
&  0.16498E+00_JPRB, 0.16555E+00_JPRB, 0.16605E+00_JPRB, 0.16739E+00_JPRB, 0.17232E+00_JPRB, 0.17082E+00_JPRB, 0.16961E+00_JPRB ,&
&  0.16943E+00_JPRB, 0.16932E+00_JPRB, 0.16983E+00_JPRB, 0.17020E+00_JPRB, 0.17208E+00_JPRB, 0.17154E+00_JPRB, 0.17404E+00_JPRB ,&
&  0.17145E+00_JPRB, 0.16980E+00_JPRB, 0.16757E+00_JPRB, 0.16474E+00_JPRB, 0.16115E+00_JPRB, 0.15887E+00_JPRB, 0.15670E+00_JPRB ,&
&  0.15433E+00_JPRB, 0.15233E+00_JPRB, 0.14976E+00_JPRB, 0.14883E+00_JPRB, 0.14751E+00_JPRB, 0.14632E+00_JPRB, 0.14524E+00_JPRB ,&
&  0.14441E+00_JPRB, 0.14327E+00_JPRB, 0.14227E+00_JPRB, 0.14161E+00_JPRB, 0.14141E+00_JPRB, 0.14154E+00_JPRB, 0.14254E+00_JPRB ,&
&  0.14446E+00_JPRB, 0.14603E+00_JPRB, 0.14726E+00_JPRB, 0.14828E+00_JPRB, 0.14689E+00_JPRB, 0.14439E+00_JPRB, 0.14187E+00_JPRB ,&
&  0.13857E+00_JPRB, 0.13674E+00_JPRB, 0.13503E+00_JPRB, 0.13401E+00_JPRB, 0.13375E+00_JPRB, 0.13371E+00_JPRB, 0.13398E+00_JPRB ,&
&  0.13427E+00_JPRB, 0.13471E+00_JPRB, 0.13502E+00_JPRB, 0.13543E+00_JPRB, 0.13596E+00_JPRB, 0.13648E+00_JPRB, 0.13726E+00_JPRB ,&
&  0.13850E+00_JPRB, 0.14010E+00_JPRB, 0.14232E+00_JPRB, 0.14492E+00_JPRB, 0.14795E+00_JPRB, 0.15191E+00_JPRB, 0.15773E+00_JPRB ,&
&  0.16382E+00_JPRB, 0.17406E+00_JPRB, 0.17510E+00_JPRB, 0.17996E+00_JPRB, 0.18281E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.18635E+00_JPRB, 0.18806E+00_JPRB, 0.18867E+00_JPRB, 0.18989E+00_JPRB, 0.19034E+00_JPRB, 0.19165E+00_JPRB, 0.19272E+00_JPRB ,&
&  0.19516E+00_JPRB, 0.19737E+00_JPRB, 0.19795E+00_JPRB, 0.19630E+00_JPRB, 0.19536E+00_JPRB, 0.19276E+00_JPRB, 0.19180E+00_JPRB ,&
&  0.19164E+00_JPRB, 0.19271E+00_JPRB, 0.19291E+00_JPRB, 0.19302E+00_JPRB, 0.19300E+00_JPRB, 0.19298E+00_JPRB, 0.19344E+00_JPRB ,&
&  0.19153E+00_JPRB, 0.19087E+00_JPRB, 0.19295E+00_JPRB, 0.19221E+00_JPRB, 0.19160E+00_JPRB, 0.19031E+00_JPRB, 0.18960E+00_JPRB ,&
&  0.18568E+00_JPRB, 0.18002E+00_JPRB, 0.17335E+00_JPRB, 0.17622E+00_JPRB, 0.18340E+00_JPRB, 0.18125E+00_JPRB, 0.18051E+00_JPRB ,&
&  0.17993E+00_JPRB, 0.17879E+00_JPRB, 0.17835E+00_JPRB, 0.17791E+00_JPRB, 0.17847E+00_JPRB, 0.17904E+00_JPRB, 0.18057E+00_JPRB ,&
&  0.18271E+00_JPRB, 0.18584E+00_JPRB, 0.18569E+00_JPRB, 0.17958E+00_JPRB, 0.17644E+00_JPRB, 0.17533E+00_JPRB, 0.17216E+00_JPRB ,&
&  0.16845E+00_JPRB, 0.16695E+00_JPRB, 0.16604E+00_JPRB, 0.16379E+00_JPRB, 0.16314E+00_JPRB, 0.16269E+00_JPRB, 0.16040E+00_JPRB ,&
&  0.15735E+00_JPRB, 0.15509E+00_JPRB, 0.15183E+00_JPRB, 0.14849E+00_JPRB, 0.14615E+00_JPRB, 0.14504E+00_JPRB, 0.14509E+00_JPRB ,&
&  0.14678E+00_JPRB, 0.14860E+00_JPRB, 0.15422E+00_JPRB, 0.15791E+00_JPRB, 0.15499E+00_JPRB, 0.15167E+00_JPRB, 0.14817E+00_JPRB ,&
&  0.14315E+00_JPRB, 0.14076E+00_JPRB, 0.13886E+00_JPRB, 0.13748E+00_JPRB, 0.13734E+00_JPRB, 0.13806E+00_JPRB, 0.13864E+00_JPRB ,&
&  0.13936E+00_JPRB, 0.14030E+00_JPRB, 0.14076E+00_JPRB, 0.14095E+00_JPRB, 0.14099E+00_JPRB, 0.14123E+00_JPRB, 0.14179E+00_JPRB ,&
&  0.14253E+00_JPRB, 0.14427E+00_JPRB, 0.14677E+00_JPRB, 0.14946E+00_JPRB, 0.15133E+00_JPRB, 0.15279E+00_JPRB, 0.15586E+00_JPRB ,&
&  0.15945E+00_JPRB, 0.16778E+00_JPRB, 0.17960E+00_JPRB, 0.18859E+00_JPRB, 0.18524E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.19992E+00_JPRB, 0.20092E+00_JPRB, 0.20205E+00_JPRB, 0.20336E+00_JPRB, 0.20487E+00_JPRB, 0.20652E+00_JPRB, 0.20792E+00_JPRB ,&
&  0.20945E+00_JPRB, 0.21043E+00_JPRB, 0.21068E+00_JPRB, 0.20966E+00_JPRB, 0.20888E+00_JPRB, 0.20912E+00_JPRB, 0.21017E+00_JPRB ,&
&  0.20843E+00_JPRB, 0.20566E+00_JPRB, 0.20454E+00_JPRB, 0.20443E+00_JPRB, 0.20388E+00_JPRB, 0.20358E+00_JPRB, 0.20430E+00_JPRB ,&
&  0.20521E+00_JPRB, 0.20492E+00_JPRB, 0.20462E+00_JPRB, 0.20362E+00_JPRB, 0.20279E+00_JPRB, 0.20214E+00_JPRB, 0.20119E+00_JPRB ,&
&  0.19810E+00_JPRB, 0.18825E+00_JPRB, 0.18171E+00_JPRB, 0.18486E+00_JPRB, 0.19191E+00_JPRB, 0.19130E+00_JPRB, 0.19129E+00_JPRB ,&
&  0.19079E+00_JPRB, 0.18949E+00_JPRB, 0.18804E+00_JPRB, 0.18666E+00_JPRB, 0.18609E+00_JPRB, 0.18634E+00_JPRB, 0.18711E+00_JPRB ,&
&  0.18793E+00_JPRB, 0.18987E+00_JPRB, 0.19214E+00_JPRB, 0.19320E+00_JPRB, 0.19425E+00_JPRB, 0.19474E+00_JPRB, 0.19492E+00_JPRB ,&
&  0.19461E+00_JPRB, 0.19370E+00_JPRB, 0.19311E+00_JPRB, 0.19250E+00_JPRB, 0.19189E+00_JPRB, 0.19086E+00_JPRB, 0.18741E+00_JPRB ,&
&  0.18210E+00_JPRB, 0.17700E+00_JPRB, 0.17080E+00_JPRB, 0.16460E+00_JPRB, 0.15895E+00_JPRB, 0.15485E+00_JPRB, 0.15153E+00_JPRB ,&
&  0.15100E+00_JPRB, 0.15454E+00_JPRB, 0.16202E+00_JPRB, 0.17130E+00_JPRB, 0.17743E+00_JPRB, 0.16735E+00_JPRB, 0.16122E+00_JPRB ,&
&  0.15556E+00_JPRB, 0.15111E+00_JPRB, 0.14836E+00_JPRB, 0.14690E+00_JPRB, 0.14593E+00_JPRB, 0.14595E+00_JPRB, 0.14655E+00_JPRB ,&
&  0.14779E+00_JPRB, 0.14916E+00_JPRB, 0.15044E+00_JPRB, 0.15122E+00_JPRB, 0.15150E+00_JPRB, 0.15133E+00_JPRB, 0.15094E+00_JPRB ,&
&  0.15134E+00_JPRB, 0.15350E+00_JPRB, 0.15747E+00_JPRB, 0.16220E+00_JPRB, 0.16619E+00_JPRB, 0.16993E+00_JPRB, 0.17515E+00_JPRB ,&
&  0.18174E+00_JPRB, 0.18723E+00_JPRB, 0.19152E+00_JPRB, 0.19601E+00_JPRB, 0.19841E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.21839E+00_JPRB, 0.21954E+00_JPRB, 0.22063E+00_JPRB, 0.22172E+00_JPRB, 0.22358E+00_JPRB, 0.22502E+00_JPRB, 0.22669E+00_JPRB ,&
&  0.22789E+00_JPRB, 0.22659E+00_JPRB, 0.22483E+00_JPRB, 0.22342E+00_JPRB, 0.22284E+00_JPRB, 0.22221E+00_JPRB, 0.22178E+00_JPRB ,&
&  0.22258E+00_JPRB, 0.22195E+00_JPRB, 0.21939E+00_JPRB, 0.21758E+00_JPRB, 0.21455E+00_JPRB, 0.21253E+00_JPRB, 0.21212E+00_JPRB ,&
&  0.21368E+00_JPRB, 0.21349E+00_JPRB, 0.21302E+00_JPRB, 0.21215E+00_JPRB, 0.21093E+00_JPRB, 0.21034E+00_JPRB, 0.20928E+00_JPRB ,&
&  0.20872E+00_JPRB, 0.20784E+00_JPRB, 0.20314E+00_JPRB, 0.20376E+00_JPRB, 0.20336E+00_JPRB, 0.20326E+00_JPRB, 0.20271E+00_JPRB ,&
&  0.20208E+00_JPRB, 0.20108E+00_JPRB, 0.19930E+00_JPRB, 0.19737E+00_JPRB, 0.19575E+00_JPRB, 0.19486E+00_JPRB, 0.19475E+00_JPRB ,&
&  0.19505E+00_JPRB, 0.19589E+00_JPRB, 0.19736E+00_JPRB, 0.19933E+00_JPRB, 0.20018E+00_JPRB, 0.20079E+00_JPRB, 0.20172E+00_JPRB ,&
&  0.20213E+00_JPRB, 0.20223E+00_JPRB, 0.20228E+00_JPRB, 0.20231E+00_JPRB, 0.20213E+00_JPRB, 0.20141E+00_JPRB, 0.20081E+00_JPRB ,&
&  0.19931E+00_JPRB, 0.19704E+00_JPRB, 0.19553E+00_JPRB, 0.19364E+00_JPRB, 0.18842E+00_JPRB, 0.17824E+00_JPRB, 0.17253E+00_JPRB ,&
&  0.17258E+00_JPRB, 0.17715E+00_JPRB, 0.18295E+00_JPRB, 0.18606E+00_JPRB, 0.19091E+00_JPRB, 0.19111E+00_JPRB, 0.18769E+00_JPRB ,&
&  0.17990E+00_JPRB, 0.17430E+00_JPRB, 0.17028E+00_JPRB, 0.16620E+00_JPRB, 0.16369E+00_JPRB, 0.16312E+00_JPRB, 0.16361E+00_JPRB ,&
&  0.16554E+00_JPRB, 0.16838E+00_JPRB, 0.17114E+00_JPRB, 0.17264E+00_JPRB, 0.17274E+00_JPRB, 0.17138E+00_JPRB, 0.16981E+00_JPRB ,&
&  0.16934E+00_JPRB, 0.17312E+00_JPRB, 0.18112E+00_JPRB, 0.19030E+00_JPRB, 0.19434E+00_JPRB, 0.19585E+00_JPRB, 0.19740E+00_JPRB ,&
&  0.20073E+00_JPRB, 0.20536E+00_JPRB, 0.21008E+00_JPRB, 0.21343E+00_JPRB, 0.21621E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.23734E+00_JPRB, 0.23834E+00_JPRB, 0.23940E+00_JPRB, 0.24075E+00_JPRB, 0.24251E+00_JPRB, 0.24354E+00_JPRB, 0.24432E+00_JPRB ,&
&  0.24368E+00_JPRB, 0.24185E+00_JPRB, 0.23969E+00_JPRB, 0.23742E+00_JPRB, 0.23639E+00_JPRB, 0.23627E+00_JPRB, 0.23639E+00_JPRB ,&
&  0.23592E+00_JPRB, 0.23574E+00_JPRB, 0.23402E+00_JPRB, 0.23112E+00_JPRB, 0.22776E+00_JPRB, 0.22551E+00_JPRB, 0.22488E+00_JPRB ,&
&  0.22436E+00_JPRB, 0.22357E+00_JPRB, 0.22327E+00_JPRB, 0.22296E+00_JPRB, 0.22105E+00_JPRB, 0.22006E+00_JPRB, 0.22046E+00_JPRB ,&
&  0.22129E+00_JPRB, 0.22271E+00_JPRB, 0.22298E+00_JPRB, 0.22041E+00_JPRB, 0.22266E+00_JPRB, 0.22241E+00_JPRB, 0.22059E+00_JPRB ,&
&  0.21824E+00_JPRB, 0.21580E+00_JPRB, 0.21359E+00_JPRB, 0.21142E+00_JPRB, 0.20925E+00_JPRB, 0.20761E+00_JPRB, 0.20657E+00_JPRB ,&
&  0.20629E+00_JPRB, 0.20643E+00_JPRB, 0.20680E+00_JPRB, 0.20762E+00_JPRB, 0.20824E+00_JPRB, 0.20858E+00_JPRB, 0.20860E+00_JPRB ,&
&  0.20870E+00_JPRB, 0.20890E+00_JPRB, 0.20928E+00_JPRB, 0.20954E+00_JPRB, 0.20983E+00_JPRB, 0.21011E+00_JPRB, 0.21004E+00_JPRB ,&
&  0.20904E+00_JPRB, 0.20734E+00_JPRB, 0.20507E+00_JPRB, 0.20429E+00_JPRB, 0.20428E+00_JPRB, 0.20290E+00_JPRB, 0.20060E+00_JPRB ,&
&  0.19836E+00_JPRB, 0.19781E+00_JPRB, 0.19883E+00_JPRB, 0.20003E+00_JPRB, 0.20099E+00_JPRB, 0.20134E+00_JPRB, 0.20109E+00_JPRB ,&
&  0.20015E+00_JPRB, 0.19803E+00_JPRB, 0.19535E+00_JPRB, 0.19323E+00_JPRB, 0.19207E+00_JPRB, 0.19199E+00_JPRB, 0.19287E+00_JPRB ,&
&  0.19300E+00_JPRB, 0.19298E+00_JPRB, 0.19395E+00_JPRB, 0.19474E+00_JPRB, 0.19527E+00_JPRB, 0.19599E+00_JPRB, 0.19737E+00_JPRB ,&
&  0.19931E+00_JPRB, 0.20240E+00_JPRB, 0.20619E+00_JPRB, 0.21023E+00_JPRB, 0.21302E+00_JPRB, 0.21221E+00_JPRB, 0.21274E+00_JPRB ,&
&  0.21721E+00_JPRB, 0.22273E+00_JPRB, 0.22741E+00_JPRB, 0.23121E+00_JPRB, 0.23470E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.25984E+00_JPRB, 0.26172E+00_JPRB, 0.26365E+00_JPRB, 0.26575E+00_JPRB, 0.26727E+00_JPRB, 0.26783E+00_JPRB, 0.26648E+00_JPRB ,&
&  0.26374E+00_JPRB, 0.26060E+00_JPRB, 0.25744E+00_JPRB, 0.25469E+00_JPRB, 0.25298E+00_JPRB, 0.25194E+00_JPRB, 0.25130E+00_JPRB ,&
&  0.25077E+00_JPRB, 0.25023E+00_JPRB, 0.24827E+00_JPRB, 0.24546E+00_JPRB, 0.24182E+00_JPRB, 0.23874E+00_JPRB, 0.23792E+00_JPRB ,&
&  0.23643E+00_JPRB, 0.23546E+00_JPRB, 0.23481E+00_JPRB, 0.23409E+00_JPRB, 0.23387E+00_JPRB, 0.23490E+00_JPRB, 0.23672E+00_JPRB ,&
&  0.23867E+00_JPRB, 0.24020E+00_JPRB, 0.24139E+00_JPRB, 0.24224E+00_JPRB, 0.24170E+00_JPRB, 0.24061E+00_JPRB, 0.23934E+00_JPRB ,&
&  0.23783E+00_JPRB, 0.23643E+00_JPRB, 0.23523E+00_JPRB, 0.23358E+00_JPRB, 0.23108E+00_JPRB, 0.22805E+00_JPRB, 0.22592E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.22268E+00_JPRB, 0.22100E+00_JPRB, 0.22010E+00_JPRB, 0.21947E+00_JPRB, 0.21921E+00_JPRB, 0.21929E+00_JPRB ,&
&  0.21971E+00_JPRB, 0.22093E+00_JPRB, 0.22260E+00_JPRB, 0.22441E+00_JPRB, 0.22489E+00_JPRB, 0.22502E+00_JPRB, 0.22523E+00_JPRB ,&
&  0.22456E+00_JPRB, 0.22327E+00_JPRB, 0.22155E+00_JPRB, 0.21970E+00_JPRB, 0.21853E+00_JPRB, 0.21761E+00_JPRB, 0.21657E+00_JPRB ,&
&  0.21473E+00_JPRB, 0.21360E+00_JPRB, 0.21296E+00_JPRB, 0.21284E+00_JPRB, 0.21307E+00_JPRB, 0.21314E+00_JPRB, 0.21248E+00_JPRB ,&
&  0.21165E+00_JPRB, 0.21090E+00_JPRB, 0.20985E+00_JPRB, 0.20871E+00_JPRB, 0.20692E+00_JPRB, 0.20717E+00_JPRB, 0.20806E+00_JPRB ,&
&  0.20849E+00_JPRB, 0.20869E+00_JPRB, 0.20886E+00_JPRB, 0.20988E+00_JPRB, 0.21099E+00_JPRB, 0.21258E+00_JPRB, 0.21494E+00_JPRB ,&
&  0.21690E+00_JPRB, 0.21949E+00_JPRB, 0.22338E+00_JPRB, 0.22696E+00_JPRB, 0.22955E+00_JPRB, 0.23085E+00_JPRB, 0.23348E+00_JPRB ,&
&  0.23936E+00_JPRB, 0.24598E+00_JPRB, 0.25048E+00_JPRB, 0.25471E+00_JPRB, 0.25828E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.28320E+00_JPRB, 0.28848E+00_JPRB, 0.29266E+00_JPRB, 0.29508E+00_JPRB, 0.29526E+00_JPRB, 0.29399E+00_JPRB, 0.29116E+00_JPRB ,&
&  0.28709E+00_JPRB, 0.28294E+00_JPRB, 0.27873E+00_JPRB, 0.27525E+00_JPRB, 0.27300E+00_JPRB, 0.27115E+00_JPRB, 0.26965E+00_JPRB ,&
&  0.26769E+00_JPRB, 0.26537E+00_JPRB, 0.26252E+00_JPRB, 0.26001E+00_JPRB, 0.25713E+00_JPRB, 0.25314E+00_JPRB, 0.25027E+00_JPRB ,&
&  0.24831E+00_JPRB, 0.24643E+00_JPRB, 0.24592E+00_JPRB, 0.24720E+00_JPRB, 0.24959E+00_JPRB, 0.25189E+00_JPRB, 0.25443E+00_JPRB ,&
&  0.25620E+00_JPRB, 0.25706E+00_JPRB, 0.25772E+00_JPRB, 0.25757E+00_JPRB, 0.25756E+00_JPRB, 0.25728E+00_JPRB, 0.25665E+00_JPRB ,&
&  0.25607E+00_JPRB, 0.25567E+00_JPRB, 0.25473E+00_JPRB, 0.25474E+00_JPRB, 0.25468E+00_JPRB, 0.25351E+00_JPRB, 0.25135E+00_JPRB ,&
&  0.24789E+00_JPRB, 0.24316E+00_JPRB, 0.23938E+00_JPRB, 0.23744E+00_JPRB, 0.23678E+00_JPRB, 0.23701E+00_JPRB, 0.23783E+00_JPRB ,&
&  0.23894E+00_JPRB, 0.24051E+00_JPRB, 0.24194E+00_JPRB, 0.24259E+00_JPRB, 0.24234E+00_JPRB, 0.24210E+00_JPRB, 0.24082E+00_JPRB ,&
&  0.23957E+00_JPRB, 0.23877E+00_JPRB, 0.23818E+00_JPRB, 0.23753E+00_JPRB, 0.23544E+00_JPRB, 0.23288E+00_JPRB, 0.23233E+00_JPRB ,&
&  0.23092E+00_JPRB, 0.23003E+00_JPRB, 0.22877E+00_JPRB, 0.22821E+00_JPRB, 0.22750E+00_JPRB, 0.22635E+00_JPRB, 0.22508E+00_JPRB ,&
&  0.22400E+00_JPRB, 0.22340E+00_JPRB, 0.22298E+00_JPRB, 0.22201E+00_JPRB, 0.22220E+00_JPRB, 0.22266E+00_JPRB, 0.22394E+00_JPRB ,&
&  0.22512E+00_JPRB, 0.22631E+00_JPRB, 0.22785E+00_JPRB, 0.22925E+00_JPRB, 0.23143E+00_JPRB, 0.23402E+00_JPRB, 0.23700E+00_JPRB ,&
&  0.23953E+00_JPRB, 0.24309E+00_JPRB, 0.24725E+00_JPRB, 0.25114E+00_JPRB, 0.25422E+00_JPRB, 0.25665E+00_JPRB, 0.26025E+00_JPRB ,&
&  0.26611E+00_JPRB, 0.27120E+00_JPRB, 0.27409E+00_JPRB, 0.27681E+00_JPRB, 0.27908E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.31581E+00_JPRB, 0.32101E+00_JPRB, 0.32392E+00_JPRB, 0.32520E+00_JPRB, 0.32550E+00_JPRB, 0.32378E+00_JPRB, 0.32087E+00_JPRB ,&
&  0.31644E+00_JPRB, 0.31204E+00_JPRB, 0.30722E+00_JPRB, 0.30181E+00_JPRB, 0.29699E+00_JPRB, 0.29298E+00_JPRB, 0.28953E+00_JPRB ,&
&  0.28617E+00_JPRB, 0.28309E+00_JPRB, 0.28056E+00_JPRB, 0.27794E+00_JPRB, 0.27503E+00_JPRB, 0.27147E+00_JPRB, 0.26682E+00_JPRB ,&
&  0.26407E+00_JPRB, 0.26196E+00_JPRB, 0.26296E+00_JPRB, 0.26582E+00_JPRB, 0.26802E+00_JPRB, 0.26934E+00_JPRB, 0.27034E+00_JPRB ,&
&  0.27174E+00_JPRB, 0.27314E+00_JPRB, 0.27441E+00_JPRB, 0.27704E+00_JPRB, 0.27949E+00_JPRB, 0.27945E+00_JPRB, 0.27817E+00_JPRB ,&
&  0.27737E+00_JPRB, 0.27836E+00_JPRB, 0.27957E+00_JPRB, 0.28006E+00_JPRB, 0.27930E+00_JPRB, 0.27691E+00_JPRB, 0.27312E+00_JPRB ,&
&  0.26665E+00_JPRB, 0.26185E+00_JPRB, 0.26073E+00_JPRB, 0.26177E+00_JPRB, 0.26356E+00_JPRB, 0.26490E+00_JPRB, 0.26568E+00_JPRB ,&
&  0.26586E+00_JPRB, 0.26516E+00_JPRB, 0.26393E+00_JPRB, 0.26249E+00_JPRB, 0.26061E+00_JPRB, 0.25846E+00_JPRB, 0.25716E+00_JPRB ,&
&  0.25577E+00_JPRB, 0.25457E+00_JPRB, 0.25373E+00_JPRB, 0.25318E+00_JPRB, 0.25201E+00_JPRB, 0.25039E+00_JPRB, 0.24946E+00_JPRB ,&
&  0.24821E+00_JPRB, 0.24752E+00_JPRB, 0.24652E+00_JPRB, 0.24553E+00_JPRB, 0.24424E+00_JPRB, 0.24300E+00_JPRB, 0.24268E+00_JPRB ,&
&  0.24280E+00_JPRB, 0.24165E+00_JPRB, 0.24020E+00_JPRB, 0.23996E+00_JPRB, 0.24033E+00_JPRB, 0.24102E+00_JPRB, 0.24161E+00_JPRB ,&
&  0.24222E+00_JPRB, 0.24303E+00_JPRB, 0.24484E+00_JPRB, 0.24688E+00_JPRB, 0.24917E+00_JPRB, 0.25200E+00_JPRB, 0.25528E+00_JPRB ,&
&  0.25934E+00_JPRB, 0.26423E+00_JPRB, 0.26981E+00_JPRB, 0.27482E+00_JPRB, 0.27918E+00_JPRB, 0.28283E+00_JPRB, 0.28840E+00_JPRB ,&
&  0.29516E+00_JPRB, 0.30034E+00_JPRB, 0.30407E+00_JPRB, 0.30736E+00_JPRB, 0.31084E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.34928E+00_JPRB, 0.35095E+00_JPRB, 0.35095E+00_JPRB, 0.35133E+00_JPRB, 0.35080E+00_JPRB, 0.34876E+00_JPRB, 0.34509E+00_JPRB ,&
&  0.34127E+00_JPRB, 0.33693E+00_JPRB, 0.33237E+00_JPRB, 0.32649E+00_JPRB, 0.32043E+00_JPRB, 0.31560E+00_JPRB, 0.31045E+00_JPRB ,&
&  0.30529E+00_JPRB, 0.30058E+00_JPRB, 0.29737E+00_JPRB, 0.29474E+00_JPRB, 0.29223E+00_JPRB, 0.28989E+00_JPRB, 0.28684E+00_JPRB ,&
&  0.28423E+00_JPRB, 0.28184E+00_JPRB, 0.28226E+00_JPRB, 0.28467E+00_JPRB, 0.28661E+00_JPRB, 0.28798E+00_JPRB, 0.28910E+00_JPRB ,&
&  0.28938E+00_JPRB, 0.29561E+00_JPRB, 0.28997E+00_JPRB, 0.29966E+00_JPRB, 0.30307E+00_JPRB, 0.30466E+00_JPRB, 0.30559E+00_JPRB ,&
&  0.30832E+00_JPRB, 0.31127E+00_JPRB, 0.31286E+00_JPRB, 0.30955E+00_JPRB, 0.30491E+00_JPRB, 0.30132E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.29218E+00_JPRB, 0.29014E+00_JPRB, 0.29018E+00_JPRB, 0.29049E+00_JPRB, 0.29055E+00_JPRB, 0.28999E+00_JPRB, 0.28874E+00_JPRB ,&
&  0.28700E+00_JPRB, 0.28460E+00_JPRB, 0.28187E+00_JPRB, 0.27911E+00_JPRB, 0.27732E+00_JPRB, 0.27524E+00_JPRB, 0.27334E+00_JPRB ,&
&  0.27255E+00_JPRB, 0.27292E+00_JPRB, 0.27317E+00_JPRB, 0.27308E+00_JPRB, 0.27234E+00_JPRB, 0.27181E+00_JPRB, 0.27162E+00_JPRB ,&
&  0.27088E+00_JPRB, 0.26993E+00_JPRB, 0.26876E+00_JPRB, 0.26720E+00_JPRB, 0.26555E+00_JPRB, 0.26470E+00_JPRB, 0.26468E+00_JPRB ,&
&  0.26438E+00_JPRB, 0.26291E+00_JPRB, 0.26063E+00_JPRB, 0.26000E+00_JPRB, 0.25973E+00_JPRB, 0.25984E+00_JPRB, 0.26021E+00_JPRB ,&
&  0.26124E+00_JPRB, 0.26318E+00_JPRB, 0.26529E+00_JPRB, 0.26726E+00_JPRB, 0.26921E+00_JPRB, 0.27220E+00_JPRB, 0.27597E+00_JPRB ,&
&  0.28038E+00_JPRB, 0.28520E+00_JPRB, 0.28988E+00_JPRB, 0.29428E+00_JPRB, 0.29883E+00_JPRB, 0.30575E+00_JPRB, 0.31595E+00_JPRB ,&
&  0.32520E+00_JPRB, 0.33190E+00_JPRB, 0.33620E+00_JPRB, 0.34059E+00_JPRB, 0.34536E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.38851E+00_JPRB, 0.38643E+00_JPRB, 0.38284E+00_JPRB, 0.38018E+00_JPRB, 0.37688E+00_JPRB, 0.37271E+00_JPRB, 0.36846E+00_JPRB ,&
&  0.36510E+00_JPRB, 0.36166E+00_JPRB, 0.35783E+00_JPRB, 0.35257E+00_JPRB, 0.34564E+00_JPRB, 0.33910E+00_JPRB, 0.33179E+00_JPRB ,&
&  0.32511E+00_JPRB, 0.31942E+00_JPRB, 0.31559E+00_JPRB, 0.31342E+00_JPRB, 0.31158E+00_JPRB, 0.31007E+00_JPRB, 0.30835E+00_JPRB ,&
&  0.30746E+00_JPRB, 0.30732E+00_JPRB, 0.30730E+00_JPRB, 0.30758E+00_JPRB, 0.30816E+00_JPRB, 0.31068E+00_JPRB, 0.31227E+00_JPRB ,&
&  0.31431E+00_JPRB, 0.31170E+00_JPRB, 0.31926E+00_JPRB, 0.32659E+00_JPRB, 0.33599E+00_JPRB, 0.34283E+00_JPRB, 0.34830E+00_JPRB ,&
&  0.35352E+00_JPRB, 0.35594E+00_JPRB, 0.35178E+00_JPRB, 0.34552E+00_JPRB, 0.34072E+00_JPRB, 0.33778E+00_JPRB, 0.33249E+00_JPRB ,&
&  0.32726E+00_JPRB, 0.32348E+00_JPRB, 0.32048E+00_JPRB, 0.31834E+00_JPRB, 0.31607E+00_JPRB, 0.31271E+00_JPRB, 0.30896E+00_JPRB ,&
&  0.30434E+00_JPRB, 0.30094E+00_JPRB, 0.29825E+00_JPRB, 0.29720E+00_JPRB, 0.29736E+00_JPRB, 0.29851E+00_JPRB, 0.30031E+00_JPRB ,&
&  0.30307E+00_JPRB, 0.30550E+00_JPRB, 0.30647E+00_JPRB, 0.30668E+00_JPRB, 0.30657E+00_JPRB, 0.30674E+00_JPRB, 0.30631E+00_JPRB ,&
&  0.30497E+00_JPRB, 0.30311E+00_JPRB, 0.30099E+00_JPRB, 0.29854E+00_JPRB, 0.29594E+00_JPRB, 0.29399E+00_JPRB, 0.29230E+00_JPRB ,&
&  0.29048E+00_JPRB, 0.28830E+00_JPRB, 0.28595E+00_JPRB, 0.28463E+00_JPRB, 0.28371E+00_JPRB, 0.28323E+00_JPRB, 0.28399E+00_JPRB ,&
&  0.28598E+00_JPRB, 0.28833E+00_JPRB, 0.29080E+00_JPRB, 0.29290E+00_JPRB, 0.29488E+00_JPRB, 0.29677E+00_JPRB, 0.29949E+00_JPRB ,&
&  0.30426E+00_JPRB, 0.30955E+00_JPRB, 0.31489E+00_JPRB, 0.32050E+00_JPRB, 0.32933E+00_JPRB, 0.34319E+00_JPRB, 0.35892E+00_JPRB ,&
&  0.36954E+00_JPRB, 0.37478E+00_JPRB, 0.37853E+00_JPRB, 0.38269E+00_JPRB, 0.38721E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.42075E+00_JPRB, 0.41726E+00_JPRB, 0.41336E+00_JPRB, 0.40879E+00_JPRB, 0.40316E+00_JPRB, 0.39610E+00_JPRB, 0.39091E+00_JPRB ,&
&  0.38726E+00_JPRB, 0.38273E+00_JPRB, 0.37847E+00_JPRB, 0.37317E+00_JPRB, 0.36691E+00_JPRB, 0.35983E+00_JPRB, 0.35123E+00_JPRB ,&
&  0.34377E+00_JPRB, 0.33708E+00_JPRB, 0.33275E+00_JPRB, 0.33063E+00_JPRB, 0.33049E+00_JPRB, 0.33057E+00_JPRB, 0.32910E+00_JPRB ,&
&  0.32830E+00_JPRB, 0.32820E+00_JPRB, 0.32765E+00_JPRB, 0.32747E+00_JPRB, 0.32786E+00_JPRB, 0.33086E+00_JPRB, 0.33435E+00_JPRB ,&
&  0.33921E+00_JPRB, 0.31697E+00_JPRB, 0.32973E+00_JPRB, 0.33671E+00_JPRB, 0.34826E+00_JPRB, 0.36597E+00_JPRB, 0.38802E+00_JPRB ,&
&  0.40470E+00_JPRB, 0.40497E+00_JPRB, 0.39770E+00_JPRB, 0.38761E+00_JPRB, 0.38016E+00_JPRB, 0.37323E+00_JPRB, 0.36605E+00_JPRB ,&
&  0.35925E+00_JPRB, 0.35476E+00_JPRB, 0.35097E+00_JPRB, 0.34720E+00_JPRB, 0.34183E+00_JPRB, 0.33495E+00_JPRB, 0.32828E+00_JPRB ,&
&  0.32353E+00_JPRB, 0.32169E+00_JPRB, 0.32221E+00_JPRB, 0.32393E+00_JPRB, 0.32620E+00_JPRB, 0.33015E+00_JPRB, 0.33506E+00_JPRB ,&
&  0.33981E+00_JPRB, 0.34173E+00_JPRB, 0.34292E+00_JPRB, 0.34532E+00_JPRB, 0.34764E+00_JPRB, 0.34861E+00_JPRB, 0.34799E+00_JPRB ,&
&  0.34620E+00_JPRB, 0.34280E+00_JPRB, 0.33864E+00_JPRB, 0.33372E+00_JPRB, 0.32875E+00_JPRB, 0.32369E+00_JPRB, 0.31916E+00_JPRB ,&
&  0.31537E+00_JPRB, 0.31164E+00_JPRB, 0.30800E+00_JPRB, 0.30530E+00_JPRB, 0.30453E+00_JPRB, 0.30512E+00_JPRB, 0.30666E+00_JPRB ,&
&  0.30954E+00_JPRB, 0.31258E+00_JPRB, 0.31467E+00_JPRB, 0.31690E+00_JPRB, 0.31919E+00_JPRB, 0.32160E+00_JPRB, 0.32539E+00_JPRB ,&
&  0.33100E+00_JPRB, 0.33571E+00_JPRB, 0.34041E+00_JPRB, 0.34926E+00_JPRB, 0.36531E+00_JPRB, 0.38540E+00_JPRB, 0.40256E+00_JPRB ,&
&  0.41166E+00_JPRB, 0.41270E+00_JPRB, 0.41478E+00_JPRB, 0.41649E+00_JPRB, 0.42135E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.47275E+00_JPRB, 0.46953E+00_JPRB, 0.46211E+00_JPRB, 0.44888E+00_JPRB, 0.43490E+00_JPRB, 0.42311E+00_JPRB, 0.41608E+00_JPRB ,&
&  0.41127E+00_JPRB, 0.40419E+00_JPRB, 0.39800E+00_JPRB, 0.39332E+00_JPRB, 0.38754E+00_JPRB, 0.38121E+00_JPRB, 0.37330E+00_JPRB ,&
&  0.36714E+00_JPRB, 0.36246E+00_JPRB, 0.35900E+00_JPRB, 0.35678E+00_JPRB, 0.35526E+00_JPRB, 0.35379E+00_JPRB, 0.35164E+00_JPRB ,&
&  0.34928E+00_JPRB, 0.34729E+00_JPRB, 0.34554E+00_JPRB, 0.34539E+00_JPRB, 0.34606E+00_JPRB, 0.34963E+00_JPRB, 0.35543E+00_JPRB ,&
&  0.34317E+00_JPRB, 0.31949E+00_JPRB, 0.33514E+00_JPRB, 0.35440E+00_JPRB, 0.37413E+00_JPRB, 0.40463E+00_JPRB, 0.43908E+00_JPRB ,&
&  0.45713E+00_JPRB, 0.46623E+00_JPRB, 0.45749E+00_JPRB, 0.44503E+00_JPRB, 0.43134E+00_JPRB, 0.41787E+00_JPRB, 0.40726E+00_JPRB ,&
&  0.39757E+00_JPRB, 0.39105E+00_JPRB, 0.38546E+00_JPRB, 0.37731E+00_JPRB, 0.36727E+00_JPRB, 0.35903E+00_JPRB, 0.35415E+00_JPRB ,&
&  0.35241E+00_JPRB, 0.35299E+00_JPRB, 0.35399E+00_JPRB, 0.35538E+00_JPRB, 0.35806E+00_JPRB, 0.36348E+00_JPRB, 0.37033E+00_JPRB ,&
&  0.37615E+00_JPRB, 0.38381E+00_JPRB, 0.39393E+00_JPRB, 0.40342E+00_JPRB, 0.41182E+00_JPRB, 0.41583E+00_JPRB, 0.41635E+00_JPRB ,&
&  0.41217E+00_JPRB, 0.40374E+00_JPRB, 0.39424E+00_JPRB, 0.38508E+00_JPRB, 0.37553E+00_JPRB, 0.36517E+00_JPRB, 0.35475E+00_JPRB ,&
&  0.34613E+00_JPRB, 0.33891E+00_JPRB, 0.33364E+00_JPRB, 0.33162E+00_JPRB, 0.33168E+00_JPRB, 0.33397E+00_JPRB, 0.33692E+00_JPRB ,&
&  0.33980E+00_JPRB, 0.34269E+00_JPRB, 0.34473E+00_JPRB, 0.34835E+00_JPRB, 0.35251E+00_JPRB, 0.35774E+00_JPRB, 0.36354E+00_JPRB ,&
&  0.37024E+00_JPRB, 0.37703E+00_JPRB, 0.38483E+00_JPRB, 0.40563E+00_JPRB, 0.43033E+00_JPRB, 0.45174E+00_JPRB, 0.46420E+00_JPRB ,&
&  0.46620E+00_JPRB, 0.46519E+00_JPRB, 0.46749E+00_JPRB, 0.46763E+00_JPRB, 0.47410E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.54718E+00_JPRB, 0.54110E+00_JPRB, 0.52281E+00_JPRB, 0.50387E+00_JPRB, 0.48532E+00_JPRB, 0.46781E+00_JPRB, 0.45537E+00_JPRB ,&
&  0.44663E+00_JPRB, 0.43530E+00_JPRB, 0.42563E+00_JPRB, 0.41891E+00_JPRB, 0.41150E+00_JPRB, 0.40435E+00_JPRB, 0.39640E+00_JPRB ,&
&  0.39015E+00_JPRB, 0.38646E+00_JPRB, 0.38395E+00_JPRB, 0.38114E+00_JPRB, 0.37771E+00_JPRB, 0.37331E+00_JPRB, 0.36860E+00_JPRB ,&
&  0.36384E+00_JPRB, 0.36011E+00_JPRB, 0.35645E+00_JPRB, 0.35485E+00_JPRB, 0.35765E+00_JPRB, 0.36379E+00_JPRB, 0.36965E+00_JPRB ,&
&  0.39155E+00_JPRB, 0.34545E+00_JPRB, 0.36387E+00_JPRB, 0.39489E+00_JPRB, 0.42725E+00_JPRB, 0.46921E+00_JPRB, 0.50380E+00_JPRB ,&
&  0.51930E+00_JPRB, 0.53381E+00_JPRB, 0.52538E+00_JPRB, 0.51025E+00_JPRB, 0.48681E+00_JPRB, 0.46366E+00_JPRB, 0.44489E+00_JPRB ,&
&  0.43296E+00_JPRB, 0.42430E+00_JPRB, 0.41422E+00_JPRB, 0.40285E+00_JPRB, 0.39200E+00_JPRB, 0.38436E+00_JPRB, 0.38143E+00_JPRB ,&
&  0.38205E+00_JPRB, 0.38304E+00_JPRB, 0.38338E+00_JPRB, 0.38626E+00_JPRB, 0.39235E+00_JPRB, 0.40040E+00_JPRB, 0.41075E+00_JPRB ,&
&  0.42680E+00_JPRB, 0.44899E+00_JPRB, 0.46783E+00_JPRB, 0.48394E+00_JPRB, 0.49174E+00_JPRB, 0.49010E+00_JPRB, 0.47363E+00_JPRB ,&
&  0.45268E+00_JPRB, 0.43385E+00_JPRB, 0.42102E+00_JPRB, 0.41352E+00_JPRB, 0.40793E+00_JPRB, 0.40367E+00_JPRB, 0.39863E+00_JPRB ,&
&  0.39201E+00_JPRB, 0.38509E+00_JPRB, 0.37988E+00_JPRB, 0.37646E+00_JPRB, 0.37449E+00_JPRB, 0.37497E+00_JPRB, 0.37699E+00_JPRB ,&
&  0.37940E+00_JPRB, 0.38176E+00_JPRB, 0.38462E+00_JPRB, 0.38933E+00_JPRB, 0.39513E+00_JPRB, 0.40163E+00_JPRB, 0.40723E+00_JPRB ,&
&  0.41389E+00_JPRB, 0.42215E+00_JPRB, 0.43100E+00_JPRB, 0.46567E+00_JPRB, 0.49646E+00_JPRB, 0.51349E+00_JPRB, 0.52469E+00_JPRB ,&
&  0.53029E+00_JPRB, 0.54134E+00_JPRB, 0.46586E+00_JPRB, 0.51375E+00_JPRB, 0.53962E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.64210E+00_JPRB, 0.62449E+00_JPRB, 0.59331E+00_JPRB, 0.56053E+00_JPRB, 0.53243E+00_JPRB, 0.50735E+00_JPRB, 0.48820E+00_JPRB ,&
&  0.47621E+00_JPRB, 0.46707E+00_JPRB, 0.45790E+00_JPRB, 0.44935E+00_JPRB, 0.43973E+00_JPRB, 0.43093E+00_JPRB, 0.42131E+00_JPRB ,&
&  0.41467E+00_JPRB, 0.41042E+00_JPRB, 0.40678E+00_JPRB, 0.40221E+00_JPRB, 0.39713E+00_JPRB, 0.39166E+00_JPRB, 0.38489E+00_JPRB ,&
&  0.37903E+00_JPRB, 0.37590E+00_JPRB, 0.37094E+00_JPRB, 0.36960E+00_JPRB, 0.37594E+00_JPRB, 0.38412E+00_JPRB, 0.39325E+00_JPRB ,&
&  0.42014E+00_JPRB, 0.38864E+00_JPRB, 0.40804E+00_JPRB, 0.47101E+00_JPRB, 0.51486E+00_JPRB, 0.56561E+00_JPRB, 0.58929E+00_JPRB ,&
&  0.59444E+00_JPRB, 0.59852E+00_JPRB, 0.58315E+00_JPRB, 0.56211E+00_JPRB, 0.53028E+00_JPRB, 0.50304E+00_JPRB, 0.48224E+00_JPRB ,&
&  0.46436E+00_JPRB, 0.44882E+00_JPRB, 0.43627E+00_JPRB, 0.42589E+00_JPRB, 0.41676E+00_JPRB, 0.41274E+00_JPRB, 0.41391E+00_JPRB ,&
&  0.41682E+00_JPRB, 0.42033E+00_JPRB, 0.42518E+00_JPRB, 0.43306E+00_JPRB, 0.44167E+00_JPRB, 0.45234E+00_JPRB, 0.47936E+00_JPRB ,&
&  0.52306E+00_JPRB, 0.55352E+00_JPRB, 0.57638E+00_JPRB, 0.58387E+00_JPRB, 0.57599E+00_JPRB, 0.55628E+00_JPRB, 0.52548E+00_JPRB ,&
&  0.49376E+00_JPRB, 0.46936E+00_JPRB, 0.45333E+00_JPRB, 0.44318E+00_JPRB, 0.43715E+00_JPRB, 0.43195E+00_JPRB, 0.42765E+00_JPRB ,&
&  0.42364E+00_JPRB, 0.42099E+00_JPRB, 0.42158E+00_JPRB, 0.42550E+00_JPRB, 0.43110E+00_JPRB, 0.43596E+00_JPRB, 0.43850E+00_JPRB ,&
&  0.43930E+00_JPRB, 0.43951E+00_JPRB, 0.44197E+00_JPRB, 0.44594E+00_JPRB, 0.45047E+00_JPRB, 0.45341E+00_JPRB, 0.45812E+00_JPRB ,&
&  0.46485E+00_JPRB, 0.47484E+00_JPRB, 0.48835E+00_JPRB, 0.56181E+00_JPRB, 0.59336E+00_JPRB, 0.60464E+00_JPRB, 0.61129E+00_JPRB ,&
&  0.62450E+00_JPRB, 0.62853E+00_JPRB, 0.63560E+00_JPRB, 0.56272E+00_JPRB, 0.63621E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.73714E+00_JPRB, 0.68951E+00_JPRB, 0.62949E+00_JPRB, 0.58826E+00_JPRB, 0.55930E+00_JPRB, 0.52906E+00_JPRB, 0.50718E+00_JPRB ,&
&  0.49349E+00_JPRB, 0.48584E+00_JPRB, 0.47889E+00_JPRB, 0.46993E+00_JPRB, 0.46080E+00_JPRB, 0.45403E+00_JPRB, 0.44682E+00_JPRB ,&
&  0.43945E+00_JPRB, 0.43237E+00_JPRB, 0.42560E+00_JPRB, 0.42044E+00_JPRB, 0.41389E+00_JPRB, 0.40838E+00_JPRB, 0.40323E+00_JPRB ,&
&  0.39736E+00_JPRB, 0.39198E+00_JPRB, 0.38599E+00_JPRB, 0.38986E+00_JPRB, 0.39838E+00_JPRB, 0.40593E+00_JPRB, 0.41584E+00_JPRB ,&
&  0.44858E+00_JPRB, 0.45014E+00_JPRB, 0.51075E+00_JPRB, 0.57860E+00_JPRB, 0.61814E+00_JPRB, 0.64994E+00_JPRB, 0.65317E+00_JPRB ,&
&  0.63427E+00_JPRB, 0.64012E+00_JPRB, 0.62881E+00_JPRB, 0.60350E+00_JPRB, 0.58511E+00_JPRB, 0.54955E+00_JPRB, 0.51789E+00_JPRB ,&
&  0.49544E+00_JPRB, 0.48079E+00_JPRB, 0.46791E+00_JPRB, 0.45389E+00_JPRB, 0.44512E+00_JPRB, 0.44300E+00_JPRB, 0.44580E+00_JPRB ,&
&  0.45041E+00_JPRB, 0.45793E+00_JPRB, 0.46525E+00_JPRB, 0.47271E+00_JPRB, 0.48146E+00_JPRB, 0.51720E+00_JPRB, 0.59405E+00_JPRB ,&
&  0.65370E+00_JPRB, 0.68743E+00_JPRB, 0.70477E+00_JPRB, 0.71053E+00_JPRB, 0.69255E+00_JPRB, 0.66437E+00_JPRB, 0.63065E+00_JPRB ,&
&  0.59526E+00_JPRB, 0.55857E+00_JPRB, 0.52516E+00_JPRB, 0.49907E+00_JPRB, 0.47847E+00_JPRB, 0.46447E+00_JPRB, 0.45731E+00_JPRB ,&
&  0.45672E+00_JPRB, 0.46133E+00_JPRB, 0.46883E+00_JPRB, 0.47643E+00_JPRB, 0.48134E+00_JPRB, 0.48235E+00_JPRB, 0.48138E+00_JPRB ,&
&  0.47965E+00_JPRB, 0.47776E+00_JPRB, 0.47805E+00_JPRB, 0.48093E+00_JPRB, 0.48572E+00_JPRB, 0.49127E+00_JPRB, 0.49765E+00_JPRB ,&
&  0.50474E+00_JPRB, 0.51637E+00_JPRB, 0.53250E+00_JPRB, 0.59902E+00_JPRB, 0.62876E+00_JPRB, 0.66011E+00_JPRB, 0.68922E+00_JPRB ,&
&  0.72471E+00_JPRB, 0.74132E+00_JPRB, 0.72904E+00_JPRB, 0.70320E+00_JPRB, 0.71598E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.79851E+00_JPRB, 0.69187E+00_JPRB, 0.62473E+00_JPRB, 0.59209E+00_JPRB, 0.58075E+00_JPRB, 0.56205E+00_JPRB, 0.54163E+00_JPRB ,&
&  0.52267E+00_JPRB, 0.50937E+00_JPRB, 0.50130E+00_JPRB, 0.49129E+00_JPRB, 0.48043E+00_JPRB, 0.47098E+00_JPRB, 0.46103E+00_JPRB ,&
&  0.45069E+00_JPRB, 0.44421E+00_JPRB, 0.43905E+00_JPRB, 0.43530E+00_JPRB, 0.43228E+00_JPRB, 0.42653E+00_JPRB, 0.42027E+00_JPRB ,&
&  0.41554E+00_JPRB, 0.40966E+00_JPRB, 0.40862E+00_JPRB, 0.41497E+00_JPRB, 0.42065E+00_JPRB, 0.42462E+00_JPRB, 0.44008E+00_JPRB ,&
&  0.48646E+00_JPRB, 0.51465E+00_JPRB, 0.59238E+00_JPRB, 0.66693E+00_JPRB, 0.72071E+00_JPRB, 0.78141E+00_JPRB, 0.79956E+00_JPRB ,&
&  0.78236E+00_JPRB, 0.78618E+00_JPRB, 0.76862E+00_JPRB, 0.71602E+00_JPRB, 0.65902E+00_JPRB, 0.62110E+00_JPRB, 0.59376E+00_JPRB ,&
&  0.57047E+00_JPRB, 0.54485E+00_JPRB, 0.51899E+00_JPRB, 0.49868E+00_JPRB, 0.48706E+00_JPRB, 0.48325E+00_JPRB, 0.48324E+00_JPRB ,&
&  0.48603E+00_JPRB, 0.49182E+00_JPRB, 0.49613E+00_JPRB, 0.50507E+00_JPRB, 0.55068E+00_JPRB, 0.67203E+00_JPRB, 0.84510E+00_JPRB ,&
&  0.92621E+00_JPRB, 0.95072E+00_JPRB, 0.96802E+00_JPRB, 0.96610E+00_JPRB, 0.91015E+00_JPRB, 0.84942E+00_JPRB, 0.79015E+00_JPRB ,&
&  0.73392E+00_JPRB, 0.67487E+00_JPRB, 0.62505E+00_JPRB, 0.58567E+00_JPRB, 0.55495E+00_JPRB, 0.53518E+00_JPRB, 0.52597E+00_JPRB ,&
&  0.52287E+00_JPRB, 0.52185E+00_JPRB, 0.52104E+00_JPRB, 0.51758E+00_JPRB, 0.51149E+00_JPRB, 0.50811E+00_JPRB, 0.50623E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.50453E+00_JPRB, 0.50868E+00_JPRB, 0.51428E+00_JPRB, 0.52314E+00_JPRB, 0.52957E+00_JPRB, 0.53813E+00_JPRB ,&
&  0.54893E+00_JPRB, 0.57269E+00_JPRB, 0.58389E+00_JPRB, 0.67275E+00_JPRB, 0.76382E+00_JPRB, 0.79056E+00_JPRB, 0.82649E+00_JPRB ,&
&  0.86189E+00_JPRB, 0.84659E+00_JPRB, 0.79244E+00_JPRB, 0.77481E+00_JPRB, 0.88099E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.65304E+00_JPRB, 0.61793E+00_JPRB, 0.59860E+00_JPRB, 0.59422E+00_JPRB, 0.59434E+00_JPRB, 0.58359E+00_JPRB, 0.56156E+00_JPRB ,&
&  0.53698E+00_JPRB, 0.51638E+00_JPRB, 0.50156E+00_JPRB, 0.49058E+00_JPRB, 0.48231E+00_JPRB, 0.47456E+00_JPRB, 0.46617E+00_JPRB ,&
&  0.45801E+00_JPRB, 0.45372E+00_JPRB, 0.45067E+00_JPRB, 0.44617E+00_JPRB, 0.43680E+00_JPRB, 0.42935E+00_JPRB, 0.42685E+00_JPRB ,&
&  0.42542E+00_JPRB, 0.42612E+00_JPRB, 0.43133E+00_JPRB, 0.43735E+00_JPRB, 0.43982E+00_JPRB, 0.44322E+00_JPRB, 0.46020E+00_JPRB ,&
&  0.51234E+00_JPRB, 0.61540E+00_JPRB, 0.65421E+00_JPRB, 0.74913E+00_JPRB, 0.88318E+00_JPRB, 0.99440E+00_JPRB, 0.10744E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.10117E+01_JPRB, 0.91334E+00_JPRB, 0.81583E+00_JPRB, 0.73865E+00_JPRB, 0.69320E+00_JPRB, 0.65979E+00_JPRB ,&
&  0.62815E+00_JPRB, 0.59662E+00_JPRB, 0.56990E+00_JPRB, 0.55041E+00_JPRB, 0.53745E+00_JPRB, 0.53300E+00_JPRB, 0.53328E+00_JPRB ,&
&  0.53349E+00_JPRB, 0.53293E+00_JPRB, 0.53671E+00_JPRB, 0.57851E+00_JPRB, 0.77220E+00_JPRB, 0.10527E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.14528E+01_JPRB, 0.14351E+01_JPRB, 0.13470E+01_JPRB, 0.12081E+01_JPRB, 0.10710E+01_JPRB, 0.92872E+00_JPRB ,&
&  0.82600E+00_JPRB, 0.75249E+00_JPRB, 0.70017E+00_JPRB, 0.65683E+00_JPRB, 0.61880E+00_JPRB, 0.59047E+00_JPRB, 0.57195E+00_JPRB ,&
&  0.55924E+00_JPRB, 0.54846E+00_JPRB, 0.53822E+00_JPRB, 0.52824E+00_JPRB, 0.52014E+00_JPRB, 0.51522E+00_JPRB, 0.51671E+00_JPRB ,&
&  0.51843E+00_JPRB, 0.51877E+00_JPRB, 0.53443E+00_JPRB, 0.55137E+00_JPRB, 0.56891E+00_JPRB, 0.58054E+00_JPRB, 0.58918E+00_JPRB ,&
&  0.60103E+00_JPRB, 0.64637E+00_JPRB, 0.64272E+00_JPRB, 0.68622E+00_JPRB, 0.82982E+00_JPRB, 0.90332E+00_JPRB, 0.89961E+00_JPRB ,&
&  0.86381E+00_JPRB, 0.80856E+00_JPRB, 0.75879E+00_JPRB, 0.72766E+00_JPRB, 0.70622E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.64569E+00_JPRB, 0.62187E+00_JPRB, 0.61188E+00_JPRB, 0.60403E+00_JPRB, 0.59623E+00_JPRB, 0.58226E+00_JPRB, 0.56021E+00_JPRB ,&
&  0.54308E+00_JPRB, 0.52885E+00_JPRB, 0.51259E+00_JPRB, 0.49866E+00_JPRB, 0.48892E+00_JPRB, 0.48131E+00_JPRB, 0.47261E+00_JPRB ,&
&  0.46656E+00_JPRB, 0.46203E+00_JPRB, 0.45094E+00_JPRB, 0.44027E+00_JPRB, 0.43564E+00_JPRB, 0.43484E+00_JPRB, 0.43850E+00_JPRB ,&
&  0.44258E+00_JPRB, 0.44767E+00_JPRB, 0.45136E+00_JPRB, 0.45432E+00_JPRB, 0.45428E+00_JPRB, 0.46003E+00_JPRB, 0.48526E+00_JPRB ,&
&  0.55417E+00_JPRB, 0.67306E+00_JPRB, 0.83487E+00_JPRB, 0.89228E+00_JPRB, 0.98613E+00_JPRB, 0.11101E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.12355E+01_JPRB, 0.11262E+01_JPRB, 0.98736E+00_JPRB, 0.89325E+00_JPRB, 0.82262E+00_JPRB, 0.77132E+00_JPRB, 0.71819E+00_JPRB ,&
&  0.67446E+00_JPRB, 0.64581E+00_JPRB, 0.61818E+00_JPRB, 0.59365E+00_JPRB, 0.58386E+00_JPRB, 0.58131E+00_JPRB, 0.57905E+00_JPRB ,&
&  0.57595E+00_JPRB, 0.57427E+00_JPRB, 0.60112E+00_JPRB, 0.73242E+00_JPRB, 0.10787E+01_JPRB, 0.13788E+01_JPRB, 0.17673E+01_JPRB ,&
&  0.21112E+01_JPRB, 0.21032E+01_JPRB, 0.19302E+01_JPRB, 0.16504E+01_JPRB, 0.13554E+01_JPRB, 0.11058E+01_JPRB, 0.94957E+00_JPRB ,&
&  0.85026E+00_JPRB, 0.76793E+00_JPRB, 0.71683E+00_JPRB, 0.68457E+00_JPRB, 0.65947E+00_JPRB, 0.63776E+00_JPRB, 0.61235E+00_JPRB ,&
&  0.58520E+00_JPRB, 0.56371E+00_JPRB, 0.54932E+00_JPRB, 0.53820E+00_JPRB, 0.53177E+00_JPRB, 0.53007E+00_JPRB, 0.53315E+00_JPRB ,&
&  0.54142E+00_JPRB, 0.56302E+00_JPRB, 0.60738E+00_JPRB, 0.60832E+00_JPRB, 0.62869E+00_JPRB, 0.65789E+00_JPRB, 0.68219E+00_JPRB ,&
&  0.67529E+00_JPRB, 0.68460E+00_JPRB, 0.66710E+00_JPRB, 0.69274E+00_JPRB, 0.87073E+00_JPRB, 0.95776E+00_JPRB, 0.88624E+00_JPRB ,&
&  0.83415E+00_JPRB, 0.78359E+00_JPRB, 0.73620E+00_JPRB, 0.69482E+00_JPRB, 0.68285E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.65391E+00_JPRB, 0.63108E+00_JPRB, 0.61454E+00_JPRB, 0.59665E+00_JPRB, 0.58423E+00_JPRB, 0.57564E+00_JPRB, 0.56303E+00_JPRB ,&
&  0.54970E+00_JPRB, 0.53615E+00_JPRB, 0.52171E+00_JPRB, 0.50795E+00_JPRB, 0.49633E+00_JPRB, 0.48266E+00_JPRB, 0.47154E+00_JPRB ,&
&  0.46123E+00_JPRB, 0.44888E+00_JPRB, 0.44098E+00_JPRB, 0.43865E+00_JPRB, 0.44238E+00_JPRB, 0.44754E+00_JPRB, 0.45418E+00_JPRB ,&
&  0.46221E+00_JPRB, 0.46749E+00_JPRB, 0.46851E+00_JPRB, 0.47081E+00_JPRB, 0.46921E+00_JPRB, 0.47440E+00_JPRB, 0.50329E+00_JPRB ,&
&  0.56949E+00_JPRB, 0.54030E+00_JPRB, 0.84782E+00_JPRB, 0.10495E+01_JPRB, 0.11903E+01_JPRB, 0.13046E+01_JPRB, 0.13631E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.12497E+01_JPRB, 0.10927E+01_JPRB, 0.99019E+00_JPRB, 0.90736E+00_JPRB, 0.84370E+00_JPRB, 0.79326E+00_JPRB ,&
&  0.74475E+00_JPRB, 0.70912E+00_JPRB, 0.68153E+00_JPRB, 0.64380E+00_JPRB, 0.61496E+00_JPRB, 0.60087E+00_JPRB, 0.59913E+00_JPRB ,&
&  0.59698E+00_JPRB, 0.61389E+00_JPRB, 0.69240E+00_JPRB, 0.91092E+00_JPRB, 0.13780E+01_JPRB, 0.17578E+01_JPRB, 0.19316E+01_JPRB ,&
&  0.21125E+01_JPRB, 0.22349E+01_JPRB, 0.20480E+01_JPRB, 0.17168E+01_JPRB, 0.13907E+01_JPRB, 0.11575E+01_JPRB, 0.96975E+00_JPRB ,&
&  0.86058E+00_JPRB, 0.79724E+00_JPRB, 0.75988E+00_JPRB, 0.73113E+00_JPRB, 0.70162E+00_JPRB, 0.66440E+00_JPRB, 0.62349E+00_JPRB ,&
&  0.59247E+00_JPRB, 0.57544E+00_JPRB, 0.56472E+00_JPRB, 0.56035E+00_JPRB, 0.55936E+00_JPRB, 0.56118E+00_JPRB, 0.56899E+00_JPRB ,&
&  0.58578E+00_JPRB, 0.61626E+00_JPRB, 0.63812E+00_JPRB, 0.64569E+00_JPRB, 0.66525E+00_JPRB, 0.69055E+00_JPRB, 0.71903E+00_JPRB ,&
&  0.73801E+00_JPRB, 0.72697E+00_JPRB, 0.70670E+00_JPRB, 0.73736E+00_JPRB, 0.80386E+00_JPRB, 0.88486E+00_JPRB, 0.84643E+00_JPRB ,&
&  0.81637E+00_JPRB, 0.77155E+00_JPRB, 0.71916E+00_JPRB, 0.69385E+00_JPRB, 0.68175E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.65783E+00_JPRB, 0.64046E+00_JPRB, 0.62026E+00_JPRB, 0.60085E+00_JPRB, 0.59335E+00_JPRB, 0.58840E+00_JPRB, 0.57524E+00_JPRB ,&
&  0.56333E+00_JPRB, 0.55377E+00_JPRB, 0.54091E+00_JPRB, 0.52480E+00_JPRB, 0.50559E+00_JPRB, 0.48155E+00_JPRB, 0.45952E+00_JPRB ,&
&  0.44925E+00_JPRB, 0.44103E+00_JPRB, 0.44401E+00_JPRB, 0.45001E+00_JPRB, 0.45884E+00_JPRB, 0.46571E+00_JPRB, 0.47334E+00_JPRB ,&
&  0.47908E+00_JPRB, 0.48171E+00_JPRB, 0.48569E+00_JPRB, 0.48920E+00_JPRB, 0.49070E+00_JPRB, 0.49884E+00_JPRB, 0.52916E+00_JPRB ,&
&  0.59883E+00_JPRB, 0.89510E+00_JPRB, 0.76407E+00_JPRB, 0.13015E+01_JPRB, 0.14983E+01_JPRB, 0.15842E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.14674E+01_JPRB, 0.13914E+01_JPRB, 0.12133E+01_JPRB, 0.10734E+01_JPRB, 0.96589E+00_JPRB, 0.89423E+00_JPRB, 0.84520E+00_JPRB ,&
&  0.81228E+00_JPRB, 0.77461E+00_JPRB, 0.72994E+00_JPRB, 0.68388E+00_JPRB, 0.65418E+00_JPRB, 0.64598E+00_JPRB, 0.64961E+00_JPRB ,&
&  0.65243E+00_JPRB, 0.69634E+00_JPRB, 0.84779E+00_JPRB, 0.12398E+01_JPRB, 0.17220E+01_JPRB, 0.19686E+01_JPRB, 0.19166E+01_JPRB ,&
&  0.18114E+01_JPRB, 0.20090E+01_JPRB, 0.18628E+01_JPRB, 0.15353E+01_JPRB, 0.12812E+01_JPRB, 0.10817E+01_JPRB, 0.93537E+00_JPRB ,&
&  0.84620E+00_JPRB, 0.79130E+00_JPRB, 0.75492E+00_JPRB, 0.72678E+00_JPRB, 0.69843E+00_JPRB, 0.66779E+00_JPRB, 0.63967E+00_JPRB ,&
&  0.61912E+00_JPRB, 0.60428E+00_JPRB, 0.59502E+00_JPRB, 0.59601E+00_JPRB, 0.59704E+00_JPRB, 0.60037E+00_JPRB, 0.60831E+00_JPRB ,&
&  0.62301E+00_JPRB, 0.65162E+00_JPRB, 0.65899E+00_JPRB, 0.67317E+00_JPRB, 0.68211E+00_JPRB, 0.69292E+00_JPRB, 0.70534E+00_JPRB ,&
&  0.75035E+00_JPRB, 0.75694E+00_JPRB, 0.75041E+00_JPRB, 0.76711E+00_JPRB, 0.81551E+00_JPRB, 0.88152E+00_JPRB, 0.81936E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.75688E+00_JPRB, 0.71988E+00_JPRB, 0.69604E+00_JPRB, 0.67894E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.66016E+00_JPRB, 0.63928E+00_JPRB, 0.62455E+00_JPRB, 0.61778E+00_JPRB, 0.61147E+00_JPRB, 0.60491E+00_JPRB, 0.59026E+00_JPRB ,&
&  0.57430E+00_JPRB, 0.56447E+00_JPRB, 0.55002E+00_JPRB, 0.53609E+00_JPRB, 0.50851E+00_JPRB, 0.47401E+00_JPRB, 0.45966E+00_JPRB ,&
&  0.45177E+00_JPRB, 0.45256E+00_JPRB, 0.46251E+00_JPRB, 0.46709E+00_JPRB, 0.47184E+00_JPRB, 0.48106E+00_JPRB, 0.49011E+00_JPRB ,&
&  0.49704E+00_JPRB, 0.49794E+00_JPRB, 0.50323E+00_JPRB, 0.51020E+00_JPRB, 0.51715E+00_JPRB, 0.53880E+00_JPRB, 0.57043E+00_JPRB ,&
&  0.66387E+00_JPRB, 0.88557E+00_JPRB, 0.77860E+00_JPRB, 0.15618E+01_JPRB, 0.18024E+01_JPRB, 0.18050E+01_JPRB, 0.16700E+01_JPRB ,&
&  0.16260E+01_JPRB, 0.16041E+01_JPRB, 0.13220E+01_JPRB, 0.11304E+01_JPRB, 0.99526E+00_JPRB, 0.92286E+00_JPRB, 0.88109E+00_JPRB ,&
&  0.85329E+00_JPRB, 0.82284E+00_JPRB, 0.78055E+00_JPRB, 0.73932E+00_JPRB, 0.72209E+00_JPRB, 0.71913E+00_JPRB, 0.71312E+00_JPRB ,&
&  0.71609E+00_JPRB, 0.81467E+00_JPRB, 0.10732E+01_JPRB, 0.15632E+01_JPRB, 0.20476E+01_JPRB, 0.20700E+01_JPRB, 0.18430E+01_JPRB ,&
&  0.17258E+01_JPRB, 0.18068E+01_JPRB, 0.16217E+01_JPRB, 0.13189E+01_JPRB, 0.11087E+01_JPRB, 0.98777E+00_JPRB, 0.87230E+00_JPRB ,&
&  0.79955E+00_JPRB, 0.75193E+00_JPRB, 0.72685E+00_JPRB, 0.70745E+00_JPRB, 0.69087E+00_JPRB, 0.67704E+00_JPRB, 0.66145E+00_JPRB ,&
&  0.64719E+00_JPRB, 0.63215E+00_JPRB, 0.62453E+00_JPRB, 0.61735E+00_JPRB, 0.62022E+00_JPRB, 0.62476E+00_JPRB, 0.63106E+00_JPRB ,&
&  0.65357E+00_JPRB, 0.69145E+00_JPRB, 0.68959E+00_JPRB, 0.70399E+00_JPRB, 0.70719E+00_JPRB, 0.70351E+00_JPRB, 0.71837E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.76414E+00_JPRB, 0.77864E+00_JPRB, 0.77337E+00_JPRB, 0.79698E+00_JPRB, 0.82602E+00_JPRB, 0.79409E+00_JPRB ,&
&  0.78533E+00_JPRB, 0.76066E+00_JPRB, 0.73326E+00_JPRB, 0.70206E+00_JPRB, 0.68096E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.64476E+00_JPRB, 0.62364E+00_JPRB, 0.61723E+00_JPRB, 0.61619E+00_JPRB, 0.61236E+00_JPRB, 0.60237E+00_JPRB, 0.58422E+00_JPRB ,&
&  0.56222E+00_JPRB, 0.54626E+00_JPRB, 0.52889E+00_JPRB, 0.50715E+00_JPRB, 0.47523E+00_JPRB, 0.44600E+00_JPRB, 0.44474E+00_JPRB ,&
&  0.46229E+00_JPRB, 0.47165E+00_JPRB, 0.48009E+00_JPRB, 0.48937E+00_JPRB, 0.50015E+00_JPRB, 0.51288E+00_JPRB, 0.52261E+00_JPRB ,&
&  0.52619E+00_JPRB, 0.53183E+00_JPRB, 0.54261E+00_JPRB, 0.55583E+00_JPRB, 0.57366E+00_JPRB, 0.60382E+00_JPRB, 0.65281E+00_JPRB ,&
&  0.78430E+00_JPRB, 0.95113E+00_JPRB, 0.96426E+00_JPRB, 0.16112E+01_JPRB, 0.19169E+01_JPRB, 0.18019E+01_JPRB, 0.17565E+01_JPRB ,&
&  0.16601E+01_JPRB, 0.14414E+01_JPRB, 0.13281E+01_JPRB, 0.11204E+01_JPRB, 0.10023E+01_JPRB, 0.94469E+00_JPRB, 0.91260E+00_JPRB ,&
&  0.89501E+00_JPRB, 0.87238E+00_JPRB, 0.83803E+00_JPRB, 0.82540E+00_JPRB, 0.82497E+00_JPRB, 0.80489E+00_JPRB, 0.78482E+00_JPRB ,&
&  0.82913E+00_JPRB, 0.10091E+01_JPRB, 0.13910E+01_JPRB, 0.19845E+01_JPRB, 0.22392E+01_JPRB, 0.20167E+01_JPRB, 0.16562E+01_JPRB ,&
&  0.15651E+01_JPRB, 0.14962E+01_JPRB, 0.13355E+01_JPRB, 0.11190E+01_JPRB, 0.94310E+00_JPRB, 0.92034E+00_JPRB, 0.82058E+00_JPRB ,&
&  0.76172E+00_JPRB, 0.72282E+00_JPRB, 0.70291E+00_JPRB, 0.68820E+00_JPRB, 0.67923E+00_JPRB, 0.67018E+00_JPRB, 0.65831E+00_JPRB ,&
&  0.65034E+00_JPRB, 0.64867E+00_JPRB, 0.64185E+00_JPRB, 0.64194E+00_JPRB, 0.64526E+00_JPRB, 0.65306E+00_JPRB, 0.66992E+00_JPRB ,&
&  0.70665E+00_JPRB, 0.74514E+00_JPRB, 0.74943E+00_JPRB, 0.75841E+00_JPRB, 0.75510E+00_JPRB, 0.74997E+00_JPRB, 0.75068E+00_JPRB ,&
&  0.76107E+00_JPRB, 0.79604E+00_JPRB, 0.80530E+00_JPRB, 0.80335E+00_JPRB, 0.80728E+00_JPRB, 0.80926E+00_JPRB, 0.80264E+00_JPRB ,&
&  0.80827E+00_JPRB, 0.79427E+00_JPRB, 0.76595E+00_JPRB, 0.70156E+00_JPRB, 0.67506E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.60657E+00_JPRB, 0.59142E+00_JPRB, 0.57688E+00_JPRB, 0.57317E+00_JPRB, 0.56906E+00_JPRB, 0.55333E+00_JPRB, 0.53324E+00_JPRB ,&
&  0.50965E+00_JPRB, 0.49207E+00_JPRB, 0.46911E+00_JPRB, 0.44934E+00_JPRB, 0.44777E+00_JPRB, 0.46132E+00_JPRB, 0.48859E+00_JPRB ,&
&  0.51549E+00_JPRB, 0.53049E+00_JPRB, 0.54188E+00_JPRB, 0.55302E+00_JPRB, 0.56380E+00_JPRB, 0.56964E+00_JPRB, 0.57049E+00_JPRB ,&
&  0.57727E+00_JPRB, 0.59251E+00_JPRB, 0.61464E+00_JPRB, 0.64003E+00_JPRB, 0.67343E+00_JPRB, 0.72673E+00_JPRB, 0.82118E+00_JPRB ,&
&  0.10157E+01_JPRB, 0.11087E+01_JPRB, 0.61837E+00_JPRB, 0.11787E+01_JPRB, 0.16634E+01_JPRB, 0.16416E+01_JPRB, 0.15370E+01_JPRB ,&
&  0.14000E+01_JPRB, 0.11895E+01_JPRB, 0.11415E+01_JPRB, 0.10851E+01_JPRB, 0.99449E+00_JPRB, 0.96505E+00_JPRB, 0.96100E+00_JPRB ,&
&  0.96540E+00_JPRB, 0.94934E+00_JPRB, 0.94277E+00_JPRB, 0.95812E+00_JPRB, 0.94213E+00_JPRB, 0.89644E+00_JPRB, 0.90370E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.12834E+01_JPRB, 0.18455E+01_JPRB, 0.22159E+01_JPRB, 0.23733E+01_JPRB, 0.18771E+01_JPRB, 0.14264E+01_JPRB ,&
&  0.12049E+01_JPRB, 0.12136E+01_JPRB, 0.11235E+01_JPRB, 0.98579E+00_JPRB, 0.85848E+00_JPRB, 0.80123E+00_JPRB, 0.78936E+00_JPRB ,&
&  0.73784E+00_JPRB, 0.69752E+00_JPRB, 0.67211E+00_JPRB, 0.65262E+00_JPRB, 0.64127E+00_JPRB, 0.63491E+00_JPRB, 0.63402E+00_JPRB ,&
&  0.64337E+00_JPRB, 0.64855E+00_JPRB, 0.65569E+00_JPRB, 0.66705E+00_JPRB, 0.67988E+00_JPRB, 0.69772E+00_JPRB, 0.72938E+00_JPRB ,&
&  0.77197E+00_JPRB, 0.80490E+00_JPRB, 0.83254E+00_JPRB, 0.82323E+00_JPRB, 0.80925E+00_JPRB, 0.80658E+00_JPRB, 0.82465E+00_JPRB ,&
&  0.83809E+00_JPRB, 0.87052E+00_JPRB, 0.79919E+00_JPRB, 0.79083E+00_JPRB, 0.82261E+00_JPRB, 0.82285E+00_JPRB, 0.85432E+00_JPRB ,&
&  0.90084E+00_JPRB, 0.96555E+00_JPRB, 0.95804E+00_JPRB, 0.68581E+00_JPRB, 0.63977E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.55108E+00_JPRB, 0.53285E+00_JPRB, 0.50610E+00_JPRB, 0.49699E+00_JPRB, 0.50306E+00_JPRB, 0.49707E+00_JPRB, 0.49221E+00_JPRB ,&
&  0.48751E+00_JPRB, 0.48354E+00_JPRB, 0.47551E+00_JPRB, 0.48577E+00_JPRB, 0.51493E+00_JPRB, 0.55654E+00_JPRB, 0.59251E+00_JPRB ,&
&  0.61936E+00_JPRB, 0.63102E+00_JPRB, 0.63431E+00_JPRB, 0.63927E+00_JPRB, 0.64299E+00_JPRB, 0.64696E+00_JPRB, 0.65372E+00_JPRB ,&
&  0.67627E+00_JPRB, 0.71210E+00_JPRB, 0.75831E+00_JPRB, 0.81037E+00_JPRB, 0.87786E+00_JPRB, 0.98482E+00_JPRB, 0.11545E+01_JPRB ,&
&  0.14263E+01_JPRB, 0.10760E+01_JPRB, 0.73546E+00_JPRB, 0.14969E+01_JPRB, 0.16698E+01_JPRB, 0.16229E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.12692E+01_JPRB, 0.11366E+01_JPRB, 0.10570E+01_JPRB, 0.10866E+01_JPRB, 0.10109E+01_JPRB, 0.10061E+01_JPRB, 0.10296E+01_JPRB ,&
&  0.10480E+01_JPRB, 0.10673E+01_JPRB, 0.10895E+01_JPRB, 0.10855E+01_JPRB, 0.10491E+01_JPRB, 0.10371E+01_JPRB, 0.11025E+01_JPRB ,&
&  0.13069E+01_JPRB, 0.17183E+01_JPRB, 0.24214E+01_JPRB, 0.24684E+01_JPRB, 0.25244E+01_JPRB, 0.18002E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.10443E+01_JPRB, 0.10006E+01_JPRB, 0.92234E+00_JPRB, 0.82807E+00_JPRB, 0.75460E+00_JPRB, 0.75545E+00_JPRB ,&
&  0.69885E+00_JPRB, 0.65900E+00_JPRB, 0.62644E+00_JPRB, 0.61039E+00_JPRB, 0.60864E+00_JPRB, 0.61365E+00_JPRB, 0.61709E+00_JPRB ,&
&  0.62919E+00_JPRB, 0.63724E+00_JPRB, 0.65681E+00_JPRB, 0.68049E+00_JPRB, 0.70628E+00_JPRB, 0.73845E+00_JPRB, 0.78251E+00_JPRB ,&
&  0.81977E+00_JPRB, 0.84687E+00_JPRB, 0.86395E+00_JPRB, 0.87054E+00_JPRB, 0.85263E+00_JPRB, 0.83080E+00_JPRB, 0.84479E+00_JPRB ,&
&  0.83729E+00_JPRB, 0.83956E+00_JPRB, 0.81853E+00_JPRB, 0.83823E+00_JPRB, 0.84616E+00_JPRB, 0.85428E+00_JPRB, 0.88223E+00_JPRB ,&
&  0.91660E+00_JPRB, 0.90700E+00_JPRB, 0.81320E+00_JPRB, 0.62421E+00_JPRB, 0.57566E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.52359E+00_JPRB, 0.50089E+00_JPRB, 0.49138E+00_JPRB, 0.49129E+00_JPRB, 0.49978E+00_JPRB, 0.50505E+00_JPRB, 0.50510E+00_JPRB ,&
&  0.50845E+00_JPRB, 0.52039E+00_JPRB, 0.53531E+00_JPRB, 0.58132E+00_JPRB, 0.62956E+00_JPRB, 0.66599E+00_JPRB, 0.68903E+00_JPRB ,&
&  0.70939E+00_JPRB, 0.71966E+00_JPRB, 0.72419E+00_JPRB, 0.73107E+00_JPRB, 0.73721E+00_JPRB, 0.75232E+00_JPRB, 0.78397E+00_JPRB ,&
&  0.83594E+00_JPRB, 0.89296E+00_JPRB, 0.96379E+00_JPRB, 0.10546E+01_JPRB, 0.11705E+01_JPRB, 0.12837E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.10630E+01_JPRB, 0.60538E+00_JPRB, 0.14665E+01_JPRB, 0.17364E+01_JPRB, 0.15365E+01_JPRB, 0.14422E+01_JPRB, 0.13175E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.10963E+01_JPRB, 0.10551E+01_JPRB, 0.10994E+01_JPRB, 0.10586E+01_JPRB, 0.10746E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.11336E+01_JPRB, 0.11831E+01_JPRB, 0.11966E+01_JPRB, 0.11881E+01_JPRB, 0.11919E+01_JPRB, 0.12473E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.17681E+01_JPRB, 0.22944E+01_JPRB, 0.28124E+01_JPRB, 0.25276E+01_JPRB, 0.25706E+01_JPRB, 0.20515E+01_JPRB, 0.15819E+01_JPRB ,&
&  0.11767E+01_JPRB, 0.93218E+00_JPRB, 0.90120E+00_JPRB, 0.89905E+00_JPRB, 0.82369E+00_JPRB, 0.73540E+00_JPRB, 0.72201E+00_JPRB ,&
&  0.65984E+00_JPRB, 0.61730E+00_JPRB, 0.59681E+00_JPRB, 0.59328E+00_JPRB, 0.59942E+00_JPRB, 0.61096E+00_JPRB, 0.62314E+00_JPRB ,&
&  0.62779E+00_JPRB, 0.64035E+00_JPRB, 0.65715E+00_JPRB, 0.68914E+00_JPRB, 0.72518E+00_JPRB, 0.77487E+00_JPRB, 0.81766E+00_JPRB ,&
&  0.84480E+00_JPRB, 0.86930E+00_JPRB, 0.89358E+00_JPRB, 0.91132E+00_JPRB, 0.92726E+00_JPRB, 0.90089E+00_JPRB, 0.87919E+00_JPRB ,&
&  0.87575E+00_JPRB, 0.87734E+00_JPRB, 0.88965E+00_JPRB, 0.91268E+00_JPRB, 0.87803E+00_JPRB, 0.85519E+00_JPRB, 0.87236E+00_JPRB ,&
&  0.86238E+00_JPRB, 0.80705E+00_JPRB, 0.69658E+00_JPRB, 0.58589E+00_JPRB, 0.54585E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.56476E+00_JPRB, 0.55278E+00_JPRB, 0.53887E+00_JPRB, 0.54290E+00_JPRB, 0.54761E+00_JPRB, 0.56815E+00_JPRB, 0.57003E+00_JPRB ,&
&  0.58537E+00_JPRB, 0.62967E+00_JPRB, 0.68396E+00_JPRB, 0.71479E+00_JPRB, 0.74113E+00_JPRB, 0.76662E+00_JPRB, 0.78807E+00_JPRB ,&
&  0.80394E+00_JPRB, 0.81866E+00_JPRB, 0.83272E+00_JPRB, 0.84877E+00_JPRB, 0.87414E+00_JPRB, 0.91180E+00_JPRB, 0.97002E+00_JPRB ,&
&  0.10292E+01_JPRB, 0.10950E+01_JPRB, 0.11779E+01_JPRB, 0.12825E+01_JPRB, 0.13960E+01_JPRB, 0.14634E+01_JPRB, 0.15160E+01_JPRB ,&
&  0.95425E+00_JPRB, 0.14294E+01_JPRB, 0.16830E+01_JPRB, 0.15747E+01_JPRB, 0.14388E+01_JPRB, 0.13610E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.11146E+01_JPRB, 0.10957E+01_JPRB, 0.11896E+01_JPRB, 0.11430E+01_JPRB, 0.11613E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.12573E+01_JPRB, 0.13102E+01_JPRB, 0.13388E+01_JPRB, 0.13823E+01_JPRB, 0.14549E+01_JPRB, 0.16135E+01_JPRB, 0.18793E+01_JPRB ,&
&  0.22437E+01_JPRB, 0.26292E+01_JPRB, 0.28367E+01_JPRB, 0.24404E+01_JPRB, 0.24819E+01_JPRB, 0.23185E+01_JPRB, 0.19961E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.88378E+00_JPRB, 0.89737E+00_JPRB, 0.90087E+00_JPRB, 0.81310E+00_JPRB, 0.74632E+00_JPRB, 0.69074E+00_JPRB ,&
&  0.63475E+00_JPRB, 0.60252E+00_JPRB, 0.59538E+00_JPRB, 0.59191E+00_JPRB, 0.59934E+00_JPRB, 0.60755E+00_JPRB, 0.62314E+00_JPRB ,&
&  0.63208E+00_JPRB, 0.65106E+00_JPRB, 0.67824E+00_JPRB, 0.70836E+00_JPRB, 0.74472E+00_JPRB, 0.78406E+00_JPRB, 0.82694E+00_JPRB ,&
&  0.85830E+00_JPRB, 0.88754E+00_JPRB, 0.91485E+00_JPRB, 0.93611E+00_JPRB, 0.94631E+00_JPRB, 0.94974E+00_JPRB, 0.92878E+00_JPRB ,&
&  0.90996E+00_JPRB, 0.91882E+00_JPRB, 0.92431E+00_JPRB, 0.91552E+00_JPRB, 0.88148E+00_JPRB, 0.84835E+00_JPRB, 0.81782E+00_JPRB ,&
&  0.76065E+00_JPRB, 0.67576E+00_JPRB, 0.59979E+00_JPRB, 0.57904E+00_JPRB, 0.56756E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.64229E+00_JPRB, 0.65021E+00_JPRB, 0.65138E+00_JPRB, 0.68759E+00_JPRB, 0.69573E+00_JPRB, 0.71136E+00_JPRB, 0.72545E+00_JPRB ,&
&  0.73850E+00_JPRB, 0.76145E+00_JPRB, 0.78860E+00_JPRB, 0.81797E+00_JPRB, 0.84215E+00_JPRB, 0.86099E+00_JPRB, 0.87745E+00_JPRB ,&
&  0.89655E+00_JPRB, 0.91876E+00_JPRB, 0.94626E+00_JPRB, 0.98088E+00_JPRB, 0.10347E+01_JPRB, 0.10946E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.13053E+01_JPRB, 0.14091E+01_JPRB, 0.15318E+01_JPRB, 0.16607E+01_JPRB, 0.17547E+01_JPRB, 0.18046E+01_JPRB ,&
&  0.14523E+01_JPRB, 0.17337E+01_JPRB, 0.16839E+01_JPRB, 0.15327E+01_JPRB, 0.14113E+01_JPRB, 0.13465E+01_JPRB, 0.12880E+01_JPRB ,&
&  0.12460E+01_JPRB, 0.11713E+01_JPRB, 0.11545E+01_JPRB, 0.11769E+01_JPRB, 0.12212E+01_JPRB, 0.12467E+01_JPRB, 0.13096E+01_JPRB ,&
&  0.14022E+01_JPRB, 0.14786E+01_JPRB, 0.15666E+01_JPRB, 0.16583E+01_JPRB, 0.17816E+01_JPRB, 0.19652E+01_JPRB, 0.21765E+01_JPRB ,&
&  0.23990E+01_JPRB, 0.25624E+01_JPRB, 0.26742E+01_JPRB, 0.23373E+01_JPRB, 0.21038E+01_JPRB, 0.19087E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.97170E+00_JPRB, 0.83907E+00_JPRB, 0.88827E+00_JPRB, 0.92212E+00_JPRB, 0.83334E+00_JPRB, 0.74883E+00_JPRB, 0.67929E+00_JPRB ,&
&  0.62693E+00_JPRB, 0.59269E+00_JPRB, 0.59449E+00_JPRB, 0.58381E+00_JPRB, 0.58456E+00_JPRB, 0.60326E+00_JPRB, 0.62254E+00_JPRB ,&
&  0.66319E+00_JPRB, 0.68113E+00_JPRB, 0.72136E+00_JPRB, 0.75509E+00_JPRB, 0.79187E+00_JPRB, 0.82463E+00_JPRB, 0.85548E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.88966E+00_JPRB, 0.89419E+00_JPRB, 0.92014E+00_JPRB, 0.92900E+00_JPRB, 0.94449E+00_JPRB, 0.96779E+00_JPRB ,&
&  0.99428E+00_JPRB, 0.10105E+01_JPRB, 0.99166E+00_JPRB, 0.96992E+00_JPRB, 0.93572E+00_JPRB, 0.89268E+00_JPRB, 0.68608E+00_JPRB ,&
&  0.65276E+00_JPRB, 0.61673E+00_JPRB, 0.63344E+00_JPRB, 0.65460E+00_JPRB, 0.64701E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.78601E+00_JPRB, 0.81964E+00_JPRB, 0.82485E+00_JPRB, 0.85075E+00_JPRB, 0.84832E+00_JPRB, 0.83708E+00_JPRB, 0.84814E+00_JPRB ,&
&  0.86492E+00_JPRB, 0.87790E+00_JPRB, 0.90301E+00_JPRB, 0.92833E+00_JPRB, 0.94829E+00_JPRB, 0.95702E+00_JPRB, 0.97558E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.10449E+01_JPRB, 0.10906E+01_JPRB, 0.11511E+01_JPRB, 0.11948E+01_JPRB, 0.12279E+01_JPRB, 0.12827E+01_JPRB ,&
&  0.13283E+01_JPRB, 0.14161E+01_JPRB, 0.15113E+01_JPRB, 0.16049E+01_JPRB, 0.16670E+01_JPRB, 0.16957E+01_JPRB, 0.13661E+01_JPRB ,&
&  0.18454E+01_JPRB, 0.18935E+01_JPRB, 0.17954E+01_JPRB, 0.15403E+01_JPRB, 0.14184E+01_JPRB, 0.13405E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.12083E+01_JPRB, 0.11675E+01_JPRB, 0.11521E+01_JPRB, 0.12746E+01_JPRB, 0.13127E+01_JPRB, 0.14024E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.15916E+01_JPRB, 0.16842E+01_JPRB, 0.17801E+01_JPRB, 0.19097E+01_JPRB, 0.20563E+01_JPRB, 0.21887E+01_JPRB ,&
&  0.22942E+01_JPRB, 0.23433E+01_JPRB, 0.24286E+01_JPRB, 0.21326E+01_JPRB, 0.20072E+01_JPRB, 0.18331E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.90738E+00_JPRB, 0.84845E+00_JPRB, 0.90702E+00_JPRB, 0.91697E+00_JPRB, 0.85227E+00_JPRB, 0.75288E+00_JPRB, 0.68542E+00_JPRB ,&
&  0.61622E+00_JPRB, 0.60422E+00_JPRB, 0.59699E+00_JPRB, 0.58450E+00_JPRB, 0.57652E+00_JPRB, 0.60425E+00_JPRB, 0.62970E+00_JPRB ,&
&  0.68561E+00_JPRB, 0.72514E+00_JPRB, 0.77533E+00_JPRB, 0.82064E+00_JPRB, 0.88559E+00_JPRB, 0.96378E+00_JPRB, 0.10371E+01_JPRB ,&
&  0.91580E+00_JPRB, 0.85866E+00_JPRB, 0.89949E+00_JPRB, 0.95703E+00_JPRB, 0.99645E+00_JPRB, 0.10832E+01_JPRB, 0.11857E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.14048E+01_JPRB, 0.12851E+01_JPRB, 0.10807E+01_JPRB, 0.14589E+01_JPRB, 0.15039E+01_JPRB, 0.70589E+00_JPRB ,&
&  0.69788E+00_JPRB, 0.73138E+00_JPRB, 0.75115E+00_JPRB, 0.77401E+00_JPRB, 0.79532E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.87233E+00_JPRB, 0.88740E+00_JPRB, 0.90152E+00_JPRB, 0.91148E+00_JPRB, 0.91363E+00_JPRB, 0.92662E+00_JPRB, 0.95014E+00_JPRB ,&
&  0.97222E+00_JPRB, 0.99378E+00_JPRB, 0.10148E+01_JPRB, 0.10323E+01_JPRB, 0.10511E+01_JPRB, 0.10641E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11630E+01_JPRB, 0.11820E+01_JPRB, 0.12191E+01_JPRB, 0.12426E+01_JPRB, 0.12557E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.13520E+01_JPRB, 0.14131E+01_JPRB, 0.14580E+01_JPRB, 0.14848E+01_JPRB, 0.14815E+01_JPRB, 0.15707E+01_JPRB, 0.14117E+01_JPRB ,&
&  0.17565E+01_JPRB, 0.19787E+01_JPRB, 0.18444E+01_JPRB, 0.15103E+01_JPRB, 0.13966E+01_JPRB, 0.13384E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.12725E+01_JPRB, 0.12426E+01_JPRB, 0.11912E+01_JPRB, 0.12455E+01_JPRB, 0.12925E+01_JPRB, 0.13611E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.15376E+01_JPRB, 0.16220E+01_JPRB, 0.17120E+01_JPRB, 0.18008E+01_JPRB, 0.19193E+01_JPRB, 0.20218E+01_JPRB, 0.20815E+01_JPRB ,&
&  0.21221E+01_JPRB, 0.21458E+01_JPRB, 0.21483E+01_JPRB, 0.19827E+01_JPRB, 0.19046E+01_JPRB, 0.16635E+01_JPRB, 0.12989E+01_JPRB ,&
&  0.94415E+00_JPRB, 0.83649E+00_JPRB, 0.87223E+00_JPRB, 0.90859E+00_JPRB, 0.86756E+00_JPRB, 0.75800E+00_JPRB, 0.72200E+00_JPRB ,&
&  0.63296E+00_JPRB, 0.58151E+00_JPRB, 0.58041E+00_JPRB, 0.57073E+00_JPRB, 0.56212E+00_JPRB, 0.59527E+00_JPRB, 0.62261E+00_JPRB ,&
&  0.66613E+00_JPRB, 0.72659E+00_JPRB, 0.77104E+00_JPRB, 0.79941E+00_JPRB, 0.87278E+00_JPRB, 0.94283E+00_JPRB, 0.89301E+00_JPRB ,&
&  0.93636E+00_JPRB, 0.96502E+00_JPRB, 0.99760E+00_JPRB, 0.11121E+01_JPRB, 0.10996E+01_JPRB, 0.12989E+01_JPRB, 0.15164E+01_JPRB ,&
&  0.18172E+01_JPRB, 0.17604E+01_JPRB, 0.20687E+01_JPRB, 0.30973E+01_JPRB, 0.26650E+01_JPRB, 0.22465E+01_JPRB, 0.84370E+00_JPRB ,&
&  0.83493E+00_JPRB, 0.85036E+00_JPRB, 0.87421E+00_JPRB, 0.87624E+00_JPRB, 0.87742E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.90770E+00_JPRB, 0.92528E+00_JPRB, 0.93846E+00_JPRB, 0.95809E+00_JPRB, 0.98239E+00_JPRB, 0.10106E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.10736E+01_JPRB, 0.10909E+01_JPRB, 0.11109E+01_JPRB, 0.11256E+01_JPRB, 0.11458E+01_JPRB, 0.11515E+01_JPRB ,&
&  0.11714E+01_JPRB, 0.11682E+01_JPRB, 0.11785E+01_JPRB, 0.11890E+01_JPRB, 0.11990E+01_JPRB, 0.12090E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.12389E+01_JPRB, 0.12371E+01_JPRB, 0.12471E+01_JPRB, 0.12629E+01_JPRB, 0.12680E+01_JPRB, 0.13465E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.18653E+01_JPRB, 0.17828E+01_JPRB, 0.14406E+01_JPRB, 0.13167E+01_JPRB, 0.12760E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12474E+01_JPRB, 0.12420E+01_JPRB, 0.12283E+01_JPRB, 0.12423E+01_JPRB, 0.12838E+01_JPRB, 0.13511E+01_JPRB, 0.14107E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.15077E+01_JPRB, 0.15754E+01_JPRB, 0.16484E+01_JPRB, 0.17534E+01_JPRB, 0.18090E+01_JPRB, 0.18080E+01_JPRB ,&
&  0.17853E+01_JPRB, 0.17547E+01_JPRB, 0.15915E+01_JPRB, 0.15956E+01_JPRB, 0.15807E+01_JPRB, 0.12799E+01_JPRB, 0.10553E+01_JPRB ,&
&  0.90706E+00_JPRB, 0.85542E+00_JPRB, 0.88729E+00_JPRB, 0.95284E+00_JPRB, 0.88159E+00_JPRB, 0.81251E+00_JPRB, 0.76373E+00_JPRB ,&
&  0.66015E+00_JPRB, 0.60469E+00_JPRB, 0.58666E+00_JPRB, 0.56076E+00_JPRB, 0.56126E+00_JPRB, 0.58148E+00_JPRB, 0.63034E+00_JPRB ,&
&  0.65544E+00_JPRB, 0.68051E+00_JPRB, 0.71180E+00_JPRB, 0.72528E+00_JPRB, 0.76536E+00_JPRB, 0.73550E+00_JPRB, 0.86210E+00_JPRB ,&
&  0.85826E+00_JPRB, 0.85200E+00_JPRB, 0.90024E+00_JPRB, 0.78067E+00_JPRB, 0.10381E+01_JPRB, 0.11207E+01_JPRB, 0.12710E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.17764E+01_JPRB, 0.21533E+01_JPRB, 0.18117E+01_JPRB, 0.13242E+01_JPRB, 0.92038E+00_JPRB, 0.82791E+00_JPRB ,&
&  0.82661E+00_JPRB, 0.84853E+00_JPRB, 0.87053E+00_JPRB, 0.88155E+00_JPRB, 0.89413E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.90666E+00_JPRB, 0.92376E+00_JPRB, 0.94141E+00_JPRB, 0.96998E+00_JPRB, 0.10010E+01_JPRB, 0.10307E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10761E+01_JPRB, 0.10934E+01_JPRB, 0.11148E+01_JPRB, 0.11343E+01_JPRB, 0.11481E+01_JPRB, 0.11556E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11636E+01_JPRB, 0.11781E+01_JPRB, 0.11892E+01_JPRB, 0.11887E+01_JPRB, 0.11982E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.12464E+01_JPRB, 0.12790E+01_JPRB, 0.13257E+01_JPRB, 0.13898E+01_JPRB, 0.15194E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.14982E+01_JPRB, 0.17107E+01_JPRB, 0.16964E+01_JPRB, 0.13442E+01_JPRB, 0.12179E+01_JPRB, 0.11766E+01_JPRB, 0.11650E+01_JPRB ,&
&  0.11631E+01_JPRB, 0.11564E+01_JPRB, 0.11628E+01_JPRB, 0.11894E+01_JPRB, 0.12346E+01_JPRB, 0.12844E+01_JPRB, 0.13034E+01_JPRB ,&
&  0.13306E+01_JPRB, 0.13583E+01_JPRB, 0.13905E+01_JPRB, 0.14180E+01_JPRB, 0.14713E+01_JPRB, 0.14805E+01_JPRB, 0.14452E+01_JPRB ,&
&  0.13819E+01_JPRB, 0.13269E+01_JPRB, 0.11379E+01_JPRB, 0.10522E+01_JPRB, 0.10491E+01_JPRB, 0.92828E+00_JPRB, 0.82589E+00_JPRB ,&
&  0.86910E+00_JPRB, 0.85427E+00_JPRB, 0.83210E+00_JPRB, 0.99030E+00_JPRB, 0.96180E+00_JPRB, 0.84770E+00_JPRB, 0.81302E+00_JPRB ,&
&  0.71538E+00_JPRB, 0.62883E+00_JPRB, 0.58424E+00_JPRB, 0.58076E+00_JPRB, 0.58823E+00_JPRB, 0.61496E+00_JPRB, 0.61288E+00_JPRB ,&
&  0.62001E+00_JPRB, 0.63624E+00_JPRB, 0.64217E+00_JPRB, 0.64448E+00_JPRB, 0.71261E+00_JPRB, 0.64741E+00_JPRB, 0.81685E+00_JPRB ,&
&  0.72757E+00_JPRB, 0.62693E+00_JPRB, 0.71360E+00_JPRB, 0.87421E+00_JPRB, 0.93606E+00_JPRB, 0.10190E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.13538E+01_JPRB, 0.14934E+01_JPRB, 0.13504E+01_JPRB, 0.11494E+01_JPRB, 0.94226E+00_JPRB, 0.85388E+00_JPRB, 0.81734E+00_JPRB ,&
&  0.81308E+00_JPRB, 0.83970E+00_JPRB, 0.86074E+00_JPRB, 0.87225E+00_JPRB, 0.88853E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.86445E+00_JPRB, 0.88367E+00_JPRB, 0.90473E+00_JPRB, 0.92941E+00_JPRB, 0.95292E+00_JPRB, 0.97448E+00_JPRB, 0.99788E+00_JPRB ,&
&  0.10279E+01_JPRB, 0.10509E+01_JPRB, 0.10746E+01_JPRB, 0.10892E+01_JPRB, 0.10970E+01_JPRB, 0.11121E+01_JPRB, 0.11175E+01_JPRB ,&
&  0.11206E+01_JPRB, 0.11231E+01_JPRB, 0.11406E+01_JPRB, 0.11524E+01_JPRB, 0.11541E+01_JPRB, 0.11527E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11736E+01_JPRB, 0.12086E+01_JPRB, 0.12370E+01_JPRB, 0.12725E+01_JPRB, 0.13023E+01_JPRB, 0.13132E+01_JPRB, 0.13928E+01_JPRB ,&
&  0.10968E+01_JPRB, 0.16420E+01_JPRB, 0.15764E+01_JPRB, 0.12391E+01_JPRB, 0.11699E+01_JPRB, 0.10982E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10688E+01_JPRB, 0.10707E+01_JPRB, 0.10870E+01_JPRB, 0.11195E+01_JPRB, 0.11536E+01_JPRB, 0.11907E+01_JPRB, 0.12019E+01_JPRB ,&
&  0.12151E+01_JPRB, 0.12212E+01_JPRB, 0.12228E+01_JPRB, 0.12186E+01_JPRB, 0.12232E+01_JPRB, 0.12049E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.10167E+01_JPRB, 0.73822E+00_JPRB, 0.68501E+00_JPRB, 0.71292E+00_JPRB, 0.68467E+00_JPRB, 0.62064E+00_JPRB ,&
&  0.74940E+00_JPRB, 0.89574E+00_JPRB, 0.96434E+00_JPRB, 0.10235E+01_JPRB, 0.10204E+01_JPRB, 0.95375E+00_JPRB, 0.92125E+00_JPRB ,&
&  0.85391E+00_JPRB, 0.75673E+00_JPRB, 0.70738E+00_JPRB, 0.67636E+00_JPRB, 0.67995E+00_JPRB, 0.66343E+00_JPRB, 0.63680E+00_JPRB ,&
&  0.60125E+00_JPRB, 0.58463E+00_JPRB, 0.57528E+00_JPRB, 0.55878E+00_JPRB, 0.52378E+00_JPRB, 0.86181E+00_JPRB, 0.10378E+01_JPRB ,&
&  0.68994E+00_JPRB, 0.64644E+00_JPRB, 0.56868E+00_JPRB, 0.74688E+00_JPRB, 0.77014E+00_JPRB, 0.80701E+00_JPRB, 0.92468E+00_JPRB ,&
&  0.10195E+01_JPRB, 0.10317E+01_JPRB, 0.94225E+00_JPRB, 0.83780E+00_JPRB, 0.79734E+00_JPRB, 0.78235E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.75674E+00_JPRB, 0.78151E+00_JPRB, 0.80531E+00_JPRB, 0.82405E+00_JPRB, 0.84412E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.81977E+00_JPRB, 0.81391E+00_JPRB, 0.82565E+00_JPRB, 0.84341E+00_JPRB, 0.86040E+00_JPRB, 0.87919E+00_JPRB, 0.90951E+00_JPRB ,&
&  0.94626E+00_JPRB, 0.97549E+00_JPRB, 0.10038E+01_JPRB, 0.10240E+01_JPRB, 0.10308E+01_JPRB, 0.10451E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.10728E+01_JPRB, 0.10915E+01_JPRB, 0.11224E+01_JPRB, 0.11584E+01_JPRB, 0.11728E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.12460E+01_JPRB, 0.12943E+01_JPRB, 0.13517E+01_JPRB, 0.14500E+01_JPRB, 0.16229E+01_JPRB, 0.17990E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.14857E+01_JPRB, 0.13530E+01_JPRB, 0.10906E+01_JPRB, 0.10129E+01_JPRB, 0.10342E+01_JPRB, 0.99051E+00_JPRB ,&
&  0.95699E+00_JPRB, 0.97038E+00_JPRB, 0.98384E+00_JPRB, 0.99519E+00_JPRB, 0.10173E+01_JPRB, 0.10545E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10688E+01_JPRB, 0.10599E+01_JPRB, 0.10735E+01_JPRB, 0.99597E+00_JPRB, 0.11209E+01_JPRB, 0.10675E+01_JPRB ,&
&  0.10020E+01_JPRB, 0.89541E+00_JPRB, 0.64929E+00_JPRB, 0.65058E+00_JPRB, 0.67526E+00_JPRB, 0.61536E+00_JPRB, 0.58133E+00_JPRB ,&
&  0.73256E+00_JPRB, 0.87023E+00_JPRB, 0.77925E+00_JPRB, 0.98127E+00_JPRB, 0.10841E+01_JPRB, 0.11226E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.10002E+01_JPRB, 0.90492E+00_JPRB, 0.84538E+00_JPRB, 0.81832E+00_JPRB, 0.77593E+00_JPRB, 0.71745E+00_JPRB ,&
&  0.66497E+00_JPRB, 0.62519E+00_JPRB, 0.57799E+00_JPRB, 0.54668E+00_JPRB, 0.62795E+00_JPRB, 0.71355E+00_JPRB, 0.10153E+01_JPRB ,&
&  0.73300E+00_JPRB, 0.72880E+00_JPRB, 0.65001E+00_JPRB, 0.97499E+00_JPRB, 0.78000E+00_JPRB, 0.62878E+00_JPRB, 0.68149E+00_JPRB ,&
&  0.73132E+00_JPRB, 0.76547E+00_JPRB, 0.75314E+00_JPRB, 0.72753E+00_JPRB, 0.71904E+00_JPRB, 0.72088E+00_JPRB, 0.69967E+00_JPRB ,&
&  0.68979E+00_JPRB, 0.70659E+00_JPRB, 0.74080E+00_JPRB, 0.75384E+00_JPRB, 0.78762E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.72205E+00_JPRB, 0.72636E+00_JPRB, 0.73195E+00_JPRB, 0.74095E+00_JPRB, 0.75986E+00_JPRB, 0.78099E+00_JPRB, 0.79662E+00_JPRB ,&
&  0.82729E+00_JPRB, 0.85137E+00_JPRB, 0.87572E+00_JPRB, 0.88591E+00_JPRB, 0.90216E+00_JPRB, 0.91616E+00_JPRB, 0.92198E+00_JPRB ,&
&  0.94467E+00_JPRB, 0.97212E+00_JPRB, 0.97442E+00_JPRB, 0.98438E+00_JPRB, 0.10126E+01_JPRB, 0.10320E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.99592E+00_JPRB, 0.97226E+00_JPRB, 0.99483E+00_JPRB, 0.10375E+01_JPRB, 0.12414E+01_JPRB, 0.13656E+01_JPRB, 0.13516E+01_JPRB ,&
&  0.13043E+01_JPRB, 0.14596E+01_JPRB, 0.14439E+01_JPRB, 0.10909E+01_JPRB, 0.98670E+00_JPRB, 0.92658E+00_JPRB, 0.86350E+00_JPRB ,&
&  0.87186E+00_JPRB, 0.89984E+00_JPRB, 0.93679E+00_JPRB, 0.97263E+00_JPRB, 0.10212E+01_JPRB, 0.10439E+01_JPRB, 0.10912E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.11021E+01_JPRB, 0.11051E+01_JPRB, 0.11015E+01_JPRB, 0.11352E+01_JPRB, 0.12639E+01_JPRB, 0.12442E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.12445E+01_JPRB, 0.11282E+01_JPRB, 0.11017E+01_JPRB, 0.10501E+01_JPRB, 0.90853E+00_JPRB, 0.78439E+00_JPRB ,&
&  0.77667E+00_JPRB, 0.77482E+00_JPRB, 0.67367E+00_JPRB, 0.91664E+00_JPRB, 0.11192E+01_JPRB, 0.11957E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.13808E+01_JPRB, 0.12554E+01_JPRB, 0.10842E+01_JPRB, 0.10699E+01_JPRB, 0.10544E+01_JPRB, 0.89348E+00_JPRB, 0.84171E+00_JPRB ,&
&  0.74471E+00_JPRB, 0.66521E+00_JPRB, 0.61223E+00_JPRB, 0.59605E+00_JPRB, 0.57981E+00_JPRB, 0.66710E+00_JPRB, 0.79788E+00_JPRB ,&
&  0.73874E+00_JPRB, 0.74134E+00_JPRB, 0.94181E+00_JPRB, 0.15090E+01_JPRB, 0.86874E+00_JPRB, 0.66309E+00_JPRB, 0.57642E+00_JPRB ,&
&  0.58217E+00_JPRB, 0.58570E+00_JPRB, 0.58940E+00_JPRB, 0.59375E+00_JPRB, 0.60094E+00_JPRB, 0.61996E+00_JPRB, 0.61571E+00_JPRB ,&
&  0.61130E+00_JPRB, 0.60837E+00_JPRB, 0.62540E+00_JPRB, 0.64861E+00_JPRB, 0.69039E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.58027E+00_JPRB, 0.57899E+00_JPRB, 0.56699E+00_JPRB, 0.56042E+00_JPRB, 0.56366E+00_JPRB, 0.57294E+00_JPRB, 0.58655E+00_JPRB ,&
&  0.60823E+00_JPRB, 0.63681E+00_JPRB, 0.66609E+00_JPRB, 0.70205E+00_JPRB, 0.71116E+00_JPRB, 0.67295E+00_JPRB, 0.67517E+00_JPRB ,&
&  0.67134E+00_JPRB, 0.68544E+00_JPRB, 0.68093E+00_JPRB, 0.76233E+00_JPRB, 0.79047E+00_JPRB, 0.80696E+00_JPRB, 0.90934E+00_JPRB ,&
&  0.90819E+00_JPRB, 0.10146E+01_JPRB, 0.11021E+01_JPRB, 0.12382E+01_JPRB, 0.20320E+01_JPRB, 0.18884E+01_JPRB, 0.13822E+01_JPRB ,&
&  0.17508E+01_JPRB, 0.18048E+01_JPRB, 0.13804E+01_JPRB, 0.11054E+01_JPRB, 0.10311E+01_JPRB, 0.93532E+00_JPRB, 0.90101E+00_JPRB ,&
&  0.95549E+00_JPRB, 0.10359E+01_JPRB, 0.11782E+01_JPRB, 0.13123E+01_JPRB, 0.14167E+01_JPRB, 0.15187E+01_JPRB, 0.16424E+01_JPRB ,&
&  0.17163E+01_JPRB, 0.17753E+01_JPRB, 0.17117E+01_JPRB, 0.16314E+01_JPRB, 0.18042E+01_JPRB, 0.19509E+01_JPRB, 0.20391E+01_JPRB ,&
&  0.20970E+01_JPRB, 0.21838E+01_JPRB, 0.22928E+01_JPRB, 0.21567E+01_JPRB, 0.19294E+01_JPRB, 0.17701E+01_JPRB, 0.14403E+01_JPRB ,&
&  0.11780E+01_JPRB, 0.98579E+00_JPRB, 0.92730E+00_JPRB, 0.12480E+01_JPRB, 0.14263E+01_JPRB, 0.13284E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.16198E+01_JPRB, 0.16427E+01_JPRB, 0.14589E+01_JPRB, 0.13045E+01_JPRB, 0.11199E+01_JPRB, 0.10359E+01_JPRB, 0.94565E+00_JPRB ,&
&  0.85835E+00_JPRB, 0.73206E+00_JPRB, 0.65493E+00_JPRB, 0.60872E+00_JPRB, 0.56010E+00_JPRB, 0.55738E+00_JPRB, 0.60084E+00_JPRB ,&
&  0.63887E+00_JPRB, 0.65935E+00_JPRB, 0.78475E+00_JPRB, 0.12468E+01_JPRB, 0.11766E+01_JPRB, 0.72298E+00_JPRB, 0.54445E+00_JPRB ,&
&  0.51337E+00_JPRB, 0.52044E+00_JPRB, 0.52175E+00_JPRB, 0.52380E+00_JPRB, 0.53783E+00_JPRB, 0.55341E+00_JPRB, 0.54652E+00_JPRB ,&
&  0.54929E+00_JPRB, 0.54276E+00_JPRB, 0.53839E+00_JPRB, 0.54070E+00_JPRB, 0.55329E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.52201E+00_JPRB, 0.52213E+00_JPRB, 0.53829E+00_JPRB, 0.59241E+00_JPRB, 0.62699E+00_JPRB, 0.64877E+00_JPRB, 0.68867E+00_JPRB ,&
&  0.73374E+00_JPRB, 0.81006E+00_JPRB, 0.86900E+00_JPRB, 0.89067E+00_JPRB, 0.86412E+00_JPRB, 0.83315E+00_JPRB, 0.81836E+00_JPRB ,&
&  0.81309E+00_JPRB, 0.75555E+00_JPRB, 0.72274E+00_JPRB, 0.74727E+00_JPRB, 0.81916E+00_JPRB, 0.94350E+00_JPRB, 0.10929E+01_JPRB ,&
&  0.12949E+01_JPRB, 0.15204E+01_JPRB, 0.17974E+01_JPRB, 0.24054E+01_JPRB, 0.25836E+01_JPRB, 0.14192E+01_JPRB, 0.15461E+01_JPRB ,&
&  0.20437E+01_JPRB, 0.20425E+01_JPRB, 0.14145E+01_JPRB, 0.12209E+01_JPRB, 0.11420E+01_JPRB, 0.10998E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.13416E+01_JPRB, 0.14679E+01_JPRB, 0.16309E+01_JPRB, 0.18111E+01_JPRB, 0.19751E+01_JPRB, 0.21777E+01_JPRB ,&
&  0.23508E+01_JPRB, 0.24008E+01_JPRB, 0.24225E+01_JPRB, 0.24805E+01_JPRB, 0.26394E+01_JPRB, 0.28338E+01_JPRB, 0.29961E+01_JPRB ,&
&  0.30843E+01_JPRB, 0.31175E+01_JPRB, 0.32039E+01_JPRB, 0.31756E+01_JPRB, 0.28331E+01_JPRB, 0.25603E+01_JPRB, 0.21738E+01_JPRB ,&
&  0.19023E+01_JPRB, 0.16375E+01_JPRB, 0.15257E+01_JPRB, 0.20123E+01_JPRB, 0.22059E+01_JPRB, 0.20050E+01_JPRB, 0.19546E+01_JPRB ,&
&  0.19880E+01_JPRB, 0.19334E+01_JPRB, 0.17538E+01_JPRB, 0.14187E+01_JPRB, 0.11675E+01_JPRB, 0.11840E+01_JPRB, 0.10495E+01_JPRB ,&
&  0.99744E+00_JPRB, 0.84697E+00_JPRB, 0.74925E+00_JPRB, 0.67860E+00_JPRB, 0.61020E+00_JPRB, 0.59589E+00_JPRB, 0.56780E+00_JPRB ,&
&  0.54396E+00_JPRB, 0.71793E+00_JPRB, 0.80962E+00_JPRB, 0.13012E+01_JPRB, 0.22153E+01_JPRB, 0.11298E+01_JPRB, 0.69615E+00_JPRB ,&
&  0.53774E+00_JPRB, 0.52664E+00_JPRB, 0.53163E+00_JPRB, 0.53595E+00_JPRB, 0.54277E+00_JPRB, 0.55495E+00_JPRB, 0.55805E+00_JPRB ,&
&  0.57191E+00_JPRB, 0.56890E+00_JPRB, 0.57341E+00_JPRB, 0.55514E+00_JPRB, 0.52119E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.69403E+00_JPRB, 0.71997E+00_JPRB, 0.76425E+00_JPRB, 0.82676E+00_JPRB, 0.86950E+00_JPRB, 0.90345E+00_JPRB, 0.90938E+00_JPRB ,&
&  0.94090E+00_JPRB, 0.96907E+00_JPRB, 0.10146E+01_JPRB, 0.10551E+01_JPRB, 0.10872E+01_JPRB, 0.11529E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.12049E+01_JPRB, 0.12580E+01_JPRB, 0.13510E+01_JPRB, 0.15373E+01_JPRB, 0.17778E+01_JPRB, 0.20160E+01_JPRB ,&
&  0.22655E+01_JPRB, 0.23119E+01_JPRB, 0.22805E+01_JPRB, 0.15909E+01_JPRB, 0.12392E+01_JPRB, 0.14454E+01_JPRB, 0.14914E+01_JPRB ,&
&  0.15134E+01_JPRB, 0.14899E+01_JPRB, 0.13093E+01_JPRB, 0.12250E+01_JPRB, 0.11803E+01_JPRB, 0.12031E+01_JPRB, 0.13119E+01_JPRB ,&
&  0.14726E+01_JPRB, 0.16566E+01_JPRB, 0.18386E+01_JPRB, 0.19655E+01_JPRB, 0.21112E+01_JPRB, 0.23214E+01_JPRB, 0.25695E+01_JPRB ,&
&  0.27877E+01_JPRB, 0.29245E+01_JPRB, 0.29839E+01_JPRB, 0.31220E+01_JPRB, 0.32156E+01_JPRB, 0.33627E+01_JPRB, 0.34712E+01_JPRB ,&
&  0.35613E+01_JPRB, 0.36090E+01_JPRB, 0.36603E+01_JPRB, 0.37167E+01_JPRB, 0.35852E+01_JPRB, 0.31770E+01_JPRB, 0.28139E+01_JPRB ,&
&  0.26416E+01_JPRB, 0.24141E+01_JPRB, 0.24686E+01_JPRB, 0.27177E+01_JPRB, 0.19766E+01_JPRB, 0.20709E+01_JPRB, 0.23705E+01_JPRB ,&
&  0.23111E+01_JPRB, 0.22318E+01_JPRB, 0.20426E+01_JPRB, 0.17583E+01_JPRB, 0.16092E+01_JPRB, 0.13933E+01_JPRB, 0.12527E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.94693E+00_JPRB, 0.81950E+00_JPRB, 0.68933E+00_JPRB, 0.67156E+00_JPRB, 0.74024E+00_JPRB, 0.71792E+00_JPRB ,&
&  0.83363E+00_JPRB, 0.94633E+00_JPRB, 0.97217E+00_JPRB, 0.14407E+01_JPRB, 0.12993E+01_JPRB, 0.82905E+00_JPRB, 0.62717E+00_JPRB ,&
&  0.54032E+00_JPRB, 0.53280E+00_JPRB, 0.53934E+00_JPRB, 0.52988E+00_JPRB, 0.54409E+00_JPRB, 0.56702E+00_JPRB, 0.56707E+00_JPRB ,&
&  0.61103E+00_JPRB, 0.63579E+00_JPRB, 0.66507E+00_JPRB, 0.69063E+00_JPRB, 0.68164E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.85953E+00_JPRB, 0.95307E+00_JPRB, 0.10259E+01_JPRB, 0.11024E+01_JPRB, 0.11522E+01_JPRB, 0.12247E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.10994E+01_JPRB, 0.11721E+01_JPRB, 0.12703E+01_JPRB, 0.13603E+01_JPRB, 0.14315E+01_JPRB, 0.14944E+01_JPRB ,&
&  0.15560E+01_JPRB, 0.16108E+01_JPRB, 0.16662E+01_JPRB, 0.17049E+01_JPRB, 0.17663E+01_JPRB, 0.18329E+01_JPRB, 0.18354E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.13353E+01_JPRB, 0.17102E+01_JPRB, 0.19954E+01_JPRB, 0.20584E+01_JPRB, 0.21931E+01_JPRB, 0.22983E+01_JPRB ,&
&  0.16898E+01_JPRB, 0.14073E+01_JPRB, 0.14337E+01_JPRB, 0.14293E+01_JPRB, 0.11959E+01_JPRB, 0.12956E+01_JPRB, 0.15178E+01_JPRB ,&
&  0.17414E+01_JPRB, 0.19509E+01_JPRB, 0.21307E+01_JPRB, 0.22657E+01_JPRB, 0.24130E+01_JPRB, 0.26257E+01_JPRB, 0.28433E+01_JPRB ,&
&  0.30498E+01_JPRB, 0.31973E+01_JPRB, 0.32858E+01_JPRB, 0.33871E+01_JPRB, 0.35212E+01_JPRB, 0.36999E+01_JPRB, 0.37923E+01_JPRB ,&
&  0.38872E+01_JPRB, 0.40096E+01_JPRB, 0.41840E+01_JPRB, 0.43557E+01_JPRB, 0.43903E+01_JPRB, 0.38796E+01_JPRB, 0.36134E+01_JPRB ,&
&  0.34282E+01_JPRB, 0.31351E+01_JPRB, 0.32121E+01_JPRB, 0.33494E+01_JPRB, 0.24831E+01_JPRB, 0.24462E+01_JPRB, 0.24913E+01_JPRB ,&
&  0.27532E+01_JPRB, 0.26039E+01_JPRB, 0.24398E+01_JPRB, 0.24381E+01_JPRB, 0.22743E+01_JPRB, 0.20326E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.98719E+00_JPRB, 0.82257E+00_JPRB, 0.75911E+00_JPRB, 0.69222E+00_JPRB, 0.85285E+00_JPRB, 0.99051E+00_JPRB ,&
&  0.13299E+01_JPRB, 0.10339E+01_JPRB, 0.10357E+01_JPRB, 0.12817E+01_JPRB, 0.94797E+00_JPRB, 0.68407E+00_JPRB, 0.56975E+00_JPRB ,&
&  0.53340E+00_JPRB, 0.55385E+00_JPRB, 0.56267E+00_JPRB, 0.55768E+00_JPRB, 0.55725E+00_JPRB, 0.58386E+00_JPRB, 0.61470E+00_JPRB ,&
&  0.64406E+00_JPRB, 0.68151E+00_JPRB, 0.72034E+00_JPRB, 0.76753E+00_JPRB, 0.82833E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.89708E+00_JPRB, 0.95961E+00_JPRB, 0.10736E+01_JPRB, 0.12247E+01_JPRB, 0.14339E+01_JPRB, 0.16404E+01_JPRB, 0.17029E+01_JPRB ,&
&  0.14650E+01_JPRB, 0.12302E+01_JPRB, 0.13350E+01_JPRB, 0.14411E+01_JPRB, 0.15335E+01_JPRB, 0.16388E+01_JPRB, 0.17332E+01_JPRB ,&
&  0.18409E+01_JPRB, 0.19485E+01_JPRB, 0.20422E+01_JPRB, 0.21250E+01_JPRB, 0.21616E+01_JPRB, 0.21607E+01_JPRB, 0.19480E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.14518E+01_JPRB, 0.20622E+01_JPRB, 0.20466E+01_JPRB, 0.21578E+01_JPRB, 0.23554E+01_JPRB, 0.21807E+01_JPRB ,&
&  0.16642E+01_JPRB, 0.14388E+01_JPRB, 0.14993E+01_JPRB, 0.14568E+01_JPRB, 0.13101E+01_JPRB, 0.14628E+01_JPRB, 0.17439E+01_JPRB ,&
&  0.19257E+01_JPRB, 0.21077E+01_JPRB, 0.23067E+01_JPRB, 0.24852E+01_JPRB, 0.26012E+01_JPRB, 0.27677E+01_JPRB, 0.29213E+01_JPRB ,&
&  0.30498E+01_JPRB, 0.31887E+01_JPRB, 0.32786E+01_JPRB, 0.33811E+01_JPRB, 0.36446E+01_JPRB, 0.39580E+01_JPRB, 0.42070E+01_JPRB ,&
&  0.43299E+01_JPRB, 0.46723E+01_JPRB, 0.48094E+01_JPRB, 0.47350E+01_JPRB, 0.43818E+01_JPRB, 0.46321E+01_JPRB, 0.41691E+01_JPRB ,&
&  0.41434E+01_JPRB, 0.38379E+01_JPRB, 0.40598E+01_JPRB, 0.33632E+01_JPRB, 0.33009E+01_JPRB, 0.33093E+01_JPRB, 0.32757E+01_JPRB ,&
&  0.31213E+01_JPRB, 0.31273E+01_JPRB, 0.28133E+01_JPRB, 0.26979E+01_JPRB, 0.27573E+01_JPRB, 0.25515E+01_JPRB, 0.25436E+01_JPRB ,&
&  0.16270E+01_JPRB, 0.11611E+01_JPRB, 0.84153E+00_JPRB, 0.73701E+00_JPRB, 0.71059E+00_JPRB, 0.91881E+00_JPRB, 0.13557E+01_JPRB ,&
&  0.18808E+01_JPRB, 0.15635E+01_JPRB, 0.13830E+01_JPRB, 0.12382E+01_JPRB, 0.88803E+00_JPRB, 0.70408E+00_JPRB, 0.59650E+00_JPRB ,&
&  0.54184E+00_JPRB, 0.55765E+00_JPRB, 0.59744E+00_JPRB, 0.60802E+00_JPRB, 0.60369E+00_JPRB, 0.63032E+00_JPRB, 0.67192E+00_JPRB ,&
&  0.69994E+00_JPRB, 0.72662E+00_JPRB, 0.78157E+00_JPRB, 0.80204E+00_JPRB, 0.85621E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.84486E+00_JPRB, 0.88806E+00_JPRB, 0.95436E+00_JPRB, 0.10544E+01_JPRB, 0.11651E+01_JPRB, 0.12545E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.11988E+01_JPRB, 0.12494E+01_JPRB, 0.13227E+01_JPRB, 0.14101E+01_JPRB, 0.15050E+01_JPRB, 0.16164E+01_JPRB, 0.17247E+01_JPRB ,&
&  0.18294E+01_JPRB, 0.19508E+01_JPRB, 0.20881E+01_JPRB, 0.22166E+01_JPRB, 0.22652E+01_JPRB, 0.21608E+01_JPRB, 0.15867E+01_JPRB ,&
&  0.12674E+01_JPRB, 0.21773E+01_JPRB, 0.18641E+01_JPRB, 0.19884E+01_JPRB, 0.21145E+01_JPRB, 0.22872E+01_JPRB, 0.18636E+01_JPRB ,&
&  0.14496E+01_JPRB, 0.14176E+01_JPRB, 0.14773E+01_JPRB, 0.14716E+01_JPRB, 0.15247E+01_JPRB, 0.16796E+01_JPRB, 0.18597E+01_JPRB ,&
&  0.19804E+01_JPRB, 0.21434E+01_JPRB, 0.23322E+01_JPRB, 0.25248E+01_JPRB, 0.26308E+01_JPRB, 0.27263E+01_JPRB, 0.28041E+01_JPRB ,&
&  0.28845E+01_JPRB, 0.29764E+01_JPRB, 0.30575E+01_JPRB, 0.33491E+01_JPRB, 0.36430E+01_JPRB, 0.40515E+01_JPRB, 0.45323E+01_JPRB ,&
&  0.47111E+01_JPRB, 0.48030E+01_JPRB, 0.52756E+01_JPRB, 0.52996E+01_JPRB, 0.52114E+01_JPRB, 0.53389E+01_JPRB, 0.44588E+01_JPRB ,&
&  0.46346E+01_JPRB, 0.42928E+01_JPRB, 0.46423E+01_JPRB, 0.38488E+01_JPRB, 0.37791E+01_JPRB, 0.40875E+01_JPRB, 0.40603E+01_JPRB ,&
&  0.36951E+01_JPRB, 0.37149E+01_JPRB, 0.33393E+01_JPRB, 0.29752E+01_JPRB, 0.28569E+01_JPRB, 0.28738E+01_JPRB, 0.30809E+01_JPRB ,&
&  0.25424E+01_JPRB, 0.14340E+01_JPRB, 0.10172E+01_JPRB, 0.80686E+00_JPRB, 0.76517E+00_JPRB, 0.98843E+00_JPRB, 0.15738E+01_JPRB ,&
&  0.24562E+01_JPRB, 0.25885E+01_JPRB, 0.26576E+01_JPRB, 0.19713E+01_JPRB, 0.13083E+01_JPRB, 0.10295E+01_JPRB, 0.82487E+00_JPRB ,&
&  0.62936E+00_JPRB, 0.60222E+00_JPRB, 0.65897E+00_JPRB, 0.68414E+00_JPRB, 0.68069E+00_JPRB, 0.71162E+00_JPRB, 0.73823E+00_JPRB ,&
&  0.72966E+00_JPRB, 0.72529E+00_JPRB, 0.76850E+00_JPRB, 0.77181E+00_JPRB, 0.80341E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.79679E+00_JPRB, 0.80544E+00_JPRB, 0.86109E+00_JPRB, 0.91647E+00_JPRB, 0.95923E+00_JPRB, 0.99179E+00_JPRB, 0.10206E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.11815E+01_JPRB, 0.12394E+01_JPRB, 0.12803E+01_JPRB, 0.13386E+01_JPRB, 0.14506E+01_JPRB, 0.15947E+01_JPRB ,&
&  0.17643E+01_JPRB, 0.19623E+01_JPRB, 0.22015E+01_JPRB, 0.24480E+01_JPRB, 0.26406E+01_JPRB, 0.25650E+01_JPRB, 0.16282E+01_JPRB ,&
&  0.15332E+01_JPRB, 0.20813E+01_JPRB, 0.20512E+01_JPRB, 0.20721E+01_JPRB, 0.20863E+01_JPRB, 0.21283E+01_JPRB, 0.17368E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.14984E+01_JPRB, 0.15285E+01_JPRB, 0.15914E+01_JPRB, 0.16631E+01_JPRB, 0.17661E+01_JPRB, 0.17912E+01_JPRB ,&
&  0.18063E+01_JPRB, 0.18952E+01_JPRB, 0.20018E+01_JPRB, 0.21959E+01_JPRB, 0.24107E+01_JPRB, 0.25794E+01_JPRB, 0.26756E+01_JPRB ,&
&  0.27314E+01_JPRB, 0.27781E+01_JPRB, 0.28993E+01_JPRB, 0.30607E+01_JPRB, 0.36174E+01_JPRB, 0.40329E+01_JPRB, 0.45862E+01_JPRB ,&
&  0.49212E+01_JPRB, 0.50660E+01_JPRB, 0.59709E+01_JPRB, 0.66748E+01_JPRB, 0.68927E+01_JPRB, 0.62221E+01_JPRB, 0.51535E+01_JPRB ,&
&  0.48122E+01_JPRB, 0.46585E+01_JPRB, 0.46423E+01_JPRB, 0.42004E+01_JPRB, 0.41662E+01_JPRB, 0.46757E+01_JPRB, 0.51232E+01_JPRB ,&
&  0.45667E+01_JPRB, 0.41668E+01_JPRB, 0.38741E+01_JPRB, 0.33040E+01_JPRB, 0.30733E+01_JPRB, 0.30452E+01_JPRB, 0.36019E+01_JPRB ,&
&  0.36928E+01_JPRB, 0.26051E+01_JPRB, 0.15027E+01_JPRB, 0.96317E+00_JPRB, 0.93043E+00_JPRB, 0.11514E+01_JPRB, 0.14563E+01_JPRB ,&
&  0.23388E+01_JPRB, 0.27941E+01_JPRB, 0.31399E+01_JPRB, 0.32421E+01_JPRB, 0.21752E+01_JPRB, 0.16951E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.96419E+00_JPRB, 0.78793E+00_JPRB, 0.77979E+00_JPRB, 0.77436E+00_JPRB, 0.75711E+00_JPRB, 0.80193E+00_JPRB, 0.86640E+00_JPRB ,&
&  0.82747E+00_JPRB, 0.72621E+00_JPRB, 0.72916E+00_JPRB, 0.73785E+00_JPRB, 0.76111E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.77392E+00_JPRB, 0.73872E+00_JPRB, 0.76117E+00_JPRB, 0.78167E+00_JPRB, 0.82242E+00_JPRB, 0.85756E+00_JPRB, 0.91789E+00_JPRB ,&
&  0.10146E+01_JPRB, 0.10921E+01_JPRB, 0.11445E+01_JPRB, 0.11473E+01_JPRB, 0.11316E+01_JPRB, 0.11822E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.14602E+01_JPRB, 0.17248E+01_JPRB, 0.20975E+01_JPRB, 0.24061E+01_JPRB, 0.25439E+01_JPRB, 0.22830E+01_JPRB, 0.21310E+01_JPRB ,&
&  0.21262E+01_JPRB, 0.24617E+01_JPRB, 0.25729E+01_JPRB, 0.26916E+01_JPRB, 0.26795E+01_JPRB, 0.25969E+01_JPRB, 0.18786E+01_JPRB ,&
&  0.18057E+01_JPRB, 0.17953E+01_JPRB, 0.17881E+01_JPRB, 0.18475E+01_JPRB, 0.18891E+01_JPRB, 0.19251E+01_JPRB, 0.18893E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.17110E+01_JPRB, 0.17113E+01_JPRB, 0.18061E+01_JPRB, 0.20382E+01_JPRB, 0.22607E+01_JPRB, 0.24874E+01_JPRB ,&
&  0.26799E+01_JPRB, 0.27701E+01_JPRB, 0.28635E+01_JPRB, 0.29064E+01_JPRB, 0.33886E+01_JPRB, 0.41424E+01_JPRB, 0.48256E+01_JPRB ,&
&  0.53523E+01_JPRB, 0.61982E+01_JPRB, 0.68953E+01_JPRB, 0.73963E+01_JPRB, 0.79045E+01_JPRB, 0.76149E+01_JPRB, 0.64556E+01_JPRB ,&
&  0.53411E+01_JPRB, 0.46634E+01_JPRB, 0.42248E+01_JPRB, 0.43109E+01_JPRB, 0.45717E+01_JPRB, 0.52192E+01_JPRB, 0.50141E+01_JPRB ,&
&  0.43454E+01_JPRB, 0.37496E+01_JPRB, 0.35418E+01_JPRB, 0.35663E+01_JPRB, 0.35925E+01_JPRB, 0.34417E+01_JPRB, 0.41951E+01_JPRB ,&
&  0.37619E+01_JPRB, 0.18194E+01_JPRB, 0.11756E+01_JPRB, 0.12398E+01_JPRB, 0.84383E+00_JPRB, 0.91497E+00_JPRB, 0.16552E+01_JPRB ,&
&  0.23457E+01_JPRB, 0.34763E+01_JPRB, 0.40132E+01_JPRB, 0.33253E+01_JPRB, 0.33019E+01_JPRB, 0.28175E+01_JPRB, 0.24442E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.11984E+01_JPRB, 0.92815E+00_JPRB, 0.86079E+00_JPRB, 0.82758E+00_JPRB, 0.84834E+00_JPRB, 0.86625E+00_JPRB ,&
&  0.82811E+00_JPRB, 0.75025E+00_JPRB, 0.74291E+00_JPRB, 0.75501E+00_JPRB, 0.76206E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.86114E+00_JPRB, 0.77742E+00_JPRB, 0.73310E+00_JPRB, 0.71141E+00_JPRB, 0.77998E+00_JPRB, 0.85712E+00_JPRB, 0.88296E+00_JPRB ,&
&  0.96993E+00_JPRB, 0.10528E+01_JPRB, 0.10870E+01_JPRB, 0.10787E+01_JPRB, 0.10442E+01_JPRB, 0.10430E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.11731E+01_JPRB, 0.13746E+01_JPRB, 0.17814E+01_JPRB, 0.22081E+01_JPRB, 0.24348E+01_JPRB, 0.23880E+01_JPRB, 0.25378E+01_JPRB ,&
&  0.24896E+01_JPRB, 0.29597E+01_JPRB, 0.37766E+01_JPRB, 0.40599E+01_JPRB, 0.32454E+01_JPRB, 0.28743E+01_JPRB, 0.22096E+01_JPRB ,&
&  0.21253E+01_JPRB, 0.21503E+01_JPRB, 0.21879E+01_JPRB, 0.22731E+01_JPRB, 0.22985E+01_JPRB, 0.22490E+01_JPRB, 0.21770E+01_JPRB ,&
&  0.19599E+01_JPRB, 0.18045E+01_JPRB, 0.16839E+01_JPRB, 0.16547E+01_JPRB, 0.17944E+01_JPRB, 0.20461E+01_JPRB, 0.23216E+01_JPRB ,&
&  0.26323E+01_JPRB, 0.28462E+01_JPRB, 0.29501E+01_JPRB, 0.32695E+01_JPRB, 0.27708E+01_JPRB, 0.41202E+01_JPRB, 0.52722E+01_JPRB ,&
&  0.60531E+01_JPRB, 0.68572E+01_JPRB, 0.70278E+01_JPRB, 0.84014E+01_JPRB, 0.92171E+01_JPRB, 0.85235E+01_JPRB, 0.73403E+01_JPRB ,&
&  0.62237E+01_JPRB, 0.48890E+01_JPRB, 0.45500E+01_JPRB, 0.47029E+01_JPRB, 0.47639E+01_JPRB, 0.53746E+01_JPRB, 0.39076E+01_JPRB ,&
&  0.37193E+01_JPRB, 0.45006E+01_JPRB, 0.41440E+01_JPRB, 0.34976E+01_JPRB, 0.37432E+01_JPRB, 0.36183E+01_JPRB, 0.32399E+01_JPRB ,&
&  0.14662E+01_JPRB, 0.92725E+00_JPRB, 0.86024E+00_JPRB, 0.77318E+00_JPRB, 0.67230E+00_JPRB, 0.92112E+00_JPRB, 0.29023E+01_JPRB ,&
&  0.24176E+01_JPRB, 0.41591E+01_JPRB, 0.47593E+01_JPRB, 0.41340E+01_JPRB, 0.39798E+01_JPRB, 0.39447E+01_JPRB, 0.41363E+01_JPRB ,&
&  0.26326E+01_JPRB, 0.17397E+01_JPRB, 0.11564E+01_JPRB, 0.96026E+00_JPRB, 0.93089E+00_JPRB, 0.90890E+00_JPRB, 0.86474E+00_JPRB ,&
&  0.82878E+00_JPRB, 0.80333E+00_JPRB, 0.80921E+00_JPRB, 0.79282E+00_JPRB, 0.85044E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.10078E+01_JPRB, 0.90739E+00_JPRB, 0.81279E+00_JPRB, 0.80052E+00_JPRB, 0.86202E+00_JPRB, 0.89196E+00_JPRB, 0.91365E+00_JPRB ,&
&  0.96812E+00_JPRB, 0.10400E+01_JPRB, 0.10748E+01_JPRB, 0.10521E+01_JPRB, 0.10259E+01_JPRB, 0.10164E+01_JPRB, 0.10133E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.11736E+01_JPRB, 0.15060E+01_JPRB, 0.17575E+01_JPRB, 0.20975E+01_JPRB, 0.21375E+01_JPRB, 0.25652E+01_JPRB ,&
&  0.26909E+01_JPRB, 0.30956E+01_JPRB, 0.38016E+01_JPRB, 0.39580E+01_JPRB, 0.41748E+01_JPRB, 0.34060E+01_JPRB, 0.30116E+01_JPRB ,&
&  0.25075E+01_JPRB, 0.23336E+01_JPRB, 0.23012E+01_JPRB, 0.24619E+01_JPRB, 0.26171E+01_JPRB, 0.26263E+01_JPRB, 0.24801E+01_JPRB ,&
&  0.21217E+01_JPRB, 0.19164E+01_JPRB, 0.16697E+01_JPRB, 0.16105E+01_JPRB, 0.15852E+01_JPRB, 0.17257E+01_JPRB, 0.20337E+01_JPRB ,&
&  0.25502E+01_JPRB, 0.30271E+01_JPRB, 0.31175E+01_JPRB, 0.33000E+01_JPRB, 0.32524E+01_JPRB, 0.35271E+01_JPRB, 0.53387E+01_JPRB ,&
&  0.69669E+01_JPRB, 0.82523E+01_JPRB, 0.86857E+01_JPRB, 0.10580E+02_JPRB, 0.10263E+02_JPRB, 0.91287E+01_JPRB, 0.84659E+01_JPRB ,&
&  0.70710E+01_JPRB, 0.62192E+01_JPRB, 0.48881E+01_JPRB, 0.47812E+01_JPRB, 0.48084E+01_JPRB, 0.39255E+01_JPRB, 0.38466E+01_JPRB ,&
&  0.46591E+01_JPRB, 0.45301E+01_JPRB, 0.40500E+01_JPRB, 0.33500E+01_JPRB, 0.39198E+01_JPRB, 0.31413E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.12621E+01_JPRB, 0.10911E+01_JPRB, 0.84939E+00_JPRB, 0.73858E+00_JPRB, 0.71566E+00_JPRB, 0.94898E+00_JPRB, 0.18558E+01_JPRB ,&
&  0.25556E+01_JPRB, 0.48271E+01_JPRB, 0.55100E+01_JPRB, 0.48794E+01_JPRB, 0.42770E+01_JPRB, 0.49048E+01_JPRB, 0.47082E+01_JPRB ,&
&  0.39637E+01_JPRB, 0.29849E+01_JPRB, 0.14908E+01_JPRB, 0.10915E+01_JPRB, 0.10115E+01_JPRB, 0.96929E+00_JPRB, 0.91810E+00_JPRB ,&
&  0.86551E+00_JPRB, 0.86273E+00_JPRB, 0.88888E+00_JPRB, 0.91139E+00_JPRB, 0.92905E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.10686E+01_JPRB, 0.10085E+01_JPRB, 0.91222E+00_JPRB, 0.91195E+00_JPRB, 0.94802E+00_JPRB, 0.99718E+00_JPRB, 0.99667E+00_JPRB ,&
&  0.10115E+01_JPRB, 0.10410E+01_JPRB, 0.10715E+01_JPRB, 0.10539E+01_JPRB, 0.10101E+01_JPRB, 0.10121E+01_JPRB, 0.10044E+01_JPRB ,&
&  0.10417E+01_JPRB, 0.11782E+01_JPRB, 0.13397E+01_JPRB, 0.14048E+01_JPRB, 0.15251E+01_JPRB, 0.16423E+01_JPRB, 0.20951E+01_JPRB ,&
&  0.26653E+01_JPRB, 0.30575E+01_JPRB, 0.35184E+01_JPRB, 0.40802E+01_JPRB, 0.46185E+01_JPRB, 0.40635E+01_JPRB, 0.40101E+01_JPRB ,&
&  0.33553E+01_JPRB, 0.28227E+01_JPRB, 0.26475E+01_JPRB, 0.26208E+01_JPRB, 0.27804E+01_JPRB, 0.28612E+01_JPRB, 0.27468E+01_JPRB ,&
&  0.23658E+01_JPRB, 0.19560E+01_JPRB, 0.17138E+01_JPRB, 0.15588E+01_JPRB, 0.14814E+01_JPRB, 0.15464E+01_JPRB, 0.18468E+01_JPRB ,&
&  0.25541E+01_JPRB, 0.33430E+01_JPRB, 0.35827E+01_JPRB, 0.34927E+01_JPRB, 0.43216E+01_JPRB, 0.48474E+01_JPRB, 0.58448E+01_JPRB ,&
&  0.65892E+01_JPRB, 0.71918E+01_JPRB, 0.10383E+02_JPRB, 0.11258E+02_JPRB, 0.10745E+02_JPRB, 0.99419E+01_JPRB, 0.87670E+01_JPRB ,&
&  0.76019E+01_JPRB, 0.67235E+01_JPRB, 0.53874E+01_JPRB, 0.52110E+01_JPRB, 0.43939E+01_JPRB, 0.37314E+01_JPRB, 0.45221E+01_JPRB ,&
&  0.51334E+01_JPRB, 0.49882E+01_JPRB, 0.41854E+01_JPRB, 0.33968E+01_JPRB, 0.31948E+01_JPRB, 0.21258E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.10905E+01_JPRB, 0.90746E+00_JPRB, 0.99174E+00_JPRB, 0.88926E+00_JPRB, 0.10852E+01_JPRB, 0.17126E+01_JPRB ,&
&  0.32117E+01_JPRB, 0.48600E+01_JPRB, 0.53352E+01_JPRB, 0.47056E+01_JPRB, 0.43130E+01_JPRB, 0.43718E+01_JPRB, 0.56076E+01_JPRB ,&
&  0.42621E+01_JPRB, 0.39223E+01_JPRB, 0.24945E+01_JPRB, 0.15237E+01_JPRB, 0.11739E+01_JPRB, 0.10447E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.10004E+01_JPRB, 0.10114E+01_JPRB, 0.10259E+01_JPRB, 0.10227E+01_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.10805E+01_JPRB, 0.10781E+01_JPRB, 0.10225E+01_JPRB, 0.10261E+01_JPRB, 0.10648E+01_JPRB, 0.10958E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10674E+01_JPRB, 0.10630E+01_JPRB, 0.10316E+01_JPRB, 0.10026E+01_JPRB, 0.10002E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.10629E+01_JPRB, 0.12698E+01_JPRB, 0.10709E+01_JPRB, 0.10559E+01_JPRB, 0.11770E+01_JPRB, 0.13450E+01_JPRB, 0.15834E+01_JPRB ,&
&  0.23679E+01_JPRB, 0.30100E+01_JPRB, 0.34716E+01_JPRB, 0.43861E+01_JPRB, 0.57205E+01_JPRB, 0.54438E+01_JPRB, 0.51348E+01_JPRB ,&
&  0.44022E+01_JPRB, 0.36554E+01_JPRB, 0.33836E+01_JPRB, 0.31950E+01_JPRB, 0.31557E+01_JPRB, 0.31239E+01_JPRB, 0.29323E+01_JPRB ,&
&  0.25405E+01_JPRB, 0.20254E+01_JPRB, 0.16873E+01_JPRB, 0.16304E+01_JPRB, 0.15609E+01_JPRB, 0.16182E+01_JPRB, 0.18265E+01_JPRB ,&
&  0.25987E+01_JPRB, 0.35821E+01_JPRB, 0.41263E+01_JPRB, 0.43726E+01_JPRB, 0.42988E+01_JPRB, 0.54186E+01_JPRB, 0.88697E+01_JPRB ,&
&  0.90368E+01_JPRB, 0.97911E+01_JPRB, 0.10942E+02_JPRB, 0.12807E+02_JPRB, 0.11693E+02_JPRB, 0.94739E+01_JPRB, 0.90066E+01_JPRB ,&
&  0.63643E+01_JPRB, 0.51785E+01_JPRB, 0.53696E+01_JPRB, 0.52188E+01_JPRB, 0.43587E+01_JPRB, 0.48444E+01_JPRB, 0.71409E+01_JPRB ,&
&  0.56994E+01_JPRB, 0.60492E+01_JPRB, 0.61616E+01_JPRB, 0.60721E+01_JPRB, 0.33805E+01_JPRB, 0.24915E+01_JPRB, 0.37409E+01_JPRB ,&
&  0.40136E+01_JPRB, 0.23601E+01_JPRB, 0.14129E+01_JPRB, 0.20143E+01_JPRB, 0.13960E+01_JPRB, 0.15847E+01_JPRB, 0.22777E+01_JPRB ,&
&  0.28804E+01_JPRB, 0.37035E+01_JPRB, 0.57463E+01_JPRB, 0.51651E+01_JPRB, 0.45872E+01_JPRB, 0.44990E+01_JPRB, 0.45181E+01_JPRB ,&
&  0.46905E+01_JPRB, 0.39445E+01_JPRB, 0.32376E+01_JPRB, 0.21322E+01_JPRB, 0.15342E+01_JPRB, 0.13281E+01_JPRB, 0.12491E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.12349E+01_JPRB, 0.12223E+01_JPRB, 0.11690E+01_JPRB, 0.11128E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.12159E+01_JPRB, 0.12050E+01_JPRB, 0.11520E+01_JPRB, 0.11669E+01_JPRB, 0.12121E+01_JPRB, 0.11808E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.10981E+01_JPRB, 0.10677E+01_JPRB, 0.10691E+01_JPRB, 0.10570E+01_JPRB, 0.10312E+01_JPRB, 0.10201E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10727E+01_JPRB, 0.11174E+01_JPRB, 0.10136E+01_JPRB, 0.98359E+00_JPRB, 0.11104E+01_JPRB, 0.12022E+01_JPRB, 0.14605E+01_JPRB ,&
&  0.21325E+01_JPRB, 0.28889E+01_JPRB, 0.37062E+01_JPRB, 0.46908E+01_JPRB, 0.54510E+01_JPRB, 0.60812E+01_JPRB, 0.63102E+01_JPRB ,&
&  0.60770E+01_JPRB, 0.50841E+01_JPRB, 0.44731E+01_JPRB, 0.40178E+01_JPRB, 0.36176E+01_JPRB, 0.32940E+01_JPRB, 0.30243E+01_JPRB ,&
&  0.27842E+01_JPRB, 0.23765E+01_JPRB, 0.19402E+01_JPRB, 0.16913E+01_JPRB, 0.16674E+01_JPRB, 0.16835E+01_JPRB, 0.18438E+01_JPRB ,&
&  0.25236E+01_JPRB, 0.34653E+01_JPRB, 0.45701E+01_JPRB, 0.54496E+01_JPRB, 0.51060E+01_JPRB, 0.59772E+01_JPRB, 0.83902E+01_JPRB ,&
&  0.97719E+01_JPRB, 0.10265E+02_JPRB, 0.12251E+02_JPRB, 0.13238E+02_JPRB, 0.13105E+02_JPRB, 0.10618E+02_JPRB, 0.10488E+02_JPRB ,&
&  0.75530E+01_JPRB, 0.58972E+01_JPRB, 0.51623E+01_JPRB, 0.46650E+01_JPRB, 0.54148E+01_JPRB, 0.76417E+01_JPRB, 0.71397E+01_JPRB ,&
&  0.68517E+01_JPRB, 0.65679E+01_JPRB, 0.60437E+01_JPRB, 0.51594E+01_JPRB, 0.48118E+01_JPRB, 0.42651E+01_JPRB, 0.53461E+01_JPRB ,&
&  0.52485E+01_JPRB, 0.49667E+01_JPRB, 0.40992E+01_JPRB, 0.32902E+01_JPRB, 0.25696E+01_JPRB, 0.28140E+01_JPRB, 0.28301E+01_JPRB ,&
&  0.29310E+01_JPRB, 0.30718E+01_JPRB, 0.47011E+01_JPRB, 0.49836E+01_JPRB, 0.45328E+01_JPRB, 0.40620E+01_JPRB, 0.42404E+01_JPRB ,&
&  0.38540E+01_JPRB, 0.40389E+01_JPRB, 0.34159E+01_JPRB, 0.24811E+01_JPRB, 0.19699E+01_JPRB, 0.17678E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.16061E+01_JPRB, 0.15820E+01_JPRB, 0.15091E+01_JPRB, 0.13976E+01_JPRB, 0.13047E+01_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.13870E+01_JPRB, 0.13460E+01_JPRB, 0.13027E+01_JPRB, 0.12900E+01_JPRB, 0.13103E+01_JPRB, 0.12692E+01_JPRB, 0.12223E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11211E+01_JPRB, 0.11079E+01_JPRB, 0.10866E+01_JPRB, 0.10565E+01_JPRB, 0.10376E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.11410E+01_JPRB, 0.10211E+01_JPRB, 0.10642E+01_JPRB, 0.11054E+01_JPRB, 0.11918E+01_JPRB, 0.15996E+01_JPRB ,&
&  0.20498E+01_JPRB, 0.26136E+01_JPRB, 0.33828E+01_JPRB, 0.42745E+01_JPRB, 0.50024E+01_JPRB, 0.58365E+01_JPRB, 0.66912E+01_JPRB ,&
&  0.62744E+01_JPRB, 0.61868E+01_JPRB, 0.53908E+01_JPRB, 0.45790E+01_JPRB, 0.38951E+01_JPRB, 0.33778E+01_JPRB, 0.31122E+01_JPRB ,&
&  0.29006E+01_JPRB, 0.27582E+01_JPRB, 0.24551E+01_JPRB, 0.21051E+01_JPRB, 0.19301E+01_JPRB, 0.18793E+01_JPRB, 0.19227E+01_JPRB ,&
&  0.21557E+01_JPRB, 0.30859E+01_JPRB, 0.46001E+01_JPRB, 0.63983E+01_JPRB, 0.58900E+01_JPRB, 0.69704E+01_JPRB, 0.80298E+01_JPRB ,&
&  0.11996E+02_JPRB, 0.13133E+02_JPRB, 0.13359E+02_JPRB, 0.13737E+02_JPRB, 0.10728E+02_JPRB, 0.10811E+02_JPRB, 0.10816E+02_JPRB ,&
&  0.94088E+01_JPRB, 0.86090E+01_JPRB, 0.91140E+01_JPRB, 0.72350E+01_JPRB, 0.62789E+01_JPRB, 0.75051E+01_JPRB, 0.63769E+01_JPRB ,&
&  0.58164E+01_JPRB, 0.55538E+01_JPRB, 0.51082E+01_JPRB, 0.54248E+01_JPRB, 0.49906E+01_JPRB, 0.49645E+01_JPRB, 0.40880E+01_JPRB ,&
&  0.39164E+01_JPRB, 0.32437E+01_JPRB, 0.35819E+01_JPRB, 0.31446E+01_JPRB, 0.29279E+01_JPRB, 0.28054E+01_JPRB, 0.26904E+01_JPRB ,&
&  0.28824E+01_JPRB, 0.29453E+01_JPRB, 0.35328E+01_JPRB, 0.40427E+01_JPRB, 0.41357E+01_JPRB, 0.34504E+01_JPRB, 0.33118E+01_JPRB ,&
&  0.33995E+01_JPRB, 0.35723E+01_JPRB, 0.31402E+01_JPRB, 0.25567E+01_JPRB, 0.21771E+01_JPRB, 0.20415E+01_JPRB, 0.19628E+01_JPRB ,&
&  0.18550E+01_JPRB, 0.17528E+01_JPRB, 0.16618E+01_JPRB, 0.15908E+01_JPRB, 0.15124E+01_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.14305E+01_JPRB, 0.13647E+01_JPRB, 0.14120E+01_JPRB, 0.13801E+01_JPRB, 0.13796E+01_JPRB, 0.13317E+01_JPRB, 0.13041E+01_JPRB ,&
&  0.12642E+01_JPRB, 0.12185E+01_JPRB, 0.11703E+01_JPRB, 0.11212E+01_JPRB, 0.10793E+01_JPRB, 0.10505E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.12501E+01_JPRB, 0.11404E+01_JPRB, 0.10745E+01_JPRB, 0.10778E+01_JPRB, 0.12900E+01_JPRB, 0.16248E+01_JPRB ,&
&  0.19605E+01_JPRB, 0.22802E+01_JPRB, 0.28831E+01_JPRB, 0.35074E+01_JPRB, 0.41927E+01_JPRB, 0.48993E+01_JPRB, 0.57616E+01_JPRB ,&
&  0.64096E+01_JPRB, 0.54267E+01_JPRB, 0.50853E+01_JPRB, 0.44607E+01_JPRB, 0.39363E+01_JPRB, 0.34835E+01_JPRB, 0.31736E+01_JPRB ,&
&  0.29236E+01_JPRB, 0.27536E+01_JPRB, 0.25353E+01_JPRB, 0.22781E+01_JPRB, 0.20284E+01_JPRB, 0.18971E+01_JPRB, 0.18728E+01_JPRB ,&
&  0.19759E+01_JPRB, 0.24495E+01_JPRB, 0.36963E+01_JPRB, 0.56992E+01_JPRB, 0.87359E+01_JPRB, 0.10043E+02_JPRB, 0.10119E+02_JPRB ,&
&  0.91615E+01_JPRB, 0.97674E+01_JPRB, 0.15448E+02_JPRB, 0.12396E+02_JPRB, 0.11955E+02_JPRB, 0.12146E+02_JPRB, 0.11075E+02_JPRB ,&
&  0.97220E+01_JPRB, 0.79406E+01_JPRB, 0.74714E+01_JPRB, 0.64810E+01_JPRB, 0.72802E+01_JPRB, 0.62331E+01_JPRB, 0.53762E+01_JPRB ,&
&  0.48728E+01_JPRB, 0.48805E+01_JPRB, 0.46930E+01_JPRB, 0.46425E+01_JPRB, 0.48514E+01_JPRB, 0.55385E+01_JPRB, 0.55093E+01_JPRB ,&
&  0.48392E+01_JPRB, 0.49514E+01_JPRB, 0.40292E+01_JPRB, 0.31654E+01_JPRB, 0.25434E+01_JPRB, 0.25118E+01_JPRB, 0.25527E+01_JPRB ,&
&  0.26597E+01_JPRB, 0.28224E+01_JPRB, 0.32288E+01_JPRB, 0.34466E+01_JPRB, 0.34499E+01_JPRB, 0.31884E+01_JPRB, 0.29367E+01_JPRB ,&
&  0.27766E+01_JPRB, 0.30093E+01_JPRB, 0.30483E+01_JPRB, 0.24995E+01_JPRB, 0.22293E+01_JPRB, 0.21240E+01_JPRB, 0.20374E+01_JPRB ,&
&  0.18955E+01_JPRB, 0.17486E+01_JPRB, 0.16503E+01_JPRB, 0.16004E+01_JPRB, 0.15546E+01_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.14925E+01_JPRB, 0.13690E+01_JPRB, 0.14114E+01_JPRB, 0.13988E+01_JPRB, 0.13699E+01_JPRB, 0.13319E+01_JPRB, 0.12994E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.12359E+01_JPRB, 0.12044E+01_JPRB, 0.11511E+01_JPRB, 0.11057E+01_JPRB, 0.10759E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10678E+01_JPRB, 0.13080E+01_JPRB, 0.12649E+01_JPRB, 0.12952E+01_JPRB, 0.12730E+01_JPRB, 0.14254E+01_JPRB, 0.15726E+01_JPRB ,&
&  0.17904E+01_JPRB, 0.19664E+01_JPRB, 0.22143E+01_JPRB, 0.26011E+01_JPRB, 0.30327E+01_JPRB, 0.34853E+01_JPRB, 0.41555E+01_JPRB ,&
&  0.43650E+01_JPRB, 0.43094E+01_JPRB, 0.40727E+01_JPRB, 0.41546E+01_JPRB, 0.39693E+01_JPRB, 0.34097E+01_JPRB, 0.31893E+01_JPRB ,&
&  0.28443E+01_JPRB, 0.26049E+01_JPRB, 0.24174E+01_JPRB, 0.23027E+01_JPRB, 0.21988E+01_JPRB, 0.20705E+01_JPRB, 0.19900E+01_JPRB ,&
&  0.19205E+01_JPRB, 0.20283E+01_JPRB, 0.26218E+01_JPRB, 0.41261E+01_JPRB, 0.68183E+01_JPRB, 0.92081E+01_JPRB, 0.99552E+01_JPRB ,&
&  0.11110E+02_JPRB, 0.10955E+02_JPRB, 0.88852E+01_JPRB, 0.10131E+02_JPRB, 0.12846E+02_JPRB, 0.11690E+02_JPRB, 0.10123E+02_JPRB ,&
&  0.99139E+01_JPRB, 0.91205E+01_JPRB, 0.86690E+01_JPRB, 0.75013E+01_JPRB, 0.63399E+01_JPRB, 0.56857E+01_JPRB, 0.51986E+01_JPRB ,&
&  0.48436E+01_JPRB, 0.47902E+01_JPRB, 0.48169E+01_JPRB, 0.48165E+01_JPRB, 0.45740E+01_JPRB, 0.46068E+01_JPRB, 0.50077E+01_JPRB ,&
&  0.49211E+01_JPRB, 0.44627E+01_JPRB, 0.32662E+01_JPRB, 0.31534E+01_JPRB, 0.22619E+01_JPRB, 0.21600E+01_JPRB, 0.24256E+01_JPRB ,&
&  0.23511E+01_JPRB, 0.26419E+01_JPRB, 0.29061E+01_JPRB, 0.26354E+01_JPRB, 0.26782E+01_JPRB, 0.26043E+01_JPRB, 0.26522E+01_JPRB ,&
&  0.26228E+01_JPRB, 0.23341E+01_JPRB, 0.26215E+01_JPRB, 0.23069E+01_JPRB, 0.21251E+01_JPRB, 0.20607E+01_JPRB, 0.19904E+01_JPRB ,&
&  0.19221E+01_JPRB, 0.18253E+01_JPRB, 0.17294E+01_JPRB, 0.16769E+01_JPRB, 0.16168E+01_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.15217E+01_JPRB, 0.13932E+01_JPRB, 0.13825E+01_JPRB, 0.13490E+01_JPRB, 0.13088E+01_JPRB, 0.12707E+01_JPRB, 0.12433E+01_JPRB ,&
&  0.12243E+01_JPRB, 0.12046E+01_JPRB, 0.11800E+01_JPRB, 0.11416E+01_JPRB, 0.11073E+01_JPRB, 0.10724E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.11921E+01_JPRB, 0.12386E+01_JPRB, 0.12004E+01_JPRB, 0.13918E+01_JPRB, 0.13975E+01_JPRB, 0.14622E+01_JPRB ,&
&  0.15442E+01_JPRB, 0.16890E+01_JPRB, 0.17324E+01_JPRB, 0.17999E+01_JPRB, 0.19472E+01_JPRB, 0.22766E+01_JPRB, 0.26999E+01_JPRB ,&
&  0.28430E+01_JPRB, 0.27383E+01_JPRB, 0.27351E+01_JPRB, 0.28825E+01_JPRB, 0.30850E+01_JPRB, 0.27397E+01_JPRB, 0.26290E+01_JPRB ,&
&  0.24044E+01_JPRB, 0.22204E+01_JPRB, 0.20866E+01_JPRB, 0.20015E+01_JPRB, 0.19838E+01_JPRB, 0.20027E+01_JPRB, 0.20383E+01_JPRB ,&
&  0.20338E+01_JPRB, 0.19612E+01_JPRB, 0.20984E+01_JPRB, 0.30746E+01_JPRB, 0.59420E+01_JPRB, 0.91443E+01_JPRB, 0.11426E+02_JPRB ,&
&  0.11795E+02_JPRB, 0.11587E+02_JPRB, 0.12055E+02_JPRB, 0.13124E+02_JPRB, 0.11976E+02_JPRB, 0.10390E+02_JPRB, 0.94312E+01_JPRB ,&
&  0.88228E+01_JPRB, 0.84930E+01_JPRB, 0.84987E+01_JPRB, 0.78447E+01_JPRB, 0.69538E+01_JPRB, 0.64684E+01_JPRB, 0.60212E+01_JPRB ,&
&  0.56552E+01_JPRB, 0.56468E+01_JPRB, 0.56820E+01_JPRB, 0.53735E+01_JPRB, 0.51745E+01_JPRB, 0.48374E+01_JPRB, 0.50591E+01_JPRB ,&
&  0.55111E+01_JPRB, 0.40014E+01_JPRB, 0.32063E+01_JPRB, 0.36034E+01_JPRB, 0.26314E+01_JPRB, 0.25320E+01_JPRB, 0.29184E+01_JPRB ,&
&  0.26391E+01_JPRB, 0.24776E+01_JPRB, 0.25219E+01_JPRB, 0.22468E+01_JPRB, 0.22099E+01_JPRB, 0.23312E+01_JPRB, 0.22947E+01_JPRB ,&
&  0.22170E+01_JPRB, 0.21169E+01_JPRB, 0.23746E+01_JPRB, 0.21012E+01_JPRB, 0.19444E+01_JPRB, 0.18958E+01_JPRB, 0.18300E+01_JPRB ,&
&  0.17762E+01_JPRB, 0.17146E+01_JPRB, 0.16402E+01_JPRB, 0.15969E+01_JPRB, 0.15854E+01_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.14245E+01_JPRB, 0.13822E+01_JPRB, 0.13549E+01_JPRB, 0.13400E+01_JPRB, 0.13102E+01_JPRB, 0.12738E+01_JPRB, 0.12387E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.11768E+01_JPRB, 0.11382E+01_JPRB, 0.10939E+01_JPRB, 0.10590E+01_JPRB, 0.10243E+01_JPRB, 0.99949E+00_JPRB ,&
&  0.89899E+00_JPRB, 0.92934E+00_JPRB, 0.94789E+00_JPRB, 0.10224E+01_JPRB, 0.12741E+01_JPRB, 0.12455E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.13367E+01_JPRB, 0.14005E+01_JPRB, 0.14161E+01_JPRB, 0.14537E+01_JPRB, 0.15414E+01_JPRB, 0.17367E+01_JPRB, 0.19811E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.17652E+01_JPRB, 0.17215E+01_JPRB, 0.17066E+01_JPRB, 0.17933E+01_JPRB, 0.18842E+01_JPRB, 0.19965E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.18644E+01_JPRB, 0.18054E+01_JPRB, 0.17650E+01_JPRB, 0.17364E+01_JPRB, 0.17359E+01_JPRB, 0.17658E+01_JPRB ,&
&  0.18068E+01_JPRB, 0.18025E+01_JPRB, 0.18400E+01_JPRB, 0.23814E+01_JPRB, 0.49789E+01_JPRB, 0.87134E+01_JPRB, 0.12072E+02_JPRB ,&
&  0.11919E+02_JPRB, 0.10757E+02_JPRB, 0.11138E+02_JPRB, 0.12248E+02_JPRB, 0.10965E+02_JPRB, 0.92307E+01_JPRB, 0.78695E+01_JPRB ,&
&  0.72480E+01_JPRB, 0.68123E+01_JPRB, 0.69539E+01_JPRB, 0.70759E+01_JPRB, 0.67468E+01_JPRB, 0.66539E+01_JPRB, 0.64768E+01_JPRB ,&
&  0.62575E+01_JPRB, 0.62207E+01_JPRB, 0.60084E+01_JPRB, 0.57855E+01_JPRB, 0.56546E+01_JPRB, 0.58153E+01_JPRB, 0.57601E+01_JPRB ,&
&  0.58626E+01_JPRB, 0.55564E+01_JPRB, 0.47697E+01_JPRB, 0.46421E+01_JPRB, 0.35829E+01_JPRB, 0.33880E+01_JPRB, 0.39884E+01_JPRB ,&
&  0.32729E+01_JPRB, 0.26643E+01_JPRB, 0.24226E+01_JPRB, 0.23260E+01_JPRB, 0.22859E+01_JPRB, 0.23664E+01_JPRB, 0.22301E+01_JPRB ,&
&  0.19798E+01_JPRB, 0.20687E+01_JPRB, 0.21726E+01_JPRB, 0.18586E+01_JPRB, 0.16960E+01_JPRB, 0.16122E+01_JPRB, 0.14909E+01_JPRB ,&
&  0.15099E+01_JPRB, 0.14723E+01_JPRB, 0.14433E+01_JPRB, 0.14209E+01_JPRB, 0.14223E+01_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.12918E+01_JPRB, 0.13020E+01_JPRB, 0.13024E+01_JPRB, 0.12944E+01_JPRB, 0.12698E+01_JPRB, 0.12350E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.11608E+01_JPRB, 0.11123E+01_JPRB, 0.10649E+01_JPRB, 0.10199E+01_JPRB, 0.99184E+00_JPRB, 0.97006E+00_JPRB, 0.89424E+00_JPRB ,&
&  0.81636E+00_JPRB, 0.83158E+00_JPRB, 0.83233E+00_JPRB, 0.86992E+00_JPRB, 0.95263E+00_JPRB, 0.98736E+00_JPRB, 0.10307E+01_JPRB ,&
&  0.11375E+01_JPRB, 0.12201E+01_JPRB, 0.12743E+01_JPRB, 0.13293E+01_JPRB, 0.13669E+01_JPRB, 0.14112E+01_JPRB, 0.14640E+01_JPRB ,&
&  0.13993E+01_JPRB, 0.12728E+01_JPRB, 0.11870E+01_JPRB, 0.11291E+01_JPRB, 0.11921E+01_JPRB, 0.13000E+01_JPRB, 0.13829E+01_JPRB ,&
&  0.14227E+01_JPRB, 0.14342E+01_JPRB, 0.14456E+01_JPRB, 0.14478E+01_JPRB, 0.14351E+01_JPRB, 0.14322E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.14970E+01_JPRB, 0.15439E+01_JPRB, 0.17944E+01_JPRB, 0.36977E+01_JPRB, 0.65294E+01_JPRB, 0.83213E+01_JPRB ,&
&  0.75529E+01_JPRB, 0.73640E+01_JPRB, 0.74975E+01_JPRB, 0.77036E+01_JPRB, 0.73942E+01_JPRB, 0.65788E+01_JPRB, 0.59049E+01_JPRB ,&
&  0.57823E+01_JPRB, 0.54474E+01_JPRB, 0.58047E+01_JPRB, 0.57133E+01_JPRB, 0.57199E+01_JPRB, 0.58014E+01_JPRB, 0.56461E+01_JPRB ,&
&  0.56274E+01_JPRB, 0.54502E+01_JPRB, 0.54534E+01_JPRB, 0.53287E+01_JPRB, 0.53189E+01_JPRB, 0.54357E+01_JPRB, 0.54670E+01_JPRB ,&
&  0.55345E+01_JPRB, 0.55498E+01_JPRB, 0.51465E+01_JPRB, 0.48399E+01_JPRB, 0.44913E+01_JPRB, 0.47017E+01_JPRB, 0.42505E+01_JPRB ,&
&  0.35184E+01_JPRB, 0.26492E+01_JPRB, 0.25394E+01_JPRB, 0.23577E+01_JPRB, 0.22503E+01_JPRB, 0.21436E+01_JPRB, 0.19819E+01_JPRB ,&
&  0.19900E+01_JPRB, 0.21110E+01_JPRB, 0.19713E+01_JPRB, 0.16778E+01_JPRB, 0.15305E+01_JPRB, 0.14056E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.12706E+01_JPRB, 0.12887E+01_JPRB, 0.12813E+01_JPRB, 0.12734E+01_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.12325E+01_JPRB, 0.12399E+01_JPRB, 0.12363E+01_JPRB, 0.12305E+01_JPRB, 0.12153E+01_JPRB, 0.11816E+01_JPRB, 0.11186E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.10716E+01_JPRB, 0.10219E+01_JPRB, 0.98101E+00_JPRB, 0.95790E+00_JPRB, 0.88331E+00_JPRB, 0.77274E+00_JPRB ,&
&  0.73874E+00_JPRB, 0.75189E+00_JPRB, 0.77901E+00_JPRB, 0.79945E+00_JPRB, 0.81928E+00_JPRB, 0.83316E+00_JPRB, 0.86817E+00_JPRB ,&
&  0.91574E+00_JPRB, 0.96154E+00_JPRB, 0.10194E+01_JPRB, 0.10761E+01_JPRB, 0.10853E+01_JPRB, 0.10817E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.98560E+00_JPRB, 0.94896E+00_JPRB, 0.87420E+00_JPRB, 0.93313E+00_JPRB, 0.97203E+00_JPRB, 0.10133E+01_JPRB ,&
&  0.10525E+01_JPRB, 0.10802E+01_JPRB, 0.11136E+01_JPRB, 0.11327E+01_JPRB, 0.11342E+01_JPRB, 0.11395E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.12097E+01_JPRB, 0.12788E+01_JPRB, 0.14120E+01_JPRB, 0.25556E+01_JPRB, 0.43067E+01_JPRB, 0.52052E+01_JPRB ,&
&  0.48551E+01_JPRB, 0.52076E+01_JPRB, 0.54352E+01_JPRB, 0.49332E+01_JPRB, 0.50529E+01_JPRB, 0.51552E+01_JPRB, 0.49201E+01_JPRB ,&
&  0.50786E+01_JPRB, 0.50049E+01_JPRB, 0.48593E+01_JPRB, 0.47017E+01_JPRB, 0.45611E+01_JPRB, 0.43904E+01_JPRB, 0.42540E+01_JPRB ,&
&  0.42763E+01_JPRB, 0.44696E+01_JPRB, 0.45829E+01_JPRB, 0.47913E+01_JPRB, 0.48870E+01_JPRB, 0.50338E+01_JPRB, 0.51571E+01_JPRB ,&
&  0.51930E+01_JPRB, 0.52191E+01_JPRB, 0.51525E+01_JPRB, 0.49225E+01_JPRB, 0.46639E+01_JPRB, 0.43848E+01_JPRB, 0.40600E+01_JPRB ,&
&  0.37604E+01_JPRB, 0.32398E+01_JPRB, 0.28259E+01_JPRB, 0.26166E+01_JPRB, 0.23868E+01_JPRB, 0.21007E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.19096E+01_JPRB, 0.17170E+01_JPRB, 0.16722E+01_JPRB, 0.14935E+01_JPRB, 0.13701E+01_JPRB, 0.12727E+01_JPRB, 0.11303E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11470E+01_JPRB, 0.12039E+01_JPRB, 0.12335E+01_JPRB, 0.12336E+01_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.11829E+01_JPRB, 0.11937E+01_JPRB, 0.11948E+01_JPRB, 0.11912E+01_JPRB, 0.11541E+01_JPRB, 0.10807E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.10399E+01_JPRB, 0.98963E+00_JPRB, 0.98050E+00_JPRB, 0.84492E+00_JPRB, 0.79386E+00_JPRB, 0.77497E+00_JPRB, 0.74058E+00_JPRB ,&
&  0.72416E+00_JPRB, 0.72494E+00_JPRB, 0.72585E+00_JPRB, 0.73330E+00_JPRB, 0.74407E+00_JPRB, 0.74616E+00_JPRB, 0.75182E+00_JPRB ,&
&  0.76734E+00_JPRB, 0.79123E+00_JPRB, 0.83347E+00_JPRB, 0.85874E+00_JPRB, 0.87745E+00_JPRB, 0.88655E+00_JPRB, 0.89472E+00_JPRB ,&
&  0.85216E+00_JPRB, 0.82967E+00_JPRB, 0.82948E+00_JPRB, 0.80934E+00_JPRB, 0.80656E+00_JPRB, 0.81870E+00_JPRB, 0.83968E+00_JPRB ,&
&  0.85773E+00_JPRB, 0.82381E+00_JPRB, 0.89254E+00_JPRB, 0.92793E+00_JPRB, 0.94983E+00_JPRB, 0.96475E+00_JPRB, 0.97830E+00_JPRB ,&
&  0.99456E+00_JPRB, 0.10270E+01_JPRB, 0.10796E+01_JPRB, 0.11849E+01_JPRB, 0.17531E+01_JPRB, 0.29822E+01_JPRB, 0.34596E+01_JPRB ,&
&  0.36063E+01_JPRB, 0.29385E+01_JPRB, 0.33889E+01_JPRB, 0.35510E+01_JPRB, 0.38100E+01_JPRB, 0.40322E+01_JPRB, 0.41178E+01_JPRB ,&
&  0.45030E+01_JPRB, 0.45745E+01_JPRB, 0.43163E+01_JPRB, 0.38723E+01_JPRB, 0.35079E+01_JPRB, 0.33296E+01_JPRB, 0.32561E+01_JPRB ,&
&  0.32591E+01_JPRB, 0.31252E+01_JPRB, 0.35308E+01_JPRB, 0.39220E+01_JPRB, 0.41657E+01_JPRB, 0.44080E+01_JPRB, 0.46983E+01_JPRB ,&
&  0.48860E+01_JPRB, 0.48801E+01_JPRB, 0.48236E+01_JPRB, 0.44719E+01_JPRB, 0.42849E+01_JPRB, 0.41389E+01_JPRB, 0.38916E+01_JPRB ,&
&  0.37216E+01_JPRB, 0.35665E+01_JPRB, 0.33472E+01_JPRB, 0.32918E+01_JPRB, 0.28259E+01_JPRB, 0.25056E+01_JPRB, 0.22777E+01_JPRB ,&
&  0.18744E+01_JPRB, 0.14568E+01_JPRB, 0.12865E+01_JPRB, 0.11972E+01_JPRB, 0.11335E+01_JPRB, 0.11166E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10804E+01_JPRB, 0.11430E+01_JPRB, 0.11585E+01_JPRB, 0.11673E+01_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.11487E+01_JPRB, 0.11764E+01_JPRB, 0.11933E+01_JPRB, 0.11803E+01_JPRB, 0.11197E+01_JPRB, 0.10245E+01_JPRB, 0.91918E+00_JPRB ,&
&  0.87237E+00_JPRB, 0.83193E+00_JPRB, 0.79878E+00_JPRB, 0.77027E+00_JPRB, 0.74941E+00_JPRB, 0.72098E+00_JPRB, 0.69187E+00_JPRB ,&
&  0.68636E+00_JPRB, 0.69682E+00_JPRB, 0.70873E+00_JPRB, 0.70788E+00_JPRB, 0.70476E+00_JPRB, 0.70272E+00_JPRB, 0.69996E+00_JPRB ,&
&  0.69582E+00_JPRB, 0.70017E+00_JPRB, 0.71796E+00_JPRB, 0.74539E+00_JPRB, 0.75852E+00_JPRB, 0.76070E+00_JPRB, 0.76450E+00_JPRB ,&
&  0.75515E+00_JPRB, 0.73781E+00_JPRB, 0.72702E+00_JPRB, 0.72854E+00_JPRB, 0.72696E+00_JPRB, 0.74127E+00_JPRB, 0.73791E+00_JPRB ,&
&  0.67888E+00_JPRB, 0.70075E+00_JPRB, 0.77882E+00_JPRB, 0.81598E+00_JPRB, 0.84207E+00_JPRB, 0.86089E+00_JPRB, 0.87782E+00_JPRB ,&
&  0.89391E+00_JPRB, 0.93066E+00_JPRB, 0.98136E+00_JPRB, 0.10561E+01_JPRB, 0.13595E+01_JPRB, 0.20520E+01_JPRB, 0.25030E+01_JPRB ,&
&  0.28834E+01_JPRB, 0.25507E+01_JPRB, 0.24168E+01_JPRB, 0.25288E+01_JPRB, 0.30206E+01_JPRB, 0.32799E+01_JPRB, 0.34983E+01_JPRB ,&
&  0.37159E+01_JPRB, 0.36809E+01_JPRB, 0.35477E+01_JPRB, 0.33185E+01_JPRB, 0.29489E+01_JPRB, 0.26959E+01_JPRB, 0.25852E+01_JPRB ,&
&  0.24607E+01_JPRB, 0.24185E+01_JPRB, 0.25286E+01_JPRB, 0.28550E+01_JPRB, 0.32629E+01_JPRB, 0.36082E+01_JPRB, 0.38813E+01_JPRB ,&
&  0.41679E+01_JPRB, 0.43126E+01_JPRB, 0.42494E+01_JPRB, 0.41134E+01_JPRB, 0.38819E+01_JPRB, 0.37389E+01_JPRB, 0.36788E+01_JPRB ,&
&  0.35959E+01_JPRB, 0.35139E+01_JPRB, 0.33511E+01_JPRB, 0.31344E+01_JPRB, 0.27925E+01_JPRB, 0.24755E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.12852E+01_JPRB, 0.10846E+01_JPRB, 0.10155E+01_JPRB, 0.99852E+00_JPRB, 0.10020E+01_JPRB, 0.98814E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10350E+01_JPRB, 0.10370E+01_JPRB, 0.10732E+01_JPRB, 0.11170E+01_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.10990E+01_JPRB, 0.11216E+01_JPRB, 0.11200E+01_JPRB, 0.11010E+01_JPRB, 0.10237E+01_JPRB, 0.94982E+00_JPRB, 0.87926E+00_JPRB ,&
&  0.83499E+00_JPRB, 0.79706E+00_JPRB, 0.76959E+00_JPRB, 0.74136E+00_JPRB, 0.71912E+00_JPRB, 0.70884E+00_JPRB, 0.70997E+00_JPRB ,&
&  0.71578E+00_JPRB, 0.71102E+00_JPRB, 0.70280E+00_JPRB, 0.68925E+00_JPRB, 0.67915E+00_JPRB, 0.67070E+00_JPRB, 0.66197E+00_JPRB ,&
&  0.65122E+00_JPRB, 0.65073E+00_JPRB, 0.65793E+00_JPRB, 0.67451E+00_JPRB, 0.68395E+00_JPRB, 0.68819E+00_JPRB, 0.68563E+00_JPRB ,&
&  0.67089E+00_JPRB, 0.65227E+00_JPRB, 0.65444E+00_JPRB, 0.66644E+00_JPRB, 0.67325E+00_JPRB, 0.67616E+00_JPRB, 0.64542E+00_JPRB ,&
&  0.62508E+00_JPRB, 0.62358E+00_JPRB, 0.64509E+00_JPRB, 0.72591E+00_JPRB, 0.76852E+00_JPRB, 0.79100E+00_JPRB, 0.80876E+00_JPRB ,&
&  0.79831E+00_JPRB, 0.81490E+00_JPRB, 0.86168E+00_JPRB, 0.96936E+00_JPRB, 0.11323E+01_JPRB, 0.14257E+01_JPRB, 0.18135E+01_JPRB ,&
&  0.21066E+01_JPRB, 0.23211E+01_JPRB, 0.24540E+01_JPRB, 0.22579E+01_JPRB, 0.22520E+01_JPRB, 0.26809E+01_JPRB, 0.28691E+01_JPRB ,&
&  0.29655E+01_JPRB, 0.29789E+01_JPRB, 0.30494E+01_JPRB, 0.28581E+01_JPRB, 0.24923E+01_JPRB, 0.22485E+01_JPRB, 0.20812E+01_JPRB ,&
&  0.19019E+01_JPRB, 0.17938E+01_JPRB, 0.17730E+01_JPRB, 0.18210E+01_JPRB, 0.19471E+01_JPRB, 0.22242E+01_JPRB, 0.25898E+01_JPRB ,&
&  0.29509E+01_JPRB, 0.31678E+01_JPRB, 0.32860E+01_JPRB, 0.32373E+01_JPRB, 0.31647E+01_JPRB, 0.30630E+01_JPRB, 0.29893E+01_JPRB ,&
&  0.29058E+01_JPRB, 0.28746E+01_JPRB, 0.28346E+01_JPRB, 0.27781E+01_JPRB, 0.24881E+01_JPRB, 0.20382E+01_JPRB, 0.16128E+01_JPRB ,&
&  0.13199E+01_JPRB, 0.11489E+01_JPRB, 0.10308E+01_JPRB, 0.99630E+00_JPRB, 0.98808E+00_JPRB, 0.97765E+00_JPRB, 0.92982E+00_JPRB ,&
&  0.90686E+00_JPRB, 0.92080E+00_JPRB, 0.95157E+00_JPRB, 0.98776E+00_JPRB, 0.10428E+01_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.10731E+01_JPRB, 0.10677E+01_JPRB, 0.10281E+01_JPRB, 0.97466E+00_JPRB, 0.92595E+00_JPRB, 0.85750E+00_JPRB, 0.80600E+00_JPRB ,&
&  0.77753E+00_JPRB, 0.76604E+00_JPRB, 0.76244E+00_JPRB, 0.75726E+00_JPRB, 0.75289E+00_JPRB, 0.74821E+00_JPRB, 0.73860E+00_JPRB ,&
&  0.72483E+00_JPRB, 0.71586E+00_JPRB, 0.71024E+00_JPRB, 0.69952E+00_JPRB, 0.68287E+00_JPRB, 0.66252E+00_JPRB, 0.64405E+00_JPRB ,&
&  0.63094E+00_JPRB, 0.62774E+00_JPRB, 0.64002E+00_JPRB, 0.65284E+00_JPRB, 0.66074E+00_JPRB, 0.65868E+00_JPRB, 0.64851E+00_JPRB ,&
&  0.62719E+00_JPRB, 0.61920E+00_JPRB, 0.62856E+00_JPRB, 0.63261E+00_JPRB, 0.63999E+00_JPRB, 0.63588E+00_JPRB, 0.61424E+00_JPRB ,&
&  0.59508E+00_JPRB, 0.57948E+00_JPRB, 0.57635E+00_JPRB, 0.58853E+00_JPRB, 0.64349E+00_JPRB, 0.71843E+00_JPRB, 0.74998E+00_JPRB ,&
&  0.77970E+00_JPRB, 0.80741E+00_JPRB, 0.84059E+00_JPRB, 0.88538E+00_JPRB, 0.95640E+00_JPRB, 0.10646E+01_JPRB, 0.12741E+01_JPRB ,&
&  0.15270E+01_JPRB, 0.17241E+01_JPRB, 0.19158E+01_JPRB, 0.17898E+01_JPRB, 0.17188E+01_JPRB, 0.18103E+01_JPRB, 0.19737E+01_JPRB ,&
&  0.21232E+01_JPRB, 0.21719E+01_JPRB, 0.21694E+01_JPRB, 0.22065E+01_JPRB, 0.20806E+01_JPRB, 0.19154E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.15737E+01_JPRB, 0.14813E+01_JPRB, 0.14533E+01_JPRB, 0.14023E+01_JPRB, 0.14082E+01_JPRB, 0.14735E+01_JPRB, 0.15962E+01_JPRB ,&
&  0.17948E+01_JPRB, 0.20307E+01_JPRB, 0.21400E+01_JPRB, 0.20633E+01_JPRB, 0.21155E+01_JPRB, 0.22317E+01_JPRB, 0.23291E+01_JPRB ,&
&  0.23345E+01_JPRB, 0.23258E+01_JPRB, 0.23419E+01_JPRB, 0.23103E+01_JPRB, 0.20934E+01_JPRB, 0.18955E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.14641E+01_JPRB, 0.12802E+01_JPRB, 0.11737E+01_JPRB, 0.11338E+01_JPRB, 0.10993E+01_JPRB, 0.10613E+01_JPRB, 0.10124E+01_JPRB ,&
&  0.97071E+00_JPRB, 0.96071E+00_JPRB, 0.98232E+00_JPRB, 0.10146E+01_JPRB, 0.10543E+01_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.10437E+01_JPRB, 0.96103E+00_JPRB, 0.90323E+00_JPRB, 0.87366E+00_JPRB, 0.85208E+00_JPRB, 0.82398E+00_JPRB, 0.80274E+00_JPRB ,&
&  0.78777E+00_JPRB, 0.78668E+00_JPRB, 0.77323E+00_JPRB, 0.75931E+00_JPRB, 0.75072E+00_JPRB, 0.74402E+00_JPRB, 0.74110E+00_JPRB ,&
&  0.73777E+00_JPRB, 0.72907E+00_JPRB, 0.71027E+00_JPRB, 0.68725E+00_JPRB, 0.67048E+00_JPRB, 0.65691E+00_JPRB, 0.64603E+00_JPRB ,&
&  0.63940E+00_JPRB, 0.64042E+00_JPRB, 0.65245E+00_JPRB, 0.65248E+00_JPRB, 0.64565E+00_JPRB, 0.63327E+00_JPRB, 0.61918E+00_JPRB ,&
&  0.61569E+00_JPRB, 0.62930E+00_JPRB, 0.61390E+00_JPRB, 0.62086E+00_JPRB, 0.63283E+00_JPRB, 0.61005E+00_JPRB, 0.58988E+00_JPRB ,&
&  0.57069E+00_JPRB, 0.55312E+00_JPRB, 0.54091E+00_JPRB, 0.53775E+00_JPRB, 0.55830E+00_JPRB, 0.58587E+00_JPRB, 0.63749E+00_JPRB ,&
&  0.70624E+00_JPRB, 0.74734E+00_JPRB, 0.77612E+00_JPRB, 0.80934E+00_JPRB, 0.84217E+00_JPRB, 0.88419E+00_JPRB, 0.96170E+00_JPRB ,&
&  0.10884E+01_JPRB, 0.12262E+01_JPRB, 0.13618E+01_JPRB, 0.14638E+01_JPRB, 0.15550E+01_JPRB, 0.15033E+01_JPRB, 0.15153E+01_JPRB ,&
&  0.17507E+01_JPRB, 0.18593E+01_JPRB, 0.17477E+01_JPRB, 0.16367E+01_JPRB, 0.15109E+01_JPRB, 0.14039E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.12731E+01_JPRB, 0.12499E+01_JPRB, 0.11909E+01_JPRB, 0.11435E+01_JPRB, 0.11433E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.15767E+01_JPRB, 0.20904E+01_JPRB, 0.18443E+01_JPRB, 0.17717E+01_JPRB, 0.17895E+01_JPRB, 0.18201E+01_JPRB ,&
&  0.18840E+01_JPRB, 0.19325E+01_JPRB, 0.19617E+01_JPRB, 0.19231E+01_JPRB, 0.18316E+01_JPRB, 0.17263E+01_JPRB, 0.16493E+01_JPRB ,&
&  0.15868E+01_JPRB, 0.15233E+01_JPRB, 0.14757E+01_JPRB, 0.14519E+01_JPRB, 0.14095E+01_JPRB, 0.13571E+01_JPRB, 0.13091E+01_JPRB ,&
&  0.12669E+01_JPRB, 0.12354E+01_JPRB, 0.12132E+01_JPRB, 0.11797E+01_JPRB, 0.11314E+01_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.91001E+00_JPRB, 0.87126E+00_JPRB, 0.84293E+00_JPRB, 0.82524E+00_JPRB, 0.81099E+00_JPRB, 0.80082E+00_JPRB, 0.79196E+00_JPRB ,&
&  0.78409E+00_JPRB, 0.77727E+00_JPRB, 0.76780E+00_JPRB, 0.75902E+00_JPRB, 0.75223E+00_JPRB, 0.74534E+00_JPRB, 0.73946E+00_JPRB ,&
&  0.72987E+00_JPRB, 0.70539E+00_JPRB, 0.69830E+00_JPRB, 0.68949E+00_JPRB, 0.67445E+00_JPRB, 0.66650E+00_JPRB, 0.66155E+00_JPRB ,&
&  0.65389E+00_JPRB, 0.64906E+00_JPRB, 0.64733E+00_JPRB, 0.63774E+00_JPRB, 0.62783E+00_JPRB, 0.62826E+00_JPRB, 0.62547E+00_JPRB ,&
&  0.60337E+00_JPRB, 0.60325E+00_JPRB, 0.63615E+00_JPRB, 0.63209E+00_JPRB, 0.59085E+00_JPRB, 0.58275E+00_JPRB, 0.57721E+00_JPRB ,&
&  0.56406E+00_JPRB, 0.53082E+00_JPRB, 0.52318E+00_JPRB, 0.51894E+00_JPRB, 0.53072E+00_JPRB, 0.54679E+00_JPRB, 0.58407E+00_JPRB ,&
&  0.61849E+00_JPRB, 0.68761E+00_JPRB, 0.72007E+00_JPRB, 0.74616E+00_JPRB, 0.76785E+00_JPRB, 0.79120E+00_JPRB, 0.82098E+00_JPRB ,&
&  0.86352E+00_JPRB, 0.92334E+00_JPRB, 0.10006E+01_JPRB, 0.10727E+01_JPRB, 0.11408E+01_JPRB, 0.11790E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.12420E+01_JPRB, 0.12466E+01_JPRB, 0.11973E+01_JPRB, 0.11520E+01_JPRB, 0.10999E+01_JPRB, 0.10510E+01_JPRB, 0.99813E+00_JPRB ,&
&  0.98573E+00_JPRB, 0.10068E+01_JPRB, 0.99825E+00_JPRB, 0.98509E+00_JPRB, 0.97271E+00_JPRB, 0.97160E+00_JPRB, 0.98144E+00_JPRB ,&
&  0.98413E+00_JPRB, 0.10475E+01_JPRB, 0.12385E+01_JPRB, 0.13001E+01_JPRB, 0.13479E+01_JPRB, 0.13820E+01_JPRB, 0.14072E+01_JPRB ,&
&  0.14397E+01_JPRB, 0.14775E+01_JPRB, 0.15151E+01_JPRB, 0.15343E+01_JPRB, 0.15487E+01_JPRB, 0.15343E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.15083E+01_JPRB, 0.14758E+01_JPRB, 0.14464E+01_JPRB, 0.14121E+01_JPRB, 0.13752E+01_JPRB, 0.13194E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.11777E+01_JPRB, 0.11055E+01_JPRB, 0.10328E+01_JPRB, 0.96528E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.82336E+00_JPRB, 0.80375E+00_JPRB, 0.78903E+00_JPRB, 0.77768E+00_JPRB, 0.77025E+00_JPRB, 0.76413E+00_JPRB, 0.75917E+00_JPRB ,&
&  0.75524E+00_JPRB, 0.74873E+00_JPRB, 0.74177E+00_JPRB, 0.73364E+00_JPRB, 0.72709E+00_JPRB, 0.72127E+00_JPRB, 0.71500E+00_JPRB ,&
&  0.70680E+00_JPRB, 0.69822E+00_JPRB, 0.68944E+00_JPRB, 0.68228E+00_JPRB, 0.68603E+00_JPRB, 0.68608E+00_JPRB, 0.68410E+00_JPRB ,&
&  0.67395E+00_JPRB, 0.65616E+00_JPRB, 0.63721E+00_JPRB, 0.63449E+00_JPRB, 0.62937E+00_JPRB, 0.61351E+00_JPRB, 0.61191E+00_JPRB ,&
&  0.62005E+00_JPRB, 0.62003E+00_JPRB, 0.61480E+00_JPRB, 0.60900E+00_JPRB, 0.57884E+00_JPRB, 0.52998E+00_JPRB, 0.54895E+00_JPRB ,&
&  0.54343E+00_JPRB, 0.52259E+00_JPRB, 0.51790E+00_JPRB, 0.51579E+00_JPRB, 0.52419E+00_JPRB, 0.54164E+00_JPRB, 0.55889E+00_JPRB ,&
&  0.56694E+00_JPRB, 0.64408E+00_JPRB, 0.67609E+00_JPRB, 0.69797E+00_JPRB, 0.71362E+00_JPRB, 0.72783E+00_JPRB, 0.74140E+00_JPRB ,&
&  0.75513E+00_JPRB, 0.77588E+00_JPRB, 0.80522E+00_JPRB, 0.83401E+00_JPRB, 0.86610E+00_JPRB, 0.88859E+00_JPRB, 0.91489E+00_JPRB ,&
&  0.94203E+00_JPRB, 0.94627E+00_JPRB, 0.92322E+00_JPRB, 0.89924E+00_JPRB, 0.87794E+00_JPRB, 0.85809E+00_JPRB, 0.84134E+00_JPRB ,&
&  0.83712E+00_JPRB, 0.83535E+00_JPRB, 0.84823E+00_JPRB, 0.85531E+00_JPRB, 0.86272E+00_JPRB, 0.87214E+00_JPRB, 0.89190E+00_JPRB ,&
&  0.90836E+00_JPRB, 0.93353E+00_JPRB, 0.96028E+00_JPRB, 0.99831E+00_JPRB, 0.10362E+01_JPRB, 0.10653E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11362E+01_JPRB, 0.11519E+01_JPRB, 0.11702E+01_JPRB, 0.11917E+01_JPRB, 0.12020E+01_JPRB, 0.12115E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.12103E+01_JPRB, 0.11951E+01_JPRB, 0.11701E+01_JPRB, 0.11364E+01_JPRB, 0.10977E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.99871E+00_JPRB, 0.94205E+00_JPRB, 0.89910E+00_JPRB, 0.86919E+00_JPRB, 0.84406E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.78230E+00_JPRB, 0.76827E+00_JPRB, 0.75761E+00_JPRB, 0.74899E+00_JPRB, 0.74228E+00_JPRB, 0.73668E+00_JPRB, 0.73121E+00_JPRB ,&
&  0.72671E+00_JPRB, 0.72218E+00_JPRB, 0.71804E+00_JPRB, 0.71457E+00_JPRB, 0.71124E+00_JPRB, 0.70870E+00_JPRB, 0.70548E+00_JPRB ,&
&  0.70452E+00_JPRB, 0.70234E+00_JPRB, 0.70452E+00_JPRB, 0.70264E+00_JPRB, 0.70546E+00_JPRB, 0.70282E+00_JPRB, 0.69155E+00_JPRB ,&
&  0.67827E+00_JPRB, 0.66024E+00_JPRB, 0.63716E+00_JPRB, 0.64169E+00_JPRB, 0.63173E+00_JPRB, 0.62791E+00_JPRB, 0.61211E+00_JPRB ,&
&  0.60370E+00_JPRB, 0.58192E+00_JPRB, 0.59974E+00_JPRB, 0.58686E+00_JPRB, 0.57809E+00_JPRB, 0.56817E+00_JPRB, 0.55025E+00_JPRB ,&
&  0.54601E+00_JPRB, 0.53447E+00_JPRB, 0.53331E+00_JPRB, 0.53688E+00_JPRB, 0.54358E+00_JPRB, 0.55510E+00_JPRB, 0.56635E+00_JPRB ,&
&  0.56960E+00_JPRB, 0.61759E+00_JPRB, 0.64568E+00_JPRB, 0.66519E+00_JPRB, 0.68496E+00_JPRB, 0.70123E+00_JPRB, 0.71356E+00_JPRB ,&
&  0.72541E+00_JPRB, 0.73523E+00_JPRB, 0.74754E+00_JPRB, 0.75214E+00_JPRB, 0.76869E+00_JPRB, 0.77471E+00_JPRB, 0.78367E+00_JPRB ,&
&  0.79281E+00_JPRB, 0.79369E+00_JPRB, 0.79054E+00_JPRB, 0.78296E+00_JPRB, 0.77746E+00_JPRB, 0.77304E+00_JPRB, 0.77325E+00_JPRB ,&
&  0.77425E+00_JPRB, 0.77711E+00_JPRB, 0.78759E+00_JPRB, 0.79738E+00_JPRB, 0.80637E+00_JPRB, 0.81557E+00_JPRB, 0.82468E+00_JPRB ,&
&  0.83499E+00_JPRB, 0.85119E+00_JPRB, 0.86833E+00_JPRB, 0.89607E+00_JPRB, 0.92278E+00_JPRB, 0.94074E+00_JPRB, 0.95641E+00_JPRB ,&
&  0.97394E+00_JPRB, 0.97172E+00_JPRB, 0.97030E+00_JPRB, 0.97227E+00_JPRB, 0.97375E+00_JPRB, 0.97649E+00_JPRB, 0.97882E+00_JPRB ,&
&  0.97772E+00_JPRB, 0.96848E+00_JPRB, 0.95607E+00_JPRB, 0.94070E+00_JPRB, 0.92557E+00_JPRB, 0.91112E+00_JPRB, 0.89700E+00_JPRB ,&
&  0.88083E+00_JPRB, 0.86147E+00_JPRB, 0.84016E+00_JPRB, 0.81842E+00_JPRB, 0.79875E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.76335E+00_JPRB, 0.75198E+00_JPRB, 0.74225E+00_JPRB, 0.73600E+00_JPRB, 0.72959E+00_JPRB, 0.72481E+00_JPRB, 0.72037E+00_JPRB ,&
&  0.71689E+00_JPRB, 0.71360E+00_JPRB, 0.71166E+00_JPRB, 0.70960E+00_JPRB, 0.70793E+00_JPRB, 0.70653E+00_JPRB, 0.70514E+00_JPRB ,&
&  0.70679E+00_JPRB, 0.70597E+00_JPRB, 0.70868E+00_JPRB, 0.70734E+00_JPRB, 0.70627E+00_JPRB, 0.70279E+00_JPRB, 0.69175E+00_JPRB ,&
&  0.68316E+00_JPRB, 0.67387E+00_JPRB, 0.64408E+00_JPRB, 0.65142E+00_JPRB, 0.63785E+00_JPRB, 0.63016E+00_JPRB, 0.62301E+00_JPRB ,&
&  0.60737E+00_JPRB, 0.59971E+00_JPRB, 0.59066E+00_JPRB, 0.58838E+00_JPRB, 0.59034E+00_JPRB, 0.58356E+00_JPRB, 0.58482E+00_JPRB ,&
&  0.57838E+00_JPRB, 0.57731E+00_JPRB, 0.57866E+00_JPRB, 0.58038E+00_JPRB, 0.58786E+00_JPRB, 0.59097E+00_JPRB, 0.59808E+00_JPRB ,&
&  0.61038E+00_JPRB, 0.62911E+00_JPRB, 0.65948E+00_JPRB, 0.66934E+00_JPRB, 0.68840E+00_JPRB, 0.70401E+00_JPRB, 0.71839E+00_JPRB ,&
&  0.72791E+00_JPRB, 0.73424E+00_JPRB, 0.73966E+00_JPRB, 0.74241E+00_JPRB, 0.74351E+00_JPRB, 0.74016E+00_JPRB, 0.74116E+00_JPRB ,&
&  0.74399E+00_JPRB, 0.74729E+00_JPRB, 0.75089E+00_JPRB, 0.75161E+00_JPRB, 0.75338E+00_JPRB, 0.75363E+00_JPRB, 0.75707E+00_JPRB ,&
&  0.75973E+00_JPRB, 0.76244E+00_JPRB, 0.76373E+00_JPRB, 0.76670E+00_JPRB, 0.77097E+00_JPRB, 0.77588E+00_JPRB, 0.78099E+00_JPRB ,&
&  0.78702E+00_JPRB, 0.79529E+00_JPRB, 0.80545E+00_JPRB, 0.80527E+00_JPRB, 0.81335E+00_JPRB, 0.83625E+00_JPRB, 0.84698E+00_JPRB ,&
&  0.85904E+00_JPRB, 0.86583E+00_JPRB, 0.86927E+00_JPRB, 0.87283E+00_JPRB, 0.87493E+00_JPRB, 0.87943E+00_JPRB, 0.88215E+00_JPRB ,&
&  0.88441E+00_JPRB, 0.87661E+00_JPRB, 0.86724E+00_JPRB, 0.85908E+00_JPRB, 0.85062E+00_JPRB, 0.84351E+00_JPRB, 0.83522E+00_JPRB ,&
&  0.82776E+00_JPRB, 0.81917E+00_JPRB, 0.80985E+00_JPRB, 0.79526E+00_JPRB, 0.77943E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.75046E+00_JPRB, 0.74264E+00_JPRB, 0.73685E+00_JPRB, 0.73244E+00_JPRB, 0.72787E+00_JPRB, 0.72450E+00_JPRB, 0.72185E+00_JPRB ,&
&  0.72002E+00_JPRB, 0.71731E+00_JPRB, 0.71608E+00_JPRB, 0.71475E+00_JPRB, 0.71279E+00_JPRB, 0.71295E+00_JPRB, 0.71246E+00_JPRB ,&
&  0.71258E+00_JPRB, 0.71142E+00_JPRB, 0.71322E+00_JPRB, 0.71141E+00_JPRB, 0.71031E+00_JPRB, 0.70944E+00_JPRB, 0.70444E+00_JPRB ,&
&  0.70061E+00_JPRB, 0.69557E+00_JPRB, 0.68524E+00_JPRB, 0.67481E+00_JPRB, 0.66668E+00_JPRB, 0.63543E+00_JPRB, 0.63207E+00_JPRB ,&
&  0.63789E+00_JPRB, 0.62553E+00_JPRB, 0.61970E+00_JPRB, 0.61493E+00_JPRB, 0.61158E+00_JPRB, 0.60914E+00_JPRB, 0.61157E+00_JPRB ,&
&  0.59779E+00_JPRB, 0.61284E+00_JPRB, 0.61389E+00_JPRB, 0.61465E+00_JPRB, 0.61749E+00_JPRB, 0.61644E+00_JPRB, 0.62478E+00_JPRB ,&
&  0.65309E+00_JPRB, 0.65973E+00_JPRB, 0.67292E+00_JPRB, 0.68452E+00_JPRB, 0.69756E+00_JPRB, 0.70906E+00_JPRB, 0.72038E+00_JPRB ,&
&  0.72917E+00_JPRB, 0.73534E+00_JPRB, 0.73957E+00_JPRB, 0.74037E+00_JPRB, 0.74113E+00_JPRB, 0.74104E+00_JPRB, 0.74272E+00_JPRB ,&
&  0.74386E+00_JPRB, 0.74599E+00_JPRB, 0.74882E+00_JPRB, 0.75083E+00_JPRB, 0.75293E+00_JPRB, 0.75314E+00_JPRB, 0.75380E+00_JPRB ,&
&  0.75383E+00_JPRB, 0.75304E+00_JPRB, 0.75194E+00_JPRB, 0.75101E+00_JPRB, 0.74961E+00_JPRB, 0.74935E+00_JPRB, 0.75112E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.75760E+00_JPRB, 0.76206E+00_JPRB, 0.76732E+00_JPRB, 0.77406E+00_JPRB, 0.78317E+00_JPRB, 0.79012E+00_JPRB ,&
&  0.80056E+00_JPRB, 0.80773E+00_JPRB, 0.81407E+00_JPRB, 0.81916E+00_JPRB, 0.82327E+00_JPRB, 0.82866E+00_JPRB, 0.83285E+00_JPRB ,&
&  0.83813E+00_JPRB, 0.83517E+00_JPRB, 0.82998E+00_JPRB, 0.82394E+00_JPRB, 0.81895E+00_JPRB, 0.81451E+00_JPRB, 0.80951E+00_JPRB ,&
&  0.80412E+00_JPRB, 0.79529E+00_JPRB, 0.78501E+00_JPRB, 0.77243E+00_JPRB, 0.76042E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.75189E+00_JPRB, 0.74628E+00_JPRB, 0.74199E+00_JPRB, 0.73860E+00_JPRB, 0.73617E+00_JPRB, 0.73404E+00_JPRB, 0.73226E+00_JPRB ,&
&  0.73056E+00_JPRB, 0.73047E+00_JPRB, 0.72991E+00_JPRB, 0.72916E+00_JPRB, 0.72800E+00_JPRB, 0.72818E+00_JPRB, 0.72786E+00_JPRB ,&
&  0.72639E+00_JPRB, 0.72282E+00_JPRB, 0.71939E+00_JPRB, 0.71629E+00_JPRB, 0.71534E+00_JPRB, 0.71420E+00_JPRB, 0.71369E+00_JPRB ,&
&  0.71140E+00_JPRB, 0.70874E+00_JPRB, 0.70525E+00_JPRB, 0.69758E+00_JPRB, 0.68936E+00_JPRB, 0.68154E+00_JPRB, 0.67455E+00_JPRB ,&
&  0.66856E+00_JPRB, 0.66415E+00_JPRB, 0.66131E+00_JPRB, 0.65961E+00_JPRB, 0.66152E+00_JPRB, 0.66141E+00_JPRB, 0.66076E+00_JPRB ,&
&  0.65910E+00_JPRB, 0.65503E+00_JPRB, 0.65556E+00_JPRB, 0.65652E+00_JPRB, 0.65818E+00_JPRB, 0.66302E+00_JPRB, 0.66779E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.68039E+00_JPRB, 0.69098E+00_JPRB, 0.69886E+00_JPRB, 0.70663E+00_JPRB, 0.71297E+00_JPRB, 0.71667E+00_JPRB ,&
&  0.71982E+00_JPRB, 0.72258E+00_JPRB, 0.72541E+00_JPRB, 0.72777E+00_JPRB, 0.72962E+00_JPRB, 0.73169E+00_JPRB, 0.73405E+00_JPRB ,&
&  0.73537E+00_JPRB, 0.73628E+00_JPRB, 0.73706E+00_JPRB, 0.73779E+00_JPRB, 0.73852E+00_JPRB, 0.73970E+00_JPRB, 0.74126E+00_JPRB ,&
&  0.74271E+00_JPRB, 0.74308E+00_JPRB, 0.74319E+00_JPRB, 0.74433E+00_JPRB, 0.74565E+00_JPRB, 0.74718E+00_JPRB, 0.74865E+00_JPRB ,&
&  0.75082E+00_JPRB, 0.75309E+00_JPRB, 0.75606E+00_JPRB, 0.76055E+00_JPRB, 0.76419E+00_JPRB, 0.76755E+00_JPRB, 0.76935E+00_JPRB ,&
&  0.77161E+00_JPRB, 0.77227E+00_JPRB, 0.77292E+00_JPRB, 0.77478E+00_JPRB, 0.77680E+00_JPRB, 0.78037E+00_JPRB, 0.78432E+00_JPRB ,&
&  0.78914E+00_JPRB, 0.79078E+00_JPRB, 0.79128E+00_JPRB, 0.79020E+00_JPRB, 0.78910E+00_JPRB, 0.78727E+00_JPRB, 0.78539E+00_JPRB ,&
&  0.78300E+00_JPRB, 0.77888E+00_JPRB, 0.77256E+00_JPRB, 0.76546E+00_JPRB, 0.75836E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.75371E+00_JPRB, 0.75175E+00_JPRB, 0.74991E+00_JPRB, 0.74798E+00_JPRB, 0.74619E+00_JPRB, 0.74471E+00_JPRB, 0.74341E+00_JPRB ,&
&  0.74206E+00_JPRB, 0.74057E+00_JPRB, 0.73879E+00_JPRB, 0.73711E+00_JPRB, 0.73557E+00_JPRB, 0.73400E+00_JPRB, 0.73275E+00_JPRB ,&
&  0.73151E+00_JPRB, 0.73041E+00_JPRB, 0.72928E+00_JPRB, 0.72775E+00_JPRB, 0.72643E+00_JPRB, 0.72507E+00_JPRB, 0.72300E+00_JPRB ,&
&  0.72050E+00_JPRB, 0.71797E+00_JPRB, 0.71514E+00_JPRB, 0.71251E+00_JPRB, 0.71036E+00_JPRB, 0.70771E+00_JPRB, 0.70467E+00_JPRB ,&
&  0.70184E+00_JPRB, 0.69970E+00_JPRB, 0.69790E+00_JPRB, 0.69639E+00_JPRB, 0.69537E+00_JPRB, 0.69450E+00_JPRB, 0.69360E+00_JPRB ,&
&  0.69292E+00_JPRB, 0.69274E+00_JPRB, 0.69303E+00_JPRB, 0.69365E+00_JPRB, 0.69456E+00_JPRB, 0.69608E+00_JPRB, 0.69879E+00_JPRB ,&
&  0.70167E+00_JPRB, 0.70456E+00_JPRB, 0.70765E+00_JPRB, 0.71057E+00_JPRB, 0.71329E+00_JPRB, 0.71548E+00_JPRB, 0.71727E+00_JPRB ,&
&  0.71928E+00_JPRB, 0.72129E+00_JPRB, 0.72341E+00_JPRB, 0.72502E+00_JPRB, 0.72640E+00_JPRB, 0.72758E+00_JPRB, 0.72883E+00_JPRB ,&
&  0.72953E+00_JPRB, 0.73027E+00_JPRB, 0.73119E+00_JPRB, 0.73234E+00_JPRB, 0.73337E+00_JPRB, 0.73430E+00_JPRB, 0.73538E+00_JPRB ,&
&  0.73674E+00_JPRB, 0.73821E+00_JPRB, 0.73949E+00_JPRB, 0.74070E+00_JPRB, 0.74207E+00_JPRB, 0.74378E+00_JPRB, 0.74570E+00_JPRB ,&
&  0.74759E+00_JPRB, 0.74944E+00_JPRB, 0.75122E+00_JPRB, 0.75294E+00_JPRB, 0.75416E+00_JPRB, 0.75486E+00_JPRB, 0.75460E+00_JPRB ,&
&  0.75409E+00_JPRB, 0.75333E+00_JPRB, 0.75316E+00_JPRB, 0.75360E+00_JPRB, 0.75396E+00_JPRB, 0.75462E+00_JPRB, 0.75567E+00_JPRB ,&
&  0.75706E+00_JPRB, 0.75832E+00_JPRB, 0.75954E+00_JPRB, 0.76088E+00_JPRB, 0.76181E+00_JPRB, 0.76208E+00_JPRB, 0.76221E+00_JPRB ,&
&  0.76211E+00_JPRB, 0.76129E+00_JPRB, 0.75970E+00_JPRB, 0.75782E+00_JPRB, 0.75571E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB, 0.73688E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB ,&
&  0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB, 0.14159E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.14742E+00_JPRB, 0.14759E+00_JPRB, 0.14767E+00_JPRB, 0.14770E+00_JPRB, 0.14755E+00_JPRB, 0.14754E+00_JPRB, 0.14752E+00_JPRB ,&
&  0.14751E+00_JPRB, 0.14736E+00_JPRB, 0.14755E+00_JPRB, 0.14765E+00_JPRB, 0.14769E+00_JPRB, 0.14770E+00_JPRB, 0.14758E+00_JPRB ,&
&  0.14731E+00_JPRB, 0.14699E+00_JPRB, 0.14613E+00_JPRB, 0.14598E+00_JPRB, 0.14601E+00_JPRB, 0.14622E+00_JPRB, 0.14635E+00_JPRB ,&
&  0.14658E+00_JPRB, 0.14680E+00_JPRB, 0.14698E+00_JPRB, 0.14751E+00_JPRB, 0.14775E+00_JPRB, 0.14801E+00_JPRB, 0.14828E+00_JPRB ,&
&  0.14853E+00_JPRB, 0.14883E+00_JPRB, 0.14904E+00_JPRB, 0.14927E+00_JPRB, 0.14947E+00_JPRB, 0.14942E+00_JPRB, 0.14923E+00_JPRB ,&
&  0.14893E+00_JPRB, 0.14846E+00_JPRB, 0.14797E+00_JPRB, 0.14753E+00_JPRB, 0.14713E+00_JPRB, 0.14671E+00_JPRB, 0.14631E+00_JPRB ,&
&  0.14588E+00_JPRB, 0.14539E+00_JPRB, 0.14486E+00_JPRB, 0.14439E+00_JPRB, 0.14391E+00_JPRB, 0.14349E+00_JPRB, 0.14302E+00_JPRB ,&
&  0.14265E+00_JPRB, 0.14228E+00_JPRB, 0.14193E+00_JPRB, 0.14156E+00_JPRB, 0.14124E+00_JPRB, 0.14092E+00_JPRB, 0.14060E+00_JPRB ,&
&  0.14027E+00_JPRB, 0.14010E+00_JPRB, 0.13986E+00_JPRB, 0.13964E+00_JPRB, 0.13945E+00_JPRB, 0.13930E+00_JPRB, 0.13911E+00_JPRB ,&
&  0.13893E+00_JPRB, 0.13876E+00_JPRB, 0.13869E+00_JPRB, 0.13856E+00_JPRB, 0.13851E+00_JPRB, 0.13849E+00_JPRB, 0.13855E+00_JPRB ,&
&  0.13860E+00_JPRB, 0.13869E+00_JPRB, 0.13885E+00_JPRB, 0.13910E+00_JPRB, 0.13932E+00_JPRB, 0.13957E+00_JPRB, 0.13983E+00_JPRB ,&
&  0.14011E+00_JPRB, 0.14040E+00_JPRB, 0.14070E+00_JPRB, 0.14099E+00_JPRB, 0.14135E+00_JPRB, 0.14171E+00_JPRB, 0.14212E+00_JPRB ,&
&  0.14248E+00_JPRB, 0.14289E+00_JPRB, 0.14322E+00_JPRB, 0.14362E+00_JPRB, 0.14394E+00_JPRB, 0.14414E+00_JPRB, 0.14433E+00_JPRB ,&
&  0.14466E+00_JPRB, 0.14499E+00_JPRB, 0.14540E+00_JPRB, 0.14576E+00_JPRB, 0.14641E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.16124E+00_JPRB, 0.16355E+00_JPRB, 0.16520E+00_JPRB, 0.16700E+00_JPRB, 0.16566E+00_JPRB, 0.17714E+00_JPRB, 0.17519E+00_JPRB ,&
&  0.17487E+00_JPRB, 0.16971E+00_JPRB, 0.16996E+00_JPRB, 0.16913E+00_JPRB, 0.16769E+00_JPRB, 0.16608E+00_JPRB, 0.16431E+00_JPRB ,&
&  0.16259E+00_JPRB, 0.16043E+00_JPRB, 0.15861E+00_JPRB, 0.15710E+00_JPRB, 0.15572E+00_JPRB, 0.15451E+00_JPRB, 0.15328E+00_JPRB ,&
&  0.15309E+00_JPRB, 0.15370E+00_JPRB, 0.15468E+00_JPRB, 0.15408E+00_JPRB, 0.15447E+00_JPRB, 0.15530E+00_JPRB, 0.15643E+00_JPRB ,&
&  0.15740E+00_JPRB, 0.15872E+00_JPRB, 0.15956E+00_JPRB, 0.15979E+00_JPRB, 0.16005E+00_JPRB, 0.15936E+00_JPRB, 0.15889E+00_JPRB ,&
&  0.15824E+00_JPRB, 0.15734E+00_JPRB, 0.15619E+00_JPRB, 0.15521E+00_JPRB, 0.15411E+00_JPRB, 0.15269E+00_JPRB, 0.15120E+00_JPRB ,&
&  0.14987E+00_JPRB, 0.14869E+00_JPRB, 0.14746E+00_JPRB, 0.14643E+00_JPRB, 0.14545E+00_JPRB, 0.14447E+00_JPRB, 0.14358E+00_JPRB ,&
&  0.14287E+00_JPRB, 0.14224E+00_JPRB, 0.14155E+00_JPRB, 0.14083E+00_JPRB, 0.14003E+00_JPRB, 0.13925E+00_JPRB, 0.13850E+00_JPRB ,&
&  0.13813E+00_JPRB, 0.13780E+00_JPRB, 0.13747E+00_JPRB, 0.13711E+00_JPRB, 0.13698E+00_JPRB, 0.13685E+00_JPRB, 0.13668E+00_JPRB ,&
&  0.13650E+00_JPRB, 0.13625E+00_JPRB, 0.13616E+00_JPRB, 0.13607E+00_JPRB, 0.13598E+00_JPRB, 0.13592E+00_JPRB, 0.13601E+00_JPRB ,&
&  0.13618E+00_JPRB, 0.13640E+00_JPRB, 0.13686E+00_JPRB, 0.13736E+00_JPRB, 0.13788E+00_JPRB, 0.13839E+00_JPRB, 0.13912E+00_JPRB ,&
&  0.13993E+00_JPRB, 0.14069E+00_JPRB, 0.14136E+00_JPRB, 0.14192E+00_JPRB, 0.14278E+00_JPRB, 0.14359E+00_JPRB, 0.14439E+00_JPRB ,&
&  0.14526E+00_JPRB, 0.14638E+00_JPRB, 0.14755E+00_JPRB, 0.14874E+00_JPRB, 0.15012E+00_JPRB, 0.15131E+00_JPRB, 0.15249E+00_JPRB ,&
&  0.15389E+00_JPRB, 0.15645E+00_JPRB, 0.15822E+00_JPRB, 0.15940E+00_JPRB, 0.16070E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.18222E+00_JPRB, 0.18344E+00_JPRB, 0.18448E+00_JPRB, 0.18500E+00_JPRB, 0.18554E+00_JPRB, 0.18557E+00_JPRB, 0.18549E+00_JPRB ,&
&  0.18278E+00_JPRB, 0.18160E+00_JPRB, 0.17989E+00_JPRB, 0.17796E+00_JPRB, 0.17628E+00_JPRB, 0.17418E+00_JPRB, 0.17211E+00_JPRB ,&
&  0.17015E+00_JPRB, 0.16784E+00_JPRB, 0.16540E+00_JPRB, 0.16253E+00_JPRB, 0.15975E+00_JPRB, 0.15713E+00_JPRB, 0.15565E+00_JPRB ,&
&  0.15491E+00_JPRB, 0.15518E+00_JPRB, 0.15666E+00_JPRB, 0.15786E+00_JPRB, 0.15959E+00_JPRB, 0.16181E+00_JPRB, 0.16535E+00_JPRB ,&
&  0.16806E+00_JPRB, 0.17142E+00_JPRB, 0.17367E+00_JPRB, 0.17440E+00_JPRB, 0.17372E+00_JPRB, 0.17337E+00_JPRB, 0.17252E+00_JPRB ,&
&  0.17077E+00_JPRB, 0.16895E+00_JPRB, 0.16736E+00_JPRB, 0.16566E+00_JPRB, 0.16377E+00_JPRB, 0.16143E+00_JPRB, 0.15857E+00_JPRB ,&
&  0.15597E+00_JPRB, 0.15368E+00_JPRB, 0.15156E+00_JPRB, 0.14992E+00_JPRB, 0.14833E+00_JPRB, 0.14689E+00_JPRB, 0.14536E+00_JPRB ,&
&  0.14418E+00_JPRB, 0.14315E+00_JPRB, 0.14186E+00_JPRB, 0.14079E+00_JPRB, 0.13957E+00_JPRB, 0.13855E+00_JPRB, 0.13745E+00_JPRB ,&
&  0.13659E+00_JPRB, 0.13583E+00_JPRB, 0.13520E+00_JPRB, 0.13461E+00_JPRB, 0.13423E+00_JPRB, 0.13386E+00_JPRB, 0.13344E+00_JPRB ,&
&  0.13307E+00_JPRB, 0.13301E+00_JPRB, 0.13300E+00_JPRB, 0.13300E+00_JPRB, 0.13306E+00_JPRB, 0.13275E+00_JPRB, 0.13280E+00_JPRB ,&
&  0.13310E+00_JPRB, 0.13343E+00_JPRB, 0.13355E+00_JPRB, 0.13399E+00_JPRB, 0.13466E+00_JPRB, 0.13546E+00_JPRB, 0.13663E+00_JPRB ,&
&  0.13755E+00_JPRB, 0.13848E+00_JPRB, 0.13936E+00_JPRB, 0.14025E+00_JPRB, 0.14140E+00_JPRB, 0.14263E+00_JPRB, 0.14393E+00_JPRB ,&
&  0.14547E+00_JPRB, 0.14692E+00_JPRB, 0.14843E+00_JPRB, 0.15000E+00_JPRB, 0.15247E+00_JPRB, 0.15491E+00_JPRB, 0.15865E+00_JPRB ,&
&  0.16245E+00_JPRB, 0.16443E+00_JPRB, 0.17727E+00_JPRB, 0.17831E+00_JPRB, 0.17980E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.18273E+00_JPRB, 0.18416E+00_JPRB, 0.18495E+00_JPRB, 0.18602E+00_JPRB, 0.18653E+00_JPRB, 0.18804E+00_JPRB, 0.18828E+00_JPRB ,&
&  0.18841E+00_JPRB, 0.18456E+00_JPRB, 0.18222E+00_JPRB, 0.17959E+00_JPRB, 0.17731E+00_JPRB, 0.17470E+00_JPRB, 0.17215E+00_JPRB ,&
&  0.17015E+00_JPRB, 0.16842E+00_JPRB, 0.16657E+00_JPRB, 0.16399E+00_JPRB, 0.16138E+00_JPRB, 0.15945E+00_JPRB, 0.15802E+00_JPRB ,&
&  0.15662E+00_JPRB, 0.15724E+00_JPRB, 0.15868E+00_JPRB, 0.16169E+00_JPRB, 0.16583E+00_JPRB, 0.16936E+00_JPRB, 0.17403E+00_JPRB ,&
&  0.18010E+00_JPRB, 0.18195E+00_JPRB, 0.18408E+00_JPRB, 0.18508E+00_JPRB, 0.18481E+00_JPRB, 0.18530E+00_JPRB, 0.18298E+00_JPRB ,&
&  0.18318E+00_JPRB, 0.18097E+00_JPRB, 0.18543E+00_JPRB, 0.18273E+00_JPRB, 0.17632E+00_JPRB, 0.17327E+00_JPRB, 0.16729E+00_JPRB ,&
&  0.16279E+00_JPRB, 0.15805E+00_JPRB, 0.15484E+00_JPRB, 0.15302E+00_JPRB, 0.15018E+00_JPRB, 0.14862E+00_JPRB, 0.14627E+00_JPRB ,&
&  0.14410E+00_JPRB, 0.14335E+00_JPRB, 0.14197E+00_JPRB, 0.14138E+00_JPRB, 0.14063E+00_JPRB, 0.13959E+00_JPRB, 0.13846E+00_JPRB ,&
&  0.13770E+00_JPRB, 0.13701E+00_JPRB, 0.13665E+00_JPRB, 0.13614E+00_JPRB, 0.13543E+00_JPRB, 0.13472E+00_JPRB, 0.13393E+00_JPRB ,&
&  0.13339E+00_JPRB, 0.13307E+00_JPRB, 0.13282E+00_JPRB, 0.13272E+00_JPRB, 0.13261E+00_JPRB, 0.13236E+00_JPRB, 0.13244E+00_JPRB ,&
&  0.13268E+00_JPRB, 0.13326E+00_JPRB, 0.13376E+00_JPRB, 0.13443E+00_JPRB, 0.13505E+00_JPRB, 0.13587E+00_JPRB, 0.13650E+00_JPRB ,&
&  0.13717E+00_JPRB, 0.13809E+00_JPRB, 0.13888E+00_JPRB, 0.13982E+00_JPRB, 0.14085E+00_JPRB, 0.14173E+00_JPRB, 0.14310E+00_JPRB ,&
&  0.14450E+00_JPRB, 0.14588E+00_JPRB, 0.14779E+00_JPRB, 0.14977E+00_JPRB, 0.15229E+00_JPRB, 0.15661E+00_JPRB, 0.16158E+00_JPRB ,&
&  0.16779E+00_JPRB, 0.17711E+00_JPRB, 0.17745E+00_JPRB, 0.18001E+00_JPRB, 0.18152E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.18785E+00_JPRB, 0.18976E+00_JPRB, 0.19127E+00_JPRB, 0.19267E+00_JPRB, 0.19290E+00_JPRB, 0.19318E+00_JPRB, 0.19392E+00_JPRB ,&
&  0.19019E+00_JPRB, 0.18748E+00_JPRB, 0.18296E+00_JPRB, 0.17866E+00_JPRB, 0.17459E+00_JPRB, 0.17113E+00_JPRB, 0.16906E+00_JPRB ,&
&  0.16800E+00_JPRB, 0.16807E+00_JPRB, 0.16838E+00_JPRB, 0.16907E+00_JPRB, 0.16846E+00_JPRB, 0.16756E+00_JPRB, 0.16665E+00_JPRB ,&
&  0.16548E+00_JPRB, 0.16568E+00_JPRB, 0.16613E+00_JPRB, 0.16937E+00_JPRB, 0.17394E+00_JPRB, 0.17912E+00_JPRB, 0.18284E+00_JPRB ,&
&  0.18549E+00_JPRB, 0.18749E+00_JPRB, 0.18993E+00_JPRB, 0.19097E+00_JPRB, 0.19193E+00_JPRB, 0.19319E+00_JPRB, 0.19376E+00_JPRB ,&
&  0.19508E+00_JPRB, 0.19524E+00_JPRB, 0.19521E+00_JPRB, 0.19559E+00_JPRB, 0.18744E+00_JPRB, 0.18133E+00_JPRB, 0.17512E+00_JPRB ,&
&  0.16910E+00_JPRB, 0.16304E+00_JPRB, 0.15760E+00_JPRB, 0.15454E+00_JPRB, 0.15144E+00_JPRB, 0.14964E+00_JPRB, 0.14807E+00_JPRB ,&
&  0.14689E+00_JPRB, 0.14568E+00_JPRB, 0.14396E+00_JPRB, 0.14227E+00_JPRB, 0.14129E+00_JPRB, 0.14046E+00_JPRB, 0.13941E+00_JPRB ,&
&  0.13931E+00_JPRB, 0.13869E+00_JPRB, 0.13861E+00_JPRB, 0.13822E+00_JPRB, 0.13794E+00_JPRB, 0.13769E+00_JPRB, 0.13768E+00_JPRB ,&
&  0.13803E+00_JPRB, 0.13873E+00_JPRB, 0.13898E+00_JPRB, 0.13929E+00_JPRB, 0.13915E+00_JPRB, 0.13854E+00_JPRB, 0.13802E+00_JPRB ,&
&  0.13746E+00_JPRB, 0.13701E+00_JPRB, 0.13701E+00_JPRB, 0.13692E+00_JPRB, 0.13742E+00_JPRB, 0.13808E+00_JPRB, 0.13875E+00_JPRB ,&
&  0.13966E+00_JPRB, 0.14036E+00_JPRB, 0.14106E+00_JPRB, 0.14193E+00_JPRB, 0.14269E+00_JPRB, 0.14367E+00_JPRB, 0.14480E+00_JPRB ,&
&  0.14568E+00_JPRB, 0.14736E+00_JPRB, 0.14875E+00_JPRB, 0.15044E+00_JPRB, 0.15363E+00_JPRB, 0.15715E+00_JPRB, 0.16528E+00_JPRB ,&
&  0.17320E+00_JPRB, 0.18991E+00_JPRB, 0.19135E+00_JPRB, 0.18619E+00_JPRB, 0.18671E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.19635E+00_JPRB, 0.19866E+00_JPRB, 0.20094E+00_JPRB, 0.19970E+00_JPRB, 0.19978E+00_JPRB, 0.20007E+00_JPRB, 0.20127E+00_JPRB ,&
&  0.20185E+00_JPRB, 0.20147E+00_JPRB, 0.20115E+00_JPRB, 0.19975E+00_JPRB, 0.19824E+00_JPRB, 0.19136E+00_JPRB, 0.18840E+00_JPRB ,&
&  0.18610E+00_JPRB, 0.18659E+00_JPRB, 0.18710E+00_JPRB, 0.18910E+00_JPRB, 0.18934E+00_JPRB, 0.18848E+00_JPRB, 0.18550E+00_JPRB ,&
&  0.18306E+00_JPRB, 0.18150E+00_JPRB, 0.17990E+00_JPRB, 0.18006E+00_JPRB, 0.18203E+00_JPRB, 0.18418E+00_JPRB, 0.18547E+00_JPRB ,&
&  0.18596E+00_JPRB, 0.18787E+00_JPRB, 0.18954E+00_JPRB, 0.19223E+00_JPRB, 0.19916E+00_JPRB, 0.20041E+00_JPRB, 0.20118E+00_JPRB ,&
&  0.20206E+00_JPRB, 0.20333E+00_JPRB, 0.20414E+00_JPRB, 0.20401E+00_JPRB, 0.20243E+00_JPRB, 0.19777E+00_JPRB, 0.19653E+00_JPRB ,&
&  0.18401E+00_JPRB, 0.17661E+00_JPRB, 0.16959E+00_JPRB, 0.16442E+00_JPRB, 0.15935E+00_JPRB, 0.15658E+00_JPRB, 0.15349E+00_JPRB ,&
&  0.15088E+00_JPRB, 0.14805E+00_JPRB, 0.14593E+00_JPRB, 0.14497E+00_JPRB, 0.14469E+00_JPRB, 0.14443E+00_JPRB, 0.14383E+00_JPRB ,&
&  0.14391E+00_JPRB, 0.14348E+00_JPRB, 0.14316E+00_JPRB, 0.14283E+00_JPRB, 0.14261E+00_JPRB, 0.14268E+00_JPRB, 0.14338E+00_JPRB ,&
&  0.14522E+00_JPRB, 0.14698E+00_JPRB, 0.14875E+00_JPRB, 0.15034E+00_JPRB, 0.14987E+00_JPRB, 0.14825E+00_JPRB, 0.14579E+00_JPRB ,&
&  0.14366E+00_JPRB, 0.14230E+00_JPRB, 0.14083E+00_JPRB, 0.14046E+00_JPRB, 0.14104E+00_JPRB, 0.14177E+00_JPRB, 0.14303E+00_JPRB ,&
&  0.14414E+00_JPRB, 0.14505E+00_JPRB, 0.14566E+00_JPRB, 0.14608E+00_JPRB, 0.14676E+00_JPRB, 0.14740E+00_JPRB, 0.14848E+00_JPRB ,&
&  0.14968E+00_JPRB, 0.15117E+00_JPRB, 0.15252E+00_JPRB, 0.15435E+00_JPRB, 0.15705E+00_JPRB, 0.16117E+00_JPRB, 0.16824E+00_JPRB ,&
&  0.17530E+00_JPRB, 0.19393E+00_JPRB, 0.19609E+00_JPRB, 0.19480E+00_JPRB, 0.19610E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.20409E+00_JPRB, 0.20839E+00_JPRB, 0.21079E+00_JPRB, 0.21031E+00_JPRB, 0.20995E+00_JPRB, 0.21059E+00_JPRB, 0.21138E+00_JPRB ,&
&  0.21243E+00_JPRB, 0.21199E+00_JPRB, 0.21061E+00_JPRB, 0.20923E+00_JPRB, 0.20774E+00_JPRB, 0.20576E+00_JPRB, 0.20442E+00_JPRB ,&
&  0.20337E+00_JPRB, 0.20336E+00_JPRB, 0.20301E+00_JPRB, 0.20385E+00_JPRB, 0.20439E+00_JPRB, 0.20677E+00_JPRB, 0.20540E+00_JPRB ,&
&  0.20197E+00_JPRB, 0.20011E+00_JPRB, 0.20236E+00_JPRB, 0.20274E+00_JPRB, 0.20331E+00_JPRB, 0.20410E+00_JPRB, 0.20545E+00_JPRB ,&
&  0.20305E+00_JPRB, 0.19999E+00_JPRB, 0.19574E+00_JPRB, 0.20086E+00_JPRB, 0.20968E+00_JPRB, 0.20955E+00_JPRB, 0.20885E+00_JPRB ,&
&  0.21025E+00_JPRB, 0.21293E+00_JPRB, 0.21424E+00_JPRB, 0.21459E+00_JPRB, 0.21317E+00_JPRB, 0.21251E+00_JPRB, 0.21142E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.20591E+00_JPRB, 0.19751E+00_JPRB, 0.18175E+00_JPRB, 0.17526E+00_JPRB, 0.17217E+00_JPRB, 0.16823E+00_JPRB ,&
&  0.16322E+00_JPRB, 0.16085E+00_JPRB, 0.16060E+00_JPRB, 0.15910E+00_JPRB, 0.15931E+00_JPRB, 0.15886E+00_JPRB, 0.15790E+00_JPRB ,&
&  0.15665E+00_JPRB, 0.15577E+00_JPRB, 0.15428E+00_JPRB, 0.15190E+00_JPRB, 0.15011E+00_JPRB, 0.14968E+00_JPRB, 0.15040E+00_JPRB ,&
&  0.15225E+00_JPRB, 0.15515E+00_JPRB, 0.16335E+00_JPRB, 0.16814E+00_JPRB, 0.16620E+00_JPRB, 0.16220E+00_JPRB, 0.15622E+00_JPRB ,&
&  0.15095E+00_JPRB, 0.14932E+00_JPRB, 0.14819E+00_JPRB, 0.14724E+00_JPRB, 0.14759E+00_JPRB, 0.14862E+00_JPRB, 0.14967E+00_JPRB ,&
&  0.15086E+00_JPRB, 0.15222E+00_JPRB, 0.15306E+00_JPRB, 0.15352E+00_JPRB, 0.15398E+00_JPRB, 0.15423E+00_JPRB, 0.15471E+00_JPRB ,&
&  0.15485E+00_JPRB, 0.15617E+00_JPRB, 0.15735E+00_JPRB, 0.15965E+00_JPRB, 0.16123E+00_JPRB, 0.16321E+00_JPRB, 0.16731E+00_JPRB ,&
&  0.17084E+00_JPRB, 0.18125E+00_JPRB, 0.19583E+00_JPRB, 0.20355E+00_JPRB, 0.20070E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.21518E+00_JPRB, 0.21633E+00_JPRB, 0.21646E+00_JPRB, 0.21558E+00_JPRB, 0.21437E+00_JPRB, 0.21461E+00_JPRB, 0.21606E+00_JPRB ,&
&  0.21896E+00_JPRB, 0.22110E+00_JPRB, 0.22163E+00_JPRB, 0.22168E+00_JPRB, 0.22077E+00_JPRB, 0.21871E+00_JPRB, 0.21696E+00_JPRB ,&
&  0.21558E+00_JPRB, 0.21473E+00_JPRB, 0.21405E+00_JPRB, 0.21381E+00_JPRB, 0.21378E+00_JPRB, 0.21486E+00_JPRB, 0.21593E+00_JPRB ,&
&  0.21968E+00_JPRB, 0.21978E+00_JPRB, 0.21912E+00_JPRB, 0.21890E+00_JPRB, 0.21869E+00_JPRB, 0.21867E+00_JPRB, 0.21861E+00_JPRB ,&
&  0.21612E+00_JPRB, 0.20839E+00_JPRB, 0.20145E+00_JPRB, 0.20680E+00_JPRB, 0.21900E+00_JPRB, 0.21937E+00_JPRB, 0.21941E+00_JPRB ,&
&  0.22076E+00_JPRB, 0.22364E+00_JPRB, 0.22565E+00_JPRB, 0.22667E+00_JPRB, 0.22553E+00_JPRB, 0.22381E+00_JPRB, 0.22335E+00_JPRB ,&
&  0.22357E+00_JPRB, 0.22176E+00_JPRB, 0.21662E+00_JPRB, 0.21079E+00_JPRB, 0.20493E+00_JPRB, 0.20261E+00_JPRB, 0.20093E+00_JPRB ,&
&  0.19848E+00_JPRB, 0.19705E+00_JPRB, 0.19650E+00_JPRB, 0.19600E+00_JPRB, 0.19639E+00_JPRB, 0.19442E+00_JPRB, 0.19168E+00_JPRB ,&
&  0.18804E+00_JPRB, 0.18405E+00_JPRB, 0.17915E+00_JPRB, 0.17395E+00_JPRB, 0.16945E+00_JPRB, 0.16465E+00_JPRB, 0.16134E+00_JPRB ,&
&  0.16066E+00_JPRB, 0.16437E+00_JPRB, 0.17372E+00_JPRB, 0.18382E+00_JPRB, 0.19164E+00_JPRB, 0.18055E+00_JPRB, 0.17366E+00_JPRB ,&
&  0.16648E+00_JPRB, 0.16080E+00_JPRB, 0.15760E+00_JPRB, 0.15689E+00_JPRB, 0.15710E+00_JPRB, 0.15796E+00_JPRB, 0.15954E+00_JPRB ,&
&  0.16154E+00_JPRB, 0.16408E+00_JPRB, 0.16603E+00_JPRB, 0.16738E+00_JPRB, 0.16833E+00_JPRB, 0.16860E+00_JPRB, 0.16839E+00_JPRB ,&
&  0.16814E+00_JPRB, 0.16934E+00_JPRB, 0.17173E+00_JPRB, 0.17591E+00_JPRB, 0.17893E+00_JPRB, 0.18277E+00_JPRB, 0.18810E+00_JPRB ,&
&  0.19578E+00_JPRB, 0.20482E+00_JPRB, 0.20792E+00_JPRB, 0.20999E+00_JPRB, 0.21285E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.22571E+00_JPRB, 0.22699E+00_JPRB, 0.22714E+00_JPRB, 0.22749E+00_JPRB, 0.22806E+00_JPRB, 0.22990E+00_JPRB, 0.23404E+00_JPRB ,&
&  0.23810E+00_JPRB, 0.23996E+00_JPRB, 0.23845E+00_JPRB, 0.23619E+00_JPRB, 0.23501E+00_JPRB, 0.23436E+00_JPRB, 0.23413E+00_JPRB ,&
&  0.23395E+00_JPRB, 0.23294E+00_JPRB, 0.23010E+00_JPRB, 0.22621E+00_JPRB, 0.22339E+00_JPRB, 0.22275E+00_JPRB, 0.22324E+00_JPRB ,&
&  0.22705E+00_JPRB, 0.22971E+00_JPRB, 0.23219E+00_JPRB, 0.23430E+00_JPRB, 0.23476E+00_JPRB, 0.23441E+00_JPRB, 0.23582E+00_JPRB ,&
&  0.23690E+00_JPRB, 0.23469E+00_JPRB, 0.22942E+00_JPRB, 0.23359E+00_JPRB, 0.23567E+00_JPRB, 0.23694E+00_JPRB, 0.23623E+00_JPRB ,&
&  0.23691E+00_JPRB, 0.23903E+00_JPRB, 0.24105E+00_JPRB, 0.24263E+00_JPRB, 0.24194E+00_JPRB, 0.23891E+00_JPRB, 0.23801E+00_JPRB ,&
&  0.23828E+00_JPRB, 0.23718E+00_JPRB, 0.23446E+00_JPRB, 0.23199E+00_JPRB, 0.22926E+00_JPRB, 0.22805E+00_JPRB, 0.22781E+00_JPRB ,&
&  0.22687E+00_JPRB, 0.22465E+00_JPRB, 0.22028E+00_JPRB, 0.21602E+00_JPRB, 0.21292E+00_JPRB, 0.20992E+00_JPRB, 0.20872E+00_JPRB ,&
&  0.20919E+00_JPRB, 0.21040E+00_JPRB, 0.21047E+00_JPRB, 0.20910E+00_JPRB, 0.20446E+00_JPRB, 0.19425E+00_JPRB, 0.18712E+00_JPRB ,&
&  0.18585E+00_JPRB, 0.19041E+00_JPRB, 0.19853E+00_JPRB, 0.19975E+00_JPRB, 0.20186E+00_JPRB, 0.20335E+00_JPRB, 0.20478E+00_JPRB ,&
&  0.20357E+00_JPRB, 0.19050E+00_JPRB, 0.18472E+00_JPRB, 0.18147E+00_JPRB, 0.17835E+00_JPRB, 0.17789E+00_JPRB, 0.17942E+00_JPRB ,&
&  0.18252E+00_JPRB, 0.18661E+00_JPRB, 0.18939E+00_JPRB, 0.19096E+00_JPRB, 0.19136E+00_JPRB, 0.19097E+00_JPRB, 0.19024E+00_JPRB ,&
&  0.19038E+00_JPRB, 0.19418E+00_JPRB, 0.20192E+00_JPRB, 0.21029E+00_JPRB, 0.21129E+00_JPRB, 0.21291E+00_JPRB, 0.21345E+00_JPRB ,&
&  0.21393E+00_JPRB, 0.21577E+00_JPRB, 0.21691E+00_JPRB, 0.21964E+00_JPRB, 0.22300E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.23768E+00_JPRB, 0.24077E+00_JPRB, 0.24377E+00_JPRB, 0.24548E+00_JPRB, 0.24728E+00_JPRB, 0.24914E+00_JPRB, 0.25246E+00_JPRB ,&
&  0.25636E+00_JPRB, 0.25760E+00_JPRB, 0.25588E+00_JPRB, 0.25309E+00_JPRB, 0.25128E+00_JPRB, 0.25049E+00_JPRB, 0.25026E+00_JPRB ,&
&  0.25010E+00_JPRB, 0.24996E+00_JPRB, 0.24922E+00_JPRB, 0.24745E+00_JPRB, 0.24496E+00_JPRB, 0.24246E+00_JPRB, 0.24061E+00_JPRB ,&
&  0.24039E+00_JPRB, 0.24189E+00_JPRB, 0.24424E+00_JPRB, 0.24456E+00_JPRB, 0.24475E+00_JPRB, 0.24600E+00_JPRB, 0.24800E+00_JPRB ,&
&  0.25191E+00_JPRB, 0.25391E+00_JPRB, 0.25603E+00_JPRB, 0.25201E+00_JPRB, 0.25657E+00_JPRB, 0.25820E+00_JPRB, 0.25874E+00_JPRB ,&
&  0.26001E+00_JPRB, 0.26103E+00_JPRB, 0.26200E+00_JPRB, 0.26132E+00_JPRB, 0.25705E+00_JPRB, 0.25522E+00_JPRB, 0.25597E+00_JPRB ,&
&  0.25666E+00_JPRB, 0.25510E+00_JPRB, 0.25220E+00_JPRB, 0.24948E+00_JPRB, 0.24726E+00_JPRB, 0.24569E+00_JPRB, 0.24435E+00_JPRB ,&
&  0.24364E+00_JPRB, 0.24283E+00_JPRB, 0.24088E+00_JPRB, 0.23570E+00_JPRB, 0.22980E+00_JPRB, 0.22592E+00_JPRB, 0.22324E+00_JPRB ,&
&  0.22329E+00_JPRB, 0.22523E+00_JPRB, 0.22691E+00_JPRB, 0.22705E+00_JPRB, 0.22581E+00_JPRB, 0.22358E+00_JPRB, 0.22027E+00_JPRB ,&
&  0.21540E+00_JPRB, 0.21333E+00_JPRB, 0.21269E+00_JPRB, 0.21179E+00_JPRB, 0.21224E+00_JPRB, 0.21404E+00_JPRB, 0.21588E+00_JPRB ,&
&  0.21808E+00_JPRB, 0.21813E+00_JPRB, 0.21673E+00_JPRB, 0.21436E+00_JPRB, 0.21186E+00_JPRB, 0.21164E+00_JPRB, 0.21221E+00_JPRB ,&
&  0.21271E+00_JPRB, 0.21314E+00_JPRB, 0.21395E+00_JPRB, 0.21506E+00_JPRB, 0.21657E+00_JPRB, 0.21883E+00_JPRB, 0.22072E+00_JPRB ,&
&  0.22069E+00_JPRB, 0.21959E+00_JPRB, 0.22063E+00_JPRB, 0.22231E+00_JPRB, 0.22320E+00_JPRB, 0.22381E+00_JPRB, 0.22425E+00_JPRB ,&
&  0.22397E+00_JPRB, 0.22408E+00_JPRB, 0.22601E+00_JPRB, 0.23062E+00_JPRB, 0.23484E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.25127E+00_JPRB, 0.25523E+00_JPRB, 0.26044E+00_JPRB, 0.26173E+00_JPRB, 0.26428E+00_JPRB, 0.26693E+00_JPRB, 0.27036E+00_JPRB ,&
&  0.27253E+00_JPRB, 0.27266E+00_JPRB, 0.27110E+00_JPRB, 0.26953E+00_JPRB, 0.26868E+00_JPRB, 0.26835E+00_JPRB, 0.26779E+00_JPRB ,&
&  0.26756E+00_JPRB, 0.26768E+00_JPRB, 0.26762E+00_JPRB, 0.26745E+00_JPRB, 0.26631E+00_JPRB, 0.26460E+00_JPRB, 0.26270E+00_JPRB ,&
&  0.25949E+00_JPRB, 0.25740E+00_JPRB, 0.25681E+00_JPRB, 0.25669E+00_JPRB, 0.25610E+00_JPRB, 0.25671E+00_JPRB, 0.25849E+00_JPRB ,&
&  0.26227E+00_JPRB, 0.26531E+00_JPRB, 0.26923E+00_JPRB, 0.27326E+00_JPRB, 0.27570E+00_JPRB, 0.27747E+00_JPRB, 0.27952E+00_JPRB ,&
&  0.28082E+00_JPRB, 0.28115E+00_JPRB, 0.28116E+00_JPRB, 0.28110E+00_JPRB, 0.27829E+00_JPRB, 0.27734E+00_JPRB, 0.27692E+00_JPRB ,&
&  0.27583E+00_JPRB, 0.27319E+00_JPRB, 0.26948E+00_JPRB, 0.26627E+00_JPRB, 0.26309E+00_JPRB, 0.26078E+00_JPRB, 0.25887E+00_JPRB ,&
&  0.25746E+00_JPRB, 0.25681E+00_JPRB, 0.25682E+00_JPRB, 0.25539E+00_JPRB, 0.25030E+00_JPRB, 0.24526E+00_JPRB, 0.24243E+00_JPRB ,&
&  0.24110E+00_JPRB, 0.24083E+00_JPRB, 0.24221E+00_JPRB, 0.24107E+00_JPRB, 0.23742E+00_JPRB, 0.23448E+00_JPRB, 0.23327E+00_JPRB ,&
&  0.23169E+00_JPRB, 0.23093E+00_JPRB, 0.23158E+00_JPRB, 0.23221E+00_JPRB, 0.23287E+00_JPRB, 0.23316E+00_JPRB, 0.23437E+00_JPRB ,&
&  0.23546E+00_JPRB, 0.23693E+00_JPRB, 0.23675E+00_JPRB, 0.23407E+00_JPRB, 0.23036E+00_JPRB, 0.22767E+00_JPRB, 0.22715E+00_JPRB ,&
&  0.22858E+00_JPRB, 0.23001E+00_JPRB, 0.23037E+00_JPRB, 0.23047E+00_JPRB, 0.23152E+00_JPRB, 0.23396E+00_JPRB, 0.23712E+00_JPRB ,&
&  0.23847E+00_JPRB, 0.23976E+00_JPRB, 0.24158E+00_JPRB, 0.24346E+00_JPRB, 0.24479E+00_JPRB, 0.24568E+00_JPRB, 0.24608E+00_JPRB ,&
&  0.24558E+00_JPRB, 0.24635E+00_JPRB, 0.24789E+00_JPRB, 0.24967E+00_JPRB, 0.25091E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.27240E+00_JPRB, 0.27501E+00_JPRB, 0.27837E+00_JPRB, 0.27913E+00_JPRB, 0.27957E+00_JPRB, 0.28111E+00_JPRB, 0.28414E+00_JPRB ,&
&  0.28617E+00_JPRB, 0.28668E+00_JPRB, 0.28649E+00_JPRB, 0.28617E+00_JPRB, 0.28643E+00_JPRB, 0.28794E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28792E+00_JPRB, 0.28727E+00_JPRB, 0.28641E+00_JPRB, 0.28579E+00_JPRB, 0.28437E+00_JPRB, 0.28226E+00_JPRB, 0.28054E+00_JPRB ,&
&  0.27812E+00_JPRB, 0.27592E+00_JPRB, 0.27434E+00_JPRB, 0.27316E+00_JPRB, 0.27260E+00_JPRB, 0.27277E+00_JPRB, 0.27443E+00_JPRB ,&
&  0.27646E+00_JPRB, 0.27901E+00_JPRB, 0.28215E+00_JPRB, 0.28577E+00_JPRB, 0.28940E+00_JPRB, 0.29349E+00_JPRB, 0.29710E+00_JPRB ,&
&  0.29993E+00_JPRB, 0.30227E+00_JPRB, 0.30376E+00_JPRB, 0.30092E+00_JPRB, 0.29742E+00_JPRB, 0.29659E+00_JPRB, 0.29621E+00_JPRB ,&
&  0.29375E+00_JPRB, 0.29013E+00_JPRB, 0.28532E+00_JPRB, 0.28257E+00_JPRB, 0.28176E+00_JPRB, 0.28067E+00_JPRB, 0.27936E+00_JPRB ,&
&  0.27850E+00_JPRB, 0.27787E+00_JPRB, 0.27672E+00_JPRB, 0.27469E+00_JPRB, 0.27132E+00_JPRB, 0.26668E+00_JPRB, 0.26201E+00_JPRB ,&
&  0.25901E+00_JPRB, 0.25696E+00_JPRB, 0.25702E+00_JPRB, 0.25509E+00_JPRB, 0.25109E+00_JPRB, 0.24736E+00_JPRB, 0.24681E+00_JPRB ,&
&  0.24680E+00_JPRB, 0.24725E+00_JPRB, 0.24713E+00_JPRB, 0.24612E+00_JPRB, 0.24647E+00_JPRB, 0.24745E+00_JPRB, 0.24899E+00_JPRB ,&
&  0.25301E+00_JPRB, 0.25582E+00_JPRB, 0.25568E+00_JPRB, 0.25458E+00_JPRB, 0.25266E+00_JPRB, 0.25220E+00_JPRB, 0.25276E+00_JPRB ,&
&  0.25331E+00_JPRB, 0.25351E+00_JPRB, 0.25368E+00_JPRB, 0.25309E+00_JPRB, 0.25320E+00_JPRB, 0.25466E+00_JPRB, 0.25650E+00_JPRB ,&
&  0.25872E+00_JPRB, 0.26145E+00_JPRB, 0.26388E+00_JPRB, 0.26657E+00_JPRB, 0.26921E+00_JPRB, 0.27090E+00_JPRB, 0.27240E+00_JPRB ,&
&  0.27368E+00_JPRB, 0.27397E+00_JPRB, 0.27361E+00_JPRB, 0.27336E+00_JPRB, 0.27310E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.30092E+00_JPRB, 0.30092E+00_JPRB, 0.30175E+00_JPRB, 0.30206E+00_JPRB, 0.30177E+00_JPRB, 0.30221E+00_JPRB, 0.30468E+00_JPRB ,&
&  0.30680E+00_JPRB, 0.30796E+00_JPRB, 0.30884E+00_JPRB, 0.30905E+00_JPRB, 0.30892E+00_JPRB, 0.31030E+00_JPRB, 0.31139E+00_JPRB ,&
&  0.31144E+00_JPRB, 0.31029E+00_JPRB, 0.30901E+00_JPRB, 0.30746E+00_JPRB, 0.30538E+00_JPRB, 0.30322E+00_JPRB, 0.30154E+00_JPRB ,&
&  0.29930E+00_JPRB, 0.29680E+00_JPRB, 0.29439E+00_JPRB, 0.29359E+00_JPRB, 0.29377E+00_JPRB, 0.29412E+00_JPRB, 0.29519E+00_JPRB ,&
&  0.29621E+00_JPRB, 0.29883E+00_JPRB, 0.30194E+00_JPRB, 0.30554E+00_JPRB, 0.30910E+00_JPRB, 0.31382E+00_JPRB, 0.31899E+00_JPRB ,&
&  0.32378E+00_JPRB, 0.32618E+00_JPRB, 0.32798E+00_JPRB, 0.32623E+00_JPRB, 0.32612E+00_JPRB, 0.32575E+00_JPRB, 0.32544E+00_JPRB ,&
&  0.32433E+00_JPRB, 0.31940E+00_JPRB, 0.31541E+00_JPRB, 0.31501E+00_JPRB, 0.31582E+00_JPRB, 0.31643E+00_JPRB, 0.31670E+00_JPRB ,&
&  0.31591E+00_JPRB, 0.31450E+00_JPRB, 0.31151E+00_JPRB, 0.30737E+00_JPRB, 0.30294E+00_JPRB, 0.29758E+00_JPRB, 0.28996E+00_JPRB ,&
&  0.28279E+00_JPRB, 0.27695E+00_JPRB, 0.27252E+00_JPRB, 0.26870E+00_JPRB, 0.26607E+00_JPRB, 0.26319E+00_JPRB, 0.26227E+00_JPRB ,&
&  0.26256E+00_JPRB, 0.26439E+00_JPRB, 0.26548E+00_JPRB, 0.26487E+00_JPRB, 0.26378E+00_JPRB, 0.26401E+00_JPRB, 0.26623E+00_JPRB ,&
&  0.27026E+00_JPRB, 0.27335E+00_JPRB, 0.27435E+00_JPRB, 0.27396E+00_JPRB, 0.27317E+00_JPRB, 0.27398E+00_JPRB, 0.27574E+00_JPRB ,&
&  0.27661E+00_JPRB, 0.27712E+00_JPRB, 0.27826E+00_JPRB, 0.27818E+00_JPRB, 0.28011E+00_JPRB, 0.28335E+00_JPRB, 0.28691E+00_JPRB ,&
&  0.29033E+00_JPRB, 0.29464E+00_JPRB, 0.29921E+00_JPRB, 0.30339E+00_JPRB, 0.30736E+00_JPRB, 0.30850E+00_JPRB, 0.30966E+00_JPRB ,&
&  0.31029E+00_JPRB, 0.30926E+00_JPRB, 0.30774E+00_JPRB, 0.30550E+00_JPRB, 0.30275E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.33520E+00_JPRB, 0.33345E+00_JPRB, 0.33186E+00_JPRB, 0.33094E+00_JPRB, 0.33007E+00_JPRB, 0.32917E+00_JPRB, 0.32874E+00_JPRB ,&
&  0.32903E+00_JPRB, 0.32889E+00_JPRB, 0.33114E+00_JPRB, 0.33178E+00_JPRB, 0.33162E+00_JPRB, 0.33233E+00_JPRB, 0.33280E+00_JPRB ,&
&  0.33137E+00_JPRB, 0.33022E+00_JPRB, 0.32902E+00_JPRB, 0.32681E+00_JPRB, 0.32327E+00_JPRB, 0.32147E+00_JPRB, 0.31997E+00_JPRB ,&
&  0.31748E+00_JPRB, 0.31447E+00_JPRB, 0.31192E+00_JPRB, 0.31083E+00_JPRB, 0.31137E+00_JPRB, 0.31142E+00_JPRB, 0.31239E+00_JPRB ,&
&  0.31284E+00_JPRB, 0.31623E+00_JPRB, 0.30490E+00_JPRB, 0.31458E+00_JPRB, 0.32373E+00_JPRB, 0.33280E+00_JPRB, 0.34232E+00_JPRB ,&
&  0.34889E+00_JPRB, 0.35501E+00_JPRB, 0.36209E+00_JPRB, 0.36446E+00_JPRB, 0.36362E+00_JPRB, 0.36052E+00_JPRB, 0.35957E+00_JPRB ,&
&  0.35906E+00_JPRB, 0.35657E+00_JPRB, 0.35496E+00_JPRB, 0.35510E+00_JPRB, 0.35559E+00_JPRB, 0.35451E+00_JPRB, 0.35260E+00_JPRB ,&
&  0.35099E+00_JPRB, 0.34844E+00_JPRB, 0.34409E+00_JPRB, 0.33916E+00_JPRB, 0.33392E+00_JPRB, 0.32750E+00_JPRB, 0.31889E+00_JPRB ,&
&  0.30906E+00_JPRB, 0.30118E+00_JPRB, 0.29450E+00_JPRB, 0.29025E+00_JPRB, 0.28809E+00_JPRB, 0.28590E+00_JPRB, 0.28386E+00_JPRB ,&
&  0.28332E+00_JPRB, 0.28377E+00_JPRB, 0.28407E+00_JPRB, 0.28403E+00_JPRB, 0.28492E+00_JPRB, 0.28571E+00_JPRB, 0.28834E+00_JPRB ,&
&  0.29119E+00_JPRB, 0.29225E+00_JPRB, 0.29231E+00_JPRB, 0.29250E+00_JPRB, 0.29129E+00_JPRB, 0.29186E+00_JPRB, 0.29360E+00_JPRB ,&
&  0.29555E+00_JPRB, 0.29818E+00_JPRB, 0.30015E+00_JPRB, 0.30192E+00_JPRB, 0.30425E+00_JPRB, 0.30778E+00_JPRB, 0.31312E+00_JPRB ,&
&  0.31826E+00_JPRB, 0.32422E+00_JPRB, 0.33137E+00_JPRB, 0.33843E+00_JPRB, 0.34351E+00_JPRB, 0.34635E+00_JPRB, 0.34882E+00_JPRB ,&
&  0.34935E+00_JPRB, 0.34637E+00_JPRB, 0.34255E+00_JPRB, 0.33979E+00_JPRB, 0.33735E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.38045E+00_JPRB, 0.37659E+00_JPRB, 0.37191E+00_JPRB, 0.36823E+00_JPRB, 0.36465E+00_JPRB, 0.36055E+00_JPRB, 0.35801E+00_JPRB ,&
&  0.35674E+00_JPRB, 0.35635E+00_JPRB, 0.35703E+00_JPRB, 0.35697E+00_JPRB, 0.35664E+00_JPRB, 0.35700E+00_JPRB, 0.35640E+00_JPRB ,&
&  0.35534E+00_JPRB, 0.35380E+00_JPRB, 0.35189E+00_JPRB, 0.34924E+00_JPRB, 0.34595E+00_JPRB, 0.34292E+00_JPRB, 0.33888E+00_JPRB ,&
&  0.33613E+00_JPRB, 0.33467E+00_JPRB, 0.33398E+00_JPRB, 0.33400E+00_JPRB, 0.33538E+00_JPRB, 0.33721E+00_JPRB, 0.33895E+00_JPRB ,&
&  0.33895E+00_JPRB, 0.32940E+00_JPRB, 0.33506E+00_JPRB, 0.33826E+00_JPRB, 0.34711E+00_JPRB, 0.36189E+00_JPRB, 0.37502E+00_JPRB ,&
&  0.38320E+00_JPRB, 0.39224E+00_JPRB, 0.40058E+00_JPRB, 0.40528E+00_JPRB, 0.40382E+00_JPRB, 0.40186E+00_JPRB, 0.40342E+00_JPRB ,&
&  0.40448E+00_JPRB, 0.40346E+00_JPRB, 0.40230E+00_JPRB, 0.40043E+00_JPRB, 0.39741E+00_JPRB, 0.39406E+00_JPRB, 0.38882E+00_JPRB ,&
&  0.38354E+00_JPRB, 0.37964E+00_JPRB, 0.37565E+00_JPRB, 0.37153E+00_JPRB, 0.36548E+00_JPRB, 0.35836E+00_JPRB, 0.34998E+00_JPRB ,&
&  0.34102E+00_JPRB, 0.33438E+00_JPRB, 0.32989E+00_JPRB, 0.32564E+00_JPRB, 0.32152E+00_JPRB, 0.31875E+00_JPRB, 0.31569E+00_JPRB ,&
&  0.31457E+00_JPRB, 0.31531E+00_JPRB, 0.31479E+00_JPRB, 0.31457E+00_JPRB, 0.31528E+00_JPRB, 0.31518E+00_JPRB, 0.31559E+00_JPRB ,&
&  0.31450E+00_JPRB, 0.31315E+00_JPRB, 0.31235E+00_JPRB, 0.31183E+00_JPRB, 0.31138E+00_JPRB, 0.31278E+00_JPRB, 0.31529E+00_JPRB ,&
&  0.31800E+00_JPRB, 0.32194E+00_JPRB, 0.32622E+00_JPRB, 0.32951E+00_JPRB, 0.33240E+00_JPRB, 0.33703E+00_JPRB, 0.34298E+00_JPRB ,&
&  0.34999E+00_JPRB, 0.35857E+00_JPRB, 0.36734E+00_JPRB, 0.37624E+00_JPRB, 0.38240E+00_JPRB, 0.38590E+00_JPRB, 0.38876E+00_JPRB ,&
&  0.39049E+00_JPRB, 0.38736E+00_JPRB, 0.38645E+00_JPRB, 0.38590E+00_JPRB, 0.38352E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.41461E+00_JPRB, 0.41246E+00_JPRB, 0.40853E+00_JPRB, 0.40370E+00_JPRB, 0.39840E+00_JPRB, 0.39538E+00_JPRB, 0.39368E+00_JPRB ,&
&  0.39423E+00_JPRB, 0.39315E+00_JPRB, 0.39350E+00_JPRB, 0.39077E+00_JPRB, 0.38774E+00_JPRB, 0.38538E+00_JPRB, 0.38290E+00_JPRB ,&
&  0.38009E+00_JPRB, 0.37664E+00_JPRB, 0.37330E+00_JPRB, 0.37048E+00_JPRB, 0.36711E+00_JPRB, 0.36257E+00_JPRB, 0.35840E+00_JPRB ,&
&  0.35534E+00_JPRB, 0.35405E+00_JPRB, 0.35342E+00_JPRB, 0.35312E+00_JPRB, 0.35293E+00_JPRB, 0.35425E+00_JPRB, 0.35533E+00_JPRB ,&
&  0.35810E+00_JPRB, 0.32969E+00_JPRB, 0.34064E+00_JPRB, 0.34636E+00_JPRB, 0.35555E+00_JPRB, 0.37312E+00_JPRB, 0.39364E+00_JPRB ,&
&  0.41512E+00_JPRB, 0.43482E+00_JPRB, 0.44845E+00_JPRB, 0.45066E+00_JPRB, 0.44440E+00_JPRB, 0.43997E+00_JPRB, 0.44008E+00_JPRB ,&
&  0.43828E+00_JPRB, 0.43401E+00_JPRB, 0.43023E+00_JPRB, 0.42589E+00_JPRB, 0.41961E+00_JPRB, 0.41053E+00_JPRB, 0.40140E+00_JPRB ,&
&  0.39617E+00_JPRB, 0.39327E+00_JPRB, 0.39070E+00_JPRB, 0.38785E+00_JPRB, 0.38297E+00_JPRB, 0.37837E+00_JPRB, 0.37437E+00_JPRB ,&
&  0.37044E+00_JPRB, 0.36458E+00_JPRB, 0.36079E+00_JPRB, 0.35821E+00_JPRB, 0.35489E+00_JPRB, 0.35284E+00_JPRB, 0.35131E+00_JPRB ,&
&  0.34962E+00_JPRB, 0.34752E+00_JPRB, 0.34615E+00_JPRB, 0.34552E+00_JPRB, 0.34325E+00_JPRB, 0.34088E+00_JPRB, 0.33815E+00_JPRB ,&
&  0.33690E+00_JPRB, 0.33608E+00_JPRB, 0.33506E+00_JPRB, 0.33496E+00_JPRB, 0.33587E+00_JPRB, 0.33832E+00_JPRB, 0.34161E+00_JPRB ,&
&  0.34552E+00_JPRB, 0.35008E+00_JPRB, 0.35426E+00_JPRB, 0.35772E+00_JPRB, 0.36241E+00_JPRB, 0.36643E+00_JPRB, 0.37202E+00_JPRB ,&
&  0.37915E+00_JPRB, 0.38683E+00_JPRB, 0.39501E+00_JPRB, 0.40543E+00_JPRB, 0.41639E+00_JPRB, 0.42261E+00_JPRB, 0.42749E+00_JPRB ,&
&  0.43085E+00_JPRB, 0.42688E+00_JPRB, 0.42346E+00_JPRB, 0.41884E+00_JPRB, 0.41513E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.45888E+00_JPRB, 0.45525E+00_JPRB, 0.45051E+00_JPRB, 0.44434E+00_JPRB, 0.43892E+00_JPRB, 0.43489E+00_JPRB, 0.43259E+00_JPRB ,&
&  0.43244E+00_JPRB, 0.43304E+00_JPRB, 0.43133E+00_JPRB, 0.42607E+00_JPRB, 0.42094E+00_JPRB, 0.41738E+00_JPRB, 0.41530E+00_JPRB ,&
&  0.41251E+00_JPRB, 0.40809E+00_JPRB, 0.40234E+00_JPRB, 0.39679E+00_JPRB, 0.39067E+00_JPRB, 0.38566E+00_JPRB, 0.38159E+00_JPRB ,&
&  0.37836E+00_JPRB, 0.37700E+00_JPRB, 0.37611E+00_JPRB, 0.37562E+00_JPRB, 0.37636E+00_JPRB, 0.37636E+00_JPRB, 0.38137E+00_JPRB ,&
&  0.35625E+00_JPRB, 0.32631E+00_JPRB, 0.34177E+00_JPRB, 0.35699E+00_JPRB, 0.36862E+00_JPRB, 0.39401E+00_JPRB, 0.42902E+00_JPRB ,&
&  0.46834E+00_JPRB, 0.50216E+00_JPRB, 0.52263E+00_JPRB, 0.51996E+00_JPRB, 0.50688E+00_JPRB, 0.49705E+00_JPRB, 0.49127E+00_JPRB ,&
&  0.48096E+00_JPRB, 0.46936E+00_JPRB, 0.45995E+00_JPRB, 0.44903E+00_JPRB, 0.43733E+00_JPRB, 0.42636E+00_JPRB, 0.41742E+00_JPRB ,&
&  0.41032E+00_JPRB, 0.40604E+00_JPRB, 0.40488E+00_JPRB, 0.40341E+00_JPRB, 0.40224E+00_JPRB, 0.40270E+00_JPRB, 0.40181E+00_JPRB ,&
&  0.40032E+00_JPRB, 0.39800E+00_JPRB, 0.39694E+00_JPRB, 0.39578E+00_JPRB, 0.39577E+00_JPRB, 0.39479E+00_JPRB, 0.39300E+00_JPRB ,&
&  0.39056E+00_JPRB, 0.38858E+00_JPRB, 0.38750E+00_JPRB, 0.38460E+00_JPRB, 0.37769E+00_JPRB, 0.37104E+00_JPRB, 0.36734E+00_JPRB ,&
&  0.36619E+00_JPRB, 0.36593E+00_JPRB, 0.36571E+00_JPRB, 0.36676E+00_JPRB, 0.36872E+00_JPRB, 0.37135E+00_JPRB, 0.37514E+00_JPRB ,&
&  0.38016E+00_JPRB, 0.38545E+00_JPRB, 0.38983E+00_JPRB, 0.39401E+00_JPRB, 0.40036E+00_JPRB, 0.40664E+00_JPRB, 0.41331E+00_JPRB ,&
&  0.42048E+00_JPRB, 0.42881E+00_JPRB, 0.43801E+00_JPRB, 0.45267E+00_JPRB, 0.47081E+00_JPRB, 0.48454E+00_JPRB, 0.48886E+00_JPRB ,&
&  0.48535E+00_JPRB, 0.47278E+00_JPRB, 0.46954E+00_JPRB, 0.46293E+00_JPRB, 0.45952E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.48655E+00_JPRB, 0.48728E+00_JPRB, 0.48384E+00_JPRB, 0.47924E+00_JPRB, 0.47573E+00_JPRB, 0.47058E+00_JPRB, 0.46625E+00_JPRB ,&
&  0.46062E+00_JPRB, 0.45656E+00_JPRB, 0.44993E+00_JPRB, 0.44542E+00_JPRB, 0.44248E+00_JPRB, 0.44098E+00_JPRB, 0.43880E+00_JPRB ,&
&  0.43406E+00_JPRB, 0.42668E+00_JPRB, 0.41925E+00_JPRB, 0.41337E+00_JPRB, 0.40922E+00_JPRB, 0.40504E+00_JPRB, 0.40130E+00_JPRB ,&
&  0.39788E+00_JPRB, 0.39473E+00_JPRB, 0.39177E+00_JPRB, 0.39178E+00_JPRB, 0.39210E+00_JPRB, 0.39158E+00_JPRB, 0.38986E+00_JPRB ,&
&  0.39675E+00_JPRB, 0.34956E+00_JPRB, 0.36876E+00_JPRB, 0.38711E+00_JPRB, 0.40142E+00_JPRB, 0.43800E+00_JPRB, 0.49119E+00_JPRB ,&
&  0.53769E+00_JPRB, 0.57377E+00_JPRB, 0.58815E+00_JPRB, 0.57645E+00_JPRB, 0.55856E+00_JPRB, 0.54650E+00_JPRB, 0.53483E+00_JPRB ,&
&  0.51454E+00_JPRB, 0.49711E+00_JPRB, 0.48185E+00_JPRB, 0.46699E+00_JPRB, 0.45286E+00_JPRB, 0.44084E+00_JPRB, 0.43030E+00_JPRB ,&
&  0.42364E+00_JPRB, 0.42088E+00_JPRB, 0.41898E+00_JPRB, 0.41759E+00_JPRB, 0.41828E+00_JPRB, 0.42069E+00_JPRB, 0.42038E+00_JPRB ,&
&  0.41971E+00_JPRB, 0.42167E+00_JPRB, 0.42163E+00_JPRB, 0.42150E+00_JPRB, 0.42392E+00_JPRB, 0.43023E+00_JPRB, 0.43117E+00_JPRB ,&
&  0.42745E+00_JPRB, 0.42067E+00_JPRB, 0.41381E+00_JPRB, 0.40840E+00_JPRB, 0.40384E+00_JPRB, 0.39936E+00_JPRB, 0.39773E+00_JPRB ,&
&  0.39576E+00_JPRB, 0.39567E+00_JPRB, 0.39611E+00_JPRB, 0.39854E+00_JPRB, 0.40149E+00_JPRB, 0.40523E+00_JPRB, 0.41017E+00_JPRB ,&
&  0.41614E+00_JPRB, 0.42233E+00_JPRB, 0.43031E+00_JPRB, 0.43897E+00_JPRB, 0.44664E+00_JPRB, 0.45227E+00_JPRB, 0.45663E+00_JPRB ,&
&  0.46108E+00_JPRB, 0.46879E+00_JPRB, 0.47469E+00_JPRB, 0.50277E+00_JPRB, 0.52163E+00_JPRB, 0.53833E+00_JPRB, 0.53597E+00_JPRB ,&
&  0.52401E+00_JPRB, 0.50986E+00_JPRB, 0.43658E+00_JPRB, 0.46822E+00_JPRB, 0.47969E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.54045E+00_JPRB, 0.53752E+00_JPRB, 0.53234E+00_JPRB, 0.52827E+00_JPRB, 0.52013E+00_JPRB, 0.51140E+00_JPRB, 0.50286E+00_JPRB ,&
&  0.49221E+00_JPRB, 0.48142E+00_JPRB, 0.47200E+00_JPRB, 0.46822E+00_JPRB, 0.46489E+00_JPRB, 0.46131E+00_JPRB, 0.45379E+00_JPRB ,&
&  0.44375E+00_JPRB, 0.43418E+00_JPRB, 0.42579E+00_JPRB, 0.42071E+00_JPRB, 0.41946E+00_JPRB, 0.41757E+00_JPRB, 0.41456E+00_JPRB ,&
&  0.41266E+00_JPRB, 0.41328E+00_JPRB, 0.41243E+00_JPRB, 0.41141E+00_JPRB, 0.40995E+00_JPRB, 0.40488E+00_JPRB, 0.40484E+00_JPRB ,&
&  0.41525E+00_JPRB, 0.38232E+00_JPRB, 0.42383E+00_JPRB, 0.46493E+00_JPRB, 0.49264E+00_JPRB, 0.53718E+00_JPRB, 0.59650E+00_JPRB ,&
&  0.65392E+00_JPRB, 0.67007E+00_JPRB, 0.66202E+00_JPRB, 0.63026E+00_JPRB, 0.60512E+00_JPRB, 0.58575E+00_JPRB, 0.55816E+00_JPRB ,&
&  0.53131E+00_JPRB, 0.51306E+00_JPRB, 0.49952E+00_JPRB, 0.48801E+00_JPRB, 0.47961E+00_JPRB, 0.47271E+00_JPRB, 0.46693E+00_JPRB ,&
&  0.45958E+00_JPRB, 0.45225E+00_JPRB, 0.44567E+00_JPRB, 0.44219E+00_JPRB, 0.44327E+00_JPRB, 0.44766E+00_JPRB, 0.45436E+00_JPRB ,&
&  0.46057E+00_JPRB, 0.46509E+00_JPRB, 0.46619E+00_JPRB, 0.46943E+00_JPRB, 0.47552E+00_JPRB, 0.48028E+00_JPRB, 0.47801E+00_JPRB ,&
&  0.46800E+00_JPRB, 0.45619E+00_JPRB, 0.44766E+00_JPRB, 0.44230E+00_JPRB, 0.44006E+00_JPRB, 0.44053E+00_JPRB, 0.44310E+00_JPRB ,&
&  0.44300E+00_JPRB, 0.44156E+00_JPRB, 0.44022E+00_JPRB, 0.44109E+00_JPRB, 0.44305E+00_JPRB, 0.44797E+00_JPRB, 0.45521E+00_JPRB ,&
&  0.46360E+00_JPRB, 0.47255E+00_JPRB, 0.48286E+00_JPRB, 0.49300E+00_JPRB, 0.49896E+00_JPRB, 0.50436E+00_JPRB, 0.50736E+00_JPRB ,&
&  0.51052E+00_JPRB, 0.52016E+00_JPRB, 0.53143E+00_JPRB, 0.59217E+00_JPRB, 0.62095E+00_JPRB, 0.62185E+00_JPRB, 0.59053E+00_JPRB ,&
&  0.56218E+00_JPRB, 0.53514E+00_JPRB, 0.53052E+00_JPRB, 0.49236E+00_JPRB, 0.53108E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.60321E+00_JPRB, 0.59459E+00_JPRB, 0.58342E+00_JPRB, 0.57349E+00_JPRB, 0.55957E+00_JPRB, 0.54760E+00_JPRB, 0.53274E+00_JPRB ,&
&  0.51912E+00_JPRB, 0.50596E+00_JPRB, 0.49451E+00_JPRB, 0.49006E+00_JPRB, 0.48248E+00_JPRB, 0.47171E+00_JPRB, 0.45421E+00_JPRB ,&
&  0.43976E+00_JPRB, 0.42861E+00_JPRB, 0.42212E+00_JPRB, 0.41981E+00_JPRB, 0.41836E+00_JPRB, 0.41552E+00_JPRB, 0.41417E+00_JPRB ,&
&  0.41482E+00_JPRB, 0.41424E+00_JPRB, 0.41628E+00_JPRB, 0.41567E+00_JPRB, 0.41873E+00_JPRB, 0.41482E+00_JPRB, 0.41215E+00_JPRB ,&
&  0.42957E+00_JPRB, 0.42991E+00_JPRB, 0.50026E+00_JPRB, 0.57660E+00_JPRB, 0.64545E+00_JPRB, 0.69342E+00_JPRB, 0.72836E+00_JPRB ,&
&  0.75341E+00_JPRB, 0.73948E+00_JPRB, 0.70953E+00_JPRB, 0.66460E+00_JPRB, 0.62829E+00_JPRB, 0.58959E+00_JPRB, 0.55185E+00_JPRB ,&
&  0.53284E+00_JPRB, 0.52300E+00_JPRB, 0.51467E+00_JPRB, 0.50725E+00_JPRB, 0.49966E+00_JPRB, 0.49536E+00_JPRB, 0.49355E+00_JPRB ,&
&  0.48799E+00_JPRB, 0.48080E+00_JPRB, 0.47197E+00_JPRB, 0.46843E+00_JPRB, 0.46767E+00_JPRB, 0.47960E+00_JPRB, 0.50043E+00_JPRB ,&
&  0.51140E+00_JPRB, 0.52118E+00_JPRB, 0.53034E+00_JPRB, 0.54338E+00_JPRB, 0.54873E+00_JPRB, 0.54644E+00_JPRB, 0.52936E+00_JPRB ,&
&  0.51499E+00_JPRB, 0.50535E+00_JPRB, 0.50033E+00_JPRB, 0.49804E+00_JPRB, 0.49761E+00_JPRB, 0.49589E+00_JPRB, 0.49566E+00_JPRB ,&
&  0.49198E+00_JPRB, 0.48767E+00_JPRB, 0.48651E+00_JPRB, 0.48736E+00_JPRB, 0.48915E+00_JPRB, 0.49132E+00_JPRB, 0.49463E+00_JPRB ,&
&  0.49960E+00_JPRB, 0.50520E+00_JPRB, 0.51277E+00_JPRB, 0.51961E+00_JPRB, 0.52116E+00_JPRB, 0.52444E+00_JPRB, 0.53233E+00_JPRB ,&
&  0.54054E+00_JPRB, 0.55772E+00_JPRB, 0.57348E+00_JPRB, 0.63865E+00_JPRB, 0.66678E+00_JPRB, 0.64893E+00_JPRB, 0.60778E+00_JPRB ,&
&  0.57546E+00_JPRB, 0.56015E+00_JPRB, 0.55960E+00_JPRB, 0.58106E+00_JPRB, 0.58002E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.72106E+00_JPRB, 0.67581E+00_JPRB, 0.63399E+00_JPRB, 0.60792E+00_JPRB, 0.59007E+00_JPRB, 0.57642E+00_JPRB, 0.56017E+00_JPRB ,&
&  0.54480E+00_JPRB, 0.52954E+00_JPRB, 0.51724E+00_JPRB, 0.50835E+00_JPRB, 0.49904E+00_JPRB, 0.48109E+00_JPRB, 0.45750E+00_JPRB ,&
&  0.44171E+00_JPRB, 0.43841E+00_JPRB, 0.43619E+00_JPRB, 0.43079E+00_JPRB, 0.42546E+00_JPRB, 0.42356E+00_JPRB, 0.42072E+00_JPRB ,&
&  0.42029E+00_JPRB, 0.42024E+00_JPRB, 0.42295E+00_JPRB, 0.42422E+00_JPRB, 0.42219E+00_JPRB, 0.42835E+00_JPRB, 0.42800E+00_JPRB ,&
&  0.44865E+00_JPRB, 0.47848E+00_JPRB, 0.59530E+00_JPRB, 0.72864E+00_JPRB, 0.82878E+00_JPRB, 0.86770E+00_JPRB, 0.84552E+00_JPRB ,&
&  0.81804E+00_JPRB, 0.78705E+00_JPRB, 0.74480E+00_JPRB, 0.70171E+00_JPRB, 0.65372E+00_JPRB, 0.60239E+00_JPRB, 0.57202E+00_JPRB ,&
&  0.56186E+00_JPRB, 0.55002E+00_JPRB, 0.53756E+00_JPRB, 0.52239E+00_JPRB, 0.50811E+00_JPRB, 0.50383E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.50280E+00_JPRB, 0.49917E+00_JPRB, 0.49511E+00_JPRB, 0.49807E+00_JPRB, 0.51690E+00_JPRB, 0.56098E+00_JPRB, 0.60606E+00_JPRB ,&
&  0.62551E+00_JPRB, 0.64483E+00_JPRB, 0.65624E+00_JPRB, 0.66067E+00_JPRB, 0.65089E+00_JPRB, 0.63186E+00_JPRB, 0.60819E+00_JPRB ,&
&  0.58761E+00_JPRB, 0.57248E+00_JPRB, 0.56498E+00_JPRB, 0.55987E+00_JPRB, 0.55538E+00_JPRB, 0.55027E+00_JPRB, 0.54405E+00_JPRB ,&
&  0.54014E+00_JPRB, 0.53832E+00_JPRB, 0.53897E+00_JPRB, 0.53750E+00_JPRB, 0.53566E+00_JPRB, 0.53255E+00_JPRB, 0.52938E+00_JPRB ,&
&  0.52494E+00_JPRB, 0.52294E+00_JPRB, 0.53204E+00_JPRB, 0.54820E+00_JPRB, 0.56614E+00_JPRB, 0.57925E+00_JPRB, 0.58879E+00_JPRB ,&
&  0.59634E+00_JPRB, 0.62303E+00_JPRB, 0.62892E+00_JPRB, 0.72333E+00_JPRB, 0.79519E+00_JPRB, 0.73374E+00_JPRB, 0.68140E+00_JPRB ,&
&  0.63857E+00_JPRB, 0.61840E+00_JPRB, 0.61477E+00_JPRB, 0.63864E+00_JPRB, 0.71783E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.67021E+00_JPRB, 0.62860E+00_JPRB, 0.60657E+00_JPRB, 0.59399E+00_JPRB, 0.58696E+00_JPRB, 0.58086E+00_JPRB, 0.56855E+00_JPRB ,&
&  0.55179E+00_JPRB, 0.53694E+00_JPRB, 0.52592E+00_JPRB, 0.51767E+00_JPRB, 0.50171E+00_JPRB, 0.47729E+00_JPRB, 0.45919E+00_JPRB ,&
&  0.45550E+00_JPRB, 0.45243E+00_JPRB, 0.44999E+00_JPRB, 0.44342E+00_JPRB, 0.43671E+00_JPRB, 0.43345E+00_JPRB, 0.42960E+00_JPRB ,&
&  0.42496E+00_JPRB, 0.42067E+00_JPRB, 0.42175E+00_JPRB, 0.42552E+00_JPRB, 0.42918E+00_JPRB, 0.43260E+00_JPRB, 0.43677E+00_JPRB ,&
&  0.46059E+00_JPRB, 0.59694E+00_JPRB, 0.78199E+00_JPRB, 0.10248E+01_JPRB, 0.11368E+01_JPRB, 0.10786E+01_JPRB, 0.10211E+01_JPRB ,&
&  0.94685E+00_JPRB, 0.86610E+00_JPRB, 0.79588E+00_JPRB, 0.73306E+00_JPRB, 0.66416E+00_JPRB, 0.62535E+00_JPRB, 0.60727E+00_JPRB ,&
&  0.59307E+00_JPRB, 0.57380E+00_JPRB, 0.55393E+00_JPRB, 0.53356E+00_JPRB, 0.52007E+00_JPRB, 0.52353E+00_JPRB, 0.53400E+00_JPRB ,&
&  0.53343E+00_JPRB, 0.53465E+00_JPRB, 0.53760E+00_JPRB, 0.56310E+00_JPRB, 0.64377E+00_JPRB, 0.72103E+00_JPRB, 0.80253E+00_JPRB ,&
&  0.82829E+00_JPRB, 0.88041E+00_JPRB, 0.84834E+00_JPRB, 0.81078E+00_JPRB, 0.76950E+00_JPRB, 0.73282E+00_JPRB, 0.70322E+00_JPRB ,&
&  0.68231E+00_JPRB, 0.66465E+00_JPRB, 0.64918E+00_JPRB, 0.63595E+00_JPRB, 0.62370E+00_JPRB, 0.61070E+00_JPRB, 0.60525E+00_JPRB ,&
&  0.60291E+00_JPRB, 0.59465E+00_JPRB, 0.58665E+00_JPRB, 0.57784E+00_JPRB, 0.56429E+00_JPRB, 0.55165E+00_JPRB, 0.54190E+00_JPRB ,&
&  0.53384E+00_JPRB, 0.53638E+00_JPRB, 0.56813E+00_JPRB, 0.59963E+00_JPRB, 0.62044E+00_JPRB, 0.63874E+00_JPRB, 0.65312E+00_JPRB ,&
&  0.66766E+00_JPRB, 0.70857E+00_JPRB, 0.69887E+00_JPRB, 0.75002E+00_JPRB, 0.84665E+00_JPRB, 0.80778E+00_JPRB, 0.73056E+00_JPRB ,&
&  0.69593E+00_JPRB, 0.67639E+00_JPRB, 0.66484E+00_JPRB, 0.66169E+00_JPRB, 0.71101E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.63921E+00_JPRB, 0.61231E+00_JPRB, 0.59892E+00_JPRB, 0.59041E+00_JPRB, 0.58341E+00_JPRB, 0.57720E+00_JPRB, 0.56493E+00_JPRB ,&
&  0.54814E+00_JPRB, 0.53780E+00_JPRB, 0.52605E+00_JPRB, 0.51017E+00_JPRB, 0.48789E+00_JPRB, 0.47367E+00_JPRB, 0.47186E+00_JPRB ,&
&  0.47037E+00_JPRB, 0.46622E+00_JPRB, 0.45843E+00_JPRB, 0.45446E+00_JPRB, 0.44875E+00_JPRB, 0.44157E+00_JPRB, 0.43721E+00_JPRB ,&
&  0.42779E+00_JPRB, 0.42374E+00_JPRB, 0.42593E+00_JPRB, 0.43147E+00_JPRB, 0.43624E+00_JPRB, 0.44242E+00_JPRB, 0.44947E+00_JPRB ,&
&  0.50443E+00_JPRB, 0.73866E+00_JPRB, 0.11378E+01_JPRB, 0.13659E+01_JPRB, 0.14055E+01_JPRB, 0.13140E+01_JPRB, 0.13052E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.10438E+01_JPRB, 0.92384E+00_JPRB, 0.81237E+00_JPRB, 0.72740E+00_JPRB, 0.68300E+00_JPRB, 0.65759E+00_JPRB ,&
&  0.62789E+00_JPRB, 0.60220E+00_JPRB, 0.57910E+00_JPRB, 0.55914E+00_JPRB, 0.55257E+00_JPRB, 0.56201E+00_JPRB, 0.57455E+00_JPRB ,&
&  0.57987E+00_JPRB, 0.58625E+00_JPRB, 0.60429E+00_JPRB, 0.66108E+00_JPRB, 0.76419E+00_JPRB, 0.83898E+00_JPRB, 0.10887E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.12689E+01_JPRB, 0.11462E+01_JPRB, 0.10447E+01_JPRB, 0.95446E+00_JPRB, 0.87997E+00_JPRB, 0.82454E+00_JPRB ,&
&  0.79368E+00_JPRB, 0.76903E+00_JPRB, 0.74878E+00_JPRB, 0.72643E+00_JPRB, 0.70530E+00_JPRB, 0.69105E+00_JPRB, 0.67897E+00_JPRB ,&
&  0.66453E+00_JPRB, 0.64825E+00_JPRB, 0.62926E+00_JPRB, 0.60463E+00_JPRB, 0.57845E+00_JPRB, 0.56371E+00_JPRB, 0.55874E+00_JPRB ,&
&  0.56228E+00_JPRB, 0.59210E+00_JPRB, 0.63921E+00_JPRB, 0.64649E+00_JPRB, 0.68025E+00_JPRB, 0.70817E+00_JPRB, 0.73569E+00_JPRB ,&
&  0.72356E+00_JPRB, 0.74312E+00_JPRB, 0.75607E+00_JPRB, 0.81202E+00_JPRB, 0.92401E+00_JPRB, 0.88590E+00_JPRB, 0.78087E+00_JPRB ,&
&  0.74156E+00_JPRB, 0.72507E+00_JPRB, 0.70535E+00_JPRB, 0.67989E+00_JPRB, 0.67605E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.62683E+00_JPRB, 0.60953E+00_JPRB, 0.59604E+00_JPRB, 0.58861E+00_JPRB, 0.58197E+00_JPRB, 0.56943E+00_JPRB, 0.55744E+00_JPRB ,&
&  0.54550E+00_JPRB, 0.53252E+00_JPRB, 0.51444E+00_JPRB, 0.48817E+00_JPRB, 0.46965E+00_JPRB, 0.46381E+00_JPRB, 0.46905E+00_JPRB ,&
&  0.47036E+00_JPRB, 0.46300E+00_JPRB, 0.45861E+00_JPRB, 0.45834E+00_JPRB, 0.45766E+00_JPRB, 0.45156E+00_JPRB, 0.44309E+00_JPRB ,&
&  0.43625E+00_JPRB, 0.43704E+00_JPRB, 0.44010E+00_JPRB, 0.44495E+00_JPRB, 0.44867E+00_JPRB, 0.45408E+00_JPRB, 0.47082E+00_JPRB ,&
&  0.54844E+00_JPRB, 0.55946E+00_JPRB, 0.12494E+01_JPRB, 0.16919E+01_JPRB, 0.17241E+01_JPRB, 0.16530E+01_JPRB, 0.16150E+01_JPRB ,&
&  0.14896E+01_JPRB, 0.13115E+01_JPRB, 0.11267E+01_JPRB, 0.96802E+00_JPRB, 0.84940E+00_JPRB, 0.77020E+00_JPRB, 0.71158E+00_JPRB ,&
&  0.66867E+00_JPRB, 0.63561E+00_JPRB, 0.60877E+00_JPRB, 0.59496E+00_JPRB, 0.59790E+00_JPRB, 0.60261E+00_JPRB, 0.60683E+00_JPRB ,&
&  0.61592E+00_JPRB, 0.63231E+00_JPRB, 0.67355E+00_JPRB, 0.76673E+00_JPRB, 0.90651E+00_JPRB, 0.10993E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.17899E+01_JPRB, 0.17408E+01_JPRB, 0.14888E+01_JPRB, 0.12835E+01_JPRB, 0.11409E+01_JPRB, 0.10479E+01_JPRB, 0.98105E+00_JPRB ,&
&  0.92792E+00_JPRB, 0.88774E+00_JPRB, 0.85422E+00_JPRB, 0.81107E+00_JPRB, 0.77866E+00_JPRB, 0.75639E+00_JPRB, 0.72802E+00_JPRB ,&
&  0.69571E+00_JPRB, 0.66487E+00_JPRB, 0.63481E+00_JPRB, 0.60825E+00_JPRB, 0.58663E+00_JPRB, 0.58420E+00_JPRB, 0.59122E+00_JPRB ,&
&  0.60632E+00_JPRB, 0.63765E+00_JPRB, 0.65939E+00_JPRB, 0.67553E+00_JPRB, 0.70905E+00_JPRB, 0.73473E+00_JPRB, 0.76085E+00_JPRB ,&
&  0.76589E+00_JPRB, 0.78663E+00_JPRB, 0.80996E+00_JPRB, 0.85877E+00_JPRB, 0.83451E+00_JPRB, 0.90189E+00_JPRB, 0.81159E+00_JPRB ,&
&  0.78484E+00_JPRB, 0.76097E+00_JPRB, 0.70975E+00_JPRB, 0.68287E+00_JPRB, 0.65643E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.62387E+00_JPRB, 0.60522E+00_JPRB, 0.59038E+00_JPRB, 0.57704E+00_JPRB, 0.56711E+00_JPRB, 0.56067E+00_JPRB, 0.55245E+00_JPRB ,&
&  0.54056E+00_JPRB, 0.52455E+00_JPRB, 0.50821E+00_JPRB, 0.48599E+00_JPRB, 0.47487E+00_JPRB, 0.47502E+00_JPRB, 0.48222E+00_JPRB ,&
&  0.47605E+00_JPRB, 0.46321E+00_JPRB, 0.46318E+00_JPRB, 0.46494E+00_JPRB, 0.46219E+00_JPRB, 0.45518E+00_JPRB, 0.44809E+00_JPRB ,&
&  0.45109E+00_JPRB, 0.45488E+00_JPRB, 0.45771E+00_JPRB, 0.46052E+00_JPRB, 0.46405E+00_JPRB, 0.47445E+00_JPRB, 0.50272E+00_JPRB ,&
&  0.59659E+00_JPRB, 0.99216E+00_JPRB, 0.10817E+01_JPRB, 0.19859E+01_JPRB, 0.21500E+01_JPRB, 0.20961E+01_JPRB, 0.18826E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.15401E+01_JPRB, 0.13206E+01_JPRB, 0.11466E+01_JPRB, 0.97687E+00_JPRB, 0.85051E+00_JPRB, 0.77141E+00_JPRB ,&
&  0.71907E+00_JPRB, 0.67963E+00_JPRB, 0.65625E+00_JPRB, 0.64886E+00_JPRB, 0.64450E+00_JPRB, 0.64408E+00_JPRB, 0.64970E+00_JPRB ,&
&  0.66402E+00_JPRB, 0.69817E+00_JPRB, 0.76822E+00_JPRB, 0.89334E+00_JPRB, 0.10030E+01_JPRB, 0.13713E+01_JPRB, 0.18503E+01_JPRB ,&
&  0.20838E+01_JPRB, 0.20928E+01_JPRB, 0.17469E+01_JPRB, 0.14525E+01_JPRB, 0.12725E+01_JPRB, 0.11630E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.96194E+00_JPRB, 0.90712E+00_JPRB, 0.83823E+00_JPRB, 0.79599E+00_JPRB, 0.76399E+00_JPRB, 0.72243E+00_JPRB ,&
&  0.67848E+00_JPRB, 0.64726E+00_JPRB, 0.62959E+00_JPRB, 0.61670E+00_JPRB, 0.61040E+00_JPRB, 0.61705E+00_JPRB, 0.62996E+00_JPRB ,&
&  0.64840E+00_JPRB, 0.68112E+00_JPRB, 0.68124E+00_JPRB, 0.69204E+00_JPRB, 0.72318E+00_JPRB, 0.76222E+00_JPRB, 0.76795E+00_JPRB ,&
&  0.78983E+00_JPRB, 0.81207E+00_JPRB, 0.83835E+00_JPRB, 0.87496E+00_JPRB, 0.85763E+00_JPRB, 0.95100E+00_JPRB, 0.82451E+00_JPRB ,&
&  0.80395E+00_JPRB, 0.74308E+00_JPRB, 0.69296E+00_JPRB, 0.67368E+00_JPRB, 0.65141E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.61601E+00_JPRB, 0.58788E+00_JPRB, 0.56911E+00_JPRB, 0.55781E+00_JPRB, 0.55184E+00_JPRB, 0.54986E+00_JPRB, 0.54186E+00_JPRB ,&
&  0.53377E+00_JPRB, 0.51755E+00_JPRB, 0.49747E+00_JPRB, 0.48042E+00_JPRB, 0.48432E+00_JPRB, 0.48616E+00_JPRB, 0.48568E+00_JPRB ,&
&  0.47764E+00_JPRB, 0.47404E+00_JPRB, 0.47450E+00_JPRB, 0.47253E+00_JPRB, 0.46802E+00_JPRB, 0.46407E+00_JPRB, 0.46347E+00_JPRB ,&
&  0.46914E+00_JPRB, 0.47025E+00_JPRB, 0.46984E+00_JPRB, 0.47361E+00_JPRB, 0.48424E+00_JPRB, 0.51123E+00_JPRB, 0.55369E+00_JPRB ,&
&  0.67752E+00_JPRB, 0.87730E+00_JPRB, 0.10564E+01_JPRB, 0.21332E+01_JPRB, 0.25218E+01_JPRB, 0.24333E+01_JPRB, 0.21062E+01_JPRB ,&
&  0.18068E+01_JPRB, 0.17595E+01_JPRB, 0.15048E+01_JPRB, 0.12512E+01_JPRB, 0.10318E+01_JPRB, 0.90669E+00_JPRB, 0.81702E+00_JPRB ,&
&  0.75513E+00_JPRB, 0.72324E+00_JPRB, 0.71372E+00_JPRB, 0.71200E+00_JPRB, 0.70995E+00_JPRB, 0.72234E+00_JPRB, 0.73762E+00_JPRB ,&
&  0.75549E+00_JPRB, 0.79876E+00_JPRB, 0.87380E+00_JPRB, 0.93575E+00_JPRB, 0.12268E+01_JPRB, 0.18586E+01_JPRB, 0.23107E+01_JPRB ,&
&  0.24075E+01_JPRB, 0.22637E+01_JPRB, 0.18957E+01_JPRB, 0.15512E+01_JPRB, 0.13209E+01_JPRB, 0.12275E+01_JPRB, 0.11320E+01_JPRB ,&
&  0.10308E+01_JPRB, 0.96577E+00_JPRB, 0.87782E+00_JPRB, 0.82095E+00_JPRB, 0.79568E+00_JPRB, 0.73796E+00_JPRB, 0.69045E+00_JPRB ,&
&  0.66475E+00_JPRB, 0.65344E+00_JPRB, 0.64734E+00_JPRB, 0.64032E+00_JPRB, 0.64149E+00_JPRB, 0.65132E+00_JPRB, 0.67007E+00_JPRB ,&
&  0.69623E+00_JPRB, 0.73180E+00_JPRB, 0.71836E+00_JPRB, 0.72929E+00_JPRB, 0.77426E+00_JPRB, 0.81774E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.82496E+00_JPRB, 0.84092E+00_JPRB, 0.85525E+00_JPRB, 0.83668E+00_JPRB, 0.81362E+00_JPRB, 0.87391E+00_JPRB, 0.82099E+00_JPRB ,&
&  0.81341E+00_JPRB, 0.74243E+00_JPRB, 0.70683E+00_JPRB, 0.66706E+00_JPRB, 0.64361E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.58734E+00_JPRB, 0.56079E+00_JPRB, 0.55485E+00_JPRB, 0.55004E+00_JPRB, 0.53747E+00_JPRB, 0.52718E+00_JPRB, 0.51810E+00_JPRB ,&
&  0.50822E+00_JPRB, 0.49148E+00_JPRB, 0.47855E+00_JPRB, 0.48347E+00_JPRB, 0.49919E+00_JPRB, 0.51042E+00_JPRB, 0.50203E+00_JPRB ,&
&  0.49193E+00_JPRB, 0.48701E+00_JPRB, 0.49330E+00_JPRB, 0.49571E+00_JPRB, 0.49259E+00_JPRB, 0.49522E+00_JPRB, 0.49509E+00_JPRB ,&
&  0.49591E+00_JPRB, 0.49310E+00_JPRB, 0.49582E+00_JPRB, 0.50936E+00_JPRB, 0.53454E+00_JPRB, 0.58011E+00_JPRB, 0.64863E+00_JPRB ,&
&  0.85672E+00_JPRB, 0.96839E+00_JPRB, 0.11659E+01_JPRB, 0.18833E+01_JPRB, 0.23720E+01_JPRB, 0.22338E+01_JPRB, 0.20551E+01_JPRB ,&
&  0.18414E+01_JPRB, 0.15100E+01_JPRB, 0.13787E+01_JPRB, 0.12422E+01_JPRB, 0.10655E+01_JPRB, 0.92754E+00_JPRB, 0.84259E+00_JPRB ,&
&  0.79968E+00_JPRB, 0.77424E+00_JPRB, 0.77589E+00_JPRB, 0.79549E+00_JPRB, 0.82880E+00_JPRB, 0.86684E+00_JPRB, 0.88891E+00_JPRB ,&
&  0.90826E+00_JPRB, 0.94740E+00_JPRB, 0.10322E+01_JPRB, 0.11682E+01_JPRB, 0.17144E+01_JPRB, 0.25147E+01_JPRB, 0.28346E+01_JPRB ,&
&  0.26557E+01_JPRB, 0.23163E+01_JPRB, 0.18481E+01_JPRB, 0.15148E+01_JPRB, 0.12049E+01_JPRB, 0.12000E+01_JPRB, 0.11009E+01_JPRB ,&
&  0.98550E+00_JPRB, 0.89747E+00_JPRB, 0.84183E+00_JPRB, 0.79476E+00_JPRB, 0.74387E+00_JPRB, 0.69843E+00_JPRB, 0.67395E+00_JPRB ,&
&  0.67088E+00_JPRB, 0.67666E+00_JPRB, 0.67367E+00_JPRB, 0.66535E+00_JPRB, 0.67687E+00_JPRB, 0.68804E+00_JPRB, 0.71311E+00_JPRB ,&
&  0.73959E+00_JPRB, 0.76875E+00_JPRB, 0.76666E+00_JPRB, 0.80150E+00_JPRB, 0.84954E+00_JPRB, 0.88772E+00_JPRB, 0.88199E+00_JPRB ,&
&  0.85643E+00_JPRB, 0.90055E+00_JPRB, 0.89987E+00_JPRB, 0.87320E+00_JPRB, 0.85300E+00_JPRB, 0.84038E+00_JPRB, 0.80555E+00_JPRB ,&
&  0.81918E+00_JPRB, 0.80051E+00_JPRB, 0.74666E+00_JPRB, 0.65640E+00_JPRB, 0.61845E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.56166E+00_JPRB, 0.54420E+00_JPRB, 0.54448E+00_JPRB, 0.53718E+00_JPRB, 0.52495E+00_JPRB, 0.51840E+00_JPRB, 0.51094E+00_JPRB ,&
&  0.50320E+00_JPRB, 0.50268E+00_JPRB, 0.50356E+00_JPRB, 0.53343E+00_JPRB, 0.54776E+00_JPRB, 0.53909E+00_JPRB, 0.52915E+00_JPRB ,&
&  0.52243E+00_JPRB, 0.52360E+00_JPRB, 0.52770E+00_JPRB, 0.53027E+00_JPRB, 0.53146E+00_JPRB, 0.53927E+00_JPRB, 0.54789E+00_JPRB ,&
&  0.54695E+00_JPRB, 0.54977E+00_JPRB, 0.56229E+00_JPRB, 0.58962E+00_JPRB, 0.64216E+00_JPRB, 0.73142E+00_JPRB, 0.88588E+00_JPRB ,&
&  0.12259E+01_JPRB, 0.11923E+01_JPRB, 0.59769E+00_JPRB, 0.11345E+01_JPRB, 0.18229E+01_JPRB, 0.19061E+01_JPRB, 0.18059E+01_JPRB ,&
&  0.16212E+01_JPRB, 0.12698E+01_JPRB, 0.11681E+01_JPRB, 0.11937E+01_JPRB, 0.10515E+01_JPRB, 0.94401E+00_JPRB, 0.88146E+00_JPRB ,&
&  0.85097E+00_JPRB, 0.86157E+00_JPRB, 0.91604E+00_JPRB, 0.97390E+00_JPRB, 0.10303E+01_JPRB, 0.10596E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.10897E+01_JPRB, 0.11628E+01_JPRB, 0.13332E+01_JPRB, 0.15767E+01_JPRB, 0.25446E+01_JPRB, 0.29587E+01_JPRB, 0.29973E+01_JPRB ,&
&  0.21481E+01_JPRB, 0.18366E+01_JPRB, 0.15713E+01_JPRB, 0.13578E+01_JPRB, 0.11331E+01_JPRB, 0.10216E+01_JPRB, 0.10170E+01_JPRB ,&
&  0.89943E+00_JPRB, 0.80763E+00_JPRB, 0.77344E+00_JPRB, 0.74054E+00_JPRB, 0.71004E+00_JPRB, 0.69452E+00_JPRB, 0.68915E+00_JPRB ,&
&  0.70118E+00_JPRB, 0.71064E+00_JPRB, 0.71023E+00_JPRB, 0.70974E+00_JPRB, 0.71827E+00_JPRB, 0.73180E+00_JPRB, 0.75632E+00_JPRB ,&
&  0.77279E+00_JPRB, 0.80513E+00_JPRB, 0.84798E+00_JPRB, 0.85647E+00_JPRB, 0.88338E+00_JPRB, 0.94023E+00_JPRB, 0.94445E+00_JPRB ,&
&  0.94950E+00_JPRB, 0.10353E+01_JPRB, 0.92557E+00_JPRB, 0.84667E+00_JPRB, 0.84525E+00_JPRB, 0.81797E+00_JPRB, 0.80233E+00_JPRB ,&
&  0.85187E+00_JPRB, 0.97316E+00_JPRB, 0.93263E+00_JPRB, 0.62942E+00_JPRB, 0.57852E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.51278E+00_JPRB, 0.50277E+00_JPRB, 0.49850E+00_JPRB, 0.50598E+00_JPRB, 0.51135E+00_JPRB, 0.51769E+00_JPRB, 0.50498E+00_JPRB ,&
&  0.51888E+00_JPRB, 0.55454E+00_JPRB, 0.57917E+00_JPRB, 0.59836E+00_JPRB, 0.59308E+00_JPRB, 0.58200E+00_JPRB, 0.57731E+00_JPRB ,&
&  0.57757E+00_JPRB, 0.58250E+00_JPRB, 0.59731E+00_JPRB, 0.60442E+00_JPRB, 0.61013E+00_JPRB, 0.62160E+00_JPRB, 0.63011E+00_JPRB ,&
&  0.63882E+00_JPRB, 0.65529E+00_JPRB, 0.68763E+00_JPRB, 0.75093E+00_JPRB, 0.86240E+00_JPRB, 0.10527E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.17531E+01_JPRB, 0.12637E+01_JPRB, 0.63614E+00_JPRB, 0.14321E+01_JPRB, 0.18115E+01_JPRB, 0.18111E+01_JPRB, 0.15561E+01_JPRB ,&
&  0.14307E+01_JPRB, 0.12350E+01_JPRB, 0.11240E+01_JPRB, 0.11968E+01_JPRB, 0.10711E+01_JPRB, 0.10192E+01_JPRB, 0.10045E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10799E+01_JPRB, 0.11367E+01_JPRB, 0.11841E+01_JPRB, 0.12057E+01_JPRB, 0.12091E+01_JPRB, 0.12369E+01_JPRB ,&
&  0.13438E+01_JPRB, 0.14949E+01_JPRB, 0.17652E+01_JPRB, 0.21920E+01_JPRB, 0.30923E+01_JPRB, 0.28731E+01_JPRB, 0.27659E+01_JPRB ,&
&  0.17406E+01_JPRB, 0.14474E+01_JPRB, 0.13372E+01_JPRB, 0.12305E+01_JPRB, 0.10454E+01_JPRB, 0.89768E+00_JPRB, 0.90183E+00_JPRB ,&
&  0.80348E+00_JPRB, 0.74274E+00_JPRB, 0.72504E+00_JPRB, 0.70257E+00_JPRB, 0.68564E+00_JPRB, 0.69011E+00_JPRB, 0.71124E+00_JPRB ,&
&  0.74124E+00_JPRB, 0.74927E+00_JPRB, 0.75078E+00_JPRB, 0.75129E+00_JPRB, 0.75838E+00_JPRB, 0.76856E+00_JPRB, 0.77512E+00_JPRB ,&
&  0.79578E+00_JPRB, 0.83069E+00_JPRB, 0.86873E+00_JPRB, 0.88428E+00_JPRB, 0.88520E+00_JPRB, 0.90752E+00_JPRB, 0.97386E+00_JPRB ,&
&  0.97212E+00_JPRB, 0.98687E+00_JPRB, 0.88909E+00_JPRB, 0.86272E+00_JPRB, 0.80661E+00_JPRB, 0.78226E+00_JPRB, 0.76497E+00_JPRB ,&
&  0.74019E+00_JPRB, 0.67362E+00_JPRB, 0.59753E+00_JPRB, 0.52396E+00_JPRB, 0.51361E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.49496E+00_JPRB, 0.48769E+00_JPRB, 0.48559E+00_JPRB, 0.50303E+00_JPRB, 0.49969E+00_JPRB, 0.51253E+00_JPRB, 0.53075E+00_JPRB ,&
&  0.57799E+00_JPRB, 0.62109E+00_JPRB, 0.65807E+00_JPRB, 0.65290E+00_JPRB, 0.65185E+00_JPRB, 0.64007E+00_JPRB, 0.64662E+00_JPRB ,&
&  0.66091E+00_JPRB, 0.68323E+00_JPRB, 0.70755E+00_JPRB, 0.71932E+00_JPRB, 0.72898E+00_JPRB, 0.74079E+00_JPRB, 0.75721E+00_JPRB ,&
&  0.78265E+00_JPRB, 0.82015E+00_JPRB, 0.89002E+00_JPRB, 0.10176E+01_JPRB, 0.12196E+01_JPRB, 0.14246E+01_JPRB, 0.15728E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.49764E+00_JPRB, 0.13526E+01_JPRB, 0.17705E+01_JPRB, 0.17272E+01_JPRB, 0.16420E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.13431E+01_JPRB, 0.12391E+01_JPRB, 0.11362E+01_JPRB, 0.12560E+01_JPRB, 0.11602E+01_JPRB, 0.11519E+01_JPRB, 0.11792E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.12781E+01_JPRB, 0.13284E+01_JPRB, 0.13514E+01_JPRB, 0.13589E+01_JPRB, 0.13926E+01_JPRB, 0.14722E+01_JPRB ,&
&  0.16191E+01_JPRB, 0.18481E+01_JPRB, 0.21361E+01_JPRB, 0.22396E+01_JPRB, 0.25875E+01_JPRB, 0.24913E+01_JPRB, 0.26820E+01_JPRB ,&
&  0.17828E+01_JPRB, 0.11864E+01_JPRB, 0.11095E+01_JPRB, 0.11248E+01_JPRB, 0.97891E+00_JPRB, 0.81314E+00_JPRB, 0.80844E+00_JPRB ,&
&  0.73737E+00_JPRB, 0.70339E+00_JPRB, 0.69861E+00_JPRB, 0.68298E+00_JPRB, 0.66719E+00_JPRB, 0.68655E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.74927E+00_JPRB, 0.76444E+00_JPRB, 0.76027E+00_JPRB, 0.76044E+00_JPRB, 0.76502E+00_JPRB, 0.76377E+00_JPRB, 0.76304E+00_JPRB ,&
&  0.78819E+00_JPRB, 0.81756E+00_JPRB, 0.85253E+00_JPRB, 0.89024E+00_JPRB, 0.93528E+00_JPRB, 0.93899E+00_JPRB, 0.93461E+00_JPRB ,&
&  0.91733E+00_JPRB, 0.89916E+00_JPRB, 0.83355E+00_JPRB, 0.81703E+00_JPRB, 0.78599E+00_JPRB, 0.74811E+00_JPRB, 0.71551E+00_JPRB ,&
&  0.66142E+00_JPRB, 0.57774E+00_JPRB, 0.52612E+00_JPRB, 0.51437E+00_JPRB, 0.50121E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.51898E+00_JPRB, 0.51148E+00_JPRB, 0.51664E+00_JPRB, 0.52579E+00_JPRB, 0.54150E+00_JPRB, 0.56861E+00_JPRB, 0.61083E+00_JPRB ,&
&  0.65944E+00_JPRB, 0.68643E+00_JPRB, 0.70935E+00_JPRB, 0.70026E+00_JPRB, 0.70329E+00_JPRB, 0.72082E+00_JPRB, 0.73276E+00_JPRB ,&
&  0.76387E+00_JPRB, 0.79641E+00_JPRB, 0.81554E+00_JPRB, 0.82135E+00_JPRB, 0.83302E+00_JPRB, 0.86421E+00_JPRB, 0.90873E+00_JPRB ,&
&  0.96498E+00_JPRB, 0.10432E+01_JPRB, 0.11507E+01_JPRB, 0.12975E+01_JPRB, 0.14616E+01_JPRB, 0.15242E+01_JPRB, 0.15242E+01_JPRB ,&
&  0.78351E+00_JPRB, 0.13449E+01_JPRB, 0.17814E+01_JPRB, 0.17506E+01_JPRB, 0.16597E+01_JPRB, 0.15902E+01_JPRB, 0.15402E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.12336E+01_JPRB, 0.12130E+01_JPRB, 0.13422E+01_JPRB, 0.12661E+01_JPRB, 0.12928E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.14364E+01_JPRB, 0.14641E+01_JPRB, 0.14842E+01_JPRB, 0.15338E+01_JPRB, 0.16191E+01_JPRB, 0.17285E+01_JPRB ,&
&  0.18721E+01_JPRB, 0.20547E+01_JPRB, 0.21853E+01_JPRB, 0.20700E+01_JPRB, 0.20433E+01_JPRB, 0.22440E+01_JPRB, 0.28239E+01_JPRB ,&
&  0.17719E+01_JPRB, 0.10395E+01_JPRB, 0.10186E+01_JPRB, 0.10308E+01_JPRB, 0.89432E+00_JPRB, 0.80234E+00_JPRB, 0.74427E+00_JPRB ,&
&  0.69408E+00_JPRB, 0.65820E+00_JPRB, 0.64203E+00_JPRB, 0.63280E+00_JPRB, 0.64642E+00_JPRB, 0.66665E+00_JPRB, 0.69960E+00_JPRB ,&
&  0.71575E+00_JPRB, 0.71939E+00_JPRB, 0.71244E+00_JPRB, 0.70999E+00_JPRB, 0.71671E+00_JPRB, 0.72664E+00_JPRB, 0.72726E+00_JPRB ,&
&  0.74935E+00_JPRB, 0.75607E+00_JPRB, 0.77952E+00_JPRB, 0.82594E+00_JPRB, 0.88569E+00_JPRB, 0.90814E+00_JPRB, 0.90374E+00_JPRB ,&
&  0.84738E+00_JPRB, 0.83917E+00_JPRB, 0.82849E+00_JPRB, 0.80749E+00_JPRB, 0.77922E+00_JPRB, 0.72640E+00_JPRB, 0.63184E+00_JPRB ,&
&  0.58803E+00_JPRB, 0.54528E+00_JPRB, 0.52630E+00_JPRB, 0.51884E+00_JPRB, 0.53006E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.59077E+00_JPRB, 0.57894E+00_JPRB, 0.58715E+00_JPRB, 0.60555E+00_JPRB, 0.62134E+00_JPRB, 0.65237E+00_JPRB, 0.69096E+00_JPRB ,&
&  0.73995E+00_JPRB, 0.77259E+00_JPRB, 0.78506E+00_JPRB, 0.78758E+00_JPRB, 0.79763E+00_JPRB, 0.80276E+00_JPRB, 0.81878E+00_JPRB ,&
&  0.84752E+00_JPRB, 0.87735E+00_JPRB, 0.91536E+00_JPRB, 0.94695E+00_JPRB, 0.97304E+00_JPRB, 0.10308E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.11771E+01_JPRB, 0.12650E+01_JPRB, 0.13717E+01_JPRB, 0.14869E+01_JPRB, 0.16090E+01_JPRB, 0.16625E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.12999E+01_JPRB, 0.18356E+01_JPRB, 0.18651E+01_JPRB, 0.16934E+01_JPRB, 0.15434E+01_JPRB, 0.15012E+01_JPRB, 0.14578E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.13161E+01_JPRB, 0.12949E+01_JPRB, 0.13114E+01_JPRB, 0.13723E+01_JPRB, 0.13927E+01_JPRB, 0.14547E+01_JPRB ,&
&  0.15182E+01_JPRB, 0.15675E+01_JPRB, 0.16035E+01_JPRB, 0.16385E+01_JPRB, 0.16940E+01_JPRB, 0.17793E+01_JPRB, 0.18957E+01_JPRB ,&
&  0.19886E+01_JPRB, 0.20732E+01_JPRB, 0.21344E+01_JPRB, 0.19830E+01_JPRB, 0.18373E+01_JPRB, 0.19923E+01_JPRB, 0.19291E+01_JPRB ,&
&  0.13322E+01_JPRB, 0.91400E+00_JPRB, 0.92314E+00_JPRB, 0.94095E+00_JPRB, 0.83976E+00_JPRB, 0.75730E+00_JPRB, 0.69499E+00_JPRB ,&
&  0.64154E+00_JPRB, 0.59912E+00_JPRB, 0.60451E+00_JPRB, 0.61138E+00_JPRB, 0.61699E+00_JPRB, 0.62050E+00_JPRB, 0.63798E+00_JPRB ,&
&  0.65881E+00_JPRB, 0.65217E+00_JPRB, 0.64360E+00_JPRB, 0.63891E+00_JPRB, 0.65468E+00_JPRB, 0.67990E+00_JPRB, 0.70687E+00_JPRB ,&
&  0.76237E+00_JPRB, 0.71988E+00_JPRB, 0.73398E+00_JPRB, 0.77746E+00_JPRB, 0.82353E+00_JPRB, 0.87906E+00_JPRB, 0.92809E+00_JPRB ,&
&  0.96133E+00_JPRB, 0.99628E+00_JPRB, 0.10335E+01_JPRB, 0.10326E+01_JPRB, 0.95698E+00_JPRB, 0.87332E+00_JPRB, 0.63538E+00_JPRB ,&
&  0.62311E+00_JPRB, 0.61055E+00_JPRB, 0.61348E+00_JPRB, 0.61249E+00_JPRB, 0.61099E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.71786E+00_JPRB, 0.71672E+00_JPRB, 0.74275E+00_JPRB, 0.77008E+00_JPRB, 0.79443E+00_JPRB, 0.81808E+00_JPRB, 0.83571E+00_JPRB ,&
&  0.86082E+00_JPRB, 0.88152E+00_JPRB, 0.88687E+00_JPRB, 0.88436E+00_JPRB, 0.89864E+00_JPRB, 0.90260E+00_JPRB, 0.92777E+00_JPRB ,&
&  0.96525E+00_JPRB, 0.10087E+01_JPRB, 0.10651E+01_JPRB, 0.11015E+01_JPRB, 0.11361E+01_JPRB, 0.11909E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.13024E+01_JPRB, 0.13789E+01_JPRB, 0.14467E+01_JPRB, 0.15212E+01_JPRB, 0.15946E+01_JPRB, 0.15915E+01_JPRB, 0.12296E+01_JPRB ,&
&  0.17526E+01_JPRB, 0.19997E+01_JPRB, 0.18971E+01_JPRB, 0.16435E+01_JPRB, 0.14990E+01_JPRB, 0.14454E+01_JPRB, 0.14425E+01_JPRB ,&
&  0.14354E+01_JPRB, 0.13821E+01_JPRB, 0.13236E+01_JPRB, 0.13125E+01_JPRB, 0.14811E+01_JPRB, 0.14645E+01_JPRB, 0.15367E+01_JPRB ,&
&  0.16132E+01_JPRB, 0.16802E+01_JPRB, 0.17321E+01_JPRB, 0.17738E+01_JPRB, 0.18045E+01_JPRB, 0.18479E+01_JPRB, 0.18938E+01_JPRB ,&
&  0.19338E+01_JPRB, 0.19596E+01_JPRB, 0.20094E+01_JPRB, 0.18106E+01_JPRB, 0.17164E+01_JPRB, 0.17968E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.87114E+00_JPRB, 0.87359E+00_JPRB, 0.87470E+00_JPRB, 0.81339E+00_JPRB, 0.73052E+00_JPRB, 0.68231E+00_JPRB ,&
&  0.64307E+00_JPRB, 0.61765E+00_JPRB, 0.61139E+00_JPRB, 0.59811E+00_JPRB, 0.59699E+00_JPRB, 0.60971E+00_JPRB, 0.61421E+00_JPRB ,&
&  0.63960E+00_JPRB, 0.62929E+00_JPRB, 0.62520E+00_JPRB, 0.62855E+00_JPRB, 0.66264E+00_JPRB, 0.73090E+00_JPRB, 0.86775E+00_JPRB ,&
&  0.99273E+00_JPRB, 0.65007E+00_JPRB, 0.71016E+00_JPRB, 0.75200E+00_JPRB, 0.82431E+00_JPRB, 0.95056E+00_JPRB, 0.11482E+01_JPRB ,&
&  0.14864E+01_JPRB, 0.19559E+01_JPRB, 0.18928E+01_JPRB, 0.15018E+01_JPRB, 0.19024E+01_JPRB, 0.15981E+01_JPRB, 0.71842E+00_JPRB ,&
&  0.72365E+00_JPRB, 0.70457E+00_JPRB, 0.71163E+00_JPRB, 0.71921E+00_JPRB, 0.72225E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.82050E+00_JPRB, 0.84144E+00_JPRB, 0.86534E+00_JPRB, 0.88689E+00_JPRB, 0.90100E+00_JPRB, 0.91399E+00_JPRB, 0.92997E+00_JPRB ,&
&  0.93877E+00_JPRB, 0.94222E+00_JPRB, 0.95596E+00_JPRB, 0.97756E+00_JPRB, 0.10040E+01_JPRB, 0.10326E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.11107E+01_JPRB, 0.11515E+01_JPRB, 0.12028E+01_JPRB, 0.12393E+01_JPRB, 0.12615E+01_JPRB, 0.12856E+01_JPRB, 0.13085E+01_JPRB ,&
&  0.13478E+01_JPRB, 0.13744E+01_JPRB, 0.13998E+01_JPRB, 0.14537E+01_JPRB, 0.14971E+01_JPRB, 0.15491E+01_JPRB, 0.12987E+01_JPRB ,&
&  0.16654E+01_JPRB, 0.19455E+01_JPRB, 0.17852E+01_JPRB, 0.15463E+01_JPRB, 0.14320E+01_JPRB, 0.14209E+01_JPRB, 0.14216E+01_JPRB ,&
&  0.14339E+01_JPRB, 0.14276E+01_JPRB, 0.13449E+01_JPRB, 0.14021E+01_JPRB, 0.14451E+01_JPRB, 0.14442E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.16460E+01_JPRB, 0.17112E+01_JPRB, 0.17707E+01_JPRB, 0.18145E+01_JPRB, 0.18683E+01_JPRB, 0.18576E+01_JPRB ,&
&  0.18425E+01_JPRB, 0.18296E+01_JPRB, 0.18038E+01_JPRB, 0.16275E+01_JPRB, 0.15405E+01_JPRB, 0.14369E+01_JPRB, 0.11721E+01_JPRB ,&
&  0.97607E+00_JPRB, 0.85781E+00_JPRB, 0.82517E+00_JPRB, 0.89859E+00_JPRB, 0.86818E+00_JPRB, 0.79261E+00_JPRB, 0.73148E+00_JPRB ,&
&  0.68072E+00_JPRB, 0.64312E+00_JPRB, 0.65616E+00_JPRB, 0.63917E+00_JPRB, 0.58288E+00_JPRB, 0.59528E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.60460E+00_JPRB, 0.61996E+00_JPRB, 0.64109E+00_JPRB, 0.63183E+00_JPRB, 0.63223E+00_JPRB, 0.65808E+00_JPRB, 0.66191E+00_JPRB ,&
&  0.72285E+00_JPRB, 0.67894E+00_JPRB, 0.69494E+00_JPRB, 0.74448E+00_JPRB, 0.73887E+00_JPRB, 0.95513E+00_JPRB, 0.13046E+01_JPRB ,&
&  0.18947E+01_JPRB, 0.20537E+01_JPRB, 0.26721E+01_JPRB, 0.41615E+01_JPRB, 0.31639E+01_JPRB, 0.21034E+01_JPRB, 0.81326E+00_JPRB ,&
&  0.81449E+00_JPRB, 0.80756E+00_JPRB, 0.82215E+00_JPRB, 0.81887E+00_JPRB, 0.82153E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.83338E+00_JPRB, 0.86050E+00_JPRB, 0.88271E+00_JPRB, 0.90497E+00_JPRB, 0.92574E+00_JPRB, 0.94565E+00_JPRB, 0.96726E+00_JPRB ,&
&  0.98246E+00_JPRB, 0.10041E+01_JPRB, 0.10301E+01_JPRB, 0.10565E+01_JPRB, 0.10793E+01_JPRB, 0.11093E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11695E+01_JPRB, 0.11935E+01_JPRB, 0.12167E+01_JPRB, 0.12196E+01_JPRB, 0.12199E+01_JPRB, 0.12336E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.12467E+01_JPRB, 0.12608E+01_JPRB, 0.12861E+01_JPRB, 0.12973E+01_JPRB, 0.13520E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.15296E+01_JPRB, 0.17193E+01_JPRB, 0.16148E+01_JPRB, 0.14317E+01_JPRB, 0.13379E+01_JPRB, 0.13351E+01_JPRB, 0.13477E+01_JPRB ,&
&  0.13716E+01_JPRB, 0.14198E+01_JPRB, 0.13339E+01_JPRB, 0.12878E+01_JPRB, 0.12751E+01_JPRB, 0.12868E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13739E+01_JPRB, 0.14221E+01_JPRB, 0.14747E+01_JPRB, 0.15227E+01_JPRB, 0.15615E+01_JPRB, 0.15775E+01_JPRB, 0.15529E+01_JPRB ,&
&  0.14978E+01_JPRB, 0.14340E+01_JPRB, 0.12918E+01_JPRB, 0.12522E+01_JPRB, 0.11761E+01_JPRB, 0.97976E+00_JPRB, 0.79107E+00_JPRB ,&
&  0.73746E+00_JPRB, 0.80111E+00_JPRB, 0.86567E+00_JPRB, 0.10246E+01_JPRB, 0.99349E+00_JPRB, 0.92400E+00_JPRB, 0.83909E+00_JPRB ,&
&  0.76145E+00_JPRB, 0.70172E+00_JPRB, 0.69502E+00_JPRB, 0.66187E+00_JPRB, 0.63522E+00_JPRB, 0.62031E+00_JPRB, 0.60596E+00_JPRB ,&
&  0.61469E+00_JPRB, 0.62510E+00_JPRB, 0.63760E+00_JPRB, 0.63759E+00_JPRB, 0.62228E+00_JPRB, 0.55813E+00_JPRB, 0.70841E+00_JPRB ,&
&  0.67235E+00_JPRB, 0.60535E+00_JPRB, 0.70652E+00_JPRB, 0.54303E+00_JPRB, 0.65286E+00_JPRB, 0.71778E+00_JPRB, 0.84757E+00_JPRB ,&
&  0.93551E+00_JPRB, 0.13044E+01_JPRB, 0.17941E+01_JPRB, 0.14330E+01_JPRB, 0.11520E+01_JPRB, 0.87363E+00_JPRB, 0.78331E+00_JPRB ,&
&  0.78127E+00_JPRB, 0.78436E+00_JPRB, 0.79352E+00_JPRB, 0.79689E+00_JPRB, 0.80893E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.84220E+00_JPRB, 0.86649E+00_JPRB, 0.88499E+00_JPRB, 0.90699E+00_JPRB, 0.92992E+00_JPRB, 0.95127E+00_JPRB, 0.97131E+00_JPRB ,&
&  0.98815E+00_JPRB, 0.10024E+01_JPRB, 0.10151E+01_JPRB, 0.10314E+01_JPRB, 0.10511E+01_JPRB, 0.10668E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.11192E+01_JPRB, 0.11444E+01_JPRB, 0.11722E+01_JPRB, 0.11701E+01_JPRB, 0.11703E+01_JPRB, 0.11930E+01_JPRB, 0.12154E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.12420E+01_JPRB, 0.12694E+01_JPRB, 0.13009E+01_JPRB, 0.13403E+01_JPRB, 0.14105E+01_JPRB, 0.13252E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.15799E+01_JPRB, 0.14874E+01_JPRB, 0.13031E+01_JPRB, 0.12224E+01_JPRB, 0.11860E+01_JPRB, 0.11802E+01_JPRB ,&
&  0.11824E+01_JPRB, 0.11449E+01_JPRB, 0.11179E+01_JPRB, 0.10846E+01_JPRB, 0.10722E+01_JPRB, 0.10798E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.11145E+01_JPRB, 0.11484E+01_JPRB, 0.11949E+01_JPRB, 0.12448E+01_JPRB, 0.12209E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.10939E+01_JPRB, 0.10145E+01_JPRB, 0.81346E+00_JPRB, 0.74977E+00_JPRB, 0.69979E+00_JPRB, 0.61646E+00_JPRB, 0.53970E+00_JPRB ,&
&  0.62036E+00_JPRB, 0.71788E+00_JPRB, 0.89593E+00_JPRB, 0.11162E+01_JPRB, 0.10731E+01_JPRB, 0.10181E+01_JPRB, 0.92036E+00_JPRB ,&
&  0.82534E+00_JPRB, 0.76108E+00_JPRB, 0.73735E+00_JPRB, 0.69835E+00_JPRB, 0.68438E+00_JPRB, 0.68641E+00_JPRB, 0.64135E+00_JPRB ,&
&  0.62889E+00_JPRB, 0.62013E+00_JPRB, 0.61817E+00_JPRB, 0.61350E+00_JPRB, 0.63455E+00_JPRB, 0.59738E+00_JPRB, 0.78754E+00_JPRB ,&
&  0.67629E+00_JPRB, 0.51469E+00_JPRB, 0.54050E+00_JPRB, 0.63115E+00_JPRB, 0.61680E+00_JPRB, 0.62300E+00_JPRB, 0.70183E+00_JPRB ,&
&  0.85484E+00_JPRB, 0.96883E+00_JPRB, 0.99052E+00_JPRB, 0.93513E+00_JPRB, 0.84289E+00_JPRB, 0.78943E+00_JPRB, 0.76260E+00_JPRB ,&
&  0.77351E+00_JPRB, 0.78379E+00_JPRB, 0.79706E+00_JPRB, 0.80248E+00_JPRB, 0.81611E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.82939E+00_JPRB, 0.85529E+00_JPRB, 0.87068E+00_JPRB, 0.88658E+00_JPRB, 0.90693E+00_JPRB, 0.91858E+00_JPRB, 0.92838E+00_JPRB ,&
&  0.93231E+00_JPRB, 0.92998E+00_JPRB, 0.93788E+00_JPRB, 0.95153E+00_JPRB, 0.95816E+00_JPRB, 0.95589E+00_JPRB, 0.96263E+00_JPRB ,&
&  0.97779E+00_JPRB, 0.10017E+01_JPRB, 0.10276E+01_JPRB, 0.10221E+01_JPRB, 0.10186E+01_JPRB, 0.10338E+01_JPRB, 0.10560E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10590E+01_JPRB, 0.10702E+01_JPRB, 0.10828E+01_JPRB, 0.11042E+01_JPRB, 0.11089E+01_JPRB, 0.12027E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.15160E+01_JPRB, 0.13997E+01_JPRB, 0.11888E+01_JPRB, 0.11586E+01_JPRB, 0.10598E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.97151E+00_JPRB, 0.93598E+00_JPRB, 0.91010E+00_JPRB, 0.90823E+00_JPRB, 0.91385E+00_JPRB, 0.88382E+00_JPRB ,&
&  0.88191E+00_JPRB, 0.89199E+00_JPRB, 0.92605E+00_JPRB, 0.96899E+00_JPRB, 0.99686E+00_JPRB, 0.98275E+00_JPRB, 0.91506E+00_JPRB ,&
&  0.86564E+00_JPRB, 0.78706E+00_JPRB, 0.61238E+00_JPRB, 0.59159E+00_JPRB, 0.58083E+00_JPRB, 0.51086E+00_JPRB, 0.47968E+00_JPRB ,&
&  0.60749E+00_JPRB, 0.75713E+00_JPRB, 0.10513E+01_JPRB, 0.11814E+01_JPRB, 0.11718E+01_JPRB, 0.11974E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.91551E+00_JPRB, 0.83424E+00_JPRB, 0.82084E+00_JPRB, 0.77651E+00_JPRB, 0.76019E+00_JPRB, 0.73676E+00_JPRB, 0.68848E+00_JPRB ,&
&  0.65316E+00_JPRB, 0.62691E+00_JPRB, 0.61271E+00_JPRB, 0.61479E+00_JPRB, 0.55791E+00_JPRB, 0.78574E+00_JPRB, 0.11127E+01_JPRB ,&
&  0.72072E+00_JPRB, 0.61769E+00_JPRB, 0.49000E+00_JPRB, 0.64929E+00_JPRB, 0.59071E+00_JPRB, 0.57079E+00_JPRB, 0.62702E+00_JPRB ,&
&  0.73930E+00_JPRB, 0.78218E+00_JPRB, 0.78667E+00_JPRB, 0.77693E+00_JPRB, 0.75436E+00_JPRB, 0.73392E+00_JPRB, 0.72576E+00_JPRB ,&
&  0.74196E+00_JPRB, 0.75796E+00_JPRB, 0.77646E+00_JPRB, 0.78743E+00_JPRB, 0.80365E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.80533E+00_JPRB, 0.82187E+00_JPRB, 0.82330E+00_JPRB, 0.83991E+00_JPRB, 0.86140E+00_JPRB, 0.86908E+00_JPRB, 0.87352E+00_JPRB ,&
&  0.86537E+00_JPRB, 0.86822E+00_JPRB, 0.86370E+00_JPRB, 0.87983E+00_JPRB, 0.88066E+00_JPRB, 0.88216E+00_JPRB, 0.88501E+00_JPRB ,&
&  0.89976E+00_JPRB, 0.91064E+00_JPRB, 0.94074E+00_JPRB, 0.94623E+00_JPRB, 0.94472E+00_JPRB, 0.95255E+00_JPRB, 0.97598E+00_JPRB ,&
&  0.10029E+01_JPRB, 0.10347E+01_JPRB, 0.10775E+01_JPRB, 0.11136E+01_JPRB, 0.12304E+01_JPRB, 0.13931E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.14375E+01_JPRB, 0.12861E+01_JPRB, 0.10911E+01_JPRB, 0.10600E+01_JPRB, 0.10466E+01_JPRB, 0.97924E+00_JPRB ,&
&  0.91955E+00_JPRB, 0.88528E+00_JPRB, 0.85663E+00_JPRB, 0.84619E+00_JPRB, 0.85110E+00_JPRB, 0.83868E+00_JPRB, 0.82900E+00_JPRB ,&
&  0.79398E+00_JPRB, 0.79158E+00_JPRB, 0.91961E+00_JPRB, 0.10653E+01_JPRB, 0.11567E+01_JPRB, 0.12560E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.10892E+01_JPRB, 0.10541E+01_JPRB, 0.92427E+00_JPRB, 0.87815E+00_JPRB, 0.82205E+00_JPRB, 0.77102E+00_JPRB, 0.74680E+00_JPRB ,&
&  0.83157E+00_JPRB, 0.98001E+00_JPRB, 0.10253E+01_JPRB, 0.12440E+01_JPRB, 0.13391E+01_JPRB, 0.13553E+01_JPRB, 0.11644E+01_JPRB ,&
&  0.99333E+00_JPRB, 0.90585E+00_JPRB, 0.88675E+00_JPRB, 0.85406E+00_JPRB, 0.83799E+00_JPRB, 0.78097E+00_JPRB, 0.71765E+00_JPRB ,&
&  0.69225E+00_JPRB, 0.64531E+00_JPRB, 0.61711E+00_JPRB, 0.57695E+00_JPRB, 0.62484E+00_JPRB, 0.64245E+00_JPRB, 0.78360E+00_JPRB ,&
&  0.64980E+00_JPRB, 0.66993E+00_JPRB, 0.65213E+00_JPRB, 0.10357E+01_JPRB, 0.69479E+00_JPRB, 0.54265E+00_JPRB, 0.55485E+00_JPRB ,&
&  0.61957E+00_JPRB, 0.65713E+00_JPRB, 0.67352E+00_JPRB, 0.70184E+00_JPRB, 0.71059E+00_JPRB, 0.70540E+00_JPRB, 0.71471E+00_JPRB ,&
&  0.75188E+00_JPRB, 0.75883E+00_JPRB, 0.76601E+00_JPRB, 0.76767E+00_JPRB, 0.78577E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.74704E+00_JPRB, 0.76035E+00_JPRB, 0.75463E+00_JPRB, 0.76198E+00_JPRB, 0.76681E+00_JPRB, 0.78243E+00_JPRB, 0.78277E+00_JPRB ,&
&  0.77987E+00_JPRB, 0.78591E+00_JPRB, 0.76617E+00_JPRB, 0.77037E+00_JPRB, 0.76593E+00_JPRB, 0.76893E+00_JPRB, 0.75775E+00_JPRB ,&
&  0.76079E+00_JPRB, 0.77580E+00_JPRB, 0.78203E+00_JPRB, 0.78968E+00_JPRB, 0.78691E+00_JPRB, 0.77078E+00_JPRB, 0.75688E+00_JPRB ,&
&  0.80733E+00_JPRB, 0.92738E+00_JPRB, 0.10676E+01_JPRB, 0.11639E+01_JPRB, 0.13897E+01_JPRB, 0.13931E+01_JPRB, 0.15150E+01_JPRB ,&
&  0.16453E+01_JPRB, 0.14455E+01_JPRB, 0.14096E+01_JPRB, 0.11985E+01_JPRB, 0.11457E+01_JPRB, 0.10989E+01_JPRB, 0.10749E+01_JPRB ,&
&  0.10951E+01_JPRB, 0.11445E+01_JPRB, 0.12054E+01_JPRB, 0.12744E+01_JPRB, 0.13321E+01_JPRB, 0.13402E+01_JPRB, 0.13296E+01_JPRB ,&
&  0.13056E+01_JPRB, 0.13659E+01_JPRB, 0.15796E+01_JPRB, 0.16852E+01_JPRB, 0.18150E+01_JPRB, 0.18748E+01_JPRB, 0.18586E+01_JPRB ,&
&  0.17981E+01_JPRB, 0.17443E+01_JPRB, 0.15236E+01_JPRB, 0.13742E+01_JPRB, 0.13503E+01_JPRB, 0.12542E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.13196E+01_JPRB, 0.13523E+01_JPRB, 0.11436E+01_JPRB, 0.12788E+01_JPRB, 0.13975E+01_JPRB, 0.13182E+01_JPRB, 0.13535E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.10569E+01_JPRB, 0.10094E+01_JPRB, 0.98157E+00_JPRB, 0.93486E+00_JPRB, 0.86392E+00_JPRB, 0.84018E+00_JPRB ,&
&  0.73186E+00_JPRB, 0.65190E+00_JPRB, 0.60398E+00_JPRB, 0.57970E+00_JPRB, 0.59408E+00_JPRB, 0.68807E+00_JPRB, 0.63386E+00_JPRB ,&
&  0.64243E+00_JPRB, 0.66284E+00_JPRB, 0.95374E+00_JPRB, 0.14572E+01_JPRB, 0.75109E+00_JPRB, 0.59093E+00_JPRB, 0.51184E+00_JPRB ,&
&  0.51802E+00_JPRB, 0.54697E+00_JPRB, 0.57225E+00_JPRB, 0.60559E+00_JPRB, 0.62300E+00_JPRB, 0.62614E+00_JPRB, 0.63743E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.69111E+00_JPRB, 0.72791E+00_JPRB, 0.73259E+00_JPRB, 0.73065E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.65046E+00_JPRB, 0.65114E+00_JPRB, 0.65484E+00_JPRB, 0.65040E+00_JPRB, 0.64588E+00_JPRB, 0.64565E+00_JPRB, 0.64863E+00_JPRB ,&
&  0.65011E+00_JPRB, 0.64135E+00_JPRB, 0.63194E+00_JPRB, 0.62512E+00_JPRB, 0.61371E+00_JPRB, 0.58525E+00_JPRB, 0.54600E+00_JPRB ,&
&  0.55504E+00_JPRB, 0.57032E+00_JPRB, 0.58927E+00_JPRB, 0.63385E+00_JPRB, 0.73266E+00_JPRB, 0.88118E+00_JPRB, 0.10853E+01_JPRB ,&
&  0.14734E+01_JPRB, 0.19026E+01_JPRB, 0.22067E+01_JPRB, 0.22747E+01_JPRB, 0.25100E+01_JPRB, 0.18859E+01_JPRB, 0.18184E+01_JPRB ,&
&  0.20454E+01_JPRB, 0.16646E+01_JPRB, 0.14483E+01_JPRB, 0.13201E+01_JPRB, 0.12655E+01_JPRB, 0.12837E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.15128E+01_JPRB, 0.16582E+01_JPRB, 0.17801E+01_JPRB, 0.18981E+01_JPRB, 0.19971E+01_JPRB, 0.20903E+01_JPRB, 0.21436E+01_JPRB ,&
&  0.21387E+01_JPRB, 0.21409E+01_JPRB, 0.21800E+01_JPRB, 0.21511E+01_JPRB, 0.22995E+01_JPRB, 0.23676E+01_JPRB, 0.23728E+01_JPRB ,&
&  0.22880E+01_JPRB, 0.21684E+01_JPRB, 0.21211E+01_JPRB, 0.19983E+01_JPRB, 0.18956E+01_JPRB, 0.18210E+01_JPRB, 0.17363E+01_JPRB ,&
&  0.16649E+01_JPRB, 0.15894E+01_JPRB, 0.13429E+01_JPRB, 0.15247E+01_JPRB, 0.15693E+01_JPRB, 0.13188E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.13074E+01_JPRB, 0.12508E+01_JPRB, 0.11825E+01_JPRB, 0.11203E+01_JPRB, 0.10682E+01_JPRB, 0.11484E+01_JPRB, 0.10275E+01_JPRB ,&
&  0.83620E+00_JPRB, 0.67984E+00_JPRB, 0.59170E+00_JPRB, 0.58697E+00_JPRB, 0.61533E+00_JPRB, 0.66438E+00_JPRB, 0.67152E+00_JPRB ,&
&  0.66467E+00_JPRB, 0.66830E+00_JPRB, 0.82258E+00_JPRB, 0.11280E+01_JPRB, 0.81770E+00_JPRB, 0.55525E+00_JPRB, 0.49897E+00_JPRB ,&
&  0.48173E+00_JPRB, 0.48204E+00_JPRB, 0.48988E+00_JPRB, 0.51429E+00_JPRB, 0.53691E+00_JPRB, 0.55680E+00_JPRB, 0.57278E+00_JPRB ,&
&  0.58125E+00_JPRB, 0.60091E+00_JPRB, 0.61925E+00_JPRB, 0.63556E+00_JPRB, 0.64908E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.57077E+00_JPRB, 0.56684E+00_JPRB, 0.57362E+00_JPRB, 0.57290E+00_JPRB, 0.58565E+00_JPRB, 0.58723E+00_JPRB, 0.59370E+00_JPRB ,&
&  0.60953E+00_JPRB, 0.63179E+00_JPRB, 0.62694E+00_JPRB, 0.60943E+00_JPRB, 0.60986E+00_JPRB, 0.62609E+00_JPRB, 0.67387E+00_JPRB ,&
&  0.77948E+00_JPRB, 0.89245E+00_JPRB, 0.98637E+00_JPRB, 0.11199E+01_JPRB, 0.13637E+01_JPRB, 0.17832E+01_JPRB, 0.21969E+01_JPRB ,&
&  0.25180E+01_JPRB, 0.27400E+01_JPRB, 0.27303E+01_JPRB, 0.27950E+01_JPRB, 0.21876E+01_JPRB, 0.16412E+01_JPRB, 0.18117E+01_JPRB ,&
&  0.19410E+01_JPRB, 0.18173E+01_JPRB, 0.14964E+01_JPRB, 0.14336E+01_JPRB, 0.14117E+01_JPRB, 0.14770E+01_JPRB, 0.16233E+01_JPRB ,&
&  0.17742E+01_JPRB, 0.19367E+01_JPRB, 0.20668E+01_JPRB, 0.21645E+01_JPRB, 0.22288E+01_JPRB, 0.23032E+01_JPRB, 0.23695E+01_JPRB ,&
&  0.24477E+01_JPRB, 0.24716E+01_JPRB, 0.24774E+01_JPRB, 0.25192E+01_JPRB, 0.25825E+01_JPRB, 0.26707E+01_JPRB, 0.27113E+01_JPRB ,&
&  0.27105E+01_JPRB, 0.25660E+01_JPRB, 0.25165E+01_JPRB, 0.24501E+01_JPRB, 0.22502E+01_JPRB, 0.20439E+01_JPRB, 0.19763E+01_JPRB ,&
&  0.19215E+01_JPRB, 0.18944E+01_JPRB, 0.15409E+01_JPRB, 0.18584E+01_JPRB, 0.20201E+01_JPRB, 0.17127E+01_JPRB, 0.15948E+01_JPRB ,&
&  0.14408E+01_JPRB, 0.13535E+01_JPRB, 0.12396E+01_JPRB, 0.12246E+01_JPRB, 0.12605E+01_JPRB, 0.14407E+01_JPRB, 0.11036E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.78604E+00_JPRB, 0.74309E+00_JPRB, 0.79050E+00_JPRB, 0.84646E+00_JPRB, 0.81836E+00_JPRB, 0.73118E+00_JPRB ,&
&  0.67306E+00_JPRB, 0.80838E+00_JPRB, 0.98917E+00_JPRB, 0.15323E+01_JPRB, 0.18783E+01_JPRB, 0.58604E+00_JPRB, 0.51372E+00_JPRB ,&
&  0.50537E+00_JPRB, 0.50430E+00_JPRB, 0.50246E+00_JPRB, 0.51670E+00_JPRB, 0.52368E+00_JPRB, 0.53356E+00_JPRB, 0.55240E+00_JPRB ,&
&  0.56408E+00_JPRB, 0.56793E+00_JPRB, 0.57062E+00_JPRB, 0.56096E+00_JPRB, 0.56810E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.55751E+00_JPRB, 0.56331E+00_JPRB, 0.58553E+00_JPRB, 0.63370E+00_JPRB, 0.70712E+00_JPRB, 0.74836E+00_JPRB, 0.75398E+00_JPRB ,&
&  0.73468E+00_JPRB, 0.74826E+00_JPRB, 0.79061E+00_JPRB, 0.81841E+00_JPRB, 0.89000E+00_JPRB, 0.10080E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.14426E+01_JPRB, 0.15704E+01_JPRB, 0.17631E+01_JPRB, 0.19649E+01_JPRB, 0.22271E+01_JPRB, 0.24276E+01_JPRB ,&
&  0.24743E+01_JPRB, 0.22490E+01_JPRB, 0.21486E+01_JPRB, 0.16541E+01_JPRB, 0.14819E+01_JPRB, 0.17119E+01_JPRB, 0.15612E+01_JPRB ,&
&  0.14887E+01_JPRB, 0.14957E+01_JPRB, 0.14828E+01_JPRB, 0.15168E+01_JPRB, 0.15911E+01_JPRB, 0.16851E+01_JPRB, 0.18114E+01_JPRB ,&
&  0.19611E+01_JPRB, 0.20974E+01_JPRB, 0.22172E+01_JPRB, 0.23205E+01_JPRB, 0.23741E+01_JPRB, 0.24482E+01_JPRB, 0.25674E+01_JPRB ,&
&  0.27090E+01_JPRB, 0.27672E+01_JPRB, 0.28130E+01_JPRB, 0.28891E+01_JPRB, 0.28231E+01_JPRB, 0.28578E+01_JPRB, 0.28606E+01_JPRB ,&
&  0.28717E+01_JPRB, 0.28159E+01_JPRB, 0.27777E+01_JPRB, 0.27814E+01_JPRB, 0.26647E+01_JPRB, 0.22884E+01_JPRB, 0.21837E+01_JPRB ,&
&  0.21888E+01_JPRB, 0.20881E+01_JPRB, 0.19121E+01_JPRB, 0.21824E+01_JPRB, 0.14950E+01_JPRB, 0.16411E+01_JPRB, 0.17887E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.16047E+01_JPRB, 0.14479E+01_JPRB, 0.13945E+01_JPRB, 0.13542E+01_JPRB, 0.11841E+01_JPRB, 0.11580E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.10416E+01_JPRB, 0.10093E+01_JPRB, 0.10749E+01_JPRB, 0.11635E+01_JPRB, 0.10981E+01_JPRB, 0.10197E+01_JPRB ,&
&  0.90605E+00_JPRB, 0.93180E+00_JPRB, 0.92078E+00_JPRB, 0.11415E+01_JPRB, 0.95775E+00_JPRB, 0.55925E+00_JPRB, 0.54317E+00_JPRB ,&
&  0.54176E+00_JPRB, 0.55169E+00_JPRB, 0.55334E+00_JPRB, 0.58305E+00_JPRB, 0.59605E+00_JPRB, 0.58950E+00_JPRB, 0.59549E+00_JPRB ,&
&  0.60736E+00_JPRB, 0.60461E+00_JPRB, 0.60024E+00_JPRB, 0.58845E+00_JPRB, 0.57771E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.66205E+00_JPRB, 0.64354E+00_JPRB, 0.65300E+00_JPRB, 0.74834E+00_JPRB, 0.85274E+00_JPRB, 0.91558E+00_JPRB, 0.89778E+00_JPRB ,&
&  0.86121E+00_JPRB, 0.83150E+00_JPRB, 0.89883E+00_JPRB, 0.98650E+00_JPRB, 0.11086E+01_JPRB, 0.12574E+01_JPRB, 0.14133E+01_JPRB ,&
&  0.15882E+01_JPRB, 0.17251E+01_JPRB, 0.18209E+01_JPRB, 0.19221E+01_JPRB, 0.20612E+01_JPRB, 0.22159E+01_JPRB, 0.23173E+01_JPRB ,&
&  0.19415E+01_JPRB, 0.14934E+01_JPRB, 0.19494E+01_JPRB, 0.21540E+01_JPRB, 0.22999E+01_JPRB, 0.23507E+01_JPRB, 0.21100E+01_JPRB ,&
&  0.16838E+01_JPRB, 0.15778E+01_JPRB, 0.16515E+01_JPRB, 0.16972E+01_JPRB, 0.16760E+01_JPRB, 0.17761E+01_JPRB, 0.19016E+01_JPRB ,&
&  0.20452E+01_JPRB, 0.21569E+01_JPRB, 0.22297E+01_JPRB, 0.23056E+01_JPRB, 0.23646E+01_JPRB, 0.24405E+01_JPRB, 0.25646E+01_JPRB ,&
&  0.26974E+01_JPRB, 0.28080E+01_JPRB, 0.28498E+01_JPRB, 0.27949E+01_JPRB, 0.27700E+01_JPRB, 0.28028E+01_JPRB, 0.27558E+01_JPRB ,&
&  0.27431E+01_JPRB, 0.27094E+01_JPRB, 0.28578E+01_JPRB, 0.30352E+01_JPRB, 0.30770E+01_JPRB, 0.26491E+01_JPRB, 0.26359E+01_JPRB ,&
&  0.25103E+01_JPRB, 0.23596E+01_JPRB, 0.22542E+01_JPRB, 0.24120E+01_JPRB, 0.17439E+01_JPRB, 0.18207E+01_JPRB, 0.17822E+01_JPRB ,&
&  0.19170E+01_JPRB, 0.18448E+01_JPRB, 0.17334E+01_JPRB, 0.16127E+01_JPRB, 0.16184E+01_JPRB, 0.13534E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.12691E+01_JPRB, 0.13565E+01_JPRB, 0.12891E+01_JPRB, 0.13095E+01_JPRB, 0.12648E+01_JPRB, 0.13648E+01_JPRB, 0.14029E+01_JPRB ,&
&  0.15004E+01_JPRB, 0.11930E+01_JPRB, 0.10394E+01_JPRB, 0.91804E+00_JPRB, 0.67293E+00_JPRB, 0.57895E+00_JPRB, 0.58283E+00_JPRB ,&
&  0.59235E+00_JPRB, 0.61192E+00_JPRB, 0.62019E+00_JPRB, 0.64803E+00_JPRB, 0.67078E+00_JPRB, 0.66608E+00_JPRB, 0.65458E+00_JPRB ,&
&  0.65825E+00_JPRB, 0.68004E+00_JPRB, 0.67372E+00_JPRB, 0.67264E+00_JPRB, 0.67169E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.77482E+00_JPRB, 0.76334E+00_JPRB, 0.76594E+00_JPRB, 0.85043E+00_JPRB, 0.98272E+00_JPRB, 0.10948E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.11201E+01_JPRB, 0.95982E+00_JPRB, 0.10229E+01_JPRB, 0.10763E+01_JPRB, 0.12043E+01_JPRB, 0.13514E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.16632E+01_JPRB, 0.18106E+01_JPRB, 0.19219E+01_JPRB, 0.20250E+01_JPRB, 0.21354E+01_JPRB, 0.22382E+01_JPRB, 0.19319E+01_JPRB ,&
&  0.14958E+01_JPRB, 0.16456E+01_JPRB, 0.25877E+01_JPRB, 0.23667E+01_JPRB, 0.22179E+01_JPRB, 0.22006E+01_JPRB, 0.20748E+01_JPRB ,&
&  0.18511E+01_JPRB, 0.16542E+01_JPRB, 0.16909E+01_JPRB, 0.17380E+01_JPRB, 0.17179E+01_JPRB, 0.18189E+01_JPRB, 0.19601E+01_JPRB ,&
&  0.20553E+01_JPRB, 0.21516E+01_JPRB, 0.21984E+01_JPRB, 0.22302E+01_JPRB, 0.22762E+01_JPRB, 0.23499E+01_JPRB, 0.24605E+01_JPRB ,&
&  0.25903E+01_JPRB, 0.27132E+01_JPRB, 0.27679E+01_JPRB, 0.26510E+01_JPRB, 0.26322E+01_JPRB, 0.27012E+01_JPRB, 0.27214E+01_JPRB ,&
&  0.26890E+01_JPRB, 0.27928E+01_JPRB, 0.28750E+01_JPRB, 0.28691E+01_JPRB, 0.27006E+01_JPRB, 0.30390E+01_JPRB, 0.28160E+01_JPRB ,&
&  0.27733E+01_JPRB, 0.25904E+01_JPRB, 0.27547E+01_JPRB, 0.20865E+01_JPRB, 0.21109E+01_JPRB, 0.21295E+01_JPRB, 0.20045E+01_JPRB ,&
&  0.19086E+01_JPRB, 0.19424E+01_JPRB, 0.18644E+01_JPRB, 0.17821E+01_JPRB, 0.17529E+01_JPRB, 0.15063E+01_JPRB, 0.14658E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.15855E+01_JPRB, 0.13859E+01_JPRB, 0.13232E+01_JPRB, 0.12411E+01_JPRB, 0.13791E+01_JPRB, 0.15980E+01_JPRB ,&
&  0.18357E+01_JPRB, 0.17213E+01_JPRB, 0.15345E+01_JPRB, 0.12677E+01_JPRB, 0.97209E+00_JPRB, 0.76602E+00_JPRB, 0.66324E+00_JPRB ,&
&  0.64046E+00_JPRB, 0.65050E+00_JPRB, 0.68154E+00_JPRB, 0.70319E+00_JPRB, 0.73008E+00_JPRB, 0.75572E+00_JPRB, 0.76671E+00_JPRB ,&
&  0.77143E+00_JPRB, 0.76462E+00_JPRB, 0.75164E+00_JPRB, 0.77920E+00_JPRB, 0.79234E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.84509E+00_JPRB, 0.81465E+00_JPRB, 0.78995E+00_JPRB, 0.81791E+00_JPRB, 0.89090E+00_JPRB, 0.92606E+00_JPRB, 0.93654E+00_JPRB ,&
&  0.93769E+00_JPRB, 0.93571E+00_JPRB, 0.98658E+00_JPRB, 0.10203E+01_JPRB, 0.11440E+01_JPRB, 0.12836E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.15258E+01_JPRB, 0.16246E+01_JPRB, 0.17431E+01_JPRB, 0.19030E+01_JPRB, 0.20212E+01_JPRB, 0.21142E+01_JPRB, 0.16293E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.24667E+01_JPRB, 0.23131E+01_JPRB, 0.24360E+01_JPRB, 0.22878E+01_JPRB, 0.22213E+01_JPRB, 0.19289E+01_JPRB ,&
&  0.17374E+01_JPRB, 0.15954E+01_JPRB, 0.16231E+01_JPRB, 0.16573E+01_JPRB, 0.16641E+01_JPRB, 0.17276E+01_JPRB, 0.18218E+01_JPRB ,&
&  0.19113E+01_JPRB, 0.20128E+01_JPRB, 0.20744E+01_JPRB, 0.21219E+01_JPRB, 0.21848E+01_JPRB, 0.22349E+01_JPRB, 0.23420E+01_JPRB ,&
&  0.24849E+01_JPRB, 0.26182E+01_JPRB, 0.26822E+01_JPRB, 0.27299E+01_JPRB, 0.26476E+01_JPRB, 0.27596E+01_JPRB, 0.29194E+01_JPRB ,&
&  0.28387E+01_JPRB, 0.27331E+01_JPRB, 0.30125E+01_JPRB, 0.30527E+01_JPRB, 0.29886E+01_JPRB, 0.31916E+01_JPRB, 0.28886E+01_JPRB ,&
&  0.30257E+01_JPRB, 0.27269E+01_JPRB, 0.30046E+01_JPRB, 0.23050E+01_JPRB, 0.22817E+01_JPRB, 0.24146E+01_JPRB, 0.21967E+01_JPRB ,&
&  0.19802E+01_JPRB, 0.20451E+01_JPRB, 0.19425E+01_JPRB, 0.19208E+01_JPRB, 0.18825E+01_JPRB, 0.17129E+01_JPRB, 0.15412E+01_JPRB ,&
&  0.16033E+01_JPRB, 0.17007E+01_JPRB, 0.16069E+01_JPRB, 0.12637E+01_JPRB, 0.12366E+01_JPRB, 0.13496E+01_JPRB, 0.18700E+01_JPRB ,&
&  0.22002E+01_JPRB, 0.21161E+01_JPRB, 0.20732E+01_JPRB, 0.19521E+01_JPRB, 0.15824E+01_JPRB, 0.13021E+01_JPRB, 0.10129E+01_JPRB ,&
&  0.77920E+00_JPRB, 0.68789E+00_JPRB, 0.73813E+00_JPRB, 0.72826E+00_JPRB, 0.75808E+00_JPRB, 0.79572E+00_JPRB, 0.81741E+00_JPRB ,&
&  0.84267E+00_JPRB, 0.85496E+00_JPRB, 0.86330E+00_JPRB, 0.85561E+00_JPRB, 0.85719E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.84852E+00_JPRB, 0.81555E+00_JPRB, 0.79482E+00_JPRB, 0.80175E+00_JPRB, 0.87679E+00_JPRB, 0.91610E+00_JPRB, 0.90295E+00_JPRB ,&
&  0.89295E+00_JPRB, 0.89766E+00_JPRB, 0.93115E+00_JPRB, 0.92118E+00_JPRB, 0.98603E+00_JPRB, 0.11236E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.14212E+01_JPRB, 0.15753E+01_JPRB, 0.17760E+01_JPRB, 0.19774E+01_JPRB, 0.22166E+01_JPRB, 0.14077E+01_JPRB ,&
&  0.14473E+01_JPRB, 0.21304E+01_JPRB, 0.22798E+01_JPRB, 0.24479E+01_JPRB, 0.23474E+01_JPRB, 0.22231E+01_JPRB, 0.19123E+01_JPRB ,&
&  0.17628E+01_JPRB, 0.15925E+01_JPRB, 0.14864E+01_JPRB, 0.14526E+01_JPRB, 0.14380E+01_JPRB, 0.15227E+01_JPRB, 0.15616E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.16128E+01_JPRB, 0.17392E+01_JPRB, 0.18593E+01_JPRB, 0.19740E+01_JPRB, 0.20815E+01_JPRB, 0.22152E+01_JPRB ,&
&  0.23888E+01_JPRB, 0.25493E+01_JPRB, 0.27775E+01_JPRB, 0.27614E+01_JPRB, 0.28209E+01_JPRB, 0.29359E+01_JPRB, 0.31696E+01_JPRB ,&
&  0.31069E+01_JPRB, 0.30172E+01_JPRB, 0.34470E+01_JPRB, 0.38848E+01_JPRB, 0.39322E+01_JPRB, 0.35531E+01_JPRB, 0.31438E+01_JPRB ,&
&  0.30683E+01_JPRB, 0.29092E+01_JPRB, 0.28610E+01_JPRB, 0.25461E+01_JPRB, 0.25988E+01_JPRB, 0.28100E+01_JPRB, 0.27575E+01_JPRB ,&
&  0.25271E+01_JPRB, 0.23103E+01_JPRB, 0.21535E+01_JPRB, 0.20812E+01_JPRB, 0.20304E+01_JPRB, 0.18378E+01_JPRB, 0.17816E+01_JPRB ,&
&  0.20427E+01_JPRB, 0.24066E+01_JPRB, 0.20980E+01_JPRB, 0.15345E+01_JPRB, 0.13162E+01_JPRB, 0.12183E+01_JPRB, 0.15221E+01_JPRB ,&
&  0.23914E+01_JPRB, 0.23346E+01_JPRB, 0.22184E+01_JPRB, 0.25083E+01_JPRB, 0.22133E+01_JPRB, 0.19299E+01_JPRB, 0.15578E+01_JPRB ,&
&  0.12011E+01_JPRB, 0.96212E+00_JPRB, 0.87060E+00_JPRB, 0.79854E+00_JPRB, 0.79233E+00_JPRB, 0.80866E+00_JPRB, 0.83861E+00_JPRB ,&
&  0.86952E+00_JPRB, 0.88060E+00_JPRB, 0.87447E+00_JPRB, 0.86261E+00_JPRB, 0.86713E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.87721E+00_JPRB, 0.83909E+00_JPRB, 0.82651E+00_JPRB, 0.81678E+00_JPRB, 0.86381E+00_JPRB, 0.90363E+00_JPRB, 0.86562E+00_JPRB ,&
&  0.86603E+00_JPRB, 0.85597E+00_JPRB, 0.86357E+00_JPRB, 0.86009E+00_JPRB, 0.88781E+00_JPRB, 0.95883E+00_JPRB, 0.10199E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.11932E+01_JPRB, 0.13522E+01_JPRB, 0.15527E+01_JPRB, 0.16650E+01_JPRB, 0.16045E+01_JPRB, 0.16660E+01_JPRB ,&
&  0.19070E+01_JPRB, 0.22390E+01_JPRB, 0.23736E+01_JPRB, 0.26654E+01_JPRB, 0.27315E+01_JPRB, 0.28492E+01_JPRB, 0.23614E+01_JPRB ,&
&  0.21426E+01_JPRB, 0.18799E+01_JPRB, 0.16187E+01_JPRB, 0.14611E+01_JPRB, 0.13310E+01_JPRB, 0.13194E+01_JPRB, 0.13755E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.13913E+01_JPRB, 0.14649E+01_JPRB, 0.15482E+01_JPRB, 0.16887E+01_JPRB, 0.18338E+01_JPRB, 0.20250E+01_JPRB ,&
&  0.22710E+01_JPRB, 0.25028E+01_JPRB, 0.27454E+01_JPRB, 0.28831E+01_JPRB, 0.29668E+01_JPRB, 0.31256E+01_JPRB, 0.34796E+01_JPRB ,&
&  0.36654E+01_JPRB, 0.38520E+01_JPRB, 0.39982E+01_JPRB, 0.42831E+01_JPRB, 0.45256E+01_JPRB, 0.43382E+01_JPRB, 0.37497E+01_JPRB ,&
&  0.33247E+01_JPRB, 0.29520E+01_JPRB, 0.25439E+01_JPRB, 0.26202E+01_JPRB, 0.28139E+01_JPRB, 0.32967E+01_JPRB, 0.30000E+01_JPRB ,&
&  0.24121E+01_JPRB, 0.20294E+01_JPRB, 0.19393E+01_JPRB, 0.21130E+01_JPRB, 0.23152E+01_JPRB, 0.22067E+01_JPRB, 0.22563E+01_JPRB ,&
&  0.21577E+01_JPRB, 0.13046E+01_JPRB, 0.10253E+01_JPRB, 0.13470E+01_JPRB, 0.99643E+00_JPRB, 0.89375E+00_JPRB, 0.15701E+01_JPRB ,&
&  0.23579E+01_JPRB, 0.28409E+01_JPRB, 0.27111E+01_JPRB, 0.24569E+01_JPRB, 0.26215E+01_JPRB, 0.25522E+01_JPRB, 0.21775E+01_JPRB ,&
&  0.16155E+01_JPRB, 0.12694E+01_JPRB, 0.10923E+01_JPRB, 0.97367E+00_JPRB, 0.88278E+00_JPRB, 0.85479E+00_JPRB, 0.85470E+00_JPRB ,&
&  0.83966E+00_JPRB, 0.85258E+00_JPRB, 0.85688E+00_JPRB, 0.85575E+00_JPRB, 0.87269E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.87644E+00_JPRB, 0.86015E+00_JPRB, 0.85100E+00_JPRB, 0.84823E+00_JPRB, 0.83096E+00_JPRB, 0.86119E+00_JPRB, 0.88046E+00_JPRB ,&
&  0.86352E+00_JPRB, 0.81634E+00_JPRB, 0.79324E+00_JPRB, 0.80509E+00_JPRB, 0.84146E+00_JPRB, 0.87618E+00_JPRB, 0.90553E+00_JPRB ,&
&  0.95146E+00_JPRB, 0.10205E+01_JPRB, 0.11780E+01_JPRB, 0.13420E+01_JPRB, 0.14165E+01_JPRB, 0.13655E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.20054E+01_JPRB, 0.23414E+01_JPRB, 0.28495E+01_JPRB, 0.30357E+01_JPRB, 0.29833E+01_JPRB, 0.29640E+01_JPRB, 0.26251E+01_JPRB ,&
&  0.23582E+01_JPRB, 0.21241E+01_JPRB, 0.18801E+01_JPRB, 0.16820E+01_JPRB, 0.15558E+01_JPRB, 0.14226E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.13223E+01_JPRB, 0.13103E+01_JPRB, 0.13562E+01_JPRB, 0.13929E+01_JPRB, 0.15124E+01_JPRB, 0.16462E+01_JPRB, 0.18558E+01_JPRB ,&
&  0.21442E+01_JPRB, 0.24386E+01_JPRB, 0.27312E+01_JPRB, 0.31414E+01_JPRB, 0.26404E+01_JPRB, 0.32322E+01_JPRB, 0.34718E+01_JPRB ,&
&  0.37900E+01_JPRB, 0.39724E+01_JPRB, 0.39250E+01_JPRB, 0.48714E+01_JPRB, 0.53666E+01_JPRB, 0.48758E+01_JPRB, 0.42604E+01_JPRB ,&
&  0.37742E+01_JPRB, 0.31891E+01_JPRB, 0.27722E+01_JPRB, 0.28827E+01_JPRB, 0.30158E+01_JPRB, 0.34625E+01_JPRB, 0.24260E+01_JPRB ,&
&  0.21812E+01_JPRB, 0.25484E+01_JPRB, 0.23315E+01_JPRB, 0.19008E+01_JPRB, 0.22609E+01_JPRB, 0.26012E+01_JPRB, 0.17672E+01_JPRB ,&
&  0.80459E+00_JPRB, 0.67410E+00_JPRB, 0.72546E+00_JPRB, 0.72888E+00_JPRB, 0.67190E+00_JPRB, 0.90316E+00_JPRB, 0.30127E+01_JPRB ,&
&  0.23956E+01_JPRB, 0.31183E+01_JPRB, 0.27247E+01_JPRB, 0.25455E+01_JPRB, 0.25525E+01_JPRB, 0.27693E+01_JPRB, 0.27092E+01_JPRB ,&
&  0.19484E+01_JPRB, 0.15945E+01_JPRB, 0.12873E+01_JPRB, 0.11040E+01_JPRB, 0.96687E+00_JPRB, 0.89616E+00_JPRB, 0.86912E+00_JPRB ,&
&  0.84667E+00_JPRB, 0.83779E+00_JPRB, 0.83610E+00_JPRB, 0.84266E+00_JPRB, 0.85903E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.90211E+00_JPRB, 0.88567E+00_JPRB, 0.86802E+00_JPRB, 0.85645E+00_JPRB, 0.83082E+00_JPRB, 0.83677E+00_JPRB, 0.88546E+00_JPRB ,&
&  0.88594E+00_JPRB, 0.82813E+00_JPRB, 0.79381E+00_JPRB, 0.79068E+00_JPRB, 0.82081E+00_JPRB, 0.85366E+00_JPRB, 0.86093E+00_JPRB ,&
&  0.89260E+00_JPRB, 0.94625E+00_JPRB, 0.11631E+01_JPRB, 0.11569E+01_JPRB, 0.12165E+01_JPRB, 0.11741E+01_JPRB, 0.15070E+01_JPRB ,&
&  0.19693E+01_JPRB, 0.24490E+01_JPRB, 0.28998E+01_JPRB, 0.30951E+01_JPRB, 0.34799E+01_JPRB, 0.35538E+01_JPRB, 0.34227E+01_JPRB ,&
&  0.29223E+01_JPRB, 0.25285E+01_JPRB, 0.22074E+01_JPRB, 0.19704E+01_JPRB, 0.17868E+01_JPRB, 0.15821E+01_JPRB, 0.14623E+01_JPRB ,&
&  0.13536E+01_JPRB, 0.13118E+01_JPRB, 0.12814E+01_JPRB, 0.12916E+01_JPRB, 0.13563E+01_JPRB, 0.14782E+01_JPRB, 0.16816E+01_JPRB ,&
&  0.19641E+01_JPRB, 0.23249E+01_JPRB, 0.26551E+01_JPRB, 0.31376E+01_JPRB, 0.27992E+01_JPRB, 0.25298E+01_JPRB, 0.31095E+01_JPRB ,&
&  0.37021E+01_JPRB, 0.42295E+01_JPRB, 0.44262E+01_JPRB, 0.58447E+01_JPRB, 0.59253E+01_JPRB, 0.52995E+01_JPRB, 0.50217E+01_JPRB ,&
&  0.42172E+01_JPRB, 0.40286E+01_JPRB, 0.31368E+01_JPRB, 0.31481E+01_JPRB, 0.33499E+01_JPRB, 0.27124E+01_JPRB, 0.26108E+01_JPRB ,&
&  0.30076E+01_JPRB, 0.27891E+01_JPRB, 0.24353E+01_JPRB, 0.19066E+01_JPRB, 0.23310E+01_JPRB, 0.21055E+01_JPRB, 0.10083E+01_JPRB ,&
&  0.83838E+00_JPRB, 0.80448E+00_JPRB, 0.77801E+00_JPRB, 0.73912E+00_JPRB, 0.75476E+00_JPRB, 0.10154E+01_JPRB, 0.19328E+01_JPRB ,&
&  0.20124E+01_JPRB, 0.29413E+01_JPRB, 0.27621E+01_JPRB, 0.26103E+01_JPRB, 0.23782E+01_JPRB, 0.27252E+01_JPRB, 0.24692E+01_JPRB ,&
&  0.21007E+01_JPRB, 0.17966E+01_JPRB, 0.14023E+01_JPRB, 0.12045E+01_JPRB, 0.10424E+01_JPRB, 0.92600E+00_JPRB, 0.89869E+00_JPRB ,&
&  0.88127E+00_JPRB, 0.85780E+00_JPRB, 0.85392E+00_JPRB, 0.86405E+00_JPRB, 0.87789E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.93270E+00_JPRB, 0.91727E+00_JPRB, 0.90249E+00_JPRB, 0.92152E+00_JPRB, 0.91531E+00_JPRB, 0.89492E+00_JPRB, 0.91449E+00_JPRB ,&
&  0.90276E+00_JPRB, 0.86272E+00_JPRB, 0.83141E+00_JPRB, 0.82268E+00_JPRB, 0.81704E+00_JPRB, 0.84393E+00_JPRB, 0.85526E+00_JPRB ,&
&  0.87850E+00_JPRB, 0.96231E+00_JPRB, 0.10832E+01_JPRB, 0.10356E+01_JPRB, 0.99072E+00_JPRB, 0.98051E+00_JPRB, 0.11865E+01_JPRB ,&
&  0.18263E+01_JPRB, 0.24587E+01_JPRB, 0.29094E+01_JPRB, 0.33528E+01_JPRB, 0.37647E+01_JPRB, 0.37237E+01_JPRB, 0.38431E+01_JPRB ,&
&  0.34341E+01_JPRB, 0.29314E+01_JPRB, 0.25690E+01_JPRB, 0.22586E+01_JPRB, 0.20925E+01_JPRB, 0.19378E+01_JPRB, 0.17741E+01_JPRB ,&
&  0.15607E+01_JPRB, 0.14034E+01_JPRB, 0.13274E+01_JPRB, 0.12589E+01_JPRB, 0.12452E+01_JPRB, 0.13250E+01_JPRB, 0.14847E+01_JPRB ,&
&  0.17618E+01_JPRB, 0.21389E+01_JPRB, 0.24394E+01_JPRB, 0.29227E+01_JPRB, 0.32582E+01_JPRB, 0.32651E+01_JPRB, 0.31807E+01_JPRB ,&
&  0.32599E+01_JPRB, 0.34726E+01_JPRB, 0.51310E+01_JPRB, 0.60102E+01_JPRB, 0.60562E+01_JPRB, 0.58472E+01_JPRB, 0.54197E+01_JPRB ,&
&  0.45009E+01_JPRB, 0.40958E+01_JPRB, 0.34055E+01_JPRB, 0.35435E+01_JPRB, 0.31459E+01_JPRB, 0.25576E+01_JPRB, 0.31453E+01_JPRB ,&
&  0.34816E+01_JPRB, 0.31721E+01_JPRB, 0.25822E+01_JPRB, 0.20877E+01_JPRB, 0.18093E+01_JPRB, 0.13429E+01_JPRB, 0.96822E+00_JPRB ,&
&  0.89794E+00_JPRB, 0.88842E+00_JPRB, 0.82132E+00_JPRB, 0.85880E+00_JPRB, 0.85073E+00_JPRB, 0.10867E+01_JPRB, 0.14944E+01_JPRB ,&
&  0.21306E+01_JPRB, 0.25336E+01_JPRB, 0.26794E+01_JPRB, 0.25595E+01_JPRB, 0.22882E+01_JPRB, 0.23465E+01_JPRB, 0.25025E+01_JPRB ,&
&  0.21131E+01_JPRB, 0.19298E+01_JPRB, 0.16069E+01_JPRB, 0.13176E+01_JPRB, 0.11462E+01_JPRB, 0.10119E+01_JPRB, 0.92552E+00_JPRB ,&
&  0.89025E+00_JPRB, 0.89824E+00_JPRB, 0.91091E+00_JPRB, 0.91495E+00_JPRB, 0.90053E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.95969E+00_JPRB, 0.97024E+00_JPRB, 0.97152E+00_JPRB, 0.98398E+00_JPRB, 0.98508E+00_JPRB, 0.97123E+00_JPRB, 0.96134E+00_JPRB ,&
&  0.93384E+00_JPRB, 0.90497E+00_JPRB, 0.88267E+00_JPRB, 0.87254E+00_JPRB, 0.85902E+00_JPRB, 0.85914E+00_JPRB, 0.84651E+00_JPRB ,&
&  0.85910E+00_JPRB, 0.99431E+00_JPRB, 0.85533E+00_JPRB, 0.83532E+00_JPRB, 0.88037E+00_JPRB, 0.91935E+00_JPRB, 0.10346E+01_JPRB ,&
&  0.16795E+01_JPRB, 0.24856E+01_JPRB, 0.31038E+01_JPRB, 0.39614E+01_JPRB, 0.47366E+01_JPRB, 0.43630E+01_JPRB, 0.41615E+01_JPRB ,&
&  0.39929E+01_JPRB, 0.33601E+01_JPRB, 0.29520E+01_JPRB, 0.26331E+01_JPRB, 0.23738E+01_JPRB, 0.21840E+01_JPRB, 0.19720E+01_JPRB ,&
&  0.17365E+01_JPRB, 0.15572E+01_JPRB, 0.14274E+01_JPRB, 0.13309E+01_JPRB, 0.12616E+01_JPRB, 0.13198E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.15805E+01_JPRB, 0.18829E+01_JPRB, 0.22005E+01_JPRB, 0.25774E+01_JPRB, 0.28965E+01_JPRB, 0.31062E+01_JPRB, 0.42807E+01_JPRB ,&
&  0.43203E+01_JPRB, 0.45641E+01_JPRB, 0.52695E+01_JPRB, 0.70155E+01_JPRB, 0.66016E+01_JPRB, 0.57153E+01_JPRB, 0.58279E+01_JPRB ,&
&  0.38827E+01_JPRB, 0.32338E+01_JPRB, 0.32987E+01_JPRB, 0.33491E+01_JPRB, 0.28747E+01_JPRB, 0.31712E+01_JPRB, 0.49322E+01_JPRB ,&
&  0.38140E+01_JPRB, 0.36653E+01_JPRB, 0.37530E+01_JPRB, 0.40806E+01_JPRB, 0.18816E+01_JPRB, 0.12865E+01_JPRB, 0.20614E+01_JPRB ,&
&  0.22353E+01_JPRB, 0.14161E+01_JPRB, 0.10369E+01_JPRB, 0.13780E+01_JPRB, 0.10594E+01_JPRB, 0.12383E+01_JPRB, 0.17184E+01_JPRB ,&
&  0.18172E+01_JPRB, 0.19477E+01_JPRB, 0.26445E+01_JPRB, 0.24612E+01_JPRB, 0.22327E+01_JPRB, 0.22283E+01_JPRB, 0.22779E+01_JPRB ,&
&  0.22729E+01_JPRB, 0.20322E+01_JPRB, 0.18786E+01_JPRB, 0.15552E+01_JPRB, 0.13493E+01_JPRB, 0.11991E+01_JPRB, 0.10925E+01_JPRB ,&
&  0.99198E+00_JPRB, 0.94132E+00_JPRB, 0.94406E+00_JPRB, 0.94647E+00_JPRB, 0.93443E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.10044E+01_JPRB, 0.10054E+01_JPRB, 0.10238E+01_JPRB, 0.10300E+01_JPRB, 0.10410E+01_JPRB, 0.10318E+01_JPRB, 0.10209E+01_JPRB ,&
&  0.97969E+00_JPRB, 0.93997E+00_JPRB, 0.93187E+00_JPRB, 0.92408E+00_JPRB, 0.91145E+00_JPRB, 0.89198E+00_JPRB, 0.85954E+00_JPRB ,&
&  0.86448E+00_JPRB, 0.88264E+00_JPRB, 0.77353E+00_JPRB, 0.76066E+00_JPRB, 0.86589E+00_JPRB, 0.89369E+00_JPRB, 0.10339E+01_JPRB ,&
&  0.16307E+01_JPRB, 0.24474E+01_JPRB, 0.32435E+01_JPRB, 0.42901E+01_JPRB, 0.49337E+01_JPRB, 0.48075E+01_JPRB, 0.44261E+01_JPRB ,&
&  0.43811E+01_JPRB, 0.35848E+01_JPRB, 0.31546E+01_JPRB, 0.28904E+01_JPRB, 0.25817E+01_JPRB, 0.23007E+01_JPRB, 0.20664E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.16353E+01_JPRB, 0.15075E+01_JPRB, 0.14026E+01_JPRB, 0.13705E+01_JPRB, 0.13793E+01_JPRB, 0.13931E+01_JPRB ,&
&  0.14329E+01_JPRB, 0.15978E+01_JPRB, 0.19765E+01_JPRB, 0.23625E+01_JPRB, 0.25600E+01_JPRB, 0.28615E+01_JPRB, 0.36863E+01_JPRB ,&
&  0.43630E+01_JPRB, 0.45575E+01_JPRB, 0.54658E+01_JPRB, 0.65248E+01_JPRB, 0.73873E+01_JPRB, 0.63601E+01_JPRB, 0.69617E+01_JPRB ,&
&  0.48235E+01_JPRB, 0.38454E+01_JPRB, 0.32805E+01_JPRB, 0.27883E+01_JPRB, 0.31092E+01_JPRB, 0.49233E+01_JPRB, 0.44977E+01_JPRB ,&
&  0.41183E+01_JPRB, 0.37754E+01_JPRB, 0.37194E+01_JPRB, 0.32908E+01_JPRB, 0.28600E+01_JPRB, 0.19827E+01_JPRB, 0.25648E+01_JPRB ,&
&  0.27313E+01_JPRB, 0.26933E+01_JPRB, 0.23712E+01_JPRB, 0.19718E+01_JPRB, 0.15752E+01_JPRB, 0.17898E+01_JPRB, 0.17795E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.16917E+01_JPRB, 0.20019E+01_JPRB, 0.21618E+01_JPRB, 0.22125E+01_JPRB, 0.20900E+01_JPRB, 0.22706E+01_JPRB ,&
&  0.21757E+01_JPRB, 0.22195E+01_JPRB, 0.20797E+01_JPRB, 0.17656E+01_JPRB, 0.15292E+01_JPRB, 0.13922E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.11778E+01_JPRB, 0.10607E+01_JPRB, 0.10039E+01_JPRB, 0.97122E+00_JPRB, 0.96429E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.10498E+01_JPRB, 0.10622E+01_JPRB, 0.10644E+01_JPRB, 0.10614E+01_JPRB, 0.10494E+01_JPRB, 0.10281E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.98949E+00_JPRB, 0.94572E+00_JPRB, 0.92916E+00_JPRB, 0.92088E+00_JPRB, 0.91221E+00_JPRB, 0.88767E+00_JPRB, 0.86325E+00_JPRB ,&
&  0.86506E+00_JPRB, 0.85644E+00_JPRB, 0.74425E+00_JPRB, 0.78511E+00_JPRB, 0.83745E+00_JPRB, 0.86837E+00_JPRB, 0.10658E+01_JPRB ,&
&  0.13875E+01_JPRB, 0.20295E+01_JPRB, 0.27414E+01_JPRB, 0.37301E+01_JPRB, 0.45952E+01_JPRB, 0.49382E+01_JPRB, 0.49055E+01_JPRB ,&
&  0.43319E+01_JPRB, 0.39312E+01_JPRB, 0.34019E+01_JPRB, 0.29724E+01_JPRB, 0.26139E+01_JPRB, 0.23582E+01_JPRB, 0.21020E+01_JPRB ,&
&  0.18868E+01_JPRB, 0.17132E+01_JPRB, 0.16016E+01_JPRB, 0.14987E+01_JPRB, 0.14594E+01_JPRB, 0.14550E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14000E+01_JPRB, 0.14356E+01_JPRB, 0.17150E+01_JPRB, 0.21673E+01_JPRB, 0.23318E+01_JPRB, 0.26536E+01_JPRB, 0.30098E+01_JPRB ,&
&  0.45182E+01_JPRB, 0.52685E+01_JPRB, 0.54042E+01_JPRB, 0.72188E+01_JPRB, 0.51906E+01_JPRB, 0.64112E+01_JPRB, 0.79059E+01_JPRB ,&
&  0.70363E+01_JPRB, 0.61408E+01_JPRB, 0.58314E+01_JPRB, 0.45105E+01_JPRB, 0.35673E+01_JPRB, 0.48270E+01_JPRB, 0.40780E+01_JPRB ,&
&  0.35750E+01_JPRB, 0.36029E+01_JPRB, 0.35021E+01_JPRB, 0.37787E+01_JPRB, 0.28713E+01_JPRB, 0.24477E+01_JPRB, 0.19213E+01_JPRB ,&
&  0.19817E+01_JPRB, 0.18164E+01_JPRB, 0.21283E+01_JPRB, 0.18714E+01_JPRB, 0.18250E+01_JPRB, 0.17204E+01_JPRB, 0.16227E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.16095E+01_JPRB, 0.15798E+01_JPRB, 0.17823E+01_JPRB, 0.20823E+01_JPRB, 0.19114E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.21675E+01_JPRB, 0.21840E+01_JPRB, 0.20514E+01_JPRB, 0.19125E+01_JPRB, 0.16530E+01_JPRB, 0.14789E+01_JPRB, 0.13870E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12161E+01_JPRB, 0.11073E+01_JPRB, 0.10629E+01_JPRB, 0.10250E+01_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.10874E+01_JPRB, 0.10806E+01_JPRB, 0.10480E+01_JPRB, 0.10499E+01_JPRB, 0.10451E+01_JPRB, 0.10204E+01_JPRB, 0.98400E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.91954E+00_JPRB, 0.89565E+00_JPRB, 0.88282E+00_JPRB, 0.88324E+00_JPRB, 0.87444E+00_JPRB, 0.85503E+00_JPRB ,&
&  0.84465E+00_JPRB, 0.97558E+00_JPRB, 0.86191E+00_JPRB, 0.77920E+00_JPRB, 0.73533E+00_JPRB, 0.84268E+00_JPRB, 0.10300E+01_JPRB ,&
&  0.13133E+01_JPRB, 0.16565E+01_JPRB, 0.21838E+01_JPRB, 0.26382E+01_JPRB, 0.32614E+01_JPRB, 0.38087E+01_JPRB, 0.41435E+01_JPRB ,&
&  0.42576E+01_JPRB, 0.32278E+01_JPRB, 0.29287E+01_JPRB, 0.25230E+01_JPRB, 0.22666E+01_JPRB, 0.20287E+01_JPRB, 0.18725E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.16260E+01_JPRB, 0.15440E+01_JPRB, 0.14661E+01_JPRB, 0.14240E+01_JPRB, 0.14004E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.13610E+01_JPRB, 0.14919E+01_JPRB, 0.18421E+01_JPRB, 0.24605E+01_JPRB, 0.29919E+01_JPRB, 0.33729E+01_JPRB ,&
&  0.32616E+01_JPRB, 0.33619E+01_JPRB, 0.60117E+01_JPRB, 0.66114E+01_JPRB, 0.65916E+01_JPRB, 0.77909E+01_JPRB, 0.83509E+01_JPRB ,&
&  0.77723E+01_JPRB, 0.61961E+01_JPRB, 0.51362E+01_JPRB, 0.43117E+01_JPRB, 0.48210E+01_JPRB, 0.43512E+01_JPRB, 0.38993E+01_JPRB ,&
&  0.35316E+01_JPRB, 0.37234E+01_JPRB, 0.36497E+01_JPRB, 0.36191E+01_JPRB, 0.35596E+01_JPRB, 0.34881E+01_JPRB, 0.29346E+01_JPRB ,&
&  0.25403E+01_JPRB, 0.30019E+01_JPRB, 0.24094E+01_JPRB, 0.18293E+01_JPRB, 0.14931E+01_JPRB, 0.15290E+01_JPRB, 0.16332E+01_JPRB ,&
&  0.16923E+01_JPRB, 0.17056E+01_JPRB, 0.16672E+01_JPRB, 0.17314E+01_JPRB, 0.18778E+01_JPRB, 0.19150E+01_JPRB, 0.19687E+01_JPRB ,&
&  0.19311E+01_JPRB, 0.20662E+01_JPRB, 0.20410E+01_JPRB, 0.19430E+01_JPRB, 0.17726E+01_JPRB, 0.15730E+01_JPRB, 0.14571E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.13556E+01_JPRB, 0.12537E+01_JPRB, 0.11522E+01_JPRB, 0.10898E+01_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.10972E+01_JPRB, 0.10690E+01_JPRB, 0.10325E+01_JPRB, 0.10349E+01_JPRB, 0.10457E+01_JPRB, 0.10326E+01_JPRB, 0.98500E+00_JPRB ,&
&  0.90959E+00_JPRB, 0.86856E+00_JPRB, 0.85242E+00_JPRB, 0.85794E+00_JPRB, 0.86935E+00_JPRB, 0.87784E+00_JPRB, 0.87932E+00_JPRB ,&
&  0.86837E+00_JPRB, 0.11508E+01_JPRB, 0.10236E+01_JPRB, 0.98177E+00_JPRB, 0.87730E+00_JPRB, 0.99444E+00_JPRB, 0.11511E+01_JPRB ,&
&  0.12803E+01_JPRB, 0.13656E+01_JPRB, 0.15482E+01_JPRB, 0.18247E+01_JPRB, 0.21086E+01_JPRB, 0.23566E+01_JPRB, 0.27658E+01_JPRB ,&
&  0.27366E+01_JPRB, 0.25170E+01_JPRB, 0.20799E+01_JPRB, 0.19804E+01_JPRB, 0.18574E+01_JPRB, 0.16196E+01_JPRB, 0.16196E+01_JPRB ,&
&  0.15264E+01_JPRB, 0.14710E+01_JPRB, 0.14191E+01_JPRB, 0.13809E+01_JPRB, 0.13420E+01_JPRB, 0.12868E+01_JPRB, 0.12589E+01_JPRB ,&
&  0.12502E+01_JPRB, 0.12591E+01_JPRB, 0.13364E+01_JPRB, 0.14718E+01_JPRB, 0.19120E+01_JPRB, 0.24864E+01_JPRB, 0.29526E+01_JPRB ,&
&  0.35920E+01_JPRB, 0.33254E+01_JPRB, 0.31321E+01_JPRB, 0.44455E+01_JPRB, 0.69587E+01_JPRB, 0.75872E+01_JPRB, 0.71898E+01_JPRB ,&
&  0.78300E+01_JPRB, 0.72931E+01_JPRB, 0.63226E+01_JPRB, 0.55029E+01_JPRB, 0.49565E+01_JPRB, 0.44798E+01_JPRB, 0.42457E+01_JPRB ,&
&  0.38897E+01_JPRB, 0.37922E+01_JPRB, 0.39647E+01_JPRB, 0.40521E+01_JPRB, 0.37995E+01_JPRB, 0.35101E+01_JPRB, 0.34820E+01_JPRB ,&
&  0.33028E+01_JPRB, 0.27284E+01_JPRB, 0.18337E+01_JPRB, 0.17835E+01_JPRB, 0.13680E+01_JPRB, 0.13601E+01_JPRB, 0.16084E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.17036E+01_JPRB, 0.17382E+01_JPRB, 0.16402E+01_JPRB, 0.17197E+01_JPRB, 0.17143E+01_JPRB, 0.17871E+01_JPRB ,&
&  0.18460E+01_JPRB, 0.16004E+01_JPRB, 0.16846E+01_JPRB, 0.17338E+01_JPRB, 0.17159E+01_JPRB, 0.16045E+01_JPRB, 0.14987E+01_JPRB ,&
&  0.14392E+01_JPRB, 0.13902E+01_JPRB, 0.13198E+01_JPRB, 0.12166E+01_JPRB, 0.11405E+01_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.11133E+01_JPRB, 0.10764E+01_JPRB, 0.10381E+01_JPRB, 0.10228E+01_JPRB, 0.10172E+01_JPRB, 0.10023E+01_JPRB, 0.95989E+00_JPRB ,&
&  0.90170E+00_JPRB, 0.86165E+00_JPRB, 0.84642E+00_JPRB, 0.85201E+00_JPRB, 0.85953E+00_JPRB, 0.85774E+00_JPRB, 0.88934E+00_JPRB ,&
&  0.10273E+01_JPRB, 0.12123E+01_JPRB, 0.11064E+01_JPRB, 0.97317E+00_JPRB, 0.11452E+01_JPRB, 0.10955E+01_JPRB, 0.11415E+01_JPRB ,&
&  0.11143E+01_JPRB, 0.12553E+01_JPRB, 0.12902E+01_JPRB, 0.13514E+01_JPRB, 0.14307E+01_JPRB, 0.15729E+01_JPRB, 0.18074E+01_JPRB ,&
&  0.18498E+01_JPRB, 0.16518E+01_JPRB, 0.15068E+01_JPRB, 0.14766E+01_JPRB, 0.15251E+01_JPRB, 0.13845E+01_JPRB, 0.13589E+01_JPRB ,&
&  0.12993E+01_JPRB, 0.12518E+01_JPRB, 0.12155E+01_JPRB, 0.11980E+01_JPRB, 0.11816E+01_JPRB, 0.11618E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.11698E+01_JPRB, 0.12200E+01_JPRB, 0.13069E+01_JPRB, 0.15428E+01_JPRB, 0.20958E+01_JPRB, 0.30039E+01_JPRB ,&
&  0.36546E+01_JPRB, 0.38397E+01_JPRB, 0.42677E+01_JPRB, 0.51566E+01_JPRB, 0.56551E+01_JPRB, 0.59509E+01_JPRB, 0.63364E+01_JPRB ,&
&  0.64738E+01_JPRB, 0.64820E+01_JPRB, 0.64671E+01_JPRB, 0.60980E+01_JPRB, 0.54940E+01_JPRB, 0.50071E+01_JPRB, 0.47194E+01_JPRB ,&
&  0.43960E+01_JPRB, 0.42595E+01_JPRB, 0.43419E+01_JPRB, 0.41755E+01_JPRB, 0.40753E+01_JPRB, 0.37681E+01_JPRB, 0.38896E+01_JPRB ,&
&  0.37593E+01_JPRB, 0.21965E+01_JPRB, 0.16907E+01_JPRB, 0.19095E+01_JPRB, 0.15055E+01_JPRB, 0.15473E+01_JPRB, 0.17972E+01_JPRB ,&
&  0.17264E+01_JPRB, 0.16729E+01_JPRB, 0.17078E+01_JPRB, 0.16349E+01_JPRB, 0.15582E+01_JPRB, 0.16116E+01_JPRB, 0.15499E+01_JPRB ,&
&  0.14253E+01_JPRB, 0.13205E+01_JPRB, 0.13687E+01_JPRB, 0.14391E+01_JPRB, 0.14995E+01_JPRB, 0.14664E+01_JPRB, 0.14041E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.13391E+01_JPRB, 0.13088E+01_JPRB, 0.12464E+01_JPRB, 0.11663E+01_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.11575E+01_JPRB, 0.11242E+01_JPRB, 0.10917E+01_JPRB, 0.10625E+01_JPRB, 0.10238E+01_JPRB, 0.97928E+00_JPRB, 0.92573E+00_JPRB ,&
&  0.87988E+00_JPRB, 0.84439E+00_JPRB, 0.82916E+00_JPRB, 0.82518E+00_JPRB, 0.82188E+00_JPRB, 0.81376E+00_JPRB, 0.87040E+00_JPRB ,&
&  0.83458E+00_JPRB, 0.97861E+00_JPRB, 0.90013E+00_JPRB, 0.96419E+00_JPRB, 0.11986E+01_JPRB, 0.10744E+01_JPRB, 0.10657E+01_JPRB ,&
&  0.10731E+01_JPRB, 0.11206E+01_JPRB, 0.11131E+01_JPRB, 0.10999E+01_JPRB, 0.10966E+01_JPRB, 0.11288E+01_JPRB, 0.12441E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.11661E+01_JPRB, 0.11219E+01_JPRB, 0.11253E+01_JPRB, 0.11662E+01_JPRB, 0.12204E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.11973E+01_JPRB, 0.11538E+01_JPRB, 0.11222E+01_JPRB, 0.10941E+01_JPRB, 0.10643E+01_JPRB, 0.10386E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10154E+01_JPRB, 0.10314E+01_JPRB, 0.10889E+01_JPRB, 0.11972E+01_JPRB, 0.13152E+01_JPRB, 0.17959E+01_JPRB, 0.28125E+01_JPRB ,&
&  0.37442E+01_JPRB, 0.38174E+01_JPRB, 0.42825E+01_JPRB, 0.52221E+01_JPRB, 0.54098E+01_JPRB, 0.54128E+01_JPRB, 0.53981E+01_JPRB ,&
&  0.54326E+01_JPRB, 0.53997E+01_JPRB, 0.53891E+01_JPRB, 0.56271E+01_JPRB, 0.56257E+01_JPRB, 0.56624E+01_JPRB, 0.54493E+01_JPRB ,&
&  0.50394E+01_JPRB, 0.48705E+01_JPRB, 0.48058E+01_JPRB, 0.45791E+01_JPRB, 0.42861E+01_JPRB, 0.42021E+01_JPRB, 0.40474E+01_JPRB ,&
&  0.37679E+01_JPRB, 0.33247E+01_JPRB, 0.24728E+01_JPRB, 0.22332E+01_JPRB, 0.16476E+01_JPRB, 0.16627E+01_JPRB, 0.20240E+01_JPRB ,&
&  0.19108E+01_JPRB, 0.16815E+01_JPRB, 0.15314E+01_JPRB, 0.14406E+01_JPRB, 0.13785E+01_JPRB, 0.13921E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.11138E+01_JPRB, 0.11360E+01_JPRB, 0.11752E+01_JPRB, 0.12300E+01_JPRB, 0.12880E+01_JPRB, 0.12976E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.12309E+01_JPRB, 0.12318E+01_JPRB, 0.12281E+01_JPRB, 0.11995E+01_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.11183E+01_JPRB, 0.10951E+01_JPRB, 0.10728E+01_JPRB, 0.10475E+01_JPRB, 0.99643E+00_JPRB, 0.92528E+00_JPRB, 0.86913E+00_JPRB ,&
&  0.84691E+00_JPRB, 0.82303E+00_JPRB, 0.80549E+00_JPRB, 0.79347E+00_JPRB, 0.78187E+00_JPRB, 0.77030E+00_JPRB, 0.76526E+00_JPRB ,&
&  0.75526E+00_JPRB, 0.83968E+00_JPRB, 0.83646E+00_JPRB, 0.87333E+00_JPRB, 0.90744E+00_JPRB, 0.88052E+00_JPRB, 0.88057E+00_JPRB ,&
&  0.93616E+00_JPRB, 0.95500E+00_JPRB, 0.95701E+00_JPRB, 0.94334E+00_JPRB, 0.93284E+00_JPRB, 0.95368E+00_JPRB, 0.10130E+01_JPRB ,&
&  0.98870E+00_JPRB, 0.96984E+00_JPRB, 0.95675E+00_JPRB, 0.97424E+00_JPRB, 0.10485E+01_JPRB, 0.11343E+01_JPRB, 0.11405E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.10656E+01_JPRB, 0.10275E+01_JPRB, 0.99745E+00_JPRB, 0.96974E+00_JPRB, 0.94330E+00_JPRB, 0.92280E+00_JPRB ,&
&  0.91980E+00_JPRB, 0.93872E+00_JPRB, 0.98821E+00_JPRB, 0.10827E+01_JPRB, 0.12041E+01_JPRB, 0.15267E+01_JPRB, 0.20408E+01_JPRB ,&
&  0.24423E+01_JPRB, 0.27737E+01_JPRB, 0.31525E+01_JPRB, 0.36007E+01_JPRB, 0.38963E+01_JPRB, 0.39971E+01_JPRB, 0.40398E+01_JPRB ,&
&  0.42261E+01_JPRB, 0.42222E+01_JPRB, 0.45613E+01_JPRB, 0.45665E+01_JPRB, 0.46117E+01_JPRB, 0.48211E+01_JPRB, 0.47988E+01_JPRB ,&
&  0.46647E+01_JPRB, 0.44091E+01_JPRB, 0.45348E+01_JPRB, 0.44559E+01_JPRB, 0.42945E+01_JPRB, 0.41245E+01_JPRB, 0.38702E+01_JPRB ,&
&  0.36127E+01_JPRB, 0.33597E+01_JPRB, 0.28910E+01_JPRB, 0.25691E+01_JPRB, 0.21926E+01_JPRB, 0.22063E+01_JPRB, 0.19727E+01_JPRB ,&
&  0.17035E+01_JPRB, 0.13451E+01_JPRB, 0.12946E+01_JPRB, 0.12494E+01_JPRB, 0.12413E+01_JPRB, 0.11520E+01_JPRB, 0.10240E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.10696E+01_JPRB, 0.10800E+01_JPRB, 0.10944E+01_JPRB, 0.11343E+01_JPRB, 0.11643E+01_JPRB, 0.99741E+00_JPRB ,&
&  0.98741E+00_JPRB, 0.10780E+01_JPRB, 0.11124E+01_JPRB, 0.11310E+01_JPRB, 0.11322E+01_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.10348E+01_JPRB, 0.10180E+01_JPRB, 0.99322E+00_JPRB, 0.96703E+00_JPRB, 0.92089E+00_JPRB, 0.86151E+00_JPRB, 0.80348E+00_JPRB ,&
&  0.80338E+00_JPRB, 0.79175E+00_JPRB, 0.77893E+00_JPRB, 0.76134E+00_JPRB, 0.74050E+00_JPRB, 0.70262E+00_JPRB, 0.64599E+00_JPRB ,&
&  0.64695E+00_JPRB, 0.68841E+00_JPRB, 0.74713E+00_JPRB, 0.77740E+00_JPRB, 0.77516E+00_JPRB, 0.77125E+00_JPRB, 0.78158E+00_JPRB ,&
&  0.79571E+00_JPRB, 0.80891E+00_JPRB, 0.82959E+00_JPRB, 0.84289E+00_JPRB, 0.83690E+00_JPRB, 0.83366E+00_JPRB, 0.85662E+00_JPRB ,&
&  0.82568E+00_JPRB, 0.81388E+00_JPRB, 0.83652E+00_JPRB, 0.80828E+00_JPRB, 0.89405E+00_JPRB, 0.92949E+00_JPRB, 0.93764E+00_JPRB ,&
&  0.91236E+00_JPRB, 0.87367E+00_JPRB, 0.84520E+00_JPRB, 0.83908E+00_JPRB, 0.83561E+00_JPRB, 0.83003E+00_JPRB, 0.82958E+00_JPRB ,&
&  0.83961E+00_JPRB, 0.86229E+00_JPRB, 0.90765E+00_JPRB, 0.98270E+00_JPRB, 0.10766E+01_JPRB, 0.12714E+01_JPRB, 0.14909E+01_JPRB ,&
&  0.17742E+01_JPRB, 0.21342E+01_JPRB, 0.25316E+01_JPRB, 0.26078E+01_JPRB, 0.29234E+01_JPRB, 0.32028E+01_JPRB, 0.32619E+01_JPRB ,&
&  0.34928E+01_JPRB, 0.35503E+01_JPRB, 0.37788E+01_JPRB, 0.38697E+01_JPRB, 0.39345E+01_JPRB, 0.39737E+01_JPRB, 0.39987E+01_JPRB ,&
&  0.39998E+01_JPRB, 0.39862E+01_JPRB, 0.40431E+01_JPRB, 0.39732E+01_JPRB, 0.37979E+01_JPRB, 0.36262E+01_JPRB, 0.34345E+01_JPRB ,&
&  0.32720E+01_JPRB, 0.30926E+01_JPRB, 0.29074E+01_JPRB, 0.27471E+01_JPRB, 0.25668E+01_JPRB, 0.23236E+01_JPRB, 0.20626E+01_JPRB ,&
&  0.17952E+01_JPRB, 0.14629E+01_JPRB, 0.12850E+01_JPRB, 0.12365E+01_JPRB, 0.11804E+01_JPRB, 0.10333E+01_JPRB, 0.95132E+00_JPRB ,&
&  0.92216E+00_JPRB, 0.94149E+00_JPRB, 0.10005E+01_JPRB, 0.10041E+01_JPRB, 0.10243E+01_JPRB, 0.10271E+01_JPRB, 0.94854E+00_JPRB ,&
&  0.89243E+00_JPRB, 0.96163E+00_JPRB, 0.99719E+00_JPRB, 0.10187E+01_JPRB, 0.10346E+01_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.95380E+00_JPRB, 0.94385E+00_JPRB, 0.91999E+00_JPRB, 0.89822E+00_JPRB, 0.87406E+00_JPRB, 0.81942E+00_JPRB, 0.77126E+00_JPRB ,&
&  0.77991E+00_JPRB, 0.74392E+00_JPRB, 0.73259E+00_JPRB, 0.67619E+00_JPRB, 0.64739E+00_JPRB, 0.63379E+00_JPRB, 0.62299E+00_JPRB ,&
&  0.62960E+00_JPRB, 0.65309E+00_JPRB, 0.67472E+00_JPRB, 0.69379E+00_JPRB, 0.71231E+00_JPRB, 0.71523E+00_JPRB, 0.71664E+00_JPRB ,&
&  0.71817E+00_JPRB, 0.72554E+00_JPRB, 0.73994E+00_JPRB, 0.74493E+00_JPRB, 0.74869E+00_JPRB, 0.74781E+00_JPRB, 0.74884E+00_JPRB ,&
&  0.73228E+00_JPRB, 0.73010E+00_JPRB, 0.75735E+00_JPRB, 0.76029E+00_JPRB, 0.77153E+00_JPRB, 0.77407E+00_JPRB, 0.76226E+00_JPRB ,&
&  0.72944E+00_JPRB, 0.70045E+00_JPRB, 0.71935E+00_JPRB, 0.73196E+00_JPRB, 0.74345E+00_JPRB, 0.74942E+00_JPRB, 0.75263E+00_JPRB ,&
&  0.75642E+00_JPRB, 0.76609E+00_JPRB, 0.78583E+00_JPRB, 0.81814E+00_JPRB, 0.86972E+00_JPRB, 0.97673E+00_JPRB, 0.10804E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.13154E+01_JPRB, 0.16554E+01_JPRB, 0.19088E+01_JPRB, 0.22466E+01_JPRB, 0.25473E+01_JPRB, 0.26664E+01_JPRB ,&
&  0.28788E+01_JPRB, 0.29251E+01_JPRB, 0.29774E+01_JPRB, 0.29736E+01_JPRB, 0.29547E+01_JPRB, 0.29680E+01_JPRB, 0.30446E+01_JPRB ,&
&  0.31200E+01_JPRB, 0.30195E+01_JPRB, 0.32200E+01_JPRB, 0.31424E+01_JPRB, 0.30399E+01_JPRB, 0.29944E+01_JPRB, 0.29493E+01_JPRB ,&
&  0.28688E+01_JPRB, 0.27325E+01_JPRB, 0.25462E+01_JPRB, 0.22155E+01_JPRB, 0.21016E+01_JPRB, 0.19964E+01_JPRB, 0.17923E+01_JPRB ,&
&  0.16701E+01_JPRB, 0.15278E+01_JPRB, 0.13780E+01_JPRB, 0.13475E+01_JPRB, 0.11947E+01_JPRB, 0.10827E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.90812E+00_JPRB, 0.85787E+00_JPRB, 0.88017E+00_JPRB, 0.89214E+00_JPRB, 0.88838E+00_JPRB, 0.87212E+00_JPRB, 0.91237E+00_JPRB ,&
&  0.87785E+00_JPRB, 0.88686E+00_JPRB, 0.93833E+00_JPRB, 0.94454E+00_JPRB, 0.94957E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.90601E+00_JPRB, 0.89860E+00_JPRB, 0.87531E+00_JPRB, 0.85563E+00_JPRB, 0.83195E+00_JPRB, 0.79370E+00_JPRB, 0.73376E+00_JPRB ,&
&  0.70499E+00_JPRB, 0.66788E+00_JPRB, 0.63747E+00_JPRB, 0.62668E+00_JPRB, 0.62223E+00_JPRB, 0.61362E+00_JPRB, 0.60166E+00_JPRB ,&
&  0.60535E+00_JPRB, 0.62558E+00_JPRB, 0.65016E+00_JPRB, 0.66448E+00_JPRB, 0.67550E+00_JPRB, 0.67858E+00_JPRB, 0.68302E+00_JPRB ,&
&  0.68773E+00_JPRB, 0.68842E+00_JPRB, 0.69114E+00_JPRB, 0.68857E+00_JPRB, 0.68484E+00_JPRB, 0.68790E+00_JPRB, 0.69115E+00_JPRB ,&
&  0.69931E+00_JPRB, 0.70255E+00_JPRB, 0.70860E+00_JPRB, 0.71745E+00_JPRB, 0.71974E+00_JPRB, 0.72045E+00_JPRB, 0.67500E+00_JPRB ,&
&  0.61302E+00_JPRB, 0.62919E+00_JPRB, 0.65336E+00_JPRB, 0.66895E+00_JPRB, 0.69110E+00_JPRB, 0.70310E+00_JPRB, 0.70893E+00_JPRB ,&
&  0.70929E+00_JPRB, 0.70872E+00_JPRB, 0.71475E+00_JPRB, 0.73341E+00_JPRB, 0.76716E+00_JPRB, 0.81683E+00_JPRB, 0.86919E+00_JPRB ,&
&  0.99375E+00_JPRB, 0.10152E+01_JPRB, 0.10846E+01_JPRB, 0.12572E+01_JPRB, 0.16486E+01_JPRB, 0.19004E+01_JPRB, 0.20239E+01_JPRB ,&
&  0.21379E+01_JPRB, 0.22131E+01_JPRB, 0.22419E+01_JPRB, 0.22911E+01_JPRB, 0.22817E+01_JPRB, 0.22774E+01_JPRB, 0.23216E+01_JPRB ,&
&  0.23243E+01_JPRB, 0.23825E+01_JPRB, 0.24097E+01_JPRB, 0.24448E+01_JPRB, 0.23933E+01_JPRB, 0.23658E+01_JPRB, 0.23418E+01_JPRB ,&
&  0.23027E+01_JPRB, 0.22223E+01_JPRB, 0.20568E+01_JPRB, 0.18897E+01_JPRB, 0.17350E+01_JPRB, 0.16442E+01_JPRB, 0.15710E+01_JPRB ,&
&  0.14883E+01_JPRB, 0.14255E+01_JPRB, 0.13617E+01_JPRB, 0.12837E+01_JPRB, 0.11840E+01_JPRB, 0.10902E+01_JPRB, 0.10025E+01_JPRB ,&
&  0.86951E+00_JPRB, 0.82303E+00_JPRB, 0.79997E+00_JPRB, 0.78781E+00_JPRB, 0.79371E+00_JPRB, 0.81615E+00_JPRB, 0.80097E+00_JPRB ,&
&  0.83034E+00_JPRB, 0.83994E+00_JPRB, 0.83454E+00_JPRB, 0.85076E+00_JPRB, 0.87588E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.83077E+00_JPRB, 0.83394E+00_JPRB, 0.82121E+00_JPRB, 0.81660E+00_JPRB, 0.79120E+00_JPRB, 0.76252E+00_JPRB, 0.72887E+00_JPRB ,&
&  0.70578E+00_JPRB, 0.67844E+00_JPRB, 0.65033E+00_JPRB, 0.62549E+00_JPRB, 0.61345E+00_JPRB, 0.61018E+00_JPRB, 0.61666E+00_JPRB ,&
&  0.62952E+00_JPRB, 0.64429E+00_JPRB, 0.65595E+00_JPRB, 0.66072E+00_JPRB, 0.65917E+00_JPRB, 0.65782E+00_JPRB, 0.66097E+00_JPRB ,&
&  0.66253E+00_JPRB, 0.66302E+00_JPRB, 0.66556E+00_JPRB, 0.66479E+00_JPRB, 0.66467E+00_JPRB, 0.66550E+00_JPRB, 0.66759E+00_JPRB ,&
&  0.66792E+00_JPRB, 0.66437E+00_JPRB, 0.67138E+00_JPRB, 0.68200E+00_JPRB, 0.68945E+00_JPRB, 0.67450E+00_JPRB, 0.62124E+00_JPRB ,&
&  0.57814E+00_JPRB, 0.57065E+00_JPRB, 0.61667E+00_JPRB, 0.62304E+00_JPRB, 0.64754E+00_JPRB, 0.65850E+00_JPRB, 0.65736E+00_JPRB ,&
&  0.64613E+00_JPRB, 0.64225E+00_JPRB, 0.65513E+00_JPRB, 0.67828E+00_JPRB, 0.69699E+00_JPRB, 0.72411E+00_JPRB, 0.75477E+00_JPRB ,&
&  0.80439E+00_JPRB, 0.87432E+00_JPRB, 0.95171E+00_JPRB, 0.97486E+00_JPRB, 0.11077E+01_JPRB, 0.15182E+01_JPRB, 0.16422E+01_JPRB ,&
&  0.16939E+01_JPRB, 0.17433E+01_JPRB, 0.18375E+01_JPRB, 0.18966E+01_JPRB, 0.18679E+01_JPRB, 0.18511E+01_JPRB, 0.18455E+01_JPRB ,&
&  0.18020E+01_JPRB, 0.17988E+01_JPRB, 0.18194E+01_JPRB, 0.18810E+01_JPRB, 0.18763E+01_JPRB, 0.18471E+01_JPRB, 0.18188E+01_JPRB ,&
&  0.17916E+01_JPRB, 0.17088E+01_JPRB, 0.16130E+01_JPRB, 0.14827E+01_JPRB, 0.14066E+01_JPRB, 0.13530E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.12636E+01_JPRB, 0.12474E+01_JPRB, 0.12203E+01_JPRB, 0.11038E+01_JPRB, 0.96615E+00_JPRB, 0.86051E+00_JPRB ,&
&  0.79721E+00_JPRB, 0.76257E+00_JPRB, 0.75007E+00_JPRB, 0.75209E+00_JPRB, 0.76187E+00_JPRB, 0.77028E+00_JPRB, 0.74473E+00_JPRB ,&
&  0.73725E+00_JPRB, 0.74202E+00_JPRB, 0.75766E+00_JPRB, 0.78481E+00_JPRB, 0.81039E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.74435E+00_JPRB, 0.76982E+00_JPRB, 0.78787E+00_JPRB, 0.78890E+00_JPRB, 0.77752E+00_JPRB, 0.72930E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.65953E+00_JPRB, 0.65181E+00_JPRB, 0.65278E+00_JPRB, 0.65427E+00_JPRB, 0.65720E+00_JPRB, 0.66170E+00_JPRB, 0.66441E+00_JPRB ,&
&  0.66481E+00_JPRB, 0.66736E+00_JPRB, 0.67216E+00_JPRB, 0.67625E+00_JPRB, 0.67344E+00_JPRB, 0.66935E+00_JPRB, 0.66942E+00_JPRB ,&
&  0.66891E+00_JPRB, 0.66624E+00_JPRB, 0.66712E+00_JPRB, 0.66339E+00_JPRB, 0.66311E+00_JPRB, 0.65973E+00_JPRB, 0.65237E+00_JPRB ,&
&  0.63946E+00_JPRB, 0.63548E+00_JPRB, 0.64323E+00_JPRB, 0.64811E+00_JPRB, 0.65578E+00_JPRB, 0.63093E+00_JPRB, 0.58595E+00_JPRB ,&
&  0.55183E+00_JPRB, 0.53510E+00_JPRB, 0.54360E+00_JPRB, 0.57898E+00_JPRB, 0.60874E+00_JPRB, 0.59268E+00_JPRB, 0.60646E+00_JPRB ,&
&  0.60632E+00_JPRB, 0.60790E+00_JPRB, 0.61452E+00_JPRB, 0.63192E+00_JPRB, 0.65112E+00_JPRB, 0.67048E+00_JPRB, 0.68863E+00_JPRB ,&
&  0.71209E+00_JPRB, 0.74512E+00_JPRB, 0.79175E+00_JPRB, 0.82406E+00_JPRB, 0.84722E+00_JPRB, 0.94494E+00_JPRB, 0.10727E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.13123E+01_JPRB, 0.13732E+01_JPRB, 0.14618E+01_JPRB, 0.15232E+01_JPRB, 0.15451E+01_JPRB, 0.15431E+01_JPRB ,&
&  0.15378E+01_JPRB, 0.15089E+01_JPRB, 0.14909E+01_JPRB, 0.14992E+01_JPRB, 0.15321E+01_JPRB, 0.15496E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.14792E+01_JPRB, 0.13898E+01_JPRB, 0.12989E+01_JPRB, 0.11813E+01_JPRB, 0.11578E+01_JPRB, 0.11862E+01_JPRB, 0.12227E+01_JPRB ,&
&  0.12120E+01_JPRB, 0.11896E+01_JPRB, 0.11709E+01_JPRB, 0.11528E+01_JPRB, 0.10283E+01_JPRB, 0.93892E+00_JPRB, 0.87383E+00_JPRB ,&
&  0.84123E+00_JPRB, 0.81442E+00_JPRB, 0.80552E+00_JPRB, 0.80580E+00_JPRB, 0.80158E+00_JPRB, 0.79230E+00_JPRB, 0.76208E+00_JPRB ,&
&  0.73825E+00_JPRB, 0.72434E+00_JPRB, 0.72172E+00_JPRB, 0.72075E+00_JPRB, 0.73054E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.74388E+00_JPRB, 0.74993E+00_JPRB, 0.75641E+00_JPRB, 0.75935E+00_JPRB, 0.75384E+00_JPRB, 0.73300E+00_JPRB, 0.71394E+00_JPRB ,&
&  0.70123E+00_JPRB, 0.69928E+00_JPRB, 0.69782E+00_JPRB, 0.69510E+00_JPRB, 0.69388E+00_JPRB, 0.69288E+00_JPRB, 0.69270E+00_JPRB ,&
&  0.69408E+00_JPRB, 0.69468E+00_JPRB, 0.69194E+00_JPRB, 0.68450E+00_JPRB, 0.67967E+00_JPRB, 0.67924E+00_JPRB, 0.68141E+00_JPRB ,&
&  0.68093E+00_JPRB, 0.67714E+00_JPRB, 0.67691E+00_JPRB, 0.66763E+00_JPRB, 0.65821E+00_JPRB, 0.64351E+00_JPRB, 0.63275E+00_JPRB ,&
&  0.63183E+00_JPRB, 0.63394E+00_JPRB, 0.62372E+00_JPRB, 0.62356E+00_JPRB, 0.62920E+00_JPRB, 0.59297E+00_JPRB, 0.55941E+00_JPRB ,&
&  0.53384E+00_JPRB, 0.51535E+00_JPRB, 0.50652E+00_JPRB, 0.50761E+00_JPRB, 0.53173E+00_JPRB, 0.56618E+00_JPRB, 0.58176E+00_JPRB ,&
&  0.56114E+00_JPRB, 0.57953E+00_JPRB, 0.60300E+00_JPRB, 0.61628E+00_JPRB, 0.62714E+00_JPRB, 0.64145E+00_JPRB, 0.65566E+00_JPRB ,&
&  0.67051E+00_JPRB, 0.68590E+00_JPRB, 0.70389E+00_JPRB, 0.74170E+00_JPRB, 0.76917E+00_JPRB, 0.77023E+00_JPRB, 0.80508E+00_JPRB ,&
&  0.98827E+00_JPRB, 0.11380E+01_JPRB, 0.12383E+01_JPRB, 0.12920E+01_JPRB, 0.13166E+01_JPRB, 0.13019E+01_JPRB, 0.12878E+01_JPRB ,&
&  0.12758E+01_JPRB, 0.12755E+01_JPRB, 0.12916E+01_JPRB, 0.13066E+01_JPRB, 0.13221E+01_JPRB, 0.13234E+01_JPRB, 0.13184E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.13055E+01_JPRB, 0.16242E+01_JPRB, 0.15323E+01_JPRB, 0.14442E+01_JPRB, 0.13956E+01_JPRB, 0.13281E+01_JPRB ,&
&  0.12671E+01_JPRB, 0.12204E+01_JPRB, 0.11937E+01_JPRB, 0.11622E+01_JPRB, 0.10776E+01_JPRB, 0.10129E+01_JPRB, 0.96977E+00_JPRB ,&
&  0.94741E+00_JPRB, 0.92486E+00_JPRB, 0.90320E+00_JPRB, 0.88459E+00_JPRB, 0.86265E+00_JPRB, 0.83548E+00_JPRB, 0.79760E+00_JPRB ,&
&  0.76862E+00_JPRB, 0.75504E+00_JPRB, 0.74731E+00_JPRB, 0.74503E+00_JPRB, 0.74406E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.72223E+00_JPRB, 0.72620E+00_JPRB, 0.73108E+00_JPRB, 0.73425E+00_JPRB, 0.73253E+00_JPRB, 0.72909E+00_JPRB, 0.72272E+00_JPRB ,&
&  0.71658E+00_JPRB, 0.71420E+00_JPRB, 0.71240E+00_JPRB, 0.71174E+00_JPRB, 0.71125E+00_JPRB, 0.71027E+00_JPRB, 0.70926E+00_JPRB ,&
&  0.70961E+00_JPRB, 0.69613E+00_JPRB, 0.69000E+00_JPRB, 0.69154E+00_JPRB, 0.69129E+00_JPRB, 0.69174E+00_JPRB, 0.69253E+00_JPRB ,&
&  0.68645E+00_JPRB, 0.67645E+00_JPRB, 0.66904E+00_JPRB, 0.65952E+00_JPRB, 0.64916E+00_JPRB, 0.64990E+00_JPRB, 0.64769E+00_JPRB ,&
&  0.63502E+00_JPRB, 0.62286E+00_JPRB, 0.62765E+00_JPRB, 0.62807E+00_JPRB, 0.59718E+00_JPRB, 0.56501E+00_JPRB, 0.53089E+00_JPRB ,&
&  0.52282E+00_JPRB, 0.50473E+00_JPRB, 0.50083E+00_JPRB, 0.50107E+00_JPRB, 0.51153E+00_JPRB, 0.52624E+00_JPRB, 0.55200E+00_JPRB ,&
&  0.56423E+00_JPRB, 0.57484E+00_JPRB, 0.59442E+00_JPRB, 0.61230E+00_JPRB, 0.62374E+00_JPRB, 0.63392E+00_JPRB, 0.63700E+00_JPRB ,&
&  0.64607E+00_JPRB, 0.65584E+00_JPRB, 0.66732E+00_JPRB, 0.68111E+00_JPRB, 0.70041E+00_JPRB, 0.71564E+00_JPRB, 0.73602E+00_JPRB ,&
&  0.80797E+00_JPRB, 0.86929E+00_JPRB, 0.92134E+00_JPRB, 0.96854E+00_JPRB, 0.10046E+01_JPRB, 0.10240E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.10101E+01_JPRB, 0.10452E+01_JPRB, 0.10802E+01_JPRB, 0.11144E+01_JPRB, 0.11466E+01_JPRB, 0.11752E+01_JPRB, 0.11879E+01_JPRB ,&
&  0.11703E+01_JPRB, 0.11747E+01_JPRB, 0.12930E+01_JPRB, 0.13195E+01_JPRB, 0.13051E+01_JPRB, 0.12664E+01_JPRB, 0.12114E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11331E+01_JPRB, 0.11114E+01_JPRB, 0.10905E+01_JPRB, 0.10710E+01_JPRB, 0.10513E+01_JPRB, 0.10421E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.99845E+00_JPRB, 0.96838E+00_JPRB, 0.93835E+00_JPRB, 0.90947E+00_JPRB, 0.87896E+00_JPRB, 0.83145E+00_JPRB ,&
&  0.78196E+00_JPRB, 0.74664E+00_JPRB, 0.72848E+00_JPRB, 0.72025E+00_JPRB, 0.72015E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.71206E+00_JPRB, 0.71052E+00_JPRB, 0.71275E+00_JPRB, 0.71179E+00_JPRB, 0.71203E+00_JPRB, 0.70909E+00_JPRB, 0.70755E+00_JPRB ,&
&  0.70491E+00_JPRB, 0.70405E+00_JPRB, 0.70340E+00_JPRB, 0.70391E+00_JPRB, 0.70398E+00_JPRB, 0.70441E+00_JPRB, 0.70425E+00_JPRB ,&
&  0.70354E+00_JPRB, 0.70248E+00_JPRB, 0.69991E+00_JPRB, 0.69721E+00_JPRB, 0.69516E+00_JPRB, 0.69579E+00_JPRB, 0.69368E+00_JPRB ,&
&  0.68801E+00_JPRB, 0.67663E+00_JPRB, 0.65623E+00_JPRB, 0.64760E+00_JPRB, 0.64666E+00_JPRB, 0.63608E+00_JPRB, 0.63659E+00_JPRB ,&
&  0.63121E+00_JPRB, 0.62934E+00_JPRB, 0.62165E+00_JPRB, 0.61893E+00_JPRB, 0.59933E+00_JPRB, 0.52295E+00_JPRB, 0.51521E+00_JPRB ,&
&  0.50837E+00_JPRB, 0.49855E+00_JPRB, 0.49673E+00_JPRB, 0.49837E+00_JPRB, 0.50561E+00_JPRB, 0.52805E+00_JPRB, 0.55004E+00_JPRB ,&
&  0.55421E+00_JPRB, 0.58693E+00_JPRB, 0.59862E+00_JPRB, 0.60866E+00_JPRB, 0.62626E+00_JPRB, 0.63997E+00_JPRB, 0.64059E+00_JPRB ,&
&  0.64367E+00_JPRB, 0.64722E+00_JPRB, 0.65852E+00_JPRB, 0.67073E+00_JPRB, 0.68168E+00_JPRB, 0.69273E+00_JPRB, 0.70739E+00_JPRB ,&
&  0.72552E+00_JPRB, 0.74352E+00_JPRB, 0.76474E+00_JPRB, 0.78137E+00_JPRB, 0.79837E+00_JPRB, 0.81895E+00_JPRB, 0.84121E+00_JPRB ,&
&  0.86895E+00_JPRB, 0.89723E+00_JPRB, 0.93473E+00_JPRB, 0.95937E+00_JPRB, 0.98767E+00_JPRB, 0.10135E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10713E+01_JPRB, 0.11087E+01_JPRB, 0.11192E+01_JPRB, 0.11272E+01_JPRB, 0.11088E+01_JPRB, 0.10896E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.10794E+01_JPRB, 0.10657E+01_JPRB, 0.10547E+01_JPRB, 0.10468E+01_JPRB, 0.10415E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10220E+01_JPRB, 0.10028E+01_JPRB, 0.97812E+00_JPRB, 0.94726E+00_JPRB, 0.91937E+00_JPRB, 0.89140E+00_JPRB, 0.84741E+00_JPRB ,&
&  0.80148E+00_JPRB, 0.75707E+00_JPRB, 0.73115E+00_JPRB, 0.71563E+00_JPRB, 0.71168E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.71566E+00_JPRB, 0.71051E+00_JPRB, 0.71016E+00_JPRB, 0.70683E+00_JPRB, 0.70668E+00_JPRB, 0.70443E+00_JPRB, 0.70343E+00_JPRB ,&
&  0.70106E+00_JPRB, 0.69951E+00_JPRB, 0.69734E+00_JPRB, 0.69678E+00_JPRB, 0.69605E+00_JPRB, 0.69656E+00_JPRB, 0.69667E+00_JPRB ,&
&  0.69728E+00_JPRB, 0.69708E+00_JPRB, 0.69729E+00_JPRB, 0.69625E+00_JPRB, 0.69586E+00_JPRB, 0.69450E+00_JPRB, 0.68972E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.66794E+00_JPRB, 0.63959E+00_JPRB, 0.63407E+00_JPRB, 0.62734E+00_JPRB, 0.62634E+00_JPRB, 0.62259E+00_JPRB ,&
&  0.61607E+00_JPRB, 0.56438E+00_JPRB, 0.56893E+00_JPRB, 0.55639E+00_JPRB, 0.54728E+00_JPRB, 0.53811E+00_JPRB, 0.52417E+00_JPRB ,&
&  0.51897E+00_JPRB, 0.51348E+00_JPRB, 0.51255E+00_JPRB, 0.51823E+00_JPRB, 0.52531E+00_JPRB, 0.53787E+00_JPRB, 0.55338E+00_JPRB ,&
&  0.55800E+00_JPRB, 0.58604E+00_JPRB, 0.59914E+00_JPRB, 0.60684E+00_JPRB, 0.62665E+00_JPRB, 0.64181E+00_JPRB, 0.64410E+00_JPRB ,&
&  0.64876E+00_JPRB, 0.64870E+00_JPRB, 0.65614E+00_JPRB, 0.66668E+00_JPRB, 0.67873E+00_JPRB, 0.68434E+00_JPRB, 0.69090E+00_JPRB ,&
&  0.69729E+00_JPRB, 0.70379E+00_JPRB, 0.71096E+00_JPRB, 0.71607E+00_JPRB, 0.72443E+00_JPRB, 0.73902E+00_JPRB, 0.75730E+00_JPRB ,&
&  0.77943E+00_JPRB, 0.80381E+00_JPRB, 0.83018E+00_JPRB, 0.85761E+00_JPRB, 0.88102E+00_JPRB, 0.90347E+00_JPRB, 0.92276E+00_JPRB ,&
&  0.94266E+00_JPRB, 0.95831E+00_JPRB, 0.96776E+00_JPRB, 0.97555E+00_JPRB, 0.97580E+00_JPRB, 0.97154E+00_JPRB, 0.97208E+00_JPRB ,&
&  0.98100E+00_JPRB, 0.98475E+00_JPRB, 0.98631E+00_JPRB, 0.98541E+00_JPRB, 0.98565E+00_JPRB, 0.98302E+00_JPRB, 0.97778E+00_JPRB ,&
&  0.96943E+00_JPRB, 0.95715E+00_JPRB, 0.94107E+00_JPRB, 0.91725E+00_JPRB, 0.89191E+00_JPRB, 0.87139E+00_JPRB, 0.84672E+00_JPRB ,&
&  0.81931E+00_JPRB, 0.78673E+00_JPRB, 0.75667E+00_JPRB, 0.73316E+00_JPRB, 0.72203E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.72998E+00_JPRB, 0.72582E+00_JPRB, 0.72457E+00_JPRB, 0.72238E+00_JPRB, 0.72199E+00_JPRB, 0.71981E+00_JPRB, 0.71895E+00_JPRB ,&
&  0.71619E+00_JPRB, 0.71435E+00_JPRB, 0.71222E+00_JPRB, 0.71053E+00_JPRB, 0.70876E+00_JPRB, 0.70688E+00_JPRB, 0.70528E+00_JPRB ,&
&  0.70445E+00_JPRB, 0.70301E+00_JPRB, 0.70298E+00_JPRB, 0.70189E+00_JPRB, 0.69985E+00_JPRB, 0.69730E+00_JPRB, 0.68816E+00_JPRB ,&
&  0.67809E+00_JPRB, 0.66808E+00_JPRB, 0.62708E+00_JPRB, 0.62219E+00_JPRB, 0.61337E+00_JPRB, 0.60814E+00_JPRB, 0.60870E+00_JPRB ,&
&  0.60310E+00_JPRB, 0.59734E+00_JPRB, 0.58054E+00_JPRB, 0.56376E+00_JPRB, 0.55048E+00_JPRB, 0.53418E+00_JPRB, 0.53035E+00_JPRB ,&
&  0.52631E+00_JPRB, 0.52962E+00_JPRB, 0.53410E+00_JPRB, 0.53913E+00_JPRB, 0.55095E+00_JPRB, 0.56016E+00_JPRB, 0.57367E+00_JPRB ,&
&  0.59007E+00_JPRB, 0.60294E+00_JPRB, 0.60823E+00_JPRB, 0.60698E+00_JPRB, 0.62584E+00_JPRB, 0.63837E+00_JPRB, 0.64871E+00_JPRB ,&
&  0.65448E+00_JPRB, 0.65573E+00_JPRB, 0.66054E+00_JPRB, 0.66290E+00_JPRB, 0.66778E+00_JPRB, 0.67540E+00_JPRB, 0.68121E+00_JPRB ,&
&  0.68722E+00_JPRB, 0.69070E+00_JPRB, 0.69312E+00_JPRB, 0.69413E+00_JPRB, 0.69298E+00_JPRB, 0.69363E+00_JPRB, 0.69780E+00_JPRB ,&
&  0.70567E+00_JPRB, 0.71795E+00_JPRB, 0.73332E+00_JPRB, 0.75108E+00_JPRB, 0.76887E+00_JPRB, 0.78576E+00_JPRB, 0.79964E+00_JPRB ,&
&  0.81264E+00_JPRB, 0.82276E+00_JPRB, 0.83584E+00_JPRB, 0.83865E+00_JPRB, 0.84781E+00_JPRB, 0.86722E+00_JPRB, 0.87210E+00_JPRB ,&
&  0.87871E+00_JPRB, 0.88710E+00_JPRB, 0.89395E+00_JPRB, 0.89918E+00_JPRB, 0.90233E+00_JPRB, 0.90269E+00_JPRB, 0.90181E+00_JPRB ,&
&  0.89910E+00_JPRB, 0.89156E+00_JPRB, 0.88095E+00_JPRB, 0.86752E+00_JPRB, 0.85485E+00_JPRB, 0.84174E+00_JPRB, 0.82711E+00_JPRB ,&
&  0.81099E+00_JPRB, 0.79314E+00_JPRB, 0.77189E+00_JPRB, 0.75102E+00_JPRB, 0.73823E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.74714E+00_JPRB, 0.74116E+00_JPRB, 0.73669E+00_JPRB, 0.73364E+00_JPRB, 0.73128E+00_JPRB, 0.72901E+00_JPRB, 0.72738E+00_JPRB ,&
&  0.72524E+00_JPRB, 0.72276E+00_JPRB, 0.71962E+00_JPRB, 0.71786E+00_JPRB, 0.71584E+00_JPRB, 0.71435E+00_JPRB, 0.71160E+00_JPRB ,&
&  0.70946E+00_JPRB, 0.70627E+00_JPRB, 0.70473E+00_JPRB, 0.70296E+00_JPRB, 0.70081E+00_JPRB, 0.69887E+00_JPRB, 0.69077E+00_JPRB ,&
&  0.68312E+00_JPRB, 0.67648E+00_JPRB, 0.66765E+00_JPRB, 0.65987E+00_JPRB, 0.65424E+00_JPRB, 0.61892E+00_JPRB, 0.60760E+00_JPRB ,&
&  0.61507E+00_JPRB, 0.62069E+00_JPRB, 0.62714E+00_JPRB, 0.63226E+00_JPRB, 0.62576E+00_JPRB, 0.61086E+00_JPRB, 0.60154E+00_JPRB ,&
&  0.57200E+00_JPRB, 0.57671E+00_JPRB, 0.58054E+00_JPRB, 0.58429E+00_JPRB, 0.59133E+00_JPRB, 0.59521E+00_JPRB, 0.60365E+00_JPRB ,&
&  0.62457E+00_JPRB, 0.62921E+00_JPRB, 0.63536E+00_JPRB, 0.64043E+00_JPRB, 0.64228E+00_JPRB, 0.64483E+00_JPRB, 0.65059E+00_JPRB ,&
&  0.65315E+00_JPRB, 0.65565E+00_JPRB, 0.65773E+00_JPRB, 0.65833E+00_JPRB, 0.66099E+00_JPRB, 0.66317E+00_JPRB, 0.66491E+00_JPRB ,&
&  0.66486E+00_JPRB, 0.66572E+00_JPRB, 0.66672E+00_JPRB, 0.66773E+00_JPRB, 0.66920E+00_JPRB, 0.66908E+00_JPRB, 0.67009E+00_JPRB ,&
&  0.67199E+00_JPRB, 0.67600E+00_JPRB, 0.68098E+00_JPRB, 0.68773E+00_JPRB, 0.69641E+00_JPRB, 0.70835E+00_JPRB, 0.72077E+00_JPRB ,&
&  0.73507E+00_JPRB, 0.74867E+00_JPRB, 0.76120E+00_JPRB, 0.77161E+00_JPRB, 0.78067E+00_JPRB, 0.78941E+00_JPRB, 0.79664E+00_JPRB ,&
&  0.80452E+00_JPRB, 0.81258E+00_JPRB, 0.81759E+00_JPRB, 0.81950E+00_JPRB, 0.82412E+00_JPRB, 0.82819E+00_JPRB, 0.82991E+00_JPRB ,&
&  0.83018E+00_JPRB, 0.82946E+00_JPRB, 0.82678E+00_JPRB, 0.82214E+00_JPRB, 0.81765E+00_JPRB, 0.81215E+00_JPRB, 0.80536E+00_JPRB ,&
&  0.79795E+00_JPRB, 0.79038E+00_JPRB, 0.78045E+00_JPRB, 0.76865E+00_JPRB, 0.75724E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.75650E+00_JPRB, 0.75101E+00_JPRB, 0.74572E+00_JPRB, 0.74105E+00_JPRB, 0.73632E+00_JPRB, 0.73282E+00_JPRB, 0.72961E+00_JPRB ,&
&  0.72667E+00_JPRB, 0.72358E+00_JPRB, 0.72066E+00_JPRB, 0.71822E+00_JPRB, 0.71639E+00_JPRB, 0.71488E+00_JPRB, 0.71268E+00_JPRB ,&
&  0.71062E+00_JPRB, 0.70809E+00_JPRB, 0.70593E+00_JPRB, 0.70389E+00_JPRB, 0.70125E+00_JPRB, 0.69814E+00_JPRB, 0.69423E+00_JPRB ,&
&  0.69117E+00_JPRB, 0.68780E+00_JPRB, 0.68425E+00_JPRB, 0.67938E+00_JPRB, 0.67532E+00_JPRB, 0.67124E+00_JPRB, 0.66716E+00_JPRB ,&
&  0.66267E+00_JPRB, 0.65912E+00_JPRB, 0.65600E+00_JPRB, 0.65305E+00_JPRB, 0.65504E+00_JPRB, 0.65617E+00_JPRB, 0.65666E+00_JPRB ,&
&  0.65615E+00_JPRB, 0.65339E+00_JPRB, 0.65234E+00_JPRB, 0.65154E+00_JPRB, 0.65083E+00_JPRB, 0.65355E+00_JPRB, 0.65535E+00_JPRB ,&
&  0.65647E+00_JPRB, 0.65666E+00_JPRB, 0.65828E+00_JPRB, 0.65907E+00_JPRB, 0.65890E+00_JPRB, 0.65766E+00_JPRB, 0.65581E+00_JPRB ,&
&  0.65566E+00_JPRB, 0.65549E+00_JPRB, 0.65505E+00_JPRB, 0.65399E+00_JPRB, 0.65444E+00_JPRB, 0.65514E+00_JPRB, 0.65536E+00_JPRB ,&
&  0.65483E+00_JPRB, 0.65504E+00_JPRB, 0.65518E+00_JPRB, 0.65541E+00_JPRB, 0.65565E+00_JPRB, 0.65685E+00_JPRB, 0.65835E+00_JPRB ,&
&  0.66024E+00_JPRB, 0.66280E+00_JPRB, 0.66608E+00_JPRB, 0.66980E+00_JPRB, 0.67414E+00_JPRB, 0.67988E+00_JPRB, 0.68707E+00_JPRB ,&
&  0.69571E+00_JPRB, 0.70546E+00_JPRB, 0.71577E+00_JPRB, 0.72409E+00_JPRB, 0.73089E+00_JPRB, 0.73686E+00_JPRB, 0.74321E+00_JPRB ,&
&  0.74888E+00_JPRB, 0.75392E+00_JPRB, 0.75902E+00_JPRB, 0.76415E+00_JPRB, 0.76936E+00_JPRB, 0.77347E+00_JPRB, 0.77652E+00_JPRB ,&
&  0.77874E+00_JPRB, 0.78041E+00_JPRB, 0.78194E+00_JPRB, 0.78360E+00_JPRB, 0.78465E+00_JPRB, 0.78398E+00_JPRB, 0.78229E+00_JPRB ,&
&  0.77998E+00_JPRB, 0.77719E+00_JPRB, 0.77347E+00_JPRB, 0.76826E+00_JPRB, 0.76263E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.74058E+00_JPRB, 0.73868E+00_JPRB, 0.73667E+00_JPRB, 0.73465E+00_JPRB, 0.73244E+00_JPRB, 0.73031E+00_JPRB, 0.72812E+00_JPRB ,&
&  0.72619E+00_JPRB, 0.72412E+00_JPRB, 0.72229E+00_JPRB, 0.72048E+00_JPRB, 0.71894E+00_JPRB, 0.71732E+00_JPRB, 0.71584E+00_JPRB ,&
&  0.71427E+00_JPRB, 0.71282E+00_JPRB, 0.71085E+00_JPRB, 0.70918E+00_JPRB, 0.70727E+00_JPRB, 0.70549E+00_JPRB, 0.70324E+00_JPRB ,&
&  0.70152E+00_JPRB, 0.69946E+00_JPRB, 0.69755E+00_JPRB, 0.69501E+00_JPRB, 0.69305E+00_JPRB, 0.69102E+00_JPRB, 0.68922E+00_JPRB ,&
&  0.68765E+00_JPRB, 0.68651E+00_JPRB, 0.68540E+00_JPRB, 0.68419E+00_JPRB, 0.68365E+00_JPRB, 0.68375E+00_JPRB, 0.68392E+00_JPRB ,&
&  0.68400E+00_JPRB, 0.68431E+00_JPRB, 0.68486E+00_JPRB, 0.68510E+00_JPRB, 0.68463E+00_JPRB, 0.68417E+00_JPRB, 0.68382E+00_JPRB ,&
&  0.68321E+00_JPRB, 0.68216E+00_JPRB, 0.68098E+00_JPRB, 0.67998E+00_JPRB, 0.67883E+00_JPRB, 0.67736E+00_JPRB, 0.67607E+00_JPRB ,&
&  0.67568E+00_JPRB, 0.67533E+00_JPRB, 0.67483E+00_JPRB, 0.67429E+00_JPRB, 0.67429E+00_JPRB, 0.67434E+00_JPRB, 0.67437E+00_JPRB ,&
&  0.67451E+00_JPRB, 0.67499E+00_JPRB, 0.67521E+00_JPRB, 0.67537E+00_JPRB, 0.67571E+00_JPRB, 0.67672E+00_JPRB, 0.67796E+00_JPRB ,&
&  0.67946E+00_JPRB, 0.68142E+00_JPRB, 0.68390E+00_JPRB, 0.68638E+00_JPRB, 0.68900E+00_JPRB, 0.69194E+00_JPRB, 0.69509E+00_JPRB ,&
&  0.69836E+00_JPRB, 0.70176E+00_JPRB, 0.70553E+00_JPRB, 0.70930E+00_JPRB, 0.71296E+00_JPRB, 0.71659E+00_JPRB, 0.72056E+00_JPRB ,&
&  0.72392E+00_JPRB, 0.72688E+00_JPRB, 0.72949E+00_JPRB, 0.73214E+00_JPRB, 0.73469E+00_JPRB, 0.73705E+00_JPRB, 0.73905E+00_JPRB ,&
&  0.74073E+00_JPRB, 0.74216E+00_JPRB, 0.74341E+00_JPRB, 0.74473E+00_JPRB, 0.74513E+00_JPRB, 0.74524E+00_JPRB, 0.74526E+00_JPRB ,&
&  0.74512E+00_JPRB, 0.74461E+00_JPRB, 0.74391E+00_JPRB, 0.74311E+00_JPRB, 0.74211E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB ,&
&  0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB, 0.70793E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB ,&
&  0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB, 0.17207E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.18616E+00_JPRB, 0.18637E+00_JPRB, 0.18686E+00_JPRB, 0.18743E+00_JPRB, 0.18801E+00_JPRB, 0.18879E+00_JPRB, 0.18905E+00_JPRB ,&
&  0.18912E+00_JPRB, 0.18906E+00_JPRB, 0.18939E+00_JPRB, 0.18977E+00_JPRB, 0.19001E+00_JPRB, 0.19001E+00_JPRB, 0.18962E+00_JPRB ,&
&  0.18895E+00_JPRB, 0.18817E+00_JPRB, 0.18701E+00_JPRB, 0.18649E+00_JPRB, 0.18610E+00_JPRB, 0.18570E+00_JPRB, 0.18492E+00_JPRB ,&
&  0.18423E+00_JPRB, 0.18302E+00_JPRB, 0.18135E+00_JPRB, 0.18001E+00_JPRB, 0.17972E+00_JPRB, 0.17990E+00_JPRB, 0.18043E+00_JPRB ,&
&  0.18084E+00_JPRB, 0.18110E+00_JPRB, 0.18116E+00_JPRB, 0.18129E+00_JPRB, 0.18150E+00_JPRB, 0.18135E+00_JPRB, 0.18108E+00_JPRB ,&
&  0.18076E+00_JPRB, 0.18024E+00_JPRB, 0.17974E+00_JPRB, 0.17928E+00_JPRB, 0.17886E+00_JPRB, 0.17838E+00_JPRB, 0.17781E+00_JPRB ,&
&  0.17715E+00_JPRB, 0.17634E+00_JPRB, 0.17539E+00_JPRB, 0.17462E+00_JPRB, 0.17383E+00_JPRB, 0.17320E+00_JPRB, 0.17243E+00_JPRB ,&
&  0.17179E+00_JPRB, 0.17120E+00_JPRB, 0.17060E+00_JPRB, 0.17004E+00_JPRB, 0.16951E+00_JPRB, 0.16900E+00_JPRB, 0.16850E+00_JPRB ,&
&  0.16801E+00_JPRB, 0.16762E+00_JPRB, 0.16717E+00_JPRB, 0.16675E+00_JPRB, 0.16647E+00_JPRB, 0.16624E+00_JPRB, 0.16603E+00_JPRB ,&
&  0.16586E+00_JPRB, 0.16580E+00_JPRB, 0.16578E+00_JPRB, 0.16575E+00_JPRB, 0.16574E+00_JPRB, 0.16585E+00_JPRB, 0.16596E+00_JPRB ,&
&  0.16612E+00_JPRB, 0.16623E+00_JPRB, 0.16636E+00_JPRB, 0.16654E+00_JPRB, 0.16675E+00_JPRB, 0.16698E+00_JPRB, 0.16730E+00_JPRB ,&
&  0.16769E+00_JPRB, 0.16816E+00_JPRB, 0.16865E+00_JPRB, 0.16898E+00_JPRB, 0.16963E+00_JPRB, 0.17037E+00_JPRB, 0.17123E+00_JPRB ,&
&  0.17218E+00_JPRB, 0.17329E+00_JPRB, 0.17448E+00_JPRB, 0.17584E+00_JPRB, 0.17723E+00_JPRB, 0.17813E+00_JPRB, 0.17904E+00_JPRB ,&
&  0.18000E+00_JPRB, 0.18112E+00_JPRB, 0.18218E+00_JPRB, 0.18328E+00_JPRB, 0.18461E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.20769E+00_JPRB, 0.21454E+00_JPRB, 0.22145E+00_JPRB, 0.23127E+00_JPRB, 0.23843E+00_JPRB, 0.27381E+00_JPRB, 0.26870E+00_JPRB ,&
&  0.26522E+00_JPRB, 0.25133E+00_JPRB, 0.24972E+00_JPRB, 0.24606E+00_JPRB, 0.24152E+00_JPRB, 0.23659E+00_JPRB, 0.23163E+00_JPRB ,&
&  0.22698E+00_JPRB, 0.22153E+00_JPRB, 0.21682E+00_JPRB, 0.21312E+00_JPRB, 0.20981E+00_JPRB, 0.20658E+00_JPRB, 0.20374E+00_JPRB ,&
&  0.20249E+00_JPRB, 0.20231E+00_JPRB, 0.20150E+00_JPRB, 0.19754E+00_JPRB, 0.19415E+00_JPRB, 0.19206E+00_JPRB, 0.19138E+00_JPRB ,&
&  0.19128E+00_JPRB, 0.19284E+00_JPRB, 0.19375E+00_JPRB, 0.19447E+00_JPRB, 0.19558E+00_JPRB, 0.19475E+00_JPRB, 0.19392E+00_JPRB ,&
&  0.19276E+00_JPRB, 0.19144E+00_JPRB, 0.19006E+00_JPRB, 0.18887E+00_JPRB, 0.18768E+00_JPRB, 0.18591E+00_JPRB, 0.18437E+00_JPRB ,&
&  0.18283E+00_JPRB, 0.18089E+00_JPRB, 0.17837E+00_JPRB, 0.17668E+00_JPRB, 0.17474E+00_JPRB, 0.17308E+00_JPRB, 0.17143E+00_JPRB ,&
&  0.17020E+00_JPRB, 0.16908E+00_JPRB, 0.16774E+00_JPRB, 0.16611E+00_JPRB, 0.16450E+00_JPRB, 0.16306E+00_JPRB, 0.16170E+00_JPRB ,&
&  0.16094E+00_JPRB, 0.16017E+00_JPRB, 0.15940E+00_JPRB, 0.15858E+00_JPRB, 0.15817E+00_JPRB, 0.15776E+00_JPRB, 0.15738E+00_JPRB ,&
&  0.15707E+00_JPRB, 0.15691E+00_JPRB, 0.15694E+00_JPRB, 0.15704E+00_JPRB, 0.15713E+00_JPRB, 0.15749E+00_JPRB, 0.15793E+00_JPRB ,&
&  0.15851E+00_JPRB, 0.15924E+00_JPRB, 0.16022E+00_JPRB, 0.16124E+00_JPRB, 0.16216E+00_JPRB, 0.16291E+00_JPRB, 0.16389E+00_JPRB ,&
&  0.16546E+00_JPRB, 0.16727E+00_JPRB, 0.16925E+00_JPRB, 0.17079E+00_JPRB, 0.17293E+00_JPRB, 0.17486E+00_JPRB, 0.17678E+00_JPRB ,&
&  0.17868E+00_JPRB, 0.18091E+00_JPRB, 0.18337E+00_JPRB, 0.18600E+00_JPRB, 0.18901E+00_JPRB, 0.19173E+00_JPRB, 0.19424E+00_JPRB ,&
&  0.19681E+00_JPRB, 0.20038E+00_JPRB, 0.20244E+00_JPRB, 0.20371E+00_JPRB, 0.20536E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.28096E+00_JPRB, 0.28578E+00_JPRB, 0.28948E+00_JPRB, 0.29184E+00_JPRB, 0.29405E+00_JPRB, 0.29564E+00_JPRB, 0.29620E+00_JPRB ,&
&  0.28976E+00_JPRB, 0.28475E+00_JPRB, 0.27781E+00_JPRB, 0.26950E+00_JPRB, 0.26192E+00_JPRB, 0.25512E+00_JPRB, 0.24720E+00_JPRB ,&
&  0.24072E+00_JPRB, 0.23460E+00_JPRB, 0.22815E+00_JPRB, 0.22141E+00_JPRB, 0.21435E+00_JPRB, 0.20723E+00_JPRB, 0.20365E+00_JPRB ,&
&  0.20254E+00_JPRB, 0.20351E+00_JPRB, 0.20533E+00_JPRB, 0.20687E+00_JPRB, 0.20782E+00_JPRB, 0.20846E+00_JPRB, 0.20733E+00_JPRB ,&
&  0.20369E+00_JPRB, 0.20427E+00_JPRB, 0.20509E+00_JPRB, 0.20560E+00_JPRB, 0.20704E+00_JPRB, 0.20832E+00_JPRB, 0.20811E+00_JPRB ,&
&  0.20635E+00_JPRB, 0.20397E+00_JPRB, 0.20071E+00_JPRB, 0.19767E+00_JPRB, 0.19646E+00_JPRB, 0.19425E+00_JPRB, 0.19176E+00_JPRB ,&
&  0.18968E+00_JPRB, 0.18582E+00_JPRB, 0.18196E+00_JPRB, 0.17924E+00_JPRB, 0.17575E+00_JPRB, 0.17335E+00_JPRB, 0.17022E+00_JPRB ,&
&  0.16768E+00_JPRB, 0.16537E+00_JPRB, 0.16282E+00_JPRB, 0.16083E+00_JPRB, 0.15896E+00_JPRB, 0.15757E+00_JPRB, 0.15619E+00_JPRB ,&
&  0.15539E+00_JPRB, 0.15463E+00_JPRB, 0.15416E+00_JPRB, 0.15364E+00_JPRB, 0.15334E+00_JPRB, 0.15304E+00_JPRB, 0.15280E+00_JPRB ,&
&  0.15268E+00_JPRB, 0.15300E+00_JPRB, 0.15323E+00_JPRB, 0.15364E+00_JPRB, 0.15412E+00_JPRB, 0.15442E+00_JPRB, 0.15496E+00_JPRB ,&
&  0.15598E+00_JPRB, 0.15707E+00_JPRB, 0.15781E+00_JPRB, 0.15876E+00_JPRB, 0.15983E+00_JPRB, 0.16113E+00_JPRB, 0.16289E+00_JPRB ,&
&  0.16474E+00_JPRB, 0.16680E+00_JPRB, 0.16911E+00_JPRB, 0.17140E+00_JPRB, 0.17433E+00_JPRB, 0.17733E+00_JPRB, 0.18048E+00_JPRB ,&
&  0.18422E+00_JPRB, 0.18822E+00_JPRB, 0.19230E+00_JPRB, 0.19681E+00_JPRB, 0.20331E+00_JPRB, 0.20937E+00_JPRB, 0.21688E+00_JPRB ,&
&  0.22425E+00_JPRB, 0.23028E+00_JPRB, 0.26214E+00_JPRB, 0.26694E+00_JPRB, 0.27364E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.28841E+00_JPRB, 0.29921E+00_JPRB, 0.30796E+00_JPRB, 0.31343E+00_JPRB, 0.31907E+00_JPRB, 0.32128E+00_JPRB, 0.32964E+00_JPRB ,&
&  0.32415E+00_JPRB, 0.30602E+00_JPRB, 0.29484E+00_JPRB, 0.27843E+00_JPRB, 0.25814E+00_JPRB, 0.24162E+00_JPRB, 0.23335E+00_JPRB ,&
&  0.22848E+00_JPRB, 0.22464E+00_JPRB, 0.21999E+00_JPRB, 0.21487E+00_JPRB, 0.20970E+00_JPRB, 0.20588E+00_JPRB, 0.20339E+00_JPRB ,&
&  0.20114E+00_JPRB, 0.20168E+00_JPRB, 0.20413E+00_JPRB, 0.20836E+00_JPRB, 0.21434E+00_JPRB, 0.21862E+00_JPRB, 0.22194E+00_JPRB ,&
&  0.22432E+00_JPRB, 0.21569E+00_JPRB, 0.21706E+00_JPRB, 0.21879E+00_JPRB, 0.21740E+00_JPRB, 0.22009E+00_JPRB, 0.22438E+00_JPRB ,&
&  0.22229E+00_JPRB, 0.21604E+00_JPRB, 0.21881E+00_JPRB, 0.21428E+00_JPRB, 0.20598E+00_JPRB, 0.20396E+00_JPRB, 0.19818E+00_JPRB ,&
&  0.19500E+00_JPRB, 0.18785E+00_JPRB, 0.18279E+00_JPRB, 0.17945E+00_JPRB, 0.17422E+00_JPRB, 0.17204E+00_JPRB, 0.16873E+00_JPRB ,&
&  0.16529E+00_JPRB, 0.16285E+00_JPRB, 0.15995E+00_JPRB, 0.15847E+00_JPRB, 0.15722E+00_JPRB, 0.15601E+00_JPRB, 0.15505E+00_JPRB ,&
&  0.15451E+00_JPRB, 0.15401E+00_JPRB, 0.15405E+00_JPRB, 0.15408E+00_JPRB, 0.15412E+00_JPRB, 0.15446E+00_JPRB, 0.15469E+00_JPRB ,&
&  0.15532E+00_JPRB, 0.15605E+00_JPRB, 0.15664E+00_JPRB, 0.15757E+00_JPRB, 0.15800E+00_JPRB, 0.15855E+00_JPRB, 0.15898E+00_JPRB ,&
&  0.15944E+00_JPRB, 0.16012E+00_JPRB, 0.16049E+00_JPRB, 0.16119E+00_JPRB, 0.16198E+00_JPRB, 0.16347E+00_JPRB, 0.16495E+00_JPRB ,&
&  0.16674E+00_JPRB, 0.16881E+00_JPRB, 0.17102E+00_JPRB, 0.17335E+00_JPRB, 0.17623E+00_JPRB, 0.17889E+00_JPRB, 0.18237E+00_JPRB ,&
&  0.18613E+00_JPRB, 0.18970E+00_JPRB, 0.19382E+00_JPRB, 0.19907E+00_JPRB, 0.20600E+00_JPRB, 0.21554E+00_JPRB, 0.22454E+00_JPRB ,&
&  0.23486E+00_JPRB, 0.26125E+00_JPRB, 0.26762E+00_JPRB, 0.27144E+00_JPRB, 0.27989E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.31508E+00_JPRB, 0.32455E+00_JPRB, 0.33859E+00_JPRB, 0.34764E+00_JPRB, 0.35488E+00_JPRB, 0.35074E+00_JPRB, 0.34681E+00_JPRB ,&
&  0.34273E+00_JPRB, 0.32858E+00_JPRB, 0.31143E+00_JPRB, 0.28222E+00_JPRB, 0.25761E+00_JPRB, 0.23878E+00_JPRB, 0.22844E+00_JPRB ,&
&  0.22233E+00_JPRB, 0.22033E+00_JPRB, 0.21848E+00_JPRB, 0.21806E+00_JPRB, 0.21696E+00_JPRB, 0.21862E+00_JPRB, 0.22004E+00_JPRB ,&
&  0.21968E+00_JPRB, 0.21861E+00_JPRB, 0.21797E+00_JPRB, 0.22106E+00_JPRB, 0.22704E+00_JPRB, 0.23174E+00_JPRB, 0.23626E+00_JPRB ,&
&  0.23958E+00_JPRB, 0.23705E+00_JPRB, 0.23817E+00_JPRB, 0.23616E+00_JPRB, 0.23393E+00_JPRB, 0.23400E+00_JPRB, 0.23481E+00_JPRB ,&
&  0.23524E+00_JPRB, 0.23582E+00_JPRB, 0.23515E+00_JPRB, 0.23225E+00_JPRB, 0.22071E+00_JPRB, 0.21473E+00_JPRB, 0.20708E+00_JPRB ,&
&  0.19905E+00_JPRB, 0.18971E+00_JPRB, 0.18224E+00_JPRB, 0.17863E+00_JPRB, 0.17442E+00_JPRB, 0.17273E+00_JPRB, 0.16961E+00_JPRB ,&
&  0.16605E+00_JPRB, 0.16265E+00_JPRB, 0.16032E+00_JPRB, 0.15913E+00_JPRB, 0.15867E+00_JPRB, 0.15765E+00_JPRB, 0.15726E+00_JPRB ,&
&  0.15749E+00_JPRB, 0.15777E+00_JPRB, 0.15856E+00_JPRB, 0.15936E+00_JPRB, 0.16027E+00_JPRB, 0.16110E+00_JPRB, 0.16212E+00_JPRB ,&
&  0.16373E+00_JPRB, 0.16565E+00_JPRB, 0.16654E+00_JPRB, 0.16730E+00_JPRB, 0.16708E+00_JPRB, 0.16665E+00_JPRB, 0.16599E+00_JPRB ,&
&  0.16545E+00_JPRB, 0.16531E+00_JPRB, 0.16532E+00_JPRB, 0.16528E+00_JPRB, 0.16576E+00_JPRB, 0.16665E+00_JPRB, 0.16761E+00_JPRB ,&
&  0.16916E+00_JPRB, 0.17057E+00_JPRB, 0.17216E+00_JPRB, 0.17401E+00_JPRB, 0.17615E+00_JPRB, 0.17866E+00_JPRB, 0.18148E+00_JPRB ,&
&  0.18405E+00_JPRB, 0.18728E+00_JPRB, 0.19000E+00_JPRB, 0.19216E+00_JPRB, 0.19709E+00_JPRB, 0.20409E+00_JPRB, 0.21806E+00_JPRB ,&
&  0.23436E+00_JPRB, 0.27459E+00_JPRB, 0.31705E+00_JPRB, 0.30219E+00_JPRB, 0.30707E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.31438E+00_JPRB, 0.32636E+00_JPRB, 0.34162E+00_JPRB, 0.35104E+00_JPRB, 0.35624E+00_JPRB, 0.35884E+00_JPRB, 0.35766E+00_JPRB ,&
&  0.35425E+00_JPRB, 0.35376E+00_JPRB, 0.36486E+00_JPRB, 0.34711E+00_JPRB, 0.32935E+00_JPRB, 0.29598E+00_JPRB, 0.27954E+00_JPRB ,&
&  0.26727E+00_JPRB, 0.26518E+00_JPRB, 0.26499E+00_JPRB, 0.26975E+00_JPRB, 0.27372E+00_JPRB, 0.27685E+00_JPRB, 0.27327E+00_JPRB ,&
&  0.26809E+00_JPRB, 0.25897E+00_JPRB, 0.25131E+00_JPRB, 0.24409E+00_JPRB, 0.24417E+00_JPRB, 0.24308E+00_JPRB, 0.24051E+00_JPRB ,&
&  0.23751E+00_JPRB, 0.23557E+00_JPRB, 0.23541E+00_JPRB, 0.23553E+00_JPRB, 0.24717E+00_JPRB, 0.24656E+00_JPRB, 0.24644E+00_JPRB ,&
&  0.24600E+00_JPRB, 0.24595E+00_JPRB, 0.24498E+00_JPRB, 0.24327E+00_JPRB, 0.23863E+00_JPRB, 0.22628E+00_JPRB, 0.22585E+00_JPRB ,&
&  0.21632E+00_JPRB, 0.20568E+00_JPRB, 0.19549E+00_JPRB, 0.18839E+00_JPRB, 0.18409E+00_JPRB, 0.18214E+00_JPRB, 0.17751E+00_JPRB ,&
&  0.17256E+00_JPRB, 0.16692E+00_JPRB, 0.16422E+00_JPRB, 0.16388E+00_JPRB, 0.16438E+00_JPRB, 0.16491E+00_JPRB, 0.16538E+00_JPRB ,&
&  0.16654E+00_JPRB, 0.16711E+00_JPRB, 0.16742E+00_JPRB, 0.16804E+00_JPRB, 0.16866E+00_JPRB, 0.16963E+00_JPRB, 0.17133E+00_JPRB ,&
&  0.17438E+00_JPRB, 0.17771E+00_JPRB, 0.18079E+00_JPRB, 0.18324E+00_JPRB, 0.18229E+00_JPRB, 0.17916E+00_JPRB, 0.17497E+00_JPRB ,&
&  0.17195E+00_JPRB, 0.17100E+00_JPRB, 0.17013E+00_JPRB, 0.17012E+00_JPRB, 0.17035E+00_JPRB, 0.17082E+00_JPRB, 0.17147E+00_JPRB ,&
&  0.17231E+00_JPRB, 0.17301E+00_JPRB, 0.17386E+00_JPRB, 0.17496E+00_JPRB, 0.17653E+00_JPRB, 0.17795E+00_JPRB, 0.17995E+00_JPRB ,&
&  0.18293E+00_JPRB, 0.18619E+00_JPRB, 0.19078E+00_JPRB, 0.19487E+00_JPRB, 0.20030E+00_JPRB, 0.20834E+00_JPRB, 0.22158E+00_JPRB ,&
&  0.23598E+00_JPRB, 0.27048E+00_JPRB, 0.28645E+00_JPRB, 0.29688E+00_JPRB, 0.30415E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.31300E+00_JPRB, 0.32183E+00_JPRB, 0.33387E+00_JPRB, 0.34422E+00_JPRB, 0.35314E+00_JPRB, 0.36163E+00_JPRB, 0.36892E+00_JPRB ,&
&  0.37275E+00_JPRB, 0.37497E+00_JPRB, 0.37575E+00_JPRB, 0.37920E+00_JPRB, 0.37622E+00_JPRB, 0.36957E+00_JPRB, 0.36412E+00_JPRB ,&
&  0.35964E+00_JPRB, 0.35609E+00_JPRB, 0.35087E+00_JPRB, 0.34814E+00_JPRB, 0.34791E+00_JPRB, 0.34806E+00_JPRB, 0.34247E+00_JPRB ,&
&  0.33216E+00_JPRB, 0.33143E+00_JPRB, 0.33623E+00_JPRB, 0.33099E+00_JPRB, 0.32541E+00_JPRB, 0.32015E+00_JPRB, 0.31437E+00_JPRB ,&
&  0.29877E+00_JPRB, 0.27896E+00_JPRB, 0.25188E+00_JPRB, 0.24734E+00_JPRB, 0.26321E+00_JPRB, 0.25719E+00_JPRB, 0.26010E+00_JPRB ,&
&  0.26052E+00_JPRB, 0.26089E+00_JPRB, 0.25942E+00_JPRB, 0.25794E+00_JPRB, 0.25443E+00_JPRB, 0.25319E+00_JPRB, 0.25181E+00_JPRB ,&
&  0.24649E+00_JPRB, 0.24165E+00_JPRB, 0.23393E+00_JPRB, 0.21645E+00_JPRB, 0.21109E+00_JPRB, 0.20907E+00_JPRB, 0.20404E+00_JPRB ,&
&  0.19637E+00_JPRB, 0.19042E+00_JPRB, 0.18785E+00_JPRB, 0.18432E+00_JPRB, 0.18638E+00_JPRB, 0.18809E+00_JPRB, 0.18805E+00_JPRB ,&
&  0.18800E+00_JPRB, 0.18757E+00_JPRB, 0.18526E+00_JPRB, 0.18145E+00_JPRB, 0.17845E+00_JPRB, 0.17733E+00_JPRB, 0.17790E+00_JPRB ,&
&  0.18012E+00_JPRB, 0.18457E+00_JPRB, 0.19513E+00_JPRB, 0.20342E+00_JPRB, 0.20256E+00_JPRB, 0.19798E+00_JPRB, 0.18921E+00_JPRB ,&
&  0.18194E+00_JPRB, 0.17917E+00_JPRB, 0.17684E+00_JPRB, 0.17567E+00_JPRB, 0.17607E+00_JPRB, 0.17754E+00_JPRB, 0.17841E+00_JPRB ,&
&  0.17950E+00_JPRB, 0.18053E+00_JPRB, 0.18140E+00_JPRB, 0.18197E+00_JPRB, 0.18284E+00_JPRB, 0.18384E+00_JPRB, 0.18566E+00_JPRB ,&
&  0.18833E+00_JPRB, 0.19269E+00_JPRB, 0.19841E+00_JPRB, 0.20453E+00_JPRB, 0.20894E+00_JPRB, 0.21107E+00_JPRB, 0.21472E+00_JPRB ,&
&  0.22050E+00_JPRB, 0.23509E+00_JPRB, 0.26334E+00_JPRB, 0.30872E+00_JPRB, 0.31283E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.31757E+00_JPRB, 0.32179E+00_JPRB, 0.32453E+00_JPRB, 0.32804E+00_JPRB, 0.33564E+00_JPRB, 0.34638E+00_JPRB, 0.35851E+00_JPRB ,&
&  0.36959E+00_JPRB, 0.37849E+00_JPRB, 0.38284E+00_JPRB, 0.38796E+00_JPRB, 0.39288E+00_JPRB, 0.39526E+00_JPRB, 0.40049E+00_JPRB ,&
&  0.40127E+00_JPRB, 0.39792E+00_JPRB, 0.39189E+00_JPRB, 0.38388E+00_JPRB, 0.37968E+00_JPRB, 0.38103E+00_JPRB, 0.38401E+00_JPRB ,&
&  0.38599E+00_JPRB, 0.38752E+00_JPRB, 0.38764E+00_JPRB, 0.38428E+00_JPRB, 0.38102E+00_JPRB, 0.37910E+00_JPRB, 0.37518E+00_JPRB ,&
&  0.36644E+00_JPRB, 0.31709E+00_JPRB, 0.27941E+00_JPRB, 0.27489E+00_JPRB, 0.29177E+00_JPRB, 0.29414E+00_JPRB, 0.28887E+00_JPRB ,&
&  0.28310E+00_JPRB, 0.27846E+00_JPRB, 0.27580E+00_JPRB, 0.27370E+00_JPRB, 0.27106E+00_JPRB, 0.26878E+00_JPRB, 0.26969E+00_JPRB ,&
&  0.27103E+00_JPRB, 0.26885E+00_JPRB, 0.26581E+00_JPRB, 0.25954E+00_JPRB, 0.25494E+00_JPRB, 0.25280E+00_JPRB, 0.25104E+00_JPRB ,&
&  0.24679E+00_JPRB, 0.24260E+00_JPRB, 0.24227E+00_JPRB, 0.24325E+00_JPRB, 0.24491E+00_JPRB, 0.24605E+00_JPRB, 0.24344E+00_JPRB ,&
&  0.23720E+00_JPRB, 0.23073E+00_JPRB, 0.22246E+00_JPRB, 0.21261E+00_JPRB, 0.20321E+00_JPRB, 0.19657E+00_JPRB, 0.19137E+00_JPRB ,&
&  0.19027E+00_JPRB, 0.19542E+00_JPRB, 0.20797E+00_JPRB, 0.22342E+00_JPRB, 0.24345E+00_JPRB, 0.23207E+00_JPRB, 0.22084E+00_JPRB ,&
&  0.21049E+00_JPRB, 0.20225E+00_JPRB, 0.19618E+00_JPRB, 0.19342E+00_JPRB, 0.19240E+00_JPRB, 0.19251E+00_JPRB, 0.19368E+00_JPRB ,&
&  0.19574E+00_JPRB, 0.19844E+00_JPRB, 0.20089E+00_JPRB, 0.20208E+00_JPRB, 0.20276E+00_JPRB, 0.20336E+00_JPRB, 0.20335E+00_JPRB ,&
&  0.20270E+00_JPRB, 0.20493E+00_JPRB, 0.21156E+00_JPRB, 0.22005E+00_JPRB, 0.22839E+00_JPRB, 0.23611E+00_JPRB, 0.24905E+00_JPRB ,&
&  0.26489E+00_JPRB, 0.28496E+00_JPRB, 0.29699E+00_JPRB, 0.30608E+00_JPRB, 0.31400E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.34340E+00_JPRB, 0.34487E+00_JPRB, 0.34235E+00_JPRB, 0.33952E+00_JPRB, 0.33919E+00_JPRB, 0.34099E+00_JPRB, 0.34683E+00_JPRB ,&
&  0.35657E+00_JPRB, 0.36901E+00_JPRB, 0.38091E+00_JPRB, 0.39083E+00_JPRB, 0.39984E+00_JPRB, 0.40383E+00_JPRB, 0.40988E+00_JPRB ,&
&  0.41637E+00_JPRB, 0.42231E+00_JPRB, 0.42314E+00_JPRB, 0.41294E+00_JPRB, 0.40301E+00_JPRB, 0.39774E+00_JPRB, 0.39775E+00_JPRB ,&
&  0.39959E+00_JPRB, 0.40231E+00_JPRB, 0.40856E+00_JPRB, 0.41092E+00_JPRB, 0.40509E+00_JPRB, 0.40368E+00_JPRB, 0.40153E+00_JPRB ,&
&  0.39602E+00_JPRB, 0.38490E+00_JPRB, 0.35267E+00_JPRB, 0.34726E+00_JPRB, 0.33917E+00_JPRB, 0.33019E+00_JPRB, 0.32115E+00_JPRB ,&
&  0.31091E+00_JPRB, 0.30070E+00_JPRB, 0.29376E+00_JPRB, 0.28973E+00_JPRB, 0.29052E+00_JPRB, 0.29177E+00_JPRB, 0.29132E+00_JPRB ,&
&  0.29343E+00_JPRB, 0.29622E+00_JPRB, 0.29853E+00_JPRB, 0.29876E+00_JPRB, 0.29819E+00_JPRB, 0.29684E+00_JPRB, 0.29207E+00_JPRB ,&
&  0.28777E+00_JPRB, 0.28538E+00_JPRB, 0.28348E+00_JPRB, 0.28178E+00_JPRB, 0.28137E+00_JPRB, 0.28171E+00_JPRB, 0.28179E+00_JPRB ,&
&  0.27959E+00_JPRB, 0.27593E+00_JPRB, 0.27181E+00_JPRB, 0.26489E+00_JPRB, 0.25323E+00_JPRB, 0.23752E+00_JPRB, 0.22803E+00_JPRB ,&
&  0.22640E+00_JPRB, 0.23730E+00_JPRB, 0.25888E+00_JPRB, 0.26730E+00_JPRB, 0.27460E+00_JPRB, 0.27862E+00_JPRB, 0.28324E+00_JPRB ,&
&  0.27922E+00_JPRB, 0.25477E+00_JPRB, 0.24147E+00_JPRB, 0.23463E+00_JPRB, 0.22905E+00_JPRB, 0.22468E+00_JPRB, 0.22450E+00_JPRB ,&
&  0.22767E+00_JPRB, 0.23426E+00_JPRB, 0.24067E+00_JPRB, 0.24379E+00_JPRB, 0.24314E+00_JPRB, 0.23978E+00_JPRB, 0.23641E+00_JPRB ,&
&  0.23343E+00_JPRB, 0.23784E+00_JPRB, 0.25251E+00_JPRB, 0.27533E+00_JPRB, 0.28344E+00_JPRB, 0.28966E+00_JPRB, 0.29544E+00_JPRB ,&
&  0.30164E+00_JPRB, 0.31101E+00_JPRB, 0.32246E+00_JPRB, 0.33154E+00_JPRB, 0.33962E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.36021E+00_JPRB, 0.36005E+00_JPRB, 0.36159E+00_JPRB, 0.36057E+00_JPRB, 0.35805E+00_JPRB, 0.35623E+00_JPRB, 0.35997E+00_JPRB ,&
&  0.36693E+00_JPRB, 0.37849E+00_JPRB, 0.39379E+00_JPRB, 0.40722E+00_JPRB, 0.41537E+00_JPRB, 0.42044E+00_JPRB, 0.42817E+00_JPRB ,&
&  0.43535E+00_JPRB, 0.44334E+00_JPRB, 0.44358E+00_JPRB, 0.43476E+00_JPRB, 0.42726E+00_JPRB, 0.42330E+00_JPRB, 0.42273E+00_JPRB ,&
&  0.42206E+00_JPRB, 0.42384E+00_JPRB, 0.42708E+00_JPRB, 0.43055E+00_JPRB, 0.42996E+00_JPRB, 0.42817E+00_JPRB, 0.43185E+00_JPRB ,&
&  0.43000E+00_JPRB, 0.42053E+00_JPRB, 0.41100E+00_JPRB, 0.38616E+00_JPRB, 0.38569E+00_JPRB, 0.37550E+00_JPRB, 0.36762E+00_JPRB ,&
&  0.35703E+00_JPRB, 0.34401E+00_JPRB, 0.33457E+00_JPRB, 0.32854E+00_JPRB, 0.32586E+00_JPRB, 0.32332E+00_JPRB, 0.32133E+00_JPRB ,&
&  0.32006E+00_JPRB, 0.31989E+00_JPRB, 0.32129E+00_JPRB, 0.31911E+00_JPRB, 0.31792E+00_JPRB, 0.31867E+00_JPRB, 0.32026E+00_JPRB ,&
&  0.31986E+00_JPRB, 0.31934E+00_JPRB, 0.31893E+00_JPRB, 0.31778E+00_JPRB, 0.31599E+00_JPRB, 0.31348E+00_JPRB, 0.31109E+00_JPRB ,&
&  0.31003E+00_JPRB, 0.31091E+00_JPRB, 0.31110E+00_JPRB, 0.30720E+00_JPRB, 0.29952E+00_JPRB, 0.29037E+00_JPRB, 0.28437E+00_JPRB ,&
&  0.28172E+00_JPRB, 0.28365E+00_JPRB, 0.29134E+00_JPRB, 0.30240E+00_JPRB, 0.31275E+00_JPRB, 0.31888E+00_JPRB, 0.32263E+00_JPRB ,&
&  0.32715E+00_JPRB, 0.32564E+00_JPRB, 0.32206E+00_JPRB, 0.31882E+00_JPRB, 0.31582E+00_JPRB, 0.31181E+00_JPRB, 0.31174E+00_JPRB ,&
&  0.31426E+00_JPRB, 0.31670E+00_JPRB, 0.31741E+00_JPRB, 0.31620E+00_JPRB, 0.31410E+00_JPRB, 0.30979E+00_JPRB, 0.30375E+00_JPRB ,&
&  0.29633E+00_JPRB, 0.28773E+00_JPRB, 0.28563E+00_JPRB, 0.28613E+00_JPRB, 0.28995E+00_JPRB, 0.29781E+00_JPRB, 0.30851E+00_JPRB ,&
&  0.31976E+00_JPRB, 0.32909E+00_JPRB, 0.33913E+00_JPRB, 0.34839E+00_JPRB, 0.35664E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.38705E+00_JPRB, 0.38904E+00_JPRB, 0.39147E+00_JPRB, 0.39459E+00_JPRB, 0.39803E+00_JPRB, 0.40016E+00_JPRB, 0.40415E+00_JPRB ,&
&  0.40642E+00_JPRB, 0.41287E+00_JPRB, 0.42346E+00_JPRB, 0.43413E+00_JPRB, 0.44199E+00_JPRB, 0.44873E+00_JPRB, 0.45529E+00_JPRB ,&
&  0.46062E+00_JPRB, 0.46770E+00_JPRB, 0.46859E+00_JPRB, 0.46614E+00_JPRB, 0.46341E+00_JPRB, 0.46128E+00_JPRB, 0.45934E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.45658E+00_JPRB, 0.46031E+00_JPRB, 0.46457E+00_JPRB, 0.46896E+00_JPRB, 0.47027E+00_JPRB, 0.46909E+00_JPRB ,&
&  0.47294E+00_JPRB, 0.46370E+00_JPRB, 0.45777E+00_JPRB, 0.45076E+00_JPRB, 0.44458E+00_JPRB, 0.43327E+00_JPRB, 0.42497E+00_JPRB ,&
&  0.41514E+00_JPRB, 0.40655E+00_JPRB, 0.40294E+00_JPRB, 0.39907E+00_JPRB, 0.38945E+00_JPRB, 0.37822E+00_JPRB, 0.36987E+00_JPRB ,&
&  0.36439E+00_JPRB, 0.35893E+00_JPRB, 0.35242E+00_JPRB, 0.34708E+00_JPRB, 0.34259E+00_JPRB, 0.34097E+00_JPRB, 0.34269E+00_JPRB ,&
&  0.34672E+00_JPRB, 0.35035E+00_JPRB, 0.35235E+00_JPRB, 0.35270E+00_JPRB, 0.35102E+00_JPRB, 0.34836E+00_JPRB, 0.34674E+00_JPRB ,&
&  0.34717E+00_JPRB, 0.34840E+00_JPRB, 0.35051E+00_JPRB, 0.35565E+00_JPRB, 0.35627E+00_JPRB, 0.35463E+00_JPRB, 0.34893E+00_JPRB ,&
&  0.34616E+00_JPRB, 0.34467E+00_JPRB, 0.34614E+00_JPRB, 0.35112E+00_JPRB, 0.35942E+00_JPRB, 0.36749E+00_JPRB, 0.37331E+00_JPRB ,&
&  0.37627E+00_JPRB, 0.37814E+00_JPRB, 0.38132E+00_JPRB, 0.38499E+00_JPRB, 0.38644E+00_JPRB, 0.38574E+00_JPRB, 0.38418E+00_JPRB ,&
&  0.38691E+00_JPRB, 0.39426E+00_JPRB, 0.40202E+00_JPRB, 0.40686E+00_JPRB, 0.40917E+00_JPRB, 0.40809E+00_JPRB, 0.40124E+00_JPRB ,&
&  0.38886E+00_JPRB, 0.37465E+00_JPRB, 0.36086E+00_JPRB, 0.34915E+00_JPRB, 0.34102E+00_JPRB, 0.33723E+00_JPRB, 0.34018E+00_JPRB ,&
&  0.34799E+00_JPRB, 0.35799E+00_JPRB, 0.36724E+00_JPRB, 0.37417E+00_JPRB, 0.38176E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.41430E+00_JPRB, 0.41455E+00_JPRB, 0.41770E+00_JPRB, 0.42428E+00_JPRB, 0.42864E+00_JPRB, 0.43260E+00_JPRB, 0.44041E+00_JPRB ,&
&  0.44750E+00_JPRB, 0.45371E+00_JPRB, 0.46117E+00_JPRB, 0.46938E+00_JPRB, 0.47582E+00_JPRB, 0.48499E+00_JPRB, 0.49364E+00_JPRB ,&
&  0.49920E+00_JPRB, 0.50141E+00_JPRB, 0.50263E+00_JPRB, 0.50366E+00_JPRB, 0.50338E+00_JPRB, 0.50073E+00_JPRB, 0.50016E+00_JPRB ,&
&  0.49826E+00_JPRB, 0.49617E+00_JPRB, 0.49829E+00_JPRB, 0.49863E+00_JPRB, 0.50036E+00_JPRB, 0.50327E+00_JPRB, 0.50361E+00_JPRB ,&
&  0.50180E+00_JPRB, 0.48907E+00_JPRB, 0.48455E+00_JPRB, 0.48182E+00_JPRB, 0.48622E+00_JPRB, 0.48746E+00_JPRB, 0.48618E+00_JPRB ,&
&  0.47998E+00_JPRB, 0.47627E+00_JPRB, 0.47825E+00_JPRB, 0.48069E+00_JPRB, 0.47501E+00_JPRB, 0.46211E+00_JPRB, 0.45185E+00_JPRB ,&
&  0.44381E+00_JPRB, 0.43174E+00_JPRB, 0.41426E+00_JPRB, 0.40392E+00_JPRB, 0.39949E+00_JPRB, 0.39455E+00_JPRB, 0.38992E+00_JPRB ,&
&  0.38742E+00_JPRB, 0.38657E+00_JPRB, 0.38794E+00_JPRB, 0.38701E+00_JPRB, 0.38326E+00_JPRB, 0.37869E+00_JPRB, 0.37685E+00_JPRB ,&
&  0.37935E+00_JPRB, 0.38156E+00_JPRB, 0.38494E+00_JPRB, 0.38749E+00_JPRB, 0.38684E+00_JPRB, 0.38714E+00_JPRB, 0.39071E+00_JPRB ,&
&  0.39611E+00_JPRB, 0.40114E+00_JPRB, 0.40648E+00_JPRB, 0.40869E+00_JPRB, 0.41166E+00_JPRB, 0.41820E+00_JPRB, 0.41863E+00_JPRB ,&
&  0.41720E+00_JPRB, 0.41924E+00_JPRB, 0.42338E+00_JPRB, 0.43117E+00_JPRB, 0.44315E+00_JPRB, 0.45109E+00_JPRB, 0.45234E+00_JPRB ,&
&  0.45248E+00_JPRB, 0.45476E+00_JPRB, 0.45853E+00_JPRB, 0.46762E+00_JPRB, 0.47755E+00_JPRB, 0.48474E+00_JPRB, 0.48783E+00_JPRB ,&
&  0.48491E+00_JPRB, 0.47839E+00_JPRB, 0.46647E+00_JPRB, 0.45254E+00_JPRB, 0.44001E+00_JPRB, 0.43022E+00_JPRB, 0.42219E+00_JPRB ,&
&  0.41421E+00_JPRB, 0.41073E+00_JPRB, 0.41039E+00_JPRB, 0.41234E+00_JPRB, 0.41359E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.45026E+00_JPRB, 0.45141E+00_JPRB, 0.45648E+00_JPRB, 0.46148E+00_JPRB, 0.46503E+00_JPRB, 0.47001E+00_JPRB, 0.47960E+00_JPRB ,&
&  0.48927E+00_JPRB, 0.49855E+00_JPRB, 0.50494E+00_JPRB, 0.51268E+00_JPRB, 0.52362E+00_JPRB, 0.53103E+00_JPRB, 0.53691E+00_JPRB ,&
&  0.53936E+00_JPRB, 0.53965E+00_JPRB, 0.54050E+00_JPRB, 0.54345E+00_JPRB, 0.54289E+00_JPRB, 0.53918E+00_JPRB, 0.53779E+00_JPRB ,&
&  0.53759E+00_JPRB, 0.53585E+00_JPRB, 0.53558E+00_JPRB, 0.53670E+00_JPRB, 0.53451E+00_JPRB, 0.53090E+00_JPRB, 0.53064E+00_JPRB ,&
&  0.53114E+00_JPRB, 0.51774E+00_JPRB, 0.51043E+00_JPRB, 0.50364E+00_JPRB, 0.50746E+00_JPRB, 0.51903E+00_JPRB, 0.53172E+00_JPRB ,&
&  0.53938E+00_JPRB, 0.54457E+00_JPRB, 0.54435E+00_JPRB, 0.54349E+00_JPRB, 0.53892E+00_JPRB, 0.52760E+00_JPRB, 0.51800E+00_JPRB ,&
&  0.51161E+00_JPRB, 0.50249E+00_JPRB, 0.48029E+00_JPRB, 0.46747E+00_JPRB, 0.46412E+00_JPRB, 0.46672E+00_JPRB, 0.47058E+00_JPRB ,&
&  0.47220E+00_JPRB, 0.46973E+00_JPRB, 0.46371E+00_JPRB, 0.45645E+00_JPRB, 0.44654E+00_JPRB, 0.43503E+00_JPRB, 0.42495E+00_JPRB ,&
&  0.41912E+00_JPRB, 0.41471E+00_JPRB, 0.41105E+00_JPRB, 0.41343E+00_JPRB, 0.41264E+00_JPRB, 0.40873E+00_JPRB, 0.41044E+00_JPRB ,&
&  0.42144E+00_JPRB, 0.43524E+00_JPRB, 0.44802E+00_JPRB, 0.45504E+00_JPRB, 0.45788E+00_JPRB, 0.46115E+00_JPRB, 0.46509E+00_JPRB ,&
&  0.47094E+00_JPRB, 0.47680E+00_JPRB, 0.48181E+00_JPRB, 0.48812E+00_JPRB, 0.49603E+00_JPRB, 0.50330E+00_JPRB, 0.50642E+00_JPRB ,&
&  0.50910E+00_JPRB, 0.51065E+00_JPRB, 0.51168E+00_JPRB, 0.51661E+00_JPRB, 0.52430E+00_JPRB, 0.52981E+00_JPRB, 0.53441E+00_JPRB ,&
&  0.53922E+00_JPRB, 0.54158E+00_JPRB, 0.54043E+00_JPRB, 0.53675E+00_JPRB, 0.53136E+00_JPRB, 0.52050E+00_JPRB, 0.50704E+00_JPRB ,&
&  0.49141E+00_JPRB, 0.47712E+00_JPRB, 0.46659E+00_JPRB, 0.45761E+00_JPRB, 0.45155E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.49891E+00_JPRB, 0.49721E+00_JPRB, 0.49786E+00_JPRB, 0.50115E+00_JPRB, 0.50474E+00_JPRB, 0.50751E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.52318E+00_JPRB, 0.53094E+00_JPRB, 0.54039E+00_JPRB, 0.55006E+00_JPRB, 0.55743E+00_JPRB, 0.56243E+00_JPRB, 0.56513E+00_JPRB ,&
&  0.56341E+00_JPRB, 0.56066E+00_JPRB, 0.56336E+00_JPRB, 0.56784E+00_JPRB, 0.56913E+00_JPRB, 0.56859E+00_JPRB, 0.56850E+00_JPRB ,&
&  0.56789E+00_JPRB, 0.56554E+00_JPRB, 0.56258E+00_JPRB, 0.55829E+00_JPRB, 0.55327E+00_JPRB, 0.55101E+00_JPRB, 0.54892E+00_JPRB ,&
&  0.54253E+00_JPRB, 0.54246E+00_JPRB, 0.49241E+00_JPRB, 0.50773E+00_JPRB, 0.52071E+00_JPRB, 0.54135E+00_JPRB, 0.56150E+00_JPRB ,&
&  0.57580E+00_JPRB, 0.58476E+00_JPRB, 0.58917E+00_JPRB, 0.58752E+00_JPRB, 0.57524E+00_JPRB, 0.56756E+00_JPRB, 0.55890E+00_JPRB ,&
&  0.54802E+00_JPRB, 0.54028E+00_JPRB, 0.52732E+00_JPRB, 0.51332E+00_JPRB, 0.50859E+00_JPRB, 0.50902E+00_JPRB, 0.50891E+00_JPRB ,&
&  0.50519E+00_JPRB, 0.50171E+00_JPRB, 0.50309E+00_JPRB, 0.50410E+00_JPRB, 0.50389E+00_JPRB, 0.49855E+00_JPRB, 0.49173E+00_JPRB ,&
&  0.48113E+00_JPRB, 0.47069E+00_JPRB, 0.46344E+00_JPRB, 0.45802E+00_JPRB, 0.45319E+00_JPRB, 0.44784E+00_JPRB, 0.44257E+00_JPRB ,&
&  0.44578E+00_JPRB, 0.45660E+00_JPRB, 0.46680E+00_JPRB, 0.47531E+00_JPRB, 0.48092E+00_JPRB, 0.48473E+00_JPRB, 0.48995E+00_JPRB ,&
&  0.49601E+00_JPRB, 0.50617E+00_JPRB, 0.51311E+00_JPRB, 0.51931E+00_JPRB, 0.52404E+00_JPRB, 0.52853E+00_JPRB, 0.53372E+00_JPRB ,&
&  0.53794E+00_JPRB, 0.54463E+00_JPRB, 0.54942E+00_JPRB, 0.55526E+00_JPRB, 0.55860E+00_JPRB, 0.56348E+00_JPRB, 0.56950E+00_JPRB ,&
&  0.57629E+00_JPRB, 0.58237E+00_JPRB, 0.58492E+00_JPRB, 0.58379E+00_JPRB, 0.57671E+00_JPRB, 0.56711E+00_JPRB, 0.55793E+00_JPRB ,&
&  0.54607E+00_JPRB, 0.53186E+00_JPRB, 0.51753E+00_JPRB, 0.50846E+00_JPRB, 0.50111E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.56529E+00_JPRB, 0.56048E+00_JPRB, 0.55513E+00_JPRB, 0.55424E+00_JPRB, 0.55298E+00_JPRB, 0.55468E+00_JPRB, 0.55738E+00_JPRB ,&
&  0.56401E+00_JPRB, 0.56826E+00_JPRB, 0.57738E+00_JPRB, 0.58482E+00_JPRB, 0.58849E+00_JPRB, 0.59285E+00_JPRB, 0.59564E+00_JPRB ,&
&  0.59668E+00_JPRB, 0.59800E+00_JPRB, 0.59850E+00_JPRB, 0.60252E+00_JPRB, 0.60301E+00_JPRB, 0.60205E+00_JPRB, 0.60089E+00_JPRB ,&
&  0.59884E+00_JPRB, 0.59662E+00_JPRB, 0.59153E+00_JPRB, 0.58452E+00_JPRB, 0.57576E+00_JPRB, 0.57207E+00_JPRB, 0.56611E+00_JPRB ,&
&  0.56737E+00_JPRB, 0.53663E+00_JPRB, 0.54853E+00_JPRB, 0.54725E+00_JPRB, 0.56271E+00_JPRB, 0.58690E+00_JPRB, 0.61941E+00_JPRB ,&
&  0.64170E+00_JPRB, 0.65328E+00_JPRB, 0.65496E+00_JPRB, 0.65292E+00_JPRB, 0.63903E+00_JPRB, 0.62460E+00_JPRB, 0.60820E+00_JPRB ,&
&  0.58931E+00_JPRB, 0.57855E+00_JPRB, 0.56488E+00_JPRB, 0.55480E+00_JPRB, 0.55062E+00_JPRB, 0.54762E+00_JPRB, 0.54268E+00_JPRB ,&
&  0.53522E+00_JPRB, 0.52878E+00_JPRB, 0.52929E+00_JPRB, 0.53278E+00_JPRB, 0.53640E+00_JPRB, 0.53860E+00_JPRB, 0.53738E+00_JPRB ,&
&  0.53101E+00_JPRB, 0.52542E+00_JPRB, 0.52268E+00_JPRB, 0.52211E+00_JPRB, 0.51758E+00_JPRB, 0.50976E+00_JPRB, 0.50166E+00_JPRB ,&
&  0.49957E+00_JPRB, 0.50218E+00_JPRB, 0.50590E+00_JPRB, 0.51068E+00_JPRB, 0.51428E+00_JPRB, 0.51864E+00_JPRB, 0.52073E+00_JPRB ,&
&  0.52548E+00_JPRB, 0.53004E+00_JPRB, 0.53929E+00_JPRB, 0.54820E+00_JPRB, 0.55060E+00_JPRB, 0.55938E+00_JPRB, 0.56839E+00_JPRB ,&
&  0.57718E+00_JPRB, 0.58726E+00_JPRB, 0.59486E+00_JPRB, 0.59981E+00_JPRB, 0.60503E+00_JPRB, 0.60811E+00_JPRB, 0.61364E+00_JPRB ,&
&  0.62117E+00_JPRB, 0.62296E+00_JPRB, 0.62834E+00_JPRB, 0.62916E+00_JPRB, 0.62450E+00_JPRB, 0.61670E+00_JPRB, 0.60698E+00_JPRB ,&
&  0.59616E+00_JPRB, 0.58094E+00_JPRB, 0.57309E+00_JPRB, 0.56864E+00_JPRB, 0.56653E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.61958E+00_JPRB, 0.61279E+00_JPRB, 0.60123E+00_JPRB, 0.59423E+00_JPRB, 0.59098E+00_JPRB, 0.59353E+00_JPRB, 0.59740E+00_JPRB ,&
&  0.60188E+00_JPRB, 0.60355E+00_JPRB, 0.61102E+00_JPRB, 0.61875E+00_JPRB, 0.62347E+00_JPRB, 0.62664E+00_JPRB, 0.62572E+00_JPRB ,&
&  0.62534E+00_JPRB, 0.62482E+00_JPRB, 0.61970E+00_JPRB, 0.61527E+00_JPRB, 0.60937E+00_JPRB, 0.60949E+00_JPRB, 0.60907E+00_JPRB ,&
&  0.60752E+00_JPRB, 0.60431E+00_JPRB, 0.59801E+00_JPRB, 0.58950E+00_JPRB, 0.58575E+00_JPRB, 0.58532E+00_JPRB, 0.58311E+00_JPRB ,&
&  0.58950E+00_JPRB, 0.50986E+00_JPRB, 0.54709E+00_JPRB, 0.56401E+00_JPRB, 0.58771E+00_JPRB, 0.61681E+00_JPRB, 0.64847E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.70625E+00_JPRB, 0.71583E+00_JPRB, 0.71667E+00_JPRB, 0.70483E+00_JPRB, 0.68447E+00_JPRB, 0.66076E+00_JPRB ,&
&  0.64019E+00_JPRB, 0.62435E+00_JPRB, 0.60583E+00_JPRB, 0.59494E+00_JPRB, 0.58617E+00_JPRB, 0.57504E+00_JPRB, 0.56560E+00_JPRB ,&
&  0.56145E+00_JPRB, 0.55895E+00_JPRB, 0.55867E+00_JPRB, 0.56040E+00_JPRB, 0.56327E+00_JPRB, 0.56528E+00_JPRB, 0.56502E+00_JPRB ,&
&  0.56407E+00_JPRB, 0.56214E+00_JPRB, 0.55846E+00_JPRB, 0.55697E+00_JPRB, 0.55657E+00_JPRB, 0.55839E+00_JPRB, 0.55500E+00_JPRB ,&
&  0.54876E+00_JPRB, 0.55023E+00_JPRB, 0.55343E+00_JPRB, 0.55535E+00_JPRB, 0.55810E+00_JPRB, 0.56091E+00_JPRB, 0.56093E+00_JPRB ,&
&  0.56092E+00_JPRB, 0.56462E+00_JPRB, 0.56872E+00_JPRB, 0.57555E+00_JPRB, 0.58044E+00_JPRB, 0.58968E+00_JPRB, 0.59743E+00_JPRB ,&
&  0.60493E+00_JPRB, 0.61634E+00_JPRB, 0.62699E+00_JPRB, 0.63409E+00_JPRB, 0.63841E+00_JPRB, 0.64289E+00_JPRB, 0.64788E+00_JPRB ,&
&  0.65170E+00_JPRB, 0.65507E+00_JPRB, 0.65815E+00_JPRB, 0.66004E+00_JPRB, 0.65694E+00_JPRB, 0.65337E+00_JPRB, 0.65125E+00_JPRB ,&
&  0.64338E+00_JPRB, 0.63184E+00_JPRB, 0.62787E+00_JPRB, 0.62463E+00_JPRB, 0.62116E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.70112E+00_JPRB, 0.69204E+00_JPRB, 0.67649E+00_JPRB, 0.66292E+00_JPRB, 0.65150E+00_JPRB, 0.64696E+00_JPRB, 0.64450E+00_JPRB ,&
&  0.64939E+00_JPRB, 0.65795E+00_JPRB, 0.66576E+00_JPRB, 0.66967E+00_JPRB, 0.67215E+00_JPRB, 0.67130E+00_JPRB, 0.66678E+00_JPRB ,&
&  0.66281E+00_JPRB, 0.65050E+00_JPRB, 0.64099E+00_JPRB, 0.64018E+00_JPRB, 0.63633E+00_JPRB, 0.63205E+00_JPRB, 0.62722E+00_JPRB ,&
&  0.62280E+00_JPRB, 0.61946E+00_JPRB, 0.61067E+00_JPRB, 0.60670E+00_JPRB, 0.60591E+00_JPRB, 0.60825E+00_JPRB, 0.61846E+00_JPRB ,&
&  0.54430E+00_JPRB, 0.46655E+00_JPRB, 0.52509E+00_JPRB, 0.58796E+00_JPRB, 0.62116E+00_JPRB, 0.66148E+00_JPRB, 0.69789E+00_JPRB ,&
&  0.74595E+00_JPRB, 0.77696E+00_JPRB, 0.79420E+00_JPRB, 0.78947E+00_JPRB, 0.77180E+00_JPRB, 0.75114E+00_JPRB, 0.72918E+00_JPRB ,&
&  0.70248E+00_JPRB, 0.67230E+00_JPRB, 0.65267E+00_JPRB, 0.64285E+00_JPRB, 0.63457E+00_JPRB, 0.62565E+00_JPRB, 0.62020E+00_JPRB ,&
&  0.61819E+00_JPRB, 0.61692E+00_JPRB, 0.61511E+00_JPRB, 0.61403E+00_JPRB, 0.61595E+00_JPRB, 0.61822E+00_JPRB, 0.62117E+00_JPRB ,&
&  0.62224E+00_JPRB, 0.62197E+00_JPRB, 0.61980E+00_JPRB, 0.61418E+00_JPRB, 0.60991E+00_JPRB, 0.61124E+00_JPRB, 0.61040E+00_JPRB ,&
&  0.61037E+00_JPRB, 0.60933E+00_JPRB, 0.60921E+00_JPRB, 0.60948E+00_JPRB, 0.60820E+00_JPRB, 0.60710E+00_JPRB, 0.60858E+00_JPRB ,&
&  0.61313E+00_JPRB, 0.61286E+00_JPRB, 0.61550E+00_JPRB, 0.62077E+00_JPRB, 0.62642E+00_JPRB, 0.63183E+00_JPRB, 0.63546E+00_JPRB ,&
&  0.64198E+00_JPRB, 0.65486E+00_JPRB, 0.66434E+00_JPRB, 0.67046E+00_JPRB, 0.67537E+00_JPRB, 0.68107E+00_JPRB, 0.68923E+00_JPRB ,&
&  0.69608E+00_JPRB, 0.70243E+00_JPRB, 0.70981E+00_JPRB, 0.72743E+00_JPRB, 0.73580E+00_JPRB, 0.73641E+00_JPRB, 0.73308E+00_JPRB ,&
&  0.72312E+00_JPRB, 0.70648E+00_JPRB, 0.70741E+00_JPRB, 0.70459E+00_JPRB, 0.70232E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.74829E+00_JPRB, 0.74377E+00_JPRB, 0.73011E+00_JPRB, 0.71420E+00_JPRB, 0.70283E+00_JPRB, 0.69871E+00_JPRB, 0.69789E+00_JPRB ,&
&  0.70224E+00_JPRB, 0.70562E+00_JPRB, 0.70547E+00_JPRB, 0.70078E+00_JPRB, 0.69366E+00_JPRB, 0.68307E+00_JPRB, 0.67415E+00_JPRB ,&
&  0.65956E+00_JPRB, 0.64427E+00_JPRB, 0.63880E+00_JPRB, 0.63826E+00_JPRB, 0.63574E+00_JPRB, 0.62901E+00_JPRB, 0.62544E+00_JPRB ,&
&  0.62438E+00_JPRB, 0.62703E+00_JPRB, 0.62127E+00_JPRB, 0.61776E+00_JPRB, 0.61316E+00_JPRB, 0.61514E+00_JPRB, 0.61643E+00_JPRB ,&
&  0.61589E+00_JPRB, 0.49345E+00_JPRB, 0.56379E+00_JPRB, 0.62236E+00_JPRB, 0.64964E+00_JPRB, 0.70384E+00_JPRB, 0.76123E+00_JPRB ,&
&  0.80250E+00_JPRB, 0.82201E+00_JPRB, 0.81613E+00_JPRB, 0.79904E+00_JPRB, 0.77213E+00_JPRB, 0.74573E+00_JPRB, 0.72215E+00_JPRB ,&
&  0.69983E+00_JPRB, 0.68374E+00_JPRB, 0.67064E+00_JPRB, 0.65894E+00_JPRB, 0.65217E+00_JPRB, 0.64549E+00_JPRB, 0.63960E+00_JPRB ,&
&  0.64022E+00_JPRB, 0.64697E+00_JPRB, 0.64788E+00_JPRB, 0.65069E+00_JPRB, 0.65456E+00_JPRB, 0.65689E+00_JPRB, 0.66646E+00_JPRB ,&
&  0.67294E+00_JPRB, 0.67424E+00_JPRB, 0.67551E+00_JPRB, 0.67365E+00_JPRB, 0.66548E+00_JPRB, 0.66281E+00_JPRB, 0.65856E+00_JPRB ,&
&  0.65439E+00_JPRB, 0.64644E+00_JPRB, 0.63962E+00_JPRB, 0.63641E+00_JPRB, 0.63581E+00_JPRB, 0.63543E+00_JPRB, 0.64118E+00_JPRB ,&
&  0.65131E+00_JPRB, 0.65363E+00_JPRB, 0.65548E+00_JPRB, 0.65990E+00_JPRB, 0.66378E+00_JPRB, 0.66665E+00_JPRB, 0.67162E+00_JPRB ,&
&  0.68226E+00_JPRB, 0.69876E+00_JPRB, 0.71382E+00_JPRB, 0.72038E+00_JPRB, 0.72299E+00_JPRB, 0.73124E+00_JPRB, 0.74096E+00_JPRB ,&
&  0.75111E+00_JPRB, 0.75601E+00_JPRB, 0.75884E+00_JPRB, 0.80205E+00_JPRB, 0.82003E+00_JPRB, 0.81934E+00_JPRB, 0.81207E+00_JPRB ,&
&  0.79141E+00_JPRB, 0.78406E+00_JPRB, 0.63660E+00_JPRB, 0.70488E+00_JPRB, 0.72741E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.81427E+00_JPRB, 0.80854E+00_JPRB, 0.79082E+00_JPRB, 0.77154E+00_JPRB, 0.76341E+00_JPRB, 0.76029E+00_JPRB, 0.75795E+00_JPRB ,&
&  0.74951E+00_JPRB, 0.73732E+00_JPRB, 0.72351E+00_JPRB, 0.70795E+00_JPRB, 0.69170E+00_JPRB, 0.67816E+00_JPRB, 0.66526E+00_JPRB ,&
&  0.64525E+00_JPRB, 0.63369E+00_JPRB, 0.63277E+00_JPRB, 0.62969E+00_JPRB, 0.62698E+00_JPRB, 0.62303E+00_JPRB, 0.62393E+00_JPRB ,&
&  0.62727E+00_JPRB, 0.62853E+00_JPRB, 0.62875E+00_JPRB, 0.62238E+00_JPRB, 0.61744E+00_JPRB, 0.61746E+00_JPRB, 0.61353E+00_JPRB ,&
&  0.61341E+00_JPRB, 0.51477E+00_JPRB, 0.58588E+00_JPRB, 0.67623E+00_JPRB, 0.72228E+00_JPRB, 0.78891E+00_JPRB, 0.87123E+00_JPRB ,&
&  0.91033E+00_JPRB, 0.89184E+00_JPRB, 0.85601E+00_JPRB, 0.82314E+00_JPRB, 0.78812E+00_JPRB, 0.75677E+00_JPRB, 0.73111E+00_JPRB ,&
&  0.71516E+00_JPRB, 0.70574E+00_JPRB, 0.69829E+00_JPRB, 0.68912E+00_JPRB, 0.67942E+00_JPRB, 0.67532E+00_JPRB, 0.67292E+00_JPRB ,&
&  0.67317E+00_JPRB, 0.67724E+00_JPRB, 0.67953E+00_JPRB, 0.68642E+00_JPRB, 0.69223E+00_JPRB, 0.70473E+00_JPRB, 0.72629E+00_JPRB ,&
&  0.73536E+00_JPRB, 0.74938E+00_JPRB, 0.77259E+00_JPRB, 0.77734E+00_JPRB, 0.76569E+00_JPRB, 0.75763E+00_JPRB, 0.74779E+00_JPRB ,&
&  0.73443E+00_JPRB, 0.71251E+00_JPRB, 0.70558E+00_JPRB, 0.69887E+00_JPRB, 0.69304E+00_JPRB, 0.69117E+00_JPRB, 0.70007E+00_JPRB ,&
&  0.70923E+00_JPRB, 0.71233E+00_JPRB, 0.71782E+00_JPRB, 0.72660E+00_JPRB, 0.73118E+00_JPRB, 0.72841E+00_JPRB, 0.72939E+00_JPRB ,&
&  0.73860E+00_JPRB, 0.75226E+00_JPRB, 0.75846E+00_JPRB, 0.75902E+00_JPRB, 0.76732E+00_JPRB, 0.78316E+00_JPRB, 0.79499E+00_JPRB ,&
&  0.80384E+00_JPRB, 0.80982E+00_JPRB, 0.82063E+00_JPRB, 0.85098E+00_JPRB, 0.91262E+00_JPRB, 0.90461E+00_JPRB, 0.88439E+00_JPRB ,&
&  0.86910E+00_JPRB, 0.85128E+00_JPRB, 0.84596E+00_JPRB, 0.73192E+00_JPRB, 0.78389E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.87227E+00_JPRB, 0.86979E+00_JPRB, 0.84466E+00_JPRB, 0.82108E+00_JPRB, 0.80754E+00_JPRB, 0.79298E+00_JPRB, 0.76821E+00_JPRB ,&
&  0.74097E+00_JPRB, 0.71639E+00_JPRB, 0.70112E+00_JPRB, 0.68143E+00_JPRB, 0.66290E+00_JPRB, 0.65159E+00_JPRB, 0.63991E+00_JPRB ,&
&  0.62328E+00_JPRB, 0.61891E+00_JPRB, 0.61892E+00_JPRB, 0.61704E+00_JPRB, 0.61289E+00_JPRB, 0.61079E+00_JPRB, 0.61829E+00_JPRB ,&
&  0.61958E+00_JPRB, 0.61756E+00_JPRB, 0.61710E+00_JPRB, 0.61282E+00_JPRB, 0.60730E+00_JPRB, 0.60640E+00_JPRB, 0.59563E+00_JPRB ,&
&  0.60164E+00_JPRB, 0.57212E+00_JPRB, 0.67002E+00_JPRB, 0.78237E+00_JPRB, 0.87853E+00_JPRB, 0.94463E+00_JPRB, 0.98919E+00_JPRB ,&
&  0.99092E+00_JPRB, 0.94370E+00_JPRB, 0.88862E+00_JPRB, 0.84805E+00_JPRB, 0.80447E+00_JPRB, 0.77053E+00_JPRB, 0.74413E+00_JPRB ,&
&  0.72722E+00_JPRB, 0.71858E+00_JPRB, 0.71220E+00_JPRB, 0.70631E+00_JPRB, 0.70001E+00_JPRB, 0.69472E+00_JPRB, 0.69281E+00_JPRB ,&
&  0.68942E+00_JPRB, 0.69016E+00_JPRB, 0.69215E+00_JPRB, 0.69001E+00_JPRB, 0.69579E+00_JPRB, 0.72405E+00_JPRB, 0.76516E+00_JPRB ,&
&  0.78303E+00_JPRB, 0.81052E+00_JPRB, 0.84756E+00_JPRB, 0.86483E+00_JPRB, 0.85570E+00_JPRB, 0.84572E+00_JPRB, 0.82987E+00_JPRB ,&
&  0.81677E+00_JPRB, 0.79737E+00_JPRB, 0.78971E+00_JPRB, 0.77889E+00_JPRB, 0.76553E+00_JPRB, 0.75707E+00_JPRB, 0.75448E+00_JPRB ,&
&  0.75495E+00_JPRB, 0.75620E+00_JPRB, 0.76364E+00_JPRB, 0.77298E+00_JPRB, 0.77270E+00_JPRB, 0.76842E+00_JPRB, 0.77016E+00_JPRB ,&
&  0.77465E+00_JPRB, 0.76957E+00_JPRB, 0.75919E+00_JPRB, 0.76958E+00_JPRB, 0.79312E+00_JPRB, 0.80972E+00_JPRB, 0.82089E+00_JPRB ,&
&  0.83193E+00_JPRB, 0.84846E+00_JPRB, 0.87184E+00_JPRB, 0.94624E+00_JPRB, 0.96935E+00_JPRB, 0.95398E+00_JPRB, 0.93970E+00_JPRB ,&
&  0.93139E+00_JPRB, 0.91568E+00_JPRB, 0.89555E+00_JPRB, 0.88219E+00_JPRB, 0.81884E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.10355E+01_JPRB, 0.95941E+00_JPRB, 0.89774E+00_JPRB, 0.85266E+00_JPRB, 0.82170E+00_JPRB, 0.78481E+00_JPRB, 0.74286E+00_JPRB ,&
&  0.71222E+00_JPRB, 0.69976E+00_JPRB, 0.68710E+00_JPRB, 0.66862E+00_JPRB, 0.65552E+00_JPRB, 0.64137E+00_JPRB, 0.62913E+00_JPRB ,&
&  0.61321E+00_JPRB, 0.60859E+00_JPRB, 0.60449E+00_JPRB, 0.60880E+00_JPRB, 0.61003E+00_JPRB, 0.61513E+00_JPRB, 0.62679E+00_JPRB ,&
&  0.62945E+00_JPRB, 0.60975E+00_JPRB, 0.60640E+00_JPRB, 0.60760E+00_JPRB, 0.60793E+00_JPRB, 0.60855E+00_JPRB, 0.59825E+00_JPRB ,&
&  0.61976E+00_JPRB, 0.64479E+00_JPRB, 0.84156E+00_JPRB, 0.10362E+01_JPRB, 0.11367E+01_JPRB, 0.11870E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.10461E+01_JPRB, 0.97333E+00_JPRB, 0.89777E+00_JPRB, 0.82791E+00_JPRB, 0.78230E+00_JPRB, 0.75709E+00_JPRB ,&
&  0.74309E+00_JPRB, 0.73836E+00_JPRB, 0.73278E+00_JPRB, 0.72979E+00_JPRB, 0.72661E+00_JPRB, 0.72133E+00_JPRB, 0.72266E+00_JPRB ,&
&  0.72856E+00_JPRB, 0.71862E+00_JPRB, 0.71395E+00_JPRB, 0.72487E+00_JPRB, 0.74441E+00_JPRB, 0.77404E+00_JPRB, 0.85442E+00_JPRB ,&
&  0.90327E+00_JPRB, 0.94812E+00_JPRB, 0.99579E+00_JPRB, 0.10141E+01_JPRB, 0.10024E+01_JPRB, 0.98496E+00_JPRB, 0.95276E+00_JPRB ,&
&  0.93010E+00_JPRB, 0.90941E+00_JPRB, 0.88777E+00_JPRB, 0.86569E+00_JPRB, 0.84530E+00_JPRB, 0.83236E+00_JPRB, 0.82421E+00_JPRB ,&
&  0.81725E+00_JPRB, 0.82473E+00_JPRB, 0.83283E+00_JPRB, 0.83642E+00_JPRB, 0.82995E+00_JPRB, 0.82424E+00_JPRB, 0.81889E+00_JPRB ,&
&  0.80263E+00_JPRB, 0.77889E+00_JPRB, 0.77201E+00_JPRB, 0.78505E+00_JPRB, 0.81634E+00_JPRB, 0.83238E+00_JPRB, 0.84060E+00_JPRB ,&
&  0.86065E+00_JPRB, 0.89297E+00_JPRB, 0.90329E+00_JPRB, 0.10123E+01_JPRB, 0.11003E+01_JPRB, 0.10780E+01_JPRB, 0.10449E+01_JPRB ,&
&  0.10045E+01_JPRB, 0.98096E+00_JPRB, 0.94864E+00_JPRB, 0.92637E+00_JPRB, 0.10259E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.93302E+00_JPRB, 0.89018E+00_JPRB, 0.85097E+00_JPRB, 0.81054E+00_JPRB, 0.78247E+00_JPRB, 0.75229E+00_JPRB, 0.72324E+00_JPRB ,&
&  0.69803E+00_JPRB, 0.68188E+00_JPRB, 0.66825E+00_JPRB, 0.65510E+00_JPRB, 0.65421E+00_JPRB, 0.64719E+00_JPRB, 0.63295E+00_JPRB ,&
&  0.61332E+00_JPRB, 0.59973E+00_JPRB, 0.59593E+00_JPRB, 0.59895E+00_JPRB, 0.61149E+00_JPRB, 0.62015E+00_JPRB, 0.62896E+00_JPRB ,&
&  0.63372E+00_JPRB, 0.61415E+00_JPRB, 0.60467E+00_JPRB, 0.60675E+00_JPRB, 0.60792E+00_JPRB, 0.61091E+00_JPRB, 0.59639E+00_JPRB ,&
&  0.61044E+00_JPRB, 0.71154E+00_JPRB, 0.10390E+01_JPRB, 0.14704E+01_JPRB, 0.15524E+01_JPRB, 0.14433E+01_JPRB, 0.13227E+01_JPRB ,&
&  0.12398E+01_JPRB, 0.11642E+01_JPRB, 0.10702E+01_JPRB, 0.97112E+00_JPRB, 0.89582E+00_JPRB, 0.84763E+00_JPRB, 0.81932E+00_JPRB ,&
&  0.80993E+00_JPRB, 0.80448E+00_JPRB, 0.79277E+00_JPRB, 0.78434E+00_JPRB, 0.77525E+00_JPRB, 0.76872E+00_JPRB, 0.76773E+00_JPRB ,&
&  0.76559E+00_JPRB, 0.76072E+00_JPRB, 0.75480E+00_JPRB, 0.75638E+00_JPRB, 0.82324E+00_JPRB, 0.90775E+00_JPRB, 0.10791E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.12388E+01_JPRB, 0.12816E+01_JPRB, 0.12735E+01_JPRB, 0.12292E+01_JPRB, 0.11836E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10270E+01_JPRB, 0.98516E+00_JPRB, 0.95443E+00_JPRB, 0.92548E+00_JPRB, 0.90272E+00_JPRB, 0.88554E+00_JPRB ,&
&  0.88128E+00_JPRB, 0.89068E+00_JPRB, 0.89618E+00_JPRB, 0.89475E+00_JPRB, 0.87926E+00_JPRB, 0.85866E+00_JPRB, 0.83619E+00_JPRB ,&
&  0.80423E+00_JPRB, 0.78406E+00_JPRB, 0.79328E+00_JPRB, 0.81492E+00_JPRB, 0.84348E+00_JPRB, 0.85452E+00_JPRB, 0.86848E+00_JPRB ,&
&  0.89649E+00_JPRB, 0.95476E+00_JPRB, 0.95187E+00_JPRB, 0.97530E+00_JPRB, 0.10308E+01_JPRB, 0.10951E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10042E+01_JPRB, 0.95850E+00_JPRB, 0.93218E+00_JPRB, 0.91258E+00_JPRB, 0.96315E+00_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.88597E+00_JPRB, 0.87121E+00_JPRB, 0.83328E+00_JPRB, 0.79488E+00_JPRB, 0.76572E+00_JPRB, 0.74300E+00_JPRB, 0.72218E+00_JPRB ,&
&  0.69793E+00_JPRB, 0.66950E+00_JPRB, 0.65047E+00_JPRB, 0.64312E+00_JPRB, 0.64142E+00_JPRB, 0.63781E+00_JPRB, 0.62066E+00_JPRB ,&
&  0.60020E+00_JPRB, 0.58592E+00_JPRB, 0.58831E+00_JPRB, 0.59576E+00_JPRB, 0.60904E+00_JPRB, 0.60666E+00_JPRB, 0.61814E+00_JPRB ,&
&  0.62453E+00_JPRB, 0.62263E+00_JPRB, 0.62025E+00_JPRB, 0.61802E+00_JPRB, 0.61299E+00_JPRB, 0.60854E+00_JPRB, 0.58974E+00_JPRB ,&
&  0.62133E+00_JPRB, 0.81368E+00_JPRB, 0.13658E+01_JPRB, 0.17962E+01_JPRB, 0.18007E+01_JPRB, 0.15827E+01_JPRB, 0.14553E+01_JPRB ,&
&  0.13554E+01_JPRB, 0.12815E+01_JPRB, 0.11626E+01_JPRB, 0.10858E+01_JPRB, 0.10160E+01_JPRB, 0.96390E+00_JPRB, 0.92593E+00_JPRB ,&
&  0.90231E+00_JPRB, 0.87859E+00_JPRB, 0.84771E+00_JPRB, 0.82158E+00_JPRB, 0.81049E+00_JPRB, 0.80567E+00_JPRB, 0.80859E+00_JPRB ,&
&  0.81300E+00_JPRB, 0.81026E+00_JPRB, 0.80837E+00_JPRB, 0.82092E+00_JPRB, 0.91733E+00_JPRB, 0.10146E+01_JPRB, 0.14535E+01_JPRB ,&
&  0.19441E+01_JPRB, 0.19088E+01_JPRB, 0.18416E+01_JPRB, 0.17141E+01_JPRB, 0.15647E+01_JPRB, 0.14477E+01_JPRB, 0.13381E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.11650E+01_JPRB, 0.11019E+01_JPRB, 0.10614E+01_JPRB, 0.10341E+01_JPRB, 0.10135E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.98918E+00_JPRB, 0.97499E+00_JPRB, 0.95739E+00_JPRB, 0.93454E+00_JPRB, 0.90129E+00_JPRB, 0.86791E+00_JPRB, 0.84015E+00_JPRB ,&
&  0.81229E+00_JPRB, 0.80896E+00_JPRB, 0.83658E+00_JPRB, 0.83001E+00_JPRB, 0.87027E+00_JPRB, 0.90754E+00_JPRB, 0.91256E+00_JPRB ,&
&  0.90694E+00_JPRB, 0.93923E+00_JPRB, 0.93705E+00_JPRB, 0.94008E+00_JPRB, 0.10002E+01_JPRB, 0.10669E+01_JPRB, 0.10000E+01_JPRB ,&
&  0.94582E+00_JPRB, 0.91550E+00_JPRB, 0.89661E+00_JPRB, 0.87770E+00_JPRB, 0.89132E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.84717E+00_JPRB, 0.81748E+00_JPRB, 0.78522E+00_JPRB, 0.75643E+00_JPRB, 0.73257E+00_JPRB, 0.71643E+00_JPRB, 0.70053E+00_JPRB ,&
&  0.67489E+00_JPRB, 0.65070E+00_JPRB, 0.63969E+00_JPRB, 0.63061E+00_JPRB, 0.62256E+00_JPRB, 0.61635E+00_JPRB, 0.59860E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.56466E+00_JPRB, 0.57328E+00_JPRB, 0.58543E+00_JPRB, 0.59757E+00_JPRB, 0.60764E+00_JPRB, 0.62498E+00_JPRB ,&
&  0.62192E+00_JPRB, 0.62307E+00_JPRB, 0.62608E+00_JPRB, 0.62675E+00_JPRB, 0.61646E+00_JPRB, 0.61612E+00_JPRB, 0.59027E+00_JPRB ,&
&  0.66028E+00_JPRB, 0.73896E+00_JPRB, 0.14792E+01_JPRB, 0.20909E+01_JPRB, 0.19238E+01_JPRB, 0.17097E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.14893E+01_JPRB, 0.13939E+01_JPRB, 0.12484E+01_JPRB, 0.11542E+01_JPRB, 0.10722E+01_JPRB, 0.10083E+01_JPRB, 0.96269E+00_JPRB ,&
&  0.92590E+00_JPRB, 0.88346E+00_JPRB, 0.85810E+00_JPRB, 0.84306E+00_JPRB, 0.84630E+00_JPRB, 0.85192E+00_JPRB, 0.86184E+00_JPRB ,&
&  0.86584E+00_JPRB, 0.86121E+00_JPRB, 0.86261E+00_JPRB, 0.91310E+00_JPRB, 0.10756E+01_JPRB, 0.14536E+01_JPRB, 0.18777E+01_JPRB ,&
&  0.22128E+01_JPRB, 0.24662E+01_JPRB, 0.22547E+01_JPRB, 0.19948E+01_JPRB, 0.17724E+01_JPRB, 0.16082E+01_JPRB, 0.14931E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.13051E+01_JPRB, 0.12465E+01_JPRB, 0.12151E+01_JPRB, 0.11914E+01_JPRB, 0.11654E+01_JPRB, 0.11268E+01_JPRB ,&
&  0.10766E+01_JPRB, 0.10203E+01_JPRB, 0.96633E+00_JPRB, 0.92117E+00_JPRB, 0.88355E+00_JPRB, 0.87148E+00_JPRB, 0.86713E+00_JPRB ,&
&  0.85128E+00_JPRB, 0.85540E+00_JPRB, 0.86632E+00_JPRB, 0.85076E+00_JPRB, 0.87391E+00_JPRB, 0.88311E+00_JPRB, 0.88933E+00_JPRB ,&
&  0.90992E+00_JPRB, 0.91977E+00_JPRB, 0.91088E+00_JPRB, 0.95788E+00_JPRB, 0.95332E+00_JPRB, 0.10186E+01_JPRB, 0.94922E+00_JPRB ,&
&  0.89694E+00_JPRB, 0.88348E+00_JPRB, 0.86687E+00_JPRB, 0.85994E+00_JPRB, 0.86008E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.79889E+00_JPRB, 0.75834E+00_JPRB, 0.72650E+00_JPRB, 0.71239E+00_JPRB, 0.70331E+00_JPRB, 0.68976E+00_JPRB, 0.67815E+00_JPRB ,&
&  0.65696E+00_JPRB, 0.63359E+00_JPRB, 0.61777E+00_JPRB, 0.60364E+00_JPRB, 0.60419E+00_JPRB, 0.59794E+00_JPRB, 0.58751E+00_JPRB ,&
&  0.57437E+00_JPRB, 0.56990E+00_JPRB, 0.57023E+00_JPRB, 0.57973E+00_JPRB, 0.59850E+00_JPRB, 0.60879E+00_JPRB, 0.62348E+00_JPRB ,&
&  0.63240E+00_JPRB, 0.64182E+00_JPRB, 0.64245E+00_JPRB, 0.63734E+00_JPRB, 0.62999E+00_JPRB, 0.61961E+00_JPRB, 0.60657E+00_JPRB ,&
&  0.70013E+00_JPRB, 0.15631E+01_JPRB, 0.13439E+01_JPRB, 0.24306E+01_JPRB, 0.21073E+01_JPRB, 0.18812E+01_JPRB, 0.14748E+01_JPRB ,&
&  0.13626E+01_JPRB, 0.14462E+01_JPRB, 0.13193E+01_JPRB, 0.11814E+01_JPRB, 0.10661E+01_JPRB, 0.96430E+00_JPRB, 0.89682E+00_JPRB ,&
&  0.87727E+00_JPRB, 0.88989E+00_JPRB, 0.90497E+00_JPRB, 0.90098E+00_JPRB, 0.90390E+00_JPRB, 0.90107E+00_JPRB, 0.90610E+00_JPRB ,&
&  0.90923E+00_JPRB, 0.91241E+00_JPRB, 0.94298E+00_JPRB, 0.10821E+01_JPRB, 0.14158E+01_JPRB, 0.20216E+01_JPRB, 0.23233E+01_JPRB ,&
&  0.22717E+01_JPRB, 0.25166E+01_JPRB, 0.22266E+01_JPRB, 0.19205E+01_JPRB, 0.17053E+01_JPRB, 0.15566E+01_JPRB, 0.14460E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.12869E+01_JPRB, 0.12724E+01_JPRB, 0.12594E+01_JPRB, 0.12298E+01_JPRB, 0.11678E+01_JPRB, 0.10846E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.95231E+00_JPRB, 0.90976E+00_JPRB, 0.88727E+00_JPRB, 0.88084E+00_JPRB, 0.88935E+00_JPRB, 0.90291E+00_JPRB ,&
&  0.89969E+00_JPRB, 0.90966E+00_JPRB, 0.89363E+00_JPRB, 0.85506E+00_JPRB, 0.86306E+00_JPRB, 0.85606E+00_JPRB, 0.84235E+00_JPRB ,&
&  0.88201E+00_JPRB, 0.90580E+00_JPRB, 0.91154E+00_JPRB, 0.94678E+00_JPRB, 0.94831E+00_JPRB, 0.10385E+01_JPRB, 0.92085E+00_JPRB ,&
&  0.87497E+00_JPRB, 0.86492E+00_JPRB, 0.86348E+00_JPRB, 0.84485E+00_JPRB, 0.83064E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.74991E+00_JPRB, 0.72598E+00_JPRB, 0.70975E+00_JPRB, 0.71190E+00_JPRB, 0.68914E+00_JPRB, 0.67319E+00_JPRB, 0.66792E+00_JPRB ,&
&  0.63569E+00_JPRB, 0.61492E+00_JPRB, 0.60008E+00_JPRB, 0.59820E+00_JPRB, 0.60080E+00_JPRB, 0.59399E+00_JPRB, 0.59336E+00_JPRB ,&
&  0.58323E+00_JPRB, 0.57449E+00_JPRB, 0.57712E+00_JPRB, 0.59804E+00_JPRB, 0.62604E+00_JPRB, 0.63339E+00_JPRB, 0.64804E+00_JPRB ,&
&  0.66334E+00_JPRB, 0.65867E+00_JPRB, 0.64903E+00_JPRB, 0.64930E+00_JPRB, 0.65578E+00_JPRB, 0.65465E+00_JPRB, 0.67312E+00_JPRB ,&
&  0.73588E+00_JPRB, 0.14293E+01_JPRB, 0.15366E+01_JPRB, 0.26391E+01_JPRB, 0.22919E+01_JPRB, 0.19232E+01_JPRB, 0.14624E+01_JPRB ,&
&  0.13790E+01_JPRB, 0.14978E+01_JPRB, 0.12917E+01_JPRB, 0.11271E+01_JPRB, 0.97463E+00_JPRB, 0.86644E+00_JPRB, 0.84416E+00_JPRB ,&
&  0.89071E+00_JPRB, 0.94975E+00_JPRB, 0.98925E+00_JPRB, 0.97635E+00_JPRB, 0.95110E+00_JPRB, 0.94138E+00_JPRB, 0.94279E+00_JPRB ,&
&  0.95123E+00_JPRB, 0.98638E+00_JPRB, 0.11119E+01_JPRB, 0.13964E+01_JPRB, 0.21573E+01_JPRB, 0.26153E+01_JPRB, 0.23744E+01_JPRB ,&
&  0.21018E+01_JPRB, 0.21434E+01_JPRB, 0.19043E+01_JPRB, 0.16822E+01_JPRB, 0.14665E+01_JPRB, 0.14259E+01_JPRB, 0.13604E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.12870E+01_JPRB, 0.12467E+01_JPRB, 0.11839E+01_JPRB, 0.11152E+01_JPRB, 0.10417E+01_JPRB, 0.96950E+00_JPRB ,&
&  0.92194E+00_JPRB, 0.89844E+00_JPRB, 0.88442E+00_JPRB, 0.89179E+00_JPRB, 0.91598E+00_JPRB, 0.93894E+00_JPRB, 0.95665E+00_JPRB ,&
&  0.95343E+00_JPRB, 0.95801E+00_JPRB, 0.91430E+00_JPRB, 0.88275E+00_JPRB, 0.89028E+00_JPRB, 0.87064E+00_JPRB, 0.86195E+00_JPRB ,&
&  0.90293E+00_JPRB, 0.94034E+00_JPRB, 0.95385E+00_JPRB, 0.95014E+00_JPRB, 0.96058E+00_JPRB, 0.10076E+01_JPRB, 0.91649E+00_JPRB ,&
&  0.87097E+00_JPRB, 0.88833E+00_JPRB, 0.89045E+00_JPRB, 0.82514E+00_JPRB, 0.78801E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.73595E+00_JPRB, 0.73362E+00_JPRB, 0.71884E+00_JPRB, 0.71338E+00_JPRB, 0.67952E+00_JPRB, 0.64664E+00_JPRB, 0.61920E+00_JPRB ,&
&  0.60461E+00_JPRB, 0.59882E+00_JPRB, 0.60694E+00_JPRB, 0.60858E+00_JPRB, 0.60935E+00_JPRB, 0.60117E+00_JPRB, 0.59550E+00_JPRB ,&
&  0.58268E+00_JPRB, 0.58647E+00_JPRB, 0.60092E+00_JPRB, 0.62488E+00_JPRB, 0.65225E+00_JPRB, 0.66252E+00_JPRB, 0.67374E+00_JPRB ,&
&  0.67544E+00_JPRB, 0.67253E+00_JPRB, 0.67585E+00_JPRB, 0.68755E+00_JPRB, 0.70708E+00_JPRB, 0.72865E+00_JPRB, 0.75860E+00_JPRB ,&
&  0.81657E+00_JPRB, 0.13871E+01_JPRB, 0.20468E+01_JPRB, 0.23546E+01_JPRB, 0.20429E+01_JPRB, 0.16799E+01_JPRB, 0.14527E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.11400E+01_JPRB, 0.11726E+01_JPRB, 0.10565E+01_JPRB, 0.93563E+00_JPRB, 0.89357E+00_JPRB, 0.91813E+00_JPRB ,&
&  0.99015E+00_JPRB, 0.10261E+01_JPRB, 0.10429E+01_JPRB, 0.10470E+01_JPRB, 0.10305E+01_JPRB, 0.10237E+01_JPRB, 0.10355E+01_JPRB ,&
&  0.11014E+01_JPRB, 0.12592E+01_JPRB, 0.15723E+01_JPRB, 0.21552E+01_JPRB, 0.29161E+01_JPRB, 0.27071E+01_JPRB, 0.21267E+01_JPRB ,&
&  0.18462E+01_JPRB, 0.16968E+01_JPRB, 0.15495E+01_JPRB, 0.14503E+01_JPRB, 0.12262E+01_JPRB, 0.13559E+01_JPRB, 0.12934E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.11858E+01_JPRB, 0.11233E+01_JPRB, 0.10640E+01_JPRB, 0.10127E+01_JPRB, 0.96172E+00_JPRB, 0.92561E+00_JPRB ,&
&  0.91427E+00_JPRB, 0.91575E+00_JPRB, 0.92008E+00_JPRB, 0.93138E+00_JPRB, 0.94963E+00_JPRB, 0.96871E+00_JPRB, 0.97762E+00_JPRB ,&
&  0.97465E+00_JPRB, 0.98256E+00_JPRB, 0.96429E+00_JPRB, 0.97235E+00_JPRB, 0.97114E+00_JPRB, 0.94750E+00_JPRB, 0.92736E+00_JPRB ,&
&  0.95044E+00_JPRB, 0.99650E+00_JPRB, 0.99055E+00_JPRB, 0.98816E+00_JPRB, 0.98683E+00_JPRB, 0.99530E+00_JPRB, 0.90919E+00_JPRB ,&
&  0.88368E+00_JPRB, 0.93846E+00_JPRB, 0.88084E+00_JPRB, 0.79807E+00_JPRB, 0.76025E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.72787E+00_JPRB, 0.73258E+00_JPRB, 0.72321E+00_JPRB, 0.68436E+00_JPRB, 0.64867E+00_JPRB, 0.61238E+00_JPRB, 0.58818E+00_JPRB ,&
&  0.58596E+00_JPRB, 0.59608E+00_JPRB, 0.62652E+00_JPRB, 0.64619E+00_JPRB, 0.63688E+00_JPRB, 0.61802E+00_JPRB, 0.61191E+00_JPRB ,&
&  0.61757E+00_JPRB, 0.63395E+00_JPRB, 0.64594E+00_JPRB, 0.66727E+00_JPRB, 0.69070E+00_JPRB, 0.71115E+00_JPRB, 0.73359E+00_JPRB ,&
&  0.72383E+00_JPRB, 0.72891E+00_JPRB, 0.74359E+00_JPRB, 0.75569E+00_JPRB, 0.77588E+00_JPRB, 0.82258E+00_JPRB, 0.87010E+00_JPRB ,&
&  0.10333E+01_JPRB, 0.15790E+01_JPRB, 0.97636E+00_JPRB, 0.13555E+01_JPRB, 0.15620E+01_JPRB, 0.15580E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.10411E+01_JPRB, 0.10425E+01_JPRB, 0.10972E+01_JPRB, 0.10394E+01_JPRB, 0.10574E+01_JPRB, 0.10854E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.11208E+01_JPRB, 0.11356E+01_JPRB, 0.11480E+01_JPRB, 0.11495E+01_JPRB, 0.11734E+01_JPRB, 0.12635E+01_JPRB ,&
&  0.14578E+01_JPRB, 0.17374E+01_JPRB, 0.22233E+01_JPRB, 0.26441E+01_JPRB, 0.32805E+01_JPRB, 0.26155E+01_JPRB, 0.18970E+01_JPRB ,&
&  0.13893E+01_JPRB, 0.13522E+01_JPRB, 0.13643E+01_JPRB, 0.13334E+01_JPRB, 0.12083E+01_JPRB, 0.11370E+01_JPRB, 0.11740E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.10514E+01_JPRB, 0.10279E+01_JPRB, 0.10040E+01_JPRB, 0.97175E+00_JPRB, 0.94945E+00_JPRB, 0.93732E+00_JPRB ,&
&  0.92888E+00_JPRB, 0.93119E+00_JPRB, 0.94095E+00_JPRB, 0.94827E+00_JPRB, 0.96139E+00_JPRB, 0.97054E+00_JPRB, 0.98538E+00_JPRB ,&
&  0.10028E+01_JPRB, 0.10320E+01_JPRB, 0.10661E+01_JPRB, 0.10775E+01_JPRB, 0.10694E+01_JPRB, 0.10475E+01_JPRB, 0.10418E+01_JPRB ,&
&  0.10541E+01_JPRB, 0.10828E+01_JPRB, 0.10072E+01_JPRB, 0.99021E+00_JPRB, 0.10267E+01_JPRB, 0.97936E+00_JPRB, 0.91153E+00_JPRB ,&
&  0.94766E+00_JPRB, 0.10920E+01_JPRB, 0.11315E+01_JPRB, 0.77501E+00_JPRB, 0.73883E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.68224E+00_JPRB, 0.66703E+00_JPRB, 0.63741E+00_JPRB, 0.59940E+00_JPRB, 0.59482E+00_JPRB, 0.59180E+00_JPRB, 0.60412E+00_JPRB ,&
&  0.64342E+00_JPRB, 0.69504E+00_JPRB, 0.70691E+00_JPRB, 0.70161E+00_JPRB, 0.68376E+00_JPRB, 0.68160E+00_JPRB, 0.68287E+00_JPRB ,&
&  0.69131E+00_JPRB, 0.69661E+00_JPRB, 0.71647E+00_JPRB, 0.74232E+00_JPRB, 0.76996E+00_JPRB, 0.79165E+00_JPRB, 0.79935E+00_JPRB ,&
&  0.80873E+00_JPRB, 0.81569E+00_JPRB, 0.83641E+00_JPRB, 0.86637E+00_JPRB, 0.92868E+00_JPRB, 0.10565E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.17565E+01_JPRB, 0.14341E+01_JPRB, 0.73083E+00_JPRB, 0.13170E+01_JPRB, 0.14965E+01_JPRB, 0.16018E+01_JPRB, 0.13698E+01_JPRB ,&
&  0.12836E+01_JPRB, 0.11549E+01_JPRB, 0.11311E+01_JPRB, 0.12605E+01_JPRB, 0.12754E+01_JPRB, 0.13030E+01_JPRB, 0.13096E+01_JPRB ,&
&  0.12805E+01_JPRB, 0.12862E+01_JPRB, 0.12903E+01_JPRB, 0.13051E+01_JPRB, 0.13418E+01_JPRB, 0.14366E+01_JPRB, 0.16226E+01_JPRB ,&
&  0.18648E+01_JPRB, 0.21854E+01_JPRB, 0.26374E+01_JPRB, 0.28550E+01_JPRB, 0.31813E+01_JPRB, 0.24307E+01_JPRB, 0.18893E+01_JPRB ,&
&  0.12751E+01_JPRB, 0.12009E+01_JPRB, 0.12679E+01_JPRB, 0.12663E+01_JPRB, 0.11671E+01_JPRB, 0.10320E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.10101E+01_JPRB, 0.99497E+00_JPRB, 0.96837E+00_JPRB, 0.94894E+00_JPRB, 0.94088E+00_JPRB, 0.93753E+00_JPRB ,&
&  0.93368E+00_JPRB, 0.94131E+00_JPRB, 0.94639E+00_JPRB, 0.94949E+00_JPRB, 0.95520E+00_JPRB, 0.95894E+00_JPRB, 0.98766E+00_JPRB ,&
&  0.10152E+01_JPRB, 0.10406E+01_JPRB, 0.10742E+01_JPRB, 0.11025E+01_JPRB, 0.10872E+01_JPRB, 0.10452E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10226E+01_JPRB, 0.98991E+00_JPRB, 0.10069E+01_JPRB, 0.96317E+00_JPRB, 0.89988E+00_JPRB, 0.83203E+00_JPRB ,&
&  0.82727E+00_JPRB, 0.81788E+00_JPRB, 0.77260E+00_JPRB, 0.71305E+00_JPRB, 0.68847E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.64669E+00_JPRB, 0.62837E+00_JPRB, 0.61442E+00_JPRB, 0.62039E+00_JPRB, 0.62977E+00_JPRB, 0.64904E+00_JPRB, 0.69872E+00_JPRB ,&
&  0.73880E+00_JPRB, 0.75650E+00_JPRB, 0.77850E+00_JPRB, 0.78007E+00_JPRB, 0.79052E+00_JPRB, 0.77644E+00_JPRB, 0.77550E+00_JPRB ,&
&  0.76637E+00_JPRB, 0.77982E+00_JPRB, 0.80222E+00_JPRB, 0.82346E+00_JPRB, 0.84023E+00_JPRB, 0.85733E+00_JPRB, 0.88140E+00_JPRB ,&
&  0.91134E+00_JPRB, 0.95287E+00_JPRB, 0.10180E+01_JPRB, 0.11176E+01_JPRB, 0.12746E+01_JPRB, 0.14673E+01_JPRB, 0.15961E+01_JPRB ,&
&  0.12076E+01_JPRB, 0.56028E+00_JPRB, 0.10529E+01_JPRB, 0.15077E+01_JPRB, 0.15805E+01_JPRB, 0.16377E+01_JPRB, 0.15634E+01_JPRB ,&
&  0.14251E+01_JPRB, 0.13352E+01_JPRB, 0.12968E+01_JPRB, 0.14776E+01_JPRB, 0.14770E+01_JPRB, 0.14794E+01_JPRB, 0.14774E+01_JPRB ,&
&  0.14669E+01_JPRB, 0.14676E+01_JPRB, 0.14778E+01_JPRB, 0.15175E+01_JPRB, 0.16003E+01_JPRB, 0.17366E+01_JPRB, 0.19227E+01_JPRB ,&
&  0.21817E+01_JPRB, 0.25295E+01_JPRB, 0.28529E+01_JPRB, 0.26827E+01_JPRB, 0.28122E+01_JPRB, 0.25238E+01_JPRB, 0.22371E+01_JPRB ,&
&  0.14096E+01_JPRB, 0.10811E+01_JPRB, 0.11128E+01_JPRB, 0.12360E+01_JPRB, 0.11395E+01_JPRB, 0.98081E+00_JPRB, 0.10327E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.97416E+00_JPRB, 0.95863E+00_JPRB, 0.93341E+00_JPRB, 0.92657E+00_JPRB, 0.92573E+00_JPRB, 0.92975E+00_JPRB ,&
&  0.92914E+00_JPRB, 0.93533E+00_JPRB, 0.93998E+00_JPRB, 0.93759E+00_JPRB, 0.92215E+00_JPRB, 0.92423E+00_JPRB, 0.94752E+00_JPRB ,&
&  0.96810E+00_JPRB, 0.97528E+00_JPRB, 0.10025E+01_JPRB, 0.10292E+01_JPRB, 0.10544E+01_JPRB, 0.10294E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.99067E+00_JPRB, 0.97726E+00_JPRB, 0.97032E+00_JPRB, 0.96300E+00_JPRB, 0.95374E+00_JPRB, 0.88965E+00_JPRB, 0.80496E+00_JPRB ,&
&  0.77595E+00_JPRB, 0.74310E+00_JPRB, 0.71802E+00_JPRB, 0.69669E+00_JPRB, 0.67327E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.70670E+00_JPRB, 0.71736E+00_JPRB, 0.71085E+00_JPRB, 0.73385E+00_JPRB, 0.75378E+00_JPRB, 0.78385E+00_JPRB, 0.81281E+00_JPRB ,&
&  0.83783E+00_JPRB, 0.86175E+00_JPRB, 0.89303E+00_JPRB, 0.87334E+00_JPRB, 0.88456E+00_JPRB, 0.87600E+00_JPRB, 0.85559E+00_JPRB ,&
&  0.85478E+00_JPRB, 0.87231E+00_JPRB, 0.89111E+00_JPRB, 0.90971E+00_JPRB, 0.93128E+00_JPRB, 0.97044E+00_JPRB, 0.10197E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.11615E+01_JPRB, 0.12670E+01_JPRB, 0.13601E+01_JPRB, 0.14529E+01_JPRB, 0.14910E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.71746E+00_JPRB, 0.91597E+00_JPRB, 0.14498E+01_JPRB, 0.16548E+01_JPRB, 0.17699E+01_JPRB, 0.17941E+01_JPRB, 0.17583E+01_JPRB ,&
&  0.16231E+01_JPRB, 0.14409E+01_JPRB, 0.14381E+01_JPRB, 0.16384E+01_JPRB, 0.15925E+01_JPRB, 0.15990E+01_JPRB, 0.15958E+01_JPRB ,&
&  0.15993E+01_JPRB, 0.16300E+01_JPRB, 0.16813E+01_JPRB, 0.17419E+01_JPRB, 0.18492E+01_JPRB, 0.19912E+01_JPRB, 0.21907E+01_JPRB ,&
&  0.24408E+01_JPRB, 0.26934E+01_JPRB, 0.29205E+01_JPRB, 0.26252E+01_JPRB, 0.26014E+01_JPRB, 0.25584E+01_JPRB, 0.27558E+01_JPRB ,&
&  0.15234E+01_JPRB, 0.10140E+01_JPRB, 0.10758E+01_JPRB, 0.11857E+01_JPRB, 0.10937E+01_JPRB, 0.10319E+01_JPRB, 0.99811E+00_JPRB ,&
&  0.97926E+00_JPRB, 0.93312E+00_JPRB, 0.92121E+00_JPRB, 0.90046E+00_JPRB, 0.89093E+00_JPRB, 0.90022E+00_JPRB, 0.91996E+00_JPRB ,&
&  0.91838E+00_JPRB, 0.90850E+00_JPRB, 0.90736E+00_JPRB, 0.91198E+00_JPRB, 0.89433E+00_JPRB, 0.87438E+00_JPRB, 0.86752E+00_JPRB ,&
&  0.87227E+00_JPRB, 0.86021E+00_JPRB, 0.87002E+00_JPRB, 0.90338E+00_JPRB, 0.94685E+00_JPRB, 0.96940E+00_JPRB, 0.97562E+00_JPRB ,&
&  0.95571E+00_JPRB, 0.96992E+00_JPRB, 0.10983E+01_JPRB, 0.11482E+01_JPRB, 0.11464E+01_JPRB, 0.96852E+00_JPRB, 0.80128E+00_JPRB ,&
&  0.78098E+00_JPRB, 0.75183E+00_JPRB, 0.72510E+00_JPRB, 0.71607E+00_JPRB, 0.70664E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.78121E+00_JPRB, 0.79674E+00_JPRB, 0.81883E+00_JPRB, 0.85387E+00_JPRB, 0.89282E+00_JPRB, 0.92021E+00_JPRB, 0.94749E+00_JPRB ,&
&  0.95814E+00_JPRB, 0.97367E+00_JPRB, 0.98767E+00_JPRB, 0.94778E+00_JPRB, 0.96034E+00_JPRB, 0.94634E+00_JPRB, 0.94293E+00_JPRB ,&
&  0.94292E+00_JPRB, 0.96484E+00_JPRB, 0.99176E+00_JPRB, 0.10187E+01_JPRB, 0.10583E+01_JPRB, 0.11078E+01_JPRB, 0.11760E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.13378E+01_JPRB, 0.14251E+01_JPRB, 0.14767E+01_JPRB, 0.15250E+01_JPRB, 0.15521E+01_JPRB, 0.15354E+01_JPRB ,&
&  0.93388E+00_JPRB, 0.15529E+01_JPRB, 0.17231E+01_JPRB, 0.18284E+01_JPRB, 0.18858E+01_JPRB, 0.17734E+01_JPRB, 0.17595E+01_JPRB ,&
&  0.16717E+01_JPRB, 0.15473E+01_JPRB, 0.15561E+01_JPRB, 0.16377E+01_JPRB, 0.17449E+01_JPRB, 0.17362E+01_JPRB, 0.17497E+01_JPRB ,&
&  0.17682E+01_JPRB, 0.18208E+01_JPRB, 0.18908E+01_JPRB, 0.19703E+01_JPRB, 0.20737E+01_JPRB, 0.21953E+01_JPRB, 0.23628E+01_JPRB ,&
&  0.25265E+01_JPRB, 0.26814E+01_JPRB, 0.28075E+01_JPRB, 0.25223E+01_JPRB, 0.22251E+01_JPRB, 0.21910E+01_JPRB, 0.18237E+01_JPRB ,&
&  0.11583E+01_JPRB, 0.91003E+00_JPRB, 0.98460E+00_JPRB, 0.11080E+01_JPRB, 0.10471E+01_JPRB, 0.96122E+00_JPRB, 0.89205E+00_JPRB ,&
&  0.86443E+00_JPRB, 0.82756E+00_JPRB, 0.84590E+00_JPRB, 0.81938E+00_JPRB, 0.84714E+00_JPRB, 0.84482E+00_JPRB, 0.85469E+00_JPRB ,&
&  0.86431E+00_JPRB, 0.83943E+00_JPRB, 0.82656E+00_JPRB, 0.82147E+00_JPRB, 0.81064E+00_JPRB, 0.79717E+00_JPRB, 0.78495E+00_JPRB ,&
&  0.78873E+00_JPRB, 0.74892E+00_JPRB, 0.72121E+00_JPRB, 0.73702E+00_JPRB, 0.77746E+00_JPRB, 0.82168E+00_JPRB, 0.91186E+00_JPRB ,&
&  0.10659E+01_JPRB, 0.12312E+01_JPRB, 0.15168E+01_JPRB, 0.16048E+01_JPRB, 0.15606E+01_JPRB, 0.11786E+01_JPRB, 0.86856E+00_JPRB ,&
&  0.86259E+00_JPRB, 0.81865E+00_JPRB, 0.78414E+00_JPRB, 0.76459E+00_JPRB, 0.76798E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.93407E+00_JPRB, 0.95819E+00_JPRB, 0.98471E+00_JPRB, 0.10081E+01_JPRB, 0.10159E+01_JPRB, 0.10187E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.10446E+01_JPRB, 0.10249E+01_JPRB, 0.10506E+01_JPRB, 0.10294E+01_JPRB, 0.10521E+01_JPRB, 0.10386E+01_JPRB, 0.10474E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10976E+01_JPRB, 0.11359E+01_JPRB, 0.11561E+01_JPRB, 0.11643E+01_JPRB, 0.12064E+01_JPRB, 0.12672E+01_JPRB ,&
&  0.13386E+01_JPRB, 0.13776E+01_JPRB, 0.14245E+01_JPRB, 0.14478E+01_JPRB, 0.14213E+01_JPRB, 0.13991E+01_JPRB, 0.10118E+01_JPRB ,&
&  0.14573E+01_JPRB, 0.17258E+01_JPRB, 0.19019E+01_JPRB, 0.18738E+01_JPRB, 0.17424E+01_JPRB, 0.16392E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.16576E+01_JPRB, 0.16393E+01_JPRB, 0.16125E+01_JPRB, 0.16501E+01_JPRB, 0.18586E+01_JPRB, 0.18199E+01_JPRB, 0.18605E+01_JPRB ,&
&  0.19012E+01_JPRB, 0.19510E+01_JPRB, 0.20117E+01_JPRB, 0.20830E+01_JPRB, 0.21487E+01_JPRB, 0.22179E+01_JPRB, 0.22966E+01_JPRB ,&
&  0.23443E+01_JPRB, 0.23527E+01_JPRB, 0.23625E+01_JPRB, 0.20956E+01_JPRB, 0.18914E+01_JPRB, 0.18142E+01_JPRB, 0.13840E+01_JPRB ,&
&  0.92992E+00_JPRB, 0.85094E+00_JPRB, 0.94085E+00_JPRB, 0.10197E+01_JPRB, 0.98422E+00_JPRB, 0.88506E+00_JPRB, 0.82577E+00_JPRB ,&
&  0.79015E+00_JPRB, 0.76980E+00_JPRB, 0.75084E+00_JPRB, 0.76611E+00_JPRB, 0.77100E+00_JPRB, 0.78746E+00_JPRB, 0.79819E+00_JPRB ,&
&  0.81021E+00_JPRB, 0.78860E+00_JPRB, 0.76224E+00_JPRB, 0.74921E+00_JPRB, 0.76354E+00_JPRB, 0.83629E+00_JPRB, 0.89160E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.58217E+00_JPRB, 0.63230E+00_JPRB, 0.61949E+00_JPRB, 0.64108E+00_JPRB, 0.71210E+00_JPRB, 0.89295E+00_JPRB ,&
&  0.13517E+01_JPRB, 0.20858E+01_JPRB, 0.23547E+01_JPRB, 0.20180E+01_JPRB, 0.24892E+01_JPRB, 0.17850E+01_JPRB, 0.99688E+00_JPRB ,&
&  0.97711E+00_JPRB, 0.94700E+00_JPRB, 0.90952E+00_JPRB, 0.89857E+00_JPRB, 0.90759E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.10016E+01_JPRB, 0.10183E+01_JPRB, 0.10188E+01_JPRB, 0.10350E+01_JPRB, 0.10157E+01_JPRB, 0.10341E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10669E+01_JPRB, 0.10870E+01_JPRB, 0.11046E+01_JPRB, 0.11329E+01_JPRB, 0.11583E+01_JPRB, 0.11789E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12372E+01_JPRB, 0.12627E+01_JPRB, 0.12851E+01_JPRB, 0.12726E+01_JPRB, 0.12843E+01_JPRB, 0.12919E+01_JPRB ,&
&  0.13233E+01_JPRB, 0.13169E+01_JPRB, 0.13287E+01_JPRB, 0.13358E+01_JPRB, 0.13205E+01_JPRB, 0.13570E+01_JPRB, 0.99816E+00_JPRB ,&
&  0.15107E+01_JPRB, 0.17181E+01_JPRB, 0.17976E+01_JPRB, 0.17027E+01_JPRB, 0.15972E+01_JPRB, 0.15739E+01_JPRB, 0.15869E+01_JPRB ,&
&  0.16419E+01_JPRB, 0.16844E+01_JPRB, 0.16528E+01_JPRB, 0.17307E+01_JPRB, 0.17652E+01_JPRB, 0.17223E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.17810E+01_JPRB, 0.18063E+01_JPRB, 0.18289E+01_JPRB, 0.18355E+01_JPRB, 0.18274E+01_JPRB, 0.18603E+01_JPRB, 0.18628E+01_JPRB ,&
&  0.18341E+01_JPRB, 0.18262E+01_JPRB, 0.18015E+01_JPRB, 0.16790E+01_JPRB, 0.15282E+01_JPRB, 0.12671E+01_JPRB, 0.10317E+01_JPRB ,&
&  0.79091E+00_JPRB, 0.76776E+00_JPRB, 0.85843E+00_JPRB, 0.99747E+00_JPRB, 0.96161E+00_JPRB, 0.90163E+00_JPRB, 0.83624E+00_JPRB ,&
&  0.76374E+00_JPRB, 0.74626E+00_JPRB, 0.72436E+00_JPRB, 0.72114E+00_JPRB, 0.72489E+00_JPRB, 0.74170E+00_JPRB, 0.75181E+00_JPRB ,&
&  0.76147E+00_JPRB, 0.76649E+00_JPRB, 0.74454E+00_JPRB, 0.72006E+00_JPRB, 0.71389E+00_JPRB, 0.74551E+00_JPRB, 0.65606E+00_JPRB ,&
&  0.72570E+00_JPRB, 0.61248E+00_JPRB, 0.60294E+00_JPRB, 0.57764E+00_JPRB, 0.53793E+00_JPRB, 0.57454E+00_JPRB, 0.75445E+00_JPRB ,&
&  0.12510E+01_JPRB, 0.16229E+01_JPRB, 0.23320E+01_JPRB, 0.39564E+01_JPRB, 0.27946E+01_JPRB, 0.20668E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.10008E+01_JPRB, 0.97807E+00_JPRB, 0.96615E+00_JPRB, 0.95734E+00_JPRB, 0.97735E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.96962E+00_JPRB, 0.98282E+00_JPRB, 0.99562E+00_JPRB, 0.10091E+01_JPRB, 0.10195E+01_JPRB, 0.10392E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10671E+01_JPRB, 0.10839E+01_JPRB, 0.11046E+01_JPRB, 0.11223E+01_JPRB, 0.11481E+01_JPRB, 0.11790E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.12418E+01_JPRB, 0.12730E+01_JPRB, 0.12878E+01_JPRB, 0.12810E+01_JPRB, 0.12892E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.12836E+01_JPRB, 0.12747E+01_JPRB, 0.12883E+01_JPRB, 0.13084E+01_JPRB, 0.13050E+01_JPRB, 0.13208E+01_JPRB, 0.10242E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.16372E+01_JPRB, 0.16717E+01_JPRB, 0.15447E+01_JPRB, 0.14649E+01_JPRB, 0.14730E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.15799E+01_JPRB, 0.16486E+01_JPRB, 0.16083E+01_JPRB, 0.15198E+01_JPRB, 0.14541E+01_JPRB, 0.14476E+01_JPRB, 0.14494E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.14618E+01_JPRB, 0.14497E+01_JPRB, 0.14207E+01_JPRB, 0.13927E+01_JPRB, 0.13648E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.12625E+01_JPRB, 0.12230E+01_JPRB, 0.10807E+01_JPRB, 0.11169E+01_JPRB, 0.10271E+01_JPRB, 0.85750E+00_JPRB, 0.69977E+00_JPRB ,&
&  0.61220E+00_JPRB, 0.64440E+00_JPRB, 0.80063E+00_JPRB, 0.10001E+01_JPRB, 0.98458E+00_JPRB, 0.93364E+00_JPRB, 0.89579E+00_JPRB ,&
&  0.82999E+00_JPRB, 0.76665E+00_JPRB, 0.72226E+00_JPRB, 0.70255E+00_JPRB, 0.69883E+00_JPRB, 0.71610E+00_JPRB, 0.73036E+00_JPRB ,&
&  0.73446E+00_JPRB, 0.73520E+00_JPRB, 0.71948E+00_JPRB, 0.70458E+00_JPRB, 0.69502E+00_JPRB, 0.65261E+00_JPRB, 0.78770E+00_JPRB ,&
&  0.71377E+00_JPRB, 0.56153E+00_JPRB, 0.63815E+00_JPRB, 0.41620E+00_JPRB, 0.52643E+00_JPRB, 0.54781E+00_JPRB, 0.65104E+00_JPRB ,&
&  0.78395E+00_JPRB, 0.11041E+01_JPRB, 0.15254E+01_JPRB, 0.13365E+01_JPRB, 0.12068E+01_JPRB, 0.10968E+01_JPRB, 0.10048E+01_JPRB ,&
&  0.96814E+00_JPRB, 0.95565E+00_JPRB, 0.95241E+00_JPRB, 0.94815E+00_JPRB, 0.95606E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.95203E+00_JPRB, 0.96298E+00_JPRB, 0.97367E+00_JPRB, 0.98896E+00_JPRB, 0.10030E+01_JPRB, 0.10212E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.10531E+01_JPRB, 0.10539E+01_JPRB, 0.10676E+01_JPRB, 0.10799E+01_JPRB, 0.11036E+01_JPRB, 0.11316E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.11946E+01_JPRB, 0.12310E+01_JPRB, 0.12525E+01_JPRB, 0.12700E+01_JPRB, 0.12905E+01_JPRB, 0.13079E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.13219E+01_JPRB, 0.13401E+01_JPRB, 0.13721E+01_JPRB, 0.14092E+01_JPRB, 0.14597E+01_JPRB, 0.12535E+01_JPRB ,&
&  0.14354E+01_JPRB, 0.15403E+01_JPRB, 0.14904E+01_JPRB, 0.13610E+01_JPRB, 0.13133E+01_JPRB, 0.12825E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12742E+01_JPRB, 0.12329E+01_JPRB, 0.11583E+01_JPRB, 0.11214E+01_JPRB, 0.11409E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.11898E+01_JPRB, 0.12024E+01_JPRB, 0.11907E+01_JPRB, 0.11611E+01_JPRB, 0.11449E+01_JPRB, 0.11238E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.10174E+01_JPRB, 0.86726E+00_JPRB, 0.85294E+00_JPRB, 0.84005E+00_JPRB, 0.76286E+00_JPRB, 0.65230E+00_JPRB ,&
&  0.59183E+00_JPRB, 0.58346E+00_JPRB, 0.66173E+00_JPRB, 0.97982E+00_JPRB, 0.10702E+01_JPRB, 0.10344E+01_JPRB, 0.98088E+00_JPRB ,&
&  0.91590E+00_JPRB, 0.85688E+00_JPRB, 0.77095E+00_JPRB, 0.71842E+00_JPRB, 0.69278E+00_JPRB, 0.70647E+00_JPRB, 0.71253E+00_JPRB ,&
&  0.70885E+00_JPRB, 0.69814E+00_JPRB, 0.68637E+00_JPRB, 0.67596E+00_JPRB, 0.71008E+00_JPRB, 0.72583E+00_JPRB, 0.95798E+00_JPRB ,&
&  0.75920E+00_JPRB, 0.51853E+00_JPRB, 0.51887E+00_JPRB, 0.65823E+00_JPRB, 0.58840E+00_JPRB, 0.61828E+00_JPRB, 0.72710E+00_JPRB ,&
&  0.93356E+00_JPRB, 0.10538E+01_JPRB, 0.10376E+01_JPRB, 0.10081E+01_JPRB, 0.95393E+00_JPRB, 0.95346E+00_JPRB, 0.93356E+00_JPRB ,&
&  0.91894E+00_JPRB, 0.91897E+00_JPRB, 0.92330E+00_JPRB, 0.92872E+00_JPRB, 0.93924E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.91501E+00_JPRB, 0.93090E+00_JPRB, 0.94425E+00_JPRB, 0.95392E+00_JPRB, 0.95880E+00_JPRB, 0.96307E+00_JPRB, 0.96961E+00_JPRB ,&
&  0.97165E+00_JPRB, 0.96921E+00_JPRB, 0.96757E+00_JPRB, 0.98278E+00_JPRB, 0.10032E+01_JPRB, 0.10132E+01_JPRB, 0.10366E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.10455E+01_JPRB, 0.10656E+01_JPRB, 0.10815E+01_JPRB, 0.11078E+01_JPRB, 0.11100E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10856E+01_JPRB, 0.10885E+01_JPRB, 0.10917E+01_JPRB, 0.11049E+01_JPRB, 0.11227E+01_JPRB, 0.12990E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.14989E+01_JPRB, 0.13482E+01_JPRB, 0.11876E+01_JPRB, 0.11715E+01_JPRB, 0.10776E+01_JPRB, 0.10297E+01_JPRB ,&
&  0.96460E+00_JPRB, 0.89603E+00_JPRB, 0.85967E+00_JPRB, 0.83961E+00_JPRB, 0.87038E+00_JPRB, 0.90980E+00_JPRB, 0.92694E+00_JPRB ,&
&  0.97407E+00_JPRB, 0.99443E+00_JPRB, 0.10091E+01_JPRB, 0.10111E+01_JPRB, 0.10021E+01_JPRB, 0.10451E+01_JPRB, 0.10102E+01_JPRB ,&
&  0.10394E+01_JPRB, 0.11105E+01_JPRB, 0.10380E+01_JPRB, 0.10691E+01_JPRB, 0.10513E+01_JPRB, 0.99814E+00_JPRB, 0.90633E+00_JPRB ,&
&  0.80608E+00_JPRB, 0.79261E+00_JPRB, 0.90157E+00_JPRB, 0.10262E+01_JPRB, 0.11111E+01_JPRB, 0.11438E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.96703E+00_JPRB, 0.91462E+00_JPRB, 0.84552E+00_JPRB, 0.77339E+00_JPRB, 0.73759E+00_JPRB, 0.72950E+00_JPRB, 0.71584E+00_JPRB ,&
&  0.69231E+00_JPRB, 0.67885E+00_JPRB, 0.67267E+00_JPRB, 0.68229E+00_JPRB, 0.66154E+00_JPRB, 0.93029E+00_JPRB, 0.10991E+01_JPRB ,&
&  0.78962E+00_JPRB, 0.67640E+00_JPRB, 0.53033E+00_JPRB, 0.81307E+00_JPRB, 0.65931E+00_JPRB, 0.66306E+00_JPRB, 0.73194E+00_JPRB ,&
&  0.82745E+00_JPRB, 0.89896E+00_JPRB, 0.92433E+00_JPRB, 0.95201E+00_JPRB, 0.85159E+00_JPRB, 0.83320E+00_JPRB, 0.82846E+00_JPRB ,&
&  0.83655E+00_JPRB, 0.85087E+00_JPRB, 0.86605E+00_JPRB, 0.88031E+00_JPRB, 0.89814E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.84655E+00_JPRB, 0.86506E+00_JPRB, 0.88750E+00_JPRB, 0.89053E+00_JPRB, 0.89722E+00_JPRB, 0.89294E+00_JPRB, 0.89281E+00_JPRB ,&
&  0.88313E+00_JPRB, 0.88127E+00_JPRB, 0.88364E+00_JPRB, 0.88877E+00_JPRB, 0.89865E+00_JPRB, 0.91129E+00_JPRB, 0.94249E+00_JPRB ,&
&  0.94439E+00_JPRB, 0.95088E+00_JPRB, 0.95980E+00_JPRB, 0.97726E+00_JPRB, 0.99549E+00_JPRB, 0.99749E+00_JPRB, 0.97554E+00_JPRB ,&
&  0.96161E+00_JPRB, 0.96495E+00_JPRB, 0.98376E+00_JPRB, 0.10181E+01_JPRB, 0.11936E+01_JPRB, 0.15072E+01_JPRB, 0.20080E+01_JPRB ,&
&  0.18485E+01_JPRB, 0.14585E+01_JPRB, 0.12657E+01_JPRB, 0.10847E+01_JPRB, 0.10677E+01_JPRB, 0.10814E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.99794E+00_JPRB, 0.95104E+00_JPRB, 0.93501E+00_JPRB, 0.94470E+00_JPRB, 0.95550E+00_JPRB, 0.93552E+00_JPRB, 0.91849E+00_JPRB ,&
&  0.86161E+00_JPRB, 0.84219E+00_JPRB, 0.86763E+00_JPRB, 0.93475E+00_JPRB, 0.10912E+01_JPRB, 0.12000E+01_JPRB, 0.11669E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.13502E+01_JPRB, 0.13604E+01_JPRB, 0.13999E+01_JPRB, 0.14076E+01_JPRB, 0.13593E+01_JPRB, 0.13107E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.11827E+01_JPRB, 0.96952E+00_JPRB, 0.10937E+01_JPRB, 0.11602E+01_JPRB, 0.12278E+01_JPRB, 0.11271E+01_JPRB ,&
&  0.10048E+01_JPRB, 0.92923E+00_JPRB, 0.86412E+00_JPRB, 0.79440E+00_JPRB, 0.76053E+00_JPRB, 0.77768E+00_JPRB, 0.77037E+00_JPRB ,&
&  0.72796E+00_JPRB, 0.72764E+00_JPRB, 0.73647E+00_JPRB, 0.74516E+00_JPRB, 0.81643E+00_JPRB, 0.75671E+00_JPRB, 0.87207E+00_JPRB ,&
&  0.78611E+00_JPRB, 0.79265E+00_JPRB, 0.72209E+00_JPRB, 0.11253E+01_JPRB, 0.76651E+00_JPRB, 0.68134E+00_JPRB, 0.70443E+00_JPRB ,&
&  0.74039E+00_JPRB, 0.75651E+00_JPRB, 0.78056E+00_JPRB, 0.80023E+00_JPRB, 0.79577E+00_JPRB, 0.78888E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.81247E+00_JPRB, 0.81449E+00_JPRB, 0.81859E+00_JPRB, 0.82004E+00_JPRB, 0.83261E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.74252E+00_JPRB, 0.75092E+00_JPRB, 0.76648E+00_JPRB, 0.77897E+00_JPRB, 0.80138E+00_JPRB, 0.79378E+00_JPRB, 0.78618E+00_JPRB ,&
&  0.75884E+00_JPRB, 0.75456E+00_JPRB, 0.74842E+00_JPRB, 0.74663E+00_JPRB, 0.75224E+00_JPRB, 0.77595E+00_JPRB, 0.82102E+00_JPRB ,&
&  0.83501E+00_JPRB, 0.83999E+00_JPRB, 0.83428E+00_JPRB, 0.82493E+00_JPRB, 0.79381E+00_JPRB, 0.76791E+00_JPRB, 0.75520E+00_JPRB ,&
&  0.79389E+00_JPRB, 0.10231E+01_JPRB, 0.13160E+01_JPRB, 0.15866E+01_JPRB, 0.19167E+01_JPRB, 0.17872E+01_JPRB, 0.20520E+01_JPRB ,&
&  0.21971E+01_JPRB, 0.15266E+01_JPRB, 0.14500E+01_JPRB, 0.12582E+01_JPRB, 0.12338E+01_JPRB, 0.12109E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12572E+01_JPRB, 0.12496E+01_JPRB, 0.12521E+01_JPRB, 0.12755E+01_JPRB, 0.13061E+01_JPRB, 0.13205E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.12776E+01_JPRB, 0.12716E+01_JPRB, 0.13766E+01_JPRB, 0.14670E+01_JPRB, 0.15191E+01_JPRB, 0.15769E+01_JPRB ,&
&  0.16434E+01_JPRB, 0.16685E+01_JPRB, 0.15589E+01_JPRB, 0.15665E+01_JPRB, 0.16554E+01_JPRB, 0.15333E+01_JPRB, 0.15538E+01_JPRB ,&
&  0.16151E+01_JPRB, 0.14228E+01_JPRB, 0.98836E+00_JPRB, 0.11104E+01_JPRB, 0.12078E+01_JPRB, 0.11605E+01_JPRB, 0.12313E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10062E+01_JPRB, 0.88082E+00_JPRB, 0.82968E+00_JPRB, 0.83614E+00_JPRB, 0.90177E+00_JPRB, 0.89856E+00_JPRB ,&
&  0.86206E+00_JPRB, 0.79244E+00_JPRB, 0.83500E+00_JPRB, 0.87665E+00_JPRB, 0.87795E+00_JPRB, 0.84753E+00_JPRB, 0.82984E+00_JPRB ,&
&  0.85429E+00_JPRB, 0.86135E+00_JPRB, 0.10636E+01_JPRB, 0.11413E+01_JPRB, 0.74361E+00_JPRB, 0.72389E+00_JPRB, 0.71408E+00_JPRB ,&
&  0.71700E+00_JPRB, 0.70777E+00_JPRB, 0.69564E+00_JPRB, 0.69789E+00_JPRB, 0.69194E+00_JPRB, 0.68585E+00_JPRB, 0.70333E+00_JPRB ,&
&  0.72736E+00_JPRB, 0.72559E+00_JPRB, 0.72409E+00_JPRB, 0.72432E+00_JPRB, 0.73097E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.65066E+00_JPRB, 0.64863E+00_JPRB, 0.64618E+00_JPRB, 0.64955E+00_JPRB, 0.65997E+00_JPRB, 0.66134E+00_JPRB, 0.65352E+00_JPRB ,&
&  0.63706E+00_JPRB, 0.62963E+00_JPRB, 0.60934E+00_JPRB, 0.58584E+00_JPRB, 0.58735E+00_JPRB, 0.63469E+00_JPRB, 0.68547E+00_JPRB ,&
&  0.71942E+00_JPRB, 0.73676E+00_JPRB, 0.74209E+00_JPRB, 0.74914E+00_JPRB, 0.79540E+00_JPRB, 0.84578E+00_JPRB, 0.10960E+01_JPRB ,&
&  0.15304E+01_JPRB, 0.21347E+01_JPRB, 0.25866E+01_JPRB, 0.28182E+01_JPRB, 0.26843E+01_JPRB, 0.22729E+01_JPRB, 0.25720E+01_JPRB ,&
&  0.22981E+01_JPRB, 0.17255E+01_JPRB, 0.15031E+01_JPRB, 0.14035E+01_JPRB, 0.13604E+01_JPRB, 0.13308E+01_JPRB, 0.13316E+01_JPRB ,&
&  0.13540E+01_JPRB, 0.13822E+01_JPRB, 0.14075E+01_JPRB, 0.14393E+01_JPRB, 0.14702E+01_JPRB, 0.15078E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.15820E+01_JPRB, 0.15831E+01_JPRB, 0.16355E+01_JPRB, 0.15050E+01_JPRB, 0.15415E+01_JPRB, 0.15913E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.16193E+01_JPRB, 0.15687E+01_JPRB, 0.16352E+01_JPRB, 0.16619E+01_JPRB, 0.16717E+01_JPRB, 0.16221E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.14661E+01_JPRB, 0.11118E+01_JPRB, 0.11939E+01_JPRB, 0.13345E+01_JPRB, 0.11139E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.11850E+01_JPRB, 0.10969E+01_JPRB, 0.10169E+01_JPRB, 0.97497E+00_JPRB, 0.10266E+01_JPRB, 0.10938E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.10536E+01_JPRB, 0.98676E+00_JPRB, 0.10259E+01_JPRB, 0.10665E+01_JPRB, 0.10475E+01_JPRB, 0.99807E+00_JPRB, 0.94615E+00_JPRB ,&
&  0.93971E+00_JPRB, 0.92756E+00_JPRB, 0.89736E+00_JPRB, 0.91401E+00_JPRB, 0.73932E+00_JPRB, 0.70761E+00_JPRB, 0.71954E+00_JPRB ,&
&  0.71626E+00_JPRB, 0.70012E+00_JPRB, 0.70019E+00_JPRB, 0.69930E+00_JPRB, 0.70386E+00_JPRB, 0.69319E+00_JPRB, 0.70000E+00_JPRB ,&
&  0.69895E+00_JPRB, 0.69322E+00_JPRB, 0.68518E+00_JPRB, 0.68276E+00_JPRB, 0.66555E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.70737E+00_JPRB, 0.69284E+00_JPRB, 0.66955E+00_JPRB, 0.64515E+00_JPRB, 0.62151E+00_JPRB, 0.59151E+00_JPRB, 0.59288E+00_JPRB ,&
&  0.58345E+00_JPRB, 0.59417E+00_JPRB, 0.62052E+00_JPRB, 0.64507E+00_JPRB, 0.68638E+00_JPRB, 0.74992E+00_JPRB, 0.81175E+00_JPRB ,&
&  0.85773E+00_JPRB, 0.88281E+00_JPRB, 0.95608E+00_JPRB, 0.10709E+01_JPRB, 0.12679E+01_JPRB, 0.15996E+01_JPRB, 0.21157E+01_JPRB ,&
&  0.23904E+01_JPRB, 0.25739E+01_JPRB, 0.25811E+01_JPRB, 0.24206E+01_JPRB, 0.20954E+01_JPRB, 0.20861E+01_JPRB, 0.21794E+01_JPRB ,&
&  0.22113E+01_JPRB, 0.19454E+01_JPRB, 0.16012E+01_JPRB, 0.15169E+01_JPRB, 0.14668E+01_JPRB, 0.14366E+01_JPRB, 0.14326E+01_JPRB ,&
&  0.14498E+01_JPRB, 0.14665E+01_JPRB, 0.14733E+01_JPRB, 0.15022E+01_JPRB, 0.15199E+01_JPRB, 0.15355E+01_JPRB, 0.15590E+01_JPRB ,&
&  0.15686E+01_JPRB, 0.15618E+01_JPRB, 0.15509E+01_JPRB, 0.15076E+01_JPRB, 0.15200E+01_JPRB, 0.15435E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.14694E+01_JPRB, 0.15116E+01_JPRB, 0.15702E+01_JPRB, 0.15304E+01_JPRB, 0.14306E+01_JPRB, 0.14255E+01_JPRB ,&
&  0.14764E+01_JPRB, 0.15203E+01_JPRB, 0.11591E+01_JPRB, 0.14312E+01_JPRB, 0.16569E+01_JPRB, 0.14647E+01_JPRB, 0.13817E+01_JPRB ,&
&  0.13211E+01_JPRB, 0.12429E+01_JPRB, 0.12364E+01_JPRB, 0.11932E+01_JPRB, 0.12552E+01_JPRB, 0.12687E+01_JPRB, 0.11613E+01_JPRB ,&
&  0.13544E+01_JPRB, 0.13059E+01_JPRB, 0.13754E+01_JPRB, 0.14131E+01_JPRB, 0.13741E+01_JPRB, 0.12723E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.98179E+00_JPRB, 0.11527E+01_JPRB, 0.11829E+01_JPRB, 0.13205E+01_JPRB, 0.12127E+01_JPRB, 0.73534E+00_JPRB, 0.76370E+00_JPRB ,&
&  0.76229E+00_JPRB, 0.76649E+00_JPRB, 0.77561E+00_JPRB, 0.77655E+00_JPRB, 0.77574E+00_JPRB, 0.77573E+00_JPRB, 0.78111E+00_JPRB ,&
&  0.77298E+00_JPRB, 0.76209E+00_JPRB, 0.75092E+00_JPRB, 0.74193E+00_JPRB, 0.72726E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.83076E+00_JPRB, 0.82394E+00_JPRB, 0.80203E+00_JPRB, 0.76139E+00_JPRB, 0.71397E+00_JPRB, 0.66731E+00_JPRB, 0.64098E+00_JPRB ,&
&  0.61851E+00_JPRB, 0.64380E+00_JPRB, 0.69667E+00_JPRB, 0.76707E+00_JPRB, 0.85045E+00_JPRB, 0.93753E+00_JPRB, 0.10220E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.11359E+01_JPRB, 0.12381E+01_JPRB, 0.14635E+01_JPRB, 0.17206E+01_JPRB, 0.19202E+01_JPRB, 0.20440E+01_JPRB ,&
&  0.21478E+01_JPRB, 0.22174E+01_JPRB, 0.21723E+01_JPRB, 0.17935E+01_JPRB, 0.16664E+01_JPRB, 0.20826E+01_JPRB, 0.17058E+01_JPRB ,&
&  0.16145E+01_JPRB, 0.15249E+01_JPRB, 0.14919E+01_JPRB, 0.14863E+01_JPRB, 0.14432E+01_JPRB, 0.14100E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14128E+01_JPRB, 0.14383E+01_JPRB, 0.14652E+01_JPRB, 0.14732E+01_JPRB, 0.14979E+01_JPRB, 0.15308E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.15245E+01_JPRB, 0.15198E+01_JPRB, 0.15375E+01_JPRB, 0.15070E+01_JPRB, 0.14992E+01_JPRB, 0.14662E+01_JPRB ,&
&  0.14382E+01_JPRB, 0.14133E+01_JPRB, 0.14259E+01_JPRB, 0.14924E+01_JPRB, 0.15135E+01_JPRB, 0.13649E+01_JPRB, 0.13883E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.15113E+01_JPRB, 0.13824E+01_JPRB, 0.16914E+01_JPRB, 0.11470E+01_JPRB, 0.13198E+01_JPRB, 0.15120E+01_JPRB ,&
&  0.14842E+01_JPRB, 0.13684E+01_JPRB, 0.13186E+01_JPRB, 0.13176E+01_JPRB, 0.12993E+01_JPRB, 0.12789E+01_JPRB, 0.14650E+01_JPRB ,&
&  0.16147E+01_JPRB, 0.16321E+01_JPRB, 0.16701E+01_JPRB, 0.16733E+01_JPRB, 0.16858E+01_JPRB, 0.16094E+01_JPRB, 0.15245E+01_JPRB ,&
&  0.13294E+01_JPRB, 0.12882E+01_JPRB, 0.11863E+01_JPRB, 0.11529E+01_JPRB, 0.96521E+00_JPRB, 0.73420E+00_JPRB, 0.76647E+00_JPRB ,&
&  0.77842E+00_JPRB, 0.77904E+00_JPRB, 0.79298E+00_JPRB, 0.80060E+00_JPRB, 0.79376E+00_JPRB, 0.79728E+00_JPRB, 0.81142E+00_JPRB ,&
&  0.81204E+00_JPRB, 0.81343E+00_JPRB, 0.81782E+00_JPRB, 0.81907E+00_JPRB, 0.82602E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.84535E+00_JPRB, 0.85598E+00_JPRB, 0.87171E+00_JPRB, 0.87298E+00_JPRB, 0.86397E+00_JPRB, 0.84056E+00_JPRB, 0.78682E+00_JPRB ,&
&  0.73777E+00_JPRB, 0.72067E+00_JPRB, 0.76327E+00_JPRB, 0.81552E+00_JPRB, 0.87288E+00_JPRB, 0.94043E+00_JPRB, 0.10376E+01_JPRB ,&
&  0.11568E+01_JPRB, 0.12592E+01_JPRB, 0.13742E+01_JPRB, 0.15665E+01_JPRB, 0.17851E+01_JPRB, 0.18426E+01_JPRB, 0.18957E+01_JPRB ,&
&  0.17027E+01_JPRB, 0.14435E+01_JPRB, 0.20449E+01_JPRB, 0.24726E+01_JPRB, 0.25697E+01_JPRB, 0.25750E+01_JPRB, 0.23419E+01_JPRB ,&
&  0.16835E+01_JPRB, 0.15058E+01_JPRB, 0.15288E+01_JPRB, 0.15153E+01_JPRB, 0.13699E+01_JPRB, 0.13656E+01_JPRB, 0.13591E+01_JPRB ,&
&  0.13648E+01_JPRB, 0.13553E+01_JPRB, 0.13795E+01_JPRB, 0.13983E+01_JPRB, 0.13941E+01_JPRB, 0.14220E+01_JPRB, 0.14456E+01_JPRB ,&
&  0.14438E+01_JPRB, 0.14344E+01_JPRB, 0.14503E+01_JPRB, 0.14548E+01_JPRB, 0.14710E+01_JPRB, 0.14751E+01_JPRB, 0.14367E+01_JPRB ,&
&  0.13940E+01_JPRB, 0.13466E+01_JPRB, 0.13952E+01_JPRB, 0.14962E+01_JPRB, 0.15752E+01_JPRB, 0.14458E+01_JPRB, 0.15399E+01_JPRB ,&
&  0.15539E+01_JPRB, 0.15555E+01_JPRB, 0.15415E+01_JPRB, 0.17205E+01_JPRB, 0.13091E+01_JPRB, 0.14328E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.15561E+01_JPRB, 0.15128E+01_JPRB, 0.14557E+01_JPRB, 0.14536E+01_JPRB, 0.14845E+01_JPRB, 0.13764E+01_JPRB, 0.15006E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.17567E+01_JPRB, 0.17831E+01_JPRB, 0.18340E+01_JPRB, 0.17395E+01_JPRB, 0.17628E+01_JPRB, 0.17958E+01_JPRB ,&
&  0.19405E+01_JPRB, 0.15405E+01_JPRB, 0.12224E+01_JPRB, 0.99175E+00_JPRB, 0.84411E+00_JPRB, 0.72741E+00_JPRB, 0.75582E+00_JPRB ,&
&  0.77078E+00_JPRB, 0.78213E+00_JPRB, 0.78478E+00_JPRB, 0.79513E+00_JPRB, 0.80259E+00_JPRB, 0.80477E+00_JPRB, 0.81783E+00_JPRB ,&
&  0.82499E+00_JPRB, 0.82934E+00_JPRB, 0.83122E+00_JPRB, 0.83410E+00_JPRB, 0.83769E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.83699E+00_JPRB, 0.86021E+00_JPRB, 0.89732E+00_JPRB, 0.95863E+00_JPRB, 0.10192E+01_JPRB, 0.11167E+01_JPRB, 0.12002E+01_JPRB ,&
&  0.10803E+01_JPRB, 0.75341E+00_JPRB, 0.78249E+00_JPRB, 0.81134E+00_JPRB, 0.84991E+00_JPRB, 0.90239E+00_JPRB, 0.97348E+00_JPRB ,&
&  0.10903E+01_JPRB, 0.12377E+01_JPRB, 0.14200E+01_JPRB, 0.15853E+01_JPRB, 0.17451E+01_JPRB, 0.18257E+01_JPRB, 0.17398E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.16181E+01_JPRB, 0.25282E+01_JPRB, 0.25217E+01_JPRB, 0.24631E+01_JPRB, 0.24291E+01_JPRB, 0.23449E+01_JPRB ,&
&  0.17882E+01_JPRB, 0.14193E+01_JPRB, 0.13878E+01_JPRB, 0.13436E+01_JPRB, 0.12406E+01_JPRB, 0.12508E+01_JPRB, 0.12659E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.12555E+01_JPRB, 0.12543E+01_JPRB, 0.12740E+01_JPRB, 0.12702E+01_JPRB, 0.12780E+01_JPRB, 0.13065E+01_JPRB ,&
&  0.13557E+01_JPRB, 0.13776E+01_JPRB, 0.13837E+01_JPRB, 0.13624E+01_JPRB, 0.14068E+01_JPRB, 0.14539E+01_JPRB, 0.14293E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.13807E+01_JPRB, 0.14022E+01_JPRB, 0.13916E+01_JPRB, 0.13259E+01_JPRB, 0.14878E+01_JPRB, 0.15005E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.15993E+01_JPRB, 0.17737E+01_JPRB, 0.14396E+01_JPRB, 0.15354E+01_JPRB, 0.16372E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.15879E+01_JPRB, 0.17002E+01_JPRB, 0.16851E+01_JPRB, 0.17211E+01_JPRB, 0.17353E+01_JPRB, 0.15675E+01_JPRB, 0.16808E+01_JPRB ,&
&  0.16617E+01_JPRB, 0.18899E+01_JPRB, 0.18698E+01_JPRB, 0.18318E+01_JPRB, 0.15996E+01_JPRB, 0.15653E+01_JPRB, 0.16494E+01_JPRB ,&
&  0.18607E+01_JPRB, 0.18297E+01_JPRB, 0.15611E+01_JPRB, 0.12662E+01_JPRB, 0.95996E+00_JPRB, 0.80705E+00_JPRB, 0.78329E+00_JPRB ,&
&  0.76742E+00_JPRB, 0.76907E+00_JPRB, 0.78074E+00_JPRB, 0.79958E+00_JPRB, 0.81194E+00_JPRB, 0.81374E+00_JPRB, 0.81257E+00_JPRB ,&
&  0.81706E+00_JPRB, 0.81969E+00_JPRB, 0.81868E+00_JPRB, 0.82244E+00_JPRB, 0.82809E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.81133E+00_JPRB, 0.81395E+00_JPRB, 0.83340E+00_JPRB, 0.85834E+00_JPRB, 0.88016E+00_JPRB, 0.91657E+00_JPRB, 0.90653E+00_JPRB ,&
&  0.82070E+00_JPRB, 0.77362E+00_JPRB, 0.80032E+00_JPRB, 0.80230E+00_JPRB, 0.81059E+00_JPRB, 0.84292E+00_JPRB, 0.89998E+00_JPRB ,&
&  0.96930E+00_JPRB, 0.11035E+01_JPRB, 0.12894E+01_JPRB, 0.15403E+01_JPRB, 0.16547E+01_JPRB, 0.17339E+01_JPRB, 0.16705E+01_JPRB ,&
&  0.13723E+01_JPRB, 0.25925E+01_JPRB, 0.23598E+01_JPRB, 0.25252E+01_JPRB, 0.24690E+01_JPRB, 0.25179E+01_JPRB, 0.22008E+01_JPRB ,&
&  0.17967E+01_JPRB, 0.14347E+01_JPRB, 0.13410E+01_JPRB, 0.12226E+01_JPRB, 0.11529E+01_JPRB, 0.11440E+01_JPRB, 0.11639E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11714E+01_JPRB, 0.11708E+01_JPRB, 0.11846E+01_JPRB, 0.11915E+01_JPRB, 0.12077E+01_JPRB, 0.12373E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.13220E+01_JPRB, 0.13176E+01_JPRB, 0.13323E+01_JPRB, 0.13565E+01_JPRB, 0.14258E+01_JPRB, 0.14537E+01_JPRB ,&
&  0.13806E+01_JPRB, 0.13045E+01_JPRB, 0.13715E+01_JPRB, 0.13990E+01_JPRB, 0.14071E+01_JPRB, 0.15097E+01_JPRB, 0.14551E+01_JPRB ,&
&  0.16600E+01_JPRB, 0.16270E+01_JPRB, 0.18171E+01_JPRB, 0.15216E+01_JPRB, 0.15680E+01_JPRB, 0.17788E+01_JPRB, 0.17258E+01_JPRB ,&
&  0.16473E+01_JPRB, 0.18805E+01_JPRB, 0.18967E+01_JPRB, 0.19203E+01_JPRB, 0.18931E+01_JPRB, 0.17549E+01_JPRB, 0.16512E+01_JPRB ,&
&  0.17227E+01_JPRB, 0.18719E+01_JPRB, 0.19862E+01_JPRB, 0.17355E+01_JPRB, 0.16786E+01_JPRB, 0.15415E+01_JPRB, 0.16625E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.19600E+01_JPRB, 0.18448E+01_JPRB, 0.16487E+01_JPRB, 0.12821E+01_JPRB, 0.10726E+01_JPRB, 0.92571E+00_JPRB ,&
&  0.84528E+00_JPRB, 0.79159E+00_JPRB, 0.78078E+00_JPRB, 0.79473E+00_JPRB, 0.81294E+00_JPRB, 0.82328E+00_JPRB, 0.81798E+00_JPRB ,&
&  0.81767E+00_JPRB, 0.82238E+00_JPRB, 0.82599E+00_JPRB, 0.82482E+00_JPRB, 0.82031E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.84216E+00_JPRB, 0.81523E+00_JPRB, 0.81126E+00_JPRB, 0.82550E+00_JPRB, 0.82292E+00_JPRB, 0.82064E+00_JPRB, 0.79902E+00_JPRB ,&
&  0.73863E+00_JPRB, 0.72946E+00_JPRB, 0.74036E+00_JPRB, 0.74879E+00_JPRB, 0.75808E+00_JPRB, 0.79516E+00_JPRB, 0.85030E+00_JPRB ,&
&  0.91419E+00_JPRB, 0.10075E+01_JPRB, 0.10930E+01_JPRB, 0.12782E+01_JPRB, 0.14510E+01_JPRB, 0.16930E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.14921E+01_JPRB, 0.24564E+01_JPRB, 0.25014E+01_JPRB, 0.26079E+01_JPRB, 0.26358E+01_JPRB, 0.26047E+01_JPRB, 0.21718E+01_JPRB ,&
&  0.18893E+01_JPRB, 0.16234E+01_JPRB, 0.13780E+01_JPRB, 0.11978E+01_JPRB, 0.10823E+01_JPRB, 0.10847E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.11617E+01_JPRB, 0.11322E+01_JPRB, 0.11288E+01_JPRB, 0.11242E+01_JPRB, 0.11250E+01_JPRB, 0.11788E+01_JPRB ,&
&  0.12413E+01_JPRB, 0.12830E+01_JPRB, 0.13168E+01_JPRB, 0.12821E+01_JPRB, 0.13627E+01_JPRB, 0.14147E+01_JPRB, 0.14577E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.12813E+01_JPRB, 0.13975E+01_JPRB, 0.15469E+01_JPRB, 0.16422E+01_JPRB, 0.16197E+01_JPRB, 0.15469E+01_JPRB ,&
&  0.16349E+01_JPRB, 0.16999E+01_JPRB, 0.17109E+01_JPRB, 0.15438E+01_JPRB, 0.16411E+01_JPRB, 0.19212E+01_JPRB, 0.20101E+01_JPRB ,&
&  0.20208E+01_JPRB, 0.20351E+01_JPRB, 0.20453E+01_JPRB, 0.19520E+01_JPRB, 0.19085E+01_JPRB, 0.18034E+01_JPRB, 0.18824E+01_JPRB ,&
&  0.21365E+01_JPRB, 0.23379E+01_JPRB, 0.20021E+01_JPRB, 0.17055E+01_JPRB, 0.17425E+01_JPRB, 0.15608E+01_JPRB, 0.15112E+01_JPRB ,&
&  0.17611E+01_JPRB, 0.17981E+01_JPRB, 0.17154E+01_JPRB, 0.17778E+01_JPRB, 0.15680E+01_JPRB, 0.14239E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.94708E+00_JPRB, 0.88178E+00_JPRB, 0.84700E+00_JPRB, 0.85265E+00_JPRB, 0.85293E+00_JPRB, 0.85126E+00_JPRB ,&
&  0.84115E+00_JPRB, 0.84613E+00_JPRB, 0.85720E+00_JPRB, 0.86810E+00_JPRB, 0.86482E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.90146E+00_JPRB, 0.87639E+00_JPRB, 0.83917E+00_JPRB, 0.81316E+00_JPRB, 0.81034E+00_JPRB, 0.77867E+00_JPRB, 0.74827E+00_JPRB ,&
&  0.70666E+00_JPRB, 0.68865E+00_JPRB, 0.66679E+00_JPRB, 0.66414E+00_JPRB, 0.67652E+00_JPRB, 0.71527E+00_JPRB, 0.77798E+00_JPRB ,&
&  0.86300E+00_JPRB, 0.93956E+00_JPRB, 0.10008E+01_JPRB, 0.10619E+01_JPRB, 0.11627E+01_JPRB, 0.11185E+01_JPRB, 0.12615E+01_JPRB ,&
&  0.18112E+01_JPRB, 0.22779E+01_JPRB, 0.24488E+01_JPRB, 0.25603E+01_JPRB, 0.26880E+01_JPRB, 0.28305E+01_JPRB, 0.24512E+01_JPRB ,&
&  0.21383E+01_JPRB, 0.18682E+01_JPRB, 0.15306E+01_JPRB, 0.12874E+01_JPRB, 0.11202E+01_JPRB, 0.10358E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10537E+01_JPRB, 0.10892E+01_JPRB, 0.10862E+01_JPRB, 0.10681E+01_JPRB, 0.10570E+01_JPRB, 0.10467E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.12085E+01_JPRB, 0.12521E+01_JPRB, 0.12642E+01_JPRB, 0.13299E+01_JPRB, 0.13890E+01_JPRB, 0.14736E+01_JPRB ,&
&  0.14628E+01_JPRB, 0.14903E+01_JPRB, 0.15454E+01_JPRB, 0.15898E+01_JPRB, 0.17255E+01_JPRB, 0.18001E+01_JPRB, 0.17719E+01_JPRB ,&
&  0.17176E+01_JPRB, 0.16147E+01_JPRB, 0.14008E+01_JPRB, 0.14846E+01_JPRB, 0.17541E+01_JPRB, 0.21460E+01_JPRB, 0.19517E+01_JPRB ,&
&  0.17025E+01_JPRB, 0.16631E+01_JPRB, 0.17900E+01_JPRB, 0.20636E+01_JPRB, 0.22607E+01_JPRB, 0.21735E+01_JPRB, 0.24772E+01_JPRB ,&
&  0.20981E+01_JPRB, 0.11218E+01_JPRB, 0.95238E+00_JPRB, 0.11995E+01_JPRB, 0.97729E+00_JPRB, 0.10822E+01_JPRB, 0.15572E+01_JPRB ,&
&  0.16711E+01_JPRB, 0.19071E+01_JPRB, 0.19482E+01_JPRB, 0.16387E+01_JPRB, 0.15928E+01_JPRB, 0.15621E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.12054E+01_JPRB, 0.11220E+01_JPRB, 0.10513E+01_JPRB, 0.10031E+01_JPRB, 0.96501E+00_JPRB, 0.94535E+00_JPRB ,&
&  0.92844E+00_JPRB, 0.91933E+00_JPRB, 0.91579E+00_JPRB, 0.91298E+00_JPRB, 0.91418E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.97467E+00_JPRB, 0.94985E+00_JPRB, 0.91959E+00_JPRB, 0.87659E+00_JPRB, 0.84340E+00_JPRB, 0.80408E+00_JPRB, 0.74972E+00_JPRB ,&
&  0.70180E+00_JPRB, 0.67719E+00_JPRB, 0.65158E+00_JPRB, 0.63346E+00_JPRB, 0.64028E+00_JPRB, 0.66208E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.80140E+00_JPRB, 0.91981E+00_JPRB, 0.10397E+01_JPRB, 0.10734E+01_JPRB, 0.10619E+01_JPRB, 0.99745E+00_JPRB, 0.12465E+01_JPRB ,&
&  0.15949E+01_JPRB, 0.18845E+01_JPRB, 0.24307E+01_JPRB, 0.27087E+01_JPRB, 0.27981E+01_JPRB, 0.27878E+01_JPRB, 0.26354E+01_JPRB ,&
&  0.24142E+01_JPRB, 0.21483E+01_JPRB, 0.17393E+01_JPRB, 0.14247E+01_JPRB, 0.12085E+01_JPRB, 0.10680E+01_JPRB, 0.99495E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10485E+01_JPRB, 0.10552E+01_JPRB, 0.10272E+01_JPRB, 0.10080E+01_JPRB, 0.10025E+01_JPRB, 0.10152E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.11055E+01_JPRB, 0.11683E+01_JPRB, 0.12541E+01_JPRB, 0.10889E+01_JPRB, 0.13296E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.16206E+01_JPRB, 0.15725E+01_JPRB, 0.17245E+01_JPRB, 0.19245E+01_JPRB, 0.19282E+01_JPRB, 0.19241E+01_JPRB ,&
&  0.19104E+01_JPRB, 0.17053E+01_JPRB, 0.14209E+01_JPRB, 0.15172E+01_JPRB, 0.17987E+01_JPRB, 0.23746E+01_JPRB, 0.14964E+01_JPRB ,&
&  0.13360E+01_JPRB, 0.18627E+01_JPRB, 0.19272E+01_JPRB, 0.18210E+01_JPRB, 0.24938E+01_JPRB, 0.28933E+01_JPRB, 0.19498E+01_JPRB ,&
&  0.72752E+00_JPRB, 0.59567E+00_JPRB, 0.64521E+00_JPRB, 0.60958E+00_JPRB, 0.62076E+00_JPRB, 0.84249E+00_JPRB, 0.20888E+01_JPRB ,&
&  0.16431E+01_JPRB, 0.18973E+01_JPRB, 0.19009E+01_JPRB, 0.18614E+01_JPRB, 0.17368E+01_JPRB, 0.17025E+01_JPRB, 0.17425E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.14135E+01_JPRB, 0.12668E+01_JPRB, 0.11887E+01_JPRB, 0.11437E+01_JPRB, 0.11162E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.10478E+01_JPRB, 0.10337E+01_JPRB, 0.10184E+01_JPRB, 0.99997E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.10239E+01_JPRB, 0.10036E+01_JPRB, 0.98171E+00_JPRB, 0.94973E+00_JPRB, 0.90981E+00_JPRB, 0.85887E+00_JPRB, 0.79687E+00_JPRB ,&
&  0.73619E+00_JPRB, 0.69238E+00_JPRB, 0.66807E+00_JPRB, 0.65075E+00_JPRB, 0.64513E+00_JPRB, 0.64961E+00_JPRB, 0.68692E+00_JPRB ,&
&  0.76505E+00_JPRB, 0.91040E+00_JPRB, 0.11564E+01_JPRB, 0.10874E+01_JPRB, 0.11034E+01_JPRB, 0.10262E+01_JPRB, 0.11138E+01_JPRB ,&
&  0.14149E+01_JPRB, 0.16674E+01_JPRB, 0.20994E+01_JPRB, 0.25038E+01_JPRB, 0.29580E+01_JPRB, 0.29928E+01_JPRB, 0.29377E+01_JPRB ,&
&  0.27078E+01_JPRB, 0.24455E+01_JPRB, 0.20210E+01_JPRB, 0.16327E+01_JPRB, 0.13171E+01_JPRB, 0.11232E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.99797E+00_JPRB, 0.10248E+01_JPRB, 0.10281E+01_JPRB, 0.99066E+00_JPRB, 0.96564E+00_JPRB, 0.96002E+00_JPRB, 0.97453E+00_JPRB ,&
&  0.98403E+00_JPRB, 0.10152E+01_JPRB, 0.10601E+01_JPRB, 0.11644E+01_JPRB, 0.10701E+01_JPRB, 0.98642E+00_JPRB, 0.12556E+01_JPRB ,&
&  0.15026E+01_JPRB, 0.17940E+01_JPRB, 0.18267E+01_JPRB, 0.20514E+01_JPRB, 0.21823E+01_JPRB, 0.21464E+01_JPRB, 0.22511E+01_JPRB ,&
&  0.21291E+01_JPRB, 0.22655E+01_JPRB, 0.16258E+01_JPRB, 0.16274E+01_JPRB, 0.19455E+01_JPRB, 0.16672E+01_JPRB, 0.15896E+01_JPRB ,&
&  0.18535E+01_JPRB, 0.17185E+01_JPRB, 0.17225E+01_JPRB, 0.14681E+01_JPRB, 0.21897E+01_JPRB, 0.22063E+01_JPRB, 0.84447E+00_JPRB ,&
&  0.66252E+00_JPRB, 0.61978E+00_JPRB, 0.58801E+00_JPRB, 0.56944E+00_JPRB, 0.62984E+00_JPRB, 0.83084E+00_JPRB, 0.12235E+01_JPRB ,&
&  0.14458E+01_JPRB, 0.17831E+01_JPRB, 0.18193E+01_JPRB, 0.19075E+01_JPRB, 0.17982E+01_JPRB, 0.18752E+01_JPRB, 0.19059E+01_JPRB ,&
&  0.18695E+01_JPRB, 0.18014E+01_JPRB, 0.15573E+01_JPRB, 0.13922E+01_JPRB, 0.12918E+01_JPRB, 0.12230E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.11061E+01_JPRB, 0.10874E+01_JPRB, 0.10642E+01_JPRB, 0.10429E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.10398E+01_JPRB, 0.10253E+01_JPRB, 0.10080E+01_JPRB, 0.98614E+00_JPRB, 0.95682E+00_JPRB, 0.91386E+00_JPRB, 0.85513E+00_JPRB ,&
&  0.79449E+00_JPRB, 0.74526E+00_JPRB, 0.70913E+00_JPRB, 0.69021E+00_JPRB, 0.67202E+00_JPRB, 0.65974E+00_JPRB, 0.67990E+00_JPRB ,&
&  0.74859E+00_JPRB, 0.91693E+00_JPRB, 0.10437E+01_JPRB, 0.94928E+00_JPRB, 0.91087E+00_JPRB, 0.90048E+00_JPRB, 0.91026E+00_JPRB ,&
&  0.12495E+01_JPRB, 0.14844E+01_JPRB, 0.18168E+01_JPRB, 0.23928E+01_JPRB, 0.29045E+01_JPRB, 0.28990E+01_JPRB, 0.31373E+01_JPRB ,&
&  0.29512E+01_JPRB, 0.26415E+01_JPRB, 0.22557E+01_JPRB, 0.18460E+01_JPRB, 0.15014E+01_JPRB, 0.12273E+01_JPRB, 0.10909E+01_JPRB ,&
&  0.10275E+01_JPRB, 0.10328E+01_JPRB, 0.10244E+01_JPRB, 0.98165E+00_JPRB, 0.95994E+00_JPRB, 0.93468E+00_JPRB, 0.91764E+00_JPRB ,&
&  0.91668E+00_JPRB, 0.94729E+00_JPRB, 0.98111E+00_JPRB, 0.10968E+01_JPRB, 0.12757E+01_JPRB, 0.13068E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.15833E+01_JPRB, 0.20055E+01_JPRB, 0.21879E+01_JPRB, 0.24173E+01_JPRB, 0.25893E+01_JPRB, 0.25406E+01_JPRB ,&
&  0.23839E+01_JPRB, 0.22846E+01_JPRB, 0.18711E+01_JPRB, 0.19275E+01_JPRB, 0.17281E+01_JPRB, 0.13954E+01_JPRB, 0.16792E+01_JPRB ,&
&  0.18190E+01_JPRB, 0.17780E+01_JPRB, 0.15113E+01_JPRB, 0.11955E+01_JPRB, 0.12647E+01_JPRB, 0.95454E+00_JPRB, 0.67587E+00_JPRB ,&
&  0.63901E+00_JPRB, 0.61823E+00_JPRB, 0.59680E+00_JPRB, 0.63876E+00_JPRB, 0.64741E+00_JPRB, 0.77653E+00_JPRB, 0.10094E+01_JPRB ,&
&  0.14190E+01_JPRB, 0.16754E+01_JPRB, 0.17965E+01_JPRB, 0.18862E+01_JPRB, 0.19126E+01_JPRB, 0.20040E+01_JPRB, 0.22542E+01_JPRB ,&
&  0.20547E+01_JPRB, 0.20133E+01_JPRB, 0.17675E+01_JPRB, 0.15713E+01_JPRB, 0.14413E+01_JPRB, 0.13346E+01_JPRB, 0.12470E+01_JPRB ,&
&  0.11893E+01_JPRB, 0.11439E+01_JPRB, 0.11103E+01_JPRB, 0.10827E+01_JPRB, 0.10564E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.10413E+01_JPRB, 0.10254E+01_JPRB, 0.10113E+01_JPRB, 0.99515E+00_JPRB, 0.97054E+00_JPRB, 0.93835E+00_JPRB, 0.89667E+00_JPRB ,&
&  0.83814E+00_JPRB, 0.78870E+00_JPRB, 0.74139E+00_JPRB, 0.72002E+00_JPRB, 0.69881E+00_JPRB, 0.68005E+00_JPRB, 0.68672E+00_JPRB ,&
&  0.74648E+00_JPRB, 0.93295E+00_JPRB, 0.80808E+00_JPRB, 0.78501E+00_JPRB, 0.84615E+00_JPRB, 0.89326E+00_JPRB, 0.78738E+00_JPRB ,&
&  0.10876E+01_JPRB, 0.13520E+01_JPRB, 0.17016E+01_JPRB, 0.23865E+01_JPRB, 0.33678E+01_JPRB, 0.31767E+01_JPRB, 0.32519E+01_JPRB ,&
&  0.32468E+01_JPRB, 0.27810E+01_JPRB, 0.24393E+01_JPRB, 0.20471E+01_JPRB, 0.16640E+01_JPRB, 0.13885E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.10509E+01_JPRB, 0.10313E+01_JPRB, 0.10024E+01_JPRB, 0.97694E+00_JPRB, 0.94932E+00_JPRB, 0.91066E+00_JPRB ,&
&  0.89655E+00_JPRB, 0.89761E+00_JPRB, 0.91000E+00_JPRB, 0.10376E+01_JPRB, 0.11317E+01_JPRB, 0.11622E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.17323E+01_JPRB, 0.19869E+01_JPRB, 0.20562E+01_JPRB, 0.24259E+01_JPRB, 0.27655E+01_JPRB, 0.27556E+01_JPRB, 0.29294E+01_JPRB ,&
&  0.22327E+01_JPRB, 0.18422E+01_JPRB, 0.18299E+01_JPRB, 0.18127E+01_JPRB, 0.14964E+01_JPRB, 0.16608E+01_JPRB, 0.26408E+01_JPRB ,&
&  0.19202E+01_JPRB, 0.20912E+01_JPRB, 0.23475E+01_JPRB, 0.24730E+01_JPRB, 0.12586E+01_JPRB, 0.88812E+00_JPRB, 0.14438E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.95188E+00_JPRB, 0.75336E+00_JPRB, 0.96619E+00_JPRB, 0.76857E+00_JPRB, 0.86727E+00_JPRB, 0.11069E+01_JPRB ,&
&  0.12591E+01_JPRB, 0.13831E+01_JPRB, 0.17883E+01_JPRB, 0.20199E+01_JPRB, 0.21367E+01_JPRB, 0.23050E+01_JPRB, 0.23142E+01_JPRB ,&
&  0.22682E+01_JPRB, 0.20729E+01_JPRB, 0.19936E+01_JPRB, 0.17632E+01_JPRB, 0.15919E+01_JPRB, 0.14391E+01_JPRB, 0.13145E+01_JPRB ,&
&  0.12241E+01_JPRB, 0.11557E+01_JPRB, 0.11096E+01_JPRB, 0.10735E+01_JPRB, 0.10575E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.10587E+01_JPRB, 0.10255E+01_JPRB, 0.10098E+01_JPRB, 0.10050E+01_JPRB, 0.99414E+00_JPRB, 0.96597E+00_JPRB, 0.92915E+00_JPRB ,&
&  0.88878E+00_JPRB, 0.83425E+00_JPRB, 0.78330E+00_JPRB, 0.74809E+00_JPRB, 0.73111E+00_JPRB, 0.71016E+00_JPRB, 0.70065E+00_JPRB ,&
&  0.73584E+00_JPRB, 0.77018E+00_JPRB, 0.70970E+00_JPRB, 0.70378E+00_JPRB, 0.86627E+00_JPRB, 0.81981E+00_JPRB, 0.81608E+00_JPRB ,&
&  0.99196E+00_JPRB, 0.12175E+01_JPRB, 0.16126E+01_JPRB, 0.21807E+01_JPRB, 0.26506E+01_JPRB, 0.30807E+01_JPRB, 0.32792E+01_JPRB ,&
&  0.33070E+01_JPRB, 0.29090E+01_JPRB, 0.24942E+01_JPRB, 0.21387E+01_JPRB, 0.17841E+01_JPRB, 0.15105E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.11081E+01_JPRB, 0.10718E+01_JPRB, 0.10330E+01_JPRB, 0.10244E+01_JPRB, 0.98291E+00_JPRB, 0.94643E+00_JPRB ,&
&  0.93362E+00_JPRB, 0.92550E+00_JPRB, 0.90836E+00_JPRB, 0.96283E+00_JPRB, 0.10157E+01_JPRB, 0.11274E+01_JPRB, 0.12827E+01_JPRB ,&
&  0.15410E+01_JPRB, 0.18612E+01_JPRB, 0.20551E+01_JPRB, 0.24520E+01_JPRB, 0.30032E+01_JPRB, 0.28748E+01_JPRB, 0.32825E+01_JPRB ,&
&  0.27436E+01_JPRB, 0.22601E+01_JPRB, 0.18689E+01_JPRB, 0.14975E+01_JPRB, 0.17634E+01_JPRB, 0.28427E+01_JPRB, 0.28975E+01_JPRB ,&
&  0.25839E+01_JPRB, 0.25272E+01_JPRB, 0.25625E+01_JPRB, 0.23019E+01_JPRB, 0.19029E+01_JPRB, 0.14594E+01_JPRB, 0.19239E+01_JPRB ,&
&  0.19944E+01_JPRB, 0.19126E+01_JPRB, 0.17340E+01_JPRB, 0.14440E+01_JPRB, 0.11710E+01_JPRB, 0.13534E+01_JPRB, 0.13221E+01_JPRB ,&
&  0.12746E+01_JPRB, 0.12623E+01_JPRB, 0.15317E+01_JPRB, 0.19567E+01_JPRB, 0.20868E+01_JPRB, 0.21361E+01_JPRB, 0.21952E+01_JPRB ,&
&  0.20823E+01_JPRB, 0.20858E+01_JPRB, 0.20110E+01_JPRB, 0.18735E+01_JPRB, 0.17434E+01_JPRB, 0.16037E+01_JPRB, 0.14892E+01_JPRB ,&
&  0.13943E+01_JPRB, 0.13175E+01_JPRB, 0.12388E+01_JPRB, 0.11663E+01_JPRB, 0.11041E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.10627E+01_JPRB, 0.10160E+01_JPRB, 0.99001E+00_JPRB, 0.98388E+00_JPRB, 0.99331E+00_JPRB, 0.98078E+00_JPRB, 0.95362E+00_JPRB ,&
&  0.91927E+00_JPRB, 0.88367E+00_JPRB, 0.84059E+00_JPRB, 0.79817E+00_JPRB, 0.76980E+00_JPRB, 0.72929E+00_JPRB, 0.71140E+00_JPRB ,&
&  0.73850E+00_JPRB, 0.72389E+00_JPRB, 0.65304E+00_JPRB, 0.72072E+00_JPRB, 0.77007E+00_JPRB, 0.72736E+00_JPRB, 0.81140E+00_JPRB ,&
&  0.90900E+00_JPRB, 0.10964E+01_JPRB, 0.13762E+01_JPRB, 0.17096E+01_JPRB, 0.20278E+01_JPRB, 0.24918E+01_JPRB, 0.28614E+01_JPRB ,&
&  0.28135E+01_JPRB, 0.27400E+01_JPRB, 0.25304E+01_JPRB, 0.22899E+01_JPRB, 0.19600E+01_JPRB, 0.16876E+01_JPRB, 0.14902E+01_JPRB ,&
&  0.13229E+01_JPRB, 0.12203E+01_JPRB, 0.11637E+01_JPRB, 0.11218E+01_JPRB, 0.10966E+01_JPRB, 0.10746E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.98767E+00_JPRB, 0.94594E+00_JPRB, 0.96394E+00_JPRB, 0.10281E+01_JPRB, 0.11370E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.16257E+01_JPRB, 0.20680E+01_JPRB, 0.23103E+01_JPRB, 0.29110E+01_JPRB, 0.25609E+01_JPRB, 0.26827E+01_JPRB, 0.34295E+01_JPRB ,&
&  0.36819E+01_JPRB, 0.37101E+01_JPRB, 0.35851E+01_JPRB, 0.28103E+01_JPRB, 0.24154E+01_JPRB, 0.33533E+01_JPRB, 0.31911E+01_JPRB ,&
&  0.28310E+01_JPRB, 0.26967E+01_JPRB, 0.25084E+01_JPRB, 0.27258E+01_JPRB, 0.22202E+01_JPRB, 0.19877E+01_JPRB, 0.15809E+01_JPRB ,&
&  0.16214E+01_JPRB, 0.14057E+01_JPRB, 0.16466E+01_JPRB, 0.14778E+01_JPRB, 0.14246E+01_JPRB, 0.13888E+01_JPRB, 0.12988E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.12704E+01_JPRB, 0.12935E+01_JPRB, 0.15454E+01_JPRB, 0.18057E+01_JPRB, 0.17277E+01_JPRB, 0.18295E+01_JPRB ,&
&  0.18807E+01_JPRB, 0.19146E+01_JPRB, 0.18456E+01_JPRB, 0.18662E+01_JPRB, 0.18102E+01_JPRB, 0.17235E+01_JPRB, 0.16153E+01_JPRB ,&
&  0.15051E+01_JPRB, 0.14122E+01_JPRB, 0.13172E+01_JPRB, 0.12185E+01_JPRB, 0.11302E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.10436E+01_JPRB, 0.99812E+00_JPRB, 0.96747E+00_JPRB, 0.96295E+00_JPRB, 0.97742E+00_JPRB, 0.98040E+00_JPRB, 0.95691E+00_JPRB ,&
&  0.93541E+00_JPRB, 0.91000E+00_JPRB, 0.89021E+00_JPRB, 0.84515E+00_JPRB, 0.80301E+00_JPRB, 0.75197E+00_JPRB, 0.72509E+00_JPRB ,&
&  0.75740E+00_JPRB, 0.82219E+00_JPRB, 0.70970E+00_JPRB, 0.65796E+00_JPRB, 0.61558E+00_JPRB, 0.67970E+00_JPRB, 0.78955E+00_JPRB ,&
&  0.91021E+00_JPRB, 0.10012E+01_JPRB, 0.12079E+01_JPRB, 0.12897E+01_JPRB, 0.14024E+01_JPRB, 0.17419E+01_JPRB, 0.22050E+01_JPRB ,&
&  0.27567E+01_JPRB, 0.23061E+01_JPRB, 0.23123E+01_JPRB, 0.21545E+01_JPRB, 0.20001E+01_JPRB, 0.18068E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.12864E+01_JPRB, 0.12066E+01_JPRB, 0.11448E+01_JPRB, 0.10945E+01_JPRB, 0.10547E+01_JPRB, 0.10248E+01_JPRB ,&
&  0.10067E+01_JPRB, 0.10057E+01_JPRB, 0.10099E+01_JPRB, 0.99884E+00_JPRB, 0.10821E+01_JPRB, 0.12305E+01_JPRB, 0.13604E+01_JPRB ,&
&  0.14539E+01_JPRB, 0.17575E+01_JPRB, 0.27346E+01_JPRB, 0.29773E+01_JPRB, 0.32344E+01_JPRB, 0.34621E+01_JPRB, 0.37256E+01_JPRB ,&
&  0.39870E+01_JPRB, 0.40321E+01_JPRB, 0.39637E+01_JPRB, 0.33996E+01_JPRB, 0.38097E+01_JPRB, 0.37782E+01_JPRB, 0.35205E+01_JPRB ,&
&  0.31984E+01_JPRB, 0.31086E+01_JPRB, 0.28135E+01_JPRB, 0.27169E+01_JPRB, 0.27597E+01_JPRB, 0.29374E+01_JPRB, 0.25578E+01_JPRB ,&
&  0.21276E+01_JPRB, 0.23198E+01_JPRB, 0.20306E+01_JPRB, 0.16236E+01_JPRB, 0.12994E+01_JPRB, 0.12974E+01_JPRB, 0.13201E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.13287E+01_JPRB, 0.12989E+01_JPRB, 0.13726E+01_JPRB, 0.15408E+01_JPRB, 0.16040E+01_JPRB, 0.15734E+01_JPRB ,&
&  0.15385E+01_JPRB, 0.16520E+01_JPRB, 0.16728E+01_JPRB, 0.16413E+01_JPRB, 0.16318E+01_JPRB, 0.15934E+01_JPRB, 0.15064E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.12958E+01_JPRB, 0.12220E+01_JPRB, 0.11563E+01_JPRB, 0.10963E+01_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.96918E+00_JPRB, 0.94667E+00_JPRB, 0.92961E+00_JPRB, 0.93324E+00_JPRB, 0.95725E+00_JPRB, 0.96895E+00_JPRB, 0.95101E+00_JPRB ,&
&  0.93596E+00_JPRB, 0.91680E+00_JPRB, 0.89700E+00_JPRB, 0.86844E+00_JPRB, 0.82260E+00_JPRB, 0.77384E+00_JPRB, 0.75446E+00_JPRB ,&
&  0.80028E+00_JPRB, 0.98012E+00_JPRB, 0.82918E+00_JPRB, 0.78974E+00_JPRB, 0.72019E+00_JPRB, 0.78190E+00_JPRB, 0.85145E+00_JPRB ,&
&  0.93640E+00_JPRB, 0.97329E+00_JPRB, 0.10284E+01_JPRB, 0.10496E+01_JPRB, 0.10919E+01_JPRB, 0.13198E+01_JPRB, 0.16874E+01_JPRB ,&
&  0.19251E+01_JPRB, 0.20903E+01_JPRB, 0.19473E+01_JPRB, 0.20551E+01_JPRB, 0.18967E+01_JPRB, 0.16694E+01_JPRB, 0.16661E+01_JPRB ,&
&  0.15119E+01_JPRB, 0.13747E+01_JPRB, 0.12414E+01_JPRB, 0.11628E+01_JPRB, 0.11109E+01_JPRB, 0.10601E+01_JPRB, 0.10185E+01_JPRB ,&
&  0.98777E+00_JPRB, 0.98625E+00_JPRB, 0.10161E+01_JPRB, 0.10348E+01_JPRB, 0.10583E+01_JPRB, 0.12199E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.19088E+01_JPRB, 0.19891E+01_JPRB, 0.19598E+01_JPRB, 0.27362E+01_JPRB, 0.39117E+01_JPRB, 0.40287E+01_JPRB, 0.38189E+01_JPRB ,&
&  0.41029E+01_JPRB, 0.42230E+01_JPRB, 0.42524E+01_JPRB, 0.42230E+01_JPRB, 0.42368E+01_JPRB, 0.40911E+01_JPRB, 0.38566E+01_JPRB ,&
&  0.35763E+01_JPRB, 0.34054E+01_JPRB, 0.32294E+01_JPRB, 0.30512E+01_JPRB, 0.28138E+01_JPRB, 0.27449E+01_JPRB, 0.28187E+01_JPRB ,&
&  0.25878E+01_JPRB, 0.22217E+01_JPRB, 0.14819E+01_JPRB, 0.15167E+01_JPRB, 0.11601E+01_JPRB, 0.11252E+01_JPRB, 0.12837E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.13470E+01_JPRB, 0.13493E+01_JPRB, 0.12536E+01_JPRB, 0.13283E+01_JPRB, 0.14227E+01_JPRB, 0.14124E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.13597E+01_JPRB, 0.14688E+01_JPRB, 0.14300E+01_JPRB, 0.13767E+01_JPRB, 0.13359E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.10952E+01_JPRB, 0.10250E+01_JPRB, 0.99551E+00_JPRB, 0.98047E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.84989E+00_JPRB, 0.85448E+00_JPRB, 0.86087E+00_JPRB, 0.87476E+00_JPRB, 0.90657E+00_JPRB, 0.92922E+00_JPRB, 0.92107E+00_JPRB ,&
&  0.90699E+00_JPRB, 0.89560E+00_JPRB, 0.88021E+00_JPRB, 0.86182E+00_JPRB, 0.83021E+00_JPRB, 0.78439E+00_JPRB, 0.77525E+00_JPRB ,&
&  0.90767E+00_JPRB, 0.98765E+00_JPRB, 0.88779E+00_JPRB, 0.75878E+00_JPRB, 0.87530E+00_JPRB, 0.83877E+00_JPRB, 0.87150E+00_JPRB ,&
&  0.83873E+00_JPRB, 0.92609E+00_JPRB, 0.96563E+00_JPRB, 0.92417E+00_JPRB, 0.90382E+00_JPRB, 0.10559E+01_JPRB, 0.12539E+01_JPRB ,&
&  0.14293E+01_JPRB, 0.15562E+01_JPRB, 0.15932E+01_JPRB, 0.16106E+01_JPRB, 0.16741E+01_JPRB, 0.15165E+01_JPRB, 0.14994E+01_JPRB ,&
&  0.14088E+01_JPRB, 0.13125E+01_JPRB, 0.12137E+01_JPRB, 0.11398E+01_JPRB, 0.10763E+01_JPRB, 0.10263E+01_JPRB, 0.99434E+00_JPRB ,&
&  0.99328E+00_JPRB, 0.99985E+00_JPRB, 0.10081E+01_JPRB, 0.10403E+01_JPRB, 0.10838E+01_JPRB, 0.13236E+01_JPRB, 0.18193E+01_JPRB ,&
&  0.24610E+01_JPRB, 0.27815E+01_JPRB, 0.32417E+01_JPRB, 0.39259E+01_JPRB, 0.41527E+01_JPRB, 0.42605E+01_JPRB, 0.44188E+01_JPRB ,&
&  0.44973E+01_JPRB, 0.44946E+01_JPRB, 0.44181E+01_JPRB, 0.42891E+01_JPRB, 0.42668E+01_JPRB, 0.42059E+01_JPRB, 0.40267E+01_JPRB ,&
&  0.36461E+01_JPRB, 0.34485E+01_JPRB, 0.34406E+01_JPRB, 0.31879E+01_JPRB, 0.30054E+01_JPRB, 0.27502E+01_JPRB, 0.28568E+01_JPRB ,&
&  0.27920E+01_JPRB, 0.16462E+01_JPRB, 0.12829E+01_JPRB, 0.15258E+01_JPRB, 0.12368E+01_JPRB, 0.12162E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13278E+01_JPRB, 0.13143E+01_JPRB, 0.12106E+01_JPRB, 0.11777E+01_JPRB, 0.13318E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.11594E+01_JPRB, 0.12136E+01_JPRB, 0.11623E+01_JPRB, 0.11074E+01_JPRB, 0.10819E+01_JPRB, 0.10370E+01_JPRB ,&
&  0.96168E+00_JPRB, 0.90257E+00_JPRB, 0.85527E+00_JPRB, 0.83817E+00_JPRB, 0.84323E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.77689E+00_JPRB, 0.79899E+00_JPRB, 0.81267E+00_JPRB, 0.82993E+00_JPRB, 0.85906E+00_JPRB, 0.89180E+00_JPRB, 0.89595E+00_JPRB ,&
&  0.88523E+00_JPRB, 0.87454E+00_JPRB, 0.86369E+00_JPRB, 0.84609E+00_JPRB, 0.82597E+00_JPRB, 0.79325E+00_JPRB, 0.79539E+00_JPRB ,&
&  0.70183E+00_JPRB, 0.79801E+00_JPRB, 0.70217E+00_JPRB, 0.72802E+00_JPRB, 0.91401E+00_JPRB, 0.83693E+00_JPRB, 0.80696E+00_JPRB ,&
&  0.77469E+00_JPRB, 0.80990E+00_JPRB, 0.85840E+00_JPRB, 0.87294E+00_JPRB, 0.82452E+00_JPRB, 0.88888E+00_JPRB, 0.10251E+01_JPRB ,&
&  0.10864E+01_JPRB, 0.11000E+01_JPRB, 0.11465E+01_JPRB, 0.11603E+01_JPRB, 0.11998E+01_JPRB, 0.13079E+01_JPRB, 0.13478E+01_JPRB ,&
&  0.12828E+01_JPRB, 0.12076E+01_JPRB, 0.11319E+01_JPRB, 0.10735E+01_JPRB, 0.10222E+01_JPRB, 0.96940E+00_JPRB, 0.92620E+00_JPRB ,&
&  0.90003E+00_JPRB, 0.89115E+00_JPRB, 0.92046E+00_JPRB, 0.99533E+00_JPRB, 0.10740E+01_JPRB, 0.14421E+01_JPRB, 0.21739E+01_JPRB ,&
&  0.27052E+01_JPRB, 0.28927E+01_JPRB, 0.33377E+01_JPRB, 0.40178E+01_JPRB, 0.41818E+01_JPRB, 0.42715E+01_JPRB, 0.42985E+01_JPRB ,&
&  0.43861E+01_JPRB, 0.44166E+01_JPRB, 0.43299E+01_JPRB, 0.43404E+01_JPRB, 0.41534E+01_JPRB, 0.41652E+01_JPRB, 0.39258E+01_JPRB ,&
&  0.35122E+01_JPRB, 0.34012E+01_JPRB, 0.34814E+01_JPRB, 0.34223E+01_JPRB, 0.33164E+01_JPRB, 0.33337E+01_JPRB, 0.33245E+01_JPRB ,&
&  0.31502E+01_JPRB, 0.27069E+01_JPRB, 0.20060E+01_JPRB, 0.18085E+01_JPRB, 0.13053E+01_JPRB, 0.13145E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.13733E+01_JPRB, 0.12049E+01_JPRB, 0.11147E+01_JPRB, 0.10845E+01_JPRB, 0.11783E+01_JPRB, 0.10749E+01_JPRB ,&
&  0.94780E+00_JPRB, 0.96544E+00_JPRB, 0.97509E+00_JPRB, 0.97855E+00_JPRB, 0.97539E+00_JPRB, 0.94242E+00_JPRB, 0.87066E+00_JPRB ,&
&  0.84370E+00_JPRB, 0.79619E+00_JPRB, 0.75159E+00_JPRB, 0.73737E+00_JPRB, 0.75026E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.74073E+00_JPRB, 0.76661E+00_JPRB, 0.78369E+00_JPRB, 0.79583E+00_JPRB, 0.81637E+00_JPRB, 0.84936E+00_JPRB, 0.85695E+00_JPRB ,&
&  0.84973E+00_JPRB, 0.84121E+00_JPRB, 0.83429E+00_JPRB, 0.82139E+00_JPRB, 0.79678E+00_JPRB, 0.76917E+00_JPRB, 0.70833E+00_JPRB ,&
&  0.62520E+00_JPRB, 0.71343E+00_JPRB, 0.69035E+00_JPRB, 0.71847E+00_JPRB, 0.72707E+00_JPRB, 0.70661E+00_JPRB, 0.68552E+00_JPRB ,&
&  0.71080E+00_JPRB, 0.73566E+00_JPRB, 0.79638E+00_JPRB, 0.86041E+00_JPRB, 0.84408E+00_JPRB, 0.86142E+00_JPRB, 0.95668E+00_JPRB ,&
&  0.96753E+00_JPRB, 0.96824E+00_JPRB, 0.92838E+00_JPRB, 0.91828E+00_JPRB, 0.95866E+00_JPRB, 0.10338E+01_JPRB, 0.11020E+01_JPRB ,&
&  0.11052E+01_JPRB, 0.10550E+01_JPRB, 0.10018E+01_JPRB, 0.96098E+00_JPRB, 0.92805E+00_JPRB, 0.89298E+00_JPRB, 0.85657E+00_JPRB ,&
&  0.82143E+00_JPRB, 0.80424E+00_JPRB, 0.81830E+00_JPRB, 0.86029E+00_JPRB, 0.94921E+00_JPRB, 0.12525E+01_JPRB, 0.15349E+01_JPRB ,&
&  0.17087E+01_JPRB, 0.19469E+01_JPRB, 0.22938E+01_JPRB, 0.26908E+01_JPRB, 0.30147E+01_JPRB, 0.33000E+01_JPRB, 0.35615E+01_JPRB ,&
&  0.38849E+01_JPRB, 0.39252E+01_JPRB, 0.43704E+01_JPRB, 0.43300E+01_JPRB, 0.41402E+01_JPRB, 0.40573E+01_JPRB, 0.37837E+01_JPRB ,&
&  0.35421E+01_JPRB, 0.32968E+01_JPRB, 0.34637E+01_JPRB, 0.34760E+01_JPRB, 0.34275E+01_JPRB, 0.33995E+01_JPRB, 0.32942E+01_JPRB ,&
&  0.31926E+01_JPRB, 0.30605E+01_JPRB, 0.26272E+01_JPRB, 0.22613E+01_JPRB, 0.18249E+01_JPRB, 0.18268E+01_JPRB, 0.16112E+01_JPRB ,&
&  0.13535E+01_JPRB, 0.10652E+01_JPRB, 0.10388E+01_JPRB, 0.10062E+01_JPRB, 0.10132E+01_JPRB, 0.95441E+00_JPRB, 0.88480E+00_JPRB ,&
&  0.87643E+00_JPRB, 0.88991E+00_JPRB, 0.85427E+00_JPRB, 0.84344E+00_JPRB, 0.85887E+00_JPRB, 0.83016E+00_JPRB, 0.74169E+00_JPRB ,&
&  0.71233E+00_JPRB, 0.70831E+00_JPRB, 0.68184E+00_JPRB, 0.68174E+00_JPRB, 0.70520E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.71210E+00_JPRB, 0.73177E+00_JPRB, 0.74733E+00_JPRB, 0.76206E+00_JPRB, 0.78004E+00_JPRB, 0.79841E+00_JPRB, 0.78833E+00_JPRB ,&
&  0.79374E+00_JPRB, 0.79234E+00_JPRB, 0.78758E+00_JPRB, 0.77144E+00_JPRB, 0.74181E+00_JPRB, 0.66324E+00_JPRB, 0.56224E+00_JPRB ,&
&  0.54423E+00_JPRB, 0.57872E+00_JPRB, 0.61973E+00_JPRB, 0.63851E+00_JPRB, 0.64638E+00_JPRB, 0.65480E+00_JPRB, 0.65404E+00_JPRB ,&
&  0.66364E+00_JPRB, 0.68283E+00_JPRB, 0.72822E+00_JPRB, 0.80140E+00_JPRB, 0.86495E+00_JPRB, 0.87420E+00_JPRB, 0.89792E+00_JPRB ,&
&  0.87782E+00_JPRB, 0.88433E+00_JPRB, 0.86308E+00_JPRB, 0.79306E+00_JPRB, 0.80999E+00_JPRB, 0.81634E+00_JPRB, 0.85533E+00_JPRB ,&
&  0.89316E+00_JPRB, 0.88906E+00_JPRB, 0.86274E+00_JPRB, 0.84180E+00_JPRB, 0.82338E+00_JPRB, 0.80076E+00_JPRB, 0.77769E+00_JPRB ,&
&  0.75415E+00_JPRB, 0.74039E+00_JPRB, 0.74779E+00_JPRB, 0.76848E+00_JPRB, 0.80308E+00_JPRB, 0.99764E+00_JPRB, 0.11288E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.15055E+01_JPRB, 0.18961E+01_JPRB, 0.20791E+01_JPRB, 0.24966E+01_JPRB, 0.28209E+01_JPRB, 0.32018E+01_JPRB ,&
&  0.35431E+01_JPRB, 0.35211E+01_JPRB, 0.38189E+01_JPRB, 0.38882E+01_JPRB, 0.38837E+01_JPRB, 0.37553E+01_JPRB, 0.35368E+01_JPRB ,&
&  0.34083E+01_JPRB, 0.34348E+01_JPRB, 0.35119E+01_JPRB, 0.34384E+01_JPRB, 0.32433E+01_JPRB, 0.31106E+01_JPRB, 0.29678E+01_JPRB ,&
&  0.28886E+01_JPRB, 0.28091E+01_JPRB, 0.26436E+01_JPRB, 0.24130E+01_JPRB, 0.21294E+01_JPRB, 0.18413E+01_JPRB, 0.15840E+01_JPRB ,&
&  0.13632E+01_JPRB, 0.11263E+01_JPRB, 0.10195E+01_JPRB, 0.99206E+00_JPRB, 0.97316E+00_JPRB, 0.88142E+00_JPRB, 0.84430E+00_JPRB ,&
&  0.82884E+00_JPRB, 0.79410E+00_JPRB, 0.80483E+00_JPRB, 0.78155E+00_JPRB, 0.78939E+00_JPRB, 0.78099E+00_JPRB, 0.72542E+00_JPRB ,&
&  0.68553E+00_JPRB, 0.69044E+00_JPRB, 0.68037E+00_JPRB, 0.67619E+00_JPRB, 0.69085E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.69750E+00_JPRB, 0.71507E+00_JPRB, 0.72120E+00_JPRB, 0.72894E+00_JPRB, 0.73953E+00_JPRB, 0.72911E+00_JPRB, 0.71717E+00_JPRB ,&
&  0.72824E+00_JPRB, 0.69156E+00_JPRB, 0.72643E+00_JPRB, 0.59969E+00_JPRB, 0.54963E+00_JPRB, 0.54151E+00_JPRB, 0.53215E+00_JPRB ,&
&  0.54031E+00_JPRB, 0.56228E+00_JPRB, 0.58297E+00_JPRB, 0.59774E+00_JPRB, 0.62060E+00_JPRB, 0.62585E+00_JPRB, 0.62855E+00_JPRB ,&
&  0.63666E+00_JPRB, 0.65211E+00_JPRB, 0.68986E+00_JPRB, 0.71275E+00_JPRB, 0.74154E+00_JPRB, 0.76577E+00_JPRB, 0.77204E+00_JPRB ,&
&  0.75021E+00_JPRB, 0.75900E+00_JPRB, 0.78476E+00_JPRB, 0.75970E+00_JPRB, 0.73145E+00_JPRB, 0.71417E+00_JPRB, 0.72781E+00_JPRB ,&
&  0.73000E+00_JPRB, 0.68034E+00_JPRB, 0.70018E+00_JPRB, 0.70124E+00_JPRB, 0.69847E+00_JPRB, 0.69477E+00_JPRB, 0.69112E+00_JPRB ,&
&  0.68909E+00_JPRB, 0.68949E+00_JPRB, 0.70499E+00_JPRB, 0.72744E+00_JPRB, 0.77249E+00_JPRB, 0.85451E+00_JPRB, 0.90640E+00_JPRB ,&
&  0.10474E+01_JPRB, 0.10638E+01_JPRB, 0.14266E+01_JPRB, 0.17895E+01_JPRB, 0.23091E+01_JPRB, 0.26368E+01_JPRB, 0.29792E+01_JPRB ,&
&  0.34030E+01_JPRB, 0.33593E+01_JPRB, 0.34972E+01_JPRB, 0.34671E+01_JPRB, 0.34062E+01_JPRB, 0.33255E+01_JPRB, 0.32473E+01_JPRB ,&
&  0.31941E+01_JPRB, 0.30390E+01_JPRB, 0.31687E+01_JPRB, 0.30522E+01_JPRB, 0.28572E+01_JPRB, 0.27207E+01_JPRB, 0.25904E+01_JPRB ,&
&  0.24615E+01_JPRB, 0.23210E+01_JPRB, 0.21513E+01_JPRB, 0.18032E+01_JPRB, 0.16507E+01_JPRB, 0.15272E+01_JPRB, 0.13391E+01_JPRB ,&
&  0.12488E+01_JPRB, 0.11518E+01_JPRB, 0.10574E+01_JPRB, 0.10500E+01_JPRB, 0.97733E+00_JPRB, 0.90585E+00_JPRB, 0.87625E+00_JPRB ,&
&  0.81604E+00_JPRB, 0.74863E+00_JPRB, 0.74780E+00_JPRB, 0.73860E+00_JPRB, 0.72718E+00_JPRB, 0.71461E+00_JPRB, 0.74378E+00_JPRB ,&
&  0.70740E+00_JPRB, 0.68422E+00_JPRB, 0.69438E+00_JPRB, 0.68188E+00_JPRB, 0.68513E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.66689E+00_JPRB, 0.69007E+00_JPRB, 0.69830E+00_JPRB, 0.69930E+00_JPRB, 0.70191E+00_JPRB, 0.69251E+00_JPRB, 0.65132E+00_JPRB ,&
&  0.63670E+00_JPRB, 0.58294E+00_JPRB, 0.55302E+00_JPRB, 0.53453E+00_JPRB, 0.52815E+00_JPRB, 0.52160E+00_JPRB, 0.51223E+00_JPRB ,&
&  0.51776E+00_JPRB, 0.53553E+00_JPRB, 0.55918E+00_JPRB, 0.57627E+00_JPRB, 0.59069E+00_JPRB, 0.60135E+00_JPRB, 0.61110E+00_JPRB ,&
&  0.62425E+00_JPRB, 0.63941E+00_JPRB, 0.65801E+00_JPRB, 0.67234E+00_JPRB, 0.67874E+00_JPRB, 0.68220E+00_JPRB, 0.67851E+00_JPRB ,&
&  0.67983E+00_JPRB, 0.68383E+00_JPRB, 0.70661E+00_JPRB, 0.71521E+00_JPRB, 0.70532E+00_JPRB, 0.68476E+00_JPRB, 0.64787E+00_JPRB ,&
&  0.57145E+00_JPRB, 0.57016E+00_JPRB, 0.59575E+00_JPRB, 0.60944E+00_JPRB, 0.62130E+00_JPRB, 0.63208E+00_JPRB, 0.64015E+00_JPRB ,&
&  0.64855E+00_JPRB, 0.66292E+00_JPRB, 0.68831E+00_JPRB, 0.71645E+00_JPRB, 0.75795E+00_JPRB, 0.81958E+00_JPRB, 0.87103E+00_JPRB ,&
&  0.94357E+00_JPRB, 0.94153E+00_JPRB, 0.11120E+01_JPRB, 0.14715E+01_JPRB, 0.21960E+01_JPRB, 0.24646E+01_JPRB, 0.26516E+01_JPRB ,&
&  0.28400E+01_JPRB, 0.29042E+01_JPRB, 0.30141E+01_JPRB, 0.30968E+01_JPRB, 0.30773E+01_JPRB, 0.30319E+01_JPRB, 0.30123E+01_JPRB ,&
&  0.28652E+01_JPRB, 0.28394E+01_JPRB, 0.27365E+01_JPRB, 0.26829E+01_JPRB, 0.25617E+01_JPRB, 0.24602E+01_JPRB, 0.23246E+01_JPRB ,&
&  0.21575E+01_JPRB, 0.19809E+01_JPRB, 0.17603E+01_JPRB, 0.15493E+01_JPRB, 0.13828E+01_JPRB, 0.12958E+01_JPRB, 0.12222E+01_JPRB ,&
&  0.11671E+01_JPRB, 0.11332E+01_JPRB, 0.10886E+01_JPRB, 0.10479E+01_JPRB, 0.98137E+00_JPRB, 0.89664E+00_JPRB, 0.86283E+00_JPRB ,&
&  0.77628E+00_JPRB, 0.73175E+00_JPRB, 0.68873E+00_JPRB, 0.67122E+00_JPRB, 0.67253E+00_JPRB, 0.66604E+00_JPRB, 0.65380E+00_JPRB ,&
&  0.66725E+00_JPRB, 0.65129E+00_JPRB, 0.63409E+00_JPRB, 0.63826E+00_JPRB, 0.64498E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.62465E+00_JPRB, 0.64174E+00_JPRB, 0.65806E+00_JPRB, 0.66437E+00_JPRB, 0.66122E+00_JPRB, 0.65378E+00_JPRB, 0.63059E+00_JPRB ,&
&  0.62110E+00_JPRB, 0.59344E+00_JPRB, 0.57184E+00_JPRB, 0.55065E+00_JPRB, 0.53612E+00_JPRB, 0.52746E+00_JPRB, 0.53042E+00_JPRB ,&
&  0.53923E+00_JPRB, 0.54955E+00_JPRB, 0.56053E+00_JPRB, 0.56960E+00_JPRB, 0.58095E+00_JPRB, 0.59523E+00_JPRB, 0.61309E+00_JPRB ,&
&  0.63086E+00_JPRB, 0.64251E+00_JPRB, 0.65283E+00_JPRB, 0.66068E+00_JPRB, 0.66714E+00_JPRB, 0.66995E+00_JPRB, 0.65923E+00_JPRB ,&
&  0.65989E+00_JPRB, 0.66069E+00_JPRB, 0.67732E+00_JPRB, 0.68776E+00_JPRB, 0.68937E+00_JPRB, 0.65998E+00_JPRB, 0.58342E+00_JPRB ,&
&  0.53302E+00_JPRB, 0.50914E+00_JPRB, 0.51282E+00_JPRB, 0.54075E+00_JPRB, 0.55898E+00_JPRB, 0.57635E+00_JPRB, 0.59144E+00_JPRB ,&
&  0.59251E+00_JPRB, 0.59985E+00_JPRB, 0.62739E+00_JPRB, 0.66894E+00_JPRB, 0.70357E+00_JPRB, 0.76960E+00_JPRB, 0.84450E+00_JPRB ,&
&  0.95824E+00_JPRB, 0.11242E+01_JPRB, 0.14071E+01_JPRB, 0.14815E+01_JPRB, 0.16505E+01_JPRB, 0.21665E+01_JPRB, 0.22296E+01_JPRB ,&
&  0.23157E+01_JPRB, 0.24212E+01_JPRB, 0.26061E+01_JPRB, 0.27904E+01_JPRB, 0.27688E+01_JPRB, 0.27223E+01_JPRB, 0.26457E+01_JPRB ,&
&  0.24578E+01_JPRB, 0.23599E+01_JPRB, 0.22686E+01_JPRB, 0.22840E+01_JPRB, 0.22178E+01_JPRB, 0.21341E+01_JPRB, 0.20535E+01_JPRB ,&
&  0.19068E+01_JPRB, 0.17033E+01_JPRB, 0.15052E+01_JPRB, 0.13287E+01_JPRB, 0.12392E+01_JPRB, 0.11851E+01_JPRB, 0.11497E+01_JPRB ,&
&  0.11090E+01_JPRB, 0.10772E+01_JPRB, 0.10429E+01_JPRB, 0.10070E+01_JPRB, 0.91070E+00_JPRB, 0.79490E+00_JPRB, 0.73310E+00_JPRB ,&
&  0.69743E+00_JPRB, 0.67944E+00_JPRB, 0.65776E+00_JPRB, 0.64810E+00_JPRB, 0.64241E+00_JPRB, 0.62846E+00_JPRB, 0.60016E+00_JPRB ,&
&  0.59015E+00_JPRB, 0.58073E+00_JPRB, 0.56926E+00_JPRB, 0.57480E+00_JPRB, 0.59990E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.59398E+00_JPRB, 0.60037E+00_JPRB, 0.60759E+00_JPRB, 0.61833E+00_JPRB, 0.61941E+00_JPRB, 0.60583E+00_JPRB, 0.58060E+00_JPRB ,&
&  0.56339E+00_JPRB, 0.55618E+00_JPRB, 0.55850E+00_JPRB, 0.56284E+00_JPRB, 0.56546E+00_JPRB, 0.56648E+00_JPRB, 0.56584E+00_JPRB ,&
&  0.56288E+00_JPRB, 0.56355E+00_JPRB, 0.57078E+00_JPRB, 0.58325E+00_JPRB, 0.59891E+00_JPRB, 0.61536E+00_JPRB, 0.63221E+00_JPRB ,&
&  0.64490E+00_JPRB, 0.64886E+00_JPRB, 0.65072E+00_JPRB, 0.64911E+00_JPRB, 0.65030E+00_JPRB, 0.65418E+00_JPRB, 0.65138E+00_JPRB ,&
&  0.64400E+00_JPRB, 0.64544E+00_JPRB, 0.65682E+00_JPRB, 0.65959E+00_JPRB, 0.66386E+00_JPRB, 0.63489E+00_JPRB, 0.57588E+00_JPRB ,&
&  0.52064E+00_JPRB, 0.47556E+00_JPRB, 0.46133E+00_JPRB, 0.46501E+00_JPRB, 0.48247E+00_JPRB, 0.50790E+00_JPRB, 0.53163E+00_JPRB ,&
&  0.55105E+00_JPRB, 0.57109E+00_JPRB, 0.58854E+00_JPRB, 0.60049E+00_JPRB, 0.61865E+00_JPRB, 0.65594E+00_JPRB, 0.72146E+00_JPRB ,&
&  0.83729E+00_JPRB, 0.99479E+00_JPRB, 0.12047E+01_JPRB, 0.12114E+01_JPRB, 0.13011E+01_JPRB, 0.14936E+01_JPRB, 0.17452E+01_JPRB ,&
&  0.19002E+01_JPRB, 0.19069E+01_JPRB, 0.19713E+01_JPRB, 0.21106E+01_JPRB, 0.21999E+01_JPRB, 0.22685E+01_JPRB, 0.22097E+01_JPRB ,&
&  0.21187E+01_JPRB, 0.20057E+01_JPRB, 0.19068E+01_JPRB, 0.18646E+01_JPRB, 0.18620E+01_JPRB, 0.18481E+01_JPRB, 0.17918E+01_JPRB ,&
&  0.16945E+01_JPRB, 0.15454E+01_JPRB, 0.13716E+01_JPRB, 0.12192E+01_JPRB, 0.11685E+01_JPRB, 0.11658E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11380E+01_JPRB, 0.10883E+01_JPRB, 0.10289E+01_JPRB, 0.97821E+00_JPRB, 0.87929E+00_JPRB, 0.78948E+00_JPRB, 0.73536E+00_JPRB ,&
&  0.71544E+00_JPRB, 0.70628E+00_JPRB, 0.69870E+00_JPRB, 0.68726E+00_JPRB, 0.67264E+00_JPRB, 0.65979E+00_JPRB, 0.63039E+00_JPRB ,&
&  0.60508E+00_JPRB, 0.58704E+00_JPRB, 0.57044E+00_JPRB, 0.56816E+00_JPRB, 0.57753E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.60507E+00_JPRB, 0.60819E+00_JPRB, 0.60912E+00_JPRB, 0.60807E+00_JPRB, 0.59934E+00_JPRB, 0.59505E+00_JPRB, 0.58510E+00_JPRB ,&
&  0.57912E+00_JPRB, 0.57874E+00_JPRB, 0.57404E+00_JPRB, 0.57226E+00_JPRB, 0.57141E+00_JPRB, 0.57069E+00_JPRB, 0.57250E+00_JPRB ,&
&  0.57771E+00_JPRB, 0.58678E+00_JPRB, 0.59869E+00_JPRB, 0.61140E+00_JPRB, 0.62477E+00_JPRB, 0.63866E+00_JPRB, 0.65052E+00_JPRB ,&
&  0.65672E+00_JPRB, 0.65563E+00_JPRB, 0.65755E+00_JPRB, 0.65480E+00_JPRB, 0.64723E+00_JPRB, 0.63674E+00_JPRB, 0.62536E+00_JPRB ,&
&  0.62541E+00_JPRB, 0.63658E+00_JPRB, 0.63846E+00_JPRB, 0.63900E+00_JPRB, 0.63979E+00_JPRB, 0.60528E+00_JPRB, 0.55420E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.45093E+00_JPRB, 0.43118E+00_JPRB, 0.42434E+00_JPRB, 0.43260E+00_JPRB, 0.44926E+00_JPRB, 0.46944E+00_JPRB ,&
&  0.48543E+00_JPRB, 0.51242E+00_JPRB, 0.54002E+00_JPRB, 0.55990E+00_JPRB, 0.57771E+00_JPRB, 0.59841E+00_JPRB, 0.61913E+00_JPRB ,&
&  0.65206E+00_JPRB, 0.71135E+00_JPRB, 0.81974E+00_JPRB, 0.96021E+00_JPRB, 0.10909E+01_JPRB, 0.11397E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.15684E+01_JPRB, 0.15330E+01_JPRB, 0.15208E+01_JPRB, 0.15190E+01_JPRB, 0.15096E+01_JPRB, 0.15135E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.15962E+01_JPRB, 0.16339E+01_JPRB, 0.16221E+01_JPRB, 0.15816E+01_JPRB, 0.15504E+01_JPRB, 0.15354E+01_JPRB, 0.15424E+01_JPRB ,&
&  0.15401E+01_JPRB, 0.15754E+01_JPRB, 0.17117E+01_JPRB, 0.14564E+01_JPRB, 0.13387E+01_JPRB, 0.12869E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.11057E+01_JPRB, 0.10489E+01_JPRB, 0.98538E+00_JPRB, 0.91310E+00_JPRB, 0.84538E+00_JPRB, 0.79683E+00_JPRB ,&
&  0.77537E+00_JPRB, 0.76177E+00_JPRB, 0.75258E+00_JPRB, 0.73919E+00_JPRB, 0.71458E+00_JPRB, 0.69514E+00_JPRB, 0.68040E+00_JPRB ,&
&  0.65887E+00_JPRB, 0.63297E+00_JPRB, 0.61503E+00_JPRB, 0.60971E+00_JPRB, 0.60475E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.62827E+00_JPRB, 0.62834E+00_JPRB, 0.62684E+00_JPRB, 0.62901E+00_JPRB, 0.62380E+00_JPRB, 0.62139E+00_JPRB, 0.61093E+00_JPRB ,&
&  0.60408E+00_JPRB, 0.59876E+00_JPRB, 0.59575E+00_JPRB, 0.59546E+00_JPRB, 0.59709E+00_JPRB, 0.60255E+00_JPRB, 0.61094E+00_JPRB ,&
&  0.62145E+00_JPRB, 0.62456E+00_JPRB, 0.63239E+00_JPRB, 0.64365E+00_JPRB, 0.65091E+00_JPRB, 0.65955E+00_JPRB, 0.66487E+00_JPRB ,&
&  0.66518E+00_JPRB, 0.65903E+00_JPRB, 0.65409E+00_JPRB, 0.64351E+00_JPRB, 0.63519E+00_JPRB, 0.62712E+00_JPRB, 0.62567E+00_JPRB ,&
&  0.61558E+00_JPRB, 0.61061E+00_JPRB, 0.62804E+00_JPRB, 0.63355E+00_JPRB, 0.59793E+00_JPRB, 0.56128E+00_JPRB, 0.52068E+00_JPRB ,&
&  0.47769E+00_JPRB, 0.43620E+00_JPRB, 0.42161E+00_JPRB, 0.41326E+00_JPRB, 0.41771E+00_JPRB, 0.42566E+00_JPRB, 0.44180E+00_JPRB ,&
&  0.45724E+00_JPRB, 0.47958E+00_JPRB, 0.50058E+00_JPRB, 0.52574E+00_JPRB, 0.54468E+00_JPRB, 0.56748E+00_JPRB, 0.58498E+00_JPRB ,&
&  0.60676E+00_JPRB, 0.63709E+00_JPRB, 0.68415E+00_JPRB, 0.74663E+00_JPRB, 0.80020E+00_JPRB, 0.85259E+00_JPRB, 0.90605E+00_JPRB ,&
&  0.10156E+01_JPRB, 0.10445E+01_JPRB, 0.10904E+01_JPRB, 0.11124E+01_JPRB, 0.11323E+01_JPRB, 0.11644E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.12292E+01_JPRB, 0.13130E+01_JPRB, 0.13327E+01_JPRB, 0.13332E+01_JPRB, 0.13239E+01_JPRB, 0.13322E+01_JPRB, 0.13572E+01_JPRB ,&
&  0.13598E+01_JPRB, 0.13783E+01_JPRB, 0.14753E+01_JPRB, 0.14168E+01_JPRB, 0.13643E+01_JPRB, 0.13049E+01_JPRB, 0.12365E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11090E+01_JPRB, 0.10457E+01_JPRB, 0.98479E+00_JPRB, 0.93419E+00_JPRB, 0.88892E+00_JPRB, 0.85427E+00_JPRB ,&
&  0.81939E+00_JPRB, 0.78941E+00_JPRB, 0.76646E+00_JPRB, 0.74724E+00_JPRB, 0.72263E+00_JPRB, 0.69899E+00_JPRB, 0.68065E+00_JPRB ,&
&  0.65997E+00_JPRB, 0.64668E+00_JPRB, 0.63586E+00_JPRB, 0.63187E+00_JPRB, 0.62884E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.66386E+00_JPRB, 0.66883E+00_JPRB, 0.67456E+00_JPRB, 0.67830E+00_JPRB, 0.67688E+00_JPRB, 0.67518E+00_JPRB, 0.66841E+00_JPRB ,&
&  0.66478E+00_JPRB, 0.66101E+00_JPRB, 0.65872E+00_JPRB, 0.65843E+00_JPRB, 0.65826E+00_JPRB, 0.66018E+00_JPRB, 0.66286E+00_JPRB ,&
&  0.66519E+00_JPRB, 0.66855E+00_JPRB, 0.66923E+00_JPRB, 0.67124E+00_JPRB, 0.67201E+00_JPRB, 0.67589E+00_JPRB, 0.67598E+00_JPRB ,&
&  0.67630E+00_JPRB, 0.66793E+00_JPRB, 0.65039E+00_JPRB, 0.63988E+00_JPRB, 0.63768E+00_JPRB, 0.61918E+00_JPRB, 0.62015E+00_JPRB ,&
&  0.62217E+00_JPRB, 0.61991E+00_JPRB, 0.61133E+00_JPRB, 0.60596E+00_JPRB, 0.58348E+00_JPRB, 0.49792E+00_JPRB, 0.49096E+00_JPRB ,&
&  0.47018E+00_JPRB, 0.44338E+00_JPRB, 0.42965E+00_JPRB, 0.41923E+00_JPRB, 0.41603E+00_JPRB, 0.42606E+00_JPRB, 0.44185E+00_JPRB ,&
&  0.45213E+00_JPRB, 0.48705E+00_JPRB, 0.50197E+00_JPRB, 0.52226E+00_JPRB, 0.54006E+00_JPRB, 0.56495E+00_JPRB, 0.57992E+00_JPRB ,&
&  0.59775E+00_JPRB, 0.61676E+00_JPRB, 0.65039E+00_JPRB, 0.69883E+00_JPRB, 0.74024E+00_JPRB, 0.77649E+00_JPRB, 0.80046E+00_JPRB ,&
&  0.86360E+00_JPRB, 0.88751E+00_JPRB, 0.92610E+00_JPRB, 0.95328E+00_JPRB, 0.97265E+00_JPRB, 0.99436E+00_JPRB, 0.10108E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10888E+01_JPRB, 0.11333E+01_JPRB, 0.11565E+01_JPRB, 0.11682E+01_JPRB, 0.11833E+01_JPRB, 0.12022E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.12475E+01_JPRB, 0.12494E+01_JPRB, 0.12181E+01_JPRB, 0.11980E+01_JPRB, 0.11547E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.10577E+01_JPRB, 0.10121E+01_JPRB, 0.97620E+00_JPRB, 0.93680E+00_JPRB, 0.90514E+00_JPRB, 0.86611E+00_JPRB ,&
&  0.82504E+00_JPRB, 0.78453E+00_JPRB, 0.75330E+00_JPRB, 0.72957E+00_JPRB, 0.70884E+00_JPRB, 0.68659E+00_JPRB, 0.67317E+00_JPRB ,&
&  0.66351E+00_JPRB, 0.65677E+00_JPRB, 0.65218E+00_JPRB, 0.65515E+00_JPRB, 0.65756E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.67736E+00_JPRB, 0.68496E+00_JPRB, 0.69703E+00_JPRB, 0.70708E+00_JPRB, 0.71375E+00_JPRB, 0.71747E+00_JPRB, 0.71601E+00_JPRB ,&
&  0.71371E+00_JPRB, 0.70891E+00_JPRB, 0.70607E+00_JPRB, 0.70244E+00_JPRB, 0.70095E+00_JPRB, 0.69745E+00_JPRB, 0.69573E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.69150E+00_JPRB, 0.68873E+00_JPRB, 0.68630E+00_JPRB, 0.68392E+00_JPRB, 0.68299E+00_JPRB, 0.67962E+00_JPRB ,&
&  0.67760E+00_JPRB, 0.66911E+00_JPRB, 0.64850E+00_JPRB, 0.64461E+00_JPRB, 0.63507E+00_JPRB, 0.62385E+00_JPRB, 0.61094E+00_JPRB ,&
&  0.59656E+00_JPRB, 0.55459E+00_JPRB, 0.56714E+00_JPRB, 0.54808E+00_JPRB, 0.52823E+00_JPRB, 0.50915E+00_JPRB, 0.48905E+00_JPRB ,&
&  0.47700E+00_JPRB, 0.46169E+00_JPRB, 0.45027E+00_JPRB, 0.44572E+00_JPRB, 0.44545E+00_JPRB, 0.45445E+00_JPRB, 0.46712E+00_JPRB ,&
&  0.48021E+00_JPRB, 0.51023E+00_JPRB, 0.52637E+00_JPRB, 0.54572E+00_JPRB, 0.56474E+00_JPRB, 0.58742E+00_JPRB, 0.59661E+00_JPRB ,&
&  0.61413E+00_JPRB, 0.62120E+00_JPRB, 0.64474E+00_JPRB, 0.67620E+00_JPRB, 0.71466E+00_JPRB, 0.74779E+00_JPRB, 0.77846E+00_JPRB ,&
&  0.82325E+00_JPRB, 0.85520E+00_JPRB, 0.88549E+00_JPRB, 0.90464E+00_JPRB, 0.92599E+00_JPRB, 0.94137E+00_JPRB, 0.95708E+00_JPRB ,&
&  0.96656E+00_JPRB, 0.98211E+00_JPRB, 0.10006E+01_JPRB, 0.10214E+01_JPRB, 0.10313E+01_JPRB, 0.10461E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.10886E+01_JPRB, 0.11089E+01_JPRB, 0.10967E+01_JPRB, 0.10898E+01_JPRB, 0.10568E+01_JPRB, 0.10306E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.98581E+00_JPRB, 0.96024E+00_JPRB, 0.93441E+00_JPRB, 0.90914E+00_JPRB, 0.88555E+00_JPRB, 0.85996E+00_JPRB ,&
&  0.83041E+00_JPRB, 0.79388E+00_JPRB, 0.76282E+00_JPRB, 0.72964E+00_JPRB, 0.70743E+00_JPRB, 0.69139E+00_JPRB, 0.68405E+00_JPRB ,&
&  0.67849E+00_JPRB, 0.67556E+00_JPRB, 0.67316E+00_JPRB, 0.67565E+00_JPRB, 0.67564E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.72445E+00_JPRB, 0.72491E+00_JPRB, 0.72636E+00_JPRB, 0.72661E+00_JPRB, 0.72668E+00_JPRB, 0.72524E+00_JPRB, 0.72579E+00_JPRB ,&
&  0.72521E+00_JPRB, 0.72324E+00_JPRB, 0.72051E+00_JPRB, 0.71667E+00_JPRB, 0.71450E+00_JPRB, 0.71058E+00_JPRB, 0.70699E+00_JPRB ,&
&  0.70226E+00_JPRB, 0.69744E+00_JPRB, 0.69447E+00_JPRB, 0.69277E+00_JPRB, 0.69026E+00_JPRB, 0.68865E+00_JPRB, 0.68135E+00_JPRB ,&
&  0.67626E+00_JPRB, 0.67167E+00_JPRB, 0.64420E+00_JPRB, 0.64496E+00_JPRB, 0.63277E+00_JPRB, 0.62085E+00_JPRB, 0.61193E+00_JPRB ,&
&  0.58946E+00_JPRB, 0.57863E+00_JPRB, 0.55775E+00_JPRB, 0.53993E+00_JPRB, 0.53466E+00_JPRB, 0.51885E+00_JPRB, 0.51479E+00_JPRB ,&
&  0.50883E+00_JPRB, 0.50371E+00_JPRB, 0.50165E+00_JPRB, 0.50309E+00_JPRB, 0.50951E+00_JPRB, 0.51815E+00_JPRB, 0.52801E+00_JPRB ,&
&  0.54434E+00_JPRB, 0.56051E+00_JPRB, 0.58399E+00_JPRB, 0.59597E+00_JPRB, 0.61658E+00_JPRB, 0.63968E+00_JPRB, 0.66397E+00_JPRB ,&
&  0.67950E+00_JPRB, 0.67705E+00_JPRB, 0.68320E+00_JPRB, 0.69693E+00_JPRB, 0.71078E+00_JPRB, 0.72683E+00_JPRB, 0.75202E+00_JPRB ,&
&  0.78596E+00_JPRB, 0.81872E+00_JPRB, 0.84079E+00_JPRB, 0.85098E+00_JPRB, 0.86107E+00_JPRB, 0.86830E+00_JPRB, 0.88113E+00_JPRB ,&
&  0.89264E+00_JPRB, 0.90714E+00_JPRB, 0.91797E+00_JPRB, 0.92842E+00_JPRB, 0.93216E+00_JPRB, 0.94009E+00_JPRB, 0.94409E+00_JPRB ,&
&  0.94871E+00_JPRB, 0.95064E+00_JPRB, 0.96059E+00_JPRB, 0.94892E+00_JPRB, 0.95052E+00_JPRB, 0.94753E+00_JPRB, 0.93916E+00_JPRB ,&
&  0.92690E+00_JPRB, 0.91647E+00_JPRB, 0.90284E+00_JPRB, 0.88778E+00_JPRB, 0.86922E+00_JPRB, 0.85213E+00_JPRB, 0.83463E+00_JPRB ,&
&  0.81831E+00_JPRB, 0.79947E+00_JPRB, 0.78268E+00_JPRB, 0.75799E+00_JPRB, 0.74304E+00_JPRB, 0.72951E+00_JPRB, 0.72389E+00_JPRB ,&
&  0.71905E+00_JPRB, 0.71945E+00_JPRB, 0.71838E+00_JPRB, 0.72135E+00_JPRB, 0.72255E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.75703E+00_JPRB, 0.75706E+00_JPRB, 0.75676E+00_JPRB, 0.75527E+00_JPRB, 0.75362E+00_JPRB, 0.75143E+00_JPRB, 0.74904E+00_JPRB ,&
&  0.74636E+00_JPRB, 0.74308E+00_JPRB, 0.73932E+00_JPRB, 0.73500E+00_JPRB, 0.73026E+00_JPRB, 0.72448E+00_JPRB, 0.72040E+00_JPRB ,&
&  0.71557E+00_JPRB, 0.71115E+00_JPRB, 0.70697E+00_JPRB, 0.70330E+00_JPRB, 0.69942E+00_JPRB, 0.69585E+00_JPRB, 0.68839E+00_JPRB ,&
&  0.68387E+00_JPRB, 0.67994E+00_JPRB, 0.67460E+00_JPRB, 0.66651E+00_JPRB, 0.66205E+00_JPRB, 0.63299E+00_JPRB, 0.63357E+00_JPRB ,&
&  0.64567E+00_JPRB, 0.64150E+00_JPRB, 0.63633E+00_JPRB, 0.63005E+00_JPRB, 0.61440E+00_JPRB, 0.60260E+00_JPRB, 0.59842E+00_JPRB ,&
&  0.57448E+00_JPRB, 0.58384E+00_JPRB, 0.58531E+00_JPRB, 0.58645E+00_JPRB, 0.59088E+00_JPRB, 0.59155E+00_JPRB, 0.59686E+00_JPRB ,&
&  0.61794E+00_JPRB, 0.62435E+00_JPRB, 0.63759E+00_JPRB, 0.65345E+00_JPRB, 0.67098E+00_JPRB, 0.68834E+00_JPRB, 0.70550E+00_JPRB ,&
&  0.71852E+00_JPRB, 0.72762E+00_JPRB, 0.73399E+00_JPRB, 0.73886E+00_JPRB, 0.74596E+00_JPRB, 0.75551E+00_JPRB, 0.77094E+00_JPRB ,&
&  0.79503E+00_JPRB, 0.82136E+00_JPRB, 0.84016E+00_JPRB, 0.84970E+00_JPRB, 0.86334E+00_JPRB, 0.87666E+00_JPRB, 0.88927E+00_JPRB ,&
&  0.89919E+00_JPRB, 0.90659E+00_JPRB, 0.91091E+00_JPRB, 0.91404E+00_JPRB, 0.91442E+00_JPRB, 0.91053E+00_JPRB, 0.90523E+00_JPRB ,&
&  0.89809E+00_JPRB, 0.88974E+00_JPRB, 0.88179E+00_JPRB, 0.87349E+00_JPRB, 0.86952E+00_JPRB, 0.86522E+00_JPRB, 0.86561E+00_JPRB ,&
&  0.86265E+00_JPRB, 0.86146E+00_JPRB, 0.85576E+00_JPRB, 0.84927E+00_JPRB, 0.83804E+00_JPRB, 0.82683E+00_JPRB, 0.81306E+00_JPRB ,&
&  0.80177E+00_JPRB, 0.79182E+00_JPRB, 0.78312E+00_JPRB, 0.77245E+00_JPRB, 0.76633E+00_JPRB, 0.76043E+00_JPRB, 0.75749E+00_JPRB ,&
&  0.75423E+00_JPRB, 0.75490E+00_JPRB, 0.75504E+00_JPRB, 0.75632E+00_JPRB, 0.75637E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.75870E+00_JPRB, 0.75815E+00_JPRB, 0.75715E+00_JPRB, 0.75579E+00_JPRB, 0.75435E+00_JPRB, 0.75293E+00_JPRB, 0.75113E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.74658E+00_JPRB, 0.74444E+00_JPRB, 0.74206E+00_JPRB, 0.73925E+00_JPRB, 0.73640E+00_JPRB, 0.73416E+00_JPRB ,&
&  0.73179E+00_JPRB, 0.72924E+00_JPRB, 0.72576E+00_JPRB, 0.72297E+00_JPRB, 0.72038E+00_JPRB, 0.71749E+00_JPRB, 0.71550E+00_JPRB ,&
&  0.71357E+00_JPRB, 0.71026E+00_JPRB, 0.70572E+00_JPRB, 0.69749E+00_JPRB, 0.69158E+00_JPRB, 0.68601E+00_JPRB, 0.68085E+00_JPRB ,&
&  0.67561E+00_JPRB, 0.67472E+00_JPRB, 0.67430E+00_JPRB, 0.67389E+00_JPRB, 0.67608E+00_JPRB, 0.67529E+00_JPRB, 0.67397E+00_JPRB ,&
&  0.67117E+00_JPRB, 0.66646E+00_JPRB, 0.66777E+00_JPRB, 0.66932E+00_JPRB, 0.67081E+00_JPRB, 0.67508E+00_JPRB, 0.68015E+00_JPRB ,&
&  0.68608E+00_JPRB, 0.69269E+00_JPRB, 0.70232E+00_JPRB, 0.71339E+00_JPRB, 0.72427E+00_JPRB, 0.73469E+00_JPRB, 0.74500E+00_JPRB ,&
&  0.75368E+00_JPRB, 0.76295E+00_JPRB, 0.77318E+00_JPRB, 0.78474E+00_JPRB, 0.79583E+00_JPRB, 0.80818E+00_JPRB, 0.81876E+00_JPRB ,&
&  0.82606E+00_JPRB, 0.83323E+00_JPRB, 0.84382E+00_JPRB, 0.85629E+00_JPRB, 0.87006E+00_JPRB, 0.87877E+00_JPRB, 0.88585E+00_JPRB ,&
&  0.89106E+00_JPRB, 0.89568E+00_JPRB, 0.89659E+00_JPRB, 0.89585E+00_JPRB, 0.89309E+00_JPRB, 0.88807E+00_JPRB, 0.88162E+00_JPRB ,&
&  0.87460E+00_JPRB, 0.86797E+00_JPRB, 0.86153E+00_JPRB, 0.85475E+00_JPRB, 0.84862E+00_JPRB, 0.84239E+00_JPRB, 0.83774E+00_JPRB ,&
&  0.83239E+00_JPRB, 0.82804E+00_JPRB, 0.82348E+00_JPRB, 0.81937E+00_JPRB, 0.81289E+00_JPRB, 0.80643E+00_JPRB, 0.79912E+00_JPRB ,&
&  0.79304E+00_JPRB, 0.78616E+00_JPRB, 0.77972E+00_JPRB, 0.77376E+00_JPRB, 0.77025E+00_JPRB, 0.76741E+00_JPRB, 0.76506E+00_JPRB ,&
&  0.76279E+00_JPRB, 0.76168E+00_JPRB, 0.76095E+00_JPRB, 0.76009E+00_JPRB, 0.75925E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.76348E+00_JPRB, 0.76227E+00_JPRB, 0.76096E+00_JPRB, 0.75967E+00_JPRB, 0.75819E+00_JPRB, 0.75676E+00_JPRB, 0.75524E+00_JPRB ,&
&  0.75355E+00_JPRB, 0.75183E+00_JPRB, 0.75041E+00_JPRB, 0.74909E+00_JPRB, 0.74780E+00_JPRB, 0.74649E+00_JPRB, 0.74543E+00_JPRB ,&
&  0.74446E+00_JPRB, 0.74330E+00_JPRB, 0.74200E+00_JPRB, 0.74130E+00_JPRB, 0.74074E+00_JPRB, 0.73998E+00_JPRB, 0.73940E+00_JPRB ,&
&  0.73862E+00_JPRB, 0.73700E+00_JPRB, 0.73515E+00_JPRB, 0.73405E+00_JPRB, 0.73318E+00_JPRB, 0.73249E+00_JPRB, 0.73174E+00_JPRB ,&
&  0.73102E+00_JPRB, 0.73057E+00_JPRB, 0.72992E+00_JPRB, 0.72929E+00_JPRB, 0.72926E+00_JPRB, 0.72971E+00_JPRB, 0.73031E+00_JPRB ,&
&  0.73098E+00_JPRB, 0.73188E+00_JPRB, 0.73330E+00_JPRB, 0.73473E+00_JPRB, 0.73614E+00_JPRB, 0.73753E+00_JPRB, 0.73945E+00_JPRB ,&
&  0.74115E+00_JPRB, 0.74291E+00_JPRB, 0.74521E+00_JPRB, 0.74792E+00_JPRB, 0.75076E+00_JPRB, 0.75378E+00_JPRB, 0.75691E+00_JPRB ,&
&  0.76102E+00_JPRB, 0.76570E+00_JPRB, 0.77071E+00_JPRB, 0.77568E+00_JPRB, 0.78038E+00_JPRB, 0.78519E+00_JPRB, 0.79043E+00_JPRB ,&
&  0.79513E+00_JPRB, 0.79921E+00_JPRB, 0.80378E+00_JPRB, 0.80871E+00_JPRB, 0.81324E+00_JPRB, 0.81590E+00_JPRB, 0.81881E+00_JPRB ,&
&  0.82196E+00_JPRB, 0.82493E+00_JPRB, 0.82546E+00_JPRB, 0.82621E+00_JPRB, 0.82681E+00_JPRB, 0.82687E+00_JPRB, 0.82617E+00_JPRB ,&
&  0.82533E+00_JPRB, 0.82460E+00_JPRB, 0.82278E+00_JPRB, 0.82016E+00_JPRB, 0.81766E+00_JPRB, 0.81486E+00_JPRB, 0.81167E+00_JPRB ,&
&  0.80828E+00_JPRB, 0.80528E+00_JPRB, 0.80241E+00_JPRB, 0.79938E+00_JPRB, 0.79603E+00_JPRB, 0.79295E+00_JPRB, 0.78990E+00_JPRB ,&
&  0.78708E+00_JPRB, 0.78407E+00_JPRB, 0.78127E+00_JPRB, 0.77856E+00_JPRB, 0.77627E+00_JPRB, 0.77421E+00_JPRB, 0.77236E+00_JPRB ,&
&  0.77056E+00_JPRB, 0.76897E+00_JPRB, 0.76761E+00_JPRB, 0.76623E+00_JPRB, 0.76486E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB ,&
&  0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB, 0.76367E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB ,&
&  0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB, 0.23250E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.26073E+00_JPRB, 0.26076E+00_JPRB, 0.26082E+00_JPRB, 0.26047E+00_JPRB, 0.25896E+00_JPRB, 0.25783E+00_JPRB, 0.25683E+00_JPRB ,&
&  0.25635E+00_JPRB, 0.25551E+00_JPRB, 0.25509E+00_JPRB, 0.25492E+00_JPRB, 0.25455E+00_JPRB, 0.25400E+00_JPRB, 0.25329E+00_JPRB ,&
&  0.25236E+00_JPRB, 0.25114E+00_JPRB, 0.24992E+00_JPRB, 0.24972E+00_JPRB, 0.24988E+00_JPRB, 0.25005E+00_JPRB, 0.25016E+00_JPRB ,&
&  0.25016E+00_JPRB, 0.25013E+00_JPRB, 0.24965E+00_JPRB, 0.24970E+00_JPRB, 0.24972E+00_JPRB, 0.24972E+00_JPRB, 0.24985E+00_JPRB ,&
&  0.25008E+00_JPRB, 0.25028E+00_JPRB, 0.25004E+00_JPRB, 0.24957E+00_JPRB, 0.24867E+00_JPRB, 0.24700E+00_JPRB, 0.24438E+00_JPRB ,&
&  0.24141E+00_JPRB, 0.23843E+00_JPRB, 0.23604E+00_JPRB, 0.23385E+00_JPRB, 0.23183E+00_JPRB, 0.23000E+00_JPRB, 0.22848E+00_JPRB ,&
&  0.22666E+00_JPRB, 0.22487E+00_JPRB, 0.22323E+00_JPRB, 0.22207E+00_JPRB, 0.22081E+00_JPRB, 0.21971E+00_JPRB, 0.21873E+00_JPRB ,&
&  0.21805E+00_JPRB, 0.21746E+00_JPRB, 0.21694E+00_JPRB, 0.21668E+00_JPRB, 0.21648E+00_JPRB, 0.21628E+00_JPRB, 0.21606E+00_JPRB ,&
&  0.21586E+00_JPRB, 0.21571E+00_JPRB, 0.21558E+00_JPRB, 0.21553E+00_JPRB, 0.21560E+00_JPRB, 0.21547E+00_JPRB, 0.21532E+00_JPRB ,&
&  0.21526E+00_JPRB, 0.21550E+00_JPRB, 0.21568E+00_JPRB, 0.21598E+00_JPRB, 0.21645E+00_JPRB, 0.21712E+00_JPRB, 0.21784E+00_JPRB ,&
&  0.21857E+00_JPRB, 0.21942E+00_JPRB, 0.22035E+00_JPRB, 0.22148E+00_JPRB, 0.22269E+00_JPRB, 0.22389E+00_JPRB, 0.22504E+00_JPRB ,&
&  0.22628E+00_JPRB, 0.22741E+00_JPRB, 0.22848E+00_JPRB, 0.22925E+00_JPRB, 0.23031E+00_JPRB, 0.23139E+00_JPRB, 0.23263E+00_JPRB ,&
&  0.23405E+00_JPRB, 0.23592E+00_JPRB, 0.23847E+00_JPRB, 0.24150E+00_JPRB, 0.24525E+00_JPRB, 0.24805E+00_JPRB, 0.25092E+00_JPRB ,&
&  0.25302E+00_JPRB, 0.25463E+00_JPRB, 0.25581E+00_JPRB, 0.25734E+00_JPRB, 0.25902E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.29597E+00_JPRB, 0.30174E+00_JPRB, 0.30581E+00_JPRB, 0.31494E+00_JPRB, 0.32864E+00_JPRB, 0.42493E+00_JPRB, 0.41993E+00_JPRB ,&
&  0.41894E+00_JPRB, 0.37611E+00_JPRB, 0.36796E+00_JPRB, 0.35577E+00_JPRB, 0.34044E+00_JPRB, 0.32147E+00_JPRB, 0.30968E+00_JPRB ,&
&  0.29988E+00_JPRB, 0.28851E+00_JPRB, 0.27780E+00_JPRB, 0.27156E+00_JPRB, 0.26659E+00_JPRB, 0.26225E+00_JPRB, 0.25978E+00_JPRB ,&
&  0.25894E+00_JPRB, 0.25950E+00_JPRB, 0.26040E+00_JPRB, 0.26180E+00_JPRB, 0.26524E+00_JPRB, 0.26896E+00_JPRB, 0.27268E+00_JPRB ,&
&  0.27459E+00_JPRB, 0.27867E+00_JPRB, 0.27940E+00_JPRB, 0.27620E+00_JPRB, 0.27090E+00_JPRB, 0.26376E+00_JPRB, 0.25655E+00_JPRB ,&
&  0.24911E+00_JPRB, 0.24217E+00_JPRB, 0.23644E+00_JPRB, 0.23124E+00_JPRB, 0.22537E+00_JPRB, 0.21990E+00_JPRB, 0.21641E+00_JPRB ,&
&  0.21328E+00_JPRB, 0.21010E+00_JPRB, 0.20762E+00_JPRB, 0.20636E+00_JPRB, 0.20495E+00_JPRB, 0.20400E+00_JPRB, 0.20331E+00_JPRB ,&
&  0.20312E+00_JPRB, 0.20302E+00_JPRB, 0.20309E+00_JPRB, 0.20338E+00_JPRB, 0.20335E+00_JPRB, 0.20328E+00_JPRB, 0.20325E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20381E+00_JPRB, 0.20371E+00_JPRB, 0.20347E+00_JPRB, 0.20407E+00_JPRB, 0.20422E+00_JPRB, 0.20453E+00_JPRB ,&
&  0.20510E+00_JPRB, 0.20591E+00_JPRB, 0.20655E+00_JPRB, 0.20728E+00_JPRB, 0.20800E+00_JPRB, 0.20884E+00_JPRB, 0.20982E+00_JPRB ,&
&  0.21082E+00_JPRB, 0.21229E+00_JPRB, 0.21380E+00_JPRB, 0.21515E+00_JPRB, 0.21634E+00_JPRB, 0.21796E+00_JPRB, 0.21979E+00_JPRB ,&
&  0.22252E+00_JPRB, 0.22519E+00_JPRB, 0.22793E+00_JPRB, 0.22965E+00_JPRB, 0.23235E+00_JPRB, 0.23460E+00_JPRB, 0.23697E+00_JPRB ,&
&  0.23970E+00_JPRB, 0.24403E+00_JPRB, 0.25022E+00_JPRB, 0.25690E+00_JPRB, 0.26423E+00_JPRB, 0.27076E+00_JPRB, 0.27510E+00_JPRB ,&
&  0.27894E+00_JPRB, 0.28305E+00_JPRB, 0.28888E+00_JPRB, 0.29200E+00_JPRB, 0.29450E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.51419E+00_JPRB, 0.51649E+00_JPRB, 0.51899E+00_JPRB, 0.52299E+00_JPRB, 0.52726E+00_JPRB, 0.53156E+00_JPRB, 0.53536E+00_JPRB ,&
&  0.51718E+00_JPRB, 0.49404E+00_JPRB, 0.47508E+00_JPRB, 0.45256E+00_JPRB, 0.43202E+00_JPRB, 0.41523E+00_JPRB, 0.39513E+00_JPRB ,&
&  0.37511E+00_JPRB, 0.35509E+00_JPRB, 0.33246E+00_JPRB, 0.31391E+00_JPRB, 0.29565E+00_JPRB, 0.28087E+00_JPRB, 0.27173E+00_JPRB ,&
&  0.26684E+00_JPRB, 0.26554E+00_JPRB, 0.26706E+00_JPRB, 0.27029E+00_JPRB, 0.27329E+00_JPRB, 0.27805E+00_JPRB, 0.28460E+00_JPRB ,&
&  0.29363E+00_JPRB, 0.30799E+00_JPRB, 0.32096E+00_JPRB, 0.32572E+00_JPRB, 0.32229E+00_JPRB, 0.31397E+00_JPRB, 0.30681E+00_JPRB ,&
&  0.29812E+00_JPRB, 0.28590E+00_JPRB, 0.27116E+00_JPRB, 0.25660E+00_JPRB, 0.24305E+00_JPRB, 0.23316E+00_JPRB, 0.22501E+00_JPRB ,&
&  0.21814E+00_JPRB, 0.21211E+00_JPRB, 0.20749E+00_JPRB, 0.20437E+00_JPRB, 0.20169E+00_JPRB, 0.19953E+00_JPRB, 0.19749E+00_JPRB ,&
&  0.19645E+00_JPRB, 0.19536E+00_JPRB, 0.19470E+00_JPRB, 0.19433E+00_JPRB, 0.19385E+00_JPRB, 0.19371E+00_JPRB, 0.19361E+00_JPRB ,&
&  0.19327E+00_JPRB, 0.19264E+00_JPRB, 0.19228E+00_JPRB, 0.19204E+00_JPRB, 0.19243E+00_JPRB, 0.19273E+00_JPRB, 0.19324E+00_JPRB ,&
&  0.19404E+00_JPRB, 0.19577E+00_JPRB, 0.19687E+00_JPRB, 0.19791E+00_JPRB, 0.19845E+00_JPRB, 0.19890E+00_JPRB, 0.19951E+00_JPRB ,&
&  0.20052E+00_JPRB, 0.20175E+00_JPRB, 0.20288E+00_JPRB, 0.20438E+00_JPRB, 0.20592E+00_JPRB, 0.20803E+00_JPRB, 0.21029E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21619E+00_JPRB, 0.22025E+00_JPRB, 0.22398E+00_JPRB, 0.22963E+00_JPRB, 0.23531E+00_JPRB, 0.24164E+00_JPRB ,&
&  0.24810E+00_JPRB, 0.25507E+00_JPRB, 0.26415E+00_JPRB, 0.27488E+00_JPRB, 0.28824E+00_JPRB, 0.29682E+00_JPRB, 0.30838E+00_JPRB ,&
&  0.32602E+00_JPRB, 0.36317E+00_JPRB, 0.49836E+00_JPRB, 0.51181E+00_JPRB, 0.51630E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.52912E+00_JPRB, 0.53823E+00_JPRB, 0.55259E+00_JPRB, 0.56424E+00_JPRB, 0.58861E+00_JPRB, 0.60233E+00_JPRB, 0.64852E+00_JPRB ,&
&  0.60272E+00_JPRB, 0.52367E+00_JPRB, 0.49170E+00_JPRB, 0.45705E+00_JPRB, 0.41744E+00_JPRB, 0.38895E+00_JPRB, 0.36884E+00_JPRB ,&
&  0.35685E+00_JPRB, 0.34662E+00_JPRB, 0.33678E+00_JPRB, 0.32449E+00_JPRB, 0.31183E+00_JPRB, 0.29766E+00_JPRB, 0.28695E+00_JPRB ,&
&  0.28052E+00_JPRB, 0.27868E+00_JPRB, 0.27830E+00_JPRB, 0.28259E+00_JPRB, 0.28852E+00_JPRB, 0.29637E+00_JPRB, 0.30538E+00_JPRB ,&
&  0.32575E+00_JPRB, 0.33188E+00_JPRB, 0.33209E+00_JPRB, 0.33212E+00_JPRB, 0.33327E+00_JPRB, 0.33550E+00_JPRB, 0.32592E+00_JPRB ,&
&  0.33507E+00_JPRB, 0.35686E+00_JPRB, 0.36281E+00_JPRB, 0.34415E+00_JPRB, 0.28963E+00_JPRB, 0.25682E+00_JPRB, 0.23426E+00_JPRB ,&
&  0.22566E+00_JPRB, 0.21600E+00_JPRB, 0.20998E+00_JPRB, 0.20431E+00_JPRB, 0.20068E+00_JPRB, 0.19883E+00_JPRB, 0.19701E+00_JPRB ,&
&  0.19568E+00_JPRB, 0.19465E+00_JPRB, 0.19268E+00_JPRB, 0.19177E+00_JPRB, 0.19072E+00_JPRB, 0.19014E+00_JPRB, 0.18982E+00_JPRB ,&
&  0.18964E+00_JPRB, 0.18930E+00_JPRB, 0.18947E+00_JPRB, 0.18996E+00_JPRB, 0.19085E+00_JPRB, 0.19156E+00_JPRB, 0.19222E+00_JPRB ,&
&  0.19303E+00_JPRB, 0.19377E+00_JPRB, 0.19392E+00_JPRB, 0.19436E+00_JPRB, 0.19474E+00_JPRB, 0.19584E+00_JPRB, 0.19632E+00_JPRB ,&
&  0.19685E+00_JPRB, 0.19727E+00_JPRB, 0.19726E+00_JPRB, 0.19782E+00_JPRB, 0.19872E+00_JPRB, 0.20052E+00_JPRB, 0.20261E+00_JPRB ,&
&  0.20530E+00_JPRB, 0.20842E+00_JPRB, 0.21188E+00_JPRB, 0.21533E+00_JPRB, 0.21984E+00_JPRB, 0.22490E+00_JPRB, 0.23139E+00_JPRB ,&
&  0.23847E+00_JPRB, 0.24727E+00_JPRB, 0.25906E+00_JPRB, 0.27350E+00_JPRB, 0.29109E+00_JPRB, 0.30932E+00_JPRB, 0.33028E+00_JPRB ,&
&  0.36555E+00_JPRB, 0.46565E+00_JPRB, 0.52575E+00_JPRB, 0.51291E+00_JPRB, 0.52321E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.60760E+00_JPRB, 0.62411E+00_JPRB, 0.64940E+00_JPRB, 0.64223E+00_JPRB, 0.66257E+00_JPRB, 0.64913E+00_JPRB, 0.62646E+00_JPRB ,&
&  0.57917E+00_JPRB, 0.52225E+00_JPRB, 0.47578E+00_JPRB, 0.42369E+00_JPRB, 0.38325E+00_JPRB, 0.35367E+00_JPRB, 0.33922E+00_JPRB ,&
&  0.33220E+00_JPRB, 0.33002E+00_JPRB, 0.32881E+00_JPRB, 0.32799E+00_JPRB, 0.32341E+00_JPRB, 0.32014E+00_JPRB, 0.31446E+00_JPRB ,&
&  0.30853E+00_JPRB, 0.30405E+00_JPRB, 0.30075E+00_JPRB, 0.30573E+00_JPRB, 0.31194E+00_JPRB, 0.31847E+00_JPRB, 0.32202E+00_JPRB ,&
&  0.32516E+00_JPRB, 0.32647E+00_JPRB, 0.32912E+00_JPRB, 0.32722E+00_JPRB, 0.33825E+00_JPRB, 0.35954E+00_JPRB, 0.37787E+00_JPRB ,&
&  0.39563E+00_JPRB, 0.41042E+00_JPRB, 0.40640E+00_JPRB, 0.38272E+00_JPRB, 0.31705E+00_JPRB, 0.28283E+00_JPRB, 0.25298E+00_JPRB ,&
&  0.23425E+00_JPRB, 0.22038E+00_JPRB, 0.21114E+00_JPRB, 0.20584E+00_JPRB, 0.20170E+00_JPRB, 0.19935E+00_JPRB, 0.19684E+00_JPRB ,&
&  0.19442E+00_JPRB, 0.19291E+00_JPRB, 0.19124E+00_JPRB, 0.19042E+00_JPRB, 0.18948E+00_JPRB, 0.18824E+00_JPRB, 0.18729E+00_JPRB ,&
&  0.18742E+00_JPRB, 0.18721E+00_JPRB, 0.18829E+00_JPRB, 0.18924E+00_JPRB, 0.19113E+00_JPRB, 0.19273E+00_JPRB, 0.19492E+00_JPRB ,&
&  0.19671E+00_JPRB, 0.19818E+00_JPRB, 0.19802E+00_JPRB, 0.19752E+00_JPRB, 0.19665E+00_JPRB, 0.19613E+00_JPRB, 0.19525E+00_JPRB ,&
&  0.19414E+00_JPRB, 0.19430E+00_JPRB, 0.19452E+00_JPRB, 0.19486E+00_JPRB, 0.19594E+00_JPRB, 0.19751E+00_JPRB, 0.19992E+00_JPRB ,&
&  0.20309E+00_JPRB, 0.20642E+00_JPRB, 0.20921E+00_JPRB, 0.21216E+00_JPRB, 0.21596E+00_JPRB, 0.22030E+00_JPRB, 0.22572E+00_JPRB ,&
&  0.23062E+00_JPRB, 0.23883E+00_JPRB, 0.25009E+00_JPRB, 0.26171E+00_JPRB, 0.27896E+00_JPRB, 0.29623E+00_JPRB, 0.32515E+00_JPRB ,&
&  0.37439E+00_JPRB, 0.54731E+00_JPRB, 0.83122E+00_JPRB, 0.60521E+00_JPRB, 0.59340E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.64446E+00_JPRB, 0.65575E+00_JPRB, 0.67001E+00_JPRB, 0.67311E+00_JPRB, 0.68329E+00_JPRB, 0.68219E+00_JPRB, 0.66510E+00_JPRB ,&
&  0.63693E+00_JPRB, 0.61051E+00_JPRB, 0.59602E+00_JPRB, 0.57864E+00_JPRB, 0.55104E+00_JPRB, 0.49064E+00_JPRB, 0.46750E+00_JPRB ,&
&  0.44906E+00_JPRB, 0.44873E+00_JPRB, 0.43961E+00_JPRB, 0.43981E+00_JPRB, 0.42881E+00_JPRB, 0.41746E+00_JPRB, 0.39892E+00_JPRB ,&
&  0.38175E+00_JPRB, 0.36454E+00_JPRB, 0.35268E+00_JPRB, 0.34684E+00_JPRB, 0.34468E+00_JPRB, 0.34023E+00_JPRB, 0.32824E+00_JPRB ,&
&  0.31961E+00_JPRB, 0.31783E+00_JPRB, 0.32034E+00_JPRB, 0.33011E+00_JPRB, 0.37378E+00_JPRB, 0.42989E+00_JPRB, 0.43032E+00_JPRB ,&
&  0.45308E+00_JPRB, 0.46658E+00_JPRB, 0.47974E+00_JPRB, 0.46889E+00_JPRB, 0.43637E+00_JPRB, 0.39787E+00_JPRB, 0.34385E+00_JPRB ,&
&  0.29242E+00_JPRB, 0.26365E+00_JPRB, 0.24033E+00_JPRB, 0.22413E+00_JPRB, 0.21321E+00_JPRB, 0.20870E+00_JPRB, 0.20295E+00_JPRB ,&
&  0.19916E+00_JPRB, 0.19628E+00_JPRB, 0.19509E+00_JPRB, 0.19442E+00_JPRB, 0.19367E+00_JPRB, 0.19162E+00_JPRB, 0.19102E+00_JPRB ,&
&  0.19152E+00_JPRB, 0.19169E+00_JPRB, 0.19152E+00_JPRB, 0.19230E+00_JPRB, 0.19326E+00_JPRB, 0.19464E+00_JPRB, 0.19589E+00_JPRB ,&
&  0.19774E+00_JPRB, 0.19876E+00_JPRB, 0.20043E+00_JPRB, 0.20225E+00_JPRB, 0.20133E+00_JPRB, 0.20015E+00_JPRB, 0.19739E+00_JPRB ,&
&  0.19433E+00_JPRB, 0.19397E+00_JPRB, 0.19345E+00_JPRB, 0.19404E+00_JPRB, 0.19563E+00_JPRB, 0.19779E+00_JPRB, 0.20064E+00_JPRB ,&
&  0.20338E+00_JPRB, 0.20622E+00_JPRB, 0.20843E+00_JPRB, 0.21063E+00_JPRB, 0.21425E+00_JPRB, 0.21813E+00_JPRB, 0.22303E+00_JPRB ,&
&  0.22812E+00_JPRB, 0.23386E+00_JPRB, 0.24179E+00_JPRB, 0.25202E+00_JPRB, 0.26654E+00_JPRB, 0.28307E+00_JPRB, 0.30867E+00_JPRB ,&
&  0.33282E+00_JPRB, 0.43333E+00_JPRB, 0.66986E+00_JPRB, 0.59770E+00_JPRB, 0.61658E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.72084E+00_JPRB, 0.73265E+00_JPRB, 0.74271E+00_JPRB, 0.74471E+00_JPRB, 0.74406E+00_JPRB, 0.73916E+00_JPRB, 0.72718E+00_JPRB ,&
&  0.70755E+00_JPRB, 0.68131E+00_JPRB, 0.65581E+00_JPRB, 0.63550E+00_JPRB, 0.62277E+00_JPRB, 0.62595E+00_JPRB, 0.63470E+00_JPRB ,&
&  0.65232E+00_JPRB, 0.67798E+00_JPRB, 0.68338E+00_JPRB, 0.67694E+00_JPRB, 0.64129E+00_JPRB, 0.60404E+00_JPRB, 0.56934E+00_JPRB ,&
&  0.53860E+00_JPRB, 0.50973E+00_JPRB, 0.49879E+00_JPRB, 0.47994E+00_JPRB, 0.46666E+00_JPRB, 0.45005E+00_JPRB, 0.42662E+00_JPRB ,&
&  0.38825E+00_JPRB, 0.35765E+00_JPRB, 0.34022E+00_JPRB, 0.35351E+00_JPRB, 0.44229E+00_JPRB, 0.50297E+00_JPRB, 0.51243E+00_JPRB ,&
&  0.54696E+00_JPRB, 0.57163E+00_JPRB, 0.56679E+00_JPRB, 0.54697E+00_JPRB, 0.52552E+00_JPRB, 0.50218E+00_JPRB, 0.48298E+00_JPRB ,&
&  0.46386E+00_JPRB, 0.43753E+00_JPRB, 0.38815E+00_JPRB, 0.31816E+00_JPRB, 0.28261E+00_JPRB, 0.26916E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.24383E+00_JPRB, 0.23498E+00_JPRB, 0.23151E+00_JPRB, 0.22819E+00_JPRB, 0.22746E+00_JPRB, 0.22346E+00_JPRB, 0.22000E+00_JPRB ,&
&  0.21566E+00_JPRB, 0.21249E+00_JPRB, 0.20781E+00_JPRB, 0.20318E+00_JPRB, 0.19977E+00_JPRB, 0.19917E+00_JPRB, 0.19888E+00_JPRB ,&
&  0.19987E+00_JPRB, 0.20388E+00_JPRB, 0.21526E+00_JPRB, 0.22158E+00_JPRB, 0.22199E+00_JPRB, 0.21734E+00_JPRB, 0.21019E+00_JPRB ,&
&  0.20367E+00_JPRB, 0.20176E+00_JPRB, 0.20036E+00_JPRB, 0.20092E+00_JPRB, 0.20366E+00_JPRB, 0.20730E+00_JPRB, 0.21033E+00_JPRB ,&
&  0.21274E+00_JPRB, 0.21516E+00_JPRB, 0.21746E+00_JPRB, 0.21994E+00_JPRB, 0.22294E+00_JPRB, 0.22646E+00_JPRB, 0.23124E+00_JPRB ,&
&  0.23633E+00_JPRB, 0.24343E+00_JPRB, 0.25360E+00_JPRB, 0.26823E+00_JPRB, 0.28119E+00_JPRB, 0.28942E+00_JPRB, 0.30057E+00_JPRB ,&
&  0.31658E+00_JPRB, 0.36362E+00_JPRB, 0.49783E+00_JPRB, 0.72414E+00_JPRB, 0.75634E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.78296E+00_JPRB, 0.80127E+00_JPRB, 0.82256E+00_JPRB, 0.83386E+00_JPRB, 0.83499E+00_JPRB, 0.83214E+00_JPRB, 0.82355E+00_JPRB ,&
&  0.80870E+00_JPRB, 0.79075E+00_JPRB, 0.77751E+00_JPRB, 0.76857E+00_JPRB, 0.76462E+00_JPRB, 0.76757E+00_JPRB, 0.77305E+00_JPRB ,&
&  0.77243E+00_JPRB, 0.76844E+00_JPRB, 0.75648E+00_JPRB, 0.74092E+00_JPRB, 0.72445E+00_JPRB, 0.71158E+00_JPRB, 0.70153E+00_JPRB ,&
&  0.69161E+00_JPRB, 0.67672E+00_JPRB, 0.65365E+00_JPRB, 0.61825E+00_JPRB, 0.57650E+00_JPRB, 0.53833E+00_JPRB, 0.50912E+00_JPRB ,&
&  0.48412E+00_JPRB, 0.41700E+00_JPRB, 0.37978E+00_JPRB, 0.39173E+00_JPRB, 0.48423E+00_JPRB, 0.51492E+00_JPRB, 0.53896E+00_JPRB ,&
&  0.57596E+00_JPRB, 0.61533E+00_JPRB, 0.63411E+00_JPRB, 0.63458E+00_JPRB, 0.62905E+00_JPRB, 0.61647E+00_JPRB, 0.59686E+00_JPRB ,&
&  0.57195E+00_JPRB, 0.54148E+00_JPRB, 0.51556E+00_JPRB, 0.48665E+00_JPRB, 0.45686E+00_JPRB, 0.43487E+00_JPRB, 0.41955E+00_JPRB ,&
&  0.40897E+00_JPRB, 0.39939E+00_JPRB, 0.39181E+00_JPRB, 0.38206E+00_JPRB, 0.37288E+00_JPRB, 0.35178E+00_JPRB, 0.32984E+00_JPRB ,&
&  0.30645E+00_JPRB, 0.28894E+00_JPRB, 0.27029E+00_JPRB, 0.25209E+00_JPRB, 0.23715E+00_JPRB, 0.22742E+00_JPRB, 0.22029E+00_JPRB ,&
&  0.21805E+00_JPRB, 0.22391E+00_JPRB, 0.24257E+00_JPRB, 0.27553E+00_JPRB, 0.31362E+00_JPRB, 0.28418E+00_JPRB, 0.26360E+00_JPRB ,&
&  0.24298E+00_JPRB, 0.23085E+00_JPRB, 0.22635E+00_JPRB, 0.22583E+00_JPRB, 0.22712E+00_JPRB, 0.22945E+00_JPRB, 0.23184E+00_JPRB ,&
&  0.23508E+00_JPRB, 0.23813E+00_JPRB, 0.24134E+00_JPRB, 0.24371E+00_JPRB, 0.24529E+00_JPRB, 0.24807E+00_JPRB, 0.25280E+00_JPRB ,&
&  0.26007E+00_JPRB, 0.27208E+00_JPRB, 0.29188E+00_JPRB, 0.31759E+00_JPRB, 0.34420E+00_JPRB, 0.36846E+00_JPRB, 0.42159E+00_JPRB ,&
&  0.50910E+00_JPRB, 0.62137E+00_JPRB, 0.67817E+00_JPRB, 0.73389E+00_JPRB, 0.76147E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.84431E+00_JPRB, 0.87090E+00_JPRB, 0.90025E+00_JPRB, 0.91709E+00_JPRB, 0.91794E+00_JPRB, 0.91473E+00_JPRB, 0.89970E+00_JPRB ,&
&  0.86870E+00_JPRB, 0.84313E+00_JPRB, 0.83873E+00_JPRB, 0.84250E+00_JPRB, 0.85420E+00_JPRB, 0.87285E+00_JPRB, 0.87453E+00_JPRB ,&
&  0.84820E+00_JPRB, 0.82899E+00_JPRB, 0.81949E+00_JPRB, 0.81271E+00_JPRB, 0.79605E+00_JPRB, 0.77161E+00_JPRB, 0.75564E+00_JPRB ,&
&  0.75105E+00_JPRB, 0.75091E+00_JPRB, 0.74184E+00_JPRB, 0.72778E+00_JPRB, 0.69619E+00_JPRB, 0.65731E+00_JPRB, 0.62691E+00_JPRB ,&
&  0.61660E+00_JPRB, 0.60785E+00_JPRB, 0.56698E+00_JPRB, 0.58372E+00_JPRB, 0.58738E+00_JPRB, 0.58755E+00_JPRB, 0.58506E+00_JPRB ,&
&  0.59747E+00_JPRB, 0.63083E+00_JPRB, 0.66377E+00_JPRB, 0.67351E+00_JPRB, 0.68294E+00_JPRB, 0.68469E+00_JPRB, 0.68624E+00_JPRB ,&
&  0.69085E+00_JPRB, 0.68289E+00_JPRB, 0.65329E+00_JPRB, 0.60941E+00_JPRB, 0.56612E+00_JPRB, 0.53245E+00_JPRB, 0.50377E+00_JPRB ,&
&  0.48401E+00_JPRB, 0.47167E+00_JPRB, 0.46203E+00_JPRB, 0.45866E+00_JPRB, 0.46622E+00_JPRB, 0.48113E+00_JPRB, 0.47599E+00_JPRB ,&
&  0.45780E+00_JPRB, 0.44116E+00_JPRB, 0.41910E+00_JPRB, 0.38921E+00_JPRB, 0.35876E+00_JPRB, 0.32006E+00_JPRB, 0.30029E+00_JPRB ,&
&  0.30477E+00_JPRB, 0.34386E+00_JPRB, 0.39533E+00_JPRB, 0.41455E+00_JPRB, 0.44174E+00_JPRB, 0.46810E+00_JPRB, 0.45752E+00_JPRB ,&
&  0.41051E+00_JPRB, 0.34146E+00_JPRB, 0.31865E+00_JPRB, 0.30632E+00_JPRB, 0.29952E+00_JPRB, 0.29472E+00_JPRB, 0.29398E+00_JPRB ,&
&  0.29828E+00_JPRB, 0.30968E+00_JPRB, 0.31915E+00_JPRB, 0.31608E+00_JPRB, 0.30587E+00_JPRB, 0.29961E+00_JPRB, 0.30033E+00_JPRB ,&
&  0.30957E+00_JPRB, 0.33775E+00_JPRB, 0.40134E+00_JPRB, 0.51732E+00_JPRB, 0.55314E+00_JPRB, 0.58616E+00_JPRB, 0.62434E+00_JPRB ,&
&  0.66096E+00_JPRB, 0.69433E+00_JPRB, 0.72288E+00_JPRB, 0.75806E+00_JPRB, 0.80510E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.84056E+00_JPRB, 0.86222E+00_JPRB, 0.88101E+00_JPRB, 0.89381E+00_JPRB, 0.89563E+00_JPRB, 0.88889E+00_JPRB, 0.87297E+00_JPRB ,&
&  0.84297E+00_JPRB, 0.82589E+00_JPRB, 0.83483E+00_JPRB, 0.84946E+00_JPRB, 0.85841E+00_JPRB, 0.85838E+00_JPRB, 0.82686E+00_JPRB ,&
&  0.77261E+00_JPRB, 0.75200E+00_JPRB, 0.74893E+00_JPRB, 0.75853E+00_JPRB, 0.77682E+00_JPRB, 0.78393E+00_JPRB, 0.77200E+00_JPRB ,&
&  0.75976E+00_JPRB, 0.76141E+00_JPRB, 0.76415E+00_JPRB, 0.77091E+00_JPRB, 0.76004E+00_JPRB, 0.73898E+00_JPRB, 0.72999E+00_JPRB ,&
&  0.72741E+00_JPRB, 0.72000E+00_JPRB, 0.71226E+00_JPRB, 0.66333E+00_JPRB, 0.68764E+00_JPRB, 0.67593E+00_JPRB, 0.65760E+00_JPRB ,&
&  0.64089E+00_JPRB, 0.64350E+00_JPRB, 0.68508E+00_JPRB, 0.73143E+00_JPRB, 0.74285E+00_JPRB, 0.74557E+00_JPRB, 0.74070E+00_JPRB ,&
&  0.73727E+00_JPRB, 0.73765E+00_JPRB, 0.73903E+00_JPRB, 0.73089E+00_JPRB, 0.71051E+00_JPRB, 0.68425E+00_JPRB, 0.65545E+00_JPRB ,&
&  0.63800E+00_JPRB, 0.61304E+00_JPRB, 0.56953E+00_JPRB, 0.54928E+00_JPRB, 0.54732E+00_JPRB, 0.55520E+00_JPRB, 0.55898E+00_JPRB ,&
&  0.55587E+00_JPRB, 0.54844E+00_JPRB, 0.55245E+00_JPRB, 0.54961E+00_JPRB, 0.52789E+00_JPRB, 0.51476E+00_JPRB, 0.50874E+00_JPRB ,&
&  0.49718E+00_JPRB, 0.49587E+00_JPRB, 0.50046E+00_JPRB, 0.51191E+00_JPRB, 0.52196E+00_JPRB, 0.53548E+00_JPRB, 0.54098E+00_JPRB ,&
&  0.54312E+00_JPRB, 0.53974E+00_JPRB, 0.53406E+00_JPRB, 0.52227E+00_JPRB, 0.50406E+00_JPRB, 0.49717E+00_JPRB, 0.49874E+00_JPRB ,&
&  0.50761E+00_JPRB, 0.51352E+00_JPRB, 0.51292E+00_JPRB, 0.51183E+00_JPRB, 0.50039E+00_JPRB, 0.48783E+00_JPRB, 0.47817E+00_JPRB ,&
&  0.47357E+00_JPRB, 0.47622E+00_JPRB, 0.49125E+00_JPRB, 0.51600E+00_JPRB, 0.54784E+00_JPRB, 0.58635E+00_JPRB, 0.63110E+00_JPRB ,&
&  0.66298E+00_JPRB, 0.68190E+00_JPRB, 0.70645E+00_JPRB, 0.75407E+00_JPRB, 0.79993E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.80405E+00_JPRB, 0.81245E+00_JPRB, 0.82238E+00_JPRB, 0.83568E+00_JPRB, 0.84774E+00_JPRB, 0.84474E+00_JPRB, 0.83232E+00_JPRB ,&
&  0.80864E+00_JPRB, 0.79979E+00_JPRB, 0.80443E+00_JPRB, 0.80680E+00_JPRB, 0.80850E+00_JPRB, 0.80618E+00_JPRB, 0.79008E+00_JPRB ,&
&  0.75296E+00_JPRB, 0.72315E+00_JPRB, 0.70879E+00_JPRB, 0.70129E+00_JPRB, 0.69880E+00_JPRB, 0.70576E+00_JPRB, 0.71428E+00_JPRB ,&
&  0.73138E+00_JPRB, 0.74175E+00_JPRB, 0.75398E+00_JPRB, 0.76339E+00_JPRB, 0.76597E+00_JPRB, 0.76975E+00_JPRB, 0.77081E+00_JPRB ,&
&  0.75679E+00_JPRB, 0.75709E+00_JPRB, 0.75874E+00_JPRB, 0.74924E+00_JPRB, 0.72808E+00_JPRB, 0.70678E+00_JPRB, 0.68047E+00_JPRB ,&
&  0.66308E+00_JPRB, 0.65443E+00_JPRB, 0.67998E+00_JPRB, 0.74165E+00_JPRB, 0.78492E+00_JPRB, 0.79297E+00_JPRB, 0.78510E+00_JPRB ,&
&  0.78710E+00_JPRB, 0.79452E+00_JPRB, 0.79803E+00_JPRB, 0.78654E+00_JPRB, 0.76685E+00_JPRB, 0.74959E+00_JPRB, 0.73303E+00_JPRB ,&
&  0.72735E+00_JPRB, 0.72640E+00_JPRB, 0.70883E+00_JPRB, 0.67649E+00_JPRB, 0.64879E+00_JPRB, 0.62919E+00_JPRB, 0.61890E+00_JPRB ,&
&  0.61274E+00_JPRB, 0.60330E+00_JPRB, 0.59753E+00_JPRB, 0.60254E+00_JPRB, 0.58277E+00_JPRB, 0.56483E+00_JPRB, 0.56855E+00_JPRB ,&
&  0.58483E+00_JPRB, 0.59007E+00_JPRB, 0.59900E+00_JPRB, 0.60175E+00_JPRB, 0.59822E+00_JPRB, 0.59402E+00_JPRB, 0.59497E+00_JPRB ,&
&  0.60339E+00_JPRB, 0.61304E+00_JPRB, 0.62015E+00_JPRB, 0.62513E+00_JPRB, 0.63026E+00_JPRB, 0.61874E+00_JPRB, 0.59780E+00_JPRB ,&
&  0.58024E+00_JPRB, 0.57689E+00_JPRB, 0.58136E+00_JPRB, 0.58069E+00_JPRB, 0.57912E+00_JPRB, 0.57925E+00_JPRB, 0.57784E+00_JPRB ,&
&  0.57496E+00_JPRB, 0.56744E+00_JPRB, 0.55852E+00_JPRB, 0.55877E+00_JPRB, 0.57021E+00_JPRB, 0.59283E+00_JPRB, 0.62787E+00_JPRB ,&
&  0.68018E+00_JPRB, 0.71976E+00_JPRB, 0.75767E+00_JPRB, 0.77302E+00_JPRB, 0.77647E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.77530E+00_JPRB, 0.78366E+00_JPRB, 0.78980E+00_JPRB, 0.79430E+00_JPRB, 0.80317E+00_JPRB, 0.81417E+00_JPRB, 0.81405E+00_JPRB ,&
&  0.79934E+00_JPRB, 0.78559E+00_JPRB, 0.77596E+00_JPRB, 0.77314E+00_JPRB, 0.77472E+00_JPRB, 0.77681E+00_JPRB, 0.77095E+00_JPRB ,&
&  0.75017E+00_JPRB, 0.72788E+00_JPRB, 0.71414E+00_JPRB, 0.70047E+00_JPRB, 0.69197E+00_JPRB, 0.68656E+00_JPRB, 0.69344E+00_JPRB ,&
&  0.70366E+00_JPRB, 0.71981E+00_JPRB, 0.73304E+00_JPRB, 0.74262E+00_JPRB, 0.74652E+00_JPRB, 0.73520E+00_JPRB, 0.73659E+00_JPRB ,&
&  0.74379E+00_JPRB, 0.74606E+00_JPRB, 0.76123E+00_JPRB, 0.76081E+00_JPRB, 0.74775E+00_JPRB, 0.73736E+00_JPRB, 0.72696E+00_JPRB ,&
&  0.72424E+00_JPRB, 0.73560E+00_JPRB, 0.76102E+00_JPRB, 0.79759E+00_JPRB, 0.81863E+00_JPRB, 0.82655E+00_JPRB, 0.82349E+00_JPRB ,&
&  0.82631E+00_JPRB, 0.83829E+00_JPRB, 0.84502E+00_JPRB, 0.83677E+00_JPRB, 0.81849E+00_JPRB, 0.80357E+00_JPRB, 0.79034E+00_JPRB ,&
&  0.78052E+00_JPRB, 0.76861E+00_JPRB, 0.76048E+00_JPRB, 0.75038E+00_JPRB, 0.72846E+00_JPRB, 0.70248E+00_JPRB, 0.67989E+00_JPRB ,&
&  0.66317E+00_JPRB, 0.64684E+00_JPRB, 0.62688E+00_JPRB, 0.62074E+00_JPRB, 0.62492E+00_JPRB, 0.60192E+00_JPRB, 0.57691E+00_JPRB ,&
&  0.58025E+00_JPRB, 0.61119E+00_JPRB, 0.63534E+00_JPRB, 0.64483E+00_JPRB, 0.64525E+00_JPRB, 0.64916E+00_JPRB, 0.65224E+00_JPRB ,&
&  0.65845E+00_JPRB, 0.66688E+00_JPRB, 0.67340E+00_JPRB, 0.68448E+00_JPRB, 0.69604E+00_JPRB, 0.69464E+00_JPRB, 0.68075E+00_JPRB ,&
&  0.66541E+00_JPRB, 0.65658E+00_JPRB, 0.65271E+00_JPRB, 0.65476E+00_JPRB, 0.65802E+00_JPRB, 0.65586E+00_JPRB, 0.65004E+00_JPRB ,&
&  0.63845E+00_JPRB, 0.62004E+00_JPRB, 0.60440E+00_JPRB, 0.59582E+00_JPRB, 0.59558E+00_JPRB, 0.60247E+00_JPRB, 0.61887E+00_JPRB ,&
&  0.65052E+00_JPRB, 0.69845E+00_JPRB, 0.73362E+00_JPRB, 0.75310E+00_JPRB, 0.75805E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.74984E+00_JPRB, 0.75646E+00_JPRB, 0.76619E+00_JPRB, 0.78016E+00_JPRB, 0.79443E+00_JPRB, 0.80352E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.79175E+00_JPRB, 0.77826E+00_JPRB, 0.77366E+00_JPRB, 0.77281E+00_JPRB, 0.76966E+00_JPRB, 0.76819E+00_JPRB, 0.76365E+00_JPRB ,&
&  0.75270E+00_JPRB, 0.73600E+00_JPRB, 0.72813E+00_JPRB, 0.71440E+00_JPRB, 0.70131E+00_JPRB, 0.69064E+00_JPRB, 0.69075E+00_JPRB ,&
&  0.69952E+00_JPRB, 0.70964E+00_JPRB, 0.72213E+00_JPRB, 0.73644E+00_JPRB, 0.74431E+00_JPRB, 0.74311E+00_JPRB, 0.74006E+00_JPRB ,&
&  0.73762E+00_JPRB, 0.73494E+00_JPRB, 0.74601E+00_JPRB, 0.75509E+00_JPRB, 0.76117E+00_JPRB, 0.77238E+00_JPRB, 0.78161E+00_JPRB ,&
&  0.78926E+00_JPRB, 0.80049E+00_JPRB, 0.81921E+00_JPRB, 0.83310E+00_JPRB, 0.84311E+00_JPRB, 0.85373E+00_JPRB, 0.85738E+00_JPRB ,&
&  0.86357E+00_JPRB, 0.87474E+00_JPRB, 0.87708E+00_JPRB, 0.86811E+00_JPRB, 0.85380E+00_JPRB, 0.84458E+00_JPRB, 0.83936E+00_JPRB ,&
&  0.83001E+00_JPRB, 0.81061E+00_JPRB, 0.79593E+00_JPRB, 0.78422E+00_JPRB, 0.76522E+00_JPRB, 0.74487E+00_JPRB, 0.72115E+00_JPRB ,&
&  0.69415E+00_JPRB, 0.67404E+00_JPRB, 0.65839E+00_JPRB, 0.64429E+00_JPRB, 0.64086E+00_JPRB, 0.63222E+00_JPRB, 0.62547E+00_JPRB ,&
&  0.62983E+00_JPRB, 0.64783E+00_JPRB, 0.66122E+00_JPRB, 0.66554E+00_JPRB, 0.66768E+00_JPRB, 0.68310E+00_JPRB, 0.70190E+00_JPRB ,&
&  0.71657E+00_JPRB, 0.72266E+00_JPRB, 0.71858E+00_JPRB, 0.72083E+00_JPRB, 0.72592E+00_JPRB, 0.73032E+00_JPRB, 0.72456E+00_JPRB ,&
&  0.71619E+00_JPRB, 0.71071E+00_JPRB, 0.70956E+00_JPRB, 0.70600E+00_JPRB, 0.70750E+00_JPRB, 0.70736E+00_JPRB, 0.70453E+00_JPRB ,&
&  0.69877E+00_JPRB, 0.68950E+00_JPRB, 0.67765E+00_JPRB, 0.67069E+00_JPRB, 0.66051E+00_JPRB, 0.64517E+00_JPRB, 0.64667E+00_JPRB ,&
&  0.65861E+00_JPRB, 0.67732E+00_JPRB, 0.69465E+00_JPRB, 0.71494E+00_JPRB, 0.73864E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.73273E+00_JPRB, 0.74594E+00_JPRB, 0.76240E+00_JPRB, 0.77901E+00_JPRB, 0.78937E+00_JPRB, 0.79562E+00_JPRB, 0.79316E+00_JPRB ,&
&  0.78189E+00_JPRB, 0.77076E+00_JPRB, 0.77195E+00_JPRB, 0.77144E+00_JPRB, 0.77007E+00_JPRB, 0.76803E+00_JPRB, 0.76127E+00_JPRB ,&
&  0.75174E+00_JPRB, 0.74372E+00_JPRB, 0.74038E+00_JPRB, 0.73736E+00_JPRB, 0.72742E+00_JPRB, 0.71978E+00_JPRB, 0.71160E+00_JPRB ,&
&  0.71150E+00_JPRB, 0.71424E+00_JPRB, 0.71735E+00_JPRB, 0.71792E+00_JPRB, 0.72623E+00_JPRB, 0.73677E+00_JPRB, 0.74138E+00_JPRB ,&
&  0.73605E+00_JPRB, 0.74272E+00_JPRB, 0.66056E+00_JPRB, 0.73030E+00_JPRB, 0.75548E+00_JPRB, 0.77042E+00_JPRB, 0.78569E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.80750E+00_JPRB, 0.81613E+00_JPRB, 0.82104E+00_JPRB, 0.83835E+00_JPRB, 0.85254E+00_JPRB, 0.85609E+00_JPRB ,&
&  0.85738E+00_JPRB, 0.85634E+00_JPRB, 0.85766E+00_JPRB, 0.85781E+00_JPRB, 0.86220E+00_JPRB, 0.86202E+00_JPRB, 0.85591E+00_JPRB ,&
&  0.84776E+00_JPRB, 0.83597E+00_JPRB, 0.81854E+00_JPRB, 0.79840E+00_JPRB, 0.76660E+00_JPRB, 0.73884E+00_JPRB, 0.72117E+00_JPRB ,&
&  0.70655E+00_JPRB, 0.70213E+00_JPRB, 0.69666E+00_JPRB, 0.69098E+00_JPRB, 0.68868E+00_JPRB, 0.69078E+00_JPRB, 0.68944E+00_JPRB ,&
&  0.69573E+00_JPRB, 0.70425E+00_JPRB, 0.70837E+00_JPRB, 0.71553E+00_JPRB, 0.72489E+00_JPRB, 0.73862E+00_JPRB, 0.75162E+00_JPRB ,&
&  0.76395E+00_JPRB, 0.77238E+00_JPRB, 0.77365E+00_JPRB, 0.77374E+00_JPRB, 0.77163E+00_JPRB, 0.77077E+00_JPRB, 0.76392E+00_JPRB ,&
&  0.75845E+00_JPRB, 0.75560E+00_JPRB, 0.74944E+00_JPRB, 0.73770E+00_JPRB, 0.73061E+00_JPRB, 0.72460E+00_JPRB, 0.72311E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.72574E+00_JPRB, 0.72341E+00_JPRB, 0.72426E+00_JPRB, 0.71822E+00_JPRB, 0.69869E+00_JPRB, 0.68364E+00_JPRB ,&
&  0.68351E+00_JPRB, 0.68922E+00_JPRB, 0.69447E+00_JPRB, 0.70402E+00_JPRB, 0.71974E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.77858E+00_JPRB, 0.78501E+00_JPRB, 0.79462E+00_JPRB, 0.80525E+00_JPRB, 0.80880E+00_JPRB, 0.80806E+00_JPRB, 0.80513E+00_JPRB ,&
&  0.80528E+00_JPRB, 0.79659E+00_JPRB, 0.79005E+00_JPRB, 0.78604E+00_JPRB, 0.78500E+00_JPRB, 0.78138E+00_JPRB, 0.77203E+00_JPRB ,&
&  0.76428E+00_JPRB, 0.76126E+00_JPRB, 0.76579E+00_JPRB, 0.76727E+00_JPRB, 0.75204E+00_JPRB, 0.74709E+00_JPRB, 0.73917E+00_JPRB ,&
&  0.73794E+00_JPRB, 0.73828E+00_JPRB, 0.74147E+00_JPRB, 0.74795E+00_JPRB, 0.76461E+00_JPRB, 0.77650E+00_JPRB, 0.78296E+00_JPRB ,&
&  0.78162E+00_JPRB, 0.70250E+00_JPRB, 0.73021E+00_JPRB, 0.74270E+00_JPRB, 0.76538E+00_JPRB, 0.78826E+00_JPRB, 0.80880E+00_JPRB ,&
&  0.83586E+00_JPRB, 0.85130E+00_JPRB, 0.85831E+00_JPRB, 0.85988E+00_JPRB, 0.86158E+00_JPRB, 0.86130E+00_JPRB, 0.85390E+00_JPRB ,&
&  0.84869E+00_JPRB, 0.85543E+00_JPRB, 0.87166E+00_JPRB, 0.87967E+00_JPRB, 0.88345E+00_JPRB, 0.88535E+00_JPRB, 0.88173E+00_JPRB ,&
&  0.87439E+00_JPRB, 0.85979E+00_JPRB, 0.83571E+00_JPRB, 0.80691E+00_JPRB, 0.77436E+00_JPRB, 0.75186E+00_JPRB, 0.73925E+00_JPRB ,&
&  0.73090E+00_JPRB, 0.73046E+00_JPRB, 0.73067E+00_JPRB, 0.73148E+00_JPRB, 0.73423E+00_JPRB, 0.74213E+00_JPRB, 0.74494E+00_JPRB ,&
&  0.75217E+00_JPRB, 0.75940E+00_JPRB, 0.77137E+00_JPRB, 0.79082E+00_JPRB, 0.81057E+00_JPRB, 0.82180E+00_JPRB, 0.82277E+00_JPRB ,&
&  0.82874E+00_JPRB, 0.83163E+00_JPRB, 0.82809E+00_JPRB, 0.82812E+00_JPRB, 0.82455E+00_JPRB, 0.82603E+00_JPRB, 0.82347E+00_JPRB ,&
&  0.81505E+00_JPRB, 0.80894E+00_JPRB, 0.80145E+00_JPRB, 0.78662E+00_JPRB, 0.77380E+00_JPRB, 0.76458E+00_JPRB, 0.75689E+00_JPRB ,&
&  0.75634E+00_JPRB, 0.76018E+00_JPRB, 0.76552E+00_JPRB, 0.76075E+00_JPRB, 0.74738E+00_JPRB, 0.73979E+00_JPRB, 0.73032E+00_JPRB ,&
&  0.72430E+00_JPRB, 0.72734E+00_JPRB, 0.72887E+00_JPRB, 0.73849E+00_JPRB, 0.75992E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.82042E+00_JPRB, 0.82449E+00_JPRB, 0.82856E+00_JPRB, 0.82929E+00_JPRB, 0.82553E+00_JPRB, 0.82275E+00_JPRB, 0.82395E+00_JPRB ,&
&  0.82444E+00_JPRB, 0.81599E+00_JPRB, 0.80756E+00_JPRB, 0.80135E+00_JPRB, 0.79890E+00_JPRB, 0.79035E+00_JPRB, 0.77723E+00_JPRB ,&
&  0.76769E+00_JPRB, 0.76639E+00_JPRB, 0.77170E+00_JPRB, 0.77480E+00_JPRB, 0.76642E+00_JPRB, 0.75740E+00_JPRB, 0.75434E+00_JPRB ,&
&  0.75542E+00_JPRB, 0.76170E+00_JPRB, 0.77218E+00_JPRB, 0.77859E+00_JPRB, 0.78767E+00_JPRB, 0.80217E+00_JPRB, 0.80377E+00_JPRB ,&
&  0.80239E+00_JPRB, 0.66650E+00_JPRB, 0.70429E+00_JPRB, 0.71931E+00_JPRB, 0.74969E+00_JPRB, 0.79481E+00_JPRB, 0.83266E+00_JPRB ,&
&  0.86915E+00_JPRB, 0.89088E+00_JPRB, 0.90095E+00_JPRB, 0.89495E+00_JPRB, 0.87970E+00_JPRB, 0.86499E+00_JPRB, 0.86044E+00_JPRB ,&
&  0.85877E+00_JPRB, 0.86467E+00_JPRB, 0.87744E+00_JPRB, 0.88627E+00_JPRB, 0.88906E+00_JPRB, 0.88933E+00_JPRB, 0.89124E+00_JPRB ,&
&  0.88777E+00_JPRB, 0.87162E+00_JPRB, 0.84889E+00_JPRB, 0.82137E+00_JPRB, 0.79992E+00_JPRB, 0.79005E+00_JPRB, 0.77741E+00_JPRB ,&
&  0.75988E+00_JPRB, 0.74902E+00_JPRB, 0.74690E+00_JPRB, 0.74911E+00_JPRB, 0.75728E+00_JPRB, 0.76645E+00_JPRB, 0.77450E+00_JPRB ,&
&  0.78420E+00_JPRB, 0.79979E+00_JPRB, 0.81678E+00_JPRB, 0.83316E+00_JPRB, 0.85021E+00_JPRB, 0.86647E+00_JPRB, 0.87929E+00_JPRB ,&
&  0.88792E+00_JPRB, 0.88658E+00_JPRB, 0.87535E+00_JPRB, 0.86651E+00_JPRB, 0.86305E+00_JPRB, 0.86742E+00_JPRB, 0.86827E+00_JPRB ,&
&  0.86052E+00_JPRB, 0.85424E+00_JPRB, 0.85169E+00_JPRB, 0.84181E+00_JPRB, 0.83075E+00_JPRB, 0.82071E+00_JPRB, 0.81011E+00_JPRB ,&
&  0.80189E+00_JPRB, 0.79915E+00_JPRB, 0.79645E+00_JPRB, 0.79111E+00_JPRB, 0.78333E+00_JPRB, 0.78139E+00_JPRB, 0.77981E+00_JPRB ,&
&  0.77821E+00_JPRB, 0.77565E+00_JPRB, 0.77896E+00_JPRB, 0.79142E+00_JPRB, 0.80867E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.84940E+00_JPRB, 0.85623E+00_JPRB, 0.86001E+00_JPRB, 0.86418E+00_JPRB, 0.86379E+00_JPRB, 0.86115E+00_JPRB, 0.86436E+00_JPRB ,&
&  0.86797E+00_JPRB, 0.86220E+00_JPRB, 0.85304E+00_JPRB, 0.84967E+00_JPRB, 0.84005E+00_JPRB, 0.82588E+00_JPRB, 0.81415E+00_JPRB ,&
&  0.79953E+00_JPRB, 0.79339E+00_JPRB, 0.78958E+00_JPRB, 0.78584E+00_JPRB, 0.77926E+00_JPRB, 0.77112E+00_JPRB, 0.77467E+00_JPRB ,&
&  0.78140E+00_JPRB, 0.78867E+00_JPRB, 0.79083E+00_JPRB, 0.79515E+00_JPRB, 0.80153E+00_JPRB, 0.80875E+00_JPRB, 0.82166E+00_JPRB ,&
&  0.71123E+00_JPRB, 0.58719E+00_JPRB, 0.63893E+00_JPRB, 0.69182E+00_JPRB, 0.73962E+00_JPRB, 0.81146E+00_JPRB, 0.86463E+00_JPRB ,&
&  0.90678E+00_JPRB, 0.94242E+00_JPRB, 0.94999E+00_JPRB, 0.92749E+00_JPRB, 0.90362E+00_JPRB, 0.89494E+00_JPRB, 0.89305E+00_JPRB ,&
&  0.88876E+00_JPRB, 0.88716E+00_JPRB, 0.89058E+00_JPRB, 0.89419E+00_JPRB, 0.89789E+00_JPRB, 0.90329E+00_JPRB, 0.90493E+00_JPRB ,&
&  0.90244E+00_JPRB, 0.88975E+00_JPRB, 0.87700E+00_JPRB, 0.85749E+00_JPRB, 0.84058E+00_JPRB, 0.83515E+00_JPRB, 0.82629E+00_JPRB ,&
&  0.81117E+00_JPRB, 0.80162E+00_JPRB, 0.79943E+00_JPRB, 0.80099E+00_JPRB, 0.80857E+00_JPRB, 0.81826E+00_JPRB, 0.83574E+00_JPRB ,&
&  0.85689E+00_JPRB, 0.87299E+00_JPRB, 0.88445E+00_JPRB, 0.89774E+00_JPRB, 0.90551E+00_JPRB, 0.91572E+00_JPRB, 0.93367E+00_JPRB ,&
&  0.94850E+00_JPRB, 0.93993E+00_JPRB, 0.93148E+00_JPRB, 0.92260E+00_JPRB, 0.91959E+00_JPRB, 0.91729E+00_JPRB, 0.90820E+00_JPRB ,&
&  0.89963E+00_JPRB, 0.89452E+00_JPRB, 0.89243E+00_JPRB, 0.89754E+00_JPRB, 0.89693E+00_JPRB, 0.89215E+00_JPRB, 0.89073E+00_JPRB ,&
&  0.89010E+00_JPRB, 0.88263E+00_JPRB, 0.87275E+00_JPRB, 0.86947E+00_JPRB, 0.86449E+00_JPRB, 0.86405E+00_JPRB, 0.86132E+00_JPRB ,&
&  0.85867E+00_JPRB, 0.84489E+00_JPRB, 0.85121E+00_JPRB, 0.84600E+00_JPRB, 0.84333E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.87009E+00_JPRB, 0.89145E+00_JPRB, 0.89377E+00_JPRB, 0.90386E+00_JPRB, 0.90951E+00_JPRB, 0.91334E+00_JPRB, 0.92109E+00_JPRB ,&
&  0.92614E+00_JPRB, 0.92142E+00_JPRB, 0.91324E+00_JPRB, 0.89769E+00_JPRB, 0.87321E+00_JPRB, 0.84684E+00_JPRB, 0.82504E+00_JPRB ,&
&  0.80932E+00_JPRB, 0.80600E+00_JPRB, 0.79735E+00_JPRB, 0.78660E+00_JPRB, 0.78014E+00_JPRB, 0.77677E+00_JPRB, 0.77903E+00_JPRB ,&
&  0.78703E+00_JPRB, 0.79128E+00_JPRB, 0.78649E+00_JPRB, 0.78550E+00_JPRB, 0.79461E+00_JPRB, 0.81108E+00_JPRB, 0.81907E+00_JPRB ,&
&  0.82147E+00_JPRB, 0.62713E+00_JPRB, 0.66902E+00_JPRB, 0.72034E+00_JPRB, 0.76577E+00_JPRB, 0.83005E+00_JPRB, 0.90130E+00_JPRB ,&
&  0.94886E+00_JPRB, 0.97758E+00_JPRB, 0.97854E+00_JPRB, 0.96782E+00_JPRB, 0.94758E+00_JPRB, 0.92902E+00_JPRB, 0.90776E+00_JPRB ,&
&  0.88941E+00_JPRB, 0.88357E+00_JPRB, 0.88742E+00_JPRB, 0.89157E+00_JPRB, 0.89313E+00_JPRB, 0.89665E+00_JPRB, 0.89075E+00_JPRB ,&
&  0.88854E+00_JPRB, 0.88499E+00_JPRB, 0.88263E+00_JPRB, 0.88209E+00_JPRB, 0.88035E+00_JPRB, 0.87906E+00_JPRB, 0.87430E+00_JPRB ,&
&  0.85985E+00_JPRB, 0.84819E+00_JPRB, 0.83814E+00_JPRB, 0.82908E+00_JPRB, 0.83066E+00_JPRB, 0.84183E+00_JPRB, 0.86126E+00_JPRB ,&
&  0.88346E+00_JPRB, 0.88903E+00_JPRB, 0.89638E+00_JPRB, 0.90521E+00_JPRB, 0.91843E+00_JPRB, 0.93227E+00_JPRB, 0.94868E+00_JPRB ,&
&  0.95530E+00_JPRB, 0.94992E+00_JPRB, 0.94523E+00_JPRB, 0.93515E+00_JPRB, 0.93406E+00_JPRB, 0.91962E+00_JPRB, 0.90207E+00_JPRB ,&
&  0.89993E+00_JPRB, 0.90073E+00_JPRB, 0.90326E+00_JPRB, 0.91517E+00_JPRB, 0.92347E+00_JPRB, 0.92491E+00_JPRB, 0.93365E+00_JPRB ,&
&  0.94614E+00_JPRB, 0.95514E+00_JPRB, 0.95220E+00_JPRB, 0.96144E+00_JPRB, 0.96098E+00_JPRB, 0.96251E+00_JPRB, 0.94798E+00_JPRB ,&
&  0.92613E+00_JPRB, 0.91022E+00_JPRB, 0.71450E+00_JPRB, 0.81049E+00_JPRB, 0.83858E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.94292E+00_JPRB, 0.97342E+00_JPRB, 0.98266E+00_JPRB, 0.97738E+00_JPRB, 0.97656E+00_JPRB, 0.97441E+00_JPRB, 0.96886E+00_JPRB ,&
&  0.96575E+00_JPRB, 0.95966E+00_JPRB, 0.94552E+00_JPRB, 0.92353E+00_JPRB, 0.90068E+00_JPRB, 0.87583E+00_JPRB, 0.85623E+00_JPRB ,&
&  0.84930E+00_JPRB, 0.84502E+00_JPRB, 0.83102E+00_JPRB, 0.81812E+00_JPRB, 0.80605E+00_JPRB, 0.79651E+00_JPRB, 0.79290E+00_JPRB ,&
&  0.78837E+00_JPRB, 0.78672E+00_JPRB, 0.79237E+00_JPRB, 0.79981E+00_JPRB, 0.80953E+00_JPRB, 0.82033E+00_JPRB, 0.83494E+00_JPRB ,&
&  0.83790E+00_JPRB, 0.65395E+00_JPRB, 0.68230E+00_JPRB, 0.77114E+00_JPRB, 0.83636E+00_JPRB, 0.91813E+00_JPRB, 0.99107E+00_JPRB ,&
&  0.10161E+01_JPRB, 0.10393E+01_JPRB, 0.10303E+01_JPRB, 0.10136E+01_JPRB, 0.10092E+01_JPRB, 0.99007E+00_JPRB, 0.95047E+00_JPRB ,&
&  0.92031E+00_JPRB, 0.91690E+00_JPRB, 0.92783E+00_JPRB, 0.93485E+00_JPRB, 0.93215E+00_JPRB, 0.92439E+00_JPRB, 0.91979E+00_JPRB ,&
&  0.91438E+00_JPRB, 0.90792E+00_JPRB, 0.90016E+00_JPRB, 0.88952E+00_JPRB, 0.89360E+00_JPRB, 0.91083E+00_JPRB, 0.92448E+00_JPRB ,&
&  0.92224E+00_JPRB, 0.90539E+00_JPRB, 0.88629E+00_JPRB, 0.87427E+00_JPRB, 0.87506E+00_JPRB, 0.89181E+00_JPRB, 0.91168E+00_JPRB ,&
&  0.92291E+00_JPRB, 0.92854E+00_JPRB, 0.93814E+00_JPRB, 0.94137E+00_JPRB, 0.94917E+00_JPRB, 0.95740E+00_JPRB, 0.97048E+00_JPRB ,&
&  0.97693E+00_JPRB, 0.97072E+00_JPRB, 0.96273E+00_JPRB, 0.95014E+00_JPRB, 0.93253E+00_JPRB, 0.92176E+00_JPRB, 0.91873E+00_JPRB ,&
&  0.91556E+00_JPRB, 0.92244E+00_JPRB, 0.94209E+00_JPRB, 0.96567E+00_JPRB, 0.97507E+00_JPRB, 0.97901E+00_JPRB, 0.98381E+00_JPRB ,&
&  0.99358E+00_JPRB, 0.10006E+01_JPRB, 0.10053E+01_JPRB, 0.10081E+01_JPRB, 0.10600E+01_JPRB, 0.10647E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10250E+01_JPRB, 0.99609E+00_JPRB, 0.99322E+00_JPRB, 0.83441E+00_JPRB, 0.91346E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.10344E+01_JPRB, 0.10595E+01_JPRB, 0.10468E+01_JPRB, 0.10350E+01_JPRB, 0.10242E+01_JPRB, 0.10048E+01_JPRB, 0.98003E+00_JPRB ,&
&  0.95491E+00_JPRB, 0.93994E+00_JPRB, 0.92067E+00_JPRB, 0.89593E+00_JPRB, 0.87917E+00_JPRB, 0.86868E+00_JPRB, 0.85885E+00_JPRB ,&
&  0.85426E+00_JPRB, 0.84940E+00_JPRB, 0.84557E+00_JPRB, 0.84110E+00_JPRB, 0.83137E+00_JPRB, 0.82271E+00_JPRB, 0.80941E+00_JPRB ,&
&  0.80074E+00_JPRB, 0.79991E+00_JPRB, 0.80047E+00_JPRB, 0.80444E+00_JPRB, 0.81183E+00_JPRB, 0.82025E+00_JPRB, 0.82609E+00_JPRB ,&
&  0.82549E+00_JPRB, 0.71996E+00_JPRB, 0.75854E+00_JPRB, 0.85695E+00_JPRB, 0.97061E+00_JPRB, 0.10684E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10828E+01_JPRB, 0.10661E+01_JPRB, 0.10459E+01_JPRB, 0.10249E+01_JPRB, 0.10115E+01_JPRB, 0.98282E+00_JPRB ,&
&  0.96567E+00_JPRB, 0.96019E+00_JPRB, 0.96438E+00_JPRB, 0.96407E+00_JPRB, 0.95821E+00_JPRB, 0.94976E+00_JPRB, 0.94635E+00_JPRB ,&
&  0.93792E+00_JPRB, 0.91311E+00_JPRB, 0.88509E+00_JPRB, 0.86405E+00_JPRB, 0.86027E+00_JPRB, 0.89433E+00_JPRB, 0.93485E+00_JPRB ,&
&  0.96506E+00_JPRB, 0.94195E+00_JPRB, 0.91422E+00_JPRB, 0.90060E+00_JPRB, 0.90132E+00_JPRB, 0.91836E+00_JPRB, 0.94448E+00_JPRB ,&
&  0.95393E+00_JPRB, 0.95647E+00_JPRB, 0.96997E+00_JPRB, 0.97357E+00_JPRB, 0.97682E+00_JPRB, 0.97512E+00_JPRB, 0.97241E+00_JPRB ,&
&  0.96386E+00_JPRB, 0.95377E+00_JPRB, 0.94851E+00_JPRB, 0.94383E+00_JPRB, 0.94486E+00_JPRB, 0.93914E+00_JPRB, 0.92824E+00_JPRB ,&
&  0.91640E+00_JPRB, 0.92931E+00_JPRB, 0.96628E+00_JPRB, 0.99960E+00_JPRB, 0.10003E+01_JPRB, 0.99216E+00_JPRB, 0.98777E+00_JPRB ,&
&  0.99467E+00_JPRB, 0.99244E+00_JPRB, 0.98888E+00_JPRB, 0.10747E+01_JPRB, 0.10888E+01_JPRB, 0.10962E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.11159E+01_JPRB, 0.11062E+01_JPRB, 0.10900E+01_JPRB, 0.10619E+01_JPRB, 0.97741E+00_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.11285E+01_JPRB, 0.11141E+01_JPRB, 0.10966E+01_JPRB, 0.10852E+01_JPRB, 0.10668E+01_JPRB, 0.10323E+01_JPRB, 0.99468E+00_JPRB ,&
&  0.96699E+00_JPRB, 0.93610E+00_JPRB, 0.89565E+00_JPRB, 0.87471E+00_JPRB, 0.86828E+00_JPRB, 0.86722E+00_JPRB, 0.86192E+00_JPRB ,&
&  0.85605E+00_JPRB, 0.85916E+00_JPRB, 0.85300E+00_JPRB, 0.85316E+00_JPRB, 0.84562E+00_JPRB, 0.84160E+00_JPRB, 0.82228E+00_JPRB ,&
&  0.80970E+00_JPRB, 0.80072E+00_JPRB, 0.79815E+00_JPRB, 0.80464E+00_JPRB, 0.81035E+00_JPRB, 0.81428E+00_JPRB, 0.81928E+00_JPRB ,&
&  0.81186E+00_JPRB, 0.74267E+00_JPRB, 0.85616E+00_JPRB, 0.10268E+01_JPRB, 0.11490E+01_JPRB, 0.12263E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.11305E+01_JPRB, 0.11306E+01_JPRB, 0.11114E+01_JPRB, 0.10766E+01_JPRB, 0.10459E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.10249E+01_JPRB, 0.10185E+01_JPRB, 0.10140E+01_JPRB, 0.10064E+01_JPRB, 0.99609E+00_JPRB, 0.98531E+00_JPRB, 0.98441E+00_JPRB ,&
&  0.96931E+00_JPRB, 0.93226E+00_JPRB, 0.89479E+00_JPRB, 0.86985E+00_JPRB, 0.88496E+00_JPRB, 0.92714E+00_JPRB, 0.99342E+00_JPRB ,&
&  0.10282E+01_JPRB, 0.10029E+01_JPRB, 0.98417E+00_JPRB, 0.97420E+00_JPRB, 0.97291E+00_JPRB, 0.98698E+00_JPRB, 0.99244E+00_JPRB ,&
&  0.99687E+00_JPRB, 0.99235E+00_JPRB, 0.99071E+00_JPRB, 0.98440E+00_JPRB, 0.97830E+00_JPRB, 0.97450E+00_JPRB, 0.96633E+00_JPRB ,&
&  0.95724E+00_JPRB, 0.95611E+00_JPRB, 0.94630E+00_JPRB, 0.94617E+00_JPRB, 0.93954E+00_JPRB, 0.92784E+00_JPRB, 0.91771E+00_JPRB ,&
&  0.92896E+00_JPRB, 0.96073E+00_JPRB, 0.98631E+00_JPRB, 0.10180E+01_JPRB, 0.10256E+01_JPRB, 0.10155E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10123E+01_JPRB, 0.10093E+01_JPRB, 0.98012E+00_JPRB, 0.11298E+01_JPRB, 0.12590E+01_JPRB, 0.12522E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.12280E+01_JPRB, 0.11975E+01_JPRB, 0.11297E+01_JPRB, 0.10745E+01_JPRB, 0.11119E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.10562E+01_JPRB, 0.10368E+01_JPRB, 0.10423E+01_JPRB, 0.10470E+01_JPRB, 0.10320E+01_JPRB, 0.99406E+00_JPRB, 0.96074E+00_JPRB ,&
&  0.94450E+00_JPRB, 0.90910E+00_JPRB, 0.87624E+00_JPRB, 0.85458E+00_JPRB, 0.85847E+00_JPRB, 0.86331E+00_JPRB, 0.86197E+00_JPRB ,&
&  0.85000E+00_JPRB, 0.84606E+00_JPRB, 0.83158E+00_JPRB, 0.82032E+00_JPRB, 0.81519E+00_JPRB, 0.81304E+00_JPRB, 0.80348E+00_JPRB ,&
&  0.80676E+00_JPRB, 0.80711E+00_JPRB, 0.81110E+00_JPRB, 0.81842E+00_JPRB, 0.81541E+00_JPRB, 0.81846E+00_JPRB, 0.81101E+00_JPRB ,&
&  0.79228E+00_JPRB, 0.86865E+00_JPRB, 0.10648E+01_JPRB, 0.13269E+01_JPRB, 0.14323E+01_JPRB, 0.13865E+01_JPRB, 0.12996E+01_JPRB ,&
&  0.12271E+01_JPRB, 0.11965E+01_JPRB, 0.12004E+01_JPRB, 0.11795E+01_JPRB, 0.11476E+01_JPRB, 0.11207E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.10673E+01_JPRB, 0.10465E+01_JPRB, 0.10397E+01_JPRB, 0.10226E+01_JPRB, 0.10005E+01_JPRB, 0.99286E+00_JPRB, 0.99395E+00_JPRB ,&
&  0.94981E+00_JPRB, 0.91579E+00_JPRB, 0.88896E+00_JPRB, 0.87482E+00_JPRB, 0.95007E+00_JPRB, 0.10523E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.11081E+01_JPRB, 0.10905E+01_JPRB, 0.10813E+01_JPRB, 0.10911E+01_JPRB, 0.10599E+01_JPRB, 0.10357E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.10015E+01_JPRB, 0.10002E+01_JPRB, 0.10063E+01_JPRB, 0.10020E+01_JPRB, 0.99826E+00_JPRB, 0.96672E+00_JPRB ,&
&  0.94982E+00_JPRB, 0.93535E+00_JPRB, 0.92806E+00_JPRB, 0.92635E+00_JPRB, 0.93040E+00_JPRB, 0.92013E+00_JPRB, 0.91227E+00_JPRB ,&
&  0.92632E+00_JPRB, 0.94579E+00_JPRB, 0.10065E+01_JPRB, 0.10814E+01_JPRB, 0.10855E+01_JPRB, 0.10625E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10666E+01_JPRB, 0.10381E+01_JPRB, 0.12032E+01_JPRB, 0.13471E+01_JPRB, 0.14242E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.11354E+01_JPRB, 0.10748E+01_JPRB, 0.10479E+01_JPRB, 0.10697E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.98828E+00_JPRB, 0.98472E+00_JPRB, 0.10015E+01_JPRB, 0.10141E+01_JPRB, 0.99604E+00_JPRB, 0.95786E+00_JPRB, 0.92155E+00_JPRB ,&
&  0.90578E+00_JPRB, 0.89651E+00_JPRB, 0.88484E+00_JPRB, 0.86060E+00_JPRB, 0.84786E+00_JPRB, 0.84475E+00_JPRB, 0.84334E+00_JPRB ,&
&  0.84280E+00_JPRB, 0.82907E+00_JPRB, 0.81066E+00_JPRB, 0.79233E+00_JPRB, 0.79138E+00_JPRB, 0.79759E+00_JPRB, 0.81167E+00_JPRB ,&
&  0.80832E+00_JPRB, 0.80733E+00_JPRB, 0.81312E+00_JPRB, 0.81963E+00_JPRB, 0.82788E+00_JPRB, 0.81873E+00_JPRB, 0.80646E+00_JPRB ,&
&  0.81396E+00_JPRB, 0.10619E+01_JPRB, 0.14325E+01_JPRB, 0.16019E+01_JPRB, 0.15526E+01_JPRB, 0.14075E+01_JPRB, 0.13330E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.12556E+01_JPRB, 0.12634E+01_JPRB, 0.12302E+01_JPRB, 0.11858E+01_JPRB, 0.11448E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10717E+01_JPRB, 0.10397E+01_JPRB, 0.10118E+01_JPRB, 0.10050E+01_JPRB, 0.10027E+01_JPRB, 0.10070E+01_JPRB, 0.98422E+00_JPRB ,&
&  0.95304E+00_JPRB, 0.92487E+00_JPRB, 0.90140E+00_JPRB, 0.90917E+00_JPRB, 0.10236E+01_JPRB, 0.10863E+01_JPRB, 0.13059E+01_JPRB ,&
&  0.14910E+01_JPRB, 0.13166E+01_JPRB, 0.12336E+01_JPRB, 0.12174E+01_JPRB, 0.11981E+01_JPRB, 0.11554E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.11071E+01_JPRB, 0.11104E+01_JPRB, 0.11359E+01_JPRB, 0.11339E+01_JPRB, 0.10903E+01_JPRB, 0.10348E+01_JPRB, 0.10087E+01_JPRB ,&
&  0.99666E+00_JPRB, 0.98369E+00_JPRB, 0.97661E+00_JPRB, 0.96844E+00_JPRB, 0.95858E+00_JPRB, 0.93538E+00_JPRB, 0.92877E+00_JPRB ,&
&  0.94229E+00_JPRB, 0.98730E+00_JPRB, 0.10923E+01_JPRB, 0.11198E+01_JPRB, 0.11509E+01_JPRB, 0.11616E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.10524E+01_JPRB, 0.10595E+01_JPRB, 0.10653E+01_JPRB, 0.11862E+01_JPRB, 0.13903E+01_JPRB, 0.14564E+01_JPRB, 0.13210E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.10709E+01_JPRB, 0.10147E+01_JPRB, 0.98829E+00_JPRB, 0.10054E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.92550E+00_JPRB, 0.91723E+00_JPRB, 0.92781E+00_JPRB, 0.94768E+00_JPRB, 0.93095E+00_JPRB, 0.90073E+00_JPRB, 0.87166E+00_JPRB ,&
&  0.88807E+00_JPRB, 0.89398E+00_JPRB, 0.89247E+00_JPRB, 0.86054E+00_JPRB, 0.84243E+00_JPRB, 0.82208E+00_JPRB, 0.80991E+00_JPRB ,&
&  0.80900E+00_JPRB, 0.79795E+00_JPRB, 0.78099E+00_JPRB, 0.77271E+00_JPRB, 0.77581E+00_JPRB, 0.78035E+00_JPRB, 0.79663E+00_JPRB ,&
&  0.78926E+00_JPRB, 0.79576E+00_JPRB, 0.81838E+00_JPRB, 0.82939E+00_JPRB, 0.83873E+00_JPRB, 0.82123E+00_JPRB, 0.81420E+00_JPRB ,&
&  0.85545E+00_JPRB, 0.95637E+00_JPRB, 0.15732E+01_JPRB, 0.19060E+01_JPRB, 0.15796E+01_JPRB, 0.13844E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.13229E+01_JPRB, 0.12906E+01_JPRB, 0.12274E+01_JPRB, 0.11930E+01_JPRB, 0.11440E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10309E+01_JPRB, 0.10249E+01_JPRB, 0.10338E+01_JPRB, 0.10317E+01_JPRB, 0.10185E+01_JPRB, 0.98975E+00_JPRB ,&
&  0.96883E+00_JPRB, 0.94777E+00_JPRB, 0.93066E+00_JPRB, 0.98637E+00_JPRB, 0.11744E+01_JPRB, 0.14127E+01_JPRB, 0.14874E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.15144E+01_JPRB, 0.13709E+01_JPRB, 0.13363E+01_JPRB, 0.12859E+01_JPRB, 0.12455E+01_JPRB, 0.11883E+01_JPRB ,&
&  0.11561E+01_JPRB, 0.11686E+01_JPRB, 0.11541E+01_JPRB, 0.11130E+01_JPRB, 0.10710E+01_JPRB, 0.10511E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.10266E+01_JPRB, 0.10292E+01_JPRB, 0.10092E+01_JPRB, 0.98960E+00_JPRB, 0.96879E+00_JPRB, 0.96745E+00_JPRB ,&
&  0.99327E+00_JPRB, 0.10323E+01_JPRB, 0.11165E+01_JPRB, 0.11874E+01_JPRB, 0.11758E+01_JPRB, 0.11411E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.11003E+01_JPRB, 0.10812E+01_JPRB, 0.10773E+01_JPRB, 0.11893E+01_JPRB, 0.12576E+01_JPRB, 0.13303E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.10764E+01_JPRB, 0.99135E+00_JPRB, 0.95065E+00_JPRB, 0.93514E+00_JPRB, 0.93698E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.89315E+00_JPRB, 0.87537E+00_JPRB, 0.88837E+00_JPRB, 0.89780E+00_JPRB, 0.89127E+00_JPRB, 0.86519E+00_JPRB, 0.84882E+00_JPRB ,&
&  0.86208E+00_JPRB, 0.86239E+00_JPRB, 0.85585E+00_JPRB, 0.82938E+00_JPRB, 0.81436E+00_JPRB, 0.79137E+00_JPRB, 0.77584E+00_JPRB ,&
&  0.77675E+00_JPRB, 0.77856E+00_JPRB, 0.76934E+00_JPRB, 0.76886E+00_JPRB, 0.77007E+00_JPRB, 0.78531E+00_JPRB, 0.79686E+00_JPRB ,&
&  0.80879E+00_JPRB, 0.81872E+00_JPRB, 0.83563E+00_JPRB, 0.84505E+00_JPRB, 0.84009E+00_JPRB, 0.82729E+00_JPRB, 0.82856E+00_JPRB ,&
&  0.92710E+00_JPRB, 0.21986E+01_JPRB, 0.14181E+01_JPRB, 0.20719E+01_JPRB, 0.15491E+01_JPRB, 0.13260E+01_JPRB, 0.10926E+01_JPRB ,&
&  0.11528E+01_JPRB, 0.13659E+01_JPRB, 0.13356E+01_JPRB, 0.12382E+01_JPRB, 0.12081E+01_JPRB, 0.11730E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.10774E+01_JPRB, 0.10592E+01_JPRB, 0.10567E+01_JPRB, 0.10483E+01_JPRB, 0.10306E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.10085E+01_JPRB, 0.99601E+00_JPRB, 0.10088E+01_JPRB, 0.11668E+01_JPRB, 0.13193E+01_JPRB, 0.15813E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.14756E+01_JPRB, 0.16119E+01_JPRB, 0.14439E+01_JPRB, 0.13552E+01_JPRB, 0.13021E+01_JPRB, 0.12492E+01_JPRB, 0.12192E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.11598E+01_JPRB, 0.11031E+01_JPRB, 0.10764E+01_JPRB, 0.10750E+01_JPRB, 0.10655E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10009E+01_JPRB, 0.10042E+01_JPRB, 0.10178E+01_JPRB, 0.10172E+01_JPRB, 0.10039E+01_JPRB, 0.97643E+00_JPRB, 0.97469E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.10324E+01_JPRB, 0.10834E+01_JPRB, 0.11916E+01_JPRB, 0.11713E+01_JPRB, 0.11149E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.10973E+01_JPRB, 0.10992E+01_JPRB, 0.11601E+01_JPRB, 0.12078E+01_JPRB, 0.12498E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.98559E+00_JPRB, 0.93379E+00_JPRB, 0.90777E+00_JPRB, 0.89581E+00_JPRB, 0.90049E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.89850E+00_JPRB, 0.87144E+00_JPRB, 0.85444E+00_JPRB, 0.86215E+00_JPRB, 0.86650E+00_JPRB, 0.85166E+00_JPRB, 0.82961E+00_JPRB ,&
&  0.81517E+00_JPRB, 0.79866E+00_JPRB, 0.80807E+00_JPRB, 0.79592E+00_JPRB, 0.78858E+00_JPRB, 0.77770E+00_JPRB, 0.77094E+00_JPRB ,&
&  0.77283E+00_JPRB, 0.78074E+00_JPRB, 0.76819E+00_JPRB, 0.77245E+00_JPRB, 0.79897E+00_JPRB, 0.82429E+00_JPRB, 0.83371E+00_JPRB ,&
&  0.84385E+00_JPRB, 0.85535E+00_JPRB, 0.85678E+00_JPRB, 0.85719E+00_JPRB, 0.84611E+00_JPRB, 0.84964E+00_JPRB, 0.86462E+00_JPRB ,&
&  0.99255E+00_JPRB, 0.22805E+01_JPRB, 0.16956E+01_JPRB, 0.20608E+01_JPRB, 0.15534E+01_JPRB, 0.12940E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.13657E+01_JPRB, 0.13282E+01_JPRB, 0.13025E+01_JPRB, 0.12363E+01_JPRB, 0.11859E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11067E+01_JPRB, 0.11075E+01_JPRB, 0.10939E+01_JPRB, 0.10867E+01_JPRB, 0.10632E+01_JPRB, 0.10682E+01_JPRB, 0.10726E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10931E+01_JPRB, 0.11650E+01_JPRB, 0.12902E+01_JPRB, 0.16084E+01_JPRB, 0.17962E+01_JPRB, 0.16924E+01_JPRB ,&
&  0.15440E+01_JPRB, 0.16795E+01_JPRB, 0.15263E+01_JPRB, 0.13623E+01_JPRB, 0.12755E+01_JPRB, 0.12192E+01_JPRB, 0.12079E+01_JPRB ,&
&  0.11430E+01_JPRB, 0.11033E+01_JPRB, 0.10729E+01_JPRB, 0.10570E+01_JPRB, 0.10658E+01_JPRB, 0.10620E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.98966E+00_JPRB, 0.99892E+00_JPRB, 0.10121E+01_JPRB, 0.10206E+01_JPRB, 0.10096E+01_JPRB, 0.99220E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.10326E+01_JPRB, 0.10707E+01_JPRB, 0.10979E+01_JPRB, 0.11764E+01_JPRB, 0.11616E+01_JPRB, 0.11236E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.11181E+01_JPRB, 0.11095E+01_JPRB, 0.11118E+01_JPRB, 0.11208E+01_JPRB, 0.10916E+01_JPRB, 0.97028E+00_JPRB ,&
&  0.94960E+00_JPRB, 0.94301E+00_JPRB, 0.93504E+00_JPRB, 0.90911E+00_JPRB, 0.92110E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.85978E+00_JPRB, 0.81978E+00_JPRB, 0.81644E+00_JPRB, 0.83504E+00_JPRB, 0.82968E+00_JPRB, 0.81026E+00_JPRB, 0.77734E+00_JPRB ,&
&  0.74262E+00_JPRB, 0.75430E+00_JPRB, 0.77961E+00_JPRB, 0.79809E+00_JPRB, 0.80979E+00_JPRB, 0.81979E+00_JPRB, 0.80396E+00_JPRB ,&
&  0.77927E+00_JPRB, 0.78421E+00_JPRB, 0.79981E+00_JPRB, 0.82885E+00_JPRB, 0.85049E+00_JPRB, 0.87189E+00_JPRB, 0.90250E+00_JPRB ,&
&  0.91094E+00_JPRB, 0.90595E+00_JPRB, 0.89219E+00_JPRB, 0.88070E+00_JPRB, 0.87882E+00_JPRB, 0.89971E+00_JPRB, 0.94415E+00_JPRB ,&
&  0.11855E+01_JPRB, 0.28974E+01_JPRB, 0.28430E+01_JPRB, 0.20441E+01_JPRB, 0.15740E+01_JPRB, 0.13010E+01_JPRB, 0.11489E+01_JPRB ,&
&  0.11709E+01_JPRB, 0.11896E+01_JPRB, 0.13782E+01_JPRB, 0.13372E+01_JPRB, 0.12558E+01_JPRB, 0.12135E+01_JPRB, 0.12152E+01_JPRB ,&
&  0.11903E+01_JPRB, 0.11881E+01_JPRB, 0.11668E+01_JPRB, 0.11454E+01_JPRB, 0.11277E+01_JPRB, 0.11478E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.12016E+01_JPRB, 0.12661E+01_JPRB, 0.14283E+01_JPRB, 0.16470E+01_JPRB, 0.17444E+01_JPRB, 0.19104E+01_JPRB, 0.17830E+01_JPRB ,&
&  0.16775E+01_JPRB, 0.16627E+01_JPRB, 0.14887E+01_JPRB, 0.13611E+01_JPRB, 0.12248E+01_JPRB, 0.12544E+01_JPRB, 0.11978E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.10767E+01_JPRB, 0.10644E+01_JPRB, 0.10703E+01_JPRB, 0.10771E+01_JPRB, 0.10764E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.10327E+01_JPRB, 0.10333E+01_JPRB, 0.10366E+01_JPRB, 0.10366E+01_JPRB, 0.10421E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.11048E+01_JPRB, 0.11526E+01_JPRB, 0.11740E+01_JPRB, 0.12355E+01_JPRB, 0.12295E+01_JPRB, 0.11617E+01_JPRB, 0.10963E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11581E+01_JPRB, 0.11308E+01_JPRB, 0.11063E+01_JPRB, 0.10645E+01_JPRB, 0.10066E+01_JPRB, 0.96859E+00_JPRB ,&
&  0.98253E+00_JPRB, 0.10025E+01_JPRB, 0.95805E+00_JPRB, 0.89961E+00_JPRB, 0.90661E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.81972E+00_JPRB, 0.78446E+00_JPRB, 0.77717E+00_JPRB, 0.77068E+00_JPRB, 0.75601E+00_JPRB, 0.75056E+00_JPRB, 0.72192E+00_JPRB ,&
&  0.73117E+00_JPRB, 0.78667E+00_JPRB, 0.85446E+00_JPRB, 0.88703E+00_JPRB, 0.90638E+00_JPRB, 0.90630E+00_JPRB, 0.88128E+00_JPRB ,&
&  0.85117E+00_JPRB, 0.86014E+00_JPRB, 0.87700E+00_JPRB, 0.88823E+00_JPRB, 0.90754E+00_JPRB, 0.92266E+00_JPRB, 0.94821E+00_JPRB ,&
&  0.96928E+00_JPRB, 0.94854E+00_JPRB, 0.91907E+00_JPRB, 0.90993E+00_JPRB, 0.96180E+00_JPRB, 0.10428E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.16726E+01_JPRB, 0.33061E+01_JPRB, 0.17533E+01_JPRB, 0.16036E+01_JPRB, 0.14927E+01_JPRB, 0.13451E+01_JPRB, 0.12177E+01_JPRB ,&
&  0.12067E+01_JPRB, 0.10808E+01_JPRB, 0.11583E+01_JPRB, 0.13322E+01_JPRB, 0.13080E+01_JPRB, 0.12840E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.12985E+01_JPRB, 0.12968E+01_JPRB, 0.12970E+01_JPRB, 0.12623E+01_JPRB, 0.12629E+01_JPRB, 0.12719E+01_JPRB, 0.13157E+01_JPRB ,&
&  0.13702E+01_JPRB, 0.14475E+01_JPRB, 0.16131E+01_JPRB, 0.16887E+01_JPRB, 0.19298E+01_JPRB, 0.20427E+01_JPRB, 0.17442E+01_JPRB ,&
&  0.14137E+01_JPRB, 0.13970E+01_JPRB, 0.13986E+01_JPRB, 0.14324E+01_JPRB, 0.13637E+01_JPRB, 0.11614E+01_JPRB, 0.11919E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.10746E+01_JPRB, 0.10809E+01_JPRB, 0.10821E+01_JPRB, 0.11046E+01_JPRB, 0.11063E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.10797E+01_JPRB, 0.10724E+01_JPRB, 0.10709E+01_JPRB, 0.10746E+01_JPRB, 0.10976E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.12359E+01_JPRB, 0.13054E+01_JPRB, 0.13336E+01_JPRB, 0.13254E+01_JPRB, 0.12539E+01_JPRB, 0.11776E+01_JPRB ,&
&  0.11855E+01_JPRB, 0.12238E+01_JPRB, 0.10993E+01_JPRB, 0.10437E+01_JPRB, 0.10241E+01_JPRB, 0.99013E+00_JPRB, 0.10232E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.12250E+01_JPRB, 0.13080E+01_JPRB, 0.86403E+00_JPRB, 0.85614E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.73602E+00_JPRB, 0.73639E+00_JPRB, 0.70803E+00_JPRB, 0.69001E+00_JPRB, 0.68516E+00_JPRB, 0.71019E+00_JPRB, 0.75600E+00_JPRB ,&
&  0.82697E+00_JPRB, 0.91923E+00_JPRB, 0.99623E+00_JPRB, 0.10280E+01_JPRB, 0.10184E+01_JPRB, 0.10232E+01_JPRB, 0.97735E+00_JPRB ,&
&  0.92957E+00_JPRB, 0.92371E+00_JPRB, 0.93790E+00_JPRB, 0.94905E+00_JPRB, 0.96430E+00_JPRB, 0.98692E+00_JPRB, 0.10138E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10398E+01_JPRB, 0.10537E+01_JPRB, 0.11306E+01_JPRB, 0.12601E+01_JPRB, 0.14627E+01_JPRB, 0.18242E+01_JPRB ,&
&  0.26012E+01_JPRB, 0.25771E+01_JPRB, 0.13361E+01_JPRB, 0.14016E+01_JPRB, 0.12125E+01_JPRB, 0.12627E+01_JPRB, 0.12188E+01_JPRB ,&
&  0.12813E+01_JPRB, 0.12291E+01_JPRB, 0.12822E+01_JPRB, 0.14334E+01_JPRB, 0.14273E+01_JPRB, 0.14307E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.14419E+01_JPRB, 0.14387E+01_JPRB, 0.14177E+01_JPRB, 0.13862E+01_JPRB, 0.13782E+01_JPRB, 0.14070E+01_JPRB, 0.14618E+01_JPRB ,&
&  0.15582E+01_JPRB, 0.16347E+01_JPRB, 0.18015E+01_JPRB, 0.17751E+01_JPRB, 0.21483E+01_JPRB, 0.22198E+01_JPRB, 0.19071E+01_JPRB ,&
&  0.15365E+01_JPRB, 0.12859E+01_JPRB, 0.13494E+01_JPRB, 0.14561E+01_JPRB, 0.13924E+01_JPRB, 0.11105E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.10593E+01_JPRB, 0.10527E+01_JPRB, 0.10548E+01_JPRB, 0.10795E+01_JPRB, 0.11232E+01_JPRB, 0.11336E+01_JPRB, 0.11420E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.10987E+01_JPRB, 0.10939E+01_JPRB, 0.10967E+01_JPRB, 0.11113E+01_JPRB, 0.11520E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.12077E+01_JPRB, 0.12530E+01_JPRB, 0.13127E+01_JPRB, 0.13760E+01_JPRB, 0.13654E+01_JPRB, 0.12930E+01_JPRB, 0.12010E+01_JPRB ,&
&  0.11746E+01_JPRB, 0.11615E+01_JPRB, 0.10567E+01_JPRB, 0.99716E+00_JPRB, 0.92668E+00_JPRB, 0.91973E+00_JPRB, 0.89901E+00_JPRB ,&
&  0.89698E+00_JPRB, 0.89536E+00_JPRB, 0.85185E+00_JPRB, 0.74347E+00_JPRB, 0.74030E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.70656E+00_JPRB, 0.71092E+00_JPRB, 0.70364E+00_JPRB, 0.72074E+00_JPRB, 0.72285E+00_JPRB, 0.78842E+00_JPRB, 0.85913E+00_JPRB ,&
&  0.96756E+00_JPRB, 0.10481E+01_JPRB, 0.10988E+01_JPRB, 0.11111E+01_JPRB, 0.10928E+01_JPRB, 0.10383E+01_JPRB, 0.98735E+00_JPRB ,&
&  0.98854E+00_JPRB, 0.99446E+00_JPRB, 0.10109E+01_JPRB, 0.10354E+01_JPRB, 0.10598E+01_JPRB, 0.10973E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.11864E+01_JPRB, 0.12619E+01_JPRB, 0.13564E+01_JPRB, 0.14938E+01_JPRB, 0.16710E+01_JPRB, 0.19131E+01_JPRB, 0.21495E+01_JPRB ,&
&  0.16632E+01_JPRB, 0.84066E+00_JPRB, 0.11894E+01_JPRB, 0.11769E+01_JPRB, 0.10949E+01_JPRB, 0.12720E+01_JPRB, 0.14975E+01_JPRB ,&
&  0.13994E+01_JPRB, 0.13234E+01_JPRB, 0.13027E+01_JPRB, 0.15218E+01_JPRB, 0.15293E+01_JPRB, 0.15410E+01_JPRB, 0.15523E+01_JPRB ,&
&  0.15506E+01_JPRB, 0.15350E+01_JPRB, 0.15083E+01_JPRB, 0.15132E+01_JPRB, 0.15281E+01_JPRB, 0.15820E+01_JPRB, 0.16556E+01_JPRB ,&
&  0.17559E+01_JPRB, 0.18748E+01_JPRB, 0.20560E+01_JPRB, 0.18269E+01_JPRB, 0.21863E+01_JPRB, 0.24863E+01_JPRB, 0.24538E+01_JPRB ,&
&  0.23701E+01_JPRB, 0.13157E+01_JPRB, 0.11524E+01_JPRB, 0.14243E+01_JPRB, 0.13573E+01_JPRB, 0.10691E+01_JPRB, 0.10765E+01_JPRB ,&
&  0.10312E+01_JPRB, 0.10558E+01_JPRB, 0.10438E+01_JPRB, 0.11111E+01_JPRB, 0.11474E+01_JPRB, 0.11707E+01_JPRB, 0.11452E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.10861E+01_JPRB, 0.10769E+01_JPRB, 0.10884E+01_JPRB, 0.11008E+01_JPRB, 0.11349E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11821E+01_JPRB, 0.12398E+01_JPRB, 0.12955E+01_JPRB, 0.13184E+01_JPRB, 0.12482E+01_JPRB, 0.11878E+01_JPRB ,&
&  0.11378E+01_JPRB, 0.10866E+01_JPRB, 0.98011E+00_JPRB, 0.86794E+00_JPRB, 0.83178E+00_JPRB, 0.77088E+00_JPRB, 0.71209E+00_JPRB ,&
&  0.72360E+00_JPRB, 0.72125E+00_JPRB, 0.70577E+00_JPRB, 0.70438E+00_JPRB, 0.70854E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.79287E+00_JPRB, 0.78489E+00_JPRB, 0.77748E+00_JPRB, 0.78133E+00_JPRB, 0.79042E+00_JPRB, 0.83873E+00_JPRB, 0.91730E+00_JPRB ,&
&  0.99296E+00_JPRB, 0.10398E+01_JPRB, 0.10824E+01_JPRB, 0.11040E+01_JPRB, 0.11087E+01_JPRB, 0.10695E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10614E+01_JPRB, 0.10628E+01_JPRB, 0.10885E+01_JPRB, 0.11273E+01_JPRB, 0.11619E+01_JPRB, 0.12000E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.12958E+01_JPRB, 0.13745E+01_JPRB, 0.14560E+01_JPRB, 0.15805E+01_JPRB, 0.16793E+01_JPRB, 0.17417E+01_JPRB, 0.17117E+01_JPRB ,&
&  0.78635E+00_JPRB, 0.89527E+00_JPRB, 0.13172E+01_JPRB, 0.12916E+01_JPRB, 0.13727E+01_JPRB, 0.14835E+01_JPRB, 0.16154E+01_JPRB ,&
&  0.15258E+01_JPRB, 0.13442E+01_JPRB, 0.13617E+01_JPRB, 0.16187E+01_JPRB, 0.16121E+01_JPRB, 0.16607E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.16783E+01_JPRB, 0.16562E+01_JPRB, 0.16462E+01_JPRB, 0.16751E+01_JPRB, 0.17306E+01_JPRB, 0.17982E+01_JPRB, 0.18969E+01_JPRB ,&
&  0.19808E+01_JPRB, 0.20950E+01_JPRB, 0.22267E+01_JPRB, 0.18957E+01_JPRB, 0.21155E+01_JPRB, 0.24911E+01_JPRB, 0.31327E+01_JPRB ,&
&  0.25438E+01_JPRB, 0.13234E+01_JPRB, 0.11292E+01_JPRB, 0.13686E+01_JPRB, 0.12501E+01_JPRB, 0.12284E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10196E+01_JPRB, 0.96904E+00_JPRB, 0.99648E+00_JPRB, 0.10593E+01_JPRB, 0.11140E+01_JPRB, 0.11496E+01_JPRB, 0.11306E+01_JPRB ,&
&  0.10686E+01_JPRB, 0.10311E+01_JPRB, 0.10143E+01_JPRB, 0.10227E+01_JPRB, 0.10442E+01_JPRB, 0.10580E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10471E+01_JPRB, 0.10496E+01_JPRB, 0.10898E+01_JPRB, 0.11153E+01_JPRB, 0.11419E+01_JPRB, 0.11699E+01_JPRB, 0.12029E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11310E+01_JPRB, 0.12202E+01_JPRB, 0.11269E+01_JPRB, 0.10072E+01_JPRB, 0.86601E+00_JPRB, 0.73438E+00_JPRB ,&
&  0.75530E+00_JPRB, 0.76562E+00_JPRB, 0.76297E+00_JPRB, 0.77098E+00_JPRB, 0.78905E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.88425E+00_JPRB, 0.86758E+00_JPRB, 0.85614E+00_JPRB, 0.84557E+00_JPRB, 0.85749E+00_JPRB, 0.89214E+00_JPRB, 0.93212E+00_JPRB ,&
&  0.97254E+00_JPRB, 0.10191E+01_JPRB, 0.10820E+01_JPRB, 0.11054E+01_JPRB, 0.11462E+01_JPRB, 0.11463E+01_JPRB, 0.11360E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.11310E+01_JPRB, 0.11579E+01_JPRB, 0.11867E+01_JPRB, 0.11982E+01_JPRB, 0.12282E+01_JPRB, 0.12679E+01_JPRB ,&
&  0.13226E+01_JPRB, 0.13755E+01_JPRB, 0.14430E+01_JPRB, 0.15562E+01_JPRB, 0.15964E+01_JPRB, 0.16076E+01_JPRB, 0.16363E+01_JPRB ,&
&  0.87849E+00_JPRB, 0.13359E+01_JPRB, 0.14219E+01_JPRB, 0.13969E+01_JPRB, 0.14785E+01_JPRB, 0.14817E+01_JPRB, 0.14724E+01_JPRB ,&
&  0.15121E+01_JPRB, 0.14793E+01_JPRB, 0.14757E+01_JPRB, 0.15751E+01_JPRB, 0.16998E+01_JPRB, 0.17592E+01_JPRB, 0.17998E+01_JPRB ,&
&  0.17979E+01_JPRB, 0.17914E+01_JPRB, 0.18278E+01_JPRB, 0.19027E+01_JPRB, 0.19736E+01_JPRB, 0.20475E+01_JPRB, 0.21479E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.22195E+01_JPRB, 0.23195E+01_JPRB, 0.20596E+01_JPRB, 0.19680E+01_JPRB, 0.21149E+01_JPRB, 0.19991E+01_JPRB ,&
&  0.15752E+01_JPRB, 0.93025E+00_JPRB, 0.98997E+00_JPRB, 0.12125E+01_JPRB, 0.10624E+01_JPRB, 0.10126E+01_JPRB, 0.94187E+00_JPRB ,&
&  0.84458E+00_JPRB, 0.80754E+00_JPRB, 0.88299E+00_JPRB, 0.95897E+00_JPRB, 0.10271E+01_JPRB, 0.10795E+01_JPRB, 0.10696E+01_JPRB ,&
&  0.10095E+01_JPRB, 0.96414E+00_JPRB, 0.92850E+00_JPRB, 0.92479E+00_JPRB, 0.94896E+00_JPRB, 0.96474E+00_JPRB, 0.95725E+00_JPRB ,&
&  0.95377E+00_JPRB, 0.93920E+00_JPRB, 0.97856E+00_JPRB, 0.10097E+01_JPRB, 0.10510E+01_JPRB, 0.10968E+01_JPRB, 0.12408E+01_JPRB ,&
&  0.14793E+01_JPRB, 0.16960E+01_JPRB, 0.19549E+01_JPRB, 0.19042E+01_JPRB, 0.17304E+01_JPRB, 0.12467E+01_JPRB, 0.84475E+00_JPRB ,&
&  0.86592E+00_JPRB, 0.88745E+00_JPRB, 0.89114E+00_JPRB, 0.90180E+00_JPRB, 0.89327E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10237E+01_JPRB, 0.10088E+01_JPRB, 0.10079E+01_JPRB, 0.10064E+01_JPRB, 0.10049E+01_JPRB, 0.10143E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10866E+01_JPRB, 0.11432E+01_JPRB, 0.11806E+01_JPRB, 0.12289E+01_JPRB, 0.12347E+01_JPRB, 0.12044E+01_JPRB ,&
&  0.12002E+01_JPRB, 0.12366E+01_JPRB, 0.12693E+01_JPRB, 0.12827E+01_JPRB, 0.12472E+01_JPRB, 0.12286E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.13213E+01_JPRB, 0.13553E+01_JPRB, 0.14092E+01_JPRB, 0.14194E+01_JPRB, 0.13503E+01_JPRB, 0.93193E+00_JPRB ,&
&  0.12920E+01_JPRB, 0.14804E+01_JPRB, 0.14994E+01_JPRB, 0.14408E+01_JPRB, 0.14185E+01_JPRB, 0.13520E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.14490E+01_JPRB, 0.15294E+01_JPRB, 0.15306E+01_JPRB, 0.14907E+01_JPRB, 0.16768E+01_JPRB, 0.17033E+01_JPRB, 0.17450E+01_JPRB ,&
&  0.17690E+01_JPRB, 0.17917E+01_JPRB, 0.18278E+01_JPRB, 0.18751E+01_JPRB, 0.19200E+01_JPRB, 0.19749E+01_JPRB, 0.20136E+01_JPRB ,&
&  0.20062E+01_JPRB, 0.19768E+01_JPRB, 0.19935E+01_JPRB, 0.18556E+01_JPRB, 0.17932E+01_JPRB, 0.17984E+01_JPRB, 0.15158E+01_JPRB ,&
&  0.93634E+00_JPRB, 0.68729E+00_JPRB, 0.94659E+00_JPRB, 0.11721E+01_JPRB, 0.10200E+01_JPRB, 0.93297E+00_JPRB, 0.90693E+00_JPRB ,&
&  0.83864E+00_JPRB, 0.76409E+00_JPRB, 0.74650E+00_JPRB, 0.80201E+00_JPRB, 0.88182E+00_JPRB, 0.94478E+00_JPRB, 0.96069E+00_JPRB ,&
&  0.92686E+00_JPRB, 0.90256E+00_JPRB, 0.90292E+00_JPRB, 0.92359E+00_JPRB, 0.97029E+00_JPRB, 0.10665E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.87601E+00_JPRB, 0.99676E+00_JPRB, 0.10612E+01_JPRB, 0.11780E+01_JPRB, 0.13410E+01_JPRB, 0.16431E+01_JPRB ,&
&  0.21156E+01_JPRB, 0.26154E+01_JPRB, 0.25629E+01_JPRB, 0.21438E+01_JPRB, 0.27061E+01_JPRB, 0.18168E+01_JPRB, 0.97343E+00_JPRB ,&
&  0.10038E+01_JPRB, 0.10232E+01_JPRB, 0.10422E+01_JPRB, 0.10381E+01_JPRB, 0.10338E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10589E+01_JPRB, 0.10621E+01_JPRB, 0.10557E+01_JPRB, 0.10870E+01_JPRB, 0.10858E+01_JPRB, 0.11022E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.11495E+01_JPRB, 0.11746E+01_JPRB, 0.12155E+01_JPRB, 0.12292E+01_JPRB, 0.12480E+01_JPRB, 0.12804E+01_JPRB ,&
&  0.13125E+01_JPRB, 0.13523E+01_JPRB, 0.13528E+01_JPRB, 0.13402E+01_JPRB, 0.13082E+01_JPRB, 0.12876E+01_JPRB, 0.12610E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.12370E+01_JPRB, 0.12459E+01_JPRB, 0.12648E+01_JPRB, 0.12835E+01_JPRB, 0.13384E+01_JPRB, 0.91632E+00_JPRB ,&
&  0.13791E+01_JPRB, 0.15154E+01_JPRB, 0.14336E+01_JPRB, 0.13712E+01_JPRB, 0.12828E+01_JPRB, 0.12977E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.14141E+01_JPRB, 0.14937E+01_JPRB, 0.15193E+01_JPRB, 0.15131E+01_JPRB, 0.14820E+01_JPRB, 0.14978E+01_JPRB, 0.15066E+01_JPRB ,&
&  0.15091E+01_JPRB, 0.15038E+01_JPRB, 0.14965E+01_JPRB, 0.14847E+01_JPRB, 0.14836E+01_JPRB, 0.15044E+01_JPRB, 0.14995E+01_JPRB ,&
&  0.14648E+01_JPRB, 0.14177E+01_JPRB, 0.13868E+01_JPRB, 0.14453E+01_JPRB, 0.14090E+01_JPRB, 0.12035E+01_JPRB, 0.92526E+00_JPRB ,&
&  0.61600E+00_JPRB, 0.60456E+00_JPRB, 0.87105E+00_JPRB, 0.12036E+01_JPRB, 0.10792E+01_JPRB, 0.96117E+00_JPRB, 0.86169E+00_JPRB ,&
&  0.81281E+00_JPRB, 0.73742E+00_JPRB, 0.66902E+00_JPRB, 0.66637E+00_JPRB, 0.69041E+00_JPRB, 0.74351E+00_JPRB, 0.74426E+00_JPRB ,&
&  0.76031E+00_JPRB, 0.76117E+00_JPRB, 0.74608E+00_JPRB, 0.74463E+00_JPRB, 0.79507E+00_JPRB, 0.80468E+00_JPRB, 0.72330E+00_JPRB ,&
&  0.82915E+00_JPRB, 0.74898E+00_JPRB, 0.79962E+00_JPRB, 0.88481E+00_JPRB, 0.94852E+00_JPRB, 0.11408E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.18897E+01_JPRB, 0.18535E+01_JPRB, 0.24626E+01_JPRB, 0.37568E+01_JPRB, 0.29509E+01_JPRB, 0.21027E+01_JPRB, 0.11352E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.10923E+01_JPRB, 0.11004E+01_JPRB, 0.10815E+01_JPRB, 0.10669E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.98081E+00_JPRB, 0.99324E+00_JPRB, 0.10132E+01_JPRB, 0.10408E+01_JPRB, 0.10561E+01_JPRB, 0.10801E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.11036E+01_JPRB, 0.11353E+01_JPRB, 0.11653E+01_JPRB, 0.11982E+01_JPRB, 0.12277E+01_JPRB, 0.12562E+01_JPRB, 0.12834E+01_JPRB ,&
&  0.13070E+01_JPRB, 0.13289E+01_JPRB, 0.13389E+01_JPRB, 0.13360E+01_JPRB, 0.13189E+01_JPRB, 0.13094E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.12719E+01_JPRB, 0.12743E+01_JPRB, 0.12857E+01_JPRB, 0.12931E+01_JPRB, 0.13077E+01_JPRB, 0.13884E+01_JPRB, 0.99926E+00_JPRB ,&
&  0.14312E+01_JPRB, 0.14800E+01_JPRB, 0.14176E+01_JPRB, 0.12670E+01_JPRB, 0.11888E+01_JPRB, 0.11833E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.12544E+01_JPRB, 0.13283E+01_JPRB, 0.13953E+01_JPRB, 0.13181E+01_JPRB, 0.12087E+01_JPRB, 0.12106E+01_JPRB, 0.12078E+01_JPRB ,&
&  0.12069E+01_JPRB, 0.11929E+01_JPRB, 0.11777E+01_JPRB, 0.11515E+01_JPRB, 0.11303E+01_JPRB, 0.11077E+01_JPRB, 0.10738E+01_JPRB ,&
&  0.10232E+01_JPRB, 0.96125E+00_JPRB, 0.86179E+00_JPRB, 0.87436E+00_JPRB, 0.86184E+00_JPRB, 0.74829E+00_JPRB, 0.57777E+00_JPRB ,&
&  0.52741E+00_JPRB, 0.70590E+00_JPRB, 0.90243E+00_JPRB, 0.12378E+01_JPRB, 0.11299E+01_JPRB, 0.10295E+01_JPRB, 0.93020E+00_JPRB ,&
&  0.77955E+00_JPRB, 0.69440E+00_JPRB, 0.64065E+00_JPRB, 0.63300E+00_JPRB, 0.63128E+00_JPRB, 0.64097E+00_JPRB, 0.63127E+00_JPRB ,&
&  0.61043E+00_JPRB, 0.58710E+00_JPRB, 0.59643E+00_JPRB, 0.61321E+00_JPRB, 0.65083E+00_JPRB, 0.60177E+00_JPRB, 0.81783E+00_JPRB ,&
&  0.69080E+00_JPRB, 0.57998E+00_JPRB, 0.74558E+00_JPRB, 0.51869E+00_JPRB, 0.67046E+00_JPRB, 0.76184E+00_JPRB, 0.88355E+00_JPRB ,&
&  0.92135E+00_JPRB, 0.12164E+01_JPRB, 0.15189E+01_JPRB, 0.13470E+01_JPRB, 0.12062E+01_JPRB, 0.10899E+01_JPRB, 0.10433E+01_JPRB ,&
&  0.10298E+01_JPRB, 0.10170E+01_JPRB, 0.10106E+01_JPRB, 0.99925E+00_JPRB, 0.98435E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.93657E+00_JPRB, 0.94526E+00_JPRB, 0.95850E+00_JPRB, 0.97623E+00_JPRB, 0.99378E+00_JPRB, 0.10101E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.10325E+01_JPRB, 0.10485E+01_JPRB, 0.10731E+01_JPRB, 0.11021E+01_JPRB, 0.11149E+01_JPRB, 0.11326E+01_JPRB, 0.11381E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11902E+01_JPRB, 0.12036E+01_JPRB, 0.12190E+01_JPRB, 0.12298E+01_JPRB, 0.12456E+01_JPRB, 0.12660E+01_JPRB ,&
&  0.12838E+01_JPRB, 0.13253E+01_JPRB, 0.13861E+01_JPRB, 0.14342E+01_JPRB, 0.15006E+01_JPRB, 0.16250E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.15284E+01_JPRB, 0.15178E+01_JPRB, 0.13752E+01_JPRB, 0.11868E+01_JPRB, 0.11215E+01_JPRB, 0.10688E+01_JPRB, 0.10301E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.96519E+00_JPRB, 0.97754E+00_JPRB, 0.97327E+00_JPRB, 0.96471E+00_JPRB, 0.10003E+01_JPRB, 0.99831E+00_JPRB ,&
&  0.10049E+01_JPRB, 0.10128E+01_JPRB, 0.10252E+01_JPRB, 0.10249E+01_JPRB, 0.10365E+01_JPRB, 0.10246E+01_JPRB, 0.98196E+00_JPRB ,&
&  0.92560E+00_JPRB, 0.88563E+00_JPRB, 0.82442E+00_JPRB, 0.81127E+00_JPRB, 0.79970E+00_JPRB, 0.72875E+00_JPRB, 0.65477E+00_JPRB ,&
&  0.73238E+00_JPRB, 0.79956E+00_JPRB, 0.86410E+00_JPRB, 0.12015E+01_JPRB, 0.11745E+01_JPRB, 0.11187E+01_JPRB, 0.94123E+00_JPRB ,&
&  0.83645E+00_JPRB, 0.70875E+00_JPRB, 0.66272E+00_JPRB, 0.66929E+00_JPRB, 0.65996E+00_JPRB, 0.65038E+00_JPRB, 0.62230E+00_JPRB ,&
&  0.57202E+00_JPRB, 0.54167E+00_JPRB, 0.56525E+00_JPRB, 0.61107E+00_JPRB, 0.68600E+00_JPRB, 0.70311E+00_JPRB, 0.89169E+00_JPRB ,&
&  0.66617E+00_JPRB, 0.44648E+00_JPRB, 0.49530E+00_JPRB, 0.59972E+00_JPRB, 0.59060E+00_JPRB, 0.63288E+00_JPRB, 0.76413E+00_JPRB ,&
&  0.97517E+00_JPRB, 0.10406E+01_JPRB, 0.10607E+01_JPRB, 0.10038E+01_JPRB, 0.99117E+00_JPRB, 0.98822E+00_JPRB, 0.97037E+00_JPRB ,&
&  0.96207E+00_JPRB, 0.95740E+00_JPRB, 0.95320E+00_JPRB, 0.94225E+00_JPRB, 0.93501E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.90461E+00_JPRB, 0.90716E+00_JPRB, 0.91317E+00_JPRB, 0.92338E+00_JPRB, 0.93346E+00_JPRB, 0.94391E+00_JPRB, 0.94897E+00_JPRB ,&
&  0.94480E+00_JPRB, 0.94793E+00_JPRB, 0.95841E+00_JPRB, 0.97439E+00_JPRB, 0.98966E+00_JPRB, 0.10030E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10177E+01_JPRB, 0.10206E+01_JPRB, 0.10398E+01_JPRB, 0.10623E+01_JPRB, 0.10757E+01_JPRB, 0.10788E+01_JPRB, 0.10861E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.10987E+01_JPRB, 0.11305E+01_JPRB, 0.11485E+01_JPRB, 0.11741E+01_JPRB, 0.12375E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14226E+01_JPRB, 0.15163E+01_JPRB, 0.12919E+01_JPRB, 0.11342E+01_JPRB, 0.11142E+01_JPRB, 0.10604E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.96770E+00_JPRB, 0.93518E+00_JPRB, 0.88745E+00_JPRB, 0.88534E+00_JPRB, 0.87066E+00_JPRB, 0.89421E+00_JPRB, 0.88689E+00_JPRB ,&
&  0.91637E+00_JPRB, 0.93846E+00_JPRB, 0.97959E+00_JPRB, 0.10126E+01_JPRB, 0.10691E+01_JPRB, 0.10792E+01_JPRB, 0.10070E+01_JPRB ,&
&  0.98698E+00_JPRB, 0.10486E+01_JPRB, 0.10904E+01_JPRB, 0.11502E+01_JPRB, 0.11168E+01_JPRB, 0.10558E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.97603E+00_JPRB, 0.96546E+00_JPRB, 0.11137E+01_JPRB, 0.11257E+01_JPRB, 0.11867E+01_JPRB, 0.11712E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.94868E+00_JPRB, 0.83013E+00_JPRB, 0.82054E+00_JPRB, 0.81979E+00_JPRB, 0.84516E+00_JPRB, 0.82837E+00_JPRB, 0.80482E+00_JPRB ,&
&  0.73215E+00_JPRB, 0.66732E+00_JPRB, 0.69371E+00_JPRB, 0.71995E+00_JPRB, 0.73442E+00_JPRB, 0.88309E+00_JPRB, 0.87996E+00_JPRB ,&
&  0.71405E+00_JPRB, 0.65274E+00_JPRB, 0.50772E+00_JPRB, 0.71419E+00_JPRB, 0.64818E+00_JPRB, 0.69345E+00_JPRB, 0.77198E+00_JPRB ,&
&  0.88434E+00_JPRB, 0.98165E+00_JPRB, 0.10432E+01_JPRB, 0.99957E+00_JPRB, 0.92440E+00_JPRB, 0.90879E+00_JPRB, 0.89874E+00_JPRB ,&
&  0.90244E+00_JPRB, 0.90373E+00_JPRB, 0.90338E+00_JPRB, 0.90137E+00_JPRB, 0.89621E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.84638E+00_JPRB, 0.84829E+00_JPRB, 0.86452E+00_JPRB, 0.86713E+00_JPRB, 0.86657E+00_JPRB, 0.87418E+00_JPRB, 0.87714E+00_JPRB ,&
&  0.87221E+00_JPRB, 0.87999E+00_JPRB, 0.89035E+00_JPRB, 0.90197E+00_JPRB, 0.91649E+00_JPRB, 0.92980E+00_JPRB, 0.93765E+00_JPRB ,&
&  0.92928E+00_JPRB, 0.92056E+00_JPRB, 0.94396E+00_JPRB, 0.96377E+00_JPRB, 0.96799E+00_JPRB, 0.97569E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.11564E+01_JPRB, 0.11700E+01_JPRB, 0.11506E+01_JPRB, 0.11832E+01_JPRB, 0.13766E+01_JPRB, 0.20874E+01_JPRB ,&
&  0.20336E+01_JPRB, 0.14345E+01_JPRB, 0.12583E+01_JPRB, 0.11267E+01_JPRB, 0.11310E+01_JPRB, 0.11645E+01_JPRB, 0.11396E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10907E+01_JPRB, 0.10927E+01_JPRB, 0.10950E+01_JPRB, 0.11159E+01_JPRB, 0.11369E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.10846E+01_JPRB, 0.10961E+01_JPRB, 0.11171E+01_JPRB, 0.12041E+01_JPRB, 0.13740E+01_JPRB, 0.13453E+01_JPRB ,&
&  0.13329E+01_JPRB, 0.13577E+01_JPRB, 0.14121E+01_JPRB, 0.14396E+01_JPRB, 0.14188E+01_JPRB, 0.13446E+01_JPRB, 0.12752E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.11186E+01_JPRB, 0.95715E+00_JPRB, 0.10915E+01_JPRB, 0.11829E+01_JPRB, 0.12665E+01_JPRB, 0.12003E+01_JPRB ,&
&  0.11426E+01_JPRB, 0.10364E+01_JPRB, 0.10169E+01_JPRB, 0.10765E+01_JPRB, 0.11445E+01_JPRB, 0.11341E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.98075E+00_JPRB, 0.90118E+00_JPRB, 0.88782E+00_JPRB, 0.82972E+00_JPRB, 0.80867E+00_JPRB, 0.72667E+00_JPRB, 0.77682E+00_JPRB ,&
&  0.81422E+00_JPRB, 0.89806E+00_JPRB, 0.76359E+00_JPRB, 0.10128E+01_JPRB, 0.77429E+00_JPRB, 0.74257E+00_JPRB, 0.75653E+00_JPRB ,&
&  0.77761E+00_JPRB, 0.80627E+00_JPRB, 0.81173E+00_JPRB, 0.89198E+00_JPRB, 0.87823E+00_JPRB, 0.86730E+00_JPRB, 0.87126E+00_JPRB ,&
&  0.84087E+00_JPRB, 0.83368E+00_JPRB, 0.83261E+00_JPRB, 0.84178E+00_JPRB, 0.84469E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.78181E+00_JPRB, 0.77742E+00_JPRB, 0.77606E+00_JPRB, 0.79094E+00_JPRB, 0.80386E+00_JPRB, 0.79402E+00_JPRB, 0.80302E+00_JPRB ,&
&  0.78378E+00_JPRB, 0.76850E+00_JPRB, 0.78029E+00_JPRB, 0.78701E+00_JPRB, 0.80479E+00_JPRB, 0.82197E+00_JPRB, 0.83432E+00_JPRB ,&
&  0.83718E+00_JPRB, 0.83574E+00_JPRB, 0.80676E+00_JPRB, 0.82316E+00_JPRB, 0.93011E+00_JPRB, 0.11177E+01_JPRB, 0.14613E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.20308E+01_JPRB, 0.21327E+01_JPRB, 0.21308E+01_JPRB, 0.20358E+01_JPRB, 0.18746E+01_JPRB, 0.23621E+01_JPRB ,&
&  0.22539E+01_JPRB, 0.15049E+01_JPRB, 0.14340E+01_JPRB, 0.13011E+01_JPRB, 0.13032E+01_JPRB, 0.12887E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.12700E+01_JPRB, 0.12993E+01_JPRB, 0.13589E+01_JPRB, 0.14199E+01_JPRB, 0.14891E+01_JPRB, 0.15362E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.15516E+01_JPRB, 0.15973E+01_JPRB, 0.15564E+01_JPRB, 0.15266E+01_JPRB, 0.15765E+01_JPRB, 0.16228E+01_JPRB ,&
&  0.16508E+01_JPRB, 0.17106E+01_JPRB, 0.15494E+01_JPRB, 0.15258E+01_JPRB, 0.15641E+01_JPRB, 0.14180E+01_JPRB, 0.13930E+01_JPRB ,&
&  0.14010E+01_JPRB, 0.12188E+01_JPRB, 0.90616E+00_JPRB, 0.98030E+00_JPRB, 0.10944E+01_JPRB, 0.12173E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.13011E+01_JPRB, 0.12343E+01_JPRB, 0.12333E+01_JPRB, 0.13054E+01_JPRB, 0.13481E+01_JPRB, 0.13414E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.12932E+01_JPRB, 0.11374E+01_JPRB, 0.10609E+01_JPRB, 0.95830E+00_JPRB, 0.89167E+00_JPRB, 0.86507E+00_JPRB, 0.83161E+00_JPRB ,&
&  0.87803E+00_JPRB, 0.95142E+00_JPRB, 0.11228E+01_JPRB, 0.10703E+01_JPRB, 0.79646E+00_JPRB, 0.81688E+00_JPRB, 0.80696E+00_JPRB ,&
&  0.80966E+00_JPRB, 0.81401E+00_JPRB, 0.82928E+00_JPRB, 0.82158E+00_JPRB, 0.80349E+00_JPRB, 0.78577E+00_JPRB, 0.76977E+00_JPRB ,&
&  0.78305E+00_JPRB, 0.80013E+00_JPRB, 0.80716E+00_JPRB, 0.79469E+00_JPRB, 0.78397E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.77866E+00_JPRB, 0.77931E+00_JPRB, 0.77403E+00_JPRB, 0.78679E+00_JPRB, 0.78116E+00_JPRB, 0.79729E+00_JPRB, 0.78440E+00_JPRB ,&
&  0.77217E+00_JPRB, 0.77348E+00_JPRB, 0.78328E+00_JPRB, 0.79410E+00_JPRB, 0.82428E+00_JPRB, 0.88305E+00_JPRB, 0.95779E+00_JPRB ,&
&  0.10384E+01_JPRB, 0.10739E+01_JPRB, 0.11409E+01_JPRB, 0.12871E+01_JPRB, 0.15800E+01_JPRB, 0.18789E+01_JPRB, 0.19960E+01_JPRB ,&
&  0.21280E+01_JPRB, 0.22918E+01_JPRB, 0.24007E+01_JPRB, 0.24678E+01_JPRB, 0.24506E+01_JPRB, 0.24158E+01_JPRB, 0.24141E+01_JPRB ,&
&  0.20351E+01_JPRB, 0.15975E+01_JPRB, 0.14347E+01_JPRB, 0.13652E+01_JPRB, 0.13617E+01_JPRB, 0.13564E+01_JPRB, 0.13473E+01_JPRB ,&
&  0.13442E+01_JPRB, 0.13588E+01_JPRB, 0.13685E+01_JPRB, 0.14023E+01_JPRB, 0.14638E+01_JPRB, 0.15418E+01_JPRB, 0.15801E+01_JPRB ,&
&  0.15798E+01_JPRB, 0.15827E+01_JPRB, 0.15931E+01_JPRB, 0.14873E+01_JPRB, 0.15724E+01_JPRB, 0.16059E+01_JPRB, 0.16102E+01_JPRB ,&
&  0.15826E+01_JPRB, 0.15231E+01_JPRB, 0.15522E+01_JPRB, 0.15305E+01_JPRB, 0.14995E+01_JPRB, 0.14121E+01_JPRB, 0.13610E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.12295E+01_JPRB, 0.96436E+00_JPRB, 0.10721E+01_JPRB, 0.12041E+01_JPRB, 0.11591E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.14414E+01_JPRB, 0.13954E+01_JPRB, 0.14025E+01_JPRB, 0.14146E+01_JPRB, 0.14803E+01_JPRB, 0.14860E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.15606E+01_JPRB, 0.14277E+01_JPRB, 0.13111E+01_JPRB, 0.12007E+01_JPRB, 0.10882E+01_JPRB, 0.99175E+00_JPRB, 0.94425E+00_JPRB ,&
&  0.93968E+00_JPRB, 0.96491E+00_JPRB, 0.93472E+00_JPRB, 0.91698E+00_JPRB, 0.82274E+00_JPRB, 0.82392E+00_JPRB, 0.82052E+00_JPRB ,&
&  0.84386E+00_JPRB, 0.83983E+00_JPRB, 0.82826E+00_JPRB, 0.80798E+00_JPRB, 0.79669E+00_JPRB, 0.78886E+00_JPRB, 0.78784E+00_JPRB ,&
&  0.79646E+00_JPRB, 0.80755E+00_JPRB, 0.80189E+00_JPRB, 0.79511E+00_JPRB, 0.78612E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.77256E+00_JPRB, 0.76453E+00_JPRB, 0.75837E+00_JPRB, 0.78860E+00_JPRB, 0.78536E+00_JPRB, 0.77631E+00_JPRB, 0.78850E+00_JPRB ,&
&  0.80650E+00_JPRB, 0.83182E+00_JPRB, 0.86422E+00_JPRB, 0.89741E+00_JPRB, 0.94104E+00_JPRB, 0.10052E+01_JPRB, 0.10740E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.12580E+01_JPRB, 0.14266E+01_JPRB, 0.16501E+01_JPRB, 0.19264E+01_JPRB, 0.20917E+01_JPRB, 0.21394E+01_JPRB ,&
&  0.20465E+01_JPRB, 0.21005E+01_JPRB, 0.21706E+01_JPRB, 0.21509E+01_JPRB, 0.18349E+01_JPRB, 0.19484E+01_JPRB, 0.19741E+01_JPRB ,&
&  0.18849E+01_JPRB, 0.16055E+01_JPRB, 0.13761E+01_JPRB, 0.13178E+01_JPRB, 0.12897E+01_JPRB, 0.13085E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.13177E+01_JPRB, 0.13330E+01_JPRB, 0.13422E+01_JPRB, 0.13593E+01_JPRB, 0.14003E+01_JPRB, 0.14726E+01_JPRB, 0.15027E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.15495E+01_JPRB, 0.15333E+01_JPRB, 0.14780E+01_JPRB, 0.14895E+01_JPRB, 0.14935E+01_JPRB, 0.14908E+01_JPRB ,&
&  0.15159E+01_JPRB, 0.14467E+01_JPRB, 0.14612E+01_JPRB, 0.14738E+01_JPRB, 0.14004E+01_JPRB, 0.12778E+01_JPRB, 0.12653E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.13603E+01_JPRB, 0.10430E+01_JPRB, 0.13739E+01_JPRB, 0.16044E+01_JPRB, 0.14509E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.14429E+01_JPRB, 0.14758E+01_JPRB, 0.15080E+01_JPRB, 0.15220E+01_JPRB, 0.15345E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.16403E+01_JPRB, 0.15549E+01_JPRB, 0.14681E+01_JPRB, 0.14098E+01_JPRB, 0.13335E+01_JPRB, 0.11993E+01_JPRB, 0.10741E+01_JPRB ,&
&  0.96069E+00_JPRB, 0.12027E+01_JPRB, 0.11698E+01_JPRB, 0.12547E+01_JPRB, 0.12391E+01_JPRB, 0.85088E+00_JPRB, 0.87734E+00_JPRB ,&
&  0.87023E+00_JPRB, 0.86482E+00_JPRB, 0.84846E+00_JPRB, 0.82473E+00_JPRB, 0.82409E+00_JPRB, 0.82753E+00_JPRB, 0.83003E+00_JPRB ,&
&  0.82642E+00_JPRB, 0.81453E+00_JPRB, 0.80826E+00_JPRB, 0.79081E+00_JPRB, 0.78193E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.80440E+00_JPRB, 0.79631E+00_JPRB, 0.78730E+00_JPRB, 0.79928E+00_JPRB, 0.81003E+00_JPRB, 0.81110E+00_JPRB, 0.80654E+00_JPRB ,&
&  0.82014E+00_JPRB, 0.85613E+00_JPRB, 0.89734E+00_JPRB, 0.92209E+00_JPRB, 0.93246E+00_JPRB, 0.94786E+00_JPRB, 0.10038E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.12356E+01_JPRB, 0.14180E+01_JPRB, 0.15783E+01_JPRB, 0.17375E+01_JPRB, 0.18760E+01_JPRB, 0.20574E+01_JPRB ,&
&  0.21000E+01_JPRB, 0.21513E+01_JPRB, 0.20931E+01_JPRB, 0.16508E+01_JPRB, 0.15929E+01_JPRB, 0.19656E+01_JPRB, 0.17274E+01_JPRB ,&
&  0.14902E+01_JPRB, 0.13881E+01_JPRB, 0.13174E+01_JPRB, 0.12590E+01_JPRB, 0.12223E+01_JPRB, 0.12440E+01_JPRB, 0.12520E+01_JPRB ,&
&  0.12391E+01_JPRB, 0.12500E+01_JPRB, 0.12661E+01_JPRB, 0.12860E+01_JPRB, 0.13087E+01_JPRB, 0.13394E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.13820E+01_JPRB, 0.14038E+01_JPRB, 0.14304E+01_JPRB, 0.14386E+01_JPRB, 0.14151E+01_JPRB, 0.14339E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.14252E+01_JPRB, 0.14062E+01_JPRB, 0.14296E+01_JPRB, 0.14214E+01_JPRB, 0.12445E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.13163E+01_JPRB, 0.13506E+01_JPRB, 0.12424E+01_JPRB, 0.15039E+01_JPRB, 0.10023E+01_JPRB, 0.12135E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.14951E+01_JPRB, 0.14991E+01_JPRB, 0.15082E+01_JPRB, 0.15276E+01_JPRB, 0.15365E+01_JPRB, 0.14353E+01_JPRB, 0.15086E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.15641E+01_JPRB, 0.15439E+01_JPRB, 0.15414E+01_JPRB, 0.15182E+01_JPRB, 0.14365E+01_JPRB, 0.13332E+01_JPRB ,&
&  0.11453E+01_JPRB, 0.12728E+01_JPRB, 0.11651E+01_JPRB, 0.11068E+01_JPRB, 0.96728E+00_JPRB, 0.83868E+00_JPRB, 0.85718E+00_JPRB ,&
&  0.87302E+00_JPRB, 0.85915E+00_JPRB, 0.83589E+00_JPRB, 0.83905E+00_JPRB, 0.84060E+00_JPRB, 0.84555E+00_JPRB, 0.83510E+00_JPRB ,&
&  0.83675E+00_JPRB, 0.83842E+00_JPRB, 0.83712E+00_JPRB, 0.82671E+00_JPRB, 0.81549E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.81059E+00_JPRB, 0.80729E+00_JPRB, 0.80819E+00_JPRB, 0.81876E+00_JPRB, 0.84338E+00_JPRB, 0.86246E+00_JPRB, 0.82472E+00_JPRB ,&
&  0.81557E+00_JPRB, 0.81937E+00_JPRB, 0.85635E+00_JPRB, 0.87337E+00_JPRB, 0.88463E+00_JPRB, 0.89619E+00_JPRB, 0.94597E+00_JPRB ,&
&  0.10388E+01_JPRB, 0.11769E+01_JPRB, 0.13255E+01_JPRB, 0.14378E+01_JPRB, 0.15613E+01_JPRB, 0.17252E+01_JPRB, 0.19018E+01_JPRB ,&
&  0.17013E+01_JPRB, 0.14443E+01_JPRB, 0.20117E+01_JPRB, 0.22537E+01_JPRB, 0.22598E+01_JPRB, 0.22737E+01_JPRB, 0.23028E+01_JPRB ,&
&  0.16808E+01_JPRB, 0.14097E+01_JPRB, 0.14118E+01_JPRB, 0.13087E+01_JPRB, 0.11627E+01_JPRB, 0.11616E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.11726E+01_JPRB, 0.11669E+01_JPRB, 0.11834E+01_JPRB, 0.12074E+01_JPRB, 0.12305E+01_JPRB, 0.12267E+01_JPRB, 0.12112E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.12367E+01_JPRB, 0.12595E+01_JPRB, 0.12598E+01_JPRB, 0.12829E+01_JPRB, 0.13175E+01_JPRB, 0.13373E+01_JPRB ,&
&  0.13549E+01_JPRB, 0.13440E+01_JPRB, 0.13891E+01_JPRB, 0.14635E+01_JPRB, 0.15234E+01_JPRB, 0.13667E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.13851E+01_JPRB, 0.13720E+01_JPRB, 0.13412E+01_JPRB, 0.15025E+01_JPRB, 0.10942E+01_JPRB, 0.12387E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.15599E+01_JPRB, 0.15548E+01_JPRB, 0.15772E+01_JPRB, 0.16446E+01_JPRB, 0.16953E+01_JPRB, 0.15581E+01_JPRB, 0.15583E+01_JPRB ,&
&  0.16071E+01_JPRB, 0.16542E+01_JPRB, 0.16564E+01_JPRB, 0.16601E+01_JPRB, 0.15726E+01_JPRB, 0.16079E+01_JPRB, 0.16504E+01_JPRB ,&
&  0.17027E+01_JPRB, 0.14749E+01_JPRB, 0.13093E+01_JPRB, 0.11453E+01_JPRB, 0.97123E+00_JPRB, 0.89442E+00_JPRB, 0.87369E+00_JPRB ,&
&  0.86450E+00_JPRB, 0.87383E+00_JPRB, 0.83254E+00_JPRB, 0.81126E+00_JPRB, 0.81525E+00_JPRB, 0.84088E+00_JPRB, 0.83062E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.82960E+00_JPRB, 0.83477E+00_JPRB, 0.83142E+00_JPRB, 0.82453E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.82603E+00_JPRB, 0.83824E+00_JPRB, 0.86775E+00_JPRB, 0.90157E+00_JPRB, 0.93353E+00_JPRB, 0.98404E+00_JPRB, 0.95114E+00_JPRB ,&
&  0.92050E+00_JPRB, 0.76203E+00_JPRB, 0.78477E+00_JPRB, 0.80107E+00_JPRB, 0.81895E+00_JPRB, 0.81814E+00_JPRB, 0.84189E+00_JPRB ,&
&  0.93012E+00_JPRB, 0.10660E+01_JPRB, 0.12012E+01_JPRB, 0.13239E+01_JPRB, 0.14523E+01_JPRB, 0.16217E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.12258E+01_JPRB, 0.14320E+01_JPRB, 0.22015E+01_JPRB, 0.22226E+01_JPRB, 0.21286E+01_JPRB, 0.20583E+01_JPRB, 0.18389E+01_JPRB ,&
&  0.15269E+01_JPRB, 0.13635E+01_JPRB, 0.14164E+01_JPRB, 0.12727E+01_JPRB, 0.11124E+01_JPRB, 0.10986E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.11049E+01_JPRB, 0.11033E+01_JPRB, 0.11061E+01_JPRB, 0.11230E+01_JPRB, 0.11392E+01_JPRB, 0.11357E+01_JPRB, 0.11270E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11475E+01_JPRB, 0.11535E+01_JPRB, 0.11459E+01_JPRB, 0.11885E+01_JPRB, 0.12326E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.13274E+01_JPRB, 0.13688E+01_JPRB, 0.13831E+01_JPRB, 0.13280E+01_JPRB, 0.14690E+01_JPRB, 0.14353E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.14387E+01_JPRB, 0.15412E+01_JPRB, 0.12128E+01_JPRB, 0.12735E+01_JPRB, 0.13589E+01_JPRB, 0.14622E+01_JPRB ,&
&  0.14891E+01_JPRB, 0.15892E+01_JPRB, 0.16272E+01_JPRB, 0.17156E+01_JPRB, 0.18225E+01_JPRB, 0.17431E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.16845E+01_JPRB, 0.18636E+01_JPRB, 0.17993E+01_JPRB, 0.17091E+01_JPRB, 0.15450E+01_JPRB, 0.15503E+01_JPRB, 0.17062E+01_JPRB ,&
&  0.18669E+01_JPRB, 0.17361E+01_JPRB, 0.16198E+01_JPRB, 0.14418E+01_JPRB, 0.12201E+01_JPRB, 0.10935E+01_JPRB, 0.95430E+00_JPRB ,&
&  0.89223E+00_JPRB, 0.81611E+00_JPRB, 0.78851E+00_JPRB, 0.79904E+00_JPRB, 0.80034E+00_JPRB, 0.82665E+00_JPRB, 0.82573E+00_JPRB ,&
&  0.81446E+00_JPRB, 0.82050E+00_JPRB, 0.82629E+00_JPRB, 0.82700E+00_JPRB, 0.82494E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.79984E+00_JPRB, 0.80795E+00_JPRB, 0.83143E+00_JPRB, 0.84243E+00_JPRB, 0.83081E+00_JPRB, 0.83522E+00_JPRB, 0.86254E+00_JPRB ,&
&  0.86490E+00_JPRB, 0.72934E+00_JPRB, 0.71386E+00_JPRB, 0.72373E+00_JPRB, 0.74774E+00_JPRB, 0.76114E+00_JPRB, 0.77051E+00_JPRB ,&
&  0.80926E+00_JPRB, 0.92034E+00_JPRB, 0.10984E+01_JPRB, 0.11983E+01_JPRB, 0.12830E+01_JPRB, 0.14505E+01_JPRB, 0.13679E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.18908E+01_JPRB, 0.18645E+01_JPRB, 0.21222E+01_JPRB, 0.21071E+01_JPRB, 0.20469E+01_JPRB, 0.17065E+01_JPRB ,&
&  0.13669E+01_JPRB, 0.12410E+01_JPRB, 0.12351E+01_JPRB, 0.11719E+01_JPRB, 0.11182E+01_JPRB, 0.10569E+01_JPRB, 0.10507E+01_JPRB ,&
&  0.10456E+01_JPRB, 0.10361E+01_JPRB, 0.10290E+01_JPRB, 0.10450E+01_JPRB, 0.10571E+01_JPRB, 0.10580E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10826E+01_JPRB, 0.11004E+01_JPRB, 0.10980E+01_JPRB, 0.11249E+01_JPRB, 0.11542E+01_JPRB, 0.12069E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.12185E+01_JPRB, 0.11935E+01_JPRB, 0.13035E+01_JPRB, 0.13618E+01_JPRB, 0.13904E+01_JPRB, 0.14926E+01_JPRB, 0.14303E+01_JPRB ,&
&  0.15248E+01_JPRB, 0.14478E+01_JPRB, 0.15663E+01_JPRB, 0.12365E+01_JPRB, 0.12636E+01_JPRB, 0.13941E+01_JPRB, 0.14544E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.15690E+01_JPRB, 0.16408E+01_JPRB, 0.17754E+01_JPRB, 0.19009E+01_JPRB, 0.20346E+01_JPRB, 0.19151E+01_JPRB ,&
&  0.19199E+01_JPRB, 0.20148E+01_JPRB, 0.20681E+01_JPRB, 0.17871E+01_JPRB, 0.17253E+01_JPRB, 0.16340E+01_JPRB, 0.18802E+01_JPRB ,&
&  0.20417E+01_JPRB, 0.19123E+01_JPRB, 0.18480E+01_JPRB, 0.17541E+01_JPRB, 0.15066E+01_JPRB, 0.14141E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.10704E+01_JPRB, 0.95183E+00_JPRB, 0.83752E+00_JPRB, 0.81633E+00_JPRB, 0.81931E+00_JPRB, 0.81681E+00_JPRB, 0.83086E+00_JPRB ,&
&  0.83051E+00_JPRB, 0.82527E+00_JPRB, 0.82167E+00_JPRB, 0.81366E+00_JPRB, 0.80052E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.78909E+00_JPRB, 0.78786E+00_JPRB, 0.79831E+00_JPRB, 0.80147E+00_JPRB, 0.79260E+00_JPRB, 0.76821E+00_JPRB, 0.78465E+00_JPRB ,&
&  0.81244E+00_JPRB, 0.71858E+00_JPRB, 0.67035E+00_JPRB, 0.65410E+00_JPRB, 0.66781E+00_JPRB, 0.69976E+00_JPRB, 0.72270E+00_JPRB ,&
&  0.74624E+00_JPRB, 0.79649E+00_JPRB, 0.96451E+00_JPRB, 0.11022E+01_JPRB, 0.12087E+01_JPRB, 0.14354E+01_JPRB, 0.10406E+01_JPRB ,&
&  0.11380E+01_JPRB, 0.16294E+01_JPRB, 0.17830E+01_JPRB, 0.20023E+01_JPRB, 0.21393E+01_JPRB, 0.21182E+01_JPRB, 0.18047E+01_JPRB ,&
&  0.14539E+01_JPRB, 0.12535E+01_JPRB, 0.11479E+01_JPRB, 0.10826E+01_JPRB, 0.10515E+01_JPRB, 0.10279E+01_JPRB, 0.98815E+00_JPRB ,&
&  0.97987E+00_JPRB, 0.97636E+00_JPRB, 0.96268E+00_JPRB, 0.97463E+00_JPRB, 0.98551E+00_JPRB, 0.98519E+00_JPRB, 0.10103E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10795E+01_JPRB, 0.11086E+01_JPRB, 0.10877E+01_JPRB, 0.11459E+01_JPRB, 0.11906E+01_JPRB, 0.12377E+01_JPRB ,&
&  0.11844E+01_JPRB, 0.11422E+01_JPRB, 0.13278E+01_JPRB, 0.15030E+01_JPRB, 0.15852E+01_JPRB, 0.15239E+01_JPRB, 0.14781E+01_JPRB ,&
&  0.15339E+01_JPRB, 0.15434E+01_JPRB, 0.14750E+01_JPRB, 0.12720E+01_JPRB, 0.13203E+01_JPRB, 0.14730E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.15930E+01_JPRB, 0.16312E+01_JPRB, 0.16749E+01_JPRB, 0.17114E+01_JPRB, 0.18148E+01_JPRB, 0.20819E+01_JPRB, 0.21195E+01_JPRB ,&
&  0.24091E+01_JPRB, 0.28050E+01_JPRB, 0.24772E+01_JPRB, 0.20694E+01_JPRB, 0.18894E+01_JPRB, 0.18035E+01_JPRB, 0.18877E+01_JPRB ,&
&  0.21282E+01_JPRB, 0.19906E+01_JPRB, 0.17502E+01_JPRB, 0.17526E+01_JPRB, 0.16477E+01_JPRB, 0.17189E+01_JPRB, 0.16682E+01_JPRB ,&
&  0.14282E+01_JPRB, 0.12591E+01_JPRB, 0.10748E+01_JPRB, 0.96532E+00_JPRB, 0.90628E+00_JPRB, 0.88940E+00_JPRB, 0.87927E+00_JPRB ,&
&  0.87677E+00_JPRB, 0.86790E+00_JPRB, 0.85154E+00_JPRB, 0.83358E+00_JPRB, 0.80820E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.81804E+00_JPRB, 0.79252E+00_JPRB, 0.77894E+00_JPRB, 0.77040E+00_JPRB, 0.77203E+00_JPRB, 0.75237E+00_JPRB, 0.73371E+00_JPRB ,&
&  0.76876E+00_JPRB, 0.69122E+00_JPRB, 0.65633E+00_JPRB, 0.63234E+00_JPRB, 0.63963E+00_JPRB, 0.66708E+00_JPRB, 0.69804E+00_JPRB ,&
&  0.72358E+00_JPRB, 0.75897E+00_JPRB, 0.86317E+00_JPRB, 0.10180E+01_JPRB, 0.10598E+01_JPRB, 0.98756E+00_JPRB, 0.10268E+01_JPRB ,&
&  0.12956E+01_JPRB, 0.16152E+01_JPRB, 0.17850E+01_JPRB, 0.19574E+01_JPRB, 0.21933E+01_JPRB, 0.23301E+01_JPRB, 0.20187E+01_JPRB ,&
&  0.17316E+01_JPRB, 0.14454E+01_JPRB, 0.12154E+01_JPRB, 0.10842E+01_JPRB, 0.10339E+01_JPRB, 0.99243E+00_JPRB, 0.96063E+00_JPRB ,&
&  0.92391E+00_JPRB, 0.90672E+00_JPRB, 0.90828E+00_JPRB, 0.91648E+00_JPRB, 0.93464E+00_JPRB, 0.92999E+00_JPRB, 0.94141E+00_JPRB ,&
&  0.97995E+00_JPRB, 0.10224E+01_JPRB, 0.10757E+01_JPRB, 0.10679E+01_JPRB, 0.11206E+01_JPRB, 0.11721E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12980E+01_JPRB, 0.13842E+01_JPRB, 0.14709E+01_JPRB, 0.15206E+01_JPRB, 0.16156E+01_JPRB, 0.16441E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.15729E+01_JPRB, 0.15120E+01_JPRB, 0.13395E+01_JPRB, 0.13605E+01_JPRB, 0.14579E+01_JPRB, 0.16641E+01_JPRB, 0.14559E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.11981E+01_JPRB, 0.13180E+01_JPRB, 0.15527E+01_JPRB, 0.16999E+01_JPRB, 0.19682E+01_JPRB, 0.24076E+01_JPRB ,&
&  0.26005E+01_JPRB, 0.15550E+01_JPRB, 0.12813E+01_JPRB, 0.15506E+01_JPRB, 0.13035E+01_JPRB, 0.12393E+01_JPRB, 0.17366E+01_JPRB ,&
&  0.20222E+01_JPRB, 0.22024E+01_JPRB, 0.20914E+01_JPRB, 0.16668E+01_JPRB, 0.16342E+01_JPRB, 0.18199E+01_JPRB, 0.19729E+01_JPRB ,&
&  0.17501E+01_JPRB, 0.16144E+01_JPRB, 0.14475E+01_JPRB, 0.12839E+01_JPRB, 0.11398E+01_JPRB, 0.10369E+01_JPRB, 0.97579E+00_JPRB ,&
&  0.94542E+00_JPRB, 0.92841E+00_JPRB, 0.91276E+00_JPRB, 0.88525E+00_JPRB, 0.85112E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.86098E+00_JPRB, 0.82609E+00_JPRB, 0.79944E+00_JPRB, 0.77433E+00_JPRB, 0.76127E+00_JPRB, 0.74055E+00_JPRB, 0.70839E+00_JPRB ,&
&  0.73325E+00_JPRB, 0.69087E+00_JPRB, 0.65090E+00_JPRB, 0.62862E+00_JPRB, 0.62888E+00_JPRB, 0.64630E+00_JPRB, 0.68247E+00_JPRB ,&
&  0.73363E+00_JPRB, 0.76571E+00_JPRB, 0.86012E+00_JPRB, 0.97585E+00_JPRB, 0.97728E+00_JPRB, 0.87365E+00_JPRB, 0.99840E+00_JPRB ,&
&  0.11033E+01_JPRB, 0.14141E+01_JPRB, 0.19135E+01_JPRB, 0.21649E+01_JPRB, 0.22168E+01_JPRB, 0.23248E+01_JPRB, 0.21547E+01_JPRB ,&
&  0.19953E+01_JPRB, 0.17459E+01_JPRB, 0.14343E+01_JPRB, 0.12161E+01_JPRB, 0.10704E+01_JPRB, 0.10116E+01_JPRB, 0.95705E+00_JPRB ,&
&  0.93991E+00_JPRB, 0.90542E+00_JPRB, 0.87428E+00_JPRB, 0.87374E+00_JPRB, 0.88541E+00_JPRB, 0.89014E+00_JPRB, 0.88497E+00_JPRB ,&
&  0.90384E+00_JPRB, 0.94375E+00_JPRB, 0.10195E+01_JPRB, 0.10841E+01_JPRB, 0.93597E+00_JPRB, 0.11350E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.13601E+01_JPRB, 0.14735E+01_JPRB, 0.14547E+01_JPRB, 0.16573E+01_JPRB, 0.17312E+01_JPRB, 0.16485E+01_JPRB, 0.16348E+01_JPRB ,&
&  0.17086E+01_JPRB, 0.15864E+01_JPRB, 0.14423E+01_JPRB, 0.15555E+01_JPRB, 0.16170E+01_JPRB, 0.19181E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.10833E+01_JPRB, 0.13214E+01_JPRB, 0.13058E+01_JPRB, 0.12709E+01_JPRB, 0.14761E+01_JPRB, 0.17959E+01_JPRB, 0.16000E+01_JPRB ,&
&  0.76491E+00_JPRB, 0.64013E+00_JPRB, 0.69590E+00_JPRB, 0.70426E+00_JPRB, 0.67692E+00_JPRB, 0.89820E+00_JPRB, 0.20134E+01_JPRB ,&
&  0.19419E+01_JPRB, 0.22535E+01_JPRB, 0.22258E+01_JPRB, 0.19497E+01_JPRB, 0.17129E+01_JPRB, 0.17303E+01_JPRB, 0.21004E+01_JPRB ,&
&  0.20115E+01_JPRB, 0.18370E+01_JPRB, 0.16683E+01_JPRB, 0.15474E+01_JPRB, 0.13946E+01_JPRB, 0.12424E+01_JPRB, 0.11152E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.97861E+00_JPRB, 0.94960E+00_JPRB, 0.92442E+00_JPRB, 0.89449E+00_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.87383E+00_JPRB, 0.83708E+00_JPRB, 0.81115E+00_JPRB, 0.78985E+00_JPRB, 0.76537E+00_JPRB, 0.74735E+00_JPRB, 0.70946E+00_JPRB ,&
&  0.70439E+00_JPRB, 0.70054E+00_JPRB, 0.66298E+00_JPRB, 0.63778E+00_JPRB, 0.62770E+00_JPRB, 0.64461E+00_JPRB, 0.69295E+00_JPRB ,&
&  0.73539E+00_JPRB, 0.79473E+00_JPRB, 0.94642E+00_JPRB, 0.90381E+00_JPRB, 0.93183E+00_JPRB, 0.83067E+00_JPRB, 0.88481E+00_JPRB ,&
&  0.98251E+00_JPRB, 0.11951E+01_JPRB, 0.15994E+01_JPRB, 0.19067E+01_JPRB, 0.22563E+01_JPRB, 0.23878E+01_JPRB, 0.23518E+01_JPRB ,&
&  0.22000E+01_JPRB, 0.20487E+01_JPRB, 0.17889E+01_JPRB, 0.14861E+01_JPRB, 0.12649E+01_JPRB, 0.11160E+01_JPRB, 0.10229E+01_JPRB ,&
&  0.98202E+00_JPRB, 0.97502E+00_JPRB, 0.91411E+00_JPRB, 0.87135E+00_JPRB, 0.86227E+00_JPRB, 0.85491E+00_JPRB, 0.83250E+00_JPRB ,&
&  0.83980E+00_JPRB, 0.88040E+00_JPRB, 0.95139E+00_JPRB, 0.10252E+01_JPRB, 0.95462E+00_JPRB, 0.89198E+00_JPRB, 0.11074E+01_JPRB ,&
&  0.13400E+01_JPRB, 0.16362E+01_JPRB, 0.17257E+01_JPRB, 0.20361E+01_JPRB, 0.19229E+01_JPRB, 0.17589E+01_JPRB, 0.18113E+01_JPRB ,&
&  0.18335E+01_JPRB, 0.20421E+01_JPRB, 0.16173E+01_JPRB, 0.16954E+01_JPRB, 0.18900E+01_JPRB, 0.15235E+01_JPRB, 0.13684E+01_JPRB ,&
&  0.15466E+01_JPRB, 0.13939E+01_JPRB, 0.12747E+01_JPRB, 0.11100E+01_JPRB, 0.13838E+01_JPRB, 0.13988E+01_JPRB, 0.76724E+00_JPRB ,&
&  0.63460E+00_JPRB, 0.60878E+00_JPRB, 0.58599E+00_JPRB, 0.55995E+00_JPRB, 0.58255E+00_JPRB, 0.77077E+00_JPRB, 0.12294E+01_JPRB ,&
&  0.15108E+01_JPRB, 0.19518E+01_JPRB, 0.21425E+01_JPRB, 0.21544E+01_JPRB, 0.18592E+01_JPRB, 0.18832E+01_JPRB, 0.19163E+01_JPRB ,&
&  0.21566E+01_JPRB, 0.21457E+01_JPRB, 0.19337E+01_JPRB, 0.17599E+01_JPRB, 0.15874E+01_JPRB, 0.14240E+01_JPRB, 0.12872E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.10944E+01_JPRB, 0.10283E+01_JPRB, 0.97367E+00_JPRB, 0.91869E+00_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.93225E+00_JPRB, 0.88386E+00_JPRB, 0.83333E+00_JPRB, 0.79828E+00_JPRB, 0.77741E+00_JPRB, 0.75104E+00_JPRB, 0.71802E+00_JPRB ,&
&  0.68578E+00_JPRB, 0.70032E+00_JPRB, 0.67592E+00_JPRB, 0.65647E+00_JPRB, 0.64215E+00_JPRB, 0.65115E+00_JPRB, 0.69295E+00_JPRB ,&
&  0.74274E+00_JPRB, 0.83356E+00_JPRB, 0.89580E+00_JPRB, 0.81210E+00_JPRB, 0.77512E+00_JPRB, 0.73337E+00_JPRB, 0.73205E+00_JPRB ,&
&  0.87622E+00_JPRB, 0.10239E+01_JPRB, 0.12745E+01_JPRB, 0.16939E+01_JPRB, 0.21793E+01_JPRB, 0.23414E+01_JPRB, 0.25761E+01_JPRB ,&
&  0.24393E+01_JPRB, 0.22234E+01_JPRB, 0.20580E+01_JPRB, 0.18238E+01_JPRB, 0.15281E+01_JPRB, 0.13259E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.10921E+01_JPRB, 0.10416E+01_JPRB, 0.10057E+01_JPRB, 0.92559E+00_JPRB, 0.86313E+00_JPRB, 0.84354E+00_JPRB, 0.81732E+00_JPRB ,&
&  0.81408E+00_JPRB, 0.85391E+00_JPRB, 0.91787E+00_JPRB, 0.10072E+01_JPRB, 0.11144E+01_JPRB, 0.11421E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.12304E+01_JPRB, 0.14303E+01_JPRB, 0.19734E+01_JPRB, 0.21164E+01_JPRB, 0.20918E+01_JPRB, 0.20631E+01_JPRB, 0.19973E+01_JPRB ,&
&  0.20170E+01_JPRB, 0.22425E+01_JPRB, 0.19194E+01_JPRB, 0.20725E+01_JPRB, 0.19005E+01_JPRB, 0.14718E+01_JPRB, 0.16927E+01_JPRB ,&
&  0.17855E+01_JPRB, 0.16301E+01_JPRB, 0.13508E+01_JPRB, 0.10506E+01_JPRB, 0.10191E+01_JPRB, 0.81939E+00_JPRB, 0.59251E+00_JPRB ,&
&  0.55839E+00_JPRB, 0.54002E+00_JPRB, 0.52201E+00_JPRB, 0.53670E+00_JPRB, 0.54734E+00_JPRB, 0.69433E+00_JPRB, 0.97358E+00_JPRB ,&
&  0.13869E+01_JPRB, 0.16981E+01_JPRB, 0.19610E+01_JPRB, 0.20372E+01_JPRB, 0.18893E+01_JPRB, 0.19161E+01_JPRB, 0.22521E+01_JPRB ,&
&  0.21501E+01_JPRB, 0.21212E+01_JPRB, 0.20703E+01_JPRB, 0.19067E+01_JPRB, 0.17219E+01_JPRB, 0.15554E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.11841E+01_JPRB, 0.11159E+01_JPRB, 0.10489E+01_JPRB, 0.98668E+00_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.96359E+00_JPRB, 0.91454E+00_JPRB, 0.86981E+00_JPRB, 0.83907E+00_JPRB, 0.81211E+00_JPRB, 0.78121E+00_JPRB, 0.74346E+00_JPRB ,&
&  0.69544E+00_JPRB, 0.68826E+00_JPRB, 0.68241E+00_JPRB, 0.65298E+00_JPRB, 0.64215E+00_JPRB, 0.65245E+00_JPRB, 0.68786E+00_JPRB ,&
&  0.74321E+00_JPRB, 0.90390E+00_JPRB, 0.77210E+00_JPRB, 0.70791E+00_JPRB, 0.73081E+00_JPRB, 0.72985E+00_JPRB, 0.67558E+00_JPRB ,&
&  0.80654E+00_JPRB, 0.93831E+00_JPRB, 0.11455E+01_JPRB, 0.16269E+01_JPRB, 0.24299E+01_JPRB, 0.25677E+01_JPRB, 0.27273E+01_JPRB ,&
&  0.28324E+01_JPRB, 0.25835E+01_JPRB, 0.23618E+01_JPRB, 0.21173E+01_JPRB, 0.17757E+01_JPRB, 0.15129E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.11071E+01_JPRB, 0.10342E+01_JPRB, 0.98039E+00_JPRB, 0.90952E+00_JPRB, 0.86726E+00_JPRB, 0.84925E+00_JPRB ,&
&  0.83718E+00_JPRB, 0.85756E+00_JPRB, 0.90961E+00_JPRB, 0.10067E+01_JPRB, 0.11212E+01_JPRB, 0.11321E+01_JPRB, 0.14722E+01_JPRB ,&
&  0.16827E+01_JPRB, 0.19195E+01_JPRB, 0.20680E+01_JPRB, 0.24414E+01_JPRB, 0.26051E+01_JPRB, 0.24557E+01_JPRB, 0.26124E+01_JPRB ,&
&  0.21470E+01_JPRB, 0.20700E+01_JPRB, 0.22261E+01_JPRB, 0.23159E+01_JPRB, 0.18169E+01_JPRB, 0.18920E+01_JPRB, 0.27340E+01_JPRB ,&
&  0.19534E+01_JPRB, 0.20643E+01_JPRB, 0.22008E+01_JPRB, 0.22176E+01_JPRB, 0.94361E+00_JPRB, 0.68283E+00_JPRB, 0.10264E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.72132E+00_JPRB, 0.59354E+00_JPRB, 0.70566E+00_JPRB, 0.60986E+00_JPRB, 0.74504E+00_JPRB, 0.10716E+01_JPRB ,&
&  0.11569E+01_JPRB, 0.12314E+01_JPRB, 0.16888E+01_JPRB, 0.18732E+01_JPRB, 0.18999E+01_JPRB, 0.19953E+01_JPRB, 0.21468E+01_JPRB ,&
&  0.22351E+01_JPRB, 0.21909E+01_JPRB, 0.22614E+01_JPRB, 0.20892E+01_JPRB, 0.18601E+01_JPRB, 0.16423E+01_JPRB, 0.14640E+01_JPRB ,&
&  0.13079E+01_JPRB, 0.12006E+01_JPRB, 0.11304E+01_JPRB, 0.10614E+01_JPRB, 0.10071E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.93087E+00_JPRB, 0.88793E+00_JPRB, 0.85052E+00_JPRB, 0.81847E+00_JPRB, 0.79846E+00_JPRB, 0.77879E+00_JPRB, 0.75401E+00_JPRB ,&
&  0.71530E+00_JPRB, 0.68287E+00_JPRB, 0.68483E+00_JPRB, 0.65698E+00_JPRB, 0.63850E+00_JPRB, 0.64247E+00_JPRB, 0.68280E+00_JPRB ,&
&  0.72970E+00_JPRB, 0.77368E+00_JPRB, 0.65792E+00_JPRB, 0.61227E+00_JPRB, 0.70941E+00_JPRB, 0.67341E+00_JPRB, 0.68350E+00_JPRB ,&
&  0.78485E+00_JPRB, 0.88901E+00_JPRB, 0.10745E+01_JPRB, 0.14998E+01_JPRB, 0.19294E+01_JPRB, 0.23887E+01_JPRB, 0.27261E+01_JPRB ,&
&  0.28780E+01_JPRB, 0.28059E+01_JPRB, 0.26224E+01_JPRB, 0.23897E+01_JPRB, 0.20107E+01_JPRB, 0.17006E+01_JPRB, 0.14798E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.11603E+01_JPRB, 0.10559E+01_JPRB, 0.94973E+00_JPRB, 0.89198E+00_JPRB, 0.84338E+00_JPRB, 0.83418E+00_JPRB ,&
&  0.84251E+00_JPRB, 0.86976E+00_JPRB, 0.91847E+00_JPRB, 0.10222E+01_JPRB, 0.10127E+01_JPRB, 0.10611E+01_JPRB, 0.12787E+01_JPRB ,&
&  0.17914E+01_JPRB, 0.20603E+01_JPRB, 0.23337E+01_JPRB, 0.25957E+01_JPRB, 0.29566E+01_JPRB, 0.27530E+01_JPRB, 0.30671E+01_JPRB ,&
&  0.23301E+01_JPRB, 0.19512E+01_JPRB, 0.17313E+01_JPRB, 0.14426E+01_JPRB, 0.18181E+01_JPRB, 0.27918E+01_JPRB, 0.27193E+01_JPRB ,&
&  0.24239E+01_JPRB, 0.23441E+01_JPRB, 0.23051E+01_JPRB, 0.18316E+01_JPRB, 0.14243E+01_JPRB, 0.11034E+01_JPRB, 0.13438E+01_JPRB ,&
&  0.13852E+01_JPRB, 0.13320E+01_JPRB, 0.11709E+01_JPRB, 0.10049E+01_JPRB, 0.87564E+00_JPRB, 0.10717E+01_JPRB, 0.10856E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.10035E+01_JPRB, 0.12300E+01_JPRB, 0.15150E+01_JPRB, 0.17095E+01_JPRB, 0.16963E+01_JPRB, 0.19085E+01_JPRB ,&
&  0.19558E+01_JPRB, 0.20503E+01_JPRB, 0.20355E+01_JPRB, 0.19220E+01_JPRB, 0.17561E+01_JPRB, 0.15690E+01_JPRB, 0.14143E+01_JPRB ,&
&  0.12920E+01_JPRB, 0.11864E+01_JPRB, 0.10959E+01_JPRB, 0.10257E+01_JPRB, 0.97554E+00_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.88234E+00_JPRB, 0.84998E+00_JPRB, 0.81979E+00_JPRB, 0.79809E+00_JPRB, 0.77893E+00_JPRB, 0.76469E+00_JPRB, 0.74881E+00_JPRB ,&
&  0.72338E+00_JPRB, 0.68547E+00_JPRB, 0.68284E+00_JPRB, 0.66968E+00_JPRB, 0.65037E+00_JPRB, 0.64338E+00_JPRB, 0.66352E+00_JPRB ,&
&  0.70596E+00_JPRB, 0.68262E+00_JPRB, 0.56466E+00_JPRB, 0.58341E+00_JPRB, 0.63312E+00_JPRB, 0.60765E+00_JPRB, 0.66046E+00_JPRB ,&
&  0.73933E+00_JPRB, 0.83898E+00_JPRB, 0.94589E+00_JPRB, 0.11869E+01_JPRB, 0.14784E+01_JPRB, 0.18577E+01_JPRB, 0.22290E+01_JPRB ,&
&  0.23366E+01_JPRB, 0.25256E+01_JPRB, 0.26343E+01_JPRB, 0.24609E+01_JPRB, 0.22037E+01_JPRB, 0.19238E+01_JPRB, 0.16891E+01_JPRB ,&
&  0.15164E+01_JPRB, 0.13521E+01_JPRB, 0.11788E+01_JPRB, 0.10416E+01_JPRB, 0.95271E+00_JPRB, 0.90283E+00_JPRB, 0.88415E+00_JPRB ,&
&  0.89003E+00_JPRB, 0.90210E+00_JPRB, 0.92067E+00_JPRB, 0.10199E+01_JPRB, 0.10232E+01_JPRB, 0.11352E+01_JPRB, 0.13251E+01_JPRB ,&
&  0.21153E+01_JPRB, 0.24988E+01_JPRB, 0.26940E+01_JPRB, 0.30553E+01_JPRB, 0.23511E+01_JPRB, 0.24382E+01_JPRB, 0.30492E+01_JPRB ,&
&  0.31277E+01_JPRB, 0.30171E+01_JPRB, 0.29590E+01_JPRB, 0.22840E+01_JPRB, 0.20183E+01_JPRB, 0.27820E+01_JPRB, 0.26224E+01_JPRB ,&
&  0.23329E+01_JPRB, 0.22439E+01_JPRB, 0.21068E+01_JPRB, 0.22865E+01_JPRB, 0.16220E+01_JPRB, 0.13860E+01_JPRB, 0.10755E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.96974E+00_JPRB, 0.11195E+01_JPRB, 0.10590E+01_JPRB, 0.10483E+01_JPRB, 0.10493E+01_JPRB, 0.10173E+01_JPRB ,&
&  0.10343E+01_JPRB, 0.10207E+01_JPRB, 0.10183E+01_JPRB, 0.11784E+01_JPRB, 0.14536E+01_JPRB, 0.14174E+01_JPRB, 0.15501E+01_JPRB ,&
&  0.15825E+01_JPRB, 0.16718E+01_JPRB, 0.16289E+01_JPRB, 0.16337E+01_JPRB, 0.15753E+01_JPRB, 0.14723E+01_JPRB, 0.13361E+01_JPRB ,&
&  0.12316E+01_JPRB, 0.11424E+01_JPRB, 0.10552E+01_JPRB, 0.98355E+00_JPRB, 0.92613E+00_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.79860E+00_JPRB, 0.77251E+00_JPRB, 0.75573E+00_JPRB, 0.74792E+00_JPRB, 0.74433E+00_JPRB, 0.73862E+00_JPRB, 0.72843E+00_JPRB ,&
&  0.70936E+00_JPRB, 0.67564E+00_JPRB, 0.66088E+00_JPRB, 0.65668E+00_JPRB, 0.63815E+00_JPRB, 0.62951E+00_JPRB, 0.64382E+00_JPRB ,&
&  0.68154E+00_JPRB, 0.71518E+00_JPRB, 0.60579E+00_JPRB, 0.53791E+00_JPRB, 0.51507E+00_JPRB, 0.57204E+00_JPRB, 0.64202E+00_JPRB ,&
&  0.74494E+00_JPRB, 0.82957E+00_JPRB, 0.94106E+00_JPRB, 0.10726E+01_JPRB, 0.12528E+01_JPRB, 0.14783E+01_JPRB, 0.18373E+01_JPRB ,&
&  0.23927E+01_JPRB, 0.22085E+01_JPRB, 0.23593E+01_JPRB, 0.23048E+01_JPRB, 0.22341E+01_JPRB, 0.20119E+01_JPRB, 0.17879E+01_JPRB ,&
&  0.15781E+01_JPRB, 0.14228E+01_JPRB, 0.12632E+01_JPRB, 0.11272E+01_JPRB, 0.10223E+01_JPRB, 0.94339E+00_JPRB, 0.89811E+00_JPRB ,&
&  0.90126E+00_JPRB, 0.91457E+00_JPRB, 0.91990E+00_JPRB, 0.96119E+00_JPRB, 0.11119E+01_JPRB, 0.12877E+01_JPRB, 0.15593E+01_JPRB ,&
&  0.17795E+01_JPRB, 0.19580E+01_JPRB, 0.29084E+01_JPRB, 0.28803E+01_JPRB, 0.29793E+01_JPRB, 0.30284E+01_JPRB, 0.30879E+01_JPRB ,&
&  0.32611E+01_JPRB, 0.31146E+01_JPRB, 0.30019E+01_JPRB, 0.26013E+01_JPRB, 0.29672E+01_JPRB, 0.28779E+01_JPRB, 0.26721E+01_JPRB ,&
&  0.24175E+01_JPRB, 0.23456E+01_JPRB, 0.21647E+01_JPRB, 0.21167E+01_JPRB, 0.20942E+01_JPRB, 0.21249E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.14771E+01_JPRB, 0.13132E+01_JPRB, 0.11371E+01_JPRB, 0.98622E+00_JPRB, 0.10405E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11558E+01_JPRB, 0.11750E+01_JPRB, 0.11512E+01_JPRB, 0.12198E+01_JPRB, 0.13828E+01_JPRB, 0.14858E+01_JPRB, 0.14780E+01_JPRB ,&
&  0.13758E+01_JPRB, 0.14093E+01_JPRB, 0.14389E+01_JPRB, 0.14066E+01_JPRB, 0.13847E+01_JPRB, 0.13070E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.10386E+01_JPRB, 0.96839E+00_JPRB, 0.90075E+00_JPRB, 0.83858E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.72167E+00_JPRB, 0.70705E+00_JPRB, 0.69603E+00_JPRB, 0.69293E+00_JPRB, 0.69320E+00_JPRB, 0.69914E+00_JPRB, 0.69786E+00_JPRB ,&
&  0.68858E+00_JPRB, 0.66324E+00_JPRB, 0.64122E+00_JPRB, 0.64213E+00_JPRB, 0.63187E+00_JPRB, 0.62240E+00_JPRB, 0.63134E+00_JPRB ,&
&  0.66743E+00_JPRB, 0.79879E+00_JPRB, 0.68623E+00_JPRB, 0.63859E+00_JPRB, 0.59278E+00_JPRB, 0.63840E+00_JPRB, 0.68915E+00_JPRB ,&
&  0.80403E+00_JPRB, 0.86527E+00_JPRB, 0.90281E+00_JPRB, 0.97487E+00_JPRB, 0.11345E+01_JPRB, 0.13444E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.17317E+01_JPRB, 0.19455E+01_JPRB, 0.19679E+01_JPRB, 0.22203E+01_JPRB, 0.21096E+01_JPRB, 0.18202E+01_JPRB, 0.17840E+01_JPRB ,&
&  0.15878E+01_JPRB, 0.14022E+01_JPRB, 0.12319E+01_JPRB, 0.11147E+01_JPRB, 0.10275E+01_JPRB, 0.95905E+00_JPRB, 0.90458E+00_JPRB ,&
&  0.87367E+00_JPRB, 0.85994E+00_JPRB, 0.84563E+00_JPRB, 0.83798E+00_JPRB, 0.91191E+00_JPRB, 0.11929E+01_JPRB, 0.15552E+01_JPRB ,&
&  0.22560E+01_JPRB, 0.21266E+01_JPRB, 0.18628E+01_JPRB, 0.25280E+01_JPRB, 0.33977E+01_JPRB, 0.33664E+01_JPRB, 0.31808E+01_JPRB ,&
&  0.34751E+01_JPRB, 0.35082E+01_JPRB, 0.34327E+01_JPRB, 0.34198E+01_JPRB, 0.33258E+01_JPRB, 0.31767E+01_JPRB, 0.29689E+01_JPRB ,&
&  0.26145E+01_JPRB, 0.24375E+01_JPRB, 0.23234E+01_JPRB, 0.22779E+01_JPRB, 0.21126E+01_JPRB, 0.20131E+01_JPRB, 0.19770E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.15253E+01_JPRB, 0.10830E+01_JPRB, 0.11401E+01_JPRB, 0.92365E+00_JPRB, 0.95711E+00_JPRB, 0.11521E+01_JPRB ,&
&  0.11689E+01_JPRB, 0.12791E+01_JPRB, 0.13186E+01_JPRB, 0.12590E+01_JPRB, 0.14290E+01_JPRB, 0.14392E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.13728E+01_JPRB, 0.12045E+01_JPRB, 0.12417E+01_JPRB, 0.12359E+01_JPRB, 0.11736E+01_JPRB, 0.10954E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.94846E+00_JPRB, 0.88946E+00_JPRB, 0.84614E+00_JPRB, 0.79351E+00_JPRB, 0.74864E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.65695E+00_JPRB, 0.64378E+00_JPRB, 0.63163E+00_JPRB, 0.63096E+00_JPRB, 0.63692E+00_JPRB, 0.64820E+00_JPRB, 0.65939E+00_JPRB ,&
&  0.66361E+00_JPRB, 0.65377E+00_JPRB, 0.63418E+00_JPRB, 0.62718E+00_JPRB, 0.62041E+00_JPRB, 0.61306E+00_JPRB, 0.62279E+00_JPRB ,&
&  0.67552E+00_JPRB, 0.80019E+00_JPRB, 0.73850E+00_JPRB, 0.65587E+00_JPRB, 0.68821E+00_JPRB, 0.69644E+00_JPRB, 0.75295E+00_JPRB ,&
&  0.78556E+00_JPRB, 0.86914E+00_JPRB, 0.90455E+00_JPRB, 0.93860E+00_JPRB, 0.10170E+01_JPRB, 0.11839E+01_JPRB, 0.13602E+01_JPRB ,&
&  0.14412E+01_JPRB, 0.15174E+01_JPRB, 0.16432E+01_JPRB, 0.17698E+01_JPRB, 0.17803E+01_JPRB, 0.15967E+01_JPRB, 0.15802E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.12927E+01_JPRB, 0.11282E+01_JPRB, 0.10031E+01_JPRB, 0.92190E+00_JPRB, 0.87102E+00_JPRB, 0.84283E+00_JPRB ,&
&  0.83164E+00_JPRB, 0.82238E+00_JPRB, 0.79989E+00_JPRB, 0.80898E+00_JPRB, 0.86902E+00_JPRB, 0.12383E+01_JPRB, 0.19090E+01_JPRB ,&
&  0.26830E+01_JPRB, 0.28466E+01_JPRB, 0.30620E+01_JPRB, 0.35978E+01_JPRB, 0.37734E+01_JPRB, 0.38692E+01_JPRB, 0.39269E+01_JPRB ,&
&  0.38460E+01_JPRB, 0.37053E+01_JPRB, 0.34997E+01_JPRB, 0.32359E+01_JPRB, 0.31627E+01_JPRB, 0.31525E+01_JPRB, 0.30243E+01_JPRB ,&
&  0.27421E+01_JPRB, 0.25653E+01_JPRB, 0.25300E+01_JPRB, 0.23810E+01_JPRB, 0.22760E+01_JPRB, 0.20676E+01_JPRB, 0.20882E+01_JPRB ,&
&  0.20839E+01_JPRB, 0.12632E+01_JPRB, 0.10258E+01_JPRB, 0.12314E+01_JPRB, 0.10475E+01_JPRB, 0.11098E+01_JPRB, 0.13505E+01_JPRB ,&
&  0.13786E+01_JPRB, 0.13618E+01_JPRB, 0.14192E+01_JPRB, 0.13416E+01_JPRB, 0.13435E+01_JPRB, 0.14047E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.11699E+01_JPRB, 0.11865E+01_JPRB, 0.11194E+01_JPRB, 0.10265E+01_JPRB, 0.94418E+00_JPRB, 0.87946E+00_JPRB ,&
&  0.82743E+00_JPRB, 0.78987E+00_JPRB, 0.76256E+00_JPRB, 0.71424E+00_JPRB, 0.67516E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.62016E+00_JPRB, 0.60342E+00_JPRB, 0.59359E+00_JPRB, 0.59411E+00_JPRB, 0.60010E+00_JPRB, 0.61421E+00_JPRB, 0.63108E+00_JPRB ,&
&  0.64497E+00_JPRB, 0.64596E+00_JPRB, 0.62926E+00_JPRB, 0.61518E+00_JPRB, 0.60670E+00_JPRB, 0.60183E+00_JPRB, 0.61763E+00_JPRB ,&
&  0.57303E+00_JPRB, 0.64171E+00_JPRB, 0.59194E+00_JPRB, 0.61901E+00_JPRB, 0.70476E+00_JPRB, 0.72108E+00_JPRB, 0.74409E+00_JPRB ,&
&  0.76058E+00_JPRB, 0.79837E+00_JPRB, 0.81491E+00_JPRB, 0.84171E+00_JPRB, 0.89082E+00_JPRB, 0.10071E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.11385E+01_JPRB, 0.12050E+01_JPRB, 0.12846E+01_JPRB, 0.12768E+01_JPRB, 0.13285E+01_JPRB, 0.13570E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.11763E+01_JPRB, 0.10672E+01_JPRB, 0.96424E+00_JPRB, 0.88198E+00_JPRB, 0.81516E+00_JPRB, 0.77451E+00_JPRB ,&
&  0.75126E+00_JPRB, 0.74202E+00_JPRB, 0.74285E+00_JPRB, 0.80235E+00_JPRB, 0.87434E+00_JPRB, 0.13677E+01_JPRB, 0.22472E+01_JPRB ,&
&  0.28813E+01_JPRB, 0.29842E+01_JPRB, 0.33088E+01_JPRB, 0.39490E+01_JPRB, 0.39180E+01_JPRB, 0.38139E+01_JPRB, 0.36838E+01_JPRB ,&
&  0.35737E+01_JPRB, 0.33636E+01_JPRB, 0.32540E+01_JPRB, 0.32671E+01_JPRB, 0.31707E+01_JPRB, 0.32560E+01_JPRB, 0.30832E+01_JPRB ,&
&  0.27788E+01_JPRB, 0.26712E+01_JPRB, 0.26577E+01_JPRB, 0.26035E+01_JPRB, 0.25190E+01_JPRB, 0.25688E+01_JPRB, 0.25296E+01_JPRB ,&
&  0.23964E+01_JPRB, 0.20894E+01_JPRB, 0.15988E+01_JPRB, 0.15152E+01_JPRB, 0.11876E+01_JPRB, 0.12638E+01_JPRB, 0.15939E+01_JPRB ,&
&  0.15996E+01_JPRB, 0.14747E+01_JPRB, 0.13487E+01_JPRB, 0.12873E+01_JPRB, 0.12703E+01_JPRB, 0.13535E+01_JPRB, 0.12447E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.11384E+01_JPRB, 0.11189E+01_JPRB, 0.10598E+01_JPRB, 0.97921E+00_JPRB, 0.89236E+00_JPRB, 0.83119E+00_JPRB ,&
&  0.80001E+00_JPRB, 0.76567E+00_JPRB, 0.71838E+00_JPRB, 0.66792E+00_JPRB, 0.63762E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.60859E+00_JPRB, 0.58793E+00_JPRB, 0.57323E+00_JPRB, 0.57216E+00_JPRB, 0.57842E+00_JPRB, 0.58954E+00_JPRB, 0.60792E+00_JPRB ,&
&  0.62641E+00_JPRB, 0.63424E+00_JPRB, 0.62411E+00_JPRB, 0.60626E+00_JPRB, 0.59238E+00_JPRB, 0.58577E+00_JPRB, 0.55861E+00_JPRB ,&
&  0.52020E+00_JPRB, 0.57117E+00_JPRB, 0.56689E+00_JPRB, 0.58444E+00_JPRB, 0.63190E+00_JPRB, 0.64716E+00_JPRB, 0.66668E+00_JPRB ,&
&  0.70430E+00_JPRB, 0.71779E+00_JPRB, 0.73865E+00_JPRB, 0.76501E+00_JPRB, 0.80144E+00_JPRB, 0.88535E+00_JPRB, 0.98556E+00_JPRB ,&
&  0.96991E+00_JPRB, 0.96661E+00_JPRB, 0.97754E+00_JPRB, 0.10222E+01_JPRB, 0.10790E+01_JPRB, 0.11097E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.10323E+01_JPRB, 0.97579E+00_JPRB, 0.92751E+00_JPRB, 0.88501E+00_JPRB, 0.83157E+00_JPRB, 0.77619E+00_JPRB, 0.73603E+00_JPRB ,&
&  0.70889E+00_JPRB, 0.70013E+00_JPRB, 0.71460E+00_JPRB, 0.74326E+00_JPRB, 0.82900E+00_JPRB, 0.12166E+01_JPRB, 0.17841E+01_JPRB ,&
&  0.21008E+01_JPRB, 0.23672E+01_JPRB, 0.24997E+01_JPRB, 0.26318E+01_JPRB, 0.27466E+01_JPRB, 0.28456E+01_JPRB, 0.29463E+01_JPRB ,&
&  0.30590E+01_JPRB, 0.29234E+01_JPRB, 0.31824E+01_JPRB, 0.32560E+01_JPRB, 0.32546E+01_JPRB, 0.32920E+01_JPRB, 0.30474E+01_JPRB ,&
&  0.28279E+01_JPRB, 0.26220E+01_JPRB, 0.26907E+01_JPRB, 0.26809E+01_JPRB, 0.26189E+01_JPRB, 0.26448E+01_JPRB, 0.25752E+01_JPRB ,&
&  0.25230E+01_JPRB, 0.24470E+01_JPRB, 0.21492E+01_JPRB, 0.19278E+01_JPRB, 0.16364E+01_JPRB, 0.17308E+01_JPRB, 0.15842E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.11380E+01_JPRB, 0.11377E+01_JPRB, 0.11290E+01_JPRB, 0.11690E+01_JPRB, 0.11819E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.11190E+01_JPRB, 0.10523E+01_JPRB, 0.98842E+00_JPRB, 0.92115E+00_JPRB, 0.83709E+00_JPRB, 0.76908E+00_JPRB ,&
&  0.73950E+00_JPRB, 0.72682E+00_JPRB, 0.68584E+00_JPRB, 0.64736E+00_JPRB, 0.62551E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.61897E+00_JPRB, 0.60494E+00_JPRB, 0.58461E+00_JPRB, 0.57379E+00_JPRB, 0.57546E+00_JPRB, 0.58103E+00_JPRB, 0.58274E+00_JPRB ,&
&  0.60091E+00_JPRB, 0.60505E+00_JPRB, 0.60106E+00_JPRB, 0.59016E+00_JPRB, 0.57238E+00_JPRB, 0.52272E+00_JPRB, 0.45727E+00_JPRB ,&
&  0.45116E+00_JPRB, 0.48048E+00_JPRB, 0.51718E+00_JPRB, 0.53778E+00_JPRB, 0.55918E+00_JPRB, 0.57876E+00_JPRB, 0.59496E+00_JPRB ,&
&  0.60710E+00_JPRB, 0.61829E+00_JPRB, 0.64756E+00_JPRB, 0.69028E+00_JPRB, 0.73528E+00_JPRB, 0.78274E+00_JPRB, 0.84001E+00_JPRB ,&
&  0.82217E+00_JPRB, 0.82766E+00_JPRB, 0.85719E+00_JPRB, 0.82618E+00_JPRB, 0.91930E+00_JPRB, 0.92892E+00_JPRB, 0.91626E+00_JPRB ,&
&  0.89475E+00_JPRB, 0.85562E+00_JPRB, 0.80797E+00_JPRB, 0.76980E+00_JPRB, 0.73741E+00_JPRB, 0.71259E+00_JPRB, 0.68181E+00_JPRB ,&
&  0.66176E+00_JPRB, 0.65456E+00_JPRB, 0.66461E+00_JPRB, 0.68513E+00_JPRB, 0.73864E+00_JPRB, 0.97595E+00_JPRB, 0.12133E+01_JPRB ,&
&  0.14065E+01_JPRB, 0.15875E+01_JPRB, 0.16746E+01_JPRB, 0.16933E+01_JPRB, 0.19842E+01_JPRB, 0.23097E+01_JPRB, 0.24775E+01_JPRB ,&
&  0.27422E+01_JPRB, 0.27016E+01_JPRB, 0.28152E+01_JPRB, 0.29197E+01_JPRB, 0.30339E+01_JPRB, 0.29873E+01_JPRB, 0.27809E+01_JPRB ,&
&  0.26481E+01_JPRB, 0.26496E+01_JPRB, 0.26227E+01_JPRB, 0.25766E+01_JPRB, 0.24529E+01_JPRB, 0.23828E+01_JPRB, 0.22899E+01_JPRB ,&
&  0.22746E+01_JPRB, 0.22681E+01_JPRB, 0.21923E+01_JPRB, 0.20916E+01_JPRB, 0.19668E+01_JPRB, 0.17929E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.14213E+01_JPRB, 0.12017E+01_JPRB, 0.11053E+01_JPRB, 0.10968E+01_JPRB, 0.11179E+01_JPRB, 0.10592E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.97382E+00_JPRB, 0.10075E+01_JPRB, 0.95682E+00_JPRB, 0.89263E+00_JPRB, 0.82025E+00_JPRB, 0.77036E+00_JPRB ,&
&  0.70822E+00_JPRB, 0.71879E+00_JPRB, 0.69805E+00_JPRB, 0.66485E+00_JPRB, 0.63592E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.63227E+00_JPRB, 0.61586E+00_JPRB, 0.59536E+00_JPRB, 0.57594E+00_JPRB, 0.57134E+00_JPRB, 0.56231E+00_JPRB, 0.55111E+00_JPRB ,&
&  0.58017E+00_JPRB, 0.55479E+00_JPRB, 0.56419E+00_JPRB, 0.50004E+00_JPRB, 0.47283E+00_JPRB, 0.46274E+00_JPRB, 0.44971E+00_JPRB ,&
&  0.45427E+00_JPRB, 0.47156E+00_JPRB, 0.48783E+00_JPRB, 0.50079E+00_JPRB, 0.52316E+00_JPRB, 0.53777E+00_JPRB, 0.54902E+00_JPRB ,&
&  0.56538E+00_JPRB, 0.58882E+00_JPRB, 0.62676E+00_JPRB, 0.65983E+00_JPRB, 0.70291E+00_JPRB, 0.75712E+00_JPRB, 0.80802E+00_JPRB ,&
&  0.79967E+00_JPRB, 0.81118E+00_JPRB, 0.86335E+00_JPRB, 0.87692E+00_JPRB, 0.89243E+00_JPRB, 0.87810E+00_JPRB, 0.85770E+00_JPRB ,&
&  0.82227E+00_JPRB, 0.71438E+00_JPRB, 0.72797E+00_JPRB, 0.69738E+00_JPRB, 0.67158E+00_JPRB, 0.64749E+00_JPRB, 0.62755E+00_JPRB ,&
&  0.61407E+00_JPRB, 0.60955E+00_JPRB, 0.61401E+00_JPRB, 0.62079E+00_JPRB, 0.63652E+00_JPRB, 0.71533E+00_JPRB, 0.82715E+00_JPRB ,&
&  0.10326E+01_JPRB, 0.98867E+00_JPRB, 0.12480E+01_JPRB, 0.13637E+01_JPRB, 0.16786E+01_JPRB, 0.20103E+01_JPRB, 0.22343E+01_JPRB ,&
&  0.26325E+01_JPRB, 0.26712E+01_JPRB, 0.26286E+01_JPRB, 0.25819E+01_JPRB, 0.26010E+01_JPRB, 0.25726E+01_JPRB, 0.25040E+01_JPRB ,&
&  0.24363E+01_JPRB, 0.22740E+01_JPRB, 0.23805E+01_JPRB, 0.23286E+01_JPRB, 0.22231E+01_JPRB, 0.21561E+01_JPRB, 0.20689E+01_JPRB ,&
&  0.19973E+01_JPRB, 0.19232E+01_JPRB, 0.18445E+01_JPRB, 0.16415E+01_JPRB, 0.15847E+01_JPRB, 0.15323E+01_JPRB, 0.13935E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.12611E+01_JPRB, 0.11733E+01_JPRB, 0.12026E+01_JPRB, 0.11427E+01_JPRB, 0.11093E+01_JPRB, 0.10902E+01_JPRB ,&
&  0.10250E+01_JPRB, 0.91801E+00_JPRB, 0.89995E+00_JPRB, 0.88742E+00_JPRB, 0.85058E+00_JPRB, 0.80827E+00_JPRB, 0.83061E+00_JPRB ,&
&  0.77877E+00_JPRB, 0.75427E+00_JPRB, 0.75512E+00_JPRB, 0.71006E+00_JPRB, 0.66850E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.65983E+00_JPRB, 0.63588E+00_JPRB, 0.61210E+00_JPRB, 0.58526E+00_JPRB, 0.56755E+00_JPRB, 0.54941E+00_JPRB, 0.52313E+00_JPRB ,&
&  0.52315E+00_JPRB, 0.48678E+00_JPRB, 0.46248E+00_JPRB, 0.45303E+00_JPRB, 0.45386E+00_JPRB, 0.45078E+00_JPRB, 0.44340E+00_JPRB ,&
&  0.44642E+00_JPRB, 0.46020E+00_JPRB, 0.48055E+00_JPRB, 0.49679E+00_JPRB, 0.51204E+00_JPRB, 0.52591E+00_JPRB, 0.54002E+00_JPRB ,&
&  0.56064E+00_JPRB, 0.58716E+00_JPRB, 0.61965E+00_JPRB, 0.66165E+00_JPRB, 0.70174E+00_JPRB, 0.75227E+00_JPRB, 0.79090E+00_JPRB ,&
&  0.83306E+00_JPRB, 0.86788E+00_JPRB, 0.87384E+00_JPRB, 0.89611E+00_JPRB, 0.89769E+00_JPRB, 0.87062E+00_JPRB, 0.81129E+00_JPRB ,&
&  0.58796E+00_JPRB, 0.55154E+00_JPRB, 0.63038E+00_JPRB, 0.62233E+00_JPRB, 0.61090E+00_JPRB, 0.59535E+00_JPRB, 0.58750E+00_JPRB ,&
&  0.58433E+00_JPRB, 0.58061E+00_JPRB, 0.58204E+00_JPRB, 0.58224E+00_JPRB, 0.58861E+00_JPRB, 0.60928E+00_JPRB, 0.66221E+00_JPRB ,&
&  0.78423E+00_JPRB, 0.79515E+00_JPRB, 0.85873E+00_JPRB, 0.97819E+00_JPRB, 0.13362E+01_JPRB, 0.16285E+01_JPRB, 0.18207E+01_JPRB ,&
&  0.20488E+01_JPRB, 0.21197E+01_JPRB, 0.21302E+01_JPRB, 0.21588E+01_JPRB, 0.21866E+01_JPRB, 0.22177E+01_JPRB, 0.22699E+01_JPRB ,&
&  0.22166E+01_JPRB, 0.22065E+01_JPRB, 0.21906E+01_JPRB, 0.22004E+01_JPRB, 0.21371E+01_JPRB, 0.20670E+01_JPRB, 0.19855E+01_JPRB ,&
&  0.18969E+01_JPRB, 0.17922E+01_JPRB, 0.16417E+01_JPRB, 0.14986E+01_JPRB, 0.13776E+01_JPRB, 0.13235E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.12530E+01_JPRB, 0.12327E+01_JPRB, 0.12097E+01_JPRB, 0.11820E+01_JPRB, 0.11506E+01_JPRB, 0.11084E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.97951E+00_JPRB, 0.91073E+00_JPRB, 0.81973E+00_JPRB, 0.79290E+00_JPRB, 0.79338E+00_JPRB, 0.78950E+00_JPRB, 0.78151E+00_JPRB ,&
&  0.78474E+00_JPRB, 0.75815E+00_JPRB, 0.72838E+00_JPRB, 0.70824E+00_JPRB, 0.68573E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.68934E+00_JPRB, 0.66348E+00_JPRB, 0.63180E+00_JPRB, 0.59466E+00_JPRB, 0.55931E+00_JPRB, 0.53609E+00_JPRB, 0.51975E+00_JPRB ,&
&  0.51746E+00_JPRB, 0.49454E+00_JPRB, 0.47171E+00_JPRB, 0.45543E+00_JPRB, 0.44737E+00_JPRB, 0.44442E+00_JPRB, 0.44912E+00_JPRB ,&
&  0.45919E+00_JPRB, 0.47045E+00_JPRB, 0.48140E+00_JPRB, 0.49240E+00_JPRB, 0.50426E+00_JPRB, 0.51561E+00_JPRB, 0.52973E+00_JPRB ,&
&  0.54991E+00_JPRB, 0.57277E+00_JPRB, 0.59681E+00_JPRB, 0.62671E+00_JPRB, 0.65815E+00_JPRB, 0.70299E+00_JPRB, 0.73218E+00_JPRB ,&
&  0.75299E+00_JPRB, 0.76507E+00_JPRB, 0.79122E+00_JPRB, 0.85039E+00_JPRB, 0.87810E+00_JPRB, 0.83863E+00_JPRB, 0.64822E+00_JPRB ,&
&  0.50612E+00_JPRB, 0.46711E+00_JPRB, 0.48510E+00_JPRB, 0.54762E+00_JPRB, 0.55008E+00_JPRB, 0.55073E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.53535E+00_JPRB, 0.52821E+00_JPRB, 0.53671E+00_JPRB, 0.55600E+00_JPRB, 0.55967E+00_JPRB, 0.56828E+00_JPRB, 0.59727E+00_JPRB ,&
&  0.65628E+00_JPRB, 0.73412E+00_JPRB, 0.82314E+00_JPRB, 0.82729E+00_JPRB, 0.94632E+00_JPRB, 0.13974E+01_JPRB, 0.15487E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.16790E+01_JPRB, 0.17591E+01_JPRB, 0.18200E+01_JPRB, 0.18004E+01_JPRB, 0.18045E+01_JPRB, 0.18309E+01_JPRB ,&
&  0.18034E+01_JPRB, 0.18033E+01_JPRB, 0.18058E+01_JPRB, 0.18806E+01_JPRB, 0.18906E+01_JPRB, 0.18607E+01_JPRB, 0.18321E+01_JPRB ,&
&  0.17757E+01_JPRB, 0.16357E+01_JPRB, 0.14845E+01_JPRB, 0.13381E+01_JPRB, 0.12640E+01_JPRB, 0.12139E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.11344E+01_JPRB, 0.11307E+01_JPRB, 0.11320E+01_JPRB, 0.10682E+01_JPRB, 0.97368E+00_JPRB, 0.90140E+00_JPRB ,&
&  0.85982E+00_JPRB, 0.83528E+00_JPRB, 0.80028E+00_JPRB, 0.79025E+00_JPRB, 0.79580E+00_JPRB, 0.79992E+00_JPRB, 0.77651E+00_JPRB ,&
&  0.75235E+00_JPRB, 0.73056E+00_JPRB, 0.71878E+00_JPRB, 0.71346E+00_JPRB, 0.70779E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.68610E+00_JPRB, 0.67886E+00_JPRB, 0.65666E+00_JPRB, 0.61716E+00_JPRB, 0.56775E+00_JPRB, 0.52598E+00_JPRB, 0.49920E+00_JPRB ,&
&  0.48556E+00_JPRB, 0.47608E+00_JPRB, 0.47085E+00_JPRB, 0.46872E+00_JPRB, 0.46930E+00_JPRB, 0.47303E+00_JPRB, 0.47696E+00_JPRB ,&
&  0.48011E+00_JPRB, 0.48438E+00_JPRB, 0.49058E+00_JPRB, 0.49795E+00_JPRB, 0.50693E+00_JPRB, 0.51850E+00_JPRB, 0.53285E+00_JPRB ,&
&  0.55186E+00_JPRB, 0.56807E+00_JPRB, 0.57748E+00_JPRB, 0.58498E+00_JPRB, 0.59720E+00_JPRB, 0.62298E+00_JPRB, 0.65116E+00_JPRB ,&
&  0.67067E+00_JPRB, 0.67512E+00_JPRB, 0.70883E+00_JPRB, 0.77701E+00_JPRB, 0.81250E+00_JPRB, 0.76953E+00_JPRB, 0.61100E+00_JPRB ,&
&  0.49383E+00_JPRB, 0.43355E+00_JPRB, 0.41783E+00_JPRB, 0.42929E+00_JPRB, 0.45839E+00_JPRB, 0.49512E+00_JPRB, 0.50564E+00_JPRB ,&
&  0.51233E+00_JPRB, 0.51504E+00_JPRB, 0.51752E+00_JPRB, 0.52157E+00_JPRB, 0.52378E+00_JPRB, 0.52885E+00_JPRB, 0.54702E+00_JPRB ,&
&  0.58482E+00_JPRB, 0.64148E+00_JPRB, 0.70562E+00_JPRB, 0.75094E+00_JPRB, 0.82855E+00_JPRB, 0.98931E+00_JPRB, 0.11692E+01_JPRB ,&
&  0.12542E+01_JPRB, 0.13151E+01_JPRB, 0.13644E+01_JPRB, 0.14636E+01_JPRB, 0.15189E+01_JPRB, 0.15575E+01_JPRB, 0.15675E+01_JPRB ,&
&  0.15600E+01_JPRB, 0.15246E+01_JPRB, 0.14855E+01_JPRB, 0.14802E+01_JPRB, 0.15220E+01_JPRB, 0.16014E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.16376E+01_JPRB, 0.15473E+01_JPRB, 0.13715E+01_JPRB, 0.11980E+01_JPRB, 0.11336E+01_JPRB, 0.11355E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11316E+01_JPRB, 0.11072E+01_JPRB, 0.11023E+01_JPRB, 0.10161E+01_JPRB, 0.96132E+00_JPRB, 0.91612E+00_JPRB ,&
&  0.89352E+00_JPRB, 0.87785E+00_JPRB, 0.87433E+00_JPRB, 0.87884E+00_JPRB, 0.86623E+00_JPRB, 0.84089E+00_JPRB, 0.81080E+00_JPRB ,&
&  0.78415E+00_JPRB, 0.75421E+00_JPRB, 0.72821E+00_JPRB, 0.70389E+00_JPRB, 0.69200E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.73549E+00_JPRB, 0.71843E+00_JPRB, 0.69361E+00_JPRB, 0.65874E+00_JPRB, 0.61496E+00_JPRB, 0.57128E+00_JPRB, 0.54300E+00_JPRB ,&
&  0.52858E+00_JPRB, 0.52091E+00_JPRB, 0.51162E+00_JPRB, 0.50532E+00_JPRB, 0.50168E+00_JPRB, 0.50063E+00_JPRB, 0.50206E+00_JPRB ,&
&  0.50552E+00_JPRB, 0.51021E+00_JPRB, 0.51506E+00_JPRB, 0.51780E+00_JPRB, 0.52361E+00_JPRB, 0.53413E+00_JPRB, 0.54874E+00_JPRB ,&
&  0.56517E+00_JPRB, 0.57360E+00_JPRB, 0.57513E+00_JPRB, 0.57743E+00_JPRB, 0.57542E+00_JPRB, 0.58249E+00_JPRB, 0.58432E+00_JPRB ,&
&  0.59105E+00_JPRB, 0.61175E+00_JPRB, 0.66285E+00_JPRB, 0.71690E+00_JPRB, 0.74216E+00_JPRB, 0.68963E+00_JPRB, 0.57484E+00_JPRB ,&
&  0.48202E+00_JPRB, 0.42343E+00_JPRB, 0.39888E+00_JPRB, 0.39358E+00_JPRB, 0.40326E+00_JPRB, 0.42623E+00_JPRB, 0.45927E+00_JPRB ,&
&  0.48610E+00_JPRB, 0.49214E+00_JPRB, 0.49906E+00_JPRB, 0.50432E+00_JPRB, 0.50709E+00_JPRB, 0.50883E+00_JPRB, 0.51565E+00_JPRB ,&
&  0.53145E+00_JPRB, 0.56481E+00_JPRB, 0.61882E+00_JPRB, 0.70008E+00_JPRB, 0.81469E+00_JPRB, 0.85219E+00_JPRB, 0.94010E+00_JPRB ,&
&  0.10981E+01_JPRB, 0.11658E+01_JPRB, 0.12149E+01_JPRB, 0.12390E+01_JPRB, 0.12544E+01_JPRB, 0.12565E+01_JPRB, 0.12693E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.12917E+01_JPRB, 0.12793E+01_JPRB, 0.12562E+01_JPRB, 0.12477E+01_JPRB, 0.12553E+01_JPRB, 0.12797E+01_JPRB ,&
&  0.13203E+01_JPRB, 0.13872E+01_JPRB, 0.15459E+01_JPRB, 0.13458E+01_JPRB, 0.12638E+01_JPRB, 0.12475E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.11684E+01_JPRB, 0.11346E+01_JPRB, 0.11156E+01_JPRB, 0.10952E+01_JPRB, 0.10374E+01_JPRB, 0.99825E+00_JPRB, 0.96834E+00_JPRB ,&
&  0.95386E+00_JPRB, 0.93561E+00_JPRB, 0.92024E+00_JPRB, 0.90707E+00_JPRB, 0.88858E+00_JPRB, 0.86316E+00_JPRB, 0.84125E+00_JPRB ,&
&  0.82059E+00_JPRB, 0.80060E+00_JPRB, 0.78445E+00_JPRB, 0.76973E+00_JPRB, 0.75410E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.74509E+00_JPRB, 0.72970E+00_JPRB, 0.70557E+00_JPRB, 0.68767E+00_JPRB, 0.66253E+00_JPRB, 0.63565E+00_JPRB, 0.60677E+00_JPRB ,&
&  0.58357E+00_JPRB, 0.56481E+00_JPRB, 0.54924E+00_JPRB, 0.53899E+00_JPRB, 0.53267E+00_JPRB, 0.52882E+00_JPRB, 0.52862E+00_JPRB ,&
&  0.52976E+00_JPRB, 0.52638E+00_JPRB, 0.52811E+00_JPRB, 0.53371E+00_JPRB, 0.54213E+00_JPRB, 0.55115E+00_JPRB, 0.56292E+00_JPRB ,&
&  0.56901E+00_JPRB, 0.56869E+00_JPRB, 0.56336E+00_JPRB, 0.55673E+00_JPRB, 0.55363E+00_JPRB, 0.55860E+00_JPRB, 0.57728E+00_JPRB ,&
&  0.59992E+00_JPRB, 0.62073E+00_JPRB, 0.64327E+00_JPRB, 0.66805E+00_JPRB, 0.67153E+00_JPRB, 0.62595E+00_JPRB, 0.52889E+00_JPRB ,&
&  0.46830E+00_JPRB, 0.42661E+00_JPRB, 0.40756E+00_JPRB, 0.39965E+00_JPRB, 0.40516E+00_JPRB, 0.42137E+00_JPRB, 0.44963E+00_JPRB ,&
&  0.47206E+00_JPRB, 0.48635E+00_JPRB, 0.48888E+00_JPRB, 0.49141E+00_JPRB, 0.49225E+00_JPRB, 0.49597E+00_JPRB, 0.50097E+00_JPRB ,&
&  0.51397E+00_JPRB, 0.52810E+00_JPRB, 0.54758E+00_JPRB, 0.57803E+00_JPRB, 0.61994E+00_JPRB, 0.66531E+00_JPRB, 0.72297E+00_JPRB ,&
&  0.82506E+00_JPRB, 0.88838E+00_JPRB, 0.93256E+00_JPRB, 0.96104E+00_JPRB, 0.99602E+00_JPRB, 0.10190E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.10187E+01_JPRB, 0.10624E+01_JPRB, 0.10799E+01_JPRB, 0.10939E+01_JPRB, 0.11014E+01_JPRB, 0.11144E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11007E+01_JPRB, 0.11068E+01_JPRB, 0.12065E+01_JPRB, 0.12299E+01_JPRB, 0.12052E+01_JPRB, 0.11705E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.10865E+01_JPRB, 0.10497E+01_JPRB, 0.10186E+01_JPRB, 0.98988E+00_JPRB, 0.96789E+00_JPRB, 0.95588E+00_JPRB, 0.95341E+00_JPRB ,&
&  0.94201E+00_JPRB, 0.92620E+00_JPRB, 0.90429E+00_JPRB, 0.88703E+00_JPRB, 0.87220E+00_JPRB, 0.84972E+00_JPRB, 0.82644E+00_JPRB ,&
&  0.80436E+00_JPRB, 0.79012E+00_JPRB, 0.78321E+00_JPRB, 0.77576E+00_JPRB, 0.76240E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.74933E+00_JPRB, 0.74155E+00_JPRB, 0.72512E+00_JPRB, 0.71076E+00_JPRB, 0.69155E+00_JPRB, 0.67409E+00_JPRB, 0.65190E+00_JPRB ,&
&  0.63138E+00_JPRB, 0.61117E+00_JPRB, 0.59415E+00_JPRB, 0.58129E+00_JPRB, 0.57126E+00_JPRB, 0.56397E+00_JPRB, 0.56000E+00_JPRB ,&
&  0.55839E+00_JPRB, 0.55833E+00_JPRB, 0.56045E+00_JPRB, 0.56388E+00_JPRB, 0.56945E+00_JPRB, 0.57590E+00_JPRB, 0.58411E+00_JPRB ,&
&  0.58787E+00_JPRB, 0.58364E+00_JPRB, 0.56762E+00_JPRB, 0.56226E+00_JPRB, 0.56702E+00_JPRB, 0.56851E+00_JPRB, 0.59175E+00_JPRB ,&
&  0.60583E+00_JPRB, 0.62346E+00_JPRB, 0.62502E+00_JPRB, 0.62331E+00_JPRB, 0.61420E+00_JPRB, 0.50687E+00_JPRB, 0.48596E+00_JPRB ,&
&  0.45943E+00_JPRB, 0.43734E+00_JPRB, 0.42626E+00_JPRB, 0.41976E+00_JPRB, 0.41908E+00_JPRB, 0.42841E+00_JPRB, 0.44350E+00_JPRB ,&
&  0.45505E+00_JPRB, 0.49175E+00_JPRB, 0.49195E+00_JPRB, 0.49093E+00_JPRB, 0.49031E+00_JPRB, 0.49372E+00_JPRB, 0.49262E+00_JPRB ,&
&  0.49844E+00_JPRB, 0.50698E+00_JPRB, 0.51932E+00_JPRB, 0.54011E+00_JPRB, 0.56114E+00_JPRB, 0.58214E+00_JPRB, 0.60662E+00_JPRB ,&
&  0.64260E+00_JPRB, 0.67215E+00_JPRB, 0.68989E+00_JPRB, 0.70498E+00_JPRB, 0.72543E+00_JPRB, 0.75292E+00_JPRB, 0.78859E+00_JPRB ,&
&  0.81693E+00_JPRB, 0.84534E+00_JPRB, 0.87583E+00_JPRB, 0.91199E+00_JPRB, 0.94368E+00_JPRB, 0.97802E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10522E+01_JPRB, 0.10587E+01_JPRB, 0.10847E+01_JPRB, 0.10693E+01_JPRB, 0.10351E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.99137E+00_JPRB, 0.99240E+00_JPRB, 0.97407E+00_JPRB, 0.95523E+00_JPRB, 0.93727E+00_JPRB, 0.92413E+00_JPRB, 0.91035E+00_JPRB ,&
&  0.89953E+00_JPRB, 0.88719E+00_JPRB, 0.87394E+00_JPRB, 0.85816E+00_JPRB, 0.84650E+00_JPRB, 0.83105E+00_JPRB, 0.81562E+00_JPRB ,&
&  0.79662E+00_JPRB, 0.78322E+00_JPRB, 0.77408E+00_JPRB, 0.76555E+00_JPRB, 0.75572E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.74439E+00_JPRB, 0.73415E+00_JPRB, 0.72498E+00_JPRB, 0.71386E+00_JPRB, 0.70119E+00_JPRB, 0.68866E+00_JPRB, 0.67238E+00_JPRB ,&
&  0.65675E+00_JPRB, 0.64133E+00_JPRB, 0.62788E+00_JPRB, 0.61715E+00_JPRB, 0.60878E+00_JPRB, 0.60262E+00_JPRB, 0.59931E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.59552E+00_JPRB, 0.59601E+00_JPRB, 0.59795E+00_JPRB, 0.60121E+00_JPRB, 0.60415E+00_JPRB, 0.60822E+00_JPRB ,&
&  0.60547E+00_JPRB, 0.59212E+00_JPRB, 0.57309E+00_JPRB, 0.56941E+00_JPRB, 0.56535E+00_JPRB, 0.56818E+00_JPRB, 0.57014E+00_JPRB ,&
&  0.58577E+00_JPRB, 0.55565E+00_JPRB, 0.55453E+00_JPRB, 0.52495E+00_JPRB, 0.49948E+00_JPRB, 0.47287E+00_JPRB, 0.45001E+00_JPRB ,&
&  0.44084E+00_JPRB, 0.43137E+00_JPRB, 0.42655E+00_JPRB, 0.42717E+00_JPRB, 0.43085E+00_JPRB, 0.44128E+00_JPRB, 0.45429E+00_JPRB ,&
&  0.46395E+00_JPRB, 0.49348E+00_JPRB, 0.49763E+00_JPRB, 0.50076E+00_JPRB, 0.50609E+00_JPRB, 0.51274E+00_JPRB, 0.51234E+00_JPRB ,&
&  0.51712E+00_JPRB, 0.52266E+00_JPRB, 0.53021E+00_JPRB, 0.54233E+00_JPRB, 0.55903E+00_JPRB, 0.57160E+00_JPRB, 0.58483E+00_JPRB ,&
&  0.60081E+00_JPRB, 0.61334E+00_JPRB, 0.62402E+00_JPRB, 0.63313E+00_JPRB, 0.64399E+00_JPRB, 0.65394E+00_JPRB, 0.66869E+00_JPRB ,&
&  0.68485E+00_JPRB, 0.71335E+00_JPRB, 0.73950E+00_JPRB, 0.77852E+00_JPRB, 0.81459E+00_JPRB, 0.85977E+00_JPRB, 0.89672E+00_JPRB ,&
&  0.93379E+00_JPRB, 0.96073E+00_JPRB, 0.98861E+00_JPRB, 0.10147E+01_JPRB, 0.10350E+01_JPRB, 0.10237E+01_JPRB, 0.99298E+00_JPRB ,&
&  0.97361E+00_JPRB, 0.94444E+00_JPRB, 0.92417E+00_JPRB, 0.90530E+00_JPRB, 0.88912E+00_JPRB, 0.87846E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.85979E+00_JPRB, 0.85106E+00_JPRB, 0.84237E+00_JPRB, 0.83232E+00_JPRB, 0.82365E+00_JPRB, 0.81260E+00_JPRB, 0.80142E+00_JPRB ,&
&  0.78997E+00_JPRB, 0.78264E+00_JPRB, 0.77290E+00_JPRB, 0.76254E+00_JPRB, 0.75284E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.74312E+00_JPRB, 0.73079E+00_JPRB, 0.72178E+00_JPRB, 0.71113E+00_JPRB, 0.70106E+00_JPRB, 0.68973E+00_JPRB, 0.67965E+00_JPRB ,&
&  0.66912E+00_JPRB, 0.65890E+00_JPRB, 0.64906E+00_JPRB, 0.64071E+00_JPRB, 0.63479E+00_JPRB, 0.63035E+00_JPRB, 0.62721E+00_JPRB ,&
&  0.62482E+00_JPRB, 0.62307E+00_JPRB, 0.62249E+00_JPRB, 0.62266E+00_JPRB, 0.62329E+00_JPRB, 0.62372E+00_JPRB, 0.62380E+00_JPRB ,&
&  0.61580E+00_JPRB, 0.60166E+00_JPRB, 0.57270E+00_JPRB, 0.56606E+00_JPRB, 0.55587E+00_JPRB, 0.54935E+00_JPRB, 0.54902E+00_JPRB ,&
&  0.54312E+00_JPRB, 0.54149E+00_JPRB, 0.53471E+00_JPRB, 0.52335E+00_JPRB, 0.50847E+00_JPRB, 0.48708E+00_JPRB, 0.47285E+00_JPRB ,&
&  0.46136E+00_JPRB, 0.45436E+00_JPRB, 0.45263E+00_JPRB, 0.45309E+00_JPRB, 0.46001E+00_JPRB, 0.46788E+00_JPRB, 0.47736E+00_JPRB ,&
&  0.49204E+00_JPRB, 0.50602E+00_JPRB, 0.52333E+00_JPRB, 0.52749E+00_JPRB, 0.53622E+00_JPRB, 0.54469E+00_JPRB, 0.55366E+00_JPRB ,&
&  0.56197E+00_JPRB, 0.57034E+00_JPRB, 0.57815E+00_JPRB, 0.58561E+00_JPRB, 0.59318E+00_JPRB, 0.60015E+00_JPRB, 0.60739E+00_JPRB ,&
&  0.61475E+00_JPRB, 0.62134E+00_JPRB, 0.62780E+00_JPRB, 0.63553E+00_JPRB, 0.64517E+00_JPRB, 0.65324E+00_JPRB, 0.66299E+00_JPRB ,&
&  0.67166E+00_JPRB, 0.68178E+00_JPRB, 0.69160E+00_JPRB, 0.70244E+00_JPRB, 0.71288E+00_JPRB, 0.72685E+00_JPRB, 0.74168E+00_JPRB ,&
&  0.76248E+00_JPRB, 0.78148E+00_JPRB, 0.80696E+00_JPRB, 0.82176E+00_JPRB, 0.85352E+00_JPRB, 0.88431E+00_JPRB, 0.90330E+00_JPRB ,&
&  0.90168E+00_JPRB, 0.89332E+00_JPRB, 0.88484E+00_JPRB, 0.87501E+00_JPRB, 0.86619E+00_JPRB, 0.85453E+00_JPRB, 0.84687E+00_JPRB ,&
&  0.83650E+00_JPRB, 0.82882E+00_JPRB, 0.82025E+00_JPRB, 0.81359E+00_JPRB, 0.80767E+00_JPRB, 0.80047E+00_JPRB, 0.79195E+00_JPRB ,&
&  0.78397E+00_JPRB, 0.77756E+00_JPRB, 0.77111E+00_JPRB, 0.76354E+00_JPRB, 0.75410E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.74023E+00_JPRB, 0.73332E+00_JPRB, 0.72649E+00_JPRB, 0.71872E+00_JPRB, 0.71265E+00_JPRB, 0.70564E+00_JPRB, 0.69930E+00_JPRB ,&
&  0.69229E+00_JPRB, 0.68519E+00_JPRB, 0.67856E+00_JPRB, 0.67223E+00_JPRB, 0.66638E+00_JPRB, 0.66100E+00_JPRB, 0.65695E+00_JPRB ,&
&  0.65317E+00_JPRB, 0.65012E+00_JPRB, 0.64791E+00_JPRB, 0.64651E+00_JPRB, 0.64460E+00_JPRB, 0.64198E+00_JPRB, 0.63759E+00_JPRB ,&
&  0.63126E+00_JPRB, 0.62264E+00_JPRB, 0.61287E+00_JPRB, 0.60198E+00_JPRB, 0.59651E+00_JPRB, 0.56406E+00_JPRB, 0.55757E+00_JPRB ,&
&  0.55720E+00_JPRB, 0.55673E+00_JPRB, 0.56038E+00_JPRB, 0.56448E+00_JPRB, 0.56935E+00_JPRB, 0.56051E+00_JPRB, 0.55388E+00_JPRB ,&
&  0.53062E+00_JPRB, 0.53137E+00_JPRB, 0.52951E+00_JPRB, 0.52945E+00_JPRB, 0.53295E+00_JPRB, 0.53551E+00_JPRB, 0.54278E+00_JPRB ,&
&  0.56349E+00_JPRB, 0.56792E+00_JPRB, 0.57445E+00_JPRB, 0.58050E+00_JPRB, 0.58656E+00_JPRB, 0.59198E+00_JPRB, 0.59823E+00_JPRB ,&
&  0.60398E+00_JPRB, 0.61040E+00_JPRB, 0.61733E+00_JPRB, 0.62246E+00_JPRB, 0.62694E+00_JPRB, 0.63170E+00_JPRB, 0.63599E+00_JPRB ,&
&  0.63859E+00_JPRB, 0.64113E+00_JPRB, 0.64393E+00_JPRB, 0.64782E+00_JPRB, 0.65198E+00_JPRB, 0.65613E+00_JPRB, 0.66024E+00_JPRB ,&
&  0.66396E+00_JPRB, 0.66694E+00_JPRB, 0.66861E+00_JPRB, 0.67025E+00_JPRB, 0.67098E+00_JPRB, 0.67190E+00_JPRB, 0.67362E+00_JPRB ,&
&  0.67671E+00_JPRB, 0.68020E+00_JPRB, 0.68479E+00_JPRB, 0.69051E+00_JPRB, 0.69964E+00_JPRB, 0.71090E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.74612E+00_JPRB, 0.75925E+00_JPRB, 0.76485E+00_JPRB, 0.77063E+00_JPRB, 0.77719E+00_JPRB, 0.78081E+00_JPRB, 0.78396E+00_JPRB ,&
&  0.78406E+00_JPRB, 0.78296E+00_JPRB, 0.77961E+00_JPRB, 0.77578E+00_JPRB, 0.77117E+00_JPRB, 0.76646E+00_JPRB, 0.76291E+00_JPRB ,&
&  0.75999E+00_JPRB, 0.75818E+00_JPRB, 0.75546E+00_JPRB, 0.75128E+00_JPRB, 0.74600E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.73584E+00_JPRB, 0.73332E+00_JPRB, 0.73036E+00_JPRB, 0.72692E+00_JPRB, 0.72333E+00_JPRB, 0.71922E+00_JPRB, 0.71519E+00_JPRB ,&
&  0.71073E+00_JPRB, 0.70683E+00_JPRB, 0.70269E+00_JPRB, 0.69851E+00_JPRB, 0.69417E+00_JPRB, 0.69024E+00_JPRB, 0.68648E+00_JPRB ,&
&  0.68244E+00_JPRB, 0.67797E+00_JPRB, 0.67318E+00_JPRB, 0.66902E+00_JPRB, 0.66453E+00_JPRB, 0.65948E+00_JPRB, 0.65372E+00_JPRB ,&
&  0.64921E+00_JPRB, 0.64415E+00_JPRB, 0.63884E+00_JPRB, 0.63200E+00_JPRB, 0.62631E+00_JPRB, 0.62079E+00_JPRB, 0.61591E+00_JPRB ,&
&  0.61107E+00_JPRB, 0.60738E+00_JPRB, 0.60440E+00_JPRB, 0.60194E+00_JPRB, 0.60107E+00_JPRB, 0.60061E+00_JPRB, 0.60006E+00_JPRB ,&
&  0.59924E+00_JPRB, 0.59831E+00_JPRB, 0.59880E+00_JPRB, 0.59961E+00_JPRB, 0.60042E+00_JPRB, 0.60329E+00_JPRB, 0.60668E+00_JPRB ,&
&  0.61009E+00_JPRB, 0.61321E+00_JPRB, 0.61757E+00_JPRB, 0.62204E+00_JPRB, 0.62615E+00_JPRB, 0.63006E+00_JPRB, 0.63366E+00_JPRB ,&
&  0.63760E+00_JPRB, 0.64253E+00_JPRB, 0.64726E+00_JPRB, 0.65219E+00_JPRB, 0.65574E+00_JPRB, 0.65876E+00_JPRB, 0.66116E+00_JPRB ,&
&  0.66305E+00_JPRB, 0.66489E+00_JPRB, 0.66640E+00_JPRB, 0.66774E+00_JPRB, 0.66844E+00_JPRB, 0.66960E+00_JPRB, 0.67057E+00_JPRB ,&
&  0.67166E+00_JPRB, 0.67205E+00_JPRB, 0.67333E+00_JPRB, 0.67446E+00_JPRB, 0.67520E+00_JPRB, 0.67557E+00_JPRB, 0.67672E+00_JPRB ,&
&  0.67771E+00_JPRB, 0.67843E+00_JPRB, 0.67878E+00_JPRB, 0.68086E+00_JPRB, 0.68266E+00_JPRB, 0.68417E+00_JPRB, 0.68496E+00_JPRB ,&
&  0.68954E+00_JPRB, 0.69619E+00_JPRB, 0.70453E+00_JPRB, 0.71579E+00_JPRB, 0.72457E+00_JPRB, 0.73069E+00_JPRB, 0.73544E+00_JPRB ,&
&  0.73831E+00_JPRB, 0.74015E+00_JPRB, 0.74102E+00_JPRB, 0.74169E+00_JPRB, 0.74081E+00_JPRB, 0.74060E+00_JPRB, 0.74118E+00_JPRB ,&
&  0.74150E+00_JPRB, 0.74113E+00_JPRB, 0.74038E+00_JPRB, 0.73950E+00_JPRB, 0.73795E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.72238E+00_JPRB, 0.72104E+00_JPRB, 0.71971E+00_JPRB, 0.71817E+00_JPRB, 0.71636E+00_JPRB, 0.71433E+00_JPRB, 0.71251E+00_JPRB ,&
&  0.71056E+00_JPRB, 0.70893E+00_JPRB, 0.70676E+00_JPRB, 0.70458E+00_JPRB, 0.70235E+00_JPRB, 0.70034E+00_JPRB, 0.69823E+00_JPRB ,&
&  0.69592E+00_JPRB, 0.69356E+00_JPRB, 0.69115E+00_JPRB, 0.68864E+00_JPRB, 0.68598E+00_JPRB, 0.68345E+00_JPRB, 0.68075E+00_JPRB ,&
&  0.67837E+00_JPRB, 0.67575E+00_JPRB, 0.67317E+00_JPRB, 0.67040E+00_JPRB, 0.66845E+00_JPRB, 0.66600E+00_JPRB, 0.66392E+00_JPRB ,&
&  0.66163E+00_JPRB, 0.66010E+00_JPRB, 0.65835E+00_JPRB, 0.65688E+00_JPRB, 0.65546E+00_JPRB, 0.65525E+00_JPRB, 0.65464E+00_JPRB ,&
&  0.65431E+00_JPRB, 0.65424E+00_JPRB, 0.65468E+00_JPRB, 0.65497E+00_JPRB, 0.65526E+00_JPRB, 0.65595E+00_JPRB, 0.65728E+00_JPRB ,&
&  0.65849E+00_JPRB, 0.65972E+00_JPRB, 0.66136E+00_JPRB, 0.66306E+00_JPRB, 0.66458E+00_JPRB, 0.66602E+00_JPRB, 0.66755E+00_JPRB ,&
&  0.66998E+00_JPRB, 0.67259E+00_JPRB, 0.67503E+00_JPRB, 0.67739E+00_JPRB, 0.67948E+00_JPRB, 0.68105E+00_JPRB, 0.68242E+00_JPRB ,&
&  0.68349E+00_JPRB, 0.68469E+00_JPRB, 0.68557E+00_JPRB, 0.68635E+00_JPRB, 0.68665E+00_JPRB, 0.68755E+00_JPRB, 0.68818E+00_JPRB ,&
&  0.68863E+00_JPRB, 0.68847E+00_JPRB, 0.68911E+00_JPRB, 0.68966E+00_JPRB, 0.69035E+00_JPRB, 0.69097E+00_JPRB, 0.69237E+00_JPRB ,&
&  0.69380E+00_JPRB, 0.69562E+00_JPRB, 0.69764E+00_JPRB, 0.69993E+00_JPRB, 0.70170E+00_JPRB, 0.70346E+00_JPRB, 0.70510E+00_JPRB ,&
&  0.70684E+00_JPRB, 0.70872E+00_JPRB, 0.71082E+00_JPRB, 0.71280E+00_JPRB, 0.71460E+00_JPRB, 0.71655E+00_JPRB, 0.71839E+00_JPRB ,&
&  0.72045E+00_JPRB, 0.72176E+00_JPRB, 0.72287E+00_JPRB, 0.72376E+00_JPRB, 0.72445E+00_JPRB, 0.72474E+00_JPRB, 0.72485E+00_JPRB ,&
&  0.72474E+00_JPRB, 0.72447E+00_JPRB, 0.72406E+00_JPRB, 0.72373E+00_JPRB, 0.72334E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB ,&
&  0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB, 0.69436E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1970:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1970
