!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1980 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1980* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB ,&
&  0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB, 0.14938E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.18300E+00_JPRB, 0.18551E+00_JPRB, 0.18717E+00_JPRB, 0.18823E+00_JPRB, 0.18800E+00_JPRB, 0.18892E+00_JPRB, 0.18989E+00_JPRB ,&
&  0.19116E+00_JPRB, 0.19272E+00_JPRB, 0.19380E+00_JPRB, 0.19479E+00_JPRB, 0.19527E+00_JPRB, 0.19526E+00_JPRB, 0.19524E+00_JPRB ,&
&  0.19504E+00_JPRB, 0.19479E+00_JPRB, 0.19170E+00_JPRB, 0.18893E+00_JPRB, 0.18696E+00_JPRB, 0.18588E+00_JPRB, 0.18489E+00_JPRB ,&
&  0.18407E+00_JPRB, 0.18290E+00_JPRB, 0.18148E+00_JPRB, 0.17932E+00_JPRB, 0.17611E+00_JPRB, 0.17298E+00_JPRB, 0.16978E+00_JPRB ,&
&  0.16690E+00_JPRB, 0.16452E+00_JPRB, 0.16233E+00_JPRB, 0.16011E+00_JPRB, 0.15873E+00_JPRB, 0.15717E+00_JPRB, 0.15558E+00_JPRB ,&
&  0.15360E+00_JPRB, 0.15148E+00_JPRB, 0.14946E+00_JPRB, 0.14739E+00_JPRB, 0.14539E+00_JPRB, 0.14342E+00_JPRB, 0.14198E+00_JPRB ,&
&  0.14047E+00_JPRB, 0.13932E+00_JPRB, 0.13806E+00_JPRB, 0.13690E+00_JPRB, 0.13568E+00_JPRB, 0.13454E+00_JPRB, 0.13326E+00_JPRB ,&
&  0.13238E+00_JPRB, 0.13147E+00_JPRB, 0.13055E+00_JPRB, 0.12969E+00_JPRB, 0.12884E+00_JPRB, 0.12801E+00_JPRB, 0.12707E+00_JPRB ,&
&  0.12622E+00_JPRB, 0.12567E+00_JPRB, 0.12510E+00_JPRB, 0.12445E+00_JPRB, 0.12385E+00_JPRB, 0.12344E+00_JPRB, 0.12309E+00_JPRB ,&
&  0.12276E+00_JPRB, 0.12273E+00_JPRB, 0.12289E+00_JPRB, 0.12305E+00_JPRB, 0.12322E+00_JPRB, 0.12360E+00_JPRB, 0.12417E+00_JPRB ,&
&  0.12483E+00_JPRB, 0.12556E+00_JPRB, 0.12649E+00_JPRB, 0.12781E+00_JPRB, 0.12928E+00_JPRB, 0.13096E+00_JPRB, 0.13280E+00_JPRB ,&
&  0.13478E+00_JPRB, 0.13692E+00_JPRB, 0.13917E+00_JPRB, 0.14140E+00_JPRB, 0.14379E+00_JPRB, 0.14619E+00_JPRB, 0.14849E+00_JPRB ,&
&  0.15089E+00_JPRB, 0.15324E+00_JPRB, 0.15563E+00_JPRB, 0.15780E+00_JPRB, 0.16107E+00_JPRB, 0.16319E+00_JPRB, 0.16546E+00_JPRB ,&
&  0.16759E+00_JPRB, 0.17006E+00_JPRB, 0.17254E+00_JPRB, 0.17563E+00_JPRB, 0.17868E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.21036E+00_JPRB, 0.21056E+00_JPRB, 0.21150E+00_JPRB, 0.21821E+00_JPRB, 0.23136E+00_JPRB, 0.30841E+00_JPRB, 0.29979E+00_JPRB ,&
&  0.29275E+00_JPRB, 0.26436E+00_JPRB, 0.26057E+00_JPRB, 0.25332E+00_JPRB, 0.24368E+00_JPRB, 0.23566E+00_JPRB, 0.23083E+00_JPRB ,&
&  0.22562E+00_JPRB, 0.21997E+00_JPRB, 0.21659E+00_JPRB, 0.21326E+00_JPRB, 0.21004E+00_JPRB, 0.20645E+00_JPRB, 0.20371E+00_JPRB ,&
&  0.20132E+00_JPRB, 0.19966E+00_JPRB, 0.19782E+00_JPRB, 0.19431E+00_JPRB, 0.19273E+00_JPRB, 0.18960E+00_JPRB, 0.18490E+00_JPRB ,&
&  0.17880E+00_JPRB, 0.17543E+00_JPRB, 0.17169E+00_JPRB, 0.16834E+00_JPRB, 0.17086E+00_JPRB, 0.16975E+00_JPRB, 0.16751E+00_JPRB ,&
&  0.16402E+00_JPRB, 0.16042E+00_JPRB, 0.15577E+00_JPRB, 0.15133E+00_JPRB, 0.14759E+00_JPRB, 0.14452E+00_JPRB, 0.14185E+00_JPRB ,&
&  0.13971E+00_JPRB, 0.13791E+00_JPRB, 0.13552E+00_JPRB, 0.13336E+00_JPRB, 0.13107E+00_JPRB, 0.12905E+00_JPRB, 0.12703E+00_JPRB ,&
&  0.12544E+00_JPRB, 0.12393E+00_JPRB, 0.12236E+00_JPRB, 0.12075E+00_JPRB, 0.11947E+00_JPRB, 0.11780E+00_JPRB, 0.11655E+00_JPRB ,&
&  0.11545E+00_JPRB, 0.11407E+00_JPRB, 0.11306E+00_JPRB, 0.11182E+00_JPRB, 0.11091E+00_JPRB, 0.11016E+00_JPRB, 0.10943E+00_JPRB ,&
&  0.10844E+00_JPRB, 0.10778E+00_JPRB, 0.10752E+00_JPRB, 0.10731E+00_JPRB, 0.10700E+00_JPRB, 0.10703E+00_JPRB, 0.10732E+00_JPRB ,&
&  0.10775E+00_JPRB, 0.10825E+00_JPRB, 0.10909E+00_JPRB, 0.11010E+00_JPRB, 0.11158E+00_JPRB, 0.11355E+00_JPRB, 0.11668E+00_JPRB ,&
&  0.12008E+00_JPRB, 0.12385E+00_JPRB, 0.12777E+00_JPRB, 0.13223E+00_JPRB, 0.13672E+00_JPRB, 0.14131E+00_JPRB, 0.14659E+00_JPRB ,&
&  0.15229E+00_JPRB, 0.15665E+00_JPRB, 0.16064E+00_JPRB, 0.16487E+00_JPRB, 0.17349E+00_JPRB, 0.18001E+00_JPRB, 0.18510E+00_JPRB ,&
&  0.19079E+00_JPRB, 0.19846E+00_JPRB, 0.20334E+00_JPRB, 0.20672E+00_JPRB, 0.20883E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.46726E+00_JPRB, 0.47870E+00_JPRB, 0.49373E+00_JPRB, 0.50282E+00_JPRB, 0.51267E+00_JPRB, 0.47997E+00_JPRB, 0.45421E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.37160E+00_JPRB, 0.34663E+00_JPRB, 0.32370E+00_JPRB, 0.30331E+00_JPRB, 0.28547E+00_JPRB, 0.26823E+00_JPRB ,&
&  0.25715E+00_JPRB, 0.25041E+00_JPRB, 0.24125E+00_JPRB, 0.23113E+00_JPRB, 0.22034E+00_JPRB, 0.21092E+00_JPRB, 0.20585E+00_JPRB ,&
&  0.20129E+00_JPRB, 0.19801E+00_JPRB, 0.19667E+00_JPRB, 0.19696E+00_JPRB, 0.19890E+00_JPRB, 0.19925E+00_JPRB, 0.20004E+00_JPRB ,&
&  0.19728E+00_JPRB, 0.20435E+00_JPRB, 0.21363E+00_JPRB, 0.21785E+00_JPRB, 0.22557E+00_JPRB, 0.22411E+00_JPRB, 0.21935E+00_JPRB ,&
&  0.21246E+00_JPRB, 0.20418E+00_JPRB, 0.19494E+00_JPRB, 0.18351E+00_JPRB, 0.17305E+00_JPRB, 0.16522E+00_JPRB, 0.15758E+00_JPRB ,&
&  0.15292E+00_JPRB, 0.14747E+00_JPRB, 0.14123E+00_JPRB, 0.13589E+00_JPRB, 0.13127E+00_JPRB, 0.12742E+00_JPRB, 0.12395E+00_JPRB ,&
&  0.12135E+00_JPRB, 0.11977E+00_JPRB, 0.11825E+00_JPRB, 0.11774E+00_JPRB, 0.11662E+00_JPRB, 0.11555E+00_JPRB, 0.11450E+00_JPRB ,&
&  0.11390E+00_JPRB, 0.11274E+00_JPRB, 0.11185E+00_JPRB, 0.11104E+00_JPRB, 0.10981E+00_JPRB, 0.10874E+00_JPRB, 0.10752E+00_JPRB ,&
&  0.10646E+00_JPRB, 0.10589E+00_JPRB, 0.10558E+00_JPRB, 0.10551E+00_JPRB, 0.10577E+00_JPRB, 0.10573E+00_JPRB, 0.10597E+00_JPRB ,&
&  0.10626E+00_JPRB, 0.10669E+00_JPRB, 0.10670E+00_JPRB, 0.10700E+00_JPRB, 0.10783E+00_JPRB, 0.10897E+00_JPRB, 0.11169E+00_JPRB ,&
&  0.11484E+00_JPRB, 0.11889E+00_JPRB, 0.12333E+00_JPRB, 0.12848E+00_JPRB, 0.13366E+00_JPRB, 0.13887E+00_JPRB, 0.14460E+00_JPRB ,&
&  0.15205E+00_JPRB, 0.15874E+00_JPRB, 0.16645E+00_JPRB, 0.17811E+00_JPRB, 0.20273E+00_JPRB, 0.21664E+00_JPRB, 0.23190E+00_JPRB ,&
&  0.25893E+00_JPRB, 0.29635E+00_JPRB, 0.39954E+00_JPRB, 0.43238E+00_JPRB, 0.44962E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.64645E+00_JPRB, 0.67747E+00_JPRB, 0.70180E+00_JPRB, 0.70393E+00_JPRB, 0.70277E+00_JPRB, 0.66547E+00_JPRB, 0.60759E+00_JPRB ,&
&  0.50076E+00_JPRB, 0.43109E+00_JPRB, 0.38722E+00_JPRB, 0.34583E+00_JPRB, 0.30827E+00_JPRB, 0.28302E+00_JPRB, 0.26573E+00_JPRB ,&
&  0.25260E+00_JPRB, 0.24539E+00_JPRB, 0.23990E+00_JPRB, 0.23528E+00_JPRB, 0.23030E+00_JPRB, 0.22550E+00_JPRB, 0.21677E+00_JPRB ,&
&  0.21152E+00_JPRB, 0.20801E+00_JPRB, 0.20662E+00_JPRB, 0.20907E+00_JPRB, 0.21594E+00_JPRB, 0.22306E+00_JPRB, 0.23377E+00_JPRB ,&
&  0.24717E+00_JPRB, 0.26097E+00_JPRB, 0.26808E+00_JPRB, 0.27131E+00_JPRB, 0.27421E+00_JPRB, 0.27230E+00_JPRB, 0.24964E+00_JPRB ,&
&  0.25467E+00_JPRB, 0.27766E+00_JPRB, 0.29598E+00_JPRB, 0.30641E+00_JPRB, 0.26236E+00_JPRB, 0.22114E+00_JPRB, 0.18059E+00_JPRB ,&
&  0.16108E+00_JPRB, 0.14590E+00_JPRB, 0.13807E+00_JPRB, 0.13217E+00_JPRB, 0.12793E+00_JPRB, 0.12540E+00_JPRB, 0.12317E+00_JPRB ,&
&  0.12114E+00_JPRB, 0.12027E+00_JPRB, 0.11891E+00_JPRB, 0.11831E+00_JPRB, 0.11739E+00_JPRB, 0.11726E+00_JPRB, 0.11611E+00_JPRB ,&
&  0.11493E+00_JPRB, 0.11400E+00_JPRB, 0.11342E+00_JPRB, 0.11313E+00_JPRB, 0.11238E+00_JPRB, 0.11163E+00_JPRB, 0.10990E+00_JPRB ,&
&  0.10869E+00_JPRB, 0.10749E+00_JPRB, 0.10659E+00_JPRB, 0.10663E+00_JPRB, 0.10711E+00_JPRB, 0.10879E+00_JPRB, 0.10974E+00_JPRB ,&
&  0.10995E+00_JPRB, 0.11004E+00_JPRB, 0.11008E+00_JPRB, 0.11078E+00_JPRB, 0.11183E+00_JPRB, 0.11365E+00_JPRB, 0.11619E+00_JPRB ,&
&  0.11925E+00_JPRB, 0.12391E+00_JPRB, 0.12865E+00_JPRB, 0.13397E+00_JPRB, 0.13899E+00_JPRB, 0.14399E+00_JPRB, 0.15050E+00_JPRB ,&
&  0.15719E+00_JPRB, 0.16494E+00_JPRB, 0.17629E+00_JPRB, 0.18850E+00_JPRB, 0.21037E+00_JPRB, 0.22986E+00_JPRB, 0.26161E+00_JPRB ,&
&  0.33227E+00_JPRB, 0.47941E+00_JPRB, 0.56487E+00_JPRB, 0.57592E+00_JPRB, 0.60883E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.76763E+00_JPRB, 0.78592E+00_JPRB, 0.80213E+00_JPRB, 0.79945E+00_JPRB, 0.78214E+00_JPRB, 0.72444E+00_JPRB, 0.66099E+00_JPRB ,&
&  0.58227E+00_JPRB, 0.51605E+00_JPRB, 0.44672E+00_JPRB, 0.38661E+00_JPRB, 0.33624E+00_JPRB, 0.29751E+00_JPRB, 0.27442E+00_JPRB ,&
&  0.25814E+00_JPRB, 0.25102E+00_JPRB, 0.24839E+00_JPRB, 0.25294E+00_JPRB, 0.25538E+00_JPRB, 0.26079E+00_JPRB, 0.25517E+00_JPRB ,&
&  0.24822E+00_JPRB, 0.24014E+00_JPRB, 0.23816E+00_JPRB, 0.24690E+00_JPRB, 0.26004E+00_JPRB, 0.27283E+00_JPRB, 0.28943E+00_JPRB ,&
&  0.30771E+00_JPRB, 0.31354E+00_JPRB, 0.32749E+00_JPRB, 0.33117E+00_JPRB, 0.31642E+00_JPRB, 0.31808E+00_JPRB, 0.32388E+00_JPRB ,&
&  0.33699E+00_JPRB, 0.35551E+00_JPRB, 0.36611E+00_JPRB, 0.38621E+00_JPRB, 0.33365E+00_JPRB, 0.27355E+00_JPRB, 0.22196E+00_JPRB ,&
&  0.18362E+00_JPRB, 0.16185E+00_JPRB, 0.14675E+00_JPRB, 0.13906E+00_JPRB, 0.13175E+00_JPRB, 0.12776E+00_JPRB, 0.12353E+00_JPRB ,&
&  0.12166E+00_JPRB, 0.12158E+00_JPRB, 0.12110E+00_JPRB, 0.12079E+00_JPRB, 0.12120E+00_JPRB, 0.12188E+00_JPRB, 0.12138E+00_JPRB ,&
&  0.12071E+00_JPRB, 0.12048E+00_JPRB, 0.12005E+00_JPRB, 0.11975E+00_JPRB, 0.11896E+00_JPRB, 0.11888E+00_JPRB, 0.11746E+00_JPRB ,&
&  0.11676E+00_JPRB, 0.11548E+00_JPRB, 0.11476E+00_JPRB, 0.11468E+00_JPRB, 0.11506E+00_JPRB, 0.11544E+00_JPRB, 0.11605E+00_JPRB ,&
&  0.11721E+00_JPRB, 0.11828E+00_JPRB, 0.12005E+00_JPRB, 0.12180E+00_JPRB, 0.12458E+00_JPRB, 0.12713E+00_JPRB, 0.13070E+00_JPRB ,&
&  0.13449E+00_JPRB, 0.13907E+00_JPRB, 0.14430E+00_JPRB, 0.15017E+00_JPRB, 0.15526E+00_JPRB, 0.16026E+00_JPRB, 0.16572E+00_JPRB ,&
&  0.17004E+00_JPRB, 0.17704E+00_JPRB, 0.18493E+00_JPRB, 0.19800E+00_JPRB, 0.22527E+00_JPRB, 0.24905E+00_JPRB, 0.29793E+00_JPRB ,&
&  0.38104E+00_JPRB, 0.63529E+00_JPRB, 0.86667E+00_JPRB, 0.71827E+00_JPRB, 0.74929E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.77682E+00_JPRB, 0.78921E+00_JPRB, 0.80618E+00_JPRB, 0.78261E+00_JPRB, 0.76724E+00_JPRB, 0.77053E+00_JPRB, 0.77126E+00_JPRB ,&
&  0.74807E+00_JPRB, 0.71291E+00_JPRB, 0.66376E+00_JPRB, 0.62334E+00_JPRB, 0.55780E+00_JPRB, 0.47365E+00_JPRB, 0.43595E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.39802E+00_JPRB, 0.40020E+00_JPRB, 0.41616E+00_JPRB, 0.42491E+00_JPRB, 0.42144E+00_JPRB, 0.39969E+00_JPRB ,&
&  0.37257E+00_JPRB, 0.34538E+00_JPRB, 0.32788E+00_JPRB, 0.31394E+00_JPRB, 0.31374E+00_JPRB, 0.31190E+00_JPRB, 0.30539E+00_JPRB ,&
&  0.29683E+00_JPRB, 0.30113E+00_JPRB, 0.31845E+00_JPRB, 0.34684E+00_JPRB, 0.39636E+00_JPRB, 0.37630E+00_JPRB, 0.38263E+00_JPRB ,&
&  0.41264E+00_JPRB, 0.43425E+00_JPRB, 0.44279E+00_JPRB, 0.44586E+00_JPRB, 0.43908E+00_JPRB, 0.46384E+00_JPRB, 0.40728E+00_JPRB ,&
&  0.30023E+00_JPRB, 0.24133E+00_JPRB, 0.19938E+00_JPRB, 0.17246E+00_JPRB, 0.15462E+00_JPRB, 0.14427E+00_JPRB, 0.13295E+00_JPRB ,&
&  0.12778E+00_JPRB, 0.12399E+00_JPRB, 0.12313E+00_JPRB, 0.12467E+00_JPRB, 0.12720E+00_JPRB, 0.13042E+00_JPRB, 0.13161E+00_JPRB ,&
&  0.13348E+00_JPRB, 0.13362E+00_JPRB, 0.13289E+00_JPRB, 0.13277E+00_JPRB, 0.13182E+00_JPRB, 0.13254E+00_JPRB, 0.13261E+00_JPRB ,&
&  0.13272E+00_JPRB, 0.13275E+00_JPRB, 0.13436E+00_JPRB, 0.13498E+00_JPRB, 0.13589E+00_JPRB, 0.13299E+00_JPRB, 0.13133E+00_JPRB ,&
&  0.13151E+00_JPRB, 0.13354E+00_JPRB, 0.13686E+00_JPRB, 0.14067E+00_JPRB, 0.14589E+00_JPRB, 0.15008E+00_JPRB, 0.15492E+00_JPRB ,&
&  0.15915E+00_JPRB, 0.16330E+00_JPRB, 0.16735E+00_JPRB, 0.17148E+00_JPRB, 0.17589E+00_JPRB, 0.18031E+00_JPRB, 0.18491E+00_JPRB ,&
&  0.18967E+00_JPRB, 0.19785E+00_JPRB, 0.20789E+00_JPRB, 0.22543E+00_JPRB, 0.24911E+00_JPRB, 0.27611E+00_JPRB, 0.32332E+00_JPRB ,&
&  0.38371E+00_JPRB, 0.61292E+00_JPRB, 0.91241E+00_JPRB, 0.74063E+00_JPRB, 0.75841E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.82797E+00_JPRB, 0.82560E+00_JPRB, 0.83338E+00_JPRB, 0.82296E+00_JPRB, 0.80382E+00_JPRB, 0.79469E+00_JPRB, 0.81159E+00_JPRB ,&
&  0.83129E+00_JPRB, 0.80178E+00_JPRB, 0.78008E+00_JPRB, 0.76082E+00_JPRB, 0.74715E+00_JPRB, 0.73332E+00_JPRB, 0.71111E+00_JPRB ,&
&  0.69097E+00_JPRB, 0.68357E+00_JPRB, 0.68160E+00_JPRB, 0.67648E+00_JPRB, 0.66614E+00_JPRB, 0.65599E+00_JPRB, 0.63820E+00_JPRB ,&
&  0.60550E+00_JPRB, 0.57118E+00_JPRB, 0.55314E+00_JPRB, 0.52953E+00_JPRB, 0.51269E+00_JPRB, 0.49326E+00_JPRB, 0.46562E+00_JPRB ,&
&  0.42070E+00_JPRB, 0.36267E+00_JPRB, 0.32656E+00_JPRB, 0.36302E+00_JPRB, 0.45820E+00_JPRB, 0.45209E+00_JPRB, 0.47129E+00_JPRB ,&
&  0.50598E+00_JPRB, 0.52723E+00_JPRB, 0.53762E+00_JPRB, 0.54086E+00_JPRB, 0.54992E+00_JPRB, 0.55538E+00_JPRB, 0.55181E+00_JPRB ,&
&  0.53719E+00_JPRB, 0.48492E+00_JPRB, 0.44242E+00_JPRB, 0.33967E+00_JPRB, 0.26832E+00_JPRB, 0.23071E+00_JPRB, 0.20167E+00_JPRB ,&
&  0.18291E+00_JPRB, 0.17185E+00_JPRB, 0.16942E+00_JPRB, 0.16928E+00_JPRB, 0.17464E+00_JPRB, 0.18031E+00_JPRB, 0.18046E+00_JPRB ,&
&  0.17683E+00_JPRB, 0.16921E+00_JPRB, 0.15822E+00_JPRB, 0.15166E+00_JPRB, 0.14833E+00_JPRB, 0.14859E+00_JPRB, 0.15069E+00_JPRB ,&
&  0.15242E+00_JPRB, 0.15665E+00_JPRB, 0.16912E+00_JPRB, 0.18005E+00_JPRB, 0.19356E+00_JPRB, 0.17574E+00_JPRB, 0.16118E+00_JPRB ,&
&  0.15598E+00_JPRB, 0.16027E+00_JPRB, 0.16695E+00_JPRB, 0.17198E+00_JPRB, 0.17728E+00_JPRB, 0.18184E+00_JPRB, 0.18647E+00_JPRB ,&
&  0.19099E+00_JPRB, 0.19421E+00_JPRB, 0.19586E+00_JPRB, 0.19763E+00_JPRB, 0.19922E+00_JPRB, 0.20161E+00_JPRB, 0.20457E+00_JPRB ,&
&  0.20959E+00_JPRB, 0.21890E+00_JPRB, 0.23102E+00_JPRB, 0.25100E+00_JPRB, 0.26976E+00_JPRB, 0.28849E+00_JPRB, 0.31002E+00_JPRB ,&
&  0.33104E+00_JPRB, 0.41127E+00_JPRB, 0.60496E+00_JPRB, 0.92601E+00_JPRB, 0.91930E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.82253E+00_JPRB, 0.84242E+00_JPRB, 0.85332E+00_JPRB, 0.86724E+00_JPRB, 0.86734E+00_JPRB, 0.83702E+00_JPRB, 0.82227E+00_JPRB ,&
&  0.82685E+00_JPRB, 0.82187E+00_JPRB, 0.81095E+00_JPRB, 0.80418E+00_JPRB, 0.79516E+00_JPRB, 0.78528E+00_JPRB, 0.77686E+00_JPRB ,&
&  0.76849E+00_JPRB, 0.76343E+00_JPRB, 0.77035E+00_JPRB, 0.78424E+00_JPRB, 0.79563E+00_JPRB, 0.80185E+00_JPRB, 0.80674E+00_JPRB ,&
&  0.80617E+00_JPRB, 0.77306E+00_JPRB, 0.72795E+00_JPRB, 0.69504E+00_JPRB, 0.67847E+00_JPRB, 0.66467E+00_JPRB, 0.63162E+00_JPRB ,&
&  0.56416E+00_JPRB, 0.44259E+00_JPRB, 0.38606E+00_JPRB, 0.42723E+00_JPRB, 0.54572E+00_JPRB, 0.51053E+00_JPRB, 0.53102E+00_JPRB ,&
&  0.56143E+00_JPRB, 0.57718E+00_JPRB, 0.57293E+00_JPRB, 0.56238E+00_JPRB, 0.55881E+00_JPRB, 0.56232E+00_JPRB, 0.55888E+00_JPRB ,&
&  0.55533E+00_JPRB, 0.53926E+00_JPRB, 0.52964E+00_JPRB, 0.51880E+00_JPRB, 0.50121E+00_JPRB, 0.47543E+00_JPRB, 0.45985E+00_JPRB ,&
&  0.42175E+00_JPRB, 0.38425E+00_JPRB, 0.37347E+00_JPRB, 0.37437E+00_JPRB, 0.34373E+00_JPRB, 0.33144E+00_JPRB, 0.31691E+00_JPRB ,&
&  0.27727E+00_JPRB, 0.25607E+00_JPRB, 0.22408E+00_JPRB, 0.20403E+00_JPRB, 0.18950E+00_JPRB, 0.17573E+00_JPRB, 0.17387E+00_JPRB ,&
&  0.17872E+00_JPRB, 0.19123E+00_JPRB, 0.21274E+00_JPRB, 0.24681E+00_JPRB, 0.27782E+00_JPRB, 0.23472E+00_JPRB, 0.21675E+00_JPRB ,&
&  0.20272E+00_JPRB, 0.19958E+00_JPRB, 0.20505E+00_JPRB, 0.21150E+00_JPRB, 0.21813E+00_JPRB, 0.22600E+00_JPRB, 0.23357E+00_JPRB ,&
&  0.23910E+00_JPRB, 0.24223E+00_JPRB, 0.24319E+00_JPRB, 0.24254E+00_JPRB, 0.24159E+00_JPRB, 0.24248E+00_JPRB, 0.24599E+00_JPRB ,&
&  0.25216E+00_JPRB, 0.26443E+00_JPRB, 0.28826E+00_JPRB, 0.32350E+00_JPRB, 0.36957E+00_JPRB, 0.42988E+00_JPRB, 0.52220E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.81140E+00_JPRB, 0.85581E+00_JPRB, 0.86486E+00_JPRB, 0.83413E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.86241E+00_JPRB, 0.87301E+00_JPRB, 0.87079E+00_JPRB, 0.86965E+00_JPRB, 0.87130E+00_JPRB, 0.86413E+00_JPRB, 0.84986E+00_JPRB ,&
&  0.83560E+00_JPRB, 0.80901E+00_JPRB, 0.77856E+00_JPRB, 0.76485E+00_JPRB, 0.77226E+00_JPRB, 0.77204E+00_JPRB, 0.75332E+00_JPRB ,&
&  0.75150E+00_JPRB, 0.76525E+00_JPRB, 0.78814E+00_JPRB, 0.81367E+00_JPRB, 0.84809E+00_JPRB, 0.88381E+00_JPRB, 0.91046E+00_JPRB ,&
&  0.92089E+00_JPRB, 0.91882E+00_JPRB, 0.91117E+00_JPRB, 0.88448E+00_JPRB, 0.84883E+00_JPRB, 0.82155E+00_JPRB, 0.80077E+00_JPRB ,&
&  0.77504E+00_JPRB, 0.72730E+00_JPRB, 0.65778E+00_JPRB, 0.65228E+00_JPRB, 0.58842E+00_JPRB, 0.56807E+00_JPRB, 0.59009E+00_JPRB ,&
&  0.60542E+00_JPRB, 0.58745E+00_JPRB, 0.56356E+00_JPRB, 0.54371E+00_JPRB, 0.53980E+00_JPRB, 0.53835E+00_JPRB, 0.53317E+00_JPRB ,&
&  0.52921E+00_JPRB, 0.52236E+00_JPRB, 0.50817E+00_JPRB, 0.49510E+00_JPRB, 0.48273E+00_JPRB, 0.47038E+00_JPRB, 0.46193E+00_JPRB ,&
&  0.43385E+00_JPRB, 0.39095E+00_JPRB, 0.35909E+00_JPRB, 0.34529E+00_JPRB, 0.33771E+00_JPRB, 0.34153E+00_JPRB, 0.35437E+00_JPRB ,&
&  0.36474E+00_JPRB, 0.36542E+00_JPRB, 0.35994E+00_JPRB, 0.34762E+00_JPRB, 0.32874E+00_JPRB, 0.28380E+00_JPRB, 0.25815E+00_JPRB ,&
&  0.27416E+00_JPRB, 0.30783E+00_JPRB, 0.34282E+00_JPRB, 0.35219E+00_JPRB, 0.36168E+00_JPRB, 0.36395E+00_JPRB, 0.34924E+00_JPRB ,&
&  0.32091E+00_JPRB, 0.29050E+00_JPRB, 0.28453E+00_JPRB, 0.28658E+00_JPRB, 0.29113E+00_JPRB, 0.29971E+00_JPRB, 0.31569E+00_JPRB ,&
&  0.33974E+00_JPRB, 0.37832E+00_JPRB, 0.40045E+00_JPRB, 0.40310E+00_JPRB, 0.39575E+00_JPRB, 0.38430E+00_JPRB, 0.38230E+00_JPRB ,&
&  0.39627E+00_JPRB, 0.45710E+00_JPRB, 0.57898E+00_JPRB, 0.72614E+00_JPRB, 0.72558E+00_JPRB, 0.76960E+00_JPRB, 0.80436E+00_JPRB ,&
&  0.83414E+00_JPRB, 0.84754E+00_JPRB, 0.83426E+00_JPRB, 0.83561E+00_JPRB, 0.84231E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.87658E+00_JPRB, 0.88098E+00_JPRB, 0.87124E+00_JPRB, 0.86688E+00_JPRB, 0.85624E+00_JPRB, 0.84262E+00_JPRB, 0.82019E+00_JPRB ,&
&  0.79377E+00_JPRB, 0.76557E+00_JPRB, 0.73895E+00_JPRB, 0.71717E+00_JPRB, 0.72114E+00_JPRB, 0.75368E+00_JPRB, 0.78111E+00_JPRB ,&
&  0.77009E+00_JPRB, 0.76606E+00_JPRB, 0.78303E+00_JPRB, 0.81351E+00_JPRB, 0.84672E+00_JPRB, 0.87738E+00_JPRB, 0.89940E+00_JPRB ,&
&  0.91726E+00_JPRB, 0.89895E+00_JPRB, 0.90256E+00_JPRB, 0.90459E+00_JPRB, 0.89856E+00_JPRB, 0.88073E+00_JPRB, 0.85206E+00_JPRB ,&
&  0.82357E+00_JPRB, 0.79639E+00_JPRB, 0.76275E+00_JPRB, 0.70381E+00_JPRB, 0.77715E+00_JPRB, 0.70036E+00_JPRB, 0.67164E+00_JPRB ,&
&  0.64430E+00_JPRB, 0.62596E+00_JPRB, 0.60533E+00_JPRB, 0.58307E+00_JPRB, 0.57536E+00_JPRB, 0.57069E+00_JPRB, 0.56275E+00_JPRB ,&
&  0.55620E+00_JPRB, 0.54907E+00_JPRB, 0.53824E+00_JPRB, 0.52443E+00_JPRB, 0.51144E+00_JPRB, 0.49757E+00_JPRB, 0.48485E+00_JPRB ,&
&  0.46646E+00_JPRB, 0.43964E+00_JPRB, 0.41144E+00_JPRB, 0.39074E+00_JPRB, 0.37999E+00_JPRB, 0.38915E+00_JPRB, 0.42164E+00_JPRB ,&
&  0.45511E+00_JPRB, 0.46086E+00_JPRB, 0.47067E+00_JPRB, 0.47359E+00_JPRB, 0.46529E+00_JPRB, 0.45552E+00_JPRB, 0.43813E+00_JPRB ,&
&  0.42484E+00_JPRB, 0.42279E+00_JPRB, 0.42402E+00_JPRB, 0.42792E+00_JPRB, 0.43316E+00_JPRB, 0.44045E+00_JPRB, 0.45058E+00_JPRB ,&
&  0.45989E+00_JPRB, 0.46301E+00_JPRB, 0.47518E+00_JPRB, 0.50187E+00_JPRB, 0.53277E+00_JPRB, 0.56414E+00_JPRB, 0.59096E+00_JPRB ,&
&  0.60906E+00_JPRB, 0.62165E+00_JPRB, 0.62991E+00_JPRB, 0.64508E+00_JPRB, 0.66077E+00_JPRB, 0.67659E+00_JPRB, 0.68482E+00_JPRB ,&
&  0.68244E+00_JPRB, 0.68602E+00_JPRB, 0.70311E+00_JPRB, 0.73139E+00_JPRB, 0.75117E+00_JPRB, 0.78505E+00_JPRB, 0.80624E+00_JPRB ,&
&  0.82874E+00_JPRB, 0.83456E+00_JPRB, 0.82923E+00_JPRB, 0.84186E+00_JPRB, 0.85881E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.83681E+00_JPRB, 0.85458E+00_JPRB, 0.86652E+00_JPRB, 0.86733E+00_JPRB, 0.85186E+00_JPRB, 0.82516E+00_JPRB, 0.80227E+00_JPRB ,&
&  0.77611E+00_JPRB, 0.75073E+00_JPRB, 0.72661E+00_JPRB, 0.69386E+00_JPRB, 0.67624E+00_JPRB, 0.67957E+00_JPRB, 0.72844E+00_JPRB ,&
&  0.79371E+00_JPRB, 0.79857E+00_JPRB, 0.79243E+00_JPRB, 0.81758E+00_JPRB, 0.86995E+00_JPRB, 0.90169E+00_JPRB, 0.92258E+00_JPRB ,&
&  0.92548E+00_JPRB, 0.89250E+00_JPRB, 0.85928E+00_JPRB, 0.84573E+00_JPRB, 0.86904E+00_JPRB, 0.88687E+00_JPRB, 0.89209E+00_JPRB ,&
&  0.88771E+00_JPRB, 0.86649E+00_JPRB, 0.84738E+00_JPRB, 0.83125E+00_JPRB, 0.83538E+00_JPRB, 0.81038E+00_JPRB, 0.75791E+00_JPRB ,&
&  0.69156E+00_JPRB, 0.66492E+00_JPRB, 0.65316E+00_JPRB, 0.63846E+00_JPRB, 0.62745E+00_JPRB, 0.62753E+00_JPRB, 0.61833E+00_JPRB ,&
&  0.60487E+00_JPRB, 0.58922E+00_JPRB, 0.57416E+00_JPRB, 0.55633E+00_JPRB, 0.54231E+00_JPRB, 0.52258E+00_JPRB, 0.50412E+00_JPRB ,&
&  0.48702E+00_JPRB, 0.46605E+00_JPRB, 0.44679E+00_JPRB, 0.43335E+00_JPRB, 0.42743E+00_JPRB, 0.43755E+00_JPRB, 0.47717E+00_JPRB ,&
&  0.50838E+00_JPRB, 0.50540E+00_JPRB, 0.50303E+00_JPRB, 0.50703E+00_JPRB, 0.50000E+00_JPRB, 0.48972E+00_JPRB, 0.48424E+00_JPRB ,&
&  0.47940E+00_JPRB, 0.47558E+00_JPRB, 0.47403E+00_JPRB, 0.47612E+00_JPRB, 0.47847E+00_JPRB, 0.47673E+00_JPRB, 0.47917E+00_JPRB ,&
&  0.48381E+00_JPRB, 0.48898E+00_JPRB, 0.50599E+00_JPRB, 0.53042E+00_JPRB, 0.55103E+00_JPRB, 0.56564E+00_JPRB, 0.57920E+00_JPRB ,&
&  0.60017E+00_JPRB, 0.61304E+00_JPRB, 0.60040E+00_JPRB, 0.58833E+00_JPRB, 0.58206E+00_JPRB, 0.58722E+00_JPRB, 0.59779E+00_JPRB ,&
&  0.61852E+00_JPRB, 0.63153E+00_JPRB, 0.63773E+00_JPRB, 0.63936E+00_JPRB, 0.64271E+00_JPRB, 0.64367E+00_JPRB, 0.65634E+00_JPRB ,&
&  0.69021E+00_JPRB, 0.71939E+00_JPRB, 0.72685E+00_JPRB, 0.75962E+00_JPRB, 0.80164E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.76949E+00_JPRB, 0.80132E+00_JPRB, 0.82573E+00_JPRB, 0.83473E+00_JPRB, 0.83284E+00_JPRB, 0.82414E+00_JPRB, 0.80164E+00_JPRB ,&
&  0.77715E+00_JPRB, 0.75978E+00_JPRB, 0.74126E+00_JPRB, 0.71470E+00_JPRB, 0.69264E+00_JPRB, 0.68799E+00_JPRB, 0.70056E+00_JPRB ,&
&  0.74688E+00_JPRB, 0.78797E+00_JPRB, 0.80448E+00_JPRB, 0.81801E+00_JPRB, 0.86776E+00_JPRB, 0.90960E+00_JPRB, 0.91876E+00_JPRB ,&
&  0.91395E+00_JPRB, 0.91570E+00_JPRB, 0.87925E+00_JPRB, 0.84595E+00_JPRB, 0.84591E+00_JPRB, 0.85124E+00_JPRB, 0.83316E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.87608E+00_JPRB, 0.86985E+00_JPRB, 0.84477E+00_JPRB, 0.83727E+00_JPRB, 0.82717E+00_JPRB, 0.81569E+00_JPRB ,&
&  0.79377E+00_JPRB, 0.74536E+00_JPRB, 0.71487E+00_JPRB, 0.71285E+00_JPRB, 0.70557E+00_JPRB, 0.69824E+00_JPRB, 0.68468E+00_JPRB ,&
&  0.64469E+00_JPRB, 0.60883E+00_JPRB, 0.58184E+00_JPRB, 0.56323E+00_JPRB, 0.55155E+00_JPRB, 0.54251E+00_JPRB, 0.53306E+00_JPRB ,&
&  0.52489E+00_JPRB, 0.49782E+00_JPRB, 0.47007E+00_JPRB, 0.46433E+00_JPRB, 0.47033E+00_JPRB, 0.47720E+00_JPRB, 0.50392E+00_JPRB ,&
&  0.52881E+00_JPRB, 0.52711E+00_JPRB, 0.52328E+00_JPRB, 0.52551E+00_JPRB, 0.53419E+00_JPRB, 0.53436E+00_JPRB, 0.52430E+00_JPRB ,&
&  0.51935E+00_JPRB, 0.51702E+00_JPRB, 0.50774E+00_JPRB, 0.50097E+00_JPRB, 0.50048E+00_JPRB, 0.50473E+00_JPRB, 0.51940E+00_JPRB ,&
&  0.52841E+00_JPRB, 0.52973E+00_JPRB, 0.54327E+00_JPRB, 0.56048E+00_JPRB, 0.57085E+00_JPRB, 0.57901E+00_JPRB, 0.58480E+00_JPRB ,&
&  0.59827E+00_JPRB, 0.61786E+00_JPRB, 0.63294E+00_JPRB, 0.63502E+00_JPRB, 0.61209E+00_JPRB, 0.58787E+00_JPRB, 0.56886E+00_JPRB ,&
&  0.56073E+00_JPRB, 0.56534E+00_JPRB, 0.57295E+00_JPRB, 0.58028E+00_JPRB, 0.58697E+00_JPRB, 0.59138E+00_JPRB, 0.59057E+00_JPRB ,&
&  0.61009E+00_JPRB, 0.64131E+00_JPRB, 0.66666E+00_JPRB, 0.69709E+00_JPRB, 0.72888E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.73080E+00_JPRB, 0.74523E+00_JPRB, 0.75692E+00_JPRB, 0.77815E+00_JPRB, 0.79744E+00_JPRB, 0.79476E+00_JPRB, 0.78781E+00_JPRB ,&
&  0.78284E+00_JPRB, 0.77811E+00_JPRB, 0.77063E+00_JPRB, 0.76072E+00_JPRB, 0.75400E+00_JPRB, 0.75332E+00_JPRB, 0.75400E+00_JPRB ,&
&  0.75578E+00_JPRB, 0.77598E+00_JPRB, 0.80904E+00_JPRB, 0.83491E+00_JPRB, 0.86312E+00_JPRB, 0.90649E+00_JPRB, 0.92701E+00_JPRB ,&
&  0.91062E+00_JPRB, 0.90978E+00_JPRB, 0.89898E+00_JPRB, 0.88443E+00_JPRB, 0.88623E+00_JPRB, 0.87690E+00_JPRB, 0.84877E+00_JPRB ,&
&  0.84251E+00_JPRB, 0.83537E+00_JPRB, 0.83163E+00_JPRB, 0.85962E+00_JPRB, 0.88145E+00_JPRB, 0.88154E+00_JPRB, 0.87883E+00_JPRB ,&
&  0.85745E+00_JPRB, 0.82014E+00_JPRB, 0.78458E+00_JPRB, 0.76747E+00_JPRB, 0.76545E+00_JPRB, 0.74538E+00_JPRB, 0.71732E+00_JPRB ,&
&  0.65986E+00_JPRB, 0.61269E+00_JPRB, 0.58658E+00_JPRB, 0.57110E+00_JPRB, 0.56613E+00_JPRB, 0.56352E+00_JPRB, 0.55917E+00_JPRB ,&
&  0.56461E+00_JPRB, 0.55958E+00_JPRB, 0.53210E+00_JPRB, 0.50344E+00_JPRB, 0.49885E+00_JPRB, 0.50760E+00_JPRB, 0.52685E+00_JPRB ,&
&  0.54184E+00_JPRB, 0.53745E+00_JPRB, 0.54415E+00_JPRB, 0.55143E+00_JPRB, 0.56813E+00_JPRB, 0.58443E+00_JPRB, 0.58545E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.58014E+00_JPRB, 0.56707E+00_JPRB, 0.55868E+00_JPRB, 0.54775E+00_JPRB, 0.54376E+00_JPRB, 0.55559E+00_JPRB ,&
&  0.57313E+00_JPRB, 0.58284E+00_JPRB, 0.59377E+00_JPRB, 0.60720E+00_JPRB, 0.61786E+00_JPRB, 0.62619E+00_JPRB, 0.63300E+00_JPRB ,&
&  0.64330E+00_JPRB, 0.65797E+00_JPRB, 0.67273E+00_JPRB, 0.68561E+00_JPRB, 0.68567E+00_JPRB, 0.67170E+00_JPRB, 0.64646E+00_JPRB ,&
&  0.62944E+00_JPRB, 0.61601E+00_JPRB, 0.61263E+00_JPRB, 0.61780E+00_JPRB, 0.61303E+00_JPRB, 0.59376E+00_JPRB, 0.58426E+00_JPRB ,&
&  0.59516E+00_JPRB, 0.61895E+00_JPRB, 0.64183E+00_JPRB, 0.67322E+00_JPRB, 0.70642E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.68993E+00_JPRB, 0.70200E+00_JPRB, 0.71545E+00_JPRB, 0.72836E+00_JPRB, 0.74188E+00_JPRB, 0.74171E+00_JPRB, 0.74316E+00_JPRB ,&
&  0.76222E+00_JPRB, 0.76484E+00_JPRB, 0.76366E+00_JPRB, 0.75451E+00_JPRB, 0.74779E+00_JPRB, 0.74748E+00_JPRB, 0.73852E+00_JPRB ,&
&  0.73421E+00_JPRB, 0.74084E+00_JPRB, 0.75943E+00_JPRB, 0.78778E+00_JPRB, 0.81544E+00_JPRB, 0.85165E+00_JPRB, 0.87836E+00_JPRB ,&
&  0.88266E+00_JPRB, 0.87194E+00_JPRB, 0.87208E+00_JPRB, 0.87531E+00_JPRB, 0.87551E+00_JPRB, 0.85858E+00_JPRB, 0.85404E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.83995E+00_JPRB, 0.71091E+00_JPRB, 0.78084E+00_JPRB, 0.82469E+00_JPRB, 0.85950E+00_JPRB, 0.87129E+00_JPRB ,&
&  0.86443E+00_JPRB, 0.85150E+00_JPRB, 0.82735E+00_JPRB, 0.80048E+00_JPRB, 0.79095E+00_JPRB, 0.77061E+00_JPRB, 0.74726E+00_JPRB ,&
&  0.69840E+00_JPRB, 0.63622E+00_JPRB, 0.60446E+00_JPRB, 0.59295E+00_JPRB, 0.58747E+00_JPRB, 0.58883E+00_JPRB, 0.59415E+00_JPRB ,&
&  0.59793E+00_JPRB, 0.59858E+00_JPRB, 0.58764E+00_JPRB, 0.57133E+00_JPRB, 0.55213E+00_JPRB, 0.54195E+00_JPRB, 0.53907E+00_JPRB ,&
&  0.53537E+00_JPRB, 0.52274E+00_JPRB, 0.52593E+00_JPRB, 0.55537E+00_JPRB, 0.57975E+00_JPRB, 0.59644E+00_JPRB, 0.60316E+00_JPRB ,&
&  0.60786E+00_JPRB, 0.60134E+00_JPRB, 0.59735E+00_JPRB, 0.58860E+00_JPRB, 0.56597E+00_JPRB, 0.56026E+00_JPRB, 0.58146E+00_JPRB ,&
&  0.60515E+00_JPRB, 0.61868E+00_JPRB, 0.62671E+00_JPRB, 0.63862E+00_JPRB, 0.65103E+00_JPRB, 0.65807E+00_JPRB, 0.66442E+00_JPRB ,&
&  0.66930E+00_JPRB, 0.67406E+00_JPRB, 0.68757E+00_JPRB, 0.70235E+00_JPRB, 0.70934E+00_JPRB, 0.71055E+00_JPRB, 0.71026E+00_JPRB ,&
&  0.70698E+00_JPRB, 0.69907E+00_JPRB, 0.69001E+00_JPRB, 0.68134E+00_JPRB, 0.66213E+00_JPRB, 0.63851E+00_JPRB, 0.62115E+00_JPRB ,&
&  0.61242E+00_JPRB, 0.62282E+00_JPRB, 0.63122E+00_JPRB, 0.64438E+00_JPRB, 0.67195E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.70053E+00_JPRB, 0.69789E+00_JPRB, 0.69760E+00_JPRB, 0.70919E+00_JPRB, 0.72681E+00_JPRB, 0.73009E+00_JPRB, 0.73057E+00_JPRB ,&
&  0.74177E+00_JPRB, 0.74990E+00_JPRB, 0.74870E+00_JPRB, 0.74680E+00_JPRB, 0.74068E+00_JPRB, 0.72689E+00_JPRB, 0.71640E+00_JPRB ,&
&  0.72186E+00_JPRB, 0.72777E+00_JPRB, 0.73048E+00_JPRB, 0.75486E+00_JPRB, 0.78701E+00_JPRB, 0.82490E+00_JPRB, 0.85061E+00_JPRB ,&
&  0.85978E+00_JPRB, 0.85250E+00_JPRB, 0.84880E+00_JPRB, 0.84348E+00_JPRB, 0.83814E+00_JPRB, 0.83984E+00_JPRB, 0.83432E+00_JPRB ,&
&  0.82430E+00_JPRB, 0.73051E+00_JPRB, 0.76065E+00_JPRB, 0.77031E+00_JPRB, 0.78662E+00_JPRB, 0.81241E+00_JPRB, 0.84361E+00_JPRB ,&
&  0.84541E+00_JPRB, 0.84015E+00_JPRB, 0.82885E+00_JPRB, 0.80754E+00_JPRB, 0.78985E+00_JPRB, 0.77962E+00_JPRB, 0.76243E+00_JPRB ,&
&  0.73467E+00_JPRB, 0.69736E+00_JPRB, 0.66423E+00_JPRB, 0.64171E+00_JPRB, 0.63161E+00_JPRB, 0.62912E+00_JPRB, 0.62473E+00_JPRB ,&
&  0.62290E+00_JPRB, 0.62155E+00_JPRB, 0.61111E+00_JPRB, 0.60172E+00_JPRB, 0.59217E+00_JPRB, 0.58606E+00_JPRB, 0.57335E+00_JPRB ,&
&  0.56521E+00_JPRB, 0.55439E+00_JPRB, 0.55361E+00_JPRB, 0.57302E+00_JPRB, 0.59290E+00_JPRB, 0.59626E+00_JPRB, 0.60811E+00_JPRB ,&
&  0.62401E+00_JPRB, 0.63024E+00_JPRB, 0.63125E+00_JPRB, 0.62823E+00_JPRB, 0.62097E+00_JPRB, 0.62066E+00_JPRB, 0.62843E+00_JPRB ,&
&  0.63512E+00_JPRB, 0.64482E+00_JPRB, 0.64791E+00_JPRB, 0.65226E+00_JPRB, 0.65871E+00_JPRB, 0.66359E+00_JPRB, 0.67457E+00_JPRB ,&
&  0.68025E+00_JPRB, 0.68446E+00_JPRB, 0.69611E+00_JPRB, 0.70761E+00_JPRB, 0.71226E+00_JPRB, 0.71438E+00_JPRB, 0.72302E+00_JPRB ,&
&  0.72598E+00_JPRB, 0.72369E+00_JPRB, 0.71569E+00_JPRB, 0.70425E+00_JPRB, 0.69739E+00_JPRB, 0.68567E+00_JPRB, 0.66690E+00_JPRB ,&
&  0.65021E+00_JPRB, 0.64885E+00_JPRB, 0.65945E+00_JPRB, 0.67106E+00_JPRB, 0.68979E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.71185E+00_JPRB, 0.70503E+00_JPRB, 0.69968E+00_JPRB, 0.70797E+00_JPRB, 0.72222E+00_JPRB, 0.73173E+00_JPRB, 0.72983E+00_JPRB ,&
&  0.72796E+00_JPRB, 0.73133E+00_JPRB, 0.73888E+00_JPRB, 0.74115E+00_JPRB, 0.73465E+00_JPRB, 0.71863E+00_JPRB, 0.71073E+00_JPRB ,&
&  0.70915E+00_JPRB, 0.71069E+00_JPRB, 0.71113E+00_JPRB, 0.71592E+00_JPRB, 0.73597E+00_JPRB, 0.76698E+00_JPRB, 0.79634E+00_JPRB ,&
&  0.81100E+00_JPRB, 0.80582E+00_JPRB, 0.79975E+00_JPRB, 0.79470E+00_JPRB, 0.79345E+00_JPRB, 0.79820E+00_JPRB, 0.79910E+00_JPRB ,&
&  0.79070E+00_JPRB, 0.64509E+00_JPRB, 0.70012E+00_JPRB, 0.72065E+00_JPRB, 0.74578E+00_JPRB, 0.77887E+00_JPRB, 0.82059E+00_JPRB ,&
&  0.82908E+00_JPRB, 0.81813E+00_JPRB, 0.79711E+00_JPRB, 0.78139E+00_JPRB, 0.76491E+00_JPRB, 0.75943E+00_JPRB, 0.75602E+00_JPRB ,&
&  0.75371E+00_JPRB, 0.75021E+00_JPRB, 0.72783E+00_JPRB, 0.69224E+00_JPRB, 0.67261E+00_JPRB, 0.66061E+00_JPRB, 0.65058E+00_JPRB ,&
&  0.64136E+00_JPRB, 0.62901E+00_JPRB, 0.61692E+00_JPRB, 0.60691E+00_JPRB, 0.60856E+00_JPRB, 0.60706E+00_JPRB, 0.59483E+00_JPRB ,&
&  0.58993E+00_JPRB, 0.58795E+00_JPRB, 0.59534E+00_JPRB, 0.60506E+00_JPRB, 0.60834E+00_JPRB, 0.60242E+00_JPRB, 0.61029E+00_JPRB ,&
&  0.62950E+00_JPRB, 0.64141E+00_JPRB, 0.65204E+00_JPRB, 0.66628E+00_JPRB, 0.67331E+00_JPRB, 0.67235E+00_JPRB, 0.67292E+00_JPRB ,&
&  0.67855E+00_JPRB, 0.67675E+00_JPRB, 0.66915E+00_JPRB, 0.66307E+00_JPRB, 0.66430E+00_JPRB, 0.66507E+00_JPRB, 0.67509E+00_JPRB ,&
&  0.68444E+00_JPRB, 0.68745E+00_JPRB, 0.68343E+00_JPRB, 0.68606E+00_JPRB, 0.69165E+00_JPRB, 0.70094E+00_JPRB, 0.70719E+00_JPRB ,&
&  0.71385E+00_JPRB, 0.71177E+00_JPRB, 0.70020E+00_JPRB, 0.70387E+00_JPRB, 0.71652E+00_JPRB, 0.71727E+00_JPRB, 0.70684E+00_JPRB ,&
&  0.69727E+00_JPRB, 0.68892E+00_JPRB, 0.69273E+00_JPRB, 0.69628E+00_JPRB, 0.70465E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.76170E+00_JPRB, 0.75757E+00_JPRB, 0.74942E+00_JPRB, 0.74871E+00_JPRB, 0.74753E+00_JPRB, 0.74173E+00_JPRB, 0.73741E+00_JPRB ,&
&  0.73318E+00_JPRB, 0.72877E+00_JPRB, 0.73662E+00_JPRB, 0.74431E+00_JPRB, 0.74567E+00_JPRB, 0.73998E+00_JPRB, 0.73438E+00_JPRB ,&
&  0.73057E+00_JPRB, 0.72319E+00_JPRB, 0.71531E+00_JPRB, 0.71225E+00_JPRB, 0.71511E+00_JPRB, 0.72943E+00_JPRB, 0.75117E+00_JPRB ,&
&  0.76748E+00_JPRB, 0.76715E+00_JPRB, 0.77098E+00_JPRB, 0.78212E+00_JPRB, 0.78462E+00_JPRB, 0.78609E+00_JPRB, 0.79301E+00_JPRB ,&
&  0.67423E+00_JPRB, 0.54990E+00_JPRB, 0.61314E+00_JPRB, 0.67076E+00_JPRB, 0.72100E+00_JPRB, 0.77978E+00_JPRB, 0.84687E+00_JPRB ,&
&  0.86873E+00_JPRB, 0.85429E+00_JPRB, 0.83753E+00_JPRB, 0.81638E+00_JPRB, 0.79142E+00_JPRB, 0.77460E+00_JPRB, 0.76129E+00_JPRB ,&
&  0.75781E+00_JPRB, 0.74635E+00_JPRB, 0.72882E+00_JPRB, 0.71439E+00_JPRB, 0.70229E+00_JPRB, 0.68690E+00_JPRB, 0.67206E+00_JPRB ,&
&  0.65685E+00_JPRB, 0.64393E+00_JPRB, 0.64015E+00_JPRB, 0.63842E+00_JPRB, 0.64113E+00_JPRB, 0.64148E+00_JPRB, 0.63707E+00_JPRB ,&
&  0.63371E+00_JPRB, 0.63516E+00_JPRB, 0.64370E+00_JPRB, 0.65093E+00_JPRB, 0.65228E+00_JPRB, 0.64926E+00_JPRB, 0.65628E+00_JPRB ,&
&  0.66026E+00_JPRB, 0.66971E+00_JPRB, 0.68222E+00_JPRB, 0.70498E+00_JPRB, 0.72223E+00_JPRB, 0.72588E+00_JPRB, 0.72705E+00_JPRB ,&
&  0.73130E+00_JPRB, 0.72548E+00_JPRB, 0.71025E+00_JPRB, 0.69967E+00_JPRB, 0.69449E+00_JPRB, 0.68728E+00_JPRB, 0.68487E+00_JPRB ,&
&  0.68967E+00_JPRB, 0.69187E+00_JPRB, 0.68736E+00_JPRB, 0.69015E+00_JPRB, 0.69971E+00_JPRB, 0.71338E+00_JPRB, 0.72610E+00_JPRB ,&
&  0.71783E+00_JPRB, 0.72650E+00_JPRB, 0.72761E+00_JPRB, 0.74651E+00_JPRB, 0.77180E+00_JPRB, 0.77162E+00_JPRB, 0.76829E+00_JPRB ,&
&  0.76323E+00_JPRB, 0.73978E+00_JPRB, 0.74233E+00_JPRB, 0.74477E+00_JPRB, 0.74998E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.77298E+00_JPRB, 0.80137E+00_JPRB, 0.80596E+00_JPRB, 0.78835E+00_JPRB, 0.75933E+00_JPRB, 0.74217E+00_JPRB, 0.73212E+00_JPRB ,&
&  0.72624E+00_JPRB, 0.72199E+00_JPRB, 0.73091E+00_JPRB, 0.73675E+00_JPRB, 0.73756E+00_JPRB, 0.72861E+00_JPRB, 0.71973E+00_JPRB ,&
&  0.70657E+00_JPRB, 0.69071E+00_JPRB, 0.67938E+00_JPRB, 0.66953E+00_JPRB, 0.66897E+00_JPRB, 0.67760E+00_JPRB, 0.69439E+00_JPRB ,&
&  0.71574E+00_JPRB, 0.73067E+00_JPRB, 0.74276E+00_JPRB, 0.75531E+00_JPRB, 0.75767E+00_JPRB, 0.75781E+00_JPRB, 0.76104E+00_JPRB ,&
&  0.76746E+00_JPRB, 0.58286E+00_JPRB, 0.62594E+00_JPRB, 0.69047E+00_JPRB, 0.76078E+00_JPRB, 0.85888E+00_JPRB, 0.94115E+00_JPRB ,&
&  0.96921E+00_JPRB, 0.93566E+00_JPRB, 0.90734E+00_JPRB, 0.87328E+00_JPRB, 0.83557E+00_JPRB, 0.80185E+00_JPRB, 0.76499E+00_JPRB ,&
&  0.73702E+00_JPRB, 0.71730E+00_JPRB, 0.70784E+00_JPRB, 0.69933E+00_JPRB, 0.68995E+00_JPRB, 0.67798E+00_JPRB, 0.66422E+00_JPRB ,&
&  0.65049E+00_JPRB, 0.64324E+00_JPRB, 0.63892E+00_JPRB, 0.63542E+00_JPRB, 0.63725E+00_JPRB, 0.64202E+00_JPRB, 0.64622E+00_JPRB ,&
&  0.65499E+00_JPRB, 0.66175E+00_JPRB, 0.66298E+00_JPRB, 0.66079E+00_JPRB, 0.65788E+00_JPRB, 0.65827E+00_JPRB, 0.67165E+00_JPRB ,&
&  0.67845E+00_JPRB, 0.69142E+00_JPRB, 0.70739E+00_JPRB, 0.73684E+00_JPRB, 0.76272E+00_JPRB, 0.77096E+00_JPRB, 0.77213E+00_JPRB ,&
&  0.77202E+00_JPRB, 0.76366E+00_JPRB, 0.74880E+00_JPRB, 0.73261E+00_JPRB, 0.72123E+00_JPRB, 0.71783E+00_JPRB, 0.71110E+00_JPRB ,&
&  0.71107E+00_JPRB, 0.71587E+00_JPRB, 0.71896E+00_JPRB, 0.73060E+00_JPRB, 0.74923E+00_JPRB, 0.76103E+00_JPRB, 0.75609E+00_JPRB ,&
&  0.75436E+00_JPRB, 0.75904E+00_JPRB, 0.76025E+00_JPRB, 0.80939E+00_JPRB, 0.83239E+00_JPRB, 0.84287E+00_JPRB, 0.83725E+00_JPRB ,&
&  0.82880E+00_JPRB, 0.81954E+00_JPRB, 0.61475E+00_JPRB, 0.71555E+00_JPRB, 0.74036E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.82480E+00_JPRB, 0.85363E+00_JPRB, 0.84729E+00_JPRB, 0.80711E+00_JPRB, 0.76562E+00_JPRB, 0.73789E+00_JPRB, 0.72014E+00_JPRB ,&
&  0.71335E+00_JPRB, 0.70446E+00_JPRB, 0.70459E+00_JPRB, 0.71406E+00_JPRB, 0.72478E+00_JPRB, 0.72659E+00_JPRB, 0.70938E+00_JPRB ,&
&  0.68813E+00_JPRB, 0.67224E+00_JPRB, 0.66533E+00_JPRB, 0.66334E+00_JPRB, 0.65970E+00_JPRB, 0.65790E+00_JPRB, 0.66663E+00_JPRB ,&
&  0.67687E+00_JPRB, 0.68908E+00_JPRB, 0.70209E+00_JPRB, 0.71270E+00_JPRB, 0.72068E+00_JPRB, 0.72820E+00_JPRB, 0.74735E+00_JPRB ,&
&  0.75792E+00_JPRB, 0.58662E+00_JPRB, 0.61897E+00_JPRB, 0.76000E+00_JPRB, 0.89912E+00_JPRB, 0.10520E+01_JPRB, 0.11237E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.10765E+01_JPRB, 0.10230E+01_JPRB, 0.96313E+00_JPRB, 0.89989E+00_JPRB, 0.84402E+00_JPRB, 0.79187E+00_JPRB ,&
&  0.75553E+00_JPRB, 0.72811E+00_JPRB, 0.71118E+00_JPRB, 0.69274E+00_JPRB, 0.68096E+00_JPRB, 0.67274E+00_JPRB, 0.66544E+00_JPRB ,&
&  0.65972E+00_JPRB, 0.65273E+00_JPRB, 0.65325E+00_JPRB, 0.65817E+00_JPRB, 0.65775E+00_JPRB, 0.66410E+00_JPRB, 0.66453E+00_JPRB ,&
&  0.67581E+00_JPRB, 0.69184E+00_JPRB, 0.69638E+00_JPRB, 0.69390E+00_JPRB, 0.69535E+00_JPRB, 0.70548E+00_JPRB, 0.72053E+00_JPRB ,&
&  0.73354E+00_JPRB, 0.76292E+00_JPRB, 0.79089E+00_JPRB, 0.82020E+00_JPRB, 0.83853E+00_JPRB, 0.83880E+00_JPRB, 0.82675E+00_JPRB ,&
&  0.82767E+00_JPRB, 0.82030E+00_JPRB, 0.80964E+00_JPRB, 0.79548E+00_JPRB, 0.77956E+00_JPRB, 0.76779E+00_JPRB, 0.76288E+00_JPRB ,&
&  0.76500E+00_JPRB, 0.77610E+00_JPRB, 0.78515E+00_JPRB, 0.80621E+00_JPRB, 0.81911E+00_JPRB, 0.81996E+00_JPRB, 0.82193E+00_JPRB ,&
&  0.82445E+00_JPRB, 0.82479E+00_JPRB, 0.82177E+00_JPRB, 0.86232E+00_JPRB, 0.93167E+00_JPRB, 0.93991E+00_JPRB, 0.92872E+00_JPRB ,&
&  0.90899E+00_JPRB, 0.86849E+00_JPRB, 0.84997E+00_JPRB, 0.69539E+00_JPRB, 0.78374E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.95780E+00_JPRB, 0.93712E+00_JPRB, 0.88643E+00_JPRB, 0.82681E+00_JPRB, 0.77359E+00_JPRB, 0.73080E+00_JPRB, 0.70563E+00_JPRB ,&
&  0.68534E+00_JPRB, 0.67060E+00_JPRB, 0.67213E+00_JPRB, 0.68188E+00_JPRB, 0.69346E+00_JPRB, 0.69443E+00_JPRB, 0.68642E+00_JPRB ,&
&  0.67474E+00_JPRB, 0.65956E+00_JPRB, 0.65082E+00_JPRB, 0.64614E+00_JPRB, 0.63794E+00_JPRB, 0.63436E+00_JPRB, 0.63259E+00_JPRB ,&
&  0.63545E+00_JPRB, 0.64010E+00_JPRB, 0.64251E+00_JPRB, 0.65104E+00_JPRB, 0.66583E+00_JPRB, 0.68885E+00_JPRB, 0.71328E+00_JPRB ,&
&  0.71692E+00_JPRB, 0.61563E+00_JPRB, 0.70462E+00_JPRB, 0.91395E+00_JPRB, 0.11314E+01_JPRB, 0.12405E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.11304E+01_JPRB, 0.10613E+01_JPRB, 0.98491E+00_JPRB, 0.91343E+00_JPRB, 0.85275E+00_JPRB, 0.79802E+00_JPRB ,&
&  0.75627E+00_JPRB, 0.72687E+00_JPRB, 0.71010E+00_JPRB, 0.69840E+00_JPRB, 0.68892E+00_JPRB, 0.67086E+00_JPRB, 0.65928E+00_JPRB ,&
&  0.65731E+00_JPRB, 0.65750E+00_JPRB, 0.66017E+00_JPRB, 0.66365E+00_JPRB, 0.67778E+00_JPRB, 0.69966E+00_JPRB, 0.71508E+00_JPRB ,&
&  0.72447E+00_JPRB, 0.73893E+00_JPRB, 0.74673E+00_JPRB, 0.74372E+00_JPRB, 0.74267E+00_JPRB, 0.77302E+00_JPRB, 0.80049E+00_JPRB ,&
&  0.83517E+00_JPRB, 0.88512E+00_JPRB, 0.94451E+00_JPRB, 0.97916E+00_JPRB, 0.98217E+00_JPRB, 0.96290E+00_JPRB, 0.92770E+00_JPRB ,&
&  0.90580E+00_JPRB, 0.87965E+00_JPRB, 0.85480E+00_JPRB, 0.83720E+00_JPRB, 0.82550E+00_JPRB, 0.81842E+00_JPRB, 0.81131E+00_JPRB ,&
&  0.80540E+00_JPRB, 0.79601E+00_JPRB, 0.79557E+00_JPRB, 0.81442E+00_JPRB, 0.83615E+00_JPRB, 0.84727E+00_JPRB, 0.84850E+00_JPRB ,&
&  0.84447E+00_JPRB, 0.84723E+00_JPRB, 0.86152E+00_JPRB, 0.96633E+00_JPRB, 0.96785E+00_JPRB, 0.97857E+00_JPRB, 0.96812E+00_JPRB ,&
&  0.95589E+00_JPRB, 0.92908E+00_JPRB, 0.89331E+00_JPRB, 0.87053E+00_JPRB, 0.88647E+00_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.10362E+01_JPRB, 0.96685E+00_JPRB, 0.90192E+00_JPRB, 0.82788E+00_JPRB, 0.77221E+00_JPRB, 0.74574E+00_JPRB, 0.72469E+00_JPRB ,&
&  0.69937E+00_JPRB, 0.67668E+00_JPRB, 0.67138E+00_JPRB, 0.67335E+00_JPRB, 0.67269E+00_JPRB, 0.66782E+00_JPRB, 0.65553E+00_JPRB ,&
&  0.65031E+00_JPRB, 0.65447E+00_JPRB, 0.64952E+00_JPRB, 0.64010E+00_JPRB, 0.62959E+00_JPRB, 0.62034E+00_JPRB, 0.61300E+00_JPRB ,&
&  0.60860E+00_JPRB, 0.61117E+00_JPRB, 0.61288E+00_JPRB, 0.62325E+00_JPRB, 0.64238E+00_JPRB, 0.66576E+00_JPRB, 0.69625E+00_JPRB ,&
&  0.70010E+00_JPRB, 0.64658E+00_JPRB, 0.87800E+00_JPRB, 0.12151E+01_JPRB, 0.14919E+01_JPRB, 0.15124E+01_JPRB, 0.13754E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.11425E+01_JPRB, 0.10495E+01_JPRB, 0.97557E+00_JPRB, 0.91182E+00_JPRB, 0.86096E+00_JPRB, 0.82341E+00_JPRB ,&
&  0.78579E+00_JPRB, 0.75598E+00_JPRB, 0.72790E+00_JPRB, 0.70249E+00_JPRB, 0.69457E+00_JPRB, 0.68713E+00_JPRB, 0.67818E+00_JPRB ,&
&  0.67175E+00_JPRB, 0.67102E+00_JPRB, 0.67266E+00_JPRB, 0.67801E+00_JPRB, 0.69744E+00_JPRB, 0.75403E+00_JPRB, 0.83068E+00_JPRB ,&
&  0.86336E+00_JPRB, 0.88592E+00_JPRB, 0.88887E+00_JPRB, 0.88492E+00_JPRB, 0.89803E+00_JPRB, 0.92659E+00_JPRB, 0.97661E+00_JPRB ,&
&  0.10377E+01_JPRB, 0.10821E+01_JPRB, 0.11114E+01_JPRB, 0.11312E+01_JPRB, 0.11254E+01_JPRB, 0.11068E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10467E+01_JPRB, 0.10025E+01_JPRB, 0.96328E+00_JPRB, 0.93390E+00_JPRB, 0.91069E+00_JPRB, 0.89347E+00_JPRB, 0.87176E+00_JPRB ,&
&  0.84541E+00_JPRB, 0.81840E+00_JPRB, 0.81831E+00_JPRB, 0.83013E+00_JPRB, 0.83895E+00_JPRB, 0.83870E+00_JPRB, 0.85005E+00_JPRB ,&
&  0.86247E+00_JPRB, 0.90175E+00_JPRB, 0.92790E+00_JPRB, 0.11279E+01_JPRB, 0.11732E+01_JPRB, 0.11387E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.10527E+01_JPRB, 0.10005E+01_JPRB, 0.94625E+00_JPRB, 0.92116E+00_JPRB, 0.10430E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.90497E+00_JPRB, 0.84670E+00_JPRB, 0.78083E+00_JPRB, 0.73621E+00_JPRB, 0.70005E+00_JPRB, 0.67221E+00_JPRB, 0.66476E+00_JPRB ,&
&  0.65947E+00_JPRB, 0.64981E+00_JPRB, 0.65436E+00_JPRB, 0.65096E+00_JPRB, 0.64432E+00_JPRB, 0.63180E+00_JPRB, 0.63649E+00_JPRB ,&
&  0.64151E+00_JPRB, 0.63677E+00_JPRB, 0.63055E+00_JPRB, 0.62262E+00_JPRB, 0.61379E+00_JPRB, 0.60119E+00_JPRB, 0.59273E+00_JPRB ,&
&  0.58650E+00_JPRB, 0.58701E+00_JPRB, 0.59395E+00_JPRB, 0.60737E+00_JPRB, 0.63170E+00_JPRB, 0.66280E+00_JPRB, 0.69374E+00_JPRB ,&
&  0.69114E+00_JPRB, 0.83474E+00_JPRB, 0.11966E+01_JPRB, 0.18272E+01_JPRB, 0.20412E+01_JPRB, 0.17165E+01_JPRB, 0.14369E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.11140E+01_JPRB, 0.10268E+01_JPRB, 0.96580E+00_JPRB, 0.90383E+00_JPRB, 0.86878E+00_JPRB, 0.84025E+00_JPRB ,&
&  0.81191E+00_JPRB, 0.77787E+00_JPRB, 0.73735E+00_JPRB, 0.70890E+00_JPRB, 0.69691E+00_JPRB, 0.69650E+00_JPRB, 0.69180E+00_JPRB ,&
&  0.68734E+00_JPRB, 0.69594E+00_JPRB, 0.70331E+00_JPRB, 0.71661E+00_JPRB, 0.83234E+00_JPRB, 0.97200E+00_JPRB, 0.12191E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.11718E+01_JPRB, 0.11661E+01_JPRB, 0.11597E+01_JPRB, 0.11771E+01_JPRB, 0.11873E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.12193E+01_JPRB, 0.11872E+01_JPRB, 0.11618E+01_JPRB, 0.11334E+01_JPRB, 0.11134E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.98696E+00_JPRB, 0.96554E+00_JPRB, 0.93459E+00_JPRB, 0.92130E+00_JPRB, 0.90145E+00_JPRB, 0.87637E+00_JPRB ,&
&  0.83920E+00_JPRB, 0.81224E+00_JPRB, 0.83910E+00_JPRB, 0.85156E+00_JPRB, 0.85096E+00_JPRB, 0.85064E+00_JPRB, 0.87064E+00_JPRB ,&
&  0.90850E+00_JPRB, 0.99569E+00_JPRB, 0.10480E+01_JPRB, 0.11708E+01_JPRB, 0.12126E+01_JPRB, 0.12356E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.99992E+00_JPRB, 0.91451E+00_JPRB, 0.87581E+00_JPRB, 0.93018E+00_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.83305E+00_JPRB, 0.75058E+00_JPRB, 0.68663E+00_JPRB, 0.65002E+00_JPRB, 0.61926E+00_JPRB, 0.60511E+00_JPRB, 0.59881E+00_JPRB ,&
&  0.60921E+00_JPRB, 0.62000E+00_JPRB, 0.63562E+00_JPRB, 0.63248E+00_JPRB, 0.61086E+00_JPRB, 0.60482E+00_JPRB, 0.61353E+00_JPRB ,&
&  0.60466E+00_JPRB, 0.60722E+00_JPRB, 0.61740E+00_JPRB, 0.61101E+00_JPRB, 0.60117E+00_JPRB, 0.58898E+00_JPRB, 0.57603E+00_JPRB ,&
&  0.57979E+00_JPRB, 0.58644E+00_JPRB, 0.60053E+00_JPRB, 0.62368E+00_JPRB, 0.65469E+00_JPRB, 0.68478E+00_JPRB, 0.70860E+00_JPRB ,&
&  0.74235E+00_JPRB, 0.11670E+01_JPRB, 0.17919E+01_JPRB, 0.22676E+01_JPRB, 0.21269E+01_JPRB, 0.16581E+01_JPRB, 0.14220E+01_JPRB ,&
&  0.12180E+01_JPRB, 0.11189E+01_JPRB, 0.10481E+01_JPRB, 0.98809E+00_JPRB, 0.92071E+00_JPRB, 0.88236E+00_JPRB, 0.86744E+00_JPRB ,&
&  0.84273E+00_JPRB, 0.80411E+00_JPRB, 0.76258E+00_JPRB, 0.72907E+00_JPRB, 0.71341E+00_JPRB, 0.71268E+00_JPRB, 0.71691E+00_JPRB ,&
&  0.72438E+00_JPRB, 0.73607E+00_JPRB, 0.75983E+00_JPRB, 0.80023E+00_JPRB, 0.98434E+00_JPRB, 0.10833E+01_JPRB, 0.16573E+01_JPRB ,&
&  0.20370E+01_JPRB, 0.18393E+01_JPRB, 0.17076E+01_JPRB, 0.16135E+01_JPRB, 0.15133E+01_JPRB, 0.14191E+01_JPRB, 0.13684E+01_JPRB ,&
&  0.13087E+01_JPRB, 0.12293E+01_JPRB, 0.11608E+01_JPRB, 0.11128E+01_JPRB, 0.10606E+01_JPRB, 0.10206E+01_JPRB, 0.96554E+00_JPRB ,&
&  0.94041E+00_JPRB, 0.91235E+00_JPRB, 0.89741E+00_JPRB, 0.90278E+00_JPRB, 0.90873E+00_JPRB, 0.88156E+00_JPRB, 0.85727E+00_JPRB ,&
&  0.83003E+00_JPRB, 0.84125E+00_JPRB, 0.88902E+00_JPRB, 0.85263E+00_JPRB, 0.85177E+00_JPRB, 0.88250E+00_JPRB, 0.90849E+00_JPRB ,&
&  0.91735E+00_JPRB, 0.10008E+01_JPRB, 0.10869E+01_JPRB, 0.11593E+01_JPRB, 0.11987E+01_JPRB, 0.12474E+01_JPRB, 0.11347E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.94214E+00_JPRB, 0.88340E+00_JPRB, 0.84444E+00_JPRB, 0.87228E+00_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.71989E+00_JPRB, 0.65992E+00_JPRB, 0.62510E+00_JPRB, 0.59933E+00_JPRB, 0.59932E+00_JPRB, 0.59254E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.60736E+00_JPRB, 0.61978E+00_JPRB, 0.63327E+00_JPRB, 0.62137E+00_JPRB, 0.59768E+00_JPRB, 0.58973E+00_JPRB, 0.57754E+00_JPRB ,&
&  0.56659E+00_JPRB, 0.59478E+00_JPRB, 0.62351E+00_JPRB, 0.63423E+00_JPRB, 0.62425E+00_JPRB, 0.59705E+00_JPRB, 0.58374E+00_JPRB ,&
&  0.59166E+00_JPRB, 0.61155E+00_JPRB, 0.64351E+00_JPRB, 0.67903E+00_JPRB, 0.70841E+00_JPRB, 0.73509E+00_JPRB, 0.77841E+00_JPRB ,&
&  0.92726E+00_JPRB, 0.12270E+01_JPRB, 0.19554E+01_JPRB, 0.25775E+01_JPRB, 0.20992E+01_JPRB, 0.16318E+01_JPRB, 0.13232E+01_JPRB ,&
&  0.12359E+01_JPRB, 0.11721E+01_JPRB, 0.11002E+01_JPRB, 0.10230E+01_JPRB, 0.95521E+00_JPRB, 0.90565E+00_JPRB, 0.88686E+00_JPRB ,&
&  0.85628E+00_JPRB, 0.83070E+00_JPRB, 0.80899E+00_JPRB, 0.79837E+00_JPRB, 0.79227E+00_JPRB, 0.78933E+00_JPRB, 0.78853E+00_JPRB ,&
&  0.79962E+00_JPRB, 0.81229E+00_JPRB, 0.84321E+00_JPRB, 0.93421E+00_JPRB, 0.11358E+01_JPRB, 0.14469E+01_JPRB, 0.19693E+01_JPRB ,&
&  0.23971E+01_JPRB, 0.23035E+01_JPRB, 0.20090E+01_JPRB, 0.17792E+01_JPRB, 0.16086E+01_JPRB, 0.15109E+01_JPRB, 0.14006E+01_JPRB ,&
&  0.13010E+01_JPRB, 0.11912E+01_JPRB, 0.10982E+01_JPRB, 0.10332E+01_JPRB, 0.95400E+00_JPRB, 0.91301E+00_JPRB, 0.90718E+00_JPRB ,&
&  0.88647E+00_JPRB, 0.88181E+00_JPRB, 0.87532E+00_JPRB, 0.89233E+00_JPRB, 0.87026E+00_JPRB, 0.84994E+00_JPRB, 0.83796E+00_JPRB ,&
&  0.84385E+00_JPRB, 0.90007E+00_JPRB, 0.92939E+00_JPRB, 0.85995E+00_JPRB, 0.83899E+00_JPRB, 0.85726E+00_JPRB, 0.89485E+00_JPRB ,&
&  0.95893E+00_JPRB, 0.10448E+01_JPRB, 0.10967E+01_JPRB, 0.11460E+01_JPRB, 0.10948E+01_JPRB, 0.11598E+01_JPRB, 0.10408E+01_JPRB ,&
&  0.94770E+00_JPRB, 0.89114E+00_JPRB, 0.86051E+00_JPRB, 0.82517E+00_JPRB, 0.79367E+00_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.69763E+00_JPRB, 0.65579E+00_JPRB, 0.60685E+00_JPRB, 0.58529E+00_JPRB, 0.59195E+00_JPRB, 0.57984E+00_JPRB, 0.58734E+00_JPRB ,&
&  0.60707E+00_JPRB, 0.60846E+00_JPRB, 0.60815E+00_JPRB, 0.59858E+00_JPRB, 0.58922E+00_JPRB, 0.57387E+00_JPRB, 0.56190E+00_JPRB ,&
&  0.57699E+00_JPRB, 0.61919E+00_JPRB, 0.63316E+00_JPRB, 0.62144E+00_JPRB, 0.61180E+00_JPRB, 0.59532E+00_JPRB, 0.59198E+00_JPRB ,&
&  0.61941E+00_JPRB, 0.67468E+00_JPRB, 0.73390E+00_JPRB, 0.76757E+00_JPRB, 0.78987E+00_JPRB, 0.81733E+00_JPRB, 0.94924E+00_JPRB ,&
&  0.13156E+01_JPRB, 0.32727E+01_JPRB, 0.14650E+01_JPRB, 0.26779E+01_JPRB, 0.19327E+01_JPRB, 0.15173E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.12644E+01_JPRB, 0.11545E+01_JPRB, 0.10662E+01_JPRB, 0.97907E+00_JPRB, 0.92081E+00_JPRB, 0.90354E+00_JPRB ,&
&  0.88819E+00_JPRB, 0.88342E+00_JPRB, 0.90115E+00_JPRB, 0.92037E+00_JPRB, 0.92238E+00_JPRB, 0.90788E+00_JPRB, 0.89723E+00_JPRB ,&
&  0.90001E+00_JPRB, 0.90504E+00_JPRB, 0.96595E+00_JPRB, 0.10795E+01_JPRB, 0.12745E+01_JPRB, 0.16023E+01_JPRB, 0.19970E+01_JPRB ,&
&  0.21762E+01_JPRB, 0.21643E+01_JPRB, 0.18474E+01_JPRB, 0.16389E+01_JPRB, 0.14785E+01_JPRB, 0.13859E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.10514E+01_JPRB, 0.98408E+00_JPRB, 0.93418E+00_JPRB, 0.88257E+00_JPRB, 0.86537E+00_JPRB, 0.84397E+00_JPRB ,&
&  0.84868E+00_JPRB, 0.88048E+00_JPRB, 0.87336E+00_JPRB, 0.86214E+00_JPRB, 0.84505E+00_JPRB, 0.85411E+00_JPRB, 0.86801E+00_JPRB ,&
&  0.89957E+00_JPRB, 0.96212E+00_JPRB, 0.94938E+00_JPRB, 0.84145E+00_JPRB, 0.81135E+00_JPRB, 0.84489E+00_JPRB, 0.88492E+00_JPRB ,&
&  0.98216E+00_JPRB, 0.10669E+01_JPRB, 0.10734E+01_JPRB, 0.10747E+01_JPRB, 0.10659E+01_JPRB, 0.11227E+01_JPRB, 0.97234E+00_JPRB ,&
&  0.89153E+00_JPRB, 0.85388E+00_JPRB, 0.82196E+00_JPRB, 0.77516E+00_JPRB, 0.75226E+00_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.69303E+00_JPRB, 0.66978E+00_JPRB, 0.62232E+00_JPRB, 0.58363E+00_JPRB, 0.58041E+00_JPRB, 0.58697E+00_JPRB, 0.59695E+00_JPRB ,&
&  0.59999E+00_JPRB, 0.60007E+00_JPRB, 0.59704E+00_JPRB, 0.59107E+00_JPRB, 0.59582E+00_JPRB, 0.58536E+00_JPRB, 0.60972E+00_JPRB ,&
&  0.63493E+00_JPRB, 0.63492E+00_JPRB, 0.64242E+00_JPRB, 0.61982E+00_JPRB, 0.61794E+00_JPRB, 0.61483E+00_JPRB, 0.64270E+00_JPRB ,&
&  0.70821E+00_JPRB, 0.78820E+00_JPRB, 0.83750E+00_JPRB, 0.85420E+00_JPRB, 0.87091E+00_JPRB, 0.92072E+00_JPRB, 0.11219E+01_JPRB ,&
&  0.16581E+01_JPRB, 0.34269E+01_JPRB, 0.16990E+01_JPRB, 0.24774E+01_JPRB, 0.16693E+01_JPRB, 0.13023E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.14616E+01_JPRB, 0.12344E+01_JPRB, 0.10820E+01_JPRB, 0.95652E+00_JPRB, 0.90898E+00_JPRB, 0.89742E+00_JPRB ,&
&  0.91850E+00_JPRB, 0.98777E+00_JPRB, 0.10516E+01_JPRB, 0.10822E+01_JPRB, 0.10642E+01_JPRB, 0.10279E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.10191E+01_JPRB, 0.11278E+01_JPRB, 0.12418E+01_JPRB, 0.15942E+01_JPRB, 0.17516E+01_JPRB, 0.18073E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.18275E+01_JPRB, 0.16289E+01_JPRB, 0.14749E+01_JPRB, 0.13089E+01_JPRB, 0.12405E+01_JPRB, 0.11139E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.97898E+00_JPRB, 0.92437E+00_JPRB, 0.86475E+00_JPRB, 0.83772E+00_JPRB, 0.83492E+00_JPRB, 0.81883E+00_JPRB ,&
&  0.84741E+00_JPRB, 0.88212E+00_JPRB, 0.87675E+00_JPRB, 0.87432E+00_JPRB, 0.88164E+00_JPRB, 0.90140E+00_JPRB, 0.92389E+00_JPRB ,&
&  0.95709E+00_JPRB, 0.10075E+01_JPRB, 0.95027E+00_JPRB, 0.83667E+00_JPRB, 0.83931E+00_JPRB, 0.87913E+00_JPRB, 0.90313E+00_JPRB ,&
&  0.98154E+00_JPRB, 0.10425E+01_JPRB, 0.10278E+01_JPRB, 0.98773E+00_JPRB, 0.98460E+00_JPRB, 0.10174E+01_JPRB, 0.91648E+00_JPRB ,&
&  0.86630E+00_JPRB, 0.83694E+00_JPRB, 0.81533E+00_JPRB, 0.79046E+00_JPRB, 0.74877E+00_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.64395E+00_JPRB, 0.62506E+00_JPRB, 0.60668E+00_JPRB, 0.59094E+00_JPRB, 0.58198E+00_JPRB, 0.58816E+00_JPRB, 0.59300E+00_JPRB ,&
&  0.58776E+00_JPRB, 0.58823E+00_JPRB, 0.58339E+00_JPRB, 0.58853E+00_JPRB, 0.61286E+00_JPRB, 0.66129E+00_JPRB, 0.71159E+00_JPRB ,&
&  0.71326E+00_JPRB, 0.68275E+00_JPRB, 0.67711E+00_JPRB, 0.67228E+00_JPRB, 0.69110E+00_JPRB, 0.71780E+00_JPRB, 0.76958E+00_JPRB ,&
&  0.84093E+00_JPRB, 0.89934E+00_JPRB, 0.92763E+00_JPRB, 0.93798E+00_JPRB, 0.96366E+00_JPRB, 0.10088E+01_JPRB, 0.13216E+01_JPRB ,&
&  0.19831E+01_JPRB, 0.39955E+01_JPRB, 0.29738E+01_JPRB, 0.19875E+01_JPRB, 0.13865E+01_JPRB, 0.12029E+01_JPRB, 0.12023E+01_JPRB ,&
&  0.13504E+01_JPRB, 0.12977E+01_JPRB, 0.12754E+01_JPRB, 0.11115E+01_JPRB, 0.10015E+01_JPRB, 0.95592E+00_JPRB, 0.98831E+00_JPRB ,&
&  0.10790E+01_JPRB, 0.11594E+01_JPRB, 0.12122E+01_JPRB, 0.12178E+01_JPRB, 0.11837E+01_JPRB, 0.11630E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11529E+01_JPRB, 0.12137E+01_JPRB, 0.14679E+01_JPRB, 0.16631E+01_JPRB, 0.17789E+01_JPRB, 0.17002E+01_JPRB, 0.16247E+01_JPRB ,&
&  0.16079E+01_JPRB, 0.15378E+01_JPRB, 0.14885E+01_JPRB, 0.13839E+01_JPRB, 0.11633E+01_JPRB, 0.11843E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10144E+01_JPRB, 0.96217E+00_JPRB, 0.89994E+00_JPRB, 0.82666E+00_JPRB, 0.83610E+00_JPRB, 0.84312E+00_JPRB, 0.82473E+00_JPRB ,&
&  0.85801E+00_JPRB, 0.89797E+00_JPRB, 0.88508E+00_JPRB, 0.89719E+00_JPRB, 0.90921E+00_JPRB, 0.93381E+00_JPRB, 0.95619E+00_JPRB ,&
&  0.97295E+00_JPRB, 0.10204E+01_JPRB, 0.97119E+00_JPRB, 0.91875E+00_JPRB, 0.91248E+00_JPRB, 0.90497E+00_JPRB, 0.91924E+00_JPRB ,&
&  0.97054E+00_JPRB, 0.10473E+01_JPRB, 0.10078E+01_JPRB, 0.10026E+01_JPRB, 0.95383E+00_JPRB, 0.94039E+00_JPRB, 0.88437E+00_JPRB ,&
&  0.87136E+00_JPRB, 0.84858E+00_JPRB, 0.86916E+00_JPRB, 0.79717E+00_JPRB, 0.70369E+00_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.58108E+00_JPRB, 0.56884E+00_JPRB, 0.56424E+00_JPRB, 0.57064E+00_JPRB, 0.57516E+00_JPRB, 0.56775E+00_JPRB, 0.57020E+00_JPRB ,&
&  0.58047E+00_JPRB, 0.59466E+00_JPRB, 0.59427E+00_JPRB, 0.62707E+00_JPRB, 0.70166E+00_JPRB, 0.78019E+00_JPRB, 0.78785E+00_JPRB ,&
&  0.77874E+00_JPRB, 0.76700E+00_JPRB, 0.76220E+00_JPRB, 0.77752E+00_JPRB, 0.79468E+00_JPRB, 0.84285E+00_JPRB, 0.89342E+00_JPRB ,&
&  0.94590E+00_JPRB, 0.10029E+01_JPRB, 0.10517E+01_JPRB, 0.10955E+01_JPRB, 0.11781E+01_JPRB, 0.13132E+01_JPRB, 0.16730E+01_JPRB ,&
&  0.23199E+01_JPRB, 0.41302E+01_JPRB, 0.12528E+01_JPRB, 0.13043E+01_JPRB, 0.12944E+01_JPRB, 0.11782E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10254E+01_JPRB, 0.93753E+00_JPRB, 0.10431E+01_JPRB, 0.11327E+01_JPRB, 0.10708E+01_JPRB, 0.10994E+01_JPRB, 0.11545E+01_JPRB ,&
&  0.12633E+01_JPRB, 0.13298E+01_JPRB, 0.13610E+01_JPRB, 0.13376E+01_JPRB, 0.13106E+01_JPRB, 0.13027E+01_JPRB, 0.13339E+01_JPRB ,&
&  0.14272E+01_JPRB, 0.15966E+01_JPRB, 0.17525E+01_JPRB, 0.16934E+01_JPRB, 0.17709E+01_JPRB, 0.16215E+01_JPRB, 0.15048E+01_JPRB ,&
&  0.13009E+01_JPRB, 0.12936E+01_JPRB, 0.13373E+01_JPRB, 0.13138E+01_JPRB, 0.11080E+01_JPRB, 0.10067E+01_JPRB, 0.10241E+01_JPRB ,&
&  0.98025E+00_JPRB, 0.90891E+00_JPRB, 0.83932E+00_JPRB, 0.78952E+00_JPRB, 0.79344E+00_JPRB, 0.81793E+00_JPRB, 0.80591E+00_JPRB ,&
&  0.83988E+00_JPRB, 0.88002E+00_JPRB, 0.86113E+00_JPRB, 0.89378E+00_JPRB, 0.91877E+00_JPRB, 0.93806E+00_JPRB, 0.95274E+00_JPRB ,&
&  0.97187E+00_JPRB, 0.10235E+01_JPRB, 0.10600E+01_JPRB, 0.10414E+01_JPRB, 0.98398E+00_JPRB, 0.95847E+00_JPRB, 0.99496E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.11049E+01_JPRB, 0.99404E+00_JPRB, 0.92782E+00_JPRB, 0.92326E+00_JPRB, 0.90587E+00_JPRB, 0.90300E+00_JPRB ,&
&  0.92844E+00_JPRB, 0.99591E+00_JPRB, 0.10327E+01_JPRB, 0.73943E+00_JPRB, 0.63380E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.54920E+00_JPRB, 0.53602E+00_JPRB, 0.52736E+00_JPRB, 0.54125E+00_JPRB, 0.54069E+00_JPRB, 0.54929E+00_JPRB, 0.56801E+00_JPRB ,&
&  0.59844E+00_JPRB, 0.63812E+00_JPRB, 0.67071E+00_JPRB, 0.71689E+00_JPRB, 0.78099E+00_JPRB, 0.84083E+00_JPRB, 0.84963E+00_JPRB ,&
&  0.83952E+00_JPRB, 0.83560E+00_JPRB, 0.84301E+00_JPRB, 0.85530E+00_JPRB, 0.88736E+00_JPRB, 0.92804E+00_JPRB, 0.97097E+00_JPRB ,&
&  0.10325E+01_JPRB, 0.11173E+01_JPRB, 0.12062E+01_JPRB, 0.13108E+01_JPRB, 0.14482E+01_JPRB, 0.16626E+01_JPRB, 0.19596E+01_JPRB ,&
&  0.27716E+01_JPRB, 0.27434E+01_JPRB, 0.10475E+01_JPRB, 0.13475E+01_JPRB, 0.12179E+01_JPRB, 0.10554E+01_JPRB, 0.85475E+00_JPRB ,&
&  0.79442E+00_JPRB, 0.91940E+00_JPRB, 0.10603E+01_JPRB, 0.11878E+01_JPRB, 0.11759E+01_JPRB, 0.12573E+01_JPRB, 0.13298E+01_JPRB ,&
&  0.14089E+01_JPRB, 0.14511E+01_JPRB, 0.14598E+01_JPRB, 0.14550E+01_JPRB, 0.14735E+01_JPRB, 0.15201E+01_JPRB, 0.16082E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.18397E+01_JPRB, 0.19003E+01_JPRB, 0.17103E+01_JPRB, 0.17665E+01_JPRB, 0.15946E+01_JPRB, 0.14787E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.11292E+01_JPRB, 0.11646E+01_JPRB, 0.11955E+01_JPRB, 0.10328E+01_JPRB, 0.91495E+00_JPRB, 0.97169E+00_JPRB ,&
&  0.90723E+00_JPRB, 0.82433E+00_JPRB, 0.79184E+00_JPRB, 0.77803E+00_JPRB, 0.77556E+00_JPRB, 0.77457E+00_JPRB, 0.78901E+00_JPRB ,&
&  0.81506E+00_JPRB, 0.81611E+00_JPRB, 0.79361E+00_JPRB, 0.81904E+00_JPRB, 0.84871E+00_JPRB, 0.88170E+00_JPRB, 0.91128E+00_JPRB ,&
&  0.95104E+00_JPRB, 0.99951E+00_JPRB, 0.10377E+01_JPRB, 0.10634E+01_JPRB, 0.10292E+01_JPRB, 0.98891E+00_JPRB, 0.10118E+01_JPRB ,&
&  0.95691E+00_JPRB, 0.94476E+00_JPRB, 0.92610E+00_JPRB, 0.90046E+00_JPRB, 0.85586E+00_JPRB, 0.87533E+00_JPRB, 0.88552E+00_JPRB ,&
&  0.92202E+00_JPRB, 0.95252E+00_JPRB, 0.86812E+00_JPRB, 0.66377E+00_JPRB, 0.58381E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.54874E+00_JPRB, 0.54828E+00_JPRB, 0.54039E+00_JPRB, 0.54013E+00_JPRB, 0.53330E+00_JPRB, 0.53972E+00_JPRB, 0.55430E+00_JPRB ,&
&  0.58576E+00_JPRB, 0.64095E+00_JPRB, 0.70949E+00_JPRB, 0.77144E+00_JPRB, 0.84460E+00_JPRB, 0.90188E+00_JPRB, 0.90858E+00_JPRB ,&
&  0.89500E+00_JPRB, 0.89422E+00_JPRB, 0.88743E+00_JPRB, 0.91590E+00_JPRB, 0.94411E+00_JPRB, 0.98590E+00_JPRB, 0.10366E+01_JPRB ,&
&  0.10980E+01_JPRB, 0.11772E+01_JPRB, 0.12763E+01_JPRB, 0.13933E+01_JPRB, 0.15518E+01_JPRB, 0.17261E+01_JPRB, 0.18928E+01_JPRB ,&
&  0.13158E+01_JPRB, 0.70486E+00_JPRB, 0.12085E+01_JPRB, 0.13313E+01_JPRB, 0.10367E+01_JPRB, 0.92373E+00_JPRB, 0.78514E+00_JPRB ,&
&  0.75386E+00_JPRB, 0.99565E+00_JPRB, 0.11235E+01_JPRB, 0.12425E+01_JPRB, 0.12797E+01_JPRB, 0.13760E+01_JPRB, 0.14482E+01_JPRB ,&
&  0.14858E+01_JPRB, 0.15197E+01_JPRB, 0.15579E+01_JPRB, 0.15980E+01_JPRB, 0.16580E+01_JPRB, 0.17236E+01_JPRB, 0.18103E+01_JPRB ,&
&  0.19127E+01_JPRB, 0.19506E+01_JPRB, 0.19707E+01_JPRB, 0.16833E+01_JPRB, 0.17956E+01_JPRB, 0.17510E+01_JPRB, 0.17530E+01_JPRB ,&
&  0.17029E+01_JPRB, 0.11225E+01_JPRB, 0.97854E+00_JPRB, 0.12091E+01_JPRB, 0.11081E+01_JPRB, 0.86184E+00_JPRB, 0.92045E+00_JPRB ,&
&  0.82877E+00_JPRB, 0.75065E+00_JPRB, 0.74227E+00_JPRB, 0.75481E+00_JPRB, 0.73404E+00_JPRB, 0.74368E+00_JPRB, 0.75349E+00_JPRB ,&
&  0.78130E+00_JPRB, 0.76523E+00_JPRB, 0.74252E+00_JPRB, 0.75772E+00_JPRB, 0.78607E+00_JPRB, 0.82518E+00_JPRB, 0.85550E+00_JPRB ,&
&  0.89326E+00_JPRB, 0.92547E+00_JPRB, 0.95806E+00_JPRB, 0.99789E+00_JPRB, 0.10314E+01_JPRB, 0.98963E+00_JPRB, 0.97206E+00_JPRB ,&
&  0.88692E+00_JPRB, 0.85480E+00_JPRB, 0.81925E+00_JPRB, 0.79370E+00_JPRB, 0.76469E+00_JPRB, 0.77772E+00_JPRB, 0.80692E+00_JPRB ,&
&  0.83490E+00_JPRB, 0.83851E+00_JPRB, 0.76688E+00_JPRB, 0.64168E+00_JPRB, 0.57097E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.53505E+00_JPRB, 0.53609E+00_JPRB, 0.53415E+00_JPRB, 0.54502E+00_JPRB, 0.54476E+00_JPRB, 0.54578E+00_JPRB, 0.54241E+00_JPRB ,&
&  0.56747E+00_JPRB, 0.61679E+00_JPRB, 0.69276E+00_JPRB, 0.77882E+00_JPRB, 0.84313E+00_JPRB, 0.92258E+00_JPRB, 0.96294E+00_JPRB ,&
&  0.96236E+00_JPRB, 0.94655E+00_JPRB, 0.94593E+00_JPRB, 0.97371E+00_JPRB, 0.99761E+00_JPRB, 0.10364E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11890E+01_JPRB, 0.12711E+01_JPRB, 0.13664E+01_JPRB, 0.14781E+01_JPRB, 0.15618E+01_JPRB, 0.16384E+01_JPRB ,&
&  0.65323E+00_JPRB, 0.85287E+00_JPRB, 0.11560E+01_JPRB, 0.12090E+01_JPRB, 0.94369E+00_JPRB, 0.84364E+00_JPRB, 0.77436E+00_JPRB ,&
&  0.90420E+00_JPRB, 0.10923E+01_JPRB, 0.12016E+01_JPRB, 0.13180E+01_JPRB, 0.13081E+01_JPRB, 0.14047E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15178E+01_JPRB, 0.15495E+01_JPRB, 0.16182E+01_JPRB, 0.17036E+01_JPRB, 0.17772E+01_JPRB, 0.18584E+01_JPRB, 0.19372E+01_JPRB ,&
&  0.20230E+01_JPRB, 0.20426E+01_JPRB, 0.20903E+01_JPRB, 0.17604E+01_JPRB, 0.18249E+01_JPRB, 0.19385E+01_JPRB, 0.22852E+01_JPRB ,&
&  0.20249E+01_JPRB, 0.12961E+01_JPRB, 0.10882E+01_JPRB, 0.12036E+01_JPRB, 0.11354E+01_JPRB, 0.10658E+01_JPRB, 0.93276E+00_JPRB ,&
&  0.81131E+00_JPRB, 0.72385E+00_JPRB, 0.68257E+00_JPRB, 0.67016E+00_JPRB, 0.67184E+00_JPRB, 0.70448E+00_JPRB, 0.73595E+00_JPRB ,&
&  0.72727E+00_JPRB, 0.71389E+00_JPRB, 0.70294E+00_JPRB, 0.70205E+00_JPRB, 0.73954E+00_JPRB, 0.78068E+00_JPRB, 0.80307E+00_JPRB ,&
&  0.80904E+00_JPRB, 0.81757E+00_JPRB, 0.84076E+00_JPRB, 0.86931E+00_JPRB, 0.90042E+00_JPRB, 0.91780E+00_JPRB, 0.92967E+00_JPRB ,&
&  0.82903E+00_JPRB, 0.77591E+00_JPRB, 0.75950E+00_JPRB, 0.75460E+00_JPRB, 0.76744E+00_JPRB, 0.80046E+00_JPRB, 0.75360E+00_JPRB ,&
&  0.72469E+00_JPRB, 0.71133E+00_JPRB, 0.67870E+00_JPRB, 0.61039E+00_JPRB, 0.54709E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.54824E+00_JPRB, 0.56221E+00_JPRB, 0.58225E+00_JPRB, 0.60709E+00_JPRB, 0.60512E+00_JPRB, 0.60693E+00_JPRB, 0.59477E+00_JPRB ,&
&  0.59300E+00_JPRB, 0.61245E+00_JPRB, 0.66074E+00_JPRB, 0.74179E+00_JPRB, 0.80269E+00_JPRB, 0.88689E+00_JPRB, 0.10004E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.10298E+01_JPRB, 0.10249E+01_JPRB, 0.10347E+01_JPRB, 0.10608E+01_JPRB, 0.10958E+01_JPRB, 0.11280E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.12377E+01_JPRB, 0.13032E+01_JPRB, 0.13922E+01_JPRB, 0.14829E+01_JPRB, 0.15456E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.92419E+00_JPRB, 0.11881E+01_JPRB, 0.11668E+01_JPRB, 0.11383E+01_JPRB, 0.98431E+00_JPRB, 0.89826E+00_JPRB, 0.92646E+00_JPRB ,&
&  0.10630E+01_JPRB, 0.11829E+01_JPRB, 0.13288E+01_JPRB, 0.11747E+01_JPRB, 0.12576E+01_JPRB, 0.13019E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.15370E+01_JPRB, 0.15973E+01_JPRB, 0.16656E+01_JPRB, 0.17315E+01_JPRB, 0.18243E+01_JPRB, 0.19128E+01_JPRB ,&
&  0.19778E+01_JPRB, 0.19924E+01_JPRB, 0.20228E+01_JPRB, 0.17981E+01_JPRB, 0.15771E+01_JPRB, 0.16271E+01_JPRB, 0.15931E+01_JPRB ,&
&  0.13764E+01_JPRB, 0.99058E+00_JPRB, 0.10035E+01_JPRB, 0.11468E+01_JPRB, 0.10965E+01_JPRB, 0.10202E+01_JPRB, 0.92405E+00_JPRB ,&
&  0.81672E+00_JPRB, 0.71718E+00_JPRB, 0.67027E+00_JPRB, 0.62857E+00_JPRB, 0.61004E+00_JPRB, 0.63159E+00_JPRB, 0.66684E+00_JPRB ,&
&  0.66523E+00_JPRB, 0.65339E+00_JPRB, 0.64729E+00_JPRB, 0.66287E+00_JPRB, 0.70670E+00_JPRB, 0.74600E+00_JPRB, 0.77683E+00_JPRB ,&
&  0.74833E+00_JPRB, 0.71606E+00_JPRB, 0.70484E+00_JPRB, 0.71831E+00_JPRB, 0.73991E+00_JPRB, 0.75911E+00_JPRB, 0.79171E+00_JPRB ,&
&  0.80713E+00_JPRB, 0.80675E+00_JPRB, 0.89616E+00_JPRB, 0.10895E+01_JPRB, 0.11983E+01_JPRB, 0.12794E+01_JPRB, 0.10596E+01_JPRB ,&
&  0.85059E+00_JPRB, 0.71867E+00_JPRB, 0.63490E+00_JPRB, 0.57968E+00_JPRB, 0.53756E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.54682E+00_JPRB, 0.59268E+00_JPRB, 0.63093E+00_JPRB, 0.68734E+00_JPRB, 0.70856E+00_JPRB, 0.71544E+00_JPRB, 0.71617E+00_JPRB ,&
&  0.70687E+00_JPRB, 0.71462E+00_JPRB, 0.73858E+00_JPRB, 0.79524E+00_JPRB, 0.85599E+00_JPRB, 0.91629E+00_JPRB, 0.98149E+00_JPRB ,&
&  0.10641E+01_JPRB, 0.11210E+01_JPRB, 0.11138E+01_JPRB, 0.11263E+01_JPRB, 0.11625E+01_JPRB, 0.12002E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.12522E+01_JPRB, 0.12908E+01_JPRB, 0.13570E+01_JPRB, 0.14453E+01_JPRB, 0.15356E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.12286E+01_JPRB, 0.11245E+01_JPRB, 0.10963E+01_JPRB, 0.11070E+01_JPRB, 0.10597E+01_JPRB, 0.99707E+00_JPRB, 0.98876E+00_JPRB ,&
&  0.10290E+01_JPRB, 0.10853E+01_JPRB, 0.11074E+01_JPRB, 0.10929E+01_JPRB, 0.11694E+01_JPRB, 0.11322E+01_JPRB, 0.12598E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.14099E+01_JPRB, 0.14321E+01_JPRB, 0.14555E+01_JPRB, 0.14840E+01_JPRB, 0.15317E+01_JPRB, 0.15868E+01_JPRB ,&
&  0.16136E+01_JPRB, 0.16174E+01_JPRB, 0.16053E+01_JPRB, 0.15775E+01_JPRB, 0.14063E+01_JPRB, 0.14572E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.82937E+00_JPRB, 0.69949E+00_JPRB, 0.93288E+00_JPRB, 0.11581E+01_JPRB, 0.10786E+01_JPRB, 0.99911E+00_JPRB, 0.90187E+00_JPRB ,&
&  0.79372E+00_JPRB, 0.73079E+00_JPRB, 0.69000E+00_JPRB, 0.65183E+00_JPRB, 0.62216E+00_JPRB, 0.62189E+00_JPRB, 0.63420E+00_JPRB ,&
&  0.63963E+00_JPRB, 0.63580E+00_JPRB, 0.63454E+00_JPRB, 0.65912E+00_JPRB, 0.72905E+00_JPRB, 0.92132E+00_JPRB, 0.12353E+01_JPRB ,&
&  0.14639E+01_JPRB, 0.71926E+00_JPRB, 0.61994E+00_JPRB, 0.61553E+00_JPRB, 0.63861E+00_JPRB, 0.67737E+00_JPRB, 0.73175E+00_JPRB ,&
&  0.88317E+00_JPRB, 0.11361E+01_JPRB, 0.10643E+01_JPRB, 0.12598E+01_JPRB, 0.25394E+01_JPRB, 0.27664E+01_JPRB, 0.16020E+01_JPRB ,&
&  0.10157E+01_JPRB, 0.72323E+00_JPRB, 0.58984E+00_JPRB, 0.54942E+00_JPRB, 0.53983E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.57800E+00_JPRB, 0.63627E+00_JPRB, 0.70627E+00_JPRB, 0.75420E+00_JPRB, 0.73556E+00_JPRB, 0.73626E+00_JPRB, 0.75483E+00_JPRB ,&
&  0.78068E+00_JPRB, 0.81492E+00_JPRB, 0.85084E+00_JPRB, 0.89743E+00_JPRB, 0.93056E+00_JPRB, 0.97251E+00_JPRB, 0.10295E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.11165E+01_JPRB, 0.11432E+01_JPRB, 0.11728E+01_JPRB, 0.11983E+01_JPRB, 0.12256E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.12909E+01_JPRB, 0.13094E+01_JPRB, 0.13704E+01_JPRB, 0.14162E+01_JPRB, 0.16058E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.11588E+01_JPRB, 0.10793E+01_JPRB, 0.10898E+01_JPRB, 0.10756E+01_JPRB, 0.10546E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.10613E+01_JPRB, 0.10710E+01_JPRB, 0.10412E+01_JPRB, 0.10657E+01_JPRB, 0.10170E+01_JPRB, 0.99059E+00_JPRB, 0.10593E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.11754E+01_JPRB, 0.11904E+01_JPRB, 0.12194E+01_JPRB, 0.12240E+01_JPRB, 0.12064E+01_JPRB, 0.11793E+01_JPRB ,&
&  0.11446E+01_JPRB, 0.11311E+01_JPRB, 0.11214E+01_JPRB, 0.11241E+01_JPRB, 0.11304E+01_JPRB, 0.10008E+01_JPRB, 0.81798E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.64354E+00_JPRB, 0.10011E+01_JPRB, 0.12737E+01_JPRB, 0.11290E+01_JPRB, 0.10542E+01_JPRB, 0.92463E+00_JPRB ,&
&  0.84678E+00_JPRB, 0.75823E+00_JPRB, 0.71652E+00_JPRB, 0.66129E+00_JPRB, 0.64267E+00_JPRB, 0.64840E+00_JPRB, 0.68310E+00_JPRB ,&
&  0.69902E+00_JPRB, 0.68421E+00_JPRB, 0.68055E+00_JPRB, 0.67687E+00_JPRB, 0.76075E+00_JPRB, 0.91843E+00_JPRB, 0.91408E+00_JPRB ,&
&  0.91227E+00_JPRB, 0.67316E+00_JPRB, 0.62352E+00_JPRB, 0.66537E+00_JPRB, 0.55456E+00_JPRB, 0.59425E+00_JPRB, 0.65168E+00_JPRB ,&
&  0.83431E+00_JPRB, 0.91405E+00_JPRB, 0.12525E+01_JPRB, 0.34604E+01_JPRB, 0.39769E+01_JPRB, 0.49746E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.94476E+00_JPRB, 0.63869E+00_JPRB, 0.57354E+00_JPRB, 0.57439E+00_JPRB, 0.56492E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.60569E+00_JPRB, 0.68061E+00_JPRB, 0.75004E+00_JPRB, 0.68295E+00_JPRB, 0.69366E+00_JPRB, 0.71661E+00_JPRB, 0.74992E+00_JPRB ,&
&  0.78774E+00_JPRB, 0.83246E+00_JPRB, 0.87913E+00_JPRB, 0.93383E+00_JPRB, 0.97777E+00_JPRB, 0.10184E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11263E+01_JPRB, 0.11639E+01_JPRB, 0.11740E+01_JPRB, 0.12005E+01_JPRB, 0.12064E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.12821E+01_JPRB, 0.13180E+01_JPRB, 0.13579E+01_JPRB, 0.13932E+01_JPRB, 0.14013E+01_JPRB, 0.14952E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.13482E+01_JPRB, 0.12029E+01_JPRB, 0.11172E+01_JPRB, 0.11127E+01_JPRB, 0.10813E+01_JPRB, 0.10720E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.10606E+01_JPRB, 0.10585E+01_JPRB, 0.10290E+01_JPRB, 0.95303E+00_JPRB, 0.97611E+00_JPRB, 0.99541E+00_JPRB ,&
&  0.10367E+01_JPRB, 0.10933E+01_JPRB, 0.11387E+01_JPRB, 0.11702E+01_JPRB, 0.11888E+01_JPRB, 0.11718E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11270E+01_JPRB, 0.11308E+01_JPRB, 0.10557E+01_JPRB, 0.10649E+01_JPRB, 0.11251E+01_JPRB, 0.96874E+00_JPRB, 0.76649E+00_JPRB ,&
&  0.66720E+00_JPRB, 0.88000E+00_JPRB, 0.11821E+01_JPRB, 0.14089E+01_JPRB, 0.12359E+01_JPRB, 0.11232E+01_JPRB, 0.10379E+01_JPRB ,&
&  0.92894E+00_JPRB, 0.83970E+00_JPRB, 0.79056E+00_JPRB, 0.72093E+00_JPRB, 0.69952E+00_JPRB, 0.70026E+00_JPRB, 0.71010E+00_JPRB ,&
&  0.70469E+00_JPRB, 0.70001E+00_JPRB, 0.70855E+00_JPRB, 0.73232E+00_JPRB, 0.82217E+00_JPRB, 0.89468E+00_JPRB, 0.15222E+01_JPRB ,&
&  0.81250E+00_JPRB, 0.56151E+00_JPRB, 0.83864E+00_JPRB, 0.46929E+00_JPRB, 0.58114E+00_JPRB, 0.63511E+00_JPRB, 0.67898E+00_JPRB ,&
&  0.63709E+00_JPRB, 0.80432E+00_JPRB, 0.11582E+01_JPRB, 0.11900E+01_JPRB, 0.12487E+01_JPRB, 0.13240E+01_JPRB, 0.80342E+00_JPRB ,&
&  0.65744E+00_JPRB, 0.64853E+00_JPRB, 0.65206E+00_JPRB, 0.60145E+00_JPRB, 0.59260E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.63252E+00_JPRB, 0.67285E+00_JPRB, 0.76578E+00_JPRB, 0.69948E+00_JPRB, 0.69862E+00_JPRB, 0.71558E+00_JPRB, 0.74136E+00_JPRB ,&
&  0.76854E+00_JPRB, 0.80361E+00_JPRB, 0.84131E+00_JPRB, 0.89263E+00_JPRB, 0.93787E+00_JPRB, 0.97734E+00_JPRB, 0.10151E+01_JPRB ,&
&  0.10499E+01_JPRB, 0.10803E+01_JPRB, 0.11095E+01_JPRB, 0.11244E+01_JPRB, 0.11375E+01_JPRB, 0.11634E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.12569E+01_JPRB, 0.12868E+01_JPRB, 0.13233E+01_JPRB, 0.13821E+01_JPRB, 0.14248E+01_JPRB, 0.14820E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.13834E+01_JPRB, 0.12344E+01_JPRB, 0.11625E+01_JPRB, 0.11281E+01_JPRB, 0.10902E+01_JPRB, 0.10638E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10967E+01_JPRB, 0.10589E+01_JPRB, 0.10674E+01_JPRB, 0.10475E+01_JPRB, 0.10600E+01_JPRB, 0.11107E+01_JPRB, 0.11343E+01_JPRB ,&
&  0.12107E+01_JPRB, 0.13066E+01_JPRB, 0.14041E+01_JPRB, 0.14750E+01_JPRB, 0.15418E+01_JPRB, 0.15337E+01_JPRB, 0.15308E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.16174E+01_JPRB, 0.15612E+01_JPRB, 0.15531E+01_JPRB, 0.15063E+01_JPRB, 0.13721E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.98618E+00_JPRB, 0.11386E+01_JPRB, 0.11948E+01_JPRB, 0.14236E+01_JPRB, 0.13157E+01_JPRB, 0.11890E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.10638E+01_JPRB, 0.98788E+00_JPRB, 0.93872E+00_JPRB, 0.85791E+00_JPRB, 0.86891E+00_JPRB, 0.88587E+00_JPRB, 0.90081E+00_JPRB ,&
&  0.87062E+00_JPRB, 0.84451E+00_JPRB, 0.83267E+00_JPRB, 0.89165E+00_JPRB, 0.11634E+01_JPRB, 0.12615E+01_JPRB, 0.15034E+01_JPRB ,&
&  0.83228E+00_JPRB, 0.54384E+00_JPRB, 0.64865E+00_JPRB, 0.77292E+00_JPRB, 0.73709E+00_JPRB, 0.76257E+00_JPRB, 0.82384E+00_JPRB ,&
&  0.89789E+00_JPRB, 0.89678E+00_JPRB, 0.86932E+00_JPRB, 0.86870E+00_JPRB, 0.86167E+00_JPRB, 0.86149E+00_JPRB, 0.78168E+00_JPRB ,&
&  0.75499E+00_JPRB, 0.69808E+00_JPRB, 0.66484E+00_JPRB, 0.64827E+00_JPRB, 0.61479E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.71363E+00_JPRB, 0.70205E+00_JPRB, 0.76021E+00_JPRB, 0.72271E+00_JPRB, 0.72160E+00_JPRB, 0.73706E+00_JPRB, 0.75479E+00_JPRB ,&
&  0.75905E+00_JPRB, 0.77278E+00_JPRB, 0.79852E+00_JPRB, 0.83913E+00_JPRB, 0.87784E+00_JPRB, 0.90759E+00_JPRB, 0.94103E+00_JPRB ,&
&  0.96749E+00_JPRB, 0.98468E+00_JPRB, 0.99537E+00_JPRB, 0.10067E+01_JPRB, 0.10372E+01_JPRB, 0.10732E+01_JPRB, 0.11273E+01_JPRB ,&
&  0.11970E+01_JPRB, 0.12177E+01_JPRB, 0.12195E+01_JPRB, 0.12519E+01_JPRB, 0.12834E+01_JPRB, 0.13960E+01_JPRB, 0.16236E+01_JPRB ,&
&  0.12298E+01_JPRB, 0.12783E+01_JPRB, 0.12005E+01_JPRB, 0.11217E+01_JPRB, 0.11405E+01_JPRB, 0.11087E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.12073E+01_JPRB, 0.12433E+01_JPRB, 0.12703E+01_JPRB, 0.13022E+01_JPRB, 0.13522E+01_JPRB, 0.14040E+01_JPRB ,&
&  0.14788E+01_JPRB, 0.15855E+01_JPRB, 0.16772E+01_JPRB, 0.18180E+01_JPRB, 0.19506E+01_JPRB, 0.20752E+01_JPRB, 0.21338E+01_JPRB ,&
&  0.21652E+01_JPRB, 0.21733E+01_JPRB, 0.20049E+01_JPRB, 0.19608E+01_JPRB, 0.19076E+01_JPRB, 0.17912E+01_JPRB, 0.15982E+01_JPRB ,&
&  0.14181E+01_JPRB, 0.13791E+01_JPRB, 0.13852E+01_JPRB, 0.13367E+01_JPRB, 0.13332E+01_JPRB, 0.13119E+01_JPRB, 0.12644E+01_JPRB ,&
&  0.12430E+01_JPRB, 0.11735E+01_JPRB, 0.11180E+01_JPRB, 0.10605E+01_JPRB, 0.10567E+01_JPRB, 0.10821E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10251E+01_JPRB, 0.10183E+01_JPRB, 0.10678E+01_JPRB, 0.11621E+01_JPRB, 0.15286E+01_JPRB, 0.13623E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.97705E+00_JPRB, 0.79530E+00_JPRB, 0.10563E+01_JPRB, 0.91380E+00_JPRB, 0.89411E+00_JPRB, 0.92791E+00_JPRB ,&
&  0.94215E+00_JPRB, 0.93531E+00_JPRB, 0.96573E+00_JPRB, 0.92601E+00_JPRB, 0.89189E+00_JPRB, 0.87326E+00_JPRB, 0.86736E+00_JPRB ,&
&  0.81534E+00_JPRB, 0.76266E+00_JPRB, 0.71122E+00_JPRB, 0.71630E+00_JPRB, 0.67833E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.74672E+00_JPRB, 0.71310E+00_JPRB, 0.70719E+00_JPRB, 0.74599E+00_JPRB, 0.72153E+00_JPRB, 0.74198E+00_JPRB, 0.73528E+00_JPRB ,&
&  0.73600E+00_JPRB, 0.73361E+00_JPRB, 0.74817E+00_JPRB, 0.76660E+00_JPRB, 0.79143E+00_JPRB, 0.81342E+00_JPRB, 0.82266E+00_JPRB ,&
&  0.83574E+00_JPRB, 0.84974E+00_JPRB, 0.87177E+00_JPRB, 0.91106E+00_JPRB, 0.98114E+00_JPRB, 0.10974E+01_JPRB, 0.13194E+01_JPRB ,&
&  0.15483E+01_JPRB, 0.16720E+01_JPRB, 0.17145E+01_JPRB, 0.17104E+01_JPRB, 0.18806E+01_JPRB, 0.23100E+01_JPRB, 0.26004E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.13357E+01_JPRB, 0.12148E+01_JPRB, 0.11030E+01_JPRB, 0.11309E+01_JPRB, 0.12327E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.12959E+01_JPRB, 0.13515E+01_JPRB, 0.14281E+01_JPRB, 0.15162E+01_JPRB, 0.15856E+01_JPRB, 0.16432E+01_JPRB, 0.17064E+01_JPRB ,&
&  0.17771E+01_JPRB, 0.18840E+01_JPRB, 0.20046E+01_JPRB, 0.21376E+01_JPRB, 0.21293E+01_JPRB, 0.23444E+01_JPRB, 0.23866E+01_JPRB ,&
&  0.23993E+01_JPRB, 0.22795E+01_JPRB, 0.20026E+01_JPRB, 0.19607E+01_JPRB, 0.20173E+01_JPRB, 0.19682E+01_JPRB, 0.18390E+01_JPRB ,&
&  0.18419E+01_JPRB, 0.15693E+01_JPRB, 0.12311E+01_JPRB, 0.12577E+01_JPRB, 0.13164E+01_JPRB, 0.14258E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.14337E+01_JPRB, 0.13961E+01_JPRB, 0.13537E+01_JPRB, 0.13107E+01_JPRB, 0.12802E+01_JPRB, 0.12937E+01_JPRB, 0.12984E+01_JPRB ,&
&  0.11969E+01_JPRB, 0.11645E+01_JPRB, 0.11164E+01_JPRB, 0.11270E+01_JPRB, 0.11841E+01_JPRB, 0.10649E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.12994E+01_JPRB, 0.13896E+01_JPRB, 0.12470E+01_JPRB, 0.14388E+01_JPRB, 0.10532E+01_JPRB, 0.96562E+00_JPRB, 0.97600E+00_JPRB ,&
&  0.10060E+01_JPRB, 0.96726E+00_JPRB, 0.95005E+00_JPRB, 0.93374E+00_JPRB, 0.92657E+00_JPRB, 0.90465E+00_JPRB, 0.85530E+00_JPRB ,&
&  0.81769E+00_JPRB, 0.79531E+00_JPRB, 0.78763E+00_JPRB, 0.77654E+00_JPRB, 0.76831E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.76900E+00_JPRB, 0.75227E+00_JPRB, 0.72849E+00_JPRB, 0.70245E+00_JPRB, 0.70182E+00_JPRB, 0.70652E+00_JPRB, 0.69312E+00_JPRB ,&
&  0.68595E+00_JPRB, 0.68135E+00_JPRB, 0.69714E+00_JPRB, 0.69905E+00_JPRB, 0.70276E+00_JPRB, 0.71812E+00_JPRB, 0.73843E+00_JPRB ,&
&  0.75857E+00_JPRB, 0.77681E+00_JPRB, 0.85127E+00_JPRB, 0.10063E+01_JPRB, 0.12921E+01_JPRB, 0.16659E+01_JPRB, 0.19440E+01_JPRB ,&
&  0.21020E+01_JPRB, 0.22296E+01_JPRB, 0.23307E+01_JPRB, 0.24164E+01_JPRB, 0.22559E+01_JPRB, 0.22704E+01_JPRB, 0.24210E+01_JPRB ,&
&  0.17520E+01_JPRB, 0.12969E+01_JPRB, 0.13502E+01_JPRB, 0.12425E+01_JPRB, 0.12494E+01_JPRB, 0.12897E+01_JPRB, 0.13220E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.14243E+01_JPRB, 0.15069E+01_JPRB, 0.16222E+01_JPRB, 0.17294E+01_JPRB, 0.17993E+01_JPRB, 0.18423E+01_JPRB ,&
&  0.18695E+01_JPRB, 0.19120E+01_JPRB, 0.19890E+01_JPRB, 0.19671E+01_JPRB, 0.19389E+01_JPRB, 0.20210E+01_JPRB, 0.20556E+01_JPRB ,&
&  0.20711E+01_JPRB, 0.20452E+01_JPRB, 0.18023E+01_JPRB, 0.17820E+01_JPRB, 0.19189E+01_JPRB, 0.17926E+01_JPRB, 0.17649E+01_JPRB ,&
&  0.18449E+01_JPRB, 0.16092E+01_JPRB, 0.11178E+01_JPRB, 0.11111E+01_JPRB, 0.12363E+01_JPRB, 0.14073E+01_JPRB, 0.15800E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.16252E+01_JPRB, 0.15894E+01_JPRB, 0.15906E+01_JPRB, 0.15454E+01_JPRB, 0.15080E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.13092E+01_JPRB, 0.12602E+01_JPRB, 0.12182E+01_JPRB, 0.12037E+01_JPRB, 0.12328E+01_JPRB, 0.12406E+01_JPRB, 0.12806E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.14871E+01_JPRB, 0.16884E+01_JPRB, 0.13533E+01_JPRB, 0.10785E+01_JPRB, 0.10106E+01_JPRB, 0.99324E+00_JPRB ,&
&  0.10164E+01_JPRB, 0.99417E+00_JPRB, 0.97449E+00_JPRB, 0.95475E+00_JPRB, 0.93871E+00_JPRB, 0.91505E+00_JPRB, 0.88277E+00_JPRB ,&
&  0.85152E+00_JPRB, 0.82810E+00_JPRB, 0.81368E+00_JPRB, 0.79644E+00_JPRB, 0.77627E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.78249E+00_JPRB, 0.76711E+00_JPRB, 0.74900E+00_JPRB, 0.71903E+00_JPRB, 0.68468E+00_JPRB, 0.67074E+00_JPRB, 0.66446E+00_JPRB ,&
&  0.67150E+00_JPRB, 0.69495E+00_JPRB, 0.70929E+00_JPRB, 0.70993E+00_JPRB, 0.70894E+00_JPRB, 0.72425E+00_JPRB, 0.76881E+00_JPRB ,&
&  0.82767E+00_JPRB, 0.90493E+00_JPRB, 0.10532E+01_JPRB, 0.13629E+01_JPRB, 0.17443E+01_JPRB, 0.20532E+01_JPRB, 0.20171E+01_JPRB ,&
&  0.18924E+01_JPRB, 0.18854E+01_JPRB, 0.18626E+01_JPRB, 0.18660E+01_JPRB, 0.18759E+01_JPRB, 0.19157E+01_JPRB, 0.19557E+01_JPRB ,&
&  0.17278E+01_JPRB, 0.14218E+01_JPRB, 0.13245E+01_JPRB, 0.12752E+01_JPRB, 0.12764E+01_JPRB, 0.12881E+01_JPRB, 0.13164E+01_JPRB ,&
&  0.13480E+01_JPRB, 0.13993E+01_JPRB, 0.14747E+01_JPRB, 0.15918E+01_JPRB, 0.17177E+01_JPRB, 0.17963E+01_JPRB, 0.18259E+01_JPRB ,&
&  0.18311E+01_JPRB, 0.18304E+01_JPRB, 0.18567E+01_JPRB, 0.16980E+01_JPRB, 0.17530E+01_JPRB, 0.18066E+01_JPRB, 0.18404E+01_JPRB ,&
&  0.18013E+01_JPRB, 0.17103E+01_JPRB, 0.17937E+01_JPRB, 0.17984E+01_JPRB, 0.17964E+01_JPRB, 0.17519E+01_JPRB, 0.17101E+01_JPRB ,&
&  0.17349E+01_JPRB, 0.15926E+01_JPRB, 0.11806E+01_JPRB, 0.12040E+01_JPRB, 0.13136E+01_JPRB, 0.12742E+01_JPRB, 0.16172E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.16984E+01_JPRB, 0.17444E+01_JPRB, 0.18097E+01_JPRB, 0.18898E+01_JPRB, 0.19458E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.15111E+01_JPRB, 0.14008E+01_JPRB, 0.13799E+01_JPRB, 0.13719E+01_JPRB, 0.13997E+01_JPRB, 0.13799E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.13701E+01_JPRB, 0.13874E+01_JPRB, 0.13963E+01_JPRB, 0.13026E+01_JPRB, 0.11375E+01_JPRB, 0.99578E+00_JPRB, 0.10041E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.97135E+00_JPRB, 0.96537E+00_JPRB, 0.95521E+00_JPRB, 0.94758E+00_JPRB, 0.93704E+00_JPRB, 0.92347E+00_JPRB ,&
&  0.89999E+00_JPRB, 0.87479E+00_JPRB, 0.84943E+00_JPRB, 0.82064E+00_JPRB, 0.79618E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.80527E+00_JPRB, 0.79049E+00_JPRB, 0.77328E+00_JPRB, 0.75582E+00_JPRB, 0.72221E+00_JPRB, 0.70575E+00_JPRB, 0.70650E+00_JPRB ,&
&  0.72497E+00_JPRB, 0.75163E+00_JPRB, 0.77312E+00_JPRB, 0.78610E+00_JPRB, 0.81689E+00_JPRB, 0.83526E+00_JPRB, 0.86718E+00_JPRB ,&
&  0.93036E+00_JPRB, 0.10356E+01_JPRB, 0.12374E+01_JPRB, 0.15562E+01_JPRB, 0.18556E+01_JPRB, 0.21303E+01_JPRB, 0.20174E+01_JPRB ,&
&  0.19032E+01_JPRB, 0.18249E+01_JPRB, 0.17701E+01_JPRB, 0.17033E+01_JPRB, 0.14975E+01_JPRB, 0.15919E+01_JPRB, 0.16179E+01_JPRB ,&
&  0.16097E+01_JPRB, 0.14915E+01_JPRB, 0.13227E+01_JPRB, 0.12686E+01_JPRB, 0.12338E+01_JPRB, 0.12436E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.13160E+01_JPRB, 0.13561E+01_JPRB, 0.14227E+01_JPRB, 0.15088E+01_JPRB, 0.16112E+01_JPRB, 0.17087E+01_JPRB, 0.17567E+01_JPRB ,&
&  0.17436E+01_JPRB, 0.17288E+01_JPRB, 0.17066E+01_JPRB, 0.16245E+01_JPRB, 0.16017E+01_JPRB, 0.16292E+01_JPRB, 0.16610E+01_JPRB ,&
&  0.17025E+01_JPRB, 0.16338E+01_JPRB, 0.16776E+01_JPRB, 0.17452E+01_JPRB, 0.16746E+01_JPRB, 0.15680E+01_JPRB, 0.15537E+01_JPRB ,&
&  0.16091E+01_JPRB, 0.17212E+01_JPRB, 0.12798E+01_JPRB, 0.15105E+01_JPRB, 0.17808E+01_JPRB, 0.15928E+01_JPRB, 0.15876E+01_JPRB ,&
&  0.16983E+01_JPRB, 0.17858E+01_JPRB, 0.19053E+01_JPRB, 0.20381E+01_JPRB, 0.21836E+01_JPRB, 0.23105E+01_JPRB, 0.18208E+01_JPRB ,&
&  0.17561E+01_JPRB, 0.15871E+01_JPRB, 0.15587E+01_JPRB, 0.15818E+01_JPRB, 0.16339E+01_JPRB, 0.16199E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.13641E+01_JPRB, 0.15572E+01_JPRB, 0.16523E+01_JPRB, 0.17533E+01_JPRB, 0.16367E+01_JPRB, 0.10152E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.10005E+01_JPRB, 0.99177E+00_JPRB, 0.98499E+00_JPRB, 0.98180E+00_JPRB, 0.97473E+00_JPRB, 0.96859E+00_JPRB ,&
&  0.95064E+00_JPRB, 0.92152E+00_JPRB, 0.88969E+00_JPRB, 0.85790E+00_JPRB, 0.82870E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.83892E+00_JPRB, 0.82055E+00_JPRB, 0.80249E+00_JPRB, 0.77908E+00_JPRB, 0.76954E+00_JPRB, 0.75782E+00_JPRB, 0.76001E+00_JPRB ,&
&  0.76581E+00_JPRB, 0.78779E+00_JPRB, 0.80887E+00_JPRB, 0.84478E+00_JPRB, 0.88648E+00_JPRB, 0.95851E+00_JPRB, 0.10289E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.12372E+01_JPRB, 0.14582E+01_JPRB, 0.17363E+01_JPRB, 0.19615E+01_JPRB, 0.20863E+01_JPRB, 0.20161E+01_JPRB ,&
&  0.19250E+01_JPRB, 0.19036E+01_JPRB, 0.18062E+01_JPRB, 0.14092E+01_JPRB, 0.13340E+01_JPRB, 0.16125E+01_JPRB, 0.14885E+01_JPRB ,&
&  0.13466E+01_JPRB, 0.12960E+01_JPRB, 0.12427E+01_JPRB, 0.11954E+01_JPRB, 0.11586E+01_JPRB, 0.11716E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.12549E+01_JPRB, 0.13070E+01_JPRB, 0.13748E+01_JPRB, 0.14462E+01_JPRB, 0.15203E+01_JPRB, 0.16223E+01_JPRB, 0.16878E+01_JPRB ,&
&  0.16864E+01_JPRB, 0.16731E+01_JPRB, 0.16504E+01_JPRB, 0.16092E+01_JPRB, 0.15377E+01_JPRB, 0.15309E+01_JPRB, 0.15472E+01_JPRB ,&
&  0.15704E+01_JPRB, 0.15832E+01_JPRB, 0.16002E+01_JPRB, 0.16648E+01_JPRB, 0.16929E+01_JPRB, 0.15110E+01_JPRB, 0.14841E+01_JPRB ,&
&  0.15699E+01_JPRB, 0.16757E+01_JPRB, 0.15052E+01_JPRB, 0.17371E+01_JPRB, 0.11244E+01_JPRB, 0.13268E+01_JPRB, 0.15851E+01_JPRB ,&
&  0.16362E+01_JPRB, 0.17422E+01_JPRB, 0.18707E+01_JPRB, 0.20278E+01_JPRB, 0.21692E+01_JPRB, 0.20134E+01_JPRB, 0.20779E+01_JPRB ,&
&  0.19991E+01_JPRB, 0.18122E+01_JPRB, 0.17864E+01_JPRB, 0.17704E+01_JPRB, 0.18451E+01_JPRB, 0.18823E+01_JPRB, 0.16583E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.16943E+01_JPRB, 0.18132E+01_JPRB, 0.18098E+01_JPRB, 0.14303E+01_JPRB, 0.11098E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10618E+01_JPRB, 0.10292E+01_JPRB, 0.10142E+01_JPRB, 0.10059E+01_JPRB, 0.10045E+01_JPRB, 0.99686E+00_JPRB, 0.98832E+00_JPRB ,&
&  0.97537E+00_JPRB, 0.94908E+00_JPRB, 0.92281E+00_JPRB, 0.89322E+00_JPRB, 0.86397E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.86331E+00_JPRB, 0.85134E+00_JPRB, 0.84151E+00_JPRB, 0.80462E+00_JPRB, 0.78939E+00_JPRB, 0.79883E+00_JPRB, 0.78957E+00_JPRB ,&
&  0.77352E+00_JPRB, 0.75218E+00_JPRB, 0.77432E+00_JPRB, 0.82138E+00_JPRB, 0.87478E+00_JPRB, 0.95775E+00_JPRB, 0.10665E+01_JPRB ,&
&  0.12061E+01_JPRB, 0.13446E+01_JPRB, 0.15673E+01_JPRB, 0.17900E+01_JPRB, 0.19735E+01_JPRB, 0.21214E+01_JPRB, 0.21833E+01_JPRB ,&
&  0.17690E+01_JPRB, 0.13562E+01_JPRB, 0.18030E+01_JPRB, 0.18941E+01_JPRB, 0.17750E+01_JPRB, 0.17283E+01_JPRB, 0.16635E+01_JPRB ,&
&  0.14428E+01_JPRB, 0.13348E+01_JPRB, 0.13290E+01_JPRB, 0.12309E+01_JPRB, 0.10897E+01_JPRB, 0.10963E+01_JPRB, 0.11322E+01_JPRB ,&
&  0.11893E+01_JPRB, 0.12436E+01_JPRB, 0.13147E+01_JPRB, 0.14029E+01_JPRB, 0.14788E+01_JPRB, 0.15656E+01_JPRB, 0.16276E+01_JPRB ,&
&  0.16411E+01_JPRB, 0.16354E+01_JPRB, 0.16330E+01_JPRB, 0.15779E+01_JPRB, 0.15503E+01_JPRB, 0.15358E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.15148E+01_JPRB, 0.15738E+01_JPRB, 0.16324E+01_JPRB, 0.17223E+01_JPRB, 0.15860E+01_JPRB, 0.16561E+01_JPRB ,&
&  0.16235E+01_JPRB, 0.16634E+01_JPRB, 0.16455E+01_JPRB, 0.17745E+01_JPRB, 0.12399E+01_JPRB, 0.14150E+01_JPRB, 0.13971E+01_JPRB ,&
&  0.16921E+01_JPRB, 0.17519E+01_JPRB, 0.18427E+01_JPRB, 0.19416E+01_JPRB, 0.20889E+01_JPRB, 0.19980E+01_JPRB, 0.21218E+01_JPRB ,&
&  0.21377E+01_JPRB, 0.21171E+01_JPRB, 0.20009E+01_JPRB, 0.20242E+01_JPRB, 0.19407E+01_JPRB, 0.21224E+01_JPRB, 0.20332E+01_JPRB ,&
&  0.20663E+01_JPRB, 0.19353E+01_JPRB, 0.18365E+01_JPRB, 0.17609E+01_JPRB, 0.14992E+01_JPRB, 0.12863E+01_JPRB, 0.11729E+01_JPRB ,&
&  0.11195E+01_JPRB, 0.10875E+01_JPRB, 0.10525E+01_JPRB, 0.10285E+01_JPRB, 0.10192E+01_JPRB, 0.10090E+01_JPRB, 0.99558E+00_JPRB ,&
&  0.97700E+00_JPRB, 0.95462E+00_JPRB, 0.93208E+00_JPRB, 0.90322E+00_JPRB, 0.88123E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.87299E+00_JPRB, 0.86938E+00_JPRB, 0.87324E+00_JPRB, 0.86976E+00_JPRB, 0.95026E+00_JPRB, 0.10696E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.74102E+00_JPRB, 0.73873E+00_JPRB, 0.77230E+00_JPRB, 0.81850E+00_JPRB, 0.89567E+00_JPRB, 0.98754E+00_JPRB ,&
&  0.11116E+01_JPRB, 0.12712E+01_JPRB, 0.15166E+01_JPRB, 0.18100E+01_JPRB, 0.20178E+01_JPRB, 0.22404E+01_JPRB, 0.22005E+01_JPRB ,&
&  0.15748E+01_JPRB, 0.15086E+01_JPRB, 0.19715E+01_JPRB, 0.18864E+01_JPRB, 0.17602E+01_JPRB, 0.16755E+01_JPRB, 0.15685E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.12657E+01_JPRB, 0.12492E+01_JPRB, 0.12021E+01_JPRB, 0.10533E+01_JPRB, 0.10189E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.11514E+01_JPRB, 0.12384E+01_JPRB, 0.13313E+01_JPRB, 0.14470E+01_JPRB, 0.15213E+01_JPRB, 0.15902E+01_JPRB ,&
&  0.16223E+01_JPRB, 0.16298E+01_JPRB, 0.15956E+01_JPRB, 0.15391E+01_JPRB, 0.15659E+01_JPRB, 0.15848E+01_JPRB, 0.15773E+01_JPRB ,&
&  0.15793E+01_JPRB, 0.16635E+01_JPRB, 0.17017E+01_JPRB, 0.16096E+01_JPRB, 0.14711E+01_JPRB, 0.16316E+01_JPRB, 0.16021E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.16007E+01_JPRB, 0.18400E+01_JPRB, 0.14312E+01_JPRB, 0.14488E+01_JPRB, 0.15842E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.17260E+01_JPRB, 0.18227E+01_JPRB, 0.18926E+01_JPRB, 0.19549E+01_JPRB, 0.18365E+01_JPRB, 0.20933E+01_JPRB ,&
&  0.20095E+01_JPRB, 0.23028E+01_JPRB, 0.22048E+01_JPRB, 0.21832E+01_JPRB, 0.19502E+01_JPRB, 0.20454E+01_JPRB, 0.22084E+01_JPRB ,&
&  0.24240E+01_JPRB, 0.23256E+01_JPRB, 0.21661E+01_JPRB, 0.19476E+01_JPRB, 0.16342E+01_JPRB, 0.15030E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.11640E+01_JPRB, 0.11164E+01_JPRB, 0.10675E+01_JPRB, 0.10402E+01_JPRB, 0.10184E+01_JPRB, 0.99987E+00_JPRB ,&
&  0.97969E+00_JPRB, 0.96256E+00_JPRB, 0.94392E+00_JPRB, 0.91169E+00_JPRB, 0.88599E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.88382E+00_JPRB, 0.86571E+00_JPRB, 0.85792E+00_JPRB, 0.86156E+00_JPRB, 0.94034E+00_JPRB, 0.97087E+00_JPRB, 0.96923E+00_JPRB ,&
&  0.94963E+00_JPRB, 0.71344E+00_JPRB, 0.69894E+00_JPRB, 0.72795E+00_JPRB, 0.77793E+00_JPRB, 0.83987E+00_JPRB, 0.92522E+00_JPRB ,&
&  0.10129E+01_JPRB, 0.11585E+01_JPRB, 0.14026E+01_JPRB, 0.16723E+01_JPRB, 0.18856E+01_JPRB, 0.20150E+01_JPRB, 0.19172E+01_JPRB ,&
&  0.13391E+01_JPRB, 0.22036E+01_JPRB, 0.20339E+01_JPRB, 0.20200E+01_JPRB, 0.19668E+01_JPRB, 0.18355E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.13336E+01_JPRB, 0.12387E+01_JPRB, 0.11691E+01_JPRB, 0.11345E+01_JPRB, 0.10824E+01_JPRB, 0.98413E+00_JPRB, 0.95119E+00_JPRB ,&
&  0.98324E+00_JPRB, 0.10644E+01_JPRB, 0.11595E+01_JPRB, 0.12558E+01_JPRB, 0.13825E+01_JPRB, 0.15032E+01_JPRB, 0.15707E+01_JPRB ,&
&  0.16358E+01_JPRB, 0.16372E+01_JPRB, 0.15690E+01_JPRB, 0.15735E+01_JPRB, 0.15968E+01_JPRB, 0.16575E+01_JPRB, 0.16884E+01_JPRB ,&
&  0.16198E+01_JPRB, 0.15648E+01_JPRB, 0.17554E+01_JPRB, 0.17926E+01_JPRB, 0.16978E+01_JPRB, 0.17263E+01_JPRB, 0.15799E+01_JPRB ,&
&  0.17099E+01_JPRB, 0.15687E+01_JPRB, 0.17090E+01_JPRB, 0.14324E+01_JPRB, 0.14969E+01_JPRB, 0.16692E+01_JPRB, 0.16538E+01_JPRB ,&
&  0.15100E+01_JPRB, 0.16596E+01_JPRB, 0.16888E+01_JPRB, 0.17920E+01_JPRB, 0.18560E+01_JPRB, 0.18793E+01_JPRB, 0.18511E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.21275E+01_JPRB, 0.24071E+01_JPRB, 0.20666E+01_JPRB, 0.20150E+01_JPRB, 0.20800E+01_JPRB, 0.24858E+01_JPRB ,&
&  0.28670E+01_JPRB, 0.27514E+01_JPRB, 0.28023E+01_JPRB, 0.24945E+01_JPRB, 0.19657E+01_JPRB, 0.17592E+01_JPRB, 0.16023E+01_JPRB ,&
&  0.14829E+01_JPRB, 0.13442E+01_JPRB, 0.12323E+01_JPRB, 0.11235E+01_JPRB, 0.10421E+01_JPRB, 0.10127E+01_JPRB, 0.10046E+01_JPRB ,&
&  0.99345E+00_JPRB, 0.98418E+00_JPRB, 0.96790E+00_JPRB, 0.93430E+00_JPRB, 0.90474E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.91148E+00_JPRB, 0.89851E+00_JPRB, 0.88748E+00_JPRB, 0.87208E+00_JPRB, 0.89781E+00_JPRB, 0.90168E+00_JPRB, 0.83141E+00_JPRB ,&
&  0.76290E+00_JPRB, 0.66154E+00_JPRB, 0.65671E+00_JPRB, 0.68343E+00_JPRB, 0.72790E+00_JPRB, 0.78086E+00_JPRB, 0.85408E+00_JPRB ,&
&  0.95438E+00_JPRB, 0.10555E+01_JPRB, 0.12984E+01_JPRB, 0.15348E+01_JPRB, 0.17430E+01_JPRB, 0.20462E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.15243E+01_JPRB, 0.21797E+01_JPRB, 0.21077E+01_JPRB, 0.20879E+01_JPRB, 0.20630E+01_JPRB, 0.20390E+01_JPRB, 0.16891E+01_JPRB ,&
&  0.14866E+01_JPRB, 0.13618E+01_JPRB, 0.12084E+01_JPRB, 0.10848E+01_JPRB, 0.10528E+01_JPRB, 0.10192E+01_JPRB, 0.93405E+00_JPRB ,&
&  0.92474E+00_JPRB, 0.98755E+00_JPRB, 0.10894E+01_JPRB, 0.11979E+01_JPRB, 0.13285E+01_JPRB, 0.14788E+01_JPRB, 0.15674E+01_JPRB ,&
&  0.16441E+01_JPRB, 0.16589E+01_JPRB, 0.16415E+01_JPRB, 0.15443E+01_JPRB, 0.16176E+01_JPRB, 0.16794E+01_JPRB, 0.17434E+01_JPRB ,&
&  0.16540E+01_JPRB, 0.16067E+01_JPRB, 0.19033E+01_JPRB, 0.21410E+01_JPRB, 0.21439E+01_JPRB, 0.19683E+01_JPRB, 0.17706E+01_JPRB ,&
&  0.17774E+01_JPRB, 0.16870E+01_JPRB, 0.15343E+01_JPRB, 0.14092E+01_JPRB, 0.15551E+01_JPRB, 0.17023E+01_JPRB, 0.17424E+01_JPRB ,&
&  0.16818E+01_JPRB, 0.16404E+01_JPRB, 0.16059E+01_JPRB, 0.16140E+01_JPRB, 0.16336E+01_JPRB, 0.16634E+01_JPRB, 0.17737E+01_JPRB ,&
&  0.20004E+01_JPRB, 0.22743E+01_JPRB, 0.23977E+01_JPRB, 0.20628E+01_JPRB, 0.20158E+01_JPRB, 0.19609E+01_JPRB, 0.24158E+01_JPRB ,&
&  0.31489E+01_JPRB, 0.31464E+01_JPRB, 0.30014E+01_JPRB, 0.29598E+01_JPRB, 0.23662E+01_JPRB, 0.20937E+01_JPRB, 0.18881E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.15361E+01_JPRB, 0.13992E+01_JPRB, 0.12832E+01_JPRB, 0.11624E+01_JPRB, 0.10924E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.10513E+01_JPRB, 0.10369E+01_JPRB, 0.10130E+01_JPRB, 0.97052E+00_JPRB, 0.93556E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.93418E+00_JPRB, 0.90736E+00_JPRB, 0.89868E+00_JPRB, 0.88011E+00_JPRB, 0.84470E+00_JPRB, 0.80970E+00_JPRB, 0.73544E+00_JPRB ,&
&  0.67347E+00_JPRB, 0.63884E+00_JPRB, 0.62706E+00_JPRB, 0.64009E+00_JPRB, 0.68184E+00_JPRB, 0.73831E+00_JPRB, 0.78347E+00_JPRB ,&
&  0.89296E+00_JPRB, 0.99131E+00_JPRB, 0.11650E+01_JPRB, 0.13913E+01_JPRB, 0.14379E+01_JPRB, 0.12798E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.19631E+01_JPRB, 0.24030E+01_JPRB, 0.22440E+01_JPRB, 0.21835E+01_JPRB, 0.22035E+01_JPRB, 0.22629E+01_JPRB, 0.18964E+01_JPRB ,&
&  0.16522E+01_JPRB, 0.14840E+01_JPRB, 0.12957E+01_JPRB, 0.11646E+01_JPRB, 0.10671E+01_JPRB, 0.10473E+01_JPRB, 0.98799E+00_JPRB ,&
&  0.92101E+00_JPRB, 0.95379E+00_JPRB, 0.10724E+01_JPRB, 0.11987E+01_JPRB, 0.13300E+01_JPRB, 0.14699E+01_JPRB, 0.16036E+01_JPRB ,&
&  0.16546E+01_JPRB, 0.16539E+01_JPRB, 0.16307E+01_JPRB, 0.15651E+01_JPRB, 0.16078E+01_JPRB, 0.16827E+01_JPRB, 0.18585E+01_JPRB ,&
&  0.19437E+01_JPRB, 0.21104E+01_JPRB, 0.22094E+01_JPRB, 0.23144E+01_JPRB, 0.23361E+01_JPRB, 0.23005E+01_JPRB, 0.21374E+01_JPRB ,&
&  0.20372E+01_JPRB, 0.18058E+01_JPRB, 0.14237E+01_JPRB, 0.14547E+01_JPRB, 0.16858E+01_JPRB, 0.18242E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.14062E+01_JPRB, 0.12368E+01_JPRB, 0.12287E+01_JPRB, 0.13444E+01_JPRB, 0.15054E+01_JPRB, 0.15794E+01_JPRB, 0.17956E+01_JPRB ,&
&  0.18204E+01_JPRB, 0.11884E+01_JPRB, 0.10408E+01_JPRB, 0.13287E+01_JPRB, 0.12322E+01_JPRB, 0.14111E+01_JPRB, 0.24060E+01_JPRB ,&
&  0.33566E+01_JPRB, 0.39196E+01_JPRB, 0.37384E+01_JPRB, 0.31067E+01_JPRB, 0.28250E+01_JPRB, 0.25312E+01_JPRB, 0.23066E+01_JPRB ,&
&  0.20017E+01_JPRB, 0.17556E+01_JPRB, 0.15887E+01_JPRB, 0.14763E+01_JPRB, 0.13612E+01_JPRB, 0.12614E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11287E+01_JPRB, 0.10901E+01_JPRB, 0.10265E+01_JPRB, 0.96918E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.94146E+00_JPRB, 0.90093E+00_JPRB, 0.88881E+00_JPRB, 0.87141E+00_JPRB, 0.81837E+00_JPRB, 0.75023E+00_JPRB, 0.69368E+00_JPRB ,&
&  0.65836E+00_JPRB, 0.63638E+00_JPRB, 0.61849E+00_JPRB, 0.61931E+00_JPRB, 0.64287E+00_JPRB, 0.69980E+00_JPRB, 0.76220E+00_JPRB ,&
&  0.84998E+00_JPRB, 0.97767E+00_JPRB, 0.11167E+01_JPRB, 0.13034E+01_JPRB, 0.12773E+01_JPRB, 0.11383E+01_JPRB, 0.13136E+01_JPRB ,&
&  0.16261E+01_JPRB, 0.21096E+01_JPRB, 0.23179E+01_JPRB, 0.22920E+01_JPRB, 0.22370E+01_JPRB, 0.22059E+01_JPRB, 0.20413E+01_JPRB ,&
&  0.18158E+01_JPRB, 0.15689E+01_JPRB, 0.13728E+01_JPRB, 0.12458E+01_JPRB, 0.11229E+01_JPRB, 0.10367E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.97393E+00_JPRB, 0.93348E+00_JPRB, 0.10304E+01_JPRB, 0.11867E+01_JPRB, 0.13154E+01_JPRB, 0.14496E+01_JPRB, 0.15735E+01_JPRB ,&
&  0.16197E+01_JPRB, 0.16354E+01_JPRB, 0.16425E+01_JPRB, 0.16540E+01_JPRB, 0.13631E+01_JPRB, 0.16523E+01_JPRB, 0.17680E+01_JPRB ,&
&  0.19732E+01_JPRB, 0.21974E+01_JPRB, 0.22625E+01_JPRB, 0.25971E+01_JPRB, 0.25843E+01_JPRB, 0.24754E+01_JPRB, 0.23487E+01_JPRB ,&
&  0.23595E+01_JPRB, 0.20977E+01_JPRB, 0.17231E+01_JPRB, 0.17059E+01_JPRB, 0.18493E+01_JPRB, 0.19264E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.14414E+01_JPRB, 0.13085E+01_JPRB, 0.11521E+01_JPRB, 0.13955E+01_JPRB, 0.15511E+01_JPRB, 0.11674E+01_JPRB ,&
&  0.66847E+00_JPRB, 0.56857E+00_JPRB, 0.58648E+00_JPRB, 0.62510E+00_JPRB, 0.63534E+00_JPRB, 0.10318E+01_JPRB, 0.32777E+01_JPRB ,&
&  0.35781E+01_JPRB, 0.42494E+01_JPRB, 0.39203E+01_JPRB, 0.33960E+01_JPRB, 0.30280E+01_JPRB, 0.27805E+01_JPRB, 0.25303E+01_JPRB ,&
&  0.22067E+01_JPRB, 0.20077E+01_JPRB, 0.18346E+01_JPRB, 0.17199E+01_JPRB, 0.16109E+01_JPRB, 0.14955E+01_JPRB, 0.13887E+01_JPRB ,&
&  0.13166E+01_JPRB, 0.12380E+01_JPRB, 0.11687E+01_JPRB, 0.10784E+01_JPRB, 0.99715E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.93144E+00_JPRB, 0.87235E+00_JPRB, 0.84058E+00_JPRB, 0.81307E+00_JPRB, 0.79613E+00_JPRB, 0.72522E+00_JPRB, 0.70416E+00_JPRB ,&
&  0.66321E+00_JPRB, 0.65159E+00_JPRB, 0.62074E+00_JPRB, 0.60810E+00_JPRB, 0.61324E+00_JPRB, 0.66604E+00_JPRB, 0.74578E+00_JPRB ,&
&  0.82297E+00_JPRB, 0.94959E+00_JPRB, 0.11953E+01_JPRB, 0.11725E+01_JPRB, 0.11783E+01_JPRB, 0.10861E+01_JPRB, 0.10393E+01_JPRB ,&
&  0.12586E+01_JPRB, 0.17448E+01_JPRB, 0.19687E+01_JPRB, 0.20674E+01_JPRB, 0.22839E+01_JPRB, 0.23133E+01_JPRB, 0.21998E+01_JPRB ,&
&  0.20524E+01_JPRB, 0.17744E+01_JPRB, 0.15471E+01_JPRB, 0.13778E+01_JPRB, 0.12292E+01_JPRB, 0.10955E+01_JPRB, 0.10138E+01_JPRB ,&
&  0.10074E+01_JPRB, 0.96625E+00_JPRB, 0.99756E+00_JPRB, 0.11461E+01_JPRB, 0.13019E+01_JPRB, 0.14032E+01_JPRB, 0.15027E+01_JPRB ,&
&  0.15866E+01_JPRB, 0.16073E+01_JPRB, 0.16148E+01_JPRB, 0.16115E+01_JPRB, 0.13741E+01_JPRB, 0.12464E+01_JPRB, 0.16122E+01_JPRB ,&
&  0.19069E+01_JPRB, 0.23068E+01_JPRB, 0.25478E+01_JPRB, 0.29166E+01_JPRB, 0.26909E+01_JPRB, 0.26159E+01_JPRB, 0.26446E+01_JPRB ,&
&  0.25147E+01_JPRB, 0.27052E+01_JPRB, 0.20573E+01_JPRB, 0.19510E+01_JPRB, 0.20752E+01_JPRB, 0.15493E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.15826E+01_JPRB, 0.13157E+01_JPRB, 0.10394E+01_JPRB, 0.13161E+01_JPRB, 0.12356E+01_JPRB, 0.67147E+00_JPRB ,&
&  0.56354E+00_JPRB, 0.53609E+00_JPRB, 0.51456E+00_JPRB, 0.50639E+00_JPRB, 0.53708E+00_JPRB, 0.86284E+00_JPRB, 0.17713E+01_JPRB ,&
&  0.26621E+01_JPRB, 0.38622E+01_JPRB, 0.40029E+01_JPRB, 0.37167E+01_JPRB, 0.32278E+01_JPRB, 0.30504E+01_JPRB, 0.27250E+01_JPRB ,&
&  0.23695E+01_JPRB, 0.21272E+01_JPRB, 0.19374E+01_JPRB, 0.18393E+01_JPRB, 0.17550E+01_JPRB, 0.16711E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.14462E+01_JPRB, 0.13391E+01_JPRB, 0.12355E+01_JPRB, 0.11088E+01_JPRB, 0.10075E+01_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.91319E+00_JPRB, 0.84290E+00_JPRB, 0.77753E+00_JPRB, 0.76667E+00_JPRB, 0.78559E+00_JPRB, 0.73178E+00_JPRB, 0.70122E+00_JPRB ,&
&  0.67402E+00_JPRB, 0.65897E+00_JPRB, 0.64086E+00_JPRB, 0.62166E+00_JPRB, 0.61410E+00_JPRB, 0.64985E+00_JPRB, 0.71083E+00_JPRB ,&
&  0.78895E+00_JPRB, 0.90262E+00_JPRB, 0.10813E+01_JPRB, 0.94850E+00_JPRB, 0.85676E+00_JPRB, 0.77299E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.10204E+01_JPRB, 0.13796E+01_JPRB, 0.16456E+01_JPRB, 0.19312E+01_JPRB, 0.22343E+01_JPRB, 0.22589E+01_JPRB, 0.24437E+01_JPRB ,&
&  0.23977E+01_JPRB, 0.20553E+01_JPRB, 0.17639E+01_JPRB, 0.15197E+01_JPRB, 0.13491E+01_JPRB, 0.11857E+01_JPRB, 0.10637E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.98651E+00_JPRB, 0.97858E+00_JPRB, 0.10783E+01_JPRB, 0.12408E+01_JPRB, 0.13530E+01_JPRB, 0.14503E+01_JPRB ,&
&  0.15278E+01_JPRB, 0.15664E+01_JPRB, 0.15935E+01_JPRB, 0.16446E+01_JPRB, 0.17590E+01_JPRB, 0.17064E+01_JPRB, 0.15823E+01_JPRB ,&
&  0.15664E+01_JPRB, 0.17678E+01_JPRB, 0.25263E+01_JPRB, 0.27115E+01_JPRB, 0.26885E+01_JPRB, 0.28350E+01_JPRB, 0.28581E+01_JPRB ,&
&  0.27289E+01_JPRB, 0.27577E+01_JPRB, 0.23057E+01_JPRB, 0.23400E+01_JPRB, 0.20369E+01_JPRB, 0.14497E+01_JPRB, 0.17727E+01_JPRB ,&
&  0.19272E+01_JPRB, 0.18239E+01_JPRB, 0.14067E+01_JPRB, 0.10104E+01_JPRB, 0.99175E+00_JPRB, 0.73628E+00_JPRB, 0.55007E+00_JPRB ,&
&  0.53153E+00_JPRB, 0.51896E+00_JPRB, 0.48512E+00_JPRB, 0.52571E+00_JPRB, 0.54999E+00_JPRB, 0.81898E+00_JPRB, 0.13793E+01_JPRB ,&
&  0.23149E+01_JPRB, 0.32716E+01_JPRB, 0.38293E+01_JPRB, 0.36944E+01_JPRB, 0.33166E+01_JPRB, 0.32447E+01_JPRB, 0.31749E+01_JPRB ,&
&  0.25517E+01_JPRB, 0.22811E+01_JPRB, 0.20633E+01_JPRB, 0.19402E+01_JPRB, 0.18356E+01_JPRB, 0.17365E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.14552E+01_JPRB, 0.13427E+01_JPRB, 0.12355E+01_JPRB, 0.11050E+01_JPRB, 0.10016E+01_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.88851E+00_JPRB, 0.81630E+00_JPRB, 0.73617E+00_JPRB, 0.73464E+00_JPRB, 0.76807E+00_JPRB, 0.74615E+00_JPRB, 0.69838E+00_JPRB ,&
&  0.68518E+00_JPRB, 0.65687E+00_JPRB, 0.64552E+00_JPRB, 0.63143E+00_JPRB, 0.62128E+00_JPRB, 0.63781E+00_JPRB, 0.68738E+00_JPRB ,&
&  0.78655E+00_JPRB, 0.92512E+00_JPRB, 0.81928E+00_JPRB, 0.72890E+00_JPRB, 0.71109E+00_JPRB, 0.69827E+00_JPRB, 0.61401E+00_JPRB ,&
&  0.89396E+00_JPRB, 0.11529E+01_JPRB, 0.13824E+01_JPRB, 0.18208E+01_JPRB, 0.25368E+01_JPRB, 0.26316E+01_JPRB, 0.27021E+01_JPRB ,&
&  0.26743E+01_JPRB, 0.24790E+01_JPRB, 0.20936E+01_JPRB, 0.17279E+01_JPRB, 0.14707E+01_JPRB, 0.13045E+01_JPRB, 0.11340E+01_JPRB ,&
&  0.10102E+01_JPRB, 0.98365E+00_JPRB, 0.99568E+00_JPRB, 0.10343E+01_JPRB, 0.11379E+01_JPRB, 0.12343E+01_JPRB, 0.13101E+01_JPRB ,&
&  0.14203E+01_JPRB, 0.15164E+01_JPRB, 0.15652E+01_JPRB, 0.15954E+01_JPRB, 0.16310E+01_JPRB, 0.15275E+01_JPRB, 0.21110E+01_JPRB ,&
&  0.20568E+01_JPRB, 0.21168E+01_JPRB, 0.21353E+01_JPRB, 0.26913E+01_JPRB, 0.29009E+01_JPRB, 0.28278E+01_JPRB, 0.32673E+01_JPRB ,&
&  0.23932E+01_JPRB, 0.21019E+01_JPRB, 0.21440E+01_JPRB, 0.20790E+01_JPRB, 0.16523E+01_JPRB, 0.18034E+01_JPRB, 0.26227E+01_JPRB ,&
&  0.20242E+01_JPRB, 0.22100E+01_JPRB, 0.22914E+01_JPRB, 0.22710E+01_JPRB, 0.10249E+01_JPRB, 0.69158E+00_JPRB, 0.11815E+01_JPRB ,&
&  0.13600E+01_JPRB, 0.90285E+00_JPRB, 0.68515E+00_JPRB, 0.90954E+00_JPRB, 0.73275E+00_JPRB, 0.91890E+00_JPRB, 0.14345E+01_JPRB ,&
&  0.18046E+01_JPRB, 0.22801E+01_JPRB, 0.34685E+01_JPRB, 0.36087E+01_JPRB, 0.33404E+01_JPRB, 0.34095E+01_JPRB, 0.32194E+01_JPRB ,&
&  0.29987E+01_JPRB, 0.24853E+01_JPRB, 0.23516E+01_JPRB, 0.20703E+01_JPRB, 0.18506E+01_JPRB, 0.17080E+01_JPRB, 0.15807E+01_JPRB ,&
&  0.14463E+01_JPRB, 0.13223E+01_JPRB, 0.12013E+01_JPRB, 0.10760E+01_JPRB, 0.97205E+00_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.83211E+00_JPRB, 0.77444E+00_JPRB, 0.69881E+00_JPRB, 0.69677E+00_JPRB, 0.73080E+00_JPRB, 0.74122E+00_JPRB, 0.70893E+00_JPRB ,&
&  0.68037E+00_JPRB, 0.64953E+00_JPRB, 0.63986E+00_JPRB, 0.63311E+00_JPRB, 0.62855E+00_JPRB, 0.64059E+00_JPRB, 0.66140E+00_JPRB ,&
&  0.72754E+00_JPRB, 0.73871E+00_JPRB, 0.65948E+00_JPRB, 0.61461E+00_JPRB, 0.70596E+00_JPRB, 0.65838E+00_JPRB, 0.65427E+00_JPRB ,&
&  0.83808E+00_JPRB, 0.10100E+01_JPRB, 0.11977E+01_JPRB, 0.15282E+01_JPRB, 0.18973E+01_JPRB, 0.24996E+01_JPRB, 0.27924E+01_JPRB ,&
&  0.28211E+01_JPRB, 0.27578E+01_JPRB, 0.24594E+01_JPRB, 0.20702E+01_JPRB, 0.16928E+01_JPRB, 0.14580E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.99839E+00_JPRB, 0.98186E+00_JPRB, 0.10214E+01_JPRB, 0.10967E+01_JPRB, 0.11486E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.13731E+01_JPRB, 0.14418E+01_JPRB, 0.14491E+01_JPRB, 0.13975E+01_JPRB, 0.12717E+01_JPRB, 0.14723E+01_JPRB ,&
&  0.18585E+01_JPRB, 0.19100E+01_JPRB, 0.21532E+01_JPRB, 0.26055E+01_JPRB, 0.30852E+01_JPRB, 0.29713E+01_JPRB, 0.37749E+01_JPRB ,&
&  0.30255E+01_JPRB, 0.24816E+01_JPRB, 0.20925E+01_JPRB, 0.15329E+01_JPRB, 0.18537E+01_JPRB, 0.27692E+01_JPRB, 0.28327E+01_JPRB ,&
&  0.26960E+01_JPRB, 0.26482E+01_JPRB, 0.25788E+01_JPRB, 0.21420E+01_JPRB, 0.16219E+01_JPRB, 0.12748E+01_JPRB, 0.16161E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.17141E+01_JPRB, 0.16181E+01_JPRB, 0.13984E+01_JPRB, 0.11668E+01_JPRB, 0.13716E+01_JPRB, 0.14418E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.16896E+01_JPRB, 0.23359E+01_JPRB, 0.30412E+01_JPRB, 0.32315E+01_JPRB, 0.32047E+01_JPRB, 0.34024E+01_JPRB ,&
&  0.30081E+01_JPRB, 0.27695E+01_JPRB, 0.24054E+01_JPRB, 0.20714E+01_JPRB, 0.18411E+01_JPRB, 0.16772E+01_JPRB, 0.15494E+01_JPRB ,&
&  0.13981E+01_JPRB, 0.12818E+01_JPRB, 0.11630E+01_JPRB, 0.10194E+01_JPRB, 0.90322E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.79083E+00_JPRB, 0.73502E+00_JPRB, 0.66412E+00_JPRB, 0.64972E+00_JPRB, 0.67662E+00_JPRB, 0.71224E+00_JPRB, 0.71864E+00_JPRB ,&
&  0.68090E+00_JPRB, 0.65689E+00_JPRB, 0.64167E+00_JPRB, 0.64413E+00_JPRB, 0.64431E+00_JPRB, 0.63841E+00_JPRB, 0.64746E+00_JPRB ,&
&  0.69345E+00_JPRB, 0.66431E+00_JPRB, 0.59035E+00_JPRB, 0.64202E+00_JPRB, 0.68347E+00_JPRB, 0.63055E+00_JPRB, 0.71122E+00_JPRB ,&
&  0.81981E+00_JPRB, 0.94468E+00_JPRB, 0.10412E+01_JPRB, 0.12253E+01_JPRB, 0.14661E+01_JPRB, 0.18323E+01_JPRB, 0.22705E+01_JPRB ,&
&  0.24293E+01_JPRB, 0.25881E+01_JPRB, 0.26580E+01_JPRB, 0.24610E+01_JPRB, 0.20472E+01_JPRB, 0.17047E+01_JPRB, 0.15068E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.11174E+01_JPRB, 0.10216E+01_JPRB, 0.10143E+01_JPRB, 0.10700E+01_JPRB, 0.11379E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.11950E+01_JPRB, 0.12257E+01_JPRB, 0.12758E+01_JPRB, 0.14501E+01_JPRB, 0.12115E+01_JPRB, 0.12361E+01_JPRB, 0.12902E+01_JPRB ,&
&  0.19479E+01_JPRB, 0.21522E+01_JPRB, 0.23242E+01_JPRB, 0.29478E+01_JPRB, 0.26641E+01_JPRB, 0.30319E+01_JPRB, 0.38919E+01_JPRB ,&
&  0.41910E+01_JPRB, 0.41939E+01_JPRB, 0.41750E+01_JPRB, 0.26308E+01_JPRB, 0.22743E+01_JPRB, 0.30999E+01_JPRB, 0.30224E+01_JPRB ,&
&  0.28044E+01_JPRB, 0.28127E+01_JPRB, 0.26668E+01_JPRB, 0.27378E+01_JPRB, 0.20378E+01_JPRB, 0.17337E+01_JPRB, 0.13205E+01_JPRB ,&
&  0.12795E+01_JPRB, 0.11502E+01_JPRB, 0.13736E+01_JPRB, 0.12638E+01_JPRB, 0.12733E+01_JPRB, 0.12641E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.13050E+01_JPRB, 0.13976E+01_JPRB, 0.16005E+01_JPRB, 0.21669E+01_JPRB, 0.27801E+01_JPRB, 0.26267E+01_JPRB, 0.29006E+01_JPRB ,&
&  0.28507E+01_JPRB, 0.25624E+01_JPRB, 0.21512E+01_JPRB, 0.19490E+01_JPRB, 0.17705E+01_JPRB, 0.16161E+01_JPRB, 0.14644E+01_JPRB ,&
&  0.13086E+01_JPRB, 0.11893E+01_JPRB, 0.10764E+01_JPRB, 0.95925E+00_JPRB, 0.85718E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.78256E+00_JPRB, 0.73086E+00_JPRB, 0.65144E+00_JPRB, 0.63125E+00_JPRB, 0.63731E+00_JPRB, 0.67125E+00_JPRB, 0.69252E+00_JPRB ,&
&  0.68245E+00_JPRB, 0.65804E+00_JPRB, 0.64576E+00_JPRB, 0.64601E+00_JPRB, 0.64635E+00_JPRB, 0.64355E+00_JPRB, 0.66239E+00_JPRB ,&
&  0.69235E+00_JPRB, 0.73694E+00_JPRB, 0.65118E+00_JPRB, 0.59839E+00_JPRB, 0.56350E+00_JPRB, 0.62902E+00_JPRB, 0.74494E+00_JPRB ,&
&  0.85651E+00_JPRB, 0.93459E+00_JPRB, 0.10332E+01_JPRB, 0.11126E+01_JPRB, 0.11507E+01_JPRB, 0.13355E+01_JPRB, 0.17318E+01_JPRB ,&
&  0.24174E+01_JPRB, 0.23859E+01_JPRB, 0.25496E+01_JPRB, 0.24189E+01_JPRB, 0.21850E+01_JPRB, 0.19278E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.14822E+01_JPRB, 0.12680E+01_JPRB, 0.11117E+01_JPRB, 0.10209E+01_JPRB, 0.10086E+01_JPRB, 0.10336E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.11692E+01_JPRB, 0.11678E+01_JPRB, 0.11462E+01_JPRB, 0.12041E+01_JPRB, 0.13623E+01_JPRB, 0.14819E+01_JPRB, 0.16292E+01_JPRB ,&
&  0.16773E+01_JPRB, 0.17154E+01_JPRB, 0.24764E+01_JPRB, 0.27998E+01_JPRB, 0.35450E+01_JPRB, 0.39444E+01_JPRB, 0.38929E+01_JPRB ,&
&  0.40011E+01_JPRB, 0.38823E+01_JPRB, 0.38496E+01_JPRB, 0.31758E+01_JPRB, 0.36650E+01_JPRB, 0.35633E+01_JPRB, 0.32865E+01_JPRB ,&
&  0.30311E+01_JPRB, 0.28602E+01_JPRB, 0.27770E+01_JPRB, 0.27379E+01_JPRB, 0.27629E+01_JPRB, 0.28242E+01_JPRB, 0.21742E+01_JPRB ,&
&  0.16599E+01_JPRB, 0.18098E+01_JPRB, 0.16214E+01_JPRB, 0.13420E+01_JPRB, 0.11126E+01_JPRB, 0.11588E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12800E+01_JPRB, 0.14104E+01_JPRB, 0.17867E+01_JPRB, 0.23741E+01_JPRB, 0.26423E+01_JPRB, 0.26443E+01_JPRB ,&
&  0.23638E+01_JPRB, 0.23699E+01_JPRB, 0.21668E+01_JPRB, 0.18743E+01_JPRB, 0.16745E+01_JPRB, 0.15058E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.11929E+01_JPRB, 0.10911E+01_JPRB, 0.10058E+01_JPRB, 0.92011E+00_JPRB, 0.83372E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.75954E+00_JPRB, 0.72688E+00_JPRB, 0.66461E+00_JPRB, 0.64138E+00_JPRB, 0.63877E+00_JPRB, 0.65145E+00_JPRB, 0.66569E+00_JPRB ,&
&  0.66830E+00_JPRB, 0.64361E+00_JPRB, 0.62835E+00_JPRB, 0.63082E+00_JPRB, 0.64273E+00_JPRB, 0.65831E+00_JPRB, 0.69297E+00_JPRB ,&
&  0.76097E+00_JPRB, 0.89458E+00_JPRB, 0.74855E+00_JPRB, 0.71634E+00_JPRB, 0.64241E+00_JPRB, 0.70587E+00_JPRB, 0.80581E+00_JPRB ,&
&  0.93070E+00_JPRB, 0.94740E+00_JPRB, 0.92159E+00_JPRB, 0.95298E+00_JPRB, 0.98294E+00_JPRB, 0.10748E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.16422E+01_JPRB, 0.20557E+01_JPRB, 0.21763E+01_JPRB, 0.23480E+01_JPRB, 0.21450E+01_JPRB, 0.18168E+01_JPRB, 0.17692E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.13876E+01_JPRB, 0.12093E+01_JPRB, 0.11087E+01_JPRB, 0.10573E+01_JPRB, 0.10218E+01_JPRB, 0.10407E+01_JPRB ,&
&  0.10703E+01_JPRB, 0.10913E+01_JPRB, 0.10919E+01_JPRB, 0.10937E+01_JPRB, 0.11950E+01_JPRB, 0.14098E+01_JPRB, 0.15486E+01_JPRB ,&
&  0.21149E+01_JPRB, 0.21786E+01_JPRB, 0.19027E+01_JPRB, 0.27238E+01_JPRB, 0.40945E+01_JPRB, 0.40431E+01_JPRB, 0.35213E+01_JPRB ,&
&  0.36969E+01_JPRB, 0.36648E+01_JPRB, 0.37405E+01_JPRB, 0.38367E+01_JPRB, 0.38351E+01_JPRB, 0.36920E+01_JPRB, 0.34454E+01_JPRB ,&
&  0.31301E+01_JPRB, 0.29328E+01_JPRB, 0.28072E+01_JPRB, 0.28092E+01_JPRB, 0.26729E+01_JPRB, 0.25756E+01_JPRB, 0.25067E+01_JPRB ,&
&  0.21160E+01_JPRB, 0.17503E+01_JPRB, 0.11845E+01_JPRB, 0.12289E+01_JPRB, 0.95997E+00_JPRB, 0.98863E+00_JPRB, 0.11842E+01_JPRB ,&
&  0.11921E+01_JPRB, 0.13225E+01_JPRB, 0.14890E+01_JPRB, 0.16084E+01_JPRB, 0.20667E+01_JPRB, 0.22791E+01_JPRB, 0.23529E+01_JPRB ,&
&  0.22692E+01_JPRB, 0.19336E+01_JPRB, 0.19363E+01_JPRB, 0.17558E+01_JPRB, 0.16341E+01_JPRB, 0.14197E+01_JPRB, 0.12034E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.96506E+00_JPRB, 0.90523E+00_JPRB, 0.84474E+00_JPRB, 0.78874E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.69485E+00_JPRB, 0.67100E+00_JPRB, 0.63314E+00_JPRB, 0.62430E+00_JPRB, 0.61980E+00_JPRB, 0.62009E+00_JPRB, 0.63594E+00_JPRB ,&
&  0.65309E+00_JPRB, 0.63920E+00_JPRB, 0.61109E+00_JPRB, 0.60343E+00_JPRB, 0.61460E+00_JPRB, 0.64740E+00_JPRB, 0.72508E+00_JPRB ,&
&  0.87120E+00_JPRB, 0.91032E+00_JPRB, 0.79058E+00_JPRB, 0.65834E+00_JPRB, 0.74454E+00_JPRB, 0.73243E+00_JPRB, 0.80527E+00_JPRB ,&
&  0.78089E+00_JPRB, 0.85258E+00_JPRB, 0.81626E+00_JPRB, 0.79958E+00_JPRB, 0.79677E+00_JPRB, 0.85872E+00_JPRB, 0.98823E+00_JPRB ,&
&  0.11621E+01_JPRB, 0.14466E+01_JPRB, 0.17150E+01_JPRB, 0.17191E+01_JPRB, 0.17930E+01_JPRB, 0.16186E+01_JPRB, 0.15936E+01_JPRB ,&
&  0.14802E+01_JPRB, 0.13173E+01_JPRB, 0.11668E+01_JPRB, 0.10755E+01_JPRB, 0.10253E+01_JPRB, 0.99684E+00_JPRB, 0.10021E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.10498E+01_JPRB, 0.10346E+01_JPRB, 0.10359E+01_JPRB, 0.11183E+01_JPRB, 0.13359E+01_JPRB, 0.17288E+01_JPRB ,&
&  0.23553E+01_JPRB, 0.26661E+01_JPRB, 0.28590E+01_JPRB, 0.34442E+01_JPRB, 0.37318E+01_JPRB, 0.37699E+01_JPRB, 0.36471E+01_JPRB ,&
&  0.34546E+01_JPRB, 0.32721E+01_JPRB, 0.33497E+01_JPRB, 0.34098E+01_JPRB, 0.34979E+01_JPRB, 0.35993E+01_JPRB, 0.34831E+01_JPRB ,&
&  0.31007E+01_JPRB, 0.28817E+01_JPRB, 0.28127E+01_JPRB, 0.26011E+01_JPRB, 0.25155E+01_JPRB, 0.23524E+01_JPRB, 0.24364E+01_JPRB ,&
&  0.24503E+01_JPRB, 0.14220E+01_JPRB, 0.11237E+01_JPRB, 0.13376E+01_JPRB, 0.11279E+01_JPRB, 0.11881E+01_JPRB, 0.14138E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.14455E+01_JPRB, 0.16245E+01_JPRB, 0.17075E+01_JPRB, 0.18212E+01_JPRB, 0.20892E+01_JPRB, 0.20366E+01_JPRB ,&
&  0.18773E+01_JPRB, 0.17038E+01_JPRB, 0.17107E+01_JPRB, 0.15965E+01_JPRB, 0.14348E+01_JPRB, 0.11874E+01_JPRB, 0.10095E+01_JPRB ,&
&  0.90475E+00_JPRB, 0.84685E+00_JPRB, 0.80441E+00_JPRB, 0.76070E+00_JPRB, 0.71519E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.65479E+00_JPRB, 0.62863E+00_JPRB, 0.60241E+00_JPRB, 0.59998E+00_JPRB, 0.60950E+00_JPRB, 0.61059E+00_JPRB, 0.62584E+00_JPRB ,&
&  0.64250E+00_JPRB, 0.64157E+00_JPRB, 0.61717E+00_JPRB, 0.59953E+00_JPRB, 0.59915E+00_JPRB, 0.63072E+00_JPRB, 0.70623E+00_JPRB ,&
&  0.65135E+00_JPRB, 0.67624E+00_JPRB, 0.61046E+00_JPRB, 0.63521E+00_JPRB, 0.76148E+00_JPRB, 0.68941E+00_JPRB, 0.68072E+00_JPRB ,&
&  0.65841E+00_JPRB, 0.64953E+00_JPRB, 0.62411E+00_JPRB, 0.61169E+00_JPRB, 0.61851E+00_JPRB, 0.66095E+00_JPRB, 0.74464E+00_JPRB ,&
&  0.80839E+00_JPRB, 0.97012E+00_JPRB, 0.11311E+01_JPRB, 0.11233E+01_JPRB, 0.12374E+01_JPRB, 0.13157E+01_JPRB, 0.13676E+01_JPRB ,&
&  0.13212E+01_JPRB, 0.11824E+01_JPRB, 0.10533E+01_JPRB, 0.97770E+00_JPRB, 0.93198E+00_JPRB, 0.91406E+00_JPRB, 0.90390E+00_JPRB ,&
&  0.89886E+00_JPRB, 0.90569E+00_JPRB, 0.91683E+00_JPRB, 0.95464E+00_JPRB, 0.10145E+01_JPRB, 0.13191E+01_JPRB, 0.18422E+01_JPRB ,&
&  0.23664E+01_JPRB, 0.25615E+01_JPRB, 0.28304E+01_JPRB, 0.33330E+01_JPRB, 0.34065E+01_JPRB, 0.32701E+01_JPRB, 0.29792E+01_JPRB ,&
&  0.28135E+01_JPRB, 0.26885E+01_JPRB, 0.28803E+01_JPRB, 0.31303E+01_JPRB, 0.31500E+01_JPRB, 0.34675E+01_JPRB, 0.34249E+01_JPRB ,&
&  0.31263E+01_JPRB, 0.29450E+01_JPRB, 0.28353E+01_JPRB, 0.27983E+01_JPRB, 0.27156E+01_JPRB, 0.28208E+01_JPRB, 0.28153E+01_JPRB ,&
&  0.27676E+01_JPRB, 0.24235E+01_JPRB, 0.18418E+01_JPRB, 0.17214E+01_JPRB, 0.13287E+01_JPRB, 0.13789E+01_JPRB, 0.17583E+01_JPRB ,&
&  0.17353E+01_JPRB, 0.17029E+01_JPRB, 0.16939E+01_JPRB, 0.17068E+01_JPRB, 0.16763E+01_JPRB, 0.18538E+01_JPRB, 0.18374E+01_JPRB ,&
&  0.15873E+01_JPRB, 0.15709E+01_JPRB, 0.14809E+01_JPRB, 0.13171E+01_JPRB, 0.11652E+01_JPRB, 0.97696E+00_JPRB, 0.84766E+00_JPRB ,&
&  0.80435E+00_JPRB, 0.76231E+00_JPRB, 0.74077E+00_JPRB, 0.70979E+00_JPRB, 0.67532E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.63473E+00_JPRB, 0.61248E+00_JPRB, 0.59267E+00_JPRB, 0.58756E+00_JPRB, 0.60180E+00_JPRB, 0.60711E+00_JPRB, 0.62155E+00_JPRB ,&
&  0.62972E+00_JPRB, 0.62923E+00_JPRB, 0.62308E+00_JPRB, 0.60041E+00_JPRB, 0.59660E+00_JPRB, 0.61526E+00_JPRB, 0.60560E+00_JPRB ,&
&  0.55050E+00_JPRB, 0.57379E+00_JPRB, 0.56326E+00_JPRB, 0.57478E+00_JPRB, 0.58250E+00_JPRB, 0.55426E+00_JPRB, 0.53793E+00_JPRB ,&
&  0.55519E+00_JPRB, 0.54528E+00_JPRB, 0.55057E+00_JPRB, 0.56615E+00_JPRB, 0.58827E+00_JPRB, 0.62304E+00_JPRB, 0.66811E+00_JPRB ,&
&  0.68339E+00_JPRB, 0.76168E+00_JPRB, 0.84155E+00_JPRB, 0.82194E+00_JPRB, 0.93527E+00_JPRB, 0.10605E+01_JPRB, 0.11041E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.10535E+01_JPRB, 0.96943E+00_JPRB, 0.90042E+00_JPRB, 0.83728E+00_JPRB, 0.80621E+00_JPRB, 0.79689E+00_JPRB ,&
&  0.79407E+00_JPRB, 0.79182E+00_JPRB, 0.79331E+00_JPRB, 0.81331E+00_JPRB, 0.85305E+00_JPRB, 0.10916E+01_JPRB, 0.13831E+01_JPRB ,&
&  0.16022E+01_JPRB, 0.17698E+01_JPRB, 0.19529E+01_JPRB, 0.20723E+01_JPRB, 0.21853E+01_JPRB, 0.22269E+01_JPRB, 0.21710E+01_JPRB ,&
&  0.22898E+01_JPRB, 0.22842E+01_JPRB, 0.28962E+01_JPRB, 0.30076E+01_JPRB, 0.30718E+01_JPRB, 0.32531E+01_JPRB, 0.32371E+01_JPRB ,&
&  0.32389E+01_JPRB, 0.30384E+01_JPRB, 0.29866E+01_JPRB, 0.28875E+01_JPRB, 0.28122E+01_JPRB, 0.28797E+01_JPRB, 0.27763E+01_JPRB ,&
&  0.27802E+01_JPRB, 0.27587E+01_JPRB, 0.24493E+01_JPRB, 0.22158E+01_JPRB, 0.19166E+01_JPRB, 0.20443E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.17767E+01_JPRB, 0.14672E+01_JPRB, 0.15604E+01_JPRB, 0.15916E+01_JPRB, 0.16054E+01_JPRB, 0.15628E+01_JPRB, 0.14717E+01_JPRB ,&
&  0.14474E+01_JPRB, 0.14365E+01_JPRB, 0.12200E+01_JPRB, 0.10438E+01_JPRB, 0.93699E+00_JPRB, 0.83896E+00_JPRB, 0.70710E+00_JPRB ,&
&  0.66993E+00_JPRB, 0.69536E+00_JPRB, 0.69038E+00_JPRB, 0.67624E+00_JPRB, 0.65586E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.62616E+00_JPRB, 0.61397E+00_JPRB, 0.60006E+00_JPRB, 0.58925E+00_JPRB, 0.60138E+00_JPRB, 0.60448E+00_JPRB, 0.59979E+00_JPRB ,&
&  0.61668E+00_JPRB, 0.60845E+00_JPRB, 0.60520E+00_JPRB, 0.59370E+00_JPRB, 0.58511E+00_JPRB, 0.54003E+00_JPRB, 0.45860E+00_JPRB ,&
&  0.43917E+00_JPRB, 0.45933E+00_JPRB, 0.49776E+00_JPRB, 0.51710E+00_JPRB, 0.50848E+00_JPRB, 0.49084E+00_JPRB, 0.48138E+00_JPRB ,&
&  0.48920E+00_JPRB, 0.50278E+00_JPRB, 0.53262E+00_JPRB, 0.55998E+00_JPRB, 0.57568E+00_JPRB, 0.58914E+00_JPRB, 0.61237E+00_JPRB ,&
&  0.61630E+00_JPRB, 0.65669E+00_JPRB, 0.72396E+00_JPRB, 0.66641E+00_JPRB, 0.74245E+00_JPRB, 0.82492E+00_JPRB, 0.83865E+00_JPRB ,&
&  0.87775E+00_JPRB, 0.86758E+00_JPRB, 0.82497E+00_JPRB, 0.80062E+00_JPRB, 0.75530E+00_JPRB, 0.71416E+00_JPRB, 0.69444E+00_JPRB ,&
&  0.68824E+00_JPRB, 0.68845E+00_JPRB, 0.69589E+00_JPRB, 0.70576E+00_JPRB, 0.73177E+00_JPRB, 0.85276E+00_JPRB, 0.97665E+00_JPRB ,&
&  0.10505E+01_JPRB, 0.11652E+01_JPRB, 0.13516E+01_JPRB, 0.13786E+01_JPRB, 0.14960E+01_JPRB, 0.16685E+01_JPRB, 0.17512E+01_JPRB ,&
&  0.19846E+01_JPRB, 0.20191E+01_JPRB, 0.22824E+01_JPRB, 0.24792E+01_JPRB, 0.26723E+01_JPRB, 0.27874E+01_JPRB, 0.28670E+01_JPRB ,&
&  0.30518E+01_JPRB, 0.31292E+01_JPRB, 0.28702E+01_JPRB, 0.27024E+01_JPRB, 0.25595E+01_JPRB, 0.25457E+01_JPRB, 0.24807E+01_JPRB ,&
&  0.25022E+01_JPRB, 0.25610E+01_JPRB, 0.25533E+01_JPRB, 0.24981E+01_JPRB, 0.23780E+01_JPRB, 0.21970E+01_JPRB, 0.20136E+01_JPRB ,&
&  0.19048E+01_JPRB, 0.16895E+01_JPRB, 0.15652E+01_JPRB, 0.15652E+01_JPRB, 0.15507E+01_JPRB, 0.13398E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.13120E+01_JPRB, 0.11352E+01_JPRB, 0.10476E+01_JPRB, 0.92161E+00_JPRB, 0.86029E+00_JPRB, 0.79674E+00_JPRB, 0.70643E+00_JPRB ,&
&  0.63792E+00_JPRB, 0.67614E+00_JPRB, 0.66546E+00_JPRB, 0.64383E+00_JPRB, 0.63352E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.62658E+00_JPRB, 0.62410E+00_JPRB, 0.60768E+00_JPRB, 0.59336E+00_JPRB, 0.59448E+00_JPRB, 0.57562E+00_JPRB, 0.56976E+00_JPRB ,&
&  0.57391E+00_JPRB, 0.55998E+00_JPRB, 0.58302E+00_JPRB, 0.50640E+00_JPRB, 0.46139E+00_JPRB, 0.45204E+00_JPRB, 0.43702E+00_JPRB ,&
&  0.43687E+00_JPRB, 0.45159E+00_JPRB, 0.46354E+00_JPRB, 0.47362E+00_JPRB, 0.48180E+00_JPRB, 0.47073E+00_JPRB, 0.46709E+00_JPRB ,&
&  0.47675E+00_JPRB, 0.50009E+00_JPRB, 0.53672E+00_JPRB, 0.55389E+00_JPRB, 0.55954E+00_JPRB, 0.56031E+00_JPRB, 0.56493E+00_JPRB ,&
&  0.55400E+00_JPRB, 0.57642E+00_JPRB, 0.62615E+00_JPRB, 0.62371E+00_JPRB, 0.64368E+00_JPRB, 0.69499E+00_JPRB, 0.69254E+00_JPRB ,&
&  0.70735E+00_JPRB, 0.63512E+00_JPRB, 0.66591E+00_JPRB, 0.67118E+00_JPRB, 0.66414E+00_JPRB, 0.63965E+00_JPRB, 0.61546E+00_JPRB ,&
&  0.60273E+00_JPRB, 0.60182E+00_JPRB, 0.60769E+00_JPRB, 0.61741E+00_JPRB, 0.62644E+00_JPRB, 0.65947E+00_JPRB, 0.69306E+00_JPRB ,&
&  0.74622E+00_JPRB, 0.72249E+00_JPRB, 0.88394E+00_JPRB, 0.95811E+00_JPRB, 0.11268E+01_JPRB, 0.13623E+01_JPRB, 0.15545E+01_JPRB ,&
&  0.19538E+01_JPRB, 0.19667E+01_JPRB, 0.21064E+01_JPRB, 0.21583E+01_JPRB, 0.22192E+01_JPRB, 0.23031E+01_JPRB, 0.24101E+01_JPRB ,&
&  0.25488E+01_JPRB, 0.23925E+01_JPRB, 0.24862E+01_JPRB, 0.23285E+01_JPRB, 0.22190E+01_JPRB, 0.22141E+01_JPRB, 0.22203E+01_JPRB ,&
&  0.22587E+01_JPRB, 0.22790E+01_JPRB, 0.22504E+01_JPRB, 0.19871E+01_JPRB, 0.19396E+01_JPRB, 0.19243E+01_JPRB, 0.17939E+01_JPRB ,&
&  0.18227E+01_JPRB, 0.17953E+01_JPRB, 0.16671E+01_JPRB, 0.17449E+01_JPRB, 0.15492E+01_JPRB, 0.14164E+01_JPRB, 0.13548E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.10052E+01_JPRB, 0.93273E+00_JPRB, 0.88081E+00_JPRB, 0.80909E+00_JPRB, 0.73790E+00_JPRB, 0.74780E+00_JPRB ,&
&  0.68378E+00_JPRB, 0.67139E+00_JPRB, 0.67958E+00_JPRB, 0.65573E+00_JPRB, 0.63486E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.64820E+00_JPRB, 0.63729E+00_JPRB, 0.61706E+00_JPRB, 0.60023E+00_JPRB, 0.59232E+00_JPRB, 0.57174E+00_JPRB, 0.53730E+00_JPRB ,&
&  0.51764E+00_JPRB, 0.48790E+00_JPRB, 0.46647E+00_JPRB, 0.45461E+00_JPRB, 0.45020E+00_JPRB, 0.44336E+00_JPRB, 0.42707E+00_JPRB ,&
&  0.42388E+00_JPRB, 0.43275E+00_JPRB, 0.45012E+00_JPRB, 0.45579E+00_JPRB, 0.45533E+00_JPRB, 0.45504E+00_JPRB, 0.46153E+00_JPRB ,&
&  0.47865E+00_JPRB, 0.50642E+00_JPRB, 0.53743E+00_JPRB, 0.55286E+00_JPRB, 0.55007E+00_JPRB, 0.54516E+00_JPRB, 0.54532E+00_JPRB ,&
&  0.54118E+00_JPRB, 0.54156E+00_JPRB, 0.55398E+00_JPRB, 0.57201E+00_JPRB, 0.61229E+00_JPRB, 0.63731E+00_JPRB, 0.61898E+00_JPRB ,&
&  0.47983E+00_JPRB, 0.46336E+00_JPRB, 0.54191E+00_JPRB, 0.55503E+00_JPRB, 0.56387E+00_JPRB, 0.55313E+00_JPRB, 0.54441E+00_JPRB ,&
&  0.53830E+00_JPRB, 0.53586E+00_JPRB, 0.53316E+00_JPRB, 0.53518E+00_JPRB, 0.54111E+00_JPRB, 0.55107E+00_JPRB, 0.55870E+00_JPRB ,&
&  0.58087E+00_JPRB, 0.55601E+00_JPRB, 0.58352E+00_JPRB, 0.65115E+00_JPRB, 0.91204E+00_JPRB, 0.11383E+01_JPRB, 0.13309E+01_JPRB ,&
&  0.15570E+01_JPRB, 0.17332E+01_JPRB, 0.19365E+01_JPRB, 0.20663E+01_JPRB, 0.21207E+01_JPRB, 0.21354E+01_JPRB, 0.22184E+01_JPRB ,&
&  0.22524E+01_JPRB, 0.22358E+01_JPRB, 0.22090E+01_JPRB, 0.21533E+01_JPRB, 0.20283E+01_JPRB, 0.19832E+01_JPRB, 0.19642E+01_JPRB ,&
&  0.19980E+01_JPRB, 0.20188E+01_JPRB, 0.19282E+01_JPRB, 0.18041E+01_JPRB, 0.17050E+01_JPRB, 0.16988E+01_JPRB, 0.17140E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.17169E+01_JPRB, 0.16821E+01_JPRB, 0.16188E+01_JPRB, 0.14985E+01_JPRB, 0.13696E+01_JPRB, 0.12753E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.94970E+00_JPRB, 0.81836E+00_JPRB, 0.80138E+00_JPRB, 0.80768E+00_JPRB, 0.76716E+00_JPRB, 0.70640E+00_JPRB ,&
&  0.68532E+00_JPRB, 0.66295E+00_JPRB, 0.63676E+00_JPRB, 0.64816E+00_JPRB, 0.64909E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.61823E+00_JPRB, 0.60672E+00_JPRB, 0.59130E+00_JPRB, 0.58194E+00_JPRB, 0.56476E+00_JPRB, 0.54413E+00_JPRB, 0.52262E+00_JPRB ,&
&  0.51197E+00_JPRB, 0.49600E+00_JPRB, 0.47899E+00_JPRB, 0.45891E+00_JPRB, 0.44455E+00_JPRB, 0.43327E+00_JPRB, 0.43477E+00_JPRB ,&
&  0.44350E+00_JPRB, 0.45229E+00_JPRB, 0.45320E+00_JPRB, 0.44690E+00_JPRB, 0.44343E+00_JPRB, 0.44946E+00_JPRB, 0.46645E+00_JPRB ,&
&  0.49264E+00_JPRB, 0.52353E+00_JPRB, 0.54463E+00_JPRB, 0.55094E+00_JPRB, 0.54873E+00_JPRB, 0.54533E+00_JPRB, 0.53685E+00_JPRB ,&
&  0.51887E+00_JPRB, 0.51169E+00_JPRB, 0.51956E+00_JPRB, 0.54035E+00_JPRB, 0.57589E+00_JPRB, 0.57814E+00_JPRB, 0.50291E+00_JPRB ,&
&  0.40695E+00_JPRB, 0.37357E+00_JPRB, 0.37963E+00_JPRB, 0.44219E+00_JPRB, 0.46165E+00_JPRB, 0.47474E+00_JPRB, 0.48510E+00_JPRB ,&
&  0.47300E+00_JPRB, 0.46112E+00_JPRB, 0.46763E+00_JPRB, 0.49612E+00_JPRB, 0.50556E+00_JPRB, 0.51418E+00_JPRB, 0.52179E+00_JPRB ,&
&  0.53176E+00_JPRB, 0.55076E+00_JPRB, 0.58080E+00_JPRB, 0.59060E+00_JPRB, 0.74839E+00_JPRB, 0.10756E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.14727E+01_JPRB, 0.16527E+01_JPRB, 0.19129E+01_JPRB, 0.19944E+01_JPRB, 0.20040E+01_JPRB, 0.20461E+01_JPRB, 0.21406E+01_JPRB ,&
&  0.21175E+01_JPRB, 0.20833E+01_JPRB, 0.20322E+01_JPRB, 0.20353E+01_JPRB, 0.19732E+01_JPRB, 0.18960E+01_JPRB, 0.18275E+01_JPRB ,&
&  0.18284E+01_JPRB, 0.18017E+01_JPRB, 0.17126E+01_JPRB, 0.15864E+01_JPRB, 0.15346E+01_JPRB, 0.15106E+01_JPRB, 0.15323E+01_JPRB ,&
&  0.15333E+01_JPRB, 0.15542E+01_JPRB, 0.15583E+01_JPRB, 0.15761E+01_JPRB, 0.14095E+01_JPRB, 0.12056E+01_JPRB, 0.10591E+01_JPRB ,&
&  0.98456E+00_JPRB, 0.92152E+00_JPRB, 0.84758E+00_JPRB, 0.85841E+00_JPRB, 0.86918E+00_JPRB, 0.84341E+00_JPRB, 0.74109E+00_JPRB ,&
&  0.66214E+00_JPRB, 0.62855E+00_JPRB, 0.60907E+00_JPRB, 0.62064E+00_JPRB, 0.62675E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.54460E+00_JPRB, 0.55100E+00_JPRB, 0.56144E+00_JPRB, 0.55957E+00_JPRB, 0.55415E+00_JPRB, 0.51705E+00_JPRB, 0.48186E+00_JPRB ,&
&  0.46029E+00_JPRB, 0.45190E+00_JPRB, 0.45311E+00_JPRB, 0.45529E+00_JPRB, 0.45733E+00_JPRB, 0.46106E+00_JPRB, 0.46444E+00_JPRB ,&
&  0.46239E+00_JPRB, 0.45710E+00_JPRB, 0.45295E+00_JPRB, 0.45254E+00_JPRB, 0.45909E+00_JPRB, 0.47460E+00_JPRB, 0.49830E+00_JPRB ,&
&  0.52912E+00_JPRB, 0.55405E+00_JPRB, 0.56225E+00_JPRB, 0.55677E+00_JPRB, 0.54465E+00_JPRB, 0.53445E+00_JPRB, 0.51962E+00_JPRB ,&
&  0.49846E+00_JPRB, 0.49181E+00_JPRB, 0.49776E+00_JPRB, 0.51660E+00_JPRB, 0.54208E+00_JPRB, 0.54298E+00_JPRB, 0.46649E+00_JPRB ,&
&  0.38671E+00_JPRB, 0.33843E+00_JPRB, 0.31904E+00_JPRB, 0.32853E+00_JPRB, 0.36400E+00_JPRB, 0.42661E+00_JPRB, 0.44395E+00_JPRB ,&
&  0.45998E+00_JPRB, 0.47804E+00_JPRB, 0.49317E+00_JPRB, 0.50784E+00_JPRB, 0.51955E+00_JPRB, 0.52665E+00_JPRB, 0.53131E+00_JPRB ,&
&  0.53720E+00_JPRB, 0.55196E+00_JPRB, 0.57872E+00_JPRB, 0.58848E+00_JPRB, 0.63837E+00_JPRB, 0.83689E+00_JPRB, 0.10861E+01_JPRB ,&
&  0.13169E+01_JPRB, 0.14175E+01_JPRB, 0.15623E+01_JPRB, 0.18167E+01_JPRB, 0.19999E+01_JPRB, 0.21251E+01_JPRB, 0.21734E+01_JPRB ,&
&  0.21601E+01_JPRB, 0.20438E+01_JPRB, 0.19523E+01_JPRB, 0.19285E+01_JPRB, 0.19189E+01_JPRB, 0.19137E+01_JPRB, 0.18505E+01_JPRB ,&
&  0.18104E+01_JPRB, 0.17374E+01_JPRB, 0.16040E+01_JPRB, 0.14662E+01_JPRB, 0.14351E+01_JPRB, 0.14749E+01_JPRB, 0.15518E+01_JPRB ,&
&  0.15614E+01_JPRB, 0.15546E+01_JPRB, 0.15433E+01_JPRB, 0.15675E+01_JPRB, 0.13702E+01_JPRB, 0.12373E+01_JPRB, 0.11357E+01_JPRB ,&
&  0.11015E+01_JPRB, 0.10747E+01_JPRB, 0.10317E+01_JPRB, 0.10563E+01_JPRB, 0.10284E+01_JPRB, 0.97278E+00_JPRB, 0.88125E+00_JPRB ,&
&  0.77587E+00_JPRB, 0.69432E+00_JPRB, 0.63973E+00_JPRB, 0.59235E+00_JPRB, 0.56528E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.59265E+00_JPRB, 0.56739E+00_JPRB, 0.55405E+00_JPRB, 0.54799E+00_JPRB, 0.54299E+00_JPRB, 0.52516E+00_JPRB, 0.50938E+00_JPRB ,&
&  0.49900E+00_JPRB, 0.49679E+00_JPRB, 0.49475E+00_JPRB, 0.49008E+00_JPRB, 0.48700E+00_JPRB, 0.48440E+00_JPRB, 0.48229E+00_JPRB ,&
&  0.48049E+00_JPRB, 0.48101E+00_JPRB, 0.48510E+00_JPRB, 0.48824E+00_JPRB, 0.49788E+00_JPRB, 0.51779E+00_JPRB, 0.54788E+00_JPRB ,&
&  0.57959E+00_JPRB, 0.59194E+00_JPRB, 0.58653E+00_JPRB, 0.57231E+00_JPRB, 0.54496E+00_JPRB, 0.52108E+00_JPRB, 0.50123E+00_JPRB ,&
&  0.48087E+00_JPRB, 0.47639E+00_JPRB, 0.48970E+00_JPRB, 0.50181E+00_JPRB, 0.51635E+00_JPRB, 0.50445E+00_JPRB, 0.43002E+00_JPRB ,&
&  0.36369E+00_JPRB, 0.31727E+00_JPRB, 0.29472E+00_JPRB, 0.28921E+00_JPRB, 0.30056E+00_JPRB, 0.32947E+00_JPRB, 0.37934E+00_JPRB ,&
&  0.47373E+00_JPRB, 0.53559E+00_JPRB, 0.55732E+00_JPRB, 0.58358E+00_JPRB, 0.58720E+00_JPRB, 0.58287E+00_JPRB, 0.58069E+00_JPRB ,&
&  0.59331E+00_JPRB, 0.61803E+00_JPRB, 0.65478E+00_JPRB, 0.69157E+00_JPRB, 0.74587E+00_JPRB, 0.81795E+00_JPRB, 0.96722E+00_JPRB ,&
&  0.13011E+01_JPRB, 0.13955E+01_JPRB, 0.13440E+01_JPRB, 0.15491E+01_JPRB, 0.18414E+01_JPRB, 0.20738E+01_JPRB, 0.22094E+01_JPRB ,&
&  0.21924E+01_JPRB, 0.20750E+01_JPRB, 0.19787E+01_JPRB, 0.19159E+01_JPRB, 0.18775E+01_JPRB, 0.18517E+01_JPRB, 0.18434E+01_JPRB ,&
&  0.18564E+01_JPRB, 0.18949E+01_JPRB, 0.19887E+01_JPRB, 0.17180E+01_JPRB, 0.16122E+01_JPRB, 0.15975E+01_JPRB, 0.15652E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.15064E+01_JPRB, 0.15162E+01_JPRB, 0.15332E+01_JPRB, 0.14379E+01_JPRB, 0.13511E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.12480E+01_JPRB, 0.12079E+01_JPRB, 0.11689E+01_JPRB, 0.11531E+01_JPRB, 0.11182E+01_JPRB, 0.10493E+01_JPRB, 0.99977E+00_JPRB ,&
&  0.93445E+00_JPRB, 0.83622E+00_JPRB, 0.76970E+00_JPRB, 0.70838E+00_JPRB, 0.64269E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.61516E+00_JPRB, 0.58633E+00_JPRB, 0.56576E+00_JPRB, 0.55227E+00_JPRB, 0.54074E+00_JPRB, 0.53298E+00_JPRB, 0.52345E+00_JPRB ,&
&  0.51538E+00_JPRB, 0.50834E+00_JPRB, 0.50324E+00_JPRB, 0.49916E+00_JPRB, 0.49760E+00_JPRB, 0.49771E+00_JPRB, 0.50021E+00_JPRB ,&
&  0.50470E+00_JPRB, 0.50547E+00_JPRB, 0.51307E+00_JPRB, 0.52926E+00_JPRB, 0.55367E+00_JPRB, 0.57667E+00_JPRB, 0.60226E+00_JPRB ,&
&  0.61032E+00_JPRB, 0.60399E+00_JPRB, 0.58583E+00_JPRB, 0.55864E+00_JPRB, 0.53200E+00_JPRB, 0.50900E+00_JPRB, 0.49492E+00_JPRB ,&
&  0.49244E+00_JPRB, 0.48337E+00_JPRB, 0.48747E+00_JPRB, 0.48748E+00_JPRB, 0.47390E+00_JPRB, 0.46866E+00_JPRB, 0.39188E+00_JPRB ,&
&  0.33507E+00_JPRB, 0.29370E+00_JPRB, 0.28065E+00_JPRB, 0.27477E+00_JPRB, 0.28206E+00_JPRB, 0.29994E+00_JPRB, 0.34334E+00_JPRB ,&
&  0.43133E+00_JPRB, 0.59704E+00_JPRB, 0.62492E+00_JPRB, 0.63790E+00_JPRB, 0.64621E+00_JPRB, 0.64514E+00_JPRB, 0.64561E+00_JPRB ,&
&  0.67130E+00_JPRB, 0.70062E+00_JPRB, 0.74635E+00_JPRB, 0.79099E+00_JPRB, 0.84462E+00_JPRB, 0.88890E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.10098E+01_JPRB, 0.10875E+01_JPRB, 0.11882E+01_JPRB, 0.13635E+01_JPRB, 0.16415E+01_JPRB, 0.19058E+01_JPRB, 0.19748E+01_JPRB ,&
&  0.19489E+01_JPRB, 0.19443E+01_JPRB, 0.18694E+01_JPRB, 0.18127E+01_JPRB, 0.17863E+01_JPRB, 0.17805E+01_JPRB, 0.17848E+01_JPRB ,&
&  0.17951E+01_JPRB, 0.18284E+01_JPRB, 0.19337E+01_JPRB, 0.18500E+01_JPRB, 0.17984E+01_JPRB, 0.17430E+01_JPRB, 0.16743E+01_JPRB ,&
&  0.16172E+01_JPRB, 0.15857E+01_JPRB, 0.15662E+01_JPRB, 0.15328E+01_JPRB, 0.14896E+01_JPRB, 0.14477E+01_JPRB, 0.14160E+01_JPRB ,&
&  0.13726E+01_JPRB, 0.13190E+01_JPRB, 0.12460E+01_JPRB, 0.11967E+01_JPRB, 0.11548E+01_JPRB, 0.11145E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.98497E+00_JPRB, 0.87529E+00_JPRB, 0.79016E+00_JPRB, 0.71938E+00_JPRB, 0.65946E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.67636E+00_JPRB, 0.64227E+00_JPRB, 0.61249E+00_JPRB, 0.59338E+00_JPRB, 0.57509E+00_JPRB, 0.56521E+00_JPRB, 0.55462E+00_JPRB ,&
&  0.54932E+00_JPRB, 0.54493E+00_JPRB, 0.54351E+00_JPRB, 0.54368E+00_JPRB, 0.54643E+00_JPRB, 0.55108E+00_JPRB, 0.55877E+00_JPRB ,&
&  0.56971E+00_JPRB, 0.58356E+00_JPRB, 0.60163E+00_JPRB, 0.61482E+00_JPRB, 0.62650E+00_JPRB, 0.63319E+00_JPRB, 0.64006E+00_JPRB ,&
&  0.63686E+00_JPRB, 0.63170E+00_JPRB, 0.59790E+00_JPRB, 0.56098E+00_JPRB, 0.53066E+00_JPRB, 0.50246E+00_JPRB, 0.49486E+00_JPRB ,&
&  0.48924E+00_JPRB, 0.48009E+00_JPRB, 0.46620E+00_JPRB, 0.45843E+00_JPRB, 0.44737E+00_JPRB, 0.37958E+00_JPRB, 0.36089E+00_JPRB ,&
&  0.32797E+00_JPRB, 0.29444E+00_JPRB, 0.28220E+00_JPRB, 0.27556E+00_JPRB, 0.27807E+00_JPRB, 0.29316E+00_JPRB, 0.32214E+00_JPRB ,&
&  0.39951E+00_JPRB, 0.61784E+00_JPRB, 0.66701E+00_JPRB, 0.68605E+00_JPRB, 0.66266E+00_JPRB, 0.66933E+00_JPRB, 0.67861E+00_JPRB ,&
&  0.71421E+00_JPRB, 0.74987E+00_JPRB, 0.79485E+00_JPRB, 0.83297E+00_JPRB, 0.87208E+00_JPRB, 0.89632E+00_JPRB, 0.94174E+00_JPRB ,&
&  0.10150E+01_JPRB, 0.11096E+01_JPRB, 0.12021E+01_JPRB, 0.12976E+01_JPRB, 0.13883E+01_JPRB, 0.15471E+01_JPRB, 0.17106E+01_JPRB ,&
&  0.17251E+01_JPRB, 0.16984E+01_JPRB, 0.16957E+01_JPRB, 0.16891E+01_JPRB, 0.16882E+01_JPRB, 0.16843E+01_JPRB, 0.16924E+01_JPRB ,&
&  0.17183E+01_JPRB, 0.17412E+01_JPRB, 0.17028E+01_JPRB, 0.16446E+01_JPRB, 0.16131E+01_JPRB, 0.15631E+01_JPRB, 0.15549E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.15694E+01_JPRB, 0.15513E+01_JPRB, 0.15114E+01_JPRB, 0.14700E+01_JPRB, 0.14213E+01_JPRB, 0.13799E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.12843E+01_JPRB, 0.12309E+01_JPRB, 0.11873E+01_JPRB, 0.11528E+01_JPRB, 0.11233E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.90998E+00_JPRB, 0.83624E+00_JPRB, 0.77028E+00_JPRB, 0.71679E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.75298E+00_JPRB, 0.71936E+00_JPRB, 0.69101E+00_JPRB, 0.67039E+00_JPRB, 0.65139E+00_JPRB, 0.63874E+00_JPRB, 0.62707E+00_JPRB ,&
&  0.62113E+00_JPRB, 0.61554E+00_JPRB, 0.61470E+00_JPRB, 0.61428E+00_JPRB, 0.61734E+00_JPRB, 0.62296E+00_JPRB, 0.63035E+00_JPRB ,&
&  0.64016E+00_JPRB, 0.64842E+00_JPRB, 0.65723E+00_JPRB, 0.66116E+00_JPRB, 0.66597E+00_JPRB, 0.66730E+00_JPRB, 0.67192E+00_JPRB ,&
&  0.66418E+00_JPRB, 0.65247E+00_JPRB, 0.61088E+00_JPRB, 0.57278E+00_JPRB, 0.52797E+00_JPRB, 0.49557E+00_JPRB, 0.47394E+00_JPRB ,&
&  0.47646E+00_JPRB, 0.43701E+00_JPRB, 0.43008E+00_JPRB, 0.40139E+00_JPRB, 0.37377E+00_JPRB, 0.35132E+00_JPRB, 0.33190E+00_JPRB ,&
&  0.31783E+00_JPRB, 0.29855E+00_JPRB, 0.29004E+00_JPRB, 0.28977E+00_JPRB, 0.29503E+00_JPRB, 0.31930E+00_JPRB, 0.35676E+00_JPRB ,&
&  0.44272E+00_JPRB, 0.64326E+00_JPRB, 0.70753E+00_JPRB, 0.73705E+00_JPRB, 0.73061E+00_JPRB, 0.74148E+00_JPRB, 0.74268E+00_JPRB ,&
&  0.78741E+00_JPRB, 0.84559E+00_JPRB, 0.90396E+00_JPRB, 0.94725E+00_JPRB, 0.99870E+00_JPRB, 0.10348E+01_JPRB, 0.10692E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.11535E+01_JPRB, 0.11959E+01_JPRB, 0.12304E+01_JPRB, 0.12679E+01_JPRB, 0.12989E+01_JPRB, 0.13419E+01_JPRB ,&
&  0.13686E+01_JPRB, 0.14308E+01_JPRB, 0.14534E+01_JPRB, 0.14841E+01_JPRB, 0.14923E+01_JPRB, 0.14917E+01_JPRB, 0.14979E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.15262E+01_JPRB, 0.15391E+01_JPRB, 0.15355E+01_JPRB, 0.15323E+01_JPRB, 0.15029E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.14693E+01_JPRB, 0.14528E+01_JPRB, 0.14285E+01_JPRB, 0.13970E+01_JPRB, 0.13616E+01_JPRB, 0.13354E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.12652E+01_JPRB, 0.12232E+01_JPRB, 0.11866E+01_JPRB, 0.11545E+01_JPRB, 0.11198E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.99402E+00_JPRB, 0.93415E+00_JPRB, 0.87984E+00_JPRB, 0.83636E+00_JPRB, 0.79203E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.84098E+00_JPRB, 0.81420E+00_JPRB, 0.78671E+00_JPRB, 0.76388E+00_JPRB, 0.74231E+00_JPRB, 0.72636E+00_JPRB, 0.71134E+00_JPRB ,&
&  0.70056E+00_JPRB, 0.69133E+00_JPRB, 0.68557E+00_JPRB, 0.68212E+00_JPRB, 0.68043E+00_JPRB, 0.68231E+00_JPRB, 0.68403E+00_JPRB ,&
&  0.68824E+00_JPRB, 0.69184E+00_JPRB, 0.69739E+00_JPRB, 0.70207E+00_JPRB, 0.70812E+00_JPRB, 0.70809E+00_JPRB, 0.70801E+00_JPRB ,&
&  0.69914E+00_JPRB, 0.68505E+00_JPRB, 0.62923E+00_JPRB, 0.59274E+00_JPRB, 0.54904E+00_JPRB, 0.51464E+00_JPRB, 0.49737E+00_JPRB ,&
&  0.47870E+00_JPRB, 0.46810E+00_JPRB, 0.44989E+00_JPRB, 0.43111E+00_JPRB, 0.41295E+00_JPRB, 0.39168E+00_JPRB, 0.37565E+00_JPRB ,&
&  0.36181E+00_JPRB, 0.34743E+00_JPRB, 0.34115E+00_JPRB, 0.34273E+00_JPRB, 0.35288E+00_JPRB, 0.38196E+00_JPRB, 0.42787E+00_JPRB ,&
&  0.51443E+00_JPRB, 0.61208E+00_JPRB, 0.74290E+00_JPRB, 0.78499E+00_JPRB, 0.79084E+00_JPRB, 0.82418E+00_JPRB, 0.85383E+00_JPRB ,&
&  0.87384E+00_JPRB, 0.91591E+00_JPRB, 0.95876E+00_JPRB, 0.98406E+00_JPRB, 0.10037E+01_JPRB, 0.10157E+01_JPRB, 0.10277E+01_JPRB ,&
&  0.10378E+01_JPRB, 0.10568E+01_JPRB, 0.10669E+01_JPRB, 0.10727E+01_JPRB, 0.10784E+01_JPRB, 0.10836E+01_JPRB, 0.11009E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11546E+01_JPRB, 0.11883E+01_JPRB, 0.12417E+01_JPRB, 0.12653E+01_JPRB, 0.12941E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.13169E+01_JPRB, 0.13248E+01_JPRB, 0.13281E+01_JPRB, 0.13116E+01_JPRB, 0.13088E+01_JPRB, 0.13186E+01_JPRB, 0.13143E+01_JPRB ,&
&  0.13047E+01_JPRB, 0.12936E+01_JPRB, 0.12811E+01_JPRB, 0.12667E+01_JPRB, 0.12533E+01_JPRB, 0.12449E+01_JPRB, 0.12318E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.12018E+01_JPRB, 0.11794E+01_JPRB, 0.11526E+01_JPRB, 0.11228E+01_JPRB, 0.10820E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.98624E+00_JPRB, 0.95254E+00_JPRB, 0.91906E+00_JPRB, 0.89343E+00_JPRB, 0.86659E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.85096E+00_JPRB, 0.84075E+00_JPRB, 0.82645E+00_JPRB, 0.80868E+00_JPRB, 0.79228E+00_JPRB, 0.78034E+00_JPRB, 0.76804E+00_JPRB ,&
&  0.75802E+00_JPRB, 0.75127E+00_JPRB, 0.74652E+00_JPRB, 0.74332E+00_JPRB, 0.74009E+00_JPRB, 0.74152E+00_JPRB, 0.74347E+00_JPRB ,&
&  0.74591E+00_JPRB, 0.74661E+00_JPRB, 0.75055E+00_JPRB, 0.75162E+00_JPRB, 0.75079E+00_JPRB, 0.74606E+00_JPRB, 0.74135E+00_JPRB ,&
&  0.73090E+00_JPRB, 0.71821E+00_JPRB, 0.70288E+00_JPRB, 0.68359E+00_JPRB, 0.66610E+00_JPRB, 0.60301E+00_JPRB, 0.58258E+00_JPRB ,&
&  0.56766E+00_JPRB, 0.56043E+00_JPRB, 0.55900E+00_JPRB, 0.56251E+00_JPRB, 0.58327E+00_JPRB, 0.57834E+00_JPRB, 0.56996E+00_JPRB ,&
&  0.52706E+00_JPRB, 0.51216E+00_JPRB, 0.50585E+00_JPRB, 0.50478E+00_JPRB, 0.51587E+00_JPRB, 0.55221E+00_JPRB, 0.59993E+00_JPRB ,&
&  0.68146E+00_JPRB, 0.72157E+00_JPRB, 0.75383E+00_JPRB, 0.78882E+00_JPRB, 0.81847E+00_JPRB, 0.84298E+00_JPRB, 0.86580E+00_JPRB ,&
&  0.88646E+00_JPRB, 0.90749E+00_JPRB, 0.92469E+00_JPRB, 0.93621E+00_JPRB, 0.94186E+00_JPRB, 0.94611E+00_JPRB, 0.94908E+00_JPRB ,&
&  0.95087E+00_JPRB, 0.95029E+00_JPRB, 0.95018E+00_JPRB, 0.94933E+00_JPRB, 0.94596E+00_JPRB, 0.94744E+00_JPRB, 0.95570E+00_JPRB ,&
&  0.96946E+00_JPRB, 0.98981E+00_JPRB, 0.10088E+01_JPRB, 0.10360E+01_JPRB, 0.10620E+01_JPRB, 0.10957E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.11386E+01_JPRB, 0.11538E+01_JPRB, 0.11535E+01_JPRB, 0.11537E+01_JPRB, 0.11505E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.11298E+01_JPRB, 0.11177E+01_JPRB, 0.11117E+01_JPRB, 0.10960E+01_JPRB, 0.10872E+01_JPRB, 0.10748E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10507E+01_JPRB, 0.10377E+01_JPRB, 0.10154E+01_JPRB, 0.99563E+00_JPRB, 0.96467E+00_JPRB, 0.93707E+00_JPRB ,&
&  0.91285E+00_JPRB, 0.89574E+00_JPRB, 0.88153E+00_JPRB, 0.87038E+00_JPRB, 0.85970E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.84062E+00_JPRB, 0.83528E+00_JPRB, 0.82962E+00_JPRB, 0.82351E+00_JPRB, 0.81793E+00_JPRB, 0.81361E+00_JPRB, 0.80897E+00_JPRB ,&
&  0.80376E+00_JPRB, 0.79979E+00_JPRB, 0.79614E+00_JPRB, 0.79284E+00_JPRB, 0.78950E+00_JPRB, 0.78824E+00_JPRB, 0.78650E+00_JPRB ,&
&  0.78381E+00_JPRB, 0.77990E+00_JPRB, 0.77704E+00_JPRB, 0.77292E+00_JPRB, 0.76802E+00_JPRB, 0.76134E+00_JPRB, 0.75512E+00_JPRB ,&
&  0.74764E+00_JPRB, 0.73850E+00_JPRB, 0.73080E+00_JPRB, 0.72680E+00_JPRB, 0.72225E+00_JPRB, 0.71558E+00_JPRB, 0.71103E+00_JPRB ,&
&  0.70883E+00_JPRB, 0.70659E+00_JPRB, 0.70472E+00_JPRB, 0.70273E+00_JPRB, 0.69816E+00_JPRB, 0.69681E+00_JPRB, 0.69586E+00_JPRB ,&
&  0.69409E+00_JPRB, 0.69496E+00_JPRB, 0.69970E+00_JPRB, 0.70498E+00_JPRB, 0.71180E+00_JPRB, 0.72141E+00_JPRB, 0.73756E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.77334E+00_JPRB, 0.79000E+00_JPRB, 0.80630E+00_JPRB, 0.81991E+00_JPRB, 0.83111E+00_JPRB, 0.84130E+00_JPRB ,&
&  0.84872E+00_JPRB, 0.85448E+00_JPRB, 0.85777E+00_JPRB, 0.85862E+00_JPRB, 0.85783E+00_JPRB, 0.85796E+00_JPRB, 0.85864E+00_JPRB ,&
&  0.85954E+00_JPRB, 0.85599E+00_JPRB, 0.85514E+00_JPRB, 0.85708E+00_JPRB, 0.86064E+00_JPRB, 0.86457E+00_JPRB, 0.87126E+00_JPRB ,&
&  0.88006E+00_JPRB, 0.88991E+00_JPRB, 0.89933E+00_JPRB, 0.91176E+00_JPRB, 0.92608E+00_JPRB, 0.94179E+00_JPRB, 0.95112E+00_JPRB ,&
&  0.96316E+00_JPRB, 0.97630E+00_JPRB, 0.99035E+00_JPRB, 0.99249E+00_JPRB, 0.99599E+00_JPRB, 0.99879E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.99572E+00_JPRB, 0.99230E+00_JPRB, 0.98889E+00_JPRB, 0.98914E+00_JPRB, 0.97953E+00_JPRB, 0.97366E+00_JPRB, 0.96778E+00_JPRB ,&
&  0.96698E+00_JPRB, 0.95277E+00_JPRB, 0.94290E+00_JPRB, 0.93063E+00_JPRB, 0.92129E+00_JPRB, 0.90504E+00_JPRB, 0.89163E+00_JPRB ,&
&  0.87942E+00_JPRB, 0.87109E+00_JPRB, 0.86177E+00_JPRB, 0.85373E+00_JPRB, 0.84669E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.82960E+00_JPRB, 0.82649E+00_JPRB, 0.82369E+00_JPRB, 0.82105E+00_JPRB, 0.81858E+00_JPRB, 0.81672E+00_JPRB, 0.81485E+00_JPRB ,&
&  0.81210E+00_JPRB, 0.80959E+00_JPRB, 0.80734E+00_JPRB, 0.80508E+00_JPRB, 0.80265E+00_JPRB, 0.80026E+00_JPRB, 0.79776E+00_JPRB ,&
&  0.79515E+00_JPRB, 0.79276E+00_JPRB, 0.79101E+00_JPRB, 0.78877E+00_JPRB, 0.78616E+00_JPRB, 0.78311E+00_JPRB, 0.78013E+00_JPRB ,&
&  0.77676E+00_JPRB, 0.77333E+00_JPRB, 0.76959E+00_JPRB, 0.76606E+00_JPRB, 0.76238E+00_JPRB, 0.75763E+00_JPRB, 0.75351E+00_JPRB ,&
&  0.75090E+00_JPRB, 0.74878E+00_JPRB, 0.74702E+00_JPRB, 0.74507E+00_JPRB, 0.74397E+00_JPRB, 0.74406E+00_JPRB, 0.74428E+00_JPRB ,&
&  0.74483E+00_JPRB, 0.74600E+00_JPRB, 0.74832E+00_JPRB, 0.75090E+00_JPRB, 0.75376E+00_JPRB, 0.75760E+00_JPRB, 0.76305E+00_JPRB ,&
&  0.76803E+00_JPRB, 0.77272E+00_JPRB, 0.77795E+00_JPRB, 0.78345E+00_JPRB, 0.78915E+00_JPRB, 0.79464E+00_JPRB, 0.79984E+00_JPRB ,&
&  0.80370E+00_JPRB, 0.80717E+00_JPRB, 0.81064E+00_JPRB, 0.81365E+00_JPRB, 0.81521E+00_JPRB, 0.81673E+00_JPRB, 0.81824E+00_JPRB ,&
&  0.81954E+00_JPRB, 0.82047E+00_JPRB, 0.82186E+00_JPRB, 0.82393E+00_JPRB, 0.82605E+00_JPRB, 0.82821E+00_JPRB, 0.83091E+00_JPRB ,&
&  0.83435E+00_JPRB, 0.83653E+00_JPRB, 0.83912E+00_JPRB, 0.84234E+00_JPRB, 0.84645E+00_JPRB, 0.85112E+00_JPRB, 0.85484E+00_JPRB ,&
&  0.85868E+00_JPRB, 0.86283E+00_JPRB, 0.86674E+00_JPRB, 0.86890E+00_JPRB, 0.87140E+00_JPRB, 0.87369E+00_JPRB, 0.87579E+00_JPRB ,&
&  0.87535E+00_JPRB, 0.87568E+00_JPRB, 0.87667E+00_JPRB, 0.87800E+00_JPRB, 0.87567E+00_JPRB, 0.87375E+00_JPRB, 0.87193E+00_JPRB ,&
&  0.87188E+00_JPRB, 0.86757E+00_JPRB, 0.86446E+00_JPRB, 0.86054E+00_JPRB, 0.85802E+00_JPRB, 0.85368E+00_JPRB, 0.85017E+00_JPRB ,&
&  0.84633E+00_JPRB, 0.84324E+00_JPRB, 0.83968E+00_JPRB, 0.83601E+00_JPRB, 0.83251E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB, 0.80472E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB, 0.25767E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.28266E+00_JPRB, 0.28541E+00_JPRB, 0.28774E+00_JPRB, 0.28942E+00_JPRB, 0.28968E+00_JPRB, 0.29137E+00_JPRB, 0.29174E+00_JPRB ,&
&  0.29190E+00_JPRB, 0.29175E+00_JPRB, 0.29251E+00_JPRB, 0.29332E+00_JPRB, 0.29384E+00_JPRB, 0.29412E+00_JPRB, 0.29415E+00_JPRB ,&
&  0.29365E+00_JPRB, 0.29268E+00_JPRB, 0.28886E+00_JPRB, 0.28673E+00_JPRB, 0.28548E+00_JPRB, 0.28489E+00_JPRB, 0.28458E+00_JPRB ,&
&  0.28382E+00_JPRB, 0.28291E+00_JPRB, 0.28142E+00_JPRB, 0.28121E+00_JPRB, 0.28041E+00_JPRB, 0.27935E+00_JPRB, 0.27786E+00_JPRB ,&
&  0.27637E+00_JPRB, 0.27520E+00_JPRB, 0.27374E+00_JPRB, 0.27225E+00_JPRB, 0.27043E+00_JPRB, 0.26823E+00_JPRB, 0.26547E+00_JPRB ,&
&  0.26262E+00_JPRB, 0.25972E+00_JPRB, 0.25732E+00_JPRB, 0.25509E+00_JPRB, 0.25316E+00_JPRB, 0.25162E+00_JPRB, 0.25051E+00_JPRB ,&
&  0.24929E+00_JPRB, 0.24790E+00_JPRB, 0.24649E+00_JPRB, 0.24531E+00_JPRB, 0.24414E+00_JPRB, 0.24313E+00_JPRB, 0.24262E+00_JPRB ,&
&  0.24233E+00_JPRB, 0.24198E+00_JPRB, 0.24154E+00_JPRB, 0.24116E+00_JPRB, 0.24067E+00_JPRB, 0.24022E+00_JPRB, 0.23976E+00_JPRB ,&
&  0.23938E+00_JPRB, 0.23915E+00_JPRB, 0.23879E+00_JPRB, 0.23842E+00_JPRB, 0.23819E+00_JPRB, 0.23780E+00_JPRB, 0.23737E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.23657E+00_JPRB, 0.23625E+00_JPRB, 0.23584E+00_JPRB, 0.23547E+00_JPRB, 0.23521E+00_JPRB, 0.23522E+00_JPRB ,&
&  0.23519E+00_JPRB, 0.23509E+00_JPRB, 0.23514E+00_JPRB, 0.23588E+00_JPRB, 0.23666E+00_JPRB, 0.23760E+00_JPRB, 0.23865E+00_JPRB ,&
&  0.24003E+00_JPRB, 0.24146E+00_JPRB, 0.24307E+00_JPRB, 0.24461E+00_JPRB, 0.24666E+00_JPRB, 0.24869E+00_JPRB, 0.25071E+00_JPRB ,&
&  0.25265E+00_JPRB, 0.25465E+00_JPRB, 0.25665E+00_JPRB, 0.25869E+00_JPRB, 0.26136E+00_JPRB, 0.26363E+00_JPRB, 0.26620E+00_JPRB ,&
&  0.26865E+00_JPRB, 0.27137E+00_JPRB, 0.27415E+00_JPRB, 0.27660E+00_JPRB, 0.27909E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.32504E+00_JPRB, 0.33529E+00_JPRB, 0.34390E+00_JPRB, 0.35214E+00_JPRB, 0.35390E+00_JPRB, 0.41851E+00_JPRB, 0.39869E+00_JPRB ,&
&  0.38908E+00_JPRB, 0.36133E+00_JPRB, 0.36252E+00_JPRB, 0.35829E+00_JPRB, 0.35187E+00_JPRB, 0.34462E+00_JPRB, 0.33845E+00_JPRB ,&
&  0.33231E+00_JPRB, 0.32459E+00_JPRB, 0.31975E+00_JPRB, 0.31381E+00_JPRB, 0.30813E+00_JPRB, 0.30245E+00_JPRB, 0.29906E+00_JPRB ,&
&  0.29652E+00_JPRB, 0.29505E+00_JPRB, 0.29394E+00_JPRB, 0.29285E+00_JPRB, 0.29525E+00_JPRB, 0.29646E+00_JPRB, 0.29459E+00_JPRB ,&
&  0.29087E+00_JPRB, 0.28738E+00_JPRB, 0.28303E+00_JPRB, 0.27859E+00_JPRB, 0.27562E+00_JPRB, 0.27295E+00_JPRB, 0.27017E+00_JPRB ,&
&  0.26647E+00_JPRB, 0.26235E+00_JPRB, 0.25817E+00_JPRB, 0.25484E+00_JPRB, 0.25169E+00_JPRB, 0.24846E+00_JPRB, 0.24545E+00_JPRB ,&
&  0.24232E+00_JPRB, 0.23921E+00_JPRB, 0.23610E+00_JPRB, 0.23408E+00_JPRB, 0.23239E+00_JPRB, 0.23081E+00_JPRB, 0.22938E+00_JPRB ,&
&  0.22879E+00_JPRB, 0.22807E+00_JPRB, 0.22718E+00_JPRB, 0.22606E+00_JPRB, 0.22501E+00_JPRB, 0.22388E+00_JPRB, 0.22296E+00_JPRB ,&
&  0.22283E+00_JPRB, 0.22226E+00_JPRB, 0.22166E+00_JPRB, 0.22088E+00_JPRB, 0.22052E+00_JPRB, 0.21973E+00_JPRB, 0.21874E+00_JPRB ,&
&  0.21783E+00_JPRB, 0.21701E+00_JPRB, 0.21683E+00_JPRB, 0.21658E+00_JPRB, 0.21632E+00_JPRB, 0.21613E+00_JPRB, 0.21653E+00_JPRB ,&
&  0.21703E+00_JPRB, 0.21765E+00_JPRB, 0.21866E+00_JPRB, 0.21987E+00_JPRB, 0.22094E+00_JPRB, 0.22199E+00_JPRB, 0.22347E+00_JPRB ,&
&  0.22542E+00_JPRB, 0.22738E+00_JPRB, 0.22957E+00_JPRB, 0.23157E+00_JPRB, 0.23497E+00_JPRB, 0.23864E+00_JPRB, 0.24267E+00_JPRB ,&
&  0.24758E+00_JPRB, 0.25433E+00_JPRB, 0.26210E+00_JPRB, 0.26909E+00_JPRB, 0.27602E+00_JPRB, 0.28354E+00_JPRB, 0.28974E+00_JPRB ,&
&  0.29514E+00_JPRB, 0.30055E+00_JPRB, 0.30614E+00_JPRB, 0.31222E+00_JPRB, 0.31881E+00_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.45040E+00_JPRB, 0.45172E+00_JPRB, 0.45248E+00_JPRB, 0.44917E+00_JPRB, 0.44808E+00_JPRB, 0.44808E+00_JPRB, 0.44536E+00_JPRB ,&
&  0.43743E+00_JPRB, 0.43505E+00_JPRB, 0.43015E+00_JPRB, 0.42299E+00_JPRB, 0.41593E+00_JPRB, 0.40445E+00_JPRB, 0.38811E+00_JPRB ,&
&  0.36625E+00_JPRB, 0.34697E+00_JPRB, 0.33246E+00_JPRB, 0.31904E+00_JPRB, 0.30589E+00_JPRB, 0.29306E+00_JPRB, 0.28792E+00_JPRB ,&
&  0.28342E+00_JPRB, 0.28152E+00_JPRB, 0.28095E+00_JPRB, 0.28441E+00_JPRB, 0.29195E+00_JPRB, 0.30138E+00_JPRB, 0.31171E+00_JPRB ,&
&  0.32345E+00_JPRB, 0.33440E+00_JPRB, 0.33572E+00_JPRB, 0.32846E+00_JPRB, 0.32150E+00_JPRB, 0.31714E+00_JPRB, 0.31377E+00_JPRB ,&
&  0.30983E+00_JPRB, 0.30154E+00_JPRB, 0.29209E+00_JPRB, 0.28217E+00_JPRB, 0.27215E+00_JPRB, 0.26486E+00_JPRB, 0.25632E+00_JPRB ,&
&  0.25008E+00_JPRB, 0.24442E+00_JPRB, 0.23840E+00_JPRB, 0.23323E+00_JPRB, 0.22972E+00_JPRB, 0.22679E+00_JPRB, 0.22382E+00_JPRB ,&
&  0.22234E+00_JPRB, 0.22086E+00_JPRB, 0.21921E+00_JPRB, 0.21760E+00_JPRB, 0.21561E+00_JPRB, 0.21408E+00_JPRB, 0.21270E+00_JPRB ,&
&  0.21165E+00_JPRB, 0.21073E+00_JPRB, 0.21018E+00_JPRB, 0.20961E+00_JPRB, 0.20914E+00_JPRB, 0.20819E+00_JPRB, 0.20706E+00_JPRB ,&
&  0.20635E+00_JPRB, 0.20609E+00_JPRB, 0.20533E+00_JPRB, 0.20493E+00_JPRB, 0.20460E+00_JPRB, 0.20386E+00_JPRB, 0.20433E+00_JPRB ,&
&  0.20548E+00_JPRB, 0.20677E+00_JPRB, 0.20705E+00_JPRB, 0.20811E+00_JPRB, 0.20939E+00_JPRB, 0.21100E+00_JPRB, 0.21355E+00_JPRB ,&
&  0.21551E+00_JPRB, 0.21754E+00_JPRB, 0.22009E+00_JPRB, 0.22230E+00_JPRB, 0.22572E+00_JPRB, 0.22994E+00_JPRB, 0.23515E+00_JPRB ,&
&  0.24227E+00_JPRB, 0.25089E+00_JPRB, 0.26294E+00_JPRB, 0.27763E+00_JPRB, 0.29878E+00_JPRB, 0.30706E+00_JPRB, 0.31514E+00_JPRB ,&
&  0.32184E+00_JPRB, 0.33492E+00_JPRB, 0.41710E+00_JPRB, 0.43190E+00_JPRB, 0.44675E+00_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.45223E+00_JPRB, 0.46442E+00_JPRB, 0.48160E+00_JPRB, 0.49223E+00_JPRB, 0.50639E+00_JPRB, 0.50905E+00_JPRB, 0.50358E+00_JPRB ,&
&  0.48483E+00_JPRB, 0.45534E+00_JPRB, 0.43878E+00_JPRB, 0.41823E+00_JPRB, 0.39700E+00_JPRB, 0.37560E+00_JPRB, 0.36611E+00_JPRB ,&
&  0.35473E+00_JPRB, 0.34187E+00_JPRB, 0.32870E+00_JPRB, 0.31734E+00_JPRB, 0.30642E+00_JPRB, 0.29630E+00_JPRB, 0.29040E+00_JPRB ,&
&  0.28435E+00_JPRB, 0.28374E+00_JPRB, 0.28310E+00_JPRB, 0.28721E+00_JPRB, 0.29838E+00_JPRB, 0.31723E+00_JPRB, 0.34306E+00_JPRB ,&
&  0.38189E+00_JPRB, 0.38763E+00_JPRB, 0.37280E+00_JPRB, 0.36645E+00_JPRB, 0.35907E+00_JPRB, 0.35266E+00_JPRB, 0.33924E+00_JPRB ,&
&  0.34654E+00_JPRB, 0.35871E+00_JPRB, 0.37430E+00_JPRB, 0.36336E+00_JPRB, 0.31797E+00_JPRB, 0.29776E+00_JPRB, 0.27425E+00_JPRB ,&
&  0.26203E+00_JPRB, 0.25193E+00_JPRB, 0.24401E+00_JPRB, 0.23645E+00_JPRB, 0.22857E+00_JPRB, 0.22374E+00_JPRB, 0.21841E+00_JPRB ,&
&  0.21462E+00_JPRB, 0.21306E+00_JPRB, 0.21067E+00_JPRB, 0.21064E+00_JPRB, 0.21046E+00_JPRB, 0.20968E+00_JPRB, 0.20892E+00_JPRB ,&
&  0.20766E+00_JPRB, 0.20631E+00_JPRB, 0.20569E+00_JPRB, 0.20623E+00_JPRB, 0.20711E+00_JPRB, 0.20798E+00_JPRB, 0.20607E+00_JPRB ,&
&  0.20480E+00_JPRB, 0.20355E+00_JPRB, 0.20343E+00_JPRB, 0.20464E+00_JPRB, 0.20505E+00_JPRB, 0.20587E+00_JPRB, 0.20603E+00_JPRB ,&
&  0.20614E+00_JPRB, 0.20669E+00_JPRB, 0.20636E+00_JPRB, 0.20705E+00_JPRB, 0.20733E+00_JPRB, 0.20844E+00_JPRB, 0.20924E+00_JPRB ,&
&  0.21011E+00_JPRB, 0.21214E+00_JPRB, 0.21427E+00_JPRB, 0.21731E+00_JPRB, 0.22143E+00_JPRB, 0.22418E+00_JPRB, 0.22861E+00_JPRB ,&
&  0.23394E+00_JPRB, 0.24274E+00_JPRB, 0.25931E+00_JPRB, 0.27790E+00_JPRB, 0.30093E+00_JPRB, 0.31246E+00_JPRB, 0.32110E+00_JPRB ,&
&  0.33446E+00_JPRB, 0.36796E+00_JPRB, 0.41843E+00_JPRB, 0.43973E+00_JPRB, 0.44503E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.50963E+00_JPRB, 0.51709E+00_JPRB, 0.52817E+00_JPRB, 0.53279E+00_JPRB, 0.54093E+00_JPRB, 0.52927E+00_JPRB, 0.52230E+00_JPRB ,&
&  0.49214E+00_JPRB, 0.46991E+00_JPRB, 0.43891E+00_JPRB, 0.40001E+00_JPRB, 0.37396E+00_JPRB, 0.35383E+00_JPRB, 0.34691E+00_JPRB ,&
&  0.33927E+00_JPRB, 0.33440E+00_JPRB, 0.32964E+00_JPRB, 0.32774E+00_JPRB, 0.32659E+00_JPRB, 0.32772E+00_JPRB, 0.32754E+00_JPRB ,&
&  0.32399E+00_JPRB, 0.31837E+00_JPRB, 0.31177E+00_JPRB, 0.31471E+00_JPRB, 0.32757E+00_JPRB, 0.34247E+00_JPRB, 0.35997E+00_JPRB ,&
&  0.38015E+00_JPRB, 0.39092E+00_JPRB, 0.40107E+00_JPRB, 0.40520E+00_JPRB, 0.38979E+00_JPRB, 0.39234E+00_JPRB, 0.39513E+00_JPRB ,&
&  0.40308E+00_JPRB, 0.40995E+00_JPRB, 0.40705E+00_JPRB, 0.40279E+00_JPRB, 0.35209E+00_JPRB, 0.31904E+00_JPRB, 0.29050E+00_JPRB ,&
&  0.26913E+00_JPRB, 0.25544E+00_JPRB, 0.24409E+00_JPRB, 0.23446E+00_JPRB, 0.22473E+00_JPRB, 0.21845E+00_JPRB, 0.21374E+00_JPRB ,&
&  0.20992E+00_JPRB, 0.20823E+00_JPRB, 0.20660E+00_JPRB, 0.20607E+00_JPRB, 0.20636E+00_JPRB, 0.20571E+00_JPRB, 0.20554E+00_JPRB ,&
&  0.20524E+00_JPRB, 0.20445E+00_JPRB, 0.20439E+00_JPRB, 0.20548E+00_JPRB, 0.20738E+00_JPRB, 0.21054E+00_JPRB, 0.21165E+00_JPRB ,&
&  0.21254E+00_JPRB, 0.21312E+00_JPRB, 0.21318E+00_JPRB, 0.21506E+00_JPRB, 0.21606E+00_JPRB, 0.21624E+00_JPRB, 0.21465E+00_JPRB ,&
&  0.21300E+00_JPRB, 0.21176E+00_JPRB, 0.21160E+00_JPRB, 0.21206E+00_JPRB, 0.21344E+00_JPRB, 0.21560E+00_JPRB, 0.21608E+00_JPRB ,&
&  0.21692E+00_JPRB, 0.21653E+00_JPRB, 0.21620E+00_JPRB, 0.21800E+00_JPRB, 0.22011E+00_JPRB, 0.22243E+00_JPRB, 0.22555E+00_JPRB ,&
&  0.22732E+00_JPRB, 0.23428E+00_JPRB, 0.24698E+00_JPRB, 0.26652E+00_JPRB, 0.29662E+00_JPRB, 0.31317E+00_JPRB, 0.35260E+00_JPRB ,&
&  0.41522E+00_JPRB, 0.55837E+00_JPRB, 0.62243E+00_JPRB, 0.47936E+00_JPRB, 0.49431E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.53741E+00_JPRB, 0.54867E+00_JPRB, 0.56111E+00_JPRB, 0.55295E+00_JPRB, 0.53929E+00_JPRB, 0.53161E+00_JPRB, 0.53138E+00_JPRB ,&
&  0.53103E+00_JPRB, 0.52827E+00_JPRB, 0.51618E+00_JPRB, 0.50371E+00_JPRB, 0.48580E+00_JPRB, 0.43767E+00_JPRB, 0.41943E+00_JPRB ,&
&  0.40195E+00_JPRB, 0.40449E+00_JPRB, 0.40913E+00_JPRB, 0.41580E+00_JPRB, 0.42995E+00_JPRB, 0.44093E+00_JPRB, 0.44313E+00_JPRB ,&
&  0.43334E+00_JPRB, 0.41513E+00_JPRB, 0.39768E+00_JPRB, 0.38404E+00_JPRB, 0.38366E+00_JPRB, 0.38376E+00_JPRB, 0.37683E+00_JPRB ,&
&  0.36320E+00_JPRB, 0.35700E+00_JPRB, 0.35659E+00_JPRB, 0.37656E+00_JPRB, 0.42915E+00_JPRB, 0.44883E+00_JPRB, 0.42358E+00_JPRB ,&
&  0.42788E+00_JPRB, 0.42297E+00_JPRB, 0.41894E+00_JPRB, 0.42035E+00_JPRB, 0.41036E+00_JPRB, 0.39143E+00_JPRB, 0.35763E+00_JPRB ,&
&  0.31392E+00_JPRB, 0.29125E+00_JPRB, 0.27031E+00_JPRB, 0.25072E+00_JPRB, 0.23575E+00_JPRB, 0.22700E+00_JPRB, 0.21955E+00_JPRB ,&
&  0.21470E+00_JPRB, 0.21061E+00_JPRB, 0.20791E+00_JPRB, 0.20773E+00_JPRB, 0.20910E+00_JPRB, 0.20972E+00_JPRB, 0.21052E+00_JPRB ,&
&  0.21181E+00_JPRB, 0.21246E+00_JPRB, 0.21278E+00_JPRB, 0.21391E+00_JPRB, 0.21527E+00_JPRB, 0.21779E+00_JPRB, 0.22135E+00_JPRB ,&
&  0.22575E+00_JPRB, 0.22946E+00_JPRB, 0.23304E+00_JPRB, 0.23904E+00_JPRB, 0.23958E+00_JPRB, 0.23842E+00_JPRB, 0.23067E+00_JPRB ,&
&  0.22506E+00_JPRB, 0.22255E+00_JPRB, 0.21997E+00_JPRB, 0.21925E+00_JPRB, 0.22029E+00_JPRB, 0.22213E+00_JPRB, 0.22397E+00_JPRB ,&
&  0.22523E+00_JPRB, 0.22526E+00_JPRB, 0.22450E+00_JPRB, 0.22445E+00_JPRB, 0.22507E+00_JPRB, 0.22524E+00_JPRB, 0.22872E+00_JPRB ,&
&  0.23637E+00_JPRB, 0.24767E+00_JPRB, 0.26018E+00_JPRB, 0.27743E+00_JPRB, 0.29358E+00_JPRB, 0.31014E+00_JPRB, 0.33778E+00_JPRB ,&
&  0.37743E+00_JPRB, 0.48555E+00_JPRB, 0.55490E+00_JPRB, 0.49303E+00_JPRB, 0.51704E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.56911E+00_JPRB, 0.57269E+00_JPRB, 0.57682E+00_JPRB, 0.57914E+00_JPRB, 0.57369E+00_JPRB, 0.56639E+00_JPRB, 0.56174E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.56685E+00_JPRB, 0.54348E+00_JPRB, 0.52518E+00_JPRB, 0.51467E+00_JPRB, 0.51110E+00_JPRB, 0.50806E+00_JPRB ,&
&  0.50197E+00_JPRB, 0.49991E+00_JPRB, 0.50120E+00_JPRB, 0.51256E+00_JPRB, 0.52196E+00_JPRB, 0.53018E+00_JPRB, 0.53762E+00_JPRB ,&
&  0.53321E+00_JPRB, 0.54606E+00_JPRB, 0.56776E+00_JPRB, 0.56703E+00_JPRB, 0.56434E+00_JPRB, 0.55717E+00_JPRB, 0.54273E+00_JPRB ,&
&  0.49468E+00_JPRB, 0.43052E+00_JPRB, 0.36879E+00_JPRB, 0.38027E+00_JPRB, 0.45772E+00_JPRB, 0.46706E+00_JPRB, 0.45154E+00_JPRB ,&
&  0.46197E+00_JPRB, 0.45777E+00_JPRB, 0.43985E+00_JPRB, 0.42668E+00_JPRB, 0.41121E+00_JPRB, 0.40114E+00_JPRB, 0.39412E+00_JPRB ,&
&  0.39130E+00_JPRB, 0.39013E+00_JPRB, 0.37542E+00_JPRB, 0.32713E+00_JPRB, 0.29892E+00_JPRB, 0.28175E+00_JPRB, 0.26409E+00_JPRB ,&
&  0.25021E+00_JPRB, 0.24146E+00_JPRB, 0.23994E+00_JPRB, 0.24016E+00_JPRB, 0.24475E+00_JPRB, 0.24673E+00_JPRB, 0.24856E+00_JPRB ,&
&  0.24830E+00_JPRB, 0.24710E+00_JPRB, 0.24304E+00_JPRB, 0.23852E+00_JPRB, 0.23538E+00_JPRB, 0.23486E+00_JPRB, 0.23459E+00_JPRB ,&
&  0.23719E+00_JPRB, 0.24113E+00_JPRB, 0.25742E+00_JPRB, 0.27490E+00_JPRB, 0.27395E+00_JPRB, 0.25883E+00_JPRB, 0.24410E+00_JPRB ,&
&  0.23440E+00_JPRB, 0.22945E+00_JPRB, 0.22635E+00_JPRB, 0.22582E+00_JPRB, 0.22864E+00_JPRB, 0.23262E+00_JPRB, 0.23368E+00_JPRB ,&
&  0.23355E+00_JPRB, 0.23424E+00_JPRB, 0.23435E+00_JPRB, 0.23484E+00_JPRB, 0.23555E+00_JPRB, 0.23812E+00_JPRB, 0.24289E+00_JPRB ,&
&  0.24857E+00_JPRB, 0.25876E+00_JPRB, 0.27032E+00_JPRB, 0.28580E+00_JPRB, 0.29708E+00_JPRB, 0.30536E+00_JPRB, 0.31607E+00_JPRB ,&
&  0.33362E+00_JPRB, 0.37201E+00_JPRB, 0.44108E+00_JPRB, 0.56694E+00_JPRB, 0.56163E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.62695E+00_JPRB, 0.61475E+00_JPRB, 0.60658E+00_JPRB, 0.60086E+00_JPRB, 0.59404E+00_JPRB, 0.59279E+00_JPRB, 0.59210E+00_JPRB ,&
&  0.58631E+00_JPRB, 0.57792E+00_JPRB, 0.57880E+00_JPRB, 0.56154E+00_JPRB, 0.53695E+00_JPRB, 0.52049E+00_JPRB, 0.51827E+00_JPRB ,&
&  0.52227E+00_JPRB, 0.52950E+00_JPRB, 0.53868E+00_JPRB, 0.54755E+00_JPRB, 0.55257E+00_JPRB, 0.56434E+00_JPRB, 0.58534E+00_JPRB ,&
&  0.60980E+00_JPRB, 0.62364E+00_JPRB, 0.62792E+00_JPRB, 0.61992E+00_JPRB, 0.60750E+00_JPRB, 0.59462E+00_JPRB, 0.58367E+00_JPRB ,&
&  0.57512E+00_JPRB, 0.47310E+00_JPRB, 0.40481E+00_JPRB, 0.41123E+00_JPRB, 0.48079E+00_JPRB, 0.46467E+00_JPRB, 0.46268E+00_JPRB ,&
&  0.47521E+00_JPRB, 0.47542E+00_JPRB, 0.46222E+00_JPRB, 0.44519E+00_JPRB, 0.42925E+00_JPRB, 0.41767E+00_JPRB, 0.40919E+00_JPRB ,&
&  0.40527E+00_JPRB, 0.41195E+00_JPRB, 0.42014E+00_JPRB, 0.43281E+00_JPRB, 0.43609E+00_JPRB, 0.42348E+00_JPRB, 0.40830E+00_JPRB ,&
&  0.39364E+00_JPRB, 0.38839E+00_JPRB, 0.39071E+00_JPRB, 0.39706E+00_JPRB, 0.40542E+00_JPRB, 0.41193E+00_JPRB, 0.40977E+00_JPRB ,&
&  0.38092E+00_JPRB, 0.35795E+00_JPRB, 0.32982E+00_JPRB, 0.30336E+00_JPRB, 0.28037E+00_JPRB, 0.26494E+00_JPRB, 0.25699E+00_JPRB ,&
&  0.25498E+00_JPRB, 0.26214E+00_JPRB, 0.28341E+00_JPRB, 0.33222E+00_JPRB, 0.39948E+00_JPRB, 0.32068E+00_JPRB, 0.28167E+00_JPRB ,&
&  0.26017E+00_JPRB, 0.24720E+00_JPRB, 0.24235E+00_JPRB, 0.24312E+00_JPRB, 0.24720E+00_JPRB, 0.25319E+00_JPRB, 0.25789E+00_JPRB ,&
&  0.26164E+00_JPRB, 0.26505E+00_JPRB, 0.26700E+00_JPRB, 0.26795E+00_JPRB, 0.26954E+00_JPRB, 0.27071E+00_JPRB, 0.26968E+00_JPRB ,&
&  0.26819E+00_JPRB, 0.27430E+00_JPRB, 0.29170E+00_JPRB, 0.31918E+00_JPRB, 0.35158E+00_JPRB, 0.38670E+00_JPRB, 0.43879E+00_JPRB ,&
&  0.50445E+00_JPRB, 0.57975E+00_JPRB, 0.59775E+00_JPRB, 0.61038E+00_JPRB, 0.62070E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.66727E+00_JPRB, 0.64105E+00_JPRB, 0.62361E+00_JPRB, 0.61008E+00_JPRB, 0.59174E+00_JPRB, 0.57780E+00_JPRB, 0.57216E+00_JPRB ,&
&  0.58030E+00_JPRB, 0.57892E+00_JPRB, 0.56914E+00_JPRB, 0.56093E+00_JPRB, 0.54500E+00_JPRB, 0.53076E+00_JPRB, 0.52494E+00_JPRB ,&
&  0.53351E+00_JPRB, 0.54980E+00_JPRB, 0.56976E+00_JPRB, 0.57804E+00_JPRB, 0.57747E+00_JPRB, 0.58598E+00_JPRB, 0.60528E+00_JPRB ,&
&  0.63391E+00_JPRB, 0.65392E+00_JPRB, 0.66100E+00_JPRB, 0.64784E+00_JPRB, 0.63023E+00_JPRB, 0.62200E+00_JPRB, 0.60757E+00_JPRB ,&
&  0.59082E+00_JPRB, 0.59420E+00_JPRB, 0.52661E+00_JPRB, 0.51907E+00_JPRB, 0.49578E+00_JPRB, 0.48136E+00_JPRB, 0.47767E+00_JPRB ,&
&  0.49090E+00_JPRB, 0.49500E+00_JPRB, 0.48667E+00_JPRB, 0.47453E+00_JPRB, 0.46616E+00_JPRB, 0.45494E+00_JPRB, 0.44453E+00_JPRB ,&
&  0.43730E+00_JPRB, 0.43984E+00_JPRB, 0.44885E+00_JPRB, 0.45818E+00_JPRB, 0.46926E+00_JPRB, 0.47987E+00_JPRB, 0.47640E+00_JPRB ,&
&  0.46471E+00_JPRB, 0.46262E+00_JPRB, 0.46951E+00_JPRB, 0.48396E+00_JPRB, 0.49971E+00_JPRB, 0.50701E+00_JPRB, 0.51067E+00_JPRB ,&
&  0.50279E+00_JPRB, 0.48589E+00_JPRB, 0.47143E+00_JPRB, 0.45952E+00_JPRB, 0.43376E+00_JPRB, 0.37053E+00_JPRB, 0.34781E+00_JPRB ,&
&  0.35867E+00_JPRB, 0.41731E+00_JPRB, 0.48724E+00_JPRB, 0.47865E+00_JPRB, 0.47431E+00_JPRB, 0.46017E+00_JPRB, 0.42405E+00_JPRB ,&
&  0.38253E+00_JPRB, 0.32118E+00_JPRB, 0.29981E+00_JPRB, 0.29626E+00_JPRB, 0.29542E+00_JPRB, 0.30117E+00_JPRB, 0.31344E+00_JPRB ,&
&  0.32957E+00_JPRB, 0.35645E+00_JPRB, 0.37380E+00_JPRB, 0.38046E+00_JPRB, 0.37899E+00_JPRB, 0.36296E+00_JPRB, 0.34915E+00_JPRB ,&
&  0.33996E+00_JPRB, 0.36484E+00_JPRB, 0.42348E+00_JPRB, 0.50625E+00_JPRB, 0.50548E+00_JPRB, 0.53028E+00_JPRB, 0.55523E+00_JPRB ,&
&  0.57908E+00_JPRB, 0.61895E+00_JPRB, 0.64514E+00_JPRB, 0.65092E+00_JPRB, 0.66921E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.69164E+00_JPRB, 0.66340E+00_JPRB, 0.63373E+00_JPRB, 0.60860E+00_JPRB, 0.59643E+00_JPRB, 0.57879E+00_JPRB, 0.56404E+00_JPRB ,&
&  0.56318E+00_JPRB, 0.57103E+00_JPRB, 0.56639E+00_JPRB, 0.56045E+00_JPRB, 0.55110E+00_JPRB, 0.54341E+00_JPRB, 0.53424E+00_JPRB ,&
&  0.53030E+00_JPRB, 0.54192E+00_JPRB, 0.56767E+00_JPRB, 0.58062E+00_JPRB, 0.58695E+00_JPRB, 0.61756E+00_JPRB, 0.64896E+00_JPRB ,&
&  0.65822E+00_JPRB, 0.67223E+00_JPRB, 0.67643E+00_JPRB, 0.65125E+00_JPRB, 0.63473E+00_JPRB, 0.62826E+00_JPRB, 0.63203E+00_JPRB ,&
&  0.62726E+00_JPRB, 0.63143E+00_JPRB, 0.62325E+00_JPRB, 0.55336E+00_JPRB, 0.55257E+00_JPRB, 0.52197E+00_JPRB, 0.51150E+00_JPRB ,&
&  0.50635E+00_JPRB, 0.51176E+00_JPRB, 0.50796E+00_JPRB, 0.49878E+00_JPRB, 0.49506E+00_JPRB, 0.49822E+00_JPRB, 0.49955E+00_JPRB ,&
&  0.49510E+00_JPRB, 0.48449E+00_JPRB, 0.48896E+00_JPRB, 0.49539E+00_JPRB, 0.49916E+00_JPRB, 0.50632E+00_JPRB, 0.51691E+00_JPRB ,&
&  0.50677E+00_JPRB, 0.49761E+00_JPRB, 0.48893E+00_JPRB, 0.48912E+00_JPRB, 0.49549E+00_JPRB, 0.50962E+00_JPRB, 0.51434E+00_JPRB ,&
&  0.51211E+00_JPRB, 0.51076E+00_JPRB, 0.51173E+00_JPRB, 0.51810E+00_JPRB, 0.53305E+00_JPRB, 0.54790E+00_JPRB, 0.54915E+00_JPRB ,&
&  0.55020E+00_JPRB, 0.55465E+00_JPRB, 0.55948E+00_JPRB, 0.56135E+00_JPRB, 0.55738E+00_JPRB, 0.54223E+00_JPRB, 0.52475E+00_JPRB ,&
&  0.50820E+00_JPRB, 0.49140E+00_JPRB, 0.47728E+00_JPRB, 0.47284E+00_JPRB, 0.48607E+00_JPRB, 0.51542E+00_JPRB, 0.53186E+00_JPRB ,&
&  0.53580E+00_JPRB, 0.53789E+00_JPRB, 0.54134E+00_JPRB, 0.54249E+00_JPRB, 0.54461E+00_JPRB, 0.54275E+00_JPRB, 0.53628E+00_JPRB ,&
&  0.52505E+00_JPRB, 0.51393E+00_JPRB, 0.51125E+00_JPRB, 0.51564E+00_JPRB, 0.52092E+00_JPRB, 0.53308E+00_JPRB, 0.54973E+00_JPRB ,&
&  0.57186E+00_JPRB, 0.61032E+00_JPRB, 0.64289E+00_JPRB, 0.66812E+00_JPRB, 0.68615E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.71093E+00_JPRB, 0.70851E+00_JPRB, 0.67866E+00_JPRB, 0.64637E+00_JPRB, 0.62719E+00_JPRB, 0.60616E+00_JPRB, 0.58939E+00_JPRB ,&
&  0.58800E+00_JPRB, 0.58540E+00_JPRB, 0.58336E+00_JPRB, 0.58085E+00_JPRB, 0.57911E+00_JPRB, 0.57596E+00_JPRB, 0.56379E+00_JPRB ,&
&  0.55928E+00_JPRB, 0.56583E+00_JPRB, 0.58826E+00_JPRB, 0.61354E+00_JPRB, 0.63388E+00_JPRB, 0.66030E+00_JPRB, 0.67153E+00_JPRB ,&
&  0.68336E+00_JPRB, 0.69588E+00_JPRB, 0.69811E+00_JPRB, 0.66527E+00_JPRB, 0.63824E+00_JPRB, 0.63927E+00_JPRB, 0.65034E+00_JPRB ,&
&  0.65066E+00_JPRB, 0.65379E+00_JPRB, 0.65709E+00_JPRB, 0.65365E+00_JPRB, 0.63302E+00_JPRB, 0.59484E+00_JPRB, 0.56473E+00_JPRB ,&
&  0.53996E+00_JPRB, 0.52082E+00_JPRB, 0.51494E+00_JPRB, 0.50348E+00_JPRB, 0.50170E+00_JPRB, 0.51170E+00_JPRB, 0.52486E+00_JPRB ,&
&  0.53850E+00_JPRB, 0.54470E+00_JPRB, 0.55171E+00_JPRB, 0.54824E+00_JPRB, 0.54391E+00_JPRB, 0.54664E+00_JPRB, 0.54650E+00_JPRB ,&
&  0.54587E+00_JPRB, 0.53675E+00_JPRB, 0.53393E+00_JPRB, 0.52685E+00_JPRB, 0.52276E+00_JPRB, 0.52296E+00_JPRB, 0.52567E+00_JPRB ,&
&  0.52665E+00_JPRB, 0.53195E+00_JPRB, 0.53352E+00_JPRB, 0.53572E+00_JPRB, 0.54526E+00_JPRB, 0.56199E+00_JPRB, 0.57876E+00_JPRB ,&
&  0.59130E+00_JPRB, 0.60110E+00_JPRB, 0.60882E+00_JPRB, 0.61460E+00_JPRB, 0.61438E+00_JPRB, 0.60967E+00_JPRB, 0.59578E+00_JPRB ,&
&  0.56518E+00_JPRB, 0.53239E+00_JPRB, 0.50753E+00_JPRB, 0.48768E+00_JPRB, 0.48398E+00_JPRB, 0.49495E+00_JPRB, 0.51281E+00_JPRB ,&
&  0.52629E+00_JPRB, 0.53034E+00_JPRB, 0.52439E+00_JPRB, 0.52266E+00_JPRB, 0.52767E+00_JPRB, 0.53727E+00_JPRB, 0.54473E+00_JPRB ,&
&  0.54400E+00_JPRB, 0.54097E+00_JPRB, 0.54493E+00_JPRB, 0.55206E+00_JPRB, 0.56629E+00_JPRB, 0.58430E+00_JPRB, 0.60367E+00_JPRB ,&
&  0.62307E+00_JPRB, 0.63820E+00_JPRB, 0.65786E+00_JPRB, 0.67780E+00_JPRB, 0.70317E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.71785E+00_JPRB, 0.70942E+00_JPRB, 0.69396E+00_JPRB, 0.66638E+00_JPRB, 0.64848E+00_JPRB, 0.62127E+00_JPRB, 0.60603E+00_JPRB ,&
&  0.60837E+00_JPRB, 0.60981E+00_JPRB, 0.61507E+00_JPRB, 0.62443E+00_JPRB, 0.62249E+00_JPRB, 0.61791E+00_JPRB, 0.61330E+00_JPRB ,&
&  0.61633E+00_JPRB, 0.61679E+00_JPRB, 0.62534E+00_JPRB, 0.64092E+00_JPRB, 0.65727E+00_JPRB, 0.66656E+00_JPRB, 0.67638E+00_JPRB ,&
&  0.68394E+00_JPRB, 0.67779E+00_JPRB, 0.67340E+00_JPRB, 0.64289E+00_JPRB, 0.62514E+00_JPRB, 0.63021E+00_JPRB, 0.63758E+00_JPRB ,&
&  0.64764E+00_JPRB, 0.65817E+00_JPRB, 0.66796E+00_JPRB, 0.67394E+00_JPRB, 0.66018E+00_JPRB, 0.65637E+00_JPRB, 0.64563E+00_JPRB ,&
&  0.60313E+00_JPRB, 0.56160E+00_JPRB, 0.53913E+00_JPRB, 0.53163E+00_JPRB, 0.52653E+00_JPRB, 0.53244E+00_JPRB, 0.53978E+00_JPRB ,&
&  0.55277E+00_JPRB, 0.56471E+00_JPRB, 0.56779E+00_JPRB, 0.57103E+00_JPRB, 0.57262E+00_JPRB, 0.57179E+00_JPRB, 0.57223E+00_JPRB ,&
&  0.56946E+00_JPRB, 0.56777E+00_JPRB, 0.57490E+00_JPRB, 0.57539E+00_JPRB, 0.56647E+00_JPRB, 0.56377E+00_JPRB, 0.56156E+00_JPRB ,&
&  0.55880E+00_JPRB, 0.55610E+00_JPRB, 0.55294E+00_JPRB, 0.56307E+00_JPRB, 0.58878E+00_JPRB, 0.61828E+00_JPRB, 0.64376E+00_JPRB ,&
&  0.65832E+00_JPRB, 0.67017E+00_JPRB, 0.67333E+00_JPRB, 0.67533E+00_JPRB, 0.67479E+00_JPRB, 0.66604E+00_JPRB, 0.66276E+00_JPRB ,&
&  0.65493E+00_JPRB, 0.63574E+00_JPRB, 0.61013E+00_JPRB, 0.57039E+00_JPRB, 0.53183E+00_JPRB, 0.50986E+00_JPRB, 0.50947E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.53416E+00_JPRB, 0.54062E+00_JPRB, 0.54878E+00_JPRB, 0.56365E+00_JPRB, 0.58152E+00_JPRB, 0.60031E+00_JPRB ,&
&  0.61216E+00_JPRB, 0.61240E+00_JPRB, 0.61023E+00_JPRB, 0.61872E+00_JPRB, 0.63015E+00_JPRB, 0.64858E+00_JPRB, 0.66850E+00_JPRB ,&
&  0.68607E+00_JPRB, 0.68593E+00_JPRB, 0.68652E+00_JPRB, 0.68793E+00_JPRB, 0.70761E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.72349E+00_JPRB, 0.72436E+00_JPRB, 0.70805E+00_JPRB, 0.69570E+00_JPRB, 0.67841E+00_JPRB, 0.65666E+00_JPRB, 0.63751E+00_JPRB ,&
&  0.63894E+00_JPRB, 0.65406E+00_JPRB, 0.66872E+00_JPRB, 0.67933E+00_JPRB, 0.69487E+00_JPRB, 0.69812E+00_JPRB, 0.70472E+00_JPRB ,&
&  0.70072E+00_JPRB, 0.66383E+00_JPRB, 0.65804E+00_JPRB, 0.66572E+00_JPRB, 0.66758E+00_JPRB, 0.67352E+00_JPRB, 0.68431E+00_JPRB ,&
&  0.69419E+00_JPRB, 0.68950E+00_JPRB, 0.68131E+00_JPRB, 0.65161E+00_JPRB, 0.64477E+00_JPRB, 0.64958E+00_JPRB, 0.65451E+00_JPRB ,&
&  0.66477E+00_JPRB, 0.68158E+00_JPRB, 0.68501E+00_JPRB, 0.69366E+00_JPRB, 0.70231E+00_JPRB, 0.70622E+00_JPRB, 0.70314E+00_JPRB ,&
&  0.67790E+00_JPRB, 0.65243E+00_JPRB, 0.64196E+00_JPRB, 0.62467E+00_JPRB, 0.60256E+00_JPRB, 0.58700E+00_JPRB, 0.57917E+00_JPRB ,&
&  0.57491E+00_JPRB, 0.57992E+00_JPRB, 0.58618E+00_JPRB, 0.60020E+00_JPRB, 0.60542E+00_JPRB, 0.60957E+00_JPRB, 0.61486E+00_JPRB ,&
&  0.61435E+00_JPRB, 0.61011E+00_JPRB, 0.60998E+00_JPRB, 0.61965E+00_JPRB, 0.62545E+00_JPRB, 0.62305E+00_JPRB, 0.61852E+00_JPRB ,&
&  0.60540E+00_JPRB, 0.59122E+00_JPRB, 0.59836E+00_JPRB, 0.61337E+00_JPRB, 0.62049E+00_JPRB, 0.64608E+00_JPRB, 0.67905E+00_JPRB ,&
&  0.70702E+00_JPRB, 0.72530E+00_JPRB, 0.72514E+00_JPRB, 0.72172E+00_JPRB, 0.72032E+00_JPRB, 0.72772E+00_JPRB, 0.73991E+00_JPRB ,&
&  0.72405E+00_JPRB, 0.71474E+00_JPRB, 0.71258E+00_JPRB, 0.69329E+00_JPRB, 0.65698E+00_JPRB, 0.62591E+00_JPRB, 0.60294E+00_JPRB ,&
&  0.58455E+00_JPRB, 0.57414E+00_JPRB, 0.57791E+00_JPRB, 0.58146E+00_JPRB, 0.59434E+00_JPRB, 0.63016E+00_JPRB, 0.66757E+00_JPRB ,&
&  0.70818E+00_JPRB, 0.72949E+00_JPRB, 0.73661E+00_JPRB, 0.74529E+00_JPRB, 0.75210E+00_JPRB, 0.74245E+00_JPRB, 0.73426E+00_JPRB ,&
&  0.73229E+00_JPRB, 0.72397E+00_JPRB, 0.71458E+00_JPRB, 0.70448E+00_JPRB, 0.71120E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.72191E+00_JPRB, 0.74311E+00_JPRB, 0.70930E+00_JPRB, 0.68940E+00_JPRB, 0.66765E+00_JPRB, 0.66210E+00_JPRB, 0.65245E+00_JPRB ,&
&  0.66285E+00_JPRB, 0.67967E+00_JPRB, 0.70344E+00_JPRB, 0.71147E+00_JPRB, 0.71844E+00_JPRB, 0.72132E+00_JPRB, 0.73838E+00_JPRB ,&
&  0.73383E+00_JPRB, 0.72566E+00_JPRB, 0.72889E+00_JPRB, 0.71864E+00_JPRB, 0.70996E+00_JPRB, 0.71483E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.73366E+00_JPRB, 0.72496E+00_JPRB, 0.70759E+00_JPRB, 0.68085E+00_JPRB, 0.66814E+00_JPRB, 0.66540E+00_JPRB, 0.67296E+00_JPRB ,&
&  0.67849E+00_JPRB, 0.70322E+00_JPRB, 0.64009E+00_JPRB, 0.69701E+00_JPRB, 0.70496E+00_JPRB, 0.70294E+00_JPRB, 0.70218E+00_JPRB ,&
&  0.70775E+00_JPRB, 0.69794E+00_JPRB, 0.68962E+00_JPRB, 0.68091E+00_JPRB, 0.65774E+00_JPRB, 0.63896E+00_JPRB, 0.61962E+00_JPRB ,&
&  0.60262E+00_JPRB, 0.59215E+00_JPRB, 0.59736E+00_JPRB, 0.61495E+00_JPRB, 0.63290E+00_JPRB, 0.64283E+00_JPRB, 0.64890E+00_JPRB ,&
&  0.65216E+00_JPRB, 0.64476E+00_JPRB, 0.63456E+00_JPRB, 0.63436E+00_JPRB, 0.64053E+00_JPRB, 0.64525E+00_JPRB, 0.64827E+00_JPRB ,&
&  0.64859E+00_JPRB, 0.62537E+00_JPRB, 0.61218E+00_JPRB, 0.62826E+00_JPRB, 0.64582E+00_JPRB, 0.66516E+00_JPRB, 0.69049E+00_JPRB ,&
&  0.70978E+00_JPRB, 0.71740E+00_JPRB, 0.71689E+00_JPRB, 0.72377E+00_JPRB, 0.73078E+00_JPRB, 0.73956E+00_JPRB, 0.74645E+00_JPRB ,&
&  0.73610E+00_JPRB, 0.73360E+00_JPRB, 0.73425E+00_JPRB, 0.73402E+00_JPRB, 0.72493E+00_JPRB, 0.70200E+00_JPRB, 0.66989E+00_JPRB ,&
&  0.65510E+00_JPRB, 0.64006E+00_JPRB, 0.62895E+00_JPRB, 0.62597E+00_JPRB, 0.62816E+00_JPRB, 0.65446E+00_JPRB, 0.69187E+00_JPRB ,&
&  0.72915E+00_JPRB, 0.76000E+00_JPRB, 0.78685E+00_JPRB, 0.81131E+00_JPRB, 0.81046E+00_JPRB, 0.79067E+00_JPRB, 0.78329E+00_JPRB ,&
&  0.77006E+00_JPRB, 0.75463E+00_JPRB, 0.74321E+00_JPRB, 0.73352E+00_JPRB, 0.72228E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.76780E+00_JPRB, 0.76738E+00_JPRB, 0.75570E+00_JPRB, 0.71487E+00_JPRB, 0.68364E+00_JPRB, 0.69193E+00_JPRB, 0.70034E+00_JPRB ,&
&  0.69954E+00_JPRB, 0.71428E+00_JPRB, 0.73930E+00_JPRB, 0.76463E+00_JPRB, 0.78159E+00_JPRB, 0.78150E+00_JPRB, 0.78555E+00_JPRB ,&
&  0.78297E+00_JPRB, 0.78082E+00_JPRB, 0.79077E+00_JPRB, 0.79571E+00_JPRB, 0.78509E+00_JPRB, 0.76413E+00_JPRB, 0.76168E+00_JPRB ,&
&  0.76768E+00_JPRB, 0.76345E+00_JPRB, 0.74515E+00_JPRB, 0.73264E+00_JPRB, 0.72245E+00_JPRB, 0.72758E+00_JPRB, 0.73432E+00_JPRB ,&
&  0.72884E+00_JPRB, 0.67909E+00_JPRB, 0.71742E+00_JPRB, 0.72919E+00_JPRB, 0.73648E+00_JPRB, 0.72477E+00_JPRB, 0.73026E+00_JPRB ,&
&  0.74202E+00_JPRB, 0.74930E+00_JPRB, 0.72981E+00_JPRB, 0.70710E+00_JPRB, 0.68615E+00_JPRB, 0.66833E+00_JPRB, 0.64021E+00_JPRB ,&
&  0.62973E+00_JPRB, 0.62670E+00_JPRB, 0.63991E+00_JPRB, 0.65402E+00_JPRB, 0.67037E+00_JPRB, 0.67694E+00_JPRB, 0.68725E+00_JPRB ,&
&  0.68930E+00_JPRB, 0.68360E+00_JPRB, 0.67834E+00_JPRB, 0.67417E+00_JPRB, 0.67322E+00_JPRB, 0.66730E+00_JPRB, 0.65865E+00_JPRB ,&
&  0.65184E+00_JPRB, 0.65000E+00_JPRB, 0.65434E+00_JPRB, 0.66362E+00_JPRB, 0.67534E+00_JPRB, 0.69375E+00_JPRB, 0.70834E+00_JPRB ,&
&  0.72619E+00_JPRB, 0.73227E+00_JPRB, 0.73759E+00_JPRB, 0.75079E+00_JPRB, 0.75943E+00_JPRB, 0.75817E+00_JPRB, 0.75526E+00_JPRB ,&
&  0.75203E+00_JPRB, 0.74572E+00_JPRB, 0.74957E+00_JPRB, 0.74791E+00_JPRB, 0.74780E+00_JPRB, 0.73900E+00_JPRB, 0.72374E+00_JPRB ,&
&  0.71792E+00_JPRB, 0.71358E+00_JPRB, 0.70510E+00_JPRB, 0.70237E+00_JPRB, 0.70029E+00_JPRB, 0.71817E+00_JPRB, 0.74959E+00_JPRB ,&
&  0.76719E+00_JPRB, 0.79128E+00_JPRB, 0.82502E+00_JPRB, 0.84796E+00_JPRB, 0.85005E+00_JPRB, 0.85318E+00_JPRB, 0.84510E+00_JPRB ,&
&  0.82804E+00_JPRB, 0.81540E+00_JPRB, 0.80135E+00_JPRB, 0.79532E+00_JPRB, 0.78975E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.81311E+00_JPRB, 0.80490E+00_JPRB, 0.78838E+00_JPRB, 0.75894E+00_JPRB, 0.72230E+00_JPRB, 0.72894E+00_JPRB, 0.73995E+00_JPRB ,&
&  0.73351E+00_JPRB, 0.72354E+00_JPRB, 0.73374E+00_JPRB, 0.75850E+00_JPRB, 0.78291E+00_JPRB, 0.79687E+00_JPRB, 0.79912E+00_JPRB ,&
&  0.79405E+00_JPRB, 0.79604E+00_JPRB, 0.80437E+00_JPRB, 0.82142E+00_JPRB, 0.82051E+00_JPRB, 0.80257E+00_JPRB, 0.79158E+00_JPRB ,&
&  0.78846E+00_JPRB, 0.78656E+00_JPRB, 0.78372E+00_JPRB, 0.76677E+00_JPRB, 0.75547E+00_JPRB, 0.75244E+00_JPRB, 0.75299E+00_JPRB ,&
&  0.74907E+00_JPRB, 0.61162E+00_JPRB, 0.71790E+00_JPRB, 0.73016E+00_JPRB, 0.74233E+00_JPRB, 0.73600E+00_JPRB, 0.73632E+00_JPRB ,&
&  0.74090E+00_JPRB, 0.75631E+00_JPRB, 0.74904E+00_JPRB, 0.72500E+00_JPRB, 0.69976E+00_JPRB, 0.67237E+00_JPRB, 0.65356E+00_JPRB ,&
&  0.64373E+00_JPRB, 0.64618E+00_JPRB, 0.65579E+00_JPRB, 0.66606E+00_JPRB, 0.67715E+00_JPRB, 0.67865E+00_JPRB, 0.68239E+00_JPRB ,&
&  0.68481E+00_JPRB, 0.68973E+00_JPRB, 0.69213E+00_JPRB, 0.69767E+00_JPRB, 0.69972E+00_JPRB, 0.69654E+00_JPRB, 0.68920E+00_JPRB ,&
&  0.67881E+00_JPRB, 0.67245E+00_JPRB, 0.67174E+00_JPRB, 0.67430E+00_JPRB, 0.68208E+00_JPRB, 0.69596E+00_JPRB, 0.71028E+00_JPRB ,&
&  0.72619E+00_JPRB, 0.73674E+00_JPRB, 0.74262E+00_JPRB, 0.74535E+00_JPRB, 0.75459E+00_JPRB, 0.76425E+00_JPRB, 0.76360E+00_JPRB ,&
&  0.75623E+00_JPRB, 0.74796E+00_JPRB, 0.74484E+00_JPRB, 0.74062E+00_JPRB, 0.74018E+00_JPRB, 0.74013E+00_JPRB, 0.73550E+00_JPRB ,&
&  0.72722E+00_JPRB, 0.72547E+00_JPRB, 0.72678E+00_JPRB, 0.71622E+00_JPRB, 0.71815E+00_JPRB, 0.72840E+00_JPRB, 0.74787E+00_JPRB ,&
&  0.77257E+00_JPRB, 0.80509E+00_JPRB, 0.83711E+00_JPRB, 0.86880E+00_JPRB, 0.89099E+00_JPRB, 0.90401E+00_JPRB, 0.90426E+00_JPRB ,&
&  0.88979E+00_JPRB, 0.86539E+00_JPRB, 0.84418E+00_JPRB, 0.83127E+00_JPRB, 0.82946E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.85822E+00_JPRB, 0.84506E+00_JPRB, 0.82862E+00_JPRB, 0.80815E+00_JPRB, 0.78910E+00_JPRB, 0.77727E+00_JPRB, 0.78148E+00_JPRB ,&
&  0.77949E+00_JPRB, 0.76660E+00_JPRB, 0.77597E+00_JPRB, 0.78862E+00_JPRB, 0.79893E+00_JPRB, 0.80878E+00_JPRB, 0.81252E+00_JPRB ,&
&  0.81644E+00_JPRB, 0.81350E+00_JPRB, 0.81646E+00_JPRB, 0.82818E+00_JPRB, 0.84638E+00_JPRB, 0.84783E+00_JPRB, 0.84476E+00_JPRB ,&
&  0.82215E+00_JPRB, 0.81066E+00_JPRB, 0.80013E+00_JPRB, 0.78734E+00_JPRB, 0.77590E+00_JPRB, 0.77195E+00_JPRB, 0.77459E+00_JPRB ,&
&  0.66101E+00_JPRB, 0.56767E+00_JPRB, 0.68225E+00_JPRB, 0.78425E+00_JPRB, 0.79080E+00_JPRB, 0.78860E+00_JPRB, 0.80448E+00_JPRB ,&
&  0.80923E+00_JPRB, 0.80782E+00_JPRB, 0.80505E+00_JPRB, 0.79391E+00_JPRB, 0.76292E+00_JPRB, 0.72927E+00_JPRB, 0.71116E+00_JPRB ,&
&  0.69993E+00_JPRB, 0.70408E+00_JPRB, 0.70501E+00_JPRB, 0.71132E+00_JPRB, 0.71079E+00_JPRB, 0.70325E+00_JPRB, 0.70136E+00_JPRB ,&
&  0.70195E+00_JPRB, 0.71194E+00_JPRB, 0.72128E+00_JPRB, 0.72811E+00_JPRB, 0.73055E+00_JPRB, 0.73117E+00_JPRB, 0.72816E+00_JPRB ,&
&  0.72588E+00_JPRB, 0.72374E+00_JPRB, 0.71996E+00_JPRB, 0.72159E+00_JPRB, 0.72552E+00_JPRB, 0.73555E+00_JPRB, 0.75170E+00_JPRB ,&
&  0.76053E+00_JPRB, 0.76267E+00_JPRB, 0.76449E+00_JPRB, 0.76573E+00_JPRB, 0.77615E+00_JPRB, 0.78385E+00_JPRB, 0.77966E+00_JPRB ,&
&  0.77257E+00_JPRB, 0.76755E+00_JPRB, 0.77169E+00_JPRB, 0.77456E+00_JPRB, 0.77885E+00_JPRB, 0.77988E+00_JPRB, 0.77437E+00_JPRB ,&
&  0.76525E+00_JPRB, 0.75106E+00_JPRB, 0.74053E+00_JPRB, 0.73969E+00_JPRB, 0.74924E+00_JPRB, 0.75679E+00_JPRB, 0.77489E+00_JPRB ,&
&  0.80747E+00_JPRB, 0.84777E+00_JPRB, 0.88179E+00_JPRB, 0.93467E+00_JPRB, 0.96990E+00_JPRB, 0.99075E+00_JPRB, 0.99200E+00_JPRB ,&
&  0.97701E+00_JPRB, 0.93537E+00_JPRB, 0.92064E+00_JPRB, 0.89185E+00_JPRB, 0.87348E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.86382E+00_JPRB, 0.85996E+00_JPRB, 0.85546E+00_JPRB, 0.83890E+00_JPRB, 0.81993E+00_JPRB, 0.81151E+00_JPRB, 0.81597E+00_JPRB ,&
&  0.81003E+00_JPRB, 0.79559E+00_JPRB, 0.79963E+00_JPRB, 0.80249E+00_JPRB, 0.80947E+00_JPRB, 0.81293E+00_JPRB, 0.81712E+00_JPRB ,&
&  0.82145E+00_JPRB, 0.82526E+00_JPRB, 0.82294E+00_JPRB, 0.83059E+00_JPRB, 0.82257E+00_JPRB, 0.81648E+00_JPRB, 0.81316E+00_JPRB ,&
&  0.80441E+00_JPRB, 0.79545E+00_JPRB, 0.78812E+00_JPRB, 0.77799E+00_JPRB, 0.77044E+00_JPRB, 0.76143E+00_JPRB, 0.74591E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.60723E+00_JPRB, 0.69638E+00_JPRB, 0.85574E+00_JPRB, 0.86419E+00_JPRB, 0.85570E+00_JPRB, 0.86555E+00_JPRB ,&
&  0.88193E+00_JPRB, 0.88918E+00_JPRB, 0.87958E+00_JPRB, 0.85400E+00_JPRB, 0.80506E+00_JPRB, 0.78094E+00_JPRB, 0.76658E+00_JPRB ,&
&  0.75039E+00_JPRB, 0.73828E+00_JPRB, 0.73075E+00_JPRB, 0.73165E+00_JPRB, 0.73101E+00_JPRB, 0.72740E+00_JPRB, 0.72354E+00_JPRB ,&
&  0.72674E+00_JPRB, 0.73616E+00_JPRB, 0.74175E+00_JPRB, 0.74720E+00_JPRB, 0.75259E+00_JPRB, 0.74880E+00_JPRB, 0.74771E+00_JPRB ,&
&  0.74646E+00_JPRB, 0.74657E+00_JPRB, 0.74470E+00_JPRB, 0.74567E+00_JPRB, 0.74929E+00_JPRB, 0.77105E+00_JPRB, 0.78946E+00_JPRB ,&
&  0.79315E+00_JPRB, 0.79277E+00_JPRB, 0.79659E+00_JPRB, 0.80215E+00_JPRB, 0.80712E+00_JPRB, 0.80879E+00_JPRB, 0.80204E+00_JPRB ,&
&  0.79443E+00_JPRB, 0.78351E+00_JPRB, 0.77880E+00_JPRB, 0.77531E+00_JPRB, 0.76818E+00_JPRB, 0.76431E+00_JPRB, 0.76554E+00_JPRB ,&
&  0.75905E+00_JPRB, 0.74733E+00_JPRB, 0.74929E+00_JPRB, 0.76280E+00_JPRB, 0.77473E+00_JPRB, 0.78265E+00_JPRB, 0.80041E+00_JPRB ,&
&  0.83435E+00_JPRB, 0.87350E+00_JPRB, 0.90380E+00_JPRB, 0.96787E+00_JPRB, 0.10077E+01_JPRB, 0.10331E+01_JPRB, 0.10397E+01_JPRB ,&
&  0.10251E+01_JPRB, 0.10050E+01_JPRB, 0.72557E+00_JPRB, 0.84103E+00_JPRB, 0.86163E+00_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.89866E+00_JPRB, 0.89701E+00_JPRB, 0.88215E+00_JPRB, 0.86283E+00_JPRB, 0.84918E+00_JPRB, 0.83988E+00_JPRB, 0.84076E+00_JPRB ,&
&  0.83737E+00_JPRB, 0.83065E+00_JPRB, 0.83274E+00_JPRB, 0.83469E+00_JPRB, 0.83691E+00_JPRB, 0.83431E+00_JPRB, 0.83929E+00_JPRB ,&
&  0.84027E+00_JPRB, 0.84181E+00_JPRB, 0.84478E+00_JPRB, 0.83791E+00_JPRB, 0.81735E+00_JPRB, 0.80252E+00_JPRB, 0.78779E+00_JPRB ,&
&  0.78651E+00_JPRB, 0.78635E+00_JPRB, 0.78714E+00_JPRB, 0.78105E+00_JPRB, 0.76126E+00_JPRB, 0.74391E+00_JPRB, 0.73283E+00_JPRB ,&
&  0.73229E+00_JPRB, 0.59285E+00_JPRB, 0.68269E+00_JPRB, 0.91460E+00_JPRB, 0.10281E+01_JPRB, 0.98994E+00_JPRB, 0.97239E+00_JPRB ,&
&  0.97232E+00_JPRB, 0.96671E+00_JPRB, 0.94530E+00_JPRB, 0.91088E+00_JPRB, 0.86412E+00_JPRB, 0.84755E+00_JPRB, 0.82596E+00_JPRB ,&
&  0.80643E+00_JPRB, 0.79865E+00_JPRB, 0.79137E+00_JPRB, 0.78958E+00_JPRB, 0.77821E+00_JPRB, 0.76543E+00_JPRB, 0.76554E+00_JPRB ,&
&  0.77276E+00_JPRB, 0.77167E+00_JPRB, 0.77489E+00_JPRB, 0.77867E+00_JPRB, 0.77319E+00_JPRB, 0.78785E+00_JPRB, 0.79399E+00_JPRB ,&
&  0.79843E+00_JPRB, 0.79912E+00_JPRB, 0.80018E+00_JPRB, 0.80200E+00_JPRB, 0.81369E+00_JPRB, 0.84471E+00_JPRB, 0.86569E+00_JPRB ,&
&  0.86056E+00_JPRB, 0.85031E+00_JPRB, 0.85593E+00_JPRB, 0.85843E+00_JPRB, 0.85058E+00_JPRB, 0.83352E+00_JPRB, 0.81390E+00_JPRB ,&
&  0.80432E+00_JPRB, 0.79649E+00_JPRB, 0.79035E+00_JPRB, 0.78086E+00_JPRB, 0.76925E+00_JPRB, 0.75927E+00_JPRB, 0.75266E+00_JPRB ,&
&  0.74906E+00_JPRB, 0.74796E+00_JPRB, 0.75443E+00_JPRB, 0.77033E+00_JPRB, 0.79150E+00_JPRB, 0.81650E+00_JPRB, 0.84381E+00_JPRB ,&
&  0.87761E+00_JPRB, 0.91729E+00_JPRB, 0.95483E+00_JPRB, 0.10042E+01_JPRB, 0.10996E+01_JPRB, 0.11294E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.10865E+01_JPRB, 0.10128E+01_JPRB, 0.79110E+00_JPRB, 0.87141E+00_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.95327E+00_JPRB, 0.96330E+00_JPRB, 0.93692E+00_JPRB, 0.89507E+00_JPRB, 0.87157E+00_JPRB, 0.86146E+00_JPRB, 0.85865E+00_JPRB ,&
&  0.85346E+00_JPRB, 0.84180E+00_JPRB, 0.84571E+00_JPRB, 0.85126E+00_JPRB, 0.84779E+00_JPRB, 0.83524E+00_JPRB, 0.82626E+00_JPRB ,&
&  0.82344E+00_JPRB, 0.82680E+00_JPRB, 0.83140E+00_JPRB, 0.82072E+00_JPRB, 0.79868E+00_JPRB, 0.78328E+00_JPRB, 0.77017E+00_JPRB ,&
&  0.76848E+00_JPRB, 0.76866E+00_JPRB, 0.76695E+00_JPRB, 0.76894E+00_JPRB, 0.74804E+00_JPRB, 0.72411E+00_JPRB, 0.70382E+00_JPRB ,&
&  0.68859E+00_JPRB, 0.62631E+00_JPRB, 0.82113E+00_JPRB, 0.11076E+01_JPRB, 0.12495E+01_JPRB, 0.12366E+01_JPRB, 0.11517E+01_JPRB ,&
&  0.10974E+01_JPRB, 0.10513E+01_JPRB, 0.10055E+01_JPRB, 0.95618E+00_JPRB, 0.92626E+00_JPRB, 0.90829E+00_JPRB, 0.89620E+00_JPRB ,&
&  0.88548E+00_JPRB, 0.87320E+00_JPRB, 0.86394E+00_JPRB, 0.84461E+00_JPRB, 0.82822E+00_JPRB, 0.82410E+00_JPRB, 0.82291E+00_JPRB ,&
&  0.82396E+00_JPRB, 0.81643E+00_JPRB, 0.81402E+00_JPRB, 0.81313E+00_JPRB, 0.79294E+00_JPRB, 0.78793E+00_JPRB, 0.78881E+00_JPRB ,&
&  0.80448E+00_JPRB, 0.82538E+00_JPRB, 0.83475E+00_JPRB, 0.87507E+00_JPRB, 0.89917E+00_JPRB, 0.92014E+00_JPRB, 0.92074E+00_JPRB ,&
&  0.90860E+00_JPRB, 0.90174E+00_JPRB, 0.89019E+00_JPRB, 0.88558E+00_JPRB, 0.87396E+00_JPRB, 0.84779E+00_JPRB, 0.82254E+00_JPRB ,&
&  0.81441E+00_JPRB, 0.81148E+00_JPRB, 0.79892E+00_JPRB, 0.79386E+00_JPRB, 0.78971E+00_JPRB, 0.77850E+00_JPRB, 0.76972E+00_JPRB ,&
&  0.76403E+00_JPRB, 0.76491E+00_JPRB, 0.76790E+00_JPRB, 0.78256E+00_JPRB, 0.80670E+00_JPRB, 0.83697E+00_JPRB, 0.86849E+00_JPRB ,&
&  0.90010E+00_JPRB, 0.93243E+00_JPRB, 0.97503E+00_JPRB, 0.11041E+01_JPRB, 0.11567E+01_JPRB, 0.11840E+01_JPRB, 0.12156E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.11453E+01_JPRB, 0.10857E+01_JPRB, 0.10041E+01_JPRB, 0.90808E+00_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.10841E+01_JPRB, 0.10450E+01_JPRB, 0.99941E+00_JPRB, 0.94595E+00_JPRB, 0.91902E+00_JPRB, 0.89660E+00_JPRB, 0.87565E+00_JPRB ,&
&  0.86442E+00_JPRB, 0.85289E+00_JPRB, 0.84450E+00_JPRB, 0.82813E+00_JPRB, 0.81210E+00_JPRB, 0.80214E+00_JPRB, 0.79049E+00_JPRB ,&
&  0.79033E+00_JPRB, 0.79005E+00_JPRB, 0.78185E+00_JPRB, 0.77321E+00_JPRB, 0.76887E+00_JPRB, 0.76484E+00_JPRB, 0.76797E+00_JPRB ,&
&  0.75939E+00_JPRB, 0.74937E+00_JPRB, 0.74940E+00_JPRB, 0.74239E+00_JPRB, 0.72939E+00_JPRB, 0.71725E+00_JPRB, 0.68489E+00_JPRB ,&
&  0.68999E+00_JPRB, 0.76563E+00_JPRB, 0.11591E+01_JPRB, 0.15995E+01_JPRB, 0.16012E+01_JPRB, 0.15360E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.11979E+01_JPRB, 0.10937E+01_JPRB, 0.10246E+01_JPRB, 0.10028E+01_JPRB, 0.97973E+00_JPRB, 0.95368E+00_JPRB ,&
&  0.92281E+00_JPRB, 0.89432E+00_JPRB, 0.87526E+00_JPRB, 0.87834E+00_JPRB, 0.87209E+00_JPRB, 0.87123E+00_JPRB, 0.87305E+00_JPRB ,&
&  0.86722E+00_JPRB, 0.86093E+00_JPRB, 0.84513E+00_JPRB, 0.83441E+00_JPRB, 0.82702E+00_JPRB, 0.83349E+00_JPRB, 0.87730E+00_JPRB ,&
&  0.94508E+00_JPRB, 0.96904E+00_JPRB, 0.98723E+00_JPRB, 0.10079E+01_JPRB, 0.10242E+01_JPRB, 0.10118E+01_JPRB, 0.97372E+00_JPRB ,&
&  0.94086E+00_JPRB, 0.92254E+00_JPRB, 0.88956E+00_JPRB, 0.89545E+00_JPRB, 0.88302E+00_JPRB, 0.86494E+00_JPRB, 0.85111E+00_JPRB ,&
&  0.83499E+00_JPRB, 0.83488E+00_JPRB, 0.82390E+00_JPRB, 0.82116E+00_JPRB, 0.82021E+00_JPRB, 0.81388E+00_JPRB, 0.80470E+00_JPRB ,&
&  0.80571E+00_JPRB, 0.80162E+00_JPRB, 0.80908E+00_JPRB, 0.83708E+00_JPRB, 0.87457E+00_JPRB, 0.89996E+00_JPRB, 0.92772E+00_JPRB ,&
&  0.95692E+00_JPRB, 0.10136E+01_JPRB, 0.10182E+01_JPRB, 0.11856E+01_JPRB, 0.13781E+01_JPRB, 0.14052E+01_JPRB, 0.13336E+01_JPRB ,&
&  0.12599E+01_JPRB, 0.11929E+01_JPRB, 0.11489E+01_JPRB, 0.10917E+01_JPRB, 0.10780E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.10703E+01_JPRB, 0.10120E+01_JPRB, 0.96574E+00_JPRB, 0.92389E+00_JPRB, 0.90439E+00_JPRB, 0.87873E+00_JPRB, 0.84971E+00_JPRB ,&
&  0.83168E+00_JPRB, 0.82586E+00_JPRB, 0.81465E+00_JPRB, 0.79328E+00_JPRB, 0.77465E+00_JPRB, 0.76735E+00_JPRB, 0.76576E+00_JPRB ,&
&  0.76309E+00_JPRB, 0.76141E+00_JPRB, 0.75899E+00_JPRB, 0.75107E+00_JPRB, 0.75351E+00_JPRB, 0.76211E+00_JPRB, 0.77122E+00_JPRB ,&
&  0.75405E+00_JPRB, 0.74063E+00_JPRB, 0.72980E+00_JPRB, 0.71884E+00_JPRB, 0.71496E+00_JPRB, 0.69473E+00_JPRB, 0.66972E+00_JPRB ,&
&  0.76985E+00_JPRB, 0.10822E+01_JPRB, 0.16352E+01_JPRB, 0.20876E+01_JPRB, 0.18913E+01_JPRB, 0.15891E+01_JPRB, 0.14305E+01_JPRB ,&
&  0.13288E+01_JPRB, 0.12181E+01_JPRB, 0.11336E+01_JPRB, 0.10725E+01_JPRB, 0.10386E+01_JPRB, 0.10001E+01_JPRB, 0.94065E+00_JPRB ,&
&  0.89716E+00_JPRB, 0.87061E+00_JPRB, 0.85907E+00_JPRB, 0.85834E+00_JPRB, 0.85081E+00_JPRB, 0.85901E+00_JPRB, 0.86312E+00_JPRB ,&
&  0.86403E+00_JPRB, 0.85943E+00_JPRB, 0.85170E+00_JPRB, 0.84659E+00_JPRB, 0.91461E+00_JPRB, 0.99053E+00_JPRB, 0.11667E+01_JPRB ,&
&  0.12337E+01_JPRB, 0.12641E+01_JPRB, 0.12267E+01_JPRB, 0.11960E+01_JPRB, 0.11478E+01_JPRB, 0.10515E+01_JPRB, 0.98340E+00_JPRB ,&
&  0.93981E+00_JPRB, 0.92458E+00_JPRB, 0.89597E+00_JPRB, 0.88965E+00_JPRB, 0.88487E+00_JPRB, 0.87514E+00_JPRB, 0.87176E+00_JPRB ,&
&  0.86542E+00_JPRB, 0.86067E+00_JPRB, 0.85569E+00_JPRB, 0.86212E+00_JPRB, 0.85716E+00_JPRB, 0.84091E+00_JPRB, 0.82974E+00_JPRB ,&
&  0.82734E+00_JPRB, 0.81451E+00_JPRB, 0.83017E+00_JPRB, 0.87299E+00_JPRB, 0.91593E+00_JPRB, 0.94439E+00_JPRB, 0.97820E+00_JPRB ,&
&  0.10210E+01_JPRB, 0.10979E+01_JPRB, 0.10819E+01_JPRB, 0.11916E+01_JPRB, 0.13641E+01_JPRB, 0.14917E+01_JPRB, 0.13340E+01_JPRB ,&
&  0.12405E+01_JPRB, 0.11862E+01_JPRB, 0.11386E+01_JPRB, 0.10828E+01_JPRB, 0.11154E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.10143E+01_JPRB, 0.96952E+00_JPRB, 0.93982E+00_JPRB, 0.91615E+00_JPRB, 0.88356E+00_JPRB, 0.85066E+00_JPRB, 0.83654E+00_JPRB ,&
&  0.82532E+00_JPRB, 0.79914E+00_JPRB, 0.76984E+00_JPRB, 0.73493E+00_JPRB, 0.72363E+00_JPRB, 0.72732E+00_JPRB, 0.73221E+00_JPRB ,&
&  0.72999E+00_JPRB, 0.73024E+00_JPRB, 0.72856E+00_JPRB, 0.72706E+00_JPRB, 0.72585E+00_JPRB, 0.74859E+00_JPRB, 0.75852E+00_JPRB ,&
&  0.75104E+00_JPRB, 0.75233E+00_JPRB, 0.73185E+00_JPRB, 0.70988E+00_JPRB, 0.69870E+00_JPRB, 0.68387E+00_JPRB, 0.68293E+00_JPRB ,&
&  0.93023E+00_JPRB, 0.14749E+01_JPRB, 0.23199E+01_JPRB, 0.22452E+01_JPRB, 0.18152E+01_JPRB, 0.14633E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.13064E+01_JPRB, 0.12409E+01_JPRB, 0.11493E+01_JPRB, 0.10730E+01_JPRB, 0.10095E+01_JPRB, 0.96308E+00_JPRB, 0.91574E+00_JPRB ,&
&  0.88736E+00_JPRB, 0.87398E+00_JPRB, 0.84998E+00_JPRB, 0.83861E+00_JPRB, 0.83207E+00_JPRB, 0.83317E+00_JPRB, 0.83306E+00_JPRB ,&
&  0.83754E+00_JPRB, 0.84023E+00_JPRB, 0.84187E+00_JPRB, 0.85122E+00_JPRB, 0.96344E+00_JPRB, 0.10348E+01_JPRB, 0.15729E+01_JPRB ,&
&  0.21731E+01_JPRB, 0.18511E+01_JPRB, 0.15185E+01_JPRB, 0.13391E+01_JPRB, 0.11948E+01_JPRB, 0.10537E+01_JPRB, 0.99570E+00_JPRB ,&
&  0.97100E+00_JPRB, 0.96161E+00_JPRB, 0.93637E+00_JPRB, 0.93476E+00_JPRB, 0.92574E+00_JPRB, 0.91548E+00_JPRB, 0.90527E+00_JPRB ,&
&  0.88582E+00_JPRB, 0.88456E+00_JPRB, 0.88480E+00_JPRB, 0.88397E+00_JPRB, 0.86943E+00_JPRB, 0.86385E+00_JPRB, 0.85786E+00_JPRB ,&
&  0.85333E+00_JPRB, 0.83984E+00_JPRB, 0.88407E+00_JPRB, 0.89646E+00_JPRB, 0.97144E+00_JPRB, 0.10344E+01_JPRB, 0.10459E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10516E+01_JPRB, 0.10522E+01_JPRB, 0.11278E+01_JPRB, 0.12673E+01_JPRB, 0.14073E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.11600E+01_JPRB, 0.11327E+01_JPRB, 0.10966E+01_JPRB, 0.10552E+01_JPRB, 0.10468E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.99949E+00_JPRB, 0.96112E+00_JPRB, 0.92668E+00_JPRB, 0.88864E+00_JPRB, 0.84339E+00_JPRB, 0.81468E+00_JPRB, 0.79230E+00_JPRB ,&
&  0.77726E+00_JPRB, 0.74405E+00_JPRB, 0.71878E+00_JPRB, 0.71324E+00_JPRB, 0.71645E+00_JPRB, 0.70848E+00_JPRB, 0.70533E+00_JPRB ,&
&  0.71019E+00_JPRB, 0.71222E+00_JPRB, 0.70272E+00_JPRB, 0.69398E+00_JPRB, 0.69650E+00_JPRB, 0.73252E+00_JPRB, 0.74935E+00_JPRB ,&
&  0.75594E+00_JPRB, 0.74658E+00_JPRB, 0.72466E+00_JPRB, 0.69599E+00_JPRB, 0.68793E+00_JPRB, 0.69671E+00_JPRB, 0.75232E+00_JPRB ,&
&  0.11252E+01_JPRB, 0.13824E+01_JPRB, 0.24188E+01_JPRB, 0.25038E+01_JPRB, 0.18078E+01_JPRB, 0.14737E+01_JPRB, 0.12970E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.12453E+01_JPRB, 0.11450E+01_JPRB, 0.10576E+01_JPRB, 0.98175E+00_JPRB, 0.91870E+00_JPRB, 0.87344E+00_JPRB ,&
&  0.85635E+00_JPRB, 0.84297E+00_JPRB, 0.83314E+00_JPRB, 0.83125E+00_JPRB, 0.82297E+00_JPRB, 0.81939E+00_JPRB, 0.81859E+00_JPRB ,&
&  0.82147E+00_JPRB, 0.81776E+00_JPRB, 0.81940E+00_JPRB, 0.86609E+00_JPRB, 0.10724E+01_JPRB, 0.14756E+01_JPRB, 0.20898E+01_JPRB ,&
&  0.24755E+01_JPRB, 0.24147E+01_JPRB, 0.18364E+01_JPRB, 0.14738E+01_JPRB, 0.12003E+01_JPRB, 0.10508E+01_JPRB, 0.10191E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.10014E+01_JPRB, 0.97159E+00_JPRB, 0.96363E+00_JPRB, 0.96079E+00_JPRB, 0.93789E+00_JPRB, 0.92750E+00_JPRB ,&
&  0.92134E+00_JPRB, 0.91814E+00_JPRB, 0.90714E+00_JPRB, 0.89368E+00_JPRB, 0.87747E+00_JPRB, 0.86741E+00_JPRB, 0.86525E+00_JPRB ,&
&  0.86688E+00_JPRB, 0.86616E+00_JPRB, 0.90989E+00_JPRB, 0.92884E+00_JPRB, 0.97863E+00_JPRB, 0.10014E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.10197E+01_JPRB, 0.10197E+01_JPRB, 0.10949E+01_JPRB, 0.11128E+01_JPRB, 0.12458E+01_JPRB, 0.11295E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.10738E+01_JPRB, 0.10397E+01_JPRB, 0.10102E+01_JPRB, 0.10053E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.93680E+00_JPRB, 0.88322E+00_JPRB, 0.84532E+00_JPRB, 0.80863E+00_JPRB, 0.77824E+00_JPRB, 0.75712E+00_JPRB, 0.75714E+00_JPRB ,&
&  0.74612E+00_JPRB, 0.71353E+00_JPRB, 0.68978E+00_JPRB, 0.68962E+00_JPRB, 0.68849E+00_JPRB, 0.69185E+00_JPRB, 0.69369E+00_JPRB ,&
&  0.68775E+00_JPRB, 0.67640E+00_JPRB, 0.67256E+00_JPRB, 0.67750E+00_JPRB, 0.70682E+00_JPRB, 0.74652E+00_JPRB, 0.75708E+00_JPRB ,&
&  0.74843E+00_JPRB, 0.74150E+00_JPRB, 0.72171E+00_JPRB, 0.70347E+00_JPRB, 0.70106E+00_JPRB, 0.72395E+00_JPRB, 0.80116E+00_JPRB ,&
&  0.12901E+01_JPRB, 0.32445E+01_JPRB, 0.15394E+01_JPRB, 0.24249E+01_JPRB, 0.17131E+01_JPRB, 0.13960E+01_JPRB, 0.11308E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.12644E+01_JPRB, 0.11486E+01_JPRB, 0.10343E+01_JPRB, 0.92549E+00_JPRB, 0.86026E+00_JPRB, 0.83556E+00_JPRB ,&
&  0.86782E+00_JPRB, 0.85483E+00_JPRB, 0.84179E+00_JPRB, 0.83924E+00_JPRB, 0.82370E+00_JPRB, 0.81917E+00_JPRB, 0.82792E+00_JPRB ,&
&  0.82368E+00_JPRB, 0.82302E+00_JPRB, 0.85436E+00_JPRB, 0.10173E+01_JPRB, 0.12023E+01_JPRB, 0.17015E+01_JPRB, 0.23614E+01_JPRB ,&
&  0.23324E+01_JPRB, 0.25215E+01_JPRB, 0.19346E+01_JPRB, 0.14746E+01_JPRB, 0.11815E+01_JPRB, 0.10718E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.98991E+00_JPRB, 0.95615E+00_JPRB, 0.92869E+00_JPRB, 0.92700E+00_JPRB, 0.94900E+00_JPRB, 0.93821E+00_JPRB, 0.94409E+00_JPRB ,&
&  0.95193E+00_JPRB, 0.91481E+00_JPRB, 0.90062E+00_JPRB, 0.89303E+00_JPRB, 0.90483E+00_JPRB, 0.91144E+00_JPRB, 0.90713E+00_JPRB ,&
&  0.90301E+00_JPRB, 0.90796E+00_JPRB, 0.93123E+00_JPRB, 0.94396E+00_JPRB, 0.96102E+00_JPRB, 0.97020E+00_JPRB, 0.95202E+00_JPRB ,&
&  0.98837E+00_JPRB, 0.10111E+01_JPRB, 0.10167E+01_JPRB, 0.10689E+01_JPRB, 0.10931E+01_JPRB, 0.11841E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.10397E+01_JPRB, 0.99844E+00_JPRB, 0.98449E+00_JPRB, 0.98096E+00_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.86828E+00_JPRB, 0.82942E+00_JPRB, 0.78866E+00_JPRB, 0.74122E+00_JPRB, 0.71042E+00_JPRB, 0.71094E+00_JPRB, 0.72242E+00_JPRB ,&
&  0.69489E+00_JPRB, 0.66661E+00_JPRB, 0.65516E+00_JPRB, 0.66902E+00_JPRB, 0.69053E+00_JPRB, 0.69195E+00_JPRB, 0.69088E+00_JPRB ,&
&  0.67740E+00_JPRB, 0.67176E+00_JPRB, 0.65982E+00_JPRB, 0.66809E+00_JPRB, 0.71923E+00_JPRB, 0.72008E+00_JPRB, 0.71228E+00_JPRB ,&
&  0.73159E+00_JPRB, 0.72819E+00_JPRB, 0.70821E+00_JPRB, 0.70749E+00_JPRB, 0.72112E+00_JPRB, 0.76993E+00_JPRB, 0.87211E+00_JPRB ,&
&  0.13950E+01_JPRB, 0.31254E+01_JPRB, 0.16611E+01_JPRB, 0.22560E+01_JPRB, 0.16136E+01_JPRB, 0.13350E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.13313E+01_JPRB, 0.11237E+01_JPRB, 0.98586E+00_JPRB, 0.85318E+00_JPRB, 0.79126E+00_JPRB, 0.82111E+00_JPRB ,&
&  0.85881E+00_JPRB, 0.85910E+00_JPRB, 0.86686E+00_JPRB, 0.88276E+00_JPRB, 0.87811E+00_JPRB, 0.87728E+00_JPRB, 0.89067E+00_JPRB ,&
&  0.90010E+00_JPRB, 0.93996E+00_JPRB, 0.10227E+01_JPRB, 0.11302E+01_JPRB, 0.14466E+01_JPRB, 0.18940E+01_JPRB, 0.22719E+01_JPRB ,&
&  0.22025E+01_JPRB, 0.23196E+01_JPRB, 0.18119E+01_JPRB, 0.13998E+01_JPRB, 0.11348E+01_JPRB, 0.10525E+01_JPRB, 0.97498E+00_JPRB ,&
&  0.92680E+00_JPRB, 0.90333E+00_JPRB, 0.87329E+00_JPRB, 0.87722E+00_JPRB, 0.92845E+00_JPRB, 0.95734E+00_JPRB, 0.94231E+00_JPRB ,&
&  0.95563E+00_JPRB, 0.95206E+00_JPRB, 0.95223E+00_JPRB, 0.93807E+00_JPRB, 0.94756E+00_JPRB, 0.94625E+00_JPRB, 0.93676E+00_JPRB ,&
&  0.93471E+00_JPRB, 0.95627E+00_JPRB, 0.95549E+00_JPRB, 0.96081E+00_JPRB, 0.97081E+00_JPRB, 0.98144E+00_JPRB, 0.95439E+00_JPRB ,&
&  0.98422E+00_JPRB, 0.10177E+01_JPRB, 0.10339E+01_JPRB, 0.10370E+01_JPRB, 0.10200E+01_JPRB, 0.10585E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.10440E+01_JPRB, 0.10250E+01_JPRB, 0.96611E+00_JPRB, 0.93989E+00_JPRB, 0.92402E+00_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.81749E+00_JPRB, 0.77097E+00_JPRB, 0.72238E+00_JPRB, 0.66168E+00_JPRB, 0.66023E+00_JPRB, 0.66631E+00_JPRB, 0.63811E+00_JPRB ,&
&  0.58793E+00_JPRB, 0.58071E+00_JPRB, 0.59055E+00_JPRB, 0.61358E+00_JPRB, 0.63340E+00_JPRB, 0.63453E+00_JPRB, 0.65209E+00_JPRB ,&
&  0.64434E+00_JPRB, 0.64113E+00_JPRB, 0.66325E+00_JPRB, 0.69869E+00_JPRB, 0.70514E+00_JPRB, 0.70655E+00_JPRB, 0.72208E+00_JPRB ,&
&  0.72986E+00_JPRB, 0.72068E+00_JPRB, 0.72008E+00_JPRB, 0.73726E+00_JPRB, 0.78551E+00_JPRB, 0.86821E+00_JPRB, 0.10167E+01_JPRB ,&
&  0.15331E+01_JPRB, 0.33885E+01_JPRB, 0.23915E+01_JPRB, 0.19702E+01_JPRB, 0.14770E+01_JPRB, 0.12041E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.11831E+01_JPRB, 0.11497E+01_JPRB, 0.11537E+01_JPRB, 0.94861E+00_JPRB, 0.80939E+00_JPRB, 0.75406E+00_JPRB, 0.83078E+00_JPRB ,&
&  0.90464E+00_JPRB, 0.94569E+00_JPRB, 0.98035E+00_JPRB, 0.98559E+00_JPRB, 0.98982E+00_JPRB, 0.99835E+00_JPRB, 0.10059E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.11264E+01_JPRB, 0.12634E+01_JPRB, 0.14180E+01_JPRB, 0.16735E+01_JPRB, 0.20346E+01_JPRB, 0.22950E+01_JPRB ,&
&  0.21637E+01_JPRB, 0.20006E+01_JPRB, 0.16573E+01_JPRB, 0.13408E+01_JPRB, 0.10507E+01_JPRB, 0.10274E+01_JPRB, 0.92510E+00_JPRB ,&
&  0.88728E+00_JPRB, 0.84269E+00_JPRB, 0.83907E+00_JPRB, 0.85859E+00_JPRB, 0.88706E+00_JPRB, 0.92049E+00_JPRB, 0.93535E+00_JPRB ,&
&  0.94953E+00_JPRB, 0.95716E+00_JPRB, 0.96115E+00_JPRB, 0.95634E+00_JPRB, 0.95821E+00_JPRB, 0.96170E+00_JPRB, 0.97483E+00_JPRB ,&
&  0.98875E+00_JPRB, 0.10246E+01_JPRB, 0.99947E+00_JPRB, 0.99507E+00_JPRB, 0.10088E+01_JPRB, 0.10159E+01_JPRB, 0.96616E+00_JPRB ,&
&  0.98343E+00_JPRB, 0.10541E+01_JPRB, 0.10399E+01_JPRB, 0.10137E+01_JPRB, 0.98949E+00_JPRB, 0.99574E+00_JPRB, 0.98076E+00_JPRB ,&
&  0.99766E+00_JPRB, 0.98603E+00_JPRB, 0.92235E+00_JPRB, 0.88164E+00_JPRB, 0.87330E+00_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.78079E+00_JPRB, 0.73300E+00_JPRB, 0.65874E+00_JPRB, 0.62873E+00_JPRB, 0.62157E+00_JPRB, 0.61002E+00_JPRB, 0.59321E+00_JPRB ,&
&  0.56996E+00_JPRB, 0.56198E+00_JPRB, 0.58839E+00_JPRB, 0.61894E+00_JPRB, 0.64635E+00_JPRB, 0.64807E+00_JPRB, 0.65454E+00_JPRB ,&
&  0.65978E+00_JPRB, 0.68241E+00_JPRB, 0.72559E+00_JPRB, 0.73701E+00_JPRB, 0.72667E+00_JPRB, 0.72323E+00_JPRB, 0.72843E+00_JPRB ,&
&  0.74158E+00_JPRB, 0.76174E+00_JPRB, 0.80270E+00_JPRB, 0.85151E+00_JPRB, 0.92572E+00_JPRB, 0.10511E+01_JPRB, 0.13174E+01_JPRB ,&
&  0.18730E+01_JPRB, 0.30932E+01_JPRB, 0.85232E+00_JPRB, 0.11264E+01_JPRB, 0.11122E+01_JPRB, 0.10362E+01_JPRB, 0.97175E+00_JPRB ,&
&  0.97182E+00_JPRB, 0.98475E+00_JPRB, 0.98585E+00_JPRB, 0.90807E+00_JPRB, 0.78055E+00_JPRB, 0.85123E+00_JPRB, 0.98579E+00_JPRB ,&
&  0.10973E+01_JPRB, 0.11332E+01_JPRB, 0.11366E+01_JPRB, 0.11218E+01_JPRB, 0.11266E+01_JPRB, 0.11373E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.13830E+01_JPRB, 0.15572E+01_JPRB, 0.15996E+01_JPRB, 0.20911E+01_JPRB, 0.23476E+01_JPRB, 0.22715E+01_JPRB ,&
&  0.18633E+01_JPRB, 0.17501E+01_JPRB, 0.15263E+01_JPRB, 0.12976E+01_JPRB, 0.10113E+01_JPRB, 0.85600E+00_JPRB, 0.89968E+00_JPRB ,&
&  0.87903E+00_JPRB, 0.83565E+00_JPRB, 0.81951E+00_JPRB, 0.80816E+00_JPRB, 0.81899E+00_JPRB, 0.88199E+00_JPRB, 0.93181E+00_JPRB ,&
&  0.93897E+00_JPRB, 0.96241E+00_JPRB, 0.96332E+00_JPRB, 0.95914E+00_JPRB, 0.96681E+00_JPRB, 0.97975E+00_JPRB, 0.10207E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10746E+01_JPRB, 0.10914E+01_JPRB, 0.10732E+01_JPRB, 0.10945E+01_JPRB, 0.10625E+01_JPRB, 0.10027E+01_JPRB ,&
&  0.10175E+01_JPRB, 0.10780E+01_JPRB, 0.97823E+00_JPRB, 0.92119E+00_JPRB, 0.92764E+00_JPRB, 0.91229E+00_JPRB, 0.89750E+00_JPRB ,&
&  0.93956E+00_JPRB, 0.95818E+00_JPRB, 0.93732E+00_JPRB, 0.81331E+00_JPRB, 0.81972E+00_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.75727E+00_JPRB, 0.66704E+00_JPRB, 0.62322E+00_JPRB, 0.61693E+00_JPRB, 0.60336E+00_JPRB, 0.60875E+00_JPRB, 0.61746E+00_JPRB ,&
&  0.61447E+00_JPRB, 0.63254E+00_JPRB, 0.62789E+00_JPRB, 0.64130E+00_JPRB, 0.66205E+00_JPRB, 0.66866E+00_JPRB, 0.69240E+00_JPRB ,&
&  0.71661E+00_JPRB, 0.74508E+00_JPRB, 0.76910E+00_JPRB, 0.73550E+00_JPRB, 0.73163E+00_JPRB, 0.73842E+00_JPRB, 0.76019E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.86190E+00_JPRB, 0.92976E+00_JPRB, 0.10166E+01_JPRB, 0.11434E+01_JPRB, 0.13710E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.28067E+01_JPRB, 0.22577E+01_JPRB, 0.74165E+00_JPRB, 0.91526E+00_JPRB, 0.90562E+00_JPRB, 0.90977E+00_JPRB, 0.81083E+00_JPRB ,&
&  0.86714E+00_JPRB, 0.94322E+00_JPRB, 0.88588E+00_JPRB, 0.87742E+00_JPRB, 0.92699E+00_JPRB, 0.10521E+01_JPRB, 0.11638E+01_JPRB ,&
&  0.12277E+01_JPRB, 0.12654E+01_JPRB, 0.12694E+01_JPRB, 0.12778E+01_JPRB, 0.12972E+01_JPRB, 0.13409E+01_JPRB, 0.14105E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.16801E+01_JPRB, 0.18804E+01_JPRB, 0.18400E+01_JPRB, 0.22978E+01_JPRB, 0.23698E+01_JPRB, 0.22960E+01_JPRB ,&
&  0.19601E+01_JPRB, 0.16636E+01_JPRB, 0.14020E+01_JPRB, 0.12142E+01_JPRB, 0.92554E+00_JPRB, 0.78551E+00_JPRB, 0.86389E+00_JPRB ,&
&  0.90011E+00_JPRB, 0.83915E+00_JPRB, 0.82062E+00_JPRB, 0.81335E+00_JPRB, 0.77628E+00_JPRB, 0.83675E+00_JPRB, 0.91890E+00_JPRB ,&
&  0.94611E+00_JPRB, 0.94967E+00_JPRB, 0.95980E+00_JPRB, 0.95392E+00_JPRB, 0.95302E+00_JPRB, 0.97149E+00_JPRB, 0.10134E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10714E+01_JPRB, 0.10847E+01_JPRB, 0.11220E+01_JPRB, 0.11586E+01_JPRB, 0.10833E+01_JPRB, 0.97368E+00_JPRB ,&
&  0.94568E+00_JPRB, 0.95690E+00_JPRB, 0.92411E+00_JPRB, 0.89309E+00_JPRB, 0.84961E+00_JPRB, 0.83008E+00_JPRB, 0.83242E+00_JPRB ,&
&  0.91710E+00_JPRB, 0.85820E+00_JPRB, 0.80610E+00_JPRB, 0.74357E+00_JPRB, 0.77528E+00_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.64757E+00_JPRB, 0.61272E+00_JPRB, 0.60471E+00_JPRB, 0.58890E+00_JPRB, 0.60218E+00_JPRB, 0.62759E+00_JPRB, 0.64669E+00_JPRB ,&
&  0.65082E+00_JPRB, 0.64994E+00_JPRB, 0.65128E+00_JPRB, 0.67103E+00_JPRB, 0.69841E+00_JPRB, 0.72756E+00_JPRB, 0.76439E+00_JPRB ,&
&  0.78889E+00_JPRB, 0.80280E+00_JPRB, 0.77923E+00_JPRB, 0.74666E+00_JPRB, 0.76119E+00_JPRB, 0.78926E+00_JPRB, 0.83193E+00_JPRB ,&
&  0.88989E+00_JPRB, 0.95655E+00_JPRB, 0.10501E+01_JPRB, 0.12054E+01_JPRB, 0.14396E+01_JPRB, 0.17198E+01_JPRB, 0.19994E+01_JPRB ,&
&  0.15705E+01_JPRB, 0.74533E+00_JPRB, 0.76037E+00_JPRB, 0.81383E+00_JPRB, 0.74356E+00_JPRB, 0.78315E+00_JPRB, 0.81245E+00_JPRB ,&
&  0.82713E+00_JPRB, 0.85447E+00_JPRB, 0.85672E+00_JPRB, 0.97715E+00_JPRB, 0.11135E+01_JPRB, 0.11949E+01_JPRB, 0.12708E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.13699E+01_JPRB, 0.13861E+01_JPRB, 0.14030E+01_JPRB, 0.14440E+01_JPRB, 0.15093E+01_JPRB, 0.15995E+01_JPRB ,&
&  0.17341E+01_JPRB, 0.18986E+01_JPRB, 0.21063E+01_JPRB, 0.19454E+01_JPRB, 0.22448E+01_JPRB, 0.23883E+01_JPRB, 0.25522E+01_JPRB ,&
&  0.24376E+01_JPRB, 0.17320E+01_JPRB, 0.13080E+01_JPRB, 0.12626E+01_JPRB, 0.10523E+01_JPRB, 0.82931E+00_JPRB, 0.92982E+00_JPRB ,&
&  0.94185E+00_JPRB, 0.93343E+00_JPRB, 0.91810E+00_JPRB, 0.84008E+00_JPRB, 0.77119E+00_JPRB, 0.76029E+00_JPRB, 0.82084E+00_JPRB ,&
&  0.86712E+00_JPRB, 0.89477E+00_JPRB, 0.87939E+00_JPRB, 0.87913E+00_JPRB, 0.86368E+00_JPRB, 0.88694E+00_JPRB, 0.93260E+00_JPRB ,&
&  0.97332E+00_JPRB, 0.10121E+01_JPRB, 0.10318E+01_JPRB, 0.10631E+01_JPRB, 0.11020E+01_JPRB, 0.10358E+01_JPRB, 0.94261E+00_JPRB ,&
&  0.91351E+00_JPRB, 0.89823E+00_JPRB, 0.85198E+00_JPRB, 0.82188E+00_JPRB, 0.80836E+00_JPRB, 0.84786E+00_JPRB, 0.95413E+00_JPRB ,&
&  0.94080E+00_JPRB, 0.83205E+00_JPRB, 0.72049E+00_JPRB, 0.69273E+00_JPRB, 0.70008E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.54091E+00_JPRB, 0.53176E+00_JPRB, 0.53257E+00_JPRB, 0.52990E+00_JPRB, 0.56210E+00_JPRB, 0.59805E+00_JPRB, 0.60837E+00_JPRB ,&
&  0.60356E+00_JPRB, 0.62224E+00_JPRB, 0.64038E+00_JPRB, 0.67337E+00_JPRB, 0.74007E+00_JPRB, 0.77279E+00_JPRB, 0.81124E+00_JPRB ,&
&  0.84947E+00_JPRB, 0.85086E+00_JPRB, 0.82619E+00_JPRB, 0.81855E+00_JPRB, 0.84833E+00_JPRB, 0.87539E+00_JPRB, 0.92151E+00_JPRB ,&
&  0.98534E+00_JPRB, 0.10615E+01_JPRB, 0.11551E+01_JPRB, 0.12661E+01_JPRB, 0.13928E+01_JPRB, 0.15065E+01_JPRB, 0.15338E+01_JPRB ,&
&  0.65633E+00_JPRB, 0.62717E+00_JPRB, 0.74693E+00_JPRB, 0.73730E+00_JPRB, 0.70050E+00_JPRB, 0.73551E+00_JPRB, 0.81076E+00_JPRB ,&
&  0.86790E+00_JPRB, 0.83842E+00_JPRB, 0.91942E+00_JPRB, 0.11659E+01_JPRB, 0.11619E+01_JPRB, 0.12313E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.13371E+01_JPRB, 0.13925E+01_JPRB, 0.14366E+01_JPRB, 0.14761E+01_JPRB, 0.15308E+01_JPRB, 0.16038E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.18368E+01_JPRB, 0.20448E+01_JPRB, 0.22527E+01_JPRB, 0.20857E+01_JPRB, 0.21944E+01_JPRB, 0.24256E+01_JPRB, 0.28537E+01_JPRB ,&
&  0.26632E+01_JPRB, 0.19280E+01_JPRB, 0.15104E+01_JPRB, 0.13556E+01_JPRB, 0.11617E+01_JPRB, 0.11261E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.98751E+00_JPRB, 0.93216E+00_JPRB, 0.87676E+00_JPRB, 0.81398E+00_JPRB, 0.74320E+00_JPRB, 0.72910E+00_JPRB, 0.72673E+00_JPRB ,&
&  0.76658E+00_JPRB, 0.78798E+00_JPRB, 0.78460E+00_JPRB, 0.77429E+00_JPRB, 0.74138E+00_JPRB, 0.75862E+00_JPRB, 0.80618E+00_JPRB ,&
&  0.84597E+00_JPRB, 0.86781E+00_JPRB, 0.87672E+00_JPRB, 0.88565E+00_JPRB, 0.89646E+00_JPRB, 0.89536E+00_JPRB, 0.86457E+00_JPRB ,&
&  0.81328E+00_JPRB, 0.77542E+00_JPRB, 0.77955E+00_JPRB, 0.77112E+00_JPRB, 0.93209E+00_JPRB, 0.12316E+01_JPRB, 0.12707E+01_JPRB ,&
&  0.10832E+01_JPRB, 0.83353E+00_JPRB, 0.67181E+00_JPRB, 0.62154E+00_JPRB, 0.59800E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.50208E+00_JPRB, 0.49084E+00_JPRB, 0.49035E+00_JPRB, 0.48820E+00_JPRB, 0.52254E+00_JPRB, 0.54546E+00_JPRB, 0.56697E+00_JPRB ,&
&  0.60039E+00_JPRB, 0.62217E+00_JPRB, 0.65566E+00_JPRB, 0.71675E+00_JPRB, 0.78290E+00_JPRB, 0.83709E+00_JPRB, 0.86413E+00_JPRB ,&
&  0.88102E+00_JPRB, 0.88236E+00_JPRB, 0.87554E+00_JPRB, 0.86483E+00_JPRB, 0.88389E+00_JPRB, 0.89448E+00_JPRB, 0.93098E+00_JPRB ,&
&  0.98303E+00_JPRB, 0.10386E+01_JPRB, 0.11065E+01_JPRB, 0.11696E+01_JPRB, 0.12367E+01_JPRB, 0.12917E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.60140E+00_JPRB, 0.80311E+00_JPRB, 0.83491E+00_JPRB, 0.82093E+00_JPRB, 0.77022E+00_JPRB, 0.75548E+00_JPRB, 0.82717E+00_JPRB ,&
&  0.92614E+00_JPRB, 0.95870E+00_JPRB, 0.10560E+01_JPRB, 0.97663E+00_JPRB, 0.10954E+01_JPRB, 0.11251E+01_JPRB, 0.12040E+01_JPRB ,&
&  0.12575E+01_JPRB, 0.12927E+01_JPRB, 0.13457E+01_JPRB, 0.14025E+01_JPRB, 0.14693E+01_JPRB, 0.15416E+01_JPRB, 0.16362E+01_JPRB ,&
&  0.17236E+01_JPRB, 0.18355E+01_JPRB, 0.19890E+01_JPRB, 0.19284E+01_JPRB, 0.18364E+01_JPRB, 0.20166E+01_JPRB, 0.19739E+01_JPRB ,&
&  0.17328E+01_JPRB, 0.12522E+01_JPRB, 0.12326E+01_JPRB, 0.12839E+01_JPRB, 0.12289E+01_JPRB, 0.11899E+01_JPRB, 0.11122E+01_JPRB ,&
&  0.97945E+00_JPRB, 0.90046E+00_JPRB, 0.83008E+00_JPRB, 0.78132E+00_JPRB, 0.73284E+00_JPRB, 0.70300E+00_JPRB, 0.68150E+00_JPRB ,&
&  0.67078E+00_JPRB, 0.68985E+00_JPRB, 0.70777E+00_JPRB, 0.67996E+00_JPRB, 0.64891E+00_JPRB, 0.67258E+00_JPRB, 0.74823E+00_JPRB ,&
&  0.85020E+00_JPRB, 0.75595E+00_JPRB, 0.68680E+00_JPRB, 0.68241E+00_JPRB, 0.66136E+00_JPRB, 0.63815E+00_JPRB, 0.62572E+00_JPRB ,&
&  0.63189E+00_JPRB, 0.62364E+00_JPRB, 0.72109E+00_JPRB, 0.11026E+01_JPRB, 0.14848E+01_JPRB, 0.17286E+01_JPRB, 0.14272E+01_JPRB ,&
&  0.10143E+01_JPRB, 0.75287E+00_JPRB, 0.60267E+00_JPRB, 0.56320E+00_JPRB, 0.54218E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.53736E+00_JPRB, 0.51711E+00_JPRB, 0.49653E+00_JPRB, 0.50557E+00_JPRB, 0.51248E+00_JPRB, 0.51842E+00_JPRB, 0.54526E+00_JPRB ,&
&  0.60169E+00_JPRB, 0.61639E+00_JPRB, 0.64444E+00_JPRB, 0.68943E+00_JPRB, 0.75466E+00_JPRB, 0.80637E+00_JPRB, 0.85438E+00_JPRB ,&
&  0.90367E+00_JPRB, 0.93541E+00_JPRB, 0.93566E+00_JPRB, 0.92658E+00_JPRB, 0.92938E+00_JPRB, 0.93884E+00_JPRB, 0.95685E+00_JPRB ,&
&  0.96358E+00_JPRB, 0.98936E+00_JPRB, 0.10367E+01_JPRB, 0.10914E+01_JPRB, 0.11421E+01_JPRB, 0.11450E+01_JPRB, 0.73132E+00_JPRB ,&
&  0.87737E+00_JPRB, 0.92240E+00_JPRB, 0.94376E+00_JPRB, 0.91766E+00_JPRB, 0.85366E+00_JPRB, 0.79189E+00_JPRB, 0.78936E+00_JPRB ,&
&  0.83154E+00_JPRB, 0.87834E+00_JPRB, 0.88918E+00_JPRB, 0.88422E+00_JPRB, 0.99291E+00_JPRB, 0.96706E+00_JPRB, 0.10626E+01_JPRB ,&
&  0.11083E+01_JPRB, 0.11507E+01_JPRB, 0.11964E+01_JPRB, 0.12253E+01_JPRB, 0.12654E+01_JPRB, 0.13135E+01_JPRB, 0.13574E+01_JPRB ,&
&  0.13701E+01_JPRB, 0.13531E+01_JPRB, 0.13659E+01_JPRB, 0.13826E+01_JPRB, 0.12642E+01_JPRB, 0.14425E+01_JPRB, 0.11903E+01_JPRB ,&
&  0.83236E+00_JPRB, 0.71661E+00_JPRB, 0.96584E+00_JPRB, 0.12546E+01_JPRB, 0.12175E+01_JPRB, 0.11875E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.94647E+00_JPRB, 0.84597E+00_JPRB, 0.80045E+00_JPRB, 0.76634E+00_JPRB, 0.72931E+00_JPRB, 0.69053E+00_JPRB, 0.66137E+00_JPRB ,&
&  0.62552E+00_JPRB, 0.60066E+00_JPRB, 0.62349E+00_JPRB, 0.62104E+00_JPRB, 0.63675E+00_JPRB, 0.74386E+00_JPRB, 0.99995E+00_JPRB ,&
&  0.15278E+01_JPRB, 0.82142E+00_JPRB, 0.66729E+00_JPRB, 0.60438E+00_JPRB, 0.53687E+00_JPRB, 0.49739E+00_JPRB, 0.47518E+00_JPRB ,&
&  0.47820E+00_JPRB, 0.54375E+00_JPRB, 0.68365E+00_JPRB, 0.13985E+01_JPRB, 0.25491E+01_JPRB, 0.26114E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.74032E+00_JPRB, 0.59671E+00_JPRB, 0.56478E+00_JPRB, 0.54199E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.59816E+00_JPRB, 0.56396E+00_JPRB, 0.54948E+00_JPRB, 0.54062E+00_JPRB, 0.53006E+00_JPRB, 0.54873E+00_JPRB, 0.56394E+00_JPRB ,&
&  0.58640E+00_JPRB, 0.61484E+00_JPRB, 0.64953E+00_JPRB, 0.67995E+00_JPRB, 0.72269E+00_JPRB, 0.76434E+00_JPRB, 0.79772E+00_JPRB ,&
&  0.82499E+00_JPRB, 0.84674E+00_JPRB, 0.86368E+00_JPRB, 0.87199E+00_JPRB, 0.87571E+00_JPRB, 0.87815E+00_JPRB, 0.87864E+00_JPRB ,&
&  0.87857E+00_JPRB, 0.90186E+00_JPRB, 0.94824E+00_JPRB, 0.10025E+01_JPRB, 0.10386E+01_JPRB, 0.10949E+01_JPRB, 0.70492E+00_JPRB ,&
&  0.10559E+01_JPRB, 0.10755E+01_JPRB, 0.10439E+01_JPRB, 0.96606E+00_JPRB, 0.89151E+00_JPRB, 0.83923E+00_JPRB, 0.80135E+00_JPRB ,&
&  0.78103E+00_JPRB, 0.79694E+00_JPRB, 0.77111E+00_JPRB, 0.82376E+00_JPRB, 0.84555E+00_JPRB, 0.81526E+00_JPRB, 0.87947E+00_JPRB ,&
&  0.93027E+00_JPRB, 0.10005E+01_JPRB, 0.10475E+01_JPRB, 0.10848E+01_JPRB, 0.11089E+01_JPRB, 0.11358E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.11017E+01_JPRB, 0.10350E+01_JPRB, 0.98730E+00_JPRB, 0.96999E+00_JPRB, 0.85532E+00_JPRB, 0.78777E+00_JPRB ,&
&  0.61956E+00_JPRB, 0.65148E+00_JPRB, 0.10462E+01_JPRB, 0.13612E+01_JPRB, 0.12338E+01_JPRB, 0.11767E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.93845E+00_JPRB, 0.87113E+00_JPRB, 0.83507E+00_JPRB, 0.80275E+00_JPRB, 0.75265E+00_JPRB, 0.71716E+00_JPRB, 0.70765E+00_JPRB ,&
&  0.70171E+00_JPRB, 0.64506E+00_JPRB, 0.62667E+00_JPRB, 0.66333E+00_JPRB, 0.67650E+00_JPRB, 0.76232E+00_JPRB, 0.77517E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.79035E+00_JPRB, 0.63680E+00_JPRB, 0.61154E+00_JPRB, 0.50895E+00_JPRB, 0.48339E+00_JPRB, 0.48092E+00_JPRB ,&
&  0.50449E+00_JPRB, 0.45799E+00_JPRB, 0.95570E+00_JPRB, 0.37434E+01_JPRB, 0.36980E+01_JPRB, 0.46892E+01_JPRB, 0.24504E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.72801E+00_JPRB, 0.63649E+00_JPRB, 0.62899E+00_JPRB, 0.61123E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.63209E+00_JPRB, 0.62105E+00_JPRB, 0.59534E+00_JPRB, 0.56090E+00_JPRB, 0.54749E+00_JPRB, 0.56442E+00_JPRB, 0.58958E+00_JPRB ,&
&  0.60946E+00_JPRB, 0.63380E+00_JPRB, 0.65938E+00_JPRB, 0.68753E+00_JPRB, 0.71801E+00_JPRB, 0.74657E+00_JPRB, 0.76879E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.80658E+00_JPRB, 0.82586E+00_JPRB, 0.84236E+00_JPRB, 0.84959E+00_JPRB, 0.85372E+00_JPRB, 0.85740E+00_JPRB ,&
&  0.89514E+00_JPRB, 0.93372E+00_JPRB, 0.98363E+00_JPRB, 0.10354E+01_JPRB, 0.10721E+01_JPRB, 0.10899E+01_JPRB, 0.90580E+00_JPRB ,&
&  0.12235E+01_JPRB, 0.11982E+01_JPRB, 0.10972E+01_JPRB, 0.99910E+00_JPRB, 0.91709E+00_JPRB, 0.85614E+00_JPRB, 0.80296E+00_JPRB ,&
&  0.76315E+00_JPRB, 0.78396E+00_JPRB, 0.75187E+00_JPRB, 0.75162E+00_JPRB, 0.69173E+00_JPRB, 0.72383E+00_JPRB, 0.75592E+00_JPRB ,&
&  0.83328E+00_JPRB, 0.91962E+00_JPRB, 0.99656E+00_JPRB, 0.10596E+01_JPRB, 0.10910E+01_JPRB, 0.11124E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10595E+01_JPRB, 0.97328E+00_JPRB, 0.94849E+00_JPRB, 0.95527E+00_JPRB, 0.85687E+00_JPRB, 0.78864E+00_JPRB ,&
&  0.71366E+00_JPRB, 0.87008E+00_JPRB, 0.12248E+01_JPRB, 0.14038E+01_JPRB, 0.12070E+01_JPRB, 0.11420E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.96692E+00_JPRB, 0.94054E+00_JPRB, 0.92672E+00_JPRB, 0.89753E+00_JPRB, 0.83445E+00_JPRB, 0.79324E+00_JPRB, 0.80195E+00_JPRB ,&
&  0.82056E+00_JPRB, 0.76647E+00_JPRB, 0.74564E+00_JPRB, 0.82713E+00_JPRB, 0.88258E+00_JPRB, 0.83249E+00_JPRB, 0.13403E+01_JPRB ,&
&  0.83995E+00_JPRB, 0.59120E+00_JPRB, 0.76021E+00_JPRB, 0.45004E+00_JPRB, 0.54118E+00_JPRB, 0.53456E+00_JPRB, 0.55901E+00_JPRB ,&
&  0.48262E+00_JPRB, 0.55918E+00_JPRB, 0.92195E+00_JPRB, 0.12464E+01_JPRB, 0.14314E+01_JPRB, 0.14219E+01_JPRB, 0.82007E+00_JPRB ,&
&  0.72347E+00_JPRB, 0.69742E+00_JPRB, 0.67919E+00_JPRB, 0.65756E+00_JPRB, 0.63465E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.66344E+00_JPRB, 0.66202E+00_JPRB, 0.63691E+00_JPRB, 0.60288E+00_JPRB, 0.58719E+00_JPRB, 0.59486E+00_JPRB, 0.61121E+00_JPRB ,&
&  0.62674E+00_JPRB, 0.64766E+00_JPRB, 0.67214E+00_JPRB, 0.69586E+00_JPRB, 0.71697E+00_JPRB, 0.73980E+00_JPRB, 0.76682E+00_JPRB ,&
&  0.79214E+00_JPRB, 0.82345E+00_JPRB, 0.85529E+00_JPRB, 0.88803E+00_JPRB, 0.91703E+00_JPRB, 0.96700E+00_JPRB, 0.10021E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.11102E+01_JPRB, 0.11765E+01_JPRB, 0.12410E+01_JPRB, 0.13229E+01_JPRB, 0.13864E+01_JPRB, 0.12173E+01_JPRB ,&
&  0.13659E+01_JPRB, 0.12633E+01_JPRB, 0.11267E+01_JPRB, 0.10268E+01_JPRB, 0.96295E+00_JPRB, 0.91401E+00_JPRB, 0.86769E+00_JPRB ,&
&  0.84364E+00_JPRB, 0.80867E+00_JPRB, 0.76976E+00_JPRB, 0.74063E+00_JPRB, 0.72851E+00_JPRB, 0.77130E+00_JPRB, 0.75278E+00_JPRB ,&
&  0.80195E+00_JPRB, 0.89925E+00_JPRB, 0.97877E+00_JPRB, 0.11070E+01_JPRB, 0.12211E+01_JPRB, 0.12914E+01_JPRB, 0.12500E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.13418E+01_JPRB, 0.13090E+01_JPRB, 0.13024E+01_JPRB, 0.12749E+01_JPRB, 0.12312E+01_JPRB, 0.12124E+01_JPRB ,&
&  0.10732E+01_JPRB, 0.11262E+01_JPRB, 0.12672E+01_JPRB, 0.13921E+01_JPRB, 0.12243E+01_JPRB, 0.11143E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.10194E+01_JPRB, 0.10177E+01_JPRB, 0.10249E+01_JPRB, 0.98571E+00_JPRB, 0.95569E+00_JPRB, 0.92459E+00_JPRB, 0.93578E+00_JPRB ,&
&  0.96078E+00_JPRB, 0.89448E+00_JPRB, 0.90219E+00_JPRB, 0.98038E+00_JPRB, 0.11248E+01_JPRB, 0.12377E+01_JPRB, 0.15888E+01_JPRB ,&
&  0.87360E+00_JPRB, 0.54838E+00_JPRB, 0.60172E+00_JPRB, 0.73325E+00_JPRB, 0.64281E+00_JPRB, 0.64279E+00_JPRB, 0.68758E+00_JPRB ,&
&  0.73137E+00_JPRB, 0.75570E+00_JPRB, 0.74452E+00_JPRB, 0.76498E+00_JPRB, 0.80107E+00_JPRB, 0.78973E+00_JPRB, 0.73491E+00_JPRB ,&
&  0.74193E+00_JPRB, 0.70561E+00_JPRB, 0.68728E+00_JPRB, 0.67988E+00_JPRB, 0.67348E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.68836E+00_JPRB, 0.69135E+00_JPRB, 0.67436E+00_JPRB, 0.64773E+00_JPRB, 0.62925E+00_JPRB, 0.61884E+00_JPRB, 0.62772E+00_JPRB ,&
&  0.63817E+00_JPRB, 0.66079E+00_JPRB, 0.68014E+00_JPRB, 0.70043E+00_JPRB, 0.71005E+00_JPRB, 0.71730E+00_JPRB, 0.73304E+00_JPRB ,&
&  0.75423E+00_JPRB, 0.77621E+00_JPRB, 0.80516E+00_JPRB, 0.84572E+00_JPRB, 0.88550E+00_JPRB, 0.93472E+00_JPRB, 0.95697E+00_JPRB ,&
&  0.95848E+00_JPRB, 0.98585E+00_JPRB, 0.10058E+01_JPRB, 0.10492E+01_JPRB, 0.12594E+01_JPRB, 0.14783E+01_JPRB, 0.17333E+01_JPRB ,&
&  0.12981E+01_JPRB, 0.12753E+01_JPRB, 0.11218E+01_JPRB, 0.10215E+01_JPRB, 0.10364E+01_JPRB, 0.10143E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10290E+01_JPRB, 0.10296E+01_JPRB, 0.10381E+01_JPRB, 0.10536E+01_JPRB, 0.11017E+01_JPRB, 0.10594E+01_JPRB ,&
&  0.10322E+01_JPRB, 0.10685E+01_JPRB, 0.11203E+01_JPRB, 0.12712E+01_JPRB, 0.14864E+01_JPRB, 0.16760E+01_JPRB, 0.16932E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.17983E+01_JPRB, 0.17601E+01_JPRB, 0.17291E+01_JPRB, 0.16748E+01_JPRB, 0.16314E+01_JPRB, 0.16553E+01_JPRB ,&
&  0.15627E+01_JPRB, 0.14960E+01_JPRB, 0.14829E+01_JPRB, 0.13360E+01_JPRB, 0.12471E+01_JPRB, 0.11739E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.11245E+01_JPRB, 0.11454E+01_JPRB, 0.11588E+01_JPRB, 0.11241E+01_JPRB, 0.11020E+01_JPRB, 0.10706E+01_JPRB, 0.10914E+01_JPRB ,&
&  0.11080E+01_JPRB, 0.10575E+01_JPRB, 0.10409E+01_JPRB, 0.10772E+01_JPRB, 0.11363E+01_JPRB, 0.17827E+01_JPRB, 0.17669E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.87298E+00_JPRB, 0.70172E+00_JPRB, 0.10327E+01_JPRB, 0.77194E+00_JPRB, 0.74611E+00_JPRB, 0.76329E+00_JPRB ,&
&  0.78371E+00_JPRB, 0.79836E+00_JPRB, 0.80770E+00_JPRB, 0.79961E+00_JPRB, 0.79658E+00_JPRB, 0.79435E+00_JPRB, 0.77750E+00_JPRB ,&
&  0.76446E+00_JPRB, 0.74714E+00_JPRB, 0.74268E+00_JPRB, 0.70990E+00_JPRB, 0.69298E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.70410E+00_JPRB, 0.69174E+00_JPRB, 0.69335E+00_JPRB, 0.66751E+00_JPRB, 0.68709E+00_JPRB, 0.66928E+00_JPRB, 0.70120E+00_JPRB ,&
&  0.67772E+00_JPRB, 0.67533E+00_JPRB, 0.67803E+00_JPRB, 0.69340E+00_JPRB, 0.70935E+00_JPRB, 0.73094E+00_JPRB, 0.74863E+00_JPRB ,&
&  0.77345E+00_JPRB, 0.82212E+00_JPRB, 0.89173E+00_JPRB, 0.97494E+00_JPRB, 0.10847E+01_JPRB, 0.11923E+01_JPRB, 0.12849E+01_JPRB ,&
&  0.14084E+01_JPRB, 0.15100E+01_JPRB, 0.15621E+01_JPRB, 0.17095E+01_JPRB, 0.21263E+01_JPRB, 0.26290E+01_JPRB, 0.26524E+01_JPRB ,&
&  0.16733E+01_JPRB, 0.12091E+01_JPRB, 0.10854E+01_JPRB, 0.99922E+00_JPRB, 0.10356E+01_JPRB, 0.11356E+01_JPRB, 0.11614E+01_JPRB ,&
&  0.11835E+01_JPRB, 0.12127E+01_JPRB, 0.12580E+01_JPRB, 0.13097E+01_JPRB, 0.13834E+01_JPRB, 0.14748E+01_JPRB, 0.15027E+01_JPRB ,&
&  0.14823E+01_JPRB, 0.15035E+01_JPRB, 0.15030E+01_JPRB, 0.15933E+01_JPRB, 0.17574E+01_JPRB, 0.19682E+01_JPRB, 0.21046E+01_JPRB ,&
&  0.21850E+01_JPRB, 0.21497E+01_JPRB, 0.18739E+01_JPRB, 0.18303E+01_JPRB, 0.18760E+01_JPRB, 0.18564E+01_JPRB, 0.18143E+01_JPRB ,&
&  0.19507E+01_JPRB, 0.18528E+01_JPRB, 0.13662E+01_JPRB, 0.12995E+01_JPRB, 0.12815E+01_JPRB, 0.12839E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.13020E+01_JPRB, 0.13320E+01_JPRB, 0.13229E+01_JPRB, 0.12903E+01_JPRB, 0.12446E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12023E+01_JPRB, 0.12160E+01_JPRB, 0.12146E+01_JPRB, 0.13590E+01_JPRB, 0.13177E+01_JPRB, 0.14322E+01_JPRB ,&
&  0.13095E+01_JPRB, 0.12899E+01_JPRB, 0.10942E+01_JPRB, 0.13322E+01_JPRB, 0.97042E+00_JPRB, 0.82991E+00_JPRB, 0.84219E+00_JPRB ,&
&  0.85767E+00_JPRB, 0.88349E+00_JPRB, 0.87642E+00_JPRB, 0.88004E+00_JPRB, 0.88487E+00_JPRB, 0.87283E+00_JPRB, 0.86258E+00_JPRB ,&
&  0.84685E+00_JPRB, 0.82722E+00_JPRB, 0.79295E+00_JPRB, 0.76247E+00_JPRB, 0.73264E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.78011E+00_JPRB, 0.76970E+00_JPRB, 0.76586E+00_JPRB, 0.76496E+00_JPRB, 0.75335E+00_JPRB, 0.74441E+00_JPRB, 0.73906E+00_JPRB ,&
&  0.70835E+00_JPRB, 0.70786E+00_JPRB, 0.69754E+00_JPRB, 0.71015E+00_JPRB, 0.74534E+00_JPRB, 0.78146E+00_JPRB, 0.83474E+00_JPRB ,&
&  0.88348E+00_JPRB, 0.94721E+00_JPRB, 0.10464E+01_JPRB, 0.12588E+01_JPRB, 0.14994E+01_JPRB, 0.16759E+01_JPRB, 0.18774E+01_JPRB ,&
&  0.19559E+01_JPRB, 0.20217E+01_JPRB, 0.21525E+01_JPRB, 0.23322E+01_JPRB, 0.24233E+01_JPRB, 0.24588E+01_JPRB, 0.23770E+01_JPRB ,&
&  0.17382E+01_JPRB, 0.11891E+01_JPRB, 0.11839E+01_JPRB, 0.10956E+01_JPRB, 0.11117E+01_JPRB, 0.11393E+01_JPRB, 0.11742E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.12921E+01_JPRB, 0.13520E+01_JPRB, 0.14071E+01_JPRB, 0.14901E+01_JPRB, 0.15822E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.16329E+01_JPRB, 0.16597E+01_JPRB, 0.16848E+01_JPRB, 0.16376E+01_JPRB, 0.16873E+01_JPRB, 0.17960E+01_JPRB, 0.18886E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.19682E+01_JPRB, 0.16913E+01_JPRB, 0.16832E+01_JPRB, 0.18515E+01_JPRB, 0.17283E+01_JPRB, 0.17731E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.19995E+01_JPRB, 0.12749E+01_JPRB, 0.11704E+01_JPRB, 0.12306E+01_JPRB, 0.12682E+01_JPRB, 0.13368E+01_JPRB ,&
&  0.13427E+01_JPRB, 0.13989E+01_JPRB, 0.14704E+01_JPRB, 0.15179E+01_JPRB, 0.15465E+01_JPRB, 0.14875E+01_JPRB, 0.14450E+01_JPRB ,&
&  0.12964E+01_JPRB, 0.12515E+01_JPRB, 0.12822E+01_JPRB, 0.13173E+01_JPRB, 0.13677E+01_JPRB, 0.14609E+01_JPRB, 0.14925E+01_JPRB ,&
&  0.15384E+01_JPRB, 0.15681E+01_JPRB, 0.17280E+01_JPRB, 0.14163E+01_JPRB, 0.10278E+01_JPRB, 0.94102E+00_JPRB, 0.89436E+00_JPRB ,&
&  0.92692E+00_JPRB, 0.94204E+00_JPRB, 0.94506E+00_JPRB, 0.94298E+00_JPRB, 0.93794E+00_JPRB, 0.92337E+00_JPRB, 0.90683E+00_JPRB ,&
&  0.88683E+00_JPRB, 0.86377E+00_JPRB, 0.83944E+00_JPRB, 0.81694E+00_JPRB, 0.79760E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.81035E+00_JPRB, 0.80817E+00_JPRB, 0.80911E+00_JPRB, 0.79929E+00_JPRB, 0.78735E+00_JPRB, 0.79260E+00_JPRB, 0.79191E+00_JPRB ,&
&  0.77804E+00_JPRB, 0.76454E+00_JPRB, 0.74841E+00_JPRB, 0.76588E+00_JPRB, 0.79924E+00_JPRB, 0.86152E+00_JPRB, 0.93769E+00_JPRB ,&
&  0.10320E+01_JPRB, 0.11646E+01_JPRB, 0.13405E+01_JPRB, 0.14839E+01_JPRB, 0.16798E+01_JPRB, 0.17750E+01_JPRB, 0.18197E+01_JPRB ,&
&  0.18888E+01_JPRB, 0.19648E+01_JPRB, 0.20070E+01_JPRB, 0.20126E+01_JPRB, 0.19656E+01_JPRB, 0.19459E+01_JPRB, 0.19944E+01_JPRB ,&
&  0.16597E+01_JPRB, 0.12589E+01_JPRB, 0.11507E+01_JPRB, 0.10931E+01_JPRB, 0.10756E+01_JPRB, 0.10767E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.11745E+01_JPRB, 0.12406E+01_JPRB, 0.13300E+01_JPRB, 0.14059E+01_JPRB, 0.14789E+01_JPRB, 0.15311E+01_JPRB, 0.15589E+01_JPRB ,&
&  0.16044E+01_JPRB, 0.16460E+01_JPRB, 0.17014E+01_JPRB, 0.15996E+01_JPRB, 0.16966E+01_JPRB, 0.17227E+01_JPRB, 0.17443E+01_JPRB ,&
&  0.16970E+01_JPRB, 0.15817E+01_JPRB, 0.16244E+01_JPRB, 0.16395E+01_JPRB, 0.16558E+01_JPRB, 0.16396E+01_JPRB, 0.16263E+01_JPRB ,&
&  0.17591E+01_JPRB, 0.17561E+01_JPRB, 0.13431E+01_JPRB, 0.12628E+01_JPRB, 0.13283E+01_JPRB, 0.11879E+01_JPRB, 0.14028E+01_JPRB ,&
&  0.14243E+01_JPRB, 0.14803E+01_JPRB, 0.15433E+01_JPRB, 0.16657E+01_JPRB, 0.18026E+01_JPRB, 0.19025E+01_JPRB, 0.17272E+01_JPRB ,&
&  0.15298E+01_JPRB, 0.14833E+01_JPRB, 0.15087E+01_JPRB, 0.15343E+01_JPRB, 0.15863E+01_JPRB, 0.16579E+01_JPRB, 0.17002E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.16016E+01_JPRB, 0.15835E+01_JPRB, 0.13130E+01_JPRB, 0.11375E+01_JPRB, 0.10411E+01_JPRB, 0.96977E+00_JPRB ,&
&  0.95200E+00_JPRB, 0.96203E+00_JPRB, 0.95731E+00_JPRB, 0.94155E+00_JPRB, 0.93028E+00_JPRB, 0.90688E+00_JPRB, 0.89342E+00_JPRB ,&
&  0.88639E+00_JPRB, 0.87009E+00_JPRB, 0.85514E+00_JPRB, 0.84636E+00_JPRB, 0.82400E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.83106E+00_JPRB, 0.82055E+00_JPRB, 0.82396E+00_JPRB, 0.81012E+00_JPRB, 0.79724E+00_JPRB, 0.80188E+00_JPRB, 0.81406E+00_JPRB ,&
&  0.81338E+00_JPRB, 0.81308E+00_JPRB, 0.80718E+00_JPRB, 0.80551E+00_JPRB, 0.81908E+00_JPRB, 0.86325E+00_JPRB, 0.93701E+00_JPRB ,&
&  0.10321E+01_JPRB, 0.11685E+01_JPRB, 0.13422E+01_JPRB, 0.15050E+01_JPRB, 0.17101E+01_JPRB, 0.18031E+01_JPRB, 0.17870E+01_JPRB ,&
&  0.18513E+01_JPRB, 0.19274E+01_JPRB, 0.19000E+01_JPRB, 0.18348E+01_JPRB, 0.14807E+01_JPRB, 0.15407E+01_JPRB, 0.15583E+01_JPRB ,&
&  0.14907E+01_JPRB, 0.12486E+01_JPRB, 0.11082E+01_JPRB, 0.10729E+01_JPRB, 0.10348E+01_JPRB, 0.10220E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.11233E+01_JPRB, 0.12096E+01_JPRB, 0.13230E+01_JPRB, 0.14067E+01_JPRB, 0.14440E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.14957E+01_JPRB, 0.15121E+01_JPRB, 0.15207E+01_JPRB, 0.14893E+01_JPRB, 0.14781E+01_JPRB, 0.14825E+01_JPRB, 0.15039E+01_JPRB ,&
&  0.15239E+01_JPRB, 0.14385E+01_JPRB, 0.14420E+01_JPRB, 0.14786E+01_JPRB, 0.14225E+01_JPRB, 0.13567E+01_JPRB, 0.13918E+01_JPRB ,&
&  0.15293E+01_JPRB, 0.17718E+01_JPRB, 0.14116E+01_JPRB, 0.15259E+01_JPRB, 0.17475E+01_JPRB, 0.14925E+01_JPRB, 0.13836E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.15272E+01_JPRB, 0.16118E+01_JPRB, 0.17445E+01_JPRB, 0.19411E+01_JPRB, 0.21956E+01_JPRB, 0.18981E+01_JPRB ,&
&  0.17756E+01_JPRB, 0.17023E+01_JPRB, 0.17669E+01_JPRB, 0.18332E+01_JPRB, 0.18559E+01_JPRB, 0.18083E+01_JPRB, 0.17714E+01_JPRB ,&
&  0.16630E+01_JPRB, 0.17715E+01_JPRB, 0.18178E+01_JPRB, 0.17784E+01_JPRB, 0.17241E+01_JPRB, 0.11703E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.10379E+01_JPRB, 0.10200E+01_JPRB, 0.10094E+01_JPRB, 0.99100E+00_JPRB, 0.97243E+00_JPRB, 0.95111E+00_JPRB, 0.93526E+00_JPRB ,&
&  0.92260E+00_JPRB, 0.90424E+00_JPRB, 0.88949E+00_JPRB, 0.87760E+00_JPRB, 0.85062E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.87095E+00_JPRB, 0.85269E+00_JPRB, 0.84710E+00_JPRB, 0.84048E+00_JPRB, 0.83831E+00_JPRB, 0.83676E+00_JPRB, 0.83294E+00_JPRB ,&
&  0.81609E+00_JPRB, 0.81245E+00_JPRB, 0.81933E+00_JPRB, 0.82208E+00_JPRB, 0.81047E+00_JPRB, 0.81511E+00_JPRB, 0.85317E+00_JPRB ,&
&  0.93154E+00_JPRB, 0.10532E+01_JPRB, 0.12048E+01_JPRB, 0.13772E+01_JPRB, 0.16213E+01_JPRB, 0.18039E+01_JPRB, 0.18808E+01_JPRB ,&
&  0.20206E+01_JPRB, 0.20722E+01_JPRB, 0.18497E+01_JPRB, 0.13370E+01_JPRB, 0.11962E+01_JPRB, 0.14116E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.11611E+01_JPRB, 0.11299E+01_JPRB, 0.11126E+01_JPRB, 0.10909E+01_JPRB, 0.10550E+01_JPRB, 0.10295E+01_JPRB, 0.10077E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.10209E+01_JPRB, 0.10752E+01_JPRB, 0.11826E+01_JPRB, 0.12908E+01_JPRB, 0.13625E+01_JPRB, 0.13952E+01_JPRB ,&
&  0.14082E+01_JPRB, 0.14197E+01_JPRB, 0.14228E+01_JPRB, 0.13892E+01_JPRB, 0.13555E+01_JPRB, 0.13704E+01_JPRB, 0.13712E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.13560E+01_JPRB, 0.13402E+01_JPRB, 0.13597E+01_JPRB, 0.13553E+01_JPRB, 0.12240E+01_JPRB, 0.12565E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.15949E+01_JPRB, 0.15467E+01_JPRB, 0.18112E+01_JPRB, 0.11258E+01_JPRB, 0.12553E+01_JPRB, 0.13739E+01_JPRB ,&
&  0.13469E+01_JPRB, 0.14656E+01_JPRB, 0.15630E+01_JPRB, 0.16934E+01_JPRB, 0.18923E+01_JPRB, 0.19722E+01_JPRB, 0.21514E+01_JPRB ,&
&  0.19707E+01_JPRB, 0.18639E+01_JPRB, 0.19073E+01_JPRB, 0.20340E+01_JPRB, 0.20873E+01_JPRB, 0.20265E+01_JPRB, 0.18775E+01_JPRB ,&
&  0.17364E+01_JPRB, 0.19073E+01_JPRB, 0.18758E+01_JPRB, 0.19595E+01_JPRB, 0.15272E+01_JPRB, 0.12155E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11123E+01_JPRB, 0.11026E+01_JPRB, 0.10775E+01_JPRB, 0.10527E+01_JPRB, 0.10304E+01_JPRB, 0.10079E+01_JPRB ,&
&  0.99505E+00_JPRB, 0.97805E+00_JPRB, 0.95438E+00_JPRB, 0.92916E+00_JPRB, 0.89996E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.92458E+00_JPRB, 0.90407E+00_JPRB, 0.89831E+00_JPRB, 0.88648E+00_JPRB, 0.87657E+00_JPRB, 0.88067E+00_JPRB, 0.87537E+00_JPRB ,&
&  0.82771E+00_JPRB, 0.78892E+00_JPRB, 0.78630E+00_JPRB, 0.79741E+00_JPRB, 0.80113E+00_JPRB, 0.80245E+00_JPRB, 0.82151E+00_JPRB ,&
&  0.86936E+00_JPRB, 0.94529E+00_JPRB, 0.10447E+01_JPRB, 0.12311E+01_JPRB, 0.14783E+01_JPRB, 0.17725E+01_JPRB, 0.20457E+01_JPRB ,&
&  0.20324E+01_JPRB, 0.14971E+01_JPRB, 0.18077E+01_JPRB, 0.18256E+01_JPRB, 0.16058E+01_JPRB, 0.16090E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.12210E+01_JPRB, 0.12098E+01_JPRB, 0.11692E+01_JPRB, 0.10827E+01_JPRB, 0.10577E+01_JPRB, 0.10236E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.99673E+00_JPRB, 0.10137E+01_JPRB, 0.10788E+01_JPRB, 0.11606E+01_JPRB, 0.12584E+01_JPRB, 0.13443E+01_JPRB ,&
&  0.13511E+01_JPRB, 0.13265E+01_JPRB, 0.13255E+01_JPRB, 0.12742E+01_JPRB, 0.12906E+01_JPRB, 0.13143E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.12873E+01_JPRB, 0.13259E+01_JPRB, 0.13771E+01_JPRB, 0.14071E+01_JPRB, 0.12892E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.14843E+01_JPRB, 0.15545E+01_JPRB, 0.17075E+01_JPRB, 0.12558E+01_JPRB, 0.13619E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.13108E+01_JPRB, 0.13922E+01_JPRB, 0.14881E+01_JPRB, 0.16958E+01_JPRB, 0.18701E+01_JPRB, 0.22653E+01_JPRB ,&
&  0.21132E+01_JPRB, 0.20705E+01_JPRB, 0.20311E+01_JPRB, 0.21735E+01_JPRB, 0.20989E+01_JPRB, 0.21488E+01_JPRB, 0.21222E+01_JPRB ,&
&  0.22668E+01_JPRB, 0.21831E+01_JPRB, 0.21049E+01_JPRB, 0.20797E+01_JPRB, 0.17568E+01_JPRB, 0.14667E+01_JPRB, 0.13528E+01_JPRB ,&
&  0.12788E+01_JPRB, 0.12566E+01_JPRB, 0.12357E+01_JPRB, 0.12110E+01_JPRB, 0.11771E+01_JPRB, 0.11381E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.10607E+01_JPRB, 0.10337E+01_JPRB, 0.10036E+01_JPRB, 0.96510E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.99791E+00_JPRB, 0.95497E+00_JPRB, 0.93561E+00_JPRB, 0.94081E+00_JPRB, 0.93958E+00_JPRB, 0.98203E+00_JPRB, 0.10357E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.87357E+00_JPRB, 0.78691E+00_JPRB, 0.76287E+00_JPRB, 0.76702E+00_JPRB, 0.77817E+00_JPRB, 0.79892E+00_JPRB ,&
&  0.82876E+00_JPRB, 0.89306E+00_JPRB, 0.97302E+00_JPRB, 0.10671E+01_JPRB, 0.12977E+01_JPRB, 0.16092E+01_JPRB, 0.18054E+01_JPRB ,&
&  0.19294E+01_JPRB, 0.20163E+01_JPRB, 0.21066E+01_JPRB, 0.18290E+01_JPRB, 0.16588E+01_JPRB, 0.17091E+01_JPRB, 0.16214E+01_JPRB ,&
&  0.14224E+01_JPRB, 0.12846E+01_JPRB, 0.12292E+01_JPRB, 0.11548E+01_JPRB, 0.11170E+01_JPRB, 0.10891E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10347E+01_JPRB, 0.10118E+01_JPRB, 0.10082E+01_JPRB, 0.10432E+01_JPRB, 0.10932E+01_JPRB, 0.11634E+01_JPRB, 0.12748E+01_JPRB ,&
&  0.13313E+01_JPRB, 0.12970E+01_JPRB, 0.12664E+01_JPRB, 0.12144E+01_JPRB, 0.12531E+01_JPRB, 0.12861E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.12923E+01_JPRB, 0.13422E+01_JPRB, 0.13763E+01_JPRB, 0.13503E+01_JPRB, 0.12681E+01_JPRB, 0.14068E+01_JPRB, 0.13729E+01_JPRB ,&
&  0.14451E+01_JPRB, 0.14272E+01_JPRB, 0.16419E+01_JPRB, 0.13126E+01_JPRB, 0.14621E+01_JPRB, 0.15212E+01_JPRB, 0.14374E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.12533E+01_JPRB, 0.12339E+01_JPRB, 0.12659E+01_JPRB, 0.14416E+01_JPRB, 0.16158E+01_JPRB, 0.20451E+01_JPRB ,&
&  0.20388E+01_JPRB, 0.22763E+01_JPRB, 0.21713E+01_JPRB, 0.21282E+01_JPRB, 0.19522E+01_JPRB, 0.19514E+01_JPRB, 0.21540E+01_JPRB ,&
&  0.25314E+01_JPRB, 0.26160E+01_JPRB, 0.25637E+01_JPRB, 0.23914E+01_JPRB, 0.20297E+01_JPRB, 0.17965E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.16011E+01_JPRB, 0.15098E+01_JPRB, 0.14156E+01_JPRB, 0.13696E+01_JPRB, 0.13203E+01_JPRB, 0.12573E+01_JPRB, 0.11929E+01_JPRB ,&
&  0.11614E+01_JPRB, 0.11400E+01_JPRB, 0.11189E+01_JPRB, 0.10905E+01_JPRB, 0.10517E+01_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.10790E+01_JPRB, 0.10146E+01_JPRB, 0.95199E+00_JPRB, 0.92806E+00_JPRB, 0.90906E+00_JPRB, 0.91829E+00_JPRB, 0.95563E+00_JPRB ,&
&  0.10371E+01_JPRB, 0.88984E+00_JPRB, 0.81122E+00_JPRB, 0.75748E+00_JPRB, 0.74407E+00_JPRB, 0.75819E+00_JPRB, 0.77322E+00_JPRB ,&
&  0.79162E+00_JPRB, 0.82727E+00_JPRB, 0.89522E+00_JPRB, 0.94947E+00_JPRB, 0.11208E+01_JPRB, 0.13615E+01_JPRB, 0.13672E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.25110E+01_JPRB, 0.21347E+01_JPRB, 0.19400E+01_JPRB, 0.17862E+01_JPRB, 0.18530E+01_JPRB, 0.15987E+01_JPRB ,&
&  0.13579E+01_JPRB, 0.12908E+01_JPRB, 0.12695E+01_JPRB, 0.12019E+01_JPRB, 0.11732E+01_JPRB, 0.11186E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10535E+01_JPRB, 0.10285E+01_JPRB, 0.10370E+01_JPRB, 0.10722E+01_JPRB, 0.11204E+01_JPRB, 0.12048E+01_JPRB ,&
&  0.12900E+01_JPRB, 0.13001E+01_JPRB, 0.12483E+01_JPRB, 0.12453E+01_JPRB, 0.12675E+01_JPRB, 0.13100E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.12682E+01_JPRB, 0.14187E+01_JPRB, 0.14779E+01_JPRB, 0.14850E+01_JPRB, 0.15591E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.15863E+01_JPRB, 0.14661E+01_JPRB, 0.15912E+01_JPRB, 0.13417E+01_JPRB, 0.14898E+01_JPRB, 0.16135E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13221E+01_JPRB, 0.12245E+01_JPRB, 0.12167E+01_JPRB, 0.13075E+01_JPRB, 0.15000E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.19139E+01_JPRB, 0.21889E+01_JPRB, 0.24394E+01_JPRB, 0.19947E+01_JPRB, 0.18529E+01_JPRB, 0.18594E+01_JPRB, 0.21426E+01_JPRB ,&
&  0.28312E+01_JPRB, 0.29853E+01_JPRB, 0.30844E+01_JPRB, 0.28921E+01_JPRB, 0.23383E+01_JPRB, 0.20590E+01_JPRB, 0.19455E+01_JPRB ,&
&  0.18740E+01_JPRB, 0.18093E+01_JPRB, 0.16591E+01_JPRB, 0.15297E+01_JPRB, 0.14446E+01_JPRB, 0.13704E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.11910E+01_JPRB, 0.11703E+01_JPRB, 0.11579E+01_JPRB, 0.11269E+01_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.11246E+01_JPRB, 0.10539E+01_JPRB, 0.10052E+01_JPRB, 0.96280E+00_JPRB, 0.91527E+00_JPRB, 0.90060E+00_JPRB, 0.95281E+00_JPRB ,&
&  0.95003E+00_JPRB, 0.85688E+00_JPRB, 0.80624E+00_JPRB, 0.76743E+00_JPRB, 0.74363E+00_JPRB, 0.74230E+00_JPRB, 0.75744E+00_JPRB ,&
&  0.76405E+00_JPRB, 0.78755E+00_JPRB, 0.82140E+00_JPRB, 0.92144E+00_JPRB, 0.10579E+01_JPRB, 0.13066E+01_JPRB, 0.10165E+01_JPRB ,&
&  0.14373E+01_JPRB, 0.20969E+01_JPRB, 0.21530E+01_JPRB, 0.21198E+01_JPRB, 0.20223E+01_JPRB, 0.19658E+01_JPRB, 0.17735E+01_JPRB ,&
&  0.15169E+01_JPRB, 0.13830E+01_JPRB, 0.12912E+01_JPRB, 0.12494E+01_JPRB, 0.12345E+01_JPRB, 0.11926E+01_JPRB, 0.11377E+01_JPRB ,&
&  0.11054E+01_JPRB, 0.10727E+01_JPRB, 0.10385E+01_JPRB, 0.10290E+01_JPRB, 0.10494E+01_JPRB, 0.10901E+01_JPRB, 0.11565E+01_JPRB ,&
&  0.12375E+01_JPRB, 0.12950E+01_JPRB, 0.13084E+01_JPRB, 0.12321E+01_JPRB, 0.12670E+01_JPRB, 0.12934E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.12844E+01_JPRB, 0.12484E+01_JPRB, 0.15023E+01_JPRB, 0.17633E+01_JPRB, 0.18894E+01_JPRB, 0.18084E+01_JPRB, 0.16686E+01_JPRB ,&
&  0.17289E+01_JPRB, 0.16898E+01_JPRB, 0.15792E+01_JPRB, 0.14059E+01_JPRB, 0.15148E+01_JPRB, 0.16572E+01_JPRB, 0.17251E+01_JPRB ,&
&  0.15951E+01_JPRB, 0.14148E+01_JPRB, 0.12958E+01_JPRB, 0.12377E+01_JPRB, 0.12412E+01_JPRB, 0.13707E+01_JPRB, 0.16459E+01_JPRB ,&
&  0.19405E+01_JPRB, 0.23737E+01_JPRB, 0.24087E+01_JPRB, 0.19510E+01_JPRB, 0.17879E+01_JPRB, 0.18419E+01_JPRB, 0.20255E+01_JPRB ,&
&  0.29373E+01_JPRB, 0.31564E+01_JPRB, 0.29495E+01_JPRB, 0.31605E+01_JPRB, 0.26006E+01_JPRB, 0.22636E+01_JPRB, 0.21159E+01_JPRB ,&
&  0.20610E+01_JPRB, 0.19810E+01_JPRB, 0.18884E+01_JPRB, 0.17856E+01_JPRB, 0.16222E+01_JPRB, 0.14587E+01_JPRB, 0.13457E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12508E+01_JPRB, 0.12199E+01_JPRB, 0.11985E+01_JPRB, 0.11721E+01_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.11271E+01_JPRB, 0.10728E+01_JPRB, 0.10165E+01_JPRB, 0.97716E+00_JPRB, 0.92826E+00_JPRB, 0.88883E+00_JPRB, 0.90104E+00_JPRB ,&
&  0.87999E+00_JPRB, 0.82847E+00_JPRB, 0.77324E+00_JPRB, 0.75360E+00_JPRB, 0.74087E+00_JPRB, 0.73206E+00_JPRB, 0.73835E+00_JPRB ,&
&  0.74174E+00_JPRB, 0.76440E+00_JPRB, 0.82780E+00_JPRB, 0.92940E+00_JPRB, 0.94892E+00_JPRB, 0.87831E+00_JPRB, 0.10200E+01_JPRB ,&
&  0.16051E+01_JPRB, 0.20657E+01_JPRB, 0.21592E+01_JPRB, 0.23076E+01_JPRB, 0.23528E+01_JPRB, 0.24037E+01_JPRB, 0.22033E+01_JPRB ,&
&  0.18907E+01_JPRB, 0.16803E+01_JPRB, 0.15219E+01_JPRB, 0.14294E+01_JPRB, 0.13471E+01_JPRB, 0.12768E+01_JPRB, 0.12364E+01_JPRB ,&
&  0.11869E+01_JPRB, 0.11511E+01_JPRB, 0.11006E+01_JPRB, 0.10408E+01_JPRB, 0.10491E+01_JPRB, 0.10862E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11806E+01_JPRB, 0.12621E+01_JPRB, 0.12939E+01_JPRB, 0.12282E+01_JPRB, 0.12356E+01_JPRB, 0.12667E+01_JPRB, 0.13645E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.15260E+01_JPRB, 0.16867E+01_JPRB, 0.18462E+01_JPRB, 0.20634E+01_JPRB, 0.21394E+01_JPRB, 0.20482E+01_JPRB ,&
&  0.20026E+01_JPRB, 0.17986E+01_JPRB, 0.14967E+01_JPRB, 0.15076E+01_JPRB, 0.16284E+01_JPRB, 0.18110E+01_JPRB, 0.16707E+01_JPRB ,&
&  0.14088E+01_JPRB, 0.11983E+01_JPRB, 0.11322E+01_JPRB, 0.11537E+01_JPRB, 0.12828E+01_JPRB, 0.14083E+01_JPRB, 0.17291E+01_JPRB ,&
&  0.16759E+01_JPRB, 0.11939E+01_JPRB, 0.10374E+01_JPRB, 0.13036E+01_JPRB, 0.12184E+01_JPRB, 0.13147E+01_JPRB, 0.21211E+01_JPRB ,&
&  0.29502E+01_JPRB, 0.35475E+01_JPRB, 0.34643E+01_JPRB, 0.30313E+01_JPRB, 0.27172E+01_JPRB, 0.23500E+01_JPRB, 0.22711E+01_JPRB ,&
&  0.21565E+01_JPRB, 0.20412E+01_JPRB, 0.19995E+01_JPRB, 0.19173E+01_JPRB, 0.17623E+01_JPRB, 0.15697E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.12648E+01_JPRB, 0.12195E+01_JPRB, 0.11865E+01_JPRB, 0.11589E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.10908E+01_JPRB, 0.10594E+01_JPRB, 0.10106E+01_JPRB, 0.97029E+00_JPRB, 0.92781E+00_JPRB, 0.88450E+00_JPRB, 0.86614E+00_JPRB ,&
&  0.83214E+00_JPRB, 0.78508E+00_JPRB, 0.74601E+00_JPRB, 0.73267E+00_JPRB, 0.72650E+00_JPRB, 0.72303E+00_JPRB, 0.72833E+00_JPRB ,&
&  0.73447E+00_JPRB, 0.75430E+00_JPRB, 0.85206E+00_JPRB, 0.97459E+00_JPRB, 0.95814E+00_JPRB, 0.90841E+00_JPRB, 0.10352E+01_JPRB ,&
&  0.15381E+01_JPRB, 0.20501E+01_JPRB, 0.23691E+01_JPRB, 0.26466E+01_JPRB, 0.26712E+01_JPRB, 0.26824E+01_JPRB, 0.25509E+01_JPRB ,&
&  0.23266E+01_JPRB, 0.21008E+01_JPRB, 0.18879E+01_JPRB, 0.16476E+01_JPRB, 0.14522E+01_JPRB, 0.13405E+01_JPRB, 0.12637E+01_JPRB ,&
&  0.12328E+01_JPRB, 0.11996E+01_JPRB, 0.11607E+01_JPRB, 0.10958E+01_JPRB, 0.10615E+01_JPRB, 0.10826E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.11327E+01_JPRB, 0.12073E+01_JPRB, 0.12616E+01_JPRB, 0.12666E+01_JPRB, 0.10604E+01_JPRB, 0.12366E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.14544E+01_JPRB, 0.16049E+01_JPRB, 0.16432E+01_JPRB, 0.20393E+01_JPRB, 0.23480E+01_JPRB, 0.23319E+01_JPRB, 0.22749E+01_JPRB ,&
&  0.23285E+01_JPRB, 0.20663E+01_JPRB, 0.17451E+01_JPRB, 0.17871E+01_JPRB, 0.18805E+01_JPRB, 0.19808E+01_JPRB, 0.13307E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.14758E+01_JPRB, 0.13410E+01_JPRB, 0.10938E+01_JPRB, 0.13159E+01_JPRB, 0.14663E+01_JPRB, 0.11655E+01_JPRB ,&
&  0.65734E+00_JPRB, 0.56028E+00_JPRB, 0.59269E+00_JPRB, 0.64054E+00_JPRB, 0.61571E+00_JPRB, 0.10170E+01_JPRB, 0.30612E+01_JPRB ,&
&  0.29463E+01_JPRB, 0.37193E+01_JPRB, 0.37040E+01_JPRB, 0.34114E+01_JPRB, 0.29492E+01_JPRB, 0.25040E+01_JPRB, 0.24229E+01_JPRB ,&
&  0.22702E+01_JPRB, 0.21733E+01_JPRB, 0.20520E+01_JPRB, 0.19377E+01_JPRB, 0.17972E+01_JPRB, 0.16191E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.13656E+01_JPRB, 0.12900E+01_JPRB, 0.12197E+01_JPRB, 0.11695E+01_JPRB, 0.11208E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10712E+01_JPRB, 0.10233E+01_JPRB, 0.98120E+00_JPRB, 0.94199E+00_JPRB, 0.89969E+00_JPRB, 0.86397E+00_JPRB, 0.83816E+00_JPRB ,&
&  0.79976E+00_JPRB, 0.75112E+00_JPRB, 0.71915E+00_JPRB, 0.70312E+00_JPRB, 0.70439E+00_JPRB, 0.71879E+00_JPRB, 0.72877E+00_JPRB ,&
&  0.74827E+00_JPRB, 0.78763E+00_JPRB, 0.94033E+00_JPRB, 0.94316E+00_JPRB, 0.93903E+00_JPRB, 0.91117E+00_JPRB, 0.98173E+00_JPRB ,&
&  0.12910E+01_JPRB, 0.18247E+01_JPRB, 0.23006E+01_JPRB, 0.26725E+01_JPRB, 0.30450E+01_JPRB, 0.31529E+01_JPRB, 0.30468E+01_JPRB ,&
&  0.27581E+01_JPRB, 0.24849E+01_JPRB, 0.21767E+01_JPRB, 0.18334E+01_JPRB, 0.15793E+01_JPRB, 0.14260E+01_JPRB, 0.13142E+01_JPRB ,&
&  0.12283E+01_JPRB, 0.12004E+01_JPRB, 0.11658E+01_JPRB, 0.11281E+01_JPRB, 0.10952E+01_JPRB, 0.11068E+01_JPRB, 0.11363E+01_JPRB ,&
&  0.11452E+01_JPRB, 0.11695E+01_JPRB, 0.12156E+01_JPRB, 0.12287E+01_JPRB, 0.11016E+01_JPRB, 0.97507E+00_JPRB, 0.11485E+01_JPRB ,&
&  0.13740E+01_JPRB, 0.17499E+01_JPRB, 0.19300E+01_JPRB, 0.24627E+01_JPRB, 0.25962E+01_JPRB, 0.25118E+01_JPRB, 0.25813E+01_JPRB ,&
&  0.25073E+01_JPRB, 0.25754E+01_JPRB, 0.19821E+01_JPRB, 0.20088E+01_JPRB, 0.21815E+01_JPRB, 0.16653E+01_JPRB, 0.15650E+01_JPRB ,&
&  0.18335E+01_JPRB, 0.17192E+01_JPRB, 0.14827E+01_JPRB, 0.11329E+01_JPRB, 0.13956E+01_JPRB, 0.12572E+01_JPRB, 0.69101E+00_JPRB ,&
&  0.58368E+00_JPRB, 0.54711E+00_JPRB, 0.52326E+00_JPRB, 0.51543E+00_JPRB, 0.54799E+00_JPRB, 0.89266E+00_JPRB, 0.16689E+01_JPRB ,&
&  0.20626E+01_JPRB, 0.30780E+01_JPRB, 0.35503E+01_JPRB, 0.36390E+01_JPRB, 0.32141E+01_JPRB, 0.28294E+01_JPRB, 0.25651E+01_JPRB ,&
&  0.24398E+01_JPRB, 0.24112E+01_JPRB, 0.21531E+01_JPRB, 0.19579E+01_JPRB, 0.17796E+01_JPRB, 0.16246E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.13730E+01_JPRB, 0.12859E+01_JPRB, 0.12164E+01_JPRB, 0.11779E+01_JPRB, 0.11252E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.10307E+01_JPRB, 0.98035E+00_JPRB, 0.93521E+00_JPRB, 0.89120E+00_JPRB, 0.84390E+00_JPRB, 0.80457E+00_JPRB, 0.79150E+00_JPRB ,&
&  0.77057E+00_JPRB, 0.72867E+00_JPRB, 0.70025E+00_JPRB, 0.68172E+00_JPRB, 0.69309E+00_JPRB, 0.71906E+00_JPRB, 0.73016E+00_JPRB ,&
&  0.75463E+00_JPRB, 0.83904E+00_JPRB, 0.95404E+00_JPRB, 0.85930E+00_JPRB, 0.78606E+00_JPRB, 0.74453E+00_JPRB, 0.76214E+00_JPRB ,&
&  0.11055E+01_JPRB, 0.14803E+01_JPRB, 0.18730E+01_JPRB, 0.23842E+01_JPRB, 0.28290E+01_JPRB, 0.28295E+01_JPRB, 0.30908E+01_JPRB ,&
&  0.29455E+01_JPRB, 0.25866E+01_JPRB, 0.22397E+01_JPRB, 0.19030E+01_JPRB, 0.16603E+01_JPRB, 0.14722E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.11952E+01_JPRB, 0.11629E+01_JPRB, 0.11408E+01_JPRB, 0.11334E+01_JPRB, 0.11397E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11727E+01_JPRB, 0.11872E+01_JPRB, 0.12063E+01_JPRB, 0.12795E+01_JPRB, 0.12931E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.13924E+01_JPRB, 0.21062E+01_JPRB, 0.25076E+01_JPRB, 0.26306E+01_JPRB, 0.27481E+01_JPRB, 0.27119E+01_JPRB ,&
&  0.26251E+01_JPRB, 0.26674E+01_JPRB, 0.23330E+01_JPRB, 0.24368E+01_JPRB, 0.21156E+01_JPRB, 0.14868E+01_JPRB, 0.18552E+01_JPRB ,&
&  0.20558E+01_JPRB, 0.20148E+01_JPRB, 0.16284E+01_JPRB, 0.11533E+01_JPRB, 0.11267E+01_JPRB, 0.81551E+00_JPRB, 0.58691E+00_JPRB ,&
&  0.55821E+00_JPRB, 0.53756E+00_JPRB, 0.49764E+00_JPRB, 0.52052E+00_JPRB, 0.53199E+00_JPRB, 0.77176E+00_JPRB, 0.11769E+01_JPRB ,&
&  0.17425E+01_JPRB, 0.24391E+01_JPRB, 0.31103E+01_JPRB, 0.33988E+01_JPRB, 0.31890E+01_JPRB, 0.31203E+01_JPRB, 0.30529E+01_JPRB ,&
&  0.26515E+01_JPRB, 0.24834E+01_JPRB, 0.22596E+01_JPRB, 0.19728E+01_JPRB, 0.17467E+01_JPRB, 0.15840E+01_JPRB, 0.14540E+01_JPRB ,&
&  0.13441E+01_JPRB, 0.12424E+01_JPRB, 0.11702E+01_JPRB, 0.11224E+01_JPRB, 0.10805E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.97217E+00_JPRB, 0.92910E+00_JPRB, 0.89008E+00_JPRB, 0.84733E+00_JPRB, 0.80698E+00_JPRB, 0.76200E+00_JPRB, 0.74880E+00_JPRB ,&
&  0.74562E+00_JPRB, 0.70604E+00_JPRB, 0.66927E+00_JPRB, 0.66210E+00_JPRB, 0.68254E+00_JPRB, 0.71201E+00_JPRB, 0.74880E+00_JPRB ,&
&  0.80933E+00_JPRB, 0.10516E+01_JPRB, 0.78679E+00_JPRB, 0.64619E+00_JPRB, 0.70526E+00_JPRB, 0.73261E+00_JPRB, 0.69116E+00_JPRB ,&
&  0.96538E+00_JPRB, 0.12652E+01_JPRB, 0.16081E+01_JPRB, 0.22302E+01_JPRB, 0.30816E+01_JPRB, 0.30756E+01_JPRB, 0.32221E+01_JPRB ,&
&  0.32451E+01_JPRB, 0.27868E+01_JPRB, 0.23209E+01_JPRB, 0.19459E+01_JPRB, 0.16805E+01_JPRB, 0.14889E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.12802E+01_JPRB, 0.12146E+01_JPRB, 0.11679E+01_JPRB, 0.11624E+01_JPRB, 0.11650E+01_JPRB, 0.11664E+01_JPRB, 0.12045E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.12007E+01_JPRB, 0.12107E+01_JPRB, 0.12544E+01_JPRB, 0.13929E+01_JPRB, 0.13154E+01_JPRB, 0.18076E+01_JPRB ,&
&  0.16486E+01_JPRB, 0.18874E+01_JPRB, 0.22124E+01_JPRB, 0.27926E+01_JPRB, 0.29691E+01_JPRB, 0.29041E+01_JPRB, 0.31442E+01_JPRB ,&
&  0.24305E+01_JPRB, 0.21319E+01_JPRB, 0.22260E+01_JPRB, 0.21902E+01_JPRB, 0.17005E+01_JPRB, 0.18127E+01_JPRB, 0.28101E+01_JPRB ,&
&  0.22016E+01_JPRB, 0.24527E+01_JPRB, 0.25617E+01_JPRB, 0.25303E+01_JPRB, 0.11885E+01_JPRB, 0.77517E+00_JPRB, 0.11598E+01_JPRB ,&
&  0.12775E+01_JPRB, 0.87244E+00_JPRB, 0.65921E+00_JPRB, 0.84158E+00_JPRB, 0.67962E+00_JPRB, 0.86353E+00_JPRB, 0.12328E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.17256E+01_JPRB, 0.27475E+01_JPRB, 0.31646E+01_JPRB, 0.32061E+01_JPRB, 0.31789E+01_JPRB, 0.29584E+01_JPRB ,&
&  0.28368E+01_JPRB, 0.25028E+01_JPRB, 0.24126E+01_JPRB, 0.20724E+01_JPRB, 0.17861E+01_JPRB, 0.15757E+01_JPRB, 0.14079E+01_JPRB ,&
&  0.12869E+01_JPRB, 0.11781E+01_JPRB, 0.11001E+01_JPRB, 0.10537E+01_JPRB, 0.10151E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.93373E+00_JPRB, 0.89368E+00_JPRB, 0.85402E+00_JPRB, 0.80835E+00_JPRB, 0.76461E+00_JPRB, 0.72998E+00_JPRB, 0.71291E+00_JPRB ,&
&  0.71696E+00_JPRB, 0.67846E+00_JPRB, 0.64725E+00_JPRB, 0.64083E+00_JPRB, 0.65050E+00_JPRB, 0.67282E+00_JPRB, 0.71286E+00_JPRB ,&
&  0.77663E+00_JPRB, 0.79194E+00_JPRB, 0.59225E+00_JPRB, 0.54003E+00_JPRB, 0.69393E+00_JPRB, 0.68680E+00_JPRB, 0.74276E+00_JPRB ,&
&  0.93654E+00_JPRB, 0.11428E+01_JPRB, 0.14549E+01_JPRB, 0.20181E+01_JPRB, 0.24950E+01_JPRB, 0.29817E+01_JPRB, 0.32503E+01_JPRB ,&
&  0.33913E+01_JPRB, 0.30906E+01_JPRB, 0.26348E+01_JPRB, 0.21735E+01_JPRB, 0.18121E+01_JPRB, 0.15652E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.12866E+01_JPRB, 0.11896E+01_JPRB, 0.11325E+01_JPRB, 0.10915E+01_JPRB, 0.11176E+01_JPRB, 0.11367E+01_JPRB, 0.11832E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.12063E+01_JPRB, 0.12266E+01_JPRB, 0.12534E+01_JPRB, 0.12862E+01_JPRB, 0.13304E+01_JPRB, 0.14786E+01_JPRB ,&
&  0.17027E+01_JPRB, 0.20479E+01_JPRB, 0.24341E+01_JPRB, 0.27243E+01_JPRB, 0.29863E+01_JPRB, 0.29424E+01_JPRB, 0.34296E+01_JPRB ,&
&  0.28229E+01_JPRB, 0.23046E+01_JPRB, 0.19479E+01_JPRB, 0.15482E+01_JPRB, 0.19046E+01_JPRB, 0.29060E+01_JPRB, 0.29664E+01_JPRB ,&
&  0.28438E+01_JPRB, 0.29078E+01_JPRB, 0.28837E+01_JPRB, 0.24130E+01_JPRB, 0.19696E+01_JPRB, 0.14655E+01_JPRB, 0.17496E+01_JPRB ,&
&  0.17694E+01_JPRB, 0.16595E+01_JPRB, 0.15088E+01_JPRB, 0.13184E+01_JPRB, 0.10875E+01_JPRB, 0.13037E+01_JPRB, 0.13394E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.14350E+01_JPRB, 0.21926E+01_JPRB, 0.28628E+01_JPRB, 0.29881E+01_JPRB, 0.27813E+01_JPRB, 0.28649E+01_JPRB ,&
&  0.26334E+01_JPRB, 0.25438E+01_JPRB, 0.23094E+01_JPRB, 0.20254E+01_JPRB, 0.17682E+01_JPRB, 0.15442E+01_JPRB, 0.13579E+01_JPRB ,&
&  0.12325E+01_JPRB, 0.11365E+01_JPRB, 0.10474E+01_JPRB, 0.10038E+01_JPRB, 0.96764E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.89891E+00_JPRB, 0.86814E+00_JPRB, 0.82873E+00_JPRB, 0.78664E+00_JPRB, 0.74054E+00_JPRB, 0.70824E+00_JPRB, 0.69038E+00_JPRB ,&
&  0.68835E+00_JPRB, 0.65468E+00_JPRB, 0.64021E+00_JPRB, 0.63171E+00_JPRB, 0.63493E+00_JPRB, 0.65157E+00_JPRB, 0.68857E+00_JPRB ,&
&  0.73905E+00_JPRB, 0.65664E+00_JPRB, 0.51141E+00_JPRB, 0.53822E+00_JPRB, 0.61087E+00_JPRB, 0.62405E+00_JPRB, 0.74178E+00_JPRB ,&
&  0.88872E+00_JPRB, 0.10502E+01_JPRB, 0.12539E+01_JPRB, 0.16306E+01_JPRB, 0.20118E+01_JPRB, 0.23597E+01_JPRB, 0.26510E+01_JPRB ,&
&  0.27113E+01_JPRB, 0.28275E+01_JPRB, 0.27565E+01_JPRB, 0.24947E+01_JPRB, 0.21220E+01_JPRB, 0.17963E+01_JPRB, 0.15906E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.12703E+01_JPRB, 0.11576E+01_JPRB, 0.10955E+01_JPRB, 0.10769E+01_JPRB, 0.11050E+01_JPRB, 0.11647E+01_JPRB ,&
&  0.12108E+01_JPRB, 0.11937E+01_JPRB, 0.11693E+01_JPRB, 0.12333E+01_JPRB, 0.12539E+01_JPRB, 0.13394E+01_JPRB, 0.13936E+01_JPRB ,&
&  0.19819E+01_JPRB, 0.24440E+01_JPRB, 0.27111E+01_JPRB, 0.30341E+01_JPRB, 0.26173E+01_JPRB, 0.27572E+01_JPRB, 0.34172E+01_JPRB ,&
&  0.36836E+01_JPRB, 0.37116E+01_JPRB, 0.38347E+01_JPRB, 0.26737E+01_JPRB, 0.23640E+01_JPRB, 0.33166E+01_JPRB, 0.31950E+01_JPRB ,&
&  0.29075E+01_JPRB, 0.29708E+01_JPRB, 0.28645E+01_JPRB, 0.30188E+01_JPRB, 0.22578E+01_JPRB, 0.19837E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.13325E+01_JPRB, 0.11676E+01_JPRB, 0.13428E+01_JPRB, 0.12471E+01_JPRB, 0.12349E+01_JPRB, 0.12486E+01_JPRB, 0.12228E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.13242E+01_JPRB, 0.15709E+01_JPRB, 0.20899E+01_JPRB, 0.25631E+01_JPRB, 0.23053E+01_JPRB, 0.23404E+01_JPRB ,&
&  0.22519E+01_JPRB, 0.21613E+01_JPRB, 0.19565E+01_JPRB, 0.19011E+01_JPRB, 0.17853E+01_JPRB, 0.16035E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.12219E+01_JPRB, 0.11168E+01_JPRB, 0.10297E+01_JPRB, 0.97052E+00_JPRB, 0.92981E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.85468E+00_JPRB, 0.83480E+00_JPRB, 0.80223E+00_JPRB, 0.76254E+00_JPRB, 0.72174E+00_JPRB, 0.69512E+00_JPRB, 0.67764E+00_JPRB ,&
&  0.66999E+00_JPRB, 0.65110E+00_JPRB, 0.63371E+00_JPRB, 0.62294E+00_JPRB, 0.62256E+00_JPRB, 0.62944E+00_JPRB, 0.66109E+00_JPRB ,&
&  0.72841E+00_JPRB, 0.72231E+00_JPRB, 0.57588E+00_JPRB, 0.52063E+00_JPRB, 0.50193E+00_JPRB, 0.60662E+00_JPRB, 0.73611E+00_JPRB ,&
&  0.89712E+00_JPRB, 0.99678E+00_JPRB, 0.11738E+01_JPRB, 0.13956E+01_JPRB, 0.16487E+01_JPRB, 0.18619E+01_JPRB, 0.20295E+01_JPRB ,&
&  0.24827E+01_JPRB, 0.22781E+01_JPRB, 0.23478E+01_JPRB, 0.22345E+01_JPRB, 0.21740E+01_JPRB, 0.19456E+01_JPRB, 0.17012E+01_JPRB ,&
&  0.15107E+01_JPRB, 0.13469E+01_JPRB, 0.11992E+01_JPRB, 0.10876E+01_JPRB, 0.10305E+01_JPRB, 0.10290E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.11774E+01_JPRB, 0.11477E+01_JPRB, 0.11713E+01_JPRB, 0.12929E+01_JPRB, 0.14466E+01_JPRB, 0.16278E+01_JPRB ,&
&  0.17310E+01_JPRB, 0.19095E+01_JPRB, 0.29319E+01_JPRB, 0.30923E+01_JPRB, 0.34025E+01_JPRB, 0.33732E+01_JPRB, 0.32907E+01_JPRB ,&
&  0.35886E+01_JPRB, 0.37294E+01_JPRB, 0.38892E+01_JPRB, 0.31872E+01_JPRB, 0.37332E+01_JPRB, 0.37044E+01_JPRB, 0.33996E+01_JPRB ,&
&  0.31155E+01_JPRB, 0.30954E+01_JPRB, 0.29304E+01_JPRB, 0.28639E+01_JPRB, 0.28770E+01_JPRB, 0.29568E+01_JPRB, 0.23073E+01_JPRB ,&
&  0.17409E+01_JPRB, 0.18616E+01_JPRB, 0.16308E+01_JPRB, 0.13246E+01_JPRB, 0.10882E+01_JPRB, 0.11336E+01_JPRB, 0.11841E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.12870E+01_JPRB, 0.13756E+01_JPRB, 0.16613E+01_JPRB, 0.20161E+01_JPRB, 0.20878E+01_JPRB, 0.19358E+01_JPRB ,&
&  0.16538E+01_JPRB, 0.17309E+01_JPRB, 0.16885E+01_JPRB, 0.16049E+01_JPRB, 0.15746E+01_JPRB, 0.14730E+01_JPRB, 0.13276E+01_JPRB ,&
&  0.11773E+01_JPRB, 0.10688E+01_JPRB, 0.99058E+00_JPRB, 0.92472E+00_JPRB, 0.87784E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.76975E+00_JPRB, 0.78165E+00_JPRB, 0.77583E+00_JPRB, 0.75360E+00_JPRB, 0.72086E+00_JPRB, 0.69147E+00_JPRB, 0.66592E+00_JPRB ,&
&  0.65509E+00_JPRB, 0.64006E+00_JPRB, 0.61693E+00_JPRB, 0.61430E+00_JPRB, 0.61105E+00_JPRB, 0.61552E+00_JPRB, 0.64802E+00_JPRB ,&
&  0.72657E+00_JPRB, 0.86615E+00_JPRB, 0.70125E+00_JPRB, 0.66979E+00_JPRB, 0.64667E+00_JPRB, 0.74439E+00_JPRB, 0.83501E+00_JPRB ,&
&  0.94158E+00_JPRB, 0.97750E+00_JPRB, 0.10364E+01_JPRB, 0.11579E+01_JPRB, 0.13524E+01_JPRB, 0.14837E+01_JPRB, 0.16767E+01_JPRB ,&
&  0.17550E+01_JPRB, 0.18839E+01_JPRB, 0.17608E+01_JPRB, 0.17793E+01_JPRB, 0.17777E+01_JPRB, 0.16371E+01_JPRB, 0.16314E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.12920E+01_JPRB, 0.11843E+01_JPRB, 0.11023E+01_JPRB, 0.10609E+01_JPRB, 0.10416E+01_JPRB, 0.10715E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.11640E+01_JPRB, 0.11649E+01_JPRB, 0.11694E+01_JPRB, 0.12257E+01_JPRB, 0.14001E+01_JPRB, 0.15734E+01_JPRB ,&
&  0.20589E+01_JPRB, 0.21033E+01_JPRB, 0.18116E+01_JPRB, 0.26642E+01_JPRB, 0.36959E+01_JPRB, 0.34343E+01_JPRB, 0.30841E+01_JPRB ,&
&  0.34492E+01_JPRB, 0.37521E+01_JPRB, 0.40474E+01_JPRB, 0.41583E+01_JPRB, 0.42108E+01_JPRB, 0.41035E+01_JPRB, 0.37687E+01_JPRB ,&
&  0.33221E+01_JPRB, 0.30825E+01_JPRB, 0.30094E+01_JPRB, 0.30157E+01_JPRB, 0.27996E+01_JPRB, 0.26143E+01_JPRB, 0.25020E+01_JPRB ,&
&  0.21387E+01_JPRB, 0.17552E+01_JPRB, 0.11354E+01_JPRB, 0.12058E+01_JPRB, 0.93416E+00_JPRB, 0.96324E+00_JPRB, 0.11510E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.11956E+01_JPRB, 0.12955E+01_JPRB, 0.13782E+01_JPRB, 0.16938E+01_JPRB, 0.17444E+01_JPRB, 0.16925E+01_JPRB ,&
&  0.15379E+01_JPRB, 0.13434E+01_JPRB, 0.13963E+01_JPRB, 0.13550E+01_JPRB, 0.13454E+01_JPRB, 0.12622E+01_JPRB, 0.11596E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.95948E+00_JPRB, 0.89996E+00_JPRB, 0.83896E+00_JPRB, 0.78602E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.69695E+00_JPRB, 0.72692E+00_JPRB, 0.73659E+00_JPRB, 0.71294E+00_JPRB, 0.69030E+00_JPRB, 0.64887E+00_JPRB, 0.62816E+00_JPRB ,&
&  0.63232E+00_JPRB, 0.62318E+00_JPRB, 0.59912E+00_JPRB, 0.59983E+00_JPRB, 0.60456E+00_JPRB, 0.61085E+00_JPRB, 0.64451E+00_JPRB ,&
&  0.75532E+00_JPRB, 0.86671E+00_JPRB, 0.76593E+00_JPRB, 0.67632E+00_JPRB, 0.84451E+00_JPRB, 0.82500E+00_JPRB, 0.86901E+00_JPRB ,&
&  0.83328E+00_JPRB, 0.88987E+00_JPRB, 0.89034E+00_JPRB, 0.92230E+00_JPRB, 0.10324E+01_JPRB, 0.11757E+01_JPRB, 0.13073E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.14115E+01_JPRB, 0.13714E+01_JPRB, 0.13213E+01_JPRB, 0.12908E+01_JPRB, 0.11799E+01_JPRB, 0.11777E+01_JPRB ,&
&  0.11323E+01_JPRB, 0.10534E+01_JPRB, 0.98672E+00_JPRB, 0.97314E+00_JPRB, 0.96815E+00_JPRB, 0.97793E+00_JPRB, 0.10316E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11671E+01_JPRB, 0.11747E+01_JPRB, 0.11964E+01_JPRB, 0.12818E+01_JPRB, 0.14955E+01_JPRB, 0.19191E+01_JPRB ,&
&  0.24004E+01_JPRB, 0.25787E+01_JPRB, 0.27558E+01_JPRB, 0.33103E+01_JPRB, 0.35373E+01_JPRB, 0.34984E+01_JPRB, 0.35274E+01_JPRB ,&
&  0.36886E+01_JPRB, 0.38180E+01_JPRB, 0.40358E+01_JPRB, 0.41751E+01_JPRB, 0.42881E+01_JPRB, 0.43284E+01_JPRB, 0.39924E+01_JPRB ,&
&  0.34505E+01_JPRB, 0.31082E+01_JPRB, 0.29907E+01_JPRB, 0.27479E+01_JPRB, 0.25951E+01_JPRB, 0.23568E+01_JPRB, 0.24569E+01_JPRB ,&
&  0.25385E+01_JPRB, 0.13745E+01_JPRB, 0.10278E+01_JPRB, 0.12622E+01_JPRB, 0.10556E+01_JPRB, 0.11110E+01_JPRB, 0.13414E+01_JPRB ,&
&  0.13017E+01_JPRB, 0.12384E+01_JPRB, 0.13102E+01_JPRB, 0.13443E+01_JPRB, 0.14553E+01_JPRB, 0.16550E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12025E+01_JPRB, 0.12073E+01_JPRB, 0.11339E+01_JPRB, 0.11142E+01_JPRB, 0.10556E+01_JPRB, 0.98458E+00_JPRB ,&
&  0.89703E+00_JPRB, 0.83950E+00_JPRB, 0.78304E+00_JPRB, 0.74176E+00_JPRB, 0.71324E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.66883E+00_JPRB, 0.70173E+00_JPRB, 0.69980E+00_JPRB, 0.68043E+00_JPRB, 0.66624E+00_JPRB, 0.61955E+00_JPRB, 0.60426E+00_JPRB ,&
&  0.60929E+00_JPRB, 0.60399E+00_JPRB, 0.58452E+00_JPRB, 0.58486E+00_JPRB, 0.58931E+00_JPRB, 0.59094E+00_JPRB, 0.60129E+00_JPRB ,&
&  0.53242E+00_JPRB, 0.64709E+00_JPRB, 0.59727E+00_JPRB, 0.65108E+00_JPRB, 0.84682E+00_JPRB, 0.79808E+00_JPRB, 0.77770E+00_JPRB ,&
&  0.75839E+00_JPRB, 0.74873E+00_JPRB, 0.72271E+00_JPRB, 0.73828E+00_JPRB, 0.78774E+00_JPRB, 0.90698E+00_JPRB, 0.10305E+01_JPRB ,&
&  0.10249E+01_JPRB, 0.98887E+00_JPRB, 0.94628E+00_JPRB, 0.92021E+00_JPRB, 0.86590E+00_JPRB, 0.84496E+00_JPRB, 0.84790E+00_JPRB ,&
&  0.83567E+00_JPRB, 0.81123E+00_JPRB, 0.81080E+00_JPRB, 0.83583E+00_JPRB, 0.85622E+00_JPRB, 0.86203E+00_JPRB, 0.89463E+00_JPRB ,&
&  0.97510E+00_JPRB, 0.10621E+01_JPRB, 0.11127E+01_JPRB, 0.12061E+01_JPRB, 0.13332E+01_JPRB, 0.17079E+01_JPRB, 0.21578E+01_JPRB ,&
&  0.25822E+01_JPRB, 0.25812E+01_JPRB, 0.28133E+01_JPRB, 0.32678E+01_JPRB, 0.33909E+01_JPRB, 0.33952E+01_JPRB, 0.33902E+01_JPRB ,&
&  0.35173E+01_JPRB, 0.35202E+01_JPRB, 0.37489E+01_JPRB, 0.40593E+01_JPRB, 0.40328E+01_JPRB, 0.42462E+01_JPRB, 0.40189E+01_JPRB ,&
&  0.35619E+01_JPRB, 0.33182E+01_JPRB, 0.32195E+01_JPRB, 0.30656E+01_JPRB, 0.28697E+01_JPRB, 0.29045E+01_JPRB, 0.27808E+01_JPRB ,&
&  0.26829E+01_JPRB, 0.23563E+01_JPRB, 0.17438E+01_JPRB, 0.15957E+01_JPRB, 0.12033E+01_JPRB, 0.12480E+01_JPRB, 0.16221E+01_JPRB ,&
&  0.15710E+01_JPRB, 0.13954E+01_JPRB, 0.13060E+01_JPRB, 0.13027E+01_JPRB, 0.13729E+01_JPRB, 0.15209E+01_JPRB, 0.12912E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.10246E+01_JPRB, 0.99661E+00_JPRB, 0.94444E+00_JPRB, 0.91081E+00_JPRB, 0.89087E+00_JPRB, 0.83630E+00_JPRB ,&
&  0.77331E+00_JPRB, 0.73401E+00_JPRB, 0.71143E+00_JPRB, 0.68502E+00_JPRB, 0.67297E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.64999E+00_JPRB, 0.66870E+00_JPRB, 0.64985E+00_JPRB, 0.64356E+00_JPRB, 0.63940E+00_JPRB, 0.59937E+00_JPRB, 0.58382E+00_JPRB ,&
&  0.58134E+00_JPRB, 0.57775E+00_JPRB, 0.57114E+00_JPRB, 0.57147E+00_JPRB, 0.57298E+00_JPRB, 0.56890E+00_JPRB, 0.52521E+00_JPRB ,&
&  0.48275E+00_JPRB, 0.55567E+00_JPRB, 0.56849E+00_JPRB, 0.62419E+00_JPRB, 0.68668E+00_JPRB, 0.66278E+00_JPRB, 0.64301E+00_JPRB ,&
&  0.66563E+00_JPRB, 0.66119E+00_JPRB, 0.67125E+00_JPRB, 0.69977E+00_JPRB, 0.74025E+00_JPRB, 0.81289E+00_JPRB, 0.87414E+00_JPRB ,&
&  0.83581E+00_JPRB, 0.82676E+00_JPRB, 0.78915E+00_JPRB, 0.76759E+00_JPRB, 0.75775E+00_JPRB, 0.74050E+00_JPRB, 0.70481E+00_JPRB ,&
&  0.67808E+00_JPRB, 0.67731E+00_JPRB, 0.68445E+00_JPRB, 0.70813E+00_JPRB, 0.73111E+00_JPRB, 0.74347E+00_JPRB, 0.75219E+00_JPRB ,&
&  0.80208E+00_JPRB, 0.88369E+00_JPRB, 0.95885E+00_JPRB, 0.10657E+01_JPRB, 0.12134E+01_JPRB, 0.15891E+01_JPRB, 0.19146E+01_JPRB ,&
&  0.20333E+01_JPRB, 0.20827E+01_JPRB, 0.21571E+01_JPRB, 0.22454E+01_JPRB, 0.24787E+01_JPRB, 0.26710E+01_JPRB, 0.28400E+01_JPRB ,&
&  0.30933E+01_JPRB, 0.30755E+01_JPRB, 0.36709E+01_JPRB, 0.39031E+01_JPRB, 0.39496E+01_JPRB, 0.40427E+01_JPRB, 0.38498E+01_JPRB ,&
&  0.37218E+01_JPRB, 0.34022E+01_JPRB, 0.32960E+01_JPRB, 0.31415E+01_JPRB, 0.29788E+01_JPRB, 0.29113E+01_JPRB, 0.27119E+01_JPRB ,&
&  0.26119E+01_JPRB, 0.24925E+01_JPRB, 0.21610E+01_JPRB, 0.19300E+01_JPRB, 0.16451E+01_JPRB, 0.17534E+01_JPRB, 0.16532E+01_JPRB ,&
&  0.14761E+01_JPRB, 0.11889E+01_JPRB, 0.12132E+01_JPRB, 0.12339E+01_JPRB, 0.13026E+01_JPRB, 0.12480E+01_JPRB, 0.10149E+01_JPRB ,&
&  0.91659E+00_JPRB, 0.94295E+00_JPRB, 0.89148E+00_JPRB, 0.85123E+00_JPRB, 0.82316E+00_JPRB, 0.78795E+00_JPRB, 0.68691E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.69557E+00_JPRB, 0.69755E+00_JPRB, 0.67395E+00_JPRB, 0.65337E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.63296E+00_JPRB, 0.63896E+00_JPRB, 0.62151E+00_JPRB, 0.61774E+00_JPRB, 0.61136E+00_JPRB, 0.58041E+00_JPRB, 0.54578E+00_JPRB ,&
&  0.55372E+00_JPRB, 0.55156E+00_JPRB, 0.55277E+00_JPRB, 0.55525E+00_JPRB, 0.55855E+00_JPRB, 0.50935E+00_JPRB, 0.42407E+00_JPRB ,&
&  0.41361E+00_JPRB, 0.45655E+00_JPRB, 0.53123E+00_JPRB, 0.58566E+00_JPRB, 0.60249E+00_JPRB, 0.59552E+00_JPRB, 0.59631E+00_JPRB ,&
&  0.60533E+00_JPRB, 0.61442E+00_JPRB, 0.65581E+00_JPRB, 0.68705E+00_JPRB, 0.70113E+00_JPRB, 0.72509E+00_JPRB, 0.77000E+00_JPRB ,&
&  0.74512E+00_JPRB, 0.74965E+00_JPRB, 0.74286E+00_JPRB, 0.67316E+00_JPRB, 0.68195E+00_JPRB, 0.64951E+00_JPRB, 0.60796E+00_JPRB ,&
&  0.58866E+00_JPRB, 0.59908E+00_JPRB, 0.61755E+00_JPRB, 0.64512E+00_JPRB, 0.66128E+00_JPRB, 0.66451E+00_JPRB, 0.67027E+00_JPRB ,&
&  0.69565E+00_JPRB, 0.75308E+00_JPRB, 0.82693E+00_JPRB, 0.93838E+00_JPRB, 0.10856E+01_JPRB, 0.12983E+01_JPRB, 0.16107E+01_JPRB ,&
&  0.16702E+01_JPRB, 0.17659E+01_JPRB, 0.18711E+01_JPRB, 0.17684E+01_JPRB, 0.20159E+01_JPRB, 0.23853E+01_JPRB, 0.25184E+01_JPRB ,&
&  0.27712E+01_JPRB, 0.27509E+01_JPRB, 0.30311E+01_JPRB, 0.33398E+01_JPRB, 0.36158E+01_JPRB, 0.36362E+01_JPRB, 0.35500E+01_JPRB ,&
&  0.35606E+01_JPRB, 0.34030E+01_JPRB, 0.30869E+01_JPRB, 0.29265E+01_JPRB, 0.27576E+01_JPRB, 0.26764E+01_JPRB, 0.25088E+01_JPRB ,&
&  0.23967E+01_JPRB, 0.23284E+01_JPRB, 0.22419E+01_JPRB, 0.21279E+01_JPRB, 0.19912E+01_JPRB, 0.18418E+01_JPRB, 0.16849E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.13112E+01_JPRB, 0.12064E+01_JPRB, 0.12341E+01_JPRB, 0.12969E+01_JPRB, 0.11016E+01_JPRB, 0.94024E+00_JPRB ,&
&  0.86811E+00_JPRB, 0.82173E+00_JPRB, 0.86538E+00_JPRB, 0.81188E+00_JPRB, 0.76709E+00_JPRB, 0.72893E+00_JPRB, 0.68995E+00_JPRB ,&
&  0.65272E+00_JPRB, 0.71782E+00_JPRB, 0.71128E+00_JPRB, 0.68120E+00_JPRB, 0.64630E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.63794E+00_JPRB, 0.63322E+00_JPRB, 0.61644E+00_JPRB, 0.60599E+00_JPRB, 0.59343E+00_JPRB, 0.55157E+00_JPRB, 0.52865E+00_JPRB ,&
&  0.55398E+00_JPRB, 0.50816E+00_JPRB, 0.53596E+00_JPRB, 0.45855E+00_JPRB, 0.42766E+00_JPRB, 0.42432E+00_JPRB, 0.41558E+00_JPRB ,&
&  0.42854E+00_JPRB, 0.46957E+00_JPRB, 0.52031E+00_JPRB, 0.56429E+00_JPRB, 0.60799E+00_JPRB, 0.59455E+00_JPRB, 0.58156E+00_JPRB ,&
&  0.56342E+00_JPRB, 0.56980E+00_JPRB, 0.62441E+00_JPRB, 0.66606E+00_JPRB, 0.70048E+00_JPRB, 0.71820E+00_JPRB, 0.73234E+00_JPRB ,&
&  0.68442E+00_JPRB, 0.67244E+00_JPRB, 0.67846E+00_JPRB, 0.65275E+00_JPRB, 0.61841E+00_JPRB, 0.59370E+00_JPRB, 0.55916E+00_JPRB ,&
&  0.52851E+00_JPRB, 0.51574E+00_JPRB, 0.57490E+00_JPRB, 0.61008E+00_JPRB, 0.62498E+00_JPRB, 0.62436E+00_JPRB, 0.63668E+00_JPRB ,&
&  0.65987E+00_JPRB, 0.70112E+00_JPRB, 0.77402E+00_JPRB, 0.88160E+00_JPRB, 0.99087E+00_JPRB, 0.11349E+01_JPRB, 0.12838E+01_JPRB ,&
&  0.14766E+01_JPRB, 0.12358E+01_JPRB, 0.15069E+01_JPRB, 0.15760E+01_JPRB, 0.18555E+01_JPRB, 0.22486E+01_JPRB, 0.23983E+01_JPRB ,&
&  0.27491E+01_JPRB, 0.27439E+01_JPRB, 0.28735E+01_JPRB, 0.29840E+01_JPRB, 0.31198E+01_JPRB, 0.31897E+01_JPRB, 0.31738E+01_JPRB ,&
&  0.31769E+01_JPRB, 0.28135E+01_JPRB, 0.28484E+01_JPRB, 0.27068E+01_JPRB, 0.25695E+01_JPRB, 0.25091E+01_JPRB, 0.24007E+01_JPRB ,&
&  0.23133E+01_JPRB, 0.22487E+01_JPRB, 0.21574E+01_JPRB, 0.18841E+01_JPRB, 0.17983E+01_JPRB, 0.17226E+01_JPRB, 0.15503E+01_JPRB ,&
&  0.14889E+01_JPRB, 0.14201E+01_JPRB, 0.13405E+01_JPRB, 0.14607E+01_JPRB, 0.14029E+01_JPRB, 0.12313E+01_JPRB, 0.10753E+01_JPRB ,&
&  0.92811E+00_JPRB, 0.77537E+00_JPRB, 0.77199E+00_JPRB, 0.74534E+00_JPRB, 0.69916E+00_JPRB, 0.68022E+00_JPRB, 0.74599E+00_JPRB ,&
&  0.70980E+00_JPRB, 0.70585E+00_JPRB, 0.73117E+00_JPRB, 0.71009E+00_JPRB, 0.66937E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.66847E+00_JPRB, 0.64585E+00_JPRB, 0.61505E+00_JPRB, 0.59231E+00_JPRB, 0.57354E+00_JPRB, 0.54123E+00_JPRB, 0.49839E+00_JPRB ,&
&  0.49045E+00_JPRB, 0.44161E+00_JPRB, 0.41619E+00_JPRB, 0.40688E+00_JPRB, 0.41035E+00_JPRB, 0.41006E+00_JPRB, 0.40499E+00_JPRB ,&
&  0.42206E+00_JPRB, 0.46531E+00_JPRB, 0.53873E+00_JPRB, 0.58124E+00_JPRB, 0.59658E+00_JPRB, 0.59483E+00_JPRB, 0.57945E+00_JPRB ,&
&  0.56079E+00_JPRB, 0.56547E+00_JPRB, 0.60323E+00_JPRB, 0.67163E+00_JPRB, 0.70099E+00_JPRB, 0.69622E+00_JPRB, 0.68215E+00_JPRB ,&
&  0.66195E+00_JPRB, 0.64737E+00_JPRB, 0.63356E+00_JPRB, 0.62009E+00_JPRB, 0.58278E+00_JPRB, 0.54628E+00_JPRB, 0.48649E+00_JPRB ,&
&  0.41856E+00_JPRB, 0.42223E+00_JPRB, 0.53502E+00_JPRB, 0.57998E+00_JPRB, 0.60564E+00_JPRB, 0.61575E+00_JPRB, 0.63132E+00_JPRB ,&
&  0.66506E+00_JPRB, 0.71279E+00_JPRB, 0.77305E+00_JPRB, 0.84597E+00_JPRB, 0.91306E+00_JPRB, 0.98933E+00_JPRB, 0.10699E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.10232E+01_JPRB, 0.10253E+01_JPRB, 0.11650E+01_JPRB, 0.16996E+01_JPRB, 0.20616E+01_JPRB, 0.22716E+01_JPRB ,&
&  0.24593E+01_JPRB, 0.24477E+01_JPRB, 0.25607E+01_JPRB, 0.27540E+01_JPRB, 0.28799E+01_JPRB, 0.29081E+01_JPRB, 0.28322E+01_JPRB ,&
&  0.27094E+01_JPRB, 0.26133E+01_JPRB, 0.25060E+01_JPRB, 0.24782E+01_JPRB, 0.24052E+01_JPRB, 0.23887E+01_JPRB, 0.23351E+01_JPRB ,&
&  0.22721E+01_JPRB, 0.22191E+01_JPRB, 0.20635E+01_JPRB, 0.18857E+01_JPRB, 0.17367E+01_JPRB, 0.16381E+01_JPRB, 0.15603E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.14985E+01_JPRB, 0.15156E+01_JPRB, 0.15364E+01_JPRB, 0.14457E+01_JPRB, 0.12586E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.90330E+00_JPRB, 0.77510E+00_JPRB, 0.66423E+00_JPRB, 0.63826E+00_JPRB, 0.64284E+00_JPRB, 0.64507E+00_JPRB, 0.64991E+00_JPRB ,&
&  0.66619E+00_JPRB, 0.66833E+00_JPRB, 0.66997E+00_JPRB, 0.68931E+00_JPRB, 0.68135E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.68865E+00_JPRB, 0.67801E+00_JPRB, 0.63253E+00_JPRB, 0.59558E+00_JPRB, 0.55547E+00_JPRB, 0.52515E+00_JPRB, 0.49795E+00_JPRB ,&
&  0.49106E+00_JPRB, 0.46681E+00_JPRB, 0.44594E+00_JPRB, 0.42823E+00_JPRB, 0.42144E+00_JPRB, 0.42130E+00_JPRB, 0.44279E+00_JPRB ,&
&  0.48813E+00_JPRB, 0.54775E+00_JPRB, 0.59086E+00_JPRB, 0.59974E+00_JPRB, 0.60199E+00_JPRB, 0.59777E+00_JPRB, 0.59425E+00_JPRB ,&
&  0.58068E+00_JPRB, 0.57583E+00_JPRB, 0.58591E+00_JPRB, 0.61258E+00_JPRB, 0.64283E+00_JPRB, 0.65334E+00_JPRB, 0.64170E+00_JPRB ,&
&  0.62034E+00_JPRB, 0.60608E+00_JPRB, 0.60441E+00_JPRB, 0.59945E+00_JPRB, 0.55150E+00_JPRB, 0.48558E+00_JPRB, 0.41834E+00_JPRB ,&
&  0.37043E+00_JPRB, 0.36382E+00_JPRB, 0.40955E+00_JPRB, 0.53944E+00_JPRB, 0.57518E+00_JPRB, 0.60240E+00_JPRB, 0.62685E+00_JPRB ,&
&  0.61179E+00_JPRB, 0.62262E+00_JPRB, 0.68061E+00_JPRB, 0.79292E+00_JPRB, 0.82255E+00_JPRB, 0.87652E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.10030E+01_JPRB, 0.10224E+01_JPRB, 0.11091E+01_JPRB, 0.11155E+01_JPRB, 0.13505E+01_JPRB, 0.19569E+01_JPRB, 0.20040E+01_JPRB ,&
&  0.21381E+01_JPRB, 0.22398E+01_JPRB, 0.24762E+01_JPRB, 0.25770E+01_JPRB, 0.26123E+01_JPRB, 0.26486E+01_JPRB, 0.25786E+01_JPRB ,&
&  0.24176E+01_JPRB, 0.23346E+01_JPRB, 0.22759E+01_JPRB, 0.23377E+01_JPRB, 0.23616E+01_JPRB, 0.23564E+01_JPRB, 0.23227E+01_JPRB ,&
&  0.22871E+01_JPRB, 0.22104E+01_JPRB, 0.20434E+01_JPRB, 0.18196E+01_JPRB, 0.16962E+01_JPRB, 0.15860E+01_JPRB, 0.15126E+01_JPRB ,&
&  0.14605E+01_JPRB, 0.14728E+01_JPRB, 0.15116E+01_JPRB, 0.15275E+01_JPRB, 0.13608E+01_JPRB, 0.11139E+01_JPRB, 0.92420E+00_JPRB ,&
&  0.80566E+00_JPRB, 0.74371E+00_JPRB, 0.67116E+00_JPRB, 0.64759E+00_JPRB, 0.63716E+00_JPRB, 0.62727E+00_JPRB, 0.59686E+00_JPRB ,&
&  0.57057E+00_JPRB, 0.57271E+00_JPRB, 0.61473E+00_JPRB, 0.66407E+00_JPRB, 0.68278E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.64170E+00_JPRB, 0.67929E+00_JPRB, 0.68463E+00_JPRB, 0.64748E+00_JPRB, 0.61153E+00_JPRB, 0.55375E+00_JPRB, 0.51349E+00_JPRB ,&
&  0.49837E+00_JPRB, 0.49654E+00_JPRB, 0.50470E+00_JPRB, 0.51567E+00_JPRB, 0.53273E+00_JPRB, 0.55968E+00_JPRB, 0.59200E+00_JPRB ,&
&  0.61321E+00_JPRB, 0.62584E+00_JPRB, 0.64084E+00_JPRB, 0.65974E+00_JPRB, 0.66808E+00_JPRB, 0.66220E+00_JPRB, 0.64851E+00_JPRB ,&
&  0.62348E+00_JPRB, 0.60379E+00_JPRB, 0.58671E+00_JPRB, 0.57858E+00_JPRB, 0.58074E+00_JPRB, 0.59028E+00_JPRB, 0.60209E+00_JPRB ,&
&  0.60500E+00_JPRB, 0.59091E+00_JPRB, 0.58131E+00_JPRB, 0.56531E+00_JPRB, 0.50856E+00_JPRB, 0.45673E+00_JPRB, 0.39560E+00_JPRB ,&
&  0.35518E+00_JPRB, 0.33801E+00_JPRB, 0.33623E+00_JPRB, 0.36050E+00_JPRB, 0.42866E+00_JPRB, 0.57263E+00_JPRB, 0.61311E+00_JPRB ,&
&  0.65275E+00_JPRB, 0.68229E+00_JPRB, 0.70730E+00_JPRB, 0.73647E+00_JPRB, 0.76416E+00_JPRB, 0.80603E+00_JPRB, 0.85247E+00_JPRB ,&
&  0.91579E+00_JPRB, 0.97850E+00_JPRB, 0.10704E+01_JPRB, 0.10093E+01_JPRB, 0.10669E+01_JPRB, 0.13124E+01_JPRB, 0.15732E+01_JPRB ,&
&  0.17998E+01_JPRB, 0.18787E+01_JPRB, 0.19591E+01_JPRB, 0.21729E+01_JPRB, 0.22871E+01_JPRB, 0.24325E+01_JPRB, 0.24374E+01_JPRB ,&
&  0.23802E+01_JPRB, 0.23039E+01_JPRB, 0.22443E+01_JPRB, 0.22497E+01_JPRB, 0.23286E+01_JPRB, 0.24057E+01_JPRB, 0.23477E+01_JPRB ,&
&  0.22831E+01_JPRB, 0.21974E+01_JPRB, 0.19819E+01_JPRB, 0.17025E+01_JPRB, 0.15894E+01_JPRB, 0.15601E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.15515E+01_JPRB, 0.15482E+01_JPRB, 0.15481E+01_JPRB, 0.15481E+01_JPRB, 0.13366E+01_JPRB, 0.11312E+01_JPRB, 0.97236E+00_JPRB ,&
&  0.88884E+00_JPRB, 0.83939E+00_JPRB, 0.77713E+00_JPRB, 0.77383E+00_JPRB, 0.72864E+00_JPRB, 0.67694E+00_JPRB, 0.61589E+00_JPRB ,&
&  0.57983E+00_JPRB, 0.56439E+00_JPRB, 0.57849E+00_JPRB, 0.60458E+00_JPRB, 0.62431E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.69123E+00_JPRB, 0.70404E+00_JPRB, 0.71148E+00_JPRB, 0.70934E+00_JPRB, 0.69629E+00_JPRB, 0.67356E+00_JPRB, 0.66495E+00_JPRB ,&
&  0.66896E+00_JPRB, 0.68257E+00_JPRB, 0.69685E+00_JPRB, 0.69997E+00_JPRB, 0.70718E+00_JPRB, 0.71465E+00_JPRB, 0.72296E+00_JPRB ,&
&  0.73062E+00_JPRB, 0.74175E+00_JPRB, 0.75469E+00_JPRB, 0.74413E+00_JPRB, 0.72723E+00_JPRB, 0.71081E+00_JPRB, 0.69424E+00_JPRB ,&
&  0.66900E+00_JPRB, 0.63942E+00_JPRB, 0.60787E+00_JPRB, 0.58993E+00_JPRB, 0.57123E+00_JPRB, 0.56308E+00_JPRB, 0.55452E+00_JPRB ,&
&  0.54537E+00_JPRB, 0.54473E+00_JPRB, 0.55183E+00_JPRB, 0.51323E+00_JPRB, 0.46000E+00_JPRB, 0.41863E+00_JPRB, 0.37040E+00_JPRB ,&
&  0.33650E+00_JPRB, 0.31540E+00_JPRB, 0.30250E+00_JPRB, 0.30073E+00_JPRB, 0.31396E+00_JPRB, 0.34978E+00_JPRB, 0.43729E+00_JPRB ,&
&  0.63578E+00_JPRB, 0.68979E+00_JPRB, 0.71682E+00_JPRB, 0.74681E+00_JPRB, 0.78051E+00_JPRB, 0.81551E+00_JPRB, 0.85530E+00_JPRB ,&
&  0.90054E+00_JPRB, 0.94062E+00_JPRB, 0.10051E+01_JPRB, 0.10652E+01_JPRB, 0.11539E+01_JPRB, 0.12032E+01_JPRB, 0.13851E+01_JPRB ,&
&  0.17257E+01_JPRB, 0.18091E+01_JPRB, 0.17004E+01_JPRB, 0.17555E+01_JPRB, 0.18745E+01_JPRB, 0.19903E+01_JPRB, 0.20829E+01_JPRB ,&
&  0.21423E+01_JPRB, 0.21596E+01_JPRB, 0.21821E+01_JPRB, 0.22068E+01_JPRB, 0.22675E+01_JPRB, 0.23131E+01_JPRB, 0.23146E+01_JPRB ,&
&  0.22707E+01_JPRB, 0.22503E+01_JPRB, 0.22597E+01_JPRB, 0.18756E+01_JPRB, 0.17175E+01_JPRB, 0.16843E+01_JPRB, 0.16165E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.15575E+01_JPRB, 0.15750E+01_JPRB, 0.15718E+01_JPRB, 0.14344E+01_JPRB, 0.12720E+01_JPRB, 0.11108E+01_JPRB ,&
&  0.10208E+01_JPRB, 0.94114E+00_JPRB, 0.85473E+00_JPRB, 0.81217E+00_JPRB, 0.76355E+00_JPRB, 0.70546E+00_JPRB, 0.65015E+00_JPRB ,&
&  0.62904E+00_JPRB, 0.62324E+00_JPRB, 0.62905E+00_JPRB, 0.66355E+00_JPRB, 0.68292E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.69658E+00_JPRB, 0.72259E+00_JPRB, 0.74307E+00_JPRB, 0.75564E+00_JPRB, 0.75924E+00_JPRB, 0.76515E+00_JPRB, 0.77006E+00_JPRB ,&
&  0.77651E+00_JPRB, 0.78241E+00_JPRB, 0.78703E+00_JPRB, 0.78588E+00_JPRB, 0.78570E+00_JPRB, 0.78444E+00_JPRB, 0.78700E+00_JPRB ,&
&  0.78963E+00_JPRB, 0.78379E+00_JPRB, 0.78015E+00_JPRB, 0.77804E+00_JPRB, 0.77012E+00_JPRB, 0.75034E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.67991E+00_JPRB, 0.63798E+00_JPRB, 0.60321E+00_JPRB, 0.57168E+00_JPRB, 0.54642E+00_JPRB, 0.52766E+00_JPRB, 0.52379E+00_JPRB ,&
&  0.53699E+00_JPRB, 0.52522E+00_JPRB, 0.50927E+00_JPRB, 0.46531E+00_JPRB, 0.42142E+00_JPRB, 0.39640E+00_JPRB, 0.34845E+00_JPRB ,&
&  0.31477E+00_JPRB, 0.29501E+00_JPRB, 0.28863E+00_JPRB, 0.28842E+00_JPRB, 0.29806E+00_JPRB, 0.32390E+00_JPRB, 0.37855E+00_JPRB ,&
&  0.47960E+00_JPRB, 0.66528E+00_JPRB, 0.71051E+00_JPRB, 0.75127E+00_JPRB, 0.79713E+00_JPRB, 0.84151E+00_JPRB, 0.87607E+00_JPRB ,&
&  0.91129E+00_JPRB, 0.94886E+00_JPRB, 0.10237E+01_JPRB, 0.10992E+01_JPRB, 0.12015E+01_JPRB, 0.12905E+01_JPRB, 0.13687E+01_JPRB ,&
&  0.14434E+01_JPRB, 0.14344E+01_JPRB, 0.14832E+01_JPRB, 0.15222E+01_JPRB, 0.16207E+01_JPRB, 0.17111E+01_JPRB, 0.16883E+01_JPRB ,&
&  0.17012E+01_JPRB, 0.18011E+01_JPRB, 0.18284E+01_JPRB, 0.18902E+01_JPRB, 0.20193E+01_JPRB, 0.21663E+01_JPRB, 0.22207E+01_JPRB ,&
&  0.21496E+01_JPRB, 0.20991E+01_JPRB, 0.21388E+01_JPRB, 0.20203E+01_JPRB, 0.19351E+01_JPRB, 0.18307E+01_JPRB, 0.17097E+01_JPRB ,&
&  0.16346E+01_JPRB, 0.15867E+01_JPRB, 0.15569E+01_JPRB, 0.15252E+01_JPRB, 0.14587E+01_JPRB, 0.13782E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.11185E+01_JPRB, 0.10022E+01_JPRB, 0.91053E+00_JPRB, 0.85262E+00_JPRB, 0.80289E+00_JPRB, 0.75040E+00_JPRB, 0.71113E+00_JPRB ,&
&  0.67831E+00_JPRB, 0.66315E+00_JPRB, 0.65800E+00_JPRB, 0.66839E+00_JPRB, 0.68065E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.75993E+00_JPRB, 0.77580E+00_JPRB, 0.79152E+00_JPRB, 0.80211E+00_JPRB, 0.81004E+00_JPRB, 0.81645E+00_JPRB, 0.81753E+00_JPRB ,&
&  0.81961E+00_JPRB, 0.81981E+00_JPRB, 0.82316E+00_JPRB, 0.82547E+00_JPRB, 0.83229E+00_JPRB, 0.83796E+00_JPRB, 0.84642E+00_JPRB ,&
&  0.85865E+00_JPRB, 0.86744E+00_JPRB, 0.86577E+00_JPRB, 0.84746E+00_JPRB, 0.81606E+00_JPRB, 0.77919E+00_JPRB, 0.72921E+00_JPRB ,&
&  0.68228E+00_JPRB, 0.64913E+00_JPRB, 0.60425E+00_JPRB, 0.56338E+00_JPRB, 0.53979E+00_JPRB, 0.52293E+00_JPRB, 0.51729E+00_JPRB ,&
&  0.51027E+00_JPRB, 0.49134E+00_JPRB, 0.46718E+00_JPRB, 0.43208E+00_JPRB, 0.40287E+00_JPRB, 0.33949E+00_JPRB, 0.32538E+00_JPRB ,&
&  0.30839E+00_JPRB, 0.29719E+00_JPRB, 0.29358E+00_JPRB, 0.29550E+00_JPRB, 0.30216E+00_JPRB, 0.31980E+00_JPRB, 0.34793E+00_JPRB ,&
&  0.39678E+00_JPRB, 0.61127E+00_JPRB, 0.67456E+00_JPRB, 0.69812E+00_JPRB, 0.74037E+00_JPRB, 0.78252E+00_JPRB, 0.81986E+00_JPRB ,&
&  0.86080E+00_JPRB, 0.90287E+00_JPRB, 0.96647E+00_JPRB, 0.10280E+01_JPRB, 0.10921E+01_JPRB, 0.11481E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.12757E+01_JPRB, 0.13482E+01_JPRB, 0.13751E+01_JPRB, 0.13716E+01_JPRB, 0.14420E+01_JPRB, 0.14860E+01_JPRB ,&
&  0.14915E+01_JPRB, 0.14840E+01_JPRB, 0.15316E+01_JPRB, 0.16380E+01_JPRB, 0.17818E+01_JPRB, 0.19008E+01_JPRB, 0.19371E+01_JPRB ,&
&  0.19488E+01_JPRB, 0.19162E+01_JPRB, 0.18593E+01_JPRB, 0.18198E+01_JPRB, 0.17434E+01_JPRB, 0.16596E+01_JPRB, 0.16063E+01_JPRB ,&
&  0.15522E+01_JPRB, 0.15400E+01_JPRB, 0.14986E+01_JPRB, 0.14541E+01_JPRB, 0.13982E+01_JPRB, 0.13417E+01_JPRB, 0.12701E+01_JPRB ,&
&  0.11791E+01_JPRB, 0.10802E+01_JPRB, 0.99958E+00_JPRB, 0.93040E+00_JPRB, 0.88072E+00_JPRB, 0.82940E+00_JPRB, 0.80142E+00_JPRB ,&
&  0.77093E+00_JPRB, 0.75828E+00_JPRB, 0.74238E+00_JPRB, 0.74213E+00_JPRB, 0.74384E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.85840E+00_JPRB, 0.86200E+00_JPRB, 0.86791E+00_JPRB, 0.87293E+00_JPRB, 0.87833E+00_JPRB, 0.88239E+00_JPRB, 0.88368E+00_JPRB ,&
&  0.88396E+00_JPRB, 0.88408E+00_JPRB, 0.88646E+00_JPRB, 0.89066E+00_JPRB, 0.89609E+00_JPRB, 0.90356E+00_JPRB, 0.91055E+00_JPRB ,&
&  0.91978E+00_JPRB, 0.91549E+00_JPRB, 0.89215E+00_JPRB, 0.86350E+00_JPRB, 0.83105E+00_JPRB, 0.78618E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.68449E+00_JPRB, 0.64755E+00_JPRB, 0.59356E+00_JPRB, 0.54801E+00_JPRB, 0.51194E+00_JPRB, 0.49346E+00_JPRB, 0.47169E+00_JPRB ,&
&  0.47156E+00_JPRB, 0.41685E+00_JPRB, 0.39796E+00_JPRB, 0.36139E+00_JPRB, 0.33723E+00_JPRB, 0.31940E+00_JPRB, 0.30938E+00_JPRB ,&
&  0.30576E+00_JPRB, 0.30330E+00_JPRB, 0.30459E+00_JPRB, 0.31090E+00_JPRB, 0.31988E+00_JPRB, 0.33909E+00_JPRB, 0.36230E+00_JPRB ,&
&  0.42520E+00_JPRB, 0.61419E+00_JPRB, 0.68926E+00_JPRB, 0.71155E+00_JPRB, 0.72077E+00_JPRB, 0.75060E+00_JPRB, 0.78531E+00_JPRB ,&
&  0.83777E+00_JPRB, 0.88278E+00_JPRB, 0.92922E+00_JPRB, 0.95755E+00_JPRB, 0.10093E+01_JPRB, 0.10567E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.11456E+01_JPRB, 0.11869E+01_JPRB, 0.12298E+01_JPRB, 0.12614E+01_JPRB, 0.12703E+01_JPRB, 0.12868E+01_JPRB, 0.12993E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13444E+01_JPRB, 0.13990E+01_JPRB, 0.14717E+01_JPRB, 0.15490E+01_JPRB, 0.16046E+01_JPRB, 0.16346E+01_JPRB ,&
&  0.16644E+01_JPRB, 0.16793E+01_JPRB, 0.16954E+01_JPRB, 0.17012E+01_JPRB, 0.16724E+01_JPRB, 0.16230E+01_JPRB, 0.15924E+01_JPRB ,&
&  0.15613E+01_JPRB, 0.15015E+01_JPRB, 0.14424E+01_JPRB, 0.13889E+01_JPRB, 0.13412E+01_JPRB, 0.12976E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.11658E+01_JPRB, 0.11153E+01_JPRB, 0.10550E+01_JPRB, 0.10000E+01_JPRB, 0.94777E+00_JPRB, 0.91515E+00_JPRB ,&
&  0.88491E+00_JPRB, 0.87267E+00_JPRB, 0.85837E+00_JPRB, 0.85670E+00_JPRB, 0.85287E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.96632E+00_JPRB, 0.96174E+00_JPRB, 0.96055E+00_JPRB, 0.95860E+00_JPRB, 0.95991E+00_JPRB, 0.96044E+00_JPRB, 0.96212E+00_JPRB ,&
&  0.96205E+00_JPRB, 0.96324E+00_JPRB, 0.96282E+00_JPRB, 0.96286E+00_JPRB, 0.96074E+00_JPRB, 0.95996E+00_JPRB, 0.95827E+00_JPRB ,&
&  0.95364E+00_JPRB, 0.93913E+00_JPRB, 0.91023E+00_JPRB, 0.87870E+00_JPRB, 0.84402E+00_JPRB, 0.80364E+00_JPRB, 0.75926E+00_JPRB ,&
&  0.70431E+00_JPRB, 0.65385E+00_JPRB, 0.58653E+00_JPRB, 0.54252E+00_JPRB, 0.50183E+00_JPRB, 0.47213E+00_JPRB, 0.45384E+00_JPRB ,&
&  0.43705E+00_JPRB, 0.42142E+00_JPRB, 0.40597E+00_JPRB, 0.38832E+00_JPRB, 0.37370E+00_JPRB, 0.35605E+00_JPRB, 0.34567E+00_JPRB ,&
&  0.33810E+00_JPRB, 0.33566E+00_JPRB, 0.33629E+00_JPRB, 0.34294E+00_JPRB, 0.35425E+00_JPRB, 0.37237E+00_JPRB, 0.40183E+00_JPRB ,&
&  0.46768E+00_JPRB, 0.55903E+00_JPRB, 0.70429E+00_JPRB, 0.73858E+00_JPRB, 0.73674E+00_JPRB, 0.75818E+00_JPRB, 0.78364E+00_JPRB ,&
&  0.80715E+00_JPRB, 0.83744E+00_JPRB, 0.86154E+00_JPRB, 0.87117E+00_JPRB, 0.88274E+00_JPRB, 0.89297E+00_JPRB, 0.90569E+00_JPRB ,&
&  0.92083E+00_JPRB, 0.95213E+00_JPRB, 0.10014E+01_JPRB, 0.10608E+01_JPRB, 0.11285E+01_JPRB, 0.11925E+01_JPRB, 0.12386E+01_JPRB ,&
&  0.12686E+01_JPRB, 0.12922E+01_JPRB, 0.13297E+01_JPRB, 0.13713E+01_JPRB, 0.14153E+01_JPRB, 0.14457E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.14964E+01_JPRB, 0.15076E+01_JPRB, 0.15001E+01_JPRB, 0.14818E+01_JPRB, 0.14674E+01_JPRB, 0.14702E+01_JPRB, 0.14599E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.14262E+01_JPRB, 0.13907E+01_JPRB, 0.13472E+01_JPRB, 0.13097E+01_JPRB, 0.12750E+01_JPRB, 0.12491E+01_JPRB ,&
&  0.12247E+01_JPRB, 0.12014E+01_JPRB, 0.11745E+01_JPRB, 0.11408E+01_JPRB, 0.11073E+01_JPRB, 0.10733E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10217E+01_JPRB, 0.10049E+01_JPRB, 0.98986E+00_JPRB, 0.98060E+00_JPRB, 0.97132E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.11051E+01_JPRB, 0.10995E+01_JPRB, 0.10943E+01_JPRB, 0.10889E+01_JPRB, 0.10810E+01_JPRB, 0.10720E+01_JPRB, 0.10617E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.10342E+01_JPRB, 0.10186E+01_JPRB, 0.10024E+01_JPRB, 0.98729E+00_JPRB, 0.97509E+00_JPRB, 0.96460E+00_JPRB ,&
&  0.95578E+00_JPRB, 0.94228E+00_JPRB, 0.91803E+00_JPRB, 0.88839E+00_JPRB, 0.86211E+00_JPRB, 0.83925E+00_JPRB, 0.81741E+00_JPRB ,&
&  0.77918E+00_JPRB, 0.73758E+00_JPRB, 0.69821E+00_JPRB, 0.65564E+00_JPRB, 0.62295E+00_JPRB, 0.55653E+00_JPRB, 0.53139E+00_JPRB ,&
&  0.50694E+00_JPRB, 0.49330E+00_JPRB, 0.48726E+00_JPRB, 0.48444E+00_JPRB, 0.49558E+00_JPRB, 0.48520E+00_JPRB, 0.47790E+00_JPRB ,&
&  0.45289E+00_JPRB, 0.45143E+00_JPRB, 0.45688E+00_JPRB, 0.46616E+00_JPRB, 0.48425E+00_JPRB, 0.50908E+00_JPRB, 0.54373E+00_JPRB ,&
&  0.60286E+00_JPRB, 0.63195E+00_JPRB, 0.67502E+00_JPRB, 0.70871E+00_JPRB, 0.73002E+00_JPRB, 0.74215E+00_JPRB, 0.75676E+00_JPRB ,&
&  0.77179E+00_JPRB, 0.79539E+00_JPRB, 0.80945E+00_JPRB, 0.81934E+00_JPRB, 0.81963E+00_JPRB, 0.81748E+00_JPRB, 0.81886E+00_JPRB ,&
&  0.81887E+00_JPRB, 0.82893E+00_JPRB, 0.84632E+00_JPRB, 0.87923E+00_JPRB, 0.92486E+00_JPRB, 0.97863E+00_JPRB, 0.10310E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.11249E+01_JPRB, 0.11639E+01_JPRB, 0.12017E+01_JPRB, 0.12422E+01_JPRB, 0.12787E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.13557E+01_JPRB, 0.13625E+01_JPRB, 0.13644E+01_JPRB, 0.13579E+01_JPRB, 0.13507E+01_JPRB, 0.13466E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13365E+01_JPRB, 0.13337E+01_JPRB, 0.13232E+01_JPRB, 0.13072E+01_JPRB, 0.12891E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.12519E+01_JPRB, 0.12335E+01_JPRB, 0.12162E+01_JPRB, 0.12000E+01_JPRB, 0.11864E+01_JPRB, 0.11702E+01_JPRB, 0.11567E+01_JPRB ,&
&  0.11430E+01_JPRB, 0.11359E+01_JPRB, 0.11267E+01_JPRB, 0.11190E+01_JPRB, 0.11110E+01_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.10834E+01_JPRB, 0.10729E+01_JPRB, 0.10621E+01_JPRB, 0.10521E+01_JPRB, 0.10399E+01_JPRB, 0.10275E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.98878E+00_JPRB, 0.97497E+00_JPRB, 0.96056E+00_JPRB, 0.94633E+00_JPRB, 0.93520E+00_JPRB, 0.92751E+00_JPRB ,&
&  0.92053E+00_JPRB, 0.91126E+00_JPRB, 0.90014E+00_JPRB, 0.88538E+00_JPRB, 0.86975E+00_JPRB, 0.85468E+00_JPRB, 0.84118E+00_JPRB ,&
&  0.82458E+00_JPRB, 0.80707E+00_JPRB, 0.78900E+00_JPRB, 0.77163E+00_JPRB, 0.75400E+00_JPRB, 0.73636E+00_JPRB, 0.71887E+00_JPRB ,&
&  0.70328E+00_JPRB, 0.69163E+00_JPRB, 0.67994E+00_JPRB, 0.66718E+00_JPRB, 0.65281E+00_JPRB, 0.64755E+00_JPRB, 0.64209E+00_JPRB ,&
&  0.63712E+00_JPRB, 0.63586E+00_JPRB, 0.64206E+00_JPRB, 0.64753E+00_JPRB, 0.65341E+00_JPRB, 0.66034E+00_JPRB, 0.67276E+00_JPRB ,&
&  0.68430E+00_JPRB, 0.69785E+00_JPRB, 0.71558E+00_JPRB, 0.73434E+00_JPRB, 0.75007E+00_JPRB, 0.76200E+00_JPRB, 0.77855E+00_JPRB ,&
&  0.79070E+00_JPRB, 0.80628E+00_JPRB, 0.81778E+00_JPRB, 0.82371E+00_JPRB, 0.82451E+00_JPRB, 0.82532E+00_JPRB, 0.82787E+00_JPRB ,&
&  0.82731E+00_JPRB, 0.83451E+00_JPRB, 0.84126E+00_JPRB, 0.84841E+00_JPRB, 0.85081E+00_JPRB, 0.85974E+00_JPRB, 0.86545E+00_JPRB ,&
&  0.87745E+00_JPRB, 0.89528E+00_JPRB, 0.91485E+00_JPRB, 0.93740E+00_JPRB, 0.96506E+00_JPRB, 0.99965E+00_JPRB, 0.10287E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10830E+01_JPRB, 0.11081E+01_JPRB, 0.11248E+01_JPRB, 0.11398E+01_JPRB, 0.11535E+01_JPRB, 0.11665E+01_JPRB ,&
&  0.11720E+01_JPRB, 0.11769E+01_JPRB, 0.11809E+01_JPRB, 0.11840E+01_JPRB, 0.11812E+01_JPRB, 0.11768E+01_JPRB, 0.11723E+01_JPRB ,&
&  0.11677E+01_JPRB, 0.11615E+01_JPRB, 0.11564E+01_JPRB, 0.11530E+01_JPRB, 0.11485E+01_JPRB, 0.11422E+01_JPRB, 0.11357E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.11215E+01_JPRB, 0.11123E+01_JPRB, 0.11029E+01_JPRB, 0.10940E+01_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.94414E+00_JPRB, 0.93928E+00_JPRB, 0.93381E+00_JPRB, 0.92880E+00_JPRB, 0.92261E+00_JPRB, 0.91700E+00_JPRB, 0.91130E+00_JPRB ,&
&  0.90619E+00_JPRB, 0.90120E+00_JPRB, 0.89646E+00_JPRB, 0.89114E+00_JPRB, 0.88598E+00_JPRB, 0.88096E+00_JPRB, 0.87601E+00_JPRB ,&
&  0.87090E+00_JPRB, 0.86622E+00_JPRB, 0.86117E+00_JPRB, 0.85542E+00_JPRB, 0.85077E+00_JPRB, 0.84551E+00_JPRB, 0.83790E+00_JPRB ,&
&  0.82673E+00_JPRB, 0.81496E+00_JPRB, 0.80385E+00_JPRB, 0.79502E+00_JPRB, 0.78862E+00_JPRB, 0.78350E+00_JPRB, 0.77908E+00_JPRB ,&
&  0.77631E+00_JPRB, 0.77448E+00_JPRB, 0.77285E+00_JPRB, 0.76999E+00_JPRB, 0.76781E+00_JPRB, 0.76845E+00_JPRB, 0.76855E+00_JPRB ,&
&  0.76764E+00_JPRB, 0.76738E+00_JPRB, 0.76875E+00_JPRB, 0.77009E+00_JPRB, 0.77084E+00_JPRB, 0.77325E+00_JPRB, 0.77789E+00_JPRB ,&
&  0.78272E+00_JPRB, 0.78691E+00_JPRB, 0.79266E+00_JPRB, 0.79856E+00_JPRB, 0.80446E+00_JPRB, 0.80981E+00_JPRB, 0.81772E+00_JPRB ,&
&  0.82601E+00_JPRB, 0.83356E+00_JPRB, 0.84051E+00_JPRB, 0.84568E+00_JPRB, 0.84969E+00_JPRB, 0.85145E+00_JPRB, 0.85200E+00_JPRB ,&
&  0.85108E+00_JPRB, 0.85221E+00_JPRB, 0.85182E+00_JPRB, 0.85141E+00_JPRB, 0.84968E+00_JPRB, 0.85090E+00_JPRB, 0.85174E+00_JPRB ,&
&  0.85487E+00_JPRB, 0.85688E+00_JPRB, 0.86107E+00_JPRB, 0.86522E+00_JPRB, 0.87162E+00_JPRB, 0.87850E+00_JPRB, 0.88588E+00_JPRB ,&
&  0.89420E+00_JPRB, 0.90411E+00_JPRB, 0.91631E+00_JPRB, 0.92444E+00_JPRB, 0.93226E+00_JPRB, 0.93964E+00_JPRB, 0.94761E+00_JPRB ,&
&  0.95404E+00_JPRB, 0.95981E+00_JPRB, 0.96536E+00_JPRB, 0.96942E+00_JPRB, 0.97179E+00_JPRB, 0.97380E+00_JPRB, 0.97571E+00_JPRB ,&
&  0.97636E+00_JPRB, 0.97646E+00_JPRB, 0.97632E+00_JPRB, 0.97626E+00_JPRB, 0.97433E+00_JPRB, 0.97141E+00_JPRB, 0.96813E+00_JPRB ,&
&  0.96531E+00_JPRB, 0.96138E+00_JPRB, 0.95747E+00_JPRB, 0.95318E+00_JPRB, 0.94904E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB ,&
&  0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB, 0.85269E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB, 0.25651E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.27130E+00_JPRB, 0.27147E+00_JPRB, 0.27159E+00_JPRB, 0.27169E+00_JPRB, 0.27055E+00_JPRB, 0.27106E+00_JPRB, 0.27150E+00_JPRB ,&
&  0.27221E+00_JPRB, 0.27353E+00_JPRB, 0.27507E+00_JPRB, 0.27624E+00_JPRB, 0.27711E+00_JPRB, 0.27766E+00_JPRB, 0.27805E+00_JPRB ,&
&  0.27812E+00_JPRB, 0.27835E+00_JPRB, 0.27518E+00_JPRB, 0.27405E+00_JPRB, 0.27369E+00_JPRB, 0.27373E+00_JPRB, 0.27393E+00_JPRB ,&
&  0.27407E+00_JPRB, 0.27408E+00_JPRB, 0.27361E+00_JPRB, 0.27390E+00_JPRB, 0.27421E+00_JPRB, 0.27442E+00_JPRB, 0.27467E+00_JPRB ,&
&  0.27490E+00_JPRB, 0.27508E+00_JPRB, 0.27501E+00_JPRB, 0.27479E+00_JPRB, 0.27464E+00_JPRB, 0.27412E+00_JPRB, 0.27326E+00_JPRB ,&
&  0.27220E+00_JPRB, 0.27089E+00_JPRB, 0.26959E+00_JPRB, 0.26839E+00_JPRB, 0.26735E+00_JPRB, 0.26646E+00_JPRB, 0.26561E+00_JPRB ,&
&  0.26460E+00_JPRB, 0.26328E+00_JPRB, 0.26188E+00_JPRB, 0.26053E+00_JPRB, 0.25923E+00_JPRB, 0.25805E+00_JPRB, 0.25681E+00_JPRB ,&
&  0.25586E+00_JPRB, 0.25505E+00_JPRB, 0.25424E+00_JPRB, 0.25355E+00_JPRB, 0.25288E+00_JPRB, 0.25224E+00_JPRB, 0.25162E+00_JPRB ,&
&  0.25079E+00_JPRB, 0.25032E+00_JPRB, 0.24981E+00_JPRB, 0.24956E+00_JPRB, 0.24960E+00_JPRB, 0.24961E+00_JPRB, 0.24971E+00_JPRB ,&
&  0.24987E+00_JPRB, 0.25009E+00_JPRB, 0.25021E+00_JPRB, 0.25041E+00_JPRB, 0.25072E+00_JPRB, 0.25112E+00_JPRB, 0.25162E+00_JPRB ,&
&  0.25216E+00_JPRB, 0.25275E+00_JPRB, 0.25339E+00_JPRB, 0.25412E+00_JPRB, 0.25488E+00_JPRB, 0.25571E+00_JPRB, 0.25651E+00_JPRB ,&
&  0.25731E+00_JPRB, 0.25805E+00_JPRB, 0.25880E+00_JPRB, 0.25937E+00_JPRB, 0.26020E+00_JPRB, 0.26099E+00_JPRB, 0.26194E+00_JPRB ,&
&  0.26290E+00_JPRB, 0.26392E+00_JPRB, 0.26481E+00_JPRB, 0.26577E+00_JPRB, 0.26681E+00_JPRB, 0.26666E+00_JPRB, 0.26657E+00_JPRB ,&
&  0.26656E+00_JPRB, 0.26685E+00_JPRB, 0.26686E+00_JPRB, 0.26721E+00_JPRB, 0.26805E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.29925E+00_JPRB, 0.30577E+00_JPRB, 0.31049E+00_JPRB, 0.31529E+00_JPRB, 0.31092E+00_JPRB, 0.34799E+00_JPRB, 0.34639E+00_JPRB ,&
&  0.34704E+00_JPRB, 0.33374E+00_JPRB, 0.33431E+00_JPRB, 0.33297E+00_JPRB, 0.33008E+00_JPRB, 0.32584E+00_JPRB, 0.32188E+00_JPRB ,&
&  0.31839E+00_JPRB, 0.31370E+00_JPRB, 0.30822E+00_JPRB, 0.30355E+00_JPRB, 0.29982E+00_JPRB, 0.29664E+00_JPRB, 0.29390E+00_JPRB ,&
&  0.29327E+00_JPRB, 0.29451E+00_JPRB, 0.29691E+00_JPRB, 0.29581E+00_JPRB, 0.29684E+00_JPRB, 0.29877E+00_JPRB, 0.30106E+00_JPRB ,&
&  0.30321E+00_JPRB, 0.30505E+00_JPRB, 0.30510E+00_JPRB, 0.30341E+00_JPRB, 0.30222E+00_JPRB, 0.29987E+00_JPRB, 0.29769E+00_JPRB ,&
&  0.29505E+00_JPRB, 0.29214E+00_JPRB, 0.28872E+00_JPRB, 0.28598E+00_JPRB, 0.28300E+00_JPRB, 0.27936E+00_JPRB, 0.27585E+00_JPRB ,&
&  0.27285E+00_JPRB, 0.27015E+00_JPRB, 0.26722E+00_JPRB, 0.26450E+00_JPRB, 0.26190E+00_JPRB, 0.25926E+00_JPRB, 0.25693E+00_JPRB ,&
&  0.25517E+00_JPRB, 0.25368E+00_JPRB, 0.25219E+00_JPRB, 0.25062E+00_JPRB, 0.24909E+00_JPRB, 0.24762E+00_JPRB, 0.24641E+00_JPRB ,&
&  0.24621E+00_JPRB, 0.24552E+00_JPRB, 0.24483E+00_JPRB, 0.24445E+00_JPRB, 0.24448E+00_JPRB, 0.24398E+00_JPRB, 0.24377E+00_JPRB ,&
&  0.24370E+00_JPRB, 0.24372E+00_JPRB, 0.24396E+00_JPRB, 0.24426E+00_JPRB, 0.24468E+00_JPRB, 0.24523E+00_JPRB, 0.24612E+00_JPRB ,&
&  0.24718E+00_JPRB, 0.24846E+00_JPRB, 0.24983E+00_JPRB, 0.25124E+00_JPRB, 0.25268E+00_JPRB, 0.25420E+00_JPRB, 0.25590E+00_JPRB ,&
&  0.25762E+00_JPRB, 0.25929E+00_JPRB, 0.26092E+00_JPRB, 0.26253E+00_JPRB, 0.26515E+00_JPRB, 0.26767E+00_JPRB, 0.27037E+00_JPRB ,&
&  0.27327E+00_JPRB, 0.27671E+00_JPRB, 0.28032E+00_JPRB, 0.28420E+00_JPRB, 0.28855E+00_JPRB, 0.29014E+00_JPRB, 0.29140E+00_JPRB ,&
&  0.29224E+00_JPRB, 0.29517E+00_JPRB, 0.29725E+00_JPRB, 0.29708E+00_JPRB, 0.29680E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.35691E+00_JPRB, 0.35506E+00_JPRB, 0.35562E+00_JPRB, 0.35665E+00_JPRB, 0.36030E+00_JPRB, 0.36561E+00_JPRB, 0.37137E+00_JPRB ,&
&  0.37095E+00_JPRB, 0.37410E+00_JPRB, 0.37325E+00_JPRB, 0.37061E+00_JPRB, 0.36621E+00_JPRB, 0.36199E+00_JPRB, 0.35716E+00_JPRB ,&
&  0.35033E+00_JPRB, 0.34161E+00_JPRB, 0.33311E+00_JPRB, 0.32445E+00_JPRB, 0.31623E+00_JPRB, 0.30771E+00_JPRB, 0.30264E+00_JPRB ,&
&  0.29935E+00_JPRB, 0.29848E+00_JPRB, 0.30047E+00_JPRB, 0.30323E+00_JPRB, 0.30858E+00_JPRB, 0.31590E+00_JPRB, 0.32561E+00_JPRB ,&
&  0.33511E+00_JPRB, 0.34579E+00_JPRB, 0.35135E+00_JPRB, 0.35123E+00_JPRB, 0.35096E+00_JPRB, 0.34886E+00_JPRB, 0.34536E+00_JPRB ,&
&  0.33882E+00_JPRB, 0.33140E+00_JPRB, 0.32283E+00_JPRB, 0.31377E+00_JPRB, 0.30454E+00_JPRB, 0.29581E+00_JPRB, 0.28770E+00_JPRB ,&
&  0.28112E+00_JPRB, 0.27552E+00_JPRB, 0.27040E+00_JPRB, 0.26611E+00_JPRB, 0.26279E+00_JPRB, 0.26032E+00_JPRB, 0.25732E+00_JPRB ,&
&  0.25481E+00_JPRB, 0.25260E+00_JPRB, 0.24997E+00_JPRB, 0.24816E+00_JPRB, 0.24653E+00_JPRB, 0.24547E+00_JPRB, 0.24459E+00_JPRB ,&
&  0.24450E+00_JPRB, 0.24441E+00_JPRB, 0.24500E+00_JPRB, 0.24559E+00_JPRB, 0.24628E+00_JPRB, 0.24621E+00_JPRB, 0.24597E+00_JPRB ,&
&  0.24553E+00_JPRB, 0.24512E+00_JPRB, 0.24376E+00_JPRB, 0.24271E+00_JPRB, 0.24198E+00_JPRB, 0.24118E+00_JPRB, 0.24115E+00_JPRB ,&
&  0.24198E+00_JPRB, 0.24285E+00_JPRB, 0.24356E+00_JPRB, 0.24516E+00_JPRB, 0.24676E+00_JPRB, 0.24908E+00_JPRB, 0.25188E+00_JPRB ,&
&  0.25393E+00_JPRB, 0.25635E+00_JPRB, 0.25904E+00_JPRB, 0.26217E+00_JPRB, 0.26591E+00_JPRB, 0.26980E+00_JPRB, 0.27436E+00_JPRB ,&
&  0.27950E+00_JPRB, 0.28479E+00_JPRB, 0.29076E+00_JPRB, 0.29734E+00_JPRB, 0.30536E+00_JPRB, 0.31217E+00_JPRB, 0.32223E+00_JPRB ,&
&  0.33019E+00_JPRB, 0.32931E+00_JPRB, 0.36956E+00_JPRB, 0.36322E+00_JPRB, 0.35906E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.35553E+00_JPRB, 0.35837E+00_JPRB, 0.36165E+00_JPRB, 0.36566E+00_JPRB, 0.37061E+00_JPRB, 0.37505E+00_JPRB, 0.37989E+00_JPRB ,&
&  0.38278E+00_JPRB, 0.37958E+00_JPRB, 0.38026E+00_JPRB, 0.37596E+00_JPRB, 0.37156E+00_JPRB, 0.36586E+00_JPRB, 0.35977E+00_JPRB ,&
&  0.35339E+00_JPRB, 0.34439E+00_JPRB, 0.33477E+00_JPRB, 0.32625E+00_JPRB, 0.31905E+00_JPRB, 0.31342E+00_JPRB, 0.30892E+00_JPRB ,&
&  0.30581E+00_JPRB, 0.30617E+00_JPRB, 0.30828E+00_JPRB, 0.31431E+00_JPRB, 0.32441E+00_JPRB, 0.33696E+00_JPRB, 0.35599E+00_JPRB ,&
&  0.38287E+00_JPRB, 0.38501E+00_JPRB, 0.39008E+00_JPRB, 0.38913E+00_JPRB, 0.38492E+00_JPRB, 0.38288E+00_JPRB, 0.37891E+00_JPRB ,&
&  0.37944E+00_JPRB, 0.37193E+00_JPRB, 0.38171E+00_JPRB, 0.36526E+00_JPRB, 0.33844E+00_JPRB, 0.32307E+00_JPRB, 0.30476E+00_JPRB ,&
&  0.29136E+00_JPRB, 0.27997E+00_JPRB, 0.27144E+00_JPRB, 0.26782E+00_JPRB, 0.26403E+00_JPRB, 0.26224E+00_JPRB, 0.25855E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25281E+00_JPRB, 0.24969E+00_JPRB, 0.24944E+00_JPRB, 0.24953E+00_JPRB, 0.24896E+00_JPRB, 0.24905E+00_JPRB ,&
&  0.24945E+00_JPRB, 0.25048E+00_JPRB, 0.25233E+00_JPRB, 0.25402E+00_JPRB, 0.25502E+00_JPRB, 0.25561E+00_JPRB, 0.25545E+00_JPRB ,&
&  0.25589E+00_JPRB, 0.25557E+00_JPRB, 0.25473E+00_JPRB, 0.25341E+00_JPRB, 0.25143E+00_JPRB, 0.24936E+00_JPRB, 0.24737E+00_JPRB ,&
&  0.24578E+00_JPRB, 0.24518E+00_JPRB, 0.24467E+00_JPRB, 0.24489E+00_JPRB, 0.24548E+00_JPRB, 0.24724E+00_JPRB, 0.24889E+00_JPRB ,&
&  0.25079E+00_JPRB, 0.25344E+00_JPRB, 0.25577E+00_JPRB, 0.25863E+00_JPRB, 0.26124E+00_JPRB, 0.26279E+00_JPRB, 0.26685E+00_JPRB ,&
&  0.27208E+00_JPRB, 0.27915E+00_JPRB, 0.28744E+00_JPRB, 0.29543E+00_JPRB, 0.30561E+00_JPRB, 0.32125E+00_JPRB, 0.34244E+00_JPRB ,&
&  0.36408E+00_JPRB, 0.38743E+00_JPRB, 0.38151E+00_JPRB, 0.35382E+00_JPRB, 0.35246E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.36683E+00_JPRB, 0.37365E+00_JPRB, 0.38069E+00_JPRB, 0.38823E+00_JPRB, 0.39766E+00_JPRB, 0.40170E+00_JPRB, 0.40366E+00_JPRB ,&
&  0.39958E+00_JPRB, 0.39499E+00_JPRB, 0.38417E+00_JPRB, 0.37429E+00_JPRB, 0.36486E+00_JPRB, 0.35737E+00_JPRB, 0.35160E+00_JPRB ,&
&  0.34825E+00_JPRB, 0.34564E+00_JPRB, 0.34165E+00_JPRB, 0.33828E+00_JPRB, 0.33535E+00_JPRB, 0.33537E+00_JPRB, 0.33561E+00_JPRB ,&
&  0.33495E+00_JPRB, 0.33518E+00_JPRB, 0.33388E+00_JPRB, 0.33906E+00_JPRB, 0.34822E+00_JPRB, 0.35941E+00_JPRB, 0.37403E+00_JPRB ,&
&  0.39447E+00_JPRB, 0.40561E+00_JPRB, 0.42112E+00_JPRB, 0.42150E+00_JPRB, 0.41964E+00_JPRB, 0.41582E+00_JPRB, 0.41270E+00_JPRB ,&
&  0.40802E+00_JPRB, 0.40130E+00_JPRB, 0.39635E+00_JPRB, 0.39016E+00_JPRB, 0.36081E+00_JPRB, 0.33781E+00_JPRB, 0.32013E+00_JPRB ,&
&  0.30488E+00_JPRB, 0.29047E+00_JPRB, 0.27783E+00_JPRB, 0.27145E+00_JPRB, 0.26633E+00_JPRB, 0.26331E+00_JPRB, 0.26014E+00_JPRB ,&
&  0.25676E+00_JPRB, 0.25554E+00_JPRB, 0.25364E+00_JPRB, 0.25295E+00_JPRB, 0.25399E+00_JPRB, 0.25394E+00_JPRB, 0.25472E+00_JPRB ,&
&  0.25541E+00_JPRB, 0.25647E+00_JPRB, 0.25895E+00_JPRB, 0.26083E+00_JPRB, 0.26278E+00_JPRB, 0.26462E+00_JPRB, 0.26553E+00_JPRB ,&
&  0.26735E+00_JPRB, 0.26890E+00_JPRB, 0.26886E+00_JPRB, 0.26813E+00_JPRB, 0.26636E+00_JPRB, 0.26240E+00_JPRB, 0.25845E+00_JPRB ,&
&  0.25420E+00_JPRB, 0.25113E+00_JPRB, 0.24992E+00_JPRB, 0.24892E+00_JPRB, 0.24976E+00_JPRB, 0.25056E+00_JPRB, 0.25146E+00_JPRB ,&
&  0.25335E+00_JPRB, 0.25512E+00_JPRB, 0.25717E+00_JPRB, 0.26011E+00_JPRB, 0.26235E+00_JPRB, 0.26430E+00_JPRB, 0.26677E+00_JPRB ,&
&  0.26708E+00_JPRB, 0.27137E+00_JPRB, 0.27677E+00_JPRB, 0.28378E+00_JPRB, 0.29599E+00_JPRB, 0.30607E+00_JPRB, 0.32885E+00_JPRB ,&
&  0.36457E+00_JPRB, 0.43181E+00_JPRB, 0.42552E+00_JPRB, 0.35746E+00_JPRB, 0.36076E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.39017E+00_JPRB, 0.39892E+00_JPRB, 0.40661E+00_JPRB, 0.41255E+00_JPRB, 0.41628E+00_JPRB, 0.41699E+00_JPRB, 0.41969E+00_JPRB ,&
&  0.42716E+00_JPRB, 0.43154E+00_JPRB, 0.43477E+00_JPRB, 0.43403E+00_JPRB, 0.43348E+00_JPRB, 0.41488E+00_JPRB, 0.41024E+00_JPRB ,&
&  0.40427E+00_JPRB, 0.40010E+00_JPRB, 0.39429E+00_JPRB, 0.39418E+00_JPRB, 0.39394E+00_JPRB, 0.39595E+00_JPRB, 0.39671E+00_JPRB ,&
&  0.39692E+00_JPRB, 0.39510E+00_JPRB, 0.39199E+00_JPRB, 0.39008E+00_JPRB, 0.39134E+00_JPRB, 0.39272E+00_JPRB, 0.38976E+00_JPRB ,&
&  0.38545E+00_JPRB, 0.38447E+00_JPRB, 0.38767E+00_JPRB, 0.39484E+00_JPRB, 0.43511E+00_JPRB, 0.42980E+00_JPRB, 0.42748E+00_JPRB ,&
&  0.42416E+00_JPRB, 0.41996E+00_JPRB, 0.41934E+00_JPRB, 0.41276E+00_JPRB, 0.40317E+00_JPRB, 0.38174E+00_JPRB, 0.36979E+00_JPRB ,&
&  0.33535E+00_JPRB, 0.31632E+00_JPRB, 0.29960E+00_JPRB, 0.28871E+00_JPRB, 0.27848E+00_JPRB, 0.27255E+00_JPRB, 0.26724E+00_JPRB ,&
&  0.26319E+00_JPRB, 0.26000E+00_JPRB, 0.25715E+00_JPRB, 0.25766E+00_JPRB, 0.25943E+00_JPRB, 0.26039E+00_JPRB, 0.26137E+00_JPRB ,&
&  0.26192E+00_JPRB, 0.26241E+00_JPRB, 0.26303E+00_JPRB, 0.26435E+00_JPRB, 0.26613E+00_JPRB, 0.26847E+00_JPRB, 0.27147E+00_JPRB ,&
&  0.27634E+00_JPRB, 0.28017E+00_JPRB, 0.28168E+00_JPRB, 0.28433E+00_JPRB, 0.28404E+00_JPRB, 0.27839E+00_JPRB, 0.27187E+00_JPRB ,&
&  0.26507E+00_JPRB, 0.26153E+00_JPRB, 0.25805E+00_JPRB, 0.25562E+00_JPRB, 0.25503E+00_JPRB, 0.25501E+00_JPRB, 0.25694E+00_JPRB ,&
&  0.25906E+00_JPRB, 0.26095E+00_JPRB, 0.26226E+00_JPRB, 0.26369E+00_JPRB, 0.26554E+00_JPRB, 0.26696E+00_JPRB, 0.26882E+00_JPRB ,&
&  0.27120E+00_JPRB, 0.27444E+00_JPRB, 0.27925E+00_JPRB, 0.28483E+00_JPRB, 0.29190E+00_JPRB, 0.30245E+00_JPRB, 0.32103E+00_JPRB ,&
&  0.34568E+00_JPRB, 0.40147E+00_JPRB, 0.40682E+00_JPRB, 0.37689E+00_JPRB, 0.38306E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.40275E+00_JPRB, 0.41311E+00_JPRB, 0.41845E+00_JPRB, 0.42374E+00_JPRB, 0.42590E+00_JPRB, 0.42725E+00_JPRB, 0.43512E+00_JPRB ,&
&  0.43991E+00_JPRB, 0.44369E+00_JPRB, 0.44603E+00_JPRB, 0.44705E+00_JPRB, 0.44290E+00_JPRB, 0.43633E+00_JPRB, 0.43428E+00_JPRB ,&
&  0.43312E+00_JPRB, 0.43361E+00_JPRB, 0.43256E+00_JPRB, 0.43255E+00_JPRB, 0.43305E+00_JPRB, 0.43783E+00_JPRB, 0.44467E+00_JPRB ,&
&  0.44630E+00_JPRB, 0.45080E+00_JPRB, 0.46353E+00_JPRB, 0.47128E+00_JPRB, 0.47617E+00_JPRB, 0.47572E+00_JPRB, 0.46993E+00_JPRB ,&
&  0.44228E+00_JPRB, 0.41427E+00_JPRB, 0.38709E+00_JPRB, 0.38857E+00_JPRB, 0.44146E+00_JPRB, 0.44766E+00_JPRB, 0.44701E+00_JPRB ,&
&  0.44546E+00_JPRB, 0.44974E+00_JPRB, 0.44990E+00_JPRB, 0.43529E+00_JPRB, 0.42463E+00_JPRB, 0.40994E+00_JPRB, 0.39724E+00_JPRB ,&
&  0.38630E+00_JPRB, 0.37858E+00_JPRB, 0.36531E+00_JPRB, 0.33793E+00_JPRB, 0.32053E+00_JPRB, 0.31164E+00_JPRB, 0.29794E+00_JPRB ,&
&  0.28759E+00_JPRB, 0.28361E+00_JPRB, 0.28264E+00_JPRB, 0.28201E+00_JPRB, 0.28603E+00_JPRB, 0.28800E+00_JPRB, 0.28666E+00_JPRB ,&
&  0.28328E+00_JPRB, 0.28046E+00_JPRB, 0.27671E+00_JPRB, 0.27151E+00_JPRB, 0.26873E+00_JPRB, 0.26786E+00_JPRB, 0.26833E+00_JPRB ,&
&  0.27223E+00_JPRB, 0.27829E+00_JPRB, 0.29172E+00_JPRB, 0.30122E+00_JPRB, 0.29757E+00_JPRB, 0.28627E+00_JPRB, 0.27597E+00_JPRB ,&
&  0.26927E+00_JPRB, 0.26654E+00_JPRB, 0.26450E+00_JPRB, 0.26239E+00_JPRB, 0.26300E+00_JPRB, 0.26505E+00_JPRB, 0.26651E+00_JPRB ,&
&  0.26680E+00_JPRB, 0.26851E+00_JPRB, 0.26951E+00_JPRB, 0.27072E+00_JPRB, 0.27175E+00_JPRB, 0.27346E+00_JPRB, 0.27581E+00_JPRB ,&
&  0.27807E+00_JPRB, 0.28334E+00_JPRB, 0.28920E+00_JPRB, 0.29580E+00_JPRB, 0.29995E+00_JPRB, 0.30306E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.31556E+00_JPRB, 0.33131E+00_JPRB, 0.36234E+00_JPRB, 0.39915E+00_JPRB, 0.39336E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.41456E+00_JPRB, 0.42164E+00_JPRB, 0.43003E+00_JPRB, 0.44095E+00_JPRB, 0.44325E+00_JPRB, 0.44478E+00_JPRB, 0.45385E+00_JPRB ,&
&  0.45420E+00_JPRB, 0.45121E+00_JPRB, 0.44797E+00_JPRB, 0.44823E+00_JPRB, 0.45035E+00_JPRB, 0.45276E+00_JPRB, 0.45230E+00_JPRB ,&
&  0.44313E+00_JPRB, 0.43786E+00_JPRB, 0.44058E+00_JPRB, 0.44356E+00_JPRB, 0.44813E+00_JPRB, 0.45463E+00_JPRB, 0.46032E+00_JPRB ,&
&  0.46341E+00_JPRB, 0.46705E+00_JPRB, 0.46958E+00_JPRB, 0.47583E+00_JPRB, 0.48545E+00_JPRB, 0.49211E+00_JPRB, 0.49837E+00_JPRB ,&
&  0.48404E+00_JPRB, 0.43635E+00_JPRB, 0.40596E+00_JPRB, 0.41226E+00_JPRB, 0.46138E+00_JPRB, 0.47170E+00_JPRB, 0.47254E+00_JPRB ,&
&  0.47552E+00_JPRB, 0.48307E+00_JPRB, 0.48253E+00_JPRB, 0.47668E+00_JPRB, 0.46483E+00_JPRB, 0.44580E+00_JPRB, 0.42256E+00_JPRB ,&
&  0.40700E+00_JPRB, 0.39669E+00_JPRB, 0.38989E+00_JPRB, 0.38316E+00_JPRB, 0.37965E+00_JPRB, 0.37474E+00_JPRB, 0.36712E+00_JPRB ,&
&  0.36410E+00_JPRB, 0.36277E+00_JPRB, 0.36383E+00_JPRB, 0.36539E+00_JPRB, 0.36729E+00_JPRB, 0.36856E+00_JPRB, 0.36536E+00_JPRB ,&
&  0.35315E+00_JPRB, 0.33750E+00_JPRB, 0.32255E+00_JPRB, 0.30628E+00_JPRB, 0.29243E+00_JPRB, 0.28373E+00_JPRB, 0.27675E+00_JPRB ,&
&  0.27399E+00_JPRB, 0.27975E+00_JPRB, 0.29680E+00_JPRB, 0.31635E+00_JPRB, 0.34792E+00_JPRB, 0.32329E+00_JPRB, 0.29984E+00_JPRB ,&
&  0.28852E+00_JPRB, 0.28305E+00_JPRB, 0.28002E+00_JPRB, 0.28008E+00_JPRB, 0.28173E+00_JPRB, 0.28221E+00_JPRB, 0.28231E+00_JPRB ,&
&  0.28362E+00_JPRB, 0.28604E+00_JPRB, 0.28791E+00_JPRB, 0.28951E+00_JPRB, 0.29200E+00_JPRB, 0.29488E+00_JPRB, 0.29641E+00_JPRB ,&
&  0.29718E+00_JPRB, 0.30195E+00_JPRB, 0.31159E+00_JPRB, 0.32274E+00_JPRB, 0.33386E+00_JPRB, 0.34544E+00_JPRB, 0.36303E+00_JPRB ,&
&  0.38492E+00_JPRB, 0.41120E+00_JPRB, 0.41077E+00_JPRB, 0.41104E+00_JPRB, 0.41141E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.43798E+00_JPRB, 0.44952E+00_JPRB, 0.46295E+00_JPRB, 0.47224E+00_JPRB, 0.47116E+00_JPRB, 0.47103E+00_JPRB, 0.47713E+00_JPRB ,&
&  0.47100E+00_JPRB, 0.46376E+00_JPRB, 0.45781E+00_JPRB, 0.45170E+00_JPRB, 0.44958E+00_JPRB, 0.44830E+00_JPRB, 0.45133E+00_JPRB ,&
&  0.45355E+00_JPRB, 0.45251E+00_JPRB, 0.45088E+00_JPRB, 0.45162E+00_JPRB, 0.45819E+00_JPRB, 0.46646E+00_JPRB, 0.47340E+00_JPRB ,&
&  0.47476E+00_JPRB, 0.47834E+00_JPRB, 0.48410E+00_JPRB, 0.48899E+00_JPRB, 0.49146E+00_JPRB, 0.49297E+00_JPRB, 0.49177E+00_JPRB ,&
&  0.48923E+00_JPRB, 0.48698E+00_JPRB, 0.47001E+00_JPRB, 0.48350E+00_JPRB, 0.48796E+00_JPRB, 0.48878E+00_JPRB, 0.49416E+00_JPRB ,&
&  0.50192E+00_JPRB, 0.51074E+00_JPRB, 0.51483E+00_JPRB, 0.51613E+00_JPRB, 0.51144E+00_JPRB, 0.50209E+00_JPRB, 0.47777E+00_JPRB ,&
&  0.45733E+00_JPRB, 0.43610E+00_JPRB, 0.42283E+00_JPRB, 0.41473E+00_JPRB, 0.40835E+00_JPRB, 0.40286E+00_JPRB, 0.39768E+00_JPRB ,&
&  0.39053E+00_JPRB, 0.38413E+00_JPRB, 0.38022E+00_JPRB, 0.38022E+00_JPRB, 0.38434E+00_JPRB, 0.39520E+00_JPRB, 0.40859E+00_JPRB ,&
&  0.41446E+00_JPRB, 0.40685E+00_JPRB, 0.39871E+00_JPRB, 0.38989E+00_JPRB, 0.37553E+00_JPRB, 0.34811E+00_JPRB, 0.33139E+00_JPRB ,&
&  0.32736E+00_JPRB, 0.33992E+00_JPRB, 0.36100E+00_JPRB, 0.36693E+00_JPRB, 0.37224E+00_JPRB, 0.36995E+00_JPRB, 0.36462E+00_JPRB ,&
&  0.35578E+00_JPRB, 0.33256E+00_JPRB, 0.32016E+00_JPRB, 0.31660E+00_JPRB, 0.31445E+00_JPRB, 0.31483E+00_JPRB, 0.31748E+00_JPRB ,&
&  0.32176E+00_JPRB, 0.32901E+00_JPRB, 0.33406E+00_JPRB, 0.33606E+00_JPRB, 0.33905E+00_JPRB, 0.33918E+00_JPRB, 0.33807E+00_JPRB ,&
&  0.33448E+00_JPRB, 0.34118E+00_JPRB, 0.35959E+00_JPRB, 0.38799E+00_JPRB, 0.39442E+00_JPRB, 0.40211E+00_JPRB, 0.41108E+00_JPRB ,&
&  0.41652E+00_JPRB, 0.42177E+00_JPRB, 0.42237E+00_JPRB, 0.42339E+00_JPRB, 0.42883E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.45769E+00_JPRB, 0.47068E+00_JPRB, 0.48697E+00_JPRB, 0.49844E+00_JPRB, 0.50328E+00_JPRB, 0.50860E+00_JPRB, 0.51094E+00_JPRB ,&
&  0.50578E+00_JPRB, 0.50365E+00_JPRB, 0.49805E+00_JPRB, 0.48995E+00_JPRB, 0.48296E+00_JPRB, 0.47947E+00_JPRB, 0.47986E+00_JPRB ,&
&  0.48205E+00_JPRB, 0.48393E+00_JPRB, 0.48051E+00_JPRB, 0.47875E+00_JPRB, 0.48263E+00_JPRB, 0.48794E+00_JPRB, 0.49141E+00_JPRB ,&
&  0.49283E+00_JPRB, 0.49432E+00_JPRB, 0.49912E+00_JPRB, 0.50218E+00_JPRB, 0.50232E+00_JPRB, 0.49576E+00_JPRB, 0.49086E+00_JPRB ,&
&  0.48586E+00_JPRB, 0.48412E+00_JPRB, 0.49061E+00_JPRB, 0.48263E+00_JPRB, 0.49734E+00_JPRB, 0.49924E+00_JPRB, 0.50471E+00_JPRB ,&
&  0.51285E+00_JPRB, 0.52647E+00_JPRB, 0.53703E+00_JPRB, 0.54517E+00_JPRB, 0.54768E+00_JPRB, 0.55088E+00_JPRB, 0.54106E+00_JPRB ,&
&  0.52732E+00_JPRB, 0.50868E+00_JPRB, 0.48996E+00_JPRB, 0.46964E+00_JPRB, 0.45005E+00_JPRB, 0.43484E+00_JPRB, 0.42743E+00_JPRB ,&
&  0.41845E+00_JPRB, 0.40457E+00_JPRB, 0.39124E+00_JPRB, 0.38604E+00_JPRB, 0.39068E+00_JPRB, 0.40086E+00_JPRB, 0.41638E+00_JPRB ,&
&  0.42583E+00_JPRB, 0.42845E+00_JPRB, 0.42524E+00_JPRB, 0.42099E+00_JPRB, 0.41658E+00_JPRB, 0.40937E+00_JPRB, 0.40516E+00_JPRB ,&
&  0.40173E+00_JPRB, 0.39607E+00_JPRB, 0.39318E+00_JPRB, 0.39490E+00_JPRB, 0.39802E+00_JPRB, 0.40002E+00_JPRB, 0.39854E+00_JPRB ,&
&  0.39563E+00_JPRB, 0.39116E+00_JPRB, 0.38314E+00_JPRB, 0.37877E+00_JPRB, 0.37913E+00_JPRB, 0.38365E+00_JPRB, 0.39129E+00_JPRB ,&
&  0.39377E+00_JPRB, 0.39976E+00_JPRB, 0.40554E+00_JPRB, 0.40760E+00_JPRB, 0.41011E+00_JPRB, 0.41212E+00_JPRB, 0.41220E+00_JPRB ,&
&  0.41041E+00_JPRB, 0.40534E+00_JPRB, 0.39835E+00_JPRB, 0.39538E+00_JPRB, 0.39648E+00_JPRB, 0.40327E+00_JPRB, 0.41251E+00_JPRB ,&
&  0.42168E+00_JPRB, 0.43571E+00_JPRB, 0.44653E+00_JPRB, 0.44905E+00_JPRB, 0.45142E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.48756E+00_JPRB, 0.49294E+00_JPRB, 0.50617E+00_JPRB, 0.51707E+00_JPRB, 0.52050E+00_JPRB, 0.52991E+00_JPRB, 0.54092E+00_JPRB ,&
&  0.54995E+00_JPRB, 0.55170E+00_JPRB, 0.54786E+00_JPRB, 0.53750E+00_JPRB, 0.52535E+00_JPRB, 0.51535E+00_JPRB, 0.51137E+00_JPRB ,&
&  0.51359E+00_JPRB, 0.51670E+00_JPRB, 0.52228E+00_JPRB, 0.52536E+00_JPRB, 0.52906E+00_JPRB, 0.52786E+00_JPRB, 0.52829E+00_JPRB ,&
&  0.53010E+00_JPRB, 0.53114E+00_JPRB, 0.53168E+00_JPRB, 0.52803E+00_JPRB, 0.52512E+00_JPRB, 0.52258E+00_JPRB, 0.51397E+00_JPRB ,&
&  0.50437E+00_JPRB, 0.49796E+00_JPRB, 0.50181E+00_JPRB, 0.51313E+00_JPRB, 0.52036E+00_JPRB, 0.51748E+00_JPRB, 0.51612E+00_JPRB ,&
&  0.51634E+00_JPRB, 0.52138E+00_JPRB, 0.52942E+00_JPRB, 0.54141E+00_JPRB, 0.54976E+00_JPRB, 0.56074E+00_JPRB, 0.56134E+00_JPRB ,&
&  0.55488E+00_JPRB, 0.54556E+00_JPRB, 0.54256E+00_JPRB, 0.54136E+00_JPRB, 0.51238E+00_JPRB, 0.48056E+00_JPRB, 0.46495E+00_JPRB ,&
&  0.45659E+00_JPRB, 0.44620E+00_JPRB, 0.43042E+00_JPRB, 0.41935E+00_JPRB, 0.41820E+00_JPRB, 0.42305E+00_JPRB, 0.43450E+00_JPRB ,&
&  0.44277E+00_JPRB, 0.45508E+00_JPRB, 0.47007E+00_JPRB, 0.47977E+00_JPRB, 0.47991E+00_JPRB, 0.46778E+00_JPRB, 0.45659E+00_JPRB ,&
&  0.44997E+00_JPRB, 0.44810E+00_JPRB, 0.44658E+00_JPRB, 0.44453E+00_JPRB, 0.44259E+00_JPRB, 0.44204E+00_JPRB, 0.44097E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.42935E+00_JPRB, 0.42042E+00_JPRB, 0.41546E+00_JPRB, 0.41453E+00_JPRB, 0.41459E+00_JPRB, 0.41630E+00_JPRB ,&
&  0.42528E+00_JPRB, 0.43651E+00_JPRB, 0.44553E+00_JPRB, 0.44953E+00_JPRB, 0.44645E+00_JPRB, 0.43845E+00_JPRB, 0.43358E+00_JPRB ,&
&  0.42986E+00_JPRB, 0.42721E+00_JPRB, 0.42377E+00_JPRB, 0.42210E+00_JPRB, 0.42176E+00_JPRB, 0.42897E+00_JPRB, 0.44464E+00_JPRB ,&
&  0.46413E+00_JPRB, 0.48977E+00_JPRB, 0.50152E+00_JPRB, 0.50028E+00_JPRB, 0.49346E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.54385E+00_JPRB, 0.52697E+00_JPRB, 0.52726E+00_JPRB, 0.53439E+00_JPRB, 0.53841E+00_JPRB, 0.54607E+00_JPRB, 0.56532E+00_JPRB ,&
&  0.57657E+00_JPRB, 0.58379E+00_JPRB, 0.58652E+00_JPRB, 0.58097E+00_JPRB, 0.57984E+00_JPRB, 0.57257E+00_JPRB, 0.55979E+00_JPRB ,&
&  0.55449E+00_JPRB, 0.55347E+00_JPRB, 0.55515E+00_JPRB, 0.55898E+00_JPRB, 0.56124E+00_JPRB, 0.56408E+00_JPRB, 0.56730E+00_JPRB ,&
&  0.56904E+00_JPRB, 0.56901E+00_JPRB, 0.56814E+00_JPRB, 0.56839E+00_JPRB, 0.56746E+00_JPRB, 0.55986E+00_JPRB, 0.55122E+00_JPRB ,&
&  0.54185E+00_JPRB, 0.53269E+00_JPRB, 0.53000E+00_JPRB, 0.53426E+00_JPRB, 0.53922E+00_JPRB, 0.54235E+00_JPRB, 0.54017E+00_JPRB ,&
&  0.53566E+00_JPRB, 0.53329E+00_JPRB, 0.53584E+00_JPRB, 0.54283E+00_JPRB, 0.55140E+00_JPRB, 0.56100E+00_JPRB, 0.56605E+00_JPRB ,&
&  0.56035E+00_JPRB, 0.54263E+00_JPRB, 0.53179E+00_JPRB, 0.53121E+00_JPRB, 0.52750E+00_JPRB, 0.51286E+00_JPRB, 0.50264E+00_JPRB ,&
&  0.49784E+00_JPRB, 0.48902E+00_JPRB, 0.47969E+00_JPRB, 0.46701E+00_JPRB, 0.45540E+00_JPRB, 0.45107E+00_JPRB, 0.44973E+00_JPRB ,&
&  0.45263E+00_JPRB, 0.47045E+00_JPRB, 0.50350E+00_JPRB, 0.53586E+00_JPRB, 0.55016E+00_JPRB, 0.54386E+00_JPRB, 0.52718E+00_JPRB ,&
&  0.51956E+00_JPRB, 0.51635E+00_JPRB, 0.51773E+00_JPRB, 0.51745E+00_JPRB, 0.51406E+00_JPRB, 0.50528E+00_JPRB, 0.49809E+00_JPRB ,&
&  0.48780E+00_JPRB, 0.47646E+00_JPRB, 0.47188E+00_JPRB, 0.46914E+00_JPRB, 0.46620E+00_JPRB, 0.46630E+00_JPRB, 0.46542E+00_JPRB ,&
&  0.46592E+00_JPRB, 0.46973E+00_JPRB, 0.48123E+00_JPRB, 0.48761E+00_JPRB, 0.48855E+00_JPRB, 0.47829E+00_JPRB, 0.47926E+00_JPRB ,&
&  0.48201E+00_JPRB, 0.47980E+00_JPRB, 0.47529E+00_JPRB, 0.47514E+00_JPRB, 0.47508E+00_JPRB, 0.47694E+00_JPRB, 0.48834E+00_JPRB ,&
&  0.51477E+00_JPRB, 0.53811E+00_JPRB, 0.55767E+00_JPRB, 0.56392E+00_JPRB, 0.56059E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.61853E+00_JPRB, 0.59025E+00_JPRB, 0.57856E+00_JPRB, 0.58025E+00_JPRB, 0.58362E+00_JPRB, 0.57917E+00_JPRB, 0.58614E+00_JPRB ,&
&  0.59728E+00_JPRB, 0.60574E+00_JPRB, 0.61017E+00_JPRB, 0.60613E+00_JPRB, 0.60346E+00_JPRB, 0.60049E+00_JPRB, 0.58575E+00_JPRB ,&
&  0.58090E+00_JPRB, 0.57918E+00_JPRB, 0.58331E+00_JPRB, 0.59039E+00_JPRB, 0.59811E+00_JPRB, 0.61030E+00_JPRB, 0.61933E+00_JPRB ,&
&  0.62525E+00_JPRB, 0.62894E+00_JPRB, 0.63308E+00_JPRB, 0.63328E+00_JPRB, 0.62972E+00_JPRB, 0.62059E+00_JPRB, 0.61360E+00_JPRB ,&
&  0.60946E+00_JPRB, 0.59803E+00_JPRB, 0.58768E+00_JPRB, 0.58636E+00_JPRB, 0.58954E+00_JPRB, 0.59285E+00_JPRB, 0.59277E+00_JPRB ,&
&  0.58033E+00_JPRB, 0.56388E+00_JPRB, 0.56227E+00_JPRB, 0.56209E+00_JPRB, 0.56718E+00_JPRB, 0.57928E+00_JPRB, 0.58260E+00_JPRB ,&
&  0.57919E+00_JPRB, 0.56488E+00_JPRB, 0.54553E+00_JPRB, 0.54175E+00_JPRB, 0.54400E+00_JPRB, 0.53733E+00_JPRB, 0.53076E+00_JPRB ,&
&  0.52957E+00_JPRB, 0.52656E+00_JPRB, 0.52059E+00_JPRB, 0.51245E+00_JPRB, 0.49971E+00_JPRB, 0.48310E+00_JPRB, 0.47450E+00_JPRB ,&
&  0.47371E+00_JPRB, 0.48846E+00_JPRB, 0.51517E+00_JPRB, 0.54549E+00_JPRB, 0.57445E+00_JPRB, 0.58867E+00_JPRB, 0.58821E+00_JPRB ,&
&  0.57643E+00_JPRB, 0.57376E+00_JPRB, 0.57905E+00_JPRB, 0.58347E+00_JPRB, 0.58444E+00_JPRB, 0.57371E+00_JPRB, 0.55659E+00_JPRB ,&
&  0.54421E+00_JPRB, 0.53492E+00_JPRB, 0.53381E+00_JPRB, 0.53876E+00_JPRB, 0.54368E+00_JPRB, 0.54487E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.53217E+00_JPRB, 0.52536E+00_JPRB, 0.52141E+00_JPRB, 0.52417E+00_JPRB, 0.52950E+00_JPRB, 0.53488E+00_JPRB, 0.54772E+00_JPRB ,&
&  0.56618E+00_JPRB, 0.58475E+00_JPRB, 0.58920E+00_JPRB, 0.58544E+00_JPRB, 0.58531E+00_JPRB, 0.58259E+00_JPRB, 0.57979E+00_JPRB ,&
&  0.58929E+00_JPRB, 0.61151E+00_JPRB, 0.62798E+00_JPRB, 0.63775E+00_JPRB, 0.63856E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.67032E+00_JPRB, 0.64560E+00_JPRB, 0.62694E+00_JPRB, 0.62001E+00_JPRB, 0.61627E+00_JPRB, 0.60387E+00_JPRB, 0.59817E+00_JPRB ,&
&  0.60413E+00_JPRB, 0.60598E+00_JPRB, 0.61277E+00_JPRB, 0.61200E+00_JPRB, 0.61109E+00_JPRB, 0.61365E+00_JPRB, 0.60595E+00_JPRB ,&
&  0.60592E+00_JPRB, 0.61493E+00_JPRB, 0.62550E+00_JPRB, 0.63980E+00_JPRB, 0.65179E+00_JPRB, 0.66089E+00_JPRB, 0.66860E+00_JPRB ,&
&  0.67164E+00_JPRB, 0.67814E+00_JPRB, 0.68170E+00_JPRB, 0.67751E+00_JPRB, 0.66658E+00_JPRB, 0.65808E+00_JPRB, 0.65671E+00_JPRB ,&
&  0.65543E+00_JPRB, 0.64991E+00_JPRB, 0.60572E+00_JPRB, 0.62987E+00_JPRB, 0.63719E+00_JPRB, 0.64169E+00_JPRB, 0.64501E+00_JPRB ,&
&  0.64025E+00_JPRB, 0.62546E+00_JPRB, 0.61833E+00_JPRB, 0.60616E+00_JPRB, 0.60487E+00_JPRB, 0.61079E+00_JPRB, 0.60905E+00_JPRB ,&
&  0.60446E+00_JPRB, 0.59371E+00_JPRB, 0.57943E+00_JPRB, 0.56839E+00_JPRB, 0.56557E+00_JPRB, 0.56355E+00_JPRB, 0.55588E+00_JPRB ,&
&  0.55061E+00_JPRB, 0.54974E+00_JPRB, 0.54802E+00_JPRB, 0.54224E+00_JPRB, 0.52756E+00_JPRB, 0.51406E+00_JPRB, 0.51143E+00_JPRB ,&
&  0.51466E+00_JPRB, 0.52153E+00_JPRB, 0.53531E+00_JPRB, 0.55339E+00_JPRB, 0.57302E+00_JPRB, 0.59149E+00_JPRB, 0.60832E+00_JPRB ,&
&  0.61522E+00_JPRB, 0.61912E+00_JPRB, 0.62296E+00_JPRB, 0.62780E+00_JPRB, 0.62864E+00_JPRB, 0.62070E+00_JPRB, 0.60103E+00_JPRB ,&
&  0.58019E+00_JPRB, 0.57470E+00_JPRB, 0.57867E+00_JPRB, 0.58774E+00_JPRB, 0.59237E+00_JPRB, 0.59177E+00_JPRB, 0.58500E+00_JPRB ,&
&  0.58053E+00_JPRB, 0.57195E+00_JPRB, 0.56306E+00_JPRB, 0.55874E+00_JPRB, 0.55850E+00_JPRB, 0.56468E+00_JPRB, 0.58217E+00_JPRB ,&
&  0.60309E+00_JPRB, 0.62651E+00_JPRB, 0.64755E+00_JPRB, 0.65605E+00_JPRB, 0.66127E+00_JPRB, 0.66123E+00_JPRB, 0.65231E+00_JPRB ,&
&  0.65609E+00_JPRB, 0.67489E+00_JPRB, 0.68524E+00_JPRB, 0.68716E+00_JPRB, 0.68525E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.73036E+00_JPRB, 0.70908E+00_JPRB, 0.69556E+00_JPRB, 0.68309E+00_JPRB, 0.67414E+00_JPRB, 0.65556E+00_JPRB, 0.64501E+00_JPRB ,&
&  0.63829E+00_JPRB, 0.63394E+00_JPRB, 0.63242E+00_JPRB, 0.63199E+00_JPRB, 0.64195E+00_JPRB, 0.64835E+00_JPRB, 0.65065E+00_JPRB ,&
&  0.65804E+00_JPRB, 0.67537E+00_JPRB, 0.69134E+00_JPRB, 0.70475E+00_JPRB, 0.71532E+00_JPRB, 0.72302E+00_JPRB, 0.72294E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.73470E+00_JPRB, 0.73862E+00_JPRB, 0.72869E+00_JPRB, 0.71617E+00_JPRB, 0.70973E+00_JPRB, 0.71042E+00_JPRB ,&
&  0.70008E+00_JPRB, 0.65550E+00_JPRB, 0.67045E+00_JPRB, 0.68274E+00_JPRB, 0.69771E+00_JPRB, 0.70655E+00_JPRB, 0.71601E+00_JPRB ,&
&  0.71806E+00_JPRB, 0.70103E+00_JPRB, 0.67932E+00_JPRB, 0.66370E+00_JPRB, 0.66689E+00_JPRB, 0.66405E+00_JPRB, 0.65445E+00_JPRB ,&
&  0.64995E+00_JPRB, 0.64608E+00_JPRB, 0.63827E+00_JPRB, 0.62509E+00_JPRB, 0.61329E+00_JPRB, 0.60529E+00_JPRB, 0.59999E+00_JPRB ,&
&  0.58874E+00_JPRB, 0.57945E+00_JPRB, 0.57445E+00_JPRB, 0.56893E+00_JPRB, 0.56431E+00_JPRB, 0.56849E+00_JPRB, 0.57198E+00_JPRB ,&
&  0.57135E+00_JPRB, 0.57227E+00_JPRB, 0.58030E+00_JPRB, 0.59484E+00_JPRB, 0.60753E+00_JPRB, 0.62200E+00_JPRB, 0.63886E+00_JPRB ,&
&  0.65556E+00_JPRB, 0.67034E+00_JPRB, 0.67540E+00_JPRB, 0.67931E+00_JPRB, 0.67385E+00_JPRB, 0.66508E+00_JPRB, 0.64733E+00_JPRB ,&
&  0.62784E+00_JPRB, 0.62448E+00_JPRB, 0.62588E+00_JPRB, 0.63222E+00_JPRB, 0.63109E+00_JPRB, 0.62845E+00_JPRB, 0.62669E+00_JPRB ,&
&  0.62256E+00_JPRB, 0.61899E+00_JPRB, 0.61402E+00_JPRB, 0.60930E+00_JPRB, 0.61244E+00_JPRB, 0.61835E+00_JPRB, 0.63189E+00_JPRB ,&
&  0.65005E+00_JPRB, 0.66956E+00_JPRB, 0.68984E+00_JPRB, 0.70582E+00_JPRB, 0.71334E+00_JPRB, 0.71796E+00_JPRB, 0.72206E+00_JPRB ,&
&  0.72337E+00_JPRB, 0.72341E+00_JPRB, 0.73133E+00_JPRB, 0.73654E+00_JPRB, 0.74111E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.77849E+00_JPRB, 0.75543E+00_JPRB, 0.74079E+00_JPRB, 0.73327E+00_JPRB, 0.72249E+00_JPRB, 0.70233E+00_JPRB, 0.68600E+00_JPRB ,&
&  0.67033E+00_JPRB, 0.66101E+00_JPRB, 0.65639E+00_JPRB, 0.65466E+00_JPRB, 0.65997E+00_JPRB, 0.66504E+00_JPRB, 0.67151E+00_JPRB ,&
&  0.68252E+00_JPRB, 0.70056E+00_JPRB, 0.71605E+00_JPRB, 0.73038E+00_JPRB, 0.74331E+00_JPRB, 0.75615E+00_JPRB, 0.75959E+00_JPRB ,&
&  0.76235E+00_JPRB, 0.76986E+00_JPRB, 0.77148E+00_JPRB, 0.75926E+00_JPRB, 0.74648E+00_JPRB, 0.74956E+00_JPRB, 0.74678E+00_JPRB ,&
&  0.73737E+00_JPRB, 0.62409E+00_JPRB, 0.67842E+00_JPRB, 0.70564E+00_JPRB, 0.73717E+00_JPRB, 0.76575E+00_JPRB, 0.78245E+00_JPRB ,&
&  0.77747E+00_JPRB, 0.75844E+00_JPRB, 0.73662E+00_JPRB, 0.70870E+00_JPRB, 0.70022E+00_JPRB, 0.69279E+00_JPRB, 0.68322E+00_JPRB ,&
&  0.67660E+00_JPRB, 0.67013E+00_JPRB, 0.66145E+00_JPRB, 0.64983E+00_JPRB, 0.63499E+00_JPRB, 0.62398E+00_JPRB, 0.61426E+00_JPRB ,&
&  0.61163E+00_JPRB, 0.60418E+00_JPRB, 0.59738E+00_JPRB, 0.59815E+00_JPRB, 0.60989E+00_JPRB, 0.61534E+00_JPRB, 0.61563E+00_JPRB ,&
&  0.61607E+00_JPRB, 0.61808E+00_JPRB, 0.62730E+00_JPRB, 0.64206E+00_JPRB, 0.65446E+00_JPRB, 0.66320E+00_JPRB, 0.67560E+00_JPRB ,&
&  0.69546E+00_JPRB, 0.70990E+00_JPRB, 0.71053E+00_JPRB, 0.69936E+00_JPRB, 0.69240E+00_JPRB, 0.69213E+00_JPRB, 0.68227E+00_JPRB ,&
&  0.66591E+00_JPRB, 0.65586E+00_JPRB, 0.65395E+00_JPRB, 0.65370E+00_JPRB, 0.65050E+00_JPRB, 0.64487E+00_JPRB, 0.64415E+00_JPRB ,&
&  0.64547E+00_JPRB, 0.64604E+00_JPRB, 0.64528E+00_JPRB, 0.64789E+00_JPRB, 0.65400E+00_JPRB, 0.66239E+00_JPRB, 0.67400E+00_JPRB ,&
&  0.69092E+00_JPRB, 0.70719E+00_JPRB, 0.71662E+00_JPRB, 0.73373E+00_JPRB, 0.75316E+00_JPRB, 0.76236E+00_JPRB, 0.77911E+00_JPRB ,&
&  0.78383E+00_JPRB, 0.78531E+00_JPRB, 0.78627E+00_JPRB, 0.79218E+00_JPRB, 0.79340E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.83108E+00_JPRB, 0.80461E+00_JPRB, 0.78835E+00_JPRB, 0.77755E+00_JPRB, 0.76938E+00_JPRB, 0.76037E+00_JPRB, 0.74431E+00_JPRB ,&
&  0.72702E+00_JPRB, 0.71746E+00_JPRB, 0.70944E+00_JPRB, 0.69987E+00_JPRB, 0.70019E+00_JPRB, 0.70547E+00_JPRB, 0.71628E+00_JPRB ,&
&  0.72836E+00_JPRB, 0.74397E+00_JPRB, 0.75902E+00_JPRB, 0.77804E+00_JPRB, 0.79055E+00_JPRB, 0.79921E+00_JPRB, 0.80384E+00_JPRB ,&
&  0.80704E+00_JPRB, 0.80756E+00_JPRB, 0.80022E+00_JPRB, 0.79073E+00_JPRB, 0.78849E+00_JPRB, 0.79001E+00_JPRB, 0.78515E+00_JPRB ,&
&  0.68629E+00_JPRB, 0.60189E+00_JPRB, 0.65755E+00_JPRB, 0.72452E+00_JPRB, 0.79347E+00_JPRB, 0.84994E+00_JPRB, 0.87615E+00_JPRB ,&
&  0.86177E+00_JPRB, 0.82820E+00_JPRB, 0.79832E+00_JPRB, 0.76466E+00_JPRB, 0.75029E+00_JPRB, 0.74346E+00_JPRB, 0.73323E+00_JPRB ,&
&  0.72215E+00_JPRB, 0.70937E+00_JPRB, 0.69360E+00_JPRB, 0.66953E+00_JPRB, 0.65244E+00_JPRB, 0.64007E+00_JPRB, 0.63773E+00_JPRB ,&
&  0.63972E+00_JPRB, 0.64341E+00_JPRB, 0.64677E+00_JPRB, 0.64939E+00_JPRB, 0.65629E+00_JPRB, 0.66628E+00_JPRB, 0.67792E+00_JPRB ,&
&  0.69424E+00_JPRB, 0.71009E+00_JPRB, 0.73295E+00_JPRB, 0.74861E+00_JPRB, 0.76016E+00_JPRB, 0.76980E+00_JPRB, 0.78059E+00_JPRB ,&
&  0.78396E+00_JPRB, 0.77957E+00_JPRB, 0.76154E+00_JPRB, 0.75249E+00_JPRB, 0.75225E+00_JPRB, 0.75206E+00_JPRB, 0.74572E+00_JPRB ,&
&  0.73180E+00_JPRB, 0.71479E+00_JPRB, 0.69753E+00_JPRB, 0.68823E+00_JPRB, 0.68827E+00_JPRB, 0.68713E+00_JPRB, 0.68248E+00_JPRB ,&
&  0.67865E+00_JPRB, 0.67861E+00_JPRB, 0.68447E+00_JPRB, 0.69590E+00_JPRB, 0.71145E+00_JPRB, 0.73096E+00_JPRB, 0.74765E+00_JPRB ,&
&  0.76524E+00_JPRB, 0.78302E+00_JPRB, 0.79531E+00_JPRB, 0.82488E+00_JPRB, 0.85149E+00_JPRB, 0.86604E+00_JPRB, 0.86992E+00_JPRB ,&
&  0.87899E+00_JPRB, 0.87618E+00_JPRB, 0.87021E+00_JPRB, 0.86597E+00_JPRB, 0.85294E+00_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.82864E+00_JPRB, 0.82722E+00_JPRB, 0.82638E+00_JPRB, 0.81767E+00_JPRB, 0.80978E+00_JPRB, 0.79689E+00_JPRB, 0.77428E+00_JPRB ,&
&  0.75368E+00_JPRB, 0.74464E+00_JPRB, 0.74256E+00_JPRB, 0.74237E+00_JPRB, 0.74412E+00_JPRB, 0.74966E+00_JPRB, 0.76085E+00_JPRB ,&
&  0.77459E+00_JPRB, 0.78921E+00_JPRB, 0.80632E+00_JPRB, 0.82206E+00_JPRB, 0.82558E+00_JPRB, 0.82246E+00_JPRB, 0.82278E+00_JPRB ,&
&  0.82899E+00_JPRB, 0.82804E+00_JPRB, 0.82161E+00_JPRB, 0.82423E+00_JPRB, 0.83288E+00_JPRB, 0.82427E+00_JPRB, 0.80608E+00_JPRB ,&
&  0.79506E+00_JPRB, 0.65458E+00_JPRB, 0.69525E+00_JPRB, 0.77050E+00_JPRB, 0.85873E+00_JPRB, 0.93999E+00_JPRB, 0.97321E+00_JPRB ,&
&  0.95990E+00_JPRB, 0.91853E+00_JPRB, 0.88412E+00_JPRB, 0.84317E+00_JPRB, 0.82205E+00_JPRB, 0.80642E+00_JPRB, 0.78817E+00_JPRB ,&
&  0.76293E+00_JPRB, 0.73733E+00_JPRB, 0.70973E+00_JPRB, 0.68560E+00_JPRB, 0.66646E+00_JPRB, 0.65738E+00_JPRB, 0.65455E+00_JPRB ,&
&  0.65471E+00_JPRB, 0.66551E+00_JPRB, 0.67162E+00_JPRB, 0.68165E+00_JPRB, 0.69790E+00_JPRB, 0.72144E+00_JPRB, 0.74909E+00_JPRB ,&
&  0.77960E+00_JPRB, 0.81007E+00_JPRB, 0.84020E+00_JPRB, 0.86178E+00_JPRB, 0.87984E+00_JPRB, 0.89367E+00_JPRB, 0.88775E+00_JPRB ,&
&  0.85967E+00_JPRB, 0.82824E+00_JPRB, 0.82425E+00_JPRB, 0.82691E+00_JPRB, 0.82113E+00_JPRB, 0.81660E+00_JPRB, 0.80885E+00_JPRB ,&
&  0.79353E+00_JPRB, 0.77493E+00_JPRB, 0.76146E+00_JPRB, 0.74692E+00_JPRB, 0.73782E+00_JPRB, 0.73467E+00_JPRB, 0.72110E+00_JPRB ,&
&  0.71216E+00_JPRB, 0.71191E+00_JPRB, 0.72080E+00_JPRB, 0.74166E+00_JPRB, 0.77013E+00_JPRB, 0.79839E+00_JPRB, 0.81825E+00_JPRB ,&
&  0.83067E+00_JPRB, 0.84267E+00_JPRB, 0.84951E+00_JPRB, 0.89359E+00_JPRB, 0.91596E+00_JPRB, 0.93303E+00_JPRB, 0.94744E+00_JPRB ,&
&  0.95695E+00_JPRB, 0.96273E+00_JPRB, 0.73222E+00_JPRB, 0.80479E+00_JPRB, 0.81641E+00_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.87186E+00_JPRB, 0.88863E+00_JPRB, 0.88940E+00_JPRB, 0.88302E+00_JPRB, 0.87260E+00_JPRB, 0.85023E+00_JPRB, 0.82500E+00_JPRB ,&
&  0.79968E+00_JPRB, 0.78779E+00_JPRB, 0.78237E+00_JPRB, 0.78540E+00_JPRB, 0.79071E+00_JPRB, 0.79936E+00_JPRB, 0.81197E+00_JPRB ,&
&  0.82391E+00_JPRB, 0.82973E+00_JPRB, 0.83437E+00_JPRB, 0.83254E+00_JPRB, 0.83076E+00_JPRB, 0.83305E+00_JPRB, 0.83744E+00_JPRB ,&
&  0.84648E+00_JPRB, 0.84982E+00_JPRB, 0.84700E+00_JPRB, 0.85606E+00_JPRB, 0.86940E+00_JPRB, 0.85912E+00_JPRB, 0.83993E+00_JPRB ,&
&  0.82898E+00_JPRB, 0.71031E+00_JPRB, 0.75027E+00_JPRB, 0.88630E+00_JPRB, 0.99998E+00_JPRB, 0.11028E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.10379E+01_JPRB, 0.98358E+00_JPRB, 0.93234E+00_JPRB, 0.89545E+00_JPRB, 0.85824E+00_JPRB, 0.83268E+00_JPRB ,&
&  0.81569E+00_JPRB, 0.79801E+00_JPRB, 0.77323E+00_JPRB, 0.75311E+00_JPRB, 0.73691E+00_JPRB, 0.72100E+00_JPRB, 0.71206E+00_JPRB ,&
&  0.71262E+00_JPRB, 0.72339E+00_JPRB, 0.73625E+00_JPRB, 0.75264E+00_JPRB, 0.78320E+00_JPRB, 0.81733E+00_JPRB, 0.86125E+00_JPRB ,&
&  0.89811E+00_JPRB, 0.93737E+00_JPRB, 0.97975E+00_JPRB, 0.10172E+01_JPRB, 0.10525E+01_JPRB, 0.10604E+01_JPRB, 0.10268E+01_JPRB ,&
&  0.96122E+00_JPRB, 0.91358E+00_JPRB, 0.88067E+00_JPRB, 0.85888E+00_JPRB, 0.85653E+00_JPRB, 0.86685E+00_JPRB, 0.87209E+00_JPRB ,&
&  0.86479E+00_JPRB, 0.84950E+00_JPRB, 0.82931E+00_JPRB, 0.81055E+00_JPRB, 0.80390E+00_JPRB, 0.80148E+00_JPRB, 0.79462E+00_JPRB ,&
&  0.78816E+00_JPRB, 0.78972E+00_JPRB, 0.79996E+00_JPRB, 0.82169E+00_JPRB, 0.84977E+00_JPRB, 0.88242E+00_JPRB, 0.90364E+00_JPRB ,&
&  0.91782E+00_JPRB, 0.93135E+00_JPRB, 0.94581E+00_JPRB, 0.96237E+00_JPRB, 0.10249E+01_JPRB, 0.10513E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10126E+01_JPRB, 0.97054E+00_JPRB, 0.79132E+00_JPRB, 0.84862E+00_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.94069E+00_JPRB, 0.96032E+00_JPRB, 0.95922E+00_JPRB, 0.94958E+00_JPRB, 0.92593E+00_JPRB, 0.89384E+00_JPRB, 0.86969E+00_JPRB ,&
&  0.85672E+00_JPRB, 0.83720E+00_JPRB, 0.82799E+00_JPRB, 0.82465E+00_JPRB, 0.82819E+00_JPRB, 0.83559E+00_JPRB, 0.83603E+00_JPRB ,&
&  0.82666E+00_JPRB, 0.81211E+00_JPRB, 0.80173E+00_JPRB, 0.80034E+00_JPRB, 0.80964E+00_JPRB, 0.82011E+00_JPRB, 0.82712E+00_JPRB ,&
&  0.83024E+00_JPRB, 0.82406E+00_JPRB, 0.82155E+00_JPRB, 0.83210E+00_JPRB, 0.84048E+00_JPRB, 0.83296E+00_JPRB, 0.81310E+00_JPRB ,&
&  0.81565E+00_JPRB, 0.79374E+00_JPRB, 0.94739E+00_JPRB, 0.10993E+01_JPRB, 0.12232E+01_JPRB, 0.12690E+01_JPRB, 0.12483E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.11173E+01_JPRB, 0.10380E+01_JPRB, 0.97303E+00_JPRB, 0.92546E+00_JPRB, 0.89365E+00_JPRB, 0.87259E+00_JPRB ,&
&  0.84362E+00_JPRB, 0.81587E+00_JPRB, 0.79147E+00_JPRB, 0.77767E+00_JPRB, 0.77271E+00_JPRB, 0.76858E+00_JPRB, 0.76467E+00_JPRB ,&
&  0.76013E+00_JPRB, 0.75664E+00_JPRB, 0.76162E+00_JPRB, 0.77897E+00_JPRB, 0.80713E+00_JPRB, 0.86776E+00_JPRB, 0.95747E+00_JPRB ,&
&  0.10232E+01_JPRB, 0.10952E+01_JPRB, 0.11698E+01_JPRB, 0.12340E+01_JPRB, 0.12389E+01_JPRB, 0.12071E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.10919E+01_JPRB, 0.10108E+01_JPRB, 0.94132E+00_JPRB, 0.89722E+00_JPRB, 0.88091E+00_JPRB, 0.87753E+00_JPRB, 0.89180E+00_JPRB ,&
&  0.89982E+00_JPRB, 0.90037E+00_JPRB, 0.88268E+00_JPRB, 0.86374E+00_JPRB, 0.85367E+00_JPRB, 0.83598E+00_JPRB, 0.83170E+00_JPRB ,&
&  0.83394E+00_JPRB, 0.84765E+00_JPRB, 0.87269E+00_JPRB, 0.89756E+00_JPRB, 0.93145E+00_JPRB, 0.95776E+00_JPRB, 0.96915E+00_JPRB ,&
&  0.97336E+00_JPRB, 0.97707E+00_JPRB, 0.97803E+00_JPRB, 0.10701E+01_JPRB, 0.11052E+01_JPRB, 0.11262E+01_JPRB, 0.11158E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.10404E+01_JPRB, 0.97792E+00_JPRB, 0.95109E+00_JPRB, 0.88073E+00_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.10957E+01_JPRB, 0.10754E+01_JPRB, 0.10397E+01_JPRB, 0.10076E+01_JPRB, 0.97103E+00_JPRB, 0.94455E+00_JPRB, 0.92279E+00_JPRB ,&
&  0.90938E+00_JPRB, 0.88802E+00_JPRB, 0.87650E+00_JPRB, 0.86724E+00_JPRB, 0.86292E+00_JPRB, 0.85324E+00_JPRB, 0.83883E+00_JPRB ,&
&  0.82292E+00_JPRB, 0.81255E+00_JPRB, 0.80830E+00_JPRB, 0.81154E+00_JPRB, 0.82084E+00_JPRB, 0.82686E+00_JPRB, 0.82154E+00_JPRB ,&
&  0.81263E+00_JPRB, 0.79870E+00_JPRB, 0.78266E+00_JPRB, 0.79790E+00_JPRB, 0.81418E+00_JPRB, 0.81912E+00_JPRB, 0.80513E+00_JPRB ,&
&  0.83325E+00_JPRB, 0.95400E+00_JPRB, 0.13385E+01_JPRB, 0.14967E+01_JPRB, 0.15242E+01_JPRB, 0.15421E+01_JPRB, 0.14170E+01_JPRB ,&
&  0.13017E+01_JPRB, 0.11713E+01_JPRB, 0.10644E+01_JPRB, 0.10084E+01_JPRB, 0.97375E+00_JPRB, 0.92847E+00_JPRB, 0.89120E+00_JPRB ,&
&  0.84382E+00_JPRB, 0.81956E+00_JPRB, 0.80382E+00_JPRB, 0.79743E+00_JPRB, 0.79427E+00_JPRB, 0.79488E+00_JPRB, 0.80019E+00_JPRB ,&
&  0.80559E+00_JPRB, 0.80842E+00_JPRB, 0.81742E+00_JPRB, 0.84532E+00_JPRB, 0.89055E+00_JPRB, 0.10198E+01_JPRB, 0.11929E+01_JPRB ,&
&  0.13035E+01_JPRB, 0.13842E+01_JPRB, 0.14512E+01_JPRB, 0.14531E+01_JPRB, 0.13792E+01_JPRB, 0.12851E+01_JPRB, 0.12149E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.10897E+01_JPRB, 0.10295E+01_JPRB, 0.98468E+00_JPRB, 0.94379E+00_JPRB, 0.92665E+00_JPRB, 0.94431E+00_JPRB ,&
&  0.95346E+00_JPRB, 0.95582E+00_JPRB, 0.93830E+00_JPRB, 0.91489E+00_JPRB, 0.88044E+00_JPRB, 0.85730E+00_JPRB, 0.85535E+00_JPRB ,&
&  0.86628E+00_JPRB, 0.88986E+00_JPRB, 0.92607E+00_JPRB, 0.96608E+00_JPRB, 0.10011E+01_JPRB, 0.10221E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.10534E+01_JPRB, 0.10134E+01_JPRB, 0.11423E+01_JPRB, 0.12888E+01_JPRB, 0.12727E+01_JPRB, 0.12392E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.10904E+01_JPRB, 0.10254E+01_JPRB, 0.99908E+00_JPRB, 0.10563E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.10524E+01_JPRB, 0.10328E+01_JPRB, 0.10000E+01_JPRB, 0.97481E+00_JPRB, 0.96447E+00_JPRB, 0.94753E+00_JPRB, 0.93319E+00_JPRB ,&
&  0.92362E+00_JPRB, 0.90807E+00_JPRB, 0.89491E+00_JPRB, 0.87744E+00_JPRB, 0.85634E+00_JPRB, 0.83880E+00_JPRB, 0.83168E+00_JPRB ,&
&  0.82972E+00_JPRB, 0.81854E+00_JPRB, 0.82167E+00_JPRB, 0.82318E+00_JPRB, 0.81951E+00_JPRB, 0.80629E+00_JPRB, 0.79303E+00_JPRB ,&
&  0.78582E+00_JPRB, 0.77337E+00_JPRB, 0.77222E+00_JPRB, 0.79409E+00_JPRB, 0.81244E+00_JPRB, 0.81481E+00_JPRB, 0.80121E+00_JPRB ,&
&  0.86824E+00_JPRB, 0.11955E+01_JPRB, 0.18256E+01_JPRB, 0.20463E+01_JPRB, 0.19675E+01_JPRB, 0.17319E+01_JPRB, 0.15044E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.11953E+01_JPRB, 0.11143E+01_JPRB, 0.10468E+01_JPRB, 0.98164E+00_JPRB, 0.92812E+00_JPRB, 0.88594E+00_JPRB ,&
&  0.84351E+00_JPRB, 0.82506E+00_JPRB, 0.81572E+00_JPRB, 0.81357E+00_JPRB, 0.81109E+00_JPRB, 0.80827E+00_JPRB, 0.81158E+00_JPRB ,&
&  0.81900E+00_JPRB, 0.82928E+00_JPRB, 0.85333E+00_JPRB, 0.89313E+00_JPRB, 0.10541E+01_JPRB, 0.13228E+01_JPRB, 0.16863E+01_JPRB ,&
&  0.17693E+01_JPRB, 0.17958E+01_JPRB, 0.16843E+01_JPRB, 0.15380E+01_JPRB, 0.14178E+01_JPRB, 0.13205E+01_JPRB, 0.12549E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11349E+01_JPRB, 0.10973E+01_JPRB, 0.10596E+01_JPRB, 0.10158E+01_JPRB, 0.99907E+00_JPRB, 0.10061E+01_JPRB ,&
&  0.10202E+01_JPRB, 0.10164E+01_JPRB, 0.98646E+00_JPRB, 0.95281E+00_JPRB, 0.91449E+00_JPRB, 0.90271E+00_JPRB, 0.90180E+00_JPRB ,&
&  0.90649E+00_JPRB, 0.92597E+00_JPRB, 0.97214E+00_JPRB, 0.10166E+01_JPRB, 0.10501E+01_JPRB, 0.10668E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.11282E+01_JPRB, 0.11086E+01_JPRB, 0.11974E+01_JPRB, 0.12841E+01_JPRB, 0.13493E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.10450E+01_JPRB, 0.10014E+01_JPRB, 0.98803E+00_JPRB, 0.10432E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.10082E+01_JPRB, 0.98374E+00_JPRB, 0.96152E+00_JPRB, 0.94147E+00_JPRB, 0.92231E+00_JPRB, 0.91040E+00_JPRB, 0.89669E+00_JPRB ,&
&  0.89609E+00_JPRB, 0.89359E+00_JPRB, 0.87920E+00_JPRB, 0.84815E+00_JPRB, 0.82269E+00_JPRB, 0.81687E+00_JPRB, 0.82099E+00_JPRB ,&
&  0.81425E+00_JPRB, 0.80285E+00_JPRB, 0.80594E+00_JPRB, 0.78875E+00_JPRB, 0.76655E+00_JPRB, 0.75205E+00_JPRB, 0.74608E+00_JPRB ,&
&  0.75035E+00_JPRB, 0.76214E+00_JPRB, 0.79059E+00_JPRB, 0.81702E+00_JPRB, 0.81994E+00_JPRB, 0.81227E+00_JPRB, 0.80405E+00_JPRB ,&
&  0.96641E+00_JPRB, 0.13886E+01_JPRB, 0.23918E+01_JPRB, 0.23544E+01_JPRB, 0.20617E+01_JPRB, 0.17456E+01_JPRB, 0.16062E+01_JPRB ,&
&  0.14018E+01_JPRB, 0.12756E+01_JPRB, 0.11459E+01_JPRB, 0.10309E+01_JPRB, 0.94639E+00_JPRB, 0.89622E+00_JPRB, 0.86872E+00_JPRB ,&
&  0.84108E+00_JPRB, 0.82561E+00_JPRB, 0.82751E+00_JPRB, 0.83275E+00_JPRB, 0.83470E+00_JPRB, 0.83008E+00_JPRB, 0.82034E+00_JPRB ,&
&  0.82729E+00_JPRB, 0.84620E+00_JPRB, 0.88262E+00_JPRB, 0.96726E+00_JPRB, 0.12474E+01_JPRB, 0.13962E+01_JPRB, 0.19552E+01_JPRB ,&
&  0.25666E+01_JPRB, 0.22500E+01_JPRB, 0.18753E+01_JPRB, 0.16390E+01_JPRB, 0.14947E+01_JPRB, 0.13820E+01_JPRB, 0.13131E+01_JPRB ,&
&  0.12411E+01_JPRB, 0.11962E+01_JPRB, 0.11723E+01_JPRB, 0.11426E+01_JPRB, 0.11247E+01_JPRB, 0.10816E+01_JPRB, 0.10448E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.10065E+01_JPRB, 0.97376E+00_JPRB, 0.93988E+00_JPRB, 0.94068E+00_JPRB, 0.95268E+00_JPRB, 0.95842E+00_JPRB ,&
&  0.96933E+00_JPRB, 0.97829E+00_JPRB, 0.10317E+01_JPRB, 0.10302E+01_JPRB, 0.10767E+01_JPRB, 0.11184E+01_JPRB, 0.11157E+01_JPRB ,&
&  0.10736E+01_JPRB, 0.11111E+01_JPRB, 0.11313E+01_JPRB, 0.11499E+01_JPRB, 0.12328E+01_JPRB, 0.13123E+01_JPRB, 0.11482E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.96841E+00_JPRB, 0.93565E+00_JPRB, 0.94457E+00_JPRB, 0.10122E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.10061E+01_JPRB, 0.98034E+00_JPRB, 0.95060E+00_JPRB, 0.91073E+00_JPRB, 0.89896E+00_JPRB, 0.88417E+00_JPRB, 0.87510E+00_JPRB ,&
&  0.87123E+00_JPRB, 0.85576E+00_JPRB, 0.83896E+00_JPRB, 0.81566E+00_JPRB, 0.80594E+00_JPRB, 0.81653E+00_JPRB, 0.81121E+00_JPRB ,&
&  0.79324E+00_JPRB, 0.76978E+00_JPRB, 0.76151E+00_JPRB, 0.75222E+00_JPRB, 0.74192E+00_JPRB, 0.74144E+00_JPRB, 0.75057E+00_JPRB ,&
&  0.75534E+00_JPRB, 0.77472E+00_JPRB, 0.80651E+00_JPRB, 0.83563E+00_JPRB, 0.83559E+00_JPRB, 0.83253E+00_JPRB, 0.88056E+00_JPRB ,&
&  0.10538E+01_JPRB, 0.10558E+01_JPRB, 0.21917E+01_JPRB, 0.25606E+01_JPRB, 0.21484E+01_JPRB, 0.18771E+01_JPRB, 0.16153E+01_JPRB ,&
&  0.14891E+01_JPRB, 0.13016E+01_JPRB, 0.11503E+01_JPRB, 0.10394E+01_JPRB, 0.97191E+00_JPRB, 0.89776E+00_JPRB, 0.86572E+00_JPRB ,&
&  0.85545E+00_JPRB, 0.84118E+00_JPRB, 0.84104E+00_JPRB, 0.85829E+00_JPRB, 0.86344E+00_JPRB, 0.86817E+00_JPRB, 0.85989E+00_JPRB ,&
&  0.86597E+00_JPRB, 0.88498E+00_JPRB, 0.94582E+00_JPRB, 0.11098E+01_JPRB, 0.13933E+01_JPRB, 0.16976E+01_JPRB, 0.21719E+01_JPRB ,&
&  0.24083E+01_JPRB, 0.23702E+01_JPRB, 0.18718E+01_JPRB, 0.16048E+01_JPRB, 0.14549E+01_JPRB, 0.13406E+01_JPRB, 0.12914E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.11928E+01_JPRB, 0.11606E+01_JPRB, 0.11801E+01_JPRB, 0.11341E+01_JPRB, 0.11051E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.99553E+00_JPRB, 0.96508E+00_JPRB, 0.94999E+00_JPRB, 0.97006E+00_JPRB, 0.98137E+00_JPRB, 0.97903E+00_JPRB ,&
&  0.98171E+00_JPRB, 0.10125E+01_JPRB, 0.10560E+01_JPRB, 0.10648E+01_JPRB, 0.10985E+01_JPRB, 0.11073E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11420E+01_JPRB, 0.10963E+01_JPRB, 0.11226E+01_JPRB, 0.11006E+01_JPRB, 0.11508E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.95241E+00_JPRB, 0.90932E+00_JPRB, 0.91923E+00_JPRB, 0.97392E+00_JPRB, 0.10142E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.98367E+00_JPRB, 0.97503E+00_JPRB, 0.92006E+00_JPRB, 0.90963E+00_JPRB, 0.88961E+00_JPRB, 0.88620E+00_JPRB, 0.89750E+00_JPRB ,&
&  0.88860E+00_JPRB, 0.86219E+00_JPRB, 0.82864E+00_JPRB, 0.81237E+00_JPRB, 0.82295E+00_JPRB, 0.83561E+00_JPRB, 0.80365E+00_JPRB ,&
&  0.76627E+00_JPRB, 0.75336E+00_JPRB, 0.74630E+00_JPRB, 0.74909E+00_JPRB, 0.75958E+00_JPRB, 0.77555E+00_JPRB, 0.78485E+00_JPRB ,&
&  0.78051E+00_JPRB, 0.81997E+00_JPRB, 0.86551E+00_JPRB, 0.88388E+00_JPRB, 0.87158E+00_JPRB, 0.86874E+00_JPRB, 0.90631E+00_JPRB ,&
&  0.10400E+01_JPRB, 0.24917E+01_JPRB, 0.16127E+01_JPRB, 0.26490E+01_JPRB, 0.21013E+01_JPRB, 0.18328E+01_JPRB, 0.15010E+01_JPRB ,&
&  0.14059E+01_JPRB, 0.13700E+01_JPRB, 0.12350E+01_JPRB, 0.11049E+01_JPRB, 0.10013E+01_JPRB, 0.92329E+00_JPRB, 0.86427E+00_JPRB ,&
&  0.83487E+00_JPRB, 0.83147E+00_JPRB, 0.84084E+00_JPRB, 0.88920E+00_JPRB, 0.92512E+00_JPRB, 0.93468E+00_JPRB, 0.93147E+00_JPRB ,&
&  0.94205E+00_JPRB, 0.96906E+00_JPRB, 0.10720E+01_JPRB, 0.13693E+01_JPRB, 0.15904E+01_JPRB, 0.17628E+01_JPRB, 0.22342E+01_JPRB ,&
&  0.21538E+01_JPRB, 0.21637E+01_JPRB, 0.17667E+01_JPRB, 0.15369E+01_JPRB, 0.13853E+01_JPRB, 0.12854E+01_JPRB, 0.12490E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.11337E+01_JPRB, 0.11090E+01_JPRB, 0.10894E+01_JPRB, 0.10879E+01_JPRB, 0.10863E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.96844E+00_JPRB, 0.97401E+00_JPRB, 0.98922E+00_JPRB, 0.10099E+01_JPRB, 0.10009E+01_JPRB, 0.99400E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10707E+01_JPRB, 0.10956E+01_JPRB, 0.11185E+01_JPRB, 0.11124E+01_JPRB, 0.10882E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.11180E+01_JPRB, 0.11185E+01_JPRB, 0.10518E+01_JPRB, 0.10473E+01_JPRB, 0.10442E+01_JPRB, 0.10875E+01_JPRB, 0.94342E+00_JPRB ,&
&  0.90859E+00_JPRB, 0.90052E+00_JPRB, 0.96732E+00_JPRB, 0.97960E+00_JPRB, 0.99494E+00_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.96272E+00_JPRB, 0.94149E+00_JPRB, 0.89950E+00_JPRB, 0.89533E+00_JPRB, 0.86980E+00_JPRB, 0.89642E+00_JPRB, 0.90046E+00_JPRB ,&
&  0.86569E+00_JPRB, 0.83143E+00_JPRB, 0.80802E+00_JPRB, 0.82610E+00_JPRB, 0.84675E+00_JPRB, 0.83376E+00_JPRB, 0.79525E+00_JPRB ,&
&  0.78270E+00_JPRB, 0.78039E+00_JPRB, 0.78021E+00_JPRB, 0.78416E+00_JPRB, 0.79991E+00_JPRB, 0.82268E+00_JPRB, 0.82032E+00_JPRB ,&
&  0.81714E+00_JPRB, 0.84940E+00_JPRB, 0.88073E+00_JPRB, 0.88409E+00_JPRB, 0.88066E+00_JPRB, 0.87929E+00_JPRB, 0.94603E+00_JPRB ,&
&  0.10461E+01_JPRB, 0.20573E+01_JPRB, 0.15016E+01_JPRB, 0.25045E+01_JPRB, 0.20249E+01_JPRB, 0.18332E+01_JPRB, 0.15337E+01_JPRB ,&
&  0.14307E+01_JPRB, 0.15281E+01_JPRB, 0.13048E+01_JPRB, 0.11162E+01_JPRB, 0.97316E+00_JPRB, 0.91178E+00_JPRB, 0.86467E+00_JPRB ,&
&  0.82602E+00_JPRB, 0.85502E+00_JPRB, 0.90624E+00_JPRB, 0.97563E+00_JPRB, 0.10285E+01_JPRB, 0.10462E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11368E+01_JPRB, 0.13373E+01_JPRB, 0.15855E+01_JPRB, 0.18394E+01_JPRB, 0.19055E+01_JPRB, 0.19258E+01_JPRB ,&
&  0.18953E+01_JPRB, 0.19131E+01_JPRB, 0.16705E+01_JPRB, 0.14699E+01_JPRB, 0.13123E+01_JPRB, 0.12534E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.11024E+01_JPRB, 0.10187E+01_JPRB, 0.10025E+01_JPRB, 0.10039E+01_JPRB, 0.10534E+01_JPRB, 0.10572E+01_JPRB, 0.10075E+01_JPRB ,&
&  0.96334E+00_JPRB, 0.95134E+00_JPRB, 0.95910E+00_JPRB, 0.97948E+00_JPRB, 0.10037E+01_JPRB, 0.10085E+01_JPRB, 0.10155E+01_JPRB ,&
&  0.10620E+01_JPRB, 0.11390E+01_JPRB, 0.11463E+01_JPRB, 0.11521E+01_JPRB, 0.11011E+01_JPRB, 0.10646E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10556E+01_JPRB, 0.10136E+01_JPRB, 0.97596E+00_JPRB, 0.96783E+00_JPRB, 0.96821E+00_JPRB, 0.90625E+00_JPRB ,&
&  0.90359E+00_JPRB, 0.94256E+00_JPRB, 0.99667E+00_JPRB, 0.96541E+00_JPRB, 0.96203E+00_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.91023E+00_JPRB, 0.87040E+00_JPRB, 0.85096E+00_JPRB, 0.83325E+00_JPRB, 0.81719E+00_JPRB, 0.84880E+00_JPRB, 0.83918E+00_JPRB ,&
&  0.81437E+00_JPRB, 0.78396E+00_JPRB, 0.80003E+00_JPRB, 0.84507E+00_JPRB, 0.84598E+00_JPRB, 0.82573E+00_JPRB, 0.81210E+00_JPRB ,&
&  0.81181E+00_JPRB, 0.82954E+00_JPRB, 0.83332E+00_JPRB, 0.84577E+00_JPRB, 0.84666E+00_JPRB, 0.83618E+00_JPRB, 0.82934E+00_JPRB ,&
&  0.84574E+00_JPRB, 0.85929E+00_JPRB, 0.87726E+00_JPRB, 0.89465E+00_JPRB, 0.92455E+00_JPRB, 0.95455E+00_JPRB, 0.11100E+01_JPRB ,&
&  0.13211E+01_JPRB, 0.19912E+01_JPRB, 0.16086E+01_JPRB, 0.19365E+01_JPRB, 0.18333E+01_JPRB, 0.16991E+01_JPRB, 0.15146E+01_JPRB ,&
&  0.15654E+01_JPRB, 0.14604E+01_JPRB, 0.14391E+01_JPRB, 0.11479E+01_JPRB, 0.10326E+01_JPRB, 0.10331E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.10267E+01_JPRB, 0.10368E+01_JPRB, 0.11587E+01_JPRB, 0.12208E+01_JPRB, 0.12531E+01_JPRB, 0.12657E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.13372E+01_JPRB, 0.14509E+01_JPRB, 0.17821E+01_JPRB, 0.20139E+01_JPRB, 0.19623E+01_JPRB, 0.19100E+01_JPRB, 0.18090E+01_JPRB ,&
&  0.17892E+01_JPRB, 0.17178E+01_JPRB, 0.15879E+01_JPRB, 0.14209E+01_JPRB, 0.12127E+01_JPRB, 0.12097E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.99251E+00_JPRB, 0.92817E+00_JPRB, 0.92542E+00_JPRB, 0.95228E+00_JPRB, 0.99435E+00_JPRB, 0.99012E+00_JPRB, 0.96520E+00_JPRB ,&
&  0.95712E+00_JPRB, 0.96769E+00_JPRB, 0.95811E+00_JPRB, 0.98146E+00_JPRB, 0.10049E+01_JPRB, 0.10480E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.11478E+01_JPRB, 0.12025E+01_JPRB, 0.11823E+01_JPRB, 0.11477E+01_JPRB, 0.10885E+01_JPRB, 0.10477E+01_JPRB, 0.99420E+00_JPRB ,&
&  0.99720E+00_JPRB, 0.10270E+01_JPRB, 0.10078E+01_JPRB, 0.97086E+00_JPRB, 0.94207E+00_JPRB, 0.92090E+00_JPRB, 0.92152E+00_JPRB ,&
&  0.94295E+00_JPRB, 0.99838E+00_JPRB, 0.99884E+00_JPRB, 0.94888E+00_JPRB, 0.93643E+00_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.82225E+00_JPRB, 0.80750E+00_JPRB, 0.78087E+00_JPRB, 0.76306E+00_JPRB, 0.78618E+00_JPRB, 0.82054E+00_JPRB, 0.81941E+00_JPRB ,&
&  0.79358E+00_JPRB, 0.77850E+00_JPRB, 0.81347E+00_JPRB, 0.84924E+00_JPRB, 0.84591E+00_JPRB, 0.86293E+00_JPRB, 0.87252E+00_JPRB ,&
&  0.90550E+00_JPRB, 0.90023E+00_JPRB, 0.88103E+00_JPRB, 0.85438E+00_JPRB, 0.85191E+00_JPRB, 0.83945E+00_JPRB, 0.83944E+00_JPRB ,&
&  0.85159E+00_JPRB, 0.87167E+00_JPRB, 0.91300E+00_JPRB, 0.97838E+00_JPRB, 0.10563E+01_JPRB, 0.12180E+01_JPRB, 0.15450E+01_JPRB ,&
&  0.19380E+01_JPRB, 0.23560E+01_JPRB, 0.87019E+00_JPRB, 0.11314E+01_JPRB, 0.13696E+01_JPRB, 0.13298E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.11845E+01_JPRB, 0.11456E+01_JPRB, 0.11360E+01_JPRB, 0.11410E+01_JPRB, 0.11844E+01_JPRB, 0.12523E+01_JPRB, 0.12672E+01_JPRB ,&
&  0.13082E+01_JPRB, 0.13677E+01_JPRB, 0.14607E+01_JPRB, 0.14963E+01_JPRB, 0.15089E+01_JPRB, 0.15108E+01_JPRB, 0.15452E+01_JPRB ,&
&  0.16920E+01_JPRB, 0.19358E+01_JPRB, 0.22978E+01_JPRB, 0.21458E+01_JPRB, 0.21116E+01_JPRB, 0.18797E+01_JPRB, 0.16613E+01_JPRB ,&
&  0.14897E+01_JPRB, 0.15464E+01_JPRB, 0.15166E+01_JPRB, 0.13555E+01_JPRB, 0.11490E+01_JPRB, 0.10447E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.96477E+00_JPRB, 0.89502E+00_JPRB, 0.87780E+00_JPRB, 0.88837E+00_JPRB, 0.94403E+00_JPRB, 0.94395E+00_JPRB, 0.93060E+00_JPRB ,&
&  0.95155E+00_JPRB, 0.94708E+00_JPRB, 0.97187E+00_JPRB, 0.10107E+01_JPRB, 0.10533E+01_JPRB, 0.11242E+01_JPRB, 0.11658E+01_JPRB ,&
&  0.11798E+01_JPRB, 0.12069E+01_JPRB, 0.12161E+01_JPRB, 0.11746E+01_JPRB, 0.11110E+01_JPRB, 0.10644E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10632E+01_JPRB, 0.10751E+01_JPRB, 0.96213E+00_JPRB, 0.90736E+00_JPRB, 0.93070E+00_JPRB, 0.93669E+00_JPRB, 0.10281E+01_JPRB ,&
&  0.11346E+01_JPRB, 0.12068E+01_JPRB, 0.11621E+01_JPRB, 0.89070E+00_JPRB, 0.87825E+00_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.77103E+00_JPRB, 0.71864E+00_JPRB, 0.68598E+00_JPRB, 0.69615E+00_JPRB, 0.74629E+00_JPRB, 0.79783E+00_JPRB, 0.79560E+00_JPRB ,&
&  0.78082E+00_JPRB, 0.80114E+00_JPRB, 0.87103E+00_JPRB, 0.92987E+00_JPRB, 0.93814E+00_JPRB, 0.94730E+00_JPRB, 0.96657E+00_JPRB ,&
&  0.91637E+00_JPRB, 0.88555E+00_JPRB, 0.87106E+00_JPRB, 0.85307E+00_JPRB, 0.85444E+00_JPRB, 0.85143E+00_JPRB, 0.85801E+00_JPRB ,&
&  0.88290E+00_JPRB, 0.91883E+00_JPRB, 0.99579E+00_JPRB, 0.11154E+01_JPRB, 0.13057E+01_JPRB, 0.16835E+01_JPRB, 0.22516E+01_JPRB ,&
&  0.31705E+01_JPRB, 0.21055E+01_JPRB, 0.78881E+00_JPRB, 0.10120E+01_JPRB, 0.10977E+01_JPRB, 0.11306E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.11118E+01_JPRB, 0.11129E+01_JPRB, 0.12592E+01_JPRB, 0.13653E+01_JPRB, 0.14628E+01_JPRB, 0.15236E+01_JPRB ,&
&  0.15553E+01_JPRB, 0.16111E+01_JPRB, 0.16584E+01_JPRB, 0.17034E+01_JPRB, 0.17513E+01_JPRB, 0.18209E+01_JPRB, 0.19379E+01_JPRB ,&
&  0.21598E+01_JPRB, 0.24177E+01_JPRB, 0.26536E+01_JPRB, 0.23523E+01_JPRB, 0.23414E+01_JPRB, 0.18777E+01_JPRB, 0.16206E+01_JPRB ,&
&  0.14126E+01_JPRB, 0.14165E+01_JPRB, 0.14021E+01_JPRB, 0.12475E+01_JPRB, 0.10710E+01_JPRB, 0.95137E+00_JPRB, 0.10281E+01_JPRB ,&
&  0.91254E+00_JPRB, 0.84880E+00_JPRB, 0.81847E+00_JPRB, 0.83785E+00_JPRB, 0.87015E+00_JPRB, 0.88720E+00_JPRB, 0.89766E+00_JPRB ,&
&  0.92221E+00_JPRB, 0.93148E+00_JPRB, 0.97492E+00_JPRB, 0.10058E+01_JPRB, 0.10337E+01_JPRB, 0.10865E+01_JPRB, 0.11008E+01_JPRB ,&
&  0.11045E+01_JPRB, 0.11086E+01_JPRB, 0.11260E+01_JPRB, 0.11440E+01_JPRB, 0.10873E+01_JPRB, 0.99698E+00_JPRB, 0.97963E+00_JPRB ,&
&  0.96671E+00_JPRB, 0.94603E+00_JPRB, 0.88653E+00_JPRB, 0.88784E+00_JPRB, 0.88808E+00_JPRB, 0.94663E+00_JPRB, 0.10201E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.10283E+01_JPRB, 0.96078E+00_JPRB, 0.83478E+00_JPRB, 0.82205E+00_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.70825E+00_JPRB, 0.67720E+00_JPRB, 0.63796E+00_JPRB, 0.63154E+00_JPRB, 0.68707E+00_JPRB, 0.73495E+00_JPRB, 0.73716E+00_JPRB ,&
&  0.74544E+00_JPRB, 0.81007E+00_JPRB, 0.92649E+00_JPRB, 0.10184E+01_JPRB, 0.10484E+01_JPRB, 0.10402E+01_JPRB, 0.94660E+00_JPRB ,&
&  0.91123E+00_JPRB, 0.89009E+00_JPRB, 0.86656E+00_JPRB, 0.85811E+00_JPRB, 0.85995E+00_JPRB, 0.87324E+00_JPRB, 0.90473E+00_JPRB ,&
&  0.95201E+00_JPRB, 0.10324E+01_JPRB, 0.11610E+01_JPRB, 0.13422E+01_JPRB, 0.16048E+01_JPRB, 0.19068E+01_JPRB, 0.21633E+01_JPRB ,&
&  0.16256E+01_JPRB, 0.70273E+00_JPRB, 0.79941E+00_JPRB, 0.10513E+01_JPRB, 0.99066E+00_JPRB, 0.10139E+01_JPRB, 0.99704E+00_JPRB ,&
&  0.10434E+01_JPRB, 0.11118E+01_JPRB, 0.11860E+01_JPRB, 0.13556E+01_JPRB, 0.14746E+01_JPRB, 0.16083E+01_JPRB, 0.16664E+01_JPRB ,&
&  0.17281E+01_JPRB, 0.18067E+01_JPRB, 0.18944E+01_JPRB, 0.19834E+01_JPRB, 0.20822E+01_JPRB, 0.21914E+01_JPRB, 0.23666E+01_JPRB ,&
&  0.25859E+01_JPRB, 0.27937E+01_JPRB, 0.29651E+01_JPRB, 0.26029E+01_JPRB, 0.27157E+01_JPRB, 0.22282E+01_JPRB, 0.17732E+01_JPRB ,&
&  0.14610E+01_JPRB, 0.12150E+01_JPRB, 0.11408E+01_JPRB, 0.12079E+01_JPRB, 0.10825E+01_JPRB, 0.87962E+00_JPRB, 0.90612E+00_JPRB ,&
&  0.84174E+00_JPRB, 0.82389E+00_JPRB, 0.82418E+00_JPRB, 0.77480E+00_JPRB, 0.83400E+00_JPRB, 0.86439E+00_JPRB, 0.88837E+00_JPRB ,&
&  0.87938E+00_JPRB, 0.90062E+00_JPRB, 0.92439E+00_JPRB, 0.96586E+00_JPRB, 0.96289E+00_JPRB, 0.97050E+00_JPRB, 0.95993E+00_JPRB ,&
&  0.97119E+00_JPRB, 0.98296E+00_JPRB, 0.10053E+01_JPRB, 0.10290E+01_JPRB, 0.10288E+01_JPRB, 0.94634E+00_JPRB, 0.88430E+00_JPRB ,&
&  0.84998E+00_JPRB, 0.83024E+00_JPRB, 0.78790E+00_JPRB, 0.82696E+00_JPRB, 0.83509E+00_JPRB, 0.84551E+00_JPRB, 0.87697E+00_JPRB ,&
&  0.89224E+00_JPRB, 0.89509E+00_JPRB, 0.84637E+00_JPRB, 0.79058E+00_JPRB, 0.75338E+00_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.59359E+00_JPRB, 0.57935E+00_JPRB, 0.56197E+00_JPRB, 0.57799E+00_JPRB, 0.61138E+00_JPRB, 0.65348E+00_JPRB, 0.67208E+00_JPRB ,&
&  0.69614E+00_JPRB, 0.79683E+00_JPRB, 0.92958E+00_JPRB, 0.10470E+01_JPRB, 0.10677E+01_JPRB, 0.10366E+01_JPRB, 0.93902E+00_JPRB ,&
&  0.91424E+00_JPRB, 0.89085E+00_JPRB, 0.87980E+00_JPRB, 0.86872E+00_JPRB, 0.87871E+00_JPRB, 0.91923E+00_JPRB, 0.98519E+00_JPRB ,&
&  0.10696E+01_JPRB, 0.11609E+01_JPRB, 0.12640E+01_JPRB, 0.13748E+01_JPRB, 0.14727E+01_JPRB, 0.15100E+01_JPRB, 0.14132E+01_JPRB ,&
&  0.60466E+00_JPRB, 0.70368E+00_JPRB, 0.95931E+00_JPRB, 0.94464E+00_JPRB, 0.90701E+00_JPRB, 0.90867E+00_JPRB, 0.98541E+00_JPRB ,&
&  0.10680E+01_JPRB, 0.10895E+01_JPRB, 0.12114E+01_JPRB, 0.14171E+01_JPRB, 0.14938E+01_JPRB, 0.16605E+01_JPRB, 0.17489E+01_JPRB ,&
&  0.18347E+01_JPRB, 0.19355E+01_JPRB, 0.20615E+01_JPRB, 0.21816E+01_JPRB, 0.22984E+01_JPRB, 0.24096E+01_JPRB, 0.25660E+01_JPRB ,&
&  0.27848E+01_JPRB, 0.30214E+01_JPRB, 0.32229E+01_JPRB, 0.29433E+01_JPRB, 0.31185E+01_JPRB, 0.27806E+01_JPRB, 0.24274E+01_JPRB ,&
&  0.15709E+01_JPRB, 0.10994E+01_JPRB, 0.11132E+01_JPRB, 0.12040E+01_JPRB, 0.10897E+01_JPRB, 0.99549E+00_JPRB, 0.90893E+00_JPRB ,&
&  0.85291E+00_JPRB, 0.79075E+00_JPRB, 0.81077E+00_JPRB, 0.83344E+00_JPRB, 0.82888E+00_JPRB, 0.86978E+00_JPRB, 0.83838E+00_JPRB ,&
&  0.85157E+00_JPRB, 0.86205E+00_JPRB, 0.85480E+00_JPRB, 0.86117E+00_JPRB, 0.86747E+00_JPRB, 0.87947E+00_JPRB, 0.87000E+00_JPRB ,&
&  0.87123E+00_JPRB, 0.86642E+00_JPRB, 0.87892E+00_JPRB, 0.89371E+00_JPRB, 0.88844E+00_JPRB, 0.86998E+00_JPRB, 0.81875E+00_JPRB ,&
&  0.76473E+00_JPRB, 0.77348E+00_JPRB, 0.87174E+00_JPRB, 0.99989E+00_JPRB, 0.96231E+00_JPRB, 0.83059E+00_JPRB, 0.65972E+00_JPRB ,&
&  0.64624E+00_JPRB, 0.66781E+00_JPRB, 0.66992E+00_JPRB, 0.64961E+00_JPRB, 0.62438E+00_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.48551E+00_JPRB, 0.47361E+00_JPRB, 0.47227E+00_JPRB, 0.48242E+00_JPRB, 0.48884E+00_JPRB, 0.51348E+00_JPRB, 0.54427E+00_JPRB ,&
&  0.56018E+00_JPRB, 0.63031E+00_JPRB, 0.78663E+00_JPRB, 0.92043E+00_JPRB, 0.97680E+00_JPRB, 0.10179E+01_JPRB, 0.97371E+00_JPRB ,&
&  0.96391E+00_JPRB, 0.93721E+00_JPRB, 0.89322E+00_JPRB, 0.89183E+00_JPRB, 0.91369E+00_JPRB, 0.95534E+00_JPRB, 0.10101E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.11367E+01_JPRB, 0.11919E+01_JPRB, 0.12432E+01_JPRB, 0.13159E+01_JPRB, 0.13132E+01_JPRB, 0.12671E+01_JPRB ,&
&  0.62917E+00_JPRB, 0.96852E+00_JPRB, 0.96017E+00_JPRB, 0.90329E+00_JPRB, 0.84639E+00_JPRB, 0.84981E+00_JPRB, 0.96867E+00_JPRB ,&
&  0.10778E+01_JPRB, 0.10943E+01_JPRB, 0.12418E+01_JPRB, 0.12094E+01_JPRB, 0.14545E+01_JPRB, 0.15843E+01_JPRB, 0.17219E+01_JPRB ,&
&  0.18465E+01_JPRB, 0.19414E+01_JPRB, 0.20528E+01_JPRB, 0.21629E+01_JPRB, 0.22773E+01_JPRB, 0.23704E+01_JPRB, 0.24886E+01_JPRB ,&
&  0.26315E+01_JPRB, 0.28337E+01_JPRB, 0.30200E+01_JPRB, 0.29000E+01_JPRB, 0.25159E+01_JPRB, 0.23807E+01_JPRB, 0.19760E+01_JPRB ,&
&  0.14488E+01_JPRB, 0.10451E+01_JPRB, 0.11104E+01_JPRB, 0.11652E+01_JPRB, 0.11165E+01_JPRB, 0.10486E+01_JPRB, 0.97497E+00_JPRB ,&
&  0.91918E+00_JPRB, 0.86639E+00_JPRB, 0.79764E+00_JPRB, 0.80234E+00_JPRB, 0.82519E+00_JPRB, 0.82362E+00_JPRB, 0.80793E+00_JPRB ,&
&  0.80768E+00_JPRB, 0.78648E+00_JPRB, 0.76477E+00_JPRB, 0.74580E+00_JPRB, 0.75691E+00_JPRB, 0.80811E+00_JPRB, 0.84490E+00_JPRB ,&
&  0.86728E+00_JPRB, 0.79689E+00_JPRB, 0.74730E+00_JPRB, 0.74963E+00_JPRB, 0.74857E+00_JPRB, 0.75354E+00_JPRB, 0.74172E+00_JPRB ,&
&  0.78403E+00_JPRB, 0.94054E+00_JPRB, 0.13326E+01_JPRB, 0.17252E+01_JPRB, 0.14026E+01_JPRB, 0.11063E+01_JPRB, 0.63246E+00_JPRB ,&
&  0.50610E+00_JPRB, 0.48900E+00_JPRB, 0.50667E+00_JPRB, 0.50993E+00_JPRB, 0.50875E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.43551E+00_JPRB, 0.42675E+00_JPRB, 0.43444E+00_JPRB, 0.43252E+00_JPRB, 0.43609E+00_JPRB, 0.44319E+00_JPRB, 0.44757E+00_JPRB ,&
&  0.45126E+00_JPRB, 0.46802E+00_JPRB, 0.55756E+00_JPRB, 0.68088E+00_JPRB, 0.79795E+00_JPRB, 0.88707E+00_JPRB, 0.92692E+00_JPRB ,&
&  0.96548E+00_JPRB, 0.98147E+00_JPRB, 0.97092E+00_JPRB, 0.95446E+00_JPRB, 0.96432E+00_JPRB, 0.10089E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.11096E+01_JPRB, 0.11454E+01_JPRB, 0.12029E+01_JPRB, 0.12618E+01_JPRB, 0.12177E+01_JPRB, 0.76491E+00_JPRB ,&
&  0.98320E+00_JPRB, 0.10113E+01_JPRB, 0.10098E+01_JPRB, 0.93270E+00_JPRB, 0.86235E+00_JPRB, 0.82194E+00_JPRB, 0.86897E+00_JPRB ,&
&  0.92813E+00_JPRB, 0.95763E+00_JPRB, 0.95626E+00_JPRB, 0.99237E+00_JPRB, 0.12994E+01_JPRB, 0.13211E+01_JPRB, 0.14762E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.17156E+01_JPRB, 0.18019E+01_JPRB, 0.19074E+01_JPRB, 0.20014E+01_JPRB, 0.20657E+01_JPRB, 0.21530E+01_JPRB ,&
&  0.21818E+01_JPRB, 0.22247E+01_JPRB, 0.22046E+01_JPRB, 0.21273E+01_JPRB, 0.18300E+01_JPRB, 0.19323E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.10952E+01_JPRB, 0.99858E+00_JPRB, 0.11173E+01_JPRB, 0.11985E+01_JPRB, 0.11881E+01_JPRB, 0.11121E+01_JPRB, 0.99674E+00_JPRB ,&
&  0.93681E+00_JPRB, 0.89446E+00_JPRB, 0.85233E+00_JPRB, 0.80381E+00_JPRB, 0.79138E+00_JPRB, 0.77311E+00_JPRB, 0.75378E+00_JPRB ,&
&  0.74203E+00_JPRB, 0.70837E+00_JPRB, 0.67196E+00_JPRB, 0.67865E+00_JPRB, 0.72127E+00_JPRB, 0.86720E+00_JPRB, 0.11378E+01_JPRB ,&
&  0.12423E+01_JPRB, 0.72846E+00_JPRB, 0.66877E+00_JPRB, 0.61854E+00_JPRB, 0.60618E+00_JPRB, 0.63927E+00_JPRB, 0.71869E+00_JPRB ,&
&  0.92690E+00_JPRB, 0.16450E+01_JPRB, 0.25361E+01_JPRB, 0.19296E+01_JPRB, 0.22049E+01_JPRB, 0.15152E+01_JPRB, 0.61688E+00_JPRB ,&
&  0.48106E+00_JPRB, 0.46930E+00_JPRB, 0.47398E+00_JPRB, 0.46405E+00_JPRB, 0.45112E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.44856E+00_JPRB, 0.43896E+00_JPRB, 0.44657E+00_JPRB, 0.45594E+00_JPRB, 0.45187E+00_JPRB, 0.45146E+00_JPRB, 0.44654E+00_JPRB ,&
&  0.43725E+00_JPRB, 0.43122E+00_JPRB, 0.45322E+00_JPRB, 0.49487E+00_JPRB, 0.55280E+00_JPRB, 0.62793E+00_JPRB, 0.70176E+00_JPRB ,&
&  0.74969E+00_JPRB, 0.80431E+00_JPRB, 0.85118E+00_JPRB, 0.88861E+00_JPRB, 0.89057E+00_JPRB, 0.89007E+00_JPRB, 0.87386E+00_JPRB ,&
&  0.86351E+00_JPRB, 0.87825E+00_JPRB, 0.92600E+00_JPRB, 0.97301E+00_JPRB, 0.10579E+01_JPRB, 0.11812E+01_JPRB, 0.77323E+00_JPRB ,&
&  0.11357E+01_JPRB, 0.11188E+01_JPRB, 0.10593E+01_JPRB, 0.96437E+00_JPRB, 0.91349E+00_JPRB, 0.83692E+00_JPRB, 0.77522E+00_JPRB ,&
&  0.77372E+00_JPRB, 0.80757E+00_JPRB, 0.81116E+00_JPRB, 0.89122E+00_JPRB, 0.97144E+00_JPRB, 0.98924E+00_JPRB, 0.11024E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.12746E+01_JPRB, 0.13664E+01_JPRB, 0.14340E+01_JPRB, 0.14920E+01_JPRB, 0.15099E+01_JPRB, 0.14870E+01_JPRB ,&
&  0.13918E+01_JPRB, 0.13182E+01_JPRB, 0.12934E+01_JPRB, 0.13856E+01_JPRB, 0.12860E+01_JPRB, 0.12216E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.86514E+00_JPRB, 0.84121E+00_JPRB, 0.96852E+00_JPRB, 0.12517E+01_JPRB, 0.13005E+01_JPRB, 0.12473E+01_JPRB, 0.11550E+01_JPRB ,&
&  0.10761E+01_JPRB, 0.99854E+00_JPRB, 0.93055E+00_JPRB, 0.86461E+00_JPRB, 0.81511E+00_JPRB, 0.77515E+00_JPRB, 0.73739E+00_JPRB ,&
&  0.71034E+00_JPRB, 0.67455E+00_JPRB, 0.63671E+00_JPRB, 0.63463E+00_JPRB, 0.65768E+00_JPRB, 0.74545E+00_JPRB, 0.79116E+00_JPRB ,&
&  0.88695E+00_JPRB, 0.67065E+00_JPRB, 0.59160E+00_JPRB, 0.57064E+00_JPRB, 0.49044E+00_JPRB, 0.53324E+00_JPRB, 0.70140E+00_JPRB ,&
&  0.11083E+01_JPRB, 0.16455E+01_JPRB, 0.27903E+01_JPRB, 0.50191E+01_JPRB, 0.27780E+01_JPRB, 0.21426E+01_JPRB, 0.51864E+00_JPRB ,&
&  0.49651E+00_JPRB, 0.49778E+00_JPRB, 0.49439E+00_JPRB, 0.47743E+00_JPRB, 0.47243E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.49464E+00_JPRB, 0.49262E+00_JPRB, 0.50019E+00_JPRB, 0.50796E+00_JPRB, 0.50963E+00_JPRB, 0.51130E+00_JPRB, 0.51275E+00_JPRB ,&
&  0.51572E+00_JPRB, 0.51579E+00_JPRB, 0.51810E+00_JPRB, 0.52549E+00_JPRB, 0.54144E+00_JPRB, 0.57617E+00_JPRB, 0.61656E+00_JPRB ,&
&  0.65214E+00_JPRB, 0.68532E+00_JPRB, 0.71753E+00_JPRB, 0.74490E+00_JPRB, 0.77345E+00_JPRB, 0.79222E+00_JPRB, 0.84670E+00_JPRB ,&
&  0.91609E+00_JPRB, 0.10006E+01_JPRB, 0.10868E+01_JPRB, 0.11503E+01_JPRB, 0.12052E+01_JPRB, 0.11785E+01_JPRB, 0.97189E+00_JPRB ,&
&  0.13906E+01_JPRB, 0.13253E+01_JPRB, 0.11970E+01_JPRB, 0.10747E+01_JPRB, 0.98804E+00_JPRB, 0.90929E+00_JPRB, 0.83550E+00_JPRB ,&
&  0.77123E+00_JPRB, 0.79832E+00_JPRB, 0.75580E+00_JPRB, 0.79083E+00_JPRB, 0.69908E+00_JPRB, 0.72767E+00_JPRB, 0.83578E+00_JPRB ,&
&  0.92585E+00_JPRB, 0.10081E+01_JPRB, 0.11003E+01_JPRB, 0.12027E+01_JPRB, 0.12570E+01_JPRB, 0.12240E+01_JPRB, 0.10928E+01_JPRB ,&
&  0.10169E+01_JPRB, 0.97563E+00_JPRB, 0.91781E+00_JPRB, 0.96763E+00_JPRB, 0.97204E+00_JPRB, 0.95417E+00_JPRB, 0.96081E+00_JPRB ,&
&  0.86551E+00_JPRB, 0.90039E+00_JPRB, 0.10565E+01_JPRB, 0.13858E+01_JPRB, 0.13479E+01_JPRB, 0.12754E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.10744E+01_JPRB, 0.10140E+01_JPRB, 0.94007E+00_JPRB, 0.87093E+00_JPRB, 0.81948E+00_JPRB, 0.77694E+00_JPRB ,&
&  0.73592E+00_JPRB, 0.68708E+00_JPRB, 0.65106E+00_JPRB, 0.63833E+00_JPRB, 0.67238E+00_JPRB, 0.71127E+00_JPRB, 0.11600E+01_JPRB ,&
&  0.82594E+00_JPRB, 0.65341E+00_JPRB, 0.68310E+00_JPRB, 0.41067E+00_JPRB, 0.50042E+00_JPRB, 0.54307E+00_JPRB, 0.65529E+00_JPRB ,&
&  0.81437E+00_JPRB, 0.12975E+01_JPRB, 0.20230E+01_JPRB, 0.18242E+01_JPRB, 0.13924E+01_JPRB, 0.89526E+00_JPRB, 0.53691E+00_JPRB ,&
&  0.51598E+00_JPRB, 0.51099E+00_JPRB, 0.51535E+00_JPRB, 0.50633E+00_JPRB, 0.50272E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.52793E+00_JPRB, 0.52703E+00_JPRB, 0.54066E+00_JPRB, 0.55403E+00_JPRB, 0.56180E+00_JPRB, 0.56812E+00_JPRB, 0.57842E+00_JPRB ,&
&  0.58468E+00_JPRB, 0.59330E+00_JPRB, 0.60293E+00_JPRB, 0.62017E+00_JPRB, 0.64545E+00_JPRB, 0.68421E+00_JPRB, 0.72896E+00_JPRB ,&
&  0.78372E+00_JPRB, 0.84274E+00_JPRB, 0.90863E+00_JPRB, 0.97968E+00_JPRB, 0.10679E+01_JPRB, 0.11896E+01_JPRB, 0.13560E+01_JPRB ,&
&  0.14471E+01_JPRB, 0.15625E+01_JPRB, 0.17018E+01_JPRB, 0.18211E+01_JPRB, 0.18385E+01_JPRB, 0.17945E+01_JPRB, 0.14649E+01_JPRB ,&
&  0.16049E+01_JPRB, 0.14609E+01_JPRB, 0.13185E+01_JPRB, 0.11904E+01_JPRB, 0.11001E+01_JPRB, 0.10228E+01_JPRB, 0.96407E+00_JPRB ,&
&  0.94265E+00_JPRB, 0.89955E+00_JPRB, 0.84311E+00_JPRB, 0.80210E+00_JPRB, 0.77533E+00_JPRB, 0.78757E+00_JPRB, 0.76378E+00_JPRB ,&
&  0.82730E+00_JPRB, 0.97498E+00_JPRB, 0.11645E+01_JPRB, 0.13989E+01_JPRB, 0.15277E+01_JPRB, 0.14968E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.11232E+01_JPRB, 0.10469E+01_JPRB, 0.10199E+01_JPRB, 0.10482E+01_JPRB, 0.10727E+01_JPRB, 0.11324E+01_JPRB, 0.11618E+01_JPRB ,&
&  0.11454E+01_JPRB, 0.10783E+01_JPRB, 0.11486E+01_JPRB, 0.14400E+01_JPRB, 0.13850E+01_JPRB, 0.12466E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.10728E+01_JPRB, 0.10340E+01_JPRB, 0.96621E+00_JPRB, 0.89017E+00_JPRB, 0.83548E+00_JPRB, 0.81169E+00_JPRB ,&
&  0.76190E+00_JPRB, 0.70718E+00_JPRB, 0.67634E+00_JPRB, 0.70679E+00_JPRB, 0.84297E+00_JPRB, 0.10026E+01_JPRB, 0.14857E+01_JPRB ,&
&  0.84611E+00_JPRB, 0.55438E+00_JPRB, 0.58228E+00_JPRB, 0.67124E+00_JPRB, 0.61400E+00_JPRB, 0.61115E+00_JPRB, 0.66407E+00_JPRB ,&
&  0.73070E+00_JPRB, 0.79754E+00_JPRB, 0.79283E+00_JPRB, 0.68351E+00_JPRB, 0.64064E+00_JPRB, 0.61966E+00_JPRB, 0.59472E+00_JPRB ,&
&  0.57439E+00_JPRB, 0.53753E+00_JPRB, 0.55057E+00_JPRB, 0.53455E+00_JPRB, 0.53192E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.57846E+00_JPRB, 0.57056E+00_JPRB, 0.56865E+00_JPRB, 0.56614E+00_JPRB, 0.57608E+00_JPRB, 0.57919E+00_JPRB, 0.58143E+00_JPRB ,&
&  0.58230E+00_JPRB, 0.59426E+00_JPRB, 0.60357E+00_JPRB, 0.61649E+00_JPRB, 0.63083E+00_JPRB, 0.66304E+00_JPRB, 0.70081E+00_JPRB ,&
&  0.74629E+00_JPRB, 0.79904E+00_JPRB, 0.85768E+00_JPRB, 0.94150E+00_JPRB, 0.10593E+01_JPRB, 0.11432E+01_JPRB, 0.12342E+01_JPRB ,&
&  0.13485E+01_JPRB, 0.15171E+01_JPRB, 0.16987E+01_JPRB, 0.19410E+01_JPRB, 0.22227E+01_JPRB, 0.22668E+01_JPRB, 0.21051E+01_JPRB ,&
&  0.15083E+01_JPRB, 0.14621E+01_JPRB, 0.13617E+01_JPRB, 0.12362E+01_JPRB, 0.12489E+01_JPRB, 0.12149E+01_JPRB, 0.12297E+01_JPRB ,&
&  0.12623E+01_JPRB, 0.12641E+01_JPRB, 0.12473E+01_JPRB, 0.12526E+01_JPRB, 0.12574E+01_JPRB, 0.12636E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.12376E+01_JPRB, 0.12817E+01_JPRB, 0.14757E+01_JPRB, 0.17264E+01_JPRB, 0.19007E+01_JPRB, 0.19688E+01_JPRB, 0.18599E+01_JPRB ,&
&  0.17172E+01_JPRB, 0.16046E+01_JPRB, 0.15157E+01_JPRB, 0.14546E+01_JPRB, 0.14788E+01_JPRB, 0.15058E+01_JPRB, 0.16162E+01_JPRB ,&
&  0.15399E+01_JPRB, 0.13619E+01_JPRB, 0.13536E+01_JPRB, 0.13677E+01_JPRB, 0.13480E+01_JPRB, 0.12856E+01_JPRB, 0.11927E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11105E+01_JPRB, 0.10743E+01_JPRB, 0.10017E+01_JPRB, 0.93225E+00_JPRB, 0.88368E+00_JPRB, 0.85664E+00_JPRB ,&
&  0.80303E+00_JPRB, 0.76372E+00_JPRB, 0.75965E+00_JPRB, 0.81189E+00_JPRB, 0.93760E+00_JPRB, 0.14287E+01_JPRB, 0.14294E+01_JPRB ,&
&  0.90883E+00_JPRB, 0.78745E+00_JPRB, 0.64294E+00_JPRB, 0.89191E+00_JPRB, 0.71411E+00_JPRB, 0.67675E+00_JPRB, 0.67373E+00_JPRB ,&
&  0.67793E+00_JPRB, 0.67434E+00_JPRB, 0.65363E+00_JPRB, 0.64933E+00_JPRB, 0.65834E+00_JPRB, 0.66647E+00_JPRB, 0.65633E+00_JPRB ,&
&  0.62002E+00_JPRB, 0.60299E+00_JPRB, 0.62828E+00_JPRB, 0.58937E+00_JPRB, 0.58114E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.67462E+00_JPRB, 0.64430E+00_JPRB, 0.61761E+00_JPRB, 0.61953E+00_JPRB, 0.62593E+00_JPRB, 0.61445E+00_JPRB, 0.64712E+00_JPRB ,&
&  0.62426E+00_JPRB, 0.64985E+00_JPRB, 0.66491E+00_JPRB, 0.66682E+00_JPRB, 0.70990E+00_JPRB, 0.75259E+00_JPRB, 0.80698E+00_JPRB ,&
&  0.88259E+00_JPRB, 0.10037E+01_JPRB, 0.11565E+01_JPRB, 0.13470E+01_JPRB, 0.15778E+01_JPRB, 0.16905E+01_JPRB, 0.18036E+01_JPRB ,&
&  0.19609E+01_JPRB, 0.21678E+01_JPRB, 0.24591E+01_JPRB, 0.27806E+01_JPRB, 0.31464E+01_JPRB, 0.29356E+01_JPRB, 0.24528E+01_JPRB ,&
&  0.17691E+01_JPRB, 0.13980E+01_JPRB, 0.13242E+01_JPRB, 0.12300E+01_JPRB, 0.12982E+01_JPRB, 0.14458E+01_JPRB, 0.15029E+01_JPRB ,&
&  0.15433E+01_JPRB, 0.15839E+01_JPRB, 0.16236E+01_JPRB, 0.16725E+01_JPRB, 0.17326E+01_JPRB, 0.17998E+01_JPRB, 0.18268E+01_JPRB ,&
&  0.18446E+01_JPRB, 0.18645E+01_JPRB, 0.18851E+01_JPRB, 0.19770E+01_JPRB, 0.21130E+01_JPRB, 0.22770E+01_JPRB, 0.22990E+01_JPRB ,&
&  0.22990E+01_JPRB, 0.22102E+01_JPRB, 0.19554E+01_JPRB, 0.18759E+01_JPRB, 0.18899E+01_JPRB, 0.19093E+01_JPRB, 0.19562E+01_JPRB ,&
&  0.21080E+01_JPRB, 0.17607E+01_JPRB, 0.12448E+01_JPRB, 0.12960E+01_JPRB, 0.13323E+01_JPRB, 0.13672E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.11926E+01_JPRB, 0.11681E+01_JPRB, 0.11741E+01_JPRB, 0.11304E+01_JPRB, 0.11150E+01_JPRB, 0.10893E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.89106E+00_JPRB, 0.86298E+00_JPRB, 0.83968E+00_JPRB, 0.83552E+00_JPRB, 0.98676E+00_JPRB, 0.99241E+00_JPRB, 0.10523E+01_JPRB ,&
&  0.10318E+01_JPRB, 0.11507E+01_JPRB, 0.10579E+01_JPRB, 0.13585E+01_JPRB, 0.88304E+00_JPRB, 0.76322E+00_JPRB, 0.73973E+00_JPRB ,&
&  0.76020E+00_JPRB, 0.76922E+00_JPRB, 0.78226E+00_JPRB, 0.77155E+00_JPRB, 0.78688E+00_JPRB, 0.81515E+00_JPRB, 0.81602E+00_JPRB ,&
&  0.77179E+00_JPRB, 0.75563E+00_JPRB, 0.72747E+00_JPRB, 0.69358E+00_JPRB, 0.68585E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.75934E+00_JPRB, 0.76253E+00_JPRB, 0.75680E+00_JPRB, 0.75167E+00_JPRB, 0.75109E+00_JPRB, 0.73711E+00_JPRB, 0.76105E+00_JPRB ,&
&  0.76009E+00_JPRB, 0.78454E+00_JPRB, 0.81155E+00_JPRB, 0.84049E+00_JPRB, 0.89137E+00_JPRB, 0.94352E+00_JPRB, 0.10152E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.13259E+01_JPRB, 0.15717E+01_JPRB, 0.18789E+01_JPRB, 0.19847E+01_JPRB, 0.20524E+01_JPRB, 0.21653E+01_JPRB ,&
&  0.23080E+01_JPRB, 0.24238E+01_JPRB, 0.24865E+01_JPRB, 0.25619E+01_JPRB, 0.26136E+01_JPRB, 0.25080E+01_JPRB, 0.23824E+01_JPRB ,&
&  0.19223E+01_JPRB, 0.14381E+01_JPRB, 0.15007E+01_JPRB, 0.14054E+01_JPRB, 0.14599E+01_JPRB, 0.15199E+01_JPRB, 0.15661E+01_JPRB ,&
&  0.16086E+01_JPRB, 0.16336E+01_JPRB, 0.16649E+01_JPRB, 0.17090E+01_JPRB, 0.17729E+01_JPRB, 0.18286E+01_JPRB, 0.18804E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.19511E+01_JPRB, 0.19725E+01_JPRB, 0.19116E+01_JPRB, 0.19760E+01_JPRB, 0.21207E+01_JPRB, 0.22454E+01_JPRB ,&
&  0.23371E+01_JPRB, 0.23225E+01_JPRB, 0.20709E+01_JPRB, 0.20782E+01_JPRB, 0.21930E+01_JPRB, 0.20954E+01_JPRB, 0.21773E+01_JPRB ,&
&  0.23720E+01_JPRB, 0.20922E+01_JPRB, 0.12173E+01_JPRB, 0.11429E+01_JPRB, 0.12466E+01_JPRB, 0.13110E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.13222E+01_JPRB, 0.13001E+01_JPRB, 0.13054E+01_JPRB, 0.13270E+01_JPRB, 0.13148E+01_JPRB, 0.13263E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.83365E+00_JPRB, 0.81753E+00_JPRB, 0.86124E+00_JPRB, 0.90779E+00_JPRB, 0.10017E+01_JPRB, 0.10209E+01_JPRB, 0.97660E+00_JPRB ,&
&  0.10988E+01_JPRB, 0.13089E+01_JPRB, 0.16258E+01_JPRB, 0.14546E+01_JPRB, 0.97918E+00_JPRB, 0.86015E+00_JPRB, 0.84088E+00_JPRB ,&
&  0.86763E+00_JPRB, 0.86952E+00_JPRB, 0.88384E+00_JPRB, 0.89251E+00_JPRB, 0.90549E+00_JPRB, 0.89116E+00_JPRB, 0.87371E+00_JPRB ,&
&  0.85278E+00_JPRB, 0.83456E+00_JPRB, 0.81690E+00_JPRB, 0.79009E+00_JPRB, 0.76941E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.85590E+00_JPRB, 0.83361E+00_JPRB, 0.82257E+00_JPRB, 0.80735E+00_JPRB, 0.80006E+00_JPRB, 0.79051E+00_JPRB, 0.78759E+00_JPRB ,&
&  0.78993E+00_JPRB, 0.79813E+00_JPRB, 0.82374E+00_JPRB, 0.86463E+00_JPRB, 0.91865E+00_JPRB, 0.98992E+00_JPRB, 0.11108E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.14035E+01_JPRB, 0.16145E+01_JPRB, 0.18600E+01_JPRB, 0.20395E+01_JPRB, 0.20973E+01_JPRB, 0.22092E+01_JPRB ,&
&  0.22826E+01_JPRB, 0.23034E+01_JPRB, 0.22586E+01_JPRB, 0.21802E+01_JPRB, 0.20652E+01_JPRB, 0.20506E+01_JPRB, 0.21508E+01_JPRB ,&
&  0.19562E+01_JPRB, 0.15323E+01_JPRB, 0.14345E+01_JPRB, 0.14189E+01_JPRB, 0.14383E+01_JPRB, 0.14748E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.15723E+01_JPRB, 0.16058E+01_JPRB, 0.16352E+01_JPRB, 0.16690E+01_JPRB, 0.17487E+01_JPRB, 0.17806E+01_JPRB, 0.18179E+01_JPRB ,&
&  0.18497E+01_JPRB, 0.18769E+01_JPRB, 0.19306E+01_JPRB, 0.18162E+01_JPRB, 0.19400E+01_JPRB, 0.20316E+01_JPRB, 0.21069E+01_JPRB ,&
&  0.20939E+01_JPRB, 0.20208E+01_JPRB, 0.21248E+01_JPRB, 0.21742E+01_JPRB, 0.22124E+01_JPRB, 0.22032E+01_JPRB, 0.21490E+01_JPRB ,&
&  0.22242E+01_JPRB, 0.20516E+01_JPRB, 0.12788E+01_JPRB, 0.12185E+01_JPRB, 0.13370E+01_JPRB, 0.12351E+01_JPRB, 0.14330E+01_JPRB ,&
&  0.14171E+01_JPRB, 0.14156E+01_JPRB, 0.14098E+01_JPRB, 0.14548E+01_JPRB, 0.15428E+01_JPRB, 0.17891E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.94970E+00_JPRB, 0.90259E+00_JPRB, 0.91216E+00_JPRB, 0.94195E+00_JPRB, 0.99841E+00_JPRB, 0.10077E+01_JPRB, 0.10185E+01_JPRB ,&
&  0.10650E+01_JPRB, 0.11580E+01_JPRB, 0.12696E+01_JPRB, 0.12264E+01_JPRB, 0.10286E+01_JPRB, 0.91504E+00_JPRB, 0.95290E+00_JPRB ,&
&  0.98107E+00_JPRB, 0.99457E+00_JPRB, 0.98813E+00_JPRB, 0.99187E+00_JPRB, 0.98273E+00_JPRB, 0.97667E+00_JPRB, 0.96971E+00_JPRB ,&
&  0.95779E+00_JPRB, 0.95294E+00_JPRB, 0.93836E+00_JPRB, 0.90149E+00_JPRB, 0.87464E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.93901E+00_JPRB, 0.91091E+00_JPRB, 0.89151E+00_JPRB, 0.87673E+00_JPRB, 0.86995E+00_JPRB, 0.84824E+00_JPRB, 0.81722E+00_JPRB ,&
&  0.80171E+00_JPRB, 0.79852E+00_JPRB, 0.81615E+00_JPRB, 0.85424E+00_JPRB, 0.90717E+00_JPRB, 0.96118E+00_JPRB, 0.10541E+01_JPRB ,&
&  0.11665E+01_JPRB, 0.13260E+01_JPRB, 0.15214E+01_JPRB, 0.17971E+01_JPRB, 0.20224E+01_JPRB, 0.22346E+01_JPRB, 0.22934E+01_JPRB ,&
&  0.24017E+01_JPRB, 0.24200E+01_JPRB, 0.22803E+01_JPRB, 0.21021E+01_JPRB, 0.16436E+01_JPRB, 0.16150E+01_JPRB, 0.17308E+01_JPRB ,&
&  0.17982E+01_JPRB, 0.15337E+01_JPRB, 0.13553E+01_JPRB, 0.13506E+01_JPRB, 0.13483E+01_JPRB, 0.13925E+01_JPRB, 0.14358E+01_JPRB ,&
&  0.14783E+01_JPRB, 0.15178E+01_JPRB, 0.15566E+01_JPRB, 0.16160E+01_JPRB, 0.16823E+01_JPRB, 0.17120E+01_JPRB, 0.17495E+01_JPRB ,&
&  0.17570E+01_JPRB, 0.17677E+01_JPRB, 0.17665E+01_JPRB, 0.17369E+01_JPRB, 0.17416E+01_JPRB, 0.17805E+01_JPRB, 0.18435E+01_JPRB ,&
&  0.18984E+01_JPRB, 0.18167E+01_JPRB, 0.18911E+01_JPRB, 0.19900E+01_JPRB, 0.19427E+01_JPRB, 0.18514E+01_JPRB, 0.18522E+01_JPRB ,&
&  0.19931E+01_JPRB, 0.21134E+01_JPRB, 0.13310E+01_JPRB, 0.14990E+01_JPRB, 0.17656E+01_JPRB, 0.15701E+01_JPRB, 0.14731E+01_JPRB ,&
&  0.14617E+01_JPRB, 0.14497E+01_JPRB, 0.14561E+01_JPRB, 0.14909E+01_JPRB, 0.16333E+01_JPRB, 0.19273E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11017E+01_JPRB, 0.11402E+01_JPRB, 0.11773E+01_JPRB, 0.11829E+01_JPRB, 0.10858E+01_JPRB, 0.96861E+00_JPRB ,&
&  0.94298E+00_JPRB, 0.13230E+01_JPRB, 0.15087E+01_JPRB, 0.16710E+01_JPRB, 0.14995E+01_JPRB, 0.95549E+00_JPRB, 0.10442E+01_JPRB ,&
&  0.10791E+01_JPRB, 0.10897E+01_JPRB, 0.10617E+01_JPRB, 0.10524E+01_JPRB, 0.10448E+01_JPRB, 0.10336E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10250E+01_JPRB, 0.10259E+01_JPRB, 0.10299E+01_JPRB, 0.10068E+01_JPRB, 0.96896E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.10216E+01_JPRB, 0.99968E+00_JPRB, 0.97825E+00_JPRB, 0.95778E+00_JPRB, 0.94064E+00_JPRB, 0.89735E+00_JPRB, 0.84804E+00_JPRB ,&
&  0.81691E+00_JPRB, 0.80276E+00_JPRB, 0.80759E+00_JPRB, 0.82252E+00_JPRB, 0.85743E+00_JPRB, 0.89718E+00_JPRB, 0.96509E+00_JPRB ,&
&  0.10686E+01_JPRB, 0.12120E+01_JPRB, 0.14020E+01_JPRB, 0.16252E+01_JPRB, 0.18964E+01_JPRB, 0.22156E+01_JPRB, 0.24511E+01_JPRB ,&
&  0.26821E+01_JPRB, 0.27204E+01_JPRB, 0.23399E+01_JPRB, 0.16895E+01_JPRB, 0.15593E+01_JPRB, 0.16006E+01_JPRB, 0.14097E+01_JPRB ,&
&  0.13663E+01_JPRB, 0.13172E+01_JPRB, 0.12757E+01_JPRB, 0.12620E+01_JPRB, 0.12686E+01_JPRB, 0.13014E+01_JPRB, 0.13379E+01_JPRB ,&
&  0.13682E+01_JPRB, 0.13906E+01_JPRB, 0.14208E+01_JPRB, 0.14868E+01_JPRB, 0.15415E+01_JPRB, 0.16044E+01_JPRB, 0.16615E+01_JPRB ,&
&  0.16942E+01_JPRB, 0.17195E+01_JPRB, 0.16930E+01_JPRB, 0.16483E+01_JPRB, 0.15971E+01_JPRB, 0.16133E+01_JPRB, 0.16376E+01_JPRB ,&
&  0.16623E+01_JPRB, 0.16576E+01_JPRB, 0.16574E+01_JPRB, 0.17359E+01_JPRB, 0.17821E+01_JPRB, 0.16015E+01_JPRB, 0.16398E+01_JPRB ,&
&  0.18329E+01_JPRB, 0.19506E+01_JPRB, 0.15654E+01_JPRB, 0.18155E+01_JPRB, 0.11609E+01_JPRB, 0.14485E+01_JPRB, 0.16139E+01_JPRB ,&
&  0.15131E+01_JPRB, 0.14731E+01_JPRB, 0.14309E+01_JPRB, 0.14961E+01_JPRB, 0.16501E+01_JPRB, 0.17254E+01_JPRB, 0.15591E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.12695E+01_JPRB, 0.13461E+01_JPRB, 0.14289E+01_JPRB, 0.14494E+01_JPRB, 0.13453E+01_JPRB, 0.11527E+01_JPRB ,&
&  0.10407E+01_JPRB, 0.13027E+01_JPRB, 0.13735E+01_JPRB, 0.14258E+01_JPRB, 0.11361E+01_JPRB, 0.96177E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.10693E+01_JPRB, 0.10972E+01_JPRB, 0.10993E+01_JPRB, 0.10849E+01_JPRB, 0.10617E+01_JPRB, 0.10416E+01_JPRB, 0.10340E+01_JPRB ,&
&  0.10313E+01_JPRB, 0.10245E+01_JPRB, 0.10269E+01_JPRB, 0.10344E+01_JPRB, 0.10359E+01_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.10449E+01_JPRB, 0.10380E+01_JPRB, 0.10349E+01_JPRB, 0.10337E+01_JPRB, 0.10266E+01_JPRB, 0.99714E+00_JPRB, 0.93448E+00_JPRB ,&
&  0.84623E+00_JPRB, 0.77911E+00_JPRB, 0.76777E+00_JPRB, 0.76646E+00_JPRB, 0.77714E+00_JPRB, 0.81358E+00_JPRB, 0.87928E+00_JPRB ,&
&  0.95784E+00_JPRB, 0.10891E+01_JPRB, 0.12682E+01_JPRB, 0.14967E+01_JPRB, 0.17748E+01_JPRB, 0.22015E+01_JPRB, 0.26905E+01_JPRB ,&
&  0.26851E+01_JPRB, 0.20030E+01_JPRB, 0.23551E+01_JPRB, 0.23013E+01_JPRB, 0.21180E+01_JPRB, 0.19828E+01_JPRB, 0.17248E+01_JPRB ,&
&  0.14663E+01_JPRB, 0.13778E+01_JPRB, 0.13439E+01_JPRB, 0.12822E+01_JPRB, 0.12008E+01_JPRB, 0.12249E+01_JPRB, 0.12548E+01_JPRB ,&
&  0.12874E+01_JPRB, 0.13156E+01_JPRB, 0.13276E+01_JPRB, 0.13872E+01_JPRB, 0.14274E+01_JPRB, 0.14801E+01_JPRB, 0.15522E+01_JPRB ,&
&  0.15909E+01_JPRB, 0.15973E+01_JPRB, 0.15863E+01_JPRB, 0.14939E+01_JPRB, 0.14851E+01_JPRB, 0.15134E+01_JPRB, 0.15267E+01_JPRB ,&
&  0.15525E+01_JPRB, 0.15472E+01_JPRB, 0.16075E+01_JPRB, 0.16913E+01_JPRB, 0.17807E+01_JPRB, 0.16156E+01_JPRB, 0.17359E+01_JPRB ,&
&  0.17789E+01_JPRB, 0.19163E+01_JPRB, 0.18359E+01_JPRB, 0.20475E+01_JPRB, 0.14349E+01_JPRB, 0.16038E+01_JPRB, 0.15440E+01_JPRB ,&
&  0.16652E+01_JPRB, 0.15362E+01_JPRB, 0.14518E+01_JPRB, 0.14634E+01_JPRB, 0.15958E+01_JPRB, 0.17807E+01_JPRB, 0.18085E+01_JPRB ,&
&  0.14590E+01_JPRB, 0.13919E+01_JPRB, 0.14602E+01_JPRB, 0.16084E+01_JPRB, 0.16314E+01_JPRB, 0.15897E+01_JPRB, 0.14960E+01_JPRB ,&
&  0.16562E+01_JPRB, 0.15759E+01_JPRB, 0.15153E+01_JPRB, 0.14062E+01_JPRB, 0.11582E+01_JPRB, 0.10315E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10666E+01_JPRB, 0.10697E+01_JPRB, 0.10627E+01_JPRB, 0.10397E+01_JPRB, 0.10175E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.99541E+00_JPRB, 0.99959E+00_JPRB, 0.10054E+01_JPRB, 0.10133E+01_JPRB, 0.10303E+01_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.98738E+00_JPRB, 0.10013E+01_JPRB, 0.10117E+01_JPRB, 0.10466E+01_JPRB, 0.10965E+01_JPRB, 0.11651E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.10540E+01_JPRB, 0.77059E+00_JPRB, 0.75505E+00_JPRB, 0.74322E+00_JPRB, 0.73450E+00_JPRB, 0.74522E+00_JPRB, 0.79754E+00_JPRB ,&
&  0.87976E+00_JPRB, 0.99543E+00_JPRB, 0.11366E+01_JPRB, 0.13325E+01_JPRB, 0.15857E+01_JPRB, 0.20170E+01_JPRB, 0.24045E+01_JPRB ,&
&  0.23331E+01_JPRB, 0.26380E+01_JPRB, 0.29523E+01_JPRB, 0.26089E+01_JPRB, 0.23437E+01_JPRB, 0.22024E+01_JPRB, 0.20810E+01_JPRB ,&
&  0.16964E+01_JPRB, 0.15235E+01_JPRB, 0.14747E+01_JPRB, 0.14320E+01_JPRB, 0.12394E+01_JPRB, 0.12159E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.12459E+01_JPRB, 0.12638E+01_JPRB, 0.13304E+01_JPRB, 0.13955E+01_JPRB, 0.14450E+01_JPRB, 0.14904E+01_JPRB ,&
&  0.15256E+01_JPRB, 0.15286E+01_JPRB, 0.15187E+01_JPRB, 0.14239E+01_JPRB, 0.14511E+01_JPRB, 0.14974E+01_JPRB, 0.15442E+01_JPRB ,&
&  0.15921E+01_JPRB, 0.16962E+01_JPRB, 0.17522E+01_JPRB, 0.17099E+01_JPRB, 0.15939E+01_JPRB, 0.17710E+01_JPRB, 0.17021E+01_JPRB ,&
&  0.17604E+01_JPRB, 0.18019E+01_JPRB, 0.22357E+01_JPRB, 0.16701E+01_JPRB, 0.17629E+01_JPRB, 0.18902E+01_JPRB, 0.18880E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.17395E+01_JPRB, 0.15895E+01_JPRB, 0.14870E+01_JPRB, 0.15347E+01_JPRB, 0.16315E+01_JPRB, 0.19553E+01_JPRB ,&
&  0.17102E+01_JPRB, 0.17842E+01_JPRB, 0.16098E+01_JPRB, 0.16232E+01_JPRB, 0.15946E+01_JPRB, 0.15627E+01_JPRB, 0.16547E+01_JPRB ,&
&  0.21671E+01_JPRB, 0.20938E+01_JPRB, 0.19314E+01_JPRB, 0.17440E+01_JPRB, 0.14737E+01_JPRB, 0.13387E+01_JPRB, 0.12091E+01_JPRB ,&
&  0.11460E+01_JPRB, 0.11187E+01_JPRB, 0.11094E+01_JPRB, 0.10736E+01_JPRB, 0.10468E+01_JPRB, 0.10141E+01_JPRB, 0.99256E+00_JPRB ,&
&  0.97624E+00_JPRB, 0.96779E+00_JPRB, 0.96275E+00_JPRB, 0.96587E+00_JPRB, 0.97206E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.96184E+00_JPRB, 0.95907E+00_JPRB, 0.95072E+00_JPRB, 0.91988E+00_JPRB, 0.89702E+00_JPRB, 0.88082E+00_JPRB, 0.84562E+00_JPRB ,&
&  0.81942E+00_JPRB, 0.80225E+00_JPRB, 0.77804E+00_JPRB, 0.75552E+00_JPRB, 0.74317E+00_JPRB, 0.73296E+00_JPRB, 0.74677E+00_JPRB ,&
&  0.79957E+00_JPRB, 0.89166E+00_JPRB, 0.10488E+01_JPRB, 0.11966E+01_JPRB, 0.13962E+01_JPRB, 0.16986E+01_JPRB, 0.18564E+01_JPRB ,&
&  0.19753E+01_JPRB, 0.34111E+01_JPRB, 0.30468E+01_JPRB, 0.29235E+01_JPRB, 0.27904E+01_JPRB, 0.25816E+01_JPRB, 0.22965E+01_JPRB ,&
&  0.18705E+01_JPRB, 0.16557E+01_JPRB, 0.15336E+01_JPRB, 0.14309E+01_JPRB, 0.13821E+01_JPRB, 0.12826E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.12329E+01_JPRB, 0.12277E+01_JPRB, 0.12778E+01_JPRB, 0.13607E+01_JPRB, 0.14258E+01_JPRB, 0.14963E+01_JPRB ,&
&  0.15364E+01_JPRB, 0.15474E+01_JPRB, 0.15140E+01_JPRB, 0.15068E+01_JPRB, 0.15382E+01_JPRB, 0.16203E+01_JPRB, 0.16980E+01_JPRB ,&
&  0.16963E+01_JPRB, 0.16748E+01_JPRB, 0.18575E+01_JPRB, 0.19156E+01_JPRB, 0.19000E+01_JPRB, 0.19828E+01_JPRB, 0.18024E+01_JPRB ,&
&  0.19130E+01_JPRB, 0.17549E+01_JPRB, 0.21219E+01_JPRB, 0.18301E+01_JPRB, 0.19311E+01_JPRB, 0.20932E+01_JPRB, 0.20730E+01_JPRB ,&
&  0.18697E+01_JPRB, 0.19129E+01_JPRB, 0.17842E+01_JPRB, 0.16884E+01_JPRB, 0.16079E+01_JPRB, 0.16473E+01_JPRB, 0.18813E+01_JPRB ,&
&  0.19569E+01_JPRB, 0.19382E+01_JPRB, 0.20158E+01_JPRB, 0.17482E+01_JPRB, 0.17128E+01_JPRB, 0.16797E+01_JPRB, 0.19053E+01_JPRB ,&
&  0.26760E+01_JPRB, 0.27510E+01_JPRB, 0.26093E+01_JPRB, 0.23176E+01_JPRB, 0.18987E+01_JPRB, 0.17443E+01_JPRB, 0.15531E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.13126E+01_JPRB, 0.12415E+01_JPRB, 0.11750E+01_JPRB, 0.11092E+01_JPRB, 0.10478E+01_JPRB, 0.10138E+01_JPRB ,&
&  0.99112E+00_JPRB, 0.98102E+00_JPRB, 0.97902E+00_JPRB, 0.97168E+00_JPRB, 0.96507E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.98948E+00_JPRB, 0.97289E+00_JPRB, 0.97221E+00_JPRB, 0.94955E+00_JPRB, 0.92102E+00_JPRB, 0.89808E+00_JPRB, 0.88851E+00_JPRB ,&
&  0.87279E+00_JPRB, 0.84237E+00_JPRB, 0.81290E+00_JPRB, 0.78637E+00_JPRB, 0.75989E+00_JPRB, 0.74817E+00_JPRB, 0.75033E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.81188E+00_JPRB, 0.91565E+00_JPRB, 0.10766E+01_JPRB, 0.12522E+01_JPRB, 0.15198E+01_JPRB, 0.13161E+01_JPRB ,&
&  0.19004E+01_JPRB, 0.30856E+01_JPRB, 0.31101E+01_JPRB, 0.31178E+01_JPRB, 0.30744E+01_JPRB, 0.29932E+01_JPRB, 0.26374E+01_JPRB ,&
&  0.22437E+01_JPRB, 0.19423E+01_JPRB, 0.16832E+01_JPRB, 0.15308E+01_JPRB, 0.14642E+01_JPRB, 0.14126E+01_JPRB, 0.13385E+01_JPRB ,&
&  0.12809E+01_JPRB, 0.12409E+01_JPRB, 0.12343E+01_JPRB, 0.12818E+01_JPRB, 0.13582E+01_JPRB, 0.14600E+01_JPRB, 0.15363E+01_JPRB ,&
&  0.15758E+01_JPRB, 0.15976E+01_JPRB, 0.16289E+01_JPRB, 0.15612E+01_JPRB, 0.16545E+01_JPRB, 0.17329E+01_JPRB, 0.18067E+01_JPRB ,&
&  0.17401E+01_JPRB, 0.16788E+01_JPRB, 0.19697E+01_JPRB, 0.22288E+01_JPRB, 0.23430E+01_JPRB, 0.22531E+01_JPRB, 0.20833E+01_JPRB ,&
&  0.20803E+01_JPRB, 0.20015E+01_JPRB, 0.20773E+01_JPRB, 0.20055E+01_JPRB, 0.21198E+01_JPRB, 0.22540E+01_JPRB, 0.23617E+01_JPRB ,&
&  0.23277E+01_JPRB, 0.21282E+01_JPRB, 0.19799E+01_JPRB, 0.18544E+01_JPRB, 0.17199E+01_JPRB, 0.16356E+01_JPRB, 0.19158E+01_JPRB ,&
&  0.22190E+01_JPRB, 0.24391E+01_JPRB, 0.25381E+01_JPRB, 0.21973E+01_JPRB, 0.19598E+01_JPRB, 0.18595E+01_JPRB, 0.20674E+01_JPRB ,&
&  0.29241E+01_JPRB, 0.31568E+01_JPRB, 0.28035E+01_JPRB, 0.27229E+01_JPRB, 0.21592E+01_JPRB, 0.19932E+01_JPRB, 0.17928E+01_JPRB ,&
&  0.16054E+01_JPRB, 0.14660E+01_JPRB, 0.13430E+01_JPRB, 0.12511E+01_JPRB, 0.11922E+01_JPRB, 0.11270E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10474E+01_JPRB, 0.10433E+01_JPRB, 0.10331E+01_JPRB, 0.10102E+01_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10637E+01_JPRB, 0.10313E+01_JPRB, 0.10017E+01_JPRB, 0.98852E+00_JPRB, 0.97887E+00_JPRB, 0.95750E+00_JPRB, 0.94053E+00_JPRB ,&
&  0.92281E+00_JPRB, 0.88556E+00_JPRB, 0.85173E+00_JPRB, 0.81407E+00_JPRB, 0.78338E+00_JPRB, 0.76370E+00_JPRB, 0.75850E+00_JPRB ,&
&  0.77219E+00_JPRB, 0.79685E+00_JPRB, 0.89492E+00_JPRB, 0.10338E+01_JPRB, 0.11618E+01_JPRB, 0.11068E+01_JPRB, 0.12880E+01_JPRB ,&
&  0.22122E+01_JPRB, 0.29570E+01_JPRB, 0.28715E+01_JPRB, 0.29892E+01_JPRB, 0.31707E+01_JPRB, 0.33590E+01_JPRB, 0.29449E+01_JPRB ,&
&  0.25440E+01_JPRB, 0.22058E+01_JPRB, 0.19064E+01_JPRB, 0.17074E+01_JPRB, 0.15698E+01_JPRB, 0.14870E+01_JPRB, 0.14248E+01_JPRB ,&
&  0.13529E+01_JPRB, 0.12871E+01_JPRB, 0.12537E+01_JPRB, 0.12878E+01_JPRB, 0.13916E+01_JPRB, 0.14983E+01_JPRB, 0.15913E+01_JPRB ,&
&  0.16182E+01_JPRB, 0.16312E+01_JPRB, 0.16694E+01_JPRB, 0.16559E+01_JPRB, 0.17140E+01_JPRB, 0.17692E+01_JPRB, 0.18702E+01_JPRB ,&
&  0.19073E+01_JPRB, 0.20543E+01_JPRB, 0.22173E+01_JPRB, 0.23046E+01_JPRB, 0.24547E+01_JPRB, 0.25794E+01_JPRB, 0.24993E+01_JPRB ,&
&  0.23687E+01_JPRB, 0.20488E+01_JPRB, 0.18318E+01_JPRB, 0.20343E+01_JPRB, 0.22211E+01_JPRB, 0.24371E+01_JPRB, 0.23272E+01_JPRB ,&
&  0.20392E+01_JPRB, 0.17145E+01_JPRB, 0.17052E+01_JPRB, 0.18362E+01_JPRB, 0.19593E+01_JPRB, 0.18285E+01_JPRB, 0.22047E+01_JPRB ,&
&  0.20749E+01_JPRB, 0.14793E+01_JPRB, 0.12923E+01_JPRB, 0.15854E+01_JPRB, 0.13984E+01_JPRB, 0.14166E+01_JPRB, 0.21887E+01_JPRB ,&
&  0.29456E+01_JPRB, 0.37679E+01_JPRB, 0.34793E+01_JPRB, 0.26973E+01_JPRB, 0.24538E+01_JPRB, 0.21922E+01_JPRB, 0.19517E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.15459E+01_JPRB, 0.14377E+01_JPRB, 0.13254E+01_JPRB, 0.12367E+01_JPRB, 0.11937E+01_JPRB, 0.11722E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.11272E+01_JPRB, 0.11205E+01_JPRB, 0.11143E+01_JPRB, 0.10925E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.11390E+01_JPRB, 0.11047E+01_JPRB, 0.10733E+01_JPRB, 0.10358E+01_JPRB, 0.10010E+01_JPRB, 0.97450E+00_JPRB, 0.95242E+00_JPRB ,&
&  0.94009E+00_JPRB, 0.90523E+00_JPRB, 0.86635E+00_JPRB, 0.82742E+00_JPRB, 0.79405E+00_JPRB, 0.77157E+00_JPRB, 0.76683E+00_JPRB ,&
&  0.76608E+00_JPRB, 0.79510E+00_JPRB, 0.94310E+00_JPRB, 0.10751E+01_JPRB, 0.11206E+01_JPRB, 0.10553E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.17798E+01_JPRB, 0.23964E+01_JPRB, 0.29508E+01_JPRB, 0.31660E+01_JPRB, 0.33387E+01_JPRB, 0.33100E+01_JPRB, 0.30519E+01_JPRB ,&
&  0.27309E+01_JPRB, 0.24652E+01_JPRB, 0.22040E+01_JPRB, 0.19576E+01_JPRB, 0.17338E+01_JPRB, 0.15892E+01_JPRB, 0.14868E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.13312E+01_JPRB, 0.12578E+01_JPRB, 0.12673E+01_JPRB, 0.13549E+01_JPRB, 0.14752E+01_JPRB, 0.15637E+01_JPRB ,&
&  0.16082E+01_JPRB, 0.16239E+01_JPRB, 0.16863E+01_JPRB, 0.17875E+01_JPRB, 0.14829E+01_JPRB, 0.17455E+01_JPRB, 0.18000E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.20757E+01_JPRB, 0.20993E+01_JPRB, 0.24893E+01_JPRB, 0.26811E+01_JPRB, 0.27299E+01_JPRB, 0.27879E+01_JPRB ,&
&  0.27898E+01_JPRB, 0.23196E+01_JPRB, 0.19955E+01_JPRB, 0.21531E+01_JPRB, 0.23578E+01_JPRB, 0.26765E+01_JPRB, 0.18385E+01_JPRB ,&
&  0.17209E+01_JPRB, 0.20888E+01_JPRB, 0.19937E+01_JPRB, 0.17669E+01_JPRB, 0.21048E+01_JPRB, 0.21136E+01_JPRB, 0.16425E+01_JPRB ,&
&  0.83412E+00_JPRB, 0.66260E+00_JPRB, 0.69321E+00_JPRB, 0.70621E+00_JPRB, 0.69128E+00_JPRB, 0.10580E+01_JPRB, 0.34374E+01_JPRB ,&
&  0.30608E+01_JPRB, 0.40661E+01_JPRB, 0.37797E+01_JPRB, 0.32014E+01_JPRB, 0.26536E+01_JPRB, 0.24642E+01_JPRB, 0.23354E+01_JPRB ,&
&  0.19817E+01_JPRB, 0.17925E+01_JPRB, 0.15974E+01_JPRB, 0.14552E+01_JPRB, 0.13780E+01_JPRB, 0.13205E+01_JPRB, 0.12812E+01_JPRB ,&
&  0.12449E+01_JPRB, 0.12148E+01_JPRB, 0.11971E+01_JPRB, 0.11865E+01_JPRB, 0.11704E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.12142E+01_JPRB, 0.11897E+01_JPRB, 0.11578E+01_JPRB, 0.11139E+01_JPRB, 0.10566E+01_JPRB, 0.10003E+01_JPRB, 0.95699E+00_JPRB ,&
&  0.93461E+00_JPRB, 0.89549E+00_JPRB, 0.85504E+00_JPRB, 0.82104E+00_JPRB, 0.79203E+00_JPRB, 0.77273E+00_JPRB, 0.76711E+00_JPRB ,&
&  0.77204E+00_JPRB, 0.81507E+00_JPRB, 0.10832E+01_JPRB, 0.10508E+01_JPRB, 0.10861E+01_JPRB, 0.10624E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.20134E+01_JPRB, 0.26200E+01_JPRB, 0.30058E+01_JPRB, 0.34741E+01_JPRB, 0.36210E+01_JPRB, 0.34599E+01_JPRB ,&
&  0.30376E+01_JPRB, 0.26339E+01_JPRB, 0.22774E+01_JPRB, 0.20077E+01_JPRB, 0.17461E+01_JPRB, 0.15654E+01_JPRB, 0.14576E+01_JPRB ,&
&  0.13929E+01_JPRB, 0.13621E+01_JPRB, 0.12751E+01_JPRB, 0.12627E+01_JPRB, 0.13509E+01_JPRB, 0.14623E+01_JPRB, 0.15346E+01_JPRB ,&
&  0.15607E+01_JPRB, 0.15783E+01_JPRB, 0.16340E+01_JPRB, 0.17417E+01_JPRB, 0.15463E+01_JPRB, 0.13837E+01_JPRB, 0.16309E+01_JPRB ,&
&  0.18925E+01_JPRB, 0.22474E+01_JPRB, 0.22916E+01_JPRB, 0.27434E+01_JPRB, 0.28367E+01_JPRB, 0.28630E+01_JPRB, 0.32183E+01_JPRB ,&
&  0.31439E+01_JPRB, 0.30654E+01_JPRB, 0.22695E+01_JPRB, 0.22907E+01_JPRB, 0.25903E+01_JPRB, 0.21125E+01_JPRB, 0.20706E+01_JPRB ,&
&  0.24193E+01_JPRB, 0.21915E+01_JPRB, 0.19583E+01_JPRB, 0.16829E+01_JPRB, 0.21685E+01_JPRB, 0.18490E+01_JPRB, 0.93146E+00_JPRB ,&
&  0.77402E+00_JPRB, 0.69524E+00_JPRB, 0.64208E+00_JPRB, 0.59468E+00_JPRB, 0.60500E+00_JPRB, 0.99535E+00_JPRB, 0.22532E+01_JPRB ,&
&  0.25581E+01_JPRB, 0.37398E+01_JPRB, 0.37555E+01_JPRB, 0.33496E+01_JPRB, 0.27535E+01_JPRB, 0.27122E+01_JPRB, 0.24604E+01_JPRB ,&
&  0.21962E+01_JPRB, 0.21027E+01_JPRB, 0.19012E+01_JPRB, 0.17305E+01_JPRB, 0.15768E+01_JPRB, 0.14732E+01_JPRB, 0.14164E+01_JPRB ,&
&  0.13691E+01_JPRB, 0.13294E+01_JPRB, 0.13001E+01_JPRB, 0.12694E+01_JPRB, 0.12426E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.12495E+01_JPRB, 0.12165E+01_JPRB, 0.11905E+01_JPRB, 0.11535E+01_JPRB, 0.11005E+01_JPRB, 0.10350E+01_JPRB, 0.97209E+00_JPRB ,&
&  0.93016E+00_JPRB, 0.87996E+00_JPRB, 0.84074E+00_JPRB, 0.80908E+00_JPRB, 0.78250E+00_JPRB, 0.76835E+00_JPRB, 0.76420E+00_JPRB ,&
&  0.76679E+00_JPRB, 0.87975E+00_JPRB, 0.10878E+01_JPRB, 0.94468E+00_JPRB, 0.88671E+00_JPRB, 0.83889E+00_JPRB, 0.84719E+00_JPRB ,&
&  0.12679E+01_JPRB, 0.17667E+01_JPRB, 0.22670E+01_JPRB, 0.29464E+01_JPRB, 0.34934E+01_JPRB, 0.33955E+01_JPRB, 0.35809E+01_JPRB ,&
&  0.32684E+01_JPRB, 0.27877E+01_JPRB, 0.23768E+01_JPRB, 0.20609E+01_JPRB, 0.17962E+01_JPRB, 0.16014E+01_JPRB, 0.14609E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13559E+01_JPRB, 0.12992E+01_JPRB, 0.12662E+01_JPRB, 0.13362E+01_JPRB, 0.14809E+01_JPRB, 0.15612E+01_JPRB ,&
&  0.15205E+01_JPRB, 0.15277E+01_JPRB, 0.15548E+01_JPRB, 0.16472E+01_JPRB, 0.17573E+01_JPRB, 0.17667E+01_JPRB, 0.16320E+01_JPRB ,&
&  0.15987E+01_JPRB, 0.17993E+01_JPRB, 0.24480E+01_JPRB, 0.26250E+01_JPRB, 0.28518E+01_JPRB, 0.31406E+01_JPRB, 0.33574E+01_JPRB ,&
&  0.32427E+01_JPRB, 0.31589E+01_JPRB, 0.26067E+01_JPRB, 0.26288E+01_JPRB, 0.22981E+01_JPRB, 0.19528E+01_JPRB, 0.25091E+01_JPRB ,&
&  0.26852E+01_JPRB, 0.25584E+01_JPRB, 0.20914E+01_JPRB, 0.16115E+01_JPRB, 0.15008E+01_JPRB, 0.10882E+01_JPRB, 0.75141E+00_JPRB ,&
&  0.72514E+00_JPRB, 0.69470E+00_JPRB, 0.62264E+00_JPRB, 0.63400E+00_JPRB, 0.62846E+00_JPRB, 0.94997E+00_JPRB, 0.16488E+01_JPRB ,&
&  0.25365E+01_JPRB, 0.30128E+01_JPRB, 0.32272E+01_JPRB, 0.29562E+01_JPRB, 0.25817E+01_JPRB, 0.27140E+01_JPRB, 0.26675E+01_JPRB ,&
&  0.22946E+01_JPRB, 0.22405E+01_JPRB, 0.21159E+01_JPRB, 0.19108E+01_JPRB, 0.17813E+01_JPRB, 0.16712E+01_JPRB, 0.15828E+01_JPRB ,&
&  0.15065E+01_JPRB, 0.14508E+01_JPRB, 0.13995E+01_JPRB, 0.13415E+01_JPRB, 0.12931E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.12551E+01_JPRB, 0.12105E+01_JPRB, 0.11799E+01_JPRB, 0.11484E+01_JPRB, 0.11105E+01_JPRB, 0.10561E+01_JPRB, 0.98958E+00_JPRB ,&
&  0.93026E+00_JPRB, 0.87928E+00_JPRB, 0.83914E+00_JPRB, 0.80681E+00_JPRB, 0.77728E+00_JPRB, 0.76931E+00_JPRB, 0.76890E+00_JPRB ,&
&  0.79214E+00_JPRB, 0.10744E+01_JPRB, 0.81670E+00_JPRB, 0.67694E+00_JPRB, 0.74780E+00_JPRB, 0.76558E+00_JPRB, 0.72306E+00_JPRB ,&
&  0.11035E+01_JPRB, 0.16280E+01_JPRB, 0.20454E+01_JPRB, 0.29447E+01_JPRB, 0.38942E+01_JPRB, 0.36777E+01_JPRB, 0.36473E+01_JPRB ,&
&  0.36037E+01_JPRB, 0.29784E+01_JPRB, 0.25287E+01_JPRB, 0.21834E+01_JPRB, 0.18553E+01_JPRB, 0.16221E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.13329E+01_JPRB, 0.13220E+01_JPRB, 0.12939E+01_JPRB, 0.13049E+01_JPRB, 0.13359E+01_JPRB, 0.14416E+01_JPRB, 0.15553E+01_JPRB ,&
&  0.15324E+01_JPRB, 0.15023E+01_JPRB, 0.15409E+01_JPRB, 0.16692E+01_JPRB, 0.17490E+01_JPRB, 0.17356E+01_JPRB, 0.20908E+01_JPRB ,&
&  0.20483E+01_JPRB, 0.22765E+01_JPRB, 0.24170E+01_JPRB, 0.28892E+01_JPRB, 0.32449E+01_JPRB, 0.32599E+01_JPRB, 0.36539E+01_JPRB ,&
&  0.27442E+01_JPRB, 0.24542E+01_JPRB, 0.25092E+01_JPRB, 0.24215E+01_JPRB, 0.20247E+01_JPRB, 0.23769E+01_JPRB, 0.36910E+01_JPRB ,&
&  0.28093E+01_JPRB, 0.31228E+01_JPRB, 0.32136E+01_JPRB, 0.32000E+01_JPRB, 0.15598E+01_JPRB, 0.10649E+01_JPRB, 0.17696E+01_JPRB ,&
&  0.19217E+01_JPRB, 0.12041E+01_JPRB, 0.84291E+00_JPRB, 0.10536E+01_JPRB, 0.84987E+00_JPRB, 0.11469E+01_JPRB, 0.17284E+01_JPRB ,&
&  0.20161E+01_JPRB, 0.22427E+01_JPRB, 0.29691E+01_JPRB, 0.26586E+01_JPRB, 0.24706E+01_JPRB, 0.26933E+01_JPRB, 0.25527E+01_JPRB ,&
&  0.24533E+01_JPRB, 0.22688E+01_JPRB, 0.22740E+01_JPRB, 0.20115E+01_JPRB, 0.18232E+01_JPRB, 0.17020E+01_JPRB, 0.16313E+01_JPRB ,&
&  0.15858E+01_JPRB, 0.15311E+01_JPRB, 0.14629E+01_JPRB, 0.13864E+01_JPRB, 0.13106E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.12049E+01_JPRB, 0.11553E+01_JPRB, 0.11170E+01_JPRB, 0.10952E+01_JPRB, 0.10826E+01_JPRB, 0.10584E+01_JPRB, 0.10010E+01_JPRB ,&
&  0.93550E+00_JPRB, 0.88612E+00_JPRB, 0.83887E+00_JPRB, 0.80754E+00_JPRB, 0.77595E+00_JPRB, 0.77165E+00_JPRB, 0.77415E+00_JPRB ,&
&  0.78362E+00_JPRB, 0.82448E+00_JPRB, 0.63789E+00_JPRB, 0.60269E+00_JPRB, 0.75628E+00_JPRB, 0.73151E+00_JPRB, 0.75841E+00_JPRB ,&
&  0.10570E+01_JPRB, 0.14412E+01_JPRB, 0.19212E+01_JPRB, 0.27765E+01_JPRB, 0.31914E+01_JPRB, 0.35864E+01_JPRB, 0.36221E+01_JPRB ,&
&  0.37286E+01_JPRB, 0.31550E+01_JPRB, 0.26135E+01_JPRB, 0.21828E+01_JPRB, 0.18328E+01_JPRB, 0.16113E+01_JPRB, 0.14579E+01_JPRB ,&
&  0.13118E+01_JPRB, 0.12329E+01_JPRB, 0.12466E+01_JPRB, 0.12533E+01_JPRB, 0.13116E+01_JPRB, 0.13747E+01_JPRB, 0.14810E+01_JPRB ,&
&  0.15305E+01_JPRB, 0.14928E+01_JPRB, 0.14758E+01_JPRB, 0.15933E+01_JPRB, 0.16019E+01_JPRB, 0.16413E+01_JPRB, 0.18131E+01_JPRB ,&
&  0.20963E+01_JPRB, 0.23112E+01_JPRB, 0.25824E+01_JPRB, 0.29960E+01_JPRB, 0.35553E+01_JPRB, 0.34249E+01_JPRB, 0.41512E+01_JPRB ,&
&  0.32975E+01_JPRB, 0.28844E+01_JPRB, 0.25165E+01_JPRB, 0.21077E+01_JPRB, 0.24510E+01_JPRB, 0.40096E+01_JPRB, 0.39305E+01_JPRB ,&
&  0.36357E+01_JPRB, 0.36967E+01_JPRB, 0.36283E+01_JPRB, 0.30450E+01_JPRB, 0.24053E+01_JPRB, 0.18241E+01_JPRB, 0.22234E+01_JPRB ,&
&  0.21860E+01_JPRB, 0.22148E+01_JPRB, 0.19410E+01_JPRB, 0.15521E+01_JPRB, 0.12970E+01_JPRB, 0.15845E+01_JPRB, 0.16108E+01_JPRB ,&
&  0.16586E+01_JPRB, 0.17446E+01_JPRB, 0.22792E+01_JPRB, 0.23182E+01_JPRB, 0.22608E+01_JPRB, 0.22877E+01_JPRB, 0.24860E+01_JPRB ,&
&  0.23253E+01_JPRB, 0.23123E+01_JPRB, 0.22503E+01_JPRB, 0.21034E+01_JPRB, 0.19673E+01_JPRB, 0.18271E+01_JPRB, 0.17034E+01_JPRB ,&
&  0.16063E+01_JPRB, 0.15138E+01_JPRB, 0.14101E+01_JPRB, 0.13302E+01_JPRB, 0.12608E+01_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.11049E+01_JPRB, 0.10520E+01_JPRB, 0.10138E+01_JPRB, 0.99066E+00_JPRB, 0.98699E+00_JPRB, 0.99130E+00_JPRB, 0.96688E+00_JPRB ,&
&  0.91888E+00_JPRB, 0.87545E+00_JPRB, 0.83770E+00_JPRB, 0.81130E+00_JPRB, 0.78781E+00_JPRB, 0.78092E+00_JPRB, 0.77466E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.70993E+00_JPRB, 0.55285E+00_JPRB, 0.58834E+00_JPRB, 0.68120E+00_JPRB, 0.66609E+00_JPRB, 0.81785E+00_JPRB ,&
&  0.99329E+00_JPRB, 0.12727E+01_JPRB, 0.16258E+01_JPRB, 0.22956E+01_JPRB, 0.27492E+01_JPRB, 0.30426E+01_JPRB, 0.31332E+01_JPRB ,&
&  0.30005E+01_JPRB, 0.28105E+01_JPRB, 0.25345E+01_JPRB, 0.22106E+01_JPRB, 0.18283E+01_JPRB, 0.16645E+01_JPRB, 0.15498E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.13028E+01_JPRB, 0.12222E+01_JPRB, 0.12246E+01_JPRB, 0.12589E+01_JPRB, 0.13261E+01_JPRB, 0.14499E+01_JPRB ,&
&  0.15135E+01_JPRB, 0.15057E+01_JPRB, 0.14083E+01_JPRB, 0.14883E+01_JPRB, 0.14629E+01_JPRB, 0.15615E+01_JPRB, 0.16597E+01_JPRB ,&
&  0.23774E+01_JPRB, 0.26993E+01_JPRB, 0.28461E+01_JPRB, 0.35124E+01_JPRB, 0.31691E+01_JPRB, 0.33952E+01_JPRB, 0.45467E+01_JPRB ,&
&  0.47009E+01_JPRB, 0.46598E+01_JPRB, 0.47941E+01_JPRB, 0.36850E+01_JPRB, 0.30280E+01_JPRB, 0.42971E+01_JPRB, 0.41136E+01_JPRB ,&
&  0.38432E+01_JPRB, 0.37617E+01_JPRB, 0.34509E+01_JPRB, 0.35388E+01_JPRB, 0.27088E+01_JPRB, 0.22341E+01_JPRB, 0.16143E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.13263E+01_JPRB, 0.14872E+01_JPRB, 0.13556E+01_JPRB, 0.13694E+01_JPRB, 0.13001E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.14271E+01_JPRB, 0.15333E+01_JPRB, 0.17803E+01_JPRB, 0.20351E+01_JPRB, 0.19335E+01_JPRB, 0.22003E+01_JPRB ,&
&  0.22829E+01_JPRB, 0.22485E+01_JPRB, 0.20431E+01_JPRB, 0.20651E+01_JPRB, 0.19810E+01_JPRB, 0.18795E+01_JPRB, 0.17565E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.14810E+01_JPRB, 0.13660E+01_JPRB, 0.12628E+01_JPRB, 0.11757E+01_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.97279E+00_JPRB, 0.93316E+00_JPRB, 0.90925E+00_JPRB, 0.89565E+00_JPRB, 0.89353E+00_JPRB, 0.90310E+00_JPRB, 0.89367E+00_JPRB ,&
&  0.87155E+00_JPRB, 0.84154E+00_JPRB, 0.80527E+00_JPRB, 0.77612E+00_JPRB, 0.76228E+00_JPRB, 0.76918E+00_JPRB, 0.77231E+00_JPRB ,&
&  0.75271E+00_JPRB, 0.76923E+00_JPRB, 0.61664E+00_JPRB, 0.55272E+00_JPRB, 0.54586E+00_JPRB, 0.65990E+00_JPRB, 0.85745E+00_JPRB ,&
&  0.99810E+00_JPRB, 0.11193E+01_JPRB, 0.14174E+01_JPRB, 0.18654E+01_JPRB, 0.22114E+01_JPRB, 0.23745E+01_JPRB, 0.25423E+01_JPRB ,&
&  0.29850E+01_JPRB, 0.23443E+01_JPRB, 0.22505E+01_JPRB, 0.20065E+01_JPRB, 0.18139E+01_JPRB, 0.16756E+01_JPRB, 0.15528E+01_JPRB ,&
&  0.14333E+01_JPRB, 0.13488E+01_JPRB, 0.12287E+01_JPRB, 0.11659E+01_JPRB, 0.11470E+01_JPRB, 0.11629E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.14173E+01_JPRB, 0.14128E+01_JPRB, 0.14402E+01_JPRB, 0.16125E+01_JPRB, 0.17567E+01_JPRB, 0.19065E+01_JPRB ,&
&  0.19613E+01_JPRB, 0.20562E+01_JPRB, 0.30117E+01_JPRB, 0.35155E+01_JPRB, 0.39992E+01_JPRB, 0.42552E+01_JPRB, 0.44177E+01_JPRB ,&
&  0.46602E+01_JPRB, 0.45607E+01_JPRB, 0.45906E+01_JPRB, 0.40731E+01_JPRB, 0.45941E+01_JPRB, 0.45450E+01_JPRB, 0.42490E+01_JPRB ,&
&  0.38938E+01_JPRB, 0.37042E+01_JPRB, 0.33185E+01_JPRB, 0.31404E+01_JPRB, 0.31112E+01_JPRB, 0.31239E+01_JPRB, 0.23680E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.19041E+01_JPRB, 0.15739E+01_JPRB, 0.13411E+01_JPRB, 0.11498E+01_JPRB, 0.12118E+01_JPRB, 0.12816E+01_JPRB ,&
&  0.13355E+01_JPRB, 0.14066E+01_JPRB, 0.13990E+01_JPRB, 0.15562E+01_JPRB, 0.18129E+01_JPRB, 0.19524E+01_JPRB, 0.21022E+01_JPRB ,&
&  0.19421E+01_JPRB, 0.20241E+01_JPRB, 0.19731E+01_JPRB, 0.18790E+01_JPRB, 0.18643E+01_JPRB, 0.17944E+01_JPRB, 0.16694E+01_JPRB ,&
&  0.15038E+01_JPRB, 0.13552E+01_JPRB, 0.12313E+01_JPRB, 0.11238E+01_JPRB, 0.10357E+01_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.92381E+00_JPRB, 0.88968E+00_JPRB, 0.86430E+00_JPRB, 0.84870E+00_JPRB, 0.84507E+00_JPRB, 0.85715E+00_JPRB, 0.85602E+00_JPRB ,&
&  0.83745E+00_JPRB, 0.81923E+00_JPRB, 0.78739E+00_JPRB, 0.76101E+00_JPRB, 0.74405E+00_JPRB, 0.74919E+00_JPRB, 0.75190E+00_JPRB ,&
&  0.75698E+00_JPRB, 0.96745E+00_JPRB, 0.73772E+00_JPRB, 0.68022E+00_JPRB, 0.64097E+00_JPRB, 0.77766E+00_JPRB, 0.90325E+00_JPRB ,&
&  0.96008E+00_JPRB, 0.10209E+01_JPRB, 0.11144E+01_JPRB, 0.14504E+01_JPRB, 0.17676E+01_JPRB, 0.17924E+01_JPRB, 0.19780E+01_JPRB ,&
&  0.20180E+01_JPRB, 0.19881E+01_JPRB, 0.17588E+01_JPRB, 0.17807E+01_JPRB, 0.15638E+01_JPRB, 0.13518E+01_JPRB, 0.14003E+01_JPRB ,&
&  0.13074E+01_JPRB, 0.12654E+01_JPRB, 0.12019E+01_JPRB, 0.11265E+01_JPRB, 0.10991E+01_JPRB, 0.10845E+01_JPRB, 0.11006E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.11960E+01_JPRB, 0.12721E+01_JPRB, 0.13477E+01_JPRB, 0.14688E+01_JPRB, 0.16973E+01_JPRB, 0.18556E+01_JPRB ,&
&  0.23584E+01_JPRB, 0.24946E+01_JPRB, 0.22571E+01_JPRB, 0.30689E+01_JPRB, 0.45524E+01_JPRB, 0.44658E+01_JPRB, 0.40938E+01_JPRB ,&
&  0.44058E+01_JPRB, 0.44479E+01_JPRB, 0.45541E+01_JPRB, 0.46902E+01_JPRB, 0.46849E+01_JPRB, 0.45509E+01_JPRB, 0.42398E+01_JPRB ,&
&  0.37779E+01_JPRB, 0.34583E+01_JPRB, 0.32279E+01_JPRB, 0.31695E+01_JPRB, 0.28529E+01_JPRB, 0.26579E+01_JPRB, 0.25043E+01_JPRB ,&
&  0.21342E+01_JPRB, 0.17550E+01_JPRB, 0.12116E+01_JPRB, 0.12820E+01_JPRB, 0.10042E+01_JPRB, 0.10466E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.12235E+01_JPRB, 0.13684E+01_JPRB, 0.14511E+01_JPRB, 0.14127E+01_JPRB, 0.16499E+01_JPRB, 0.17957E+01_JPRB, 0.19658E+01_JPRB ,&
&  0.19572E+01_JPRB, 0.16454E+01_JPRB, 0.17005E+01_JPRB, 0.16081E+01_JPRB, 0.15784E+01_JPRB, 0.15054E+01_JPRB, 0.14155E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.11881E+01_JPRB, 0.10928E+01_JPRB, 0.10184E+01_JPRB, 0.96582E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.82465E+00_JPRB, 0.79398E+00_JPRB, 0.76296E+00_JPRB, 0.75582E+00_JPRB, 0.76890E+00_JPRB, 0.78955E+00_JPRB, 0.79898E+00_JPRB ,&
&  0.78590E+00_JPRB, 0.77720E+00_JPRB, 0.75487E+00_JPRB, 0.73494E+00_JPRB, 0.71895E+00_JPRB, 0.70778E+00_JPRB, 0.71389E+00_JPRB ,&
&  0.79879E+00_JPRB, 0.10321E+01_JPRB, 0.83612E+00_JPRB, 0.69486E+00_JPRB, 0.81860E+00_JPRB, 0.83984E+00_JPRB, 0.92836E+00_JPRB ,&
&  0.89781E+00_JPRB, 0.10050E+01_JPRB, 0.10271E+01_JPRB, 0.11215E+01_JPRB, 0.13662E+01_JPRB, 0.14595E+01_JPRB, 0.14798E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.14155E+01_JPRB, 0.14173E+01_JPRB, 0.14499E+01_JPRB, 0.14645E+01_JPRB, 0.12905E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11177E+01_JPRB, 0.10571E+01_JPRB, 0.10318E+01_JPRB, 0.10247E+01_JPRB, 0.10312E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.11212E+01_JPRB, 0.11502E+01_JPRB, 0.12296E+01_JPRB, 0.13500E+01_JPRB, 0.16195E+01_JPRB, 0.20563E+01_JPRB ,&
&  0.26708E+01_JPRB, 0.30802E+01_JPRB, 0.34580E+01_JPRB, 0.42052E+01_JPRB, 0.44477E+01_JPRB, 0.43270E+01_JPRB, 0.41936E+01_JPRB ,&
&  0.41294E+01_JPRB, 0.40369E+01_JPRB, 0.41069E+01_JPRB, 0.41541E+01_JPRB, 0.42449E+01_JPRB, 0.42855E+01_JPRB, 0.40443E+01_JPRB ,&
&  0.36329E+01_JPRB, 0.33576E+01_JPRB, 0.32458E+01_JPRB, 0.29821E+01_JPRB, 0.27630E+01_JPRB, 0.25156E+01_JPRB, 0.24773E+01_JPRB ,&
&  0.25300E+01_JPRB, 0.14323E+01_JPRB, 0.11160E+01_JPRB, 0.13250E+01_JPRB, 0.11053E+01_JPRB, 0.11782E+01_JPRB, 0.13993E+01_JPRB ,&
&  0.14017E+01_JPRB, 0.13761E+01_JPRB, 0.14179E+01_JPRB, 0.13497E+01_JPRB, 0.13579E+01_JPRB, 0.16224E+01_JPRB, 0.15866E+01_JPRB ,&
&  0.14656E+01_JPRB, 0.13820E+01_JPRB, 0.14218E+01_JPRB, 0.13692E+01_JPRB, 0.13187E+01_JPRB, 0.12434E+01_JPRB, 0.11446E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10000E+01_JPRB, 0.94013E+00_JPRB, 0.89071E+00_JPRB, 0.85568E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.75339E+00_JPRB, 0.73561E+00_JPRB, 0.70896E+00_JPRB, 0.70390E+00_JPRB, 0.72642E+00_JPRB, 0.75174E+00_JPRB, 0.76432E+00_JPRB ,&
&  0.75569E+00_JPRB, 0.75332E+00_JPRB, 0.73487E+00_JPRB, 0.72024E+00_JPRB, 0.70472E+00_JPRB, 0.67756E+00_JPRB, 0.69593E+00_JPRB ,&
&  0.60566E+00_JPRB, 0.80385E+00_JPRB, 0.67366E+00_JPRB, 0.68476E+00_JPRB, 0.92585E+00_JPRB, 0.85923E+00_JPRB, 0.86903E+00_JPRB ,&
&  0.84472E+00_JPRB, 0.88534E+00_JPRB, 0.90898E+00_JPRB, 0.98337E+00_JPRB, 0.11984E+01_JPRB, 0.13325E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.11979E+01_JPRB, 0.11223E+01_JPRB, 0.11675E+01_JPRB, 0.12011E+01_JPRB, 0.12008E+01_JPRB, 0.12305E+01_JPRB, 0.12255E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.10704E+01_JPRB, 0.99101E+00_JPRB, 0.95371E+00_JPRB, 0.93605E+00_JPRB, 0.92868E+00_JPRB, 0.92134E+00_JPRB ,&
&  0.93091E+00_JPRB, 0.96230E+00_JPRB, 0.10078E+01_JPRB, 0.11215E+01_JPRB, 0.12425E+01_JPRB, 0.16872E+01_JPRB, 0.23419E+01_JPRB ,&
&  0.29869E+01_JPRB, 0.31074E+01_JPRB, 0.34521E+01_JPRB, 0.41210E+01_JPRB, 0.41556E+01_JPRB, 0.39801E+01_JPRB, 0.37105E+01_JPRB ,&
&  0.36554E+01_JPRB, 0.36183E+01_JPRB, 0.36821E+01_JPRB, 0.38378E+01_JPRB, 0.38630E+01_JPRB, 0.40403E+01_JPRB, 0.39165E+01_JPRB ,&
&  0.35242E+01_JPRB, 0.33514E+01_JPRB, 0.33167E+01_JPRB, 0.32589E+01_JPRB, 0.31161E+01_JPRB, 0.31356E+01_JPRB, 0.29709E+01_JPRB ,&
&  0.28516E+01_JPRB, 0.24456E+01_JPRB, 0.18220E+01_JPRB, 0.16540E+01_JPRB, 0.12905E+01_JPRB, 0.13421E+01_JPRB, 0.17210E+01_JPRB ,&
&  0.16667E+01_JPRB, 0.14658E+01_JPRB, 0.13139E+01_JPRB, 0.12212E+01_JPRB, 0.11891E+01_JPRB, 0.13562E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.11846E+01_JPRB, 0.12044E+01_JPRB, 0.11909E+01_JPRB, 0.11230E+01_JPRB, 0.10367E+01_JPRB, 0.91330E+00_JPRB ,&
&  0.86062E+00_JPRB, 0.82117E+00_JPRB, 0.80055E+00_JPRB, 0.78503E+00_JPRB, 0.77005E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.72883E+00_JPRB, 0.70955E+00_JPRB, 0.68402E+00_JPRB, 0.68102E+00_JPRB, 0.70551E+00_JPRB, 0.73069E+00_JPRB, 0.74415E+00_JPRB ,&
&  0.73677E+00_JPRB, 0.73357E+00_JPRB, 0.72300E+00_JPRB, 0.71021E+00_JPRB, 0.68728E+00_JPRB, 0.66079E+00_JPRB, 0.60372E+00_JPRB ,&
&  0.54623E+00_JPRB, 0.68175E+00_JPRB, 0.64723E+00_JPRB, 0.67413E+00_JPRB, 0.77975E+00_JPRB, 0.74971E+00_JPRB, 0.73463E+00_JPRB ,&
&  0.75179E+00_JPRB, 0.76977E+00_JPRB, 0.83317E+00_JPRB, 0.90512E+00_JPRB, 0.10185E+01_JPRB, 0.12212E+01_JPRB, 0.12344E+01_JPRB ,&
&  0.10923E+01_JPRB, 0.10155E+01_JPRB, 0.10487E+01_JPRB, 0.11271E+01_JPRB, 0.12027E+01_JPRB, 0.12656E+01_JPRB, 0.12372E+01_JPRB ,&
&  0.11716E+01_JPRB, 0.10947E+01_JPRB, 0.98126E+00_JPRB, 0.90310E+00_JPRB, 0.85771E+00_JPRB, 0.83408E+00_JPRB, 0.82400E+00_JPRB ,&
&  0.82370E+00_JPRB, 0.84522E+00_JPRB, 0.90031E+00_JPRB, 0.98953E+00_JPRB, 0.11221E+01_JPRB, 0.14995E+01_JPRB, 0.18399E+01_JPRB ,&
&  0.19881E+01_JPRB, 0.21462E+01_JPRB, 0.24124E+01_JPRB, 0.26829E+01_JPRB, 0.29766E+01_JPRB, 0.30453E+01_JPRB, 0.31178E+01_JPRB ,&
&  0.32496E+01_JPRB, 0.32501E+01_JPRB, 0.36709E+01_JPRB, 0.37138E+01_JPRB, 0.37183E+01_JPRB, 0.37970E+01_JPRB, 0.36570E+01_JPRB ,&
&  0.35250E+01_JPRB, 0.33731E+01_JPRB, 0.33936E+01_JPRB, 0.33139E+01_JPRB, 0.32202E+01_JPRB, 0.32157E+01_JPRB, 0.30467E+01_JPRB ,&
&  0.29758E+01_JPRB, 0.28528E+01_JPRB, 0.24518E+01_JPRB, 0.21482E+01_JPRB, 0.18093E+01_JPRB, 0.18440E+01_JPRB, 0.17519E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.12453E+01_JPRB, 0.12120E+01_JPRB, 0.11617E+01_JPRB, 0.11585E+01_JPRB, 0.11369E+01_JPRB, 0.10488E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10764E+01_JPRB, 0.10194E+01_JPRB, 0.98787E+00_JPRB, 0.92258E+00_JPRB, 0.86045E+00_JPRB, 0.72259E+00_JPRB ,&
&  0.67694E+00_JPRB, 0.70512E+00_JPRB, 0.72166E+00_JPRB, 0.73594E+00_JPRB, 0.73962E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.72878E+00_JPRB, 0.70946E+00_JPRB, 0.69661E+00_JPRB, 0.69018E+00_JPRB, 0.69717E+00_JPRB, 0.72024E+00_JPRB, 0.70361E+00_JPRB ,&
&  0.72395E+00_JPRB, 0.72731E+00_JPRB, 0.71399E+00_JPRB, 0.69641E+00_JPRB, 0.66411E+00_JPRB, 0.57638E+00_JPRB, 0.47663E+00_JPRB ,&
&  0.47041E+00_JPRB, 0.53547E+00_JPRB, 0.59692E+00_JPRB, 0.62626E+00_JPRB, 0.67524E+00_JPRB, 0.67862E+00_JPRB, 0.67723E+00_JPRB ,&
&  0.67819E+00_JPRB, 0.69243E+00_JPRB, 0.73961E+00_JPRB, 0.82606E+00_JPRB, 0.91623E+00_JPRB, 0.10450E+01_JPRB, 0.11645E+01_JPRB ,&
&  0.10784E+01_JPRB, 0.10358E+01_JPRB, 0.10767E+01_JPRB, 0.10535E+01_JPRB, 0.11945E+01_JPRB, 0.12226E+01_JPRB, 0.12002E+01_JPRB ,&
&  0.11488E+01_JPRB, 0.10809E+01_JPRB, 0.99897E+00_JPRB, 0.94642E+00_JPRB, 0.89426E+00_JPRB, 0.84686E+00_JPRB, 0.81995E+00_JPRB ,&
&  0.82362E+00_JPRB, 0.83629E+00_JPRB, 0.86714E+00_JPRB, 0.92392E+00_JPRB, 0.99295E+00_JPRB, 0.11586E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.17475E+01_JPRB, 0.20654E+01_JPRB, 0.20461E+01_JPRB, 0.22785E+01_JPRB, 0.25026E+01_JPRB, 0.25941E+01_JPRB ,&
&  0.28395E+01_JPRB, 0.28696E+01_JPRB, 0.30366E+01_JPRB, 0.31341E+01_JPRB, 0.32102E+01_JPRB, 0.32352E+01_JPRB, 0.31990E+01_JPRB ,&
&  0.32059E+01_JPRB, 0.33923E+01_JPRB, 0.34703E+01_JPRB, 0.33650E+01_JPRB, 0.31333E+01_JPRB, 0.29687E+01_JPRB, 0.27814E+01_JPRB ,&
&  0.26905E+01_JPRB, 0.26610E+01_JPRB, 0.25630E+01_JPRB, 0.24478E+01_JPRB, 0.22686E+01_JPRB, 0.20417E+01_JPRB, 0.18484E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.13704E+01_JPRB, 0.12184E+01_JPRB, 0.11737E+01_JPRB, 0.11450E+01_JPRB, 0.10248E+01_JPRB, 0.98678E+00_JPRB ,&
&  0.97744E+00_JPRB, 0.90492E+00_JPRB, 0.91990E+00_JPRB, 0.86509E+00_JPRB, 0.81572E+00_JPRB, 0.78108E+00_JPRB, 0.73822E+00_JPRB ,&
&  0.69189E+00_JPRB, 0.73767E+00_JPRB, 0.75915E+00_JPRB, 0.76106E+00_JPRB, 0.74949E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.73291E+00_JPRB, 0.72677E+00_JPRB, 0.72023E+00_JPRB, 0.70486E+00_JPRB, 0.69505E+00_JPRB, 0.67877E+00_JPRB, 0.67780E+00_JPRB ,&
&  0.69698E+00_JPRB, 0.65047E+00_JPRB, 0.69099E+00_JPRB, 0.56882E+00_JPRB, 0.50582E+00_JPRB, 0.47659E+00_JPRB, 0.45912E+00_JPRB ,&
&  0.47057E+00_JPRB, 0.51234E+00_JPRB, 0.55002E+00_JPRB, 0.57600E+00_JPRB, 0.62852E+00_JPRB, 0.63980E+00_JPRB, 0.63907E+00_JPRB ,&
&  0.64621E+00_JPRB, 0.65635E+00_JPRB, 0.70445E+00_JPRB, 0.73353E+00_JPRB, 0.80101E+00_JPRB, 0.89286E+00_JPRB, 0.10257E+01_JPRB ,&
&  0.98484E+00_JPRB, 0.10006E+01_JPRB, 0.11028E+01_JPRB, 0.11459E+01_JPRB, 0.11771E+01_JPRB, 0.11486E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.10297E+01_JPRB, 0.83908E+00_JPRB, 0.84898E+00_JPRB, 0.81614E+00_JPRB, 0.78491E+00_JPRB, 0.75096E+00_JPRB, 0.70288E+00_JPRB ,&
&  0.66877E+00_JPRB, 0.66301E+00_JPRB, 0.67870E+00_JPRB, 0.70811E+00_JPRB, 0.75783E+00_JPRB, 0.84610E+00_JPRB, 0.94406E+00_JPRB ,&
&  0.11370E+01_JPRB, 0.11231E+01_JPRB, 0.14398E+01_JPRB, 0.15905E+01_JPRB, 0.18042E+01_JPRB, 0.20934E+01_JPRB, 0.23429E+01_JPRB ,&
&  0.26839E+01_JPRB, 0.26373E+01_JPRB, 0.26586E+01_JPRB, 0.26508E+01_JPRB, 0.26180E+01_JPRB, 0.26211E+01_JPRB, 0.26909E+01_JPRB ,&
&  0.28079E+01_JPRB, 0.27883E+01_JPRB, 0.30125E+01_JPRB, 0.29681E+01_JPRB, 0.28000E+01_JPRB, 0.26857E+01_JPRB, 0.25509E+01_JPRB ,&
&  0.24301E+01_JPRB, 0.23405E+01_JPRB, 0.22372E+01_JPRB, 0.19744E+01_JPRB, 0.19058E+01_JPRB, 0.18528E+01_JPRB, 0.16899E+01_JPRB ,&
&  0.16094E+01_JPRB, 0.14896E+01_JPRB, 0.13363E+01_JPRB, 0.13414E+01_JPRB, 0.12188E+01_JPRB, 0.11167E+01_JPRB, 0.10852E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.84301E+00_JPRB, 0.81255E+00_JPRB, 0.80344E+00_JPRB, 0.75699E+00_JPRB, 0.72864E+00_JPRB, 0.77524E+00_JPRB ,&
&  0.75606E+00_JPRB, 0.75883E+00_JPRB, 0.79640E+00_JPRB, 0.77226E+00_JPRB, 0.75842E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.73292E+00_JPRB, 0.73526E+00_JPRB, 0.73877E+00_JPRB, 0.72020E+00_JPRB, 0.69345E+00_JPRB, 0.66897E+00_JPRB, 0.64757E+00_JPRB ,&
&  0.63615E+00_JPRB, 0.55804E+00_JPRB, 0.51630E+00_JPRB, 0.49705E+00_JPRB, 0.48671E+00_JPRB, 0.46528E+00_JPRB, 0.44773E+00_JPRB ,&
&  0.45001E+00_JPRB, 0.47090E+00_JPRB, 0.51013E+00_JPRB, 0.54151E+00_JPRB, 0.57013E+00_JPRB, 0.59250E+00_JPRB, 0.60310E+00_JPRB ,&
&  0.61498E+00_JPRB, 0.63133E+00_JPRB, 0.65045E+00_JPRB, 0.67517E+00_JPRB, 0.70199E+00_JPRB, 0.74701E+00_JPRB, 0.80642E+00_JPRB ,&
&  0.85603E+00_JPRB, 0.87918E+00_JPRB, 0.84318E+00_JPRB, 0.85392E+00_JPRB, 0.87320E+00_JPRB, 0.87734E+00_JPRB, 0.87475E+00_JPRB ,&
&  0.58749E+00_JPRB, 0.54770E+00_JPRB, 0.62579E+00_JPRB, 0.61411E+00_JPRB, 0.61635E+00_JPRB, 0.60683E+00_JPRB, 0.59487E+00_JPRB ,&
&  0.59253E+00_JPRB, 0.60304E+00_JPRB, 0.63191E+00_JPRB, 0.67729E+00_JPRB, 0.72172E+00_JPRB, 0.77588E+00_JPRB, 0.85868E+00_JPRB ,&
&  0.10390E+01_JPRB, 0.10336E+01_JPRB, 0.11035E+01_JPRB, 0.12613E+01_JPRB, 0.15714E+01_JPRB, 0.18303E+01_JPRB, 0.19747E+01_JPRB ,&
&  0.20974E+01_JPRB, 0.21144E+01_JPRB, 0.21356E+01_JPRB, 0.22120E+01_JPRB, 0.22541E+01_JPRB, 0.22681E+01_JPRB, 0.23307E+01_JPRB ,&
&  0.23555E+01_JPRB, 0.23965E+01_JPRB, 0.24505E+01_JPRB, 0.25162E+01_JPRB, 0.24443E+01_JPRB, 0.23773E+01_JPRB, 0.22988E+01_JPRB ,&
&  0.21996E+01_JPRB, 0.20700E+01_JPRB, 0.18968E+01_JPRB, 0.17497E+01_JPRB, 0.16353E+01_JPRB, 0.16084E+01_JPRB, 0.15864E+01_JPRB ,&
&  0.15350E+01_JPRB, 0.14968E+01_JPRB, 0.14403E+01_JPRB, 0.13806E+01_JPRB, 0.12727E+01_JPRB, 0.11599E+01_JPRB, 0.10999E+01_JPRB ,&
&  0.94860E+00_JPRB, 0.83622E+00_JPRB, 0.71169E+00_JPRB, 0.69412E+00_JPRB, 0.72056E+00_JPRB, 0.72799E+00_JPRB, 0.71983E+00_JPRB ,&
&  0.74287E+00_JPRB, 0.74937E+00_JPRB, 0.73360E+00_JPRB, 0.73419E+00_JPRB, 0.73314E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.70880E+00_JPRB, 0.71456E+00_JPRB, 0.71851E+00_JPRB, 0.71756E+00_JPRB, 0.69080E+00_JPRB, 0.66818E+00_JPRB, 0.64894E+00_JPRB ,&
&  0.63861E+00_JPRB, 0.58334E+00_JPRB, 0.54068E+00_JPRB, 0.51187E+00_JPRB, 0.49265E+00_JPRB, 0.47430E+00_JPRB, 0.47028E+00_JPRB ,&
&  0.47639E+00_JPRB, 0.48840E+00_JPRB, 0.50611E+00_JPRB, 0.52526E+00_JPRB, 0.54157E+00_JPRB, 0.54954E+00_JPRB, 0.56178E+00_JPRB ,&
&  0.58119E+00_JPRB, 0.59989E+00_JPRB, 0.61228E+00_JPRB, 0.62432E+00_JPRB, 0.63386E+00_JPRB, 0.65510E+00_JPRB, 0.66444E+00_JPRB ,&
&  0.66533E+00_JPRB, 0.65464E+00_JPRB, 0.64876E+00_JPRB, 0.66961E+00_JPRB, 0.68098E+00_JPRB, 0.68111E+00_JPRB, 0.60553E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.44058E+00_JPRB, 0.45117E+00_JPRB, 0.52964E+00_JPRB, 0.55162E+00_JPRB, 0.56701E+00_JPRB, 0.58248E+00_JPRB ,&
&  0.56634E+00_JPRB, 0.56062E+00_JPRB, 0.58709E+00_JPRB, 0.66534E+00_JPRB, 0.71520E+00_JPRB, 0.77235E+00_JPRB, 0.84794E+00_JPRB ,&
&  0.98143E+00_JPRB, 0.11039E+01_JPRB, 0.12452E+01_JPRB, 0.12138E+01_JPRB, 0.13913E+01_JPRB, 0.17871E+01_JPRB, 0.17655E+01_JPRB ,&
&  0.17717E+01_JPRB, 0.18089E+01_JPRB, 0.19328E+01_JPRB, 0.19989E+01_JPRB, 0.20239E+01_JPRB, 0.20123E+01_JPRB, 0.20079E+01_JPRB ,&
&  0.19917E+01_JPRB, 0.19865E+01_JPRB, 0.20001E+01_JPRB, 0.21264E+01_JPRB, 0.21578E+01_JPRB, 0.21206E+01_JPRB, 0.20693E+01_JPRB ,&
&  0.19719E+01_JPRB, 0.18106E+01_JPRB, 0.16503E+01_JPRB, 0.15169E+01_JPRB, 0.14695E+01_JPRB, 0.14662E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.14147E+01_JPRB, 0.13958E+01_JPRB, 0.13807E+01_JPRB, 0.13600E+01_JPRB, 0.12152E+01_JPRB, 0.10477E+01_JPRB, 0.92112E+00_JPRB ,&
&  0.83335E+00_JPRB, 0.77393E+00_JPRB, 0.72074E+00_JPRB, 0.71696E+00_JPRB, 0.72037E+00_JPRB, 0.72685E+00_JPRB, 0.69747E+00_JPRB ,&
&  0.67326E+00_JPRB, 0.65193E+00_JPRB, 0.64946E+00_JPRB, 0.67510E+00_JPRB, 0.69532E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.63838E+00_JPRB, 0.65582E+00_JPRB, 0.68810E+00_JPRB, 0.70235E+00_JPRB, 0.70269E+00_JPRB, 0.66600E+00_JPRB, 0.62768E+00_JPRB ,&
&  0.59304E+00_JPRB, 0.55969E+00_JPRB, 0.53879E+00_JPRB, 0.52731E+00_JPRB, 0.52240E+00_JPRB, 0.52061E+00_JPRB, 0.52156E+00_JPRB ,&
&  0.52471E+00_JPRB, 0.52744E+00_JPRB, 0.53530E+00_JPRB, 0.55183E+00_JPRB, 0.56698E+00_JPRB, 0.57282E+00_JPRB, 0.57804E+00_JPRB ,&
&  0.58600E+00_JPRB, 0.59182E+00_JPRB, 0.58872E+00_JPRB, 0.57852E+00_JPRB, 0.57688E+00_JPRB, 0.59708E+00_JPRB, 0.60656E+00_JPRB ,&
&  0.60532E+00_JPRB, 0.58783E+00_JPRB, 0.58693E+00_JPRB, 0.60423E+00_JPRB, 0.60354E+00_JPRB, 0.58971E+00_JPRB, 0.53188E+00_JPRB ,&
&  0.46045E+00_JPRB, 0.40433E+00_JPRB, 0.38524E+00_JPRB, 0.40444E+00_JPRB, 0.45054E+00_JPRB, 0.52718E+00_JPRB, 0.54667E+00_JPRB ,&
&  0.55938E+00_JPRB, 0.56895E+00_JPRB, 0.58785E+00_JPRB, 0.62726E+00_JPRB, 0.68442E+00_JPRB, 0.74459E+00_JPRB, 0.80617E+00_JPRB ,&
&  0.89631E+00_JPRB, 0.99550E+00_JPRB, 0.11269E+01_JPRB, 0.11227E+01_JPRB, 0.11979E+01_JPRB, 0.13664E+01_JPRB, 0.14760E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.16161E+01_JPRB, 0.16999E+01_JPRB, 0.17882E+01_JPRB, 0.17783E+01_JPRB, 0.17502E+01_JPRB, 0.17128E+01_JPRB ,&
&  0.17253E+01_JPRB, 0.17335E+01_JPRB, 0.17440E+01_JPRB, 0.18257E+01_JPRB, 0.19383E+01_JPRB, 0.19669E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.18136E+01_JPRB, 0.16550E+01_JPRB, 0.14821E+01_JPRB, 0.13529E+01_JPRB, 0.13293E+01_JPRB, 0.13673E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.14052E+01_JPRB, 0.13933E+01_JPRB, 0.13360E+01_JPRB, 0.12848E+01_JPRB, 0.11361E+01_JPRB, 0.10265E+01_JPRB, 0.94287E+00_JPRB ,&
&  0.90712E+00_JPRB, 0.87367E+00_JPRB, 0.84207E+00_JPRB, 0.84880E+00_JPRB, 0.81977E+00_JPRB, 0.79531E+00_JPRB, 0.76791E+00_JPRB ,&
&  0.73240E+00_JPRB, 0.69310E+00_JPRB, 0.67010E+00_JPRB, 0.64981E+00_JPRB, 0.63954E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.71428E+00_JPRB, 0.70544E+00_JPRB, 0.69724E+00_JPRB, 0.69616E+00_JPRB, 0.69847E+00_JPRB, 0.68853E+00_JPRB, 0.67352E+00_JPRB ,&
&  0.65121E+00_JPRB, 0.62965E+00_JPRB, 0.61122E+00_JPRB, 0.59683E+00_JPRB, 0.58890E+00_JPRB, 0.58539E+00_JPRB, 0.58399E+00_JPRB ,&
&  0.58487E+00_JPRB, 0.58804E+00_JPRB, 0.58997E+00_JPRB, 0.58729E+00_JPRB, 0.58904E+00_JPRB, 0.59380E+00_JPRB, 0.59913E+00_JPRB ,&
&  0.60376E+00_JPRB, 0.60128E+00_JPRB, 0.59479E+00_JPRB, 0.58939E+00_JPRB, 0.58004E+00_JPRB, 0.59075E+00_JPRB, 0.58178E+00_JPRB ,&
&  0.56706E+00_JPRB, 0.56393E+00_JPRB, 0.57598E+00_JPRB, 0.57528E+00_JPRB, 0.56852E+00_JPRB, 0.54959E+00_JPRB, 0.48913E+00_JPRB ,&
&  0.43099E+00_JPRB, 0.38695E+00_JPRB, 0.36386E+00_JPRB, 0.35874E+00_JPRB, 0.37121E+00_JPRB, 0.40437E+00_JPRB, 0.45928E+00_JPRB ,&
&  0.53386E+00_JPRB, 0.55141E+00_JPRB, 0.56704E+00_JPRB, 0.59244E+00_JPRB, 0.62468E+00_JPRB, 0.66804E+00_JPRB, 0.71195E+00_JPRB ,&
&  0.76247E+00_JPRB, 0.82908E+00_JPRB, 0.92130E+00_JPRB, 0.10433E+01_JPRB, 0.11779E+01_JPRB, 0.11967E+01_JPRB, 0.13182E+01_JPRB ,&
&  0.15899E+01_JPRB, 0.16139E+01_JPRB, 0.15879E+01_JPRB, 0.14995E+01_JPRB, 0.14484E+01_JPRB, 0.14307E+01_JPRB, 0.14493E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.15395E+01_JPRB, 0.16271E+01_JPRB, 0.17111E+01_JPRB, 0.17462E+01_JPRB, 0.17519E+01_JPRB, 0.16856E+01_JPRB ,&
&  0.16139E+01_JPRB, 0.16295E+01_JPRB, 0.19063E+01_JPRB, 0.17211E+01_JPRB, 0.15932E+01_JPRB, 0.15498E+01_JPRB, 0.14784E+01_JPRB ,&
&  0.14340E+01_JPRB, 0.13862E+01_JPRB, 0.13089E+01_JPRB, 0.12346E+01_JPRB, 0.11401E+01_JPRB, 0.10594E+01_JPRB, 0.99165E+00_JPRB ,&
&  0.96967E+00_JPRB, 0.95388E+00_JPRB, 0.93347E+00_JPRB, 0.91636E+00_JPRB, 0.88822E+00_JPRB, 0.86506E+00_JPRB, 0.83935E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.76921E+00_JPRB, 0.75157E+00_JPRB, 0.73681E+00_JPRB, 0.72433E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.73177E+00_JPRB, 0.72071E+00_JPRB, 0.70933E+00_JPRB, 0.69960E+00_JPRB, 0.69393E+00_JPRB, 0.69033E+00_JPRB, 0.68515E+00_JPRB ,&
&  0.67743E+00_JPRB, 0.67151E+00_JPRB, 0.66284E+00_JPRB, 0.65835E+00_JPRB, 0.65331E+00_JPRB, 0.64828E+00_JPRB, 0.64395E+00_JPRB ,&
&  0.63871E+00_JPRB, 0.62417E+00_JPRB, 0.61580E+00_JPRB, 0.61438E+00_JPRB, 0.61569E+00_JPRB, 0.61677E+00_JPRB, 0.61842E+00_JPRB ,&
&  0.61393E+00_JPRB, 0.60449E+00_JPRB, 0.59663E+00_JPRB, 0.58420E+00_JPRB, 0.57627E+00_JPRB, 0.57444E+00_JPRB, 0.57872E+00_JPRB ,&
&  0.58015E+00_JPRB, 0.56052E+00_JPRB, 0.55683E+00_JPRB, 0.55192E+00_JPRB, 0.53603E+00_JPRB, 0.52663E+00_JPRB, 0.45213E+00_JPRB ,&
&  0.40461E+00_JPRB, 0.37419E+00_JPRB, 0.36273E+00_JPRB, 0.35692E+00_JPRB, 0.36752E+00_JPRB, 0.39431E+00_JPRB, 0.44760E+00_JPRB ,&
&  0.50543E+00_JPRB, 0.56508E+00_JPRB, 0.58035E+00_JPRB, 0.60244E+00_JPRB, 0.61969E+00_JPRB, 0.64998E+00_JPRB, 0.67219E+00_JPRB ,&
&  0.69002E+00_JPRB, 0.70655E+00_JPRB, 0.74033E+00_JPRB, 0.77467E+00_JPRB, 0.83802E+00_JPRB, 0.89839E+00_JPRB, 0.93772E+00_JPRB ,&
&  0.97356E+00_JPRB, 0.98911E+00_JPRB, 0.10241E+01_JPRB, 0.10593E+01_JPRB, 0.11025E+01_JPRB, 0.11217E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.12378E+01_JPRB, 0.13583E+01_JPRB, 0.14992E+01_JPRB, 0.15500E+01_JPRB, 0.15642E+01_JPRB, 0.15226E+01_JPRB ,&
&  0.14541E+01_JPRB, 0.14752E+01_JPRB, 0.16462E+01_JPRB, 0.16347E+01_JPRB, 0.16278E+01_JPRB, 0.16116E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.13843E+01_JPRB, 0.13317E+01_JPRB, 0.12341E+01_JPRB, 0.11443E+01_JPRB, 0.10911E+01_JPRB, 0.10627E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.10386E+01_JPRB, 0.10081E+01_JPRB, 0.97828E+00_JPRB, 0.94240E+00_JPRB, 0.90058E+00_JPRB, 0.86545E+00_JPRB ,&
&  0.83390E+00_JPRB, 0.80823E+00_JPRB, 0.78553E+00_JPRB, 0.76429E+00_JPRB, 0.74735E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.75374E+00_JPRB, 0.74463E+00_JPRB, 0.73517E+00_JPRB, 0.72725E+00_JPRB, 0.72053E+00_JPRB, 0.71636E+00_JPRB, 0.71386E+00_JPRB ,&
&  0.71001E+00_JPRB, 0.70600E+00_JPRB, 0.70087E+00_JPRB, 0.69578E+00_JPRB, 0.68946E+00_JPRB, 0.68381E+00_JPRB, 0.67806E+00_JPRB ,&
&  0.67063E+00_JPRB, 0.66184E+00_JPRB, 0.65158E+00_JPRB, 0.64107E+00_JPRB, 0.63094E+00_JPRB, 0.62480E+00_JPRB, 0.61823E+00_JPRB ,&
&  0.61190E+00_JPRB, 0.60310E+00_JPRB, 0.58770E+00_JPRB, 0.57450E+00_JPRB, 0.56981E+00_JPRB, 0.56128E+00_JPRB, 0.56186E+00_JPRB ,&
&  0.55474E+00_JPRB, 0.54128E+00_JPRB, 0.52343E+00_JPRB, 0.51706E+00_JPRB, 0.50845E+00_JPRB, 0.43695E+00_JPRB, 0.41643E+00_JPRB ,&
&  0.39255E+00_JPRB, 0.37202E+00_JPRB, 0.36578E+00_JPRB, 0.36723E+00_JPRB, 0.37588E+00_JPRB, 0.40085E+00_JPRB, 0.43674E+00_JPRB ,&
&  0.48167E+00_JPRB, 0.59451E+00_JPRB, 0.62658E+00_JPRB, 0.65927E+00_JPRB, 0.67983E+00_JPRB, 0.69518E+00_JPRB, 0.69454E+00_JPRB ,&
&  0.70148E+00_JPRB, 0.71289E+00_JPRB, 0.74494E+00_JPRB, 0.78993E+00_JPRB, 0.82969E+00_JPRB, 0.84395E+00_JPRB, 0.84365E+00_JPRB ,&
&  0.83586E+00_JPRB, 0.84196E+00_JPRB, 0.84997E+00_JPRB, 0.86794E+00_JPRB, 0.91499E+00_JPRB, 0.95654E+00_JPRB, 0.10152E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.11145E+01_JPRB, 0.11854E+01_JPRB, 0.12719E+01_JPRB, 0.13366E+01_JPRB, 0.13848E+01_JPRB, 0.13997E+01_JPRB ,&
&  0.14001E+01_JPRB, 0.14264E+01_JPRB, 0.14299E+01_JPRB, 0.14332E+01_JPRB, 0.14176E+01_JPRB, 0.13796E+01_JPRB, 0.13304E+01_JPRB ,&
&  0.13023E+01_JPRB, 0.13100E+01_JPRB, 0.12879E+01_JPRB, 0.12227E+01_JPRB, 0.11611E+01_JPRB, 0.11122E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.10292E+01_JPRB, 0.99672E+00_JPRB, 0.96129E+00_JPRB, 0.92264E+00_JPRB, 0.88363E+00_JPRB, 0.85804E+00_JPRB ,&
&  0.83668E+00_JPRB, 0.82039E+00_JPRB, 0.80072E+00_JPRB, 0.78247E+00_JPRB, 0.76824E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.77308E+00_JPRB, 0.76882E+00_JPRB, 0.76055E+00_JPRB, 0.75326E+00_JPRB, 0.74392E+00_JPRB, 0.73645E+00_JPRB, 0.72847E+00_JPRB ,&
&  0.72125E+00_JPRB, 0.71413E+00_JPRB, 0.70752E+00_JPRB, 0.70187E+00_JPRB, 0.69527E+00_JPRB, 0.68976E+00_JPRB, 0.68432E+00_JPRB ,&
&  0.67911E+00_JPRB, 0.67135E+00_JPRB, 0.66170E+00_JPRB, 0.65078E+00_JPRB, 0.64071E+00_JPRB, 0.62999E+00_JPRB, 0.61798E+00_JPRB ,&
&  0.60611E+00_JPRB, 0.59043E+00_JPRB, 0.56491E+00_JPRB, 0.54799E+00_JPRB, 0.53047E+00_JPRB, 0.52526E+00_JPRB, 0.51808E+00_JPRB ,&
&  0.52737E+00_JPRB, 0.47972E+00_JPRB, 0.46776E+00_JPRB, 0.44396E+00_JPRB, 0.42451E+00_JPRB, 0.41046E+00_JPRB, 0.39564E+00_JPRB ,&
&  0.38968E+00_JPRB, 0.38449E+00_JPRB, 0.38551E+00_JPRB, 0.39516E+00_JPRB, 0.41019E+00_JPRB, 0.44028E+00_JPRB, 0.47794E+00_JPRB ,&
&  0.53056E+00_JPRB, 0.63303E+00_JPRB, 0.66438E+00_JPRB, 0.69527E+00_JPRB, 0.71954E+00_JPRB, 0.74312E+00_JPRB, 0.73723E+00_JPRB ,&
&  0.74971E+00_JPRB, 0.76329E+00_JPRB, 0.79154E+00_JPRB, 0.82159E+00_JPRB, 0.85228E+00_JPRB, 0.86127E+00_JPRB, 0.86551E+00_JPRB ,&
&  0.85700E+00_JPRB, 0.85306E+00_JPRB, 0.85307E+00_JPRB, 0.87438E+00_JPRB, 0.91996E+00_JPRB, 0.97633E+00_JPRB, 0.10286E+01_JPRB ,&
&  0.10758E+01_JPRB, 0.11099E+01_JPRB, 0.11375E+01_JPRB, 0.11742E+01_JPRB, 0.12225E+01_JPRB, 0.12821E+01_JPRB, 0.13175E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13523E+01_JPRB, 0.13496E+01_JPRB, 0.13370E+01_JPRB, 0.13019E+01_JPRB, 0.12655E+01_JPRB, 0.12437E+01_JPRB ,&
&  0.12454E+01_JPRB, 0.12297E+01_JPRB, 0.12057E+01_JPRB, 0.11702E+01_JPRB, 0.11404E+01_JPRB, 0.11052E+01_JPRB, 0.10672E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10071E+01_JPRB, 0.97221E+00_JPRB, 0.94018E+00_JPRB, 0.90427E+00_JPRB, 0.87309E+00_JPRB, 0.85150E+00_JPRB ,&
&  0.83112E+00_JPRB, 0.81513E+00_JPRB, 0.79813E+00_JPRB, 0.78706E+00_JPRB, 0.77931E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.78111E+00_JPRB, 0.77852E+00_JPRB, 0.77374E+00_JPRB, 0.76959E+00_JPRB, 0.76082E+00_JPRB, 0.75321E+00_JPRB, 0.74635E+00_JPRB ,&
&  0.74078E+00_JPRB, 0.73711E+00_JPRB, 0.73285E+00_JPRB, 0.73084E+00_JPRB, 0.72669E+00_JPRB, 0.72380E+00_JPRB, 0.71797E+00_JPRB ,&
&  0.71401E+00_JPRB, 0.70757E+00_JPRB, 0.70042E+00_JPRB, 0.69011E+00_JPRB, 0.67886E+00_JPRB, 0.66550E+00_JPRB, 0.64511E+00_JPRB ,&
&  0.62077E+00_JPRB, 0.59899E+00_JPRB, 0.55789E+00_JPRB, 0.54092E+00_JPRB, 0.51917E+00_JPRB, 0.50672E+00_JPRB, 0.50372E+00_JPRB ,&
&  0.49986E+00_JPRB, 0.49759E+00_JPRB, 0.49011E+00_JPRB, 0.47967E+00_JPRB, 0.46675E+00_JPRB, 0.44906E+00_JPRB, 0.43803E+00_JPRB ,&
&  0.42844E+00_JPRB, 0.42679E+00_JPRB, 0.43234E+00_JPRB, 0.44556E+00_JPRB, 0.46796E+00_JPRB, 0.49906E+00_JPRB, 0.53566E+00_JPRB ,&
&  0.58462E+00_JPRB, 0.63282E+00_JPRB, 0.68905E+00_JPRB, 0.70977E+00_JPRB, 0.72696E+00_JPRB, 0.74732E+00_JPRB, 0.76549E+00_JPRB ,&
&  0.76860E+00_JPRB, 0.77657E+00_JPRB, 0.80587E+00_JPRB, 0.81830E+00_JPRB, 0.82894E+00_JPRB, 0.83510E+00_JPRB, 0.84314E+00_JPRB ,&
&  0.84986E+00_JPRB, 0.86224E+00_JPRB, 0.87695E+00_JPRB, 0.89877E+00_JPRB, 0.93219E+00_JPRB, 0.97304E+00_JPRB, 0.10106E+01_JPRB ,&
&  0.10433E+01_JPRB, 0.10780E+01_JPRB, 0.11043E+01_JPRB, 0.11295E+01_JPRB, 0.11547E+01_JPRB, 0.11870E+01_JPRB, 0.12118E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.12245E+01_JPRB, 0.12079E+01_JPRB, 0.11741E+01_JPRB, 0.11580E+01_JPRB, 0.11693E+01_JPRB, 0.11625E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.11429E+01_JPRB, 0.11316E+01_JPRB, 0.11155E+01_JPRB, 0.10976E+01_JPRB, 0.10736E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10108E+01_JPRB, 0.98501E+00_JPRB, 0.95897E+00_JPRB, 0.93424E+00_JPRB, 0.90592E+00_JPRB, 0.87875E+00_JPRB, 0.85530E+00_JPRB ,&
&  0.83536E+00_JPRB, 0.81879E+00_JPRB, 0.80375E+00_JPRB, 0.79288E+00_JPRB, 0.78663E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.79277E+00_JPRB, 0.79072E+00_JPRB, 0.78731E+00_JPRB, 0.78234E+00_JPRB, 0.77418E+00_JPRB, 0.76651E+00_JPRB, 0.75947E+00_JPRB ,&
&  0.75398E+00_JPRB, 0.74950E+00_JPRB, 0.74651E+00_JPRB, 0.74507E+00_JPRB, 0.74437E+00_JPRB, 0.74241E+00_JPRB, 0.74014E+00_JPRB ,&
&  0.73807E+00_JPRB, 0.73435E+00_JPRB, 0.72981E+00_JPRB, 0.72442E+00_JPRB, 0.71512E+00_JPRB, 0.70071E+00_JPRB, 0.68568E+00_JPRB ,&
&  0.67091E+00_JPRB, 0.65843E+00_JPRB, 0.64660E+00_JPRB, 0.63488E+00_JPRB, 0.62606E+00_JPRB, 0.57611E+00_JPRB, 0.56115E+00_JPRB ,&
&  0.55722E+00_JPRB, 0.55981E+00_JPRB, 0.56809E+00_JPRB, 0.57995E+00_JPRB, 0.60159E+00_JPRB, 0.59808E+00_JPRB, 0.59833E+00_JPRB ,&
&  0.56850E+00_JPRB, 0.56780E+00_JPRB, 0.57090E+00_JPRB, 0.57659E+00_JPRB, 0.58853E+00_JPRB, 0.60418E+00_JPRB, 0.62581E+00_JPRB ,&
&  0.66801E+00_JPRB, 0.67914E+00_JPRB, 0.69012E+00_JPRB, 0.70389E+00_JPRB, 0.71690E+00_JPRB, 0.72843E+00_JPRB, 0.74063E+00_JPRB ,&
&  0.75177E+00_JPRB, 0.76381E+00_JPRB, 0.77542E+00_JPRB, 0.78827E+00_JPRB, 0.80196E+00_JPRB, 0.81600E+00_JPRB, 0.83196E+00_JPRB ,&
&  0.84838E+00_JPRB, 0.86713E+00_JPRB, 0.88664E+00_JPRB, 0.90730E+00_JPRB, 0.92776E+00_JPRB, 0.94888E+00_JPRB, 0.97141E+00_JPRB ,&
&  0.99415E+00_JPRB, 0.10187E+01_JPRB, 0.10357E+01_JPRB, 0.10531E+01_JPRB, 0.10659E+01_JPRB, 0.10816E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10920E+01_JPRB, 0.10933E+01_JPRB, 0.10896E+01_JPRB, 0.10879E+01_JPRB, 0.10866E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10771E+01_JPRB, 0.10693E+01_JPRB, 0.10620E+01_JPRB, 0.10505E+01_JPRB, 0.10351E+01_JPRB, 0.10146E+01_JPRB, 0.99221E+00_JPRB ,&
&  0.97222E+00_JPRB, 0.95442E+00_JPRB, 0.93598E+00_JPRB, 0.91844E+00_JPRB, 0.89914E+00_JPRB, 0.87823E+00_JPRB, 0.85748E+00_JPRB ,&
&  0.83999E+00_JPRB, 0.82527E+00_JPRB, 0.81332E+00_JPRB, 0.80335E+00_JPRB, 0.79761E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.80611E+00_JPRB, 0.80532E+00_JPRB, 0.80441E+00_JPRB, 0.80233E+00_JPRB, 0.79737E+00_JPRB, 0.79253E+00_JPRB, 0.78744E+00_JPRB ,&
&  0.78340E+00_JPRB, 0.78010E+00_JPRB, 0.77565E+00_JPRB, 0.77058E+00_JPRB, 0.76537E+00_JPRB, 0.76130E+00_JPRB, 0.75703E+00_JPRB ,&
&  0.75275E+00_JPRB, 0.74759E+00_JPRB, 0.74199E+00_JPRB, 0.73327E+00_JPRB, 0.72434E+00_JPRB, 0.71571E+00_JPRB, 0.70966E+00_JPRB ,&
&  0.70510E+00_JPRB, 0.70112E+00_JPRB, 0.69654E+00_JPRB, 0.69228E+00_JPRB, 0.68829E+00_JPRB, 0.68486E+00_JPRB, 0.68111E+00_JPRB ,&
&  0.67926E+00_JPRB, 0.67888E+00_JPRB, 0.67853E+00_JPRB, 0.67771E+00_JPRB, 0.67872E+00_JPRB, 0.68101E+00_JPRB, 0.68305E+00_JPRB ,&
&  0.68457E+00_JPRB, 0.68717E+00_JPRB, 0.69172E+00_JPRB, 0.69546E+00_JPRB, 0.69858E+00_JPRB, 0.70286E+00_JPRB, 0.70611E+00_JPRB ,&
&  0.70764E+00_JPRB, 0.71032E+00_JPRB, 0.71627E+00_JPRB, 0.72268E+00_JPRB, 0.72944E+00_JPRB, 0.73674E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.75497E+00_JPRB, 0.76419E+00_JPRB, 0.77422E+00_JPRB, 0.78572E+00_JPRB, 0.79761E+00_JPRB, 0.81050E+00_JPRB, 0.82371E+00_JPRB ,&
&  0.83679E+00_JPRB, 0.85025E+00_JPRB, 0.86353E+00_JPRB, 0.87670E+00_JPRB, 0.88902E+00_JPRB, 0.89950E+00_JPRB, 0.91049E+00_JPRB ,&
&  0.92196E+00_JPRB, 0.93556E+00_JPRB, 0.94511E+00_JPRB, 0.95549E+00_JPRB, 0.96664E+00_JPRB, 0.98193E+00_JPRB, 0.98863E+00_JPRB ,&
&  0.99473E+00_JPRB, 0.10003E+01_JPRB, 0.10063E+01_JPRB, 0.10084E+01_JPRB, 0.10104E+01_JPRB, 0.10120E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10090E+01_JPRB, 0.10046E+01_JPRB, 0.99828E+00_JPRB, 0.98851E+00_JPRB, 0.97658E+00_JPRB, 0.96369E+00_JPRB, 0.94864E+00_JPRB ,&
&  0.93417E+00_JPRB, 0.91729E+00_JPRB, 0.89998E+00_JPRB, 0.88504E+00_JPRB, 0.87318E+00_JPRB, 0.85979E+00_JPRB, 0.84640E+00_JPRB ,&
&  0.83386E+00_JPRB, 0.82385E+00_JPRB, 0.81790E+00_JPRB, 0.81264E+00_JPRB, 0.80938E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.82034E+00_JPRB, 0.81867E+00_JPRB, 0.81722E+00_JPRB, 0.81618E+00_JPRB, 0.81428E+00_JPRB, 0.81166E+00_JPRB, 0.80836E+00_JPRB ,&
&  0.80472E+00_JPRB, 0.80109E+00_JPRB, 0.79702E+00_JPRB, 0.79340E+00_JPRB, 0.79003E+00_JPRB, 0.78658E+00_JPRB, 0.78249E+00_JPRB ,&
&  0.77790E+00_JPRB, 0.77251E+00_JPRB, 0.76681E+00_JPRB, 0.76068E+00_JPRB, 0.75573E+00_JPRB, 0.75193E+00_JPRB, 0.74897E+00_JPRB ,&
&  0.74634E+00_JPRB, 0.74356E+00_JPRB, 0.74064E+00_JPRB, 0.73843E+00_JPRB, 0.73781E+00_JPRB, 0.73696E+00_JPRB, 0.73641E+00_JPRB ,&
&  0.73605E+00_JPRB, 0.73645E+00_JPRB, 0.73646E+00_JPRB, 0.73633E+00_JPRB, 0.73632E+00_JPRB, 0.73776E+00_JPRB, 0.73886E+00_JPRB ,&
&  0.74027E+00_JPRB, 0.74128E+00_JPRB, 0.74201E+00_JPRB, 0.74268E+00_JPRB, 0.74370E+00_JPRB, 0.74512E+00_JPRB, 0.74668E+00_JPRB ,&
&  0.74856E+00_JPRB, 0.75111E+00_JPRB, 0.75503E+00_JPRB, 0.75964E+00_JPRB, 0.76423E+00_JPRB, 0.76894E+00_JPRB, 0.77469E+00_JPRB ,&
&  0.78046E+00_JPRB, 0.78628E+00_JPRB, 0.79203E+00_JPRB, 0.79830E+00_JPRB, 0.80478E+00_JPRB, 0.81106E+00_JPRB, 0.81752E+00_JPRB ,&
&  0.82414E+00_JPRB, 0.83085E+00_JPRB, 0.83788E+00_JPRB, 0.84494E+00_JPRB, 0.85135E+00_JPRB, 0.85697E+00_JPRB, 0.86294E+00_JPRB ,&
&  0.86924E+00_JPRB, 0.87545E+00_JPRB, 0.88012E+00_JPRB, 0.88519E+00_JPRB, 0.89031E+00_JPRB, 0.89522E+00_JPRB, 0.89873E+00_JPRB ,&
&  0.90244E+00_JPRB, 0.90658E+00_JPRB, 0.91085E+00_JPRB, 0.91195E+00_JPRB, 0.91268E+00_JPRB, 0.91294E+00_JPRB, 0.91205E+00_JPRB ,&
&  0.90938E+00_JPRB, 0.90642E+00_JPRB, 0.90320E+00_JPRB, 0.89920E+00_JPRB, 0.89349E+00_JPRB, 0.88743E+00_JPRB, 0.88072E+00_JPRB ,&
&  0.87384E+00_JPRB, 0.86593E+00_JPRB, 0.85797E+00_JPRB, 0.85118E+00_JPRB, 0.84599E+00_JPRB, 0.84090E+00_JPRB, 0.83571E+00_JPRB ,&
&  0.83044E+00_JPRB, 0.82719E+00_JPRB, 0.82571E+00_JPRB, 0.82422E+00_JPRB, 0.82257E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB, 0.80725E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB ,&
&  0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB, 0.25118E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.26395E+00_JPRB, 0.26424E+00_JPRB, 0.26433E+00_JPRB, 0.26429E+00_JPRB, 0.26230E+00_JPRB, 0.26262E+00_JPRB, 0.26264E+00_JPRB ,&
&  0.26285E+00_JPRB, 0.26310E+00_JPRB, 0.26372E+00_JPRB, 0.26445E+00_JPRB, 0.26518E+00_JPRB, 0.26585E+00_JPRB, 0.26645E+00_JPRB ,&
&  0.26682E+00_JPRB, 0.26729E+00_JPRB, 0.26598E+00_JPRB, 0.26519E+00_JPRB, 0.26509E+00_JPRB, 0.26519E+00_JPRB, 0.26517E+00_JPRB ,&
&  0.26511E+00_JPRB, 0.26499E+00_JPRB, 0.26452E+00_JPRB, 0.26418E+00_JPRB, 0.26403E+00_JPRB, 0.26383E+00_JPRB, 0.26352E+00_JPRB ,&
&  0.26312E+00_JPRB, 0.26282E+00_JPRB, 0.26245E+00_JPRB, 0.26202E+00_JPRB, 0.26189E+00_JPRB, 0.26171E+00_JPRB, 0.26112E+00_JPRB ,&
&  0.26042E+00_JPRB, 0.25940E+00_JPRB, 0.25846E+00_JPRB, 0.25761E+00_JPRB, 0.25693E+00_JPRB, 0.25659E+00_JPRB, 0.25623E+00_JPRB ,&
&  0.25575E+00_JPRB, 0.25509E+00_JPRB, 0.25445E+00_JPRB, 0.25369E+00_JPRB, 0.25300E+00_JPRB, 0.25239E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25087E+00_JPRB, 0.25005E+00_JPRB, 0.24921E+00_JPRB, 0.24828E+00_JPRB, 0.24738E+00_JPRB, 0.24647E+00_JPRB, 0.24547E+00_JPRB ,&
&  0.24449E+00_JPRB, 0.24406E+00_JPRB, 0.24357E+00_JPRB, 0.24316E+00_JPRB, 0.24284E+00_JPRB, 0.24260E+00_JPRB, 0.24236E+00_JPRB ,&
&  0.24218E+00_JPRB, 0.24197E+00_JPRB, 0.24187E+00_JPRB, 0.24178E+00_JPRB, 0.24178E+00_JPRB, 0.24184E+00_JPRB, 0.24202E+00_JPRB ,&
&  0.24222E+00_JPRB, 0.24248E+00_JPRB, 0.24283E+00_JPRB, 0.24341E+00_JPRB, 0.24398E+00_JPRB, 0.24464E+00_JPRB, 0.24530E+00_JPRB ,&
&  0.24602E+00_JPRB, 0.24673E+00_JPRB, 0.24746E+00_JPRB, 0.24815E+00_JPRB, 0.24908E+00_JPRB, 0.24998E+00_JPRB, 0.25097E+00_JPRB ,&
&  0.25195E+00_JPRB, 0.25301E+00_JPRB, 0.25406E+00_JPRB, 0.25518E+00_JPRB, 0.25646E+00_JPRB, 0.25684E+00_JPRB, 0.25724E+00_JPRB ,&
&  0.25773E+00_JPRB, 0.25845E+00_JPRB, 0.25910E+00_JPRB, 0.25996E+00_JPRB, 0.26141E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.29084E+00_JPRB, 0.29741E+00_JPRB, 0.30176E+00_JPRB, 0.30639E+00_JPRB, 0.30437E+00_JPRB, 0.34278E+00_JPRB, 0.34181E+00_JPRB ,&
&  0.34072E+00_JPRB, 0.32669E+00_JPRB, 0.32451E+00_JPRB, 0.31983E+00_JPRB, 0.31428E+00_JPRB, 0.30817E+00_JPRB, 0.30289E+00_JPRB ,&
&  0.29805E+00_JPRB, 0.29265E+00_JPRB, 0.28826E+00_JPRB, 0.28420E+00_JPRB, 0.28087E+00_JPRB, 0.27791E+00_JPRB, 0.27583E+00_JPRB ,&
&  0.27532E+00_JPRB, 0.27682E+00_JPRB, 0.27999E+00_JPRB, 0.28174E+00_JPRB, 0.28245E+00_JPRB, 0.28304E+00_JPRB, 0.28364E+00_JPRB ,&
&  0.28363E+00_JPRB, 0.28356E+00_JPRB, 0.28236E+00_JPRB, 0.28014E+00_JPRB, 0.27821E+00_JPRB, 0.27671E+00_JPRB, 0.27555E+00_JPRB ,&
&  0.27445E+00_JPRB, 0.27337E+00_JPRB, 0.27221E+00_JPRB, 0.27151E+00_JPRB, 0.27047E+00_JPRB, 0.26901E+00_JPRB, 0.26693E+00_JPRB ,&
&  0.26482E+00_JPRB, 0.26267E+00_JPRB, 0.26008E+00_JPRB, 0.25801E+00_JPRB, 0.25600E+00_JPRB, 0.25396E+00_JPRB, 0.25192E+00_JPRB ,&
&  0.25077E+00_JPRB, 0.24965E+00_JPRB, 0.24823E+00_JPRB, 0.24645E+00_JPRB, 0.24509E+00_JPRB, 0.24366E+00_JPRB, 0.24241E+00_JPRB ,&
&  0.24159E+00_JPRB, 0.24074E+00_JPRB, 0.23987E+00_JPRB, 0.23896E+00_JPRB, 0.23831E+00_JPRB, 0.23755E+00_JPRB, 0.23685E+00_JPRB ,&
&  0.23623E+00_JPRB, 0.23552E+00_JPRB, 0.23535E+00_JPRB, 0.23528E+00_JPRB, 0.23535E+00_JPRB, 0.23540E+00_JPRB, 0.23572E+00_JPRB ,&
&  0.23619E+00_JPRB, 0.23689E+00_JPRB, 0.23775E+00_JPRB, 0.23855E+00_JPRB, 0.23933E+00_JPRB, 0.24014E+00_JPRB, 0.24132E+00_JPRB ,&
&  0.24261E+00_JPRB, 0.24397E+00_JPRB, 0.24539E+00_JPRB, 0.24649E+00_JPRB, 0.24884E+00_JPRB, 0.25106E+00_JPRB, 0.25342E+00_JPRB ,&
&  0.25612E+00_JPRB, 0.25935E+00_JPRB, 0.26281E+00_JPRB, 0.26651E+00_JPRB, 0.27140E+00_JPRB, 0.27446E+00_JPRB, 0.27668E+00_JPRB ,&
&  0.27912E+00_JPRB, 0.28325E+00_JPRB, 0.28478E+00_JPRB, 0.28612E+00_JPRB, 0.28815E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.36328E+00_JPRB, 0.36819E+00_JPRB, 0.37237E+00_JPRB, 0.37609E+00_JPRB, 0.37941E+00_JPRB, 0.38032E+00_JPRB, 0.37940E+00_JPRB ,&
&  0.37031E+00_JPRB, 0.36360E+00_JPRB, 0.35538E+00_JPRB, 0.34803E+00_JPRB, 0.34122E+00_JPRB, 0.33308E+00_JPRB, 0.32612E+00_JPRB ,&
&  0.31903E+00_JPRB, 0.31193E+00_JPRB, 0.30522E+00_JPRB, 0.29735E+00_JPRB, 0.28915E+00_JPRB, 0.28129E+00_JPRB, 0.27650E+00_JPRB ,&
&  0.27377E+00_JPRB, 0.27367E+00_JPRB, 0.27668E+00_JPRB, 0.28055E+00_JPRB, 0.28556E+00_JPRB, 0.29199E+00_JPRB, 0.29888E+00_JPRB ,&
&  0.30324E+00_JPRB, 0.30560E+00_JPRB, 0.30512E+00_JPRB, 0.30355E+00_JPRB, 0.30148E+00_JPRB, 0.29975E+00_JPRB, 0.29707E+00_JPRB ,&
&  0.29364E+00_JPRB, 0.29130E+00_JPRB, 0.29062E+00_JPRB, 0.29175E+00_JPRB, 0.29162E+00_JPRB, 0.28982E+00_JPRB, 0.28591E+00_JPRB ,&
&  0.28150E+00_JPRB, 0.27704E+00_JPRB, 0.27244E+00_JPRB, 0.26814E+00_JPRB, 0.26463E+00_JPRB, 0.26093E+00_JPRB, 0.25692E+00_JPRB ,&
&  0.25440E+00_JPRB, 0.25174E+00_JPRB, 0.24880E+00_JPRB, 0.24603E+00_JPRB, 0.24372E+00_JPRB, 0.24194E+00_JPRB, 0.24044E+00_JPRB ,&
&  0.23929E+00_JPRB, 0.23841E+00_JPRB, 0.23770E+00_JPRB, 0.23718E+00_JPRB, 0.23668E+00_JPRB, 0.23590E+00_JPRB, 0.23512E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23408E+00_JPRB, 0.23360E+00_JPRB, 0.23362E+00_JPRB, 0.23388E+00_JPRB, 0.23404E+00_JPRB, 0.23456E+00_JPRB ,&
&  0.23569E+00_JPRB, 0.23688E+00_JPRB, 0.23749E+00_JPRB, 0.23861E+00_JPRB, 0.23969E+00_JPRB, 0.24110E+00_JPRB, 0.24295E+00_JPRB ,&
&  0.24412E+00_JPRB, 0.24541E+00_JPRB, 0.24692E+00_JPRB, 0.24854E+00_JPRB, 0.25125E+00_JPRB, 0.25419E+00_JPRB, 0.25747E+00_JPRB ,&
&  0.26168E+00_JPRB, 0.26579E+00_JPRB, 0.27048E+00_JPRB, 0.27604E+00_JPRB, 0.28597E+00_JPRB, 0.29269E+00_JPRB, 0.30124E+00_JPRB ,&
&  0.30972E+00_JPRB, 0.31426E+00_JPRB, 0.34856E+00_JPRB, 0.35122E+00_JPRB, 0.35665E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.37356E+00_JPRB, 0.38031E+00_JPRB, 0.38710E+00_JPRB, 0.39224E+00_JPRB, 0.39589E+00_JPRB, 0.39900E+00_JPRB, 0.39740E+00_JPRB ,&
&  0.39131E+00_JPRB, 0.37633E+00_JPRB, 0.36497E+00_JPRB, 0.35378E+00_JPRB, 0.34435E+00_JPRB, 0.33535E+00_JPRB, 0.32696E+00_JPRB ,&
&  0.31892E+00_JPRB, 0.31225E+00_JPRB, 0.30671E+00_JPRB, 0.30186E+00_JPRB, 0.29526E+00_JPRB, 0.28946E+00_JPRB, 0.28479E+00_JPRB ,&
&  0.28092E+00_JPRB, 0.28079E+00_JPRB, 0.28311E+00_JPRB, 0.28753E+00_JPRB, 0.29578E+00_JPRB, 0.30409E+00_JPRB, 0.31340E+00_JPRB ,&
&  0.32764E+00_JPRB, 0.33127E+00_JPRB, 0.33021E+00_JPRB, 0.32956E+00_JPRB, 0.32461E+00_JPRB, 0.31845E+00_JPRB, 0.30924E+00_JPRB ,&
&  0.31027E+00_JPRB, 0.30830E+00_JPRB, 0.31556E+00_JPRB, 0.31568E+00_JPRB, 0.31439E+00_JPRB, 0.31457E+00_JPRB, 0.30717E+00_JPRB ,&
&  0.30184E+00_JPRB, 0.29271E+00_JPRB, 0.28565E+00_JPRB, 0.27828E+00_JPRB, 0.27093E+00_JPRB, 0.26657E+00_JPRB, 0.26088E+00_JPRB ,&
&  0.25668E+00_JPRB, 0.25355E+00_JPRB, 0.24949E+00_JPRB, 0.24692E+00_JPRB, 0.24471E+00_JPRB, 0.24240E+00_JPRB, 0.24121E+00_JPRB ,&
&  0.24030E+00_JPRB, 0.23998E+00_JPRB, 0.24011E+00_JPRB, 0.24032E+00_JPRB, 0.24082E+00_JPRB, 0.24175E+00_JPRB, 0.24245E+00_JPRB ,&
&  0.24322E+00_JPRB, 0.24349E+00_JPRB, 0.24311E+00_JPRB, 0.24270E+00_JPRB, 0.24155E+00_JPRB, 0.24080E+00_JPRB, 0.23980E+00_JPRB ,&
&  0.23920E+00_JPRB, 0.23952E+00_JPRB, 0.23982E+00_JPRB, 0.24051E+00_JPRB, 0.24122E+00_JPRB, 0.24278E+00_JPRB, 0.24425E+00_JPRB ,&
&  0.24582E+00_JPRB, 0.24777E+00_JPRB, 0.24937E+00_JPRB, 0.25141E+00_JPRB, 0.25384E+00_JPRB, 0.25548E+00_JPRB, 0.25834E+00_JPRB ,&
&  0.26115E+00_JPRB, 0.26466E+00_JPRB, 0.27069E+00_JPRB, 0.27735E+00_JPRB, 0.28632E+00_JPRB, 0.29732E+00_JPRB, 0.31117E+00_JPRB ,&
&  0.32890E+00_JPRB, 0.35368E+00_JPRB, 0.35531E+00_JPRB, 0.36292E+00_JPRB, 0.36820E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.37896E+00_JPRB, 0.38156E+00_JPRB, 0.38553E+00_JPRB, 0.39098E+00_JPRB, 0.39784E+00_JPRB, 0.40409E+00_JPRB, 0.40556E+00_JPRB ,&
&  0.39618E+00_JPRB, 0.38327E+00_JPRB, 0.36503E+00_JPRB, 0.35059E+00_JPRB, 0.33659E+00_JPRB, 0.32611E+00_JPRB, 0.31761E+00_JPRB ,&
&  0.31216E+00_JPRB, 0.31170E+00_JPRB, 0.31209E+00_JPRB, 0.31330E+00_JPRB, 0.31224E+00_JPRB, 0.31328E+00_JPRB, 0.31359E+00_JPRB ,&
&  0.31234E+00_JPRB, 0.31222E+00_JPRB, 0.31214E+00_JPRB, 0.31563E+00_JPRB, 0.32246E+00_JPRB, 0.32878E+00_JPRB, 0.33505E+00_JPRB ,&
&  0.34239E+00_JPRB, 0.34451E+00_JPRB, 0.34872E+00_JPRB, 0.34744E+00_JPRB, 0.34397E+00_JPRB, 0.33949E+00_JPRB, 0.33319E+00_JPRB ,&
&  0.32871E+00_JPRB, 0.32475E+00_JPRB, 0.32342E+00_JPRB, 0.32381E+00_JPRB, 0.32385E+00_JPRB, 0.32590E+00_JPRB, 0.31995E+00_JPRB ,&
&  0.31278E+00_JPRB, 0.30237E+00_JPRB, 0.29033E+00_JPRB, 0.28141E+00_JPRB, 0.27288E+00_JPRB, 0.26674E+00_JPRB, 0.26126E+00_JPRB ,&
&  0.25619E+00_JPRB, 0.25280E+00_JPRB, 0.24928E+00_JPRB, 0.24669E+00_JPRB, 0.24587E+00_JPRB, 0.24407E+00_JPRB, 0.24307E+00_JPRB ,&
&  0.24267E+00_JPRB, 0.24310E+00_JPRB, 0.24476E+00_JPRB, 0.24626E+00_JPRB, 0.24832E+00_JPRB, 0.25065E+00_JPRB, 0.25263E+00_JPRB ,&
&  0.25527E+00_JPRB, 0.25741E+00_JPRB, 0.25767E+00_JPRB, 0.25707E+00_JPRB, 0.25496E+00_JPRB, 0.25216E+00_JPRB, 0.24970E+00_JPRB ,&
&  0.24718E+00_JPRB, 0.24542E+00_JPRB, 0.24477E+00_JPRB, 0.24391E+00_JPRB, 0.24435E+00_JPRB, 0.24551E+00_JPRB, 0.24639E+00_JPRB ,&
&  0.24811E+00_JPRB, 0.24952E+00_JPRB, 0.25116E+00_JPRB, 0.25398E+00_JPRB, 0.25660E+00_JPRB, 0.25893E+00_JPRB, 0.26132E+00_JPRB ,&
&  0.26148E+00_JPRB, 0.26321E+00_JPRB, 0.26555E+00_JPRB, 0.26976E+00_JPRB, 0.28210E+00_JPRB, 0.29079E+00_JPRB, 0.31079E+00_JPRB ,&
&  0.32809E+00_JPRB, 0.36305E+00_JPRB, 0.36303E+00_JPRB, 0.36884E+00_JPRB, 0.37525E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.38272E+00_JPRB, 0.38708E+00_JPRB, 0.38784E+00_JPRB, 0.39114E+00_JPRB, 0.39594E+00_JPRB, 0.40152E+00_JPRB, 0.40619E+00_JPRB ,&
&  0.41138E+00_JPRB, 0.41110E+00_JPRB, 0.40598E+00_JPRB, 0.39874E+00_JPRB, 0.39268E+00_JPRB, 0.37588E+00_JPRB, 0.36703E+00_JPRB ,&
&  0.35976E+00_JPRB, 0.35938E+00_JPRB, 0.35989E+00_JPRB, 0.36156E+00_JPRB, 0.36260E+00_JPRB, 0.36761E+00_JPRB, 0.36908E+00_JPRB ,&
&  0.36650E+00_JPRB, 0.36133E+00_JPRB, 0.35609E+00_JPRB, 0.35518E+00_JPRB, 0.35780E+00_JPRB, 0.35975E+00_JPRB, 0.35685E+00_JPRB ,&
&  0.35322E+00_JPRB, 0.35300E+00_JPRB, 0.35360E+00_JPRB, 0.35505E+00_JPRB, 0.36466E+00_JPRB, 0.35976E+00_JPRB, 0.35529E+00_JPRB ,&
&  0.34819E+00_JPRB, 0.34207E+00_JPRB, 0.33783E+00_JPRB, 0.33815E+00_JPRB, 0.34112E+00_JPRB, 0.34060E+00_JPRB, 0.34315E+00_JPRB ,&
&  0.33162E+00_JPRB, 0.32207E+00_JPRB, 0.30720E+00_JPRB, 0.29541E+00_JPRB, 0.28355E+00_JPRB, 0.27543E+00_JPRB, 0.26789E+00_JPRB ,&
&  0.26147E+00_JPRB, 0.25608E+00_JPRB, 0.25101E+00_JPRB, 0.24939E+00_JPRB, 0.25031E+00_JPRB, 0.25038E+00_JPRB, 0.24995E+00_JPRB ,&
&  0.25049E+00_JPRB, 0.25145E+00_JPRB, 0.25218E+00_JPRB, 0.25318E+00_JPRB, 0.25479E+00_JPRB, 0.25744E+00_JPRB, 0.26101E+00_JPRB ,&
&  0.26660E+00_JPRB, 0.27103E+00_JPRB, 0.27342E+00_JPRB, 0.27462E+00_JPRB, 0.27152E+00_JPRB, 0.26618E+00_JPRB, 0.26077E+00_JPRB ,&
&  0.25535E+00_JPRB, 0.25280E+00_JPRB, 0.25052E+00_JPRB, 0.24945E+00_JPRB, 0.24923E+00_JPRB, 0.25010E+00_JPRB, 0.25191E+00_JPRB ,&
&  0.25369E+00_JPRB, 0.25596E+00_JPRB, 0.25746E+00_JPRB, 0.25928E+00_JPRB, 0.26149E+00_JPRB, 0.26341E+00_JPRB, 0.26542E+00_JPRB ,&
&  0.26768E+00_JPRB, 0.26945E+00_JPRB, 0.27246E+00_JPRB, 0.27618E+00_JPRB, 0.28328E+00_JPRB, 0.29162E+00_JPRB, 0.30647E+00_JPRB ,&
&  0.32389E+00_JPRB, 0.35613E+00_JPRB, 0.36644E+00_JPRB, 0.37363E+00_JPRB, 0.38088E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.38086E+00_JPRB, 0.38762E+00_JPRB, 0.39264E+00_JPRB, 0.40266E+00_JPRB, 0.40550E+00_JPRB, 0.40832E+00_JPRB, 0.41071E+00_JPRB ,&
&  0.41690E+00_JPRB, 0.42051E+00_JPRB, 0.41831E+00_JPRB, 0.41769E+00_JPRB, 0.41680E+00_JPRB, 0.41554E+00_JPRB, 0.41648E+00_JPRB ,&
&  0.41650E+00_JPRB, 0.41858E+00_JPRB, 0.41624E+00_JPRB, 0.41346E+00_JPRB, 0.41536E+00_JPRB, 0.42016E+00_JPRB, 0.42338E+00_JPRB ,&
&  0.41823E+00_JPRB, 0.41685E+00_JPRB, 0.42541E+00_JPRB, 0.42756E+00_JPRB, 0.42882E+00_JPRB, 0.43085E+00_JPRB, 0.43062E+00_JPRB ,&
&  0.41681E+00_JPRB, 0.39143E+00_JPRB, 0.36388E+00_JPRB, 0.36545E+00_JPRB, 0.38008E+00_JPRB, 0.37448E+00_JPRB, 0.37284E+00_JPRB ,&
&  0.36997E+00_JPRB, 0.36548E+00_JPRB, 0.36120E+00_JPRB, 0.35917E+00_JPRB, 0.35639E+00_JPRB, 0.35172E+00_JPRB, 0.35009E+00_JPRB ,&
&  0.34897E+00_JPRB, 0.35753E+00_JPRB, 0.35285E+00_JPRB, 0.33102E+00_JPRB, 0.31454E+00_JPRB, 0.30557E+00_JPRB, 0.29401E+00_JPRB ,&
&  0.28059E+00_JPRB, 0.27279E+00_JPRB, 0.27160E+00_JPRB, 0.26980E+00_JPRB, 0.27130E+00_JPRB, 0.27241E+00_JPRB, 0.27354E+00_JPRB ,&
&  0.27439E+00_JPRB, 0.27417E+00_JPRB, 0.26957E+00_JPRB, 0.26362E+00_JPRB, 0.26036E+00_JPRB, 0.26033E+00_JPRB, 0.26301E+00_JPRB ,&
&  0.26888E+00_JPRB, 0.27514E+00_JPRB, 0.28947E+00_JPRB, 0.29721E+00_JPRB, 0.28858E+00_JPRB, 0.28316E+00_JPRB, 0.27709E+00_JPRB ,&
&  0.26670E+00_JPRB, 0.26271E+00_JPRB, 0.25942E+00_JPRB, 0.25762E+00_JPRB, 0.25848E+00_JPRB, 0.26177E+00_JPRB, 0.26341E+00_JPRB ,&
&  0.26584E+00_JPRB, 0.26849E+00_JPRB, 0.26957E+00_JPRB, 0.27044E+00_JPRB, 0.27119E+00_JPRB, 0.27311E+00_JPRB, 0.27507E+00_JPRB ,&
&  0.27669E+00_JPRB, 0.27958E+00_JPRB, 0.28311E+00_JPRB, 0.28780E+00_JPRB, 0.29194E+00_JPRB, 0.29440E+00_JPRB, 0.30160E+00_JPRB ,&
&  0.31004E+00_JPRB, 0.32743E+00_JPRB, 0.35581E+00_JPRB, 0.38037E+00_JPRB, 0.37613E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.38959E+00_JPRB, 0.39465E+00_JPRB, 0.40150E+00_JPRB, 0.40911E+00_JPRB, 0.41452E+00_JPRB, 0.41692E+00_JPRB, 0.41948E+00_JPRB ,&
&  0.42282E+00_JPRB, 0.42423E+00_JPRB, 0.42529E+00_JPRB, 0.42806E+00_JPRB, 0.43367E+00_JPRB, 0.43665E+00_JPRB, 0.44142E+00_JPRB ,&
&  0.44328E+00_JPRB, 0.44108E+00_JPRB, 0.43427E+00_JPRB, 0.43226E+00_JPRB, 0.43582E+00_JPRB, 0.44059E+00_JPRB, 0.44586E+00_JPRB ,&
&  0.45132E+00_JPRB, 0.45632E+00_JPRB, 0.45870E+00_JPRB, 0.45940E+00_JPRB, 0.45962E+00_JPRB, 0.45952E+00_JPRB, 0.45693E+00_JPRB ,&
&  0.45181E+00_JPRB, 0.41231E+00_JPRB, 0.38576E+00_JPRB, 0.38906E+00_JPRB, 0.40060E+00_JPRB, 0.39498E+00_JPRB, 0.39343E+00_JPRB ,&
&  0.39186E+00_JPRB, 0.38834E+00_JPRB, 0.38310E+00_JPRB, 0.37885E+00_JPRB, 0.37357E+00_JPRB, 0.36675E+00_JPRB, 0.36122E+00_JPRB ,&
&  0.35991E+00_JPRB, 0.36471E+00_JPRB, 0.36624E+00_JPRB, 0.36842E+00_JPRB, 0.36718E+00_JPRB, 0.36914E+00_JPRB, 0.36794E+00_JPRB ,&
&  0.36255E+00_JPRB, 0.35639E+00_JPRB, 0.35434E+00_JPRB, 0.35394E+00_JPRB, 0.35502E+00_JPRB, 0.35765E+00_JPRB, 0.35747E+00_JPRB ,&
&  0.34438E+00_JPRB, 0.33285E+00_JPRB, 0.31435E+00_JPRB, 0.29640E+00_JPRB, 0.28332E+00_JPRB, 0.27738E+00_JPRB, 0.27305E+00_JPRB ,&
&  0.27217E+00_JPRB, 0.27801E+00_JPRB, 0.29473E+00_JPRB, 0.31504E+00_JPRB, 0.32875E+00_JPRB, 0.31049E+00_JPRB, 0.29495E+00_JPRB ,&
&  0.28548E+00_JPRB, 0.28044E+00_JPRB, 0.27619E+00_JPRB, 0.27470E+00_JPRB, 0.27512E+00_JPRB, 0.27731E+00_JPRB, 0.28048E+00_JPRB ,&
&  0.28512E+00_JPRB, 0.28954E+00_JPRB, 0.29232E+00_JPRB, 0.29366E+00_JPRB, 0.29439E+00_JPRB, 0.29514E+00_JPRB, 0.29453E+00_JPRB ,&
&  0.29431E+00_JPRB, 0.29763E+00_JPRB, 0.30439E+00_JPRB, 0.31315E+00_JPRB, 0.32133E+00_JPRB, 0.33044E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.35754E+00_JPRB, 0.36928E+00_JPRB, 0.37352E+00_JPRB, 0.37960E+00_JPRB, 0.38487E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.41106E+00_JPRB, 0.41552E+00_JPRB, 0.41969E+00_JPRB, 0.42425E+00_JPRB, 0.43065E+00_JPRB, 0.43385E+00_JPRB, 0.43484E+00_JPRB ,&
&  0.43618E+00_JPRB, 0.43678E+00_JPRB, 0.43831E+00_JPRB, 0.44266E+00_JPRB, 0.44870E+00_JPRB, 0.45297E+00_JPRB, 0.45864E+00_JPRB ,&
&  0.45870E+00_JPRB, 0.45001E+00_JPRB, 0.44214E+00_JPRB, 0.44185E+00_JPRB, 0.44736E+00_JPRB, 0.45310E+00_JPRB, 0.46036E+00_JPRB ,&
&  0.46755E+00_JPRB, 0.47279E+00_JPRB, 0.47573E+00_JPRB, 0.47721E+00_JPRB, 0.47918E+00_JPRB, 0.47882E+00_JPRB, 0.47584E+00_JPRB ,&
&  0.47175E+00_JPRB, 0.46283E+00_JPRB, 0.44333E+00_JPRB, 0.44054E+00_JPRB, 0.43179E+00_JPRB, 0.42418E+00_JPRB, 0.41887E+00_JPRB ,&
&  0.41457E+00_JPRB, 0.41108E+00_JPRB, 0.40715E+00_JPRB, 0.40378E+00_JPRB, 0.39994E+00_JPRB, 0.39303E+00_JPRB, 0.38950E+00_JPRB ,&
&  0.38626E+00_JPRB, 0.38359E+00_JPRB, 0.38099E+00_JPRB, 0.38122E+00_JPRB, 0.38164E+00_JPRB, 0.38451E+00_JPRB, 0.38596E+00_JPRB ,&
&  0.38290E+00_JPRB, 0.37925E+00_JPRB, 0.37644E+00_JPRB, 0.37808E+00_JPRB, 0.38031E+00_JPRB, 0.38186E+00_JPRB, 0.38242E+00_JPRB ,&
&  0.37993E+00_JPRB, 0.37534E+00_JPRB, 0.37061E+00_JPRB, 0.36515E+00_JPRB, 0.35973E+00_JPRB, 0.33406E+00_JPRB, 0.32283E+00_JPRB ,&
&  0.31958E+00_JPRB, 0.32856E+00_JPRB, 0.34082E+00_JPRB, 0.33961E+00_JPRB, 0.34182E+00_JPRB, 0.34197E+00_JPRB, 0.33943E+00_JPRB ,&
&  0.33041E+00_JPRB, 0.31687E+00_JPRB, 0.30783E+00_JPRB, 0.30380E+00_JPRB, 0.30084E+00_JPRB, 0.30208E+00_JPRB, 0.30806E+00_JPRB ,&
&  0.31815E+00_JPRB, 0.32980E+00_JPRB, 0.33862E+00_JPRB, 0.34264E+00_JPRB, 0.34154E+00_JPRB, 0.33846E+00_JPRB, 0.33432E+00_JPRB ,&
&  0.32980E+00_JPRB, 0.33543E+00_JPRB, 0.34955E+00_JPRB, 0.36941E+00_JPRB, 0.37203E+00_JPRB, 0.37705E+00_JPRB, 0.38050E+00_JPRB ,&
&  0.38578E+00_JPRB, 0.39242E+00_JPRB, 0.39819E+00_JPRB, 0.40429E+00_JPRB, 0.40807E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.43838E+00_JPRB, 0.44093E+00_JPRB, 0.44345E+00_JPRB, 0.44520E+00_JPRB, 0.44800E+00_JPRB, 0.45287E+00_JPRB, 0.45482E+00_JPRB ,&
&  0.45294E+00_JPRB, 0.45079E+00_JPRB, 0.45058E+00_JPRB, 0.45394E+00_JPRB, 0.46053E+00_JPRB, 0.46739E+00_JPRB, 0.47505E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.45861E+00_JPRB, 0.45412E+00_JPRB, 0.45615E+00_JPRB, 0.46100E+00_JPRB, 0.46640E+00_JPRB, 0.47310E+00_JPRB ,&
&  0.47948E+00_JPRB, 0.48369E+00_JPRB, 0.48807E+00_JPRB, 0.49200E+00_JPRB, 0.49455E+00_JPRB, 0.49281E+00_JPRB, 0.48839E+00_JPRB ,&
&  0.48312E+00_JPRB, 0.47782E+00_JPRB, 0.47312E+00_JPRB, 0.45736E+00_JPRB, 0.45478E+00_JPRB, 0.44580E+00_JPRB, 0.43831E+00_JPRB ,&
&  0.43311E+00_JPRB, 0.42880E+00_JPRB, 0.42606E+00_JPRB, 0.42303E+00_JPRB, 0.41881E+00_JPRB, 0.41166E+00_JPRB, 0.40967E+00_JPRB ,&
&  0.40880E+00_JPRB, 0.40993E+00_JPRB, 0.40540E+00_JPRB, 0.40303E+00_JPRB, 0.40042E+00_JPRB, 0.39840E+00_JPRB, 0.40272E+00_JPRB ,&
&  0.40795E+00_JPRB, 0.40551E+00_JPRB, 0.40160E+00_JPRB, 0.40129E+00_JPRB, 0.40556E+00_JPRB, 0.40720E+00_JPRB, 0.40814E+00_JPRB ,&
&  0.40991E+00_JPRB, 0.41048E+00_JPRB, 0.40595E+00_JPRB, 0.40088E+00_JPRB, 0.39231E+00_JPRB, 0.38156E+00_JPRB, 0.37866E+00_JPRB ,&
&  0.37566E+00_JPRB, 0.36984E+00_JPRB, 0.36636E+00_JPRB, 0.36524E+00_JPRB, 0.36482E+00_JPRB, 0.36560E+00_JPRB, 0.36564E+00_JPRB ,&
&  0.36557E+00_JPRB, 0.36200E+00_JPRB, 0.35674E+00_JPRB, 0.35217E+00_JPRB, 0.34597E+00_JPRB, 0.34842E+00_JPRB, 0.35799E+00_JPRB ,&
&  0.36998E+00_JPRB, 0.37729E+00_JPRB, 0.38206E+00_JPRB, 0.38550E+00_JPRB, 0.38856E+00_JPRB, 0.38998E+00_JPRB, 0.39230E+00_JPRB ,&
&  0.39542E+00_JPRB, 0.39685E+00_JPRB, 0.39544E+00_JPRB, 0.39541E+00_JPRB, 0.39842E+00_JPRB, 0.40347E+00_JPRB, 0.40963E+00_JPRB ,&
&  0.41581E+00_JPRB, 0.42110E+00_JPRB, 0.42557E+00_JPRB, 0.43116E+00_JPRB, 0.43487E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.47250E+00_JPRB, 0.47281E+00_JPRB, 0.47352E+00_JPRB, 0.47500E+00_JPRB, 0.47602E+00_JPRB, 0.47513E+00_JPRB, 0.47478E+00_JPRB ,&
&  0.47148E+00_JPRB, 0.46668E+00_JPRB, 0.46364E+00_JPRB, 0.46494E+00_JPRB, 0.47390E+00_JPRB, 0.48546E+00_JPRB, 0.49244E+00_JPRB ,&
&  0.48819E+00_JPRB, 0.47594E+00_JPRB, 0.47297E+00_JPRB, 0.47532E+00_JPRB, 0.47756E+00_JPRB, 0.48173E+00_JPRB, 0.48791E+00_JPRB ,&
&  0.49355E+00_JPRB, 0.49741E+00_JPRB, 0.50158E+00_JPRB, 0.50328E+00_JPRB, 0.50348E+00_JPRB, 0.49633E+00_JPRB, 0.48509E+00_JPRB ,&
&  0.47426E+00_JPRB, 0.46650E+00_JPRB, 0.46177E+00_JPRB, 0.45824E+00_JPRB, 0.45317E+00_JPRB, 0.44807E+00_JPRB, 0.44147E+00_JPRB ,&
&  0.43823E+00_JPRB, 0.43792E+00_JPRB, 0.43880E+00_JPRB, 0.43691E+00_JPRB, 0.43437E+00_JPRB, 0.42930E+00_JPRB, 0.42510E+00_JPRB ,&
&  0.42353E+00_JPRB, 0.42483E+00_JPRB, 0.42620E+00_JPRB, 0.42542E+00_JPRB, 0.42212E+00_JPRB, 0.41983E+00_JPRB, 0.42132E+00_JPRB ,&
&  0.42726E+00_JPRB, 0.42715E+00_JPRB, 0.42621E+00_JPRB, 0.42702E+00_JPRB, 0.42923E+00_JPRB, 0.43122E+00_JPRB, 0.43065E+00_JPRB ,&
&  0.43000E+00_JPRB, 0.42998E+00_JPRB, 0.43031E+00_JPRB, 0.43226E+00_JPRB, 0.43247E+00_JPRB, 0.43047E+00_JPRB, 0.42503E+00_JPRB ,&
&  0.41668E+00_JPRB, 0.40611E+00_JPRB, 0.40174E+00_JPRB, 0.39861E+00_JPRB, 0.39555E+00_JPRB, 0.39481E+00_JPRB, 0.39608E+00_JPRB ,&
&  0.39463E+00_JPRB, 0.38844E+00_JPRB, 0.38562E+00_JPRB, 0.38602E+00_JPRB, 0.38482E+00_JPRB, 0.38678E+00_JPRB, 0.39034E+00_JPRB ,&
&  0.40032E+00_JPRB, 0.40583E+00_JPRB, 0.40745E+00_JPRB, 0.40529E+00_JPRB, 0.40555E+00_JPRB, 0.40727E+00_JPRB, 0.41159E+00_JPRB ,&
&  0.41818E+00_JPRB, 0.42530E+00_JPRB, 0.43261E+00_JPRB, 0.44067E+00_JPRB, 0.45083E+00_JPRB, 0.46059E+00_JPRB, 0.46437E+00_JPRB ,&
&  0.46463E+00_JPRB, 0.46505E+00_JPRB, 0.46633E+00_JPRB, 0.46782E+00_JPRB, 0.47097E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.50124E+00_JPRB, 0.50203E+00_JPRB, 0.50113E+00_JPRB, 0.50023E+00_JPRB, 0.49900E+00_JPRB, 0.49770E+00_JPRB, 0.49547E+00_JPRB ,&
&  0.49018E+00_JPRB, 0.48344E+00_JPRB, 0.47940E+00_JPRB, 0.47751E+00_JPRB, 0.48301E+00_JPRB, 0.49129E+00_JPRB, 0.49877E+00_JPRB ,&
&  0.49801E+00_JPRB, 0.48907E+00_JPRB, 0.48642E+00_JPRB, 0.48846E+00_JPRB, 0.49109E+00_JPRB, 0.49398E+00_JPRB, 0.49786E+00_JPRB ,&
&  0.50209E+00_JPRB, 0.50638E+00_JPRB, 0.50825E+00_JPRB, 0.50567E+00_JPRB, 0.49905E+00_JPRB, 0.48518E+00_JPRB, 0.47308E+00_JPRB ,&
&  0.46501E+00_JPRB, 0.45878E+00_JPRB, 0.45528E+00_JPRB, 0.45440E+00_JPRB, 0.45222E+00_JPRB, 0.44842E+00_JPRB, 0.44310E+00_JPRB ,&
&  0.43846E+00_JPRB, 0.43630E+00_JPRB, 0.43847E+00_JPRB, 0.44018E+00_JPRB, 0.44106E+00_JPRB, 0.44083E+00_JPRB, 0.43797E+00_JPRB ,&
&  0.43643E+00_JPRB, 0.43605E+00_JPRB, 0.43590E+00_JPRB, 0.43764E+00_JPRB, 0.43549E+00_JPRB, 0.43371E+00_JPRB, 0.43593E+00_JPRB ,&
&  0.44054E+00_JPRB, 0.44455E+00_JPRB, 0.44817E+00_JPRB, 0.45095E+00_JPRB, 0.45212E+00_JPRB, 0.45427E+00_JPRB, 0.45564E+00_JPRB ,&
&  0.45476E+00_JPRB, 0.45121E+00_JPRB, 0.44793E+00_JPRB, 0.44853E+00_JPRB, 0.45102E+00_JPRB, 0.45382E+00_JPRB, 0.45746E+00_JPRB ,&
&  0.45813E+00_JPRB, 0.45305E+00_JPRB, 0.44795E+00_JPRB, 0.44339E+00_JPRB, 0.43915E+00_JPRB, 0.43581E+00_JPRB, 0.43410E+00_JPRB ,&
&  0.43354E+00_JPRB, 0.43042E+00_JPRB, 0.42446E+00_JPRB, 0.41954E+00_JPRB, 0.41874E+00_JPRB, 0.41762E+00_JPRB, 0.41357E+00_JPRB ,&
&  0.41376E+00_JPRB, 0.41609E+00_JPRB, 0.41489E+00_JPRB, 0.41299E+00_JPRB, 0.41502E+00_JPRB, 0.42047E+00_JPRB, 0.43160E+00_JPRB ,&
&  0.44647E+00_JPRB, 0.46351E+00_JPRB, 0.47932E+00_JPRB, 0.49316E+00_JPRB, 0.50162E+00_JPRB, 0.50388E+00_JPRB, 0.50239E+00_JPRB ,&
&  0.50185E+00_JPRB, 0.50251E+00_JPRB, 0.50140E+00_JPRB, 0.49959E+00_JPRB, 0.49991E+00_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.53272E+00_JPRB, 0.53121E+00_JPRB, 0.52948E+00_JPRB, 0.52729E+00_JPRB, 0.52393E+00_JPRB, 0.52068E+00_JPRB, 0.51549E+00_JPRB ,&
&  0.51004E+00_JPRB, 0.50546E+00_JPRB, 0.50332E+00_JPRB, 0.50250E+00_JPRB, 0.50370E+00_JPRB, 0.50772E+00_JPRB, 0.51452E+00_JPRB ,&
&  0.51469E+00_JPRB, 0.50811E+00_JPRB, 0.50512E+00_JPRB, 0.50525E+00_JPRB, 0.50777E+00_JPRB, 0.50966E+00_JPRB, 0.51255E+00_JPRB ,&
&  0.51447E+00_JPRB, 0.51511E+00_JPRB, 0.51441E+00_JPRB, 0.50734E+00_JPRB, 0.49043E+00_JPRB, 0.48012E+00_JPRB, 0.47105E+00_JPRB ,&
&  0.46567E+00_JPRB, 0.46124E+00_JPRB, 0.45839E+00_JPRB, 0.45767E+00_JPRB, 0.45664E+00_JPRB, 0.45479E+00_JPRB, 0.45471E+00_JPRB ,&
&  0.45273E+00_JPRB, 0.45012E+00_JPRB, 0.44866E+00_JPRB, 0.44914E+00_JPRB, 0.45082E+00_JPRB, 0.45242E+00_JPRB, 0.45240E+00_JPRB ,&
&  0.45350E+00_JPRB, 0.45736E+00_JPRB, 0.45862E+00_JPRB, 0.45900E+00_JPRB, 0.45900E+00_JPRB, 0.45868E+00_JPRB, 0.45888E+00_JPRB ,&
&  0.46127E+00_JPRB, 0.46638E+00_JPRB, 0.47070E+00_JPRB, 0.47504E+00_JPRB, 0.47689E+00_JPRB, 0.47778E+00_JPRB, 0.47779E+00_JPRB ,&
&  0.47474E+00_JPRB, 0.47075E+00_JPRB, 0.46947E+00_JPRB, 0.47197E+00_JPRB, 0.47707E+00_JPRB, 0.48319E+00_JPRB, 0.48705E+00_JPRB ,&
&  0.48997E+00_JPRB, 0.49100E+00_JPRB, 0.48876E+00_JPRB, 0.48748E+00_JPRB, 0.48471E+00_JPRB, 0.48010E+00_JPRB, 0.47743E+00_JPRB ,&
&  0.47487E+00_JPRB, 0.47263E+00_JPRB, 0.47067E+00_JPRB, 0.46179E+00_JPRB, 0.45408E+00_JPRB, 0.44931E+00_JPRB, 0.44616E+00_JPRB ,&
&  0.44491E+00_JPRB, 0.44393E+00_JPRB, 0.44358E+00_JPRB, 0.44483E+00_JPRB, 0.44698E+00_JPRB, 0.45355E+00_JPRB, 0.46663E+00_JPRB ,&
&  0.48611E+00_JPRB, 0.50439E+00_JPRB, 0.51480E+00_JPRB, 0.52287E+00_JPRB, 0.52895E+00_JPRB, 0.53262E+00_JPRB, 0.53589E+00_JPRB ,&
&  0.53992E+00_JPRB, 0.54118E+00_JPRB, 0.53901E+00_JPRB, 0.53618E+00_JPRB, 0.53482E+00_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.56657E+00_JPRB, 0.55731E+00_JPRB, 0.54998E+00_JPRB, 0.54374E+00_JPRB, 0.53821E+00_JPRB, 0.53537E+00_JPRB, 0.53244E+00_JPRB ,&
&  0.52969E+00_JPRB, 0.52643E+00_JPRB, 0.52451E+00_JPRB, 0.52073E+00_JPRB, 0.51749E+00_JPRB, 0.52015E+00_JPRB, 0.52562E+00_JPRB ,&
&  0.52428E+00_JPRB, 0.51758E+00_JPRB, 0.51228E+00_JPRB, 0.51227E+00_JPRB, 0.51588E+00_JPRB, 0.51862E+00_JPRB, 0.52185E+00_JPRB ,&
&  0.52183E+00_JPRB, 0.52129E+00_JPRB, 0.51984E+00_JPRB, 0.50420E+00_JPRB, 0.48902E+00_JPRB, 0.48148E+00_JPRB, 0.47717E+00_JPRB ,&
&  0.47687E+00_JPRB, 0.48252E+00_JPRB, 0.46604E+00_JPRB, 0.47623E+00_JPRB, 0.47232E+00_JPRB, 0.46619E+00_JPRB, 0.46264E+00_JPRB ,&
&  0.46047E+00_JPRB, 0.45802E+00_JPRB, 0.45620E+00_JPRB, 0.45805E+00_JPRB, 0.46064E+00_JPRB, 0.46138E+00_JPRB, 0.46318E+00_JPRB ,&
&  0.46306E+00_JPRB, 0.46571E+00_JPRB, 0.46811E+00_JPRB, 0.47344E+00_JPRB, 0.47748E+00_JPRB, 0.48002E+00_JPRB, 0.48326E+00_JPRB ,&
&  0.48777E+00_JPRB, 0.49196E+00_JPRB, 0.49593E+00_JPRB, 0.49923E+00_JPRB, 0.50054E+00_JPRB, 0.50025E+00_JPRB, 0.49611E+00_JPRB ,&
&  0.49278E+00_JPRB, 0.49257E+00_JPRB, 0.49533E+00_JPRB, 0.50120E+00_JPRB, 0.50691E+00_JPRB, 0.50900E+00_JPRB, 0.51000E+00_JPRB ,&
&  0.51166E+00_JPRB, 0.51532E+00_JPRB, 0.51595E+00_JPRB, 0.51458E+00_JPRB, 0.51249E+00_JPRB, 0.50981E+00_JPRB, 0.50521E+00_JPRB ,&
&  0.49923E+00_JPRB, 0.49448E+00_JPRB, 0.48965E+00_JPRB, 0.47871E+00_JPRB, 0.47032E+00_JPRB, 0.46951E+00_JPRB, 0.47107E+00_JPRB ,&
&  0.47060E+00_JPRB, 0.46910E+00_JPRB, 0.47031E+00_JPRB, 0.47600E+00_JPRB, 0.47602E+00_JPRB, 0.48279E+00_JPRB, 0.49665E+00_JPRB ,&
&  0.51118E+00_JPRB, 0.52477E+00_JPRB, 0.53745E+00_JPRB, 0.55133E+00_JPRB, 0.56580E+00_JPRB, 0.57796E+00_JPRB, 0.58818E+00_JPRB ,&
&  0.59271E+00_JPRB, 0.59059E+00_JPRB, 0.58399E+00_JPRB, 0.57948E+00_JPRB, 0.57551E+00_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.62465E+00_JPRB, 0.60705E+00_JPRB, 0.59306E+00_JPRB, 0.58206E+00_JPRB, 0.57423E+00_JPRB, 0.57063E+00_JPRB, 0.56656E+00_JPRB ,&
&  0.56310E+00_JPRB, 0.56041E+00_JPRB, 0.55347E+00_JPRB, 0.54644E+00_JPRB, 0.54558E+00_JPRB, 0.55103E+00_JPRB, 0.55545E+00_JPRB ,&
&  0.55256E+00_JPRB, 0.54209E+00_JPRB, 0.53225E+00_JPRB, 0.53026E+00_JPRB, 0.53336E+00_JPRB, 0.53600E+00_JPRB, 0.53589E+00_JPRB ,&
&  0.53771E+00_JPRB, 0.53884E+00_JPRB, 0.53050E+00_JPRB, 0.51340E+00_JPRB, 0.50528E+00_JPRB, 0.50055E+00_JPRB, 0.50511E+00_JPRB ,&
&  0.51391E+00_JPRB, 0.50977E+00_JPRB, 0.52100E+00_JPRB, 0.51957E+00_JPRB, 0.51884E+00_JPRB, 0.51540E+00_JPRB, 0.50880E+00_JPRB ,&
&  0.50333E+00_JPRB, 0.49757E+00_JPRB, 0.49381E+00_JPRB, 0.49196E+00_JPRB, 0.49216E+00_JPRB, 0.49324E+00_JPRB, 0.49760E+00_JPRB ,&
&  0.49856E+00_JPRB, 0.49664E+00_JPRB, 0.49912E+00_JPRB, 0.50607E+00_JPRB, 0.51475E+00_JPRB, 0.51969E+00_JPRB, 0.52419E+00_JPRB ,&
&  0.52792E+00_JPRB, 0.53000E+00_JPRB, 0.53283E+00_JPRB, 0.53357E+00_JPRB, 0.52984E+00_JPRB, 0.52904E+00_JPRB, 0.52893E+00_JPRB ,&
&  0.53109E+00_JPRB, 0.53473E+00_JPRB, 0.53994E+00_JPRB, 0.54563E+00_JPRB, 0.54816E+00_JPRB, 0.54541E+00_JPRB, 0.54234E+00_JPRB ,&
&  0.54289E+00_JPRB, 0.54659E+00_JPRB, 0.55082E+00_JPRB, 0.55318E+00_JPRB, 0.55217E+00_JPRB, 0.54506E+00_JPRB, 0.53658E+00_JPRB ,&
&  0.52767E+00_JPRB, 0.51854E+00_JPRB, 0.51032E+00_JPRB, 0.50531E+00_JPRB, 0.50148E+00_JPRB, 0.49905E+00_JPRB, 0.49765E+00_JPRB ,&
&  0.49730E+00_JPRB, 0.49759E+00_JPRB, 0.50100E+00_JPRB, 0.50744E+00_JPRB, 0.51121E+00_JPRB, 0.51980E+00_JPRB, 0.53103E+00_JPRB ,&
&  0.54373E+00_JPRB, 0.55763E+00_JPRB, 0.57452E+00_JPRB, 0.59740E+00_JPRB, 0.62351E+00_JPRB, 0.64547E+00_JPRB, 0.66198E+00_JPRB ,&
&  0.67238E+00_JPRB, 0.67256E+00_JPRB, 0.66512E+00_JPRB, 0.65554E+00_JPRB, 0.64200E+00_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.68827E+00_JPRB, 0.66129E+00_JPRB, 0.63736E+00_JPRB, 0.61870E+00_JPRB, 0.60656E+00_JPRB, 0.60112E+00_JPRB, 0.59540E+00_JPRB ,&
&  0.59170E+00_JPRB, 0.58739E+00_JPRB, 0.58460E+00_JPRB, 0.58645E+00_JPRB, 0.58718E+00_JPRB, 0.58722E+00_JPRB, 0.58811E+00_JPRB ,&
&  0.58232E+00_JPRB, 0.56938E+00_JPRB, 0.55906E+00_JPRB, 0.55627E+00_JPRB, 0.55737E+00_JPRB, 0.55362E+00_JPRB, 0.55231E+00_JPRB ,&
&  0.55541E+00_JPRB, 0.54741E+00_JPRB, 0.53369E+00_JPRB, 0.52458E+00_JPRB, 0.52476E+00_JPRB, 0.52212E+00_JPRB, 0.51965E+00_JPRB ,&
&  0.54352E+00_JPRB, 0.49651E+00_JPRB, 0.54482E+00_JPRB, 0.55458E+00_JPRB, 0.56194E+00_JPRB, 0.56588E+00_JPRB, 0.56516E+00_JPRB ,&
&  0.56057E+00_JPRB, 0.55399E+00_JPRB, 0.54837E+00_JPRB, 0.54300E+00_JPRB, 0.53898E+00_JPRB, 0.53367E+00_JPRB, 0.52888E+00_JPRB ,&
&  0.52899E+00_JPRB, 0.52961E+00_JPRB, 0.53373E+00_JPRB, 0.53902E+00_JPRB, 0.54631E+00_JPRB, 0.54867E+00_JPRB, 0.55075E+00_JPRB ,&
&  0.55333E+00_JPRB, 0.55491E+00_JPRB, 0.55473E+00_JPRB, 0.55355E+00_JPRB, 0.55211E+00_JPRB, 0.55675E+00_JPRB, 0.56324E+00_JPRB ,&
&  0.56888E+00_JPRB, 0.57617E+00_JPRB, 0.58658E+00_JPRB, 0.59336E+00_JPRB, 0.59586E+00_JPRB, 0.59242E+00_JPRB, 0.58500E+00_JPRB ,&
&  0.57962E+00_JPRB, 0.57846E+00_JPRB, 0.57816E+00_JPRB, 0.57302E+00_JPRB, 0.56544E+00_JPRB, 0.55847E+00_JPRB, 0.54956E+00_JPRB ,&
&  0.53820E+00_JPRB, 0.52897E+00_JPRB, 0.52387E+00_JPRB, 0.52206E+00_JPRB, 0.52267E+00_JPRB, 0.52283E+00_JPRB, 0.52297E+00_JPRB ,&
&  0.52531E+00_JPRB, 0.52605E+00_JPRB, 0.52804E+00_JPRB, 0.53336E+00_JPRB, 0.53833E+00_JPRB, 0.54562E+00_JPRB, 0.55618E+00_JPRB ,&
&  0.57093E+00_JPRB, 0.58892E+00_JPRB, 0.60537E+00_JPRB, 0.63283E+00_JPRB, 0.66958E+00_JPRB, 0.69832E+00_JPRB, 0.71351E+00_JPRB ,&
&  0.72655E+00_JPRB, 0.73096E+00_JPRB, 0.72671E+00_JPRB, 0.72132E+00_JPRB, 0.70738E+00_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.75007E+00_JPRB, 0.72971E+00_JPRB, 0.70348E+00_JPRB, 0.68252E+00_JPRB, 0.66800E+00_JPRB, 0.65898E+00_JPRB, 0.65057E+00_JPRB ,&
&  0.64185E+00_JPRB, 0.63962E+00_JPRB, 0.63893E+00_JPRB, 0.63849E+00_JPRB, 0.63785E+00_JPRB, 0.63365E+00_JPRB, 0.62505E+00_JPRB ,&
&  0.61547E+00_JPRB, 0.60356E+00_JPRB, 0.60143E+00_JPRB, 0.60065E+00_JPRB, 0.59437E+00_JPRB, 0.59000E+00_JPRB, 0.58947E+00_JPRB ,&
&  0.58293E+00_JPRB, 0.57245E+00_JPRB, 0.55361E+00_JPRB, 0.54733E+00_JPRB, 0.54972E+00_JPRB, 0.54957E+00_JPRB, 0.55692E+00_JPRB ,&
&  0.54395E+00_JPRB, 0.52635E+00_JPRB, 0.58092E+00_JPRB, 0.62023E+00_JPRB, 0.64096E+00_JPRB, 0.65741E+00_JPRB, 0.67413E+00_JPRB ,&
&  0.66935E+00_JPRB, 0.67333E+00_JPRB, 0.65675E+00_JPRB, 0.62664E+00_JPRB, 0.60523E+00_JPRB, 0.59354E+00_JPRB, 0.58775E+00_JPRB ,&
&  0.58985E+00_JPRB, 0.59158E+00_JPRB, 0.59080E+00_JPRB, 0.58929E+00_JPRB, 0.58932E+00_JPRB, 0.58537E+00_JPRB, 0.58276E+00_JPRB ,&
&  0.58394E+00_JPRB, 0.58726E+00_JPRB, 0.58904E+00_JPRB, 0.59339E+00_JPRB, 0.60266E+00_JPRB, 0.61673E+00_JPRB, 0.63097E+00_JPRB ,&
&  0.64411E+00_JPRB, 0.65826E+00_JPRB, 0.67916E+00_JPRB, 0.69436E+00_JPRB, 0.69661E+00_JPRB, 0.68445E+00_JPRB, 0.66668E+00_JPRB ,&
&  0.64911E+00_JPRB, 0.63624E+00_JPRB, 0.62677E+00_JPRB, 0.61940E+00_JPRB, 0.60921E+00_JPRB, 0.59963E+00_JPRB, 0.58932E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.57283E+00_JPRB, 0.56742E+00_JPRB, 0.56360E+00_JPRB, 0.56292E+00_JPRB, 0.56448E+00_JPRB, 0.56761E+00_JPRB ,&
&  0.56962E+00_JPRB, 0.56713E+00_JPRB, 0.56657E+00_JPRB, 0.57165E+00_JPRB, 0.57798E+00_JPRB, 0.58917E+00_JPRB, 0.60510E+00_JPRB ,&
&  0.62467E+00_JPRB, 0.64457E+00_JPRB, 0.66379E+00_JPRB, 0.70686E+00_JPRB, 0.74976E+00_JPRB, 0.78199E+00_JPRB, 0.79735E+00_JPRB ,&
&  0.80506E+00_JPRB, 0.79613E+00_JPRB, 0.79441E+00_JPRB, 0.78181E+00_JPRB, 0.76480E+00_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.74846E+00_JPRB, 0.74217E+00_JPRB, 0.72608E+00_JPRB, 0.71520E+00_JPRB, 0.70636E+00_JPRB, 0.69606E+00_JPRB, 0.68195E+00_JPRB ,&
&  0.67125E+00_JPRB, 0.66601E+00_JPRB, 0.66217E+00_JPRB, 0.66098E+00_JPRB, 0.66049E+00_JPRB, 0.65444E+00_JPRB, 0.64533E+00_JPRB ,&
&  0.63600E+00_JPRB, 0.62753E+00_JPRB, 0.62837E+00_JPRB, 0.62743E+00_JPRB, 0.62221E+00_JPRB, 0.61778E+00_JPRB, 0.61193E+00_JPRB ,&
&  0.59703E+00_JPRB, 0.58146E+00_JPRB, 0.56925E+00_JPRB, 0.57174E+00_JPRB, 0.57811E+00_JPRB, 0.57323E+00_JPRB, 0.58998E+00_JPRB ,&
&  0.65387E+00_JPRB, 0.58667E+00_JPRB, 0.61598E+00_JPRB, 0.69568E+00_JPRB, 0.73563E+00_JPRB, 0.77581E+00_JPRB, 0.80769E+00_JPRB ,&
&  0.83097E+00_JPRB, 0.84822E+00_JPRB, 0.80996E+00_JPRB, 0.76108E+00_JPRB, 0.71688E+00_JPRB, 0.68134E+00_JPRB, 0.65580E+00_JPRB ,&
&  0.64183E+00_JPRB, 0.63905E+00_JPRB, 0.63770E+00_JPRB, 0.63174E+00_JPRB, 0.62687E+00_JPRB, 0.62070E+00_JPRB, 0.61697E+00_JPRB ,&
&  0.61528E+00_JPRB, 0.61732E+00_JPRB, 0.62237E+00_JPRB, 0.63429E+00_JPRB, 0.64660E+00_JPRB, 0.66053E+00_JPRB, 0.68069E+00_JPRB ,&
&  0.71080E+00_JPRB, 0.75210E+00_JPRB, 0.79275E+00_JPRB, 0.81096E+00_JPRB, 0.80617E+00_JPRB, 0.78797E+00_JPRB, 0.76066E+00_JPRB ,&
&  0.73074E+00_JPRB, 0.70433E+00_JPRB, 0.68731E+00_JPRB, 0.67687E+00_JPRB, 0.66624E+00_JPRB, 0.65670E+00_JPRB, 0.64896E+00_JPRB ,&
&  0.64254E+00_JPRB, 0.63516E+00_JPRB, 0.63373E+00_JPRB, 0.63183E+00_JPRB, 0.62794E+00_JPRB, 0.61875E+00_JPRB, 0.60948E+00_JPRB ,&
&  0.60132E+00_JPRB, 0.59551E+00_JPRB, 0.59540E+00_JPRB, 0.60175E+00_JPRB, 0.61157E+00_JPRB, 0.62979E+00_JPRB, 0.65337E+00_JPRB ,&
&  0.67267E+00_JPRB, 0.69338E+00_JPRB, 0.71074E+00_JPRB, 0.76778E+00_JPRB, 0.80351E+00_JPRB, 0.82225E+00_JPRB, 0.83100E+00_JPRB ,&
&  0.83191E+00_JPRB, 0.83364E+00_JPRB, 0.65162E+00_JPRB, 0.72304E+00_JPRB, 0.74053E+00_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.76254E+00_JPRB, 0.75550E+00_JPRB, 0.74528E+00_JPRB, 0.74085E+00_JPRB, 0.73880E+00_JPRB, 0.72369E+00_JPRB, 0.71019E+00_JPRB ,&
&  0.70064E+00_JPRB, 0.69800E+00_JPRB, 0.69875E+00_JPRB, 0.69773E+00_JPRB, 0.69094E+00_JPRB, 0.67930E+00_JPRB, 0.67245E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.65345E+00_JPRB, 0.65067E+00_JPRB, 0.64952E+00_JPRB, 0.64355E+00_JPRB, 0.63548E+00_JPRB, 0.62828E+00_JPRB ,&
&  0.61330E+00_JPRB, 0.60300E+00_JPRB, 0.60053E+00_JPRB, 0.60367E+00_JPRB, 0.60746E+00_JPRB, 0.60650E+00_JPRB, 0.62578E+00_JPRB ,&
&  0.68668E+00_JPRB, 0.64244E+00_JPRB, 0.65436E+00_JPRB, 0.79944E+00_JPRB, 0.89588E+00_JPRB, 0.91532E+00_JPRB, 0.93837E+00_JPRB ,&
&  0.96385E+00_JPRB, 0.95784E+00_JPRB, 0.93062E+00_JPRB, 0.87292E+00_JPRB, 0.82789E+00_JPRB, 0.79476E+00_JPRB, 0.76538E+00_JPRB ,&
&  0.74246E+00_JPRB, 0.72344E+00_JPRB, 0.71332E+00_JPRB, 0.70190E+00_JPRB, 0.69644E+00_JPRB, 0.68576E+00_JPRB, 0.68208E+00_JPRB ,&
&  0.68304E+00_JPRB, 0.68986E+00_JPRB, 0.69734E+00_JPRB, 0.70251E+00_JPRB, 0.71400E+00_JPRB, 0.73421E+00_JPRB, 0.77394E+00_JPRB ,&
&  0.83254E+00_JPRB, 0.90662E+00_JPRB, 0.95205E+00_JPRB, 0.96032E+00_JPRB, 0.94534E+00_JPRB, 0.91706E+00_JPRB, 0.87619E+00_JPRB ,&
&  0.82999E+00_JPRB, 0.78821E+00_JPRB, 0.75863E+00_JPRB, 0.74268E+00_JPRB, 0.73239E+00_JPRB, 0.72472E+00_JPRB, 0.71706E+00_JPRB ,&
&  0.71020E+00_JPRB, 0.70717E+00_JPRB, 0.70609E+00_JPRB, 0.70320E+00_JPRB, 0.69869E+00_JPRB, 0.68514E+00_JPRB, 0.66449E+00_JPRB ,&
&  0.64934E+00_JPRB, 0.64346E+00_JPRB, 0.64995E+00_JPRB, 0.65611E+00_JPRB, 0.66473E+00_JPRB, 0.68530E+00_JPRB, 0.70896E+00_JPRB ,&
&  0.72848E+00_JPRB, 0.75410E+00_JPRB, 0.77998E+00_JPRB, 0.82961E+00_JPRB, 0.87890E+00_JPRB, 0.87104E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.85951E+00_JPRB, 0.83447E+00_JPRB, 0.80984E+00_JPRB, 0.68495E+00_JPRB, 0.74798E+00_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.77993E+00_JPRB, 0.77215E+00_JPRB, 0.76322E+00_JPRB, 0.75870E+00_JPRB, 0.75366E+00_JPRB, 0.74093E+00_JPRB, 0.72974E+00_JPRB ,&
&  0.72504E+00_JPRB, 0.71854E+00_JPRB, 0.71180E+00_JPRB, 0.70265E+00_JPRB, 0.69086E+00_JPRB, 0.68233E+00_JPRB, 0.68080E+00_JPRB ,&
&  0.67655E+00_JPRB, 0.67404E+00_JPRB, 0.67236E+00_JPRB, 0.66673E+00_JPRB, 0.66000E+00_JPRB, 0.65029E+00_JPRB, 0.63460E+00_JPRB ,&
&  0.61495E+00_JPRB, 0.60833E+00_JPRB, 0.61683E+00_JPRB, 0.62447E+00_JPRB, 0.62983E+00_JPRB, 0.63255E+00_JPRB, 0.64518E+00_JPRB ,&
&  0.69447E+00_JPRB, 0.72737E+00_JPRB, 0.85870E+00_JPRB, 0.10454E+01_JPRB, 0.11454E+01_JPRB, 0.11223E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.10135E+01_JPRB, 0.96178E+00_JPRB, 0.90256E+00_JPRB, 0.85177E+00_JPRB, 0.81626E+00_JPRB, 0.79774E+00_JPRB ,&
&  0.78553E+00_JPRB, 0.77463E+00_JPRB, 0.76823E+00_JPRB, 0.75692E+00_JPRB, 0.74725E+00_JPRB, 0.73999E+00_JPRB, 0.74037E+00_JPRB ,&
&  0.74296E+00_JPRB, 0.74712E+00_JPRB, 0.75048E+00_JPRB, 0.75479E+00_JPRB, 0.76383E+00_JPRB, 0.79032E+00_JPRB, 0.86276E+00_JPRB ,&
&  0.95672E+00_JPRB, 0.10469E+01_JPRB, 0.10836E+01_JPRB, 0.10669E+01_JPRB, 0.10485E+01_JPRB, 0.10278E+01_JPRB, 0.97554E+00_JPRB ,&
&  0.90798E+00_JPRB, 0.86255E+00_JPRB, 0.82695E+00_JPRB, 0.79654E+00_JPRB, 0.77522E+00_JPRB, 0.76075E+00_JPRB, 0.75024E+00_JPRB ,&
&  0.74388E+00_JPRB, 0.74087E+00_JPRB, 0.73785E+00_JPRB, 0.73582E+00_JPRB, 0.73256E+00_JPRB, 0.72161E+00_JPRB, 0.70659E+00_JPRB ,&
&  0.69055E+00_JPRB, 0.68978E+00_JPRB, 0.70228E+00_JPRB, 0.70868E+00_JPRB, 0.71428E+00_JPRB, 0.72857E+00_JPRB, 0.75132E+00_JPRB ,&
&  0.76915E+00_JPRB, 0.79909E+00_JPRB, 0.81688E+00_JPRB, 0.90915E+00_JPRB, 0.92940E+00_JPRB, 0.90808E+00_JPRB, 0.89505E+00_JPRB ,&
&  0.86682E+00_JPRB, 0.82630E+00_JPRB, 0.79149E+00_JPRB, 0.79256E+00_JPRB, 0.74621E+00_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.87448E+00_JPRB, 0.83230E+00_JPRB, 0.79152E+00_JPRB, 0.76645E+00_JPRB, 0.75563E+00_JPRB, 0.75108E+00_JPRB, 0.74767E+00_JPRB ,&
&  0.74209E+00_JPRB, 0.73416E+00_JPRB, 0.71517E+00_JPRB, 0.70089E+00_JPRB, 0.69105E+00_JPRB, 0.68741E+00_JPRB, 0.68420E+00_JPRB ,&
&  0.68305E+00_JPRB, 0.68062E+00_JPRB, 0.67411E+00_JPRB, 0.67147E+00_JPRB, 0.67184E+00_JPRB, 0.66584E+00_JPRB, 0.64791E+00_JPRB ,&
&  0.62951E+00_JPRB, 0.63018E+00_JPRB, 0.63566E+00_JPRB, 0.63992E+00_JPRB, 0.64782E+00_JPRB, 0.65764E+00_JPRB, 0.67432E+00_JPRB ,&
&  0.71211E+00_JPRB, 0.79013E+00_JPRB, 0.11008E+01_JPRB, 0.12917E+01_JPRB, 0.13797E+01_JPRB, 0.13939E+01_JPRB, 0.13135E+01_JPRB ,&
&  0.12233E+01_JPRB, 0.11207E+01_JPRB, 0.10362E+01_JPRB, 0.94706E+00_JPRB, 0.88652E+00_JPRB, 0.84308E+00_JPRB, 0.81373E+00_JPRB ,&
&  0.79827E+00_JPRB, 0.79146E+00_JPRB, 0.78521E+00_JPRB, 0.77714E+00_JPRB, 0.77134E+00_JPRB, 0.76802E+00_JPRB, 0.76742E+00_JPRB ,&
&  0.76992E+00_JPRB, 0.77663E+00_JPRB, 0.78255E+00_JPRB, 0.78933E+00_JPRB, 0.80758E+00_JPRB, 0.88916E+00_JPRB, 0.10478E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.12706E+01_JPRB, 0.12514E+01_JPRB, 0.12210E+01_JPRB, 0.11936E+01_JPRB, 0.11325E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.10014E+01_JPRB, 0.93827E+00_JPRB, 0.87275E+00_JPRB, 0.82127E+00_JPRB, 0.78960E+00_JPRB, 0.77230E+00_JPRB, 0.76366E+00_JPRB ,&
&  0.76411E+00_JPRB, 0.77136E+00_JPRB, 0.77235E+00_JPRB, 0.77499E+00_JPRB, 0.77055E+00_JPRB, 0.76308E+00_JPRB, 0.75746E+00_JPRB ,&
&  0.75276E+00_JPRB, 0.74915E+00_JPRB, 0.76309E+00_JPRB, 0.77083E+00_JPRB, 0.77521E+00_JPRB, 0.78576E+00_JPRB, 0.80056E+00_JPRB ,&
&  0.81983E+00_JPRB, 0.86409E+00_JPRB, 0.85870E+00_JPRB, 0.95923E+00_JPRB, 0.10480E+01_JPRB, 0.98175E+00_JPRB, 0.91150E+00_JPRB ,&
&  0.85035E+00_JPRB, 0.80872E+00_JPRB, 0.79287E+00_JPRB, 0.81845E+00_JPRB, 0.87024E+00_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.84051E+00_JPRB, 0.78664E+00_JPRB, 0.75719E+00_JPRB, 0.75286E+00_JPRB, 0.75761E+00_JPRB, 0.75294E+00_JPRB, 0.75537E+00_JPRB ,&
&  0.74776E+00_JPRB, 0.73193E+00_JPRB, 0.71449E+00_JPRB, 0.69929E+00_JPRB, 0.69200E+00_JPRB, 0.68595E+00_JPRB, 0.68096E+00_JPRB ,&
&  0.67695E+00_JPRB, 0.67316E+00_JPRB, 0.66627E+00_JPRB, 0.66814E+00_JPRB, 0.66899E+00_JPRB, 0.66608E+00_JPRB, 0.64295E+00_JPRB ,&
&  0.63111E+00_JPRB, 0.63228E+00_JPRB, 0.63829E+00_JPRB, 0.64373E+00_JPRB, 0.65258E+00_JPRB, 0.66467E+00_JPRB, 0.67745E+00_JPRB ,&
&  0.71400E+00_JPRB, 0.90363E+00_JPRB, 0.12648E+01_JPRB, 0.14982E+01_JPRB, 0.16121E+01_JPRB, 0.15394E+01_JPRB, 0.14017E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.12433E+01_JPRB, 0.11929E+01_JPRB, 0.10686E+01_JPRB, 0.93992E+00_JPRB, 0.86981E+00_JPRB, 0.83393E+00_JPRB ,&
&  0.81735E+00_JPRB, 0.81107E+00_JPRB, 0.80322E+00_JPRB, 0.79660E+00_JPRB, 0.79014E+00_JPRB, 0.78866E+00_JPRB, 0.79238E+00_JPRB ,&
&  0.79807E+00_JPRB, 0.79885E+00_JPRB, 0.80738E+00_JPRB, 0.81674E+00_JPRB, 0.97483E+00_JPRB, 0.11607E+01_JPRB, 0.14418E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.15935E+01_JPRB, 0.15026E+01_JPRB, 0.13730E+01_JPRB, 0.12537E+01_JPRB, 0.11545E+01_JPRB, 0.10706E+01_JPRB ,&
&  0.99917E+00_JPRB, 0.93627E+00_JPRB, 0.88449E+00_JPRB, 0.85597E+00_JPRB, 0.84337E+00_JPRB, 0.82543E+00_JPRB, 0.81655E+00_JPRB ,&
&  0.81772E+00_JPRB, 0.82270E+00_JPRB, 0.82159E+00_JPRB, 0.81217E+00_JPRB, 0.79722E+00_JPRB, 0.78740E+00_JPRB, 0.78375E+00_JPRB ,&
&  0.79224E+00_JPRB, 0.79632E+00_JPRB, 0.82817E+00_JPRB, 0.82939E+00_JPRB, 0.82893E+00_JPRB, 0.83393E+00_JPRB, 0.85218E+00_JPRB ,&
&  0.88683E+00_JPRB, 0.93786E+00_JPRB, 0.93613E+00_JPRB, 0.97703E+00_JPRB, 0.10317E+01_JPRB, 0.10205E+01_JPRB, 0.90229E+00_JPRB ,&
&  0.81865E+00_JPRB, 0.78127E+00_JPRB, 0.77791E+00_JPRB, 0.80555E+00_JPRB, 0.85566E+00_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.80013E+00_JPRB, 0.76665E+00_JPRB, 0.75281E+00_JPRB, 0.75039E+00_JPRB, 0.74294E+00_JPRB, 0.74442E+00_JPRB, 0.74789E+00_JPRB ,&
&  0.74467E+00_JPRB, 0.72906E+00_JPRB, 0.71481E+00_JPRB, 0.69914E+00_JPRB, 0.69360E+00_JPRB, 0.68594E+00_JPRB, 0.68374E+00_JPRB ,&
&  0.67563E+00_JPRB, 0.67112E+00_JPRB, 0.66322E+00_JPRB, 0.66003E+00_JPRB, 0.65895E+00_JPRB, 0.65291E+00_JPRB, 0.63205E+00_JPRB ,&
&  0.62619E+00_JPRB, 0.62541E+00_JPRB, 0.63436E+00_JPRB, 0.64636E+00_JPRB, 0.66120E+00_JPRB, 0.66793E+00_JPRB, 0.67159E+00_JPRB ,&
&  0.74238E+00_JPRB, 0.93601E+00_JPRB, 0.15517E+01_JPRB, 0.16970E+01_JPRB, 0.17201E+01_JPRB, 0.15551E+01_JPRB, 0.14359E+01_JPRB ,&
&  0.13360E+01_JPRB, 0.12941E+01_JPRB, 0.12859E+01_JPRB, 0.11813E+01_JPRB, 0.10324E+01_JPRB, 0.91815E+00_JPRB, 0.87674E+00_JPRB ,&
&  0.85338E+00_JPRB, 0.83353E+00_JPRB, 0.82372E+00_JPRB, 0.82219E+00_JPRB, 0.81835E+00_JPRB, 0.81674E+00_JPRB, 0.82481E+00_JPRB ,&
&  0.83207E+00_JPRB, 0.84393E+00_JPRB, 0.86477E+00_JPRB, 0.94337E+00_JPRB, 0.11884E+01_JPRB, 0.12289E+01_JPRB, 0.16375E+01_JPRB ,&
&  0.22492E+01_JPRB, 0.20893E+01_JPRB, 0.17439E+01_JPRB, 0.14078E+01_JPRB, 0.12110E+01_JPRB, 0.11177E+01_JPRB, 0.10471E+01_JPRB ,&
&  0.99249E+00_JPRB, 0.94386E+00_JPRB, 0.90531E+00_JPRB, 0.89270E+00_JPRB, 0.87735E+00_JPRB, 0.85870E+00_JPRB, 0.85052E+00_JPRB ,&
&  0.85929E+00_JPRB, 0.86531E+00_JPRB, 0.85308E+00_JPRB, 0.82857E+00_JPRB, 0.81118E+00_JPRB, 0.80893E+00_JPRB, 0.80695E+00_JPRB ,&
&  0.81029E+00_JPRB, 0.82214E+00_JPRB, 0.86033E+00_JPRB, 0.82759E+00_JPRB, 0.84075E+00_JPRB, 0.87949E+00_JPRB, 0.90318E+00_JPRB ,&
&  0.91080E+00_JPRB, 0.94002E+00_JPRB, 0.97360E+00_JPRB, 0.97436E+00_JPRB, 0.10544E+01_JPRB, 0.10201E+01_JPRB, 0.86778E+00_JPRB ,&
&  0.78590E+00_JPRB, 0.76417E+00_JPRB, 0.78653E+00_JPRB, 0.81983E+00_JPRB, 0.85591E+00_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.77224E+00_JPRB, 0.74972E+00_JPRB, 0.74130E+00_JPRB, 0.73289E+00_JPRB, 0.71923E+00_JPRB, 0.70504E+00_JPRB, 0.70512E+00_JPRB ,&
&  0.70834E+00_JPRB, 0.71090E+00_JPRB, 0.70247E+00_JPRB, 0.68680E+00_JPRB, 0.68473E+00_JPRB, 0.68233E+00_JPRB, 0.67795E+00_JPRB ,&
&  0.67314E+00_JPRB, 0.67189E+00_JPRB, 0.66228E+00_JPRB, 0.65647E+00_JPRB, 0.65760E+00_JPRB, 0.65404E+00_JPRB, 0.64475E+00_JPRB ,&
&  0.63498E+00_JPRB, 0.63021E+00_JPRB, 0.63664E+00_JPRB, 0.64339E+00_JPRB, 0.65875E+00_JPRB, 0.66179E+00_JPRB, 0.66576E+00_JPRB ,&
&  0.74932E+00_JPRB, 0.70912E+00_JPRB, 0.15425E+01_JPRB, 0.20584E+01_JPRB, 0.19935E+01_JPRB, 0.17005E+01_JPRB, 0.15115E+01_JPRB ,&
&  0.14672E+01_JPRB, 0.13842E+01_JPRB, 0.13159E+01_JPRB, 0.12116E+01_JPRB, 0.11045E+01_JPRB, 0.99360E+00_JPRB, 0.93778E+00_JPRB ,&
&  0.90304E+00_JPRB, 0.87173E+00_JPRB, 0.86203E+00_JPRB, 0.86013E+00_JPRB, 0.86572E+00_JPRB, 0.87000E+00_JPRB, 0.88840E+00_JPRB ,&
&  0.90593E+00_JPRB, 0.93398E+00_JPRB, 0.10017E+01_JPRB, 0.11609E+01_JPRB, 0.13506E+01_JPRB, 0.14532E+01_JPRB, 0.17773E+01_JPRB ,&
&  0.21431E+01_JPRB, 0.21368E+01_JPRB, 0.16894E+01_JPRB, 0.13042E+01_JPRB, 0.11277E+01_JPRB, 0.10602E+01_JPRB, 0.10061E+01_JPRB ,&
&  0.96032E+00_JPRB, 0.91874E+00_JPRB, 0.90542E+00_JPRB, 0.88883E+00_JPRB, 0.86472E+00_JPRB, 0.84299E+00_JPRB, 0.85466E+00_JPRB ,&
&  0.87012E+00_JPRB, 0.86872E+00_JPRB, 0.85040E+00_JPRB, 0.83880E+00_JPRB, 0.82804E+00_JPRB, 0.81081E+00_JPRB, 0.80460E+00_JPRB ,&
&  0.80463E+00_JPRB, 0.82807E+00_JPRB, 0.84624E+00_JPRB, 0.84248E+00_JPRB, 0.85107E+00_JPRB, 0.87860E+00_JPRB, 0.91300E+00_JPRB ,&
&  0.96461E+00_JPRB, 0.98548E+00_JPRB, 0.96269E+00_JPRB, 0.97081E+00_JPRB, 0.93092E+00_JPRB, 0.93277E+00_JPRB, 0.80773E+00_JPRB ,&
&  0.76645E+00_JPRB, 0.76042E+00_JPRB, 0.77668E+00_JPRB, 0.79125E+00_JPRB, 0.80404E+00_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.74158E+00_JPRB, 0.72886E+00_JPRB, 0.72505E+00_JPRB, 0.71291E+00_JPRB, 0.69242E+00_JPRB, 0.67378E+00_JPRB, 0.66784E+00_JPRB ,&
&  0.66403E+00_JPRB, 0.66827E+00_JPRB, 0.67697E+00_JPRB, 0.66877E+00_JPRB, 0.66948E+00_JPRB, 0.67545E+00_JPRB, 0.67706E+00_JPRB ,&
&  0.67024E+00_JPRB, 0.66673E+00_JPRB, 0.66123E+00_JPRB, 0.65710E+00_JPRB, 0.65471E+00_JPRB, 0.64418E+00_JPRB, 0.63849E+00_JPRB ,&
&  0.63819E+00_JPRB, 0.64476E+00_JPRB, 0.65083E+00_JPRB, 0.65308E+00_JPRB, 0.66594E+00_JPRB, 0.67133E+00_JPRB, 0.67882E+00_JPRB ,&
&  0.75049E+00_JPRB, 0.15005E+01_JPRB, 0.12878E+01_JPRB, 0.23860E+01_JPRB, 0.20777E+01_JPRB, 0.17214E+01_JPRB, 0.14423E+01_JPRB ,&
&  0.14386E+01_JPRB, 0.14756E+01_JPRB, 0.13441E+01_JPRB, 0.12209E+01_JPRB, 0.11161E+01_JPRB, 0.10219E+01_JPRB, 0.95023E+00_JPRB ,&
&  0.89627E+00_JPRB, 0.87168E+00_JPRB, 0.87311E+00_JPRB, 0.88743E+00_JPRB, 0.91921E+00_JPRB, 0.95799E+00_JPRB, 0.99241E+00_JPRB ,&
&  0.10413E+01_JPRB, 0.11322E+01_JPRB, 0.12793E+01_JPRB, 0.14896E+01_JPRB, 0.14661E+01_JPRB, 0.15186E+01_JPRB, 0.19406E+01_JPRB ,&
&  0.20082E+01_JPRB, 0.19394E+01_JPRB, 0.15139E+01_JPRB, 0.11979E+01_JPRB, 0.10761E+01_JPRB, 0.10245E+01_JPRB, 0.97120E+00_JPRB ,&
&  0.92177E+00_JPRB, 0.89372E+00_JPRB, 0.89920E+00_JPRB, 0.87185E+00_JPRB, 0.84159E+00_JPRB, 0.83834E+00_JPRB, 0.84615E+00_JPRB ,&
&  0.85424E+00_JPRB, 0.84497E+00_JPRB, 0.86082E+00_JPRB, 0.86071E+00_JPRB, 0.85131E+00_JPRB, 0.83487E+00_JPRB, 0.82321E+00_JPRB ,&
&  0.82642E+00_JPRB, 0.85113E+00_JPRB, 0.85058E+00_JPRB, 0.84091E+00_JPRB, 0.84797E+00_JPRB, 0.86099E+00_JPRB, 0.87310E+00_JPRB ,&
&  0.95168E+00_JPRB, 0.97239E+00_JPRB, 0.94750E+00_JPRB, 0.93274E+00_JPRB, 0.91356E+00_JPRB, 0.93076E+00_JPRB, 0.77837E+00_JPRB ,&
&  0.75504E+00_JPRB, 0.74888E+00_JPRB, 0.75699E+00_JPRB, 0.76793E+00_JPRB, 0.76303E+00_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.72649E+00_JPRB, 0.71756E+00_JPRB, 0.71368E+00_JPRB, 0.69156E+00_JPRB, 0.66530E+00_JPRB, 0.64506E+00_JPRB, 0.63710E+00_JPRB ,&
&  0.63958E+00_JPRB, 0.65217E+00_JPRB, 0.66240E+00_JPRB, 0.65088E+00_JPRB, 0.64508E+00_JPRB, 0.65446E+00_JPRB, 0.65834E+00_JPRB ,&
&  0.65940E+00_JPRB, 0.65649E+00_JPRB, 0.65420E+00_JPRB, 0.65354E+00_JPRB, 0.64945E+00_JPRB, 0.63807E+00_JPRB, 0.63839E+00_JPRB ,&
&  0.65413E+00_JPRB, 0.66882E+00_JPRB, 0.67444E+00_JPRB, 0.67531E+00_JPRB, 0.69716E+00_JPRB, 0.70715E+00_JPRB, 0.72468E+00_JPRB ,&
&  0.80914E+00_JPRB, 0.14896E+01_JPRB, 0.14006E+01_JPRB, 0.25515E+01_JPRB, 0.21819E+01_JPRB, 0.18383E+01_JPRB, 0.15765E+01_JPRB ,&
&  0.15314E+01_JPRB, 0.16193E+01_JPRB, 0.13283E+01_JPRB, 0.11148E+01_JPRB, 0.10039E+01_JPRB, 0.97166E+00_JPRB, 0.93896E+00_JPRB ,&
&  0.90924E+00_JPRB, 0.92900E+00_JPRB, 0.96310E+00_JPRB, 0.10119E+01_JPRB, 0.10700E+01_JPRB, 0.11188E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.14435E+01_JPRB, 0.16556E+01_JPRB, 0.16364E+01_JPRB, 0.15551E+01_JPRB, 0.16539E+01_JPRB, 0.17783E+01_JPRB ,&
&  0.17740E+01_JPRB, 0.16517E+01_JPRB, 0.13174E+01_JPRB, 0.11131E+01_JPRB, 0.10055E+01_JPRB, 0.97709E+00_JPRB, 0.91858E+00_JPRB ,&
&  0.86648E+00_JPRB, 0.85120E+00_JPRB, 0.84577E+00_JPRB, 0.83319E+00_JPRB, 0.82523E+00_JPRB, 0.81856E+00_JPRB, 0.81192E+00_JPRB ,&
&  0.81824E+00_JPRB, 0.84720E+00_JPRB, 0.85564E+00_JPRB, 0.85832E+00_JPRB, 0.86320E+00_JPRB, 0.86502E+00_JPRB, 0.85502E+00_JPRB ,&
&  0.85990E+00_JPRB, 0.87713E+00_JPRB, 0.85300E+00_JPRB, 0.85061E+00_JPRB, 0.86482E+00_JPRB, 0.86576E+00_JPRB, 0.87294E+00_JPRB ,&
&  0.91725E+00_JPRB, 0.93383E+00_JPRB, 0.89774E+00_JPRB, 0.84629E+00_JPRB, 0.85760E+00_JPRB, 0.87130E+00_JPRB, 0.76674E+00_JPRB ,&
&  0.75437E+00_JPRB, 0.74756E+00_JPRB, 0.74919E+00_JPRB, 0.74639E+00_JPRB, 0.73684E+00_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.68829E+00_JPRB, 0.68053E+00_JPRB, 0.67048E+00_JPRB, 0.62623E+00_JPRB, 0.58648E+00_JPRB, 0.56931E+00_JPRB, 0.58447E+00_JPRB ,&
&  0.61908E+00_JPRB, 0.64314E+00_JPRB, 0.64179E+00_JPRB, 0.62972E+00_JPRB, 0.62539E+00_JPRB, 0.63082E+00_JPRB, 0.64088E+00_JPRB ,&
&  0.64297E+00_JPRB, 0.63410E+00_JPRB, 0.62560E+00_JPRB, 0.63083E+00_JPRB, 0.63284E+00_JPRB, 0.64302E+00_JPRB, 0.66813E+00_JPRB ,&
&  0.70351E+00_JPRB, 0.72159E+00_JPRB, 0.72375E+00_JPRB, 0.73493E+00_JPRB, 0.77928E+00_JPRB, 0.79209E+00_JPRB, 0.81808E+00_JPRB ,&
&  0.98887E+00_JPRB, 0.16891E+01_JPRB, 0.18263E+01_JPRB, 0.22208E+01_JPRB, 0.20179E+01_JPRB, 0.16688E+01_JPRB, 0.15821E+01_JPRB ,&
&  0.15669E+01_JPRB, 0.13390E+01_JPRB, 0.11780E+01_JPRB, 0.99424E+00_JPRB, 0.98019E+00_JPRB, 0.98833E+00_JPRB, 0.99345E+00_JPRB ,&
&  0.10320E+01_JPRB, 0.11221E+01_JPRB, 0.11802E+01_JPRB, 0.12329E+01_JPRB, 0.12850E+01_JPRB, 0.13590E+01_JPRB, 0.14746E+01_JPRB ,&
&  0.15993E+01_JPRB, 0.17497E+01_JPRB, 0.19023E+01_JPRB, 0.17203E+01_JPRB, 0.14689E+01_JPRB, 0.15754E+01_JPRB, 0.15964E+01_JPRB ,&
&  0.15600E+01_JPRB, 0.13744E+01_JPRB, 0.11707E+01_JPRB, 0.10214E+01_JPRB, 0.89805E+00_JPRB, 0.92242E+00_JPRB, 0.86225E+00_JPRB ,&
&  0.80671E+00_JPRB, 0.80201E+00_JPRB, 0.79338E+00_JPRB, 0.79887E+00_JPRB, 0.79911E+00_JPRB, 0.79055E+00_JPRB, 0.79481E+00_JPRB ,&
&  0.81377E+00_JPRB, 0.82951E+00_JPRB, 0.84217E+00_JPRB, 0.85115E+00_JPRB, 0.86544E+00_JPRB, 0.87638E+00_JPRB, 0.87695E+00_JPRB ,&
&  0.87727E+00_JPRB, 0.89591E+00_JPRB, 0.87896E+00_JPRB, 0.89835E+00_JPRB, 0.90347E+00_JPRB, 0.89658E+00_JPRB, 0.86360E+00_JPRB ,&
&  0.86608E+00_JPRB, 0.88189E+00_JPRB, 0.84104E+00_JPRB, 0.81886E+00_JPRB, 0.82383E+00_JPRB, 0.80666E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.75654E+00_JPRB, 0.74502E+00_JPRB, 0.73552E+00_JPRB, 0.71496E+00_JPRB, 0.69967E+00_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.66683E+00_JPRB, 0.65385E+00_JPRB, 0.62157E+00_JPRB, 0.55610E+00_JPRB, 0.53609E+00_JPRB, 0.55155E+00_JPRB, 0.61294E+00_JPRB ,&
&  0.63939E+00_JPRB, 0.64634E+00_JPRB, 0.63217E+00_JPRB, 0.61627E+00_JPRB, 0.60730E+00_JPRB, 0.60815E+00_JPRB, 0.62459E+00_JPRB ,&
&  0.64160E+00_JPRB, 0.63594E+00_JPRB, 0.63433E+00_JPRB, 0.64207E+00_JPRB, 0.66200E+00_JPRB, 0.69500E+00_JPRB, 0.74125E+00_JPRB ,&
&  0.78045E+00_JPRB, 0.79974E+00_JPRB, 0.81578E+00_JPRB, 0.86717E+00_JPRB, 0.93383E+00_JPRB, 0.96692E+00_JPRB, 0.10651E+01_JPRB ,&
&  0.15048E+01_JPRB, 0.20627E+01_JPRB, 0.94204E+00_JPRB, 0.14441E+01_JPRB, 0.16625E+01_JPRB, 0.14823E+01_JPRB, 0.13552E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.11172E+01_JPRB, 0.98445E+00_JPRB, 0.10127E+01_JPRB, 0.10628E+01_JPRB, 0.11305E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.13078E+01_JPRB, 0.13656E+01_JPRB, 0.14176E+01_JPRB, 0.14620E+01_JPRB, 0.15263E+01_JPRB, 0.16210E+01_JPRB, 0.17066E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.18332E+01_JPRB, 0.18416E+01_JPRB, 0.14726E+01_JPRB, 0.13567E+01_JPRB, 0.13937E+01_JPRB, 0.13274E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.11544E+01_JPRB, 0.10507E+01_JPRB, 0.92584E+00_JPRB, 0.82534E+00_JPRB, 0.78677E+00_JPRB, 0.79702E+00_JPRB ,&
&  0.76994E+00_JPRB, 0.74723E+00_JPRB, 0.73930E+00_JPRB, 0.74109E+00_JPRB, 0.74196E+00_JPRB, 0.74628E+00_JPRB, 0.75862E+00_JPRB ,&
&  0.78348E+00_JPRB, 0.80546E+00_JPRB, 0.82136E+00_JPRB, 0.83202E+00_JPRB, 0.84813E+00_JPRB, 0.86952E+00_JPRB, 0.87868E+00_JPRB ,&
&  0.90058E+00_JPRB, 0.91843E+00_JPRB, 0.93760E+00_JPRB, 0.93861E+00_JPRB, 0.91727E+00_JPRB, 0.90097E+00_JPRB, 0.89628E+00_JPRB ,&
&  0.89233E+00_JPRB, 0.87803E+00_JPRB, 0.79629E+00_JPRB, 0.78124E+00_JPRB, 0.81258E+00_JPRB, 0.78973E+00_JPRB, 0.82729E+00_JPRB ,&
&  0.89116E+00_JPRB, 0.93931E+00_JPRB, 0.89218E+00_JPRB, 0.69789E+00_JPRB, 0.67460E+00_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.64908E+00_JPRB, 0.60156E+00_JPRB, 0.54324E+00_JPRB, 0.50548E+00_JPRB, 0.52742E+00_JPRB, 0.59264E+00_JPRB, 0.64436E+00_JPRB ,&
&  0.66355E+00_JPRB, 0.66217E+00_JPRB, 0.65413E+00_JPRB, 0.65262E+00_JPRB, 0.66454E+00_JPRB, 0.65946E+00_JPRB, 0.67042E+00_JPRB ,&
&  0.68944E+00_JPRB, 0.70089E+00_JPRB, 0.71529E+00_JPRB, 0.73738E+00_JPRB, 0.76336E+00_JPRB, 0.80879E+00_JPRB, 0.85414E+00_JPRB ,&
&  0.89013E+00_JPRB, 0.92574E+00_JPRB, 0.98378E+00_JPRB, 0.10675E+01_JPRB, 0.11868E+01_JPRB, 0.13955E+01_JPRB, 0.17577E+01_JPRB ,&
&  0.24367E+01_JPRB, 0.16904E+01_JPRB, 0.83815E+00_JPRB, 0.13998E+01_JPRB, 0.14799E+01_JPRB, 0.13940E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12040E+01_JPRB, 0.11162E+01_JPRB, 0.10687E+01_JPRB, 0.11815E+01_JPRB, 0.13159E+01_JPRB, 0.15002E+01_JPRB, 0.16102E+01_JPRB ,&
&  0.16326E+01_JPRB, 0.16478E+01_JPRB, 0.16668E+01_JPRB, 0.17004E+01_JPRB, 0.17548E+01_JPRB, 0.18137E+01_JPRB, 0.18287E+01_JPRB ,&
&  0.18385E+01_JPRB, 0.18552E+01_JPRB, 0.17916E+01_JPRB, 0.13727E+01_JPRB, 0.12534E+01_JPRB, 0.11902E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.99377E+00_JPRB, 0.94438E+00_JPRB, 0.85645E+00_JPRB, 0.78092E+00_JPRB, 0.73359E+00_JPRB, 0.75161E+00_JPRB ,&
&  0.74428E+00_JPRB, 0.70347E+00_JPRB, 0.67511E+00_JPRB, 0.65658E+00_JPRB, 0.67871E+00_JPRB, 0.70438E+00_JPRB, 0.70699E+00_JPRB ,&
&  0.72394E+00_JPRB, 0.74963E+00_JPRB, 0.77012E+00_JPRB, 0.79036E+00_JPRB, 0.81996E+00_JPRB, 0.85214E+00_JPRB, 0.87356E+00_JPRB ,&
&  0.89591E+00_JPRB, 0.90436E+00_JPRB, 0.91171E+00_JPRB, 0.90619E+00_JPRB, 0.86739E+00_JPRB, 0.82675E+00_JPRB, 0.83143E+00_JPRB ,&
&  0.81940E+00_JPRB, 0.81289E+00_JPRB, 0.78810E+00_JPRB, 0.79924E+00_JPRB, 0.79908E+00_JPRB, 0.81842E+00_JPRB, 0.84680E+00_JPRB ,&
&  0.87307E+00_JPRB, 0.85669E+00_JPRB, 0.77550E+00_JPRB, 0.68405E+00_JPRB, 0.66426E+00_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.59610E+00_JPRB, 0.52454E+00_JPRB, 0.47062E+00_JPRB, 0.48577E+00_JPRB, 0.54894E+00_JPRB, 0.60251E+00_JPRB, 0.63055E+00_JPRB ,&
&  0.66327E+00_JPRB, 0.67250E+00_JPRB, 0.68010E+00_JPRB, 0.69067E+00_JPRB, 0.71316E+00_JPRB, 0.72016E+00_JPRB, 0.74313E+00_JPRB ,&
&  0.75685E+00_JPRB, 0.76639E+00_JPRB, 0.78246E+00_JPRB, 0.82320E+00_JPRB, 0.87360E+00_JPRB, 0.92476E+00_JPRB, 0.97476E+00_JPRB ,&
&  0.10328E+01_JPRB, 0.11004E+01_JPRB, 0.11965E+01_JPRB, 0.13111E+01_JPRB, 0.14655E+01_JPRB, 0.16347E+01_JPRB, 0.17604E+01_JPRB ,&
&  0.12716E+01_JPRB, 0.66741E+00_JPRB, 0.12372E+01_JPRB, 0.14735E+01_JPRB, 0.13827E+01_JPRB, 0.13258E+01_JPRB, 0.12576E+01_JPRB ,&
&  0.11952E+01_JPRB, 0.11543E+01_JPRB, 0.11630E+01_JPRB, 0.14397E+01_JPRB, 0.16870E+01_JPRB, 0.17866E+01_JPRB, 0.18253E+01_JPRB ,&
&  0.18601E+01_JPRB, 0.18787E+01_JPRB, 0.19095E+01_JPRB, 0.19574E+01_JPRB, 0.20132E+01_JPRB, 0.20524E+01_JPRB, 0.20704E+01_JPRB ,&
&  0.20862E+01_JPRB, 0.20668E+01_JPRB, 0.19911E+01_JPRB, 0.14918E+01_JPRB, 0.13510E+01_JPRB, 0.12013E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.10155E+01_JPRB, 0.86075E+00_JPRB, 0.81971E+00_JPRB, 0.82688E+00_JPRB, 0.77425E+00_JPRB, 0.68809E+00_JPRB, 0.67321E+00_JPRB ,&
&  0.67961E+00_JPRB, 0.66383E+00_JPRB, 0.63693E+00_JPRB, 0.62065E+00_JPRB, 0.65081E+00_JPRB, 0.65402E+00_JPRB, 0.66017E+00_JPRB ,&
&  0.65913E+00_JPRB, 0.69246E+00_JPRB, 0.72655E+00_JPRB, 0.75641E+00_JPRB, 0.78761E+00_JPRB, 0.82381E+00_JPRB, 0.83107E+00_JPRB ,&
&  0.84454E+00_JPRB, 0.84271E+00_JPRB, 0.84636E+00_JPRB, 0.82962E+00_JPRB, 0.81196E+00_JPRB, 0.77751E+00_JPRB, 0.76754E+00_JPRB ,&
&  0.76571E+00_JPRB, 0.76760E+00_JPRB, 0.75948E+00_JPRB, 0.74937E+00_JPRB, 0.77000E+00_JPRB, 0.76497E+00_JPRB, 0.77319E+00_JPRB ,&
&  0.77029E+00_JPRB, 0.75043E+00_JPRB, 0.69512E+00_JPRB, 0.65246E+00_JPRB, 0.63516E+00_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.49375E+00_JPRB, 0.43530E+00_JPRB, 0.43006E+00_JPRB, 0.47150E+00_JPRB, 0.51397E+00_JPRB, 0.55795E+00_JPRB, 0.59812E+00_JPRB ,&
&  0.63963E+00_JPRB, 0.66351E+00_JPRB, 0.71027E+00_JPRB, 0.73359E+00_JPRB, 0.75109E+00_JPRB, 0.75669E+00_JPRB, 0.76513E+00_JPRB ,&
&  0.79523E+00_JPRB, 0.82901E+00_JPRB, 0.86071E+00_JPRB, 0.90909E+00_JPRB, 0.96071E+00_JPRB, 0.10116E+01_JPRB, 0.10719E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.12001E+01_JPRB, 0.12701E+01_JPRB, 0.13527E+01_JPRB, 0.14177E+01_JPRB, 0.14477E+01_JPRB, 0.14140E+01_JPRB ,&
&  0.76754E+00_JPRB, 0.11745E+01_JPRB, 0.14629E+01_JPRB, 0.14345E+01_JPRB, 0.13873E+01_JPRB, 0.13331E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.12898E+01_JPRB, 0.12087E+01_JPRB, 0.12804E+01_JPRB, 0.16613E+01_JPRB, 0.17795E+01_JPRB, 0.18639E+01_JPRB, 0.19297E+01_JPRB ,&
&  0.20075E+01_JPRB, 0.20664E+01_JPRB, 0.21352E+01_JPRB, 0.22090E+01_JPRB, 0.22977E+01_JPRB, 0.23806E+01_JPRB, 0.24562E+01_JPRB ,&
&  0.25131E+01_JPRB, 0.25409E+01_JPRB, 0.25335E+01_JPRB, 0.20239E+01_JPRB, 0.19715E+01_JPRB, 0.16916E+01_JPRB, 0.14715E+01_JPRB ,&
&  0.10105E+01_JPRB, 0.80658E+00_JPRB, 0.80735E+00_JPRB, 0.84289E+00_JPRB, 0.76938E+00_JPRB, 0.72192E+00_JPRB, 0.65358E+00_JPRB ,&
&  0.65523E+00_JPRB, 0.65523E+00_JPRB, 0.65649E+00_JPRB, 0.64414E+00_JPRB, 0.64548E+00_JPRB, 0.61252E+00_JPRB, 0.64384E+00_JPRB ,&
&  0.66431E+00_JPRB, 0.65977E+00_JPRB, 0.67036E+00_JPRB, 0.70186E+00_JPRB, 0.74092E+00_JPRB, 0.77781E+00_JPRB, 0.78811E+00_JPRB ,&
&  0.78218E+00_JPRB, 0.76559E+00_JPRB, 0.76268E+00_JPRB, 0.75890E+00_JPRB, 0.75172E+00_JPRB, 0.74490E+00_JPRB, 0.72830E+00_JPRB ,&
&  0.71104E+00_JPRB, 0.72319E+00_JPRB, 0.75855E+00_JPRB, 0.70599E+00_JPRB, 0.74856E+00_JPRB, 0.68075E+00_JPRB, 0.66536E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.63187E+00_JPRB, 0.58707E+00_JPRB, 0.56472E+00_JPRB, 0.53343E+00_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.39886E+00_JPRB, 0.38076E+00_JPRB, 0.39068E+00_JPRB, 0.41466E+00_JPRB, 0.44331E+00_JPRB, 0.48699E+00_JPRB, 0.53984E+00_JPRB ,&
&  0.58610E+00_JPRB, 0.63173E+00_JPRB, 0.71055E+00_JPRB, 0.73028E+00_JPRB, 0.74786E+00_JPRB, 0.79107E+00_JPRB, 0.82100E+00_JPRB ,&
&  0.85604E+00_JPRB, 0.88424E+00_JPRB, 0.91638E+00_JPRB, 0.94846E+00_JPRB, 0.99668E+00_JPRB, 0.10494E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.12171E+01_JPRB, 0.13106E+01_JPRB, 0.14088E+01_JPRB, 0.15032E+01_JPRB, 0.15278E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.14133E+01_JPRB, 0.14347E+01_JPRB, 0.14206E+01_JPRB, 0.13726E+01_JPRB, 0.13320E+01_JPRB, 0.13497E+01_JPRB ,&
&  0.13804E+01_JPRB, 0.13380E+01_JPRB, 0.13876E+01_JPRB, 0.14627E+01_JPRB, 0.17455E+01_JPRB, 0.18128E+01_JPRB, 0.19202E+01_JPRB ,&
&  0.20339E+01_JPRB, 0.21362E+01_JPRB, 0.22477E+01_JPRB, 0.23851E+01_JPRB, 0.25259E+01_JPRB, 0.26587E+01_JPRB, 0.27959E+01_JPRB ,&
&  0.29105E+01_JPRB, 0.30179E+01_JPRB, 0.31535E+01_JPRB, 0.26431E+01_JPRB, 0.22152E+01_JPRB, 0.18065E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.91018E+00_JPRB, 0.77110E+00_JPRB, 0.80330E+00_JPRB, 0.84312E+00_JPRB, 0.76978E+00_JPRB, 0.72309E+00_JPRB, 0.66222E+00_JPRB ,&
&  0.65585E+00_JPRB, 0.66506E+00_JPRB, 0.67173E+00_JPRB, 0.66611E+00_JPRB, 0.67123E+00_JPRB, 0.64176E+00_JPRB, 0.63442E+00_JPRB ,&
&  0.65373E+00_JPRB, 0.64867E+00_JPRB, 0.65273E+00_JPRB, 0.67454E+00_JPRB, 0.71698E+00_JPRB, 0.75595E+00_JPRB, 0.75513E+00_JPRB ,&
&  0.75781E+00_JPRB, 0.68406E+00_JPRB, 0.65266E+00_JPRB, 0.66601E+00_JPRB, 0.66148E+00_JPRB, 0.65708E+00_JPRB, 0.67763E+00_JPRB ,&
&  0.76147E+00_JPRB, 0.86741E+00_JPRB, 0.95812E+00_JPRB, 0.94683E+00_JPRB, 0.99464E+00_JPRB, 0.78467E+00_JPRB, 0.58744E+00_JPRB ,&
&  0.55116E+00_JPRB, 0.54013E+00_JPRB, 0.49586E+00_JPRB, 0.46489E+00_JPRB, 0.42971E+00_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.39405E+00_JPRB, 0.38790E+00_JPRB, 0.39801E+00_JPRB, 0.41518E+00_JPRB, 0.42817E+00_JPRB, 0.45768E+00_JPRB, 0.49325E+00_JPRB ,&
&  0.53215E+00_JPRB, 0.58020E+00_JPRB, 0.62066E+00_JPRB, 0.64922E+00_JPRB, 0.69853E+00_JPRB, 0.76852E+00_JPRB, 0.80014E+00_JPRB ,&
&  0.86078E+00_JPRB, 0.90038E+00_JPRB, 0.92060E+00_JPRB, 0.93334E+00_JPRB, 0.96349E+00_JPRB, 0.10121E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.11216E+01_JPRB, 0.11887E+01_JPRB, 0.12845E+01_JPRB, 0.13483E+01_JPRB, 0.13813E+01_JPRB, 0.13935E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.12992E+01_JPRB, 0.13662E+01_JPRB, 0.13839E+01_JPRB, 0.13489E+01_JPRB, 0.13091E+01_JPRB, 0.12815E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13231E+01_JPRB, 0.12936E+01_JPRB, 0.12978E+01_JPRB, 0.15600E+01_JPRB, 0.15979E+01_JPRB, 0.17462E+01_JPRB ,&
&  0.18630E+01_JPRB, 0.19591E+01_JPRB, 0.20919E+01_JPRB, 0.22338E+01_JPRB, 0.23782E+01_JPRB, 0.25029E+01_JPRB, 0.26100E+01_JPRB ,&
&  0.27074E+01_JPRB, 0.28188E+01_JPRB, 0.29208E+01_JPRB, 0.25160E+01_JPRB, 0.20484E+01_JPRB, 0.17206E+01_JPRB, 0.11901E+01_JPRB ,&
&  0.85319E+00_JPRB, 0.77084E+00_JPRB, 0.80100E+00_JPRB, 0.82045E+00_JPRB, 0.75707E+00_JPRB, 0.71776E+00_JPRB, 0.68448E+00_JPRB ,&
&  0.68875E+00_JPRB, 0.71191E+00_JPRB, 0.73130E+00_JPRB, 0.73320E+00_JPRB, 0.70406E+00_JPRB, 0.66570E+00_JPRB, 0.63567E+00_JPRB ,&
&  0.65492E+00_JPRB, 0.65553E+00_JPRB, 0.65468E+00_JPRB, 0.69414E+00_JPRB, 0.73537E+00_JPRB, 0.86047E+00_JPRB, 0.10221E+01_JPRB ,&
&  0.98342E+00_JPRB, 0.60978E+00_JPRB, 0.59474E+00_JPRB, 0.60151E+00_JPRB, 0.62667E+00_JPRB, 0.69892E+00_JPRB, 0.85374E+00_JPRB ,&
&  0.12053E+01_JPRB, 0.19758E+01_JPRB, 0.22427E+01_JPRB, 0.14846E+01_JPRB, 0.20084E+01_JPRB, 0.13097E+01_JPRB, 0.53356E+00_JPRB ,&
&  0.50620E+00_JPRB, 0.49016E+00_JPRB, 0.46189E+00_JPRB, 0.43131E+00_JPRB, 0.40980E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.47767E+00_JPRB, 0.48032E+00_JPRB, 0.48915E+00_JPRB, 0.49531E+00_JPRB, 0.50152E+00_JPRB, 0.50608E+00_JPRB, 0.51055E+00_JPRB ,&
&  0.52474E+00_JPRB, 0.53315E+00_JPRB, 0.55452E+00_JPRB, 0.57121E+00_JPRB, 0.58693E+00_JPRB, 0.62007E+00_JPRB, 0.64793E+00_JPRB ,&
&  0.69260E+00_JPRB, 0.75056E+00_JPRB, 0.77506E+00_JPRB, 0.80891E+00_JPRB, 0.82899E+00_JPRB, 0.85077E+00_JPRB, 0.86710E+00_JPRB ,&
&  0.94030E+00_JPRB, 0.10810E+01_JPRB, 0.11778E+01_JPRB, 0.12816E+01_JPRB, 0.12902E+01_JPRB, 0.13735E+01_JPRB, 0.10522E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.13009E+01_JPRB, 0.12236E+01_JPRB, 0.11812E+01_JPRB, 0.11541E+01_JPRB, 0.11324E+01_JPRB, 0.11172E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.12045E+01_JPRB, 0.11813E+01_JPRB, 0.12293E+01_JPRB, 0.12845E+01_JPRB, 0.12585E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.14764E+01_JPRB, 0.15472E+01_JPRB, 0.16366E+01_JPRB, 0.17564E+01_JPRB, 0.18399E+01_JPRB, 0.18909E+01_JPRB, 0.18984E+01_JPRB ,&
&  0.18907E+01_JPRB, 0.19236E+01_JPRB, 0.20165E+01_JPRB, 0.19946E+01_JPRB, 0.16613E+01_JPRB, 0.14123E+01_JPRB, 0.11513E+01_JPRB ,&
&  0.86455E+00_JPRB, 0.75153E+00_JPRB, 0.75791E+00_JPRB, 0.82183E+00_JPRB, 0.79280E+00_JPRB, 0.77097E+00_JPRB, 0.74905E+00_JPRB ,&
&  0.73382E+00_JPRB, 0.74519E+00_JPRB, 0.75103E+00_JPRB, 0.74667E+00_JPRB, 0.72308E+00_JPRB, 0.67463E+00_JPRB, 0.62713E+00_JPRB ,&
&  0.62043E+00_JPRB, 0.61429E+00_JPRB, 0.61523E+00_JPRB, 0.63173E+00_JPRB, 0.66240E+00_JPRB, 0.73811E+00_JPRB, 0.70731E+00_JPRB ,&
&  0.75329E+00_JPRB, 0.57226E+00_JPRB, 0.57152E+00_JPRB, 0.60650E+00_JPRB, 0.61710E+00_JPRB, 0.85263E+00_JPRB, 0.12526E+01_JPRB ,&
&  0.18514E+01_JPRB, 0.21132E+01_JPRB, 0.27951E+01_JPRB, 0.40977E+01_JPRB, 0.25856E+01_JPRB, 0.16561E+01_JPRB, 0.55043E+00_JPRB ,&
&  0.53967E+00_JPRB, 0.51445E+00_JPRB, 0.50641E+00_JPRB, 0.49058E+00_JPRB, 0.48448E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.54933E+00_JPRB, 0.55886E+00_JPRB, 0.57157E+00_JPRB, 0.58305E+00_JPRB, 0.59077E+00_JPRB, 0.60358E+00_JPRB, 0.61293E+00_JPRB ,&
&  0.62350E+00_JPRB, 0.62800E+00_JPRB, 0.63620E+00_JPRB, 0.64364E+00_JPRB, 0.65397E+00_JPRB, 0.66903E+00_JPRB, 0.68412E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.71497E+00_JPRB, 0.73702E+00_JPRB, 0.75235E+00_JPRB, 0.78825E+00_JPRB, 0.85555E+00_JPRB, 0.93208E+00_JPRB ,&
&  0.99388E+00_JPRB, 0.10747E+01_JPRB, 0.11508E+01_JPRB, 0.11863E+01_JPRB, 0.11899E+01_JPRB, 0.12435E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.13557E+01_JPRB, 0.13211E+01_JPRB, 0.11955E+01_JPRB, 0.11191E+01_JPRB, 0.10495E+01_JPRB, 0.98267E+00_JPRB, 0.94007E+00_JPRB ,&
&  0.94239E+00_JPRB, 0.10818E+01_JPRB, 0.10651E+01_JPRB, 0.10911E+01_JPRB, 0.97828E+00_JPRB, 0.97056E+00_JPRB, 0.10548E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.11870E+01_JPRB, 0.11660E+01_JPRB, 0.12120E+01_JPRB, 0.12661E+01_JPRB, 0.12784E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.11556E+01_JPRB, 0.10502E+01_JPRB, 0.11878E+01_JPRB, 0.11042E+01_JPRB, 0.10086E+01_JPRB, 0.93741E+00_JPRB ,&
&  0.75904E+00_JPRB, 0.74594E+00_JPRB, 0.79965E+00_JPRB, 0.89236E+00_JPRB, 0.86685E+00_JPRB, 0.85470E+00_JPRB, 0.82048E+00_JPRB ,&
&  0.78234E+00_JPRB, 0.76110E+00_JPRB, 0.72511E+00_JPRB, 0.71220E+00_JPRB, 0.71197E+00_JPRB, 0.68712E+00_JPRB, 0.63803E+00_JPRB ,&
&  0.60533E+00_JPRB, 0.60574E+00_JPRB, 0.60030E+00_JPRB, 0.60247E+00_JPRB, 0.63551E+00_JPRB, 0.61821E+00_JPRB, 0.84554E+00_JPRB ,&
&  0.66473E+00_JPRB, 0.56900E+00_JPRB, 0.71892E+00_JPRB, 0.43530E+00_JPRB, 0.53791E+00_JPRB, 0.65470E+00_JPRB, 0.84617E+00_JPRB ,&
&  0.89645E+00_JPRB, 0.11531E+01_JPRB, 0.15002E+01_JPRB, 0.11918E+01_JPRB, 0.90919E+00_JPRB, 0.61730E+00_JPRB, 0.58293E+00_JPRB ,&
&  0.57245E+00_JPRB, 0.56179E+00_JPRB, 0.56330E+00_JPRB, 0.55488E+00_JPRB, 0.55167E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.62505E+00_JPRB, 0.63866E+00_JPRB, 0.65640E+00_JPRB, 0.67925E+00_JPRB, 0.69532E+00_JPRB, 0.71692E+00_JPRB, 0.73437E+00_JPRB ,&
&  0.75707E+00_JPRB, 0.77505E+00_JPRB, 0.79034E+00_JPRB, 0.80689E+00_JPRB, 0.82827E+00_JPRB, 0.85424E+00_JPRB, 0.88729E+00_JPRB ,&
&  0.92294E+00_JPRB, 0.95843E+00_JPRB, 0.99785E+00_JPRB, 0.10427E+01_JPRB, 0.10950E+01_JPRB, 0.11586E+01_JPRB, 0.12187E+01_JPRB ,&
&  0.12750E+01_JPRB, 0.13327E+01_JPRB, 0.13915E+01_JPRB, 0.14380E+01_JPRB, 0.14549E+01_JPRB, 0.14941E+01_JPRB, 0.13395E+01_JPRB ,&
&  0.15427E+01_JPRB, 0.14801E+01_JPRB, 0.13763E+01_JPRB, 0.12533E+01_JPRB, 0.11472E+01_JPRB, 0.10502E+01_JPRB, 0.96281E+00_JPRB ,&
&  0.96088E+00_JPRB, 0.90535E+00_JPRB, 0.88164E+00_JPRB, 0.86898E+00_JPRB, 0.80862E+00_JPRB, 0.81228E+00_JPRB, 0.83898E+00_JPRB ,&
&  0.88985E+00_JPRB, 0.98611E+00_JPRB, 0.10452E+01_JPRB, 0.11127E+01_JPRB, 0.11997E+01_JPRB, 0.11500E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.95041E+00_JPRB, 0.76479E+00_JPRB, 0.82709E+00_JPRB, 0.87375E+00_JPRB, 0.91198E+00_JPRB, 0.95092E+00_JPRB ,&
&  0.87266E+00_JPRB, 0.76355E+00_JPRB, 0.80821E+00_JPRB, 0.10190E+01_JPRB, 0.99689E+00_JPRB, 0.93357E+00_JPRB, 0.87529E+00_JPRB ,&
&  0.80289E+00_JPRB, 0.74770E+00_JPRB, 0.69559E+00_JPRB, 0.67618E+00_JPRB, 0.71093E+00_JPRB, 0.71035E+00_JPRB, 0.67240E+00_JPRB ,&
&  0.62040E+00_JPRB, 0.60135E+00_JPRB, 0.60594E+00_JPRB, 0.62243E+00_JPRB, 0.70973E+00_JPRB, 0.81605E+00_JPRB, 0.13078E+01_JPRB ,&
&  0.75009E+00_JPRB, 0.47147E+00_JPRB, 0.54531E+00_JPRB, 0.65419E+00_JPRB, 0.58182E+00_JPRB, 0.61114E+00_JPRB, 0.64393E+00_JPRB ,&
&  0.67630E+00_JPRB, 0.68763E+00_JPRB, 0.64475E+00_JPRB, 0.64280E+00_JPRB, 0.63543E+00_JPRB, 0.62851E+00_JPRB, 0.62190E+00_JPRB ,&
&  0.61328E+00_JPRB, 0.61310E+00_JPRB, 0.62360E+00_JPRB, 0.62046E+00_JPRB, 0.61935E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.63294E+00_JPRB, 0.64334E+00_JPRB, 0.64442E+00_JPRB, 0.65206E+00_JPRB, 0.66823E+00_JPRB, 0.68057E+00_JPRB, 0.69752E+00_JPRB ,&
&  0.71521E+00_JPRB, 0.72354E+00_JPRB, 0.73177E+00_JPRB, 0.74304E+00_JPRB, 0.75144E+00_JPRB, 0.77426E+00_JPRB, 0.79659E+00_JPRB ,&
&  0.82504E+00_JPRB, 0.84701E+00_JPRB, 0.87109E+00_JPRB, 0.90825E+00_JPRB, 0.95535E+00_JPRB, 0.10261E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.13127E+01_JPRB, 0.14490E+01_JPRB, 0.15594E+01_JPRB, 0.16126E+01_JPRB, 0.16365E+01_JPRB, 0.18059E+01_JPRB ,&
&  0.15181E+01_JPRB, 0.15811E+01_JPRB, 0.14928E+01_JPRB, 0.13310E+01_JPRB, 0.13124E+01_JPRB, 0.12192E+01_JPRB, 0.11775E+01_JPRB ,&
&  0.11674E+01_JPRB, 0.11107E+01_JPRB, 0.10791E+01_JPRB, 0.10508E+01_JPRB, 0.10207E+01_JPRB, 0.96641E+00_JPRB, 0.90954E+00_JPRB ,&
&  0.89279E+00_JPRB, 0.93216E+00_JPRB, 0.10368E+01_JPRB, 0.12654E+01_JPRB, 0.14873E+01_JPRB, 0.15556E+01_JPRB, 0.14547E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.13610E+01_JPRB, 0.12897E+01_JPRB, 0.12181E+01_JPRB, 0.12649E+01_JPRB, 0.13161E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.12533E+01_JPRB, 0.11228E+01_JPRB, 0.10650E+01_JPRB, 0.10889E+01_JPRB, 0.10954E+01_JPRB, 0.10607E+01_JPRB, 0.95272E+00_JPRB ,&
&  0.81504E+00_JPRB, 0.73422E+00_JPRB, 0.70068E+00_JPRB, 0.72699E+00_JPRB, 0.82928E+00_JPRB, 0.85102E+00_JPRB, 0.81227E+00_JPRB ,&
&  0.71709E+00_JPRB, 0.66246E+00_JPRB, 0.64678E+00_JPRB, 0.65838E+00_JPRB, 0.71456E+00_JPRB, 0.13701E+01_JPRB, 0.20523E+01_JPRB ,&
&  0.81731E+00_JPRB, 0.70919E+00_JPRB, 0.57201E+00_JPRB, 0.80897E+00_JPRB, 0.66332E+00_JPRB, 0.65596E+00_JPRB, 0.65604E+00_JPRB ,&
&  0.64930E+00_JPRB, 0.64845E+00_JPRB, 0.64643E+00_JPRB, 0.63114E+00_JPRB, 0.62035E+00_JPRB, 0.62188E+00_JPRB, 0.65296E+00_JPRB ,&
&  0.64972E+00_JPRB, 0.65239E+00_JPRB, 0.64143E+00_JPRB, 0.64698E+00_JPRB, 0.63900E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.66107E+00_JPRB, 0.65947E+00_JPRB, 0.66644E+00_JPRB, 0.68360E+00_JPRB, 0.68251E+00_JPRB, 0.68166E+00_JPRB, 0.71146E+00_JPRB ,&
&  0.69817E+00_JPRB, 0.68758E+00_JPRB, 0.68887E+00_JPRB, 0.66540E+00_JPRB, 0.68762E+00_JPRB, 0.70995E+00_JPRB, 0.75388E+00_JPRB ,&
&  0.79930E+00_JPRB, 0.82816E+00_JPRB, 0.85292E+00_JPRB, 0.88800E+00_JPRB, 0.95550E+00_JPRB, 0.10073E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.12488E+01_JPRB, 0.13779E+01_JPRB, 0.16714E+01_JPRB, 0.23509E+01_JPRB, 0.28730E+01_JPRB, 0.28247E+01_JPRB ,&
&  0.20701E+01_JPRB, 0.15176E+01_JPRB, 0.14363E+01_JPRB, 0.13031E+01_JPRB, 0.13541E+01_JPRB, 0.14820E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.14838E+01_JPRB, 0.14830E+01_JPRB, 0.14772E+01_JPRB, 0.14761E+01_JPRB, 0.14867E+01_JPRB, 0.14919E+01_JPRB, 0.14720E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.13464E+01_JPRB, 0.13736E+01_JPRB, 0.16194E+01_JPRB, 0.18201E+01_JPRB, 0.21683E+01_JPRB, 0.21347E+01_JPRB ,&
&  0.20421E+01_JPRB, 0.19474E+01_JPRB, 0.19460E+01_JPRB, 0.18781E+01_JPRB, 0.18014E+01_JPRB, 0.17553E+01_JPRB, 0.18031E+01_JPRB ,&
&  0.18613E+01_JPRB, 0.16732E+01_JPRB, 0.11359E+01_JPRB, 0.12129E+01_JPRB, 0.12088E+01_JPRB, 0.12028E+01_JPRB, 0.10156E+01_JPRB ,&
&  0.83547E+00_JPRB, 0.79384E+00_JPRB, 0.81532E+00_JPRB, 0.89281E+00_JPRB, 0.10402E+01_JPRB, 0.11213E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.88229E+00_JPRB, 0.76414E+00_JPRB, 0.70214E+00_JPRB, 0.69501E+00_JPRB, 0.79585E+00_JPRB, 0.91065E+00_JPRB, 0.11464E+01_JPRB ,&
&  0.90582E+00_JPRB, 0.10454E+01_JPRB, 0.96323E+00_JPRB, 0.12547E+01_JPRB, 0.84965E+00_JPRB, 0.78151E+00_JPRB, 0.77123E+00_JPRB ,&
&  0.77278E+00_JPRB, 0.76011E+00_JPRB, 0.74669E+00_JPRB, 0.72664E+00_JPRB, 0.70563E+00_JPRB, 0.74277E+00_JPRB, 0.78481E+00_JPRB ,&
&  0.74753E+00_JPRB, 0.77082E+00_JPRB, 0.74521E+00_JPRB, 0.70355E+00_JPRB, 0.68929E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.74681E+00_JPRB, 0.71387E+00_JPRB, 0.71385E+00_JPRB, 0.73090E+00_JPRB, 0.74376E+00_JPRB, 0.75629E+00_JPRB, 0.77048E+00_JPRB ,&
&  0.77901E+00_JPRB, 0.78586E+00_JPRB, 0.79683E+00_JPRB, 0.80365E+00_JPRB, 0.84398E+00_JPRB, 0.88993E+00_JPRB, 0.94878E+00_JPRB ,&
&  0.10360E+01_JPRB, 0.11156E+01_JPRB, 0.11867E+01_JPRB, 0.13085E+01_JPRB, 0.13941E+01_JPRB, 0.14791E+01_JPRB, 0.16149E+01_JPRB ,&
&  0.17926E+01_JPRB, 0.19905E+01_JPRB, 0.21790E+01_JPRB, 0.23725E+01_JPRB, 0.26136E+01_JPRB, 0.28046E+01_JPRB, 0.27965E+01_JPRB ,&
&  0.20838E+01_JPRB, 0.15756E+01_JPRB, 0.16769E+01_JPRB, 0.15725E+01_JPRB, 0.16222E+01_JPRB, 0.16589E+01_JPRB, 0.16649E+01_JPRB ,&
&  0.16857E+01_JPRB, 0.16931E+01_JPRB, 0.16998E+01_JPRB, 0.17114E+01_JPRB, 0.17665E+01_JPRB, 0.18438E+01_JPRB, 0.19441E+01_JPRB ,&
&  0.19887E+01_JPRB, 0.20234E+01_JPRB, 0.20307E+01_JPRB, 0.20087E+01_JPRB, 0.20822E+01_JPRB, 0.22838E+01_JPRB, 0.24422E+01_JPRB ,&
&  0.25194E+01_JPRB, 0.25708E+01_JPRB, 0.23553E+01_JPRB, 0.23286E+01_JPRB, 0.23009E+01_JPRB, 0.21335E+01_JPRB, 0.22041E+01_JPRB ,&
&  0.22955E+01_JPRB, 0.20212E+01_JPRB, 0.13198E+01_JPRB, 0.13155E+01_JPRB, 0.13144E+01_JPRB, 0.12117E+01_JPRB, 0.11283E+01_JPRB ,&
&  0.92986E+00_JPRB, 0.92017E+00_JPRB, 0.10037E+01_JPRB, 0.12067E+01_JPRB, 0.13207E+01_JPRB, 0.13767E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.96818E+00_JPRB, 0.82258E+00_JPRB, 0.73823E+00_JPRB, 0.73670E+00_JPRB, 0.76141E+00_JPRB, 0.89451E+00_JPRB, 0.95143E+00_JPRB ,&
&  0.10469E+01_JPRB, 0.12347E+01_JPRB, 0.15504E+01_JPRB, 0.12981E+01_JPRB, 0.94794E+00_JPRB, 0.94202E+00_JPRB, 0.93168E+00_JPRB ,&
&  0.92879E+00_JPRB, 0.93560E+00_JPRB, 0.93406E+00_JPRB, 0.93093E+00_JPRB, 0.92684E+00_JPRB, 0.93307E+00_JPRB, 0.92585E+00_JPRB ,&
&  0.91730E+00_JPRB, 0.89946E+00_JPRB, 0.88373E+00_JPRB, 0.83507E+00_JPRB, 0.78836E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.86547E+00_JPRB, 0.81681E+00_JPRB, 0.78440E+00_JPRB, 0.78227E+00_JPRB, 0.80100E+00_JPRB, 0.81857E+00_JPRB, 0.82861E+00_JPRB ,&
&  0.85264E+00_JPRB, 0.88586E+00_JPRB, 0.93320E+00_JPRB, 0.97584E+00_JPRB, 0.10277E+01_JPRB, 0.10765E+01_JPRB, 0.11510E+01_JPRB ,&
&  0.12280E+01_JPRB, 0.13158E+01_JPRB, 0.14605E+01_JPRB, 0.16744E+01_JPRB, 0.18445E+01_JPRB, 0.19953E+01_JPRB, 0.22129E+01_JPRB ,&
&  0.24334E+01_JPRB, 0.25632E+01_JPRB, 0.27079E+01_JPRB, 0.27648E+01_JPRB, 0.26378E+01_JPRB, 0.24607E+01_JPRB, 0.25144E+01_JPRB ,&
&  0.20859E+01_JPRB, 0.16869E+01_JPRB, 0.16573E+01_JPRB, 0.16685E+01_JPRB, 0.17025E+01_JPRB, 0.17076E+01_JPRB, 0.17204E+01_JPRB ,&
&  0.17424E+01_JPRB, 0.17530E+01_JPRB, 0.17614E+01_JPRB, 0.17809E+01_JPRB, 0.18279E+01_JPRB, 0.19182E+01_JPRB, 0.20456E+01_JPRB ,&
&  0.20779E+01_JPRB, 0.21242E+01_JPRB, 0.22061E+01_JPRB, 0.20253E+01_JPRB, 0.21265E+01_JPRB, 0.23047E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.24444E+01_JPRB, 0.23182E+01_JPRB, 0.23757E+01_JPRB, 0.23871E+01_JPRB, 0.23646E+01_JPRB, 0.23052E+01_JPRB, 0.22862E+01_JPRB ,&
&  0.23783E+01_JPRB, 0.22168E+01_JPRB, 0.15913E+01_JPRB, 0.15232E+01_JPRB, 0.15836E+01_JPRB, 0.12836E+01_JPRB, 0.12692E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11173E+01_JPRB, 0.12891E+01_JPRB, 0.14218E+01_JPRB, 0.14976E+01_JPRB, 0.16669E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.91839E+00_JPRB, 0.83665E+00_JPRB, 0.80955E+00_JPRB, 0.87267E+00_JPRB, 0.96590E+00_JPRB, 0.10078E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.10722E+01_JPRB, 0.11417E+01_JPRB, 0.11622E+01_JPRB, 0.10500E+01_JPRB, 0.10210E+01_JPRB, 0.10339E+01_JPRB ,&
&  0.10289E+01_JPRB, 0.10428E+01_JPRB, 0.10462E+01_JPRB, 0.10351E+01_JPRB, 0.10311E+01_JPRB, 0.10232E+01_JPRB, 0.10239E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10236E+01_JPRB, 0.10060E+01_JPRB, 0.97898E+00_JPRB, 0.92702E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.10089E+01_JPRB, 0.95098E+00_JPRB, 0.89063E+00_JPRB, 0.85238E+00_JPRB, 0.84073E+00_JPRB, 0.84264E+00_JPRB, 0.85901E+00_JPRB ,&
&  0.89075E+00_JPRB, 0.93038E+00_JPRB, 0.97597E+00_JPRB, 0.10333E+01_JPRB, 0.10900E+01_JPRB, 0.11507E+01_JPRB, 0.12345E+01_JPRB ,&
&  0.13335E+01_JPRB, 0.14675E+01_JPRB, 0.16543E+01_JPRB, 0.18646E+01_JPRB, 0.20497E+01_JPRB, 0.22393E+01_JPRB, 0.24206E+01_JPRB ,&
&  0.26104E+01_JPRB, 0.26802E+01_JPRB, 0.25952E+01_JPRB, 0.24178E+01_JPRB, 0.18938E+01_JPRB, 0.19286E+01_JPRB, 0.20473E+01_JPRB ,&
&  0.21047E+01_JPRB, 0.17786E+01_JPRB, 0.16571E+01_JPRB, 0.16338E+01_JPRB, 0.16407E+01_JPRB, 0.16594E+01_JPRB, 0.16700E+01_JPRB ,&
&  0.16961E+01_JPRB, 0.17231E+01_JPRB, 0.17461E+01_JPRB, 0.17660E+01_JPRB, 0.17944E+01_JPRB, 0.18602E+01_JPRB, 0.19869E+01_JPRB ,&
&  0.20313E+01_JPRB, 0.20706E+01_JPRB, 0.20880E+01_JPRB, 0.20464E+01_JPRB, 0.21059E+01_JPRB, 0.21814E+01_JPRB, 0.21867E+01_JPRB ,&
&  0.21742E+01_JPRB, 0.20462E+01_JPRB, 0.20746E+01_JPRB, 0.21626E+01_JPRB, 0.21173E+01_JPRB, 0.19763E+01_JPRB, 0.19744E+01_JPRB ,&
&  0.21625E+01_JPRB, 0.23605E+01_JPRB, 0.16584E+01_JPRB, 0.18753E+01_JPRB, 0.21706E+01_JPRB, 0.18197E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.14225E+01_JPRB, 0.15980E+01_JPRB, 0.16585E+01_JPRB, 0.16249E+01_JPRB, 0.18007E+01_JPRB, 0.14128E+01_JPRB ,&
&  0.12571E+01_JPRB, 0.10758E+01_JPRB, 0.10158E+01_JPRB, 0.10268E+01_JPRB, 0.11281E+01_JPRB, 0.11792E+01_JPRB, 0.10927E+01_JPRB ,&
&  0.99516E+00_JPRB, 0.13127E+01_JPRB, 0.14835E+01_JPRB, 0.17573E+01_JPRB, 0.17049E+01_JPRB, 0.10799E+01_JPRB, 0.11103E+01_JPRB ,&
&  0.11321E+01_JPRB, 0.11432E+01_JPRB, 0.11382E+01_JPRB, 0.11291E+01_JPRB, 0.11213E+01_JPRB, 0.11119E+01_JPRB, 0.11093E+01_JPRB ,&
&  0.11118E+01_JPRB, 0.11162E+01_JPRB, 0.11139E+01_JPRB, 0.10885E+01_JPRB, 0.10516E+01_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.11090E+01_JPRB, 0.10761E+01_JPRB, 0.10221E+01_JPRB, 0.96163E+00_JPRB, 0.92554E+00_JPRB, 0.90662E+00_JPRB, 0.88991E+00_JPRB ,&
&  0.90466E+00_JPRB, 0.93179E+00_JPRB, 0.96424E+00_JPRB, 0.10062E+01_JPRB, 0.10485E+01_JPRB, 0.10858E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.12548E+01_JPRB, 0.14068E+01_JPRB, 0.16405E+01_JPRB, 0.18939E+01_JPRB, 0.21297E+01_JPRB, 0.23901E+01_JPRB, 0.26453E+01_JPRB ,&
&  0.29304E+01_JPRB, 0.29658E+01_JPRB, 0.26445E+01_JPRB, 0.18205E+01_JPRB, 0.15873E+01_JPRB, 0.17008E+01_JPRB, 0.16376E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.15796E+01_JPRB, 0.15575E+01_JPRB, 0.15507E+01_JPRB, 0.15555E+01_JPRB, 0.15729E+01_JPRB, 0.15736E+01_JPRB ,&
&  0.15864E+01_JPRB, 0.16141E+01_JPRB, 0.16477E+01_JPRB, 0.16895E+01_JPRB, 0.17321E+01_JPRB, 0.17918E+01_JPRB, 0.18648E+01_JPRB ,&
&  0.19141E+01_JPRB, 0.19855E+01_JPRB, 0.19997E+01_JPRB, 0.19771E+01_JPRB, 0.19095E+01_JPRB, 0.18912E+01_JPRB, 0.18629E+01_JPRB ,&
&  0.18521E+01_JPRB, 0.18143E+01_JPRB, 0.18166E+01_JPRB, 0.19045E+01_JPRB, 0.19477E+01_JPRB, 0.17589E+01_JPRB, 0.17875E+01_JPRB ,&
&  0.20059E+01_JPRB, 0.21826E+01_JPRB, 0.19068E+01_JPRB, 0.23344E+01_JPRB, 0.14540E+01_JPRB, 0.17411E+01_JPRB, 0.19162E+01_JPRB ,&
&  0.18162E+01_JPRB, 0.18475E+01_JPRB, 0.19145E+01_JPRB, 0.19928E+01_JPRB, 0.19966E+01_JPRB, 0.17670E+01_JPRB, 0.16161E+01_JPRB ,&
&  0.14219E+01_JPRB, 0.12079E+01_JPRB, 0.11044E+01_JPRB, 0.11682E+01_JPRB, 0.13183E+01_JPRB, 0.14013E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.13592E+01_JPRB, 0.14486E+01_JPRB, 0.16434E+01_JPRB, 0.13805E+01_JPRB, 0.11082E+01_JPRB, 0.11467E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.11874E+01_JPRB, 0.11911E+01_JPRB, 0.11722E+01_JPRB, 0.11707E+01_JPRB, 0.11662E+01_JPRB, 0.11629E+01_JPRB ,&
&  0.11690E+01_JPRB, 0.11848E+01_JPRB, 0.11889E+01_JPRB, 0.11643E+01_JPRB, 0.11324E+01_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.11985E+01_JPRB, 0.11634E+01_JPRB, 0.11396E+01_JPRB, 0.11050E+01_JPRB, 0.10404E+01_JPRB, 0.10083E+01_JPRB, 0.98839E+00_JPRB ,&
&  0.94796E+00_JPRB, 0.90562E+00_JPRB, 0.91824E+00_JPRB, 0.93523E+00_JPRB, 0.95924E+00_JPRB, 0.99265E+00_JPRB, 0.10458E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.11883E+01_JPRB, 0.13685E+01_JPRB, 0.16565E+01_JPRB, 0.20048E+01_JPRB, 0.24314E+01_JPRB, 0.29606E+01_JPRB ,&
&  0.29242E+01_JPRB, 0.19779E+01_JPRB, 0.22887E+01_JPRB, 0.21806E+01_JPRB, 0.20286E+01_JPRB, 0.19162E+01_JPRB, 0.18055E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.15886E+01_JPRB, 0.15730E+01_JPRB, 0.15304E+01_JPRB, 0.14663E+01_JPRB, 0.14785E+01_JPRB, 0.14877E+01_JPRB ,&
&  0.14820E+01_JPRB, 0.14861E+01_JPRB, 0.15009E+01_JPRB, 0.15316E+01_JPRB, 0.15957E+01_JPRB, 0.16809E+01_JPRB, 0.17498E+01_JPRB ,&
&  0.18008E+01_JPRB, 0.18499E+01_JPRB, 0.18130E+01_JPRB, 0.17548E+01_JPRB, 0.17134E+01_JPRB, 0.17292E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.16891E+01_JPRB, 0.16611E+01_JPRB, 0.17253E+01_JPRB, 0.18381E+01_JPRB, 0.19089E+01_JPRB, 0.17925E+01_JPRB, 0.19277E+01_JPRB ,&
&  0.20075E+01_JPRB, 0.21118E+01_JPRB, 0.21183E+01_JPRB, 0.24927E+01_JPRB, 0.17911E+01_JPRB, 0.19408E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.20646E+01_JPRB, 0.20916E+01_JPRB, 0.21227E+01_JPRB, 0.21673E+01_JPRB, 0.23320E+01_JPRB, 0.20542E+01_JPRB, 0.18768E+01_JPRB ,&
&  0.15532E+01_JPRB, 0.13465E+01_JPRB, 0.12172E+01_JPRB, 0.13693E+01_JPRB, 0.14790E+01_JPRB, 0.15449E+01_JPRB, 0.17120E+01_JPRB ,&
&  0.19488E+01_JPRB, 0.16884E+01_JPRB, 0.14581E+01_JPRB, 0.14141E+01_JPRB, 0.11939E+01_JPRB, 0.11045E+01_JPRB, 0.11254E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.12072E+01_JPRB, 0.12084E+01_JPRB, 0.11981E+01_JPRB, 0.11935E+01_JPRB, 0.11965E+01_JPRB, 0.12156E+01_JPRB ,&
&  0.12324E+01_JPRB, 0.12512E+01_JPRB, 0.12584E+01_JPRB, 0.12464E+01_JPRB, 0.12240E+01_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.12899E+01_JPRB, 0.12481E+01_JPRB, 0.12019E+01_JPRB, 0.11710E+01_JPRB, 0.11734E+01_JPRB, 0.11552E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.98799E+00_JPRB, 0.89498E+00_JPRB, 0.89161E+00_JPRB, 0.90296E+00_JPRB, 0.91435E+00_JPRB, 0.94766E+00_JPRB ,&
&  0.98325E+00_JPRB, 0.10211E+01_JPRB, 0.10950E+01_JPRB, 0.12623E+01_JPRB, 0.15826E+01_JPRB, 0.21089E+01_JPRB, 0.24219E+01_JPRB ,&
&  0.25860E+01_JPRB, 0.26668E+01_JPRB, 0.26456E+01_JPRB, 0.21604E+01_JPRB, 0.20210E+01_JPRB, 0.20267E+01_JPRB, 0.20565E+01_JPRB ,&
&  0.17050E+01_JPRB, 0.15802E+01_JPRB, 0.15637E+01_JPRB, 0.15249E+01_JPRB, 0.14815E+01_JPRB, 0.14720E+01_JPRB, 0.14682E+01_JPRB ,&
&  0.14559E+01_JPRB, 0.14334E+01_JPRB, 0.14179E+01_JPRB, 0.14174E+01_JPRB, 0.14451E+01_JPRB, 0.15195E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.16836E+01_JPRB, 0.17587E+01_JPRB, 0.17205E+01_JPRB, 0.16312E+01_JPRB, 0.16135E+01_JPRB, 0.16660E+01_JPRB, 0.16821E+01_JPRB ,&
&  0.16624E+01_JPRB, 0.16800E+01_JPRB, 0.17243E+01_JPRB, 0.17395E+01_JPRB, 0.16781E+01_JPRB, 0.18837E+01_JPRB, 0.19045E+01_JPRB ,&
&  0.20490E+01_JPRB, 0.20918E+01_JPRB, 0.24617E+01_JPRB, 0.19676E+01_JPRB, 0.21088E+01_JPRB, 0.21934E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.19744E+01_JPRB, 0.21663E+01_JPRB, 0.23263E+01_JPRB, 0.23175E+01_JPRB, 0.23851E+01_JPRB, 0.21559E+01_JPRB, 0.21421E+01_JPRB ,&
&  0.16795E+01_JPRB, 0.16694E+01_JPRB, 0.13898E+01_JPRB, 0.15134E+01_JPRB, 0.15837E+01_JPRB, 0.15616E+01_JPRB, 0.18701E+01_JPRB ,&
&  0.23684E+01_JPRB, 0.23172E+01_JPRB, 0.20663E+01_JPRB, 0.16639E+01_JPRB, 0.12255E+01_JPRB, 0.11036E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.11196E+01_JPRB, 0.11972E+01_JPRB, 0.12063E+01_JPRB, 0.12095E+01_JPRB, 0.12054E+01_JPRB, 0.12139E+01_JPRB, 0.12398E+01_JPRB ,&
&  0.12840E+01_JPRB, 0.13195E+01_JPRB, 0.13386E+01_JPRB, 0.13350E+01_JPRB, 0.13228E+01_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.13693E+01_JPRB, 0.13192E+01_JPRB, 0.12484E+01_JPRB, 0.11800E+01_JPRB, 0.11216E+01_JPRB, 0.10827E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.13151E+01_JPRB, 0.10312E+01_JPRB, 0.92526E+00_JPRB, 0.90291E+00_JPRB, 0.91145E+00_JPRB, 0.90707E+00_JPRB, 0.90868E+00_JPRB ,&
&  0.91933E+00_JPRB, 0.93722E+00_JPRB, 0.97799E+00_JPRB, 0.10382E+01_JPRB, 0.11894E+01_JPRB, 0.15416E+01_JPRB, 0.17685E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.33570E+01_JPRB, 0.25880E+01_JPRB, 0.23180E+01_JPRB, 0.20762E+01_JPRB, 0.20584E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.16214E+01_JPRB, 0.15873E+01_JPRB, 0.16232E+01_JPRB, 0.16357E+01_JPRB, 0.16396E+01_JPRB, 0.16028E+01_JPRB, 0.15601E+01_JPRB ,&
&  0.15099E+01_JPRB, 0.14718E+01_JPRB, 0.14446E+01_JPRB, 0.14227E+01_JPRB, 0.14148E+01_JPRB, 0.14493E+01_JPRB, 0.14882E+01_JPRB ,&
&  0.15666E+01_JPRB, 0.17015E+01_JPRB, 0.16901E+01_JPRB, 0.16438E+01_JPRB, 0.16180E+01_JPRB, 0.16729E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.16305E+01_JPRB, 0.15543E+01_JPRB, 0.16695E+01_JPRB, 0.17402E+01_JPRB, 0.17896E+01_JPRB, 0.19748E+01_JPRB, 0.19670E+01_JPRB ,&
&  0.21841E+01_JPRB, 0.21312E+01_JPRB, 0.24363E+01_JPRB, 0.20974E+01_JPRB, 0.20702E+01_JPRB, 0.23359E+01_JPRB, 0.22869E+01_JPRB ,&
&  0.20342E+01_JPRB, 0.21825E+01_JPRB, 0.23416E+01_JPRB, 0.24542E+01_JPRB, 0.23591E+01_JPRB, 0.22239E+01_JPRB, 0.20791E+01_JPRB ,&
&  0.19043E+01_JPRB, 0.19804E+01_JPRB, 0.17993E+01_JPRB, 0.15609E+01_JPRB, 0.16948E+01_JPRB, 0.16237E+01_JPRB, 0.21378E+01_JPRB ,&
&  0.27270E+01_JPRB, 0.27428E+01_JPRB, 0.28198E+01_JPRB, 0.25708E+01_JPRB, 0.17744E+01_JPRB, 0.13647E+01_JPRB, 0.11758E+01_JPRB ,&
&  0.11589E+01_JPRB, 0.11915E+01_JPRB, 0.12001E+01_JPRB, 0.11819E+01_JPRB, 0.11736E+01_JPRB, 0.11766E+01_JPRB, 0.11953E+01_JPRB ,&
&  0.12435E+01_JPRB, 0.13054E+01_JPRB, 0.13491E+01_JPRB, 0.13763E+01_JPRB, 0.13945E+01_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.14554E+01_JPRB, 0.14316E+01_JPRB, 0.13345E+01_JPRB, 0.12345E+01_JPRB, 0.11691E+01_JPRB, 0.11196E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.10650E+01_JPRB, 0.96290E+00_JPRB, 0.92222E+00_JPRB, 0.91337E+00_JPRB, 0.92101E+00_JPRB, 0.91887E+00_JPRB ,&
&  0.91382E+00_JPRB, 0.91711E+00_JPRB, 0.95058E+00_JPRB, 0.97351E+00_JPRB, 0.10428E+01_JPRB, 0.12561E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.17430E+01_JPRB, 0.29895E+01_JPRB, 0.26126E+01_JPRB, 0.24363E+01_JPRB, 0.21817E+01_JPRB, 0.21177E+01_JPRB, 0.17202E+01_JPRB ,&
&  0.16057E+01_JPRB, 0.16351E+01_JPRB, 0.16950E+01_JPRB, 0.17631E+01_JPRB, 0.18008E+01_JPRB, 0.17875E+01_JPRB, 0.17004E+01_JPRB ,&
&  0.16145E+01_JPRB, 0.15423E+01_JPRB, 0.15057E+01_JPRB, 0.14621E+01_JPRB, 0.14340E+01_JPRB, 0.14452E+01_JPRB, 0.14686E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.16053E+01_JPRB, 0.17233E+01_JPRB, 0.16230E+01_JPRB, 0.16256E+01_JPRB, 0.16646E+01_JPRB, 0.17012E+01_JPRB ,&
&  0.16069E+01_JPRB, 0.15103E+01_JPRB, 0.16545E+01_JPRB, 0.18272E+01_JPRB, 0.19871E+01_JPRB, 0.21152E+01_JPRB, 0.21805E+01_JPRB ,&
&  0.23590E+01_JPRB, 0.23430E+01_JPRB, 0.22715E+01_JPRB, 0.20664E+01_JPRB, 0.21742E+01_JPRB, 0.24637E+01_JPRB, 0.25479E+01_JPRB ,&
&  0.25509E+01_JPRB, 0.23375E+01_JPRB, 0.23709E+01_JPRB, 0.24456E+01_JPRB, 0.23643E+01_JPRB, 0.21747E+01_JPRB, 0.22985E+01_JPRB ,&
&  0.25133E+01_JPRB, 0.28792E+01_JPRB, 0.26614E+01_JPRB, 0.21653E+01_JPRB, 0.18420E+01_JPRB, 0.15593E+01_JPRB, 0.19886E+01_JPRB ,&
&  0.26600E+01_JPRB, 0.27902E+01_JPRB, 0.27960E+01_JPRB, 0.32846E+01_JPRB, 0.25189E+01_JPRB, 0.19835E+01_JPRB, 0.15552E+01_JPRB ,&
&  0.13035E+01_JPRB, 0.12354E+01_JPRB, 0.12273E+01_JPRB, 0.11967E+01_JPRB, 0.11763E+01_JPRB, 0.11595E+01_JPRB, 0.11572E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.12372E+01_JPRB, 0.13107E+01_JPRB, 0.13900E+01_JPRB, 0.14467E+01_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.14966E+01_JPRB, 0.14903E+01_JPRB, 0.13818E+01_JPRB, 0.12574E+01_JPRB, 0.11713E+01_JPRB, 0.11211E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.10789E+01_JPRB, 0.10342E+01_JPRB, 0.97867E+00_JPRB, 0.94383E+00_JPRB, 0.94220E+00_JPRB, 0.94113E+00_JPRB, 0.94122E+00_JPRB ,&
&  0.92522E+00_JPRB, 0.91664E+00_JPRB, 0.95770E+00_JPRB, 0.98131E+00_JPRB, 0.10382E+01_JPRB, 0.91695E+00_JPRB, 0.10645E+01_JPRB ,&
&  0.18932E+01_JPRB, 0.28447E+01_JPRB, 0.26181E+01_JPRB, 0.26134E+01_JPRB, 0.25057E+01_JPRB, 0.25099E+01_JPRB, 0.17727E+01_JPRB ,&
&  0.16348E+01_JPRB, 0.17112E+01_JPRB, 0.17768E+01_JPRB, 0.18931E+01_JPRB, 0.19310E+01_JPRB, 0.19086E+01_JPRB, 0.18607E+01_JPRB ,&
&  0.17547E+01_JPRB, 0.16386E+01_JPRB, 0.15656E+01_JPRB, 0.14978E+01_JPRB, 0.14313E+01_JPRB, 0.14050E+01_JPRB, 0.14377E+01_JPRB ,&
&  0.14824E+01_JPRB, 0.15341E+01_JPRB, 0.16592E+01_JPRB, 0.16320E+01_JPRB, 0.15852E+01_JPRB, 0.15997E+01_JPRB, 0.16649E+01_JPRB ,&
&  0.16674E+01_JPRB, 0.17142E+01_JPRB, 0.17584E+01_JPRB, 0.17978E+01_JPRB, 0.19978E+01_JPRB, 0.22768E+01_JPRB, 0.24967E+01_JPRB ,&
&  0.26842E+01_JPRB, 0.25362E+01_JPRB, 0.21994E+01_JPRB, 0.22064E+01_JPRB, 0.24303E+01_JPRB, 0.27903E+01_JPRB, 0.26154E+01_JPRB ,&
&  0.22748E+01_JPRB, 0.18383E+01_JPRB, 0.18120E+01_JPRB, 0.21332E+01_JPRB, 0.23496E+01_JPRB, 0.23487E+01_JPRB, 0.28336E+01_JPRB ,&
&  0.27274E+01_JPRB, 0.16103E+01_JPRB, 0.13409E+01_JPRB, 0.16901E+01_JPRB, 0.12978E+01_JPRB, 0.11628E+01_JPRB, 0.20852E+01_JPRB ,&
&  0.27098E+01_JPRB, 0.32497E+01_JPRB, 0.33699E+01_JPRB, 0.30686E+01_JPRB, 0.29929E+01_JPRB, 0.27545E+01_JPRB, 0.23250E+01_JPRB ,&
&  0.19398E+01_JPRB, 0.16211E+01_JPRB, 0.13981E+01_JPRB, 0.12735E+01_JPRB, 0.12194E+01_JPRB, 0.11879E+01_JPRB, 0.11678E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.12110E+01_JPRB, 0.12760E+01_JPRB, 0.13774E+01_JPRB, 0.14665E+01_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.15380E+01_JPRB, 0.14983E+01_JPRB, 0.13831E+01_JPRB, 0.12470E+01_JPRB, 0.11613E+01_JPRB, 0.10993E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.10066E+01_JPRB, 0.98583E+00_JPRB, 0.97043E+00_JPRB, 0.96965E+00_JPRB, 0.96591E+00_JPRB, 0.96045E+00_JPRB ,&
&  0.94435E+00_JPRB, 0.92046E+00_JPRB, 0.98425E+00_JPRB, 0.10497E+01_JPRB, 0.10517E+01_JPRB, 0.98598E+00_JPRB, 0.11957E+01_JPRB ,&
&  0.17875E+01_JPRB, 0.26341E+01_JPRB, 0.29904E+01_JPRB, 0.30280E+01_JPRB, 0.27643E+01_JPRB, 0.26290E+01_JPRB, 0.20347E+01_JPRB ,&
&  0.18043E+01_JPRB, 0.18464E+01_JPRB, 0.18683E+01_JPRB, 0.19582E+01_JPRB, 0.20348E+01_JPRB, 0.20204E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.18492E+01_JPRB, 0.17231E+01_JPRB, 0.16228E+01_JPRB, 0.15432E+01_JPRB, 0.14736E+01_JPRB, 0.14113E+01_JPRB, 0.14011E+01_JPRB ,&
&  0.14441E+01_JPRB, 0.14937E+01_JPRB, 0.15895E+01_JPRB, 0.17017E+01_JPRB, 0.14135E+01_JPRB, 0.15626E+01_JPRB, 0.16403E+01_JPRB ,&
&  0.17089E+01_JPRB, 0.17928E+01_JPRB, 0.17718E+01_JPRB, 0.19112E+01_JPRB, 0.21221E+01_JPRB, 0.23058E+01_JPRB, 0.26177E+01_JPRB ,&
&  0.30565E+01_JPRB, 0.28819E+01_JPRB, 0.25776E+01_JPRB, 0.25832E+01_JPRB, 0.27571E+01_JPRB, 0.30903E+01_JPRB, 0.21001E+01_JPRB ,&
&  0.19120E+01_JPRB, 0.22324E+01_JPRB, 0.20404E+01_JPRB, 0.18671E+01_JPRB, 0.22559E+01_JPRB, 0.24738E+01_JPRB, 0.21037E+01_JPRB ,&
&  0.97767E+00_JPRB, 0.71754E+00_JPRB, 0.72093E+00_JPRB, 0.73110E+00_JPRB, 0.73394E+00_JPRB, 0.10754E+01_JPRB, 0.33819E+01_JPRB ,&
&  0.29374E+01_JPRB, 0.35496E+01_JPRB, 0.36155E+01_JPRB, 0.33216E+01_JPRB, 0.32155E+01_JPRB, 0.30541E+01_JPRB, 0.29244E+01_JPRB ,&
&  0.25920E+01_JPRB, 0.22907E+01_JPRB, 0.19998E+01_JPRB, 0.17002E+01_JPRB, 0.14583E+01_JPRB, 0.13026E+01_JPRB, 0.12414E+01_JPRB ,&
&  0.12392E+01_JPRB, 0.12732E+01_JPRB, 0.13277E+01_JPRB, 0.14124E+01_JPRB, 0.15133E+01_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.15936E+01_JPRB, 0.15056E+01_JPRB, 0.13678E+01_JPRB, 0.12171E+01_JPRB, 0.11316E+01_JPRB, 0.10667E+01_JPRB, 0.10200E+01_JPRB ,&
&  0.99993E+00_JPRB, 0.98775E+00_JPRB, 0.98226E+00_JPRB, 0.97521E+00_JPRB, 0.96980E+00_JPRB, 0.96746E+00_JPRB, 0.96118E+00_JPRB ,&
&  0.94240E+00_JPRB, 0.92311E+00_JPRB, 0.10445E+01_JPRB, 0.10359E+01_JPRB, 0.10643E+01_JPRB, 0.10053E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.15865E+01_JPRB, 0.23861E+01_JPRB, 0.29892E+01_JPRB, 0.30919E+01_JPRB, 0.31572E+01_JPRB, 0.30921E+01_JPRB, 0.27851E+01_JPRB ,&
&  0.24752E+01_JPRB, 0.23451E+01_JPRB, 0.22199E+01_JPRB, 0.21556E+01_JPRB, 0.21039E+01_JPRB, 0.20681E+01_JPRB, 0.19696E+01_JPRB ,&
&  0.18567E+01_JPRB, 0.17756E+01_JPRB, 0.16872E+01_JPRB, 0.16209E+01_JPRB, 0.15534E+01_JPRB, 0.14935E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.14426E+01_JPRB, 0.14762E+01_JPRB, 0.15306E+01_JPRB, 0.16637E+01_JPRB, 0.15670E+01_JPRB, 0.14468E+01_JPRB, 0.16166E+01_JPRB ,&
&  0.17877E+01_JPRB, 0.20000E+01_JPRB, 0.20145E+01_JPRB, 0.21962E+01_JPRB, 0.22644E+01_JPRB, 0.23980E+01_JPRB, 0.28730E+01_JPRB ,&
&  0.32767E+01_JPRB, 0.36460E+01_JPRB, 0.30004E+01_JPRB, 0.29615E+01_JPRB, 0.31762E+01_JPRB, 0.25218E+01_JPRB, 0.23702E+01_JPRB ,&
&  0.27195E+01_JPRB, 0.23679E+01_JPRB, 0.20707E+01_JPRB, 0.17363E+01_JPRB, 0.21635E+01_JPRB, 0.19830E+01_JPRB, 0.10231E+01_JPRB ,&
&  0.76662E+00_JPRB, 0.67581E+00_JPRB, 0.62818E+00_JPRB, 0.63078E+00_JPRB, 0.68548E+00_JPRB, 0.10970E+01_JPRB, 0.21079E+01_JPRB ,&
&  0.25194E+01_JPRB, 0.32129E+01_JPRB, 0.35547E+01_JPRB, 0.35808E+01_JPRB, 0.34017E+01_JPRB, 0.33089E+01_JPRB, 0.31817E+01_JPRB ,&
&  0.32257E+01_JPRB, 0.29872E+01_JPRB, 0.25997E+01_JPRB, 0.22820E+01_JPRB, 0.20384E+01_JPRB, 0.17960E+01_JPRB, 0.16039E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.14465E+01_JPRB, 0.14603E+01_JPRB, 0.15317E+01_JPRB, 0.16280E+01_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.16584E+01_JPRB, 0.15657E+01_JPRB, 0.14046E+01_JPRB, 0.12278E+01_JPRB, 0.11219E+01_JPRB, 0.10403E+01_JPRB, 0.99140E+00_JPRB ,&
&  0.97862E+00_JPRB, 0.97784E+00_JPRB, 0.97386E+00_JPRB, 0.96426E+00_JPRB, 0.95266E+00_JPRB, 0.94677E+00_JPRB, 0.94530E+00_JPRB ,&
&  0.92316E+00_JPRB, 0.94260E+00_JPRB, 0.98758E+00_JPRB, 0.88594E+00_JPRB, 0.87875E+00_JPRB, 0.88220E+00_JPRB, 0.98528E+00_JPRB ,&
&  0.14470E+01_JPRB, 0.22233E+01_JPRB, 0.28585E+01_JPRB, 0.32157E+01_JPRB, 0.34456E+01_JPRB, 0.33237E+01_JPRB, 0.36327E+01_JPRB ,&
&  0.33437E+01_JPRB, 0.30430E+01_JPRB, 0.27686E+01_JPRB, 0.25514E+01_JPRB, 0.23245E+01_JPRB, 0.21659E+01_JPRB, 0.20579E+01_JPRB ,&
&  0.18950E+01_JPRB, 0.17574E+01_JPRB, 0.16862E+01_JPRB, 0.16327E+01_JPRB, 0.15857E+01_JPRB, 0.15515E+01_JPRB, 0.15216E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.15120E+01_JPRB, 0.15377E+01_JPRB, 0.16007E+01_JPRB, 0.17961E+01_JPRB, 0.18030E+01_JPRB, 0.17539E+01_JPRB ,&
&  0.17546E+01_JPRB, 0.18582E+01_JPRB, 0.21960E+01_JPRB, 0.22311E+01_JPRB, 0.23532E+01_JPRB, 0.26558E+01_JPRB, 0.31515E+01_JPRB ,&
&  0.35701E+01_JPRB, 0.38694E+01_JPRB, 0.34128E+01_JPRB, 0.34590E+01_JPRB, 0.30979E+01_JPRB, 0.24931E+01_JPRB, 0.28858E+01_JPRB ,&
&  0.31872E+01_JPRB, 0.27947E+01_JPRB, 0.22430E+01_JPRB, 0.17499E+01_JPRB, 0.16562E+01_JPRB, 0.12133E+01_JPRB, 0.77196E+00_JPRB ,&
&  0.71564E+00_JPRB, 0.68446E+00_JPRB, 0.63971E+00_JPRB, 0.74224E+00_JPRB, 0.76474E+00_JPRB, 0.11055E+01_JPRB, 0.18359E+01_JPRB ,&
&  0.25847E+01_JPRB, 0.27890E+01_JPRB, 0.32103E+01_JPRB, 0.33892E+01_JPRB, 0.33753E+01_JPRB, 0.36018E+01_JPRB, 0.38786E+01_JPRB ,&
&  0.34214E+01_JPRB, 0.32532E+01_JPRB, 0.30766E+01_JPRB, 0.26563E+01_JPRB, 0.24193E+01_JPRB, 0.22573E+01_JPRB, 0.20807E+01_JPRB ,&
&  0.18785E+01_JPRB, 0.17079E+01_JPRB, 0.16575E+01_JPRB, 0.16886E+01_JPRB, 0.17140E+01_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.17168E+01_JPRB, 0.16266E+01_JPRB, 0.14670E+01_JPRB, 0.12811E+01_JPRB, 0.11459E+01_JPRB, 0.10528E+01_JPRB, 0.10096E+01_JPRB ,&
&  0.10071E+01_JPRB, 0.10031E+01_JPRB, 0.99855E+00_JPRB, 0.98569E+00_JPRB, 0.97085E+00_JPRB, 0.95489E+00_JPRB, 0.95104E+00_JPRB ,&
&  0.92183E+00_JPRB, 0.10487E+01_JPRB, 0.80357E+00_JPRB, 0.67121E+00_JPRB, 0.73359E+00_JPRB, 0.81211E+00_JPRB, 0.82123E+00_JPRB ,&
&  0.13217E+01_JPRB, 0.21002E+01_JPRB, 0.26979E+01_JPRB, 0.34197E+01_JPRB, 0.43918E+01_JPRB, 0.41535E+01_JPRB, 0.41982E+01_JPRB ,&
&  0.40986E+01_JPRB, 0.37609E+01_JPRB, 0.33734E+01_JPRB, 0.29508E+01_JPRB, 0.25392E+01_JPRB, 0.22814E+01_JPRB, 0.21087E+01_JPRB ,&
&  0.19255E+01_JPRB, 0.17656E+01_JPRB, 0.16704E+01_JPRB, 0.16309E+01_JPRB, 0.15886E+01_JPRB, 0.15556E+01_JPRB, 0.15466E+01_JPRB ,&
&  0.15478E+01_JPRB, 0.15592E+01_JPRB, 0.15780E+01_JPRB, 0.16826E+01_JPRB, 0.17554E+01_JPRB, 0.17560E+01_JPRB, 0.21471E+01_JPRB ,&
&  0.21094E+01_JPRB, 0.21251E+01_JPRB, 0.21509E+01_JPRB, 0.23754E+01_JPRB, 0.25912E+01_JPRB, 0.28715E+01_JPRB, 0.35705E+01_JPRB ,&
&  0.33674E+01_JPRB, 0.33016E+01_JPRB, 0.34079E+01_JPRB, 0.33714E+01_JPRB, 0.30067E+01_JPRB, 0.33486E+01_JPRB, 0.45347E+01_JPRB ,&
&  0.35218E+01_JPRB, 0.35660E+01_JPRB, 0.35882E+01_JPRB, 0.34980E+01_JPRB, 0.16968E+01_JPRB, 0.11706E+01_JPRB, 0.18634E+01_JPRB ,&
&  0.20112E+01_JPRB, 0.12454E+01_JPRB, 0.96098E+00_JPRB, 0.13658E+01_JPRB, 0.11118E+01_JPRB, 0.13846E+01_JPRB, 0.20443E+01_JPRB ,&
&  0.22554E+01_JPRB, 0.21655E+01_JPRB, 0.29153E+01_JPRB, 0.33953E+01_JPRB, 0.35712E+01_JPRB, 0.37786E+01_JPRB, 0.38755E+01_JPRB ,&
&  0.38474E+01_JPRB, 0.33849E+01_JPRB, 0.33485E+01_JPRB, 0.29423E+01_JPRB, 0.26093E+01_JPRB, 0.23986E+01_JPRB, 0.22413E+01_JPRB ,&
&  0.20790E+01_JPRB, 0.19385E+01_JPRB, 0.18684E+01_JPRB, 0.18477E+01_JPRB, 0.17891E+01_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.17333E+01_JPRB, 0.16510E+01_JPRB, 0.14956E+01_JPRB, 0.13091E+01_JPRB, 0.11378E+01_JPRB, 0.10185E+01_JPRB, 0.98897E+00_JPRB ,&
&  0.10032E+01_JPRB, 0.10169E+01_JPRB, 0.10171E+01_JPRB, 0.10029E+01_JPRB, 0.98217E+00_JPRB, 0.96165E+00_JPRB, 0.93635E+00_JPRB ,&
&  0.89461E+00_JPRB, 0.82270E+00_JPRB, 0.63788E+00_JPRB, 0.56358E+00_JPRB, 0.71086E+00_JPRB, 0.72909E+00_JPRB, 0.79856E+00_JPRB ,&
&  0.11763E+01_JPRB, 0.18095E+01_JPRB, 0.25297E+01_JPRB, 0.33202E+01_JPRB, 0.40765E+01_JPRB, 0.44408E+01_JPRB, 0.44159E+01_JPRB ,&
&  0.45354E+01_JPRB, 0.41821E+01_JPRB, 0.36294E+01_JPRB, 0.31418E+01_JPRB, 0.27821E+01_JPRB, 0.25346E+01_JPRB, 0.22643E+01_JPRB ,&
&  0.19897E+01_JPRB, 0.17932E+01_JPRB, 0.16758E+01_JPRB, 0.16031E+01_JPRB, 0.15695E+01_JPRB, 0.15285E+01_JPRB, 0.14944E+01_JPRB ,&
&  0.15162E+01_JPRB, 0.15728E+01_JPRB, 0.16353E+01_JPRB, 0.17076E+01_JPRB, 0.16317E+01_JPRB, 0.17311E+01_JPRB, 0.20226E+01_JPRB ,&
&  0.21918E+01_JPRB, 0.21759E+01_JPRB, 0.22616E+01_JPRB, 0.24737E+01_JPRB, 0.28918E+01_JPRB, 0.30947E+01_JPRB, 0.40668E+01_JPRB ,&
&  0.37857E+01_JPRB, 0.34860E+01_JPRB, 0.32272E+01_JPRB, 0.27415E+01_JPRB, 0.31537E+01_JPRB, 0.47322E+01_JPRB, 0.47372E+01_JPRB ,&
&  0.43563E+01_JPRB, 0.41249E+01_JPRB, 0.38777E+01_JPRB, 0.33957E+01_JPRB, 0.27267E+01_JPRB, 0.20184E+01_JPRB, 0.24811E+01_JPRB ,&
&  0.26533E+01_JPRB, 0.26580E+01_JPRB, 0.25060E+01_JPRB, 0.21279E+01_JPRB, 0.18676E+01_JPRB, 0.22133E+01_JPRB, 0.20840E+01_JPRB ,&
&  0.18094E+01_JPRB, 0.17618E+01_JPRB, 0.23690E+01_JPRB, 0.31702E+01_JPRB, 0.35227E+01_JPRB, 0.34302E+01_JPRB, 0.37716E+01_JPRB ,&
&  0.36429E+01_JPRB, 0.35707E+01_JPRB, 0.33628E+01_JPRB, 0.30206E+01_JPRB, 0.27198E+01_JPRB, 0.24697E+01_JPRB, 0.22650E+01_JPRB ,&
&  0.21508E+01_JPRB, 0.20840E+01_JPRB, 0.20056E+01_JPRB, 0.19224E+01_JPRB, 0.18153E+01_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.17372E+01_JPRB, 0.16454E+01_JPRB, 0.14841E+01_JPRB, 0.12915E+01_JPRB, 0.11121E+01_JPRB, 0.97558E+00_JPRB, 0.94220E+00_JPRB ,&
&  0.95873E+00_JPRB, 0.97314E+00_JPRB, 0.97318E+00_JPRB, 0.95960E+00_JPRB, 0.93755E+00_JPRB, 0.90304E+00_JPRB, 0.86959E+00_JPRB ,&
&  0.84436E+00_JPRB, 0.74900E+00_JPRB, 0.59445E+00_JPRB, 0.57328E+00_JPRB, 0.63297E+00_JPRB, 0.67960E+00_JPRB, 0.82232E+00_JPRB ,&
&  0.10721E+01_JPRB, 0.16580E+01_JPRB, 0.21903E+01_JPRB, 0.28160E+01_JPRB, 0.34296E+01_JPRB, 0.38947E+01_JPRB, 0.39331E+01_JPRB ,&
&  0.37122E+01_JPRB, 0.37215E+01_JPRB, 0.34910E+01_JPRB, 0.31472E+01_JPRB, 0.28262E+01_JPRB, 0.25728E+01_JPRB, 0.23338E+01_JPRB ,&
&  0.20837E+01_JPRB, 0.18449E+01_JPRB, 0.16986E+01_JPRB, 0.16163E+01_JPRB, 0.15689E+01_JPRB, 0.15348E+01_JPRB, 0.15159E+01_JPRB ,&
&  0.15217E+01_JPRB, 0.15636E+01_JPRB, 0.16046E+01_JPRB, 0.16862E+01_JPRB, 0.16477E+01_JPRB, 0.18682E+01_JPRB, 0.21035E+01_JPRB ,&
&  0.25213E+01_JPRB, 0.24925E+01_JPRB, 0.25728E+01_JPRB, 0.29866E+01_JPRB, 0.29142E+01_JPRB, 0.33970E+01_JPRB, 0.47596E+01_JPRB ,&
&  0.51534E+01_JPRB, 0.51258E+01_JPRB, 0.51774E+01_JPRB, 0.41897E+01_JPRB, 0.36630E+01_JPRB, 0.52373E+01_JPRB, 0.49459E+01_JPRB ,&
&  0.45188E+01_JPRB, 0.43243E+01_JPRB, 0.39665E+01_JPRB, 0.39850E+01_JPRB, 0.32841E+01_JPRB, 0.28717E+01_JPRB, 0.21294E+01_JPRB ,&
&  0.22581E+01_JPRB, 0.21700E+01_JPRB, 0.24811E+01_JPRB, 0.23048E+01_JPRB, 0.21816E+01_JPRB, 0.19906E+01_JPRB, 0.16932E+01_JPRB ,&
&  0.15341E+01_JPRB, 0.14573E+01_JPRB, 0.15779E+01_JPRB, 0.21271E+01_JPRB, 0.27344E+01_JPRB, 0.26573E+01_JPRB, 0.30883E+01_JPRB ,&
&  0.33137E+01_JPRB, 0.33853E+01_JPRB, 0.30322E+01_JPRB, 0.29618E+01_JPRB, 0.27859E+01_JPRB, 0.26035E+01_JPRB, 0.23963E+01_JPRB ,&
&  0.22340E+01_JPRB, 0.21216E+01_JPRB, 0.20214E+01_JPRB, 0.19143E+01_JPRB, 0.18117E+01_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.17193E+01_JPRB, 0.16181E+01_JPRB, 0.14497E+01_JPRB, 0.12662E+01_JPRB, 0.10791E+01_JPRB, 0.92954E+00_JPRB, 0.90689E+00_JPRB ,&
&  0.93791E+00_JPRB, 0.93231E+00_JPRB, 0.90080E+00_JPRB, 0.84996E+00_JPRB, 0.82712E+00_JPRB, 0.82493E+00_JPRB, 0.82135E+00_JPRB ,&
&  0.80835E+00_JPRB, 0.89975E+00_JPRB, 0.73916E+00_JPRB, 0.61532E+00_JPRB, 0.59962E+00_JPRB, 0.68203E+00_JPRB, 0.87774E+00_JPRB ,&
&  0.10918E+01_JPRB, 0.14714E+01_JPRB, 0.18371E+01_JPRB, 0.21501E+01_JPRB, 0.25531E+01_JPRB, 0.28760E+01_JPRB, 0.30276E+01_JPRB ,&
&  0.32241E+01_JPRB, 0.27413E+01_JPRB, 0.27471E+01_JPRB, 0.25512E+01_JPRB, 0.23513E+01_JPRB, 0.21856E+01_JPRB, 0.20508E+01_JPRB ,&
&  0.18792E+01_JPRB, 0.17458E+01_JPRB, 0.15883E+01_JPRB, 0.14832E+01_JPRB, 0.14307E+01_JPRB, 0.13849E+01_JPRB, 0.13511E+01_JPRB ,&
&  0.13832E+01_JPRB, 0.14482E+01_JPRB, 0.15197E+01_JPRB, 0.15850E+01_JPRB, 0.17740E+01_JPRB, 0.20567E+01_JPRB, 0.22869E+01_JPRB ,&
&  0.22751E+01_JPRB, 0.23566E+01_JPRB, 0.31800E+01_JPRB, 0.35063E+01_JPRB, 0.40380E+01_JPRB, 0.46015E+01_JPRB, 0.50999E+01_JPRB ,&
&  0.55290E+01_JPRB, 0.54329E+01_JPRB, 0.53729E+01_JPRB, 0.48276E+01_JPRB, 0.54392E+01_JPRB, 0.53141E+01_JPRB, 0.49903E+01_JPRB ,&
&  0.45966E+01_JPRB, 0.45313E+01_JPRB, 0.41953E+01_JPRB, 0.39888E+01_JPRB, 0.39188E+01_JPRB, 0.40830E+01_JPRB, 0.36162E+01_JPRB ,&
&  0.29359E+01_JPRB, 0.32287E+01_JPRB, 0.28194E+01_JPRB, 0.23120E+01_JPRB, 0.17783E+01_JPRB, 0.15880E+01_JPRB, 0.14677E+01_JPRB ,&
&  0.13750E+01_JPRB, 0.13261E+01_JPRB, 0.13566E+01_JPRB, 0.15793E+01_JPRB, 0.19329E+01_JPRB, 0.21631E+01_JPRB, 0.22804E+01_JPRB ,&
&  0.22781E+01_JPRB, 0.26789E+01_JPRB, 0.26669E+01_JPRB, 0.25548E+01_JPRB, 0.25053E+01_JPRB, 0.24738E+01_JPRB, 0.23849E+01_JPRB ,&
&  0.22267E+01_JPRB, 0.21021E+01_JPRB, 0.19929E+01_JPRB, 0.18944E+01_JPRB, 0.17962E+01_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.16317E+01_JPRB, 0.15480E+01_JPRB, 0.14052E+01_JPRB, 0.12366E+01_JPRB, 0.10497E+01_JPRB, 0.91393E+00_JPRB, 0.88425E+00_JPRB ,&
&  0.86950E+00_JPRB, 0.84669E+00_JPRB, 0.80998E+00_JPRB, 0.78839E+00_JPRB, 0.78288E+00_JPRB, 0.78671E+00_JPRB, 0.81787E+00_JPRB ,&
&  0.86469E+00_JPRB, 0.12420E+01_JPRB, 0.10176E+01_JPRB, 0.85249E+00_JPRB, 0.76313E+00_JPRB, 0.90124E+00_JPRB, 0.10216E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.13756E+01_JPRB, 0.14514E+01_JPRB, 0.17130E+01_JPRB, 0.19433E+01_JPRB, 0.20215E+01_JPRB, 0.21696E+01_JPRB ,&
&  0.21742E+01_JPRB, 0.22043E+01_JPRB, 0.19860E+01_JPRB, 0.19630E+01_JPRB, 0.18589E+01_JPRB, 0.16769E+01_JPRB, 0.17184E+01_JPRB ,&
&  0.15850E+01_JPRB, 0.14797E+01_JPRB, 0.13738E+01_JPRB, 0.13279E+01_JPRB, 0.13348E+01_JPRB, 0.13331E+01_JPRB, 0.12857E+01_JPRB ,&
&  0.12930E+01_JPRB, 0.13351E+01_JPRB, 0.14163E+01_JPRB, 0.15106E+01_JPRB, 0.16808E+01_JPRB, 0.19732E+01_JPRB, 0.21873E+01_JPRB ,&
&  0.26367E+01_JPRB, 0.26022E+01_JPRB, 0.24070E+01_JPRB, 0.32132E+01_JPRB, 0.48180E+01_JPRB, 0.51656E+01_JPRB, 0.48807E+01_JPRB ,&
&  0.54363E+01_JPRB, 0.56677E+01_JPRB, 0.56549E+01_JPRB, 0.54243E+01_JPRB, 0.52464E+01_JPRB, 0.51246E+01_JPRB, 0.50038E+01_JPRB ,&
&  0.46177E+01_JPRB, 0.44274E+01_JPRB, 0.42747E+01_JPRB, 0.41828E+01_JPRB, 0.39253E+01_JPRB, 0.38267E+01_JPRB, 0.38185E+01_JPRB ,&
&  0.34527E+01_JPRB, 0.27866E+01_JPRB, 0.18328E+01_JPRB, 0.18197E+01_JPRB, 0.13192E+01_JPRB, 0.11387E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.12173E+01_JPRB, 0.12972E+01_JPRB, 0.13849E+01_JPRB, 0.13479E+01_JPRB, 0.14734E+01_JPRB, 0.15608E+01_JPRB, 0.17114E+01_JPRB ,&
&  0.18062E+01_JPRB, 0.17105E+01_JPRB, 0.20160E+01_JPRB, 0.20599E+01_JPRB, 0.20547E+01_JPRB, 0.20489E+01_JPRB, 0.20291E+01_JPRB ,&
&  0.19727E+01_JPRB, 0.18959E+01_JPRB, 0.18264E+01_JPRB, 0.17531E+01_JPRB, 0.16950E+01_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.14323E+01_JPRB, 0.13583E+01_JPRB, 0.12537E+01_JPRB, 0.11322E+01_JPRB, 0.99226E+00_JPRB, 0.88179E+00_JPRB, 0.82986E+00_JPRB ,&
&  0.81571E+00_JPRB, 0.78834E+00_JPRB, 0.75479E+00_JPRB, 0.74021E+00_JPRB, 0.73905E+00_JPRB, 0.75202E+00_JPRB, 0.81871E+00_JPRB ,&
&  0.98799E+00_JPRB, 0.13150E+01_JPRB, 0.11561E+01_JPRB, 0.98836E+00_JPRB, 0.11677E+01_JPRB, 0.11336E+01_JPRB, 0.11960E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.13940E+01_JPRB, 0.13223E+01_JPRB, 0.13295E+01_JPRB, 0.15030E+01_JPRB, 0.15885E+01_JPRB, 0.16220E+01_JPRB ,&
&  0.15883E+01_JPRB, 0.15056E+01_JPRB, 0.14123E+01_JPRB, 0.14091E+01_JPRB, 0.14762E+01_JPRB, 0.14001E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.13189E+01_JPRB, 0.12400E+01_JPRB, 0.11447E+01_JPRB, 0.11194E+01_JPRB, 0.11516E+01_JPRB, 0.12142E+01_JPRB, 0.12493E+01_JPRB ,&
&  0.13159E+01_JPRB, 0.13767E+01_JPRB, 0.14106E+01_JPRB, 0.15231E+01_JPRB, 0.17039E+01_JPRB, 0.20335E+01_JPRB, 0.24633E+01_JPRB ,&
&  0.30980E+01_JPRB, 0.33622E+01_JPRB, 0.36467E+01_JPRB, 0.45272E+01_JPRB, 0.51063E+01_JPRB, 0.51470E+01_JPRB, 0.51376E+01_JPRB ,&
&  0.50964E+01_JPRB, 0.50666E+01_JPRB, 0.51554E+01_JPRB, 0.49526E+01_JPRB, 0.48010E+01_JPRB, 0.48042E+01_JPRB, 0.47289E+01_JPRB ,&
&  0.43050E+01_JPRB, 0.41078E+01_JPRB, 0.41266E+01_JPRB, 0.39263E+01_JPRB, 0.37431E+01_JPRB, 0.35037E+01_JPRB, 0.35111E+01_JPRB ,&
&  0.34672E+01_JPRB, 0.20153E+01_JPRB, 0.15022E+01_JPRB, 0.16636E+01_JPRB, 0.12824E+01_JPRB, 0.11665E+01_JPRB, 0.13766E+01_JPRB ,&
&  0.13599E+01_JPRB, 0.13334E+01_JPRB, 0.13576E+01_JPRB, 0.12687E+01_JPRB, 0.11777E+01_JPRB, 0.12241E+01_JPRB, 0.12158E+01_JPRB ,&
&  0.12711E+01_JPRB, 0.12817E+01_JPRB, 0.15165E+01_JPRB, 0.16668E+01_JPRB, 0.17182E+01_JPRB, 0.17038E+01_JPRB, 0.16663E+01_JPRB ,&
&  0.16269E+01_JPRB, 0.15844E+01_JPRB, 0.15510E+01_JPRB, 0.15296E+01_JPRB, 0.14985E+01_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.12326E+01_JPRB, 0.11898E+01_JPRB, 0.11360E+01_JPRB, 0.10653E+01_JPRB, 0.96923E+00_JPRB, 0.88833E+00_JPRB, 0.81471E+00_JPRB ,&
&  0.80274E+00_JPRB, 0.77451E+00_JPRB, 0.73886E+00_JPRB, 0.72444E+00_JPRB, 0.72541E+00_JPRB, 0.73133E+00_JPRB, 0.75984E+00_JPRB ,&
&  0.70174E+00_JPRB, 0.89984E+00_JPRB, 0.85862E+00_JPRB, 0.10324E+01_JPRB, 0.14259E+01_JPRB, 0.11608E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.11792E+01_JPRB, 0.12565E+01_JPRB, 0.12330E+01_JPRB, 0.12093E+01_JPRB, 0.12589E+01_JPRB, 0.12858E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.10537E+01_JPRB, 0.10139E+01_JPRB, 0.10084E+01_JPRB, 0.10930E+01_JPRB, 0.11593E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.10561E+01_JPRB, 0.10056E+01_JPRB, 0.10172E+01_JPRB, 0.10534E+01_JPRB, 0.10821E+01_JPRB, 0.11406E+01_JPRB ,&
&  0.12370E+01_JPRB, 0.13380E+01_JPRB, 0.14541E+01_JPRB, 0.16097E+01_JPRB, 0.16548E+01_JPRB, 0.22083E+01_JPRB, 0.31531E+01_JPRB ,&
&  0.38784E+01_JPRB, 0.38406E+01_JPRB, 0.42140E+01_JPRB, 0.52101E+01_JPRB, 0.55220E+01_JPRB, 0.52740E+01_JPRB, 0.49231E+01_JPRB ,&
&  0.46262E+01_JPRB, 0.44264E+01_JPRB, 0.45814E+01_JPRB, 0.45980E+01_JPRB, 0.43335E+01_JPRB, 0.44340E+01_JPRB, 0.43420E+01_JPRB ,&
&  0.40826E+01_JPRB, 0.39535E+01_JPRB, 0.39923E+01_JPRB, 0.39685E+01_JPRB, 0.37532E+01_JPRB, 0.38025E+01_JPRB, 0.36652E+01_JPRB ,&
&  0.35032E+01_JPRB, 0.29734E+01_JPRB, 0.21334E+01_JPRB, 0.17238E+01_JPRB, 0.12663E+01_JPRB, 0.12608E+01_JPRB, 0.15625E+01_JPRB ,&
&  0.15025E+01_JPRB, 0.13699E+01_JPRB, 0.11906E+01_JPRB, 0.10524E+01_JPRB, 0.95902E+00_JPRB, 0.95407E+00_JPRB, 0.91432E+00_JPRB ,&
&  0.89194E+00_JPRB, 0.98819E+00_JPRB, 0.11271E+01_JPRB, 0.12702E+01_JPRB, 0.13755E+01_JPRB, 0.14086E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.13807E+01_JPRB, 0.13589E+01_JPRB, 0.13517E+01_JPRB, 0.13435E+01_JPRB, 0.13023E+01_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.10418E+01_JPRB, 0.10495E+01_JPRB, 0.10212E+01_JPRB, 0.99361E+00_JPRB, 0.92146E+00_JPRB, 0.87324E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.77826E+00_JPRB, 0.72974E+00_JPRB, 0.71150E+00_JPRB, 0.71998E+00_JPRB, 0.71695E+00_JPRB, 0.65913E+00_JPRB ,&
&  0.63023E+00_JPRB, 0.71183E+00_JPRB, 0.72144E+00_JPRB, 0.84658E+00_JPRB, 0.94768E+00_JPRB, 0.89091E+00_JPRB, 0.92282E+00_JPRB ,&
&  0.10362E+01_JPRB, 0.10617E+01_JPRB, 0.10792E+01_JPRB, 0.10821E+01_JPRB, 0.10880E+01_JPRB, 0.10946E+01_JPRB, 0.10788E+01_JPRB ,&
&  0.99818E+00_JPRB, 0.94500E+00_JPRB, 0.90584E+00_JPRB, 0.90291E+00_JPRB, 0.98993E+00_JPRB, 0.10319E+01_JPRB, 0.98217E+00_JPRB ,&
&  0.92236E+00_JPRB, 0.89388E+00_JPRB, 0.89568E+00_JPRB, 0.95794E+00_JPRB, 0.99260E+00_JPRB, 0.99701E+00_JPRB, 0.10533E+01_JPRB ,&
&  0.11585E+01_JPRB, 0.13014E+01_JPRB, 0.14530E+01_JPRB, 0.16346E+01_JPRB, 0.17354E+01_JPRB, 0.22278E+01_JPRB, 0.29513E+01_JPRB ,&
&  0.32146E+01_JPRB, 0.35025E+01_JPRB, 0.38848E+01_JPRB, 0.43152E+01_JPRB, 0.46021E+01_JPRB, 0.42912E+01_JPRB, 0.40105E+01_JPRB ,&
&  0.39729E+01_JPRB, 0.38784E+01_JPRB, 0.46046E+01_JPRB, 0.43907E+01_JPRB, 0.41443E+01_JPRB, 0.40383E+01_JPRB, 0.38732E+01_JPRB ,&
&  0.38176E+01_JPRB, 0.37555E+01_JPRB, 0.37101E+01_JPRB, 0.36295E+01_JPRB, 0.35337E+01_JPRB, 0.34932E+01_JPRB, 0.32901E+01_JPRB ,&
&  0.31424E+01_JPRB, 0.28677E+01_JPRB, 0.23372E+01_JPRB, 0.19080E+01_JPRB, 0.15219E+01_JPRB, 0.15331E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.11734E+01_JPRB, 0.95884E+00_JPRB, 0.91543E+00_JPRB, 0.86215E+00_JPRB, 0.84835E+00_JPRB, 0.81454E+00_JPRB, 0.77524E+00_JPRB ,&
&  0.81117E+00_JPRB, 0.90472E+00_JPRB, 0.92670E+00_JPRB, 0.99208E+00_JPRB, 0.10844E+01_JPRB, 0.11688E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.10070E+01_JPRB, 0.11019E+01_JPRB, 0.11149E+01_JPRB, 0.11025E+01_JPRB, 0.10686E+01_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.95163E+00_JPRB, 0.96731E+00_JPRB, 0.94347E+00_JPRB, 0.94092E+00_JPRB, 0.88701E+00_JPRB, 0.85530E+00_JPRB, 0.76937E+00_JPRB ,&
&  0.79217E+00_JPRB, 0.76641E+00_JPRB, 0.71028E+00_JPRB, 0.69532E+00_JPRB, 0.71029E+00_JPRB, 0.63440E+00_JPRB, 0.56646E+00_JPRB ,&
&  0.54678E+00_JPRB, 0.57985E+00_JPRB, 0.67181E+00_JPRB, 0.71848E+00_JPRB, 0.73051E+00_JPRB, 0.74037E+00_JPRB, 0.73543E+00_JPRB ,&
&  0.82362E+00_JPRB, 0.84890E+00_JPRB, 0.92429E+00_JPRB, 0.96604E+00_JPRB, 0.95000E+00_JPRB, 0.95798E+00_JPRB, 0.98087E+00_JPRB ,&
&  0.91651E+00_JPRB, 0.89482E+00_JPRB, 0.89790E+00_JPRB, 0.82732E+00_JPRB, 0.90511E+00_JPRB, 0.86219E+00_JPRB, 0.80089E+00_JPRB ,&
&  0.78593E+00_JPRB, 0.81633E+00_JPRB, 0.85113E+00_JPRB, 0.88543E+00_JPRB, 0.89755E+00_JPRB, 0.92113E+00_JPRB, 0.97406E+00_JPRB ,&
&  0.10666E+01_JPRB, 0.12271E+01_JPRB, 0.14156E+01_JPRB, 0.16711E+01_JPRB, 0.19380E+01_JPRB, 0.21697E+01_JPRB, 0.26879E+01_JPRB ,&
&  0.29218E+01_JPRB, 0.32361E+01_JPRB, 0.36276E+01_JPRB, 0.34066E+01_JPRB, 0.38065E+01_JPRB, 0.38179E+01_JPRB, 0.35346E+01_JPRB ,&
&  0.35526E+01_JPRB, 0.32303E+01_JPRB, 0.31245E+01_JPRB, 0.31539E+01_JPRB, 0.33355E+01_JPRB, 0.32678E+01_JPRB, 0.32381E+01_JPRB ,&
&  0.33090E+01_JPRB, 0.35342E+01_JPRB, 0.33119E+01_JPRB, 0.32265E+01_JPRB, 0.29983E+01_JPRB, 0.27867E+01_JPRB, 0.25360E+01_JPRB ,&
&  0.23532E+01_JPRB, 0.21395E+01_JPRB, 0.19411E+01_JPRB, 0.17831E+01_JPRB, 0.15797E+01_JPRB, 0.14107E+01_JPRB, 0.12673E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.95994E+00_JPRB, 0.89693E+00_JPRB, 0.85309E+00_JPRB, 0.83003E+00_JPRB, 0.77510E+00_JPRB, 0.77626E+00_JPRB ,&
&  0.82469E+00_JPRB, 0.87073E+00_JPRB, 0.93365E+00_JPRB, 0.94082E+00_JPRB, 0.97546E+00_JPRB, 0.10253E+01_JPRB, 0.95285E+00_JPRB ,&
&  0.88209E+00_JPRB, 0.95901E+00_JPRB, 0.96565E+00_JPRB, 0.96065E+00_JPRB, 0.95372E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.92175E+00_JPRB, 0.92402E+00_JPRB, 0.90909E+00_JPRB, 0.90357E+00_JPRB, 0.87113E+00_JPRB, 0.79604E+00_JPRB, 0.72318E+00_JPRB ,&
&  0.77901E+00_JPRB, 0.71263E+00_JPRB, 0.72239E+00_JPRB, 0.61494E+00_JPRB, 0.58092E+00_JPRB, 0.57366E+00_JPRB, 0.56134E+00_JPRB ,&
&  0.54981E+00_JPRB, 0.58887E+00_JPRB, 0.63584E+00_JPRB, 0.65646E+00_JPRB, 0.68138E+00_JPRB, 0.69066E+00_JPRB, 0.68123E+00_JPRB ,&
&  0.69924E+00_JPRB, 0.73695E+00_JPRB, 0.80963E+00_JPRB, 0.82819E+00_JPRB, 0.79825E+00_JPRB, 0.79937E+00_JPRB, 0.81129E+00_JPRB ,&
&  0.78217E+00_JPRB, 0.76879E+00_JPRB, 0.79810E+00_JPRB, 0.79538E+00_JPRB, 0.77566E+00_JPRB, 0.72078E+00_JPRB, 0.68904E+00_JPRB ,&
&  0.70139E+00_JPRB, 0.67409E+00_JPRB, 0.73861E+00_JPRB, 0.76724E+00_JPRB, 0.79996E+00_JPRB, 0.83741E+00_JPRB, 0.87959E+00_JPRB ,&
&  0.95267E+00_JPRB, 0.10757E+01_JPRB, 0.12402E+01_JPRB, 0.14399E+01_JPRB, 0.16864E+01_JPRB, 0.20835E+01_JPRB, 0.23888E+01_JPRB ,&
&  0.27426E+01_JPRB, 0.21280E+01_JPRB, 0.26692E+01_JPRB, 0.28840E+01_JPRB, 0.32857E+01_JPRB, 0.32914E+01_JPRB, 0.31964E+01_JPRB ,&
&  0.31209E+01_JPRB, 0.27101E+01_JPRB, 0.24583E+01_JPRB, 0.23323E+01_JPRB, 0.23884E+01_JPRB, 0.26100E+01_JPRB, 0.27877E+01_JPRB ,&
&  0.29440E+01_JPRB, 0.27798E+01_JPRB, 0.25702E+01_JPRB, 0.23488E+01_JPRB, 0.21692E+01_JPRB, 0.20404E+01_JPRB, 0.19059E+01_JPRB ,&
&  0.17831E+01_JPRB, 0.16490E+01_JPRB, 0.15153E+01_JPRB, 0.13397E+01_JPRB, 0.12512E+01_JPRB, 0.11731E+01_JPRB, 0.10883E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.97776E+00_JPRB, 0.90234E+00_JPRB, 0.86907E+00_JPRB, 0.82881E+00_JPRB, 0.82000E+00_JPRB, 0.85078E+00_JPRB ,&
&  0.86012E+00_JPRB, 0.81845E+00_JPRB, 0.85800E+00_JPRB, 0.87482E+00_JPRB, 0.87693E+00_JPRB, 0.88431E+00_JPRB, 0.93907E+00_JPRB ,&
&  0.88495E+00_JPRB, 0.89006E+00_JPRB, 0.93938E+00_JPRB, 0.92762E+00_JPRB, 0.92296E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.91179E+00_JPRB, 0.90533E+00_JPRB, 0.89094E+00_JPRB, 0.86874E+00_JPRB, 0.83707E+00_JPRB, 0.78901E+00_JPRB, 0.69585E+00_JPRB ,&
&  0.68559E+00_JPRB, 0.64217E+00_JPRB, 0.60082E+00_JPRB, 0.57699E+00_JPRB, 0.57380E+00_JPRB, 0.56484E+00_JPRB, 0.56236E+00_JPRB ,&
&  0.56785E+00_JPRB, 0.59321E+00_JPRB, 0.62694E+00_JPRB, 0.63956E+00_JPRB, 0.65281E+00_JPRB, 0.65884E+00_JPRB, 0.65111E+00_JPRB ,&
&  0.66203E+00_JPRB, 0.68778E+00_JPRB, 0.71953E+00_JPRB, 0.74707E+00_JPRB, 0.73371E+00_JPRB, 0.72241E+00_JPRB, 0.71384E+00_JPRB ,&
&  0.70693E+00_JPRB, 0.70027E+00_JPRB, 0.69258E+00_JPRB, 0.70196E+00_JPRB, 0.68013E+00_JPRB, 0.65731E+00_JPRB, 0.65741E+00_JPRB ,&
&  0.54708E+00_JPRB, 0.55233E+00_JPRB, 0.64869E+00_JPRB, 0.68300E+00_JPRB, 0.71668E+00_JPRB, 0.75350E+00_JPRB, 0.81355E+00_JPRB ,&
&  0.87671E+00_JPRB, 0.95472E+00_JPRB, 0.10554E+01_JPRB, 0.11839E+01_JPRB, 0.13754E+01_JPRB, 0.16620E+01_JPRB, 0.19475E+01_JPRB ,&
&  0.22327E+01_JPRB, 0.18658E+01_JPRB, 0.18622E+01_JPRB, 0.21613E+01_JPRB, 0.27304E+01_JPRB, 0.25031E+01_JPRB, 0.23260E+01_JPRB ,&
&  0.21583E+01_JPRB, 0.20233E+01_JPRB, 0.19821E+01_JPRB, 0.21514E+01_JPRB, 0.22304E+01_JPRB, 0.23942E+01_JPRB, 0.24962E+01_JPRB ,&
&  0.22516E+01_JPRB, 0.20665E+01_JPRB, 0.18703E+01_JPRB, 0.17655E+01_JPRB, 0.16952E+01_JPRB, 0.16473E+01_JPRB, 0.16052E+01_JPRB ,&
&  0.15727E+01_JPRB, 0.15085E+01_JPRB, 0.13696E+01_JPRB, 0.12125E+01_JPRB, 0.10983E+01_JPRB, 0.10326E+01_JPRB, 0.99311E+00_JPRB ,&
&  0.95777E+00_JPRB, 0.92491E+00_JPRB, 0.89877E+00_JPRB, 0.88329E+00_JPRB, 0.87844E+00_JPRB, 0.88789E+00_JPRB, 0.93649E+00_JPRB ,&
&  0.87810E+00_JPRB, 0.83516E+00_JPRB, 0.77924E+00_JPRB, 0.78195E+00_JPRB, 0.81499E+00_JPRB, 0.85031E+00_JPRB, 0.86194E+00_JPRB ,&
&  0.89917E+00_JPRB, 0.89343E+00_JPRB, 0.86352E+00_JPRB, 0.88131E+00_JPRB, 0.89289E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.89669E+00_JPRB, 0.88658E+00_JPRB, 0.86290E+00_JPRB, 0.84780E+00_JPRB, 0.80308E+00_JPRB, 0.77520E+00_JPRB, 0.73247E+00_JPRB ,&
&  0.71817E+00_JPRB, 0.69150E+00_JPRB, 0.65263E+00_JPRB, 0.60916E+00_JPRB, 0.58705E+00_JPRB, 0.57895E+00_JPRB, 0.58405E+00_JPRB ,&
&  0.59837E+00_JPRB, 0.61172E+00_JPRB, 0.62351E+00_JPRB, 0.63107E+00_JPRB, 0.64478E+00_JPRB, 0.65244E+00_JPRB, 0.65122E+00_JPRB ,&
&  0.65725E+00_JPRB, 0.67007E+00_JPRB, 0.67632E+00_JPRB, 0.67301E+00_JPRB, 0.67265E+00_JPRB, 0.67634E+00_JPRB, 0.67216E+00_JPRB ,&
&  0.65265E+00_JPRB, 0.63186E+00_JPRB, 0.63305E+00_JPRB, 0.66193E+00_JPRB, 0.64224E+00_JPRB, 0.64349E+00_JPRB, 0.57959E+00_JPRB ,&
&  0.50759E+00_JPRB, 0.48856E+00_JPRB, 0.52335E+00_JPRB, 0.63327E+00_JPRB, 0.66608E+00_JPRB, 0.70328E+00_JPRB, 0.73589E+00_JPRB ,&
&  0.73620E+00_JPRB, 0.77490E+00_JPRB, 0.87645E+00_JPRB, 0.10791E+01_JPRB, 0.12044E+01_JPRB, 0.14042E+01_JPRB, 0.16126E+01_JPRB ,&
&  0.17820E+01_JPRB, 0.19060E+01_JPRB, 0.20158E+01_JPRB, 0.18608E+01_JPRB, 0.18813E+01_JPRB, 0.21527E+01_JPRB, 0.19577E+01_JPRB ,&
&  0.19069E+01_JPRB, 0.18328E+01_JPRB, 0.18876E+01_JPRB, 0.20225E+01_JPRB, 0.19309E+01_JPRB, 0.20022E+01_JPRB, 0.19706E+01_JPRB ,&
&  0.17946E+01_JPRB, 0.16749E+01_JPRB, 0.15754E+01_JPRB, 0.15256E+01_JPRB, 0.14823E+01_JPRB, 0.14657E+01_JPRB, 0.14813E+01_JPRB ,&
&  0.15566E+01_JPRB, 0.15588E+01_JPRB, 0.13811E+01_JPRB, 0.11147E+01_JPRB, 0.10106E+01_JPRB, 0.96832E+00_JPRB, 0.94440E+00_JPRB ,&
&  0.91120E+00_JPRB, 0.88887E+00_JPRB, 0.88626E+00_JPRB, 0.88611E+00_JPRB, 0.86584E+00_JPRB, 0.82996E+00_JPRB, 0.79460E+00_JPRB ,&
&  0.76597E+00_JPRB, 0.75186E+00_JPRB, 0.73135E+00_JPRB, 0.75862E+00_JPRB, 0.79317E+00_JPRB, 0.82242E+00_JPRB, 0.81374E+00_JPRB ,&
&  0.80261E+00_JPRB, 0.81124E+00_JPRB, 0.82527E+00_JPRB, 0.86461E+00_JPRB, 0.88289E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.78533E+00_JPRB, 0.80967E+00_JPRB, 0.83698E+00_JPRB, 0.82722E+00_JPRB, 0.81383E+00_JPRB, 0.76356E+00_JPRB, 0.71358E+00_JPRB ,&
&  0.68393E+00_JPRB, 0.67265E+00_JPRB, 0.66905E+00_JPRB, 0.66123E+00_JPRB, 0.65417E+00_JPRB, 0.65086E+00_JPRB, 0.64833E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.64732E+00_JPRB, 0.64744E+00_JPRB, 0.65087E+00_JPRB, 0.66458E+00_JPRB, 0.67355E+00_JPRB, 0.67552E+00_JPRB ,&
&  0.67791E+00_JPRB, 0.68106E+00_JPRB, 0.66915E+00_JPRB, 0.64726E+00_JPRB, 0.63669E+00_JPRB, 0.63791E+00_JPRB, 0.64531E+00_JPRB ,&
&  0.63536E+00_JPRB, 0.61789E+00_JPRB, 0.62271E+00_JPRB, 0.64055E+00_JPRB, 0.63395E+00_JPRB, 0.63521E+00_JPRB, 0.58115E+00_JPRB ,&
&  0.50716E+00_JPRB, 0.45669E+00_JPRB, 0.43730E+00_JPRB, 0.46777E+00_JPRB, 0.53513E+00_JPRB, 0.64089E+00_JPRB, 0.68826E+00_JPRB ,&
&  0.72715E+00_JPRB, 0.78683E+00_JPRB, 0.85495E+00_JPRB, 0.94167E+00_JPRB, 0.10556E+01_JPRB, 0.12124E+01_JPRB, 0.13820E+01_JPRB ,&
&  0.15219E+01_JPRB, 0.16047E+01_JPRB, 0.16454E+01_JPRB, 0.14458E+01_JPRB, 0.14085E+01_JPRB, 0.14509E+01_JPRB, 0.16547E+01_JPRB ,&
&  0.18417E+01_JPRB, 0.16916E+01_JPRB, 0.15734E+01_JPRB, 0.15796E+01_JPRB, 0.15501E+01_JPRB, 0.15679E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.15503E+01_JPRB, 0.14819E+01_JPRB, 0.13989E+01_JPRB, 0.13361E+01_JPRB, 0.13199E+01_JPRB, 0.13424E+01_JPRB, 0.13643E+01_JPRB ,&
&  0.14480E+01_JPRB, 0.16053E+01_JPRB, 0.14887E+01_JPRB, 0.11082E+01_JPRB, 0.10436E+01_JPRB, 0.10267E+01_JPRB, 0.99909E+00_JPRB ,&
&  0.93784E+00_JPRB, 0.89687E+00_JPRB, 0.88226E+00_JPRB, 0.87697E+00_JPRB, 0.84783E+00_JPRB, 0.82717E+00_JPRB, 0.81432E+00_JPRB ,&
&  0.81034E+00_JPRB, 0.81464E+00_JPRB, 0.81990E+00_JPRB, 0.84841E+00_JPRB, 0.85537E+00_JPRB, 0.85112E+00_JPRB, 0.82845E+00_JPRB ,&
&  0.80953E+00_JPRB, 0.80040E+00_JPRB, 0.79296E+00_JPRB, 0.78408E+00_JPRB, 0.78292E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.80663E+00_JPRB, 0.80008E+00_JPRB, 0.79960E+00_JPRB, 0.80135E+00_JPRB, 0.80596E+00_JPRB, 0.78903E+00_JPRB, 0.77182E+00_JPRB ,&
&  0.76670E+00_JPRB, 0.76820E+00_JPRB, 0.75568E+00_JPRB, 0.74471E+00_JPRB, 0.72715E+00_JPRB, 0.71240E+00_JPRB, 0.70814E+00_JPRB ,&
&  0.70805E+00_JPRB, 0.70427E+00_JPRB, 0.69928E+00_JPRB, 0.69271E+00_JPRB, 0.69959E+00_JPRB, 0.70624E+00_JPRB, 0.69976E+00_JPRB ,&
&  0.69532E+00_JPRB, 0.69083E+00_JPRB, 0.67831E+00_JPRB, 0.66061E+00_JPRB, 0.63578E+00_JPRB, 0.62373E+00_JPRB, 0.61516E+00_JPRB ,&
&  0.60789E+00_JPRB, 0.60927E+00_JPRB, 0.62693E+00_JPRB, 0.62390E+00_JPRB, 0.62484E+00_JPRB, 0.62249E+00_JPRB, 0.55884E+00_JPRB ,&
&  0.49015E+00_JPRB, 0.44331E+00_JPRB, 0.41717E+00_JPRB, 0.41540E+00_JPRB, 0.43910E+00_JPRB, 0.49335E+00_JPRB, 0.56348E+00_JPRB ,&
&  0.64644E+00_JPRB, 0.70391E+00_JPRB, 0.75442E+00_JPRB, 0.80021E+00_JPRB, 0.84945E+00_JPRB, 0.92776E+00_JPRB, 0.10145E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.12559E+01_JPRB, 0.13758E+01_JPRB, 0.14890E+01_JPRB, 0.15736E+01_JPRB, 0.15064E+01_JPRB, 0.16268E+01_JPRB ,&
&  0.20596E+01_JPRB, 0.16928E+01_JPRB, 0.13594E+01_JPRB, 0.12212E+01_JPRB, 0.11875E+01_JPRB, 0.11879E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11920E+01_JPRB, 0.11814E+01_JPRB, 0.11707E+01_JPRB, 0.11719E+01_JPRB, 0.11733E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.15229E+01_JPRB, 0.18767E+01_JPRB, 0.13837E+01_JPRB, 0.12246E+01_JPRB, 0.11133E+01_JPRB, 0.97998E+00_JPRB ,&
&  0.92221E+00_JPRB, 0.90027E+00_JPRB, 0.89843E+00_JPRB, 0.89861E+00_JPRB, 0.88783E+00_JPRB, 0.88544E+00_JPRB, 0.88686E+00_JPRB ,&
&  0.89729E+00_JPRB, 0.90513E+00_JPRB, 0.90342E+00_JPRB, 0.89788E+00_JPRB, 0.88219E+00_JPRB, 0.85833E+00_JPRB, 0.83401E+00_JPRB ,&
&  0.83060E+00_JPRB, 0.83149E+00_JPRB, 0.83099E+00_JPRB, 0.82811E+00_JPRB, 0.82014E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.79777E+00_JPRB, 0.78565E+00_JPRB, 0.78293E+00_JPRB, 0.78133E+00_JPRB, 0.78540E+00_JPRB, 0.79107E+00_JPRB, 0.79221E+00_JPRB ,&
&  0.80175E+00_JPRB, 0.80114E+00_JPRB, 0.78832E+00_JPRB, 0.77409E+00_JPRB, 0.75667E+00_JPRB, 0.74350E+00_JPRB, 0.73721E+00_JPRB ,&
&  0.73647E+00_JPRB, 0.72657E+00_JPRB, 0.72161E+00_JPRB, 0.72048E+00_JPRB, 0.72586E+00_JPRB, 0.73560E+00_JPRB, 0.72904E+00_JPRB ,&
&  0.70654E+00_JPRB, 0.69094E+00_JPRB, 0.67209E+00_JPRB, 0.64596E+00_JPRB, 0.61926E+00_JPRB, 0.59986E+00_JPRB, 0.60181E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.60532E+00_JPRB, 0.60487E+00_JPRB, 0.60482E+00_JPRB, 0.60157E+00_JPRB, 0.59133E+00_JPRB, 0.49677E+00_JPRB ,&
&  0.45813E+00_JPRB, 0.42915E+00_JPRB, 0.41728E+00_JPRB, 0.41496E+00_JPRB, 0.43313E+00_JPRB, 0.46954E+00_JPRB, 0.53547E+00_JPRB ,&
&  0.59686E+00_JPRB, 0.65648E+00_JPRB, 0.69368E+00_JPRB, 0.72785E+00_JPRB, 0.77170E+00_JPRB, 0.82565E+00_JPRB, 0.87410E+00_JPRB ,&
&  0.94114E+00_JPRB, 0.10277E+01_JPRB, 0.11175E+01_JPRB, 0.12034E+01_JPRB, 0.12592E+01_JPRB, 0.12879E+01_JPRB, 0.13063E+01_JPRB ,&
&  0.13039E+01_JPRB, 0.11802E+01_JPRB, 0.11299E+01_JPRB, 0.10955E+01_JPRB, 0.10723E+01_JPRB, 0.10692E+01_JPRB, 0.10361E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10319E+01_JPRB, 0.10352E+01_JPRB, 0.10360E+01_JPRB, 0.10399E+01_JPRB, 0.10399E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10757E+01_JPRB, 0.11256E+01_JPRB, 0.10865E+01_JPRB, 0.10321E+01_JPRB, 0.97326E+00_JPRB, 0.93463E+00_JPRB ,&
&  0.92276E+00_JPRB, 0.92217E+00_JPRB, 0.92550E+00_JPRB, 0.91852E+00_JPRB, 0.91414E+00_JPRB, 0.91232E+00_JPRB, 0.92425E+00_JPRB ,&
&  0.92359E+00_JPRB, 0.92114E+00_JPRB, 0.91043E+00_JPRB, 0.89385E+00_JPRB, 0.87377E+00_JPRB, 0.85238E+00_JPRB, 0.83593E+00_JPRB ,&
&  0.82697E+00_JPRB, 0.82178E+00_JPRB, 0.81482E+00_JPRB, 0.81150E+00_JPRB, 0.80672E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.77927E+00_JPRB, 0.77112E+00_JPRB, 0.76968E+00_JPRB, 0.77054E+00_JPRB, 0.77567E+00_JPRB, 0.78333E+00_JPRB, 0.78725E+00_JPRB ,&
&  0.79057E+00_JPRB, 0.79305E+00_JPRB, 0.79189E+00_JPRB, 0.78130E+00_JPRB, 0.77214E+00_JPRB, 0.76701E+00_JPRB, 0.76209E+00_JPRB ,&
&  0.75348E+00_JPRB, 0.74493E+00_JPRB, 0.73967E+00_JPRB, 0.73796E+00_JPRB, 0.73618E+00_JPRB, 0.74222E+00_JPRB, 0.73791E+00_JPRB ,&
&  0.71784E+00_JPRB, 0.70046E+00_JPRB, 0.66902E+00_JPRB, 0.64212E+00_JPRB, 0.62304E+00_JPRB, 0.60829E+00_JPRB, 0.60982E+00_JPRB ,&
&  0.60389E+00_JPRB, 0.59969E+00_JPRB, 0.59761E+00_JPRB, 0.59749E+00_JPRB, 0.60027E+00_JPRB, 0.49616E+00_JPRB, 0.46563E+00_JPRB ,&
&  0.44336E+00_JPRB, 0.42725E+00_JPRB, 0.42200E+00_JPRB, 0.42121E+00_JPRB, 0.42978E+00_JPRB, 0.46026E+00_JPRB, 0.50776E+00_JPRB ,&
&  0.54173E+00_JPRB, 0.64682E+00_JPRB, 0.66911E+00_JPRB, 0.69306E+00_JPRB, 0.73083E+00_JPRB, 0.76660E+00_JPRB, 0.78801E+00_JPRB ,&
&  0.81707E+00_JPRB, 0.85937E+00_JPRB, 0.91150E+00_JPRB, 0.96573E+00_JPRB, 0.98849E+00_JPRB, 0.10237E+01_JPRB, 0.10427E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10474E+01_JPRB, 0.10412E+01_JPRB, 0.10377E+01_JPRB, 0.10264E+01_JPRB, 0.10235E+01_JPRB, 0.10084E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.98934E+00_JPRB, 0.99874E+00_JPRB, 0.10012E+01_JPRB, 0.10022E+01_JPRB, 0.10030E+01_JPRB, 0.99681E+00_JPRB ,&
&  0.98913E+00_JPRB, 0.97754E+00_JPRB, 0.95601E+00_JPRB, 0.92698E+00_JPRB, 0.91071E+00_JPRB, 0.89166E+00_JPRB, 0.88929E+00_JPRB ,&
&  0.89219E+00_JPRB, 0.90826E+00_JPRB, 0.90915E+00_JPRB, 0.90760E+00_JPRB, 0.90683E+00_JPRB, 0.90136E+00_JPRB, 0.90041E+00_JPRB ,&
&  0.89774E+00_JPRB, 0.89556E+00_JPRB, 0.88668E+00_JPRB, 0.87194E+00_JPRB, 0.85474E+00_JPRB, 0.83671E+00_JPRB, 0.82112E+00_JPRB ,&
&  0.81061E+00_JPRB, 0.80484E+00_JPRB, 0.79383E+00_JPRB, 0.78861E+00_JPRB, 0.78335E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.76213E+00_JPRB, 0.75644E+00_JPRB, 0.75430E+00_JPRB, 0.75654E+00_JPRB, 0.76577E+00_JPRB, 0.77668E+00_JPRB, 0.77784E+00_JPRB ,&
&  0.77837E+00_JPRB, 0.77912E+00_JPRB, 0.78146E+00_JPRB, 0.78041E+00_JPRB, 0.77884E+00_JPRB, 0.77372E+00_JPRB, 0.76953E+00_JPRB ,&
&  0.76540E+00_JPRB, 0.75644E+00_JPRB, 0.74528E+00_JPRB, 0.73840E+00_JPRB, 0.73544E+00_JPRB, 0.73646E+00_JPRB, 0.73524E+00_JPRB ,&
&  0.73006E+00_JPRB, 0.71560E+00_JPRB, 0.67872E+00_JPRB, 0.65308E+00_JPRB, 0.62653E+00_JPRB, 0.61768E+00_JPRB, 0.60506E+00_JPRB ,&
&  0.61834E+00_JPRB, 0.55898E+00_JPRB, 0.55292E+00_JPRB, 0.52491E+00_JPRB, 0.50202E+00_JPRB, 0.47774E+00_JPRB, 0.45769E+00_JPRB ,&
&  0.45014E+00_JPRB, 0.44240E+00_JPRB, 0.44047E+00_JPRB, 0.44797E+00_JPRB, 0.46069E+00_JPRB, 0.48812E+00_JPRB, 0.52002E+00_JPRB ,&
&  0.54780E+00_JPRB, 0.62960E+00_JPRB, 0.65312E+00_JPRB, 0.67046E+00_JPRB, 0.70086E+00_JPRB, 0.72614E+00_JPRB, 0.74289E+00_JPRB ,&
&  0.75861E+00_JPRB, 0.77309E+00_JPRB, 0.80002E+00_JPRB, 0.81721E+00_JPRB, 0.84196E+00_JPRB, 0.85303E+00_JPRB, 0.86157E+00_JPRB ,&
&  0.87351E+00_JPRB, 0.88701E+00_JPRB, 0.89611E+00_JPRB, 0.90488E+00_JPRB, 0.90949E+00_JPRB, 0.91078E+00_JPRB, 0.90935E+00_JPRB ,&
&  0.90570E+00_JPRB, 0.90137E+00_JPRB, 0.89628E+00_JPRB, 0.88923E+00_JPRB, 0.88072E+00_JPRB, 0.87569E+00_JPRB, 0.86821E+00_JPRB ,&
&  0.86240E+00_JPRB, 0.85298E+00_JPRB, 0.84383E+00_JPRB, 0.83613E+00_JPRB, 0.83701E+00_JPRB, 0.83715E+00_JPRB, 0.84525E+00_JPRB ,&
&  0.85812E+00_JPRB, 0.86592E+00_JPRB, 0.87055E+00_JPRB, 0.87423E+00_JPRB, 0.87697E+00_JPRB, 0.87455E+00_JPRB, 0.87011E+00_JPRB ,&
&  0.86616E+00_JPRB, 0.86061E+00_JPRB, 0.85327E+00_JPRB, 0.84022E+00_JPRB, 0.82693E+00_JPRB, 0.81342E+00_JPRB, 0.80265E+00_JPRB ,&
&  0.79009E+00_JPRB, 0.78262E+00_JPRB, 0.77077E+00_JPRB, 0.76744E+00_JPRB, 0.76355E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.75098E+00_JPRB, 0.74876E+00_JPRB, 0.74649E+00_JPRB, 0.74699E+00_JPRB, 0.75011E+00_JPRB, 0.75552E+00_JPRB, 0.75721E+00_JPRB ,&
&  0.76203E+00_JPRB, 0.76669E+00_JPRB, 0.77189E+00_JPRB, 0.77353E+00_JPRB, 0.77502E+00_JPRB, 0.77291E+00_JPRB, 0.76898E+00_JPRB ,&
&  0.76496E+00_JPRB, 0.75928E+00_JPRB, 0.75546E+00_JPRB, 0.75140E+00_JPRB, 0.74617E+00_JPRB, 0.74253E+00_JPRB, 0.73732E+00_JPRB ,&
&  0.73380E+00_JPRB, 0.72948E+00_JPRB, 0.67764E+00_JPRB, 0.65864E+00_JPRB, 0.63253E+00_JPRB, 0.61781E+00_JPRB, 0.61170E+00_JPRB ,&
&  0.60313E+00_JPRB, 0.59517E+00_JPRB, 0.58382E+00_JPRB, 0.56670E+00_JPRB, 0.54503E+00_JPRB, 0.51607E+00_JPRB, 0.49975E+00_JPRB ,&
&  0.48634E+00_JPRB, 0.48243E+00_JPRB, 0.48429E+00_JPRB, 0.49345E+00_JPRB, 0.51072E+00_JPRB, 0.53156E+00_JPRB, 0.55588E+00_JPRB ,&
&  0.58892E+00_JPRB, 0.62484E+00_JPRB, 0.64734E+00_JPRB, 0.65362E+00_JPRB, 0.68173E+00_JPRB, 0.70084E+00_JPRB, 0.71590E+00_JPRB ,&
&  0.72679E+00_JPRB, 0.73505E+00_JPRB, 0.75658E+00_JPRB, 0.76986E+00_JPRB, 0.77824E+00_JPRB, 0.78323E+00_JPRB, 0.79051E+00_JPRB ,&
&  0.79837E+00_JPRB, 0.80618E+00_JPRB, 0.81067E+00_JPRB, 0.81444E+00_JPRB, 0.81605E+00_JPRB, 0.81566E+00_JPRB, 0.81479E+00_JPRB ,&
&  0.81377E+00_JPRB, 0.81178E+00_JPRB, 0.80850E+00_JPRB, 0.80320E+00_JPRB, 0.79808E+00_JPRB, 0.79319E+00_JPRB, 0.78769E+00_JPRB ,&
&  0.78542E+00_JPRB, 0.78404E+00_JPRB, 0.78414E+00_JPRB, 0.77980E+00_JPRB, 0.78339E+00_JPRB, 0.79603E+00_JPRB, 0.80323E+00_JPRB ,&
&  0.81213E+00_JPRB, 0.81875E+00_JPRB, 0.82491E+00_JPRB, 0.83001E+00_JPRB, 0.83296E+00_JPRB, 0.83331E+00_JPRB, 0.83074E+00_JPRB ,&
&  0.82856E+00_JPRB, 0.82413E+00_JPRB, 0.81833E+00_JPRB, 0.80804E+00_JPRB, 0.79802E+00_JPRB, 0.78757E+00_JPRB, 0.77911E+00_JPRB ,&
&  0.76878E+00_JPRB, 0.76205E+00_JPRB, 0.75338E+00_JPRB, 0.75183E+00_JPRB, 0.75046E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.74639E+00_JPRB, 0.74658E+00_JPRB, 0.74637E+00_JPRB, 0.74676E+00_JPRB, 0.74916E+00_JPRB, 0.75164E+00_JPRB, 0.75282E+00_JPRB ,&
&  0.75506E+00_JPRB, 0.75818E+00_JPRB, 0.76224E+00_JPRB, 0.76409E+00_JPRB, 0.76544E+00_JPRB, 0.76532E+00_JPRB, 0.76421E+00_JPRB ,&
&  0.76206E+00_JPRB, 0.75951E+00_JPRB, 0.75726E+00_JPRB, 0.75551E+00_JPRB, 0.75363E+00_JPRB, 0.75202E+00_JPRB, 0.74787E+00_JPRB ,&
&  0.74377E+00_JPRB, 0.74156E+00_JPRB, 0.73484E+00_JPRB, 0.72532E+00_JPRB, 0.71778E+00_JPRB, 0.66136E+00_JPRB, 0.64288E+00_JPRB ,&
&  0.63859E+00_JPRB, 0.63440E+00_JPRB, 0.63664E+00_JPRB, 0.64062E+00_JPRB, 0.65397E+00_JPRB, 0.64624E+00_JPRB, 0.64052E+00_JPRB ,&
&  0.60619E+00_JPRB, 0.60373E+00_JPRB, 0.59873E+00_JPRB, 0.59735E+00_JPRB, 0.60301E+00_JPRB, 0.60876E+00_JPRB, 0.62287E+00_JPRB ,&
&  0.65847E+00_JPRB, 0.67049E+00_JPRB, 0.67529E+00_JPRB, 0.68024E+00_JPRB, 0.69015E+00_JPRB, 0.69856E+00_JPRB, 0.70536E+00_JPRB ,&
&  0.71066E+00_JPRB, 0.71537E+00_JPRB, 0.72111E+00_JPRB, 0.72751E+00_JPRB, 0.73221E+00_JPRB, 0.73383E+00_JPRB, 0.73590E+00_JPRB ,&
&  0.73789E+00_JPRB, 0.74127E+00_JPRB, 0.74516E+00_JPRB, 0.74895E+00_JPRB, 0.75313E+00_JPRB, 0.75522E+00_JPRB, 0.75680E+00_JPRB ,&
&  0.75834E+00_JPRB, 0.75903E+00_JPRB, 0.75839E+00_JPRB, 0.75851E+00_JPRB, 0.75946E+00_JPRB, 0.75956E+00_JPRB, 0.76011E+00_JPRB ,&
&  0.76094E+00_JPRB, 0.76282E+00_JPRB, 0.76455E+00_JPRB, 0.76922E+00_JPRB, 0.77356E+00_JPRB, 0.77975E+00_JPRB, 0.78537E+00_JPRB ,&
&  0.79061E+00_JPRB, 0.79455E+00_JPRB, 0.79707E+00_JPRB, 0.79825E+00_JPRB, 0.79839E+00_JPRB, 0.79809E+00_JPRB, 0.79657E+00_JPRB ,&
&  0.79460E+00_JPRB, 0.79080E+00_JPRB, 0.78542E+00_JPRB, 0.77848E+00_JPRB, 0.77181E+00_JPRB, 0.76465E+00_JPRB, 0.75822E+00_JPRB ,&
&  0.75164E+00_JPRB, 0.74787E+00_JPRB, 0.74582E+00_JPRB, 0.74584E+00_JPRB, 0.74628E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.75154E+00_JPRB, 0.75216E+00_JPRB, 0.75170E+00_JPRB, 0.75139E+00_JPRB, 0.75091E+00_JPRB, 0.75169E+00_JPRB, 0.75235E+00_JPRB ,&
&  0.75310E+00_JPRB, 0.75434E+00_JPRB, 0.75660E+00_JPRB, 0.75760E+00_JPRB, 0.75828E+00_JPRB, 0.75831E+00_JPRB, 0.75845E+00_JPRB ,&
&  0.75808E+00_JPRB, 0.75717E+00_JPRB, 0.75612E+00_JPRB, 0.75548E+00_JPRB, 0.75484E+00_JPRB, 0.75439E+00_JPRB, 0.75371E+00_JPRB ,&
&  0.75222E+00_JPRB, 0.75041E+00_JPRB, 0.74832E+00_JPRB, 0.74456E+00_JPRB, 0.74006E+00_JPRB, 0.73498E+00_JPRB, 0.72935E+00_JPRB ,&
&  0.72331E+00_JPRB, 0.71868E+00_JPRB, 0.71417E+00_JPRB, 0.71004E+00_JPRB, 0.70867E+00_JPRB, 0.70806E+00_JPRB, 0.70740E+00_JPRB ,&
&  0.70691E+00_JPRB, 0.70545E+00_JPRB, 0.70564E+00_JPRB, 0.70627E+00_JPRB, 0.70734E+00_JPRB, 0.70937E+00_JPRB, 0.71102E+00_JPRB ,&
&  0.71211E+00_JPRB, 0.71183E+00_JPRB, 0.71065E+00_JPRB, 0.71159E+00_JPRB, 0.71416E+00_JPRB, 0.71551E+00_JPRB, 0.71494E+00_JPRB ,&
&  0.71457E+00_JPRB, 0.71324E+00_JPRB, 0.71175E+00_JPRB, 0.71016E+00_JPRB, 0.71027E+00_JPRB, 0.70964E+00_JPRB, 0.70876E+00_JPRB ,&
&  0.70863E+00_JPRB, 0.70939E+00_JPRB, 0.71066E+00_JPRB, 0.71226E+00_JPRB, 0.71477E+00_JPRB, 0.71685E+00_JPRB, 0.71912E+00_JPRB ,&
&  0.72171E+00_JPRB, 0.72588E+00_JPRB, 0.72959E+00_JPRB, 0.73340E+00_JPRB, 0.73780E+00_JPRB, 0.74269E+00_JPRB, 0.74724E+00_JPRB ,&
&  0.75146E+00_JPRB, 0.75508E+00_JPRB, 0.75714E+00_JPRB, 0.75973E+00_JPRB, 0.76247E+00_JPRB, 0.76599E+00_JPRB, 0.76842E+00_JPRB ,&
&  0.77085E+00_JPRB, 0.77254E+00_JPRB, 0.77341E+00_JPRB, 0.77386E+00_JPRB, 0.77330E+00_JPRB, 0.77224E+00_JPRB, 0.77063E+00_JPRB ,&
&  0.76853E+00_JPRB, 0.76634E+00_JPRB, 0.76418E+00_JPRB, 0.76179E+00_JPRB, 0.75939E+00_JPRB, 0.75697E+00_JPRB, 0.75462E+00_JPRB ,&
&  0.75287E+00_JPRB, 0.75228E+00_JPRB, 0.75239E+00_JPRB, 0.75204E+00_JPRB, 0.75174E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.75123E+00_JPRB, 0.75077E+00_JPRB, 0.75028E+00_JPRB, 0.74967E+00_JPRB, 0.74902E+00_JPRB, 0.74872E+00_JPRB, 0.74841E+00_JPRB ,&
&  0.74817E+00_JPRB, 0.74789E+00_JPRB, 0.74819E+00_JPRB, 0.74811E+00_JPRB, 0.74817E+00_JPRB, 0.74804E+00_JPRB, 0.74808E+00_JPRB ,&
&  0.74800E+00_JPRB, 0.74801E+00_JPRB, 0.74800E+00_JPRB, 0.74824E+00_JPRB, 0.74843E+00_JPRB, 0.74887E+00_JPRB, 0.74912E+00_JPRB ,&
&  0.74925E+00_JPRB, 0.74883E+00_JPRB, 0.74869E+00_JPRB, 0.74798E+00_JPRB, 0.74716E+00_JPRB, 0.74563E+00_JPRB, 0.74392E+00_JPRB ,&
&  0.74172E+00_JPRB, 0.73984E+00_JPRB, 0.73815E+00_JPRB, 0.73682E+00_JPRB, 0.73569E+00_JPRB, 0.73505E+00_JPRB, 0.73442E+00_JPRB ,&
&  0.73363E+00_JPRB, 0.73292E+00_JPRB, 0.73236E+00_JPRB, 0.73165E+00_JPRB, 0.73049E+00_JPRB, 0.72907E+00_JPRB, 0.72767E+00_JPRB ,&
&  0.72607E+00_JPRB, 0.72432E+00_JPRB, 0.72308E+00_JPRB, 0.72237E+00_JPRB, 0.72143E+00_JPRB, 0.71973E+00_JPRB, 0.71783E+00_JPRB ,&
&  0.71609E+00_JPRB, 0.71399E+00_JPRB, 0.71178E+00_JPRB, 0.70974E+00_JPRB, 0.70843E+00_JPRB, 0.70732E+00_JPRB, 0.70640E+00_JPRB ,&
&  0.70654E+00_JPRB, 0.70791E+00_JPRB, 0.70933E+00_JPRB, 0.71059E+00_JPRB, 0.71235E+00_JPRB, 0.71419E+00_JPRB, 0.71625E+00_JPRB ,&
&  0.71836E+00_JPRB, 0.72070E+00_JPRB, 0.72349E+00_JPRB, 0.72601E+00_JPRB, 0.72895E+00_JPRB, 0.73207E+00_JPRB, 0.73553E+00_JPRB ,&
&  0.73881E+00_JPRB, 0.74239E+00_JPRB, 0.74548E+00_JPRB, 0.74846E+00_JPRB, 0.75024E+00_JPRB, 0.75190E+00_JPRB, 0.75257E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.75378E+00_JPRB, 0.75460E+00_JPRB, 0.75485E+00_JPRB, 0.75508E+00_JPRB, 0.75509E+00_JPRB, 0.75521E+00_JPRB ,&
&  0.75512E+00_JPRB, 0.75514E+00_JPRB, 0.75495E+00_JPRB, 0.75482E+00_JPRB, 0.75426E+00_JPRB, 0.75406E+00_JPRB, 0.75370E+00_JPRB ,&
&  0.75343E+00_JPRB, 0.75319E+00_JPRB, 0.75304E+00_JPRB, 0.75272E+00_JPRB, 0.75213E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB ,&
&  0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB, 0.73664E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB, 0.21642E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.22024E+00_JPRB, 0.22053E+00_JPRB, 0.22058E+00_JPRB, 0.22051E+00_JPRB, 0.21879E+00_JPRB, 0.21812E+00_JPRB, 0.21750E+00_JPRB ,&
&  0.21705E+00_JPRB, 0.21682E+00_JPRB, 0.21723E+00_JPRB, 0.21776E+00_JPRB, 0.21829E+00_JPRB, 0.21883E+00_JPRB, 0.21928E+00_JPRB ,&
&  0.21949E+00_JPRB, 0.21972E+00_JPRB, 0.21870E+00_JPRB, 0.21821E+00_JPRB, 0.21807E+00_JPRB, 0.21814E+00_JPRB, 0.21813E+00_JPRB ,&
&  0.21825E+00_JPRB, 0.21834E+00_JPRB, 0.21834E+00_JPRB, 0.21868E+00_JPRB, 0.21926E+00_JPRB, 0.21975E+00_JPRB, 0.22026E+00_JPRB ,&
&  0.22076E+00_JPRB, 0.22146E+00_JPRB, 0.22210E+00_JPRB, 0.22270E+00_JPRB, 0.22347E+00_JPRB, 0.22421E+00_JPRB, 0.22463E+00_JPRB ,&
&  0.22494E+00_JPRB, 0.22501E+00_JPRB, 0.22515E+00_JPRB, 0.22526E+00_JPRB, 0.22548E+00_JPRB, 0.22576E+00_JPRB, 0.22580E+00_JPRB ,&
&  0.22572E+00_JPRB, 0.22556E+00_JPRB, 0.22532E+00_JPRB, 0.22478E+00_JPRB, 0.22421E+00_JPRB, 0.22366E+00_JPRB, 0.22311E+00_JPRB ,&
&  0.22264E+00_JPRB, 0.22211E+00_JPRB, 0.22152E+00_JPRB, 0.22090E+00_JPRB, 0.22025E+00_JPRB, 0.21961E+00_JPRB, 0.21897E+00_JPRB ,&
&  0.21832E+00_JPRB, 0.21790E+00_JPRB, 0.21743E+00_JPRB, 0.21703E+00_JPRB, 0.21672E+00_JPRB, 0.21641E+00_JPRB, 0.21611E+00_JPRB ,&
&  0.21581E+00_JPRB, 0.21545E+00_JPRB, 0.21512E+00_JPRB, 0.21475E+00_JPRB, 0.21442E+00_JPRB, 0.21405E+00_JPRB, 0.21376E+00_JPRB ,&
&  0.21347E+00_JPRB, 0.21322E+00_JPRB, 0.21301E+00_JPRB, 0.21291E+00_JPRB, 0.21285E+00_JPRB, 0.21288E+00_JPRB, 0.21291E+00_JPRB ,&
&  0.21296E+00_JPRB, 0.21300E+00_JPRB, 0.21302E+00_JPRB, 0.21289E+00_JPRB, 0.21308E+00_JPRB, 0.21329E+00_JPRB, 0.21358E+00_JPRB ,&
&  0.21385E+00_JPRB, 0.21431E+00_JPRB, 0.21476E+00_JPRB, 0.21527E+00_JPRB, 0.21606E+00_JPRB, 0.21634E+00_JPRB, 0.21659E+00_JPRB ,&
&  0.21687E+00_JPRB, 0.21745E+00_JPRB, 0.21795E+00_JPRB, 0.21821E+00_JPRB, 0.21878E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.23213E+00_JPRB, 0.23718E+00_JPRB, 0.24095E+00_JPRB, 0.24441E+00_JPRB, 0.24213E+00_JPRB, 0.26180E+00_JPRB, 0.25983E+00_JPRB ,&
&  0.25971E+00_JPRB, 0.25349E+00_JPRB, 0.25468E+00_JPRB, 0.25344E+00_JPRB, 0.25112E+00_JPRB, 0.24812E+00_JPRB, 0.24493E+00_JPRB ,&
&  0.24163E+00_JPRB, 0.23771E+00_JPRB, 0.23409E+00_JPRB, 0.23062E+00_JPRB, 0.22776E+00_JPRB, 0.22526E+00_JPRB, 0.22388E+00_JPRB ,&
&  0.22393E+00_JPRB, 0.22542E+00_JPRB, 0.22798E+00_JPRB, 0.22940E+00_JPRB, 0.23093E+00_JPRB, 0.23254E+00_JPRB, 0.23425E+00_JPRB ,&
&  0.23569E+00_JPRB, 0.23760E+00_JPRB, 0.23861E+00_JPRB, 0.23826E+00_JPRB, 0.23720E+00_JPRB, 0.23673E+00_JPRB, 0.23657E+00_JPRB ,&
&  0.23705E+00_JPRB, 0.23727E+00_JPRB, 0.23736E+00_JPRB, 0.23760E+00_JPRB, 0.23782E+00_JPRB, 0.23731E+00_JPRB, 0.23612E+00_JPRB ,&
&  0.23511E+00_JPRB, 0.23430E+00_JPRB, 0.23269E+00_JPRB, 0.23116E+00_JPRB, 0.22972E+00_JPRB, 0.22822E+00_JPRB, 0.22666E+00_JPRB ,&
&  0.22553E+00_JPRB, 0.22440E+00_JPRB, 0.22298E+00_JPRB, 0.22135E+00_JPRB, 0.21981E+00_JPRB, 0.21845E+00_JPRB, 0.21737E+00_JPRB ,&
&  0.21713E+00_JPRB, 0.21654E+00_JPRB, 0.21603E+00_JPRB, 0.21553E+00_JPRB, 0.21539E+00_JPRB, 0.21495E+00_JPRB, 0.21464E+00_JPRB ,&
&  0.21441E+00_JPRB, 0.21411E+00_JPRB, 0.21379E+00_JPRB, 0.21347E+00_JPRB, 0.21326E+00_JPRB, 0.21296E+00_JPRB, 0.21268E+00_JPRB ,&
&  0.21249E+00_JPRB, 0.21235E+00_JPRB, 0.21218E+00_JPRB, 0.21180E+00_JPRB, 0.21142E+00_JPRB, 0.21095E+00_JPRB, 0.21094E+00_JPRB ,&
&  0.21108E+00_JPRB, 0.21135E+00_JPRB, 0.21184E+00_JPRB, 0.21202E+00_JPRB, 0.21284E+00_JPRB, 0.21364E+00_JPRB, 0.21458E+00_JPRB ,&
&  0.21557E+00_JPRB, 0.21710E+00_JPRB, 0.21879E+00_JPRB, 0.22061E+00_JPRB, 0.22351E+00_JPRB, 0.22453E+00_JPRB, 0.22540E+00_JPRB ,&
&  0.22646E+00_JPRB, 0.23000E+00_JPRB, 0.23109E+00_JPRB, 0.23107E+00_JPRB, 0.23182E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.27520E+00_JPRB, 0.27869E+00_JPRB, 0.28105E+00_JPRB, 0.28261E+00_JPRB, 0.28413E+00_JPRB, 0.28346E+00_JPRB, 0.28194E+00_JPRB ,&
&  0.27781E+00_JPRB, 0.27637E+00_JPRB, 0.27363E+00_JPRB, 0.27071E+00_JPRB, 0.26730E+00_JPRB, 0.26344E+00_JPRB, 0.25941E+00_JPRB ,&
&  0.25493E+00_JPRB, 0.25006E+00_JPRB, 0.24426E+00_JPRB, 0.23794E+00_JPRB, 0.23216E+00_JPRB, 0.22676E+00_JPRB, 0.22358E+00_JPRB ,&
&  0.22212E+00_JPRB, 0.22290E+00_JPRB, 0.22574E+00_JPRB, 0.22925E+00_JPRB, 0.23353E+00_JPRB, 0.23836E+00_JPRB, 0.24438E+00_JPRB ,&
&  0.24954E+00_JPRB, 0.25573E+00_JPRB, 0.25949E+00_JPRB, 0.26110E+00_JPRB, 0.25971E+00_JPRB, 0.25961E+00_JPRB, 0.25864E+00_JPRB ,&
&  0.25735E+00_JPRB, 0.25531E+00_JPRB, 0.25447E+00_JPRB, 0.25453E+00_JPRB, 0.25365E+00_JPRB, 0.25173E+00_JPRB, 0.24908E+00_JPRB ,&
&  0.24662E+00_JPRB, 0.24451E+00_JPRB, 0.24186E+00_JPRB, 0.23961E+00_JPRB, 0.23791E+00_JPRB, 0.23581E+00_JPRB, 0.23302E+00_JPRB ,&
&  0.23052E+00_JPRB, 0.22802E+00_JPRB, 0.22506E+00_JPRB, 0.22255E+00_JPRB, 0.22006E+00_JPRB, 0.21824E+00_JPRB, 0.21661E+00_JPRB ,&
&  0.21591E+00_JPRB, 0.21517E+00_JPRB, 0.21495E+00_JPRB, 0.21483E+00_JPRB, 0.21524E+00_JPRB, 0.21496E+00_JPRB, 0.21455E+00_JPRB ,&
&  0.21405E+00_JPRB, 0.21392E+00_JPRB, 0.21340E+00_JPRB, 0.21321E+00_JPRB, 0.21302E+00_JPRB, 0.21236E+00_JPRB, 0.21203E+00_JPRB ,&
&  0.21211E+00_JPRB, 0.21202E+00_JPRB, 0.21148E+00_JPRB, 0.21143E+00_JPRB, 0.21108E+00_JPRB, 0.21058E+00_JPRB, 0.21085E+00_JPRB ,&
&  0.21066E+00_JPRB, 0.21086E+00_JPRB, 0.21118E+00_JPRB, 0.21168E+00_JPRB, 0.21276E+00_JPRB, 0.21415E+00_JPRB, 0.21602E+00_JPRB ,&
&  0.21888E+00_JPRB, 0.22149E+00_JPRB, 0.22411E+00_JPRB, 0.22717E+00_JPRB, 0.23292E+00_JPRB, 0.23584E+00_JPRB, 0.24092E+00_JPRB ,&
&  0.24568E+00_JPRB, 0.24464E+00_JPRB, 0.26745E+00_JPRB, 0.26855E+00_JPRB, 0.27063E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.28983E+00_JPRB, 0.29191E+00_JPRB, 0.29292E+00_JPRB, 0.29492E+00_JPRB, 0.29699E+00_JPRB, 0.29774E+00_JPRB, 0.29824E+00_JPRB ,&
&  0.29573E+00_JPRB, 0.28939E+00_JPRB, 0.28460E+00_JPRB, 0.27815E+00_JPRB, 0.27184E+00_JPRB, 0.26524E+00_JPRB, 0.25926E+00_JPRB ,&
&  0.25307E+00_JPRB, 0.24746E+00_JPRB, 0.24179E+00_JPRB, 0.23702E+00_JPRB, 0.23367E+00_JPRB, 0.23091E+00_JPRB, 0.22793E+00_JPRB ,&
&  0.22537E+00_JPRB, 0.22641E+00_JPRB, 0.22896E+00_JPRB, 0.23387E+00_JPRB, 0.24159E+00_JPRB, 0.24876E+00_JPRB, 0.25660E+00_JPRB ,&
&  0.27036E+00_JPRB, 0.27405E+00_JPRB, 0.27395E+00_JPRB, 0.27592E+00_JPRB, 0.27532E+00_JPRB, 0.27584E+00_JPRB, 0.26588E+00_JPRB ,&
&  0.26806E+00_JPRB, 0.27016E+00_JPRB, 0.27769E+00_JPRB, 0.28086E+00_JPRB, 0.27573E+00_JPRB, 0.27135E+00_JPRB, 0.26345E+00_JPRB ,&
&  0.25695E+00_JPRB, 0.25185E+00_JPRB, 0.24847E+00_JPRB, 0.24645E+00_JPRB, 0.24191E+00_JPRB, 0.23855E+00_JPRB, 0.23294E+00_JPRB ,&
&  0.22825E+00_JPRB, 0.22513E+00_JPRB, 0.22121E+00_JPRB, 0.21876E+00_JPRB, 0.21676E+00_JPRB, 0.21484E+00_JPRB, 0.21338E+00_JPRB ,&
&  0.21257E+00_JPRB, 0.21191E+00_JPRB, 0.21258E+00_JPRB, 0.21335E+00_JPRB, 0.21484E+00_JPRB, 0.21603E+00_JPRB, 0.21630E+00_JPRB ,&
&  0.21657E+00_JPRB, 0.21699E+00_JPRB, 0.21713E+00_JPRB, 0.21773E+00_JPRB, 0.21722E+00_JPRB, 0.21636E+00_JPRB, 0.21529E+00_JPRB ,&
&  0.21413E+00_JPRB, 0.21369E+00_JPRB, 0.21291E+00_JPRB, 0.21242E+00_JPRB, 0.21074E+00_JPRB, 0.20958E+00_JPRB, 0.20893E+00_JPRB ,&
&  0.20833E+00_JPRB, 0.20873E+00_JPRB, 0.20885E+00_JPRB, 0.20967E+00_JPRB, 0.21096E+00_JPRB, 0.21215E+00_JPRB, 0.21422E+00_JPRB ,&
&  0.21685E+00_JPRB, 0.21990E+00_JPRB, 0.22507E+00_JPRB, 0.23056E+00_JPRB, 0.23661E+00_JPRB, 0.24435E+00_JPRB, 0.25155E+00_JPRB ,&
&  0.26160E+00_JPRB, 0.27696E+00_JPRB, 0.27908E+00_JPRB, 0.28251E+00_JPRB, 0.28708E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.30003E+00_JPRB, 0.30121E+00_JPRB, 0.30251E+00_JPRB, 0.30481E+00_JPRB, 0.30764E+00_JPRB, 0.30868E+00_JPRB, 0.30894E+00_JPRB ,&
&  0.29977E+00_JPRB, 0.29219E+00_JPRB, 0.28542E+00_JPRB, 0.27514E+00_JPRB, 0.26699E+00_JPRB, 0.25967E+00_JPRB, 0.25438E+00_JPRB ,&
&  0.24959E+00_JPRB, 0.24710E+00_JPRB, 0.24444E+00_JPRB, 0.24362E+00_JPRB, 0.24352E+00_JPRB, 0.24421E+00_JPRB, 0.24280E+00_JPRB ,&
&  0.24114E+00_JPRB, 0.24223E+00_JPRB, 0.24292E+00_JPRB, 0.24738E+00_JPRB, 0.25392E+00_JPRB, 0.26134E+00_JPRB, 0.26858E+00_JPRB ,&
&  0.27781E+00_JPRB, 0.28180E+00_JPRB, 0.28491E+00_JPRB, 0.28566E+00_JPRB, 0.28430E+00_JPRB, 0.28366E+00_JPRB, 0.28157E+00_JPRB ,&
&  0.28115E+00_JPRB, 0.28068E+00_JPRB, 0.28195E+00_JPRB, 0.28301E+00_JPRB, 0.28163E+00_JPRB, 0.27914E+00_JPRB, 0.27356E+00_JPRB ,&
&  0.26541E+00_JPRB, 0.25880E+00_JPRB, 0.25262E+00_JPRB, 0.24780E+00_JPRB, 0.24115E+00_JPRB, 0.23530E+00_JPRB, 0.22953E+00_JPRB ,&
&  0.22449E+00_JPRB, 0.22068E+00_JPRB, 0.21616E+00_JPRB, 0.21272E+00_JPRB, 0.21112E+00_JPRB, 0.20921E+00_JPRB, 0.20838E+00_JPRB ,&
&  0.20833E+00_JPRB, 0.20870E+00_JPRB, 0.21076E+00_JPRB, 0.21246E+00_JPRB, 0.21513E+00_JPRB, 0.21822E+00_JPRB, 0.22012E+00_JPRB ,&
&  0.22341E+00_JPRB, 0.22624E+00_JPRB, 0.22778E+00_JPRB, 0.22890E+00_JPRB, 0.22796E+00_JPRB, 0.22514E+00_JPRB, 0.22224E+00_JPRB ,&
&  0.21847E+00_JPRB, 0.21579E+00_JPRB, 0.21405E+00_JPRB, 0.21216E+00_JPRB, 0.21119E+00_JPRB, 0.21034E+00_JPRB, 0.20944E+00_JPRB ,&
&  0.20874E+00_JPRB, 0.20843E+00_JPRB, 0.20811E+00_JPRB, 0.20912E+00_JPRB, 0.21023E+00_JPRB, 0.21225E+00_JPRB, 0.21471E+00_JPRB ,&
&  0.21701E+00_JPRB, 0.22051E+00_JPRB, 0.22434E+00_JPRB, 0.22956E+00_JPRB, 0.23888E+00_JPRB, 0.24665E+00_JPRB, 0.26352E+00_JPRB ,&
&  0.27654E+00_JPRB, 0.30298E+00_JPRB, 0.31371E+00_JPRB, 0.30177E+00_JPRB, 0.30035E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.31532E+00_JPRB, 0.31266E+00_JPRB, 0.31008E+00_JPRB, 0.30976E+00_JPRB, 0.31041E+00_JPRB, 0.31325E+00_JPRB, 0.31094E+00_JPRB ,&
&  0.30645E+00_JPRB, 0.30481E+00_JPRB, 0.30619E+00_JPRB, 0.30629E+00_JPRB, 0.30566E+00_JPRB, 0.29528E+00_JPRB, 0.28941E+00_JPRB ,&
&  0.28261E+00_JPRB, 0.28028E+00_JPRB, 0.27843E+00_JPRB, 0.27920E+00_JPRB, 0.28147E+00_JPRB, 0.28290E+00_JPRB, 0.27880E+00_JPRB ,&
&  0.27441E+00_JPRB, 0.27020E+00_JPRB, 0.26586E+00_JPRB, 0.26402E+00_JPRB, 0.26535E+00_JPRB, 0.26678E+00_JPRB, 0.26701E+00_JPRB ,&
&  0.26773E+00_JPRB, 0.27052E+00_JPRB, 0.27308E+00_JPRB, 0.27661E+00_JPRB, 0.28933E+00_JPRB, 0.28756E+00_JPRB, 0.28626E+00_JPRB ,&
&  0.28658E+00_JPRB, 0.28700E+00_JPRB, 0.28805E+00_JPRB, 0.28869E+00_JPRB, 0.29267E+00_JPRB, 0.29565E+00_JPRB, 0.29774E+00_JPRB ,&
&  0.28406E+00_JPRB, 0.27613E+00_JPRB, 0.26837E+00_JPRB, 0.25930E+00_JPRB, 0.24874E+00_JPRB, 0.24017E+00_JPRB, 0.23229E+00_JPRB ,&
&  0.22559E+00_JPRB, 0.21924E+00_JPRB, 0.21353E+00_JPRB, 0.21129E+00_JPRB, 0.21049E+00_JPRB, 0.20928E+00_JPRB, 0.20919E+00_JPRB ,&
&  0.20974E+00_JPRB, 0.21049E+00_JPRB, 0.21098E+00_JPRB, 0.21249E+00_JPRB, 0.21484E+00_JPRB, 0.21852E+00_JPRB, 0.22245E+00_JPRB ,&
&  0.22834E+00_JPRB, 0.23271E+00_JPRB, 0.23783E+00_JPRB, 0.24210E+00_JPRB, 0.24064E+00_JPRB, 0.23564E+00_JPRB, 0.23028E+00_JPRB ,&
&  0.22383E+00_JPRB, 0.22009E+00_JPRB, 0.21631E+00_JPRB, 0.21356E+00_JPRB, 0.21199E+00_JPRB, 0.21107E+00_JPRB, 0.21135E+00_JPRB ,&
&  0.21146E+00_JPRB, 0.21192E+00_JPRB, 0.21147E+00_JPRB, 0.21173E+00_JPRB, 0.21256E+00_JPRB, 0.21439E+00_JPRB, 0.21678E+00_JPRB ,&
&  0.22047E+00_JPRB, 0.22476E+00_JPRB, 0.22986E+00_JPRB, 0.23545E+00_JPRB, 0.24354E+00_JPRB, 0.25426E+00_JPRB, 0.26961E+00_JPRB ,&
&  0.28642E+00_JPRB, 0.31795E+00_JPRB, 0.33056E+00_JPRB, 0.32163E+00_JPRB, 0.31871E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.33333E+00_JPRB, 0.33009E+00_JPRB, 0.32817E+00_JPRB, 0.32847E+00_JPRB, 0.32863E+00_JPRB, 0.33126E+00_JPRB, 0.33060E+00_JPRB ,&
&  0.32629E+00_JPRB, 0.32169E+00_JPRB, 0.32027E+00_JPRB, 0.32011E+00_JPRB, 0.31899E+00_JPRB, 0.31773E+00_JPRB, 0.31837E+00_JPRB ,&
&  0.31682E+00_JPRB, 0.31858E+00_JPRB, 0.32133E+00_JPRB, 0.32276E+00_JPRB, 0.32349E+00_JPRB, 0.32372E+00_JPRB, 0.32391E+00_JPRB ,&
&  0.31785E+00_JPRB, 0.31615E+00_JPRB, 0.32174E+00_JPRB, 0.31908E+00_JPRB, 0.31525E+00_JPRB, 0.31196E+00_JPRB, 0.31087E+00_JPRB ,&
&  0.30259E+00_JPRB, 0.29104E+00_JPRB, 0.27828E+00_JPRB, 0.28363E+00_JPRB, 0.30051E+00_JPRB, 0.29819E+00_JPRB, 0.29592E+00_JPRB ,&
&  0.29563E+00_JPRB, 0.29497E+00_JPRB, 0.29581E+00_JPRB, 0.29632E+00_JPRB, 0.29724E+00_JPRB, 0.29735E+00_JPRB, 0.29850E+00_JPRB ,&
&  0.29990E+00_JPRB, 0.30567E+00_JPRB, 0.30927E+00_JPRB, 0.29024E+00_JPRB, 0.27367E+00_JPRB, 0.26102E+00_JPRB, 0.24894E+00_JPRB ,&
&  0.23818E+00_JPRB, 0.23548E+00_JPRB, 0.23198E+00_JPRB, 0.22589E+00_JPRB, 0.22638E+00_JPRB, 0.22698E+00_JPRB, 0.22620E+00_JPRB ,&
&  0.22426E+00_JPRB, 0.22307E+00_JPRB, 0.22167E+00_JPRB, 0.21933E+00_JPRB, 0.21844E+00_JPRB, 0.21967E+00_JPRB, 0.22316E+00_JPRB ,&
&  0.22867E+00_JPRB, 0.23491E+00_JPRB, 0.24911E+00_JPRB, 0.26128E+00_JPRB, 0.25779E+00_JPRB, 0.25022E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.23256E+00_JPRB, 0.22739E+00_JPRB, 0.22230E+00_JPRB, 0.21818E+00_JPRB, 0.21631E+00_JPRB, 0.21653E+00_JPRB, 0.21737E+00_JPRB ,&
&  0.21860E+00_JPRB, 0.22058E+00_JPRB, 0.22137E+00_JPRB, 0.22098E+00_JPRB, 0.22022E+00_JPRB, 0.22072E+00_JPRB, 0.22233E+00_JPRB ,&
&  0.22530E+00_JPRB, 0.23045E+00_JPRB, 0.23673E+00_JPRB, 0.24447E+00_JPRB, 0.25115E+00_JPRB, 0.25737E+00_JPRB, 0.26696E+00_JPRB ,&
&  0.27491E+00_JPRB, 0.29302E+00_JPRB, 0.31896E+00_JPRB, 0.34462E+00_JPRB, 0.33965E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.33759E+00_JPRB, 0.33422E+00_JPRB, 0.33437E+00_JPRB, 0.33472E+00_JPRB, 0.33364E+00_JPRB, 0.33359E+00_JPRB, 0.33297E+00_JPRB ,&
&  0.33280E+00_JPRB, 0.33095E+00_JPRB, 0.32901E+00_JPRB, 0.32932E+00_JPRB, 0.33132E+00_JPRB, 0.33359E+00_JPRB, 0.33411E+00_JPRB ,&
&  0.33382E+00_JPRB, 0.33504E+00_JPRB, 0.33826E+00_JPRB, 0.34174E+00_JPRB, 0.34247E+00_JPRB, 0.34279E+00_JPRB, 0.34462E+00_JPRB ,&
&  0.34696E+00_JPRB, 0.34779E+00_JPRB, 0.35054E+00_JPRB, 0.34840E+00_JPRB, 0.34402E+00_JPRB, 0.34227E+00_JPRB, 0.34072E+00_JPRB ,&
&  0.33452E+00_JPRB, 0.31137E+00_JPRB, 0.29618E+00_JPRB, 0.30039E+00_JPRB, 0.31625E+00_JPRB, 0.31580E+00_JPRB, 0.31369E+00_JPRB ,&
&  0.31151E+00_JPRB, 0.30863E+00_JPRB, 0.30615E+00_JPRB, 0.30521E+00_JPRB, 0.30525E+00_JPRB, 0.30512E+00_JPRB, 0.30361E+00_JPRB ,&
&  0.30391E+00_JPRB, 0.30847E+00_JPRB, 0.31450E+00_JPRB, 0.31214E+00_JPRB, 0.30818E+00_JPRB, 0.30152E+00_JPRB, 0.29725E+00_JPRB ,&
&  0.29323E+00_JPRB, 0.28895E+00_JPRB, 0.28637E+00_JPRB, 0.28550E+00_JPRB, 0.28519E+00_JPRB, 0.28319E+00_JPRB, 0.27980E+00_JPRB ,&
&  0.27228E+00_JPRB, 0.26328E+00_JPRB, 0.25390E+00_JPRB, 0.24468E+00_JPRB, 0.23785E+00_JPRB, 0.23309E+00_JPRB, 0.22945E+00_JPRB ,&
&  0.23104E+00_JPRB, 0.24201E+00_JPRB, 0.26040E+00_JPRB, 0.28012E+00_JPRB, 0.30002E+00_JPRB, 0.27775E+00_JPRB, 0.26698E+00_JPRB ,&
&  0.25582E+00_JPRB, 0.24619E+00_JPRB, 0.23950E+00_JPRB, 0.23490E+00_JPRB, 0.23159E+00_JPRB, 0.22972E+00_JPRB, 0.22962E+00_JPRB ,&
&  0.23247E+00_JPRB, 0.23644E+00_JPRB, 0.24004E+00_JPRB, 0.24089E+00_JPRB, 0.24018E+00_JPRB, 0.23843E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.23861E+00_JPRB, 0.24393E+00_JPRB, 0.25285E+00_JPRB, 0.26573E+00_JPRB, 0.27745E+00_JPRB, 0.28959E+00_JPRB, 0.30380E+00_JPRB ,&
&  0.31902E+00_JPRB, 0.33344E+00_JPRB, 0.33712E+00_JPRB, 0.34028E+00_JPRB, 0.34039E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.34789E+00_JPRB, 0.34563E+00_JPRB, 0.34584E+00_JPRB, 0.34684E+00_JPRB, 0.34656E+00_JPRB, 0.34597E+00_JPRB, 0.34353E+00_JPRB ,&
&  0.34134E+00_JPRB, 0.34177E+00_JPRB, 0.34311E+00_JPRB, 0.34557E+00_JPRB, 0.34987E+00_JPRB, 0.35444E+00_JPRB, 0.35378E+00_JPRB ,&
&  0.35060E+00_JPRB, 0.34913E+00_JPRB, 0.35116E+00_JPRB, 0.35326E+00_JPRB, 0.35365E+00_JPRB, 0.35433E+00_JPRB, 0.35798E+00_JPRB ,&
&  0.36261E+00_JPRB, 0.36470E+00_JPRB, 0.36763E+00_JPRB, 0.36726E+00_JPRB, 0.36365E+00_JPRB, 0.36391E+00_JPRB, 0.36214E+00_JPRB ,&
&  0.35728E+00_JPRB, 0.35142E+00_JPRB, 0.34236E+00_JPRB, 0.34325E+00_JPRB, 0.33962E+00_JPRB, 0.33663E+00_JPRB, 0.33219E+00_JPRB ,&
&  0.32734E+00_JPRB, 0.32334E+00_JPRB, 0.31887E+00_JPRB, 0.31525E+00_JPRB, 0.31339E+00_JPRB, 0.31328E+00_JPRB, 0.31109E+00_JPRB ,&
&  0.30968E+00_JPRB, 0.31125E+00_JPRB, 0.31267E+00_JPRB, 0.31011E+00_JPRB, 0.31023E+00_JPRB, 0.31675E+00_JPRB, 0.32190E+00_JPRB ,&
&  0.32382E+00_JPRB, 0.32374E+00_JPRB, 0.32111E+00_JPRB, 0.32040E+00_JPRB, 0.31933E+00_JPRB, 0.31735E+00_JPRB, 0.31398E+00_JPRB ,&
&  0.30896E+00_JPRB, 0.30647E+00_JPRB, 0.30477E+00_JPRB, 0.30135E+00_JPRB, 0.29586E+00_JPRB, 0.27899E+00_JPRB, 0.26970E+00_JPRB ,&
&  0.26953E+00_JPRB, 0.27840E+00_JPRB, 0.29325E+00_JPRB, 0.29983E+00_JPRB, 0.30547E+00_JPRB, 0.31049E+00_JPRB, 0.31128E+00_JPRB ,&
&  0.31265E+00_JPRB, 0.29245E+00_JPRB, 0.28142E+00_JPRB, 0.27274E+00_JPRB, 0.26488E+00_JPRB, 0.25979E+00_JPRB, 0.25824E+00_JPRB ,&
&  0.26094E+00_JPRB, 0.26675E+00_JPRB, 0.27183E+00_JPRB, 0.27432E+00_JPRB, 0.27404E+00_JPRB, 0.27098E+00_JPRB, 0.26779E+00_JPRB ,&
&  0.26752E+00_JPRB, 0.27410E+00_JPRB, 0.28701E+00_JPRB, 0.30000E+00_JPRB, 0.30904E+00_JPRB, 0.31866E+00_JPRB, 0.32676E+00_JPRB ,&
&  0.33235E+00_JPRB, 0.33758E+00_JPRB, 0.34401E+00_JPRB, 0.34639E+00_JPRB, 0.34750E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.34920E+00_JPRB, 0.34928E+00_JPRB, 0.35268E+00_JPRB, 0.35633E+00_JPRB, 0.35731E+00_JPRB, 0.35727E+00_JPRB, 0.35492E+00_JPRB ,&
&  0.35333E+00_JPRB, 0.35451E+00_JPRB, 0.35627E+00_JPRB, 0.35749E+00_JPRB, 0.35947E+00_JPRB, 0.36209E+00_JPRB, 0.36413E+00_JPRB ,&
&  0.36412E+00_JPRB, 0.36224E+00_JPRB, 0.35904E+00_JPRB, 0.35963E+00_JPRB, 0.35960E+00_JPRB, 0.36167E+00_JPRB, 0.36914E+00_JPRB ,&
&  0.37452E+00_JPRB, 0.37822E+00_JPRB, 0.38136E+00_JPRB, 0.38181E+00_JPRB, 0.38135E+00_JPRB, 0.38292E+00_JPRB, 0.37965E+00_JPRB ,&
&  0.37351E+00_JPRB, 0.36785E+00_JPRB, 0.36269E+00_JPRB, 0.35562E+00_JPRB, 0.35815E+00_JPRB, 0.35560E+00_JPRB, 0.35250E+00_JPRB ,&
&  0.34835E+00_JPRB, 0.34308E+00_JPRB, 0.33697E+00_JPRB, 0.33126E+00_JPRB, 0.32705E+00_JPRB, 0.32314E+00_JPRB, 0.31800E+00_JPRB ,&
&  0.31630E+00_JPRB, 0.31826E+00_JPRB, 0.32403E+00_JPRB, 0.32680E+00_JPRB, 0.32451E+00_JPRB, 0.32553E+00_JPRB, 0.33223E+00_JPRB ,&
&  0.33901E+00_JPRB, 0.34216E+00_JPRB, 0.34196E+00_JPRB, 0.34244E+00_JPRB, 0.34303E+00_JPRB, 0.34439E+00_JPRB, 0.34505E+00_JPRB ,&
&  0.34402E+00_JPRB, 0.34237E+00_JPRB, 0.33982E+00_JPRB, 0.33381E+00_JPRB, 0.32666E+00_JPRB, 0.32062E+00_JPRB, 0.31817E+00_JPRB ,&
&  0.32072E+00_JPRB, 0.32056E+00_JPRB, 0.32073E+00_JPRB, 0.32232E+00_JPRB, 0.32288E+00_JPRB, 0.32215E+00_JPRB, 0.32118E+00_JPRB ,&
&  0.32246E+00_JPRB, 0.32165E+00_JPRB, 0.32310E+00_JPRB, 0.32029E+00_JPRB, 0.31573E+00_JPRB, 0.31125E+00_JPRB, 0.30700E+00_JPRB ,&
&  0.30451E+00_JPRB, 0.30365E+00_JPRB, 0.30561E+00_JPRB, 0.30629E+00_JPRB, 0.30817E+00_JPRB, 0.31039E+00_JPRB, 0.31228E+00_JPRB ,&
&  0.31335E+00_JPRB, 0.31471E+00_JPRB, 0.31728E+00_JPRB, 0.32249E+00_JPRB, 0.32797E+00_JPRB, 0.33274E+00_JPRB, 0.33766E+00_JPRB ,&
&  0.34224E+00_JPRB, 0.34551E+00_JPRB, 0.35139E+00_JPRB, 0.35270E+00_JPRB, 0.35212E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.35609E+00_JPRB, 0.35839E+00_JPRB, 0.36066E+00_JPRB, 0.36268E+00_JPRB, 0.36401E+00_JPRB, 0.36502E+00_JPRB, 0.36416E+00_JPRB ,&
&  0.36396E+00_JPRB, 0.36550E+00_JPRB, 0.36753E+00_JPRB, 0.36915E+00_JPRB, 0.37006E+00_JPRB, 0.37187E+00_JPRB, 0.37366E+00_JPRB ,&
&  0.37482E+00_JPRB, 0.37384E+00_JPRB, 0.37045E+00_JPRB, 0.36899E+00_JPRB, 0.36796E+00_JPRB, 0.37299E+00_JPRB, 0.38155E+00_JPRB ,&
&  0.38538E+00_JPRB, 0.38777E+00_JPRB, 0.39066E+00_JPRB, 0.39129E+00_JPRB, 0.39163E+00_JPRB, 0.39146E+00_JPRB, 0.38705E+00_JPRB ,&
&  0.38215E+00_JPRB, 0.37767E+00_JPRB, 0.37482E+00_JPRB, 0.37154E+00_JPRB, 0.37143E+00_JPRB, 0.36917E+00_JPRB, 0.36758E+00_JPRB ,&
&  0.36642E+00_JPRB, 0.36392E+00_JPRB, 0.36074E+00_JPRB, 0.35547E+00_JPRB, 0.34895E+00_JPRB, 0.34054E+00_JPRB, 0.33477E+00_JPRB ,&
&  0.33248E+00_JPRB, 0.33604E+00_JPRB, 0.34450E+00_JPRB, 0.34852E+00_JPRB, 0.34555E+00_JPRB, 0.34500E+00_JPRB, 0.34830E+00_JPRB ,&
&  0.35309E+00_JPRB, 0.35831E+00_JPRB, 0.36010E+00_JPRB, 0.35921E+00_JPRB, 0.35727E+00_JPRB, 0.35882E+00_JPRB, 0.36301E+00_JPRB ,&
&  0.36671E+00_JPRB, 0.36868E+00_JPRB, 0.36799E+00_JPRB, 0.36707E+00_JPRB, 0.36448E+00_JPRB, 0.35868E+00_JPRB, 0.35087E+00_JPRB ,&
&  0.34378E+00_JPRB, 0.33936E+00_JPRB, 0.33602E+00_JPRB, 0.33281E+00_JPRB, 0.33167E+00_JPRB, 0.33212E+00_JPRB, 0.33438E+00_JPRB ,&
&  0.33532E+00_JPRB, 0.33719E+00_JPRB, 0.33734E+00_JPRB, 0.33319E+00_JPRB, 0.32988E+00_JPRB, 0.33117E+00_JPRB, 0.33271E+00_JPRB ,&
&  0.33525E+00_JPRB, 0.33777E+00_JPRB, 0.34035E+00_JPRB, 0.34144E+00_JPRB, 0.34305E+00_JPRB, 0.34570E+00_JPRB, 0.34726E+00_JPRB ,&
&  0.34805E+00_JPRB, 0.34865E+00_JPRB, 0.35045E+00_JPRB, 0.35336E+00_JPRB, 0.35596E+00_JPRB, 0.35806E+00_JPRB, 0.35753E+00_JPRB ,&
&  0.35661E+00_JPRB, 0.35556E+00_JPRB, 0.35631E+00_JPRB, 0.35694E+00_JPRB, 0.35580E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.38193E+00_JPRB, 0.38209E+00_JPRB, 0.38138E+00_JPRB, 0.38010E+00_JPRB, 0.37872E+00_JPRB, 0.37756E+00_JPRB, 0.37740E+00_JPRB ,&
&  0.37739E+00_JPRB, 0.37783E+00_JPRB, 0.37848E+00_JPRB, 0.37901E+00_JPRB, 0.38050E+00_JPRB, 0.38229E+00_JPRB, 0.38376E+00_JPRB ,&
&  0.38534E+00_JPRB, 0.38456E+00_JPRB, 0.38194E+00_JPRB, 0.37949E+00_JPRB, 0.38029E+00_JPRB, 0.38574E+00_JPRB, 0.39178E+00_JPRB ,&
&  0.39341E+00_JPRB, 0.39456E+00_JPRB, 0.39506E+00_JPRB, 0.39393E+00_JPRB, 0.39252E+00_JPRB, 0.39212E+00_JPRB, 0.38853E+00_JPRB ,&
&  0.38403E+00_JPRB, 0.38278E+00_JPRB, 0.38236E+00_JPRB, 0.38005E+00_JPRB, 0.37999E+00_JPRB, 0.38260E+00_JPRB, 0.38182E+00_JPRB ,&
&  0.38091E+00_JPRB, 0.37909E+00_JPRB, 0.37592E+00_JPRB, 0.37312E+00_JPRB, 0.36982E+00_JPRB, 0.36364E+00_JPRB, 0.35648E+00_JPRB ,&
&  0.35073E+00_JPRB, 0.35084E+00_JPRB, 0.35624E+00_JPRB, 0.36202E+00_JPRB, 0.36130E+00_JPRB, 0.36046E+00_JPRB, 0.36207E+00_JPRB ,&
&  0.36568E+00_JPRB, 0.37015E+00_JPRB, 0.37298E+00_JPRB, 0.37216E+00_JPRB, 0.37100E+00_JPRB, 0.37283E+00_JPRB, 0.37545E+00_JPRB ,&
&  0.37672E+00_JPRB, 0.37818E+00_JPRB, 0.37922E+00_JPRB, 0.38067E+00_JPRB, 0.38120E+00_JPRB, 0.37979E+00_JPRB, 0.37894E+00_JPRB ,&
&  0.37646E+00_JPRB, 0.37284E+00_JPRB, 0.36808E+00_JPRB, 0.36256E+00_JPRB, 0.35856E+00_JPRB, 0.35615E+00_JPRB, 0.35620E+00_JPRB ,&
&  0.35558E+00_JPRB, 0.35702E+00_JPRB, 0.35736E+00_JPRB, 0.35511E+00_JPRB, 0.35210E+00_JPRB, 0.35209E+00_JPRB, 0.35559E+00_JPRB ,&
&  0.36035E+00_JPRB, 0.36374E+00_JPRB, 0.36942E+00_JPRB, 0.37231E+00_JPRB, 0.37436E+00_JPRB, 0.37683E+00_JPRB, 0.37746E+00_JPRB ,&
&  0.37484E+00_JPRB, 0.37270E+00_JPRB, 0.37222E+00_JPRB, 0.37490E+00_JPRB, 0.37593E+00_JPRB, 0.37600E+00_JPRB, 0.37457E+00_JPRB ,&
&  0.37471E+00_JPRB, 0.37729E+00_JPRB, 0.37866E+00_JPRB, 0.37911E+00_JPRB, 0.38027E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.41429E+00_JPRB, 0.41244E+00_JPRB, 0.41051E+00_JPRB, 0.40753E+00_JPRB, 0.40422E+00_JPRB, 0.40110E+00_JPRB, 0.39813E+00_JPRB ,&
&  0.39590E+00_JPRB, 0.39495E+00_JPRB, 0.39503E+00_JPRB, 0.39485E+00_JPRB, 0.39436E+00_JPRB, 0.39490E+00_JPRB, 0.39503E+00_JPRB ,&
&  0.39636E+00_JPRB, 0.39651E+00_JPRB, 0.39430E+00_JPRB, 0.39394E+00_JPRB, 0.39652E+00_JPRB, 0.40138E+00_JPRB, 0.40335E+00_JPRB ,&
&  0.40124E+00_JPRB, 0.40112E+00_JPRB, 0.40097E+00_JPRB, 0.39892E+00_JPRB, 0.39748E+00_JPRB, 0.39656E+00_JPRB, 0.39268E+00_JPRB ,&
&  0.38782E+00_JPRB, 0.38834E+00_JPRB, 0.39004E+00_JPRB, 0.39371E+00_JPRB, 0.39390E+00_JPRB, 0.39229E+00_JPRB, 0.39264E+00_JPRB ,&
&  0.39068E+00_JPRB, 0.39015E+00_JPRB, 0.38969E+00_JPRB, 0.39013E+00_JPRB, 0.38847E+00_JPRB, 0.38603E+00_JPRB, 0.38144E+00_JPRB ,&
&  0.37235E+00_JPRB, 0.36942E+00_JPRB, 0.37158E+00_JPRB, 0.37602E+00_JPRB, 0.37568E+00_JPRB, 0.37651E+00_JPRB, 0.37964E+00_JPRB ,&
&  0.38358E+00_JPRB, 0.38648E+00_JPRB, 0.38826E+00_JPRB, 0.38944E+00_JPRB, 0.39104E+00_JPRB, 0.39255E+00_JPRB, 0.39339E+00_JPRB ,&
&  0.39453E+00_JPRB, 0.39654E+00_JPRB, 0.39896E+00_JPRB, 0.40152E+00_JPRB, 0.40317E+00_JPRB, 0.40499E+00_JPRB, 0.40626E+00_JPRB ,&
&  0.40582E+00_JPRB, 0.40289E+00_JPRB, 0.39927E+00_JPRB, 0.39389E+00_JPRB, 0.38825E+00_JPRB, 0.38477E+00_JPRB, 0.38276E+00_JPRB ,&
&  0.37880E+00_JPRB, 0.37542E+00_JPRB, 0.37297E+00_JPRB, 0.37238E+00_JPRB, 0.37254E+00_JPRB, 0.37368E+00_JPRB, 0.37580E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.38048E+00_JPRB, 0.38682E+00_JPRB, 0.39119E+00_JPRB, 0.39338E+00_JPRB, 0.39646E+00_JPRB, 0.39785E+00_JPRB ,&
&  0.39610E+00_JPRB, 0.39451E+00_JPRB, 0.39339E+00_JPRB, 0.39559E+00_JPRB, 0.39949E+00_JPRB, 0.40308E+00_JPRB, 0.40587E+00_JPRB ,&
&  0.41013E+00_JPRB, 0.41425E+00_JPRB, 0.41555E+00_JPRB, 0.41582E+00_JPRB, 0.41584E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.44350E+00_JPRB, 0.43857E+00_JPRB, 0.43325E+00_JPRB, 0.42836E+00_JPRB, 0.42401E+00_JPRB, 0.41960E+00_JPRB, 0.41551E+00_JPRB ,&
&  0.41143E+00_JPRB, 0.40858E+00_JPRB, 0.40643E+00_JPRB, 0.40465E+00_JPRB, 0.40102E+00_JPRB, 0.39931E+00_JPRB, 0.39982E+00_JPRB ,&
&  0.40213E+00_JPRB, 0.40452E+00_JPRB, 0.40636E+00_JPRB, 0.40592E+00_JPRB, 0.40823E+00_JPRB, 0.41048E+00_JPRB, 0.41074E+00_JPRB ,&
&  0.41001E+00_JPRB, 0.40928E+00_JPRB, 0.40727E+00_JPRB, 0.40465E+00_JPRB, 0.40467E+00_JPRB, 0.40507E+00_JPRB, 0.40147E+00_JPRB ,&
&  0.39472E+00_JPRB, 0.39650E+00_JPRB, 0.38891E+00_JPRB, 0.40284E+00_JPRB, 0.40631E+00_JPRB, 0.40125E+00_JPRB, 0.39483E+00_JPRB ,&
&  0.39241E+00_JPRB, 0.39302E+00_JPRB, 0.39605E+00_JPRB, 0.39829E+00_JPRB, 0.39931E+00_JPRB, 0.39971E+00_JPRB, 0.39984E+00_JPRB ,&
&  0.39272E+00_JPRB, 0.38856E+00_JPRB, 0.38824E+00_JPRB, 0.38912E+00_JPRB, 0.38925E+00_JPRB, 0.39001E+00_JPRB, 0.39200E+00_JPRB ,&
&  0.39357E+00_JPRB, 0.39464E+00_JPRB, 0.39582E+00_JPRB, 0.39840E+00_JPRB, 0.40020E+00_JPRB, 0.40109E+00_JPRB, 0.40294E+00_JPRB ,&
&  0.40589E+00_JPRB, 0.40962E+00_JPRB, 0.41293E+00_JPRB, 0.41561E+00_JPRB, 0.42102E+00_JPRB, 0.42826E+00_JPRB, 0.43395E+00_JPRB ,&
&  0.43601E+00_JPRB, 0.43523E+00_JPRB, 0.43281E+00_JPRB, 0.42748E+00_JPRB, 0.42012E+00_JPRB, 0.41426E+00_JPRB, 0.40960E+00_JPRB ,&
&  0.40296E+00_JPRB, 0.39960E+00_JPRB, 0.39747E+00_JPRB, 0.39453E+00_JPRB, 0.39205E+00_JPRB, 0.39089E+00_JPRB, 0.39084E+00_JPRB ,&
&  0.39244E+00_JPRB, 0.39572E+00_JPRB, 0.39783E+00_JPRB, 0.40050E+00_JPRB, 0.40425E+00_JPRB, 0.40800E+00_JPRB, 0.40817E+00_JPRB ,&
&  0.40682E+00_JPRB, 0.40645E+00_JPRB, 0.40865E+00_JPRB, 0.41675E+00_JPRB, 0.42780E+00_JPRB, 0.43705E+00_JPRB, 0.44523E+00_JPRB ,&
&  0.45107E+00_JPRB, 0.45386E+00_JPRB, 0.45327E+00_JPRB, 0.45183E+00_JPRB, 0.44828E+00_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.48337E+00_JPRB, 0.47345E+00_JPRB, 0.46313E+00_JPRB, 0.45512E+00_JPRB, 0.44914E+00_JPRB, 0.44316E+00_JPRB, 0.43635E+00_JPRB ,&
&  0.43107E+00_JPRB, 0.42466E+00_JPRB, 0.41956E+00_JPRB, 0.41470E+00_JPRB, 0.41069E+00_JPRB, 0.40910E+00_JPRB, 0.40952E+00_JPRB ,&
&  0.41185E+00_JPRB, 0.41722E+00_JPRB, 0.42028E+00_JPRB, 0.41887E+00_JPRB, 0.42063E+00_JPRB, 0.42285E+00_JPRB, 0.42334E+00_JPRB ,&
&  0.42106E+00_JPRB, 0.41801E+00_JPRB, 0.41461E+00_JPRB, 0.41255E+00_JPRB, 0.41251E+00_JPRB, 0.41371E+00_JPRB, 0.41089E+00_JPRB ,&
&  0.40606E+00_JPRB, 0.39884E+00_JPRB, 0.41111E+00_JPRB, 0.41914E+00_JPRB, 0.41940E+00_JPRB, 0.41507E+00_JPRB, 0.40963E+00_JPRB ,&
&  0.40246E+00_JPRB, 0.39848E+00_JPRB, 0.39915E+00_JPRB, 0.40167E+00_JPRB, 0.40457E+00_JPRB, 0.40620E+00_JPRB, 0.40786E+00_JPRB ,&
&  0.40499E+00_JPRB, 0.40343E+00_JPRB, 0.40149E+00_JPRB, 0.40021E+00_JPRB, 0.40087E+00_JPRB, 0.40195E+00_JPRB, 0.40268E+00_JPRB ,&
&  0.40184E+00_JPRB, 0.40253E+00_JPRB, 0.40334E+00_JPRB, 0.40253E+00_JPRB, 0.40402E+00_JPRB, 0.40635E+00_JPRB, 0.40963E+00_JPRB ,&
&  0.41421E+00_JPRB, 0.42054E+00_JPRB, 0.42756E+00_JPRB, 0.43701E+00_JPRB, 0.45049E+00_JPRB, 0.46411E+00_JPRB, 0.47435E+00_JPRB ,&
&  0.47922E+00_JPRB, 0.47840E+00_JPRB, 0.47349E+00_JPRB, 0.46486E+00_JPRB, 0.45652E+00_JPRB, 0.44928E+00_JPRB, 0.44048E+00_JPRB ,&
&  0.43011E+00_JPRB, 0.42321E+00_JPRB, 0.41868E+00_JPRB, 0.41596E+00_JPRB, 0.41318E+00_JPRB, 0.41153E+00_JPRB, 0.41181E+00_JPRB ,&
&  0.41393E+00_JPRB, 0.41617E+00_JPRB, 0.41711E+00_JPRB, 0.41739E+00_JPRB, 0.41819E+00_JPRB, 0.41692E+00_JPRB, 0.41631E+00_JPRB ,&
&  0.41859E+00_JPRB, 0.42456E+00_JPRB, 0.43668E+00_JPRB, 0.46022E+00_JPRB, 0.47836E+00_JPRB, 0.49176E+00_JPRB, 0.50040E+00_JPRB ,&
&  0.50443E+00_JPRB, 0.50586E+00_JPRB, 0.50342E+00_JPRB, 0.49905E+00_JPRB, 0.49255E+00_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.51559E+00_JPRB, 0.50176E+00_JPRB, 0.48726E+00_JPRB, 0.47707E+00_JPRB, 0.46948E+00_JPRB, 0.46232E+00_JPRB, 0.45486E+00_JPRB ,&
&  0.44665E+00_JPRB, 0.43924E+00_JPRB, 0.43217E+00_JPRB, 0.42506E+00_JPRB, 0.42109E+00_JPRB, 0.41888E+00_JPRB, 0.41892E+00_JPRB ,&
&  0.42115E+00_JPRB, 0.42563E+00_JPRB, 0.42755E+00_JPRB, 0.42715E+00_JPRB, 0.42774E+00_JPRB, 0.42746E+00_JPRB, 0.42570E+00_JPRB ,&
&  0.42154E+00_JPRB, 0.41653E+00_JPRB, 0.41380E+00_JPRB, 0.41455E+00_JPRB, 0.41478E+00_JPRB, 0.41569E+00_JPRB, 0.41616E+00_JPRB ,&
&  0.41979E+00_JPRB, 0.38640E+00_JPRB, 0.41420E+00_JPRB, 0.43628E+00_JPRB, 0.43018E+00_JPRB, 0.42306E+00_JPRB, 0.41576E+00_JPRB ,&
&  0.40804E+00_JPRB, 0.40429E+00_JPRB, 0.40292E+00_JPRB, 0.40508E+00_JPRB, 0.40887E+00_JPRB, 0.41074E+00_JPRB, 0.41103E+00_JPRB ,&
&  0.40821E+00_JPRB, 0.40809E+00_JPRB, 0.40728E+00_JPRB, 0.40709E+00_JPRB, 0.40766E+00_JPRB, 0.40848E+00_JPRB, 0.40836E+00_JPRB ,&
&  0.40670E+00_JPRB, 0.40342E+00_JPRB, 0.40015E+00_JPRB, 0.39976E+00_JPRB, 0.40134E+00_JPRB, 0.40541E+00_JPRB, 0.41298E+00_JPRB ,&
&  0.42444E+00_JPRB, 0.43819E+00_JPRB, 0.45337E+00_JPRB, 0.47012E+00_JPRB, 0.49081E+00_JPRB, 0.51054E+00_JPRB, 0.52555E+00_JPRB ,&
&  0.52841E+00_JPRB, 0.52300E+00_JPRB, 0.51105E+00_JPRB, 0.49650E+00_JPRB, 0.48422E+00_JPRB, 0.47230E+00_JPRB, 0.46126E+00_JPRB ,&
&  0.45302E+00_JPRB, 0.44613E+00_JPRB, 0.44166E+00_JPRB, 0.44016E+00_JPRB, 0.43936E+00_JPRB, 0.43728E+00_JPRB, 0.43555E+00_JPRB ,&
&  0.43400E+00_JPRB, 0.43263E+00_JPRB, 0.43170E+00_JPRB, 0.43056E+00_JPRB, 0.42846E+00_JPRB, 0.42658E+00_JPRB, 0.42739E+00_JPRB ,&
&  0.43232E+00_JPRB, 0.44372E+00_JPRB, 0.46335E+00_JPRB, 0.49351E+00_JPRB, 0.52130E+00_JPRB, 0.54176E+00_JPRB, 0.55247E+00_JPRB ,&
&  0.55329E+00_JPRB, 0.54962E+00_JPRB, 0.54075E+00_JPRB, 0.53313E+00_JPRB, 0.52680E+00_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.53953E+00_JPRB, 0.53035E+00_JPRB, 0.52130E+00_JPRB, 0.51174E+00_JPRB, 0.50127E+00_JPRB, 0.48999E+00_JPRB, 0.47863E+00_JPRB ,&
&  0.46937E+00_JPRB, 0.46165E+00_JPRB, 0.45435E+00_JPRB, 0.44799E+00_JPRB, 0.44182E+00_JPRB, 0.43875E+00_JPRB, 0.43768E+00_JPRB ,&
&  0.43779E+00_JPRB, 0.43690E+00_JPRB, 0.43513E+00_JPRB, 0.43500E+00_JPRB, 0.43449E+00_JPRB, 0.43199E+00_JPRB, 0.42855E+00_JPRB ,&
&  0.42300E+00_JPRB, 0.41824E+00_JPRB, 0.41656E+00_JPRB, 0.41897E+00_JPRB, 0.41808E+00_JPRB, 0.42025E+00_JPRB, 0.42884E+00_JPRB ,&
&  0.41451E+00_JPRB, 0.39031E+00_JPRB, 0.42582E+00_JPRB, 0.46616E+00_JPRB, 0.44985E+00_JPRB, 0.43347E+00_JPRB, 0.42847E+00_JPRB ,&
&  0.42383E+00_JPRB, 0.42257E+00_JPRB, 0.42102E+00_JPRB, 0.42010E+00_JPRB, 0.42069E+00_JPRB, 0.41950E+00_JPRB, 0.41709E+00_JPRB ,&
&  0.41560E+00_JPRB, 0.41783E+00_JPRB, 0.42035E+00_JPRB, 0.42173E+00_JPRB, 0.42125E+00_JPRB, 0.41959E+00_JPRB, 0.41564E+00_JPRB ,&
&  0.41199E+00_JPRB, 0.40986E+00_JPRB, 0.40997E+00_JPRB, 0.41369E+00_JPRB, 0.42030E+00_JPRB, 0.43129E+00_JPRB, 0.44919E+00_JPRB ,&
&  0.47327E+00_JPRB, 0.49783E+00_JPRB, 0.51959E+00_JPRB, 0.54059E+00_JPRB, 0.56320E+00_JPRB, 0.58266E+00_JPRB, 0.59266E+00_JPRB ,&
&  0.58491E+00_JPRB, 0.56564E+00_JPRB, 0.54321E+00_JPRB, 0.52549E+00_JPRB, 0.51272E+00_JPRB, 0.50454E+00_JPRB, 0.49977E+00_JPRB ,&
&  0.49835E+00_JPRB, 0.49742E+00_JPRB, 0.49638E+00_JPRB, 0.49609E+00_JPRB, 0.49589E+00_JPRB, 0.49376E+00_JPRB, 0.49077E+00_JPRB ,&
&  0.48599E+00_JPRB, 0.47846E+00_JPRB, 0.46943E+00_JPRB, 0.46146E+00_JPRB, 0.45589E+00_JPRB, 0.45454E+00_JPRB, 0.45696E+00_JPRB ,&
&  0.46558E+00_JPRB, 0.48379E+00_JPRB, 0.50446E+00_JPRB, 0.55213E+00_JPRB, 0.58519E+00_JPRB, 0.60673E+00_JPRB, 0.61514E+00_JPRB ,&
&  0.60950E+00_JPRB, 0.58801E+00_JPRB, 0.56803E+00_JPRB, 0.55639E+00_JPRB, 0.55130E+00_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.54829E+00_JPRB, 0.54420E+00_JPRB, 0.53676E+00_JPRB, 0.52867E+00_JPRB, 0.52098E+00_JPRB, 0.51097E+00_JPRB, 0.49827E+00_JPRB ,&
&  0.48672E+00_JPRB, 0.47650E+00_JPRB, 0.46996E+00_JPRB, 0.46357E+00_JPRB, 0.45402E+00_JPRB, 0.44801E+00_JPRB, 0.44692E+00_JPRB ,&
&  0.44497E+00_JPRB, 0.43921E+00_JPRB, 0.43263E+00_JPRB, 0.43145E+00_JPRB, 0.42948E+00_JPRB, 0.42694E+00_JPRB, 0.42326E+00_JPRB ,&
&  0.41947E+00_JPRB, 0.41850E+00_JPRB, 0.41920E+00_JPRB, 0.42024E+00_JPRB, 0.41705E+00_JPRB, 0.41840E+00_JPRB, 0.43095E+00_JPRB ,&
&  0.47996E+00_JPRB, 0.41758E+00_JPRB, 0.46629E+00_JPRB, 0.51956E+00_JPRB, 0.49724E+00_JPRB, 0.48013E+00_JPRB, 0.46716E+00_JPRB ,&
&  0.45778E+00_JPRB, 0.45446E+00_JPRB, 0.45158E+00_JPRB, 0.44617E+00_JPRB, 0.44049E+00_JPRB, 0.43428E+00_JPRB, 0.43170E+00_JPRB ,&
&  0.43156E+00_JPRB, 0.43605E+00_JPRB, 0.43841E+00_JPRB, 0.43802E+00_JPRB, 0.43439E+00_JPRB, 0.43066E+00_JPRB, 0.42766E+00_JPRB ,&
&  0.42650E+00_JPRB, 0.42792E+00_JPRB, 0.43047E+00_JPRB, 0.43473E+00_JPRB, 0.44280E+00_JPRB, 0.45984E+00_JPRB, 0.48883E+00_JPRB ,&
&  0.52640E+00_JPRB, 0.55993E+00_JPRB, 0.58780E+00_JPRB, 0.61530E+00_JPRB, 0.63828E+00_JPRB, 0.65101E+00_JPRB, 0.64908E+00_JPRB ,&
&  0.63448E+00_JPRB, 0.60793E+00_JPRB, 0.58716E+00_JPRB, 0.56830E+00_JPRB, 0.55005E+00_JPRB, 0.53560E+00_JPRB, 0.52607E+00_JPRB ,&
&  0.52029E+00_JPRB, 0.51653E+00_JPRB, 0.51371E+00_JPRB, 0.51045E+00_JPRB, 0.50991E+00_JPRB, 0.51011E+00_JPRB, 0.51210E+00_JPRB ,&
&  0.51143E+00_JPRB, 0.50899E+00_JPRB, 0.50085E+00_JPRB, 0.48964E+00_JPRB, 0.48401E+00_JPRB, 0.48321E+00_JPRB, 0.48678E+00_JPRB ,&
&  0.49654E+00_JPRB, 0.51523E+00_JPRB, 0.53501E+00_JPRB, 0.60168E+00_JPRB, 0.63746E+00_JPRB, 0.65324E+00_JPRB, 0.65346E+00_JPRB ,&
&  0.63443E+00_JPRB, 0.60184E+00_JPRB, 0.48001E+00_JPRB, 0.54239E+00_JPRB, 0.55607E+00_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.59312E+00_JPRB, 0.58145E+00_JPRB, 0.56659E+00_JPRB, 0.54698E+00_JPRB, 0.53293E+00_JPRB, 0.52187E+00_JPRB, 0.51007E+00_JPRB ,&
&  0.49868E+00_JPRB, 0.48901E+00_JPRB, 0.48075E+00_JPRB, 0.47117E+00_JPRB, 0.46061E+00_JPRB, 0.45727E+00_JPRB, 0.45601E+00_JPRB ,&
&  0.45195E+00_JPRB, 0.44379E+00_JPRB, 0.43653E+00_JPRB, 0.43507E+00_JPRB, 0.43461E+00_JPRB, 0.43445E+00_JPRB, 0.43146E+00_JPRB ,&
&  0.42646E+00_JPRB, 0.42583E+00_JPRB, 0.42914E+00_JPRB, 0.42973E+00_JPRB, 0.42561E+00_JPRB, 0.42695E+00_JPRB, 0.43928E+00_JPRB ,&
&  0.49998E+00_JPRB, 0.45887E+00_JPRB, 0.51144E+00_JPRB, 0.60698E+00_JPRB, 0.59117E+00_JPRB, 0.58651E+00_JPRB, 0.56262E+00_JPRB ,&
&  0.53631E+00_JPRB, 0.52206E+00_JPRB, 0.50898E+00_JPRB, 0.49023E+00_JPRB, 0.47174E+00_JPRB, 0.46154E+00_JPRB, 0.46041E+00_JPRB ,&
&  0.46416E+00_JPRB, 0.46616E+00_JPRB, 0.46675E+00_JPRB, 0.46210E+00_JPRB, 0.45915E+00_JPRB, 0.45708E+00_JPRB, 0.45626E+00_JPRB ,&
&  0.45593E+00_JPRB, 0.45910E+00_JPRB, 0.46371E+00_JPRB, 0.46769E+00_JPRB, 0.47889E+00_JPRB, 0.51118E+00_JPRB, 0.55930E+00_JPRB ,&
&  0.60566E+00_JPRB, 0.64673E+00_JPRB, 0.67694E+00_JPRB, 0.70199E+00_JPRB, 0.71574E+00_JPRB, 0.72599E+00_JPRB, 0.72584E+00_JPRB ,&
&  0.70686E+00_JPRB, 0.67795E+00_JPRB, 0.64969E+00_JPRB, 0.61903E+00_JPRB, 0.59088E+00_JPRB, 0.56878E+00_JPRB, 0.55128E+00_JPRB ,&
&  0.53795E+00_JPRB, 0.52856E+00_JPRB, 0.52116E+00_JPRB, 0.51219E+00_JPRB, 0.50679E+00_JPRB, 0.50721E+00_JPRB, 0.51023E+00_JPRB ,&
&  0.51509E+00_JPRB, 0.52119E+00_JPRB, 0.52506E+00_JPRB, 0.52653E+00_JPRB, 0.52736E+00_JPRB, 0.53062E+00_JPRB, 0.53535E+00_JPRB ,&
&  0.54854E+00_JPRB, 0.57449E+00_JPRB, 0.59892E+00_JPRB, 0.68354E+00_JPRB, 0.73191E+00_JPRB, 0.72333E+00_JPRB, 0.69214E+00_JPRB ,&
&  0.65193E+00_JPRB, 0.60984E+00_JPRB, 0.58644E+00_JPRB, 0.52451E+00_JPRB, 0.60245E+00_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.65342E+00_JPRB, 0.62745E+00_JPRB, 0.58968E+00_JPRB, 0.55390E+00_JPRB, 0.52951E+00_JPRB, 0.51459E+00_JPRB, 0.50593E+00_JPRB ,&
&  0.49877E+00_JPRB, 0.49104E+00_JPRB, 0.48263E+00_JPRB, 0.47165E+00_JPRB, 0.46485E+00_JPRB, 0.46159E+00_JPRB, 0.45911E+00_JPRB ,&
&  0.45455E+00_JPRB, 0.44513E+00_JPRB, 0.44224E+00_JPRB, 0.43975E+00_JPRB, 0.43867E+00_JPRB, 0.43710E+00_JPRB, 0.43213E+00_JPRB ,&
&  0.42588E+00_JPRB, 0.42287E+00_JPRB, 0.42616E+00_JPRB, 0.42797E+00_JPRB, 0.43005E+00_JPRB, 0.43533E+00_JPRB, 0.45147E+00_JPRB ,&
&  0.52541E+00_JPRB, 0.56022E+00_JPRB, 0.64221E+00_JPRB, 0.71602E+00_JPRB, 0.69849E+00_JPRB, 0.67226E+00_JPRB, 0.65435E+00_JPRB ,&
&  0.63042E+00_JPRB, 0.61239E+00_JPRB, 0.58044E+00_JPRB, 0.54148E+00_JPRB, 0.50963E+00_JPRB, 0.50032E+00_JPRB, 0.49331E+00_JPRB ,&
&  0.48735E+00_JPRB, 0.48412E+00_JPRB, 0.48247E+00_JPRB, 0.48208E+00_JPRB, 0.48253E+00_JPRB, 0.48166E+00_JPRB, 0.48073E+00_JPRB ,&
&  0.48203E+00_JPRB, 0.48639E+00_JPRB, 0.49152E+00_JPRB, 0.50258E+00_JPRB, 0.53511E+00_JPRB, 0.58997E+00_JPRB, 0.64846E+00_JPRB ,&
&  0.68723E+00_JPRB, 0.72060E+00_JPRB, 0.76459E+00_JPRB, 0.79820E+00_JPRB, 0.81595E+00_JPRB, 0.81663E+00_JPRB, 0.80263E+00_JPRB ,&
&  0.78471E+00_JPRB, 0.76183E+00_JPRB, 0.71441E+00_JPRB, 0.66391E+00_JPRB, 0.63167E+00_JPRB, 0.61180E+00_JPRB, 0.59756E+00_JPRB ,&
&  0.58703E+00_JPRB, 0.57349E+00_JPRB, 0.55858E+00_JPRB, 0.54594E+00_JPRB, 0.53855E+00_JPRB, 0.53140E+00_JPRB, 0.52470E+00_JPRB ,&
&  0.52578E+00_JPRB, 0.53501E+00_JPRB, 0.54838E+00_JPRB, 0.55739E+00_JPRB, 0.56532E+00_JPRB, 0.57507E+00_JPRB, 0.58143E+00_JPRB ,&
&  0.59064E+00_JPRB, 0.62040E+00_JPRB, 0.64387E+00_JPRB, 0.72471E+00_JPRB, 0.73876E+00_JPRB, 0.72518E+00_JPRB, 0.68740E+00_JPRB ,&
&  0.64035E+00_JPRB, 0.60216E+00_JPRB, 0.57468E+00_JPRB, 0.59146E+00_JPRB, 0.65003E+00_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.69865E+00_JPRB, 0.64081E+00_JPRB, 0.59296E+00_JPRB, 0.55309E+00_JPRB, 0.52790E+00_JPRB, 0.51306E+00_JPRB, 0.50310E+00_JPRB ,&
&  0.49814E+00_JPRB, 0.49441E+00_JPRB, 0.48680E+00_JPRB, 0.48164E+00_JPRB, 0.47768E+00_JPRB, 0.47569E+00_JPRB, 0.47386E+00_JPRB ,&
&  0.46485E+00_JPRB, 0.46285E+00_JPRB, 0.46262E+00_JPRB, 0.45810E+00_JPRB, 0.45201E+00_JPRB, 0.44553E+00_JPRB, 0.43708E+00_JPRB ,&
&  0.43138E+00_JPRB, 0.42959E+00_JPRB, 0.43152E+00_JPRB, 0.43544E+00_JPRB, 0.43956E+00_JPRB, 0.45096E+00_JPRB, 0.47539E+00_JPRB ,&
&  0.58135E+00_JPRB, 0.67549E+00_JPRB, 0.79040E+00_JPRB, 0.86387E+00_JPRB, 0.78794E+00_JPRB, 0.74510E+00_JPRB, 0.72027E+00_JPRB ,&
&  0.72082E+00_JPRB, 0.72562E+00_JPRB, 0.69326E+00_JPRB, 0.64246E+00_JPRB, 0.59685E+00_JPRB, 0.57203E+00_JPRB, 0.55599E+00_JPRB ,&
&  0.54115E+00_JPRB, 0.52993E+00_JPRB, 0.51965E+00_JPRB, 0.51993E+00_JPRB, 0.52070E+00_JPRB, 0.51770E+00_JPRB, 0.51510E+00_JPRB ,&
&  0.51476E+00_JPRB, 0.51845E+00_JPRB, 0.52754E+00_JPRB, 0.56100E+00_JPRB, 0.63943E+00_JPRB, 0.72180E+00_JPRB, 0.80076E+00_JPRB ,&
&  0.83968E+00_JPRB, 0.88015E+00_JPRB, 0.92820E+00_JPRB, 0.96341E+00_JPRB, 0.96456E+00_JPRB, 0.95493E+00_JPRB, 0.94352E+00_JPRB ,&
&  0.91959E+00_JPRB, 0.85571E+00_JPRB, 0.77127E+00_JPRB, 0.70946E+00_JPRB, 0.67627E+00_JPRB, 0.65950E+00_JPRB, 0.65045E+00_JPRB ,&
&  0.64268E+00_JPRB, 0.63484E+00_JPRB, 0.62240E+00_JPRB, 0.60466E+00_JPRB, 0.58423E+00_JPRB, 0.57014E+00_JPRB, 0.55925E+00_JPRB ,&
&  0.55280E+00_JPRB, 0.55389E+00_JPRB, 0.56410E+00_JPRB, 0.58195E+00_JPRB, 0.59660E+00_JPRB, 0.60862E+00_JPRB, 0.61685E+00_JPRB ,&
&  0.62905E+00_JPRB, 0.67430E+00_JPRB, 0.66914E+00_JPRB, 0.75546E+00_JPRB, 0.83420E+00_JPRB, 0.78402E+00_JPRB, 0.71535E+00_JPRB ,&
&  0.64791E+00_JPRB, 0.60349E+00_JPRB, 0.58915E+00_JPRB, 0.61610E+00_JPRB, 0.73352E+00_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.60040E+00_JPRB, 0.57745E+00_JPRB, 0.55736E+00_JPRB, 0.53553E+00_JPRB, 0.51604E+00_JPRB, 0.50657E+00_JPRB, 0.50434E+00_JPRB ,&
&  0.50609E+00_JPRB, 0.50598E+00_JPRB, 0.50331E+00_JPRB, 0.49713E+00_JPRB, 0.49258E+00_JPRB, 0.48824E+00_JPRB, 0.48441E+00_JPRB ,&
&  0.47714E+00_JPRB, 0.47718E+00_JPRB, 0.47301E+00_JPRB, 0.46620E+00_JPRB, 0.45875E+00_JPRB, 0.45014E+00_JPRB, 0.44177E+00_JPRB ,&
&  0.43658E+00_JPRB, 0.43500E+00_JPRB, 0.43580E+00_JPRB, 0.44017E+00_JPRB, 0.45236E+00_JPRB, 0.46527E+00_JPRB, 0.48401E+00_JPRB ,&
&  0.60897E+00_JPRB, 0.79686E+00_JPRB, 0.99426E+00_JPRB, 0.11486E+01_JPRB, 0.10744E+01_JPRB, 0.94568E+00_JPRB, 0.87217E+00_JPRB ,&
&  0.84326E+00_JPRB, 0.83135E+00_JPRB, 0.79814E+00_JPRB, 0.75860E+00_JPRB, 0.71336E+00_JPRB, 0.69158E+00_JPRB, 0.66680E+00_JPRB ,&
&  0.63877E+00_JPRB, 0.61329E+00_JPRB, 0.58889E+00_JPRB, 0.57769E+00_JPRB, 0.56868E+00_JPRB, 0.55759E+00_JPRB, 0.55042E+00_JPRB ,&
&  0.55223E+00_JPRB, 0.56086E+00_JPRB, 0.58522E+00_JPRB, 0.65872E+00_JPRB, 0.80367E+00_JPRB, 0.92351E+00_JPRB, 0.10697E+01_JPRB ,&
&  0.11139E+01_JPRB, 0.11477E+01_JPRB, 0.11744E+01_JPRB, 0.11693E+01_JPRB, 0.11094E+01_JPRB, 0.10654E+01_JPRB, 0.10566E+01_JPRB ,&
&  0.10109E+01_JPRB, 0.90174E+00_JPRB, 0.79463E+00_JPRB, 0.72825E+00_JPRB, 0.69409E+00_JPRB, 0.67941E+00_JPRB, 0.66867E+00_JPRB ,&
&  0.66409E+00_JPRB, 0.65769E+00_JPRB, 0.64164E+00_JPRB, 0.61981E+00_JPRB, 0.59929E+00_JPRB, 0.58496E+00_JPRB, 0.57681E+00_JPRB ,&
&  0.56947E+00_JPRB, 0.56767E+00_JPRB, 0.58489E+00_JPRB, 0.59668E+00_JPRB, 0.60481E+00_JPRB, 0.61558E+00_JPRB, 0.63077E+00_JPRB ,&
&  0.65196E+00_JPRB, 0.70266E+00_JPRB, 0.69594E+00_JPRB, 0.74103E+00_JPRB, 0.79571E+00_JPRB, 0.79060E+00_JPRB, 0.71967E+00_JPRB ,&
&  0.64692E+00_JPRB, 0.61415E+00_JPRB, 0.61446E+00_JPRB, 0.62220E+00_JPRB, 0.62445E+00_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.57125E+00_JPRB, 0.55392E+00_JPRB, 0.54078E+00_JPRB, 0.51837E+00_JPRB, 0.50438E+00_JPRB, 0.49758E+00_JPRB, 0.50038E+00_JPRB ,&
&  0.50656E+00_JPRB, 0.51345E+00_JPRB, 0.51087E+00_JPRB, 0.50785E+00_JPRB, 0.50186E+00_JPRB, 0.49732E+00_JPRB, 0.48925E+00_JPRB ,&
&  0.48453E+00_JPRB, 0.47968E+00_JPRB, 0.47246E+00_JPRB, 0.46382E+00_JPRB, 0.45233E+00_JPRB, 0.44724E+00_JPRB, 0.44115E+00_JPRB ,&
&  0.43560E+00_JPRB, 0.43349E+00_JPRB, 0.43549E+00_JPRB, 0.44431E+00_JPRB, 0.45575E+00_JPRB, 0.47058E+00_JPRB, 0.50156E+00_JPRB ,&
&  0.66341E+00_JPRB, 0.78406E+00_JPRB, 0.12898E+01_JPRB, 0.13654E+01_JPRB, 0.12607E+01_JPRB, 0.11188E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.10882E+01_JPRB, 0.10311E+01_JPRB, 0.96570E+00_JPRB, 0.91872E+00_JPRB, 0.87006E+00_JPRB, 0.83661E+00_JPRB, 0.78586E+00_JPRB ,&
&  0.73866E+00_JPRB, 0.69026E+00_JPRB, 0.63993E+00_JPRB, 0.60382E+00_JPRB, 0.58074E+00_JPRB, 0.57145E+00_JPRB, 0.57772E+00_JPRB ,&
&  0.59152E+00_JPRB, 0.61288E+00_JPRB, 0.67500E+00_JPRB, 0.79087E+00_JPRB, 0.94060E+00_JPRB, 0.96454E+00_JPRB, 0.11816E+01_JPRB ,&
&  0.15028E+01_JPRB, 0.14906E+01_JPRB, 0.13704E+01_JPRB, 0.12685E+01_JPRB, 0.11204E+01_JPRB, 0.10581E+01_JPRB, 0.10385E+01_JPRB ,&
&  0.98553E+00_JPRB, 0.90373E+00_JPRB, 0.81844E+00_JPRB, 0.75425E+00_JPRB, 0.70953E+00_JPRB, 0.68271E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.66115E+00_JPRB, 0.64100E+00_JPRB, 0.61491E+00_JPRB, 0.59353E+00_JPRB, 0.58198E+00_JPRB, 0.57850E+00_JPRB, 0.57832E+00_JPRB ,&
&  0.58006E+00_JPRB, 0.58606E+00_JPRB, 0.61811E+00_JPRB, 0.59780E+00_JPRB, 0.60457E+00_JPRB, 0.62527E+00_JPRB, 0.63642E+00_JPRB ,&
&  0.63670E+00_JPRB, 0.66248E+00_JPRB, 0.68586E+00_JPRB, 0.72363E+00_JPRB, 0.79846E+00_JPRB, 0.82214E+00_JPRB, 0.74189E+00_JPRB ,&
&  0.65856E+00_JPRB, 0.62462E+00_JPRB, 0.62350E+00_JPRB, 0.61789E+00_JPRB, 0.60545E+00_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.54786E+00_JPRB, 0.53059E+00_JPRB, 0.52056E+00_JPRB, 0.50632E+00_JPRB, 0.49343E+00_JPRB, 0.48724E+00_JPRB, 0.49091E+00_JPRB ,&
&  0.50532E+00_JPRB, 0.51840E+00_JPRB, 0.51406E+00_JPRB, 0.51227E+00_JPRB, 0.51085E+00_JPRB, 0.50335E+00_JPRB, 0.49227E+00_JPRB ,&
&  0.48105E+00_JPRB, 0.47035E+00_JPRB, 0.45733E+00_JPRB, 0.44327E+00_JPRB, 0.43374E+00_JPRB, 0.43416E+00_JPRB, 0.43255E+00_JPRB ,&
&  0.43092E+00_JPRB, 0.43506E+00_JPRB, 0.43991E+00_JPRB, 0.44836E+00_JPRB, 0.45685E+00_JPRB, 0.47237E+00_JPRB, 0.51510E+00_JPRB ,&
&  0.65914E+00_JPRB, 0.52047E+00_JPRB, 0.12079E+01_JPRB, 0.15753E+01_JPRB, 0.16414E+01_JPRB, 0.15385E+01_JPRB, 0.14828E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.12871E+01_JPRB, 0.11597E+01_JPRB, 0.10475E+01_JPRB, 0.91187E+00_JPRB, 0.82271E+00_JPRB, 0.76775E+00_JPRB ,&
&  0.72609E+00_JPRB, 0.69016E+00_JPRB, 0.65965E+00_JPRB, 0.63233E+00_JPRB, 0.61216E+00_JPRB, 0.60970E+00_JPRB, 0.62715E+00_JPRB ,&
&  0.65466E+00_JPRB, 0.70054E+00_JPRB, 0.80645E+00_JPRB, 0.94590E+00_JPRB, 0.10072E+01_JPRB, 0.10813E+01_JPRB, 0.12391E+01_JPRB ,&
&  0.15034E+01_JPRB, 0.16605E+01_JPRB, 0.13739E+01_JPRB, 0.11268E+01_JPRB, 0.98197E+00_JPRB, 0.94420E+00_JPRB, 0.94082E+00_JPRB ,&
&  0.90789E+00_JPRB, 0.86680E+00_JPRB, 0.81647E+00_JPRB, 0.76101E+00_JPRB, 0.71332E+00_JPRB, 0.68258E+00_JPRB, 0.66037E+00_JPRB ,&
&  0.63607E+00_JPRB, 0.61025E+00_JPRB, 0.59341E+00_JPRB, 0.58391E+00_JPRB, 0.57967E+00_JPRB, 0.57821E+00_JPRB, 0.58294E+00_JPRB ,&
&  0.59621E+00_JPRB, 0.60654E+00_JPRB, 0.61073E+00_JPRB, 0.59132E+00_JPRB, 0.58506E+00_JPRB, 0.59150E+00_JPRB, 0.60137E+00_JPRB ,&
&  0.64505E+00_JPRB, 0.66310E+00_JPRB, 0.67964E+00_JPRB, 0.72128E+00_JPRB, 0.71584E+00_JPRB, 0.77490E+00_JPRB, 0.71483E+00_JPRB ,&
&  0.65491E+00_JPRB, 0.62544E+00_JPRB, 0.61475E+00_JPRB, 0.60063E+00_JPRB, 0.57574E+00_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.52003E+00_JPRB, 0.50677E+00_JPRB, 0.49676E+00_JPRB, 0.49258E+00_JPRB, 0.48514E+00_JPRB, 0.48307E+00_JPRB, 0.48953E+00_JPRB ,&
&  0.50192E+00_JPRB, 0.50637E+00_JPRB, 0.50534E+00_JPRB, 0.50331E+00_JPRB, 0.49766E+00_JPRB, 0.48911E+00_JPRB, 0.47827E+00_JPRB ,&
&  0.46867E+00_JPRB, 0.45412E+00_JPRB, 0.43512E+00_JPRB, 0.42128E+00_JPRB, 0.42484E+00_JPRB, 0.43003E+00_JPRB, 0.43058E+00_JPRB ,&
&  0.43003E+00_JPRB, 0.43665E+00_JPRB, 0.43975E+00_JPRB, 0.45029E+00_JPRB, 0.45848E+00_JPRB, 0.47987E+00_JPRB, 0.52736E+00_JPRB ,&
&  0.65214E+00_JPRB, 0.10416E+01_JPRB, 0.10207E+01_JPRB, 0.17966E+01_JPRB, 0.20197E+01_JPRB, 0.19565E+01_JPRB, 0.17193E+01_JPRB ,&
&  0.16011E+01_JPRB, 0.15208E+01_JPRB, 0.12893E+01_JPRB, 0.10951E+01_JPRB, 0.96815E+00_JPRB, 0.87184E+00_JPRB, 0.78736E+00_JPRB ,&
&  0.72218E+00_JPRB, 0.68125E+00_JPRB, 0.65530E+00_JPRB, 0.63681E+00_JPRB, 0.63717E+00_JPRB, 0.65480E+00_JPRB, 0.68458E+00_JPRB ,&
&  0.72810E+00_JPRB, 0.80230E+00_JPRB, 0.95250E+00_JPRB, 0.10938E+01_JPRB, 0.10580E+01_JPRB, 0.11539E+01_JPRB, 0.14583E+01_JPRB ,&
&  0.16205E+01_JPRB, 0.16646E+01_JPRB, 0.12325E+01_JPRB, 0.98214E+00_JPRB, 0.89900E+00_JPRB, 0.90394E+00_JPRB, 0.89750E+00_JPRB ,&
&  0.85979E+00_JPRB, 0.83005E+00_JPRB, 0.78529E+00_JPRB, 0.73791E+00_JPRB, 0.70198E+00_JPRB, 0.67492E+00_JPRB, 0.64481E+00_JPRB ,&
&  0.62213E+00_JPRB, 0.61290E+00_JPRB, 0.60765E+00_JPRB, 0.60272E+00_JPRB, 0.60215E+00_JPRB, 0.60702E+00_JPRB, 0.61411E+00_JPRB ,&
&  0.62294E+00_JPRB, 0.61996E+00_JPRB, 0.59856E+00_JPRB, 0.58564E+00_JPRB, 0.59184E+00_JPRB, 0.60333E+00_JPRB, 0.60537E+00_JPRB ,&
&  0.64833E+00_JPRB, 0.67561E+00_JPRB, 0.68455E+00_JPRB, 0.71679E+00_JPRB, 0.74783E+00_JPRB, 0.87981E+00_JPRB, 0.73451E+00_JPRB ,&
&  0.66368E+00_JPRB, 0.62275E+00_JPRB, 0.60159E+00_JPRB, 0.58443E+00_JPRB, 0.55328E+00_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.49265E+00_JPRB, 0.47603E+00_JPRB, 0.47018E+00_JPRB, 0.47199E+00_JPRB, 0.47346E+00_JPRB, 0.47605E+00_JPRB, 0.48494E+00_JPRB ,&
&  0.48910E+00_JPRB, 0.49005E+00_JPRB, 0.48856E+00_JPRB, 0.48603E+00_JPRB, 0.47836E+00_JPRB, 0.46926E+00_JPRB, 0.45913E+00_JPRB ,&
&  0.45148E+00_JPRB, 0.43237E+00_JPRB, 0.42246E+00_JPRB, 0.43009E+00_JPRB, 0.43393E+00_JPRB, 0.43362E+00_JPRB, 0.43490E+00_JPRB ,&
&  0.43581E+00_JPRB, 0.44068E+00_JPRB, 0.44149E+00_JPRB, 0.44889E+00_JPRB, 0.46286E+00_JPRB, 0.49783E+00_JPRB, 0.54983E+00_JPRB ,&
&  0.70878E+00_JPRB, 0.95373E+00_JPRB, 0.98281E+00_JPRB, 0.19261E+01_JPRB, 0.21400E+01_JPRB, 0.20941E+01_JPRB, 0.18772E+01_JPRB ,&
&  0.17439E+01_JPRB, 0.18013E+01_JPRB, 0.14475E+01_JPRB, 0.12084E+01_JPRB, 0.10713E+01_JPRB, 0.94933E+00_JPRB, 0.81944E+00_JPRB ,&
&  0.72056E+00_JPRB, 0.66928E+00_JPRB, 0.65140E+00_JPRB, 0.66289E+00_JPRB, 0.69241E+00_JPRB, 0.72080E+00_JPRB, 0.75253E+00_JPRB ,&
&  0.81642E+00_JPRB, 0.94426E+00_JPRB, 0.11224E+01_JPRB, 0.11208E+01_JPRB, 0.10890E+01_JPRB, 0.11751E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.14929E+01_JPRB, 0.13329E+01_JPRB, 0.10157E+01_JPRB, 0.88672E+00_JPRB, 0.84820E+00_JPRB, 0.86777E+00_JPRB, 0.85358E+00_JPRB ,&
&  0.81339E+00_JPRB, 0.77789E+00_JPRB, 0.73434E+00_JPRB, 0.70423E+00_JPRB, 0.68275E+00_JPRB, 0.65597E+00_JPRB, 0.63876E+00_JPRB ,&
&  0.63150E+00_JPRB, 0.62846E+00_JPRB, 0.63134E+00_JPRB, 0.63102E+00_JPRB, 0.63412E+00_JPRB, 0.63682E+00_JPRB, 0.63679E+00_JPRB ,&
&  0.62956E+00_JPRB, 0.61181E+00_JPRB, 0.60045E+00_JPRB, 0.61372E+00_JPRB, 0.64365E+00_JPRB, 0.65232E+00_JPRB, 0.65006E+00_JPRB ,&
&  0.67678E+00_JPRB, 0.69977E+00_JPRB, 0.71111E+00_JPRB, 0.71233E+00_JPRB, 0.74587E+00_JPRB, 0.80246E+00_JPRB, 0.69601E+00_JPRB ,&
&  0.65024E+00_JPRB, 0.61296E+00_JPRB, 0.58562E+00_JPRB, 0.56562E+00_JPRB, 0.52994E+00_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.49084E+00_JPRB, 0.46548E+00_JPRB, 0.45815E+00_JPRB, 0.45527E+00_JPRB, 0.45708E+00_JPRB, 0.46435E+00_JPRB, 0.47212E+00_JPRB ,&
&  0.47286E+00_JPRB, 0.47271E+00_JPRB, 0.47299E+00_JPRB, 0.46912E+00_JPRB, 0.45781E+00_JPRB, 0.44645E+00_JPRB, 0.43576E+00_JPRB ,&
&  0.43343E+00_JPRB, 0.43544E+00_JPRB, 0.44869E+00_JPRB, 0.45672E+00_JPRB, 0.45783E+00_JPRB, 0.45557E+00_JPRB, 0.45704E+00_JPRB ,&
&  0.46138E+00_JPRB, 0.46141E+00_JPRB, 0.46045E+00_JPRB, 0.47332E+00_JPRB, 0.49920E+00_JPRB, 0.54966E+00_JPRB, 0.63533E+00_JPRB ,&
&  0.87110E+00_JPRB, 0.10273E+01_JPRB, 0.10598E+01_JPRB, 0.16563E+01_JPRB, 0.20282E+01_JPRB, 0.19805E+01_JPRB, 0.19424E+01_JPRB ,&
&  0.18321E+01_JPRB, 0.16485E+01_JPRB, 0.15673E+01_JPRB, 0.12918E+01_JPRB, 0.10981E+01_JPRB, 0.95694E+00_JPRB, 0.85282E+00_JPRB ,&
&  0.77251E+00_JPRB, 0.73576E+00_JPRB, 0.74883E+00_JPRB, 0.78006E+00_JPRB, 0.80101E+00_JPRB, 0.81443E+00_JPRB, 0.86183E+00_JPRB ,&
&  0.95814E+00_JPRB, 0.11233E+01_JPRB, 0.12785E+01_JPRB, 0.12255E+01_JPRB, 0.10618E+01_JPRB, 0.11117E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.10610E+01_JPRB, 0.91048E+00_JPRB, 0.84875E+00_JPRB, 0.79452E+00_JPRB, 0.83359E+00_JPRB, 0.80800E+00_JPRB ,&
&  0.77425E+00_JPRB, 0.73387E+00_JPRB, 0.69643E+00_JPRB, 0.67286E+00_JPRB, 0.65424E+00_JPRB, 0.64107E+00_JPRB, 0.63132E+00_JPRB ,&
&  0.62662E+00_JPRB, 0.62632E+00_JPRB, 0.63748E+00_JPRB, 0.64388E+00_JPRB, 0.64332E+00_JPRB, 0.63076E+00_JPRB, 0.61277E+00_JPRB ,&
&  0.60274E+00_JPRB, 0.61804E+00_JPRB, 0.63446E+00_JPRB, 0.66392E+00_JPRB, 0.69399E+00_JPRB, 0.70876E+00_JPRB, 0.69216E+00_JPRB ,&
&  0.68732E+00_JPRB, 0.71521E+00_JPRB, 0.73260E+00_JPRB, 0.75203E+00_JPRB, 0.75304E+00_JPRB, 0.73944E+00_JPRB, 0.66406E+00_JPRB ,&
&  0.62283E+00_JPRB, 0.59367E+00_JPRB, 0.57626E+00_JPRB, 0.55687E+00_JPRB, 0.51953E+00_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.49722E+00_JPRB, 0.48131E+00_JPRB, 0.44274E+00_JPRB, 0.44636E+00_JPRB, 0.44858E+00_JPRB, 0.45551E+00_JPRB, 0.45448E+00_JPRB ,&
&  0.45320E+00_JPRB, 0.45443E+00_JPRB, 0.45657E+00_JPRB, 0.44823E+00_JPRB, 0.42500E+00_JPRB, 0.41513E+00_JPRB, 0.41842E+00_JPRB ,&
&  0.43002E+00_JPRB, 0.45565E+00_JPRB, 0.48699E+00_JPRB, 0.49297E+00_JPRB, 0.49393E+00_JPRB, 0.49042E+00_JPRB, 0.48726E+00_JPRB ,&
&  0.49088E+00_JPRB, 0.49886E+00_JPRB, 0.51151E+00_JPRB, 0.54081E+00_JPRB, 0.59331E+00_JPRB, 0.67469E+00_JPRB, 0.81019E+00_JPRB ,&
&  0.11449E+01_JPRB, 0.11772E+01_JPRB, 0.59764E+00_JPRB, 0.11146E+01_JPRB, 0.16473E+01_JPRB, 0.17585E+01_JPRB, 0.16971E+01_JPRB ,&
&  0.16197E+01_JPRB, 0.14537E+01_JPRB, 0.13447E+01_JPRB, 0.12267E+01_JPRB, 0.10646E+01_JPRB, 0.97845E+00_JPRB, 0.90713E+00_JPRB ,&
&  0.85233E+00_JPRB, 0.84995E+00_JPRB, 0.88547E+00_JPRB, 0.90519E+00_JPRB, 0.91267E+00_JPRB, 0.95341E+00_JPRB, 0.10447E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.13233E+01_JPRB, 0.14163E+01_JPRB, 0.11939E+01_JPRB, 0.10821E+01_JPRB, 0.10598E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.95702E+00_JPRB, 0.91745E+00_JPRB, 0.86396E+00_JPRB, 0.82134E+00_JPRB, 0.77309E+00_JPRB, 0.75543E+00_JPRB, 0.76792E+00_JPRB ,&
&  0.73479E+00_JPRB, 0.68691E+00_JPRB, 0.65523E+00_JPRB, 0.63660E+00_JPRB, 0.62220E+00_JPRB, 0.61450E+00_JPRB, 0.60683E+00_JPRB ,&
&  0.60563E+00_JPRB, 0.61531E+00_JPRB, 0.62709E+00_JPRB, 0.62654E+00_JPRB, 0.61629E+00_JPRB, 0.60073E+00_JPRB, 0.58984E+00_JPRB ,&
&  0.60428E+00_JPRB, 0.63782E+00_JPRB, 0.68577E+00_JPRB, 0.70449E+00_JPRB, 0.72185E+00_JPRB, 0.74457E+00_JPRB, 0.74729E+00_JPRB ,&
&  0.75441E+00_JPRB, 0.77804E+00_JPRB, 0.72511E+00_JPRB, 0.71699E+00_JPRB, 0.73308E+00_JPRB, 0.69680E+00_JPRB, 0.65960E+00_JPRB ,&
&  0.68033E+00_JPRB, 0.74506E+00_JPRB, 0.86063E+00_JPRB, 0.59635E+00_JPRB, 0.53541E+00_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.51312E+00_JPRB, 0.46467E+00_JPRB, 0.42843E+00_JPRB, 0.42891E+00_JPRB, 0.44909E+00_JPRB, 0.45508E+00_JPRB, 0.45386E+00_JPRB ,&
&  0.44378E+00_JPRB, 0.43228E+00_JPRB, 0.41541E+00_JPRB, 0.40723E+00_JPRB, 0.41152E+00_JPRB, 0.42421E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.49788E+00_JPRB, 0.53319E+00_JPRB, 0.56231E+00_JPRB, 0.56618E+00_JPRB, 0.55147E+00_JPRB, 0.55064E+00_JPRB, 0.56008E+00_JPRB ,&
&  0.57100E+00_JPRB, 0.59878E+00_JPRB, 0.63836E+00_JPRB, 0.69507E+00_JPRB, 0.78151E+00_JPRB, 0.92572E+00_JPRB, 0.11532E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.11086E+01_JPRB, 0.73693E+00_JPRB, 0.13661E+01_JPRB, 0.16246E+01_JPRB, 0.16863E+01_JPRB, 0.14241E+01_JPRB ,&
&  0.13732E+01_JPRB, 0.12873E+01_JPRB, 0.12042E+01_JPRB, 0.11947E+01_JPRB, 0.10628E+01_JPRB, 0.10192E+01_JPRB, 0.97699E+00_JPRB ,&
&  0.95124E+00_JPRB, 0.97397E+00_JPRB, 0.10163E+01_JPRB, 0.10502E+01_JPRB, 0.10927E+01_JPRB, 0.11923E+01_JPRB, 0.13098E+01_JPRB ,&
&  0.14472E+01_JPRB, 0.15484E+01_JPRB, 0.15961E+01_JPRB, 0.12702E+01_JPRB, 0.11112E+01_JPRB, 0.10138E+01_JPRB, 0.95975E+00_JPRB ,&
&  0.87495E+00_JPRB, 0.84426E+00_JPRB, 0.82275E+00_JPRB, 0.78705E+00_JPRB, 0.74873E+00_JPRB, 0.71580E+00_JPRB, 0.72489E+00_JPRB ,&
&  0.67883E+00_JPRB, 0.63459E+00_JPRB, 0.60390E+00_JPRB, 0.58561E+00_JPRB, 0.57947E+00_JPRB, 0.57927E+00_JPRB, 0.57412E+00_JPRB ,&
&  0.58084E+00_JPRB, 0.58656E+00_JPRB, 0.59607E+00_JPRB, 0.59079E+00_JPRB, 0.58635E+00_JPRB, 0.58831E+00_JPRB, 0.58277E+00_JPRB ,&
&  0.61596E+00_JPRB, 0.65968E+00_JPRB, 0.68435E+00_JPRB, 0.71561E+00_JPRB, 0.72088E+00_JPRB, 0.71513E+00_JPRB, 0.73268E+00_JPRB ,&
&  0.72862E+00_JPRB, 0.73748E+00_JPRB, 0.70837E+00_JPRB, 0.71777E+00_JPRB, 0.69541E+00_JPRB, 0.67639E+00_JPRB, 0.67146E+00_JPRB ,&
&  0.70696E+00_JPRB, 0.75413E+00_JPRB, 0.74539E+00_JPRB, 0.59287E+00_JPRB, 0.54109E+00_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.48177E+00_JPRB, 0.44875E+00_JPRB, 0.40957E+00_JPRB, 0.39425E+00_JPRB, 0.41728E+00_JPRB, 0.43647E+00_JPRB, 0.43134E+00_JPRB ,&
&  0.42482E+00_JPRB, 0.41544E+00_JPRB, 0.41118E+00_JPRB, 0.42075E+00_JPRB, 0.45632E+00_JPRB, 0.50859E+00_JPRB, 0.54922E+00_JPRB ,&
&  0.58825E+00_JPRB, 0.61963E+00_JPRB, 0.64482E+00_JPRB, 0.65371E+00_JPRB, 0.64954E+00_JPRB, 0.66076E+00_JPRB, 0.67908E+00_JPRB ,&
&  0.71197E+00_JPRB, 0.76056E+00_JPRB, 0.82589E+00_JPRB, 0.92140E+00_JPRB, 0.10554E+01_JPRB, 0.12194E+01_JPRB, 0.13858E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.51160E+00_JPRB, 0.12808E+01_JPRB, 0.14933E+01_JPRB, 0.14064E+01_JPRB, 0.13918E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11632E+01_JPRB, 0.10877E+01_JPRB, 0.11226E+01_JPRB, 0.10403E+01_JPRB, 0.10475E+01_JPRB, 0.10668E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.11496E+01_JPRB, 0.12161E+01_JPRB, 0.12928E+01_JPRB, 0.13799E+01_JPRB, 0.15216E+01_JPRB, 0.16348E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.18672E+01_JPRB, 0.19048E+01_JPRB, 0.14473E+01_JPRB, 0.12303E+01_JPRB, 0.11136E+01_JPRB, 0.10179E+01_JPRB ,&
&  0.90434E+00_JPRB, 0.77074E+00_JPRB, 0.74245E+00_JPRB, 0.75322E+00_JPRB, 0.72924E+00_JPRB, 0.68034E+00_JPRB, 0.67998E+00_JPRB ,&
&  0.62246E+00_JPRB, 0.57761E+00_JPRB, 0.54590E+00_JPRB, 0.52238E+00_JPRB, 0.52762E+00_JPRB, 0.53453E+00_JPRB, 0.53883E+00_JPRB ,&
&  0.53130E+00_JPRB, 0.53642E+00_JPRB, 0.55628E+00_JPRB, 0.56297E+00_JPRB, 0.57305E+00_JPRB, 0.58782E+00_JPRB, 0.59827E+00_JPRB ,&
&  0.63696E+00_JPRB, 0.65208E+00_JPRB, 0.67127E+00_JPRB, 0.69658E+00_JPRB, 0.70896E+00_JPRB, 0.68972E+00_JPRB, 0.68489E+00_JPRB ,&
&  0.68610E+00_JPRB, 0.68857E+00_JPRB, 0.67501E+00_JPRB, 0.68308E+00_JPRB, 0.67206E+00_JPRB, 0.66569E+00_JPRB, 0.67118E+00_JPRB ,&
&  0.68541E+00_JPRB, 0.68383E+00_JPRB, 0.64815E+00_JPRB, 0.56639E+00_JPRB, 0.52182E+00_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.45819E+00_JPRB, 0.43028E+00_JPRB, 0.39127E+00_JPRB, 0.35675E+00_JPRB, 0.37903E+00_JPRB, 0.37925E+00_JPRB, 0.39427E+00_JPRB ,&
&  0.39985E+00_JPRB, 0.42468E+00_JPRB, 0.45057E+00_JPRB, 0.48907E+00_JPRB, 0.53794E+00_JPRB, 0.58701E+00_JPRB, 0.62830E+00_JPRB ,&
&  0.66589E+00_JPRB, 0.70245E+00_JPRB, 0.72992E+00_JPRB, 0.74160E+00_JPRB, 0.76036E+00_JPRB, 0.79185E+00_JPRB, 0.83011E+00_JPRB ,&
&  0.88236E+00_JPRB, 0.95139E+00_JPRB, 0.10464E+01_JPRB, 0.11661E+01_JPRB, 0.13078E+01_JPRB, 0.13884E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.83335E+00_JPRB, 0.12148E+01_JPRB, 0.14156E+01_JPRB, 0.13294E+01_JPRB, 0.12916E+01_JPRB, 0.12611E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.11564E+01_JPRB, 0.10594E+01_JPRB, 0.10330E+01_JPRB, 0.11304E+01_JPRB, 0.10900E+01_JPRB, 0.11794E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.13288E+01_JPRB, 0.14085E+01_JPRB, 0.14983E+01_JPRB, 0.16082E+01_JPRB, 0.17278E+01_JPRB, 0.18409E+01_JPRB, 0.19527E+01_JPRB ,&
&  0.20934E+01_JPRB, 0.22071E+01_JPRB, 0.22463E+01_JPRB, 0.17739E+01_JPRB, 0.16267E+01_JPRB, 0.15369E+01_JPRB, 0.13784E+01_JPRB ,&
&  0.92115E+00_JPRB, 0.72395E+00_JPRB, 0.72238E+00_JPRB, 0.74043E+00_JPRB, 0.70492E+00_JPRB, 0.67104E+00_JPRB, 0.62976E+00_JPRB ,&
&  0.57897E+00_JPRB, 0.53653E+00_JPRB, 0.51050E+00_JPRB, 0.50113E+00_JPRB, 0.51815E+00_JPRB, 0.51881E+00_JPRB, 0.50313E+00_JPRB ,&
&  0.48475E+00_JPRB, 0.49167E+00_JPRB, 0.52865E+00_JPRB, 0.55840E+00_JPRB, 0.59931E+00_JPRB, 0.62418E+00_JPRB, 0.64514E+00_JPRB ,&
&  0.65446E+00_JPRB, 0.63454E+00_JPRB, 0.64802E+00_JPRB, 0.66433E+00_JPRB, 0.66598E+00_JPRB, 0.66221E+00_JPRB, 0.65120E+00_JPRB ,&
&  0.63963E+00_JPRB, 0.65214E+00_JPRB, 0.65923E+00_JPRB, 0.66672E+00_JPRB, 0.65493E+00_JPRB, 0.63815E+00_JPRB, 0.63177E+00_JPRB ,&
&  0.61683E+00_JPRB, 0.59352E+00_JPRB, 0.56919E+00_JPRB, 0.53007E+00_JPRB, 0.49207E+00_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.44700E+00_JPRB, 0.43141E+00_JPRB, 0.41353E+00_JPRB, 0.40400E+00_JPRB, 0.39624E+00_JPRB, 0.38757E+00_JPRB, 0.39809E+00_JPRB ,&
&  0.42596E+00_JPRB, 0.45128E+00_JPRB, 0.49423E+00_JPRB, 0.55878E+00_JPRB, 0.62025E+00_JPRB, 0.65644E+00_JPRB, 0.69641E+00_JPRB ,&
&  0.73143E+00_JPRB, 0.76970E+00_JPRB, 0.80081E+00_JPRB, 0.83027E+00_JPRB, 0.86897E+00_JPRB, 0.91864E+00_JPRB, 0.98625E+00_JPRB ,&
&  0.10672E+01_JPRB, 0.11494E+01_JPRB, 0.12510E+01_JPRB, 0.13767E+01_JPRB, 0.14994E+01_JPRB, 0.15525E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.13781E+01_JPRB, 0.13684E+01_JPRB, 0.13228E+01_JPRB, 0.12656E+01_JPRB, 0.12152E+01_JPRB, 0.11894E+01_JPRB ,&
&  0.11763E+01_JPRB, 0.11061E+01_JPRB, 0.10980E+01_JPRB, 0.11195E+01_JPRB, 0.12454E+01_JPRB, 0.13381E+01_JPRB, 0.14294E+01_JPRB ,&
&  0.15243E+01_JPRB, 0.16335E+01_JPRB, 0.17466E+01_JPRB, 0.18621E+01_JPRB, 0.19642E+01_JPRB, 0.20865E+01_JPRB, 0.21778E+01_JPRB ,&
&  0.22785E+01_JPRB, 0.23558E+01_JPRB, 0.23820E+01_JPRB, 0.19671E+01_JPRB, 0.16124E+01_JPRB, 0.14144E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.82080E+00_JPRB, 0.70998E+00_JPRB, 0.72783E+00_JPRB, 0.74430E+00_JPRB, 0.69673E+00_JPRB, 0.64899E+00_JPRB, 0.59316E+00_JPRB ,&
&  0.55925E+00_JPRB, 0.50835E+00_JPRB, 0.49056E+00_JPRB, 0.50209E+00_JPRB, 0.53111E+00_JPRB, 0.50888E+00_JPRB, 0.48012E+00_JPRB ,&
&  0.46080E+00_JPRB, 0.45493E+00_JPRB, 0.50015E+00_JPRB, 0.55408E+00_JPRB, 0.62169E+00_JPRB, 0.64740E+00_JPRB, 0.64265E+00_JPRB ,&
&  0.67814E+00_JPRB, 0.60922E+00_JPRB, 0.60420E+00_JPRB, 0.61401E+00_JPRB, 0.61678E+00_JPRB, 0.62345E+00_JPRB, 0.63536E+00_JPRB ,&
&  0.66801E+00_JPRB, 0.72892E+00_JPRB, 0.74727E+00_JPRB, 0.69403E+00_JPRB, 0.64223E+00_JPRB, 0.59526E+00_JPRB, 0.55052E+00_JPRB ,&
&  0.55389E+00_JPRB, 0.54166E+00_JPRB, 0.52814E+00_JPRB, 0.50664E+00_JPRB, 0.47796E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.50532E+00_JPRB, 0.49076E+00_JPRB, 0.48284E+00_JPRB, 0.47721E+00_JPRB, 0.46192E+00_JPRB, 0.46269E+00_JPRB, 0.46010E+00_JPRB ,&
&  0.47721E+00_JPRB, 0.49912E+00_JPRB, 0.53776E+00_JPRB, 0.58513E+00_JPRB, 0.64742E+00_JPRB, 0.69311E+00_JPRB, 0.74821E+00_JPRB ,&
&  0.78298E+00_JPRB, 0.82772E+00_JPRB, 0.87799E+00_JPRB, 0.92235E+00_JPRB, 0.97496E+00_JPRB, 0.10145E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.12055E+01_JPRB, 0.12969E+01_JPRB, 0.13807E+01_JPRB, 0.14501E+01_JPRB, 0.14873E+01_JPRB, 0.11665E+01_JPRB ,&
&  0.13415E+01_JPRB, 0.13548E+01_JPRB, 0.13448E+01_JPRB, 0.12854E+01_JPRB, 0.12059E+01_JPRB, 0.11369E+01_JPRB, 0.11231E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11264E+01_JPRB, 0.11250E+01_JPRB, 0.11538E+01_JPRB, 0.13664E+01_JPRB, 0.14451E+01_JPRB, 0.15377E+01_JPRB ,&
&  0.16401E+01_JPRB, 0.17531E+01_JPRB, 0.18708E+01_JPRB, 0.19973E+01_JPRB, 0.20934E+01_JPRB, 0.22019E+01_JPRB, 0.23139E+01_JPRB ,&
&  0.23855E+01_JPRB, 0.24489E+01_JPRB, 0.23553E+01_JPRB, 0.18628E+01_JPRB, 0.15809E+01_JPRB, 0.13441E+01_JPRB, 0.99460E+00_JPRB ,&
&  0.82076E+00_JPRB, 0.77288E+00_JPRB, 0.78927E+00_JPRB, 0.78724E+00_JPRB, 0.71998E+00_JPRB, 0.65856E+00_JPRB, 0.61646E+00_JPRB ,&
&  0.55844E+00_JPRB, 0.54629E+00_JPRB, 0.53841E+00_JPRB, 0.56950E+00_JPRB, 0.58518E+00_JPRB, 0.56136E+00_JPRB, 0.55046E+00_JPRB ,&
&  0.52716E+00_JPRB, 0.50677E+00_JPRB, 0.53730E+00_JPRB, 0.59262E+00_JPRB, 0.70709E+00_JPRB, 0.80948E+00_JPRB, 0.84689E+00_JPRB ,&
&  0.87894E+00_JPRB, 0.56268E+00_JPRB, 0.56426E+00_JPRB, 0.58058E+00_JPRB, 0.61612E+00_JPRB, 0.66295E+00_JPRB, 0.75214E+00_JPRB ,&
&  0.94794E+00_JPRB, 0.12966E+01_JPRB, 0.11498E+01_JPRB, 0.87808E+00_JPRB, 0.95574E+00_JPRB, 0.87835E+00_JPRB, 0.48933E+00_JPRB ,&
&  0.49162E+00_JPRB, 0.49142E+00_JPRB, 0.50733E+00_JPRB, 0.51214E+00_JPRB, 0.51503E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.59079E+00_JPRB, 0.58552E+00_JPRB, 0.58512E+00_JPRB, 0.57588E+00_JPRB, 0.57147E+00_JPRB, 0.57261E+00_JPRB, 0.56118E+00_JPRB ,&
&  0.57022E+00_JPRB, 0.58748E+00_JPRB, 0.59427E+00_JPRB, 0.61423E+00_JPRB, 0.62658E+00_JPRB, 0.66586E+00_JPRB, 0.72351E+00_JPRB ,&
&  0.76456E+00_JPRB, 0.85359E+00_JPRB, 0.90077E+00_JPRB, 0.95748E+00_JPRB, 0.10064E+01_JPRB, 0.10586E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.12432E+01_JPRB, 0.13253E+01_JPRB, 0.13852E+01_JPRB, 0.14004E+01_JPRB, 0.15713E+01_JPRB, 0.12235E+01_JPRB ,&
&  0.13269E+01_JPRB, 0.13074E+01_JPRB, 0.12174E+01_JPRB, 0.11657E+01_JPRB, 0.11155E+01_JPRB, 0.10872E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11081E+01_JPRB, 0.11239E+01_JPRB, 0.12247E+01_JPRB, 0.13395E+01_JPRB, 0.14231E+01_JPRB, 0.15202E+01_JPRB ,&
&  0.16116E+01_JPRB, 0.16953E+01_JPRB, 0.18070E+01_JPRB, 0.19259E+01_JPRB, 0.20524E+01_JPRB, 0.21384E+01_JPRB, 0.22247E+01_JPRB ,&
&  0.22584E+01_JPRB, 0.22859E+01_JPRB, 0.21268E+01_JPRB, 0.18752E+01_JPRB, 0.16186E+01_JPRB, 0.13546E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.89411E+00_JPRB, 0.85889E+00_JPRB, 0.87188E+00_JPRB, 0.89990E+00_JPRB, 0.81773E+00_JPRB, 0.76160E+00_JPRB, 0.72620E+00_JPRB ,&
&  0.68786E+00_JPRB, 0.63502E+00_JPRB, 0.65915E+00_JPRB, 0.70725E+00_JPRB, 0.72251E+00_JPRB, 0.71242E+00_JPRB, 0.69273E+00_JPRB ,&
&  0.66251E+00_JPRB, 0.61988E+00_JPRB, 0.61657E+00_JPRB, 0.62562E+00_JPRB, 0.65176E+00_JPRB, 0.70190E+00_JPRB, 0.61749E+00_JPRB ,&
&  0.67284E+00_JPRB, 0.55076E+00_JPRB, 0.55477E+00_JPRB, 0.59050E+00_JPRB, 0.65197E+00_JPRB, 0.83436E+00_JPRB, 0.11623E+01_JPRB ,&
&  0.17534E+01_JPRB, 0.20041E+01_JPRB, 0.22520E+01_JPRB, 0.28008E+01_JPRB, 0.24466E+01_JPRB, 0.18094E+01_JPRB, 0.54380E+00_JPRB ,&
&  0.54203E+00_JPRB, 0.54687E+00_JPRB, 0.57082E+00_JPRB, 0.57975E+00_JPRB, 0.58463E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.63971E+00_JPRB, 0.64996E+00_JPRB, 0.66354E+00_JPRB, 0.67803E+00_JPRB, 0.69589E+00_JPRB, 0.71415E+00_JPRB, 0.72834E+00_JPRB ,&
&  0.74389E+00_JPRB, 0.76323E+00_JPRB, 0.78175E+00_JPRB, 0.79470E+00_JPRB, 0.80787E+00_JPRB, 0.83250E+00_JPRB, 0.86135E+00_JPRB ,&
&  0.87416E+00_JPRB, 0.92083E+00_JPRB, 0.94438E+00_JPRB, 0.99001E+00_JPRB, 0.10333E+01_JPRB, 0.10855E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.12199E+01_JPRB, 0.12439E+01_JPRB, 0.12376E+01_JPRB, 0.12590E+01_JPRB, 0.13756E+01_JPRB, 0.11361E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.12555E+01_JPRB, 0.11204E+01_JPRB, 0.10545E+01_JPRB, 0.10022E+01_JPRB, 0.98360E+00_JPRB, 0.98952E+00_JPRB ,&
&  0.10185E+01_JPRB, 0.10982E+01_JPRB, 0.10725E+01_JPRB, 0.10937E+01_JPRB, 0.11384E+01_JPRB, 0.12686E+01_JPRB, 0.13955E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.15197E+01_JPRB, 0.15519E+01_JPRB, 0.15919E+01_JPRB, 0.16695E+01_JPRB, 0.17441E+01_JPRB, 0.17873E+01_JPRB ,&
&  0.17601E+01_JPRB, 0.17147E+01_JPRB, 0.15773E+01_JPRB, 0.16609E+01_JPRB, 0.15885E+01_JPRB, 0.13609E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.91012E+00_JPRB, 0.96080E+00_JPRB, 0.10249E+01_JPRB, 0.11119E+01_JPRB, 0.10515E+01_JPRB, 0.99900E+00_JPRB, 0.87881E+00_JPRB ,&
&  0.82987E+00_JPRB, 0.81893E+00_JPRB, 0.86242E+00_JPRB, 0.92305E+00_JPRB, 0.95937E+00_JPRB, 0.94699E+00_JPRB, 0.88061E+00_JPRB ,&
&  0.81542E+00_JPRB, 0.74216E+00_JPRB, 0.70612E+00_JPRB, 0.69772E+00_JPRB, 0.67838E+00_JPRB, 0.62757E+00_JPRB, 0.77282E+00_JPRB ,&
&  0.62138E+00_JPRB, 0.55715E+00_JPRB, 0.63756E+00_JPRB, 0.43070E+00_JPRB, 0.61359E+00_JPRB, 0.78268E+00_JPRB, 0.10468E+01_JPRB ,&
&  0.12517E+01_JPRB, 0.17624E+01_JPRB, 0.22138E+01_JPRB, 0.16856E+01_JPRB, 0.11695E+01_JPRB, 0.67995E+00_JPRB, 0.58915E+00_JPRB ,&
&  0.59832E+00_JPRB, 0.60390E+00_JPRB, 0.61321E+00_JPRB, 0.61405E+00_JPRB, 0.62684E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.65800E+00_JPRB, 0.67232E+00_JPRB, 0.69133E+00_JPRB, 0.71672E+00_JPRB, 0.74105E+00_JPRB, 0.76002E+00_JPRB, 0.77311E+00_JPRB ,&
&  0.79487E+00_JPRB, 0.82422E+00_JPRB, 0.84239E+00_JPRB, 0.85098E+00_JPRB, 0.86980E+00_JPRB, 0.88657E+00_JPRB, 0.91644E+00_JPRB ,&
&  0.93589E+00_JPRB, 0.97933E+00_JPRB, 0.10140E+01_JPRB, 0.10485E+01_JPRB, 0.10764E+01_JPRB, 0.11091E+01_JPRB, 0.11400E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.12165E+01_JPRB, 0.12602E+01_JPRB, 0.12892E+01_JPRB, 0.13393E+01_JPRB, 0.14069E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.13941E+01_JPRB, 0.12900E+01_JPRB, 0.11286E+01_JPRB, 0.10026E+01_JPRB, 0.91843E+00_JPRB, 0.85500E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.82265E+00_JPRB, 0.85292E+00_JPRB, 0.87887E+00_JPRB, 0.85726E+00_JPRB, 0.81731E+00_JPRB, 0.89664E+00_JPRB, 0.10620E+01_JPRB ,&
&  0.11638E+01_JPRB, 0.12137E+01_JPRB, 0.12547E+01_JPRB, 0.12572E+01_JPRB, 0.13079E+01_JPRB, 0.13711E+01_JPRB, 0.13892E+01_JPRB ,&
&  0.13791E+01_JPRB, 0.13281E+01_JPRB, 0.11976E+01_JPRB, 0.14035E+01_JPRB, 0.14165E+01_JPRB, 0.13800E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.11179E+01_JPRB, 0.11710E+01_JPRB, 0.12929E+01_JPRB, 0.12571E+01_JPRB, 0.12290E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.10819E+01_JPRB, 0.10589E+01_JPRB, 0.10798E+01_JPRB, 0.11259E+01_JPRB, 0.11585E+01_JPRB, 0.11461E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.96320E+00_JPRB, 0.88349E+00_JPRB, 0.83319E+00_JPRB, 0.80090E+00_JPRB, 0.80161E+00_JPRB, 0.87116E+00_JPRB, 0.13230E+01_JPRB ,&
&  0.72186E+00_JPRB, 0.47429E+00_JPRB, 0.50301E+00_JPRB, 0.54560E+00_JPRB, 0.57232E+00_JPRB, 0.62084E+00_JPRB, 0.78095E+00_JPRB ,&
&  0.99044E+00_JPRB, 0.11373E+01_JPRB, 0.11137E+01_JPRB, 0.85150E+00_JPRB, 0.67880E+00_JPRB, 0.62640E+00_JPRB, 0.62211E+00_JPRB ,&
&  0.63175E+00_JPRB, 0.63048E+00_JPRB, 0.63381E+00_JPRB, 0.63970E+00_JPRB, 0.64963E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.63158E+00_JPRB, 0.63214E+00_JPRB, 0.64091E+00_JPRB, 0.65716E+00_JPRB, 0.66570E+00_JPRB, 0.67244E+00_JPRB, 0.67795E+00_JPRB ,&
&  0.69767E+00_JPRB, 0.72015E+00_JPRB, 0.74053E+00_JPRB, 0.76016E+00_JPRB, 0.77832E+00_JPRB, 0.79357E+00_JPRB, 0.83052E+00_JPRB ,&
&  0.85038E+00_JPRB, 0.87842E+00_JPRB, 0.89721E+00_JPRB, 0.91740E+00_JPRB, 0.93491E+00_JPRB, 0.92794E+00_JPRB, 0.93472E+00_JPRB ,&
&  0.94710E+00_JPRB, 0.97955E+00_JPRB, 0.10123E+01_JPRB, 0.10689E+01_JPRB, 0.11430E+01_JPRB, 0.11637E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.12777E+01_JPRB, 0.14263E+01_JPRB, 0.12346E+01_JPRB, 0.10514E+01_JPRB, 0.10018E+01_JPRB, 0.89296E+00_JPRB, 0.83010E+00_JPRB ,&
&  0.82545E+00_JPRB, 0.80345E+00_JPRB, 0.76579E+00_JPRB, 0.72362E+00_JPRB, 0.67931E+00_JPRB, 0.70774E+00_JPRB, 0.74274E+00_JPRB ,&
&  0.87080E+00_JPRB, 0.95073E+00_JPRB, 0.10668E+01_JPRB, 0.11857E+01_JPRB, 0.13103E+01_JPRB, 0.13712E+01_JPRB, 0.13387E+01_JPRB ,&
&  0.13352E+01_JPRB, 0.13122E+01_JPRB, 0.12693E+01_JPRB, 0.14129E+01_JPRB, 0.15316E+01_JPRB, 0.15977E+01_JPRB, 0.16310E+01_JPRB ,&
&  0.16424E+01_JPRB, 0.15269E+01_JPRB, 0.14991E+01_JPRB, 0.14953E+01_JPRB, 0.14443E+01_JPRB, 0.14577E+01_JPRB, 0.13816E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.12304E+01_JPRB, 0.12063E+01_JPRB, 0.12076E+01_JPRB, 0.12178E+01_JPRB, 0.11916E+01_JPRB, 0.10792E+01_JPRB ,&
&  0.10098E+01_JPRB, 0.96437E+00_JPRB, 0.93751E+00_JPRB, 0.91799E+00_JPRB, 0.89592E+00_JPRB, 0.15828E+01_JPRB, 0.23634E+01_JPRB ,&
&  0.90303E+00_JPRB, 0.67213E+00_JPRB, 0.49866E+00_JPRB, 0.63991E+00_JPRB, 0.58566E+00_JPRB, 0.60623E+00_JPRB, 0.67558E+00_JPRB ,&
&  0.68912E+00_JPRB, 0.67593E+00_JPRB, 0.61236E+00_JPRB, 0.55087E+00_JPRB, 0.56890E+00_JPRB, 0.57638E+00_JPRB, 0.57809E+00_JPRB ,&
&  0.59315E+00_JPRB, 0.60023E+00_JPRB, 0.61061E+00_JPRB, 0.63108E+00_JPRB, 0.62635E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.61704E+00_JPRB, 0.60736E+00_JPRB, 0.59752E+00_JPRB, 0.58691E+00_JPRB, 0.59362E+00_JPRB, 0.59531E+00_JPRB, 0.60580E+00_JPRB ,&
&  0.61801E+00_JPRB, 0.61905E+00_JPRB, 0.62614E+00_JPRB, 0.62908E+00_JPRB, 0.64557E+00_JPRB, 0.65987E+00_JPRB, 0.69023E+00_JPRB ,&
&  0.71963E+00_JPRB, 0.73540E+00_JPRB, 0.75546E+00_JPRB, 0.77538E+00_JPRB, 0.78860E+00_JPRB, 0.80322E+00_JPRB, 0.78910E+00_JPRB ,&
&  0.77246E+00_JPRB, 0.77506E+00_JPRB, 0.81309E+00_JPRB, 0.95083E+00_JPRB, 0.12102E+01_JPRB, 0.15407E+01_JPRB, 0.21855E+01_JPRB ,&
&  0.21294E+01_JPRB, 0.14853E+01_JPRB, 0.13094E+01_JPRB, 0.11472E+01_JPRB, 0.11958E+01_JPRB, 0.12792E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.12594E+01_JPRB, 0.12469E+01_JPRB, 0.12280E+01_JPRB, 0.11976E+01_JPRB, 0.11261E+01_JPRB, 0.10361E+01_JPRB, 0.96833E+00_JPRB ,&
&  0.93650E+00_JPRB, 0.90273E+00_JPRB, 0.10276E+01_JPRB, 0.13507E+01_JPRB, 0.16087E+01_JPRB, 0.20160E+01_JPRB, 0.19168E+01_JPRB ,&
&  0.19342E+01_JPRB, 0.19972E+01_JPRB, 0.20137E+01_JPRB, 0.20904E+01_JPRB, 0.21009E+01_JPRB, 0.21631E+01_JPRB, 0.21689E+01_JPRB ,&
&  0.22926E+01_JPRB, 0.20405E+01_JPRB, 0.14707E+01_JPRB, 0.16746E+01_JPRB, 0.16344E+01_JPRB, 0.16435E+01_JPRB, 0.15471E+01_JPRB ,&
&  0.14615E+01_JPRB, 0.13537E+01_JPRB, 0.12680E+01_JPRB, 0.11965E+01_JPRB, 0.11626E+01_JPRB, 0.11013E+01_JPRB, 0.99458E+00_JPRB ,&
&  0.97086E+00_JPRB, 0.96406E+00_JPRB, 0.95874E+00_JPRB, 0.92355E+00_JPRB, 0.99631E+00_JPRB, 0.10084E+01_JPRB, 0.14057E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.95904E+00_JPRB, 0.83053E+00_JPRB, 0.11148E+01_JPRB, 0.74345E+00_JPRB, 0.59913E+00_JPRB, 0.59625E+00_JPRB ,&
&  0.57627E+00_JPRB, 0.55553E+00_JPRB, 0.53927E+00_JPRB, 0.54011E+00_JPRB, 0.55569E+00_JPRB, 0.58589E+00_JPRB, 0.58889E+00_JPRB ,&
&  0.59588E+00_JPRB, 0.60491E+00_JPRB, 0.62055E+00_JPRB, 0.64021E+00_JPRB, 0.63518E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.67217E+00_JPRB, 0.67402E+00_JPRB, 0.67721E+00_JPRB, 0.68634E+00_JPRB, 0.68725E+00_JPRB, 0.68344E+00_JPRB, 0.66888E+00_JPRB ,&
&  0.65858E+00_JPRB, 0.64271E+00_JPRB, 0.63479E+00_JPRB, 0.63262E+00_JPRB, 0.65313E+00_JPRB, 0.69477E+00_JPRB, 0.75220E+00_JPRB ,&
&  0.76810E+00_JPRB, 0.76360E+00_JPRB, 0.72837E+00_JPRB, 0.70291E+00_JPRB, 0.71995E+00_JPRB, 0.71584E+00_JPRB, 0.68664E+00_JPRB ,&
&  0.66755E+00_JPRB, 0.73840E+00_JPRB, 0.92878E+00_JPRB, 0.12175E+01_JPRB, 0.17359E+01_JPRB, 0.20188E+01_JPRB, 0.25410E+01_JPRB ,&
&  0.23479E+01_JPRB, 0.16730E+01_JPRB, 0.17311E+01_JPRB, 0.16436E+01_JPRB, 0.17483E+01_JPRB, 0.18081E+01_JPRB, 0.18319E+01_JPRB ,&
&  0.18836E+01_JPRB, 0.19270E+01_JPRB, 0.19679E+01_JPRB, 0.20173E+01_JPRB, 0.20395E+01_JPRB, 0.20180E+01_JPRB, 0.19944E+01_JPRB ,&
&  0.19638E+01_JPRB, 0.19069E+01_JPRB, 0.19204E+01_JPRB, 0.20061E+01_JPRB, 0.22555E+01_JPRB, 0.25337E+01_JPRB, 0.25923E+01_JPRB ,&
&  0.26444E+01_JPRB, 0.28109E+01_JPRB, 0.27573E+01_JPRB, 0.29238E+01_JPRB, 0.29291E+01_JPRB, 0.28327E+01_JPRB, 0.30115E+01_JPRB ,&
&  0.30735E+01_JPRB, 0.25727E+01_JPRB, 0.17054E+01_JPRB, 0.18779E+01_JPRB, 0.18006E+01_JPRB, 0.16626E+01_JPRB, 0.16902E+01_JPRB ,&
&  0.15604E+01_JPRB, 0.14273E+01_JPRB, 0.12992E+01_JPRB, 0.12068E+01_JPRB, 0.11644E+01_JPRB, 0.10828E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.99617E+00_JPRB, 0.96330E+00_JPRB, 0.93454E+00_JPRB, 0.90783E+00_JPRB, 0.89942E+00_JPRB, 0.94140E+00_JPRB, 0.10093E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.11373E+01_JPRB, 0.14140E+01_JPRB, 0.16020E+01_JPRB, 0.92807E+00_JPRB, 0.72960E+00_JPRB, 0.67039E+00_JPRB ,&
&  0.67394E+00_JPRB, 0.66844E+00_JPRB, 0.66368E+00_JPRB, 0.65405E+00_JPRB, 0.68391E+00_JPRB, 0.73350E+00_JPRB, 0.73327E+00_JPRB ,&
&  0.74003E+00_JPRB, 0.70513E+00_JPRB, 0.69569E+00_JPRB, 0.69589E+00_JPRB, 0.69390E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.84689E+00_JPRB, 0.84083E+00_JPRB, 0.82944E+00_JPRB, 0.84474E+00_JPRB, 0.86062E+00_JPRB, 0.86077E+00_JPRB, 0.87726E+00_JPRB ,&
&  0.90035E+00_JPRB, 0.92497E+00_JPRB, 0.94696E+00_JPRB, 0.96691E+00_JPRB, 0.10378E+01_JPRB, 0.11174E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.12594E+01_JPRB, 0.13012E+01_JPRB, 0.13031E+01_JPRB, 0.12710E+01_JPRB, 0.12423E+01_JPRB, 0.11727E+01_JPRB, 0.10997E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.14967E+01_JPRB, 0.20319E+01_JPRB, 0.25440E+01_JPRB, 0.30284E+01_JPRB, 0.25881E+01_JPRB, 0.27589E+01_JPRB ,&
&  0.24022E+01_JPRB, 0.19025E+01_JPRB, 0.18732E+01_JPRB, 0.19640E+01_JPRB, 0.20676E+01_JPRB, 0.20808E+01_JPRB, 0.21278E+01_JPRB ,&
&  0.21573E+01_JPRB, 0.21890E+01_JPRB, 0.22395E+01_JPRB, 0.23490E+01_JPRB, 0.24249E+01_JPRB, 0.24785E+01_JPRB, 0.25469E+01_JPRB ,&
&  0.25960E+01_JPRB, 0.25856E+01_JPRB, 0.25670E+01_JPRB, 0.23697E+01_JPRB, 0.24774E+01_JPRB, 0.27159E+01_JPRB, 0.29465E+01_JPRB ,&
&  0.31022E+01_JPRB, 0.31769E+01_JPRB, 0.34648E+01_JPRB, 0.35375E+01_JPRB, 0.34741E+01_JPRB, 0.34308E+01_JPRB, 0.35124E+01_JPRB ,&
&  0.34496E+01_JPRB, 0.30762E+01_JPRB, 0.21723E+01_JPRB, 0.24327E+01_JPRB, 0.22384E+01_JPRB, 0.18268E+01_JPRB, 0.18408E+01_JPRB ,&
&  0.16525E+01_JPRB, 0.14699E+01_JPRB, 0.12510E+01_JPRB, 0.12092E+01_JPRB, 0.12625E+01_JPRB, 0.13157E+01_JPRB, 0.14417E+01_JPRB ,&
&  0.12823E+01_JPRB, 0.10826E+01_JPRB, 0.89173E+00_JPRB, 0.86075E+00_JPRB, 0.85704E+00_JPRB, 0.93014E+00_JPRB, 0.11493E+01_JPRB ,&
&  0.11651E+01_JPRB, 0.11168E+01_JPRB, 0.12466E+01_JPRB, 0.14877E+01_JPRB, 0.11125E+01_JPRB, 0.88707E+00_JPRB, 0.85447E+00_JPRB ,&
&  0.86031E+00_JPRB, 0.87059E+00_JPRB, 0.87744E+00_JPRB, 0.87620E+00_JPRB, 0.89103E+00_JPRB, 0.92003E+00_JPRB, 0.94239E+00_JPRB ,&
&  0.95032E+00_JPRB, 0.92369E+00_JPRB, 0.89300E+00_JPRB, 0.87150E+00_JPRB, 0.85330E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.10170E+01_JPRB, 0.97023E+00_JPRB, 0.93551E+00_JPRB, 0.92370E+00_JPRB, 0.92474E+00_JPRB, 0.93742E+00_JPRB, 0.96648E+00_JPRB ,&
&  0.10203E+01_JPRB, 0.10912E+01_JPRB, 0.11656E+01_JPRB, 0.12353E+01_JPRB, 0.13275E+01_JPRB, 0.14340E+01_JPRB, 0.15640E+01_JPRB ,&
&  0.17048E+01_JPRB, 0.18537E+01_JPRB, 0.19273E+01_JPRB, 0.19431E+01_JPRB, 0.19969E+01_JPRB, 0.20773E+01_JPRB, 0.23101E+01_JPRB ,&
&  0.26851E+01_JPRB, 0.30115E+01_JPRB, 0.31756E+01_JPRB, 0.29433E+01_JPRB, 0.22695E+01_JPRB, 0.22308E+01_JPRB, 0.24361E+01_JPRB ,&
&  0.25126E+01_JPRB, 0.22367E+01_JPRB, 0.20271E+01_JPRB, 0.20141E+01_JPRB, 0.20319E+01_JPRB, 0.20891E+01_JPRB, 0.21272E+01_JPRB ,&
&  0.21457E+01_JPRB, 0.21832E+01_JPRB, 0.22729E+01_JPRB, 0.23720E+01_JPRB, 0.25117E+01_JPRB, 0.26555E+01_JPRB, 0.28044E+01_JPRB ,&
&  0.28677E+01_JPRB, 0.29286E+01_JPRB, 0.28691E+01_JPRB, 0.27412E+01_JPRB, 0.27059E+01_JPRB, 0.28262E+01_JPRB, 0.30617E+01_JPRB ,&
&  0.33142E+01_JPRB, 0.34404E+01_JPRB, 0.37018E+01_JPRB, 0.38771E+01_JPRB, 0.37321E+01_JPRB, 0.34952E+01_JPRB, 0.33824E+01_JPRB ,&
&  0.33427E+01_JPRB, 0.35296E+01_JPRB, 0.24982E+01_JPRB, 0.30528E+01_JPRB, 0.31011E+01_JPRB, 0.26244E+01_JPRB, 0.21779E+01_JPRB ,&
&  0.19594E+01_JPRB, 0.17494E+01_JPRB, 0.15396E+01_JPRB, 0.15907E+01_JPRB, 0.16238E+01_JPRB, 0.16701E+01_JPRB, 0.16965E+01_JPRB ,&
&  0.16395E+01_JPRB, 0.13073E+01_JPRB, 0.99684E+00_JPRB, 0.82842E+00_JPRB, 0.91157E+00_JPRB, 0.11237E+01_JPRB, 0.11683E+01_JPRB ,&
&  0.10932E+01_JPRB, 0.12477E+01_JPRB, 0.14065E+01_JPRB, 0.17695E+01_JPRB, 0.18537E+01_JPRB, 0.10260E+01_JPRB, 0.96627E+00_JPRB ,&
&  0.95778E+00_JPRB, 0.99478E+00_JPRB, 0.10071E+01_JPRB, 0.10177E+01_JPRB, 0.10319E+01_JPRB, 0.10660E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11477E+01_JPRB, 0.11469E+01_JPRB, 0.11171E+01_JPRB, 0.10633E+01_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.11660E+01_JPRB, 0.11381E+01_JPRB, 0.11149E+01_JPRB, 0.10876E+01_JPRB, 0.10528E+01_JPRB, 0.10268E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.10455E+01_JPRB, 0.11006E+01_JPRB, 0.11902E+01_JPRB, 0.13007E+01_JPRB, 0.14423E+01_JPRB, 0.16076E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.19980E+01_JPRB, 0.22170E+01_JPRB, 0.23527E+01_JPRB, 0.24586E+01_JPRB, 0.25869E+01_JPRB, 0.27903E+01_JPRB, 0.30125E+01_JPRB ,&
&  0.31965E+01_JPRB, 0.31422E+01_JPRB, 0.28736E+01_JPRB, 0.21182E+01_JPRB, 0.18356E+01_JPRB, 0.21571E+01_JPRB, 0.21258E+01_JPRB ,&
&  0.21077E+01_JPRB, 0.20408E+01_JPRB, 0.20012E+01_JPRB, 0.19657E+01_JPRB, 0.19681E+01_JPRB, 0.20189E+01_JPRB, 0.20613E+01_JPRB ,&
&  0.20675E+01_JPRB, 0.20912E+01_JPRB, 0.21672E+01_JPRB, 0.23191E+01_JPRB, 0.25291E+01_JPRB, 0.26473E+01_JPRB, 0.27670E+01_JPRB ,&
&  0.28624E+01_JPRB, 0.30120E+01_JPRB, 0.30543E+01_JPRB, 0.30513E+01_JPRB, 0.29636E+01_JPRB, 0.29806E+01_JPRB, 0.31747E+01_JPRB ,&
&  0.34187E+01_JPRB, 0.36169E+01_JPRB, 0.38239E+01_JPRB, 0.40631E+01_JPRB, 0.41349E+01_JPRB, 0.35779E+01_JPRB, 0.33283E+01_JPRB ,&
&  0.33188E+01_JPRB, 0.32705E+01_JPRB, 0.29650E+01_JPRB, 0.38337E+01_JPRB, 0.23877E+01_JPRB, 0.25426E+01_JPRB, 0.24491E+01_JPRB ,&
&  0.22554E+01_JPRB, 0.22432E+01_JPRB, 0.22928E+01_JPRB, 0.23148E+01_JPRB, 0.21879E+01_JPRB, 0.18554E+01_JPRB, 0.19588E+01_JPRB ,&
&  0.18316E+01_JPRB, 0.14194E+01_JPRB, 0.11094E+01_JPRB, 0.81127E+00_JPRB, 0.95034E+00_JPRB, 0.13478E+01_JPRB, 0.12934E+01_JPRB ,&
&  0.11298E+01_JPRB, 0.12502E+01_JPRB, 0.14655E+01_JPRB, 0.17822E+01_JPRB, 0.15990E+01_JPRB, 0.10664E+01_JPRB, 0.10044E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.10042E+01_JPRB, 0.10147E+01_JPRB, 0.10234E+01_JPRB, 0.10565E+01_JPRB, 0.11071E+01_JPRB, 0.11561E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.11794E+01_JPRB, 0.12089E+01_JPRB, 0.12133E+01_JPRB, 0.11963E+01_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.12203E+01_JPRB, 0.12134E+01_JPRB, 0.12263E+01_JPRB, 0.12323E+01_JPRB, 0.12316E+01_JPRB, 0.11774E+01_JPRB, 0.11327E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.10320E+01_JPRB, 0.11092E+01_JPRB, 0.12123E+01_JPRB, 0.13386E+01_JPRB, 0.14743E+01_JPRB, 0.16265E+01_JPRB ,&
&  0.18186E+01_JPRB, 0.20565E+01_JPRB, 0.23565E+01_JPRB, 0.26138E+01_JPRB, 0.28704E+01_JPRB, 0.32170E+01_JPRB, 0.34775E+01_JPRB ,&
&  0.29919E+01_JPRB, 0.19878E+01_JPRB, 0.23446E+01_JPRB, 0.25095E+01_JPRB, 0.23718E+01_JPRB, 0.24693E+01_JPRB, 0.23708E+01_JPRB ,&
&  0.21184E+01_JPRB, 0.20249E+01_JPRB, 0.20180E+01_JPRB, 0.19878E+01_JPRB, 0.19080E+01_JPRB, 0.19418E+01_JPRB, 0.19941E+01_JPRB ,&
&  0.20265E+01_JPRB, 0.20275E+01_JPRB, 0.20515E+01_JPRB, 0.21754E+01_JPRB, 0.23829E+01_JPRB, 0.24849E+01_JPRB, 0.26505E+01_JPRB ,&
&  0.28024E+01_JPRB, 0.30001E+01_JPRB, 0.30581E+01_JPRB, 0.30092E+01_JPRB, 0.30591E+01_JPRB, 0.31500E+01_JPRB, 0.33083E+01_JPRB ,&
&  0.35164E+01_JPRB, 0.36350E+01_JPRB, 0.38018E+01_JPRB, 0.40760E+01_JPRB, 0.41876E+01_JPRB, 0.36769E+01_JPRB, 0.36733E+01_JPRB ,&
&  0.35693E+01_JPRB, 0.33788E+01_JPRB, 0.32381E+01_JPRB, 0.38670E+01_JPRB, 0.28707E+01_JPRB, 0.28074E+01_JPRB, 0.24301E+01_JPRB ,&
&  0.25629E+01_JPRB, 0.26192E+01_JPRB, 0.26556E+01_JPRB, 0.25951E+01_JPRB, 0.24136E+01_JPRB, 0.20565E+01_JPRB, 0.20306E+01_JPRB ,&
&  0.17518E+01_JPRB, 0.15461E+01_JPRB, 0.11499E+01_JPRB, 0.87730E+00_JPRB, 0.84087E+00_JPRB, 0.11736E+01_JPRB, 0.13587E+01_JPRB ,&
&  0.15290E+01_JPRB, 0.13240E+01_JPRB, 0.13643E+01_JPRB, 0.16172E+01_JPRB, 0.14137E+01_JPRB, 0.11047E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.10178E+01_JPRB, 0.10011E+01_JPRB, 0.95277E+00_JPRB, 0.95433E+00_JPRB, 0.10011E+01_JPRB, 0.10744E+01_JPRB, 0.11252E+01_JPRB ,&
&  0.11494E+01_JPRB, 0.11417E+01_JPRB, 0.11533E+01_JPRB, 0.11730E+01_JPRB, 0.12147E+01_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.12116E+01_JPRB, 0.12253E+01_JPRB, 0.12365E+01_JPRB, 0.12682E+01_JPRB, 0.13195E+01_JPRB, 0.13931E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.10042E+01_JPRB, 0.10212E+01_JPRB, 0.10500E+01_JPRB, 0.11215E+01_JPRB, 0.11890E+01_JPRB, 0.12787E+01_JPRB ,&
&  0.14168E+01_JPRB, 0.16055E+01_JPRB, 0.19126E+01_JPRB, 0.23226E+01_JPRB, 0.27875E+01_JPRB, 0.33553E+01_JPRB, 0.33891E+01_JPRB ,&
&  0.29585E+01_JPRB, 0.23754E+01_JPRB, 0.23965E+01_JPRB, 0.23509E+01_JPRB, 0.22564E+01_JPRB, 0.25030E+01_JPRB, 0.24566E+01_JPRB ,&
&  0.20926E+01_JPRB, 0.19535E+01_JPRB, 0.19430E+01_JPRB, 0.18716E+01_JPRB, 0.18009E+01_JPRB, 0.18166E+01_JPRB, 0.18715E+01_JPRB ,&
&  0.19374E+01_JPRB, 0.19893E+01_JPRB, 0.20217E+01_JPRB, 0.20764E+01_JPRB, 0.21849E+01_JPRB, 0.22899E+01_JPRB, 0.24784E+01_JPRB ,&
&  0.26969E+01_JPRB, 0.28949E+01_JPRB, 0.29845E+01_JPRB, 0.29523E+01_JPRB, 0.30572E+01_JPRB, 0.32642E+01_JPRB, 0.35039E+01_JPRB ,&
&  0.37073E+01_JPRB, 0.39609E+01_JPRB, 0.41136E+01_JPRB, 0.41526E+01_JPRB, 0.38709E+01_JPRB, 0.42527E+01_JPRB, 0.39048E+01_JPRB ,&
&  0.38934E+01_JPRB, 0.35882E+01_JPRB, 0.38989E+01_JPRB, 0.31106E+01_JPRB, 0.33330E+01_JPRB, 0.31994E+01_JPRB, 0.29256E+01_JPRB ,&
&  0.27715E+01_JPRB, 0.30442E+01_JPRB, 0.30262E+01_JPRB, 0.27936E+01_JPRB, 0.26382E+01_JPRB, 0.21974E+01_JPRB, 0.21460E+01_JPRB ,&
&  0.16718E+01_JPRB, 0.16672E+01_JPRB, 0.12002E+01_JPRB, 0.90565E+00_JPRB, 0.75342E+00_JPRB, 0.87458E+00_JPRB, 0.11754E+01_JPRB ,&
&  0.16247E+01_JPRB, 0.16929E+01_JPRB, 0.15010E+01_JPRB, 0.14360E+01_JPRB, 0.12704E+01_JPRB, 0.11073E+01_JPRB, 0.10695E+01_JPRB ,&
&  0.10619E+01_JPRB, 0.10125E+01_JPRB, 0.94803E+00_JPRB, 0.94014E+00_JPRB, 0.96811E+00_JPRB, 0.10464E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11606E+01_JPRB, 0.11614E+01_JPRB, 0.11690E+01_JPRB, 0.11849E+01_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.12224E+01_JPRB, 0.12208E+01_JPRB, 0.11838E+01_JPRB, 0.11367E+01_JPRB, 0.11020E+01_JPRB, 0.10776E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.10152E+01_JPRB, 0.10108E+01_JPRB, 0.99568E+00_JPRB, 0.10039E+01_JPRB, 0.10160E+01_JPRB, 0.10458E+01_JPRB ,&
&  0.10889E+01_JPRB, 0.11478E+01_JPRB, 0.13321E+01_JPRB, 0.17179E+01_JPRB, 0.21892E+01_JPRB, 0.27958E+01_JPRB, 0.27395E+01_JPRB ,&
&  0.23886E+01_JPRB, 0.32697E+01_JPRB, 0.22227E+01_JPRB, 0.22738E+01_JPRB, 0.23107E+01_JPRB, 0.24958E+01_JPRB, 0.21986E+01_JPRB ,&
&  0.19036E+01_JPRB, 0.18497E+01_JPRB, 0.18757E+01_JPRB, 0.18288E+01_JPRB, 0.18281E+01_JPRB, 0.18115E+01_JPRB, 0.18208E+01_JPRB ,&
&  0.18618E+01_JPRB, 0.19087E+01_JPRB, 0.19384E+01_JPRB, 0.19794E+01_JPRB, 0.20524E+01_JPRB, 0.21350E+01_JPRB, 0.22699E+01_JPRB ,&
&  0.24822E+01_JPRB, 0.27229E+01_JPRB, 0.28413E+01_JPRB, 0.29807E+01_JPRB, 0.31615E+01_JPRB, 0.35206E+01_JPRB, 0.38139E+01_JPRB ,&
&  0.38936E+01_JPRB, 0.38760E+01_JPRB, 0.42981E+01_JPRB, 0.44436E+01_JPRB, 0.44768E+01_JPRB, 0.47800E+01_JPRB, 0.42851E+01_JPRB ,&
&  0.43999E+01_JPRB, 0.39208E+01_JPRB, 0.42153E+01_JPRB, 0.35012E+01_JPRB, 0.35528E+01_JPRB, 0.36941E+01_JPRB, 0.32052E+01_JPRB ,&
&  0.30247E+01_JPRB, 0.33262E+01_JPRB, 0.32760E+01_JPRB, 0.30628E+01_JPRB, 0.27703E+01_JPRB, 0.25107E+01_JPRB, 0.22405E+01_JPRB ,&
&  0.19261E+01_JPRB, 0.18172E+01_JPRB, 0.16388E+01_JPRB, 0.11128E+01_JPRB, 0.96486E+00_JPRB, 0.11368E+01_JPRB, 0.13832E+01_JPRB ,&
&  0.19017E+01_JPRB, 0.21259E+01_JPRB, 0.24056E+01_JPRB, 0.20955E+01_JPRB, 0.14633E+01_JPRB, 0.12000E+01_JPRB, 0.11405E+01_JPRB ,&
&  0.11066E+01_JPRB, 0.10837E+01_JPRB, 0.10226E+01_JPRB, 0.97934E+00_JPRB, 0.10029E+01_JPRB, 0.10695E+01_JPRB, 0.11331E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12564E+01_JPRB, 0.12573E+01_JPRB, 0.12322E+01_JPRB, 0.12272E+01_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.13249E+01_JPRB, 0.13055E+01_JPRB, 0.12783E+01_JPRB, 0.12108E+01_JPRB, 0.11467E+01_JPRB, 0.10801E+01_JPRB, 0.10642E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.10210E+01_JPRB, 0.10154E+01_JPRB, 0.99140E+00_JPRB, 0.97981E+00_JPRB, 0.95997E+00_JPRB, 0.96865E+00_JPRB ,&
&  0.98025E+00_JPRB, 0.95206E+00_JPRB, 0.10036E+01_JPRB, 0.12358E+01_JPRB, 0.15956E+01_JPRB, 0.22785E+01_JPRB, 0.18528E+01_JPRB ,&
&  0.24878E+01_JPRB, 0.30446E+01_JPRB, 0.23319E+01_JPRB, 0.22948E+01_JPRB, 0.23579E+01_JPRB, 0.23780E+01_JPRB, 0.19372E+01_JPRB ,&
&  0.17682E+01_JPRB, 0.17501E+01_JPRB, 0.17760E+01_JPRB, 0.18307E+01_JPRB, 0.19218E+01_JPRB, 0.19893E+01_JPRB, 0.19521E+01_JPRB ,&
&  0.19452E+01_JPRB, 0.19588E+01_JPRB, 0.19562E+01_JPRB, 0.19407E+01_JPRB, 0.19613E+01_JPRB, 0.20241E+01_JPRB, 0.21079E+01_JPRB ,&
&  0.22447E+01_JPRB, 0.24577E+01_JPRB, 0.27795E+01_JPRB, 0.28548E+01_JPRB, 0.31452E+01_JPRB, 0.36932E+01_JPRB, 0.40289E+01_JPRB ,&
&  0.40124E+01_JPRB, 0.39511E+01_JPRB, 0.45449E+01_JPRB, 0.50920E+01_JPRB, 0.53299E+01_JPRB, 0.53942E+01_JPRB, 0.49760E+01_JPRB ,&
&  0.48472E+01_JPRB, 0.44526E+01_JPRB, 0.42484E+01_JPRB, 0.36041E+01_JPRB, 0.36392E+01_JPRB, 0.40206E+01_JPRB, 0.38748E+01_JPRB ,&
&  0.41149E+01_JPRB, 0.38288E+01_JPRB, 0.36793E+01_JPRB, 0.32722E+01_JPRB, 0.28475E+01_JPRB, 0.26179E+01_JPRB, 0.26158E+01_JPRB ,&
&  0.25915E+01_JPRB, 0.26928E+01_JPRB, 0.25592E+01_JPRB, 0.16308E+01_JPRB, 0.13054E+01_JPRB, 0.13676E+01_JPRB, 0.17728E+01_JPRB ,&
&  0.25842E+01_JPRB, 0.27840E+01_JPRB, 0.26152E+01_JPRB, 0.31410E+01_JPRB, 0.21757E+01_JPRB, 0.16385E+01_JPRB, 0.13838E+01_JPRB ,&
&  0.12605E+01_JPRB, 0.12463E+01_JPRB, 0.12098E+01_JPRB, 0.11464E+01_JPRB, 0.11519E+01_JPRB, 0.12077E+01_JPRB, 0.12846E+01_JPRB ,&
&  0.13599E+01_JPRB, 0.14187E+01_JPRB, 0.14493E+01_JPRB, 0.14150E+01_JPRB, 0.13625E+01_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.13982E+01_JPRB, 0.13743E+01_JPRB, 0.13293E+01_JPRB, 0.12604E+01_JPRB, 0.12011E+01_JPRB, 0.11202E+01_JPRB, 0.10549E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.10225E+01_JPRB, 0.10050E+01_JPRB, 0.98721E+00_JPRB, 0.97033E+00_JPRB, 0.95709E+00_JPRB, 0.95852E+00_JPRB ,&
&  0.95647E+00_JPRB, 0.92523E+00_JPRB, 0.95967E+00_JPRB, 0.10798E+01_JPRB, 0.13116E+01_JPRB, 0.14625E+01_JPRB, 0.17033E+01_JPRB ,&
&  0.26966E+01_JPRB, 0.30229E+01_JPRB, 0.24787E+01_JPRB, 0.26338E+01_JPRB, 0.29040E+01_JPRB, 0.31806E+01_JPRB, 0.23449E+01_JPRB ,&
&  0.20378E+01_JPRB, 0.18959E+01_JPRB, 0.17968E+01_JPRB, 0.18991E+01_JPRB, 0.20414E+01_JPRB, 0.21471E+01_JPRB, 0.21560E+01_JPRB ,&
&  0.20581E+01_JPRB, 0.19873E+01_JPRB, 0.19613E+01_JPRB, 0.19206E+01_JPRB, 0.18981E+01_JPRB, 0.19530E+01_JPRB, 0.20229E+01_JPRB ,&
&  0.21060E+01_JPRB, 0.22621E+01_JPRB, 0.25714E+01_JPRB, 0.27463E+01_JPRB, 0.30471E+01_JPRB, 0.35515E+01_JPRB, 0.41316E+01_JPRB ,&
&  0.43534E+01_JPRB, 0.47077E+01_JPRB, 0.50751E+01_JPRB, 0.53929E+01_JPRB, 0.58334E+01_JPRB, 0.63128E+01_JPRB, 0.58995E+01_JPRB ,&
&  0.55439E+01_JPRB, 0.46839E+01_JPRB, 0.39566E+01_JPRB, 0.37167E+01_JPRB, 0.38063E+01_JPRB, 0.43832E+01_JPRB, 0.41683E+01_JPRB ,&
&  0.38898E+01_JPRB, 0.31443E+01_JPRB, 0.29996E+01_JPRB, 0.32642E+01_JPRB, 0.30504E+01_JPRB, 0.28581E+01_JPRB, 0.32926E+01_JPRB ,&
&  0.28651E+01_JPRB, 0.16185E+01_JPRB, 0.13507E+01_JPRB, 0.16280E+01_JPRB, 0.11489E+01_JPRB, 0.11582E+01_JPRB, 0.22717E+01_JPRB ,&
&  0.30409E+01_JPRB, 0.37260E+01_JPRB, 0.35821E+01_JPRB, 0.30641E+01_JPRB, 0.28129E+01_JPRB, 0.21998E+01_JPRB, 0.18585E+01_JPRB ,&
&  0.15498E+01_JPRB, 0.14636E+01_JPRB, 0.14151E+01_JPRB, 0.13444E+01_JPRB, 0.13330E+01_JPRB, 0.14087E+01_JPRB, 0.14999E+01_JPRB ,&
&  0.15644E+01_JPRB, 0.15857E+01_JPRB, 0.15645E+01_JPRB, 0.15121E+01_JPRB, 0.14417E+01_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.14578E+01_JPRB, 0.14206E+01_JPRB, 0.13895E+01_JPRB, 0.13300E+01_JPRB, 0.12667E+01_JPRB, 0.11905E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.10427E+01_JPRB, 0.10220E+01_JPRB, 0.10173E+01_JPRB, 0.99361E+00_JPRB, 0.97250E+00_JPRB, 0.96660E+00_JPRB, 0.97303E+00_JPRB ,&
&  0.96188E+00_JPRB, 0.94940E+00_JPRB, 0.10485E+01_JPRB, 0.11695E+01_JPRB, 0.12406E+01_JPRB, 0.12866E+01_JPRB, 0.17134E+01_JPRB ,&
&  0.25728E+01_JPRB, 0.31332E+01_JPRB, 0.34806E+01_JPRB, 0.36721E+01_JPRB, 0.39023E+01_JPRB, 0.42081E+01_JPRB, 0.36084E+01_JPRB ,&
&  0.29478E+01_JPRB, 0.24480E+01_JPRB, 0.21120E+01_JPRB, 0.21138E+01_JPRB, 0.22674E+01_JPRB, 0.23724E+01_JPRB, 0.23064E+01_JPRB ,&
&  0.22023E+01_JPRB, 0.20517E+01_JPRB, 0.19505E+01_JPRB, 0.18605E+01_JPRB, 0.18469E+01_JPRB, 0.18974E+01_JPRB, 0.19589E+01_JPRB ,&
&  0.20090E+01_JPRB, 0.20831E+01_JPRB, 0.23352E+01_JPRB, 0.27485E+01_JPRB, 0.26579E+01_JPRB, 0.34306E+01_JPRB, 0.40511E+01_JPRB ,&
&  0.46118E+01_JPRB, 0.49427E+01_JPRB, 0.49864E+01_JPRB, 0.58425E+01_JPRB, 0.66018E+01_JPRB, 0.69368E+01_JPRB, 0.66146E+01_JPRB ,&
&  0.64893E+01_JPRB, 0.54169E+01_JPRB, 0.45900E+01_JPRB, 0.41956E+01_JPRB, 0.41968E+01_JPRB, 0.47152E+01_JPRB, 0.33997E+01_JPRB ,&
&  0.33681E+01_JPRB, 0.40483E+01_JPRB, 0.35665E+01_JPRB, 0.31030E+01_JPRB, 0.33209E+01_JPRB, 0.32463E+01_JPRB, 0.23777E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.82374E+00_JPRB, 0.83027E+00_JPRB, 0.80870E+00_JPRB, 0.75455E+00_JPRB, 0.11805E+01_JPRB, 0.38841E+01_JPRB ,&
&  0.32740E+01_JPRB, 0.42900E+01_JPRB, 0.40662E+01_JPRB, 0.34676E+01_JPRB, 0.30251E+01_JPRB, 0.25367E+01_JPRB, 0.24319E+01_JPRB ,&
&  0.20356E+01_JPRB, 0.17849E+01_JPRB, 0.16552E+01_JPRB, 0.15718E+01_JPRB, 0.15655E+01_JPRB, 0.15989E+01_JPRB, 0.16966E+01_JPRB ,&
&  0.17563E+01_JPRB, 0.17683E+01_JPRB, 0.17083E+01_JPRB, 0.16132E+01_JPRB, 0.15237E+01_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.15374E+01_JPRB, 0.14555E+01_JPRB, 0.13981E+01_JPRB, 0.13585E+01_JPRB, 0.13182E+01_JPRB, 0.12489E+01_JPRB, 0.11337E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.10177E+01_JPRB, 0.10123E+01_JPRB, 0.10023E+01_JPRB, 0.99505E+00_JPRB, 0.99417E+00_JPRB, 0.99001E+00_JPRB ,&
&  0.96673E+00_JPRB, 0.99505E+00_JPRB, 0.11766E+01_JPRB, 0.12228E+01_JPRB, 0.12448E+01_JPRB, 0.12051E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.24455E+01_JPRB, 0.31602E+01_JPRB, 0.35933E+01_JPRB, 0.37241E+01_JPRB, 0.47616E+01_JPRB, 0.53986E+01_JPRB, 0.53693E+01_JPRB ,&
&  0.44731E+01_JPRB, 0.36835E+01_JPRB, 0.29765E+01_JPRB, 0.25984E+01_JPRB, 0.25715E+01_JPRB, 0.25811E+01_JPRB, 0.25189E+01_JPRB ,&
&  0.23394E+01_JPRB, 0.21913E+01_JPRB, 0.20449E+01_JPRB, 0.19238E+01_JPRB, 0.18727E+01_JPRB, 0.18924E+01_JPRB, 0.19598E+01_JPRB ,&
&  0.19865E+01_JPRB, 0.19734E+01_JPRB, 0.20815E+01_JPRB, 0.25326E+01_JPRB, 0.29889E+01_JPRB, 0.31727E+01_JPRB, 0.38817E+01_JPRB ,&
&  0.46570E+01_JPRB, 0.52863E+01_JPRB, 0.52970E+01_JPRB, 0.66069E+01_JPRB, 0.70041E+01_JPRB, 0.72572E+01_JPRB, 0.76062E+01_JPRB ,&
&  0.73569E+01_JPRB, 0.71178E+01_JPRB, 0.52242E+01_JPRB, 0.46355E+01_JPRB, 0.48052E+01_JPRB, 0.40371E+01_JPRB, 0.39745E+01_JPRB ,&
&  0.46986E+01_JPRB, 0.40727E+01_JPRB, 0.34071E+01_JPRB, 0.28730E+01_JPRB, 0.34727E+01_JPRB, 0.28794E+01_JPRB, 0.12699E+01_JPRB ,&
&  0.98119E+00_JPRB, 0.90952E+00_JPRB, 0.79780E+00_JPRB, 0.77477E+00_JPRB, 0.82642E+00_JPRB, 0.12138E+01_JPRB, 0.24462E+01_JPRB ,&
&  0.28617E+01_JPRB, 0.38992E+01_JPRB, 0.39330E+01_JPRB, 0.36064E+01_JPRB, 0.30670E+01_JPRB, 0.29368E+01_JPRB, 0.30503E+01_JPRB ,&
&  0.27310E+01_JPRB, 0.24117E+01_JPRB, 0.20420E+01_JPRB, 0.18153E+01_JPRB, 0.17163E+01_JPRB, 0.17364E+01_JPRB, 0.18484E+01_JPRB ,&
&  0.18989E+01_JPRB, 0.18966E+01_JPRB, 0.18354E+01_JPRB, 0.17307E+01_JPRB, 0.16205E+01_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.15847E+01_JPRB, 0.15162E+01_JPRB, 0.14432E+01_JPRB, 0.13726E+01_JPRB, 0.13140E+01_JPRB, 0.12623E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.10504E+01_JPRB, 0.10306E+01_JPRB, 0.10101E+01_JPRB, 0.10009E+01_JPRB, 0.10040E+01_JPRB, 0.99662E+00_JPRB ,&
&  0.98184E+00_JPRB, 0.10360E+01_JPRB, 0.11442E+01_JPRB, 0.11088E+01_JPRB, 0.10940E+01_JPRB, 0.10670E+01_JPRB, 0.13315E+01_JPRB ,&
&  0.21729E+01_JPRB, 0.29937E+01_JPRB, 0.34341E+01_JPRB, 0.39520E+01_JPRB, 0.50434E+01_JPRB, 0.52189E+01_JPRB, 0.59764E+01_JPRB ,&
&  0.54354E+01_JPRB, 0.47221E+01_JPRB, 0.41104E+01_JPRB, 0.36336E+01_JPRB, 0.32903E+01_JPRB, 0.30078E+01_JPRB, 0.28140E+01_JPRB ,&
&  0.25972E+01_JPRB, 0.23701E+01_JPRB, 0.22559E+01_JPRB, 0.21142E+01_JPRB, 0.20217E+01_JPRB, 0.19803E+01_JPRB, 0.20379E+01_JPRB ,&
&  0.20432E+01_JPRB, 0.19904E+01_JPRB, 0.19391E+01_JPRB, 0.22106E+01_JPRB, 0.30540E+01_JPRB, 0.36800E+01_JPRB, 0.38541E+01_JPRB ,&
&  0.40436E+01_JPRB, 0.43338E+01_JPRB, 0.55165E+01_JPRB, 0.64411E+01_JPRB, 0.72108E+01_JPRB, 0.80296E+01_JPRB, 0.82537E+01_JPRB ,&
&  0.79749E+01_JPRB, 0.72966E+01_JPRB, 0.56873E+01_JPRB, 0.51875E+01_JPRB, 0.46479E+01_JPRB, 0.40091E+01_JPRB, 0.48131E+01_JPRB ,&
&  0.52541E+01_JPRB, 0.46644E+01_JPRB, 0.36918E+01_JPRB, 0.29089E+01_JPRB, 0.26957E+01_JPRB, 0.18183E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.11056E+01_JPRB, 0.10163E+01_JPRB, 0.91911E+00_JPRB, 0.99837E+00_JPRB, 0.98017E+00_JPRB, 0.13281E+01_JPRB, 0.21616E+01_JPRB ,&
&  0.30813E+01_JPRB, 0.37451E+01_JPRB, 0.42365E+01_JPRB, 0.37638E+01_JPRB, 0.33500E+01_JPRB, 0.34695E+01_JPRB, 0.37524E+01_JPRB ,&
&  0.29010E+01_JPRB, 0.27358E+01_JPRB, 0.25882E+01_JPRB, 0.22339E+01_JPRB, 0.20148E+01_JPRB, 0.19316E+01_JPRB, 0.19541E+01_JPRB ,&
&  0.19942E+01_JPRB, 0.19625E+01_JPRB, 0.18796E+01_JPRB, 0.17605E+01_JPRB, 0.16577E+01_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.15775E+01_JPRB, 0.14941E+01_JPRB, 0.14202E+01_JPRB, 0.13459E+01_JPRB, 0.12762E+01_JPRB, 0.12242E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.11354E+01_JPRB, 0.10860E+01_JPRB, 0.10357E+01_JPRB, 0.10024E+01_JPRB, 0.98738E+00_JPRB, 0.98056E+00_JPRB, 0.96970E+00_JPRB ,&
&  0.97164E+00_JPRB, 0.10469E+01_JPRB, 0.88897E+00_JPRB, 0.84534E+00_JPRB, 0.93273E+00_JPRB, 0.99384E+00_JPRB, 0.10888E+01_JPRB ,&
&  0.17235E+01_JPRB, 0.25929E+01_JPRB, 0.31228E+01_JPRB, 0.41456E+01_JPRB, 0.62244E+01_JPRB, 0.59221E+01_JPRB, 0.60522E+01_JPRB ,&
&  0.61405E+01_JPRB, 0.52148E+01_JPRB, 0.46360E+01_JPRB, 0.41007E+01_JPRB, 0.36580E+01_JPRB, 0.32686E+01_JPRB, 0.28654E+01_JPRB ,&
&  0.25294E+01_JPRB, 0.23092E+01_JPRB, 0.21883E+01_JPRB, 0.21568E+01_JPRB, 0.21332E+01_JPRB, 0.21326E+01_JPRB, 0.21306E+01_JPRB ,&
&  0.21146E+01_JPRB, 0.20636E+01_JPRB, 0.19683E+01_JPRB, 0.21447E+01_JPRB, 0.27097E+01_JPRB, 0.33046E+01_JPRB, 0.45357E+01_JPRB ,&
&  0.44717E+01_JPRB, 0.48618E+01_JPRB, 0.49898E+01_JPRB, 0.67529E+01_JPRB, 0.79143E+01_JPRB, 0.82275E+01_JPRB, 0.90857E+01_JPRB ,&
&  0.70549E+01_JPRB, 0.59740E+01_JPRB, 0.54008E+01_JPRB, 0.48637E+01_JPRB, 0.45775E+01_JPRB, 0.53710E+01_JPRB, 0.75703E+01_JPRB ,&
&  0.57944E+01_JPRB, 0.60188E+01_JPRB, 0.60752E+01_JPRB, 0.55575E+01_JPRB, 0.28208E+01_JPRB, 0.19910E+01_JPRB, 0.32009E+01_JPRB ,&
&  0.33480E+01_JPRB, 0.19543E+01_JPRB, 0.14033E+01_JPRB, 0.18332E+01_JPRB, 0.13915E+01_JPRB, 0.16372E+01_JPRB, 0.22949E+01_JPRB ,&
&  0.25545E+01_JPRB, 0.31353E+01_JPRB, 0.46922E+01_JPRB, 0.43884E+01_JPRB, 0.40327E+01_JPRB, 0.39425E+01_JPRB, 0.36873E+01_JPRB ,&
&  0.32702E+01_JPRB, 0.27798E+01_JPRB, 0.26524E+01_JPRB, 0.22837E+01_JPRB, 0.20663E+01_JPRB, 0.19337E+01_JPRB, 0.18860E+01_JPRB ,&
&  0.18971E+01_JPRB, 0.18820E+01_JPRB, 0.18255E+01_JPRB, 0.17352E+01_JPRB, 0.16511E+01_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.15775E+01_JPRB, 0.14909E+01_JPRB, 0.14015E+01_JPRB, 0.13301E+01_JPRB, 0.12651E+01_JPRB, 0.12051E+01_JPRB, 0.11549E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.10839E+01_JPRB, 0.10260E+01_JPRB, 0.98545E+00_JPRB, 0.96512E+00_JPRB, 0.96720E+00_JPRB, 0.96177E+00_JPRB ,&
&  0.96833E+00_JPRB, 0.93211E+00_JPRB, 0.78392E+00_JPRB, 0.73656E+00_JPRB, 0.86799E+00_JPRB, 0.85875E+00_JPRB, 0.92047E+00_JPRB ,&
&  0.13356E+01_JPRB, 0.21554E+01_JPRB, 0.30350E+01_JPRB, 0.40983E+01_JPRB, 0.55921E+01_JPRB, 0.64391E+01_JPRB, 0.62408E+01_JPRB ,&
&  0.63487E+01_JPRB, 0.54945E+01_JPRB, 0.48350E+01_JPRB, 0.42887E+01_JPRB, 0.37315E+01_JPRB, 0.31567E+01_JPRB, 0.26303E+01_JPRB ,&
&  0.23175E+01_JPRB, 0.21141E+01_JPRB, 0.19957E+01_JPRB, 0.18950E+01_JPRB, 0.19148E+01_JPRB, 0.19845E+01_JPRB, 0.20683E+01_JPRB ,&
&  0.20923E+01_JPRB, 0.20696E+01_JPRB, 0.20313E+01_JPRB, 0.20993E+01_JPRB, 0.26385E+01_JPRB, 0.33807E+01_JPRB, 0.39329E+01_JPRB ,&
&  0.43338E+01_JPRB, 0.47324E+01_JPRB, 0.52366E+01_JPRB, 0.62636E+01_JPRB, 0.79917E+01_JPRB, 0.82784E+01_JPRB, 0.98684E+01_JPRB ,&
&  0.77982E+01_JPRB, 0.64393E+01_JPRB, 0.52608E+01_JPRB, 0.39974E+01_JPRB, 0.49733E+01_JPRB, 0.81275E+01_JPRB, 0.79067E+01_JPRB ,&
&  0.72685E+01_JPRB, 0.71316E+01_JPRB, 0.66444E+01_JPRB, 0.53561E+01_JPRB, 0.44339E+01_JPRB, 0.34882E+01_JPRB, 0.42292E+01_JPRB ,&
&  0.41242E+01_JPRB, 0.38825E+01_JPRB, 0.34474E+01_JPRB, 0.28391E+01_JPRB, 0.23456E+01_JPRB, 0.25542E+01_JPRB, 0.23425E+01_JPRB ,&
&  0.21904E+01_JPRB, 0.23939E+01_JPRB, 0.39924E+01_JPRB, 0.48015E+01_JPRB, 0.44942E+01_JPRB, 0.38051E+01_JPRB, 0.40501E+01_JPRB ,&
&  0.34567E+01_JPRB, 0.30071E+01_JPRB, 0.25787E+01_JPRB, 0.22219E+01_JPRB, 0.20328E+01_JPRB, 0.19206E+01_JPRB, 0.18568E+01_JPRB ,&
&  0.18019E+01_JPRB, 0.17719E+01_JPRB, 0.17248E+01_JPRB, 0.16718E+01_JPRB, 0.16266E+01_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.15580E+01_JPRB, 0.14802E+01_JPRB, 0.14009E+01_JPRB, 0.13334E+01_JPRB, 0.12634E+01_JPRB, 0.12012E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10604E+01_JPRB, 0.10289E+01_JPRB, 0.10027E+01_JPRB, 0.98710E+00_JPRB, 0.97523E+00_JPRB, 0.97845E+00_JPRB ,&
&  0.99501E+00_JPRB, 0.93818E+00_JPRB, 0.75691E+00_JPRB, 0.72096E+00_JPRB, 0.74540E+00_JPRB, 0.73189E+00_JPRB, 0.85400E+00_JPRB ,&
&  0.11374E+01_JPRB, 0.17212E+01_JPRB, 0.24599E+01_JPRB, 0.33958E+01_JPRB, 0.44503E+01_JPRB, 0.52934E+01_JPRB, 0.54680E+01_JPRB ,&
&  0.48532E+01_JPRB, 0.47863E+01_JPRB, 0.41919E+01_JPRB, 0.36550E+01_JPRB, 0.30766E+01_JPRB, 0.26367E+01_JPRB, 0.22960E+01_JPRB ,&
&  0.20040E+01_JPRB, 0.18134E+01_JPRB, 0.17270E+01_JPRB, 0.17051E+01_JPRB, 0.17346E+01_JPRB, 0.17984E+01_JPRB, 0.18979E+01_JPRB ,&
&  0.19403E+01_JPRB, 0.19484E+01_JPRB, 0.19103E+01_JPRB, 0.20589E+01_JPRB, 0.24163E+01_JPRB, 0.33354E+01_JPRB, 0.36823E+01_JPRB ,&
&  0.46623E+01_JPRB, 0.54826E+01_JPRB, 0.58998E+01_JPRB, 0.76372E+01_JPRB, 0.67534E+01_JPRB, 0.84159E+01_JPRB, 0.10688E+02_JPRB ,&
&  0.95092E+01_JPRB, 0.83894E+01_JPRB, 0.80281E+01_JPRB, 0.66003E+01_JPRB, 0.56856E+01_JPRB, 0.79849E+01_JPRB, 0.74030E+01_JPRB ,&
&  0.66876E+01_JPRB, 0.65225E+01_JPRB, 0.59607E+01_JPRB, 0.57103E+01_JPRB, 0.50396E+01_JPRB, 0.45013E+01_JPRB, 0.33721E+01_JPRB ,&
&  0.33873E+01_JPRB, 0.30763E+01_JPRB, 0.33021E+01_JPRB, 0.29568E+01_JPRB, 0.27380E+01_JPRB, 0.25165E+01_JPRB, 0.22441E+01_JPRB ,&
&  0.20735E+01_JPRB, 0.20493E+01_JPRB, 0.26806E+01_JPRB, 0.34487E+01_JPRB, 0.38020E+01_JPRB, 0.33025E+01_JPRB, 0.35717E+01_JPRB ,&
&  0.33842E+01_JPRB, 0.30465E+01_JPRB, 0.26000E+01_JPRB, 0.24045E+01_JPRB, 0.22247E+01_JPRB, 0.20703E+01_JPRB, 0.19523E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.17523E+01_JPRB, 0.16906E+01_JPRB, 0.16587E+01_JPRB, 0.16160E+01_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.14652E+01_JPRB, 0.13978E+01_JPRB, 0.13313E+01_JPRB, 0.12806E+01_JPRB, 0.12390E+01_JPRB, 0.11946E+01_JPRB, 0.11288E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10241E+01_JPRB, 0.10070E+01_JPRB, 0.98880E+00_JPRB, 0.97685E+00_JPRB, 0.97674E+00_JPRB, 0.98297E+00_JPRB ,&
&  0.10052E+01_JPRB, 0.11068E+01_JPRB, 0.87978E+00_JPRB, 0.71999E+00_JPRB, 0.63816E+00_JPRB, 0.71256E+00_JPRB, 0.88494E+00_JPRB ,&
&  0.11191E+01_JPRB, 0.15144E+01_JPRB, 0.21407E+01_JPRB, 0.26508E+01_JPRB, 0.32139E+01_JPRB, 0.37901E+01_JPRB, 0.39871E+01_JPRB ,&
&  0.43079E+01_JPRB, 0.34400E+01_JPRB, 0.31126E+01_JPRB, 0.25780E+01_JPRB, 0.22129E+01_JPRB, 0.19735E+01_JPRB, 0.17567E+01_JPRB ,&
&  0.15687E+01_JPRB, 0.14682E+01_JPRB, 0.14117E+01_JPRB, 0.14197E+01_JPRB, 0.14401E+01_JPRB, 0.14880E+01_JPRB, 0.15717E+01_JPRB ,&
&  0.16702E+01_JPRB, 0.17252E+01_JPRB, 0.17086E+01_JPRB, 0.17728E+01_JPRB, 0.22388E+01_JPRB, 0.28821E+01_JPRB, 0.34133E+01_JPRB ,&
&  0.35585E+01_JPRB, 0.40626E+01_JPRB, 0.74712E+01_JPRB, 0.85956E+01_JPRB, 0.91871E+01_JPRB, 0.10001E+02_JPRB, 0.10138E+02_JPRB ,&
&  0.93444E+01_JPRB, 0.85613E+01_JPRB, 0.81450E+01_JPRB, 0.71397E+01_JPRB, 0.79580E+01_JPRB, 0.78477E+01_JPRB, 0.73702E+01_JPRB ,&
&  0.66121E+01_JPRB, 0.61890E+01_JPRB, 0.57161E+01_JPRB, 0.54155E+01_JPRB, 0.52854E+01_JPRB, 0.53488E+01_JPRB, 0.49702E+01_JPRB ,&
&  0.41963E+01_JPRB, 0.44218E+01_JPRB, 0.37285E+01_JPRB, 0.29989E+01_JPRB, 0.24331E+01_JPRB, 0.22613E+01_JPRB, 0.21158E+01_JPRB ,&
&  0.19478E+01_JPRB, 0.18884E+01_JPRB, 0.20889E+01_JPRB, 0.23390E+01_JPRB, 0.26543E+01_JPRB, 0.28316E+01_JPRB, 0.27298E+01_JPRB ,&
&  0.24905E+01_JPRB, 0.27050E+01_JPRB, 0.27007E+01_JPRB, 0.25486E+01_JPRB, 0.24335E+01_JPRB, 0.22366E+01_JPRB, 0.20633E+01_JPRB ,&
&  0.18839E+01_JPRB, 0.17667E+01_JPRB, 0.16848E+01_JPRB, 0.16138E+01_JPRB, 0.15392E+01_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.13032E+01_JPRB, 0.12578E+01_JPRB, 0.12173E+01_JPRB, 0.11688E+01_JPRB, 0.11365E+01_JPRB, 0.11118E+01_JPRB, 0.10767E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.98465E+00_JPRB, 0.95776E+00_JPRB, 0.94502E+00_JPRB, 0.94107E+00_JPRB, 0.94814E+00_JPRB, 0.95129E+00_JPRB ,&
&  0.98344E+00_JPRB, 0.12925E+01_JPRB, 0.10279E+01_JPRB, 0.87843E+00_JPRB, 0.78128E+00_JPRB, 0.87030E+00_JPRB, 0.99583E+00_JPRB ,&
&  0.11980E+01_JPRB, 0.13478E+01_JPRB, 0.16038E+01_JPRB, 0.20095E+01_JPRB, 0.23569E+01_JPRB, 0.26563E+01_JPRB, 0.28361E+01_JPRB ,&
&  0.26422E+01_JPRB, 0.24661E+01_JPRB, 0.19226E+01_JPRB, 0.17793E+01_JPRB, 0.15854E+01_JPRB, 0.13586E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.13114E+01_JPRB, 0.12951E+01_JPRB, 0.13138E+01_JPRB, 0.13415E+01_JPRB, 0.13733E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.14580E+01_JPRB, 0.15092E+01_JPRB, 0.15578E+01_JPRB, 0.16219E+01_JPRB, 0.18351E+01_JPRB, 0.23097E+01_JPRB, 0.27032E+01_JPRB ,&
&  0.35450E+01_JPRB, 0.37518E+01_JPRB, 0.40293E+01_JPRB, 0.61953E+01_JPRB, 0.92200E+01_JPRB, 0.90962E+01_JPRB, 0.82021E+01_JPRB ,&
&  0.80817E+01_JPRB, 0.80763E+01_JPRB, 0.81993E+01_JPRB, 0.78554E+01_JPRB, 0.71541E+01_JPRB, 0.67917E+01_JPRB, 0.63795E+01_JPRB ,&
&  0.57388E+01_JPRB, 0.52811E+01_JPRB, 0.50346E+01_JPRB, 0.49694E+01_JPRB, 0.47176E+01_JPRB, 0.46307E+01_JPRB, 0.46401E+01_JPRB ,&
&  0.44859E+01_JPRB, 0.38094E+01_JPRB, 0.25015E+01_JPRB, 0.24435E+01_JPRB, 0.18930E+01_JPRB, 0.16794E+01_JPRB, 0.17437E+01_JPRB ,&
&  0.15584E+01_JPRB, 0.16839E+01_JPRB, 0.18342E+01_JPRB, 0.17768E+01_JPRB, 0.21424E+01_JPRB, 0.21923E+01_JPRB, 0.21217E+01_JPRB ,&
&  0.20076E+01_JPRB, 0.18467E+01_JPRB, 0.21967E+01_JPRB, 0.22791E+01_JPRB, 0.22420E+01_JPRB, 0.20863E+01_JPRB, 0.18738E+01_JPRB ,&
&  0.16498E+01_JPRB, 0.15150E+01_JPRB, 0.14496E+01_JPRB, 0.14087E+01_JPRB, 0.13539E+01_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.11426E+01_JPRB, 0.11085E+01_JPRB, 0.10654E+01_JPRB, 0.10095E+01_JPRB, 0.97796E+00_JPRB, 0.97315E+00_JPRB, 0.98706E+00_JPRB ,&
&  0.98597E+00_JPRB, 0.95406E+00_JPRB, 0.93041E+00_JPRB, 0.91067E+00_JPRB, 0.90268E+00_JPRB, 0.90712E+00_JPRB, 0.92267E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.13611E+01_JPRB, 0.11250E+01_JPRB, 0.93515E+00_JPRB, 0.11085E+01_JPRB, 0.10447E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.12737E+01_JPRB, 0.12966E+01_JPRB, 0.14272E+01_JPRB, 0.15972E+01_JPRB, 0.18721E+01_JPRB, 0.19412E+01_JPRB ,&
&  0.17597E+01_JPRB, 0.14416E+01_JPRB, 0.11783E+01_JPRB, 0.11033E+01_JPRB, 0.11476E+01_JPRB, 0.10403E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.11183E+01_JPRB, 0.11288E+01_JPRB, 0.11329E+01_JPRB, 0.11627E+01_JPRB, 0.12177E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.13693E+01_JPRB, 0.14415E+01_JPRB, 0.14780E+01_JPRB, 0.15393E+01_JPRB, 0.17368E+01_JPRB, 0.21863E+01_JPRB, 0.26900E+01_JPRB ,&
&  0.35131E+01_JPRB, 0.41038E+01_JPRB, 0.51031E+01_JPRB, 0.69014E+01_JPRB, 0.77346E+01_JPRB, 0.78595E+01_JPRB, 0.75065E+01_JPRB ,&
&  0.68620E+01_JPRB, 0.62405E+01_JPRB, 0.60494E+01_JPRB, 0.54781E+01_JPRB, 0.51543E+01_JPRB, 0.51197E+01_JPRB, 0.49856E+01_JPRB ,&
&  0.46515E+01_JPRB, 0.43907E+01_JPRB, 0.43172E+01_JPRB, 0.41310E+01_JPRB, 0.40121E+01_JPRB, 0.38299E+01_JPRB, 0.38668E+01_JPRB ,&
&  0.42086E+01_JPRB, 0.27005E+01_JPRB, 0.20249E+01_JPRB, 0.21590E+01_JPRB, 0.17238E+01_JPRB, 0.15209E+01_JPRB, 0.16807E+01_JPRB ,&
&  0.14810E+01_JPRB, 0.14874E+01_JPRB, 0.16331E+01_JPRB, 0.15094E+01_JPRB, 0.15765E+01_JPRB, 0.17383E+01_JPRB, 0.14983E+01_JPRB ,&
&  0.14275E+01_JPRB, 0.14036E+01_JPRB, 0.16398E+01_JPRB, 0.17880E+01_JPRB, 0.17972E+01_JPRB, 0.16692E+01_JPRB, 0.14930E+01_JPRB ,&
&  0.13341E+01_JPRB, 0.12541E+01_JPRB, 0.12215E+01_JPRB, 0.11941E+01_JPRB, 0.11753E+01_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.98641E+00_JPRB, 0.96234E+00_JPRB, 0.93775E+00_JPRB, 0.92057E+00_JPRB, 0.91171E+00_JPRB, 0.90623E+00_JPRB, 0.92065E+00_JPRB ,&
&  0.93461E+00_JPRB, 0.92018E+00_JPRB, 0.89539E+00_JPRB, 0.87447E+00_JPRB, 0.85243E+00_JPRB, 0.85011E+00_JPRB, 0.87813E+00_JPRB ,&
&  0.78328E+00_JPRB, 0.98756E+00_JPRB, 0.83946E+00_JPRB, 0.91439E+00_JPRB, 0.13635E+01_JPRB, 0.11682E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.99713E+00_JPRB, 0.98888E+00_JPRB, 0.10982E+01_JPRB, 0.11836E+01_JPRB, 0.13003E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.11730E+01_JPRB, 0.93478E+00_JPRB, 0.81669E+00_JPRB, 0.77580E+00_JPRB, 0.81598E+00_JPRB, 0.84976E+00_JPRB, 0.92986E+00_JPRB ,&
&  0.96189E+00_JPRB, 0.97955E+00_JPRB, 0.99525E+00_JPRB, 0.10277E+01_JPRB, 0.10441E+01_JPRB, 0.10712E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.11817E+01_JPRB, 0.12665E+01_JPRB, 0.13531E+01_JPRB, 0.15020E+01_JPRB, 0.16166E+01_JPRB, 0.21763E+01_JPRB, 0.32355E+01_JPRB ,&
&  0.39563E+01_JPRB, 0.42634E+01_JPRB, 0.52185E+01_JPRB, 0.70793E+01_JPRB, 0.75891E+01_JPRB, 0.72254E+01_JPRB, 0.63906E+01_JPRB ,&
&  0.56207E+01_JPRB, 0.49018E+01_JPRB, 0.47783E+01_JPRB, 0.45514E+01_JPRB, 0.42270E+01_JPRB, 0.44073E+01_JPRB, 0.43180E+01_JPRB ,&
&  0.40899E+01_JPRB, 0.40481E+01_JPRB, 0.39934E+01_JPRB, 0.40080E+01_JPRB, 0.37854E+01_JPRB, 0.37944E+01_JPRB, 0.36641E+01_JPRB ,&
&  0.36693E+01_JPRB, 0.33102E+01_JPRB, 0.24273E+01_JPRB, 0.19979E+01_JPRB, 0.15908E+01_JPRB, 0.14877E+01_JPRB, 0.18470E+01_JPRB ,&
&  0.16122E+01_JPRB, 0.14534E+01_JPRB, 0.13280E+01_JPRB, 0.11875E+01_JPRB, 0.11369E+01_JPRB, 0.12287E+01_JPRB, 0.10839E+01_JPRB ,&
&  0.10273E+01_JPRB, 0.11401E+01_JPRB, 0.12796E+01_JPRB, 0.13653E+01_JPRB, 0.13988E+01_JPRB, 0.13662E+01_JPRB, 0.11602E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.10699E+01_JPRB, 0.10448E+01_JPRB, 0.10132E+01_JPRB, 0.99592E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.89274E+00_JPRB, 0.88182E+00_JPRB, 0.86903E+00_JPRB, 0.87177E+00_JPRB, 0.88645E+00_JPRB, 0.88583E+00_JPRB, 0.88691E+00_JPRB ,&
&  0.90053E+00_JPRB, 0.89108E+00_JPRB, 0.86149E+00_JPRB, 0.84258E+00_JPRB, 0.82204E+00_JPRB, 0.81857E+00_JPRB, 0.73257E+00_JPRB ,&
&  0.69245E+00_JPRB, 0.80352E+00_JPRB, 0.79086E+00_JPRB, 0.87990E+00_JPRB, 0.10192E+01_JPRB, 0.98284E+00_JPRB, 0.98079E+00_JPRB ,&
&  0.94344E+00_JPRB, 0.87300E+00_JPRB, 0.85114E+00_JPRB, 0.86634E+00_JPRB, 0.89743E+00_JPRB, 0.97476E+00_JPRB, 0.10350E+01_JPRB ,&
&  0.92636E+00_JPRB, 0.77134E+00_JPRB, 0.69162E+00_JPRB, 0.68424E+00_JPRB, 0.74471E+00_JPRB, 0.79880E+00_JPRB, 0.82567E+00_JPRB ,&
&  0.84649E+00_JPRB, 0.87072E+00_JPRB, 0.90818E+00_JPRB, 0.95789E+00_JPRB, 0.97875E+00_JPRB, 0.99927E+00_JPRB, 0.10284E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.11280E+01_JPRB, 0.12160E+01_JPRB, 0.13694E+01_JPRB, 0.14745E+01_JPRB, 0.20480E+01_JPRB, 0.29583E+01_JPRB ,&
&  0.34473E+01_JPRB, 0.38410E+01_JPRB, 0.44527E+01_JPRB, 0.52589E+01_JPRB, 0.55867E+01_JPRB, 0.52105E+01_JPRB, 0.47976E+01_JPRB ,&
&  0.46027E+01_JPRB, 0.41586E+01_JPRB, 0.45929E+01_JPRB, 0.42881E+01_JPRB, 0.40500E+01_JPRB, 0.40706E+01_JPRB, 0.39165E+01_JPRB ,&
&  0.38654E+01_JPRB, 0.38971E+01_JPRB, 0.38045E+01_JPRB, 0.36915E+01_JPRB, 0.33974E+01_JPRB, 0.33180E+01_JPRB, 0.30834E+01_JPRB ,&
&  0.30925E+01_JPRB, 0.30205E+01_JPRB, 0.25597E+01_JPRB, 0.21427E+01_JPRB, 0.17372E+01_JPRB, 0.16659E+01_JPRB, 0.14978E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.11973E+01_JPRB, 0.10823E+01_JPRB, 0.94543E+00_JPRB, 0.88372E+00_JPRB, 0.86234E+00_JPRB, 0.85327E+00_JPRB ,&
&  0.93896E+00_JPRB, 0.10871E+01_JPRB, 0.11232E+01_JPRB, 0.11254E+01_JPRB, 0.11371E+01_JPRB, 0.11630E+01_JPRB, 0.93548E+00_JPRB ,&
&  0.89721E+00_JPRB, 0.95983E+00_JPRB, 0.94043E+00_JPRB, 0.91478E+00_JPRB, 0.89705E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.83014E+00_JPRB, 0.82533E+00_JPRB, 0.82505E+00_JPRB, 0.84087E+00_JPRB, 0.86229E+00_JPRB, 0.86948E+00_JPRB, 0.82986E+00_JPRB ,&
&  0.87479E+00_JPRB, 0.84517E+00_JPRB, 0.82619E+00_JPRB, 0.81550E+00_JPRB, 0.79896E+00_JPRB, 0.70361E+00_JPRB, 0.60932E+00_JPRB ,&
&  0.60321E+00_JPRB, 0.64932E+00_JPRB, 0.73503E+00_JPRB, 0.79701E+00_JPRB, 0.86422E+00_JPRB, 0.86299E+00_JPRB, 0.85161E+00_JPRB ,&
&  0.83638E+00_JPRB, 0.80124E+00_JPRB, 0.79694E+00_JPRB, 0.79182E+00_JPRB, 0.78026E+00_JPRB, 0.78609E+00_JPRB, 0.80097E+00_JPRB ,&
&  0.74031E+00_JPRB, 0.69630E+00_JPRB, 0.68920E+00_JPRB, 0.66926E+00_JPRB, 0.73330E+00_JPRB, 0.75308E+00_JPRB, 0.76978E+00_JPRB ,&
&  0.79549E+00_JPRB, 0.83235E+00_JPRB, 0.89400E+00_JPRB, 0.94249E+00_JPRB, 0.97525E+00_JPRB, 0.99504E+00_JPRB, 0.10288E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.11047E+01_JPRB, 0.11801E+01_JPRB, 0.13159E+01_JPRB, 0.15363E+01_JPRB, 0.19018E+01_JPRB, 0.28008E+01_JPRB ,&
&  0.31041E+01_JPRB, 0.35466E+01_JPRB, 0.40692E+01_JPRB, 0.38036E+01_JPRB, 0.42067E+01_JPRB, 0.44768E+01_JPRB, 0.43489E+01_JPRB ,&
&  0.43835E+01_JPRB, 0.39507E+01_JPRB, 0.38541E+01_JPRB, 0.37895E+01_JPRB, 0.37796E+01_JPRB, 0.35042E+01_JPRB, 0.32375E+01_JPRB ,&
&  0.33902E+01_JPRB, 0.36888E+01_JPRB, 0.30175E+01_JPRB, 0.27135E+01_JPRB, 0.24521E+01_JPRB, 0.23638E+01_JPRB, 0.22231E+01_JPRB ,&
&  0.21859E+01_JPRB, 0.21713E+01_JPRB, 0.20596E+01_JPRB, 0.19077E+01_JPRB, 0.16417E+01_JPRB, 0.15140E+01_JPRB, 0.13709E+01_JPRB ,&
&  0.12903E+01_JPRB, 0.11691E+01_JPRB, 0.10076E+01_JPRB, 0.90870E+00_JPRB, 0.83367E+00_JPRB, 0.76930E+00_JPRB, 0.77523E+00_JPRB ,&
&  0.84488E+00_JPRB, 0.90042E+00_JPRB, 0.10343E+01_JPRB, 0.10425E+01_JPRB, 0.10481E+01_JPRB, 0.10700E+01_JPRB, 0.93712E+00_JPRB ,&
&  0.87160E+00_JPRB, 0.94991E+00_JPRB, 0.91339E+00_JPRB, 0.87236E+00_JPRB, 0.84239E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.79616E+00_JPRB, 0.80388E+00_JPRB, 0.81034E+00_JPRB, 0.82723E+00_JPRB, 0.84980E+00_JPRB, 0.80295E+00_JPRB, 0.75755E+00_JPRB ,&
&  0.81069E+00_JPRB, 0.72456E+00_JPRB, 0.77060E+00_JPRB, 0.67625E+00_JPRB, 0.62898E+00_JPRB, 0.60525E+00_JPRB, 0.59262E+00_JPRB ,&
&  0.58922E+00_JPRB, 0.62901E+00_JPRB, 0.67671E+00_JPRB, 0.72404E+00_JPRB, 0.80219E+00_JPRB, 0.79600E+00_JPRB, 0.77959E+00_JPRB ,&
&  0.76046E+00_JPRB, 0.74494E+00_JPRB, 0.75598E+00_JPRB, 0.74984E+00_JPRB, 0.74729E+00_JPRB, 0.74241E+00_JPRB, 0.72576E+00_JPRB ,&
&  0.67755E+00_JPRB, 0.66552E+00_JPRB, 0.68802E+00_JPRB, 0.70627E+00_JPRB, 0.72056E+00_JPRB, 0.72144E+00_JPRB, 0.72844E+00_JPRB ,&
&  0.74978E+00_JPRB, 0.76871E+00_JPRB, 0.91650E+00_JPRB, 0.95477E+00_JPRB, 0.99836E+00_JPRB, 0.10257E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11648E+01_JPRB, 0.12105E+01_JPRB, 0.12844E+01_JPRB, 0.14260E+01_JPRB, 0.17752E+01_JPRB, 0.23352E+01_JPRB ,&
&  0.30652E+01_JPRB, 0.24334E+01_JPRB, 0.30634E+01_JPRB, 0.31170E+01_JPRB, 0.35852E+01_JPRB, 0.40000E+01_JPRB, 0.41229E+01_JPRB ,&
&  0.45257E+01_JPRB, 0.40505E+01_JPRB, 0.36263E+01_JPRB, 0.31790E+01_JPRB, 0.28089E+01_JPRB, 0.27696E+01_JPRB, 0.27176E+01_JPRB ,&
&  0.27546E+01_JPRB, 0.25868E+01_JPRB, 0.22361E+01_JPRB, 0.20229E+01_JPRB, 0.19404E+01_JPRB, 0.18897E+01_JPRB, 0.17338E+01_JPRB ,&
&  0.16454E+01_JPRB, 0.15661E+01_JPRB, 0.14608E+01_JPRB, 0.13380E+01_JPRB, 0.12554E+01_JPRB, 0.12093E+01_JPRB, 0.11364E+01_JPRB ,&
&  0.11133E+01_JPRB, 0.10700E+01_JPRB, 0.95249E+00_JPRB, 0.89798E+00_JPRB, 0.85184E+00_JPRB, 0.80184E+00_JPRB, 0.79716E+00_JPRB ,&
&  0.80203E+00_JPRB, 0.78400E+00_JPRB, 0.82866E+00_JPRB, 0.88116E+00_JPRB, 0.89354E+00_JPRB, 0.89607E+00_JPRB, 0.98225E+00_JPRB ,&
&  0.91224E+00_JPRB, 0.90950E+00_JPRB, 0.90734E+00_JPRB, 0.84671E+00_JPRB, 0.81039E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.77540E+00_JPRB, 0.77021E+00_JPRB, 0.79158E+00_JPRB, 0.81237E+00_JPRB, 0.83120E+00_JPRB, 0.77495E+00_JPRB, 0.71204E+00_JPRB ,&
&  0.72594E+00_JPRB, 0.65789E+00_JPRB, 0.61359E+00_JPRB, 0.60272E+00_JPRB, 0.60129E+00_JPRB, 0.58349E+00_JPRB, 0.57158E+00_JPRB ,&
&  0.57102E+00_JPRB, 0.59043E+00_JPRB, 0.62398E+00_JPRB, 0.65052E+00_JPRB, 0.67893E+00_JPRB, 0.69977E+00_JPRB, 0.70527E+00_JPRB ,&
&  0.70767E+00_JPRB, 0.71072E+00_JPRB, 0.71247E+00_JPRB, 0.71301E+00_JPRB, 0.70980E+00_JPRB, 0.71032E+00_JPRB, 0.70396E+00_JPRB ,&
&  0.68772E+00_JPRB, 0.67594E+00_JPRB, 0.67863E+00_JPRB, 0.69790E+00_JPRB, 0.69926E+00_JPRB, 0.68837E+00_JPRB, 0.66378E+00_JPRB ,&
&  0.58092E+00_JPRB, 0.64584E+00_JPRB, 0.89061E+00_JPRB, 0.95619E+00_JPRB, 0.99516E+00_JPRB, 0.10481E+01_JPRB, 0.10945E+01_JPRB ,&
&  0.11412E+01_JPRB, 0.11935E+01_JPRB, 0.12493E+01_JPRB, 0.13208E+01_JPRB, 0.14183E+01_JPRB, 0.15905E+01_JPRB, 0.19212E+01_JPRB ,&
&  0.25709E+01_JPRB, 0.22038E+01_JPRB, 0.21581E+01_JPRB, 0.23694E+01_JPRB, 0.34779E+01_JPRB, 0.38698E+01_JPRB, 0.35981E+01_JPRB ,&
&  0.35015E+01_JPRB, 0.33600E+01_JPRB, 0.31760E+01_JPRB, 0.28603E+01_JPRB, 0.26474E+01_JPRB, 0.26219E+01_JPRB, 0.25535E+01_JPRB ,&
&  0.23406E+01_JPRB, 0.21940E+01_JPRB, 0.18993E+01_JPRB, 0.18191E+01_JPRB, 0.17996E+01_JPRB, 0.16825E+01_JPRB, 0.15251E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.12703E+01_JPRB, 0.11775E+01_JPRB, 0.11140E+01_JPRB, 0.10605E+01_JPRB, 0.10263E+01_JPRB, 0.99874E+00_JPRB ,&
&  0.97747E+00_JPRB, 0.95898E+00_JPRB, 0.91247E+00_JPRB, 0.88961E+00_JPRB, 0.86435E+00_JPRB, 0.84755E+00_JPRB, 0.84897E+00_JPRB ,&
&  0.81254E+00_JPRB, 0.79450E+00_JPRB, 0.75099E+00_JPRB, 0.74651E+00_JPRB, 0.76841E+00_JPRB, 0.81944E+00_JPRB, 0.85515E+00_JPRB ,&
&  0.91958E+00_JPRB, 0.89616E+00_JPRB, 0.83854E+00_JPRB, 0.82017E+00_JPRB, 0.79157E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.70581E+00_JPRB, 0.74169E+00_JPRB, 0.77990E+00_JPRB, 0.80521E+00_JPRB, 0.80322E+00_JPRB, 0.77406E+00_JPRB, 0.70025E+00_JPRB ,&
&  0.67105E+00_JPRB, 0.63507E+00_JPRB, 0.61676E+00_JPRB, 0.60201E+00_JPRB, 0.59160E+00_JPRB, 0.58283E+00_JPRB, 0.58215E+00_JPRB ,&
&  0.58984E+00_JPRB, 0.60041E+00_JPRB, 0.61087E+00_JPRB, 0.62103E+00_JPRB, 0.63007E+00_JPRB, 0.64165E+00_JPRB, 0.65428E+00_JPRB ,&
&  0.66230E+00_JPRB, 0.66775E+00_JPRB, 0.67362E+00_JPRB, 0.67770E+00_JPRB, 0.68278E+00_JPRB, 0.68887E+00_JPRB, 0.68687E+00_JPRB ,&
&  0.67268E+00_JPRB, 0.65408E+00_JPRB, 0.66037E+00_JPRB, 0.67935E+00_JPRB, 0.67094E+00_JPRB, 0.64485E+00_JPRB, 0.57992E+00_JPRB ,&
&  0.53711E+00_JPRB, 0.55362E+00_JPRB, 0.65973E+00_JPRB, 0.86980E+00_JPRB, 0.93392E+00_JPRB, 0.99524E+00_JPRB, 0.10246E+01_JPRB ,&
&  0.96232E+00_JPRB, 0.97195E+00_JPRB, 0.10538E+01_JPRB, 0.12833E+01_JPRB, 0.13513E+01_JPRB, 0.14761E+01_JPRB, 0.16548E+01_JPRB ,&
&  0.19815E+01_JPRB, 0.22373E+01_JPRB, 0.24237E+01_JPRB, 0.21695E+01_JPRB, 0.25965E+01_JPRB, 0.34286E+01_JPRB, 0.31733E+01_JPRB ,&
&  0.30779E+01_JPRB, 0.29897E+01_JPRB, 0.30521E+01_JPRB, 0.27872E+01_JPRB, 0.24625E+01_JPRB, 0.22665E+01_JPRB, 0.22016E+01_JPRB ,&
&  0.21117E+01_JPRB, 0.20157E+01_JPRB, 0.17133E+01_JPRB, 0.15793E+01_JPRB, 0.15596E+01_JPRB, 0.14908E+01_JPRB, 0.14355E+01_JPRB ,&
&  0.13737E+01_JPRB, 0.12448E+01_JPRB, 0.11247E+01_JPRB, 0.10072E+01_JPRB, 0.92929E+00_JPRB, 0.90423E+00_JPRB, 0.90065E+00_JPRB ,&
&  0.89510E+00_JPRB, 0.88440E+00_JPRB, 0.87360E+00_JPRB, 0.86255E+00_JPRB, 0.83492E+00_JPRB, 0.79323E+00_JPRB, 0.76549E+00_JPRB ,&
&  0.73770E+00_JPRB, 0.70632E+00_JPRB, 0.67952E+00_JPRB, 0.68158E+00_JPRB, 0.68993E+00_JPRB, 0.71431E+00_JPRB, 0.73756E+00_JPRB ,&
&  0.75662E+00_JPRB, 0.75912E+00_JPRB, 0.74609E+00_JPRB, 0.74842E+00_JPRB, 0.71925E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.65384E+00_JPRB, 0.69677E+00_JPRB, 0.76311E+00_JPRB, 0.80256E+00_JPRB, 0.80442E+00_JPRB, 0.76103E+00_JPRB, 0.70257E+00_JPRB ,&
&  0.66288E+00_JPRB, 0.64056E+00_JPRB, 0.63267E+00_JPRB, 0.62962E+00_JPRB, 0.62955E+00_JPRB, 0.63041E+00_JPRB, 0.63057E+00_JPRB ,&
&  0.62927E+00_JPRB, 0.62825E+00_JPRB, 0.62790E+00_JPRB, 0.63102E+00_JPRB, 0.63677E+00_JPRB, 0.64275E+00_JPRB, 0.65107E+00_JPRB ,&
&  0.66184E+00_JPRB, 0.66995E+00_JPRB, 0.67278E+00_JPRB, 0.67111E+00_JPRB, 0.67158E+00_JPRB, 0.67804E+00_JPRB, 0.68158E+00_JPRB ,&
&  0.66685E+00_JPRB, 0.64562E+00_JPRB, 0.64960E+00_JPRB, 0.66761E+00_JPRB, 0.65847E+00_JPRB, 0.62147E+00_JPRB, 0.56078E+00_JPRB ,&
&  0.52299E+00_JPRB, 0.49714E+00_JPRB, 0.51277E+00_JPRB, 0.57165E+00_JPRB, 0.67047E+00_JPRB, 0.86548E+00_JPRB, 0.95849E+00_JPRB ,&
&  0.10414E+01_JPRB, 0.10932E+01_JPRB, 0.11302E+01_JPRB, 0.11706E+01_JPRB, 0.12445E+01_JPRB, 0.13391E+01_JPRB, 0.14334E+01_JPRB ,&
&  0.15675E+01_JPRB, 0.17467E+01_JPRB, 0.20049E+01_JPRB, 0.16768E+01_JPRB, 0.17206E+01_JPRB, 0.20604E+01_JPRB, 0.24382E+01_JPRB ,&
&  0.25891E+01_JPRB, 0.23855E+01_JPRB, 0.22888E+01_JPRB, 0.23361E+01_JPRB, 0.21550E+01_JPRB, 0.19891E+01_JPRB, 0.18675E+01_JPRB ,&
&  0.17814E+01_JPRB, 0.17524E+01_JPRB, 0.15840E+01_JPRB, 0.13868E+01_JPRB, 0.13307E+01_JPRB, 0.13149E+01_JPRB, 0.13365E+01_JPRB ,&
&  0.14158E+01_JPRB, 0.14640E+01_JPRB, 0.12883E+01_JPRB, 0.97228E+00_JPRB, 0.89026E+00_JPRB, 0.85274E+00_JPRB, 0.85056E+00_JPRB ,&
&  0.85567E+00_JPRB, 0.86014E+00_JPRB, 0.86298E+00_JPRB, 0.85375E+00_JPRB, 0.82823E+00_JPRB, 0.79538E+00_JPRB, 0.76108E+00_JPRB ,&
&  0.73159E+00_JPRB, 0.71487E+00_JPRB, 0.71205E+00_JPRB, 0.71147E+00_JPRB, 0.72731E+00_JPRB, 0.74402E+00_JPRB, 0.76246E+00_JPRB ,&
&  0.75713E+00_JPRB, 0.73785E+00_JPRB, 0.71709E+00_JPRB, 0.67748E+00_JPRB, 0.64565E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.66624E+00_JPRB, 0.70287E+00_JPRB, 0.75476E+00_JPRB, 0.78984E+00_JPRB, 0.79308E+00_JPRB, 0.76574E+00_JPRB, 0.73507E+00_JPRB ,&
&  0.71494E+00_JPRB, 0.70723E+00_JPRB, 0.69725E+00_JPRB, 0.69085E+00_JPRB, 0.68579E+00_JPRB, 0.68073E+00_JPRB, 0.67581E+00_JPRB ,&
&  0.67217E+00_JPRB, 0.66929E+00_JPRB, 0.66500E+00_JPRB, 0.65745E+00_JPRB, 0.65459E+00_JPRB, 0.66093E+00_JPRB, 0.67302E+00_JPRB ,&
&  0.68679E+00_JPRB, 0.69259E+00_JPRB, 0.69575E+00_JPRB, 0.68885E+00_JPRB, 0.67319E+00_JPRB, 0.65504E+00_JPRB, 0.64063E+00_JPRB ,&
&  0.63194E+00_JPRB, 0.63638E+00_JPRB, 0.63800E+00_JPRB, 0.65753E+00_JPRB, 0.64383E+00_JPRB, 0.60002E+00_JPRB, 0.54430E+00_JPRB ,&
&  0.51271E+00_JPRB, 0.47685E+00_JPRB, 0.46105E+00_JPRB, 0.46850E+00_JPRB, 0.50761E+00_JPRB, 0.56472E+00_JPRB, 0.66408E+00_JPRB ,&
&  0.84662E+00_JPRB, 0.92365E+00_JPRB, 0.98078E+00_JPRB, 0.10330E+01_JPRB, 0.10772E+01_JPRB, 0.11283E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.12870E+01_JPRB, 0.14114E+01_JPRB, 0.15550E+01_JPRB, 0.16267E+01_JPRB, 0.17333E+01_JPRB, 0.17312E+01_JPRB, 0.20666E+01_JPRB ,&
&  0.27495E+01_JPRB, 0.23811E+01_JPRB, 0.20444E+01_JPRB, 0.18467E+01_JPRB, 0.17355E+01_JPRB, 0.16248E+01_JPRB, 0.15247E+01_JPRB ,&
&  0.14280E+01_JPRB, 0.13682E+01_JPRB, 0.13357E+01_JPRB, 0.12486E+01_JPRB, 0.11579E+01_JPRB, 0.11401E+01_JPRB, 0.12147E+01_JPRB ,&
&  0.14225E+01_JPRB, 0.17093E+01_JPRB, 0.19999E+01_JPRB, 0.13676E+01_JPRB, 0.11838E+01_JPRB, 0.10541E+01_JPRB, 0.97521E+00_JPRB ,&
&  0.94305E+00_JPRB, 0.92882E+00_JPRB, 0.91964E+00_JPRB, 0.89692E+00_JPRB, 0.86582E+00_JPRB, 0.83664E+00_JPRB, 0.80880E+00_JPRB ,&
&  0.78915E+00_JPRB, 0.78319E+00_JPRB, 0.78419E+00_JPRB, 0.78681E+00_JPRB, 0.78954E+00_JPRB, 0.78459E+00_JPRB, 0.77883E+00_JPRB ,&
&  0.77102E+00_JPRB, 0.74976E+00_JPRB, 0.72650E+00_JPRB, 0.69152E+00_JPRB, 0.66653E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.69383E+00_JPRB, 0.71504E+00_JPRB, 0.74891E+00_JPRB, 0.77277E+00_JPRB, 0.78359E+00_JPRB, 0.77120E+00_JPRB, 0.75307E+00_JPRB ,&
&  0.73758E+00_JPRB, 0.72694E+00_JPRB, 0.71745E+00_JPRB, 0.71055E+00_JPRB, 0.70316E+00_JPRB, 0.69606E+00_JPRB, 0.69079E+00_JPRB ,&
&  0.68741E+00_JPRB, 0.67510E+00_JPRB, 0.67124E+00_JPRB, 0.67520E+00_JPRB, 0.67881E+00_JPRB, 0.68571E+00_JPRB, 0.69359E+00_JPRB ,&
&  0.69466E+00_JPRB, 0.69000E+00_JPRB, 0.68583E+00_JPRB, 0.66799E+00_JPRB, 0.64871E+00_JPRB, 0.64108E+00_JPRB, 0.65106E+00_JPRB ,&
&  0.64184E+00_JPRB, 0.61735E+00_JPRB, 0.64212E+00_JPRB, 0.65698E+00_JPRB, 0.62034E+00_JPRB, 0.57998E+00_JPRB, 0.51293E+00_JPRB ,&
&  0.49147E+00_JPRB, 0.46365E+00_JPRB, 0.45684E+00_JPRB, 0.45681E+00_JPRB, 0.47497E+00_JPRB, 0.51624E+00_JPRB, 0.58838E+00_JPRB ,&
&  0.66789E+00_JPRB, 0.80593E+00_JPRB, 0.85137E+00_JPRB, 0.91755E+00_JPRB, 0.98011E+00_JPRB, 0.10359E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.12681E+01_JPRB, 0.13800E+01_JPRB, 0.14694E+01_JPRB, 0.15678E+01_JPRB, 0.16723E+01_JPRB, 0.18153E+01_JPRB ,&
&  0.19830E+01_JPRB, 0.16854E+01_JPRB, 0.15123E+01_JPRB, 0.14002E+01_JPRB, 0.13182E+01_JPRB, 0.12863E+01_JPRB, 0.11956E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11528E+01_JPRB, 0.11306E+01_JPRB, 0.11214E+01_JPRB, 0.11114E+01_JPRB, 0.10972E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.10345E+01_JPRB, 0.10467E+01_JPRB, 0.11993E+01_JPRB, 0.12262E+01_JPRB, 0.12047E+01_JPRB, 0.11178E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.93387E+00_JPRB, 0.89582E+00_JPRB, 0.87384E+00_JPRB, 0.85480E+00_JPRB, 0.83967E+00_JPRB, 0.82976E+00_JPRB, 0.83162E+00_JPRB ,&
&  0.81502E+00_JPRB, 0.81194E+00_JPRB, 0.81144E+00_JPRB, 0.80769E+00_JPRB, 0.79777E+00_JPRB, 0.78094E+00_JPRB, 0.76799E+00_JPRB ,&
&  0.75529E+00_JPRB, 0.74275E+00_JPRB, 0.72802E+00_JPRB, 0.71089E+00_JPRB, 0.69862E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.71397E+00_JPRB, 0.71862E+00_JPRB, 0.74147E+00_JPRB, 0.76117E+00_JPRB, 0.77225E+00_JPRB, 0.77087E+00_JPRB, 0.75924E+00_JPRB ,&
&  0.74979E+00_JPRB, 0.74305E+00_JPRB, 0.73498E+00_JPRB, 0.72723E+00_JPRB, 0.71774E+00_JPRB, 0.70909E+00_JPRB, 0.70297E+00_JPRB ,&
&  0.69890E+00_JPRB, 0.69838E+00_JPRB, 0.69903E+00_JPRB, 0.69948E+00_JPRB, 0.70109E+00_JPRB, 0.70377E+00_JPRB, 0.70414E+00_JPRB ,&
&  0.70465E+00_JPRB, 0.69418E+00_JPRB, 0.66889E+00_JPRB, 0.64910E+00_JPRB, 0.64353E+00_JPRB, 0.63377E+00_JPRB, 0.64413E+00_JPRB ,&
&  0.64564E+00_JPRB, 0.64591E+00_JPRB, 0.64277E+00_JPRB, 0.64030E+00_JPRB, 0.61132E+00_JPRB, 0.50289E+00_JPRB, 0.50235E+00_JPRB ,&
&  0.48693E+00_JPRB, 0.46884E+00_JPRB, 0.46664E+00_JPRB, 0.46863E+00_JPRB, 0.47644E+00_JPRB, 0.50724E+00_JPRB, 0.55204E+00_JPRB ,&
&  0.59616E+00_JPRB, 0.77728E+00_JPRB, 0.83056E+00_JPRB, 0.88908E+00_JPRB, 0.94143E+00_JPRB, 0.10076E+01_JPRB, 0.10847E+01_JPRB ,&
&  0.11418E+01_JPRB, 0.12051E+01_JPRB, 0.12708E+01_JPRB, 0.13303E+01_JPRB, 0.13839E+01_JPRB, 0.14261E+01_JPRB, 0.14610E+01_JPRB ,&
&  0.14567E+01_JPRB, 0.13139E+01_JPRB, 0.12141E+01_JPRB, 0.11492E+01_JPRB, 0.10793E+01_JPRB, 0.10324E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.98610E+00_JPRB, 0.98861E+00_JPRB, 0.97114E+00_JPRB, 0.95935E+00_JPRB, 0.94927E+00_JPRB, 0.93495E+00_JPRB ,&
&  0.91873E+00_JPRB, 0.92167E+00_JPRB, 0.95081E+00_JPRB, 0.95972E+00_JPRB, 0.97409E+00_JPRB, 0.96538E+00_JPRB, 0.94117E+00_JPRB ,&
&  0.90540E+00_JPRB, 0.89280E+00_JPRB, 0.87252E+00_JPRB, 0.85783E+00_JPRB, 0.84099E+00_JPRB, 0.83628E+00_JPRB, 0.83492E+00_JPRB ,&
&  0.83242E+00_JPRB, 0.82418E+00_JPRB, 0.81517E+00_JPRB, 0.80845E+00_JPRB, 0.80128E+00_JPRB, 0.79062E+00_JPRB, 0.77864E+00_JPRB ,&
&  0.76549E+00_JPRB, 0.75744E+00_JPRB, 0.74366E+00_JPRB, 0.73268E+00_JPRB, 0.72417E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.73490E+00_JPRB, 0.73060E+00_JPRB, 0.73389E+00_JPRB, 0.73859E+00_JPRB, 0.74296E+00_JPRB, 0.74491E+00_JPRB, 0.73860E+00_JPRB ,&
&  0.73363E+00_JPRB, 0.73057E+00_JPRB, 0.72846E+00_JPRB, 0.72835E+00_JPRB, 0.73079E+00_JPRB, 0.72856E+00_JPRB, 0.72234E+00_JPRB ,&
&  0.71548E+00_JPRB, 0.71085E+00_JPRB, 0.70776E+00_JPRB, 0.70547E+00_JPRB, 0.70646E+00_JPRB, 0.70532E+00_JPRB, 0.70280E+00_JPRB ,&
&  0.69978E+00_JPRB, 0.68454E+00_JPRB, 0.65188E+00_JPRB, 0.64056E+00_JPRB, 0.62974E+00_JPRB, 0.63201E+00_JPRB, 0.63423E+00_JPRB ,&
&  0.63767E+00_JPRB, 0.56378E+00_JPRB, 0.57055E+00_JPRB, 0.54965E+00_JPRB, 0.53449E+00_JPRB, 0.53087E+00_JPRB, 0.51649E+00_JPRB ,&
&  0.50732E+00_JPRB, 0.49535E+00_JPRB, 0.49557E+00_JPRB, 0.50369E+00_JPRB, 0.50749E+00_JPRB, 0.52543E+00_JPRB, 0.55239E+00_JPRB ,&
&  0.59222E+00_JPRB, 0.74888E+00_JPRB, 0.82318E+00_JPRB, 0.87450E+00_JPRB, 0.91574E+00_JPRB, 0.96392E+00_JPRB, 0.10185E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.11233E+01_JPRB, 0.11610E+01_JPRB, 0.11690E+01_JPRB, 0.12081E+01_JPRB, 0.12122E+01_JPRB, 0.11929E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.10647E+01_JPRB, 0.99892E+00_JPRB, 0.95670E+00_JPRB, 0.92563E+00_JPRB, 0.90896E+00_JPRB, 0.90264E+00_JPRB ,&
&  0.91540E+00_JPRB, 0.91573E+00_JPRB, 0.90992E+00_JPRB, 0.88608E+00_JPRB, 0.87082E+00_JPRB, 0.85884E+00_JPRB, 0.85530E+00_JPRB ,&
&  0.85726E+00_JPRB, 0.86794E+00_JPRB, 0.89008E+00_JPRB, 0.90523E+00_JPRB, 0.91177E+00_JPRB, 0.90997E+00_JPRB, 0.91311E+00_JPRB ,&
&  0.90739E+00_JPRB, 0.89306E+00_JPRB, 0.87705E+00_JPRB, 0.86017E+00_JPRB, 0.84123E+00_JPRB, 0.82661E+00_JPRB, 0.81806E+00_JPRB ,&
&  0.81300E+00_JPRB, 0.80625E+00_JPRB, 0.79978E+00_JPRB, 0.79494E+00_JPRB, 0.79305E+00_JPRB, 0.79069E+00_JPRB, 0.78747E+00_JPRB ,&
&  0.77675E+00_JPRB, 0.76534E+00_JPRB, 0.75484E+00_JPRB, 0.75059E+00_JPRB, 0.74470E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.72858E+00_JPRB, 0.72186E+00_JPRB, 0.71813E+00_JPRB, 0.71532E+00_JPRB, 0.71475E+00_JPRB, 0.71509E+00_JPRB, 0.71680E+00_JPRB ,&
&  0.72016E+00_JPRB, 0.72518E+00_JPRB, 0.72935E+00_JPRB, 0.73184E+00_JPRB, 0.73611E+00_JPRB, 0.74267E+00_JPRB, 0.74473E+00_JPRB ,&
&  0.73820E+00_JPRB, 0.72917E+00_JPRB, 0.71953E+00_JPRB, 0.70981E+00_JPRB, 0.70097E+00_JPRB, 0.69416E+00_JPRB, 0.68359E+00_JPRB ,&
&  0.67648E+00_JPRB, 0.66555E+00_JPRB, 0.62451E+00_JPRB, 0.62637E+00_JPRB, 0.61806E+00_JPRB, 0.61666E+00_JPRB, 0.62120E+00_JPRB ,&
&  0.61200E+00_JPRB, 0.61124E+00_JPRB, 0.59786E+00_JPRB, 0.57504E+00_JPRB, 0.55895E+00_JPRB, 0.53561E+00_JPRB, 0.53336E+00_JPRB ,&
&  0.52907E+00_JPRB, 0.53411E+00_JPRB, 0.54727E+00_JPRB, 0.55415E+00_JPRB, 0.55543E+00_JPRB, 0.56035E+00_JPRB, 0.58017E+00_JPRB ,&
&  0.62132E+00_JPRB, 0.67983E+00_JPRB, 0.76288E+00_JPRB, 0.81073E+00_JPRB, 0.85304E+00_JPRB, 0.89256E+00_JPRB, 0.93190E+00_JPRB ,&
&  0.96457E+00_JPRB, 0.99351E+00_JPRB, 0.10077E+01_JPRB, 0.10165E+01_JPRB, 0.10107E+01_JPRB, 0.98859E+00_JPRB, 0.95733E+00_JPRB ,&
&  0.92254E+00_JPRB, 0.89699E+00_JPRB, 0.87814E+00_JPRB, 0.87178E+00_JPRB, 0.87000E+00_JPRB, 0.87028E+00_JPRB, 0.87206E+00_JPRB ,&
&  0.87154E+00_JPRB, 0.85408E+00_JPRB, 0.83889E+00_JPRB, 0.81725E+00_JPRB, 0.80663E+00_JPRB, 0.79716E+00_JPRB, 0.79571E+00_JPRB ,&
&  0.79686E+00_JPRB, 0.80590E+00_JPRB, 0.81922E+00_JPRB, 0.82345E+00_JPRB, 0.83291E+00_JPRB, 0.84657E+00_JPRB, 0.84609E+00_JPRB ,&
&  0.84461E+00_JPRB, 0.83808E+00_JPRB, 0.82687E+00_JPRB, 0.81372E+00_JPRB, 0.79896E+00_JPRB, 0.78496E+00_JPRB, 0.77171E+00_JPRB ,&
&  0.76245E+00_JPRB, 0.75597E+00_JPRB, 0.75505E+00_JPRB, 0.75500E+00_JPRB, 0.75784E+00_JPRB, 0.75792E+00_JPRB, 0.75451E+00_JPRB ,&
&  0.74619E+00_JPRB, 0.73977E+00_JPRB, 0.73424E+00_JPRB, 0.73370E+00_JPRB, 0.73230E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.72939E+00_JPRB, 0.72925E+00_JPRB, 0.72849E+00_JPRB, 0.72597E+00_JPRB, 0.72511E+00_JPRB, 0.72532E+00_JPRB, 0.72668E+00_JPRB ,&
&  0.72878E+00_JPRB, 0.73213E+00_JPRB, 0.73555E+00_JPRB, 0.73798E+00_JPRB, 0.73976E+00_JPRB, 0.75007E+00_JPRB, 0.75425E+00_JPRB ,&
&  0.75325E+00_JPRB, 0.74930E+00_JPRB, 0.74371E+00_JPRB, 0.73819E+00_JPRB, 0.73112E+00_JPRB, 0.72231E+00_JPRB, 0.70842E+00_JPRB ,&
&  0.70000E+00_JPRB, 0.69123E+00_JPRB, 0.68137E+00_JPRB, 0.67809E+00_JPRB, 0.67480E+00_JPRB, 0.64035E+00_JPRB, 0.63739E+00_JPRB ,&
&  0.65916E+00_JPRB, 0.66988E+00_JPRB, 0.67378E+00_JPRB, 0.67927E+00_JPRB, 0.66560E+00_JPRB, 0.63169E+00_JPRB, 0.60712E+00_JPRB ,&
&  0.56456E+00_JPRB, 0.59007E+00_JPRB, 0.60853E+00_JPRB, 0.61567E+00_JPRB, 0.61666E+00_JPRB, 0.61862E+00_JPRB, 0.65010E+00_JPRB ,&
&  0.72106E+00_JPRB, 0.73571E+00_JPRB, 0.75361E+00_JPRB, 0.77368E+00_JPRB, 0.79604E+00_JPRB, 0.81372E+00_JPRB, 0.83483E+00_JPRB ,&
&  0.85011E+00_JPRB, 0.86248E+00_JPRB, 0.86682E+00_JPRB, 0.86901E+00_JPRB, 0.86201E+00_JPRB, 0.85575E+00_JPRB, 0.84863E+00_JPRB ,&
&  0.84636E+00_JPRB, 0.84469E+00_JPRB, 0.84276E+00_JPRB, 0.83741E+00_JPRB, 0.82747E+00_JPRB, 0.81719E+00_JPRB, 0.80577E+00_JPRB ,&
&  0.79395E+00_JPRB, 0.77840E+00_JPRB, 0.76720E+00_JPRB, 0.75677E+00_JPRB, 0.75355E+00_JPRB, 0.74854E+00_JPRB, 0.74950E+00_JPRB ,&
&  0.75083E+00_JPRB, 0.75601E+00_JPRB, 0.76193E+00_JPRB, 0.77256E+00_JPRB, 0.78105E+00_JPRB, 0.78835E+00_JPRB, 0.79181E+00_JPRB ,&
&  0.79375E+00_JPRB, 0.79271E+00_JPRB, 0.79008E+00_JPRB, 0.78454E+00_JPRB, 0.77966E+00_JPRB, 0.77511E+00_JPRB, 0.76991E+00_JPRB ,&
&  0.76492E+00_JPRB, 0.75979E+00_JPRB, 0.75540E+00_JPRB, 0.75083E+00_JPRB, 0.74631E+00_JPRB, 0.74082E+00_JPRB, 0.73515E+00_JPRB ,&
&  0.72923E+00_JPRB, 0.72605E+00_JPRB, 0.72422E+00_JPRB, 0.72524E+00_JPRB, 0.72661E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.75283E+00_JPRB, 0.75343E+00_JPRB, 0.75377E+00_JPRB, 0.75329E+00_JPRB, 0.75365E+00_JPRB, 0.75377E+00_JPRB, 0.75333E+00_JPRB ,&
&  0.75201E+00_JPRB, 0.75188E+00_JPRB, 0.75260E+00_JPRB, 0.75297E+00_JPRB, 0.75243E+00_JPRB, 0.75681E+00_JPRB, 0.76508E+00_JPRB ,&
&  0.76655E+00_JPRB, 0.76696E+00_JPRB, 0.76614E+00_JPRB, 0.76525E+00_JPRB, 0.76352E+00_JPRB, 0.76093E+00_JPRB, 0.75811E+00_JPRB ,&
&  0.75765E+00_JPRB, 0.75644E+00_JPRB, 0.75421E+00_JPRB, 0.75091E+00_JPRB, 0.75081E+00_JPRB, 0.75061E+00_JPRB, 0.74979E+00_JPRB ,&
&  0.74925E+00_JPRB, 0.74706E+00_JPRB, 0.74489E+00_JPRB, 0.74413E+00_JPRB, 0.74821E+00_JPRB, 0.75275E+00_JPRB, 0.75794E+00_JPRB ,&
&  0.76308E+00_JPRB, 0.77043E+00_JPRB, 0.77828E+00_JPRB, 0.78353E+00_JPRB, 0.78543E+00_JPRB, 0.79054E+00_JPRB, 0.79478E+00_JPRB ,&
&  0.79583E+00_JPRB, 0.79367E+00_JPRB, 0.79225E+00_JPRB, 0.79390E+00_JPRB, 0.79557E+00_JPRB, 0.79650E+00_JPRB, 0.79971E+00_JPRB ,&
&  0.80357E+00_JPRB, 0.80689E+00_JPRB, 0.80915E+00_JPRB, 0.81155E+00_JPRB, 0.81201E+00_JPRB, 0.81082E+00_JPRB, 0.80807E+00_JPRB ,&
&  0.80530E+00_JPRB, 0.80130E+00_JPRB, 0.79662E+00_JPRB, 0.79163E+00_JPRB, 0.78679E+00_JPRB, 0.78091E+00_JPRB, 0.77398E+00_JPRB ,&
&  0.76862E+00_JPRB, 0.76499E+00_JPRB, 0.76372E+00_JPRB, 0.76048E+00_JPRB, 0.75777E+00_JPRB, 0.75492E+00_JPRB, 0.75598E+00_JPRB ,&
&  0.75712E+00_JPRB, 0.76103E+00_JPRB, 0.76458E+00_JPRB, 0.77099E+00_JPRB, 0.77634E+00_JPRB, 0.78145E+00_JPRB, 0.78412E+00_JPRB ,&
&  0.78723E+00_JPRB, 0.78844E+00_JPRB, 0.78815E+00_JPRB, 0.78654E+00_JPRB, 0.78539E+00_JPRB, 0.78335E+00_JPRB, 0.78017E+00_JPRB ,&
&  0.77619E+00_JPRB, 0.77330E+00_JPRB, 0.76976E+00_JPRB, 0.76561E+00_JPRB, 0.76198E+00_JPRB, 0.75855E+00_JPRB, 0.75575E+00_JPRB ,&
&  0.75309E+00_JPRB, 0.75160E+00_JPRB, 0.75062E+00_JPRB, 0.75099E+00_JPRB, 0.75182E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.76698E+00_JPRB, 0.76802E+00_JPRB, 0.76892E+00_JPRB, 0.76956E+00_JPRB, 0.77032E+00_JPRB, 0.77095E+00_JPRB, 0.77143E+00_JPRB ,&
&  0.77138E+00_JPRB, 0.77231E+00_JPRB, 0.77370E+00_JPRB, 0.77501E+00_JPRB, 0.77618E+00_JPRB, 0.77736E+00_JPRB, 0.77832E+00_JPRB ,&
&  0.77998E+00_JPRB, 0.78607E+00_JPRB, 0.79076E+00_JPRB, 0.79321E+00_JPRB, 0.79519E+00_JPRB, 0.79703E+00_JPRB, 0.79857E+00_JPRB ,&
&  0.80013E+00_JPRB, 0.80138E+00_JPRB, 0.80239E+00_JPRB, 0.80307E+00_JPRB, 0.80392E+00_JPRB, 0.80485E+00_JPRB, 0.80584E+00_JPRB ,&
&  0.80704E+00_JPRB, 0.80899E+00_JPRB, 0.81132E+00_JPRB, 0.81378E+00_JPRB, 0.81656E+00_JPRB, 0.82036E+00_JPRB, 0.82374E+00_JPRB ,&
&  0.82650E+00_JPRB, 0.82837E+00_JPRB, 0.82811E+00_JPRB, 0.82619E+00_JPRB, 0.82335E+00_JPRB, 0.82055E+00_JPRB, 0.81750E+00_JPRB ,&
&  0.81380E+00_JPRB, 0.81034E+00_JPRB, 0.80756E+00_JPRB, 0.80510E+00_JPRB, 0.80285E+00_JPRB, 0.80105E+00_JPRB, 0.79943E+00_JPRB ,&
&  0.79795E+00_JPRB, 0.79628E+00_JPRB, 0.79472E+00_JPRB, 0.79315E+00_JPRB, 0.79132E+00_JPRB, 0.78890E+00_JPRB, 0.78646E+00_JPRB ,&
&  0.78448E+00_JPRB, 0.78264E+00_JPRB, 0.78115E+00_JPRB, 0.77948E+00_JPRB, 0.77785E+00_JPRB, 0.77697E+00_JPRB, 0.77645E+00_JPRB ,&
&  0.77591E+00_JPRB, 0.77573E+00_JPRB, 0.77605E+00_JPRB, 0.77593E+00_JPRB, 0.77545E+00_JPRB, 0.77464E+00_JPRB, 0.77399E+00_JPRB ,&
&  0.77341E+00_JPRB, 0.77372E+00_JPRB, 0.77372E+00_JPRB, 0.77448E+00_JPRB, 0.77516E+00_JPRB, 0.77597E+00_JPRB, 0.77684E+00_JPRB ,&
&  0.77728E+00_JPRB, 0.77770E+00_JPRB, 0.77778E+00_JPRB, 0.77808E+00_JPRB, 0.77810E+00_JPRB, 0.77761E+00_JPRB, 0.77663E+00_JPRB ,&
&  0.77554E+00_JPRB, 0.77422E+00_JPRB, 0.77282E+00_JPRB, 0.77113E+00_JPRB, 0.76996E+00_JPRB, 0.76911E+00_JPRB, 0.76815E+00_JPRB ,&
&  0.76712E+00_JPRB, 0.76654E+00_JPRB, 0.76616E+00_JPRB, 0.76629E+00_JPRB, 0.76631E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB, 0.79327E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB ,&
&  0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB, 0.16950E+00_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.17538E+00_JPRB, 0.17536E+00_JPRB, 0.17518E+00_JPRB, 0.17446E+00_JPRB, 0.17224E+00_JPRB, 0.17183E+00_JPRB, 0.17180E+00_JPRB ,&
&  0.17196E+00_JPRB, 0.17207E+00_JPRB, 0.17292E+00_JPRB, 0.17385E+00_JPRB, 0.17477E+00_JPRB, 0.17548E+00_JPRB, 0.17608E+00_JPRB ,&
&  0.17642E+00_JPRB, 0.17675E+00_JPRB, 0.17567E+00_JPRB, 0.17514E+00_JPRB, 0.17498E+00_JPRB, 0.17498E+00_JPRB, 0.17497E+00_JPRB ,&
&  0.17491E+00_JPRB, 0.17489E+00_JPRB, 0.17475E+00_JPRB, 0.17494E+00_JPRB, 0.17524E+00_JPRB, 0.17548E+00_JPRB, 0.17578E+00_JPRB ,&
&  0.17600E+00_JPRB, 0.17620E+00_JPRB, 0.17632E+00_JPRB, 0.17641E+00_JPRB, 0.17672E+00_JPRB, 0.17687E+00_JPRB, 0.17679E+00_JPRB ,&
&  0.17661E+00_JPRB, 0.17626E+00_JPRB, 0.17587E+00_JPRB, 0.17546E+00_JPRB, 0.17516E+00_JPRB, 0.17488E+00_JPRB, 0.17460E+00_JPRB ,&
&  0.17431E+00_JPRB, 0.17401E+00_JPRB, 0.17368E+00_JPRB, 0.17331E+00_JPRB, 0.17296E+00_JPRB, 0.17270E+00_JPRB, 0.17243E+00_JPRB ,&
&  0.17220E+00_JPRB, 0.17198E+00_JPRB, 0.17174E+00_JPRB, 0.17143E+00_JPRB, 0.17111E+00_JPRB, 0.17079E+00_JPRB, 0.17044E+00_JPRB ,&
&  0.16999E+00_JPRB, 0.16968E+00_JPRB, 0.16936E+00_JPRB, 0.16909E+00_JPRB, 0.16885E+00_JPRB, 0.16861E+00_JPRB, 0.16839E+00_JPRB ,&
&  0.16815E+00_JPRB, 0.16792E+00_JPRB, 0.16766E+00_JPRB, 0.16742E+00_JPRB, 0.16720E+00_JPRB, 0.16700E+00_JPRB, 0.16684E+00_JPRB ,&
&  0.16669E+00_JPRB, 0.16653E+00_JPRB, 0.16640E+00_JPRB, 0.16641E+00_JPRB, 0.16639E+00_JPRB, 0.16641E+00_JPRB, 0.16643E+00_JPRB ,&
&  0.16646E+00_JPRB, 0.16648E+00_JPRB, 0.16649E+00_JPRB, 0.16638E+00_JPRB, 0.16652E+00_JPRB, 0.16673E+00_JPRB, 0.16704E+00_JPRB ,&
&  0.16741E+00_JPRB, 0.16787E+00_JPRB, 0.16837E+00_JPRB, 0.16889E+00_JPRB, 0.16950E+00_JPRB, 0.17001E+00_JPRB, 0.17057E+00_JPRB ,&
&  0.17121E+00_JPRB, 0.17188E+00_JPRB, 0.17236E+00_JPRB, 0.17293E+00_JPRB, 0.17379E+00_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.18635E+00_JPRB, 0.18925E+00_JPRB, 0.19108E+00_JPRB, 0.19265E+00_JPRB, 0.18680E+00_JPRB, 0.19812E+00_JPRB, 0.19488E+00_JPRB ,&
&  0.19433E+00_JPRB, 0.19305E+00_JPRB, 0.19471E+00_JPRB, 0.19476E+00_JPRB, 0.19414E+00_JPRB, 0.19322E+00_JPRB, 0.19204E+00_JPRB ,&
&  0.19091E+00_JPRB, 0.18949E+00_JPRB, 0.18858E+00_JPRB, 0.18758E+00_JPRB, 0.18680E+00_JPRB, 0.18602E+00_JPRB, 0.18548E+00_JPRB ,&
&  0.18548E+00_JPRB, 0.18628E+00_JPRB, 0.18769E+00_JPRB, 0.18709E+00_JPRB, 0.18709E+00_JPRB, 0.18747E+00_JPRB, 0.18806E+00_JPRB ,&
&  0.18831E+00_JPRB, 0.18841E+00_JPRB, 0.18812E+00_JPRB, 0.18705E+00_JPRB, 0.18704E+00_JPRB, 0.18708E+00_JPRB, 0.18720E+00_JPRB ,&
&  0.18713E+00_JPRB, 0.18684E+00_JPRB, 0.18597E+00_JPRB, 0.18517E+00_JPRB, 0.18434E+00_JPRB, 0.18335E+00_JPRB, 0.18216E+00_JPRB ,&
&  0.18118E+00_JPRB, 0.18042E+00_JPRB, 0.17950E+00_JPRB, 0.17860E+00_JPRB, 0.17782E+00_JPRB, 0.17706E+00_JPRB, 0.17635E+00_JPRB ,&
&  0.17589E+00_JPRB, 0.17556E+00_JPRB, 0.17518E+00_JPRB, 0.17464E+00_JPRB, 0.17402E+00_JPRB, 0.17343E+00_JPRB, 0.17291E+00_JPRB ,&
&  0.17256E+00_JPRB, 0.17199E+00_JPRB, 0.17142E+00_JPRB, 0.17080E+00_JPRB, 0.17033E+00_JPRB, 0.16955E+00_JPRB, 0.16888E+00_JPRB ,&
&  0.16822E+00_JPRB, 0.16762E+00_JPRB, 0.16717E+00_JPRB, 0.16684E+00_JPRB, 0.16654E+00_JPRB, 0.16634E+00_JPRB, 0.16619E+00_JPRB ,&
&  0.16610E+00_JPRB, 0.16603E+00_JPRB, 0.16606E+00_JPRB, 0.16601E+00_JPRB, 0.16593E+00_JPRB, 0.16592E+00_JPRB, 0.16620E+00_JPRB ,&
&  0.16644E+00_JPRB, 0.16661E+00_JPRB, 0.16671E+00_JPRB, 0.16642E+00_JPRB, 0.16681E+00_JPRB, 0.16716E+00_JPRB, 0.16757E+00_JPRB ,&
&  0.16794E+00_JPRB, 0.16872E+00_JPRB, 0.16966E+00_JPRB, 0.17080E+00_JPRB, 0.17298E+00_JPRB, 0.17492E+00_JPRB, 0.17697E+00_JPRB ,&
&  0.17928E+00_JPRB, 0.18261E+00_JPRB, 0.18401E+00_JPRB, 0.18494E+00_JPRB, 0.18567E+00_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.21124E+00_JPRB, 0.21279E+00_JPRB, 0.21430E+00_JPRB, 0.21571E+00_JPRB, 0.21732E+00_JPRB, 0.21798E+00_JPRB, 0.21794E+00_JPRB ,&
&  0.21562E+00_JPRB, 0.21508E+00_JPRB, 0.21359E+00_JPRB, 0.21170E+00_JPRB, 0.21019E+00_JPRB, 0.20726E+00_JPRB, 0.20448E+00_JPRB ,&
&  0.20167E+00_JPRB, 0.19896E+00_JPRB, 0.19657E+00_JPRB, 0.19326E+00_JPRB, 0.19010E+00_JPRB, 0.18698E+00_JPRB, 0.18606E+00_JPRB ,&
&  0.18571E+00_JPRB, 0.18623E+00_JPRB, 0.18795E+00_JPRB, 0.18934E+00_JPRB, 0.19137E+00_JPRB, 0.19353E+00_JPRB, 0.19679E+00_JPRB ,&
&  0.19992E+00_JPRB, 0.20223E+00_JPRB, 0.20254E+00_JPRB, 0.20137E+00_JPRB, 0.20040E+00_JPRB, 0.20012E+00_JPRB, 0.19973E+00_JPRB ,&
&  0.19897E+00_JPRB, 0.19854E+00_JPRB, 0.19793E+00_JPRB, 0.19741E+00_JPRB, 0.19598E+00_JPRB, 0.19346E+00_JPRB, 0.19090E+00_JPRB ,&
&  0.18852E+00_JPRB, 0.18641E+00_JPRB, 0.18441E+00_JPRB, 0.18267E+00_JPRB, 0.18164E+00_JPRB, 0.18061E+00_JPRB, 0.17932E+00_JPRB ,&
&  0.17856E+00_JPRB, 0.17775E+00_JPRB, 0.17683E+00_JPRB, 0.17615E+00_JPRB, 0.17538E+00_JPRB, 0.17484E+00_JPRB, 0.17422E+00_JPRB ,&
&  0.17374E+00_JPRB, 0.17301E+00_JPRB, 0.17238E+00_JPRB, 0.17169E+00_JPRB, 0.17137E+00_JPRB, 0.17059E+00_JPRB, 0.16986E+00_JPRB ,&
&  0.16894E+00_JPRB, 0.16837E+00_JPRB, 0.16733E+00_JPRB, 0.16671E+00_JPRB, 0.16598E+00_JPRB, 0.16532E+00_JPRB, 0.16486E+00_JPRB ,&
&  0.16492E+00_JPRB, 0.16492E+00_JPRB, 0.16448E+00_JPRB, 0.16440E+00_JPRB, 0.16444E+00_JPRB, 0.16486E+00_JPRB, 0.16616E+00_JPRB ,&
&  0.16700E+00_JPRB, 0.16792E+00_JPRB, 0.16881E+00_JPRB, 0.16950E+00_JPRB, 0.17067E+00_JPRB, 0.17179E+00_JPRB, 0.17295E+00_JPRB ,&
&  0.17430E+00_JPRB, 0.17547E+00_JPRB, 0.17684E+00_JPRB, 0.17846E+00_JPRB, 0.18240E+00_JPRB, 0.18514E+00_JPRB, 0.18803E+00_JPRB ,&
&  0.19009E+00_JPRB, 0.18884E+00_JPRB, 0.20289E+00_JPRB, 0.20561E+00_JPRB, 0.20766E+00_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.22069E+00_JPRB, 0.22157E+00_JPRB, 0.22154E+00_JPRB, 0.22252E+00_JPRB, 0.22333E+00_JPRB, 0.22439E+00_JPRB, 0.22453E+00_JPRB ,&
&  0.22397E+00_JPRB, 0.21980E+00_JPRB, 0.21751E+00_JPRB, 0.21421E+00_JPRB, 0.21108E+00_JPRB, 0.20780E+00_JPRB, 0.20509E+00_JPRB ,&
&  0.20334E+00_JPRB, 0.20198E+00_JPRB, 0.20072E+00_JPRB, 0.19900E+00_JPRB, 0.19690E+00_JPRB, 0.19476E+00_JPRB, 0.19282E+00_JPRB ,&
&  0.19106E+00_JPRB, 0.19110E+00_JPRB, 0.19232E+00_JPRB, 0.19449E+00_JPRB, 0.19847E+00_JPRB, 0.20153E+00_JPRB, 0.20547E+00_JPRB ,&
&  0.21163E+00_JPRB, 0.20844E+00_JPRB, 0.20861E+00_JPRB, 0.20889E+00_JPRB, 0.20863E+00_JPRB, 0.20846E+00_JPRB, 0.20559E+00_JPRB ,&
&  0.20915E+00_JPRB, 0.20936E+00_JPRB, 0.21281E+00_JPRB, 0.21189E+00_JPRB, 0.20730E+00_JPRB, 0.20367E+00_JPRB, 0.19870E+00_JPRB ,&
&  0.19515E+00_JPRB, 0.19070E+00_JPRB, 0.18771E+00_JPRB, 0.18499E+00_JPRB, 0.18178E+00_JPRB, 0.18045E+00_JPRB, 0.17822E+00_JPRB ,&
&  0.17687E+00_JPRB, 0.17658E+00_JPRB, 0.17561E+00_JPRB, 0.17557E+00_JPRB, 0.17475E+00_JPRB, 0.17360E+00_JPRB, 0.17284E+00_JPRB ,&
&  0.17216E+00_JPRB, 0.17144E+00_JPRB, 0.17119E+00_JPRB, 0.17093E+00_JPRB, 0.17105E+00_JPRB, 0.17106E+00_JPRB, 0.17110E+00_JPRB ,&
&  0.17130E+00_JPRB, 0.17144E+00_JPRB, 0.17103E+00_JPRB, 0.17087E+00_JPRB, 0.16997E+00_JPRB, 0.16919E+00_JPRB, 0.16797E+00_JPRB ,&
&  0.16722E+00_JPRB, 0.16677E+00_JPRB, 0.16610E+00_JPRB, 0.16567E+00_JPRB, 0.16555E+00_JPRB, 0.16604E+00_JPRB, 0.16667E+00_JPRB ,&
&  0.16738E+00_JPRB, 0.16869E+00_JPRB, 0.16976E+00_JPRB, 0.17120E+00_JPRB, 0.17279E+00_JPRB, 0.17404E+00_JPRB, 0.17584E+00_JPRB ,&
&  0.17762E+00_JPRB, 0.17946E+00_JPRB, 0.18258E+00_JPRB, 0.18604E+00_JPRB, 0.19063E+00_JPRB, 0.19572E+00_JPRB, 0.19849E+00_JPRB ,&
&  0.20160E+00_JPRB, 0.20841E+00_JPRB, 0.21107E+00_JPRB, 0.21861E+00_JPRB, 0.21983E+00_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.22609E+00_JPRB, 0.22602E+00_JPRB, 0.22528E+00_JPRB, 0.22480E+00_JPRB, 0.22527E+00_JPRB, 0.22605E+00_JPRB, 0.22705E+00_JPRB ,&
&  0.22360E+00_JPRB, 0.22101E+00_JPRB, 0.21671E+00_JPRB, 0.21194E+00_JPRB, 0.20774E+00_JPRB, 0.20442E+00_JPRB, 0.20249E+00_JPRB ,&
&  0.20227E+00_JPRB, 0.20430E+00_JPRB, 0.20589E+00_JPRB, 0.20784E+00_JPRB, 0.20794E+00_JPRB, 0.20808E+00_JPRB, 0.20668E+00_JPRB ,&
&  0.20518E+00_JPRB, 0.20458E+00_JPRB, 0.20483E+00_JPRB, 0.20652E+00_JPRB, 0.20919E+00_JPRB, 0.21098E+00_JPRB, 0.21313E+00_JPRB ,&
&  0.21601E+00_JPRB, 0.21535E+00_JPRB, 0.21608E+00_JPRB, 0.21549E+00_JPRB, 0.21580E+00_JPRB, 0.21672E+00_JPRB, 0.21717E+00_JPRB ,&
&  0.21810E+00_JPRB, 0.21902E+00_JPRB, 0.21893E+00_JPRB, 0.21818E+00_JPRB, 0.21392E+00_JPRB, 0.21128E+00_JPRB, 0.20662E+00_JPRB ,&
&  0.20110E+00_JPRB, 0.19620E+00_JPRB, 0.19061E+00_JPRB, 0.18669E+00_JPRB, 0.18207E+00_JPRB, 0.17934E+00_JPRB, 0.17722E+00_JPRB ,&
&  0.17523E+00_JPRB, 0.17447E+00_JPRB, 0.17341E+00_JPRB, 0.17225E+00_JPRB, 0.17199E+00_JPRB, 0.17085E+00_JPRB, 0.17076E+00_JPRB ,&
&  0.17105E+00_JPRB, 0.17130E+00_JPRB, 0.17226E+00_JPRB, 0.17272E+00_JPRB, 0.17351E+00_JPRB, 0.17425E+00_JPRB, 0.17480E+00_JPRB ,&
&  0.17592E+00_JPRB, 0.17734E+00_JPRB, 0.17733E+00_JPRB, 0.17761E+00_JPRB, 0.17677E+00_JPRB, 0.17500E+00_JPRB, 0.17307E+00_JPRB ,&
&  0.17100E+00_JPRB, 0.16939E+00_JPRB, 0.16889E+00_JPRB, 0.16818E+00_JPRB, 0.16833E+00_JPRB, 0.16870E+00_JPRB, 0.16888E+00_JPRB ,&
&  0.16955E+00_JPRB, 0.17015E+00_JPRB, 0.17099E+00_JPRB, 0.17284E+00_JPRB, 0.17415E+00_JPRB, 0.17582E+00_JPRB, 0.17757E+00_JPRB ,&
&  0.17895E+00_JPRB, 0.18146E+00_JPRB, 0.18368E+00_JPRB, 0.18673E+00_JPRB, 0.19374E+00_JPRB, 0.19989E+00_JPRB, 0.21087E+00_JPRB ,&
&  0.21606E+00_JPRB, 0.22745E+00_JPRB, 0.22550E+00_JPRB, 0.22536E+00_JPRB, 0.22646E+00_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.23312E+00_JPRB, 0.23195E+00_JPRB, 0.23069E+00_JPRB, 0.22943E+00_JPRB, 0.23030E+00_JPRB, 0.23146E+00_JPRB, 0.23294E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.23506E+00_JPRB, 0.23492E+00_JPRB, 0.23274E+00_JPRB, 0.23146E+00_JPRB, 0.22526E+00_JPRB, 0.22302E+00_JPRB ,&
&  0.22135E+00_JPRB, 0.22317E+00_JPRB, 0.22569E+00_JPRB, 0.22954E+00_JPRB, 0.23116E+00_JPRB, 0.23288E+00_JPRB, 0.23195E+00_JPRB ,&
&  0.22979E+00_JPRB, 0.22718E+00_JPRB, 0.22467E+00_JPRB, 0.22268E+00_JPRB, 0.22210E+00_JPRB, 0.22051E+00_JPRB, 0.21882E+00_JPRB ,&
&  0.21881E+00_JPRB, 0.22025E+00_JPRB, 0.22106E+00_JPRB, 0.22188E+00_JPRB, 0.22783E+00_JPRB, 0.22845E+00_JPRB, 0.23054E+00_JPRB ,&
&  0.23111E+00_JPRB, 0.23170E+00_JPRB, 0.23104E+00_JPRB, 0.22967E+00_JPRB, 0.22838E+00_JPRB, 0.22304E+00_JPRB, 0.22064E+00_JPRB ,&
&  0.21340E+00_JPRB, 0.20914E+00_JPRB, 0.20240E+00_JPRB, 0.19635E+00_JPRB, 0.18974E+00_JPRB, 0.18478E+00_JPRB, 0.18064E+00_JPRB ,&
&  0.17736E+00_JPRB, 0.17491E+00_JPRB, 0.17287E+00_JPRB, 0.17258E+00_JPRB, 0.17264E+00_JPRB, 0.17226E+00_JPRB, 0.17242E+00_JPRB ,&
&  0.17323E+00_JPRB, 0.17412E+00_JPRB, 0.17406E+00_JPRB, 0.17464E+00_JPRB, 0.17517E+00_JPRB, 0.17594E+00_JPRB, 0.17739E+00_JPRB ,&
&  0.17963E+00_JPRB, 0.18156E+00_JPRB, 0.18229E+00_JPRB, 0.18408E+00_JPRB, 0.18340E+00_JPRB, 0.18076E+00_JPRB, 0.17816E+00_JPRB ,&
&  0.17528E+00_JPRB, 0.17401E+00_JPRB, 0.17263E+00_JPRB, 0.17183E+00_JPRB, 0.17172E+00_JPRB, 0.17190E+00_JPRB, 0.17268E+00_JPRB ,&
&  0.17352E+00_JPRB, 0.17441E+00_JPRB, 0.17478E+00_JPRB, 0.17525E+00_JPRB, 0.17569E+00_JPRB, 0.17653E+00_JPRB, 0.17811E+00_JPRB ,&
&  0.18026E+00_JPRB, 0.18274E+00_JPRB, 0.18514E+00_JPRB, 0.18809E+00_JPRB, 0.19295E+00_JPRB, 0.19948E+00_JPRB, 0.20934E+00_JPRB ,&
&  0.21723E+00_JPRB, 0.22981E+00_JPRB, 0.23007E+00_JPRB, 0.23199E+00_JPRB, 0.23342E+00_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.24265E+00_JPRB, 0.24326E+00_JPRB, 0.23969E+00_JPRB, 0.23654E+00_JPRB, 0.23500E+00_JPRB, 0.23453E+00_JPRB, 0.23572E+00_JPRB ,&
&  0.23862E+00_JPRB, 0.24085E+00_JPRB, 0.24314E+00_JPRB, 0.24513E+00_JPRB, 0.24596E+00_JPRB, 0.24576E+00_JPRB, 0.24666E+00_JPRB ,&
&  0.24752E+00_JPRB, 0.24991E+00_JPRB, 0.25123E+00_JPRB, 0.25367E+00_JPRB, 0.25380E+00_JPRB, 0.25445E+00_JPRB, 0.25562E+00_JPRB ,&
&  0.25518E+00_JPRB, 0.25572E+00_JPRB, 0.25716E+00_JPRB, 0.25496E+00_JPRB, 0.25351E+00_JPRB, 0.25134E+00_JPRB, 0.24962E+00_JPRB ,&
&  0.24348E+00_JPRB, 0.23698E+00_JPRB, 0.23039E+00_JPRB, 0.23384E+00_JPRB, 0.24178E+00_JPRB, 0.24365E+00_JPRB, 0.24757E+00_JPRB ,&
&  0.24869E+00_JPRB, 0.24701E+00_JPRB, 0.24525E+00_JPRB, 0.24200E+00_JPRB, 0.23885E+00_JPRB, 0.23531E+00_JPRB, 0.23249E+00_JPRB ,&
&  0.22940E+00_JPRB, 0.22993E+00_JPRB, 0.22601E+00_JPRB, 0.21271E+00_JPRB, 0.20571E+00_JPRB, 0.20137E+00_JPRB, 0.19459E+00_JPRB ,&
&  0.18781E+00_JPRB, 0.18584E+00_JPRB, 0.18538E+00_JPRB, 0.18309E+00_JPRB, 0.18297E+00_JPRB, 0.18328E+00_JPRB, 0.18321E+00_JPRB ,&
&  0.18192E+00_JPRB, 0.18166E+00_JPRB, 0.18037E+00_JPRB, 0.17846E+00_JPRB, 0.17664E+00_JPRB, 0.17591E+00_JPRB, 0.17632E+00_JPRB ,&
&  0.17778E+00_JPRB, 0.17938E+00_JPRB, 0.18485E+00_JPRB, 0.18833E+00_JPRB, 0.18375E+00_JPRB, 0.18104E+00_JPRB, 0.18129E+00_JPRB ,&
&  0.17747E+00_JPRB, 0.17671E+00_JPRB, 0.17669E+00_JPRB, 0.17664E+00_JPRB, 0.17698E+00_JPRB, 0.17786E+00_JPRB, 0.17779E+00_JPRB ,&
&  0.17842E+00_JPRB, 0.17971E+00_JPRB, 0.18007E+00_JPRB, 0.18012E+00_JPRB, 0.17990E+00_JPRB, 0.18036E+00_JPRB, 0.18158E+00_JPRB ,&
&  0.18299E+00_JPRB, 0.18560E+00_JPRB, 0.18836E+00_JPRB, 0.19237E+00_JPRB, 0.19556E+00_JPRB, 0.19807E+00_JPRB, 0.20259E+00_JPRB ,&
&  0.20660E+00_JPRB, 0.21573E+00_JPRB, 0.23125E+00_JPRB, 0.24177E+00_JPRB, 0.23846E+00_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.24991E+00_JPRB, 0.24994E+00_JPRB, 0.24894E+00_JPRB, 0.24795E+00_JPRB, 0.24646E+00_JPRB, 0.24495E+00_JPRB, 0.24553E+00_JPRB ,&
&  0.24791E+00_JPRB, 0.25105E+00_JPRB, 0.25411E+00_JPRB, 0.25648E+00_JPRB, 0.25833E+00_JPRB, 0.25918E+00_JPRB, 0.26056E+00_JPRB ,&
&  0.26215E+00_JPRB, 0.26486E+00_JPRB, 0.26559E+00_JPRB, 0.26622E+00_JPRB, 0.26535E+00_JPRB, 0.26456E+00_JPRB, 0.26558E+00_JPRB ,&
&  0.26873E+00_JPRB, 0.27022E+00_JPRB, 0.26938E+00_JPRB, 0.26731E+00_JPRB, 0.26785E+00_JPRB, 0.26981E+00_JPRB, 0.26931E+00_JPRB ,&
&  0.26307E+00_JPRB, 0.24983E+00_JPRB, 0.24062E+00_JPRB, 0.24343E+00_JPRB, 0.25222E+00_JPRB, 0.25627E+00_JPRB, 0.25904E+00_JPRB ,&
&  0.26079E+00_JPRB, 0.26103E+00_JPRB, 0.26019E+00_JPRB, 0.25862E+00_JPRB, 0.25595E+00_JPRB, 0.25135E+00_JPRB, 0.24590E+00_JPRB ,&
&  0.24100E+00_JPRB, 0.23849E+00_JPRB, 0.23555E+00_JPRB, 0.23038E+00_JPRB, 0.22526E+00_JPRB, 0.22202E+00_JPRB, 0.21826E+00_JPRB ,&
&  0.21443E+00_JPRB, 0.21147E+00_JPRB, 0.21072E+00_JPRB, 0.21029E+00_JPRB, 0.21018E+00_JPRB, 0.20963E+00_JPRB, 0.20744E+00_JPRB ,&
&  0.20408E+00_JPRB, 0.19986E+00_JPRB, 0.19426E+00_JPRB, 0.18921E+00_JPRB, 0.18570E+00_JPRB, 0.18221E+00_JPRB, 0.17803E+00_JPRB ,&
&  0.17590E+00_JPRB, 0.17825E+00_JPRB, 0.18668E+00_JPRB, 0.19610E+00_JPRB, 0.20130E+00_JPRB, 0.18976E+00_JPRB, 0.18562E+00_JPRB ,&
&  0.18468E+00_JPRB, 0.18437E+00_JPRB, 0.18465E+00_JPRB, 0.18602E+00_JPRB, 0.18671E+00_JPRB, 0.18648E+00_JPRB, 0.18702E+00_JPRB ,&
&  0.18893E+00_JPRB, 0.19091E+00_JPRB, 0.19197E+00_JPRB, 0.19314E+00_JPRB, 0.19386E+00_JPRB, 0.19367E+00_JPRB, 0.19336E+00_JPRB ,&
&  0.19399E+00_JPRB, 0.19688E+00_JPRB, 0.20166E+00_JPRB, 0.20847E+00_JPRB, 0.21343E+00_JPRB, 0.21801E+00_JPRB, 0.22390E+00_JPRB ,&
&  0.23142E+00_JPRB, 0.23891E+00_JPRB, 0.24312E+00_JPRB, 0.24693E+00_JPRB, 0.24881E+00_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.25794E+00_JPRB, 0.25952E+00_JPRB, 0.26179E+00_JPRB, 0.26396E+00_JPRB, 0.26482E+00_JPRB, 0.26429E+00_JPRB, 0.26356E+00_JPRB ,&
&  0.26395E+00_JPRB, 0.26592E+00_JPRB, 0.26804E+00_JPRB, 0.26852E+00_JPRB, 0.26873E+00_JPRB, 0.26903E+00_JPRB, 0.27066E+00_JPRB ,&
&  0.27315E+00_JPRB, 0.27597E+00_JPRB, 0.27567E+00_JPRB, 0.27510E+00_JPRB, 0.27454E+00_JPRB, 0.27343E+00_JPRB, 0.27449E+00_JPRB ,&
&  0.27739E+00_JPRB, 0.27752E+00_JPRB, 0.27733E+00_JPRB, 0.27661E+00_JPRB, 0.27827E+00_JPRB, 0.28148E+00_JPRB, 0.28340E+00_JPRB ,&
&  0.28220E+00_JPRB, 0.27557E+00_JPRB, 0.26745E+00_JPRB, 0.26712E+00_JPRB, 0.26885E+00_JPRB, 0.27061E+00_JPRB, 0.27086E+00_JPRB ,&
&  0.27045E+00_JPRB, 0.26964E+00_JPRB, 0.26859E+00_JPRB, 0.26753E+00_JPRB, 0.26619E+00_JPRB, 0.26241E+00_JPRB, 0.25801E+00_JPRB ,&
&  0.25447E+00_JPRB, 0.25117E+00_JPRB, 0.24793E+00_JPRB, 0.24361E+00_JPRB, 0.23918E+00_JPRB, 0.23595E+00_JPRB, 0.23231E+00_JPRB ,&
&  0.22783E+00_JPRB, 0.22461E+00_JPRB, 0.22362E+00_JPRB, 0.22356E+00_JPRB, 0.22368E+00_JPRB, 0.22290E+00_JPRB, 0.22193E+00_JPRB ,&
&  0.22023E+00_JPRB, 0.21891E+00_JPRB, 0.21742E+00_JPRB, 0.21662E+00_JPRB, 0.21162E+00_JPRB, 0.20344E+00_JPRB, 0.19769E+00_JPRB ,&
&  0.19695E+00_JPRB, 0.19960E+00_JPRB, 0.20323E+00_JPRB, 0.20391E+00_JPRB, 0.20577E+00_JPRB, 0.20673E+00_JPRB, 0.20636E+00_JPRB ,&
&  0.20380E+00_JPRB, 0.20010E+00_JPRB, 0.20023E+00_JPRB, 0.20125E+00_JPRB, 0.20175E+00_JPRB, 0.20296E+00_JPRB, 0.20450E+00_JPRB ,&
&  0.20733E+00_JPRB, 0.20963E+00_JPRB, 0.21179E+00_JPRB, 0.21403E+00_JPRB, 0.21542E+00_JPRB, 0.21512E+00_JPRB, 0.21314E+00_JPRB ,&
&  0.21305E+00_JPRB, 0.21843E+00_JPRB, 0.22808E+00_JPRB, 0.23855E+00_JPRB, 0.24101E+00_JPRB, 0.24329E+00_JPRB, 0.24540E+00_JPRB ,&
&  0.24654E+00_JPRB, 0.24991E+00_JPRB, 0.25296E+00_JPRB, 0.25486E+00_JPRB, 0.25623E+00_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.27018E+00_JPRB, 0.27221E+00_JPRB, 0.27313E+00_JPRB, 0.27411E+00_JPRB, 0.27631E+00_JPRB, 0.27786E+00_JPRB, 0.27812E+00_JPRB ,&
&  0.27841E+00_JPRB, 0.27974E+00_JPRB, 0.28068E+00_JPRB, 0.28107E+00_JPRB, 0.27871E+00_JPRB, 0.27828E+00_JPRB, 0.27984E+00_JPRB ,&
&  0.28195E+00_JPRB, 0.28400E+00_JPRB, 0.28395E+00_JPRB, 0.28366E+00_JPRB, 0.28296E+00_JPRB, 0.28174E+00_JPRB, 0.28315E+00_JPRB ,&
&  0.28431E+00_JPRB, 0.28478E+00_JPRB, 0.28609E+00_JPRB, 0.28747E+00_JPRB, 0.29050E+00_JPRB, 0.29407E+00_JPRB, 0.29638E+00_JPRB ,&
&  0.29687E+00_JPRB, 0.29533E+00_JPRB, 0.28873E+00_JPRB, 0.28186E+00_JPRB, 0.28221E+00_JPRB, 0.28334E+00_JPRB, 0.28332E+00_JPRB ,&
&  0.28130E+00_JPRB, 0.27852E+00_JPRB, 0.27664E+00_JPRB, 0.27514E+00_JPRB, 0.27324E+00_JPRB, 0.26989E+00_JPRB, 0.26625E+00_JPRB ,&
&  0.26308E+00_JPRB, 0.26195E+00_JPRB, 0.26054E+00_JPRB, 0.25821E+00_JPRB, 0.25478E+00_JPRB, 0.25199E+00_JPRB, 0.24912E+00_JPRB ,&
&  0.24409E+00_JPRB, 0.23922E+00_JPRB, 0.23657E+00_JPRB, 0.23593E+00_JPRB, 0.23628E+00_JPRB, 0.23583E+00_JPRB, 0.23410E+00_JPRB ,&
&  0.23211E+00_JPRB, 0.22959E+00_JPRB, 0.22803E+00_JPRB, 0.22755E+00_JPRB, 0.22864E+00_JPRB, 0.22743E+00_JPRB, 0.22311E+00_JPRB ,&
&  0.22062E+00_JPRB, 0.21905E+00_JPRB, 0.21912E+00_JPRB, 0.21808E+00_JPRB, 0.21717E+00_JPRB, 0.21592E+00_JPRB, 0.21635E+00_JPRB ,&
&  0.21740E+00_JPRB, 0.21772E+00_JPRB, 0.22027E+00_JPRB, 0.22480E+00_JPRB, 0.22894E+00_JPRB, 0.23135E+00_JPRB, 0.23277E+00_JPRB ,&
&  0.23328E+00_JPRB, 0.23263E+00_JPRB, 0.23201E+00_JPRB, 0.23251E+00_JPRB, 0.23449E+00_JPRB, 0.23721E+00_JPRB, 0.23986E+00_JPRB ,&
&  0.24064E+00_JPRB, 0.24178E+00_JPRB, 0.24438E+00_JPRB, 0.24830E+00_JPRB, 0.25066E+00_JPRB, 0.25260E+00_JPRB, 0.25433E+00_JPRB ,&
&  0.25531E+00_JPRB, 0.25699E+00_JPRB, 0.26102E+00_JPRB, 0.26385E+00_JPRB, 0.26666E+00_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.28183E+00_JPRB, 0.28265E+00_JPRB, 0.28325E+00_JPRB, 0.28304E+00_JPRB, 0.28407E+00_JPRB, 0.28496E+00_JPRB, 0.28654E+00_JPRB ,&
&  0.28737E+00_JPRB, 0.28900E+00_JPRB, 0.29121E+00_JPRB, 0.29118E+00_JPRB, 0.29078E+00_JPRB, 0.29049E+00_JPRB, 0.29099E+00_JPRB ,&
&  0.29315E+00_JPRB, 0.29422E+00_JPRB, 0.29354E+00_JPRB, 0.29318E+00_JPRB, 0.29228E+00_JPRB, 0.29151E+00_JPRB, 0.29342E+00_JPRB ,&
&  0.29382E+00_JPRB, 0.29447E+00_JPRB, 0.29630E+00_JPRB, 0.29788E+00_JPRB, 0.30167E+00_JPRB, 0.30638E+00_JPRB, 0.30988E+00_JPRB ,&
&  0.31018E+00_JPRB, 0.30712E+00_JPRB, 0.30183E+00_JPRB, 0.29785E+00_JPRB, 0.29718E+00_JPRB, 0.29823E+00_JPRB, 0.30067E+00_JPRB ,&
&  0.29996E+00_JPRB, 0.29436E+00_JPRB, 0.28713E+00_JPRB, 0.28313E+00_JPRB, 0.28153E+00_JPRB, 0.27938E+00_JPRB, 0.27597E+00_JPRB ,&
&  0.27323E+00_JPRB, 0.27246E+00_JPRB, 0.27255E+00_JPRB, 0.27175E+00_JPRB, 0.27006E+00_JPRB, 0.26714E+00_JPRB, 0.26339E+00_JPRB ,&
&  0.26009E+00_JPRB, 0.25682E+00_JPRB, 0.25477E+00_JPRB, 0.25333E+00_JPRB, 0.25316E+00_JPRB, 0.25329E+00_JPRB, 0.25242E+00_JPRB ,&
&  0.25052E+00_JPRB, 0.24930E+00_JPRB, 0.24764E+00_JPRB, 0.24448E+00_JPRB, 0.24221E+00_JPRB, 0.24147E+00_JPRB, 0.24035E+00_JPRB ,&
&  0.23877E+00_JPRB, 0.23818E+00_JPRB, 0.23795E+00_JPRB, 0.23745E+00_JPRB, 0.23828E+00_JPRB, 0.23995E+00_JPRB, 0.23756E+00_JPRB ,&
&  0.23545E+00_JPRB, 0.23407E+00_JPRB, 0.23418E+00_JPRB, 0.23575E+00_JPRB, 0.23835E+00_JPRB, 0.24118E+00_JPRB, 0.24352E+00_JPRB ,&
&  0.24450E+00_JPRB, 0.24479E+00_JPRB, 0.24513E+00_JPRB, 0.24526E+00_JPRB, 0.24634E+00_JPRB, 0.24858E+00_JPRB, 0.25132E+00_JPRB ,&
&  0.25275E+00_JPRB, 0.25527E+00_JPRB, 0.25800E+00_JPRB, 0.26140E+00_JPRB, 0.26341E+00_JPRB, 0.26535E+00_JPRB, 0.26735E+00_JPRB ,&
&  0.26900E+00_JPRB, 0.27112E+00_JPRB, 0.27418E+00_JPRB, 0.27668E+00_JPRB, 0.27931E+00_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.29681E+00_JPRB, 0.29519E+00_JPRB, 0.29392E+00_JPRB, 0.29422E+00_JPRB, 0.29647E+00_JPRB, 0.29705E+00_JPRB, 0.29704E+00_JPRB ,&
&  0.29688E+00_JPRB, 0.29888E+00_JPRB, 0.30254E+00_JPRB, 0.30483E+00_JPRB, 0.30516E+00_JPRB, 0.30495E+00_JPRB, 0.30443E+00_JPRB ,&
&  0.30407E+00_JPRB, 0.30372E+00_JPRB, 0.30249E+00_JPRB, 0.30247E+00_JPRB, 0.30243E+00_JPRB, 0.30147E+00_JPRB, 0.30224E+00_JPRB ,&
&  0.30226E+00_JPRB, 0.30265E+00_JPRB, 0.30482E+00_JPRB, 0.30660E+00_JPRB, 0.31048E+00_JPRB, 0.31674E+00_JPRB, 0.32006E+00_JPRB ,&
&  0.31981E+00_JPRB, 0.31560E+00_JPRB, 0.31212E+00_JPRB, 0.31278E+00_JPRB, 0.31474E+00_JPRB, 0.31880E+00_JPRB, 0.32330E+00_JPRB ,&
&  0.32470E+00_JPRB, 0.31903E+00_JPRB, 0.30976E+00_JPRB, 0.30290E+00_JPRB, 0.29823E+00_JPRB, 0.29493E+00_JPRB, 0.29203E+00_JPRB ,&
&  0.28878E+00_JPRB, 0.28646E+00_JPRB, 0.28539E+00_JPRB, 0.28522E+00_JPRB, 0.28485E+00_JPRB, 0.28202E+00_JPRB, 0.27881E+00_JPRB ,&
&  0.27465E+00_JPRB, 0.27175E+00_JPRB, 0.26937E+00_JPRB, 0.26854E+00_JPRB, 0.26861E+00_JPRB, 0.26832E+00_JPRB, 0.26728E+00_JPRB ,&
&  0.26597E+00_JPRB, 0.26555E+00_JPRB, 0.26515E+00_JPRB, 0.26395E+00_JPRB, 0.26249E+00_JPRB, 0.26207E+00_JPRB, 0.26100E+00_JPRB ,&
&  0.25888E+00_JPRB, 0.25764E+00_JPRB, 0.25773E+00_JPRB, 0.25877E+00_JPRB, 0.25875E+00_JPRB, 0.25956E+00_JPRB, 0.26112E+00_JPRB ,&
&  0.26023E+00_JPRB, 0.25780E+00_JPRB, 0.25657E+00_JPRB, 0.25712E+00_JPRB, 0.25894E+00_JPRB, 0.26135E+00_JPRB, 0.26257E+00_JPRB ,&
&  0.26365E+00_JPRB, 0.26372E+00_JPRB, 0.26467E+00_JPRB, 0.26604E+00_JPRB, 0.26771E+00_JPRB, 0.26869E+00_JPRB, 0.26913E+00_JPRB ,&
&  0.26965E+00_JPRB, 0.27029E+00_JPRB, 0.27247E+00_JPRB, 0.27584E+00_JPRB, 0.27896E+00_JPRB, 0.28248E+00_JPRB, 0.28538E+00_JPRB ,&
&  0.28757E+00_JPRB, 0.29003E+00_JPRB, 0.29199E+00_JPRB, 0.29338E+00_JPRB, 0.29510E+00_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.32000E+00_JPRB, 0.31619E+00_JPRB, 0.31352E+00_JPRB, 0.31332E+00_JPRB, 0.31531E+00_JPRB, 0.31574E+00_JPRB, 0.31421E+00_JPRB ,&
&  0.31276E+00_JPRB, 0.31413E+00_JPRB, 0.31727E+00_JPRB, 0.31897E+00_JPRB, 0.31955E+00_JPRB, 0.32024E+00_JPRB, 0.31908E+00_JPRB ,&
&  0.31565E+00_JPRB, 0.31219E+00_JPRB, 0.31239E+00_JPRB, 0.31351E+00_JPRB, 0.31437E+00_JPRB, 0.31290E+00_JPRB, 0.31116E+00_JPRB ,&
&  0.31157E+00_JPRB, 0.31160E+00_JPRB, 0.31329E+00_JPRB, 0.31516E+00_JPRB, 0.31894E+00_JPRB, 0.32510E+00_JPRB, 0.32969E+00_JPRB ,&
&  0.33125E+00_JPRB, 0.32950E+00_JPRB, 0.32899E+00_JPRB, 0.33181E+00_JPRB, 0.33504E+00_JPRB, 0.34249E+00_JPRB, 0.35040E+00_JPRB ,&
&  0.35379E+00_JPRB, 0.35163E+00_JPRB, 0.34160E+00_JPRB, 0.33252E+00_JPRB, 0.32623E+00_JPRB, 0.31973E+00_JPRB, 0.31333E+00_JPRB ,&
&  0.30675E+00_JPRB, 0.30188E+00_JPRB, 0.29895E+00_JPRB, 0.29755E+00_JPRB, 0.29699E+00_JPRB, 0.29622E+00_JPRB, 0.29394E+00_JPRB ,&
&  0.29105E+00_JPRB, 0.28882E+00_JPRB, 0.28713E+00_JPRB, 0.28582E+00_JPRB, 0.28460E+00_JPRB, 0.28401E+00_JPRB, 0.28332E+00_JPRB ,&
&  0.28299E+00_JPRB, 0.28336E+00_JPRB, 0.28414E+00_JPRB, 0.28413E+00_JPRB, 0.28392E+00_JPRB, 0.28320E+00_JPRB, 0.28247E+00_JPRB ,&
&  0.28135E+00_JPRB, 0.27913E+00_JPRB, 0.27728E+00_JPRB, 0.27692E+00_JPRB, 0.27678E+00_JPRB, 0.27722E+00_JPRB, 0.27759E+00_JPRB ,&
&  0.27731E+00_JPRB, 0.27730E+00_JPRB, 0.27804E+00_JPRB, 0.27904E+00_JPRB, 0.27981E+00_JPRB, 0.28027E+00_JPRB, 0.28091E+00_JPRB ,&
&  0.28255E+00_JPRB, 0.28447E+00_JPRB, 0.28583E+00_JPRB, 0.28688E+00_JPRB, 0.28760E+00_JPRB, 0.28883E+00_JPRB, 0.28898E+00_JPRB ,&
&  0.28925E+00_JPRB, 0.29052E+00_JPRB, 0.29357E+00_JPRB, 0.29810E+00_JPRB, 0.30307E+00_JPRB, 0.30715E+00_JPRB, 0.31121E+00_JPRB ,&
&  0.31474E+00_JPRB, 0.31765E+00_JPRB, 0.31942E+00_JPRB, 0.32004E+00_JPRB, 0.32061E+00_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.34527E+00_JPRB, 0.34123E+00_JPRB, 0.33823E+00_JPRB, 0.33587E+00_JPRB, 0.33587E+00_JPRB, 0.33479E+00_JPRB, 0.33206E+00_JPRB ,&
&  0.33010E+00_JPRB, 0.33138E+00_JPRB, 0.33391E+00_JPRB, 0.33490E+00_JPRB, 0.33406E+00_JPRB, 0.33256E+00_JPRB, 0.32969E+00_JPRB ,&
&  0.32582E+00_JPRB, 0.32393E+00_JPRB, 0.32366E+00_JPRB, 0.32414E+00_JPRB, 0.32381E+00_JPRB, 0.32206E+00_JPRB, 0.32076E+00_JPRB ,&
&  0.32164E+00_JPRB, 0.32307E+00_JPRB, 0.32348E+00_JPRB, 0.32562E+00_JPRB, 0.32976E+00_JPRB, 0.33726E+00_JPRB, 0.34411E+00_JPRB ,&
&  0.34692E+00_JPRB, 0.34753E+00_JPRB, 0.34199E+00_JPRB, 0.35415E+00_JPRB, 0.36181E+00_JPRB, 0.37195E+00_JPRB, 0.37848E+00_JPRB ,&
&  0.38080E+00_JPRB, 0.37761E+00_JPRB, 0.36708E+00_JPRB, 0.35851E+00_JPRB, 0.35289E+00_JPRB, 0.34702E+00_JPRB, 0.33802E+00_JPRB ,&
&  0.32877E+00_JPRB, 0.32078E+00_JPRB, 0.31591E+00_JPRB, 0.31352E+00_JPRB, 0.31339E+00_JPRB, 0.31374E+00_JPRB, 0.31195E+00_JPRB ,&
&  0.30995E+00_JPRB, 0.30802E+00_JPRB, 0.30722E+00_JPRB, 0.30638E+00_JPRB, 0.30514E+00_JPRB, 0.30414E+00_JPRB, 0.30381E+00_JPRB ,&
&  0.30386E+00_JPRB, 0.30379E+00_JPRB, 0.30406E+00_JPRB, 0.30480E+00_JPRB, 0.30518E+00_JPRB, 0.30466E+00_JPRB, 0.30346E+00_JPRB ,&
&  0.30209E+00_JPRB, 0.30093E+00_JPRB, 0.29994E+00_JPRB, 0.29890E+00_JPRB, 0.29794E+00_JPRB, 0.29712E+00_JPRB, 0.29605E+00_JPRB ,&
&  0.29475E+00_JPRB, 0.29475E+00_JPRB, 0.29574E+00_JPRB, 0.29613E+00_JPRB, 0.29624E+00_JPRB, 0.29712E+00_JPRB, 0.29889E+00_JPRB ,&
&  0.30064E+00_JPRB, 0.30264E+00_JPRB, 0.30413E+00_JPRB, 0.30484E+00_JPRB, 0.30433E+00_JPRB, 0.30437E+00_JPRB, 0.30517E+00_JPRB ,&
&  0.30707E+00_JPRB, 0.31009E+00_JPRB, 0.31465E+00_JPRB, 0.32135E+00_JPRB, 0.32966E+00_JPRB, 0.33629E+00_JPRB, 0.34207E+00_JPRB ,&
&  0.34649E+00_JPRB, 0.34839E+00_JPRB, 0.34861E+00_JPRB, 0.34908E+00_JPRB, 0.34842E+00_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.36910E+00_JPRB, 0.36595E+00_JPRB, 0.36359E+00_JPRB, 0.35979E+00_JPRB, 0.35711E+00_JPRB, 0.35519E+00_JPRB, 0.35303E+00_JPRB ,&
&  0.35146E+00_JPRB, 0.35185E+00_JPRB, 0.35084E+00_JPRB, 0.35044E+00_JPRB, 0.34969E+00_JPRB, 0.34707E+00_JPRB, 0.34366E+00_JPRB ,&
&  0.34172E+00_JPRB, 0.33993E+00_JPRB, 0.33833E+00_JPRB, 0.33655E+00_JPRB, 0.33477E+00_JPRB, 0.33239E+00_JPRB, 0.33248E+00_JPRB ,&
&  0.33360E+00_JPRB, 0.33629E+00_JPRB, 0.33953E+00_JPRB, 0.34331E+00_JPRB, 0.34904E+00_JPRB, 0.35685E+00_JPRB, 0.36421E+00_JPRB ,&
&  0.37228E+00_JPRB, 0.35980E+00_JPRB, 0.37415E+00_JPRB, 0.38259E+00_JPRB, 0.39694E+00_JPRB, 0.40881E+00_JPRB, 0.41035E+00_JPRB ,&
&  0.40557E+00_JPRB, 0.39275E+00_JPRB, 0.38026E+00_JPRB, 0.37220E+00_JPRB, 0.36993E+00_JPRB, 0.36794E+00_JPRB, 0.36222E+00_JPRB ,&
&  0.35484E+00_JPRB, 0.34628E+00_JPRB, 0.33963E+00_JPRB, 0.33587E+00_JPRB, 0.33439E+00_JPRB, 0.33396E+00_JPRB, 0.33247E+00_JPRB ,&
&  0.33092E+00_JPRB, 0.32936E+00_JPRB, 0.32856E+00_JPRB, 0.32812E+00_JPRB, 0.32761E+00_JPRB, 0.32773E+00_JPRB, 0.32832E+00_JPRB ,&
&  0.32890E+00_JPRB, 0.32918E+00_JPRB, 0.32930E+00_JPRB, 0.33004E+00_JPRB, 0.33083E+00_JPRB, 0.33133E+00_JPRB, 0.33160E+00_JPRB ,&
&  0.33107E+00_JPRB, 0.33058E+00_JPRB, 0.32970E+00_JPRB, 0.32908E+00_JPRB, 0.32747E+00_JPRB, 0.32470E+00_JPRB, 0.32169E+00_JPRB ,&
&  0.31961E+00_JPRB, 0.31847E+00_JPRB, 0.31773E+00_JPRB, 0.31737E+00_JPRB, 0.31764E+00_JPRB, 0.31859E+00_JPRB, 0.31978E+00_JPRB ,&
&  0.32100E+00_JPRB, 0.32207E+00_JPRB, 0.32254E+00_JPRB, 0.32271E+00_JPRB, 0.32223E+00_JPRB, 0.32263E+00_JPRB, 0.32417E+00_JPRB ,&
&  0.32781E+00_JPRB, 0.33260E+00_JPRB, 0.33988E+00_JPRB, 0.35234E+00_JPRB, 0.36506E+00_JPRB, 0.37298E+00_JPRB, 0.37844E+00_JPRB ,&
&  0.37902E+00_JPRB, 0.37670E+00_JPRB, 0.37404E+00_JPRB, 0.37296E+00_JPRB, 0.37220E+00_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.38677E+00_JPRB, 0.38519E+00_JPRB, 0.38368E+00_JPRB, 0.38134E+00_JPRB, 0.37902E+00_JPRB, 0.37705E+00_JPRB, 0.37354E+00_JPRB ,&
&  0.37228E+00_JPRB, 0.36879E+00_JPRB, 0.36561E+00_JPRB, 0.36454E+00_JPRB, 0.36313E+00_JPRB, 0.36091E+00_JPRB, 0.35908E+00_JPRB ,&
&  0.35684E+00_JPRB, 0.35426E+00_JPRB, 0.35249E+00_JPRB, 0.35016E+00_JPRB, 0.34673E+00_JPRB, 0.34427E+00_JPRB, 0.34474E+00_JPRB ,&
&  0.34562E+00_JPRB, 0.34819E+00_JPRB, 0.35612E+00_JPRB, 0.36421E+00_JPRB, 0.37302E+00_JPRB, 0.38145E+00_JPRB, 0.38948E+00_JPRB ,&
&  0.40147E+00_JPRB, 0.35967E+00_JPRB, 0.38265E+00_JPRB, 0.39891E+00_JPRB, 0.42380E+00_JPRB, 0.44560E+00_JPRB, 0.44238E+00_JPRB ,&
&  0.42779E+00_JPRB, 0.40830E+00_JPRB, 0.39372E+00_JPRB, 0.38467E+00_JPRB, 0.38402E+00_JPRB, 0.38444E+00_JPRB, 0.38191E+00_JPRB ,&
&  0.37795E+00_JPRB, 0.37074E+00_JPRB, 0.36295E+00_JPRB, 0.35785E+00_JPRB, 0.35504E+00_JPRB, 0.35293E+00_JPRB, 0.35075E+00_JPRB ,&
&  0.34949E+00_JPRB, 0.34923E+00_JPRB, 0.34905E+00_JPRB, 0.34883E+00_JPRB, 0.34931E+00_JPRB, 0.35071E+00_JPRB, 0.35252E+00_JPRB ,&
&  0.35418E+00_JPRB, 0.35493E+00_JPRB, 0.35648E+00_JPRB, 0.35955E+00_JPRB, 0.36392E+00_JPRB, 0.36811E+00_JPRB, 0.37085E+00_JPRB ,&
&  0.37176E+00_JPRB, 0.37183E+00_JPRB, 0.37104E+00_JPRB, 0.36878E+00_JPRB, 0.36489E+00_JPRB, 0.35971E+00_JPRB, 0.35493E+00_JPRB ,&
&  0.35051E+00_JPRB, 0.34647E+00_JPRB, 0.34125E+00_JPRB, 0.33899E+00_JPRB, 0.33876E+00_JPRB, 0.33887E+00_JPRB, 0.33921E+00_JPRB ,&
&  0.33958E+00_JPRB, 0.33932E+00_JPRB, 0.33885E+00_JPRB, 0.33841E+00_JPRB, 0.33849E+00_JPRB, 0.33958E+00_JPRB, 0.34196E+00_JPRB ,&
&  0.34539E+00_JPRB, 0.35204E+00_JPRB, 0.36144E+00_JPRB, 0.37801E+00_JPRB, 0.39240E+00_JPRB, 0.39906E+00_JPRB, 0.40134E+00_JPRB ,&
&  0.39806E+00_JPRB, 0.39341E+00_JPRB, 0.38948E+00_JPRB, 0.38646E+00_JPRB, 0.38721E+00_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.41376E+00_JPRB, 0.41095E+00_JPRB, 0.40941E+00_JPRB, 0.40650E+00_JPRB, 0.40270E+00_JPRB, 0.39863E+00_JPRB, 0.39412E+00_JPRB ,&
&  0.38939E+00_JPRB, 0.38658E+00_JPRB, 0.38401E+00_JPRB, 0.38084E+00_JPRB, 0.37765E+00_JPRB, 0.37514E+00_JPRB, 0.37262E+00_JPRB ,&
&  0.36979E+00_JPRB, 0.36655E+00_JPRB, 0.36427E+00_JPRB, 0.36269E+00_JPRB, 0.36092E+00_JPRB, 0.36025E+00_JPRB, 0.36119E+00_JPRB ,&
&  0.36116E+00_JPRB, 0.36350E+00_JPRB, 0.37333E+00_JPRB, 0.38503E+00_JPRB, 0.39899E+00_JPRB, 0.41119E+00_JPRB, 0.42631E+00_JPRB ,&
&  0.41383E+00_JPRB, 0.37330E+00_JPRB, 0.39884E+00_JPRB, 0.42840E+00_JPRB, 0.45953E+00_JPRB, 0.48753E+00_JPRB, 0.47837E+00_JPRB ,&
&  0.46332E+00_JPRB, 0.44688E+00_JPRB, 0.43315E+00_JPRB, 0.42300E+00_JPRB, 0.41979E+00_JPRB, 0.41654E+00_JPRB, 0.41274E+00_JPRB ,&
&  0.40735E+00_JPRB, 0.39907E+00_JPRB, 0.39071E+00_JPRB, 0.38336E+00_JPRB, 0.37825E+00_JPRB, 0.37423E+00_JPRB, 0.37084E+00_JPRB ,&
&  0.36930E+00_JPRB, 0.36900E+00_JPRB, 0.36889E+00_JPRB, 0.36897E+00_JPRB, 0.36957E+00_JPRB, 0.37207E+00_JPRB, 0.37551E+00_JPRB ,&
&  0.37972E+00_JPRB, 0.38651E+00_JPRB, 0.39745E+00_JPRB, 0.41026E+00_JPRB, 0.42249E+00_JPRB, 0.43055E+00_JPRB, 0.43373E+00_JPRB ,&
&  0.43418E+00_JPRB, 0.43155E+00_JPRB, 0.42614E+00_JPRB, 0.41792E+00_JPRB, 0.40789E+00_JPRB, 0.39852E+00_JPRB, 0.39019E+00_JPRB ,&
&  0.38284E+00_JPRB, 0.37529E+00_JPRB, 0.36849E+00_JPRB, 0.36487E+00_JPRB, 0.36345E+00_JPRB, 0.36196E+00_JPRB, 0.36056E+00_JPRB ,&
&  0.35979E+00_JPRB, 0.35925E+00_JPRB, 0.35898E+00_JPRB, 0.35950E+00_JPRB, 0.36146E+00_JPRB, 0.36488E+00_JPRB, 0.36835E+00_JPRB ,&
&  0.37080E+00_JPRB, 0.37770E+00_JPRB, 0.38698E+00_JPRB, 0.40859E+00_JPRB, 0.42277E+00_JPRB, 0.42799E+00_JPRB, 0.42720E+00_JPRB ,&
&  0.42323E+00_JPRB, 0.41850E+00_JPRB, 0.41574E+00_JPRB, 0.41307E+00_JPRB, 0.41360E+00_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.45007E+00_JPRB, 0.44860E+00_JPRB, 0.44258E+00_JPRB, 0.43181E+00_JPRB, 0.42301E+00_JPRB, 0.41618E+00_JPRB, 0.41164E+00_JPRB ,&
&  0.40913E+00_JPRB, 0.40694E+00_JPRB, 0.40406E+00_JPRB, 0.39903E+00_JPRB, 0.39478E+00_JPRB, 0.38962E+00_JPRB, 0.38436E+00_JPRB ,&
&  0.37990E+00_JPRB, 0.37574E+00_JPRB, 0.37333E+00_JPRB, 0.37014E+00_JPRB, 0.36868E+00_JPRB, 0.36984E+00_JPRB, 0.37161E+00_JPRB ,&
&  0.37425E+00_JPRB, 0.37686E+00_JPRB, 0.38540E+00_JPRB, 0.39625E+00_JPRB, 0.41549E+00_JPRB, 0.42859E+00_JPRB, 0.44814E+00_JPRB ,&
&  0.49105E+00_JPRB, 0.41834E+00_JPRB, 0.44935E+00_JPRB, 0.48539E+00_JPRB, 0.51385E+00_JPRB, 0.53926E+00_JPRB, 0.52423E+00_JPRB ,&
&  0.51085E+00_JPRB, 0.50108E+00_JPRB, 0.49188E+00_JPRB, 0.48421E+00_JPRB, 0.47843E+00_JPRB, 0.46570E+00_JPRB, 0.45008E+00_JPRB ,&
&  0.43321E+00_JPRB, 0.42105E+00_JPRB, 0.41156E+00_JPRB, 0.40246E+00_JPRB, 0.39434E+00_JPRB, 0.38910E+00_JPRB, 0.38663E+00_JPRB ,&
&  0.38622E+00_JPRB, 0.38630E+00_JPRB, 0.38602E+00_JPRB, 0.38641E+00_JPRB, 0.38812E+00_JPRB, 0.39207E+00_JPRB, 0.39921E+00_JPRB ,&
&  0.41523E+00_JPRB, 0.43937E+00_JPRB, 0.46425E+00_JPRB, 0.48686E+00_JPRB, 0.50256E+00_JPRB, 0.50668E+00_JPRB, 0.50378E+00_JPRB ,&
&  0.49366E+00_JPRB, 0.47922E+00_JPRB, 0.46444E+00_JPRB, 0.45073E+00_JPRB, 0.43876E+00_JPRB, 0.42839E+00_JPRB, 0.41856E+00_JPRB ,&
&  0.41053E+00_JPRB, 0.40379E+00_JPRB, 0.39920E+00_JPRB, 0.39592E+00_JPRB, 0.39266E+00_JPRB, 0.38980E+00_JPRB, 0.38853E+00_JPRB ,&
&  0.38757E+00_JPRB, 0.38690E+00_JPRB, 0.38691E+00_JPRB, 0.38917E+00_JPRB, 0.39326E+00_JPRB, 0.39670E+00_JPRB, 0.39867E+00_JPRB ,&
&  0.39927E+00_JPRB, 0.40589E+00_JPRB, 0.41213E+00_JPRB, 0.44206E+00_JPRB, 0.45677E+00_JPRB, 0.45966E+00_JPRB, 0.45865E+00_JPRB ,&
&  0.45791E+00_JPRB, 0.45090E+00_JPRB, 0.40266E+00_JPRB, 0.43151E+00_JPRB, 0.44419E+00_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.49450E+00_JPRB, 0.48627E+00_JPRB, 0.47282E+00_JPRB, 0.45917E+00_JPRB, 0.45070E+00_JPRB, 0.44523E+00_JPRB, 0.44116E+00_JPRB ,&
&  0.43640E+00_JPRB, 0.43080E+00_JPRB, 0.42530E+00_JPRB, 0.42189E+00_JPRB, 0.41778E+00_JPRB, 0.41211E+00_JPRB, 0.40515E+00_JPRB ,&
&  0.39872E+00_JPRB, 0.39274E+00_JPRB, 0.38811E+00_JPRB, 0.38289E+00_JPRB, 0.37874E+00_JPRB, 0.37855E+00_JPRB, 0.37996E+00_JPRB ,&
&  0.38552E+00_JPRB, 0.39317E+00_JPRB, 0.40019E+00_JPRB, 0.41454E+00_JPRB, 0.43116E+00_JPRB, 0.45488E+00_JPRB, 0.47860E+00_JPRB ,&
&  0.52226E+00_JPRB, 0.45665E+00_JPRB, 0.47900E+00_JPRB, 0.52666E+00_JPRB, 0.55926E+00_JPRB, 0.58426E+00_JPRB, 0.58363E+00_JPRB ,&
&  0.58032E+00_JPRB, 0.57986E+00_JPRB, 0.57754E+00_JPRB, 0.56999E+00_JPRB, 0.54906E+00_JPRB, 0.51139E+00_JPRB, 0.47497E+00_JPRB ,&
&  0.44986E+00_JPRB, 0.44121E+00_JPRB, 0.43714E+00_JPRB, 0.43141E+00_JPRB, 0.42580E+00_JPRB, 0.42326E+00_JPRB, 0.42359E+00_JPRB ,&
&  0.42365E+00_JPRB, 0.42320E+00_JPRB, 0.42204E+00_JPRB, 0.42080E+00_JPRB, 0.42191E+00_JPRB, 0.43189E+00_JPRB, 0.45734E+00_JPRB ,&
&  0.50156E+00_JPRB, 0.54937E+00_JPRB, 0.59052E+00_JPRB, 0.62046E+00_JPRB, 0.62404E+00_JPRB, 0.60827E+00_JPRB, 0.57919E+00_JPRB ,&
&  0.54608E+00_JPRB, 0.51714E+00_JPRB, 0.49581E+00_JPRB, 0.47918E+00_JPRB, 0.46597E+00_JPRB, 0.45556E+00_JPRB, 0.44856E+00_JPRB ,&
&  0.44494E+00_JPRB, 0.44272E+00_JPRB, 0.44067E+00_JPRB, 0.43749E+00_JPRB, 0.43487E+00_JPRB, 0.43277E+00_JPRB, 0.43174E+00_JPRB ,&
&  0.43027E+00_JPRB, 0.42865E+00_JPRB, 0.42868E+00_JPRB, 0.43073E+00_JPRB, 0.43214E+00_JPRB, 0.43178E+00_JPRB, 0.43118E+00_JPRB ,&
&  0.43283E+00_JPRB, 0.44256E+00_JPRB, 0.45320E+00_JPRB, 0.52400E+00_JPRB, 0.54171E+00_JPRB, 0.52820E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.50535E+00_JPRB, 0.49496E+00_JPRB, 0.49253E+00_JPRB, 0.46360E+00_JPRB, 0.49518E+00_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.52277E+00_JPRB, 0.51754E+00_JPRB, 0.50413E+00_JPRB, 0.48978E+00_JPRB, 0.47703E+00_JPRB, 0.46857E+00_JPRB, 0.46119E+00_JPRB ,&
&  0.45586E+00_JPRB, 0.45196E+00_JPRB, 0.44945E+00_JPRB, 0.44648E+00_JPRB, 0.44226E+00_JPRB, 0.43589E+00_JPRB, 0.42762E+00_JPRB ,&
&  0.42003E+00_JPRB, 0.41261E+00_JPRB, 0.40640E+00_JPRB, 0.39944E+00_JPRB, 0.39080E+00_JPRB, 0.38628E+00_JPRB, 0.38855E+00_JPRB ,&
&  0.39420E+00_JPRB, 0.40124E+00_JPRB, 0.40787E+00_JPRB, 0.41798E+00_JPRB, 0.44160E+00_JPRB, 0.46727E+00_JPRB, 0.49138E+00_JPRB ,&
&  0.53930E+00_JPRB, 0.51306E+00_JPRB, 0.53782E+00_JPRB, 0.57278E+00_JPRB, 0.61017E+00_JPRB, 0.63748E+00_JPRB, 0.65948E+00_JPRB ,&
&  0.67807E+00_JPRB, 0.69373E+00_JPRB, 0.69076E+00_JPRB, 0.66160E+00_JPRB, 0.60800E+00_JPRB, 0.55847E+00_JPRB, 0.52935E+00_JPRB ,&
&  0.51439E+00_JPRB, 0.50728E+00_JPRB, 0.50045E+00_JPRB, 0.49441E+00_JPRB, 0.49030E+00_JPRB, 0.48658E+00_JPRB, 0.48271E+00_JPRB ,&
&  0.47912E+00_JPRB, 0.47638E+00_JPRB, 0.47491E+00_JPRB, 0.47566E+00_JPRB, 0.48477E+00_JPRB, 0.51168E+00_JPRB, 0.56079E+00_JPRB ,&
&  0.63059E+00_JPRB, 0.69372E+00_JPRB, 0.72330E+00_JPRB, 0.73994E+00_JPRB, 0.75132E+00_JPRB, 0.72860E+00_JPRB, 0.66720E+00_JPRB ,&
&  0.61062E+00_JPRB, 0.56467E+00_JPRB, 0.53259E+00_JPRB, 0.51044E+00_JPRB, 0.49616E+00_JPRB, 0.48829E+00_JPRB, 0.48542E+00_JPRB ,&
&  0.48488E+00_JPRB, 0.48524E+00_JPRB, 0.48795E+00_JPRB, 0.48999E+00_JPRB, 0.49109E+00_JPRB, 0.48925E+00_JPRB, 0.48322E+00_JPRB ,&
&  0.47492E+00_JPRB, 0.46900E+00_JPRB, 0.46585E+00_JPRB, 0.46379E+00_JPRB, 0.46180E+00_JPRB, 0.45922E+00_JPRB, 0.45706E+00_JPRB ,&
&  0.45679E+00_JPRB, 0.47771E+00_JPRB, 0.51296E+00_JPRB, 0.59840E+00_JPRB, 0.59140E+00_JPRB, 0.57735E+00_JPRB, 0.55531E+00_JPRB ,&
&  0.54073E+00_JPRB, 0.53333E+00_JPRB, 0.51968E+00_JPRB, 0.51760E+00_JPRB, 0.52529E+00_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.58832E+00_JPRB, 0.56887E+00_JPRB, 0.54911E+00_JPRB, 0.52717E+00_JPRB, 0.50805E+00_JPRB, 0.49407E+00_JPRB, 0.48341E+00_JPRB ,&
&  0.47608E+00_JPRB, 0.47137E+00_JPRB, 0.46826E+00_JPRB, 0.46601E+00_JPRB, 0.46338E+00_JPRB, 0.46029E+00_JPRB, 0.45307E+00_JPRB ,&
&  0.44405E+00_JPRB, 0.43361E+00_JPRB, 0.42548E+00_JPRB, 0.41841E+00_JPRB, 0.40570E+00_JPRB, 0.39806E+00_JPRB, 0.39932E+00_JPRB ,&
&  0.40389E+00_JPRB, 0.40964E+00_JPRB, 0.41441E+00_JPRB, 0.42505E+00_JPRB, 0.44586E+00_JPRB, 0.47161E+00_JPRB, 0.50295E+00_JPRB ,&
&  0.55736E+00_JPRB, 0.57432E+00_JPRB, 0.63239E+00_JPRB, 0.65613E+00_JPRB, 0.67990E+00_JPRB, 0.72730E+00_JPRB, 0.77348E+00_JPRB ,&
&  0.80005E+00_JPRB, 0.85600E+00_JPRB, 0.84718E+00_JPRB, 0.77796E+00_JPRB, 0.70639E+00_JPRB, 0.65602E+00_JPRB, 0.62445E+00_JPRB ,&
&  0.60037E+00_JPRB, 0.57884E+00_JPRB, 0.55999E+00_JPRB, 0.54725E+00_JPRB, 0.53542E+00_JPRB, 0.52393E+00_JPRB, 0.51302E+00_JPRB ,&
&  0.50674E+00_JPRB, 0.50301E+00_JPRB, 0.50304E+00_JPRB, 0.51453E+00_JPRB, 0.55521E+00_JPRB, 0.62568E+00_JPRB, 0.71642E+00_JPRB ,&
&  0.79007E+00_JPRB, 0.83237E+00_JPRB, 0.86849E+00_JPRB, 0.86308E+00_JPRB, 0.82841E+00_JPRB, 0.79080E+00_JPRB, 0.75087E+00_JPRB ,&
&  0.70044E+00_JPRB, 0.64433E+00_JPRB, 0.60303E+00_JPRB, 0.57457E+00_JPRB, 0.55625E+00_JPRB, 0.54608E+00_JPRB, 0.54097E+00_JPRB ,&
&  0.53712E+00_JPRB, 0.53755E+00_JPRB, 0.54637E+00_JPRB, 0.55605E+00_JPRB, 0.55622E+00_JPRB, 0.54563E+00_JPRB, 0.52931E+00_JPRB ,&
&  0.51466E+00_JPRB, 0.50276E+00_JPRB, 0.50049E+00_JPRB, 0.50023E+00_JPRB, 0.50092E+00_JPRB, 0.50288E+00_JPRB, 0.50182E+00_JPRB ,&
&  0.50259E+00_JPRB, 0.54543E+00_JPRB, 0.58993E+00_JPRB, 0.70624E+00_JPRB, 0.75827E+00_JPRB, 0.70984E+00_JPRB, 0.65521E+00_JPRB ,&
&  0.61269E+00_JPRB, 0.58489E+00_JPRB, 0.55707E+00_JPRB, 0.55010E+00_JPRB, 0.60854E+00_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.57142E+00_JPRB, 0.55936E+00_JPRB, 0.54699E+00_JPRB, 0.53601E+00_JPRB, 0.52157E+00_JPRB, 0.50801E+00_JPRB, 0.49505E+00_JPRB ,&
&  0.48508E+00_JPRB, 0.47933E+00_JPRB, 0.47910E+00_JPRB, 0.48002E+00_JPRB, 0.47794E+00_JPRB, 0.47043E+00_JPRB, 0.45698E+00_JPRB ,&
&  0.44770E+00_JPRB, 0.43997E+00_JPRB, 0.43088E+00_JPRB, 0.41985E+00_JPRB, 0.40949E+00_JPRB, 0.40524E+00_JPRB, 0.40547E+00_JPRB ,&
&  0.40993E+00_JPRB, 0.41575E+00_JPRB, 0.41997E+00_JPRB, 0.43168E+00_JPRB, 0.44887E+00_JPRB, 0.47477E+00_JPRB, 0.51161E+00_JPRB ,&
&  0.56501E+00_JPRB, 0.71196E+00_JPRB, 0.76792E+00_JPRB, 0.77578E+00_JPRB, 0.84157E+00_JPRB, 0.86166E+00_JPRB, 0.93032E+00_JPRB ,&
&  0.99868E+00_JPRB, 0.10469E+01_JPRB, 0.10196E+01_JPRB, 0.91797E+00_JPRB, 0.83148E+00_JPRB, 0.76899E+00_JPRB, 0.70859E+00_JPRB ,&
&  0.65194E+00_JPRB, 0.60791E+00_JPRB, 0.57292E+00_JPRB, 0.54904E+00_JPRB, 0.53120E+00_JPRB, 0.51821E+00_JPRB, 0.51145E+00_JPRB ,&
&  0.51103E+00_JPRB, 0.51539E+00_JPRB, 0.53249E+00_JPRB, 0.57215E+00_JPRB, 0.72248E+00_JPRB, 0.86074E+00_JPRB, 0.97281E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10351E+01_JPRB, 0.10452E+01_JPRB, 0.99956E+00_JPRB, 0.92780E+00_JPRB, 0.84331E+00_JPRB, 0.77943E+00_JPRB ,&
&  0.73898E+00_JPRB, 0.69854E+00_JPRB, 0.65972E+00_JPRB, 0.62412E+00_JPRB, 0.60002E+00_JPRB, 0.58822E+00_JPRB, 0.58275E+00_JPRB ,&
&  0.57690E+00_JPRB, 0.57492E+00_JPRB, 0.57442E+00_JPRB, 0.57262E+00_JPRB, 0.56910E+00_JPRB, 0.56332E+00_JPRB, 0.55435E+00_JPRB ,&
&  0.54337E+00_JPRB, 0.53422E+00_JPRB, 0.54804E+00_JPRB, 0.55265E+00_JPRB, 0.55354E+00_JPRB, 0.54917E+00_JPRB, 0.54078E+00_JPRB ,&
&  0.54459E+00_JPRB, 0.60694E+00_JPRB, 0.62062E+00_JPRB, 0.68028E+00_JPRB, 0.74343E+00_JPRB, 0.73730E+00_JPRB, 0.67977E+00_JPRB ,&
&  0.63563E+00_JPRB, 0.59638E+00_JPRB, 0.56510E+00_JPRB, 0.55587E+00_JPRB, 0.57783E+00_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.56340E+00_JPRB, 0.55586E+00_JPRB, 0.54543E+00_JPRB, 0.53506E+00_JPRB, 0.52266E+00_JPRB, 0.50959E+00_JPRB, 0.50001E+00_JPRB ,&
&  0.49377E+00_JPRB, 0.49180E+00_JPRB, 0.49191E+00_JPRB, 0.48197E+00_JPRB, 0.46830E+00_JPRB, 0.45578E+00_JPRB, 0.44813E+00_JPRB ,&
&  0.44178E+00_JPRB, 0.43438E+00_JPRB, 0.42531E+00_JPRB, 0.41474E+00_JPRB, 0.40994E+00_JPRB, 0.40776E+00_JPRB, 0.40646E+00_JPRB ,&
&  0.41168E+00_JPRB, 0.41656E+00_JPRB, 0.42311E+00_JPRB, 0.43207E+00_JPRB, 0.44789E+00_JPRB, 0.47373E+00_JPRB, 0.51409E+00_JPRB ,&
&  0.58523E+00_JPRB, 0.72878E+00_JPRB, 0.92481E+00_JPRB, 0.88578E+00_JPRB, 0.92837E+00_JPRB, 0.96174E+00_JPRB, 0.11167E+01_JPRB ,&
&  0.12279E+01_JPRB, 0.12599E+01_JPRB, 0.11624E+01_JPRB, 0.10279E+01_JPRB, 0.90535E+00_JPRB, 0.80789E+00_JPRB, 0.73762E+00_JPRB ,&
&  0.67556E+00_JPRB, 0.62241E+00_JPRB, 0.58461E+00_JPRB, 0.55821E+00_JPRB, 0.54100E+00_JPRB, 0.53065E+00_JPRB, 0.52690E+00_JPRB ,&
&  0.53307E+00_JPRB, 0.55628E+00_JPRB, 0.60931E+00_JPRB, 0.71696E+00_JPRB, 0.92552E+00_JPRB, 0.92352E+00_JPRB, 0.10523E+01_JPRB ,&
&  0.13099E+01_JPRB, 0.12971E+01_JPRB, 0.12164E+01_JPRB, 0.11026E+01_JPRB, 0.99529E+00_JPRB, 0.89845E+00_JPRB, 0.81371E+00_JPRB ,&
&  0.75566E+00_JPRB, 0.71240E+00_JPRB, 0.67394E+00_JPRB, 0.64639E+00_JPRB, 0.62998E+00_JPRB, 0.62476E+00_JPRB, 0.62123E+00_JPRB ,&
&  0.61504E+00_JPRB, 0.60746E+00_JPRB, 0.60259E+00_JPRB, 0.60153E+00_JPRB, 0.60089E+00_JPRB, 0.60115E+00_JPRB, 0.59112E+00_JPRB ,&
&  0.58192E+00_JPRB, 0.58008E+00_JPRB, 0.61881E+00_JPRB, 0.59849E+00_JPRB, 0.59042E+00_JPRB, 0.58144E+00_JPRB, 0.56728E+00_JPRB ,&
&  0.55894E+00_JPRB, 0.58400E+00_JPRB, 0.60668E+00_JPRB, 0.65660E+00_JPRB, 0.76150E+00_JPRB, 0.78584E+00_JPRB, 0.68334E+00_JPRB ,&
&  0.64623E+00_JPRB, 0.61115E+00_JPRB, 0.58094E+00_JPRB, 0.56826E+00_JPRB, 0.57370E+00_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.55538E+00_JPRB, 0.54751E+00_JPRB, 0.53931E+00_JPRB, 0.53027E+00_JPRB, 0.51897E+00_JPRB, 0.50820E+00_JPRB, 0.50158E+00_JPRB ,&
&  0.49906E+00_JPRB, 0.49990E+00_JPRB, 0.49525E+00_JPRB, 0.48216E+00_JPRB, 0.46665E+00_JPRB, 0.45235E+00_JPRB, 0.44437E+00_JPRB ,&
&  0.43696E+00_JPRB, 0.42693E+00_JPRB, 0.41447E+00_JPRB, 0.40273E+00_JPRB, 0.39932E+00_JPRB, 0.40088E+00_JPRB, 0.39600E+00_JPRB ,&
&  0.40173E+00_JPRB, 0.40936E+00_JPRB, 0.41677E+00_JPRB, 0.42397E+00_JPRB, 0.44089E+00_JPRB, 0.46849E+00_JPRB, 0.50866E+00_JPRB ,&
&  0.58501E+00_JPRB, 0.50783E+00_JPRB, 0.84310E+00_JPRB, 0.10085E+01_JPRB, 0.11087E+01_JPRB, 0.11911E+01_JPRB, 0.13597E+01_JPRB ,&
&  0.15007E+01_JPRB, 0.14568E+01_JPRB, 0.13248E+01_JPRB, 0.11286E+01_JPRB, 0.95050E+00_JPRB, 0.82200E+00_JPRB, 0.73853E+00_JPRB ,&
&  0.68681E+00_JPRB, 0.64840E+00_JPRB, 0.61444E+00_JPRB, 0.58569E+00_JPRB, 0.56613E+00_JPRB, 0.55512E+00_JPRB, 0.55798E+00_JPRB ,&
&  0.59167E+00_JPRB, 0.65954E+00_JPRB, 0.75469E+00_JPRB, 0.92870E+00_JPRB, 0.10046E+01_JPRB, 0.10314E+01_JPRB, 0.11073E+01_JPRB ,&
&  0.13461E+01_JPRB, 0.15209E+01_JPRB, 0.13050E+01_JPRB, 0.11094E+01_JPRB, 0.10079E+01_JPRB, 0.90803E+00_JPRB, 0.81886E+00_JPRB ,&
&  0.74926E+00_JPRB, 0.70699E+00_JPRB, 0.67987E+00_JPRB, 0.66430E+00_JPRB, 0.65677E+00_JPRB, 0.64582E+00_JPRB, 0.62840E+00_JPRB ,&
&  0.61385E+00_JPRB, 0.60544E+00_JPRB, 0.60351E+00_JPRB, 0.60581E+00_JPRB, 0.60727E+00_JPRB, 0.61102E+00_JPRB, 0.61191E+00_JPRB ,&
&  0.61110E+00_JPRB, 0.61553E+00_JPRB, 0.61854E+00_JPRB, 0.59872E+00_JPRB, 0.58844E+00_JPRB, 0.57487E+00_JPRB, 0.56561E+00_JPRB ,&
&  0.58175E+00_JPRB, 0.59349E+00_JPRB, 0.61666E+00_JPRB, 0.67694E+00_JPRB, 0.71884E+00_JPRB, 0.75460E+00_JPRB, 0.66549E+00_JPRB ,&
&  0.64627E+00_JPRB, 0.62186E+00_JPRB, 0.60066E+00_JPRB, 0.57938E+00_JPRB, 0.56990E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.55271E+00_JPRB, 0.54385E+00_JPRB, 0.53602E+00_JPRB, 0.52561E+00_JPRB, 0.51195E+00_JPRB, 0.50361E+00_JPRB, 0.50476E+00_JPRB ,&
&  0.50880E+00_JPRB, 0.50284E+00_JPRB, 0.49444E+00_JPRB, 0.48195E+00_JPRB, 0.46349E+00_JPRB, 0.45389E+00_JPRB, 0.44701E+00_JPRB ,&
&  0.43638E+00_JPRB, 0.42825E+00_JPRB, 0.41616E+00_JPRB, 0.41033E+00_JPRB, 0.40869E+00_JPRB, 0.40372E+00_JPRB, 0.39441E+00_JPRB ,&
&  0.39696E+00_JPRB, 0.40475E+00_JPRB, 0.41435E+00_JPRB, 0.42432E+00_JPRB, 0.44391E+00_JPRB, 0.47435E+00_JPRB, 0.51739E+00_JPRB ,&
&  0.60430E+00_JPRB, 0.90834E+00_JPRB, 0.77153E+00_JPRB, 0.13219E+01_JPRB, 0.13516E+01_JPRB, 0.14467E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.15204E+01_JPRB, 0.15670E+01_JPRB, 0.14716E+01_JPRB, 0.12249E+01_JPRB, 0.98690E+00_JPRB, 0.83673E+00_JPRB, 0.74282E+00_JPRB ,&
&  0.68151E+00_JPRB, 0.63594E+00_JPRB, 0.60363E+00_JPRB, 0.58190E+00_JPRB, 0.56569E+00_JPRB, 0.56759E+00_JPRB, 0.61881E+00_JPRB ,&
&  0.71711E+00_JPRB, 0.80944E+00_JPRB, 0.96236E+00_JPRB, 0.11185E+01_JPRB, 0.10405E+01_JPRB, 0.10751E+01_JPRB, 0.13012E+01_JPRB ,&
&  0.14743E+01_JPRB, 0.15432E+01_JPRB, 0.11814E+01_JPRB, 0.10132E+01_JPRB, 0.93851E+00_JPRB, 0.88971E+00_JPRB, 0.81767E+00_JPRB ,&
&  0.75227E+00_JPRB, 0.72019E+00_JPRB, 0.70163E+00_JPRB, 0.68860E+00_JPRB, 0.67220E+00_JPRB, 0.64879E+00_JPRB, 0.62484E+00_JPRB ,&
&  0.60479E+00_JPRB, 0.60360E+00_JPRB, 0.60640E+00_JPRB, 0.60971E+00_JPRB, 0.61443E+00_JPRB, 0.61725E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.63110E+00_JPRB, 0.64048E+00_JPRB, 0.61512E+00_JPRB, 0.58891E+00_JPRB, 0.57559E+00_JPRB, 0.56672E+00_JPRB, 0.56309E+00_JPRB ,&
&  0.58503E+00_JPRB, 0.61289E+00_JPRB, 0.64489E+00_JPRB, 0.69385E+00_JPRB, 0.75226E+00_JPRB, 0.80791E+00_JPRB, 0.67364E+00_JPRB ,&
&  0.66129E+00_JPRB, 0.63415E+00_JPRB, 0.60955E+00_JPRB, 0.58169E+00_JPRB, 0.56713E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.54824E+00_JPRB, 0.53689E+00_JPRB, 0.52973E+00_JPRB, 0.52165E+00_JPRB, 0.51290E+00_JPRB, 0.50979E+00_JPRB, 0.51274E+00_JPRB ,&
&  0.51098E+00_JPRB, 0.50169E+00_JPRB, 0.48812E+00_JPRB, 0.47471E+00_JPRB, 0.45560E+00_JPRB, 0.44506E+00_JPRB, 0.43886E+00_JPRB ,&
&  0.43447E+00_JPRB, 0.43422E+00_JPRB, 0.42975E+00_JPRB, 0.42679E+00_JPRB, 0.42396E+00_JPRB, 0.41669E+00_JPRB, 0.41367E+00_JPRB ,&
&  0.41132E+00_JPRB, 0.41497E+00_JPRB, 0.42396E+00_JPRB, 0.44146E+00_JPRB, 0.46722E+00_JPRB, 0.50644E+00_JPRB, 0.55364E+00_JPRB ,&
&  0.67549E+00_JPRB, 0.83945E+00_JPRB, 0.83728E+00_JPRB, 0.16771E+01_JPRB, 0.16679E+01_JPRB, 0.16313E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.15391E+01_JPRB, 0.17039E+01_JPRB, 0.15381E+01_JPRB, 0.12997E+01_JPRB, 0.10501E+01_JPRB, 0.87105E+00_JPRB, 0.73861E+00_JPRB ,&
&  0.65753E+00_JPRB, 0.60901E+00_JPRB, 0.58629E+00_JPRB, 0.57714E+00_JPRB, 0.58005E+00_JPRB, 0.64218E+00_JPRB, 0.75835E+00_JPRB ,&
&  0.85532E+00_JPRB, 0.96751E+00_JPRB, 0.11817E+01_JPRB, 0.11317E+01_JPRB, 0.10375E+01_JPRB, 0.11003E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.14150E+01_JPRB, 0.12383E+01_JPRB, 0.10164E+01_JPRB, 0.91608E+00_JPRB, 0.85298E+00_JPRB, 0.84576E+00_JPRB, 0.80250E+00_JPRB ,&
&  0.75552E+00_JPRB, 0.72921E+00_JPRB, 0.70730E+00_JPRB, 0.68775E+00_JPRB, 0.66409E+00_JPRB, 0.64199E+00_JPRB, 0.61623E+00_JPRB ,&
&  0.60946E+00_JPRB, 0.60906E+00_JPRB, 0.60877E+00_JPRB, 0.61519E+00_JPRB, 0.61338E+00_JPRB, 0.61833E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.64204E+00_JPRB, 0.64810E+00_JPRB, 0.61033E+00_JPRB, 0.58497E+00_JPRB, 0.57914E+00_JPRB, 0.57504E+00_JPRB, 0.57797E+00_JPRB ,&
&  0.60786E+00_JPRB, 0.64624E+00_JPRB, 0.67446E+00_JPRB, 0.68922E+00_JPRB, 0.74257E+00_JPRB, 0.75399E+00_JPRB, 0.68184E+00_JPRB ,&
&  0.65077E+00_JPRB, 0.62759E+00_JPRB, 0.60671E+00_JPRB, 0.57746E+00_JPRB, 0.56337E+00_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.53672E+00_JPRB, 0.52463E+00_JPRB, 0.51526E+00_JPRB, 0.51253E+00_JPRB, 0.50777E+00_JPRB, 0.50439E+00_JPRB, 0.51024E+00_JPRB ,&
&  0.50241E+00_JPRB, 0.48809E+00_JPRB, 0.46770E+00_JPRB, 0.44677E+00_JPRB, 0.43075E+00_JPRB, 0.43265E+00_JPRB, 0.43814E+00_JPRB ,&
&  0.43901E+00_JPRB, 0.43661E+00_JPRB, 0.43553E+00_JPRB, 0.43797E+00_JPRB, 0.43677E+00_JPRB, 0.43522E+00_JPRB, 0.43321E+00_JPRB ,&
&  0.43328E+00_JPRB, 0.43919E+00_JPRB, 0.45833E+00_JPRB, 0.49111E+00_JPRB, 0.53043E+00_JPRB, 0.59173E+00_JPRB, 0.67768E+00_JPRB ,&
&  0.87455E+00_JPRB, 0.96102E+00_JPRB, 0.10251E+01_JPRB, 0.15844E+01_JPRB, 0.16920E+01_JPRB, 0.15843E+01_JPRB, 0.15479E+01_JPRB ,&
&  0.14993E+01_JPRB, 0.14272E+01_JPRB, 0.15260E+01_JPRB, 0.12531E+01_JPRB, 0.10233E+01_JPRB, 0.83605E+00_JPRB, 0.71282E+00_JPRB ,&
&  0.64411E+00_JPRB, 0.60022E+00_JPRB, 0.59003E+00_JPRB, 0.60685E+00_JPRB, 0.67543E+00_JPRB, 0.80235E+00_JPRB, 0.92137E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.11698E+01_JPRB, 0.14034E+01_JPRB, 0.11947E+01_JPRB, 0.98139E+00_JPRB, 0.10213E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.11593E+01_JPRB, 0.10098E+01_JPRB, 0.92094E+00_JPRB, 0.85841E+00_JPRB, 0.78693E+00_JPRB, 0.82942E+00_JPRB, 0.78050E+00_JPRB ,&
&  0.74747E+00_JPRB, 0.72108E+00_JPRB, 0.69444E+00_JPRB, 0.67049E+00_JPRB, 0.64044E+00_JPRB, 0.60488E+00_JPRB, 0.58659E+00_JPRB ,&
&  0.58846E+00_JPRB, 0.59597E+00_JPRB, 0.59624E+00_JPRB, 0.60102E+00_JPRB, 0.60056E+00_JPRB, 0.60982E+00_JPRB, 0.62198E+00_JPRB ,&
&  0.64376E+00_JPRB, 0.65577E+00_JPRB, 0.63288E+00_JPRB, 0.62343E+00_JPRB, 0.62090E+00_JPRB, 0.62207E+00_JPRB, 0.62192E+00_JPRB ,&
&  0.64133E+00_JPRB, 0.68357E+00_JPRB, 0.70676E+00_JPRB, 0.72622E+00_JPRB, 0.73863E+00_JPRB, 0.71806E+00_JPRB, 0.66973E+00_JPRB ,&
&  0.65378E+00_JPRB, 0.66025E+00_JPRB, 0.65674E+00_JPRB, 0.59011E+00_JPRB, 0.55916E+00_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.50787E+00_JPRB, 0.49859E+00_JPRB, 0.49644E+00_JPRB, 0.49693E+00_JPRB, 0.49460E+00_JPRB, 0.49603E+00_JPRB, 0.49815E+00_JPRB ,&
&  0.47540E+00_JPRB, 0.45276E+00_JPRB, 0.43619E+00_JPRB, 0.42885E+00_JPRB, 0.42706E+00_JPRB, 0.43976E+00_JPRB, 0.44523E+00_JPRB ,&
&  0.44269E+00_JPRB, 0.44197E+00_JPRB, 0.45626E+00_JPRB, 0.46001E+00_JPRB, 0.46135E+00_JPRB, 0.45922E+00_JPRB, 0.46029E+00_JPRB ,&
&  0.47206E+00_JPRB, 0.49634E+00_JPRB, 0.54375E+00_JPRB, 0.61088E+00_JPRB, 0.68357E+00_JPRB, 0.80428E+00_JPRB, 0.94816E+00_JPRB ,&
&  0.11959E+01_JPRB, 0.11958E+01_JPRB, 0.63058E+00_JPRB, 0.10415E+01_JPRB, 0.13728E+01_JPRB, 0.14079E+01_JPRB, 0.13137E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.11519E+01_JPRB, 0.11967E+01_JPRB, 0.11140E+01_JPRB, 0.91072E+00_JPRB, 0.78385E+00_JPRB, 0.69357E+00_JPRB ,&
&  0.64213E+00_JPRB, 0.62689E+00_JPRB, 0.64644E+00_JPRB, 0.71654E+00_JPRB, 0.85802E+00_JPRB, 0.10087E+01_JPRB, 0.10996E+01_JPRB ,&
&  0.12037E+01_JPRB, 0.14433E+01_JPRB, 0.15940E+01_JPRB, 0.11630E+01_JPRB, 0.94539E+00_JPRB, 0.92582E+00_JPRB, 0.89813E+00_JPRB ,&
&  0.87297E+00_JPRB, 0.89428E+00_JPRB, 0.87824E+00_JPRB, 0.82943E+00_JPRB, 0.77226E+00_JPRB, 0.74750E+00_JPRB, 0.75894E+00_JPRB ,&
&  0.72660E+00_JPRB, 0.69823E+00_JPRB, 0.67185E+00_JPRB, 0.63671E+00_JPRB, 0.58977E+00_JPRB, 0.55089E+00_JPRB, 0.55378E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.57222E+00_JPRB, 0.57497E+00_JPRB, 0.58841E+00_JPRB, 0.59988E+00_JPRB, 0.61435E+00_JPRB, 0.62954E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.67388E+00_JPRB, 0.69558E+00_JPRB, 0.68992E+00_JPRB, 0.68506E+00_JPRB, 0.69019E+00_JPRB, 0.70160E+00_JPRB ,&
&  0.72510E+00_JPRB, 0.74758E+00_JPRB, 0.71440E+00_JPRB, 0.72126E+00_JPRB, 0.74129E+00_JPRB, 0.71728E+00_JPRB, 0.68724E+00_JPRB ,&
&  0.72674E+00_JPRB, 0.76476E+00_JPRB, 0.70536E+00_JPRB, 0.55652E+00_JPRB, 0.52594E+00_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.49360E+00_JPRB, 0.48739E+00_JPRB, 0.48695E+00_JPRB, 0.48894E+00_JPRB, 0.48400E+00_JPRB, 0.48604E+00_JPRB, 0.47174E+00_JPRB ,&
&  0.44710E+00_JPRB, 0.43872E+00_JPRB, 0.45298E+00_JPRB, 0.45845E+00_JPRB, 0.46119E+00_JPRB, 0.47870E+00_JPRB, 0.49767E+00_JPRB ,&
&  0.50514E+00_JPRB, 0.50652E+00_JPRB, 0.50485E+00_JPRB, 0.50563E+00_JPRB, 0.50149E+00_JPRB, 0.50534E+00_JPRB, 0.52615E+00_JPRB ,&
&  0.56027E+00_JPRB, 0.62183E+00_JPRB, 0.71381E+00_JPRB, 0.82969E+00_JPRB, 0.95991E+00_JPRB, 0.11360E+01_JPRB, 0.12943E+01_JPRB ,&
&  0.16029E+01_JPRB, 0.11361E+01_JPRB, 0.71180E+00_JPRB, 0.11962E+01_JPRB, 0.13345E+01_JPRB, 0.13225E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.10640E+01_JPRB, 0.10200E+01_JPRB, 0.97872E+00_JPRB, 0.90958E+00_JPRB, 0.79658E+00_JPRB, 0.74745E+00_JPRB, 0.70378E+00_JPRB ,&
&  0.68346E+00_JPRB, 0.70295E+00_JPRB, 0.77028E+00_JPRB, 0.91283E+00_JPRB, 0.11016E+01_JPRB, 0.12223E+01_JPRB, 0.13051E+01_JPRB ,&
&  0.15082E+01_JPRB, 0.17549E+01_JPRB, 0.18090E+01_JPRB, 0.12720E+01_JPRB, 0.97411E+00_JPRB, 0.88807E+00_JPRB, 0.86126E+00_JPRB ,&
&  0.83034E+00_JPRB, 0.85343E+00_JPRB, 0.86187E+00_JPRB, 0.81448E+00_JPRB, 0.76299E+00_JPRB, 0.72097E+00_JPRB, 0.73262E+00_JPRB ,&
&  0.69833E+00_JPRB, 0.66426E+00_JPRB, 0.62817E+00_JPRB, 0.58255E+00_JPRB, 0.53883E+00_JPRB, 0.53489E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.53638E+00_JPRB, 0.55387E+00_JPRB, 0.55761E+00_JPRB, 0.57622E+00_JPRB, 0.59765E+00_JPRB, 0.62508E+00_JPRB, 0.65397E+00_JPRB ,&
&  0.67620E+00_JPRB, 0.70578E+00_JPRB, 0.73366E+00_JPRB, 0.74020E+00_JPRB, 0.73140E+00_JPRB, 0.71584E+00_JPRB, 0.73673E+00_JPRB ,&
&  0.74335E+00_JPRB, 0.75162E+00_JPRB, 0.72530E+00_JPRB, 0.73311E+00_JPRB, 0.69118E+00_JPRB, 0.65572E+00_JPRB, 0.60634E+00_JPRB ,&
&  0.59916E+00_JPRB, 0.56408E+00_JPRB, 0.53146E+00_JPRB, 0.48759E+00_JPRB, 0.48868E+00_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.48837E+00_JPRB, 0.48125E+00_JPRB, 0.47473E+00_JPRB, 0.46443E+00_JPRB, 0.45300E+00_JPRB, 0.45272E+00_JPRB, 0.44136E+00_JPRB ,&
&  0.43908E+00_JPRB, 0.46040E+00_JPRB, 0.49286E+00_JPRB, 0.51992E+00_JPRB, 0.54332E+00_JPRB, 0.55572E+00_JPRB, 0.57535E+00_JPRB ,&
&  0.57888E+00_JPRB, 0.57871E+00_JPRB, 0.57813E+00_JPRB, 0.57901E+00_JPRB, 0.58887E+00_JPRB, 0.61991E+00_JPRB, 0.67407E+00_JPRB ,&
&  0.75066E+00_JPRB, 0.85307E+00_JPRB, 0.97868E+00_JPRB, 0.11140E+01_JPRB, 0.12451E+01_JPRB, 0.13716E+01_JPRB, 0.14322E+01_JPRB ,&
&  0.98771E+00_JPRB, 0.55794E+00_JPRB, 0.11882E+01_JPRB, 0.13201E+01_JPRB, 0.12191E+01_JPRB, 0.11588E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.94305E+00_JPRB, 0.90858E+00_JPRB, 0.85904E+00_JPRB, 0.82532E+00_JPRB, 0.75404E+00_JPRB, 0.74123E+00_JPRB, 0.73965E+00_JPRB ,&
&  0.75983E+00_JPRB, 0.81707E+00_JPRB, 0.94309E+00_JPRB, 0.11547E+01_JPRB, 0.13106E+01_JPRB, 0.13881E+01_JPRB, 0.15433E+01_JPRB ,&
&  0.18290E+01_JPRB, 0.20733E+01_JPRB, 0.21692E+01_JPRB, 0.15253E+01_JPRB, 0.11322E+01_JPRB, 0.10391E+01_JPRB, 0.97409E+00_JPRB ,&
&  0.93608E+00_JPRB, 0.81867E+00_JPRB, 0.80965E+00_JPRB, 0.80652E+00_JPRB, 0.75936E+00_JPRB, 0.69700E+00_JPRB, 0.69480E+00_JPRB ,&
&  0.65140E+00_JPRB, 0.59966E+00_JPRB, 0.55064E+00_JPRB, 0.52684E+00_JPRB, 0.51610E+00_JPRB, 0.52662E+00_JPRB, 0.52615E+00_JPRB ,&
&  0.52920E+00_JPRB, 0.54621E+00_JPRB, 0.55500E+00_JPRB, 0.57040E+00_JPRB, 0.61032E+00_JPRB, 0.65176E+00_JPRB, 0.68193E+00_JPRB ,&
&  0.70066E+00_JPRB, 0.72859E+00_JPRB, 0.74849E+00_JPRB, 0.75778E+00_JPRB, 0.75891E+00_JPRB, 0.73594E+00_JPRB, 0.73174E+00_JPRB ,&
&  0.72988E+00_JPRB, 0.71950E+00_JPRB, 0.69342E+00_JPRB, 0.68079E+00_JPRB, 0.63638E+00_JPRB, 0.60633E+00_JPRB, 0.54477E+00_JPRB ,&
&  0.52050E+00_JPRB, 0.48305E+00_JPRB, 0.45708E+00_JPRB, 0.45111E+00_JPRB, 0.46165E+00_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.47949E+00_JPRB, 0.46124E+00_JPRB, 0.45120E+00_JPRB, 0.44289E+00_JPRB, 0.43061E+00_JPRB, 0.45370E+00_JPRB, 0.45940E+00_JPRB ,&
&  0.48333E+00_JPRB, 0.52274E+00_JPRB, 0.57037E+00_JPRB, 0.59813E+00_JPRB, 0.60671E+00_JPRB, 0.63280E+00_JPRB, 0.65541E+00_JPRB ,&
&  0.67043E+00_JPRB, 0.68191E+00_JPRB, 0.69869E+00_JPRB, 0.72063E+00_JPRB, 0.75833E+00_JPRB, 0.81536E+00_JPRB, 0.89983E+00_JPRB ,&
&  0.99651E+00_JPRB, 0.11010E+01_JPRB, 0.12127E+01_JPRB, 0.13165E+01_JPRB, 0.13865E+01_JPRB, 0.14457E+01_JPRB, 0.14206E+01_JPRB ,&
&  0.85262E+00_JPRB, 0.11354E+01_JPRB, 0.12961E+01_JPRB, 0.12105E+01_JPRB, 0.11355E+01_JPRB, 0.10685E+01_JPRB, 0.10043E+01_JPRB ,&
&  0.94149E+00_JPRB, 0.87453E+00_JPRB, 0.84881E+00_JPRB, 0.85354E+00_JPRB, 0.77467E+00_JPRB, 0.77336E+00_JPRB, 0.79885E+00_JPRB ,&
&  0.84998E+00_JPRB, 0.94858E+00_JPRB, 0.11326E+01_JPRB, 0.13402E+01_JPRB, 0.14663E+01_JPRB, 0.15934E+01_JPRB, 0.18180E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.23982E+01_JPRB, 0.25771E+01_JPRB, 0.19754E+01_JPRB, 0.17179E+01_JPRB, 0.16788E+01_JPRB, 0.15352E+01_JPRB ,&
&  0.10399E+01_JPRB, 0.82377E+00_JPRB, 0.82581E+00_JPRB, 0.81115E+00_JPRB, 0.72750E+00_JPRB, 0.68187E+00_JPRB, 0.62831E+00_JPRB ,&
&  0.56454E+00_JPRB, 0.51458E+00_JPRB, 0.48103E+00_JPRB, 0.50991E+00_JPRB, 0.53965E+00_JPRB, 0.55637E+00_JPRB, 0.53895E+00_JPRB ,&
&  0.56562E+00_JPRB, 0.55532E+00_JPRB, 0.55564E+00_JPRB, 0.57116E+00_JPRB, 0.63420E+00_JPRB, 0.66363E+00_JPRB, 0.67822E+00_JPRB ,&
&  0.69128E+00_JPRB, 0.70527E+00_JPRB, 0.71179E+00_JPRB, 0.71931E+00_JPRB, 0.72166E+00_JPRB, 0.71671E+00_JPRB, 0.70398E+00_JPRB ,&
&  0.68431E+00_JPRB, 0.67715E+00_JPRB, 0.65846E+00_JPRB, 0.63885E+00_JPRB, 0.60630E+00_JPRB, 0.61828E+00_JPRB, 0.55228E+00_JPRB ,&
&  0.50742E+00_JPRB, 0.47602E+00_JPRB, 0.45434E+00_JPRB, 0.44956E+00_JPRB, 0.45777E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.47688E+00_JPRB, 0.46556E+00_JPRB, 0.44819E+00_JPRB, 0.45708E+00_JPRB, 0.45897E+00_JPRB, 0.47656E+00_JPRB, 0.49674E+00_JPRB ,&
&  0.52400E+00_JPRB, 0.57530E+00_JPRB, 0.62530E+00_JPRB, 0.66687E+00_JPRB, 0.69287E+00_JPRB, 0.72583E+00_JPRB, 0.75745E+00_JPRB ,&
&  0.78016E+00_JPRB, 0.80270E+00_JPRB, 0.83421E+00_JPRB, 0.87932E+00_JPRB, 0.93750E+00_JPRB, 0.10032E+01_JPRB, 0.10828E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.12666E+01_JPRB, 0.13873E+01_JPRB, 0.14331E+01_JPRB, 0.14709E+01_JPRB, 0.15158E+01_JPRB, 0.15006E+01_JPRB ,&
&  0.10419E+01_JPRB, 0.12362E+01_JPRB, 0.12027E+01_JPRB, 0.11534E+01_JPRB, 0.11147E+01_JPRB, 0.10521E+01_JPRB, 0.99777E+00_JPRB ,&
&  0.95723E+00_JPRB, 0.88533E+00_JPRB, 0.85147E+00_JPRB, 0.79023E+00_JPRB, 0.81379E+00_JPRB, 0.82332E+00_JPRB, 0.87918E+00_JPRB ,&
&  0.96949E+00_JPRB, 0.11101E+01_JPRB, 0.12871E+01_JPRB, 0.14628E+01_JPRB, 0.16363E+01_JPRB, 0.18240E+01_JPRB, 0.20846E+01_JPRB ,&
&  0.23623E+01_JPRB, 0.25469E+01_JPRB, 0.26530E+01_JPRB, 0.23056E+01_JPRB, 0.18798E+01_JPRB, 0.16590E+01_JPRB, 0.12924E+01_JPRB ,&
&  0.94119E+00_JPRB, 0.81813E+00_JPRB, 0.83875E+00_JPRB, 0.81560E+00_JPRB, 0.72289E+00_JPRB, 0.65256E+00_JPRB, 0.58713E+00_JPRB ,&
&  0.53183E+00_JPRB, 0.49754E+00_JPRB, 0.51913E+00_JPRB, 0.59324E+00_JPRB, 0.63336E+00_JPRB, 0.65719E+00_JPRB, 0.64917E+00_JPRB ,&
&  0.63412E+00_JPRB, 0.59796E+00_JPRB, 0.57032E+00_JPRB, 0.58376E+00_JPRB, 0.62670E+00_JPRB, 0.66758E+00_JPRB, 0.66872E+00_JPRB ,&
&  0.65001E+00_JPRB, 0.64261E+00_JPRB, 0.65127E+00_JPRB, 0.67122E+00_JPRB, 0.68556E+00_JPRB, 0.70538E+00_JPRB, 0.72194E+00_JPRB ,&
&  0.74555E+00_JPRB, 0.78480E+00_JPRB, 0.76889E+00_JPRB, 0.75285E+00_JPRB, 0.83297E+00_JPRB, 0.88313E+00_JPRB, 0.57230E+00_JPRB ,&
&  0.48972E+00_JPRB, 0.45712E+00_JPRB, 0.45700E+00_JPRB, 0.45142E+00_JPRB, 0.48517E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.51224E+00_JPRB, 0.53478E+00_JPRB, 0.52581E+00_JPRB, 0.52960E+00_JPRB, 0.53678E+00_JPRB, 0.54826E+00_JPRB, 0.57250E+00_JPRB ,&
&  0.57901E+00_JPRB, 0.61191E+00_JPRB, 0.64729E+00_JPRB, 0.67660E+00_JPRB, 0.71682E+00_JPRB, 0.77268E+00_JPRB, 0.84285E+00_JPRB ,&
&  0.91837E+00_JPRB, 0.96415E+00_JPRB, 0.10049E+01_JPRB, 0.10368E+01_JPRB, 0.10914E+01_JPRB, 0.11629E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.12895E+01_JPRB, 0.13512E+01_JPRB, 0.14241E+01_JPRB, 0.15282E+01_JPRB, 0.14630E+01_JPRB, 0.14544E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.11921E+01_JPRB, 0.11762E+01_JPRB, 0.11542E+01_JPRB, 0.11069E+01_JPRB, 0.10293E+01_JPRB, 0.97839E+00_JPRB ,&
&  0.93447E+00_JPRB, 0.88770E+00_JPRB, 0.83155E+00_JPRB, 0.80042E+00_JPRB, 0.86913E+00_JPRB, 0.87841E+00_JPRB, 0.98126E+00_JPRB ,&
&  0.11080E+01_JPRB, 0.12488E+01_JPRB, 0.13851E+01_JPRB, 0.15488E+01_JPRB, 0.17581E+01_JPRB, 0.19913E+01_JPRB, 0.22312E+01_JPRB ,&
&  0.24107E+01_JPRB, 0.25555E+01_JPRB, 0.25137E+01_JPRB, 0.20926E+01_JPRB, 0.18950E+01_JPRB, 0.14883E+01_JPRB, 0.10808E+01_JPRB ,&
&  0.90738E+00_JPRB, 0.85668E+00_JPRB, 0.87620E+00_JPRB, 0.84956E+00_JPRB, 0.74425E+00_JPRB, 0.66025E+00_JPRB, 0.59248E+00_JPRB ,&
&  0.54555E+00_JPRB, 0.54452E+00_JPRB, 0.59162E+00_JPRB, 0.64927E+00_JPRB, 0.70882E+00_JPRB, 0.76530E+00_JPRB, 0.76883E+00_JPRB ,&
&  0.73875E+00_JPRB, 0.68261E+00_JPRB, 0.65595E+00_JPRB, 0.67953E+00_JPRB, 0.76811E+00_JPRB, 0.85817E+00_JPRB, 0.92766E+00_JPRB ,&
&  0.72374E+00_JPRB, 0.60003E+00_JPRB, 0.61670E+00_JPRB, 0.65278E+00_JPRB, 0.70147E+00_JPRB, 0.77419E+00_JPRB, 0.88674E+00_JPRB ,&
&  0.10575E+01_JPRB, 0.14192E+01_JPRB, 0.13837E+01_JPRB, 0.11411E+01_JPRB, 0.16511E+01_JPRB, 0.13726E+01_JPRB, 0.51480E+00_JPRB ,&
&  0.49545E+00_JPRB, 0.48667E+00_JPRB, 0.49226E+00_JPRB, 0.49612E+00_JPRB, 0.50839E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.61155E+00_JPRB, 0.62472E+00_JPRB, 0.64518E+00_JPRB, 0.64868E+00_JPRB, 0.65829E+00_JPRB, 0.67782E+00_JPRB, 0.69779E+00_JPRB ,&
&  0.71780E+00_JPRB, 0.73001E+00_JPRB, 0.74437E+00_JPRB, 0.76239E+00_JPRB, 0.78482E+00_JPRB, 0.83704E+00_JPRB, 0.91349E+00_JPRB ,&
&  0.97724E+00_JPRB, 0.10384E+01_JPRB, 0.11155E+01_JPRB, 0.11479E+01_JPRB, 0.12028E+01_JPRB, 0.12729E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.13830E+01_JPRB, 0.14170E+01_JPRB, 0.14510E+01_JPRB, 0.15293E+01_JPRB, 0.14931E+01_JPRB, 0.15349E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.12175E+01_JPRB, 0.11908E+01_JPRB, 0.11356E+01_JPRB, 0.11088E+01_JPRB, 0.10739E+01_JPRB, 0.10203E+01_JPRB, 0.98010E+00_JPRB ,&
&  0.95646E+00_JPRB, 0.89962E+00_JPRB, 0.83760E+00_JPRB, 0.86138E+00_JPRB, 0.89519E+00_JPRB, 0.96014E+00_JPRB, 0.10900E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.13352E+01_JPRB, 0.14471E+01_JPRB, 0.16079E+01_JPRB, 0.18171E+01_JPRB, 0.20423E+01_JPRB, 0.22787E+01_JPRB ,&
&  0.24342E+01_JPRB, 0.25401E+01_JPRB, 0.25460E+01_JPRB, 0.21608E+01_JPRB, 0.20480E+01_JPRB, 0.16148E+01_JPRB, 0.11558E+01_JPRB ,&
&  0.92074E+00_JPRB, 0.85103E+00_JPRB, 0.87718E+00_JPRB, 0.92176E+00_JPRB, 0.82516E+00_JPRB, 0.75513E+00_JPRB, 0.67948E+00_JPRB ,&
&  0.63375E+00_JPRB, 0.62616E+00_JPRB, 0.64805E+00_JPRB, 0.70813E+00_JPRB, 0.77879E+00_JPRB, 0.86105E+00_JPRB, 0.89315E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.79927E+00_JPRB, 0.74316E+00_JPRB, 0.70707E+00_JPRB, 0.70935E+00_JPRB, 0.73870E+00_JPRB, 0.68468E+00_JPRB ,&
&  0.65948E+00_JPRB, 0.60259E+00_JPRB, 0.60178E+00_JPRB, 0.66281E+00_JPRB, 0.75238E+00_JPRB, 0.97144E+00_JPRB, 0.13762E+01_JPRB ,&
&  0.19751E+01_JPRB, 0.21160E+01_JPRB, 0.25964E+01_JPRB, 0.34611E+01_JPRB, 0.25501E+01_JPRB, 0.17984E+01_JPRB, 0.56737E+00_JPRB ,&
&  0.58257E+00_JPRB, 0.57947E+00_JPRB, 0.58004E+00_JPRB, 0.58094E+00_JPRB, 0.59057E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.65501E+00_JPRB, 0.67456E+00_JPRB, 0.69633E+00_JPRB, 0.71445E+00_JPRB, 0.73806E+00_JPRB, 0.77237E+00_JPRB, 0.80684E+00_JPRB ,&
&  0.83602E+00_JPRB, 0.85763E+00_JPRB, 0.87351E+00_JPRB, 0.89702E+00_JPRB, 0.92709E+00_JPRB, 0.96787E+00_JPRB, 0.10204E+01_JPRB ,&
&  0.10444E+01_JPRB, 0.10907E+01_JPRB, 0.11176E+01_JPRB, 0.11668E+01_JPRB, 0.11911E+01_JPRB, 0.12503E+01_JPRB, 0.12940E+01_JPRB ,&
&  0.13181E+01_JPRB, 0.13306E+01_JPRB, 0.13444E+01_JPRB, 0.13742E+01_JPRB, 0.13606E+01_JPRB, 0.14185E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.11777E+01_JPRB, 0.11233E+01_JPRB, 0.10330E+01_JPRB, 0.99057E+00_JPRB, 0.95352E+00_JPRB, 0.92550E+00_JPRB, 0.91261E+00_JPRB ,&
&  0.90302E+00_JPRB, 0.89364E+00_JPRB, 0.85921E+00_JPRB, 0.86954E+00_JPRB, 0.92780E+00_JPRB, 0.10390E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.12693E+01_JPRB, 0.13887E+01_JPRB, 0.15005E+01_JPRB, 0.16402E+01_JPRB, 0.18191E+01_JPRB, 0.20056E+01_JPRB, 0.21844E+01_JPRB ,&
&  0.22334E+01_JPRB, 0.22481E+01_JPRB, 0.20508E+01_JPRB, 0.20346E+01_JPRB, 0.20637E+01_JPRB, 0.17232E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.88098E+00_JPRB, 0.85556E+00_JPRB, 0.95168E+00_JPRB, 0.10749E+01_JPRB, 0.10179E+01_JPRB, 0.96306E+00_JPRB, 0.88268E+00_JPRB ,&
&  0.82028E+00_JPRB, 0.76971E+00_JPRB, 0.78038E+00_JPRB, 0.83884E+00_JPRB, 0.91574E+00_JPRB, 0.10019E+01_JPRB, 0.10276E+01_JPRB ,&
&  0.10032E+01_JPRB, 0.92999E+00_JPRB, 0.84818E+00_JPRB, 0.78160E+00_JPRB, 0.73111E+00_JPRB, 0.67403E+00_JPRB, 0.74196E+00_JPRB ,&
&  0.62676E+00_JPRB, 0.59688E+00_JPRB, 0.62405E+00_JPRB, 0.52562E+00_JPRB, 0.70980E+00_JPRB, 0.87325E+00_JPRB, 0.11227E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.16922E+01_JPRB, 0.21113E+01_JPRB, 0.15127E+01_JPRB, 0.11249E+01_JPRB, 0.69968E+00_JPRB, 0.62817E+00_JPRB ,&
&  0.62802E+00_JPRB, 0.61506E+00_JPRB, 0.61572E+00_JPRB, 0.62668E+00_JPRB, 0.63890E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.68798E+00_JPRB, 0.71003E+00_JPRB, 0.73222E+00_JPRB, 0.75248E+00_JPRB, 0.77233E+00_JPRB, 0.79071E+00_JPRB, 0.81033E+00_JPRB ,&
&  0.83058E+00_JPRB, 0.85227E+00_JPRB, 0.87319E+00_JPRB, 0.89907E+00_JPRB, 0.92379E+00_JPRB, 0.93796E+00_JPRB, 0.96019E+00_JPRB ,&
&  0.97005E+00_JPRB, 0.10091E+01_JPRB, 0.10279E+01_JPRB, 0.10655E+01_JPRB, 0.10984E+01_JPRB, 0.11196E+01_JPRB, 0.11582E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.11869E+01_JPRB, 0.11990E+01_JPRB, 0.12292E+01_JPRB, 0.12425E+01_JPRB, 0.13344E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.10754E+01_JPRB, 0.99576E+00_JPRB, 0.92147E+00_JPRB, 0.85879E+00_JPRB, 0.80702E+00_JPRB, 0.75477E+00_JPRB, 0.72795E+00_JPRB ,&
&  0.74079E+00_JPRB, 0.74385E+00_JPRB, 0.76485E+00_JPRB, 0.79928E+00_JPRB, 0.87257E+00_JPRB, 0.10042E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.13518E+01_JPRB, 0.14721E+01_JPRB, 0.15919E+01_JPRB, 0.17244E+01_JPRB, 0.18399E+01_JPRB, 0.19029E+01_JPRB ,&
&  0.18765E+01_JPRB, 0.18145E+01_JPRB, 0.15462E+01_JPRB, 0.16586E+01_JPRB, 0.17399E+01_JPRB, 0.15836E+01_JPRB, 0.12751E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.94513E+00_JPRB, 0.10229E+01_JPRB, 0.12527E+01_JPRB, 0.13352E+01_JPRB, 0.13447E+01_JPRB, 0.12778E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.10858E+01_JPRB, 0.10439E+01_JPRB, 0.10708E+01_JPRB, 0.11280E+01_JPRB, 0.11560E+01_JPRB, 0.10851E+01_JPRB ,&
&  0.10470E+01_JPRB, 0.10153E+01_JPRB, 0.96444E+00_JPRB, 0.89178E+00_JPRB, 0.89807E+00_JPRB, 0.86625E+00_JPRB, 0.97709E+00_JPRB ,&
&  0.59863E+00_JPRB, 0.50899E+00_JPRB, 0.52758E+00_JPRB, 0.57606E+00_JPRB, 0.64484E+00_JPRB, 0.70856E+00_JPRB, 0.87754E+00_JPRB ,&
&  0.10173E+01_JPRB, 0.10684E+01_JPRB, 0.10015E+01_JPRB, 0.82469E+00_JPRB, 0.68666E+00_JPRB, 0.63268E+00_JPRB, 0.63207E+00_JPRB ,&
&  0.63324E+00_JPRB, 0.63040E+00_JPRB, 0.63875E+00_JPRB, 0.65475E+00_JPRB, 0.66917E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.66329E+00_JPRB, 0.68365E+00_JPRB, 0.70434E+00_JPRB, 0.72531E+00_JPRB, 0.74362E+00_JPRB, 0.76099E+00_JPRB, 0.77902E+00_JPRB ,&
&  0.79698E+00_JPRB, 0.81806E+00_JPRB, 0.84024E+00_JPRB, 0.86420E+00_JPRB, 0.88440E+00_JPRB, 0.89456E+00_JPRB, 0.90959E+00_JPRB ,&
&  0.92229E+00_JPRB, 0.92870E+00_JPRB, 0.92423E+00_JPRB, 0.95156E+00_JPRB, 0.97918E+00_JPRB, 0.99734E+00_JPRB, 0.10251E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.10409E+01_JPRB, 0.10373E+01_JPRB, 0.10657E+01_JPRB, 0.10843E+01_JPRB, 0.10368E+01_JPRB, 0.11434E+01_JPRB ,&
&  0.87445E+00_JPRB, 0.10123E+01_JPRB, 0.93274E+00_JPRB, 0.81570E+00_JPRB, 0.78692E+00_JPRB, 0.69465E+00_JPRB, 0.64598E+00_JPRB ,&
&  0.68321E+00_JPRB, 0.67979E+00_JPRB, 0.67512E+00_JPRB, 0.69159E+00_JPRB, 0.76238E+00_JPRB, 0.88958E+00_JPRB, 0.97156E+00_JPRB ,&
&  0.10910E+01_JPRB, 0.12136E+01_JPRB, 0.13608E+01_JPRB, 0.15212E+01_JPRB, 0.16916E+01_JPRB, 0.17761E+01_JPRB, 0.16837E+01_JPRB ,&
&  0.15512E+01_JPRB, 0.14474E+01_JPRB, 0.11456E+01_JPRB, 0.12466E+01_JPRB, 0.13986E+01_JPRB, 0.13530E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.11161E+01_JPRB, 0.13296E+01_JPRB, 0.15904E+01_JPRB, 0.17457E+01_JPRB, 0.17864E+01_JPRB, 0.16854E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.13736E+01_JPRB, 0.12747E+01_JPRB, 0.12939E+01_JPRB, 0.12809E+01_JPRB, 0.12039E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.97967E+00_JPRB, 0.10140E+01_JPRB, 0.10582E+01_JPRB, 0.10679E+01_JPRB, 0.11076E+01_JPRB, 0.15776E+01_JPRB, 0.17891E+01_JPRB ,&
&  0.72069E+00_JPRB, 0.61671E+00_JPRB, 0.45315E+00_JPRB, 0.55079E+00_JPRB, 0.58513E+00_JPRB, 0.58839E+00_JPRB, 0.63792E+00_JPRB ,&
&  0.67255E+00_JPRB, 0.63714E+00_JPRB, 0.62836E+00_JPRB, 0.60580E+00_JPRB, 0.60654E+00_JPRB, 0.60271E+00_JPRB, 0.61584E+00_JPRB ,&
&  0.62310E+00_JPRB, 0.62308E+00_JPRB, 0.63064E+00_JPRB, 0.64147E+00_JPRB, 0.64966E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.62395E+00_JPRB, 0.63291E+00_JPRB, 0.63965E+00_JPRB, 0.65763E+00_JPRB, 0.68464E+00_JPRB, 0.69185E+00_JPRB, 0.70844E+00_JPRB ,&
&  0.71869E+00_JPRB, 0.73782E+00_JPRB, 0.75826E+00_JPRB, 0.78269E+00_JPRB, 0.79891E+00_JPRB, 0.80682E+00_JPRB, 0.82983E+00_JPRB ,&
&  0.84995E+00_JPRB, 0.84614E+00_JPRB, 0.85745E+00_JPRB, 0.88036E+00_JPRB, 0.88557E+00_JPRB, 0.91806E+00_JPRB, 0.91412E+00_JPRB ,&
&  0.94316E+00_JPRB, 0.94485E+00_JPRB, 0.97450E+00_JPRB, 0.10661E+01_JPRB, 0.11719E+01_JPRB, 0.13734E+01_JPRB, 0.17448E+01_JPRB ,&
&  0.14686E+01_JPRB, 0.11728E+01_JPRB, 0.10677E+01_JPRB, 0.90677E+00_JPRB, 0.90613E+00_JPRB, 0.93500E+00_JPRB, 0.92102E+00_JPRB ,&
&  0.89169E+00_JPRB, 0.83926E+00_JPRB, 0.81126E+00_JPRB, 0.79136E+00_JPRB, 0.80653E+00_JPRB, 0.87188E+00_JPRB, 0.90650E+00_JPRB ,&
&  0.88387E+00_JPRB, 0.98599E+00_JPRB, 0.11296E+01_JPRB, 0.13324E+01_JPRB, 0.15010E+01_JPRB, 0.18489E+01_JPRB, 0.18511E+01_JPRB ,&
&  0.17555E+01_JPRB, 0.17024E+01_JPRB, 0.15669E+01_JPRB, 0.15992E+01_JPRB, 0.16685E+01_JPRB, 0.15582E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.13204E+01_JPRB, 0.12576E+01_JPRB, 0.14000E+01_JPRB, 0.19597E+01_JPRB, 0.21854E+01_JPRB, 0.22327E+01_JPRB, 0.21112E+01_JPRB ,&
&  0.19403E+01_JPRB, 0.17107E+01_JPRB, 0.15049E+01_JPRB, 0.13916E+01_JPRB, 0.13319E+01_JPRB, 0.12027E+01_JPRB, 0.10565E+01_JPRB ,&
&  0.94558E+00_JPRB, 0.99419E+00_JPRB, 0.10713E+01_JPRB, 0.10741E+01_JPRB, 0.11851E+01_JPRB, 0.10868E+01_JPRB, 0.13191E+01_JPRB ,&
&  0.77005E+00_JPRB, 0.76282E+00_JPRB, 0.67271E+00_JPRB, 0.93244E+00_JPRB, 0.64055E+00_JPRB, 0.48315E+00_JPRB, 0.47500E+00_JPRB ,&
&  0.48893E+00_JPRB, 0.49850E+00_JPRB, 0.51281E+00_JPRB, 0.52454E+00_JPRB, 0.53884E+00_JPRB, 0.56313E+00_JPRB, 0.58883E+00_JPRB ,&
&  0.61087E+00_JPRB, 0.59983E+00_JPRB, 0.60048E+00_JPRB, 0.60604E+00_JPRB, 0.62096E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.58199E+00_JPRB, 0.58083E+00_JPRB, 0.57358E+00_JPRB, 0.57475E+00_JPRB, 0.58015E+00_JPRB, 0.59475E+00_JPRB, 0.60398E+00_JPRB ,&
&  0.60281E+00_JPRB, 0.60875E+00_JPRB, 0.59224E+00_JPRB, 0.59463E+00_JPRB, 0.61442E+00_JPRB, 0.62959E+00_JPRB, 0.64419E+00_JPRB ,&
&  0.68714E+00_JPRB, 0.70034E+00_JPRB, 0.69645E+00_JPRB, 0.69940E+00_JPRB, 0.69273E+00_JPRB, 0.74172E+00_JPRB, 0.74681E+00_JPRB ,&
&  0.72908E+00_JPRB, 0.75856E+00_JPRB, 0.89673E+00_JPRB, 0.12609E+01_JPRB, 0.16420E+01_JPRB, 0.16660E+01_JPRB, 0.18819E+01_JPRB ,&
&  0.17052E+01_JPRB, 0.14236E+01_JPRB, 0.14901E+01_JPRB, 0.12965E+01_JPRB, 0.13227E+01_JPRB, 0.13860E+01_JPRB, 0.14021E+01_JPRB ,&
&  0.14265E+01_JPRB, 0.14111E+01_JPRB, 0.14169E+01_JPRB, 0.14321E+01_JPRB, 0.14245E+01_JPRB, 0.14135E+01_JPRB, 0.13870E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.12885E+01_JPRB, 0.13865E+01_JPRB, 0.16121E+01_JPRB, 0.18746E+01_JPRB, 0.21589E+01_JPRB, 0.22856E+01_JPRB ,&
&  0.23812E+01_JPRB, 0.24829E+01_JPRB, 0.23952E+01_JPRB, 0.24786E+01_JPRB, 0.26100E+01_JPRB, 0.23726E+01_JPRB, 0.21571E+01_JPRB ,&
&  0.19561E+01_JPRB, 0.15246E+01_JPRB, 0.15685E+01_JPRB, 0.23657E+01_JPRB, 0.25588E+01_JPRB, 0.24399E+01_JPRB, 0.24514E+01_JPRB ,&
&  0.23076E+01_JPRB, 0.21056E+01_JPRB, 0.17798E+01_JPRB, 0.15372E+01_JPRB, 0.13402E+01_JPRB, 0.11508E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.95611E+00_JPRB, 0.93419E+00_JPRB, 0.10230E+01_JPRB, 0.10475E+01_JPRB, 0.10043E+01_JPRB, 0.10188E+01_JPRB, 0.95968E+00_JPRB ,&
&  0.85080E+00_JPRB, 0.87906E+00_JPRB, 0.13391E+01_JPRB, 0.18716E+01_JPRB, 0.87806E+00_JPRB, 0.53762E+00_JPRB, 0.42141E+00_JPRB ,&
&  0.43867E+00_JPRB, 0.45836E+00_JPRB, 0.46520E+00_JPRB, 0.47367E+00_JPRB, 0.49041E+00_JPRB, 0.50490E+00_JPRB, 0.52784E+00_JPRB ,&
&  0.54135E+00_JPRB, 0.55241E+00_JPRB, 0.56411E+00_JPRB, 0.57992E+00_JPRB, 0.57772E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.66762E+00_JPRB, 0.70354E+00_JPRB, 0.72772E+00_JPRB, 0.75316E+00_JPRB, 0.76301E+00_JPRB, 0.75388E+00_JPRB, 0.71820E+00_JPRB ,&
&  0.69094E+00_JPRB, 0.66547E+00_JPRB, 0.65895E+00_JPRB, 0.64610E+00_JPRB, 0.67423E+00_JPRB, 0.72733E+00_JPRB, 0.76645E+00_JPRB ,&
&  0.86612E+00_JPRB, 0.81026E+00_JPRB, 0.80640E+00_JPRB, 0.83377E+00_JPRB, 0.83640E+00_JPRB, 0.87568E+00_JPRB, 0.10244E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.15339E+01_JPRB, 0.19336E+01_JPRB, 0.26018E+01_JPRB, 0.28832E+01_JPRB, 0.22880E+01_JPRB, 0.23674E+01_JPRB ,&
&  0.21117E+01_JPRB, 0.18304E+01_JPRB, 0.17933E+01_JPRB, 0.17635E+01_JPRB, 0.18072E+01_JPRB, 0.18285E+01_JPRB, 0.18805E+01_JPRB ,&
&  0.19336E+01_JPRB, 0.19916E+01_JPRB, 0.20352E+01_JPRB, 0.20661E+01_JPRB, 0.21059E+01_JPRB, 0.21583E+01_JPRB, 0.22327E+01_JPRB ,&
&  0.22849E+01_JPRB, 0.22926E+01_JPRB, 0.23231E+01_JPRB, 0.23012E+01_JPRB, 0.24699E+01_JPRB, 0.26558E+01_JPRB, 0.28570E+01_JPRB ,&
&  0.29960E+01_JPRB, 0.31114E+01_JPRB, 0.33914E+01_JPRB, 0.34637E+01_JPRB, 0.34524E+01_JPRB, 0.33849E+01_JPRB, 0.31618E+01_JPRB ,&
&  0.28735E+01_JPRB, 0.23104E+01_JPRB, 0.21707E+01_JPRB, 0.34743E+01_JPRB, 0.34225E+01_JPRB, 0.27603E+01_JPRB, 0.28112E+01_JPRB ,&
&  0.25756E+01_JPRB, 0.24124E+01_JPRB, 0.20878E+01_JPRB, 0.17041E+01_JPRB, 0.14209E+01_JPRB, 0.12355E+01_JPRB, 0.11638E+01_JPRB ,&
&  0.10915E+01_JPRB, 0.96341E+00_JPRB, 0.96963E+00_JPRB, 0.98769E+00_JPRB, 0.92225E+00_JPRB, 0.89991E+00_JPRB, 0.96424E+00_JPRB ,&
&  0.91948E+00_JPRB, 0.86264E+00_JPRB, 0.10648E+01_JPRB, 0.14367E+01_JPRB, 0.10549E+01_JPRB, 0.53232E+00_JPRB, 0.45782E+00_JPRB ,&
&  0.47120E+00_JPRB, 0.48529E+00_JPRB, 0.49845E+00_JPRB, 0.50922E+00_JPRB, 0.51806E+00_JPRB, 0.53893E+00_JPRB, 0.55546E+00_JPRB ,&
&  0.57338E+00_JPRB, 0.60505E+00_JPRB, 0.61883E+00_JPRB, 0.63595E+00_JPRB, 0.65793E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.10097E+01_JPRB, 0.10743E+01_JPRB, 0.11344E+01_JPRB, 0.11754E+01_JPRB, 0.11902E+01_JPRB, 0.12053E+01_JPRB, 0.12069E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12149E+01_JPRB, 0.12517E+01_JPRB, 0.13387E+01_JPRB, 0.15035E+01_JPRB, 0.15981E+01_JPRB, 0.17085E+01_JPRB ,&
&  0.17173E+01_JPRB, 0.17360E+01_JPRB, 0.17584E+01_JPRB, 0.18031E+01_JPRB, 0.18839E+01_JPRB, 0.20013E+01_JPRB, 0.22681E+01_JPRB ,&
&  0.27807E+01_JPRB, 0.31192E+01_JPRB, 0.31214E+01_JPRB, 0.27974E+01_JPRB, 0.22040E+01_JPRB, 0.20296E+01_JPRB, 0.22891E+01_JPRB ,&
&  0.23571E+01_JPRB, 0.21780E+01_JPRB, 0.20324E+01_JPRB, 0.20093E+01_JPRB, 0.20227E+01_JPRB, 0.20488E+01_JPRB, 0.20670E+01_JPRB ,&
&  0.20988E+01_JPRB, 0.21392E+01_JPRB, 0.21673E+01_JPRB, 0.22019E+01_JPRB, 0.22778E+01_JPRB, 0.23960E+01_JPRB, 0.25498E+01_JPRB ,&
&  0.27738E+01_JPRB, 0.29211E+01_JPRB, 0.29755E+01_JPRB, 0.29834E+01_JPRB, 0.29854E+01_JPRB, 0.30468E+01_JPRB, 0.32060E+01_JPRB ,&
&  0.33905E+01_JPRB, 0.34943E+01_JPRB, 0.38575E+01_JPRB, 0.41207E+01_JPRB, 0.40528E+01_JPRB, 0.39640E+01_JPRB, 0.38716E+01_JPRB ,&
&  0.37786E+01_JPRB, 0.36678E+01_JPRB, 0.32631E+01_JPRB, 0.45948E+01_JPRB, 0.46672E+01_JPRB, 0.39008E+01_JPRB, 0.32036E+01_JPRB ,&
&  0.28712E+01_JPRB, 0.26150E+01_JPRB, 0.21218E+01_JPRB, 0.16258E+01_JPRB, 0.13763E+01_JPRB, 0.13252E+01_JPRB, 0.12088E+01_JPRB ,&
&  0.12506E+01_JPRB, 0.10683E+01_JPRB, 0.99195E+00_JPRB, 0.97305E+00_JPRB, 0.90644E+00_JPRB, 0.96246E+00_JPRB, 0.96339E+00_JPRB ,&
&  0.86053E+00_JPRB, 0.92241E+00_JPRB, 0.11204E+01_JPRB, 0.16278E+01_JPRB, 0.23530E+01_JPRB, 0.66632E+00_JPRB, 0.51392E+00_JPRB ,&
&  0.53039E+00_JPRB, 0.56375E+00_JPRB, 0.60398E+00_JPRB, 0.62682E+00_JPRB, 0.63600E+00_JPRB, 0.67419E+00_JPRB, 0.73324E+00_JPRB ,&
&  0.76987E+00_JPRB, 0.80062E+00_JPRB, 0.83522E+00_JPRB, 0.91010E+00_JPRB, 0.95492E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.12912E+01_JPRB, 0.13621E+01_JPRB, 0.14166E+01_JPRB, 0.14826E+01_JPRB, 0.15453E+01_JPRB, 0.16102E+01_JPRB, 0.16956E+01_JPRB ,&
&  0.18059E+01_JPRB, 0.19269E+01_JPRB, 0.20435E+01_JPRB, 0.21325E+01_JPRB, 0.22180E+01_JPRB, 0.22783E+01_JPRB, 0.23295E+01_JPRB ,&
&  0.23598E+01_JPRB, 0.24308E+01_JPRB, 0.25058E+01_JPRB, 0.26094E+01_JPRB, 0.27465E+01_JPRB, 0.29049E+01_JPRB, 0.30666E+01_JPRB ,&
&  0.31737E+01_JPRB, 0.30407E+01_JPRB, 0.26431E+01_JPRB, 0.19516E+01_JPRB, 0.17433E+01_JPRB, 0.20800E+01_JPRB, 0.20948E+01_JPRB ,&
&  0.21151E+01_JPRB, 0.20836E+01_JPRB, 0.20333E+01_JPRB, 0.20803E+01_JPRB, 0.21145E+01_JPRB, 0.21473E+01_JPRB, 0.21669E+01_JPRB ,&
&  0.22053E+01_JPRB, 0.22593E+01_JPRB, 0.22998E+01_JPRB, 0.23441E+01_JPRB, 0.24358E+01_JPRB, 0.26167E+01_JPRB, 0.28874E+01_JPRB ,&
&  0.32278E+01_JPRB, 0.34296E+01_JPRB, 0.35507E+01_JPRB, 0.36361E+01_JPRB, 0.35117E+01_JPRB, 0.34179E+01_JPRB, 0.34384E+01_JPRB ,&
&  0.35942E+01_JPRB, 0.38112E+01_JPRB, 0.42210E+01_JPRB, 0.45964E+01_JPRB, 0.47394E+01_JPRB, 0.45171E+01_JPRB, 0.44638E+01_JPRB ,&
&  0.44878E+01_JPRB, 0.43207E+01_JPRB, 0.43503E+01_JPRB, 0.54450E+01_JPRB, 0.34842E+01_JPRB, 0.34137E+01_JPRB, 0.32040E+01_JPRB ,&
&  0.30037E+01_JPRB, 0.29231E+01_JPRB, 0.21838E+01_JPRB, 0.15695E+01_JPRB, 0.13421E+01_JPRB, 0.10598E+01_JPRB, 0.11646E+01_JPRB ,&
&  0.13567E+01_JPRB, 0.12349E+01_JPRB, 0.10495E+01_JPRB, 0.98195E+00_JPRB, 0.97556E+00_JPRB, 0.10142E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.96162E+00_JPRB, 0.98638E+00_JPRB, 0.11102E+01_JPRB, 0.15340E+01_JPRB, 0.14401E+01_JPRB, 0.73750E+00_JPRB, 0.58929E+00_JPRB ,&
&  0.58958E+00_JPRB, 0.64010E+00_JPRB, 0.70063E+00_JPRB, 0.75302E+00_JPRB, 0.79775E+00_JPRB, 0.85097E+00_JPRB, 0.91092E+00_JPRB ,&
&  0.98190E+00_JPRB, 0.10750E+01_JPRB, 0.11410E+01_JPRB, 0.11956E+01_JPRB, 0.12374E+01_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.13499E+01_JPRB, 0.14236E+01_JPRB, 0.15128E+01_JPRB, 0.16201E+01_JPRB, 0.17332E+01_JPRB, 0.18065E+01_JPRB, 0.18427E+01_JPRB ,&
&  0.19222E+01_JPRB, 0.20271E+01_JPRB, 0.21443E+01_JPRB, 0.22386E+01_JPRB, 0.22919E+01_JPRB, 0.23691E+01_JPRB, 0.24632E+01_JPRB ,&
&  0.25554E+01_JPRB, 0.26697E+01_JPRB, 0.28248E+01_JPRB, 0.30150E+01_JPRB, 0.32431E+01_JPRB, 0.35209E+01_JPRB, 0.36404E+01_JPRB ,&
&  0.28744E+01_JPRB, 0.19147E+01_JPRB, 0.23564E+01_JPRB, 0.23384E+01_JPRB, 0.23125E+01_JPRB, 0.24435E+01_JPRB, 0.24164E+01_JPRB ,&
&  0.21805E+01_JPRB, 0.21203E+01_JPRB, 0.21267E+01_JPRB, 0.20998E+01_JPRB, 0.20556E+01_JPRB, 0.21101E+01_JPRB, 0.21669E+01_JPRB ,&
&  0.22230E+01_JPRB, 0.22823E+01_JPRB, 0.23417E+01_JPRB, 0.24023E+01_JPRB, 0.25242E+01_JPRB, 0.27380E+01_JPRB, 0.30848E+01_JPRB ,&
&  0.34269E+01_JPRB, 0.36323E+01_JPRB, 0.38033E+01_JPRB, 0.38930E+01_JPRB, 0.37855E+01_JPRB, 0.36836E+01_JPRB, 0.37222E+01_JPRB ,&
&  0.39110E+01_JPRB, 0.41865E+01_JPRB, 0.45581E+01_JPRB, 0.48338E+01_JPRB, 0.50859E+01_JPRB, 0.47278E+01_JPRB, 0.49471E+01_JPRB ,&
&  0.51064E+01_JPRB, 0.48596E+01_JPRB, 0.50976E+01_JPRB, 0.56771E+01_JPRB, 0.37997E+01_JPRB, 0.34741E+01_JPRB, 0.29867E+01_JPRB ,&
&  0.31165E+01_JPRB, 0.29666E+01_JPRB, 0.24983E+01_JPRB, 0.19526E+01_JPRB, 0.15828E+01_JPRB, 0.12183E+01_JPRB, 0.13455E+01_JPRB ,&
&  0.15769E+01_JPRB, 0.15853E+01_JPRB, 0.11954E+01_JPRB, 0.10618E+01_JPRB, 0.92418E+00_JPRB, 0.91842E+00_JPRB, 0.99539E+00_JPRB ,&
&  0.10659E+01_JPRB, 0.10199E+01_JPRB, 0.10652E+01_JPRB, 0.12647E+01_JPRB, 0.11111E+01_JPRB, 0.81410E+00_JPRB, 0.67509E+00_JPRB ,&
&  0.65246E+00_JPRB, 0.69409E+00_JPRB, 0.76678E+00_JPRB, 0.85441E+00_JPRB, 0.89765E+00_JPRB, 0.94762E+00_JPRB, 0.98500E+00_JPRB ,&
&  0.10820E+01_JPRB, 0.11540E+01_JPRB, 0.12109E+01_JPRB, 0.12387E+01_JPRB, 0.12950E+01_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.12618E+01_JPRB, 0.13347E+01_JPRB, 0.14347E+01_JPRB, 0.15758E+01_JPRB, 0.17649E+01_JPRB, 0.19676E+01_JPRB, 0.19152E+01_JPRB ,&
&  0.18314E+01_JPRB, 0.17182E+01_JPRB, 0.17855E+01_JPRB, 0.18551E+01_JPRB, 0.19286E+01_JPRB, 0.20373E+01_JPRB, 0.21923E+01_JPRB ,&
&  0.23835E+01_JPRB, 0.25747E+01_JPRB, 0.27869E+01_JPRB, 0.30039E+01_JPRB, 0.32156E+01_JPRB, 0.35997E+01_JPRB, 0.35361E+01_JPRB ,&
&  0.28898E+01_JPRB, 0.20435E+01_JPRB, 0.22769E+01_JPRB, 0.20687E+01_JPRB, 0.21725E+01_JPRB, 0.24327E+01_JPRB, 0.24509E+01_JPRB ,&
&  0.21040E+01_JPRB, 0.20022E+01_JPRB, 0.19769E+01_JPRB, 0.19508E+01_JPRB, 0.19054E+01_JPRB, 0.19833E+01_JPRB, 0.20823E+01_JPRB ,&
&  0.21476E+01_JPRB, 0.21936E+01_JPRB, 0.22520E+01_JPRB, 0.23183E+01_JPRB, 0.24305E+01_JPRB, 0.26414E+01_JPRB, 0.30411E+01_JPRB ,&
&  0.33686E+01_JPRB, 0.36898E+01_JPRB, 0.40128E+01_JPRB, 0.40933E+01_JPRB, 0.40106E+01_JPRB, 0.40193E+01_JPRB, 0.41739E+01_JPRB ,&
&  0.43929E+01_JPRB, 0.47389E+01_JPRB, 0.48795E+01_JPRB, 0.48854E+01_JPRB, 0.45957E+01_JPRB, 0.50800E+01_JPRB, 0.48748E+01_JPRB ,&
&  0.52636E+01_JPRB, 0.51657E+01_JPRB, 0.59373E+01_JPRB, 0.48501E+01_JPRB, 0.44870E+01_JPRB, 0.42300E+01_JPRB, 0.38405E+01_JPRB ,&
&  0.32939E+01_JPRB, 0.32783E+01_JPRB, 0.29546E+01_JPRB, 0.24132E+01_JPRB, 0.19851E+01_JPRB, 0.16784E+01_JPRB, 0.19326E+01_JPRB ,&
&  0.20253E+01_JPRB, 0.20012E+01_JPRB, 0.13217E+01_JPRB, 0.99049E+00_JPRB, 0.80914E+00_JPRB, 0.77421E+00_JPRB, 0.92757E+00_JPRB ,&
&  0.11391E+01_JPRB, 0.11773E+01_JPRB, 0.11119E+01_JPRB, 0.11314E+01_JPRB, 0.10399E+01_JPRB, 0.90829E+00_JPRB, 0.81482E+00_JPRB ,&
&  0.80627E+00_JPRB, 0.79830E+00_JPRB, 0.83992E+00_JPRB, 0.91994E+00_JPRB, 0.95957E+00_JPRB, 0.10053E+01_JPRB, 0.99314E+00_JPRB ,&
&  0.10373E+01_JPRB, 0.10830E+01_JPRB, 0.11171E+01_JPRB, 0.11561E+01_JPRB, 0.11977E+01_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.10692E+01_JPRB, 0.10969E+01_JPRB, 0.11521E+01_JPRB, 0.12306E+01_JPRB, 0.13013E+01_JPRB, 0.13808E+01_JPRB, 0.14415E+01_JPRB ,&
&  0.14586E+01_JPRB, 0.14523E+01_JPRB, 0.14557E+01_JPRB, 0.14264E+01_JPRB, 0.14178E+01_JPRB, 0.14257E+01_JPRB, 0.15053E+01_JPRB ,&
&  0.17163E+01_JPRB, 0.20368E+01_JPRB, 0.23860E+01_JPRB, 0.26749E+01_JPRB, 0.28815E+01_JPRB, 0.32241E+01_JPRB, 0.29080E+01_JPRB ,&
&  0.19889E+01_JPRB, 0.26893E+01_JPRB, 0.19008E+01_JPRB, 0.20116E+01_JPRB, 0.21259E+01_JPRB, 0.23565E+01_JPRB, 0.20365E+01_JPRB ,&
&  0.18202E+01_JPRB, 0.18145E+01_JPRB, 0.18289E+01_JPRB, 0.18456E+01_JPRB, 0.18099E+01_JPRB, 0.18495E+01_JPRB, 0.19330E+01_JPRB ,&
&  0.20412E+01_JPRB, 0.21070E+01_JPRB, 0.21395E+01_JPRB, 0.21636E+01_JPRB, 0.22363E+01_JPRB, 0.23725E+01_JPRB, 0.27350E+01_JPRB ,&
&  0.30919E+01_JPRB, 0.35201E+01_JPRB, 0.39526E+01_JPRB, 0.43438E+01_JPRB, 0.42840E+01_JPRB, 0.43826E+01_JPRB, 0.47105E+01_JPRB ,&
&  0.48071E+01_JPRB, 0.47282E+01_JPRB, 0.50273E+01_JPRB, 0.51022E+01_JPRB, 0.51017E+01_JPRB, 0.54612E+01_JPRB, 0.48670E+01_JPRB ,&
&  0.54094E+01_JPRB, 0.51753E+01_JPRB, 0.59462E+01_JPRB, 0.51570E+01_JPRB, 0.48577E+01_JPRB, 0.50450E+01_JPRB, 0.46870E+01_JPRB ,&
&  0.40411E+01_JPRB, 0.39003E+01_JPRB, 0.34277E+01_JPRB, 0.29269E+01_JPRB, 0.26147E+01_JPRB, 0.24870E+01_JPRB, 0.26963E+01_JPRB ,&
&  0.26898E+01_JPRB, 0.23925E+01_JPRB, 0.17017E+01_JPRB, 0.92624E+00_JPRB, 0.74462E+00_JPRB, 0.81033E+00_JPRB, 0.10153E+01_JPRB ,&
&  0.12653E+01_JPRB, 0.13714E+01_JPRB, 0.14308E+01_JPRB, 0.13649E+01_JPRB, 0.11610E+01_JPRB, 0.10171E+01_JPRB, 0.91990E+00_JPRB ,&
&  0.87822E+00_JPRB, 0.84222E+00_JPRB, 0.84616E+00_JPRB, 0.93415E+00_JPRB, 0.10385E+01_JPRB, 0.10938E+01_JPRB, 0.99597E+00_JPRB ,&
&  0.92673E+00_JPRB, 0.91886E+00_JPRB, 0.96241E+00_JPRB, 0.10020E+01_JPRB, 0.10399E+01_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.92560E+00_JPRB, 0.92055E+00_JPRB, 0.94074E+00_JPRB, 0.98841E+00_JPRB, 0.10135E+01_JPRB, 0.10318E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.11725E+01_JPRB, 0.12022E+01_JPRB, 0.12177E+01_JPRB, 0.12077E+01_JPRB, 0.11673E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11732E+01_JPRB, 0.13381E+01_JPRB, 0.16930E+01_JPRB, 0.22193E+01_JPRB, 0.26290E+01_JPRB, 0.33393E+01_JPRB, 0.24283E+01_JPRB ,&
&  0.21765E+01_JPRB, 0.22296E+01_JPRB, 0.18322E+01_JPRB, 0.19851E+01_JPRB, 0.20746E+01_JPRB, 0.21917E+01_JPRB, 0.18651E+01_JPRB ,&
&  0.17881E+01_JPRB, 0.17878E+01_JPRB, 0.17609E+01_JPRB, 0.17743E+01_JPRB, 0.17853E+01_JPRB, 0.17988E+01_JPRB, 0.18005E+01_JPRB ,&
&  0.18519E+01_JPRB, 0.19336E+01_JPRB, 0.19898E+01_JPRB, 0.20065E+01_JPRB, 0.20329E+01_JPRB, 0.21094E+01_JPRB, 0.23942E+01_JPRB ,&
&  0.27580E+01_JPRB, 0.31747E+01_JPRB, 0.37197E+01_JPRB, 0.41371E+01_JPRB, 0.44845E+01_JPRB, 0.46485E+01_JPRB, 0.51410E+01_JPRB ,&
&  0.51524E+01_JPRB, 0.50174E+01_JPRB, 0.53640E+01_JPRB, 0.56415E+01_JPRB, 0.58717E+01_JPRB, 0.60580E+01_JPRB, 0.55025E+01_JPRB ,&
&  0.54314E+01_JPRB, 0.55166E+01_JPRB, 0.57147E+01_JPRB, 0.50983E+01_JPRB, 0.49722E+01_JPRB, 0.54544E+01_JPRB, 0.56060E+01_JPRB ,&
&  0.50462E+01_JPRB, 0.45242E+01_JPRB, 0.41483E+01_JPRB, 0.34423E+01_JPRB, 0.31284E+01_JPRB, 0.31141E+01_JPRB, 0.35768E+01_JPRB ,&
&  0.34756E+01_JPRB, 0.30173E+01_JPRB, 0.16086E+01_JPRB, 0.78927E+00_JPRB, 0.65674E+00_JPRB, 0.87436E+00_JPRB, 0.12075E+01_JPRB ,&
&  0.15571E+01_JPRB, 0.17622E+01_JPRB, 0.18928E+01_JPRB, 0.21807E+01_JPRB, 0.18983E+01_JPRB, 0.15285E+01_JPRB, 0.12668E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.92664E+00_JPRB, 0.85764E+00_JPRB, 0.87763E+00_JPRB, 0.99137E+00_JPRB, 0.11115E+01_JPRB, 0.11700E+01_JPRB ,&
&  0.10893E+01_JPRB, 0.95139E+00_JPRB, 0.88572E+00_JPRB, 0.87289E+00_JPRB, 0.89436E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.82016E+00_JPRB, 0.82384E+00_JPRB, 0.84691E+00_JPRB, 0.88805E+00_JPRB, 0.91711E+00_JPRB, 0.91489E+00_JPRB, 0.89714E+00_JPRB ,&
&  0.92419E+00_JPRB, 0.93556E+00_JPRB, 0.97914E+00_JPRB, 0.10032E+01_JPRB, 0.10322E+01_JPRB, 0.10271E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.10411E+01_JPRB, 0.12815E+01_JPRB, 0.17768E+01_JPRB, 0.22430E+01_JPRB, 0.25865E+01_JPRB, 0.27458E+01_JPRB ,&
&  0.25259E+01_JPRB, 0.22171E+01_JPRB, 0.21259E+01_JPRB, 0.24744E+01_JPRB, 0.26598E+01_JPRB, 0.30219E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.18298E+01_JPRB, 0.18119E+01_JPRB, 0.18159E+01_JPRB, 0.18422E+01_JPRB, 0.18790E+01_JPRB, 0.18560E+01_JPRB, 0.18401E+01_JPRB ,&
&  0.17936E+01_JPRB, 0.17867E+01_JPRB, 0.18074E+01_JPRB, 0.18375E+01_JPRB, 0.18728E+01_JPRB, 0.19269E+01_JPRB, 0.21282E+01_JPRB ,&
&  0.24867E+01_JPRB, 0.29515E+01_JPRB, 0.35212E+01_JPRB, 0.41261E+01_JPRB, 0.45406E+01_JPRB, 0.49525E+01_JPRB, 0.55638E+01_JPRB ,&
&  0.58271E+01_JPRB, 0.60950E+01_JPRB, 0.60770E+01_JPRB, 0.59531E+01_JPRB, 0.61534E+01_JPRB, 0.68846E+01_JPRB, 0.64930E+01_JPRB ,&
&  0.59458E+01_JPRB, 0.53779E+01_JPRB, 0.50664E+01_JPRB, 0.49192E+01_JPRB, 0.49505E+01_JPRB, 0.57856E+01_JPRB, 0.55209E+01_JPRB ,&
&  0.46632E+01_JPRB, 0.40577E+01_JPRB, 0.39194E+01_JPRB, 0.39027E+01_JPRB, 0.38461E+01_JPRB, 0.39508E+01_JPRB, 0.48318E+01_JPRB ,&
&  0.36799E+01_JPRB, 0.17111E+01_JPRB, 0.95992E+00_JPRB, 0.89408E+00_JPRB, 0.57610E+00_JPRB, 0.66927E+00_JPRB, 0.17356E+01_JPRB ,&
&  0.24301E+01_JPRB, 0.31108E+01_JPRB, 0.32720E+01_JPRB, 0.30786E+01_JPRB, 0.28314E+01_JPRB, 0.22965E+01_JPRB, 0.18043E+01_JPRB ,&
&  0.14091E+01_JPRB, 0.11744E+01_JPRB, 0.10573E+01_JPRB, 0.10204E+01_JPRB, 0.10867E+01_JPRB, 0.11894E+01_JPRB, 0.12621E+01_JPRB ,&
&  0.12473E+01_JPRB, 0.11424E+01_JPRB, 0.96781E+00_JPRB, 0.83105E+00_JPRB, 0.79861E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.82581E+00_JPRB, 0.84642E+00_JPRB, 0.88510E+00_JPRB, 0.88841E+00_JPRB, 0.87497E+00_JPRB, 0.89209E+00_JPRB, 0.87047E+00_JPRB ,&
&  0.85400E+00_JPRB, 0.85104E+00_JPRB, 0.86316E+00_JPRB, 0.88654E+00_JPRB, 0.91735E+00_JPRB, 0.93682E+00_JPRB, 0.91868E+00_JPRB ,&
&  0.89825E+00_JPRB, 0.93260E+00_JPRB, 0.10672E+01_JPRB, 0.15031E+01_JPRB, 0.20760E+01_JPRB, 0.26779E+01_JPRB, 0.28814E+01_JPRB ,&
&  0.25704E+01_JPRB, 0.29287E+01_JPRB, 0.36189E+01_JPRB, 0.38586E+01_JPRB, 0.35885E+01_JPRB, 0.36804E+01_JPRB, 0.25861E+01_JPRB ,&
&  0.21472E+01_JPRB, 0.21406E+01_JPRB, 0.21368E+01_JPRB, 0.21500E+01_JPRB, 0.21047E+01_JPRB, 0.19944E+01_JPRB, 0.19434E+01_JPRB ,&
&  0.18976E+01_JPRB, 0.17663E+01_JPRB, 0.17036E+01_JPRB, 0.16967E+01_JPRB, 0.17393E+01_JPRB, 0.17970E+01_JPRB, 0.19555E+01_JPRB ,&
&  0.22630E+01_JPRB, 0.26876E+01_JPRB, 0.32863E+01_JPRB, 0.45259E+01_JPRB, 0.39906E+01_JPRB, 0.54032E+01_JPRB, 0.58981E+01_JPRB ,&
&  0.61442E+01_JPRB, 0.63883E+01_JPRB, 0.60689E+01_JPRB, 0.66453E+01_JPRB, 0.68910E+01_JPRB, 0.74529E+01_JPRB, 0.72420E+01_JPRB ,&
&  0.66615E+01_JPRB, 0.55216E+01_JPRB, 0.52862E+01_JPRB, 0.51043E+01_JPRB, 0.50264E+01_JPRB, 0.59857E+01_JPRB, 0.44250E+01_JPRB ,&
&  0.40251E+01_JPRB, 0.46786E+01_JPRB, 0.44836E+01_JPRB, 0.40896E+01_JPRB, 0.44802E+01_JPRB, 0.47689E+01_JPRB, 0.36786E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.10317E+01_JPRB, 0.87119E+00_JPRB, 0.74807E+00_JPRB, 0.63093E+00_JPRB, 0.99366E+00_JPRB, 0.45124E+01_JPRB ,&
&  0.32018E+01_JPRB, 0.45791E+01_JPRB, 0.46980E+01_JPRB, 0.44899E+01_JPRB, 0.42002E+01_JPRB, 0.35466E+01_JPRB, 0.33613E+01_JPRB ,&
&  0.27977E+01_JPRB, 0.22796E+01_JPRB, 0.18993E+01_JPRB, 0.16360E+01_JPRB, 0.14821E+01_JPRB, 0.14309E+01_JPRB, 0.14258E+01_JPRB ,&
&  0.13894E+01_JPRB, 0.12856E+01_JPRB, 0.11674E+01_JPRB, 0.99583E+00_JPRB, 0.85136E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.98048E+00_JPRB, 0.96144E+00_JPRB, 0.96792E+00_JPRB, 0.95546E+00_JPRB, 0.93318E+00_JPRB, 0.94113E+00_JPRB, 0.93490E+00_JPRB ,&
&  0.90807E+00_JPRB, 0.87909E+00_JPRB, 0.87569E+00_JPRB, 0.89679E+00_JPRB, 0.90511E+00_JPRB, 0.90716E+00_JPRB, 0.88051E+00_JPRB ,&
&  0.85138E+00_JPRB, 0.87955E+00_JPRB, 0.11071E+01_JPRB, 0.13026E+01_JPRB, 0.19165E+01_JPRB, 0.23867E+01_JPRB, 0.28565E+01_JPRB ,&
&  0.28212E+01_JPRB, 0.32860E+01_JPRB, 0.40209E+01_JPRB, 0.42727E+01_JPRB, 0.54675E+01_JPRB, 0.52657E+01_JPRB, 0.44574E+01_JPRB ,&
&  0.33491E+01_JPRB, 0.32034E+01_JPRB, 0.30980E+01_JPRB, 0.28542E+01_JPRB, 0.25487E+01_JPRB, 0.22614E+01_JPRB, 0.20495E+01_JPRB ,&
&  0.19264E+01_JPRB, 0.18462E+01_JPRB, 0.16923E+01_JPRB, 0.16186E+01_JPRB, 0.16434E+01_JPRB, 0.16989E+01_JPRB, 0.18553E+01_JPRB ,&
&  0.21224E+01_JPRB, 0.24779E+01_JPRB, 0.29793E+01_JPRB, 0.42141E+01_JPRB, 0.46079E+01_JPRB, 0.50178E+01_JPRB, 0.59759E+01_JPRB ,&
&  0.65023E+01_JPRB, 0.70144E+01_JPRB, 0.66071E+01_JPRB, 0.78159E+01_JPRB, 0.73835E+01_JPRB, 0.80914E+01_JPRB, 0.84426E+01_JPRB ,&
&  0.74083E+01_JPRB, 0.68097E+01_JPRB, 0.53851E+01_JPRB, 0.52872E+01_JPRB, 0.55299E+01_JPRB, 0.49260E+01_JPRB, 0.46302E+01_JPRB ,&
&  0.51175E+01_JPRB, 0.45740E+01_JPRB, 0.41910E+01_JPRB, 0.39439E+01_JPRB, 0.49174E+01_JPRB, 0.44722E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.13860E+01_JPRB, 0.12536E+01_JPRB, 0.91788E+00_JPRB, 0.77795E+00_JPRB, 0.78095E+00_JPRB, 0.11202E+01_JPRB, 0.23373E+01_JPRB ,&
&  0.27176E+01_JPRB, 0.44939E+01_JPRB, 0.48086E+01_JPRB, 0.49949E+01_JPRB, 0.46204E+01_JPRB, 0.41800E+01_JPRB, 0.39940E+01_JPRB ,&
&  0.39669E+01_JPRB, 0.34697E+01_JPRB, 0.28188E+01_JPRB, 0.24393E+01_JPRB, 0.21360E+01_JPRB, 0.18913E+01_JPRB, 0.17077E+01_JPRB ,&
&  0.15772E+01_JPRB, 0.14175E+01_JPRB, 0.13171E+01_JPRB, 0.12351E+01_JPRB, 0.10862E+01_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.11751E+01_JPRB, 0.11059E+01_JPRB, 0.10671E+01_JPRB, 0.10054E+01_JPRB, 0.95638E+00_JPRB, 0.94149E+00_JPRB, 0.96400E+00_JPRB ,&
&  0.95971E+00_JPRB, 0.93757E+00_JPRB, 0.91851E+00_JPRB, 0.92285E+00_JPRB, 0.93716E+00_JPRB, 0.93505E+00_JPRB, 0.89704E+00_JPRB ,&
&  0.86341E+00_JPRB, 0.88457E+00_JPRB, 0.10835E+01_JPRB, 0.12231E+01_JPRB, 0.15799E+01_JPRB, 0.20437E+01_JPRB, 0.24206E+01_JPRB ,&
&  0.29194E+01_JPRB, 0.35042E+01_JPRB, 0.41764E+01_JPRB, 0.53565E+01_JPRB, 0.67501E+01_JPRB, 0.73214E+01_JPRB, 0.76556E+01_JPRB ,&
&  0.63081E+01_JPRB, 0.50201E+01_JPRB, 0.42568E+01_JPRB, 0.37013E+01_JPRB, 0.30682E+01_JPRB, 0.25645E+01_JPRB, 0.21840E+01_JPRB ,&
&  0.19576E+01_JPRB, 0.17872E+01_JPRB, 0.17445E+01_JPRB, 0.16371E+01_JPRB, 0.16079E+01_JPRB, 0.16378E+01_JPRB, 0.18000E+01_JPRB ,&
&  0.20314E+01_JPRB, 0.23663E+01_JPRB, 0.29499E+01_JPRB, 0.38539E+01_JPRB, 0.52370E+01_JPRB, 0.63451E+01_JPRB, 0.63818E+01_JPRB ,&
&  0.60743E+01_JPRB, 0.61118E+01_JPRB, 0.73365E+01_JPRB, 0.79773E+01_JPRB, 0.77751E+01_JPRB, 0.92221E+01_JPRB, 0.95226E+01_JPRB ,&
&  0.81986E+01_JPRB, 0.70947E+01_JPRB, 0.59017E+01_JPRB, 0.63841E+01_JPRB, 0.54932E+01_JPRB, 0.46556E+01_JPRB, 0.53465E+01_JPRB ,&
&  0.56383E+01_JPRB, 0.50111E+01_JPRB, 0.42230E+01_JPRB, 0.36648E+01_JPRB, 0.36025E+01_JPRB, 0.27462E+01_JPRB, 0.19020E+01_JPRB ,&
&  0.15843E+01_JPRB, 0.12724E+01_JPRB, 0.10332E+01_JPRB, 0.10635E+01_JPRB, 0.90452E+00_JPRB, 0.12316E+01_JPRB, 0.20885E+01_JPRB ,&
&  0.33213E+01_JPRB, 0.39675E+01_JPRB, 0.48643E+01_JPRB, 0.45786E+01_JPRB, 0.43521E+01_JPRB, 0.44943E+01_JPRB, 0.47903E+01_JPRB ,&
&  0.37041E+01_JPRB, 0.35082E+01_JPRB, 0.33667E+01_JPRB, 0.29386E+01_JPRB, 0.25896E+01_JPRB, 0.23120E+01_JPRB, 0.20317E+01_JPRB ,&
&  0.17612E+01_JPRB, 0.15306E+01_JPRB, 0.14008E+01_JPRB, 0.13507E+01_JPRB, 0.12805E+01_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.12961E+01_JPRB, 0.12349E+01_JPRB, 0.11452E+01_JPRB, 0.10208E+01_JPRB, 0.95390E+00_JPRB, 0.93581E+00_JPRB, 0.94280E+00_JPRB ,&
&  0.95306E+00_JPRB, 0.96132E+00_JPRB, 0.95955E+00_JPRB, 0.95695E+00_JPRB, 0.96873E+00_JPRB, 0.95866E+00_JPRB, 0.91731E+00_JPRB ,&
&  0.88067E+00_JPRB, 0.91220E+00_JPRB, 0.80936E+00_JPRB, 0.87673E+00_JPRB, 0.11735E+01_JPRB, 0.17610E+01_JPRB, 0.17669E+01_JPRB ,&
&  0.26049E+01_JPRB, 0.33970E+01_JPRB, 0.40142E+01_JPRB, 0.55054E+01_JPRB, 0.90689E+01_JPRB, 0.89970E+01_JPRB, 0.85515E+01_JPRB ,&
&  0.78887E+01_JPRB, 0.64163E+01_JPRB, 0.50613E+01_JPRB, 0.40044E+01_JPRB, 0.32027E+01_JPRB, 0.26705E+01_JPRB, 0.22818E+01_JPRB ,&
&  0.20402E+01_JPRB, 0.18425E+01_JPRB, 0.17134E+01_JPRB, 0.16931E+01_JPRB, 0.16485E+01_JPRB, 0.16258E+01_JPRB, 0.17599E+01_JPRB ,&
&  0.19493E+01_JPRB, 0.22431E+01_JPRB, 0.30214E+01_JPRB, 0.41210E+01_JPRB, 0.50653E+01_JPRB, 0.58839E+01_JPRB, 0.73639E+01_JPRB ,&
&  0.67236E+01_JPRB, 0.69795E+01_JPRB, 0.69973E+01_JPRB, 0.83601E+01_JPRB, 0.85433E+01_JPRB, 0.95980E+01_JPRB, 0.10621E+02_JPRB ,&
&  0.75235E+01_JPRB, 0.60388E+01_JPRB, 0.59654E+01_JPRB, 0.65009E+01_JPRB, 0.51795E+01_JPRB, 0.55924E+01_JPRB, 0.74881E+01_JPRB ,&
&  0.58026E+01_JPRB, 0.57017E+01_JPRB, 0.57673E+01_JPRB, 0.61659E+01_JPRB, 0.35889E+01_JPRB, 0.23772E+01_JPRB, 0.36599E+01_JPRB ,&
&  0.35001E+01_JPRB, 0.19534E+01_JPRB, 0.13678E+01_JPRB, 0.17366E+01_JPRB, 0.13186E+01_JPRB, 0.16389E+01_JPRB, 0.26450E+01_JPRB ,&
&  0.31265E+01_JPRB, 0.36812E+01_JPRB, 0.55931E+01_JPRB, 0.50652E+01_JPRB, 0.47957E+01_JPRB, 0.44524E+01_JPRB, 0.41223E+01_JPRB ,&
&  0.40379E+01_JPRB, 0.34716E+01_JPRB, 0.35048E+01_JPRB, 0.30642E+01_JPRB, 0.27250E+01_JPRB, 0.24790E+01_JPRB, 0.21967E+01_JPRB ,&
&  0.19464E+01_JPRB, 0.17079E+01_JPRB, 0.15178E+01_JPRB, 0.14004E+01_JPRB, 0.13458E+01_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.13534E+01_JPRB, 0.12826E+01_JPRB, 0.12053E+01_JPRB, 0.11203E+01_JPRB, 0.10250E+01_JPRB, 0.97497E+00_JPRB, 0.94967E+00_JPRB ,&
&  0.93819E+00_JPRB, 0.94854E+00_JPRB, 0.96318E+00_JPRB, 0.97263E+00_JPRB, 0.97853E+00_JPRB, 0.97910E+00_JPRB, 0.92275E+00_JPRB ,&
&  0.87543E+00_JPRB, 0.84309E+00_JPRB, 0.73662E+00_JPRB, 0.73942E+00_JPRB, 0.10288E+01_JPRB, 0.13906E+01_JPRB, 0.14966E+01_JPRB ,&
&  0.21333E+01_JPRB, 0.29476E+01_JPRB, 0.38427E+01_JPRB, 0.54704E+01_JPRB, 0.70162E+01_JPRB, 0.79068E+01_JPRB, 0.73060E+01_JPRB ,&
&  0.71585E+01_JPRB, 0.64540E+01_JPRB, 0.53977E+01_JPRB, 0.45178E+01_JPRB, 0.36891E+01_JPRB, 0.30205E+01_JPRB, 0.25159E+01_JPRB ,&
&  0.21936E+01_JPRB, 0.19766E+01_JPRB, 0.18102E+01_JPRB, 0.17269E+01_JPRB, 0.17260E+01_JPRB, 0.16840E+01_JPRB, 0.17571E+01_JPRB ,&
&  0.18624E+01_JPRB, 0.20621E+01_JPRB, 0.29061E+01_JPRB, 0.44541E+01_JPRB, 0.56827E+01_JPRB, 0.62004E+01_JPRB, 0.62709E+01_JPRB ,&
&  0.63893E+01_JPRB, 0.65980E+01_JPRB, 0.72911E+01_JPRB, 0.77064E+01_JPRB, 0.87728E+01_JPRB, 0.96987E+01_JPRB, 0.11543E+02_JPRB ,&
&  0.89852E+01_JPRB, 0.72398E+01_JPRB, 0.59132E+01_JPRB, 0.56088E+01_JPRB, 0.63964E+01_JPRB, 0.80737E+01_JPRB, 0.75295E+01_JPRB ,&
&  0.67033E+01_JPRB, 0.62547E+01_JPRB, 0.57196E+01_JPRB, 0.50775E+01_JPRB, 0.53101E+01_JPRB, 0.32547E+01_JPRB, 0.35263E+01_JPRB ,&
&  0.35487E+01_JPRB, 0.34426E+01_JPRB, 0.30644E+01_JPRB, 0.25581E+01_JPRB, 0.22518E+01_JPRB, 0.24582E+01_JPRB, 0.26034E+01_JPRB ,&
&  0.29003E+01_JPRB, 0.31322E+01_JPRB, 0.46898E+01_JPRB, 0.50841E+01_JPRB, 0.52855E+01_JPRB, 0.45991E+01_JPRB, 0.44095E+01_JPRB ,&
&  0.38477E+01_JPRB, 0.37309E+01_JPRB, 0.36303E+01_JPRB, 0.32567E+01_JPRB, 0.28240E+01_JPRB, 0.24909E+01_JPRB, 0.21867E+01_JPRB ,&
&  0.19451E+01_JPRB, 0.17382E+01_JPRB, 0.15640E+01_JPRB, 0.14513E+01_JPRB, 0.14063E+01_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.14272E+01_JPRB, 0.13489E+01_JPRB, 0.12463E+01_JPRB, 0.11668E+01_JPRB, 0.10835E+01_JPRB, 0.10307E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.99664E+00_JPRB, 0.97645E+00_JPRB, 0.98207E+00_JPRB, 0.99769E+00_JPRB, 0.10022E+01_JPRB, 0.99788E+00_JPRB, 0.93586E+00_JPRB ,&
&  0.90138E+00_JPRB, 0.88284E+00_JPRB, 0.73934E+00_JPRB, 0.75406E+00_JPRB, 0.90338E+00_JPRB, 0.10814E+01_JPRB, 0.13868E+01_JPRB ,&
&  0.17300E+01_JPRB, 0.23460E+01_JPRB, 0.30667E+01_JPRB, 0.41881E+01_JPRB, 0.48772E+01_JPRB, 0.51511E+01_JPRB, 0.50691E+01_JPRB ,&
&  0.46536E+01_JPRB, 0.46619E+01_JPRB, 0.44196E+01_JPRB, 0.39638E+01_JPRB, 0.34846E+01_JPRB, 0.30956E+01_JPRB, 0.27567E+01_JPRB ,&
&  0.24669E+01_JPRB, 0.22360E+01_JPRB, 0.20663E+01_JPRB, 0.19420E+01_JPRB, 0.19037E+01_JPRB, 0.18804E+01_JPRB, 0.18967E+01_JPRB ,&
&  0.18478E+01_JPRB, 0.18419E+01_JPRB, 0.23041E+01_JPRB, 0.38030E+01_JPRB, 0.49299E+01_JPRB, 0.58407E+01_JPRB, 0.56774E+01_JPRB ,&
&  0.67537E+01_JPRB, 0.76631E+01_JPRB, 0.81829E+01_JPRB, 0.96428E+01_JPRB, 0.86315E+01_JPRB, 0.10339E+02_JPRB, 0.12589E+02_JPRB ,&
&  0.12030E+02_JPRB, 0.10690E+02_JPRB, 0.97684E+01_JPRB, 0.87172E+01_JPRB, 0.74198E+01_JPRB, 0.86616E+01_JPRB, 0.72106E+01_JPRB ,&
&  0.63426E+01_JPRB, 0.61415E+01_JPRB, 0.56081E+01_JPRB, 0.56263E+01_JPRB, 0.46356E+01_JPRB, 0.40779E+01_JPRB, 0.28087E+01_JPRB ,&
&  0.27914E+01_JPRB, 0.24381E+01_JPRB, 0.24531E+01_JPRB, 0.21614E+01_JPRB, 0.20903E+01_JPRB, 0.19259E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.20649E+01_JPRB, 0.22229E+01_JPRB, 0.28894E+01_JPRB, 0.33326E+01_JPRB, 0.44241E+01_JPRB, 0.40916E+01_JPRB, 0.41666E+01_JPRB ,&
&  0.37822E+01_JPRB, 0.35616E+01_JPRB, 0.30273E+01_JPRB, 0.28795E+01_JPRB, 0.25398E+01_JPRB, 0.22610E+01_JPRB, 0.20405E+01_JPRB ,&
&  0.18246E+01_JPRB, 0.16818E+01_JPRB, 0.15737E+01_JPRB, 0.14834E+01_JPRB, 0.14661E+01_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.12335E+01_JPRB, 0.11816E+01_JPRB, 0.11567E+01_JPRB, 0.11431E+01_JPRB, 0.10882E+01_JPRB, 0.10491E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10465E+01_JPRB, 0.10027E+01_JPRB, 0.97228E+00_JPRB, 0.96582E+00_JPRB, 0.96638E+00_JPRB, 0.97702E+00_JPRB, 0.95786E+00_JPRB ,&
&  0.93268E+00_JPRB, 0.10814E+01_JPRB, 0.86547E+00_JPRB, 0.72205E+00_JPRB, 0.68616E+00_JPRB, 0.84563E+00_JPRB, 0.10937E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.17781E+01_JPRB, 0.23056E+01_JPRB, 0.26872E+01_JPRB, 0.29919E+01_JPRB, 0.31493E+01_JPRB, 0.33200E+01_JPRB ,&
&  0.36046E+01_JPRB, 0.28476E+01_JPRB, 0.27623E+01_JPRB, 0.26672E+01_JPRB, 0.25856E+01_JPRB, 0.24693E+01_JPRB, 0.24064E+01_JPRB ,&
&  0.22982E+01_JPRB, 0.22121E+01_JPRB, 0.21328E+01_JPRB, 0.20450E+01_JPRB, 0.19615E+01_JPRB, 0.18901E+01_JPRB, 0.18382E+01_JPRB ,&
&  0.18430E+01_JPRB, 0.17982E+01_JPRB, 0.18556E+01_JPRB, 0.24710E+01_JPRB, 0.38275E+01_JPRB, 0.48986E+01_JPRB, 0.57124E+01_JPRB ,&
&  0.56859E+01_JPRB, 0.60640E+01_JPRB, 0.99804E+01_JPRB, 0.10766E+02_JPRB, 0.11531E+02_JPRB, 0.12938E+02_JPRB, 0.13405E+02_JPRB ,&
&  0.12815E+02_JPRB, 0.11465E+02_JPRB, 0.10071E+02_JPRB, 0.87023E+01_JPRB, 0.92991E+01_JPRB, 0.85435E+01_JPRB, 0.74473E+01_JPRB ,&
&  0.67449E+01_JPRB, 0.67473E+01_JPRB, 0.61624E+01_JPRB, 0.55022E+01_JPRB, 0.50245E+01_JPRB, 0.50378E+01_JPRB, 0.39702E+01_JPRB ,&
&  0.30448E+01_JPRB, 0.28174E+01_JPRB, 0.20176E+01_JPRB, 0.15659E+01_JPRB, 0.13656E+01_JPRB, 0.12839E+01_JPRB, 0.13212E+01_JPRB ,&
&  0.14222E+01_JPRB, 0.16374E+01_JPRB, 0.19787E+01_JPRB, 0.22900E+01_JPRB, 0.30302E+01_JPRB, 0.34649E+01_JPRB, 0.34406E+01_JPRB ,&
&  0.28452E+01_JPRB, 0.27804E+01_JPRB, 0.25962E+01_JPRB, 0.24061E+01_JPRB, 0.22253E+01_JPRB, 0.19849E+01_JPRB, 0.18589E+01_JPRB ,&
&  0.17115E+01_JPRB, 0.16114E+01_JPRB, 0.15387E+01_JPRB, 0.14091E+01_JPRB, 0.13192E+01_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.10883E+01_JPRB, 0.10737E+01_JPRB, 0.10890E+01_JPRB, 0.11021E+01_JPRB, 0.10952E+01_JPRB, 0.10631E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.10236E+01_JPRB, 0.98714E+00_JPRB, 0.95657E+00_JPRB, 0.94320E+00_JPRB, 0.93636E+00_JPRB, 0.93777E+00_JPRB, 0.95318E+00_JPRB ,&
&  0.97019E+00_JPRB, 0.13038E+01_JPRB, 0.10635E+01_JPRB, 0.92120E+00_JPRB, 0.79410E+00_JPRB, 0.89007E+00_JPRB, 0.10385E+01_JPRB ,&
&  0.12768E+01_JPRB, 0.13853E+01_JPRB, 0.15687E+01_JPRB, 0.18014E+01_JPRB, 0.20154E+01_JPRB, 0.19747E+01_JPRB, 0.21246E+01_JPRB ,&
&  0.20388E+01_JPRB, 0.19296E+01_JPRB, 0.16401E+01_JPRB, 0.17979E+01_JPRB, 0.17519E+01_JPRB, 0.15595E+01_JPRB, 0.18122E+01_JPRB ,&
&  0.18840E+01_JPRB, 0.19384E+01_JPRB, 0.19820E+01_JPRB, 0.20099E+01_JPRB, 0.20165E+01_JPRB, 0.19658E+01_JPRB, 0.18973E+01_JPRB ,&
&  0.18109E+01_JPRB, 0.17877E+01_JPRB, 0.18411E+01_JPRB, 0.19698E+01_JPRB, 0.24682E+01_JPRB, 0.31904E+01_JPRB, 0.45743E+01_JPRB ,&
&  0.60084E+01_JPRB, 0.58763E+01_JPRB, 0.61234E+01_JPRB, 0.84797E+01_JPRB, 0.11119E+02_JPRB, 0.10867E+02_JPRB, 0.10766E+02_JPRB ,&
&  0.11408E+02_JPRB, 0.11562E+02_JPRB, 0.11326E+02_JPRB, 0.10658E+02_JPRB, 0.99959E+01_JPRB, 0.92334E+01_JPRB, 0.83381E+01_JPRB ,&
&  0.75275E+01_JPRB, 0.72604E+01_JPRB, 0.70565E+01_JPRB, 0.62900E+01_JPRB, 0.50697E+01_JPRB, 0.44512E+01_JPRB, 0.39680E+01_JPRB ,&
&  0.34934E+01_JPRB, 0.21005E+01_JPRB, 0.11977E+01_JPRB, 0.91647E+00_JPRB, 0.82206E+00_JPRB, 0.81127E+00_JPRB, 0.96150E+00_JPRB ,&
&  0.96208E+00_JPRB, 0.11952E+01_JPRB, 0.15934E+01_JPRB, 0.17292E+01_JPRB, 0.24091E+01_JPRB, 0.27768E+01_JPRB, 0.25873E+01_JPRB ,&
&  0.21492E+01_JPRB, 0.18146E+01_JPRB, 0.22097E+01_JPRB, 0.21597E+01_JPRB, 0.19890E+01_JPRB, 0.18089E+01_JPRB, 0.17546E+01_JPRB ,&
&  0.16324E+01_JPRB, 0.15441E+01_JPRB, 0.14331E+01_JPRB, 0.12525E+01_JPRB, 0.11373E+01_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.10234E+01_JPRB, 0.10151E+01_JPRB, 0.10218E+01_JPRB, 0.10323E+01_JPRB, 0.10409E+01_JPRB, 0.10052E+01_JPRB, 0.99839E+00_JPRB ,&
&  0.98343E+00_JPRB, 0.95998E+00_JPRB, 0.93702E+00_JPRB, 0.92897E+00_JPRB, 0.92294E+00_JPRB, 0.92298E+00_JPRB, 0.95296E+00_JPRB ,&
&  0.10894E+01_JPRB, 0.14360E+01_JPRB, 0.12808E+01_JPRB, 0.10089E+01_JPRB, 0.11632E+01_JPRB, 0.11378E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.13077E+01_JPRB, 0.12734E+01_JPRB, 0.12303E+01_JPRB, 0.13019E+01_JPRB, 0.14071E+01_JPRB, 0.14382E+01_JPRB ,&
&  0.13617E+01_JPRB, 0.12261E+01_JPRB, 0.11472E+01_JPRB, 0.11499E+01_JPRB, 0.12633E+01_JPRB, 0.11622E+01_JPRB, 0.13115E+01_JPRB ,&
&  0.13956E+01_JPRB, 0.14775E+01_JPRB, 0.15652E+01_JPRB, 0.16695E+01_JPRB, 0.17539E+01_JPRB, 0.18300E+01_JPRB, 0.18652E+01_JPRB ,&
&  0.18798E+01_JPRB, 0.18742E+01_JPRB, 0.18439E+01_JPRB, 0.19319E+01_JPRB, 0.21359E+01_JPRB, 0.27222E+01_JPRB, 0.42809E+01_JPRB ,&
&  0.53292E+01_JPRB, 0.53785E+01_JPRB, 0.65752E+01_JPRB, 0.83912E+01_JPRB, 0.85823E+01_JPRB, 0.83419E+01_JPRB, 0.86701E+01_JPRB ,&
&  0.92786E+01_JPRB, 0.98869E+01_JPRB, 0.10561E+02_JPRB, 0.10818E+02_JPRB, 0.10744E+02_JPRB, 0.99247E+01_JPRB, 0.90160E+01_JPRB ,&
&  0.81113E+01_JPRB, 0.77617E+01_JPRB, 0.75196E+01_JPRB, 0.64366E+01_JPRB, 0.51730E+01_JPRB, 0.42055E+01_JPRB, 0.37265E+01_JPRB ,&
&  0.35578E+01_JPRB, 0.16296E+01_JPRB, 0.10294E+01_JPRB, 0.82243E+00_JPRB, 0.75107E+00_JPRB, 0.80535E+00_JPRB, 0.10569E+01_JPRB ,&
&  0.91381E+00_JPRB, 0.10856E+01_JPRB, 0.14204E+01_JPRB, 0.15154E+01_JPRB, 0.16655E+01_JPRB, 0.18866E+01_JPRB, 0.14805E+01_JPRB ,&
&  0.12772E+01_JPRB, 0.13865E+01_JPRB, 0.18078E+01_JPRB, 0.18821E+01_JPRB, 0.17283E+01_JPRB, 0.16707E+01_JPRB, 0.16632E+01_JPRB ,&
&  0.15379E+01_JPRB, 0.14534E+01_JPRB, 0.12905E+01_JPRB, 0.11339E+01_JPRB, 0.10634E+01_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.10340E+01_JPRB, 0.10031E+01_JPRB, 0.99110E+00_JPRB, 0.98775E+00_JPRB, 0.97993E+00_JPRB, 0.95215E+00_JPRB, 0.95318E+00_JPRB ,&
&  0.94308E+00_JPRB, 0.92114E+00_JPRB, 0.90974E+00_JPRB, 0.90263E+00_JPRB, 0.89052E+00_JPRB, 0.90687E+00_JPRB, 0.96725E+00_JPRB ,&
&  0.91154E+00_JPRB, 0.10837E+01_JPRB, 0.94881E+00_JPRB, 0.10073E+01_JPRB, 0.14264E+01_JPRB, 0.12734E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.11759E+01_JPRB, 0.11326E+01_JPRB, 0.10340E+01_JPRB, 0.99659E+00_JPRB, 0.10016E+01_JPRB, 0.10576E+01_JPRB, 0.10840E+01_JPRB ,&
&  0.98535E+00_JPRB, 0.87813E+00_JPRB, 0.84442E+00_JPRB, 0.84273E+00_JPRB, 0.89981E+00_JPRB, 0.99475E+00_JPRB, 0.11322E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.12170E+01_JPRB, 0.12879E+01_JPRB, 0.13737E+01_JPRB, 0.14580E+01_JPRB, 0.15482E+01_JPRB, 0.16000E+01_JPRB ,&
&  0.16276E+01_JPRB, 0.16616E+01_JPRB, 0.17102E+01_JPRB, 0.18368E+01_JPRB, 0.18674E+01_JPRB, 0.24930E+01_JPRB, 0.41320E+01_JPRB ,&
&  0.50972E+01_JPRB, 0.45624E+01_JPRB, 0.50351E+01_JPRB, 0.66651E+01_JPRB, 0.70809E+01_JPRB, 0.68691E+01_JPRB, 0.67045E+01_JPRB ,&
&  0.70791E+01_JPRB, 0.75529E+01_JPRB, 0.80923E+01_JPRB, 0.88550E+01_JPRB, 0.93694E+01_JPRB, 0.96938E+01_JPRB, 0.93233E+01_JPRB ,&
&  0.86617E+01_JPRB, 0.82215E+01_JPRB, 0.77740E+01_JPRB, 0.70256E+01_JPRB, 0.57057E+01_JPRB, 0.48399E+01_JPRB, 0.37237E+01_JPRB ,&
&  0.30337E+01_JPRB, 0.22177E+01_JPRB, 0.14107E+01_JPRB, 0.11139E+01_JPRB, 0.97193E+00_JPRB, 0.10899E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.11267E+01_JPRB, 0.11348E+01_JPRB, 0.11666E+01_JPRB, 0.11831E+01_JPRB, 0.11390E+01_JPRB, 0.97190E+00_JPRB ,&
&  0.97369E+00_JPRB, 0.11798E+01_JPRB, 0.15002E+01_JPRB, 0.16126E+01_JPRB, 0.15065E+01_JPRB, 0.14824E+01_JPRB, 0.13999E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.13326E+01_JPRB, 0.12182E+01_JPRB, 0.11290E+01_JPRB, 0.10901E+01_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.10031E+01_JPRB, 0.98623E+00_JPRB, 0.97055E+00_JPRB, 0.95085E+00_JPRB, 0.93144E+00_JPRB, 0.90956E+00_JPRB, 0.89120E+00_JPRB ,&
&  0.89772E+00_JPRB, 0.89120E+00_JPRB, 0.88725E+00_JPRB, 0.87886E+00_JPRB, 0.86353E+00_JPRB, 0.87974E+00_JPRB, 0.82267E+00_JPRB ,&
&  0.78518E+00_JPRB, 0.85594E+00_JPRB, 0.85792E+00_JPRB, 0.93989E+00_JPRB, 0.10404E+01_JPRB, 0.10017E+01_JPRB, 0.98410E+00_JPRB ,&
&  0.96425E+00_JPRB, 0.90738E+00_JPRB, 0.89286E+00_JPRB, 0.90397E+00_JPRB, 0.91159E+00_JPRB, 0.93597E+00_JPRB, 0.97853E+00_JPRB ,&
&  0.90448E+00_JPRB, 0.83239E+00_JPRB, 0.78830E+00_JPRB, 0.79376E+00_JPRB, 0.87050E+00_JPRB, 0.98939E+00_JPRB, 0.10599E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.10996E+01_JPRB, 0.11612E+01_JPRB, 0.12096E+01_JPRB, 0.12493E+01_JPRB, 0.12994E+01_JPRB, 0.13490E+01_JPRB ,&
&  0.13889E+01_JPRB, 0.14407E+01_JPRB, 0.15042E+01_JPRB, 0.16112E+01_JPRB, 0.16709E+01_JPRB, 0.20712E+01_JPRB, 0.27972E+01_JPRB ,&
&  0.31403E+01_JPRB, 0.32648E+01_JPRB, 0.34723E+01_JPRB, 0.39163E+01_JPRB, 0.45165E+01_JPRB, 0.45304E+01_JPRB, 0.46294E+01_JPRB ,&
&  0.51613E+01_JPRB, 0.54660E+01_JPRB, 0.68604E+01_JPRB, 0.76259E+01_JPRB, 0.81339E+01_JPRB, 0.87971E+01_JPRB, 0.87395E+01_JPRB ,&
&  0.85940E+01_JPRB, 0.81739E+01_JPRB, 0.79803E+01_JPRB, 0.71862E+01_JPRB, 0.60006E+01_JPRB, 0.52037E+01_JPRB, 0.39815E+01_JPRB ,&
&  0.31651E+01_JPRB, 0.24534E+01_JPRB, 0.17752E+01_JPRB, 0.14922E+01_JPRB, 0.12197E+01_JPRB, 0.11801E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.99573E+00_JPRB, 0.95739E+00_JPRB, 0.92203E+00_JPRB, 0.94632E+00_JPRB, 0.92495E+00_JPRB, 0.84437E+00_JPRB, 0.88644E+00_JPRB ,&
&  0.97379E+00_JPRB, 0.12638E+01_JPRB, 0.13293E+01_JPRB, 0.14146E+01_JPRB, 0.13576E+01_JPRB, 0.13338E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11240E+01_JPRB, 0.12401E+01_JPRB, 0.11535E+01_JPRB, 0.10839E+01_JPRB, 0.10350E+01_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.96278E+00_JPRB, 0.95896E+00_JPRB, 0.93470E+00_JPRB, 0.91943E+00_JPRB, 0.90914E+00_JPRB, 0.86454E+00_JPRB, 0.82524E+00_JPRB ,&
&  0.88473E+00_JPRB, 0.87615E+00_JPRB, 0.87762E+00_JPRB, 0.86537E+00_JPRB, 0.85229E+00_JPRB, 0.78376E+00_JPRB, 0.73863E+00_JPRB ,&
&  0.70361E+00_JPRB, 0.71410E+00_JPRB, 0.77369E+00_JPRB, 0.80035E+00_JPRB, 0.80618E+00_JPRB, 0.78305E+00_JPRB, 0.77326E+00_JPRB ,&
&  0.76388E+00_JPRB, 0.77318E+00_JPRB, 0.82565E+00_JPRB, 0.87749E+00_JPRB, 0.88268E+00_JPRB, 0.88779E+00_JPRB, 0.92378E+00_JPRB ,&
&  0.87663E+00_JPRB, 0.87071E+00_JPRB, 0.86357E+00_JPRB, 0.79673E+00_JPRB, 0.90766E+00_JPRB, 0.95007E+00_JPRB, 0.97532E+00_JPRB ,&
&  0.10183E+01_JPRB, 0.10576E+01_JPRB, 0.10859E+01_JPRB, 0.11058E+01_JPRB, 0.11435E+01_JPRB, 0.11686E+01_JPRB, 0.12043E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.12817E+01_JPRB, 0.13353E+01_JPRB, 0.14099E+01_JPRB, 0.15579E+01_JPRB, 0.17209E+01_JPRB, 0.20509E+01_JPRB ,&
&  0.22841E+01_JPRB, 0.24481E+01_JPRB, 0.27907E+01_JPRB, 0.27198E+01_JPRB, 0.35936E+01_JPRB, 0.41146E+01_JPRB, 0.39734E+01_JPRB ,&
&  0.43333E+01_JPRB, 0.43794E+01_JPRB, 0.52718E+01_JPRB, 0.61322E+01_JPRB, 0.69207E+01_JPRB, 0.76199E+01_JPRB, 0.79169E+01_JPRB ,&
&  0.77861E+01_JPRB, 0.79270E+01_JPRB, 0.77706E+01_JPRB, 0.69333E+01_JPRB, 0.58264E+01_JPRB, 0.50453E+01_JPRB, 0.39907E+01_JPRB ,&
&  0.31180E+01_JPRB, 0.24280E+01_JPRB, 0.20049E+01_JPRB, 0.17879E+01_JPRB, 0.14149E+01_JPRB, 0.12898E+01_JPRB, 0.11666E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.96402E+00_JPRB, 0.91560E+00_JPRB, 0.87436E+00_JPRB, 0.84811E+00_JPRB, 0.83829E+00_JPRB, 0.82619E+00_JPRB ,&
&  0.88105E+00_JPRB, 0.10101E+01_JPRB, 0.12460E+01_JPRB, 0.12589E+01_JPRB, 0.12233E+01_JPRB, 0.11959E+01_JPRB, 0.10789E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.11404E+01_JPRB, 0.10998E+01_JPRB, 0.10262E+01_JPRB, 0.96585E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.91669E+00_JPRB, 0.91930E+00_JPRB, 0.90713E+00_JPRB, 0.89554E+00_JPRB, 0.87132E+00_JPRB, 0.79227E+00_JPRB, 0.76705E+00_JPRB ,&
&  0.83496E+00_JPRB, 0.77922E+00_JPRB, 0.83035E+00_JPRB, 0.72354E+00_JPRB, 0.69622E+00_JPRB, 0.66121E+00_JPRB, 0.63652E+00_JPRB ,&
&  0.60676E+00_JPRB, 0.61395E+00_JPRB, 0.64871E+00_JPRB, 0.68009E+00_JPRB, 0.68950E+00_JPRB, 0.67644E+00_JPRB, 0.68439E+00_JPRB ,&
&  0.70612E+00_JPRB, 0.74855E+00_JPRB, 0.81119E+00_JPRB, 0.84676E+00_JPRB, 0.86815E+00_JPRB, 0.88729E+00_JPRB, 0.89421E+00_JPRB ,&
&  0.83471E+00_JPRB, 0.81387E+00_JPRB, 0.79847E+00_JPRB, 0.80191E+00_JPRB, 0.83770E+00_JPRB, 0.89909E+00_JPRB, 0.93387E+00_JPRB ,&
&  0.94031E+00_JPRB, 0.81877E+00_JPRB, 0.94215E+00_JPRB, 0.98496E+00_JPRB, 0.10579E+01_JPRB, 0.11010E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.11701E+01_JPRB, 0.12133E+01_JPRB, 0.12619E+01_JPRB, 0.13154E+01_JPRB, 0.13821E+01_JPRB, 0.14987E+01_JPRB, 0.15904E+01_JPRB ,&
&  0.18468E+01_JPRB, 0.14969E+01_JPRB, 0.17324E+01_JPRB, 0.20107E+01_JPRB, 0.27952E+01_JPRB, 0.32432E+01_JPRB, 0.34422E+01_JPRB ,&
&  0.40955E+01_JPRB, 0.40570E+01_JPRB, 0.44441E+01_JPRB, 0.48408E+01_JPRB, 0.53769E+01_JPRB, 0.60299E+01_JPRB, 0.64600E+01_JPRB ,&
&  0.65840E+01_JPRB, 0.66454E+01_JPRB, 0.70240E+01_JPRB, 0.64092E+01_JPRB, 0.55741E+01_JPRB, 0.47496E+01_JPRB, 0.37590E+01_JPRB ,&
&  0.30991E+01_JPRB, 0.25171E+01_JPRB, 0.20246E+01_JPRB, 0.16693E+01_JPRB, 0.14176E+01_JPRB, 0.12599E+01_JPRB, 0.11710E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.10690E+01_JPRB, 0.10073E+01_JPRB, 0.93793E+00_JPRB, 0.89616E+00_JPRB, 0.87325E+00_JPRB, 0.88114E+00_JPRB ,&
&  0.86151E+00_JPRB, 0.85376E+00_JPRB, 0.88815E+00_JPRB, 0.98031E+00_JPRB, 0.99047E+00_JPRB, 0.10073E+01_JPRB, 0.11081E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.10173E+01_JPRB, 0.10652E+01_JPRB, 0.10109E+01_JPRB, 0.93984E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.93490E+00_JPRB, 0.90040E+00_JPRB, 0.87976E+00_JPRB, 0.86136E+00_JPRB, 0.83237E+00_JPRB, 0.77178E+00_JPRB, 0.73096E+00_JPRB ,&
&  0.73699E+00_JPRB, 0.71751E+00_JPRB, 0.70178E+00_JPRB, 0.66643E+00_JPRB, 0.63459E+00_JPRB, 0.60087E+00_JPRB, 0.58346E+00_JPRB ,&
&  0.56992E+00_JPRB, 0.59308E+00_JPRB, 0.63652E+00_JPRB, 0.64882E+00_JPRB, 0.65059E+00_JPRB, 0.65491E+00_JPRB, 0.68133E+00_JPRB ,&
&  0.71629E+00_JPRB, 0.75216E+00_JPRB, 0.77541E+00_JPRB, 0.79976E+00_JPRB, 0.81539E+00_JPRB, 0.82376E+00_JPRB, 0.80286E+00_JPRB ,&
&  0.77644E+00_JPRB, 0.73660E+00_JPRB, 0.70363E+00_JPRB, 0.72482E+00_JPRB, 0.78476E+00_JPRB, 0.83938E+00_JPRB, 0.79905E+00_JPRB ,&
&  0.62523E+00_JPRB, 0.63089E+00_JPRB, 0.81728E+00_JPRB, 0.89222E+00_JPRB, 0.95921E+00_JPRB, 0.99836E+00_JPRB, 0.10482E+01_JPRB ,&
&  0.10948E+01_JPRB, 0.11435E+01_JPRB, 0.11834E+01_JPRB, 0.12188E+01_JPRB, 0.12634E+01_JPRB, 0.13290E+01_JPRB, 0.13774E+01_JPRB ,&
&  0.15323E+01_JPRB, 0.13133E+01_JPRB, 0.13303E+01_JPRB, 0.15773E+01_JPRB, 0.22746E+01_JPRB, 0.26862E+01_JPRB, 0.27929E+01_JPRB ,&
&  0.30443E+01_JPRB, 0.31766E+01_JPRB, 0.34749E+01_JPRB, 0.38897E+01_JPRB, 0.41312E+01_JPRB, 0.44959E+01_JPRB, 0.49876E+01_JPRB ,&
&  0.56299E+01_JPRB, 0.62144E+01_JPRB, 0.62512E+01_JPRB, 0.60102E+01_JPRB, 0.55513E+01_JPRB, 0.50123E+01_JPRB, 0.41876E+01_JPRB ,&
&  0.32315E+01_JPRB, 0.25579E+01_JPRB, 0.20044E+01_JPRB, 0.16321E+01_JPRB, 0.14217E+01_JPRB, 0.13233E+01_JPRB, 0.12473E+01_JPRB ,&
&  0.12357E+01_JPRB, 0.12079E+01_JPRB, 0.11546E+01_JPRB, 0.10522E+01_JPRB, 0.97270E+00_JPRB, 0.93872E+00_JPRB, 0.91575E+00_JPRB ,&
&  0.87937E+00_JPRB, 0.84947E+00_JPRB, 0.77330E+00_JPRB, 0.77807E+00_JPRB, 0.85271E+00_JPRB, 0.93798E+00_JPRB, 0.96796E+00_JPRB ,&
&  0.98615E+00_JPRB, 0.95745E+00_JPRB, 0.95384E+00_JPRB, 0.97592E+00_JPRB, 0.96024E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.94820E+00_JPRB, 0.87212E+00_JPRB, 0.82719E+00_JPRB, 0.81948E+00_JPRB, 0.76497E+00_JPRB, 0.71804E+00_JPRB, 0.68895E+00_JPRB ,&
&  0.68717E+00_JPRB, 0.68758E+00_JPRB, 0.68876E+00_JPRB, 0.66162E+00_JPRB, 0.63694E+00_JPRB, 0.62373E+00_JPRB, 0.61978E+00_JPRB ,&
&  0.63472E+00_JPRB, 0.65337E+00_JPRB, 0.67012E+00_JPRB, 0.66982E+00_JPRB, 0.67074E+00_JPRB, 0.68199E+00_JPRB, 0.70792E+00_JPRB ,&
&  0.74327E+00_JPRB, 0.76190E+00_JPRB, 0.75879E+00_JPRB, 0.75403E+00_JPRB, 0.76602E+00_JPRB, 0.77367E+00_JPRB, 0.72916E+00_JPRB ,&
&  0.68606E+00_JPRB, 0.64927E+00_JPRB, 0.63654E+00_JPRB, 0.67431E+00_JPRB, 0.74227E+00_JPRB, 0.76229E+00_JPRB, 0.63517E+00_JPRB ,&
&  0.54434E+00_JPRB, 0.52526E+00_JPRB, 0.57380E+00_JPRB, 0.71610E+00_JPRB, 0.78040E+00_JPRB, 0.83773E+00_JPRB, 0.90879E+00_JPRB ,&
&  0.86011E+00_JPRB, 0.86553E+00_JPRB, 0.90148E+00_JPRB, 0.10623E+01_JPRB, 0.11283E+01_JPRB, 0.11816E+01_JPRB, 0.12340E+01_JPRB ,&
&  0.12855E+01_JPRB, 0.14083E+01_JPRB, 0.16459E+01_JPRB, 0.15337E+01_JPRB, 0.17991E+01_JPRB, 0.22606E+01_JPRB, 0.23776E+01_JPRB ,&
&  0.27322E+01_JPRB, 0.29705E+01_JPRB, 0.35056E+01_JPRB, 0.36014E+01_JPRB, 0.35110E+01_JPRB, 0.36167E+01_JPRB, 0.40555E+01_JPRB ,&
&  0.49153E+01_JPRB, 0.55628E+01_JPRB, 0.53811E+01_JPRB, 0.50809E+01_JPRB, 0.48363E+01_JPRB, 0.45321E+01_JPRB, 0.39437E+01_JPRB ,&
&  0.31460E+01_JPRB, 0.25425E+01_JPRB, 0.20894E+01_JPRB, 0.17168E+01_JPRB, 0.15457E+01_JPRB, 0.14774E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.13607E+01_JPRB, 0.13246E+01_JPRB, 0.12848E+01_JPRB, 0.11612E+01_JPRB, 0.10448E+01_JPRB, 0.92932E+00_JPRB, 0.84528E+00_JPRB ,&
&  0.79846E+00_JPRB, 0.75247E+00_JPRB, 0.72622E+00_JPRB, 0.74548E+00_JPRB, 0.82627E+00_JPRB, 0.92242E+00_JPRB, 0.96802E+00_JPRB ,&
&  0.94978E+00_JPRB, 0.91534E+00_JPRB, 0.93995E+00_JPRB, 0.98935E+00_JPRB, 0.10008E+01_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.10423E+01_JPRB, 0.10145E+01_JPRB, 0.97043E+00_JPRB, 0.91584E+00_JPRB, 0.86452E+00_JPRB, 0.80839E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.74326E+00_JPRB, 0.73109E+00_JPRB, 0.72195E+00_JPRB, 0.71787E+00_JPRB, 0.71746E+00_JPRB, 0.72253E+00_JPRB, 0.72270E+00_JPRB ,&
&  0.72601E+00_JPRB, 0.73101E+00_JPRB, 0.73925E+00_JPRB, 0.75359E+00_JPRB, 0.76766E+00_JPRB, 0.78273E+00_JPRB, 0.79873E+00_JPRB ,&
&  0.81338E+00_JPRB, 0.80267E+00_JPRB, 0.76943E+00_JPRB, 0.74428E+00_JPRB, 0.73661E+00_JPRB, 0.73640E+00_JPRB, 0.70174E+00_JPRB ,&
&  0.64926E+00_JPRB, 0.61933E+00_JPRB, 0.60782E+00_JPRB, 0.66212E+00_JPRB, 0.69875E+00_JPRB, 0.70649E+00_JPRB, 0.58069E+00_JPRB ,&
&  0.50985E+00_JPRB, 0.47867E+00_JPRB, 0.48100E+00_JPRB, 0.51827E+00_JPRB, 0.59140E+00_JPRB, 0.68102E+00_JPRB, 0.76880E+00_JPRB ,&
&  0.81888E+00_JPRB, 0.87014E+00_JPRB, 0.88951E+00_JPRB, 0.92185E+00_JPRB, 0.97750E+00_JPRB, 0.10381E+01_JPRB, 0.11092E+01_JPRB ,&
&  0.11723E+01_JPRB, 0.12644E+01_JPRB, 0.14421E+01_JPRB, 0.13993E+01_JPRB, 0.14856E+01_JPRB, 0.16531E+01_JPRB, 0.19435E+01_JPRB ,&
&  0.25527E+01_JPRB, 0.28837E+01_JPRB, 0.31143E+01_JPRB, 0.34920E+01_JPRB, 0.34924E+01_JPRB, 0.35514E+01_JPRB, 0.38285E+01_JPRB ,&
&  0.45283E+01_JPRB, 0.50994E+01_JPRB, 0.49569E+01_JPRB, 0.44412E+01_JPRB, 0.42352E+01_JPRB, 0.41088E+01_JPRB, 0.35856E+01_JPRB ,&
&  0.29261E+01_JPRB, 0.25410E+01_JPRB, 0.22024E+01_JPRB, 0.18610E+01_JPRB, 0.17199E+01_JPRB, 0.16386E+01_JPRB, 0.15792E+01_JPRB ,&
&  0.15280E+01_JPRB, 0.14736E+01_JPRB, 0.14286E+01_JPRB, 0.13067E+01_JPRB, 0.11928E+01_JPRB, 0.10636E+01_JPRB, 0.96999E+00_JPRB ,&
&  0.90704E+00_JPRB, 0.88000E+00_JPRB, 0.88031E+00_JPRB, 0.89974E+00_JPRB, 0.95370E+00_JPRB, 0.99349E+00_JPRB, 0.10112E+01_JPRB ,&
&  0.10095E+01_JPRB, 0.97933E+00_JPRB, 0.96859E+00_JPRB, 0.98042E+00_JPRB, 0.10276E+01_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.11252E+01_JPRB, 0.11205E+01_JPRB, 0.10894E+01_JPRB, 0.10454E+01_JPRB, 0.99627E+00_JPRB, 0.94139E+00_JPRB, 0.90466E+00_JPRB ,&
&  0.88407E+00_JPRB, 0.87660E+00_JPRB, 0.86595E+00_JPRB, 0.85525E+00_JPRB, 0.84872E+00_JPRB, 0.84529E+00_JPRB, 0.84521E+00_JPRB ,&
&  0.84852E+00_JPRB, 0.85286E+00_JPRB, 0.85574E+00_JPRB, 0.84550E+00_JPRB, 0.84842E+00_JPRB, 0.85498E+00_JPRB, 0.86377E+00_JPRB ,&
&  0.85863E+00_JPRB, 0.82466E+00_JPRB, 0.78140E+00_JPRB, 0.75058E+00_JPRB, 0.71403E+00_JPRB, 0.68884E+00_JPRB, 0.66238E+00_JPRB ,&
&  0.63075E+00_JPRB, 0.61567E+00_JPRB, 0.62877E+00_JPRB, 0.65410E+00_JPRB, 0.64164E+00_JPRB, 0.65238E+00_JPRB, 0.52720E+00_JPRB ,&
&  0.48319E+00_JPRB, 0.45762E+00_JPRB, 0.45241E+00_JPRB, 0.46745E+00_JPRB, 0.50962E+00_JPRB, 0.56395E+00_JPRB, 0.62481E+00_JPRB ,&
&  0.69959E+00_JPRB, 0.79537E+00_JPRB, 0.85559E+00_JPRB, 0.90109E+00_JPRB, 0.95235E+00_JPRB, 0.10020E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.10790E+01_JPRB, 0.11497E+01_JPRB, 0.12506E+01_JPRB, 0.13932E+01_JPRB, 0.16570E+01_JPRB, 0.17230E+01_JPRB, 0.19054E+01_JPRB ,&
&  0.27106E+01_JPRB, 0.31646E+01_JPRB, 0.31848E+01_JPRB, 0.32142E+01_JPRB, 0.33145E+01_JPRB, 0.35628E+01_JPRB, 0.39358E+01_JPRB ,&
&  0.44247E+01_JPRB, 0.47972E+01_JPRB, 0.47633E+01_JPRB, 0.42464E+01_JPRB, 0.39860E+01_JPRB, 0.38335E+01_JPRB, 0.34102E+01_JPRB ,&
&  0.29283E+01_JPRB, 0.29106E+01_JPRB, 0.29565E+01_JPRB, 0.24427E+01_JPRB, 0.21111E+01_JPRB, 0.19095E+01_JPRB, 0.17945E+01_JPRB ,&
&  0.17017E+01_JPRB, 0.16189E+01_JPRB, 0.15343E+01_JPRB, 0.14605E+01_JPRB, 0.13836E+01_JPRB, 0.13077E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.11854E+01_JPRB, 0.11388E+01_JPRB, 0.11080E+01_JPRB, 0.10843E+01_JPRB, 0.10567E+01_JPRB, 0.10327E+01_JPRB, 0.10238E+01_JPRB ,&
&  0.10257E+01_JPRB, 0.10225E+01_JPRB, 0.10364E+01_JPRB, 0.10614E+01_JPRB, 0.10975E+01_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.11489E+01_JPRB, 0.11511E+01_JPRB, 0.11441E+01_JPRB, 0.11267E+01_JPRB, 0.10985E+01_JPRB, 0.10693E+01_JPRB, 0.10440E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10041E+01_JPRB, 0.98419E+00_JPRB, 0.96805E+00_JPRB, 0.95232E+00_JPRB, 0.94045E+00_JPRB, 0.93104E+00_JPRB ,&
&  0.92539E+00_JPRB, 0.89314E+00_JPRB, 0.88336E+00_JPRB, 0.88433E+00_JPRB, 0.88778E+00_JPRB, 0.88541E+00_JPRB, 0.87985E+00_JPRB ,&
&  0.85064E+00_JPRB, 0.80884E+00_JPRB, 0.76981E+00_JPRB, 0.73762E+00_JPRB, 0.70812E+00_JPRB, 0.68530E+00_JPRB, 0.67951E+00_JPRB ,&
&  0.67400E+00_JPRB, 0.65712E+00_JPRB, 0.64986E+00_JPRB, 0.66250E+00_JPRB, 0.65775E+00_JPRB, 0.58447E+00_JPRB, 0.49086E+00_JPRB ,&
&  0.47553E+00_JPRB, 0.45491E+00_JPRB, 0.45578E+00_JPRB, 0.46337E+00_JPRB, 0.48518E+00_JPRB, 0.52463E+00_JPRB, 0.58797E+00_JPRB ,&
&  0.66473E+00_JPRB, 0.82268E+00_JPRB, 0.92186E+00_JPRB, 0.99038E+00_JPRB, 0.10504E+01_JPRB, 0.10945E+01_JPRB, 0.11127E+01_JPRB ,&
&  0.11350E+01_JPRB, 0.11799E+01_JPRB, 0.12503E+01_JPRB, 0.13409E+01_JPRB, 0.14789E+01_JPRB, 0.17374E+01_JPRB, 0.20167E+01_JPRB ,&
&  0.23977E+01_JPRB, 0.27174E+01_JPRB, 0.28891E+01_JPRB, 0.30003E+01_JPRB, 0.32316E+01_JPRB, 0.35234E+01_JPRB, 0.36806E+01_JPRB ,&
&  0.39176E+01_JPRB, 0.41665E+01_JPRB, 0.41264E+01_JPRB, 0.38946E+01_JPRB, 0.37475E+01_JPRB, 0.36536E+01_JPRB, 0.33442E+01_JPRB ,&
&  0.29878E+01_JPRB, 0.28256E+01_JPRB, 0.27896E+01_JPRB, 0.24781E+01_JPRB, 0.22457E+01_JPRB, 0.20230E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.17586E+01_JPRB, 0.16671E+01_JPRB, 0.15546E+01_JPRB, 0.14654E+01_JPRB, 0.14107E+01_JPRB, 0.13671E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.12795E+01_JPRB, 0.12332E+01_JPRB, 0.11861E+01_JPRB, 0.11463E+01_JPRB, 0.11054E+01_JPRB, 0.10770E+01_JPRB, 0.10607E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10606E+01_JPRB, 0.10772E+01_JPRB, 0.11050E+01_JPRB, 0.11353E+01_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.11810E+01_JPRB, 0.11792E+01_JPRB, 0.11679E+01_JPRB, 0.11470E+01_JPRB, 0.11172E+01_JPRB, 0.10848E+01_JPRB, 0.10568E+01_JPRB ,&
&  0.10319E+01_JPRB, 0.10114E+01_JPRB, 0.99065E+00_JPRB, 0.97305E+00_JPRB, 0.95306E+00_JPRB, 0.93923E+00_JPRB, 0.92367E+00_JPRB ,&
&  0.91328E+00_JPRB, 0.90755E+00_JPRB, 0.89520E+00_JPRB, 0.88038E+00_JPRB, 0.85749E+00_JPRB, 0.83802E+00_JPRB, 0.82281E+00_JPRB ,&
&  0.80753E+00_JPRB, 0.79458E+00_JPRB, 0.76359E+00_JPRB, 0.74318E+00_JPRB, 0.73045E+00_JPRB, 0.71013E+00_JPRB, 0.71201E+00_JPRB ,&
&  0.69667E+00_JPRB, 0.69417E+00_JPRB, 0.69343E+00_JPRB, 0.68298E+00_JPRB, 0.66868E+00_JPRB, 0.53231E+00_JPRB, 0.50114E+00_JPRB ,&
&  0.49201E+00_JPRB, 0.48364E+00_JPRB, 0.48294E+00_JPRB, 0.49224E+00_JPRB, 0.50975E+00_JPRB, 0.55756E+00_JPRB, 0.60298E+00_JPRB ,&
&  0.63797E+00_JPRB, 0.86202E+00_JPRB, 0.95428E+00_JPRB, 0.10395E+01_JPRB, 0.11110E+01_JPRB, 0.11594E+01_JPRB, 0.11881E+01_JPRB ,&
&  0.12281E+01_JPRB, 0.12785E+01_JPRB, 0.13256E+01_JPRB, 0.14129E+01_JPRB, 0.15612E+01_JPRB, 0.17754E+01_JPRB, 0.20130E+01_JPRB ,&
&  0.22702E+01_JPRB, 0.25058E+01_JPRB, 0.27428E+01_JPRB, 0.29528E+01_JPRB, 0.31488E+01_JPRB, 0.32801E+01_JPRB, 0.33494E+01_JPRB ,&
&  0.34291E+01_JPRB, 0.34167E+01_JPRB, 0.34803E+01_JPRB, 0.34270E+01_JPRB, 0.33677E+01_JPRB, 0.32827E+01_JPRB, 0.30629E+01_JPRB ,&
&  0.27983E+01_JPRB, 0.26169E+01_JPRB, 0.24389E+01_JPRB, 0.22242E+01_JPRB, 0.20555E+01_JPRB, 0.19308E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.18033E+01_JPRB, 0.17527E+01_JPRB, 0.16564E+01_JPRB, 0.15544E+01_JPRB, 0.14843E+01_JPRB, 0.14300E+01_JPRB, 0.13659E+01_JPRB ,&
&  0.13045E+01_JPRB, 0.12463E+01_JPRB, 0.12021E+01_JPRB, 0.11628E+01_JPRB, 0.11305E+01_JPRB, 0.11096E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.10993E+01_JPRB, 0.11156E+01_JPRB, 0.11426E+01_JPRB, 0.11723E+01_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.11910E+01_JPRB, 0.11754E+01_JPRB, 0.11402E+01_JPRB, 0.10929E+01_JPRB, 0.10522E+01_JPRB, 0.10179E+01_JPRB, 0.99881E+00_JPRB ,&
&  0.98586E+00_JPRB, 0.97977E+00_JPRB, 0.97088E+00_JPRB, 0.96227E+00_JPRB, 0.94806E+00_JPRB, 0.93340E+00_JPRB, 0.91602E+00_JPRB ,&
&  0.89340E+00_JPRB, 0.86912E+00_JPRB, 0.84770E+00_JPRB, 0.83287E+00_JPRB, 0.82081E+00_JPRB, 0.81269E+00_JPRB, 0.80438E+00_JPRB ,&
&  0.79617E+00_JPRB, 0.78613E+00_JPRB, 0.75433E+00_JPRB, 0.74337E+00_JPRB, 0.72960E+00_JPRB, 0.72781E+00_JPRB, 0.71652E+00_JPRB ,&
&  0.72423E+00_JPRB, 0.63362E+00_JPRB, 0.62790E+00_JPRB, 0.60613E+00_JPRB, 0.58687E+00_JPRB, 0.56531E+00_JPRB, 0.54710E+00_JPRB ,&
&  0.53915E+00_JPRB, 0.52860E+00_JPRB, 0.52477E+00_JPRB, 0.53684E+00_JPRB, 0.55207E+00_JPRB, 0.58407E+00_JPRB, 0.62002E+00_JPRB ,&
&  0.67858E+00_JPRB, 0.88950E+00_JPRB, 0.99895E+00_JPRB, 0.10866E+01_JPRB, 0.11800E+01_JPRB, 0.12595E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.14582E+01_JPRB, 0.15452E+01_JPRB, 0.16112E+01_JPRB, 0.18134E+01_JPRB, 0.19653E+01_JPRB, 0.21464E+01_JPRB ,&
&  0.23224E+01_JPRB, 0.25106E+01_JPRB, 0.26840E+01_JPRB, 0.28238E+01_JPRB, 0.29161E+01_JPRB, 0.29430E+01_JPRB, 0.29106E+01_JPRB ,&
&  0.29009E+01_JPRB, 0.28803E+01_JPRB, 0.28840E+01_JPRB, 0.28019E+01_JPRB, 0.27630E+01_JPRB, 0.27232E+01_JPRB, 0.26166E+01_JPRB ,&
&  0.24839E+01_JPRB, 0.23628E+01_JPRB, 0.22358E+01_JPRB, 0.20974E+01_JPRB, 0.19620E+01_JPRB, 0.18755E+01_JPRB, 0.18189E+01_JPRB ,&
&  0.17847E+01_JPRB, 0.17327E+01_JPRB, 0.16671E+01_JPRB, 0.15769E+01_JPRB, 0.14912E+01_JPRB, 0.14278E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13076E+01_JPRB, 0.12507E+01_JPRB, 0.12108E+01_JPRB, 0.11789E+01_JPRB, 0.11593E+01_JPRB, 0.11469E+01_JPRB, 0.11423E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.11409E+01_JPRB, 0.11522E+01_JPRB, 0.11695E+01_JPRB, 0.11848E+01_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.11200E+01_JPRB, 0.10929E+01_JPRB, 0.10565E+01_JPRB, 0.10325E+01_JPRB, 0.10191E+01_JPRB, 0.10034E+01_JPRB, 0.98929E+00_JPRB ,&
&  0.97826E+00_JPRB, 0.97675E+00_JPRB, 0.97772E+00_JPRB, 0.97554E+00_JPRB, 0.96869E+00_JPRB, 0.95855E+00_JPRB, 0.94445E+00_JPRB ,&
&  0.92692E+00_JPRB, 0.90694E+00_JPRB, 0.88923E+00_JPRB, 0.87243E+00_JPRB, 0.85470E+00_JPRB, 0.83816E+00_JPRB, 0.81387E+00_JPRB ,&
&  0.79310E+00_JPRB, 0.77420E+00_JPRB, 0.71551E+00_JPRB, 0.71255E+00_JPRB, 0.70043E+00_JPRB, 0.69703E+00_JPRB, 0.69968E+00_JPRB ,&
&  0.68978E+00_JPRB, 0.67000E+00_JPRB, 0.65051E+00_JPRB, 0.62850E+00_JPRB, 0.61164E+00_JPRB, 0.59192E+00_JPRB, 0.58407E+00_JPRB ,&
&  0.58181E+00_JPRB, 0.58828E+00_JPRB, 0.60126E+00_JPRB, 0.62035E+00_JPRB, 0.64249E+00_JPRB, 0.66545E+00_JPRB, 0.70074E+00_JPRB ,&
&  0.78238E+00_JPRB, 0.89073E+00_JPRB, 0.10316E+01_JPRB, 0.11168E+01_JPRB, 0.12189E+01_JPRB, 0.13158E+01_JPRB, 0.13958E+01_JPRB ,&
&  0.14855E+01_JPRB, 0.15821E+01_JPRB, 0.16884E+01_JPRB, 0.17972E+01_JPRB, 0.19157E+01_JPRB, 0.20260E+01_JPRB, 0.21495E+01_JPRB ,&
&  0.22565E+01_JPRB, 0.23538E+01_JPRB, 0.24171E+01_JPRB, 0.24504E+01_JPRB, 0.24618E+01_JPRB, 0.24438E+01_JPRB, 0.24227E+01_JPRB ,&
&  0.24107E+01_JPRB, 0.23928E+01_JPRB, 0.23696E+01_JPRB, 0.23044E+01_JPRB, 0.22836E+01_JPRB, 0.22808E+01_JPRB, 0.22478E+01_JPRB ,&
&  0.21925E+01_JPRB, 0.21275E+01_JPRB, 0.20600E+01_JPRB, 0.19663E+01_JPRB, 0.18767E+01_JPRB, 0.18399E+01_JPRB, 0.18015E+01_JPRB ,&
&  0.17666E+01_JPRB, 0.17253E+01_JPRB, 0.16854E+01_JPRB, 0.16297E+01_JPRB, 0.15538E+01_JPRB, 0.14806E+01_JPRB, 0.14180E+01_JPRB ,&
&  0.13692E+01_JPRB, 0.13216E+01_JPRB, 0.12811E+01_JPRB, 0.12393E+01_JPRB, 0.12113E+01_JPRB, 0.11854E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11602E+01_JPRB, 0.11563E+01_JPRB, 0.11523E+01_JPRB, 0.11394E+01_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.11035E+01_JPRB, 0.10740E+01_JPRB, 0.10559E+01_JPRB, 0.10472E+01_JPRB, 0.10385E+01_JPRB, 0.10301E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10094E+01_JPRB, 0.99709E+00_JPRB, 0.99410E+00_JPRB, 0.99379E+00_JPRB, 0.99035E+00_JPRB, 0.98546E+00_JPRB, 0.97821E+00_JPRB ,&
&  0.96956E+00_JPRB, 0.95798E+00_JPRB, 0.94503E+00_JPRB, 0.92883E+00_JPRB, 0.91175E+00_JPRB, 0.89290E+00_JPRB, 0.86871E+00_JPRB ,&
&  0.84969E+00_JPRB, 0.83252E+00_JPRB, 0.81146E+00_JPRB, 0.80271E+00_JPRB, 0.79915E+00_JPRB, 0.75387E+00_JPRB, 0.74315E+00_JPRB ,&
&  0.75083E+00_JPRB, 0.75237E+00_JPRB, 0.75502E+00_JPRB, 0.75915E+00_JPRB, 0.76178E+00_JPRB, 0.74475E+00_JPRB, 0.73932E+00_JPRB ,&
&  0.70744E+00_JPRB, 0.73103E+00_JPRB, 0.74726E+00_JPRB, 0.76411E+00_JPRB, 0.78297E+00_JPRB, 0.81253E+00_JPRB, 0.87028E+00_JPRB ,&
&  0.97844E+00_JPRB, 0.10305E+01_JPRB, 0.10893E+01_JPRB, 0.11497E+01_JPRB, 0.12213E+01_JPRB, 0.12943E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.14296E+01_JPRB, 0.15053E+01_JPRB, 0.15894E+01_JPRB, 0.16837E+01_JPRB, 0.17688E+01_JPRB, 0.18454E+01_JPRB, 0.19037E+01_JPRB ,&
&  0.19591E+01_JPRB, 0.19980E+01_JPRB, 0.20279E+01_JPRB, 0.20447E+01_JPRB, 0.20646E+01_JPRB, 0.20781E+01_JPRB, 0.20941E+01_JPRB ,&
&  0.21066E+01_JPRB, 0.21111E+01_JPRB, 0.20964E+01_JPRB, 0.20652E+01_JPRB, 0.20477E+01_JPRB, 0.20443E+01_JPRB, 0.20327E+01_JPRB ,&
&  0.20154E+01_JPRB, 0.19944E+01_JPRB, 0.19648E+01_JPRB, 0.19268E+01_JPRB, 0.18732E+01_JPRB, 0.18245E+01_JPRB, 0.17668E+01_JPRB ,&
&  0.17152E+01_JPRB, 0.16716E+01_JPRB, 0.16347E+01_JPRB, 0.15943E+01_JPRB, 0.15511E+01_JPRB, 0.15074E+01_JPRB, 0.14596E+01_JPRB ,&
&  0.14194E+01_JPRB, 0.13819E+01_JPRB, 0.13532E+01_JPRB, 0.13242E+01_JPRB, 0.13032E+01_JPRB, 0.12765E+01_JPRB, 0.12547E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12172E+01_JPRB, 0.11921E+01_JPRB, 0.11630E+01_JPRB, 0.11295E+01_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.10962E+01_JPRB, 0.10831E+01_JPRB, 0.10761E+01_JPRB, 0.10696E+01_JPRB, 0.10611E+01_JPRB, 0.10500E+01_JPRB, 0.10386E+01_JPRB ,&
&  0.10293E+01_JPRB, 0.10172E+01_JPRB, 0.10065E+01_JPRB, 0.10017E+01_JPRB, 0.99464E+00_JPRB, 0.98944E+00_JPRB, 0.98558E+00_JPRB ,&
&  0.98300E+00_JPRB, 0.97962E+00_JPRB, 0.97676E+00_JPRB, 0.97215E+00_JPRB, 0.96566E+00_JPRB, 0.95732E+00_JPRB, 0.95020E+00_JPRB ,&
&  0.94604E+00_JPRB, 0.94300E+00_JPRB, 0.93882E+00_JPRB, 0.93520E+00_JPRB, 0.93513E+00_JPRB, 0.93433E+00_JPRB, 0.93214E+00_JPRB ,&
&  0.93582E+00_JPRB, 0.93977E+00_JPRB, 0.94098E+00_JPRB, 0.93972E+00_JPRB, 0.93699E+00_JPRB, 0.93706E+00_JPRB, 0.94206E+00_JPRB ,&
&  0.94883E+00_JPRB, 0.96846E+00_JPRB, 0.98618E+00_JPRB, 0.10031E+01_JPRB, 0.10195E+01_JPRB, 0.10394E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.10894E+01_JPRB, 0.11144E+01_JPRB, 0.11413E+01_JPRB, 0.11734E+01_JPRB, 0.12072E+01_JPRB, 0.12427E+01_JPRB, 0.12800E+01_JPRB ,&
&  0.13144E+01_JPRB, 0.13483E+01_JPRB, 0.13870E+01_JPRB, 0.14287E+01_JPRB, 0.14654E+01_JPRB, 0.15065E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.16005E+01_JPRB, 0.16314E+01_JPRB, 0.16653E+01_JPRB, 0.17009E+01_JPRB, 0.17394E+01_JPRB, 0.17655E+01_JPRB, 0.17906E+01_JPRB ,&
&  0.18092E+01_JPRB, 0.18205E+01_JPRB, 0.18152E+01_JPRB, 0.18072E+01_JPRB, 0.18034E+01_JPRB, 0.17955E+01_JPRB, 0.17838E+01_JPRB ,&
&  0.17748E+01_JPRB, 0.17701E+01_JPRB, 0.17572E+01_JPRB, 0.17382E+01_JPRB, 0.17131E+01_JPRB, 0.16865E+01_JPRB, 0.16433E+01_JPRB ,&
&  0.16020E+01_JPRB, 0.15691E+01_JPRB, 0.15441E+01_JPRB, 0.15166E+01_JPRB, 0.14900E+01_JPRB, 0.14650E+01_JPRB, 0.14404E+01_JPRB ,&
&  0.14169E+01_JPRB, 0.13920E+01_JPRB, 0.13668E+01_JPRB, 0.13406E+01_JPRB, 0.13190E+01_JPRB, 0.12906E+01_JPRB, 0.12608E+01_JPRB ,&
&  0.12285E+01_JPRB, 0.12037E+01_JPRB, 0.11752E+01_JPRB, 0.11433E+01_JPRB, 0.11139E+01_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.11334E+01_JPRB, 0.11255E+01_JPRB, 0.11168E+01_JPRB, 0.11075E+01_JPRB, 0.10969E+01_JPRB, 0.10858E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.10604E+01_JPRB, 0.10482E+01_JPRB, 0.10373E+01_JPRB, 0.10271E+01_JPRB, 0.10178E+01_JPRB, 0.10104E+01_JPRB, 0.10071E+01_JPRB ,&
&  0.10063E+01_JPRB, 0.10057E+01_JPRB, 0.10057E+01_JPRB, 0.10058E+01_JPRB, 0.10052E+01_JPRB, 0.10041E+01_JPRB, 0.10029E+01_JPRB ,&
&  0.10022E+01_JPRB, 0.10019E+01_JPRB, 0.10012E+01_JPRB, 0.10015E+01_JPRB, 0.10024E+01_JPRB, 0.10038E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.10110E+01_JPRB, 0.10150E+01_JPRB, 0.10192E+01_JPRB, 0.10271E+01_JPRB, 0.10361E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.10552E+01_JPRB, 0.10654E+01_JPRB, 0.10750E+01_JPRB, 0.10841E+01_JPRB, 0.10924E+01_JPRB, 0.11017E+01_JPRB, 0.11123E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.11370E+01_JPRB, 0.11507E+01_JPRB, 0.11646E+01_JPRB, 0.11779E+01_JPRB, 0.11914E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.12188E+01_JPRB, 0.12325E+01_JPRB, 0.12460E+01_JPRB, 0.12591E+01_JPRB, 0.12722E+01_JPRB, 0.12868E+01_JPRB, 0.13034E+01_JPRB ,&
&  0.13204E+01_JPRB, 0.13349E+01_JPRB, 0.13505E+01_JPRB, 0.13672E+01_JPRB, 0.13870E+01_JPRB, 0.13972E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.14307E+01_JPRB, 0.14383E+01_JPRB, 0.14470E+01_JPRB, 0.14555E+01_JPRB, 0.14625E+01_JPRB, 0.14659E+01_JPRB ,&
&  0.14690E+01_JPRB, 0.14716E+01_JPRB, 0.14707E+01_JPRB, 0.14665E+01_JPRB, 0.14607E+01_JPRB, 0.14540E+01_JPRB, 0.14423E+01_JPRB ,&
&  0.14308E+01_JPRB, 0.14201E+01_JPRB, 0.14103E+01_JPRB, 0.13970E+01_JPRB, 0.13837E+01_JPRB, 0.13705E+01_JPRB, 0.13571E+01_JPRB ,&
&  0.13408E+01_JPRB, 0.13247E+01_JPRB, 0.13077E+01_JPRB, 0.12900E+01_JPRB, 0.12720E+01_JPRB, 0.12537E+01_JPRB, 0.12347E+01_JPRB ,&
&  0.12141E+01_JPRB, 0.11949E+01_JPRB, 0.11745E+01_JPRB, 0.11548E+01_JPRB, 0.11401E+01_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB ,&
&  0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB, 0.11696E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB ,&
&  0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB, 0.14553E+00_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.15422E+00_JPRB, 0.15420E+00_JPRB, 0.15409E+00_JPRB, 0.15375E+00_JPRB, 0.15310E+00_JPRB, 0.15295E+00_JPRB, 0.15274E+00_JPRB ,&
&  0.15245E+00_JPRB, 0.15182E+00_JPRB, 0.15181E+00_JPRB, 0.15172E+00_JPRB, 0.15163E+00_JPRB, 0.15151E+00_JPRB, 0.15128E+00_JPRB ,&
&  0.15082E+00_JPRB, 0.15011E+00_JPRB, 0.14800E+00_JPRB, 0.14699E+00_JPRB, 0.14635E+00_JPRB, 0.14601E+00_JPRB, 0.14573E+00_JPRB ,&
&  0.14557E+00_JPRB, 0.14548E+00_JPRB, 0.14540E+00_JPRB, 0.14581E+00_JPRB, 0.14600E+00_JPRB, 0.14607E+00_JPRB, 0.14612E+00_JPRB ,&
&  0.14612E+00_JPRB, 0.14621E+00_JPRB, 0.14622E+00_JPRB, 0.14622E+00_JPRB, 0.14633E+00_JPRB, 0.14633E+00_JPRB, 0.14616E+00_JPRB ,&
&  0.14596E+00_JPRB, 0.14571E+00_JPRB, 0.14547E+00_JPRB, 0.14517E+00_JPRB, 0.14496E+00_JPRB, 0.14478E+00_JPRB, 0.14471E+00_JPRB ,&
&  0.14456E+00_JPRB, 0.14444E+00_JPRB, 0.14432E+00_JPRB, 0.14426E+00_JPRB, 0.14416E+00_JPRB, 0.14411E+00_JPRB, 0.14400E+00_JPRB ,&
&  0.14407E+00_JPRB, 0.14409E+00_JPRB, 0.14414E+00_JPRB, 0.14416E+00_JPRB, 0.14417E+00_JPRB, 0.14413E+00_JPRB, 0.14405E+00_JPRB ,&
&  0.14388E+00_JPRB, 0.14378E+00_JPRB, 0.14366E+00_JPRB, 0.14357E+00_JPRB, 0.14354E+00_JPRB, 0.14351E+00_JPRB, 0.14351E+00_JPRB ,&
&  0.14351E+00_JPRB, 0.14359E+00_JPRB, 0.14362E+00_JPRB, 0.14367E+00_JPRB, 0.14373E+00_JPRB, 0.14380E+00_JPRB, 0.14385E+00_JPRB ,&
&  0.14393E+00_JPRB, 0.14400E+00_JPRB, 0.14410E+00_JPRB, 0.14424E+00_JPRB, 0.14440E+00_JPRB, 0.14462E+00_JPRB, 0.14489E+00_JPRB ,&
&  0.14518E+00_JPRB, 0.14547E+00_JPRB, 0.14576E+00_JPRB, 0.14604E+00_JPRB, 0.14649E+00_JPRB, 0.14694E+00_JPRB, 0.14747E+00_JPRB ,&
&  0.14799E+00_JPRB, 0.14851E+00_JPRB, 0.14899E+00_JPRB, 0.14949E+00_JPRB, 0.15013E+00_JPRB, 0.15046E+00_JPRB, 0.15073E+00_JPRB ,&
&  0.15117E+00_JPRB, 0.15162E+00_JPRB, 0.15206E+00_JPRB, 0.15246E+00_JPRB, 0.15310E+00_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.15903E+00_JPRB, 0.15922E+00_JPRB, 0.15857E+00_JPRB, 0.15787E+00_JPRB, 0.15176E+00_JPRB, 0.15785E+00_JPRB, 0.15716E+00_JPRB ,&
&  0.15759E+00_JPRB, 0.15754E+00_JPRB, 0.15879E+00_JPRB, 0.15892E+00_JPRB, 0.15861E+00_JPRB, 0.15844E+00_JPRB, 0.15817E+00_JPRB ,&
&  0.15785E+00_JPRB, 0.15713E+00_JPRB, 0.15676E+00_JPRB, 0.15622E+00_JPRB, 0.15569E+00_JPRB, 0.15479E+00_JPRB, 0.15386E+00_JPRB ,&
&  0.15344E+00_JPRB, 0.15352E+00_JPRB, 0.15389E+00_JPRB, 0.15238E+00_JPRB, 0.15172E+00_JPRB, 0.15129E+00_JPRB, 0.15108E+00_JPRB ,&
&  0.15072E+00_JPRB, 0.15063E+00_JPRB, 0.15065E+00_JPRB, 0.15060E+00_JPRB, 0.15181E+00_JPRB, 0.15217E+00_JPRB, 0.15246E+00_JPRB ,&
&  0.15250E+00_JPRB, 0.15233E+00_JPRB, 0.15167E+00_JPRB, 0.15107E+00_JPRB, 0.15046E+00_JPRB, 0.14942E+00_JPRB, 0.14832E+00_JPRB ,&
&  0.14738E+00_JPRB, 0.14666E+00_JPRB, 0.14584E+00_JPRB, 0.14522E+00_JPRB, 0.14470E+00_JPRB, 0.14421E+00_JPRB, 0.14388E+00_JPRB ,&
&  0.14379E+00_JPRB, 0.14378E+00_JPRB, 0.14375E+00_JPRB, 0.14372E+00_JPRB, 0.14366E+00_JPRB, 0.14361E+00_JPRB, 0.14357E+00_JPRB ,&
&  0.14383E+00_JPRB, 0.14379E+00_JPRB, 0.14362E+00_JPRB, 0.14334E+00_JPRB, 0.14321E+00_JPRB, 0.14289E+00_JPRB, 0.14266E+00_JPRB ,&
&  0.14248E+00_JPRB, 0.14225E+00_JPRB, 0.14215E+00_JPRB, 0.14211E+00_JPRB, 0.14210E+00_JPRB, 0.14208E+00_JPRB, 0.14216E+00_JPRB ,&
&  0.14229E+00_JPRB, 0.14247E+00_JPRB, 0.14283E+00_JPRB, 0.14312E+00_JPRB, 0.14338E+00_JPRB, 0.14362E+00_JPRB, 0.14410E+00_JPRB ,&
&  0.14461E+00_JPRB, 0.14511E+00_JPRB, 0.14561E+00_JPRB, 0.14594E+00_JPRB, 0.14666E+00_JPRB, 0.14739E+00_JPRB, 0.14818E+00_JPRB ,&
&  0.14902E+00_JPRB, 0.15007E+00_JPRB, 0.15116E+00_JPRB, 0.15225E+00_JPRB, 0.15367E+00_JPRB, 0.15483E+00_JPRB, 0.15594E+00_JPRB ,&
&  0.15718E+00_JPRB, 0.15949E+00_JPRB, 0.16013E+00_JPRB, 0.16030E+00_JPRB, 0.15988E+00_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.17442E+00_JPRB, 0.17518E+00_JPRB, 0.17556E+00_JPRB, 0.17596E+00_JPRB, 0.17697E+00_JPRB, 0.17697E+00_JPRB, 0.17678E+00_JPRB ,&
&  0.17473E+00_JPRB, 0.17393E+00_JPRB, 0.17265E+00_JPRB, 0.17087E+00_JPRB, 0.16906E+00_JPRB, 0.16607E+00_JPRB, 0.16418E+00_JPRB ,&
&  0.16233E+00_JPRB, 0.16038E+00_JPRB, 0.15910E+00_JPRB, 0.15744E+00_JPRB, 0.15602E+00_JPRB, 0.15442E+00_JPRB, 0.15388E+00_JPRB ,&
&  0.15309E+00_JPRB, 0.15303E+00_JPRB, 0.15382E+00_JPRB, 0.15445E+00_JPRB, 0.15562E+00_JPRB, 0.15686E+00_JPRB, 0.15847E+00_JPRB ,&
&  0.15890E+00_JPRB, 0.15969E+00_JPRB, 0.16044E+00_JPRB, 0.16094E+00_JPRB, 0.16262E+00_JPRB, 0.16382E+00_JPRB, 0.16425E+00_JPRB ,&
&  0.16378E+00_JPRB, 0.16294E+00_JPRB, 0.16126E+00_JPRB, 0.15935E+00_JPRB, 0.15716E+00_JPRB, 0.15476E+00_JPRB, 0.15247E+00_JPRB ,&
&  0.15069E+00_JPRB, 0.14904E+00_JPRB, 0.14778E+00_JPRB, 0.14676E+00_JPRB, 0.14612E+00_JPRB, 0.14561E+00_JPRB, 0.14511E+00_JPRB ,&
&  0.14482E+00_JPRB, 0.14458E+00_JPRB, 0.14423E+00_JPRB, 0.14421E+00_JPRB, 0.14411E+00_JPRB, 0.14416E+00_JPRB, 0.14421E+00_JPRB ,&
&  0.14429E+00_JPRB, 0.14425E+00_JPRB, 0.14414E+00_JPRB, 0.14384E+00_JPRB, 0.14354E+00_JPRB, 0.14321E+00_JPRB, 0.14287E+00_JPRB ,&
&  0.14250E+00_JPRB, 0.14244E+00_JPRB, 0.14213E+00_JPRB, 0.14192E+00_JPRB, 0.14164E+00_JPRB, 0.14119E+00_JPRB, 0.14105E+00_JPRB ,&
&  0.14112E+00_JPRB, 0.14124E+00_JPRB, 0.14111E+00_JPRB, 0.14124E+00_JPRB, 0.14145E+00_JPRB, 0.14174E+00_JPRB, 0.14253E+00_JPRB ,&
&  0.14302E+00_JPRB, 0.14353E+00_JPRB, 0.14401E+00_JPRB, 0.14468E+00_JPRB, 0.14583E+00_JPRB, 0.14698E+00_JPRB, 0.14826E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.15126E+00_JPRB, 0.15270E+00_JPRB, 0.15414E+00_JPRB, 0.15649E+00_JPRB, 0.15817E+00_JPRB, 0.16036E+00_JPRB ,&
&  0.16228E+00_JPRB, 0.16128E+00_JPRB, 0.16928E+00_JPRB, 0.17009E+00_JPRB, 0.17138E+00_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.18178E+00_JPRB, 0.18222E+00_JPRB, 0.18215E+00_JPRB, 0.18308E+00_JPRB, 0.18385E+00_JPRB, 0.18470E+00_JPRB, 0.18414E+00_JPRB ,&
&  0.18363E+00_JPRB, 0.18214E+00_JPRB, 0.18098E+00_JPRB, 0.17802E+00_JPRB, 0.17567E+00_JPRB, 0.17283E+00_JPRB, 0.17038E+00_JPRB ,&
&  0.16818E+00_JPRB, 0.16609E+00_JPRB, 0.16413E+00_JPRB, 0.16247E+00_JPRB, 0.16093E+00_JPRB, 0.15960E+00_JPRB, 0.15831E+00_JPRB ,&
&  0.15635E+00_JPRB, 0.15582E+00_JPRB, 0.15584E+00_JPRB, 0.15709E+00_JPRB, 0.15963E+00_JPRB, 0.16131E+00_JPRB, 0.16333E+00_JPRB ,&
&  0.16563E+00_JPRB, 0.16502E+00_JPRB, 0.16798E+00_JPRB, 0.17085E+00_JPRB, 0.17318E+00_JPRB, 0.17508E+00_JPRB, 0.17611E+00_JPRB ,&
&  0.17674E+00_JPRB, 0.17200E+00_JPRB, 0.17320E+00_JPRB, 0.16912E+00_JPRB, 0.16453E+00_JPRB, 0.16245E+00_JPRB, 0.15851E+00_JPRB ,&
&  0.15672E+00_JPRB, 0.15366E+00_JPRB, 0.15167E+00_JPRB, 0.14999E+00_JPRB, 0.14800E+00_JPRB, 0.14750E+00_JPRB, 0.14668E+00_JPRB ,&
&  0.14621E+00_JPRB, 0.14629E+00_JPRB, 0.14556E+00_JPRB, 0.14533E+00_JPRB, 0.14488E+00_JPRB, 0.14446E+00_JPRB, 0.14481E+00_JPRB ,&
&  0.14538E+00_JPRB, 0.14607E+00_JPRB, 0.14676E+00_JPRB, 0.14687E+00_JPRB, 0.14681E+00_JPRB, 0.14679E+00_JPRB, 0.14670E+00_JPRB ,&
&  0.14694E+00_JPRB, 0.14694E+00_JPRB, 0.14654E+00_JPRB, 0.14607E+00_JPRB, 0.14501E+00_JPRB, 0.14397E+00_JPRB, 0.14295E+00_JPRB ,&
&  0.14206E+00_JPRB, 0.14157E+00_JPRB, 0.14116E+00_JPRB, 0.14106E+00_JPRB, 0.14109E+00_JPRB, 0.14141E+00_JPRB, 0.14174E+00_JPRB ,&
&  0.14204E+00_JPRB, 0.14276E+00_JPRB, 0.14329E+00_JPRB, 0.14415E+00_JPRB, 0.14520E+00_JPRB, 0.14614E+00_JPRB, 0.14757E+00_JPRB ,&
&  0.14913E+00_JPRB, 0.15083E+00_JPRB, 0.15326E+00_JPRB, 0.15538E+00_JPRB, 0.15805E+00_JPRB, 0.16149E+00_JPRB, 0.16403E+00_JPRB ,&
&  0.16679E+00_JPRB, 0.16949E+00_JPRB, 0.16978E+00_JPRB, 0.17676E+00_JPRB, 0.18066E+00_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.18105E+00_JPRB, 0.18256E+00_JPRB, 0.18402E+00_JPRB, 0.18550E+00_JPRB, 0.18676E+00_JPRB, 0.18798E+00_JPRB, 0.18877E+00_JPRB ,&
&  0.18632E+00_JPRB, 0.18490E+00_JPRB, 0.18230E+00_JPRB, 0.17998E+00_JPRB, 0.17727E+00_JPRB, 0.17427E+00_JPRB, 0.17325E+00_JPRB ,&
&  0.17174E+00_JPRB, 0.17142E+00_JPRB, 0.17015E+00_JPRB, 0.16960E+00_JPRB, 0.16855E+00_JPRB, 0.16794E+00_JPRB, 0.16690E+00_JPRB ,&
&  0.16485E+00_JPRB, 0.16381E+00_JPRB, 0.16282E+00_JPRB, 0.16319E+00_JPRB, 0.16467E+00_JPRB, 0.16627E+00_JPRB, 0.16797E+00_JPRB ,&
&  0.17040E+00_JPRB, 0.17149E+00_JPRB, 0.17382E+00_JPRB, 0.17556E+00_JPRB, 0.17849E+00_JPRB, 0.18095E+00_JPRB, 0.18149E+00_JPRB ,&
&  0.18155E+00_JPRB, 0.17983E+00_JPRB, 0.17933E+00_JPRB, 0.17724E+00_JPRB, 0.17130E+00_JPRB, 0.16794E+00_JPRB, 0.16319E+00_JPRB ,&
&  0.15790E+00_JPRB, 0.15393E+00_JPRB, 0.15120E+00_JPRB, 0.15031E+00_JPRB, 0.14941E+00_JPRB, 0.14911E+00_JPRB, 0.14889E+00_JPRB ,&
&  0.14813E+00_JPRB, 0.14724E+00_JPRB, 0.14621E+00_JPRB, 0.14562E+00_JPRB, 0.14599E+00_JPRB, 0.14673E+00_JPRB, 0.14832E+00_JPRB ,&
&  0.14991E+00_JPRB, 0.15081E+00_JPRB, 0.15179E+00_JPRB, 0.15212E+00_JPRB, 0.15224E+00_JPRB, 0.15247E+00_JPRB, 0.15252E+00_JPRB ,&
&  0.15315E+00_JPRB, 0.15329E+00_JPRB, 0.15278E+00_JPRB, 0.15193E+00_JPRB, 0.14978E+00_JPRB, 0.14696E+00_JPRB, 0.14473E+00_JPRB ,&
&  0.14285E+00_JPRB, 0.14162E+00_JPRB, 0.14120E+00_JPRB, 0.14076E+00_JPRB, 0.14100E+00_JPRB, 0.14137E+00_JPRB, 0.14170E+00_JPRB ,&
&  0.14228E+00_JPRB, 0.14279E+00_JPRB, 0.14329E+00_JPRB, 0.14443E+00_JPRB, 0.14536E+00_JPRB, 0.14674E+00_JPRB, 0.14819E+00_JPRB ,&
&  0.14920E+00_JPRB, 0.15075E+00_JPRB, 0.15207E+00_JPRB, 0.15351E+00_JPRB, 0.15784E+00_JPRB, 0.16128E+00_JPRB, 0.16825E+00_JPRB ,&
&  0.17276E+00_JPRB, 0.17435E+00_JPRB, 0.17500E+00_JPRB, 0.18030E+00_JPRB, 0.18053E+00_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.18125E+00_JPRB, 0.18318E+00_JPRB, 0.18518E+00_JPRB, 0.18674E+00_JPRB, 0.18830E+00_JPRB, 0.18963E+00_JPRB, 0.19033E+00_JPRB ,&
&  0.19051E+00_JPRB, 0.18995E+00_JPRB, 0.19008E+00_JPRB, 0.18886E+00_JPRB, 0.18771E+00_JPRB, 0.18353E+00_JPRB, 0.18300E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.18256E+00_JPRB, 0.18208E+00_JPRB, 0.18228E+00_JPRB, 0.18153E+00_JPRB, 0.18114E+00_JPRB, 0.17959E+00_JPRB ,&
&  0.17733E+00_JPRB, 0.17564E+00_JPRB, 0.17400E+00_JPRB, 0.17354E+00_JPRB, 0.17433E+00_JPRB, 0.17490E+00_JPRB, 0.17460E+00_JPRB ,&
&  0.17463E+00_JPRB, 0.17491E+00_JPRB, 0.17532E+00_JPRB, 0.17534E+00_JPRB, 0.18049E+00_JPRB, 0.18264E+00_JPRB, 0.18379E+00_JPRB ,&
&  0.18374E+00_JPRB, 0.18417E+00_JPRB, 0.18435E+00_JPRB, 0.18352E+00_JPRB, 0.18266E+00_JPRB, 0.17862E+00_JPRB, 0.17599E+00_JPRB ,&
&  0.16819E+00_JPRB, 0.16258E+00_JPRB, 0.15831E+00_JPRB, 0.15570E+00_JPRB, 0.15459E+00_JPRB, 0.15451E+00_JPRB, 0.15398E+00_JPRB ,&
&  0.15245E+00_JPRB, 0.15104E+00_JPRB, 0.15016E+00_JPRB, 0.15083E+00_JPRB, 0.15191E+00_JPRB, 0.15314E+00_JPRB, 0.15454E+00_JPRB ,&
&  0.15578E+00_JPRB, 0.15609E+00_JPRB, 0.15607E+00_JPRB, 0.15647E+00_JPRB, 0.15678E+00_JPRB, 0.15725E+00_JPRB, 0.15760E+00_JPRB ,&
&  0.15855E+00_JPRB, 0.15775E+00_JPRB, 0.15832E+00_JPRB, 0.15944E+00_JPRB, 0.15715E+00_JPRB, 0.15348E+00_JPRB, 0.14983E+00_JPRB ,&
&  0.14646E+00_JPRB, 0.14454E+00_JPRB, 0.14299E+00_JPRB, 0.14200E+00_JPRB, 0.14177E+00_JPRB, 0.14187E+00_JPRB, 0.14241E+00_JPRB ,&
&  0.14310E+00_JPRB, 0.14406E+00_JPRB, 0.14468E+00_JPRB, 0.14549E+00_JPRB, 0.14647E+00_JPRB, 0.14753E+00_JPRB, 0.14880E+00_JPRB ,&
&  0.15038E+00_JPRB, 0.15188E+00_JPRB, 0.15293E+00_JPRB, 0.15444E+00_JPRB, 0.15729E+00_JPRB, 0.16109E+00_JPRB, 0.16685E+00_JPRB ,&
&  0.17329E+00_JPRB, 0.17854E+00_JPRB, 0.17607E+00_JPRB, 0.17844E+00_JPRB, 0.18036E+00_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.18671E+00_JPRB, 0.18929E+00_JPRB, 0.19110E+00_JPRB, 0.19281E+00_JPRB, 0.19413E+00_JPRB, 0.19562E+00_JPRB, 0.19637E+00_JPRB ,&
&  0.19708E+00_JPRB, 0.19714E+00_JPRB, 0.19762E+00_JPRB, 0.19761E+00_JPRB, 0.19781E+00_JPRB, 0.19671E+00_JPRB, 0.19612E+00_JPRB ,&
&  0.19565E+00_JPRB, 0.19590E+00_JPRB, 0.19579E+00_JPRB, 0.19553E+00_JPRB, 0.19467E+00_JPRB, 0.19360E+00_JPRB, 0.19206E+00_JPRB ,&
&  0.18903E+00_JPRB, 0.18704E+00_JPRB, 0.18742E+00_JPRB, 0.18699E+00_JPRB, 0.18740E+00_JPRB, 0.18740E+00_JPRB, 0.18782E+00_JPRB ,&
&  0.18363E+00_JPRB, 0.18209E+00_JPRB, 0.18231E+00_JPRB, 0.18132E+00_JPRB, 0.18344E+00_JPRB, 0.18438E+00_JPRB, 0.18620E+00_JPRB ,&
&  0.18677E+00_JPRB, 0.18720E+00_JPRB, 0.18803E+00_JPRB, 0.18766E+00_JPRB, 0.18801E+00_JPRB, 0.18850E+00_JPRB, 0.18738E+00_JPRB ,&
&  0.18442E+00_JPRB, 0.18206E+00_JPRB, 0.17877E+00_JPRB, 0.17078E+00_JPRB, 0.16885E+00_JPRB, 0.16912E+00_JPRB, 0.16770E+00_JPRB ,&
&  0.16498E+00_JPRB, 0.16533E+00_JPRB, 0.16486E+00_JPRB, 0.16333E+00_JPRB, 0.16507E+00_JPRB, 0.16765E+00_JPRB, 0.16780E+00_JPRB ,&
&  0.16662E+00_JPRB, 0.16634E+00_JPRB, 0.16573E+00_JPRB, 0.16451E+00_JPRB, 0.16303E+00_JPRB, 0.16241E+00_JPRB, 0.16184E+00_JPRB ,&
&  0.16156E+00_JPRB, 0.16027E+00_JPRB, 0.16352E+00_JPRB, 0.16427E+00_JPRB, 0.15947E+00_JPRB, 0.16130E+00_JPRB, 0.15904E+00_JPRB ,&
&  0.15251E+00_JPRB, 0.14956E+00_JPRB, 0.14794E+00_JPRB, 0.14659E+00_JPRB, 0.14637E+00_JPRB, 0.14651E+00_JPRB, 0.14640E+00_JPRB ,&
&  0.14730E+00_JPRB, 0.14842E+00_JPRB, 0.14895E+00_JPRB, 0.14962E+00_JPRB, 0.15055E+00_JPRB, 0.15189E+00_JPRB, 0.15365E+00_JPRB ,&
&  0.15519E+00_JPRB, 0.15732E+00_JPRB, 0.15842E+00_JPRB, 0.16022E+00_JPRB, 0.16158E+00_JPRB, 0.16319E+00_JPRB, 0.16695E+00_JPRB ,&
&  0.17007E+00_JPRB, 0.17608E+00_JPRB, 0.18424E+00_JPRB, 0.18681E+00_JPRB, 0.18426E+00_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.19290E+00_JPRB, 0.19531E+00_JPRB, 0.19826E+00_JPRB, 0.20129E+00_JPRB, 0.20362E+00_JPRB, 0.20597E+00_JPRB, 0.20802E+00_JPRB ,&
&  0.20969E+00_JPRB, 0.21078E+00_JPRB, 0.21147E+00_JPRB, 0.21182E+00_JPRB, 0.21200E+00_JPRB, 0.21161E+00_JPRB, 0.21079E+00_JPRB ,&
&  0.20996E+00_JPRB, 0.20855E+00_JPRB, 0.20674E+00_JPRB, 0.20521E+00_JPRB, 0.20414E+00_JPRB, 0.20285E+00_JPRB, 0.20107E+00_JPRB ,&
&  0.19929E+00_JPRB, 0.19780E+00_JPRB, 0.19671E+00_JPRB, 0.19564E+00_JPRB, 0.19562E+00_JPRB, 0.19619E+00_JPRB, 0.19663E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.18737E+00_JPRB, 0.18812E+00_JPRB, 0.18911E+00_JPRB, 0.18745E+00_JPRB, 0.18877E+00_JPRB, 0.18957E+00_JPRB ,&
&  0.19072E+00_JPRB, 0.19220E+00_JPRB, 0.19363E+00_JPRB, 0.19435E+00_JPRB, 0.19487E+00_JPRB, 0.19562E+00_JPRB, 0.19507E+00_JPRB ,&
&  0.19308E+00_JPRB, 0.19133E+00_JPRB, 0.18973E+00_JPRB, 0.18812E+00_JPRB, 0.18768E+00_JPRB, 0.18914E+00_JPRB, 0.19001E+00_JPRB ,&
&  0.18906E+00_JPRB, 0.18763E+00_JPRB, 0.18628E+00_JPRB, 0.18648E+00_JPRB, 0.18715E+00_JPRB, 0.18747E+00_JPRB, 0.18547E+00_JPRB ,&
&  0.18423E+00_JPRB, 0.18154E+00_JPRB, 0.17901E+00_JPRB, 0.17654E+00_JPRB, 0.17528E+00_JPRB, 0.17190E+00_JPRB, 0.16830E+00_JPRB ,&
&  0.16752E+00_JPRB, 0.17131E+00_JPRB, 0.17720E+00_JPRB, 0.17938E+00_JPRB, 0.16956E+00_JPRB, 0.16969E+00_JPRB, 0.16965E+00_JPRB ,&
&  0.16519E+00_JPRB, 0.16020E+00_JPRB, 0.15663E+00_JPRB, 0.15511E+00_JPRB, 0.15457E+00_JPRB, 0.15414E+00_JPRB, 0.15470E+00_JPRB ,&
&  0.15571E+00_JPRB, 0.15632E+00_JPRB, 0.15696E+00_JPRB, 0.15815E+00_JPRB, 0.15929E+00_JPRB, 0.16045E+00_JPRB, 0.16146E+00_JPRB ,&
&  0.16365E+00_JPRB, 0.16630E+00_JPRB, 0.16843E+00_JPRB, 0.17130E+00_JPRB, 0.17435E+00_JPRB, 0.17742E+00_JPRB, 0.18048E+00_JPRB ,&
&  0.18385E+00_JPRB, 0.18590E+00_JPRB, 0.18748E+00_JPRB, 0.19020E+00_JPRB, 0.19152E+00_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.20620E+00_JPRB, 0.21179E+00_JPRB, 0.21677E+00_JPRB, 0.21975E+00_JPRB, 0.22193E+00_JPRB, 0.22467E+00_JPRB, 0.22714E+00_JPRB ,&
&  0.22882E+00_JPRB, 0.22979E+00_JPRB, 0.23022E+00_JPRB, 0.23000E+00_JPRB, 0.22892E+00_JPRB, 0.22722E+00_JPRB, 0.22527E+00_JPRB ,&
&  0.22316E+00_JPRB, 0.22031E+00_JPRB, 0.21767E+00_JPRB, 0.21563E+00_JPRB, 0.21393E+00_JPRB, 0.21197E+00_JPRB, 0.21039E+00_JPRB ,&
&  0.20878E+00_JPRB, 0.20824E+00_JPRB, 0.20757E+00_JPRB, 0.20692E+00_JPRB, 0.20639E+00_JPRB, 0.20543E+00_JPRB, 0.20456E+00_JPRB ,&
&  0.20237E+00_JPRB, 0.19875E+00_JPRB, 0.19706E+00_JPRB, 0.19623E+00_JPRB, 0.19638E+00_JPRB, 0.19586E+00_JPRB, 0.19516E+00_JPRB ,&
&  0.19686E+00_JPRB, 0.19977E+00_JPRB, 0.20192E+00_JPRB, 0.20313E+00_JPRB, 0.20397E+00_JPRB, 0.20511E+00_JPRB, 0.20641E+00_JPRB ,&
&  0.20424E+00_JPRB, 0.20149E+00_JPRB, 0.19947E+00_JPRB, 0.19843E+00_JPRB, 0.19828E+00_JPRB, 0.19901E+00_JPRB, 0.20057E+00_JPRB ,&
&  0.20199E+00_JPRB, 0.20277E+00_JPRB, 0.20240E+00_JPRB, 0.20103E+00_JPRB, 0.19938E+00_JPRB, 0.19859E+00_JPRB, 0.19810E+00_JPRB ,&
&  0.19785E+00_JPRB, 0.19753E+00_JPRB, 0.19712E+00_JPRB, 0.19607E+00_JPRB, 0.19072E+00_JPRB, 0.18697E+00_JPRB, 0.18434E+00_JPRB ,&
&  0.18490E+00_JPRB, 0.18498E+00_JPRB, 0.18386E+00_JPRB, 0.18896E+00_JPRB, 0.19445E+00_JPRB, 0.19002E+00_JPRB, 0.18466E+00_JPRB ,&
&  0.17743E+00_JPRB, 0.17792E+00_JPRB, 0.17528E+00_JPRB, 0.17070E+00_JPRB, 0.16936E+00_JPRB, 0.16947E+00_JPRB, 0.16920E+00_JPRB ,&
&  0.17004E+00_JPRB, 0.17031E+00_JPRB, 0.17154E+00_JPRB, 0.17216E+00_JPRB, 0.17251E+00_JPRB, 0.17331E+00_JPRB, 0.17413E+00_JPRB ,&
&  0.17610E+00_JPRB, 0.17970E+00_JPRB, 0.18400E+00_JPRB, 0.18563E+00_JPRB, 0.18832E+00_JPRB, 0.19081E+00_JPRB, 0.19249E+00_JPRB ,&
&  0.19387E+00_JPRB, 0.19578E+00_JPRB, 0.19760E+00_JPRB, 0.19916E+00_JPRB, 0.20178E+00_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.22542E+00_JPRB, 0.23063E+00_JPRB, 0.23408E+00_JPRB, 0.23735E+00_JPRB, 0.24016E+00_JPRB, 0.24464E+00_JPRB, 0.24772E+00_JPRB ,&
&  0.24833E+00_JPRB, 0.24839E+00_JPRB, 0.24793E+00_JPRB, 0.24669E+00_JPRB, 0.24471E+00_JPRB, 0.24284E+00_JPRB, 0.23987E+00_JPRB ,&
&  0.23621E+00_JPRB, 0.23284E+00_JPRB, 0.22978E+00_JPRB, 0.22690E+00_JPRB, 0.22423E+00_JPRB, 0.22181E+00_JPRB, 0.21990E+00_JPRB ,&
&  0.21930E+00_JPRB, 0.21903E+00_JPRB, 0.21804E+00_JPRB, 0.21769E+00_JPRB, 0.21764E+00_JPRB, 0.21687E+00_JPRB, 0.21545E+00_JPRB ,&
&  0.21372E+00_JPRB, 0.21078E+00_JPRB, 0.20779E+00_JPRB, 0.20674E+00_JPRB, 0.20682E+00_JPRB, 0.20613E+00_JPRB, 0.20515E+00_JPRB ,&
&  0.20644E+00_JPRB, 0.20857E+00_JPRB, 0.21094E+00_JPRB, 0.21219E+00_JPRB, 0.21303E+00_JPRB, 0.21426E+00_JPRB, 0.21572E+00_JPRB ,&
&  0.21589E+00_JPRB, 0.21336E+00_JPRB, 0.21111E+00_JPRB, 0.21064E+00_JPRB, 0.21055E+00_JPRB, 0.21020E+00_JPRB, 0.21052E+00_JPRB ,&
&  0.21115E+00_JPRB, 0.21254E+00_JPRB, 0.21347E+00_JPRB, 0.21250E+00_JPRB, 0.21074E+00_JPRB, 0.20946E+00_JPRB, 0.20864E+00_JPRB ,&
&  0.20817E+00_JPRB, 0.20760E+00_JPRB, 0.20663E+00_JPRB, 0.20513E+00_JPRB, 0.20346E+00_JPRB, 0.20187E+00_JPRB, 0.20026E+00_JPRB ,&
&  0.19889E+00_JPRB, 0.19904E+00_JPRB, 0.20120E+00_JPRB, 0.20270E+00_JPRB, 0.20251E+00_JPRB, 0.20122E+00_JPRB, 0.19931E+00_JPRB ,&
&  0.19799E+00_JPRB, 0.19574E+00_JPRB, 0.19390E+00_JPRB, 0.19247E+00_JPRB, 0.19139E+00_JPRB, 0.18992E+00_JPRB, 0.18937E+00_JPRB ,&
&  0.18972E+00_JPRB, 0.19139E+00_JPRB, 0.19208E+00_JPRB, 0.19231E+00_JPRB, 0.19204E+00_JPRB, 0.19183E+00_JPRB, 0.19277E+00_JPRB ,&
&  0.19404E+00_JPRB, 0.19601E+00_JPRB, 0.19813E+00_JPRB, 0.20092E+00_JPRB, 0.20361E+00_JPRB, 0.20704E+00_JPRB, 0.20997E+00_JPRB ,&
&  0.21188E+00_JPRB, 0.21263E+00_JPRB, 0.21417E+00_JPRB, 0.21633E+00_JPRB, 0.21982E+00_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.24385E+00_JPRB, 0.24793E+00_JPRB, 0.25149E+00_JPRB, 0.25541E+00_JPRB, 0.25790E+00_JPRB, 0.26256E+00_JPRB, 0.26482E+00_JPRB ,&
&  0.26496E+00_JPRB, 0.26468E+00_JPRB, 0.26304E+00_JPRB, 0.26029E+00_JPRB, 0.25614E+00_JPRB, 0.25194E+00_JPRB, 0.24798E+00_JPRB ,&
&  0.24490E+00_JPRB, 0.24233E+00_JPRB, 0.24019E+00_JPRB, 0.23816E+00_JPRB, 0.23544E+00_JPRB, 0.23290E+00_JPRB, 0.23088E+00_JPRB ,&
&  0.22990E+00_JPRB, 0.22911E+00_JPRB, 0.22866E+00_JPRB, 0.22837E+00_JPRB, 0.22794E+00_JPRB, 0.22749E+00_JPRB, 0.22738E+00_JPRB ,&
&  0.22617E+00_JPRB, 0.22270E+00_JPRB, 0.22074E+00_JPRB, 0.22014E+00_JPRB, 0.21967E+00_JPRB, 0.22019E+00_JPRB, 0.22000E+00_JPRB ,&
&  0.21990E+00_JPRB, 0.22062E+00_JPRB, 0.22236E+00_JPRB, 0.22421E+00_JPRB, 0.22447E+00_JPRB, 0.22506E+00_JPRB, 0.22623E+00_JPRB ,&
&  0.22665E+00_JPRB, 0.22594E+00_JPRB, 0.22417E+00_JPRB, 0.22362E+00_JPRB, 0.22424E+00_JPRB, 0.22425E+00_JPRB, 0.22461E+00_JPRB ,&
&  0.22525E+00_JPRB, 0.22636E+00_JPRB, 0.22693E+00_JPRB, 0.22540E+00_JPRB, 0.22384E+00_JPRB, 0.22326E+00_JPRB, 0.22323E+00_JPRB ,&
&  0.22288E+00_JPRB, 0.22183E+00_JPRB, 0.21969E+00_JPRB, 0.21773E+00_JPRB, 0.21492E+00_JPRB, 0.21264E+00_JPRB, 0.21148E+00_JPRB ,&
&  0.21195E+00_JPRB, 0.21243E+00_JPRB, 0.21373E+00_JPRB, 0.21490E+00_JPRB, 0.21561E+00_JPRB, 0.21569E+00_JPRB, 0.21412E+00_JPRB ,&
&  0.21206E+00_JPRB, 0.21113E+00_JPRB, 0.21015E+00_JPRB, 0.20922E+00_JPRB, 0.20939E+00_JPRB, 0.20926E+00_JPRB, 0.20886E+00_JPRB ,&
&  0.20838E+00_JPRB, 0.20879E+00_JPRB, 0.20928E+00_JPRB, 0.20929E+00_JPRB, 0.20891E+00_JPRB, 0.20821E+00_JPRB, 0.20880E+00_JPRB ,&
&  0.21026E+00_JPRB, 0.21229E+00_JPRB, 0.21461E+00_JPRB, 0.21711E+00_JPRB, 0.21979E+00_JPRB, 0.22232E+00_JPRB, 0.22499E+00_JPRB ,&
&  0.22836E+00_JPRB, 0.22997E+00_JPRB, 0.23242E+00_JPRB, 0.23591E+00_JPRB, 0.23930E+00_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.26019E+00_JPRB, 0.26330E+00_JPRB, 0.26822E+00_JPRB, 0.27291E+00_JPRB, 0.27588E+00_JPRB, 0.28066E+00_JPRB, 0.28186E+00_JPRB ,&
&  0.28129E+00_JPRB, 0.27953E+00_JPRB, 0.27536E+00_JPRB, 0.27025E+00_JPRB, 0.26543E+00_JPRB, 0.26144E+00_JPRB, 0.25864E+00_JPRB ,&
&  0.25696E+00_JPRB, 0.25547E+00_JPRB, 0.25368E+00_JPRB, 0.25082E+00_JPRB, 0.24728E+00_JPRB, 0.24393E+00_JPRB, 0.24117E+00_JPRB ,&
&  0.23991E+00_JPRB, 0.23956E+00_JPRB, 0.23969E+00_JPRB, 0.23937E+00_JPRB, 0.23902E+00_JPRB, 0.24008E+00_JPRB, 0.24161E+00_JPRB ,&
&  0.24051E+00_JPRB, 0.23623E+00_JPRB, 0.23346E+00_JPRB, 0.23322E+00_JPRB, 0.23347E+00_JPRB, 0.23412E+00_JPRB, 0.23503E+00_JPRB ,&
&  0.23554E+00_JPRB, 0.23669E+00_JPRB, 0.23853E+00_JPRB, 0.24007E+00_JPRB, 0.24121E+00_JPRB, 0.24354E+00_JPRB, 0.24472E+00_JPRB ,&
&  0.24480E+00_JPRB, 0.24404E+00_JPRB, 0.24267E+00_JPRB, 0.24010E+00_JPRB, 0.23825E+00_JPRB, 0.23764E+00_JPRB, 0.23696E+00_JPRB ,&
&  0.23762E+00_JPRB, 0.23834E+00_JPRB, 0.23898E+00_JPRB, 0.23816E+00_JPRB, 0.23644E+00_JPRB, 0.23616E+00_JPRB, 0.23658E+00_JPRB ,&
&  0.23648E+00_JPRB, 0.23526E+00_JPRB, 0.23334E+00_JPRB, 0.23209E+00_JPRB, 0.23098E+00_JPRB, 0.23070E+00_JPRB, 0.23058E+00_JPRB ,&
&  0.23039E+00_JPRB, 0.22943E+00_JPRB, 0.22882E+00_JPRB, 0.22858E+00_JPRB, 0.22849E+00_JPRB, 0.22890E+00_JPRB, 0.22868E+00_JPRB ,&
&  0.22884E+00_JPRB, 0.22911E+00_JPRB, 0.22816E+00_JPRB, 0.22780E+00_JPRB, 0.22765E+00_JPRB, 0.22789E+00_JPRB, 0.22791E+00_JPRB ,&
&  0.22757E+00_JPRB, 0.22710E+00_JPRB, 0.22681E+00_JPRB, 0.22694E+00_JPRB, 0.22661E+00_JPRB, 0.22600E+00_JPRB, 0.22684E+00_JPRB ,&
&  0.23000E+00_JPRB, 0.23334E+00_JPRB, 0.23620E+00_JPRB, 0.23807E+00_JPRB, 0.23876E+00_JPRB, 0.24037E+00_JPRB, 0.24362E+00_JPRB ,&
&  0.24680E+00_JPRB, 0.24883E+00_JPRB, 0.25183E+00_JPRB, 0.25411E+00_JPRB, 0.25652E+00_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.27883E+00_JPRB, 0.28144E+00_JPRB, 0.28552E+00_JPRB, 0.29123E+00_JPRB, 0.29532E+00_JPRB, 0.29924E+00_JPRB, 0.29918E+00_JPRB ,&
&  0.29672E+00_JPRB, 0.29315E+00_JPRB, 0.28874E+00_JPRB, 0.28330E+00_JPRB, 0.27764E+00_JPRB, 0.27339E+00_JPRB, 0.27182E+00_JPRB ,&
&  0.27163E+00_JPRB, 0.27052E+00_JPRB, 0.26772E+00_JPRB, 0.26378E+00_JPRB, 0.25924E+00_JPRB, 0.25532E+00_JPRB, 0.25205E+00_JPRB ,&
&  0.25062E+00_JPRB, 0.25056E+00_JPRB, 0.25120E+00_JPRB, 0.25147E+00_JPRB, 0.25153E+00_JPRB, 0.25370E+00_JPRB, 0.25750E+00_JPRB ,&
&  0.25540E+00_JPRB, 0.25293E+00_JPRB, 0.24730E+00_JPRB, 0.24672E+00_JPRB, 0.24825E+00_JPRB, 0.25080E+00_JPRB, 0.25352E+00_JPRB ,&
&  0.25622E+00_JPRB, 0.25969E+00_JPRB, 0.26295E+00_JPRB, 0.26504E+00_JPRB, 0.26630E+00_JPRB, 0.26748E+00_JPRB, 0.26823E+00_JPRB ,&
&  0.26843E+00_JPRB, 0.26735E+00_JPRB, 0.26542E+00_JPRB, 0.26195E+00_JPRB, 0.25747E+00_JPRB, 0.25443E+00_JPRB, 0.25296E+00_JPRB ,&
&  0.25147E+00_JPRB, 0.25125E+00_JPRB, 0.25115E+00_JPRB, 0.25048E+00_JPRB, 0.24945E+00_JPRB, 0.24911E+00_JPRB, 0.24960E+00_JPRB ,&
&  0.24990E+00_JPRB, 0.24939E+00_JPRB, 0.24830E+00_JPRB, 0.24796E+00_JPRB, 0.24855E+00_JPRB, 0.24992E+00_JPRB, 0.25123E+00_JPRB ,&
&  0.25222E+00_JPRB, 0.25200E+00_JPRB, 0.25127E+00_JPRB, 0.24952E+00_JPRB, 0.24828E+00_JPRB, 0.24843E+00_JPRB, 0.24965E+00_JPRB ,&
&  0.25044E+00_JPRB, 0.25046E+00_JPRB, 0.25063E+00_JPRB, 0.24957E+00_JPRB, 0.24858E+00_JPRB, 0.24859E+00_JPRB, 0.24895E+00_JPRB ,&
&  0.24900E+00_JPRB, 0.24876E+00_JPRB, 0.24839E+00_JPRB, 0.24822E+00_JPRB, 0.24869E+00_JPRB, 0.24886E+00_JPRB, 0.24991E+00_JPRB ,&
&  0.25249E+00_JPRB, 0.25539E+00_JPRB, 0.25822E+00_JPRB, 0.26024E+00_JPRB, 0.26149E+00_JPRB, 0.26259E+00_JPRB, 0.26622E+00_JPRB ,&
&  0.27090E+00_JPRB, 0.27435E+00_JPRB, 0.27515E+00_JPRB, 0.27528E+00_JPRB, 0.27688E+00_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.30227E+00_JPRB, 0.30164E+00_JPRB, 0.30567E+00_JPRB, 0.31074E+00_JPRB, 0.31672E+00_JPRB, 0.31817E+00_JPRB, 0.31556E+00_JPRB ,&
&  0.31238E+00_JPRB, 0.30828E+00_JPRB, 0.30229E+00_JPRB, 0.29486E+00_JPRB, 0.28810E+00_JPRB, 0.28605E+00_JPRB, 0.28603E+00_JPRB ,&
&  0.28628E+00_JPRB, 0.28462E+00_JPRB, 0.28115E+00_JPRB, 0.27709E+00_JPRB, 0.27227E+00_JPRB, 0.26785E+00_JPRB, 0.26432E+00_JPRB ,&
&  0.26280E+00_JPRB, 0.26182E+00_JPRB, 0.26195E+00_JPRB, 0.26201E+00_JPRB, 0.26269E+00_JPRB, 0.26535E+00_JPRB, 0.27068E+00_JPRB ,&
&  0.26855E+00_JPRB, 0.26956E+00_JPRB, 0.26234E+00_JPRB, 0.26489E+00_JPRB, 0.26765E+00_JPRB, 0.27292E+00_JPRB, 0.27929E+00_JPRB ,&
&  0.28596E+00_JPRB, 0.29138E+00_JPRB, 0.29426E+00_JPRB, 0.29400E+00_JPRB, 0.29220E+00_JPRB, 0.29185E+00_JPRB, 0.29165E+00_JPRB ,&
&  0.29092E+00_JPRB, 0.28992E+00_JPRB, 0.28801E+00_JPRB, 0.28421E+00_JPRB, 0.27893E+00_JPRB, 0.27434E+00_JPRB, 0.27104E+00_JPRB ,&
&  0.27020E+00_JPRB, 0.26966E+00_JPRB, 0.26810E+00_JPRB, 0.26644E+00_JPRB, 0.26469E+00_JPRB, 0.26363E+00_JPRB, 0.26407E+00_JPRB ,&
&  0.26528E+00_JPRB, 0.26637E+00_JPRB, 0.26717E+00_JPRB, 0.26866E+00_JPRB, 0.27123E+00_JPRB, 0.27377E+00_JPRB, 0.27586E+00_JPRB ,&
&  0.27697E+00_JPRB, 0.27673E+00_JPRB, 0.27618E+00_JPRB, 0.27540E+00_JPRB, 0.27493E+00_JPRB, 0.27443E+00_JPRB, 0.27446E+00_JPRB ,&
&  0.27452E+00_JPRB, 0.27424E+00_JPRB, 0.27326E+00_JPRB, 0.27167E+00_JPRB, 0.27019E+00_JPRB, 0.26895E+00_JPRB, 0.26842E+00_JPRB ,&
&  0.26857E+00_JPRB, 0.26885E+00_JPRB, 0.26867E+00_JPRB, 0.26822E+00_JPRB, 0.26853E+00_JPRB, 0.26927E+00_JPRB, 0.27083E+00_JPRB ,&
&  0.27301E+00_JPRB, 0.27571E+00_JPRB, 0.27942E+00_JPRB, 0.28333E+00_JPRB, 0.28527E+00_JPRB, 0.28830E+00_JPRB, 0.29358E+00_JPRB ,&
&  0.29852E+00_JPRB, 0.30114E+00_JPRB, 0.30281E+00_JPRB, 0.30407E+00_JPRB, 0.30437E+00_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.33322E+00_JPRB, 0.32748E+00_JPRB, 0.32803E+00_JPRB, 0.33238E+00_JPRB, 0.33882E+00_JPRB, 0.33722E+00_JPRB, 0.33415E+00_JPRB ,&
&  0.32792E+00_JPRB, 0.32012E+00_JPRB, 0.31251E+00_JPRB, 0.30536E+00_JPRB, 0.30192E+00_JPRB, 0.30189E+00_JPRB, 0.30143E+00_JPRB ,&
&  0.30001E+00_JPRB, 0.29665E+00_JPRB, 0.29185E+00_JPRB, 0.28711E+00_JPRB, 0.28357E+00_JPRB, 0.28151E+00_JPRB, 0.27966E+00_JPRB ,&
&  0.27827E+00_JPRB, 0.27644E+00_JPRB, 0.27687E+00_JPRB, 0.27658E+00_JPRB, 0.27719E+00_JPRB, 0.28084E+00_JPRB, 0.28398E+00_JPRB ,&
&  0.28563E+00_JPRB, 0.28885E+00_JPRB, 0.28782E+00_JPRB, 0.28838E+00_JPRB, 0.29212E+00_JPRB, 0.30081E+00_JPRB, 0.31207E+00_JPRB ,&
&  0.32198E+00_JPRB, 0.32770E+00_JPRB, 0.32950E+00_JPRB, 0.32756E+00_JPRB, 0.32449E+00_JPRB, 0.32189E+00_JPRB, 0.32032E+00_JPRB ,&
&  0.31847E+00_JPRB, 0.31477E+00_JPRB, 0.31076E+00_JPRB, 0.30743E+00_JPRB, 0.30604E+00_JPRB, 0.30494E+00_JPRB, 0.30329E+00_JPRB ,&
&  0.30110E+00_JPRB, 0.29868E+00_JPRB, 0.29528E+00_JPRB, 0.29154E+00_JPRB, 0.28819E+00_JPRB, 0.28633E+00_JPRB, 0.28634E+00_JPRB ,&
&  0.28792E+00_JPRB, 0.29062E+00_JPRB, 0.29435E+00_JPRB, 0.29904E+00_JPRB, 0.30316E+00_JPRB, 0.30726E+00_JPRB, 0.30991E+00_JPRB ,&
&  0.31142E+00_JPRB, 0.31189E+00_JPRB, 0.31172E+00_JPRB, 0.31124E+00_JPRB, 0.31022E+00_JPRB, 0.30859E+00_JPRB, 0.30669E+00_JPRB ,&
&  0.30490E+00_JPRB, 0.30329E+00_JPRB, 0.30172E+00_JPRB, 0.30007E+00_JPRB, 0.29733E+00_JPRB, 0.29421E+00_JPRB, 0.29195E+00_JPRB ,&
&  0.29067E+00_JPRB, 0.28998E+00_JPRB, 0.28948E+00_JPRB, 0.28875E+00_JPRB, 0.28859E+00_JPRB, 0.28957E+00_JPRB, 0.29158E+00_JPRB ,&
&  0.29459E+00_JPRB, 0.29881E+00_JPRB, 0.30424E+00_JPRB, 0.31033E+00_JPRB, 0.31734E+00_JPRB, 0.32481E+00_JPRB, 0.33199E+00_JPRB ,&
&  0.33731E+00_JPRB, 0.33894E+00_JPRB, 0.34042E+00_JPRB, 0.34155E+00_JPRB, 0.33904E+00_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.36994E+00_JPRB, 0.36022E+00_JPRB, 0.35604E+00_JPRB, 0.35747E+00_JPRB, 0.35996E+00_JPRB, 0.35777E+00_JPRB, 0.35204E+00_JPRB ,&
&  0.34138E+00_JPRB, 0.33186E+00_JPRB, 0.32409E+00_JPRB, 0.32016E+00_JPRB, 0.32118E+00_JPRB, 0.32098E+00_JPRB, 0.31821E+00_JPRB ,&
&  0.31537E+00_JPRB, 0.31049E+00_JPRB, 0.30648E+00_JPRB, 0.30245E+00_JPRB, 0.29935E+00_JPRB, 0.29797E+00_JPRB, 0.29736E+00_JPRB ,&
&  0.29569E+00_JPRB, 0.29365E+00_JPRB, 0.29306E+00_JPRB, 0.29294E+00_JPRB, 0.29465E+00_JPRB, 0.29888E+00_JPRB, 0.30423E+00_JPRB ,&
&  0.31300E+00_JPRB, 0.29883E+00_JPRB, 0.31600E+00_JPRB, 0.32086E+00_JPRB, 0.32591E+00_JPRB, 0.33498E+00_JPRB, 0.34563E+00_JPRB ,&
&  0.35502E+00_JPRB, 0.35984E+00_JPRB, 0.36177E+00_JPRB, 0.36184E+00_JPRB, 0.35983E+00_JPRB, 0.35820E+00_JPRB, 0.35429E+00_JPRB ,&
&  0.34929E+00_JPRB, 0.34733E+00_JPRB, 0.34656E+00_JPRB, 0.34459E+00_JPRB, 0.34236E+00_JPRB, 0.33873E+00_JPRB, 0.33456E+00_JPRB ,&
&  0.33049E+00_JPRB, 0.32635E+00_JPRB, 0.32291E+00_JPRB, 0.31967E+00_JPRB, 0.31727E+00_JPRB, 0.31635E+00_JPRB, 0.31711E+00_JPRB ,&
&  0.31844E+00_JPRB, 0.32126E+00_JPRB, 0.32824E+00_JPRB, 0.33726E+00_JPRB, 0.34582E+00_JPRB, 0.35262E+00_JPRB, 0.35780E+00_JPRB ,&
&  0.36056E+00_JPRB, 0.36169E+00_JPRB, 0.36118E+00_JPRB, 0.35966E+00_JPRB, 0.35657E+00_JPRB, 0.35298E+00_JPRB, 0.34770E+00_JPRB ,&
&  0.34292E+00_JPRB, 0.33876E+00_JPRB, 0.33434E+00_JPRB, 0.33049E+00_JPRB, 0.32616E+00_JPRB, 0.32115E+00_JPRB, 0.31651E+00_JPRB ,&
&  0.31288E+00_JPRB, 0.31032E+00_JPRB, 0.30842E+00_JPRB, 0.30762E+00_JPRB, 0.30807E+00_JPRB, 0.31050E+00_JPRB, 0.31322E+00_JPRB ,&
&  0.31674E+00_JPRB, 0.32288E+00_JPRB, 0.33039E+00_JPRB, 0.34005E+00_JPRB, 0.35286E+00_JPRB, 0.36221E+00_JPRB, 0.36988E+00_JPRB ,&
&  0.37571E+00_JPRB, 0.37750E+00_JPRB, 0.37976E+00_JPRB, 0.37915E+00_JPRB, 0.37628E+00_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.40751E+00_JPRB, 0.39893E+00_JPRB, 0.39029E+00_JPRB, 0.38839E+00_JPRB, 0.38386E+00_JPRB, 0.37730E+00_JPRB, 0.36878E+00_JPRB ,&
&  0.35677E+00_JPRB, 0.34814E+00_JPRB, 0.34182E+00_JPRB, 0.33962E+00_JPRB, 0.34174E+00_JPRB, 0.33963E+00_JPRB, 0.33509E+00_JPRB ,&
&  0.33175E+00_JPRB, 0.32793E+00_JPRB, 0.32527E+00_JPRB, 0.32211E+00_JPRB, 0.31956E+00_JPRB, 0.31850E+00_JPRB, 0.31784E+00_JPRB ,&
&  0.31608E+00_JPRB, 0.31307E+00_JPRB, 0.31053E+00_JPRB, 0.30932E+00_JPRB, 0.31179E+00_JPRB, 0.31792E+00_JPRB, 0.32770E+00_JPRB ,&
&  0.32501E+00_JPRB, 0.30854E+00_JPRB, 0.32613E+00_JPRB, 0.34790E+00_JPRB, 0.36262E+00_JPRB, 0.37774E+00_JPRB, 0.38934E+00_JPRB ,&
&  0.40008E+00_JPRB, 0.40334E+00_JPRB, 0.40590E+00_JPRB, 0.40753E+00_JPRB, 0.40679E+00_JPRB, 0.40212E+00_JPRB, 0.39721E+00_JPRB ,&
&  0.39366E+00_JPRB, 0.39091E+00_JPRB, 0.38763E+00_JPRB, 0.38138E+00_JPRB, 0.37443E+00_JPRB, 0.36689E+00_JPRB, 0.35957E+00_JPRB ,&
&  0.35336E+00_JPRB, 0.34742E+00_JPRB, 0.34301E+00_JPRB, 0.34109E+00_JPRB, 0.34229E+00_JPRB, 0.34544E+00_JPRB, 0.35147E+00_JPRB ,&
&  0.35878E+00_JPRB, 0.37050E+00_JPRB, 0.38964E+00_JPRB, 0.40695E+00_JPRB, 0.41930E+00_JPRB, 0.42650E+00_JPRB, 0.43161E+00_JPRB ,&
&  0.43136E+00_JPRB, 0.42897E+00_JPRB, 0.42664E+00_JPRB, 0.42240E+00_JPRB, 0.41592E+00_JPRB, 0.40719E+00_JPRB, 0.39652E+00_JPRB ,&
&  0.38726E+00_JPRB, 0.37935E+00_JPRB, 0.37184E+00_JPRB, 0.36476E+00_JPRB, 0.35780E+00_JPRB, 0.35105E+00_JPRB, 0.34522E+00_JPRB ,&
&  0.33978E+00_JPRB, 0.33447E+00_JPRB, 0.33077E+00_JPRB, 0.33035E+00_JPRB, 0.33358E+00_JPRB, 0.33798E+00_JPRB, 0.34331E+00_JPRB ,&
&  0.34908E+00_JPRB, 0.35894E+00_JPRB, 0.36796E+00_JPRB, 0.38450E+00_JPRB, 0.40059E+00_JPRB, 0.40698E+00_JPRB, 0.41268E+00_JPRB ,&
&  0.41787E+00_JPRB, 0.41687E+00_JPRB, 0.41875E+00_JPRB, 0.41613E+00_JPRB, 0.41081E+00_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.44157E+00_JPRB, 0.44112E+00_JPRB, 0.43519E+00_JPRB, 0.42409E+00_JPRB, 0.41053E+00_JPRB, 0.40097E+00_JPRB, 0.38859E+00_JPRB ,&
&  0.37603E+00_JPRB, 0.36877E+00_JPRB, 0.36083E+00_JPRB, 0.35654E+00_JPRB, 0.35609E+00_JPRB, 0.35230E+00_JPRB, 0.34898E+00_JPRB ,&
&  0.34601E+00_JPRB, 0.34075E+00_JPRB, 0.33753E+00_JPRB, 0.33476E+00_JPRB, 0.33438E+00_JPRB, 0.33442E+00_JPRB, 0.33408E+00_JPRB ,&
&  0.33251E+00_JPRB, 0.32974E+00_JPRB, 0.32733E+00_JPRB, 0.32577E+00_JPRB, 0.32769E+00_JPRB, 0.33444E+00_JPRB, 0.34777E+00_JPRB ,&
&  0.37759E+00_JPRB, 0.34157E+00_JPRB, 0.35727E+00_JPRB, 0.37373E+00_JPRB, 0.40211E+00_JPRB, 0.42551E+00_JPRB, 0.43704E+00_JPRB ,&
&  0.44976E+00_JPRB, 0.46141E+00_JPRB, 0.46946E+00_JPRB, 0.47659E+00_JPRB, 0.47247E+00_JPRB, 0.46354E+00_JPRB, 0.45616E+00_JPRB ,&
&  0.44640E+00_JPRB, 0.43469E+00_JPRB, 0.42279E+00_JPRB, 0.41022E+00_JPRB, 0.40250E+00_JPRB, 0.39797E+00_JPRB, 0.39231E+00_JPRB ,&
&  0.38432E+00_JPRB, 0.37637E+00_JPRB, 0.37123E+00_JPRB, 0.37093E+00_JPRB, 0.37677E+00_JPRB, 0.38894E+00_JPRB, 0.41215E+00_JPRB ,&
&  0.43566E+00_JPRB, 0.46166E+00_JPRB, 0.48836E+00_JPRB, 0.50827E+00_JPRB, 0.52146E+00_JPRB, 0.52457E+00_JPRB, 0.52166E+00_JPRB ,&
&  0.51289E+00_JPRB, 0.50270E+00_JPRB, 0.49377E+00_JPRB, 0.48211E+00_JPRB, 0.46753E+00_JPRB, 0.45460E+00_JPRB, 0.44432E+00_JPRB ,&
&  0.43652E+00_JPRB, 0.42796E+00_JPRB, 0.41813E+00_JPRB, 0.40777E+00_JPRB, 0.39712E+00_JPRB, 0.38791E+00_JPRB, 0.38011E+00_JPRB ,&
&  0.37324E+00_JPRB, 0.36690E+00_JPRB, 0.36363E+00_JPRB, 0.36559E+00_JPRB, 0.37021E+00_JPRB, 0.37535E+00_JPRB, 0.38027E+00_JPRB ,&
&  0.38669E+00_JPRB, 0.39805E+00_JPRB, 0.40517E+00_JPRB, 0.43420E+00_JPRB, 0.44945E+00_JPRB, 0.45624E+00_JPRB, 0.46081E+00_JPRB ,&
&  0.46107E+00_JPRB, 0.45644E+00_JPRB, 0.40443E+00_JPRB, 0.43488E+00_JPRB, 0.44127E+00_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.48784E+00_JPRB, 0.48539E+00_JPRB, 0.47680E+00_JPRB, 0.45709E+00_JPRB, 0.43752E+00_JPRB, 0.42542E+00_JPRB, 0.41264E+00_JPRB ,&
&  0.39964E+00_JPRB, 0.38932E+00_JPRB, 0.37648E+00_JPRB, 0.37075E+00_JPRB, 0.36815E+00_JPRB, 0.36712E+00_JPRB, 0.36497E+00_JPRB ,&
&  0.36000E+00_JPRB, 0.35245E+00_JPRB, 0.34680E+00_JPRB, 0.34453E+00_JPRB, 0.34502E+00_JPRB, 0.34631E+00_JPRB, 0.34765E+00_JPRB ,&
&  0.34661E+00_JPRB, 0.34588E+00_JPRB, 0.34480E+00_JPRB, 0.34427E+00_JPRB, 0.34563E+00_JPRB, 0.35337E+00_JPRB, 0.36910E+00_JPRB ,&
&  0.40953E+00_JPRB, 0.37614E+00_JPRB, 0.38950E+00_JPRB, 0.41247E+00_JPRB, 0.43802E+00_JPRB, 0.47534E+00_JPRB, 0.51444E+00_JPRB ,&
&  0.53480E+00_JPRB, 0.53433E+00_JPRB, 0.54291E+00_JPRB, 0.56167E+00_JPRB, 0.55973E+00_JPRB, 0.55184E+00_JPRB, 0.53747E+00_JPRB ,&
&  0.51608E+00_JPRB, 0.49125E+00_JPRB, 0.46680E+00_JPRB, 0.45025E+00_JPRB, 0.44334E+00_JPRB, 0.44025E+00_JPRB, 0.43808E+00_JPRB ,&
&  0.43011E+00_JPRB, 0.42679E+00_JPRB, 0.43455E+00_JPRB, 0.44594E+00_JPRB, 0.46050E+00_JPRB, 0.48575E+00_JPRB, 0.52184E+00_JPRB ,&
&  0.54856E+00_JPRB, 0.57723E+00_JPRB, 0.60263E+00_JPRB, 0.62100E+00_JPRB, 0.63502E+00_JPRB, 0.63399E+00_JPRB, 0.61491E+00_JPRB ,&
&  0.59095E+00_JPRB, 0.56500E+00_JPRB, 0.54124E+00_JPRB, 0.52030E+00_JPRB, 0.50397E+00_JPRB, 0.49500E+00_JPRB, 0.48961E+00_JPRB ,&
&  0.48564E+00_JPRB, 0.48250E+00_JPRB, 0.47641E+00_JPRB, 0.46806E+00_JPRB, 0.45996E+00_JPRB, 0.45360E+00_JPRB, 0.44775E+00_JPRB ,&
&  0.44212E+00_JPRB, 0.43650E+00_JPRB, 0.43408E+00_JPRB, 0.43576E+00_JPRB, 0.43684E+00_JPRB, 0.43919E+00_JPRB, 0.44313E+00_JPRB ,&
&  0.44574E+00_JPRB, 0.45596E+00_JPRB, 0.45811E+00_JPRB, 0.53030E+00_JPRB, 0.54331E+00_JPRB, 0.53997E+00_JPRB, 0.53286E+00_JPRB ,&
&  0.51933E+00_JPRB, 0.50599E+00_JPRB, 0.49446E+00_JPRB, 0.46090E+00_JPRB, 0.48754E+00_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.54449E+00_JPRB, 0.53456E+00_JPRB, 0.51113E+00_JPRB, 0.48308E+00_JPRB, 0.46184E+00_JPRB, 0.44400E+00_JPRB, 0.42954E+00_JPRB ,&
&  0.42005E+00_JPRB, 0.40920E+00_JPRB, 0.39633E+00_JPRB, 0.38714E+00_JPRB, 0.38563E+00_JPRB, 0.38309E+00_JPRB, 0.37896E+00_JPRB ,&
&  0.37161E+00_JPRB, 0.36462E+00_JPRB, 0.35991E+00_JPRB, 0.35829E+00_JPRB, 0.35875E+00_JPRB, 0.36048E+00_JPRB, 0.36113E+00_JPRB ,&
&  0.35905E+00_JPRB, 0.35514E+00_JPRB, 0.35503E+00_JPRB, 0.35738E+00_JPRB, 0.36272E+00_JPRB, 0.37121E+00_JPRB, 0.39094E+00_JPRB ,&
&  0.43797E+00_JPRB, 0.42192E+00_JPRB, 0.44404E+00_JPRB, 0.45730E+00_JPRB, 0.47821E+00_JPRB, 0.51970E+00_JPRB, 0.56481E+00_JPRB ,&
&  0.59330E+00_JPRB, 0.61018E+00_JPRB, 0.64206E+00_JPRB, 0.67048E+00_JPRB, 0.66747E+00_JPRB, 0.64459E+00_JPRB, 0.61357E+00_JPRB ,&
&  0.58253E+00_JPRB, 0.55924E+00_JPRB, 0.53957E+00_JPRB, 0.52359E+00_JPRB, 0.51318E+00_JPRB, 0.50892E+00_JPRB, 0.50814E+00_JPRB ,&
&  0.50145E+00_JPRB, 0.50172E+00_JPRB, 0.51643E+00_JPRB, 0.54291E+00_JPRB, 0.57380E+00_JPRB, 0.60475E+00_JPRB, 0.63801E+00_JPRB ,&
&  0.66834E+00_JPRB, 0.69178E+00_JPRB, 0.70813E+00_JPRB, 0.71689E+00_JPRB, 0.70683E+00_JPRB, 0.68991E+00_JPRB, 0.65316E+00_JPRB ,&
&  0.61386E+00_JPRB, 0.58200E+00_JPRB, 0.56248E+00_JPRB, 0.54770E+00_JPRB, 0.53732E+00_JPRB, 0.53110E+00_JPRB, 0.52738E+00_JPRB ,&
&  0.52662E+00_JPRB, 0.52884E+00_JPRB, 0.52990E+00_JPRB, 0.52940E+00_JPRB, 0.52843E+00_JPRB, 0.52786E+00_JPRB, 0.52615E+00_JPRB ,&
&  0.52439E+00_JPRB, 0.52395E+00_JPRB, 0.52357E+00_JPRB, 0.52043E+00_JPRB, 0.51572E+00_JPRB, 0.51097E+00_JPRB, 0.50762E+00_JPRB ,&
&  0.50563E+00_JPRB, 0.51652E+00_JPRB, 0.52261E+00_JPRB, 0.56152E+00_JPRB, 0.57567E+00_JPRB, 0.58033E+00_JPRB, 0.57879E+00_JPRB ,&
&  0.56582E+00_JPRB, 0.55004E+00_JPRB, 0.52670E+00_JPRB, 0.51174E+00_JPRB, 0.52364E+00_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.63775E+00_JPRB, 0.59828E+00_JPRB, 0.54987E+00_JPRB, 0.51235E+00_JPRB, 0.48563E+00_JPRB, 0.46602E+00_JPRB, 0.45461E+00_JPRB ,&
&  0.45008E+00_JPRB, 0.43883E+00_JPRB, 0.42336E+00_JPRB, 0.41062E+00_JPRB, 0.40434E+00_JPRB, 0.39986E+00_JPRB, 0.39341E+00_JPRB ,&
&  0.38799E+00_JPRB, 0.38308E+00_JPRB, 0.37970E+00_JPRB, 0.37824E+00_JPRB, 0.37737E+00_JPRB, 0.37589E+00_JPRB, 0.37201E+00_JPRB ,&
&  0.36624E+00_JPRB, 0.36485E+00_JPRB, 0.36634E+00_JPRB, 0.37309E+00_JPRB, 0.38245E+00_JPRB, 0.39371E+00_JPRB, 0.41326E+00_JPRB ,&
&  0.47168E+00_JPRB, 0.48154E+00_JPRB, 0.52360E+00_JPRB, 0.53721E+00_JPRB, 0.56039E+00_JPRB, 0.60688E+00_JPRB, 0.64307E+00_JPRB ,&
&  0.68049E+00_JPRB, 0.72393E+00_JPRB, 0.77872E+00_JPRB, 0.80206E+00_JPRB, 0.77904E+00_JPRB, 0.73846E+00_JPRB, 0.70187E+00_JPRB ,&
&  0.66732E+00_JPRB, 0.64280E+00_JPRB, 0.61943E+00_JPRB, 0.59786E+00_JPRB, 0.58361E+00_JPRB, 0.58194E+00_JPRB, 0.58879E+00_JPRB ,&
&  0.59347E+00_JPRB, 0.58651E+00_JPRB, 0.60784E+00_JPRB, 0.63801E+00_JPRB, 0.68585E+00_JPRB, 0.72235E+00_JPRB, 0.75787E+00_JPRB ,&
&  0.78911E+00_JPRB, 0.80148E+00_JPRB, 0.80571E+00_JPRB, 0.79154E+00_JPRB, 0.74659E+00_JPRB, 0.69317E+00_JPRB, 0.64016E+00_JPRB ,&
&  0.59113E+00_JPRB, 0.55951E+00_JPRB, 0.54424E+00_JPRB, 0.53735E+00_JPRB, 0.53713E+00_JPRB, 0.54144E+00_JPRB, 0.55136E+00_JPRB ,&
&  0.56692E+00_JPRB, 0.58138E+00_JPRB, 0.59258E+00_JPRB, 0.59739E+00_JPRB, 0.59548E+00_JPRB, 0.59141E+00_JPRB, 0.58430E+00_JPRB ,&
&  0.57619E+00_JPRB, 0.56695E+00_JPRB, 0.56076E+00_JPRB, 0.55506E+00_JPRB, 0.55074E+00_JPRB, 0.54850E+00_JPRB, 0.54819E+00_JPRB ,&
&  0.54984E+00_JPRB, 0.56933E+00_JPRB, 0.57298E+00_JPRB, 0.64647E+00_JPRB, 0.67523E+00_JPRB, 0.66016E+00_JPRB, 0.63873E+00_JPRB ,&
&  0.61258E+00_JPRB, 0.58895E+00_JPRB, 0.56881E+00_JPRB, 0.55781E+00_JPRB, 0.62554E+00_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.58964E+00_JPRB, 0.57120E+00_JPRB, 0.54108E+00_JPRB, 0.51486E+00_JPRB, 0.49788E+00_JPRB, 0.48579E+00_JPRB, 0.47933E+00_JPRB ,&
&  0.47573E+00_JPRB, 0.46686E+00_JPRB, 0.45782E+00_JPRB, 0.44527E+00_JPRB, 0.43471E+00_JPRB, 0.42570E+00_JPRB, 0.41721E+00_JPRB ,&
&  0.40992E+00_JPRB, 0.40550E+00_JPRB, 0.40160E+00_JPRB, 0.39757E+00_JPRB, 0.39164E+00_JPRB, 0.38322E+00_JPRB, 0.37809E+00_JPRB ,&
&  0.37586E+00_JPRB, 0.37672E+00_JPRB, 0.38109E+00_JPRB, 0.38996E+00_JPRB, 0.39977E+00_JPRB, 0.41187E+00_JPRB, 0.43073E+00_JPRB ,&
&  0.48006E+00_JPRB, 0.63159E+00_JPRB, 0.67723E+00_JPRB, 0.68211E+00_JPRB, 0.72193E+00_JPRB, 0.71919E+00_JPRB, 0.74647E+00_JPRB ,&
&  0.80993E+00_JPRB, 0.87050E+00_JPRB, 0.92427E+00_JPRB, 0.94315E+00_JPRB, 0.91380E+00_JPRB, 0.88261E+00_JPRB, 0.83947E+00_JPRB ,&
&  0.79408E+00_JPRB, 0.74538E+00_JPRB, 0.69709E+00_JPRB, 0.65296E+00_JPRB, 0.61720E+00_JPRB, 0.61171E+00_JPRB, 0.62724E+00_JPRB ,&
&  0.65839E+00_JPRB, 0.68275E+00_JPRB, 0.69544E+00_JPRB, 0.73216E+00_JPRB, 0.81649E+00_JPRB, 0.88066E+00_JPRB, 0.96166E+00_JPRB ,&
&  0.98347E+00_JPRB, 0.95320E+00_JPRB, 0.90702E+00_JPRB, 0.84548E+00_JPRB, 0.76095E+00_JPRB, 0.67961E+00_JPRB, 0.61686E+00_JPRB ,&
&  0.57245E+00_JPRB, 0.54293E+00_JPRB, 0.52528E+00_JPRB, 0.51893E+00_JPRB, 0.52539E+00_JPRB, 0.54351E+00_JPRB, 0.56576E+00_JPRB ,&
&  0.59086E+00_JPRB, 0.61442E+00_JPRB, 0.62748E+00_JPRB, 0.63133E+00_JPRB, 0.62392E+00_JPRB, 0.60719E+00_JPRB, 0.58073E+00_JPRB ,&
&  0.55704E+00_JPRB, 0.54198E+00_JPRB, 0.54838E+00_JPRB, 0.55057E+00_JPRB, 0.55249E+00_JPRB, 0.55619E+00_JPRB, 0.56215E+00_JPRB ,&
&  0.57447E+00_JPRB, 0.61339E+00_JPRB, 0.61500E+00_JPRB, 0.62644E+00_JPRB, 0.66653E+00_JPRB, 0.68720E+00_JPRB, 0.66059E+00_JPRB ,&
&  0.63616E+00_JPRB, 0.61091E+00_JPRB, 0.59321E+00_JPRB, 0.57893E+00_JPRB, 0.59314E+00_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.58221E+00_JPRB, 0.57269E+00_JPRB, 0.55385E+00_JPRB, 0.53384E+00_JPRB, 0.52030E+00_JPRB, 0.50995E+00_JPRB, 0.50281E+00_JPRB ,&
&  0.49813E+00_JPRB, 0.48942E+00_JPRB, 0.48129E+00_JPRB, 0.47074E+00_JPRB, 0.45801E+00_JPRB, 0.44739E+00_JPRB, 0.44053E+00_JPRB ,&
&  0.43169E+00_JPRB, 0.42662E+00_JPRB, 0.42109E+00_JPRB, 0.41116E+00_JPRB, 0.40055E+00_JPRB, 0.39197E+00_JPRB, 0.38661E+00_JPRB ,&
&  0.38448E+00_JPRB, 0.38728E+00_JPRB, 0.39371E+00_JPRB, 0.40686E+00_JPRB, 0.41917E+00_JPRB, 0.43227E+00_JPRB, 0.45329E+00_JPRB ,&
&  0.52679E+00_JPRB, 0.74056E+00_JPRB, 0.93074E+00_JPRB, 0.85026E+00_JPRB, 0.81439E+00_JPRB, 0.77143E+00_JPRB, 0.87224E+00_JPRB ,&
&  0.95725E+00_JPRB, 0.10509E+01_JPRB, 0.11060E+01_JPRB, 0.11257E+01_JPRB, 0.10662E+01_JPRB, 0.10175E+01_JPRB, 0.96698E+00_JPRB ,&
&  0.91386E+00_JPRB, 0.85554E+00_JPRB, 0.78041E+00_JPRB, 0.70861E+00_JPRB, 0.66725E+00_JPRB, 0.66350E+00_JPRB, 0.68829E+00_JPRB ,&
&  0.73460E+00_JPRB, 0.77403E+00_JPRB, 0.78358E+00_JPRB, 0.82581E+00_JPRB, 0.88175E+00_JPRB, 0.89072E+00_JPRB, 0.10736E+01_JPRB ,&
&  0.13455E+01_JPRB, 0.12411E+01_JPRB, 0.10867E+01_JPRB, 0.91398E+00_JPRB, 0.78091E+00_JPRB, 0.67708E+00_JPRB, 0.61830E+00_JPRB ,&
&  0.57855E+00_JPRB, 0.55287E+00_JPRB, 0.53674E+00_JPRB, 0.53082E+00_JPRB, 0.54076E+00_JPRB, 0.55908E+00_JPRB, 0.57383E+00_JPRB ,&
&  0.59138E+00_JPRB, 0.60748E+00_JPRB, 0.61469E+00_JPRB, 0.61039E+00_JPRB, 0.59581E+00_JPRB, 0.57412E+00_JPRB, 0.55377E+00_JPRB ,&
&  0.54480E+00_JPRB, 0.54757E+00_JPRB, 0.57904E+00_JPRB, 0.56484E+00_JPRB, 0.56765E+00_JPRB, 0.58113E+00_JPRB, 0.58594E+00_JPRB ,&
&  0.58786E+00_JPRB, 0.60653E+00_JPRB, 0.62078E+00_JPRB, 0.62902E+00_JPRB, 0.68580E+00_JPRB, 0.73483E+00_JPRB, 0.69150E+00_JPRB ,&
&  0.64852E+00_JPRB, 0.61599E+00_JPRB, 0.60019E+00_JPRB, 0.58347E+00_JPRB, 0.58122E+00_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.58966E+00_JPRB, 0.58313E+00_JPRB, 0.56814E+00_JPRB, 0.55004E+00_JPRB, 0.53639E+00_JPRB, 0.52286E+00_JPRB, 0.51296E+00_JPRB ,&
&  0.50536E+00_JPRB, 0.49872E+00_JPRB, 0.49352E+00_JPRB, 0.48721E+00_JPRB, 0.47886E+00_JPRB, 0.46759E+00_JPRB, 0.45690E+00_JPRB ,&
&  0.44559E+00_JPRB, 0.43446E+00_JPRB, 0.42348E+00_JPRB, 0.41233E+00_JPRB, 0.40382E+00_JPRB, 0.39822E+00_JPRB, 0.39469E+00_JPRB ,&
&  0.39461E+00_JPRB, 0.39816E+00_JPRB, 0.40689E+00_JPRB, 0.42420E+00_JPRB, 0.44406E+00_JPRB, 0.45717E+00_JPRB, 0.47039E+00_JPRB ,&
&  0.53067E+00_JPRB, 0.47759E+00_JPRB, 0.82875E+00_JPRB, 0.95008E+00_JPRB, 0.93692E+00_JPRB, 0.95512E+00_JPRB, 0.11177E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.12608E+01_JPRB, 0.12617E+01_JPRB, 0.12011E+01_JPRB, 0.10772E+01_JPRB, 0.98174E+00_JPRB, 0.90742E+00_JPRB ,&
&  0.85060E+00_JPRB, 0.80307E+00_JPRB, 0.74908E+00_JPRB, 0.69620E+00_JPRB, 0.65715E+00_JPRB, 0.64901E+00_JPRB, 0.66571E+00_JPRB ,&
&  0.72982E+00_JPRB, 0.78105E+00_JPRB, 0.82503E+00_JPRB, 0.88442E+00_JPRB, 0.87867E+00_JPRB, 0.97718E+00_JPRB, 0.10838E+01_JPRB ,&
&  0.14005E+01_JPRB, 0.14823E+01_JPRB, 0.11925E+01_JPRB, 0.92875E+00_JPRB, 0.76014E+00_JPRB, 0.67301E+00_JPRB, 0.61266E+00_JPRB ,&
&  0.58191E+00_JPRB, 0.56517E+00_JPRB, 0.55518E+00_JPRB, 0.55316E+00_JPRB, 0.55938E+00_JPRB, 0.56877E+00_JPRB, 0.57405E+00_JPRB ,&
&  0.57985E+00_JPRB, 0.58425E+00_JPRB, 0.58439E+00_JPRB, 0.58209E+00_JPRB, 0.57390E+00_JPRB, 0.56201E+00_JPRB, 0.55717E+00_JPRB ,&
&  0.56334E+00_JPRB, 0.57884E+00_JPRB, 0.59595E+00_JPRB, 0.59308E+00_JPRB, 0.59856E+00_JPRB, 0.60697E+00_JPRB, 0.61292E+00_JPRB ,&
&  0.62540E+00_JPRB, 0.63072E+00_JPRB, 0.63427E+00_JPRB, 0.65772E+00_JPRB, 0.66280E+00_JPRB, 0.71607E+00_JPRB, 0.67261E+00_JPRB ,&
&  0.63725E+00_JPRB, 0.61387E+00_JPRB, 0.60017E+00_JPRB, 0.59118E+00_JPRB, 0.58825E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.60211E+00_JPRB, 0.59505E+00_JPRB, 0.58017E+00_JPRB, 0.56183E+00_JPRB, 0.54303E+00_JPRB, 0.52772E+00_JPRB, 0.52048E+00_JPRB ,&
&  0.51239E+00_JPRB, 0.50379E+00_JPRB, 0.49567E+00_JPRB, 0.49106E+00_JPRB, 0.48860E+00_JPRB, 0.47980E+00_JPRB, 0.46490E+00_JPRB ,&
&  0.44917E+00_JPRB, 0.43434E+00_JPRB, 0.42227E+00_JPRB, 0.41556E+00_JPRB, 0.40847E+00_JPRB, 0.40507E+00_JPRB, 0.40537E+00_JPRB ,&
&  0.40710E+00_JPRB, 0.41379E+00_JPRB, 0.42754E+00_JPRB, 0.44949E+00_JPRB, 0.46788E+00_JPRB, 0.47739E+00_JPRB, 0.48413E+00_JPRB ,&
&  0.53983E+00_JPRB, 0.80594E+00_JPRB, 0.75014E+00_JPRB, 0.11096E+01_JPRB, 0.11853E+01_JPRB, 0.12482E+01_JPRB, 0.13326E+01_JPRB ,&
&  0.13740E+01_JPRB, 0.14192E+01_JPRB, 0.13439E+01_JPRB, 0.12054E+01_JPRB, 0.10419E+01_JPRB, 0.92345E+00_JPRB, 0.82734E+00_JPRB ,&
&  0.75380E+00_JPRB, 0.70095E+00_JPRB, 0.66258E+00_JPRB, 0.63379E+00_JPRB, 0.61527E+00_JPRB, 0.61511E+00_JPRB, 0.64709E+00_JPRB ,&
&  0.71485E+00_JPRB, 0.79554E+00_JPRB, 0.88783E+00_JPRB, 0.98458E+00_JPRB, 0.94166E+00_JPRB, 0.10385E+01_JPRB, 0.12334E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14729E+01_JPRB, 0.11004E+01_JPRB, 0.82988E+00_JPRB, 0.70994E+00_JPRB, 0.65507E+00_JPRB, 0.62067E+00_JPRB ,&
&  0.60098E+00_JPRB, 0.59005E+00_JPRB, 0.58311E+00_JPRB, 0.57922E+00_JPRB, 0.57735E+00_JPRB, 0.57741E+00_JPRB, 0.57597E+00_JPRB ,&
&  0.57555E+00_JPRB, 0.57650E+00_JPRB, 0.57814E+00_JPRB, 0.57932E+00_JPRB, 0.57738E+00_JPRB, 0.57862E+00_JPRB, 0.58723E+00_JPRB ,&
&  0.60176E+00_JPRB, 0.61732E+00_JPRB, 0.62477E+00_JPRB, 0.62218E+00_JPRB, 0.62938E+00_JPRB, 0.63711E+00_JPRB, 0.63362E+00_JPRB ,&
&  0.64843E+00_JPRB, 0.65875E+00_JPRB, 0.66615E+00_JPRB, 0.68162E+00_JPRB, 0.69387E+00_JPRB, 0.76555E+00_JPRB, 0.67482E+00_JPRB ,&
&  0.63857E+00_JPRB, 0.61991E+00_JPRB, 0.61227E+00_JPRB, 0.60358E+00_JPRB, 0.60227E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.60904E+00_JPRB, 0.59367E+00_JPRB, 0.57707E+00_JPRB, 0.55984E+00_JPRB, 0.54101E+00_JPRB, 0.52771E+00_JPRB, 0.52100E+00_JPRB ,&
&  0.51029E+00_JPRB, 0.50085E+00_JPRB, 0.49375E+00_JPRB, 0.49061E+00_JPRB, 0.48787E+00_JPRB, 0.47900E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.44627E+00_JPRB, 0.43492E+00_JPRB, 0.42882E+00_JPRB, 0.42263E+00_JPRB, 0.41603E+00_JPRB, 0.41253E+00_JPRB, 0.41423E+00_JPRB ,&
&  0.42105E+00_JPRB, 0.43167E+00_JPRB, 0.45290E+00_JPRB, 0.47714E+00_JPRB, 0.49769E+00_JPRB, 0.51519E+00_JPRB, 0.52685E+00_JPRB ,&
&  0.62422E+00_JPRB, 0.76201E+00_JPRB, 0.82132E+00_JPRB, 0.14116E+01_JPRB, 0.15028E+01_JPRB, 0.14799E+01_JPRB, 0.14897E+01_JPRB ,&
&  0.14891E+01_JPRB, 0.15949E+01_JPRB, 0.13910E+01_JPRB, 0.11761E+01_JPRB, 0.99007E+00_JPRB, 0.86621E+00_JPRB, 0.77404E+00_JPRB ,&
&  0.70316E+00_JPRB, 0.65678E+00_JPRB, 0.63325E+00_JPRB, 0.62349E+00_JPRB, 0.61971E+00_JPRB, 0.63193E+00_JPRB, 0.67172E+00_JPRB ,&
&  0.73771E+00_JPRB, 0.84129E+00_JPRB, 0.99400E+00_JPRB, 0.10811E+01_JPRB, 0.10543E+01_JPRB, 0.10827E+01_JPRB, 0.11444E+01_JPRB ,&
&  0.13420E+01_JPRB, 0.12603E+01_JPRB, 0.94692E+00_JPRB, 0.77671E+00_JPRB, 0.69407E+00_JPRB, 0.66662E+00_JPRB, 0.63961E+00_JPRB ,&
&  0.62770E+00_JPRB, 0.61695E+00_JPRB, 0.60808E+00_JPRB, 0.59876E+00_JPRB, 0.58925E+00_JPRB, 0.58462E+00_JPRB, 0.58280E+00_JPRB ,&
&  0.58232E+00_JPRB, 0.58508E+00_JPRB, 0.58729E+00_JPRB, 0.58998E+00_JPRB, 0.59442E+00_JPRB, 0.60394E+00_JPRB, 0.61795E+00_JPRB ,&
&  0.63112E+00_JPRB, 0.65200E+00_JPRB, 0.65401E+00_JPRB, 0.64985E+00_JPRB, 0.66695E+00_JPRB, 0.67801E+00_JPRB, 0.66682E+00_JPRB ,&
&  0.67455E+00_JPRB, 0.69638E+00_JPRB, 0.70445E+00_JPRB, 0.69193E+00_JPRB, 0.71357E+00_JPRB, 0.74263E+00_JPRB, 0.67119E+00_JPRB ,&
&  0.64515E+00_JPRB, 0.63213E+00_JPRB, 0.63077E+00_JPRB, 0.62306E+00_JPRB, 0.61608E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.60351E+00_JPRB, 0.58378E+00_JPRB, 0.56834E+00_JPRB, 0.55650E+00_JPRB, 0.54175E+00_JPRB, 0.52680E+00_JPRB, 0.51507E+00_JPRB ,&
&  0.49872E+00_JPRB, 0.48897E+00_JPRB, 0.48215E+00_JPRB, 0.47438E+00_JPRB, 0.46655E+00_JPRB, 0.45901E+00_JPRB, 0.45224E+00_JPRB ,&
&  0.44686E+00_JPRB, 0.44265E+00_JPRB, 0.44013E+00_JPRB, 0.43522E+00_JPRB, 0.43119E+00_JPRB, 0.43081E+00_JPRB, 0.43529E+00_JPRB ,&
&  0.45158E+00_JPRB, 0.47513E+00_JPRB, 0.50376E+00_JPRB, 0.53442E+00_JPRB, 0.56163E+00_JPRB, 0.59796E+00_JPRB, 0.64064E+00_JPRB ,&
&  0.81203E+00_JPRB, 0.87982E+00_JPRB, 0.10283E+01_JPRB, 0.16316E+01_JPRB, 0.16968E+01_JPRB, 0.15365E+01_JPRB, 0.15210E+01_JPRB ,&
&  0.15198E+01_JPRB, 0.14114E+01_JPRB, 0.13151E+01_JPRB, 0.10811E+01_JPRB, 0.90981E+00_JPRB, 0.82189E+00_JPRB, 0.76553E+00_JPRB ,&
&  0.71552E+00_JPRB, 0.68453E+00_JPRB, 0.67614E+00_JPRB, 0.66838E+00_JPRB, 0.66298E+00_JPRB, 0.68897E+00_JPRB, 0.73777E+00_JPRB ,&
&  0.83307E+00_JPRB, 0.98269E+00_JPRB, 0.11945E+01_JPRB, 0.13185E+01_JPRB, 0.11165E+01_JPRB, 0.10704E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.11675E+01_JPRB, 0.10368E+01_JPRB, 0.85503E+00_JPRB, 0.75561E+00_JPRB, 0.69277E+00_JPRB, 0.67985E+00_JPRB, 0.65560E+00_JPRB ,&
&  0.64414E+00_JPRB, 0.63216E+00_JPRB, 0.61996E+00_JPRB, 0.60365E+00_JPRB, 0.59298E+00_JPRB, 0.58384E+00_JPRB, 0.58221E+00_JPRB ,&
&  0.58703E+00_JPRB, 0.59386E+00_JPRB, 0.60087E+00_JPRB, 0.61079E+00_JPRB, 0.62372E+00_JPRB, 0.63437E+00_JPRB, 0.64447E+00_JPRB ,&
&  0.65521E+00_JPRB, 0.68177E+00_JPRB, 0.68229E+00_JPRB, 0.69610E+00_JPRB, 0.72477E+00_JPRB, 0.74165E+00_JPRB, 0.72173E+00_JPRB ,&
&  0.70901E+00_JPRB, 0.72702E+00_JPRB, 0.72455E+00_JPRB, 0.72434E+00_JPRB, 0.73062E+00_JPRB, 0.72839E+00_JPRB, 0.67869E+00_JPRB ,&
&  0.66592E+00_JPRB, 0.66193E+00_JPRB, 0.66223E+00_JPRB, 0.64329E+00_JPRB, 0.62455E+00_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.59756E+00_JPRB, 0.57209E+00_JPRB, 0.55312E+00_JPRB, 0.53484E+00_JPRB, 0.51589E+00_JPRB, 0.49667E+00_JPRB, 0.48172E+00_JPRB ,&
&  0.46437E+00_JPRB, 0.46205E+00_JPRB, 0.45863E+00_JPRB, 0.45352E+00_JPRB, 0.44728E+00_JPRB, 0.44838E+00_JPRB, 0.45578E+00_JPRB ,&
&  0.46203E+00_JPRB, 0.46541E+00_JPRB, 0.47014E+00_JPRB, 0.47335E+00_JPRB, 0.47984E+00_JPRB, 0.49306E+00_JPRB, 0.51312E+00_JPRB ,&
&  0.54077E+00_JPRB, 0.57188E+00_JPRB, 0.61257E+00_JPRB, 0.66214E+00_JPRB, 0.71163E+00_JPRB, 0.78699E+00_JPRB, 0.88339E+00_JPRB ,&
&  0.10960E+01_JPRB, 0.10983E+01_JPRB, 0.70046E+00_JPRB, 0.11594E+01_JPRB, 0.14401E+01_JPRB, 0.13697E+01_JPRB, 0.12834E+01_JPRB ,&
&  0.12744E+01_JPRB, 0.11979E+01_JPRB, 0.11110E+01_JPRB, 0.98509E+00_JPRB, 0.86728E+00_JPRB, 0.83531E+00_JPRB, 0.81388E+00_JPRB ,&
&  0.78204E+00_JPRB, 0.75543E+00_JPRB, 0.74529E+00_JPRB, 0.74297E+00_JPRB, 0.74132E+00_JPRB, 0.76675E+00_JPRB, 0.86324E+00_JPRB ,&
&  0.10066E+01_JPRB, 0.12216E+01_JPRB, 0.14653E+01_JPRB, 0.13995E+01_JPRB, 0.11493E+01_JPRB, 0.10300E+01_JPRB, 0.94948E+00_JPRB ,&
&  0.91676E+00_JPRB, 0.89990E+00_JPRB, 0.81243E+00_JPRB, 0.75315E+00_JPRB, 0.69982E+00_JPRB, 0.66566E+00_JPRB, 0.66545E+00_JPRB ,&
&  0.65008E+00_JPRB, 0.62948E+00_JPRB, 0.61310E+00_JPRB, 0.59020E+00_JPRB, 0.58029E+00_JPRB, 0.57707E+00_JPRB, 0.58441E+00_JPRB ,&
&  0.59587E+00_JPRB, 0.60706E+00_JPRB, 0.62103E+00_JPRB, 0.63727E+00_JPRB, 0.64770E+00_JPRB, 0.65702E+00_JPRB, 0.66476E+00_JPRB ,&
&  0.68060E+00_JPRB, 0.70588E+00_JPRB, 0.73398E+00_JPRB, 0.75344E+00_JPRB, 0.76980E+00_JPRB, 0.78095E+00_JPRB, 0.78326E+00_JPRB ,&
&  0.78974E+00_JPRB, 0.78647E+00_JPRB, 0.72951E+00_JPRB, 0.72276E+00_JPRB, 0.74915E+00_JPRB, 0.74479E+00_JPRB, 0.73625E+00_JPRB ,&
&  0.76569E+00_JPRB, 0.79049E+00_JPRB, 0.78507E+00_JPRB, 0.67206E+00_JPRB, 0.62825E+00_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.56402E+00_JPRB, 0.51969E+00_JPRB, 0.48672E+00_JPRB, 0.46906E+00_JPRB, 0.45718E+00_JPRB, 0.45239E+00_JPRB, 0.45109E+00_JPRB ,&
&  0.45468E+00_JPRB, 0.45832E+00_JPRB, 0.44808E+00_JPRB, 0.45336E+00_JPRB, 0.46642E+00_JPRB, 0.48127E+00_JPRB, 0.50063E+00_JPRB ,&
&  0.51253E+00_JPRB, 0.52372E+00_JPRB, 0.53826E+00_JPRB, 0.56350E+00_JPRB, 0.59609E+00_JPRB, 0.63070E+00_JPRB, 0.66308E+00_JPRB ,&
&  0.69690E+00_JPRB, 0.73535E+00_JPRB, 0.78939E+00_JPRB, 0.86351E+00_JPRB, 0.96763E+00_JPRB, 0.11167E+01_JPRB, 0.12841E+01_JPRB ,&
&  0.14988E+01_JPRB, 0.11382E+01_JPRB, 0.81869E+00_JPRB, 0.13000E+01_JPRB, 0.13687E+01_JPRB, 0.12974E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11021E+01_JPRB, 0.10222E+01_JPRB, 0.96969E+00_JPRB, 0.88629E+00_JPRB, 0.87017E+00_JPRB, 0.85661E+00_JPRB ,&
&  0.83726E+00_JPRB, 0.81826E+00_JPRB, 0.83675E+00_JPRB, 0.85553E+00_JPRB, 0.87412E+00_JPRB, 0.94263E+00_JPRB, 0.10802E+01_JPRB ,&
&  0.12645E+01_JPRB, 0.15071E+01_JPRB, 0.17933E+01_JPRB, 0.15564E+01_JPRB, 0.12589E+01_JPRB, 0.10294E+01_JPRB, 0.92941E+00_JPRB ,&
&  0.86669E+00_JPRB, 0.85297E+00_JPRB, 0.80718E+00_JPRB, 0.75882E+00_JPRB, 0.70796E+00_JPRB, 0.66062E+00_JPRB, 0.64826E+00_JPRB ,&
&  0.62956E+00_JPRB, 0.60353E+00_JPRB, 0.58512E+00_JPRB, 0.56748E+00_JPRB, 0.56676E+00_JPRB, 0.57575E+00_JPRB, 0.58618E+00_JPRB ,&
&  0.59748E+00_JPRB, 0.61159E+00_JPRB, 0.63280E+00_JPRB, 0.65747E+00_JPRB, 0.66888E+00_JPRB, 0.67506E+00_JPRB, 0.68823E+00_JPRB ,&
&  0.70334E+00_JPRB, 0.71746E+00_JPRB, 0.74548E+00_JPRB, 0.76382E+00_JPRB, 0.76978E+00_JPRB, 0.75395E+00_JPRB, 0.76921E+00_JPRB ,&
&  0.76965E+00_JPRB, 0.77170E+00_JPRB, 0.74767E+00_JPRB, 0.75776E+00_JPRB, 0.75055E+00_JPRB, 0.76364E+00_JPRB, 0.78126E+00_JPRB ,&
&  0.79653E+00_JPRB, 0.78925E+00_JPRB, 0.75139E+00_JPRB, 0.63959E+00_JPRB, 0.59825E+00_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.46727E+00_JPRB, 0.44036E+00_JPRB, 0.44318E+00_JPRB, 0.44742E+00_JPRB, 0.46275E+00_JPRB, 0.47813E+00_JPRB, 0.47378E+00_JPRB ,&
&  0.48076E+00_JPRB, 0.49056E+00_JPRB, 0.50481E+00_JPRB, 0.52135E+00_JPRB, 0.54218E+00_JPRB, 0.56418E+00_JPRB, 0.58229E+00_JPRB ,&
&  0.60995E+00_JPRB, 0.64473E+00_JPRB, 0.68565E+00_JPRB, 0.72607E+00_JPRB, 0.76692E+00_JPRB, 0.79830E+00_JPRB, 0.82796E+00_JPRB ,&
&  0.86620E+00_JPRB, 0.92899E+00_JPRB, 0.10181E+01_JPRB, 0.11269E+01_JPRB, 0.12648E+01_JPRB, 0.14288E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.11432E+01_JPRB, 0.65180E+00_JPRB, 0.12917E+01_JPRB, 0.14283E+01_JPRB, 0.12517E+01_JPRB, 0.11707E+01_JPRB, 0.10806E+01_JPRB ,&
&  0.10168E+01_JPRB, 0.10044E+01_JPRB, 0.94969E+00_JPRB, 0.95321E+00_JPRB, 0.90572E+00_JPRB, 0.90027E+00_JPRB, 0.90425E+00_JPRB ,&
&  0.90683E+00_JPRB, 0.92591E+00_JPRB, 0.97944E+00_JPRB, 0.10174E+01_JPRB, 0.10760E+01_JPRB, 0.11821E+01_JPRB, 0.13425E+01_JPRB ,&
&  0.15710E+01_JPRB, 0.18495E+01_JPRB, 0.21346E+01_JPRB, 0.17116E+01_JPRB, 0.14068E+01_JPRB, 0.11985E+01_JPRB, 0.10438E+01_JPRB ,&
&  0.95436E+00_JPRB, 0.82517E+00_JPRB, 0.79058E+00_JPRB, 0.76268E+00_JPRB, 0.71197E+00_JPRB, 0.63707E+00_JPRB, 0.61905E+00_JPRB ,&
&  0.58499E+00_JPRB, 0.56316E+00_JPRB, 0.55121E+00_JPRB, 0.54293E+00_JPRB, 0.54239E+00_JPRB, 0.55261E+00_JPRB, 0.56489E+00_JPRB ,&
&  0.58259E+00_JPRB, 0.59473E+00_JPRB, 0.61509E+00_JPRB, 0.63857E+00_JPRB, 0.65465E+00_JPRB, 0.67081E+00_JPRB, 0.69426E+00_JPRB ,&
&  0.70402E+00_JPRB, 0.72118E+00_JPRB, 0.73726E+00_JPRB, 0.74241E+00_JPRB, 0.74616E+00_JPRB, 0.73340E+00_JPRB, 0.73444E+00_JPRB ,&
&  0.73263E+00_JPRB, 0.72662E+00_JPRB, 0.70930E+00_JPRB, 0.71247E+00_JPRB, 0.71208E+00_JPRB, 0.70765E+00_JPRB, 0.72114E+00_JPRB ,&
&  0.71109E+00_JPRB, 0.67980E+00_JPRB, 0.61570E+00_JPRB, 0.54485E+00_JPRB, 0.50876E+00_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.44574E+00_JPRB, 0.46379E+00_JPRB, 0.48530E+00_JPRB, 0.50589E+00_JPRB, 0.52468E+00_JPRB, 0.54183E+00_JPRB, 0.56882E+00_JPRB ,&
&  0.59888E+00_JPRB, 0.61937E+00_JPRB, 0.62835E+00_JPRB, 0.64775E+00_JPRB, 0.67041E+00_JPRB, 0.69993E+00_JPRB, 0.72486E+00_JPRB ,&
&  0.75452E+00_JPRB, 0.79507E+00_JPRB, 0.83038E+00_JPRB, 0.86740E+00_JPRB, 0.90326E+00_JPRB, 0.94002E+00_JPRB, 0.99375E+00_JPRB ,&
&  0.10631E+01_JPRB, 0.11552E+01_JPRB, 0.12645E+01_JPRB, 0.13677E+01_JPRB, 0.14576E+01_JPRB, 0.15533E+01_JPRB, 0.15799E+01_JPRB ,&
&  0.99610E+00_JPRB, 0.12462E+01_JPRB, 0.14033E+01_JPRB, 0.12766E+01_JPRB, 0.11731E+01_JPRB, 0.11151E+01_JPRB, 0.10872E+01_JPRB ,&
&  0.10362E+01_JPRB, 0.96233E+00_JPRB, 0.93677E+00_JPRB, 0.97902E+00_JPRB, 0.92628E+00_JPRB, 0.93818E+00_JPRB, 0.96647E+00_JPRB ,&
&  0.10133E+01_JPRB, 0.10745E+01_JPRB, 0.11366E+01_JPRB, 0.12157E+01_JPRB, 0.13295E+01_JPRB, 0.14719E+01_JPRB, 0.16557E+01_JPRB ,&
&  0.18866E+01_JPRB, 0.21441E+01_JPRB, 0.23836E+01_JPRB, 0.19484E+01_JPRB, 0.17969E+01_JPRB, 0.17232E+01_JPRB, 0.15230E+01_JPRB ,&
&  0.10496E+01_JPRB, 0.83280E+00_JPRB, 0.82348E+00_JPRB, 0.78493E+00_JPRB, 0.69656E+00_JPRB, 0.62477E+00_JPRB, 0.56673E+00_JPRB ,&
&  0.53214E+00_JPRB, 0.50485E+00_JPRB, 0.49181E+00_JPRB, 0.48609E+00_JPRB, 0.48740E+00_JPRB, 0.50367E+00_JPRB, 0.52459E+00_JPRB ,&
&  0.54413E+00_JPRB, 0.54967E+00_JPRB, 0.57477E+00_JPRB, 0.58976E+00_JPRB, 0.63127E+00_JPRB, 0.66863E+00_JPRB, 0.69055E+00_JPRB ,&
&  0.70064E+00_JPRB, 0.69519E+00_JPRB, 0.69129E+00_JPRB, 0.68648E+00_JPRB, 0.68334E+00_JPRB, 0.67930E+00_JPRB, 0.67265E+00_JPRB ,&
&  0.66125E+00_JPRB, 0.66941E+00_JPRB, 0.65522E+00_JPRB, 0.64488E+00_JPRB, 0.64748E+00_JPRB, 0.61913E+00_JPRB, 0.60261E+00_JPRB ,&
&  0.58094E+00_JPRB, 0.53196E+00_JPRB, 0.49666E+00_JPRB, 0.47081E+00_JPRB, 0.45235E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.52941E+00_JPRB, 0.57637E+00_JPRB, 0.59664E+00_JPRB, 0.62187E+00_JPRB, 0.64066E+00_JPRB, 0.66781E+00_JPRB, 0.69219E+00_JPRB ,&
&  0.71499E+00_JPRB, 0.74188E+00_JPRB, 0.75788E+00_JPRB, 0.78663E+00_JPRB, 0.81031E+00_JPRB, 0.83140E+00_JPRB, 0.85111E+00_JPRB ,&
&  0.87197E+00_JPRB, 0.90676E+00_JPRB, 0.94361E+00_JPRB, 0.98588E+00_JPRB, 0.10279E+01_JPRB, 0.10835E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.12541E+01_JPRB, 0.13436E+01_JPRB, 0.14221E+01_JPRB, 0.14853E+01_JPRB, 0.15135E+01_JPRB, 0.15699E+01_JPRB, 0.15627E+01_JPRB ,&
&  0.11130E+01_JPRB, 0.13169E+01_JPRB, 0.12744E+01_JPRB, 0.12145E+01_JPRB, 0.11695E+01_JPRB, 0.11271E+01_JPRB, 0.11058E+01_JPRB ,&
&  0.10782E+01_JPRB, 0.10005E+01_JPRB, 0.97385E+00_JPRB, 0.94696E+00_JPRB, 0.97017E+00_JPRB, 0.99611E+00_JPRB, 0.10457E+01_JPRB ,&
&  0.11151E+01_JPRB, 0.12054E+01_JPRB, 0.13029E+01_JPRB, 0.14064E+01_JPRB, 0.15478E+01_JPRB, 0.17284E+01_JPRB, 0.18890E+01_JPRB ,&
&  0.20565E+01_JPRB, 0.22304E+01_JPRB, 0.23344E+01_JPRB, 0.19969E+01_JPRB, 0.16925E+01_JPRB, 0.15483E+01_JPRB, 0.12252E+01_JPRB ,&
&  0.93748E+00_JPRB, 0.82524E+00_JPRB, 0.84101E+00_JPRB, 0.80506E+00_JPRB, 0.68710E+00_JPRB, 0.58782E+00_JPRB, 0.51543E+00_JPRB ,&
&  0.48672E+00_JPRB, 0.45376E+00_JPRB, 0.44153E+00_JPRB, 0.44343E+00_JPRB, 0.45098E+00_JPRB, 0.46202E+00_JPRB, 0.48916E+00_JPRB ,&
&  0.50856E+00_JPRB, 0.52768E+00_JPRB, 0.55496E+00_JPRB, 0.58485E+00_JPRB, 0.63857E+00_JPRB, 0.71910E+00_JPRB, 0.72626E+00_JPRB ,&
&  0.69686E+00_JPRB, 0.66344E+00_JPRB, 0.63270E+00_JPRB, 0.62849E+00_JPRB, 0.62019E+00_JPRB, 0.61906E+00_JPRB, 0.63337E+00_JPRB ,&
&  0.65578E+00_JPRB, 0.67229E+00_JPRB, 0.63701E+00_JPRB, 0.60960E+00_JPRB, 0.61045E+00_JPRB, 0.57970E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.50486E+00_JPRB, 0.48707E+00_JPRB, 0.47002E+00_JPRB, 0.47771E+00_JPRB, 0.50061E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.69273E+00_JPRB, 0.72168E+00_JPRB, 0.74355E+00_JPRB, 0.74736E+00_JPRB, 0.77784E+00_JPRB, 0.80373E+00_JPRB, 0.82028E+00_JPRB ,&
&  0.83490E+00_JPRB, 0.85915E+00_JPRB, 0.87586E+00_JPRB, 0.90544E+00_JPRB, 0.92867E+00_JPRB, 0.95277E+00_JPRB, 0.97467E+00_JPRB ,&
&  0.99750E+00_JPRB, 0.10356E+01_JPRB, 0.10770E+01_JPRB, 0.11343E+01_JPRB, 0.11916E+01_JPRB, 0.12631E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.13733E+01_JPRB, 0.14270E+01_JPRB, 0.14925E+01_JPRB, 0.15322E+01_JPRB, 0.15127E+01_JPRB, 0.14841E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.12458E+01_JPRB, 0.12577E+01_JPRB, 0.12428E+01_JPRB, 0.12117E+01_JPRB, 0.11695E+01_JPRB, 0.11155E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10176E+01_JPRB, 0.97440E+00_JPRB, 0.94336E+00_JPRB, 0.10183E+01_JPRB, 0.10392E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.12566E+01_JPRB, 0.13655E+01_JPRB, 0.15170E+01_JPRB, 0.16821E+01_JPRB, 0.18467E+01_JPRB, 0.20139E+01_JPRB ,&
&  0.21353E+01_JPRB, 0.21960E+01_JPRB, 0.22210E+01_JPRB, 0.18144E+01_JPRB, 0.15688E+01_JPRB, 0.13147E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.91891E+00_JPRB, 0.87455E+00_JPRB, 0.89182E+00_JPRB, 0.83777E+00_JPRB, 0.70533E+00_JPRB, 0.58764E+00_JPRB, 0.50536E+00_JPRB ,&
&  0.47148E+00_JPRB, 0.43517E+00_JPRB, 0.43037E+00_JPRB, 0.43180E+00_JPRB, 0.44199E+00_JPRB, 0.45103E+00_JPRB, 0.46587E+00_JPRB ,&
&  0.48604E+00_JPRB, 0.51872E+00_JPRB, 0.57651E+00_JPRB, 0.66388E+00_JPRB, 0.78097E+00_JPRB, 0.90688E+00_JPRB, 0.87136E+00_JPRB ,&
&  0.72114E+00_JPRB, 0.63639E+00_JPRB, 0.62913E+00_JPRB, 0.63212E+00_JPRB, 0.66127E+00_JPRB, 0.71944E+00_JPRB, 0.79236E+00_JPRB ,&
&  0.91825E+00_JPRB, 0.99309E+00_JPRB, 0.90030E+00_JPRB, 0.75366E+00_JPRB, 0.10280E+01_JPRB, 0.10123E+01_JPRB, 0.54603E+00_JPRB ,&
&  0.59549E+00_JPRB, 0.58538E+00_JPRB, 0.61027E+00_JPRB, 0.63990E+00_JPRB, 0.67686E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.84736E+00_JPRB, 0.87541E+00_JPRB, 0.89027E+00_JPRB, 0.89222E+00_JPRB, 0.91065E+00_JPRB, 0.94233E+00_JPRB, 0.95774E+00_JPRB ,&
&  0.97490E+00_JPRB, 0.98149E+00_JPRB, 0.10017E+01_JPRB, 0.10227E+01_JPRB, 0.10454E+01_JPRB, 0.10702E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.11199E+01_JPRB, 0.11671E+01_JPRB, 0.11949E+01_JPRB, 0.12467E+01_JPRB, 0.13021E+01_JPRB, 0.13762E+01_JPRB, 0.14179E+01_JPRB ,&
&  0.14254E+01_JPRB, 0.14649E+01_JPRB, 0.15088E+01_JPRB, 0.15614E+01_JPRB, 0.15544E+01_JPRB, 0.15670E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.12634E+01_JPRB, 0.12005E+01_JPRB, 0.11726E+01_JPRB, 0.11380E+01_JPRB, 0.11029E+01_JPRB, 0.10682E+01_JPRB ,&
&  0.10564E+01_JPRB, 0.10333E+01_JPRB, 0.98688E+00_JPRB, 0.10157E+01_JPRB, 0.10329E+01_JPRB, 0.10534E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.11913E+01_JPRB, 0.12957E+01_JPRB, 0.14241E+01_JPRB, 0.15791E+01_JPRB, 0.17434E+01_JPRB, 0.19034E+01_JPRB, 0.20574E+01_JPRB ,&
&  0.22012E+01_JPRB, 0.23140E+01_JPRB, 0.23216E+01_JPRB, 0.19456E+01_JPRB, 0.16959E+01_JPRB, 0.13085E+01_JPRB, 0.10783E+01_JPRB ,&
&  0.93758E+00_JPRB, 0.87891E+00_JPRB, 0.88269E+00_JPRB, 0.88433E+00_JPRB, 0.72441E+00_JPRB, 0.62322E+00_JPRB, 0.52865E+00_JPRB ,&
&  0.49140E+00_JPRB, 0.45772E+00_JPRB, 0.45332E+00_JPRB, 0.44940E+00_JPRB, 0.44631E+00_JPRB, 0.44611E+00_JPRB, 0.45998E+00_JPRB ,&
&  0.46889E+00_JPRB, 0.49671E+00_JPRB, 0.55610E+00_JPRB, 0.62983E+00_JPRB, 0.69666E+00_JPRB, 0.73094E+00_JPRB, 0.67759E+00_JPRB ,&
&  0.68331E+00_JPRB, 0.65025E+00_JPRB, 0.64206E+00_JPRB, 0.68974E+00_JPRB, 0.78016E+00_JPRB, 0.94780E+00_JPRB, 0.11548E+01_JPRB ,&
&  0.14096E+01_JPRB, 0.14639E+01_JPRB, 0.17656E+01_JPRB, 0.23190E+01_JPRB, 0.20960E+01_JPRB, 0.17858E+01_JPRB, 0.67454E+00_JPRB ,&
&  0.71278E+00_JPRB, 0.72432E+00_JPRB, 0.75713E+00_JPRB, 0.78939E+00_JPRB, 0.82964E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.86959E+00_JPRB, 0.89464E+00_JPRB, 0.91401E+00_JPRB, 0.92899E+00_JPRB, 0.95110E+00_JPRB, 0.97433E+00_JPRB, 0.99710E+00_JPRB ,&
&  0.10182E+01_JPRB, 0.10371E+01_JPRB, 0.10601E+01_JPRB, 0.10825E+01_JPRB, 0.11086E+01_JPRB, 0.11368E+01_JPRB, 0.11691E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.12273E+01_JPRB, 0.12473E+01_JPRB, 0.12957E+01_JPRB, 0.13555E+01_JPRB, 0.13920E+01_JPRB, 0.14140E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.14448E+01_JPRB, 0.14750E+01_JPRB, 0.15074E+01_JPRB, 0.15161E+01_JPRB, 0.15941E+01_JPRB, 0.14103E+01_JPRB ,&
&  0.14055E+01_JPRB, 0.12763E+01_JPRB, 0.11394E+01_JPRB, 0.11078E+01_JPRB, 0.10851E+01_JPRB, 0.10748E+01_JPRB, 0.10630E+01_JPRB ,&
&  0.10492E+01_JPRB, 0.10401E+01_JPRB, 0.10162E+01_JPRB, 0.10024E+01_JPRB, 0.10119E+01_JPRB, 0.10456E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.13097E+01_JPRB, 0.14455E+01_JPRB, 0.15954E+01_JPRB, 0.17619E+01_JPRB, 0.18985E+01_JPRB, 0.20219E+01_JPRB ,&
&  0.21152E+01_JPRB, 0.21668E+01_JPRB, 0.19982E+01_JPRB, 0.19616E+01_JPRB, 0.18841E+01_JPRB, 0.14178E+01_JPRB, 0.11236E+01_JPRB ,&
&  0.87622E+00_JPRB, 0.86935E+00_JPRB, 0.92524E+00_JPRB, 0.95528E+00_JPRB, 0.80051E+00_JPRB, 0.70954E+00_JPRB, 0.59947E+00_JPRB ,&
&  0.51472E+00_JPRB, 0.47883E+00_JPRB, 0.47896E+00_JPRB, 0.52370E+00_JPRB, 0.50508E+00_JPRB, 0.47851E+00_JPRB, 0.49221E+00_JPRB ,&
&  0.48460E+00_JPRB, 0.49690E+00_JPRB, 0.52415E+00_JPRB, 0.57500E+00_JPRB, 0.62734E+00_JPRB, 0.62355E+00_JPRB, 0.67047E+00_JPRB ,&
&  0.63464E+00_JPRB, 0.62443E+00_JPRB, 0.61353E+00_JPRB, 0.52660E+00_JPRB, 0.70449E+00_JPRB, 0.80099E+00_JPRB, 0.93312E+00_JPRB ,&
&  0.10704E+01_JPRB, 0.14685E+01_JPRB, 0.17183E+01_JPRB, 0.13442E+01_JPRB, 0.10379E+01_JPRB, 0.75004E+00_JPRB, 0.71913E+00_JPRB ,&
&  0.74138E+00_JPRB, 0.74973E+00_JPRB, 0.77394E+00_JPRB, 0.80484E+00_JPRB, 0.83837E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.86815E+00_JPRB, 0.89544E+00_JPRB, 0.91844E+00_JPRB, 0.93799E+00_JPRB, 0.96196E+00_JPRB, 0.98792E+00_JPRB, 0.10115E+01_JPRB ,&
&  0.10328E+01_JPRB, 0.10523E+01_JPRB, 0.10750E+01_JPRB, 0.10971E+01_JPRB, 0.11212E+01_JPRB, 0.11488E+01_JPRB, 0.11782E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.12242E+01_JPRB, 0.12601E+01_JPRB, 0.13035E+01_JPRB, 0.13333E+01_JPRB, 0.13454E+01_JPRB, 0.13488E+01_JPRB ,&
&  0.13527E+01_JPRB, 0.13790E+01_JPRB, 0.13968E+01_JPRB, 0.14145E+01_JPRB, 0.14449E+01_JPRB, 0.15379E+01_JPRB, 0.13346E+01_JPRB ,&
&  0.14104E+01_JPRB, 0.12759E+01_JPRB, 0.10836E+01_JPRB, 0.10187E+01_JPRB, 0.10022E+01_JPRB, 0.98871E+00_JPRB, 0.98409E+00_JPRB ,&
&  0.99910E+00_JPRB, 0.96560E+00_JPRB, 0.95434E+00_JPRB, 0.94791E+00_JPRB, 0.97070E+00_JPRB, 0.10093E+01_JPRB, 0.10528E+01_JPRB ,&
&  0.11283E+01_JPRB, 0.12302E+01_JPRB, 0.13502E+01_JPRB, 0.14827E+01_JPRB, 0.16375E+01_JPRB, 0.17506E+01_JPRB, 0.18131E+01_JPRB ,&
&  0.18133E+01_JPRB, 0.17897E+01_JPRB, 0.15417E+01_JPRB, 0.16035E+01_JPRB, 0.16035E+01_JPRB, 0.13929E+01_JPRB, 0.12127E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.93360E+00_JPRB, 0.10128E+01_JPRB, 0.11105E+01_JPRB, 0.10274E+01_JPRB, 0.88807E+00_JPRB, 0.75910E+00_JPRB ,&
&  0.61124E+00_JPRB, 0.56349E+00_JPRB, 0.55905E+00_JPRB, 0.64075E+00_JPRB, 0.63474E+00_JPRB, 0.56903E+00_JPRB, 0.56103E+00_JPRB ,&
&  0.53487E+00_JPRB, 0.50364E+00_JPRB, 0.50741E+00_JPRB, 0.58404E+00_JPRB, 0.72115E+00_JPRB, 0.72857E+00_JPRB, 0.78905E+00_JPRB ,&
&  0.58794E+00_JPRB, 0.52377E+00_JPRB, 0.52743E+00_JPRB, 0.55227E+00_JPRB, 0.59159E+00_JPRB, 0.63017E+00_JPRB, 0.73287E+00_JPRB ,&
&  0.89686E+00_JPRB, 0.11090E+01_JPRB, 0.10831E+01_JPRB, 0.87706E+00_JPRB, 0.72143E+00_JPRB, 0.69282E+00_JPRB, 0.71181E+00_JPRB ,&
&  0.73598E+00_JPRB, 0.75646E+00_JPRB, 0.78113E+00_JPRB, 0.80839E+00_JPRB, 0.83740E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.84618E+00_JPRB, 0.87335E+00_JPRB, 0.89730E+00_JPRB, 0.91964E+00_JPRB, 0.94359E+00_JPRB, 0.96845E+00_JPRB, 0.99027E+00_JPRB ,&
&  0.10093E+01_JPRB, 0.10306E+01_JPRB, 0.10495E+01_JPRB, 0.10684E+01_JPRB, 0.10952E+01_JPRB, 0.11204E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11587E+01_JPRB, 0.11894E+01_JPRB, 0.12238E+01_JPRB, 0.12549E+01_JPRB, 0.12704E+01_JPRB, 0.12785E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.12563E+01_JPRB, 0.12628E+01_JPRB, 0.12798E+01_JPRB, 0.13123E+01_JPRB, 0.12992E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.13135E+01_JPRB, 0.10638E+01_JPRB, 0.92998E+00_JPRB, 0.93468E+00_JPRB, 0.87950E+00_JPRB, 0.87663E+00_JPRB ,&
&  0.90337E+00_JPRB, 0.90396E+00_JPRB, 0.90300E+00_JPRB, 0.89951E+00_JPRB, 0.92435E+00_JPRB, 0.97320E+00_JPRB, 0.99297E+00_JPRB ,&
&  0.10441E+01_JPRB, 0.11086E+01_JPRB, 0.11522E+01_JPRB, 0.12239E+01_JPRB, 0.13583E+01_JPRB, 0.14640E+01_JPRB, 0.15044E+01_JPRB ,&
&  0.14759E+01_JPRB, 0.14146E+01_JPRB, 0.11211E+01_JPRB, 0.11565E+01_JPRB, 0.12189E+01_JPRB, 0.11595E+01_JPRB, 0.10921E+01_JPRB ,&
&  0.11847E+01_JPRB, 0.11316E+01_JPRB, 0.13376E+01_JPRB, 0.14259E+01_JPRB, 0.13829E+01_JPRB, 0.12452E+01_JPRB, 0.10508E+01_JPRB ,&
&  0.87536E+00_JPRB, 0.74322E+00_JPRB, 0.76204E+00_JPRB, 0.86560E+00_JPRB, 0.84382E+00_JPRB, 0.72012E+00_JPRB, 0.67824E+00_JPRB ,&
&  0.62429E+00_JPRB, 0.57234E+00_JPRB, 0.56394E+00_JPRB, 0.63973E+00_JPRB, 0.81999E+00_JPRB, 0.13731E+01_JPRB, 0.14819E+01_JPRB ,&
&  0.65101E+00_JPRB, 0.61196E+00_JPRB, 0.44820E+00_JPRB, 0.50820E+00_JPRB, 0.51592E+00_JPRB, 0.48637E+00_JPRB, 0.52785E+00_JPRB ,&
&  0.64134E+00_JPRB, 0.74061E+00_JPRB, 0.75358E+00_JPRB, 0.69490E+00_JPRB, 0.67640E+00_JPRB, 0.68676E+00_JPRB, 0.70830E+00_JPRB ,&
&  0.73064E+00_JPRB, 0.75166E+00_JPRB, 0.77279E+00_JPRB, 0.79575E+00_JPRB, 0.81966E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.79635E+00_JPRB, 0.83002E+00_JPRB, 0.85405E+00_JPRB, 0.87388E+00_JPRB, 0.88767E+00_JPRB, 0.90340E+00_JPRB, 0.92645E+00_JPRB ,&
&  0.94662E+00_JPRB, 0.96701E+00_JPRB, 0.98719E+00_JPRB, 0.10143E+01_JPRB, 0.10486E+01_JPRB, 0.10773E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11325E+01_JPRB, 0.11542E+01_JPRB, 0.11993E+01_JPRB, 0.12167E+01_JPRB, 0.12476E+01_JPRB, 0.12542E+01_JPRB ,&
&  0.12503E+01_JPRB, 0.12518E+01_JPRB, 0.12590E+01_JPRB, 0.12788E+01_JPRB, 0.12960E+01_JPRB, 0.14135E+01_JPRB, 0.15818E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.13246E+01_JPRB, 0.10483E+01_JPRB, 0.84753E+00_JPRB, 0.84214E+00_JPRB, 0.84562E+00_JPRB, 0.82801E+00_JPRB ,&
&  0.83409E+00_JPRB, 0.85075E+00_JPRB, 0.86773E+00_JPRB, 0.85102E+00_JPRB, 0.87998E+00_JPRB, 0.95040E+00_JPRB, 0.94763E+00_JPRB ,&
&  0.97225E+00_JPRB, 0.99887E+00_JPRB, 0.10076E+01_JPRB, 0.11142E+01_JPRB, 0.10991E+01_JPRB, 0.12410E+01_JPRB, 0.13206E+01_JPRB ,&
&  0.13274E+01_JPRB, 0.12653E+01_JPRB, 0.10571E+01_JPRB, 0.10580E+01_JPRB, 0.11245E+01_JPRB, 0.10911E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.12702E+01_JPRB, 0.13921E+01_JPRB, 0.12764E+01_JPRB, 0.16562E+01_JPRB, 0.17964E+01_JPRB, 0.17660E+01_JPRB, 0.15688E+01_JPRB ,&
&  0.13786E+01_JPRB, 0.11040E+01_JPRB, 0.10963E+01_JPRB, 0.11090E+01_JPRB, 0.10576E+01_JPRB, 0.98907E+00_JPRB, 0.88810E+00_JPRB ,&
&  0.78938E+00_JPRB, 0.71490E+00_JPRB, 0.68308E+00_JPRB, 0.71171E+00_JPRB, 0.96059E+00_JPRB, 0.11196E+01_JPRB, 0.15108E+01_JPRB ,&
&  0.74680E+00_JPRB, 0.74228E+00_JPRB, 0.65497E+00_JPRB, 0.86951E+00_JPRB, 0.69950E+00_JPRB, 0.48167E+00_JPRB, 0.46040E+00_JPRB ,&
&  0.51025E+00_JPRB, 0.55611E+00_JPRB, 0.59043E+00_JPRB, 0.61529E+00_JPRB, 0.63781E+00_JPRB, 0.66291E+00_JPRB, 0.67494E+00_JPRB ,&
&  0.69189E+00_JPRB, 0.71135E+00_JPRB, 0.72996E+00_JPRB, 0.75750E+00_JPRB, 0.77327E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.67936E+00_JPRB, 0.69276E+00_JPRB, 0.71148E+00_JPRB, 0.72312E+00_JPRB, 0.73774E+00_JPRB, 0.75014E+00_JPRB, 0.78305E+00_JPRB ,&
&  0.80959E+00_JPRB, 0.83451E+00_JPRB, 0.86815E+00_JPRB, 0.90372E+00_JPRB, 0.91653E+00_JPRB, 0.92851E+00_JPRB, 0.93540E+00_JPRB ,&
&  0.95395E+00_JPRB, 0.96926E+00_JPRB, 0.96313E+00_JPRB, 0.96494E+00_JPRB, 0.96793E+00_JPRB, 0.10366E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.10285E+01_JPRB, 0.10453E+01_JPRB, 0.10636E+01_JPRB, 0.11291E+01_JPRB, 0.13212E+01_JPRB, 0.14853E+01_JPRB, 0.15023E+01_JPRB ,&
&  0.15808E+01_JPRB, 0.14451E+01_JPRB, 0.14131E+01_JPRB, 0.10990E+01_JPRB, 0.10265E+01_JPRB, 0.10261E+01_JPRB, 0.96280E+00_JPRB ,&
&  0.94498E+00_JPRB, 0.96266E+00_JPRB, 0.10107E+01_JPRB, 0.98949E+00_JPRB, 0.10011E+01_JPRB, 0.99259E+00_JPRB, 0.95655E+00_JPRB ,&
&  0.95111E+00_JPRB, 0.97199E+00_JPRB, 0.10098E+01_JPRB, 0.11385E+01_JPRB, 0.12240E+01_JPRB, 0.13669E+01_JPRB, 0.14231E+01_JPRB ,&
&  0.14247E+01_JPRB, 0.14737E+01_JPRB, 0.14229E+01_JPRB, 0.14832E+01_JPRB, 0.16361E+01_JPRB, 0.16329E+01_JPRB, 0.15141E+01_JPRB ,&
&  0.14981E+01_JPRB, 0.14312E+01_JPRB, 0.11449E+01_JPRB, 0.17711E+01_JPRB, 0.20515E+01_JPRB, 0.19353E+01_JPRB, 0.19077E+01_JPRB ,&
&  0.17757E+01_JPRB, 0.15946E+01_JPRB, 0.14577E+01_JPRB, 0.13976E+01_JPRB, 0.12498E+01_JPRB, 0.11312E+01_JPRB, 0.10986E+01_JPRB ,&
&  0.95671E+00_JPRB, 0.79461E+00_JPRB, 0.69064E+00_JPRB, 0.68850E+00_JPRB, 0.74871E+00_JPRB, 0.98123E+00_JPRB, 0.11135E+01_JPRB ,&
&  0.89335E+00_JPRB, 0.82149E+00_JPRB, 0.12315E+01_JPRB, 0.18984E+01_JPRB, 0.10023E+01_JPRB, 0.61893E+00_JPRB, 0.44044E+00_JPRB ,&
&  0.44203E+00_JPRB, 0.46570E+00_JPRB, 0.49042E+00_JPRB, 0.51016E+00_JPRB, 0.53494E+00_JPRB, 0.55710E+00_JPRB, 0.57458E+00_JPRB ,&
&  0.59176E+00_JPRB, 0.61375E+00_JPRB, 0.63271E+00_JPRB, 0.65495E+00_JPRB, 0.67382E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.70489E+00_JPRB, 0.72561E+00_JPRB, 0.72673E+00_JPRB, 0.73470E+00_JPRB, 0.72952E+00_JPRB, 0.73136E+00_JPRB, 0.71124E+00_JPRB ,&
&  0.69498E+00_JPRB, 0.70895E+00_JPRB, 0.75553E+00_JPRB, 0.84146E+00_JPRB, 0.86639E+00_JPRB, 0.88850E+00_JPRB, 0.89605E+00_JPRB ,&
&  0.92275E+00_JPRB, 0.89352E+00_JPRB, 0.87554E+00_JPRB, 0.83938E+00_JPRB, 0.83824E+00_JPRB, 0.87512E+00_JPRB, 0.97016E+00_JPRB ,&
&  0.11164E+01_JPRB, 0.13054E+01_JPRB, 0.15591E+01_JPRB, 0.18560E+01_JPRB, 0.25191E+01_JPRB, 0.21988E+01_JPRB, 0.19911E+01_JPRB ,&
&  0.22103E+01_JPRB, 0.19186E+01_JPRB, 0.17351E+01_JPRB, 0.16192E+01_JPRB, 0.15281E+01_JPRB, 0.14722E+01_JPRB, 0.15366E+01_JPRB ,&
&  0.15786E+01_JPRB, 0.16428E+01_JPRB, 0.17175E+01_JPRB, 0.17412E+01_JPRB, 0.17543E+01_JPRB, 0.17126E+01_JPRB, 0.16312E+01_JPRB ,&
&  0.15040E+01_JPRB, 0.15473E+01_JPRB, 0.16137E+01_JPRB, 0.17764E+01_JPRB, 0.20671E+01_JPRB, 0.22306E+01_JPRB, 0.23518E+01_JPRB ,&
&  0.23970E+01_JPRB, 0.25647E+01_JPRB, 0.29106E+01_JPRB, 0.30226E+01_JPRB, 0.30625E+01_JPRB, 0.28550E+01_JPRB, 0.25869E+01_JPRB ,&
&  0.22533E+01_JPRB, 0.17980E+01_JPRB, 0.15384E+01_JPRB, 0.24484E+01_JPRB, 0.26300E+01_JPRB, 0.21415E+01_JPRB, 0.22534E+01_JPRB ,&
&  0.20703E+01_JPRB, 0.19411E+01_JPRB, 0.18538E+01_JPRB, 0.16026E+01_JPRB, 0.13044E+01_JPRB, 0.12368E+01_JPRB, 0.12193E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.90767E+00_JPRB, 0.74143E+00_JPRB, 0.67832E+00_JPRB, 0.66428E+00_JPRB, 0.74398E+00_JPRB, 0.90084E+00_JPRB ,&
&  0.91647E+00_JPRB, 0.78473E+00_JPRB, 0.95411E+00_JPRB, 0.15116E+01_JPRB, 0.11769E+01_JPRB, 0.60851E+00_JPRB, 0.45997E+00_JPRB ,&
&  0.46402E+00_JPRB, 0.48025E+00_JPRB, 0.49920E+00_JPRB, 0.50791E+00_JPRB, 0.51172E+00_JPRB, 0.52069E+00_JPRB, 0.55545E+00_JPRB ,&
&  0.58519E+00_JPRB, 0.61478E+00_JPRB, 0.63906E+00_JPRB, 0.67245E+00_JPRB, 0.69240E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.96793E+00_JPRB, 0.10094E+01_JPRB, 0.10404E+01_JPRB, 0.10541E+01_JPRB, 0.10570E+01_JPRB, 0.10512E+01_JPRB, 0.10274E+01_JPRB ,&
&  0.99716E+00_JPRB, 0.10175E+01_JPRB, 0.10827E+01_JPRB, 0.12298E+01_JPRB, 0.13481E+01_JPRB, 0.14212E+01_JPRB, 0.15060E+01_JPRB ,&
&  0.15903E+01_JPRB, 0.16163E+01_JPRB, 0.15839E+01_JPRB, 0.15905E+01_JPRB, 0.15890E+01_JPRB, 0.15914E+01_JPRB, 0.17972E+01_JPRB ,&
&  0.21218E+01_JPRB, 0.25053E+01_JPRB, 0.28434E+01_JPRB, 0.31147E+01_JPRB, 0.25470E+01_JPRB, 0.19733E+01_JPRB, 0.21320E+01_JPRB ,&
&  0.24017E+01_JPRB, 0.22983E+01_JPRB, 0.20236E+01_JPRB, 0.20547E+01_JPRB, 0.21254E+01_JPRB, 0.22713E+01_JPRB, 0.24655E+01_JPRB ,&
&  0.26064E+01_JPRB, 0.27062E+01_JPRB, 0.27953E+01_JPRB, 0.28637E+01_JPRB, 0.29370E+01_JPRB, 0.30018E+01_JPRB, 0.30667E+01_JPRB ,&
&  0.30796E+01_JPRB, 0.30411E+01_JPRB, 0.30803E+01_JPRB, 0.32603E+01_JPRB, 0.33989E+01_JPRB, 0.35540E+01_JPRB, 0.36622E+01_JPRB ,&
&  0.37302E+01_JPRB, 0.38313E+01_JPRB, 0.42633E+01_JPRB, 0.44909E+01_JPRB, 0.42261E+01_JPRB, 0.38979E+01_JPRB, 0.37532E+01_JPRB ,&
&  0.34835E+01_JPRB, 0.30477E+01_JPRB, 0.24615E+01_JPRB, 0.36726E+01_JPRB, 0.36993E+01_JPRB, 0.31590E+01_JPRB, 0.26530E+01_JPRB ,&
&  0.23623E+01_JPRB, 0.22171E+01_JPRB, 0.19780E+01_JPRB, 0.16839E+01_JPRB, 0.13850E+01_JPRB, 0.13607E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.12496E+01_JPRB, 0.10687E+01_JPRB, 0.85597E+00_JPRB, 0.74051E+00_JPRB, 0.71859E+00_JPRB, 0.84101E+00_JPRB, 0.89590E+00_JPRB ,&
&  0.81935E+00_JPRB, 0.85153E+00_JPRB, 0.90385E+00_JPRB, 0.14849E+01_JPRB, 0.26204E+01_JPRB, 0.75169E+00_JPRB, 0.54845E+00_JPRB ,&
&  0.55481E+00_JPRB, 0.58878E+00_JPRB, 0.62483E+00_JPRB, 0.63666E+00_JPRB, 0.63828E+00_JPRB, 0.66588E+00_JPRB, 0.71367E+00_JPRB ,&
&  0.77029E+00_JPRB, 0.81765E+00_JPRB, 0.84377E+00_JPRB, 0.89193E+00_JPRB, 0.93032E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.12066E+01_JPRB, 0.12555E+01_JPRB, 0.13036E+01_JPRB, 0.13627E+01_JPRB, 0.14248E+01_JPRB, 0.14591E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.15090E+01_JPRB, 0.15709E+01_JPRB, 0.16425E+01_JPRB, 0.17071E+01_JPRB, 0.18003E+01_JPRB, 0.18876E+01_JPRB, 0.19805E+01_JPRB ,&
&  0.20749E+01_JPRB, 0.21778E+01_JPRB, 0.23094E+01_JPRB, 0.24359E+01_JPRB, 0.25263E+01_JPRB, 0.26661E+01_JPRB, 0.28360E+01_JPRB ,&
&  0.29954E+01_JPRB, 0.27039E+01_JPRB, 0.25269E+01_JPRB, 0.20333E+01_JPRB, 0.15977E+01_JPRB, 0.20828E+01_JPRB, 0.21324E+01_JPRB ,&
&  0.22092E+01_JPRB, 0.22881E+01_JPRB, 0.23811E+01_JPRB, 0.25310E+01_JPRB, 0.27629E+01_JPRB, 0.29611E+01_JPRB, 0.31055E+01_JPRB ,&
&  0.32415E+01_JPRB, 0.33582E+01_JPRB, 0.34652E+01_JPRB, 0.35849E+01_JPRB, 0.37312E+01_JPRB, 0.38610E+01_JPRB, 0.39726E+01_JPRB ,&
&  0.41154E+01_JPRB, 0.42337E+01_JPRB, 0.43183E+01_JPRB, 0.44373E+01_JPRB, 0.43963E+01_JPRB, 0.44545E+01_JPRB, 0.45317E+01_JPRB ,&
&  0.46371E+01_JPRB, 0.47585E+01_JPRB, 0.50426E+01_JPRB, 0.52976E+01_JPRB, 0.53107E+01_JPRB, 0.49185E+01_JPRB, 0.46678E+01_JPRB ,&
&  0.44024E+01_JPRB, 0.39150E+01_JPRB, 0.38358E+01_JPRB, 0.48578E+01_JPRB, 0.29924E+01_JPRB, 0.31619E+01_JPRB, 0.30994E+01_JPRB ,&
&  0.26901E+01_JPRB, 0.25312E+01_JPRB, 0.21974E+01_JPRB, 0.17181E+01_JPRB, 0.14086E+01_JPRB, 0.10989E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.14116E+01_JPRB, 0.12485E+01_JPRB, 0.97807E+00_JPRB, 0.83184E+00_JPRB, 0.85885E+00_JPRB, 0.98413E+00_JPRB, 0.99457E+00_JPRB ,&
&  0.89716E+00_JPRB, 0.94633E+00_JPRB, 0.97505E+00_JPRB, 0.15661E+01_JPRB, 0.14566E+01_JPRB, 0.73346E+00_JPRB, 0.62957E+00_JPRB ,&
&  0.64779E+00_JPRB, 0.71132E+00_JPRB, 0.80133E+00_JPRB, 0.86256E+00_JPRB, 0.88628E+00_JPRB, 0.91777E+00_JPRB, 0.94767E+00_JPRB ,&
&  0.97689E+00_JPRB, 0.10304E+01_JPRB, 0.10973E+01_JPRB, 0.11362E+01_JPRB, 0.11721E+01_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.12537E+01_JPRB, 0.13236E+01_JPRB, 0.14379E+01_JPRB, 0.15324E+01_JPRB, 0.15994E+01_JPRB, 0.16282E+01_JPRB, 0.16249E+01_JPRB ,&
&  0.16494E+01_JPRB, 0.17122E+01_JPRB, 0.18234E+01_JPRB, 0.19284E+01_JPRB, 0.20491E+01_JPRB, 0.21787E+01_JPRB, 0.23125E+01_JPRB ,&
&  0.24407E+01_JPRB, 0.25545E+01_JPRB, 0.26801E+01_JPRB, 0.28225E+01_JPRB, 0.29654E+01_JPRB, 0.32090E+01_JPRB, 0.32379E+01_JPRB ,&
&  0.25892E+01_JPRB, 0.18147E+01_JPRB, 0.21672E+01_JPRB, 0.22896E+01_JPRB, 0.23469E+01_JPRB, 0.26993E+01_JPRB, 0.28481E+01_JPRB ,&
&  0.26550E+01_JPRB, 0.26820E+01_JPRB, 0.28215E+01_JPRB, 0.29263E+01_JPRB, 0.30200E+01_JPRB, 0.31700E+01_JPRB, 0.33015E+01_JPRB ,&
&  0.34741E+01_JPRB, 0.36527E+01_JPRB, 0.38381E+01_JPRB, 0.40500E+01_JPRB, 0.42461E+01_JPRB, 0.44679E+01_JPRB, 0.46828E+01_JPRB ,&
&  0.48913E+01_JPRB, 0.50522E+01_JPRB, 0.50268E+01_JPRB, 0.49997E+01_JPRB, 0.50050E+01_JPRB, 0.50666E+01_JPRB, 0.51318E+01_JPRB ,&
&  0.53219E+01_JPRB, 0.55971E+01_JPRB, 0.60473E+01_JPRB, 0.63115E+01_JPRB, 0.63841E+01_JPRB, 0.58625E+01_JPRB, 0.56262E+01_JPRB ,&
&  0.53006E+01_JPRB, 0.47784E+01_JPRB, 0.49222E+01_JPRB, 0.52219E+01_JPRB, 0.36439E+01_JPRB, 0.34865E+01_JPRB, 0.29396E+01_JPRB ,&
&  0.30103E+01_JPRB, 0.27594E+01_JPRB, 0.23970E+01_JPRB, 0.19939E+01_JPRB, 0.15455E+01_JPRB, 0.12572E+01_JPRB, 0.14656E+01_JPRB ,&
&  0.17616E+01_JPRB, 0.16712E+01_JPRB, 0.12358E+01_JPRB, 0.10256E+01_JPRB, 0.95083E+00_JPRB, 0.10331E+01_JPRB, 0.10146E+01_JPRB ,&
&  0.96274E+00_JPRB, 0.93010E+00_JPRB, 0.96651E+00_JPRB, 0.12508E+01_JPRB, 0.99334E+00_JPRB, 0.75573E+00_JPRB, 0.72039E+00_JPRB ,&
&  0.76496E+00_JPRB, 0.81769E+00_JPRB, 0.87082E+00_JPRB, 0.91184E+00_JPRB, 0.95078E+00_JPRB, 0.10043E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.10987E+01_JPRB, 0.11531E+01_JPRB, 0.11793E+01_JPRB, 0.12166E+01_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.12541E+01_JPRB, 0.13278E+01_JPRB, 0.14359E+01_JPRB, 0.15348E+01_JPRB, 0.15994E+01_JPRB, 0.16489E+01_JPRB, 0.16815E+01_JPRB ,&
&  0.16828E+01_JPRB, 0.16954E+01_JPRB, 0.18182E+01_JPRB, 0.19419E+01_JPRB, 0.20998E+01_JPRB, 0.22684E+01_JPRB, 0.24429E+01_JPRB ,&
&  0.26090E+01_JPRB, 0.27786E+01_JPRB, 0.29471E+01_JPRB, 0.31510E+01_JPRB, 0.33140E+01_JPRB, 0.35484E+01_JPRB, 0.32539E+01_JPRB ,&
&  0.23536E+01_JPRB, 0.21454E+01_JPRB, 0.26121E+01_JPRB, 0.26823E+01_JPRB, 0.26751E+01_JPRB, 0.29497E+01_JPRB, 0.31080E+01_JPRB ,&
&  0.29942E+01_JPRB, 0.29964E+01_JPRB, 0.30299E+01_JPRB, 0.30813E+01_JPRB, 0.30946E+01_JPRB, 0.31371E+01_JPRB, 0.31950E+01_JPRB ,&
&  0.33351E+01_JPRB, 0.35462E+01_JPRB, 0.37810E+01_JPRB, 0.40731E+01_JPRB, 0.43789E+01_JPRB, 0.46976E+01_JPRB, 0.49463E+01_JPRB ,&
&  0.51381E+01_JPRB, 0.53231E+01_JPRB, 0.53711E+01_JPRB, 0.52244E+01_JPRB, 0.53239E+01_JPRB, 0.55837E+01_JPRB, 0.58372E+01_JPRB ,&
&  0.60599E+01_JPRB, 0.65355E+01_JPRB, 0.67683E+01_JPRB, 0.66275E+01_JPRB, 0.60359E+01_JPRB, 0.62894E+01_JPRB, 0.57684E+01_JPRB ,&
&  0.58623E+01_JPRB, 0.55252E+01_JPRB, 0.59803E+01_JPRB, 0.47936E+01_JPRB, 0.43314E+01_JPRB, 0.41498E+01_JPRB, 0.37553E+01_JPRB ,&
&  0.32915E+01_JPRB, 0.30987E+01_JPRB, 0.27168E+01_JPRB, 0.19729E+01_JPRB, 0.18213E+01_JPRB, 0.15323E+01_JPRB, 0.18518E+01_JPRB ,&
&  0.20503E+01_JPRB, 0.22084E+01_JPRB, 0.14378E+01_JPRB, 0.11058E+01_JPRB, 0.97972E+00_JPRB, 0.10293E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.10290E+01_JPRB, 0.10119E+01_JPRB, 0.10003E+01_JPRB, 0.83487E+00_JPRB, 0.78195E+00_JPRB, 0.77082E+00_JPRB ,&
&  0.81479E+00_JPRB, 0.85852E+00_JPRB, 0.86674E+00_JPRB, 0.86522E+00_JPRB, 0.88405E+00_JPRB, 0.95347E+00_JPRB, 0.10003E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.11015E+01_JPRB, 0.11466E+01_JPRB, 0.11676E+01_JPRB, 0.11943E+01_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.10867E+01_JPRB, 0.11315E+01_JPRB, 0.11812E+01_JPRB, 0.11989E+01_JPRB, 0.12290E+01_JPRB, 0.12624E+01_JPRB, 0.13233E+01_JPRB ,&
&  0.13932E+01_JPRB, 0.14804E+01_JPRB, 0.15802E+01_JPRB, 0.17042E+01_JPRB, 0.18680E+01_JPRB, 0.20285E+01_JPRB, 0.22302E+01_JPRB ,&
&  0.24508E+01_JPRB, 0.26755E+01_JPRB, 0.29063E+01_JPRB, 0.31528E+01_JPRB, 0.33044E+01_JPRB, 0.33970E+01_JPRB, 0.27836E+01_JPRB ,&
&  0.18197E+01_JPRB, 0.29209E+01_JPRB, 0.25830E+01_JPRB, 0.28371E+01_JPRB, 0.29807E+01_JPRB, 0.32145E+01_JPRB, 0.31677E+01_JPRB ,&
&  0.31356E+01_JPRB, 0.31100E+01_JPRB, 0.30764E+01_JPRB, 0.30950E+01_JPRB, 0.31131E+01_JPRB, 0.31070E+01_JPRB, 0.30763E+01_JPRB ,&
&  0.31037E+01_JPRB, 0.32111E+01_JPRB, 0.34065E+01_JPRB, 0.36735E+01_JPRB, 0.40137E+01_JPRB, 0.44353E+01_JPRB, 0.48044E+01_JPRB ,&
&  0.51663E+01_JPRB, 0.55372E+01_JPRB, 0.57182E+01_JPRB, 0.56530E+01_JPRB, 0.56515E+01_JPRB, 0.59544E+01_JPRB, 0.65375E+01_JPRB ,&
&  0.65801E+01_JPRB, 0.64190E+01_JPRB, 0.67646E+01_JPRB, 0.68661E+01_JPRB, 0.65859E+01_JPRB, 0.66835E+01_JPRB, 0.59343E+01_JPRB ,&
&  0.64250E+01_JPRB, 0.59405E+01_JPRB, 0.62872E+01_JPRB, 0.51323E+01_JPRB, 0.48125E+01_JPRB, 0.51149E+01_JPRB, 0.48758E+01_JPRB ,&
&  0.42011E+01_JPRB, 0.38995E+01_JPRB, 0.32852E+01_JPRB, 0.25202E+01_JPRB, 0.22149E+01_JPRB, 0.20490E+01_JPRB, 0.21705E+01_JPRB ,&
&  0.22415E+01_JPRB, 0.21583E+01_JPRB, 0.15932E+01_JPRB, 0.10717E+01_JPRB, 0.99080E+00_JPRB, 0.11639E+01_JPRB, 0.12900E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.13871E+01_JPRB, 0.14859E+01_JPRB, 0.13534E+01_JPRB, 0.97941E+00_JPRB, 0.87052E+00_JPRB, 0.83619E+00_JPRB ,&
&  0.85033E+00_JPRB, 0.88314E+00_JPRB, 0.88837E+00_JPRB, 0.87045E+00_JPRB, 0.82008E+00_JPRB, 0.86356E+00_JPRB, 0.87917E+00_JPRB ,&
&  0.93675E+00_JPRB, 0.96278E+00_JPRB, 0.10064E+01_JPRB, 0.10428E+01_JPRB, 0.10645E+01_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.95345E+00_JPRB, 0.10101E+01_JPRB, 0.10829E+01_JPRB, 0.11124E+01_JPRB, 0.11061E+01_JPRB, 0.11049E+01_JPRB, 0.11266E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11815E+01_JPRB, 0.12347E+01_JPRB, 0.13164E+01_JPRB, 0.14527E+01_JPRB, 0.16043E+01_JPRB, 0.18125E+01_JPRB ,&
&  0.20894E+01_JPRB, 0.24288E+01_JPRB, 0.28426E+01_JPRB, 0.32700E+01_JPRB, 0.35487E+01_JPRB, 0.36264E+01_JPRB, 0.23662E+01_JPRB ,&
&  0.20516E+01_JPRB, 0.27269E+01_JPRB, 0.26133E+01_JPRB, 0.28835E+01_JPRB, 0.31530E+01_JPRB, 0.33217E+01_JPRB, 0.32186E+01_JPRB ,&
&  0.32341E+01_JPRB, 0.31918E+01_JPRB, 0.31582E+01_JPRB, 0.31225E+01_JPRB, 0.30956E+01_JPRB, 0.30629E+01_JPRB, 0.29640E+01_JPRB ,&
&  0.28639E+01_JPRB, 0.28367E+01_JPRB, 0.29127E+01_JPRB, 0.30784E+01_JPRB, 0.33238E+01_JPRB, 0.36492E+01_JPRB, 0.41428E+01_JPRB ,&
&  0.47866E+01_JPRB, 0.55236E+01_JPRB, 0.61912E+01_JPRB, 0.60812E+01_JPRB, 0.61703E+01_JPRB, 0.62196E+01_JPRB, 0.65991E+01_JPRB ,&
&  0.65486E+01_JPRB, 0.61782E+01_JPRB, 0.68307E+01_JPRB, 0.75537E+01_JPRB, 0.77027E+01_JPRB, 0.72585E+01_JPRB, 0.64293E+01_JPRB ,&
&  0.67483E+01_JPRB, 0.66504E+01_JPRB, 0.63714E+01_JPRB, 0.56489E+01_JPRB, 0.55715E+01_JPRB, 0.61777E+01_JPRB, 0.64500E+01_JPRB ,&
&  0.56362E+01_JPRB, 0.49519E+01_JPRB, 0.41761E+01_JPRB, 0.34198E+01_JPRB, 0.28329E+01_JPRB, 0.27103E+01_JPRB, 0.28223E+01_JPRB ,&
&  0.26705E+01_JPRB, 0.25850E+01_JPRB, 0.18786E+01_JPRB, 0.11599E+01_JPRB, 0.95530E+00_JPRB, 0.12443E+01_JPRB, 0.16064E+01_JPRB ,&
&  0.16993E+01_JPRB, 0.18444E+01_JPRB, 0.18716E+01_JPRB, 0.22513E+01_JPRB, 0.15198E+01_JPRB, 0.11808E+01_JPRB, 0.98234E+00_JPRB ,&
&  0.92661E+00_JPRB, 0.91988E+00_JPRB, 0.93277E+00_JPRB, 0.87175E+00_JPRB, 0.81071E+00_JPRB, 0.82459E+00_JPRB, 0.79874E+00_JPRB ,&
&  0.82119E+00_JPRB, 0.85434E+00_JPRB, 0.87004E+00_JPRB, 0.88977E+00_JPRB, 0.92573E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.86738E+00_JPRB, 0.92351E+00_JPRB, 0.97785E+00_JPRB, 0.10177E+01_JPRB, 0.10301E+01_JPRB, 0.10185E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10022E+01_JPRB, 0.10002E+01_JPRB, 0.10284E+01_JPRB, 0.10680E+01_JPRB, 0.11443E+01_JPRB, 0.12375E+01_JPRB, 0.13932E+01_JPRB ,&
&  0.16500E+01_JPRB, 0.20243E+01_JPRB, 0.25740E+01_JPRB, 0.31718E+01_JPRB, 0.33535E+01_JPRB, 0.29495E+01_JPRB, 0.27440E+01_JPRB ,&
&  0.24795E+01_JPRB, 0.26588E+01_JPRB, 0.26069E+01_JPRB, 0.30353E+01_JPRB, 0.34616E+01_JPRB, 0.36061E+01_JPRB, 0.30430E+01_JPRB ,&
&  0.30596E+01_JPRB, 0.30259E+01_JPRB, 0.29660E+01_JPRB, 0.29374E+01_JPRB, 0.29177E+01_JPRB, 0.29129E+01_JPRB, 0.28569E+01_JPRB ,&
&  0.27281E+01_JPRB, 0.26061E+01_JPRB, 0.25550E+01_JPRB, 0.25883E+01_JPRB, 0.27253E+01_JPRB, 0.29146E+01_JPRB, 0.32263E+01_JPRB ,&
&  0.38131E+01_JPRB, 0.48118E+01_JPRB, 0.60661E+01_JPRB, 0.65816E+01_JPRB, 0.66222E+01_JPRB, 0.66430E+01_JPRB, 0.66580E+01_JPRB ,&
&  0.67728E+01_JPRB, 0.73204E+01_JPRB, 0.76142E+01_JPRB, 0.77312E+01_JPRB, 0.80258E+01_JPRB, 0.81654E+01_JPRB, 0.70847E+01_JPRB ,&
&  0.70171E+01_JPRB, 0.69215E+01_JPRB, 0.63416E+01_JPRB, 0.63487E+01_JPRB, 0.65702E+01_JPRB, 0.73002E+01_JPRB, 0.68097E+01_JPRB ,&
&  0.58070E+01_JPRB, 0.50287E+01_JPRB, 0.45968E+01_JPRB, 0.43746E+01_JPRB, 0.37993E+01_JPRB, 0.37985E+01_JPRB, 0.39964E+01_JPRB ,&
&  0.30037E+01_JPRB, 0.11413E+01_JPRB, 0.70967E+00_JPRB, 0.89067E+00_JPRB, 0.66041E+00_JPRB, 0.97929E+00_JPRB, 0.19235E+01_JPRB ,&
&  0.23325E+01_JPRB, 0.28169E+01_JPRB, 0.33163E+01_JPRB, 0.34663E+01_JPRB, 0.29484E+01_JPRB, 0.20917E+01_JPRB, 0.14859E+01_JPRB ,&
&  0.11577E+01_JPRB, 0.10443E+01_JPRB, 0.10463E+01_JPRB, 0.95292E+00_JPRB, 0.86085E+00_JPRB, 0.83465E+00_JPRB, 0.80951E+00_JPRB ,&
&  0.78517E+00_JPRB, 0.80194E+00_JPRB, 0.82521E+00_JPRB, 0.83090E+00_JPRB, 0.83073E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.89156E+00_JPRB, 0.90473E+00_JPRB, 0.92826E+00_JPRB, 0.92636E+00_JPRB, 0.91797E+00_JPRB, 0.92018E+00_JPRB, 0.92058E+00_JPRB ,&
&  0.90946E+00_JPRB, 0.89991E+00_JPRB, 0.92169E+00_JPRB, 0.96137E+00_JPRB, 0.10185E+01_JPRB, 0.10837E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.12959E+01_JPRB, 0.15972E+01_JPRB, 0.21391E+01_JPRB, 0.29302E+01_JPRB, 0.33180E+01_JPRB, 0.31100E+01_JPRB, 0.31047E+01_JPRB ,&
&  0.26023E+01_JPRB, 0.27559E+01_JPRB, 0.32327E+01_JPRB, 0.40861E+01_JPRB, 0.42997E+01_JPRB, 0.40184E+01_JPRB, 0.32057E+01_JPRB ,&
&  0.31514E+01_JPRB, 0.30289E+01_JPRB, 0.28443E+01_JPRB, 0.26556E+01_JPRB, 0.25516E+01_JPRB, 0.25142E+01_JPRB, 0.25199E+01_JPRB ,&
&  0.25147E+01_JPRB, 0.24402E+01_JPRB, 0.23520E+01_JPRB, 0.23071E+01_JPRB, 0.23414E+01_JPRB, 0.24742E+01_JPRB, 0.26768E+01_JPRB ,&
&  0.29720E+01_JPRB, 0.36255E+01_JPRB, 0.50847E+01_JPRB, 0.68167E+01_JPRB, 0.62087E+01_JPRB, 0.72058E+01_JPRB, 0.69908E+01_JPRB ,&
&  0.69291E+01_JPRB, 0.75477E+01_JPRB, 0.76809E+01_JPRB, 0.88476E+01_JPRB, 0.89418E+01_JPRB, 0.85394E+01_JPRB, 0.73517E+01_JPRB ,&
&  0.71995E+01_JPRB, 0.65824E+01_JPRB, 0.66581E+01_JPRB, 0.70124E+01_JPRB, 0.75699E+01_JPRB, 0.85399E+01_JPRB, 0.60672E+01_JPRB ,&
&  0.55343E+01_JPRB, 0.60120E+01_JPRB, 0.56478E+01_JPRB, 0.49077E+01_JPRB, 0.48117E+01_JPRB, 0.51809E+01_JPRB, 0.34614E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.68174E+00_JPRB, 0.57774E+00_JPRB, 0.54974E+00_JPRB, 0.50160E+00_JPRB, 0.10026E+01_JPRB, 0.40479E+01_JPRB ,&
&  0.33390E+01_JPRB, 0.44741E+01_JPRB, 0.50908E+01_JPRB, 0.52424E+01_JPRB, 0.49923E+01_JPRB, 0.36435E+01_JPRB, 0.27317E+01_JPRB ,&
&  0.18366E+01_JPRB, 0.14209E+01_JPRB, 0.12892E+01_JPRB, 0.11957E+01_JPRB, 0.10431E+01_JPRB, 0.96154E+00_JPRB, 0.92098E+00_JPRB ,&
&  0.92493E+00_JPRB, 0.91413E+00_JPRB, 0.89968E+00_JPRB, 0.87751E+00_JPRB, 0.88328E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.10176E+01_JPRB, 0.99865E+00_JPRB, 0.98073E+00_JPRB, 0.92390E+00_JPRB, 0.87335E+00_JPRB, 0.86839E+00_JPRB, 0.86252E+00_JPRB ,&
&  0.84779E+00_JPRB, 0.83567E+00_JPRB, 0.84620E+00_JPRB, 0.87805E+00_JPRB, 0.91750E+00_JPRB, 0.97911E+00_JPRB, 0.10392E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.13625E+01_JPRB, 0.18728E+01_JPRB, 0.24595E+01_JPRB, 0.30153E+01_JPRB, 0.28473E+01_JPRB, 0.32183E+01_JPRB ,&
&  0.28072E+01_JPRB, 0.27702E+01_JPRB, 0.34202E+01_JPRB, 0.46443E+01_JPRB, 0.64198E+01_JPRB, 0.63657E+01_JPRB, 0.55070E+01_JPRB ,&
&  0.47826E+01_JPRB, 0.42594E+01_JPRB, 0.38399E+01_JPRB, 0.33741E+01_JPRB, 0.26377E+01_JPRB, 0.22840E+01_JPRB, 0.21803E+01_JPRB ,&
&  0.21107E+01_JPRB, 0.21440E+01_JPRB, 0.20969E+01_JPRB, 0.20672E+01_JPRB, 0.21239E+01_JPRB, 0.22496E+01_JPRB, 0.24498E+01_JPRB ,&
&  0.26298E+01_JPRB, 0.29130E+01_JPRB, 0.37604E+01_JPRB, 0.57961E+01_JPRB, 0.67065E+01_JPRB, 0.65695E+01_JPRB, 0.66984E+01_JPRB ,&
&  0.68359E+01_JPRB, 0.79042E+01_JPRB, 0.87583E+01_JPRB, 0.10532E+02_JPRB, 0.93340E+01_JPRB, 0.87937E+01_JPRB, 0.79557E+01_JPRB ,&
&  0.74366E+01_JPRB, 0.76230E+01_JPRB, 0.63666E+01_JPRB, 0.65919E+01_JPRB, 0.77734E+01_JPRB, 0.71172E+01_JPRB, 0.64282E+01_JPRB ,&
&  0.68483E+01_JPRB, 0.58253E+01_JPRB, 0.54362E+01_JPRB, 0.48568E+01_JPRB, 0.58415E+01_JPRB, 0.48255E+01_JPRB, 0.17205E+01_JPRB ,&
&  0.95254E+00_JPRB, 0.71903E+00_JPRB, 0.51407E+00_JPRB, 0.50081E+00_JPRB, 0.58621E+00_JPRB, 0.10167E+01_JPRB, 0.28785E+01_JPRB ,&
&  0.32825E+01_JPRB, 0.52810E+01_JPRB, 0.59946E+01_JPRB, 0.64881E+01_JPRB, 0.58905E+01_JPRB, 0.56254E+01_JPRB, 0.51791E+01_JPRB ,&
&  0.42958E+01_JPRB, 0.32493E+01_JPRB, 0.24195E+01_JPRB, 0.18885E+01_JPRB, 0.15036E+01_JPRB, 0.12974E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.11330E+01_JPRB, 0.10969E+01_JPRB, 0.10600E+01_JPRB, 0.10189E+01_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.11697E+01_JPRB, 0.11426E+01_JPRB, 0.11012E+01_JPRB, 0.99624E+00_JPRB, 0.92222E+00_JPRB, 0.89664E+00_JPRB, 0.87403E+00_JPRB ,&
&  0.84988E+00_JPRB, 0.81421E+00_JPRB, 0.80154E+00_JPRB, 0.81945E+00_JPRB, 0.84227E+00_JPRB, 0.87724E+00_JPRB, 0.93802E+00_JPRB ,&
&  0.10637E+01_JPRB, 0.12994E+01_JPRB, 0.16961E+01_JPRB, 0.20727E+01_JPRB, 0.24040E+01_JPRB, 0.24341E+01_JPRB, 0.30753E+01_JPRB ,&
&  0.32042E+01_JPRB, 0.29429E+01_JPRB, 0.33296E+01_JPRB, 0.47732E+01_JPRB, 0.71420E+01_JPRB, 0.85360E+01_JPRB, 0.89427E+01_JPRB ,&
&  0.76666E+01_JPRB, 0.62791E+01_JPRB, 0.53524E+01_JPRB, 0.47291E+01_JPRB, 0.38883E+01_JPRB, 0.28525E+01_JPRB, 0.22488E+01_JPRB ,&
&  0.19623E+01_JPRB, 0.19642E+01_JPRB, 0.19936E+01_JPRB, 0.19378E+01_JPRB, 0.19919E+01_JPRB, 0.21035E+01_JPRB, 0.23423E+01_JPRB ,&
&  0.25601E+01_JPRB, 0.27348E+01_JPRB, 0.30946E+01_JPRB, 0.44365E+01_JPRB, 0.66180E+01_JPRB, 0.78426E+01_JPRB, 0.68479E+01_JPRB ,&
&  0.59554E+01_JPRB, 0.63086E+01_JPRB, 0.98482E+01_JPRB, 0.10683E+02_JPRB, 0.99574E+01_JPRB, 0.98426E+01_JPRB, 0.86070E+01_JPRB ,&
&  0.83973E+01_JPRB, 0.84758E+01_JPRB, 0.74376E+01_JPRB, 0.74731E+01_JPRB, 0.68630E+01_JPRB, 0.63395E+01_JPRB, 0.72588E+01_JPRB ,&
&  0.72984E+01_JPRB, 0.62862E+01_JPRB, 0.52543E+01_JPRB, 0.46332E+01_JPRB, 0.49275E+01_JPRB, 0.30710E+01_JPRB, 0.17546E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.96898E+00_JPRB, 0.73765E+00_JPRB, 0.76629E+00_JPRB, 0.67967E+00_JPRB, 0.10650E+01_JPRB, 0.19286E+01_JPRB ,&
&  0.37070E+01_JPRB, 0.51392E+01_JPRB, 0.68435E+01_JPRB, 0.71602E+01_JPRB, 0.63625E+01_JPRB, 0.59301E+01_JPRB, 0.71613E+01_JPRB ,&
&  0.49695E+01_JPRB, 0.43101E+01_JPRB, 0.40132E+01_JPRB, 0.30190E+01_JPRB, 0.21496E+01_JPRB, 0.16822E+01_JPRB, 0.14384E+01_JPRB ,&
&  0.13123E+01_JPRB, 0.12398E+01_JPRB, 0.11936E+01_JPRB, 0.11921E+01_JPRB, 0.12051E+01_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.13798E+01_JPRB, 0.12874E+01_JPRB, 0.12494E+01_JPRB, 0.11628E+01_JPRB, 0.10318E+01_JPRB, 0.94046E+00_JPRB, 0.91191E+00_JPRB ,&
&  0.88300E+00_JPRB, 0.84566E+00_JPRB, 0.80669E+00_JPRB, 0.78574E+00_JPRB, 0.80312E+00_JPRB, 0.83084E+00_JPRB, 0.86982E+00_JPRB ,&
&  0.10107E+01_JPRB, 0.14203E+01_JPRB, 0.14075E+01_JPRB, 0.15720E+01_JPRB, 0.18672E+01_JPRB, 0.23374E+01_JPRB, 0.26850E+01_JPRB ,&
&  0.34094E+01_JPRB, 0.33438E+01_JPRB, 0.34396E+01_JPRB, 0.49357E+01_JPRB, 0.88624E+01_JPRB, 0.10519E+02_JPRB, 0.11558E+02_JPRB ,&
&  0.11296E+02_JPRB, 0.93179E+01_JPRB, 0.74108E+01_JPRB, 0.58229E+01_JPRB, 0.47729E+01_JPRB, 0.40625E+01_JPRB, 0.31547E+01_JPRB ,&
&  0.25126E+01_JPRB, 0.22068E+01_JPRB, 0.21285E+01_JPRB, 0.20526E+01_JPRB, 0.19992E+01_JPRB, 0.20411E+01_JPRB, 0.22318E+01_JPRB ,&
&  0.25161E+01_JPRB, 0.27865E+01_JPRB, 0.30378E+01_JPRB, 0.40145E+01_JPRB, 0.55263E+01_JPRB, 0.67782E+01_JPRB, 0.82469E+01_JPRB ,&
&  0.74427E+01_JPRB, 0.79662E+01_JPRB, 0.97785E+01_JPRB, 0.12051E+02_JPRB, 0.11293E+02_JPRB, 0.10358E+02_JPRB, 0.91735E+01_JPRB ,&
&  0.71896E+01_JPRB, 0.71454E+01_JPRB, 0.82867E+01_JPRB, 0.82610E+01_JPRB, 0.65899E+01_JPRB, 0.76390E+01_JPRB, 0.10213E+02_JPRB ,&
&  0.75446E+01_JPRB, 0.71006E+01_JPRB, 0.69316E+01_JPRB, 0.73791E+01_JPRB, 0.44482E+01_JPRB, 0.26522E+01_JPRB, 0.41474E+01_JPRB ,&
&  0.41932E+01_JPRB, 0.21107E+01_JPRB, 0.14011E+01_JPRB, 0.17445E+01_JPRB, 0.11873E+01_JPRB, 0.14040E+01_JPRB, 0.21562E+01_JPRB ,&
&  0.27158E+01_JPRB, 0.32687E+01_JPRB, 0.65060E+01_JPRB, 0.60920E+01_JPRB, 0.60158E+01_JPRB, 0.53935E+01_JPRB, 0.59615E+01_JPRB ,&
&  0.58530E+01_JPRB, 0.43123E+01_JPRB, 0.41011E+01_JPRB, 0.34393E+01_JPRB, 0.27429E+01_JPRB, 0.22417E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.16850E+01_JPRB, 0.15491E+01_JPRB, 0.14836E+01_JPRB, 0.14625E+01_JPRB, 0.14653E+01_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.14485E+01_JPRB, 0.12807E+01_JPRB, 0.11981E+01_JPRB, 0.11719E+01_JPRB, 0.11189E+01_JPRB, 0.10012E+01_JPRB, 0.92090E+00_JPRB ,&
&  0.90954E+00_JPRB, 0.87239E+00_JPRB, 0.82335E+00_JPRB, 0.79777E+00_JPRB, 0.80362E+00_JPRB, 0.81664E+00_JPRB, 0.81396E+00_JPRB ,&
&  0.89841E+00_JPRB, 0.11358E+01_JPRB, 0.12076E+01_JPRB, 0.13287E+01_JPRB, 0.16657E+01_JPRB, 0.21191E+01_JPRB, 0.24053E+01_JPRB ,&
&  0.31920E+01_JPRB, 0.35498E+01_JPRB, 0.40012E+01_JPRB, 0.56743E+01_JPRB, 0.81064E+01_JPRB, 0.10351E+02_JPRB, 0.11049E+02_JPRB ,&
&  0.11950E+02_JPRB, 0.11396E+02_JPRB, 0.90989E+01_JPRB, 0.74116E+01_JPRB, 0.62258E+01_JPRB, 0.54718E+01_JPRB, 0.46719E+01_JPRB ,&
&  0.37529E+01_JPRB, 0.29832E+01_JPRB, 0.26032E+01_JPRB, 0.23868E+01_JPRB, 0.22861E+01_JPRB, 0.21643E+01_JPRB, 0.22468E+01_JPRB ,&
&  0.24958E+01_JPRB, 0.28306E+01_JPRB, 0.30666E+01_JPRB, 0.39555E+01_JPRB, 0.57337E+01_JPRB, 0.72715E+01_JPRB, 0.79492E+01_JPRB ,&
&  0.79214E+01_JPRB, 0.79635E+01_JPRB, 0.10407E+02_JPRB, 0.11412E+02_JPRB, 0.12390E+02_JPRB, 0.11597E+02_JPRB, 0.10652E+02_JPRB ,&
&  0.79811E+01_JPRB, 0.72986E+01_JPRB, 0.76628E+01_JPRB, 0.79140E+01_JPRB, 0.85076E+01_JPRB, 0.10720E+02_JPRB, 0.10206E+02_JPRB ,&
&  0.87510E+01_JPRB, 0.77849E+01_JPRB, 0.69113E+01_JPRB, 0.58987E+01_JPRB, 0.62324E+01_JPRB, 0.43267E+01_JPRB, 0.50241E+01_JPRB ,&
&  0.51214E+01_JPRB, 0.47028E+01_JPRB, 0.37376E+01_JPRB, 0.29508E+01_JPRB, 0.22894E+01_JPRB, 0.24347E+01_JPRB, 0.22720E+01_JPRB ,&
&  0.24099E+01_JPRB, 0.25839E+01_JPRB, 0.42876E+01_JPRB, 0.49922E+01_JPRB, 0.50053E+01_JPRB, 0.43919E+01_JPRB, 0.52663E+01_JPRB ,&
&  0.50288E+01_JPRB, 0.48704E+01_JPRB, 0.40972E+01_JPRB, 0.33606E+01_JPRB, 0.27845E+01_JPRB, 0.24156E+01_JPRB, 0.21768E+01_JPRB ,&
&  0.19402E+01_JPRB, 0.18274E+01_JPRB, 0.17627E+01_JPRB, 0.16985E+01_JPRB, 0.16104E+01_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.14281E+01_JPRB, 0.12569E+01_JPRB, 0.11596E+01_JPRB, 0.10879E+01_JPRB, 0.10523E+01_JPRB, 0.10147E+01_JPRB, 0.94500E+00_JPRB ,&
&  0.88817E+00_JPRB, 0.88135E+00_JPRB, 0.86429E+00_JPRB, 0.85314E+00_JPRB, 0.84443E+00_JPRB, 0.82601E+00_JPRB, 0.80589E+00_JPRB ,&
&  0.84712E+00_JPRB, 0.10038E+01_JPRB, 0.10353E+01_JPRB, 0.12225E+01_JPRB, 0.14968E+01_JPRB, 0.19574E+01_JPRB, 0.25342E+01_JPRB ,&
&  0.29225E+01_JPRB, 0.33172E+01_JPRB, 0.38227E+01_JPRB, 0.51242E+01_JPRB, 0.66809E+01_JPRB, 0.80338E+01_JPRB, 0.83168E+01_JPRB ,&
&  0.77726E+01_JPRB, 0.79148E+01_JPRB, 0.74711E+01_JPRB, 0.67059E+01_JPRB, 0.60270E+01_JPRB, 0.55046E+01_JPRB, 0.48991E+01_JPRB ,&
&  0.42372E+01_JPRB, 0.35889E+01_JPRB, 0.30906E+01_JPRB, 0.28020E+01_JPRB, 0.26340E+01_JPRB, 0.26235E+01_JPRB, 0.26346E+01_JPRB ,&
&  0.26643E+01_JPRB, 0.28332E+01_JPRB, 0.29180E+01_JPRB, 0.35075E+01_JPRB, 0.49030E+01_JPRB, 0.68699E+01_JPRB, 0.74275E+01_JPRB ,&
&  0.86080E+01_JPRB, 0.88601E+01_JPRB, 0.11459E+02_JPRB, 0.13287E+02_JPRB, 0.10915E+02_JPRB, 0.12470E+02_JPRB, 0.13431E+02_JPRB ,&
&  0.11690E+02_JPRB, 0.10553E+02_JPRB, 0.10962E+02_JPRB, 0.10582E+02_JPRB, 0.94865E+01_JPRB, 0.11021E+02_JPRB, 0.99346E+01_JPRB ,&
&  0.86130E+01_JPRB, 0.77899E+01_JPRB, 0.67940E+01_JPRB, 0.67601E+01_JPRB, 0.57519E+01_JPRB, 0.55385E+01_JPRB, 0.43503E+01_JPRB ,&
&  0.45306E+01_JPRB, 0.37801E+01_JPRB, 0.40109E+01_JPRB, 0.33899E+01_JPRB, 0.30362E+01_JPRB, 0.26041E+01_JPRB, 0.22229E+01_JPRB ,&
&  0.20960E+01_JPRB, 0.20968E+01_JPRB, 0.23919E+01_JPRB, 0.33420E+01_JPRB, 0.43647E+01_JPRB, 0.36931E+01_JPRB, 0.41977E+01_JPRB ,&
&  0.47727E+01_JPRB, 0.47262E+01_JPRB, 0.37408E+01_JPRB, 0.32435E+01_JPRB, 0.26813E+01_JPRB, 0.22799E+01_JPRB, 0.20524E+01_JPRB ,&
&  0.18384E+01_JPRB, 0.17418E+01_JPRB, 0.16920E+01_JPRB, 0.16411E+01_JPRB, 0.15763E+01_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.13367E+01_JPRB, 0.11964E+01_JPRB, 0.10717E+01_JPRB, 0.10065E+01_JPRB, 0.98472E+00_JPRB, 0.98125E+00_JPRB, 0.97012E+00_JPRB ,&
&  0.93688E+00_JPRB, 0.92050E+00_JPRB, 0.91506E+00_JPRB, 0.89779E+00_JPRB, 0.87887E+00_JPRB, 0.86078E+00_JPRB, 0.83127E+00_JPRB ,&
&  0.82928E+00_JPRB, 0.10108E+01_JPRB, 0.99641E+00_JPRB, 0.10433E+01_JPRB, 0.12445E+01_JPRB, 0.18106E+01_JPRB, 0.23929E+01_JPRB ,&
&  0.28040E+01_JPRB, 0.31277E+01_JPRB, 0.35920E+01_JPRB, 0.42220E+01_JPRB, 0.48147E+01_JPRB, 0.51115E+01_JPRB, 0.51396E+01_JPRB ,&
&  0.56595E+01_JPRB, 0.47141E+01_JPRB, 0.47749E+01_JPRB, 0.43259E+01_JPRB, 0.40445E+01_JPRB, 0.37556E+01_JPRB, 0.35131E+01_JPRB ,&
&  0.32910E+01_JPRB, 0.30547E+01_JPRB, 0.28094E+01_JPRB, 0.26411E+01_JPRB, 0.25429E+01_JPRB, 0.24977E+01_JPRB, 0.25106E+01_JPRB ,&
&  0.25606E+01_JPRB, 0.25898E+01_JPRB, 0.27066E+01_JPRB, 0.27752E+01_JPRB, 0.35097E+01_JPRB, 0.51467E+01_JPRB, 0.67286E+01_JPRB ,&
&  0.68132E+01_JPRB, 0.69015E+01_JPRB, 0.13138E+02_JPRB, 0.14252E+02_JPRB, 0.13548E+02_JPRB, 0.14582E+02_JPRB, 0.14838E+02_JPRB ,&
&  0.13485E+02_JPRB, 0.11501E+02_JPRB, 0.97377E+01_JPRB, 0.92745E+01_JPRB, 0.11007E+02_JPRB, 0.10451E+02_JPRB, 0.95706E+01_JPRB ,&
&  0.86313E+01_JPRB, 0.80571E+01_JPRB, 0.73083E+01_JPRB, 0.67031E+01_JPRB, 0.63430E+01_JPRB, 0.65656E+01_JPRB, 0.59918E+01_JPRB ,&
&  0.54816E+01_JPRB, 0.57462E+01_JPRB, 0.45913E+01_JPRB, 0.34365E+01_JPRB, 0.27808E+01_JPRB, 0.24507E+01_JPRB, 0.22071E+01_JPRB ,&
&  0.18994E+01_JPRB, 0.18081E+01_JPRB, 0.18443E+01_JPRB, 0.22899E+01_JPRB, 0.33246E+01_JPRB, 0.33338E+01_JPRB, 0.33606E+01_JPRB ,&
&  0.31355E+01_JPRB, 0.36134E+01_JPRB, 0.33111E+01_JPRB, 0.28879E+01_JPRB, 0.24599E+01_JPRB, 0.20852E+01_JPRB, 0.18179E+01_JPRB ,&
&  0.16509E+01_JPRB, 0.16294E+01_JPRB, 0.15983E+01_JPRB, 0.15344E+01_JPRB, 0.14743E+01_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.12700E+01_JPRB, 0.11458E+01_JPRB, 0.10221E+01_JPRB, 0.95965E+00_JPRB, 0.95365E+00_JPRB, 0.95930E+00_JPRB, 0.96790E+00_JPRB ,&
&  0.94705E+00_JPRB, 0.92638E+00_JPRB, 0.93175E+00_JPRB, 0.93114E+00_JPRB, 0.92585E+00_JPRB, 0.90178E+00_JPRB, 0.86089E+00_JPRB ,&
&  0.85399E+00_JPRB, 0.10531E+01_JPRB, 0.10018E+01_JPRB, 0.10717E+01_JPRB, 0.12054E+01_JPRB, 0.16082E+01_JPRB, 0.19740E+01_JPRB ,&
&  0.23464E+01_JPRB, 0.25829E+01_JPRB, 0.28048E+01_JPRB, 0.30995E+01_JPRB, 0.32235E+01_JPRB, 0.30838E+01_JPRB, 0.29036E+01_JPRB ,&
&  0.27387E+01_JPRB, 0.27273E+01_JPRB, 0.23530E+01_JPRB, 0.24858E+01_JPRB, 0.23994E+01_JPRB, 0.20741E+01_JPRB, 0.22751E+01_JPRB ,&
&  0.22240E+01_JPRB, 0.22362E+01_JPRB, 0.22051E+01_JPRB, 0.21842E+01_JPRB, 0.21847E+01_JPRB, 0.21694E+01_JPRB, 0.21421E+01_JPRB ,&
&  0.21238E+01_JPRB, 0.21273E+01_JPRB, 0.22810E+01_JPRB, 0.24084E+01_JPRB, 0.26256E+01_JPRB, 0.34018E+01_JPRB, 0.49770E+01_JPRB ,&
&  0.70097E+01_JPRB, 0.73461E+01_JPRB, 0.79128E+01_JPRB, 0.11913E+02_JPRB, 0.14916E+02_JPRB, 0.14149E+02_JPRB, 0.13426E+02_JPRB ,&
&  0.13286E+02_JPRB, 0.12277E+02_JPRB, 0.11230E+02_JPRB, 0.10602E+02_JPRB, 0.11040E+02_JPRB, 0.10781E+02_JPRB, 0.10058E+02_JPRB ,&
&  0.90838E+01_JPRB, 0.81978E+01_JPRB, 0.77971E+01_JPRB, 0.73338E+01_JPRB, 0.64585E+01_JPRB, 0.60738E+01_JPRB, 0.61428E+01_JPRB ,&
&  0.59883E+01_JPRB, 0.49798E+01_JPRB, 0.35350E+01_JPRB, 0.30761E+01_JPRB, 0.23334E+01_JPRB, 0.19833E+01_JPRB, 0.19798E+01_JPRB ,&
&  0.16315E+01_JPRB, 0.15026E+01_JPRB, 0.15526E+01_JPRB, 0.16333E+01_JPRB, 0.23476E+01_JPRB, 0.24865E+01_JPRB, 0.27437E+01_JPRB ,&
&  0.25430E+01_JPRB, 0.21881E+01_JPRB, 0.25732E+01_JPRB, 0.24616E+01_JPRB, 0.21549E+01_JPRB, 0.18348E+01_JPRB, 0.15979E+01_JPRB ,&
&  0.15161E+01_JPRB, 0.15351E+01_JPRB, 0.14927E+01_JPRB, 0.14451E+01_JPRB, 0.13973E+01_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.11835E+01_JPRB, 0.10644E+01_JPRB, 0.96108E+00_JPRB, 0.89121E+00_JPRB, 0.88014E+00_JPRB, 0.89378E+00_JPRB, 0.90295E+00_JPRB ,&
&  0.91550E+00_JPRB, 0.92464E+00_JPRB, 0.92929E+00_JPRB, 0.93412E+00_JPRB, 0.93508E+00_JPRB, 0.91218E+00_JPRB, 0.89609E+00_JPRB ,&
&  0.90760E+00_JPRB, 0.10171E+01_JPRB, 0.10226E+01_JPRB, 0.10302E+01_JPRB, 0.14143E+01_JPRB, 0.15949E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.19720E+01_JPRB, 0.21704E+01_JPRB, 0.21305E+01_JPRB, 0.20290E+01_JPRB, 0.19361E+01_JPRB, 0.18738E+01_JPRB, 0.16732E+01_JPRB ,&
&  0.14249E+01_JPRB, 0.13428E+01_JPRB, 0.12947E+01_JPRB, 0.13716E+01_JPRB, 0.14743E+01_JPRB, 0.12866E+01_JPRB, 0.14008E+01_JPRB ,&
&  0.14542E+01_JPRB, 0.15291E+01_JPRB, 0.15712E+01_JPRB, 0.15877E+01_JPRB, 0.16225E+01_JPRB, 0.16723E+01_JPRB, 0.17299E+01_JPRB ,&
&  0.17955E+01_JPRB, 0.18111E+01_JPRB, 0.18536E+01_JPRB, 0.20923E+01_JPRB, 0.23223E+01_JPRB, 0.28868E+01_JPRB, 0.44790E+01_JPRB ,&
&  0.64206E+01_JPRB, 0.72194E+01_JPRB, 0.94470E+01_JPRB, 0.13059E+02_JPRB, 0.13112E+02_JPRB, 0.12022E+02_JPRB, 0.11566E+02_JPRB ,&
&  0.11233E+02_JPRB, 0.10782E+02_JPRB, 0.10478E+02_JPRB, 0.10142E+02_JPRB, 0.10318E+02_JPRB, 0.10678E+02_JPRB, 0.10378E+02_JPRB ,&
&  0.95017E+01_JPRB, 0.86375E+01_JPRB, 0.82185E+01_JPRB, 0.76263E+01_JPRB, 0.71032E+01_JPRB, 0.64216E+01_JPRB, 0.64345E+01_JPRB ,&
&  0.64846E+01_JPRB, 0.42406E+01_JPRB, 0.32542E+01_JPRB, 0.30664E+01_JPRB, 0.21258E+01_JPRB, 0.17788E+01_JPRB, 0.19503E+01_JPRB ,&
&  0.15115E+01_JPRB, 0.12953E+01_JPRB, 0.14563E+01_JPRB, 0.16014E+01_JPRB, 0.17420E+01_JPRB, 0.18216E+01_JPRB, 0.17371E+01_JPRB ,&
&  0.15129E+01_JPRB, 0.14790E+01_JPRB, 0.18250E+01_JPRB, 0.18479E+01_JPRB, 0.17278E+01_JPRB, 0.15764E+01_JPRB, 0.14440E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.13813E+01_JPRB, 0.13557E+01_JPRB, 0.13322E+01_JPRB, 0.12973E+01_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.11665E+01_JPRB, 0.10456E+01_JPRB, 0.92873E+00_JPRB, 0.86600E+00_JPRB, 0.87240E+00_JPRB, 0.87810E+00_JPRB, 0.87795E+00_JPRB ,&
&  0.89501E+00_JPRB, 0.91144E+00_JPRB, 0.92072E+00_JPRB, 0.92526E+00_JPRB, 0.92737E+00_JPRB, 0.92482E+00_JPRB, 0.96089E+00_JPRB ,&
&  0.82804E+00_JPRB, 0.88004E+00_JPRB, 0.86029E+00_JPRB, 0.98728E+00_JPRB, 0.15045E+01_JPRB, 0.14778E+01_JPRB, 0.15431E+01_JPRB ,&
&  0.14854E+01_JPRB, 0.14553E+01_JPRB, 0.13783E+01_JPRB, 0.12920E+01_JPRB, 0.12300E+01_JPRB, 0.12266E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.99997E+00_JPRB, 0.90408E+00_JPRB, 0.88416E+00_JPRB, 0.88934E+00_JPRB, 0.94797E+00_JPRB, 0.98215E+00_JPRB, 0.10666E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.11578E+01_JPRB, 0.12247E+01_JPRB, 0.12819E+01_JPRB, 0.13077E+01_JPRB, 0.13204E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.14104E+01_JPRB, 0.14915E+01_JPRB, 0.17354E+01_JPRB, 0.19382E+01_JPRB, 0.27599E+01_JPRB, 0.45851E+01_JPRB ,&
&  0.57350E+01_JPRB, 0.58650E+01_JPRB, 0.74514E+01_JPRB, 0.10394E+02_JPRB, 0.10500E+02_JPRB, 0.95559E+01_JPRB, 0.86389E+01_JPRB ,&
&  0.82016E+01_JPRB, 0.79703E+01_JPRB, 0.83303E+01_JPRB, 0.89262E+01_JPRB, 0.93360E+01_JPRB, 0.99608E+01_JPRB, 0.10083E+02_JPRB ,&
&  0.98422E+01_JPRB, 0.96129E+01_JPRB, 0.92982E+01_JPRB, 0.88969E+01_JPRB, 0.82533E+01_JPRB, 0.78430E+01_JPRB, 0.72862E+01_JPRB ,&
&  0.65901E+01_JPRB, 0.56500E+01_JPRB, 0.41886E+01_JPRB, 0.33559E+01_JPRB, 0.23359E+01_JPRB, 0.20734E+01_JPRB, 0.25528E+01_JPRB ,&
&  0.17743E+01_JPRB, 0.14569E+01_JPRB, 0.14367E+01_JPRB, 0.15294E+01_JPRB, 0.14997E+01_JPRB, 0.14611E+01_JPRB, 0.12504E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.11577E+01_JPRB, 0.15011E+01_JPRB, 0.15993E+01_JPRB, 0.15780E+01_JPRB, 0.14695E+01_JPRB, 0.12384E+01_JPRB ,&
&  0.12983E+01_JPRB, 0.12446E+01_JPRB, 0.12392E+01_JPRB, 0.12358E+01_JPRB, 0.12238E+01_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.11509E+01_JPRB, 0.10517E+01_JPRB, 0.91632E+00_JPRB, 0.87187E+00_JPRB, 0.88578E+00_JPRB, 0.87905E+00_JPRB, 0.86832E+00_JPRB ,&
&  0.87834E+00_JPRB, 0.88189E+00_JPRB, 0.88492E+00_JPRB, 0.89393E+00_JPRB, 0.90290E+00_JPRB, 0.92794E+00_JPRB, 0.84116E+00_JPRB ,&
&  0.75919E+00_JPRB, 0.78532E+00_JPRB, 0.77446E+00_JPRB, 0.83280E+00_JPRB, 0.10075E+01_JPRB, 0.10249E+01_JPRB, 0.10334E+01_JPRB ,&
&  0.10668E+01_JPRB, 0.10228E+01_JPRB, 0.10145E+01_JPRB, 0.98348E+00_JPRB, 0.93766E+00_JPRB, 0.94087E+00_JPRB, 0.94229E+00_JPRB ,&
&  0.85002E+00_JPRB, 0.79049E+00_JPRB, 0.76062E+00_JPRB, 0.74900E+00_JPRB, 0.80882E+00_JPRB, 0.88449E+00_JPRB, 0.91440E+00_JPRB ,&
&  0.92706E+00_JPRB, 0.95541E+00_JPRB, 0.10019E+01_JPRB, 0.10599E+01_JPRB, 0.11188E+01_JPRB, 0.11613E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.11930E+01_JPRB, 0.12159E+01_JPRB, 0.12545E+01_JPRB, 0.13974E+01_JPRB, 0.16772E+01_JPRB, 0.23254E+01_JPRB, 0.32521E+01_JPRB ,&
&  0.36771E+01_JPRB, 0.40157E+01_JPRB, 0.46536E+01_JPRB, 0.55695E+01_JPRB, 0.61217E+01_JPRB, 0.58629E+01_JPRB, 0.55654E+01_JPRB ,&
&  0.56412E+01_JPRB, 0.57524E+01_JPRB, 0.67524E+01_JPRB, 0.75212E+01_JPRB, 0.80653E+01_JPRB, 0.87097E+01_JPRB, 0.89836E+01_JPRB ,&
&  0.95163E+01_JPRB, 0.10002E+02_JPRB, 0.10039E+02_JPRB, 0.95502E+01_JPRB, 0.90695E+01_JPRB, 0.86537E+01_JPRB, 0.78754E+01_JPRB ,&
&  0.70861E+01_JPRB, 0.60683E+01_JPRB, 0.48044E+01_JPRB, 0.38023E+01_JPRB, 0.28514E+01_JPRB, 0.26175E+01_JPRB, 0.22497E+01_JPRB ,&
&  0.16608E+01_JPRB, 0.13805E+01_JPRB, 0.13347E+01_JPRB, 0.13758E+01_JPRB, 0.13135E+01_JPRB, 0.11775E+01_JPRB, 0.10879E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.12869E+01_JPRB, 0.14096E+01_JPRB, 0.15090E+01_JPRB, 0.15140E+01_JPRB, 0.14126E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.12041E+01_JPRB, 0.12099E+01_JPRB, 0.12086E+01_JPRB, 0.11995E+01_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.11344E+01_JPRB, 0.10640E+01_JPRB, 0.95604E+00_JPRB, 0.89650E+00_JPRB, 0.88701E+00_JPRB, 0.87203E+00_JPRB, 0.83484E+00_JPRB ,&
&  0.85867E+00_JPRB, 0.84057E+00_JPRB, 0.85398E+00_JPRB, 0.87928E+00_JPRB, 0.90143E+00_JPRB, 0.80430E+00_JPRB, 0.67564E+00_JPRB ,&
&  0.64476E+00_JPRB, 0.67409E+00_JPRB, 0.71628E+00_JPRB, 0.74107E+00_JPRB, 0.77368E+00_JPRB, 0.77939E+00_JPRB, 0.79091E+00_JPRB ,&
&  0.80230E+00_JPRB, 0.81755E+00_JPRB, 0.86168E+00_JPRB, 0.88178E+00_JPRB, 0.87785E+00_JPRB, 0.87666E+00_JPRB, 0.88282E+00_JPRB ,&
&  0.82268E+00_JPRB, 0.79774E+00_JPRB, 0.78177E+00_JPRB, 0.74920E+00_JPRB, 0.80573E+00_JPRB, 0.82049E+00_JPRB, 0.82625E+00_JPRB ,&
&  0.83212E+00_JPRB, 0.85065E+00_JPRB, 0.89009E+00_JPRB, 0.93831E+00_JPRB, 0.99155E+00_JPRB, 0.10405E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.11100E+01_JPRB, 0.11429E+01_JPRB, 0.11810E+01_JPRB, 0.12713E+01_JPRB, 0.15660E+01_JPRB, 0.19374E+01_JPRB, 0.25184E+01_JPRB ,&
&  0.27778E+01_JPRB, 0.31721E+01_JPRB, 0.36782E+01_JPRB, 0.36946E+01_JPRB, 0.40960E+01_JPRB, 0.45629E+01_JPRB, 0.45200E+01_JPRB ,&
&  0.46976E+01_JPRB, 0.47367E+01_JPRB, 0.51206E+01_JPRB, 0.56927E+01_JPRB, 0.64544E+01_JPRB, 0.71027E+01_JPRB, 0.76116E+01_JPRB ,&
&  0.83484E+01_JPRB, 0.95369E+01_JPRB, 0.10040E+02_JPRB, 0.96016E+01_JPRB, 0.90887E+01_JPRB, 0.86032E+01_JPRB, 0.78634E+01_JPRB ,&
&  0.71157E+01_JPRB, 0.62656E+01_JPRB, 0.52552E+01_JPRB, 0.42293E+01_JPRB, 0.33053E+01_JPRB, 0.27868E+01_JPRB, 0.22686E+01_JPRB ,&
&  0.17737E+01_JPRB, 0.14404E+01_JPRB, 0.14016E+01_JPRB, 0.13608E+01_JPRB, 0.12760E+01_JPRB, 0.11290E+01_JPRB, 0.10092E+01_JPRB ,&
&  0.10121E+01_JPRB, 0.11196E+01_JPRB, 0.13487E+01_JPRB, 0.13964E+01_JPRB, 0.14122E+01_JPRB, 0.13530E+01_JPRB, 0.11302E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.11946E+01_JPRB, 0.11835E+01_JPRB, 0.11904E+01_JPRB, 0.11802E+01_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.11426E+01_JPRB, 0.10999E+01_JPRB, 0.10409E+01_JPRB, 0.97218E+00_JPRB, 0.90637E+00_JPRB, 0.84985E+00_JPRB, 0.78418E+00_JPRB ,&
&  0.81855E+00_JPRB, 0.74561E+00_JPRB, 0.80975E+00_JPRB, 0.68744E+00_JPRB, 0.65222E+00_JPRB, 0.64198E+00_JPRB, 0.62521E+00_JPRB ,&
&  0.62230E+00_JPRB, 0.64571E+00_JPRB, 0.67801E+00_JPRB, 0.71171E+00_JPRB, 0.72270E+00_JPRB, 0.72823E+00_JPRB, 0.74514E+00_JPRB ,&
&  0.77234E+00_JPRB, 0.81086E+00_JPRB, 0.85534E+00_JPRB, 0.87228E+00_JPRB, 0.87329E+00_JPRB, 0.86279E+00_JPRB, 0.85560E+00_JPRB ,&
&  0.79821E+00_JPRB, 0.77886E+00_JPRB, 0.77690E+00_JPRB, 0.77074E+00_JPRB, 0.76345E+00_JPRB, 0.74264E+00_JPRB, 0.74451E+00_JPRB ,&
&  0.71747E+00_JPRB, 0.68712E+00_JPRB, 0.81425E+00_JPRB, 0.85090E+00_JPRB, 0.89858E+00_JPRB, 0.94739E+00_JPRB, 0.99517E+00_JPRB ,&
&  0.10377E+01_JPRB, 0.10801E+01_JPRB, 0.11290E+01_JPRB, 0.11856E+01_JPRB, 0.13358E+01_JPRB, 0.16287E+01_JPRB, 0.19490E+01_JPRB ,&
&  0.23408E+01_JPRB, 0.19805E+01_JPRB, 0.23640E+01_JPRB, 0.26369E+01_JPRB, 0.33164E+01_JPRB, 0.40214E+01_JPRB, 0.43254E+01_JPRB ,&
&  0.47354E+01_JPRB, 0.47310E+01_JPRB, 0.46554E+01_JPRB, 0.45891E+01_JPRB, 0.49645E+01_JPRB, 0.55178E+01_JPRB, 0.61316E+01_JPRB ,&
&  0.67798E+01_JPRB, 0.74491E+01_JPRB, 0.86757E+01_JPRB, 0.85941E+01_JPRB, 0.83183E+01_JPRB, 0.80147E+01_JPRB, 0.74992E+01_JPRB ,&
&  0.68551E+01_JPRB, 0.60751E+01_JPRB, 0.51226E+01_JPRB, 0.40315E+01_JPRB, 0.32992E+01_JPRB, 0.26937E+01_JPRB, 0.21943E+01_JPRB ,&
&  0.18555E+01_JPRB, 0.16526E+01_JPRB, 0.15947E+01_JPRB, 0.15096E+01_JPRB, 0.14086E+01_JPRB, 0.12897E+01_JPRB, 0.11182E+01_JPRB ,&
&  0.93377E+00_JPRB, 0.93504E+00_JPRB, 0.97427E+00_JPRB, 0.10788E+01_JPRB, 0.11428E+01_JPRB, 0.11663E+01_JPRB, 0.12166E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.11565E+01_JPRB, 0.12172E+01_JPRB, 0.11865E+01_JPRB, 0.11709E+01_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.12175E+01_JPRB, 0.11665E+01_JPRB, 0.11160E+01_JPRB, 0.10628E+01_JPRB, 0.10082E+01_JPRB, 0.93047E+00_JPRB, 0.81794E+00_JPRB ,&
&  0.71318E+00_JPRB, 0.69259E+00_JPRB, 0.70218E+00_JPRB, 0.68384E+00_JPRB, 0.65651E+00_JPRB, 0.64495E+00_JPRB, 0.65774E+00_JPRB ,&
&  0.68495E+00_JPRB, 0.74437E+00_JPRB, 0.79142E+00_JPRB, 0.81128E+00_JPRB, 0.80755E+00_JPRB, 0.79791E+00_JPRB, 0.80517E+00_JPRB ,&
&  0.81742E+00_JPRB, 0.83023E+00_JPRB, 0.83654E+00_JPRB, 0.82486E+00_JPRB, 0.81175E+00_JPRB, 0.80407E+00_JPRB, 0.79484E+00_JPRB ,&
&  0.78568E+00_JPRB, 0.77553E+00_JPRB, 0.75227E+00_JPRB, 0.74967E+00_JPRB, 0.72537E+00_JPRB, 0.71186E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.58827E+00_JPRB, 0.59778E+00_JPRB, 0.74840E+00_JPRB, 0.80849E+00_JPRB, 0.83170E+00_JPRB, 0.87714E+00_JPRB, 0.93995E+00_JPRB ,&
&  0.98951E+00_JPRB, 0.10313E+01_JPRB, 0.10747E+01_JPRB, 0.11336E+01_JPRB, 0.12208E+01_JPRB, 0.13898E+01_JPRB, 0.15908E+01_JPRB ,&
&  0.18837E+01_JPRB, 0.16736E+01_JPRB, 0.16114E+01_JPRB, 0.18576E+01_JPRB, 0.26942E+01_JPRB, 0.32849E+01_JPRB, 0.34932E+01_JPRB ,&
&  0.36030E+01_JPRB, 0.34880E+01_JPRB, 0.33834E+01_JPRB, 0.34107E+01_JPRB, 0.35981E+01_JPRB, 0.39442E+01_JPRB, 0.45026E+01_JPRB ,&
&  0.49767E+01_JPRB, 0.57780E+01_JPRB, 0.69228E+01_JPRB, 0.71813E+01_JPRB, 0.70391E+01_JPRB, 0.69037E+01_JPRB, 0.66154E+01_JPRB ,&
&  0.62819E+01_JPRB, 0.56425E+01_JPRB, 0.46280E+01_JPRB, 0.37802E+01_JPRB, 0.30674E+01_JPRB, 0.25309E+01_JPRB, 0.21083E+01_JPRB ,&
&  0.18971E+01_JPRB, 0.18280E+01_JPRB, 0.17726E+01_JPRB, 0.16766E+01_JPRB, 0.15336E+01_JPRB, 0.13875E+01_JPRB, 0.12415E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.94598E+00_JPRB, 0.87600E+00_JPRB, 0.89791E+00_JPRB, 0.98833E+00_JPRB, 0.10917E+01_JPRB, 0.11109E+01_JPRB ,&
&  0.11337E+01_JPRB, 0.11032E+01_JPRB, 0.10958E+01_JPRB, 0.11507E+01_JPRB, 0.12153E+01_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.11957E+01_JPRB, 0.11645E+01_JPRB, 0.11183E+01_JPRB, 0.10975E+01_JPRB, 0.10526E+01_JPRB, 0.10026E+01_JPRB, 0.88446E+00_JPRB ,&
&  0.75072E+00_JPRB, 0.72759E+00_JPRB, 0.78011E+00_JPRB, 0.81143E+00_JPRB, 0.79846E+00_JPRB, 0.79857E+00_JPRB, 0.81499E+00_JPRB ,&
&  0.82981E+00_JPRB, 0.87482E+00_JPRB, 0.87403E+00_JPRB, 0.84440E+00_JPRB, 0.82010E+00_JPRB, 0.80334E+00_JPRB, 0.80127E+00_JPRB ,&
&  0.79847E+00_JPRB, 0.79714E+00_JPRB, 0.78717E+00_JPRB, 0.76957E+00_JPRB, 0.76720E+00_JPRB, 0.76363E+00_JPRB, 0.75625E+00_JPRB ,&
&  0.74837E+00_JPRB, 0.73388E+00_JPRB, 0.73713E+00_JPRB, 0.73818E+00_JPRB, 0.72247E+00_JPRB, 0.67945E+00_JPRB, 0.59453E+00_JPRB ,&
&  0.55574E+00_JPRB, 0.53944E+00_JPRB, 0.57355E+00_JPRB, 0.68108E+00_JPRB, 0.74508E+00_JPRB, 0.79950E+00_JPRB, 0.85782E+00_JPRB ,&
&  0.82573E+00_JPRB, 0.82079E+00_JPRB, 0.86394E+00_JPRB, 0.10028E+01_JPRB, 0.10809E+01_JPRB, 0.11800E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.14296E+01_JPRB, 0.15351E+01_JPRB, 0.15762E+01_JPRB, 0.13960E+01_JPRB, 0.16206E+01_JPRB, 0.22522E+01_JPRB, 0.22993E+01_JPRB ,&
&  0.25125E+01_JPRB, 0.27036E+01_JPRB, 0.28894E+01_JPRB, 0.28492E+01_JPRB, 0.27013E+01_JPRB, 0.27782E+01_JPRB, 0.30894E+01_JPRB ,&
&  0.35347E+01_JPRB, 0.41839E+01_JPRB, 0.53584E+01_JPRB, 0.60011E+01_JPRB, 0.58959E+01_JPRB, 0.57515E+01_JPRB, 0.55908E+01_JPRB ,&
&  0.54750E+01_JPRB, 0.48595E+01_JPRB, 0.40741E+01_JPRB, 0.33695E+01_JPRB, 0.27698E+01_JPRB, 0.23010E+01_JPRB, 0.20087E+01_JPRB ,&
&  0.18844E+01_JPRB, 0.18154E+01_JPRB, 0.17894E+01_JPRB, 0.17019E+01_JPRB, 0.15255E+01_JPRB, 0.13431E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.10154E+01_JPRB, 0.94492E+00_JPRB, 0.92147E+00_JPRB, 0.97822E+00_JPRB, 0.10534E+01_JPRB, 0.10676E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10232E+01_JPRB, 0.10745E+01_JPRB, 0.11382E+01_JPRB, 0.11624E+01_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.11830E+01_JPRB, 0.12005E+01_JPRB, 0.11894E+01_JPRB, 0.11592E+01_JPRB, 0.11225E+01_JPRB, 0.10234E+01_JPRB, 0.92511E+00_JPRB ,&
&  0.85882E+00_JPRB, 0.87499E+00_JPRB, 0.92605E+00_JPRB, 0.97016E+00_JPRB, 0.97773E+00_JPRB, 0.95131E+00_JPRB, 0.93707E+00_JPRB ,&
&  0.94916E+00_JPRB, 0.94976E+00_JPRB, 0.90530E+00_JPRB, 0.86732E+00_JPRB, 0.80691E+00_JPRB, 0.79023E+00_JPRB, 0.79078E+00_JPRB ,&
&  0.79627E+00_JPRB, 0.79644E+00_JPRB, 0.79934E+00_JPRB, 0.79568E+00_JPRB, 0.79462E+00_JPRB, 0.77651E+00_JPRB, 0.74811E+00_JPRB ,&
&  0.71891E+00_JPRB, 0.71009E+00_JPRB, 0.71033E+00_JPRB, 0.71267E+00_JPRB, 0.69709E+00_JPRB, 0.67419E+00_JPRB, 0.58854E+00_JPRB ,&
&  0.55597E+00_JPRB, 0.53410E+00_JPRB, 0.53029E+00_JPRB, 0.54447E+00_JPRB, 0.59960E+00_JPRB, 0.63982E+00_JPRB, 0.74362E+00_JPRB ,&
&  0.81668E+00_JPRB, 0.87893E+00_JPRB, 0.89831E+00_JPRB, 0.91230E+00_JPRB, 0.94326E+00_JPRB, 0.10214E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.12181E+01_JPRB, 0.12952E+01_JPRB, 0.13135E+01_JPRB, 0.11949E+01_JPRB, 0.12436E+01_JPRB, 0.14720E+01_JPRB, 0.17810E+01_JPRB ,&
&  0.21058E+01_JPRB, 0.22017E+01_JPRB, 0.22196E+01_JPRB, 0.23884E+01_JPRB, 0.24733E+01_JPRB, 0.25527E+01_JPRB, 0.26320E+01_JPRB ,&
&  0.30699E+01_JPRB, 0.36749E+01_JPRB, 0.45641E+01_JPRB, 0.50185E+01_JPRB, 0.48917E+01_JPRB, 0.49245E+01_JPRB, 0.46873E+01_JPRB ,&
&  0.44928E+01_JPRB, 0.39365E+01_JPRB, 0.34367E+01_JPRB, 0.28774E+01_JPRB, 0.24291E+01_JPRB, 0.21726E+01_JPRB, 0.20180E+01_JPRB ,&
&  0.18815E+01_JPRB, 0.17714E+01_JPRB, 0.17482E+01_JPRB, 0.16750E+01_JPRB, 0.15279E+01_JPRB, 0.14030E+01_JPRB, 0.13120E+01_JPRB ,&
&  0.12455E+01_JPRB, 0.12018E+01_JPRB, 0.11846E+01_JPRB, 0.11396E+01_JPRB, 0.11273E+01_JPRB, 0.11312E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.10939E+01_JPRB, 0.10828E+01_JPRB, 0.10945E+01_JPRB, 0.11189E+01_JPRB, 0.11432E+01_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.12171E+01_JPRB, 0.12429E+01_JPRB, 0.12327E+01_JPRB, 0.12274E+01_JPRB, 0.12229E+01_JPRB, 0.11575E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.10928E+01_JPRB, 0.11192E+01_JPRB, 0.11327E+01_JPRB, 0.11348E+01_JPRB, 0.11212E+01_JPRB, 0.11032E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.10775E+01_JPRB, 0.10419E+01_JPRB, 0.96996E+00_JPRB, 0.87138E+00_JPRB, 0.83809E+00_JPRB, 0.82539E+00_JPRB, 0.82807E+00_JPRB ,&
&  0.82778E+00_JPRB, 0.81856E+00_JPRB, 0.81756E+00_JPRB, 0.79621E+00_JPRB, 0.78008E+00_JPRB, 0.73917E+00_JPRB, 0.70415E+00_JPRB ,&
&  0.70086E+00_JPRB, 0.69340E+00_JPRB, 0.70790E+00_JPRB, 0.68788E+00_JPRB, 0.65880E+00_JPRB, 0.63896E+00_JPRB, 0.57487E+00_JPRB ,&
&  0.56388E+00_JPRB, 0.54249E+00_JPRB, 0.53457E+00_JPRB, 0.53518E+00_JPRB, 0.56093E+00_JPRB, 0.58400E+00_JPRB, 0.63109E+00_JPRB ,&
&  0.68415E+00_JPRB, 0.83478E+00_JPRB, 0.88844E+00_JPRB, 0.93014E+00_JPRB, 0.96279E+00_JPRB, 0.10113E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.11622E+01_JPRB, 0.12477E+01_JPRB, 0.13302E+01_JPRB, 0.14772E+01_JPRB, 0.16297E+01_JPRB, 0.16565E+01_JPRB, 0.18373E+01_JPRB ,&
&  0.24315E+01_JPRB, 0.23184E+01_JPRB, 0.22065E+01_JPRB, 0.21727E+01_JPRB, 0.22378E+01_JPRB, 0.23527E+01_JPRB, 0.25274E+01_JPRB ,&
&  0.28529E+01_JPRB, 0.33428E+01_JPRB, 0.40320E+01_JPRB, 0.44946E+01_JPRB, 0.45250E+01_JPRB, 0.45800E+01_JPRB, 0.44912E+01_JPRB ,&
&  0.41419E+01_JPRB, 0.38907E+01_JPRB, 0.39792E+01_JPRB, 0.32248E+01_JPRB, 0.27110E+01_JPRB, 0.24321E+01_JPRB, 0.22935E+01_JPRB ,&
&  0.21527E+01_JPRB, 0.20069E+01_JPRB, 0.19161E+01_JPRB, 0.17997E+01_JPRB, 0.16515E+01_JPRB, 0.15627E+01_JPRB, 0.15400E+01_JPRB ,&
&  0.15192E+01_JPRB, 0.14849E+01_JPRB, 0.14528E+01_JPRB, 0.14130E+01_JPRB, 0.13548E+01_JPRB, 0.12898E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.11960E+01_JPRB, 0.11777E+01_JPRB, 0.11711E+01_JPRB, 0.11829E+01_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.12368E+01_JPRB, 0.12629E+01_JPRB, 0.12566E+01_JPRB, 0.12537E+01_JPRB, 0.12494E+01_JPRB, 0.12261E+01_JPRB, 0.12084E+01_JPRB ,&
&  0.12017E+01_JPRB, 0.12019E+01_JPRB, 0.11926E+01_JPRB, 0.11760E+01_JPRB, 0.11538E+01_JPRB, 0.11384E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.10240E+01_JPRB, 0.97713E+00_JPRB, 0.93624E+00_JPRB, 0.89690E+00_JPRB, 0.87497E+00_JPRB, 0.86082E+00_JPRB ,&
&  0.84055E+00_JPRB, 0.82791E+00_JPRB, 0.81656E+00_JPRB, 0.79751E+00_JPRB, 0.77710E+00_JPRB, 0.74728E+00_JPRB, 0.73811E+00_JPRB ,&
&  0.75104E+00_JPRB, 0.73159E+00_JPRB, 0.70732E+00_JPRB, 0.67622E+00_JPRB, 0.62587E+00_JPRB, 0.60560E+00_JPRB, 0.60235E+00_JPRB ,&
&  0.58860E+00_JPRB, 0.55343E+00_JPRB, 0.55563E+00_JPRB, 0.55898E+00_JPRB, 0.56254E+00_JPRB, 0.59085E+00_JPRB, 0.64090E+00_JPRB ,&
&  0.69156E+00_JPRB, 0.84350E+00_JPRB, 0.90183E+00_JPRB, 0.96837E+00_JPRB, 0.10101E+01_JPRB, 0.10415E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.11153E+01_JPRB, 0.11794E+01_JPRB, 0.12575E+01_JPRB, 0.13653E+01_JPRB, 0.14932E+01_JPRB, 0.15996E+01_JPRB, 0.16882E+01_JPRB ,&
&  0.18086E+01_JPRB, 0.18038E+01_JPRB, 0.18583E+01_JPRB, 0.19037E+01_JPRB, 0.19807E+01_JPRB, 0.21321E+01_JPRB, 0.22253E+01_JPRB ,&
&  0.24145E+01_JPRB, 0.26801E+01_JPRB, 0.30935E+01_JPRB, 0.34904E+01_JPRB, 0.37026E+01_JPRB, 0.39080E+01_JPRB, 0.39706E+01_JPRB ,&
&  0.37625E+01_JPRB, 0.36428E+01_JPRB, 0.37223E+01_JPRB, 0.34059E+01_JPRB, 0.30525E+01_JPRB, 0.27466E+01_JPRB, 0.25190E+01_JPRB ,&
&  0.22987E+01_JPRB, 0.20847E+01_JPRB, 0.19413E+01_JPRB, 0.18222E+01_JPRB, 0.17009E+01_JPRB, 0.15911E+01_JPRB, 0.15605E+01_JPRB ,&
&  0.15322E+01_JPRB, 0.14996E+01_JPRB, 0.14575E+01_JPRB, 0.14099E+01_JPRB, 0.13403E+01_JPRB, 0.12736E+01_JPRB, 0.12196E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.11822E+01_JPRB, 0.11820E+01_JPRB, 0.11831E+01_JPRB, 0.12023E+01_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.12423E+01_JPRB, 0.12487E+01_JPRB, 0.12323E+01_JPRB, 0.12158E+01_JPRB, 0.12096E+01_JPRB, 0.11928E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.11953E+01_JPRB, 0.11962E+01_JPRB, 0.11785E+01_JPRB, 0.11551E+01_JPRB, 0.11258E+01_JPRB, 0.11074E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10636E+01_JPRB, 0.10340E+01_JPRB, 0.10021E+01_JPRB, 0.96832E+00_JPRB, 0.94110E+00_JPRB, 0.90943E+00_JPRB, 0.88411E+00_JPRB ,&
&  0.85835E+00_JPRB, 0.82783E+00_JPRB, 0.80105E+00_JPRB, 0.79300E+00_JPRB, 0.77717E+00_JPRB, 0.75798E+00_JPRB, 0.75712E+00_JPRB ,&
&  0.75217E+00_JPRB, 0.74101E+00_JPRB, 0.73311E+00_JPRB, 0.70876E+00_JPRB, 0.65469E+00_JPRB, 0.56319E+00_JPRB, 0.58664E+00_JPRB ,&
&  0.57821E+00_JPRB, 0.56118E+00_JPRB, 0.56676E+00_JPRB, 0.58351E+00_JPRB, 0.60021E+00_JPRB, 0.64817E+00_JPRB, 0.65609E+00_JPRB ,&
&  0.64010E+00_JPRB, 0.85122E+00_JPRB, 0.89458E+00_JPRB, 0.94460E+00_JPRB, 0.98622E+00_JPRB, 0.10120E+01_JPRB, 0.10358E+01_JPRB ,&
&  0.10744E+01_JPRB, 0.11225E+01_JPRB, 0.11764E+01_JPRB, 0.12539E+01_JPRB, 0.13537E+01_JPRB, 0.14809E+01_JPRB, 0.15918E+01_JPRB ,&
&  0.16618E+01_JPRB, 0.17136E+01_JPRB, 0.17805E+01_JPRB, 0.18435E+01_JPRB, 0.19130E+01_JPRB, 0.20237E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.23495E+01_JPRB, 0.24839E+01_JPRB, 0.27376E+01_JPRB, 0.29512E+01_JPRB, 0.30852E+01_JPRB, 0.32487E+01_JPRB, 0.33889E+01_JPRB ,&
&  0.33544E+01_JPRB, 0.33503E+01_JPRB, 0.32836E+01_JPRB, 0.30112E+01_JPRB, 0.27079E+01_JPRB, 0.24925E+01_JPRB, 0.22954E+01_JPRB ,&
&  0.21347E+01_JPRB, 0.19746E+01_JPRB, 0.18164E+01_JPRB, 0.16875E+01_JPRB, 0.15849E+01_JPRB, 0.15052E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.14069E+01_JPRB, 0.13667E+01_JPRB, 0.13294E+01_JPRB, 0.13088E+01_JPRB, 0.12783E+01_JPRB, 0.12533E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12164E+01_JPRB, 0.12161E+01_JPRB, 0.12193E+01_JPRB, 0.12219E+01_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.12453E+01_JPRB, 0.12542E+01_JPRB, 0.12576E+01_JPRB, 0.12394E+01_JPRB, 0.12308E+01_JPRB, 0.12231E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.12071E+01_JPRB, 0.11673E+01_JPRB, 0.11414E+01_JPRB, 0.11153E+01_JPRB, 0.11032E+01_JPRB, 0.10911E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10708E+01_JPRB, 0.10631E+01_JPRB, 0.10402E+01_JPRB, 0.10163E+01_JPRB, 0.97661E+00_JPRB, 0.93409E+00_JPRB ,&
&  0.89112E+00_JPRB, 0.84851E+00_JPRB, 0.81063E+00_JPRB, 0.80492E+00_JPRB, 0.78907E+00_JPRB, 0.77897E+00_JPRB, 0.76619E+00_JPRB ,&
&  0.74785E+00_JPRB, 0.67948E+00_JPRB, 0.69937E+00_JPRB, 0.67408E+00_JPRB, 0.64633E+00_JPRB, 0.61600E+00_JPRB, 0.59126E+00_JPRB ,&
&  0.59433E+00_JPRB, 0.59686E+00_JPRB, 0.61616E+00_JPRB, 0.65246E+00_JPRB, 0.67106E+00_JPRB, 0.70350E+00_JPRB, 0.72367E+00_JPRB ,&
&  0.71595E+00_JPRB, 0.86037E+00_JPRB, 0.93548E+00_JPRB, 0.98427E+00_JPRB, 0.10283E+01_JPRB, 0.10646E+01_JPRB, 0.10930E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11895E+01_JPRB, 0.12496E+01_JPRB, 0.13117E+01_JPRB, 0.14098E+01_JPRB, 0.14845E+01_JPRB, 0.15524E+01_JPRB ,&
&  0.16006E+01_JPRB, 0.16330E+01_JPRB, 0.16795E+01_JPRB, 0.17569E+01_JPRB, 0.18505E+01_JPRB, 0.19538E+01_JPRB, 0.20650E+01_JPRB ,&
&  0.21880E+01_JPRB, 0.23103E+01_JPRB, 0.24407E+01_JPRB, 0.25625E+01_JPRB, 0.26704E+01_JPRB, 0.27492E+01_JPRB, 0.28205E+01_JPRB ,&
&  0.27994E+01_JPRB, 0.27572E+01_JPRB, 0.26767E+01_JPRB, 0.25505E+01_JPRB, 0.23820E+01_JPRB, 0.21935E+01_JPRB, 0.20031E+01_JPRB ,&
&  0.18915E+01_JPRB, 0.17686E+01_JPRB, 0.16576E+01_JPRB, 0.15523E+01_JPRB, 0.14660E+01_JPRB, 0.13975E+01_JPRB, 0.13569E+01_JPRB ,&
&  0.13192E+01_JPRB, 0.12936E+01_JPRB, 0.12757E+01_JPRB, 0.12630E+01_JPRB, 0.12576E+01_JPRB, 0.12444E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.12402E+01_JPRB, 0.12416E+01_JPRB, 0.12332E+01_JPRB, 0.12314E+01_JPRB, 0.12360E+01_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.12557E+01_JPRB, 0.12680E+01_JPRB, 0.12903E+01_JPRB, 0.12951E+01_JPRB, 0.12962E+01_JPRB, 0.12858E+01_JPRB, 0.12820E+01_JPRB ,&
&  0.12649E+01_JPRB, 0.12451E+01_JPRB, 0.12014E+01_JPRB, 0.11724E+01_JPRB, 0.11474E+01_JPRB, 0.11325E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.11066E+01_JPRB, 0.10936E+01_JPRB, 0.10710E+01_JPRB, 0.10403E+01_JPRB, 0.10034E+01_JPRB, 0.95757E+00_JPRB ,&
&  0.91924E+00_JPRB, 0.89062E+00_JPRB, 0.85251E+00_JPRB, 0.85706E+00_JPRB, 0.83540E+00_JPRB, 0.82004E+00_JPRB, 0.80997E+00_JPRB ,&
&  0.78377E+00_JPRB, 0.76056E+00_JPRB, 0.72300E+00_JPRB, 0.70012E+00_JPRB, 0.68192E+00_JPRB, 0.66009E+00_JPRB, 0.67995E+00_JPRB ,&
&  0.69664E+00_JPRB, 0.74925E+00_JPRB, 0.77629E+00_JPRB, 0.78901E+00_JPRB, 0.80034E+00_JPRB, 0.79994E+00_JPRB, 0.81306E+00_JPRB ,&
&  0.83070E+00_JPRB, 0.85943E+00_JPRB, 0.95269E+00_JPRB, 0.10162E+01_JPRB, 0.10565E+01_JPRB, 0.10998E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.12197E+01_JPRB, 0.12483E+01_JPRB, 0.12809E+01_JPRB, 0.13389E+01_JPRB, 0.14072E+01_JPRB, 0.14675E+01_JPRB ,&
&  0.15180E+01_JPRB, 0.15531E+01_JPRB, 0.15958E+01_JPRB, 0.16450E+01_JPRB, 0.17090E+01_JPRB, 0.17805E+01_JPRB, 0.18570E+01_JPRB ,&
&  0.19339E+01_JPRB, 0.20110E+01_JPRB, 0.20760E+01_JPRB, 0.21339E+01_JPRB, 0.21695E+01_JPRB, 0.21861E+01_JPRB, 0.21928E+01_JPRB ,&
&  0.21630E+01_JPRB, 0.21218E+01_JPRB, 0.20580E+01_JPRB, 0.19548E+01_JPRB, 0.18729E+01_JPRB, 0.18238E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.16677E+01_JPRB, 0.15960E+01_JPRB, 0.15292E+01_JPRB, 0.14614E+01_JPRB, 0.14025E+01_JPRB, 0.13497E+01_JPRB, 0.13149E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.12688E+01_JPRB, 0.12587E+01_JPRB, 0.12519E+01_JPRB, 0.12533E+01_JPRB, 0.12513E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.12563E+01_JPRB, 0.12453E+01_JPRB, 0.12432E+01_JPRB, 0.12455E+01_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.12680E+01_JPRB, 0.12706E+01_JPRB, 0.12787E+01_JPRB, 0.12828E+01_JPRB, 0.12849E+01_JPRB, 0.12816E+01_JPRB, 0.12789E+01_JPRB ,&
&  0.12696E+01_JPRB, 0.12513E+01_JPRB, 0.12208E+01_JPRB, 0.11950E+01_JPRB, 0.11709E+01_JPRB, 0.11540E+01_JPRB, 0.11409E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11179E+01_JPRB, 0.11047E+01_JPRB, 0.10890E+01_JPRB, 0.10684E+01_JPRB, 0.10457E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10008E+01_JPRB, 0.98172E+00_JPRB, 0.96603E+00_JPRB, 0.95486E+00_JPRB, 0.95019E+00_JPRB, 0.89997E+00_JPRB, 0.88885E+00_JPRB ,&
&  0.90119E+00_JPRB, 0.87920E+00_JPRB, 0.85918E+00_JPRB, 0.82917E+00_JPRB, 0.82707E+00_JPRB, 0.83981E+00_JPRB, 0.84822E+00_JPRB ,&
&  0.82005E+00_JPRB, 0.87088E+00_JPRB, 0.87714E+00_JPRB, 0.87941E+00_JPRB, 0.87519E+00_JPRB, 0.86366E+00_JPRB, 0.88284E+00_JPRB ,&
&  0.93467E+00_JPRB, 0.95559E+00_JPRB, 0.10027E+01_JPRB, 0.10595E+01_JPRB, 0.11056E+01_JPRB, 0.11538E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.12909E+01_JPRB, 0.13170E+01_JPRB, 0.13421E+01_JPRB, 0.13818E+01_JPRB, 0.14220E+01_JPRB, 0.14589E+01_JPRB ,&
&  0.14844E+01_JPRB, 0.14999E+01_JPRB, 0.15173E+01_JPRB, 0.15343E+01_JPRB, 0.15566E+01_JPRB, 0.15754E+01_JPRB, 0.15947E+01_JPRB ,&
&  0.16090E+01_JPRB, 0.16215E+01_JPRB, 0.16280E+01_JPRB, 0.16301E+01_JPRB, 0.16286E+01_JPRB, 0.16214E+01_JPRB, 0.16209E+01_JPRB ,&
&  0.16070E+01_JPRB, 0.15883E+01_JPRB, 0.15620E+01_JPRB, 0.15299E+01_JPRB, 0.15035E+01_JPRB, 0.14818E+01_JPRB, 0.14581E+01_JPRB ,&
&  0.14342E+01_JPRB, 0.14104E+01_JPRB, 0.13872E+01_JPRB, 0.13623E+01_JPRB, 0.13384E+01_JPRB, 0.13146E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.12850E+01_JPRB, 0.12755E+01_JPRB, 0.12728E+01_JPRB, 0.12687E+01_JPRB, 0.12723E+01_JPRB, 0.12717E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.12782E+01_JPRB, 0.12751E+01_JPRB, 0.12683E+01_JPRB, 0.12650E+01_JPRB, 0.12640E+01_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.12527E+01_JPRB, 0.12507E+01_JPRB, 0.12499E+01_JPRB, 0.12511E+01_JPRB, 0.12528E+01_JPRB, 0.12532E+01_JPRB, 0.12517E+01_JPRB ,&
&  0.12466E+01_JPRB, 0.12343E+01_JPRB, 0.12198E+01_JPRB, 0.12041E+01_JPRB, 0.11858E+01_JPRB, 0.11683E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.11366E+01_JPRB, 0.11257E+01_JPRB, 0.11169E+01_JPRB, 0.11063E+01_JPRB, 0.10948E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10676E+01_JPRB, 0.10584E+01_JPRB, 0.10475E+01_JPRB, 0.10393E+01_JPRB, 0.10304E+01_JPRB, 0.10222E+01_JPRB ,&
&  0.10124E+01_JPRB, 0.10065E+01_JPRB, 0.10024E+01_JPRB, 0.10001E+01_JPRB, 0.10031E+01_JPRB, 0.10065E+01_JPRB, 0.10060E+01_JPRB ,&
&  0.10035E+01_JPRB, 0.99589E+00_JPRB, 0.99626E+00_JPRB, 0.99786E+00_JPRB, 0.10025E+01_JPRB, 0.10124E+01_JPRB, 0.10266E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.10742E+01_JPRB, 0.11036E+01_JPRB, 0.11286E+01_JPRB, 0.11543E+01_JPRB, 0.11776E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.12315E+01_JPRB, 0.12470E+01_JPRB, 0.12616E+01_JPRB, 0.12810E+01_JPRB, 0.12967E+01_JPRB, 0.13100E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.13148E+01_JPRB, 0.13142E+01_JPRB, 0.13139E+01_JPRB, 0.13106E+01_JPRB, 0.13087E+01_JPRB, 0.13087E+01_JPRB ,&
&  0.13103E+01_JPRB, 0.13115E+01_JPRB, 0.13156E+01_JPRB, 0.13204E+01_JPRB, 0.13255E+01_JPRB, 0.13258E+01_JPRB, 0.13266E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.13228E+01_JPRB, 0.13168E+01_JPRB, 0.13073E+01_JPRB, 0.12994E+01_JPRB, 0.12944E+01_JPRB, 0.12879E+01_JPRB ,&
&  0.12814E+01_JPRB, 0.12763E+01_JPRB, 0.12726E+01_JPRB, 0.12680E+01_JPRB, 0.12611E+01_JPRB, 0.12550E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.12484E+01_JPRB, 0.12450E+01_JPRB, 0.12454E+01_JPRB, 0.12461E+01_JPRB, 0.12539E+01_JPRB, 0.12570E+01_JPRB, 0.12600E+01_JPRB ,&
&  0.12609E+01_JPRB, 0.12641E+01_JPRB, 0.12637E+01_JPRB, 0.12605E+01_JPRB, 0.12560E+01_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.12488E+01_JPRB, 0.12459E+01_JPRB, 0.12434E+01_JPRB, 0.12417E+01_JPRB, 0.12407E+01_JPRB, 0.12395E+01_JPRB, 0.12362E+01_JPRB ,&
&  0.12319E+01_JPRB, 0.12262E+01_JPRB, 0.12204E+01_JPRB, 0.12133E+01_JPRB, 0.12045E+01_JPRB, 0.11938E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11685E+01_JPRB, 0.11611E+01_JPRB, 0.11557E+01_JPRB, 0.11501E+01_JPRB, 0.11449E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.11330E+01_JPRB, 0.11278E+01_JPRB, 0.11228E+01_JPRB, 0.11185E+01_JPRB, 0.11165E+01_JPRB, 0.11139E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.11087E+01_JPRB, 0.11069E+01_JPRB, 0.11052E+01_JPRB, 0.11035E+01_JPRB, 0.11023E+01_JPRB, 0.11023E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.11029E+01_JPRB, 0.11050E+01_JPRB, 0.11087E+01_JPRB, 0.11120E+01_JPRB, 0.11158E+01_JPRB, 0.11214E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11424E+01_JPRB, 0.11501E+01_JPRB, 0.11588E+01_JPRB, 0.11668E+01_JPRB, 0.11747E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11887E+01_JPRB, 0.11943E+01_JPRB, 0.12002E+01_JPRB, 0.12031E+01_JPRB, 0.12080E+01_JPRB, 0.12121E+01_JPRB, 0.12170E+01_JPRB ,&
&  0.12184E+01_JPRB, 0.12194E+01_JPRB, 0.12209E+01_JPRB, 0.12233E+01_JPRB, 0.12258E+01_JPRB, 0.12284E+01_JPRB, 0.12312E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.12374E+01_JPRB, 0.12405E+01_JPRB, 0.12432E+01_JPRB, 0.12462E+01_JPRB, 0.12473E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12522E+01_JPRB, 0.12518E+01_JPRB, 0.12515E+01_JPRB, 0.12512E+01_JPRB, 0.12513E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.12520E+01_JPRB, 0.12523E+01_JPRB, 0.12525E+01_JPRB, 0.12516E+01_JPRB, 0.12500E+01_JPRB, 0.12486E+01_JPRB, 0.12476E+01_JPRB ,&
&  0.12476E+01_JPRB, 0.12473E+01_JPRB, 0.12482E+01_JPRB, 0.12493E+01_JPRB, 0.12525E+01_JPRB, 0.12537E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.12556E+01_JPRB, 0.12561E+01_JPRB, 0.12552E+01_JPRB, 0.12536E+01_JPRB, 0.12512E+01_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB ,&
&  0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB, 0.12066E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB ,&
&  0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB, 0.15004E+00_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.15230E+00_JPRB, 0.15306E+00_JPRB, 0.15360E+00_JPRB, 0.15404E+00_JPRB, 0.15387E+00_JPRB, 0.15393E+00_JPRB, 0.15399E+00_JPRB ,&
&  0.15398E+00_JPRB, 0.15395E+00_JPRB, 0.15421E+00_JPRB, 0.15444E+00_JPRB, 0.15470E+00_JPRB, 0.15498E+00_JPRB, 0.15529E+00_JPRB ,&
&  0.15543E+00_JPRB, 0.15572E+00_JPRB, 0.15462E+00_JPRB, 0.15373E+00_JPRB, 0.15317E+00_JPRB, 0.15283E+00_JPRB, 0.15265E+00_JPRB ,&
&  0.15251E+00_JPRB, 0.15242E+00_JPRB, 0.15230E+00_JPRB, 0.15311E+00_JPRB, 0.15364E+00_JPRB, 0.15411E+00_JPRB, 0.15448E+00_JPRB ,&
&  0.15463E+00_JPRB, 0.15485E+00_JPRB, 0.15503E+00_JPRB, 0.15522E+00_JPRB, 0.15545E+00_JPRB, 0.15542E+00_JPRB, 0.15530E+00_JPRB ,&
&  0.15512E+00_JPRB, 0.15481E+00_JPRB, 0.15446E+00_JPRB, 0.15412E+00_JPRB, 0.15385E+00_JPRB, 0.15353E+00_JPRB, 0.15319E+00_JPRB ,&
&  0.15285E+00_JPRB, 0.15246E+00_JPRB, 0.15209E+00_JPRB, 0.15173E+00_JPRB, 0.15139E+00_JPRB, 0.15110E+00_JPRB, 0.15078E+00_JPRB ,&
&  0.15055E+00_JPRB, 0.15035E+00_JPRB, 0.15015E+00_JPRB, 0.15000E+00_JPRB, 0.14977E+00_JPRB, 0.14967E+00_JPRB, 0.14947E+00_JPRB ,&
&  0.14939E+00_JPRB, 0.14931E+00_JPRB, 0.14924E+00_JPRB, 0.14917E+00_JPRB, 0.14916E+00_JPRB, 0.14913E+00_JPRB, 0.14911E+00_JPRB ,&
&  0.14908E+00_JPRB, 0.14919E+00_JPRB, 0.14906E+00_JPRB, 0.14891E+00_JPRB, 0.14873E+00_JPRB, 0.14856E+00_JPRB, 0.14840E+00_JPRB ,&
&  0.14826E+00_JPRB, 0.14811E+00_JPRB, 0.14802E+00_JPRB, 0.14803E+00_JPRB, 0.14805E+00_JPRB, 0.14810E+00_JPRB, 0.14817E+00_JPRB ,&
&  0.14824E+00_JPRB, 0.14834E+00_JPRB, 0.14844E+00_JPRB, 0.14840E+00_JPRB, 0.14842E+00_JPRB, 0.14844E+00_JPRB, 0.14846E+00_JPRB ,&
&  0.14848E+00_JPRB, 0.14853E+00_JPRB, 0.14861E+00_JPRB, 0.14872E+00_JPRB, 0.14895E+00_JPRB, 0.14906E+00_JPRB, 0.14921E+00_JPRB ,&
&  0.14942E+00_JPRB, 0.14979E+00_JPRB, 0.15013E+00_JPRB, 0.15048E+00_JPRB, 0.15091E+00_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.16149E+00_JPRB, 0.16331E+00_JPRB, 0.16476E+00_JPRB, 0.16669E+00_JPRB, 0.16280E+00_JPRB, 0.16973E+00_JPRB, 0.16707E+00_JPRB ,&
&  0.16431E+00_JPRB, 0.15955E+00_JPRB, 0.15962E+00_JPRB, 0.15879E+00_JPRB, 0.15763E+00_JPRB, 0.15701E+00_JPRB, 0.15682E+00_JPRB ,&
&  0.15683E+00_JPRB, 0.15660E+00_JPRB, 0.15700E+00_JPRB, 0.15729E+00_JPRB, 0.15753E+00_JPRB, 0.15770E+00_JPRB, 0.15799E+00_JPRB ,&
&  0.15850E+00_JPRB, 0.15939E+00_JPRB, 0.16031E+00_JPRB, 0.15862E+00_JPRB, 0.15847E+00_JPRB, 0.15877E+00_JPRB, 0.15941E+00_JPRB ,&
&  0.15992E+00_JPRB, 0.16099E+00_JPRB, 0.16180E+00_JPRB, 0.16237E+00_JPRB, 0.16394E+00_JPRB, 0.16399E+00_JPRB, 0.16383E+00_JPRB ,&
&  0.16343E+00_JPRB, 0.16265E+00_JPRB, 0.16152E+00_JPRB, 0.16046E+00_JPRB, 0.15937E+00_JPRB, 0.15795E+00_JPRB, 0.15659E+00_JPRB ,&
&  0.15547E+00_JPRB, 0.15455E+00_JPRB, 0.15338E+00_JPRB, 0.15225E+00_JPRB, 0.15121E+00_JPRB, 0.15021E+00_JPRB, 0.14935E+00_JPRB ,&
&  0.14872E+00_JPRB, 0.14820E+00_JPRB, 0.14765E+00_JPRB, 0.14723E+00_JPRB, 0.14678E+00_JPRB, 0.14636E+00_JPRB, 0.14597E+00_JPRB ,&
&  0.14610E+00_JPRB, 0.14621E+00_JPRB, 0.14631E+00_JPRB, 0.14641E+00_JPRB, 0.14677E+00_JPRB, 0.14683E+00_JPRB, 0.14691E+00_JPRB ,&
&  0.14695E+00_JPRB, 0.14684E+00_JPRB, 0.14672E+00_JPRB, 0.14652E+00_JPRB, 0.14628E+00_JPRB, 0.14599E+00_JPRB, 0.14586E+00_JPRB ,&
&  0.14585E+00_JPRB, 0.14596E+00_JPRB, 0.14628E+00_JPRB, 0.14649E+00_JPRB, 0.14671E+00_JPRB, 0.14688E+00_JPRB, 0.14731E+00_JPRB ,&
&  0.14774E+00_JPRB, 0.14818E+00_JPRB, 0.14860E+00_JPRB, 0.14851E+00_JPRB, 0.14874E+00_JPRB, 0.14896E+00_JPRB, 0.14916E+00_JPRB ,&
&  0.14928E+00_JPRB, 0.14973E+00_JPRB, 0.15043E+00_JPRB, 0.15139E+00_JPRB, 0.15313E+00_JPRB, 0.15464E+00_JPRB, 0.15603E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.16052E+00_JPRB, 0.16175E+00_JPRB, 0.16218E+00_JPRB, 0.16224E+00_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.17040E+00_JPRB, 0.17226E+00_JPRB, 0.17346E+00_JPRB, 0.17429E+00_JPRB, 0.17550E+00_JPRB, 0.17604E+00_JPRB, 0.17654E+00_JPRB ,&
&  0.17471E+00_JPRB, 0.17411E+00_JPRB, 0.17306E+00_JPRB, 0.17176E+00_JPRB, 0.17081E+00_JPRB, 0.16853E+00_JPRB, 0.16624E+00_JPRB ,&
&  0.16369E+00_JPRB, 0.16141E+00_JPRB, 0.16000E+00_JPRB, 0.15863E+00_JPRB, 0.15726E+00_JPRB, 0.15612E+00_JPRB, 0.15785E+00_JPRB ,&
&  0.15887E+00_JPRB, 0.15951E+00_JPRB, 0.16072E+00_JPRB, 0.16066E+00_JPRB, 0.16195E+00_JPRB, 0.16400E+00_JPRB, 0.16673E+00_JPRB ,&
&  0.16855E+00_JPRB, 0.17138E+00_JPRB, 0.17307E+00_JPRB, 0.17392E+00_JPRB, 0.17509E+00_JPRB, 0.17588E+00_JPRB, 0.17558E+00_JPRB ,&
&  0.17432E+00_JPRB, 0.17262E+00_JPRB, 0.17064E+00_JPRB, 0.16811E+00_JPRB, 0.16530E+00_JPRB, 0.16259E+00_JPRB, 0.15988E+00_JPRB ,&
&  0.15789E+00_JPRB, 0.15608E+00_JPRB, 0.15418E+00_JPRB, 0.15246E+00_JPRB, 0.15082E+00_JPRB, 0.14934E+00_JPRB, 0.14785E+00_JPRB ,&
&  0.14667E+00_JPRB, 0.14574E+00_JPRB, 0.14459E+00_JPRB, 0.14396E+00_JPRB, 0.14334E+00_JPRB, 0.14306E+00_JPRB, 0.14292E+00_JPRB ,&
&  0.14327E+00_JPRB, 0.14373E+00_JPRB, 0.14419E+00_JPRB, 0.14444E+00_JPRB, 0.14469E+00_JPRB, 0.14480E+00_JPRB, 0.14503E+00_JPRB ,&
&  0.14523E+00_JPRB, 0.14547E+00_JPRB, 0.14508E+00_JPRB, 0.14473E+00_JPRB, 0.14426E+00_JPRB, 0.14373E+00_JPRB, 0.14357E+00_JPRB ,&
&  0.14385E+00_JPRB, 0.14408E+00_JPRB, 0.14395E+00_JPRB, 0.14423E+00_JPRB, 0.14471E+00_JPRB, 0.14520E+00_JPRB, 0.14644E+00_JPRB ,&
&  0.14711E+00_JPRB, 0.14777E+00_JPRB, 0.14838E+00_JPRB, 0.14886E+00_JPRB, 0.14959E+00_JPRB, 0.15019E+00_JPRB, 0.15093E+00_JPRB ,&
&  0.15183E+00_JPRB, 0.15272E+00_JPRB, 0.15394E+00_JPRB, 0.15548E+00_JPRB, 0.15914E+00_JPRB, 0.16160E+00_JPRB, 0.16376E+00_JPRB ,&
&  0.16485E+00_JPRB, 0.16178E+00_JPRB, 0.16894E+00_JPRB, 0.16821E+00_JPRB, 0.16804E+00_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.17662E+00_JPRB, 0.17912E+00_JPRB, 0.17786E+00_JPRB, 0.17773E+00_JPRB, 0.17719E+00_JPRB, 0.17708E+00_JPRB, 0.17607E+00_JPRB ,&
&  0.17586E+00_JPRB, 0.17388E+00_JPRB, 0.17347E+00_JPRB, 0.17177E+00_JPRB, 0.16999E+00_JPRB, 0.16754E+00_JPRB, 0.16512E+00_JPRB ,&
&  0.16267E+00_JPRB, 0.16052E+00_JPRB, 0.15904E+00_JPRB, 0.15835E+00_JPRB, 0.15713E+00_JPRB, 0.15706E+00_JPRB, 0.15888E+00_JPRB ,&
&  0.15929E+00_JPRB, 0.16037E+00_JPRB, 0.16050E+00_JPRB, 0.16175E+00_JPRB, 0.16336E+00_JPRB, 0.16409E+00_JPRB, 0.16641E+00_JPRB ,&
&  0.16778E+00_JPRB, 0.16932E+00_JPRB, 0.17500E+00_JPRB, 0.17817E+00_JPRB, 0.17999E+00_JPRB, 0.18268E+00_JPRB, 0.18278E+00_JPRB ,&
&  0.18141E+00_JPRB, 0.17751E+00_JPRB, 0.18069E+00_JPRB, 0.17846E+00_JPRB, 0.17211E+00_JPRB, 0.16924E+00_JPRB, 0.16403E+00_JPRB ,&
&  0.16094E+00_JPRB, 0.15729E+00_JPRB, 0.15455E+00_JPRB, 0.15244E+00_JPRB, 0.14956E+00_JPRB, 0.14771E+00_JPRB, 0.14563E+00_JPRB ,&
&  0.14368E+00_JPRB, 0.14282E+00_JPRB, 0.14147E+00_JPRB, 0.14127E+00_JPRB, 0.14138E+00_JPRB, 0.14176E+00_JPRB, 0.14240E+00_JPRB ,&
&  0.14327E+00_JPRB, 0.14409E+00_JPRB, 0.14484E+00_JPRB, 0.14548E+00_JPRB, 0.14603E+00_JPRB, 0.14636E+00_JPRB, 0.14623E+00_JPRB ,&
&  0.14632E+00_JPRB, 0.14619E+00_JPRB, 0.14572E+00_JPRB, 0.14546E+00_JPRB, 0.14455E+00_JPRB, 0.14413E+00_JPRB, 0.14371E+00_JPRB ,&
&  0.14352E+00_JPRB, 0.14353E+00_JPRB, 0.14346E+00_JPRB, 0.14365E+00_JPRB, 0.14394E+00_JPRB, 0.14462E+00_JPRB, 0.14518E+00_JPRB ,&
&  0.14564E+00_JPRB, 0.14649E+00_JPRB, 0.14701E+00_JPRB, 0.14772E+00_JPRB, 0.14872E+00_JPRB, 0.14919E+00_JPRB, 0.15037E+00_JPRB ,&
&  0.15140E+00_JPRB, 0.15259E+00_JPRB, 0.15475E+00_JPRB, 0.15693E+00_JPRB, 0.16004E+00_JPRB, 0.16397E+00_JPRB, 0.16560E+00_JPRB ,&
&  0.16709E+00_JPRB, 0.16839E+00_JPRB, 0.16715E+00_JPRB, 0.17195E+00_JPRB, 0.17422E+00_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.17857E+00_JPRB, 0.17941E+00_JPRB, 0.17758E+00_JPRB, 0.17798E+00_JPRB, 0.17874E+00_JPRB, 0.17916E+00_JPRB, 0.17965E+00_JPRB ,&
&  0.17723E+00_JPRB, 0.17637E+00_JPRB, 0.17406E+00_JPRB, 0.17199E+00_JPRB, 0.16949E+00_JPRB, 0.16684E+00_JPRB, 0.16481E+00_JPRB ,&
&  0.16288E+00_JPRB, 0.16212E+00_JPRB, 0.16121E+00_JPRB, 0.16195E+00_JPRB, 0.16198E+00_JPRB, 0.16363E+00_JPRB, 0.16498E+00_JPRB ,&
&  0.16573E+00_JPRB, 0.16677E+00_JPRB, 0.16552E+00_JPRB, 0.16531E+00_JPRB, 0.16704E+00_JPRB, 0.16865E+00_JPRB, 0.17127E+00_JPRB ,&
&  0.17404E+00_JPRB, 0.17715E+00_JPRB, 0.18219E+00_JPRB, 0.18406E+00_JPRB, 0.18632E+00_JPRB, 0.18772E+00_JPRB, 0.18770E+00_JPRB ,&
&  0.18743E+00_JPRB, 0.18741E+00_JPRB, 0.18828E+00_JPRB, 0.18807E+00_JPRB, 0.18131E+00_JPRB, 0.17612E+00_JPRB, 0.17063E+00_JPRB ,&
&  0.16554E+00_JPRB, 0.16069E+00_JPRB, 0.15638E+00_JPRB, 0.15348E+00_JPRB, 0.15046E+00_JPRB, 0.14829E+00_JPRB, 0.14658E+00_JPRB ,&
&  0.14485E+00_JPRB, 0.14384E+00_JPRB, 0.14254E+00_JPRB, 0.14213E+00_JPRB, 0.14239E+00_JPRB, 0.14313E+00_JPRB, 0.14416E+00_JPRB ,&
&  0.14539E+00_JPRB, 0.14626E+00_JPRB, 0.14734E+00_JPRB, 0.14835E+00_JPRB, 0.14907E+00_JPRB, 0.14952E+00_JPRB, 0.14925E+00_JPRB ,&
&  0.14944E+00_JPRB, 0.14968E+00_JPRB, 0.14924E+00_JPRB, 0.14917E+00_JPRB, 0.14806E+00_JPRB, 0.14677E+00_JPRB, 0.14544E+00_JPRB ,&
&  0.14367E+00_JPRB, 0.14231E+00_JPRB, 0.14182E+00_JPRB, 0.14155E+00_JPRB, 0.14223E+00_JPRB, 0.14326E+00_JPRB, 0.14394E+00_JPRB ,&
&  0.14430E+00_JPRB, 0.14423E+00_JPRB, 0.14428E+00_JPRB, 0.14542E+00_JPRB, 0.14661E+00_JPRB, 0.14812E+00_JPRB, 0.14968E+00_JPRB ,&
&  0.15021E+00_JPRB, 0.15150E+00_JPRB, 0.15263E+00_JPRB, 0.15402E+00_JPRB, 0.15865E+00_JPRB, 0.16232E+00_JPRB, 0.16935E+00_JPRB ,&
&  0.17286E+00_JPRB, 0.17420E+00_JPRB, 0.17374E+00_JPRB, 0.17785E+00_JPRB, 0.17818E+00_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.18336E+00_JPRB, 0.18524E+00_JPRB, 0.18686E+00_JPRB, 0.18594E+00_JPRB, 0.18611E+00_JPRB, 0.18611E+00_JPRB, 0.18618E+00_JPRB ,&
&  0.18614E+00_JPRB, 0.18555E+00_JPRB, 0.18581E+00_JPRB, 0.18404E+00_JPRB, 0.18301E+00_JPRB, 0.17946E+00_JPRB, 0.17822E+00_JPRB ,&
&  0.17634E+00_JPRB, 0.17589E+00_JPRB, 0.17615E+00_JPRB, 0.17734E+00_JPRB, 0.17780E+00_JPRB, 0.17908E+00_JPRB, 0.17833E+00_JPRB ,&
&  0.17854E+00_JPRB, 0.17855E+00_JPRB, 0.17732E+00_JPRB, 0.17698E+00_JPRB, 0.17741E+00_JPRB, 0.17786E+00_JPRB, 0.17819E+00_JPRB ,&
&  0.18013E+00_JPRB, 0.18240E+00_JPRB, 0.18458E+00_JPRB, 0.18592E+00_JPRB, 0.19093E+00_JPRB, 0.19239E+00_JPRB, 0.19437E+00_JPRB ,&
&  0.19547E+00_JPRB, 0.19687E+00_JPRB, 0.19791E+00_JPRB, 0.19730E+00_JPRB, 0.19610E+00_JPRB, 0.19251E+00_JPRB, 0.19119E+00_JPRB ,&
&  0.18118E+00_JPRB, 0.17437E+00_JPRB, 0.16765E+00_JPRB, 0.16165E+00_JPRB, 0.15728E+00_JPRB, 0.15426E+00_JPRB, 0.15180E+00_JPRB ,&
&  0.14976E+00_JPRB, 0.14778E+00_JPRB, 0.14564E+00_JPRB, 0.14569E+00_JPRB, 0.14644E+00_JPRB, 0.14758E+00_JPRB, 0.14873E+00_JPRB ,&
&  0.15006E+00_JPRB, 0.15082E+00_JPRB, 0.15116E+00_JPRB, 0.15186E+00_JPRB, 0.15200E+00_JPRB, 0.15232E+00_JPRB, 0.15250E+00_JPRB ,&
&  0.15372E+00_JPRB, 0.15409E+00_JPRB, 0.15409E+00_JPRB, 0.15545E+00_JPRB, 0.15377E+00_JPRB, 0.15115E+00_JPRB, 0.14829E+00_JPRB ,&
&  0.14544E+00_JPRB, 0.14366E+00_JPRB, 0.14217E+00_JPRB, 0.14159E+00_JPRB, 0.14197E+00_JPRB, 0.14268E+00_JPRB, 0.14362E+00_JPRB ,&
&  0.14383E+00_JPRB, 0.14443E+00_JPRB, 0.14491E+00_JPRB, 0.14580E+00_JPRB, 0.14676E+00_JPRB, 0.14783E+00_JPRB, 0.14900E+00_JPRB ,&
&  0.15073E+00_JPRB, 0.15276E+00_JPRB, 0.15477E+00_JPRB, 0.15683E+00_JPRB, 0.16010E+00_JPRB, 0.16402E+00_JPRB, 0.17024E+00_JPRB ,&
&  0.17579E+00_JPRB, 0.18309E+00_JPRB, 0.18172E+00_JPRB, 0.18428E+00_JPRB, 0.18457E+00_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.19056E+00_JPRB, 0.19427E+00_JPRB, 0.19577E+00_JPRB, 0.19578E+00_JPRB, 0.19473E+00_JPRB, 0.19385E+00_JPRB, 0.19367E+00_JPRB ,&
&  0.19356E+00_JPRB, 0.19248E+00_JPRB, 0.19167E+00_JPRB, 0.19106E+00_JPRB, 0.19037E+00_JPRB, 0.18956E+00_JPRB, 0.18982E+00_JPRB ,&
&  0.19101E+00_JPRB, 0.19249E+00_JPRB, 0.19356E+00_JPRB, 0.19400E+00_JPRB, 0.19358E+00_JPRB, 0.19391E+00_JPRB, 0.19474E+00_JPRB ,&
&  0.19472E+00_JPRB, 0.19493E+00_JPRB, 0.19815E+00_JPRB, 0.19854E+00_JPRB, 0.19947E+00_JPRB, 0.19955E+00_JPRB, 0.19963E+00_JPRB ,&
&  0.19494E+00_JPRB, 0.18976E+00_JPRB, 0.18696E+00_JPRB, 0.18910E+00_JPRB, 0.19724E+00_JPRB, 0.19880E+00_JPRB, 0.20218E+00_JPRB ,&
&  0.20429E+00_JPRB, 0.20537E+00_JPRB, 0.20676E+00_JPRB, 0.20797E+00_JPRB, 0.20941E+00_JPRB, 0.20860E+00_JPRB, 0.20648E+00_JPRB ,&
&  0.20375E+00_JPRB, 0.20278E+00_JPRB, 0.19957E+00_JPRB, 0.18562E+00_JPRB, 0.17709E+00_JPRB, 0.17226E+00_JPRB, 0.16720E+00_JPRB ,&
&  0.16256E+00_JPRB, 0.15999E+00_JPRB, 0.15933E+00_JPRB, 0.15802E+00_JPRB, 0.15890E+00_JPRB, 0.16089E+00_JPRB, 0.16173E+00_JPRB ,&
&  0.16139E+00_JPRB, 0.16178E+00_JPRB, 0.16145E+00_JPRB, 0.16044E+00_JPRB, 0.15865E+00_JPRB, 0.15751E+00_JPRB, 0.15683E+00_JPRB ,&
&  0.15728E+00_JPRB, 0.15700E+00_JPRB, 0.16088E+00_JPRB, 0.16213E+00_JPRB, 0.15728E+00_JPRB, 0.15590E+00_JPRB, 0.15410E+00_JPRB ,&
&  0.14914E+00_JPRB, 0.14637E+00_JPRB, 0.14486E+00_JPRB, 0.14428E+00_JPRB, 0.14525E+00_JPRB, 0.14662E+00_JPRB, 0.14730E+00_JPRB ,&
&  0.14753E+00_JPRB, 0.14854E+00_JPRB, 0.14905E+00_JPRB, 0.14963E+00_JPRB, 0.15002E+00_JPRB, 0.15086E+00_JPRB, 0.15208E+00_JPRB ,&
&  0.15362E+00_JPRB, 0.15637E+00_JPRB, 0.15911E+00_JPRB, 0.16217E+00_JPRB, 0.16400E+00_JPRB, 0.16543E+00_JPRB, 0.16870E+00_JPRB ,&
&  0.17069E+00_JPRB, 0.17767E+00_JPRB, 0.18781E+00_JPRB, 0.19093E+00_JPRB, 0.18608E+00_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.20318E+00_JPRB, 0.20498E+00_JPRB, 0.20559E+00_JPRB, 0.20559E+00_JPRB, 0.20544E+00_JPRB, 0.20594E+00_JPRB, 0.20648E+00_JPRB ,&
&  0.20649E+00_JPRB, 0.20582E+00_JPRB, 0.20469E+00_JPRB, 0.20278E+00_JPRB, 0.20045E+00_JPRB, 0.19909E+00_JPRB, 0.20012E+00_JPRB ,&
&  0.20109E+00_JPRB, 0.20101E+00_JPRB, 0.20106E+00_JPRB, 0.20090E+00_JPRB, 0.20048E+00_JPRB, 0.20074E+00_JPRB, 0.20193E+00_JPRB ,&
&  0.20325E+00_JPRB, 0.20408E+00_JPRB, 0.20545E+00_JPRB, 0.20659E+00_JPRB, 0.20864E+00_JPRB, 0.21051E+00_JPRB, 0.21120E+00_JPRB ,&
&  0.20865E+00_JPRB, 0.20096E+00_JPRB, 0.19572E+00_JPRB, 0.19825E+00_JPRB, 0.20641E+00_JPRB, 0.21154E+00_JPRB, 0.21473E+00_JPRB ,&
&  0.21691E+00_JPRB, 0.21804E+00_JPRB, 0.21833E+00_JPRB, 0.21961E+00_JPRB, 0.22156E+00_JPRB, 0.22345E+00_JPRB, 0.22307E+00_JPRB ,&
&  0.22110E+00_JPRB, 0.21943E+00_JPRB, 0.21699E+00_JPRB, 0.21274E+00_JPRB, 0.20693E+00_JPRB, 0.20121E+00_JPRB, 0.19629E+00_JPRB ,&
&  0.19187E+00_JPRB, 0.18757E+00_JPRB, 0.18397E+00_JPRB, 0.18165E+00_JPRB, 0.18096E+00_JPRB, 0.18221E+00_JPRB, 0.18190E+00_JPRB ,&
&  0.18183E+00_JPRB, 0.18000E+00_JPRB, 0.17780E+00_JPRB, 0.17517E+00_JPRB, 0.17286E+00_JPRB, 0.16877E+00_JPRB, 0.16374E+00_JPRB ,&
&  0.16115E+00_JPRB, 0.16348E+00_JPRB, 0.17008E+00_JPRB, 0.17540E+00_JPRB, 0.17528E+00_JPRB, 0.16836E+00_JPRB, 0.16459E+00_JPRB ,&
&  0.16063E+00_JPRB, 0.15664E+00_JPRB, 0.15410E+00_JPRB, 0.15351E+00_JPRB, 0.15368E+00_JPRB, 0.15388E+00_JPRB, 0.15482E+00_JPRB ,&
&  0.15635E+00_JPRB, 0.15772E+00_JPRB, 0.15849E+00_JPRB, 0.15936E+00_JPRB, 0.15972E+00_JPRB, 0.15981E+00_JPRB, 0.15995E+00_JPRB ,&
&  0.16151E+00_JPRB, 0.16478E+00_JPRB, 0.16920E+00_JPRB, 0.17414E+00_JPRB, 0.17793E+00_JPRB, 0.18139E+00_JPRB, 0.18451E+00_JPRB ,&
&  0.18774E+00_JPRB, 0.18971E+00_JPRB, 0.19277E+00_JPRB, 0.19683E+00_JPRB, 0.20026E+00_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.21433E+00_JPRB, 0.21652E+00_JPRB, 0.21729E+00_JPRB, 0.21694E+00_JPRB, 0.21648E+00_JPRB, 0.21745E+00_JPRB, 0.21881E+00_JPRB ,&
&  0.21954E+00_JPRB, 0.21968E+00_JPRB, 0.21884E+00_JPRB, 0.21712E+00_JPRB, 0.21589E+00_JPRB, 0.21626E+00_JPRB, 0.21476E+00_JPRB ,&
&  0.21325E+00_JPRB, 0.21326E+00_JPRB, 0.21172E+00_JPRB, 0.21004E+00_JPRB, 0.20763E+00_JPRB, 0.20627E+00_JPRB, 0.20687E+00_JPRB ,&
&  0.20899E+00_JPRB, 0.20965E+00_JPRB, 0.21036E+00_JPRB, 0.21133E+00_JPRB, 0.21314E+00_JPRB, 0.21529E+00_JPRB, 0.21675E+00_JPRB ,&
&  0.21781E+00_JPRB, 0.21706E+00_JPRB, 0.21551E+00_JPRB, 0.21826E+00_JPRB, 0.22146E+00_JPRB, 0.22559E+00_JPRB, 0.22846E+00_JPRB ,&
&  0.23111E+00_JPRB, 0.23308E+00_JPRB, 0.23400E+00_JPRB, 0.23394E+00_JPRB, 0.23427E+00_JPRB, 0.23600E+00_JPRB, 0.23724E+00_JPRB ,&
&  0.23759E+00_JPRB, 0.23721E+00_JPRB, 0.23584E+00_JPRB, 0.23456E+00_JPRB, 0.23130E+00_JPRB, 0.22629E+00_JPRB, 0.21868E+00_JPRB ,&
&  0.21069E+00_JPRB, 0.20421E+00_JPRB, 0.19931E+00_JPRB, 0.19537E+00_JPRB, 0.19262E+00_JPRB, 0.19228E+00_JPRB, 0.19216E+00_JPRB ,&
&  0.19164E+00_JPRB, 0.19114E+00_JPRB, 0.19244E+00_JPRB, 0.19310E+00_JPRB, 0.18906E+00_JPRB, 0.18388E+00_JPRB, 0.17931E+00_JPRB ,&
&  0.17822E+00_JPRB, 0.17921E+00_JPRB, 0.18080E+00_JPRB, 0.18294E+00_JPRB, 0.18636E+00_JPRB, 0.18717E+00_JPRB, 0.18615E+00_JPRB ,&
&  0.18078E+00_JPRB, 0.17548E+00_JPRB, 0.17239E+00_JPRB, 0.16996E+00_JPRB, 0.16848E+00_JPRB, 0.16841E+00_JPRB, 0.16910E+00_JPRB ,&
&  0.17122E+00_JPRB, 0.17308E+00_JPRB, 0.17543E+00_JPRB, 0.17663E+00_JPRB, 0.17638E+00_JPRB, 0.17604E+00_JPRB, 0.17531E+00_JPRB ,&
&  0.17661E+00_JPRB, 0.18091E+00_JPRB, 0.18746E+00_JPRB, 0.19077E+00_JPRB, 0.19474E+00_JPRB, 0.19742E+00_JPRB, 0.19872E+00_JPRB ,&
&  0.19961E+00_JPRB, 0.20016E+00_JPRB, 0.20072E+00_JPRB, 0.20499E+00_JPRB, 0.21073E+00_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.22772E+00_JPRB, 0.23006E+00_JPRB, 0.23136E+00_JPRB, 0.23124E+00_JPRB, 0.23072E+00_JPRB, 0.23147E+00_JPRB, 0.23207E+00_JPRB ,&
&  0.23279E+00_JPRB, 0.23304E+00_JPRB, 0.23189E+00_JPRB, 0.23091E+00_JPRB, 0.23260E+00_JPRB, 0.23224E+00_JPRB, 0.23180E+00_JPRB ,&
&  0.23018E+00_JPRB, 0.22814E+00_JPRB, 0.22517E+00_JPRB, 0.22177E+00_JPRB, 0.21763E+00_JPRB, 0.21509E+00_JPRB, 0.21534E+00_JPRB ,&
&  0.21685E+00_JPRB, 0.21744E+00_JPRB, 0.21827E+00_JPRB, 0.21939E+00_JPRB, 0.22170E+00_JPRB, 0.22463E+00_JPRB, 0.22685E+00_JPRB ,&
&  0.22864E+00_JPRB, 0.23180E+00_JPRB, 0.23379E+00_JPRB, 0.23275E+00_JPRB, 0.23696E+00_JPRB, 0.24106E+00_JPRB, 0.24504E+00_JPRB ,&
&  0.24889E+00_JPRB, 0.25124E+00_JPRB, 0.25140E+00_JPRB, 0.25031E+00_JPRB, 0.24940E+00_JPRB, 0.25024E+00_JPRB, 0.25057E+00_JPRB ,&
&  0.25100E+00_JPRB, 0.25334E+00_JPRB, 0.25616E+00_JPRB, 0.25719E+00_JPRB, 0.25570E+00_JPRB, 0.25153E+00_JPRB, 0.24488E+00_JPRB ,&
&  0.23623E+00_JPRB, 0.22704E+00_JPRB, 0.21952E+00_JPRB, 0.21350E+00_JPRB, 0.20886E+00_JPRB, 0.20510E+00_JPRB, 0.20204E+00_JPRB ,&
&  0.19864E+00_JPRB, 0.19623E+00_JPRB, 0.19607E+00_JPRB, 0.19660E+00_JPRB, 0.19665E+00_JPRB, 0.19503E+00_JPRB, 0.19389E+00_JPRB ,&
&  0.19286E+00_JPRB, 0.19330E+00_JPRB, 0.19419E+00_JPRB, 0.19451E+00_JPRB, 0.19466E+00_JPRB, 0.19440E+00_JPRB, 0.19414E+00_JPRB ,&
&  0.19498E+00_JPRB, 0.19510E+00_JPRB, 0.19412E+00_JPRB, 0.19233E+00_JPRB, 0.19081E+00_JPRB, 0.19054E+00_JPRB, 0.19083E+00_JPRB ,&
&  0.19148E+00_JPRB, 0.19271E+00_JPRB, 0.19447E+00_JPRB, 0.19630E+00_JPRB, 0.19772E+00_JPRB, 0.19836E+00_JPRB, 0.19861E+00_JPRB ,&
&  0.19882E+00_JPRB, 0.19873E+00_JPRB, 0.19973E+00_JPRB, 0.20277E+00_JPRB, 0.20594E+00_JPRB, 0.20845E+00_JPRB, 0.20982E+00_JPRB ,&
&  0.20945E+00_JPRB, 0.21022E+00_JPRB, 0.21349E+00_JPRB, 0.21963E+00_JPRB, 0.22418E+00_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.24551E+00_JPRB, 0.24805E+00_JPRB, 0.25076E+00_JPRB, 0.25105E+00_JPRB, 0.24945E+00_JPRB, 0.24869E+00_JPRB, 0.24895E+00_JPRB ,&
&  0.24950E+00_JPRB, 0.24975E+00_JPRB, 0.24860E+00_JPRB, 0.24852E+00_JPRB, 0.24860E+00_JPRB, 0.24770E+00_JPRB, 0.24701E+00_JPRB ,&
&  0.24569E+00_JPRB, 0.24269E+00_JPRB, 0.23841E+00_JPRB, 0.23316E+00_JPRB, 0.22923E+00_JPRB, 0.22713E+00_JPRB, 0.22678E+00_JPRB ,&
&  0.22746E+00_JPRB, 0.22800E+00_JPRB, 0.23053E+00_JPRB, 0.23274E+00_JPRB, 0.23551E+00_JPRB, 0.23919E+00_JPRB, 0.24305E+00_JPRB ,&
&  0.24592E+00_JPRB, 0.24890E+00_JPRB, 0.25187E+00_JPRB, 0.25462E+00_JPRB, 0.25824E+00_JPRB, 0.26394E+00_JPRB, 0.26943E+00_JPRB ,&
&  0.27559E+00_JPRB, 0.27889E+00_JPRB, 0.27622E+00_JPRB, 0.27285E+00_JPRB, 0.27254E+00_JPRB, 0.27416E+00_JPRB, 0.27601E+00_JPRB ,&
&  0.27769E+00_JPRB, 0.27941E+00_JPRB, 0.28075E+00_JPRB, 0.28032E+00_JPRB, 0.27754E+00_JPRB, 0.27269E+00_JPRB, 0.26624E+00_JPRB ,&
&  0.25916E+00_JPRB, 0.25202E+00_JPRB, 0.24509E+00_JPRB, 0.23804E+00_JPRB, 0.23197E+00_JPRB, 0.22615E+00_JPRB, 0.22083E+00_JPRB ,&
&  0.21587E+00_JPRB, 0.21115E+00_JPRB, 0.20755E+00_JPRB, 0.20526E+00_JPRB, 0.20454E+00_JPRB, 0.20334E+00_JPRB, 0.20326E+00_JPRB ,&
&  0.20272E+00_JPRB, 0.20254E+00_JPRB, 0.20267E+00_JPRB, 0.20350E+00_JPRB, 0.20497E+00_JPRB, 0.20538E+00_JPRB, 0.20444E+00_JPRB ,&
&  0.20390E+00_JPRB, 0.20509E+00_JPRB, 0.20486E+00_JPRB, 0.20254E+00_JPRB, 0.19968E+00_JPRB, 0.19901E+00_JPRB, 0.20042E+00_JPRB ,&
&  0.20311E+00_JPRB, 0.20580E+00_JPRB, 0.20812E+00_JPRB, 0.21019E+00_JPRB, 0.21192E+00_JPRB, 0.21405E+00_JPRB, 0.21601E+00_JPRB ,&
&  0.21764E+00_JPRB, 0.21843E+00_JPRB, 0.21990E+00_JPRB, 0.22150E+00_JPRB, 0.22318E+00_JPRB, 0.22438E+00_JPRB, 0.22430E+00_JPRB ,&
&  0.22390E+00_JPRB, 0.22552E+00_JPRB, 0.23037E+00_JPRB, 0.23775E+00_JPRB, 0.24256E+00_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.26692E+00_JPRB, 0.26934E+00_JPRB, 0.27246E+00_JPRB, 0.27500E+00_JPRB, 0.27602E+00_JPRB, 0.27565E+00_JPRB, 0.27498E+00_JPRB ,&
&  0.27497E+00_JPRB, 0.27505E+00_JPRB, 0.27372E+00_JPRB, 0.27123E+00_JPRB, 0.26858E+00_JPRB, 0.26576E+00_JPRB, 0.26361E+00_JPRB ,&
&  0.26157E+00_JPRB, 0.25944E+00_JPRB, 0.25569E+00_JPRB, 0.25015E+00_JPRB, 0.24440E+00_JPRB, 0.24104E+00_JPRB, 0.23943E+00_JPRB ,&
&  0.23932E+00_JPRB, 0.23984E+00_JPRB, 0.24237E+00_JPRB, 0.24416E+00_JPRB, 0.24663E+00_JPRB, 0.25075E+00_JPRB, 0.25643E+00_JPRB ,&
&  0.26178E+00_JPRB, 0.26684E+00_JPRB, 0.27207E+00_JPRB, 0.27653E+00_JPRB, 0.28108E+00_JPRB, 0.28756E+00_JPRB, 0.29564E+00_JPRB ,&
&  0.30392E+00_JPRB, 0.30934E+00_JPRB, 0.30961E+00_JPRB, 0.31014E+00_JPRB, 0.31301E+00_JPRB, 0.31672E+00_JPRB, 0.31935E+00_JPRB ,&
&  0.32130E+00_JPRB, 0.31990E+00_JPRB, 0.31724E+00_JPRB, 0.31312E+00_JPRB, 0.30850E+00_JPRB, 0.30327E+00_JPRB, 0.29647E+00_JPRB ,&
&  0.28836E+00_JPRB, 0.27952E+00_JPRB, 0.27090E+00_JPRB, 0.26257E+00_JPRB, 0.25447E+00_JPRB, 0.24700E+00_JPRB, 0.24029E+00_JPRB ,&
&  0.23398E+00_JPRB, 0.22909E+00_JPRB, 0.22442E+00_JPRB, 0.22156E+00_JPRB, 0.21978E+00_JPRB, 0.21781E+00_JPRB, 0.21740E+00_JPRB ,&
&  0.21650E+00_JPRB, 0.21597E+00_JPRB, 0.21581E+00_JPRB, 0.21737E+00_JPRB, 0.22041E+00_JPRB, 0.22224E+00_JPRB, 0.22102E+00_JPRB ,&
&  0.21774E+00_JPRB, 0.21568E+00_JPRB, 0.21446E+00_JPRB, 0.21354E+00_JPRB, 0.21128E+00_JPRB, 0.21104E+00_JPRB, 0.21302E+00_JPRB ,&
&  0.21720E+00_JPRB, 0.22181E+00_JPRB, 0.22580E+00_JPRB, 0.22821E+00_JPRB, 0.23067E+00_JPRB, 0.23265E+00_JPRB, 0.23439E+00_JPRB ,&
&  0.23627E+00_JPRB, 0.23761E+00_JPRB, 0.23859E+00_JPRB, 0.23997E+00_JPRB, 0.24234E+00_JPRB, 0.24474E+00_JPRB, 0.24713E+00_JPRB ,&
&  0.24981E+00_JPRB, 0.25382E+00_JPRB, 0.25781E+00_JPRB, 0.26079E+00_JPRB, 0.26451E+00_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.29889E+00_JPRB, 0.30107E+00_JPRB, 0.30348E+00_JPRB, 0.30392E+00_JPRB, 0.30336E+00_JPRB, 0.30334E+00_JPRB, 0.30231E+00_JPRB ,&
&  0.30010E+00_JPRB, 0.29886E+00_JPRB, 0.29914E+00_JPRB, 0.29776E+00_JPRB, 0.29481E+00_JPRB, 0.29113E+00_JPRB, 0.28709E+00_JPRB ,&
&  0.28391E+00_JPRB, 0.28076E+00_JPRB, 0.27557E+00_JPRB, 0.26880E+00_JPRB, 0.26298E+00_JPRB, 0.25922E+00_JPRB, 0.25722E+00_JPRB ,&
&  0.25635E+00_JPRB, 0.25627E+00_JPRB, 0.25583E+00_JPRB, 0.25640E+00_JPRB, 0.25847E+00_JPRB, 0.26327E+00_JPRB, 0.27038E+00_JPRB ,&
&  0.27752E+00_JPRB, 0.28426E+00_JPRB, 0.29131E+00_JPRB, 0.29900E+00_JPRB, 0.30859E+00_JPRB, 0.32187E+00_JPRB, 0.33582E+00_JPRB ,&
&  0.34697E+00_JPRB, 0.35058E+00_JPRB, 0.35160E+00_JPRB, 0.35646E+00_JPRB, 0.36582E+00_JPRB, 0.37312E+00_JPRB, 0.37831E+00_JPRB ,&
&  0.38079E+00_JPRB, 0.37945E+00_JPRB, 0.37668E+00_JPRB, 0.37084E+00_JPRB, 0.36242E+00_JPRB, 0.35185E+00_JPRB, 0.34063E+00_JPRB ,&
&  0.32952E+00_JPRB, 0.31796E+00_JPRB, 0.30541E+00_JPRB, 0.29332E+00_JPRB, 0.28278E+00_JPRB, 0.27335E+00_JPRB, 0.26410E+00_JPRB ,&
&  0.25571E+00_JPRB, 0.24903E+00_JPRB, 0.24374E+00_JPRB, 0.24009E+00_JPRB, 0.23715E+00_JPRB, 0.23482E+00_JPRB, 0.23411E+00_JPRB ,&
&  0.23432E+00_JPRB, 0.23559E+00_JPRB, 0.23622E+00_JPRB, 0.23765E+00_JPRB, 0.24010E+00_JPRB, 0.24110E+00_JPRB, 0.23989E+00_JPRB ,&
&  0.23594E+00_JPRB, 0.23188E+00_JPRB, 0.22797E+00_JPRB, 0.22721E+00_JPRB, 0.22710E+00_JPRB, 0.22829E+00_JPRB, 0.23015E+00_JPRB ,&
&  0.23388E+00_JPRB, 0.23929E+00_JPRB, 0.24411E+00_JPRB, 0.24754E+00_JPRB, 0.25133E+00_JPRB, 0.25465E+00_JPRB, 0.25796E+00_JPRB ,&
&  0.26169E+00_JPRB, 0.26571E+00_JPRB, 0.27108E+00_JPRB, 0.27641E+00_JPRB, 0.28121E+00_JPRB, 0.28511E+00_JPRB, 0.28811E+00_JPRB ,&
&  0.29025E+00_JPRB, 0.29094E+00_JPRB, 0.29137E+00_JPRB, 0.29318E+00_JPRB, 0.29622E+00_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.33623E+00_JPRB, 0.33704E+00_JPRB, 0.33827E+00_JPRB, 0.33889E+00_JPRB, 0.33638E+00_JPRB, 0.33324E+00_JPRB, 0.32964E+00_JPRB ,&
&  0.32795E+00_JPRB, 0.32673E+00_JPRB, 0.32621E+00_JPRB, 0.32303E+00_JPRB, 0.32045E+00_JPRB, 0.31873E+00_JPRB, 0.31489E+00_JPRB ,&
&  0.31137E+00_JPRB, 0.30388E+00_JPRB, 0.29595E+00_JPRB, 0.29013E+00_JPRB, 0.28647E+00_JPRB, 0.28395E+00_JPRB, 0.27879E+00_JPRB ,&
&  0.27507E+00_JPRB, 0.27208E+00_JPRB, 0.27056E+00_JPRB, 0.27058E+00_JPRB, 0.27257E+00_JPRB, 0.27832E+00_JPRB, 0.28801E+00_JPRB ,&
&  0.29513E+00_JPRB, 0.30442E+00_JPRB, 0.30535E+00_JPRB, 0.32028E+00_JPRB, 0.33622E+00_JPRB, 0.36033E+00_JPRB, 0.38212E+00_JPRB ,&
&  0.39462E+00_JPRB, 0.40256E+00_JPRB, 0.41460E+00_JPRB, 0.42766E+00_JPRB, 0.43402E+00_JPRB, 0.43672E+00_JPRB, 0.43549E+00_JPRB ,&
&  0.43182E+00_JPRB, 0.42640E+00_JPRB, 0.41994E+00_JPRB, 0.41153E+00_JPRB, 0.39990E+00_JPRB, 0.38653E+00_JPRB, 0.37314E+00_JPRB ,&
&  0.36084E+00_JPRB, 0.34863E+00_JPRB, 0.33659E+00_JPRB, 0.32380E+00_JPRB, 0.31103E+00_JPRB, 0.29917E+00_JPRB, 0.28734E+00_JPRB ,&
&  0.27748E+00_JPRB, 0.26985E+00_JPRB, 0.26419E+00_JPRB, 0.26009E+00_JPRB, 0.25700E+00_JPRB, 0.25401E+00_JPRB, 0.25251E+00_JPRB ,&
&  0.25245E+00_JPRB, 0.25394E+00_JPRB, 0.25622E+00_JPRB, 0.25812E+00_JPRB, 0.26046E+00_JPRB, 0.26088E+00_JPRB, 0.25816E+00_JPRB ,&
&  0.25532E+00_JPRB, 0.25403E+00_JPRB, 0.24970E+00_JPRB, 0.24712E+00_JPRB, 0.24657E+00_JPRB, 0.24707E+00_JPRB, 0.24902E+00_JPRB ,&
&  0.25287E+00_JPRB, 0.25818E+00_JPRB, 0.26304E+00_JPRB, 0.26720E+00_JPRB, 0.27077E+00_JPRB, 0.27447E+00_JPRB, 0.27883E+00_JPRB ,&
&  0.28344E+00_JPRB, 0.28790E+00_JPRB, 0.29486E+00_JPRB, 0.30255E+00_JPRB, 0.30906E+00_JPRB, 0.31617E+00_JPRB, 0.32088E+00_JPRB ,&
&  0.32292E+00_JPRB, 0.32390E+00_JPRB, 0.32672E+00_JPRB, 0.33103E+00_JPRB, 0.33461E+00_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.38568E+00_JPRB, 0.38403E+00_JPRB, 0.38137E+00_JPRB, 0.37891E+00_JPRB, 0.37715E+00_JPRB, 0.37301E+00_JPRB, 0.36762E+00_JPRB ,&
&  0.36379E+00_JPRB, 0.36024E+00_JPRB, 0.35753E+00_JPRB, 0.35361E+00_JPRB, 0.35122E+00_JPRB, 0.34957E+00_JPRB, 0.34520E+00_JPRB ,&
&  0.34022E+00_JPRB, 0.33344E+00_JPRB, 0.32684E+00_JPRB, 0.32093E+00_JPRB, 0.31533E+00_JPRB, 0.31011E+00_JPRB, 0.30423E+00_JPRB ,&
&  0.29863E+00_JPRB, 0.29438E+00_JPRB, 0.29197E+00_JPRB, 0.28985E+00_JPRB, 0.29102E+00_JPRB, 0.29681E+00_JPRB, 0.30644E+00_JPRB ,&
&  0.31321E+00_JPRB, 0.31649E+00_JPRB, 0.32869E+00_JPRB, 0.34142E+00_JPRB, 0.36604E+00_JPRB, 0.40181E+00_JPRB, 0.42776E+00_JPRB ,&
&  0.44135E+00_JPRB, 0.45603E+00_JPRB, 0.47372E+00_JPRB, 0.48604E+00_JPRB, 0.48850E+00_JPRB, 0.48222E+00_JPRB, 0.47646E+00_JPRB ,&
&  0.46927E+00_JPRB, 0.45698E+00_JPRB, 0.44419E+00_JPRB, 0.42873E+00_JPRB, 0.41221E+00_JPRB, 0.39819E+00_JPRB, 0.38672E+00_JPRB ,&
&  0.37624E+00_JPRB, 0.36631E+00_JPRB, 0.35709E+00_JPRB, 0.34769E+00_JPRB, 0.33746E+00_JPRB, 0.32655E+00_JPRB, 0.31503E+00_JPRB ,&
&  0.30393E+00_JPRB, 0.29448E+00_JPRB, 0.28708E+00_JPRB, 0.28304E+00_JPRB, 0.28068E+00_JPRB, 0.27871E+00_JPRB, 0.27734E+00_JPRB ,&
&  0.27746E+00_JPRB, 0.27872E+00_JPRB, 0.28069E+00_JPRB, 0.28265E+00_JPRB, 0.28433E+00_JPRB, 0.28462E+00_JPRB, 0.28270E+00_JPRB ,&
&  0.28068E+00_JPRB, 0.28010E+00_JPRB, 0.28005E+00_JPRB, 0.27759E+00_JPRB, 0.27495E+00_JPRB, 0.27546E+00_JPRB, 0.27766E+00_JPRB ,&
&  0.28170E+00_JPRB, 0.28541E+00_JPRB, 0.28854E+00_JPRB, 0.29110E+00_JPRB, 0.29300E+00_JPRB, 0.29635E+00_JPRB, 0.30049E+00_JPRB ,&
&  0.30563E+00_JPRB, 0.31198E+00_JPRB, 0.32096E+00_JPRB, 0.33187E+00_JPRB, 0.34187E+00_JPRB, 0.35077E+00_JPRB, 0.35724E+00_JPRB ,&
&  0.36185E+00_JPRB, 0.36622E+00_JPRB, 0.37139E+00_JPRB, 0.37748E+00_JPRB, 0.38301E+00_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.44351E+00_JPRB, 0.43803E+00_JPRB, 0.43034E+00_JPRB, 0.42336E+00_JPRB, 0.41958E+00_JPRB, 0.41584E+00_JPRB, 0.40950E+00_JPRB ,&
&  0.40351E+00_JPRB, 0.39806E+00_JPRB, 0.39270E+00_JPRB, 0.38796E+00_JPRB, 0.38400E+00_JPRB, 0.38098E+00_JPRB, 0.37668E+00_JPRB ,&
&  0.37065E+00_JPRB, 0.36341E+00_JPRB, 0.35550E+00_JPRB, 0.34908E+00_JPRB, 0.34131E+00_JPRB, 0.33329E+00_JPRB, 0.32730E+00_JPRB ,&
&  0.32225E+00_JPRB, 0.31872E+00_JPRB, 0.31730E+00_JPRB, 0.31534E+00_JPRB, 0.31594E+00_JPRB, 0.31893E+00_JPRB, 0.32775E+00_JPRB ,&
&  0.34034E+00_JPRB, 0.32227E+00_JPRB, 0.33928E+00_JPRB, 0.35466E+00_JPRB, 0.38675E+00_JPRB, 0.43541E+00_JPRB, 0.46891E+00_JPRB ,&
&  0.48915E+00_JPRB, 0.50516E+00_JPRB, 0.51849E+00_JPRB, 0.52204E+00_JPRB, 0.51381E+00_JPRB, 0.50735E+00_JPRB, 0.50231E+00_JPRB ,&
&  0.49235E+00_JPRB, 0.47833E+00_JPRB, 0.46008E+00_JPRB, 0.44088E+00_JPRB, 0.42337E+00_JPRB, 0.41029E+00_JPRB, 0.39977E+00_JPRB ,&
&  0.39134E+00_JPRB, 0.38379E+00_JPRB, 0.37735E+00_JPRB, 0.36899E+00_JPRB, 0.35939E+00_JPRB, 0.34962E+00_JPRB, 0.34065E+00_JPRB ,&
&  0.33183E+00_JPRB, 0.32324E+00_JPRB, 0.31574E+00_JPRB, 0.30953E+00_JPRB, 0.30607E+00_JPRB, 0.30444E+00_JPRB, 0.30394E+00_JPRB ,&
&  0.30449E+00_JPRB, 0.30649E+00_JPRB, 0.30860E+00_JPRB, 0.31019E+00_JPRB, 0.31112E+00_JPRB, 0.31052E+00_JPRB, 0.30812E+00_JPRB ,&
&  0.30768E+00_JPRB, 0.30756E+00_JPRB, 0.30767E+00_JPRB, 0.30864E+00_JPRB, 0.30881E+00_JPRB, 0.30936E+00_JPRB, 0.31168E+00_JPRB ,&
&  0.31395E+00_JPRB, 0.31532E+00_JPRB, 0.31677E+00_JPRB, 0.31857E+00_JPRB, 0.32087E+00_JPRB, 0.32453E+00_JPRB, 0.32972E+00_JPRB ,&
&  0.33559E+00_JPRB, 0.34219E+00_JPRB, 0.35190E+00_JPRB, 0.36426E+00_JPRB, 0.37702E+00_JPRB, 0.39029E+00_JPRB, 0.40365E+00_JPRB ,&
&  0.41226E+00_JPRB, 0.42123E+00_JPRB, 0.43263E+00_JPRB, 0.43914E+00_JPRB, 0.44405E+00_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.51568E+00_JPRB, 0.50427E+00_JPRB, 0.48986E+00_JPRB, 0.47936E+00_JPRB, 0.47325E+00_JPRB, 0.46652E+00_JPRB, 0.45978E+00_JPRB ,&
&  0.45084E+00_JPRB, 0.44125E+00_JPRB, 0.43202E+00_JPRB, 0.42391E+00_JPRB, 0.41758E+00_JPRB, 0.41317E+00_JPRB, 0.40826E+00_JPRB ,&
&  0.40178E+00_JPRB, 0.39465E+00_JPRB, 0.38748E+00_JPRB, 0.37951E+00_JPRB, 0.36919E+00_JPRB, 0.36092E+00_JPRB, 0.35440E+00_JPRB ,&
&  0.34929E+00_JPRB, 0.34527E+00_JPRB, 0.34375E+00_JPRB, 0.34185E+00_JPRB, 0.33975E+00_JPRB, 0.33973E+00_JPRB, 0.35093E+00_JPRB ,&
&  0.34916E+00_JPRB, 0.32471E+00_JPRB, 0.34288E+00_JPRB, 0.36740E+00_JPRB, 0.40578E+00_JPRB, 0.46926E+00_JPRB, 0.51291E+00_JPRB ,&
&  0.53737E+00_JPRB, 0.54962E+00_JPRB, 0.55401E+00_JPRB, 0.54850E+00_JPRB, 0.54047E+00_JPRB, 0.53775E+00_JPRB, 0.53022E+00_JPRB ,&
&  0.51480E+00_JPRB, 0.49593E+00_JPRB, 0.47795E+00_JPRB, 0.46468E+00_JPRB, 0.45070E+00_JPRB, 0.43948E+00_JPRB, 0.42775E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.40717E+00_JPRB, 0.40169E+00_JPRB, 0.39754E+00_JPRB, 0.39242E+00_JPRB, 0.38453E+00_JPRB, 0.37577E+00_JPRB ,&
&  0.36875E+00_JPRB, 0.36292E+00_JPRB, 0.35750E+00_JPRB, 0.35270E+00_JPRB, 0.34992E+00_JPRB, 0.34834E+00_JPRB, 0.34653E+00_JPRB ,&
&  0.34610E+00_JPRB, 0.34628E+00_JPRB, 0.34691E+00_JPRB, 0.34722E+00_JPRB, 0.34673E+00_JPRB, 0.34541E+00_JPRB, 0.34564E+00_JPRB ,&
&  0.34689E+00_JPRB, 0.34557E+00_JPRB, 0.34509E+00_JPRB, 0.34752E+00_JPRB, 0.35039E+00_JPRB, 0.35215E+00_JPRB, 0.35250E+00_JPRB ,&
&  0.35147E+00_JPRB, 0.35122E+00_JPRB, 0.35247E+00_JPRB, 0.35567E+00_JPRB, 0.35981E+00_JPRB, 0.36586E+00_JPRB, 0.37326E+00_JPRB ,&
&  0.38108E+00_JPRB, 0.39089E+00_JPRB, 0.40362E+00_JPRB, 0.42565E+00_JPRB, 0.44624E+00_JPRB, 0.46627E+00_JPRB, 0.48637E+00_JPRB ,&
&  0.49725E+00_JPRB, 0.50538E+00_JPRB, 0.51578E+00_JPRB, 0.51796E+00_JPRB, 0.51902E+00_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.55369E+00_JPRB, 0.54989E+00_JPRB, 0.53875E+00_JPRB, 0.52583E+00_JPRB, 0.51465E+00_JPRB, 0.50362E+00_JPRB, 0.49471E+00_JPRB ,&
&  0.48334E+00_JPRB, 0.47026E+00_JPRB, 0.46032E+00_JPRB, 0.45520E+00_JPRB, 0.45221E+00_JPRB, 0.44817E+00_JPRB, 0.44204E+00_JPRB ,&
&  0.43450E+00_JPRB, 0.42659E+00_JPRB, 0.41909E+00_JPRB, 0.41007E+00_JPRB, 0.40145E+00_JPRB, 0.39384E+00_JPRB, 0.38797E+00_JPRB ,&
&  0.38221E+00_JPRB, 0.37597E+00_JPRB, 0.36977E+00_JPRB, 0.36560E+00_JPRB, 0.36225E+00_JPRB, 0.36074E+00_JPRB, 0.37386E+00_JPRB ,&
&  0.41455E+00_JPRB, 0.35659E+00_JPRB, 0.37831E+00_JPRB, 0.40776E+00_JPRB, 0.45351E+00_JPRB, 0.51613E+00_JPRB, 0.55373E+00_JPRB ,&
&  0.57282E+00_JPRB, 0.58436E+00_JPRB, 0.58519E+00_JPRB, 0.58376E+00_JPRB, 0.58565E+00_JPRB, 0.57797E+00_JPRB, 0.55757E+00_JPRB ,&
&  0.53538E+00_JPRB, 0.51383E+00_JPRB, 0.49759E+00_JPRB, 0.48567E+00_JPRB, 0.47561E+00_JPRB, 0.46824E+00_JPRB, 0.45824E+00_JPRB ,&
&  0.44322E+00_JPRB, 0.43475E+00_JPRB, 0.43010E+00_JPRB, 0.42635E+00_JPRB, 0.42449E+00_JPRB, 0.42088E+00_JPRB, 0.41062E+00_JPRB ,&
&  0.40145E+00_JPRB, 0.39584E+00_JPRB, 0.39415E+00_JPRB, 0.39521E+00_JPRB, 0.39701E+00_JPRB, 0.39744E+00_JPRB, 0.39488E+00_JPRB ,&
&  0.39149E+00_JPRB, 0.38688E+00_JPRB, 0.38429E+00_JPRB, 0.38263E+00_JPRB, 0.38120E+00_JPRB, 0.38224E+00_JPRB, 0.38356E+00_JPRB ,&
&  0.38352E+00_JPRB, 0.38429E+00_JPRB, 0.38664E+00_JPRB, 0.38861E+00_JPRB, 0.38876E+00_JPRB, 0.38757E+00_JPRB, 0.38587E+00_JPRB ,&
&  0.38388E+00_JPRB, 0.38290E+00_JPRB, 0.38437E+00_JPRB, 0.38965E+00_JPRB, 0.39729E+00_JPRB, 0.40632E+00_JPRB, 0.41655E+00_JPRB ,&
&  0.42820E+00_JPRB, 0.44072E+00_JPRB, 0.45203E+00_JPRB, 0.49461E+00_JPRB, 0.52469E+00_JPRB, 0.55167E+00_JPRB, 0.57612E+00_JPRB ,&
&  0.58684E+00_JPRB, 0.58871E+00_JPRB, 0.49468E+00_JPRB, 0.53669E+00_JPRB, 0.54725E+00_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.60408E+00_JPRB, 0.60176E+00_JPRB, 0.58758E+00_JPRB, 0.56826E+00_JPRB, 0.55141E+00_JPRB, 0.53683E+00_JPRB, 0.52510E+00_JPRB ,&
&  0.51064E+00_JPRB, 0.49840E+00_JPRB, 0.49294E+00_JPRB, 0.49092E+00_JPRB, 0.48872E+00_JPRB, 0.48356E+00_JPRB, 0.47693E+00_JPRB ,&
&  0.46993E+00_JPRB, 0.46211E+00_JPRB, 0.45307E+00_JPRB, 0.44390E+00_JPRB, 0.43612E+00_JPRB, 0.42792E+00_JPRB, 0.42184E+00_JPRB ,&
&  0.41630E+00_JPRB, 0.41154E+00_JPRB, 0.40380E+00_JPRB, 0.39417E+00_JPRB, 0.38997E+00_JPRB, 0.38486E+00_JPRB, 0.40094E+00_JPRB ,&
&  0.45419E+00_JPRB, 0.39461E+00_JPRB, 0.42387E+00_JPRB, 0.47274E+00_JPRB, 0.52306E+00_JPRB, 0.57998E+00_JPRB, 0.60828E+00_JPRB ,&
&  0.62549E+00_JPRB, 0.63735E+00_JPRB, 0.64260E+00_JPRB, 0.65149E+00_JPRB, 0.64949E+00_JPRB, 0.62458E+00_JPRB, 0.59145E+00_JPRB ,&
&  0.56581E+00_JPRB, 0.54127E+00_JPRB, 0.52460E+00_JPRB, 0.50872E+00_JPRB, 0.50081E+00_JPRB, 0.49771E+00_JPRB, 0.48984E+00_JPRB ,&
&  0.47866E+00_JPRB, 0.47327E+00_JPRB, 0.46853E+00_JPRB, 0.46837E+00_JPRB, 0.46972E+00_JPRB, 0.47202E+00_JPRB, 0.46615E+00_JPRB ,&
&  0.46059E+00_JPRB, 0.46698E+00_JPRB, 0.47806E+00_JPRB, 0.48292E+00_JPRB, 0.48372E+00_JPRB, 0.48029E+00_JPRB, 0.47398E+00_JPRB ,&
&  0.46871E+00_JPRB, 0.46470E+00_JPRB, 0.46395E+00_JPRB, 0.46190E+00_JPRB, 0.45915E+00_JPRB, 0.45549E+00_JPRB, 0.45191E+00_JPRB ,&
&  0.44921E+00_JPRB, 0.44713E+00_JPRB, 0.44498E+00_JPRB, 0.44226E+00_JPRB, 0.43846E+00_JPRB, 0.43284E+00_JPRB, 0.43016E+00_JPRB ,&
&  0.42956E+00_JPRB, 0.43151E+00_JPRB, 0.43609E+00_JPRB, 0.44361E+00_JPRB, 0.45421E+00_JPRB, 0.46444E+00_JPRB, 0.47627E+00_JPRB ,&
&  0.48766E+00_JPRB, 0.50062E+00_JPRB, 0.51420E+00_JPRB, 0.59938E+00_JPRB, 0.63953E+00_JPRB, 0.65349E+00_JPRB, 0.66503E+00_JPRB ,&
&  0.66909E+00_JPRB, 0.66705E+00_JPRB, 0.65916E+00_JPRB, 0.57740E+00_JPRB, 0.60109E+00_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.65342E+00_JPRB, 0.65097E+00_JPRB, 0.62969E+00_JPRB, 0.60463E+00_JPRB, 0.58222E+00_JPRB, 0.56446E+00_JPRB, 0.54927E+00_JPRB ,&
&  0.53853E+00_JPRB, 0.53056E+00_JPRB, 0.52485E+00_JPRB, 0.52093E+00_JPRB, 0.51607E+00_JPRB, 0.50918E+00_JPRB, 0.50210E+00_JPRB ,&
&  0.49647E+00_JPRB, 0.49000E+00_JPRB, 0.48014E+00_JPRB, 0.47295E+00_JPRB, 0.46857E+00_JPRB, 0.46364E+00_JPRB, 0.45887E+00_JPRB ,&
&  0.45292E+00_JPRB, 0.44095E+00_JPRB, 0.42889E+00_JPRB, 0.42095E+00_JPRB, 0.41491E+00_JPRB, 0.41250E+00_JPRB, 0.43051E+00_JPRB ,&
&  0.49011E+00_JPRB, 0.47007E+00_JPRB, 0.51200E+00_JPRB, 0.56006E+00_JPRB, 0.61719E+00_JPRB, 0.65244E+00_JPRB, 0.67112E+00_JPRB ,&
&  0.68454E+00_JPRB, 0.70498E+00_JPRB, 0.72008E+00_JPRB, 0.73992E+00_JPRB, 0.71349E+00_JPRB, 0.67140E+00_JPRB, 0.63639E+00_JPRB ,&
&  0.60412E+00_JPRB, 0.57486E+00_JPRB, 0.55031E+00_JPRB, 0.53312E+00_JPRB, 0.52692E+00_JPRB, 0.52427E+00_JPRB, 0.51929E+00_JPRB ,&
&  0.51476E+00_JPRB, 0.50679E+00_JPRB, 0.49891E+00_JPRB, 0.49934E+00_JPRB, 0.49763E+00_JPRB, 0.51292E+00_JPRB, 0.53286E+00_JPRB ,&
&  0.54923E+00_JPRB, 0.57049E+00_JPRB, 0.58054E+00_JPRB, 0.58663E+00_JPRB, 0.58127E+00_JPRB, 0.57380E+00_JPRB, 0.56577E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.55369E+00_JPRB, 0.55047E+00_JPRB, 0.54606E+00_JPRB, 0.54024E+00_JPRB, 0.53640E+00_JPRB, 0.53333E+00_JPRB ,&
&  0.53109E+00_JPRB, 0.52835E+00_JPRB, 0.52577E+00_JPRB, 0.51746E+00_JPRB, 0.50575E+00_JPRB, 0.49647E+00_JPRB, 0.48932E+00_JPRB ,&
&  0.48444E+00_JPRB, 0.48470E+00_JPRB, 0.48858E+00_JPRB, 0.49840E+00_JPRB, 0.50889E+00_JPRB, 0.52049E+00_JPRB, 0.52897E+00_JPRB ,&
&  0.53482E+00_JPRB, 0.55009E+00_JPRB, 0.57468E+00_JPRB, 0.62463E+00_JPRB, 0.65721E+00_JPRB, 0.68061E+00_JPRB, 0.70104E+00_JPRB ,&
&  0.71142E+00_JPRB, 0.71311E+00_JPRB, 0.69280E+00_JPRB, 0.67248E+00_JPRB, 0.64541E+00_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.74763E+00_JPRB, 0.71506E+00_JPRB, 0.67540E+00_JPRB, 0.63828E+00_JPRB, 0.60468E+00_JPRB, 0.57815E+00_JPRB, 0.56162E+00_JPRB ,&
&  0.55072E+00_JPRB, 0.54312E+00_JPRB, 0.53840E+00_JPRB, 0.53499E+00_JPRB, 0.53046E+00_JPRB, 0.52646E+00_JPRB, 0.52062E+00_JPRB ,&
&  0.51338E+00_JPRB, 0.50423E+00_JPRB, 0.49567E+00_JPRB, 0.48956E+00_JPRB, 0.48455E+00_JPRB, 0.48180E+00_JPRB, 0.47828E+00_JPRB ,&
&  0.47091E+00_JPRB, 0.46201E+00_JPRB, 0.45069E+00_JPRB, 0.43911E+00_JPRB, 0.43206E+00_JPRB, 0.43188E+00_JPRB, 0.46050E+00_JPRB ,&
&  0.54952E+00_JPRB, 0.56587E+00_JPRB, 0.61251E+00_JPRB, 0.66099E+00_JPRB, 0.72483E+00_JPRB, 0.76704E+00_JPRB, 0.78612E+00_JPRB ,&
&  0.80263E+00_JPRB, 0.82139E+00_JPRB, 0.85175E+00_JPRB, 0.84470E+00_JPRB, 0.78783E+00_JPRB, 0.73450E+00_JPRB, 0.68413E+00_JPRB ,&
&  0.63747E+00_JPRB, 0.59610E+00_JPRB, 0.56137E+00_JPRB, 0.54635E+00_JPRB, 0.54237E+00_JPRB, 0.54426E+00_JPRB, 0.54426E+00_JPRB ,&
&  0.54331E+00_JPRB, 0.53777E+00_JPRB, 0.53820E+00_JPRB, 0.55268E+00_JPRB, 0.58071E+00_JPRB, 0.62728E+00_JPRB, 0.70540E+00_JPRB ,&
&  0.74406E+00_JPRB, 0.73921E+00_JPRB, 0.74705E+00_JPRB, 0.74864E+00_JPRB, 0.72802E+00_JPRB, 0.70674E+00_JPRB, 0.68839E+00_JPRB ,&
&  0.67271E+00_JPRB, 0.65927E+00_JPRB, 0.64948E+00_JPRB, 0.63673E+00_JPRB, 0.62638E+00_JPRB, 0.62020E+00_JPRB, 0.61712E+00_JPRB ,&
&  0.61444E+00_JPRB, 0.61361E+00_JPRB, 0.60971E+00_JPRB, 0.59988E+00_JPRB, 0.59315E+00_JPRB, 0.58644E+00_JPRB, 0.57457E+00_JPRB ,&
&  0.55974E+00_JPRB, 0.55224E+00_JPRB, 0.55418E+00_JPRB, 0.56117E+00_JPRB, 0.57092E+00_JPRB, 0.57996E+00_JPRB, 0.58584E+00_JPRB ,&
&  0.59135E+00_JPRB, 0.61616E+00_JPRB, 0.62796E+00_JPRB, 0.69179E+00_JPRB, 0.74962E+00_JPRB, 0.76197E+00_JPRB, 0.77689E+00_JPRB ,&
&  0.78369E+00_JPRB, 0.77172E+00_JPRB, 0.74107E+00_JPRB, 0.71053E+00_JPRB, 0.75253E+00_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.72683E+00_JPRB, 0.69300E+00_JPRB, 0.65781E+00_JPRB, 0.62689E+00_JPRB, 0.59866E+00_JPRB, 0.57729E+00_JPRB, 0.56399E+00_JPRB ,&
&  0.55291E+00_JPRB, 0.54559E+00_JPRB, 0.54621E+00_JPRB, 0.55038E+00_JPRB, 0.54954E+00_JPRB, 0.54123E+00_JPRB, 0.52937E+00_JPRB ,&
&  0.51697E+00_JPRB, 0.50755E+00_JPRB, 0.50056E+00_JPRB, 0.49777E+00_JPRB, 0.49316E+00_JPRB, 0.48848E+00_JPRB, 0.48116E+00_JPRB ,&
&  0.47581E+00_JPRB, 0.47112E+00_JPRB, 0.46383E+00_JPRB, 0.45613E+00_JPRB, 0.45066E+00_JPRB, 0.45242E+00_JPRB, 0.49634E+00_JPRB ,&
&  0.61546E+00_JPRB, 0.72053E+00_JPRB, 0.74980E+00_JPRB, 0.80124E+00_JPRB, 0.89934E+00_JPRB, 0.92126E+00_JPRB, 0.97690E+00_JPRB ,&
&  0.10155E+01_JPRB, 0.10348E+01_JPRB, 0.10419E+01_JPRB, 0.98745E+00_JPRB, 0.88516E+00_JPRB, 0.80447E+00_JPRB, 0.72649E+00_JPRB ,&
&  0.65544E+00_JPRB, 0.59995E+00_JPRB, 0.56480E+00_JPRB, 0.55038E+00_JPRB, 0.55165E+00_JPRB, 0.55857E+00_JPRB, 0.56282E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.57136E+00_JPRB, 0.60138E+00_JPRB, 0.66965E+00_JPRB, 0.82250E+00_JPRB, 0.91732E+00_JPRB, 0.10761E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.11116E+01_JPRB, 0.10750E+01_JPRB, 0.10248E+01_JPRB, 0.97910E+00_JPRB, 0.92320E+00_JPRB, 0.87770E+00_JPRB ,&
&  0.82653E+00_JPRB, 0.77874E+00_JPRB, 0.74237E+00_JPRB, 0.71822E+00_JPRB, 0.70501E+00_JPRB, 0.69877E+00_JPRB, 0.69856E+00_JPRB ,&
&  0.69348E+00_JPRB, 0.69205E+00_JPRB, 0.68814E+00_JPRB, 0.68871E+00_JPRB, 0.69088E+00_JPRB, 0.67979E+00_JPRB, 0.66023E+00_JPRB ,&
&  0.63927E+00_JPRB, 0.62112E+00_JPRB, 0.62101E+00_JPRB, 0.62609E+00_JPRB, 0.62937E+00_JPRB, 0.62854E+00_JPRB, 0.62845E+00_JPRB ,&
&  0.64163E+00_JPRB, 0.68732E+00_JPRB, 0.70422E+00_JPRB, 0.72143E+00_JPRB, 0.79420E+00_JPRB, 0.85008E+00_JPRB, 0.85436E+00_JPRB ,&
&  0.84277E+00_JPRB, 0.81328E+00_JPRB, 0.77262E+00_JPRB, 0.73896E+00_JPRB, 0.74474E+00_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.72088E+00_JPRB, 0.69726E+00_JPRB, 0.66550E+00_JPRB, 0.63691E+00_JPRB, 0.61323E+00_JPRB, 0.58975E+00_JPRB, 0.57380E+00_JPRB ,&
&  0.56213E+00_JPRB, 0.56210E+00_JPRB, 0.56586E+00_JPRB, 0.56789E+00_JPRB, 0.56327E+00_JPRB, 0.55016E+00_JPRB, 0.53670E+00_JPRB ,&
&  0.52733E+00_JPRB, 0.52504E+00_JPRB, 0.52447E+00_JPRB, 0.52036E+00_JPRB, 0.51369E+00_JPRB, 0.50530E+00_JPRB, 0.49791E+00_JPRB ,&
&  0.49146E+00_JPRB, 0.48629E+00_JPRB, 0.48090E+00_JPRB, 0.47720E+00_JPRB, 0.47260E+00_JPRB, 0.47806E+00_JPRB, 0.53160E+00_JPRB ,&
&  0.67572E+00_JPRB, 0.80906E+00_JPRB, 0.10181E+01_JPRB, 0.10019E+01_JPRB, 0.10298E+01_JPRB, 0.10803E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.13481E+01_JPRB, 0.13829E+01_JPRB, 0.13114E+01_JPRB, 0.11562E+01_JPRB, 0.97306E+00_JPRB, 0.83890E+00_JPRB, 0.74003E+00_JPRB ,&
&  0.65916E+00_JPRB, 0.60824E+00_JPRB, 0.57947E+00_JPRB, 0.57132E+00_JPRB, 0.57531E+00_JPRB, 0.58577E+00_JPRB, 0.59384E+00_JPRB ,&
&  0.60763E+00_JPRB, 0.63191E+00_JPRB, 0.70788E+00_JPRB, 0.86179E+00_JPRB, 0.10684E+01_JPRB, 0.10907E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.17073E+01_JPRB, 0.15540E+01_JPRB, 0.14315E+01_JPRB, 0.13235E+01_JPRB, 0.12359E+01_JPRB, 0.11598E+01_JPRB, 0.10472E+01_JPRB ,&
&  0.96039E+00_JPRB, 0.88454E+00_JPRB, 0.82999E+00_JPRB, 0.80082E+00_JPRB, 0.78369E+00_JPRB, 0.77750E+00_JPRB, 0.77580E+00_JPRB ,&
&  0.77379E+00_JPRB, 0.77143E+00_JPRB, 0.77310E+00_JPRB, 0.76805E+00_JPRB, 0.75444E+00_JPRB, 0.73372E+00_JPRB, 0.70939E+00_JPRB ,&
&  0.69124E+00_JPRB, 0.67611E+00_JPRB, 0.68443E+00_JPRB, 0.66847E+00_JPRB, 0.67061E+00_JPRB, 0.68066E+00_JPRB, 0.68714E+00_JPRB ,&
&  0.69453E+00_JPRB, 0.72443E+00_JPRB, 0.74998E+00_JPRB, 0.76801E+00_JPRB, 0.87473E+00_JPRB, 0.95118E+00_JPRB, 0.93835E+00_JPRB ,&
&  0.90823E+00_JPRB, 0.84383E+00_JPRB, 0.78660E+00_JPRB, 0.74980E+00_JPRB, 0.73666E+00_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.71754E+00_JPRB, 0.68814E+00_JPRB, 0.66368E+00_JPRB, 0.64002E+00_JPRB, 0.62241E+00_JPRB, 0.60491E+00_JPRB, 0.59356E+00_JPRB ,&
&  0.58976E+00_JPRB, 0.59069E+00_JPRB, 0.58970E+00_JPRB, 0.58224E+00_JPRB, 0.56896E+00_JPRB, 0.55805E+00_JPRB, 0.54992E+00_JPRB ,&
&  0.54187E+00_JPRB, 0.53991E+00_JPRB, 0.53844E+00_JPRB, 0.53003E+00_JPRB, 0.52072E+00_JPRB, 0.51360E+00_JPRB, 0.50660E+00_JPRB ,&
&  0.50105E+00_JPRB, 0.49842E+00_JPRB, 0.49627E+00_JPRB, 0.49422E+00_JPRB, 0.49314E+00_JPRB, 0.50461E+00_JPRB, 0.55609E+00_JPRB ,&
&  0.66027E+00_JPRB, 0.57446E+00_JPRB, 0.10715E+01_JPRB, 0.12313E+01_JPRB, 0.12957E+01_JPRB, 0.13873E+01_JPRB, 0.16147E+01_JPRB ,&
&  0.18573E+01_JPRB, 0.18309E+01_JPRB, 0.15948E+01_JPRB, 0.12918E+01_JPRB, 0.10242E+01_JPRB, 0.84610E+00_JPRB, 0.75260E+00_JPRB ,&
&  0.68773E+00_JPRB, 0.64674E+00_JPRB, 0.62840E+00_JPRB, 0.61857E+00_JPRB, 0.62107E+00_JPRB, 0.63050E+00_JPRB, 0.64111E+00_JPRB ,&
&  0.66358E+00_JPRB, 0.70579E+00_JPRB, 0.80261E+00_JPRB, 0.96088E+00_JPRB, 0.10675E+01_JPRB, 0.11762E+01_JPRB, 0.13963E+01_JPRB ,&
&  0.18228E+01_JPRB, 0.19235E+01_JPRB, 0.16681E+01_JPRB, 0.14812E+01_JPRB, 0.13547E+01_JPRB, 0.12553E+01_JPRB, 0.11521E+01_JPRB ,&
&  0.10584E+01_JPRB, 0.97169E+00_JPRB, 0.91139E+00_JPRB, 0.86910E+00_JPRB, 0.84470E+00_JPRB, 0.83458E+00_JPRB, 0.82677E+00_JPRB ,&
&  0.81729E+00_JPRB, 0.80634E+00_JPRB, 0.79803E+00_JPRB, 0.78762E+00_JPRB, 0.77483E+00_JPRB, 0.75430E+00_JPRB, 0.72829E+00_JPRB ,&
&  0.71148E+00_JPRB, 0.70472E+00_JPRB, 0.70122E+00_JPRB, 0.69950E+00_JPRB, 0.70751E+00_JPRB, 0.71347E+00_JPRB, 0.72890E+00_JPRB ,&
&  0.77147E+00_JPRB, 0.79182E+00_JPRB, 0.80957E+00_JPRB, 0.84054E+00_JPRB, 0.87738E+00_JPRB, 0.97315E+00_JPRB, 0.91491E+00_JPRB ,&
&  0.87003E+00_JPRB, 0.81951E+00_JPRB, 0.77439E+00_JPRB, 0.75052E+00_JPRB, 0.73742E+00_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.70973E+00_JPRB, 0.68218E+00_JPRB, 0.66234E+00_JPRB, 0.64722E+00_JPRB, 0.63488E+00_JPRB, 0.62431E+00_JPRB, 0.62168E+00_JPRB ,&
&  0.61970E+00_JPRB, 0.61424E+00_JPRB, 0.59849E+00_JPRB, 0.57962E+00_JPRB, 0.57166E+00_JPRB, 0.56916E+00_JPRB, 0.55814E+00_JPRB ,&
&  0.55374E+00_JPRB, 0.55372E+00_JPRB, 0.55024E+00_JPRB, 0.54187E+00_JPRB, 0.53476E+00_JPRB, 0.53059E+00_JPRB, 0.52903E+00_JPRB ,&
&  0.52071E+00_JPRB, 0.51769E+00_JPRB, 0.51983E+00_JPRB, 0.52144E+00_JPRB, 0.52536E+00_JPRB, 0.54851E+00_JPRB, 0.57955E+00_JPRB ,&
&  0.64266E+00_JPRB, 0.86391E+00_JPRB, 0.90804E+00_JPRB, 0.13524E+01_JPRB, 0.15331E+01_JPRB, 0.17436E+01_JPRB, 0.18615E+01_JPRB ,&
&  0.20025E+01_JPRB, 0.21565E+01_JPRB, 0.17775E+01_JPRB, 0.13792E+01_JPRB, 0.10480E+01_JPRB, 0.87191E+00_JPRB, 0.79021E+00_JPRB ,&
&  0.74882E+00_JPRB, 0.71202E+00_JPRB, 0.68521E+00_JPRB, 0.67691E+00_JPRB, 0.68024E+00_JPRB, 0.68518E+00_JPRB, 0.69452E+00_JPRB ,&
&  0.72647E+00_JPRB, 0.79055E+00_JPRB, 0.91997E+00_JPRB, 0.10561E+01_JPRB, 0.11206E+01_JPRB, 0.13650E+01_JPRB, 0.16758E+01_JPRB ,&
&  0.19962E+01_JPRB, 0.21715E+01_JPRB, 0.18970E+01_JPRB, 0.16525E+01_JPRB, 0.14541E+01_JPRB, 0.12969E+01_JPRB, 0.11641E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.99301E+00_JPRB, 0.94025E+00_JPRB, 0.90221E+00_JPRB, 0.87580E+00_JPRB, 0.85712E+00_JPRB, 0.83165E+00_JPRB ,&
&  0.80302E+00_JPRB, 0.78805E+00_JPRB, 0.78287E+00_JPRB, 0.77823E+00_JPRB, 0.77116E+00_JPRB, 0.75976E+00_JPRB, 0.74492E+00_JPRB ,&
&  0.73518E+00_JPRB, 0.73509E+00_JPRB, 0.72961E+00_JPRB, 0.74508E+00_JPRB, 0.75761E+00_JPRB, 0.75054E+00_JPRB, 0.76061E+00_JPRB ,&
&  0.81492E+00_JPRB, 0.84957E+00_JPRB, 0.86255E+00_JPRB, 0.87370E+00_JPRB, 0.93663E+00_JPRB, 0.10541E+01_JPRB, 0.87898E+00_JPRB ,&
&  0.80996E+00_JPRB, 0.77951E+00_JPRB, 0.75490E+00_JPRB, 0.74036E+00_JPRB, 0.72703E+00_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.70578E+00_JPRB, 0.68752E+00_JPRB, 0.67423E+00_JPRB, 0.66005E+00_JPRB, 0.64792E+00_JPRB, 0.64263E+00_JPRB, 0.63798E+00_JPRB ,&
&  0.62855E+00_JPRB, 0.60462E+00_JPRB, 0.57972E+00_JPRB, 0.56898E+00_JPRB, 0.56607E+00_JPRB, 0.55669E+00_JPRB, 0.55862E+00_JPRB ,&
&  0.56097E+00_JPRB, 0.56557E+00_JPRB, 0.56015E+00_JPRB, 0.55589E+00_JPRB, 0.55343E+00_JPRB, 0.55255E+00_JPRB, 0.55209E+00_JPRB ,&
&  0.54969E+00_JPRB, 0.54908E+00_JPRB, 0.55444E+00_JPRB, 0.56140E+00_JPRB, 0.57424E+00_JPRB, 0.60201E+00_JPRB, 0.63327E+00_JPRB ,&
&  0.75101E+00_JPRB, 0.85761E+00_JPRB, 0.94014E+00_JPRB, 0.16289E+01_JPRB, 0.19235E+01_JPRB, 0.20431E+01_JPRB, 0.19445E+01_JPRB ,&
&  0.19859E+01_JPRB, 0.22098E+01_JPRB, 0.18895E+01_JPRB, 0.14494E+01_JPRB, 0.11446E+01_JPRB, 0.10047E+01_JPRB, 0.91934E+00_JPRB ,&
&  0.84966E+00_JPRB, 0.78721E+00_JPRB, 0.74659E+00_JPRB, 0.72301E+00_JPRB, 0.72606E+00_JPRB, 0.73885E+00_JPRB, 0.76923E+00_JPRB ,&
&  0.82073E+00_JPRB, 0.91867E+00_JPRB, 0.10505E+01_JPRB, 0.11469E+01_JPRB, 0.14756E+01_JPRB, 0.18547E+01_JPRB, 0.20340E+01_JPRB ,&
&  0.20978E+01_JPRB, 0.21752E+01_JPRB, 0.18617E+01_JPRB, 0.16495E+01_JPRB, 0.14053E+01_JPRB, 0.12801E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.10582E+01_JPRB, 0.98529E+00_JPRB, 0.93238E+00_JPRB, 0.90289E+00_JPRB, 0.87480E+00_JPRB, 0.84374E+00_JPRB, 0.80922E+00_JPRB ,&
&  0.78031E+00_JPRB, 0.77710E+00_JPRB, 0.77671E+00_JPRB, 0.76921E+00_JPRB, 0.76576E+00_JPRB, 0.76430E+00_JPRB, 0.76453E+00_JPRB ,&
&  0.76387E+00_JPRB, 0.76445E+00_JPRB, 0.75895E+00_JPRB, 0.78486E+00_JPRB, 0.80953E+00_JPRB, 0.80972E+00_JPRB, 0.82463E+00_JPRB ,&
&  0.86875E+00_JPRB, 0.90298E+00_JPRB, 0.89639E+00_JPRB, 0.86985E+00_JPRB, 0.89607E+00_JPRB, 0.91930E+00_JPRB, 0.82135E+00_JPRB ,&
&  0.79777E+00_JPRB, 0.77651E+00_JPRB, 0.74771E+00_JPRB, 0.72851E+00_JPRB, 0.71798E+00_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.70974E+00_JPRB, 0.69102E+00_JPRB, 0.67158E+00_JPRB, 0.65671E+00_JPRB, 0.64703E+00_JPRB, 0.63683E+00_JPRB, 0.61929E+00_JPRB ,&
&  0.59628E+00_JPRB, 0.58274E+00_JPRB, 0.55898E+00_JPRB, 0.55337E+00_JPRB, 0.55940E+00_JPRB, 0.56446E+00_JPRB, 0.57434E+00_JPRB ,&
&  0.58039E+00_JPRB, 0.58398E+00_JPRB, 0.58329E+00_JPRB, 0.58058E+00_JPRB, 0.58046E+00_JPRB, 0.58133E+00_JPRB, 0.58501E+00_JPRB ,&
&  0.58359E+00_JPRB, 0.58705E+00_JPRB, 0.59651E+00_JPRB, 0.61038E+00_JPRB, 0.64430E+00_JPRB, 0.70451E+00_JPRB, 0.78757E+00_JPRB ,&
&  0.98166E+00_JPRB, 0.10574E+01_JPRB, 0.11233E+01_JPRB, 0.16476E+01_JPRB, 0.19671E+01_JPRB, 0.18973E+01_JPRB, 0.18927E+01_JPRB ,&
&  0.19350E+01_JPRB, 0.19126E+01_JPRB, 0.18002E+01_JPRB, 0.14547E+01_JPRB, 0.12144E+01_JPRB, 0.10884E+01_JPRB, 0.99411E+00_JPRB ,&
&  0.90689E+00_JPRB, 0.84185E+00_JPRB, 0.79335E+00_JPRB, 0.77616E+00_JPRB, 0.79629E+00_JPRB, 0.84216E+00_JPRB, 0.88934E+00_JPRB ,&
&  0.97336E+00_JPRB, 0.11054E+01_JPRB, 0.12854E+01_JPRB, 0.15223E+01_JPRB, 0.19918E+01_JPRB, 0.23324E+01_JPRB, 0.22134E+01_JPRB ,&
&  0.20861E+01_JPRB, 0.19793E+01_JPRB, 0.17701E+01_JPRB, 0.15971E+01_JPRB, 0.12820E+01_JPRB, 0.12509E+01_JPRB, 0.11383E+01_JPRB ,&
&  0.10336E+01_JPRB, 0.95881E+00_JPRB, 0.91842E+00_JPRB, 0.89640E+00_JPRB, 0.86607E+00_JPRB, 0.82455E+00_JPRB, 0.79245E+00_JPRB ,&
&  0.77658E+00_JPRB, 0.77579E+00_JPRB, 0.77610E+00_JPRB, 0.76988E+00_JPRB, 0.76895E+00_JPRB, 0.77435E+00_JPRB, 0.78075E+00_JPRB ,&
&  0.78535E+00_JPRB, 0.79564E+00_JPRB, 0.80488E+00_JPRB, 0.84748E+00_JPRB, 0.88221E+00_JPRB, 0.89889E+00_JPRB, 0.89166E+00_JPRB ,&
&  0.89879E+00_JPRB, 0.91861E+00_JPRB, 0.90058E+00_JPRB, 0.89651E+00_JPRB, 0.88560E+00_JPRB, 0.86942E+00_JPRB, 0.83129E+00_JPRB ,&
&  0.82509E+00_JPRB, 0.79522E+00_JPRB, 0.75604E+00_JPRB, 0.73778E+00_JPRB, 0.72403E+00_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.70453E+00_JPRB, 0.68216E+00_JPRB, 0.65419E+00_JPRB, 0.62619E+00_JPRB, 0.61157E+00_JPRB, 0.59706E+00_JPRB, 0.57885E+00_JPRB ,&
&  0.56972E+00_JPRB, 0.56570E+00_JPRB, 0.55734E+00_JPRB, 0.56841E+00_JPRB, 0.59242E+00_JPRB, 0.60474E+00_JPRB, 0.61232E+00_JPRB ,&
&  0.62104E+00_JPRB, 0.62750E+00_JPRB, 0.61802E+00_JPRB, 0.61680E+00_JPRB, 0.62699E+00_JPRB, 0.63627E+00_JPRB, 0.63970E+00_JPRB ,&
&  0.64597E+00_JPRB, 0.65972E+00_JPRB, 0.67858E+00_JPRB, 0.71490E+00_JPRB, 0.79242E+00_JPRB, 0.91168E+00_JPRB, 0.10719E+01_JPRB ,&
&  0.13002E+01_JPRB, 0.12824E+01_JPRB, 0.75350E+00_JPRB, 0.12509E+01_JPRB, 0.16838E+01_JPRB, 0.17020E+01_JPRB, 0.16067E+01_JPRB ,&
&  0.15994E+01_JPRB, 0.15569E+01_JPRB, 0.15155E+01_JPRB, 0.13541E+01_JPRB, 0.12097E+01_JPRB, 0.10951E+01_JPRB, 0.99738E+00_JPRB ,&
&  0.91457E+00_JPRB, 0.86068E+00_JPRB, 0.84412E+00_JPRB, 0.87368E+00_JPRB, 0.92781E+00_JPRB, 0.96835E+00_JPRB, 0.10411E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.13469E+01_JPRB, 0.16426E+01_JPRB, 0.19365E+01_JPRB, 0.24221E+01_JPRB, 0.24113E+01_JPRB, 0.19962E+01_JPRB ,&
&  0.16401E+01_JPRB, 0.16494E+01_JPRB, 0.16156E+01_JPRB, 0.14454E+01_JPRB, 0.11861E+01_JPRB, 0.10964E+01_JPRB, 0.10832E+01_JPRB ,&
&  0.99232E+00_JPRB, 0.92383E+00_JPRB, 0.89365E+00_JPRB, 0.86344E+00_JPRB, 0.82163E+00_JPRB, 0.78555E+00_JPRB, 0.76854E+00_JPRB ,&
&  0.76776E+00_JPRB, 0.76662E+00_JPRB, 0.76448E+00_JPRB, 0.76297E+00_JPRB, 0.76776E+00_JPRB, 0.77488E+00_JPRB, 0.78680E+00_JPRB ,&
&  0.80824E+00_JPRB, 0.83651E+00_JPRB, 0.88202E+00_JPRB, 0.91128E+00_JPRB, 0.92866E+00_JPRB, 0.94245E+00_JPRB, 0.95781E+00_JPRB ,&
&  0.97243E+00_JPRB, 0.96814E+00_JPRB, 0.89328E+00_JPRB, 0.87219E+00_JPRB, 0.90856E+00_JPRB, 0.90398E+00_JPRB, 0.92194E+00_JPRB ,&
&  0.94960E+00_JPRB, 0.95058E+00_JPRB, 0.88955E+00_JPRB, 0.75590E+00_JPRB, 0.73167E+00_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.64844E+00_JPRB, 0.61886E+00_JPRB, 0.58455E+00_JPRB, 0.57660E+00_JPRB, 0.57629E+00_JPRB, 0.57499E+00_JPRB, 0.59074E+00_JPRB ,&
&  0.60545E+00_JPRB, 0.62784E+00_JPRB, 0.63474E+00_JPRB, 0.65665E+00_JPRB, 0.66839E+00_JPRB, 0.66993E+00_JPRB, 0.67979E+00_JPRB ,&
&  0.68598E+00_JPRB, 0.69228E+00_JPRB, 0.69982E+00_JPRB, 0.71245E+00_JPRB, 0.72771E+00_JPRB, 0.73423E+00_JPRB, 0.74390E+00_JPRB ,&
&  0.76528E+00_JPRB, 0.79222E+00_JPRB, 0.83179E+00_JPRB, 0.91938E+00_JPRB, 0.10683E+01_JPRB, 0.12629E+01_JPRB, 0.14783E+01_JPRB ,&
&  0.16553E+01_JPRB, 0.12268E+01_JPRB, 0.88733E+00_JPRB, 0.14777E+01_JPRB, 0.16315E+01_JPRB, 0.16055E+01_JPRB, 0.14146E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.13923E+01_JPRB, 0.13433E+01_JPRB, 0.12749E+01_JPRB, 0.11203E+01_JPRB, 0.10376E+01_JPRB, 0.97491E+00_JPRB ,&
&  0.94237E+00_JPRB, 0.93235E+00_JPRB, 0.95384E+00_JPRB, 0.10040E+01_JPRB, 0.10574E+01_JPRB, 0.11231E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.14298E+01_JPRB, 0.16518E+01_JPRB, 0.20116E+01_JPRB, 0.21474E+01_JPRB, 0.24023E+01_JPRB, 0.20912E+01_JPRB, 0.17572E+01_JPRB ,&
&  0.14419E+01_JPRB, 0.14192E+01_JPRB, 0.14420E+01_JPRB, 0.13080E+01_JPRB, 0.11105E+01_JPRB, 0.10063E+01_JPRB, 0.10055E+01_JPRB ,&
&  0.91599E+00_JPRB, 0.86883E+00_JPRB, 0.83166E+00_JPRB, 0.78342E+00_JPRB, 0.75759E+00_JPRB, 0.75496E+00_JPRB, 0.75952E+00_JPRB ,&
&  0.76138E+00_JPRB, 0.75522E+00_JPRB, 0.75206E+00_JPRB, 0.75988E+00_JPRB, 0.77457E+00_JPRB, 0.78591E+00_JPRB, 0.80377E+00_JPRB ,&
&  0.82794E+00_JPRB, 0.86007E+00_JPRB, 0.89993E+00_JPRB, 0.93005E+00_JPRB, 0.92924E+00_JPRB, 0.90959E+00_JPRB, 0.93178E+00_JPRB ,&
&  0.93406E+00_JPRB, 0.94754E+00_JPRB, 0.93493E+00_JPRB, 0.94680E+00_JPRB, 0.93962E+00_JPRB, 0.94296E+00_JPRB, 0.93863E+00_JPRB ,&
&  0.93223E+00_JPRB, 0.90263E+00_JPRB, 0.81641E+00_JPRB, 0.70633E+00_JPRB, 0.67954E+00_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.57245E+00_JPRB, 0.56583E+00_JPRB, 0.57151E+00_JPRB, 0.57817E+00_JPRB, 0.59126E+00_JPRB, 0.61806E+00_JPRB, 0.64904E+00_JPRB ,&
&  0.67492E+00_JPRB, 0.71893E+00_JPRB, 0.74994E+00_JPRB, 0.75205E+00_JPRB, 0.74684E+00_JPRB, 0.75260E+00_JPRB, 0.76596E+00_JPRB ,&
&  0.78617E+00_JPRB, 0.80257E+00_JPRB, 0.81414E+00_JPRB, 0.82573E+00_JPRB, 0.84221E+00_JPRB, 0.85978E+00_JPRB, 0.88355E+00_JPRB ,&
&  0.92049E+00_JPRB, 0.98222E+00_JPRB, 0.10750E+01_JPRB, 0.12144E+01_JPRB, 0.14027E+01_JPRB, 0.16139E+01_JPRB, 0.17399E+01_JPRB ,&
&  0.12889E+01_JPRB, 0.70323E+00_JPRB, 0.13733E+01_JPRB, 0.16256E+01_JPRB, 0.14874E+01_JPRB, 0.14433E+01_JPRB, 0.13835E+01_JPRB ,&
&  0.13102E+01_JPRB, 0.12917E+01_JPRB, 0.12356E+01_JPRB, 0.12142E+01_JPRB, 0.10876E+01_JPRB, 0.10398E+01_JPRB, 0.10230E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.10435E+01_JPRB, 0.10737E+01_JPRB, 0.11179E+01_JPRB, 0.11881E+01_JPRB, 0.12962E+01_JPRB, 0.14647E+01_JPRB ,&
&  0.16532E+01_JPRB, 0.18736E+01_JPRB, 0.21844E+01_JPRB, 0.20681E+01_JPRB, 0.21255E+01_JPRB, 0.19704E+01_JPRB, 0.17086E+01_JPRB ,&
&  0.14301E+01_JPRB, 0.12234E+01_JPRB, 0.12141E+01_JPRB, 0.12015E+01_JPRB, 0.10546E+01_JPRB, 0.91823E+00_JPRB, 0.91984E+00_JPRB ,&
&  0.84215E+00_JPRB, 0.80817E+00_JPRB, 0.75431E+00_JPRB, 0.71371E+00_JPRB, 0.70332E+00_JPRB, 0.71659E+00_JPRB, 0.73294E+00_JPRB ,&
&  0.73751E+00_JPRB, 0.73827E+00_JPRB, 0.74467E+00_JPRB, 0.75591E+00_JPRB, 0.77712E+00_JPRB, 0.79412E+00_JPRB, 0.80478E+00_JPRB ,&
&  0.83537E+00_JPRB, 0.86895E+00_JPRB, 0.90071E+00_JPRB, 0.92836E+00_JPRB, 0.94694E+00_JPRB, 0.92939E+00_JPRB, 0.92825E+00_JPRB ,&
&  0.93692E+00_JPRB, 0.94286E+00_JPRB, 0.94677E+00_JPRB, 0.96305E+00_JPRB, 0.95264E+00_JPRB, 0.90573E+00_JPRB, 0.88780E+00_JPRB ,&
&  0.82362E+00_JPRB, 0.75277E+00_JPRB, 0.66820E+00_JPRB, 0.61670E+00_JPRB, 0.59772E+00_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.58915E+00_JPRB, 0.60556E+00_JPRB, 0.62430E+00_JPRB, 0.64684E+00_JPRB, 0.68098E+00_JPRB, 0.72095E+00_JPRB, 0.77239E+00_JPRB ,&
&  0.82494E+00_JPRB, 0.84165E+00_JPRB, 0.84617E+00_JPRB, 0.84888E+00_JPRB, 0.85358E+00_JPRB, 0.86660E+00_JPRB, 0.88413E+00_JPRB ,&
&  0.90144E+00_JPRB, 0.91851E+00_JPRB, 0.93564E+00_JPRB, 0.95517E+00_JPRB, 0.97927E+00_JPRB, 0.10153E+01_JPRB, 0.10632E+01_JPRB ,&
&  0.11191E+01_JPRB, 0.12138E+01_JPRB, 0.13395E+01_JPRB, 0.14824E+01_JPRB, 0.16346E+01_JPRB, 0.17581E+01_JPRB, 0.17998E+01_JPRB ,&
&  0.10721E+01_JPRB, 0.13420E+01_JPRB, 0.15912E+01_JPRB, 0.15139E+01_JPRB, 0.14133E+01_JPRB, 0.13606E+01_JPRB, 0.13448E+01_JPRB ,&
&  0.12857E+01_JPRB, 0.11986E+01_JPRB, 0.11700E+01_JPRB, 0.12111E+01_JPRB, 0.11083E+01_JPRB, 0.10791E+01_JPRB, 0.10898E+01_JPRB ,&
&  0.11130E+01_JPRB, 0.11427E+01_JPRB, 0.11922E+01_JPRB, 0.12539E+01_JPRB, 0.13428E+01_JPRB, 0.14760E+01_JPRB, 0.16306E+01_JPRB ,&
&  0.18068E+01_JPRB, 0.19908E+01_JPRB, 0.21721E+01_JPRB, 0.19390E+01_JPRB, 0.19041E+01_JPRB, 0.19502E+01_JPRB, 0.18014E+01_JPRB ,&
&  0.13137E+01_JPRB, 0.10931E+01_JPRB, 0.11233E+01_JPRB, 0.11103E+01_JPRB, 0.98190E+00_JPRB, 0.90658E+00_JPRB, 0.84079E+00_JPRB ,&
&  0.78658E+00_JPRB, 0.73336E+00_JPRB, 0.67940E+00_JPRB, 0.66535E+00_JPRB, 0.66780E+00_JPRB, 0.68437E+00_JPRB, 0.70462E+00_JPRB ,&
&  0.71676E+00_JPRB, 0.72189E+00_JPRB, 0.73350E+00_JPRB, 0.75010E+00_JPRB, 0.78417E+00_JPRB, 0.80384E+00_JPRB, 0.81275E+00_JPRB ,&
&  0.84095E+00_JPRB, 0.85738E+00_JPRB, 0.88076E+00_JPRB, 0.90625E+00_JPRB, 0.92947E+00_JPRB, 0.94380E+00_JPRB, 0.94055E+00_JPRB ,&
&  0.93460E+00_JPRB, 0.95630E+00_JPRB, 0.98336E+00_JPRB, 0.98325E+00_JPRB, 0.94821E+00_JPRB, 0.85876E+00_JPRB, 0.78072E+00_JPRB ,&
&  0.69679E+00_JPRB, 0.67688E+00_JPRB, 0.64363E+00_JPRB, 0.61762E+00_JPRB, 0.59732E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.68612E+00_JPRB, 0.70398E+00_JPRB, 0.77880E+00_JPRB, 0.80245E+00_JPRB, 0.87643E+00_JPRB, 0.90798E+00_JPRB, 0.94139E+00_JPRB ,&
&  0.94968E+00_JPRB, 0.95444E+00_JPRB, 0.95774E+00_JPRB, 0.96484E+00_JPRB, 0.97346E+00_JPRB, 0.98784E+00_JPRB, 0.10012E+01_JPRB ,&
&  0.10195E+01_JPRB, 0.10434E+01_JPRB, 0.10726E+01_JPRB, 0.11034E+01_JPRB, 0.11438E+01_JPRB, 0.11918E+01_JPRB, 0.12550E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.14252E+01_JPRB, 0.15233E+01_JPRB, 0.16293E+01_JPRB, 0.17414E+01_JPRB, 0.18271E+01_JPRB, 0.18471E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.15745E+01_JPRB, 0.15584E+01_JPRB, 0.14872E+01_JPRB, 0.13917E+01_JPRB, 0.13274E+01_JPRB, 0.13044E+01_JPRB ,&
&  0.12800E+01_JPRB, 0.12027E+01_JPRB, 0.11801E+01_JPRB, 0.11569E+01_JPRB, 0.11468E+01_JPRB, 0.11179E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.11915E+01_JPRB, 0.12469E+01_JPRB, 0.13076E+01_JPRB, 0.13614E+01_JPRB, 0.14558E+01_JPRB, 0.15796E+01_JPRB, 0.17153E+01_JPRB ,&
&  0.18540E+01_JPRB, 0.19815E+01_JPRB, 0.20671E+01_JPRB, 0.18321E+01_JPRB, 0.16255E+01_JPRB, 0.15609E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.96476E+00_JPRB, 0.10358E+01_JPRB, 0.10492E+01_JPRB, 0.93642E+00_JPRB, 0.84448E+00_JPRB, 0.77436E+00_JPRB ,&
&  0.70626E+00_JPRB, 0.65368E+00_JPRB, 0.63761E+00_JPRB, 0.63000E+00_JPRB, 0.63210E+00_JPRB, 0.65462E+00_JPRB, 0.67475E+00_JPRB ,&
&  0.70262E+00_JPRB, 0.73667E+00_JPRB, 0.76676E+00_JPRB, 0.80172E+00_JPRB, 0.82234E+00_JPRB, 0.84376E+00_JPRB, 0.85200E+00_JPRB ,&
&  0.86659E+00_JPRB, 0.90224E+00_JPRB, 0.90387E+00_JPRB, 0.92523E+00_JPRB, 0.92530E+00_JPRB, 0.94641E+00_JPRB, 0.97187E+00_JPRB ,&
&  0.99641E+00_JPRB, 0.10191E+01_JPRB, 0.10160E+01_JPRB, 0.98645E+00_JPRB, 0.92529E+00_JPRB, 0.84662E+00_JPRB, 0.68827E+00_JPRB ,&
&  0.66326E+00_JPRB, 0.64471E+00_JPRB, 0.65247E+00_JPRB, 0.65873E+00_JPRB, 0.67700E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.86035E+00_JPRB, 0.91486E+00_JPRB, 0.96983E+00_JPRB, 0.10397E+01_JPRB, 0.10979E+01_JPRB, 0.10942E+01_JPRB, 0.10703E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10508E+01_JPRB, 0.10620E+01_JPRB, 0.10818E+01_JPRB, 0.10985E+01_JPRB, 0.11104E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11856E+01_JPRB, 0.12221E+01_JPRB, 0.12633E+01_JPRB, 0.13003E+01_JPRB, 0.13531E+01_JPRB, 0.14145E+01_JPRB ,&
&  0.14796E+01_JPRB, 0.15528E+01_JPRB, 0.16129E+01_JPRB, 0.16716E+01_JPRB, 0.17261E+01_JPRB, 0.17223E+01_JPRB, 0.12698E+01_JPRB ,&
&  0.15499E+01_JPRB, 0.15838E+01_JPRB, 0.16114E+01_JPRB, 0.15364E+01_JPRB, 0.14044E+01_JPRB, 0.13227E+01_JPRB, 0.12916E+01_JPRB ,&
&  0.12626E+01_JPRB, 0.12197E+01_JPRB, 0.11677E+01_JPRB, 0.11322E+01_JPRB, 0.12155E+01_JPRB, 0.11711E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.12601E+01_JPRB, 0.13030E+01_JPRB, 0.13419E+01_JPRB, 0.14023E+01_JPRB, 0.15053E+01_JPRB, 0.16072E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.18297E+01_JPRB, 0.19178E+01_JPRB, 0.19623E+01_JPRB, 0.17145E+01_JPRB, 0.16290E+01_JPRB, 0.15340E+01_JPRB, 0.11589E+01_JPRB ,&
&  0.97609E+00_JPRB, 0.94068E+00_JPRB, 0.10077E+01_JPRB, 0.10104E+01_JPRB, 0.90586E+00_JPRB, 0.79626E+00_JPRB, 0.73186E+00_JPRB ,&
&  0.66085E+00_JPRB, 0.62756E+00_JPRB, 0.62198E+00_JPRB, 0.60620E+00_JPRB, 0.60291E+00_JPRB, 0.63342E+00_JPRB, 0.67400E+00_JPRB ,&
&  0.72108E+00_JPRB, 0.78089E+00_JPRB, 0.83209E+00_JPRB, 0.87696E+00_JPRB, 0.93089E+00_JPRB, 0.10206E+01_JPRB, 0.10721E+01_JPRB ,&
&  0.97669E+00_JPRB, 0.92631E+00_JPRB, 0.96723E+00_JPRB, 0.10072E+01_JPRB, 0.10520E+01_JPRB, 0.11274E+01_JPRB, 0.12054E+01_JPRB ,&
&  0.13313E+01_JPRB, 0.14008E+01_JPRB, 0.11887E+01_JPRB, 0.99743E+00_JPRB, 0.13152E+01_JPRB, 0.14033E+01_JPRB, 0.73988E+00_JPRB ,&
&  0.70947E+00_JPRB, 0.72879E+00_JPRB, 0.74308E+00_JPRB, 0.77982E+00_JPRB, 0.81854E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.10362E+01_JPRB, 0.10705E+01_JPRB, 0.11072E+01_JPRB, 0.11323E+01_JPRB, 0.11476E+01_JPRB, 0.11369E+01_JPRB, 0.11267E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.11413E+01_JPRB, 0.11601E+01_JPRB, 0.11807E+01_JPRB, 0.11952E+01_JPRB, 0.12100E+01_JPRB, 0.12356E+01_JPRB ,&
&  0.12673E+01_JPRB, 0.13070E+01_JPRB, 0.13564E+01_JPRB, 0.13958E+01_JPRB, 0.14381E+01_JPRB, 0.14730E+01_JPRB, 0.15195E+01_JPRB ,&
&  0.15595E+01_JPRB, 0.16173E+01_JPRB, 0.16311E+01_JPRB, 0.16646E+01_JPRB, 0.16841E+01_JPRB, 0.17431E+01_JPRB, 0.14075E+01_JPRB ,&
&  0.15817E+01_JPRB, 0.16357E+01_JPRB, 0.16196E+01_JPRB, 0.15373E+01_JPRB, 0.14117E+01_JPRB, 0.13504E+01_JPRB, 0.13094E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.12585E+01_JPRB, 0.11941E+01_JPRB, 0.12341E+01_JPRB, 0.12638E+01_JPRB, 0.12141E+01_JPRB, 0.12428E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.13091E+01_JPRB, 0.13488E+01_JPRB, 0.14195E+01_JPRB, 0.15189E+01_JPRB, 0.16081E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.17856E+01_JPRB, 0.18819E+01_JPRB, 0.19503E+01_JPRB, 0.17542E+01_JPRB, 0.17357E+01_JPRB, 0.15843E+01_JPRB, 0.12630E+01_JPRB ,&
&  0.96477E+00_JPRB, 0.88453E+00_JPRB, 0.92911E+00_JPRB, 0.10064E+01_JPRB, 0.89776E+00_JPRB, 0.77308E+00_JPRB, 0.72648E+00_JPRB ,&
&  0.66899E+00_JPRB, 0.63403E+00_JPRB, 0.62706E+00_JPRB, 0.60374E+00_JPRB, 0.59399E+00_JPRB, 0.62650E+00_JPRB, 0.66323E+00_JPRB ,&
&  0.71174E+00_JPRB, 0.77727E+00_JPRB, 0.85384E+00_JPRB, 0.93149E+00_JPRB, 0.10117E+01_JPRB, 0.10370E+01_JPRB, 0.96222E+00_JPRB ,&
&  0.98188E+00_JPRB, 0.99516E+00_JPRB, 0.10502E+01_JPRB, 0.11568E+01_JPRB, 0.12220E+01_JPRB, 0.14422E+01_JPRB, 0.16783E+01_JPRB ,&
&  0.18999E+01_JPRB, 0.17757E+01_JPRB, 0.21000E+01_JPRB, 0.29083E+01_JPRB, 0.26837E+01_JPRB, 0.21818E+01_JPRB, 0.85698E+00_JPRB ,&
&  0.85405E+00_JPRB, 0.85667E+00_JPRB, 0.89510E+00_JPRB, 0.94426E+00_JPRB, 0.10012E+01_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.10543E+01_JPRB, 0.10838E+01_JPRB, 0.11135E+01_JPRB, 0.11423E+01_JPRB, 0.11651E+01_JPRB, 0.11829E+01_JPRB, 0.11952E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.12387E+01_JPRB, 0.12545E+01_JPRB, 0.12633E+01_JPRB, 0.12724E+01_JPRB, 0.12970E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.13698E+01_JPRB, 0.13940E+01_JPRB, 0.14364E+01_JPRB, 0.14242E+01_JPRB, 0.14103E+01_JPRB, 0.14188E+01_JPRB, 0.14530E+01_JPRB ,&
&  0.14652E+01_JPRB, 0.14608E+01_JPRB, 0.14565E+01_JPRB, 0.14903E+01_JPRB, 0.15290E+01_JPRB, 0.15956E+01_JPRB, 0.14038E+01_JPRB ,&
&  0.15353E+01_JPRB, 0.16250E+01_JPRB, 0.15786E+01_JPRB, 0.15004E+01_JPRB, 0.13665E+01_JPRB, 0.13234E+01_JPRB, 0.13092E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.12797E+01_JPRB, 0.12540E+01_JPRB, 0.12398E+01_JPRB, 0.12392E+01_JPRB, 0.12365E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.12980E+01_JPRB, 0.13429E+01_JPRB, 0.13988E+01_JPRB, 0.14837E+01_JPRB, 0.15671E+01_JPRB, 0.16164E+01_JPRB ,&
&  0.16782E+01_JPRB, 0.17350E+01_JPRB, 0.16233E+01_JPRB, 0.16156E+01_JPRB, 0.16864E+01_JPRB, 0.15223E+01_JPRB, 0.12276E+01_JPRB ,&
&  0.95120E+00_JPRB, 0.89897E+00_JPRB, 0.92928E+00_JPRB, 0.10058E+01_JPRB, 0.90061E+00_JPRB, 0.80683E+00_JPRB, 0.75126E+00_JPRB ,&
&  0.66749E+00_JPRB, 0.59654E+00_JPRB, 0.60914E+00_JPRB, 0.60462E+00_JPRB, 0.59511E+00_JPRB, 0.62029E+00_JPRB, 0.65072E+00_JPRB ,&
&  0.68621E+00_JPRB, 0.74974E+00_JPRB, 0.81500E+00_JPRB, 0.88411E+00_JPRB, 0.92587E+00_JPRB, 0.88611E+00_JPRB, 0.96179E+00_JPRB ,&
&  0.92701E+00_JPRB, 0.91180E+00_JPRB, 0.94006E+00_JPRB, 0.79032E+00_JPRB, 0.10659E+01_JPRB, 0.11309E+01_JPRB, 0.12488E+01_JPRB ,&
&  0.12182E+01_JPRB, 0.16319E+01_JPRB, 0.20096E+01_JPRB, 0.15745E+01_JPRB, 0.12518E+01_JPRB, 0.93317E+00_JPRB, 0.88254E+00_JPRB ,&
&  0.89248E+00_JPRB, 0.91108E+00_JPRB, 0.94366E+00_JPRB, 0.98383E+00_JPRB, 0.10205E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.10599E+01_JPRB, 0.10883E+01_JPRB, 0.11170E+01_JPRB, 0.11488E+01_JPRB, 0.11810E+01_JPRB, 0.12045E+01_JPRB, 0.12293E+01_JPRB ,&
&  0.12550E+01_JPRB, 0.12747E+01_JPRB, 0.12822E+01_JPRB, 0.12898E+01_JPRB, 0.13044E+01_JPRB, 0.13447E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.13718E+01_JPRB, 0.13908E+01_JPRB, 0.14015E+01_JPRB, 0.14096E+01_JPRB, 0.13976E+01_JPRB, 0.13641E+01_JPRB, 0.13743E+01_JPRB ,&
&  0.13695E+01_JPRB, 0.13729E+01_JPRB, 0.13741E+01_JPRB, 0.13983E+01_JPRB, 0.14376E+01_JPRB, 0.15385E+01_JPRB, 0.13100E+01_JPRB ,&
&  0.14095E+01_JPRB, 0.15273E+01_JPRB, 0.15226E+01_JPRB, 0.14002E+01_JPRB, 0.12620E+01_JPRB, 0.12239E+01_JPRB, 0.12164E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.12000E+01_JPRB, 0.11973E+01_JPRB, 0.12011E+01_JPRB, 0.12175E+01_JPRB, 0.12534E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.13061E+01_JPRB, 0.13385E+01_JPRB, 0.13788E+01_JPRB, 0.14280E+01_JPRB, 0.14664E+01_JPRB, 0.14889E+01_JPRB ,&
&  0.15036E+01_JPRB, 0.15125E+01_JPRB, 0.13751E+01_JPRB, 0.13520E+01_JPRB, 0.13911E+01_JPRB, 0.12651E+01_JPRB, 0.10777E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.91396E+00_JPRB, 0.85167E+00_JPRB, 0.97383E+00_JPRB, 0.91542E+00_JPRB, 0.84304E+00_JPRB, 0.74914E+00_JPRB ,&
&  0.63543E+00_JPRB, 0.57275E+00_JPRB, 0.57438E+00_JPRB, 0.61408E+00_JPRB, 0.60721E+00_JPRB, 0.60886E+00_JPRB, 0.62155E+00_JPRB ,&
&  0.64428E+00_JPRB, 0.66773E+00_JPRB, 0.70898E+00_JPRB, 0.79339E+00_JPRB, 0.89877E+00_JPRB, 0.89004E+00_JPRB, 0.10347E+01_JPRB ,&
&  0.81052E+00_JPRB, 0.68146E+00_JPRB, 0.73879E+00_JPRB, 0.83457E+00_JPRB, 0.86146E+00_JPRB, 0.85913E+00_JPRB, 0.96058E+00_JPRB ,&
&  0.11076E+01_JPRB, 0.12442E+01_JPRB, 0.12549E+01_JPRB, 0.10412E+01_JPRB, 0.92992E+00_JPRB, 0.88289E+00_JPRB, 0.87878E+00_JPRB ,&
&  0.89161E+00_JPRB, 0.91468E+00_JPRB, 0.95139E+00_JPRB, 0.99330E+00_JPRB, 0.10298E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.10414E+01_JPRB, 0.10707E+01_JPRB, 0.10960E+01_JPRB, 0.11189E+01_JPRB, 0.11409E+01_JPRB, 0.11652E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.12202E+01_JPRB, 0.12282E+01_JPRB, 0.12431E+01_JPRB, 0.12579E+01_JPRB, 0.12999E+01_JPRB, 0.13191E+01_JPRB ,&
&  0.13602E+01_JPRB, 0.13758E+01_JPRB, 0.13734E+01_JPRB, 0.13944E+01_JPRB, 0.13826E+01_JPRB, 0.13591E+01_JPRB, 0.13553E+01_JPRB ,&
&  0.13634E+01_JPRB, 0.13616E+01_JPRB, 0.13604E+01_JPRB, 0.13631E+01_JPRB, 0.13657E+01_JPRB, 0.13356E+01_JPRB, 0.14157E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.14322E+01_JPRB, 0.14826E+01_JPRB, 0.13200E+01_JPRB, 0.12513E+01_JPRB, 0.11482E+01_JPRB, 0.11379E+01_JPRB ,&
&  0.11520E+01_JPRB, 0.11465E+01_JPRB, 0.11559E+01_JPRB, 0.11815E+01_JPRB, 0.12133E+01_JPRB, 0.12481E+01_JPRB, 0.12601E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.12973E+01_JPRB, 0.13090E+01_JPRB, 0.13232E+01_JPRB, 0.13624E+01_JPRB, 0.13579E+01_JPRB, 0.13317E+01_JPRB ,&
&  0.13174E+01_JPRB, 0.13094E+01_JPRB, 0.10479E+01_JPRB, 0.10060E+01_JPRB, 0.10503E+01_JPRB, 0.99629E+00_JPRB, 0.88962E+00_JPRB ,&
&  0.89926E+00_JPRB, 0.91857E+00_JPRB, 0.93393E+00_JPRB, 0.91236E+00_JPRB, 0.90289E+00_JPRB, 0.87449E+00_JPRB, 0.82429E+00_JPRB ,&
&  0.71495E+00_JPRB, 0.61982E+00_JPRB, 0.59621E+00_JPRB, 0.62902E+00_JPRB, 0.64509E+00_JPRB, 0.64675E+00_JPRB, 0.61616E+00_JPRB ,&
&  0.60199E+00_JPRB, 0.60153E+00_JPRB, 0.62829E+00_JPRB, 0.70405E+00_JPRB, 0.78711E+00_JPRB, 0.14140E+01_JPRB, 0.17304E+01_JPRB ,&
&  0.81741E+00_JPRB, 0.74444E+00_JPRB, 0.58453E+00_JPRB, 0.73936E+00_JPRB, 0.73457E+00_JPRB, 0.66577E+00_JPRB, 0.69379E+00_JPRB ,&
&  0.79966E+00_JPRB, 0.85660E+00_JPRB, 0.87829E+00_JPRB, 0.81579E+00_JPRB, 0.83278E+00_JPRB, 0.81879E+00_JPRB, 0.82496E+00_JPRB ,&
&  0.84803E+00_JPRB, 0.88390E+00_JPRB, 0.92821E+00_JPRB, 0.97077E+00_JPRB, 0.10077E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.98022E+00_JPRB, 0.10039E+01_JPRB, 0.10212E+01_JPRB, 0.10397E+01_JPRB, 0.10640E+01_JPRB, 0.10894E+01_JPRB, 0.11057E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11502E+01_JPRB, 0.11586E+01_JPRB, 0.11781E+01_JPRB, 0.11853E+01_JPRB, 0.12409E+01_JPRB, 0.12625E+01_JPRB ,&
&  0.13086E+01_JPRB, 0.13351E+01_JPRB, 0.13442E+01_JPRB, 0.13471E+01_JPRB, 0.13603E+01_JPRB, 0.13571E+01_JPRB, 0.13524E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.14248E+01_JPRB, 0.14786E+01_JPRB, 0.15282E+01_JPRB, 0.15553E+01_JPRB, 0.16478E+01_JPRB, 0.17669E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.12960E+01_JPRB, 0.13138E+01_JPRB, 0.11285E+01_JPRB, 0.11396E+01_JPRB, 0.11488E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.11091E+01_JPRB, 0.11365E+01_JPRB, 0.11583E+01_JPRB, 0.11797E+01_JPRB, 0.12061E+01_JPRB, 0.12008E+01_JPRB ,&
&  0.12232E+01_JPRB, 0.12440E+01_JPRB, 0.12770E+01_JPRB, 0.13215E+01_JPRB, 0.12567E+01_JPRB, 0.13809E+01_JPRB, 0.13956E+01_JPRB ,&
&  0.13760E+01_JPRB, 0.12696E+01_JPRB, 0.99061E+00_JPRB, 0.92325E+00_JPRB, 0.92137E+00_JPRB, 0.85730E+00_JPRB, 0.79376E+00_JPRB ,&
&  0.85822E+00_JPRB, 0.88290E+00_JPRB, 0.72339E+00_JPRB, 0.83509E+00_JPRB, 0.90260E+00_JPRB, 0.93010E+00_JPRB, 0.88955E+00_JPRB ,&
&  0.83102E+00_JPRB, 0.73839E+00_JPRB, 0.71111E+00_JPRB, 0.72468E+00_JPRB, 0.73449E+00_JPRB, 0.72443E+00_JPRB, 0.74470E+00_JPRB ,&
&  0.73260E+00_JPRB, 0.69503E+00_JPRB, 0.69480E+00_JPRB, 0.67851E+00_JPRB, 0.92616E+00_JPRB, 0.11029E+01_JPRB, 0.17113E+01_JPRB ,&
&  0.86171E+00_JPRB, 0.84841E+00_JPRB, 0.81933E+00_JPRB, 0.12254E+01_JPRB, 0.80558E+00_JPRB, 0.56102E+00_JPRB, 0.57006E+00_JPRB ,&
&  0.63146E+00_JPRB, 0.66252E+00_JPRB, 0.67271E+00_JPRB, 0.69136E+00_JPRB, 0.73829E+00_JPRB, 0.76479E+00_JPRB, 0.78880E+00_JPRB ,&
&  0.79649E+00_JPRB, 0.83549E+00_JPRB, 0.88799E+00_JPRB, 0.92824E+00_JPRB, 0.94709E+00_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.83066E+00_JPRB, 0.84654E+00_JPRB, 0.86913E+00_JPRB, 0.89271E+00_JPRB, 0.90807E+00_JPRB, 0.93273E+00_JPRB, 0.95993E+00_JPRB ,&
&  0.99533E+00_JPRB, 0.10409E+01_JPRB, 0.10699E+01_JPRB, 0.10806E+01_JPRB, 0.10953E+01_JPRB, 0.11115E+01_JPRB, 0.11160E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11499E+01_JPRB, 0.12097E+01_JPRB, 0.12385E+01_JPRB, 0.12770E+01_JPRB, 0.13295E+01_JPRB, 0.13389E+01_JPRB ,&
&  0.13308E+01_JPRB, 0.13216E+01_JPRB, 0.13377E+01_JPRB, 0.13393E+01_JPRB, 0.14400E+01_JPRB, 0.14639E+01_JPRB, 0.13914E+01_JPRB ,&
&  0.12085E+01_JPRB, 0.11993E+01_JPRB, 0.13686E+01_JPRB, 0.11202E+01_JPRB, 0.11077E+01_JPRB, 0.10980E+01_JPRB, 0.99526E+00_JPRB ,&
&  0.97886E+00_JPRB, 0.10257E+01_JPRB, 0.10853E+01_JPRB, 0.11474E+01_JPRB, 0.12255E+01_JPRB, 0.12444E+01_JPRB, 0.12391E+01_JPRB ,&
&  0.12437E+01_JPRB, 0.12698E+01_JPRB, 0.12938E+01_JPRB, 0.12308E+01_JPRB, 0.11786E+01_JPRB, 0.12873E+01_JPRB, 0.13207E+01_JPRB ,&
&  0.12995E+01_JPRB, 0.12805E+01_JPRB, 0.10574E+01_JPRB, 0.97279E+00_JPRB, 0.97560E+00_JPRB, 0.85252E+00_JPRB, 0.79340E+00_JPRB ,&
&  0.84619E+00_JPRB, 0.76420E+00_JPRB, 0.50794E+00_JPRB, 0.68868E+00_JPRB, 0.85768E+00_JPRB, 0.94345E+00_JPRB, 0.98130E+00_JPRB ,&
&  0.94729E+00_JPRB, 0.85738E+00_JPRB, 0.83277E+00_JPRB, 0.87683E+00_JPRB, 0.91906E+00_JPRB, 0.88134E+00_JPRB, 0.88585E+00_JPRB ,&
&  0.81772E+00_JPRB, 0.71866E+00_JPRB, 0.65614E+00_JPRB, 0.71267E+00_JPRB, 0.81879E+00_JPRB, 0.10860E+01_JPRB, 0.12622E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.99225E+00_JPRB, 0.15844E+01_JPRB, 0.23067E+01_JPRB, 0.92268E+00_JPRB, 0.63849E+00_JPRB, 0.53071E+00_JPRB ,&
&  0.55268E+00_JPRB, 0.56899E+00_JPRB, 0.58120E+00_JPRB, 0.60630E+00_JPRB, 0.64318E+00_JPRB, 0.65927E+00_JPRB, 0.66527E+00_JPRB ,&
&  0.68627E+00_JPRB, 0.71947E+00_JPRB, 0.74437E+00_JPRB, 0.77576E+00_JPRB, 0.80397E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.65323E+00_JPRB, 0.65269E+00_JPRB, 0.65575E+00_JPRB, 0.66353E+00_JPRB, 0.66788E+00_JPRB, 0.67206E+00_JPRB, 0.69506E+00_JPRB ,&
&  0.73419E+00_JPRB, 0.79651E+00_JPRB, 0.88182E+00_JPRB, 0.93576E+00_JPRB, 0.91670E+00_JPRB, 0.84975E+00_JPRB, 0.81557E+00_JPRB ,&
&  0.80128E+00_JPRB, 0.79005E+00_JPRB, 0.81885E+00_JPRB, 0.87119E+00_JPRB, 0.93859E+00_JPRB, 0.96124E+00_JPRB, 0.10258E+01_JPRB ,&
&  0.10272E+01_JPRB, 0.10619E+01_JPRB, 0.11709E+01_JPRB, 0.12843E+01_JPRB, 0.19588E+01_JPRB, 0.18657E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.15400E+01_JPRB, 0.15872E+01_JPRB, 0.15293E+01_JPRB, 0.13638E+01_JPRB, 0.13565E+01_JPRB, 0.13223E+01_JPRB, 0.13631E+01_JPRB ,&
&  0.13449E+01_JPRB, 0.14330E+01_JPRB, 0.14978E+01_JPRB, 0.15735E+01_JPRB, 0.16236E+01_JPRB, 0.16242E+01_JPRB, 0.16503E+01_JPRB ,&
&  0.16589E+01_JPRB, 0.16626E+01_JPRB, 0.16648E+01_JPRB, 0.14933E+01_JPRB, 0.15747E+01_JPRB, 0.16282E+01_JPRB, 0.16360E+01_JPRB ,&
&  0.15917E+01_JPRB, 0.15681E+01_JPRB, 0.16525E+01_JPRB, 0.15532E+01_JPRB, 0.14304E+01_JPRB, 0.12582E+01_JPRB, 0.10913E+01_JPRB ,&
&  0.97313E+00_JPRB, 0.74081E+00_JPRB, 0.54934E+00_JPRB, 0.82252E+00_JPRB, 0.10639E+01_JPRB, 0.98705E+00_JPRB, 0.10988E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.10538E+01_JPRB, 0.10153E+01_JPRB, 0.10101E+01_JPRB, 0.98592E+00_JPRB, 0.10932E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.97943E+00_JPRB, 0.78169E+00_JPRB, 0.69143E+00_JPRB, 0.69710E+00_JPRB, 0.72969E+00_JPRB, 0.78046E+00_JPRB, 0.11115E+01_JPRB ,&
&  0.11414E+01_JPRB, 0.10008E+01_JPRB, 0.11999E+01_JPRB, 0.15363E+01_JPRB, 0.92631E+00_JPRB, 0.57948E+00_JPRB, 0.52742E+00_JPRB ,&
&  0.52002E+00_JPRB, 0.53167E+00_JPRB, 0.55516E+00_JPRB, 0.57055E+00_JPRB, 0.58135E+00_JPRB, 0.58899E+00_JPRB, 0.60288E+00_JPRB ,&
&  0.61283E+00_JPRB, 0.62539E+00_JPRB, 0.63146E+00_JPRB, 0.63946E+00_JPRB, 0.64835E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.71469E+00_JPRB, 0.71884E+00_JPRB, 0.70595E+00_JPRB, 0.73309E+00_JPRB, 0.78569E+00_JPRB, 0.89394E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.11237E+01_JPRB, 0.12728E+01_JPRB, 0.14683E+01_JPRB, 0.16358E+01_JPRB, 0.16898E+01_JPRB, 0.16605E+01_JPRB, 0.15083E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.10921E+01_JPRB, 0.96414E+00_JPRB, 0.91666E+00_JPRB, 0.96323E+00_JPRB, 0.99168E+00_JPRB, 0.10725E+01_JPRB ,&
&  0.12088E+01_JPRB, 0.14338E+01_JPRB, 0.16790E+01_JPRB, 0.22223E+01_JPRB, 0.24605E+01_JPRB, 0.18255E+01_JPRB, 0.17380E+01_JPRB ,&
&  0.20650E+01_JPRB, 0.21515E+01_JPRB, 0.17537E+01_JPRB, 0.17763E+01_JPRB, 0.19798E+01_JPRB, 0.21542E+01_JPRB, 0.23728E+01_JPRB ,&
&  0.24927E+01_JPRB, 0.26069E+01_JPRB, 0.26882E+01_JPRB, 0.27280E+01_JPRB, 0.27549E+01_JPRB, 0.27346E+01_JPRB, 0.26732E+01_JPRB ,&
&  0.25767E+01_JPRB, 0.24438E+01_JPRB, 0.24315E+01_JPRB, 0.25044E+01_JPRB, 0.25930E+01_JPRB, 0.25725E+01_JPRB, 0.24469E+01_JPRB ,&
&  0.23398E+01_JPRB, 0.23080E+01_JPRB, 0.24707E+01_JPRB, 0.24387E+01_JPRB, 0.21912E+01_JPRB, 0.20247E+01_JPRB, 0.17427E+01_JPRB ,&
&  0.14927E+01_JPRB, 0.12248E+01_JPRB, 0.10112E+01_JPRB, 0.15684E+01_JPRB, 0.18001E+01_JPRB, 0.15830E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.13308E+01_JPRB, 0.12496E+01_JPRB, 0.11727E+01_JPRB, 0.11327E+01_JPRB, 0.11495E+01_JPRB, 0.13909E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.11416E+01_JPRB, 0.87022E+00_JPRB, 0.80473E+00_JPRB, 0.88152E+00_JPRB, 0.95409E+00_JPRB, 0.98995E+00_JPRB, 0.10462E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.13541E+01_JPRB, 0.15574E+01_JPRB, 0.20247E+01_JPRB, 0.22315E+01_JPRB, 0.69206E+00_JPRB, 0.61987E+00_JPRB ,&
&  0.59944E+00_JPRB, 0.59716E+00_JPRB, 0.61625E+00_JPRB, 0.61518E+00_JPRB, 0.62379E+00_JPRB, 0.62086E+00_JPRB, 0.61089E+00_JPRB ,&
&  0.61258E+00_JPRB, 0.62979E+00_JPRB, 0.65084E+00_JPRB, 0.66449E+00_JPRB, 0.69410E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.10794E+01_JPRB, 0.11374E+01_JPRB, 0.11489E+01_JPRB, 0.12622E+01_JPRB, 0.13886E+01_JPRB, 0.14756E+01_JPRB, 0.15550E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.18252E+01_JPRB, 0.19442E+01_JPRB, 0.20183E+01_JPRB, 0.20914E+01_JPRB, 0.21934E+01_JPRB, 0.22185E+01_JPRB ,&
&  0.21872E+01_JPRB, 0.21561E+01_JPRB, 0.21390E+01_JPRB, 0.21221E+01_JPRB, 0.21286E+01_JPRB, 0.21523E+01_JPRB, 0.23482E+01_JPRB ,&
&  0.24371E+01_JPRB, 0.23731E+01_JPRB, 0.22794E+01_JPRB, 0.18846E+01_JPRB, 0.14570E+01_JPRB, 0.19043E+01_JPRB, 0.19543E+01_JPRB ,&
&  0.21507E+01_JPRB, 0.22417E+01_JPRB, 0.22321E+01_JPRB, 0.23932E+01_JPRB, 0.25544E+01_JPRB, 0.27342E+01_JPRB, 0.29374E+01_JPRB ,&
&  0.30901E+01_JPRB, 0.32522E+01_JPRB, 0.34147E+01_JPRB, 0.35409E+01_JPRB, 0.36616E+01_JPRB, 0.38065E+01_JPRB, 0.38273E+01_JPRB ,&
&  0.38040E+01_JPRB, 0.36790E+01_JPRB, 0.35870E+01_JPRB, 0.36226E+01_JPRB, 0.35038E+01_JPRB, 0.33607E+01_JPRB, 0.31199E+01_JPRB ,&
&  0.30281E+01_JPRB, 0.30569E+01_JPRB, 0.32165E+01_JPRB, 0.33041E+01_JPRB, 0.32029E+01_JPRB, 0.29687E+01_JPRB, 0.27017E+01_JPRB ,&
&  0.24536E+01_JPRB, 0.22214E+01_JPRB, 0.22764E+01_JPRB, 0.24575E+01_JPRB, 0.17796E+01_JPRB, 0.18052E+01_JPRB, 0.18689E+01_JPRB ,&
&  0.16286E+01_JPRB, 0.14472E+01_JPRB, 0.13306E+01_JPRB, 0.12144E+01_JPRB, 0.13046E+01_JPRB, 0.12886E+01_JPRB, 0.14386E+01_JPRB ,&
&  0.13017E+01_JPRB, 0.10081E+01_JPRB, 0.97936E+00_JPRB, 0.11470E+01_JPRB, 0.12274E+01_JPRB, 0.13222E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.13901E+01_JPRB, 0.15881E+01_JPRB, 0.18583E+01_JPRB, 0.14129E+01_JPRB, 0.78256E+00_JPRB, 0.65048E+00_JPRB ,&
&  0.62278E+00_JPRB, 0.62916E+00_JPRB, 0.64468E+00_JPRB, 0.65405E+00_JPRB, 0.68863E+00_JPRB, 0.73012E+00_JPRB, 0.75618E+00_JPRB ,&
&  0.78108E+00_JPRB, 0.79925E+00_JPRB, 0.84154E+00_JPRB, 0.90740E+00_JPRB, 0.10046E+01_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.15128E+01_JPRB, 0.16203E+01_JPRB, 0.17261E+01_JPRB, 0.17282E+01_JPRB, 0.17344E+01_JPRB, 0.17363E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.18611E+01_JPRB, 0.19594E+01_JPRB, 0.20756E+01_JPRB, 0.21679E+01_JPRB, 0.22364E+01_JPRB, 0.23539E+01_JPRB, 0.24969E+01_JPRB ,&
&  0.25862E+01_JPRB, 0.26564E+01_JPRB, 0.27328E+01_JPRB, 0.28455E+01_JPRB, 0.29353E+01_JPRB, 0.30703E+01_JPRB, 0.31364E+01_JPRB ,&
&  0.24469E+01_JPRB, 0.17025E+01_JPRB, 0.19702E+01_JPRB, 0.22294E+01_JPRB, 0.22989E+01_JPRB, 0.26565E+01_JPRB, 0.28454E+01_JPRB ,&
&  0.27291E+01_JPRB, 0.26713E+01_JPRB, 0.26899E+01_JPRB, 0.26922E+01_JPRB, 0.26989E+01_JPRB, 0.29184E+01_JPRB, 0.31373E+01_JPRB ,&
&  0.33299E+01_JPRB, 0.35017E+01_JPRB, 0.36725E+01_JPRB, 0.38488E+01_JPRB, 0.39919E+01_JPRB, 0.41268E+01_JPRB, 0.42389E+01_JPRB ,&
&  0.42963E+01_JPRB, 0.42089E+01_JPRB, 0.41420E+01_JPRB, 0.40659E+01_JPRB, 0.39435E+01_JPRB, 0.37955E+01_JPRB, 0.36381E+01_JPRB ,&
&  0.36897E+01_JPRB, 0.38833E+01_JPRB, 0.41833E+01_JPRB, 0.43969E+01_JPRB, 0.44858E+01_JPRB, 0.42625E+01_JPRB, 0.40060E+01_JPRB ,&
&  0.36125E+01_JPRB, 0.31849E+01_JPRB, 0.33088E+01_JPRB, 0.33954E+01_JPRB, 0.26072E+01_JPRB, 0.23616E+01_JPRB, 0.21309E+01_JPRB ,&
&  0.21010E+01_JPRB, 0.18161E+01_JPRB, 0.15925E+01_JPRB, 0.15719E+01_JPRB, 0.18120E+01_JPRB, 0.17576E+01_JPRB, 0.18110E+01_JPRB ,&
&  0.15859E+01_JPRB, 0.12186E+01_JPRB, 0.11039E+01_JPRB, 0.12401E+01_JPRB, 0.11722E+01_JPRB, 0.13536E+01_JPRB, 0.14364E+01_JPRB ,&
&  0.15649E+01_JPRB, 0.14418E+01_JPRB, 0.16456E+01_JPRB, 0.16355E+01_JPRB, 0.10694E+01_JPRB, 0.77373E+00_JPRB, 0.68693E+00_JPRB ,&
&  0.65351E+00_JPRB, 0.65500E+00_JPRB, 0.69784E+00_JPRB, 0.70604E+00_JPRB, 0.76256E+00_JPRB, 0.81712E+00_JPRB, 0.92648E+00_JPRB ,&
&  0.10198E+01_JPRB, 0.11061E+01_JPRB, 0.11768E+01_JPRB, 0.12713E+01_JPRB, 0.13786E+01_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.15219E+01_JPRB, 0.16696E+01_JPRB, 0.18196E+01_JPRB, 0.19241E+01_JPRB, 0.18989E+01_JPRB, 0.18730E+01_JPRB, 0.18920E+01_JPRB ,&
&  0.19380E+01_JPRB, 0.20296E+01_JPRB, 0.21473E+01_JPRB, 0.22517E+01_JPRB, 0.23316E+01_JPRB, 0.24509E+01_JPRB, 0.26377E+01_JPRB ,&
&  0.28270E+01_JPRB, 0.29969E+01_JPRB, 0.31525E+01_JPRB, 0.33403E+01_JPRB, 0.35149E+01_JPRB, 0.37757E+01_JPRB, 0.34829E+01_JPRB ,&
&  0.24580E+01_JPRB, 0.21774E+01_JPRB, 0.25666E+01_JPRB, 0.25920E+01_JPRB, 0.26308E+01_JPRB, 0.28962E+01_JPRB, 0.30582E+01_JPRB ,&
&  0.30374E+01_JPRB, 0.29530E+01_JPRB, 0.28896E+01_JPRB, 0.28282E+01_JPRB, 0.28281E+01_JPRB, 0.30228E+01_JPRB, 0.32384E+01_JPRB ,&
&  0.34271E+01_JPRB, 0.35595E+01_JPRB, 0.37170E+01_JPRB, 0.39287E+01_JPRB, 0.41733E+01_JPRB, 0.43826E+01_JPRB, 0.45918E+01_JPRB ,&
&  0.45682E+01_JPRB, 0.45811E+01_JPRB, 0.44795E+01_JPRB, 0.42622E+01_JPRB, 0.42486E+01_JPRB, 0.43198E+01_JPRB, 0.44134E+01_JPRB ,&
&  0.46438E+01_JPRB, 0.50721E+01_JPRB, 0.54827E+01_JPRB, 0.56334E+01_JPRB, 0.54068E+01_JPRB, 0.54647E+01_JPRB, 0.50824E+01_JPRB ,&
&  0.50255E+01_JPRB, 0.45023E+01_JPRB, 0.45121E+01_JPRB, 0.38819E+01_JPRB, 0.36339E+01_JPRB, 0.33502E+01_JPRB, 0.31836E+01_JPRB ,&
&  0.29436E+01_JPRB, 0.26643E+01_JPRB, 0.23400E+01_JPRB, 0.22778E+01_JPRB, 0.25459E+01_JPRB, 0.26775E+01_JPRB, 0.26819E+01_JPRB ,&
&  0.21679E+01_JPRB, 0.16733E+01_JPRB, 0.12769E+01_JPRB, 0.12897E+01_JPRB, 0.11135E+01_JPRB, 0.11935E+01_JPRB, 0.16737E+01_JPRB ,&
&  0.22747E+01_JPRB, 0.19865E+01_JPRB, 0.20976E+01_JPRB, 0.18648E+01_JPRB, 0.11797E+01_JPRB, 0.82625E+00_JPRB, 0.71518E+00_JPRB ,&
&  0.68895E+00_JPRB, 0.73930E+00_JPRB, 0.77430E+00_JPRB, 0.79092E+00_JPRB, 0.85807E+00_JPRB, 0.91972E+00_JPRB, 0.10027E+01_JPRB ,&
&  0.10437E+01_JPRB, 0.11107E+01_JPRB, 0.11840E+01_JPRB, 0.12571E+01_JPRB, 0.13737E+01_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.12624E+01_JPRB, 0.13297E+01_JPRB, 0.13874E+01_JPRB, 0.14246E+01_JPRB, 0.14746E+01_JPRB, 0.14839E+01_JPRB, 0.15496E+01_JPRB ,&
&  0.17057E+01_JPRB, 0.18957E+01_JPRB, 0.20769E+01_JPRB, 0.21881E+01_JPRB, 0.22314E+01_JPRB, 0.23175E+01_JPRB, 0.25246E+01_JPRB ,&
&  0.27438E+01_JPRB, 0.29961E+01_JPRB, 0.32206E+01_JPRB, 0.34577E+01_JPRB, 0.35367E+01_JPRB, 0.36063E+01_JPRB, 0.28978E+01_JPRB ,&
&  0.19280E+01_JPRB, 0.28291E+01_JPRB, 0.23600E+01_JPRB, 0.26034E+01_JPRB, 0.28170E+01_JPRB, 0.30745E+01_JPRB, 0.30829E+01_JPRB ,&
&  0.30263E+01_JPRB, 0.29673E+01_JPRB, 0.28729E+01_JPRB, 0.27795E+01_JPRB, 0.27393E+01_JPRB, 0.28680E+01_JPRB, 0.30650E+01_JPRB ,&
&  0.32532E+01_JPRB, 0.34128E+01_JPRB, 0.36104E+01_JPRB, 0.38981E+01_JPRB, 0.42175E+01_JPRB, 0.45511E+01_JPRB, 0.48110E+01_JPRB ,&
&  0.48583E+01_JPRB, 0.49039E+01_JPRB, 0.47193E+01_JPRB, 0.44562E+01_JPRB, 0.44583E+01_JPRB, 0.48226E+01_JPRB, 0.53139E+01_JPRB ,&
&  0.56118E+01_JPRB, 0.58926E+01_JPRB, 0.66921E+01_JPRB, 0.71368E+01_JPRB, 0.70072E+01_JPRB, 0.68631E+01_JPRB, 0.60598E+01_JPRB ,&
&  0.64522E+01_JPRB, 0.59151E+01_JPRB, 0.57247E+01_JPRB, 0.48864E+01_JPRB, 0.45322E+01_JPRB, 0.45162E+01_JPRB, 0.45017E+01_JPRB ,&
&  0.41720E+01_JPRB, 0.40389E+01_JPRB, 0.35943E+01_JPRB, 0.33671E+01_JPRB, 0.34858E+01_JPRB, 0.37286E+01_JPRB, 0.35470E+01_JPRB ,&
&  0.30687E+01_JPRB, 0.21093E+01_JPRB, 0.16088E+01_JPRB, 0.14352E+01_JPRB, 0.12700E+01_JPRB, 0.13546E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.28702E+01_JPRB, 0.28712E+01_JPRB, 0.29979E+01_JPRB, 0.26804E+01_JPRB, 0.16437E+01_JPRB, 0.10368E+01_JPRB, 0.82211E+00_JPRB ,&
&  0.80402E+00_JPRB, 0.85883E+00_JPRB, 0.88170E+00_JPRB, 0.89580E+00_JPRB, 0.94858E+00_JPRB, 0.93125E+00_JPRB, 0.99521E+00_JPRB ,&
&  0.10238E+01_JPRB, 0.10737E+01_JPRB, 0.11121E+01_JPRB, 0.11276E+01_JPRB, 0.11812E+01_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.10413E+01_JPRB, 0.10770E+01_JPRB, 0.11353E+01_JPRB, 0.11729E+01_JPRB, 0.11918E+01_JPRB, 0.12290E+01_JPRB, 0.12702E+01_JPRB ,&
&  0.13842E+01_JPRB, 0.15549E+01_JPRB, 0.17518E+01_JPRB, 0.19487E+01_JPRB, 0.20009E+01_JPRB, 0.20131E+01_JPRB, 0.21660E+01_JPRB ,&
&  0.24544E+01_JPRB, 0.28805E+01_JPRB, 0.33451E+01_JPRB, 0.37650E+01_JPRB, 0.38646E+01_JPRB, 0.37622E+01_JPRB, 0.27112E+01_JPRB ,&
&  0.21971E+01_JPRB, 0.25897E+01_JPRB, 0.23887E+01_JPRB, 0.25661E+01_JPRB, 0.28530E+01_JPRB, 0.30760E+01_JPRB, 0.30049E+01_JPRB ,&
&  0.28958E+01_JPRB, 0.27845E+01_JPRB, 0.26295E+01_JPRB, 0.25655E+01_JPRB, 0.25579E+01_JPRB, 0.25799E+01_JPRB, 0.26607E+01_JPRB ,&
&  0.28186E+01_JPRB, 0.29911E+01_JPRB, 0.32630E+01_JPRB, 0.36503E+01_JPRB, 0.40578E+01_JPRB, 0.44540E+01_JPRB, 0.48261E+01_JPRB ,&
&  0.50317E+01_JPRB, 0.50513E+01_JPRB, 0.50868E+01_JPRB, 0.45813E+01_JPRB, 0.46606E+01_JPRB, 0.52183E+01_JPRB, 0.60555E+01_JPRB ,&
&  0.65197E+01_JPRB, 0.68171E+01_JPRB, 0.78680E+01_JPRB, 0.88228E+01_JPRB, 0.88853E+01_JPRB, 0.81958E+01_JPRB, 0.73734E+01_JPRB ,&
&  0.75063E+01_JPRB, 0.72391E+01_JPRB, 0.66413E+01_JPRB, 0.61007E+01_JPRB, 0.58158E+01_JPRB, 0.59363E+01_JPRB, 0.61237E+01_JPRB ,&
&  0.55189E+01_JPRB, 0.50920E+01_JPRB, 0.46765E+01_JPRB, 0.44362E+01_JPRB, 0.42464E+01_JPRB, 0.42018E+01_JPRB, 0.44830E+01_JPRB ,&
&  0.41577E+01_JPRB, 0.31691E+01_JPRB, 0.24485E+01_JPRB, 0.16587E+01_JPRB, 0.14363E+01_JPRB, 0.16386E+01_JPRB, 0.23144E+01_JPRB ,&
&  0.35271E+01_JPRB, 0.37375E+01_JPRB, 0.35174E+01_JPRB, 0.38563E+01_JPRB, 0.26815E+01_JPRB, 0.15315E+01_JPRB, 0.95749E+00_JPRB ,&
&  0.87371E+00_JPRB, 0.92299E+00_JPRB, 0.94030E+00_JPRB, 0.90784E+00_JPRB, 0.93283E+00_JPRB, 0.96736E+00_JPRB, 0.94643E+00_JPRB ,&
&  0.98705E+00_JPRB, 0.10341E+01_JPRB, 0.10138E+01_JPRB, 0.98906E+00_JPRB, 0.10107E+01_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.92270E+00_JPRB, 0.96513E+00_JPRB, 0.97390E+00_JPRB, 0.97172E+00_JPRB, 0.99236E+00_JPRB, 0.10249E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.12533E+01_JPRB, 0.14040E+01_JPRB, 0.16057E+01_JPRB, 0.17198E+01_JPRB, 0.17174E+01_JPRB, 0.17507E+01_JPRB ,&
&  0.19446E+01_JPRB, 0.23638E+01_JPRB, 0.30203E+01_JPRB, 0.36552E+01_JPRB, 0.37500E+01_JPRB, 0.34240E+01_JPRB, 0.31894E+01_JPRB ,&
&  0.26822E+01_JPRB, 0.27801E+01_JPRB, 0.25646E+01_JPRB, 0.30326E+01_JPRB, 0.34951E+01_JPRB, 0.38688E+01_JPRB, 0.28322E+01_JPRB ,&
&  0.26187E+01_JPRB, 0.25491E+01_JPRB, 0.24191E+01_JPRB, 0.23922E+01_JPRB, 0.22869E+01_JPRB, 0.22447E+01_JPRB, 0.22192E+01_JPRB ,&
&  0.22501E+01_JPRB, 0.24667E+01_JPRB, 0.27718E+01_JPRB, 0.32100E+01_JPRB, 0.37742E+01_JPRB, 0.42486E+01_JPRB, 0.46282E+01_JPRB ,&
&  0.48675E+01_JPRB, 0.51495E+01_JPRB, 0.53354E+01_JPRB, 0.49238E+01_JPRB, 0.47790E+01_JPRB, 0.54482E+01_JPRB, 0.66147E+01_JPRB ,&
&  0.79889E+01_JPRB, 0.90371E+01_JPRB, 0.94630E+01_JPRB, 0.94987E+01_JPRB, 0.96048E+01_JPRB, 0.93543E+01_JPRB, 0.83173E+01_JPRB ,&
&  0.80642E+01_JPRB, 0.78012E+01_JPRB, 0.70130E+01_JPRB, 0.71334E+01_JPRB, 0.73260E+01_JPRB, 0.76461E+01_JPRB, 0.66336E+01_JPRB ,&
&  0.57283E+01_JPRB, 0.51444E+01_JPRB, 0.48479E+01_JPRB, 0.50771E+01_JPRB, 0.51570E+01_JPRB, 0.49112E+01_JPRB, 0.53125E+01_JPRB ,&
&  0.42219E+01_JPRB, 0.16237E+01_JPRB, 0.11076E+01_JPRB, 0.13614E+01_JPRB, 0.93785E+00_JPRB, 0.11765E+01_JPRB, 0.25574E+01_JPRB ,&
&  0.41479E+01_JPRB, 0.54519E+01_JPRB, 0.53779E+01_JPRB, 0.44426E+01_JPRB, 0.40188E+01_JPRB, 0.23454E+01_JPRB, 0.12300E+01_JPRB ,&
&  0.99759E+00_JPRB, 0.10747E+01_JPRB, 0.10823E+01_JPRB, 0.99500E+00_JPRB, 0.96610E+00_JPRB, 0.10048E+01_JPRB, 0.90951E+00_JPRB ,&
&  0.93357E+00_JPRB, 0.95137E+00_JPRB, 0.96167E+00_JPRB, 0.93808E+00_JPRB, 0.90441E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.86862E+00_JPRB, 0.90484E+00_JPRB, 0.93623E+00_JPRB, 0.88593E+00_JPRB, 0.87625E+00_JPRB, 0.90201E+00_JPRB, 0.90850E+00_JPRB ,&
&  0.98004E+00_JPRB, 0.10730E+01_JPRB, 0.11658E+01_JPRB, 0.13101E+01_JPRB, 0.14464E+01_JPRB, 0.14770E+01_JPRB, 0.14879E+01_JPRB ,&
&  0.15791E+01_JPRB, 0.18708E+01_JPRB, 0.24691E+01_JPRB, 0.33166E+01_JPRB, 0.37792E+01_JPRB, 0.36319E+01_JPRB, 0.37401E+01_JPRB ,&
&  0.29818E+01_JPRB, 0.32290E+01_JPRB, 0.40662E+01_JPRB, 0.51927E+01_JPRB, 0.55232E+01_JPRB, 0.50796E+01_JPRB, 0.32563E+01_JPRB ,&
&  0.26112E+01_JPRB, 0.25366E+01_JPRB, 0.23269E+01_JPRB, 0.23613E+01_JPRB, 0.21054E+01_JPRB, 0.20442E+01_JPRB, 0.19853E+01_JPRB ,&
&  0.20447E+01_JPRB, 0.22080E+01_JPRB, 0.23914E+01_JPRB, 0.27802E+01_JPRB, 0.34285E+01_JPRB, 0.40728E+01_JPRB, 0.46903E+01_JPRB ,&
&  0.48459E+01_JPRB, 0.51301E+01_JPRB, 0.53427E+01_JPRB, 0.57101E+01_JPRB, 0.44233E+01_JPRB, 0.59600E+01_JPRB, 0.70279E+01_JPRB ,&
&  0.87768E+01_JPRB, 0.99322E+01_JPRB, 0.97669E+01_JPRB, 0.10785E+02_JPRB, 0.11339E+02_JPRB, 0.10402E+02_JPRB, 0.88574E+01_JPRB ,&
&  0.83708E+01_JPRB, 0.75446E+01_JPRB, 0.73275E+01_JPRB, 0.75796E+01_JPRB, 0.81730E+01_JPRB, 0.94483E+01_JPRB, 0.63357E+01_JPRB ,&
&  0.57054E+01_JPRB, 0.62044E+01_JPRB, 0.56385E+01_JPRB, 0.49198E+01_JPRB, 0.55526E+01_JPRB, 0.57595E+01_JPRB, 0.43150E+01_JPRB ,&
&  0.17291E+01_JPRB, 0.90854E+00_JPRB, 0.77360E+00_JPRB, 0.77242E+00_JPRB, 0.70892E+00_JPRB, 0.12521E+01_JPRB, 0.53811E+01_JPRB ,&
&  0.42313E+01_JPRB, 0.64782E+01_JPRB, 0.66953E+01_JPRB, 0.58021E+01_JPRB, 0.51952E+01_JPRB, 0.34915E+01_JPRB, 0.23229E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.11646E+01_JPRB, 0.12873E+01_JPRB, 0.12649E+01_JPRB, 0.12521E+01_JPRB, 0.11709E+01_JPRB, 0.10361E+01_JPRB ,&
&  0.10274E+01_JPRB, 0.10447E+01_JPRB, 0.98313E+00_JPRB, 0.91776E+00_JPRB, 0.87670E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.87353E+00_JPRB, 0.87089E+00_JPRB, 0.87900E+00_JPRB, 0.86162E+00_JPRB, 0.81395E+00_JPRB, 0.82049E+00_JPRB, 0.84473E+00_JPRB ,&
&  0.88914E+00_JPRB, 0.94879E+00_JPRB, 0.10194E+01_JPRB, 0.11097E+01_JPRB, 0.12219E+01_JPRB, 0.12931E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.13954E+01_JPRB, 0.16255E+01_JPRB, 0.20913E+01_JPRB, 0.26932E+01_JPRB, 0.34396E+01_JPRB, 0.33507E+01_JPRB, 0.39511E+01_JPRB ,&
&  0.33164E+01_JPRB, 0.34993E+01_JPRB, 0.48939E+01_JPRB, 0.68632E+01_JPRB, 0.90805E+01_JPRB, 0.85303E+01_JPRB, 0.61503E+01_JPRB ,&
&  0.41653E+01_JPRB, 0.33571E+01_JPRB, 0.29894E+01_JPRB, 0.25646E+01_JPRB, 0.22318E+01_JPRB, 0.21003E+01_JPRB, 0.21021E+01_JPRB ,&
&  0.20378E+01_JPRB, 0.20662E+01_JPRB, 0.21178E+01_JPRB, 0.23990E+01_JPRB, 0.29757E+01_JPRB, 0.37083E+01_JPRB, 0.46064E+01_JPRB ,&
&  0.52119E+01_JPRB, 0.56135E+01_JPRB, 0.56598E+01_JPRB, 0.58657E+01_JPRB, 0.56708E+01_JPRB, 0.57216E+01_JPRB, 0.70009E+01_JPRB ,&
&  0.93538E+01_JPRB, 0.11540E+02_JPRB, 0.11415E+02_JPRB, 0.13075E+02_JPRB, 0.12260E+02_JPRB, 0.11277E+02_JPRB, 0.99036E+01_JPRB ,&
&  0.85587E+01_JPRB, 0.86270E+01_JPRB, 0.71339E+01_JPRB, 0.73568E+01_JPRB, 0.83683E+01_JPRB, 0.75219E+01_JPRB, 0.67185E+01_JPRB ,&
&  0.73971E+01_JPRB, 0.62099E+01_JPRB, 0.54365E+01_JPRB, 0.46242E+01_JPRB, 0.59279E+01_JPRB, 0.52585E+01_JPRB, 0.24136E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.10319E+01_JPRB, 0.71186E+00_JPRB, 0.73269E+00_JPRB, 0.79715E+00_JPRB, 0.12317E+01_JPRB, 0.34987E+01_JPRB ,&
&  0.42737E+01_JPRB, 0.66682E+01_JPRB, 0.73232E+01_JPRB, 0.64771E+01_JPRB, 0.55153E+01_JPRB, 0.47120E+01_JPRB, 0.38759E+01_JPRB ,&
&  0.24083E+01_JPRB, 0.16386E+01_JPRB, 0.14319E+01_JPRB, 0.15086E+01_JPRB, 0.15246E+01_JPRB, 0.14215E+01_JPRB, 0.12864E+01_JPRB ,&
&  0.12061E+01_JPRB, 0.11901E+01_JPRB, 0.11452E+01_JPRB, 0.10881E+01_JPRB, 0.97886E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.10173E+01_JPRB, 0.96151E+00_JPRB, 0.94932E+00_JPRB, 0.93518E+00_JPRB, 0.88671E+00_JPRB, 0.85349E+00_JPRB, 0.83267E+00_JPRB ,&
&  0.84434E+00_JPRB, 0.88403E+00_JPRB, 0.93506E+00_JPRB, 0.99853E+00_JPRB, 0.10692E+01_JPRB, 0.11394E+01_JPRB, 0.11855E+01_JPRB ,&
&  0.12790E+01_JPRB, 0.15147E+01_JPRB, 0.18194E+01_JPRB, 0.21389E+01_JPRB, 0.27130E+01_JPRB, 0.28945E+01_JPRB, 0.36537E+01_JPRB ,&
&  0.36844E+01_JPRB, 0.37738E+01_JPRB, 0.52246E+01_JPRB, 0.79459E+01_JPRB, 0.12052E+02_JPRB, 0.13139E+02_JPRB, 0.11834E+02_JPRB ,&
&  0.91198E+01_JPRB, 0.61638E+01_JPRB, 0.43677E+01_JPRB, 0.32449E+01_JPRB, 0.25559E+01_JPRB, 0.21809E+01_JPRB, 0.21370E+01_JPRB ,&
&  0.20281E+01_JPRB, 0.20005E+01_JPRB, 0.20472E+01_JPRB, 0.20893E+01_JPRB, 0.24554E+01_JPRB, 0.31630E+01_JPRB, 0.42932E+01_JPRB ,&
&  0.53562E+01_JPRB, 0.61583E+01_JPRB, 0.64480E+01_JPRB, 0.58804E+01_JPRB, 0.71247E+01_JPRB, 0.79036E+01_JPRB, 0.78039E+01_JPRB ,&
&  0.85167E+01_JPRB, 0.98780E+01_JPRB, 0.13594E+02_JPRB, 0.13100E+02_JPRB, 0.12329E+02_JPRB, 0.12538E+02_JPRB, 0.10688E+02_JPRB ,&
&  0.93981E+01_JPRB, 0.88309E+01_JPRB, 0.75946E+01_JPRB, 0.78239E+01_JPRB, 0.74926E+01_JPRB, 0.66284E+01_JPRB, 0.73471E+01_JPRB ,&
&  0.78563E+01_JPRB, 0.70550E+01_JPRB, 0.56874E+01_JPRB, 0.46178E+01_JPRB, 0.44367E+01_JPRB, 0.31016E+01_JPRB, 0.18751E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.11976E+01_JPRB, 0.84778E+00_JPRB, 0.10545E+01_JPRB, 0.96031E+00_JPRB, 0.13964E+01_JPRB, 0.28203E+01_JPRB ,&
&  0.49457E+01_JPRB, 0.61944E+01_JPRB, 0.73466E+01_JPRB, 0.60909E+01_JPRB, 0.53663E+01_JPRB, 0.54647E+01_JPRB, 0.65310E+01_JPRB ,&
&  0.38193E+01_JPRB, 0.31944E+01_JPRB, 0.23700E+01_JPRB, 0.18826E+01_JPRB, 0.16731E+01_JPRB, 0.15568E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.13850E+01_JPRB, 0.13092E+01_JPRB, 0.12803E+01_JPRB, 0.12277E+01_JPRB, 0.11222E+01_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.13084E+01_JPRB, 0.12283E+01_JPRB, 0.11670E+01_JPRB, 0.11120E+01_JPRB, 0.10497E+01_JPRB, 0.98527E+00_JPRB, 0.92560E+00_JPRB ,&
&  0.89548E+00_JPRB, 0.90183E+00_JPRB, 0.93454E+00_JPRB, 0.98812E+00_JPRB, 0.10339E+01_JPRB, 0.10818E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.12434E+01_JPRB, 0.15462E+01_JPRB, 0.15109E+01_JPRB, 0.16564E+01_JPRB, 0.21233E+01_JPRB, 0.28176E+01_JPRB, 0.32468E+01_JPRB ,&
&  0.37818E+01_JPRB, 0.38047E+01_JPRB, 0.48351E+01_JPRB, 0.73505E+01_JPRB, 0.14792E+02_JPRB, 0.17149E+02_JPRB, 0.16107E+02_JPRB ,&
&  0.15443E+02_JPRB, 0.11300E+02_JPRB, 0.71948E+01_JPRB, 0.49492E+01_JPRB, 0.35260E+01_JPRB, 0.25429E+01_JPRB, 0.20870E+01_JPRB ,&
&  0.19213E+01_JPRB, 0.18750E+01_JPRB, 0.18782E+01_JPRB, 0.19414E+01_JPRB, 0.21329E+01_JPRB, 0.26754E+01_JPRB, 0.38062E+01_JPRB ,&
&  0.50745E+01_JPRB, 0.63282E+01_JPRB, 0.71766E+01_JPRB, 0.71322E+01_JPRB, 0.65034E+01_JPRB, 0.79200E+01_JPRB, 0.11718E+02_JPRB ,&
&  0.11605E+02_JPRB, 0.12917E+02_JPRB, 0.13692E+02_JPRB, 0.14190E+02_JPRB, 0.13888E+02_JPRB, 0.12604E+02_JPRB, 0.11470E+02_JPRB ,&
&  0.80454E+01_JPRB, 0.69257E+01_JPRB, 0.74846E+01_JPRB, 0.78172E+01_JPRB, 0.70407E+01_JPRB, 0.80728E+01_JPRB, 0.10790E+02_JPRB ,&
&  0.81336E+01_JPRB, 0.81615E+01_JPRB, 0.81679E+01_JPRB, 0.80202E+01_JPRB, 0.44255E+01_JPRB, 0.28172E+01_JPRB, 0.42160E+01_JPRB ,&
&  0.40787E+01_JPRB, 0.21602E+01_JPRB, 0.15939E+01_JPRB, 0.22383E+01_JPRB, 0.14948E+01_JPRB, 0.19190E+01_JPRB, 0.32736E+01_JPRB ,&
&  0.39736E+01_JPRB, 0.46783E+01_JPRB, 0.77212E+01_JPRB, 0.65266E+01_JPRB, 0.59994E+01_JPRB, 0.56194E+01_JPRB, 0.57399E+01_JPRB ,&
&  0.56867E+01_JPRB, 0.42054E+01_JPRB, 0.33901E+01_JPRB, 0.25427E+01_JPRB, 0.20293E+01_JPRB, 0.18727E+01_JPRB, 0.17847E+01_JPRB ,&
&  0.17279E+01_JPRB, 0.16467E+01_JPRB, 0.15787E+01_JPRB, 0.14697E+01_JPRB, 0.13936E+01_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.15417E+01_JPRB, 0.14649E+01_JPRB, 0.13911E+01_JPRB, 0.13126E+01_JPRB, 0.12369E+01_JPRB, 0.11485E+01_JPRB, 0.10572E+01_JPRB ,&
&  0.99333E+00_JPRB, 0.97435E+00_JPRB, 0.99940E+00_JPRB, 0.10313E+01_JPRB, 0.10490E+01_JPRB, 0.10758E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.12166E+01_JPRB, 0.14136E+01_JPRB, 0.14220E+01_JPRB, 0.15340E+01_JPRB, 0.20386E+01_JPRB, 0.28412E+01_JPRB, 0.34044E+01_JPRB ,&
&  0.41169E+01_JPRB, 0.39176E+01_JPRB, 0.45330E+01_JPRB, 0.69926E+01_JPRB, 0.12216E+02_JPRB, 0.15776E+02_JPRB, 0.15393E+02_JPRB ,&
&  0.15010E+02_JPRB, 0.12434E+02_JPRB, 0.94130E+01_JPRB, 0.70408E+01_JPRB, 0.50725E+01_JPRB, 0.36474E+01_JPRB, 0.26513E+01_JPRB ,&
&  0.21222E+01_JPRB, 0.18872E+01_JPRB, 0.18325E+01_JPRB, 0.18445E+01_JPRB, 0.19790E+01_JPRB, 0.22052E+01_JPRB, 0.30584E+01_JPRB ,&
&  0.41714E+01_JPRB, 0.57298E+01_JPRB, 0.75696E+01_JPRB, 0.86137E+01_JPRB, 0.79427E+01_JPRB, 0.96377E+01_JPRB, 0.12569E+02_JPRB ,&
&  0.13013E+02_JPRB, 0.13697E+02_JPRB, 0.15160E+02_JPRB, 0.15105E+02_JPRB, 0.16119E+02_JPRB, 0.14407E+02_JPRB, 0.13734E+02_JPRB ,&
&  0.10035E+02_JPRB, 0.81692E+01_JPRB, 0.73308E+01_JPRB, 0.72082E+01_JPRB, 0.88854E+01_JPRB, 0.11742E+02_JPRB, 0.10975E+02_JPRB ,&
&  0.96624E+01_JPRB, 0.91389E+01_JPRB, 0.83585E+01_JPRB, 0.67218E+01_JPRB, 0.65008E+01_JPRB, 0.51316E+01_JPRB, 0.61178E+01_JPRB ,&
&  0.59488E+01_JPRB, 0.53833E+01_JPRB, 0.44086E+01_JPRB, 0.33273E+01_JPRB, 0.25834E+01_JPRB, 0.31957E+01_JPRB, 0.34939E+01_JPRB ,&
&  0.37053E+01_JPRB, 0.39486E+01_JPRB, 0.60316E+01_JPRB, 0.64824E+01_JPRB, 0.64652E+01_JPRB, 0.51005E+01_JPRB, 0.52346E+01_JPRB ,&
&  0.48133E+01_JPRB, 0.48530E+01_JPRB, 0.43371E+01_JPRB, 0.33975E+01_JPRB, 0.27791E+01_JPRB, 0.23824E+01_JPRB, 0.21763E+01_JPRB ,&
&  0.20926E+01_JPRB, 0.20137E+01_JPRB, 0.18918E+01_JPRB, 0.17560E+01_JPRB, 0.16475E+01_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.16300E+01_JPRB, 0.15705E+01_JPRB, 0.14910E+01_JPRB, 0.13986E+01_JPRB, 0.13076E+01_JPRB, 0.12276E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.11026E+01_JPRB, 0.10657E+01_JPRB, 0.10685E+01_JPRB, 0.10905E+01_JPRB, 0.11084E+01_JPRB, 0.10916E+01_JPRB, 0.10867E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.13701E+01_JPRB, 0.13587E+01_JPRB, 0.15454E+01_JPRB, 0.20435E+01_JPRB, 0.28379E+01_JPRB, 0.38925E+01_JPRB ,&
&  0.43933E+01_JPRB, 0.43900E+01_JPRB, 0.48491E+01_JPRB, 0.66821E+01_JPRB, 0.91618E+01_JPRB, 0.10770E+02_JPRB, 0.11061E+02_JPRB ,&
&  0.95323E+01_JPRB, 0.84644E+01_JPRB, 0.72483E+01_JPRB, 0.61625E+01_JPRB, 0.53229E+01_JPRB, 0.45612E+01_JPRB, 0.36808E+01_JPRB ,&
&  0.28880E+01_JPRB, 0.23617E+01_JPRB, 0.21119E+01_JPRB, 0.20118E+01_JPRB, 0.20099E+01_JPRB, 0.20957E+01_JPRB, 0.24738E+01_JPRB ,&
&  0.31964E+01_JPRB, 0.44999E+01_JPRB, 0.69254E+01_JPRB, 0.97962E+01_JPRB, 0.10192E+02_JPRB, 0.12788E+02_JPRB, 0.13113E+02_JPRB ,&
&  0.15257E+02_JPRB, 0.16784E+02_JPRB, 0.16486E+02_JPRB, 0.16829E+02_JPRB, 0.14184E+02_JPRB, 0.15445E+02_JPRB, 0.15960E+02_JPRB ,&
&  0.14729E+02_JPRB, 0.13736E+02_JPRB, 0.13590E+02_JPRB, 0.11766E+02_JPRB, 0.10845E+02_JPRB, 0.12950E+02_JPRB, 0.11203E+02_JPRB ,&
&  0.96254E+01_JPRB, 0.90592E+01_JPRB, 0.81721E+01_JPRB, 0.80468E+01_JPRB, 0.66238E+01_JPRB, 0.64415E+01_JPRB, 0.54957E+01_JPRB ,&
&  0.50902E+01_JPRB, 0.41354E+01_JPRB, 0.45994E+01_JPRB, 0.38918E+01_JPRB, 0.36117E+01_JPRB, 0.35743E+01_JPRB, 0.35705E+01_JPRB ,&
&  0.35441E+01_JPRB, 0.34226E+01_JPRB, 0.37780E+01_JPRB, 0.48373E+01_JPRB, 0.61975E+01_JPRB, 0.51220E+01_JPRB, 0.50128E+01_JPRB ,&
&  0.47328E+01_JPRB, 0.46070E+01_JPRB, 0.40249E+01_JPRB, 0.37566E+01_JPRB, 0.33581E+01_JPRB, 0.30293E+01_JPRB, 0.26764E+01_JPRB ,&
&  0.23062E+01_JPRB, 0.20040E+01_JPRB, 0.18430E+01_JPRB, 0.17436E+01_JPRB, 0.16748E+01_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.16068E+01_JPRB, 0.15827E+01_JPRB, 0.15079E+01_JPRB, 0.14087E+01_JPRB, 0.13284E+01_JPRB, 0.12617E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.11240E+01_JPRB, 0.11086E+01_JPRB, 0.11122E+01_JPRB, 0.11218E+01_JPRB, 0.11234E+01_JPRB, 0.11141E+01_JPRB ,&
&  0.11188E+01_JPRB, 0.13628E+01_JPRB, 0.13499E+01_JPRB, 0.13743E+01_JPRB, 0.18346E+01_JPRB, 0.29538E+01_JPRB, 0.40232E+01_JPRB ,&
&  0.44770E+01_JPRB, 0.45757E+01_JPRB, 0.49802E+01_JPRB, 0.56600E+01_JPRB, 0.64913E+01_JPRB, 0.70206E+01_JPRB, 0.71960E+01_JPRB ,&
&  0.70458E+01_JPRB, 0.54220E+01_JPRB, 0.49042E+01_JPRB, 0.44181E+01_JPRB, 0.41252E+01_JPRB, 0.38814E+01_JPRB, 0.36258E+01_JPRB ,&
&  0.32463E+01_JPRB, 0.27614E+01_JPRB, 0.23905E+01_JPRB, 0.21608E+01_JPRB, 0.20457E+01_JPRB, 0.19443E+01_JPRB, 0.20714E+01_JPRB ,&
&  0.23507E+01_JPRB, 0.28043E+01_JPRB, 0.42641E+01_JPRB, 0.67720E+01_JPRB, 0.10705E+02_JPRB, 0.13712E+02_JPRB, 0.14344E+02_JPRB ,&
&  0.12726E+02_JPRB, 0.13609E+02_JPRB, 0.19881E+02_JPRB, 0.17269E+02_JPRB, 0.16967E+02_JPRB, 0.18153E+02_JPRB, 0.16937E+02_JPRB ,&
&  0.15001E+02_JPRB, 0.13444E+02_JPRB, 0.12368E+02_JPRB, 0.11638E+02_JPRB, 0.13073E+02_JPRB, 0.12412E+02_JPRB, 0.10822E+02_JPRB ,&
&  0.93448E+01_JPRB, 0.90823E+01_JPRB, 0.85359E+01_JPRB, 0.79140E+01_JPRB, 0.74015E+01_JPRB, 0.76796E+01_JPRB, 0.70698E+01_JPRB ,&
&  0.64783E+01_JPRB, 0.70673E+01_JPRB, 0.55639E+01_JPRB, 0.42178E+01_JPRB, 0.34136E+01_JPRB, 0.35304E+01_JPRB, 0.37435E+01_JPRB ,&
&  0.34836E+01_JPRB, 0.30445E+01_JPRB, 0.26883E+01_JPRB, 0.30335E+01_JPRB, 0.40255E+01_JPRB, 0.46040E+01_JPRB, 0.43021E+01_JPRB ,&
&  0.38774E+01_JPRB, 0.39518E+01_JPRB, 0.37651E+01_JPRB, 0.36368E+01_JPRB, 0.35804E+01_JPRB, 0.31632E+01_JPRB, 0.26313E+01_JPRB ,&
&  0.21725E+01_JPRB, 0.18910E+01_JPRB, 0.17513E+01_JPRB, 0.16751E+01_JPRB, 0.16252E+01_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.16136E+01_JPRB, 0.15787E+01_JPRB, 0.15141E+01_JPRB, 0.14215E+01_JPRB, 0.13407E+01_JPRB, 0.12786E+01_JPRB, 0.12244E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.11695E+01_JPRB, 0.11589E+01_JPRB, 0.11455E+01_JPRB, 0.11463E+01_JPRB, 0.11551E+01_JPRB, 0.11612E+01_JPRB ,&
&  0.11596E+01_JPRB, 0.13956E+01_JPRB, 0.13492E+01_JPRB, 0.14176E+01_JPRB, 0.17091E+01_JPRB, 0.26755E+01_JPRB, 0.34278E+01_JPRB ,&
&  0.38112E+01_JPRB, 0.38793E+01_JPRB, 0.40207E+01_JPRB, 0.43907E+01_JPRB, 0.45878E+01_JPRB, 0.44495E+01_JPRB, 0.43466E+01_JPRB ,&
&  0.41530E+01_JPRB, 0.38487E+01_JPRB, 0.32192E+01_JPRB, 0.31789E+01_JPRB, 0.29482E+01_JPRB, 0.25761E+01_JPRB, 0.27715E+01_JPRB ,&
&  0.27791E+01_JPRB, 0.26879E+01_JPRB, 0.25023E+01_JPRB, 0.23337E+01_JPRB, 0.22213E+01_JPRB, 0.20841E+01_JPRB, 0.20321E+01_JPRB ,&
&  0.20676E+01_JPRB, 0.21594E+01_JPRB, 0.24834E+01_JPRB, 0.34846E+01_JPRB, 0.57870E+01_JPRB, 0.89757E+01_JPRB, 0.11696E+02_JPRB ,&
&  0.14326E+02_JPRB, 0.15611E+02_JPRB, 0.14439E+02_JPRB, 0.15489E+02_JPRB, 0.17311E+02_JPRB, 0.15888E+02_JPRB, 0.14635E+02_JPRB ,&
&  0.14433E+02_JPRB, 0.14185E+02_JPRB, 0.13458E+02_JPRB, 0.12740E+02_JPRB, 0.12565E+02_JPRB, 0.12029E+02_JPRB, 0.11130E+02_JPRB ,&
&  0.94665E+01_JPRB, 0.88575E+01_JPRB, 0.87476E+01_JPRB, 0.85648E+01_JPRB, 0.75233E+01_JPRB, 0.70275E+01_JPRB, 0.70361E+01_JPRB ,&
&  0.68080E+01_JPRB, 0.65720E+01_JPRB, 0.50250E+01_JPRB, 0.47247E+01_JPRB, 0.34998E+01_JPRB, 0.33738E+01_JPRB, 0.38078E+01_JPRB ,&
&  0.33305E+01_JPRB, 0.27910E+01_JPRB, 0.22710E+01_JPRB, 0.20692E+01_JPRB, 0.23947E+01_JPRB, 0.27944E+01_JPRB, 0.32979E+01_JPRB ,&
&  0.31730E+01_JPRB, 0.26035E+01_JPRB, 0.28001E+01_JPRB, 0.27923E+01_JPRB, 0.28549E+01_JPRB, 0.27060E+01_JPRB, 0.24454E+01_JPRB ,&
&  0.22442E+01_JPRB, 0.20902E+01_JPRB, 0.19295E+01_JPRB, 0.17777E+01_JPRB, 0.16759E+01_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.15366E+01_JPRB, 0.14530E+01_JPRB, 0.14034E+01_JPRB, 0.13449E+01_JPRB, 0.12830E+01_JPRB, 0.12283E+01_JPRB, 0.11920E+01_JPRB ,&
&  0.12047E+01_JPRB, 0.12123E+01_JPRB, 0.12058E+01_JPRB, 0.11765E+01_JPRB, 0.11516E+01_JPRB, 0.11422E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.11914E+01_JPRB, 0.13666E+01_JPRB, 0.14426E+01_JPRB, 0.15067E+01_JPRB, 0.20310E+01_JPRB, 0.23354E+01_JPRB, 0.26229E+01_JPRB ,&
&  0.27065E+01_JPRB, 0.27902E+01_JPRB, 0.27949E+01_JPRB, 0.27908E+01_JPRB, 0.27397E+01_JPRB, 0.26633E+01_JPRB, 0.25077E+01_JPRB ,&
&  0.22877E+01_JPRB, 0.20378E+01_JPRB, 0.19113E+01_JPRB, 0.18954E+01_JPRB, 0.19576E+01_JPRB, 0.17624E+01_JPRB, 0.18801E+01_JPRB ,&
&  0.19864E+01_JPRB, 0.20900E+01_JPRB, 0.21307E+01_JPRB, 0.21318E+01_JPRB, 0.21046E+01_JPRB, 0.20919E+01_JPRB, 0.21035E+01_JPRB ,&
&  0.21674E+01_JPRB, 0.21720E+01_JPRB, 0.21636E+01_JPRB, 0.25212E+01_JPRB, 0.36884E+01_JPRB, 0.62937E+01_JPRB, 0.90882E+01_JPRB ,&
&  0.10077E+02_JPRB, 0.12433E+02_JPRB, 0.15719E+02_JPRB, 0.16616E+02_JPRB, 0.14057E+02_JPRB, 0.12093E+02_JPRB, 0.11718E+02_JPRB ,&
&  0.12299E+02_JPRB, 0.13100E+02_JPRB, 0.13308E+02_JPRB, 0.12360E+02_JPRB, 0.12063E+02_JPRB, 0.11767E+02_JPRB, 0.11245E+02_JPRB ,&
&  0.10425E+02_JPRB, 0.95697E+01_JPRB, 0.90613E+01_JPRB, 0.86466E+01_JPRB, 0.82971E+01_JPRB, 0.75156E+01_JPRB, 0.72064E+01_JPRB ,&
&  0.73124E+01_JPRB, 0.59061E+01_JPRB, 0.51589E+01_JPRB, 0.54011E+01_JPRB, 0.42374E+01_JPRB, 0.41199E+01_JPRB, 0.44687E+01_JPRB ,&
&  0.35941E+01_JPRB, 0.27302E+01_JPRB, 0.23471E+01_JPRB, 0.20181E+01_JPRB, 0.19197E+01_JPRB, 0.21118E+01_JPRB, 0.22162E+01_JPRB ,&
&  0.21464E+01_JPRB, 0.20243E+01_JPRB, 0.22209E+01_JPRB, 0.22892E+01_JPRB, 0.23190E+01_JPRB, 0.22336E+01_JPRB, 0.20950E+01_JPRB ,&
&  0.19672E+01_JPRB, 0.18805E+01_JPRB, 0.18114E+01_JPRB, 0.17350E+01_JPRB, 0.16485E+01_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.14177E+01_JPRB, 0.13541E+01_JPRB, 0.13027E+01_JPRB, 0.12699E+01_JPRB, 0.12363E+01_JPRB, 0.11989E+01_JPRB, 0.11675E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11869E+01_JPRB, 0.11966E+01_JPRB, 0.11702E+01_JPRB, 0.11393E+01_JPRB, 0.11211E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.11100E+01_JPRB, 0.11886E+01_JPRB, 0.14405E+01_JPRB, 0.19989E+01_JPRB, 0.19040E+01_JPRB, 0.19493E+01_JPRB ,&
&  0.18932E+01_JPRB, 0.18231E+01_JPRB, 0.17792E+01_JPRB, 0.17681E+01_JPRB, 0.17128E+01_JPRB, 0.16716E+01_JPRB, 0.15916E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.12298E+01_JPRB, 0.11819E+01_JPRB, 0.11625E+01_JPRB, 0.12273E+01_JPRB, 0.13099E+01_JPRB, 0.14307E+01_JPRB ,&
&  0.15235E+01_JPRB, 0.16165E+01_JPRB, 0.16952E+01_JPRB, 0.17552E+01_JPRB, 0.17874E+01_JPRB, 0.18183E+01_JPRB, 0.18580E+01_JPRB ,&
&  0.19191E+01_JPRB, 0.19722E+01_JPRB, 0.20399E+01_JPRB, 0.22355E+01_JPRB, 0.28902E+01_JPRB, 0.50138E+01_JPRB, 0.71555E+01_JPRB ,&
&  0.67898E+01_JPRB, 0.68959E+01_JPRB, 0.92945E+01_JPRB, 0.11482E+02_JPRB, 0.10135E+02_JPRB, 0.86992E+01_JPRB, 0.80962E+01_JPRB ,&
&  0.85686E+01_JPRB, 0.94459E+01_JPRB, 0.10367E+02_JPRB, 0.10567E+02_JPRB, 0.10156E+02_JPRB, 0.10088E+02_JPRB, 0.99088E+01_JPRB ,&
&  0.95405E+01_JPRB, 0.92252E+01_JPRB, 0.88586E+01_JPRB, 0.85410E+01_JPRB, 0.82354E+01_JPRB, 0.83315E+01_JPRB, 0.81750E+01_JPRB ,&
&  0.79114E+01_JPRB, 0.73909E+01_JPRB, 0.66228E+01_JPRB, 0.61702E+01_JPRB, 0.49197E+01_JPRB, 0.47187E+01_JPRB, 0.53977E+01_JPRB ,&
&  0.41629E+01_JPRB, 0.30825E+01_JPRB, 0.25512E+01_JPRB, 0.21898E+01_JPRB, 0.20252E+01_JPRB, 0.18976E+01_JPRB, 0.17486E+01_JPRB ,&
&  0.15765E+01_JPRB, 0.16169E+01_JPRB, 0.17485E+01_JPRB, 0.17815E+01_JPRB, 0.17882E+01_JPRB, 0.17751E+01_JPRB, 0.16672E+01_JPRB ,&
&  0.16211E+01_JPRB, 0.15542E+01_JPRB, 0.15322E+01_JPRB, 0.15082E+01_JPRB, 0.14721E+01_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.12797E+01_JPRB, 0.12475E+01_JPRB, 0.12067E+01_JPRB, 0.11613E+01_JPRB, 0.11301E+01_JPRB, 0.11167E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11322E+01_JPRB, 0.11537E+01_JPRB, 0.11586E+01_JPRB, 0.11436E+01_JPRB, 0.11234E+01_JPRB, 0.11115E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.91086E+00_JPRB, 0.93934E+00_JPRB, 0.95431E+00_JPRB, 0.10638E+01_JPRB, 0.11907E+01_JPRB, 0.12146E+01_JPRB, 0.12352E+01_JPRB ,&
&  0.12681E+01_JPRB, 0.12239E+01_JPRB, 0.12179E+01_JPRB, 0.12126E+01_JPRB, 0.11896E+01_JPRB, 0.11633E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.94078E+00_JPRB, 0.90564E+00_JPRB, 0.88934E+00_JPRB, 0.93809E+00_JPRB, 0.10259E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.12090E+01_JPRB, 0.12878E+01_JPRB, 0.13669E+01_JPRB, 0.14320E+01_JPRB, 0.14733E+01_JPRB, 0.15100E+01_JPRB, 0.15529E+01_JPRB ,&
&  0.16075E+01_JPRB, 0.16706E+01_JPRB, 0.17132E+01_JPRB, 0.18415E+01_JPRB, 0.21079E+01_JPRB, 0.32305E+01_JPRB, 0.44602E+01_JPRB ,&
&  0.42562E+01_JPRB, 0.42573E+01_JPRB, 0.46751E+01_JPRB, 0.52528E+01_JPRB, 0.54400E+01_JPRB, 0.52728E+01_JPRB, 0.53444E+01_JPRB ,&
&  0.58541E+01_JPRB, 0.64785E+01_JPRB, 0.77301E+01_JPRB, 0.78444E+01_JPRB, 0.79591E+01_JPRB, 0.81658E+01_JPRB, 0.80391E+01_JPRB ,&
&  0.81449E+01_JPRB, 0.80873E+01_JPRB, 0.82958E+01_JPRB, 0.81417E+01_JPRB, 0.79093E+01_JPRB, 0.79036E+01_JPRB, 0.78135E+01_JPRB ,&
&  0.78684E+01_JPRB, 0.78103E+01_JPRB, 0.72390E+01_JPRB, 0.67470E+01_JPRB, 0.59540E+01_JPRB, 0.59062E+01_JPRB, 0.53165E+01_JPRB ,&
&  0.42671E+01_JPRB, 0.30389E+01_JPRB, 0.27391E+01_JPRB, 0.23498E+01_JPRB, 0.20753E+01_JPRB, 0.17705E+01_JPRB, 0.15415E+01_JPRB ,&
&  0.14406E+01_JPRB, 0.15276E+01_JPRB, 0.15427E+01_JPRB, 0.15507E+01_JPRB, 0.15641E+01_JPRB, 0.15315E+01_JPRB, 0.13938E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.13989E+01_JPRB, 0.13885E+01_JPRB, 0.13483E+01_JPRB, 0.13090E+01_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.11718E+01_JPRB, 0.11660E+01_JPRB, 0.11493E+01_JPRB, 0.11090E+01_JPRB, 0.10547E+01_JPRB, 0.10436E+01_JPRB, 0.99780E+00_JPRB ,&
&  0.10789E+01_JPRB, 0.11107E+01_JPRB, 0.11087E+01_JPRB, 0.11131E+01_JPRB, 0.11244E+01_JPRB, 0.10257E+01_JPRB, 0.86036E+00_JPRB ,&
&  0.78805E+00_JPRB, 0.79157E+00_JPRB, 0.82139E+00_JPRB, 0.85054E+00_JPRB, 0.87500E+00_JPRB, 0.89290E+00_JPRB, 0.92084E+00_JPRB ,&
&  0.94037E+00_JPRB, 0.93815E+00_JPRB, 0.94631E+00_JPRB, 0.94171E+00_JPRB, 0.92722E+00_JPRB, 0.91774E+00_JPRB, 0.91406E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.82540E+00_JPRB, 0.80564E+00_JPRB, 0.77728E+00_JPRB, 0.82308E+00_JPRB, 0.86577E+00_JPRB, 0.92535E+00_JPRB ,&
&  0.10034E+01_JPRB, 0.10834E+01_JPRB, 0.11458E+01_JPRB, 0.12056E+01_JPRB, 0.12525E+01_JPRB, 0.12929E+01_JPRB, 0.13303E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.14199E+01_JPRB, 0.14681E+01_JPRB, 0.15385E+01_JPRB, 0.16821E+01_JPRB, 0.20676E+01_JPRB, 0.27353E+01_JPRB ,&
&  0.29807E+01_JPRB, 0.32793E+01_JPRB, 0.35356E+01_JPRB, 0.32631E+01_JPRB, 0.36064E+01_JPRB, 0.40905E+01_JPRB, 0.41724E+01_JPRB ,&
&  0.43484E+01_JPRB, 0.43544E+01_JPRB, 0.45690E+01_JPRB, 0.48444E+01_JPRB, 0.55170E+01_JPRB, 0.60020E+01_JPRB, 0.62157E+01_JPRB ,&
&  0.63326E+01_JPRB, 0.67203E+01_JPRB, 0.71281E+01_JPRB, 0.72991E+01_JPRB, 0.72889E+01_JPRB, 0.73288E+01_JPRB, 0.74224E+01_JPRB ,&
&  0.75004E+01_JPRB, 0.75556E+01_JPRB, 0.74334E+01_JPRB, 0.70798E+01_JPRB, 0.65620E+01_JPRB, 0.59272E+01_JPRB, 0.51207E+01_JPRB ,&
&  0.44232E+01_JPRB, 0.36697E+01_JPRB, 0.30570E+01_JPRB, 0.26722E+01_JPRB, 0.22994E+01_JPRB, 0.19202E+01_JPRB, 0.16498E+01_JPRB ,&
&  0.14165E+01_JPRB, 0.13718E+01_JPRB, 0.14656E+01_JPRB, 0.14812E+01_JPRB, 0.14860E+01_JPRB, 0.14421E+01_JPRB, 0.13237E+01_JPRB ,&
&  0.13112E+01_JPRB, 0.13376E+01_JPRB, 0.13286E+01_JPRB, 0.12698E+01_JPRB, 0.12090E+01_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.11640E+01_JPRB, 0.11253E+01_JPRB, 0.11081E+01_JPRB, 0.11001E+01_JPRB, 0.10503E+01_JPRB, 0.97339E+00_JPRB, 0.90466E+00_JPRB ,&
&  0.10130E+01_JPRB, 0.10008E+01_JPRB, 0.10358E+01_JPRB, 0.86568E+00_JPRB, 0.81060E+00_JPRB, 0.78342E+00_JPRB, 0.74352E+00_JPRB ,&
&  0.71875E+00_JPRB, 0.71825E+00_JPRB, 0.73111E+00_JPRB, 0.76314E+00_JPRB, 0.79138E+00_JPRB, 0.80160E+00_JPRB, 0.80938E+00_JPRB ,&
&  0.81243E+00_JPRB, 0.81806E+00_JPRB, 0.83493E+00_JPRB, 0.83812E+00_JPRB, 0.83597E+00_JPRB, 0.83377E+00_JPRB, 0.83480E+00_JPRB ,&
&  0.80025E+00_JPRB, 0.78739E+00_JPRB, 0.78860E+00_JPRB, 0.78495E+00_JPRB, 0.79943E+00_JPRB, 0.81663E+00_JPRB, 0.84768E+00_JPRB ,&
&  0.90438E+00_JPRB, 0.87770E+00_JPRB, 0.10024E+01_JPRB, 0.10437E+01_JPRB, 0.10857E+01_JPRB, 0.11213E+01_JPRB, 0.11509E+01_JPRB ,&
&  0.11794E+01_JPRB, 0.12227E+01_JPRB, 0.12693E+01_JPRB, 0.13155E+01_JPRB, 0.13994E+01_JPRB, 0.15650E+01_JPRB, 0.17927E+01_JPRB ,&
&  0.22017E+01_JPRB, 0.20543E+01_JPRB, 0.23380E+01_JPRB, 0.25234E+01_JPRB, 0.30415E+01_JPRB, 0.35228E+01_JPRB, 0.35992E+01_JPRB ,&
&  0.41464E+01_JPRB, 0.39125E+01_JPRB, 0.37122E+01_JPRB, 0.35177E+01_JPRB, 0.36776E+01_JPRB, 0.39992E+01_JPRB, 0.43563E+01_JPRB ,&
&  0.46462E+01_JPRB, 0.47982E+01_JPRB, 0.52025E+01_JPRB, 0.54877E+01_JPRB, 0.58146E+01_JPRB, 0.62187E+01_JPRB, 0.65967E+01_JPRB ,&
&  0.69201E+01_JPRB, 0.70945E+01_JPRB, 0.71539E+01_JPRB, 0.66402E+01_JPRB, 0.61981E+01_JPRB, 0.57725E+01_JPRB, 0.49499E+01_JPRB ,&
&  0.43105E+01_JPRB, 0.38619E+01_JPRB, 0.34880E+01_JPRB, 0.32272E+01_JPRB, 0.27424E+01_JPRB, 0.23223E+01_JPRB, 0.19205E+01_JPRB ,&
&  0.14581E+01_JPRB, 0.12752E+01_JPRB, 0.12795E+01_JPRB, 0.13114E+01_JPRB, 0.13233E+01_JPRB, 0.13367E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.13088E+01_JPRB, 0.13456E+01_JPRB, 0.12940E+01_JPRB, 0.12275E+01_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.12860E+01_JPRB, 0.12137E+01_JPRB, 0.11497E+01_JPRB, 0.11150E+01_JPRB, 0.10694E+01_JPRB, 0.98600E+00_JPRB, 0.87940E+00_JPRB ,&
&  0.86914E+00_JPRB, 0.82545E+00_JPRB, 0.79839E+00_JPRB, 0.77238E+00_JPRB, 0.74456E+00_JPRB, 0.69728E+00_JPRB, 0.66830E+00_JPRB ,&
&  0.68347E+00_JPRB, 0.72744E+00_JPRB, 0.77747E+00_JPRB, 0.81794E+00_JPRB, 0.83635E+00_JPRB, 0.83447E+00_JPRB, 0.82510E+00_JPRB ,&
&  0.81782E+00_JPRB, 0.81726E+00_JPRB, 0.81980E+00_JPRB, 0.82384E+00_JPRB, 0.81469E+00_JPRB, 0.80172E+00_JPRB, 0.79331E+00_JPRB ,&
&  0.78065E+00_JPRB, 0.76727E+00_JPRB, 0.75790E+00_JPRB, 0.76674E+00_JPRB, 0.77873E+00_JPRB, 0.80423E+00_JPRB, 0.81196E+00_JPRB ,&
&  0.73673E+00_JPRB, 0.75058E+00_JPRB, 0.85869E+00_JPRB, 0.90219E+00_JPRB, 0.94404E+00_JPRB, 0.97988E+00_JPRB, 0.10101E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.10794E+01_JPRB, 0.11261E+01_JPRB, 0.11764E+01_JPRB, 0.12455E+01_JPRB, 0.13481E+01_JPRB, 0.14921E+01_JPRB ,&
&  0.17830E+01_JPRB, 0.17754E+01_JPRB, 0.17619E+01_JPRB, 0.20068E+01_JPRB, 0.27926E+01_JPRB, 0.30095E+01_JPRB, 0.29983E+01_JPRB ,&
&  0.29819E+01_JPRB, 0.28557E+01_JPRB, 0.27947E+01_JPRB, 0.28689E+01_JPRB, 0.29205E+01_JPRB, 0.31075E+01_JPRB, 0.34014E+01_JPRB ,&
&  0.34396E+01_JPRB, 0.37092E+01_JPRB, 0.40338E+01_JPRB, 0.43079E+01_JPRB, 0.46142E+01_JPRB, 0.50845E+01_JPRB, 0.56373E+01_JPRB ,&
&  0.61952E+01_JPRB, 0.66419E+01_JPRB, 0.67681E+01_JPRB, 0.65624E+01_JPRB, 0.61562E+01_JPRB, 0.56972E+01_JPRB, 0.51228E+01_JPRB ,&
&  0.43862E+01_JPRB, 0.38769E+01_JPRB, 0.35488E+01_JPRB, 0.32479E+01_JPRB, 0.29674E+01_JPRB, 0.25759E+01_JPRB, 0.20305E+01_JPRB ,&
&  0.14767E+01_JPRB, 0.12495E+01_JPRB, 0.11956E+01_JPRB, 0.11914E+01_JPRB, 0.12142E+01_JPRB, 0.12548E+01_JPRB, 0.12772E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.12538E+01_JPRB, 0.12293E+01_JPRB, 0.12681E+01_JPRB, 0.13123E+01_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.12365E+01_JPRB, 0.12040E+01_JPRB, 0.11337E+01_JPRB, 0.11076E+01_JPRB, 0.10522E+01_JPRB, 0.99115E+00_JPRB, 0.91417E+00_JPRB ,&
&  0.85470E+00_JPRB, 0.80002E+00_JPRB, 0.75991E+00_JPRB, 0.71660E+00_JPRB, 0.69795E+00_JPRB, 0.70072E+00_JPRB, 0.74198E+00_JPRB ,&
&  0.79597E+00_JPRB, 0.83863E+00_JPRB, 0.86688E+00_JPRB, 0.87671E+00_JPRB, 0.87351E+00_JPRB, 0.85550E+00_JPRB, 0.83544E+00_JPRB ,&
&  0.81879E+00_JPRB, 0.81555E+00_JPRB, 0.80440E+00_JPRB, 0.79183E+00_JPRB, 0.77513E+00_JPRB, 0.75280E+00_JPRB, 0.73831E+00_JPRB ,&
&  0.71949E+00_JPRB, 0.71301E+00_JPRB, 0.72203E+00_JPRB, 0.74388E+00_JPRB, 0.77344E+00_JPRB, 0.77769E+00_JPRB, 0.72656E+00_JPRB ,&
&  0.68149E+00_JPRB, 0.66813E+00_JPRB, 0.68072E+00_JPRB, 0.79471E+00_JPRB, 0.85395E+00_JPRB, 0.88673E+00_JPRB, 0.92713E+00_JPRB ,&
&  0.91336E+00_JPRB, 0.91812E+00_JPRB, 0.95206E+00_JPRB, 0.10509E+01_JPRB, 0.11008E+01_JPRB, 0.11819E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.17869E+01_JPRB, 0.19719E+01_JPRB, 0.18107E+01_JPRB, 0.19765E+01_JPRB, 0.24885E+01_JPRB, 0.22892E+01_JPRB ,&
&  0.21884E+01_JPRB, 0.20838E+01_JPRB, 0.21571E+01_JPRB, 0.22434E+01_JPRB, 0.22386E+01_JPRB, 0.23173E+01_JPRB, 0.24033E+01_JPRB ,&
&  0.23824E+01_JPRB, 0.25549E+01_JPRB, 0.29069E+01_JPRB, 0.32430E+01_JPRB, 0.35701E+01_JPRB, 0.39966E+01_JPRB, 0.44426E+01_JPRB ,&
&  0.49821E+01_JPRB, 0.54313E+01_JPRB, 0.57000E+01_JPRB, 0.55715E+01_JPRB, 0.53687E+01_JPRB, 0.50340E+01_JPRB, 0.45599E+01_JPRB ,&
&  0.39352E+01_JPRB, 0.34903E+01_JPRB, 0.31984E+01_JPRB, 0.29821E+01_JPRB, 0.27118E+01_JPRB, 0.23381E+01_JPRB, 0.18674E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.13189E+01_JPRB, 0.12354E+01_JPRB, 0.12133E+01_JPRB, 0.12204E+01_JPRB, 0.12270E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.11746E+01_JPRB, 0.11506E+01_JPRB, 0.11310E+01_JPRB, 0.11382E+01_JPRB, 0.12207E+01_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.11926E+01_JPRB, 0.12027E+01_JPRB, 0.12005E+01_JPRB, 0.11491E+01_JPRB, 0.10790E+01_JPRB, 0.98732E+00_JPRB, 0.91725E+00_JPRB ,&
&  0.86446E+00_JPRB, 0.84821E+00_JPRB, 0.84476E+00_JPRB, 0.85520E+00_JPRB, 0.87962E+00_JPRB, 0.91501E+00_JPRB, 0.94360E+00_JPRB ,&
&  0.97044E+00_JPRB, 0.97804E+00_JPRB, 0.97942E+00_JPRB, 0.96016E+00_JPRB, 0.92301E+00_JPRB, 0.88004E+00_JPRB, 0.84917E+00_JPRB ,&
&  0.83101E+00_JPRB, 0.81884E+00_JPRB, 0.80030E+00_JPRB, 0.77123E+00_JPRB, 0.74944E+00_JPRB, 0.72156E+00_JPRB, 0.72098E+00_JPRB ,&
&  0.70732E+00_JPRB, 0.71140E+00_JPRB, 0.73222E+00_JPRB, 0.74081E+00_JPRB, 0.75699E+00_JPRB, 0.75131E+00_JPRB, 0.69512E+00_JPRB ,&
&  0.64050E+00_JPRB, 0.61262E+00_JPRB, 0.60174E+00_JPRB, 0.60324E+00_JPRB, 0.66046E+00_JPRB, 0.79044E+00_JPRB, 0.86380E+00_JPRB ,&
&  0.90293E+00_JPRB, 0.92693E+00_JPRB, 0.95594E+00_JPRB, 0.98876E+00_JPRB, 0.10099E+01_JPRB, 0.10592E+01_JPRB, 0.11741E+01_JPRB ,&
&  0.13519E+01_JPRB, 0.15799E+01_JPRB, 0.17204E+01_JPRB, 0.15190E+01_JPRB, 0.14192E+01_JPRB, 0.15252E+01_JPRB, 0.16892E+01_JPRB ,&
&  0.18716E+01_JPRB, 0.17982E+01_JPRB, 0.17194E+01_JPRB, 0.17872E+01_JPRB, 0.18240E+01_JPRB, 0.18607E+01_JPRB, 0.18916E+01_JPRB ,&
&  0.19383E+01_JPRB, 0.20804E+01_JPRB, 0.23803E+01_JPRB, 0.25431E+01_JPRB, 0.28019E+01_JPRB, 0.31716E+01_JPRB, 0.34905E+01_JPRB ,&
&  0.38711E+01_JPRB, 0.41365E+01_JPRB, 0.42320E+01_JPRB, 0.40298E+01_JPRB, 0.38798E+01_JPRB, 0.38512E+01_JPRB, 0.37345E+01_JPRB ,&
&  0.33659E+01_JPRB, 0.30901E+01_JPRB, 0.28631E+01_JPRB, 0.26647E+01_JPRB, 0.23864E+01_JPRB, 0.21428E+01_JPRB, 0.18913E+01_JPRB ,&
&  0.16664E+01_JPRB, 0.15124E+01_JPRB, 0.14320E+01_JPRB, 0.13716E+01_JPRB, 0.13427E+01_JPRB, 0.13222E+01_JPRB, 0.12841E+01_JPRB ,&
&  0.12374E+01_JPRB, 0.11936E+01_JPRB, 0.11666E+01_JPRB, 0.11645E+01_JPRB, 0.11938E+01_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.12708E+01_JPRB, 0.12612E+01_JPRB, 0.12423E+01_JPRB, 0.12157E+01_JPRB, 0.11670E+01_JPRB, 0.11100E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10514E+01_JPRB, 0.10520E+01_JPRB, 0.10592E+01_JPRB, 0.10744E+01_JPRB, 0.10947E+01_JPRB, 0.11106E+01_JPRB ,&
&  0.11120E+01_JPRB, 0.10887E+01_JPRB, 0.10441E+01_JPRB, 0.97578E+00_JPRB, 0.91991E+00_JPRB, 0.88065E+00_JPRB, 0.85743E+00_JPRB ,&
&  0.84049E+00_JPRB, 0.81490E+00_JPRB, 0.78247E+00_JPRB, 0.73598E+00_JPRB, 0.70322E+00_JPRB, 0.67978E+00_JPRB, 0.68618E+00_JPRB ,&
&  0.71085E+00_JPRB, 0.72041E+00_JPRB, 0.71367E+00_JPRB, 0.72978E+00_JPRB, 0.72274E+00_JPRB, 0.71213E+00_JPRB, 0.64692E+00_JPRB ,&
&  0.59844E+00_JPRB, 0.57340E+00_JPRB, 0.55600E+00_JPRB, 0.55224E+00_JPRB, 0.56991E+00_JPRB, 0.60450E+00_JPRB, 0.67625E+00_JPRB ,&
&  0.81669E+00_JPRB, 0.87343E+00_JPRB, 0.89297E+00_JPRB, 0.91764E+00_JPRB, 0.94315E+00_JPRB, 0.97578E+00_JPRB, 0.10308E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.12446E+01_JPRB, 0.13586E+01_JPRB, 0.13987E+01_JPRB, 0.14248E+01_JPRB, 0.14558E+01_JPRB, 0.16745E+01_JPRB ,&
&  0.22748E+01_JPRB, 0.20142E+01_JPRB, 0.17104E+01_JPRB, 0.15436E+01_JPRB, 0.14736E+01_JPRB, 0.14555E+01_JPRB, 0.15562E+01_JPRB ,&
&  0.17043E+01_JPRB, 0.18454E+01_JPRB, 0.20277E+01_JPRB, 0.21714E+01_JPRB, 0.23366E+01_JPRB, 0.25546E+01_JPRB, 0.28587E+01_JPRB ,&
&  0.31601E+01_JPRB, 0.35146E+01_JPRB, 0.41381E+01_JPRB, 0.37058E+01_JPRB, 0.34638E+01_JPRB, 0.33702E+01_JPRB, 0.33119E+01_JPRB ,&
&  0.31527E+01_JPRB, 0.31159E+01_JPRB, 0.28338E+01_JPRB, 0.24761E+01_JPRB, 0.22868E+01_JPRB, 0.20754E+01_JPRB, 0.19124E+01_JPRB ,&
&  0.17659E+01_JPRB, 0.16353E+01_JPRB, 0.15559E+01_JPRB, 0.14934E+01_JPRB, 0.14632E+01_JPRB, 0.14327E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.13350E+01_JPRB, 0.13111E+01_JPRB, 0.12907E+01_JPRB, 0.12771E+01_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.13545E+01_JPRB, 0.13281E+01_JPRB, 0.12980E+01_JPRB, 0.12736E+01_JPRB, 0.12472E+01_JPRB, 0.12287E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12056E+01_JPRB, 0.12054E+01_JPRB, 0.12018E+01_JPRB, 0.11985E+01_JPRB, 0.11860E+01_JPRB, 0.11668E+01_JPRB, 0.11389E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.10147E+01_JPRB, 0.94981E+00_JPRB, 0.91193E+00_JPRB, 0.87668E+00_JPRB, 0.85013E+00_JPRB, 0.82376E+00_JPRB ,&
&  0.79022E+00_JPRB, 0.76166E+00_JPRB, 0.72838E+00_JPRB, 0.68892E+00_JPRB, 0.66846E+00_JPRB, 0.67687E+00_JPRB, 0.68363E+00_JPRB ,&
&  0.69219E+00_JPRB, 0.70970E+00_JPRB, 0.70398E+00_JPRB, 0.68356E+00_JPRB, 0.67303E+00_JPRB, 0.66875E+00_JPRB, 0.61702E+00_JPRB ,&
&  0.57416E+00_JPRB, 0.54050E+00_JPRB, 0.53532E+00_JPRB, 0.53403E+00_JPRB, 0.54430E+00_JPRB, 0.56939E+00_JPRB, 0.63136E+00_JPRB ,&
&  0.73248E+00_JPRB, 0.86454E+00_JPRB, 0.87962E+00_JPRB, 0.89487E+00_JPRB, 0.91631E+00_JPRB, 0.94416E+00_JPRB, 0.97561E+00_JPRB ,&
&  0.10128E+01_JPRB, 0.10571E+01_JPRB, 0.11095E+01_JPRB, 0.11660E+01_JPRB, 0.12232E+01_JPRB, 0.12511E+01_JPRB, 0.12788E+01_JPRB ,&
&  0.13390E+01_JPRB, 0.13329E+01_JPRB, 0.13491E+01_JPRB, 0.13612E+01_JPRB, 0.13748E+01_JPRB, 0.13569E+01_JPRB, 0.14240E+01_JPRB ,&
&  0.15735E+01_JPRB, 0.17267E+01_JPRB, 0.18770E+01_JPRB, 0.19885E+01_JPRB, 0.21130E+01_JPRB, 0.22879E+01_JPRB, 0.24764E+01_JPRB ,&
&  0.26406E+01_JPRB, 0.28667E+01_JPRB, 0.31695E+01_JPRB, 0.31451E+01_JPRB, 0.30848E+01_JPRB, 0.30410E+01_JPRB, 0.30319E+01_JPRB ,&
&  0.30076E+01_JPRB, 0.30247E+01_JPRB, 0.28457E+01_JPRB, 0.25848E+01_JPRB, 0.23419E+01_JPRB, 0.20819E+01_JPRB, 0.19309E+01_JPRB ,&
&  0.18208E+01_JPRB, 0.16627E+01_JPRB, 0.15788E+01_JPRB, 0.15270E+01_JPRB, 0.14906E+01_JPRB, 0.14550E+01_JPRB, 0.14267E+01_JPRB ,&
&  0.14130E+01_JPRB, 0.14073E+01_JPRB, 0.14048E+01_JPRB, 0.13926E+01_JPRB, 0.13760E+01_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.13661E+01_JPRB, 0.13388E+01_JPRB, 0.13136E+01_JPRB, 0.12893E+01_JPRB, 0.12657E+01_JPRB, 0.12415E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.11794E+01_JPRB, 0.11583E+01_JPRB, 0.11323E+01_JPRB, 0.11034E+01_JPRB, 0.10701E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10042E+01_JPRB, 0.96839E+00_JPRB, 0.92189E+00_JPRB, 0.87927E+00_JPRB, 0.84747E+00_JPRB, 0.82234E+00_JPRB, 0.79470E+00_JPRB ,&
&  0.77336E+00_JPRB, 0.74435E+00_JPRB, 0.70710E+00_JPRB, 0.68141E+00_JPRB, 0.64816E+00_JPRB, 0.63923E+00_JPRB, 0.67759E+00_JPRB ,&
&  0.69314E+00_JPRB, 0.68956E+00_JPRB, 0.67674E+00_JPRB, 0.65749E+00_JPRB, 0.64006E+00_JPRB, 0.57820E+00_JPRB, 0.58820E+00_JPRB ,&
&  0.56138E+00_JPRB, 0.53888E+00_JPRB, 0.53684E+00_JPRB, 0.53922E+00_JPRB, 0.54838E+00_JPRB, 0.57312E+00_JPRB, 0.60033E+00_JPRB ,&
&  0.64923E+00_JPRB, 0.80643E+00_JPRB, 0.84282E+00_JPRB, 0.87709E+00_JPRB, 0.89551E+00_JPRB, 0.91377E+00_JPRB, 0.93154E+00_JPRB ,&
&  0.95176E+00_JPRB, 0.97712E+00_JPRB, 0.10016E+01_JPRB, 0.10345E+01_JPRB, 0.10693E+01_JPRB, 0.10966E+01_JPRB, 0.11421E+01_JPRB ,&
&  0.11754E+01_JPRB, 0.12047E+01_JPRB, 0.12364E+01_JPRB, 0.12657E+01_JPRB, 0.13089E+01_JPRB, 0.13144E+01_JPRB, 0.13314E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.16121E+01_JPRB, 0.17558E+01_JPRB, 0.18830E+01_JPRB, 0.19873E+01_JPRB, 0.21052E+01_JPRB, 0.22531E+01_JPRB ,&
&  0.23580E+01_JPRB, 0.25183E+01_JPRB, 0.26951E+01_JPRB, 0.27490E+01_JPRB, 0.27400E+01_JPRB, 0.27162E+01_JPRB, 0.27118E+01_JPRB ,&
&  0.27034E+01_JPRB, 0.27107E+01_JPRB, 0.26170E+01_JPRB, 0.25468E+01_JPRB, 0.24089E+01_JPRB, 0.22456E+01_JPRB, 0.21130E+01_JPRB ,&
&  0.19562E+01_JPRB, 0.18168E+01_JPRB, 0.17270E+01_JPRB, 0.16596E+01_JPRB, 0.16094E+01_JPRB, 0.15762E+01_JPRB, 0.15487E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.14936E+01_JPRB, 0.14598E+01_JPRB, 0.14260E+01_JPRB, 0.13933E+01_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.13233E+01_JPRB, 0.12994E+01_JPRB, 0.12765E+01_JPRB, 0.12442E+01_JPRB, 0.12116E+01_JPRB, 0.11803E+01_JPRB, 0.11558E+01_JPRB ,&
&  0.11341E+01_JPRB, 0.11111E+01_JPRB, 0.10883E+01_JPRB, 0.10618E+01_JPRB, 0.10359E+01_JPRB, 0.10100E+01_JPRB, 0.98594E+00_JPRB ,&
&  0.96178E+00_JPRB, 0.93263E+00_JPRB, 0.90465E+00_JPRB, 0.87554E+00_JPRB, 0.85050E+00_JPRB, 0.82685E+00_JPRB, 0.79765E+00_JPRB ,&
&  0.77641E+00_JPRB, 0.74212E+00_JPRB, 0.70896E+00_JPRB, 0.69360E+00_JPRB, 0.65623E+00_JPRB, 0.63501E+00_JPRB, 0.62157E+00_JPRB ,&
&  0.63770E+00_JPRB, 0.62540E+00_JPRB, 0.64486E+00_JPRB, 0.61994E+00_JPRB, 0.60286E+00_JPRB, 0.58731E+00_JPRB, 0.57117E+00_JPRB ,&
&  0.55932E+00_JPRB, 0.54559E+00_JPRB, 0.54661E+00_JPRB, 0.55408E+00_JPRB, 0.55977E+00_JPRB, 0.57922E+00_JPRB, 0.60223E+00_JPRB ,&
&  0.65059E+00_JPRB, 0.78583E+00_JPRB, 0.82225E+00_JPRB, 0.85820E+00_JPRB, 0.87421E+00_JPRB, 0.89143E+00_JPRB, 0.90589E+00_JPRB ,&
&  0.92577E+00_JPRB, 0.94163E+00_JPRB, 0.96383E+00_JPRB, 0.98276E+00_JPRB, 0.10231E+01_JPRB, 0.10534E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.11122E+01_JPRB, 0.11295E+01_JPRB, 0.11683E+01_JPRB, 0.12367E+01_JPRB, 0.12759E+01_JPRB, 0.12985E+01_JPRB ,&
&  0.13643E+01_JPRB, 0.14511E+01_JPRB, 0.15801E+01_JPRB, 0.17261E+01_JPRB, 0.18323E+01_JPRB, 0.19191E+01_JPRB, 0.20603E+01_JPRB ,&
&  0.21572E+01_JPRB, 0.23218E+01_JPRB, 0.24901E+01_JPRB, 0.25635E+01_JPRB, 0.25663E+01_JPRB, 0.25288E+01_JPRB, 0.24841E+01_JPRB ,&
&  0.24652E+01_JPRB, 0.24141E+01_JPRB, 0.23767E+01_JPRB, 0.23483E+01_JPRB, 0.22848E+01_JPRB, 0.22130E+01_JPRB, 0.21405E+01_JPRB ,&
&  0.20482E+01_JPRB, 0.19652E+01_JPRB, 0.18942E+01_JPRB, 0.18136E+01_JPRB, 0.17307E+01_JPRB, 0.16511E+01_JPRB, 0.15765E+01_JPRB ,&
&  0.15075E+01_JPRB, 0.14528E+01_JPRB, 0.14014E+01_JPRB, 0.13697E+01_JPRB, 0.13398E+01_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.12620E+01_JPRB, 0.12277E+01_JPRB, 0.11917E+01_JPRB, 0.11578E+01_JPRB, 0.11293E+01_JPRB, 0.11062E+01_JPRB, 0.10877E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.10511E+01_JPRB, 0.10311E+01_JPRB, 0.10099E+01_JPRB, 0.99007E+00_JPRB, 0.97568E+00_JPRB, 0.96247E+00_JPRB ,&
&  0.95173E+00_JPRB, 0.93554E+00_JPRB, 0.91676E+00_JPRB, 0.89378E+00_JPRB, 0.86848E+00_JPRB, 0.84595E+00_JPRB, 0.81319E+00_JPRB ,&
&  0.78978E+00_JPRB, 0.76367E+00_JPRB, 0.72486E+00_JPRB, 0.70340E+00_JPRB, 0.66074E+00_JPRB, 0.63187E+00_JPRB, 0.61438E+00_JPRB ,&
&  0.60145E+00_JPRB, 0.60154E+00_JPRB, 0.60609E+00_JPRB, 0.61554E+00_JPRB, 0.61832E+00_JPRB, 0.61315E+00_JPRB, 0.61393E+00_JPRB ,&
&  0.60496E+00_JPRB, 0.59975E+00_JPRB, 0.59560E+00_JPRB, 0.59588E+00_JPRB, 0.60053E+00_JPRB, 0.61202E+00_JPRB, 0.63409E+00_JPRB ,&
&  0.67883E+00_JPRB, 0.74125E+00_JPRB, 0.80401E+00_JPRB, 0.85067E+00_JPRB, 0.86998E+00_JPRB, 0.88713E+00_JPRB, 0.89811E+00_JPRB ,&
&  0.91487E+00_JPRB, 0.93094E+00_JPRB, 0.95532E+00_JPRB, 0.97820E+00_JPRB, 0.10011E+01_JPRB, 0.10202E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10583E+01_JPRB, 0.10663E+01_JPRB, 0.10927E+01_JPRB, 0.11401E+01_JPRB, 0.11905E+01_JPRB, 0.12191E+01_JPRB ,&
&  0.12616E+01_JPRB, 0.13177E+01_JPRB, 0.14115E+01_JPRB, 0.15481E+01_JPRB, 0.16944E+01_JPRB, 0.18270E+01_JPRB, 0.19561E+01_JPRB ,&
&  0.20779E+01_JPRB, 0.22246E+01_JPRB, 0.23608E+01_JPRB, 0.24103E+01_JPRB, 0.24026E+01_JPRB, 0.24106E+01_JPRB, 0.23766E+01_JPRB ,&
&  0.23502E+01_JPRB, 0.23157E+01_JPRB, 0.22932E+01_JPRB, 0.22692E+01_JPRB, 0.22181E+01_JPRB, 0.21437E+01_JPRB, 0.20676E+01_JPRB ,&
&  0.19854E+01_JPRB, 0.18986E+01_JPRB, 0.18151E+01_JPRB, 0.17318E+01_JPRB, 0.16520E+01_JPRB, 0.15756E+01_JPRB, 0.15110E+01_JPRB ,&
&  0.14502E+01_JPRB, 0.14027E+01_JPRB, 0.13597E+01_JPRB, 0.13252E+01_JPRB, 0.12939E+01_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.11994E+01_JPRB, 0.11671E+01_JPRB, 0.11375E+01_JPRB, 0.11110E+01_JPRB, 0.10869E+01_JPRB, 0.10666E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10229E+01_JPRB, 0.10124E+01_JPRB, 0.10013E+01_JPRB, 0.99039E+00_JPRB, 0.98505E+00_JPRB, 0.97721E+00_JPRB ,&
&  0.96805E+00_JPRB, 0.95735E+00_JPRB, 0.94815E+00_JPRB, 0.93569E+00_JPRB, 0.91552E+00_JPRB, 0.89060E+00_JPRB, 0.85863E+00_JPRB ,&
&  0.83906E+00_JPRB, 0.81407E+00_JPRB, 0.78699E+00_JPRB, 0.75831E+00_JPRB, 0.73859E+00_JPRB, 0.69735E+00_JPRB, 0.68203E+00_JPRB ,&
&  0.67149E+00_JPRB, 0.65014E+00_JPRB, 0.63705E+00_JPRB, 0.63117E+00_JPRB, 0.65141E+00_JPRB, 0.67521E+00_JPRB, 0.68858E+00_JPRB ,&
&  0.67137E+00_JPRB, 0.68464E+00_JPRB, 0.68061E+00_JPRB, 0.67751E+00_JPRB, 0.68047E+00_JPRB, 0.68856E+00_JPRB, 0.72415E+00_JPRB ,&
&  0.78944E+00_JPRB, 0.82062E+00_JPRB, 0.84483E+00_JPRB, 0.86896E+00_JPRB, 0.88786E+00_JPRB, 0.90535E+00_JPRB, 0.92234E+00_JPRB ,&
&  0.94262E+00_JPRB, 0.96288E+00_JPRB, 0.98325E+00_JPRB, 0.99297E+00_JPRB, 0.10047E+01_JPRB, 0.10158E+01_JPRB, 0.10255E+01_JPRB ,&
&  0.10190E+01_JPRB, 0.10179E+01_JPRB, 0.10251E+01_JPRB, 0.10402E+01_JPRB, 0.10630E+01_JPRB, 0.11012E+01_JPRB, 0.11453E+01_JPRB ,&
&  0.11925E+01_JPRB, 0.12314E+01_JPRB, 0.12895E+01_JPRB, 0.13654E+01_JPRB, 0.14561E+01_JPRB, 0.15655E+01_JPRB, 0.16752E+01_JPRB ,&
&  0.17809E+01_JPRB, 0.18669E+01_JPRB, 0.19457E+01_JPRB, 0.19791E+01_JPRB, 0.19917E+01_JPRB, 0.19989E+01_JPRB, 0.19920E+01_JPRB ,&
&  0.19811E+01_JPRB, 0.19612E+01_JPRB, 0.19415E+01_JPRB, 0.19142E+01_JPRB, 0.18801E+01_JPRB, 0.18358E+01_JPRB, 0.17860E+01_JPRB ,&
&  0.17284E+01_JPRB, 0.16731E+01_JPRB, 0.16191E+01_JPRB, 0.15674E+01_JPRB, 0.15156E+01_JPRB, 0.14682E+01_JPRB, 0.14255E+01_JPRB ,&
&  0.13856E+01_JPRB, 0.13447E+01_JPRB, 0.13073E+01_JPRB, 0.12713E+01_JPRB, 0.12359E+01_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.11255E+01_JPRB, 0.11046E+01_JPRB, 0.10854E+01_JPRB, 0.10677E+01_JPRB, 0.10529E+01_JPRB, 0.10409E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.10145E+01_JPRB, 0.10093E+01_JPRB, 0.10042E+01_JPRB, 0.99933E+00_JPRB, 0.99490E+00_JPRB, 0.99016E+00_JPRB ,&
&  0.98586E+00_JPRB, 0.98325E+00_JPRB, 0.98145E+00_JPRB, 0.97876E+00_JPRB, 0.97250E+00_JPRB, 0.96238E+00_JPRB, 0.95044E+00_JPRB ,&
&  0.93765E+00_JPRB, 0.92158E+00_JPRB, 0.90161E+00_JPRB, 0.87915E+00_JPRB, 0.86626E+00_JPRB, 0.85147E+00_JPRB, 0.83713E+00_JPRB ,&
&  0.82486E+00_JPRB, 0.82100E+00_JPRB, 0.81715E+00_JPRB, 0.81385E+00_JPRB, 0.81167E+00_JPRB, 0.81244E+00_JPRB, 0.81226E+00_JPRB ,&
&  0.81185E+00_JPRB, 0.81417E+00_JPRB, 0.82582E+00_JPRB, 0.83857E+00_JPRB, 0.85344E+00_JPRB, 0.87139E+00_JPRB, 0.89118E+00_JPRB ,&
&  0.90724E+00_JPRB, 0.91906E+00_JPRB, 0.93341E+00_JPRB, 0.94752E+00_JPRB, 0.95920E+00_JPRB, 0.96938E+00_JPRB, 0.97923E+00_JPRB ,&
&  0.98999E+00_JPRB, 0.99812E+00_JPRB, 0.10052E+01_JPRB, 0.10093E+01_JPRB, 0.10090E+01_JPRB, 0.10070E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.99648E+00_JPRB, 0.99623E+00_JPRB, 0.10012E+01_JPRB, 0.10076E+01_JPRB, 0.10194E+01_JPRB, 0.10360E+01_JPRB, 0.10558E+01_JPRB ,&
&  0.10812E+01_JPRB, 0.11132E+01_JPRB, 0.11525E+01_JPRB, 0.11956E+01_JPRB, 0.12449E+01_JPRB, 0.13055E+01_JPRB, 0.13543E+01_JPRB ,&
&  0.13978E+01_JPRB, 0.14341E+01_JPRB, 0.14656E+01_JPRB, 0.14854E+01_JPRB, 0.14977E+01_JPRB, 0.15082E+01_JPRB, 0.15046E+01_JPRB ,&
&  0.14973E+01_JPRB, 0.14891E+01_JPRB, 0.14818E+01_JPRB, 0.14684E+01_JPRB, 0.14525E+01_JPRB, 0.14364E+01_JPRB, 0.14208E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.13760E+01_JPRB, 0.13541E+01_JPRB, 0.13333E+01_JPRB, 0.13098E+01_JPRB, 0.12873E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12426E+01_JPRB, 0.12182E+01_JPRB, 0.11947E+01_JPRB, 0.11718E+01_JPRB, 0.11491E+01_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.10631E+01_JPRB, 0.10570E+01_JPRB, 0.10514E+01_JPRB, 0.10458E+01_JPRB, 0.10399E+01_JPRB, 0.10350E+01_JPRB, 0.10306E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.10237E+01_JPRB, 0.10221E+01_JPRB, 0.10211E+01_JPRB, 0.10209E+01_JPRB, 0.10204E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.10204E+01_JPRB, 0.10209E+01_JPRB, 0.10222E+01_JPRB, 0.10228E+01_JPRB, 0.10225E+01_JPRB, 0.10218E+01_JPRB, 0.10206E+01_JPRB ,&
&  0.10185E+01_JPRB, 0.10158E+01_JPRB, 0.10125E+01_JPRB, 0.10089E+01_JPRB, 0.10047E+01_JPRB, 0.99991E+00_JPRB, 0.99452E+00_JPRB ,&
&  0.98963E+00_JPRB, 0.98619E+00_JPRB, 0.98324E+00_JPRB, 0.98000E+00_JPRB, 0.97791E+00_JPRB, 0.97837E+00_JPRB, 0.97869E+00_JPRB ,&
&  0.97896E+00_JPRB, 0.97975E+00_JPRB, 0.98262E+00_JPRB, 0.98555E+00_JPRB, 0.98823E+00_JPRB, 0.99142E+00_JPRB, 0.99569E+00_JPRB ,&
&  0.99887E+00_JPRB, 0.10015E+01_JPRB, 0.10039E+01_JPRB, 0.10063E+01_JPRB, 0.10082E+01_JPRB, 0.10103E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10139E+01_JPRB, 0.10137E+01_JPRB, 0.10117E+01_JPRB, 0.10057E+01_JPRB, 0.10011E+01_JPRB, 0.99898E+00_JPRB ,&
&  0.99933E+00_JPRB, 0.10006E+01_JPRB, 0.10026E+01_JPRB, 0.10056E+01_JPRB, 0.10104E+01_JPRB, 0.10162E+01_JPRB, 0.10228E+01_JPRB ,&
&  0.10301E+01_JPRB, 0.10388E+01_JPRB, 0.10499E+01_JPRB, 0.10615E+01_JPRB, 0.10743E+01_JPRB, 0.10892E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.11169E+01_JPRB, 0.11315E+01_JPRB, 0.11438E+01_JPRB, 0.11546E+01_JPRB, 0.11650E+01_JPRB, 0.11748E+01_JPRB, 0.11817E+01_JPRB ,&
&  0.11842E+01_JPRB, 0.11859E+01_JPRB, 0.11869E+01_JPRB, 0.11844E+01_JPRB, 0.11793E+01_JPRB, 0.11751E+01_JPRB, 0.11706E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11585E+01_JPRB, 0.11521E+01_JPRB, 0.11453E+01_JPRB, 0.11363E+01_JPRB, 0.11265E+01_JPRB, 0.11174E+01_JPRB ,&
&  0.11082E+01_JPRB, 0.10982E+01_JPRB, 0.10888E+01_JPRB, 0.10798E+01_JPRB, 0.10713E+01_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB ,&
&  0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB, 0.10554E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB, 0.14035E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.14671E+00_JPRB, 0.14652E+00_JPRB, 0.14630E+00_JPRB, 0.14606E+00_JPRB, 0.14530E+00_JPRB, 0.14494E+00_JPRB, 0.14462E+00_JPRB ,&
&  0.14444E+00_JPRB, 0.14432E+00_JPRB, 0.14438E+00_JPRB, 0.14451E+00_JPRB, 0.14464E+00_JPRB, 0.14476E+00_JPRB, 0.14486E+00_JPRB ,&
&  0.14482E+00_JPRB, 0.14482E+00_JPRB, 0.14417E+00_JPRB, 0.14367E+00_JPRB, 0.14336E+00_JPRB, 0.14314E+00_JPRB, 0.14282E+00_JPRB ,&
&  0.14258E+00_JPRB, 0.14227E+00_JPRB, 0.14190E+00_JPRB, 0.14164E+00_JPRB, 0.14166E+00_JPRB, 0.14153E+00_JPRB, 0.14154E+00_JPRB ,&
&  0.14152E+00_JPRB, 0.14168E+00_JPRB, 0.14174E+00_JPRB, 0.14182E+00_JPRB, 0.14215E+00_JPRB, 0.14245E+00_JPRB, 0.14252E+00_JPRB ,&
&  0.14255E+00_JPRB, 0.14238E+00_JPRB, 0.14221E+00_JPRB, 0.14209E+00_JPRB, 0.14203E+00_JPRB, 0.14202E+00_JPRB, 0.14197E+00_JPRB ,&
&  0.14185E+00_JPRB, 0.14165E+00_JPRB, 0.14140E+00_JPRB, 0.14112E+00_JPRB, 0.14087E+00_JPRB, 0.14073E+00_JPRB, 0.14070E+00_JPRB ,&
&  0.14067E+00_JPRB, 0.14054E+00_JPRB, 0.14033E+00_JPRB, 0.14002E+00_JPRB, 0.13973E+00_JPRB, 0.13945E+00_JPRB, 0.13918E+00_JPRB ,&
&  0.13893E+00_JPRB, 0.13881E+00_JPRB, 0.13868E+00_JPRB, 0.13857E+00_JPRB, 0.13850E+00_JPRB, 0.13839E+00_JPRB, 0.13828E+00_JPRB ,&
&  0.13819E+00_JPRB, 0.13811E+00_JPRB, 0.13802E+00_JPRB, 0.13795E+00_JPRB, 0.13790E+00_JPRB, 0.13786E+00_JPRB, 0.13785E+00_JPRB ,&
&  0.13787E+00_JPRB, 0.13789E+00_JPRB, 0.13792E+00_JPRB, 0.13806E+00_JPRB, 0.13821E+00_JPRB, 0.13841E+00_JPRB, 0.13864E+00_JPRB ,&
&  0.13889E+00_JPRB, 0.13918E+00_JPRB, 0.13950E+00_JPRB, 0.13973E+00_JPRB, 0.14009E+00_JPRB, 0.14045E+00_JPRB, 0.14083E+00_JPRB ,&
&  0.14120E+00_JPRB, 0.14167E+00_JPRB, 0.14219E+00_JPRB, 0.14275E+00_JPRB, 0.14336E+00_JPRB, 0.14371E+00_JPRB, 0.14414E+00_JPRB ,&
&  0.14463E+00_JPRB, 0.14526E+00_JPRB, 0.14562E+00_JPRB, 0.14588E+00_JPRB, 0.14617E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.15720E+00_JPRB, 0.15918E+00_JPRB, 0.16057E+00_JPRB, 0.16225E+00_JPRB, 0.16137E+00_JPRB, 0.17306E+00_JPRB, 0.17104E+00_JPRB ,&
&  0.16987E+00_JPRB, 0.16513E+00_JPRB, 0.16477E+00_JPRB, 0.16313E+00_JPRB, 0.16114E+00_JPRB, 0.15894E+00_JPRB, 0.15691E+00_JPRB ,&
&  0.15503E+00_JPRB, 0.15295E+00_JPRB, 0.15145E+00_JPRB, 0.15023E+00_JPRB, 0.14913E+00_JPRB, 0.14812E+00_JPRB, 0.14733E+00_JPRB ,&
&  0.14729E+00_JPRB, 0.14783E+00_JPRB, 0.14886E+00_JPRB, 0.14874E+00_JPRB, 0.14845E+00_JPRB, 0.14806E+00_JPRB, 0.14786E+00_JPRB ,&
&  0.14731E+00_JPRB, 0.14690E+00_JPRB, 0.14655E+00_JPRB, 0.14601E+00_JPRB, 0.14662E+00_JPRB, 0.14721E+00_JPRB, 0.14781E+00_JPRB ,&
&  0.14839E+00_JPRB, 0.14897E+00_JPRB, 0.14897E+00_JPRB, 0.14885E+00_JPRB, 0.14847E+00_JPRB, 0.14793E+00_JPRB, 0.14708E+00_JPRB ,&
&  0.14627E+00_JPRB, 0.14541E+00_JPRB, 0.14450E+00_JPRB, 0.14388E+00_JPRB, 0.14333E+00_JPRB, 0.14281E+00_JPRB, 0.14236E+00_JPRB ,&
&  0.14214E+00_JPRB, 0.14199E+00_JPRB, 0.14181E+00_JPRB, 0.14152E+00_JPRB, 0.14116E+00_JPRB, 0.14079E+00_JPRB, 0.14046E+00_JPRB ,&
&  0.14021E+00_JPRB, 0.13974E+00_JPRB, 0.13932E+00_JPRB, 0.13887E+00_JPRB, 0.13853E+00_JPRB, 0.13809E+00_JPRB, 0.13773E+00_JPRB ,&
&  0.13738E+00_JPRB, 0.13713E+00_JPRB, 0.13705E+00_JPRB, 0.13701E+00_JPRB, 0.13697E+00_JPRB, 0.13701E+00_JPRB, 0.13713E+00_JPRB ,&
&  0.13730E+00_JPRB, 0.13750E+00_JPRB, 0.13776E+00_JPRB, 0.13794E+00_JPRB, 0.13812E+00_JPRB, 0.13827E+00_JPRB, 0.13865E+00_JPRB ,&
&  0.13911E+00_JPRB, 0.13960E+00_JPRB, 0.14006E+00_JPRB, 0.14027E+00_JPRB, 0.14106E+00_JPRB, 0.14188E+00_JPRB, 0.14277E+00_JPRB ,&
&  0.14372E+00_JPRB, 0.14495E+00_JPRB, 0.14628E+00_JPRB, 0.14769E+00_JPRB, 0.14932E+00_JPRB, 0.15040E+00_JPRB, 0.15158E+00_JPRB ,&
&  0.15317E+00_JPRB, 0.15580E+00_JPRB, 0.15673E+00_JPRB, 0.15709E+00_JPRB, 0.15729E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.18264E+00_JPRB, 0.18330E+00_JPRB, 0.18391E+00_JPRB, 0.18414E+00_JPRB, 0.18419E+00_JPRB, 0.18344E+00_JPRB, 0.18251E+00_JPRB ,&
&  0.17875E+00_JPRB, 0.17601E+00_JPRB, 0.17299E+00_JPRB, 0.17000E+00_JPRB, 0.16738E+00_JPRB, 0.16473E+00_JPRB, 0.16233E+00_JPRB ,&
&  0.16015E+00_JPRB, 0.15852E+00_JPRB, 0.15700E+00_JPRB, 0.15494E+00_JPRB, 0.15232E+00_JPRB, 0.14991E+00_JPRB, 0.14837E+00_JPRB ,&
&  0.14791E+00_JPRB, 0.14820E+00_JPRB, 0.14957E+00_JPRB, 0.15099E+00_JPRB, 0.15251E+00_JPRB, 0.15366E+00_JPRB, 0.15481E+00_JPRB ,&
&  0.15425E+00_JPRB, 0.15389E+00_JPRB, 0.15357E+00_JPRB, 0.15300E+00_JPRB, 0.15358E+00_JPRB, 0.15437E+00_JPRB, 0.15483E+00_JPRB ,&
&  0.15537E+00_JPRB, 0.15679E+00_JPRB, 0.15782E+00_JPRB, 0.15878E+00_JPRB, 0.15869E+00_JPRB, 0.15792E+00_JPRB, 0.15647E+00_JPRB ,&
&  0.15465E+00_JPRB, 0.15277E+00_JPRB, 0.15111E+00_JPRB, 0.14968E+00_JPRB, 0.14859E+00_JPRB, 0.14737E+00_JPRB, 0.14636E+00_JPRB ,&
&  0.14587E+00_JPRB, 0.14542E+00_JPRB, 0.14477E+00_JPRB, 0.14411E+00_JPRB, 0.14309E+00_JPRB, 0.14221E+00_JPRB, 0.14128E+00_JPRB ,&
&  0.14044E+00_JPRB, 0.13964E+00_JPRB, 0.13891E+00_JPRB, 0.13815E+00_JPRB, 0.13763E+00_JPRB, 0.13716E+00_JPRB, 0.13670E+00_JPRB ,&
&  0.13622E+00_JPRB, 0.13605E+00_JPRB, 0.13570E+00_JPRB, 0.13558E+00_JPRB, 0.13537E+00_JPRB, 0.13524E+00_JPRB, 0.13542E+00_JPRB ,&
&  0.13593E+00_JPRB, 0.13644E+00_JPRB, 0.13670E+00_JPRB, 0.13702E+00_JPRB, 0.13733E+00_JPRB, 0.13774E+00_JPRB, 0.13841E+00_JPRB ,&
&  0.13892E+00_JPRB, 0.13956E+00_JPRB, 0.14009E+00_JPRB, 0.14074E+00_JPRB, 0.14195E+00_JPRB, 0.14319E+00_JPRB, 0.14463E+00_JPRB ,&
&  0.14641E+00_JPRB, 0.14829E+00_JPRB, 0.15051E+00_JPRB, 0.15268E+00_JPRB, 0.15554E+00_JPRB, 0.15727E+00_JPRB, 0.16051E+00_JPRB ,&
&  0.16410E+00_JPRB, 0.16539E+00_JPRB, 0.17734E+00_JPRB, 0.17851E+00_JPRB, 0.18014E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.18956E+00_JPRB, 0.19072E+00_JPRB, 0.19046E+00_JPRB, 0.19071E+00_JPRB, 0.18962E+00_JPRB, 0.18797E+00_JPRB, 0.18560E+00_JPRB ,&
&  0.18298E+00_JPRB, 0.17736E+00_JPRB, 0.17343E+00_JPRB, 0.16907E+00_JPRB, 0.16539E+00_JPRB, 0.16217E+00_JPRB, 0.15989E+00_JPRB ,&
&  0.15888E+00_JPRB, 0.15854E+00_JPRB, 0.15836E+00_JPRB, 0.15750E+00_JPRB, 0.15595E+00_JPRB, 0.15491E+00_JPRB, 0.15359E+00_JPRB ,&
&  0.15262E+00_JPRB, 0.15255E+00_JPRB, 0.15337E+00_JPRB, 0.15517E+00_JPRB, 0.15836E+00_JPRB, 0.16069E+00_JPRB, 0.16279E+00_JPRB ,&
&  0.16493E+00_JPRB, 0.16149E+00_JPRB, 0.16130E+00_JPRB, 0.16164E+00_JPRB, 0.16156E+00_JPRB, 0.16087E+00_JPRB, 0.15958E+00_JPRB ,&
&  0.16246E+00_JPRB, 0.16321E+00_JPRB, 0.16809E+00_JPRB, 0.16792E+00_JPRB, 0.16690E+00_JPRB, 0.16794E+00_JPRB, 0.16648E+00_JPRB ,&
&  0.16604E+00_JPRB, 0.16298E+00_JPRB, 0.16072E+00_JPRB, 0.15759E+00_JPRB, 0.15451E+00_JPRB, 0.15297E+00_JPRB, 0.15121E+00_JPRB ,&
&  0.15066E+00_JPRB, 0.15055E+00_JPRB, 0.14924E+00_JPRB, 0.14843E+00_JPRB, 0.14692E+00_JPRB, 0.14520E+00_JPRB, 0.14412E+00_JPRB ,&
&  0.14302E+00_JPRB, 0.14213E+00_JPRB, 0.14165E+00_JPRB, 0.14119E+00_JPRB, 0.14100E+00_JPRB, 0.14091E+00_JPRB, 0.14081E+00_JPRB ,&
&  0.14083E+00_JPRB, 0.14075E+00_JPRB, 0.14022E+00_JPRB, 0.13993E+00_JPRB, 0.13897E+00_JPRB, 0.13831E+00_JPRB, 0.13778E+00_JPRB ,&
&  0.13731E+00_JPRB, 0.13715E+00_JPRB, 0.13706E+00_JPRB, 0.13710E+00_JPRB, 0.13721E+00_JPRB, 0.13763E+00_JPRB, 0.13810E+00_JPRB ,&
&  0.13859E+00_JPRB, 0.13947E+00_JPRB, 0.14014E+00_JPRB, 0.14111E+00_JPRB, 0.14226E+00_JPRB, 0.14324E+00_JPRB, 0.14481E+00_JPRB ,&
&  0.14655E+00_JPRB, 0.14859E+00_JPRB, 0.15159E+00_JPRB, 0.15436E+00_JPRB, 0.15759E+00_JPRB, 0.16203E+00_JPRB, 0.16688E+00_JPRB ,&
&  0.17219E+00_JPRB, 0.17836E+00_JPRB, 0.17902E+00_JPRB, 0.18430E+00_JPRB, 0.18771E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.19015E+00_JPRB, 0.19193E+00_JPRB, 0.19260E+00_JPRB, 0.19352E+00_JPRB, 0.19325E+00_JPRB, 0.19087E+00_JPRB, 0.18845E+00_JPRB ,&
&  0.18312E+00_JPRB, 0.17885E+00_JPRB, 0.17367E+00_JPRB, 0.16884E+00_JPRB, 0.16426E+00_JPRB, 0.16133E+00_JPRB, 0.15960E+00_JPRB ,&
&  0.15958E+00_JPRB, 0.16100E+00_JPRB, 0.16205E+00_JPRB, 0.16265E+00_JPRB, 0.16305E+00_JPRB, 0.16403E+00_JPRB, 0.16415E+00_JPRB ,&
&  0.16372E+00_JPRB, 0.16266E+00_JPRB, 0.16188E+00_JPRB, 0.16276E+00_JPRB, 0.16475E+00_JPRB, 0.16681E+00_JPRB, 0.16852E+00_JPRB ,&
&  0.17003E+00_JPRB, 0.16938E+00_JPRB, 0.16939E+00_JPRB, 0.16861E+00_JPRB, 0.16811E+00_JPRB, 0.16750E+00_JPRB, 0.16735E+00_JPRB ,&
&  0.16816E+00_JPRB, 0.16882E+00_JPRB, 0.17063E+00_JPRB, 0.17159E+00_JPRB, 0.17043E+00_JPRB, 0.17232E+00_JPRB, 0.17248E+00_JPRB ,&
&  0.17170E+00_JPRB, 0.16867E+00_JPRB, 0.16540E+00_JPRB, 0.16284E+00_JPRB, 0.16017E+00_JPRB, 0.15803E+00_JPRB, 0.15627E+00_JPRB ,&
&  0.15465E+00_JPRB, 0.15379E+00_JPRB, 0.15198E+00_JPRB, 0.15045E+00_JPRB, 0.14947E+00_JPRB, 0.14844E+00_JPRB, 0.14778E+00_JPRB ,&
&  0.14734E+00_JPRB, 0.14661E+00_JPRB, 0.14645E+00_JPRB, 0.14606E+00_JPRB, 0.14615E+00_JPRB, 0.14629E+00_JPRB, 0.14663E+00_JPRB ,&
&  0.14740E+00_JPRB, 0.14833E+00_JPRB, 0.14824E+00_JPRB, 0.14811E+00_JPRB, 0.14666E+00_JPRB, 0.14506E+00_JPRB, 0.14369E+00_JPRB ,&
&  0.14189E+00_JPRB, 0.14056E+00_JPRB, 0.13974E+00_JPRB, 0.13893E+00_JPRB, 0.13881E+00_JPRB, 0.13884E+00_JPRB, 0.13880E+00_JPRB ,&
&  0.13898E+00_JPRB, 0.13925E+00_JPRB, 0.13969E+00_JPRB, 0.14063E+00_JPRB, 0.14145E+00_JPRB, 0.14264E+00_JPRB, 0.14394E+00_JPRB ,&
&  0.14498E+00_JPRB, 0.14690E+00_JPRB, 0.14895E+00_JPRB, 0.15139E+00_JPRB, 0.15606E+00_JPRB, 0.16013E+00_JPRB, 0.16747E+00_JPRB ,&
&  0.17322E+00_JPRB, 0.18243E+00_JPRB, 0.18054E+00_JPRB, 0.18453E+00_JPRB, 0.18818E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.19261E+00_JPRB, 0.19379E+00_JPRB, 0.19353E+00_JPRB, 0.19369E+00_JPRB, 0.19325E+00_JPRB, 0.19427E+00_JPRB, 0.19344E+00_JPRB ,&
&  0.19338E+00_JPRB, 0.19231E+00_JPRB, 0.19130E+00_JPRB, 0.18873E+00_JPRB, 0.18737E+00_JPRB, 0.18134E+00_JPRB, 0.17972E+00_JPRB ,&
&  0.17835E+00_JPRB, 0.17917E+00_JPRB, 0.17939E+00_JPRB, 0.17988E+00_JPRB, 0.18075E+00_JPRB, 0.18235E+00_JPRB, 0.18233E+00_JPRB ,&
&  0.18123E+00_JPRB, 0.17919E+00_JPRB, 0.17682E+00_JPRB, 0.17466E+00_JPRB, 0.17505E+00_JPRB, 0.17471E+00_JPRB, 0.17376E+00_JPRB ,&
&  0.17279E+00_JPRB, 0.17339E+00_JPRB, 0.17389E+00_JPRB, 0.17524E+00_JPRB, 0.18018E+00_JPRB, 0.17858E+00_JPRB, 0.17741E+00_JPRB ,&
&  0.17730E+00_JPRB, 0.17728E+00_JPRB, 0.17789E+00_JPRB, 0.17837E+00_JPRB, 0.18043E+00_JPRB, 0.17992E+00_JPRB, 0.18255E+00_JPRB ,&
&  0.18012E+00_JPRB, 0.17859E+00_JPRB, 0.17644E+00_JPRB, 0.17357E+00_JPRB, 0.16982E+00_JPRB, 0.16744E+00_JPRB, 0.16518E+00_JPRB ,&
&  0.16269E+00_JPRB, 0.16059E+00_JPRB, 0.15789E+00_JPRB, 0.15692E+00_JPRB, 0.15550E+00_JPRB, 0.15423E+00_JPRB, 0.15307E+00_JPRB ,&
&  0.15219E+00_JPRB, 0.15098E+00_JPRB, 0.14993E+00_JPRB, 0.14924E+00_JPRB, 0.14902E+00_JPRB, 0.14915E+00_JPRB, 0.15019E+00_JPRB ,&
&  0.15216E+00_JPRB, 0.15376E+00_JPRB, 0.15502E+00_JPRB, 0.15612E+00_JPRB, 0.15471E+00_JPRB, 0.15214E+00_JPRB, 0.14958E+00_JPRB ,&
&  0.14612E+00_JPRB, 0.14421E+00_JPRB, 0.14241E+00_JPRB, 0.14134E+00_JPRB, 0.14105E+00_JPRB, 0.14099E+00_JPRB, 0.14126E+00_JPRB ,&
&  0.14153E+00_JPRB, 0.14198E+00_JPRB, 0.14229E+00_JPRB, 0.14271E+00_JPRB, 0.14326E+00_JPRB, 0.14381E+00_JPRB, 0.14462E+00_JPRB ,&
&  0.14594E+00_JPRB, 0.14762E+00_JPRB, 0.14993E+00_JPRB, 0.15263E+00_JPRB, 0.15579E+00_JPRB, 0.15990E+00_JPRB, 0.16592E+00_JPRB ,&
&  0.17211E+00_JPRB, 0.18231E+00_JPRB, 0.18331E+00_JPRB, 0.18868E+00_JPRB, 0.19177E+00_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.19555E+00_JPRB, 0.19740E+00_JPRB, 0.19806E+00_JPRB, 0.19934E+00_JPRB, 0.19979E+00_JPRB, 0.20108E+00_JPRB, 0.20208E+00_JPRB ,&
&  0.20449E+00_JPRB, 0.20667E+00_JPRB, 0.20723E+00_JPRB, 0.20553E+00_JPRB, 0.20449E+00_JPRB, 0.20168E+00_JPRB, 0.20059E+00_JPRB ,&
&  0.20032E+00_JPRB, 0.20136E+00_JPRB, 0.20154E+00_JPRB, 0.20161E+00_JPRB, 0.20156E+00_JPRB, 0.20151E+00_JPRB, 0.20198E+00_JPRB ,&
&  0.20002E+00_JPRB, 0.19924E+00_JPRB, 0.20132E+00_JPRB, 0.20052E+00_JPRB, 0.19987E+00_JPRB, 0.19853E+00_JPRB, 0.19781E+00_JPRB ,&
&  0.19379E+00_JPRB, 0.18805E+00_JPRB, 0.18147E+00_JPRB, 0.18451E+00_JPRB, 0.19170E+00_JPRB, 0.18940E+00_JPRB, 0.18868E+00_JPRB ,&
&  0.18815E+00_JPRB, 0.18704E+00_JPRB, 0.18666E+00_JPRB, 0.18627E+00_JPRB, 0.18693E+00_JPRB, 0.18759E+00_JPRB, 0.18926E+00_JPRB ,&
&  0.19158E+00_JPRB, 0.19492E+00_JPRB, 0.19480E+00_JPRB, 0.18851E+00_JPRB, 0.18542E+00_JPRB, 0.18438E+00_JPRB, 0.18114E+00_JPRB ,&
&  0.17731E+00_JPRB, 0.17587E+00_JPRB, 0.17491E+00_JPRB, 0.17244E+00_JPRB, 0.17167E+00_JPRB, 0.17119E+00_JPRB, 0.16874E+00_JPRB ,&
&  0.16552E+00_JPRB, 0.16313E+00_JPRB, 0.15976E+00_JPRB, 0.15631E+00_JPRB, 0.15391E+00_JPRB, 0.15277E+00_JPRB, 0.15284E+00_JPRB ,&
&  0.15462E+00_JPRB, 0.15647E+00_JPRB, 0.16221E+00_JPRB, 0.16590E+00_JPRB, 0.16280E+00_JPRB, 0.15960E+00_JPRB, 0.15614E+00_JPRB ,&
&  0.15091E+00_JPRB, 0.14844E+00_JPRB, 0.14646E+00_JPRB, 0.14502E+00_JPRB, 0.14486E+00_JPRB, 0.14559E+00_JPRB, 0.14615E+00_JPRB ,&
&  0.14688E+00_JPRB, 0.14789E+00_JPRB, 0.14837E+00_JPRB, 0.14859E+00_JPRB, 0.14863E+00_JPRB, 0.14887E+00_JPRB, 0.14947E+00_JPRB ,&
&  0.15024E+00_JPRB, 0.15207E+00_JPRB, 0.15466E+00_JPRB, 0.15744E+00_JPRB, 0.15938E+00_JPRB, 0.16091E+00_JPRB, 0.16414E+00_JPRB ,&
&  0.16785E+00_JPRB, 0.17652E+00_JPRB, 0.18874E+00_JPRB, 0.19777E+00_JPRB, 0.19428E+00_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.20992E+00_JPRB, 0.21099E+00_JPRB, 0.21217E+00_JPRB, 0.21348E+00_JPRB, 0.21498E+00_JPRB, 0.21661E+00_JPRB, 0.21799E+00_JPRB ,&
&  0.21950E+00_JPRB, 0.22046E+00_JPRB, 0.22067E+00_JPRB, 0.21960E+00_JPRB, 0.21876E+00_JPRB, 0.21884E+00_JPRB, 0.21973E+00_JPRB ,&
&  0.21788E+00_JPRB, 0.21502E+00_JPRB, 0.21382E+00_JPRB, 0.21361E+00_JPRB, 0.21299E+00_JPRB, 0.21265E+00_JPRB, 0.21338E+00_JPRB ,&
&  0.21430E+00_JPRB, 0.21395E+00_JPRB, 0.21360E+00_JPRB, 0.21250E+00_JPRB, 0.21162E+00_JPRB, 0.21093E+00_JPRB, 0.20995E+00_JPRB ,&
&  0.20668E+00_JPRB, 0.19680E+00_JPRB, 0.19037E+00_JPRB, 0.19375E+00_JPRB, 0.20064E+00_JPRB, 0.19997E+00_JPRB, 0.20000E+00_JPRB ,&
&  0.19953E+00_JPRB, 0.19821E+00_JPRB, 0.19674E+00_JPRB, 0.19536E+00_JPRB, 0.19481E+00_JPRB, 0.19513E+00_JPRB, 0.19599E+00_JPRB ,&
&  0.19692E+00_JPRB, 0.19901E+00_JPRB, 0.20140E+00_JPRB, 0.20248E+00_JPRB, 0.20356E+00_JPRB, 0.20409E+00_JPRB, 0.20427E+00_JPRB ,&
&  0.20388E+00_JPRB, 0.20287E+00_JPRB, 0.20217E+00_JPRB, 0.20143E+00_JPRB, 0.20071E+00_JPRB, 0.19955E+00_JPRB, 0.19585E+00_JPRB ,&
&  0.19055E+00_JPRB, 0.18535E+00_JPRB, 0.17904E+00_JPRB, 0.17271E+00_JPRB, 0.16703E+00_JPRB, 0.16290E+00_JPRB, 0.15949E+00_JPRB ,&
&  0.15900E+00_JPRB, 0.16272E+00_JPRB, 0.17044E+00_JPRB, 0.17986E+00_JPRB, 0.18545E+00_JPRB, 0.17555E+00_JPRB, 0.16947E+00_JPRB ,&
&  0.16375E+00_JPRB, 0.15917E+00_JPRB, 0.15635E+00_JPRB, 0.15483E+00_JPRB, 0.15380E+00_JPRB, 0.15379E+00_JPRB, 0.15439E+00_JPRB ,&
&  0.15569E+00_JPRB, 0.15712E+00_JPRB, 0.15845E+00_JPRB, 0.15928E+00_JPRB, 0.15959E+00_JPRB, 0.15943E+00_JPRB, 0.15904E+00_JPRB ,&
&  0.15950E+00_JPRB, 0.16178E+00_JPRB, 0.16589E+00_JPRB, 0.17075E+00_JPRB, 0.17486E+00_JPRB, 0.17874E+00_JPRB, 0.18411E+00_JPRB ,&
&  0.19090E+00_JPRB, 0.19640E+00_JPRB, 0.20090E+00_JPRB, 0.20566E+00_JPRB, 0.20826E+00_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.22913E+00_JPRB, 0.23040E+00_JPRB, 0.23156E+00_JPRB, 0.23269E+00_JPRB, 0.23455E+00_JPRB, 0.23597E+00_JPRB, 0.23763E+00_JPRB ,&
&  0.23883E+00_JPRB, 0.23748E+00_JPRB, 0.23564E+00_JPRB, 0.23413E+00_JPRB, 0.23345E+00_JPRB, 0.23270E+00_JPRB, 0.23216E+00_JPRB ,&
&  0.23285E+00_JPRB, 0.23211E+00_JPRB, 0.22943E+00_JPRB, 0.22747E+00_JPRB, 0.22427E+00_JPRB, 0.22215E+00_JPRB, 0.22172E+00_JPRB ,&
&  0.22329E+00_JPRB, 0.22305E+00_JPRB, 0.22246E+00_JPRB, 0.22151E+00_JPRB, 0.22020E+00_JPRB, 0.21956E+00_JPRB, 0.21850E+00_JPRB ,&
&  0.21794E+00_JPRB, 0.21701E+00_JPRB, 0.21237E+00_JPRB, 0.21299E+00_JPRB, 0.21261E+00_JPRB, 0.21256E+00_JPRB, 0.21202E+00_JPRB ,&
&  0.21139E+00_JPRB, 0.21038E+00_JPRB, 0.20857E+00_JPRB, 0.20660E+00_JPRB, 0.20495E+00_JPRB, 0.20407E+00_JPRB, 0.20399E+00_JPRB ,&
&  0.20434E+00_JPRB, 0.20525E+00_JPRB, 0.20679E+00_JPRB, 0.20881E+00_JPRB, 0.20967E+00_JPRB, 0.21029E+00_JPRB, 0.21126E+00_JPRB ,&
&  0.21169E+00_JPRB, 0.21179E+00_JPRB, 0.21183E+00_JPRB, 0.21181E+00_JPRB, 0.21154E+00_JPRB, 0.21073E+00_JPRB, 0.21005E+00_JPRB ,&
&  0.20844E+00_JPRB, 0.20606E+00_JPRB, 0.20446E+00_JPRB, 0.20248E+00_JPRB, 0.19703E+00_JPRB, 0.18682E+00_JPRB, 0.18112E+00_JPRB ,&
&  0.18125E+00_JPRB, 0.18588E+00_JPRB, 0.19154E+00_JPRB, 0.19475E+00_JPRB, 0.19986E+00_JPRB, 0.20001E+00_JPRB, 0.19640E+00_JPRB ,&
&  0.18821E+00_JPRB, 0.18285E+00_JPRB, 0.17892E+00_JPRB, 0.17469E+00_JPRB, 0.17209E+00_JPRB, 0.17156E+00_JPRB, 0.17201E+00_JPRB ,&
&  0.17398E+00_JPRB, 0.17680E+00_JPRB, 0.17962E+00_JPRB, 0.18119E+00_JPRB, 0.18133E+00_JPRB, 0.18003E+00_JPRB, 0.17849E+00_JPRB ,&
&  0.17813E+00_JPRB, 0.18206E+00_JPRB, 0.19025E+00_JPRB, 0.19933E+00_JPRB, 0.20352E+00_JPRB, 0.20513E+00_JPRB, 0.20680E+00_JPRB ,&
&  0.21038E+00_JPRB, 0.21530E+00_JPRB, 0.22027E+00_JPRB, 0.22383E+00_JPRB, 0.22680E+00_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.24906E+00_JPRB, 0.25024E+00_JPRB, 0.25143E+00_JPRB, 0.25285E+00_JPRB, 0.25464E+00_JPRB, 0.25569E+00_JPRB, 0.25649E+00_JPRB ,&
&  0.25580E+00_JPRB, 0.25383E+00_JPRB, 0.25150E+00_JPRB, 0.24905E+00_JPRB, 0.24788E+00_JPRB, 0.24763E+00_JPRB, 0.24763E+00_JPRB ,&
&  0.24703E+00_JPRB, 0.24671E+00_JPRB, 0.24484E+00_JPRB, 0.24178E+00_JPRB, 0.23824E+00_JPRB, 0.23583E+00_JPRB, 0.23511E+00_JPRB ,&
&  0.23456E+00_JPRB, 0.23373E+00_JPRB, 0.23337E+00_JPRB, 0.23296E+00_JPRB, 0.23091E+00_JPRB, 0.22987E+00_JPRB, 0.23028E+00_JPRB ,&
&  0.23112E+00_JPRB, 0.23261E+00_JPRB, 0.23289E+00_JPRB, 0.23044E+00_JPRB, 0.23271E+00_JPRB, 0.23250E+00_JPRB, 0.23068E+00_JPRB ,&
&  0.22832E+00_JPRB, 0.22585E+00_JPRB, 0.22362E+00_JPRB, 0.22141E+00_JPRB, 0.21919E+00_JPRB, 0.21751E+00_JPRB, 0.21645E+00_JPRB ,&
&  0.21617E+00_JPRB, 0.21634E+00_JPRB, 0.21673E+00_JPRB, 0.21757E+00_JPRB, 0.21817E+00_JPRB, 0.21850E+00_JPRB, 0.21852E+00_JPRB ,&
&  0.21861E+00_JPRB, 0.21880E+00_JPRB, 0.21919E+00_JPRB, 0.21944E+00_JPRB, 0.21973E+00_JPRB, 0.21998E+00_JPRB, 0.21988E+00_JPRB ,&
&  0.21881E+00_JPRB, 0.21704E+00_JPRB, 0.21470E+00_JPRB, 0.21386E+00_JPRB, 0.21382E+00_JPRB, 0.21234E+00_JPRB, 0.20996E+00_JPRB ,&
&  0.20767E+00_JPRB, 0.20716E+00_JPRB, 0.20828E+00_JPRB, 0.20952E+00_JPRB, 0.21050E+00_JPRB, 0.21085E+00_JPRB, 0.21060E+00_JPRB ,&
&  0.20963E+00_JPRB, 0.20740E+00_JPRB, 0.20463E+00_JPRB, 0.20242E+00_JPRB, 0.20117E+00_JPRB, 0.20107E+00_JPRB, 0.20199E+00_JPRB ,&
&  0.20214E+00_JPRB, 0.20215E+00_JPRB, 0.20315E+00_JPRB, 0.20400E+00_JPRB, 0.20460E+00_JPRB, 0.20536E+00_JPRB, 0.20678E+00_JPRB ,&
&  0.20875E+00_JPRB, 0.21193E+00_JPRB, 0.21585E+00_JPRB, 0.22010E+00_JPRB, 0.22301E+00_JPRB, 0.22223E+00_JPRB, 0.22291E+00_JPRB ,&
&  0.22768E+00_JPRB, 0.23352E+00_JPRB, 0.23845E+00_JPRB, 0.24247E+00_JPRB, 0.24620E+00_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.27312E+00_JPRB, 0.27524E+00_JPRB, 0.27734E+00_JPRB, 0.27957E+00_JPRB, 0.28115E+00_JPRB, 0.28171E+00_JPRB, 0.28030E+00_JPRB ,&
&  0.27741E+00_JPRB, 0.27403E+00_JPRB, 0.27060E+00_JPRB, 0.26760E+00_JPRB, 0.26566E+00_JPRB, 0.26440E+00_JPRB, 0.26355E+00_JPRB ,&
&  0.26282E+00_JPRB, 0.26208E+00_JPRB, 0.25993E+00_JPRB, 0.25694E+00_JPRB, 0.25310E+00_JPRB, 0.24984E+00_JPRB, 0.24887E+00_JPRB ,&
&  0.24729E+00_JPRB, 0.24628E+00_JPRB, 0.24555E+00_JPRB, 0.24477E+00_JPRB, 0.24450E+00_JPRB, 0.24553E+00_JPRB, 0.24740E+00_JPRB ,&
&  0.24938E+00_JPRB, 0.25098E+00_JPRB, 0.25224E+00_JPRB, 0.25311E+00_JPRB, 0.25258E+00_JPRB, 0.25153E+00_JPRB, 0.25029E+00_JPRB ,&
&  0.24880E+00_JPRB, 0.24740E+00_JPRB, 0.24620E+00_JPRB, 0.24453E+00_JPRB, 0.24202E+00_JPRB, 0.23897E+00_JPRB, 0.23681E+00_JPRB ,&
&  0.23512E+00_JPRB, 0.23351E+00_JPRB, 0.23181E+00_JPRB, 0.23088E+00_JPRB, 0.23020E+00_JPRB, 0.22992E+00_JPRB, 0.22997E+00_JPRB ,&
&  0.23039E+00_JPRB, 0.23161E+00_JPRB, 0.23331E+00_JPRB, 0.23516E+00_JPRB, 0.23568E+00_JPRB, 0.23582E+00_JPRB, 0.23601E+00_JPRB ,&
&  0.23531E+00_JPRB, 0.23397E+00_JPRB, 0.23220E+00_JPRB, 0.23029E+00_JPRB, 0.22903E+00_JPRB, 0.22806E+00_JPRB, 0.22696E+00_JPRB ,&
&  0.22505E+00_JPRB, 0.22387E+00_JPRB, 0.22322E+00_JPRB, 0.22308E+00_JPRB, 0.22330E+00_JPRB, 0.22335E+00_JPRB, 0.22264E+00_JPRB ,&
&  0.22176E+00_JPRB, 0.22099E+00_JPRB, 0.21992E+00_JPRB, 0.21876E+00_JPRB, 0.21694E+00_JPRB, 0.21721E+00_JPRB, 0.21816E+00_JPRB ,&
&  0.21862E+00_JPRB, 0.21881E+00_JPRB, 0.21898E+00_JPRB, 0.22004E+00_JPRB, 0.22122E+00_JPRB, 0.22289E+00_JPRB, 0.22531E+00_JPRB ,&
&  0.22734E+00_JPRB, 0.23006E+00_JPRB, 0.23408E+00_JPRB, 0.23780E+00_JPRB, 0.24051E+00_JPRB, 0.24193E+00_JPRB, 0.24478E+00_JPRB ,&
&  0.25105E+00_JPRB, 0.25810E+00_JPRB, 0.26294E+00_JPRB, 0.26747E+00_JPRB, 0.27133E+00_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.29867E+00_JPRB, 0.30424E+00_JPRB, 0.30865E+00_JPRB, 0.31117E+00_JPRB, 0.31133E+00_JPRB, 0.31000E+00_JPRB, 0.30698E+00_JPRB ,&
&  0.30256E+00_JPRB, 0.29801E+00_JPRB, 0.29341E+00_JPRB, 0.28958E+00_JPRB, 0.28704E+00_JPRB, 0.28488E+00_JPRB, 0.28308E+00_JPRB ,&
&  0.28084E+00_JPRB, 0.27823E+00_JPRB, 0.27513E+00_JPRB, 0.27241E+00_JPRB, 0.26930E+00_JPRB, 0.26508E+00_JPRB, 0.26201E+00_JPRB ,&
&  0.25989E+00_JPRB, 0.25792E+00_JPRB, 0.25734E+00_JPRB, 0.25863E+00_JPRB, 0.26109E+00_JPRB, 0.26345E+00_JPRB, 0.26603E+00_JPRB ,&
&  0.26784E+00_JPRB, 0.26884E+00_JPRB, 0.26964E+00_JPRB, 0.26954E+00_JPRB, 0.26954E+00_JPRB, 0.26929E+00_JPRB, 0.26870E+00_JPRB ,&
&  0.26814E+00_JPRB, 0.26780E+00_JPRB, 0.26692E+00_JPRB, 0.26698E+00_JPRB, 0.26694E+00_JPRB, 0.26578E+00_JPRB, 0.26358E+00_JPRB ,&
&  0.25999E+00_JPRB, 0.25510E+00_JPRB, 0.25122E+00_JPRB, 0.24927E+00_JPRB, 0.24863E+00_JPRB, 0.24888E+00_JPRB, 0.24973E+00_JPRB ,&
&  0.25085E+00_JPRB, 0.25243E+00_JPRB, 0.25388E+00_JPRB, 0.25454E+00_JPRB, 0.25429E+00_JPRB, 0.25403E+00_JPRB, 0.25273E+00_JPRB ,&
&  0.25146E+00_JPRB, 0.25063E+00_JPRB, 0.25003E+00_JPRB, 0.24937E+00_JPRB, 0.24722E+00_JPRB, 0.24458E+00_JPRB, 0.24396E+00_JPRB ,&
&  0.24248E+00_JPRB, 0.24152E+00_JPRB, 0.24016E+00_JPRB, 0.23949E+00_JPRB, 0.23869E+00_JPRB, 0.23746E+00_JPRB, 0.23611E+00_JPRB ,&
&  0.23500E+00_JPRB, 0.23438E+00_JPRB, 0.23396E+00_JPRB, 0.23303E+00_JPRB, 0.23326E+00_JPRB, 0.23376E+00_JPRB, 0.23508E+00_JPRB ,&
&  0.23628E+00_JPRB, 0.23748E+00_JPRB, 0.23904E+00_JPRB, 0.24051E+00_JPRB, 0.24279E+00_JPRB, 0.24550E+00_JPRB, 0.24860E+00_JPRB ,&
&  0.25126E+00_JPRB, 0.25498E+00_JPRB, 0.25927E+00_JPRB, 0.26328E+00_JPRB, 0.26650E+00_JPRB, 0.26913E+00_JPRB, 0.27312E+00_JPRB ,&
&  0.27958E+00_JPRB, 0.28528E+00_JPRB, 0.28865E+00_JPRB, 0.29170E+00_JPRB, 0.29425E+00_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.33357E+00_JPRB, 0.33904E+00_JPRB, 0.34209E+00_JPRB, 0.34340E+00_JPRB, 0.34372E+00_JPRB, 0.34193E+00_JPRB, 0.33875E+00_JPRB ,&
&  0.33384E+00_JPRB, 0.32889E+00_JPRB, 0.32352E+00_JPRB, 0.31759E+00_JPRB, 0.31235E+00_JPRB, 0.30795E+00_JPRB, 0.30415E+00_JPRB ,&
&  0.30042E+00_JPRB, 0.29701E+00_JPRB, 0.29420E+00_JPRB, 0.29134E+00_JPRB, 0.28818E+00_JPRB, 0.28436E+00_JPRB, 0.27947E+00_JPRB ,&
&  0.27653E+00_JPRB, 0.27427E+00_JPRB, 0.27525E+00_JPRB, 0.27823E+00_JPRB, 0.28053E+00_JPRB, 0.28188E+00_JPRB, 0.28292E+00_JPRB ,&
&  0.28439E+00_JPRB, 0.28620E+00_JPRB, 0.28777E+00_JPRB, 0.29065E+00_JPRB, 0.29327E+00_JPRB, 0.29329E+00_JPRB, 0.29196E+00_JPRB ,&
&  0.29119E+00_JPRB, 0.29234E+00_JPRB, 0.29378E+00_JPRB, 0.29445E+00_JPRB, 0.29365E+00_JPRB, 0.29113E+00_JPRB, 0.28707E+00_JPRB ,&
&  0.28026E+00_JPRB, 0.27521E+00_JPRB, 0.27406E+00_JPRB, 0.27516E+00_JPRB, 0.27702E+00_JPRB, 0.27842E+00_JPRB, 0.27923E+00_JPRB ,&
&  0.27940E+00_JPRB, 0.27867E+00_JPRB, 0.27739E+00_JPRB, 0.27587E+00_JPRB, 0.27390E+00_JPRB, 0.27169E+00_JPRB, 0.27035E+00_JPRB ,&
&  0.26893E+00_JPRB, 0.26773E+00_JPRB, 0.26693E+00_JPRB, 0.26640E+00_JPRB, 0.26521E+00_JPRB, 0.26353E+00_JPRB, 0.26254E+00_JPRB ,&
&  0.26121E+00_JPRB, 0.26043E+00_JPRB, 0.25930E+00_JPRB, 0.25818E+00_JPRB, 0.25676E+00_JPRB, 0.25540E+00_JPRB, 0.25499E+00_JPRB ,&
&  0.25506E+00_JPRB, 0.25388E+00_JPRB, 0.25243E+00_JPRB, 0.25224E+00_JPRB, 0.25268E+00_JPRB, 0.25344E+00_JPRB, 0.25407E+00_JPRB ,&
&  0.25472E+00_JPRB, 0.25556E+00_JPRB, 0.25744E+00_JPRB, 0.25959E+00_JPRB, 0.26202E+00_JPRB, 0.26501E+00_JPRB, 0.26847E+00_JPRB ,&
&  0.27270E+00_JPRB, 0.27773E+00_JPRB, 0.28343E+00_JPRB, 0.28853E+00_JPRB, 0.29296E+00_JPRB, 0.29691E+00_JPRB, 0.30321E+00_JPRB ,&
&  0.31092E+00_JPRB, 0.31696E+00_JPRB, 0.32116E+00_JPRB, 0.32464E+00_JPRB, 0.32835E+00_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.36949E+00_JPRB, 0.37110E+00_JPRB, 0.37099E+00_JPRB, 0.37128E+00_JPRB, 0.37068E+00_JPRB, 0.36844E+00_JPRB, 0.36439E+00_JPRB ,&
&  0.36011E+00_JPRB, 0.35531E+00_JPRB, 0.35023E+00_JPRB, 0.34377E+00_JPRB, 0.33717E+00_JPRB, 0.33188E+00_JPRB, 0.32629E+00_JPRB ,&
&  0.32072E+00_JPRB, 0.31563E+00_JPRB, 0.31211E+00_JPRB, 0.30925E+00_JPRB, 0.30652E+00_JPRB, 0.30395E+00_JPRB, 0.30067E+00_JPRB ,&
&  0.29788E+00_JPRB, 0.29537E+00_JPRB, 0.29585E+00_JPRB, 0.29837E+00_JPRB, 0.30041E+00_JPRB, 0.30184E+00_JPRB, 0.30301E+00_JPRB ,&
&  0.30333E+00_JPRB, 0.31072E+00_JPRB, 0.30513E+00_JPRB, 0.31530E+00_JPRB, 0.31909E+00_JPRB, 0.32079E+00_JPRB, 0.32191E+00_JPRB ,&
&  0.32499E+00_JPRB, 0.32830E+00_JPRB, 0.33023E+00_JPRB, 0.32672E+00_JPRB, 0.32170E+00_JPRB, 0.31779E+00_JPRB, 0.31357E+00_JPRB ,&
&  0.30801E+00_JPRB, 0.30577E+00_JPRB, 0.30575E+00_JPRB, 0.30604E+00_JPRB, 0.30610E+00_JPRB, 0.30555E+00_JPRB, 0.30426E+00_JPRB ,&
&  0.30242E+00_JPRB, 0.29986E+00_JPRB, 0.29694E+00_JPRB, 0.29401E+00_JPRB, 0.29212E+00_JPRB, 0.28997E+00_JPRB, 0.28807E+00_JPRB ,&
&  0.28735E+00_JPRB, 0.28785E+00_JPRB, 0.28824E+00_JPRB, 0.28819E+00_JPRB, 0.28746E+00_JPRB, 0.28693E+00_JPRB, 0.28674E+00_JPRB ,&
&  0.28597E+00_JPRB, 0.28492E+00_JPRB, 0.28361E+00_JPRB, 0.28189E+00_JPRB, 0.28006E+00_JPRB, 0.27903E+00_JPRB, 0.27887E+00_JPRB ,&
&  0.27847E+00_JPRB, 0.27693E+00_JPRB, 0.27462E+00_JPRB, 0.27400E+00_JPRB, 0.27380E+00_JPRB, 0.27398E+00_JPRB, 0.27441E+00_JPRB ,&
&  0.27549E+00_JPRB, 0.27750E+00_JPRB, 0.27973E+00_JPRB, 0.28185E+00_JPRB, 0.28397E+00_JPRB, 0.28715E+00_JPRB, 0.29109E+00_JPRB ,&
&  0.29563E+00_JPRB, 0.30057E+00_JPRB, 0.30535E+00_JPRB, 0.30970E+00_JPRB, 0.31433E+00_JPRB, 0.32188E+00_JPRB, 0.33349E+00_JPRB ,&
&  0.34422E+00_JPRB, 0.35187E+00_JPRB, 0.35626E+00_JPRB, 0.36059E+00_JPRB, 0.36546E+00_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.41126E+00_JPRB, 0.40892E+00_JPRB, 0.40508E+00_JPRB, 0.40209E+00_JPRB, 0.39842E+00_JPRB, 0.39381E+00_JPRB, 0.38919E+00_JPRB ,&
&  0.38548E+00_JPRB, 0.38168E+00_JPRB, 0.37737E+00_JPRB, 0.37156E+00_JPRB, 0.36395E+00_JPRB, 0.35676E+00_JPRB, 0.34887E+00_JPRB ,&
&  0.34167E+00_JPRB, 0.33555E+00_JPRB, 0.33142E+00_JPRB, 0.32905E+00_JPRB, 0.32709E+00_JPRB, 0.32545E+00_JPRB, 0.32361E+00_JPRB ,&
&  0.32268E+00_JPRB, 0.32258E+00_JPRB, 0.32271E+00_JPRB, 0.32308E+00_JPRB, 0.32366E+00_JPRB, 0.32621E+00_JPRB, 0.32784E+00_JPRB ,&
&  0.33011E+00_JPRB, 0.32888E+00_JPRB, 0.33678E+00_JPRB, 0.34426E+00_JPRB, 0.35378E+00_JPRB, 0.36134E+00_JPRB, 0.36796E+00_JPRB ,&
&  0.37438E+00_JPRB, 0.37731E+00_JPRB, 0.37272E+00_JPRB, 0.36561E+00_JPRB, 0.36038E+00_JPRB, 0.35731E+00_JPRB, 0.35166E+00_JPRB ,&
&  0.34608E+00_JPRB, 0.34204E+00_JPRB, 0.33884E+00_JPRB, 0.33653E+00_JPRB, 0.33414E+00_JPRB, 0.33058E+00_JPRB, 0.32661E+00_JPRB ,&
&  0.32175E+00_JPRB, 0.31812E+00_JPRB, 0.31529E+00_JPRB, 0.31418E+00_JPRB, 0.31436E+00_JPRB, 0.31560E+00_JPRB, 0.31761E+00_JPRB ,&
&  0.32066E+00_JPRB, 0.32335E+00_JPRB, 0.32450E+00_JPRB, 0.32482E+00_JPRB, 0.32490E+00_JPRB, 0.32530E+00_JPRB, 0.32504E+00_JPRB ,&
&  0.32372E+00_JPRB, 0.32172E+00_JPRB, 0.31932E+00_JPRB, 0.31651E+00_JPRB, 0.31351E+00_JPRB, 0.31118E+00_JPRB, 0.30920E+00_JPRB ,&
&  0.30717E+00_JPRB, 0.30482E+00_JPRB, 0.30234E+00_JPRB, 0.30098E+00_JPRB, 0.30005E+00_JPRB, 0.29960E+00_JPRB, 0.30041E+00_JPRB ,&
&  0.30250E+00_JPRB, 0.30499E+00_JPRB, 0.30764E+00_JPRB, 0.30995E+00_JPRB, 0.31214E+00_JPRB, 0.31416E+00_JPRB, 0.31698E+00_JPRB ,&
&  0.32183E+00_JPRB, 0.32721E+00_JPRB, 0.33253E+00_JPRB, 0.33795E+00_JPRB, 0.34734E+00_JPRB, 0.36308E+00_JPRB, 0.38126E+00_JPRB ,&
&  0.39343E+00_JPRB, 0.39886E+00_JPRB, 0.40197E+00_JPRB, 0.40582E+00_JPRB, 0.41013E+00_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.44596E+00_JPRB, 0.44213E+00_JPRB, 0.43776E+00_JPRB, 0.43268E+00_JPRB, 0.42660E+00_JPRB, 0.41907E+00_JPRB, 0.41356E+00_JPRB ,&
&  0.40959E+00_JPRB, 0.40464E+00_JPRB, 0.39990E+00_JPRB, 0.39403E+00_JPRB, 0.38709E+00_JPRB, 0.37925E+00_JPRB, 0.36987E+00_JPRB ,&
&  0.36177E+00_JPRB, 0.35460E+00_JPRB, 0.34993E+00_JPRB, 0.34766E+00_JPRB, 0.34750E+00_JPRB, 0.34758E+00_JPRB, 0.34608E+00_JPRB ,&
&  0.34535E+00_JPRB, 0.34535E+00_JPRB, 0.34491E+00_JPRB, 0.34465E+00_JPRB, 0.34498E+00_JPRB, 0.34816E+00_JPRB, 0.35174E+00_JPRB ,&
&  0.35741E+00_JPRB, 0.33532E+00_JPRB, 0.34853E+00_JPRB, 0.35591E+00_JPRB, 0.36802E+00_JPRB, 0.38745E+00_JPRB, 0.41219E+00_JPRB ,&
&  0.43113E+00_JPRB, 0.43162E+00_JPRB, 0.42324E+00_JPRB, 0.41198E+00_JPRB, 0.40412E+00_JPRB, 0.39667E+00_JPRB, 0.38885E+00_JPRB ,&
&  0.38151E+00_JPRB, 0.37661E+00_JPRB, 0.37248E+00_JPRB, 0.36842E+00_JPRB, 0.36268E+00_JPRB, 0.35538E+00_JPRB, 0.34835E+00_JPRB ,&
&  0.34338E+00_JPRB, 0.34150E+00_JPRB, 0.34217E+00_JPRB, 0.34410E+00_JPRB, 0.34658E+00_JPRB, 0.35086E+00_JPRB, 0.35618E+00_JPRB ,&
&  0.36135E+00_JPRB, 0.36363E+00_JPRB, 0.36538E+00_JPRB, 0.36876E+00_JPRB, 0.37218E+00_JPRB, 0.37393E+00_JPRB, 0.37381E+00_JPRB ,&
&  0.37217E+00_JPRB, 0.36843E+00_JPRB, 0.36360E+00_JPRB, 0.35775E+00_JPRB, 0.35172E+00_JPRB, 0.34565E+00_JPRB, 0.34027E+00_JPRB ,&
&  0.33584E+00_JPRB, 0.33163E+00_JPRB, 0.32762E+00_JPRB, 0.32468E+00_JPRB, 0.32376E+00_JPRB, 0.32426E+00_JPRB, 0.32580E+00_JPRB ,&
&  0.32885E+00_JPRB, 0.33219E+00_JPRB, 0.33462E+00_JPRB, 0.33716E+00_JPRB, 0.33966E+00_JPRB, 0.34216E+00_JPRB, 0.34600E+00_JPRB ,&
&  0.35168E+00_JPRB, 0.35631E+00_JPRB, 0.36076E+00_JPRB, 0.36963E+00_JPRB, 0.38787E+00_JPRB, 0.41111E+00_JPRB, 0.43064E+00_JPRB ,&
&  0.44034E+00_JPRB, 0.44015E+00_JPRB, 0.44123E+00_JPRB, 0.44231E+00_JPRB, 0.44679E+00_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.50155E+00_JPRB, 0.49805E+00_JPRB, 0.49027E+00_JPRB, 0.47633E+00_JPRB, 0.46146E+00_JPRB, 0.44880E+00_JPRB, 0.44128E+00_JPRB ,&
&  0.43601E+00_JPRB, 0.42833E+00_JPRB, 0.42161E+00_JPRB, 0.41641E+00_JPRB, 0.40996E+00_JPRB, 0.40276E+00_JPRB, 0.39394E+00_JPRB ,&
&  0.38711E+00_JPRB, 0.38199E+00_JPRB, 0.37826E+00_JPRB, 0.37587E+00_JPRB, 0.37418E+00_JPRB, 0.37263E+00_JPRB, 0.37041E+00_JPRB ,&
&  0.36811E+00_JPRB, 0.36622E+00_JPRB, 0.36440E+00_JPRB, 0.36410E+00_JPRB, 0.36475E+00_JPRB, 0.36859E+00_JPRB, 0.37459E+00_JPRB ,&
&  0.36320E+00_JPRB, 0.33953E+00_JPRB, 0.35537E+00_JPRB, 0.37576E+00_JPRB, 0.39757E+00_JPRB, 0.43166E+00_JPRB, 0.46971E+00_JPRB ,&
&  0.49005E+00_JPRB, 0.50037E+00_JPRB, 0.49052E+00_JPRB, 0.47655E+00_JPRB, 0.46181E+00_JPRB, 0.44673E+00_JPRB, 0.43481E+00_JPRB ,&
&  0.42415E+00_JPRB, 0.41694E+00_JPRB, 0.41084E+00_JPRB, 0.40206E+00_JPRB, 0.39134E+00_JPRB, 0.38255E+00_JPRB, 0.37736E+00_JPRB ,&
&  0.37556E+00_JPRB, 0.37630E+00_JPRB, 0.37752E+00_JPRB, 0.37908E+00_JPRB, 0.38202E+00_JPRB, 0.38785E+00_JPRB, 0.39536E+00_JPRB ,&
&  0.40231E+00_JPRB, 0.41229E+00_JPRB, 0.42551E+00_JPRB, 0.43775E+00_JPRB, 0.44861E+00_JPRB, 0.45418E+00_JPRB, 0.45544E+00_JPRB ,&
&  0.45101E+00_JPRB, 0.44136E+00_JPRB, 0.43022E+00_JPRB, 0.41931E+00_JPRB, 0.40775E+00_JPRB, 0.39505E+00_JPRB, 0.38227E+00_JPRB ,&
&  0.37172E+00_JPRB, 0.36317E+00_JPRB, 0.35708E+00_JPRB, 0.35465E+00_JPRB, 0.35444E+00_JPRB, 0.35665E+00_JPRB, 0.35976E+00_JPRB ,&
&  0.36300E+00_JPRB, 0.36632E+00_JPRB, 0.36877E+00_JPRB, 0.37270E+00_JPRB, 0.37702E+00_JPRB, 0.38232E+00_JPRB, 0.38817E+00_JPRB ,&
&  0.39490E+00_JPRB, 0.40163E+00_JPRB, 0.40920E+00_JPRB, 0.43136E+00_JPRB, 0.45950E+00_JPRB, 0.48362E+00_JPRB, 0.49727E+00_JPRB ,&
&  0.49886E+00_JPRB, 0.49665E+00_JPRB, 0.49813E+00_JPRB, 0.49737E+00_JPRB, 0.50320E+00_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.58296E+00_JPRB, 0.57659E+00_JPRB, 0.55745E+00_JPRB, 0.53736E+00_JPRB, 0.51727E+00_JPRB, 0.49817E+00_JPRB, 0.48475E+00_JPRB ,&
&  0.47530E+00_JPRB, 0.46310E+00_JPRB, 0.45255E+00_JPRB, 0.44498E+00_JPRB, 0.43657E+00_JPRB, 0.42841E+00_JPRB, 0.41961E+00_JPRB ,&
&  0.41283E+00_JPRB, 0.40878E+00_JPRB, 0.40600E+00_JPRB, 0.40289E+00_JPRB, 0.39916E+00_JPRB, 0.39450E+00_JPRB, 0.38965E+00_JPRB ,&
&  0.38472E+00_JPRB, 0.38076E+00_JPRB, 0.37676E+00_JPRB, 0.37492E+00_JPRB, 0.37786E+00_JPRB, 0.38449E+00_JPRB, 0.39075E+00_JPRB ,&
&  0.41532E+00_JPRB, 0.36834E+00_JPRB, 0.38707E+00_JPRB, 0.42129E+00_JPRB, 0.45821E+00_JPRB, 0.50597E+00_JPRB, 0.54539E+00_JPRB ,&
&  0.56258E+00_JPRB, 0.57878E+00_JPRB, 0.56897E+00_JPRB, 0.55213E+00_JPRB, 0.52558E+00_JPRB, 0.49918E+00_JPRB, 0.47810E+00_JPRB ,&
&  0.46474E+00_JPRB, 0.45500E+00_JPRB, 0.44375E+00_JPRB, 0.43120E+00_JPRB, 0.41936E+00_JPRB, 0.41116E+00_JPRB, 0.40794E+00_JPRB ,&
&  0.40862E+00_JPRB, 0.40982E+00_JPRB, 0.41025E+00_JPRB, 0.41326E+00_JPRB, 0.41978E+00_JPRB, 0.42845E+00_JPRB, 0.44109E+00_JPRB ,&
&  0.46246E+00_JPRB, 0.49127E+00_JPRB, 0.51547E+00_JPRB, 0.53635E+00_JPRB, 0.54692E+00_JPRB, 0.54547E+00_JPRB, 0.52545E+00_JPRB ,&
&  0.49986E+00_JPRB, 0.47690E+00_JPRB, 0.46132E+00_JPRB, 0.45232E+00_JPRB, 0.44586E+00_JPRB, 0.44091E+00_JPRB, 0.43490E+00_JPRB ,&
&  0.42696E+00_JPRB, 0.41853E+00_JPRB, 0.41188E+00_JPRB, 0.40705E+00_JPRB, 0.40409E+00_JPRB, 0.40415E+00_JPRB, 0.40624E+00_JPRB ,&
&  0.40892E+00_JPRB, 0.41154E+00_JPRB, 0.41463E+00_JPRB, 0.41941E+00_JPRB, 0.42508E+00_JPRB, 0.43140E+00_JPRB, 0.43679E+00_JPRB ,&
&  0.44329E+00_JPRB, 0.45143E+00_JPRB, 0.45995E+00_JPRB, 0.49791E+00_JPRB, 0.53242E+00_JPRB, 0.55114E+00_JPRB, 0.56312E+00_JPRB ,&
&  0.56887E+00_JPRB, 0.58031E+00_JPRB, 0.49907E+00_JPRB, 0.54885E+00_JPRB, 0.57485E+00_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.68555E+00_JPRB, 0.66698E+00_JPRB, 0.63397E+00_JPRB, 0.59881E+00_JPRB, 0.56876E+00_JPRB, 0.54205E+00_JPRB, 0.52148E+00_JPRB ,&
&  0.50885E+00_JPRB, 0.49915E+00_JPRB, 0.48920E+00_JPRB, 0.47955E+00_JPRB, 0.46857E+00_JPRB, 0.45850E+00_JPRB, 0.44799E+00_JPRB ,&
&  0.44056E+00_JPRB, 0.43575E+00_JPRB, 0.43156E+00_JPRB, 0.42652E+00_JPRB, 0.42106E+00_JPRB, 0.41521E+00_JPRB, 0.40800E+00_JPRB ,&
&  0.40164E+00_JPRB, 0.39821E+00_JPRB, 0.39278E+00_JPRB, 0.39137E+00_JPRB, 0.39806E+00_JPRB, 0.40698E+00_JPRB, 0.41669E+00_JPRB ,&
&  0.44620E+00_JPRB, 0.41811E+00_JPRB, 0.43831E+00_JPRB, 0.50880E+00_JPRB, 0.55979E+00_JPRB, 0.61836E+00_JPRB, 0.64501E+00_JPRB ,&
&  0.65009E+00_JPRB, 0.65512E+00_JPRB, 0.63826E+00_JPRB, 0.61421E+00_JPRB, 0.57752E+00_JPRB, 0.54595E+00_JPRB, 0.52197E+00_JPRB ,&
&  0.50121E+00_JPRB, 0.48306E+00_JPRB, 0.46860E+00_JPRB, 0.45693E+00_JPRB, 0.44711E+00_JPRB, 0.44299E+00_JPRB, 0.44424E+00_JPRB ,&
&  0.44687E+00_JPRB, 0.45014E+00_JPRB, 0.45494E+00_JPRB, 0.46292E+00_JPRB, 0.47178E+00_JPRB, 0.48379E+00_JPRB, 0.51955E+00_JPRB ,&
&  0.57615E+00_JPRB, 0.61528E+00_JPRB, 0.64587E+00_JPRB, 0.65850E+00_JPRB, 0.65040E+00_JPRB, 0.62645E+00_JPRB, 0.58823E+00_JPRB ,&
&  0.54901E+00_JPRB, 0.51865E+00_JPRB, 0.49916E+00_JPRB, 0.48675E+00_JPRB, 0.47937E+00_JPRB, 0.47316E+00_JPRB, 0.46790E+00_JPRB ,&
&  0.46294E+00_JPRB, 0.45954E+00_JPRB, 0.45979E+00_JPRB, 0.46380E+00_JPRB, 0.46979E+00_JPRB, 0.47502E+00_JPRB, 0.47762E+00_JPRB ,&
&  0.47822E+00_JPRB, 0.47810E+00_JPRB, 0.48012E+00_JPRB, 0.48321E+00_JPRB, 0.48682E+00_JPRB, 0.48887E+00_JPRB, 0.49283E+00_JPRB ,&
&  0.49878E+00_JPRB, 0.50830E+00_JPRB, 0.52077E+00_JPRB, 0.60529E+00_JPRB, 0.64050E+00_JPRB, 0.65229E+00_JPRB, 0.65887E+00_JPRB ,&
&  0.67306E+00_JPRB, 0.67753E+00_JPRB, 0.68508E+00_JPRB, 0.60302E+00_JPRB, 0.67810E+00_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.78717E+00_JPRB, 0.73590E+00_JPRB, 0.67166E+00_JPRB, 0.62842E+00_JPRB, 0.59863E+00_JPRB, 0.56692E+00_JPRB, 0.54323E+00_JPRB ,&
&  0.52871E+00_JPRB, 0.52087E+00_JPRB, 0.51335E+00_JPRB, 0.50350E+00_JPRB, 0.49326E+00_JPRB, 0.48530E+00_JPRB, 0.47696E+00_JPRB ,&
&  0.46851E+00_JPRB, 0.46049E+00_JPRB, 0.45302E+00_JPRB, 0.44732E+00_JPRB, 0.44015E+00_JPRB, 0.43406E+00_JPRB, 0.42846E+00_JPRB ,&
&  0.42214E+00_JPRB, 0.41636E+00_JPRB, 0.40992E+00_JPRB, 0.41381E+00_JPRB, 0.42280E+00_JPRB, 0.43138E+00_JPRB, 0.44247E+00_JPRB ,&
&  0.47952E+00_JPRB, 0.48927E+00_JPRB, 0.55729E+00_JPRB, 0.63439E+00_JPRB, 0.68106E+00_JPRB, 0.71812E+00_JPRB, 0.72219E+00_JPRB ,&
&  0.69821E+00_JPRB, 0.70525E+00_JPRB, 0.69437E+00_JPRB, 0.66578E+00_JPRB, 0.64356E+00_JPRB, 0.60118E+00_JPRB, 0.56342E+00_JPRB ,&
&  0.53696E+00_JPRB, 0.51988E+00_JPRB, 0.50514E+00_JPRB, 0.48948E+00_JPRB, 0.47996E+00_JPRB, 0.47785E+00_JPRB, 0.48085E+00_JPRB ,&
&  0.48470E+00_JPRB, 0.49114E+00_JPRB, 0.49774E+00_JPRB, 0.50481E+00_JPRB, 0.51492E+00_JPRB, 0.55824E+00_JPRB, 0.65997E+00_JPRB ,&
&  0.73581E+00_JPRB, 0.77789E+00_JPRB, 0.80098E+00_JPRB, 0.80813E+00_JPRB, 0.78661E+00_JPRB, 0.75219E+00_JPRB, 0.71202E+00_JPRB ,&
&  0.67028E+00_JPRB, 0.62632E+00_JPRB, 0.58606E+00_JPRB, 0.55459E+00_JPRB, 0.52977E+00_JPRB, 0.51300E+00_JPRB, 0.50420E+00_JPRB ,&
&  0.50313E+00_JPRB, 0.50800E+00_JPRB, 0.51600E+00_JPRB, 0.52403E+00_JPRB, 0.52895E+00_JPRB, 0.52957E+00_JPRB, 0.52796E+00_JPRB ,&
&  0.52534E+00_JPRB, 0.52230E+00_JPRB, 0.52107E+00_JPRB, 0.52238E+00_JPRB, 0.52606E+00_JPRB, 0.53079E+00_JPRB, 0.53607E+00_JPRB ,&
&  0.54214E+00_JPRB, 0.55335E+00_JPRB, 0.56864E+00_JPRB, 0.64302E+00_JPRB, 0.67763E+00_JPRB, 0.71322E+00_JPRB, 0.74644E+00_JPRB ,&
&  0.78668E+00_JPRB, 0.80598E+00_JPRB, 0.79223E+00_JPRB, 0.76089E+00_JPRB, 0.76566E+00_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.84499E+00_JPRB, 0.73447E+00_JPRB, 0.66509E+00_JPRB, 0.63271E+00_JPRB, 0.62273E+00_JPRB, 0.60411E+00_JPRB, 0.58230E+00_JPRB ,&
&  0.56166E+00_JPRB, 0.54733E+00_JPRB, 0.53872E+00_JPRB, 0.52811E+00_JPRB, 0.51584E+00_JPRB, 0.50495E+00_JPRB, 0.49344E+00_JPRB ,&
&  0.48166E+00_JPRB, 0.47438E+00_JPRB, 0.46871E+00_JPRB, 0.46444E+00_JPRB, 0.46094E+00_JPRB, 0.45450E+00_JPRB, 0.44763E+00_JPRB ,&
&  0.44251E+00_JPRB, 0.43620E+00_JPRB, 0.43496E+00_JPRB, 0.44148E+00_JPRB, 0.44759E+00_JPRB, 0.45257E+00_JPRB, 0.47061E+00_JPRB ,&
&  0.52505E+00_JPRB, 0.56708E+00_JPRB, 0.65318E+00_JPRB, 0.73712E+00_JPRB, 0.80037E+00_JPRB, 0.87015E+00_JPRB, 0.89508E+00_JPRB ,&
&  0.87380E+00_JPRB, 0.88140E+00_JPRB, 0.86288E+00_JPRB, 0.79965E+00_JPRB, 0.73011E+00_JPRB, 0.68353E+00_JPRB, 0.65053E+00_JPRB ,&
&  0.62298E+00_JPRB, 0.59356E+00_JPRB, 0.56396E+00_JPRB, 0.54062E+00_JPRB, 0.52710E+00_JPRB, 0.52229E+00_JPRB, 0.52183E+00_JPRB ,&
&  0.52384E+00_JPRB, 0.52890E+00_JPRB, 0.53304E+00_JPRB, 0.54293E+00_JPRB, 0.59851E+00_JPRB, 0.74133E+00_JPRB, 0.95828E+00_JPRB ,&
&  0.10563E+01_JPRB, 0.10811E+01_JPRB, 0.10985E+01_JPRB, 0.10948E+01_JPRB, 0.10331E+01_JPRB, 0.96639E+00_JPRB, 0.89852E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.76237E+00_JPRB, 0.70358E+00_JPRB, 0.65728E+00_JPRB, 0.62100E+00_JPRB, 0.59724E+00_JPRB, 0.58564E+00_JPRB ,&
&  0.58108E+00_JPRB, 0.57873E+00_JPRB, 0.57659E+00_JPRB, 0.57106E+00_JPRB, 0.56279E+00_JPRB, 0.55768E+00_JPRB, 0.55453E+00_JPRB ,&
&  0.55165E+00_JPRB, 0.55039E+00_JPRB, 0.55330E+00_JPRB, 0.55788E+00_JPRB, 0.56609E+00_JPRB, 0.57162E+00_JPRB, 0.57909E+00_JPRB ,&
&  0.58912E+00_JPRB, 0.61331E+00_JPRB, 0.62469E+00_JPRB, 0.72804E+00_JPRB, 0.83132E+00_JPRB, 0.86292E+00_JPRB, 0.90444E+00_JPRB ,&
&  0.94502E+00_JPRB, 0.92688E+00_JPRB, 0.86262E+00_JPRB, 0.83670E+00_JPRB, 0.93197E+00_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.69653E+00_JPRB, 0.65858E+00_JPRB, 0.63944E+00_JPRB, 0.63732E+00_JPRB, 0.63988E+00_JPRB, 0.62963E+00_JPRB, 0.60586E+00_JPRB ,&
&  0.57796E+00_JPRB, 0.55491E+00_JPRB, 0.53908E+00_JPRB, 0.52714E+00_JPRB, 0.51771E+00_JPRB, 0.50918E+00_JPRB, 0.49966E+00_JPRB ,&
&  0.49060E+00_JPRB, 0.48578E+00_JPRB, 0.48230E+00_JPRB, 0.47714E+00_JPRB, 0.46670E+00_JPRB, 0.45846E+00_JPRB, 0.45570E+00_JPRB ,&
&  0.45410E+00_JPRB, 0.45473E+00_JPRB, 0.46005E+00_JPRB, 0.46628E+00_JPRB, 0.46903E+00_JPRB, 0.47363E+00_JPRB, 0.49488E+00_JPRB ,&
&  0.56019E+00_JPRB, 0.68847E+00_JPRB, 0.72772E+00_JPRB, 0.83241E+00_JPRB, 0.98586E+00_JPRB, 0.11114E+01_JPRB, 0.12108E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.11494E+01_JPRB, 0.10333E+01_JPRB, 0.91494E+00_JPRB, 0.82005E+00_JPRB, 0.76393E+00_JPRB, 0.72370E+00_JPRB ,&
&  0.68683E+00_JPRB, 0.65055E+00_JPRB, 0.61962E+00_JPRB, 0.59668E+00_JPRB, 0.58119E+00_JPRB, 0.57509E+00_JPRB, 0.57494E+00_JPRB ,&
&  0.57533E+00_JPRB, 0.57462E+00_JPRB, 0.57945E+00_JPRB, 0.63145E+00_JPRB, 0.87114E+00_JPRB, 0.11859E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.16051E+01_JPRB, 0.16499E+01_JPRB, 0.16218E+01_JPRB, 0.15245E+01_JPRB, 0.13681E+01_JPRB, 0.12147E+01_JPRB, 0.10524E+01_JPRB ,&
&  0.93418E+00_JPRB, 0.85052E+00_JPRB, 0.79032E+00_JPRB, 0.74042E+00_JPRB, 0.69593E+00_JPRB, 0.66135E+00_JPRB, 0.63798E+00_JPRB ,&
&  0.62102E+00_JPRB, 0.60618E+00_JPRB, 0.59202E+00_JPRB, 0.57860E+00_JPRB, 0.56812E+00_JPRB, 0.56195E+00_JPRB, 0.56308E+00_JPRB ,&
&  0.56425E+00_JPRB, 0.56384E+00_JPRB, 0.57962E+00_JPRB, 0.59617E+00_JPRB, 0.61330E+00_JPRB, 0.62452E+00_JPRB, 0.63240E+00_JPRB ,&
&  0.64378E+00_JPRB, 0.69615E+00_JPRB, 0.69061E+00_JPRB, 0.74057E+00_JPRB, 0.90804E+00_JPRB, 0.99227E+00_JPRB, 0.98899E+00_JPRB ,&
&  0.94644E+00_JPRB, 0.88143E+00_JPRB, 0.82085E+00_JPRB, 0.78143E+00_JPRB, 0.75369E+00_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.68880E+00_JPRB, 0.66368E+00_JPRB, 0.65451E+00_JPRB, 0.64838E+00_JPRB, 0.64207E+00_JPRB, 0.62806E+00_JPRB, 0.60370E+00_JPRB ,&
&  0.58428E+00_JPRB, 0.56904E+00_JPRB, 0.55154E+00_JPRB, 0.53606E+00_JPRB, 0.52540E+00_JPRB, 0.51716E+00_JPRB, 0.50744E+00_JPRB ,&
&  0.50066E+00_JPRB, 0.49531E+00_JPRB, 0.48308E+00_JPRB, 0.47135E+00_JPRB, 0.46607E+00_JPRB, 0.46512E+00_JPRB, 0.46884E+00_JPRB ,&
&  0.47306E+00_JPRB, 0.47822E+00_JPRB, 0.48191E+00_JPRB, 0.48490E+00_JPRB, 0.48503E+00_JPRB, 0.49289E+00_JPRB, 0.52482E+00_JPRB ,&
&  0.61088E+00_JPRB, 0.76027E+00_JPRB, 0.93912E+00_JPRB, 0.99839E+00_JPRB, 0.11012E+01_JPRB, 0.12424E+01_JPRB, 0.14045E+01_JPRB ,&
&  0.13966E+01_JPRB, 0.12783E+01_JPRB, 0.11128E+01_JPRB, 0.99814E+00_JPRB, 0.91167E+00_JPRB, 0.84977E+00_JPRB, 0.78791E+00_JPRB ,&
&  0.73744E+00_JPRB, 0.70390E+00_JPRB, 0.67112E+00_JPRB, 0.64198E+00_JPRB, 0.62984E+00_JPRB, 0.62637E+00_JPRB, 0.62369E+00_JPRB ,&
&  0.61979E+00_JPRB, 0.61756E+00_JPRB, 0.64728E+00_JPRB, 0.79786E+00_JPRB, 0.12112E+01_JPRB, 0.15150E+01_JPRB, 0.20164E+01_JPRB ,&
&  0.24525E+01_JPRB, 0.23974E+01_JPRB, 0.21733E+01_JPRB, 0.18538E+01_JPRB, 0.15143E+01_JPRB, 0.12303E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.94360E+00_JPRB, 0.85280E+00_JPRB, 0.79639E+00_JPRB, 0.76030E+00_JPRB, 0.73157E+00_JPRB, 0.70575E+00_JPRB, 0.67473E+00_JPRB ,&
&  0.64164E+00_JPRB, 0.61497E+00_JPRB, 0.59710E+00_JPRB, 0.58372E+00_JPRB, 0.57626E+00_JPRB, 0.57455E+00_JPRB, 0.57774E+00_JPRB ,&
&  0.58654E+00_JPRB, 0.60990E+00_JPRB, 0.65892E+00_JPRB, 0.65590E+00_JPRB, 0.67544E+00_JPRB, 0.70529E+00_JPRB, 0.73123E+00_JPRB ,&
&  0.72480E+00_JPRB, 0.73820E+00_JPRB, 0.71779E+00_JPRB, 0.74821E+00_JPRB, 0.95535E+00_JPRB, 0.10527E+01_JPRB, 0.97180E+00_JPRB ,&
&  0.91002E+00_JPRB, 0.84968E+00_JPRB, 0.79347E+00_JPRB, 0.74549E+00_JPRB, 0.72890E+00_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.69853E+00_JPRB, 0.67424E+00_JPRB, 0.65757E+00_JPRB, 0.64005E+00_JPRB, 0.62825E+00_JPRB, 0.62002E+00_JPRB, 0.60685E+00_JPRB ,&
&  0.59258E+00_JPRB, 0.57839E+00_JPRB, 0.56224E+00_JPRB, 0.54703E+00_JPRB, 0.53429E+00_JPRB, 0.51891E+00_JPRB, 0.50633E+00_JPRB ,&
&  0.49484E+00_JPRB, 0.48133E+00_JPRB, 0.47271E+00_JPRB, 0.46993E+00_JPRB, 0.47352E+00_JPRB, 0.47870E+00_JPRB, 0.48560E+00_JPRB ,&
&  0.49386E+00_JPRB, 0.49931E+00_JPRB, 0.50036E+00_JPRB, 0.50280E+00_JPRB, 0.50151E+00_JPRB, 0.50898E+00_JPRB, 0.54505E+00_JPRB ,&
&  0.62443E+00_JPRB, 0.59246E+00_JPRB, 0.93978E+00_JPRB, 0.11726E+01_JPRB, 0.13290E+01_JPRB, 0.14622E+01_JPRB, 0.15435E+01_JPRB ,&
&  0.15851E+01_JPRB, 0.14289E+01_JPRB, 0.12387E+01_JPRB, 0.11128E+01_JPRB, 0.10118E+01_JPRB, 0.93610E+00_JPRB, 0.87630E+00_JPRB ,&
&  0.81824E+00_JPRB, 0.77480E+00_JPRB, 0.74053E+00_JPRB, 0.69612E+00_JPRB, 0.66360E+00_JPRB, 0.64784E+00_JPRB, 0.64255E+00_JPRB ,&
&  0.63731E+00_JPRB, 0.65219E+00_JPRB, 0.72703E+00_JPRB, 0.95241E+00_JPRB, 0.14430E+01_JPRB, 0.18322E+01_JPRB, 0.20767E+01_JPRB ,&
&  0.23930E+01_JPRB, 0.24865E+01_JPRB, 0.22480E+01_JPRB, 0.18693E+01_JPRB, 0.15051E+01_JPRB, 0.12543E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.93869E+00_JPRB, 0.87361E+00_JPRB, 0.83547E+00_JPRB, 0.80471E+00_JPRB, 0.77152E+00_JPRB, 0.72805E+00_JPRB, 0.67983E+00_JPRB ,&
&  0.64295E+00_JPRB, 0.62250E+00_JPRB, 0.60983E+00_JPRB, 0.60472E+00_JPRB, 0.60406E+00_JPRB, 0.60701E+00_JPRB, 0.61605E+00_JPRB ,&
&  0.63425E+00_JPRB, 0.66552E+00_JPRB, 0.68671E+00_JPRB, 0.69259E+00_JPRB, 0.71275E+00_JPRB, 0.74020E+00_JPRB, 0.77201E+00_JPRB ,&
&  0.79395E+00_JPRB, 0.78274E+00_JPRB, 0.76214E+00_JPRB, 0.79917E+00_JPRB, 0.87758E+00_JPRB, 0.96742E+00_JPRB, 0.92270E+00_JPRB ,&
&  0.88689E+00_JPRB, 0.83356E+00_JPRB, 0.77323E+00_JPRB, 0.74386E+00_JPRB, 0.72890E+00_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.70408E+00_JPRB, 0.68495E+00_JPRB, 0.66362E+00_JPRB, 0.64366E+00_JPRB, 0.63657E+00_JPRB, 0.63226E+00_JPRB, 0.61917E+00_JPRB ,&
&  0.60752E+00_JPRB, 0.59770E+00_JPRB, 0.58357E+00_JPRB, 0.56593E+00_JPRB, 0.54415E+00_JPRB, 0.51723E+00_JPRB, 0.49277E+00_JPRB ,&
&  0.48141E+00_JPRB, 0.47220E+00_JPRB, 0.47494E+00_JPRB, 0.48131E+00_JPRB, 0.49039E+00_JPRB, 0.49723E+00_JPRB, 0.50481E+00_JPRB ,&
&  0.51074E+00_JPRB, 0.51374E+00_JPRB, 0.51802E+00_JPRB, 0.52188E+00_JPRB, 0.52386E+00_JPRB, 0.53372E+00_JPRB, 0.56967E+00_JPRB ,&
&  0.64834E+00_JPRB, 0.10042E+01_JPRB, 0.83939E+00_JPRB, 0.14543E+01_JPRB, 0.16651E+01_JPRB, 0.17664E+01_JPRB, 0.16696E+01_JPRB ,&
&  0.16770E+01_JPRB, 0.15976E+01_JPRB, 0.13733E+01_JPRB, 0.12029E+01_JPRB, 0.10748E+01_JPRB, 0.99097E+00_JPRB, 0.93236E+00_JPRB ,&
&  0.89045E+00_JPRB, 0.84260E+00_JPRB, 0.78788E+00_JPRB, 0.73446E+00_JPRB, 0.70120E+00_JPRB, 0.68580E+00_JPRB, 0.68231E+00_JPRB ,&
&  0.68424E+00_JPRB, 0.72467E+00_JPRB, 0.86765E+00_JPRB, 0.12719E+01_JPRB, 0.17140E+01_JPRB, 0.19763E+01_JPRB, 0.19848E+01_JPRB ,&
&  0.19725E+01_JPRB, 0.21544E+01_JPRB, 0.19864E+01_JPRB, 0.16339E+01_JPRB, 0.13616E+01_JPRB, 0.11557E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.91093E+00_JPRB, 0.85499E+00_JPRB, 0.81890E+00_JPRB, 0.78894E+00_JPRB, 0.75700E+00_JPRB, 0.72222E+00_JPRB, 0.68941E+00_JPRB ,&
&  0.66542E+00_JPRB, 0.64879E+00_JPRB, 0.63900E+00_JPRB, 0.64148E+00_JPRB, 0.64432E+00_JPRB, 0.64862E+00_JPRB, 0.65776E+00_JPRB ,&
&  0.67275E+00_JPRB, 0.70073E+00_JPRB, 0.70584E+00_JPRB, 0.71987E+00_JPRB, 0.72946E+00_JPRB, 0.74302E+00_JPRB, 0.75870E+00_JPRB ,&
&  0.80902E+00_JPRB, 0.81729E+00_JPRB, 0.81178E+00_JPRB, 0.83239E+00_JPRB, 0.88874E+00_JPRB, 0.96277E+00_JPRB, 0.89108E+00_JPRB ,&
&  0.86168E+00_JPRB, 0.81466E+00_JPRB, 0.77281E+00_JPRB, 0.74664E+00_JPRB, 0.72716E+00_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.70734E+00_JPRB, 0.68366E+00_JPRB, 0.66812E+00_JPRB, 0.66099E+00_JPRB, 0.65530E+00_JPRB, 0.64903E+00_JPRB, 0.63409E+00_JPRB ,&
&  0.61820E+00_JPRB, 0.60836E+00_JPRB, 0.59306E+00_JPRB, 0.57740E+00_JPRB, 0.54658E+00_JPRB, 0.50833E+00_JPRB, 0.49196E+00_JPRB ,&
&  0.48297E+00_JPRB, 0.48306E+00_JPRB, 0.49306E+00_JPRB, 0.49832E+00_JPRB, 0.50308E+00_JPRB, 0.51214E+00_JPRB, 0.52057E+00_JPRB ,&
&  0.52753E+00_JPRB, 0.52915E+00_JPRB, 0.53524E+00_JPRB, 0.54244E+00_JPRB, 0.54956E+00_JPRB, 0.57186E+00_JPRB, 0.60744E+00_JPRB ,&
&  0.71272E+00_JPRB, 0.97212E+00_JPRB, 0.85044E+00_JPRB, 0.17295E+01_JPRB, 0.19825E+01_JPRB, 0.19913E+01_JPRB, 0.18649E+01_JPRB ,&
&  0.18627E+01_JPRB, 0.18596E+01_JPRB, 0.14914E+01_JPRB, 0.12547E+01_JPRB, 0.10928E+01_JPRB, 0.10085E+01_JPRB, 0.95872E+00_JPRB ,&
&  0.92167E+00_JPRB, 0.88172E+00_JPRB, 0.82809E+00_JPRB, 0.77953E+00_JPRB, 0.75468E+00_JPRB, 0.74117E+00_JPRB, 0.73795E+00_JPRB ,&
&  0.74722E+00_JPRB, 0.82979E+00_JPRB, 0.10740E+01_JPRB, 0.15551E+01_JPRB, 0.19867E+01_JPRB, 0.20300E+01_JPRB, 0.18312E+01_JPRB ,&
&  0.17971E+01_JPRB, 0.18782E+01_JPRB, 0.17011E+01_JPRB, 0.13834E+01_JPRB, 0.11599E+01_JPRB, 0.10433E+01_JPRB, 0.92606E+00_JPRB ,&
&  0.85117E+00_JPRB, 0.80321E+00_JPRB, 0.77757E+00_JPRB, 0.75714E+00_JPRB, 0.73943E+00_JPRB, 0.72409E+00_JPRB, 0.70659E+00_JPRB ,&
&  0.69094E+00_JPRB, 0.67607E+00_JPRB, 0.67063E+00_JPRB, 0.66600E+00_JPRB, 0.67021E+00_JPRB, 0.67501E+00_JPRB, 0.68104E+00_JPRB ,&
&  0.70305E+00_JPRB, 0.74072E+00_JPRB, 0.73693E+00_JPRB, 0.75069E+00_JPRB, 0.75474E+00_JPRB, 0.75427E+00_JPRB, 0.77343E+00_JPRB ,&
&  0.81472E+00_JPRB, 0.82652E+00_JPRB, 0.84382E+00_JPRB, 0.84131E+00_JPRB, 0.86887E+00_JPRB, 0.90091E+00_JPRB, 0.86298E+00_JPRB ,&
&  0.84928E+00_JPRB, 0.81856E+00_JPRB, 0.78782E+00_JPRB, 0.75417E+00_JPRB, 0.73054E+00_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.69017E+00_JPRB, 0.66661E+00_JPRB, 0.65982E+00_JPRB, 0.65886E+00_JPRB, 0.65555E+00_JPRB, 0.64576E+00_JPRB, 0.62732E+00_JPRB ,&
&  0.60453E+00_JPRB, 0.58758E+00_JPRB, 0.56842E+00_JPRB, 0.54385E+00_JPRB, 0.50830E+00_JPRB, 0.47635E+00_JPRB, 0.47428E+00_JPRB ,&
&  0.49308E+00_JPRB, 0.50222E+00_JPRB, 0.51056E+00_JPRB, 0.52038E+00_JPRB, 0.53142E+00_JPRB, 0.54414E+00_JPRB, 0.55340E+00_JPRB ,&
&  0.55690E+00_JPRB, 0.56276E+00_JPRB, 0.57366E+00_JPRB, 0.58649E+00_JPRB, 0.60423E+00_JPRB, 0.63658E+00_JPRB, 0.69341E+00_JPRB ,&
&  0.84703E+00_JPRB, 0.10449E+01_JPRB, 0.10642E+01_JPRB, 0.17667E+01_JPRB, 0.20832E+01_JPRB, 0.19707E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.18950E+01_JPRB, 0.16850E+01_JPRB, 0.14959E+01_JPRB, 0.12330E+01_JPRB, 0.10894E+01_JPRB, 0.10207E+01_JPRB, 0.98052E+00_JPRB ,&
&  0.95361E+00_JPRB, 0.92017E+00_JPRB, 0.87423E+00_JPRB, 0.85153E+00_JPRB, 0.83686E+00_JPRB, 0.81740E+00_JPRB, 0.81145E+00_JPRB ,&
&  0.84727E+00_JPRB, 0.99723E+00_JPRB, 0.13629E+01_JPRB, 0.19137E+01_JPRB, 0.20924E+01_JPRB, 0.19088E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.15880E+01_JPRB, 0.15362E+01_JPRB, 0.13888E+01_JPRB, 0.11639E+01_JPRB, 0.98135E+00_JPRB, 0.96523E+00_JPRB, 0.86638E+00_JPRB ,&
&  0.80742E+00_JPRB, 0.76874E+00_JPRB, 0.74852E+00_JPRB, 0.73410E+00_JPRB, 0.72480E+00_JPRB, 0.71454E+00_JPRB, 0.70226E+00_JPRB ,&
&  0.69543E+00_JPRB, 0.69609E+00_JPRB, 0.69148E+00_JPRB, 0.69265E+00_JPRB, 0.69583E+00_JPRB, 0.70367E+00_JPRB, 0.72052E+00_JPRB ,&
&  0.75773E+00_JPRB, 0.79633E+00_JPRB, 0.79900E+00_JPRB, 0.80708E+00_JPRB, 0.80639E+00_JPRB, 0.80445E+00_JPRB, 0.80779E+00_JPRB ,&
&  0.82213E+00_JPRB, 0.86246E+00_JPRB, 0.87302E+00_JPRB, 0.87320E+00_JPRB, 0.87797E+00_JPRB, 0.87910E+00_JPRB, 0.86735E+00_JPRB ,&
&  0.86645E+00_JPRB, 0.84478E+00_JPRB, 0.82080E+00_JPRB, 0.75284E+00_JPRB, 0.72323E+00_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.64834E+00_JPRB, 0.62959E+00_JPRB, 0.61397E+00_JPRB, 0.61043E+00_JPRB, 0.60713E+00_JPRB, 0.59159E+00_JPRB, 0.57149E+00_JPRB ,&
&  0.54676E+00_JPRB, 0.52822E+00_JPRB, 0.50299E+00_JPRB, 0.48019E+00_JPRB, 0.47725E+00_JPRB, 0.49147E+00_JPRB, 0.52065E+00_JPRB ,&
&  0.54916E+00_JPRB, 0.56453E+00_JPRB, 0.57609E+00_JPRB, 0.58701E+00_JPRB, 0.59725E+00_JPRB, 0.60255E+00_JPRB, 0.60306E+00_JPRB ,&
&  0.60932E+00_JPRB, 0.62432E+00_JPRB, 0.64676E+00_JPRB, 0.67294E+00_JPRB, 0.70894E+00_JPRB, 0.76966E+00_JPRB, 0.88124E+00_JPRB ,&
&  0.11141E+01_JPRB, 0.12285E+01_JPRB, 0.67286E+00_JPRB, 0.12806E+01_JPRB, 0.18015E+01_JPRB, 0.17900E+01_JPRB, 0.16876E+01_JPRB ,&
&  0.15516E+01_JPRB, 0.13289E+01_JPRB, 0.12608E+01_JPRB, 0.11724E+01_JPRB, 0.10639E+01_JPRB, 0.10233E+01_JPRB, 0.10108E+01_JPRB ,&
&  0.10078E+01_JPRB, 0.98422E+00_JPRB, 0.96866E+00_JPRB, 0.96870E+00_JPRB, 0.94622E+00_JPRB, 0.91077E+00_JPRB, 0.91279E+00_JPRB ,&
&  0.99678E+00_JPRB, 0.12379E+01_JPRB, 0.17545E+01_JPRB, 0.20475E+01_JPRB, 0.21493E+01_JPRB, 0.17432E+01_JPRB, 0.13721E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12471E+01_JPRB, 0.11658E+01_JPRB, 0.10211E+01_JPRB, 0.89034E+00_JPRB, 0.83978E+00_JPRB, 0.83411E+00_JPRB ,&
&  0.78448E+00_JPRB, 0.74340E+00_JPRB, 0.71685E+00_JPRB, 0.69668E+00_JPRB, 0.68478E+00_JPRB, 0.67821E+00_JPRB, 0.67807E+00_JPRB ,&
&  0.68995E+00_JPRB, 0.69803E+00_JPRB, 0.70697E+00_JPRB, 0.71896E+00_JPRB, 0.73123E+00_JPRB, 0.74933E+00_JPRB, 0.78159E+00_JPRB ,&
&  0.82456E+00_JPRB, 0.85750E+00_JPRB, 0.88522E+00_JPRB, 0.87628E+00_JPRB, 0.86552E+00_JPRB, 0.86559E+00_JPRB, 0.88654E+00_JPRB ,&
&  0.90280E+00_JPRB, 0.94455E+00_JPRB, 0.86397E+00_JPRB, 0.85780E+00_JPRB, 0.89069E+00_JPRB, 0.88793E+00_JPRB, 0.91333E+00_JPRB ,&
&  0.95367E+00_JPRB, 0.10146E+01_JPRB, 0.10104E+01_JPRB, 0.73145E+00_JPRB, 0.68259E+00_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.59432E+00_JPRB, 0.56434E+00_JPRB, 0.53594E+00_JPRB, 0.52702E+00_JPRB, 0.53474E+00_JPRB, 0.52966E+00_JPRB, 0.52615E+00_JPRB ,&
&  0.52223E+00_JPRB, 0.51886E+00_JPRB, 0.50711E+00_JPRB, 0.51667E+00_JPRB, 0.54721E+00_JPRB, 0.59258E+00_JPRB, 0.63090E+00_JPRB ,&
&  0.66034E+00_JPRB, 0.67240E+00_JPRB, 0.67460E+00_JPRB, 0.67706E+00_JPRB, 0.67910E+00_JPRB, 0.68216E+00_JPRB, 0.68892E+00_JPRB ,&
&  0.71211E+00_JPRB, 0.75022E+00_JPRB, 0.80014E+00_JPRB, 0.85766E+00_JPRB, 0.93596E+00_JPRB, 0.10661E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.16100E+01_JPRB, 0.12121E+01_JPRB, 0.79966E+00_JPRB, 0.16215E+01_JPRB, 0.18057E+01_JPRB, 0.17656E+01_JPRB, 0.15002E+01_JPRB ,&
&  0.13911E+01_JPRB, 0.12475E+01_JPRB, 0.11445E+01_JPRB, 0.11613E+01_JPRB, 0.10650E+01_JPRB, 0.10470E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10801E+01_JPRB, 0.10885E+01_JPRB, 0.10739E+01_JPRB, 0.10386E+01_JPRB, 0.10216E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.12424E+01_JPRB, 0.15974E+01_JPRB, 0.21763E+01_JPRB, 0.21798E+01_JPRB, 0.22220E+01_JPRB, 0.16478E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.10763E+01_JPRB, 0.10362E+01_JPRB, 0.95577E+00_JPRB, 0.86107E+00_JPRB, 0.79469E+00_JPRB, 0.80258E+00_JPRB ,&
&  0.74508E+00_JPRB, 0.70250E+00_JPRB, 0.66763E+00_JPRB, 0.65044E+00_JPRB, 0.64897E+00_JPRB, 0.65539E+00_JPRB, 0.66081E+00_JPRB ,&
&  0.67526E+00_JPRB, 0.68477E+00_JPRB, 0.70482E+00_JPRB, 0.72852E+00_JPRB, 0.75478E+00_JPRB, 0.78889E+00_JPRB, 0.83411E+00_JPRB ,&
&  0.87177E+00_JPRB, 0.89934E+00_JPRB, 0.91638E+00_JPRB, 0.92334E+00_JPRB, 0.90651E+00_JPRB, 0.88719E+00_JPRB, 0.90433E+00_JPRB ,&
&  0.89861E+00_JPRB, 0.90322E+00_JPRB, 0.88212E+00_JPRB, 0.90461E+00_JPRB, 0.91138E+00_JPRB, 0.91490E+00_JPRB, 0.93796E+00_JPRB ,&
&  0.97005E+00_JPRB, 0.95642E+00_JPRB, 0.86045E+00_JPRB, 0.66586E+00_JPRB, 0.61768E+00_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.55188E+00_JPRB, 0.52572E+00_JPRB, 0.51699E+00_JPRB, 0.51733E+00_JPRB, 0.52746E+00_JPRB, 0.53410E+00_JPRB, 0.53461E+00_JPRB ,&
&  0.53853E+00_JPRB, 0.55147E+00_JPRB, 0.56699E+00_JPRB, 0.61588E+00_JPRB, 0.66844E+00_JPRB, 0.70889E+00_JPRB, 0.73492E+00_JPRB ,&
&  0.75648E+00_JPRB, 0.76539E+00_JPRB, 0.76683E+00_JPRB, 0.77233E+00_JPRB, 0.77814E+00_JPRB, 0.79363E+00_JPRB, 0.82768E+00_JPRB ,&
&  0.88517E+00_JPRB, 0.94922E+00_JPRB, 0.10293E+01_JPRB, 0.11344E+01_JPRB, 0.12709E+01_JPRB, 0.14074E+01_JPRB, 0.15139E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.64451E+00_JPRB, 0.15745E+01_JPRB, 0.18682E+01_JPRB, 0.16569E+01_JPRB, 0.15640E+01_JPRB, 0.14303E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.11888E+01_JPRB, 0.11312E+01_JPRB, 0.11575E+01_JPRB, 0.10912E+01_JPRB, 0.10874E+01_JPRB, 0.10962E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.11531E+01_JPRB, 0.11571E+01_JPRB, 0.11427E+01_JPRB, 0.11402E+01_JPRB, 0.11832E+01_JPRB, 0.13265E+01_JPRB ,&
&  0.16162E+01_JPRB, 0.20387E+01_JPRB, 0.24362E+01_JPRB, 0.21692E+01_JPRB, 0.22143E+01_JPRB, 0.18065E+01_JPRB, 0.14077E+01_JPRB ,&
&  0.11268E+01_JPRB, 0.96127E+00_JPRB, 0.93545E+00_JPRB, 0.93601E+00_JPRB, 0.86450E+00_JPRB, 0.77915E+00_JPRB, 0.76761E+00_JPRB ,&
&  0.70237E+00_JPRB, 0.65752E+00_JPRB, 0.63587E+00_JPRB, 0.63223E+00_JPRB, 0.63956E+00_JPRB, 0.65259E+00_JPRB, 0.66648E+00_JPRB ,&
&  0.67243E+00_JPRB, 0.68623E+00_JPRB, 0.70381E+00_JPRB, 0.73574E+00_JPRB, 0.77370E+00_JPRB, 0.82584E+00_JPRB, 0.86971E+00_JPRB ,&
&  0.89686E+00_JPRB, 0.92160E+00_JPRB, 0.94525E+00_JPRB, 0.96195E+00_JPRB, 0.97760E+00_JPRB, 0.95308E+00_JPRB, 0.93604E+00_JPRB ,&
&  0.93624E+00_JPRB, 0.94103E+00_JPRB, 0.95365E+00_JPRB, 0.97690E+00_JPRB, 0.93885E+00_JPRB, 0.90971E+00_JPRB, 0.92350E+00_JPRB ,&
&  0.91186E+00_JPRB, 0.85099E+00_JPRB, 0.73433E+00_JPRB, 0.61921E+00_JPRB, 0.57788E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.58873E+00_JPRB, 0.57622E+00_JPRB, 0.56268E+00_JPRB, 0.56772E+00_JPRB, 0.57345E+00_JPRB, 0.59615E+00_JPRB, 0.59860E+00_JPRB ,&
&  0.61500E+00_JPRB, 0.66284E+00_JPRB, 0.72187E+00_JPRB, 0.75673E+00_JPRB, 0.78579E+00_JPRB, 0.81368E+00_JPRB, 0.83773E+00_JPRB ,&
&  0.85371E+00_JPRB, 0.86770E+00_JPRB, 0.88176E+00_JPRB, 0.89881E+00_JPRB, 0.92663E+00_JPRB, 0.96885E+00_JPRB, 0.10348E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.11788E+01_JPRB, 0.12736E+01_JPRB, 0.13925E+01_JPRB, 0.15221E+01_JPRB, 0.16001E+01_JPRB, 0.16628E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.15203E+01_JPRB, 0.17977E+01_JPRB, 0.16857E+01_JPRB, 0.15458E+01_JPRB, 0.14703E+01_JPRB, 0.14016E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.11955E+01_JPRB, 0.11540E+01_JPRB, 0.12137E+01_JPRB, 0.11363E+01_JPRB, 0.11363E+01_JPRB, 0.11469E+01_JPRB ,&
&  0.12018E+01_JPRB, 0.12407E+01_JPRB, 0.12579E+01_JPRB, 0.12892E+01_JPRB, 0.13456E+01_JPRB, 0.14763E+01_JPRB, 0.16946E+01_JPRB ,&
&  0.19890E+01_JPRB, 0.22888E+01_JPRB, 0.24448E+01_JPRB, 0.20933E+01_JPRB, 0.21316E+01_JPRB, 0.19781E+01_JPRB, 0.16766E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.90602E+00_JPRB, 0.93058E+00_JPRB, 0.94199E+00_JPRB, 0.85732E+00_JPRB, 0.79017E+00_JPRB, 0.73292E+00_JPRB ,&
&  0.67400E+00_JPRB, 0.64074E+00_JPRB, 0.63316E+00_JPRB, 0.62991E+00_JPRB, 0.63776E+00_JPRB, 0.64691E+00_JPRB, 0.66401E+00_JPRB ,&
&  0.67479E+00_JPRB, 0.69550E+00_JPRB, 0.72510E+00_JPRB, 0.75758E+00_JPRB, 0.79717E+00_JPRB, 0.83896E+00_JPRB, 0.88319E+00_JPRB ,&
&  0.91410E+00_JPRB, 0.94231E+00_JPRB, 0.96858E+00_JPRB, 0.98847E+00_JPRB, 0.99834E+00_JPRB, 0.10037E+01_JPRB, 0.98554E+00_JPRB ,&
&  0.96884E+00_JPRB, 0.97880E+00_JPRB, 0.98421E+00_JPRB, 0.97230E+00_JPRB, 0.93228E+00_JPRB, 0.89297E+00_JPRB, 0.85848E+00_JPRB ,&
&  0.79583E+00_JPRB, 0.70557E+00_JPRB, 0.62595E+00_JPRB, 0.60548E+00_JPRB, 0.59334E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.66605E+00_JPRB, 0.67468E+00_JPRB, 0.67640E+00_JPRB, 0.71529E+00_JPRB, 0.72495E+00_JPRB, 0.74321E+00_JPRB, 0.75916E+00_JPRB ,&
&  0.77522E+00_JPRB, 0.80154E+00_JPRB, 0.83213E+00_JPRB, 0.86414E+00_JPRB, 0.89065E+00_JPRB, 0.91133E+00_JPRB, 0.92907E+00_JPRB ,&
&  0.94909E+00_JPRB, 0.97312E+00_JPRB, 0.10037E+01_JPRB, 0.10429E+01_JPRB, 0.11041E+01_JPRB, 0.11731E+01_JPRB, 0.12468E+01_JPRB ,&
&  0.13199E+01_JPRB, 0.14177E+01_JPRB, 0.15343E+01_JPRB, 0.16704E+01_JPRB, 0.18131E+01_JPRB, 0.19163E+01_JPRB, 0.19774E+01_JPRB ,&
&  0.15492E+01_JPRB, 0.18331E+01_JPRB, 0.17843E+01_JPRB, 0.16270E+01_JPRB, 0.15045E+01_JPRB, 0.14413E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.13339E+01_JPRB, 0.12459E+01_JPRB, 0.12022E+01_JPRB, 0.11687E+01_JPRB, 0.11859E+01_JPRB, 0.11952E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.13164E+01_JPRB, 0.13770E+01_JPRB, 0.14499E+01_JPRB, 0.15212E+01_JPRB, 0.16208E+01_JPRB, 0.17742E+01_JPRB, 0.19486E+01_JPRB ,&
&  0.21280E+01_JPRB, 0.22598E+01_JPRB, 0.23564E+01_JPRB, 0.20739E+01_JPRB, 0.19047E+01_JPRB, 0.17668E+01_JPRB, 0.12376E+01_JPRB ,&
&  0.94802E+00_JPRB, 0.86113E+00_JPRB, 0.92324E+00_JPRB, 0.96396E+00_JPRB, 0.87510E+00_JPRB, 0.78832E+00_JPRB, 0.71615E+00_JPRB ,&
&  0.66127E+00_JPRB, 0.62652E+00_JPRB, 0.62791E+00_JPRB, 0.61774E+00_JPRB, 0.61981E+00_JPRB, 0.64028E+00_JPRB, 0.66114E+00_JPRB ,&
&  0.70491E+00_JPRB, 0.72550E+00_JPRB, 0.77208E+00_JPRB, 0.81126E+00_JPRB, 0.85252E+00_JPRB, 0.88763E+00_JPRB, 0.91937E+00_JPRB ,&
&  0.94128E+00_JPRB, 0.95092E+00_JPRB, 0.95458E+00_JPRB, 0.98029E+00_JPRB, 0.98793E+00_JPRB, 0.10028E+01_JPRB, 0.10267E+01_JPRB ,&
&  0.10542E+01_JPRB, 0.10718E+01_JPRB, 0.10489E+01_JPRB, 0.10195E+01_JPRB, 0.97177E+00_JPRB, 0.92198E+00_JPRB, 0.71020E+00_JPRB ,&
&  0.67485E+00_JPRB, 0.63751E+00_JPRB, 0.65575E+00_JPRB, 0.67858E+00_JPRB, 0.67103E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.81419E+00_JPRB, 0.84990E+00_JPRB, 0.85665E+00_JPRB, 0.88567E+00_JPRB, 0.88506E+00_JPRB, 0.87532E+00_JPRB, 0.88940E+00_JPRB ,&
&  0.90810E+00_JPRB, 0.92356E+00_JPRB, 0.95087E+00_JPRB, 0.97832E+00_JPRB, 0.10002E+01_JPRB, 0.10105E+01_JPRB, 0.10309E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.11082E+01_JPRB, 0.11598E+01_JPRB, 0.12280E+01_JPRB, 0.12787E+01_JPRB, 0.13204E+01_JPRB, 0.13853E+01_JPRB ,&
&  0.14405E+01_JPRB, 0.15404E+01_JPRB, 0.16459E+01_JPRB, 0.17488E+01_JPRB, 0.18170E+01_JPRB, 0.18481E+01_JPRB, 0.14725E+01_JPRB ,&
&  0.19670E+01_JPRB, 0.19825E+01_JPRB, 0.18790E+01_JPRB, 0.16160E+01_JPRB, 0.14894E+01_JPRB, 0.14067E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.13029E+01_JPRB, 0.12445E+01_JPRB, 0.11796E+01_JPRB, 0.11390E+01_JPRB, 0.12334E+01_JPRB, 0.12540E+01_JPRB, 0.13280E+01_JPRB ,&
&  0.14073E+01_JPRB, 0.14855E+01_JPRB, 0.15617E+01_JPRB, 0.16388E+01_JPRB, 0.17470E+01_JPRB, 0.18699E+01_JPRB, 0.19803E+01_JPRB ,&
&  0.20662E+01_JPRB, 0.21093E+01_JPRB, 0.21929E+01_JPRB, 0.19421E+01_JPRB, 0.18676E+01_JPRB, 0.17698E+01_JPRB, 0.12604E+01_JPRB ,&
&  0.91238E+00_JPRB, 0.87187E+00_JPRB, 0.93953E+00_JPRB, 0.95284E+00_JPRB, 0.88914E+00_JPRB, 0.78725E+00_JPRB, 0.71884E+00_JPRB ,&
&  0.64800E+00_JPRB, 0.63574E+00_JPRB, 0.62695E+00_JPRB, 0.61480E+00_JPRB, 0.60802E+00_JPRB, 0.63808E+00_JPRB, 0.66612E+00_JPRB ,&
&  0.72855E+00_JPRB, 0.77637E+00_JPRB, 0.83679E+00_JPRB, 0.89331E+00_JPRB, 0.97415E+00_JPRB, 0.10745E+01_JPRB, 0.11718E+01_JPRB ,&
&  0.10182E+01_JPRB, 0.93665E+00_JPRB, 0.96178E+00_JPRB, 0.10169E+01_JPRB, 0.10522E+01_JPRB, 0.11386E+01_JPRB, 0.12419E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.14662E+01_JPRB, 0.13384E+01_JPRB, 0.11229E+01_JPRB, 0.14826E+01_JPRB, 0.15242E+01_JPRB, 0.72618E+00_JPRB ,&
&  0.71944E+00_JPRB, 0.75478E+00_JPRB, 0.77658E+00_JPRB, 0.80129E+00_JPRB, 0.82412E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.91191E+00_JPRB, 0.92855E+00_JPRB, 0.94397E+00_JPRB, 0.95512E+00_JPRB, 0.95814E+00_JPRB, 0.97220E+00_JPRB, 0.99734E+00_JPRB ,&
&  0.10208E+01_JPRB, 0.10438E+01_JPRB, 0.10662E+01_JPRB, 0.10856E+01_JPRB, 0.11061E+01_JPRB, 0.11216E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.11929E+01_JPRB, 0.12319E+01_JPRB, 0.12550E+01_JPRB, 0.12974E+01_JPRB, 0.13271E+01_JPRB, 0.13465E+01_JPRB, 0.14032E+01_JPRB ,&
&  0.14595E+01_JPRB, 0.15280E+01_JPRB, 0.15788E+01_JPRB, 0.16098E+01_JPRB, 0.16071E+01_JPRB, 0.17100E+01_JPRB, 0.15363E+01_JPRB ,&
&  0.18792E+01_JPRB, 0.20639E+01_JPRB, 0.19128E+01_JPRB, 0.15690E+01_JPRB, 0.14477E+01_JPRB, 0.13816E+01_JPRB, 0.13261E+01_JPRB ,&
&  0.12988E+01_JPRB, 0.12565E+01_JPRB, 0.11870E+01_JPRB, 0.12198E+01_JPRB, 0.12550E+01_JPRB, 0.13085E+01_JPRB, 0.13905E+01_JPRB ,&
&  0.14554E+01_JPRB, 0.15250E+01_JPRB, 0.15990E+01_JPRB, 0.16715E+01_JPRB, 0.17726E+01_JPRB, 0.18585E+01_JPRB, 0.19094E+01_JPRB ,&
&  0.19444E+01_JPRB, 0.19726E+01_JPRB, 0.19906E+01_JPRB, 0.18398E+01_JPRB, 0.18038E+01_JPRB, 0.16169E+01_JPRB, 0.12870E+01_JPRB ,&
&  0.95394E+00_JPRB, 0.85969E+00_JPRB, 0.90158E+00_JPRB, 0.94199E+00_JPRB, 0.90321E+00_JPRB, 0.79467E+00_JPRB, 0.75896E+00_JPRB ,&
&  0.66581E+00_JPRB, 0.61185E+00_JPRB, 0.60834E+00_JPRB, 0.59797E+00_JPRB, 0.59023E+00_JPRB, 0.62743E+00_JPRB, 0.66045E+00_JPRB ,&
&  0.71352E+00_JPRB, 0.78714E+00_JPRB, 0.84830E+00_JPRB, 0.89971E+00_JPRB, 0.10079E+01_JPRB, 0.11089E+01_JPRB, 0.10343E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10392E+01_JPRB, 0.10600E+01_JPRB, 0.11736E+01_JPRB, 0.11446E+01_JPRB, 0.13451E+01_JPRB, 0.15621E+01_JPRB ,&
&  0.18676E+01_JPRB, 0.18036E+01_JPRB, 0.21052E+01_JPRB, 0.31311E+01_JPRB, 0.26858E+01_JPRB, 0.22708E+01_JPRB, 0.86916E+00_JPRB ,&
&  0.86318E+00_JPRB, 0.88110E+00_JPRB, 0.90820E+00_JPRB, 0.91247E+00_JPRB, 0.91610E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.95300E+00_JPRB, 0.97206E+00_JPRB, 0.98655E+00_JPRB, 0.10071E+01_JPRB, 0.10328E+01_JPRB, 0.10625E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.11273E+01_JPRB, 0.11451E+01_JPRB, 0.11660E+01_JPRB, 0.11814E+01_JPRB, 0.12032E+01_JPRB, 0.12106E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12297E+01_JPRB, 0.12413E+01_JPRB, 0.12546E+01_JPRB, 0.12678E+01_JPRB, 0.12815E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.13178E+01_JPRB, 0.13170E+01_JPRB, 0.13282E+01_JPRB, 0.13468E+01_JPRB, 0.13538E+01_JPRB, 0.14478E+01_JPRB, 0.13351E+01_JPRB ,&
&  0.17530E+01_JPRB, 0.19351E+01_JPRB, 0.18375E+01_JPRB, 0.14855E+01_JPRB, 0.13540E+01_JPRB, 0.13052E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.12437E+01_JPRB, 0.12163E+01_JPRB, 0.12236E+01_JPRB, 0.12572E+01_JPRB, 0.13136E+01_JPRB, 0.13649E+01_JPRB ,&
&  0.13997E+01_JPRB, 0.14421E+01_JPRB, 0.15000E+01_JPRB, 0.15640E+01_JPRB, 0.16586E+01_JPRB, 0.17072E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.16852E+01_JPRB, 0.16680E+01_JPRB, 0.15273E+01_JPRB, 0.15227E+01_JPRB, 0.15304E+01_JPRB, 0.12643E+01_JPRB, 0.10581E+01_JPRB ,&
&  0.92353E+00_JPRB, 0.88083E+00_JPRB, 0.91876E+00_JPRB, 0.98842E+00_JPRB, 0.91998E+00_JPRB, 0.85366E+00_JPRB, 0.80113E+00_JPRB ,&
&  0.69367E+00_JPRB, 0.63698E+00_JPRB, 0.61557E+00_JPRB, 0.58827E+00_JPRB, 0.59034E+00_JPRB, 0.61461E+00_JPRB, 0.67209E+00_JPRB ,&
&  0.70797E+00_JPRB, 0.74587E+00_JPRB, 0.79524E+00_JPRB, 0.83293E+00_JPRB, 0.90687E+00_JPRB, 0.88862E+00_JPRB, 0.10271E+01_JPRB ,&
&  0.97386E+00_JPRB, 0.94992E+00_JPRB, 0.98578E+00_JPRB, 0.82367E+00_JPRB, 0.10702E+01_JPRB, 0.11533E+01_JPRB, 0.13021E+01_JPRB ,&
&  0.13487E+01_JPRB, 0.18123E+01_JPRB, 0.21849E+01_JPRB, 0.18409E+01_JPRB, 0.13523E+01_JPRB, 0.95285E+00_JPRB, 0.86103E+00_JPRB ,&
&  0.86159E+00_JPRB, 0.88594E+00_JPRB, 0.91007E+00_JPRB, 0.92307E+00_JPRB, 0.93785E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.95222E+00_JPRB, 0.97089E+00_JPRB, 0.98975E+00_JPRB, 0.10197E+01_JPRB, 0.10524E+01_JPRB, 0.10832E+01_JPRB, 0.11068E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.11474E+01_JPRB, 0.11687E+01_JPRB, 0.11873E+01_JPRB, 0.12004E+01_JPRB, 0.12083E+01_JPRB, 0.12090E+01_JPRB ,&
&  0.12111E+01_JPRB, 0.12155E+01_JPRB, 0.12305E+01_JPRB, 0.12427E+01_JPRB, 0.12433E+01_JPRB, 0.12541E+01_JPRB, 0.12759E+01_JPRB ,&
&  0.12899E+01_JPRB, 0.13036E+01_JPRB, 0.13373E+01_JPRB, 0.13859E+01_JPRB, 0.14533E+01_JPRB, 0.15971E+01_JPRB, 0.14673E+01_JPRB ,&
&  0.15750E+01_JPRB, 0.17555E+01_JPRB, 0.17399E+01_JPRB, 0.13817E+01_JPRB, 0.12513E+01_JPRB, 0.12050E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11629E+01_JPRB, 0.11619E+01_JPRB, 0.11828E+01_JPRB, 0.12207E+01_JPRB, 0.12633E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.13032E+01_JPRB, 0.13295E+01_JPRB, 0.13608E+01_JPRB, 0.13885E+01_JPRB, 0.14422E+01_JPRB, 0.14523E+01_JPRB, 0.14190E+01_JPRB ,&
&  0.13597E+01_JPRB, 0.13248E+01_JPRB, 0.11481E+01_JPRB, 0.10551E+01_JPRB, 0.10534E+01_JPRB, 0.93680E+00_JPRB, 0.84054E+00_JPRB ,&
&  0.89845E+00_JPRB, 0.89209E+00_JPRB, 0.87310E+00_JPRB, 0.10381E+01_JPRB, 0.10157E+01_JPRB, 0.90183E+00_JPRB, 0.86288E+00_JPRB ,&
&  0.75882E+00_JPRB, 0.66750E+00_JPRB, 0.61859E+00_JPRB, 0.61521E+00_JPRB, 0.62707E+00_JPRB, 0.66074E+00_JPRB, 0.66384E+00_JPRB ,&
&  0.67808E+00_JPRB, 0.70675E+00_JPRB, 0.73189E+00_JPRB, 0.76747E+00_JPRB, 0.90136E+00_JPRB, 0.87707E+00_JPRB, 0.10734E+01_JPRB ,&
&  0.84511E+00_JPRB, 0.71996E+00_JPRB, 0.79865E+00_JPRB, 0.91486E+00_JPRB, 0.97430E+00_JPRB, 0.10422E+01_JPRB, 0.11963E+01_JPRB ,&
&  0.13773E+01_JPRB, 0.15226E+01_JPRB, 0.13840E+01_JPRB, 0.11840E+01_JPRB, 0.97872E+00_JPRB, 0.89098E+00_JPRB, 0.85350E+00_JPRB ,&
&  0.84977E+00_JPRB, 0.87802E+00_JPRB, 0.90070E+00_JPRB, 0.91396E+00_JPRB, 0.93220E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.90634E+00_JPRB, 0.92732E+00_JPRB, 0.95002E+00_JPRB, 0.97587E+00_JPRB, 0.10008E+01_JPRB, 0.10238E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10799E+01_JPRB, 0.11031E+01_JPRB, 0.11263E+01_JPRB, 0.11398E+01_JPRB, 0.11463E+01_JPRB, 0.11611E+01_JPRB, 0.11657E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.11696E+01_JPRB, 0.11868E+01_JPRB, 0.11982E+01_JPRB, 0.12001E+01_JPRB, 0.11989E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.12210E+01_JPRB, 0.12567E+01_JPRB, 0.12853E+01_JPRB, 0.13220E+01_JPRB, 0.13529E+01_JPRB, 0.13651E+01_JPRB, 0.14500E+01_JPRB ,&
&  0.11366E+01_JPRB, 0.16667E+01_JPRB, 0.16121E+01_JPRB, 0.12741E+01_JPRB, 0.12022E+01_JPRB, 0.11287E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.10879E+01_JPRB, 0.10980E+01_JPRB, 0.11241E+01_JPRB, 0.11520E+01_JPRB, 0.11851E+01_JPRB, 0.11951E+01_JPRB ,&
&  0.12098E+01_JPRB, 0.12205E+01_JPRB, 0.12304E+01_JPRB, 0.12354E+01_JPRB, 0.12455E+01_JPRB, 0.12346E+01_JPRB, 0.11923E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.10624E+01_JPRB, 0.80203E+00_JPRB, 0.74815E+00_JPRB, 0.76098E+00_JPRB, 0.71630E+00_JPRB, 0.64706E+00_JPRB ,&
&  0.79089E+00_JPRB, 0.95604E+00_JPRB, 0.10256E+01_JPRB, 0.10952E+01_JPRB, 0.11034E+01_JPRB, 0.10365E+01_JPRB, 0.10001E+01_JPRB ,&
&  0.92189E+00_JPRB, 0.81623E+00_JPRB, 0.76004E+00_JPRB, 0.72810E+00_JPRB, 0.73855E+00_JPRB, 0.73157E+00_JPRB, 0.70730E+00_JPRB ,&
&  0.67075E+00_JPRB, 0.66345E+00_JPRB, 0.67617E+00_JPRB, 0.69432E+00_JPRB, 0.73864E+00_JPRB, 0.13546E+01_JPRB, 0.17309E+01_JPRB ,&
&  0.83557E+00_JPRB, 0.75658E+00_JPRB, 0.62613E+00_JPRB, 0.79546E+00_JPRB, 0.81639E+00_JPRB, 0.83072E+00_JPRB, 0.94628E+00_JPRB ,&
&  0.10446E+01_JPRB, 0.10604E+01_JPRB, 0.97609E+00_JPRB, 0.87557E+00_JPRB, 0.83580E+00_JPRB, 0.82070E+00_JPRB, 0.79516E+00_JPRB ,&
&  0.79169E+00_JPRB, 0.81694E+00_JPRB, 0.84176E+00_JPRB, 0.86213E+00_JPRB, 0.88410E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.85710E+00_JPRB, 0.85314E+00_JPRB, 0.86592E+00_JPRB, 0.88456E+00_JPRB, 0.90271E+00_JPRB, 0.92336E+00_JPRB, 0.95605E+00_JPRB ,&
&  0.99491E+00_JPRB, 0.10251E+01_JPRB, 0.10535E+01_JPRB, 0.10732E+01_JPRB, 0.10789E+01_JPRB, 0.10918E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.11207E+01_JPRB, 0.11167E+01_JPRB, 0.11345E+01_JPRB, 0.11646E+01_JPRB, 0.12008E+01_JPRB, 0.12150E+01_JPRB, 0.12292E+01_JPRB ,&
&  0.12472E+01_JPRB, 0.12898E+01_JPRB, 0.13383E+01_JPRB, 0.13965E+01_JPRB, 0.14953E+01_JPRB, 0.16687E+01_JPRB, 0.18383E+01_JPRB ,&
&  0.13689E+01_JPRB, 0.14873E+01_JPRB, 0.13888E+01_JPRB, 0.11336E+01_JPRB, 0.10552E+01_JPRB, 0.10708E+01_JPRB, 0.10303E+01_JPRB ,&
&  0.99350E+00_JPRB, 0.10006E+01_JPRB, 0.10097E+01_JPRB, 0.10175E+01_JPRB, 0.10369E+01_JPRB, 0.10713E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.10919E+01_JPRB, 0.10983E+01_JPRB, 0.11290E+01_JPRB, 0.10705E+01_JPRB, 0.12391E+01_JPRB, 0.11886E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.10750E+01_JPRB, 0.85319E+00_JPRB, 0.82820E+00_JPRB, 0.80609E+00_JPRB, 0.70123E+00_JPRB, 0.64558E+00_JPRB ,&
&  0.80806E+00_JPRB, 0.95770E+00_JPRB, 0.85801E+00_JPRB, 0.10738E+01_JPRB, 0.11920E+01_JPRB, 0.12386E+01_JPRB, 0.12573E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.11034E+01_JPRB, 0.99163E+00_JPRB, 0.93251E+00_JPRB, 0.91184E+00_JPRB, 0.87878E+00_JPRB, 0.82418E+00_JPRB ,&
&  0.77348E+00_JPRB, 0.74085E+00_JPRB, 0.70236E+00_JPRB, 0.68089E+00_JPRB, 0.86457E+00_JPRB, 0.10157E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.10044E+01_JPRB, 0.89387E+00_JPRB, 0.78750E+00_JPRB, 0.12349E+01_JPRB, 0.92466E+00_JPRB, 0.67151E+00_JPRB, 0.70682E+00_JPRB ,&
&  0.75882E+00_JPRB, 0.79453E+00_JPRB, 0.78508E+00_JPRB, 0.76232E+00_JPRB, 0.75600E+00_JPRB, 0.75826E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.72313E+00_JPRB, 0.73853E+00_JPRB, 0.77295E+00_JPRB, 0.78707E+00_JPRB, 0.82290E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.75359E+00_JPRB, 0.75837E+00_JPRB, 0.76465E+00_JPRB, 0.77428E+00_JPRB, 0.79486E+00_JPRB, 0.81812E+00_JPRB, 0.83668E+00_JPRB ,&
&  0.87116E+00_JPRB, 0.89752E+00_JPRB, 0.92349E+00_JPRB, 0.93515E+00_JPRB, 0.95010E+00_JPRB, 0.96110E+00_JPRB, 0.96528E+00_JPRB ,&
&  0.98740E+00_JPRB, 0.10150E+01_JPRB, 0.10157E+01_JPRB, 0.10246E+01_JPRB, 0.10517E+01_JPRB, 0.10704E+01_JPRB, 0.10549E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.10100E+01_JPRB, 0.10337E+01_JPRB, 0.10772E+01_JPRB, 0.12884E+01_JPRB, 0.14261E+01_JPRB, 0.14139E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.14238E+01_JPRB, 0.14716E+01_JPRB, 0.11464E+01_JPRB, 0.10457E+01_JPRB, 0.98258E+00_JPRB, 0.91878E+00_JPRB ,&
&  0.92404E+00_JPRB, 0.95347E+00_JPRB, 0.99672E+00_JPRB, 0.10395E+01_JPRB, 0.10949E+01_JPRB, 0.11178E+01_JPRB, 0.11694E+01_JPRB ,&
&  0.11666E+01_JPRB, 0.11823E+01_JPRB, 0.12015E+01_JPRB, 0.12360E+01_JPRB, 0.13313E+01_JPRB, 0.15087E+01_JPRB, 0.15045E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.15848E+01_JPRB, 0.14481E+01_JPRB, 0.13877E+01_JPRB, 0.12946E+01_JPRB, 0.10996E+01_JPRB, 0.93043E+00_JPRB ,&
&  0.90195E+00_JPRB, 0.88641E+00_JPRB, 0.77480E+00_JPRB, 0.10400E+01_JPRB, 0.12595E+01_JPRB, 0.13424E+01_JPRB, 0.15055E+01_JPRB ,&
&  0.15764E+01_JPRB, 0.14224E+01_JPRB, 0.12160E+01_JPRB, 0.12119E+01_JPRB, 0.12076E+01_JPRB, 0.10313E+01_JPRB, 0.97827E+00_JPRB ,&
&  0.87138E+00_JPRB, 0.78832E+00_JPRB, 0.73912E+00_JPRB, 0.73285E+00_JPRB, 0.74997E+00_JPRB, 0.92156E+00_JPRB, 0.12173E+01_JPRB ,&
&  0.10634E+01_JPRB, 0.97027E+00_JPRB, 0.12355E+01_JPRB, 0.20881E+01_JPRB, 0.11404E+01_JPRB, 0.79601E+00_JPRB, 0.62186E+00_JPRB ,&
&  0.61728E+00_JPRB, 0.62090E+00_JPRB, 0.62487E+00_JPRB, 0.63012E+00_JPRB, 0.63791E+00_JPRB, 0.65705E+00_JPRB, 0.65132E+00_JPRB ,&
&  0.64484E+00_JPRB, 0.63968E+00_JPRB, 0.65522E+00_JPRB, 0.67792E+00_JPRB, 0.72056E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.61143E+00_JPRB, 0.60864E+00_JPRB, 0.59608E+00_JPRB, 0.59048E+00_JPRB, 0.59678E+00_JPRB, 0.60926E+00_JPRB, 0.62771E+00_JPRB ,&
&  0.65667E+00_JPRB, 0.69258E+00_JPRB, 0.72837E+00_JPRB, 0.76990E+00_JPRB, 0.77728E+00_JPRB, 0.72722E+00_JPRB, 0.72359E+00_JPRB ,&
&  0.71671E+00_JPRB, 0.72814E+00_JPRB, 0.72109E+00_JPRB, 0.80322E+00_JPRB, 0.83026E+00_JPRB, 0.84594E+00_JPRB, 0.95259E+00_JPRB ,&
&  0.95360E+00_JPRB, 0.10681E+01_JPRB, 0.11601E+01_JPRB, 0.13003E+01_JPRB, 0.21207E+01_JPRB, 0.19587E+01_JPRB, 0.14371E+01_JPRB ,&
&  0.16702E+01_JPRB, 0.17413E+01_JPRB, 0.14185E+01_JPRB, 0.11854E+01_JPRB, 0.11234E+01_JPRB, 0.10217E+01_JPRB, 0.98494E+00_JPRB ,&
&  0.10513E+01_JPRB, 0.11508E+01_JPRB, 0.13207E+01_JPRB, 0.14827E+01_JPRB, 0.16093E+01_JPRB, 0.17311E+01_JPRB, 0.18794E+01_JPRB ,&
&  0.19663E+01_JPRB, 0.20366E+01_JPRB, 0.19706E+01_JPRB, 0.19233E+01_JPRB, 0.21685E+01_JPRB, 0.23720E+01_JPRB, 0.25107E+01_JPRB ,&
&  0.26072E+01_JPRB, 0.27401E+01_JPRB, 0.28936E+01_JPRB, 0.27127E+01_JPRB, 0.24016E+01_JPRB, 0.21902E+01_JPRB, 0.17637E+01_JPRB ,&
&  0.14202E+01_JPRB, 0.11699E+01_JPRB, 0.11035E+01_JPRB, 0.14783E+01_JPRB, 0.16653E+01_JPRB, 0.15447E+01_JPRB, 0.18151E+01_JPRB ,&
&  0.18897E+01_JPRB, 0.19248E+01_JPRB, 0.16945E+01_JPRB, 0.15117E+01_JPRB, 0.12912E+01_JPRB, 0.12132E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.86608E+00_JPRB, 0.78048E+00_JPRB, 0.73061E+00_JPRB, 0.68866E+00_JPRB, 0.73490E+00_JPRB, 0.10051E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.93544E+00_JPRB, 0.10186E+01_JPRB, 0.16213E+01_JPRB, 0.14405E+01_JPRB, 0.86596E+00_JPRB, 0.61581E+00_JPRB ,&
&  0.56467E+00_JPRB, 0.57182E+00_JPRB, 0.57093E+00_JPRB, 0.57129E+00_JPRB, 0.58450E+00_JPRB, 0.60146E+00_JPRB, 0.59298E+00_JPRB ,&
&  0.59375E+00_JPRB, 0.58351E+00_JPRB, 0.57663E+00_JPRB, 0.57539E+00_JPRB, 0.58515E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.57250E+00_JPRB, 0.57042E+00_JPRB, 0.59242E+00_JPRB, 0.66272E+00_JPRB, 0.71177E+00_JPRB, 0.74699E+00_JPRB, 0.80405E+00_JPRB ,&
&  0.87416E+00_JPRB, 0.98251E+00_JPRB, 0.10712E+01_JPRB, 0.11073E+01_JPRB, 0.10768E+01_JPRB, 0.10409E+01_JPRB, 0.10233E+01_JPRB ,&
&  0.10119E+01_JPRB, 0.92839E+00_JPRB, 0.86098E+00_JPRB, 0.85704E+00_JPRB, 0.91380E+00_JPRB, 0.10364E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.16629E+01_JPRB, 0.19226E+01_JPRB, 0.25198E+01_JPRB, 0.25883E+01_JPRB, 0.14147E+01_JPRB, 0.15259E+01_JPRB ,&
&  0.19897E+01_JPRB, 0.20407E+01_JPRB, 0.15307E+01_JPRB, 0.13563E+01_JPRB, 0.12780E+01_JPRB, 0.12318E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.14003E+01_JPRB, 0.15365E+01_JPRB, 0.16900E+01_JPRB, 0.18884E+01_JPRB, 0.21072E+01_JPRB, 0.23058E+01_JPRB, 0.25509E+01_JPRB ,&
&  0.27590E+01_JPRB, 0.28210E+01_JPRB, 0.28576E+01_JPRB, 0.29518E+01_JPRB, 0.31693E+01_JPRB, 0.34295E+01_JPRB, 0.36532E+01_JPRB ,&
&  0.37872E+01_JPRB, 0.38490E+01_JPRB, 0.39733E+01_JPRB, 0.39488E+01_JPRB, 0.35161E+01_JPRB, 0.31682E+01_JPRB, 0.26755E+01_JPRB ,&
&  0.23226E+01_JPRB, 0.19814E+01_JPRB, 0.18436E+01_JPRB, 0.24118E+01_JPRB, 0.25882E+01_JPRB, 0.23620E+01_JPRB, 0.23414E+01_JPRB ,&
&  0.23850E+01_JPRB, 0.23273E+01_JPRB, 0.20976E+01_JPRB, 0.16875E+01_JPRB, 0.13892E+01_JPRB, 0.14314E+01_JPRB, 0.13234E+01_JPRB ,&
&  0.12280E+01_JPRB, 0.10196E+01_JPRB, 0.89961E+00_JPRB, 0.82779E+00_JPRB, 0.77762E+00_JPRB, 0.87516E+00_JPRB, 0.92853E+00_JPRB ,&
&  0.89777E+00_JPRB, 0.10745E+01_JPRB, 0.10664E+01_JPRB, 0.16370E+01_JPRB, 0.25151E+01_JPRB, 0.12890E+01_JPRB, 0.79249E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.60104E+00_JPRB, 0.60685E+00_JPRB, 0.60894E+00_JPRB, 0.61280E+00_JPRB, 0.62767E+00_JPRB, 0.63148E+00_JPRB ,&
&  0.64650E+00_JPRB, 0.63978E+00_JPRB, 0.64403E+00_JPRB, 0.61875E+00_JPRB, 0.57586E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.80724E+00_JPRB, 0.83459E+00_JPRB, 0.88987E+00_JPRB, 0.97523E+00_JPRB, 0.10422E+01_JPRB, 0.10999E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.12410E+01_JPRB, 0.13126E+01_JPRB, 0.13716E+01_JPRB, 0.14207E+01_JPRB, 0.15184E+01_JPRB, 0.15814E+01_JPRB ,&
&  0.15945E+01_JPRB, 0.15780E+01_JPRB, 0.16141E+01_JPRB, 0.16822E+01_JPRB, 0.18465E+01_JPRB, 0.21027E+01_JPRB, 0.23587E+01_JPRB ,&
&  0.26182E+01_JPRB, 0.25555E+01_JPRB, 0.24175E+01_JPRB, 0.16345E+01_JPRB, 0.12406E+01_JPRB, 0.14346E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.16056E+01_JPRB, 0.16211E+01_JPRB, 0.14727E+01_JPRB, 0.13821E+01_JPRB, 0.13332E+01_JPRB, 0.13605E+01_JPRB, 0.14960E+01_JPRB ,&
&  0.16918E+01_JPRB, 0.19181E+01_JPRB, 0.21435E+01_JPRB, 0.23024E+01_JPRB, 0.24803E+01_JPRB, 0.27366E+01_JPRB, 0.30378E+01_JPRB ,&
&  0.33022E+01_JPRB, 0.34731E+01_JPRB, 0.35510E+01_JPRB, 0.37226E+01_JPRB, 0.38449E+01_JPRB, 0.40363E+01_JPRB, 0.41863E+01_JPRB ,&
&  0.43155E+01_JPRB, 0.43910E+01_JPRB, 0.44805E+01_JPRB, 0.45748E+01_JPRB, 0.44241E+01_JPRB, 0.39200E+01_JPRB, 0.34707E+01_JPRB ,&
&  0.32417E+01_JPRB, 0.29473E+01_JPRB, 0.30022E+01_JPRB, 0.32596E+01_JPRB, 0.23554E+01_JPRB, 0.24907E+01_JPRB, 0.28712E+01_JPRB ,&
&  0.28247E+01_JPRB, 0.27614E+01_JPRB, 0.25491E+01_JPRB, 0.22063E+01_JPRB, 0.20340E+01_JPRB, 0.18239E+01_JPRB, 0.16531E+01_JPRB ,&
&  0.14375E+01_JPRB, 0.11524E+01_JPRB, 0.99020E+00_JPRB, 0.86885E+00_JPRB, 0.92808E+00_JPRB, 0.11750E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.13735E+01_JPRB, 0.14157E+01_JPRB, 0.13710E+01_JPRB, 0.20210E+01_JPRB, 0.15859E+01_JPRB, 0.97699E+00_JPRB, 0.72156E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.61428E+00_JPRB, 0.62352E+00_JPRB, 0.61270E+00_JPRB, 0.62799E+00_JPRB, 0.65888E+00_JPRB, 0.66333E+00_JPRB ,&
&  0.71697E+00_JPRB, 0.74294E+00_JPRB, 0.77903E+00_JPRB, 0.81027E+00_JPRB, 0.79623E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.10139E+01_JPRB, 0.11164E+01_JPRB, 0.12015E+01_JPRB, 0.12996E+01_JPRB, 0.13809E+01_JPRB, 0.14904E+01_JPRB, 0.15191E+01_JPRB ,&
&  0.14693E+01_JPRB, 0.14302E+01_JPRB, 0.15361E+01_JPRB, 0.16710E+01_JPRB, 0.17916E+01_JPRB, 0.18905E+01_JPRB, 0.19822E+01_JPRB ,&
&  0.20627E+01_JPRB, 0.21320E+01_JPRB, 0.22069E+01_JPRB, 0.22664E+01_JPRB, 0.23710E+01_JPRB, 0.24981E+01_JPRB, 0.25628E+01_JPRB ,&
&  0.21694E+01_JPRB, 0.15867E+01_JPRB, 0.18727E+01_JPRB, 0.19910E+01_JPRB, 0.19110E+01_JPRB, 0.20178E+01_JPRB, 0.21362E+01_JPRB ,&
&  0.17901E+01_JPRB, 0.16075E+01_JPRB, 0.16270E+01_JPRB, 0.16253E+01_JPRB, 0.13443E+01_JPRB, 0.14675E+01_JPRB, 0.17379E+01_JPRB ,&
&  0.20105E+01_JPRB, 0.22699E+01_JPRB, 0.24964E+01_JPRB, 0.26680E+01_JPRB, 0.28527E+01_JPRB, 0.31137E+01_JPRB, 0.33803E+01_JPRB ,&
&  0.36319E+01_JPRB, 0.38109E+01_JPRB, 0.39124E+01_JPRB, 0.40227E+01_JPRB, 0.41811E+01_JPRB, 0.43984E+01_JPRB, 0.45169E+01_JPRB ,&
&  0.46439E+01_JPRB, 0.48028E+01_JPRB, 0.50424E+01_JPRB, 0.52815E+01_JPRB, 0.53504E+01_JPRB, 0.47573E+01_JPRB, 0.44583E+01_JPRB ,&
&  0.42144E+01_JPRB, 0.38447E+01_JPRB, 0.39269E+01_JPRB, 0.40676E+01_JPRB, 0.30448E+01_JPRB, 0.30070E+01_JPRB, 0.30882E+01_JPRB ,&
&  0.34219E+01_JPRB, 0.32826E+01_JPRB, 0.31409E+01_JPRB, 0.31817E+01_JPRB, 0.30089E+01_JPRB, 0.27332E+01_JPRB, 0.23926E+01_JPRB ,&
&  0.16977E+01_JPRB, 0.12205E+01_JPRB, 0.10189E+01_JPRB, 0.10215E+01_JPRB, 0.10516E+01_JPRB, 0.13807E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.21446E+01_JPRB, 0.15290E+01_JPRB, 0.14974E+01_JPRB, 0.18397E+01_JPRB, 0.11989E+01_JPRB, 0.82837E+00_JPRB, 0.66256E+00_JPRB ,&
&  0.61514E+00_JPRB, 0.64206E+00_JPRB, 0.65359E+00_JPRB, 0.64986E+00_JPRB, 0.65057E+00_JPRB, 0.68785E+00_JPRB, 0.73470E+00_JPRB ,&
&  0.77044E+00_JPRB, 0.81257E+00_JPRB, 0.86048E+00_JPRB, 0.91700E+00_JPRB, 0.98702E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.10712E+01_JPRB, 0.11321E+01_JPRB, 0.12576E+01_JPRB, 0.14275E+01_JPRB, 0.16676E+01_JPRB, 0.19208E+01_JPRB, 0.20265E+01_JPRB ,&
&  0.18275E+01_JPRB, 0.16224E+01_JPRB, 0.17728E+01_JPRB, 0.19160E+01_JPRB, 0.20335E+01_JPRB, 0.21715E+01_JPRB, 0.23054E+01_JPRB ,&
&  0.24559E+01_JPRB, 0.26071E+01_JPRB, 0.27453E+01_JPRB, 0.28940E+01_JPRB, 0.30307E+01_JPRB, 0.31517E+01_JPRB, 0.29335E+01_JPRB ,&
&  0.21606E+01_JPRB, 0.18569E+01_JPRB, 0.22376E+01_JPRB, 0.21325E+01_JPRB, 0.20976E+01_JPRB, 0.22321E+01_JPRB, 0.21720E+01_JPRB ,&
&  0.18474E+01_JPRB, 0.16471E+01_JPRB, 0.16904E+01_JPRB, 0.16453E+01_JPRB, 0.14697E+01_JPRB, 0.16557E+01_JPRB, 0.19973E+01_JPRB ,&
&  0.22222E+01_JPRB, 0.24464E+01_JPRB, 0.26981E+01_JPRB, 0.29271E+01_JPRB, 0.30806E+01_JPRB, 0.32877E+01_JPRB, 0.34804E+01_JPRB ,&
&  0.36412E+01_JPRB, 0.38130E+01_JPRB, 0.39067E+01_JPRB, 0.40080E+01_JPRB, 0.43112E+01_JPRB, 0.46747E+01_JPRB, 0.49633E+01_JPRB ,&
&  0.51002E+01_JPRB, 0.54843E+01_JPRB, 0.56534E+01_JPRB, 0.56037E+01_JPRB, 0.52324E+01_JPRB, 0.55936E+01_JPRB, 0.51193E+01_JPRB ,&
&  0.50909E+01_JPRB, 0.47015E+01_JPRB, 0.49724E+01_JPRB, 0.41195E+01_JPRB, 0.40491E+01_JPRB, 0.40707E+01_JPRB, 0.40504E+01_JPRB ,&
&  0.38879E+01_JPRB, 0.39603E+01_JPRB, 0.36770E+01_JPRB, 0.36146E+01_JPRB, 0.37085E+01_JPRB, 0.34614E+01_JPRB, 0.34680E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.14814E+01_JPRB, 0.10860E+01_JPRB, 0.10485E+01_JPRB, 0.10848E+01_JPRB, 0.14601E+01_JPRB, 0.22355E+01_JPRB ,&
&  0.30462E+01_JPRB, 0.23979E+01_JPRB, 0.20852E+01_JPRB, 0.18121E+01_JPRB, 0.11579E+01_JPRB, 0.87790E+00_JPRB, 0.71880E+00_JPRB ,&
&  0.63945E+00_JPRB, 0.65607E+00_JPRB, 0.70904E+00_JPRB, 0.72529E+00_JPRB, 0.71988E+00_JPRB, 0.75758E+00_JPRB, 0.81810E+00_JPRB ,&
&  0.85081E+00_JPRB, 0.88283E+00_JPRB, 0.95077E+00_JPRB, 0.97090E+00_JPRB, 0.10326E+01_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.10333E+01_JPRB, 0.10715E+01_JPRB, 0.11410E+01_JPRB, 0.12522E+01_JPRB, 0.13858E+01_JPRB, 0.15128E+01_JPRB, 0.15301E+01_JPRB ,&
&  0.15527E+01_JPRB, 0.16421E+01_JPRB, 0.17430E+01_JPRB, 0.18539E+01_JPRB, 0.19683E+01_JPRB, 0.21107E+01_JPRB, 0.22573E+01_JPRB ,&
&  0.24018E+01_JPRB, 0.25533E+01_JPRB, 0.27179E+01_JPRB, 0.28886E+01_JPRB, 0.29949E+01_JPRB, 0.29345E+01_JPRB, 0.21300E+01_JPRB ,&
&  0.15792E+01_JPRB, 0.25622E+01_JPRB, 0.20453E+01_JPRB, 0.21409E+01_JPRB, 0.21928E+01_JPRB, 0.23261E+01_JPRB, 0.19610E+01_JPRB ,&
&  0.16140E+01_JPRB, 0.15871E+01_JPRB, 0.16431E+01_JPRB, 0.16378E+01_JPRB, 0.17034E+01_JPRB, 0.18951E+01_JPRB, 0.21200E+01_JPRB ,&
&  0.22762E+01_JPRB, 0.24803E+01_JPRB, 0.27186E+01_JPRB, 0.29670E+01_JPRB, 0.31140E+01_JPRB, 0.32416E+01_JPRB, 0.33440E+01_JPRB ,&
&  0.34463E+01_JPRB, 0.35667E+01_JPRB, 0.36528E+01_JPRB, 0.39726E+01_JPRB, 0.42958E+01_JPRB, 0.47621E+01_JPRB, 0.53095E+01_JPRB ,&
&  0.54933E+01_JPRB, 0.55565E+01_JPRB, 0.60712E+01_JPRB, 0.61137E+01_JPRB, 0.60653E+01_JPRB, 0.63047E+01_JPRB, 0.54021E+01_JPRB ,&
&  0.56756E+01_JPRB, 0.52497E+01_JPRB, 0.56927E+01_JPRB, 0.47267E+01_JPRB, 0.46331E+01_JPRB, 0.50028E+01_JPRB, 0.49860E+01_JPRB ,&
&  0.45759E+01_JPRB, 0.46608E+01_JPRB, 0.43248E+01_JPRB, 0.40053E+01_JPRB, 0.39060E+01_JPRB, 0.39520E+01_JPRB, 0.42188E+01_JPRB ,&
&  0.34951E+01_JPRB, 0.19376E+01_JPRB, 0.13909E+01_JPRB, 0.11612E+01_JPRB, 0.11720E+01_JPRB, 0.15757E+01_JPRB, 0.26288E+01_JPRB ,&
&  0.40158E+01_JPRB, 0.40983E+01_JPRB, 0.41333E+01_JPRB, 0.29303E+01_JPRB, 0.17510E+01_JPRB, 0.13308E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.76926E+00_JPRB, 0.72458E+00_JPRB, 0.79667E+00_JPRB, 0.83147E+00_JPRB, 0.82634E+00_JPRB, 0.86987E+00_JPRB, 0.91376E+00_JPRB ,&
&  0.90110E+00_JPRB, 0.89294E+00_JPRB, 0.94920E+00_JPRB, 0.95167E+00_JPRB, 0.98884E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.99183E+00_JPRB, 0.98552E+00_JPRB, 0.10366E+01_JPRB, 0.10942E+01_JPRB, 0.11544E+01_JPRB, 0.12142E+01_JPRB, 0.12766E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.15296E+01_JPRB, 0.16112E+01_JPRB, 0.16598E+01_JPRB, 0.17225E+01_JPRB, 0.18607E+01_JPRB, 0.20494E+01_JPRB ,&
&  0.22832E+01_JPRB, 0.25471E+01_JPRB, 0.28390E+01_JPRB, 0.31372E+01_JPRB, 0.34113E+01_JPRB, 0.33801E+01_JPRB, 0.20459E+01_JPRB ,&
&  0.18157E+01_JPRB, 0.23324E+01_JPRB, 0.22315E+01_JPRB, 0.22418E+01_JPRB, 0.22088E+01_JPRB, 0.22235E+01_JPRB, 0.18589E+01_JPRB ,&
&  0.16996E+01_JPRB, 0.16555E+01_JPRB, 0.16831E+01_JPRB, 0.17539E+01_JPRB, 0.18415E+01_JPRB, 0.19796E+01_JPRB, 0.20253E+01_JPRB ,&
&  0.20579E+01_JPRB, 0.21794E+01_JPRB, 0.23190E+01_JPRB, 0.25652E+01_JPRB, 0.28420E+01_JPRB, 0.30623E+01_JPRB, 0.31976E+01_JPRB ,&
&  0.32750E+01_JPRB, 0.33403E+01_JPRB, 0.34850E+01_JPRB, 0.36487E+01_JPRB, 0.42715E+01_JPRB, 0.47303E+01_JPRB, 0.53473E+01_JPRB ,&
&  0.56993E+01_JPRB, 0.58260E+01_JPRB, 0.68104E+01_JPRB, 0.75944E+01_JPRB, 0.78793E+01_JPRB, 0.72056E+01_JPRB, 0.61183E+01_JPRB ,&
&  0.58384E+01_JPRB, 0.56626E+01_JPRB, 0.56965E+01_JPRB, 0.51768E+01_JPRB, 0.51325E+01_JPRB, 0.57139E+01_JPRB, 0.62672E+01_JPRB ,&
&  0.56051E+01_JPRB, 0.51191E+01_JPRB, 0.48692E+01_JPRB, 0.43892E+01_JPRB, 0.42028E+01_JPRB, 0.41969E+01_JPRB, 0.49191E+01_JPRB ,&
&  0.51176E+01_JPRB, 0.36677E+01_JPRB, 0.22518E+01_JPRB, 0.14252E+01_JPRB, 0.14282E+01_JPRB, 0.18071E+01_JPRB, 0.23602E+01_JPRB ,&
&  0.38359E+01_JPRB, 0.45006E+01_JPRB, 0.49105E+01_JPRB, 0.48901E+01_JPRB, 0.30248E+01_JPRB, 0.22243E+01_JPRB, 0.18047E+01_JPRB ,&
&  0.12365E+01_JPRB, 0.99243E+00_JPRB, 0.97381E+00_JPRB, 0.95850E+00_JPRB, 0.92928E+00_JPRB, 0.98761E+00_JPRB, 0.10844E+01_JPRB ,&
&  0.10320E+01_JPRB, 0.90373E+00_JPRB, 0.91097E+00_JPRB, 0.92183E+00_JPRB, 0.95015E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.97498E+00_JPRB, 0.91327E+00_JPRB, 0.92113E+00_JPRB, 0.93522E+00_JPRB, 0.99081E+00_JPRB, 0.10461E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.12694E+01_JPRB, 0.13841E+01_JPRB, 0.14588E+01_JPRB, 0.14644E+01_JPRB, 0.14322E+01_JPRB, 0.14864E+01_JPRB, 0.16177E+01_JPRB ,&
&  0.18401E+01_JPRB, 0.21902E+01_JPRB, 0.26693E+01_JPRB, 0.30467E+01_JPRB, 0.32181E+01_JPRB, 0.28935E+01_JPRB, 0.26225E+01_JPRB ,&
&  0.24307E+01_JPRB, 0.26996E+01_JPRB, 0.27776E+01_JPRB, 0.28902E+01_JPRB, 0.28656E+01_JPRB, 0.27527E+01_JPRB, 0.20163E+01_JPRB ,&
&  0.19895E+01_JPRB, 0.19808E+01_JPRB, 0.19618E+01_JPRB, 0.20221E+01_JPRB, 0.20720E+01_JPRB, 0.21348E+01_JPRB, 0.21100E+01_JPRB ,&
&  0.19689E+01_JPRB, 0.19477E+01_JPRB, 0.19659E+01_JPRB, 0.20905E+01_JPRB, 0.23802E+01_JPRB, 0.26634E+01_JPRB, 0.29548E+01_JPRB ,&
&  0.32159E+01_JPRB, 0.33505E+01_JPRB, 0.34765E+01_JPRB, 0.35031E+01_JPRB, 0.40111E+01_JPRB, 0.48291E+01_JPRB, 0.55747E+01_JPRB ,&
&  0.61497E+01_JPRB, 0.70872E+01_JPRB, 0.78430E+01_JPRB, 0.83854E+01_JPRB, 0.89789E+01_JPRB, 0.87383E+01_JPRB, 0.75499E+01_JPRB ,&
&  0.63899E+01_JPRB, 0.55889E+01_JPRB, 0.51532E+01_JPRB, 0.53030E+01_JPRB, 0.56590E+01_JPRB, 0.63944E+01_JPRB, 0.61238E+01_JPRB ,&
&  0.52926E+01_JPRB, 0.45484E+01_JPRB, 0.43378E+01_JPRB, 0.45294E+01_JPRB, 0.46820E+01_JPRB, 0.46012E+01_JPRB, 0.56583E+01_JPRB ,&
&  0.51591E+01_JPRB, 0.25015E+01_JPRB, 0.16369E+01_JPRB, 0.17515E+01_JPRB, 0.12087E+01_JPRB, 0.13926E+01_JPRB, 0.26630E+01_JPRB ,&
&  0.38250E+01_JPRB, 0.55981E+01_JPRB, 0.63381E+01_JPRB, 0.50709E+01_JPRB, 0.47135E+01_JPRB, 0.37611E+01_JPRB, 0.30677E+01_JPRB ,&
&  0.21451E+01_JPRB, 0.15506E+01_JPRB, 0.11834E+01_JPRB, 0.10882E+01_JPRB, 0.10311E+01_JPRB, 0.10519E+01_JPRB, 0.10829E+01_JPRB ,&
&  0.10349E+01_JPRB, 0.93960E+00_JPRB, 0.93626E+00_JPRB, 0.95542E+00_JPRB, 0.96357E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.10888E+01_JPRB, 0.96789E+00_JPRB, 0.89089E+00_JPRB, 0.85040E+00_JPRB, 0.93077E+00_JPRB, 0.10345E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.13153E+01_JPRB, 0.13606E+01_JPRB, 0.13529E+01_JPRB, 0.13028E+01_JPRB, 0.12925E+01_JPRB, 0.13294E+01_JPRB ,&
&  0.14432E+01_JPRB, 0.17067E+01_JPRB, 0.22118E+01_JPRB, 0.27716E+01_JPRB, 0.30497E+01_JPRB, 0.30328E+01_JPRB, 0.31212E+01_JPRB ,&
&  0.28269E+01_JPRB, 0.32335E+01_JPRB, 0.40737E+01_JPRB, 0.43620E+01_JPRB, 0.34853E+01_JPRB, 0.30807E+01_JPRB, 0.23714E+01_JPRB ,&
&  0.23118E+01_JPRB, 0.23630E+01_JPRB, 0.23973E+01_JPRB, 0.24772E+01_JPRB, 0.25057E+01_JPRB, 0.24680E+01_JPRB, 0.23991E+01_JPRB ,&
&  0.21633E+01_JPRB, 0.20099E+01_JPRB, 0.19115E+01_JPRB, 0.18982E+01_JPRB, 0.20716E+01_JPRB, 0.23766E+01_JPRB, 0.27184E+01_JPRB ,&
&  0.31314E+01_JPRB, 0.34301E+01_JPRB, 0.36001E+01_JPRB, 0.39981E+01_JPRB, 0.32938E+01_JPRB, 0.48037E+01_JPRB, 0.60445E+01_JPRB ,&
&  0.69037E+01_JPRB, 0.77985E+01_JPRB, 0.79968E+01_JPRB, 0.95567E+01_JPRB, 0.10464E+02_JPRB, 0.97534E+01_JPRB, 0.85126E+01_JPRB ,&
&  0.73648E+01_JPRB, 0.57734E+01_JPRB, 0.54750E+01_JPRB, 0.57055E+01_JPRB, 0.58604E+01_JPRB, 0.65737E+01_JPRB, 0.47824E+01_JPRB ,&
&  0.45266E+01_JPRB, 0.54388E+01_JPRB, 0.49970E+01_JPRB, 0.42879E+01_JPRB, 0.47087E+01_JPRB, 0.46921E+01_JPRB, 0.42401E+01_JPRB ,&
&  0.19459E+01_JPRB, 0.12349E+01_JPRB, 0.11561E+01_JPRB, 0.10414E+01_JPRB, 0.90985E+00_JPRB, 0.13599E+01_JPRB, 0.46871E+01_JPRB ,&
&  0.38759E+01_JPRB, 0.65805E+01_JPRB, 0.74174E+01_JPRB, 0.63140E+01_JPRB, 0.57379E+01_JPRB, 0.53567E+01_JPRB, 0.50531E+01_JPRB ,&
&  0.33563E+01_JPRB, 0.22435E+01_JPRB, 0.14750E+01_JPRB, 0.12140E+01_JPRB, 0.11675E+01_JPRB, 0.11365E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10366E+01_JPRB, 0.99904E+00_JPRB, 0.10154E+01_JPRB, 0.10012E+01_JPRB, 0.10794E+01_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.12718E+01_JPRB, 0.11291E+01_JPRB, 0.98952E+00_JPRB, 0.96303E+00_JPRB, 0.10337E+01_JPRB, 0.10675E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.12863E+01_JPRB, 0.13361E+01_JPRB, 0.13085E+01_JPRB, 0.12675E+01_JPRB, 0.12438E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.12730E+01_JPRB, 0.14203E+01_JPRB, 0.18182E+01_JPRB, 0.21512E+01_JPRB, 0.25764E+01_JPRB, 0.25960E+01_JPRB, 0.31041E+01_JPRB ,&
&  0.30773E+01_JPRB, 0.33836E+01_JPRB, 0.41058E+01_JPRB, 0.42634E+01_JPRB, 0.44977E+01_JPRB, 0.36669E+01_JPRB, 0.32422E+01_JPRB ,&
&  0.27119E+01_JPRB, 0.25375E+01_JPRB, 0.25089E+01_JPRB, 0.26717E+01_JPRB, 0.28436E+01_JPRB, 0.28681E+01_JPRB, 0.27156E+01_JPRB ,&
&  0.23227E+01_JPRB, 0.21107E+01_JPRB, 0.18652E+01_JPRB, 0.18229E+01_JPRB, 0.18073E+01_JPRB, 0.19779E+01_JPRB, 0.23465E+01_JPRB ,&
&  0.29865E+01_JPRB, 0.36099E+01_JPRB, 0.37834E+01_JPRB, 0.40705E+01_JPRB, 0.40162E+01_JPRB, 0.41890E+01_JPRB, 0.61107E+01_JPRB ,&
&  0.79102E+01_JPRB, 0.93676E+01_JPRB, 0.98755E+01_JPRB, 0.12042E+02_JPRB, 0.11655E+02_JPRB, 0.10465E+02_JPRB, 0.97807E+01_JPRB ,&
&  0.82788E+01_JPRB, 0.72366E+01_JPRB, 0.57874E+01_JPRB, 0.57284E+01_JPRB, 0.58699E+01_JPRB, 0.48136E+01_JPRB, 0.46992E+01_JPRB ,&
&  0.56366E+01_JPRB, 0.54566E+01_JPRB, 0.48462E+01_JPRB, 0.40212E+01_JPRB, 0.48028E+01_JPRB, 0.39598E+01_JPRB, 0.20310E+01_JPRB ,&
&  0.16178E+01_JPRB, 0.14021E+01_JPRB, 0.10993E+01_JPRB, 0.95698E+00_JPRB, 0.96084E+00_JPRB, 0.13584E+01_JPRB, 0.28387E+01_JPRB ,&
&  0.39562E+01_JPRB, 0.74344E+01_JPRB, 0.85484E+01_JPRB, 0.73729E+01_JPRB, 0.61706E+01_JPRB, 0.68166E+01_JPRB, 0.58180E+01_JPRB ,&
&  0.49362E+01_JPRB, 0.36880E+01_JPRB, 0.18872E+01_JPRB, 0.13759E+01_JPRB, 0.12729E+01_JPRB, 0.12124E+01_JPRB, 0.11458E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10737E+01_JPRB, 0.11126E+01_JPRB, 0.11524E+01_JPRB, 0.11741E+01_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.13358E+01_JPRB, 0.12465E+01_JPRB, 0.11098E+01_JPRB, 0.10981E+01_JPRB, 0.11377E+01_JPRB, 0.11950E+01_JPRB, 0.11991E+01_JPRB ,&
&  0.12244E+01_JPRB, 0.12704E+01_JPRB, 0.13212E+01_JPRB, 0.13005E+01_JPRB, 0.12414E+01_JPRB, 0.12313E+01_JPRB, 0.12133E+01_JPRB ,&
&  0.12439E+01_JPRB, 0.13928E+01_JPRB, 0.15847E+01_JPRB, 0.16719E+01_JPRB, 0.18344E+01_JPRB, 0.19649E+01_JPRB, 0.25046E+01_JPRB ,&
&  0.30719E+01_JPRB, 0.33628E+01_JPRB, 0.38053E+01_JPRB, 0.43888E+01_JPRB, 0.49594E+01_JPRB, 0.43619E+01_JPRB, 0.43126E+01_JPRB ,&
&  0.36191E+01_JPRB, 0.30582E+01_JPRB, 0.28806E+01_JPRB, 0.28449E+01_JPRB, 0.30168E+01_JPRB, 0.31170E+01_JPRB, 0.30043E+01_JPRB ,&
&  0.25879E+01_JPRB, 0.21436E+01_JPRB, 0.19029E+01_JPRB, 0.17484E+01_JPRB, 0.16670E+01_JPRB, 0.17475E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.29406E+01_JPRB, 0.39013E+01_JPRB, 0.42811E+01_JPRB, 0.42866E+01_JPRB, 0.54560E+01_JPRB, 0.59118E+01_JPRB, 0.67563E+01_JPRB ,&
&  0.75020E+01_JPRB, 0.81879E+01_JPRB, 0.11801E+02_JPRB, 0.12762E+02_JPRB, 0.12240E+02_JPRB, 0.11457E+02_JPRB, 0.10151E+02_JPRB ,&
&  0.88707E+01_JPRB, 0.78270E+01_JPRB, 0.63509E+01_JPRB, 0.62289E+01_JPRB, 0.53266E+01_JPRB, 0.45268E+01_JPRB, 0.54595E+01_JPRB ,&
&  0.61632E+01_JPRB, 0.59687E+01_JPRB, 0.49801E+01_JPRB, 0.40373E+01_JPRB, 0.38101E+01_JPRB, 0.25868E+01_JPRB, 0.16800E+01_JPRB ,&
&  0.14510E+01_JPRB, 0.13658E+01_JPRB, 0.11514E+01_JPRB, 0.12542E+01_JPRB, 0.11332E+01_JPRB, 0.14591E+01_JPRB, 0.24180E+01_JPRB ,&
&  0.47914E+01_JPRB, 0.73181E+01_JPRB, 0.82908E+01_JPRB, 0.71123E+01_JPRB, 0.63619E+01_JPRB, 0.66050E+01_JPRB, 0.77342E+01_JPRB ,&
&  0.55125E+01_JPRB, 0.47647E+01_JPRB, 0.30744E+01_JPRB, 0.19200E+01_JPRB, 0.14819E+01_JPRB, 0.13057E+01_JPRB, 0.12616E+01_JPRB ,&
&  0.12589E+01_JPRB, 0.12489E+01_JPRB, 0.12665E+01_JPRB, 0.12946E+01_JPRB, 0.12890E+01_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.13365E+01_JPRB, 0.13207E+01_JPRB, 0.12350E+01_JPRB, 0.12328E+01_JPRB, 0.12799E+01_JPRB, 0.13188E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.12890E+01_JPRB, 0.12934E+01_JPRB, 0.12969E+01_JPRB, 0.12635E+01_JPRB, 0.12290E+01_JPRB, 0.12184E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.14740E+01_JPRB, 0.12666E+01_JPRB, 0.12552E+01_JPRB, 0.13987E+01_JPRB, 0.15796E+01_JPRB, 0.18509E+01_JPRB ,&
&  0.27158E+01_JPRB, 0.33306E+01_JPRB, 0.37761E+01_JPRB, 0.47361E+01_JPRB, 0.61473E+01_JPRB, 0.58379E+01_JPRB, 0.55151E+01_JPRB ,&
&  0.47368E+01_JPRB, 0.39457E+01_JPRB, 0.36707E+01_JPRB, 0.34762E+01_JPRB, 0.34390E+01_JPRB, 0.34111E+01_JPRB, 0.32079E+01_JPRB ,&
&  0.27767E+01_JPRB, 0.22136E+01_JPRB, 0.18651E+01_JPRB, 0.18233E+01_JPRB, 0.17503E+01_JPRB, 0.18193E+01_JPRB, 0.20694E+01_JPRB ,&
&  0.29732E+01_JPRB, 0.41253E+01_JPRB, 0.48685E+01_JPRB, 0.53774E+01_JPRB, 0.54279E+01_JPRB, 0.64844E+01_JPRB, 0.10204E+02_JPRB ,&
&  0.10304E+02_JPRB, 0.11083E+02_JPRB, 0.12348E+02_JPRB, 0.14406E+02_JPRB, 0.13368E+02_JPRB, 0.11068E+02_JPRB, 0.10509E+02_JPRB ,&
&  0.74168E+01_JPRB, 0.60795E+01_JPRB, 0.63662E+01_JPRB, 0.62688E+01_JPRB, 0.52430E+01_JPRB, 0.57860E+01_JPRB, 0.84897E+01_JPRB ,&
&  0.68037E+01_JPRB, 0.71840E+01_JPRB, 0.72857E+01_JPRB, 0.71882E+01_JPRB, 0.40065E+01_JPRB, 0.29790E+01_JPRB, 0.45280E+01_JPRB ,&
&  0.48769E+01_JPRB, 0.28921E+01_JPRB, 0.17449E+01_JPRB, 0.24872E+01_JPRB, 0.17352E+01_JPRB, 0.20329E+01_JPRB, 0.30739E+01_JPRB ,&
&  0.40462E+01_JPRB, 0.53489E+01_JPRB, 0.86424E+01_JPRB, 0.76686E+01_JPRB, 0.68170E+01_JPRB, 0.69183E+01_JPRB, 0.65335E+01_JPRB ,&
&  0.65601E+01_JPRB, 0.51511E+01_JPRB, 0.41525E+01_JPRB, 0.27351E+01_JPRB, 0.19420E+01_JPRB, 0.16516E+01_JPRB, 0.15479E+01_JPRB ,&
&  0.15103E+01_JPRB, 0.15341E+01_JPRB, 0.15254E+01_JPRB, 0.14644E+01_JPRB, 0.13854E+01_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.14906E+01_JPRB, 0.14680E+01_JPRB, 0.13840E+01_JPRB, 0.13961E+01_JPRB, 0.14523E+01_JPRB, 0.14225E+01_JPRB, 0.13849E+01_JPRB ,&
&  0.13271E+01_JPRB, 0.12915E+01_JPRB, 0.12974E+01_JPRB, 0.12871E+01_JPRB, 0.12601E+01_JPRB, 0.12438E+01_JPRB, 0.12308E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.13089E+01_JPRB, 0.11970E+01_JPRB, 0.11659E+01_JPRB, 0.13021E+01_JPRB, 0.13938E+01_JPRB, 0.16738E+01_JPRB ,&
&  0.24193E+01_JPRB, 0.32111E+01_JPRB, 0.40601E+01_JPRB, 0.50972E+01_JPRB, 0.58832E+01_JPRB, 0.65310E+01_JPRB, 0.67666E+01_JPRB ,&
&  0.65215E+01_JPRB, 0.54743E+01_JPRB, 0.48397E+01_JPRB, 0.43574E+01_JPRB, 0.39336E+01_JPRB, 0.35922E+01_JPRB, 0.33036E+01_JPRB ,&
&  0.30396E+01_JPRB, 0.25974E+01_JPRB, 0.21306E+01_JPRB, 0.18772E+01_JPRB, 0.18669E+01_JPRB, 0.18911E+01_JPRB, 0.20807E+01_JPRB ,&
&  0.28743E+01_JPRB, 0.39650E+01_JPRB, 0.53068E+01_JPRB, 0.66634E+01_JPRB, 0.64358E+01_JPRB, 0.73137E+01_JPRB, 0.97888E+01_JPRB ,&
&  0.11167E+02_JPRB, 0.11515E+02_JPRB, 0.13611E+02_JPRB, 0.14852E+02_JPRB, 0.15040E+02_JPRB, 0.12453E+02_JPRB, 0.12203E+02_JPRB ,&
&  0.87223E+01_JPRB, 0.68402E+01_JPRB, 0.60319E+01_JPRB, 0.55302E+01_JPRB, 0.64698E+01_JPRB, 0.90830E+01_JPRB, 0.84792E+01_JPRB ,&
&  0.81103E+01_JPRB, 0.77335E+01_JPRB, 0.71033E+01_JPRB, 0.60835E+01_JPRB, 0.56935E+01_JPRB, 0.50580E+01_JPRB, 0.63865E+01_JPRB ,&
&  0.62907E+01_JPRB, 0.60033E+01_JPRB, 0.49862E+01_JPRB, 0.40103E+01_JPRB, 0.31791E+01_JPRB, 0.35403E+01_JPRB, 0.36294E+01_JPRB ,&
&  0.38835E+01_JPRB, 0.42099E+01_JPRB, 0.66607E+01_JPRB, 0.71556E+01_JPRB, 0.65333E+01_JPRB, 0.58537E+01_JPRB, 0.60060E+01_JPRB ,&
&  0.54237E+01_JPRB, 0.54582E+01_JPRB, 0.45273E+01_JPRB, 0.32468E+01_JPRB, 0.25289E+01_JPRB, 0.22187E+01_JPRB, 0.20405E+01_JPRB ,&
&  0.19977E+01_JPRB, 0.19570E+01_JPRB, 0.18615E+01_JPRB, 0.17266E+01_JPRB, 0.16077E+01_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.16830E+01_JPRB, 0.16216E+01_JPRB, 0.15566E+01_JPRB, 0.15324E+01_JPRB, 0.15588E+01_JPRB, 0.15227E+01_JPRB, 0.14800E+01_JPRB ,&
&  0.14201E+01_JPRB, 0.13595E+01_JPRB, 0.13408E+01_JPRB, 0.13208E+01_JPRB, 0.12890E+01_JPRB, 0.12638E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.13060E+01_JPRB, 0.13382E+01_JPRB, 0.12053E+01_JPRB, 0.12470E+01_JPRB, 0.12845E+01_JPRB, 0.13721E+01_JPRB, 0.18189E+01_JPRB ,&
&  0.23154E+01_JPRB, 0.29112E+01_JPRB, 0.37261E+01_JPRB, 0.46656E+01_JPRB, 0.54337E+01_JPRB, 0.63140E+01_JPRB, 0.72133E+01_JPRB ,&
&  0.67674E+01_JPRB, 0.66818E+01_JPRB, 0.58330E+01_JPRB, 0.49612E+01_JPRB, 0.42274E+01_JPRB, 0.36736E+01_JPRB, 0.33911E+01_JPRB ,&
&  0.31649E+01_JPRB, 0.30172E+01_JPRB, 0.26969E+01_JPRB, 0.23248E+01_JPRB, 0.21480E+01_JPRB, 0.21063E+01_JPRB, 0.21647E+01_JPRB ,&
&  0.24406E+01_JPRB, 0.35138E+01_JPRB, 0.52593E+01_JPRB, 0.75972E+01_JPRB, 0.73783E+01_JPRB, 0.85900E+01_JPRB, 0.94545E+01_JPRB ,&
&  0.13732E+02_JPRB, 0.14623E+02_JPRB, 0.14580E+02_JPRB, 0.15244E+02_JPRB, 0.12280E+02_JPRB, 0.12618E+02_JPRB, 0.12620E+02_JPRB ,&
&  0.10839E+02_JPRB, 0.99506E+01_JPRB, 0.10588E+02_JPRB, 0.84751E+01_JPRB, 0.74042E+01_JPRB, 0.88052E+01_JPRB, 0.74654E+01_JPRB ,&
&  0.67981E+01_JPRB, 0.64736E+01_JPRB, 0.59504E+01_JPRB, 0.63467E+01_JPRB, 0.58789E+01_JPRB, 0.58697E+01_JPRB, 0.48461E+01_JPRB ,&
&  0.46551E+01_JPRB, 0.38624E+01_JPRB, 0.42847E+01_JPRB, 0.37761E+01_JPRB, 0.35588E+01_JPRB, 0.34243E+01_JPRB, 0.33147E+01_JPRB ,&
&  0.36167E+01_JPRB, 0.37868E+01_JPRB, 0.46473E+01_JPRB, 0.54653E+01_JPRB, 0.58231E+01_JPRB, 0.48572E+01_JPRB, 0.46071E+01_JPRB ,&
&  0.47556E+01_JPRB, 0.49093E+01_JPRB, 0.41953E+01_JPRB, 0.33675E+01_JPRB, 0.28113E+01_JPRB, 0.25961E+01_JPRB, 0.24723E+01_JPRB ,&
&  0.23208E+01_JPRB, 0.21712E+01_JPRB, 0.20440E+01_JPRB, 0.19508E+01_JPRB, 0.18503E+01_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.17197E+01_JPRB, 0.16276E+01_JPRB, 0.16768E+01_JPRB, 0.16386E+01_JPRB, 0.16357E+01_JPRB, 0.15896E+01_JPRB, 0.15742E+01_JPRB ,&
&  0.15372E+01_JPRB, 0.14819E+01_JPRB, 0.14212E+01_JPRB, 0.13627E+01_JPRB, 0.13154E+01_JPRB, 0.12807E+01_JPRB, 0.12740E+01_JPRB ,&
&  0.12942E+01_JPRB, 0.14665E+01_JPRB, 0.13427E+01_JPRB, 0.12654E+01_JPRB, 0.12579E+01_JPRB, 0.14811E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.22032E+01_JPRB, 0.25364E+01_JPRB, 0.31833E+01_JPRB, 0.38655E+01_JPRB, 0.46052E+01_JPRB, 0.53506E+01_JPRB, 0.62663E+01_JPRB ,&
&  0.69521E+01_JPRB, 0.58909E+01_JPRB, 0.55217E+01_JPRB, 0.48524E+01_JPRB, 0.42887E+01_JPRB, 0.37966E+01_JPRB, 0.34629E+01_JPRB ,&
&  0.31922E+01_JPRB, 0.30126E+01_JPRB, 0.27802E+01_JPRB, 0.25054E+01_JPRB, 0.22413E+01_JPRB, 0.21098E+01_JPRB, 0.20934E+01_JPRB ,&
&  0.22171E+01_JPRB, 0.27586E+01_JPRB, 0.41665E+01_JPRB, 0.63869E+01_JPRB, 0.10032E+02_JPRB, 0.11869E+02_JPRB, 0.11853E+02_JPRB ,&
&  0.10516E+02_JPRB, 0.10797E+02_JPRB, 0.16461E+02_JPRB, 0.13549E+02_JPRB, 0.13538E+02_JPRB, 0.14133E+02_JPRB, 0.12930E+02_JPRB ,&
&  0.11207E+02_JPRB, 0.91273E+01_JPRB, 0.85907E+01_JPRB, 0.74999E+01_JPRB, 0.84695E+01_JPRB, 0.72391E+01_JPRB, 0.62257E+01_JPRB ,&
&  0.56389E+01_JPRB, 0.56482E+01_JPRB, 0.54374E+01_JPRB, 0.53937E+01_JPRB, 0.56667E+01_JPRB, 0.65094E+01_JPRB, 0.65043E+01_JPRB ,&
&  0.57238E+01_JPRB, 0.58733E+01_JPRB, 0.47833E+01_JPRB, 0.37644E+01_JPRB, 0.30421E+01_JPRB, 0.30167E+01_JPRB, 0.30838E+01_JPRB ,&
&  0.32354E+01_JPRB, 0.34720E+01_JPRB, 0.40443E+01_JPRB, 0.44590E+01_JPRB, 0.46629E+01_JPRB, 0.43770E+01_JPRB, 0.39865E+01_JPRB ,&
&  0.37475E+01_JPRB, 0.41037E+01_JPRB, 0.41062E+01_JPRB, 0.33098E+01_JPRB, 0.28631E+01_JPRB, 0.26900E+01_JPRB, 0.25571E+01_JPRB ,&
&  0.23605E+01_JPRB, 0.21604E+01_JPRB, 0.20205E+01_JPRB, 0.19494E+01_JPRB, 0.18865E+01_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.17931E+01_JPRB, 0.16269E+01_JPRB, 0.16714E+01_JPRB, 0.16621E+01_JPRB, 0.16298E+01_JPRB, 0.15915E+01_JPRB, 0.15628E+01_JPRB ,&
&  0.15254E+01_JPRB, 0.14993E+01_JPRB, 0.14624E+01_JPRB, 0.14005E+01_JPRB, 0.13498E+01_JPRB, 0.13132E+01_JPRB, 0.12935E+01_JPRB ,&
&  0.12904E+01_JPRB, 0.15475E+01_JPRB, 0.14970E+01_JPRB, 0.15206E+01_JPRB, 0.14859E+01_JPRB, 0.16383E+01_JPRB, 0.17866E+01_JPRB ,&
&  0.20170E+01_JPRB, 0.22002E+01_JPRB, 0.24708E+01_JPRB, 0.29030E+01_JPRB, 0.33872E+01_JPRB, 0.38777E+01_JPRB, 0.45865E+01_JPRB ,&
&  0.47940E+01_JPRB, 0.47160E+01_JPRB, 0.44563E+01_JPRB, 0.45483E+01_JPRB, 0.43511E+01_JPRB, 0.37401E+01_JPRB, 0.34970E+01_JPRB ,&
&  0.31173E+01_JPRB, 0.28553E+01_JPRB, 0.26504E+01_JPRB, 0.25281E+01_JPRB, 0.24202E+01_JPRB, 0.22880E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.21373E+01_JPRB, 0.22642E+01_JPRB, 0.29368E+01_JPRB, 0.46066E+01_JPRB, 0.75716E+01_JPRB, 0.10399E+02_JPRB, 0.11340E+02_JPRB ,&
&  0.12563E+02_JPRB, 0.12197E+02_JPRB, 0.97378E+01_JPRB, 0.11155E+02_JPRB, 0.14358E+02_JPRB, 0.13268E+02_JPRB, 0.11586E+02_JPRB ,&
&  0.11366E+02_JPRB, 0.10454E+02_JPRB, 0.99388E+01_JPRB, 0.86074E+01_JPRB, 0.72829E+01_JPRB, 0.65343E+01_JPRB, 0.59675E+01_JPRB ,&
&  0.55663E+01_JPRB, 0.55129E+01_JPRB, 0.55518E+01_JPRB, 0.55714E+01_JPRB, 0.53035E+01_JPRB, 0.53693E+01_JPRB, 0.58697E+01_JPRB ,&
&  0.57882E+01_JPRB, 0.52599E+01_JPRB, 0.38547E+01_JPRB, 0.37303E+01_JPRB, 0.26804E+01_JPRB, 0.25677E+01_JPRB, 0.29002E+01_JPRB ,&
&  0.28209E+01_JPRB, 0.31881E+01_JPRB, 0.35569E+01_JPRB, 0.33087E+01_JPRB, 0.35046E+01_JPRB, 0.34499E+01_JPRB, 0.35005E+01_JPRB ,&
&  0.34229E+01_JPRB, 0.30542E+01_JPRB, 0.34601E+01_JPRB, 0.30203E+01_JPRB, 0.27133E+01_JPRB, 0.25916E+01_JPRB, 0.24869E+01_JPRB ,&
&  0.23873E+01_JPRB, 0.22560E+01_JPRB, 0.21216E+01_JPRB, 0.20410E+01_JPRB, 0.19576E+01_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.18316E+01_JPRB, 0.16567E+01_JPRB, 0.16362E+01_JPRB, 0.16084E+01_JPRB, 0.15709E+01_JPRB, 0.15333E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.14870E+01_JPRB, 0.14663E+01_JPRB, 0.14369E+01_JPRB, 0.13936E+01_JPRB, 0.13543E+01_JPRB, 0.13116E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.14350E+01_JPRB, 0.14795E+01_JPRB, 0.14228E+01_JPRB, 0.16214E+01_JPRB, 0.16185E+01_JPRB, 0.16830E+01_JPRB ,&
&  0.17678E+01_JPRB, 0.19233E+01_JPRB, 0.19695E+01_JPRB, 0.20438E+01_JPRB, 0.22042E+01_JPRB, 0.25653E+01_JPRB, 0.30224E+01_JPRB ,&
&  0.31663E+01_JPRB, 0.30387E+01_JPRB, 0.30276E+01_JPRB, 0.31931E+01_JPRB, 0.34116E+01_JPRB, 0.30363E+01_JPRB, 0.29096E+01_JPRB ,&
&  0.26552E+01_JPRB, 0.24480E+01_JPRB, 0.22979E+01_JPRB, 0.22033E+01_JPRB, 0.21843E+01_JPRB, 0.22094E+01_JPRB, 0.22552E+01_JPRB ,&
&  0.22567E+01_JPRB, 0.21789E+01_JPRB, 0.23369E+01_JPRB, 0.34398E+01_JPRB, 0.66129E+01_JPRB, 0.98880E+01_JPRB, 0.12204E+02_JPRB ,&
&  0.12797E+02_JPRB, 0.12577E+02_JPRB, 0.12991E+02_JPRB, 0.14335E+02_JPRB, 0.13374E+02_JPRB, 0.11675E+02_JPRB, 0.10636E+02_JPRB ,&
&  0.99720E+01_JPRB, 0.96011E+01_JPRB, 0.96223E+01_JPRB, 0.88987E+01_JPRB, 0.79045E+01_JPRB, 0.73538E+01_JPRB, 0.68376E+01_JPRB ,&
&  0.64268E+01_JPRB, 0.64348E+01_JPRB, 0.64955E+01_JPRB, 0.61665E+01_JPRB, 0.59616E+01_JPRB, 0.55940E+01_JPRB, 0.58811E+01_JPRB ,&
&  0.64477E+01_JPRB, 0.46876E+01_JPRB, 0.37633E+01_JPRB, 0.42381E+01_JPRB, 0.30993E+01_JPRB, 0.29907E+01_JPRB, 0.34480E+01_JPRB ,&
&  0.31348E+01_JPRB, 0.29562E+01_JPRB, 0.30466E+01_JPRB, 0.27731E+01_JPRB, 0.27851E+01_JPRB, 0.29857E+01_JPRB, 0.29532E+01_JPRB ,&
&  0.28227E+01_JPRB, 0.26865E+01_JPRB, 0.30562E+01_JPRB, 0.27056E+01_JPRB, 0.24621E+01_JPRB, 0.23649E+01_JPRB, 0.22622E+01_JPRB ,&
&  0.21828E+01_JPRB, 0.21010E+01_JPRB, 0.20053E+01_JPRB, 0.19458E+01_JPRB, 0.19233E+01_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.17220E+01_JPRB, 0.16617E+01_JPRB, 0.16219E+01_JPRB, 0.16074E+01_JPRB, 0.15806E+01_JPRB, 0.15463E+01_JPRB, 0.15085E+01_JPRB ,&
&  0.14733E+01_JPRB, 0.14307E+01_JPRB, 0.13813E+01_JPRB, 0.13285E+01_JPRB, 0.12878E+01_JPRB, 0.12445E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.10866E+01_JPRB, 0.11184E+01_JPRB, 0.11310E+01_JPRB, 0.12047E+01_JPRB, 0.14784E+01_JPRB, 0.14462E+01_JPRB, 0.14868E+01_JPRB ,&
&  0.15467E+01_JPRB, 0.16170E+01_JPRB, 0.16312E+01_JPRB, 0.16694E+01_JPRB, 0.17659E+01_JPRB, 0.19836E+01_JPRB, 0.22558E+01_JPRB ,&
&  0.22132E+01_JPRB, 0.20015E+01_JPRB, 0.19512E+01_JPRB, 0.19409E+01_JPRB, 0.20420E+01_JPRB, 0.21439E+01_JPRB, 0.22628E+01_JPRB ,&
&  0.21986E+01_JPRB, 0.21007E+01_JPRB, 0.20262E+01_JPRB, 0.19740E+01_JPRB, 0.19361E+01_JPRB, 0.19321E+01_JPRB, 0.19664E+01_JPRB ,&
&  0.20145E+01_JPRB, 0.20121E+01_JPRB, 0.20563E+01_JPRB, 0.26648E+01_JPRB, 0.55997E+01_JPRB, 0.91849E+01_JPRB, 0.11754E+02_JPRB ,&
&  0.11922E+02_JPRB, 0.10997E+02_JPRB, 0.11536E+02_JPRB, 0.13069E+02_JPRB, 0.12004E+02_JPRB, 0.10200E+02_JPRB, 0.87307E+01_JPRB ,&
&  0.80766E+01_JPRB, 0.76010E+01_JPRB, 0.77764E+01_JPRB, 0.79366E+01_JPRB, 0.75809E+01_JPRB, 0.74871E+01_JPRB, 0.72896E+01_JPRB ,&
&  0.70628E+01_JPRB, 0.70424E+01_JPRB, 0.68228E+01_JPRB, 0.65986E+01_JPRB, 0.64716E+01_JPRB, 0.66795E+01_JPRB, 0.66410E+01_JPRB ,&
&  0.67961E+01_JPRB, 0.64730E+01_JPRB, 0.55679E+01_JPRB, 0.54303E+01_JPRB, 0.42004E+01_JPRB, 0.39738E+01_JPRB, 0.46715E+01_JPRB ,&
&  0.38523E+01_JPRB, 0.31566E+01_JPRB, 0.28982E+01_JPRB, 0.28259E+01_JPRB, 0.28106E+01_JPRB, 0.29447E+01_JPRB, 0.27882E+01_JPRB ,&
&  0.24540E+01_JPRB, 0.25671E+01_JPRB, 0.27286E+01_JPRB, 0.23539E+01_JPRB, 0.21274E+01_JPRB, 0.19975E+01_JPRB, 0.18328E+01_JPRB ,&
&  0.18425E+01_JPRB, 0.17943E+01_JPRB, 0.17575E+01_JPRB, 0.17271E+01_JPRB, 0.17244E+01_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.15611E+01_JPRB, 0.15733E+01_JPRB, 0.15729E+01_JPRB, 0.15649E+01_JPRB, 0.15406E+01_JPRB, 0.15016E+01_JPRB, 0.14514E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.13380E+01_JPRB, 0.12785E+01_JPRB, 0.12259E+01_JPRB, 0.11958E+01_JPRB, 0.11656E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.97951E+00_JPRB, 0.99291E+00_JPRB, 0.98849E+00_JPRB, 0.10234E+01_JPRB, 0.11152E+01_JPRB, 0.11550E+01_JPRB, 0.12037E+01_JPRB ,&
&  0.13246E+01_JPRB, 0.14176E+01_JPRB, 0.14784E+01_JPRB, 0.15397E+01_JPRB, 0.15811E+01_JPRB, 0.16292E+01_JPRB, 0.16881E+01_JPRB ,&
&  0.16141E+01_JPRB, 0.14681E+01_JPRB, 0.13713E+01_JPRB, 0.13100E+01_JPRB, 0.13843E+01_JPRB, 0.15095E+01_JPRB, 0.16078E+01_JPRB ,&
&  0.16533E+01_JPRB, 0.16605E+01_JPRB, 0.16650E+01_JPRB, 0.16591E+01_JPRB, 0.16367E+01_JPRB, 0.16259E+01_JPRB, 0.16230E+01_JPRB ,&
&  0.16538E+01_JPRB, 0.16906E+01_JPRB, 0.17444E+01_JPRB, 0.20313E+01_JPRB, 0.41957E+01_JPRB, 0.69001E+01_JPRB, 0.82310E+01_JPRB ,&
&  0.75376E+01_JPRB, 0.74260E+01_JPRB, 0.76972E+01_JPRB, 0.80959E+01_JPRB, 0.79123E+01_JPRB, 0.71139E+01_JPRB, 0.64510E+01_JPRB ,&
&  0.63723E+01_JPRB, 0.60220E+01_JPRB, 0.64616E+01_JPRB, 0.63650E+01_JPRB, 0.63807E+01_JPRB, 0.64974E+01_JPRB, 0.63450E+01_JPRB ,&
&  0.63457E+01_JPRB, 0.61651E+01_JPRB, 0.61859E+01_JPRB, 0.60638E+01_JPRB, 0.60725E+01_JPRB, 0.62310E+01_JPRB, 0.62874E+01_JPRB ,&
&  0.63836E+01_JPRB, 0.64185E+01_JPRB, 0.59665E+01_JPRB, 0.56199E+01_JPRB, 0.52263E+01_JPRB, 0.54798E+01_JPRB, 0.49684E+01_JPRB ,&
&  0.41280E+01_JPRB, 0.31266E+01_JPRB, 0.30221E+01_JPRB, 0.28354E+01_JPRB, 0.27398E+01_JPRB, 0.26201E+01_JPRB, 0.24121E+01_JPRB ,&
&  0.24123E+01_JPRB, 0.25704E+01_JPRB, 0.24262E+01_JPRB, 0.20823E+01_JPRB, 0.19003E+01_JPRB, 0.17245E+01_JPRB, 0.14795E+01_JPRB ,&
&  0.14366E+01_JPRB, 0.15386E+01_JPRB, 0.15635E+01_JPRB, 0.15527E+01_JPRB, 0.15400E+01_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.14861E+01_JPRB, 0.14973E+01_JPRB, 0.14941E+01_JPRB, 0.14863E+01_JPRB, 0.14669E+01_JPRB, 0.14217E+01_JPRB, 0.13428E+01_JPRB ,&
&  0.13235E+01_JPRB, 0.12723E+01_JPRB, 0.12143E+01_JPRB, 0.11663E+01_JPRB, 0.11403E+01_JPRB, 0.10509E+01_JPRB, 0.92442E+00_JPRB ,&
&  0.88254E+00_JPRB, 0.89533E+00_JPRB, 0.92356E+00_JPRB, 0.94207E+00_JPRB, 0.96072E+00_JPRB, 0.97486E+00_JPRB, 0.10150E+01_JPRB ,&
&  0.10684E+01_JPRB, 0.11195E+01_JPRB, 0.11856E+01_JPRB, 0.12515E+01_JPRB, 0.12649E+01_JPRB, 0.12626E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.11505E+01_JPRB, 0.11074E+01_JPRB, 0.10232E+01_JPRB, 0.10911E+01_JPRB, 0.11398E+01_JPRB, 0.11916E+01_JPRB ,&
&  0.12378E+01_JPRB, 0.12669E+01_JPRB, 0.13026E+01_JPRB, 0.13201E+01_JPRB, 0.13159E+01_JPRB, 0.13161E+01_JPRB, 0.13226E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.13873E+01_JPRB, 0.14657E+01_JPRB, 0.16195E+01_JPRB, 0.29536E+01_JPRB, 0.46093E+01_JPRB, 0.52453E+01_JPRB ,&
&  0.48694E+01_JPRB, 0.52225E+01_JPRB, 0.55596E+01_JPRB, 0.51311E+01_JPRB, 0.53186E+01_JPRB, 0.54764E+01_JPRB, 0.52767E+01_JPRB ,&
&  0.54937E+01_JPRB, 0.54574E+01_JPRB, 0.53383E+01_JPRB, 0.51963E+01_JPRB, 0.50591E+01_JPRB, 0.48870E+01_JPRB, 0.47580E+01_JPRB ,&
&  0.48060E+01_JPRB, 0.50416E+01_JPRB, 0.51833E+01_JPRB, 0.54351E+01_JPRB, 0.55585E+01_JPRB, 0.57448E+01_JPRB, 0.59064E+01_JPRB ,&
&  0.59647E+01_JPRB, 0.60037E+01_JPRB, 0.59340E+01_JPRB, 0.56798E+01_JPRB, 0.53921E+01_JPRB, 0.50851E+01_JPRB, 0.47262E+01_JPRB ,&
&  0.43943E+01_JPRB, 0.38052E+01_JPRB, 0.33393E+01_JPRB, 0.31117E+01_JPRB, 0.28633E+01_JPRB, 0.25303E+01_JPRB, 0.24094E+01_JPRB ,&
&  0.23022E+01_JPRB, 0.20718E+01_JPRB, 0.20249E+01_JPRB, 0.18221E+01_JPRB, 0.16787E+01_JPRB, 0.15447E+01_JPRB, 0.13641E+01_JPRB ,&
&  0.13051E+01_JPRB, 0.13793E+01_JPRB, 0.14495E+01_JPRB, 0.14863E+01_JPRB, 0.14866E+01_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.14156E+01_JPRB, 0.14321E+01_JPRB, 0.14346E+01_JPRB, 0.14288E+01_JPRB, 0.13810E+01_JPRB, 0.12903E+01_JPRB, 0.12038E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.11644E+01_JPRB, 0.11480E+01_JPRB, 0.99673E+00_JPRB, 0.94298E+00_JPRB, 0.92075E+00_JPRB, 0.87942E+00_JPRB ,&
&  0.85815E+00_JPRB, 0.85578E+00_JPRB, 0.85359E+00_JPRB, 0.86027E+00_JPRB, 0.86988E+00_JPRB, 0.87040E+00_JPRB, 0.87613E+00_JPRB ,&
&  0.89419E+00_JPRB, 0.92236E+00_JPRB, 0.97136E+00_JPRB, 0.10019E+01_JPRB, 0.10264E+01_JPRB, 0.10385E+01_JPRB, 0.10476E+01_JPRB ,&
&  0.99912E+00_JPRB, 0.97403E+00_JPRB, 0.97250E+00_JPRB, 0.94666E+00_JPRB, 0.94441E+00_JPRB, 0.96092E+00_JPRB, 0.98672E+00_JPRB ,&
&  0.10064E+01_JPRB, 0.96813E+00_JPRB, 0.10464E+01_JPRB, 0.10865E+01_JPRB, 0.11093E+01_JPRB, 0.11230E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11876E+01_JPRB, 0.12480E+01_JPRB, 0.13660E+01_JPRB, 0.20303E+01_JPRB, 0.33244E+01_JPRB, 0.36329E+01_JPRB ,&
&  0.36885E+01_JPRB, 0.30680E+01_JPRB, 0.35285E+01_JPRB, 0.37080E+01_JPRB, 0.40081E+01_JPRB, 0.42837E+01_JPRB, 0.44111E+01_JPRB ,&
&  0.48716E+01_JPRB, 0.49771E+01_JPRB, 0.47334E+01_JPRB, 0.42750E+01_JPRB, 0.38932E+01_JPRB, 0.37121E+01_JPRB, 0.36445E+01_JPRB ,&
&  0.36608E+01_JPRB, 0.35149E+01_JPRB, 0.39817E+01_JPRB, 0.44361E+01_JPRB, 0.47213E+01_JPRB, 0.50133E+01_JPRB, 0.53658E+01_JPRB ,&
&  0.56023E+01_JPRB, 0.56138E+01_JPRB, 0.55617E+01_JPRB, 0.51704E+01_JPRB, 0.49686E+01_JPRB, 0.48152E+01_JPRB, 0.45412E+01_JPRB ,&
&  0.43541E+01_JPRB, 0.41820E+01_JPRB, 0.39373E+01_JPRB, 0.38820E+01_JPRB, 0.33470E+01_JPRB, 0.29817E+01_JPRB, 0.27280E+01_JPRB ,&
&  0.22512E+01_JPRB, 0.17505E+01_JPRB, 0.15432E+01_JPRB, 0.14387E+01_JPRB, 0.13675E+01_JPRB, 0.13457E+01_JPRB, 0.13351E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12894E+01_JPRB, 0.13601E+01_JPRB, 0.13790E+01_JPRB, 0.13926E+01_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.13681E+01_JPRB, 0.14041E+01_JPRB, 0.14251E+01_JPRB, 0.14081E+01_JPRB, 0.13316E+01_JPRB, 0.12151E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.10281E+01_JPRB, 0.98077E+00_JPRB, 0.94405E+00_JPRB, 0.91078E+00_JPRB, 0.88503E+00_JPRB, 0.84998E+00_JPRB, 0.81590E+00_JPRB ,&
&  0.80854E+00_JPRB, 0.81879E+00_JPRB, 0.82881E+00_JPRB, 0.82597E+00_JPRB, 0.82189E+00_JPRB, 0.81940E+00_JPRB, 0.81640E+00_JPRB ,&
&  0.81160E+00_JPRB, 0.81648E+00_JPRB, 0.83696E+00_JPRB, 0.86890E+00_JPRB, 0.88436E+00_JPRB, 0.88760E+00_JPRB, 0.89304E+00_JPRB ,&
&  0.88267E+00_JPRB, 0.86201E+00_JPRB, 0.84903E+00_JPRB, 0.85090E+00_JPRB, 0.85054E+00_JPRB, 0.86811E+00_JPRB, 0.86341E+00_JPRB ,&
&  0.79848E+00_JPRB, 0.82515E+00_JPRB, 0.91043E+00_JPRB, 0.95328E+00_JPRB, 0.98221E+00_JPRB, 0.10020E+01_JPRB, 0.10189E+01_JPRB ,&
&  0.10346E+01_JPRB, 0.10732E+01_JPRB, 0.11265E+01_JPRB, 0.12061E+01_JPRB, 0.15534E+01_JPRB, 0.23225E+01_JPRB, 0.27420E+01_JPRB ,&
&  0.30327E+01_JPRB, 0.27060E+01_JPRB, 0.25743E+01_JPRB, 0.26795E+01_JPRB, 0.31858E+01_JPRB, 0.35147E+01_JPRB, 0.37854E+01_JPRB ,&
&  0.40469E+01_JPRB, 0.40276E+01_JPRB, 0.39025E+01_JPRB, 0.36730E+01_JPRB, 0.32826E+01_JPRB, 0.30164E+01_JPRB, 0.29041E+01_JPRB ,&
&  0.27762E+01_JPRB, 0.27348E+01_JPRB, 0.28673E+01_JPRB, 0.32479E+01_JPRB, 0.37240E+01_JPRB, 0.41267E+01_JPRB, 0.44479E+01_JPRB ,&
&  0.47925E+01_JPRB, 0.49817E+01_JPRB, 0.49288E+01_JPRB, 0.47838E+01_JPRB, 0.45285E+01_JPRB, 0.43760E+01_JPRB, 0.43138E+01_JPRB ,&
&  0.42232E+01_JPRB, 0.41325E+01_JPRB, 0.39470E+01_JPRB, 0.36972E+01_JPRB, 0.32989E+01_JPRB, 0.29289E+01_JPRB, 0.25016E+01_JPRB ,&
&  0.18865E+01_JPRB, 0.15337E+01_JPRB, 0.12943E+01_JPRB, 0.12122E+01_JPRB, 0.11924E+01_JPRB, 0.11964E+01_JPRB, 0.11773E+01_JPRB ,&
&  0.12046E+01_JPRB, 0.12292E+01_JPRB, 0.12329E+01_JPRB, 0.12772E+01_JPRB, 0.13307E+01_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.13073E+01_JPRB, 0.13358E+01_JPRB, 0.13338E+01_JPRB, 0.13066E+01_JPRB, 0.12111E+01_JPRB, 0.11202E+01_JPRB, 0.10349E+01_JPRB ,&
&  0.97956E+00_JPRB, 0.93343E+00_JPRB, 0.90096E+00_JPRB, 0.86873E+00_JPRB, 0.84323E+00_JPRB, 0.83106E+00_JPRB, 0.83119E+00_JPRB ,&
&  0.83626E+00_JPRB, 0.82848E+00_JPRB, 0.81741E+00_JPRB, 0.80243E+00_JPRB, 0.79205E+00_JPRB, 0.78366E+00_JPRB, 0.77398E+00_JPRB ,&
&  0.76075E+00_JPRB, 0.75907E+00_JPRB, 0.76638E+00_JPRB, 0.78510E+00_JPRB, 0.79610E+00_JPRB, 0.80045E+00_JPRB, 0.79803E+00_JPRB ,&
&  0.78159E+00_JPRB, 0.76074E+00_JPRB, 0.76339E+00_JPRB, 0.77763E+00_JPRB, 0.78632E+00_JPRB, 0.78941E+00_JPRB, 0.75581E+00_JPRB ,&
&  0.73490E+00_JPRB, 0.73476E+00_JPRB, 0.75939E+00_JPRB, 0.84332E+00_JPRB, 0.89188E+00_JPRB, 0.91760E+00_JPRB, 0.93464E+00_JPRB ,&
&  0.92351E+00_JPRB, 0.93983E+00_JPRB, 0.98903E+00_JPRB, 0.10970E+01_JPRB, 0.12778E+01_JPRB, 0.16074E+01_JPRB, 0.20359E+01_JPRB ,&
&  0.23241E+01_JPRB, 0.25219E+01_JPRB, 0.26460E+01_JPRB, 0.24649E+01_JPRB, 0.25075E+01_JPRB, 0.29981E+01_JPRB, 0.31978E+01_JPRB ,&
&  0.33089E+01_JPRB, 0.33236E+01_JPRB, 0.33972E+01_JPRB, 0.31965E+01_JPRB, 0.28001E+01_JPRB, 0.25381E+01_JPRB, 0.23605E+01_JPRB ,&
&  0.21676E+01_JPRB, 0.20517E+01_JPRB, 0.20343E+01_JPRB, 0.20939E+01_JPRB, 0.22456E+01_JPRB, 0.25732E+01_JPRB, 0.30021E+01_JPRB ,&
&  0.34230E+01_JPRB, 0.36798E+01_JPRB, 0.38320E+01_JPRB, 0.37958E+01_JPRB, 0.37244E+01_JPRB, 0.36121E+01_JPRB, 0.35274E+01_JPRB ,&
&  0.34306E+01_JPRB, 0.33958E+01_JPRB, 0.33497E+01_JPRB, 0.32853E+01_JPRB, 0.29444E+01_JPRB, 0.24134E+01_JPRB, 0.19126E+01_JPRB ,&
&  0.15685E+01_JPRB, 0.13669E+01_JPRB, 0.12270E+01_JPRB, 0.11858E+01_JPRB, 0.11749E+01_JPRB, 0.11612E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.10748E+01_JPRB, 0.10909E+01_JPRB, 0.11278E+01_JPRB, 0.11717E+01_JPRB, 0.12393E+01_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.12769E+01_JPRB, 0.12685E+01_JPRB, 0.12166E+01_JPRB, 0.11484E+01_JPRB, 0.10865E+01_JPRB, 0.10065E+01_JPRB, 0.94662E+00_JPRB ,&
&  0.91304E+00_JPRB, 0.89932E+00_JPRB, 0.89467E+00_JPRB, 0.88792E+00_JPRB, 0.88145E+00_JPRB, 0.87345E+00_JPRB, 0.86008E+00_JPRB ,&
&  0.84313E+00_JPRB, 0.83259E+00_JPRB, 0.82650E+00_JPRB, 0.81477E+00_JPRB, 0.79627E+00_JPRB, 0.77358E+00_JPRB, 0.75239E+00_JPRB ,&
&  0.73666E+00_JPRB, 0.73192E+00_JPRB, 0.74583E+00_JPRB, 0.76103E+00_JPRB, 0.77092E+00_JPRB, 0.76819E+00_JPRB, 0.75532E+00_JPRB ,&
&  0.72927E+00_JPRB, 0.72090E+00_JPRB, 0.73248E+00_JPRB, 0.73743E+00_JPRB, 0.74644E+00_JPRB, 0.74099E+00_JPRB, 0.71747E+00_JPRB ,&
&  0.69744E+00_JPRB, 0.68155E+00_JPRB, 0.67890E+00_JPRB, 0.69297E+00_JPRB, 0.75430E+00_JPRB, 0.82982E+00_JPRB, 0.86451E+00_JPRB ,&
&  0.89783E+00_JPRB, 0.92633E+00_JPRB, 0.96072E+00_JPRB, 0.10069E+01_JPRB, 0.10841E+01_JPRB, 0.12049E+01_JPRB, 0.14439E+01_JPRB ,&
&  0.17285E+01_JPRB, 0.19421E+01_JPRB, 0.21531E+01_JPRB, 0.20380E+01_JPRB, 0.19805E+01_JPRB, 0.20921E+01_JPRB, 0.22782E+01_JPRB ,&
&  0.24450E+01_JPRB, 0.24863E+01_JPRB, 0.24718E+01_JPRB, 0.25020E+01_JPRB, 0.23571E+01_JPRB, 0.21742E+01_JPRB, 0.19528E+01_JPRB ,&
&  0.18015E+01_JPRB, 0.17038E+01_JPRB, 0.16796E+01_JPRB, 0.16276E+01_JPRB, 0.16389E+01_JPRB, 0.17181E+01_JPRB, 0.18677E+01_JPRB ,&
&  0.21038E+01_JPRB, 0.23814E+01_JPRB, 0.25128E+01_JPRB, 0.24424E+01_JPRB, 0.25162E+01_JPRB, 0.26568E+01_JPRB, 0.27705E+01_JPRB ,&
&  0.27736E+01_JPRB, 0.27605E+01_JPRB, 0.27774E+01_JPRB, 0.27383E+01_JPRB, 0.24820E+01_JPRB, 0.22496E+01_JPRB, 0.19865E+01_JPRB ,&
&  0.17386E+01_JPRB, 0.15192E+01_JPRB, 0.13923E+01_JPRB, 0.13441E+01_JPRB, 0.13028E+01_JPRB, 0.12569E+01_JPRB, 0.11982E+01_JPRB ,&
&  0.11481E+01_JPRB, 0.11373E+01_JPRB, 0.11650E+01_JPRB, 0.12055E+01_JPRB, 0.12545E+01_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.12357E+01_JPRB, 0.11321E+01_JPRB, 0.10593E+01_JPRB, 0.10217E+01_JPRB, 0.99451E+00_JPRB, 0.96276E+00_JPRB, 0.93867E+00_JPRB ,&
&  0.92090E+00_JPRB, 0.91860E+00_JPRB, 0.90120E+00_JPRB, 0.88394E+00_JPRB, 0.87305E+00_JPRB, 0.86459E+00_JPRB, 0.86094E+00_JPRB ,&
&  0.85718E+00_JPRB, 0.84715E+00_JPRB, 0.82528E+00_JPRB, 0.80039E+00_JPRB, 0.78259E+00_JPRB, 0.76767E+00_JPRB, 0.75489E+00_JPRB ,&
&  0.74658E+00_JPRB, 0.74741E+00_JPRB, 0.76177E+00_JPRB, 0.76177E+00_JPRB, 0.75452E+00_JPRB, 0.73952E+00_JPRB, 0.72243E+00_JPRB ,&
&  0.71760E+00_JPRB, 0.73311E+00_JPRB, 0.71404E+00_JPRB, 0.72382E+00_JPRB, 0.73839E+00_JPRB, 0.70952E+00_JPRB, 0.68784E+00_JPRB ,&
&  0.66740E+00_JPRB, 0.64875E+00_JPRB, 0.63529E+00_JPRB, 0.63184E+00_JPRB, 0.65643E+00_JPRB, 0.68766E+00_JPRB, 0.74479E+00_JPRB ,&
&  0.81613E+00_JPRB, 0.86198E+00_JPRB, 0.89474E+00_JPRB, 0.93178E+00_JPRB, 0.96765E+00_JPRB, 0.10144E+01_JPRB, 0.11043E+01_JPRB ,&
&  0.12543E+01_JPRB, 0.14173E+01_JPRB, 0.15780E+01_JPRB, 0.17019E+01_JPRB, 0.18196E+01_JPRB, 0.17656E+01_JPRB, 0.17779E+01_JPRB ,&
&  0.20538E+01_JPRB, 0.21637E+01_JPRB, 0.20294E+01_JPRB, 0.18932E+01_JPRB, 0.17412E+01_JPRB, 0.16139E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.14828E+01_JPRB, 0.14651E+01_JPRB, 0.14410E+01_JPRB, 0.13765E+01_JPRB, 0.13254E+01_JPRB, 0.13299E+01_JPRB, 0.13865E+01_JPRB ,&
&  0.15061E+01_JPRB, 0.18466E+01_JPRB, 0.24061E+01_JPRB, 0.21365E+01_JPRB, 0.20765E+01_JPRB, 0.21155E+01_JPRB, 0.21651E+01_JPRB ,&
&  0.22477E+01_JPRB, 0.23066E+01_JPRB, 0.23395E+01_JPRB, 0.22908E+01_JPRB, 0.21809E+01_JPRB, 0.20548E+01_JPRB, 0.19630E+01_JPRB ,&
&  0.18886E+01_JPRB, 0.18135E+01_JPRB, 0.17576E+01_JPRB, 0.17299E+01_JPRB, 0.16799E+01_JPRB, 0.16176E+01_JPRB, 0.15597E+01_JPRB ,&
&  0.15087E+01_JPRB, 0.14707E+01_JPRB, 0.14446E+01_JPRB, 0.14046E+01_JPRB, 0.13448E+01_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.10695E+01_JPRB, 0.10204E+01_JPRB, 0.98460E+00_JPRB, 0.96220E+00_JPRB, 0.94474E+00_JPRB, 0.93308E+00_JPRB, 0.92311E+00_JPRB ,&
&  0.91379E+00_JPRB, 0.90559E+00_JPRB, 0.89416E+00_JPRB, 0.88402E+00_JPRB, 0.87636E+00_JPRB, 0.86838E+00_JPRB, 0.86155E+00_JPRB ,&
&  0.85060E+00_JPRB, 0.82321E+00_JPRB, 0.81600E+00_JPRB, 0.80615E+00_JPRB, 0.78852E+00_JPRB, 0.77929E+00_JPRB, 0.77348E+00_JPRB ,&
&  0.76470E+00_JPRB, 0.75921E+00_JPRB, 0.75675E+00_JPRB, 0.74614E+00_JPRB, 0.73414E+00_JPRB, 0.73415E+00_JPRB, 0.72863E+00_JPRB ,&
&  0.69986E+00_JPRB, 0.69990E+00_JPRB, 0.74084E+00_JPRB, 0.73689E+00_JPRB, 0.68654E+00_JPRB, 0.67588E+00_JPRB, 0.67422E+00_JPRB ,&
&  0.66042E+00_JPRB, 0.62055E+00_JPRB, 0.61220E+00_JPRB, 0.60797E+00_JPRB, 0.62208E+00_JPRB, 0.63989E+00_JPRB, 0.68185E+00_JPRB ,&
&  0.71924E+00_JPRB, 0.79622E+00_JPRB, 0.83415E+00_JPRB, 0.86479E+00_JPRB, 0.88946E+00_JPRB, 0.91582E+00_JPRB, 0.95020E+00_JPRB ,&
&  0.10008E+01_JPRB, 0.10739E+01_JPRB, 0.11682E+01_JPRB, 0.12568E+01_JPRB, 0.13406E+01_JPRB, 0.13871E+01_JPRB, 0.14065E+01_JPRB ,&
&  0.14599E+01_JPRB, 0.14630E+01_JPRB, 0.14006E+01_JPRB, 0.13445E+01_JPRB, 0.12816E+01_JPRB, 0.12223E+01_JPRB, 0.11587E+01_JPRB ,&
&  0.11423E+01_JPRB, 0.11634E+01_JPRB, 0.11512E+01_JPRB, 0.11335E+01_JPRB, 0.11203E+01_JPRB, 0.11198E+01_JPRB, 0.11325E+01_JPRB ,&
&  0.11378E+01_JPRB, 0.12124E+01_JPRB, 0.14342E+01_JPRB, 0.15165E+01_JPRB, 0.15817E+01_JPRB, 0.16303E+01_JPRB, 0.16665E+01_JPRB ,&
&  0.17092E+01_JPRB, 0.17563E+01_JPRB, 0.18024E+01_JPRB, 0.18259E+01_JPRB, 0.18434E+01_JPRB, 0.18258E+01_JPRB, 0.18424E+01_JPRB ,&
&  0.18234E+01_JPRB, 0.17946E+01_JPRB, 0.17573E+01_JPRB, 0.17236E+01_JPRB, 0.16842E+01_JPRB, 0.16416E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.14929E+01_JPRB, 0.14021E+01_JPRB, 0.13139E+01_JPRB, 0.12235E+01_JPRB, 0.11391E+01_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.96400E+00_JPRB, 0.93914E+00_JPRB, 0.92061E+00_JPRB, 0.90636E+00_JPRB, 0.89719E+00_JPRB, 0.89013E+00_JPRB, 0.88467E+00_JPRB ,&
&  0.88021E+00_JPRB, 0.87274E+00_JPRB, 0.86480E+00_JPRB, 0.85559E+00_JPRB, 0.84821E+00_JPRB, 0.84189E+00_JPRB, 0.83494E+00_JPRB ,&
&  0.82573E+00_JPRB, 0.81608E+00_JPRB, 0.80621E+00_JPRB, 0.79822E+00_JPRB, 0.80343E+00_JPRB, 0.80382E+00_JPRB, 0.80163E+00_JPRB ,&
&  0.78941E+00_JPRB, 0.76714E+00_JPRB, 0.74467E+00_JPRB, 0.74235E+00_JPRB, 0.73516E+00_JPRB, 0.71477E+00_JPRB, 0.71082E+00_JPRB ,&
&  0.72015E+00_JPRB, 0.72057E+00_JPRB, 0.71361E+00_JPRB, 0.70637E+00_JPRB, 0.67093E+00_JPRB, 0.61671E+00_JPRB, 0.64027E+00_JPRB ,&
&  0.63428E+00_JPRB, 0.60905E+00_JPRB, 0.60409E+00_JPRB, 0.60186E+00_JPRB, 0.61203E+00_JPRB, 0.63243E+00_JPRB, 0.65215E+00_JPRB ,&
&  0.66058E+00_JPRB, 0.74665E+00_JPRB, 0.78349E+00_JPRB, 0.80880E+00_JPRB, 0.82712E+00_JPRB, 0.84338E+00_JPRB, 0.85846E+00_JPRB ,&
&  0.87469E+00_JPRB, 0.89898E+00_JPRB, 0.93383E+00_JPRB, 0.96809E+00_JPRB, 0.10072E+01_JPRB, 0.10344E+01_JPRB, 0.10669E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.11062E+01_JPRB, 0.10771E+01_JPRB, 0.10471E+01_JPRB, 0.10214E+01_JPRB, 0.99810E+00_JPRB, 0.97893E+00_JPRB ,&
&  0.97458E+00_JPRB, 0.97203E+00_JPRB, 0.98527E+00_JPRB, 0.99134E+00_JPRB, 0.99966E+00_JPRB, 0.10080E+01_JPRB, 0.10304E+01_JPRB ,&
&  0.10482E+01_JPRB, 0.10780E+01_JPRB, 0.11121E+01_JPRB, 0.11621E+01_JPRB, 0.12121E+01_JPRB, 0.12505E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.13058E+01_JPRB, 0.13400E+01_JPRB, 0.13598E+01_JPRB, 0.13831E+01_JPRB, 0.14101E+01_JPRB, 0.14234E+01_JPRB, 0.14357E+01_JPRB ,&
&  0.14406E+01_JPRB, 0.14350E+01_JPRB, 0.14172E+01_JPRB, 0.13875E+01_JPRB, 0.13474E+01_JPRB, 0.13011E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.11819E+01_JPRB, 0.11128E+01_JPRB, 0.10595E+01_JPRB, 0.10221E+01_JPRB, 0.99042E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.91529E+00_JPRB, 0.89803E+00_JPRB, 0.88494E+00_JPRB, 0.87445E+00_JPRB, 0.86625E+00_JPRB, 0.85972E+00_JPRB, 0.85349E+00_JPRB ,&
&  0.84847E+00_JPRB, 0.84318E+00_JPRB, 0.83830E+00_JPRB, 0.83440E+00_JPRB, 0.83102E+00_JPRB, 0.82860E+00_JPRB, 0.82530E+00_JPRB ,&
&  0.82467E+00_JPRB, 0.82252E+00_JPRB, 0.82580E+00_JPRB, 0.82402E+00_JPRB, 0.82763E+00_JPRB, 0.82474E+00_JPRB, 0.81111E+00_JPRB ,&
&  0.79458E+00_JPRB, 0.77215E+00_JPRB, 0.74547E+00_JPRB, 0.75123E+00_JPRB, 0.73841E+00_JPRB, 0.73255E+00_JPRB, 0.71231E+00_JPRB ,&
&  0.70057E+00_JPRB, 0.67770E+00_JPRB, 0.69956E+00_JPRB, 0.68441E+00_JPRB, 0.67426E+00_JPRB, 0.66307E+00_JPRB, 0.64148E+00_JPRB ,&
&  0.63651E+00_JPRB, 0.62232E+00_JPRB, 0.62072E+00_JPRB, 0.62478E+00_JPRB, 0.63256E+00_JPRB, 0.64605E+00_JPRB, 0.65905E+00_JPRB ,&
&  0.66237E+00_JPRB, 0.71597E+00_JPRB, 0.74814E+00_JPRB, 0.77045E+00_JPRB, 0.79339E+00_JPRB, 0.81236E+00_JPRB, 0.82638E+00_JPRB ,&
&  0.84005E+00_JPRB, 0.85115E+00_JPRB, 0.86516E+00_JPRB, 0.87046E+00_JPRB, 0.88950E+00_JPRB, 0.89635E+00_JPRB, 0.90695E+00_JPRB ,&
&  0.91798E+00_JPRB, 0.91904E+00_JPRB, 0.91487E+00_JPRB, 0.90551E+00_JPRB, 0.89838E+00_JPRB, 0.89328E+00_JPRB, 0.89410E+00_JPRB ,&
&  0.89632E+00_JPRB, 0.90098E+00_JPRB, 0.91427E+00_JPRB, 0.92656E+00_JPRB, 0.93794E+00_JPRB, 0.94814E+00_JPRB, 0.95870E+00_JPRB ,&
&  0.97020E+00_JPRB, 0.99043E+00_JPRB, 0.10118E+01_JPRB, 0.10481E+01_JPRB, 0.10831E+01_JPRB, 0.11077E+01_JPRB, 0.11284E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11470E+01_JPRB, 0.11448E+01_JPRB, 0.11472E+01_JPRB, 0.11488E+01_JPRB, 0.11522E+01_JPRB, 0.11546E+01_JPRB ,&
&  0.11530E+01_JPRB, 0.11414E+01_JPRB, 0.11261E+01_JPRB, 0.11072E+01_JPRB, 0.10890E+01_JPRB, 0.10713E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10355E+01_JPRB, 0.10125E+01_JPRB, 0.98646E+00_JPRB, 0.95992E+00_JPRB, 0.93563E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.89348E+00_JPRB, 0.87996E+00_JPRB, 0.86839E+00_JPRB, 0.86097E+00_JPRB, 0.85350E+00_JPRB, 0.84803E+00_JPRB, 0.84293E+00_JPRB ,&
&  0.83904E+00_JPRB, 0.83522E+00_JPRB, 0.83290E+00_JPRB, 0.83053E+00_JPRB, 0.82892E+00_JPRB, 0.82761E+00_JPRB, 0.82631E+00_JPRB ,&
&  0.82864E+00_JPRB, 0.82805E+00_JPRB, 0.83173E+00_JPRB, 0.83042E+00_JPRB, 0.82933E+00_JPRB, 0.82541E+00_JPRB, 0.81193E+00_JPRB ,&
&  0.80094E+00_JPRB, 0.78928E+00_JPRB, 0.75487E+00_JPRB, 0.76371E+00_JPRB, 0.74680E+00_JPRB, 0.73670E+00_JPRB, 0.72689E+00_JPRB ,&
&  0.70755E+00_JPRB, 0.69822E+00_JPRB, 0.68761E+00_JPRB, 0.68499E+00_JPRB, 0.68776E+00_JPRB, 0.68019E+00_JPRB, 0.68199E+00_JPRB ,&
&  0.67463E+00_JPRB, 0.67318E+00_JPRB, 0.67454E+00_JPRB, 0.67627E+00_JPRB, 0.68489E+00_JPRB, 0.68838E+00_JPRB, 0.69641E+00_JPRB ,&
&  0.71018E+00_JPRB, 0.73145E+00_JPRB, 0.76557E+00_JPRB, 0.77564E+00_JPRB, 0.79724E+00_JPRB, 0.81523E+00_JPRB, 0.83168E+00_JPRB ,&
&  0.84268E+00_JPRB, 0.84988E+00_JPRB, 0.85603E+00_JPRB, 0.85860E+00_JPRB, 0.85934E+00_JPRB, 0.85490E+00_JPRB, 0.85582E+00_JPRB ,&
&  0.85899E+00_JPRB, 0.86264E+00_JPRB, 0.86692E+00_JPRB, 0.86776E+00_JPRB, 0.86961E+00_JPRB, 0.86936E+00_JPRB, 0.87299E+00_JPRB ,&
&  0.87611E+00_JPRB, 0.87975E+00_JPRB, 0.88215E+00_JPRB, 0.88648E+00_JPRB, 0.89304E+00_JPRB, 0.90002E+00_JPRB, 0.90712E+00_JPRB ,&
&  0.91515E+00_JPRB, 0.92591E+00_JPRB, 0.93870E+00_JPRB, 0.93990E+00_JPRB, 0.95115E+00_JPRB, 0.97994E+00_JPRB, 0.99405E+00_JPRB ,&
&  0.10095E+01_JPRB, 0.10180E+01_JPRB, 0.10217E+01_JPRB, 0.10256E+01_JPRB, 0.10274E+01_JPRB, 0.10324E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10280E+01_JPRB, 0.10164E+01_JPRB, 0.10063E+01_JPRB, 0.99594E+00_JPRB, 0.98715E+00_JPRB, 0.97732E+00_JPRB ,&
&  0.96882E+00_JPRB, 0.95919E+00_JPRB, 0.94835E+00_JPRB, 0.93143E+00_JPRB, 0.91273E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.87897E+00_JPRB, 0.86982E+00_JPRB, 0.86296E+00_JPRB, 0.85783E+00_JPRB, 0.85276E+00_JPRB, 0.84901E+00_JPRB, 0.84585E+00_JPRB ,&
&  0.84361E+00_JPRB, 0.84040E+00_JPRB, 0.83903E+00_JPRB, 0.83756E+00_JPRB, 0.83547E+00_JPRB, 0.83589E+00_JPRB, 0.83555E+00_JPRB ,&
&  0.83586E+00_JPRB, 0.83478E+00_JPRB, 0.83724E+00_JPRB, 0.83523E+00_JPRB, 0.83403E+00_JPRB, 0.83307E+00_JPRB, 0.82688E+00_JPRB ,&
&  0.82168E+00_JPRB, 0.81512E+00_JPRB, 0.80238E+00_JPRB, 0.78953E+00_JPRB, 0.77935E+00_JPRB, 0.74293E+00_JPRB, 0.73861E+00_JPRB ,&
&  0.74500E+00_JPRB, 0.72948E+00_JPRB, 0.72171E+00_JPRB, 0.71522E+00_JPRB, 0.70997E+00_JPRB, 0.70712E+00_JPRB, 0.70980E+00_JPRB ,&
&  0.69460E+00_JPRB, 0.71258E+00_JPRB, 0.71387E+00_JPRB, 0.71478E+00_JPRB, 0.71804E+00_JPRB, 0.71658E+00_JPRB, 0.72604E+00_JPRB ,&
&  0.75781E+00_JPRB, 0.76529E+00_JPRB, 0.78039E+00_JPRB, 0.79369E+00_JPRB, 0.80853E+00_JPRB, 0.82154E+00_JPRB, 0.83438E+00_JPRB ,&
&  0.84435E+00_JPRB, 0.85138E+00_JPRB, 0.85627E+00_JPRB, 0.85710E+00_JPRB, 0.85770E+00_JPRB, 0.85722E+00_JPRB, 0.85892E+00_JPRB ,&
&  0.85996E+00_JPRB, 0.86191E+00_JPRB, 0.86457E+00_JPRB, 0.86631E+00_JPRB, 0.86820E+00_JPRB, 0.86801E+00_JPRB, 0.86847E+00_JPRB ,&
&  0.86835E+00_JPRB, 0.86739E+00_JPRB, 0.86645E+00_JPRB, 0.86574E+00_JPRB, 0.86468E+00_JPRB, 0.86490E+00_JPRB, 0.86778E+00_JPRB ,&
&  0.87247E+00_JPRB, 0.87695E+00_JPRB, 0.88289E+00_JPRB, 0.88996E+00_JPRB, 0.89897E+00_JPRB, 0.91088E+00_JPRB, 0.92011E+00_JPRB ,&
&  0.93346E+00_JPRB, 0.94256E+00_JPRB, 0.95039E+00_JPRB, 0.95650E+00_JPRB, 0.96155E+00_JPRB, 0.96826E+00_JPRB, 0.97357E+00_JPRB ,&
&  0.98019E+00_JPRB, 0.97672E+00_JPRB, 0.97064E+00_JPRB, 0.96353E+00_JPRB, 0.95761E+00_JPRB, 0.95232E+00_JPRB, 0.94658E+00_JPRB ,&
&  0.94061E+00_JPRB, 0.93094E+00_JPRB, 0.91920E+00_JPRB, 0.90471E+00_JPRB, 0.89073E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.88092E+00_JPRB, 0.87437E+00_JPRB, 0.86941E+00_JPRB, 0.86558E+00_JPRB, 0.86289E+00_JPRB, 0.86054E+00_JPRB, 0.85849E+00_JPRB ,&
&  0.85650E+00_JPRB, 0.85645E+00_JPRB, 0.85590E+00_JPRB, 0.85513E+00_JPRB, 0.85387E+00_JPRB, 0.85425E+00_JPRB, 0.85401E+00_JPRB ,&
&  0.85227E+00_JPRB, 0.84797E+00_JPRB, 0.84383E+00_JPRB, 0.84013E+00_JPRB, 0.83901E+00_JPRB, 0.83750E+00_JPRB, 0.83660E+00_JPRB ,&
&  0.83355E+00_JPRB, 0.83005E+00_JPRB, 0.82560E+00_JPRB, 0.81604E+00_JPRB, 0.80563E+00_JPRB, 0.79577E+00_JPRB, 0.78692E+00_JPRB ,&
&  0.77923E+00_JPRB, 0.77343E+00_JPRB, 0.76954E+00_JPRB, 0.76704E+00_JPRB, 0.76907E+00_JPRB, 0.76887E+00_JPRB, 0.76803E+00_JPRB ,&
&  0.76597E+00_JPRB, 0.76102E+00_JPRB, 0.76153E+00_JPRB, 0.76257E+00_JPRB, 0.76440E+00_JPRB, 0.76993E+00_JPRB, 0.77543E+00_JPRB ,&
&  0.78218E+00_JPRB, 0.78985E+00_JPRB, 0.80209E+00_JPRB, 0.81121E+00_JPRB, 0.82004E+00_JPRB, 0.82714E+00_JPRB, 0.83134E+00_JPRB ,&
&  0.83497E+00_JPRB, 0.83817E+00_JPRB, 0.84144E+00_JPRB, 0.84418E+00_JPRB, 0.84622E+00_JPRB, 0.84849E+00_JPRB, 0.85100E+00_JPRB ,&
&  0.85217E+00_JPRB, 0.85276E+00_JPRB, 0.85319E+00_JPRB, 0.85361E+00_JPRB, 0.85398E+00_JPRB, 0.85493E+00_JPRB, 0.85635E+00_JPRB ,&
&  0.85768E+00_JPRB, 0.85773E+00_JPRB, 0.85773E+00_JPRB, 0.85898E+00_JPRB, 0.86052E+00_JPRB, 0.86250E+00_JPRB, 0.86468E+00_JPRB ,&
&  0.86769E+00_JPRB, 0.87087E+00_JPRB, 0.87495E+00_JPRB, 0.88096E+00_JPRB, 0.88591E+00_JPRB, 0.89050E+00_JPRB, 0.89311E+00_JPRB ,&
&  0.89627E+00_JPRB, 0.89751E+00_JPRB, 0.89876E+00_JPRB, 0.90137E+00_JPRB, 0.90418E+00_JPRB, 0.90888E+00_JPRB, 0.91400E+00_JPRB ,&
&  0.92035E+00_JPRB, 0.92293E+00_JPRB, 0.92424E+00_JPRB, 0.92361E+00_JPRB, 0.92281E+00_JPRB, 0.92101E+00_JPRB, 0.91914E+00_JPRB ,&
&  0.91672E+00_JPRB, 0.91237E+00_JPRB, 0.90517E+00_JPRB, 0.89698E+00_JPRB, 0.88854E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.88051E+00_JPRB, 0.87833E+00_JPRB, 0.87635E+00_JPRB, 0.87430E+00_JPRB, 0.87241E+00_JPRB, 0.87082E+00_JPRB, 0.86939E+00_JPRB ,&
&  0.86787E+00_JPRB, 0.86612E+00_JPRB, 0.86404E+00_JPRB, 0.86207E+00_JPRB, 0.86023E+00_JPRB, 0.85837E+00_JPRB, 0.85690E+00_JPRB ,&
&  0.85544E+00_JPRB, 0.85412E+00_JPRB, 0.85271E+00_JPRB, 0.85085E+00_JPRB, 0.84925E+00_JPRB, 0.84760E+00_JPRB, 0.84509E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.83892E+00_JPRB, 0.83539E+00_JPRB, 0.83205E+00_JPRB, 0.82928E+00_JPRB, 0.82593E+00_JPRB, 0.82212E+00_JPRB ,&
&  0.81853E+00_JPRB, 0.81577E+00_JPRB, 0.81341E+00_JPRB, 0.81138E+00_JPRB, 0.80987E+00_JPRB, 0.80862E+00_JPRB, 0.80730E+00_JPRB ,&
&  0.80628E+00_JPRB, 0.80588E+00_JPRB, 0.80608E+00_JPRB, 0.80662E+00_JPRB, 0.80750E+00_JPRB, 0.80909E+00_JPRB, 0.81210E+00_JPRB ,&
&  0.81532E+00_JPRB, 0.81856E+00_JPRB, 0.82201E+00_JPRB, 0.82525E+00_JPRB, 0.82824E+00_JPRB, 0.83072E+00_JPRB, 0.83283E+00_JPRB ,&
&  0.83519E+00_JPRB, 0.83750E+00_JPRB, 0.83993E+00_JPRB, 0.84183E+00_JPRB, 0.84345E+00_JPRB, 0.84484E+00_JPRB, 0.84629E+00_JPRB ,&
&  0.84707E+00_JPRB, 0.84781E+00_JPRB, 0.84877E+00_JPRB, 0.85003E+00_JPRB, 0.85117E+00_JPRB, 0.85224E+00_JPRB, 0.85346E+00_JPRB ,&
&  0.85500E+00_JPRB, 0.85666E+00_JPRB, 0.85816E+00_JPRB, 0.85956E+00_JPRB, 0.86118E+00_JPRB, 0.86322E+00_JPRB, 0.86564E+00_JPRB ,&
&  0.86803E+00_JPRB, 0.87041E+00_JPRB, 0.87271E+00_JPRB, 0.87493E+00_JPRB, 0.87648E+00_JPRB, 0.87740E+00_JPRB, 0.87716E+00_JPRB ,&
&  0.87659E+00_JPRB, 0.87572E+00_JPRB, 0.87559E+00_JPRB, 0.87623E+00_JPRB, 0.87676E+00_JPRB, 0.87769E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.88104E+00_JPRB, 0.88283E+00_JPRB, 0.88464E+00_JPRB, 0.88665E+00_JPRB, 0.88814E+00_JPRB, 0.88883E+00_JPRB, 0.88936E+00_JPRB ,&
&  0.88958E+00_JPRB, 0.88886E+00_JPRB, 0.88716E+00_JPRB, 0.88510E+00_JPRB, 0.88272E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB ,&
&  0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB, 0.85739E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB ,&
&  0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB, 0.14906E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.15521E+00_JPRB, 0.15538E+00_JPRB, 0.15545E+00_JPRB, 0.15546E+00_JPRB, 0.15529E+00_JPRB, 0.15525E+00_JPRB, 0.15522E+00_JPRB ,&
&  0.15520E+00_JPRB, 0.15503E+00_JPRB, 0.15522E+00_JPRB, 0.15532E+00_JPRB, 0.15535E+00_JPRB, 0.15536E+00_JPRB, 0.15523E+00_JPRB ,&
&  0.15494E+00_JPRB, 0.15459E+00_JPRB, 0.15368E+00_JPRB, 0.15352E+00_JPRB, 0.15354E+00_JPRB, 0.15374E+00_JPRB, 0.15388E+00_JPRB ,&
&  0.15411E+00_JPRB, 0.15434E+00_JPRB, 0.15453E+00_JPRB, 0.15510E+00_JPRB, 0.15535E+00_JPRB, 0.15563E+00_JPRB, 0.15591E+00_JPRB ,&
&  0.15618E+00_JPRB, 0.15649E+00_JPRB, 0.15672E+00_JPRB, 0.15697E+00_JPRB, 0.15719E+00_JPRB, 0.15714E+00_JPRB, 0.15694E+00_JPRB ,&
&  0.15664E+00_JPRB, 0.15615E+00_JPRB, 0.15564E+00_JPRB, 0.15518E+00_JPRB, 0.15477E+00_JPRB, 0.15433E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15347E+00_JPRB, 0.15296E+00_JPRB, 0.15242E+00_JPRB, 0.15193E+00_JPRB, 0.15143E+00_JPRB, 0.15100E+00_JPRB, 0.15051E+00_JPRB ,&
&  0.15013E+00_JPRB, 0.14974E+00_JPRB, 0.14938E+00_JPRB, 0.14900E+00_JPRB, 0.14867E+00_JPRB, 0.14833E+00_JPRB, 0.14801E+00_JPRB ,&
&  0.14766E+00_JPRB, 0.14748E+00_JPRB, 0.14724E+00_JPRB, 0.14702E+00_JPRB, 0.14682E+00_JPRB, 0.14668E+00_JPRB, 0.14648E+00_JPRB ,&
&  0.14629E+00_JPRB, 0.14612E+00_JPRB, 0.14605E+00_JPRB, 0.14591E+00_JPRB, 0.14587E+00_JPRB, 0.14585E+00_JPRB, 0.14591E+00_JPRB ,&
&  0.14596E+00_JPRB, 0.14606E+00_JPRB, 0.14622E+00_JPRB, 0.14648E+00_JPRB, 0.14671E+00_JPRB, 0.14697E+00_JPRB, 0.14723E+00_JPRB ,&
&  0.14752E+00_JPRB, 0.14782E+00_JPRB, 0.14813E+00_JPRB, 0.14843E+00_JPRB, 0.14881E+00_JPRB, 0.14919E+00_JPRB, 0.14961E+00_JPRB ,&
&  0.14999E+00_JPRB, 0.15042E+00_JPRB, 0.15077E+00_JPRB, 0.15120E+00_JPRB, 0.15153E+00_JPRB, 0.15176E+00_JPRB, 0.15195E+00_JPRB ,&
&  0.15231E+00_JPRB, 0.15266E+00_JPRB, 0.15309E+00_JPRB, 0.15347E+00_JPRB, 0.15415E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.16931E+00_JPRB, 0.17159E+00_JPRB, 0.17317E+00_JPRB, 0.17489E+00_JPRB, 0.17316E+00_JPRB, 0.18468E+00_JPRB, 0.18266E+00_JPRB ,&
&  0.18231E+00_JPRB, 0.17725E+00_JPRB, 0.17759E+00_JPRB, 0.17683E+00_JPRB, 0.17541E+00_JPRB, 0.17381E+00_JPRB, 0.17204E+00_JPRB ,&
&  0.17031E+00_JPRB, 0.16813E+00_JPRB, 0.16631E+00_JPRB, 0.16480E+00_JPRB, 0.16341E+00_JPRB, 0.16220E+00_JPRB, 0.16098E+00_JPRB ,&
&  0.16082E+00_JPRB, 0.16146E+00_JPRB, 0.16250E+00_JPRB, 0.16183E+00_JPRB, 0.16222E+00_JPRB, 0.16306E+00_JPRB, 0.16421E+00_JPRB ,&
&  0.16518E+00_JPRB, 0.16653E+00_JPRB, 0.16739E+00_JPRB, 0.16765E+00_JPRB, 0.16800E+00_JPRB, 0.16732E+00_JPRB, 0.16685E+00_JPRB ,&
&  0.16618E+00_JPRB, 0.16524E+00_JPRB, 0.16406E+00_JPRB, 0.16304E+00_JPRB, 0.16190E+00_JPRB, 0.16043E+00_JPRB, 0.15890E+00_JPRB ,&
&  0.15753E+00_JPRB, 0.15632E+00_JPRB, 0.15505E+00_JPRB, 0.15398E+00_JPRB, 0.15297E+00_JPRB, 0.15196E+00_JPRB, 0.15103E+00_JPRB ,&
&  0.15029E+00_JPRB, 0.14963E+00_JPRB, 0.14892E+00_JPRB, 0.14817E+00_JPRB, 0.14735E+00_JPRB, 0.14655E+00_JPRB, 0.14579E+00_JPRB ,&
&  0.14543E+00_JPRB, 0.14510E+00_JPRB, 0.14477E+00_JPRB, 0.14442E+00_JPRB, 0.14431E+00_JPRB, 0.14418E+00_JPRB, 0.14403E+00_JPRB ,&
&  0.14385E+00_JPRB, 0.14360E+00_JPRB, 0.14351E+00_JPRB, 0.14342E+00_JPRB, 0.14333E+00_JPRB, 0.14327E+00_JPRB, 0.14336E+00_JPRB ,&
&  0.14353E+00_JPRB, 0.14376E+00_JPRB, 0.14424E+00_JPRB, 0.14475E+00_JPRB, 0.14529E+00_JPRB, 0.14581E+00_JPRB, 0.14656E+00_JPRB ,&
&  0.14738E+00_JPRB, 0.14817E+00_JPRB, 0.14885E+00_JPRB, 0.14941E+00_JPRB, 0.15031E+00_JPRB, 0.15116E+00_JPRB, 0.15199E+00_JPRB ,&
&  0.15290E+00_JPRB, 0.15406E+00_JPRB, 0.15527E+00_JPRB, 0.15652E+00_JPRB, 0.15796E+00_JPRB, 0.15919E+00_JPRB, 0.16042E+00_JPRB ,&
&  0.16187E+00_JPRB, 0.16455E+00_JPRB, 0.16636E+00_JPRB, 0.16756E+00_JPRB, 0.16886E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.19035E+00_JPRB, 0.19159E+00_JPRB, 0.19263E+00_JPRB, 0.19315E+00_JPRB, 0.19372E+00_JPRB, 0.19374E+00_JPRB, 0.19365E+00_JPRB ,&
&  0.19087E+00_JPRB, 0.18967E+00_JPRB, 0.18793E+00_JPRB, 0.18595E+00_JPRB, 0.18422E+00_JPRB, 0.18198E+00_JPRB, 0.17982E+00_JPRB ,&
&  0.17779E+00_JPRB, 0.17541E+00_JPRB, 0.17294E+00_JPRB, 0.17003E+00_JPRB, 0.16725E+00_JPRB, 0.16462E+00_JPRB, 0.16320E+00_JPRB ,&
&  0.16251E+00_JPRB, 0.16285E+00_JPRB, 0.16443E+00_JPRB, 0.16567E+00_JPRB, 0.16746E+00_JPRB, 0.16975E+00_JPRB, 0.17338E+00_JPRB ,&
&  0.17612E+00_JPRB, 0.17954E+00_JPRB, 0.18181E+00_JPRB, 0.18252E+00_JPRB, 0.18186E+00_JPRB, 0.18152E+00_JPRB, 0.18067E+00_JPRB ,&
&  0.17886E+00_JPRB, 0.17697E+00_JPRB, 0.17535E+00_JPRB, 0.17360E+00_JPRB, 0.17166E+00_JPRB, 0.16932E+00_JPRB, 0.16641E+00_JPRB ,&
&  0.16379E+00_JPRB, 0.16148E+00_JPRB, 0.15932E+00_JPRB, 0.15764E+00_JPRB, 0.15601E+00_JPRB, 0.15453E+00_JPRB, 0.15293E+00_JPRB ,&
&  0.15170E+00_JPRB, 0.15063E+00_JPRB, 0.14927E+00_JPRB, 0.14817E+00_JPRB, 0.14692E+00_JPRB, 0.14587E+00_JPRB, 0.14473E+00_JPRB ,&
&  0.14385E+00_JPRB, 0.14308E+00_JPRB, 0.14245E+00_JPRB, 0.14185E+00_JPRB, 0.14148E+00_JPRB, 0.14112E+00_JPRB, 0.14071E+00_JPRB ,&
&  0.14035E+00_JPRB, 0.14032E+00_JPRB, 0.14034E+00_JPRB, 0.14036E+00_JPRB, 0.14044E+00_JPRB, 0.14013E+00_JPRB, 0.14018E+00_JPRB ,&
&  0.14049E+00_JPRB, 0.14081E+00_JPRB, 0.14090E+00_JPRB, 0.14133E+00_JPRB, 0.14200E+00_JPRB, 0.14280E+00_JPRB, 0.14399E+00_JPRB ,&
&  0.14493E+00_JPRB, 0.14588E+00_JPRB, 0.14677E+00_JPRB, 0.14768E+00_JPRB, 0.14887E+00_JPRB, 0.15014E+00_JPRB, 0.15148E+00_JPRB ,&
&  0.15306E+00_JPRB, 0.15454E+00_JPRB, 0.15609E+00_JPRB, 0.15769E+00_JPRB, 0.16024E+00_JPRB, 0.16271E+00_JPRB, 0.16650E+00_JPRB ,&
&  0.17034E+00_JPRB, 0.17220E+00_JPRB, 0.18524E+00_JPRB, 0.18628E+00_JPRB, 0.18780E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.19088E+00_JPRB, 0.19230E+00_JPRB, 0.19301E+00_JPRB, 0.19403E+00_JPRB, 0.19448E+00_JPRB, 0.19597E+00_JPRB, 0.19614E+00_JPRB ,&
&  0.19627E+00_JPRB, 0.19244E+00_JPRB, 0.19013E+00_JPRB, 0.18748E+00_JPRB, 0.18519E+00_JPRB, 0.18252E+00_JPRB, 0.17993E+00_JPRB ,&
&  0.17790E+00_JPRB, 0.17614E+00_JPRB, 0.17423E+00_JPRB, 0.17159E+00_JPRB, 0.16892E+00_JPRB, 0.16696E+00_JPRB, 0.16555E+00_JPRB ,&
&  0.16418E+00_JPRB, 0.16490E+00_JPRB, 0.16644E+00_JPRB, 0.16958E+00_JPRB, 0.17386E+00_JPRB, 0.17744E+00_JPRB, 0.18221E+00_JPRB ,&
&  0.18826E+00_JPRB, 0.19012E+00_JPRB, 0.19243E+00_JPRB, 0.19350E+00_JPRB, 0.19323E+00_JPRB, 0.19379E+00_JPRB, 0.19155E+00_JPRB ,&
&  0.19175E+00_JPRB, 0.18927E+00_JPRB, 0.19374E+00_JPRB, 0.19082E+00_JPRB, 0.18422E+00_JPRB, 0.18118E+00_JPRB, 0.17513E+00_JPRB ,&
&  0.17068E+00_JPRB, 0.16595E+00_JPRB, 0.16277E+00_JPRB, 0.16098E+00_JPRB, 0.15801E+00_JPRB, 0.15639E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15164E+00_JPRB, 0.15089E+00_JPRB, 0.14947E+00_JPRB, 0.14889E+00_JPRB, 0.14812E+00_JPRB, 0.14704E+00_JPRB, 0.14587E+00_JPRB ,&
&  0.14508E+00_JPRB, 0.14438E+00_JPRB, 0.14401E+00_JPRB, 0.14349E+00_JPRB, 0.14276E+00_JPRB, 0.14203E+00_JPRB, 0.14121E+00_JPRB ,&
&  0.14066E+00_JPRB, 0.14036E+00_JPRB, 0.14013E+00_JPRB, 0.14007E+00_JPRB, 0.14000E+00_JPRB, 0.13977E+00_JPRB, 0.13988E+00_JPRB ,&
&  0.14014E+00_JPRB, 0.14073E+00_JPRB, 0.14124E+00_JPRB, 0.14192E+00_JPRB, 0.14252E+00_JPRB, 0.14333E+00_JPRB, 0.14394E+00_JPRB ,&
&  0.14460E+00_JPRB, 0.14553E+00_JPRB, 0.14633E+00_JPRB, 0.14729E+00_JPRB, 0.14834E+00_JPRB, 0.14923E+00_JPRB, 0.15062E+00_JPRB ,&
&  0.15205E+00_JPRB, 0.15345E+00_JPRB, 0.15542E+00_JPRB, 0.15744E+00_JPRB, 0.16002E+00_JPRB, 0.16438E+00_JPRB, 0.16937E+00_JPRB ,&
&  0.17563E+00_JPRB, 0.18494E+00_JPRB, 0.18526E+00_JPRB, 0.18805E+00_JPRB, 0.18962E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.19613E+00_JPRB, 0.19805E+00_JPRB, 0.19945E+00_JPRB, 0.20077E+00_JPRB, 0.20094E+00_JPRB, 0.20119E+00_JPRB, 0.20192E+00_JPRB ,&
&  0.19813E+00_JPRB, 0.19543E+00_JPRB, 0.19088E+00_JPRB, 0.18662E+00_JPRB, 0.18257E+00_JPRB, 0.17916E+00_JPRB, 0.17713E+00_JPRB ,&
&  0.17608E+00_JPRB, 0.17617E+00_JPRB, 0.17641E+00_JPRB, 0.17705E+00_JPRB, 0.17632E+00_JPRB, 0.17534E+00_JPRB, 0.17438E+00_JPRB ,&
&  0.17322E+00_JPRB, 0.17353E+00_JPRB, 0.17406E+00_JPRB, 0.17737E+00_JPRB, 0.18204E+00_JPRB, 0.18730E+00_JPRB, 0.19112E+00_JPRB ,&
&  0.19381E+00_JPRB, 0.19587E+00_JPRB, 0.19841E+00_JPRB, 0.19952E+00_JPRB, 0.20061E+00_JPRB, 0.20193E+00_JPRB, 0.20247E+00_JPRB ,&
&  0.20379E+00_JPRB, 0.20388E+00_JPRB, 0.20380E+00_JPRB, 0.20409E+00_JPRB, 0.19578E+00_JPRB, 0.18961E+00_JPRB, 0.18331E+00_JPRB ,&
&  0.17725E+00_JPRB, 0.17114E+00_JPRB, 0.16563E+00_JPRB, 0.16257E+00_JPRB, 0.15937E+00_JPRB, 0.15752E+00_JPRB, 0.15590E+00_JPRB ,&
&  0.15469E+00_JPRB, 0.15344E+00_JPRB, 0.15166E+00_JPRB, 0.14991E+00_JPRB, 0.14888E+00_JPRB, 0.14802E+00_JPRB, 0.14692E+00_JPRB ,&
&  0.14681E+00_JPRB, 0.14617E+00_JPRB, 0.14609E+00_JPRB, 0.14570E+00_JPRB, 0.14542E+00_JPRB, 0.14517E+00_JPRB, 0.14516E+00_JPRB ,&
&  0.14552E+00_JPRB, 0.14627E+00_JPRB, 0.14656E+00_JPRB, 0.14693E+00_JPRB, 0.14682E+00_JPRB, 0.14621E+00_JPRB, 0.14569E+00_JPRB ,&
&  0.14511E+00_JPRB, 0.14464E+00_JPRB, 0.14464E+00_JPRB, 0.14454E+00_JPRB, 0.14504E+00_JPRB, 0.14571E+00_JPRB, 0.14637E+00_JPRB ,&
&  0.14730E+00_JPRB, 0.14800E+00_JPRB, 0.14871E+00_JPRB, 0.14960E+00_JPRB, 0.15037E+00_JPRB, 0.15136E+00_JPRB, 0.15251E+00_JPRB ,&
&  0.15338E+00_JPRB, 0.15511E+00_JPRB, 0.15653E+00_JPRB, 0.15827E+00_JPRB, 0.16161E+00_JPRB, 0.16516E+00_JPRB, 0.17348E+00_JPRB ,&
&  0.18137E+00_JPRB, 0.19787E+00_JPRB, 0.19922E+00_JPRB, 0.19433E+00_JPRB, 0.19495E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.20493E+00_JPRB, 0.20724E+00_JPRB, 0.20943E+00_JPRB, 0.20810E+00_JPRB, 0.20817E+00_JPRB, 0.20844E+00_JPRB, 0.20963E+00_JPRB ,&
&  0.21020E+00_JPRB, 0.20980E+00_JPRB, 0.20948E+00_JPRB, 0.20799E+00_JPRB, 0.20646E+00_JPRB, 0.19967E+00_JPRB, 0.19682E+00_JPRB ,&
&  0.19457E+00_JPRB, 0.19515E+00_JPRB, 0.19568E+00_JPRB, 0.19770E+00_JPRB, 0.19785E+00_JPRB, 0.19687E+00_JPRB, 0.19374E+00_JPRB ,&
&  0.19127E+00_JPRB, 0.18977E+00_JPRB, 0.18819E+00_JPRB, 0.18843E+00_JPRB, 0.19048E+00_JPRB, 0.19267E+00_JPRB, 0.19403E+00_JPRB ,&
&  0.19467E+00_JPRB, 0.19671E+00_JPRB, 0.19851E+00_JPRB, 0.20130E+00_JPRB, 0.20810E+00_JPRB, 0.20915E+00_JPRB, 0.21000E+00_JPRB ,&
&  0.21092E+00_JPRB, 0.21214E+00_JPRB, 0.21289E+00_JPRB, 0.21267E+00_JPRB, 0.21102E+00_JPRB, 0.20612E+00_JPRB, 0.20478E+00_JPRB ,&
&  0.19234E+00_JPRB, 0.18500E+00_JPRB, 0.17810E+00_JPRB, 0.17305E+00_JPRB, 0.16776E+00_JPRB, 0.16491E+00_JPRB, 0.16171E+00_JPRB ,&
&  0.15900E+00_JPRB, 0.15605E+00_JPRB, 0.15385E+00_JPRB, 0.15289E+00_JPRB, 0.15260E+00_JPRB, 0.15232E+00_JPRB, 0.15167E+00_JPRB ,&
&  0.15175E+00_JPRB, 0.15129E+00_JPRB, 0.15096E+00_JPRB, 0.15060E+00_JPRB, 0.15036E+00_JPRB, 0.15044E+00_JPRB, 0.15116E+00_JPRB ,&
&  0.15306E+00_JPRB, 0.15485E+00_JPRB, 0.15668E+00_JPRB, 0.15841E+00_JPRB, 0.15798E+00_JPRB, 0.15633E+00_JPRB, 0.15381E+00_JPRB ,&
&  0.15161E+00_JPRB, 0.15018E+00_JPRB, 0.14864E+00_JPRB, 0.14824E+00_JPRB, 0.14883E+00_JPRB, 0.14958E+00_JPRB, 0.15088E+00_JPRB ,&
&  0.15203E+00_JPRB, 0.15297E+00_JPRB, 0.15361E+00_JPRB, 0.15404E+00_JPRB, 0.15472E+00_JPRB, 0.15537E+00_JPRB, 0.15647E+00_JPRB ,&
&  0.15769E+00_JPRB, 0.15919E+00_JPRB, 0.16055E+00_JPRB, 0.16240E+00_JPRB, 0.16517E+00_JPRB, 0.16934E+00_JPRB, 0.17651E+00_JPRB ,&
&  0.18353E+00_JPRB, 0.20201E+00_JPRB, 0.20416E+00_JPRB, 0.20339E+00_JPRB, 0.20479E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.21299E+00_JPRB, 0.21739E+00_JPRB, 0.21974E+00_JPRB, 0.21920E+00_JPRB, 0.21881E+00_JPRB, 0.21939E+00_JPRB, 0.22013E+00_JPRB ,&
&  0.22117E+00_JPRB, 0.22072E+00_JPRB, 0.21933E+00_JPRB, 0.21797E+00_JPRB, 0.21649E+00_JPRB, 0.21455E+00_JPRB, 0.21326E+00_JPRB ,&
&  0.21226E+00_JPRB, 0.21234E+00_JPRB, 0.21209E+00_JPRB, 0.21301E+00_JPRB, 0.21356E+00_JPRB, 0.21583E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.21078E+00_JPRB, 0.20880E+00_JPRB, 0.21107E+00_JPRB, 0.21144E+00_JPRB, 0.21200E+00_JPRB, 0.21278E+00_JPRB, 0.21418E+00_JPRB ,&
&  0.21170E+00_JPRB, 0.20870E+00_JPRB, 0.20480E+00_JPRB, 0.21025E+00_JPRB, 0.21875E+00_JPRB, 0.21833E+00_JPRB, 0.21773E+00_JPRB ,&
&  0.21920E+00_JPRB, 0.22188E+00_JPRB, 0.22320E+00_JPRB, 0.22352E+00_JPRB, 0.22211E+00_JPRB, 0.22149E+00_JPRB, 0.22039E+00_JPRB ,&
&  0.21754E+00_JPRB, 0.21478E+00_JPRB, 0.20609E+00_JPRB, 0.19001E+00_JPRB, 0.18361E+00_JPRB, 0.18066E+00_JPRB, 0.17674E+00_JPRB ,&
&  0.17166E+00_JPRB, 0.16939E+00_JPRB, 0.16923E+00_JPRB, 0.16760E+00_JPRB, 0.16777E+00_JPRB, 0.16734E+00_JPRB, 0.16629E+00_JPRB ,&
&  0.16494E+00_JPRB, 0.16398E+00_JPRB, 0.16247E+00_JPRB, 0.16003E+00_JPRB, 0.15817E+00_JPRB, 0.15774E+00_JPRB, 0.15851E+00_JPRB ,&
&  0.16042E+00_JPRB, 0.16333E+00_JPRB, 0.17169E+00_JPRB, 0.17657E+00_JPRB, 0.17467E+00_JPRB, 0.17085E+00_JPRB, 0.16477E+00_JPRB ,&
&  0.15927E+00_JPRB, 0.15758E+00_JPRB, 0.15640E+00_JPRB, 0.15539E+00_JPRB, 0.15574E+00_JPRB, 0.15680E+00_JPRB, 0.15785E+00_JPRB ,&
&  0.15909E+00_JPRB, 0.16054E+00_JPRB, 0.16144E+00_JPRB, 0.16194E+00_JPRB, 0.16243E+00_JPRB, 0.16270E+00_JPRB, 0.16319E+00_JPRB ,&
&  0.16330E+00_JPRB, 0.16464E+00_JPRB, 0.16580E+00_JPRB, 0.16815E+00_JPRB, 0.16974E+00_JPRB, 0.17178E+00_JPRB, 0.17609E+00_JPRB ,&
&  0.17967E+00_JPRB, 0.19035E+00_JPRB, 0.20504E+00_JPRB, 0.21235E+00_JPRB, 0.20933E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.22460E+00_JPRB, 0.22575E+00_JPRB, 0.22586E+00_JPRB, 0.22495E+00_JPRB, 0.22370E+00_JPRB, 0.22391E+00_JPRB, 0.22534E+00_JPRB ,&
&  0.22826E+00_JPRB, 0.23039E+00_JPRB, 0.23089E+00_JPRB, 0.23093E+00_JPRB, 0.23004E+00_JPRB, 0.22802E+00_JPRB, 0.22633E+00_JPRB ,&
&  0.22500E+00_JPRB, 0.22420E+00_JPRB, 0.22357E+00_JPRB, 0.22336E+00_JPRB, 0.22333E+00_JPRB, 0.22435E+00_JPRB, 0.22532E+00_JPRB ,&
&  0.22898E+00_JPRB, 0.22900E+00_JPRB, 0.22832E+00_JPRB, 0.22810E+00_JPRB, 0.22790E+00_JPRB, 0.22792E+00_JPRB, 0.22785E+00_JPRB ,&
&  0.22520E+00_JPRB, 0.21751E+00_JPRB, 0.21073E+00_JPRB, 0.21628E+00_JPRB, 0.22817E+00_JPRB, 0.22853E+00_JPRB, 0.22863E+00_JPRB ,&
&  0.22999E+00_JPRB, 0.23286E+00_JPRB, 0.23491E+00_JPRB, 0.23594E+00_JPRB, 0.23483E+00_JPRB, 0.23317E+00_JPRB, 0.23277E+00_JPRB ,&
&  0.23307E+00_JPRB, 0.23127E+00_JPRB, 0.22604E+00_JPRB, 0.22002E+00_JPRB, 0.21401E+00_JPRB, 0.21164E+00_JPRB, 0.21000E+00_JPRB ,&
&  0.20751E+00_JPRB, 0.20604E+00_JPRB, 0.20550E+00_JPRB, 0.20501E+00_JPRB, 0.20541E+00_JPRB, 0.20334E+00_JPRB, 0.20033E+00_JPRB ,&
&  0.19683E+00_JPRB, 0.19272E+00_JPRB, 0.18779E+00_JPRB, 0.18257E+00_JPRB, 0.17810E+00_JPRB, 0.17322E+00_JPRB, 0.16986E+00_JPRB ,&
&  0.16921E+00_JPRB, 0.17309E+00_JPRB, 0.18266E+00_JPRB, 0.19273E+00_JPRB, 0.19991E+00_JPRB, 0.18931E+00_JPRB, 0.18264E+00_JPRB ,&
&  0.17545E+00_JPRB, 0.16959E+00_JPRB, 0.16626E+00_JPRB, 0.16556E+00_JPRB, 0.16580E+00_JPRB, 0.16668E+00_JPRB, 0.16834E+00_JPRB ,&
&  0.17047E+00_JPRB, 0.17315E+00_JPRB, 0.17517E+00_JPRB, 0.17662E+00_JPRB, 0.17766E+00_JPRB, 0.17795E+00_JPRB, 0.17774E+00_JPRB ,&
&  0.17749E+00_JPRB, 0.17873E+00_JPRB, 0.18113E+00_JPRB, 0.18539E+00_JPRB, 0.18834E+00_JPRB, 0.19218E+00_JPRB, 0.19740E+00_JPRB ,&
&  0.20499E+00_JPRB, 0.21383E+00_JPRB, 0.21710E+00_JPRB, 0.21932E+00_JPRB, 0.22223E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.23567E+00_JPRB, 0.23698E+00_JPRB, 0.23713E+00_JPRB, 0.23751E+00_JPRB, 0.23810E+00_JPRB, 0.23996E+00_JPRB, 0.24413E+00_JPRB ,&
&  0.24824E+00_JPRB, 0.25011E+00_JPRB, 0.24853E+00_JPRB, 0.24621E+00_JPRB, 0.24503E+00_JPRB, 0.24442E+00_JPRB, 0.24425E+00_JPRB ,&
&  0.24411E+00_JPRB, 0.24313E+00_JPRB, 0.24030E+00_JPRB, 0.23638E+00_JPRB, 0.23349E+00_JPRB, 0.23277E+00_JPRB, 0.23319E+00_JPRB ,&
&  0.23693E+00_JPRB, 0.23953E+00_JPRB, 0.24193E+00_JPRB, 0.24404E+00_JPRB, 0.24454E+00_JPRB, 0.24424E+00_JPRB, 0.24572E+00_JPRB ,&
&  0.24678E+00_JPRB, 0.24443E+00_JPRB, 0.23911E+00_JPRB, 0.24316E+00_JPRB, 0.24529E+00_JPRB, 0.24660E+00_JPRB, 0.24588E+00_JPRB ,&
&  0.24655E+00_JPRB, 0.24867E+00_JPRB, 0.25075E+00_JPRB, 0.25238E+00_JPRB, 0.25172E+00_JPRB, 0.24882E+00_JPRB, 0.24808E+00_JPRB ,&
&  0.24852E+00_JPRB, 0.24744E+00_JPRB, 0.24465E+00_JPRB, 0.24210E+00_JPRB, 0.23927E+00_JPRB, 0.23801E+00_JPRB, 0.23777E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.23463E+00_JPRB, 0.23025E+00_JPRB, 0.22597E+00_JPRB, 0.22283E+00_JPRB, 0.21973E+00_JPRB, 0.21838E+00_JPRB ,&
&  0.21876E+00_JPRB, 0.21989E+00_JPRB, 0.21989E+00_JPRB, 0.21837E+00_JPRB, 0.21338E+00_JPRB, 0.20335E+00_JPRB, 0.19615E+00_JPRB ,&
&  0.19484E+00_JPRB, 0.19933E+00_JPRB, 0.20729E+00_JPRB, 0.20879E+00_JPRB, 0.21106E+00_JPRB, 0.21263E+00_JPRB, 0.21404E+00_JPRB ,&
&  0.21260E+00_JPRB, 0.19971E+00_JPRB, 0.19402E+00_JPRB, 0.19074E+00_JPRB, 0.18763E+00_JPRB, 0.18728E+00_JPRB, 0.18891E+00_JPRB ,&
&  0.19219E+00_JPRB, 0.19633E+00_JPRB, 0.19922E+00_JPRB, 0.20090E+00_JPRB, 0.20139E+00_JPRB, 0.20110E+00_JPRB, 0.20040E+00_JPRB ,&
&  0.20063E+00_JPRB, 0.20454E+00_JPRB, 0.21238E+00_JPRB, 0.22045E+00_JPRB, 0.22147E+00_JPRB, 0.22308E+00_JPRB, 0.22352E+00_JPRB ,&
&  0.22392E+00_JPRB, 0.22575E+00_JPRB, 0.22685E+00_JPRB, 0.22955E+00_JPRB, 0.23292E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.24862E+00_JPRB, 0.25173E+00_JPRB, 0.25472E+00_JPRB, 0.25637E+00_JPRB, 0.25815E+00_JPRB, 0.26003E+00_JPRB, 0.26340E+00_JPRB ,&
&  0.26738E+00_JPRB, 0.26869E+00_JPRB, 0.26695E+00_JPRB, 0.26412E+00_JPRB, 0.26233E+00_JPRB, 0.26158E+00_JPRB, 0.26143E+00_JPRB ,&
&  0.26133E+00_JPRB, 0.26123E+00_JPRB, 0.26048E+00_JPRB, 0.25867E+00_JPRB, 0.25612E+00_JPRB, 0.25353E+00_JPRB, 0.25157E+00_JPRB ,&
&  0.25126E+00_JPRB, 0.25267E+00_JPRB, 0.25493E+00_JPRB, 0.25514E+00_JPRB, 0.25527E+00_JPRB, 0.25651E+00_JPRB, 0.25854E+00_JPRB ,&
&  0.26247E+00_JPRB, 0.26443E+00_JPRB, 0.26645E+00_JPRB, 0.26247E+00_JPRB, 0.26691E+00_JPRB, 0.26852E+00_JPRB, 0.26903E+00_JPRB ,&
&  0.27028E+00_JPRB, 0.27131E+00_JPRB, 0.27234E+00_JPRB, 0.27171E+00_JPRB, 0.26752E+00_JPRB, 0.26590E+00_JPRB, 0.26696E+00_JPRB ,&
&  0.26789E+00_JPRB, 0.26633E+00_JPRB, 0.26338E+00_JPRB, 0.26060E+00_JPRB, 0.25831E+00_JPRB, 0.25667E+00_JPRB, 0.25527E+00_JPRB ,&
&  0.25453E+00_JPRB, 0.25367E+00_JPRB, 0.25165E+00_JPRB, 0.24638E+00_JPRB, 0.24040E+00_JPRB, 0.23645E+00_JPRB, 0.23367E+00_JPRB ,&
&  0.23365E+00_JPRB, 0.23554E+00_JPRB, 0.23721E+00_JPRB, 0.23728E+00_JPRB, 0.23590E+00_JPRB, 0.23345E+00_JPRB, 0.22996E+00_JPRB ,&
&  0.22502E+00_JPRB, 0.22300E+00_JPRB, 0.22248E+00_JPRB, 0.22164E+00_JPRB, 0.22212E+00_JPRB, 0.22398E+00_JPRB, 0.22592E+00_JPRB ,&
&  0.22825E+00_JPRB, 0.22830E+00_JPRB, 0.22686E+00_JPRB, 0.22445E+00_JPRB, 0.22190E+00_JPRB, 0.22168E+00_JPRB, 0.22231E+00_JPRB ,&
&  0.22290E+00_JPRB, 0.22345E+00_JPRB, 0.22439E+00_JPRB, 0.22565E+00_JPRB, 0.22729E+00_JPRB, 0.22964E+00_JPRB, 0.23158E+00_JPRB ,&
&  0.23156E+00_JPRB, 0.23045E+00_JPRB, 0.23153E+00_JPRB, 0.23331E+00_JPRB, 0.23427E+00_JPRB, 0.23494E+00_JPRB, 0.23542E+00_JPRB ,&
&  0.23510E+00_JPRB, 0.23513E+00_JPRB, 0.23700E+00_JPRB, 0.24159E+00_JPRB, 0.24579E+00_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.26341E+00_JPRB, 0.26747E+00_JPRB, 0.27273E+00_JPRB, 0.27393E+00_JPRB, 0.27635E+00_JPRB, 0.27891E+00_JPRB, 0.28233E+00_JPRB ,&
&  0.28455E+00_JPRB, 0.28474E+00_JPRB, 0.28321E+00_JPRB, 0.28168E+00_JPRB, 0.28088E+00_JPRB, 0.28066E+00_JPRB, 0.28019E+00_JPRB ,&
&  0.28002E+00_JPRB, 0.28019E+00_JPRB, 0.28015E+00_JPRB, 0.27995E+00_JPRB, 0.27875E+00_JPRB, 0.27694E+00_JPRB, 0.27492E+00_JPRB ,&
&  0.27155E+00_JPRB, 0.26932E+00_JPRB, 0.26861E+00_JPRB, 0.26836E+00_JPRB, 0.26763E+00_JPRB, 0.26815E+00_JPRB, 0.26988E+00_JPRB ,&
&  0.27362E+00_JPRB, 0.27661E+00_JPRB, 0.28050E+00_JPRB, 0.28448E+00_JPRB, 0.28685E+00_JPRB, 0.28860E+00_JPRB, 0.29065E+00_JPRB ,&
&  0.29194E+00_JPRB, 0.29229E+00_JPRB, 0.29236E+00_JPRB, 0.29238E+00_JPRB, 0.28972E+00_JPRB, 0.28909E+00_JPRB, 0.28909E+00_JPRB ,&
&  0.28826E+00_JPRB, 0.28559E+00_JPRB, 0.28179E+00_JPRB, 0.27851E+00_JPRB, 0.27528E+00_JPRB, 0.27291E+00_JPRB, 0.27094E+00_JPRB ,&
&  0.26947E+00_JPRB, 0.26874E+00_JPRB, 0.26865E+00_JPRB, 0.26711E+00_JPRB, 0.26186E+00_JPRB, 0.25671E+00_JPRB, 0.25383E+00_JPRB ,&
&  0.25248E+00_JPRB, 0.25217E+00_JPRB, 0.25353E+00_JPRB, 0.25229E+00_JPRB, 0.24844E+00_JPRB, 0.24529E+00_JPRB, 0.24395E+00_JPRB ,&
&  0.24232E+00_JPRB, 0.24159E+00_JPRB, 0.24232E+00_JPRB, 0.24300E+00_JPRB, 0.24365E+00_JPRB, 0.24391E+00_JPRB, 0.24521E+00_JPRB ,&
&  0.24647E+00_JPRB, 0.24807E+00_JPRB, 0.24794E+00_JPRB, 0.24524E+00_JPRB, 0.24150E+00_JPRB, 0.23881E+00_JPRB, 0.23835E+00_JPRB ,&
&  0.23991E+00_JPRB, 0.24146E+00_JPRB, 0.24194E+00_JPRB, 0.24214E+00_JPRB, 0.24326E+00_JPRB, 0.24575E+00_JPRB, 0.24897E+00_JPRB ,&
&  0.25036E+00_JPRB, 0.25169E+00_JPRB, 0.25354E+00_JPRB, 0.25547E+00_JPRB, 0.25689E+00_JPRB, 0.25791E+00_JPRB, 0.25844E+00_JPRB ,&
&  0.25797E+00_JPRB, 0.25871E+00_JPRB, 0.26019E+00_JPRB, 0.26190E+00_JPRB, 0.26306E+00_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.28611E+00_JPRB, 0.28869E+00_JPRB, 0.29203E+00_JPRB, 0.29270E+00_JPRB, 0.29304E+00_JPRB, 0.29449E+00_JPRB, 0.29749E+00_JPRB ,&
&  0.29949E+00_JPRB, 0.29999E+00_JPRB, 0.29985E+00_JPRB, 0.29961E+00_JPRB, 0.30003E+00_JPRB, 0.30171E+00_JPRB, 0.30251E+00_JPRB ,&
&  0.30192E+00_JPRB, 0.30130E+00_JPRB, 0.30044E+00_JPRB, 0.29978E+00_JPRB, 0.29829E+00_JPRB, 0.29607E+00_JPRB, 0.29421E+00_JPRB ,&
&  0.29163E+00_JPRB, 0.28924E+00_JPRB, 0.28748E+00_JPRB, 0.28611E+00_JPRB, 0.28537E+00_JPRB, 0.28540E+00_JPRB, 0.28694E+00_JPRB ,&
&  0.28886E+00_JPRB, 0.29131E+00_JPRB, 0.29441E+00_JPRB, 0.29800E+00_JPRB, 0.30159E+00_JPRB, 0.30565E+00_JPRB, 0.30926E+00_JPRB ,&
&  0.31212E+00_JPRB, 0.31452E+00_JPRB, 0.31613E+00_JPRB, 0.31346E+00_JPRB, 0.31021E+00_JPRB, 0.30986E+00_JPRB, 0.30997E+00_JPRB ,&
&  0.30765E+00_JPRB, 0.30387E+00_JPRB, 0.29890E+00_JPRB, 0.29612E+00_JPRB, 0.29534E+00_JPRB, 0.29426E+00_JPRB, 0.29296E+00_JPRB ,&
&  0.29211E+00_JPRB, 0.29143E+00_JPRB, 0.29016E+00_JPRB, 0.28795E+00_JPRB, 0.28440E+00_JPRB, 0.27960E+00_JPRB, 0.27480E+00_JPRB ,&
&  0.27175E+00_JPRB, 0.26962E+00_JPRB, 0.26958E+00_JPRB, 0.26753E+00_JPRB, 0.26338E+00_JPRB, 0.25947E+00_JPRB, 0.25877E+00_JPRB ,&
&  0.25868E+00_JPRB, 0.25918E+00_JPRB, 0.25916E+00_JPRB, 0.25822E+00_JPRB, 0.25861E+00_JPRB, 0.25953E+00_JPRB, 0.26109E+00_JPRB ,&
&  0.26529E+00_JPRB, 0.26826E+00_JPRB, 0.26820E+00_JPRB, 0.26717E+00_JPRB, 0.26527E+00_JPRB, 0.26483E+00_JPRB, 0.26546E+00_JPRB ,&
&  0.26613E+00_JPRB, 0.26644E+00_JPRB, 0.26671E+00_JPRB, 0.26617E+00_JPRB, 0.26631E+00_JPRB, 0.26780E+00_JPRB, 0.26966E+00_JPRB ,&
&  0.27192E+00_JPRB, 0.27470E+00_JPRB, 0.27719E+00_JPRB, 0.28002E+00_JPRB, 0.28289E+00_JPRB, 0.28488E+00_JPRB, 0.28661E+00_JPRB ,&
&  0.28794E+00_JPRB, 0.28821E+00_JPRB, 0.28774E+00_JPRB, 0.28736E+00_JPRB, 0.28693E+00_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.31670E+00_JPRB, 0.31650E+00_JPRB, 0.31714E+00_JPRB, 0.31723E+00_JPRB, 0.31676E+00_JPRB, 0.31710E+00_JPRB, 0.31953E+00_JPRB ,&
&  0.32158E+00_JPRB, 0.32272E+00_JPRB, 0.32368E+00_JPRB, 0.32406E+00_JPRB, 0.32417E+00_JPRB, 0.32575E+00_JPRB, 0.32701E+00_JPRB ,&
&  0.32718E+00_JPRB, 0.32608E+00_JPRB, 0.32482E+00_JPRB, 0.32323E+00_JPRB, 0.32105E+00_JPRB, 0.31873E+00_JPRB, 0.31688E+00_JPRB ,&
&  0.31441E+00_JPRB, 0.31165E+00_JPRB, 0.30898E+00_JPRB, 0.30793E+00_JPRB, 0.30790E+00_JPRB, 0.30806E+00_JPRB, 0.30896E+00_JPRB ,&
&  0.30981E+00_JPRB, 0.31233E+00_JPRB, 0.31540E+00_JPRB, 0.31902E+00_JPRB, 0.32258E+00_JPRB, 0.32730E+00_JPRB, 0.33247E+00_JPRB ,&
&  0.33728E+00_JPRB, 0.33975E+00_JPRB, 0.34181E+00_JPRB, 0.34043E+00_JPRB, 0.34100E+00_JPRB, 0.34119E+00_JPRB, 0.34121E+00_JPRB ,&
&  0.34017E+00_JPRB, 0.33503E+00_JPRB, 0.33094E+00_JPRB, 0.33068E+00_JPRB, 0.33174E+00_JPRB, 0.33260E+00_JPRB, 0.33314E+00_JPRB ,&
&  0.33250E+00_JPRB, 0.33111E+00_JPRB, 0.32790E+00_JPRB, 0.32345E+00_JPRB, 0.31866E+00_JPRB, 0.31298E+00_JPRB, 0.30500E+00_JPRB ,&
&  0.29755E+00_JPRB, 0.29150E+00_JPRB, 0.28685E+00_JPRB, 0.28280E+00_JPRB, 0.28003E+00_JPRB, 0.27701E+00_JPRB, 0.27598E+00_JPRB ,&
&  0.27619E+00_JPRB, 0.27803E+00_JPRB, 0.27910E+00_JPRB, 0.27847E+00_JPRB, 0.27738E+00_JPRB, 0.27760E+00_JPRB, 0.27994E+00_JPRB ,&
&  0.28416E+00_JPRB, 0.28742E+00_JPRB, 0.28855E+00_JPRB, 0.28827E+00_JPRB, 0.28756E+00_JPRB, 0.28847E+00_JPRB, 0.29034E+00_JPRB ,&
&  0.29132E+00_JPRB, 0.29191E+00_JPRB, 0.29310E+00_JPRB, 0.29305E+00_JPRB, 0.29501E+00_JPRB, 0.29827E+00_JPRB, 0.30189E+00_JPRB ,&
&  0.30537E+00_JPRB, 0.30978E+00_JPRB, 0.31454E+00_JPRB, 0.31911E+00_JPRB, 0.32356E+00_JPRB, 0.32513E+00_JPRB, 0.32653E+00_JPRB ,&
&  0.32722E+00_JPRB, 0.32612E+00_JPRB, 0.32435E+00_JPRB, 0.32183E+00_JPRB, 0.31878E+00_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.35372E+00_JPRB, 0.35159E+00_JPRB, 0.34962E+00_JPRB, 0.34834E+00_JPRB, 0.34712E+00_JPRB, 0.34597E+00_JPRB, 0.34535E+00_JPRB ,&
&  0.34555E+00_JPRB, 0.34542E+00_JPRB, 0.34781E+00_JPRB, 0.34867E+00_JPRB, 0.34871E+00_JPRB, 0.34959E+00_JPRB, 0.35018E+00_JPRB ,&
&  0.34881E+00_JPRB, 0.34769E+00_JPRB, 0.34649E+00_JPRB, 0.34421E+00_JPRB, 0.34055E+00_JPRB, 0.33859E+00_JPRB, 0.33687E+00_JPRB ,&
&  0.33412E+00_JPRB, 0.33083E+00_JPRB, 0.32799E+00_JPRB, 0.32665E+00_JPRB, 0.32696E+00_JPRB, 0.32678E+00_JPRB, 0.32752E+00_JPRB ,&
&  0.32770E+00_JPRB, 0.33128E+00_JPRB, 0.32001E+00_JPRB, 0.32974E+00_JPRB, 0.33888E+00_JPRB, 0.34794E+00_JPRB, 0.35745E+00_JPRB ,&
&  0.36403E+00_JPRB, 0.37035E+00_JPRB, 0.37824E+00_JPRB, 0.38168E+00_JPRB, 0.38152E+00_JPRB, 0.37864E+00_JPRB, 0.37806E+00_JPRB ,&
&  0.37780E+00_JPRB, 0.37528E+00_JPRB, 0.37385E+00_JPRB, 0.37440E+00_JPRB, 0.37521E+00_JPRB, 0.37431E+00_JPRB, 0.37251E+00_JPRB ,&
&  0.37099E+00_JPRB, 0.36815E+00_JPRB, 0.36326E+00_JPRB, 0.35775E+00_JPRB, 0.35200E+00_JPRB, 0.34515E+00_JPRB, 0.33611E+00_JPRB ,&
&  0.32588E+00_JPRB, 0.31772E+00_JPRB, 0.31090E+00_JPRB, 0.30650E+00_JPRB, 0.30424E+00_JPRB, 0.30196E+00_JPRB, 0.29984E+00_JPRB ,&
&  0.29919E+00_JPRB, 0.29951E+00_JPRB, 0.29965E+00_JPRB, 0.29949E+00_JPRB, 0.30035E+00_JPRB, 0.30121E+00_JPRB, 0.30400E+00_JPRB ,&
&  0.30708E+00_JPRB, 0.30834E+00_JPRB, 0.30859E+00_JPRB, 0.30890E+00_JPRB, 0.30772E+00_JPRB, 0.30831E+00_JPRB, 0.31009E+00_JPRB ,&
&  0.31211E+00_JPRB, 0.31481E+00_JPRB, 0.31682E+00_JPRB, 0.31867E+00_JPRB, 0.32108E+00_JPRB, 0.32469E+00_JPRB, 0.33018E+00_JPRB ,&
&  0.33548E+00_JPRB, 0.34167E+00_JPRB, 0.34916E+00_JPRB, 0.35685E+00_JPRB, 0.36238E+00_JPRB, 0.36549E+00_JPRB, 0.36822E+00_JPRB ,&
&  0.36899E+00_JPRB, 0.36600E+00_JPRB, 0.36198E+00_JPRB, 0.35897E+00_JPRB, 0.35625E+00_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.40122E+00_JPRB, 0.39702E+00_JPRB, 0.39203E+00_JPRB, 0.38800E+00_JPRB, 0.38409E+00_JPRB, 0.37961E+00_JPRB, 0.37674E+00_JPRB ,&
&  0.37529E+00_JPRB, 0.37495E+00_JPRB, 0.37582E+00_JPRB, 0.37595E+00_JPRB, 0.37572E+00_JPRB, 0.37613E+00_JPRB, 0.37553E+00_JPRB ,&
&  0.37447E+00_JPRB, 0.37287E+00_JPRB, 0.37085E+00_JPRB, 0.36805E+00_JPRB, 0.36460E+00_JPRB, 0.36138E+00_JPRB, 0.35713E+00_JPRB ,&
&  0.35415E+00_JPRB, 0.35246E+00_JPRB, 0.35158E+00_JPRB, 0.35138E+00_JPRB, 0.35252E+00_JPRB, 0.35407E+00_JPRB, 0.35554E+00_JPRB ,&
&  0.35517E+00_JPRB, 0.34637E+00_JPRB, 0.35228E+00_JPRB, 0.35533E+00_JPRB, 0.36403E+00_JPRB, 0.37875E+00_JPRB, 0.39181E+00_JPRB ,&
&  0.40020E+00_JPRB, 0.41031E+00_JPRB, 0.42048E+00_JPRB, 0.42672E+00_JPRB, 0.42586E+00_JPRB, 0.42421E+00_JPRB, 0.42645E+00_JPRB ,&
&  0.42796E+00_JPRB, 0.42709E+00_JPRB, 0.42620E+00_JPRB, 0.42449E+00_JPRB, 0.42141E+00_JPRB, 0.41784E+00_JPRB, 0.41219E+00_JPRB ,&
&  0.40641E+00_JPRB, 0.40201E+00_JPRB, 0.39745E+00_JPRB, 0.39276E+00_JPRB, 0.38606E+00_JPRB, 0.37828E+00_JPRB, 0.36930E+00_JPRB ,&
&  0.35982E+00_JPRB, 0.35294E+00_JPRB, 0.34842E+00_JPRB, 0.34419E+00_JPRB, 0.34005E+00_JPRB, 0.33729E+00_JPRB, 0.33419E+00_JPRB ,&
&  0.33290E+00_JPRB, 0.33335E+00_JPRB, 0.33257E+00_JPRB, 0.33217E+00_JPRB, 0.33288E+00_JPRB, 0.33289E+00_JPRB, 0.33352E+00_JPRB ,&
&  0.33265E+00_JPRB, 0.33149E+00_JPRB, 0.33084E+00_JPRB, 0.33033E+00_JPRB, 0.32980E+00_JPRB, 0.33118E+00_JPRB, 0.33375E+00_JPRB ,&
&  0.33658E+00_JPRB, 0.34068E+00_JPRB, 0.34513E+00_JPRB, 0.34857E+00_JPRB, 0.35161E+00_JPRB, 0.35643E+00_JPRB, 0.36261E+00_JPRB ,&
&  0.36982E+00_JPRB, 0.37859E+00_JPRB, 0.38750E+00_JPRB, 0.39682E+00_JPRB, 0.40339E+00_JPRB, 0.40681E+00_JPRB, 0.40948E+00_JPRB ,&
&  0.41139E+00_JPRB, 0.40835E+00_JPRB, 0.40758E+00_JPRB, 0.40718E+00_JPRB, 0.40460E+00_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.43658E+00_JPRB, 0.43456E+00_JPRB, 0.43061E+00_JPRB, 0.42548E+00_JPRB, 0.41983E+00_JPRB, 0.41657E+00_JPRB, 0.41468E+00_JPRB ,&
&  0.41528E+00_JPRB, 0.41439E+00_JPRB, 0.41495E+00_JPRB, 0.41222E+00_JPRB, 0.40904E+00_JPRB, 0.40655E+00_JPRB, 0.40400E+00_JPRB ,&
&  0.40111E+00_JPRB, 0.39751E+00_JPRB, 0.39397E+00_JPRB, 0.39098E+00_JPRB, 0.38744E+00_JPRB, 0.38266E+00_JPRB, 0.37827E+00_JPRB ,&
&  0.37496E+00_JPRB, 0.37339E+00_JPRB, 0.37246E+00_JPRB, 0.37184E+00_JPRB, 0.37133E+00_JPRB, 0.37240E+00_JPRB, 0.37319E+00_JPRB ,&
&  0.37575E+00_JPRB, 0.34743E+00_JPRB, 0.35863E+00_JPRB, 0.36443E+00_JPRB, 0.37370E+00_JPRB, 0.39146E+00_JPRB, 0.41253E+00_JPRB ,&
&  0.43581E+00_JPRB, 0.45866E+00_JPRB, 0.47541E+00_JPRB, 0.47916E+00_JPRB, 0.47301E+00_JPRB, 0.46881E+00_JPRB, 0.46953E+00_JPRB ,&
&  0.46787E+00_JPRB, 0.46328E+00_JPRB, 0.45920E+00_JPRB, 0.45431E+00_JPRB, 0.44709E+00_JPRB, 0.43685E+00_JPRB, 0.42681E+00_JPRB ,&
&  0.42099E+00_JPRB, 0.41772E+00_JPRB, 0.41475E+00_JPRB, 0.41146E+00_JPRB, 0.40608E+00_JPRB, 0.40090E+00_JPRB, 0.39634E+00_JPRB ,&
&  0.39210E+00_JPRB, 0.38630E+00_JPRB, 0.38270E+00_JPRB, 0.38033E+00_JPRB, 0.37707E+00_JPRB, 0.37504E+00_JPRB, 0.37330E+00_JPRB ,&
&  0.37121E+00_JPRB, 0.36862E+00_JPRB, 0.36695E+00_JPRB, 0.36621E+00_JPRB, 0.36392E+00_JPRB, 0.36157E+00_JPRB, 0.35888E+00_JPRB ,&
&  0.35773E+00_JPRB, 0.35704E+00_JPRB, 0.35606E+00_JPRB, 0.35593E+00_JPRB, 0.35689E+00_JPRB, 0.35953E+00_JPRB, 0.36304E+00_JPRB ,&
&  0.36714E+00_JPRB, 0.37188E+00_JPRB, 0.37620E+00_JPRB, 0.37975E+00_JPRB, 0.38465E+00_JPRB, 0.38891E+00_JPRB, 0.39463E+00_JPRB ,&
&  0.40162E+00_JPRB, 0.40902E+00_JPRB, 0.41722E+00_JPRB, 0.42799E+00_JPRB, 0.43947E+00_JPRB, 0.44539E+00_JPRB, 0.45031E+00_JPRB ,&
&  0.45394E+00_JPRB, 0.44968E+00_JPRB, 0.44601E+00_JPRB, 0.44107E+00_JPRB, 0.43708E+00_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.48133E+00_JPRB, 0.47785E+00_JPRB, 0.47333E+00_JPRB, 0.46733E+00_JPRB, 0.46220E+00_JPRB, 0.45839E+00_JPRB, 0.45634E+00_JPRB ,&
&  0.45655E+00_JPRB, 0.45744E+00_JPRB, 0.45574E+00_JPRB, 0.45023E+00_JPRB, 0.44476E+00_JPRB, 0.44105E+00_JPRB, 0.43897E+00_JPRB ,&
&  0.43613E+00_JPRB, 0.43150E+00_JPRB, 0.42548E+00_JPRB, 0.41963E+00_JPRB, 0.41315E+00_JPRB, 0.40778E+00_JPRB, 0.40331E+00_JPRB ,&
&  0.39964E+00_JPRB, 0.39778E+00_JPRB, 0.39639E+00_JPRB, 0.39548E+00_JPRB, 0.39589E+00_JPRB, 0.39560E+00_JPRB, 0.40025E+00_JPRB ,&
&  0.37475E+00_JPRB, 0.34486E+00_JPRB, 0.36063E+00_JPRB, 0.37637E+00_JPRB, 0.38862E+00_JPRB, 0.41553E+00_JPRB, 0.45390E+00_JPRB ,&
&  0.49853E+00_JPRB, 0.53713E+00_JPRB, 0.56079E+00_JPRB, 0.55832E+00_JPRB, 0.54436E+00_JPRB, 0.53437E+00_JPRB, 0.52864E+00_JPRB ,&
&  0.51721E+00_JPRB, 0.50405E+00_JPRB, 0.49318E+00_JPRB, 0.48058E+00_JPRB, 0.46708E+00_JPRB, 0.45468E+00_JPRB, 0.44463E+00_JPRB ,&
&  0.43673E+00_JPRB, 0.43193E+00_JPRB, 0.43046E+00_JPRB, 0.42875E+00_JPRB, 0.42730E+00_JPRB, 0.42744E+00_JPRB, 0.42641E+00_JPRB ,&
&  0.42550E+00_JPRB, 0.42411E+00_JPRB, 0.42380E+00_JPRB, 0.42315E+00_JPRB, 0.42339E+00_JPRB, 0.42238E+00_JPRB, 0.42033E+00_JPRB ,&
&  0.41736E+00_JPRB, 0.41489E+00_JPRB, 0.41343E+00_JPRB, 0.41000E+00_JPRB, 0.40243E+00_JPRB, 0.39521E+00_JPRB, 0.39131E+00_JPRB ,&
&  0.39022E+00_JPRB, 0.39007E+00_JPRB, 0.38995E+00_JPRB, 0.39126E+00_JPRB, 0.39357E+00_JPRB, 0.39665E+00_JPRB, 0.40081E+00_JPRB ,&
&  0.40608E+00_JPRB, 0.41144E+00_JPRB, 0.41583E+00_JPRB, 0.42010E+00_JPRB, 0.42674E+00_JPRB, 0.43310E+00_JPRB, 0.43940E+00_JPRB ,&
&  0.44584E+00_JPRB, 0.45362E+00_JPRB, 0.46270E+00_JPRB, 0.47753E+00_JPRB, 0.49692E+00_JPRB, 0.51160E+00_JPRB, 0.51642E+00_JPRB ,&
&  0.51251E+00_JPRB, 0.49832E+00_JPRB, 0.49368E+00_JPRB, 0.48602E+00_JPRB, 0.48204E+00_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.51060E+00_JPRB, 0.51137E+00_JPRB, 0.50817E+00_JPRB, 0.50409E+00_JPRB, 0.50125E+00_JPRB, 0.49659E+00_JPRB, 0.49286E+00_JPRB ,&
&  0.48752E+00_JPRB, 0.48342E+00_JPRB, 0.47637E+00_JPRB, 0.47145E+00_JPRB, 0.46832E+00_JPRB, 0.46696E+00_JPRB, 0.46496E+00_JPRB ,&
&  0.46010E+00_JPRB, 0.45247E+00_JPRB, 0.44476E+00_JPRB, 0.43856E+00_JPRB, 0.43392E+00_JPRB, 0.42906E+00_JPRB, 0.42450E+00_JPRB ,&
&  0.42030E+00_JPRB, 0.41648E+00_JPRB, 0.41299E+00_JPRB, 0.41257E+00_JPRB, 0.41248E+00_JPRB, 0.41160E+00_JPRB, 0.40958E+00_JPRB ,&
&  0.41635E+00_JPRB, 0.36908E+00_JPRB, 0.38960E+00_JPRB, 0.40998E+00_JPRB, 0.42685E+00_JPRB, 0.46894E+00_JPRB, 0.52967E+00_JPRB ,&
&  0.58384E+00_JPRB, 0.62441E+00_JPRB, 0.63963E+00_JPRB, 0.62571E+00_JPRB, 0.60559E+00_JPRB, 0.59227E+00_JPRB, 0.57948E+00_JPRB ,&
&  0.55682E+00_JPRB, 0.53715E+00_JPRB, 0.51954E+00_JPRB, 0.50234E+00_JPRB, 0.48613E+00_JPRB, 0.47212E+00_JPRB, 0.46003E+00_JPRB ,&
&  0.45228E+00_JPRB, 0.44886E+00_JPRB, 0.44663E+00_JPRB, 0.44512E+00_JPRB, 0.44580E+00_JPRB, 0.44818E+00_JPRB, 0.44904E+00_JPRB ,&
&  0.45026E+00_JPRB, 0.45394E+00_JPRB, 0.45445E+00_JPRB, 0.45486E+00_JPRB, 0.45786E+00_JPRB, 0.46478E+00_JPRB, 0.46596E+00_JPRB ,&
&  0.46183E+00_JPRB, 0.45417E+00_JPRB, 0.44609E+00_JPRB, 0.43932E+00_JPRB, 0.43367E+00_JPRB, 0.42840E+00_JPRB, 0.42651E+00_JPRB ,&
&  0.42464E+00_JPRB, 0.42476E+00_JPRB, 0.42556E+00_JPRB, 0.42860E+00_JPRB, 0.43228E+00_JPRB, 0.43671E+00_JPRB, 0.44206E+00_JPRB ,&
&  0.44810E+00_JPRB, 0.45438E+00_JPRB, 0.46253E+00_JPRB, 0.47150E+00_JPRB, 0.47912E+00_JPRB, 0.48408E+00_JPRB, 0.48731E+00_JPRB ,&
&  0.49049E+00_JPRB, 0.49761E+00_JPRB, 0.50343E+00_JPRB, 0.53409E+00_JPRB, 0.55445E+00_JPRB, 0.57263E+00_JPRB, 0.56967E+00_JPRB ,&
&  0.55537E+00_JPRB, 0.53859E+00_JPRB, 0.46130E+00_JPRB, 0.49288E+00_JPRB, 0.50392E+00_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.56749E+00_JPRB, 0.56455E+00_JPRB, 0.55982E+00_JPRB, 0.55657E+00_JPRB, 0.54879E+00_JPRB, 0.54033E+00_JPRB, 0.53196E+00_JPRB ,&
&  0.52110E+00_JPRB, 0.50987E+00_JPRB, 0.50003E+00_JPRB, 0.49604E+00_JPRB, 0.49281E+00_JPRB, 0.48953E+00_JPRB, 0.48177E+00_JPRB ,&
&  0.47105E+00_JPRB, 0.46086E+00_JPRB, 0.45190E+00_JPRB, 0.44612E+00_JPRB, 0.44414E+00_JPRB, 0.44154E+00_JPRB, 0.43800E+00_JPRB ,&
&  0.43556E+00_JPRB, 0.43571E+00_JPRB, 0.43450E+00_JPRB, 0.43312E+00_JPRB, 0.43121E+00_JPRB, 0.42577E+00_JPRB, 0.42537E+00_JPRB ,&
&  0.43627E+00_JPRB, 0.40686E+00_JPRB, 0.45389E+00_JPRB, 0.49993E+00_JPRB, 0.53402E+00_JPRB, 0.58758E+00_JPRB, 0.65638E+00_JPRB ,&
&  0.72079E+00_JPRB, 0.73652E+00_JPRB, 0.72551E+00_JPRB, 0.68863E+00_JPRB, 0.65984E+00_JPRB, 0.63771E+00_JPRB, 0.60634E+00_JPRB ,&
&  0.57583E+00_JPRB, 0.55503E+00_JPRB, 0.53942E+00_JPRB, 0.52621E+00_JPRB, 0.51644E+00_JPRB, 0.50835E+00_JPRB, 0.50137E+00_JPRB ,&
&  0.49262E+00_JPRB, 0.48399E+00_JPRB, 0.47647E+00_JPRB, 0.47263E+00_JPRB, 0.47396E+00_JPRB, 0.47945E+00_JPRB, 0.49192E+00_JPRB ,&
&  0.50203E+00_JPRB, 0.50775E+00_JPRB, 0.50825E+00_JPRB, 0.51226E+00_JPRB, 0.51946E+00_JPRB, 0.52513E+00_JPRB, 0.52252E+00_JPRB ,&
&  0.51069E+00_JPRB, 0.49634E+00_JPRB, 0.48554E+00_JPRB, 0.47856E+00_JPRB, 0.47552E+00_JPRB, 0.47608E+00_JPRB, 0.47946E+00_JPRB ,&
&  0.47992E+00_JPRB, 0.47865E+00_JPRB, 0.47754E+00_JPRB, 0.47871E+00_JPRB, 0.48109E+00_JPRB, 0.48640E+00_JPRB, 0.49389E+00_JPRB ,&
&  0.50251E+00_JPRB, 0.51191E+00_JPRB, 0.52260E+00_JPRB, 0.53270E+00_JPRB, 0.53752E+00_JPRB, 0.54111E+00_JPRB, 0.54238E+00_JPRB ,&
&  0.54470E+00_JPRB, 0.55520E+00_JPRB, 0.56730E+00_JPRB, 0.63848E+00_JPRB, 0.66831E+00_JPRB, 0.66792E+00_JPRB, 0.63121E+00_JPRB ,&
&  0.59769E+00_JPRB, 0.56656E+00_JPRB, 0.56005E+00_JPRB, 0.51927E+00_JPRB, 0.55840E+00_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.63529E+00_JPRB, 0.62646E+00_JPRB, 0.61553E+00_JPRB, 0.60595E+00_JPRB, 0.59192E+00_JPRB, 0.57980E+00_JPRB, 0.56458E+00_JPRB ,&
&  0.55049E+00_JPRB, 0.53686E+00_JPRB, 0.52509E+00_JPRB, 0.52062E+00_JPRB, 0.51296E+00_JPRB, 0.50187E+00_JPRB, 0.48279E+00_JPRB ,&
&  0.46692E+00_JPRB, 0.45468E+00_JPRB, 0.44736E+00_JPRB, 0.44426E+00_JPRB, 0.44215E+00_JPRB, 0.43893E+00_JPRB, 0.43750E+00_JPRB ,&
&  0.43818E+00_JPRB, 0.43755E+00_JPRB, 0.43938E+00_JPRB, 0.43840E+00_JPRB, 0.44097E+00_JPRB, 0.43682E+00_JPRB, 0.43392E+00_JPRB ,&
&  0.45384E+00_JPRB, 0.46295E+00_JPRB, 0.54344E+00_JPRB, 0.63095E+00_JPRB, 0.71323E+00_JPRB, 0.77132E+00_JPRB, 0.81221E+00_JPRB ,&
&  0.83913E+00_JPRB, 0.82101E+00_JPRB, 0.78450E+00_JPRB, 0.73218E+00_JPRB, 0.68975E+00_JPRB, 0.64459E+00_JPRB, 0.60052E+00_JPRB ,&
&  0.57825E+00_JPRB, 0.56631E+00_JPRB, 0.55644E+00_JPRB, 0.54777E+00_JPRB, 0.53924E+00_JPRB, 0.53413E+00_JPRB, 0.53144E+00_JPRB ,&
&  0.52472E+00_JPRB, 0.51631E+00_JPRB, 0.50619E+00_JPRB, 0.50213E+00_JPRB, 0.50238E+00_JPRB, 0.51981E+00_JPRB, 0.55477E+00_JPRB ,&
&  0.57144E+00_JPRB, 0.58198E+00_JPRB, 0.59044E+00_JPRB, 0.60303E+00_JPRB, 0.60673E+00_JPRB, 0.60243E+00_JPRB, 0.58165E+00_JPRB ,&
&  0.56387E+00_JPRB, 0.55148E+00_JPRB, 0.54525E+00_JPRB, 0.54271E+00_JPRB, 0.54307E+00_JPRB, 0.54229E+00_JPRB, 0.54269E+00_JPRB ,&
&  0.53881E+00_JPRB, 0.53413E+00_JPRB, 0.53280E+00_JPRB, 0.53335E+00_JPRB, 0.53475E+00_JPRB, 0.53606E+00_JPRB, 0.53851E+00_JPRB ,&
&  0.54323E+00_JPRB, 0.54922E+00_JPRB, 0.55676E+00_JPRB, 0.56238E+00_JPRB, 0.56131E+00_JPRB, 0.56298E+00_JPRB, 0.57082E+00_JPRB ,&
&  0.58017E+00_JPRB, 0.59936E+00_JPRB, 0.61635E+00_JPRB, 0.68859E+00_JPRB, 0.71973E+00_JPRB, 0.69804E+00_JPRB, 0.64950E+00_JPRB ,&
&  0.61186E+00_JPRB, 0.59412E+00_JPRB, 0.59359E+00_JPRB, 0.61530E+00_JPRB, 0.61168E+00_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.75761E+00_JPRB, 0.71173E+00_JPRB, 0.66970E+00_JPRB, 0.64352E+00_JPRB, 0.62547E+00_JPRB, 0.61135E+00_JPRB, 0.59434E+00_JPRB ,&
&  0.57812E+00_JPRB, 0.56216E+00_JPRB, 0.54930E+00_JPRB, 0.54005E+00_JPRB, 0.53055E+00_JPRB, 0.51144E+00_JPRB, 0.48590E+00_JPRB ,&
&  0.46867E+00_JPRB, 0.46453E+00_JPRB, 0.46165E+00_JPRB, 0.45550E+00_JPRB, 0.44983E+00_JPRB, 0.44809E+00_JPRB, 0.44551E+00_JPRB ,&
&  0.44532E+00_JPRB, 0.44539E+00_JPRB, 0.44801E+00_JPRB, 0.44889E+00_JPRB, 0.44630E+00_JPRB, 0.45236E+00_JPRB, 0.45204E+00_JPRB ,&
&  0.47755E+00_JPRB, 0.52457E+00_JPRB, 0.65766E+00_JPRB, 0.81043E+00_JPRB, 0.92754E+00_JPRB, 0.97168E+00_JPRB, 0.94759E+00_JPRB ,&
&  0.91569E+00_JPRB, 0.88169E+00_JPRB, 0.83223E+00_JPRB, 0.78057E+00_JPRB, 0.72214E+00_JPRB, 0.66041E+00_JPRB, 0.62427E+00_JPRB ,&
&  0.61136E+00_JPRB, 0.59670E+00_JPRB, 0.58196E+00_JPRB, 0.56455E+00_JPRB, 0.54845E+00_JPRB, 0.54314E+00_JPRB, 0.54446E+00_JPRB ,&
&  0.54116E+00_JPRB, 0.53688E+00_JPRB, 0.53217E+00_JPRB, 0.53520E+00_JPRB, 0.55996E+00_JPRB, 0.62231E+00_JPRB, 0.69493E+00_JPRB ,&
&  0.72327E+00_JPRB, 0.74384E+00_JPRB, 0.75168E+00_JPRB, 0.74565E+00_JPRB, 0.72384E+00_JPRB, 0.69545E+00_JPRB, 0.66457E+00_JPRB ,&
&  0.64098E+00_JPRB, 0.62494E+00_JPRB, 0.61736E+00_JPRB, 0.61290E+00_JPRB, 0.60962E+00_JPRB, 0.60512E+00_JPRB, 0.59881E+00_JPRB ,&
&  0.59468E+00_JPRB, 0.59289E+00_JPRB, 0.59319E+00_JPRB, 0.59025E+00_JPRB, 0.58621E+00_JPRB, 0.58065E+00_JPRB, 0.57569E+00_JPRB ,&
&  0.57039E+00_JPRB, 0.56769E+00_JPRB, 0.57589E+00_JPRB, 0.59133E+00_JPRB, 0.60943E+00_JPRB, 0.62367E+00_JPRB, 0.63438E+00_JPRB ,&
&  0.64310E+00_JPRB, 0.67327E+00_JPRB, 0.68022E+00_JPRB, 0.78780E+00_JPRB, 0.86436E+00_JPRB, 0.79237E+00_JPRB, 0.73057E+00_JPRB ,&
&  0.68204E+00_JPRB, 0.65936E+00_JPRB, 0.65529E+00_JPRB, 0.67891E+00_JPRB, 0.75447E+00_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.70902E+00_JPRB, 0.66620E+00_JPRB, 0.64376E+00_JPRB, 0.63116E+00_JPRB, 0.62399E+00_JPRB, 0.61737E+00_JPRB, 0.60428E+00_JPRB ,&
&  0.58642E+00_JPRB, 0.57068E+00_JPRB, 0.55892E+00_JPRB, 0.55024E+00_JPRB, 0.53328E+00_JPRB, 0.50692E+00_JPRB, 0.48698E+00_JPRB ,&
&  0.48235E+00_JPRB, 0.47878E+00_JPRB, 0.47603E+00_JPRB, 0.46941E+00_JPRB, 0.46341E+00_JPRB, 0.46081E+00_JPRB, 0.45717E+00_JPRB ,&
&  0.45244E+00_JPRB, 0.44764E+00_JPRB, 0.44816E+00_JPRB, 0.45178E+00_JPRB, 0.45545E+00_JPRB, 0.45874E+00_JPRB, 0.46266E+00_JPRB ,&
&  0.49332E+00_JPRB, 0.67262E+00_JPRB, 0.87832E+00_JPRB, 0.11543E+01_JPRB, 0.12816E+01_JPRB, 0.12117E+01_JPRB, 0.11517E+01_JPRB ,&
&  0.10705E+01_JPRB, 0.98263E+00_JPRB, 0.90158E+00_JPRB, 0.82460E+00_JPRB, 0.73988E+00_JPRB, 0.69054E+00_JPRB, 0.66683E+00_JPRB ,&
&  0.64839E+00_JPRB, 0.62479E+00_JPRB, 0.60095E+00_JPRB, 0.57692E+00_JPRB, 0.56111E+00_JPRB, 0.56442E+00_JPRB, 0.57556E+00_JPRB ,&
&  0.57529E+00_JPRB, 0.57644E+00_JPRB, 0.58013E+00_JPRB, 0.61275E+00_JPRB, 0.73511E+00_JPRB, 0.85058E+00_JPRB, 0.99426E+00_JPRB ,&
&  0.10114E+01_JPRB, 0.10520E+01_JPRB, 0.98750E+00_JPRB, 0.91801E+00_JPRB, 0.85641E+00_JPRB, 0.80682E+00_JPRB, 0.76864E+00_JPRB ,&
&  0.74307E+00_JPRB, 0.72322E+00_JPRB, 0.70753E+00_JPRB, 0.69449E+00_JPRB, 0.68299E+00_JPRB, 0.67047E+00_JPRB, 0.66582E+00_JPRB ,&
&  0.66429E+00_JPRB, 0.65529E+00_JPRB, 0.64499E+00_JPRB, 0.63282E+00_JPRB, 0.61541E+00_JPRB, 0.59967E+00_JPRB, 0.58766E+00_JPRB ,&
&  0.57786E+00_JPRB, 0.57964E+00_JPRB, 0.61263E+00_JPRB, 0.64536E+00_JPRB, 0.66754E+00_JPRB, 0.68813E+00_JPRB, 0.70452E+00_JPRB ,&
&  0.72149E+00_JPRB, 0.76927E+00_JPRB, 0.75657E+00_JPRB, 0.81382E+00_JPRB, 0.92367E+00_JPRB, 0.87158E+00_JPRB, 0.78265E+00_JPRB ,&
&  0.74516E+00_JPRB, 0.72370E+00_JPRB, 0.70983E+00_JPRB, 0.70392E+00_JPRB, 0.75118E+00_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.67889E+00_JPRB, 0.65123E+00_JPRB, 0.63784E+00_JPRB, 0.62968E+00_JPRB, 0.62244E+00_JPRB, 0.61546E+00_JPRB, 0.60162E+00_JPRB ,&
&  0.58310E+00_JPRB, 0.57168E+00_JPRB, 0.55884E+00_JPRB, 0.54184E+00_JPRB, 0.51808E+00_JPRB, 0.50253E+00_JPRB, 0.50040E+00_JPRB ,&
&  0.49869E+00_JPRB, 0.49424E+00_JPRB, 0.48602E+00_JPRB, 0.48240E+00_JPRB, 0.47739E+00_JPRB, 0.47039E+00_JPRB, 0.46583E+00_JPRB ,&
&  0.45577E+00_JPRB, 0.45104E+00_JPRB, 0.45298E+00_JPRB, 0.45871E+00_JPRB, 0.46388E+00_JPRB, 0.46973E+00_JPRB, 0.47612E+00_JPRB ,&
&  0.54568E+00_JPRB, 0.84751E+00_JPRB, 0.12903E+01_JPRB, 0.15426E+01_JPRB, 0.15821E+01_JPRB, 0.14814E+01_JPRB, 0.14843E+01_JPRB ,&
&  0.13418E+01_JPRB, 0.12037E+01_JPRB, 0.10582E+01_JPRB, 0.92068E+00_JPRB, 0.81421E+00_JPRB, 0.75750E+00_JPRB, 0.72509E+00_JPRB ,&
&  0.68855E+00_JPRB, 0.65685E+00_JPRB, 0.62896E+00_JPRB, 0.60514E+00_JPRB, 0.59680E+00_JPRB, 0.60684E+00_JPRB, 0.62078E+00_JPRB ,&
&  0.62670E+00_JPRB, 0.63357E+00_JPRB, 0.65490E+00_JPRB, 0.73253E+00_JPRB, 0.92998E+00_JPRB, 0.10433E+01_JPRB, 0.14127E+01_JPRB ,&
&  0.16565E+01_JPRB, 0.15230E+01_JPRB, 0.13246E+01_JPRB, 0.11781E+01_JPRB, 0.10584E+01_JPRB, 0.96620E+00_JPRB, 0.89923E+00_JPRB ,&
&  0.86212E+00_JPRB, 0.83456E+00_JPRB, 0.81281E+00_JPRB, 0.79007E+00_JPRB, 0.76898E+00_JPRB, 0.75448E+00_JPRB, 0.74228E+00_JPRB ,&
&  0.72701E+00_JPRB, 0.70841E+00_JPRB, 0.68613E+00_JPRB, 0.65724E+00_JPRB, 0.62668E+00_JPRB, 0.60934E+00_JPRB, 0.60315E+00_JPRB ,&
&  0.60710E+00_JPRB, 0.63925E+00_JPRB, 0.68917E+00_JPRB, 0.69575E+00_JPRB, 0.73318E+00_JPRB, 0.76393E+00_JPRB, 0.79436E+00_JPRB ,&
&  0.78308E+00_JPRB, 0.80652E+00_JPRB, 0.81847E+00_JPRB, 0.88216E+00_JPRB, 0.10091E+01_JPRB, 0.95586E+00_JPRB, 0.83763E+00_JPRB ,&
&  0.79393E+00_JPRB, 0.77370E+00_JPRB, 0.75086E+00_JPRB, 0.72220E+00_JPRB, 0.71673E+00_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.66669E+00_JPRB, 0.64882E+00_JPRB, 0.63559E+00_JPRB, 0.62851E+00_JPRB, 0.62166E+00_JPRB, 0.60828E+00_JPRB, 0.59496E+00_JPRB ,&
&  0.58146E+00_JPRB, 0.56699E+00_JPRB, 0.54707E+00_JPRB, 0.51876E+00_JPRB, 0.49855E+00_JPRB, 0.49180E+00_JPRB, 0.49686E+00_JPRB ,&
&  0.49785E+00_JPRB, 0.48991E+00_JPRB, 0.48537E+00_JPRB, 0.48610E+00_JPRB, 0.48637E+00_JPRB, 0.48059E+00_JPRB, 0.47192E+00_JPRB ,&
&  0.46469E+00_JPRB, 0.46513E+00_JPRB, 0.46832E+00_JPRB, 0.47338E+00_JPRB, 0.47711E+00_JPRB, 0.48181E+00_JPRB, 0.50054E+00_JPRB ,&
&  0.59525E+00_JPRB, 0.61482E+00_JPRB, 0.13955E+01_JPRB, 0.18948E+01_JPRB, 0.19317E+01_JPRB, 0.18700E+01_JPRB, 0.18619E+01_JPRB ,&
&  0.17362E+01_JPRB, 0.15386E+01_JPRB, 0.12999E+01_JPRB, 0.11003E+01_JPRB, 0.95294E+00_JPRB, 0.85566E+00_JPRB, 0.78449E+00_JPRB ,&
&  0.73146E+00_JPRB, 0.69159E+00_JPRB, 0.66016E+00_JPRB, 0.64360E+00_JPRB, 0.64641E+00_JPRB, 0.65155E+00_JPRB, 0.65573E+00_JPRB ,&
&  0.66441E+00_JPRB, 0.68108E+00_JPRB, 0.72772E+00_JPRB, 0.85377E+00_JPRB, 0.10836E+01_JPRB, 0.13270E+01_JPRB, 0.17584E+01_JPRB ,&
&  0.21699E+01_JPRB, 0.20095E+01_JPRB, 0.16765E+01_JPRB, 0.14375E+01_JPRB, 0.12712E+01_JPRB, 0.11596E+01_JPRB, 0.10770E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.96699E+00_JPRB, 0.92934E+00_JPRB, 0.88160E+00_JPRB, 0.84520E+00_JPRB, 0.81940E+00_JPRB, 0.78801E+00_JPRB ,&
&  0.75192E+00_JPRB, 0.71696E+00_JPRB, 0.68305E+00_JPRB, 0.65298E+00_JPRB, 0.62896E+00_JPRB, 0.62690E+00_JPRB, 0.63593E+00_JPRB ,&
&  0.65347E+00_JPRB, 0.68690E+00_JPRB, 0.71018E+00_JPRB, 0.72832E+00_JPRB, 0.76491E+00_JPRB, 0.79245E+00_JPRB, 0.82151E+00_JPRB ,&
&  0.83036E+00_JPRB, 0.85353E+00_JPRB, 0.87719E+00_JPRB, 0.93245E+00_JPRB, 0.90590E+00_JPRB, 0.97364E+00_JPRB, 0.87043E+00_JPRB ,&
&  0.83827E+00_JPRB, 0.81073E+00_JPRB, 0.75538E+00_JPRB, 0.72558E+00_JPRB, 0.69732E+00_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.66217E+00_JPRB, 0.64310E+00_JPRB, 0.62816E+00_JPRB, 0.61450E+00_JPRB, 0.60434E+00_JPRB, 0.59765E+00_JPRB, 0.58917E+00_JPRB ,&
&  0.57617E+00_JPRB, 0.55886E+00_JPRB, 0.54059E+00_JPRB, 0.51615E+00_JPRB, 0.50356E+00_JPRB, 0.50303E+00_JPRB, 0.50996E+00_JPRB ,&
&  0.50263E+00_JPRB, 0.48906E+00_JPRB, 0.48977E+00_JPRB, 0.49278E+00_JPRB, 0.49106E+00_JPRB, 0.48426E+00_JPRB, 0.47708E+00_JPRB ,&
&  0.47993E+00_JPRB, 0.48373E+00_JPRB, 0.48685E+00_JPRB, 0.48965E+00_JPRB, 0.49288E+00_JPRB, 0.50326E+00_JPRB, 0.53536E+00_JPRB ,&
&  0.64511E+00_JPRB, 0.11212E+01_JPRB, 0.11932E+01_JPRB, 0.22083E+01_JPRB, 0.23919E+01_JPRB, 0.23655E+01_JPRB, 0.21700E+01_JPRB ,&
&  0.19640E+01_JPRB, 0.18247E+01_JPRB, 0.15280E+01_JPRB, 0.13020E+01_JPRB, 0.10948E+01_JPRB, 0.94216E+00_JPRB, 0.84589E+00_JPRB ,&
&  0.78374E+00_JPRB, 0.73763E+00_JPRB, 0.71059E+00_JPRB, 0.70168E+00_JPRB, 0.69663E+00_JPRB, 0.69455E+00_JPRB, 0.69822E+00_JPRB ,&
&  0.71064E+00_JPRB, 0.74619E+00_JPRB, 0.82825E+00_JPRB, 0.98969E+00_JPRB, 0.10953E+01_JPRB, 0.14633E+01_JPRB, 0.19930E+01_JPRB ,&
&  0.22872E+01_JPRB, 0.22669E+01_JPRB, 0.19197E+01_JPRB, 0.16249E+01_JPRB, 0.14246E+01_JPRB, 0.12936E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.11070E+01_JPRB, 0.10491E+01_JPRB, 0.98514E+00_JPRB, 0.90609E+00_JPRB, 0.85613E+00_JPRB, 0.81866E+00_JPRB, 0.77252E+00_JPRB ,&
&  0.72446E+00_JPRB, 0.68957E+00_JPRB, 0.67031E+00_JPRB, 0.65679E+00_JPRB, 0.65169E+00_JPRB, 0.66142E+00_JPRB, 0.67812E+00_JPRB ,&
&  0.69971E+00_JPRB, 0.73472E+00_JPRB, 0.73360E+00_JPRB, 0.74509E+00_JPRB, 0.77802E+00_JPRB, 0.82035E+00_JPRB, 0.82886E+00_JPRB ,&
&  0.85635E+00_JPRB, 0.88056E+00_JPRB, 0.90941E+00_JPRB, 0.95239E+00_JPRB, 0.93331E+00_JPRB, 0.10324E+01_JPRB, 0.88421E+00_JPRB ,&
&  0.85685E+00_JPRB, 0.78870E+00_JPRB, 0.73348E+00_JPRB, 0.71279E+00_JPRB, 0.69004E+00_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.65110E+00_JPRB, 0.62245E+00_JPRB, 0.60374E+00_JPRB, 0.59230E+00_JPRB, 0.58648E+00_JPRB, 0.58475E+00_JPRB, 0.57703E+00_JPRB ,&
&  0.56834E+00_JPRB, 0.55070E+00_JPRB, 0.52850E+00_JPRB, 0.50951E+00_JPRB, 0.51275E+00_JPRB, 0.51356E+00_JPRB, 0.51273E+00_JPRB ,&
&  0.50357E+00_JPRB, 0.50001E+00_JPRB, 0.50185E+00_JPRB, 0.50103E+00_JPRB, 0.49728E+00_JPRB, 0.49366E+00_JPRB, 0.49304E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.49903E+00_JPRB, 0.49858E+00_JPRB, 0.50225E+00_JPRB, 0.51308E+00_JPRB, 0.54152E+00_JPRB, 0.58858E+00_JPRB ,&
&  0.72866E+00_JPRB, 0.96521E+00_JPRB, 0.11536E+01_JPRB, 0.23446E+01_JPRB, 0.27690E+01_JPRB, 0.27144E+01_JPRB, 0.24173E+01_JPRB ,&
&  0.21410E+01_JPRB, 0.21098E+01_JPRB, 0.17399E+01_JPRB, 0.14132E+01_JPRB, 0.11484E+01_JPRB, 0.99918E+00_JPRB, 0.89256E+00_JPRB ,&
&  0.82006E+00_JPRB, 0.78169E+00_JPRB, 0.76910E+00_JPRB, 0.76528E+00_JPRB, 0.76042E+00_JPRB, 0.76997E+00_JPRB, 0.78248E+00_JPRB ,&
&  0.79829E+00_JPRB, 0.84382E+00_JPRB, 0.92703E+00_JPRB, 0.98676E+00_JPRB, 0.12260E+01_JPRB, 0.18006E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.24481E+01_JPRB, 0.23508E+01_JPRB, 0.20503E+01_JPRB, 0.17150E+01_JPRB, 0.14571E+01_JPRB, 0.13569E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11302E+01_JPRB, 0.10520E+01_JPRB, 0.94756E+00_JPRB, 0.87805E+00_JPRB, 0.84591E+00_JPRB, 0.78320E+00_JPRB, 0.73175E+00_JPRB ,&
&  0.70428E+00_JPRB, 0.69244E+00_JPRB, 0.68648E+00_JPRB, 0.68074E+00_JPRB, 0.68517E+00_JPRB, 0.69936E+00_JPRB, 0.72237E+00_JPRB ,&
&  0.75190E+00_JPRB, 0.78917E+00_JPRB, 0.77284E+00_JPRB, 0.78305E+00_JPRB, 0.83025E+00_JPRB, 0.87860E+00_JPRB, 0.88643E+00_JPRB ,&
&  0.89265E+00_JPRB, 0.91096E+00_JPRB, 0.92948E+00_JPRB, 0.91303E+00_JPRB, 0.88465E+00_JPRB, 0.94533E+00_JPRB, 0.87672E+00_JPRB ,&
&  0.86213E+00_JPRB, 0.78222E+00_JPRB, 0.74325E+00_JPRB, 0.70257E+00_JPRB, 0.67892E+00_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.61865E+00_JPRB, 0.59168E+00_JPRB, 0.58624E+00_JPRB, 0.58174E+00_JPRB, 0.56909E+00_JPRB, 0.55889E+00_JPRB, 0.55059E+00_JPRB ,&
&  0.54001E+00_JPRB, 0.52182E+00_JPRB, 0.50711E+00_JPRB, 0.51143E+00_JPRB, 0.52698E+00_JPRB, 0.53856E+00_JPRB, 0.52942E+00_JPRB ,&
&  0.51817E+00_JPRB, 0.51337E+00_JPRB, 0.52122E+00_JPRB, 0.52473E+00_JPRB, 0.52236E+00_JPRB, 0.52535E+00_JPRB, 0.52508E+00_JPRB ,&
&  0.52491E+00_JPRB, 0.52165E+00_JPRB, 0.52430E+00_JPRB, 0.53825E+00_JPRB, 0.56399E+00_JPRB, 0.61212E+00_JPRB, 0.68695E+00_JPRB ,&
&  0.92608E+00_JPRB, 0.10650E+01_JPRB, 0.12894E+01_JPRB, 0.20489E+01_JPRB, 0.25755E+01_JPRB, 0.24570E+01_JPRB, 0.23149E+01_JPRB ,&
&  0.21727E+01_JPRB, 0.18492E+01_JPRB, 0.15943E+01_JPRB, 0.13874E+01_JPRB, 0.11723E+01_JPRB, 0.10112E+01_JPRB, 0.90993E+00_JPRB ,&
&  0.85620E+00_JPRB, 0.82337E+00_JPRB, 0.82098E+00_JPRB, 0.83920E+00_JPRB, 0.87052E+00_JPRB, 0.90653E+00_JPRB, 0.92720E+00_JPRB ,&
&  0.94343E+00_JPRB, 0.97983E+00_JPRB, 0.10608E+01_JPRB, 0.11566E+01_JPRB, 0.15908E+01_JPRB, 0.23255E+01_JPRB, 0.26695E+01_JPRB ,&
&  0.26211E+01_JPRB, 0.23684E+01_JPRB, 0.19711E+01_JPRB, 0.16409E+01_JPRB, 0.12998E+01_JPRB, 0.13133E+01_JPRB, 0.12026E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.96613E+00_JPRB, 0.89573E+00_JPRB, 0.84012E+00_JPRB, 0.78459E+00_JPRB, 0.73671E+00_JPRB, 0.71154E+00_JPRB ,&
&  0.70893E+00_JPRB, 0.71577E+00_JPRB, 0.71405E+00_JPRB, 0.70806E+00_JPRB, 0.72411E+00_JPRB, 0.74006E+00_JPRB, 0.76882E+00_JPRB ,&
&  0.79707E+00_JPRB, 0.82639E+00_JPRB, 0.82098E+00_JPRB, 0.85667E+00_JPRB, 0.90755E+00_JPRB, 0.95022E+00_JPRB, 0.94612E+00_JPRB ,&
&  0.92280E+00_JPRB, 0.97652E+00_JPRB, 0.97863E+00_JPRB, 0.95102E+00_JPRB, 0.92316E+00_JPRB, 0.90246E+00_JPRB, 0.85197E+00_JPRB ,&
&  0.85536E+00_JPRB, 0.82973E+00_JPRB, 0.77942E+00_JPRB, 0.68843E+00_JPRB, 0.64970E+00_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.59029E+00_JPRB, 0.57127E+00_JPRB, 0.57246E+00_JPRB, 0.56560E+00_JPRB, 0.55373E+00_JPRB, 0.54792E+00_JPRB, 0.54083E+00_JPRB ,&
&  0.53294E+00_JPRB, 0.53202E+00_JPRB, 0.53186E+00_JPRB, 0.56302E+00_JPRB, 0.57833E+00_JPRB, 0.56883E+00_JPRB, 0.55793E+00_JPRB ,&
&  0.55010E+00_JPRB, 0.55092E+00_JPRB, 0.55590E+00_JPRB, 0.55954E+00_JPRB, 0.56192E+00_JPRB, 0.57059E+00_JPRB, 0.57903E+00_JPRB ,&
&  0.57745E+00_JPRB, 0.58006E+00_JPRB, 0.59284E+00_JPRB, 0.62097E+00_JPRB, 0.67601E+00_JPRB, 0.77401E+00_JPRB, 0.94976E+00_JPRB ,&
&  0.13527E+01_JPRB, 0.13208E+01_JPRB, 0.65552E+00_JPRB, 0.12236E+01_JPRB, 0.19637E+01_JPRB, 0.20648E+01_JPRB, 0.19627E+01_JPRB ,&
&  0.17870E+01_JPRB, 0.14241E+01_JPRB, 0.12957E+01_JPRB, 0.12901E+01_JPRB, 0.11203E+01_JPRB, 0.99446E+00_JPRB, 0.91963E+00_JPRB ,&
&  0.88310E+00_JPRB, 0.89106E+00_JPRB, 0.94427E+00_JPRB, 0.10007E+01_JPRB, 0.10558E+01_JPRB, 0.10836E+01_JPRB, 0.10906E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.11649E+01_JPRB, 0.13017E+01_JPRB, 0.14677E+01_JPRB, 0.22683E+01_JPRB, 0.26783E+01_JPRB, 0.27660E+01_JPRB ,&
&  0.20929E+01_JPRB, 0.18730E+01_JPRB, 0.16554E+01_JPRB, 0.14479E+01_JPRB, 0.12045E+01_JPRB, 0.11046E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.96763E+00_JPRB, 0.85911E+00_JPRB, 0.81735E+00_JPRB, 0.78024E+00_JPRB, 0.74683E+00_JPRB, 0.73087E+00_JPRB, 0.72643E+00_JPRB ,&
&  0.74150E+00_JPRB, 0.75392E+00_JPRB, 0.75574E+00_JPRB, 0.75938E+00_JPRB, 0.77237E+00_JPRB, 0.78764E+00_JPRB, 0.81275E+00_JPRB ,&
&  0.82773E+00_JPRB, 0.85927E+00_JPRB, 0.90183E+00_JPRB, 0.91047E+00_JPRB, 0.93893E+00_JPRB, 0.99926E+00_JPRB, 0.10068E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.11275E+01_JPRB, 0.10033E+01_JPRB, 0.91519E+00_JPRB, 0.90700E+00_JPRB, 0.86975E+00_JPRB, 0.83955E+00_JPRB ,&
&  0.88130E+00_JPRB, 0.99933E+00_JPRB, 0.96209E+00_JPRB, 0.65620E+00_JPRB, 0.60541E+00_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.54344E+00_JPRB, 0.52527E+00_JPRB, 0.52285E+00_JPRB, 0.53159E+00_JPRB, 0.53809E+00_JPRB, 0.54485E+00_JPRB, 0.53124E+00_JPRB ,&
&  0.54653E+00_JPRB, 0.58493E+00_JPRB, 0.60978E+00_JPRB, 0.63178E+00_JPRB, 0.62661E+00_JPRB, 0.61464E+00_JPRB, 0.60851E+00_JPRB ,&
&  0.60781E+00_JPRB, 0.61297E+00_JPRB, 0.62945E+00_JPRB, 0.63773E+00_JPRB, 0.64461E+00_JPRB, 0.65675E+00_JPRB, 0.66505E+00_JPRB ,&
&  0.67413E+00_JPRB, 0.69087E+00_JPRB, 0.72494E+00_JPRB, 0.79341E+00_JPRB, 0.91848E+00_JPRB, 0.11417E+01_JPRB, 0.14570E+01_JPRB ,&
&  0.20154E+01_JPRB, 0.14683E+01_JPRB, 0.70284E+00_JPRB, 0.15305E+01_JPRB, 0.19317E+01_JPRB, 0.19380E+01_JPRB, 0.16625E+01_JPRB ,&
&  0.15493E+01_JPRB, 0.13417E+01_JPRB, 0.12026E+01_JPRB, 0.12528E+01_JPRB, 0.10913E+01_JPRB, 0.10260E+01_JPRB, 0.10091E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.10891E+01_JPRB, 0.11485E+01_JPRB, 0.11936E+01_JPRB, 0.12123E+01_JPRB, 0.12113E+01_JPRB, 0.12258E+01_JPRB ,&
&  0.13044E+01_JPRB, 0.14267E+01_JPRB, 0.16467E+01_JPRB, 0.19790E+01_JPRB, 0.27244E+01_JPRB, 0.25576E+01_JPRB, 0.25121E+01_JPRB ,&
&  0.16829E+01_JPRB, 0.14823E+01_JPRB, 0.13985E+01_JPRB, 0.12975E+01_JPRB, 0.10999E+01_JPRB, 0.96179E+00_JPRB, 0.97030E+00_JPRB ,&
&  0.85597E+00_JPRB, 0.78455E+00_JPRB, 0.76267E+00_JPRB, 0.73824E+00_JPRB, 0.72090E+00_JPRB, 0.72760E+00_JPRB, 0.75261E+00_JPRB ,&
&  0.78715E+00_JPRB, 0.79824E+00_JPRB, 0.80370E+00_JPRB, 0.80856E+00_JPRB, 0.81585E+00_JPRB, 0.82446E+00_JPRB, 0.82839E+00_JPRB ,&
&  0.84764E+00_JPRB, 0.88261E+00_JPRB, 0.92164E+00_JPRB, 0.93736E+00_JPRB, 0.93723E+00_JPRB, 0.95901E+00_JPRB, 0.10346E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10634E+01_JPRB, 0.95134E+00_JPRB, 0.92134E+00_JPRB, 0.85831E+00_JPRB, 0.82516E+00_JPRB, 0.80069E+00_JPRB ,&
&  0.77274E+00_JPRB, 0.70269E+00_JPRB, 0.62670E+00_JPRB, 0.55192E+00_JPRB, 0.54401E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.51531E+00_JPRB, 0.50683E+00_JPRB, 0.50623E+00_JPRB, 0.52474E+00_JPRB, 0.52109E+00_JPRB, 0.53488E+00_JPRB, 0.55458E+00_JPRB ,&
&  0.60481E+00_JPRB, 0.65082E+00_JPRB, 0.69151E+00_JPRB, 0.68789E+00_JPRB, 0.68674E+00_JPRB, 0.67400E+00_JPRB, 0.68042E+00_JPRB ,&
&  0.69590E+00_JPRB, 0.72011E+00_JPRB, 0.74641E+00_JPRB, 0.75935E+00_JPRB, 0.76991E+00_JPRB, 0.78253E+00_JPRB, 0.80070E+00_JPRB ,&
&  0.82873E+00_JPRB, 0.87022E+00_JPRB, 0.94882E+00_JPRB, 0.10947E+01_JPRB, 0.13288E+01_JPRB, 0.15730E+01_JPRB, 0.17521E+01_JPRB ,&
&  0.12177E+01_JPRB, 0.54317E+00_JPRB, 0.14422E+01_JPRB, 0.18675E+01_JPRB, 0.18195E+01_JPRB, 0.17364E+01_JPRB, 0.16541E+01_JPRB ,&
&  0.14430E+01_JPRB, 0.13193E+01_JPRB, 0.11788E+01_JPRB, 0.12626E+01_JPRB, 0.11472E+01_JPRB, 0.11376E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.12169E+01_JPRB, 0.12640E+01_JPRB, 0.13092E+01_JPRB, 0.13227E+01_JPRB, 0.13196E+01_JPRB, 0.13400E+01_JPRB, 0.14005E+01_JPRB ,&
&  0.15211E+01_JPRB, 0.17148E+01_JPRB, 0.19533E+01_JPRB, 0.20090E+01_JPRB, 0.22836E+01_JPRB, 0.21962E+01_JPRB, 0.23401E+01_JPRB ,&
&  0.16449E+01_JPRB, 0.12137E+01_JPRB, 0.11538E+01_JPRB, 0.11777E+01_JPRB, 0.10287E+01_JPRB, 0.86576E+00_JPRB, 0.86229E+00_JPRB ,&
&  0.77803E+00_JPRB, 0.73917E+00_JPRB, 0.73341E+00_JPRB, 0.71830E+00_JPRB, 0.70334E+00_JPRB, 0.72665E+00_JPRB, 0.76032E+00_JPRB ,&
&  0.79712E+00_JPRB, 0.81608E+00_JPRB, 0.81551E+00_JPRB, 0.81751E+00_JPRB, 0.82084E+00_JPRB, 0.81711E+00_JPRB, 0.81423E+00_JPRB ,&
&  0.83885E+00_JPRB, 0.86828E+00_JPRB, 0.90306E+00_JPRB, 0.94014E+00_JPRB, 0.98396E+00_JPRB, 0.98831E+00_JPRB, 0.98828E+00_JPRB ,&
&  0.97658E+00_JPRB, 0.95777E+00_JPRB, 0.88347E+00_JPRB, 0.86634E+00_JPRB, 0.82967E+00_JPRB, 0.78607E+00_JPRB, 0.75053E+00_JPRB ,&
&  0.69347E+00_JPRB, 0.60653E+00_JPRB, 0.55234E+00_JPRB, 0.53772E+00_JPRB, 0.52237E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.53631E+00_JPRB, 0.52923E+00_JPRB, 0.53541E+00_JPRB, 0.54527E+00_JPRB, 0.56238E+00_JPRB, 0.59186E+00_JPRB, 0.63677E+00_JPRB ,&
&  0.68830E+00_JPRB, 0.71778E+00_JPRB, 0.74311E+00_JPRB, 0.73527E+00_JPRB, 0.73911E+00_JPRB, 0.75802E+00_JPRB, 0.77142E+00_JPRB ,&
&  0.80496E+00_JPRB, 0.83971E+00_JPRB, 0.86052E+00_JPRB, 0.86746E+00_JPRB, 0.88129E+00_JPRB, 0.91703E+00_JPRB, 0.96778E+00_JPRB ,&
&  0.10317E+01_JPRB, 0.11208E+01_JPRB, 0.12437E+01_JPRB, 0.14106E+01_JPRB, 0.15961E+01_JPRB, 0.16699E+01_JPRB, 0.16776E+01_JPRB ,&
&  0.85267E+00_JPRB, 0.14175E+01_JPRB, 0.18651E+01_JPRB, 0.18278E+01_JPRB, 0.17321E+01_JPRB, 0.16670E+01_JPRB, 0.16284E+01_JPRB ,&
&  0.14610E+01_JPRB, 0.12948E+01_JPRB, 0.12336E+01_JPRB, 0.13268E+01_JPRB, 0.12407E+01_JPRB, 0.12649E+01_JPRB, 0.13024E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.13907E+01_JPRB, 0.14096E+01_JPRB, 0.14205E+01_JPRB, 0.14568E+01_JPRB, 0.15251E+01_JPRB, 0.16163E+01_JPRB ,&
&  0.17353E+01_JPRB, 0.18876E+01_JPRB, 0.19983E+01_JPRB, 0.18734E+01_JPRB, 0.18410E+01_JPRB, 0.19821E+01_JPRB, 0.23871E+01_JPRB ,&
&  0.15855E+01_JPRB, 0.10559E+01_JPRB, 0.10542E+01_JPRB, 0.10765E+01_JPRB, 0.93747E+00_JPRB, 0.84816E+00_JPRB, 0.78984E+00_JPRB ,&
&  0.73125E+00_JPRB, 0.69251E+00_JPRB, 0.67549E+00_JPRB, 0.66755E+00_JPRB, 0.68338E+00_JPRB, 0.70731E+00_JPRB, 0.74460E+00_JPRB ,&
&  0.76410E+00_JPRB, 0.76742E+00_JPRB, 0.76117E+00_JPRB, 0.76086E+00_JPRB, 0.76788E+00_JPRB, 0.77816E+00_JPRB, 0.77886E+00_JPRB ,&
&  0.80209E+00_JPRB, 0.80713E+00_JPRB, 0.82814E+00_JPRB, 0.87329E+00_JPRB, 0.93177E+00_JPRB, 0.95221E+00_JPRB, 0.94689E+00_JPRB ,&
&  0.89190E+00_JPRB, 0.88417E+00_JPRB, 0.86925E+00_JPRB, 0.84832E+00_JPRB, 0.81445E+00_JPRB, 0.75829E+00_JPRB, 0.66083E+00_JPRB ,&
&  0.61512E+00_JPRB, 0.56933E+00_JPRB, 0.54669E+00_JPRB, 0.53709E+00_JPRB, 0.54752E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.60972E+00_JPRB, 0.59831E+00_JPRB, 0.60733E+00_JPRB, 0.62703E+00_JPRB, 0.64446E+00_JPRB, 0.67799E+00_JPRB, 0.71953E+00_JPRB ,&
&  0.77188E+00_JPRB, 0.80789E+00_JPRB, 0.82237E+00_JPRB, 0.82688E+00_JPRB, 0.83812E+00_JPRB, 0.84572E+00_JPRB, 0.86355E+00_JPRB ,&
&  0.89508E+00_JPRB, 0.92748E+00_JPRB, 0.96909E+00_JPRB, 0.10045E+01_JPRB, 0.10361E+01_JPRB, 0.11021E+01_JPRB, 0.11774E+01_JPRB ,&
&  0.12704E+01_JPRB, 0.13701E+01_JPRB, 0.14895E+01_JPRB, 0.16161E+01_JPRB, 0.17495E+01_JPRB, 0.18081E+01_JPRB, 0.18251E+01_JPRB ,&
&  0.13763E+01_JPRB, 0.19116E+01_JPRB, 0.19385E+01_JPRB, 0.17579E+01_JPRB, 0.16019E+01_JPRB, 0.15653E+01_JPRB, 0.15262E+01_JPRB ,&
&  0.14807E+01_JPRB, 0.13649E+01_JPRB, 0.13150E+01_JPRB, 0.12864E+01_JPRB, 0.13349E+01_JPRB, 0.13490E+01_JPRB, 0.14033E+01_JPRB ,&
&  0.14570E+01_JPRB, 0.14958E+01_JPRB, 0.15227E+01_JPRB, 0.15492E+01_JPRB, 0.15938E+01_JPRB, 0.16647E+01_JPRB, 0.17634E+01_JPRB ,&
&  0.18404E+01_JPRB, 0.19092E+01_JPRB, 0.19635E+01_JPRB, 0.18273E+01_JPRB, 0.17087E+01_JPRB, 0.18718E+01_JPRB, 0.18078E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.92947E+00_JPRB, 0.95679E+00_JPRB, 0.97756E+00_JPRB, 0.87390E+00_JPRB, 0.79211E+00_JPRB, 0.73426E+00_JPRB ,&
&  0.67635E+00_JPRB, 0.62988E+00_JPRB, 0.63775E+00_JPRB, 0.64633E+00_JPRB, 0.65368E+00_JPRB, 0.65856E+00_JPRB, 0.67787E+00_JPRB ,&
&  0.70228E+00_JPRB, 0.69408E+00_JPRB, 0.68762E+00_JPRB, 0.68564E+00_JPRB, 0.70523E+00_JPRB, 0.73472E+00_JPRB, 0.76529E+00_JPRB ,&
&  0.82396E+00_JPRB, 0.77991E+00_JPRB, 0.79087E+00_JPRB, 0.83229E+00_JPRB, 0.87388E+00_JPRB, 0.92276E+00_JPRB, 0.96857E+00_JPRB ,&
&  0.10005E+01_JPRB, 0.10365E+01_JPRB, 0.10706E+01_JPRB, 0.10676E+01_JPRB, 0.98394E+00_JPRB, 0.89828E+00_JPRB, 0.65897E+00_JPRB ,&
&  0.64580E+00_JPRB, 0.63127E+00_JPRB, 0.63300E+00_JPRB, 0.63138E+00_JPRB, 0.62986E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.74186E+00_JPRB, 0.74153E+00_JPRB, 0.76915E+00_JPRB, 0.79867E+00_JPRB, 0.82509E+00_JPRB, 0.85096E+00_JPRB, 0.87090E+00_JPRB ,&
&  0.89899E+00_JPRB, 0.92299E+00_JPRB, 0.93098E+00_JPRB, 0.93065E+00_JPRB, 0.94640E+00_JPRB, 0.95368E+00_JPRB, 0.98137E+00_JPRB ,&
&  0.10227E+01_JPRB, 0.10696E+01_JPRB, 0.11317E+01_JPRB, 0.11741E+01_JPRB, 0.12160E+01_JPRB, 0.12794E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.14096E+01_JPRB, 0.14968E+01_JPRB, 0.15725E+01_JPRB, 0.16535E+01_JPRB, 0.17352E+01_JPRB, 0.17336E+01_JPRB, 0.13230E+01_JPRB ,&
&  0.18476E+01_JPRB, 0.20724E+01_JPRB, 0.19578E+01_JPRB, 0.16963E+01_JPRB, 0.15452E+01_JPRB, 0.14895E+01_JPRB, 0.14839E+01_JPRB ,&
&  0.14683E+01_JPRB, 0.14010E+01_JPRB, 0.13225E+01_JPRB, 0.12900E+01_JPRB, 0.14336E+01_JPRB, 0.14074E+01_JPRB, 0.14688E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.15883E+01_JPRB, 0.16318E+01_JPRB, 0.16667E+01_JPRB, 0.16914E+01_JPRB, 0.17268E+01_JPRB, 0.17641E+01_JPRB ,&
&  0.17972E+01_JPRB, 0.18179E+01_JPRB, 0.18694E+01_JPRB, 0.16941E+01_JPRB, 0.16213E+01_JPRB, 0.17253E+01_JPRB, 0.15708E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.89162E+00_JPRB, 0.90380E+00_JPRB, 0.90198E+00_JPRB, 0.84030E+00_JPRB, 0.75666E+00_JPRB, 0.71094E+00_JPRB ,&
&  0.67680E+00_JPRB, 0.65044E+00_JPRB, 0.64551E+00_JPRB, 0.63356E+00_JPRB, 0.63223E+00_JPRB, 0.64552E+00_JPRB, 0.65156E+00_JPRB ,&
&  0.68099E+00_JPRB, 0.67206E+00_JPRB, 0.67251E+00_JPRB, 0.68298E+00_JPRB, 0.73328E+00_JPRB, 0.84038E+00_JPRB, 0.10497E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.74740E+00_JPRB, 0.77864E+00_JPRB, 0.80861E+00_JPRB, 0.87295E+00_JPRB, 0.99583E+00_JPRB, 0.11907E+01_JPRB ,&
&  0.15306E+01_JPRB, 0.20037E+01_JPRB, 0.19297E+01_JPRB, 0.15304E+01_JPRB, 0.19241E+01_JPRB, 0.16205E+01_JPRB, 0.74120E+00_JPRB ,&
&  0.74634E+00_JPRB, 0.72606E+00_JPRB, 0.73369E+00_JPRB, 0.74199E+00_JPRB, 0.74544E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.85422E+00_JPRB, 0.87643E+00_JPRB, 0.90216E+00_JPRB, 0.92547E+00_JPRB, 0.94141E+00_JPRB, 0.95628E+00_JPRB, 0.97464E+00_JPRB ,&
&  0.98549E+00_JPRB, 0.99104E+00_JPRB, 0.10072E+01_JPRB, 0.10319E+01_JPRB, 0.10612E+01_JPRB, 0.10936E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.12235E+01_JPRB, 0.12801E+01_JPRB, 0.13226E+01_JPRB, 0.13509E+01_JPRB, 0.13812E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.14578E+01_JPRB, 0.14896E+01_JPRB, 0.15194E+01_JPRB, 0.15787E+01_JPRB, 0.16284E+01_JPRB, 0.16950E+01_JPRB, 0.14148E+01_JPRB ,&
&  0.17668E+01_JPRB, 0.20154E+01_JPRB, 0.18397E+01_JPRB, 0.15931E+01_JPRB, 0.14703E+01_JPRB, 0.14523E+01_JPRB, 0.14471E+01_JPRB ,&
&  0.14525E+01_JPRB, 0.14377E+01_JPRB, 0.13371E+01_JPRB, 0.13726E+01_JPRB, 0.14026E+01_JPRB, 0.13935E+01_JPRB, 0.14515E+01_JPRB ,&
&  0.15138E+01_JPRB, 0.15661E+01_JPRB, 0.16218E+01_JPRB, 0.16751E+01_JPRB, 0.17107E+01_JPRB, 0.17552E+01_JPRB, 0.17417E+01_JPRB ,&
&  0.17274E+01_JPRB, 0.17194E+01_JPRB, 0.17061E+01_JPRB, 0.15453E+01_JPRB, 0.14790E+01_JPRB, 0.13957E+01_JPRB, 0.11543E+01_JPRB ,&
&  0.97900E+00_JPRB, 0.87876E+00_JPRB, 0.85383E+00_JPRB, 0.92751E+00_JPRB, 0.89796E+00_JPRB, 0.82191E+00_JPRB, 0.76275E+00_JPRB ,&
&  0.71307E+00_JPRB, 0.67585E+00_JPRB, 0.69454E+00_JPRB, 0.68078E+00_JPRB, 0.62097E+00_JPRB, 0.63586E+00_JPRB, 0.65856E+00_JPRB ,&
&  0.65220E+00_JPRB, 0.67350E+00_JPRB, 0.70561E+00_JPRB, 0.71247E+00_JPRB, 0.74010E+00_JPRB, 0.81371E+00_JPRB, 0.84996E+00_JPRB ,&
&  0.90583E+00_JPRB, 0.77512E+00_JPRB, 0.76926E+00_JPRB, 0.80774E+00_JPRB, 0.77434E+00_JPRB, 0.99211E+00_JPRB, 0.13409E+01_JPRB ,&
&  0.19333E+01_JPRB, 0.20820E+01_JPRB, 0.26947E+01_JPRB, 0.41849E+01_JPRB, 0.31853E+01_JPRB, 0.21273E+01_JPRB, 0.83854E+00_JPRB ,&
&  0.84097E+00_JPRB, 0.83527E+00_JPRB, 0.85150E+00_JPRB, 0.84967E+00_JPRB, 0.85401E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.87249E+00_JPRB, 0.90102E+00_JPRB, 0.92451E+00_JPRB, 0.94825E+00_JPRB, 0.97120E+00_JPRB, 0.99338E+00_JPRB, 0.10171E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10583E+01_JPRB, 0.10867E+01_JPRB, 0.11159E+01_JPRB, 0.11417E+01_JPRB, 0.11741E+01_JPRB, 0.12016E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.12642E+01_JPRB, 0.12906E+01_JPRB, 0.12954E+01_JPRB, 0.12982E+01_JPRB, 0.13154E+01_JPRB, 0.13291E+01_JPRB ,&
&  0.13281E+01_JPRB, 0.13366E+01_JPRB, 0.13527E+01_JPRB, 0.13804E+01_JPRB, 0.13926E+01_JPRB, 0.14640E+01_JPRB, 0.13136E+01_JPRB ,&
&  0.16321E+01_JPRB, 0.17845E+01_JPRB, 0.16657E+01_JPRB, 0.14748E+01_JPRB, 0.13705E+01_JPRB, 0.13573E+01_JPRB, 0.13611E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.14215E+01_JPRB, 0.13259E+01_JPRB, 0.12754E+01_JPRB, 0.12580E+01_JPRB, 0.12638E+01_JPRB, 0.12998E+01_JPRB ,&
&  0.13446E+01_JPRB, 0.13905E+01_JPRB, 0.14407E+01_JPRB, 0.14878E+01_JPRB, 0.15212E+01_JPRB, 0.15302E+01_JPRB, 0.15016E+01_JPRB ,&
&  0.14459E+01_JPRB, 0.13881E+01_JPRB, 0.12592E+01_JPRB, 0.12194E+01_JPRB, 0.11545E+01_JPRB, 0.97241E+00_JPRB, 0.79369E+00_JPRB ,&
&  0.75251E+00_JPRB, 0.82155E+00_JPRB, 0.89739E+00_JPRB, 0.10653E+01_JPRB, 0.10427E+01_JPRB, 0.97491E+00_JPRB, 0.88608E+00_JPRB ,&
&  0.80601E+00_JPRB, 0.74527E+00_JPRB, 0.74450E+00_JPRB, 0.71486E+00_JPRB, 0.69026E+00_JPRB, 0.67684E+00_JPRB, 0.66505E+00_JPRB ,&
&  0.67921E+00_JPRB, 0.69860E+00_JPRB, 0.72465E+00_JPRB, 0.74543E+00_JPRB, 0.75559E+00_JPRB, 0.71573E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.85379E+00_JPRB, 0.73016E+00_JPRB, 0.84942E+00_JPRB, 0.58700E+00_JPRB, 0.68391E+00_JPRB, 0.74730E+00_JPRB, 0.87506E+00_JPRB ,&
&  0.96103E+00_JPRB, 0.13323E+01_JPRB, 0.18191E+01_JPRB, 0.14568E+01_JPRB, 0.11784E+01_JPRB, 0.90297E+00_JPRB, 0.81374E+00_JPRB ,&
&  0.81307E+00_JPRB, 0.81749E+00_JPRB, 0.82832E+00_JPRB, 0.83308E+00_JPRB, 0.84664E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.88567E+00_JPRB, 0.91104E+00_JPRB, 0.93066E+00_JPRB, 0.95415E+00_JPRB, 0.97913E+00_JPRB, 0.10022E+01_JPRB, 0.10235E+01_JPRB ,&
&  0.10415E+01_JPRB, 0.10567E+01_JPRB, 0.10708E+01_JPRB, 0.10883E+01_JPRB, 0.11091E+01_JPRB, 0.11255E+01_JPRB, 0.11472E+01_JPRB ,&
&  0.11800E+01_JPRB, 0.12064E+01_JPRB, 0.12361E+01_JPRB, 0.12347E+01_JPRB, 0.12353E+01_JPRB, 0.12599E+01_JPRB, 0.12830E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.13082E+01_JPRB, 0.13353E+01_JPRB, 0.13668E+01_JPRB, 0.14066E+01_JPRB, 0.14888E+01_JPRB, 0.14306E+01_JPRB ,&
&  0.15071E+01_JPRB, 0.16367E+01_JPRB, 0.15389E+01_JPRB, 0.13474E+01_JPRB, 0.12580E+01_JPRB, 0.12134E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.11980E+01_JPRB, 0.11576E+01_JPRB, 0.11295E+01_JPRB, 0.10961E+01_JPRB, 0.10811E+01_JPRB, 0.10879E+01_JPRB, 0.10957E+01_JPRB ,&
&  0.11145E+01_JPRB, 0.11373E+01_JPRB, 0.11768E+01_JPRB, 0.12284E+01_JPRB, 0.12780E+01_JPRB, 0.12470E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11102E+01_JPRB, 0.10412E+01_JPRB, 0.84366E+00_JPRB, 0.77228E+00_JPRB, 0.71748E+00_JPRB, 0.63020E+00_JPRB, 0.55244E+00_JPRB ,&
&  0.64495E+00_JPRB, 0.75062E+00_JPRB, 0.94332E+00_JPRB, 0.11842E+01_JPRB, 0.11522E+01_JPRB, 0.10990E+01_JPRB, 0.98927E+00_JPRB ,&
&  0.88826E+00_JPRB, 0.82432E+00_JPRB, 0.80533E+00_JPRB, 0.76987E+00_JPRB, 0.76333E+00_JPRB, 0.77269E+00_JPRB, 0.72314E+00_JPRB ,&
&  0.71183E+00_JPRB, 0.71553E+00_JPRB, 0.72719E+00_JPRB, 0.74594E+00_JPRB, 0.81715E+00_JPRB, 0.85058E+00_JPRB, 0.12779E+01_JPRB ,&
&  0.92349E+00_JPRB, 0.62525E+00_JPRB, 0.64171E+00_JPRB, 0.68986E+00_JPRB, 0.66084E+00_JPRB, 0.64915E+00_JPRB, 0.72359E+00_JPRB ,&
&  0.87495E+00_JPRB, 0.99218E+00_JPRB, 0.10173E+01_JPRB, 0.96346E+00_JPRB, 0.87309E+00_JPRB, 0.82161E+00_JPRB, 0.79542E+00_JPRB ,&
&  0.80815E+00_JPRB, 0.82017E+00_JPRB, 0.83536E+00_JPRB, 0.84239E+00_JPRB, 0.85791E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.87575E+00_JPRB, 0.90302E+00_JPRB, 0.91922E+00_JPRB, 0.93595E+00_JPRB, 0.95772E+00_JPRB, 0.97073E+00_JPRB, 0.98147E+00_JPRB ,&
&  0.98567E+00_JPRB, 0.98333E+00_JPRB, 0.99180E+00_JPRB, 0.10061E+01_JPRB, 0.10130E+01_JPRB, 0.10103E+01_JPRB, 0.10172E+01_JPRB ,&
&  0.10335E+01_JPRB, 0.10587E+01_JPRB, 0.10862E+01_JPRB, 0.10803E+01_JPRB, 0.10762E+01_JPRB, 0.10916E+01_JPRB, 0.11143E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11127E+01_JPRB, 0.11219E+01_JPRB, 0.11333E+01_JPRB, 0.11538E+01_JPRB, 0.11583E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.15726E+01_JPRB, 0.14596E+01_JPRB, 0.12413E+01_JPRB, 0.12075E+01_JPRB, 0.11038E+01_JPRB, 0.10682E+01_JPRB ,&
&  0.10454E+01_JPRB, 0.10036E+01_JPRB, 0.96203E+00_JPRB, 0.93112E+00_JPRB, 0.92854E+00_JPRB, 0.93497E+00_JPRB, 0.91016E+00_JPRB ,&
&  0.91411E+00_JPRB, 0.93244E+00_JPRB, 0.98231E+00_JPRB, 0.10461E+01_JPRB, 0.10907E+01_JPRB, 0.10821E+01_JPRB, 0.10005E+01_JPRB ,&
&  0.94737E+00_JPRB, 0.87592E+00_JPRB, 0.72032E+00_JPRB, 0.69158E+00_JPRB, 0.65720E+00_JPRB, 0.56103E+00_JPRB, 0.51968E+00_JPRB ,&
&  0.65850E+00_JPRB, 0.82387E+00_JPRB, 0.11379E+01_JPRB, 0.12864E+01_JPRB, 0.12915E+01_JPRB, 0.13268E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.10149E+01_JPRB, 0.93023E+00_JPRB, 0.92202E+00_JPRB, 0.88169E+00_JPRB, 0.87597E+00_JPRB, 0.85902E+00_JPRB, 0.80217E+00_JPRB ,&
&  0.76097E+00_JPRB, 0.74192E+00_JPRB, 0.74175E+00_JPRB, 0.76606E+00_JPRB, 0.75916E+00_JPRB, 0.12318E+01_JPRB, 0.20895E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.77363E+00_JPRB, 0.56657E+00_JPRB, 0.75114E+00_JPRB, 0.65123E+00_JPRB, 0.60257E+00_JPRB, 0.65351E+00_JPRB ,&
&  0.76472E+00_JPRB, 0.81035E+00_JPRB, 0.81752E+00_JPRB, 0.80969E+00_JPRB, 0.78828E+00_JPRB, 0.76914E+00_JPRB, 0.76150E+00_JPRB ,&
&  0.77904E+00_JPRB, 0.79658E+00_JPRB, 0.81691E+00_JPRB, 0.82960E+00_JPRB, 0.84798E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.85172E+00_JPRB, 0.86938E+00_JPRB, 0.87105E+00_JPRB, 0.88831E+00_JPRB, 0.91127E+00_JPRB, 0.92034E+00_JPRB, 0.92541E+00_JPRB ,&
&  0.91722E+00_JPRB, 0.91971E+00_JPRB, 0.91531E+00_JPRB, 0.93212E+00_JPRB, 0.93365E+00_JPRB, 0.93491E+00_JPRB, 0.93789E+00_JPRB ,&
&  0.95391E+00_JPRB, 0.96549E+00_JPRB, 0.99643E+00_JPRB, 0.10007E+01_JPRB, 0.99699E+00_JPRB, 0.10031E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.10816E+01_JPRB, 0.11225E+01_JPRB, 0.11566E+01_JPRB, 0.12730E+01_JPRB, 0.14338E+01_JPRB, 0.17109E+01_JPRB ,&
&  0.15247E+01_JPRB, 0.14852E+01_JPRB, 0.13573E+01_JPRB, 0.11558E+01_JPRB, 0.11300E+01_JPRB, 0.11228E+01_JPRB, 0.10576E+01_JPRB ,&
&  0.99541E+00_JPRB, 0.95786E+00_JPRB, 0.92358E+00_JPRB, 0.90901E+00_JPRB, 0.91121E+00_JPRB, 0.89657E+00_JPRB, 0.88354E+00_JPRB ,&
&  0.84371E+00_JPRB, 0.84855E+00_JPRB, 0.10168E+01_JPRB, 0.12240E+01_JPRB, 0.13791E+01_JPRB, 0.15257E+01_JPRB, 0.13890E+01_JPRB ,&
&  0.13310E+01_JPRB, 0.13554E+01_JPRB, 0.12321E+01_JPRB, 0.11194E+01_JPRB, 0.10030E+01_JPRB, 0.91218E+00_JPRB, 0.87373E+00_JPRB ,&
&  0.96606E+00_JPRB, 0.11314E+01_JPRB, 0.11649E+01_JPRB, 0.13969E+01_JPRB, 0.15186E+01_JPRB, 0.15376E+01_JPRB, 0.13476E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.10625E+01_JPRB, 0.10429E+01_JPRB, 0.10164E+01_JPRB, 0.10074E+01_JPRB, 0.94613E+00_JPRB, 0.86668E+00_JPRB ,&
&  0.83567E+00_JPRB, 0.78314E+00_JPRB, 0.76331E+00_JPRB, 0.72510E+00_JPRB, 0.85715E+00_JPRB, 0.92367E+00_JPRB, 0.12682E+01_JPRB ,&
&  0.94177E+00_JPRB, 0.87609E+00_JPRB, 0.82220E+00_JPRB, 0.13854E+01_JPRB, 0.85206E+00_JPRB, 0.59587E+00_JPRB, 0.58792E+00_JPRB ,&
&  0.64991E+00_JPRB, 0.68870E+00_JPRB, 0.70726E+00_JPRB, 0.73755E+00_JPRB, 0.74816E+00_JPRB, 0.74506E+00_JPRB, 0.75626E+00_JPRB ,&
&  0.79463E+00_JPRB, 0.80291E+00_JPRB, 0.81117E+00_JPRB, 0.81275E+00_JPRB, 0.83123E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.79344E+00_JPRB, 0.80689E+00_JPRB, 0.80088E+00_JPRB, 0.80840E+00_JPRB, 0.81413E+00_JPRB, 0.83077E+00_JPRB, 0.83160E+00_JPRB ,&
&  0.82846E+00_JPRB, 0.83391E+00_JPRB, 0.81374E+00_JPRB, 0.81843E+00_JPRB, 0.81486E+00_JPRB, 0.81784E+00_JPRB, 0.80628E+00_JPRB ,&
&  0.81062E+00_JPRB, 0.82687E+00_JPRB, 0.83222E+00_JPRB, 0.83732E+00_JPRB, 0.83130E+00_JPRB, 0.81146E+00_JPRB, 0.79525E+00_JPRB ,&
&  0.84590E+00_JPRB, 0.96695E+00_JPRB, 0.11094E+01_JPRB, 0.12044E+01_JPRB, 0.14362E+01_JPRB, 0.14469E+01_JPRB, 0.15645E+01_JPRB ,&
&  0.16498E+01_JPRB, 0.14751E+01_JPRB, 0.14976E+01_JPRB, 0.12930E+01_JPRB, 0.12623E+01_JPRB, 0.12227E+01_JPRB, 0.12056E+01_JPRB ,&
&  0.12358E+01_JPRB, 0.13014E+01_JPRB, 0.13787E+01_JPRB, 0.14659E+01_JPRB, 0.15376E+01_JPRB, 0.15535E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15203E+01_JPRB, 0.15963E+01_JPRB, 0.18709E+01_JPRB, 0.20411E+01_JPRB, 0.22484E+01_JPRB, 0.23461E+01_JPRB, 0.23468E+01_JPRB ,&
&  0.22942E+01_JPRB, 0.22636E+01_JPRB, 0.19520E+01_JPRB, 0.17186E+01_JPRB, 0.16640E+01_JPRB, 0.15233E+01_JPRB, 0.14917E+01_JPRB ,&
&  0.15733E+01_JPRB, 0.15931E+01_JPRB, 0.13365E+01_JPRB, 0.14870E+01_JPRB, 0.16440E+01_JPRB, 0.15732E+01_JPRB, 0.16824E+01_JPRB ,&
&  0.14551E+01_JPRB, 0.13192E+01_JPRB, 0.12601E+01_JPRB, 0.12253E+01_JPRB, 0.11703E+01_JPRB, 0.10855E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.90564E+00_JPRB, 0.80103E+00_JPRB, 0.75346E+00_JPRB, 0.74597E+00_JPRB, 0.80827E+00_JPRB, 0.97962E+00_JPRB, 0.91103E+00_JPRB ,&
&  0.89949E+00_JPRB, 0.88194E+00_JPRB, 0.12678E+01_JPRB, 0.20490E+01_JPRB, 0.98978E+00_JPRB, 0.70216E+00_JPRB, 0.56242E+00_JPRB ,&
&  0.55649E+00_JPRB, 0.58349E+00_JPRB, 0.60992E+00_JPRB, 0.64506E+00_JPRB, 0.66423E+00_JPRB, 0.66954E+00_JPRB, 0.68267E+00_JPRB ,&
&  0.71244E+00_JPRB, 0.73974E+00_JPRB, 0.77836E+00_JPRB, 0.78229E+00_JPRB, 0.77719E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.69699E+00_JPRB, 0.69642E+00_JPRB, 0.70036E+00_JPRB, 0.69505E+00_JPRB, 0.68967E+00_JPRB, 0.68931E+00_JPRB, 0.69296E+00_JPRB ,&
&  0.69492E+00_JPRB, 0.68595E+00_JPRB, 0.67597E+00_JPRB, 0.66914E+00_JPRB, 0.65883E+00_JPRB, 0.62943E+00_JPRB, 0.58942E+00_JPRB ,&
&  0.60393E+00_JPRB, 0.62429E+00_JPRB, 0.64297E+00_JPRB, 0.68513E+00_JPRB, 0.78449E+00_JPRB, 0.93667E+00_JPRB, 0.11498E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.19990E+01_JPRB, 0.22996E+01_JPRB, 0.23444E+01_JPRB, 0.25792E+01_JPRB, 0.19213E+01_JPRB, 0.18260E+01_JPRB ,&
&  0.20040E+01_JPRB, 0.17043E+01_JPRB, 0.15412E+01_JPRB, 0.14395E+01_JPRB, 0.14214E+01_JPRB, 0.14615E+01_JPRB, 0.15733E+01_JPRB ,&
&  0.17518E+01_JPRB, 0.19356E+01_JPRB, 0.20898E+01_JPRB, 0.22384E+01_JPRB, 0.23678E+01_JPRB, 0.24914E+01_JPRB, 0.25701E+01_JPRB ,&
&  0.25725E+01_JPRB, 0.25841E+01_JPRB, 0.26344E+01_JPRB, 0.26264E+01_JPRB, 0.28385E+01_JPRB, 0.29481E+01_JPRB, 0.29790E+01_JPRB ,&
&  0.28863E+01_JPRB, 0.27430E+01_JPRB, 0.26762E+01_JPRB, 0.24985E+01_JPRB, 0.23442E+01_JPRB, 0.22378E+01_JPRB, 0.21183E+01_JPRB ,&
&  0.20132E+01_JPRB, 0.18980E+01_JPRB, 0.15944E+01_JPRB, 0.18133E+01_JPRB, 0.19160E+01_JPRB, 0.16723E+01_JPRB, 0.18651E+01_JPRB ,&
&  0.17005E+01_JPRB, 0.16353E+01_JPRB, 0.15479E+01_JPRB, 0.14663E+01_JPRB, 0.13974E+01_JPRB, 0.15274E+01_JPRB, 0.13956E+01_JPRB ,&
&  0.10953E+01_JPRB, 0.85591E+00_JPRB, 0.74839E+00_JPRB, 0.76089E+00_JPRB, 0.83033E+00_JPRB, 0.91800E+00_JPRB, 0.95280E+00_JPRB ,&
&  0.89304E+00_JPRB, 0.86674E+00_JPRB, 0.10475E+01_JPRB, 0.14784E+01_JPRB, 0.10440E+01_JPRB, 0.66247E+00_JPRB, 0.56474E+00_JPRB ,&
&  0.53424E+00_JPRB, 0.53058E+00_JPRB, 0.53794E+00_JPRB, 0.56385E+00_JPRB, 0.58765E+00_JPRB, 0.60962E+00_JPRB, 0.62753E+00_JPRB ,&
&  0.63600E+00_JPRB, 0.65458E+00_JPRB, 0.67091E+00_JPRB, 0.68633E+00_JPRB, 0.69798E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.62423E+00_JPRB, 0.61646E+00_JPRB, 0.62185E+00_JPRB, 0.62109E+00_JPRB, 0.63447E+00_JPRB, 0.63729E+00_JPRB, 0.64809E+00_JPRB ,&
&  0.66864E+00_JPRB, 0.69592E+00_JPRB, 0.69379E+00_JPRB, 0.67963E+00_JPRB, 0.68943E+00_JPRB, 0.71824E+00_JPRB, 0.78114E+00_JPRB ,&
&  0.91119E+00_JPRB, 0.10501E+01_JPRB, 0.11477E+01_JPRB, 0.12758E+01_JPRB, 0.15201E+01_JPRB, 0.19520E+01_JPRB, 0.23761E+01_JPRB ,&
&  0.27105E+01_JPRB, 0.29251E+01_JPRB, 0.28487E+01_JPRB, 0.28419E+01_JPRB, 0.21952E+01_JPRB, 0.16597E+01_JPRB, 0.18385E+01_JPRB ,&
&  0.19827E+01_JPRB, 0.19190E+01_JPRB, 0.16544E+01_JPRB, 0.16081E+01_JPRB, 0.16029E+01_JPRB, 0.16867E+01_JPRB, 0.18713E+01_JPRB ,&
&  0.20597E+01_JPRB, 0.22629E+01_JPRB, 0.24259E+01_JPRB, 0.25486E+01_JPRB, 0.26338E+01_JPRB, 0.27314E+01_JPRB, 0.28232E+01_JPRB ,&
&  0.29273E+01_JPRB, 0.29640E+01_JPRB, 0.29820E+01_JPRB, 0.30642E+01_JPRB, 0.31709E+01_JPRB, 0.33034E+01_JPRB, 0.33737E+01_JPRB ,&
&  0.33833E+01_JPRB, 0.31922E+01_JPRB, 0.31231E+01_JPRB, 0.30409E+01_JPRB, 0.27847E+01_JPRB, 0.25232E+01_JPRB, 0.24299E+01_JPRB ,&
&  0.23466E+01_JPRB, 0.22870E+01_JPRB, 0.18468E+01_JPRB, 0.22379E+01_JPRB, 0.24969E+01_JPRB, 0.22223E+01_JPRB, 0.20851E+01_JPRB ,&
&  0.19029E+01_JPRB, 0.17877E+01_JPRB, 0.16467E+01_JPRB, 0.16375E+01_JPRB, 0.17012E+01_JPRB, 0.19824E+01_JPRB, 0.15289E+01_JPRB ,&
&  0.14028E+01_JPRB, 0.10435E+01_JPRB, 0.98753E+00_JPRB, 0.10861E+01_JPRB, 0.11895E+01_JPRB, 0.11470E+01_JPRB, 0.98929E+00_JPRB ,&
&  0.88704E+00_JPRB, 0.10296E+01_JPRB, 0.12078E+01_JPRB, 0.18169E+01_JPRB, 0.21001E+01_JPRB, 0.69803E+00_JPRB, 0.59404E+00_JPRB ,&
&  0.57703E+00_JPRB, 0.57477E+00_JPRB, 0.57310E+00_JPRB, 0.58912E+00_JPRB, 0.59263E+00_JPRB, 0.60253E+00_JPRB, 0.62343E+00_JPRB ,&
&  0.63571E+00_JPRB, 0.63751E+00_JPRB, 0.63655E+00_JPRB, 0.62021E+00_JPRB, 0.62525E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.62763E+00_JPRB, 0.63056E+00_JPRB, 0.64979E+00_JPRB, 0.69859E+00_JPRB, 0.77890E+00_JPRB, 0.82740E+00_JPRB, 0.84391E+00_JPRB ,&
&  0.83381E+00_JPRB, 0.85641E+00_JPRB, 0.91493E+00_JPRB, 0.96555E+00_JPRB, 0.10679E+01_JPRB, 0.12169E+01_JPRB, 0.13652E+01_JPRB ,&
&  0.15565E+01_JPRB, 0.17343E+01_JPRB, 0.18765E+01_JPRB, 0.20836E+01_JPRB, 0.23008E+01_JPRB, 0.25846E+01_JPRB, 0.27912E+01_JPRB ,&
&  0.28005E+01_JPRB, 0.24537E+01_JPRB, 0.22571E+01_JPRB, 0.17046E+01_JPRB, 0.15016E+01_JPRB, 0.17550E+01_JPRB, 0.16612E+01_JPRB ,&
&  0.16388E+01_JPRB, 0.16730E+01_JPRB, 0.16758E+01_JPRB, 0.17287E+01_JPRB, 0.18203E+01_JPRB, 0.19311E+01_JPRB, 0.20870E+01_JPRB ,&
&  0.22692E+01_JPRB, 0.24341E+01_JPRB, 0.25814E+01_JPRB, 0.27092E+01_JPRB, 0.27836E+01_JPRB, 0.28827E+01_JPRB, 0.30383E+01_JPRB ,&
&  0.32161E+01_JPRB, 0.33064E+01_JPRB, 0.33803E+01_JPRB, 0.35001E+01_JPRB, 0.34328E+01_JPRB, 0.34874E+01_JPRB, 0.34980E+01_JPRB ,&
&  0.35139E+01_JPRB, 0.34466E+01_JPRB, 0.34063E+01_JPRB, 0.34181E+01_JPRB, 0.32759E+01_JPRB, 0.28128E+01_JPRB, 0.26895E+01_JPRB ,&
&  0.26933E+01_JPRB, 0.25504E+01_JPRB, 0.23110E+01_JPRB, 0.26355E+01_JPRB, 0.18406E+01_JPRB, 0.21208E+01_JPRB, 0.23475E+01_JPRB ,&
&  0.22647E+01_JPRB, 0.21505E+01_JPRB, 0.19615E+01_JPRB, 0.19063E+01_JPRB, 0.18548E+01_JPRB, 0.16329E+01_JPRB, 0.15911E+01_JPRB ,&
&  0.16176E+01_JPRB, 0.14296E+01_JPRB, 0.13991E+01_JPRB, 0.15261E+01_JPRB, 0.16714E+01_JPRB, 0.15795E+01_JPRB, 0.14337E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.12114E+01_JPRB, 0.11757E+01_JPRB, 0.14374E+01_JPRB, 0.11033E+01_JPRB, 0.66651E+00_JPRB, 0.63873E+00_JPRB ,&
&  0.62951E+00_JPRB, 0.63973E+00_JPRB, 0.64064E+00_JPRB, 0.67611E+00_JPRB, 0.68980E+00_JPRB, 0.68058E+00_JPRB, 0.68622E+00_JPRB ,&
&  0.70158E+00_JPRB, 0.69815E+00_JPRB, 0.69118E+00_JPRB, 0.67251E+00_JPRB, 0.65669E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.76736E+00_JPRB, 0.74045E+00_JPRB, 0.74321E+00_JPRB, 0.84176E+00_JPRB, 0.95457E+00_JPRB, 0.10238E+01_JPRB, 0.10114E+01_JPRB ,&
&  0.98698E+00_JPRB, 0.97168E+00_JPRB, 0.10657E+01_JPRB, 0.11888E+01_JPRB, 0.13504E+01_JPRB, 0.15560E+01_JPRB, 0.17709E+01_JPRB ,&
&  0.20141E+01_JPRB, 0.22122E+01_JPRB, 0.23548E+01_JPRB, 0.25072E+01_JPRB, 0.27063E+01_JPRB, 0.29282E+01_JPRB, 0.30518E+01_JPRB ,&
&  0.24903E+01_JPRB, 0.17476E+01_JPRB, 0.21115E+01_JPRB, 0.21265E+01_JPRB, 0.21723E+01_JPRB, 0.22246E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.18539E+01_JPRB, 0.18011E+01_JPRB, 0.18825E+01_JPRB, 0.19438E+01_JPRB, 0.19139E+01_JPRB, 0.20359E+01_JPRB, 0.21852E+01_JPRB ,&
&  0.23548E+01_JPRB, 0.24873E+01_JPRB, 0.25758E+01_JPRB, 0.26699E+01_JPRB, 0.27456E+01_JPRB, 0.28429E+01_JPRB, 0.30015E+01_JPRB ,&
&  0.31696E+01_JPRB, 0.33167E+01_JPRB, 0.33808E+01_JPRB, 0.33269E+01_JPRB, 0.33059E+01_JPRB, 0.33574E+01_JPRB, 0.33059E+01_JPRB ,&
&  0.32917E+01_JPRB, 0.32567E+01_JPRB, 0.34547E+01_JPRB, 0.36910E+01_JPRB, 0.37515E+01_JPRB, 0.32380E+01_JPRB, 0.32463E+01_JPRB ,&
&  0.31017E+01_JPRB, 0.29217E+01_JPRB, 0.28024E+01_JPRB, 0.30139E+01_JPRB, 0.22098E+01_JPRB, 0.23440E+01_JPRB, 0.23360E+01_JPRB ,&
&  0.25507E+01_JPRB, 0.24868E+01_JPRB, 0.23733E+01_JPRB, 0.22311E+01_JPRB, 0.22423E+01_JPRB, 0.18713E+01_JPRB, 0.17545E+01_JPRB ,&
&  0.17740E+01_JPRB, 0.19187E+01_JPRB, 0.18479E+01_JPRB, 0.18945E+01_JPRB, 0.18376E+01_JPRB, 0.19913E+01_JPRB, 0.20213E+01_JPRB ,&
&  0.21023E+01_JPRB, 0.15937E+01_JPRB, 0.13534E+01_JPRB, 0.11581E+01_JPRB, 0.80005E+00_JPRB, 0.68464E+00_JPRB, 0.68848E+00_JPRB ,&
&  0.69326E+00_JPRB, 0.71436E+00_JPRB, 0.72362E+00_JPRB, 0.75789E+00_JPRB, 0.78629E+00_JPRB, 0.78128E+00_JPRB, 0.76513E+00_JPRB ,&
&  0.76920E+00_JPRB, 0.79530E+00_JPRB, 0.78917E+00_JPRB, 0.78617E+00_JPRB, 0.78110E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.90661E+00_JPRB, 0.89147E+00_JPRB, 0.88728E+00_JPRB, 0.97301E+00_JPRB, 0.11122E+01_JPRB, 0.12381E+01_JPRB, 0.12659E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.11345E+01_JPRB, 0.12366E+01_JPRB, 0.13324E+01_JPRB, 0.15174E+01_JPRB, 0.17326E+01_JPRB, 0.19689E+01_JPRB ,&
&  0.21990E+01_JPRB, 0.24304E+01_JPRB, 0.26221E+01_JPRB, 0.28200E+01_JPRB, 0.30489E+01_JPRB, 0.32784E+01_JPRB, 0.28925E+01_JPRB ,&
&  0.21770E+01_JPRB, 0.20666E+01_JPRB, 0.27640E+01_JPRB, 0.24632E+01_JPRB, 0.22167E+01_JPRB, 0.22203E+01_JPRB, 0.21745E+01_JPRB ,&
&  0.20793E+01_JPRB, 0.19017E+01_JPRB, 0.19173E+01_JPRB, 0.19723E+01_JPRB, 0.19410E+01_JPRB, 0.20627E+01_JPRB, 0.22303E+01_JPRB ,&
&  0.23456E+01_JPRB, 0.24601E+01_JPRB, 0.25187E+01_JPRB, 0.25609E+01_JPRB, 0.26219E+01_JPRB, 0.27157E+01_JPRB, 0.28511E+01_JPRB ,&
&  0.30146E+01_JPRB, 0.31750E+01_JPRB, 0.32498E+01_JPRB, 0.31131E+01_JPRB, 0.30899E+01_JPRB, 0.31769E+01_JPRB, 0.32037E+01_JPRB ,&
&  0.31614E+01_JPRB, 0.32781E+01_JPRB, 0.33850E+01_JPRB, 0.34126E+01_JPRB, 0.32370E+01_JPRB, 0.36696E+01_JPRB, 0.34409E+01_JPRB ,&
&  0.34167E+01_JPRB, 0.32107E+01_JPRB, 0.34420E+01_JPRB, 0.26240E+01_JPRB, 0.26832E+01_JPRB, 0.27382E+01_JPRB, 0.26285E+01_JPRB ,&
&  0.25341E+01_JPRB, 0.26103E+01_JPRB, 0.25512E+01_JPRB, 0.24678E+01_JPRB, 0.24270E+01_JPRB, 0.20864E+01_JPRB, 0.20480E+01_JPRB ,&
&  0.19951E+01_JPRB, 0.22729E+01_JPRB, 0.20077E+01_JPRB, 0.19364E+01_JPRB, 0.18188E+01_JPRB, 0.20305E+01_JPRB, 0.23492E+01_JPRB ,&
&  0.26405E+01_JPRB, 0.24102E+01_JPRB, 0.21084E+01_JPRB, 0.16895E+01_JPRB, 0.12234E+01_JPRB, 0.93606E+00_JPRB, 0.79401E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.76670E+00_JPRB, 0.80612E+00_JPRB, 0.83036E+00_JPRB, 0.86307E+00_JPRB, 0.89592E+00_JPRB, 0.90964E+00_JPRB ,&
&  0.91584E+00_JPRB, 0.90536E+00_JPRB, 0.88755E+00_JPRB, 0.91800E+00_JPRB, 0.92956E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.99338E+00_JPRB, 0.95406E+00_JPRB, 0.92515E+00_JPRB, 0.95485E+00_JPRB, 0.10381E+01_JPRB, 0.10926E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.11286E+01_JPRB, 0.12044E+01_JPRB, 0.12734E+01_JPRB, 0.14610E+01_JPRB, 0.16708E+01_JPRB, 0.18592E+01_JPRB ,&
&  0.20275E+01_JPRB, 0.21651E+01_JPRB, 0.23476E+01_JPRB, 0.26019E+01_JPRB, 0.28493E+01_JPRB, 0.30540E+01_JPRB, 0.23760E+01_JPRB ,&
&  0.16570E+01_JPRB, 0.29462E+01_JPRB, 0.25696E+01_JPRB, 0.26119E+01_JPRB, 0.23948E+01_JPRB, 0.23570E+01_JPRB, 0.20982E+01_JPRB ,&
&  0.19410E+01_JPRB, 0.18025E+01_JPRB, 0.18177E+01_JPRB, 0.18511E+01_JPRB, 0.18516E+01_JPRB, 0.19280E+01_JPRB, 0.20422E+01_JPRB ,&
&  0.21521E+01_JPRB, 0.22727E+01_JPRB, 0.23513E+01_JPRB, 0.24121E+01_JPRB, 0.24935E+01_JPRB, 0.25625E+01_JPRB, 0.26949E+01_JPRB ,&
&  0.28696E+01_JPRB, 0.30366E+01_JPRB, 0.31157E+01_JPRB, 0.31723E+01_JPRB, 0.30670E+01_JPRB, 0.31970E+01_JPRB, 0.33854E+01_JPRB ,&
&  0.32846E+01_JPRB, 0.31465E+01_JPRB, 0.34698E+01_JPRB, 0.35438E+01_JPRB, 0.35056E+01_JPRB, 0.37885E+01_JPRB, 0.34777E+01_JPRB ,&
&  0.36898E+01_JPRB, 0.33501E+01_JPRB, 0.37449E+01_JPRB, 0.28856E+01_JPRB, 0.28833E+01_JPRB, 0.30773E+01_JPRB, 0.28618E+01_JPRB ,&
&  0.26136E+01_JPRB, 0.27222E+01_JPRB, 0.26251E+01_JPRB, 0.26472E+01_JPRB, 0.25928E+01_JPRB, 0.23565E+01_JPRB, 0.21345E+01_JPRB ,&
&  0.22534E+01_JPRB, 0.24428E+01_JPRB, 0.23548E+01_JPRB, 0.18448E+01_JPRB, 0.18232E+01_JPRB, 0.20118E+01_JPRB, 0.28236E+01_JPRB ,&
&  0.32617E+01_JPRB, 0.30633E+01_JPRB, 0.29283E+01_JPRB, 0.27068E+01_JPRB, 0.20832E+01_JPRB, 0.16770E+01_JPRB, 0.12709E+01_JPRB ,&
&  0.95042E+00_JPRB, 0.81953E+00_JPRB, 0.88705E+00_JPRB, 0.87009E+00_JPRB, 0.90669E+00_JPRB, 0.95464E+00_JPRB, 0.97887E+00_JPRB ,&
&  0.10112E+01_JPRB, 0.10244E+01_JPRB, 0.10333E+01_JPRB, 0.10207E+01_JPRB, 0.10162E+01_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.10075E+01_JPRB, 0.95794E+00_JPRB, 0.93045E+00_JPRB, 0.93763E+00_JPRB, 0.10216E+01_JPRB, 0.10776E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10693E+01_JPRB, 0.11075E+01_JPRB, 0.11103E+01_JPRB, 0.12114E+01_JPRB, 0.14127E+01_JPRB, 0.15797E+01_JPRB ,&
&  0.17314E+01_JPRB, 0.18870E+01_JPRB, 0.20835E+01_JPRB, 0.23872E+01_JPRB, 0.27647E+01_JPRB, 0.32322E+01_JPRB, 0.19991E+01_JPRB ,&
&  0.19342E+01_JPRB, 0.25018E+01_JPRB, 0.25392E+01_JPRB, 0.26761E+01_JPRB, 0.25156E+01_JPRB, 0.23987E+01_JPRB, 0.20935E+01_JPRB ,&
&  0.19649E+01_JPRB, 0.17775E+01_JPRB, 0.16464E+01_JPRB, 0.16044E+01_JPRB, 0.15833E+01_JPRB, 0.16816E+01_JPRB, 0.17288E+01_JPRB ,&
&  0.17275E+01_JPRB, 0.17978E+01_JPRB, 0.19456E+01_JPRB, 0.20893E+01_JPRB, 0.22297E+01_JPRB, 0.23646E+01_JPRB, 0.25332E+01_JPRB ,&
&  0.27432E+01_JPRB, 0.29342E+01_JPRB, 0.32012E+01_JPRB, 0.31804E+01_JPRB, 0.32395E+01_JPRB, 0.33671E+01_JPRB, 0.36354E+01_JPRB ,&
&  0.35607E+01_JPRB, 0.34381E+01_JPRB, 0.39207E+01_JPRB, 0.44333E+01_JPRB, 0.45402E+01_JPRB, 0.41563E+01_JPRB, 0.37265E+01_JPRB ,&
&  0.36880E+01_JPRB, 0.35177E+01_JPRB, 0.35221E+01_JPRB, 0.31541E+01_JPRB, 0.32455E+01_JPRB, 0.35194E+01_JPRB, 0.35257E+01_JPRB ,&
&  0.32963E+01_JPRB, 0.30278E+01_JPRB, 0.28712E+01_JPRB, 0.28480E+01_JPRB, 0.27482E+01_JPRB, 0.24951E+01_JPRB, 0.24505E+01_JPRB ,&
&  0.28711E+01_JPRB, 0.34744E+01_JPRB, 0.31414E+01_JPRB, 0.22545E+01_JPRB, 0.19380E+01_JPRB, 0.18058E+01_JPRB, 0.22825E+01_JPRB ,&
&  0.36169E+01_JPRB, 0.34616E+01_JPRB, 0.32058E+01_JPRB, 0.35109E+01_JPRB, 0.29790E+01_JPRB, 0.25276E+01_JPRB, 0.20074E+01_JPRB ,&
&  0.15359E+01_JPRB, 0.11940E+01_JPRB, 0.10651E+01_JPRB, 0.96534E+00_JPRB, 0.95240E+00_JPRB, 0.97189E+00_JPRB, 0.10072E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10590E+01_JPRB, 0.10532E+01_JPRB, 0.10389E+01_JPRB, 0.10409E+01_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.10515E+01_JPRB, 0.99399E+00_JPRB, 0.97131E+00_JPRB, 0.95774E+00_JPRB, 0.10084E+01_JPRB, 0.10536E+01_JPRB, 0.10169E+01_JPRB ,&
&  0.10203E+01_JPRB, 0.10051E+01_JPRB, 0.10080E+01_JPRB, 0.10069E+01_JPRB, 0.10567E+01_JPRB, 0.11612E+01_JPRB, 0.12599E+01_JPRB ,&
&  0.13949E+01_JPRB, 0.15470E+01_JPRB, 0.17492E+01_JPRB, 0.20356E+01_JPRB, 0.22874E+01_JPRB, 0.22262E+01_JPRB, 0.22684E+01_JPRB ,&
&  0.24210E+01_JPRB, 0.26077E+01_JPRB, 0.26409E+01_JPRB, 0.29175E+01_JPRB, 0.29618E+01_JPRB, 0.30845E+01_JPRB, 0.25843E+01_JPRB ,&
&  0.23664E+01_JPRB, 0.20819E+01_JPRB, 0.17828E+01_JPRB, 0.16023E+01_JPRB, 0.14563E+01_JPRB, 0.14454E+01_JPRB, 0.15089E+01_JPRB ,&
&  0.14879E+01_JPRB, 0.15345E+01_JPRB, 0.16258E+01_JPRB, 0.17293E+01_JPRB, 0.18945E+01_JPRB, 0.20694E+01_JPRB, 0.23072E+01_JPRB ,&
&  0.26054E+01_JPRB, 0.28762E+01_JPRB, 0.31535E+01_JPRB, 0.33094E+01_JPRB, 0.34064E+01_JPRB, 0.35787E+01_JPRB, 0.39712E+01_JPRB ,&
&  0.41724E+01_JPRB, 0.43677E+01_JPRB, 0.45170E+01_JPRB, 0.48229E+01_JPRB, 0.51611E+01_JPRB, 0.50291E+01_JPRB, 0.44043E+01_JPRB ,&
&  0.39516E+01_JPRB, 0.34881E+01_JPRB, 0.30911E+01_JPRB, 0.32181E+01_JPRB, 0.34857E+01_JPRB, 0.40458E+01_JPRB, 0.37314E+01_JPRB ,&
&  0.30450E+01_JPRB, 0.25720E+01_JPRB, 0.25135E+01_JPRB, 0.28098E+01_JPRB, 0.30482E+01_JPRB, 0.29592E+01_JPRB, 0.30973E+01_JPRB ,&
&  0.29816E+01_JPRB, 0.17515E+01_JPRB, 0.13692E+01_JPRB, 0.18813E+01_JPRB, 0.13683E+01_JPRB, 0.12621E+01_JPRB, 0.23209E+01_JPRB ,&
&  0.35546E+01_JPRB, 0.42754E+01_JPRB, 0.40149E+01_JPRB, 0.35151E+01_JPRB, 0.35793E+01_JPRB, 0.33422E+01_JPRB, 0.27447E+01_JPRB ,&
&  0.20713E+01_JPRB, 0.16216E+01_JPRB, 0.13719E+01_JPRB, 0.12040E+01_JPRB, 0.10731E+01_JPRB, 0.10299E+01_JPRB, 0.10270E+01_JPRB ,&
&  0.10049E+01_JPRB, 0.10205E+01_JPRB, 0.10260E+01_JPRB, 0.10260E+01_JPRB, 0.10483E+01_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.10492E+01_JPRB, 0.10223E+01_JPRB, 0.10037E+01_JPRB, 0.99538E+00_JPRB, 0.97355E+00_JPRB, 0.10084E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.95360E+00_JPRB, 0.91674E+00_JPRB, 0.92933E+00_JPRB, 0.98262E+00_JPRB, 0.10426E+01_JPRB, 0.10917E+01_JPRB ,&
&  0.11626E+01_JPRB, 0.12687E+01_JPRB, 0.14694E+01_JPRB, 0.17150E+01_JPRB, 0.18714E+01_JPRB, 0.18991E+01_JPRB, 0.23065E+01_JPRB ,&
&  0.24882E+01_JPRB, 0.27103E+01_JPRB, 0.31886E+01_JPRB, 0.33251E+01_JPRB, 0.32459E+01_JPRB, 0.32190E+01_JPRB, 0.28589E+01_JPRB ,&
&  0.25829E+01_JPRB, 0.23391E+01_JPRB, 0.20704E+01_JPRB, 0.18470E+01_JPRB, 0.17056E+01_JPRB, 0.15569E+01_JPRB, 0.15007E+01_JPRB ,&
&  0.14459E+01_JPRB, 0.14362E+01_JPRB, 0.14974E+01_JPRB, 0.15507E+01_JPRB, 0.16956E+01_JPRB, 0.18534E+01_JPRB, 0.21111E+01_JPRB ,&
&  0.24660E+01_JPRB, 0.28101E+01_JPRB, 0.31437E+01_JPRB, 0.36268E+01_JPRB, 0.30410E+01_JPRB, 0.37180E+01_JPRB, 0.39750E+01_JPRB ,&
&  0.43198E+01_JPRB, 0.45094E+01_JPRB, 0.44375E+01_JPRB, 0.54652E+01_JPRB, 0.60767E+01_JPRB, 0.56127E+01_JPRB, 0.49713E+01_JPRB ,&
&  0.44686E+01_JPRB, 0.37126E+01_JPRB, 0.33271E+01_JPRB, 0.34945E+01_JPRB, 0.37012E+01_JPRB, 0.42012E+01_JPRB, 0.29663E+01_JPRB ,&
&  0.26883E+01_JPRB, 0.31345E+01_JPRB, 0.29145E+01_JPRB, 0.24464E+01_JPRB, 0.28907E+01_JPRB, 0.34077E+01_JPRB, 0.23285E+01_JPRB ,&
&  0.10213E+01_JPRB, 0.84168E+00_JPRB, 0.91270E+00_JPRB, 0.91966E+00_JPRB, 0.84047E+00_JPRB, 0.12253E+01_JPRB, 0.44770E+01_JPRB ,&
&  0.35120E+01_JPRB, 0.47170E+01_JPRB, 0.40746E+01_JPRB, 0.37189E+01_JPRB, 0.35540E+01_JPRB, 0.36798E+01_JPRB, 0.33107E+01_JPRB ,&
&  0.24498E+01_JPRB, 0.20102E+01_JPRB, 0.16347E+01_JPRB, 0.13877E+01_JPRB, 0.11935E+01_JPRB, 0.10893E+01_JPRB, 0.10456E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.10003E+01_JPRB, 0.99707E+00_JPRB, 0.10024E+01_JPRB, 0.10259E+01_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.10701E+01_JPRB, 0.10443E+01_JPRB, 0.10194E+01_JPRB, 0.10060E+01_JPRB, 0.97942E+00_JPRB, 0.98136E+00_JPRB, 0.10378E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.96866E+00_JPRB, 0.92100E+00_JPRB, 0.91169E+00_JPRB, 0.95195E+00_JPRB, 0.10026E+01_JPRB, 0.10214E+01_JPRB ,&
&  0.10697E+01_JPRB, 0.11399E+01_JPRB, 0.14025E+01_JPRB, 0.14164E+01_JPRB, 0.15116E+01_JPRB, 0.15099E+01_JPRB, 0.19151E+01_JPRB ,&
&  0.24066E+01_JPRB, 0.28324E+01_JPRB, 0.32465E+01_JPRB, 0.33989E+01_JPRB, 0.37812E+01_JPRB, 0.38516E+01_JPRB, 0.37160E+01_JPRB ,&
&  0.31920E+01_JPRB, 0.27697E+01_JPRB, 0.24249E+01_JPRB, 0.21663E+01_JPRB, 0.19640E+01_JPRB, 0.17359E+01_JPRB, 0.16000E+01_JPRB ,&
&  0.14785E+01_JPRB, 0.14382E+01_JPRB, 0.14126E+01_JPRB, 0.14300E+01_JPRB, 0.15171E+01_JPRB, 0.16664E+01_JPRB, 0.19167E+01_JPRB ,&
&  0.22743E+01_JPRB, 0.26997E+01_JPRB, 0.30684E+01_JPRB, 0.36488E+01_JPRB, 0.32824E+01_JPRB, 0.29484E+01_JPRB, 0.35932E+01_JPRB ,&
&  0.42339E+01_JPRB, 0.48064E+01_JPRB, 0.50066E+01_JPRB, 0.65602E+01_JPRB, 0.66670E+01_JPRB, 0.60784E+01_JPRB, 0.58427E+01_JPRB ,&
&  0.49518E+01_JPRB, 0.46620E+01_JPRB, 0.37226E+01_JPRB, 0.37553E+01_JPRB, 0.40712E+01_JPRB, 0.32841E+01_JPRB, 0.31789E+01_JPRB ,&
&  0.36586E+01_JPRB, 0.33941E+01_JPRB, 0.29699E+01_JPRB, 0.23733E+01_JPRB, 0.29010E+01_JPRB, 0.26718E+01_JPRB, 0.12577E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.98501E+00_JPRB, 0.95266E+00_JPRB, 0.90183E+00_JPRB, 0.93252E+00_JPRB, 0.13471E+01_JPRB, 0.27390E+01_JPRB ,&
&  0.28643E+01_JPRB, 0.43062E+01_JPRB, 0.41050E+01_JPRB, 0.38661E+01_JPRB, 0.33954E+01_JPRB, 0.37252E+01_JPRB, 0.30614E+01_JPRB ,&
&  0.26054E+01_JPRB, 0.22287E+01_JPRB, 0.17710E+01_JPRB, 0.15198E+01_JPRB, 0.13031E+01_JPRB, 0.11358E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.10548E+01_JPRB, 0.10250E+01_JPRB, 0.10186E+01_JPRB, 0.10268E+01_JPRB, 0.10405E+01_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.11050E+01_JPRB, 0.10800E+01_JPRB, 0.10569E+01_JPRB, 0.10810E+01_JPRB, 0.10790E+01_JPRB, 0.10528E+01_JPRB, 0.10710E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10088E+01_JPRB, 0.96790E+00_JPRB, 0.95393E+00_JPRB, 0.94842E+00_JPRB, 0.98707E+00_JPRB, 0.10087E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.11334E+01_JPRB, 0.12647E+01_JPRB, 0.12158E+01_JPRB, 0.11844E+01_JPRB, 0.12084E+01_JPRB, 0.14793E+01_JPRB ,&
&  0.22143E+01_JPRB, 0.28469E+01_JPRB, 0.32750E+01_JPRB, 0.36956E+01_JPRB, 0.40979E+01_JPRB, 0.40456E+01_JPRB, 0.41693E+01_JPRB ,&
&  0.37448E+01_JPRB, 0.32100E+01_JPRB, 0.28176E+01_JPRB, 0.24805E+01_JPRB, 0.22990E+01_JPRB, 0.21277E+01_JPRB, 0.19433E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.15431E+01_JPRB, 0.14656E+01_JPRB, 0.13941E+01_JPRB, 0.13876E+01_JPRB, 0.14935E+01_JPRB, 0.16971E+01_JPRB ,&
&  0.20442E+01_JPRB, 0.24947E+01_JPRB, 0.28369E+01_JPRB, 0.34175E+01_JPRB, 0.39019E+01_JPRB, 0.39216E+01_JPRB, 0.37851E+01_JPRB ,&
&  0.37982E+01_JPRB, 0.39808E+01_JPRB, 0.57795E+01_JPRB, 0.66827E+01_JPRB, 0.67900E+01_JPRB, 0.67328E+01_JPRB, 0.63274E+01_JPRB ,&
&  0.52759E+01_JPRB, 0.47759E+01_JPRB, 0.40373E+01_JPRB, 0.42190E+01_JPRB, 0.37815E+01_JPRB, 0.30770E+01_JPRB, 0.37910E+01_JPRB ,&
&  0.41876E+01_JPRB, 0.38208E+01_JPRB, 0.31060E+01_JPRB, 0.25237E+01_JPRB, 0.21889E+01_JPRB, 0.16471E+01_JPRB, 0.11836E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.10736E+01_JPRB, 0.99198E+00_JPRB, 0.10354E+01_JPRB, 0.10285E+01_JPRB, 0.13999E+01_JPRB, 0.20129E+01_JPRB ,&
&  0.29236E+01_JPRB, 0.35192E+01_JPRB, 0.38857E+01_JPRB, 0.37216E+01_JPRB, 0.32849E+01_JPRB, 0.34248E+01_JPRB, 0.33252E+01_JPRB ,&
&  0.26576E+01_JPRB, 0.23674E+01_JPRB, 0.20081E+01_JPRB, 0.16589E+01_JPRB, 0.14324E+01_JPRB, 0.12500E+01_JPRB, 0.11260E+01_JPRB ,&
&  0.10715E+01_JPRB, 0.10753E+01_JPRB, 0.10883E+01_JPRB, 0.10936E+01_JPRB, 0.10722E+01_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.11411E+01_JPRB, 0.11474E+01_JPRB, 0.11453E+01_JPRB, 0.11629E+01_JPRB, 0.11692E+01_JPRB, 0.11497E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.10601E+01_JPRB, 0.10314E+01_JPRB, 0.10157E+01_JPRB, 0.10008E+01_JPRB, 0.10039E+01_JPRB, 0.99221E+00_JPRB ,&
&  0.10068E+01_JPRB, 0.11471E+01_JPRB, 0.98924E+00_JPRB, 0.96321E+00_JPRB, 0.10224E+01_JPRB, 0.10864E+01_JPRB, 0.12430E+01_JPRB ,&
&  0.19876E+01_JPRB, 0.28595E+01_JPRB, 0.34971E+01_JPRB, 0.43860E+01_JPRB, 0.51546E+01_JPRB, 0.47208E+01_JPRB, 0.45134E+01_JPRB ,&
&  0.43386E+01_JPRB, 0.36623E+01_JPRB, 0.32202E+01_JPRB, 0.28755E+01_JPRB, 0.25951E+01_JPRB, 0.23869E+01_JPRB, 0.21656E+01_JPRB ,&
&  0.19174E+01_JPRB, 0.17271E+01_JPRB, 0.15870E+01_JPRB, 0.14823E+01_JPRB, 0.14088E+01_JPRB, 0.14873E+01_JPRB, 0.16264E+01_JPRB ,&
&  0.18335E+01_JPRB, 0.21913E+01_JPRB, 0.25680E+01_JPRB, 0.30520E+01_JPRB, 0.35486E+01_JPRB, 0.37362E+01_JPRB, 0.50433E+01_JPRB ,&
&  0.50030E+01_JPRB, 0.51991E+01_JPRB, 0.58874E+01_JPRB, 0.76566E+01_JPRB, 0.73842E+01_JPRB, 0.66359E+01_JPRB, 0.68411E+01_JPRB ,&
&  0.45486E+01_JPRB, 0.37886E+01_JPRB, 0.38942E+01_JPRB, 0.39715E+01_JPRB, 0.34204E+01_JPRB, 0.37646E+01_JPRB, 0.58279E+01_JPRB ,&
&  0.45421E+01_JPRB, 0.43684E+01_JPRB, 0.44699E+01_JPRB, 0.48800E+01_JPRB, 0.22441E+01_JPRB, 0.15476E+01_JPRB, 0.24960E+01_JPRB ,&
&  0.27133E+01_JPRB, 0.17145E+01_JPRB, 0.12510E+01_JPRB, 0.16678E+01_JPRB, 0.12811E+01_JPRB, 0.15414E+01_JPRB, 0.22383E+01_JPRB ,&
&  0.24111E+01_JPRB, 0.26002E+01_JPRB, 0.37097E+01_JPRB, 0.34895E+01_JPRB, 0.31597E+01_JPRB, 0.32605E+01_JPRB, 0.31302E+01_JPRB ,&
&  0.29900E+01_JPRB, 0.25598E+01_JPRB, 0.23594E+01_JPRB, 0.19489E+01_JPRB, 0.16789E+01_JPRB, 0.14805E+01_JPRB, 0.13364E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11305E+01_JPRB, 0.11275E+01_JPRB, 0.11280E+01_JPRB, 0.11163E+01_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.12009E+01_JPRB, 0.12042E+01_JPRB, 0.12262E+01_JPRB, 0.12356E+01_JPRB, 0.12483E+01_JPRB, 0.12309E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.11043E+01_JPRB, 0.10913E+01_JPRB, 0.10788E+01_JPRB, 0.10667E+01_JPRB, 0.10469E+01_JPRB, 0.10062E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.10207E+01_JPRB, 0.89103E+00_JPRB, 0.87319E+00_JPRB, 0.99315E+00_JPRB, 0.10314E+01_JPRB, 0.11971E+01_JPRB ,&
&  0.18815E+01_JPRB, 0.27939E+01_JPRB, 0.36603E+01_JPRB, 0.47689E+01_JPRB, 0.54008E+01_JPRB, 0.52005E+01_JPRB, 0.47756E+01_JPRB ,&
&  0.47268E+01_JPRB, 0.38745E+01_JPRB, 0.34239E+01_JPRB, 0.31444E+01_JPRB, 0.28166E+01_JPRB, 0.25207E+01_JPRB, 0.22763E+01_JPRB ,&
&  0.20381E+01_JPRB, 0.18207E+01_JPRB, 0.16844E+01_JPRB, 0.15697E+01_JPRB, 0.15369E+01_JPRB, 0.15552E+01_JPRB, 0.15917E+01_JPRB ,&
&  0.16591E+01_JPRB, 0.18520E+01_JPRB, 0.22883E+01_JPRB, 0.27830E+01_JPRB, 0.31028E+01_JPRB, 0.35165E+01_JPRB, 0.43827E+01_JPRB ,&
&  0.49818E+01_JPRB, 0.50823E+01_JPRB, 0.60073E+01_JPRB, 0.71288E+01_JPRB, 0.82347E+01_JPRB, 0.73614E+01_JPRB, 0.81246E+01_JPRB ,&
&  0.56049E+01_JPRB, 0.44538E+01_JPRB, 0.38123E+01_JPRB, 0.32563E+01_JPRB, 0.36304E+01_JPRB, 0.57209E+01_JPRB, 0.52220E+01_JPRB ,&
&  0.48115E+01_JPRB, 0.44281E+01_JPRB, 0.43840E+01_JPRB, 0.39095E+01_JPRB, 0.33969E+01_JPRB, 0.23381E+01_JPRB, 0.30531E+01_JPRB ,&
&  0.32729E+01_JPRB, 0.32518E+01_JPRB, 0.28722E+01_JPRB, 0.23865E+01_JPRB, 0.19244E+01_JPRB, 0.22208E+01_JPRB, 0.22518E+01_JPRB ,&
&  0.22510E+01_JPRB, 0.21956E+01_JPRB, 0.26725E+01_JPRB, 0.29502E+01_JPRB, 0.30187E+01_JPRB, 0.28369E+01_JPRB, 0.30848E+01_JPRB ,&
&  0.28867E+01_JPRB, 0.28633E+01_JPRB, 0.26400E+01_JPRB, 0.22194E+01_JPRB, 0.19026E+01_JPRB, 0.17176E+01_JPRB, 0.15716E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.12778E+01_JPRB, 0.11992E+01_JPRB, 0.11558E+01_JPRB, 0.11485E+01_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.12568E+01_JPRB, 0.12763E+01_JPRB, 0.12833E+01_JPRB, 0.12838E+01_JPRB, 0.12703E+01_JPRB, 0.12394E+01_JPRB, 0.12175E+01_JPRB ,&
&  0.11769E+01_JPRB, 0.11215E+01_JPRB, 0.10976E+01_JPRB, 0.10861E+01_JPRB, 0.10782E+01_JPRB, 0.10509E+01_JPRB, 0.10177E+01_JPRB ,&
&  0.10111E+01_JPRB, 0.99642E+00_JPRB, 0.86537E+00_JPRB, 0.90159E+00_JPRB, 0.95756E+00_JPRB, 0.99393E+00_JPRB, 0.12145E+01_JPRB ,&
&  0.15842E+01_JPRB, 0.23282E+01_JPRB, 0.31316E+01_JPRB, 0.42014E+01_JPRB, 0.51105E+01_JPRB, 0.54206E+01_JPRB, 0.53286E+01_JPRB ,&
&  0.46915E+01_JPRB, 0.42491E+01_JPRB, 0.36830E+01_JPRB, 0.32253E+01_JPRB, 0.28434E+01_JPRB, 0.25746E+01_JPRB, 0.23034E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.18892E+01_JPRB, 0.17753E+01_JPRB, 0.16717E+01_JPRB, 0.16367E+01_JPRB, 0.16410E+01_JPRB, 0.16481E+01_JPRB ,&
&  0.16174E+01_JPRB, 0.16685E+01_JPRB, 0.19675E+01_JPRB, 0.25278E+01_JPRB, 0.28113E+01_JPRB, 0.32526E+01_JPRB, 0.36250E+01_JPRB ,&
&  0.51995E+01_JPRB, 0.58307E+01_JPRB, 0.58750E+01_JPRB, 0.78408E+01_JPRB, 0.57877E+01_JPRB, 0.73164E+01_JPRB, 0.91826E+01_JPRB ,&
&  0.81063E+01_JPRB, 0.70541E+01_JPRB, 0.66961E+01_JPRB, 0.51784E+01_JPRB, 0.40974E+01_JPRB, 0.55478E+01_JPRB, 0.46827E+01_JPRB ,&
&  0.41217E+01_JPRB, 0.41747E+01_JPRB, 0.40805E+01_JPRB, 0.44333E+01_JPRB, 0.33823E+01_JPRB, 0.28778E+01_JPRB, 0.22631E+01_JPRB ,&
&  0.23491E+01_JPRB, 0.21583E+01_JPRB, 0.25388E+01_JPRB, 0.22376E+01_JPRB, 0.22065E+01_JPRB, 0.20923E+01_JPRB, 0.19901E+01_JPRB ,&
&  0.20612E+01_JPRB, 0.20342E+01_JPRB, 0.20106E+01_JPRB, 0.23111E+01_JPRB, 0.27664E+01_JPRB, 0.25244E+01_JPRB, 0.27763E+01_JPRB ,&
&  0.28863E+01_JPRB, 0.28500E+01_JPRB, 0.26218E+01_JPRB, 0.24191E+01_JPRB, 0.20716E+01_JPRB, 0.18355E+01_JPRB, 0.17093E+01_JPRB ,&
&  0.16152E+01_JPRB, 0.14724E+01_JPRB, 0.13286E+01_JPRB, 0.12678E+01_JPRB, 0.12217E+01_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.13008E+01_JPRB, 0.12979E+01_JPRB, 0.12616E+01_JPRB, 0.12689E+01_JPRB, 0.12674E+01_JPRB, 0.12343E+01_JPRB, 0.11845E+01_JPRB ,&
&  0.11291E+01_JPRB, 0.10976E+01_JPRB, 0.10642E+01_JPRB, 0.10483E+01_JPRB, 0.10527E+01_JPRB, 0.10456E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.99756E+00_JPRB, 0.11350E+01_JPRB, 0.10068E+01_JPRB, 0.91104E+00_JPRB, 0.85791E+00_JPRB, 0.97288E+00_JPRB, 0.11843E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.19157E+01_JPRB, 0.25099E+01_JPRB, 0.30045E+01_JPRB, 0.36843E+01_JPRB, 0.42503E+01_JPRB, 0.45631E+01_JPRB ,&
&  0.46501E+01_JPRB, 0.35162E+01_JPRB, 0.31813E+01_JPRB, 0.27425E+01_JPRB, 0.24663E+01_JPRB, 0.22167E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.19000E+01_JPRB, 0.17961E+01_JPRB, 0.17098E+01_JPRB, 0.16326E+01_JPRB, 0.15956E+01_JPRB, 0.15786E+01_JPRB, 0.15942E+01_JPRB ,&
&  0.15910E+01_JPRB, 0.15808E+01_JPRB, 0.17062E+01_JPRB, 0.20995E+01_JPRB, 0.28688E+01_JPRB, 0.35416E+01_JPRB, 0.39879E+01_JPRB ,&
&  0.38168E+01_JPRB, 0.37821E+01_JPRB, 0.64436E+01_JPRB, 0.72027E+01_JPRB, 0.72879E+01_JPRB, 0.87975E+01_JPRB, 0.95989E+01_JPRB ,&
&  0.89554E+01_JPRB, 0.71271E+01_JPRB, 0.58819E+01_JPRB, 0.49241E+01_JPRB, 0.55136E+01_JPRB, 0.49668E+01_JPRB, 0.44477E+01_JPRB ,&
&  0.40362E+01_JPRB, 0.42705E+01_JPRB, 0.42061E+01_JPRB, 0.41906E+01_JPRB, 0.41475E+01_JPRB, 0.40872E+01_JPRB, 0.34449E+01_JPRB ,&
&  0.29894E+01_JPRB, 0.35553E+01_JPRB, 0.28592E+01_JPRB, 0.21741E+01_JPRB, 0.17852E+01_JPRB, 0.18361E+01_JPRB, 0.19658E+01_JPRB ,&
&  0.20453E+01_JPRB, 0.20785E+01_JPRB, 0.20458E+01_JPRB, 0.21491E+01_JPRB, 0.23892E+01_JPRB, 0.24721E+01_JPRB, 0.25518E+01_JPRB ,&
&  0.25185E+01_JPRB, 0.26938E+01_JPRB, 0.26358E+01_JPRB, 0.24898E+01_JPRB, 0.22438E+01_JPRB, 0.19697E+01_JPRB, 0.18036E+01_JPRB ,&
&  0.17202E+01_JPRB, 0.16475E+01_JPRB, 0.15092E+01_JPRB, 0.13752E+01_JPRB, 0.12981E+01_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.13159E+01_JPRB, 0.12846E+01_JPRB, 0.12413E+01_JPRB, 0.12476E+01_JPRB, 0.12650E+01_JPRB, 0.12483E+01_JPRB, 0.11869E+01_JPRB ,&
&  0.10892E+01_JPRB, 0.10358E+01_JPRB, 0.10136E+01_JPRB, 0.10209E+01_JPRB, 0.10399E+01_JPRB, 0.10554E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10369E+01_JPRB, 0.13389E+01_JPRB, 0.11962E+01_JPRB, 0.11474E+01_JPRB, 0.10262E+01_JPRB, 0.11645E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.14903E+01_JPRB, 0.15860E+01_JPRB, 0.17914E+01_JPRB, 0.20926E+01_JPRB, 0.23934E+01_JPRB, 0.26470E+01_JPRB, 0.30697E+01_JPRB ,&
&  0.30181E+01_JPRB, 0.27604E+01_JPRB, 0.22730E+01_JPRB, 0.21593E+01_JPRB, 0.20235E+01_JPRB, 0.17706E+01_JPRB, 0.17706E+01_JPRB ,&
&  0.16743E+01_JPRB, 0.16189E+01_JPRB, 0.15666E+01_JPRB, 0.15330E+01_JPRB, 0.14995E+01_JPRB, 0.14474E+01_JPRB, 0.14269E+01_JPRB ,&
&  0.14305E+01_JPRB, 0.14551E+01_JPRB, 0.15181E+01_JPRB, 0.16509E+01_JPRB, 0.21619E+01_JPRB, 0.28555E+01_JPRB, 0.34369E+01_JPRB ,&
&  0.41641E+01_JPRB, 0.37813E+01_JPRB, 0.34920E+01_JPRB, 0.48984E+01_JPRB, 0.76620E+01_JPRB, 0.84390E+01_JPRB, 0.80853E+01_JPRB ,&
&  0.89022E+01_JPRB, 0.83346E+01_JPRB, 0.72300E+01_JPRB, 0.62643E+01_JPRB, 0.56217E+01_JPRB, 0.50639E+01_JPRB, 0.47904E+01_JPRB ,&
&  0.43897E+01_JPRB, 0.42796E+01_JPRB, 0.44907E+01_JPRB, 0.46134E+01_JPRB, 0.43484E+01_JPRB, 0.40487E+01_JPRB, 0.40447E+01_JPRB ,&
&  0.38520E+01_JPRB, 0.31999E+01_JPRB, 0.21633E+01_JPRB, 0.21130E+01_JPRB, 0.16292E+01_JPRB, 0.16235E+01_JPRB, 0.19175E+01_JPRB ,&
&  0.18872E+01_JPRB, 0.20270E+01_JPRB, 0.20756E+01_JPRB, 0.19781E+01_JPRB, 0.21135E+01_JPRB, 0.21447E+01_JPRB, 0.22745E+01_JPRB ,&
&  0.23617E+01_JPRB, 0.20439E+01_JPRB, 0.21515E+01_JPRB, 0.22137E+01_JPRB, 0.21807E+01_JPRB, 0.20251E+01_JPRB, 0.18745E+01_JPRB ,&
&  0.17817E+01_JPRB, 0.17053E+01_JPRB, 0.16046E+01_JPRB, 0.14664E+01_JPRB, 0.13677E+01_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.13429E+01_JPRB, 0.12977E+01_JPRB, 0.12502E+01_JPRB, 0.12336E+01_JPRB, 0.12294E+01_JPRB, 0.12099E+01_JPRB, 0.11557E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.10298E+01_JPRB, 0.10096E+01_JPRB, 0.10167E+01_JPRB, 0.10296E+01_JPRB, 0.10321E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.12404E+01_JPRB, 0.14269E+01_JPRB, 0.12997E+01_JPRB, 0.11478E+01_JPRB, 0.13439E+01_JPRB, 0.12897E+01_JPRB, 0.13438E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.14655E+01_JPRB, 0.14952E+01_JPRB, 0.15508E+01_JPRB, 0.16239E+01_JPRB, 0.17742E+01_JPRB, 0.20292E+01_JPRB ,&
&  0.20686E+01_JPRB, 0.18359E+01_JPRB, 0.16681E+01_JPRB, 0.16253E+01_JPRB, 0.16669E+01_JPRB, 0.15149E+01_JPRB, 0.14836E+01_JPRB ,&
&  0.14189E+01_JPRB, 0.13689E+01_JPRB, 0.13310E+01_JPRB, 0.13162E+01_JPRB, 0.13048E+01_JPRB, 0.12930E+01_JPRB, 0.12928E+01_JPRB ,&
&  0.13100E+01_JPRB, 0.13345E+01_JPRB, 0.13734E+01_JPRB, 0.14572E+01_JPRB, 0.16949E+01_JPRB, 0.22930E+01_JPRB, 0.33012E+01_JPRB ,&
&  0.40758E+01_JPRB, 0.42741E+01_JPRB, 0.47017E+01_JPRB, 0.56885E+01_JPRB, 0.63529E+01_JPRB, 0.67215E+01_JPRB, 0.71629E+01_JPRB ,&
&  0.73161E+01_JPRB, 0.73282E+01_JPRB, 0.73051E+01_JPRB, 0.68762E+01_JPRB, 0.61747E+01_JPRB, 0.56041E+01_JPRB, 0.52601E+01_JPRB ,&
&  0.48983E+01_JPRB, 0.47662E+01_JPRB, 0.48841E+01_JPRB, 0.47194E+01_JPRB, 0.46346E+01_JPRB, 0.43154E+01_JPRB, 0.44874E+01_JPRB ,&
&  0.43392E+01_JPRB, 0.25472E+01_JPRB, 0.19887E+01_JPRB, 0.22596E+01_JPRB, 0.17914E+01_JPRB, 0.18433E+01_JPRB, 0.21301E+01_JPRB ,&
&  0.20440E+01_JPRB, 0.19817E+01_JPRB, 0.20276E+01_JPRB, 0.19564E+01_JPRB, 0.18802E+01_JPRB, 0.19677E+01_JPRB, 0.19148E+01_JPRB ,&
&  0.17783E+01_JPRB, 0.16526E+01_JPRB, 0.17244E+01_JPRB, 0.18216E+01_JPRB, 0.19000E+01_JPRB, 0.18523E+01_JPRB, 0.17633E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.16576E+01_JPRB, 0.16073E+01_JPRB, 0.15181E+01_JPRB, 0.14120E+01_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.14079E+01_JPRB, 0.13655E+01_JPRB, 0.13246E+01_JPRB, 0.12868E+01_JPRB, 0.12380E+01_JPRB, 0.11812E+01_JPRB, 0.11136E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10114E+01_JPRB, 0.99198E+00_JPRB, 0.98742E+00_JPRB, 0.98345E+00_JPRB, 0.97590E+00_JPRB, 0.10475E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.11715E+01_JPRB, 0.10746E+01_JPRB, 0.11477E+01_JPRB, 0.14135E+01_JPRB, 0.12628E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.12580E+01_JPRB, 0.13092E+01_JPRB, 0.12939E+01_JPRB, 0.12681E+01_JPRB, 0.12581E+01_JPRB, 0.12934E+01_JPRB, 0.14206E+01_JPRB ,&
&  0.14438E+01_JPRB, 0.13219E+01_JPRB, 0.12621E+01_JPRB, 0.12545E+01_JPRB, 0.12894E+01_JPRB, 0.13410E+01_JPRB, 0.13675E+01_JPRB ,&
&  0.13080E+01_JPRB, 0.12590E+01_JPRB, 0.12250E+01_JPRB, 0.11962E+01_JPRB, 0.11671E+01_JPRB, 0.11447E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11548E+01_JPRB, 0.12101E+01_JPRB, 0.13208E+01_JPRB, 0.14386E+01_JPRB, 0.18960E+01_JPRB, 0.28542E+01_JPRB ,&
&  0.39069E+01_JPRB, 0.40674E+01_JPRB, 0.45843E+01_JPRB, 0.56719E+01_JPRB, 0.59953E+01_JPRB, 0.60418E+01_JPRB, 0.60272E+01_JPRB ,&
&  0.60663E+01_JPRB, 0.60321E+01_JPRB, 0.60138E+01_JPRB, 0.62721E+01_JPRB, 0.62634E+01_JPRB, 0.63055E+01_JPRB, 0.60647E+01_JPRB ,&
&  0.56149E+01_JPRB, 0.54396E+01_JPRB, 0.53938E+01_JPRB, 0.51616E+01_JPRB, 0.48501E+01_JPRB, 0.47733E+01_JPRB, 0.46257E+01_JPRB ,&
&  0.43245E+01_JPRB, 0.38270E+01_JPRB, 0.28674E+01_JPRB, 0.26132E+01_JPRB, 0.19438E+01_JPRB, 0.19650E+01_JPRB, 0.23820E+01_JPRB ,&
&  0.22473E+01_JPRB, 0.19814E+01_JPRB, 0.18115E+01_JPRB, 0.17118E+01_JPRB, 0.16479E+01_JPRB, 0.16741E+01_JPRB, 0.15261E+01_JPRB ,&
&  0.13603E+01_JPRB, 0.13932E+01_JPRB, 0.14510E+01_JPRB, 0.15309E+01_JPRB, 0.16163E+01_JPRB, 0.16272E+01_JPRB, 0.14665E+01_JPRB ,&
&  0.15126E+01_JPRB, 0.15235E+01_JPRB, 0.15167E+01_JPRB, 0.15052E+01_JPRB, 0.14639E+01_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.13682E+01_JPRB, 0.13387E+01_JPRB, 0.13094E+01_JPRB, 0.12758E+01_JPRB, 0.12078E+01_JPRB, 0.11166E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10174E+01_JPRB, 0.98821E+00_JPRB, 0.96622E+00_JPRB, 0.95155E+00_JPRB, 0.93689E+00_JPRB, 0.92393E+00_JPRB, 0.91914E+00_JPRB ,&
&  0.90905E+00_JPRB, 0.10087E+01_JPRB, 0.99976E+00_JPRB, 0.10387E+01_JPRB, 0.10725E+01_JPRB, 0.10406E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.11057E+01_JPRB, 0.11249E+01_JPRB, 0.11208E+01_JPRB, 0.10996E+01_JPRB, 0.10861E+01_JPRB, 0.11090E+01_JPRB, 0.11727E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.11169E+01_JPRB, 0.10967E+01_JPRB, 0.11085E+01_JPRB, 0.11781E+01_JPRB, 0.12600E+01_JPRB, 0.12591E+01_JPRB ,&
&  0.12158E+01_JPRB, 0.11720E+01_JPRB, 0.11291E+01_JPRB, 0.10961E+01_JPRB, 0.10660E+01_JPRB, 0.10392E+01_JPRB, 0.10207E+01_JPRB ,&
&  0.10210E+01_JPRB, 0.10443E+01_JPRB, 0.10945E+01_JPRB, 0.11900E+01_JPRB, 0.13159E+01_JPRB, 0.16236E+01_JPRB, 0.21179E+01_JPRB ,&
&  0.25651E+01_JPRB, 0.29546E+01_JPRB, 0.34054E+01_JPRB, 0.39259E+01_JPRB, 0.42980E+01_JPRB, 0.44267E+01_JPRB, 0.44817E+01_JPRB ,&
&  0.46876E+01_JPRB, 0.46791E+01_JPRB, 0.50669E+01_JPRB, 0.50745E+01_JPRB, 0.51351E+01_JPRB, 0.53812E+01_JPRB, 0.53601E+01_JPRB ,&
&  0.52108E+01_JPRB, 0.49291E+01_JPRB, 0.50824E+01_JPRB, 0.50117E+01_JPRB, 0.48475E+01_JPRB, 0.46696E+01_JPRB, 0.44019E+01_JPRB ,&
&  0.41280E+01_JPRB, 0.38487E+01_JPRB, 0.33169E+01_JPRB, 0.29643E+01_JPRB, 0.25523E+01_JPRB, 0.25807E+01_JPRB, 0.23137E+01_JPRB ,&
&  0.20038E+01_JPRB, 0.15918E+01_JPRB, 0.15345E+01_JPRB, 0.14835E+01_JPRB, 0.14788E+01_JPRB, 0.13788E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.12149E+01_JPRB, 0.13045E+01_JPRB, 0.13200E+01_JPRB, 0.13439E+01_JPRB, 0.13992E+01_JPRB, 0.14371E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12121E+01_JPRB, 0.13260E+01_JPRB, 0.13664E+01_JPRB, 0.13868E+01_JPRB, 0.13872E+01_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.12603E+01_JPRB, 0.12392E+01_JPRB, 0.12076E+01_JPRB, 0.11732E+01_JPRB, 0.11124E+01_JPRB, 0.10346E+01_JPRB, 0.96419E+00_JPRB ,&
&  0.95945E+00_JPRB, 0.94734E+00_JPRB, 0.93261E+00_JPRB, 0.91194E+00_JPRB, 0.88712E+00_JPRB, 0.84323E+00_JPRB, 0.77709E+00_JPRB ,&
&  0.77713E+00_JPRB, 0.82541E+00_JPRB, 0.89245E+00_JPRB, 0.92568E+00_JPRB, 0.92014E+00_JPRB, 0.91413E+00_JPRB, 0.92647E+00_JPRB ,&
&  0.94195E+00_JPRB, 0.95497E+00_JPRB, 0.97531E+00_JPRB, 0.98887E+00_JPRB, 0.98004E+00_JPRB, 0.97500E+00_JPRB, 0.99937E+00_JPRB ,&
&  0.96357E+00_JPRB, 0.95074E+00_JPRB, 0.97448E+00_JPRB, 0.94205E+00_JPRB, 0.10239E+01_JPRB, 0.10503E+01_JPRB, 0.10501E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.97381E+00_JPRB, 0.94039E+00_JPRB, 0.93216E+00_JPRB, 0.92764E+00_JPRB, 0.92149E+00_JPRB, 0.92263E+00_JPRB ,&
&  0.93606E+00_JPRB, 0.96278E+00_JPRB, 0.10106E+01_JPRB, 0.10889E+01_JPRB, 0.11802E+01_JPRB, 0.13556E+01_JPRB, 0.15724E+01_JPRB ,&
&  0.18785E+01_JPRB, 0.22634E+01_JPRB, 0.27162E+01_JPRB, 0.28140E+01_JPRB, 0.31771E+01_JPRB, 0.35027E+01_JPRB, 0.35873E+01_JPRB ,&
&  0.38558E+01_JPRB, 0.39411E+01_JPRB, 0.42177E+01_JPRB, 0.43313E+01_JPRB, 0.44057E+01_JPRB, 0.44521E+01_JPRB, 0.44846E+01_JPRB ,&
&  0.44897E+01_JPRB, 0.44697E+01_JPRB, 0.45389E+01_JPRB, 0.44726E+01_JPRB, 0.42896E+01_JPRB, 0.41085E+01_JPRB, 0.39032E+01_JPRB ,&
&  0.37284E+01_JPRB, 0.35307E+01_JPRB, 0.33281E+01_JPRB, 0.31573E+01_JPRB, 0.29655E+01_JPRB, 0.27017E+01_JPRB, 0.24118E+01_JPRB ,&
&  0.21127E+01_JPRB, 0.17325E+01_JPRB, 0.15250E+01_JPRB, 0.14698E+01_JPRB, 0.14054E+01_JPRB, 0.12347E+01_JPRB, 0.11433E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11423E+01_JPRB, 0.12178E+01_JPRB, 0.12233E+01_JPRB, 0.12498E+01_JPRB, 0.12509E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.11685E+01_JPRB, 0.12133E+01_JPRB, 0.12398E+01_JPRB, 0.12598E+01_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.11520E+01_JPRB, 0.11409E+01_JPRB, 0.11127E+01_JPRB, 0.10868E+01_JPRB, 0.10548E+01_JPRB, 0.98508E+00_JPRB, 0.92379E+00_JPRB ,&
&  0.92277E+00_JPRB, 0.88693E+00_JPRB, 0.87220E+00_JPRB, 0.81056E+00_JPRB, 0.77885E+00_JPRB, 0.76222E+00_JPRB, 0.74829E+00_JPRB ,&
&  0.75460E+00_JPRB, 0.78018E+00_JPRB, 0.80432E+00_JPRB, 0.82625E+00_JPRB, 0.84614E+00_JPRB, 0.84867E+00_JPRB, 0.84991E+00_JPRB ,&
&  0.84966E+00_JPRB, 0.85601E+00_JPRB, 0.87238E+00_JPRB, 0.87893E+00_JPRB, 0.88344E+00_JPRB, 0.88087E+00_JPRB, 0.88072E+00_JPRB ,&
&  0.86241E+00_JPRB, 0.86019E+00_JPRB, 0.88838E+00_JPRB, 0.88541E+00_JPRB, 0.88782E+00_JPRB, 0.88494E+00_JPRB, 0.86816E+00_JPRB ,&
&  0.82780E+00_JPRB, 0.80042E+00_JPRB, 0.81522E+00_JPRB, 0.82668E+00_JPRB, 0.83771E+00_JPRB, 0.84365E+00_JPRB, 0.84720E+00_JPRB ,&
&  0.85302E+00_JPRB, 0.86498E+00_JPRB, 0.88613E+00_JPRB, 0.91806E+00_JPRB, 0.96628E+00_JPRB, 0.10658E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.14193E+01_JPRB, 0.14218E+01_JPRB, 0.17857E+01_JPRB, 0.20733E+01_JPRB, 0.24719E+01_JPRB, 0.28316E+01_JPRB, 0.29887E+01_JPRB ,&
&  0.32317E+01_JPRB, 0.32862E+01_JPRB, 0.33525E+01_JPRB, 0.33591E+01_JPRB, 0.33458E+01_JPRB, 0.33617E+01_JPRB, 0.34465E+01_JPRB ,&
&  0.35330E+01_JPRB, 0.34229E+01_JPRB, 0.36471E+01_JPRB, 0.35643E+01_JPRB, 0.34561E+01_JPRB, 0.34112E+01_JPRB, 0.33703E+01_JPRB ,&
&  0.32861E+01_JPRB, 0.31365E+01_JPRB, 0.29292E+01_JPRB, 0.25619E+01_JPRB, 0.24427E+01_JPRB, 0.23310E+01_JPRB, 0.21041E+01_JPRB ,&
&  0.19699E+01_JPRB, 0.18092E+01_JPRB, 0.16387E+01_JPRB, 0.16057E+01_JPRB, 0.14272E+01_JPRB, 0.12971E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.10965E+01_JPRB, 0.10343E+01_JPRB, 0.10620E+01_JPRB, 0.10775E+01_JPRB, 0.10731E+01_JPRB, 0.10511E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10543E+01_JPRB, 0.10659E+01_JPRB, 0.11276E+01_JPRB, 0.11368E+01_JPRB, 0.11447E+01_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.10882E+01_JPRB, 0.10804E+01_JPRB, 0.10532E+01_JPRB, 0.10312E+01_JPRB, 0.10020E+01_JPRB, 0.95353E+00_JPRB, 0.88032E+00_JPRB ,&
&  0.84099E+00_JPRB, 0.79799E+00_JPRB, 0.76449E+00_JPRB, 0.75206E+00_JPRB, 0.74617E+00_JPRB, 0.73516E+00_JPRB, 0.72093E+00_JPRB ,&
&  0.72446E+00_JPRB, 0.74680E+00_JPRB, 0.77349E+00_JPRB, 0.78818E+00_JPRB, 0.79875E+00_JPRB, 0.80184E+00_JPRB, 0.80706E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.81528E+00_JPRB, 0.81946E+00_JPRB, 0.81523E+00_JPRB, 0.81032E+00_JPRB, 0.81330E+00_JPRB, 0.81754E+00_JPRB ,&
&  0.82645E+00_JPRB, 0.82860E+00_JPRB, 0.83373E+00_JPRB, 0.83869E+00_JPRB, 0.83649E+00_JPRB, 0.83445E+00_JPRB, 0.77987E+00_JPRB ,&
&  0.71905E+00_JPRB, 0.73960E+00_JPRB, 0.75582E+00_JPRB, 0.76905E+00_JPRB, 0.79244E+00_JPRB, 0.80422E+00_JPRB, 0.81028E+00_JPRB ,&
&  0.81174E+00_JPRB, 0.81296E+00_JPRB, 0.82077E+00_JPRB, 0.83981E+00_JPRB, 0.87249E+00_JPRB, 0.91599E+00_JPRB, 0.96185E+00_JPRB ,&
&  0.10800E+01_JPRB, 0.11073E+01_JPRB, 0.11892E+01_JPRB, 0.13807E+01_JPRB, 0.18150E+01_JPRB, 0.21195E+01_JPRB, 0.22709E+01_JPRB ,&
&  0.24123E+01_JPRB, 0.25078E+01_JPRB, 0.25520E+01_JPRB, 0.26195E+01_JPRB, 0.26209E+01_JPRB, 0.26230E+01_JPRB, 0.26732E+01_JPRB ,&
&  0.26758E+01_JPRB, 0.27391E+01_JPRB, 0.27684E+01_JPRB, 0.28143E+01_JPRB, 0.27634E+01_JPRB, 0.27348E+01_JPRB, 0.27133E+01_JPRB ,&
&  0.26743E+01_JPRB, 0.25890E+01_JPRB, 0.24069E+01_JPRB, 0.22241E+01_JPRB, 0.20539E+01_JPRB, 0.19531E+01_JPRB, 0.18707E+01_JPRB ,&
&  0.17760E+01_JPRB, 0.17057E+01_JPRB, 0.16342E+01_JPRB, 0.15440E+01_JPRB, 0.14257E+01_JPRB, 0.13141E+01_JPRB, 0.12082E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.99056E+00_JPRB, 0.96133E+00_JPRB, 0.94738E+00_JPRB, 0.95491E+00_JPRB, 0.97968E+00_JPRB, 0.95951E+00_JPRB ,&
&  0.99228E+00_JPRB, 0.10048E+01_JPRB, 0.10006E+01_JPRB, 0.10204E+01_JPRB, 0.10514E+01_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.99592E+00_JPRB, 0.99927E+00_JPRB, 0.98407E+00_JPRB, 0.97838E+00_JPRB, 0.94868E+00_JPRB, 0.91399E+00_JPRB, 0.87360E+00_JPRB ,&
&  0.84333E+00_JPRB, 0.80927E+00_JPRB, 0.77561E+00_JPRB, 0.74641E+00_JPRB, 0.73267E+00_JPRB, 0.72919E+00_JPRB, 0.73658E+00_JPRB ,&
&  0.75071E+00_JPRB, 0.76621E+00_JPRB, 0.77798E+00_JPRB, 0.78070E+00_JPRB, 0.77770E+00_JPRB, 0.77641E+00_JPRB, 0.78029E+00_JPRB ,&
&  0.78322E+00_JPRB, 0.78622E+00_JPRB, 0.78979E+00_JPRB, 0.78890E+00_JPRB, 0.78766E+00_JPRB, 0.78749E+00_JPRB, 0.79070E+00_JPRB ,&
&  0.79122E+00_JPRB, 0.78667E+00_JPRB, 0.79331E+00_JPRB, 0.80214E+00_JPRB, 0.80881E+00_JPRB, 0.78761E+00_JPRB, 0.72924E+00_JPRB ,&
&  0.68381E+00_JPRB, 0.67741E+00_JPRB, 0.73101E+00_JPRB, 0.72789E+00_JPRB, 0.75428E+00_JPRB, 0.76499E+00_JPRB, 0.76194E+00_JPRB ,&
&  0.75085E+00_JPRB, 0.74754E+00_JPRB, 0.76216E+00_JPRB, 0.78323E+00_JPRB, 0.79998E+00_JPRB, 0.82337E+00_JPRB, 0.85013E+00_JPRB ,&
&  0.89862E+00_JPRB, 0.97121E+00_JPRB, 0.10516E+01_JPRB, 0.10891E+01_JPRB, 0.12587E+01_JPRB, 0.17288E+01_JPRB, 0.18753E+01_JPRB ,&
&  0.19478E+01_JPRB, 0.20095E+01_JPRB, 0.21228E+01_JPRB, 0.21986E+01_JPRB, 0.21810E+01_JPRB, 0.21767E+01_JPRB, 0.21761E+01_JPRB ,&
&  0.21268E+01_JPRB, 0.21205E+01_JPRB, 0.21413E+01_JPRB, 0.22161E+01_JPRB, 0.22160E+01_JPRB, 0.21817E+01_JPRB, 0.21475E+01_JPRB ,&
&  0.21180E+01_JPRB, 0.20288E+01_JPRB, 0.19269E+01_JPRB, 0.17777E+01_JPRB, 0.16896E+01_JPRB, 0.16292E+01_JPRB, 0.15944E+01_JPRB ,&
&  0.15508E+01_JPRB, 0.15326E+01_JPRB, 0.15154E+01_JPRB, 0.14832E+01_JPRB, 0.13401E+01_JPRB, 0.11709E+01_JPRB, 0.10409E+01_JPRB ,&
&  0.96330E+00_JPRB, 0.92055E+00_JPRB, 0.90475E+00_JPRB, 0.90674E+00_JPRB, 0.91847E+00_JPRB, 0.92814E+00_JPRB, 0.89454E+00_JPRB ,&
&  0.88407E+00_JPRB, 0.88790E+00_JPRB, 0.90761E+00_JPRB, 0.94056E+00_JPRB, 0.97211E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.89183E+00_JPRB, 0.92223E+00_JPRB, 0.94278E+00_JPRB, 0.94284E+00_JPRB, 0.92810E+00_JPRB, 0.87097E+00_JPRB, 0.82020E+00_JPRB ,&
&  0.79020E+00_JPRB, 0.78086E+00_JPRB, 0.78160E+00_JPRB, 0.78259E+00_JPRB, 0.78514E+00_JPRB, 0.78930E+00_JPRB, 0.79127E+00_JPRB ,&
&  0.79070E+00_JPRB, 0.79325E+00_JPRB, 0.79794E+00_JPRB, 0.79955E+00_JPRB, 0.79512E+00_JPRB, 0.79065E+00_JPRB, 0.79128E+00_JPRB ,&
&  0.79158E+00_JPRB, 0.78984E+00_JPRB, 0.79130E+00_JPRB, 0.78777E+00_JPRB, 0.78801E+00_JPRB, 0.78313E+00_JPRB, 0.77409E+00_JPRB ,&
&  0.75751E+00_JPRB, 0.75294E+00_JPRB, 0.76177E+00_JPRB, 0.76396E+00_JPRB, 0.77099E+00_JPRB, 0.73938E+00_JPRB, 0.68870E+00_JPRB ,&
&  0.65212E+00_JPRB, 0.63513E+00_JPRB, 0.64670E+00_JPRB, 0.68818E+00_JPRB, 0.72046E+00_JPRB, 0.69562E+00_JPRB, 0.71123E+00_JPRB ,&
&  0.70921E+00_JPRB, 0.70982E+00_JPRB, 0.71628E+00_JPRB, 0.73504E+00_JPRB, 0.75485E+00_JPRB, 0.77386E+00_JPRB, 0.79125E+00_JPRB ,&
&  0.81433E+00_JPRB, 0.84868E+00_JPRB, 0.89796E+00_JPRB, 0.94630E+00_JPRB, 0.98161E+00_JPRB, 0.10971E+01_JPRB, 0.12463E+01_JPRB ,&
&  0.14131E+01_JPRB, 0.15337E+01_JPRB, 0.16111E+01_JPRB, 0.17209E+01_JPRB, 0.18060E+01_JPRB, 0.18410E+01_JPRB, 0.18462E+01_JPRB ,&
&  0.18477E+01_JPRB, 0.18161E+01_JPRB, 0.17940E+01_JPRB, 0.18127E+01_JPRB, 0.18517E+01_JPRB, 0.18655E+01_JPRB, 0.18286E+01_JPRB ,&
&  0.17757E+01_JPRB, 0.16696E+01_JPRB, 0.15499E+01_JPRB, 0.14116E+01_JPRB, 0.13922E+01_JPRB, 0.14391E+01_JPRB, 0.14924E+01_JPRB ,&
&  0.14838E+01_JPRB, 0.14582E+01_JPRB, 0.14351E+01_JPRB, 0.14118E+01_JPRB, 0.12540E+01_JPRB, 0.11418E+01_JPRB, 0.10613E+01_JPRB ,&
&  0.10216E+01_JPRB, 0.98854E+00_JPRB, 0.97734E+00_JPRB, 0.97764E+00_JPRB, 0.97101E+00_JPRB, 0.95803E+00_JPRB, 0.91829E+00_JPRB ,&
&  0.88655E+00_JPRB, 0.86786E+00_JPRB, 0.86418E+00_JPRB, 0.86319E+00_JPRB, 0.87545E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.88905E+00_JPRB, 0.89606E+00_JPRB, 0.90265E+00_JPRB, 0.90508E+00_JPRB, 0.89731E+00_JPRB, 0.87226E+00_JPRB, 0.85050E+00_JPRB ,&
&  0.83600E+00_JPRB, 0.83366E+00_JPRB, 0.83180E+00_JPRB, 0.82803E+00_JPRB, 0.82586E+00_JPRB, 0.82388E+00_JPRB, 0.82309E+00_JPRB ,&
&  0.82417E+00_JPRB, 0.82342E+00_JPRB, 0.81778E+00_JPRB, 0.80847E+00_JPRB, 0.80371E+00_JPRB, 0.80393E+00_JPRB, 0.80666E+00_JPRB ,&
&  0.80538E+00_JPRB, 0.80137E+00_JPRB, 0.80171E+00_JPRB, 0.79056E+00_JPRB, 0.78078E+00_JPRB, 0.76355E+00_JPRB, 0.75188E+00_JPRB ,&
&  0.75136E+00_JPRB, 0.75347E+00_JPRB, 0.73953E+00_JPRB, 0.73759E+00_JPRB, 0.74207E+00_JPRB, 0.69684E+00_JPRB, 0.65901E+00_JPRB ,&
&  0.63150E+00_JPRB, 0.61154E+00_JPRB, 0.60201E+00_JPRB, 0.60371E+00_JPRB, 0.63231E+00_JPRB, 0.67219E+00_JPRB, 0.68871E+00_JPRB ,&
&  0.66104E+00_JPRB, 0.68177E+00_JPRB, 0.70851E+00_JPRB, 0.72295E+00_JPRB, 0.73393E+00_JPRB, 0.74861E+00_JPRB, 0.76320E+00_JPRB ,&
&  0.77868E+00_JPRB, 0.79516E+00_JPRB, 0.81547E+00_JPRB, 0.86089E+00_JPRB, 0.89395E+00_JPRB, 0.89821E+00_JPRB, 0.94308E+00_JPRB ,&
&  0.11717E+01_JPRB, 0.13514E+01_JPRB, 0.14788E+01_JPRB, 0.15473E+01_JPRB, 0.15795E+01_JPRB, 0.15643E+01_JPRB, 0.15506E+01_JPRB ,&
&  0.15397E+01_JPRB, 0.15418E+01_JPRB, 0.15625E+01_JPRB, 0.15846E+01_JPRB, 0.16051E+01_JPRB, 0.16046E+01_JPRB, 0.15940E+01_JPRB ,&
&  0.15592E+01_JPRB, 0.15489E+01_JPRB, 0.18848E+01_JPRB, 0.18010E+01_JPRB, 0.17234E+01_JPRB, 0.16821E+01_JPRB, 0.16128E+01_JPRB ,&
&  0.15471E+01_JPRB, 0.14949E+01_JPRB, 0.14649E+01_JPRB, 0.14262E+01_JPRB, 0.13188E+01_JPRB, 0.12370E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.11569E+01_JPRB, 0.11298E+01_JPRB, 0.11035E+01_JPRB, 0.10798E+01_JPRB, 0.10511E+01_JPRB, 0.10152E+01_JPRB, 0.96532E+00_JPRB ,&
&  0.92611E+00_JPRB, 0.90616E+00_JPRB, 0.89459E+00_JPRB, 0.89066E+00_JPRB, 0.88910E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.85921E+00_JPRB, 0.86348E+00_JPRB, 0.86865E+00_JPRB, 0.87160E+00_JPRB, 0.86906E+00_JPRB, 0.86474E+00_JPRB, 0.85775E+00_JPRB ,&
&  0.85100E+00_JPRB, 0.84857E+00_JPRB, 0.84652E+00_JPRB, 0.84556E+00_JPRB, 0.84461E+00_JPRB, 0.84269E+00_JPRB, 0.84065E+00_JPRB ,&
&  0.84000E+00_JPRB, 0.82388E+00_JPRB, 0.81678E+00_JPRB, 0.81842E+00_JPRB, 0.81784E+00_JPRB, 0.81813E+00_JPRB, 0.81854E+00_JPRB ,&
&  0.81098E+00_JPRB, 0.79954E+00_JPRB, 0.79127E+00_JPRB, 0.78070E+00_JPRB, 0.76931E+00_JPRB, 0.77139E+00_JPRB, 0.76915E+00_JPRB ,&
&  0.75321E+00_JPRB, 0.73908E+00_JPRB, 0.74456E+00_JPRB, 0.74459E+00_JPRB, 0.70544E+00_JPRB, 0.66542E+00_JPRB, 0.62895E+00_JPRB ,&
&  0.62137E+00_JPRB, 0.60026E+00_JPRB, 0.59588E+00_JPRB, 0.59617E+00_JPRB, 0.60823E+00_JPRB, 0.62501E+00_JPRB, 0.65468E+00_JPRB ,&
&  0.66783E+00_JPRB, 0.67917E+00_JPRB, 0.70204E+00_JPRB, 0.72245E+00_JPRB, 0.73499E+00_JPRB, 0.74584E+00_JPRB, 0.74778E+00_JPRB ,&
&  0.75698E+00_JPRB, 0.76667E+00_JPRB, 0.77891E+00_JPRB, 0.79451E+00_JPRB, 0.81762E+00_JPRB, 0.83676E+00_JPRB, 0.86254E+00_JPRB ,&
&  0.95348E+00_JPRB, 0.10303E+01_JPRB, 0.10960E+01_JPRB, 0.11551E+01_JPRB, 0.12000E+01_JPRB, 0.12245E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.12113E+01_JPRB, 0.12568E+01_JPRB, 0.13016E+01_JPRB, 0.13465E+01_JPRB, 0.13868E+01_JPRB, 0.14216E+01_JPRB, 0.14359E+01_JPRB ,&
&  0.14094E+01_JPRB, 0.14058E+01_JPRB, 0.15339E+01_JPRB, 0.15743E+01_JPRB, 0.15627E+01_JPRB, 0.15211E+01_JPRB, 0.14606E+01_JPRB ,&
&  0.14096E+01_JPRB, 0.13754E+01_JPRB, 0.13531E+01_JPRB, 0.13309E+01_JPRB, 0.13084E+01_JPRB, 0.12852E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.12216E+01_JPRB, 0.11843E+01_JPRB, 0.11463E+01_JPRB, 0.11094E+01_JPRB, 0.10700E+01_JPRB, 0.10086E+01_JPRB ,&
&  0.94369E+00_JPRB, 0.89644E+00_JPRB, 0.87119E+00_JPRB, 0.85916E+00_JPRB, 0.85731E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.84706E+00_JPRB, 0.84429E+00_JPRB, 0.84630E+00_JPRB, 0.84459E+00_JPRB, 0.84459E+00_JPRB, 0.84085E+00_JPRB, 0.83923E+00_JPRB ,&
&  0.83658E+00_JPRB, 0.83587E+00_JPRB, 0.83540E+00_JPRB, 0.83581E+00_JPRB, 0.83555E+00_JPRB, 0.83550E+00_JPRB, 0.83453E+00_JPRB ,&
&  0.83269E+00_JPRB, 0.83068E+00_JPRB, 0.82688E+00_JPRB, 0.82325E+00_JPRB, 0.82083E+00_JPRB, 0.82148E+00_JPRB, 0.81877E+00_JPRB ,&
&  0.81199E+00_JPRB, 0.79840E+00_JPRB, 0.77480E+00_JPRB, 0.76589E+00_JPRB, 0.76535E+00_JPRB, 0.75308E+00_JPRB, 0.75350E+00_JPRB ,&
&  0.74729E+00_JPRB, 0.74561E+00_JPRB, 0.73597E+00_JPRB, 0.73226E+00_JPRB, 0.70793E+00_JPRB, 0.62042E+00_JPRB, 0.61228E+00_JPRB ,&
&  0.60481E+00_JPRB, 0.59345E+00_JPRB, 0.59136E+00_JPRB, 0.59325E+00_JPRB, 0.60158E+00_JPRB, 0.62778E+00_JPRB, 0.65298E+00_JPRB ,&
&  0.65684E+00_JPRB, 0.69387E+00_JPRB, 0.70777E+00_JPRB, 0.71946E+00_JPRB, 0.74006E+00_JPRB, 0.75595E+00_JPRB, 0.75582E+00_JPRB ,&
&  0.75874E+00_JPRB, 0.76204E+00_JPRB, 0.77442E+00_JPRB, 0.78798E+00_JPRB, 0.80057E+00_JPRB, 0.81385E+00_JPRB, 0.83188E+00_JPRB ,&
&  0.85474E+00_JPRB, 0.87695E+00_JPRB, 0.90352E+00_JPRB, 0.92402E+00_JPRB, 0.94556E+00_JPRB, 0.97218E+00_JPRB, 0.10017E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10741E+01_JPRB, 0.11214E+01_JPRB, 0.11524E+01_JPRB, 0.11863E+01_JPRB, 0.12165E+01_JPRB, 0.12553E+01_JPRB ,&
&  0.12854E+01_JPRB, 0.13293E+01_JPRB, 0.13404E+01_JPRB, 0.13537E+01_JPRB, 0.13340E+01_JPRB, 0.13136E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.12970E+01_JPRB, 0.13078E+01_JPRB, 0.12928E+01_JPRB, 0.12822E+01_JPRB, 0.12743E+01_JPRB, 0.12690E+01_JPRB, 0.12605E+01_JPRB ,&
&  0.12456E+01_JPRB, 0.12219E+01_JPRB, 0.11912E+01_JPRB, 0.11526E+01_JPRB, 0.11177E+01_JPRB, 0.10822E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.96682E+00_JPRB, 0.90900E+00_JPRB, 0.87480E+00_JPRB, 0.85395E+00_JPRB, 0.84762E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.85403E+00_JPRB, 0.84713E+00_JPRB, 0.84621E+00_JPRB, 0.84181E+00_JPRB, 0.84115E+00_JPRB, 0.83796E+00_JPRB, 0.83637E+00_JPRB ,&
&  0.83336E+00_JPRB, 0.83158E+00_JPRB, 0.82902E+00_JPRB, 0.82814E+00_JPRB, 0.82703E+00_JPRB, 0.82703E+00_JPRB, 0.82651E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.82542E+00_JPRB, 0.82489E+00_JPRB, 0.82319E+00_JPRB, 0.82236E+00_JPRB, 0.82054E+00_JPRB, 0.81437E+00_JPRB ,&
&  0.80512E+00_JPRB, 0.78774E+00_JPRB, 0.75495E+00_JPRB, 0.74949E+00_JPRB, 0.74229E+00_JPRB, 0.74133E+00_JPRB, 0.73731E+00_JPRB ,&
&  0.72872E+00_JPRB, 0.66892E+00_JPRB, 0.67478E+00_JPRB, 0.66057E+00_JPRB, 0.65033E+00_JPRB, 0.63999E+00_JPRB, 0.62359E+00_JPRB ,&
&  0.61752E+00_JPRB, 0.61096E+00_JPRB, 0.60973E+00_JPRB, 0.61628E+00_JPRB, 0.62426E+00_JPRB, 0.63849E+00_JPRB, 0.65606E+00_JPRB ,&
&  0.66057E+00_JPRB, 0.69261E+00_JPRB, 0.70796E+00_JPRB, 0.71672E+00_JPRB, 0.74028E+00_JPRB, 0.75816E+00_JPRB, 0.76077E+00_JPRB ,&
&  0.76608E+00_JPRB, 0.76545E+00_JPRB, 0.77397E+00_JPRB, 0.78643E+00_JPRB, 0.80050E+00_JPRB, 0.80708E+00_JPRB, 0.81484E+00_JPRB ,&
&  0.82246E+00_JPRB, 0.83018E+00_JPRB, 0.83877E+00_JPRB, 0.84457E+00_JPRB, 0.85488E+00_JPRB, 0.87316E+00_JPRB, 0.89637E+00_JPRB ,&
&  0.92426E+00_JPRB, 0.95498E+00_JPRB, 0.98864E+00_JPRB, 0.10228E+01_JPRB, 0.10517E+01_JPRB, 0.10797E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11481E+01_JPRB, 0.11607E+01_JPRB, 0.11706E+01_JPRB, 0.11730E+01_JPRB, 0.11691E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11834E+01_JPRB, 0.11894E+01_JPRB, 0.11920E+01_JPRB, 0.11920E+01_JPRB, 0.11930E+01_JPRB, 0.11901E+01_JPRB, 0.11834E+01_JPRB ,&
&  0.11728E+01_JPRB, 0.11578E+01_JPRB, 0.11385E+01_JPRB, 0.11097E+01_JPRB, 0.10785E+01_JPRB, 0.10527E+01_JPRB, 0.10215E+01_JPRB ,&
&  0.98710E+00_JPRB, 0.94595E+00_JPRB, 0.90773E+00_JPRB, 0.87751E+00_JPRB, 0.86280E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.87457E+00_JPRB, 0.86846E+00_JPRB, 0.86571E+00_JPRB, 0.86204E+00_JPRB, 0.86059E+00_JPRB, 0.85730E+00_JPRB, 0.85563E+00_JPRB ,&
&  0.85217E+00_JPRB, 0.84975E+00_JPRB, 0.84723E+00_JPRB, 0.84505E+00_JPRB, 0.84277E+00_JPRB, 0.84006E+00_JPRB, 0.83766E+00_JPRB ,&
&  0.83604E+00_JPRB, 0.83390E+00_JPRB, 0.83340E+00_JPRB, 0.83180E+00_JPRB, 0.82895E+00_JPRB, 0.82558E+00_JPRB, 0.81407E+00_JPRB ,&
&  0.80152E+00_JPRB, 0.78881E+00_JPRB, 0.74109E+00_JPRB, 0.73600E+00_JPRB, 0.72631E+00_JPRB, 0.72049E+00_JPRB, 0.72144E+00_JPRB ,&
&  0.71450E+00_JPRB, 0.70747E+00_JPRB, 0.68729E+00_JPRB, 0.66743E+00_JPRB, 0.65187E+00_JPRB, 0.63302E+00_JPRB, 0.62876E+00_JPRB ,&
&  0.62415E+00_JPRB, 0.62815E+00_JPRB, 0.63340E+00_JPRB, 0.63914E+00_JPRB, 0.65281E+00_JPRB, 0.66326E+00_JPRB, 0.67887E+00_JPRB ,&
&  0.69794E+00_JPRB, 0.71306E+00_JPRB, 0.71869E+00_JPRB, 0.71682E+00_JPRB, 0.73927E+00_JPRB, 0.75389E+00_JPRB, 0.76607E+00_JPRB ,&
&  0.77290E+00_JPRB, 0.77434E+00_JPRB, 0.77990E+00_JPRB, 0.78247E+00_JPRB, 0.78827E+00_JPRB, 0.79741E+00_JPRB, 0.80431E+00_JPRB ,&
&  0.81118E+00_JPRB, 0.81484E+00_JPRB, 0.81721E+00_JPRB, 0.81787E+00_JPRB, 0.81615E+00_JPRB, 0.81666E+00_JPRB, 0.82184E+00_JPRB ,&
&  0.83169E+00_JPRB, 0.84717E+00_JPRB, 0.86647E+00_JPRB, 0.88852E+00_JPRB, 0.91064E+00_JPRB, 0.93170E+00_JPRB, 0.94940E+00_JPRB ,&
&  0.96611E+00_JPRB, 0.97925E+00_JPRB, 0.99612E+00_JPRB, 0.99979E+00_JPRB, 0.10120E+01_JPRB, 0.10365E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10640E+01_JPRB, 0.10726E+01_JPRB, 0.10793E+01_JPRB, 0.10837E+01_JPRB, 0.10849E+01_JPRB, 0.10845E+01_JPRB ,&
&  0.10815E+01_JPRB, 0.10728E+01_JPRB, 0.10602E+01_JPRB, 0.10442E+01_JPRB, 0.10291E+01_JPRB, 0.10137E+01_JPRB, 0.99625E+00_JPRB ,&
&  0.97712E+00_JPRB, 0.95537E+00_JPRB, 0.92893E+00_JPRB, 0.90232E+00_JPRB, 0.88584E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.89701E+00_JPRB, 0.88894E+00_JPRB, 0.88257E+00_JPRB, 0.87797E+00_JPRB, 0.87410E+00_JPRB, 0.87071E+00_JPRB, 0.86810E+00_JPRB ,&
&  0.86512E+00_JPRB, 0.86175E+00_JPRB, 0.85785E+00_JPRB, 0.85550E+00_JPRB, 0.85277E+00_JPRB, 0.85059E+00_JPRB, 0.84696E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.83988E+00_JPRB, 0.83770E+00_JPRB, 0.83553E+00_JPRB, 0.83269E+00_JPRB, 0.83012E+00_JPRB, 0.81976E+00_JPRB ,&
&  0.81000E+00_JPRB, 0.80136E+00_JPRB, 0.79058E+00_JPRB, 0.78100E+00_JPRB, 0.77406E+00_JPRB, 0.73282E+00_JPRB, 0.71946E+00_JPRB ,&
&  0.72867E+00_JPRB, 0.73549E+00_JPRB, 0.74299E+00_JPRB, 0.74892E+00_JPRB, 0.74065E+00_JPRB, 0.72234E+00_JPRB, 0.71066E+00_JPRB ,&
&  0.67555E+00_JPRB, 0.68128E+00_JPRB, 0.68607E+00_JPRB, 0.69072E+00_JPRB, 0.69917E+00_JPRB, 0.70398E+00_JPRB, 0.71406E+00_JPRB ,&
&  0.73866E+00_JPRB, 0.74426E+00_JPRB, 0.75163E+00_JPRB, 0.75756E+00_JPRB, 0.75942E+00_JPRB, 0.76212E+00_JPRB, 0.76874E+00_JPRB ,&
&  0.77156E+00_JPRB, 0.77444E+00_JPRB, 0.77680E+00_JPRB, 0.77741E+00_JPRB, 0.78065E+00_JPRB, 0.78331E+00_JPRB, 0.78530E+00_JPRB ,&
&  0.78504E+00_JPRB, 0.78577E+00_JPRB, 0.78654E+00_JPRB, 0.78722E+00_JPRB, 0.78839E+00_JPRB, 0.78778E+00_JPRB, 0.78881E+00_JPRB ,&
&  0.79090E+00_JPRB, 0.79563E+00_JPRB, 0.80163E+00_JPRB, 0.80983E+00_JPRB, 0.82060E+00_JPRB, 0.83555E+00_JPRB, 0.85129E+00_JPRB ,&
&  0.86933E+00_JPRB, 0.88664E+00_JPRB, 0.90266E+00_JPRB, 0.91613E+00_JPRB, 0.92808E+00_JPRB, 0.93967E+00_JPRB, 0.94942E+00_JPRB ,&
&  0.95991E+00_JPRB, 0.97039E+00_JPRB, 0.97694E+00_JPRB, 0.97963E+00_JPRB, 0.98576E+00_JPRB, 0.99134E+00_JPRB, 0.99405E+00_JPRB ,&
&  0.99496E+00_JPRB, 0.99483E+00_JPRB, 0.99216E+00_JPRB, 0.98717E+00_JPRB, 0.98222E+00_JPRB, 0.97609E+00_JPRB, 0.96839E+00_JPRB ,&
&  0.95994E+00_JPRB, 0.95121E+00_JPRB, 0.93929E+00_JPRB, 0.92460E+00_JPRB, 0.91020E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.90703E+00_JPRB, 0.89989E+00_JPRB, 0.89292E+00_JPRB, 0.88676E+00_JPRB, 0.88048E+00_JPRB, 0.87580E+00_JPRB, 0.87156E+00_JPRB ,&
&  0.86770E+00_JPRB, 0.86369E+00_JPRB, 0.86005E+00_JPRB, 0.85695E+00_JPRB, 0.85461E+00_JPRB, 0.85260E+00_JPRB, 0.84986E+00_JPRB ,&
&  0.84713E+00_JPRB, 0.84385E+00_JPRB, 0.84096E+00_JPRB, 0.83835E+00_JPRB, 0.83492E+00_JPRB, 0.83090E+00_JPRB, 0.82570E+00_JPRB ,&
&  0.82170E+00_JPRB, 0.81736E+00_JPRB, 0.81284E+00_JPRB, 0.80664E+00_JPRB, 0.80152E+00_JPRB, 0.79648E+00_JPRB, 0.79149E+00_JPRB ,&
&  0.78602E+00_JPRB, 0.78160E+00_JPRB, 0.77772E+00_JPRB, 0.77408E+00_JPRB, 0.77643E+00_JPRB, 0.77771E+00_JPRB, 0.77823E+00_JPRB ,&
&  0.77756E+00_JPRB, 0.77419E+00_JPRB, 0.77288E+00_JPRB, 0.77191E+00_JPRB, 0.77107E+00_JPRB, 0.77445E+00_JPRB, 0.77666E+00_JPRB ,&
&  0.77799E+00_JPRB, 0.77818E+00_JPRB, 0.78004E+00_JPRB, 0.78089E+00_JPRB, 0.78046E+00_JPRB, 0.77865E+00_JPRB, 0.77611E+00_JPRB ,&
&  0.77566E+00_JPRB, 0.77521E+00_JPRB, 0.77445E+00_JPRB, 0.77302E+00_JPRB, 0.77357E+00_JPRB, 0.77435E+00_JPRB, 0.77446E+00_JPRB ,&
&  0.77369E+00_JPRB, 0.77380E+00_JPRB, 0.77380E+00_JPRB, 0.77391E+00_JPRB, 0.77409E+00_JPRB, 0.77551E+00_JPRB, 0.77726E+00_JPRB ,&
&  0.77947E+00_JPRB, 0.78251E+00_JPRB, 0.78654E+00_JPRB, 0.79112E+00_JPRB, 0.79643E+00_JPRB, 0.80359E+00_JPRB, 0.81271E+00_JPRB ,&
&  0.82363E+00_JPRB, 0.83594E+00_JPRB, 0.84896E+00_JPRB, 0.85977E+00_JPRB, 0.86858E+00_JPRB, 0.87624E+00_JPRB, 0.88424E+00_JPRB ,&
&  0.89137E+00_JPRB, 0.89779E+00_JPRB, 0.90440E+00_JPRB, 0.91120E+00_JPRB, 0.91802E+00_JPRB, 0.92348E+00_JPRB, 0.92764E+00_JPRB ,&
&  0.93091E+00_JPRB, 0.93348E+00_JPRB, 0.93577E+00_JPRB, 0.93815E+00_JPRB, 0.93989E+00_JPRB, 0.93956E+00_JPRB, 0.93798E+00_JPRB ,&
&  0.93550E+00_JPRB, 0.93245E+00_JPRB, 0.92817E+00_JPRB, 0.92191E+00_JPRB, 0.91494E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.88415E+00_JPRB, 0.88171E+00_JPRB, 0.87914E+00_JPRB, 0.87657E+00_JPRB, 0.87376E+00_JPRB, 0.87104E+00_JPRB, 0.86828E+00_JPRB ,&
&  0.86582E+00_JPRB, 0.86323E+00_JPRB, 0.86093E+00_JPRB, 0.85867E+00_JPRB, 0.85673E+00_JPRB, 0.85468E+00_JPRB, 0.85282E+00_JPRB ,&
&  0.85085E+00_JPRB, 0.84898E+00_JPRB, 0.84645E+00_JPRB, 0.84425E+00_JPRB, 0.84181E+00_JPRB, 0.83951E+00_JPRB, 0.83662E+00_JPRB ,&
&  0.83442E+00_JPRB, 0.83184E+00_JPRB, 0.82942E+00_JPRB, 0.82620E+00_JPRB, 0.82371E+00_JPRB, 0.82116E+00_JPRB, 0.81893E+00_JPRB ,&
&  0.81696E+00_JPRB, 0.81551E+00_JPRB, 0.81412E+00_JPRB, 0.81261E+00_JPRB, 0.81196E+00_JPRB, 0.81207E+00_JPRB, 0.81228E+00_JPRB ,&
&  0.81236E+00_JPRB, 0.81273E+00_JPRB, 0.81337E+00_JPRB, 0.81367E+00_JPRB, 0.81307E+00_JPRB, 0.81248E+00_JPRB, 0.81201E+00_JPRB ,&
&  0.81121E+00_JPRB, 0.80985E+00_JPRB, 0.80836E+00_JPRB, 0.80710E+00_JPRB, 0.80568E+00_JPRB, 0.80385E+00_JPRB, 0.80218E+00_JPRB ,&
&  0.80162E+00_JPRB, 0.80111E+00_JPRB, 0.80046E+00_JPRB, 0.79978E+00_JPRB, 0.79979E+00_JPRB, 0.79982E+00_JPRB, 0.79980E+00_JPRB ,&
&  0.79990E+00_JPRB, 0.80044E+00_JPRB, 0.80071E+00_JPRB, 0.80086E+00_JPRB, 0.80125E+00_JPRB, 0.80251E+00_JPRB, 0.80407E+00_JPRB ,&
&  0.80594E+00_JPRB, 0.80842E+00_JPRB, 0.81153E+00_JPRB, 0.81462E+00_JPRB, 0.81790E+00_JPRB, 0.82157E+00_JPRB, 0.82552E+00_JPRB ,&
&  0.82959E+00_JPRB, 0.83381E+00_JPRB, 0.83848E+00_JPRB, 0.84316E+00_JPRB, 0.84767E+00_JPRB, 0.85216E+00_JPRB, 0.85709E+00_JPRB ,&
&  0.86131E+00_JPRB, 0.86508E+00_JPRB, 0.86843E+00_JPRB, 0.87184E+00_JPRB, 0.87513E+00_JPRB, 0.87820E+00_JPRB, 0.88084E+00_JPRB ,&
&  0.88316E+00_JPRB, 0.88512E+00_JPRB, 0.88685E+00_JPRB, 0.88864E+00_JPRB, 0.88930E+00_JPRB, 0.88957E+00_JPRB, 0.88971E+00_JPRB ,&
&  0.88963E+00_JPRB, 0.88910E+00_JPRB, 0.88831E+00_JPRB, 0.88732E+00_JPRB, 0.88609E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB ,&
&  0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB, 0.84203E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB ,&
&  0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB, 0.18001E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.19449E+00_JPRB, 0.19466E+00_JPRB, 0.19511E+00_JPRB, 0.19564E+00_JPRB, 0.19616E+00_JPRB, 0.19690E+00_JPRB, 0.19713E+00_JPRB ,&
&  0.19716E+00_JPRB, 0.19706E+00_JPRB, 0.19739E+00_JPRB, 0.19777E+00_JPRB, 0.19800E+00_JPRB, 0.19800E+00_JPRB, 0.19760E+00_JPRB ,&
&  0.19693E+00_JPRB, 0.19612E+00_JPRB, 0.19490E+00_JPRB, 0.19435E+00_JPRB, 0.19395E+00_JPRB, 0.19355E+00_JPRB, 0.19277E+00_JPRB ,&
&  0.19208E+00_JPRB, 0.19087E+00_JPRB, 0.18920E+00_JPRB, 0.18790E+00_JPRB, 0.18766E+00_JPRB, 0.18789E+00_JPRB, 0.18847E+00_JPRB ,&
&  0.18892E+00_JPRB, 0.18922E+00_JPRB, 0.18931E+00_JPRB, 0.18947E+00_JPRB, 0.18971E+00_JPRB, 0.18957E+00_JPRB, 0.18931E+00_JPRB ,&
&  0.18899E+00_JPRB, 0.18846E+00_JPRB, 0.18795E+00_JPRB, 0.18749E+00_JPRB, 0.18707E+00_JPRB, 0.18658E+00_JPRB, 0.18600E+00_JPRB ,&
&  0.18533E+00_JPRB, 0.18450E+00_JPRB, 0.18352E+00_JPRB, 0.18273E+00_JPRB, 0.18191E+00_JPRB, 0.18126E+00_JPRB, 0.18046E+00_JPRB ,&
&  0.17980E+00_JPRB, 0.17919E+00_JPRB, 0.17857E+00_JPRB, 0.17799E+00_JPRB, 0.17745E+00_JPRB, 0.17693E+00_JPRB, 0.17641E+00_JPRB ,&
&  0.17591E+00_JPRB, 0.17551E+00_JPRB, 0.17505E+00_JPRB, 0.17463E+00_JPRB, 0.17433E+00_JPRB, 0.17409E+00_JPRB, 0.17389E+00_JPRB ,&
&  0.17371E+00_JPRB, 0.17366E+00_JPRB, 0.17364E+00_JPRB, 0.17361E+00_JPRB, 0.17361E+00_JPRB, 0.17372E+00_JPRB, 0.17383E+00_JPRB ,&
&  0.17399E+00_JPRB, 0.17410E+00_JPRB, 0.17423E+00_JPRB, 0.17441E+00_JPRB, 0.17461E+00_JPRB, 0.17484E+00_JPRB, 0.17515E+00_JPRB ,&
&  0.17554E+00_JPRB, 0.17601E+00_JPRB, 0.17650E+00_JPRB, 0.17681E+00_JPRB, 0.17747E+00_JPRB, 0.17821E+00_JPRB, 0.17908E+00_JPRB ,&
&  0.18004E+00_JPRB, 0.18117E+00_JPRB, 0.18239E+00_JPRB, 0.18381E+00_JPRB, 0.18526E+00_JPRB, 0.18620E+00_JPRB, 0.18714E+00_JPRB ,&
&  0.18813E+00_JPRB, 0.18929E+00_JPRB, 0.19039E+00_JPRB, 0.19152E+00_JPRB, 0.19289E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.21582E+00_JPRB, 0.22258E+00_JPRB, 0.22936E+00_JPRB, 0.23900E+00_JPRB, 0.24565E+00_JPRB, 0.28111E+00_JPRB, 0.27600E+00_JPRB ,&
&  0.27260E+00_JPRB, 0.25899E+00_JPRB, 0.25750E+00_JPRB, 0.25390E+00_JPRB, 0.24939E+00_JPRB, 0.24450E+00_JPRB, 0.23957E+00_JPRB ,&
&  0.23494E+00_JPRB, 0.22951E+00_JPRB, 0.22484E+00_JPRB, 0.22118E+00_JPRB, 0.21790E+00_JPRB, 0.21468E+00_JPRB, 0.21187E+00_JPRB ,&
&  0.21067E+00_JPRB, 0.21052E+00_JPRB, 0.20975E+00_JPRB, 0.20567E+00_JPRB, 0.20224E+00_JPRB, 0.20013E+00_JPRB, 0.19947E+00_JPRB ,&
&  0.19938E+00_JPRB, 0.20103E+00_JPRB, 0.20204E+00_JPRB, 0.20288E+00_JPRB, 0.20413E+00_JPRB, 0.20331E+00_JPRB, 0.20248E+00_JPRB ,&
&  0.20130E+00_JPRB, 0.19997E+00_JPRB, 0.19859E+00_JPRB, 0.19738E+00_JPRB, 0.19618E+00_JPRB, 0.19439E+00_JPRB, 0.19285E+00_JPRB ,&
&  0.19132E+00_JPRB, 0.18936E+00_JPRB, 0.18675E+00_JPRB, 0.18502E+00_JPRB, 0.18301E+00_JPRB, 0.18129E+00_JPRB, 0.17957E+00_JPRB ,&
&  0.17829E+00_JPRB, 0.17712E+00_JPRB, 0.17575E+00_JPRB, 0.17408E+00_JPRB, 0.17242E+00_JPRB, 0.17096E+00_JPRB, 0.16957E+00_JPRB ,&
&  0.16881E+00_JPRB, 0.16803E+00_JPRB, 0.16725E+00_JPRB, 0.16642E+00_JPRB, 0.16601E+00_JPRB, 0.16559E+00_JPRB, 0.16520E+00_JPRB ,&
&  0.16488E+00_JPRB, 0.16471E+00_JPRB, 0.16474E+00_JPRB, 0.16482E+00_JPRB, 0.16492E+00_JPRB, 0.16529E+00_JPRB, 0.16572E+00_JPRB ,&
&  0.16631E+00_JPRB, 0.16705E+00_JPRB, 0.16805E+00_JPRB, 0.16909E+00_JPRB, 0.17003E+00_JPRB, 0.17078E+00_JPRB, 0.17175E+00_JPRB ,&
&  0.17332E+00_JPRB, 0.17514E+00_JPRB, 0.17712E+00_JPRB, 0.17864E+00_JPRB, 0.18079E+00_JPRB, 0.18274E+00_JPRB, 0.18468E+00_JPRB ,&
&  0.18660E+00_JPRB, 0.18887E+00_JPRB, 0.19141E+00_JPRB, 0.19413E+00_JPRB, 0.19728E+00_JPRB, 0.20003E+00_JPRB, 0.20259E+00_JPRB ,&
&  0.20520E+00_JPRB, 0.20889E+00_JPRB, 0.21093E+00_JPRB, 0.21213E+00_JPRB, 0.21367E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.28926E+00_JPRB, 0.29413E+00_JPRB, 0.29787E+00_JPRB, 0.30026E+00_JPRB, 0.30253E+00_JPRB, 0.30414E+00_JPRB, 0.30471E+00_JPRB ,&
&  0.29826E+00_JPRB, 0.29327E+00_JPRB, 0.28630E+00_JPRB, 0.27794E+00_JPRB, 0.27031E+00_JPRB, 0.26329E+00_JPRB, 0.25527E+00_JPRB ,&
&  0.24871E+00_JPRB, 0.24252E+00_JPRB, 0.23605E+00_JPRB, 0.22926E+00_JPRB, 0.22215E+00_JPRB, 0.21499E+00_JPRB, 0.21152E+00_JPRB ,&
&  0.21051E+00_JPRB, 0.21161E+00_JPRB, 0.21360E+00_JPRB, 0.21517E+00_JPRB, 0.21612E+00_JPRB, 0.21674E+00_JPRB, 0.21560E+00_JPRB ,&
&  0.21190E+00_JPRB, 0.21255E+00_JPRB, 0.21346E+00_JPRB, 0.21405E+00_JPRB, 0.21557E+00_JPRB, 0.21692E+00_JPRB, 0.21674E+00_JPRB ,&
&  0.21495E+00_JPRB, 0.21254E+00_JPRB, 0.20924E+00_JPRB, 0.20615E+00_JPRB, 0.20498E+00_JPRB, 0.20281E+00_JPRB, 0.20035E+00_JPRB ,&
&  0.19837E+00_JPRB, 0.19451E+00_JPRB, 0.19057E+00_JPRB, 0.18779E+00_JPRB, 0.18416E+00_JPRB, 0.18168E+00_JPRB, 0.17843E+00_JPRB ,&
&  0.17581E+00_JPRB, 0.17344E+00_JPRB, 0.17085E+00_JPRB, 0.16883E+00_JPRB, 0.16695E+00_JPRB, 0.16556E+00_JPRB, 0.16417E+00_JPRB ,&
&  0.16336E+00_JPRB, 0.16260E+00_JPRB, 0.16212E+00_JPRB, 0.16157E+00_JPRB, 0.16126E+00_JPRB, 0.16094E+00_JPRB, 0.16069E+00_JPRB ,&
&  0.16056E+00_JPRB, 0.16089E+00_JPRB, 0.16112E+00_JPRB, 0.16154E+00_JPRB, 0.16203E+00_JPRB, 0.16232E+00_JPRB, 0.16286E+00_JPRB ,&
&  0.16389E+00_JPRB, 0.16500E+00_JPRB, 0.16572E+00_JPRB, 0.16667E+00_JPRB, 0.16774E+00_JPRB, 0.16903E+00_JPRB, 0.17080E+00_JPRB ,&
&  0.17265E+00_JPRB, 0.17472E+00_JPRB, 0.17705E+00_JPRB, 0.17935E+00_JPRB, 0.18230E+00_JPRB, 0.18533E+00_JPRB, 0.18853E+00_JPRB ,&
&  0.19234E+00_JPRB, 0.19640E+00_JPRB, 0.20058E+00_JPRB, 0.20519E+00_JPRB, 0.21186E+00_JPRB, 0.21789E+00_JPRB, 0.22534E+00_JPRB ,&
&  0.23259E+00_JPRB, 0.23824E+00_JPRB, 0.27024E+00_JPRB, 0.27503E+00_JPRB, 0.28175E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.29711E+00_JPRB, 0.30799E+00_JPRB, 0.31673E+00_JPRB, 0.32220E+00_JPRB, 0.32778E+00_JPRB, 0.32994E+00_JPRB, 0.33814E+00_JPRB ,&
&  0.33264E+00_JPRB, 0.31458E+00_JPRB, 0.30342E+00_JPRB, 0.28698E+00_JPRB, 0.26669E+00_JPRB, 0.25011E+00_JPRB, 0.24178E+00_JPRB ,&
&  0.23685E+00_JPRB, 0.23294E+00_JPRB, 0.22821E+00_JPRB, 0.22302E+00_JPRB, 0.21779E+00_JPRB, 0.21391E+00_JPRB, 0.21141E+00_JPRB ,&
&  0.20914E+00_JPRB, 0.20975E+00_JPRB, 0.21233E+00_JPRB, 0.21667E+00_JPRB, 0.22282E+00_JPRB, 0.22711E+00_JPRB, 0.23048E+00_JPRB ,&
&  0.23268E+00_JPRB, 0.22381E+00_JPRB, 0.22543E+00_JPRB, 0.22739E+00_JPRB, 0.22607E+00_JPRB, 0.22902E+00_JPRB, 0.23380E+00_JPRB ,&
&  0.23154E+00_JPRB, 0.22478E+00_JPRB, 0.22745E+00_JPRB, 0.22258E+00_JPRB, 0.21420E+00_JPRB, 0.21235E+00_JPRB, 0.20658E+00_JPRB ,&
&  0.20364E+00_JPRB, 0.19649E+00_JPRB, 0.19141E+00_JPRB, 0.18805E+00_JPRB, 0.18258E+00_JPRB, 0.18032E+00_JPRB, 0.17691E+00_JPRB ,&
&  0.17342E+00_JPRB, 0.17098E+00_JPRB, 0.16808E+00_JPRB, 0.16666E+00_JPRB, 0.16544E+00_JPRB, 0.16420E+00_JPRB, 0.16321E+00_JPRB ,&
&  0.16266E+00_JPRB, 0.16214E+00_JPRB, 0.16217E+00_JPRB, 0.16220E+00_JPRB, 0.16222E+00_JPRB, 0.16255E+00_JPRB, 0.16275E+00_JPRB ,&
&  0.16339E+00_JPRB, 0.16414E+00_JPRB, 0.16475E+00_JPRB, 0.16573E+00_JPRB, 0.16616E+00_JPRB, 0.16673E+00_JPRB, 0.16715E+00_JPRB ,&
&  0.16760E+00_JPRB, 0.16827E+00_JPRB, 0.16862E+00_JPRB, 0.16930E+00_JPRB, 0.17008E+00_JPRB, 0.17157E+00_JPRB, 0.17305E+00_JPRB ,&
&  0.17484E+00_JPRB, 0.17693E+00_JPRB, 0.17915E+00_JPRB, 0.18150E+00_JPRB, 0.18440E+00_JPRB, 0.18706E+00_JPRB, 0.19056E+00_JPRB ,&
&  0.19436E+00_JPRB, 0.19798E+00_JPRB, 0.20221E+00_JPRB, 0.20756E+00_JPRB, 0.21464E+00_JPRB, 0.22426E+00_JPRB, 0.23317E+00_JPRB ,&
&  0.24338E+00_JPRB, 0.26950E+00_JPRB, 0.27582E+00_JPRB, 0.27997E+00_JPRB, 0.28853E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.32396E+00_JPRB, 0.33351E+00_JPRB, 0.34757E+00_JPRB, 0.35665E+00_JPRB, 0.36387E+00_JPRB, 0.35972E+00_JPRB, 0.35578E+00_JPRB ,&
&  0.35168E+00_JPRB, 0.33759E+00_JPRB, 0.32043E+00_JPRB, 0.29124E+00_JPRB, 0.26665E+00_JPRB, 0.24785E+00_JPRB, 0.23746E+00_JPRB ,&
&  0.23124E+00_JPRB, 0.22917E+00_JPRB, 0.22719E+00_JPRB, 0.22667E+00_JPRB, 0.22545E+00_JPRB, 0.22702E+00_JPRB, 0.22835E+00_JPRB ,&
&  0.22793E+00_JPRB, 0.22687E+00_JPRB, 0.22629E+00_JPRB, 0.22942E+00_JPRB, 0.23547E+00_JPRB, 0.24022E+00_JPRB, 0.24480E+00_JPRB ,&
&  0.24812E+00_JPRB, 0.24555E+00_JPRB, 0.24677E+00_JPRB, 0.24485E+00_JPRB, 0.24282E+00_JPRB, 0.24307E+00_JPRB, 0.24402E+00_JPRB ,&
&  0.24445E+00_JPRB, 0.24493E+00_JPRB, 0.24419E+00_JPRB, 0.24117E+00_JPRB, 0.22948E+00_JPRB, 0.22347E+00_JPRB, 0.21573E+00_JPRB ,&
&  0.20769E+00_JPRB, 0.19833E+00_JPRB, 0.19078E+00_JPRB, 0.18714E+00_JPRB, 0.18282E+00_JPRB, 0.18114E+00_JPRB, 0.17799E+00_JPRB ,&
&  0.17439E+00_JPRB, 0.17098E+00_JPRB, 0.16871E+00_JPRB, 0.16755E+00_JPRB, 0.16711E+00_JPRB, 0.16603E+00_JPRB, 0.16563E+00_JPRB ,&
&  0.16589E+00_JPRB, 0.16619E+00_JPRB, 0.16704E+00_JPRB, 0.16786E+00_JPRB, 0.16879E+00_JPRB, 0.16962E+00_JPRB, 0.17064E+00_JPRB ,&
&  0.17226E+00_JPRB, 0.17426E+00_JPRB, 0.17518E+00_JPRB, 0.17598E+00_JPRB, 0.17574E+00_JPRB, 0.17527E+00_JPRB, 0.17455E+00_JPRB ,&
&  0.17396E+00_JPRB, 0.17379E+00_JPRB, 0.17380E+00_JPRB, 0.17375E+00_JPRB, 0.17422E+00_JPRB, 0.17512E+00_JPRB, 0.17605E+00_JPRB ,&
&  0.17761E+00_JPRB, 0.17901E+00_JPRB, 0.18060E+00_JPRB, 0.18248E+00_JPRB, 0.18463E+00_JPRB, 0.18718E+00_JPRB, 0.19002E+00_JPRB ,&
&  0.19258E+00_JPRB, 0.19583E+00_JPRB, 0.19855E+00_JPRB, 0.20071E+00_JPRB, 0.20585E+00_JPRB, 0.21292E+00_JPRB, 0.22711E+00_JPRB ,&
&  0.24348E+00_JPRB, 0.28333E+00_JPRB, 0.32547E+00_JPRB, 0.31087E+00_JPRB, 0.31589E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.32349E+00_JPRB, 0.33550E+00_JPRB, 0.35077E+00_JPRB, 0.36021E+00_JPRB, 0.36550E+00_JPRB, 0.36815E+00_JPRB, 0.36703E+00_JPRB ,&
&  0.36363E+00_JPRB, 0.36310E+00_JPRB, 0.37418E+00_JPRB, 0.35626E+00_JPRB, 0.33856E+00_JPRB, 0.30535E+00_JPRB, 0.28891E+00_JPRB ,&
&  0.27658E+00_JPRB, 0.27447E+00_JPRB, 0.27423E+00_JPRB, 0.27895E+00_JPRB, 0.28285E+00_JPRB, 0.28591E+00_JPRB, 0.28221E+00_JPRB ,&
&  0.27693E+00_JPRB, 0.26775E+00_JPRB, 0.26005E+00_JPRB, 0.25283E+00_JPRB, 0.25296E+00_JPRB, 0.25186E+00_JPRB, 0.24929E+00_JPRB ,&
&  0.24633E+00_JPRB, 0.24446E+00_JPRB, 0.24435E+00_JPRB, 0.24461E+00_JPRB, 0.25625E+00_JPRB, 0.25555E+00_JPRB, 0.25558E+00_JPRB ,&
&  0.25521E+00_JPRB, 0.25516E+00_JPRB, 0.25418E+00_JPRB, 0.25247E+00_JPRB, 0.24779E+00_JPRB, 0.23493E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.22501E+00_JPRB, 0.21444E+00_JPRB, 0.20423E+00_JPRB, 0.19713E+00_JPRB, 0.19283E+00_JPRB, 0.19095E+00_JPRB, 0.18632E+00_JPRB ,&
&  0.18136E+00_JPRB, 0.17567E+00_JPRB, 0.17301E+00_JPRB, 0.17274E+00_JPRB, 0.17329E+00_JPRB, 0.17384E+00_JPRB, 0.17430E+00_JPRB ,&
&  0.17551E+00_JPRB, 0.17609E+00_JPRB, 0.17641E+00_JPRB, 0.17706E+00_JPRB, 0.17770E+00_JPRB, 0.17870E+00_JPRB, 0.18045E+00_JPRB ,&
&  0.18358E+00_JPRB, 0.18696E+00_JPRB, 0.19011E+00_JPRB, 0.19271E+00_JPRB, 0.19175E+00_JPRB, 0.18850E+00_JPRB, 0.18416E+00_JPRB ,&
&  0.18103E+00_JPRB, 0.18000E+00_JPRB, 0.17906E+00_JPRB, 0.17902E+00_JPRB, 0.17922E+00_JPRB, 0.17967E+00_JPRB, 0.18030E+00_JPRB ,&
&  0.18114E+00_JPRB, 0.18184E+00_JPRB, 0.18270E+00_JPRB, 0.18381E+00_JPRB, 0.18538E+00_JPRB, 0.18681E+00_JPRB, 0.18880E+00_JPRB ,&
&  0.19178E+00_JPRB, 0.19504E+00_JPRB, 0.19963E+00_JPRB, 0.20374E+00_JPRB, 0.20930E+00_JPRB, 0.21741E+00_JPRB, 0.23075E+00_JPRB ,&
&  0.24516E+00_JPRB, 0.27938E+00_JPRB, 0.29509E+00_JPRB, 0.30587E+00_JPRB, 0.31328E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.32281E+00_JPRB, 0.33165E+00_JPRB, 0.34362E+00_JPRB, 0.35395E+00_JPRB, 0.36291E+00_JPRB, 0.37142E+00_JPRB, 0.37878E+00_JPRB ,&
&  0.38273E+00_JPRB, 0.38500E+00_JPRB, 0.38583E+00_JPRB, 0.38932E+00_JPRB, 0.38633E+00_JPRB, 0.37958E+00_JPRB, 0.37411E+00_JPRB ,&
&  0.36959E+00_JPRB, 0.36605E+00_JPRB, 0.36082E+00_JPRB, 0.35808E+00_JPRB, 0.35785E+00_JPRB, 0.35797E+00_JPRB, 0.35234E+00_JPRB ,&
&  0.34194E+00_JPRB, 0.34105E+00_JPRB, 0.34576E+00_JPRB, 0.34042E+00_JPRB, 0.33477E+00_JPRB, 0.32941E+00_JPRB, 0.32359E+00_JPRB ,&
&  0.30779E+00_JPRB, 0.28785E+00_JPRB, 0.26088E+00_JPRB, 0.25668E+00_JPRB, 0.27251E+00_JPRB, 0.26629E+00_JPRB, 0.26936E+00_JPRB ,&
&  0.26989E+00_JPRB, 0.27020E+00_JPRB, 0.26871E+00_JPRB, 0.26718E+00_JPRB, 0.26368E+00_JPRB, 0.26251E+00_JPRB, 0.26108E+00_JPRB ,&
&  0.25558E+00_JPRB, 0.25067E+00_JPRB, 0.24281E+00_JPRB, 0.22518E+00_JPRB, 0.22004E+00_JPRB, 0.21827E+00_JPRB, 0.21338E+00_JPRB ,&
&  0.20572E+00_JPRB, 0.19999E+00_JPRB, 0.19763E+00_JPRB, 0.19393E+00_JPRB, 0.19603E+00_JPRB, 0.19787E+00_JPRB, 0.19782E+00_JPRB ,&
&  0.19776E+00_JPRB, 0.19732E+00_JPRB, 0.19499E+00_JPRB, 0.19113E+00_JPRB, 0.18807E+00_JPRB, 0.18696E+00_JPRB, 0.18764E+00_JPRB ,&
&  0.18993E+00_JPRB, 0.19439E+00_JPRB, 0.20509E+00_JPRB, 0.21344E+00_JPRB, 0.21248E+00_JPRB, 0.20799E+00_JPRB, 0.19915E+00_JPRB ,&
&  0.19160E+00_JPRB, 0.18877E+00_JPRB, 0.18637E+00_JPRB, 0.18514E+00_JPRB, 0.18552E+00_JPRB, 0.18700E+00_JPRB, 0.18783E+00_JPRB ,&
&  0.18893E+00_JPRB, 0.19001E+00_JPRB, 0.19089E+00_JPRB, 0.19144E+00_JPRB, 0.19229E+00_JPRB, 0.19327E+00_JPRB, 0.19507E+00_JPRB ,&
&  0.19771E+00_JPRB, 0.20208E+00_JPRB, 0.20782E+00_JPRB, 0.21397E+00_JPRB, 0.21842E+00_JPRB, 0.22059E+00_JPRB, 0.22443E+00_JPRB ,&
&  0.23027E+00_JPRB, 0.24511E+00_JPRB, 0.27351E+00_JPRB, 0.31858E+00_JPRB, 0.32246E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.32831E+00_JPRB, 0.33246E+00_JPRB, 0.33513E+00_JPRB, 0.33861E+00_JPRB, 0.34624E+00_JPRB, 0.35695E+00_JPRB, 0.36905E+00_JPRB ,&
&  0.38018E+00_JPRB, 0.38922E+00_JPRB, 0.39366E+00_JPRB, 0.39881E+00_JPRB, 0.40373E+00_JPRB, 0.40610E+00_JPRB, 0.41131E+00_JPRB ,&
&  0.41210E+00_JPRB, 0.40875E+00_JPRB, 0.40271E+00_JPRB, 0.39471E+00_JPRB, 0.39051E+00_JPRB, 0.39183E+00_JPRB, 0.39477E+00_JPRB ,&
&  0.39669E+00_JPRB, 0.39817E+00_JPRB, 0.39824E+00_JPRB, 0.39480E+00_JPRB, 0.39149E+00_JPRB, 0.38954E+00_JPRB, 0.38552E+00_JPRB ,&
&  0.37642E+00_JPRB, 0.32691E+00_JPRB, 0.28928E+00_JPRB, 0.28496E+00_JPRB, 0.30152E+00_JPRB, 0.30379E+00_JPRB, 0.29863E+00_JPRB ,&
&  0.29290E+00_JPRB, 0.28819E+00_JPRB, 0.28548E+00_JPRB, 0.28335E+00_JPRB, 0.28068E+00_JPRB, 0.27835E+00_JPRB, 0.27918E+00_JPRB ,&
&  0.28052E+00_JPRB, 0.27834E+00_JPRB, 0.27534E+00_JPRB, 0.26906E+00_JPRB, 0.26448E+00_JPRB, 0.26249E+00_JPRB, 0.26086E+00_JPRB ,&
&  0.25662E+00_JPRB, 0.25243E+00_JPRB, 0.25220E+00_JPRB, 0.25329E+00_JPRB, 0.25509E+00_JPRB, 0.25632E+00_JPRB, 0.25368E+00_JPRB ,&
&  0.24773E+00_JPRB, 0.24119E+00_JPRB, 0.23294E+00_JPRB, 0.22307E+00_JPRB, 0.21371E+00_JPRB, 0.20700E+00_JPRB, 0.20168E+00_JPRB ,&
&  0.20066E+00_JPRB, 0.20610E+00_JPRB, 0.21895E+00_JPRB, 0.23439E+00_JPRB, 0.25340E+00_JPRB, 0.24258E+00_JPRB, 0.23161E+00_JPRB ,&
&  0.22117E+00_JPRB, 0.21273E+00_JPRB, 0.20654E+00_JPRB, 0.20378E+00_JPRB, 0.20276E+00_JPRB, 0.20285E+00_JPRB, 0.20405E+00_JPRB ,&
&  0.20618E+00_JPRB, 0.20892E+00_JPRB, 0.21135E+00_JPRB, 0.21252E+00_JPRB, 0.21315E+00_JPRB, 0.21368E+00_JPRB, 0.21356E+00_JPRB ,&
&  0.21289E+00_JPRB, 0.21518E+00_JPRB, 0.22187E+00_JPRB, 0.23042E+00_JPRB, 0.23874E+00_JPRB, 0.24651E+00_JPRB, 0.25946E+00_JPRB ,&
&  0.27531E+00_JPRB, 0.29521E+00_JPRB, 0.30743E+00_JPRB, 0.31672E+00_JPRB, 0.32474E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.35485E+00_JPRB, 0.35632E+00_JPRB, 0.35379E+00_JPRB, 0.35100E+00_JPRB, 0.35074E+00_JPRB, 0.35249E+00_JPRB, 0.35827E+00_JPRB ,&
&  0.36798E+00_JPRB, 0.38049E+00_JPRB, 0.39250E+00_JPRB, 0.40250E+00_JPRB, 0.41154E+00_JPRB, 0.41551E+00_JPRB, 0.42153E+00_JPRB ,&
&  0.42803E+00_JPRB, 0.43402E+00_JPRB, 0.43489E+00_JPRB, 0.42471E+00_JPRB, 0.41480E+00_JPRB, 0.40953E+00_JPRB, 0.40957E+00_JPRB ,&
&  0.41139E+00_JPRB, 0.41406E+00_JPRB, 0.42025E+00_JPRB, 0.42252E+00_JPRB, 0.41659E+00_JPRB, 0.41508E+00_JPRB, 0.41284E+00_JPRB ,&
&  0.40720E+00_JPRB, 0.39580E+00_JPRB, 0.36341E+00_JPRB, 0.35778E+00_JPRB, 0.34955E+00_JPRB, 0.34049E+00_JPRB, 0.33145E+00_JPRB ,&
&  0.32117E+00_JPRB, 0.31088E+00_JPRB, 0.30387E+00_JPRB, 0.29977E+00_JPRB, 0.30051E+00_JPRB, 0.30176E+00_JPRB, 0.30133E+00_JPRB ,&
&  0.30349E+00_JPRB, 0.30625E+00_JPRB, 0.30861E+00_JPRB, 0.30891E+00_JPRB, 0.30840E+00_JPRB, 0.30718E+00_JPRB, 0.30256E+00_JPRB ,&
&  0.29841E+00_JPRB, 0.29619E+00_JPRB, 0.29440E+00_JPRB, 0.29283E+00_JPRB, 0.29255E+00_JPRB, 0.29299E+00_JPRB, 0.29313E+00_JPRB ,&
&  0.29091E+00_JPRB, 0.28727E+00_JPRB, 0.28317E+00_JPRB, 0.27617E+00_JPRB, 0.26418E+00_JPRB, 0.24865E+00_JPRB, 0.23917E+00_JPRB ,&
&  0.23760E+00_JPRB, 0.24844E+00_JPRB, 0.26978E+00_JPRB, 0.27863E+00_JPRB, 0.28598E+00_JPRB, 0.28996E+00_JPRB, 0.29447E+00_JPRB ,&
&  0.29003E+00_JPRB, 0.26588E+00_JPRB, 0.25270E+00_JPRB, 0.24578E+00_JPRB, 0.24027E+00_JPRB, 0.23606E+00_JPRB, 0.23588E+00_JPRB ,&
&  0.23911E+00_JPRB, 0.24562E+00_JPRB, 0.25206E+00_JPRB, 0.25518E+00_JPRB, 0.25448E+00_JPRB, 0.25118E+00_JPRB, 0.24778E+00_JPRB ,&
&  0.24494E+00_JPRB, 0.24944E+00_JPRB, 0.26414E+00_JPRB, 0.28646E+00_JPRB, 0.29462E+00_JPRB, 0.30092E+00_JPRB, 0.30674E+00_JPRB ,&
&  0.31299E+00_JPRB, 0.32246E+00_JPRB, 0.33396E+00_JPRB, 0.34303E+00_JPRB, 0.35108E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.37272E+00_JPRB, 0.37258E+00_JPRB, 0.37418E+00_JPRB, 0.37322E+00_JPRB, 0.37069E+00_JPRB, 0.36881E+00_JPRB, 0.37244E+00_JPRB ,&
&  0.37933E+00_JPRB, 0.39094E+00_JPRB, 0.40634E+00_JPRB, 0.41986E+00_JPRB, 0.42805E+00_JPRB, 0.43310E+00_JPRB, 0.44083E+00_JPRB ,&
&  0.44803E+00_JPRB, 0.45608E+00_JPRB, 0.45637E+00_JPRB, 0.44760E+00_JPRB, 0.44016E+00_JPRB, 0.43624E+00_JPRB, 0.43572E+00_JPRB ,&
&  0.43506E+00_JPRB, 0.43681E+00_JPRB, 0.44000E+00_JPRB, 0.44339E+00_JPRB, 0.44269E+00_JPRB, 0.44079E+00_JPRB, 0.44432E+00_JPRB ,&
&  0.44228E+00_JPRB, 0.43259E+00_JPRB, 0.42277E+00_JPRB, 0.39780E+00_JPRB, 0.39714E+00_JPRB, 0.38681E+00_JPRB, 0.37885E+00_JPRB ,&
&  0.36817E+00_JPRB, 0.35500E+00_JPRB, 0.34543E+00_JPRB, 0.33930E+00_JPRB, 0.33655E+00_JPRB, 0.33403E+00_JPRB, 0.33207E+00_JPRB ,&
&  0.33084E+00_JPRB, 0.33074E+00_JPRB, 0.33223E+00_JPRB, 0.33004E+00_JPRB, 0.32888E+00_JPRB, 0.32962E+00_JPRB, 0.33125E+00_JPRB ,&
&  0.33102E+00_JPRB, 0.33075E+00_JPRB, 0.33053E+00_JPRB, 0.32952E+00_JPRB, 0.32779E+00_JPRB, 0.32528E+00_JPRB, 0.32294E+00_JPRB ,&
&  0.32195E+00_JPRB, 0.32292E+00_JPRB, 0.32321E+00_JPRB, 0.31932E+00_JPRB, 0.31164E+00_JPRB, 0.30232E+00_JPRB, 0.29625E+00_JPRB ,&
&  0.29363E+00_JPRB, 0.29563E+00_JPRB, 0.30342E+00_JPRB, 0.31454E+00_JPRB, 0.32492E+00_JPRB, 0.33111E+00_JPRB, 0.33497E+00_JPRB ,&
&  0.33950E+00_JPRB, 0.33785E+00_JPRB, 0.33420E+00_JPRB, 0.33096E+00_JPRB, 0.32799E+00_JPRB, 0.32396E+00_JPRB, 0.32391E+00_JPRB ,&
&  0.32653E+00_JPRB, 0.32911E+00_JPRB, 0.32991E+00_JPRB, 0.32876E+00_JPRB, 0.32670E+00_JPRB, 0.32243E+00_JPRB, 0.31644E+00_JPRB ,&
&  0.30902E+00_JPRB, 0.30039E+00_JPRB, 0.29833E+00_JPRB, 0.29890E+00_JPRB, 0.30271E+00_JPRB, 0.31057E+00_JPRB, 0.32129E+00_JPRB ,&
&  0.33254E+00_JPRB, 0.34180E+00_JPRB, 0.35175E+00_JPRB, 0.36095E+00_JPRB, 0.36916E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.40107E+00_JPRB, 0.40304E+00_JPRB, 0.40546E+00_JPRB, 0.40860E+00_JPRB, 0.41200E+00_JPRB, 0.41406E+00_JPRB, 0.41798E+00_JPRB ,&
&  0.42020E+00_JPRB, 0.42671E+00_JPRB, 0.43736E+00_JPRB, 0.44806E+00_JPRB, 0.45590E+00_JPRB, 0.46262E+00_JPRB, 0.46922E+00_JPRB ,&
&  0.47460E+00_JPRB, 0.48172E+00_JPRB, 0.48267E+00_JPRB, 0.48028E+00_JPRB, 0.47763E+00_JPRB, 0.47558E+00_JPRB, 0.47370E+00_JPRB ,&
&  0.47114E+00_JPRB, 0.47094E+00_JPRB, 0.47465E+00_JPRB, 0.47886E+00_JPRB, 0.48316E+00_JPRB, 0.48435E+00_JPRB, 0.48300E+00_JPRB ,&
&  0.48662E+00_JPRB, 0.47711E+00_JPRB, 0.47094E+00_JPRB, 0.46370E+00_JPRB, 0.45735E+00_JPRB, 0.44591E+00_JPRB, 0.43749E+00_JPRB ,&
&  0.42751E+00_JPRB, 0.41874E+00_JPRB, 0.41499E+00_JPRB, 0.41099E+00_JPRB, 0.40132E+00_JPRB, 0.39010E+00_JPRB, 0.38183E+00_JPRB ,&
&  0.37643E+00_JPRB, 0.37103E+00_JPRB, 0.36454E+00_JPRB, 0.35915E+00_JPRB, 0.35463E+00_JPRB, 0.35297E+00_JPRB, 0.35465E+00_JPRB ,&
&  0.35875E+00_JPRB, 0.36256E+00_JPRB, 0.36477E+00_JPRB, 0.36525E+00_JPRB, 0.36361E+00_JPRB, 0.36098E+00_JPRB, 0.35942E+00_JPRB ,&
&  0.35994E+00_JPRB, 0.36126E+00_JPRB, 0.36336E+00_JPRB, 0.36849E+00_JPRB, 0.36918E+00_JPRB, 0.36751E+00_JPRB, 0.36179E+00_JPRB ,&
&  0.35917E+00_JPRB, 0.35781E+00_JPRB, 0.35936E+00_JPRB, 0.36440E+00_JPRB, 0.37278E+00_JPRB, 0.38092E+00_JPRB, 0.38679E+00_JPRB ,&
&  0.38974E+00_JPRB, 0.39156E+00_JPRB, 0.39472E+00_JPRB, 0.39843E+00_JPRB, 0.40000E+00_JPRB, 0.39941E+00_JPRB, 0.39799E+00_JPRB ,&
&  0.40084E+00_JPRB, 0.40833E+00_JPRB, 0.41621E+00_JPRB, 0.42121E+00_JPRB, 0.42370E+00_JPRB, 0.42274E+00_JPRB, 0.41596E+00_JPRB ,&
&  0.40360E+00_JPRB, 0.38935E+00_JPRB, 0.37550E+00_JPRB, 0.36373E+00_JPRB, 0.35556E+00_JPRB, 0.35175E+00_JPRB, 0.35470E+00_JPRB ,&
&  0.36248E+00_JPRB, 0.37246E+00_JPRB, 0.38161E+00_JPRB, 0.38842E+00_JPRB, 0.39588E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.43017E+00_JPRB, 0.43035E+00_JPRB, 0.43346E+00_JPRB, 0.43997E+00_JPRB, 0.44426E+00_JPRB, 0.44813E+00_JPRB, 0.45589E+00_JPRB ,&
&  0.46298E+00_JPRB, 0.46920E+00_JPRB, 0.47663E+00_JPRB, 0.48481E+00_JPRB, 0.49122E+00_JPRB, 0.50041E+00_JPRB, 0.50912E+00_JPRB ,&
&  0.51473E+00_JPRB, 0.51699E+00_JPRB, 0.51830E+00_JPRB, 0.51943E+00_JPRB, 0.51926E+00_JPRB, 0.51670E+00_JPRB, 0.51621E+00_JPRB ,&
&  0.51434E+00_JPRB, 0.51226E+00_JPRB, 0.51436E+00_JPRB, 0.51465E+00_JPRB, 0.51627E+00_JPRB, 0.51899E+00_JPRB, 0.51908E+00_JPRB ,&
&  0.51698E+00_JPRB, 0.50395E+00_JPRB, 0.49917E+00_JPRB, 0.49623E+00_JPRB, 0.50047E+00_JPRB, 0.50155E+00_JPRB, 0.50012E+00_JPRB ,&
&  0.49375E+00_JPRB, 0.48986E+00_JPRB, 0.49173E+00_JPRB, 0.49410E+00_JPRB, 0.48837E+00_JPRB, 0.47542E+00_JPRB, 0.46524E+00_JPRB ,&
&  0.45735E+00_JPRB, 0.44531E+00_JPRB, 0.42772E+00_JPRB, 0.41720E+00_JPRB, 0.41266E+00_JPRB, 0.40774E+00_JPRB, 0.40322E+00_JPRB ,&
&  0.40083E+00_JPRB, 0.40002E+00_JPRB, 0.40137E+00_JPRB, 0.40047E+00_JPRB, 0.39678E+00_JPRB, 0.39231E+00_JPRB, 0.39056E+00_JPRB ,&
&  0.39316E+00_JPRB, 0.39545E+00_JPRB, 0.39889E+00_JPRB, 0.40150E+00_JPRB, 0.40090E+00_JPRB, 0.40119E+00_JPRB, 0.40484E+00_JPRB ,&
&  0.41047E+00_JPRB, 0.41578E+00_JPRB, 0.42124E+00_JPRB, 0.42354E+00_JPRB, 0.42662E+00_JPRB, 0.43328E+00_JPRB, 0.43376E+00_JPRB ,&
&  0.43228E+00_JPRB, 0.43424E+00_JPRB, 0.43838E+00_JPRB, 0.44630E+00_JPRB, 0.45850E+00_JPRB, 0.46664E+00_JPRB, 0.46801E+00_JPRB ,&
&  0.46825E+00_JPRB, 0.47062E+00_JPRB, 0.47451E+00_JPRB, 0.48382E+00_JPRB, 0.49400E+00_JPRB, 0.50135E+00_JPRB, 0.50450E+00_JPRB ,&
&  0.50153E+00_JPRB, 0.49496E+00_JPRB, 0.48299E+00_JPRB, 0.46900E+00_JPRB, 0.45637E+00_JPRB, 0.44650E+00_JPRB, 0.43843E+00_JPRB ,&
&  0.43044E+00_JPRB, 0.42691E+00_JPRB, 0.42653E+00_JPRB, 0.42840E+00_JPRB, 0.42957E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.46813E+00_JPRB, 0.46928E+00_JPRB, 0.47432E+00_JPRB, 0.47920E+00_JPRB, 0.48264E+00_JPRB, 0.48750E+00_JPRB, 0.49703E+00_JPRB ,&
&  0.50664E+00_JPRB, 0.51585E+00_JPRB, 0.52218E+00_JPRB, 0.52990E+00_JPRB, 0.54092E+00_JPRB, 0.54839E+00_JPRB, 0.55431E+00_JPRB ,&
&  0.55681E+00_JPRB, 0.55719E+00_JPRB, 0.55819E+00_JPRB, 0.56127E+00_JPRB, 0.56081E+00_JPRB, 0.55716E+00_JPRB, 0.55586E+00_JPRB ,&
&  0.55567E+00_JPRB, 0.55392E+00_JPRB, 0.55357E+00_JPRB, 0.55455E+00_JPRB, 0.55213E+00_JPRB, 0.54825E+00_JPRB, 0.54772E+00_JPRB ,&
&  0.54786E+00_JPRB, 0.53421E+00_JPRB, 0.52662E+00_JPRB, 0.51960E+00_JPRB, 0.52317E+00_JPRB, 0.53453E+00_JPRB, 0.54707E+00_JPRB ,&
&  0.55463E+00_JPRB, 0.55973E+00_JPRB, 0.55954E+00_JPRB, 0.55869E+00_JPRB, 0.55406E+00_JPRB, 0.54273E+00_JPRB, 0.53325E+00_JPRB ,&
&  0.52696E+00_JPRB, 0.51784E+00_JPRB, 0.49547E+00_JPRB, 0.48242E+00_JPRB, 0.47894E+00_JPRB, 0.48160E+00_JPRB, 0.48564E+00_JPRB ,&
&  0.48729E+00_JPRB, 0.48472E+00_JPRB, 0.47856E+00_JPRB, 0.47124E+00_JPRB, 0.46135E+00_JPRB, 0.44994E+00_JPRB, 0.43994E+00_JPRB ,&
&  0.43419E+00_JPRB, 0.42986E+00_JPRB, 0.42625E+00_JPRB, 0.42872E+00_JPRB, 0.42794E+00_JPRB, 0.42400E+00_JPRB, 0.42585E+00_JPRB ,&
&  0.43711E+00_JPRB, 0.45136E+00_JPRB, 0.46449E+00_JPRB, 0.47171E+00_JPRB, 0.47463E+00_JPRB, 0.47797E+00_JPRB, 0.48193E+00_JPRB ,&
&  0.48776E+00_JPRB, 0.49352E+00_JPRB, 0.49859E+00_JPRB, 0.50511E+00_JPRB, 0.51326E+00_JPRB, 0.52074E+00_JPRB, 0.52398E+00_JPRB ,&
&  0.52676E+00_JPRB, 0.52844E+00_JPRB, 0.52971E+00_JPRB, 0.53490E+00_JPRB, 0.54282E+00_JPRB, 0.54848E+00_JPRB, 0.55316E+00_JPRB ,&
&  0.55801E+00_JPRB, 0.56038E+00_JPRB, 0.55921E+00_JPRB, 0.55551E+00_JPRB, 0.55007E+00_JPRB, 0.53922E+00_JPRB, 0.52562E+00_JPRB ,&
&  0.50985E+00_JPRB, 0.49548E+00_JPRB, 0.48484E+00_JPRB, 0.47573E+00_JPRB, 0.46952E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.51930E+00_JPRB, 0.51752E+00_JPRB, 0.51801E+00_JPRB, 0.52108E+00_JPRB, 0.52447E+00_JPRB, 0.52706E+00_JPRB, 0.53271E+00_JPRB ,&
&  0.54251E+00_JPRB, 0.55018E+00_JPRB, 0.55964E+00_JPRB, 0.56935E+00_JPRB, 0.57686E+00_JPRB, 0.58201E+00_JPRB, 0.58484E+00_JPRB ,&
&  0.58323E+00_JPRB, 0.58062E+00_JPRB, 0.58352E+00_JPRB, 0.58816E+00_JPRB, 0.58955E+00_JPRB, 0.58906E+00_JPRB, 0.58897E+00_JPRB ,&
&  0.58830E+00_JPRB, 0.58579E+00_JPRB, 0.58254E+00_JPRB, 0.57788E+00_JPRB, 0.57248E+00_JPRB, 0.56983E+00_JPRB, 0.56743E+00_JPRB ,&
&  0.56068E+00_JPRB, 0.56070E+00_JPRB, 0.51038E+00_JPRB, 0.52544E+00_JPRB, 0.53820E+00_JPRB, 0.55916E+00_JPRB, 0.57982E+00_JPRB ,&
&  0.59436E+00_JPRB, 0.60323E+00_JPRB, 0.60749E+00_JPRB, 0.60557E+00_JPRB, 0.59303E+00_JPRB, 0.58536E+00_JPRB, 0.57678E+00_JPRB ,&
&  0.56588E+00_JPRB, 0.55809E+00_JPRB, 0.54494E+00_JPRB, 0.53075E+00_JPRB, 0.52593E+00_JPRB, 0.52637E+00_JPRB, 0.52625E+00_JPRB ,&
&  0.52233E+00_JPRB, 0.51861E+00_JPRB, 0.51980E+00_JPRB, 0.52070E+00_JPRB, 0.52044E+00_JPRB, 0.51511E+00_JPRB, 0.50835E+00_JPRB ,&
&  0.49780E+00_JPRB, 0.48741E+00_JPRB, 0.48021E+00_JPRB, 0.47489E+00_JPRB, 0.47017E+00_JPRB, 0.46492E+00_JPRB, 0.45983E+00_JPRB ,&
&  0.46331E+00_JPRB, 0.47446E+00_JPRB, 0.48496E+00_JPRB, 0.49368E+00_JPRB, 0.49935E+00_JPRB, 0.50317E+00_JPRB, 0.50835E+00_JPRB ,&
&  0.51438E+00_JPRB, 0.52459E+00_JPRB, 0.53168E+00_JPRB, 0.53805E+00_JPRB, 0.54288E+00_JPRB, 0.54747E+00_JPRB, 0.55274E+00_JPRB ,&
&  0.55709E+00_JPRB, 0.56407E+00_JPRB, 0.56924E+00_JPRB, 0.57544E+00_JPRB, 0.57910E+00_JPRB, 0.58425E+00_JPRB, 0.59043E+00_JPRB ,&
&  0.59732E+00_JPRB, 0.60344E+00_JPRB, 0.60601E+00_JPRB, 0.60505E+00_JPRB, 0.59804E+00_JPRB, 0.58848E+00_JPRB, 0.57927E+00_JPRB ,&
&  0.56746E+00_JPRB, 0.55320E+00_JPRB, 0.53860E+00_JPRB, 0.52920E+00_JPRB, 0.52155E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.58889E+00_JPRB, 0.58371E+00_JPRB, 0.57802E+00_JPRB, 0.57687E+00_JPRB, 0.57534E+00_JPRB, 0.57673E+00_JPRB, 0.57916E+00_JPRB ,&
&  0.58569E+00_JPRB, 0.58990E+00_JPRB, 0.59916E+00_JPRB, 0.60684E+00_JPRB, 0.61081E+00_JPRB, 0.61556E+00_JPRB, 0.61862E+00_JPRB ,&
&  0.61978E+00_JPRB, 0.62116E+00_JPRB, 0.62164E+00_JPRB, 0.62560E+00_JPRB, 0.62600E+00_JPRB, 0.62497E+00_JPRB, 0.62371E+00_JPRB ,&
&  0.62145E+00_JPRB, 0.61890E+00_JPRB, 0.61335E+00_JPRB, 0.60588E+00_JPRB, 0.59659E+00_JPRB, 0.59239E+00_JPRB, 0.58604E+00_JPRB ,&
&  0.58698E+00_JPRB, 0.55691E+00_JPRB, 0.56895E+00_JPRB, 0.56771E+00_JPRB, 0.58438E+00_JPRB, 0.61035E+00_JPRB, 0.64413E+00_JPRB ,&
&  0.66693E+00_JPRB, 0.67839E+00_JPRB, 0.67947E+00_JPRB, 0.67659E+00_JPRB, 0.66194E+00_JPRB, 0.64696E+00_JPRB, 0.62992E+00_JPRB ,&
&  0.61044E+00_JPRB, 0.59926E+00_JPRB, 0.58534E+00_JPRB, 0.57508E+00_JPRB, 0.57079E+00_JPRB, 0.56763E+00_JPRB, 0.56241E+00_JPRB ,&
&  0.55454E+00_JPRB, 0.54768E+00_JPRB, 0.54799E+00_JPRB, 0.55135E+00_JPRB, 0.55493E+00_JPRB, 0.55715E+00_JPRB, 0.55598E+00_JPRB ,&
&  0.54966E+00_JPRB, 0.54425E+00_JPRB, 0.54183E+00_JPRB, 0.54156E+00_JPRB, 0.53738E+00_JPRB, 0.52993E+00_JPRB, 0.52218E+00_JPRB ,&
&  0.52027E+00_JPRB, 0.52293E+00_JPRB, 0.52667E+00_JPRB, 0.53143E+00_JPRB, 0.53492E+00_JPRB, 0.53916E+00_JPRB, 0.54115E+00_JPRB ,&
&  0.54584E+00_JPRB, 0.55039E+00_JPRB, 0.55964E+00_JPRB, 0.56856E+00_JPRB, 0.57095E+00_JPRB, 0.57978E+00_JPRB, 0.58890E+00_JPRB ,&
&  0.59794E+00_JPRB, 0.60843E+00_JPRB, 0.61651E+00_JPRB, 0.62202E+00_JPRB, 0.62780E+00_JPRB, 0.63131E+00_JPRB, 0.63710E+00_JPRB ,&
&  0.64478E+00_JPRB, 0.64670E+00_JPRB, 0.65218E+00_JPRB, 0.65331E+00_JPRB, 0.64871E+00_JPRB, 0.64100E+00_JPRB, 0.63165E+00_JPRB ,&
&  0.62119E+00_JPRB, 0.60582E+00_JPRB, 0.59767E+00_JPRB, 0.59287E+00_JPRB, 0.59038E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.64706E+00_JPRB, 0.63990E+00_JPRB, 0.62787E+00_JPRB, 0.62048E+00_JPRB, 0.61681E+00_JPRB, 0.61900E+00_JPRB, 0.62262E+00_JPRB ,&
&  0.62715E+00_JPRB, 0.62893E+00_JPRB, 0.63682E+00_JPRB, 0.64521E+00_JPRB, 0.65057E+00_JPRB, 0.65431E+00_JPRB, 0.65357E+00_JPRB ,&
&  0.65316E+00_JPRB, 0.65245E+00_JPRB, 0.64689E+00_JPRB, 0.64178E+00_JPRB, 0.63525E+00_JPRB, 0.63498E+00_JPRB, 0.63414E+00_JPRB ,&
&  0.63208E+00_JPRB, 0.62833E+00_JPRB, 0.62144E+00_JPRB, 0.61239E+00_JPRB, 0.60820E+00_JPRB, 0.60736E+00_JPRB, 0.60488E+00_JPRB ,&
&  0.61093E+00_JPRB, 0.53123E+00_JPRB, 0.56939E+00_JPRB, 0.58890E+00_JPRB, 0.61584E+00_JPRB, 0.64674E+00_JPRB, 0.67921E+00_JPRB ,&
&  0.71180E+00_JPRB, 0.73839E+00_JPRB, 0.74853E+00_JPRB, 0.74962E+00_JPRB, 0.73691E+00_JPRB, 0.71465E+00_JPRB, 0.68891E+00_JPRB ,&
&  0.66674E+00_JPRB, 0.64979E+00_JPRB, 0.63051E+00_JPRB, 0.61899E+00_JPRB, 0.60968E+00_JPRB, 0.59794E+00_JPRB, 0.58779E+00_JPRB ,&
&  0.58295E+00_JPRB, 0.57989E+00_JPRB, 0.57926E+00_JPRB, 0.58083E+00_JPRB, 0.58363E+00_JPRB, 0.58571E+00_JPRB, 0.58565E+00_JPRB ,&
&  0.58505E+00_JPRB, 0.58377E+00_JPRB, 0.58083E+00_JPRB, 0.58010E+00_JPRB, 0.58051E+00_JPRB, 0.58323E+00_JPRB, 0.58042E+00_JPRB ,&
&  0.57430E+00_JPRB, 0.57567E+00_JPRB, 0.57857E+00_JPRB, 0.57998E+00_JPRB, 0.58209E+00_JPRB, 0.58430E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.58352E+00_JPRB, 0.58698E+00_JPRB, 0.59086E+00_JPRB, 0.59758E+00_JPRB, 0.60245E+00_JPRB, 0.61178E+00_JPRB, 0.61978E+00_JPRB ,&
&  0.62767E+00_JPRB, 0.63969E+00_JPRB, 0.65109E+00_JPRB, 0.65902E+00_JPRB, 0.66400E+00_JPRB, 0.66884E+00_JPRB, 0.67407E+00_JPRB ,&
&  0.67813E+00_JPRB, 0.68177E+00_JPRB, 0.68513E+00_JPRB, 0.68781E+00_JPRB, 0.68462E+00_JPRB, 0.68124E+00_JPRB, 0.67965E+00_JPRB ,&
&  0.67221E+00_JPRB, 0.66077E+00_JPRB, 0.65691E+00_JPRB, 0.65329E+00_JPRB, 0.64924E+00_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.73471E+00_JPRB, 0.72496E+00_JPRB, 0.70862E+00_JPRB, 0.69435E+00_JPRB, 0.68215E+00_JPRB, 0.67708E+00_JPRB, 0.67441E+00_JPRB ,&
&  0.67958E+00_JPRB, 0.68884E+00_JPRB, 0.69750E+00_JPRB, 0.70210E+00_JPRB, 0.70522E+00_JPRB, 0.70455E+00_JPRB, 0.69972E+00_JPRB ,&
&  0.69527E+00_JPRB, 0.68235E+00_JPRB, 0.67209E+00_JPRB, 0.67061E+00_JPRB, 0.66607E+00_JPRB, 0.66101E+00_JPRB, 0.65522E+00_JPRB ,&
&  0.64989E+00_JPRB, 0.64556E+00_JPRB, 0.63588E+00_JPRB, 0.63124E+00_JPRB, 0.63006E+00_JPRB, 0.63218E+00_JPRB, 0.64204E+00_JPRB ,&
&  0.56712E+00_JPRB, 0.48955E+00_JPRB, 0.55210E+00_JPRB, 0.62060E+00_JPRB, 0.65635E+00_JPRB, 0.69833E+00_JPRB, 0.73558E+00_JPRB ,&
&  0.78462E+00_JPRB, 0.81642E+00_JPRB, 0.83445E+00_JPRB, 0.82990E+00_JPRB, 0.81155E+00_JPRB, 0.78921E+00_JPRB, 0.76544E+00_JPRB ,&
&  0.73644E+00_JPRB, 0.70371E+00_JPRB, 0.68204E+00_JPRB, 0.67081E+00_JPRB, 0.66143E+00_JPRB, 0.65154E+00_JPRB, 0.64508E+00_JPRB ,&
&  0.64221E+00_JPRB, 0.64030E+00_JPRB, 0.63810E+00_JPRB, 0.63676E+00_JPRB, 0.63863E+00_JPRB, 0.64112E+00_JPRB, 0.64475E+00_JPRB ,&
&  0.64701E+00_JPRB, 0.64816E+00_JPRB, 0.64740E+00_JPRB, 0.64318E+00_JPRB, 0.64015E+00_JPRB, 0.64271E+00_JPRB, 0.64265E+00_JPRB ,&
&  0.64307E+00_JPRB, 0.64166E+00_JPRB, 0.64057E+00_JPRB, 0.63959E+00_JPRB, 0.63703E+00_JPRB, 0.63479E+00_JPRB, 0.63528E+00_JPRB ,&
&  0.63892E+00_JPRB, 0.63799E+00_JPRB, 0.64013E+00_JPRB, 0.64517E+00_JPRB, 0.65089E+00_JPRB, 0.65651E+00_JPRB, 0.66061E+00_JPRB ,&
&  0.66792E+00_JPRB, 0.68180E+00_JPRB, 0.69229E+00_JPRB, 0.69921E+00_JPRB, 0.70459E+00_JPRB, 0.71060E+00_JPRB, 0.71916E+00_JPRB ,&
&  0.72652E+00_JPRB, 0.73345E+00_JPRB, 0.74148E+00_JPRB, 0.76126E+00_JPRB, 0.77030E+00_JPRB, 0.77172E+00_JPRB, 0.76926E+00_JPRB ,&
&  0.75948E+00_JPRB, 0.74250E+00_JPRB, 0.74285E+00_JPRB, 0.73933E+00_JPRB, 0.73640E+00_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.78947E+00_JPRB, 0.78349E+00_JPRB, 0.76825E+00_JPRB, 0.75105E+00_JPRB, 0.73896E+00_JPRB, 0.73470E+00_JPRB, 0.73412E+00_JPRB ,&
&  0.73885E+00_JPRB, 0.74244E+00_JPRB, 0.74232E+00_JPRB, 0.73757E+00_JPRB, 0.73008E+00_JPRB, 0.71870E+00_JPRB, 0.70869E+00_JPRB ,&
&  0.69312E+00_JPRB, 0.67702E+00_JPRB, 0.67097E+00_JPRB, 0.67006E+00_JPRB, 0.66723E+00_JPRB, 0.65985E+00_JPRB, 0.65537E+00_JPRB ,&
&  0.65347E+00_JPRB, 0.65546E+00_JPRB, 0.64897E+00_JPRB, 0.64479E+00_JPRB, 0.63955E+00_JPRB, 0.64108E+00_JPRB, 0.64190E+00_JPRB ,&
&  0.64090E+00_JPRB, 0.51904E+00_JPRB, 0.59718E+00_JPRB, 0.66180E+00_JPRB, 0.69211E+00_JPRB, 0.75105E+00_JPRB, 0.81227E+00_JPRB ,&
&  0.85542E+00_JPRB, 0.87481E+00_JPRB, 0.86739E+00_JPRB, 0.84763E+00_JPRB, 0.81769E+00_JPRB, 0.78907E+00_JPRB, 0.76385E+00_JPRB ,&
&  0.74022E+00_JPRB, 0.72242E+00_JPRB, 0.70727E+00_JPRB, 0.69344E+00_JPRB, 0.68488E+00_JPRB, 0.67643E+00_JPRB, 0.66900E+00_JPRB ,&
&  0.66835E+00_JPRB, 0.67417E+00_JPRB, 0.67438E+00_JPRB, 0.67691E+00_JPRB, 0.68092E+00_JPRB, 0.68376E+00_JPRB, 0.69558E+00_JPRB ,&
&  0.70454E+00_JPRB, 0.70870E+00_JPRB, 0.71317E+00_JPRB, 0.71383E+00_JPRB, 0.70719E+00_JPRB, 0.70520E+00_JPRB, 0.70068E+00_JPRB ,&
&  0.69578E+00_JPRB, 0.68641E+00_JPRB, 0.67777E+00_JPRB, 0.67251E+00_JPRB, 0.67014E+00_JPRB, 0.66849E+00_JPRB, 0.67327E+00_JPRB ,&
&  0.68267E+00_JPRB, 0.68409E+00_JPRB, 0.68516E+00_JPRB, 0.68922E+00_JPRB, 0.69313E+00_JPRB, 0.69652E+00_JPRB, 0.70261E+00_JPRB ,&
&  0.71481E+00_JPRB, 0.73293E+00_JPRB, 0.74937E+00_JPRB, 0.75669E+00_JPRB, 0.75958E+00_JPRB, 0.76819E+00_JPRB, 0.77851E+00_JPRB ,&
&  0.78917E+00_JPRB, 0.79479E+00_JPRB, 0.79830E+00_JPRB, 0.84668E+00_JPRB, 0.86684E+00_JPRB, 0.86679E+00_JPRB, 0.86007E+00_JPRB ,&
&  0.83906E+00_JPRB, 0.83073E+00_JPRB, 0.67616E+00_JPRB, 0.74606E+00_JPRB, 0.76846E+00_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.86296E+00_JPRB, 0.85483E+00_JPRB, 0.83454E+00_JPRB, 0.81358E+00_JPRB, 0.80489E+00_JPRB, 0.80173E+00_JPRB, 0.79909E+00_JPRB ,&
&  0.78972E+00_JPRB, 0.77657E+00_JPRB, 0.76167E+00_JPRB, 0.74509E+00_JPRB, 0.72763E+00_JPRB, 0.71295E+00_JPRB, 0.69910E+00_JPRB ,&
&  0.67811E+00_JPRB, 0.66574E+00_JPRB, 0.66435E+00_JPRB, 0.66113E+00_JPRB, 0.65841E+00_JPRB, 0.65422E+00_JPRB, 0.65462E+00_JPRB ,&
&  0.65752E+00_JPRB, 0.65854E+00_JPRB, 0.65845E+00_JPRB, 0.65149E+00_JPRB, 0.64590E+00_JPRB, 0.64545E+00_JPRB, 0.64103E+00_JPRB ,&
&  0.64123E+00_JPRB, 0.54524E+00_JPRB, 0.62612E+00_JPRB, 0.72602E+00_JPRB, 0.78059E+00_JPRB, 0.85723E+00_JPRB, 0.94821E+00_JPRB ,&
&  0.98562E+00_JPRB, 0.96141E+00_JPRB, 0.91915E+00_JPRB, 0.88130E+00_JPRB, 0.84207E+00_JPRB, 0.80804E+00_JPRB, 0.78067E+00_JPRB ,&
&  0.76317E+00_JPRB, 0.75202E+00_JPRB, 0.74236E+00_JPRB, 0.73077E+00_JPRB, 0.71868E+00_JPRB, 0.71256E+00_JPRB, 0.70869E+00_JPRB ,&
&  0.70786E+00_JPRB, 0.71109E+00_JPRB, 0.71272E+00_JPRB, 0.71926E+00_JPRB, 0.72482E+00_JPRB, 0.73865E+00_JPRB, 0.76646E+00_JPRB ,&
&  0.78067E+00_JPRB, 0.79974E+00_JPRB, 0.82996E+00_JPRB, 0.83760E+00_JPRB, 0.82527E+00_JPRB, 0.81493E+00_JPRB, 0.80226E+00_JPRB ,&
&  0.78593E+00_JPRB, 0.76096E+00_JPRB, 0.75178E+00_JPRB, 0.74311E+00_JPRB, 0.73571E+00_JPRB, 0.73278E+00_JPRB, 0.74088E+00_JPRB ,&
&  0.74945E+00_JPRB, 0.75219E+00_JPRB, 0.75741E+00_JPRB, 0.76606E+00_JPRB, 0.77080E+00_JPRB, 0.76856E+00_JPRB, 0.77084E+00_JPRB ,&
&  0.78172E+00_JPRB, 0.79690E+00_JPRB, 0.80349E+00_JPRB, 0.80380E+00_JPRB, 0.81202E+00_JPRB, 0.82851E+00_JPRB, 0.84120E+00_JPRB ,&
&  0.85085E+00_JPRB, 0.85889E+00_JPRB, 0.87113E+00_JPRB, 0.91068E+00_JPRB, 0.97735E+00_JPRB, 0.96808E+00_JPRB, 0.94501E+00_JPRB ,&
&  0.92879E+00_JPRB, 0.90960E+00_JPRB, 0.90243E+00_JPRB, 0.77998E+00_JPRB, 0.83284E+00_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.92699E+00_JPRB, 0.92216E+00_JPRB, 0.89419E+00_JPRB, 0.86877E+00_JPRB, 0.85401E+00_JPRB, 0.83792E+00_JPRB, 0.81124E+00_JPRB ,&
&  0.78203E+00_JPRB, 0.75553E+00_JPRB, 0.73870E+00_JPRB, 0.71722E+00_JPRB, 0.69706E+00_JPRB, 0.68457E+00_JPRB, 0.67211E+00_JPRB ,&
&  0.65500E+00_JPRB, 0.65021E+00_JPRB, 0.65019E+00_JPRB, 0.64859E+00_JPRB, 0.64466E+00_JPRB, 0.64260E+00_JPRB, 0.64986E+00_JPRB ,&
&  0.65097E+00_JPRB, 0.64906E+00_JPRB, 0.64850E+00_JPRB, 0.64357E+00_JPRB, 0.63730E+00_JPRB, 0.63569E+00_JPRB, 0.62464E+00_JPRB ,&
&  0.63378E+00_JPRB, 0.61352E+00_JPRB, 0.72430E+00_JPRB, 0.85458E+00_JPRB, 0.96739E+00_JPRB, 0.10483E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.10316E+01_JPRB, 0.96599E+00_JPRB, 0.91962E+00_JPRB, 0.87110E+00_JPRB, 0.83272E+00_JPRB, 0.80196E+00_JPRB ,&
&  0.78172E+00_JPRB, 0.77046E+00_JPRB, 0.76198E+00_JPRB, 0.75400E+00_JPRB, 0.74579E+00_JPRB, 0.73871E+00_JPRB, 0.73520E+00_JPRB ,&
&  0.73054E+00_JPRB, 0.73041E+00_JPRB, 0.73200E+00_JPRB, 0.72978E+00_JPRB, 0.73659E+00_JPRB, 0.76953E+00_JPRB, 0.82458E+00_JPRB ,&
&  0.85319E+00_JPRB, 0.88897E+00_JPRB, 0.93555E+00_JPRB, 0.95531E+00_JPRB, 0.94070E+00_JPRB, 0.92444E+00_JPRB, 0.90184E+00_JPRB ,&
&  0.88358E+00_JPRB, 0.85992E+00_JPRB, 0.84950E+00_JPRB, 0.83617E+00_JPRB, 0.82047E+00_JPRB, 0.80970E+00_JPRB, 0.80571E+00_JPRB ,&
&  0.80571E+00_JPRB, 0.80672E+00_JPRB, 0.81435E+00_JPRB, 0.82415E+00_JPRB, 0.82476E+00_JPRB, 0.82183E+00_JPRB, 0.82447E+00_JPRB ,&
&  0.82941E+00_JPRB, 0.82343E+00_JPRB, 0.81191E+00_JPRB, 0.82218E+00_JPRB, 0.84656E+00_JPRB, 0.86428E+00_JPRB, 0.87653E+00_JPRB ,&
&  0.88892E+00_JPRB, 0.90876E+00_JPRB, 0.93560E+00_JPRB, 0.10183E+01_JPRB, 0.10454E+01_JPRB, 0.10286E+01_JPRB, 0.10124E+01_JPRB ,&
&  0.10030E+01_JPRB, 0.98551E+00_JPRB, 0.96236E+00_JPRB, 0.94507E+00_JPRB, 0.87336E+00_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.10980E+01_JPRB, 0.10172E+01_JPRB, 0.95154E+00_JPRB, 0.90328E+00_JPRB, 0.86935E+00_JPRB, 0.82947E+00_JPRB, 0.78473E+00_JPRB ,&
&  0.75168E+00_JPRB, 0.73747E+00_JPRB, 0.72327E+00_JPRB, 0.70290E+00_JPRB, 0.68839E+00_JPRB, 0.67334E+00_JPRB, 0.66048E+00_JPRB ,&
&  0.64407E+00_JPRB, 0.63968E+00_JPRB, 0.63633E+00_JPRB, 0.64135E+00_JPRB, 0.64298E+00_JPRB, 0.64805E+00_JPRB, 0.65964E+00_JPRB ,&
&  0.66254E+00_JPRB, 0.64318E+00_JPRB, 0.63982E+00_JPRB, 0.64020E+00_JPRB, 0.63951E+00_JPRB, 0.63928E+00_JPRB, 0.62963E+00_JPRB ,&
&  0.65904E+00_JPRB, 0.70298E+00_JPRB, 0.92744E+00_JPRB, 0.11563E+01_JPRB, 0.12728E+01_JPRB, 0.13315E+01_JPRB, 0.12920E+01_JPRB ,&
&  0.12345E+01_JPRB, 0.11570E+01_JPRB, 0.10723E+01_JPRB, 0.98617E+00_JPRB, 0.90703E+00_JPRB, 0.85342E+00_JPRB, 0.82199E+00_JPRB ,&
&  0.80359E+00_JPRB, 0.79563E+00_JPRB, 0.78792E+00_JPRB, 0.78330E+00_JPRB, 0.77845E+00_JPRB, 0.77113E+00_JPRB, 0.77069E+00_JPRB ,&
&  0.77524E+00_JPRB, 0.76452E+00_JPRB, 0.75975E+00_JPRB, 0.77132E+00_JPRB, 0.79616E+00_JPRB, 0.83780E+00_JPRB, 0.95027E+00_JPRB ,&
&  0.10183E+01_JPRB, 0.10695E+01_JPRB, 0.11227E+01_JPRB, 0.11383E+01_JPRB, 0.11179E+01_JPRB, 0.10905E+01_JPRB, 0.10481E+01_JPRB ,&
&  0.10178E+01_JPRB, 0.99240E+00_JPRB, 0.96648E+00_JPRB, 0.94043E+00_JPRB, 0.91694E+00_JPRB, 0.90139E+00_JPRB, 0.89133E+00_JPRB ,&
&  0.88315E+00_JPRB, 0.89007E+00_JPRB, 0.89892E+00_JPRB, 0.90395E+00_JPRB, 0.89901E+00_JPRB, 0.89369E+00_JPRB, 0.88715E+00_JPRB ,&
&  0.86820E+00_JPRB, 0.84094E+00_JPRB, 0.83257E+00_JPRB, 0.84519E+00_JPRB, 0.87740E+00_JPRB, 0.89425E+00_JPRB, 0.90276E+00_JPRB ,&
&  0.92414E+00_JPRB, 0.96227E+00_JPRB, 0.97774E+00_JPRB, 0.11021E+01_JPRB, 0.11951E+01_JPRB, 0.11701E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.10867E+01_JPRB, 0.10597E+01_JPRB, 0.10239E+01_JPRB, 0.99744E+00_JPRB, 0.10902E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.99749E+00_JPRB, 0.95020E+00_JPRB, 0.90632E+00_JPRB, 0.86193E+00_JPRB, 0.83061E+00_JPRB, 0.79703E+00_JPRB, 0.76516E+00_JPRB ,&
&  0.73745E+00_JPRB, 0.71920E+00_JPRB, 0.70396E+00_JPRB, 0.68912E+00_JPRB, 0.68732E+00_JPRB, 0.67982E+00_JPRB, 0.66501E+00_JPRB ,&
&  0.64507E+00_JPRB, 0.63198E+00_JPRB, 0.62879E+00_JPRB, 0.63213E+00_JPRB, 0.64461E+00_JPRB, 0.65278E+00_JPRB, 0.66179E+00_JPRB ,&
&  0.66742E+00_JPRB, 0.64869E+00_JPRB, 0.63895E+00_JPRB, 0.64047E+00_JPRB, 0.64074E+00_JPRB, 0.64292E+00_JPRB, 0.62928E+00_JPRB ,&
&  0.65085E+00_JPRB, 0.79113E+00_JPRB, 0.11664E+01_JPRB, 0.16607E+01_JPRB, 0.17541E+01_JPRB, 0.16264E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.13866E+01_JPRB, 0.13026E+01_JPRB, 0.11988E+01_JPRB, 0.10842E+01_JPRB, 0.99435E+00_JPRB, 0.93593E+00_JPRB, 0.89982E+00_JPRB ,&
&  0.88463E+00_JPRB, 0.87416E+00_JPRB, 0.85896E+00_JPRB, 0.84703E+00_JPRB, 0.83492E+00_JPRB, 0.82508E+00_JPRB, 0.82118E+00_JPRB ,&
&  0.81696E+00_JPRB, 0.81080E+00_JPRB, 0.80565E+00_JPRB, 0.81121E+00_JPRB, 0.91404E+00_JPRB, 0.10256E+01_JPRB, 0.12658E+01_JPRB ,&
&  0.13775E+01_JPRB, 0.14244E+01_JPRB, 0.14628E+01_JPRB, 0.14473E+01_JPRB, 0.13901E+01_JPRB, 0.13336E+01_JPRB, 0.12786E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.11386E+01_JPRB, 0.10876E+01_JPRB, 0.10503E+01_JPRB, 0.10152E+01_JPRB, 0.98803E+00_JPRB, 0.96830E+00_JPRB ,&
&  0.96363E+00_JPRB, 0.97453E+00_JPRB, 0.98253E+00_JPRB, 0.98326E+00_JPRB, 0.96727E+00_JPRB, 0.94344E+00_JPRB, 0.91607E+00_JPRB ,&
&  0.87858E+00_JPRB, 0.85363E+00_JPRB, 0.86096E+00_JPRB, 0.88014E+00_JPRB, 0.90881E+00_JPRB, 0.91973E+00_JPRB, 0.93493E+00_JPRB ,&
&  0.96654E+00_JPRB, 0.10338E+01_JPRB, 0.10288E+01_JPRB, 0.10518E+01_JPRB, 0.11172E+01_JPRB, 0.11862E+01_JPRB, 0.11462E+01_JPRB ,&
&  0.10868E+01_JPRB, 0.10359E+01_JPRB, 0.10057E+01_JPRB, 0.98199E+00_JPRB, 0.10296E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.94744E+00_JPRB, 0.93139E+00_JPRB, 0.89064E+00_JPRB, 0.84833E+00_JPRB, 0.81555E+00_JPRB, 0.78969E+00_JPRB, 0.76607E+00_JPRB ,&
&  0.73947E+00_JPRB, 0.70871E+00_JPRB, 0.68700E+00_JPRB, 0.67752E+00_JPRB, 0.67452E+00_JPRB, 0.67010E+00_JPRB, 0.65277E+00_JPRB ,&
&  0.63236E+00_JPRB, 0.61814E+00_JPRB, 0.62057E+00_JPRB, 0.62802E+00_JPRB, 0.64047E+00_JPRB, 0.63743E+00_JPRB, 0.64987E+00_JPRB ,&
&  0.65705E+00_JPRB, 0.65566E+00_JPRB, 0.65348E+00_JPRB, 0.65097E+00_JPRB, 0.64554E+00_JPRB, 0.64081E+00_JPRB, 0.62220E+00_JPRB ,&
&  0.66377E+00_JPRB, 0.92117E+00_JPRB, 0.15499E+01_JPRB, 0.20274E+01_JPRB, 0.20253E+01_JPRB, 0.17779E+01_JPRB, 0.16390E+01_JPRB ,&
&  0.15227E+01_JPRB, 0.14484E+01_JPRB, 0.13161E+01_JPRB, 0.12271E+01_JPRB, 0.11439E+01_JPRB, 0.10781E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.99333E+00_JPRB, 0.96154E+00_JPRB, 0.92429E+00_JPRB, 0.89187E+00_JPRB, 0.87204E+00_JPRB, 0.85810E+00_JPRB, 0.85415E+00_JPRB ,&
&  0.85568E+00_JPRB, 0.85240E+00_JPRB, 0.85254E+00_JPRB, 0.87893E+00_JPRB, 0.10553E+01_JPRB, 0.11940E+01_JPRB, 0.17656E+01_JPRB ,&
&  0.23204E+01_JPRB, 0.22178E+01_JPRB, 0.21247E+01_JPRB, 0.19745E+01_JPRB, 0.18001E+01_JPRB, 0.16628E+01_JPRB, 0.15267E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.13104E+01_JPRB, 0.12320E+01_JPRB, 0.11807E+01_JPRB, 0.11470E+01_JPRB, 0.11241E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11004E+01_JPRB, 0.10867E+01_JPRB, 0.10684E+01_JPRB, 0.10421E+01_JPRB, 0.10021E+01_JPRB, 0.96121E+00_JPRB, 0.92685E+00_JPRB ,&
&  0.89282E+00_JPRB, 0.88429E+00_JPRB, 0.90982E+00_JPRB, 0.89482E+00_JPRB, 0.93358E+00_JPRB, 0.97222E+00_JPRB, 0.97956E+00_JPRB ,&
&  0.97723E+00_JPRB, 0.10147E+01_JPRB, 0.10089E+01_JPRB, 0.10121E+01_JPRB, 0.10836E+01_JPRB, 0.11551E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.10199E+01_JPRB, 0.98615E+00_JPRB, 0.96413E+00_JPRB, 0.94169E+00_JPRB, 0.95321E+00_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.90690E+00_JPRB, 0.87576E+00_JPRB, 0.84077E+00_JPRB, 0.80796E+00_JPRB, 0.78099E+00_JPRB, 0.76317E+00_JPRB, 0.74600E+00_JPRB ,&
&  0.71811E+00_JPRB, 0.69113E+00_JPRB, 0.67612E+00_JPRB, 0.66368E+00_JPRB, 0.65380E+00_JPRB, 0.64689E+00_JPRB, 0.62934E+00_JPRB ,&
&  0.61262E+00_JPRB, 0.59520E+00_JPRB, 0.60329E+00_JPRB, 0.61520E+00_JPRB, 0.62647E+00_JPRB, 0.63630E+00_JPRB, 0.65527E+00_JPRB ,&
&  0.65275E+00_JPRB, 0.65418E+00_JPRB, 0.65764E+00_JPRB, 0.65879E+00_JPRB, 0.64841E+00_JPRB, 0.64752E+00_JPRB, 0.62220E+00_JPRB ,&
&  0.70582E+00_JPRB, 0.81186E+00_JPRB, 0.16520E+01_JPRB, 0.23354E+01_JPRB, 0.21429E+01_JPRB, 0.19156E+01_JPRB, 0.17294E+01_JPRB ,&
&  0.16971E+01_JPRB, 0.16118E+01_JPRB, 0.14363E+01_JPRB, 0.13140E+01_JPRB, 0.12077E+01_JPRB, 0.11260E+01_JPRB, 0.10648E+01_JPRB ,&
&  0.10170E+01_JPRB, 0.96387E+00_JPRB, 0.92576E+00_JPRB, 0.89437E+00_JPRB, 0.88242E+00_JPRB, 0.87783E+00_JPRB, 0.88238E+00_JPRB ,&
&  0.88569E+00_JPRB, 0.88369E+00_JPRB, 0.89110E+00_JPRB, 0.96563E+00_JPRB, 0.12076E+01_JPRB, 0.16297E+01_JPRB, 0.21268E+01_JPRB ,&
&  0.25694E+01_JPRB, 0.27899E+01_JPRB, 0.25536E+01_JPRB, 0.22708E+01_JPRB, 0.20224E+01_JPRB, 0.18376E+01_JPRB, 0.17021E+01_JPRB ,&
&  0.15768E+01_JPRB, 0.14756E+01_JPRB, 0.14026E+01_JPRB, 0.13640E+01_JPRB, 0.13374E+01_JPRB, 0.13103E+01_JPRB, 0.12692E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11462E+01_JPRB, 0.10817E+01_JPRB, 0.10271E+01_JPRB, 0.98237E+00_JPRB, 0.96575E+00_JPRB, 0.95823E+00_JPRB ,&
&  0.93582E+00_JPRB, 0.93258E+00_JPRB, 0.93833E+00_JPRB, 0.91629E+00_JPRB, 0.93700E+00_JPRB, 0.94507E+00_JPRB, 0.95240E+00_JPRB ,&
&  0.97892E+00_JPRB, 0.98963E+00_JPRB, 0.98066E+00_JPRB, 0.10362E+01_JPRB, 0.10335E+01_JPRB, 0.10990E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.96373E+00_JPRB, 0.94773E+00_JPRB, 0.92908E+00_JPRB, 0.91986E+00_JPRB, 0.91975E+00_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.85335E+00_JPRB, 0.81024E+00_JPRB, 0.77574E+00_JPRB, 0.76018E+00_JPRB, 0.75065E+00_JPRB, 0.73662E+00_JPRB, 0.72327E+00_JPRB ,&
&  0.69913E+00_JPRB, 0.67158E+00_JPRB, 0.65125E+00_JPRB, 0.63385E+00_JPRB, 0.63290E+00_JPRB, 0.62619E+00_JPRB, 0.61624E+00_JPRB ,&
&  0.60401E+00_JPRB, 0.59917E+00_JPRB, 0.59877E+00_JPRB, 0.60802E+00_JPRB, 0.62607E+00_JPRB, 0.63691E+00_JPRB, 0.65242E+00_JPRB ,&
&  0.66152E+00_JPRB, 0.67162E+00_JPRB, 0.67332E+00_JPRB, 0.66872E+00_JPRB, 0.66095E+00_JPRB, 0.64976E+00_JPRB, 0.63743E+00_JPRB ,&
&  0.74600E+00_JPRB, 0.17506E+01_JPRB, 0.14736E+01_JPRB, 0.26965E+01_JPRB, 0.23355E+01_JPRB, 0.21013E+01_JPRB, 0.16725E+01_JPRB ,&
&  0.15827E+01_JPRB, 0.17161E+01_JPRB, 0.15232E+01_JPRB, 0.13342E+01_JPRB, 0.11910E+01_JPRB, 0.10682E+01_JPRB, 0.98378E+00_JPRB ,&
&  0.95017E+00_JPRB, 0.94343E+00_JPRB, 0.93590E+00_JPRB, 0.91701E+00_JPRB, 0.91261E+00_JPRB, 0.90646E+00_JPRB, 0.91014E+00_JPRB ,&
&  0.91407E+00_JPRB, 0.92104E+00_JPRB, 0.96055E+00_JPRB, 0.11265E+01_JPRB, 0.14632E+01_JPRB, 0.20328E+01_JPRB, 0.23839E+01_JPRB ,&
&  0.24324E+01_JPRB, 0.27066E+01_JPRB, 0.24202E+01_JPRB, 0.21029E+01_JPRB, 0.18804E+01_JPRB, 0.17331E+01_JPRB, 0.16159E+01_JPRB ,&
&  0.15041E+01_JPRB, 0.14406E+01_JPRB, 0.14262E+01_JPRB, 0.14131E+01_JPRB, 0.13808E+01_JPRB, 0.13092E+01_JPRB, 0.12123E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.10512E+01_JPRB, 0.10014E+01_JPRB, 0.97589E+00_JPRB, 0.96929E+00_JPRB, 0.97829E+00_JPRB, 0.99084E+00_JPRB ,&
&  0.98234E+00_JPRB, 0.98907E+00_JPRB, 0.96871E+00_JPRB, 0.92201E+00_JPRB, 0.92555E+00_JPRB, 0.91656E+00_JPRB, 0.90280E+00_JPRB ,&
&  0.94686E+00_JPRB, 0.97320E+00_JPRB, 0.98211E+00_JPRB, 0.10244E+01_JPRB, 0.10248E+01_JPRB, 0.11185E+01_JPRB, 0.98543E+00_JPRB ,&
&  0.93506E+00_JPRB, 0.92267E+00_JPRB, 0.92015E+00_JPRB, 0.90109E+00_JPRB, 0.88656E+00_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.79722E+00_JPRB, 0.77231E+00_JPRB, 0.75528E+00_JPRB, 0.75625E+00_JPRB, 0.73367E+00_JPRB, 0.71708E+00_JPRB, 0.70984E+00_JPRB ,&
&  0.67329E+00_JPRB, 0.64855E+00_JPRB, 0.62977E+00_JPRB, 0.62519E+00_JPRB, 0.62667E+00_JPRB, 0.61928E+00_JPRB, 0.61932E+00_JPRB ,&
&  0.61009E+00_JPRB, 0.60130E+00_JPRB, 0.60380E+00_JPRB, 0.62510E+00_JPRB, 0.65370E+00_JPRB, 0.66127E+00_JPRB, 0.67644E+00_JPRB ,&
&  0.69231E+00_JPRB, 0.68867E+00_JPRB, 0.67944E+00_JPRB, 0.67927E+00_JPRB, 0.68535E+00_JPRB, 0.68442E+00_JPRB, 0.70473E+00_JPRB ,&
&  0.78138E+00_JPRB, 0.15686E+01_JPRB, 0.16721E+01_JPRB, 0.29065E+01_JPRB, 0.25231E+01_JPRB, 0.21360E+01_JPRB, 0.16627E+01_JPRB ,&
&  0.16398E+01_JPRB, 0.18243E+01_JPRB, 0.14943E+01_JPRB, 0.12606E+01_JPRB, 0.10733E+01_JPRB, 0.94429E+00_JPRB, 0.90690E+00_JPRB ,&
&  0.92932E+00_JPRB, 0.96155E+00_JPRB, 0.98513E+00_JPRB, 0.96638E+00_JPRB, 0.94127E+00_JPRB, 0.93238E+00_JPRB, 0.93494E+00_JPRB ,&
&  0.94570E+00_JPRB, 0.98424E+00_JPRB, 0.11164E+01_JPRB, 0.13882E+01_JPRB, 0.20675E+01_JPRB, 0.24426E+01_JPRB, 0.22564E+01_JPRB ,&
&  0.21289E+01_JPRB, 0.22305E+01_JPRB, 0.20168E+01_JPRB, 0.18034E+01_JPRB, 0.15774E+01_JPRB, 0.15583E+01_JPRB, 0.14965E+01_JPRB ,&
&  0.14408E+01_JPRB, 0.14251E+01_JPRB, 0.13849E+01_JPRB, 0.13135E+01_JPRB, 0.12334E+01_JPRB, 0.11463E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.10037E+01_JPRB, 0.97610E+00_JPRB, 0.96131E+00_JPRB, 0.97143E+00_JPRB, 0.10004E+01_JPRB, 0.10258E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10380E+01_JPRB, 0.10408E+01_JPRB, 0.99086E+00_JPRB, 0.95202E+00_JPRB, 0.95556E+00_JPRB, 0.93359E+00_JPRB, 0.92584E+00_JPRB ,&
&  0.97121E+00_JPRB, 0.10112E+01_JPRB, 0.10264E+01_JPRB, 0.10237E+01_JPRB, 0.10333E+01_JPRB, 0.10801E+01_JPRB, 0.97457E+00_JPRB ,&
&  0.92349E+00_JPRB, 0.93607E+00_JPRB, 0.93978E+00_JPRB, 0.87420E+00_JPRB, 0.83642E+00_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.77680E+00_JPRB, 0.77463E+00_JPRB, 0.75941E+00_JPRB, 0.75372E+00_JPRB, 0.71947E+00_JPRB, 0.68466E+00_JPRB, 0.65430E+00_JPRB ,&
&  0.63587E+00_JPRB, 0.62711E+00_JPRB, 0.63249E+00_JPRB, 0.63253E+00_JPRB, 0.63271E+00_JPRB, 0.62422E+00_JPRB, 0.61881E+00_JPRB ,&
&  0.60654E+00_JPRB, 0.61127E+00_JPRB, 0.62658E+00_JPRB, 0.65186E+00_JPRB, 0.68101E+00_JPRB, 0.69165E+00_JPRB, 0.70314E+00_JPRB ,&
&  0.70565E+00_JPRB, 0.70299E+00_JPRB, 0.70594E+00_JPRB, 0.71732E+00_JPRB, 0.73761E+00_JPRB, 0.76121E+00_JPRB, 0.79594E+00_JPRB ,&
&  0.87048E+00_JPRB, 0.15201E+01_JPRB, 0.22452E+01_JPRB, 0.25777E+01_JPRB, 0.22321E+01_JPRB, 0.18512E+01_JPRB, 0.16327E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.14137E+01_JPRB, 0.13486E+01_JPRB, 0.11590E+01_JPRB, 0.10051E+01_JPRB, 0.94565E+00_JPRB, 0.94429E+00_JPRB ,&
&  0.98827E+00_JPRB, 0.10069E+01_JPRB, 0.10160E+01_JPRB, 0.10196E+01_JPRB, 0.10054E+01_JPRB, 0.10009E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.12370E+01_JPRB, 0.15259E+01_JPRB, 0.20284E+01_JPRB, 0.26264E+01_JPRB, 0.24226E+01_JPRB, 0.20043E+01_JPRB ,&
&  0.18535E+01_JPRB, 0.17452E+01_JPRB, 0.16141E+01_JPRB, 0.15296E+01_JPRB, 0.12960E+01_JPRB, 0.14672E+01_JPRB, 0.14082E+01_JPRB ,&
&  0.13425E+01_JPRB, 0.12931E+01_JPRB, 0.12233E+01_JPRB, 0.11547E+01_JPRB, 0.10983E+01_JPRB, 0.10416E+01_JPRB, 0.10008E+01_JPRB ,&
&  0.98839E+00_JPRB, 0.99188E+00_JPRB, 0.99984E+00_JPRB, 0.10153E+01_JPRB, 0.10371E+01_JPRB, 0.10579E+01_JPRB, 0.10660E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10657E+01_JPRB, 0.10434E+01_JPRB, 0.10485E+01_JPRB, 0.10432E+01_JPRB, 0.10176E+01_JPRB, 0.99884E+00_JPRB ,&
&  0.10243E+01_JPRB, 0.10705E+01_JPRB, 0.10611E+01_JPRB, 0.10569E+01_JPRB, 0.10529E+01_JPRB, 0.10568E+01_JPRB, 0.95444E+00_JPRB ,&
&  0.91767E+00_JPRB, 0.96537E+00_JPRB, 0.91631E+00_JPRB, 0.83532E+00_JPRB, 0.80026E+00_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.76231E+00_JPRB, 0.76568E+00_JPRB, 0.75669E+00_JPRB, 0.71593E+00_JPRB, 0.67851E+00_JPRB, 0.64146E+00_JPRB, 0.61546E+00_JPRB ,&
&  0.61136E+00_JPRB, 0.61997E+00_JPRB, 0.64907E+00_JPRB, 0.66936E+00_JPRB, 0.66016E+00_JPRB, 0.64026E+00_JPRB, 0.63386E+00_JPRB ,&
&  0.64036E+00_JPRB, 0.65862E+00_JPRB, 0.67263E+00_JPRB, 0.69611E+00_JPRB, 0.72135E+00_JPRB, 0.74310E+00_JPRB, 0.76714E+00_JPRB ,&
&  0.75830E+00_JPRB, 0.76256E+00_JPRB, 0.77657E+00_JPRB, 0.78969E+00_JPRB, 0.81255E+00_JPRB, 0.86581E+00_JPRB, 0.92580E+00_JPRB ,&
&  0.11230E+01_JPRB, 0.17459E+01_JPRB, 0.10700E+01_JPRB, 0.14792E+01_JPRB, 0.16905E+01_JPRB, 0.16866E+01_JPRB, 0.14939E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.11805E+01_JPRB, 0.11392E+01_JPRB, 0.11468E+01_JPRB, 0.10624E+01_JPRB, 0.10581E+01_JPRB, 0.10623E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10780E+01_JPRB, 0.10898E+01_JPRB, 0.11040E+01_JPRB, 0.11099E+01_JPRB, 0.11361E+01_JPRB, 0.12242E+01_JPRB ,&
&  0.14069E+01_JPRB, 0.16538E+01_JPRB, 0.20727E+01_JPRB, 0.23981E+01_JPRB, 0.28698E+01_JPRB, 0.23231E+01_JPRB, 0.17881E+01_JPRB ,&
&  0.13952E+01_JPRB, 0.13896E+01_JPRB, 0.14122E+01_JPRB, 0.13922E+01_JPRB, 0.12652E+01_JPRB, 0.12155E+01_JPRB, 0.12609E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.11237E+01_JPRB, 0.10987E+01_JPRB, 0.10749E+01_JPRB, 0.10438E+01_JPRB, 0.10229E+01_JPRB, 0.10117E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.10095E+01_JPRB, 0.10240E+01_JPRB, 0.10346E+01_JPRB, 0.10510E+01_JPRB, 0.10617E+01_JPRB, 0.10757E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.11197E+01_JPRB, 0.11530E+01_JPRB, 0.11612E+01_JPRB, 0.11483E+01_JPRB, 0.11255E+01_JPRB, 0.11235E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.11637E+01_JPRB, 0.10739E+01_JPRB, 0.10530E+01_JPRB, 0.10842E+01_JPRB, 0.10280E+01_JPRB, 0.94447E+00_JPRB ,&
&  0.97160E+00_JPRB, 0.11132E+01_JPRB, 0.11573E+01_JPRB, 0.80198E+00_JPRB, 0.77059E+00_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.71397E+00_JPRB, 0.69264E+00_JPRB, 0.66262E+00_JPRB, 0.62255E+00_JPRB, 0.61795E+00_JPRB, 0.61499E+00_JPRB, 0.62678E+00_JPRB ,&
&  0.66734E+00_JPRB, 0.71995E+00_JPRB, 0.72984E+00_JPRB, 0.72554E+00_JPRB, 0.70657E+00_JPRB, 0.70538E+00_JPRB, 0.70835E+00_JPRB ,&
&  0.71872E+00_JPRB, 0.72602E+00_JPRB, 0.74660E+00_JPRB, 0.77397E+00_JPRB, 0.80351E+00_JPRB, 0.82687E+00_JPRB, 0.83711E+00_JPRB ,&
&  0.84858E+00_JPRB, 0.85612E+00_JPRB, 0.87946E+00_JPRB, 0.91536E+00_JPRB, 0.98966E+00_JPRB, 0.11433E+01_JPRB, 0.13973E+01_JPRB ,&
&  0.20085E+01_JPRB, 0.16675E+01_JPRB, 0.82096E+00_JPRB, 0.14297E+01_JPRB, 0.16008E+01_JPRB, 0.17124E+01_JPRB, 0.14715E+01_JPRB ,&
&  0.13971E+01_JPRB, 0.12537E+01_JPRB, 0.11758E+01_JPRB, 0.12651E+01_JPRB, 0.12479E+01_JPRB, 0.12527E+01_JPRB, 0.12515E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.12221E+01_JPRB, 0.12261E+01_JPRB, 0.12437E+01_JPRB, 0.12804E+01_JPRB, 0.13684E+01_JPRB, 0.15357E+01_JPRB ,&
&  0.17481E+01_JPRB, 0.20249E+01_JPRB, 0.24140E+01_JPRB, 0.25558E+01_JPRB, 0.27752E+01_JPRB, 0.21589E+01_JPRB, 0.17425E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.12371E+01_JPRB, 0.13095E+01_JPRB, 0.13128E+01_JPRB, 0.12149E+01_JPRB, 0.10927E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.11060E+01_JPRB, 0.10709E+01_JPRB, 0.10558E+01_JPRB, 0.10313E+01_JPRB, 0.10157E+01_JPRB, 0.10115E+01_JPRB, 0.10120E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.10263E+01_JPRB, 0.10358E+01_JPRB, 0.10417E+01_JPRB, 0.10487E+01_JPRB, 0.10509E+01_JPRB, 0.10768E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11277E+01_JPRB, 0.11587E+01_JPRB, 0.11834E+01_JPRB, 0.11616E+01_JPRB, 0.11156E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.10995E+01_JPRB, 0.10895E+01_JPRB, 0.10482E+01_JPRB, 0.10593E+01_JPRB, 0.10071E+01_JPRB, 0.93454E+00_JPRB, 0.85712E+00_JPRB ,&
&  0.84882E+00_JPRB, 0.83830E+00_JPRB, 0.79400E+00_JPRB, 0.73798E+00_JPRB, 0.71774E+00_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.67006E+00_JPRB, 0.65121E+00_JPRB, 0.63704E+00_JPRB, 0.64320E+00_JPRB, 0.65302E+00_JPRB, 0.67211E+00_JPRB, 0.72244E+00_JPRB ,&
&  0.76360E+00_JPRB, 0.78094E+00_JPRB, 0.80291E+00_JPRB, 0.80540E+00_JPRB, 0.81728E+00_JPRB, 0.80580E+00_JPRB, 0.80759E+00_JPRB ,&
&  0.80038E+00_JPRB, 0.81426E+00_JPRB, 0.83750E+00_JPRB, 0.86076E+00_JPRB, 0.87991E+00_JPRB, 0.89995E+00_JPRB, 0.92939E+00_JPRB ,&
&  0.96477E+00_JPRB, 0.10132E+01_JPRB, 0.10900E+01_JPRB, 0.12099E+01_JPRB, 0.13963E+01_JPRB, 0.16236E+01_JPRB, 0.17860E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.62000E+00_JPRB, 0.11490E+01_JPRB, 0.16052E+01_JPRB, 0.16755E+01_JPRB, 0.17428E+01_JPRB, 0.16717E+01_JPRB ,&
&  0.15192E+01_JPRB, 0.13910E+01_JPRB, 0.13072E+01_JPRB, 0.14331E+01_JPRB, 0.14131E+01_JPRB, 0.14071E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.13851E+01_JPRB, 0.13931E+01_JPRB, 0.14286E+01_JPRB, 0.15019E+01_JPRB, 0.16226E+01_JPRB, 0.17859E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.23206E+01_JPRB, 0.26025E+01_JPRB, 0.24090E+01_JPRB, 0.24780E+01_JPRB, 0.22299E+01_JPRB, 0.19708E+01_JPRB ,&
&  0.13661E+01_JPRB, 0.11154E+01_JPRB, 0.11499E+01_JPRB, 0.12788E+01_JPRB, 0.11879E+01_JPRB, 0.10334E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.10594E+01_JPRB, 0.10264E+01_JPRB, 0.10109E+01_JPRB, 0.98768E+00_JPRB, 0.98516E+00_JPRB, 0.99015E+00_JPRB, 0.10009E+01_JPRB ,&
&  0.10059E+01_JPRB, 0.10194E+01_JPRB, 0.10289E+01_JPRB, 0.10259E+01_JPRB, 0.10083E+01_JPRB, 0.10074E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.10493E+01_JPRB, 0.10551E+01_JPRB, 0.10788E+01_JPRB, 0.11004E+01_JPRB, 0.11180E+01_JPRB, 0.10888E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.10489E+01_JPRB, 0.10305E+01_JPRB, 0.10161E+01_JPRB, 0.10017E+01_JPRB, 0.98642E+00_JPRB, 0.91522E+00_JPRB, 0.82595E+00_JPRB ,&
&  0.79587E+00_JPRB, 0.76231E+00_JPRB, 0.73659E+00_JPRB, 0.71668E+00_JPRB, 0.69555E+00_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.72882E+00_JPRB, 0.74119E+00_JPRB, 0.73482E+00_JPRB, 0.75911E+00_JPRB, 0.77958E+00_JPRB, 0.80990E+00_JPRB, 0.84023E+00_JPRB ,&
&  0.86604E+00_JPRB, 0.89013E+00_JPRB, 0.92260E+00_JPRB, 0.90484E+00_JPRB, 0.91834E+00_JPRB, 0.91246E+00_JPRB, 0.89380E+00_JPRB ,&
&  0.89453E+00_JPRB, 0.91314E+00_JPRB, 0.93397E+00_JPRB, 0.95608E+00_JPRB, 0.98343E+00_JPRB, 0.10305E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.11617E+01_JPRB, 0.12532E+01_JPRB, 0.13738E+01_JPRB, 0.14822E+01_JPRB, 0.15874E+01_JPRB, 0.16334E+01_JPRB, 0.15973E+01_JPRB ,&
&  0.79016E+00_JPRB, 0.97105E+00_JPRB, 0.15314E+01_JPRB, 0.17433E+01_JPRB, 0.18684E+01_JPRB, 0.18986E+01_JPRB, 0.18605E+01_JPRB ,&
&  0.17026E+01_JPRB, 0.14773E+01_JPRB, 0.14220E+01_JPRB, 0.15731E+01_JPRB, 0.15161E+01_JPRB, 0.15158E+01_JPRB, 0.15056E+01_JPRB ,&
&  0.15045E+01_JPRB, 0.15312E+01_JPRB, 0.15746E+01_JPRB, 0.16254E+01_JPRB, 0.17191E+01_JPRB, 0.18437E+01_JPRB, 0.20194E+01_JPRB ,&
&  0.22396E+01_JPRB, 0.24625E+01_JPRB, 0.26619E+01_JPRB, 0.23643E+01_JPRB, 0.23160E+01_JPRB, 0.22772E+01_JPRB, 0.23520E+01_JPRB ,&
&  0.14284E+01_JPRB, 0.10484E+01_JPRB, 0.11148E+01_JPRB, 0.12284E+01_JPRB, 0.11400E+01_JPRB, 0.10800E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.10229E+01_JPRB, 0.97481E+00_JPRB, 0.96461E+00_JPRB, 0.94545E+00_JPRB, 0.93992E+00_JPRB, 0.95504E+00_JPRB, 0.98190E+00_JPRB ,&
&  0.98660E+00_JPRB, 0.98537E+00_JPRB, 0.98533E+00_JPRB, 0.98978E+00_JPRB, 0.96980E+00_JPRB, 0.94718E+00_JPRB, 0.94130E+00_JPRB ,&
&  0.94540E+00_JPRB, 0.92962E+00_JPRB, 0.93504E+00_JPRB, 0.96581E+00_JPRB, 0.10004E+01_JPRB, 0.10148E+01_JPRB, 0.10190E+01_JPRB ,&
&  0.99614E+00_JPRB, 0.10077E+01_JPRB, 0.11323E+01_JPRB, 0.11794E+01_JPRB, 0.11702E+01_JPRB, 0.98794E+00_JPRB, 0.82003E+00_JPRB ,&
&  0.79961E+00_JPRB, 0.77026E+00_JPRB, 0.74340E+00_JPRB, 0.73541E+00_JPRB, 0.72725E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.80418E+00_JPRB, 0.82153E+00_JPRB, 0.84480E+00_JPRB, 0.88113E+00_JPRB, 0.92177E+00_JPRB, 0.95044E+00_JPRB, 0.97924E+00_JPRB ,&
&  0.99043E+00_JPRB, 0.10077E+01_JPRB, 0.10234E+01_JPRB, 0.98589E+00_JPRB, 0.10002E+01_JPRB, 0.98841E+00_JPRB, 0.98701E+00_JPRB ,&
&  0.98994E+00_JPRB, 0.10143E+01_JPRB, 0.10451E+01_JPRB, 0.10782E+01_JPRB, 0.11258E+01_JPRB, 0.11843E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.13595E+01_JPRB, 0.14457E+01_JPRB, 0.15401E+01_JPRB, 0.15965E+01_JPRB, 0.16471E+01_JPRB, 0.16763E+01_JPRB, 0.16672E+01_JPRB ,&
&  0.98748E+00_JPRB, 0.16209E+01_JPRB, 0.18031E+01_JPRB, 0.19148E+01_JPRB, 0.19757E+01_JPRB, 0.18586E+01_JPRB, 0.18417E+01_JPRB ,&
&  0.17364E+01_JPRB, 0.15844E+01_JPRB, 0.15547E+01_JPRB, 0.15710E+01_JPRB, 0.16595E+01_JPRB, 0.16424E+01_JPRB, 0.16484E+01_JPRB ,&
&  0.16609E+01_JPRB, 0.17057E+01_JPRB, 0.17653E+01_JPRB, 0.18342E+01_JPRB, 0.19250E+01_JPRB, 0.20320E+01_JPRB, 0.21818E+01_JPRB ,&
&  0.23278E+01_JPRB, 0.24627E+01_JPRB, 0.25717E+01_JPRB, 0.23076E+01_JPRB, 0.20444E+01_JPRB, 0.20540E+01_JPRB, 0.17532E+01_JPRB ,&
&  0.11621E+01_JPRB, 0.94541E+00_JPRB, 0.10227E+01_JPRB, 0.11486E+01_JPRB, 0.10900E+01_JPRB, 0.10035E+01_JPRB, 0.93213E+00_JPRB ,&
&  0.90031E+00_JPRB, 0.86309E+00_JPRB, 0.88527E+00_JPRB, 0.86130E+00_JPRB, 0.89365E+00_JPRB, 0.89573E+00_JPRB, 0.90881E+00_JPRB ,&
&  0.92232E+00_JPRB, 0.90216E+00_JPRB, 0.88959E+00_JPRB, 0.88636E+00_JPRB, 0.87473E+00_JPRB, 0.86211E+00_JPRB, 0.85491E+00_JPRB ,&
&  0.86303E+00_JPRB, 0.82144E+00_JPRB, 0.79227E+00_JPRB, 0.80341E+00_JPRB, 0.82679E+00_JPRB, 0.85552E+00_JPRB, 0.94292E+00_JPRB ,&
&  0.10967E+01_JPRB, 0.12655E+01_JPRB, 0.15515E+01_JPRB, 0.16348E+01_JPRB, 0.15789E+01_JPRB, 0.11962E+01_JPRB, 0.88646E+00_JPRB ,&
&  0.88112E+00_JPRB, 0.83756E+00_JPRB, 0.80336E+00_JPRB, 0.78455E+00_JPRB, 0.78913E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.96078E+00_JPRB, 0.98718E+00_JPRB, 0.10159E+01_JPRB, 0.10418E+01_JPRB, 0.10522E+01_JPRB, 0.10562E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.10835E+01_JPRB, 0.10655E+01_JPRB, 0.10920E+01_JPRB, 0.10723E+01_JPRB, 0.10962E+01_JPRB, 0.10854E+01_JPRB, 0.10972E+01_JPRB ,&
&  0.11187E+01_JPRB, 0.11553E+01_JPRB, 0.11984E+01_JPRB, 0.12240E+01_JPRB, 0.12389E+01_JPRB, 0.12890E+01_JPRB, 0.13588E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.14844E+01_JPRB, 0.15337E+01_JPRB, 0.15601E+01_JPRB, 0.15328E+01_JPRB, 0.15105E+01_JPRB, 0.10884E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.17845E+01_JPRB, 0.19705E+01_JPRB, 0.19400E+01_JPRB, 0.18012E+01_JPRB, 0.16909E+01_JPRB, 0.16887E+01_JPRB ,&
&  0.16916E+01_JPRB, 0.16523E+01_JPRB, 0.15944E+01_JPRB, 0.15960E+01_JPRB, 0.17768E+01_JPRB, 0.17266E+01_JPRB, 0.17593E+01_JPRB ,&
&  0.17901E+01_JPRB, 0.18299E+01_JPRB, 0.18822E+01_JPRB, 0.19478E+01_JPRB, 0.20060E+01_JPRB, 0.20678E+01_JPRB, 0.21406E+01_JPRB ,&
&  0.21844E+01_JPRB, 0.21892E+01_JPRB, 0.22018E+01_JPRB, 0.19622E+01_JPRB, 0.17963E+01_JPRB, 0.17725E+01_JPRB, 0.13862E+01_JPRB ,&
&  0.95617E+00_JPRB, 0.88673E+00_JPRB, 0.97699E+00_JPRB, 0.10554E+01_JPRB, 0.10262E+01_JPRB, 0.92512E+00_JPRB, 0.86507E+00_JPRB ,&
&  0.82732E+00_JPRB, 0.80709E+00_JPRB, 0.79031E+00_JPRB, 0.81007E+00_JPRB, 0.82010E+00_JPRB, 0.84013E+00_JPRB, 0.85195E+00_JPRB ,&
&  0.86601E+00_JPRB, 0.84600E+00_JPRB, 0.81788E+00_JPRB, 0.80556E+00_JPRB, 0.83322E+00_JPRB, 0.94456E+00_JPRB, 0.10956E+01_JPRB ,&
&  0.11567E+01_JPRB, 0.72342E+00_JPRB, 0.74298E+00_JPRB, 0.69913E+00_JPRB, 0.69368E+00_JPRB, 0.74878E+00_JPRB, 0.92330E+00_JPRB ,&
&  0.13833E+01_JPRB, 0.21211E+01_JPRB, 0.23811E+01_JPRB, 0.20406E+01_JPRB, 0.25063E+01_JPRB, 0.18041E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.99815E+00_JPRB, 0.96897E+00_JPRB, 0.93215E+00_JPRB, 0.92201E+00_JPRB, 0.93214E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.10353E+01_JPRB, 0.10542E+01_JPRB, 0.10566E+01_JPRB, 0.10745E+01_JPRB, 0.10564E+01_JPRB, 0.10757E+01_JPRB, 0.10781E+01_JPRB ,&
&  0.10985E+01_JPRB, 0.11115E+01_JPRB, 0.11331E+01_JPRB, 0.11526E+01_JPRB, 0.11826E+01_JPRB, 0.12100E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12655E+01_JPRB, 0.12996E+01_JPRB, 0.13289E+01_JPRB, 0.13576E+01_JPRB, 0.13511E+01_JPRB, 0.13692E+01_JPRB, 0.13830E+01_JPRB ,&
&  0.14205E+01_JPRB, 0.14152E+01_JPRB, 0.14278E+01_JPRB, 0.14382E+01_JPRB, 0.14266E+01_JPRB, 0.14807E+01_JPRB, 0.10871E+01_JPRB ,&
&  0.16045E+01_JPRB, 0.17718E+01_JPRB, 0.18501E+01_JPRB, 0.17492E+01_JPRB, 0.16371E+01_JPRB, 0.16068E+01_JPRB, 0.16137E+01_JPRB ,&
&  0.16591E+01_JPRB, 0.16894E+01_JPRB, 0.16321E+01_JPRB, 0.16817E+01_JPRB, 0.17001E+01_JPRB, 0.16544E+01_JPRB, 0.16826E+01_JPRB ,&
&  0.17031E+01_JPRB, 0.17230E+01_JPRB, 0.17410E+01_JPRB, 0.17485E+01_JPRB, 0.17394E+01_JPRB, 0.17678E+01_JPRB, 0.17691E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.17334E+01_JPRB, 0.17186E+01_JPRB, 0.16106E+01_JPRB, 0.14917E+01_JPRB, 0.12597E+01_JPRB, 0.10534E+01_JPRB ,&
&  0.81975E+00_JPRB, 0.80042E+00_JPRB, 0.89796E+00_JPRB, 0.10379E+01_JPRB, 0.10119E+01_JPRB, 0.95459E+00_JPRB, 0.88754E+00_JPRB ,&
&  0.81072E+00_JPRB, 0.79238E+00_JPRB, 0.77166E+00_JPRB, 0.77173E+00_JPRB, 0.78145E+00_JPRB, 0.80183E+00_JPRB, 0.81659E+00_JPRB ,&
&  0.82617E+00_JPRB, 0.83197E+00_JPRB, 0.80830E+00_JPRB, 0.78293E+00_JPRB, 0.79344E+00_JPRB, 0.86987E+00_JPRB, 0.84395E+00_JPRB ,&
&  0.96412E+00_JPRB, 0.78290E+00_JPRB, 0.72903E+00_JPRB, 0.68046E+00_JPRB, 0.59281E+00_JPRB, 0.60804E+00_JPRB, 0.77712E+00_JPRB ,&
&  0.12728E+01_JPRB, 0.16430E+01_JPRB, 0.23499E+01_JPRB, 0.39768E+01_JPRB, 0.28135E+01_JPRB, 0.20891E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10058E+01_JPRB, 0.99519E+00_JPRB, 0.98749E+00_JPRB, 0.10090E+01_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.10073E+01_JPRB, 0.10223E+01_JPRB, 0.10369E+01_JPRB, 0.10523E+01_JPRB, 0.10642E+01_JPRB, 0.10850E+01_JPRB, 0.10935E+01_JPRB ,&
&  0.11065E+01_JPRB, 0.11138E+01_JPRB, 0.11310E+01_JPRB, 0.11522E+01_JPRB, 0.11718E+01_JPRB, 0.11995E+01_JPRB, 0.12329E+01_JPRB ,&
&  0.12733E+01_JPRB, 0.13027E+01_JPRB, 0.13377E+01_JPRB, 0.13571E+01_JPRB, 0.13538E+01_JPRB, 0.13657E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.13549E+01_JPRB, 0.13701E+01_JPRB, 0.13934E+01_JPRB, 0.13931E+01_JPRB, 0.14299E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.15493E+01_JPRB, 0.16916E+01_JPRB, 0.17176E+01_JPRB, 0.15808E+01_JPRB, 0.14935E+01_JPRB, 0.14916E+01_JPRB, 0.15204E+01_JPRB ,&
&  0.15796E+01_JPRB, 0.16398E+01_JPRB, 0.15862E+01_JPRB, 0.14908E+01_JPRB, 0.14232E+01_JPRB, 0.14177E+01_JPRB, 0.14233E+01_JPRB ,&
&  0.14332E+01_JPRB, 0.14433E+01_JPRB, 0.14341E+01_JPRB, 0.14095E+01_JPRB, 0.13815E+01_JPRB, 0.13512E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.12442E+01_JPRB, 0.12042E+01_JPRB, 0.10710E+01_JPRB, 0.11042E+01_JPRB, 0.10267E+01_JPRB, 0.87110E+00_JPRB, 0.72342E+00_JPRB ,&
&  0.64361E+00_JPRB, 0.67575E+00_JPRB, 0.85065E+00_JPRB, 0.10534E+01_JPRB, 0.10593E+01_JPRB, 0.10210E+01_JPRB, 0.98389E+00_JPRB ,&
&  0.90554E+00_JPRB, 0.83341E+00_JPRB, 0.78377E+00_JPRB, 0.76465E+00_JPRB, 0.76641E+00_JPRB, 0.79077E+00_JPRB, 0.81065E+00_JPRB ,&
&  0.81717E+00_JPRB, 0.82133E+00_JPRB, 0.80941E+00_JPRB, 0.79762E+00_JPRB, 0.81446E+00_JPRB, 0.83410E+00_JPRB, 0.12183E+01_JPRB ,&
&  0.10145E+01_JPRB, 0.74922E+00_JPRB, 0.89630E+00_JPRB, 0.48549E+00_JPRB, 0.58076E+00_JPRB, 0.57724E+00_JPRB, 0.67082E+00_JPRB ,&
&  0.80386E+00_JPRB, 0.11284E+01_JPRB, 0.15474E+01_JPRB, 0.13585E+01_JPRB, 0.12326E+01_JPRB, 0.11251E+01_JPRB, 0.10344E+01_JPRB ,&
&  0.99927E+00_JPRB, 0.98842E+00_JPRB, 0.98640E+00_JPRB, 0.98304E+00_JPRB, 0.99204E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.99519E+00_JPRB, 0.10070E+01_JPRB, 0.10189E+01_JPRB, 0.10352E+01_JPRB, 0.10498E+01_JPRB, 0.10687E+01_JPRB, 0.10812E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.11000E+01_JPRB, 0.11011E+01_JPRB, 0.11153E+01_JPRB, 0.11290E+01_JPRB, 0.11546E+01_JPRB, 0.11848E+01_JPRB ,&
&  0.12173E+01_JPRB, 0.12522E+01_JPRB, 0.12915E+01_JPRB, 0.13158E+01_JPRB, 0.13360E+01_JPRB, 0.13576E+01_JPRB, 0.13751E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.13848E+01_JPRB, 0.14020E+01_JPRB, 0.14351E+01_JPRB, 0.14741E+01_JPRB, 0.15407E+01_JPRB, 0.13514E+01_JPRB ,&
&  0.15079E+01_JPRB, 0.15922E+01_JPRB, 0.15360E+01_JPRB, 0.13973E+01_JPRB, 0.13433E+01_JPRB, 0.13037E+01_JPRB, 0.12942E+01_JPRB ,&
&  0.13184E+01_JPRB, 0.12777E+01_JPRB, 0.12350E+01_JPRB, 0.11607E+01_JPRB, 0.11228E+01_JPRB, 0.11416E+01_JPRB, 0.11811E+01_JPRB ,&
&  0.12028E+01_JPRB, 0.12249E+01_JPRB, 0.12230E+01_JPRB, 0.12049E+01_JPRB, 0.11950E+01_JPRB, 0.11733E+01_JPRB, 0.11168E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.10503E+01_JPRB, 0.91868E+00_JPRB, 0.89741E+00_JPRB, 0.88253E+00_JPRB, 0.80420E+00_JPRB, 0.69416E+00_JPRB ,&
&  0.64354E+00_JPRB, 0.64074E+00_JPRB, 0.73214E+00_JPRB, 0.10691E+01_JPRB, 0.11936E+01_JPRB, 0.11760E+01_JPRB, 0.11178E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.96245E+00_JPRB, 0.86255E+00_JPRB, 0.80169E+00_JPRB, 0.77849E+00_JPRB, 0.80108E+00_JPRB, 0.81324E+00_JPRB ,&
&  0.81315E+00_JPRB, 0.80682E+00_JPRB, 0.80397E+00_JPRB, 0.81020E+00_JPRB, 0.89793E+00_JPRB, 0.10389E+01_JPRB, 0.16169E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.69105E+00_JPRB, 0.66219E+00_JPRB, 0.78622E+00_JPRB, 0.65850E+00_JPRB, 0.64446E+00_JPRB, 0.74480E+00_JPRB ,&
&  0.95208E+00_JPRB, 0.10749E+01_JPRB, 0.10637E+01_JPRB, 0.10390E+01_JPRB, 0.98726E+00_JPRB, 0.98889E+00_JPRB, 0.97039E+00_JPRB ,&
&  0.95777E+00_JPRB, 0.95965E+00_JPRB, 0.96490E+00_JPRB, 0.97088E+00_JPRB, 0.98194E+00_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.96450E+00_JPRB, 0.98048E+00_JPRB, 0.99399E+00_JPRB, 0.10042E+01_JPRB, 0.10089E+01_JPRB, 0.10126E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10189E+01_JPRB, 0.10162E+01_JPRB, 0.10148E+01_JPRB, 0.10307E+01_JPRB, 0.10528E+01_JPRB, 0.10644E+01_JPRB, 0.10895E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.10992E+01_JPRB, 0.11205E+01_JPRB, 0.11378E+01_JPRB, 0.11654E+01_JPRB, 0.11663E+01_JPRB, 0.11492E+01_JPRB ,&
&  0.11437E+01_JPRB, 0.11343E+01_JPRB, 0.11359E+01_JPRB, 0.11385E+01_JPRB, 0.11507E+01_JPRB, 0.11708E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13462E+01_JPRB, 0.15582E+01_JPRB, 0.14023E+01_JPRB, 0.12371E+01_JPRB, 0.12208E+01_JPRB, 0.11225E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10056E+01_JPRB, 0.93741E+00_JPRB, 0.89842E+00_JPRB, 0.87774E+00_JPRB, 0.90852E+00_JPRB, 0.94947E+00_JPRB, 0.96651E+00_JPRB ,&
&  0.10190E+01_JPRB, 0.10482E+01_JPRB, 0.10777E+01_JPRB, 0.10977E+01_JPRB, 0.11090E+01_JPRB, 0.11737E+01_JPRB, 0.11277E+01_JPRB ,&
&  0.11517E+01_JPRB, 0.12592E+01_JPRB, 0.12864E+01_JPRB, 0.13071E+01_JPRB, 0.12334E+01_JPRB, 0.11458E+01_JPRB, 0.10291E+01_JPRB ,&
&  0.91752E+00_JPRB, 0.91379E+00_JPRB, 0.10505E+01_JPRB, 0.11929E+01_JPRB, 0.12952E+01_JPRB, 0.13372E+01_JPRB, 0.12249E+01_JPRB ,&
&  0.11184E+01_JPRB, 0.10504E+01_JPRB, 0.96892E+00_JPRB, 0.88743E+00_JPRB, 0.84699E+00_JPRB, 0.84276E+00_JPRB, 0.82896E+00_JPRB ,&
&  0.80493E+00_JPRB, 0.79851E+00_JPRB, 0.80747E+00_JPRB, 0.84287E+00_JPRB, 0.87610E+00_JPRB, 0.13373E+01_JPRB, 0.17999E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.89048E+00_JPRB, 0.63356E+00_JPRB, 0.10065E+01_JPRB, 0.73348E+00_JPRB, 0.68954E+00_JPRB, 0.75369E+00_JPRB ,&
&  0.84989E+00_JPRB, 0.92251E+00_JPRB, 0.95176E+00_JPRB, 0.98481E+00_JPRB, 0.88957E+00_JPRB, 0.87401E+00_JPRB, 0.87202E+00_JPRB ,&
&  0.88253E+00_JPRB, 0.89921E+00_JPRB, 0.91548E+00_JPRB, 0.92993E+00_JPRB, 0.94777E+00_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.89987E+00_JPRB, 0.91765E+00_JPRB, 0.94047E+00_JPRB, 0.94376E+00_JPRB, 0.95034E+00_JPRB, 0.94480E+00_JPRB, 0.94247E+00_JPRB ,&
&  0.93030E+00_JPRB, 0.92736E+00_JPRB, 0.92999E+00_JPRB, 0.93567E+00_JPRB, 0.94727E+00_JPRB, 0.96099E+00_JPRB, 0.99291E+00_JPRB ,&
&  0.99462E+00_JPRB, 0.10011E+01_JPRB, 0.10099E+01_JPRB, 0.10275E+01_JPRB, 0.10448E+01_JPRB, 0.10441E+01_JPRB, 0.10187E+01_JPRB ,&
&  0.10024E+01_JPRB, 0.10037E+01_JPRB, 0.10207E+01_JPRB, 0.10527E+01_JPRB, 0.12276E+01_JPRB, 0.15426E+01_JPRB, 0.20428E+01_JPRB ,&
&  0.19034E+01_JPRB, 0.15140E+01_JPRB, 0.13295E+01_JPRB, 0.11470E+01_JPRB, 0.11354E+01_JPRB, 0.11586E+01_JPRB, 0.11051E+01_JPRB ,&
&  0.10860E+01_JPRB, 0.10448E+01_JPRB, 0.10344E+01_JPRB, 0.10523E+01_JPRB, 0.10701E+01_JPRB, 0.10529E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.95582E+00_JPRB, 0.92528E+00_JPRB, 0.95766E+00_JPRB, 0.10590E+01_JPRB, 0.12862E+01_JPRB, 0.14463E+01_JPRB, 0.14086E+01_JPRB ,&
&  0.15484E+01_JPRB, 0.17844E+01_JPRB, 0.18465E+01_JPRB, 0.17905E+01_JPRB, 0.17322E+01_JPRB, 0.16371E+01_JPRB, 0.15731E+01_JPRB ,&
&  0.15620E+01_JPRB, 0.14190E+01_JPRB, 0.11818E+01_JPRB, 0.13362E+01_JPRB, 0.14095E+01_JPRB, 0.14819E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.10767E+01_JPRB, 0.99542E+00_JPRB, 0.91324E+00_JPRB, 0.87506E+00_JPRB, 0.91149E+00_JPRB, 0.91133E+00_JPRB ,&
&  0.86645E+00_JPRB, 0.87939E+00_JPRB, 0.90789E+00_JPRB, 0.94334E+00_JPRB, 0.10873E+01_JPRB, 0.99628E+00_JPRB, 0.11697E+01_JPRB ,&
&  0.10373E+01_JPRB, 0.10263E+01_JPRB, 0.91073E+00_JPRB, 0.14797E+01_JPRB, 0.87476E+00_JPRB, 0.71562E+00_JPRB, 0.73227E+00_JPRB ,&
&  0.76818E+00_JPRB, 0.78385E+00_JPRB, 0.80982E+00_JPRB, 0.83255E+00_JPRB, 0.83127E+00_JPRB, 0.82836E+00_JPRB, 0.84421E+00_JPRB ,&
&  0.86073E+00_JPRB, 0.86677E+00_JPRB, 0.87397E+00_JPRB, 0.87535E+00_JPRB, 0.88692E+00_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.79947E+00_JPRB, 0.80608E+00_JPRB, 0.82053E+00_JPRB, 0.83297E+00_JPRB, 0.85467E+00_JPRB, 0.84506E+00_JPRB, 0.83442E+00_JPRB ,&
&  0.80451E+00_JPRB, 0.79986E+00_JPRB, 0.79373E+00_JPRB, 0.79166E+00_JPRB, 0.79857E+00_JPRB, 0.82358E+00_JPRB, 0.87032E+00_JPRB ,&
&  0.88421E+00_JPRB, 0.88818E+00_JPRB, 0.88052E+00_JPRB, 0.86914E+00_JPRB, 0.83473E+00_JPRB, 0.80519E+00_JPRB, 0.78958E+00_JPRB ,&
&  0.82848E+00_JPRB, 0.10626E+01_JPRB, 0.13540E+01_JPRB, 0.16146E+01_JPRB, 0.19438E+01_JPRB, 0.18213E+01_JPRB, 0.20829E+01_JPRB ,&
&  0.21828E+01_JPRB, 0.15678E+01_JPRB, 0.15327E+01_JPRB, 0.13539E+01_JPRB, 0.13458E+01_JPRB, 0.13319E+01_JPRB, 0.13513E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.14281E+01_JPRB, 0.14354E+01_JPRB, 0.14521E+01_JPRB, 0.14888E+01_JPRB, 0.15307E+01_JPRB, 0.15514E+01_JPRB ,&
&  0.15587E+01_JPRB, 0.14949E+01_JPRB, 0.14879E+01_JPRB, 0.16433E+01_JPRB, 0.17928E+01_JPRB, 0.18848E+01_JPRB, 0.19790E+01_JPRB ,&
&  0.20977E+01_JPRB, 0.21659E+01_JPRB, 0.19880E+01_JPRB, 0.19485E+01_JPRB, 0.20297E+01_JPRB, 0.18738E+01_JPRB, 0.18998E+01_JPRB ,&
&  0.19617E+01_JPRB, 0.17345E+01_JPRB, 0.12194E+01_JPRB, 0.13841E+01_JPRB, 0.15010E+01_JPRB, 0.14464E+01_JPRB, 0.15041E+01_JPRB ,&
&  0.13054E+01_JPRB, 0.11798E+01_JPRB, 0.10172E+01_JPRB, 0.96019E+00_JPRB, 0.98169E+00_JPRB, 0.10813E+01_JPRB, 0.10954E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.98007E+00_JPRB, 0.10770E+01_JPRB, 0.11662E+01_JPRB, 0.11915E+01_JPRB, 0.11436E+01_JPRB, 0.10899E+01_JPRB ,&
&  0.10950E+01_JPRB, 0.10929E+01_JPRB, 0.14064E+01_JPRB, 0.14918E+01_JPRB, 0.87515E+00_JPRB, 0.77639E+00_JPRB, 0.75262E+00_JPRB ,&
&  0.75333E+00_JPRB, 0.74382E+00_JPRB, 0.73322E+00_JPRB, 0.73853E+00_JPRB, 0.73624E+00_JPRB, 0.73211E+00_JPRB, 0.75416E+00_JPRB ,&
&  0.78252E+00_JPRB, 0.78159E+00_JPRB, 0.78206E+00_JPRB, 0.78276E+00_JPRB, 0.78909E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.71967E+00_JPRB, 0.71407E+00_JPRB, 0.70795E+00_JPRB, 0.70786E+00_JPRB, 0.71577E+00_JPRB, 0.71552E+00_JPRB, 0.70593E+00_JPRB ,&
&  0.68823E+00_JPRB, 0.68157E+00_JPRB, 0.66185E+00_JPRB, 0.63974E+00_JPRB, 0.64371E+00_JPRB, 0.69568E+00_JPRB, 0.74919E+00_JPRB ,&
&  0.78263E+00_JPRB, 0.79704E+00_JPRB, 0.79751E+00_JPRB, 0.79851E+00_JPRB, 0.84143E+00_JPRB, 0.89312E+00_JPRB, 0.11589E+01_JPRB ,&
&  0.16123E+01_JPRB, 0.22212E+01_JPRB, 0.26466E+01_JPRB, 0.28348E+01_JPRB, 0.26779E+01_JPRB, 0.22401E+01_JPRB, 0.25252E+01_JPRB ,&
&  0.22432E+01_JPRB, 0.17549E+01_JPRB, 0.15855E+01_JPRB, 0.15226E+01_JPRB, 0.15104E+01_JPRB, 0.14999E+01_JPRB, 0.15172E+01_JPRB ,&
&  0.15542E+01_JPRB, 0.15957E+01_JPRB, 0.16339E+01_JPRB, 0.16796E+01_JPRB, 0.17249E+01_JPRB, 0.17770E+01_JPRB, 0.18307E+01_JPRB ,&
&  0.18827E+01_JPRB, 0.18950E+01_JPRB, 0.19559E+01_JPRB, 0.18177E+01_JPRB, 0.18950E+01_JPRB, 0.19850E+01_JPRB, 0.20574E+01_JPRB ,&
&  0.20530E+01_JPRB, 0.19937E+01_JPRB, 0.20494E+01_JPRB, 0.20537E+01_JPRB, 0.20481E+01_JPRB, 0.19951E+01_JPRB, 0.19207E+01_JPRB ,&
&  0.19336E+01_JPRB, 0.18037E+01_JPRB, 0.13742E+01_JPRB, 0.14870E+01_JPRB, 0.16713E+01_JPRB, 0.14106E+01_JPRB, 0.16010E+01_JPRB ,&
&  0.14364E+01_JPRB, 0.13110E+01_JPRB, 0.12070E+01_JPRB, 0.11716E+01_JPRB, 0.12650E+01_JPRB, 0.13834E+01_JPRB, 0.12858E+01_JPRB ,&
&  0.13097E+01_JPRB, 0.12801E+01_JPRB, 0.13879E+01_JPRB, 0.14726E+01_JPRB, 0.14671E+01_JPRB, 0.14079E+01_JPRB, 0.13188E+01_JPRB ,&
&  0.11945E+01_JPRB, 0.11398E+01_JPRB, 0.10931E+01_JPRB, 0.10915E+01_JPRB, 0.83976E+00_JPRB, 0.75426E+00_JPRB, 0.76799E+00_JPRB ,&
&  0.76475E+00_JPRB, 0.75108E+00_JPRB, 0.75515E+00_JPRB, 0.75819E+00_JPRB, 0.76822E+00_JPRB, 0.76149E+00_JPRB, 0.77173E+00_JPRB ,&
&  0.77354E+00_JPRB, 0.76770E+00_JPRB, 0.76024E+00_JPRB, 0.75852E+00_JPRB, 0.73879E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.80287E+00_JPRB, 0.78383E+00_JPRB, 0.75306E+00_JPRB, 0.72226E+00_JPRB, 0.69245E+00_JPRB, 0.65663E+00_JPRB, 0.65793E+00_JPRB ,&
&  0.64896E+00_JPRB, 0.66399E+00_JPRB, 0.69857E+00_JPRB, 0.73327E+00_JPRB, 0.78601E+00_JPRB, 0.86216E+00_JPRB, 0.93235E+00_JPRB ,&
&  0.97617E+00_JPRB, 0.99260E+00_JPRB, 0.10617E+01_JPRB, 0.11701E+01_JPRB, 0.13700E+01_JPRB, 0.17185E+01_JPRB, 0.22612E+01_JPRB ,&
&  0.25530E+01_JPRB, 0.27147E+01_JPRB, 0.26521E+01_JPRB, 0.24115E+01_JPRB, 0.20624E+01_JPRB, 0.20188E+01_JPRB, 0.21414E+01_JPRB ,&
&  0.22151E+01_JPRB, 0.20180E+01_JPRB, 0.17382E+01_JPRB, 0.16823E+01_JPRB, 0.16514E+01_JPRB, 0.16323E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.16951E+01_JPRB, 0.17105E+01_JPRB, 0.17476E+01_JPRB, 0.17736E+01_JPRB, 0.17979E+01_JPRB, 0.18345E+01_JPRB ,&
&  0.18597E+01_JPRB, 0.18655E+01_JPRB, 0.18633E+01_JPRB, 0.18311E+01_JPRB, 0.18618E+01_JPRB, 0.19057E+01_JPRB, 0.19184E+01_JPRB ,&
&  0.19133E+01_JPRB, 0.18258E+01_JPRB, 0.18682E+01_JPRB, 0.19412E+01_JPRB, 0.18865E+01_JPRB, 0.17597E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.18180E+01_JPRB, 0.18724E+01_JPRB, 0.14204E+01_JPRB, 0.17873E+01_JPRB, 0.20888E+01_JPRB, 0.18935E+01_JPRB, 0.17498E+01_JPRB ,&
&  0.16296E+01_JPRB, 0.15208E+01_JPRB, 0.15278E+01_JPRB, 0.15002E+01_JPRB, 0.16034E+01_JPRB, 0.16378E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.17821E+01_JPRB, 0.17914E+01_JPRB, 0.19449E+01_JPRB, 0.20355E+01_JPRB, 0.19970E+01_JPRB, 0.18367E+01_JPRB, 0.16257E+01_JPRB ,&
&  0.13035E+01_JPRB, 0.14235E+01_JPRB, 0.13802E+01_JPRB, 0.14746E+01_JPRB, 0.12981E+01_JPRB, 0.79210E+00_JPRB, 0.82579E+00_JPRB ,&
&  0.82819E+00_JPRB, 0.83927E+00_JPRB, 0.85600E+00_JPRB, 0.86269E+00_JPRB, 0.86882E+00_JPRB, 0.87375E+00_JPRB, 0.88354E+00_JPRB ,&
&  0.87684E+00_JPRB, 0.86387E+00_JPRB, 0.85206E+00_JPRB, 0.84330E+00_JPRB, 0.82695E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.95617E+00_JPRB, 0.94583E+00_JPRB, 0.91663E+00_JPRB, 0.86622E+00_JPRB, 0.80980E+00_JPRB, 0.75568E+00_JPRB, 0.72723E+00_JPRB ,&
&  0.70662E+00_JPRB, 0.74128E+00_JPRB, 0.80859E+00_JPRB, 0.89654E+00_JPRB, 0.99759E+00_JPRB, 0.10990E+01_JPRB, 0.11925E+01_JPRB ,&
&  0.12512E+01_JPRB, 0.13010E+01_JPRB, 0.14050E+01_JPRB, 0.16394E+01_JPRB, 0.19071E+01_JPRB, 0.21153E+01_JPRB, 0.22431E+01_JPRB ,&
&  0.23501E+01_JPRB, 0.23916E+01_JPRB, 0.22886E+01_JPRB, 0.18372E+01_JPRB, 0.16562E+01_JPRB, 0.20059E+01_JPRB, 0.17432E+01_JPRB ,&
&  0.17014E+01_JPRB, 0.16561E+01_JPRB, 0.16546E+01_JPRB, 0.16675E+01_JPRB, 0.16313E+01_JPRB, 0.15977E+01_JPRB, 0.16071E+01_JPRB ,&
&  0.16199E+01_JPRB, 0.16534E+01_JPRB, 0.16872E+01_JPRB, 0.17007E+01_JPRB, 0.17322E+01_JPRB, 0.17759E+01_JPRB, 0.17990E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.18128E+01_JPRB, 0.18212E+01_JPRB, 0.18518E+01_JPRB, 0.18167E+01_JPRB, 0.18158E+01_JPRB, 0.17803E+01_JPRB ,&
&  0.17481E+01_JPRB, 0.17168E+01_JPRB, 0.17331E+01_JPRB, 0.18236E+01_JPRB, 0.18555E+01_JPRB, 0.16682E+01_JPRB, 0.16996E+01_JPRB ,&
&  0.18188E+01_JPRB, 0.18596E+01_JPRB, 0.16969E+01_JPRB, 0.21103E+01_JPRB, 0.14186E+01_JPRB, 0.17103E+01_JPRB, 0.19443E+01_JPRB ,&
&  0.18816E+01_JPRB, 0.17373E+01_JPRB, 0.16912E+01_JPRB, 0.17125E+01_JPRB, 0.17156E+01_JPRB, 0.17128E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.22471E+01_JPRB, 0.23170E+01_JPRB, 0.24261E+01_JPRB, 0.24657E+01_JPRB, 0.24929E+01_JPRB, 0.23666E+01_JPRB, 0.22072E+01_JPRB ,&
&  0.18677E+01_JPRB, 0.16677E+01_JPRB, 0.14535E+01_JPRB, 0.13549E+01_JPRB, 0.10419E+01_JPRB, 0.79638E+00_JPRB, 0.83509E+00_JPRB ,&
&  0.85473E+00_JPRB, 0.86546E+00_JPRB, 0.88960E+00_JPRB, 0.90626E+00_JPRB, 0.90629E+00_JPRB, 0.91481E+00_JPRB, 0.93351E+00_JPRB ,&
&  0.93560E+00_JPRB, 0.93676E+00_JPRB, 0.94120E+00_JPRB, 0.94325E+00_JPRB, 0.95140E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.97897E+00_JPRB, 0.98689E+00_JPRB, 0.99897E+00_JPRB, 0.99344E+00_JPRB, 0.97922E+00_JPRB, 0.95274E+00_JPRB, 0.89761E+00_JPRB ,&
&  0.85215E+00_JPRB, 0.84387E+00_JPRB, 0.90048E+00_JPRB, 0.97055E+00_JPRB, 0.10430E+01_JPRB, 0.11259E+01_JPRB, 0.12425E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.14880E+01_JPRB, 0.16184E+01_JPRB, 0.18399E+01_JPRB, 0.20994E+01_JPRB, 0.21907E+01_JPRB, 0.22704E+01_JPRB ,&
&  0.20334E+01_JPRB, 0.16302E+01_JPRB, 0.22140E+01_JPRB, 0.25373E+01_JPRB, 0.24848E+01_JPRB, 0.23410E+01_JPRB, 0.21165E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.16711E+01_JPRB, 0.17086E+01_JPRB, 0.17025E+01_JPRB, 0.15357E+01_JPRB, 0.15418E+01_JPRB, 0.15444E+01_JPRB ,&
&  0.15586E+01_JPRB, 0.15559E+01_JPRB, 0.15879E+01_JPRB, 0.16089E+01_JPRB, 0.16034E+01_JPRB, 0.16365E+01_JPRB, 0.16724E+01_JPRB ,&
&  0.16842E+01_JPRB, 0.16853E+01_JPRB, 0.17117E+01_JPRB, 0.17214E+01_JPRB, 0.17415E+01_JPRB, 0.17557E+01_JPRB, 0.17134E+01_JPRB ,&
&  0.16604E+01_JPRB, 0.16016E+01_JPRB, 0.16669E+01_JPRB, 0.18057E+01_JPRB, 0.19161E+01_JPRB, 0.17544E+01_JPRB, 0.18887E+01_JPRB ,&
&  0.19099E+01_JPRB, 0.19204E+01_JPRB, 0.19187E+01_JPRB, 0.21851E+01_JPRB, 0.16550E+01_JPRB, 0.18578E+01_JPRB, 0.18280E+01_JPRB ,&
&  0.20350E+01_JPRB, 0.19925E+01_JPRB, 0.19459E+01_JPRB, 0.19760E+01_JPRB, 0.20472E+01_JPRB, 0.19264E+01_JPRB, 0.21384E+01_JPRB ,&
&  0.23808E+01_JPRB, 0.25858E+01_JPRB, 0.26593E+01_JPRB, 0.27515E+01_JPRB, 0.25974E+01_JPRB, 0.26272E+01_JPRB, 0.26458E+01_JPRB ,&
&  0.27909E+01_JPRB, 0.20935E+01_JPRB, 0.15652E+01_JPRB, 0.11889E+01_JPRB, 0.94037E+00_JPRB, 0.80006E+00_JPRB, 0.82867E+00_JPRB ,&
&  0.85232E+00_JPRB, 0.87811E+00_JPRB, 0.89290E+00_JPRB, 0.91077E+00_JPRB, 0.92365E+00_JPRB, 0.93077E+00_JPRB, 0.94848E+00_JPRB ,&
&  0.95803E+00_JPRB, 0.96326E+00_JPRB, 0.96491E+00_JPRB, 0.96754E+00_JPRB, 0.97149E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.97720E+00_JPRB, 0.99832E+00_JPRB, 0.10319E+01_JPRB, 0.10878E+01_JPRB, 0.11418E+01_JPRB, 0.12392E+01_JPRB, 0.13259E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.89083E+00_JPRB, 0.93402E+00_JPRB, 0.97569E+00_JPRB, 0.10268E+01_JPRB, 0.10919E+01_JPRB, 0.11821E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.15026E+01_JPRB, 0.17217E+01_JPRB, 0.19343E+01_JPRB, 0.21592E+01_JPRB, 0.23486E+01_JPRB, 0.23155E+01_JPRB ,&
&  0.18966E+01_JPRB, 0.19369E+01_JPRB, 0.27121E+01_JPRB, 0.26617E+01_JPRB, 0.25235E+01_JPRB, 0.23966E+01_JPRB, 0.22061E+01_JPRB ,&
&  0.18195E+01_JPRB, 0.15933E+01_JPRB, 0.15527E+01_JPRB, 0.15108E+01_JPRB, 0.13847E+01_JPRB, 0.14073E+01_JPRB, 0.14321E+01_JPRB ,&
&  0.14383E+01_JPRB, 0.14337E+01_JPRB, 0.14423E+01_JPRB, 0.14646E+01_JPRB, 0.14574E+01_JPRB, 0.14667E+01_JPRB, 0.15034E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.16035E+01_JPRB, 0.16138E+01_JPRB, 0.15885E+01_JPRB, 0.16403E+01_JPRB, 0.17052E+01_JPRB, 0.16841E+01_JPRB ,&
&  0.16176E+01_JPRB, 0.16182E+01_JPRB, 0.16442E+01_JPRB, 0.16425E+01_JPRB, 0.15700E+01_JPRB, 0.17819E+01_JPRB, 0.18204E+01_JPRB ,&
&  0.19592E+01_JPRB, 0.19578E+01_JPRB, 0.21999E+01_JPRB, 0.17846E+01_JPRB, 0.19329E+01_JPRB, 0.21014E+01_JPRB, 0.21116E+01_JPRB ,&
&  0.21159E+01_JPRB, 0.23085E+01_JPRB, 0.23392E+01_JPRB, 0.24344E+01_JPRB, 0.24827E+01_JPRB, 0.22609E+01_JPRB, 0.24661E+01_JPRB ,&
&  0.24598E+01_JPRB, 0.28463E+01_JPRB, 0.28393E+01_JPRB, 0.27856E+01_JPRB, 0.23910E+01_JPRB, 0.23344E+01_JPRB, 0.24555E+01_JPRB ,&
&  0.27292E+01_JPRB, 0.26146E+01_JPRB, 0.21409E+01_JPRB, 0.16412E+01_JPRB, 0.11360E+01_JPRB, 0.91776E+00_JPRB, 0.86730E+00_JPRB ,&
&  0.85152E+00_JPRB, 0.86559E+00_JPRB, 0.89078E+00_JPRB, 0.92144E+00_JPRB, 0.94058E+00_JPRB, 0.94563E+00_JPRB, 0.94693E+00_JPRB ,&
&  0.95580E+00_JPRB, 0.96140E+00_JPRB, 0.96080E+00_JPRB, 0.96478E+00_JPRB, 0.96978E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.96315E+00_JPRB, 0.95866E+00_JPRB, 0.97428E+00_JPRB, 0.99447E+00_JPRB, 0.10085E+01_JPRB, 0.10431E+01_JPRB, 0.10362E+01_JPRB ,&
&  0.95627E+00_JPRB, 0.91915E+00_JPRB, 0.95884E+00_JPRB, 0.96386E+00_JPRB, 0.97496E+00_JPRB, 0.10131E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.11682E+01_JPRB, 0.13330E+01_JPRB, 0.15631E+01_JPRB, 0.18778E+01_JPRB, 0.20606E+01_JPRB, 0.22460E+01_JPRB, 0.22582E+01_JPRB ,&
&  0.17802E+01_JPRB, 0.29943E+01_JPRB, 0.25922E+01_JPRB, 0.27065E+01_JPRB, 0.26211E+01_JPRB, 0.26622E+01_JPRB, 0.22807E+01_JPRB ,&
&  0.18885E+01_JPRB, 0.16166E+01_JPRB, 0.15068E+01_JPRB, 0.13727E+01_JPRB, 0.12925E+01_JPRB, 0.12841E+01_JPRB, 0.13125E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.13384E+01_JPRB, 0.13483E+01_JPRB, 0.13659E+01_JPRB, 0.13715E+01_JPRB, 0.13881E+01_JPRB, 0.14224E+01_JPRB ,&
&  0.14849E+01_JPRB, 0.15298E+01_JPRB, 0.15258E+01_JPRB, 0.15433E+01_JPRB, 0.15683E+01_JPRB, 0.16536E+01_JPRB, 0.16961E+01_JPRB ,&
&  0.16103E+01_JPRB, 0.15066E+01_JPRB, 0.15809E+01_JPRB, 0.16213E+01_JPRB, 0.16423E+01_JPRB, 0.17794E+01_JPRB, 0.17425E+01_JPRB ,&
&  0.20224E+01_JPRB, 0.19644E+01_JPRB, 0.22221E+01_JPRB, 0.18546E+01_JPRB, 0.19168E+01_JPRB, 0.22211E+01_JPRB, 0.22330E+01_JPRB ,&
&  0.21845E+01_JPRB, 0.25773E+01_JPRB, 0.26698E+01_JPRB, 0.27501E+01_JPRB, 0.27168E+01_JPRB, 0.25458E+01_JPRB, 0.24168E+01_JPRB ,&
&  0.25471E+01_JPRB, 0.27985E+01_JPRB, 0.30486E+01_JPRB, 0.26346E+01_JPRB, 0.25312E+01_JPRB, 0.22981E+01_JPRB, 0.24763E+01_JPRB ,&
&  0.28377E+01_JPRB, 0.28547E+01_JPRB, 0.26228E+01_JPRB, 0.22596E+01_JPRB, 0.16167E+01_JPRB, 0.12865E+01_JPRB, 0.10587E+01_JPRB ,&
&  0.95205E+00_JPRB, 0.89363E+00_JPRB, 0.88910E+00_JPRB, 0.91679E+00_JPRB, 0.94771E+00_JPRB, 0.96375E+00_JPRB, 0.95952E+00_JPRB ,&
&  0.96208E+00_JPRB, 0.97250E+00_JPRB, 0.98076E+00_JPRB, 0.98131E+00_JPRB, 0.97599E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.10120E+01_JPRB, 0.97519E+00_JPRB, 0.96146E+00_JPRB, 0.96606E+00_JPRB, 0.95390E+00_JPRB, 0.94488E+00_JPRB, 0.92656E+00_JPRB ,&
&  0.87049E+00_JPRB, 0.86672E+00_JPRB, 0.88548E+00_JPRB, 0.89616E+00_JPRB, 0.90360E+00_JPRB, 0.94648E+00_JPRB, 0.10136E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.12028E+01_JPRB, 0.13117E+01_JPRB, 0.15569E+01_JPRB, 0.18073E+01_JPRB, 0.22343E+01_JPRB, 0.16596E+01_JPRB ,&
&  0.19657E+01_JPRB, 0.28663E+01_JPRB, 0.27765E+01_JPRB, 0.28354E+01_JPRB, 0.28353E+01_JPRB, 0.27821E+01_JPRB, 0.23280E+01_JPRB ,&
&  0.20541E+01_JPRB, 0.18238E+01_JPRB, 0.15597E+01_JPRB, 0.13516E+01_JPRB, 0.12194E+01_JPRB, 0.12197E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.13236E+01_JPRB, 0.13073E+01_JPRB, 0.13091E+01_JPRB, 0.13050E+01_JPRB, 0.13056E+01_JPRB, 0.13654E+01_JPRB ,&
&  0.14359E+01_JPRB, 0.14845E+01_JPRB, 0.15238E+01_JPRB, 0.14804E+01_JPRB, 0.15714E+01_JPRB, 0.16292E+01_JPRB, 0.16850E+01_JPRB ,&
&  0.15794E+01_JPRB, 0.14712E+01_JPRB, 0.16002E+01_JPRB, 0.17705E+01_JPRB, 0.18893E+01_JPRB, 0.18718E+01_JPRB, 0.18281E+01_JPRB ,&
&  0.19798E+01_JPRB, 0.20408E+01_JPRB, 0.20603E+01_JPRB, 0.18527E+01_JPRB, 0.19664E+01_JPRB, 0.23366E+01_JPRB, 0.25668E+01_JPRB ,&
&  0.26812E+01_JPRB, 0.27626E+01_JPRB, 0.28582E+01_JPRB, 0.27728E+01_JPRB, 0.26914E+01_JPRB, 0.25700E+01_JPRB, 0.27277E+01_JPRB ,&
&  0.31412E+01_JPRB, 0.34498E+01_JPRB, 0.30103E+01_JPRB, 0.25459E+01_JPRB, 0.26025E+01_JPRB, 0.23048E+01_JPRB, 0.21920E+01_JPRB ,&
&  0.25774E+01_JPRB, 0.26182E+01_JPRB, 0.24639E+01_JPRB, 0.24985E+01_JPRB, 0.20771E+01_JPRB, 0.17918E+01_JPRB, 0.14588E+01_JPRB ,&
&  0.12006E+01_JPRB, 0.10993E+01_JPRB, 0.10286E+01_JPRB, 0.99261E+00_JPRB, 0.10052E+01_JPRB, 0.10078E+01_JPRB, 0.10080E+01_JPRB ,&
&  0.99764E+00_JPRB, 0.10087E+01_JPRB, 0.10275E+01_JPRB, 0.10440E+01_JPRB, 0.10400E+01_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.10910E+01_JPRB, 0.10596E+01_JPRB, 0.10089E+01_JPRB, 0.96667E+00_JPRB, 0.94939E+00_JPRB, 0.90368E+00_JPRB, 0.87058E+00_JPRB ,&
&  0.83268E+00_JPRB, 0.81752E+00_JPRB, 0.79246E+00_JPRB, 0.79018E+00_JPRB, 0.80174E+00_JPRB, 0.84765E+00_JPRB, 0.92383E+00_JPRB ,&
&  0.10281E+01_JPRB, 0.11235E+01_JPRB, 0.11973E+01_JPRB, 0.13057E+01_JPRB, 0.14523E+01_JPRB, 0.14223E+01_JPRB, 0.16704E+01_JPRB ,&
&  0.23029E+01_JPRB, 0.26543E+01_JPRB, 0.27337E+01_JPRB, 0.28070E+01_JPRB, 0.29201E+01_JPRB, 0.30533E+01_JPRB, 0.26697E+01_JPRB ,&
&  0.23558E+01_JPRB, 0.20845E+01_JPRB, 0.17248E+01_JPRB, 0.14512E+01_JPRB, 0.12595E+01_JPRB, 0.11639E+01_JPRB, 0.11708E+01_JPRB ,&
&  0.11919E+01_JPRB, 0.12420E+01_JPRB, 0.12558E+01_JPRB, 0.12443E+01_JPRB, 0.12358E+01_JPRB, 0.12276E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.13385E+01_JPRB, 0.14054E+01_JPRB, 0.14529E+01_JPRB, 0.14625E+01_JPRB, 0.15352E+01_JPRB, 0.15989E+01_JPRB, 0.16960E+01_JPRB ,&
&  0.16903E+01_JPRB, 0.17188E+01_JPRB, 0.17774E+01_JPRB, 0.18179E+01_JPRB, 0.19607E+01_JPRB, 0.20563E+01_JPRB, 0.20616E+01_JPRB ,&
&  0.20553E+01_JPRB, 0.18943E+01_JPRB, 0.16574E+01_JPRB, 0.17636E+01_JPRB, 0.20931E+01_JPRB, 0.25525E+01_JPRB, 0.23836E+01_JPRB ,&
&  0.21149E+01_JPRB, 0.20828E+01_JPRB, 0.23243E+01_JPRB, 0.27981E+01_JPRB, 0.30850E+01_JPRB, 0.30237E+01_JPRB, 0.35064E+01_JPRB ,&
&  0.29589E+01_JPRB, 0.14930E+01_JPRB, 0.12598E+01_JPRB, 0.16721E+01_JPRB, 0.13378E+01_JPRB, 0.15011E+01_JPRB, 0.22137E+01_JPRB ,&
&  0.24001E+01_JPRB, 0.27883E+01_JPRB, 0.28450E+01_JPRB, 0.23290E+01_JPRB, 0.21592E+01_JPRB, 0.20263E+01_JPRB, 0.18173E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.14571E+01_JPRB, 0.13566E+01_JPRB, 0.12735E+01_JPRB, 0.12165E+01_JPRB, 0.11691E+01_JPRB, 0.11438E+01_JPRB ,&
&  0.11238E+01_JPRB, 0.11144E+01_JPRB, 0.11117E+01_JPRB, 0.11084E+01_JPRB, 0.11075E+01_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.11875E+01_JPRB, 0.11520E+01_JPRB, 0.11107E+01_JPRB, 0.10535E+01_JPRB, 0.10037E+01_JPRB, 0.94363E+00_JPRB, 0.87269E+00_JPRB ,&
&  0.82378E+00_JPRB, 0.80042E+00_JPRB, 0.77326E+00_JPRB, 0.75185E+00_JPRB, 0.75822E+00_JPRB, 0.78387E+00_JPRB, 0.84685E+00_JPRB ,&
&  0.95469E+00_JPRB, 0.11009E+01_JPRB, 0.12453E+01_JPRB, 0.13322E+01_JPRB, 0.13417E+01_JPRB, 0.12791E+01_JPRB, 0.16101E+01_JPRB ,&
&  0.19600E+01_JPRB, 0.21830E+01_JPRB, 0.27139E+01_JPRB, 0.29803E+01_JPRB, 0.30655E+01_JPRB, 0.30408E+01_JPRB, 0.28771E+01_JPRB ,&
&  0.26627E+01_JPRB, 0.23893E+01_JPRB, 0.19480E+01_JPRB, 0.16021E+01_JPRB, 0.13576E+01_JPRB, 0.11980E+01_JPRB, 0.11180E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.11943E+01_JPRB, 0.12178E+01_JPRB, 0.11995E+01_JPRB, 0.11869E+01_JPRB, 0.11835E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.12344E+01_JPRB, 0.12983E+01_JPRB, 0.13662E+01_JPRB, 0.14636E+01_JPRB, 0.12614E+01_JPRB, 0.15403E+01_JPRB, 0.16395E+01_JPRB ,&
&  0.17730E+01_JPRB, 0.18693E+01_JPRB, 0.18103E+01_JPRB, 0.19650E+01_JPRB, 0.21717E+01_JPRB, 0.21793E+01_JPRB, 0.22003E+01_JPRB ,&
&  0.22578E+01_JPRB, 0.19647E+01_JPRB, 0.16764E+01_JPRB, 0.18083E+01_JPRB, 0.21687E+01_JPRB, 0.28153E+01_JPRB, 0.17912E+01_JPRB ,&
&  0.15968E+01_JPRB, 0.22737E+01_JPRB, 0.24229E+01_JPRB, 0.23335E+01_JPRB, 0.32601E+01_JPRB, 0.39475E+01_JPRB, 0.26037E+01_JPRB ,&
&  0.89940E+00_JPRB, 0.71732E+00_JPRB, 0.79165E+00_JPRB, 0.74376E+00_JPRB, 0.75863E+00_JPRB, 0.10973E+01_JPRB, 0.29420E+01_JPRB ,&
&  0.23492E+01_JPRB, 0.27293E+01_JPRB, 0.27599E+01_JPRB, 0.26683E+01_JPRB, 0.23948E+01_JPRB, 0.22567E+01_JPRB, 0.21601E+01_JPRB ,&
&  0.18559E+01_JPRB, 0.17491E+01_JPRB, 0.15726E+01_JPRB, 0.14777E+01_JPRB, 0.14285E+01_JPRB, 0.13970E+01_JPRB, 0.13469E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.12943E+01_JPRB, 0.12727E+01_JPRB, 0.12504E+01_JPRB, 0.12238E+01_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.12559E+01_JPRB, 0.12228E+01_JPRB, 0.11894E+01_JPRB, 0.11435E+01_JPRB, 0.10892E+01_JPRB, 0.10186E+01_JPRB, 0.93614E+00_JPRB ,&
&  0.86414E+00_JPRB, 0.81294E+00_JPRB, 0.79023E+00_JPRB, 0.77277E+00_JPRB, 0.76555E+00_JPRB, 0.77214E+00_JPRB, 0.81955E+00_JPRB ,&
&  0.91610E+00_JPRB, 0.10872E+01_JPRB, 0.13702E+01_JPRB, 0.13164E+01_JPRB, 0.13522E+01_JPRB, 0.12644E+01_JPRB, 0.13716E+01_JPRB ,&
&  0.16818E+01_JPRB, 0.19242E+01_JPRB, 0.23760E+01_JPRB, 0.27769E+01_JPRB, 0.32401E+01_JPRB, 0.32612E+01_JPRB, 0.32015E+01_JPRB ,&
&  0.29686E+01_JPRB, 0.27042E+01_JPRB, 0.22512E+01_JPRB, 0.18279E+01_JPRB, 0.14774E+01_JPRB, 0.12582E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.11627E+01_JPRB, 0.11804E+01_JPRB, 0.11530E+01_JPRB, 0.11351E+01_JPRB, 0.11355E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.12022E+01_JPRB, 0.12573E+01_JPRB, 0.13813E+01_JPRB, 0.12731E+01_JPRB, 0.11671E+01_JPRB, 0.14639E+01_JPRB ,&
&  0.17115E+01_JPRB, 0.20486E+01_JPRB, 0.20877E+01_JPRB, 0.23142E+01_JPRB, 0.24301E+01_JPRB, 0.24024E+01_JPRB, 0.25413E+01_JPRB ,&
&  0.24482E+01_JPRB, 0.25626E+01_JPRB, 0.19083E+01_JPRB, 0.19371E+01_JPRB, 0.23930E+01_JPRB, 0.20101E+01_JPRB, 0.19190E+01_JPRB ,&
&  0.22464E+01_JPRB, 0.20798E+01_JPRB, 0.21033E+01_JPRB, 0.18125E+01_JPRB, 0.28000E+01_JPRB, 0.29106E+01_JPRB, 0.10457E+01_JPRB ,&
&  0.80052E+00_JPRB, 0.74220E+00_JPRB, 0.70152E+00_JPRB, 0.67807E+00_JPRB, 0.76611E+00_JPRB, 0.10724E+01_JPRB, 0.16550E+01_JPRB ,&
&  0.19901E+01_JPRB, 0.24912E+01_JPRB, 0.25814E+01_JPRB, 0.27351E+01_JPRB, 0.24988E+01_JPRB, 0.25718E+01_JPRB, 0.23822E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.22341E+01_JPRB, 0.19563E+01_JPRB, 0.17605E+01_JPRB, 0.16429E+01_JPRB, 0.15595E+01_JPRB, 0.14856E+01_JPRB ,&
&  0.14262E+01_JPRB, 0.13920E+01_JPRB, 0.13588E+01_JPRB, 0.13226E+01_JPRB, 0.12885E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.12770E+01_JPRB, 0.12510E+01_JPRB, 0.12225E+01_JPRB, 0.11884E+01_JPRB, 0.11449E+01_JPRB, 0.10848E+01_JPRB, 0.10076E+01_JPRB ,&
&  0.93458E+00_JPRB, 0.87486E+00_JPRB, 0.83323E+00_JPRB, 0.81600E+00_JPRB, 0.79666E+00_JPRB, 0.78342E+00_JPRB, 0.81357E+00_JPRB ,&
&  0.90550E+00_JPRB, 0.10946E+01_JPRB, 0.12311E+01_JPRB, 0.11297E+01_JPRB, 0.10893E+01_JPRB, 0.10836E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.14542E+01_JPRB, 0.17042E+01_JPRB, 0.20714E+01_JPRB, 0.26751E+01_JPRB, 0.31877E+01_JPRB, 0.31603E+01_JPRB, 0.34115E+01_JPRB ,&
&  0.32266E+01_JPRB, 0.29083E+01_JPRB, 0.24956E+01_JPRB, 0.20543E+01_JPRB, 0.16777E+01_JPRB, 0.13718E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.11515E+01_JPRB, 0.11657E+01_JPRB, 0.11674E+01_JPRB, 0.11314E+01_JPRB, 0.11159E+01_JPRB, 0.10940E+01_JPRB, 0.10749E+01_JPRB ,&
&  0.10762E+01_JPRB, 0.11165E+01_JPRB, 0.11697E+01_JPRB, 0.13189E+01_JPRB, 0.15707E+01_JPRB, 0.16202E+01_JPRB, 0.15780E+01_JPRB ,&
&  0.16038E+01_JPRB, 0.18131E+01_JPRB, 0.22703E+01_JPRB, 0.24366E+01_JPRB, 0.26637E+01_JPRB, 0.28831E+01_JPRB, 0.28560E+01_JPRB ,&
&  0.27147E+01_JPRB, 0.26113E+01_JPRB, 0.21973E+01_JPRB, 0.22975E+01_JPRB, 0.20893E+01_JPRB, 0.16775E+01_JPRB, 0.20241E+01_JPRB ,&
&  0.21968E+01_JPRB, 0.21555E+01_JPRB, 0.18249E+01_JPRB, 0.14303E+01_JPRB, 0.15292E+01_JPRB, 0.11749E+01_JPRB, 0.81688E+00_JPRB ,&
&  0.76476E+00_JPRB, 0.73811E+00_JPRB, 0.71225E+00_JPRB, 0.76487E+00_JPRB, 0.78088E+00_JPRB, 0.98443E+00_JPRB, 0.13155E+01_JPRB ,&
&  0.18947E+01_JPRB, 0.22620E+01_JPRB, 0.24892E+01_JPRB, 0.26356E+01_JPRB, 0.26396E+01_JPRB, 0.28802E+01_JPRB, 0.29791E+01_JPRB ,&
&  0.26296E+01_JPRB, 0.25061E+01_JPRB, 0.22376E+01_JPRB, 0.19948E+01_JPRB, 0.18338E+01_JPRB, 0.17000E+01_JPRB, 0.15846E+01_JPRB ,&
&  0.15020E+01_JPRB, 0.14360E+01_JPRB, 0.13867E+01_JPRB, 0.13452E+01_JPRB, 0.13044E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.12710E+01_JPRB, 0.12465E+01_JPRB, 0.12228E+01_JPRB, 0.11974E+01_JPRB, 0.11606E+01_JPRB, 0.11147E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.98414E+00_JPRB, 0.92410E+00_JPRB, 0.86843E+00_JPRB, 0.84622E+00_JPRB, 0.82548E+00_JPRB, 0.80506E+00_JPRB, 0.81906E+00_JPRB ,&
&  0.90412E+00_JPRB, 0.11135E+01_JPRB, 0.95992E+00_JPRB, 0.92969E+00_JPRB, 0.99648E+00_JPRB, 0.10504E+01_JPRB, 0.91713E+00_JPRB ,&
&  0.12511E+01_JPRB, 0.15534E+01_JPRB, 0.19507E+01_JPRB, 0.26861E+01_JPRB, 0.36990E+01_JPRB, 0.34560E+01_JPRB, 0.35313E+01_JPRB ,&
&  0.35369E+01_JPRB, 0.30541E+01_JPRB, 0.26935E+01_JPRB, 0.22672E+01_JPRB, 0.18522E+01_JPRB, 0.15520E+01_JPRB, 0.13011E+01_JPRB ,&
&  0.12020E+01_JPRB, 0.11784E+01_JPRB, 0.11651E+01_JPRB, 0.11421E+01_JPRB, 0.11177E+01_JPRB, 0.10911E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10562E+01_JPRB, 0.10832E+01_JPRB, 0.12524E+01_JPRB, 0.14158E+01_JPRB, 0.14329E+01_JPRB, 0.18689E+01_JPRB ,&
&  0.20316E+01_JPRB, 0.22784E+01_JPRB, 0.23237E+01_JPRB, 0.26584E+01_JPRB, 0.30118E+01_JPRB, 0.30732E+01_JPRB, 0.32875E+01_JPRB ,&
&  0.25299E+01_JPRB, 0.21216E+01_JPRB, 0.21628E+01_JPRB, 0.21609E+01_JPRB, 0.18008E+01_JPRB, 0.19765E+01_JPRB, 0.31415E+01_JPRB ,&
&  0.22809E+01_JPRB, 0.24915E+01_JPRB, 0.28138E+01_JPRB, 0.29528E+01_JPRB, 0.14922E+01_JPRB, 0.10635E+01_JPRB, 0.17498E+01_JPRB ,&
&  0.18291E+01_JPRB, 0.11500E+01_JPRB, 0.90646E+00_JPRB, 0.11732E+01_JPRB, 0.93525E+00_JPRB, 0.10842E+01_JPRB, 0.14197E+01_JPRB ,&
&  0.16217E+01_JPRB, 0.17782E+01_JPRB, 0.24226E+01_JPRB, 0.27633E+01_JPRB, 0.29304E+01_JPRB, 0.32991E+01_JPRB, 0.31630E+01_JPRB ,&
&  0.30057E+01_JPRB, 0.26199E+01_JPRB, 0.25049E+01_JPRB, 0.22150E+01_JPRB, 0.20026E+01_JPRB, 0.18118E+01_JPRB, 0.16532E+01_JPRB ,&
&  0.15344E+01_JPRB, 0.14411E+01_JPRB, 0.13763E+01_JPRB, 0.13245E+01_JPRB, 0.12974E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.12771E+01_JPRB, 0.12304E+01_JPRB, 0.12079E+01_JPRB, 0.11988E+01_JPRB, 0.11813E+01_JPRB, 0.11428E+01_JPRB, 0.10940E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.97394E+00_JPRB, 0.91403E+00_JPRB, 0.87425E+00_JPRB, 0.85793E+00_JPRB, 0.83595E+00_JPRB, 0.83000E+00_JPRB ,&
&  0.88568E+00_JPRB, 0.92456E+00_JPRB, 0.84892E+00_JPRB, 0.83876E+00_JPRB, 0.10226E+01_JPRB, 0.95288E+00_JPRB, 0.93328E+00_JPRB ,&
&  0.11376E+01_JPRB, 0.13971E+01_JPRB, 0.18469E+01_JPRB, 0.24658E+01_JPRB, 0.29353E+01_JPRB, 0.33546E+01_JPRB, 0.35406E+01_JPRB ,&
&  0.35732E+01_JPRB, 0.31651E+01_JPRB, 0.27329E+01_JPRB, 0.23575E+01_JPRB, 0.19787E+01_JPRB, 0.16851E+01_JPRB, 0.14487E+01_JPRB ,&
&  0.13028E+01_JPRB, 0.12351E+01_JPRB, 0.11971E+01_JPRB, 0.11576E+01_JPRB, 0.11559E+01_JPRB, 0.11154E+01_JPRB, 0.10813E+01_JPRB ,&
&  0.10764E+01_JPRB, 0.10718E+01_JPRB, 0.10606E+01_JPRB, 0.11388E+01_JPRB, 0.12276E+01_JPRB, 0.14002E+01_JPRB, 0.15510E+01_JPRB ,&
&  0.18068E+01_JPRB, 0.20983E+01_JPRB, 0.22756E+01_JPRB, 0.26747E+01_JPRB, 0.32550E+01_JPRB, 0.32135E+01_JPRB, 0.36838E+01_JPRB ,&
&  0.30847E+01_JPRB, 0.25563E+01_JPRB, 0.21452E+01_JPRB, 0.17339E+01_JPRB, 0.20472E+01_JPRB, 0.32971E+01_JPRB, 0.33787E+01_JPRB ,&
&  0.30204E+01_JPRB, 0.29648E+01_JPRB, 0.30245E+01_JPRB, 0.27294E+01_JPRB, 0.22243E+01_JPRB, 0.17148E+01_JPRB, 0.22822E+01_JPRB ,&
&  0.23859E+01_JPRB, 0.23181E+01_JPRB, 0.21160E+01_JPRB, 0.17614E+01_JPRB, 0.14471E+01_JPRB, 0.16809E+01_JPRB, 0.16406E+01_JPRB ,&
&  0.15859E+01_JPRB, 0.15826E+01_JPRB, 0.19992E+01_JPRB, 0.26220E+01_JPRB, 0.28258E+01_JPRB, 0.28578E+01_JPRB, 0.29298E+01_JPRB ,&
&  0.27471E+01_JPRB, 0.26790E+01_JPRB, 0.25494E+01_JPRB, 0.23639E+01_JPRB, 0.21909E+01_JPRB, 0.20127E+01_JPRB, 0.18680E+01_JPRB ,&
&  0.17434E+01_JPRB, 0.16365E+01_JPRB, 0.15275E+01_JPRB, 0.14272E+01_JPRB, 0.13415E+01_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.12718E+01_JPRB, 0.12084E+01_JPRB, 0.11727E+01_JPRB, 0.11624E+01_JPRB, 0.11704E+01_JPRB, 0.11522E+01_JPRB, 0.11162E+01_JPRB ,&
&  0.10724E+01_JPRB, 0.10279E+01_JPRB, 0.97668E+00_JPRB, 0.93012E+00_JPRB, 0.89937E+00_JPRB, 0.85163E+00_JPRB, 0.83518E+00_JPRB ,&
&  0.88213E+00_JPRB, 0.85960E+00_JPRB, 0.77665E+00_JPRB, 0.85802E+00_JPRB, 0.90611E+00_JPRB, 0.84524E+00_JPRB, 0.93736E+00_JPRB ,&
&  0.10535E+01_JPRB, 0.12693E+01_JPRB, 0.15887E+01_JPRB, 0.19524E+01_JPRB, 0.22711E+01_JPRB, 0.27348E+01_JPRB, 0.31057E+01_JPRB ,&
&  0.30388E+01_JPRB, 0.29602E+01_JPRB, 0.27484E+01_JPRB, 0.25000E+01_JPRB, 0.21551E+01_JPRB, 0.18645E+01_JPRB, 0.16508E+01_JPRB ,&
&  0.14659E+01_JPRB, 0.13517E+01_JPRB, 0.12892E+01_JPRB, 0.12454E+01_JPRB, 0.12249E+01_JPRB, 0.12114E+01_JPRB, 0.12135E+01_JPRB ,&
&  0.11657E+01_JPRB, 0.11256E+01_JPRB, 0.10751E+01_JPRB, 0.11035E+01_JPRB, 0.12589E+01_JPRB, 0.14154E+01_JPRB, 0.14892E+01_JPRB ,&
&  0.18967E+01_JPRB, 0.23112E+01_JPRB, 0.25167E+01_JPRB, 0.31539E+01_JPRB, 0.27812E+01_JPRB, 0.29638E+01_JPRB, 0.38668E+01_JPRB ,&
&  0.41144E+01_JPRB, 0.41527E+01_JPRB, 0.40494E+01_JPRB, 0.31905E+01_JPRB, 0.27517E+01_JPRB, 0.38342E+01_JPRB, 0.36562E+01_JPRB ,&
&  0.32532E+01_JPRB, 0.31150E+01_JPRB, 0.29156E+01_JPRB, 0.31930E+01_JPRB, 0.25868E+01_JPRB, 0.23281E+01_JPRB, 0.18674E+01_JPRB ,&
&  0.19277E+01_JPRB, 0.16782E+01_JPRB, 0.19748E+01_JPRB, 0.17804E+01_JPRB, 0.17329E+01_JPRB, 0.16895E+01_JPRB, 0.15832E+01_JPRB ,&
&  0.15683E+01_JPRB, 0.15672E+01_JPRB, 0.16076E+01_JPRB, 0.19712E+01_JPRB, 0.23880E+01_JPRB, 0.22622E+01_JPRB, 0.23984E+01_JPRB ,&
&  0.24722E+01_JPRB, 0.24891E+01_JPRB, 0.23543E+01_JPRB, 0.23579E+01_JPRB, 0.22761E+01_JPRB, 0.21616E+01_JPRB, 0.20227E+01_JPRB ,&
&  0.18791E+01_JPRB, 0.17504E+01_JPRB, 0.16178E+01_JPRB, 0.14819E+01_JPRB, 0.13632E+01_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.12481E+01_JPRB, 0.11864E+01_JPRB, 0.11446E+01_JPRB, 0.11336E+01_JPRB, 0.11450E+01_JPRB, 0.11439E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.10571E+01_JPRB, 0.10316E+01_JPRB, 0.97890E+00_JPRB, 0.93131E+00_JPRB, 0.87257E+00_JPRB, 0.84565E+00_JPRB ,&
&  0.89681E+00_JPRB, 0.96526E+00_JPRB, 0.83455E+00_JPRB, 0.77754E+00_JPRB, 0.72656E+00_JPRB, 0.79661E+00_JPRB, 0.92652E+00_JPRB ,&
&  0.10671E+01_JPRB, 0.11711E+01_JPRB, 0.14002E+01_JPRB, 0.14730E+01_JPRB, 0.15798E+01_JPRB, 0.19299E+01_JPRB, 0.24098E+01_JPRB ,&
&  0.29881E+01_JPRB, 0.24932E+01_JPRB, 0.25000E+01_JPRB, 0.23356E+01_JPRB, 0.21755E+01_JPRB, 0.19720E+01_JPRB, 0.17635E+01_JPRB ,&
&  0.15693E+01_JPRB, 0.14099E+01_JPRB, 0.13263E+01_JPRB, 0.12632E+01_JPRB, 0.12139E+01_JPRB, 0.11789E+01_JPRB, 0.11552E+01_JPRB ,&
&  0.11401E+01_JPRB, 0.11268E+01_JPRB, 0.11083E+01_JPRB, 0.10841E+01_JPRB, 0.12120E+01_JPRB, 0.14167E+01_JPRB, 0.15712E+01_JPRB ,&
&  0.16454E+01_JPRB, 0.19110E+01_JPRB, 0.28611E+01_JPRB, 0.31715E+01_JPRB, 0.34854E+01_JPRB, 0.38270E+01_JPRB, 0.41740E+01_JPRB ,&
&  0.44562E+01_JPRB, 0.44873E+01_JPRB, 0.44371E+01_JPRB, 0.38277E+01_JPRB, 0.43083E+01_JPRB, 0.42870E+01_JPRB, 0.39913E+01_JPRB ,&
&  0.36286E+01_JPRB, 0.35242E+01_JPRB, 0.32078E+01_JPRB, 0.31269E+01_JPRB, 0.31952E+01_JPRB, 0.34178E+01_JPRB, 0.29765E+01_JPRB ,&
&  0.25000E+01_JPRB, 0.27491E+01_JPRB, 0.24276E+01_JPRB, 0.19543E+01_JPRB, 0.15702E+01_JPRB, 0.15714E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.16182E+01_JPRB, 0.15854E+01_JPRB, 0.16939E+01_JPRB, 0.19572E+01_JPRB, 0.20729E+01_JPRB, 0.20361E+01_JPRB ,&
&  0.19844E+01_JPRB, 0.21281E+01_JPRB, 0.21399E+01_JPRB, 0.20900E+01_JPRB, 0.20595E+01_JPRB, 0.19994E+01_JPRB, 0.18831E+01_JPRB ,&
&  0.17364E+01_JPRB, 0.16017E+01_JPRB, 0.14961E+01_JPRB, 0.14019E+01_JPRB, 0.13190E+01_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.11582E+01_JPRB, 0.11243E+01_JPRB, 0.10974E+01_JPRB, 0.10952E+01_JPRB, 0.11171E+01_JPRB, 0.11248E+01_JPRB, 0.11016E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.10594E+01_JPRB, 0.10356E+01_JPRB, 0.10014E+01_JPRB, 0.94865E+00_JPRB, 0.89350E+00_JPRB, 0.87671E+00_JPRB ,&
&  0.94375E+00_JPRB, 0.11359E+01_JPRB, 0.96826E+00_JPRB, 0.92775E+00_JPRB, 0.84866E+00_JPRB, 0.92133E+00_JPRB, 0.10041E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.11402E+01_JPRB, 0.11984E+01_JPRB, 0.12025E+01_JPRB, 0.12290E+01_JPRB, 0.14694E+01_JPRB, 0.18527E+01_JPRB ,&
&  0.20998E+01_JPRB, 0.22693E+01_JPRB, 0.21086E+01_JPRB, 0.22226E+01_JPRB, 0.20579E+01_JPRB, 0.18178E+01_JPRB, 0.18129E+01_JPRB ,&
&  0.16443E+01_JPRB, 0.14978E+01_JPRB, 0.13576E+01_JPRB, 0.12795E+01_JPRB, 0.12309E+01_JPRB, 0.11816E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.11037E+01_JPRB, 0.10880E+01_JPRB, 0.10977E+01_JPRB, 0.11015E+01_JPRB, 0.11298E+01_JPRB, 0.13262E+01_JPRB, 0.15861E+01_JPRB ,&
&  0.21147E+01_JPRB, 0.21549E+01_JPRB, 0.20950E+01_JPRB, 0.29105E+01_JPRB, 0.41777E+01_JPRB, 0.43430E+01_JPRB, 0.41543E+01_JPRB ,&
&  0.45146E+01_JPRB, 0.46823E+01_JPRB, 0.47491E+01_JPRB, 0.47400E+01_JPRB, 0.47746E+01_JPRB, 0.46220E+01_JPRB, 0.43522E+01_JPRB ,&
&  0.40264E+01_JPRB, 0.38250E+01_JPRB, 0.36366E+01_JPRB, 0.34569E+01_JPRB, 0.32099E+01_JPRB, 0.31620E+01_JPRB, 0.32731E+01_JPRB ,&
&  0.30279E+01_JPRB, 0.26328E+01_JPRB, 0.17750E+01_JPRB, 0.18225E+01_JPRB, 0.13984E+01_JPRB, 0.13574E+01_JPRB, 0.15491E+01_JPRB ,&
&  0.15248E+01_JPRB, 0.16220E+01_JPRB, 0.16304E+01_JPRB, 0.15264E+01_JPRB, 0.16455E+01_JPRB, 0.17968E+01_JPRB, 0.18030E+01_JPRB ,&
&  0.18681E+01_JPRB, 0.17272E+01_JPRB, 0.18609E+01_JPRB, 0.18093E+01_JPRB, 0.17342E+01_JPRB, 0.16755E+01_JPRB, 0.15975E+01_JPRB ,&
&  0.14788E+01_JPRB, 0.13502E+01_JPRB, 0.12538E+01_JPRB, 0.12064E+01_JPRB, 0.11786E+01_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.10128E+01_JPRB, 0.10129E+01_JPRB, 0.10140E+01_JPRB, 0.10234E+01_JPRB, 0.10545E+01_JPRB, 0.10751E+01_JPRB, 0.10622E+01_JPRB ,&
&  0.10436E+01_JPRB, 0.10298E+01_JPRB, 0.10126E+01_JPRB, 0.99075E+00_JPRB, 0.95459E+00_JPRB, 0.90399E+00_JPRB, 0.90106E+00_JPRB ,&
&  0.10695E+01_JPRB, 0.11472E+01_JPRB, 0.10417E+01_JPRB, 0.89236E+00_JPRB, 0.10314E+01_JPRB, 0.98573E+00_JPRB, 0.10248E+01_JPRB ,&
&  0.98185E+00_JPRB, 0.10822E+01_JPRB, 0.11269E+01_JPRB, 0.10674E+01_JPRB, 0.10261E+01_JPRB, 0.11823E+01_JPRB, 0.13856E+01_JPRB ,&
&  0.15691E+01_JPRB, 0.17048E+01_JPRB, 0.17385E+01_JPRB, 0.17566E+01_JPRB, 0.18263E+01_JPRB, 0.16616E+01_JPRB, 0.16440E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.14441E+01_JPRB, 0.13391E+01_JPRB, 0.12620E+01_JPRB, 0.11960E+01_JPRB, 0.11464E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.11018E+01_JPRB, 0.10947E+01_JPRB, 0.10790E+01_JPRB, 0.10966E+01_JPRB, 0.11414E+01_JPRB, 0.13946E+01_JPRB, 0.19172E+01_JPRB ,&
&  0.26264E+01_JPRB, 0.29495E+01_JPRB, 0.34035E+01_JPRB, 0.41554E+01_JPRB, 0.45076E+01_JPRB, 0.46381E+01_JPRB, 0.48231E+01_JPRB ,&
&  0.49440E+01_JPRB, 0.49803E+01_JPRB, 0.49281E+01_JPRB, 0.48030E+01_JPRB, 0.47912E+01_JPRB, 0.47343E+01_JPRB, 0.45262E+01_JPRB ,&
&  0.40957E+01_JPRB, 0.38717E+01_JPRB, 0.38650E+01_JPRB, 0.35937E+01_JPRB, 0.34093E+01_JPRB, 0.31456E+01_JPRB, 0.33007E+01_JPRB ,&
&  0.32214E+01_JPRB, 0.19279E+01_JPRB, 0.15270E+01_JPRB, 0.18248E+01_JPRB, 0.14875E+01_JPRB, 0.14656E+01_JPRB, 0.16890E+01_JPRB ,&
&  0.16663E+01_JPRB, 0.15885E+01_JPRB, 0.15745E+01_JPRB, 0.14576E+01_JPRB, 0.14243E+01_JPRB, 0.16407E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.14515E+01_JPRB, 0.14430E+01_JPRB, 0.15119E+01_JPRB, 0.14520E+01_JPRB, 0.13843E+01_JPRB, 0.13508E+01_JPRB, 0.12874E+01_JPRB ,&
&  0.11878E+01_JPRB, 0.11070E+01_JPRB, 0.10413E+01_JPRB, 0.10130E+01_JPRB, 0.10108E+01_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.91798E+00_JPRB, 0.94086E+00_JPRB, 0.95386E+00_JPRB, 0.96944E+00_JPRB, 0.99760E+00_JPRB, 0.10299E+01_JPRB, 0.10300E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.10013E+01_JPRB, 0.98957E+00_JPRB, 0.97062E+00_JPRB, 0.94790E+00_JPRB, 0.91365E+00_JPRB, 0.92402E+00_JPRB ,&
&  0.82424E+00_JPRB, 0.93544E+00_JPRB, 0.82822E+00_JPRB, 0.85776E+00_JPRB, 0.10787E+01_JPRB, 0.98370E+00_JPRB, 0.94326E+00_JPRB ,&
&  0.90307E+00_JPRB, 0.94338E+00_JPRB, 0.10027E+01_JPRB, 0.10163E+01_JPRB, 0.95035E+00_JPRB, 0.10100E+01_JPRB, 0.11422E+01_JPRB ,&
&  0.12022E+01_JPRB, 0.12189E+01_JPRB, 0.12720E+01_JPRB, 0.12893E+01_JPRB, 0.13319E+01_JPRB, 0.14508E+01_JPRB, 0.14946E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.13469E+01_JPRB, 0.12656E+01_JPRB, 0.12019E+01_JPRB, 0.11459E+01_JPRB, 0.10889E+01_JPRB, 0.10363E+01_JPRB ,&
&  0.99666E+00_JPRB, 0.97453E+00_JPRB, 0.98440E+00_JPRB, 0.10454E+01_JPRB, 0.11213E+01_JPRB, 0.14758E+01_JPRB, 0.21617E+01_JPRB ,&
&  0.27427E+01_JPRB, 0.29555E+01_JPRB, 0.34252E+01_JPRB, 0.42062E+01_JPRB, 0.44887E+01_JPRB, 0.46312E+01_JPRB, 0.46833E+01_JPRB ,&
&  0.48112E+01_JPRB, 0.48763E+01_JPRB, 0.48068E+01_JPRB, 0.48323E+01_JPRB, 0.46321E+01_JPRB, 0.46580E+01_JPRB, 0.43948E+01_JPRB ,&
&  0.39412E+01_JPRB, 0.38235E+01_JPRB, 0.39240E+01_JPRB, 0.38651E+01_JPRB, 0.37564E+01_JPRB, 0.37867E+01_JPRB, 0.37980E+01_JPRB ,&
&  0.36181E+01_JPRB, 0.31277E+01_JPRB, 0.23478E+01_JPRB, 0.21457E+01_JPRB, 0.15632E+01_JPRB, 0.15759E+01_JPRB, 0.19039E+01_JPRB ,&
&  0.18713E+01_JPRB, 0.16316E+01_JPRB, 0.14377E+01_JPRB, 0.13336E+01_JPRB, 0.13000E+01_JPRB, 0.14235E+01_JPRB, 0.13102E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11872E+01_JPRB, 0.12032E+01_JPRB, 0.12108E+01_JPRB, 0.12087E+01_JPRB, 0.11674E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10340E+01_JPRB, 0.96724E+00_JPRB, 0.90501E+00_JPRB, 0.88135E+00_JPRB, 0.89104E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.87058E+00_JPRB, 0.89821E+00_JPRB, 0.91537E+00_JPRB, 0.92702E+00_JPRB, 0.94687E+00_JPRB, 0.98033E+00_JPRB, 0.98486E+00_JPRB ,&
&  0.97352E+00_JPRB, 0.96188E+00_JPRB, 0.95406E+00_JPRB, 0.94020E+00_JPRB, 0.91364E+00_JPRB, 0.88633E+00_JPRB, 0.82649E+00_JPRB ,&
&  0.73655E+00_JPRB, 0.84093E+00_JPRB, 0.81289E+00_JPRB, 0.84787E+00_JPRB, 0.85502E+00_JPRB, 0.82742E+00_JPRB, 0.80020E+00_JPRB ,&
&  0.82873E+00_JPRB, 0.85884E+00_JPRB, 0.93273E+00_JPRB, 0.10071E+01_JPRB, 0.98108E+00_JPRB, 0.99103E+00_JPRB, 0.10837E+01_JPRB ,&
&  0.10895E+01_JPRB, 0.10878E+01_JPRB, 0.10430E+01_JPRB, 0.10334E+01_JPRB, 0.10759E+01_JPRB, 0.11580E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12470E+01_JPRB, 0.11965E+01_JPRB, 0.11404E+01_JPRB, 0.10940E+01_JPRB, 0.10550E+01_JPRB, 0.10137E+01_JPRB, 0.96918E+00_JPRB ,&
&  0.92676E+00_JPRB, 0.90163E+00_JPRB, 0.90442E+00_JPRB, 0.93560E+00_JPRB, 0.10178E+01_JPRB, 0.13004E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.17505E+01_JPRB, 0.20076E+01_JPRB, 0.23925E+01_JPRB, 0.28419E+01_JPRB, 0.32412E+01_JPRB, 0.35877E+01_JPRB, 0.38975E+01_JPRB ,&
&  0.42726E+01_JPRB, 0.43380E+01_JPRB, 0.48562E+01_JPRB, 0.48109E+01_JPRB, 0.46097E+01_JPRB, 0.45320E+01_JPRB, 0.42396E+01_JPRB ,&
&  0.39780E+01_JPRB, 0.37084E+01_JPRB, 0.39143E+01_JPRB, 0.39414E+01_JPRB, 0.38978E+01_JPRB, 0.38732E+01_JPRB, 0.37693E+01_JPRB ,&
&  0.36615E+01_JPRB, 0.35157E+01_JPRB, 0.30382E+01_JPRB, 0.26473E+01_JPRB, 0.21661E+01_JPRB, 0.21788E+01_JPRB, 0.19246E+01_JPRB ,&
&  0.16171E+01_JPRB, 0.12738E+01_JPRB, 0.12422E+01_JPRB, 0.12033E+01_JPRB, 0.12139E+01_JPRB, 0.11514E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.10676E+01_JPRB, 0.10886E+01_JPRB, 0.10465E+01_JPRB, 0.10360E+01_JPRB, 0.10538E+01_JPRB, 0.10196E+01_JPRB, 0.90694E+00_JPRB ,&
&  0.86231E+00_JPRB, 0.85254E+00_JPRB, 0.81495E+00_JPRB, 0.80934E+00_JPRB, 0.83253E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.83394E+00_JPRB, 0.85411E+00_JPRB, 0.86943E+00_JPRB, 0.88422E+00_JPRB, 0.90247E+00_JPRB, 0.92065E+00_JPRB, 0.90750E+00_JPRB ,&
&  0.90880E+00_JPRB, 0.90483E+00_JPRB, 0.90007E+00_JPRB, 0.88198E+00_JPRB, 0.84939E+00_JPRB, 0.76637E+00_JPRB, 0.65899E+00_JPRB ,&
&  0.64099E+00_JPRB, 0.68071E+00_JPRB, 0.72714E+00_JPRB, 0.74894E+00_JPRB, 0.75610E+00_JPRB, 0.76618E+00_JPRB, 0.76553E+00_JPRB ,&
&  0.77774E+00_JPRB, 0.80105E+00_JPRB, 0.85433E+00_JPRB, 0.94116E+00_JPRB, 0.10169E+01_JPRB, 0.10224E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.10083E+01_JPRB, 0.10092E+01_JPRB, 0.97817E+00_JPRB, 0.90257E+00_JPRB, 0.91530E+00_JPRB, 0.92325E+00_JPRB, 0.97109E+00_JPRB ,&
&  0.10197E+01_JPRB, 0.10214E+01_JPRB, 0.99634E+00_JPRB, 0.97335E+00_JPRB, 0.94984E+00_JPRB, 0.92126E+00_JPRB, 0.89147E+00_JPRB ,&
&  0.86369E+00_JPRB, 0.84631E+00_JPRB, 0.84660E+00_JPRB, 0.86140E+00_JPRB, 0.88660E+00_JPRB, 0.10583E+01_JPRB, 0.11909E+01_JPRB ,&
&  0.13232E+01_JPRB, 0.15863E+01_JPRB, 0.20149E+01_JPRB, 0.22183E+01_JPRB, 0.26815E+01_JPRB, 0.30657E+01_JPRB, 0.35026E+01_JPRB ,&
&  0.38924E+01_JPRB, 0.38881E+01_JPRB, 0.42362E+01_JPRB, 0.43255E+01_JPRB, 0.43326E+01_JPRB, 0.42018E+01_JPRB, 0.39734E+01_JPRB ,&
&  0.38440E+01_JPRB, 0.38707E+01_JPRB, 0.39659E+01_JPRB, 0.38964E+01_JPRB, 0.36955E+01_JPRB, 0.35625E+01_JPRB, 0.34172E+01_JPRB ,&
&  0.33366E+01_JPRB, 0.32508E+01_JPRB, 0.30721E+01_JPRB, 0.28184E+01_JPRB, 0.25080E+01_JPRB, 0.21835E+01_JPRB, 0.18874E+01_JPRB ,&
&  0.16317E+01_JPRB, 0.13493E+01_JPRB, 0.12191E+01_JPRB, 0.11866E+01_JPRB, 0.11657E+01_JPRB, 0.10604E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.95912E+00_JPRB, 0.97226E+00_JPRB, 0.94685E+00_JPRB, 0.95773E+00_JPRB, 0.94634E+00_JPRB, 0.87217E+00_JPRB ,&
&  0.81956E+00_JPRB, 0.82320E+00_JPRB, 0.80770E+00_JPRB, 0.79887E+00_JPRB, 0.81249E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.81497E+00_JPRB, 0.83188E+00_JPRB, 0.83642E+00_JPRB, 0.84340E+00_JPRB, 0.85417E+00_JPRB, 0.84274E+00_JPRB, 0.82703E+00_JPRB ,&
&  0.83208E+00_JPRB, 0.79397E+00_JPRB, 0.82774E+00_JPRB, 0.69377E+00_JPRB, 0.64068E+00_JPRB, 0.63272E+00_JPRB, 0.62340E+00_JPRB ,&
&  0.63355E+00_JPRB, 0.65857E+00_JPRB, 0.68184E+00_JPRB, 0.69928E+00_JPRB, 0.72491E+00_JPRB, 0.73141E+00_JPRB, 0.73578E+00_JPRB ,&
&  0.74620E+00_JPRB, 0.76548E+00_JPRB, 0.80909E+00_JPRB, 0.83623E+00_JPRB, 0.87131E+00_JPRB, 0.89984E+00_JPRB, 0.90196E+00_JPRB ,&
&  0.87171E+00_JPRB, 0.87524E+00_JPRB, 0.89648E+00_JPRB, 0.86680E+00_JPRB, 0.83453E+00_JPRB, 0.81756E+00_JPRB, 0.83614E+00_JPRB ,&
&  0.84216E+00_JPRB, 0.79511E+00_JPRB, 0.81896E+00_JPRB, 0.82115E+00_JPRB, 0.81605E+00_JPRB, 0.80915E+00_JPRB, 0.80256E+00_JPRB ,&
&  0.79873E+00_JPRB, 0.79753E+00_JPRB, 0.81171E+00_JPRB, 0.83471E+00_JPRB, 0.88063E+00_JPRB, 0.95842E+00_JPRB, 0.10063E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.11668E+01_JPRB, 0.15440E+01_JPRB, 0.19307E+01_JPRB, 0.25018E+01_JPRB, 0.28806E+01_JPRB, 0.32671E+01_JPRB ,&
&  0.37435E+01_JPRB, 0.37007E+01_JPRB, 0.38698E+01_JPRB, 0.38536E+01_JPRB, 0.38039E+01_JPRB, 0.37328E+01_JPRB, 0.36636E+01_JPRB ,&
&  0.36187E+01_JPRB, 0.34480E+01_JPRB, 0.36031E+01_JPRB, 0.34852E+01_JPRB, 0.32783E+01_JPRB, 0.31370E+01_JPRB, 0.30031E+01_JPRB ,&
&  0.28653E+01_JPRB, 0.27108E+01_JPRB, 0.25213E+01_JPRB, 0.21235E+01_JPRB, 0.19562E+01_JPRB, 0.18190E+01_JPRB, 0.16014E+01_JPRB ,&
&  0.14985E+01_JPRB, 0.13838E+01_JPRB, 0.12693E+01_JPRB, 0.12617E+01_JPRB, 0.11737E+01_JPRB, 0.10908E+01_JPRB, 0.10564E+01_JPRB ,&
&  0.98479E+00_JPRB, 0.89990E+00_JPRB, 0.89772E+00_JPRB, 0.88663E+00_JPRB, 0.87306E+00_JPRB, 0.85470E+00_JPRB, 0.88410E+00_JPRB ,&
&  0.83897E+00_JPRB, 0.81137E+00_JPRB, 0.82099E+00_JPRB, 0.80349E+00_JPRB, 0.80433E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.77881E+00_JPRB, 0.80130E+00_JPRB, 0.80818E+00_JPRB, 0.80804E+00_JPRB, 0.81024E+00_JPRB, 0.80024E+00_JPRB, 0.75453E+00_JPRB ,&
&  0.73520E+00_JPRB, 0.67830E+00_JPRB, 0.64416E+00_JPRB, 0.62418E+00_JPRB, 0.61753E+00_JPRB, 0.61050E+00_JPRB, 0.60095E+00_JPRB ,&
&  0.60785E+00_JPRB, 0.62841E+00_JPRB, 0.65466E+00_JPRB, 0.67409E+00_JPRB, 0.69034E+00_JPRB, 0.70297E+00_JPRB, 0.71525E+00_JPRB ,&
&  0.73184E+00_JPRB, 0.75056E+00_JPRB, 0.77274E+00_JPRB, 0.78899E+00_JPRB, 0.79672E+00_JPRB, 0.80122E+00_JPRB, 0.79638E+00_JPRB ,&
&  0.79436E+00_JPRB, 0.79158E+00_JPRB, 0.81354E+00_JPRB, 0.82054E+00_JPRB, 0.80827E+00_JPRB, 0.78758E+00_JPRB, 0.74887E+00_JPRB ,&
&  0.67378E+00_JPRB, 0.67589E+00_JPRB, 0.70193E+00_JPRB, 0.71807E+00_JPRB, 0.73068E+00_JPRB, 0.74175E+00_JPRB, 0.74987E+00_JPRB ,&
&  0.75932E+00_JPRB, 0.77567E+00_JPRB, 0.80338E+00_JPRB, 0.83451E+00_JPRB, 0.87695E+00_JPRB, 0.93760E+00_JPRB, 0.99219E+00_JPRB ,&
&  0.10611E+01_JPRB, 0.10540E+01_JPRB, 0.12403E+01_JPRB, 0.16265E+01_JPRB, 0.23987E+01_JPRB, 0.27058E+01_JPRB, 0.29244E+01_JPRB ,&
&  0.31381E+01_JPRB, 0.32152E+01_JPRB, 0.33483E+01_JPRB, 0.34550E+01_JPRB, 0.34496E+01_JPRB, 0.34190E+01_JPRB, 0.34186E+01_JPRB ,&
&  0.32718E+01_JPRB, 0.32521E+01_JPRB, 0.31447E+01_JPRB, 0.30963E+01_JPRB, 0.29663E+01_JPRB, 0.28536E+01_JPRB, 0.27031E+01_JPRB ,&
&  0.25188E+01_JPRB, 0.23234E+01_JPRB, 0.20725E+01_JPRB, 0.18318E+01_JPRB, 0.16430E+01_JPRB, 0.15467E+01_JPRB, 0.14637E+01_JPRB ,&
&  0.14014E+01_JPRB, 0.13637E+01_JPRB, 0.13112E+01_JPRB, 0.12624E+01_JPRB, 0.11821E+01_JPRB, 0.10802E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.93352E+00_JPRB, 0.87796E+00_JPRB, 0.82444E+00_JPRB, 0.80299E+00_JPRB, 0.80437E+00_JPRB, 0.79337E+00_JPRB, 0.77546E+00_JPRB ,&
&  0.79202E+00_JPRB, 0.77130E+00_JPRB, 0.75036E+00_JPRB, 0.75383E+00_JPRB, 0.75937E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.73415E+00_JPRB, 0.74971E+00_JPRB, 0.76507E+00_JPRB, 0.76864E+00_JPRB, 0.76530E+00_JPRB, 0.75721E+00_JPRB, 0.73135E+00_JPRB ,&
&  0.71881E+00_JPRB, 0.68924E+00_JPRB, 0.66524E+00_JPRB, 0.64236E+00_JPRB, 0.62713E+00_JPRB, 0.61858E+00_JPRB, 0.62262E+00_JPRB ,&
&  0.63295E+00_JPRB, 0.64478E+00_JPRB, 0.65716E+00_JPRB, 0.66764E+00_JPRB, 0.68081E+00_JPRB, 0.69795E+00_JPRB, 0.71963E+00_JPRB ,&
&  0.74186E+00_JPRB, 0.75710E+00_JPRB, 0.77036E+00_JPRB, 0.78014E+00_JPRB, 0.78750E+00_JPRB, 0.79016E+00_JPRB, 0.77726E+00_JPRB ,&
&  0.77681E+00_JPRB, 0.77527E+00_JPRB, 0.79010E+00_JPRB, 0.79587E+00_JPRB, 0.79370E+00_JPRB, 0.76020E+00_JPRB, 0.68257E+00_JPRB ,&
&  0.63135E+00_JPRB, 0.60477E+00_JPRB, 0.60869E+00_JPRB, 0.63599E+00_JPRB, 0.65680E+00_JPRB, 0.67736E+00_JPRB, 0.69464E+00_JPRB ,&
&  0.69700E+00_JPRB, 0.70524E+00_JPRB, 0.73581E+00_JPRB, 0.78254E+00_JPRB, 0.81833E+00_JPRB, 0.88837E+00_JPRB, 0.97552E+00_JPRB ,&
&  0.11090E+01_JPRB, 0.12982E+01_JPRB, 0.16110E+01_JPRB, 0.16944E+01_JPRB, 0.18892E+01_JPRB, 0.24518E+01_JPRB, 0.25119E+01_JPRB ,&
&  0.26016E+01_JPRB, 0.27142E+01_JPRB, 0.29188E+01_JPRB, 0.31353E+01_JPRB, 0.31247E+01_JPRB, 0.30925E+01_JPRB, 0.30277E+01_JPRB ,&
&  0.28356E+01_JPRB, 0.27402E+01_JPRB, 0.26469E+01_JPRB, 0.26728E+01_JPRB, 0.25998E+01_JPRB, 0.25015E+01_JPRB, 0.24054E+01_JPRB ,&
&  0.22371E+01_JPRB, 0.20026E+01_JPRB, 0.17754E+01_JPRB, 0.15752E+01_JPRB, 0.14754E+01_JPRB, 0.14161E+01_JPRB, 0.13782E+01_JPRB ,&
&  0.13334E+01_JPRB, 0.12975E+01_JPRB, 0.12577E+01_JPRB, 0.12162E+01_JPRB, 0.10987E+01_JPRB, 0.95762E+00_JPRB, 0.88202E+00_JPRB ,&
&  0.83859E+00_JPRB, 0.81770E+00_JPRB, 0.79075E+00_JPRB, 0.77856E+00_JPRB, 0.77144E+00_JPRB, 0.75248E+00_JPRB, 0.71427E+00_JPRB ,&
&  0.70051E+00_JPRB, 0.68912E+00_JPRB, 0.67433E+00_JPRB, 0.67940E+00_JPRB, 0.70780E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.70210E+00_JPRB, 0.70638E+00_JPRB, 0.70999E+00_JPRB, 0.71910E+00_JPRB, 0.71802E+00_JPRB, 0.70403E+00_JPRB, 0.67689E+00_JPRB ,&
&  0.65825E+00_JPRB, 0.65079E+00_JPRB, 0.65333E+00_JPRB, 0.65791E+00_JPRB, 0.66080E+00_JPRB, 0.66212E+00_JPRB, 0.66160E+00_JPRB ,&
&  0.65896E+00_JPRB, 0.66064E+00_JPRB, 0.66952E+00_JPRB, 0.68451E+00_JPRB, 0.70381E+00_JPRB, 0.72459E+00_JPRB, 0.74590E+00_JPRB ,&
&  0.76243E+00_JPRB, 0.76800E+00_JPRB, 0.77090E+00_JPRB, 0.76956E+00_JPRB, 0.77088E+00_JPRB, 0.77491E+00_JPRB, 0.77055E+00_JPRB ,&
&  0.76048E+00_JPRB, 0.76267E+00_JPRB, 0.77500E+00_JPRB, 0.77159E+00_JPRB, 0.76992E+00_JPRB, 0.73445E+00_JPRB, 0.67255E+00_JPRB ,&
&  0.61365E+00_JPRB, 0.56247E+00_JPRB, 0.54648E+00_JPRB, 0.54998E+00_JPRB, 0.56843E+00_JPRB, 0.59397E+00_JPRB, 0.62151E+00_JPRB ,&
&  0.64417E+00_JPRB, 0.66702E+00_JPRB, 0.68629E+00_JPRB, 0.69812E+00_JPRB, 0.71531E+00_JPRB, 0.75538E+00_JPRB, 0.83481E+00_JPRB ,&
&  0.97868E+00_JPRB, 0.11721E+01_JPRB, 0.14256E+01_JPRB, 0.14323E+01_JPRB, 0.15345E+01_JPRB, 0.17381E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.21638E+01_JPRB, 0.21620E+01_JPRB, 0.22290E+01_JPRB, 0.23839E+01_JPRB, 0.24959E+01_JPRB, 0.25895E+01_JPRB, 0.25425E+01_JPRB ,&
&  0.24575E+01_JPRB, 0.23481E+01_JPRB, 0.22456E+01_JPRB, 0.22070E+01_JPRB, 0.22077E+01_JPRB, 0.21910E+01_JPRB, 0.21207E+01_JPRB ,&
&  0.20065E+01_JPRB, 0.18308E+01_JPRB, 0.16249E+01_JPRB, 0.14517E+01_JPRB, 0.13976E+01_JPRB, 0.13966E+01_JPRB, 0.14050E+01_JPRB ,&
&  0.13680E+01_JPRB, 0.13111E+01_JPRB, 0.12415E+01_JPRB, 0.11825E+01_JPRB, 0.10626E+01_JPRB, 0.95269E+00_JPRB, 0.88658E+00_JPRB ,&
&  0.86261E+00_JPRB, 0.85212E+00_JPRB, 0.84245E+00_JPRB, 0.82853E+00_JPRB, 0.81167E+00_JPRB, 0.79542E+00_JPRB, 0.75670E+00_JPRB ,&
&  0.72362E+00_JPRB, 0.70082E+00_JPRB, 0.67922E+00_JPRB, 0.67477E+00_JPRB, 0.68438E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.71668E+00_JPRB, 0.71775E+00_JPRB, 0.71565E+00_JPRB, 0.71184E+00_JPRB, 0.69976E+00_JPRB, 0.69455E+00_JPRB, 0.68322E+00_JPRB ,&
&  0.67659E+00_JPRB, 0.67618E+00_JPRB, 0.67058E+00_JPRB, 0.66909E+00_JPRB, 0.66891E+00_JPRB, 0.66888E+00_JPRB, 0.67180E+00_JPRB ,&
&  0.67882E+00_JPRB, 0.69067E+00_JPRB, 0.70617E+00_JPRB, 0.72290E+00_JPRB, 0.74051E+00_JPRB, 0.75865E+00_JPRB, 0.77376E+00_JPRB ,&
&  0.78147E+00_JPRB, 0.78013E+00_JPRB, 0.78248E+00_JPRB, 0.77831E+00_JPRB, 0.76862E+00_JPRB, 0.75548E+00_JPRB, 0.74149E+00_JPRB ,&
&  0.74108E+00_JPRB, 0.75360E+00_JPRB, 0.75378E+00_JPRB, 0.75214E+00_JPRB, 0.74895E+00_JPRB, 0.70529E+00_JPRB, 0.64922E+00_JPRB ,&
&  0.58656E+00_JPRB, 0.53246E+00_JPRB, 0.51002E+00_JPRB, 0.50188E+00_JPRB, 0.51101E+00_JPRB, 0.52948E+00_JPRB, 0.55137E+00_JPRB ,&
&  0.56555E+00_JPRB, 0.59550E+00_JPRB, 0.62713E+00_JPRB, 0.65067E+00_JPRB, 0.67157E+00_JPRB, 0.69483E+00_JPRB, 0.71999E+00_JPRB ,&
&  0.76120E+00_JPRB, 0.83546E+00_JPRB, 0.97007E+00_JPRB, 0.11437E+01_JPRB, 0.13024E+01_JPRB, 0.13510E+01_JPRB, 0.14893E+01_JPRB ,&
&  0.18112E+01_JPRB, 0.17543E+01_JPRB, 0.17372E+01_JPRB, 0.17370E+01_JPRB, 0.17290E+01_JPRB, 0.17418E+01_JPRB, 0.17946E+01_JPRB ,&
&  0.18701E+01_JPRB, 0.19312E+01_JPRB, 0.19291E+01_JPRB, 0.18915E+01_JPRB, 0.18608E+01_JPRB, 0.18444E+01_JPRB, 0.18506E+01_JPRB ,&
&  0.18461E+01_JPRB, 0.18775E+01_JPRB, 0.19989E+01_JPRB, 0.16981E+01_JPRB, 0.15681E+01_JPRB, 0.15151E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.13838E+01_JPRB, 0.13229E+01_JPRB, 0.12593E+01_JPRB, 0.11864E+01_JPRB, 0.11011E+01_JPRB, 0.10191E+01_JPRB, 0.95985E+00_JPRB ,&
&  0.93349E+00_JPRB, 0.91750E+00_JPRB, 0.90679E+00_JPRB, 0.89159E+00_JPRB, 0.86414E+00_JPRB, 0.84056E+00_JPRB, 0.82075E+00_JPRB ,&
&  0.79292E+00_JPRB, 0.75951E+00_JPRB, 0.73525E+00_JPRB, 0.72657E+00_JPRB, 0.71835E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.74699E+00_JPRB, 0.74537E+00_JPRB, 0.74135E+00_JPRB, 0.74199E+00_JPRB, 0.73446E+00_JPRB, 0.73098E+00_JPRB, 0.71811E+00_JPRB ,&
&  0.71007E+00_JPRB, 0.70416E+00_JPRB, 0.70127E+00_JPRB, 0.70208E+00_JPRB, 0.70545E+00_JPRB, 0.71346E+00_JPRB, 0.72474E+00_JPRB ,&
&  0.73857E+00_JPRB, 0.74363E+00_JPRB, 0.75419E+00_JPRB, 0.76857E+00_JPRB, 0.77796E+00_JPRB, 0.78884E+00_JPRB, 0.79534E+00_JPRB ,&
&  0.79569E+00_JPRB, 0.78817E+00_JPRB, 0.78173E+00_JPRB, 0.76772E+00_JPRB, 0.75668E+00_JPRB, 0.74586E+00_JPRB, 0.74230E+00_JPRB ,&
&  0.72821E+00_JPRB, 0.72120E+00_JPRB, 0.74179E+00_JPRB, 0.74779E+00_JPRB, 0.70360E+00_JPRB, 0.65750E+00_JPRB, 0.61466E+00_JPRB ,&
&  0.56544E+00_JPRB, 0.51617E+00_JPRB, 0.49893E+00_JPRB, 0.48883E+00_JPRB, 0.49362E+00_JPRB, 0.50190E+00_JPRB, 0.51936E+00_JPRB ,&
&  0.53534E+00_JPRB, 0.55845E+00_JPRB, 0.58165E+00_JPRB, 0.61119E+00_JPRB, 0.63410E+00_JPRB, 0.66142E+00_JPRB, 0.68308E+00_JPRB ,&
&  0.71003E+00_JPRB, 0.74762E+00_JPRB, 0.80604E+00_JPRB, 0.88314E+00_JPRB, 0.94785E+00_JPRB, 0.10075E+01_JPRB, 0.10584E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.11990E+01_JPRB, 0.12489E+01_JPRB, 0.12766E+01_JPRB, 0.13029E+01_JPRB, 0.13434E+01_JPRB, 0.13725E+01_JPRB ,&
&  0.14385E+01_JPRB, 0.15521E+01_JPRB, 0.15875E+01_JPRB, 0.15977E+01_JPRB, 0.15932E+01_JPRB, 0.16076E+01_JPRB, 0.16389E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.16594E+01_JPRB, 0.17553E+01_JPRB, 0.16765E+01_JPRB, 0.16136E+01_JPRB, 0.15442E+01_JPRB, 0.14663E+01_JPRB ,&
&  0.13911E+01_JPRB, 0.13221E+01_JPRB, 0.12496E+01_JPRB, 0.11795E+01_JPRB, 0.11206E+01_JPRB, 0.10675E+01_JPRB, 0.10258E+01_JPRB ,&
&  0.98330E+00_JPRB, 0.94633E+00_JPRB, 0.91855E+00_JPRB, 0.89633E+00_JPRB, 0.86915E+00_JPRB, 0.84195E+00_JPRB, 0.82078E+00_JPRB ,&
&  0.79538E+00_JPRB, 0.77755E+00_JPRB, 0.76208E+00_JPRB, 0.75471E+00_JPRB, 0.74924E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.79293E+00_JPRB, 0.79772E+00_JPRB, 0.80322E+00_JPRB, 0.80655E+00_JPRB, 0.80421E+00_JPRB, 0.80248E+00_JPRB, 0.79487E+00_JPRB ,&
&  0.79146E+00_JPRB, 0.78776E+00_JPRB, 0.78565E+00_JPRB, 0.78645E+00_JPRB, 0.78733E+00_JPRB, 0.79095E+00_JPRB, 0.79501E+00_JPRB ,&
&  0.79857E+00_JPRB, 0.80320E+00_JPRB, 0.80437E+00_JPRB, 0.80686E+00_JPRB, 0.80787E+00_JPRB, 0.81227E+00_JPRB, 0.81194E+00_JPRB ,&
&  0.81170E+00_JPRB, 0.80078E+00_JPRB, 0.77888E+00_JPRB, 0.76525E+00_JPRB, 0.76110E+00_JPRB, 0.73653E+00_JPRB, 0.73515E+00_JPRB ,&
&  0.73642E+00_JPRB, 0.73338E+00_JPRB, 0.72293E+00_JPRB, 0.71651E+00_JPRB, 0.68902E+00_JPRB, 0.58974E+00_JPRB, 0.58270E+00_JPRB ,&
&  0.55803E+00_JPRB, 0.52579E+00_JPRB, 0.50898E+00_JPRB, 0.49608E+00_JPRB, 0.49176E+00_JPRB, 0.50323E+00_JPRB, 0.52137E+00_JPRB ,&
&  0.53255E+00_JPRB, 0.57016E+00_JPRB, 0.58621E+00_JPRB, 0.60987E+00_JPRB, 0.63128E+00_JPRB, 0.66156E+00_JPRB, 0.68003E+00_JPRB ,&
&  0.70159E+00_JPRB, 0.72485E+00_JPRB, 0.76597E+00_JPRB, 0.82466E+00_JPRB, 0.87319E+00_JPRB, 0.91424E+00_JPRB, 0.93621E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10238E+01_JPRB, 0.10648E+01_JPRB, 0.10966E+01_JPRB, 0.11230E+01_JPRB, 0.11527E+01_JPRB, 0.11759E+01_JPRB ,&
&  0.12167E+01_JPRB, 0.12814E+01_JPRB, 0.13427E+01_JPRB, 0.13805E+01_JPRB, 0.14027E+01_JPRB, 0.14285E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.14895E+01_JPRB, 0.15140E+01_JPRB, 0.15118E+01_JPRB, 0.14667E+01_JPRB, 0.14380E+01_JPRB, 0.13821E+01_JPRB, 0.13424E+01_JPRB ,&
&  0.12897E+01_JPRB, 0.12619E+01_JPRB, 0.12080E+01_JPRB, 0.11684E+01_JPRB, 0.11236E+01_JPRB, 0.10883E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.99129E+00_JPRB, 0.94063E+00_JPRB, 0.90198E+00_JPRB, 0.87349E+00_JPRB, 0.85085E+00_JPRB, 0.82596E+00_JPRB, 0.81052E+00_JPRB ,&
&  0.79848E+00_JPRB, 0.78945E+00_JPRB, 0.78250E+00_JPRB, 0.78493E+00_JPRB, 0.78666E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.81025E+00_JPRB, 0.81853E+00_JPRB, 0.83241E+00_JPRB, 0.84425E+00_JPRB, 0.85285E+00_JPRB, 0.85828E+00_JPRB, 0.85764E+00_JPRB ,&
&  0.85631E+00_JPRB, 0.85174E+00_JPRB, 0.84903E+00_JPRB, 0.84534E+00_JPRB, 0.84400E+00_JPRB, 0.84071E+00_JPRB, 0.83916E+00_JPRB ,&
&  0.83673E+00_JPRB, 0.83442E+00_JPRB, 0.83087E+00_JPRB, 0.82749E+00_JPRB, 0.82427E+00_JPRB, 0.82260E+00_JPRB, 0.81790E+00_JPRB ,&
&  0.81463E+00_JPRB, 0.80333E+00_JPRB, 0.77724E+00_JPRB, 0.77114E+00_JPRB, 0.75821E+00_JPRB, 0.74240E+00_JPRB, 0.72535E+00_JPRB ,&
&  0.70657E+00_JPRB, 0.65791E+00_JPRB, 0.67332E+00_JPRB, 0.65125E+00_JPRB, 0.62821E+00_JPRB, 0.60581E+00_JPRB, 0.58212E+00_JPRB ,&
&  0.56745E+00_JPRB, 0.54867E+00_JPRB, 0.53441E+00_JPRB, 0.52846E+00_JPRB, 0.52763E+00_JPRB, 0.53786E+00_JPRB, 0.55238E+00_JPRB ,&
&  0.56716E+00_JPRB, 0.60058E+00_JPRB, 0.61883E+00_JPRB, 0.64150E+00_JPRB, 0.66395E+00_JPRB, 0.69152E+00_JPRB, 0.70276E+00_JPRB ,&
&  0.72410E+00_JPRB, 0.73237E+00_JPRB, 0.76046E+00_JPRB, 0.79865E+00_JPRB, 0.84319E+00_JPRB, 0.88082E+00_JPRB, 0.91382E+00_JPRB ,&
&  0.96257E+00_JPRB, 0.99773E+00_JPRB, 0.10298E+01_JPRB, 0.10518E+01_JPRB, 0.10791E+01_JPRB, 0.10999E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11358E+01_JPRB, 0.11592E+01_JPRB, 0.11863E+01_JPRB, 0.12175E+01_JPRB, 0.12352E+01_JPRB, 0.12586E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.13094E+01_JPRB, 0.13192E+01_JPRB, 0.13440E+01_JPRB, 0.13275E+01_JPRB, 0.13170E+01_JPRB, 0.12735E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.11785E+01_JPRB, 0.11467E+01_JPRB, 0.11168E+01_JPRB, 0.10882E+01_JPRB, 0.10623E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.99736E+00_JPRB, 0.95225E+00_JPRB, 0.91435E+00_JPRB, 0.87453E+00_JPRB, 0.84989E+00_JPRB, 0.83055E+00_JPRB, 0.82111E+00_JPRB ,&
&  0.81394E+00_JPRB, 0.81001E+00_JPRB, 0.80732E+00_JPRB, 0.80988E+00_JPRB, 0.80943E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.86590E+00_JPRB, 0.86643E+00_JPRB, 0.86825E+00_JPRB, 0.86883E+00_JPRB, 0.86994E+00_JPRB, 0.86914E+00_JPRB, 0.87068E+00_JPRB ,&
&  0.87077E+00_JPRB, 0.86919E+00_JPRB, 0.86693E+00_JPRB, 0.86360E+00_JPRB, 0.86168E+00_JPRB, 0.85777E+00_JPRB, 0.85374E+00_JPRB ,&
&  0.84855E+00_JPRB, 0.84288E+00_JPRB, 0.83911E+00_JPRB, 0.83646E+00_JPRB, 0.83271E+00_JPRB, 0.82988E+00_JPRB, 0.82004E+00_JPRB ,&
&  0.81285E+00_JPRB, 0.80593E+00_JPRB, 0.77164E+00_JPRB, 0.77197E+00_JPRB, 0.75688E+00_JPRB, 0.74117E+00_JPRB, 0.72915E+00_JPRB ,&
&  0.70108E+00_JPRB, 0.68764E+00_JPRB, 0.66269E+00_JPRB, 0.64173E+00_JPRB, 0.63596E+00_JPRB, 0.61750E+00_JPRB, 0.61293E+00_JPRB ,&
&  0.60585E+00_JPRB, 0.59970E+00_JPRB, 0.59703E+00_JPRB, 0.59849E+00_JPRB, 0.60576E+00_JPRB, 0.61561E+00_JPRB, 0.62676E+00_JPRB ,&
&  0.64551E+00_JPRB, 0.66414E+00_JPRB, 0.69139E+00_JPRB, 0.70569E+00_JPRB, 0.73046E+00_JPRB, 0.75871E+00_JPRB, 0.78859E+00_JPRB ,&
&  0.80766E+00_JPRB, 0.80427E+00_JPRB, 0.81148E+00_JPRB, 0.82784E+00_JPRB, 0.84360E+00_JPRB, 0.86139E+00_JPRB, 0.89026E+00_JPRB ,&
&  0.92891E+00_JPRB, 0.96683E+00_JPRB, 0.99097E+00_JPRB, 0.10016E+01_JPRB, 0.10127E+01_JPRB, 0.10221E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10745E+01_JPRB, 0.10905E+01_JPRB, 0.11068E+01_JPRB, 0.11145E+01_JPRB, 0.11272E+01_JPRB, 0.11342E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11449E+01_JPRB, 0.11577E+01_JPRB, 0.11437E+01_JPRB, 0.11460E+01_JPRB, 0.11415E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.11131E+01_JPRB, 0.10981E+01_JPRB, 0.10799E+01_JPRB, 0.10606E+01_JPRB, 0.10381E+01_JPRB, 0.10180E+01_JPRB, 0.99717E+00_JPRB ,&
&  0.97764E+00_JPRB, 0.95437E+00_JPRB, 0.93405E+00_JPRB, 0.90471E+00_JPRB, 0.88791E+00_JPRB, 0.87167E+00_JPRB, 0.86471E+00_JPRB ,&
&  0.85898E+00_JPRB, 0.85927E+00_JPRB, 0.85830E+00_JPRB, 0.86194E+00_JPRB, 0.86364E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.90163E+00_JPRB, 0.90260E+00_JPRB, 0.90308E+00_JPRB, 0.90234E+00_JPRB, 0.90150E+00_JPRB, 0.89997E+00_JPRB, 0.89789E+00_JPRB ,&
&  0.89523E+00_JPRB, 0.89167E+00_JPRB, 0.88769E+00_JPRB, 0.88318E+00_JPRB, 0.87846E+00_JPRB, 0.87276E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.86304E+00_JPRB, 0.85769E+00_JPRB, 0.85281E+00_JPRB, 0.84831E+00_JPRB, 0.84335E+00_JPRB, 0.83852E+00_JPRB, 0.82884E+00_JPRB ,&
&  0.82265E+00_JPRB, 0.81701E+00_JPRB, 0.80958E+00_JPRB, 0.79882E+00_JPRB, 0.79275E+00_JPRB, 0.75735E+00_JPRB, 0.75724E+00_JPRB ,&
&  0.77096E+00_JPRB, 0.76582E+00_JPRB, 0.75942E+00_JPRB, 0.75186E+00_JPRB, 0.73279E+00_JPRB, 0.71802E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.68361E+00_JPRB, 0.69452E+00_JPRB, 0.69617E+00_JPRB, 0.69745E+00_JPRB, 0.70261E+00_JPRB, 0.70328E+00_JPRB, 0.70941E+00_JPRB ,&
&  0.73418E+00_JPRB, 0.74196E+00_JPRB, 0.75830E+00_JPRB, 0.77793E+00_JPRB, 0.79980E+00_JPRB, 0.82158E+00_JPRB, 0.84312E+00_JPRB ,&
&  0.85936E+00_JPRB, 0.87065E+00_JPRB, 0.87844E+00_JPRB, 0.88428E+00_JPRB, 0.89278E+00_JPRB, 0.90406E+00_JPRB, 0.92253E+00_JPRB ,&
&  0.95121E+00_JPRB, 0.98246E+00_JPRB, 0.10037E+01_JPRB, 0.10134E+01_JPRB, 0.10285E+01_JPRB, 0.10439E+01_JPRB, 0.10587E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10803E+01_JPRB, 0.10864E+01_JPRB, 0.10914E+01_JPRB, 0.10931E+01_JPRB, 0.10898E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10666E+01_JPRB, 0.10575E+01_JPRB, 0.10479E+01_JPRB, 0.10435E+01_JPRB, 0.10386E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10357E+01_JPRB, 0.10336E+01_JPRB, 0.10258E+01_JPRB, 0.10166E+01_JPRB, 0.10018E+01_JPRB, 0.98707E+00_JPRB, 0.96924E+00_JPRB ,&
&  0.95460E+00_JPRB, 0.94187E+00_JPRB, 0.93116E+00_JPRB, 0.91821E+00_JPRB, 0.91065E+00_JPRB, 0.90306E+00_JPRB, 0.89922E+00_JPRB ,&
&  0.89505E+00_JPRB, 0.89617E+00_JPRB, 0.89679E+00_JPRB, 0.89901E+00_JPRB, 0.89981E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.90402E+00_JPRB, 0.90383E+00_JPRB, 0.90325E+00_JPRB, 0.90226E+00_JPRB, 0.90105E+00_JPRB, 0.89976E+00_JPRB, 0.89797E+00_JPRB ,&
&  0.89567E+00_JPRB, 0.89317E+00_JPRB, 0.89081E+00_JPRB, 0.88823E+00_JPRB, 0.88526E+00_JPRB, 0.88227E+00_JPRB, 0.87982E+00_JPRB ,&
&  0.87720E+00_JPRB, 0.87434E+00_JPRB, 0.87058E+00_JPRB, 0.86747E+00_JPRB, 0.86447E+00_JPRB, 0.86101E+00_JPRB, 0.85856E+00_JPRB ,&
&  0.85618E+00_JPRB, 0.85196E+00_JPRB, 0.84618E+00_JPRB, 0.83592E+00_JPRB, 0.82863E+00_JPRB, 0.82173E+00_JPRB, 0.81533E+00_JPRB ,&
&  0.80872E+00_JPRB, 0.80745E+00_JPRB, 0.80663E+00_JPRB, 0.80583E+00_JPRB, 0.80810E+00_JPRB, 0.80705E+00_JPRB, 0.80526E+00_JPRB ,&
&  0.80170E+00_JPRB, 0.79579E+00_JPRB, 0.79716E+00_JPRB, 0.79887E+00_JPRB, 0.80056E+00_JPRB, 0.80577E+00_JPRB, 0.81210E+00_JPRB ,&
&  0.81954E+00_JPRB, 0.82787E+00_JPRB, 0.84012E+00_JPRB, 0.85433E+00_JPRB, 0.86817E+00_JPRB, 0.88137E+00_JPRB, 0.89443E+00_JPRB ,&
&  0.90533E+00_JPRB, 0.91691E+00_JPRB, 0.92980E+00_JPRB, 0.94442E+00_JPRB, 0.95836E+00_JPRB, 0.97373E+00_JPRB, 0.98675E+00_JPRB ,&
&  0.99511E+00_JPRB, 0.10029E+01_JPRB, 0.10147E+01_JPRB, 0.10285E+01_JPRB, 0.10439E+01_JPRB, 0.10530E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.10657E+01_JPRB, 0.10708E+01_JPRB, 0.10714E+01_JPRB, 0.10703E+01_JPRB, 0.10667E+01_JPRB, 0.10610E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10369E+01_JPRB, 0.10295E+01_JPRB, 0.10218E+01_JPRB, 0.10151E+01_JPRB, 0.10081E+01_JPRB, 0.10030E+01_JPRB ,&
&  0.99665E+00_JPRB, 0.99130E+00_JPRB, 0.98543E+00_JPRB, 0.97991E+00_JPRB, 0.97137E+00_JPRB, 0.96274E+00_JPRB, 0.95305E+00_JPRB ,&
&  0.94497E+00_JPRB, 0.93616E+00_JPRB, 0.92818E+00_JPRB, 0.92087E+00_JPRB, 0.91654E+00_JPRB, 0.91304E+00_JPRB, 0.91027E+00_JPRB ,&
&  0.90761E+00_JPRB, 0.90643E+00_JPRB, 0.90574E+00_JPRB, 0.90499E+00_JPRB, 0.90431E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.90999E+00_JPRB, 0.90884E+00_JPRB, 0.90759E+00_JPRB, 0.90636E+00_JPRB, 0.90487E+00_JPRB, 0.90343E+00_JPRB, 0.90190E+00_JPRB ,&
&  0.90017E+00_JPRB, 0.89841E+00_JPRB, 0.89695E+00_JPRB, 0.89562E+00_JPRB, 0.89431E+00_JPRB, 0.89295E+00_JPRB, 0.89185E+00_JPRB ,&
&  0.89093E+00_JPRB, 0.88977E+00_JPRB, 0.88845E+00_JPRB, 0.88767E+00_JPRB, 0.88709E+00_JPRB, 0.88626E+00_JPRB, 0.88564E+00_JPRB ,&
&  0.88475E+00_JPRB, 0.88285E+00_JPRB, 0.88063E+00_JPRB, 0.87935E+00_JPRB, 0.87836E+00_JPRB, 0.87759E+00_JPRB, 0.87674E+00_JPRB ,&
&  0.87593E+00_JPRB, 0.87546E+00_JPRB, 0.87474E+00_JPRB, 0.87404E+00_JPRB, 0.87413E+00_JPRB, 0.87472E+00_JPRB, 0.87550E+00_JPRB ,&
&  0.87635E+00_JPRB, 0.87749E+00_JPRB, 0.87924E+00_JPRB, 0.88099E+00_JPRB, 0.88276E+00_JPRB, 0.88461E+00_JPRB, 0.88704E+00_JPRB ,&
&  0.88916E+00_JPRB, 0.89137E+00_JPRB, 0.89439E+00_JPRB, 0.89783E+00_JPRB, 0.90138E+00_JPRB, 0.90516E+00_JPRB, 0.90901E+00_JPRB ,&
&  0.91400E+00_JPRB, 0.91974E+00_JPRB, 0.92596E+00_JPRB, 0.93214E+00_JPRB, 0.93794E+00_JPRB, 0.94386E+00_JPRB, 0.95030E+00_JPRB ,&
&  0.95604E+00_JPRB, 0.96069E+00_JPRB, 0.96575E+00_JPRB, 0.97116E+00_JPRB, 0.97595E+00_JPRB, 0.97834E+00_JPRB, 0.98103E+00_JPRB ,&
&  0.98405E+00_JPRB, 0.98682E+00_JPRB, 0.98649E+00_JPRB, 0.98659E+00_JPRB, 0.98661E+00_JPRB, 0.98616E+00_JPRB, 0.98480E+00_JPRB ,&
&  0.98346E+00_JPRB, 0.98231E+00_JPRB, 0.98008E+00_JPRB, 0.97693E+00_JPRB, 0.97405E+00_JPRB, 0.97093E+00_JPRB, 0.96742E+00_JPRB ,&
&  0.96357E+00_JPRB, 0.96012E+00_JPRB, 0.95677E+00_JPRB, 0.95319E+00_JPRB, 0.94909E+00_JPRB, 0.94530E+00_JPRB, 0.94147E+00_JPRB ,&
&  0.93787E+00_JPRB, 0.93406E+00_JPRB, 0.93054E+00_JPRB, 0.92714E+00_JPRB, 0.92430E+00_JPRB, 0.92180E+00_JPRB, 0.91960E+00_JPRB ,&
&  0.91747E+00_JPRB, 0.91570E+00_JPRB, 0.91423E+00_JPRB, 0.91276E+00_JPRB, 0.91135E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB ,&
&  0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB, 0.91239E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB ,&
&  0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB, 0.24276E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.27200E+00_JPRB, 0.27201E+00_JPRB, 0.27207E+00_JPRB, 0.27173E+00_JPRB, 0.27019E+00_JPRB, 0.26901E+00_JPRB, 0.26797E+00_JPRB ,&
&  0.26744E+00_JPRB, 0.26657E+00_JPRB, 0.26613E+00_JPRB, 0.26594E+00_JPRB, 0.26554E+00_JPRB, 0.26497E+00_JPRB, 0.26421E+00_JPRB ,&
&  0.26323E+00_JPRB, 0.26195E+00_JPRB, 0.26063E+00_JPRB, 0.26037E+00_JPRB, 0.26047E+00_JPRB, 0.26058E+00_JPRB, 0.26063E+00_JPRB ,&
&  0.26056E+00_JPRB, 0.26048E+00_JPRB, 0.25996E+00_JPRB, 0.26002E+00_JPRB, 0.26002E+00_JPRB, 0.26002E+00_JPRB, 0.26013E+00_JPRB ,&
&  0.26035E+00_JPRB, 0.26054E+00_JPRB, 0.26031E+00_JPRB, 0.25984E+00_JPRB, 0.25895E+00_JPRB, 0.25727E+00_JPRB, 0.25464E+00_JPRB ,&
&  0.25166E+00_JPRB, 0.24867E+00_JPRB, 0.24627E+00_JPRB, 0.24407E+00_JPRB, 0.24205E+00_JPRB, 0.24021E+00_JPRB, 0.23868E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.23506E+00_JPRB, 0.23340E+00_JPRB, 0.23223E+00_JPRB, 0.23095E+00_JPRB, 0.22983E+00_JPRB, 0.22882E+00_JPRB ,&
&  0.22812E+00_JPRB, 0.22751E+00_JPRB, 0.22698E+00_JPRB, 0.22671E+00_JPRB, 0.22649E+00_JPRB, 0.22630E+00_JPRB, 0.22607E+00_JPRB ,&
&  0.22587E+00_JPRB, 0.22572E+00_JPRB, 0.22560E+00_JPRB, 0.22556E+00_JPRB, 0.22565E+00_JPRB, 0.22553E+00_JPRB, 0.22539E+00_JPRB ,&
&  0.22534E+00_JPRB, 0.22559E+00_JPRB, 0.22578E+00_JPRB, 0.22608E+00_JPRB, 0.22656E+00_JPRB, 0.22723E+00_JPRB, 0.22796E+00_JPRB ,&
&  0.22869E+00_JPRB, 0.22955E+00_JPRB, 0.23048E+00_JPRB, 0.23161E+00_JPRB, 0.23282E+00_JPRB, 0.23403E+00_JPRB, 0.23519E+00_JPRB ,&
&  0.23643E+00_JPRB, 0.23757E+00_JPRB, 0.23864E+00_JPRB, 0.23941E+00_JPRB, 0.24049E+00_JPRB, 0.24157E+00_JPRB, 0.24284E+00_JPRB ,&
&  0.24429E+00_JPRB, 0.24620E+00_JPRB, 0.24885E+00_JPRB, 0.25200E+00_JPRB, 0.25593E+00_JPRB, 0.25884E+00_JPRB, 0.26182E+00_JPRB ,&
&  0.26397E+00_JPRB, 0.26565E+00_JPRB, 0.26689E+00_JPRB, 0.26846E+00_JPRB, 0.27020E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.30716E+00_JPRB, 0.31292E+00_JPRB, 0.31694E+00_JPRB, 0.32604E+00_JPRB, 0.33926E+00_JPRB, 0.43598E+00_JPRB, 0.43100E+00_JPRB ,&
&  0.43008E+00_JPRB, 0.38721E+00_JPRB, 0.37915E+00_JPRB, 0.36699E+00_JPRB, 0.35166E+00_JPRB, 0.33271E+00_JPRB, 0.32096E+00_JPRB ,&
&  0.31117E+00_JPRB, 0.29980E+00_JPRB, 0.28911E+00_JPRB, 0.28285E+00_JPRB, 0.27788E+00_JPRB, 0.27350E+00_JPRB, 0.27097E+00_JPRB ,&
&  0.27003E+00_JPRB, 0.27050E+00_JPRB, 0.27129E+00_JPRB, 0.27248E+00_JPRB, 0.27584E+00_JPRB, 0.27951E+00_JPRB, 0.28317E+00_JPRB ,&
&  0.28499E+00_JPRB, 0.28913E+00_JPRB, 0.28992E+00_JPRB, 0.28673E+00_JPRB, 0.28153E+00_JPRB, 0.27434E+00_JPRB, 0.26708E+00_JPRB ,&
&  0.25961E+00_JPRB, 0.25267E+00_JPRB, 0.24692E+00_JPRB, 0.24171E+00_JPRB, 0.23580E+00_JPRB, 0.23025E+00_JPRB, 0.22669E+00_JPRB ,&
&  0.22347E+00_JPRB, 0.22020E+00_JPRB, 0.21763E+00_JPRB, 0.21635E+00_JPRB, 0.21489E+00_JPRB, 0.21392E+00_JPRB, 0.21321E+00_JPRB ,&
&  0.21301E+00_JPRB, 0.21291E+00_JPRB, 0.21298E+00_JPRB, 0.21327E+00_JPRB, 0.21325E+00_JPRB, 0.21317E+00_JPRB, 0.21314E+00_JPRB ,&
&  0.21369E+00_JPRB, 0.21371E+00_JPRB, 0.21363E+00_JPRB, 0.21339E+00_JPRB, 0.21402E+00_JPRB, 0.21420E+00_JPRB, 0.21454E+00_JPRB ,&
&  0.21513E+00_JPRB, 0.21596E+00_JPRB, 0.21660E+00_JPRB, 0.21734E+00_JPRB, 0.21806E+00_JPRB, 0.21892E+00_JPRB, 0.21993E+00_JPRB ,&
&  0.22095E+00_JPRB, 0.22245E+00_JPRB, 0.22398E+00_JPRB, 0.22535E+00_JPRB, 0.22656E+00_JPRB, 0.22821E+00_JPRB, 0.23006E+00_JPRB ,&
&  0.23282E+00_JPRB, 0.23550E+00_JPRB, 0.23825E+00_JPRB, 0.23993E+00_JPRB, 0.24262E+00_JPRB, 0.24488E+00_JPRB, 0.24726E+00_JPRB ,&
&  0.25003E+00_JPRB, 0.25446E+00_JPRB, 0.26087E+00_JPRB, 0.26783E+00_JPRB, 0.27555E+00_JPRB, 0.28221E+00_JPRB, 0.28662E+00_JPRB ,&
&  0.29047E+00_JPRB, 0.29462E+00_JPRB, 0.30031E+00_JPRB, 0.30333E+00_JPRB, 0.30574E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.52578E+00_JPRB, 0.52823E+00_JPRB, 0.53089E+00_JPRB, 0.53503E+00_JPRB, 0.53945E+00_JPRB, 0.54384E+00_JPRB, 0.54765E+00_JPRB ,&
&  0.52942E+00_JPRB, 0.50630E+00_JPRB, 0.48733E+00_JPRB, 0.46475E+00_JPRB, 0.44416E+00_JPRB, 0.42717E+00_JPRB, 0.40703E+00_JPRB ,&
&  0.38694E+00_JPRB, 0.36686E+00_JPRB, 0.34420E+00_JPRB, 0.32560E+00_JPRB, 0.30729E+00_JPRB, 0.29248E+00_JPRB, 0.28343E+00_JPRB ,&
&  0.27850E+00_JPRB, 0.27715E+00_JPRB, 0.27862E+00_JPRB, 0.28164E+00_JPRB, 0.28449E+00_JPRB, 0.28914E+00_JPRB, 0.29562E+00_JPRB ,&
&  0.30456E+00_JPRB, 0.31909E+00_JPRB, 0.33219E+00_JPRB, 0.33693E+00_JPRB, 0.33337E+00_JPRB, 0.32490E+00_JPRB, 0.31762E+00_JPRB ,&
&  0.30883E+00_JPRB, 0.29665E+00_JPRB, 0.28184E+00_JPRB, 0.26719E+00_JPRB, 0.25355E+00_JPRB, 0.24360E+00_JPRB, 0.23533E+00_JPRB ,&
&  0.22835E+00_JPRB, 0.22220E+00_JPRB, 0.21748E+00_JPRB, 0.21429E+00_JPRB, 0.21156E+00_JPRB, 0.20937E+00_JPRB, 0.20729E+00_JPRB ,&
&  0.20625E+00_JPRB, 0.20517E+00_JPRB, 0.20452E+00_JPRB, 0.20416E+00_JPRB, 0.20367E+00_JPRB, 0.20351E+00_JPRB, 0.20340E+00_JPRB ,&
&  0.20305E+00_JPRB, 0.20241E+00_JPRB, 0.20205E+00_JPRB, 0.20180E+00_JPRB, 0.20221E+00_JPRB, 0.20254E+00_JPRB, 0.20307E+00_JPRB ,&
&  0.20391E+00_JPRB, 0.20568E+00_JPRB, 0.20681E+00_JPRB, 0.20787E+00_JPRB, 0.20842E+00_JPRB, 0.20890E+00_JPRB, 0.20954E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.21187E+00_JPRB, 0.21302E+00_JPRB, 0.21456E+00_JPRB, 0.21613E+00_JPRB, 0.21829E+00_JPRB, 0.22061E+00_JPRB ,&
&  0.22328E+00_JPRB, 0.22659E+00_JPRB, 0.23069E+00_JPRB, 0.23443E+00_JPRB, 0.24016E+00_JPRB, 0.24593E+00_JPRB, 0.25237E+00_JPRB ,&
&  0.25898E+00_JPRB, 0.26613E+00_JPRB, 0.27549E+00_JPRB, 0.28658E+00_JPRB, 0.30046E+00_JPRB, 0.30889E+00_JPRB, 0.32024E+00_JPRB ,&
&  0.33761E+00_JPRB, 0.37406E+00_JPRB, 0.50949E+00_JPRB, 0.52296E+00_JPRB, 0.52758E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.54199E+00_JPRB, 0.55127E+00_JPRB, 0.56563E+00_JPRB, 0.57731E+00_JPRB, 0.60159E+00_JPRB, 0.61529E+00_JPRB, 0.66122E+00_JPRB ,&
&  0.61549E+00_JPRB, 0.53661E+00_JPRB, 0.50463E+00_JPRB, 0.46985E+00_JPRB, 0.43020E+00_JPRB, 0.40161E+00_JPRB, 0.38147E+00_JPRB ,&
&  0.36938E+00_JPRB, 0.35905E+00_JPRB, 0.34909E+00_JPRB, 0.33667E+00_JPRB, 0.32391E+00_JPRB, 0.30972E+00_JPRB, 0.29905E+00_JPRB ,&
&  0.29267E+00_JPRB, 0.29093E+00_JPRB, 0.29050E+00_JPRB, 0.29469E+00_JPRB, 0.30045E+00_JPRB, 0.30806E+00_JPRB, 0.31687E+00_JPRB ,&
&  0.33702E+00_JPRB, 0.34342E+00_JPRB, 0.34401E+00_JPRB, 0.34394E+00_JPRB, 0.34498E+00_JPRB, 0.34732E+00_JPRB, 0.33779E+00_JPRB ,&
&  0.34627E+00_JPRB, 0.36745E+00_JPRB, 0.37328E+00_JPRB, 0.35423E+00_JPRB, 0.29968E+00_JPRB, 0.26701E+00_JPRB, 0.24439E+00_JPRB ,&
&  0.23592E+00_JPRB, 0.22615E+00_JPRB, 0.22005E+00_JPRB, 0.21425E+00_JPRB, 0.21045E+00_JPRB, 0.20858E+00_JPRB, 0.20673E+00_JPRB ,&
&  0.20541E+00_JPRB, 0.20446E+00_JPRB, 0.20249E+00_JPRB, 0.20160E+00_JPRB, 0.20050E+00_JPRB, 0.19990E+00_JPRB, 0.19957E+00_JPRB ,&
&  0.19939E+00_JPRB, 0.19906E+00_JPRB, 0.19924E+00_JPRB, 0.19974E+00_JPRB, 0.20065E+00_JPRB, 0.20136E+00_JPRB, 0.20203E+00_JPRB ,&
&  0.20287E+00_JPRB, 0.20365E+00_JPRB, 0.20383E+00_JPRB, 0.20431E+00_JPRB, 0.20473E+00_JPRB, 0.20589E+00_JPRB, 0.20642E+00_JPRB ,&
&  0.20699E+00_JPRB, 0.20746E+00_JPRB, 0.20748E+00_JPRB, 0.20810E+00_JPRB, 0.20903E+00_JPRB, 0.21090E+00_JPRB, 0.21305E+00_JPRB ,&
&  0.21580E+00_JPRB, 0.21900E+00_JPRB, 0.22254E+00_JPRB, 0.22607E+00_JPRB, 0.23066E+00_JPRB, 0.23573E+00_JPRB, 0.24229E+00_JPRB ,&
&  0.24944E+00_JPRB, 0.25844E+00_JPRB, 0.27063E+00_JPRB, 0.28545E+00_JPRB, 0.30358E+00_JPRB, 0.32186E+00_JPRB, 0.34252E+00_JPRB ,&
&  0.37747E+00_JPRB, 0.47693E+00_JPRB, 0.53701E+00_JPRB, 0.52506E+00_JPRB, 0.53577E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.62125E+00_JPRB, 0.63804E+00_JPRB, 0.66325E+00_JPRB, 0.65619E+00_JPRB, 0.67651E+00_JPRB, 0.66314E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.59310E+00_JPRB, 0.53639E+00_JPRB, 0.48985E+00_JPRB, 0.43778E+00_JPRB, 0.39721E+00_JPRB, 0.36763E+00_JPRB, 0.35314E+00_JPRB ,&
&  0.34594E+00_JPRB, 0.34367E+00_JPRB, 0.34221E+00_JPRB, 0.34127E+00_JPRB, 0.33642E+00_JPRB, 0.33298E+00_JPRB, 0.32714E+00_JPRB ,&
&  0.32117E+00_JPRB, 0.31682E+00_JPRB, 0.31352E+00_JPRB, 0.31846E+00_JPRB, 0.32459E+00_JPRB, 0.33100E+00_JPRB, 0.33436E+00_JPRB ,&
&  0.33724E+00_JPRB, 0.33853E+00_JPRB, 0.34119E+00_JPRB, 0.33921E+00_JPRB, 0.35029E+00_JPRB, 0.37145E+00_JPRB, 0.38947E+00_JPRB ,&
&  0.40697E+00_JPRB, 0.42148E+00_JPRB, 0.41734E+00_JPRB, 0.39348E+00_JPRB, 0.32768E+00_JPRB, 0.29350E+00_JPRB, 0.26349E+00_JPRB ,&
&  0.24457E+00_JPRB, 0.23056E+00_JPRB, 0.22117E+00_JPRB, 0.21580E+00_JPRB, 0.21154E+00_JPRB, 0.20917E+00_JPRB, 0.20665E+00_JPRB ,&
&  0.20424E+00_JPRB, 0.20277E+00_JPRB, 0.20110E+00_JPRB, 0.20026E+00_JPRB, 0.19929E+00_JPRB, 0.19800E+00_JPRB, 0.19706E+00_JPRB ,&
&  0.19723E+00_JPRB, 0.19704E+00_JPRB, 0.19817E+00_JPRB, 0.19915E+00_JPRB, 0.20107E+00_JPRB, 0.20268E+00_JPRB, 0.20492E+00_JPRB ,&
&  0.20677E+00_JPRB, 0.20831E+00_JPRB, 0.20817E+00_JPRB, 0.20772E+00_JPRB, 0.20685E+00_JPRB, 0.20637E+00_JPRB, 0.20552E+00_JPRB ,&
&  0.20442E+00_JPRB, 0.20462E+00_JPRB, 0.20492E+00_JPRB, 0.20531E+00_JPRB, 0.20647E+00_JPRB, 0.20810E+00_JPRB, 0.21058E+00_JPRB ,&
&  0.21382E+00_JPRB, 0.21722E+00_JPRB, 0.22009E+00_JPRB, 0.22314E+00_JPRB, 0.22705E+00_JPRB, 0.23149E+00_JPRB, 0.23700E+00_JPRB ,&
&  0.24188E+00_JPRB, 0.25020E+00_JPRB, 0.26168E+00_JPRB, 0.27360E+00_JPRB, 0.29169E+00_JPRB, 0.30927E+00_JPRB, 0.33866E+00_JPRB ,&
&  0.38750E+00_JPRB, 0.55907E+00_JPRB, 0.84261E+00_JPRB, 0.61762E+00_JPRB, 0.60660E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.65864E+00_JPRB, 0.67032E+00_JPRB, 0.68473E+00_JPRB, 0.68802E+00_JPRB, 0.69830E+00_JPRB, 0.69725E+00_JPRB, 0.68011E+00_JPRB ,&
&  0.65198E+00_JPRB, 0.62550E+00_JPRB, 0.61103E+00_JPRB, 0.59322E+00_JPRB, 0.56543E+00_JPRB, 0.50515E+00_JPRB, 0.48211E+00_JPRB ,&
&  0.46356E+00_JPRB, 0.46317E+00_JPRB, 0.45391E+00_JPRB, 0.45402E+00_JPRB, 0.44287E+00_JPRB, 0.43138E+00_JPRB, 0.41259E+00_JPRB ,&
&  0.39525E+00_JPRB, 0.37807E+00_JPRB, 0.36617E+00_JPRB, 0.36044E+00_JPRB, 0.35822E+00_JPRB, 0.35357E+00_JPRB, 0.34131E+00_JPRB ,&
&  0.33253E+00_JPRB, 0.33067E+00_JPRB, 0.33308E+00_JPRB, 0.34232E+00_JPRB, 0.38546E+00_JPRB, 0.44172E+00_JPRB, 0.44226E+00_JPRB ,&
&  0.46475E+00_JPRB, 0.47806E+00_JPRB, 0.49105E+00_JPRB, 0.48005E+00_JPRB, 0.44735E+00_JPRB, 0.40839E+00_JPRB, 0.35454E+00_JPRB ,&
&  0.30332E+00_JPRB, 0.27444E+00_JPRB, 0.25103E+00_JPRB, 0.23466E+00_JPRB, 0.22351E+00_JPRB, 0.21898E+00_JPRB, 0.21317E+00_JPRB ,&
&  0.20938E+00_JPRB, 0.20648E+00_JPRB, 0.20530E+00_JPRB, 0.20468E+00_JPRB, 0.20393E+00_JPRB, 0.20184E+00_JPRB, 0.20124E+00_JPRB ,&
&  0.20176E+00_JPRB, 0.20192E+00_JPRB, 0.20170E+00_JPRB, 0.20251E+00_JPRB, 0.20349E+00_JPRB, 0.20491E+00_JPRB, 0.20621E+00_JPRB ,&
&  0.20817E+00_JPRB, 0.20926E+00_JPRB, 0.21102E+00_JPRB, 0.21295E+00_JPRB, 0.21202E+00_JPRB, 0.21089E+00_JPRB, 0.20814E+00_JPRB ,&
&  0.20506E+00_JPRB, 0.20478E+00_JPRB, 0.20430E+00_JPRB, 0.20494E+00_JPRB, 0.20660E+00_JPRB, 0.20883E+00_JPRB, 0.21175E+00_JPRB ,&
&  0.21456E+00_JPRB, 0.21746E+00_JPRB, 0.21973E+00_JPRB, 0.22199E+00_JPRB, 0.22572E+00_JPRB, 0.22968E+00_JPRB, 0.23470E+00_JPRB ,&
&  0.23990E+00_JPRB, 0.24578E+00_JPRB, 0.25387E+00_JPRB, 0.26430E+00_JPRB, 0.27921E+00_JPRB, 0.29606E+00_JPRB, 0.32215E+00_JPRB ,&
&  0.34634E+00_JPRB, 0.44618E+00_JPRB, 0.68237E+00_JPRB, 0.61102E+00_JPRB, 0.63046E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.73616E+00_JPRB, 0.74843E+00_JPRB, 0.75874E+00_JPRB, 0.76095E+00_JPRB, 0.76039E+00_JPRB, 0.75552E+00_JPRB, 0.74356E+00_JPRB ,&
&  0.72396E+00_JPRB, 0.69773E+00_JPRB, 0.67220E+00_JPRB, 0.65187E+00_JPRB, 0.63911E+00_JPRB, 0.64211E+00_JPRB, 0.65070E+00_JPRB ,&
&  0.66810E+00_JPRB, 0.69361E+00_JPRB, 0.69885E+00_JPRB, 0.69222E+00_JPRB, 0.65641E+00_JPRB, 0.61899E+00_JPRB, 0.58418E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.52416E+00_JPRB, 0.51309E+00_JPRB, 0.49409E+00_JPRB, 0.48075E+00_JPRB, 0.46404E+00_JPRB, 0.44035E+00_JPRB ,&
&  0.40155E+00_JPRB, 0.37102E+00_JPRB, 0.35403E+00_JPRB, 0.36655E+00_JPRB, 0.45451E+00_JPRB, 0.51525E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.55943E+00_JPRB, 0.58385E+00_JPRB, 0.57887E+00_JPRB, 0.55889E+00_JPRB, 0.53732E+00_JPRB, 0.51391E+00_JPRB, 0.49454E+00_JPRB ,&
&  0.47526E+00_JPRB, 0.44890E+00_JPRB, 0.39936E+00_JPRB, 0.32914E+00_JPRB, 0.29370E+00_JPRB, 0.28031E+00_JPRB, 0.26588E+00_JPRB ,&
&  0.25502E+00_JPRB, 0.24636E+00_JPRB, 0.24294E+00_JPRB, 0.23947E+00_JPRB, 0.23876E+00_JPRB, 0.23480E+00_JPRB, 0.23133E+00_JPRB ,&
&  0.22682E+00_JPRB, 0.22352E+00_JPRB, 0.21876E+00_JPRB, 0.21407E+00_JPRB, 0.21053E+00_JPRB, 0.20993E+00_JPRB, 0.20970E+00_JPRB ,&
&  0.21074E+00_JPRB, 0.21482E+00_JPRB, 0.22648E+00_JPRB, 0.23293E+00_JPRB, 0.23340E+00_JPRB, 0.22896E+00_JPRB, 0.22181E+00_JPRB ,&
&  0.21516E+00_JPRB, 0.21333E+00_JPRB, 0.21196E+00_JPRB, 0.21257E+00_JPRB, 0.21541E+00_JPRB, 0.21913E+00_JPRB, 0.22217E+00_JPRB ,&
&  0.22460E+00_JPRB, 0.22711E+00_JPRB, 0.22948E+00_JPRB, 0.23204E+00_JPRB, 0.23516E+00_JPRB, 0.23881E+00_JPRB, 0.24375E+00_JPRB ,&
&  0.24896E+00_JPRB, 0.25626E+00_JPRB, 0.26659E+00_JPRB, 0.28147E+00_JPRB, 0.29472E+00_JPRB, 0.30325E+00_JPRB, 0.31486E+00_JPRB ,&
&  0.33105E+00_JPRB, 0.37845E+00_JPRB, 0.51308E+00_JPRB, 0.73865E+00_JPRB, 0.77089E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.79965E+00_JPRB, 0.81825E+00_JPRB, 0.83978E+00_JPRB, 0.85125E+00_JPRB, 0.85249E+00_JPRB, 0.84971E+00_JPRB, 0.84117E+00_JPRB ,&
&  0.82635E+00_JPRB, 0.80843E+00_JPRB, 0.79521E+00_JPRB, 0.78623E+00_JPRB, 0.78219E+00_JPRB, 0.78500E+00_JPRB, 0.79024E+00_JPRB ,&
&  0.78934E+00_JPRB, 0.78517E+00_JPRB, 0.77304E+00_JPRB, 0.75730E+00_JPRB, 0.74062E+00_JPRB, 0.72755E+00_JPRB, 0.71732E+00_JPRB ,&
&  0.70722E+00_JPRB, 0.69223E+00_JPRB, 0.66908E+00_JPRB, 0.63357E+00_JPRB, 0.59167E+00_JPRB, 0.55344E+00_JPRB, 0.52391E+00_JPRB ,&
&  0.49820E+00_JPRB, 0.43128E+00_JPRB, 0.39423E+00_JPRB, 0.40564E+00_JPRB, 0.49709E+00_JPRB, 0.52825E+00_JPRB, 0.55241E+00_JPRB ,&
&  0.58913E+00_JPRB, 0.62823E+00_JPRB, 0.64693E+00_JPRB, 0.64730E+00_JPRB, 0.64170E+00_JPRB, 0.62903E+00_JPRB, 0.60933E+00_JPRB ,&
&  0.58434E+00_JPRB, 0.55384E+00_JPRB, 0.52786E+00_JPRB, 0.49879E+00_JPRB, 0.46893E+00_JPRB, 0.44694E+00_JPRB, 0.43164E+00_JPRB ,&
&  0.42108E+00_JPRB, 0.41146E+00_JPRB, 0.40379E+00_JPRB, 0.39402E+00_JPRB, 0.38483E+00_JPRB, 0.36372E+00_JPRB, 0.34153E+00_JPRB ,&
&  0.31842E+00_JPRB, 0.30076E+00_JPRB, 0.28213E+00_JPRB, 0.26389E+00_JPRB, 0.24894E+00_JPRB, 0.23919E+00_JPRB, 0.23199E+00_JPRB ,&
&  0.22981E+00_JPRB, 0.23595E+00_JPRB, 0.25496E+00_JPRB, 0.28792E+00_JPRB, 0.32528E+00_JPRB, 0.29697E+00_JPRB, 0.27670E+00_JPRB ,&
&  0.25583E+00_JPRB, 0.24360E+00_JPRB, 0.23907E+00_JPRB, 0.23856E+00_JPRB, 0.23986E+00_JPRB, 0.24218E+00_JPRB, 0.24460E+00_JPRB ,&
&  0.24796E+00_JPRB, 0.25112E+00_JPRB, 0.25445E+00_JPRB, 0.25695E+00_JPRB, 0.25863E+00_JPRB, 0.26149E+00_JPRB, 0.26630E+00_JPRB ,&
&  0.27377E+00_JPRB, 0.28601E+00_JPRB, 0.30599E+00_JPRB, 0.33188E+00_JPRB, 0.35881E+00_JPRB, 0.38332E+00_JPRB, 0.43650E+00_JPRB ,&
&  0.52405E+00_JPRB, 0.63608E+00_JPRB, 0.69349E+00_JPRB, 0.74985E+00_JPRB, 0.77785E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.86183E+00_JPRB, 0.88872E+00_JPRB, 0.91825E+00_JPRB, 0.93525E+00_JPRB, 0.93620E+00_JPRB, 0.93305E+00_JPRB, 0.91804E+00_JPRB ,&
&  0.88711E+00_JPRB, 0.86163E+00_JPRB, 0.85732E+00_JPRB, 0.86112E+00_JPRB, 0.87282E+00_JPRB, 0.89142E+00_JPRB, 0.89287E+00_JPRB ,&
&  0.86618E+00_JPRB, 0.84670E+00_JPRB, 0.83704E+00_JPRB, 0.83009E+00_JPRB, 0.81323E+00_JPRB, 0.78859E+00_JPRB, 0.77237E+00_JPRB ,&
&  0.76757E+00_JPRB, 0.76717E+00_JPRB, 0.75792E+00_JPRB, 0.74376E+00_JPRB, 0.71207E+00_JPRB, 0.67308E+00_JPRB, 0.64249E+00_JPRB ,&
&  0.63185E+00_JPRB, 0.62259E+00_JPRB, 0.58181E+00_JPRB, 0.59790E+00_JPRB, 0.60156E+00_JPRB, 0.60165E+00_JPRB, 0.59893E+00_JPRB ,&
&  0.61099E+00_JPRB, 0.64425E+00_JPRB, 0.67722E+00_JPRB, 0.68694E+00_JPRB, 0.69627E+00_JPRB, 0.69794E+00_JPRB, 0.69936E+00_JPRB ,&
&  0.70389E+00_JPRB, 0.69590E+00_JPRB, 0.66631E+00_JPRB, 0.62242E+00_JPRB, 0.57914E+00_JPRB, 0.54548E+00_JPRB, 0.51683E+00_JPRB ,&
&  0.49718E+00_JPRB, 0.48493E+00_JPRB, 0.47533E+00_JPRB, 0.47203E+00_JPRB, 0.47969E+00_JPRB, 0.49471E+00_JPRB, 0.48958E+00_JPRB ,&
&  0.47127E+00_JPRB, 0.45443E+00_JPRB, 0.43223E+00_JPRB, 0.40215E+00_JPRB, 0.37133E+00_JPRB, 0.33287E+00_JPRB, 0.31312E+00_JPRB ,&
&  0.31771E+00_JPRB, 0.35690E+00_JPRB, 0.40842E+00_JPRB, 0.42797E+00_JPRB, 0.45535E+00_JPRB, 0.48198E+00_JPRB, 0.47147E+00_JPRB ,&
&  0.42400E+00_JPRB, 0.35552E+00_JPRB, 0.33278E+00_JPRB, 0.32026E+00_JPRB, 0.31347E+00_JPRB, 0.30875E+00_JPRB, 0.30792E+00_JPRB ,&
&  0.31230E+00_JPRB, 0.32366E+00_JPRB, 0.33327E+00_JPRB, 0.33034E+00_JPRB, 0.32022E+00_JPRB, 0.31423E+00_JPRB, 0.31513E+00_JPRB ,&
&  0.32480E+00_JPRB, 0.35326E+00_JPRB, 0.41681E+00_JPRB, 0.53221E+00_JPRB, 0.56861E+00_JPRB, 0.60194E+00_JPRB, 0.64037E+00_JPRB ,&
&  0.67726E+00_JPRB, 0.71092E+00_JPRB, 0.73967E+00_JPRB, 0.77500E+00_JPRB, 0.82226E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.85915E+00_JPRB, 0.88111E+00_JPRB, 0.90008E+00_JPRB, 0.91304E+00_JPRB, 0.91505E+00_JPRB, 0.90842E+00_JPRB, 0.89247E+00_JPRB ,&
&  0.86242E+00_JPRB, 0.84532E+00_JPRB, 0.85425E+00_JPRB, 0.86886E+00_JPRB, 0.87778E+00_JPRB, 0.87775E+00_JPRB, 0.84613E+00_JPRB ,&
&  0.79158E+00_JPRB, 0.77073E+00_JPRB, 0.76748E+00_JPRB, 0.77694E+00_JPRB, 0.79506E+00_JPRB, 0.80193E+00_JPRB, 0.78973E+00_JPRB ,&
&  0.77724E+00_JPRB, 0.77865E+00_JPRB, 0.78119E+00_JPRB, 0.78778E+00_JPRB, 0.77685E+00_JPRB, 0.75566E+00_JPRB, 0.74650E+00_JPRB ,&
&  0.74369E+00_JPRB, 0.73595E+00_JPRB, 0.72788E+00_JPRB, 0.67888E+00_JPRB, 0.70256E+00_JPRB, 0.69069E+00_JPRB, 0.67214E+00_JPRB ,&
&  0.65513E+00_JPRB, 0.65756E+00_JPRB, 0.69917E+00_JPRB, 0.74562E+00_JPRB, 0.75709E+00_JPRB, 0.75972E+00_JPRB, 0.75463E+00_JPRB ,&
&  0.75104E+00_JPRB, 0.75142E+00_JPRB, 0.75282E+00_JPRB, 0.74471E+00_JPRB, 0.72437E+00_JPRB, 0.69814E+00_JPRB, 0.66941E+00_JPRB ,&
&  0.65208E+00_JPRB, 0.62728E+00_JPRB, 0.58391E+00_JPRB, 0.56374E+00_JPRB, 0.56187E+00_JPRB, 0.56984E+00_JPRB, 0.57365E+00_JPRB ,&
&  0.57055E+00_JPRB, 0.56307E+00_JPRB, 0.56687E+00_JPRB, 0.56386E+00_JPRB, 0.54207E+00_JPRB, 0.52883E+00_JPRB, 0.52276E+00_JPRB ,&
&  0.51126E+00_JPRB, 0.51015E+00_JPRB, 0.51497E+00_JPRB, 0.52665E+00_JPRB, 0.53691E+00_JPRB, 0.55075E+00_JPRB, 0.55653E+00_JPRB ,&
&  0.55880E+00_JPRB, 0.55532E+00_JPRB, 0.54951E+00_JPRB, 0.53767E+00_JPRB, 0.51938E+00_JPRB, 0.51245E+00_JPRB, 0.51405E+00_JPRB ,&
&  0.52305E+00_JPRB, 0.52912E+00_JPRB, 0.52867E+00_JPRB, 0.52766E+00_JPRB, 0.51624E+00_JPRB, 0.50370E+00_JPRB, 0.49412E+00_JPRB ,&
&  0.48962E+00_JPRB, 0.49246E+00_JPRB, 0.50779E+00_JPRB, 0.53283E+00_JPRB, 0.56494E+00_JPRB, 0.60370E+00_JPRB, 0.64859E+00_JPRB ,&
&  0.68049E+00_JPRB, 0.69939E+00_JPRB, 0.72404E+00_JPRB, 0.77193E+00_JPRB, 0.81818E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.82355E+00_JPRB, 0.83210E+00_JPRB, 0.84223E+00_JPRB, 0.85578E+00_JPRB, 0.86814E+00_JPRB, 0.86543E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.82940E+00_JPRB, 0.82044E+00_JPRB, 0.82499E+00_JPRB, 0.82730E+00_JPRB, 0.82897E+00_JPRB, 0.82666E+00_JPRB, 0.81050E+00_JPRB ,&
&  0.77311E+00_JPRB, 0.74301E+00_JPRB, 0.72844E+00_JPRB, 0.72077E+00_JPRB, 0.71814E+00_JPRB, 0.72491E+00_JPRB, 0.73315E+00_JPRB ,&
&  0.74992E+00_JPRB, 0.75995E+00_JPRB, 0.77197E+00_JPRB, 0.78126E+00_JPRB, 0.78374E+00_JPRB, 0.78740E+00_JPRB, 0.78827E+00_JPRB ,&
&  0.77402E+00_JPRB, 0.77408E+00_JPRB, 0.77547E+00_JPRB, 0.76573E+00_JPRB, 0.74426E+00_JPRB, 0.72267E+00_JPRB, 0.69613E+00_JPRB ,&
&  0.67853E+00_JPRB, 0.66973E+00_JPRB, 0.69523E+00_JPRB, 0.75697E+00_JPRB, 0.80041E+00_JPRB, 0.80845E+00_JPRB, 0.80041E+00_JPRB ,&
&  0.80222E+00_JPRB, 0.80957E+00_JPRB, 0.81309E+00_JPRB, 0.80165E+00_JPRB, 0.78200E+00_JPRB, 0.76479E+00_JPRB, 0.74837E+00_JPRB ,&
&  0.74290E+00_JPRB, 0.74219E+00_JPRB, 0.72485E+00_JPRB, 0.69265E+00_JPRB, 0.66494E+00_JPRB, 0.64531E+00_JPRB, 0.63497E+00_JPRB ,&
&  0.62872E+00_JPRB, 0.61915E+00_JPRB, 0.61323E+00_JPRB, 0.61809E+00_JPRB, 0.59820E+00_JPRB, 0.58026E+00_JPRB, 0.58401E+00_JPRB ,&
&  0.60034E+00_JPRB, 0.60573E+00_JPRB, 0.61486E+00_JPRB, 0.61785E+00_JPRB, 0.61464E+00_JPRB, 0.61076E+00_JPRB, 0.61190E+00_JPRB ,&
&  0.62054E+00_JPRB, 0.63042E+00_JPRB, 0.63771E+00_JPRB, 0.64274E+00_JPRB, 0.64789E+00_JPRB, 0.63631E+00_JPRB, 0.61532E+00_JPRB ,&
&  0.59780E+00_JPRB, 0.59454E+00_JPRB, 0.59909E+00_JPRB, 0.59845E+00_JPRB, 0.59692E+00_JPRB, 0.59708E+00_JPRB, 0.59575E+00_JPRB ,&
&  0.59295E+00_JPRB, 0.58553E+00_JPRB, 0.57666E+00_JPRB, 0.57684E+00_JPRB, 0.58831E+00_JPRB, 0.61105E+00_JPRB, 0.64614E+00_JPRB ,&
&  0.69848E+00_JPRB, 0.73814E+00_JPRB, 0.77632E+00_JPRB, 0.79200E+00_JPRB, 0.79574E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.79553E+00_JPRB, 0.80409E+00_JPRB, 0.81051E+00_JPRB, 0.81536E+00_JPRB, 0.82463E+00_JPRB, 0.83603E+00_JPRB, 0.83613E+00_JPRB ,&
&  0.82147E+00_JPRB, 0.80764E+00_JPRB, 0.79787E+00_JPRB, 0.79493E+00_JPRB, 0.79644E+00_JPRB, 0.79854E+00_JPRB, 0.79260E+00_JPRB ,&
&  0.77155E+00_JPRB, 0.74896E+00_JPRB, 0.73497E+00_JPRB, 0.72107E+00_JPRB, 0.71237E+00_JPRB, 0.70676E+00_JPRB, 0.71346E+00_JPRB ,&
&  0.72348E+00_JPRB, 0.73942E+00_JPRB, 0.75242E+00_JPRB, 0.76177E+00_JPRB, 0.76548E+00_JPRB, 0.75392E+00_JPRB, 0.75507E+00_JPRB ,&
&  0.76203E+00_JPRB, 0.76405E+00_JPRB, 0.77897E+00_JPRB, 0.77834E+00_JPRB, 0.76508E+00_JPRB, 0.75446E+00_JPRB, 0.74393E+00_JPRB ,&
&  0.74119E+00_JPRB, 0.75252E+00_JPRB, 0.77794E+00_JPRB, 0.81460E+00_JPRB, 0.83585E+00_JPRB, 0.84393E+00_JPRB, 0.84074E+00_JPRB ,&
&  0.84336E+00_JPRB, 0.85526E+00_JPRB, 0.86198E+00_JPRB, 0.85374E+00_JPRB, 0.83554E+00_JPRB, 0.82074E+00_JPRB, 0.80770E+00_JPRB ,&
&  0.79811E+00_JPRB, 0.78646E+00_JPRB, 0.77847E+00_JPRB, 0.76844E+00_JPRB, 0.74645E+00_JPRB, 0.72037E+00_JPRB, 0.69766E+00_JPRB ,&
&  0.68080E+00_JPRB, 0.66432E+00_JPRB, 0.64419E+00_JPRB, 0.63788E+00_JPRB, 0.64190E+00_JPRB, 0.61890E+00_JPRB, 0.59396E+00_JPRB ,&
&  0.59734E+00_JPRB, 0.62844E+00_JPRB, 0.65277E+00_JPRB, 0.66253E+00_JPRB, 0.66322E+00_JPRB, 0.66743E+00_JPRB, 0.67079E+00_JPRB ,&
&  0.67728E+00_JPRB, 0.68595E+00_JPRB, 0.69265E+00_JPRB, 0.70385E+00_JPRB, 0.71548E+00_JPRB, 0.71405E+00_JPRB, 0.70006E+00_JPRB ,&
&  0.68469E+00_JPRB, 0.67588E+00_JPRB, 0.67209E+00_JPRB, 0.67421E+00_JPRB, 0.67750E+00_JPRB, 0.67537E+00_JPRB, 0.66956E+00_JPRB ,&
&  0.65794E+00_JPRB, 0.63949E+00_JPRB, 0.62371E+00_JPRB, 0.61504E+00_JPRB, 0.61475E+00_JPRB, 0.62147E+00_JPRB, 0.63778E+00_JPRB ,&
&  0.66938E+00_JPRB, 0.71734E+00_JPRB, 0.75278E+00_JPRB, 0.77266E+00_JPRB, 0.77799E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.77077E+00_JPRB, 0.77771E+00_JPRB, 0.78795E+00_JPRB, 0.80258E+00_JPRB, 0.81747E+00_JPRB, 0.82703E+00_JPRB, 0.82665E+00_JPRB ,&
&  0.81540E+00_JPRB, 0.80174E+00_JPRB, 0.79699E+00_JPRB, 0.79598E+00_JPRB, 0.79269E+00_JPRB, 0.79111E+00_JPRB, 0.78638E+00_JPRB ,&
&  0.77515E+00_JPRB, 0.75818E+00_JPRB, 0.75008E+00_JPRB, 0.73614E+00_JPRB, 0.72283E+00_JPRB, 0.71195E+00_JPRB, 0.71186E+00_JPRB ,&
&  0.72048E+00_JPRB, 0.73050E+00_JPRB, 0.74288E+00_JPRB, 0.75696E+00_JPRB, 0.76452E+00_JPRB, 0.76302E+00_JPRB, 0.75969E+00_JPRB ,&
&  0.75699E+00_JPRB, 0.75407E+00_JPRB, 0.76483E+00_JPRB, 0.77369E+00_JPRB, 0.77963E+00_JPRB, 0.79078E+00_JPRB, 0.80010E+00_JPRB ,&
&  0.80794E+00_JPRB, 0.81937E+00_JPRB, 0.83823E+00_JPRB, 0.85226E+00_JPRB, 0.86253E+00_JPRB, 0.87332E+00_JPRB, 0.87679E+00_JPRB ,&
&  0.88280E+00_JPRB, 0.89399E+00_JPRB, 0.89629E+00_JPRB, 0.88739E+00_JPRB, 0.87317E+00_JPRB, 0.86416E+00_JPRB, 0.85919E+00_JPRB ,&
&  0.85007E+00_JPRB, 0.83074E+00_JPRB, 0.81606E+00_JPRB, 0.80426E+00_JPRB, 0.78508E+00_JPRB, 0.76457E+00_JPRB, 0.74069E+00_JPRB ,&
&  0.71351E+00_JPRB, 0.69325E+00_JPRB, 0.67743E+00_JPRB, 0.66318E+00_JPRB, 0.65964E+00_JPRB, 0.65092E+00_JPRB, 0.64422E+00_JPRB ,&
&  0.64865E+00_JPRB, 0.66682E+00_JPRB, 0.68041E+00_JPRB, 0.68501E+00_JPRB, 0.68740E+00_JPRB, 0.70310E+00_JPRB, 0.72222E+00_JPRB ,&
&  0.73721E+00_JPRB, 0.74354E+00_JPRB, 0.73967E+00_JPRB, 0.74208E+00_JPRB, 0.74728E+00_JPRB, 0.75171E+00_JPRB, 0.74590E+00_JPRB ,&
&  0.73749E+00_JPRB, 0.73201E+00_JPRB, 0.73084E+00_JPRB, 0.72722E+00_JPRB, 0.72866E+00_JPRB, 0.72848E+00_JPRB, 0.72567E+00_JPRB ,&
&  0.71984E+00_JPRB, 0.71031E+00_JPRB, 0.69811E+00_JPRB, 0.69112E+00_JPRB, 0.68083E+00_JPRB, 0.66511E+00_JPRB, 0.66637E+00_JPRB ,&
&  0.67832E+00_JPRB, 0.69711E+00_JPRB, 0.71458E+00_JPRB, 0.73513E+00_JPRB, 0.75921E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.75507E+00_JPRB, 0.76885E+00_JPRB, 0.78603E+00_JPRB, 0.80324E+00_JPRB, 0.81410E+00_JPRB, 0.82065E+00_JPRB, 0.81829E+00_JPRB ,&
&  0.80691E+00_JPRB, 0.79566E+00_JPRB, 0.79676E+00_JPRB, 0.79617E+00_JPRB, 0.79459E+00_JPRB, 0.79227E+00_JPRB, 0.78517E+00_JPRB ,&
&  0.77526E+00_JPRB, 0.76689E+00_JPRB, 0.76335E+00_JPRB, 0.76023E+00_JPRB, 0.75011E+00_JPRB, 0.74231E+00_JPRB, 0.73394E+00_JPRB ,&
&  0.73366E+00_JPRB, 0.73620E+00_JPRB, 0.73913E+00_JPRB, 0.73945E+00_JPRB, 0.74750E+00_JPRB, 0.75777E+00_JPRB, 0.76216E+00_JPRB ,&
&  0.75664E+00_JPRB, 0.76348E+00_JPRB, 0.68076E+00_JPRB, 0.75035E+00_JPRB, 0.77534E+00_JPRB, 0.79027E+00_JPRB, 0.80590E+00_JPRB ,&
&  0.81653E+00_JPRB, 0.82851E+00_JPRB, 0.83744E+00_JPRB, 0.84277E+00_JPRB, 0.86051E+00_JPRB, 0.87470E+00_JPRB, 0.87803E+00_JPRB ,&
&  0.87936E+00_JPRB, 0.87825E+00_JPRB, 0.87958E+00_JPRB, 0.88002E+00_JPRB, 0.88474E+00_JPRB, 0.88495E+00_JPRB, 0.87911E+00_JPRB ,&
&  0.87110E+00_JPRB, 0.85927E+00_JPRB, 0.84165E+00_JPRB, 0.82114E+00_JPRB, 0.78883E+00_JPRB, 0.76061E+00_JPRB, 0.74262E+00_JPRB ,&
&  0.72785E+00_JPRB, 0.72338E+00_JPRB, 0.71785E+00_JPRB, 0.71222E+00_JPRB, 0.71006E+00_JPRB, 0.71224E+00_JPRB, 0.71107E+00_JPRB ,&
&  0.71744E+00_JPRB, 0.72603E+00_JPRB, 0.73020E+00_JPRB, 0.73753E+00_JPRB, 0.74709E+00_JPRB, 0.76112E+00_JPRB, 0.77442E+00_JPRB ,&
&  0.78696E+00_JPRB, 0.79558E+00_JPRB, 0.79701E+00_JPRB, 0.79722E+00_JPRB, 0.79517E+00_JPRB, 0.79435E+00_JPRB, 0.78750E+00_JPRB ,&
&  0.78202E+00_JPRB, 0.77915E+00_JPRB, 0.77291E+00_JPRB, 0.76104E+00_JPRB, 0.75377E+00_JPRB, 0.74761E+00_JPRB, 0.74604E+00_JPRB ,&
&  0.74692E+00_JPRB, 0.74781E+00_JPRB, 0.74502E+00_JPRB, 0.74567E+00_JPRB, 0.73941E+00_JPRB, 0.71954E+00_JPRB, 0.70446E+00_JPRB ,&
&  0.70444E+00_JPRB, 0.71025E+00_JPRB, 0.71560E+00_JPRB, 0.72531E+00_JPRB, 0.74152E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.80361E+00_JPRB, 0.81066E+00_JPRB, 0.82070E+00_JPRB, 0.83169E+00_JPRB, 0.83566E+00_JPRB, 0.83515E+00_JPRB, 0.83220E+00_JPRB ,&
&  0.83227E+00_JPRB, 0.82346E+00_JPRB, 0.81677E+00_JPRB, 0.81251E+00_JPRB, 0.81117E+00_JPRB, 0.80703E+00_JPRB, 0.79699E+00_JPRB ,&
&  0.78861E+00_JPRB, 0.78512E+00_JPRB, 0.78937E+00_JPRB, 0.79070E+00_JPRB, 0.77530E+00_JPRB, 0.77027E+00_JPRB, 0.76225E+00_JPRB ,&
&  0.76097E+00_JPRB, 0.76119E+00_JPRB, 0.76425E+00_JPRB, 0.77054E+00_JPRB, 0.78702E+00_JPRB, 0.79875E+00_JPRB, 0.80505E+00_JPRB ,&
&  0.80340E+00_JPRB, 0.72467E+00_JPRB, 0.75241E+00_JPRB, 0.76468E+00_JPRB, 0.78731E+00_JPRB, 0.81043E+00_JPRB, 0.83160E+00_JPRB ,&
&  0.85944E+00_JPRB, 0.87559E+00_JPRB, 0.88318E+00_JPRB, 0.88519E+00_JPRB, 0.88704E+00_JPRB, 0.88665E+00_JPRB, 0.87901E+00_JPRB ,&
&  0.87394E+00_JPRB, 0.88094E+00_JPRB, 0.89764E+00_JPRB, 0.90620E+00_JPRB, 0.91031E+00_JPRB, 0.91245E+00_JPRB, 0.90901E+00_JPRB ,&
&  0.90170E+00_JPRB, 0.88693E+00_JPRB, 0.86238E+00_JPRB, 0.83289E+00_JPRB, 0.79949E+00_JPRB, 0.77617E+00_JPRB, 0.76304E+00_JPRB ,&
&  0.75448E+00_JPRB, 0.75399E+00_JPRB, 0.75440E+00_JPRB, 0.75565E+00_JPRB, 0.75885E+00_JPRB, 0.76728E+00_JPRB, 0.77067E+00_JPRB ,&
&  0.77817E+00_JPRB, 0.78553E+00_JPRB, 0.79751E+00_JPRB, 0.81705E+00_JPRB, 0.83682E+00_JPRB, 0.84810E+00_JPRB, 0.84901E+00_JPRB ,&
&  0.85490E+00_JPRB, 0.85786E+00_JPRB, 0.85435E+00_JPRB, 0.85438E+00_JPRB, 0.85082E+00_JPRB, 0.85239E+00_JPRB, 0.84995E+00_JPRB ,&
&  0.84150E+00_JPRB, 0.83530E+00_JPRB, 0.82764E+00_JPRB, 0.81262E+00_JPRB, 0.79952E+00_JPRB, 0.79004E+00_JPRB, 0.78204E+00_JPRB ,&
&  0.78086E+00_JPRB, 0.78377E+00_JPRB, 0.78851E+00_JPRB, 0.78312E+00_JPRB, 0.76942E+00_JPRB, 0.76173E+00_JPRB, 0.75231E+00_JPRB ,&
&  0.74632E+00_JPRB, 0.74950E+00_JPRB, 0.75134E+00_JPRB, 0.76169E+00_JPRB, 0.78415E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.84900E+00_JPRB, 0.85340E+00_JPRB, 0.85746E+00_JPRB, 0.85846E+00_JPRB, 0.85498E+00_JPRB, 0.85231E+00_JPRB, 0.85351E+00_JPRB ,&
&  0.85385E+00_JPRB, 0.84527E+00_JPRB, 0.83667E+00_JPRB, 0.83012E+00_JPRB, 0.82715E+00_JPRB, 0.81770E+00_JPRB, 0.80342E+00_JPRB ,&
&  0.79284E+00_JPRB, 0.79084E+00_JPRB, 0.79571E+00_JPRB, 0.79854E+00_JPRB, 0.78999E+00_JPRB, 0.78095E+00_JPRB, 0.77792E+00_JPRB ,&
&  0.77912E+00_JPRB, 0.78550E+00_JPRB, 0.79604E+00_JPRB, 0.80235E+00_JPRB, 0.81134E+00_JPRB, 0.82572E+00_JPRB, 0.82718E+00_JPRB ,&
&  0.82544E+00_JPRB, 0.68918E+00_JPRB, 0.72732E+00_JPRB, 0.74259E+00_JPRB, 0.77355E+00_JPRB, 0.81996E+00_JPRB, 0.85949E+00_JPRB ,&
&  0.89795E+00_JPRB, 0.92080E+00_JPRB, 0.93152E+00_JPRB, 0.92565E+00_JPRB, 0.91014E+00_JPRB, 0.89501E+00_JPRB, 0.89072E+00_JPRB ,&
&  0.88959E+00_JPRB, 0.89584E+00_JPRB, 0.90855E+00_JPRB, 0.91725E+00_JPRB, 0.91998E+00_JPRB, 0.92035E+00_JPRB, 0.92242E+00_JPRB ,&
&  0.91901E+00_JPRB, 0.90261E+00_JPRB, 0.87938E+00_JPRB, 0.85098E+00_JPRB, 0.82851E+00_JPRB, 0.81779E+00_JPRB, 0.80468E+00_JPRB ,&
&  0.78663E+00_JPRB, 0.77573E+00_JPRB, 0.77408E+00_JPRB, 0.77707E+00_JPRB, 0.78624E+00_JPRB, 0.79642E+00_JPRB, 0.80523E+00_JPRB ,&
&  0.81529E+00_JPRB, 0.83124E+00_JPRB, 0.84861E+00_JPRB, 0.86535E+00_JPRB, 0.88233E+00_JPRB, 0.89814E+00_JPRB, 0.91028E+00_JPRB ,&
&  0.91830E+00_JPRB, 0.91652E+00_JPRB, 0.90505E+00_JPRB, 0.89618E+00_JPRB, 0.89283E+00_JPRB, 0.89737E+00_JPRB, 0.89840E+00_JPRB ,&
&  0.89058E+00_JPRB, 0.88413E+00_JPRB, 0.88122E+00_JPRB, 0.87092E+00_JPRB, 0.85958E+00_JPRB, 0.84928E+00_JPRB, 0.83827E+00_JPRB ,&
&  0.82937E+00_JPRB, 0.82530E+00_JPRB, 0.82142E+00_JPRB, 0.81495E+00_JPRB, 0.80721E+00_JPRB, 0.80519E+00_JPRB, 0.80352E+00_JPRB ,&
&  0.80219E+00_JPRB, 0.80034E+00_JPRB, 0.80446E+00_JPRB, 0.81799E+00_JPRB, 0.83651E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.88125E+00_JPRB, 0.88808E+00_JPRB, 0.89168E+00_JPRB, 0.89605E+00_JPRB, 0.89594E+00_JPRB, 0.89338E+00_JPRB, 0.89674E+00_JPRB ,&
&  0.90041E+00_JPRB, 0.89455E+00_JPRB, 0.88497E+00_JPRB, 0.88103E+00_JPRB, 0.87060E+00_JPRB, 0.85527E+00_JPRB, 0.84225E+00_JPRB ,&
&  0.82635E+00_JPRB, 0.81914E+00_JPRB, 0.81454E+00_JPRB, 0.81028E+00_JPRB, 0.80348E+00_JPRB, 0.79531E+00_JPRB, 0.79896E+00_JPRB ,&
&  0.80593E+00_JPRB, 0.81346E+00_JPRB, 0.81577E+00_JPRB, 0.82008E+00_JPRB, 0.82636E+00_JPRB, 0.83345E+00_JPRB, 0.84615E+00_JPRB ,&
&  0.73507E+00_JPRB, 0.61079E+00_JPRB, 0.66333E+00_JPRB, 0.71732E+00_JPRB, 0.76698E+00_JPRB, 0.84210E+00_JPRB, 0.89845E+00_JPRB ,&
&  0.94312E+00_JPRB, 0.98003E+00_JPRB, 0.98865E+00_JPRB, 0.96565E+00_JPRB, 0.94109E+00_JPRB, 0.93242E+00_JPRB, 0.93065E+00_JPRB ,&
&  0.92589E+00_JPRB, 0.92337E+00_JPRB, 0.92603E+00_JPRB, 0.92906E+00_JPRB, 0.93236E+00_JPRB, 0.93757E+00_JPRB, 0.93921E+00_JPRB ,&
&  0.93677E+00_JPRB, 0.92396E+00_JPRB, 0.91083E+00_JPRB, 0.89068E+00_JPRB, 0.87313E+00_JPRB, 0.86703E+00_JPRB, 0.85800E+00_JPRB ,&
&  0.84279E+00_JPRB, 0.83406E+00_JPRB, 0.83302E+00_JPRB, 0.83583E+00_JPRB, 0.84465E+00_JPRB, 0.85559E+00_JPRB, 0.87365E+00_JPRB ,&
&  0.89501E+00_JPRB, 0.91129E+00_JPRB, 0.92301E+00_JPRB, 0.93627E+00_JPRB, 0.94355E+00_JPRB, 0.95301E+00_JPRB, 0.97038E+00_JPRB ,&
&  0.98477E+00_JPRB, 0.97565E+00_JPRB, 0.96677E+00_JPRB, 0.95759E+00_JPRB, 0.95449E+00_JPRB, 0.95215E+00_JPRB, 0.94274E+00_JPRB ,&
&  0.93384E+00_JPRB, 0.92834E+00_JPRB, 0.92584E+00_JPRB, 0.93099E+00_JPRB, 0.93064E+00_JPRB, 0.92572E+00_JPRB, 0.92393E+00_JPRB ,&
&  0.92244E+00_JPRB, 0.91389E+00_JPRB, 0.90298E+00_JPRB, 0.89768E+00_JPRB, 0.89205E+00_JPRB, 0.89148E+00_JPRB, 0.88896E+00_JPRB ,&
&  0.88689E+00_JPRB, 0.87411E+00_JPRB, 0.88150E+00_JPRB, 0.87691E+00_JPRB, 0.87470E+00_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.90788E+00_JPRB, 0.92828E+00_JPRB, 0.92984E+00_JPRB, 0.94023E+00_JPRB, 0.94647E+00_JPRB, 0.95108E+00_JPRB, 0.95963E+00_JPRB ,&
&  0.96531E+00_JPRB, 0.96064E+00_JPRB, 0.95169E+00_JPRB, 0.93462E+00_JPRB, 0.90817E+00_JPRB, 0.87995E+00_JPRB, 0.85659E+00_JPRB ,&
&  0.83938E+00_JPRB, 0.83464E+00_JPRB, 0.82473E+00_JPRB, 0.81291E+00_JPRB, 0.80585E+00_JPRB, 0.80227E+00_JPRB, 0.80455E+00_JPRB ,&
&  0.81268E+00_JPRB, 0.81711E+00_JPRB, 0.81239E+00_JPRB, 0.81138E+00_JPRB, 0.82043E+00_JPRB, 0.83677E+00_JPRB, 0.84460E+00_JPRB ,&
&  0.84680E+00_JPRB, 0.65253E+00_JPRB, 0.69630E+00_JPRB, 0.75016E+00_JPRB, 0.79976E+00_JPRB, 0.86936E+00_JPRB, 0.94558E+00_JPRB ,&
&  0.99705E+00_JPRB, 0.10279E+01_JPRB, 0.10285E+01_JPRB, 0.10164E+01_JPRB, 0.99497E+00_JPRB, 0.97502E+00_JPRB, 0.95171E+00_JPRB ,&
&  0.93124E+00_JPRB, 0.92384E+00_JPRB, 0.92662E+00_JPRB, 0.93015E+00_JPRB, 0.93115E+00_JPRB, 0.93424E+00_JPRB, 0.92788E+00_JPRB ,&
&  0.92545E+00_JPRB, 0.92196E+00_JPRB, 0.91987E+00_JPRB, 0.91961E+00_JPRB, 0.91814E+00_JPRB, 0.91707E+00_JPRB, 0.91330E+00_JPRB ,&
&  0.90005E+00_JPRB, 0.89056E+00_JPRB, 0.88282E+00_JPRB, 0.87575E+00_JPRB, 0.87851E+00_JPRB, 0.88981E+00_JPRB, 0.90800E+00_JPRB ,&
&  0.92892E+00_JPRB, 0.93339E+00_JPRB, 0.94013E+00_JPRB, 0.94848E+00_JPRB, 0.96132E+00_JPRB, 0.97485E+00_JPRB, 0.99121E+00_JPRB ,&
&  0.99833E+00_JPRB, 0.99332E+00_JPRB, 0.98874E+00_JPRB, 0.97825E+00_JPRB, 0.97644E+00_JPRB, 0.96073E+00_JPRB, 0.94209E+00_JPRB ,&
&  0.93935E+00_JPRB, 0.93964E+00_JPRB, 0.94200E+00_JPRB, 0.95481E+00_JPRB, 0.96366E+00_JPRB, 0.96524E+00_JPRB, 0.97371E+00_JPRB ,&
&  0.98564E+00_JPRB, 0.99358E+00_JPRB, 0.98878E+00_JPRB, 0.99594E+00_JPRB, 0.99504E+00_JPRB, 0.99744E+00_JPRB, 0.98331E+00_JPRB ,&
&  0.96189E+00_JPRB, 0.94710E+00_JPRB, 0.74850E+00_JPRB, 0.84743E+00_JPRB, 0.87635E+00_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.99023E+00_JPRB, 0.10191E+01_JPRB, 0.10270E+01_JPRB, 0.10211E+01_JPRB, 0.10206E+01_JPRB, 0.10193E+01_JPRB, 0.10147E+01_JPRB ,&
&  0.10121E+01_JPRB, 0.10058E+01_JPRB, 0.99059E+00_JPRB, 0.96669E+00_JPRB, 0.94197E+00_JPRB, 0.91534E+00_JPRB, 0.89387E+00_JPRB ,&
&  0.88523E+00_JPRB, 0.87907E+00_JPRB, 0.86311E+00_JPRB, 0.84840E+00_JPRB, 0.83496E+00_JPRB, 0.82446E+00_JPRB, 0.82034E+00_JPRB ,&
&  0.81555E+00_JPRB, 0.81370E+00_JPRB, 0.81926E+00_JPRB, 0.82662E+00_JPRB, 0.83635E+00_JPRB, 0.84722E+00_JPRB, 0.86189E+00_JPRB ,&
&  0.86593E+00_JPRB, 0.68405E+00_JPRB, 0.71420E+00_JPRB, 0.80817E+00_JPRB, 0.88176E+00_JPRB, 0.97296E+00_JPRB, 0.10555E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.11061E+01_JPRB, 0.10942E+01_JPRB, 0.10743E+01_JPRB, 0.10664E+01_JPRB, 0.10440E+01_JPRB, 0.10012E+01_JPRB ,&
&  0.96827E+00_JPRB, 0.96330E+00_JPRB, 0.97304E+00_JPRB, 0.97877E+00_JPRB, 0.97451E+00_JPRB, 0.96549E+00_JPRB, 0.96049E+00_JPRB ,&
&  0.95524E+00_JPRB, 0.94912E+00_JPRB, 0.94194E+00_JPRB, 0.93191E+00_JPRB, 0.93718E+00_JPRB, 0.95664E+00_JPRB, 0.97503E+00_JPRB ,&
&  0.97686E+00_JPRB, 0.96498E+00_JPRB, 0.95085E+00_JPRB, 0.93911E+00_JPRB, 0.93791E+00_JPRB, 0.95245E+00_JPRB, 0.96859E+00_JPRB ,&
&  0.97628E+00_JPRB, 0.97941E+00_JPRB, 0.98782E+00_JPRB, 0.99063E+00_JPRB, 0.99832E+00_JPRB, 0.10065E+01_JPRB, 0.10203E+01_JPRB ,&
&  0.10276E+01_JPRB, 0.10216E+01_JPRB, 0.10136E+01_JPRB, 0.10003E+01_JPRB, 0.98133E+00_JPRB, 0.96938E+00_JPRB, 0.96566E+00_JPRB ,&
&  0.96169E+00_JPRB, 0.96779E+00_JPRB, 0.98832E+00_JPRB, 0.10134E+01_JPRB, 0.10231E+01_JPRB, 0.10269E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.10391E+01_JPRB, 0.10450E+01_JPRB, 0.10471E+01_JPRB, 0.10561E+01_JPRB, 0.11086E+01_JPRB, 0.11127E+01_JPRB, 0.10967E+01_JPRB ,&
&  0.10725E+01_JPRB, 0.10449E+01_JPRB, 0.10452E+01_JPRB, 0.87982E+00_JPRB, 0.96181E+00_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.10905E+01_JPRB, 0.11151E+01_JPRB, 0.11012E+01_JPRB, 0.10886E+01_JPRB, 0.10777E+01_JPRB, 0.10587E+01_JPRB, 0.10333E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.98990E+00_JPRB, 0.96849E+00_JPRB, 0.94202E+00_JPRB, 0.92394E+00_JPRB, 0.91234E+00_JPRB, 0.90156E+00_JPRB ,&
&  0.89582E+00_JPRB, 0.88936E+00_JPRB, 0.88389E+00_JPRB, 0.87775E+00_JPRB, 0.86612E+00_JPRB, 0.85564E+00_JPRB, 0.84066E+00_JPRB ,&
&  0.83069E+00_JPRB, 0.82894E+00_JPRB, 0.82892E+00_JPRB, 0.83265E+00_JPRB, 0.84012E+00_JPRB, 0.84879E+00_JPRB, 0.85491E+00_JPRB ,&
&  0.85783E+00_JPRB, 0.75676E+00_JPRB, 0.79921E+00_JPRB, 0.90919E+00_JPRB, 0.10399E+01_JPRB, 0.11532E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.11824E+01_JPRB, 0.11686E+01_JPRB, 0.11457E+01_JPRB, 0.11187E+01_JPRB, 0.10927E+01_JPRB, 0.10753E+01_JPRB, 0.10427E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10141E+01_JPRB, 0.10160E+01_JPRB, 0.10135E+01_JPRB, 0.10059E+01_JPRB, 0.99672E+00_JPRB, 0.99324E+00_JPRB ,&
&  0.98498E+00_JPRB, 0.95978E+00_JPRB, 0.93125E+00_JPRB, 0.91027E+00_JPRB, 0.90809E+00_JPRB, 0.94766E+00_JPRB, 0.10028E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.10341E+01_JPRB, 0.10063E+01_JPRB, 0.98670E+00_JPRB, 0.98202E+00_JPRB, 0.99231E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.10159E+01_JPRB, 0.10153E+01_JPRB, 0.10275E+01_JPRB, 0.10301E+01_JPRB, 0.10324E+01_JPRB, 0.10304E+01_JPRB, 0.10276E+01_JPRB ,&
&  0.10187E+01_JPRB, 0.10075E+01_JPRB, 0.10014E+01_JPRB, 0.99659E+00_JPRB, 0.99792E+00_JPRB, 0.99306E+00_JPRB, 0.98202E+00_JPRB ,&
&  0.96917E+00_JPRB, 0.98188E+00_JPRB, 0.10203E+01_JPRB, 0.10552E+01_JPRB, 0.10555E+01_JPRB, 0.10459E+01_JPRB, 0.10398E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10415E+01_JPRB, 0.10359E+01_JPRB, 0.11277E+01_JPRB, 0.11435E+01_JPRB, 0.11540E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.11803E+01_JPRB, 0.11729E+01_JPRB, 0.11583E+01_JPRB, 0.11266E+01_JPRB, 0.10330E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.11870E+01_JPRB, 0.11743E+01_JPRB, 0.11580E+01_JPRB, 0.11477E+01_JPRB, 0.11299E+01_JPRB, 0.10944E+01_JPRB, 0.10540E+01_JPRB ,&
&  0.10227E+01_JPRB, 0.98784E+00_JPRB, 0.94380E+00_JPRB, 0.92120E+00_JPRB, 0.91484E+00_JPRB, 0.91414E+00_JPRB, 0.90879E+00_JPRB ,&
&  0.90253E+00_JPRB, 0.90472E+00_JPRB, 0.89709E+00_JPRB, 0.89547E+00_JPRB, 0.88596E+00_JPRB, 0.87899E+00_JPRB, 0.85692E+00_JPRB ,&
&  0.84180E+00_JPRB, 0.83112E+00_JPRB, 0.82779E+00_JPRB, 0.83408E+00_JPRB, 0.83984E+00_JPRB, 0.84382E+00_JPRB, 0.84959E+00_JPRB ,&
&  0.84849E+00_JPRB, 0.78940E+00_JPRB, 0.91623E+00_JPRB, 0.11116E+01_JPRB, 0.12559E+01_JPRB, 0.13473E+01_JPRB, 0.13159E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12415E+01_JPRB, 0.12344E+01_JPRB, 0.12074E+01_JPRB, 0.11645E+01_JPRB, 0.11246E+01_JPRB, 0.10976E+01_JPRB ,&
&  0.10902E+01_JPRB, 0.10795E+01_JPRB, 0.10713E+01_JPRB, 0.10613E+01_JPRB, 0.10500E+01_JPRB, 0.10387E+01_JPRB, 0.10382E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.98476E+00_JPRB, 0.94631E+00_JPRB, 0.92058E+00_JPRB, 0.94173E+00_JPRB, 0.10059E+01_JPRB, 0.11154E+01_JPRB ,&
&  0.11753E+01_JPRB, 0.11430E+01_JPRB, 0.11133E+01_JPRB, 0.10902E+01_JPRB, 0.10762E+01_JPRB, 0.10781E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.10631E+01_JPRB, 0.10585E+01_JPRB, 0.10493E+01_JPRB, 0.10408E+01_JPRB, 0.10349E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.10138E+01_JPRB, 0.10110E+01_JPRB, 0.10003E+01_JPRB, 0.10004E+01_JPRB, 0.99506E+00_JPRB, 0.98522E+00_JPRB, 0.97583E+00_JPRB ,&
&  0.98669E+00_JPRB, 0.10194E+01_JPRB, 0.10457E+01_JPRB, 0.10787E+01_JPRB, 0.10847E+01_JPRB, 0.10727E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10629E+01_JPRB, 0.10334E+01_JPRB, 0.12057E+01_JPRB, 0.13396E+01_JPRB, 0.13339E+01_JPRB, 0.13180E+01_JPRB ,&
&  0.13113E+01_JPRB, 0.12801E+01_JPRB, 0.12064E+01_JPRB, 0.11398E+01_JPRB, 0.11668E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.11192E+01_JPRB, 0.11003E+01_JPRB, 0.11064E+01_JPRB, 0.11123E+01_JPRB, 0.10966E+01_JPRB, 0.10551E+01_JPRB, 0.10175E+01_JPRB ,&
&  0.99724E+00_JPRB, 0.95800E+00_JPRB, 0.92272E+00_JPRB, 0.90020E+00_JPRB, 0.90531E+00_JPRB, 0.91230E+00_JPRB, 0.91249E+00_JPRB ,&
&  0.90086E+00_JPRB, 0.89609E+00_JPRB, 0.87969E+00_JPRB, 0.86562E+00_JPRB, 0.85760E+00_JPRB, 0.85213E+00_JPRB, 0.83944E+00_JPRB ,&
&  0.84019E+00_JPRB, 0.83900E+00_JPRB, 0.84235E+00_JPRB, 0.84957E+00_JPRB, 0.84630E+00_JPRB, 0.84915E+00_JPRB, 0.84292E+00_JPRB ,&
&  0.83061E+00_JPRB, 0.94774E+00_JPRB, 0.11657E+01_JPRB, 0.14643E+01_JPRB, 0.15920E+01_JPRB, 0.15505E+01_JPRB, 0.14662E+01_JPRB ,&
&  0.13844E+01_JPRB, 0.13438E+01_JPRB, 0.13378E+01_JPRB, 0.13061E+01_JPRB, 0.12573E+01_JPRB, 0.12132E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.11366E+01_JPRB, 0.11110E+01_JPRB, 0.11024E+01_JPRB, 0.10843E+01_JPRB, 0.10618E+01_JPRB, 0.10544E+01_JPRB, 0.10548E+01_JPRB ,&
&  0.10088E+01_JPRB, 0.97297E+00_JPRB, 0.94558E+00_JPRB, 0.94069E+00_JPRB, 0.10811E+01_JPRB, 0.12467E+01_JPRB, 0.14666E+01_JPRB ,&
&  0.14150E+01_JPRB, 0.13331E+01_JPRB, 0.12801E+01_JPRB, 0.12405E+01_JPRB, 0.12249E+01_JPRB, 0.11736E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.10922E+01_JPRB, 0.10829E+01_JPRB, 0.10752E+01_JPRB, 0.10768E+01_JPRB, 0.10691E+01_JPRB, 0.10632E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.10086E+01_JPRB, 0.99205E+00_JPRB, 0.98417E+00_JPRB, 0.98407E+00_JPRB, 0.99012E+00_JPRB, 0.98048E+00_JPRB, 0.97221E+00_JPRB ,&
&  0.98633E+00_JPRB, 0.10060E+01_JPRB, 0.10677E+01_JPRB, 0.11442E+01_JPRB, 0.11462E+01_JPRB, 0.11224E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.11339E+01_JPRB, 0.11001E+01_JPRB, 0.12888E+01_JPRB, 0.14532E+01_JPRB, 0.15341E+01_JPRB, 0.14407E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.12153E+01_JPRB, 0.11468E+01_JPRB, 0.11128E+01_JPRB, 0.11315E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.10505E+01_JPRB, 0.10461E+01_JPRB, 0.10646E+01_JPRB, 0.10787E+01_JPRB, 0.10585E+01_JPRB, 0.10155E+01_JPRB, 0.97412E+00_JPRB ,&
&  0.95449E+00_JPRB, 0.94265E+00_JPRB, 0.92893E+00_JPRB, 0.90370E+00_JPRB, 0.89292E+00_JPRB, 0.89314E+00_JPRB, 0.89403E+00_JPRB ,&
&  0.89455E+00_JPRB, 0.87988E+00_JPRB, 0.85913E+00_JPRB, 0.83816E+00_JPRB, 0.83427E+00_JPRB, 0.83746E+00_JPRB, 0.84923E+00_JPRB ,&
&  0.84402E+00_JPRB, 0.84185E+00_JPRB, 0.84683E+00_JPRB, 0.85256E+00_JPRB, 0.85983E+00_JPRB, 0.85013E+00_JPRB, 0.83936E+00_JPRB ,&
&  0.85968E+00_JPRB, 0.11926E+01_JPRB, 0.16022E+01_JPRB, 0.17879E+01_JPRB, 0.17396E+01_JPRB, 0.16009E+01_JPRB, 0.15332E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.14343E+01_JPRB, 0.14199E+01_JPRB, 0.13630E+01_JPRB, 0.12951E+01_JPRB, 0.12344E+01_JPRB, 0.11854E+01_JPRB ,&
&  0.11385E+01_JPRB, 0.11032E+01_JPRB, 0.10747E+01_JPRB, 0.10688E+01_JPRB, 0.10687E+01_JPRB, 0.10739E+01_JPRB, 0.10498E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.98658E+00_JPRB, 0.96793E+00_JPRB, 0.10182E+01_JPRB, 0.12640E+01_JPRB, 0.13983E+01_JPRB, 0.18115E+01_JPRB ,&
&  0.20515E+01_JPRB, 0.16997E+01_JPRB, 0.15022E+01_JPRB, 0.14224E+01_JPRB, 0.13616E+01_JPRB, 0.12897E+01_JPRB, 0.12338E+01_JPRB ,&
&  0.12122E+01_JPRB, 0.12090E+01_JPRB, 0.12300E+01_JPRB, 0.12225E+01_JPRB, 0.11714E+01_JPRB, 0.11098E+01_JPRB, 0.10799E+01_JPRB ,&
&  0.10637E+01_JPRB, 0.10479E+01_JPRB, 0.10408E+01_JPRB, 0.10336E+01_JPRB, 0.10230E+01_JPRB, 0.99716E+00_JPRB, 0.98878E+00_JPRB ,&
&  0.10035E+01_JPRB, 0.10504E+01_JPRB, 0.11589E+01_JPRB, 0.11844E+01_JPRB, 0.12147E+01_JPRB, 0.12269E+01_JPRB, 0.11803E+01_JPRB ,&
&  0.11198E+01_JPRB, 0.11317E+01_JPRB, 0.11364E+01_JPRB, 0.12740E+01_JPRB, 0.15066E+01_JPRB, 0.15756E+01_JPRB, 0.14290E+01_JPRB ,&
&  0.12706E+01_JPRB, 0.11477E+01_JPRB, 0.10833E+01_JPRB, 0.10519E+01_JPRB, 0.10678E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.97876E+00_JPRB, 0.97065E+00_JPRB, 0.98461E+00_JPRB, 0.10065E+01_JPRB, 0.98738E+00_JPRB, 0.95172E+00_JPRB, 0.91864E+00_JPRB ,&
&  0.93285E+00_JPRB, 0.93700E+00_JPRB, 0.93389E+00_JPRB, 0.90225E+00_JPRB, 0.88598E+00_JPRB, 0.86791E+00_JPRB, 0.85759E+00_JPRB ,&
&  0.85804E+00_JPRB, 0.84646E+00_JPRB, 0.82777E+00_JPRB, 0.81771E+00_JPRB, 0.81811E+00_JPRB, 0.81985E+00_JPRB, 0.83465E+00_JPRB ,&
&  0.82649E+00_JPRB, 0.83208E+00_JPRB, 0.85359E+00_JPRB, 0.86261E+00_JPRB, 0.87042E+00_JPRB, 0.85282E+00_JPRB, 0.84987E+00_JPRB ,&
&  0.90867E+00_JPRB, 0.10447E+01_JPRB, 0.17557E+01_JPRB, 0.21360E+01_JPRB, 0.17858E+01_JPRB, 0.16014E+01_JPRB, 0.14869E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.15565E+01_JPRB, 0.14597E+01_JPRB, 0.13507E+01_JPRB, 0.12907E+01_JPRB, 0.12242E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.10941E+01_JPRB, 0.10889E+01_JPRB, 0.11012E+01_JPRB, 0.11022E+01_JPRB, 0.10899E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10371E+01_JPRB, 0.10191E+01_JPRB, 0.10214E+01_JPRB, 0.11449E+01_JPRB, 0.14572E+01_JPRB, 0.18531E+01_JPRB, 0.21031E+01_JPRB ,&
&  0.21581E+01_JPRB, 0.20106E+01_JPRB, 0.16772E+01_JPRB, 0.15532E+01_JPRB, 0.14528E+01_JPRB, 0.13807E+01_JPRB, 0.13017E+01_JPRB ,&
&  0.12572E+01_JPRB, 0.12648E+01_JPRB, 0.12462E+01_JPRB, 0.12011E+01_JPRB, 0.11541E+01_JPRB, 0.11294E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.10949E+01_JPRB, 0.10981E+01_JPRB, 0.10766E+01_JPRB, 0.10547E+01_JPRB, 0.10307E+01_JPRB, 0.10283E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.10969E+01_JPRB, 0.11822E+01_JPRB, 0.12555E+01_JPRB, 0.12425E+01_JPRB, 0.12078E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.11742E+01_JPRB, 0.11548E+01_JPRB, 0.11525E+01_JPRB, 0.12782E+01_JPRB, 0.13584E+01_JPRB, 0.14372E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.10583E+01_JPRB, 0.10109E+01_JPRB, 0.99146E+00_JPRB, 0.99163E+00_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.93820E+00_JPRB, 0.92228E+00_JPRB, 0.93834E+00_JPRB, 0.94947E+00_JPRB, 0.94151E+00_JPRB, 0.91161E+00_JPRB, 0.89260E+00_JPRB ,&
&  0.90299E+00_JPRB, 0.90055E+00_JPRB, 0.89330E+00_JPRB, 0.86804E+00_JPRB, 0.85363E+00_JPRB, 0.83227E+00_JPRB, 0.81905E+00_JPRB ,&
&  0.82159E+00_JPRB, 0.82366E+00_JPRB, 0.81336E+00_JPRB, 0.81159E+00_JPRB, 0.80978E+00_JPRB, 0.82315E+00_JPRB, 0.83475E+00_JPRB ,&
&  0.84642E+00_JPRB, 0.85524E+00_JPRB, 0.87050E+00_JPRB, 0.87803E+00_JPRB, 0.87206E+00_JPRB, 0.86034E+00_JPRB, 0.86703E+00_JPRB ,&
&  0.98930E+00_JPRB, 0.24510E+01_JPRB, 0.15614E+01_JPRB, 0.23232E+01_JPRB, 0.17648E+01_JPRB, 0.15566E+01_JPRB, 0.13011E+01_JPRB ,&
&  0.13995E+01_JPRB, 0.16741E+01_JPRB, 0.15242E+01_JPRB, 0.13528E+01_JPRB, 0.12951E+01_JPRB, 0.12466E+01_JPRB, 0.11813E+01_JPRB ,&
&  0.11466E+01_JPRB, 0.11475E+01_JPRB, 0.11303E+01_JPRB, 0.11290E+01_JPRB, 0.11219E+01_JPRB, 0.11044E+01_JPRB, 0.11013E+01_JPRB ,&
&  0.10913E+01_JPRB, 0.10903E+01_JPRB, 0.11297E+01_JPRB, 0.13567E+01_JPRB, 0.15357E+01_JPRB, 0.18961E+01_JPRB, 0.21800E+01_JPRB ,&
&  0.20880E+01_JPRB, 0.20552E+01_JPRB, 0.17064E+01_JPRB, 0.15285E+01_JPRB, 0.14271E+01_JPRB, 0.13605E+01_JPRB, 0.13198E+01_JPRB ,&
&  0.12689E+01_JPRB, 0.12481E+01_JPRB, 0.11844E+01_JPRB, 0.11537E+01_JPRB, 0.11483E+01_JPRB, 0.11349E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.10659E+01_JPRB, 0.10782E+01_JPRB, 0.10766E+01_JPRB, 0.10629E+01_JPRB, 0.10342E+01_JPRB, 0.10324E+01_JPRB ,&
&  0.10641E+01_JPRB, 0.10931E+01_JPRB, 0.11438E+01_JPRB, 0.12596E+01_JPRB, 0.12390E+01_JPRB, 0.11833E+01_JPRB, 0.11523E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.11722E+01_JPRB, 0.11778E+01_JPRB, 0.12484E+01_JPRB, 0.13027E+01_JPRB, 0.13476E+01_JPRB, 0.11372E+01_JPRB ,&
&  0.10477E+01_JPRB, 0.98760E+00_JPRB, 0.95781E+00_JPRB, 0.94344E+00_JPRB, 0.94579E+00_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.94018E+00_JPRB, 0.91295E+00_JPRB, 0.89750E+00_JPRB, 0.90755E+00_JPRB, 0.91277E+00_JPRB, 0.89531E+00_JPRB, 0.87009E+00_JPRB ,&
&  0.85125E+00_JPRB, 0.83130E+00_JPRB, 0.83984E+00_JPRB, 0.82850E+00_JPRB, 0.82214E+00_JPRB, 0.81391E+00_JPRB, 0.80946E+00_JPRB ,&
&  0.81306E+00_JPRB, 0.82163E+00_JPRB, 0.80909E+00_JPRB, 0.81200E+00_JPRB, 0.83662E+00_JPRB, 0.86168E+00_JPRB, 0.87167E+00_JPRB ,&
&  0.88129E+00_JPRB, 0.89173E+00_JPRB, 0.89117E+00_JPRB, 0.89062E+00_JPRB, 0.87944E+00_JPRB, 0.88571E+00_JPRB, 0.90817E+00_JPRB ,&
&  0.10641E+01_JPRB, 0.25088E+01_JPRB, 0.18472E+01_JPRB, 0.22978E+01_JPRB, 0.17694E+01_JPRB, 0.15349E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.14121E+01_JPRB, 0.17475E+01_JPRB, 0.15442E+01_JPRB, 0.14133E+01_JPRB, 0.13113E+01_JPRB, 0.12518E+01_JPRB, 0.12058E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.11769E+01_JPRB, 0.11636E+01_JPRB, 0.11579E+01_JPRB, 0.11360E+01_JPRB, 0.11465E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.11706E+01_JPRB, 0.12049E+01_JPRB, 0.13026E+01_JPRB, 0.14293E+01_JPRB, 0.17410E+01_JPRB, 0.19256E+01_JPRB, 0.19248E+01_JPRB ,&
&  0.19103E+01_JPRB, 0.19264E+01_JPRB, 0.16958E+01_JPRB, 0.14709E+01_JPRB, 0.13588E+01_JPRB, 0.13079E+01_JPRB, 0.12938E+01_JPRB ,&
&  0.12192E+01_JPRB, 0.11731E+01_JPRB, 0.11382E+01_JPRB, 0.11183E+01_JPRB, 0.11237E+01_JPRB, 0.11171E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.10405E+01_JPRB, 0.10486E+01_JPRB, 0.10619E+01_JPRB, 0.10713E+01_JPRB, 0.10609E+01_JPRB, 0.10437E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10888E+01_JPRB, 0.11297E+01_JPRB, 0.11573E+01_JPRB, 0.12425E+01_JPRB, 0.12307E+01_JPRB, 0.11954E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.11966E+01_JPRB, 0.11904E+01_JPRB, 0.11967E+01_JPRB, 0.12040E+01_JPRB, 0.11673E+01_JPRB, 0.10256E+01_JPRB ,&
&  0.99703E+00_JPRB, 0.98406E+00_JPRB, 0.97464E+00_JPRB, 0.94751E+00_JPRB, 0.96012E+00_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.89895E+00_JPRB, 0.85474E+00_JPRB, 0.85285E+00_JPRB, 0.87511E+00_JPRB, 0.87146E+00_JPRB, 0.84985E+00_JPRB, 0.81309E+00_JPRB ,&
&  0.77289E+00_JPRB, 0.78107E+00_JPRB, 0.80457E+00_JPRB, 0.82459E+00_JPRB, 0.83859E+00_JPRB, 0.85117E+00_JPRB, 0.83836E+00_JPRB ,&
&  0.81584E+00_JPRB, 0.82261E+00_JPRB, 0.83810E+00_JPRB, 0.86653E+00_JPRB, 0.88823E+00_JPRB, 0.91016E+00_JPRB, 0.94064E+00_JPRB ,&
&  0.94877E+00_JPRB, 0.94233E+00_JPRB, 0.92689E+00_JPRB, 0.91520E+00_JPRB, 0.91526E+00_JPRB, 0.94214E+00_JPRB, 0.99913E+00_JPRB ,&
&  0.12810E+01_JPRB, 0.31794E+01_JPRB, 0.30948E+01_JPRB, 0.22600E+01_JPRB, 0.17803E+01_JPRB, 0.15343E+01_JPRB, 0.14397E+01_JPRB ,&
&  0.15777E+01_JPRB, 0.16290E+01_JPRB, 0.16210E+01_JPRB, 0.14386E+01_JPRB, 0.13257E+01_JPRB, 0.12790E+01_JPRB, 0.12819E+01_JPRB ,&
&  0.12577E+01_JPRB, 0.12582E+01_JPRB, 0.12397E+01_JPRB, 0.12205E+01_JPRB, 0.12081E+01_JPRB, 0.12373E+01_JPRB, 0.12789E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.14025E+01_JPRB, 0.15802E+01_JPRB, 0.17555E+01_JPRB, 0.17579E+01_JPRB, 0.18500E+01_JPRB, 0.17636E+01_JPRB ,&
&  0.17812E+01_JPRB, 0.17540E+01_JPRB, 0.15667E+01_JPRB, 0.14200E+01_JPRB, 0.12735E+01_JPRB, 0.13145E+01_JPRB, 0.12595E+01_JPRB ,&
&  0.11843E+01_JPRB, 0.11285E+01_JPRB, 0.11132E+01_JPRB, 0.11171E+01_JPRB, 0.11225E+01_JPRB, 0.11202E+01_JPRB, 0.10942E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10744E+01_JPRB, 0.10758E+01_JPRB, 0.10807E+01_JPRB, 0.10823E+01_JPRB, 0.10897E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.12111E+01_JPRB, 0.12348E+01_JPRB, 0.13033E+01_JPRB, 0.13026E+01_JPRB, 0.12342E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.12427E+01_JPRB, 0.12114E+01_JPRB, 0.11819E+01_JPRB, 0.11310E+01_JPRB, 0.10604E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.10112E+01_JPRB, 0.10203E+01_JPRB, 0.98563E+00_JPRB, 0.92917E+00_JPRB, 0.94041E+00_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.85965E+00_JPRB, 0.81321E+00_JPRB, 0.80700E+00_JPRB, 0.80190E+00_JPRB, 0.78804E+00_JPRB, 0.78318E+00_JPRB, 0.75075E+00_JPRB ,&
&  0.75625E+00_JPRB, 0.81034E+00_JPRB, 0.87759E+00_JPRB, 0.91272E+00_JPRB, 0.93446E+00_JPRB, 0.93613E+00_JPRB, 0.91355E+00_JPRB ,&
&  0.88726E+00_JPRB, 0.89890E+00_JPRB, 0.91655E+00_JPRB, 0.92771E+00_JPRB, 0.94689E+00_JPRB, 0.96278E+00_JPRB, 0.98825E+00_JPRB ,&
&  0.10090E+01_JPRB, 0.98707E+00_JPRB, 0.95639E+00_JPRB, 0.94875E+00_JPRB, 0.10095E+01_JPRB, 0.11069E+01_JPRB, 0.12416E+01_JPRB ,&
&  0.18329E+01_JPRB, 0.36502E+01_JPRB, 0.19091E+01_JPRB, 0.17728E+01_JPRB, 0.16761E+01_JPRB, 0.15535E+01_JPRB, 0.14506E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.13452E+01_JPRB, 0.13188E+01_JPRB, 0.14185E+01_JPRB, 0.13783E+01_JPRB, 0.13525E+01_JPRB, 0.13640E+01_JPRB ,&
&  0.13737E+01_JPRB, 0.13778E+01_JPRB, 0.13771E+01_JPRB, 0.13412E+01_JPRB, 0.13531E+01_JPRB, 0.13812E+01_JPRB, 0.14382E+01_JPRB ,&
&  0.15094E+01_JPRB, 0.16044E+01_JPRB, 0.17585E+01_JPRB, 0.17330E+01_JPRB, 0.18488E+01_JPRB, 0.18520E+01_JPRB, 0.15823E+01_JPRB ,&
&  0.13745E+01_JPRB, 0.14353E+01_JPRB, 0.14465E+01_JPRB, 0.14717E+01_JPRB, 0.13851E+01_JPRB, 0.11910E+01_JPRB, 0.12332E+01_JPRB ,&
&  0.11385E+01_JPRB, 0.11148E+01_JPRB, 0.11214E+01_JPRB, 0.11224E+01_JPRB, 0.11439E+01_JPRB, 0.11441E+01_JPRB, 0.11483E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.11188E+01_JPRB, 0.11130E+01_JPRB, 0.11130E+01_JPRB, 0.11180E+01_JPRB, 0.11433E+01_JPRB, 0.11887E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.12944E+01_JPRB, 0.13704E+01_JPRB, 0.14039E+01_JPRB, 0.14004E+01_JPRB, 0.13268E+01_JPRB, 0.12514E+01_JPRB ,&
&  0.12662E+01_JPRB, 0.13107E+01_JPRB, 0.11682E+01_JPRB, 0.11055E+01_JPRB, 0.10756E+01_JPRB, 0.10301E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.11292E+01_JPRB, 0.12457E+01_JPRB, 0.13333E+01_JPRB, 0.89019E+00_JPRB, 0.88865E+00_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.77676E+00_JPRB, 0.75896E+00_JPRB, 0.73034E+00_JPRB, 0.71235E+00_JPRB, 0.70809E+00_JPRB, 0.73444E+00_JPRB, 0.77988E+00_JPRB ,&
&  0.85122E+00_JPRB, 0.94517E+00_JPRB, 0.10222E+01_JPRB, 0.10572E+01_JPRB, 0.10496E+01_JPRB, 0.10561E+01_JPRB, 0.10139E+01_JPRB ,&
&  0.97013E+00_JPRB, 0.96603E+00_JPRB, 0.98072E+00_JPRB, 0.99106E+00_JPRB, 0.10074E+01_JPRB, 0.10314E+01_JPRB, 0.10598E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10891E+01_JPRB, 0.11092E+01_JPRB, 0.12012E+01_JPRB, 0.13569E+01_JPRB, 0.15996E+01_JPRB, 0.20336E+01_JPRB ,&
&  0.29493E+01_JPRB, 0.29572E+01_JPRB, 0.15016E+01_JPRB, 0.15657E+01_JPRB, 0.13530E+01_JPRB, 0.14318E+01_JPRB, 0.13975E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.14498E+01_JPRB, 0.14304E+01_JPRB, 0.15265E+01_JPRB, 0.15015E+01_JPRB, 0.15047E+01_JPRB, 0.15004E+01_JPRB ,&
&  0.15175E+01_JPRB, 0.15102E+01_JPRB, 0.14842E+01_JPRB, 0.14613E+01_JPRB, 0.14783E+01_JPRB, 0.15281E+01_JPRB, 0.15917E+01_JPRB ,&
&  0.16951E+01_JPRB, 0.17732E+01_JPRB, 0.19197E+01_JPRB, 0.17864E+01_JPRB, 0.19850E+01_JPRB, 0.19408E+01_JPRB, 0.16443E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.13132E+01_JPRB, 0.13923E+01_JPRB, 0.14871E+01_JPRB, 0.14026E+01_JPRB, 0.11238E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.10933E+01_JPRB, 0.10882E+01_JPRB, 0.10907E+01_JPRB, 0.11160E+01_JPRB, 0.11600E+01_JPRB, 0.11709E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.11393E+01_JPRB, 0.11357E+01_JPRB, 0.11397E+01_JPRB, 0.11557E+01_JPRB, 0.11991E+01_JPRB, 0.12273E+01_JPRB ,&
&  0.12604E+01_JPRB, 0.13109E+01_JPRB, 0.13761E+01_JPRB, 0.14438E+01_JPRB, 0.14349E+01_JPRB, 0.13606E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.12394E+01_JPRB, 0.12224E+01_JPRB, 0.11081E+01_JPRB, 0.10435E+01_JPRB, 0.96254E+00_JPRB, 0.94706E+00_JPRB, 0.92090E+00_JPRB ,&
&  0.91646E+00_JPRB, 0.91429E+00_JPRB, 0.87421E+00_JPRB, 0.76946E+00_JPRB, 0.77445E+00_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.72627E+00_JPRB, 0.73053E+00_JPRB, 0.72378E+00_JPRB, 0.74098E+00_JPRB, 0.74313E+00_JPRB, 0.81084E+00_JPRB, 0.88244E+00_JPRB ,&
&  0.99347E+00_JPRB, 0.10766E+01_JPRB, 0.11300E+01_JPRB, 0.11451E+01_JPRB, 0.11284E+01_JPRB, 0.10772E+01_JPRB, 0.10295E+01_JPRB ,&
&  0.10333E+01_JPRB, 0.10406E+01_JPRB, 0.10578E+01_JPRB, 0.10842E+01_JPRB, 0.11128E+01_JPRB, 0.11554E+01_JPRB, 0.12000E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.13481E+01_JPRB, 0.14628E+01_JPRB, 0.16289E+01_JPRB, 0.18447E+01_JPRB, 0.21380E+01_JPRB, 0.24293E+01_JPRB ,&
&  0.19010E+01_JPRB, 0.95792E+00_JPRB, 0.13510E+01_JPRB, 0.12999E+01_JPRB, 0.12025E+01_JPRB, 0.14189E+01_JPRB, 0.16919E+01_JPRB ,&
&  0.16212E+01_JPRB, 0.15189E+01_JPRB, 0.14589E+01_JPRB, 0.16149E+01_JPRB, 0.15930E+01_JPRB, 0.16027E+01_JPRB, 0.16108E+01_JPRB ,&
&  0.16052E+01_JPRB, 0.15832E+01_JPRB, 0.15595E+01_JPRB, 0.15758E+01_JPRB, 0.16081E+01_JPRB, 0.16701E+01_JPRB, 0.17401E+01_JPRB ,&
&  0.18334E+01_JPRB, 0.19373E+01_JPRB, 0.20895E+01_JPRB, 0.17929E+01_JPRB, 0.19950E+01_JPRB, 0.21257E+01_JPRB, 0.19932E+01_JPRB ,&
&  0.19454E+01_JPRB, 0.13125E+01_JPRB, 0.11899E+01_JPRB, 0.14560E+01_JPRB, 0.13741E+01_JPRB, 0.10869E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.10629E+01_JPRB, 0.10863E+01_JPRB, 0.10739E+01_JPRB, 0.11426E+01_JPRB, 0.11808E+01_JPRB, 0.12064E+01_JPRB, 0.11831E+01_JPRB ,&
&  0.11598E+01_JPRB, 0.11287E+01_JPRB, 0.11212E+01_JPRB, 0.11335E+01_JPRB, 0.11468E+01_JPRB, 0.11824E+01_JPRB, 0.11941E+01_JPRB ,&
&  0.12046E+01_JPRB, 0.12367E+01_JPRB, 0.12976E+01_JPRB, 0.13556E+01_JPRB, 0.13779E+01_JPRB, 0.13048E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11299E+01_JPRB, 0.10145E+01_JPRB, 0.89734E+00_JPRB, 0.85597E+00_JPRB, 0.79035E+00_JPRB, 0.72961E+00_JPRB ,&
&  0.74177E+00_JPRB, 0.74036E+00_JPRB, 0.72621E+00_JPRB, 0.72531E+00_JPRB, 0.72927E+00_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.81298E+00_JPRB, 0.80516E+00_JPRB, 0.79784E+00_JPRB, 0.80227E+00_JPRB, 0.81177E+00_JPRB, 0.86130E+00_JPRB, 0.94195E+00_JPRB ,&
&  0.10206E+01_JPRB, 0.10702E+01_JPRB, 0.11150E+01_JPRB, 0.11396E+01_JPRB, 0.11467E+01_JPRB, 0.11113E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.11141E+01_JPRB, 0.11438E+01_JPRB, 0.11880E+01_JPRB, 0.12291E+01_JPRB, 0.12749E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.13935E+01_JPRB, 0.14882E+01_JPRB, 0.15885E+01_JPRB, 0.17374E+01_JPRB, 0.18634E+01_JPRB, 0.19543E+01_JPRB, 0.19497E+01_JPRB ,&
&  0.90125E+00_JPRB, 0.10026E+01_JPRB, 0.14314E+01_JPRB, 0.13928E+01_JPRB, 0.14883E+01_JPRB, 0.16378E+01_JPRB, 0.18220E+01_JPRB ,&
&  0.17540E+01_JPRB, 0.15532E+01_JPRB, 0.15308E+01_JPRB, 0.17031E+01_JPRB, 0.16529E+01_JPRB, 0.16920E+01_JPRB, 0.17140E+01_JPRB ,&
&  0.17016E+01_JPRB, 0.16775E+01_JPRB, 0.16677E+01_JPRB, 0.16940E+01_JPRB, 0.17453E+01_JPRB, 0.18043E+01_JPRB, 0.18857E+01_JPRB ,&
&  0.19575E+01_JPRB, 0.20499E+01_JPRB, 0.21548E+01_JPRB, 0.18134E+01_JPRB, 0.19228E+01_JPRB, 0.21543E+01_JPRB, 0.25120E+01_JPRB ,&
&  0.20116E+01_JPRB, 0.11961E+01_JPRB, 0.11343E+01_JPRB, 0.14000E+01_JPRB, 0.12759E+01_JPRB, 0.12533E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.99353E+00_JPRB, 0.10219E+01_JPRB, 0.10872E+01_JPRB, 0.11465E+01_JPRB, 0.11861E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11129E+01_JPRB, 0.10775E+01_JPRB, 0.10618E+01_JPRB, 0.10702E+01_JPRB, 0.10912E+01_JPRB, 0.11051E+01_JPRB, 0.11023E+01_JPRB ,&
&  0.10970E+01_JPRB, 0.10994E+01_JPRB, 0.11396E+01_JPRB, 0.11646E+01_JPRB, 0.11884E+01_JPRB, 0.12137E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.11848E+01_JPRB, 0.11591E+01_JPRB, 0.12441E+01_JPRB, 0.11479E+01_JPRB, 0.10243E+01_JPRB, 0.88264E+00_JPRB, 0.75107E+00_JPRB ,&
&  0.77290E+00_JPRB, 0.78382E+00_JPRB, 0.78211E+00_JPRB, 0.79094E+00_JPRB, 0.80896E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.90555E+00_JPRB, 0.88904E+00_JPRB, 0.87794E+00_JPRB, 0.86792E+00_JPRB, 0.88009E+00_JPRB, 0.91568E+00_JPRB, 0.95729E+00_JPRB ,&
&  0.10002E+01_JPRB, 0.10496E+01_JPRB, 0.11155E+01_JPRB, 0.11428E+01_JPRB, 0.11865E+01_JPRB, 0.11897E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.11856E+01_JPRB, 0.12176E+01_JPRB, 0.12517E+01_JPRB, 0.12698E+01_JPRB, 0.13087E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.14270E+01_JPRB, 0.14923E+01_JPRB, 0.15737E+01_JPRB, 0.17036E+01_JPRB, 0.17594E+01_JPRB, 0.17874E+01_JPRB, 0.18421E+01_JPRB ,&
&  0.98127E+00_JPRB, 0.14363E+01_JPRB, 0.15161E+01_JPRB, 0.14993E+01_JPRB, 0.16019E+01_JPRB, 0.16312E+01_JPRB, 0.16563E+01_JPRB ,&
&  0.17323E+01_JPRB, 0.17066E+01_JPRB, 0.16591E+01_JPRB, 0.16304E+01_JPRB, 0.17247E+01_JPRB, 0.17645E+01_JPRB, 0.17955E+01_JPRB ,&
&  0.17912E+01_JPRB, 0.17820E+01_JPRB, 0.18125E+01_JPRB, 0.18751E+01_JPRB, 0.19277E+01_JPRB, 0.19828E+01_JPRB, 0.20615E+01_JPRB ,&
&  0.20884E+01_JPRB, 0.21046E+01_JPRB, 0.21875E+01_JPRB, 0.19692E+01_JPRB, 0.18446E+01_JPRB, 0.19624E+01_JPRB, 0.18326E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.85945E+00_JPRB, 0.99682E+00_JPRB, 0.12680E+01_JPRB, 0.11181E+01_JPRB, 0.10520E+01_JPRB, 0.97745E+00_JPRB ,&
&  0.87510E+00_JPRB, 0.82938E+00_JPRB, 0.90526E+00_JPRB, 0.98436E+00_JPRB, 0.10569E+01_JPRB, 0.11148E+01_JPRB, 0.11107E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10141E+01_JPRB, 0.97844E+00_JPRB, 0.97469E+00_JPRB, 0.99715E+00_JPRB, 0.10169E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.10122E+01_JPRB, 0.99460E+00_JPRB, 0.10281E+01_JPRB, 0.10590E+01_JPRB, 0.10951E+01_JPRB, 0.11318E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.15093E+01_JPRB, 0.17246E+01_JPRB, 0.19803E+01_JPRB, 0.19275E+01_JPRB, 0.17478E+01_JPRB, 0.12642E+01_JPRB, 0.86248E+00_JPRB ,&
&  0.88466E+00_JPRB, 0.90676E+00_JPRB, 0.91085E+00_JPRB, 0.92275E+00_JPRB, 0.91453E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.10485E+01_JPRB, 0.10342E+01_JPRB, 0.10346E+01_JPRB, 0.10343E+01_JPRB, 0.10330E+01_JPRB, 0.10433E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10809E+01_JPRB, 0.11193E+01_JPRB, 0.11788E+01_JPRB, 0.12196E+01_JPRB, 0.12721E+01_JPRB, 0.12818E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.12540E+01_JPRB, 0.12945E+01_JPRB, 0.13316E+01_JPRB, 0.13495E+01_JPRB, 0.13185E+01_JPRB, 0.13063E+01_JPRB, 0.13330E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.14275E+01_JPRB, 0.14705E+01_JPRB, 0.15342E+01_JPRB, 0.15541E+01_JPRB, 0.14924E+01_JPRB, 0.10395E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.15707E+01_JPRB, 0.15919E+01_JPRB, 0.15490E+01_JPRB, 0.15322E+01_JPRB, 0.14716E+01_JPRB, 0.15173E+01_JPRB ,&
&  0.16129E+01_JPRB, 0.16976E+01_JPRB, 0.16587E+01_JPRB, 0.15446E+01_JPRB, 0.16985E+01_JPRB, 0.17120E+01_JPRB, 0.17513E+01_JPRB ,&
&  0.17739E+01_JPRB, 0.17982E+01_JPRB, 0.18295E+01_JPRB, 0.18674E+01_JPRB, 0.18951E+01_JPRB, 0.19308E+01_JPRB, 0.19513E+01_JPRB ,&
&  0.19311E+01_JPRB, 0.18962E+01_JPRB, 0.19096E+01_JPRB, 0.17912E+01_JPRB, 0.17310E+01_JPRB, 0.17742E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.91990E+00_JPRB, 0.69495E+00_JPRB, 0.99409E+00_JPRB, 0.12710E+01_JPRB, 0.11201E+01_JPRB, 0.10156E+01_JPRB, 0.96844E+00_JPRB ,&
&  0.88010E+00_JPRB, 0.79219E+00_JPRB, 0.77092E+00_JPRB, 0.82893E+00_JPRB, 0.91133E+00_JPRB, 0.97865E+00_JPRB, 0.99686E+00_JPRB ,&
&  0.96744E+00_JPRB, 0.95073E+00_JPRB, 0.95959E+00_JPRB, 0.99120E+00_JPRB, 0.10600E+01_JPRB, 0.12014E+01_JPRB, 0.13926E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.97406E+00_JPRB, 0.10628E+01_JPRB, 0.11164E+01_JPRB, 0.12207E+01_JPRB, 0.13776E+01_JPRB, 0.16789E+01_JPRB ,&
&  0.21509E+01_JPRB, 0.26485E+01_JPRB, 0.25862E+01_JPRB, 0.21647E+01_JPRB, 0.27240E+01_JPRB, 0.18358E+01_JPRB, 0.99363E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.10446E+01_JPRB, 0.10646E+01_JPRB, 0.10616E+01_JPRB, 0.10580E+01_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.10903E+01_JPRB, 0.10944E+01_JPRB, 0.10889E+01_JPRB, 0.11214E+01_JPRB, 0.11214E+01_JPRB, 0.11391E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11679E+01_JPRB, 0.11902E+01_JPRB, 0.12175E+01_JPRB, 0.12608E+01_JPRB, 0.12780E+01_JPRB, 0.12989E+01_JPRB, 0.13337E+01_JPRB ,&
&  0.13685E+01_JPRB, 0.14113E+01_JPRB, 0.14149E+01_JPRB, 0.14063E+01_JPRB, 0.13787E+01_JPRB, 0.13630E+01_JPRB, 0.13414E+01_JPRB ,&
&  0.13249E+01_JPRB, 0.13250E+01_JPRB, 0.13392E+01_JPRB, 0.13642E+01_JPRB, 0.13932E+01_JPRB, 0.14754E+01_JPRB, 0.10212E+01_JPRB ,&
&  0.14860E+01_JPRB, 0.16002E+01_JPRB, 0.15175E+01_JPRB, 0.14673E+01_JPRB, 0.13818E+01_JPRB, 0.14023E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.15426E+01_JPRB, 0.16252E+01_JPRB, 0.16269E+01_JPRB, 0.15657E+01_JPRB, 0.15213E+01_JPRB, 0.15408E+01_JPRB, 0.15548E+01_JPRB ,&
&  0.15634E+01_JPRB, 0.15653E+01_JPRB, 0.15651E+01_JPRB, 0.15563E+01_JPRB, 0.15537E+01_JPRB, 0.15670E+01_JPRB, 0.15468E+01_JPRB ,&
&  0.15001E+01_JPRB, 0.14501E+01_JPRB, 0.14269E+01_JPRB, 0.14654E+01_JPRB, 0.14293E+01_JPRB, 0.12366E+01_JPRB, 0.95969E+00_JPRB ,&
&  0.64632E+00_JPRB, 0.64307E+00_JPRB, 0.95932E+00_JPRB, 0.13604E+01_JPRB, 0.12263E+01_JPRB, 0.10836E+01_JPRB, 0.95279E+00_JPRB ,&
&  0.87722E+00_JPRB, 0.77917E+00_JPRB, 0.70056E+00_JPRB, 0.70300E+00_JPRB, 0.72856E+00_JPRB, 0.78303E+00_JPRB, 0.77990E+00_JPRB ,&
&  0.79902E+00_JPRB, 0.80923E+00_JPRB, 0.80873E+00_JPRB, 0.83738E+00_JPRB, 0.93977E+00_JPRB, 0.10039E+01_JPRB, 0.93101E+00_JPRB ,&
&  0.10327E+01_JPRB, 0.86894E+00_JPRB, 0.87640E+00_JPRB, 0.96039E+00_JPRB, 0.98549E+00_JPRB, 0.11754E+01_JPRB, 0.14714E+01_JPRB ,&
&  0.19234E+01_JPRB, 0.18759E+01_JPRB, 0.24825E+01_JPRB, 0.37782E+01_JPRB, 0.29702E+01_JPRB, 0.21260E+01_JPRB, 0.11600E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11198E+01_JPRB, 0.11290E+01_JPRB, 0.11113E+01_JPRB, 0.10976E+01_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.10180E+01_JPRB, 0.10311E+01_JPRB, 0.10522E+01_JPRB, 0.10807E+01_JPRB, 0.10967E+01_JPRB, 0.11216E+01_JPRB, 0.11330E+01_JPRB ,&
&  0.11471E+01_JPRB, 0.11800E+01_JPRB, 0.12117E+01_JPRB, 0.12464E+01_JPRB, 0.12783E+01_JPRB, 0.13085E+01_JPRB, 0.13376E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.13877E+01_JPRB, 0.14002E+01_JPRB, 0.14012E+01_JPRB, 0.13874E+01_JPRB, 0.13806E+01_JPRB, 0.13624E+01_JPRB ,&
&  0.13459E+01_JPRB, 0.13505E+01_JPRB, 0.13655E+01_JPRB, 0.13769E+01_JPRB, 0.14005E+01_JPRB, 0.15196E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.15500E+01_JPRB, 0.15599E+01_JPRB, 0.14972E+01_JPRB, 0.13500E+01_JPRB, 0.12761E+01_JPRB, 0.12718E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.13522E+01_JPRB, 0.14281E+01_JPRB, 0.14882E+01_JPRB, 0.13842E+01_JPRB, 0.12682E+01_JPRB, 0.12782E+01_JPRB, 0.12838E+01_JPRB ,&
&  0.12944E+01_JPRB, 0.12908E+01_JPRB, 0.12860E+01_JPRB, 0.12662E+01_JPRB, 0.12508E+01_JPRB, 0.12299E+01_JPRB, 0.11859E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.10540E+01_JPRB, 0.95281E+00_JPRB, 0.96055E+00_JPRB, 0.94598E+00_JPRB, 0.81828E+00_JPRB, 0.62709E+00_JPRB ,&
&  0.57761E+00_JPRB, 0.78504E+00_JPRB, 0.10272E+01_JPRB, 0.14368E+01_JPRB, 0.13102E+01_JPRB, 0.11756E+01_JPRB, 0.10516E+01_JPRB ,&
&  0.86478E+00_JPRB, 0.75073E+00_JPRB, 0.68949E+00_JPRB, 0.69106E+00_JPRB, 0.69212E+00_JPRB, 0.70041E+00_JPRB, 0.68742E+00_JPRB ,&
&  0.66556E+00_JPRB, 0.65281E+00_JPRB, 0.69459E+00_JPRB, 0.76165E+00_JPRB, 0.87641E+00_JPRB, 0.88348E+00_JPRB, 0.13379E+01_JPRB ,&
&  0.92468E+00_JPRB, 0.73647E+00_JPRB, 0.95270E+00_JPRB, 0.56852E+00_JPRB, 0.70812E+00_JPRB, 0.79522E+00_JPRB, 0.91308E+00_JPRB ,&
&  0.94667E+00_JPRB, 0.12443E+01_JPRB, 0.15429E+01_JPRB, 0.13699E+01_JPRB, 0.12333E+01_JPRB, 0.11201E+01_JPRB, 0.10750E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.10510E+01_JPRB, 0.10459E+01_JPRB, 0.10354E+01_JPRB, 0.10211E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.98244E+00_JPRB, 0.99113E+00_JPRB, 0.10045E+01_JPRB, 0.10222E+01_JPRB, 0.10392E+01_JPRB, 0.10552E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.10938E+01_JPRB, 0.11194E+01_JPRB, 0.11496E+01_JPRB, 0.11636E+01_JPRB, 0.11821E+01_JPRB, 0.11882E+01_JPRB ,&
&  0.12140E+01_JPRB, 0.12439E+01_JPRB, 0.12590E+01_JPRB, 0.12767E+01_JPRB, 0.12890E+01_JPRB, 0.13058E+01_JPRB, 0.13270E+01_JPRB ,&
&  0.13446E+01_JPRB, 0.13870E+01_JPRB, 0.14506E+01_JPRB, 0.15014E+01_JPRB, 0.15739E+01_JPRB, 0.17260E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.15951E+01_JPRB, 0.14506E+01_JPRB, 0.12630E+01_JPRB, 0.12004E+01_JPRB, 0.11477E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.10851E+01_JPRB, 0.10364E+01_JPRB, 0.10475E+01_JPRB, 0.10386E+01_JPRB, 0.10309E+01_JPRB, 0.10761E+01_JPRB, 0.10816E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.11138E+01_JPRB, 0.11367E+01_JPRB, 0.11471E+01_JPRB, 0.11737E+01_JPRB, 0.11724E+01_JPRB, 0.11214E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10185E+01_JPRB, 0.96486E+00_JPRB, 0.94558E+00_JPRB, 0.92174E+00_JPRB, 0.82774E+00_JPRB, 0.73520E+00_JPRB ,&
&  0.83384E+00_JPRB, 0.92140E+00_JPRB, 0.10095E+01_JPRB, 0.14252E+01_JPRB, 0.13831E+01_JPRB, 0.12933E+01_JPRB, 0.10842E+01_JPRB ,&
&  0.94849E+00_JPRB, 0.78837E+00_JPRB, 0.74054E+00_JPRB, 0.75810E+00_JPRB, 0.75102E+00_JPRB, 0.74434E+00_JPRB, 0.71703E+00_JPRB ,&
&  0.66534E+00_JPRB, 0.64782E+00_JPRB, 0.71869E+00_JPRB, 0.83522E+00_JPRB, 0.10107E+01_JPRB, 0.11033E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.94001E+00_JPRB, 0.57127E+00_JPRB, 0.62787E+00_JPRB, 0.68730E+00_JPRB, 0.65195E+00_JPRB, 0.66138E+00_JPRB, 0.78975E+00_JPRB ,&
&  0.99946E+00_JPRB, 0.10644E+01_JPRB, 0.10855E+01_JPRB, 0.10329E+01_JPRB, 0.10239E+01_JPRB, 0.10239E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.99932E+00_JPRB, 0.99651E+00_JPRB, 0.98644E+00_JPRB, 0.98012E+00_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.95646E+00_JPRB, 0.95854E+00_JPRB, 0.96449E+00_JPRB, 0.97504E+00_JPRB, 0.98433E+00_JPRB, 0.99377E+00_JPRB, 0.99781E+00_JPRB ,&
&  0.99257E+00_JPRB, 0.99524E+00_JPRB, 0.10062E+01_JPRB, 0.10227E+01_JPRB, 0.10386E+01_JPRB, 0.10527E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.10692E+01_JPRB, 0.10731E+01_JPRB, 0.10939E+01_JPRB, 0.11174E+01_JPRB, 0.11307E+01_JPRB, 0.11329E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11357E+01_JPRB, 0.11476E+01_JPRB, 0.11772E+01_JPRB, 0.11943E+01_JPRB, 0.12199E+01_JPRB, 0.12900E+01_JPRB, 0.15215E+01_JPRB ,&
&  0.14938E+01_JPRB, 0.15907E+01_JPRB, 0.13678E+01_JPRB, 0.12135E+01_JPRB, 0.11981E+01_JPRB, 0.11441E+01_JPRB, 0.10989E+01_JPRB ,&
&  0.10522E+01_JPRB, 0.10219E+01_JPRB, 0.97173E+00_JPRB, 0.97202E+00_JPRB, 0.95974E+00_JPRB, 0.98832E+00_JPRB, 0.97419E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10386E+01_JPRB, 0.10952E+01_JPRB, 0.11456E+01_JPRB, 0.12236E+01_JPRB, 0.12513E+01_JPRB, 0.11631E+01_JPRB ,&
&  0.11491E+01_JPRB, 0.12403E+01_JPRB, 0.13212E+01_JPRB, 0.13838E+01_JPRB, 0.13151E+01_JPRB, 0.12182E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11269E+01_JPRB, 0.13259E+01_JPRB, 0.13501E+01_JPRB, 0.14089E+01_JPRB, 0.13719E+01_JPRB, 0.12391E+01_JPRB ,&
&  0.10999E+01_JPRB, 0.95197E+00_JPRB, 0.94707E+00_JPRB, 0.95694E+00_JPRB, 0.99256E+00_JPRB, 0.98042E+00_JPRB, 0.96138E+00_JPRB ,&
&  0.88852E+00_JPRB, 0.82576E+00_JPRB, 0.89256E+00_JPRB, 0.96971E+00_JPRB, 0.10419E+01_JPRB, 0.13144E+01_JPRB, 0.13213E+01_JPRB ,&
&  0.91827E+00_JPRB, 0.81829E+00_JPRB, 0.59302E+00_JPRB, 0.84064E+00_JPRB, 0.69490E+00_JPRB, 0.72036E+00_JPRB, 0.79793E+00_JPRB ,&
&  0.90851E+00_JPRB, 0.10048E+01_JPRB, 0.10686E+01_JPRB, 0.10302E+01_JPRB, 0.95849E+00_JPRB, 0.94623E+00_JPRB, 0.93949E+00_JPRB ,&
&  0.94651E+00_JPRB, 0.95000E+00_JPRB, 0.95117E+00_JPRB, 0.95111E+00_JPRB, 0.94733E+00_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.89953E+00_JPRB, 0.90138E+00_JPRB, 0.91878E+00_JPRB, 0.92191E+00_JPRB, 0.92053E+00_JPRB, 0.92759E+00_JPRB, 0.92965E+00_JPRB ,&
&  0.92397E+00_JPRB, 0.93132E+00_JPRB, 0.94117E+00_JPRB, 0.95249E+00_JPRB, 0.96766E+00_JPRB, 0.98180E+00_JPRB, 0.99036E+00_JPRB ,&
&  0.98220E+00_JPRB, 0.97386E+00_JPRB, 0.99754E+00_JPRB, 0.10166E+01_JPRB, 0.10185E+01_JPRB, 0.10238E+01_JPRB, 0.10650E+01_JPRB ,&
&  0.11465E+01_JPRB, 0.11956E+01_JPRB, 0.11981E+01_JPRB, 0.11706E+01_JPRB, 0.12030E+01_JPRB, 0.13995E+01_JPRB, 0.21182E+01_JPRB ,&
&  0.20969E+01_JPRB, 0.14878E+01_JPRB, 0.13315E+01_JPRB, 0.12074E+01_JPRB, 0.12196E+01_JPRB, 0.12632E+01_JPRB, 0.12459E+01_JPRB ,&
&  0.12036E+01_JPRB, 0.12111E+01_JPRB, 0.12197E+01_JPRB, 0.12293E+01_JPRB, 0.12602E+01_JPRB, 0.12888E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.12335E+01_JPRB, 0.12513E+01_JPRB, 0.12882E+01_JPRB, 0.14147E+01_JPRB, 0.16420E+01_JPRB, 0.16044E+01_JPRB ,&
&  0.16039E+01_JPRB, 0.16906E+01_JPRB, 0.17499E+01_JPRB, 0.17474E+01_JPRB, 0.16833E+01_JPRB, 0.15681E+01_JPRB, 0.14873E+01_JPRB ,&
&  0.14283E+01_JPRB, 0.13131E+01_JPRB, 0.11371E+01_JPRB, 0.13084E+01_JPRB, 0.14110E+01_JPRB, 0.15085E+01_JPRB, 0.14266E+01_JPRB ,&
&  0.13561E+01_JPRB, 0.12285E+01_JPRB, 0.12167E+01_JPRB, 0.13004E+01_JPRB, 0.13894E+01_JPRB, 0.13876E+01_JPRB, 0.13327E+01_JPRB ,&
&  0.12268E+01_JPRB, 0.11492E+01_JPRB, 0.11417E+01_JPRB, 0.10748E+01_JPRB, 0.10526E+01_JPRB, 0.92654E+00_JPRB, 0.96883E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.11140E+01_JPRB, 0.95095E+00_JPRB, 0.12688E+01_JPRB, 0.84877E+00_JPRB, 0.77767E+00_JPRB, 0.78467E+00_JPRB ,&
&  0.80365E+00_JPRB, 0.83171E+00_JPRB, 0.83852E+00_JPRB, 0.92357E+00_JPRB, 0.91134E+00_JPRB, 0.90321E+00_JPRB, 0.91111E+00_JPRB ,&
&  0.88433E+00_JPRB, 0.88040E+00_JPRB, 0.88112E+00_JPRB, 0.89220E+00_JPRB, 0.89674E+00_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.84335E+00_JPRB, 0.83757E+00_JPRB, 0.83514E+00_JPRB, 0.84954E+00_JPRB, 0.86179E+00_JPRB, 0.85029E+00_JPRB, 0.85942E+00_JPRB ,&
&  0.83940E+00_JPRB, 0.82371E+00_JPRB, 0.83609E+00_JPRB, 0.84300E+00_JPRB, 0.86212E+00_JPRB, 0.88183E+00_JPRB, 0.89538E+00_JPRB ,&
&  0.89804E+00_JPRB, 0.89414E+00_JPRB, 0.85789E+00_JPRB, 0.87012E+00_JPRB, 0.97898E+00_JPRB, 0.11713E+01_JPRB, 0.15211E+01_JPRB ,&
&  0.18982E+01_JPRB, 0.20694E+01_JPRB, 0.21460E+01_JPRB, 0.21296E+01_JPRB, 0.20360E+01_JPRB, 0.18770E+01_JPRB, 0.23663E+01_JPRB ,&
&  0.22472E+01_JPRB, 0.15297E+01_JPRB, 0.15046E+01_JPRB, 0.13944E+01_JPRB, 0.14111E+01_JPRB, 0.14034E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.13981E+01_JPRB, 0.14197E+01_JPRB, 0.14628E+01_JPRB, 0.15390E+01_JPRB, 0.16178E+01_JPRB, 0.17055E+01_JPRB, 0.17660E+01_JPRB ,&
&  0.17830E+01_JPRB, 0.17958E+01_JPRB, 0.18571E+01_JPRB, 0.18248E+01_JPRB, 0.18132E+01_JPRB, 0.18895E+01_JPRB, 0.19600E+01_JPRB ,&
&  0.20058E+01_JPRB, 0.20914E+01_JPRB, 0.18727E+01_JPRB, 0.18345E+01_JPRB, 0.18642E+01_JPRB, 0.16730E+01_JPRB, 0.16401E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.14278E+01_JPRB, 0.10684E+01_JPRB, 0.11680E+01_JPRB, 0.13124E+01_JPRB, 0.14615E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.15795E+01_JPRB, 0.15026E+01_JPRB, 0.15131E+01_JPRB, 0.16146E+01_JPRB, 0.16732E+01_JPRB, 0.16726E+01_JPRB, 0.16539E+01_JPRB ,&
&  0.16370E+01_JPRB, 0.14748E+01_JPRB, 0.13878E+01_JPRB, 0.12578E+01_JPRB, 0.11600E+01_JPRB, 0.11105E+01_JPRB, 0.10468E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.11814E+01_JPRB, 0.14553E+01_JPRB, 0.13499E+01_JPRB, 0.92246E+00_JPRB, 0.87265E+00_JPRB, 0.84369E+00_JPRB ,&
&  0.84365E+00_JPRB, 0.84829E+00_JPRB, 0.86725E+00_JPRB, 0.86297E+00_JPRB, 0.84776E+00_JPRB, 0.83246E+00_JPRB, 0.81912E+00_JPRB ,&
&  0.83526E+00_JPRB, 0.85733E+00_JPRB, 0.86730E+00_JPRB, 0.85602E+00_JPRB, 0.84549E+00_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.85813E+00_JPRB, 0.85598E+00_JPRB, 0.84785E+00_JPRB, 0.86060E+00_JPRB, 0.85325E+00_JPRB, 0.87115E+00_JPRB, 0.85710E+00_JPRB ,&
&  0.84536E+00_JPRB, 0.84875E+00_JPRB, 0.86203E+00_JPRB, 0.87683E+00_JPRB, 0.91208E+00_JPRB, 0.97724E+00_JPRB, 0.10579E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.11718E+01_JPRB, 0.12314E+01_JPRB, 0.13721E+01_JPRB, 0.16673E+01_JPRB, 0.19693E+01_JPRB, 0.20817E+01_JPRB ,&
&  0.21989E+01_JPRB, 0.23455E+01_JPRB, 0.24214E+01_JPRB, 0.24599E+01_JPRB, 0.24265E+01_JPRB, 0.23779E+01_JPRB, 0.23729E+01_JPRB ,&
&  0.19925E+01_JPRB, 0.15977E+01_JPRB, 0.14900E+01_JPRB, 0.14550E+01_JPRB, 0.14699E+01_JPRB, 0.14763E+01_JPRB, 0.14815E+01_JPRB ,&
&  0.14919E+01_JPRB, 0.15173E+01_JPRB, 0.15364E+01_JPRB, 0.15807E+01_JPRB, 0.16568E+01_JPRB, 0.17515E+01_JPRB, 0.18001E+01_JPRB ,&
&  0.18051E+01_JPRB, 0.18151E+01_JPRB, 0.18324E+01_JPRB, 0.17240E+01_JPRB, 0.18476E+01_JPRB, 0.19017E+01_JPRB, 0.19211E+01_JPRB ,&
&  0.18923E+01_JPRB, 0.18236E+01_JPRB, 0.18525E+01_JPRB, 0.18275E+01_JPRB, 0.17872E+01_JPRB, 0.16729E+01_JPRB, 0.16041E+01_JPRB ,&
&  0.15972E+01_JPRB, 0.14376E+01_JPRB, 0.11384E+01_JPRB, 0.12819E+01_JPRB, 0.14601E+01_JPRB, 0.14057E+01_JPRB, 0.18270E+01_JPRB ,&
&  0.17891E+01_JPRB, 0.17409E+01_JPRB, 0.17604E+01_JPRB, 0.17993E+01_JPRB, 0.19011E+01_JPRB, 0.19196E+01_JPRB, 0.19258E+01_JPRB ,&
&  0.20427E+01_JPRB, 0.19147E+01_JPRB, 0.17736E+01_JPRB, 0.16204E+01_JPRB, 0.14507E+01_JPRB, 0.13016E+01_JPRB, 0.12107E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11545E+01_JPRB, 0.11039E+01_JPRB, 0.10718E+01_JPRB, 0.93590E+00_JPRB, 0.87806E+00_JPRB, 0.86990E+00_JPRB ,&
&  0.89192E+00_JPRB, 0.89065E+00_JPRB, 0.88479E+00_JPRB, 0.86841E+00_JPRB, 0.86041E+00_JPRB, 0.85493E+00_JPRB, 0.85568E+00_JPRB ,&
&  0.86723E+00_JPRB, 0.88315E+00_JPRB, 0.87986E+00_JPRB, 0.87502E+00_JPRB, 0.86754E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.86586E+00_JPRB, 0.85406E+00_JPRB, 0.84385E+00_JPRB, 0.87551E+00_JPRB, 0.87194E+00_JPRB, 0.86370E+00_JPRB, 0.87797E+00_JPRB ,&
&  0.90086E+00_JPRB, 0.93302E+00_JPRB, 0.97457E+00_JPRB, 0.10184E+01_JPRB, 0.10733E+01_JPRB, 0.11487E+01_JPRB, 0.12248E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.14194E+01_JPRB, 0.15912E+01_JPRB, 0.18120E+01_JPRB, 0.20809E+01_JPRB, 0.22301E+01_JPRB, 0.22640E+01_JPRB ,&
&  0.21535E+01_JPRB, 0.21864E+01_JPRB, 0.22169E+01_JPRB, 0.21619E+01_JPRB, 0.18292E+01_JPRB, 0.19128E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.18735E+01_JPRB, 0.16445E+01_JPRB, 0.14689E+01_JPRB, 0.14259E+01_JPRB, 0.14036E+01_JPRB, 0.14296E+01_JPRB, 0.14560E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.14882E+01_JPRB, 0.15034E+01_JPRB, 0.15259E+01_JPRB, 0.15738E+01_JPRB, 0.16572E+01_JPRB, 0.16949E+01_JPRB ,&
&  0.17149E+01_JPRB, 0.17567E+01_JPRB, 0.17454E+01_JPRB, 0.16919E+01_JPRB, 0.17134E+01_JPRB, 0.17271E+01_JPRB, 0.17366E+01_JPRB ,&
&  0.17739E+01_JPRB, 0.16951E+01_JPRB, 0.17151E+01_JPRB, 0.17417E+01_JPRB, 0.16586E+01_JPRB, 0.15079E+01_JPRB, 0.14909E+01_JPRB ,&
&  0.15444E+01_JPRB, 0.15838E+01_JPRB, 0.12223E+01_JPRB, 0.16446E+01_JPRB, 0.19703E+01_JPRB, 0.18124E+01_JPRB, 0.18453E+01_JPRB ,&
&  0.18126E+01_JPRB, 0.18289E+01_JPRB, 0.18844E+01_JPRB, 0.19426E+01_JPRB, 0.19795E+01_JPRB, 0.20174E+01_JPRB, 0.19157E+01_JPRB ,&
&  0.22034E+01_JPRB, 0.21235E+01_JPRB, 0.20179E+01_JPRB, 0.19330E+01_JPRB, 0.18154E+01_JPRB, 0.16077E+01_JPRB, 0.14043E+01_JPRB ,&
&  0.12134E+01_JPRB, 0.14551E+01_JPRB, 0.13414E+01_JPRB, 0.13940E+01_JPRB, 0.13264E+01_JPRB, 0.91722E+00_JPRB, 0.93893E+00_JPRB ,&
&  0.93015E+00_JPRB, 0.92965E+00_JPRB, 0.92005E+00_JPRB, 0.89993E+00_JPRB, 0.90244E+00_JPRB, 0.90891E+00_JPRB, 0.91544E+00_JPRB ,&
&  0.91436E+00_JPRB, 0.90496E+00_JPRB, 0.90128E+00_JPRB, 0.88563E+00_JPRB, 0.87701E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.90819E+00_JPRB, 0.89774E+00_JPRB, 0.88346E+00_JPRB, 0.89438E+00_JPRB, 0.90501E+00_JPRB, 0.90799E+00_JPRB, 0.90880E+00_JPRB ,&
&  0.93237E+00_JPRB, 0.98096E+00_JPRB, 0.10341E+01_JPRB, 0.10690E+01_JPRB, 0.10843E+01_JPRB, 0.11068E+01_JPRB, 0.11737E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.14390E+01_JPRB, 0.16474E+01_JPRB, 0.18277E+01_JPRB, 0.19880E+01_JPRB, 0.21071E+01_JPRB, 0.22628E+01_JPRB ,&
&  0.22793E+01_JPRB, 0.23004E+01_JPRB, 0.21826E+01_JPRB, 0.16934E+01_JPRB, 0.15870E+01_JPRB, 0.19140E+01_JPRB, 0.17056E+01_JPRB ,&
&  0.15420E+01_JPRB, 0.14699E+01_JPRB, 0.14188E+01_JPRB, 0.13705E+01_JPRB, 0.13390E+01_JPRB, 0.13629E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.13785E+01_JPRB, 0.13970E+01_JPRB, 0.14188E+01_JPRB, 0.14422E+01_JPRB, 0.14643E+01_JPRB, 0.14983E+01_JPRB, 0.15058E+01_JPRB ,&
&  0.15475E+01_JPRB, 0.15761E+01_JPRB, 0.16088E+01_JPRB, 0.16216E+01_JPRB, 0.15954E+01_JPRB, 0.16209E+01_JPRB, 0.16302E+01_JPRB ,&
&  0.16504E+01_JPRB, 0.16335E+01_JPRB, 0.16168E+01_JPRB, 0.16582E+01_JPRB, 0.16658E+01_JPRB, 0.14602E+01_JPRB, 0.14591E+01_JPRB ,&
&  0.15487E+01_JPRB, 0.15659E+01_JPRB, 0.14326E+01_JPRB, 0.17303E+01_JPRB, 0.11553E+01_JPRB, 0.14930E+01_JPRB, 0.18598E+01_JPRB ,&
&  0.18965E+01_JPRB, 0.19191E+01_JPRB, 0.19454E+01_JPRB, 0.19821E+01_JPRB, 0.20078E+01_JPRB, 0.19009E+01_JPRB, 0.20426E+01_JPRB ,&
&  0.21892E+01_JPRB, 0.21719E+01_JPRB, 0.21554E+01_JPRB, 0.21520E+01_JPRB, 0.21120E+01_JPRB, 0.19777E+01_JPRB, 0.18115E+01_JPRB ,&
&  0.15219E+01_JPRB, 0.16122E+01_JPRB, 0.14064E+01_JPRB, 0.12834E+01_JPRB, 0.10561E+01_JPRB, 0.91181E+00_JPRB, 0.92033E+00_JPRB ,&
&  0.93480E+00_JPRB, 0.92987E+00_JPRB, 0.91282E+00_JPRB, 0.92202E+00_JPRB, 0.92817E+00_JPRB, 0.93730E+00_JPRB, 0.92911E+00_JPRB ,&
&  0.93465E+00_JPRB, 0.94032E+00_JPRB, 0.94225E+00_JPRB, 0.93274E+00_JPRB, 0.92106E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.91844E+00_JPRB, 0.91353E+00_JPRB, 0.91230E+00_JPRB, 0.92121E+00_JPRB, 0.94617E+00_JPRB, 0.96531E+00_JPRB, 0.93370E+00_JPRB ,&
&  0.93276E+00_JPRB, 0.94783E+00_JPRB, 0.99353E+00_JPRB, 0.10157E+01_JPRB, 0.10311E+01_JPRB, 0.10479E+01_JPRB, 0.11111E+01_JPRB ,&
&  0.12253E+01_JPRB, 0.13982E+01_JPRB, 0.15911E+01_JPRB, 0.17462E+01_JPRB, 0.19137E+01_JPRB, 0.21138E+01_JPRB, 0.23076E+01_JPRB ,&
&  0.20196E+01_JPRB, 0.16226E+01_JPRB, 0.21397E+01_JPRB, 0.22669E+01_JPRB, 0.21759E+01_JPRB, 0.21401E+01_JPRB, 0.21734E+01_JPRB ,&
&  0.17047E+01_JPRB, 0.15378E+01_JPRB, 0.15347E+01_JPRB, 0.14297E+01_JPRB, 0.12694E+01_JPRB, 0.12782E+01_JPRB, 0.12959E+01_JPRB ,&
&  0.13049E+01_JPRB, 0.13030E+01_JPRB, 0.13248E+01_JPRB, 0.13527E+01_JPRB, 0.13746E+01_JPRB, 0.13677E+01_JPRB, 0.13472E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.13752E+01_JPRB, 0.13983E+01_JPRB, 0.13977E+01_JPRB, 0.14233E+01_JPRB, 0.14648E+01_JPRB, 0.14922E+01_JPRB ,&
&  0.15171E+01_JPRB, 0.15088E+01_JPRB, 0.15675E+01_JPRB, 0.16685E+01_JPRB, 0.17567E+01_JPRB, 0.15836E+01_JPRB, 0.16685E+01_JPRB ,&
&  0.16412E+01_JPRB, 0.16044E+01_JPRB, 0.15604E+01_JPRB, 0.17462E+01_JPRB, 0.12855E+01_JPRB, 0.15083E+01_JPRB, 0.16169E+01_JPRB ,&
&  0.19850E+01_JPRB, 0.20095E+01_JPRB, 0.20610E+01_JPRB, 0.21572E+01_JPRB, 0.22375E+01_JPRB, 0.20783E+01_JPRB, 0.21249E+01_JPRB ,&
&  0.22412E+01_JPRB, 0.23309E+01_JPRB, 0.23578E+01_JPRB, 0.23745E+01_JPRB, 0.22286E+01_JPRB, 0.22683E+01_JPRB, 0.23064E+01_JPRB ,&
&  0.23466E+01_JPRB, 0.19537E+01_JPRB, 0.16614E+01_JPRB, 0.13777E+01_JPRB, 0.11088E+01_JPRB, 0.99626E+00_JPRB, 0.95274E+00_JPRB ,&
&  0.93429E+00_JPRB, 0.94487E+00_JPRB, 0.91071E+00_JPRB, 0.89613E+00_JPRB, 0.90677E+00_JPRB, 0.93956E+00_JPRB, 0.93089E+00_JPRB ,&
&  0.93792E+00_JPRB, 0.93730E+00_JPRB, 0.94625E+00_JPRB, 0.94456E+00_JPRB, 0.93623E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.93951E+00_JPRB, 0.94918E+00_JPRB, 0.97859E+00_JPRB, 0.10124E+01_JPRB, 0.10433E+01_JPRB, 0.10932E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.88552E+00_JPRB, 0.91287E+00_JPRB, 0.93160E+00_JPRB, 0.95176E+00_JPRB, 0.95039E+00_JPRB, 0.97965E+00_JPRB ,&
&  0.10866E+01_JPRB, 0.12587E+01_JPRB, 0.14489E+01_JPRB, 0.16387E+01_JPRB, 0.18450E+01_JPRB, 0.21147E+01_JPRB, 0.20896E+01_JPRB ,&
&  0.16391E+01_JPRB, 0.17508E+01_JPRB, 0.23652E+01_JPRB, 0.23495E+01_JPRB, 0.22098E+01_JPRB, 0.20933E+01_JPRB, 0.18956E+01_JPRB ,&
&  0.16447E+01_JPRB, 0.15168E+01_JPRB, 0.15498E+01_JPRB, 0.13944E+01_JPRB, 0.12158E+01_JPRB, 0.12092E+01_JPRB, 0.12253E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.12302E+01_JPRB, 0.12358E+01_JPRB, 0.12563E+01_JPRB, 0.12730E+01_JPRB, 0.12662E+01_JPRB, 0.12529E+01_JPRB ,&
&  0.12557E+01_JPRB, 0.12729E+01_JPRB, 0.12761E+01_JPRB, 0.12654E+01_JPRB, 0.13113E+01_JPRB, 0.13609E+01_JPRB, 0.13895E+01_JPRB ,&
&  0.14090E+01_JPRB, 0.14688E+01_JPRB, 0.15172E+01_JPRB, 0.15431E+01_JPRB, 0.14918E+01_JPRB, 0.16679E+01_JPRB, 0.16659E+01_JPRB ,&
&  0.17285E+01_JPRB, 0.16956E+01_JPRB, 0.18109E+01_JPRB, 0.14103E+01_JPRB, 0.15165E+01_JPRB, 0.16432E+01_JPRB, 0.17876E+01_JPRB ,&
&  0.18561E+01_JPRB, 0.20474E+01_JPRB, 0.21431E+01_JPRB, 0.22764E+01_JPRB, 0.24194E+01_JPRB, 0.23261E+01_JPRB, 0.24172E+01_JPRB ,&
&  0.23273E+01_JPRB, 0.26487E+01_JPRB, 0.26075E+01_JPRB, 0.25078E+01_JPRB, 0.22332E+01_JPRB, 0.22272E+01_JPRB, 0.24508E+01_JPRB ,&
&  0.26426E+01_JPRB, 0.23978E+01_JPRB, 0.21830E+01_JPRB, 0.18579E+01_JPRB, 0.14794E+01_JPRB, 0.12800E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.99221E+00_JPRB, 0.89841E+00_JPRB, 0.86562E+00_JPRB, 0.88590E+00_JPRB, 0.89444E+00_JPRB, 0.93099E+00_JPRB, 0.93416E+00_JPRB ,&
&  0.92431E+00_JPRB, 0.93365E+00_JPRB, 0.94381E+00_JPRB, 0.94669E+00_JPRB, 0.94308E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.92022E+00_JPRB, 0.92488E+00_JPRB, 0.94755E+00_JPRB, 0.95803E+00_JPRB, 0.94527E+00_JPRB, 0.94701E+00_JPRB, 0.97541E+00_JPRB ,&
&  0.98227E+00_JPRB, 0.84798E+00_JPRB, 0.83224E+00_JPRB, 0.84132E+00_JPRB, 0.86489E+00_JPRB, 0.87809E+00_JPRB, 0.89013E+00_JPRB ,&
&  0.93680E+00_JPRB, 0.10732E+01_JPRB, 0.13024E+01_JPRB, 0.14684E+01_JPRB, 0.16265E+01_JPRB, 0.18844E+01_JPRB, 0.18292E+01_JPRB ,&
&  0.14554E+01_JPRB, 0.23021E+01_JPRB, 0.20803E+01_JPRB, 0.23157E+01_JPRB, 0.22818E+01_JPRB, 0.21967E+01_JPRB, 0.18282E+01_JPRB ,&
&  0.14812E+01_JPRB, 0.13534E+01_JPRB, 0.13484E+01_JPRB, 0.12802E+01_JPRB, 0.12249E+01_JPRB, 0.11588E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11575E+01_JPRB, 0.11523E+01_JPRB, 0.11477E+01_JPRB, 0.11668E+01_JPRB, 0.11808E+01_JPRB, 0.11799E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12048E+01_JPRB, 0.12233E+01_JPRB, 0.12165E+01_JPRB, 0.12448E+01_JPRB, 0.12751E+01_JPRB, 0.13326E+01_JPRB, 0.13727E+01_JPRB ,&
&  0.13407E+01_JPRB, 0.13083E+01_JPRB, 0.14287E+01_JPRB, 0.15005E+01_JPRB, 0.15427E+01_JPRB, 0.16688E+01_JPRB, 0.16273E+01_JPRB ,&
&  0.17902E+01_JPRB, 0.17008E+01_JPRB, 0.18457E+01_JPRB, 0.14395E+01_JPRB, 0.14887E+01_JPRB, 0.16548E+01_JPRB, 0.17431E+01_JPRB ,&
&  0.16994E+01_JPRB, 0.19755E+01_JPRB, 0.21292E+01_JPRB, 0.23594E+01_JPRB, 0.25320E+01_JPRB, 0.27471E+01_JPRB, 0.26024E+01_JPRB ,&
&  0.26433E+01_JPRB, 0.28247E+01_JPRB, 0.29924E+01_JPRB, 0.26127E+01_JPRB, 0.25549E+01_JPRB, 0.24030E+01_JPRB, 0.27694E+01_JPRB ,&
&  0.29577E+01_JPRB, 0.27076E+01_JPRB, 0.25723E+01_JPRB, 0.23739E+01_JPRB, 0.19234E+01_JPRB, 0.17461E+01_JPRB, 0.15055E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.10928E+01_JPRB, 0.94937E+00_JPRB, 0.92099E+00_JPRB, 0.92637E+00_JPRB, 0.93037E+00_JPRB, 0.95299E+00_JPRB ,&
&  0.95583E+00_JPRB, 0.95313E+00_JPRB, 0.95104E+00_JPRB, 0.94236E+00_JPRB, 0.92613E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.92231E+00_JPRB, 0.91428E+00_JPRB, 0.92069E+00_JPRB, 0.92117E+00_JPRB, 0.90928E+00_JPRB, 0.87936E+00_JPRB, 0.89304E+00_JPRB ,&
&  0.92540E+00_JPRB, 0.83448E+00_JPRB, 0.78091E+00_JPRB, 0.76092E+00_JPRB, 0.77197E+00_JPRB, 0.80534E+00_JPRB, 0.82978E+00_JPRB ,&
&  0.85736E+00_JPRB, 0.92121E+00_JPRB, 0.11245E+01_JPRB, 0.13350E+01_JPRB, 0.15345E+01_JPRB, 0.19145E+01_JPRB, 0.13741E+01_JPRB ,&
&  0.15446E+01_JPRB, 0.20248E+01_JPRB, 0.20424E+01_JPRB, 0.22427E+01_JPRB, 0.23608E+01_JPRB, 0.23094E+01_JPRB, 0.19548E+01_JPRB ,&
&  0.15861E+01_JPRB, 0.13675E+01_JPRB, 0.12498E+01_JPRB, 0.11784E+01_JPRB, 0.11531E+01_JPRB, 0.11339E+01_JPRB, 0.10891E+01_JPRB ,&
&  0.10807E+01_JPRB, 0.10828E+01_JPRB, 0.10710E+01_JPRB, 0.10868E+01_JPRB, 0.11003E+01_JPRB, 0.11000E+01_JPRB, 0.11289E+01_JPRB ,&
&  0.11694E+01_JPRB, 0.12029E+01_JPRB, 0.12327E+01_JPRB, 0.12064E+01_JPRB, 0.12702E+01_JPRB, 0.13172E+01_JPRB, 0.13656E+01_JPRB ,&
&  0.13009E+01_JPRB, 0.12457E+01_JPRB, 0.14456E+01_JPRB, 0.16424E+01_JPRB, 0.17468E+01_JPRB, 0.16895E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.17836E+01_JPRB, 0.18035E+01_JPRB, 0.17289E+01_JPRB, 0.14781E+01_JPRB, 0.15379E+01_JPRB, 0.17232E+01_JPRB, 0.18576E+01_JPRB ,&
&  0.19328E+01_JPRB, 0.20046E+01_JPRB, 0.21424E+01_JPRB, 0.22528E+01_JPRB, 0.23760E+01_JPRB, 0.27950E+01_JPRB, 0.28782E+01_JPRB ,&
&  0.33306E+01_JPRB, 0.39599E+01_JPRB, 0.35814E+01_JPRB, 0.29808E+01_JPRB, 0.27707E+01_JPRB, 0.26288E+01_JPRB, 0.27352E+01_JPRB ,&
&  0.30928E+01_JPRB, 0.28468E+01_JPRB, 0.24446E+01_JPRB, 0.24209E+01_JPRB, 0.21800E+01_JPRB, 0.21954E+01_JPRB, 0.20626E+01_JPRB ,&
&  0.17499E+01_JPRB, 0.15211E+01_JPRB, 0.12847E+01_JPRB, 0.11387E+01_JPRB, 0.10609E+01_JPRB, 0.10383E+01_JPRB, 0.10284E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.10193E+01_JPRB, 0.10029E+01_JPRB, 0.98187E+00_JPRB, 0.94978E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.97151E+00_JPRB, 0.93676E+00_JPRB, 0.91429E+00_JPRB, 0.89833E+00_JPRB, 0.89425E+00_JPRB, 0.86552E+00_JPRB, 0.83674E+00_JPRB ,&
&  0.87340E+00_JPRB, 0.79851E+00_JPRB, 0.76170E+00_JPRB, 0.73616E+00_JPRB, 0.74113E+00_JPRB, 0.76878E+00_JPRB, 0.80163E+00_JPRB ,&
&  0.83089E+00_JPRB, 0.87702E+00_JPRB, 0.10010E+01_JPRB, 0.12203E+01_JPRB, 0.13323E+01_JPRB, 0.12643E+01_JPRB, 0.13618E+01_JPRB ,&
&  0.17584E+01_JPRB, 0.20559E+01_JPRB, 0.21021E+01_JPRB, 0.22160E+01_JPRB, 0.24410E+01_JPRB, 0.25664E+01_JPRB, 0.22096E+01_JPRB ,&
&  0.19018E+01_JPRB, 0.15916E+01_JPRB, 0.13333E+01_JPRB, 0.11842E+01_JPRB, 0.11327E+01_JPRB, 0.10954E+01_JPRB, 0.10652E+01_JPRB ,&
&  0.10223E+01_JPRB, 0.10035E+01_JPRB, 0.10080E+01_JPRB, 0.10203E+01_JPRB, 0.10422E+01_JPRB, 0.10382E+01_JPRB, 0.10549E+01_JPRB ,&
&  0.10989E+01_JPRB, 0.11440E+01_JPRB, 0.12024E+01_JPRB, 0.11889E+01_JPRB, 0.12471E+01_JPRB, 0.13023E+01_JPRB, 0.13968E+01_JPRB ,&
&  0.14232E+01_JPRB, 0.15067E+01_JPRB, 0.15936E+01_JPRB, 0.16514E+01_JPRB, 0.17714E+01_JPRB, 0.18168E+01_JPRB, 0.17915E+01_JPRB ,&
&  0.18166E+01_JPRB, 0.17363E+01_JPRB, 0.15477E+01_JPRB, 0.15791E+01_JPRB, 0.17030E+01_JPRB, 0.19370E+01_JPRB, 0.16793E+01_JPRB ,&
&  0.14677E+01_JPRB, 0.13812E+01_JPRB, 0.15679E+01_JPRB, 0.19539E+01_JPRB, 0.21769E+01_JPRB, 0.25906E+01_JPRB, 0.32423E+01_JPRB ,&
&  0.35480E+01_JPRB, 0.20487E+01_JPRB, 0.16710E+01_JPRB, 0.21046E+01_JPRB, 0.17497E+01_JPRB, 0.16923E+01_JPRB, 0.24646E+01_JPRB ,&
&  0.29015E+01_JPRB, 0.31526E+01_JPRB, 0.29697E+01_JPRB, 0.23151E+01_JPRB, 0.22172E+01_JPRB, 0.23929E+01_JPRB, 0.24684E+01_JPRB ,&
&  0.22063E+01_JPRB, 0.20236E+01_JPRB, 0.18084E+01_JPRB, 0.15878E+01_JPRB, 0.13972E+01_JPRB, 0.12584E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11088E+01_JPRB, 0.10894E+01_JPRB, 0.10553E+01_JPRB, 0.10134E+01_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.10318E+01_JPRB, 0.98811E+00_JPRB, 0.95191E+00_JPRB, 0.91625E+00_JPRB, 0.89436E+00_JPRB, 0.86205E+00_JPRB, 0.81526E+00_JPRB ,&
&  0.83590E+00_JPRB, 0.79642E+00_JPRB, 0.75350E+00_JPRB, 0.72965E+00_JPRB, 0.72826E+00_JPRB, 0.74712E+00_JPRB, 0.78747E+00_JPRB ,&
&  0.84762E+00_JPRB, 0.88556E+00_JPRB, 0.99873E+00_JPRB, 0.11672E+01_JPRB, 0.12161E+01_JPRB, 0.11278E+01_JPRB, 0.13088E+01_JPRB ,&
&  0.14507E+01_JPRB, 0.17783E+01_JPRB, 0.22517E+01_JPRB, 0.24365E+01_JPRB, 0.24632E+01_JPRB, 0.25699E+01_JPRB, 0.23762E+01_JPRB ,&
&  0.21959E+01_JPRB, 0.19336E+01_JPRB, 0.15932E+01_JPRB, 0.13445E+01_JPRB, 0.11789E+01_JPRB, 0.11163E+01_JPRB, 0.10606E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10088E+01_JPRB, 0.97300E+00_JPRB, 0.97404E+00_JPRB, 0.98804E+00_JPRB, 0.99529E+00_JPRB, 0.99649E+00_JPRB ,&
&  0.10237E+01_JPRB, 0.10664E+01_JPRB, 0.11487E+01_JPRB, 0.12195E+01_JPRB, 0.10456E+01_JPRB, 0.12706E+01_JPRB, 0.13819E+01_JPRB ,&
&  0.15041E+01_JPRB, 0.16145E+01_JPRB, 0.15827E+01_JPRB, 0.17931E+01_JPRB, 0.18960E+01_JPRB, 0.18191E+01_JPRB, 0.18223E+01_JPRB ,&
&  0.19674E+01_JPRB, 0.18002E+01_JPRB, 0.16603E+01_JPRB, 0.18050E+01_JPRB, 0.18947E+01_JPRB, 0.22178E+01_JPRB, 0.13908E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.15267E+01_JPRB, 0.15263E+01_JPRB, 0.15209E+01_JPRB, 0.18368E+01_JPRB, 0.23356E+01_JPRB, 0.20484E+01_JPRB ,&
&  0.92142E+00_JPRB, 0.75816E+00_JPRB, 0.83941E+00_JPRB, 0.85140E+00_JPRB, 0.81479E+00_JPRB, 0.11608E+01_JPRB, 0.28979E+01_JPRB ,&
&  0.27530E+01_JPRB, 0.32426E+01_JPRB, 0.31918E+01_JPRB, 0.27525E+01_JPRB, 0.23755E+01_JPRB, 0.23584E+01_JPRB, 0.26561E+01_JPRB ,&
&  0.25521E+01_JPRB, 0.23363E+01_JPRB, 0.21268E+01_JPRB, 0.19637E+01_JPRB, 0.17612E+01_JPRB, 0.15554E+01_JPRB, 0.13799E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.11884E+01_JPRB, 0.11476E+01_JPRB, 0.11120E+01_JPRB, 0.10733E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.10540E+01_JPRB, 0.10073E+01_JPRB, 0.97352E+00_JPRB, 0.94445E+00_JPRB, 0.90979E+00_JPRB, 0.88086E+00_JPRB, 0.82682E+00_JPRB ,&
&  0.81092E+00_JPRB, 0.80985E+00_JPRB, 0.76854E+00_JPRB, 0.74001E+00_JPRB, 0.72680E+00_JPRB, 0.74644E+00_JPRB, 0.80144E+00_JPRB ,&
&  0.85272E+00_JPRB, 0.91835E+00_JPRB, 0.10929E+01_JPRB, 0.10597E+01_JPRB, 0.11197E+01_JPRB, 0.10173E+01_JPRB, 0.10979E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.14709E+01_JPRB, 0.19026E+01_JPRB, 0.21863E+01_JPRB, 0.25164E+01_JPRB, 0.26422E+01_JPRB, 0.25966E+01_JPRB ,&
&  0.24230E+01_JPRB, 0.22682E+01_JPRB, 0.19947E+01_JPRB, 0.16591E+01_JPRB, 0.14082E+01_JPRB, 0.12400E+01_JPRB, 0.11351E+01_JPRB ,&
&  0.10924E+01_JPRB, 0.10934E+01_JPRB, 0.10277E+01_JPRB, 0.97799E+00_JPRB, 0.96630E+00_JPRB, 0.95950E+00_JPRB, 0.94187E+00_JPRB ,&
&  0.96050E+00_JPRB, 0.10125E+01_JPRB, 0.10903E+01_JPRB, 0.11718E+01_JPRB, 0.10897E+01_JPRB, 0.10181E+01_JPRB, 0.12657E+01_JPRB ,&
&  0.15208E+01_JPRB, 0.18299E+01_JPRB, 0.18996E+01_JPRB, 0.22026E+01_JPRB, 0.20988E+01_JPRB, 0.19411E+01_JPRB, 0.20132E+01_JPRB ,&
&  0.20734E+01_JPRB, 0.22720E+01_JPRB, 0.18550E+01_JPRB, 0.19730E+01_JPRB, 0.22514E+01_JPRB, 0.17812E+01_JPRB, 0.15885E+01_JPRB ,&
&  0.18066E+01_JPRB, 0.16165E+01_JPRB, 0.14773E+01_JPRB, 0.12945E+01_JPRB, 0.16977E+01_JPRB, 0.17825E+01_JPRB, 0.92720E+00_JPRB ,&
&  0.75319E+00_JPRB, 0.72061E+00_JPRB, 0.69396E+00_JPRB, 0.66219E+00_JPRB, 0.69708E+00_JPRB, 0.97443E+00_JPRB, 0.16406E+01_JPRB ,&
&  0.20501E+01_JPRB, 0.27939E+01_JPRB, 0.31124E+01_JPRB, 0.30916E+01_JPRB, 0.25992E+01_JPRB, 0.26136E+01_JPRB, 0.25309E+01_JPRB ,&
&  0.27597E+01_JPRB, 0.27208E+01_JPRB, 0.24706E+01_JPRB, 0.22390E+01_JPRB, 0.20142E+01_JPRB, 0.18046E+01_JPRB, 0.16235E+01_JPRB ,&
&  0.14717E+01_JPRB, 0.13572E+01_JPRB, 0.12632E+01_JPRB, 0.11865E+01_JPRB, 0.11128E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.11396E+01_JPRB, 0.10749E+01_JPRB, 0.10070E+01_JPRB, 0.96057E+00_JPRB, 0.93148E+00_JPRB, 0.89379E+00_JPRB, 0.84572E+00_JPRB ,&
&  0.79686E+00_JPRB, 0.81268E+00_JPRB, 0.78570E+00_JPRB, 0.76319E+00_JPRB, 0.74630E+00_JPRB, 0.75787E+00_JPRB, 0.80677E+00_JPRB ,&
&  0.86304E+00_JPRB, 0.96388E+00_JPRB, 0.10320E+01_JPRB, 0.93868E+00_JPRB, 0.90332E+00_JPRB, 0.86248E+00_JPRB, 0.86664E+00_JPRB ,&
&  0.10244E+01_JPRB, 0.12105E+01_JPRB, 0.15008E+01_JPRB, 0.19506E+01_JPRB, 0.24285E+01_JPRB, 0.25743E+01_JPRB, 0.28345E+01_JPRB ,&
&  0.26873E+01_JPRB, 0.24463E+01_JPRB, 0.22780E+01_JPRB, 0.20331E+01_JPRB, 0.17087E+01_JPRB, 0.14816E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11633E+01_JPRB, 0.11357E+01_JPRB, 0.10465E+01_JPRB, 0.97357E+00_JPRB, 0.95003E+00_JPRB, 0.92600E+00_JPRB ,&
&  0.93491E+00_JPRB, 0.99322E+00_JPRB, 0.10683E+01_JPRB, 0.11732E+01_JPRB, 0.13188E+01_JPRB, 0.13835E+01_JPRB, 0.13684E+01_JPRB ,&
&  0.14373E+01_JPRB, 0.16200E+01_JPRB, 0.21784E+01_JPRB, 0.22840E+01_JPRB, 0.22748E+01_JPRB, 0.22658E+01_JPRB, 0.22024E+01_JPRB ,&
&  0.22408E+01_JPRB, 0.24873E+01_JPRB, 0.21872E+01_JPRB, 0.23964E+01_JPRB, 0.22399E+01_JPRB, 0.17335E+01_JPRB, 0.19783E+01_JPRB ,&
&  0.20920E+01_JPRB, 0.19052E+01_JPRB, 0.15728E+01_JPRB, 0.12144E+01_JPRB, 0.12128E+01_JPRB, 0.10095E+01_JPRB, 0.70966E+00_JPRB ,&
&  0.66154E+00_JPRB, 0.63834E+00_JPRB, 0.61680E+00_JPRB, 0.63635E+00_JPRB, 0.65372E+00_JPRB, 0.87503E+00_JPRB, 0.12744E+01_JPRB ,&
&  0.18619E+01_JPRB, 0.23423E+01_JPRB, 0.27758E+01_JPRB, 0.28852E+01_JPRB, 0.26408E+01_JPRB, 0.27472E+01_JPRB, 0.30870E+01_JPRB ,&
&  0.28101E+01_JPRB, 0.27095E+01_JPRB, 0.26506E+01_JPRB, 0.24254E+01_JPRB, 0.21879E+01_JPRB, 0.19771E+01_JPRB, 0.17755E+01_JPRB ,&
&  0.16041E+01_JPRB, 0.14835E+01_JPRB, 0.13881E+01_JPRB, 0.12965E+01_JPRB, 0.12124E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.11871E+01_JPRB, 0.11205E+01_JPRB, 0.10600E+01_JPRB, 0.10189E+01_JPRB, 0.98162E+00_JPRB, 0.93784E+00_JPRB, 0.88475E+00_JPRB ,&
&  0.81705E+00_JPRB, 0.80226E+00_JPRB, 0.79521E+00_JPRB, 0.76181E+00_JPRB, 0.75057E+00_JPRB, 0.76208E+00_JPRB, 0.80137E+00_JPRB ,&
&  0.86371E+00_JPRB, 0.10446E+01_JPRB, 0.89319E+00_JPRB, 0.81905E+00_JPRB, 0.84484E+00_JPRB, 0.84347E+00_JPRB, 0.77901E+00_JPRB ,&
&  0.91347E+00_JPRB, 0.10623E+01_JPRB, 0.12998E+01_JPRB, 0.18236E+01_JPRB, 0.26672E+01_JPRB, 0.27958E+01_JPRB, 0.29665E+01_JPRB ,&
&  0.30905E+01_JPRB, 0.28272E+01_JPRB, 0.25956E+01_JPRB, 0.23451E+01_JPRB, 0.19764E+01_JPRB, 0.16896E+01_JPRB, 0.14849E+01_JPRB ,&
&  0.13256E+01_JPRB, 0.12349E+01_JPRB, 0.11628E+01_JPRB, 0.11135E+01_JPRB, 0.10308E+01_JPRB, 0.98023E+00_JPRB, 0.96359E+00_JPRB ,&
&  0.96035E+00_JPRB, 0.10019E+01_JPRB, 0.10728E+01_JPRB, 0.12086E+01_JPRB, 0.13992E+01_JPRB, 0.14291E+01_JPRB, 0.18394E+01_JPRB ,&
&  0.19661E+01_JPRB, 0.21588E+01_JPRB, 0.22709E+01_JPRB, 0.26074E+01_JPRB, 0.27925E+01_JPRB, 0.26750E+01_JPRB, 0.28520E+01_JPRB ,&
&  0.23466E+01_JPRB, 0.22943E+01_JPRB, 0.25174E+01_JPRB, 0.26661E+01_JPRB, 0.21229E+01_JPRB, 0.21903E+01_JPRB, 0.31766E+01_JPRB ,&
&  0.22684E+01_JPRB, 0.23969E+01_JPRB, 0.25624E+01_JPRB, 0.25644E+01_JPRB, 0.11009E+01_JPRB, 0.81818E+00_JPRB, 0.12377E+01_JPRB ,&
&  0.12977E+01_JPRB, 0.87228E+00_JPRB, 0.71232E+00_JPRB, 0.85637E+00_JPRB, 0.73845E+00_JPRB, 0.93017E+00_JPRB, 0.13951E+01_JPRB ,&
&  0.15193E+01_JPRB, 0.16193E+01_JPRB, 0.23210E+01_JPRB, 0.26054E+01_JPRB, 0.25944E+01_JPRB, 0.28104E+01_JPRB, 0.29702E+01_JPRB ,&
&  0.30251E+01_JPRB, 0.28463E+01_JPRB, 0.29067E+01_JPRB, 0.26805E+01_JPRB, 0.23906E+01_JPRB, 0.21122E+01_JPRB, 0.18803E+01_JPRB ,&
&  0.16692E+01_JPRB, 0.15201E+01_JPRB, 0.14195E+01_JPRB, 0.13223E+01_JPRB, 0.12462E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.11472E+01_JPRB, 0.10883E+01_JPRB, 0.10379E+01_JPRB, 0.99452E+00_JPRB, 0.96608E+00_JPRB, 0.93761E+00_JPRB, 0.90189E+00_JPRB ,&
&  0.84749E+00_JPRB, 0.80079E+00_JPRB, 0.80088E+00_JPRB, 0.76792E+00_JPRB, 0.74752E+00_JPRB, 0.75191E+00_JPRB, 0.79382E+00_JPRB ,&
&  0.84294E+00_JPRB, 0.89067E+00_JPRB, 0.76342E+00_JPRB, 0.70893E+00_JPRB, 0.81607E+00_JPRB, 0.76963E+00_JPRB, 0.77195E+00_JPRB ,&
&  0.87716E+00_JPRB, 0.98828E+00_JPRB, 0.11926E+01_JPRB, 0.16560E+01_JPRB, 0.21103E+01_JPRB, 0.25934E+01_JPRB, 0.29461E+01_JPRB ,&
&  0.31127E+01_JPRB, 0.30436E+01_JPRB, 0.28657E+01_JPRB, 0.26336E+01_JPRB, 0.22333E+01_JPRB, 0.18988E+01_JPRB, 0.16585E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.13025E+01_JPRB, 0.11871E+01_JPRB, 0.10794E+01_JPRB, 0.10155E+01_JPRB, 0.95613E+00_JPRB, 0.94561E+00_JPRB ,&
&  0.96322E+00_JPRB, 0.10071E+01_JPRB, 0.10812E+01_JPRB, 0.12333E+01_JPRB, 0.12715E+01_JPRB, 0.13646E+01_JPRB, 0.15876E+01_JPRB ,&
&  0.20389E+01_JPRB, 0.22301E+01_JPRB, 0.24897E+01_JPRB, 0.27576E+01_JPRB, 0.31596E+01_JPRB, 0.30222E+01_JPRB, 0.33794E+01_JPRB ,&
&  0.25607E+01_JPRB, 0.21588E+01_JPRB, 0.19382E+01_JPRB, 0.16399E+01_JPRB, 0.20679E+01_JPRB, 0.31646E+01_JPRB, 0.30908E+01_JPRB ,&
&  0.27639E+01_JPRB, 0.26864E+01_JPRB, 0.26566E+01_JPRB, 0.21066E+01_JPRB, 0.16254E+01_JPRB, 0.12679E+01_JPRB, 0.15702E+01_JPRB ,&
&  0.16456E+01_JPRB, 0.16194E+01_JPRB, 0.14357E+01_JPRB, 0.12306E+01_JPRB, 0.10880E+01_JPRB, 0.13457E+01_JPRB, 0.13723E+01_JPRB ,&
&  0.12928E+01_JPRB, 0.12689E+01_JPRB, 0.16180E+01_JPRB, 0.20637E+01_JPRB, 0.23123E+01_JPRB, 0.22821E+01_JPRB, 0.25845E+01_JPRB ,&
&  0.26280E+01_JPRB, 0.27135E+01_JPRB, 0.26508E+01_JPRB, 0.24862E+01_JPRB, 0.22678E+01_JPRB, 0.20229E+01_JPRB, 0.18223E+01_JPRB ,&
&  0.16598E+01_JPRB, 0.15154E+01_JPRB, 0.13873E+01_JPRB, 0.12845E+01_JPRB, 0.12103E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.10898E+01_JPRB, 0.10431E+01_JPRB, 0.10009E+01_JPRB, 0.97036E+00_JPRB, 0.94228E+00_JPRB, 0.91989E+00_JPRB, 0.89508E+00_JPRB ,&
&  0.85860E+00_JPRB, 0.80632E+00_JPRB, 0.79875E+00_JPRB, 0.78308E+00_JPRB, 0.76127E+00_JPRB, 0.75226E+00_JPRB, 0.77127E+00_JPRB ,&
&  0.81382E+00_JPRB, 0.78423E+00_JPRB, 0.65341E+00_JPRB, 0.67266E+00_JPRB, 0.72480E+00_JPRB, 0.69322E+00_JPRB, 0.74633E+00_JPRB ,&
&  0.82873E+00_JPRB, 0.93310E+00_JPRB, 0.10465E+01_JPRB, 0.13061E+01_JPRB, 0.16160E+01_JPRB, 0.20173E+01_JPRB, 0.24091E+01_JPRB ,&
&  0.25176E+01_JPRB, 0.27259E+01_JPRB, 0.28669E+01_JPRB, 0.26987E+01_JPRB, 0.24328E+01_JPRB, 0.21375E+01_JPRB, 0.18893E+01_JPRB ,&
&  0.17065E+01_JPRB, 0.15246E+01_JPRB, 0.13369E+01_JPRB, 0.11903E+01_JPRB, 0.10918E+01_JPRB, 0.10351E+01_JPRB, 0.10136E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10451E+01_JPRB, 0.10785E+01_JPRB, 0.12289E+01_JPRB, 0.12935E+01_JPRB, 0.14573E+01_JPRB, 0.16239E+01_JPRB ,&
&  0.24058E+01_JPRB, 0.27062E+01_JPRB, 0.28423E+01_JPRB, 0.32359E+01_JPRB, 0.25124E+01_JPRB, 0.26571E+01_JPRB, 0.34026E+01_JPRB ,&
&  0.34649E+01_JPRB, 0.33332E+01_JPRB, 0.32836E+01_JPRB, 0.25476E+01_JPRB, 0.22541E+01_JPRB, 0.30999E+01_JPRB, 0.29329E+01_JPRB ,&
&  0.26242E+01_JPRB, 0.25431E+01_JPRB, 0.24058E+01_JPRB, 0.26236E+01_JPRB, 0.18454E+01_JPRB, 0.15854E+01_JPRB, 0.12478E+01_JPRB ,&
&  0.12435E+01_JPRB, 0.11571E+01_JPRB, 0.13508E+01_JPRB, 0.12800E+01_JPRB, 0.12808E+01_JPRB, 0.12831E+01_JPRB, 0.12404E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.12630E+01_JPRB, 0.12703E+01_JPRB, 0.15047E+01_JPRB, 0.19196E+01_JPRB, 0.18629E+01_JPRB, 0.20428E+01_JPRB ,&
&  0.20775E+01_JPRB, 0.21803E+01_JPRB, 0.20818E+01_JPRB, 0.20856E+01_JPRB, 0.20136E+01_JPRB, 0.18875E+01_JPRB, 0.17168E+01_JPRB ,&
&  0.15818E+01_JPRB, 0.14632E+01_JPRB, 0.13427E+01_JPRB, 0.12389E+01_JPRB, 0.11546E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.98925E+00_JPRB, 0.94888E+00_JPRB, 0.92134E+00_JPRB, 0.90652E+00_JPRB, 0.89830E+00_JPRB, 0.88730E+00_JPRB, 0.87017E+00_JPRB ,&
&  0.84301E+00_JPRB, 0.79709E+00_JPRB, 0.77508E+00_JPRB, 0.76721E+00_JPRB, 0.74463E+00_JPRB, 0.73229E+00_JPRB, 0.74598E+00_JPRB ,&
&  0.78325E+00_JPRB, 0.81630E+00_JPRB, 0.69461E+00_JPRB, 0.62042E+00_JPRB, 0.59237E+00_JPRB, 0.65412E+00_JPRB, 0.73337E+00_JPRB ,&
&  0.84432E+00_JPRB, 0.93677E+00_JPRB, 0.10574E+01_JPRB, 0.12001E+01_JPRB, 0.13890E+01_JPRB, 0.16198E+01_JPRB, 0.19928E+01_JPRB ,&
&  0.25792E+01_JPRB, 0.23836E+01_JPRB, 0.25577E+01_JPRB, 0.25184E+01_JPRB, 0.24568E+01_JPRB, 0.22246E+01_JPRB, 0.19871E+01_JPRB ,&
&  0.17625E+01_JPRB, 0.15976E+01_JPRB, 0.14285E+01_JPRB, 0.12871E+01_JPRB, 0.11726E+01_JPRB, 0.10844E+01_JPRB, 0.10318E+01_JPRB ,&
&  0.10376E+01_JPRB, 0.10605E+01_JPRB, 0.10771E+01_JPRB, 0.11391E+01_JPRB, 0.13488E+01_JPRB, 0.15738E+01_JPRB, 0.18642E+01_JPRB ,&
&  0.20181E+01_JPRB, 0.21080E+01_JPRB, 0.30118E+01_JPRB, 0.30572E+01_JPRB, 0.31944E+01_JPRB, 0.33240E+01_JPRB, 0.34194E+01_JPRB ,&
&  0.35895E+01_JPRB, 0.34152E+01_JPRB, 0.32935E+01_JPRB, 0.28585E+01_JPRB, 0.32664E+01_JPRB, 0.31736E+01_JPRB, 0.29650E+01_JPRB ,&
&  0.27026E+01_JPRB, 0.26464E+01_JPRB, 0.24653E+01_JPRB, 0.24242E+01_JPRB, 0.24011E+01_JPRB, 0.24426E+01_JPRB, 0.19480E+01_JPRB ,&
&  0.15754E+01_JPRB, 0.17597E+01_JPRB, 0.15789E+01_JPRB, 0.13711E+01_JPRB, 0.11933E+01_JPRB, 0.12647E+01_JPRB, 0.13475E+01_JPRB ,&
&  0.14066E+01_JPRB, 0.14353E+01_JPRB, 0.14084E+01_JPRB, 0.15003E+01_JPRB, 0.17457E+01_JPRB, 0.19139E+01_JPRB, 0.19041E+01_JPRB ,&
&  0.17485E+01_JPRB, 0.17921E+01_JPRB, 0.18249E+01_JPRB, 0.17915E+01_JPRB, 0.17627E+01_JPRB, 0.16718E+01_JPRB, 0.15570E+01_JPRB ,&
&  0.14367E+01_JPRB, 0.13282E+01_JPRB, 0.12332E+01_JPRB, 0.11387E+01_JPRB, 0.10491E+01_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.89362E+00_JPRB, 0.86859E+00_JPRB, 0.84945E+00_JPRB, 0.84099E+00_JPRB, 0.83778E+00_JPRB, 0.84165E+00_JPRB, 0.83642E+00_JPRB ,&
&  0.82131E+00_JPRB, 0.78661E+00_JPRB, 0.75520E+00_JPRB, 0.75228E+00_JPRB, 0.73782E+00_JPRB, 0.72329E+00_JPRB, 0.72956E+00_JPRB ,&
&  0.76316E+00_JPRB, 0.90198E+00_JPRB, 0.78426E+00_JPRB, 0.73307E+00_JPRB, 0.68382E+00_JPRB, 0.73595E+00_JPRB, 0.79480E+00_JPRB ,&
&  0.92288E+00_JPRB, 0.99259E+00_JPRB, 0.10362E+01_JPRB, 0.11166E+01_JPRB, 0.12913E+01_JPRB, 0.15041E+01_JPRB, 0.17555E+01_JPRB ,&
&  0.18864E+01_JPRB, 0.21118E+01_JPRB, 0.21428E+01_JPRB, 0.24243E+01_JPRB, 0.23173E+01_JPRB, 0.20137E+01_JPRB, 0.19768E+01_JPRB ,&
&  0.17651E+01_JPRB, 0.15629E+01_JPRB, 0.13802E+01_JPRB, 0.12618E+01_JPRB, 0.11735E+01_JPRB, 0.11007E+01_JPRB, 0.10396E+01_JPRB ,&
&  0.10050E+01_JPRB, 0.99402E+00_JPRB, 0.98445E+00_JPRB, 0.97976E+00_JPRB, 0.10726E+01_JPRB, 0.13866E+01_JPRB, 0.17776E+01_JPRB ,&
&  0.25115E+01_JPRB, 0.23017E+01_JPRB, 0.19926E+01_JPRB, 0.26846E+01_JPRB, 0.36176E+01_JPRB, 0.36274E+01_JPRB, 0.34618E+01_JPRB ,&
&  0.37907E+01_JPRB, 0.38324E+01_JPRB, 0.37575E+01_JPRB, 0.37376E+01_JPRB, 0.36397E+01_JPRB, 0.34918E+01_JPRB, 0.32876E+01_JPRB ,&
&  0.29264E+01_JPRB, 0.27562E+01_JPRB, 0.26560E+01_JPRB, 0.26106E+01_JPRB, 0.24265E+01_JPRB, 0.23236E+01_JPRB, 0.22974E+01_JPRB ,&
&  0.20568E+01_JPRB, 0.18305E+01_JPRB, 0.13159E+01_JPRB, 0.13859E+01_JPRB, 0.11214E+01_JPRB, 0.11626E+01_JPRB, 0.13988E+01_JPRB ,&
&  0.14180E+01_JPRB, 0.15526E+01_JPRB, 0.16040E+01_JPRB, 0.15377E+01_JPRB, 0.17726E+01_JPRB, 0.18159E+01_JPRB, 0.17840E+01_JPRB ,&
&  0.17237E+01_JPRB, 0.15126E+01_JPRB, 0.15638E+01_JPRB, 0.15654E+01_JPRB, 0.14952E+01_JPRB, 0.14003E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.12112E+01_JPRB, 0.11315E+01_JPRB, 0.10718E+01_JPRB, 0.99876E+00_JPRB, 0.93498E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.81048E+00_JPRB, 0.78744E+00_JPRB, 0.76794E+00_JPRB, 0.76429E+00_JPRB, 0.76886E+00_JPRB, 0.78002E+00_JPRB, 0.79109E+00_JPRB ,&
&  0.79199E+00_JPRB, 0.77666E+00_JPRB, 0.74896E+00_JPRB, 0.73663E+00_JPRB, 0.72545E+00_JPRB, 0.71224E+00_JPRB, 0.71860E+00_JPRB ,&
&  0.76778E+00_JPRB, 0.91427E+00_JPRB, 0.85189E+00_JPRB, 0.76122E+00_JPRB, 0.79727E+00_JPRB, 0.81226E+00_JPRB, 0.88046E+00_JPRB ,&
&  0.91760E+00_JPRB, 0.10103E+01_JPRB, 0.10511E+01_JPRB, 0.10891E+01_JPRB, 0.11786E+01_JPRB, 0.13558E+01_JPRB, 0.15255E+01_JPRB ,&
&  0.15921E+01_JPRB, 0.16667E+01_JPRB, 0.18051E+01_JPRB, 0.19477E+01_JPRB, 0.19622E+01_JPRB, 0.17679E+01_JPRB, 0.17525E+01_JPRB ,&
&  0.16131E+01_JPRB, 0.14383E+01_JPRB, 0.12580E+01_JPRB, 0.11259E+01_JPRB, 0.10420E+01_JPRB, 0.99109E+00_JPRB, 0.96440E+00_JPRB ,&
&  0.95627E+00_JPRB, 0.95122E+00_JPRB, 0.92958E+00_JPRB, 0.94239E+00_JPRB, 0.10123E+01_JPRB, 0.13726E+01_JPRB, 0.20161E+01_JPRB ,&
&  0.28459E+01_JPRB, 0.29933E+01_JPRB, 0.31966E+01_JPRB, 0.37952E+01_JPRB, 0.40826E+01_JPRB, 0.42118E+01_JPRB, 0.42750E+01_JPRB ,&
&  0.41867E+01_JPRB, 0.40362E+01_JPRB, 0.38191E+01_JPRB, 0.35357E+01_JPRB, 0.34690E+01_JPRB, 0.34752E+01_JPRB, 0.33571E+01_JPRB ,&
&  0.30755E+01_JPRB, 0.29083E+01_JPRB, 0.28972E+01_JPRB, 0.27413E+01_JPRB, 0.26362E+01_JPRB, 0.24071E+01_JPRB, 0.24464E+01_JPRB ,&
&  0.24221E+01_JPRB, 0.14945E+01_JPRB, 0.12366E+01_JPRB, 0.14935E+01_JPRB, 0.12741E+01_JPRB, 0.13529E+01_JPRB, 0.16318E+01_JPRB ,&
&  0.16646E+01_JPRB, 0.16443E+01_JPRB, 0.17171E+01_JPRB, 0.16300E+01_JPRB, 0.16463E+01_JPRB, 0.17420E+01_JPRB, 0.16421E+01_JPRB ,&
&  0.15425E+01_JPRB, 0.14575E+01_JPRB, 0.14812E+01_JPRB, 0.14041E+01_JPRB, 0.12943E+01_JPRB, 0.11935E+01_JPRB, 0.11130E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.99621E+00_JPRB, 0.95914E+00_JPRB, 0.89356E+00_JPRB, 0.83903E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.76054E+00_JPRB, 0.73623E+00_JPRB, 0.72061E+00_JPRB, 0.71870E+00_JPRB, 0.72326E+00_JPRB, 0.73791E+00_JPRB, 0.75596E+00_JPRB ,&
&  0.76906E+00_JPRB, 0.76650E+00_JPRB, 0.74295E+00_JPRB, 0.72303E+00_JPRB, 0.70960E+00_JPRB, 0.69918E+00_JPRB, 0.71275E+00_JPRB ,&
&  0.66071E+00_JPRB, 0.74301E+00_JPRB, 0.68926E+00_JPRB, 0.72430E+00_JPRB, 0.82437E+00_JPRB, 0.84839E+00_JPRB, 0.87633E+00_JPRB ,&
&  0.89262E+00_JPRB, 0.93303E+00_JPRB, 0.94909E+00_JPRB, 0.97883E+00_JPRB, 0.10355E+01_JPRB, 0.11668E+01_JPRB, 0.13268E+01_JPRB ,&
&  0.13069E+01_JPRB, 0.12643E+01_JPRB, 0.13358E+01_JPRB, 0.14261E+01_JPRB, 0.14177E+01_JPRB, 0.14751E+01_JPRB, 0.15061E+01_JPRB ,&
&  0.14243E+01_JPRB, 0.13113E+01_JPRB, 0.11911E+01_JPRB, 0.10785E+01_JPRB, 0.99051E+00_JPRB, 0.92013E+00_JPRB, 0.87990E+00_JPRB ,&
&  0.85951E+00_JPRB, 0.85554E+00_JPRB, 0.86303E+00_JPRB, 0.92752E+00_JPRB, 0.99666E+00_JPRB, 0.14282E+01_JPRB, 0.21899E+01_JPRB ,&
&  0.28800E+01_JPRB, 0.30156E+01_JPRB, 0.33632E+01_JPRB, 0.41092E+01_JPRB, 0.41725E+01_JPRB, 0.41044E+01_JPRB, 0.39724E+01_JPRB ,&
&  0.38667E+01_JPRB, 0.36558E+01_JPRB, 0.35552E+01_JPRB, 0.35844E+01_JPRB, 0.35020E+01_JPRB, 0.36145E+01_JPRB, 0.34418E+01_JPRB ,&
&  0.31264E+01_JPRB, 0.30277E+01_JPRB, 0.30383E+01_JPRB, 0.29853E+01_JPRB, 0.29016E+01_JPRB, 0.29704E+01_JPRB, 0.29447E+01_JPRB ,&
&  0.27873E+01_JPRB, 0.24455E+01_JPRB, 0.18976E+01_JPRB, 0.18211E+01_JPRB, 0.14363E+01_JPRB, 0.15295E+01_JPRB, 0.19168E+01_JPRB ,&
&  0.19148E+01_JPRB, 0.17612E+01_JPRB, 0.16187E+01_JPRB, 0.15512E+01_JPRB, 0.15382E+01_JPRB, 0.16515E+01_JPRB, 0.15274E+01_JPRB ,&
&  0.13753E+01_JPRB, 0.14035E+01_JPRB, 0.13798E+01_JPRB, 0.13063E+01_JPRB, 0.12060E+01_JPRB, 0.11024E+01_JPRB, 0.10287E+01_JPRB ,&
&  0.99140E+00_JPRB, 0.95223E+00_JPRB, 0.89501E+00_JPRB, 0.82793E+00_JPRB, 0.78535E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.74387E+00_JPRB, 0.71539E+00_JPRB, 0.69412E+00_JPRB, 0.69022E+00_JPRB, 0.69516E+00_JPRB, 0.70570E+00_JPRB, 0.72500E+00_JPRB ,&
&  0.74386E+00_JPRB, 0.75044E+00_JPRB, 0.73571E+00_JPRB, 0.71165E+00_JPRB, 0.69180E+00_JPRB, 0.67932E+00_JPRB, 0.64791E+00_JPRB ,&
&  0.60788E+00_JPRB, 0.66912E+00_JPRB, 0.66431E+00_JPRB, 0.68531E+00_JPRB, 0.74109E+00_JPRB, 0.76064E+00_JPRB, 0.78441E+00_JPRB ,&
&  0.82651E+00_JPRB, 0.84037E+00_JPRB, 0.86308E+00_JPRB, 0.89270E+00_JPRB, 0.93533E+00_JPRB, 0.10294E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.10815E+01_JPRB, 0.10903E+01_JPRB, 0.11393E+01_JPRB, 0.12012E+01_JPRB, 0.12346E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.10920E+01_JPRB, 0.10380E+01_JPRB, 0.99137E+00_JPRB, 0.93412E+00_JPRB, 0.87532E+00_JPRB, 0.83477E+00_JPRB ,&
&  0.80996E+00_JPRB, 0.80655E+00_JPRB, 0.82825E+00_JPRB, 0.85547E+00_JPRB, 0.92823E+00_JPRB, 0.12599E+01_JPRB, 0.17789E+01_JPRB ,&
&  0.21101E+01_JPRB, 0.23971E+01_JPRB, 0.25643E+01_JPRB, 0.27323E+01_JPRB, 0.28906E+01_JPRB, 0.30362E+01_JPRB, 0.31776E+01_JPRB ,&
&  0.33301E+01_JPRB, 0.32170E+01_JPRB, 0.35196E+01_JPRB, 0.36036E+01_JPRB, 0.36219E+01_JPRB, 0.36813E+01_JPRB, 0.34261E+01_JPRB ,&
&  0.31953E+01_JPRB, 0.29717E+01_JPRB, 0.30689E+01_JPRB, 0.30697E+01_JPRB, 0.30146E+01_JPRB, 0.30515E+01_JPRB, 0.29861E+01_JPRB ,&
&  0.29314E+01_JPRB, 0.28447E+01_JPRB, 0.25125E+01_JPRB, 0.22794E+01_JPRB, 0.19577E+01_JPRB, 0.20837E+01_JPRB, 0.19078E+01_JPRB ,&
&  0.16756E+01_JPRB, 0.13675E+01_JPRB, 0.13711E+01_JPRB, 0.13647E+01_JPRB, 0.14166E+01_JPRB, 0.14332E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.13615E+01_JPRB, 0.12766E+01_JPRB, 0.11981E+01_JPRB, 0.11155E+01_JPRB, 0.10141E+01_JPRB, 0.93281E+00_JPRB ,&
&  0.90102E+00_JPRB, 0.88767E+00_JPRB, 0.84099E+00_JPRB, 0.79603E+00_JPRB, 0.76789E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.75294E+00_JPRB, 0.73339E+00_JPRB, 0.70530E+00_JPRB, 0.69006E+00_JPRB, 0.69002E+00_JPRB, 0.69315E+00_JPRB, 0.69203E+00_JPRB ,&
&  0.70999E+00_JPRB, 0.71303E+00_JPRB, 0.70696E+00_JPRB, 0.69152E+00_JPRB, 0.66734E+00_JPRB, 0.60825E+00_JPRB, 0.53606E+00_JPRB ,&
&  0.53126E+00_JPRB, 0.56633E+00_JPRB, 0.60776E+00_JPRB, 0.63036E+00_JPRB, 0.65559E+00_JPRB, 0.68008E+00_JPRB, 0.69996E+00_JPRB ,&
&  0.71454E+00_JPRB, 0.72770E+00_JPRB, 0.76156E+00_JPRB, 0.81175E+00_JPRB, 0.86455E+00_JPRB, 0.91813E+00_JPRB, 0.97520E+00_JPRB ,&
&  0.94171E+00_JPRB, 0.93666E+00_JPRB, 0.96215E+00_JPRB, 0.92869E+00_JPRB, 0.10330E+01_JPRB, 0.10434E+01_JPRB, 0.10305E+01_JPRB ,&
&  0.10074E+01_JPRB, 0.96381E+00_JPRB, 0.90962E+00_JPRB, 0.86662E+00_JPRB, 0.83127E+00_JPRB, 0.80560E+00_JPRB, 0.77520E+00_JPRB ,&
&  0.75728E+00_JPRB, 0.75429E+00_JPRB, 0.77060E+00_JPRB, 0.79462E+00_JPRB, 0.84799E+00_JPRB, 0.10400E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.14478E+01_JPRB, 0.16231E+01_JPRB, 0.17350E+01_JPRB, 0.17734E+01_JPRB, 0.20960E+01_JPRB, 0.24801E+01_JPRB, 0.27053E+01_JPRB ,&
&  0.30292E+01_JPRB, 0.30204E+01_JPRB, 0.31718E+01_JPRB, 0.33020E+01_JPRB, 0.34341E+01_JPRB, 0.33862E+01_JPRB, 0.31655E+01_JPRB ,&
&  0.30208E+01_JPRB, 0.30092E+01_JPRB, 0.29859E+01_JPRB, 0.29514E+01_JPRB, 0.28349E+01_JPRB, 0.27678E+01_JPRB, 0.26805E+01_JPRB ,&
&  0.26700E+01_JPRB, 0.26615E+01_JPRB, 0.25789E+01_JPRB, 0.24690E+01_JPRB, 0.23388E+01_JPRB, 0.21455E+01_JPRB, 0.19267E+01_JPRB ,&
&  0.17155E+01_JPRB, 0.14516E+01_JPRB, 0.13357E+01_JPRB, 0.13275E+01_JPRB, 0.13548E+01_JPRB, 0.12834E+01_JPRB, 0.12558E+01_JPRB ,&
&  0.12311E+01_JPRB, 0.11775E+01_JPRB, 0.12163E+01_JPRB, 0.11538E+01_JPRB, 0.10748E+01_JPRB, 0.98620E+00_JPRB, 0.92768E+00_JPRB ,&
&  0.85253E+00_JPRB, 0.86678E+00_JPRB, 0.84677E+00_JPRB, 0.81081E+00_JPRB, 0.77596E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.76539E+00_JPRB, 0.74339E+00_JPRB, 0.71507E+00_JPRB, 0.68895E+00_JPRB, 0.68186E+00_JPRB, 0.66889E+00_JPRB, 0.65318E+00_JPRB ,&
&  0.67867E+00_JPRB, 0.65275E+00_JPRB, 0.66115E+00_JPRB, 0.58910E+00_JPRB, 0.55762E+00_JPRB, 0.54468E+00_JPRB, 0.52927E+00_JPRB ,&
&  0.53436E+00_JPRB, 0.55442E+00_JPRB, 0.57302E+00_JPRB, 0.58842E+00_JPRB, 0.61499E+00_JPRB, 0.63355E+00_JPRB, 0.64798E+00_JPRB ,&
&  0.66782E+00_JPRB, 0.69651E+00_JPRB, 0.74229E+00_JPRB, 0.78362E+00_JPRB, 0.83667E+00_JPRB, 0.90279E+00_JPRB, 0.95617E+00_JPRB ,&
&  0.93659E+00_JPRB, 0.93567E+00_JPRB, 0.98021E+00_JPRB, 0.99096E+00_JPRB, 0.10092E+01_JPRB, 0.99455E+00_JPRB, 0.97458E+00_JPRB ,&
&  0.93392E+00_JPRB, 0.81536E+00_JPRB, 0.82810E+00_JPRB, 0.79248E+00_JPRB, 0.76377E+00_JPRB, 0.73812E+00_JPRB, 0.71816E+00_JPRB ,&
&  0.70638E+00_JPRB, 0.70512E+00_JPRB, 0.71353E+00_JPRB, 0.72346E+00_JPRB, 0.74075E+00_JPRB, 0.81327E+00_JPRB, 0.92226E+00_JPRB ,&
&  0.11159E+01_JPRB, 0.10759E+01_JPRB, 0.13428E+01_JPRB, 0.14710E+01_JPRB, 0.18358E+01_JPRB, 0.22346E+01_JPRB, 0.25190E+01_JPRB ,&
&  0.29747E+01_JPRB, 0.30355E+01_JPRB, 0.30136E+01_JPRB, 0.29801E+01_JPRB, 0.30081E+01_JPRB, 0.29711E+01_JPRB, 0.28900E+01_JPRB ,&
&  0.28058E+01_JPRB, 0.26154E+01_JPRB, 0.27392E+01_JPRB, 0.26950E+01_JPRB, 0.25987E+01_JPRB, 0.25341E+01_JPRB, 0.24508E+01_JPRB ,&
&  0.23797E+01_JPRB, 0.22944E+01_JPRB, 0.22046E+01_JPRB, 0.19711E+01_JPRB, 0.19140E+01_JPRB, 0.18560E+01_JPRB, 0.16921E+01_JPRB ,&
&  0.16237E+01_JPRB, 0.15315E+01_JPRB, 0.14237E+01_JPRB, 0.14598E+01_JPRB, 0.13872E+01_JPRB, 0.13462E+01_JPRB, 0.13211E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.11073E+01_JPRB, 0.10849E+01_JPRB, 0.10711E+01_JPRB, 0.10261E+01_JPRB, 0.97377E+00_JPRB, 0.99783E+00_JPRB ,&
&  0.93529E+00_JPRB, 0.90867E+00_JPRB, 0.91046E+00_JPRB, 0.85914E+00_JPRB, 0.81022E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.79754E+00_JPRB, 0.76622E+00_JPRB, 0.73371E+00_JPRB, 0.69824E+00_JPRB, 0.67552E+00_JPRB, 0.65323E+00_JPRB, 0.62154E+00_JPRB ,&
&  0.61676E+00_JPRB, 0.57585E+00_JPRB, 0.54668E+00_JPRB, 0.53553E+00_JPRB, 0.53634E+00_JPRB, 0.53252E+00_JPRB, 0.52425E+00_JPRB ,&
&  0.52762E+00_JPRB, 0.54354E+00_JPRB, 0.56745E+00_JPRB, 0.58726E+00_JPRB, 0.60611E+00_JPRB, 0.62356E+00_JPRB, 0.64131E+00_JPRB ,&
&  0.66702E+00_JPRB, 0.69965E+00_JPRB, 0.73947E+00_JPRB, 0.79168E+00_JPRB, 0.84168E+00_JPRB, 0.90423E+00_JPRB, 0.94853E+00_JPRB ,&
&  0.98960E+00_JPRB, 0.10143E+01_JPRB, 0.10048E+01_JPRB, 0.10197E+01_JPRB, 0.10202E+01_JPRB, 0.99274E+00_JPRB, 0.92746E+00_JPRB ,&
&  0.68326E+00_JPRB, 0.64096E+00_JPRB, 0.72599E+00_JPRB, 0.71636E+00_JPRB, 0.70441E+00_JPRB, 0.68798E+00_JPRB, 0.68062E+00_JPRB ,&
&  0.67858E+00_JPRB, 0.67600E+00_JPRB, 0.67933E+00_JPRB, 0.68029E+00_JPRB, 0.68850E+00_JPRB, 0.71103E+00_JPRB, 0.77062E+00_JPRB ,&
&  0.89948E+00_JPRB, 0.90642E+00_JPRB, 0.97875E+00_JPRB, 0.11137E+01_JPRB, 0.15256E+01_JPRB, 0.18816E+01_JPRB, 0.21265E+01_JPRB ,&
&  0.24143E+01_JPRB, 0.24994E+01_JPRB, 0.25167E+01_JPRB, 0.25429E+01_JPRB, 0.25635E+01_JPRB, 0.25912E+01_JPRB, 0.26418E+01_JPRB ,&
&  0.25804E+01_JPRB, 0.25675E+01_JPRB, 0.25427E+01_JPRB, 0.25693E+01_JPRB, 0.25189E+01_JPRB, 0.24488E+01_JPRB, 0.23687E+01_JPRB ,&
&  0.22795E+01_JPRB, 0.21634E+01_JPRB, 0.19882E+01_JPRB, 0.18198E+01_JPRB, 0.16767E+01_JPRB, 0.16113E+01_JPRB, 0.15694E+01_JPRB ,&
&  0.15245E+01_JPRB, 0.14983E+01_JPRB, 0.14689E+01_JPRB, 0.14342E+01_JPRB, 0.13956E+01_JPRB, 0.13432E+01_JPRB, 0.13132E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.10973E+01_JPRB, 0.98686E+00_JPRB, 0.95570E+00_JPRB, 0.95737E+00_JPRB, 0.95291E+00_JPRB, 0.94195E+00_JPRB ,&
&  0.94333E+00_JPRB, 0.91157E+00_JPRB, 0.87817E+00_JPRB, 0.85276E+00_JPRB, 0.82769E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.83158E+00_JPRB, 0.79897E+00_JPRB, 0.75832E+00_JPRB, 0.71090E+00_JPRB, 0.66609E+00_JPRB, 0.63762E+00_JPRB, 0.61734E+00_JPRB ,&
&  0.61172E+00_JPRB, 0.58505E+00_JPRB, 0.55767E+00_JPRB, 0.53851E+00_JPRB, 0.52917E+00_JPRB, 0.52594E+00_JPRB, 0.53156E+00_JPRB ,&
&  0.54341E+00_JPRB, 0.55664E+00_JPRB, 0.56999E+00_JPRB, 0.58399E+00_JPRB, 0.59900E+00_JPRB, 0.61319E+00_JPRB, 0.63053E+00_JPRB ,&
&  0.65531E+00_JPRB, 0.68315E+00_JPRB, 0.71272E+00_JPRB, 0.75042E+00_JPRB, 0.78953E+00_JPRB, 0.84392E+00_JPRB, 0.87798E+00_JPRB ,&
&  0.89637E+00_JPRB, 0.89969E+00_JPRB, 0.91658E+00_JPRB, 0.97197E+00_JPRB, 0.10003E+01_JPRB, 0.95871E+00_JPRB, 0.75083E+00_JPRB ,&
&  0.59237E+00_JPRB, 0.54973E+00_JPRB, 0.57078E+00_JPRB, 0.63812E+00_JPRB, 0.64087E+00_JPRB, 0.64268E+00_JPRB, 0.64418E+00_JPRB ,&
&  0.62753E+00_JPRB, 0.62010E+00_JPRB, 0.63055E+00_JPRB, 0.65277E+00_JPRB, 0.65773E+00_JPRB, 0.66905E+00_JPRB, 0.70265E+00_JPRB ,&
&  0.77140E+00_JPRB, 0.86426E+00_JPRB, 0.97107E+00_JPRB, 0.98197E+00_JPRB, 0.11434E+01_JPRB, 0.16980E+01_JPRB, 0.18894E+01_JPRB ,&
&  0.19802E+01_JPRB, 0.20256E+01_JPRB, 0.21166E+01_JPRB, 0.21632E+01_JPRB, 0.21347E+01_JPRB, 0.21354E+01_JPRB, 0.21583E+01_JPRB ,&
&  0.21225E+01_JPRB, 0.21208E+01_JPRB, 0.21183E+01_JPRB, 0.22133E+01_JPRB, 0.22440E+01_JPRB, 0.22225E+01_JPRB, 0.22000E+01_JPRB ,&
&  0.21466E+01_JPRB, 0.19842E+01_JPRB, 0.17986E+01_JPRB, 0.16207E+01_JPRB, 0.15322E+01_JPRB, 0.14722E+01_JPRB, 0.14348E+01_JPRB ,&
&  0.13916E+01_JPRB, 0.13737E+01_JPRB, 0.13679E+01_JPRB, 0.13684E+01_JPRB, 0.12919E+01_JPRB, 0.11778E+01_JPRB, 0.10890E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.10086E+01_JPRB, 0.96480E+00_JPRB, 0.95201E+00_JPRB, 0.95935E+00_JPRB, 0.96597E+00_JPRB, 0.93731E+00_JPRB ,&
&  0.90685E+00_JPRB, 0.88109E+00_JPRB, 0.86863E+00_JPRB, 0.86210E+00_JPRB, 0.85442E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.82614E+00_JPRB, 0.81704E+00_JPRB, 0.78989E+00_JPRB, 0.74068E+00_JPRB, 0.67772E+00_JPRB, 0.62564E+00_JPRB, 0.59319E+00_JPRB ,&
&  0.57598E+00_JPRB, 0.56443E+00_JPRB, 0.55777E+00_JPRB, 0.55484E+00_JPRB, 0.55540E+00_JPRB, 0.55982E+00_JPRB, 0.56464E+00_JPRB ,&
&  0.56872E+00_JPRB, 0.57418E+00_JPRB, 0.58194E+00_JPRB, 0.59117E+00_JPRB, 0.60257E+00_JPRB, 0.61688E+00_JPRB, 0.63435E+00_JPRB ,&
&  0.65740E+00_JPRB, 0.67679E+00_JPRB, 0.68793E+00_JPRB, 0.69727E+00_JPRB, 0.71280E+00_JPRB, 0.74410E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.79477E+00_JPRB, 0.79424E+00_JPRB, 0.82558E+00_JPRB, 0.89156E+00_JPRB, 0.92825E+00_JPRB, 0.88328E+00_JPRB, 0.71167E+00_JPRB ,&
&  0.58070E+00_JPRB, 0.51243E+00_JPRB, 0.49456E+00_JPRB, 0.50728E+00_JPRB, 0.53989E+00_JPRB, 0.58113E+00_JPRB, 0.59420E+00_JPRB ,&
&  0.60278E+00_JPRB, 0.60620E+00_JPRB, 0.60882E+00_JPRB, 0.61341E+00_JPRB, 0.61571E+00_JPRB, 0.62099E+00_JPRB, 0.64135E+00_JPRB ,&
&  0.68637E+00_JPRB, 0.75787E+00_JPRB, 0.84020E+00_JPRB, 0.90360E+00_JPRB, 0.10088E+01_JPRB, 0.12157E+01_JPRB, 0.14419E+01_JPRB ,&
&  0.15392E+01_JPRB, 0.15840E+01_JPRB, 0.16479E+01_JPRB, 0.17555E+01_JPRB, 0.18145E+01_JPRB, 0.18567E+01_JPRB, 0.18616E+01_JPRB ,&
&  0.18466E+01_JPRB, 0.17998E+01_JPRB, 0.17517E+01_JPRB, 0.17494E+01_JPRB, 0.18094E+01_JPRB, 0.19167E+01_JPRB, 0.19706E+01_JPRB ,&
&  0.19833E+01_JPRB, 0.18737E+01_JPRB, 0.16504E+01_JPRB, 0.14455E+01_JPRB, 0.13703E+01_JPRB, 0.13740E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.13936E+01_JPRB, 0.13705E+01_JPRB, 0.13407E+01_JPRB, 0.13338E+01_JPRB, 0.12291E+01_JPRB, 0.11627E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10801E+01_JPRB, 0.10608E+01_JPRB, 0.10564E+01_JPRB, 0.10610E+01_JPRB, 0.10453E+01_JPRB, 0.10148E+01_JPRB, 0.97866E+00_JPRB ,&
&  0.94626E+00_JPRB, 0.91013E+00_JPRB, 0.87935E+00_JPRB, 0.84974E+00_JPRB, 0.83419E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.88588E+00_JPRB, 0.86627E+00_JPRB, 0.83580E+00_JPRB, 0.79258E+00_JPRB, 0.73756E+00_JPRB, 0.68253E+00_JPRB, 0.64678E+00_JPRB ,&
&  0.62848E+00_JPRB, 0.61869E+00_JPRB, 0.60701E+00_JPRB, 0.59953E+00_JPRB, 0.59539E+00_JPRB, 0.59435E+00_JPRB, 0.59631E+00_JPRB ,&
&  0.60073E+00_JPRB, 0.60669E+00_JPRB, 0.61300E+00_JPRB, 0.61686E+00_JPRB, 0.62440E+00_JPRB, 0.63720E+00_JPRB, 0.65444E+00_JPRB ,&
&  0.67369E+00_JPRB, 0.68323E+00_JPRB, 0.68457E+00_JPRB, 0.68703E+00_JPRB, 0.68467E+00_JPRB, 0.69388E+00_JPRB, 0.69459E+00_JPRB ,&
&  0.69873E+00_JPRB, 0.71874E+00_JPRB, 0.77095E+00_JPRB, 0.82588E+00_JPRB, 0.85335E+00_JPRB, 0.79700E+00_JPRB, 0.67469E+00_JPRB ,&
&  0.56985E+00_JPRB, 0.50167E+00_JPRB, 0.47279E+00_JPRB, 0.46642E+00_JPRB, 0.47772E+00_JPRB, 0.50482E+00_JPRB, 0.54356E+00_JPRB ,&
&  0.57477E+00_JPRB, 0.58136E+00_JPRB, 0.58811E+00_JPRB, 0.59228E+00_JPRB, 0.59316E+00_JPRB, 0.59346E+00_JPRB, 0.60065E+00_JPRB ,&
&  0.62031E+00_JPRB, 0.66449E+00_JPRB, 0.73506E+00_JPRB, 0.84074E+00_JPRB, 0.98896E+00_JPRB, 0.10379E+01_JPRB, 0.11466E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.14048E+01_JPRB, 0.14612E+01_JPRB, 0.14849E+01_JPRB, 0.15019E+01_JPRB, 0.15039E+01_JPRB, 0.15164E+01_JPRB ,&
&  0.15303E+01_JPRB, 0.15321E+01_JPRB, 0.15159E+01_JPRB, 0.14898E+01_JPRB, 0.14839E+01_JPRB, 0.15004E+01_JPRB, 0.15378E+01_JPRB ,&
&  0.15943E+01_JPRB, 0.16625E+01_JPRB, 0.18096E+01_JPRB, 0.15793E+01_JPRB, 0.14960E+01_JPRB, 0.14876E+01_JPRB, 0.14496E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.13662E+01_JPRB, 0.13450E+01_JPRB, 0.13213E+01_JPRB, 0.12529E+01_JPRB, 0.12068E+01_JPRB, 0.11713E+01_JPRB ,&
&  0.11543E+01_JPRB, 0.11321E+01_JPRB, 0.11131E+01_JPRB, 0.10965E+01_JPRB, 0.10730E+01_JPRB, 0.10416E+01_JPRB, 0.10154E+01_JPRB ,&
&  0.99043E+00_JPRB, 0.96667E+00_JPRB, 0.94629E+00_JPRB, 0.92627E+00_JPRB, 0.90699E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.89778E+00_JPRB, 0.88019E+00_JPRB, 0.85092E+00_JPRB, 0.82895E+00_JPRB, 0.79779E+00_JPRB, 0.76407E+00_JPRB, 0.72790E+00_JPRB ,&
&  0.69875E+00_JPRB, 0.67542E+00_JPRB, 0.65642E+00_JPRB, 0.64384E+00_JPRB, 0.63618E+00_JPRB, 0.63153E+00_JPRB, 0.63138E+00_JPRB ,&
&  0.63287E+00_JPRB, 0.62910E+00_JPRB, 0.63143E+00_JPRB, 0.63838E+00_JPRB, 0.64848E+00_JPRB, 0.65890E+00_JPRB, 0.67221E+00_JPRB ,&
&  0.67875E+00_JPRB, 0.67771E+00_JPRB, 0.67031E+00_JPRB, 0.66149E+00_JPRB, 0.65737E+00_JPRB, 0.66375E+00_JPRB, 0.68507E+00_JPRB ,&
&  0.70808E+00_JPRB, 0.72687E+00_JPRB, 0.74827E+00_JPRB, 0.77328E+00_JPRB, 0.77831E+00_JPRB, 0.72940E+00_JPRB, 0.62621E+00_JPRB ,&
&  0.55713E+00_JPRB, 0.50736E+00_JPRB, 0.48409E+00_JPRB, 0.47446E+00_JPRB, 0.48090E+00_JPRB, 0.50003E+00_JPRB, 0.53328E+00_JPRB ,&
&  0.55971E+00_JPRB, 0.57600E+00_JPRB, 0.57791E+00_JPRB, 0.57927E+00_JPRB, 0.57852E+00_JPRB, 0.58166E+00_JPRB, 0.58691E+00_JPRB ,&
&  0.60298E+00_JPRB, 0.62111E+00_JPRB, 0.64662E+00_JPRB, 0.68511E+00_JPRB, 0.73675E+00_JPRB, 0.79218E+00_JPRB, 0.86139E+00_JPRB ,&
&  0.98456E+00_JPRB, 0.10569E+01_JPRB, 0.11108E+01_JPRB, 0.11452E+01_JPRB, 0.11909E+01_JPRB, 0.12178E+01_JPRB, 0.12079E+01_JPRB ,&
&  0.12154E+01_JPRB, 0.12679E+01_JPRB, 0.12892E+01_JPRB, 0.13060E+01_JPRB, 0.13167E+01_JPRB, 0.13353E+01_JPRB, 0.13531E+01_JPRB ,&
&  0.13287E+01_JPRB, 0.13362E+01_JPRB, 0.14415E+01_JPRB, 0.14613E+01_JPRB, 0.14295E+01_JPRB, 0.13894E+01_JPRB, 0.13392E+01_JPRB ,&
&  0.12960E+01_JPRB, 0.12550E+01_JPRB, 0.12199E+01_JPRB, 0.11872E+01_JPRB, 0.11624E+01_JPRB, 0.11495E+01_JPRB, 0.11477E+01_JPRB ,&
&  0.11349E+01_JPRB, 0.11164E+01_JPRB, 0.10901E+01_JPRB, 0.10691E+01_JPRB, 0.10506E+01_JPRB, 0.10229E+01_JPRB, 0.99552E+00_JPRB ,&
&  0.96973E+00_JPRB, 0.95284E+00_JPRB, 0.94258E+00_JPRB, 0.93260E+00_JPRB, 0.91733E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.90338E+00_JPRB, 0.89512E+00_JPRB, 0.87530E+00_JPRB, 0.85786E+00_JPRB, 0.83413E+00_JPRB, 0.81256E+00_JPRB, 0.78528E+00_JPRB ,&
&  0.76006E+00_JPRB, 0.73528E+00_JPRB, 0.71452E+00_JPRB, 0.69865E+00_JPRB, 0.68615E+00_JPRB, 0.67704E+00_JPRB, 0.67214E+00_JPRB ,&
&  0.67002E+00_JPRB, 0.66977E+00_JPRB, 0.67191E+00_JPRB, 0.67559E+00_JPRB, 0.68173E+00_JPRB, 0.68891E+00_JPRB, 0.69802E+00_JPRB ,&
&  0.70176E+00_JPRB, 0.69564E+00_JPRB, 0.67495E+00_JPRB, 0.66705E+00_JPRB, 0.67230E+00_JPRB, 0.67496E+00_JPRB, 0.70157E+00_JPRB ,&
&  0.71472E+00_JPRB, 0.73044E+00_JPRB, 0.72812E+00_JPRB, 0.72407E+00_JPRB, 0.71458E+00_JPRB, 0.59758E+00_JPRB, 0.57624E+00_JPRB ,&
&  0.54630E+00_JPRB, 0.52041E+00_JPRB, 0.50723E+00_JPRB, 0.49937E+00_JPRB, 0.49841E+00_JPRB, 0.50947E+00_JPRB, 0.52703E+00_JPRB ,&
&  0.54054E+00_JPRB, 0.58358E+00_JPRB, 0.58341E+00_JPRB, 0.58159E+00_JPRB, 0.57971E+00_JPRB, 0.58259E+00_JPRB, 0.58043E+00_JPRB ,&
&  0.58743E+00_JPRB, 0.59768E+00_JPRB, 0.61315E+00_JPRB, 0.63857E+00_JPRB, 0.66422E+00_JPRB, 0.68870E+00_JPRB, 0.71657E+00_JPRB ,&
&  0.75684E+00_JPRB, 0.78894E+00_JPRB, 0.80891E+00_JPRB, 0.82742E+00_JPRB, 0.85380E+00_JPRB, 0.88753E+00_JPRB, 0.93305E+00_JPRB ,&
&  0.96700E+00_JPRB, 0.10050E+01_JPRB, 0.10443E+01_JPRB, 0.10905E+01_JPRB, 0.11310E+01_JPRB, 0.11742E+01_JPRB, 0.12160E+01_JPRB ,&
&  0.12404E+01_JPRB, 0.12648E+01_JPRB, 0.12693E+01_JPRB, 0.12984E+01_JPRB, 0.12792E+01_JPRB, 0.12387E+01_JPRB, 0.12122E+01_JPRB ,&
&  0.11890E+01_JPRB, 0.11911E+01_JPRB, 0.11697E+01_JPRB, 0.11470E+01_JPRB, 0.11258E+01_JPRB, 0.11105E+01_JPRB, 0.10946E+01_JPRB ,&
&  0.10823E+01_JPRB, 0.10677E+01_JPRB, 0.10517E+01_JPRB, 0.10322E+01_JPRB, 0.10170E+01_JPRB, 0.99799E+00_JPRB, 0.98049E+00_JPRB ,&
&  0.95815E+00_JPRB, 0.94218E+00_JPRB, 0.93045E+00_JPRB, 0.92049E+00_JPRB, 0.90978E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.89538E+00_JPRB, 0.88379E+00_JPRB, 0.87312E+00_JPRB, 0.86016E+00_JPRB, 0.84519E+00_JPRB, 0.83039E+00_JPRB, 0.81091E+00_JPRB ,&
&  0.79204E+00_JPRB, 0.77324E+00_JPRB, 0.75696E+00_JPRB, 0.74375E+00_JPRB, 0.73340E+00_JPRB, 0.72548E+00_JPRB, 0.72109E+00_JPRB ,&
&  0.71673E+00_JPRB, 0.71518E+00_JPRB, 0.71479E+00_JPRB, 0.71639E+00_JPRB, 0.71949E+00_JPRB, 0.72229E+00_JPRB, 0.72658E+00_JPRB ,&
&  0.72223E+00_JPRB, 0.70503E+00_JPRB, 0.68081E+00_JPRB, 0.67497E+00_JPRB, 0.66992E+00_JPRB, 0.67335E+00_JPRB, 0.67489E+00_JPRB ,&
&  0.69038E+00_JPRB, 0.65343E+00_JPRB, 0.65108E+00_JPRB, 0.61736E+00_JPRB, 0.58827E+00_JPRB, 0.55923E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.52269E+00_JPRB, 0.51197E+00_JPRB, 0.50652E+00_JPRB, 0.50751E+00_JPRB, 0.51201E+00_JPRB, 0.52444E+00_JPRB, 0.53968E+00_JPRB ,&
&  0.55085E+00_JPRB, 0.58565E+00_JPRB, 0.59061E+00_JPRB, 0.59423E+00_JPRB, 0.60014E+00_JPRB, 0.60766E+00_JPRB, 0.60707E+00_JPRB ,&
&  0.61290E+00_JPRB, 0.61969E+00_JPRB, 0.62935E+00_JPRB, 0.64440E+00_JPRB, 0.66482E+00_JPRB, 0.67961E+00_JPRB, 0.69431E+00_JPRB ,&
&  0.71146E+00_JPRB, 0.72356E+00_JPRB, 0.73315E+00_JPRB, 0.74233E+00_JPRB, 0.75465E+00_JPRB, 0.76672E+00_JPRB, 0.78585E+00_JPRB ,&
&  0.80716E+00_JPRB, 0.84444E+00_JPRB, 0.87817E+00_JPRB, 0.92809E+00_JPRB, 0.97376E+00_JPRB, 0.10312E+01_JPRB, 0.10778E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.11591E+01_JPRB, 0.11935E+01_JPRB, 0.12252E+01_JPRB, 0.12498E+01_JPRB, 0.12360E+01_JPRB, 0.11979E+01_JPRB ,&
&  0.11741E+01_JPRB, 0.11384E+01_JPRB, 0.11140E+01_JPRB, 0.10909E+01_JPRB, 0.10708E+01_JPRB, 0.10572E+01_JPRB, 0.10446E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.10229E+01_JPRB, 0.10119E+01_JPRB, 0.99890E+00_JPRB, 0.98706E+00_JPRB, 0.97320E+00_JPRB, 0.96017E+00_JPRB ,&
&  0.94691E+00_JPRB, 0.93845E+00_JPRB, 0.92659E+00_JPRB, 0.91470E+00_JPRB, 0.90432E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.89072E+00_JPRB, 0.87638E+00_JPRB, 0.86610E+00_JPRB, 0.85371E+00_JPRB, 0.84193E+00_JPRB, 0.82864E+00_JPRB, 0.81695E+00_JPRB ,&
&  0.80455E+00_JPRB, 0.79232E+00_JPRB, 0.78054E+00_JPRB, 0.77025E+00_JPRB, 0.76280E+00_JPRB, 0.75695E+00_JPRB, 0.75271E+00_JPRB ,&
&  0.74932E+00_JPRB, 0.74662E+00_JPRB, 0.74511E+00_JPRB, 0.74457E+00_JPRB, 0.74472E+00_JPRB, 0.74489E+00_JPRB, 0.74481E+00_JPRB ,&
&  0.73483E+00_JPRB, 0.71702E+00_JPRB, 0.68121E+00_JPRB, 0.67172E+00_JPRB, 0.65915E+00_JPRB, 0.65144E+00_JPRB, 0.65115E+00_JPRB ,&
&  0.64380E+00_JPRB, 0.64116E+00_JPRB, 0.63226E+00_JPRB, 0.61875E+00_JPRB, 0.60137E+00_JPRB, 0.57685E+00_JPRB, 0.56037E+00_JPRB ,&
&  0.54726E+00_JPRB, 0.53928E+00_JPRB, 0.53742E+00_JPRB, 0.53815E+00_JPRB, 0.54637E+00_JPRB, 0.55578E+00_JPRB, 0.56696E+00_JPRB ,&
&  0.58426E+00_JPRB, 0.60064E+00_JPRB, 0.62118E+00_JPRB, 0.62624E+00_JPRB, 0.63629E+00_JPRB, 0.64644E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.66764E+00_JPRB, 0.67811E+00_JPRB, 0.68803E+00_JPRB, 0.69759E+00_JPRB, 0.70726E+00_JPRB, 0.71616E+00_JPRB, 0.72533E+00_JPRB ,&
&  0.73440E+00_JPRB, 0.74180E+00_JPRB, 0.74805E+00_JPRB, 0.75528E+00_JPRB, 0.76456E+00_JPRB, 0.77339E+00_JPRB, 0.78485E+00_JPRB ,&
&  0.79640E+00_JPRB, 0.80988E+00_JPRB, 0.82309E+00_JPRB, 0.83772E+00_JPRB, 0.85139E+00_JPRB, 0.86958E+00_JPRB, 0.88851E+00_JPRB ,&
&  0.91506E+00_JPRB, 0.93923E+00_JPRB, 0.97153E+00_JPRB, 0.98952E+00_JPRB, 0.10285E+01_JPRB, 0.10661E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.10876E+01_JPRB, 0.10778E+01_JPRB, 0.10681E+01_JPRB, 0.10567E+01_JPRB, 0.10459E+01_JPRB, 0.10309E+01_JPRB, 0.10210E+01_JPRB ,&
&  0.10074E+01_JPRB, 0.99737E+00_JPRB, 0.98599E+00_JPRB, 0.97678E+00_JPRB, 0.96818E+00_JPRB, 0.95863E+00_JPRB, 0.94833E+00_JPRB ,&
&  0.93877E+00_JPRB, 0.93092E+00_JPRB, 0.92287E+00_JPRB, 0.91396E+00_JPRB, 0.90319E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.88588E+00_JPRB, 0.87796E+00_JPRB, 0.87020E+00_JPRB, 0.86139E+00_JPRB, 0.85439E+00_JPRB, 0.84622E+00_JPRB, 0.83888E+00_JPRB ,&
&  0.83065E+00_JPRB, 0.82236E+00_JPRB, 0.81462E+00_JPRB, 0.80716E+00_JPRB, 0.80010E+00_JPRB, 0.79335E+00_JPRB, 0.78809E+00_JPRB ,&
&  0.78315E+00_JPRB, 0.77907E+00_JPRB, 0.77602E+00_JPRB, 0.77392E+00_JPRB, 0.77129E+00_JPRB, 0.76791E+00_JPRB, 0.76263E+00_JPRB ,&
&  0.75483E+00_JPRB, 0.74388E+00_JPRB, 0.73135E+00_JPRB, 0.71729E+00_JPRB, 0.71015E+00_JPRB, 0.67089E+00_JPRB, 0.66285E+00_JPRB ,&
&  0.66220E+00_JPRB, 0.66157E+00_JPRB, 0.66568E+00_JPRB, 0.67075E+00_JPRB, 0.67623E+00_JPRB, 0.66552E+00_JPRB, 0.65766E+00_JPRB ,&
&  0.63010E+00_JPRB, 0.63150E+00_JPRB, 0.62966E+00_JPRB, 0.62983E+00_JPRB, 0.63420E+00_JPRB, 0.63729E+00_JPRB, 0.64576E+00_JPRB ,&
&  0.67017E+00_JPRB, 0.67539E+00_JPRB, 0.68320E+00_JPRB, 0.69058E+00_JPRB, 0.69797E+00_JPRB, 0.70484E+00_JPRB, 0.71269E+00_JPRB ,&
&  0.71989E+00_JPRB, 0.72790E+00_JPRB, 0.73660E+00_JPRB, 0.74318E+00_JPRB, 0.74900E+00_JPRB, 0.75525E+00_JPRB, 0.76097E+00_JPRB ,&
&  0.76463E+00_JPRB, 0.76806E+00_JPRB, 0.77163E+00_JPRB, 0.77611E+00_JPRB, 0.78021E+00_JPRB, 0.78443E+00_JPRB, 0.78831E+00_JPRB ,&
&  0.79228E+00_JPRB, 0.79533E+00_JPRB, 0.79745E+00_JPRB, 0.79974E+00_JPRB, 0.80103E+00_JPRB, 0.80259E+00_JPRB, 0.80481E+00_JPRB ,&
&  0.80873E+00_JPRB, 0.81307E+00_JPRB, 0.81876E+00_JPRB, 0.82552E+00_JPRB, 0.83664E+00_JPRB, 0.85041E+00_JPRB, 0.87461E+00_JPRB ,&
&  0.89471E+00_JPRB, 0.91179E+00_JPRB, 0.91942E+00_JPRB, 0.92745E+00_JPRB, 0.93618E+00_JPRB, 0.94084E+00_JPRB, 0.94454E+00_JPRB ,&
&  0.94419E+00_JPRB, 0.94219E+00_JPRB, 0.93721E+00_JPRB, 0.93162E+00_JPRB, 0.92546E+00_JPRB, 0.91936E+00_JPRB, 0.91451E+00_JPRB ,&
&  0.91025E+00_JPRB, 0.90762E+00_JPRB, 0.90408E+00_JPRB, 0.89887E+00_JPRB, 0.89249E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.88284E+00_JPRB, 0.87981E+00_JPRB, 0.87644E+00_JPRB, 0.87243E+00_JPRB, 0.86830E+00_JPRB, 0.86355E+00_JPRB, 0.85894E+00_JPRB ,&
&  0.85375E+00_JPRB, 0.84915E+00_JPRB, 0.84419E+00_JPRB, 0.83909E+00_JPRB, 0.83373E+00_JPRB, 0.82856E+00_JPRB, 0.82351E+00_JPRB ,&
&  0.81815E+00_JPRB, 0.81226E+00_JPRB, 0.80626E+00_JPRB, 0.80116E+00_JPRB, 0.79561E+00_JPRB, 0.78934E+00_JPRB, 0.78226E+00_JPRB ,&
&  0.77670E+00_JPRB, 0.77044E+00_JPRB, 0.76393E+00_JPRB, 0.75541E+00_JPRB, 0.74830E+00_JPRB, 0.74139E+00_JPRB, 0.73523E+00_JPRB ,&
&  0.72907E+00_JPRB, 0.72436E+00_JPRB, 0.72054E+00_JPRB, 0.71740E+00_JPRB, 0.71612E+00_JPRB, 0.71537E+00_JPRB, 0.71460E+00_JPRB ,&
&  0.71357E+00_JPRB, 0.71242E+00_JPRB, 0.71295E+00_JPRB, 0.71391E+00_JPRB, 0.71487E+00_JPRB, 0.71838E+00_JPRB, 0.72260E+00_JPRB ,&
&  0.72685E+00_JPRB, 0.73076E+00_JPRB, 0.73618E+00_JPRB, 0.74179E+00_JPRB, 0.74701E+00_JPRB, 0.75205E+00_JPRB, 0.75671E+00_JPRB ,&
&  0.76164E+00_JPRB, 0.76783E+00_JPRB, 0.77375E+00_JPRB, 0.77997E+00_JPRB, 0.78456E+00_JPRB, 0.78852E+00_JPRB, 0.79177E+00_JPRB ,&
&  0.79432E+00_JPRB, 0.79685E+00_JPRB, 0.79895E+00_JPRB, 0.80085E+00_JPRB, 0.80175E+00_JPRB, 0.80317E+00_JPRB, 0.80427E+00_JPRB ,&
&  0.80539E+00_JPRB, 0.80535E+00_JPRB, 0.80656E+00_JPRB, 0.80763E+00_JPRB, 0.80831E+00_JPRB, 0.80838E+00_JPRB, 0.80963E+00_JPRB ,&
&  0.81071E+00_JPRB, 0.81146E+00_JPRB, 0.81165E+00_JPRB, 0.81406E+00_JPRB, 0.81622E+00_JPRB, 0.81807E+00_JPRB, 0.81903E+00_JPRB ,&
&  0.82492E+00_JPRB, 0.83364E+00_JPRB, 0.84468E+00_JPRB, 0.85951E+00_JPRB, 0.87116E+00_JPRB, 0.87939E+00_JPRB, 0.88571E+00_JPRB ,&
&  0.88920E+00_JPRB, 0.89112E+00_JPRB, 0.89170E+00_JPRB, 0.89196E+00_JPRB, 0.89048E+00_JPRB, 0.88996E+00_JPRB, 0.89049E+00_JPRB ,&
&  0.89068E+00_JPRB, 0.89004E+00_JPRB, 0.88890E+00_JPRB, 0.88752E+00_JPRB, 0.88542E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.86761E+00_JPRB, 0.86591E+00_JPRB, 0.86425E+00_JPRB, 0.86237E+00_JPRB, 0.86017E+00_JPRB, 0.85770E+00_JPRB, 0.85550E+00_JPRB ,&
&  0.85315E+00_JPRB, 0.85119E+00_JPRB, 0.84854E+00_JPRB, 0.84588E+00_JPRB, 0.84313E+00_JPRB, 0.84062E+00_JPRB, 0.83796E+00_JPRB ,&
&  0.83504E+00_JPRB, 0.83209E+00_JPRB, 0.82910E+00_JPRB, 0.82604E+00_JPRB, 0.82275E+00_JPRB, 0.81954E+00_JPRB, 0.81609E+00_JPRB ,&
&  0.81305E+00_JPRB, 0.80967E+00_JPRB, 0.80638E+00_JPRB, 0.80286E+00_JPRB, 0.80035E+00_JPRB, 0.79723E+00_JPRB, 0.79455E+00_JPRB ,&
&  0.79167E+00_JPRB, 0.78970E+00_JPRB, 0.78746E+00_JPRB, 0.78555E+00_JPRB, 0.78377E+00_JPRB, 0.78346E+00_JPRB, 0.78268E+00_JPRB ,&
&  0.78225E+00_JPRB, 0.78215E+00_JPRB, 0.78265E+00_JPRB, 0.78300E+00_JPRB, 0.78334E+00_JPRB, 0.78418E+00_JPRB, 0.78582E+00_JPRB ,&
&  0.78732E+00_JPRB, 0.78886E+00_JPRB, 0.79090E+00_JPRB, 0.79300E+00_JPRB, 0.79492E+00_JPRB, 0.79673E+00_JPRB, 0.79865E+00_JPRB ,&
&  0.80165E+00_JPRB, 0.80483E+00_JPRB, 0.80781E+00_JPRB, 0.81077E+00_JPRB, 0.81347E+00_JPRB, 0.81553E+00_JPRB, 0.81737E+00_JPRB ,&
&  0.81878E+00_JPRB, 0.82036E+00_JPRB, 0.82164E+00_JPRB, 0.82278E+00_JPRB, 0.82328E+00_JPRB, 0.82440E+00_JPRB, 0.82529E+00_JPRB ,&
&  0.82594E+00_JPRB, 0.82580E+00_JPRB, 0.82661E+00_JPRB, 0.82732E+00_JPRB, 0.82818E+00_JPRB, 0.82889E+00_JPRB, 0.83057E+00_JPRB ,&
&  0.83230E+00_JPRB, 0.83455E+00_JPRB, 0.83705E+00_JPRB, 0.83996E+00_JPRB, 0.84227E+00_JPRB, 0.84458E+00_JPRB, 0.84678E+00_JPRB ,&
&  0.84922E+00_JPRB, 0.85180E+00_JPRB, 0.85468E+00_JPRB, 0.85741E+00_JPRB, 0.85989E+00_JPRB, 0.86243E+00_JPRB, 0.86470E+00_JPRB ,&
&  0.86717E+00_JPRB, 0.86860E+00_JPRB, 0.86974E+00_JPRB, 0.87053E+00_JPRB, 0.87111E+00_JPRB, 0.87127E+00_JPRB, 0.87123E+00_JPRB ,&
&  0.87093E+00_JPRB, 0.87053E+00_JPRB, 0.86996E+00_JPRB, 0.86948E+00_JPRB, 0.86888E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB, 0.83270E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1980:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1980
