!option! -Nv -NO
#ifdef RS6K
@PROCESS NOOPTIMIZE
#endif
!pgi$r opt=0 
SUBROUTINE SU_SO4_OBS1990 ( PBURDSO4 )


!**   OPTICAL THICKNESS OF SULPHTE-TYPE AEROSOLS 
!     FROM O.BOUCHER'S ENSEMBLES BURDEN-TYPE FILES

!     PURPOSE.
!     --------

!**   INTERFACE.
!     ----------
!        CALL *SU_SO4_OBS1990* FROM *SUECRAD*

!        EXPLICIT ARGUMENTS :
!        --------------------
!     ==== INPUTS ===
!     ==== OUTPUTS ===

!        IMPLICIT ARGUMENTS :   NONE
!        --------------------

!     METHOD.
!     -------

!     EXTERNALS.
!     ----------

!          NONE

!     REFERENCE.
!     ----------

!        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND
!        ECMWF RESEARCH DEPARTMENT DOCUMENTATION OF THE "I.F.S"

!     AUTHOR.
!     -------
!     J.-J. MORCRETTE  E.C.M.W.F.    20080310

!     MODIFICATIONS.
!     --------------
!        Same history as suecaesu
!        M.Hamrud      01-Oct-2003 CY28 Cleaning
!     R. Elkhatib 12-10-2005 Split for faster and more robust compilation.

!-----------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

IMPLICIT NONE

!     -----------------------------------------------------------------

!*       0.1   ARGUMENTS.
!              ----------

REAL(KIND=JPRB), INTENT(OUT) :: PBURDSO4(96,73,12)

!     ----------------------------------------------------------------- 

!*       0.2   LOCAL ARRAYS.
!              -------------

REAL(KIND=JPRB) :: ZLON(96), ZLAT(73)

INTEGER(KIND=JPIM) :: IJ, ILAT, ILON, ILON2
INTEGER(KIND=JPIM) :: JI, JL, JM

REAL(KIND=JPRB),ALLOCATABLE,SAVE :: ZBURD(:,:,:)
LOGICAL,SAVE :: LLFIRSTCALL = .TRUE.

REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

!-- Boucher,O. and M. Pham, 2002: Geophys. Res. Lett., 29, 1308, 
!   doi: 10.1029/2001GL014048

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990',0,ZHOOK_HANDLE)

IF(LLFIRSTCALL) THEN

  ALLOCATE(ZBURD(96,73,12))

  CALL PART1
  CALL PART2
  CALL PART3
  CALL PART4
  CALL PART5
  CALL PART6
  CALL PART7
  CALL PART8
  CALL PART9
  CALL PART10
  CALL PART11
  CALL PART12
  CALL PART13
  CALL PART14
  CALL PART15
  CALL PART16
  CALL PART17
  CALL PART18
  CALL PART19
  CALL PART20
  CALL PART21
  CALL PART22
  CALL PART23
  CALL PART24

ENDIF


!     ------------------------------------------------------------------

!ZMAX=-1.E+8_JPRB
!ZMIN= 1.E+8_JPRB
ILAT=73
ILON=96
ILON2=ILON/2
DO JI=1,ILAT
  ZLAT(JI)=90._JPRB-(JI-1)*2.5_JPRB
ENDDO
DO JL=1,ILON
  ZLON(JL)=0._JPRB+(JL-1)*3.75_JPRB
ENDDO

DO JM=1,12
  DO JI=1,ILAT
    IJ=ILAT+1-JI

    DO JL=1,ILON2
      PBURDSO4(JL,IJ,JM)=ZBURD(JL+ILON2,JI,JM)
    ENDDO
    DO JL=ILON2+1,ILON
      PBURDSO4(JL,IJ,JM)=ZBURD(JL-ILON2,JI,JM)
    ENDDO
  ENDDO
ENDDO
!PRINT 9000,ZMIN,ZMAX
9000 FORMAT(1x,'ENSEMBLES SULPHATE AEROSOLS ',2E12.5)

!     ------------------------------------------------------------------

LLFIRSTCALL = .FALSE.

IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990',1,ZHOOK_HANDLE)

!-----------------------------------------------------------------------------

CONTAINS

SUBROUTINE PART1

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART1',0,ZHOOK_HANDLE)


!-- month=  1
ZBURD(:,  1,  1) = (/ &
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB ,&
&  0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB, 0.82994E+00_JPRB /)
ZBURD(:,  2,  1) = (/ &
&  0.85790E+00_JPRB, 0.85462E+00_JPRB, 0.85166E+00_JPRB, 0.84893E+00_JPRB, 0.84650E+00_JPRB, 0.84460E+00_JPRB, 0.84280E+00_JPRB ,&
&  0.84034E+00_JPRB, 0.83812E+00_JPRB, 0.83607E+00_JPRB, 0.83406E+00_JPRB, 0.83192E+00_JPRB, 0.82977E+00_JPRB, 0.82748E+00_JPRB ,&
&  0.82508E+00_JPRB, 0.82290E+00_JPRB, 0.82136E+00_JPRB, 0.81917E+00_JPRB, 0.81659E+00_JPRB, 0.81358E+00_JPRB, 0.81060E+00_JPRB ,&
&  0.80719E+00_JPRB, 0.80371E+00_JPRB, 0.79998E+00_JPRB, 0.79630E+00_JPRB, 0.79243E+00_JPRB, 0.78736E+00_JPRB, 0.78303E+00_JPRB ,&
&  0.78015E+00_JPRB, 0.77778E+00_JPRB, 0.77569E+00_JPRB, 0.77350E+00_JPRB, 0.77193E+00_JPRB, 0.77153E+00_JPRB, 0.77123E+00_JPRB ,&
&  0.77136E+00_JPRB, 0.77198E+00_JPRB, 0.77373E+00_JPRB, 0.77580E+00_JPRB, 0.77821E+00_JPRB, 0.78160E+00_JPRB, 0.78651E+00_JPRB ,&
&  0.79099E+00_JPRB, 0.79524E+00_JPRB, 0.80016E+00_JPRB, 0.80535E+00_JPRB, 0.81083E+00_JPRB, 0.81618E+00_JPRB, 0.82132E+00_JPRB ,&
&  0.82509E+00_JPRB, 0.82851E+00_JPRB, 0.83195E+00_JPRB, 0.83496E+00_JPRB, 0.83648E+00_JPRB, 0.83799E+00_JPRB, 0.83949E+00_JPRB ,&
&  0.84079E+00_JPRB, 0.84162E+00_JPRB, 0.84294E+00_JPRB, 0.84491E+00_JPRB, 0.84695E+00_JPRB, 0.84895E+00_JPRB, 0.85164E+00_JPRB ,&
&  0.85498E+00_JPRB, 0.85719E+00_JPRB, 0.85986E+00_JPRB, 0.86332E+00_JPRB, 0.86761E+00_JPRB, 0.87261E+00_JPRB, 0.87670E+00_JPRB ,&
&  0.88098E+00_JPRB, 0.88555E+00_JPRB, 0.88998E+00_JPRB, 0.89280E+00_JPRB, 0.89596E+00_JPRB, 0.89887E+00_JPRB, 0.90156E+00_JPRB ,&
&  0.90181E+00_JPRB, 0.90285E+00_JPRB, 0.90454E+00_JPRB, 0.90637E+00_JPRB, 0.90440E+00_JPRB, 0.90288E+00_JPRB, 0.90139E+00_JPRB ,&
&  0.90168E+00_JPRB, 0.89747E+00_JPRB, 0.89457E+00_JPRB, 0.89084E+00_JPRB, 0.88828E+00_JPRB, 0.88373E+00_JPRB, 0.87998E+00_JPRB ,&
&  0.87586E+00_JPRB, 0.87248E+00_JPRB, 0.86866E+00_JPRB, 0.86474E+00_JPRB, 0.86105E+00_JPRB /)
ZBURD(:,  3,  1) = (/ &
&  0.87137E+00_JPRB, 0.86530E+00_JPRB, 0.85913E+00_JPRB, 0.85298E+00_JPRB, 0.84748E+00_JPRB, 0.84331E+00_JPRB, 0.83902E+00_JPRB ,&
&  0.83445E+00_JPRB, 0.83109E+00_JPRB, 0.82803E+00_JPRB, 0.82545E+00_JPRB, 0.82296E+00_JPRB, 0.82256E+00_JPRB, 0.82150E+00_JPRB ,&
&  0.81953E+00_JPRB, 0.81627E+00_JPRB, 0.81399E+00_JPRB, 0.81013E+00_JPRB, 0.80553E+00_JPRB, 0.79901E+00_JPRB, 0.79297E+00_JPRB ,&
&  0.78550E+00_JPRB, 0.77633E+00_JPRB, 0.76876E+00_JPRB, 0.76506E+00_JPRB, 0.76072E+00_JPRB, 0.75411E+00_JPRB, 0.74968E+00_JPRB ,&
&  0.74750E+00_JPRB, 0.74521E+00_JPRB, 0.74340E+00_JPRB, 0.74153E+00_JPRB, 0.73676E+00_JPRB, 0.73463E+00_JPRB, 0.73289E+00_JPRB ,&
&  0.73018E+00_JPRB, 0.72974E+00_JPRB, 0.73306E+00_JPRB, 0.73683E+00_JPRB, 0.74195E+00_JPRB, 0.74944E+00_JPRB, 0.76337E+00_JPRB ,&
&  0.78058E+00_JPRB, 0.79681E+00_JPRB, 0.81318E+00_JPRB, 0.82888E+00_JPRB, 0.84210E+00_JPRB, 0.85322E+00_JPRB, 0.86321E+00_JPRB ,&
&  0.87022E+00_JPRB, 0.87567E+00_JPRB, 0.87880E+00_JPRB, 0.87947E+00_JPRB, 0.87818E+00_JPRB, 0.87781E+00_JPRB, 0.87799E+00_JPRB ,&
&  0.87833E+00_JPRB, 0.87375E+00_JPRB, 0.87197E+00_JPRB, 0.87305E+00_JPRB, 0.87600E+00_JPRB, 0.87924E+00_JPRB, 0.88536E+00_JPRB ,&
&  0.89372E+00_JPRB, 0.90337E+00_JPRB, 0.91282E+00_JPRB, 0.92530E+00_JPRB, 0.93980E+00_JPRB, 0.95576E+00_JPRB, 0.96570E+00_JPRB ,&
&  0.97818E+00_JPRB, 0.99173E+00_JPRB, 0.10063E+01_JPRB, 0.10095E+01_JPRB, 0.10139E+01_JPRB, 0.10176E+01_JPRB, 0.10226E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10156E+01_JPRB, 0.10140E+01_JPRB, 0.10159E+01_JPRB, 0.10081E+01_JPRB, 0.10040E+01_JPRB, 0.99990E+00_JPRB ,&
&  0.10006E+01_JPRB, 0.98734E+00_JPRB, 0.97822E+00_JPRB, 0.96633E+00_JPRB, 0.95705E+00_JPRB, 0.94049E+00_JPRB, 0.92658E+00_JPRB ,&
&  0.91387E+00_JPRB, 0.90477E+00_JPRB, 0.89477E+00_JPRB, 0.88598E+00_JPRB, 0.87832E+00_JPRB /)
ZBURD(:,  4,  1) = (/ &
&  0.88642E+00_JPRB, 0.87527E+00_JPRB, 0.86005E+00_JPRB, 0.84188E+00_JPRB, 0.82560E+00_JPRB, 0.81410E+00_JPRB, 0.80230E+00_JPRB ,&
&  0.79311E+00_JPRB, 0.78680E+00_JPRB, 0.78290E+00_JPRB, 0.78052E+00_JPRB, 0.77809E+00_JPRB, 0.78017E+00_JPRB, 0.78265E+00_JPRB ,&
&  0.78548E+00_JPRB, 0.78652E+00_JPRB, 0.79105E+00_JPRB, 0.79243E+00_JPRB, 0.79170E+00_JPRB, 0.78692E+00_JPRB, 0.78152E+00_JPRB ,&
&  0.77155E+00_JPRB, 0.75938E+00_JPRB, 0.74493E+00_JPRB, 0.72726E+00_JPRB, 0.71121E+00_JPRB, 0.64853E+00_JPRB, 0.62861E+00_JPRB ,&
&  0.61455E+00_JPRB, 0.60655E+00_JPRB, 0.60445E+00_JPRB, 0.60708E+00_JPRB, 0.62672E+00_JPRB, 0.62198E+00_JPRB, 0.61330E+00_JPRB ,&
&  0.56925E+00_JPRB, 0.55436E+00_JPRB, 0.54691E+00_JPRB, 0.54438E+00_JPRB, 0.55374E+00_JPRB, 0.58785E+00_JPRB, 0.63232E+00_JPRB ,&
&  0.70976E+00_JPRB, 0.74708E+00_JPRB, 0.77775E+00_JPRB, 0.81202E+00_JPRB, 0.84133E+00_JPRB, 0.86551E+00_JPRB, 0.88845E+00_JPRB ,&
&  0.90908E+00_JPRB, 0.93008E+00_JPRB, 0.94732E+00_JPRB, 0.95844E+00_JPRB, 0.96343E+00_JPRB, 0.96653E+00_JPRB, 0.96815E+00_JPRB ,&
&  0.96851E+00_JPRB, 0.96641E+00_JPRB, 0.96516E+00_JPRB, 0.96326E+00_JPRB, 0.95919E+00_JPRB, 0.95955E+00_JPRB, 0.96658E+00_JPRB ,&
&  0.97882E+00_JPRB, 0.99743E+00_JPRB, 0.10152E+01_JPRB, 0.10418E+01_JPRB, 0.10673E+01_JPRB, 0.11008E+01_JPRB, 0.11148E+01_JPRB ,&
&  0.11328E+01_JPRB, 0.11442E+01_JPRB, 0.11597E+01_JPRB, 0.11601E+01_JPRB, 0.11608E+01_JPRB, 0.11585E+01_JPRB, 0.11595E+01_JPRB ,&
&  0.11498E+01_JPRB, 0.11421E+01_JPRB, 0.11322E+01_JPRB, 0.11289E+01_JPRB, 0.11164E+01_JPRB, 0.11115E+01_JPRB, 0.11031E+01_JPRB ,&
&  0.11035E+01_JPRB, 0.10864E+01_JPRB, 0.10755E+01_JPRB, 0.10541E+01_JPRB, 0.10352E+01_JPRB, 0.10046E+01_JPRB, 0.97683E+00_JPRB ,&
&  0.95243E+00_JPRB, 0.93468E+00_JPRB, 0.91989E+00_JPRB, 0.90773E+00_JPRB, 0.89629E+00_JPRB /)
ZBURD(:,  5,  1) = (/ &
&  0.88306E+00_JPRB, 0.85617E+00_JPRB, 0.82814E+00_JPRB, 0.80530E+00_JPRB, 0.78431E+00_JPRB, 0.76941E+00_JPRB, 0.75535E+00_JPRB ,&
&  0.74556E+00_JPRB, 0.73694E+00_JPRB, 0.73190E+00_JPRB, 0.72864E+00_JPRB, 0.72742E+00_JPRB, 0.72920E+00_JPRB, 0.73144E+00_JPRB ,&
&  0.73577E+00_JPRB, 0.73993E+00_JPRB, 0.74561E+00_JPRB, 0.75026E+00_JPRB, 0.75477E+00_JPRB, 0.75328E+00_JPRB, 0.75071E+00_JPRB ,&
&  0.74230E+00_JPRB, 0.72906E+00_JPRB, 0.67580E+00_JPRB, 0.64247E+00_JPRB, 0.59741E+00_JPRB, 0.56138E+00_JPRB, 0.54263E+00_JPRB ,&
&  0.52264E+00_JPRB, 0.51149E+00_JPRB, 0.49188E+00_JPRB, 0.47221E+00_JPRB, 0.45410E+00_JPRB, 0.43249E+00_JPRB, 0.41680E+00_JPRB ,&
&  0.40203E+00_JPRB, 0.38620E+00_JPRB, 0.37867E+00_JPRB, 0.37936E+00_JPRB, 0.38891E+00_JPRB, 0.41819E+00_JPRB, 0.46346E+00_JPRB ,&
&  0.54676E+00_JPRB, 0.64205E+00_JPRB, 0.77079E+00_JPRB, 0.81370E+00_JPRB, 0.81943E+00_JPRB, 0.85130E+00_JPRB, 0.87951E+00_JPRB ,&
&  0.89884E+00_JPRB, 0.94223E+00_JPRB, 0.98498E+00_JPRB, 0.10104E+01_JPRB, 0.10293E+01_JPRB, 0.10401E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10747E+01_JPRB, 0.10828E+01_JPRB, 0.10864E+01_JPRB, 0.10902E+01_JPRB, 0.10933E+01_JPRB, 0.11086E+01_JPRB ,&
&  0.11214E+01_JPRB, 0.11564E+01_JPRB, 0.11878E+01_JPRB, 0.12392E+01_JPRB, 0.12617E+01_JPRB, 0.12897E+01_JPRB, 0.12986E+01_JPRB ,&
&  0.13120E+01_JPRB, 0.13203E+01_JPRB, 0.13247E+01_JPRB, 0.13101E+01_JPRB, 0.13084E+01_JPRB, 0.13188E+01_JPRB, 0.13160E+01_JPRB ,&
&  0.13081E+01_JPRB, 0.12990E+01_JPRB, 0.12882E+01_JPRB, 0.12759E+01_JPRB, 0.12650E+01_JPRB, 0.12601E+01_JPRB, 0.12516E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.12347E+01_JPRB, 0.12177E+01_JPRB, 0.11928E+01_JPRB, 0.11646E+01_JPRB, 0.11249E+01_JPRB, 0.10746E+01_JPRB ,&
&  0.10302E+01_JPRB, 0.99612E+00_JPRB, 0.96254E+00_JPRB, 0.93622E+00_JPRB, 0.90916E+00_JPRB /)
ZBURD(:,  6,  1) = (/ &
&  0.80343E+00_JPRB, 0.77003E+00_JPRB, 0.74117E+00_JPRB, 0.71978E+00_JPRB, 0.70037E+00_JPRB, 0.68752E+00_JPRB, 0.67577E+00_JPRB ,&
&  0.67001E+00_JPRB, 0.66450E+00_JPRB, 0.66377E+00_JPRB, 0.66325E+00_JPRB, 0.66643E+00_JPRB, 0.67205E+00_JPRB, 0.68004E+00_JPRB ,&
&  0.69050E+00_JPRB, 0.69968E+00_JPRB, 0.70954E+00_JPRB, 0.71387E+00_JPRB, 0.71831E+00_JPRB, 0.71865E+00_JPRB, 0.72066E+00_JPRB ,&
&  0.71210E+00_JPRB, 0.70032E+00_JPRB, 0.66199E+00_JPRB, 0.62626E+00_JPRB, 0.57848E+00_JPRB, 0.54312E+00_JPRB, 0.51905E+00_JPRB ,&
&  0.52031E+00_JPRB, 0.47996E+00_JPRB, 0.47485E+00_JPRB, 0.44522E+00_JPRB, 0.41633E+00_JPRB, 0.39215E+00_JPRB, 0.37034E+00_JPRB ,&
&  0.35477E+00_JPRB, 0.33248E+00_JPRB, 0.32244E+00_JPRB, 0.32180E+00_JPRB, 0.32624E+00_JPRB, 0.35008E+00_JPRB, 0.38658E+00_JPRB ,&
&  0.47217E+00_JPRB, 0.67309E+00_JPRB, 0.74024E+00_JPRB, 0.76923E+00_JPRB, 0.76050E+00_JPRB, 0.76878E+00_JPRB, 0.76806E+00_JPRB ,&
&  0.81089E+00_JPRB, 0.86878E+00_JPRB, 0.92614E+00_JPRB, 0.97002E+00_JPRB, 0.10225E+01_JPRB, 0.10600E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.11299E+01_JPRB, 0.11770E+01_JPRB, 0.12166E+01_JPRB, 0.12467E+01_JPRB, 0.12799E+01_JPRB, 0.13062E+01_JPRB, 0.13447E+01_JPRB ,&
&  0.13673E+01_JPRB, 0.14248E+01_JPRB, 0.14437E+01_JPRB, 0.14704E+01_JPRB, 0.14754E+01_JPRB, 0.14722E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.14888E+01_JPRB, 0.15034E+01_JPRB, 0.15163E+01_JPRB, 0.15142E+01_JPRB, 0.15126E+01_JPRB, 0.14863E+01_JPRB, 0.14741E+01_JPRB ,&
&  0.14702E+01_JPRB, 0.14618E+01_JPRB, 0.14489E+01_JPRB, 0.14282E+01_JPRB, 0.13998E+01_JPRB, 0.13674E+01_JPRB, 0.13454E+01_JPRB ,&
&  0.13190E+01_JPRB, 0.12880E+01_JPRB, 0.12550E+01_JPRB, 0.12240E+01_JPRB, 0.11945E+01_JPRB, 0.11606E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.97650E+00_JPRB, 0.92381E+00_JPRB, 0.88221E+00_JPRB, 0.84062E+00_JPRB /)
ZBURD(:,  7,  1) = (/ &
&  0.73390E+00_JPRB, 0.69693E+00_JPRB, 0.66410E+00_JPRB, 0.64258E+00_JPRB, 0.62276E+00_JPRB, 0.61229E+00_JPRB, 0.60161E+00_JPRB ,&
&  0.59649E+00_JPRB, 0.59251E+00_JPRB, 0.59141E+00_JPRB, 0.59198E+00_JPRB, 0.59522E+00_JPRB, 0.60034E+00_JPRB, 0.60880E+00_JPRB ,&
&  0.62042E+00_JPRB, 0.63537E+00_JPRB, 0.65418E+00_JPRB, 0.66736E+00_JPRB, 0.67987E+00_JPRB, 0.68682E+00_JPRB, 0.69254E+00_JPRB ,&
&  0.68807E+00_JPRB, 0.68284E+00_JPRB, 0.65117E+00_JPRB, 0.61605E+00_JPRB, 0.58373E+00_JPRB, 0.55172E+00_JPRB, 0.54220E+00_JPRB ,&
&  0.53635E+00_JPRB, 0.52653E+00_JPRB, 0.51105E+00_JPRB, 0.50108E+00_JPRB, 0.48576E+00_JPRB, 0.41518E+00_JPRB, 0.39763E+00_JPRB ,&
&  0.36429E+00_JPRB, 0.32796E+00_JPRB, 0.31484E+00_JPRB, 0.30740E+00_JPRB, 0.30959E+00_JPRB, 0.32478E+00_JPRB, 0.35383E+00_JPRB ,&
&  0.43353E+00_JPRB, 0.65618E+00_JPRB, 0.70700E+00_JPRB, 0.72470E+00_JPRB, 0.69882E+00_JPRB, 0.70283E+00_JPRB, 0.71060E+00_JPRB ,&
&  0.74276E+00_JPRB, 0.77459E+00_JPRB, 0.81488E+00_JPRB, 0.85097E+00_JPRB, 0.88981E+00_JPRB, 0.91590E+00_JPRB, 0.96394E+00_JPRB ,&
&  0.10413E+01_JPRB, 0.11399E+01_JPRB, 0.12310E+01_JPRB, 0.13226E+01_JPRB, 0.14107E+01_JPRB, 0.15569E+01_JPRB, 0.17060E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.16825E+01_JPRB, 0.16739E+01_JPRB, 0.16622E+01_JPRB, 0.16584E+01_JPRB, 0.16518E+01_JPRB, 0.16582E+01_JPRB ,&
&  0.16813E+01_JPRB, 0.17034E+01_JPRB, 0.16668E+01_JPRB, 0.16104E+01_JPRB, 0.15811E+01_JPRB, 0.15338E+01_JPRB, 0.15271E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.15434E+01_JPRB, 0.15282E+01_JPRB, 0.14929E+01_JPRB, 0.14559E+01_JPRB, 0.14117E+01_JPRB, 0.13756E+01_JPRB ,&
&  0.13374E+01_JPRB, 0.12931E+01_JPRB, 0.12507E+01_JPRB, 0.12180E+01_JPRB, 0.11899E+01_JPRB, 0.11624E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.10459E+01_JPRB, 0.95710E+00_JPRB, 0.88919E+00_JPRB, 0.82733E+00_JPRB, 0.77516E+00_JPRB /)
ZBURD(:,  8,  1) = (/ &
&  0.67400E+00_JPRB, 0.64055E+00_JPRB, 0.61650E+00_JPRB, 0.60097E+00_JPRB, 0.58827E+00_JPRB, 0.58025E+00_JPRB, 0.57050E+00_JPRB ,&
&  0.56248E+00_JPRB, 0.55572E+00_JPRB, 0.55085E+00_JPRB, 0.54721E+00_JPRB, 0.54613E+00_JPRB, 0.54664E+00_JPRB, 0.54959E+00_JPRB ,&
&  0.55471E+00_JPRB, 0.55620E+00_JPRB, 0.56484E+00_JPRB, 0.58225E+00_JPRB, 0.60778E+00_JPRB, 0.63103E+00_JPRB, 0.65569E+00_JPRB ,&
&  0.66178E+00_JPRB, 0.65517E+00_JPRB, 0.63983E+00_JPRB, 0.61458E+00_JPRB, 0.58727E+00_JPRB, 0.56247E+00_JPRB, 0.54556E+00_JPRB ,&
&  0.53939E+00_JPRB, 0.52970E+00_JPRB, 0.53437E+00_JPRB, 0.53266E+00_JPRB, 0.51307E+00_JPRB, 0.50139E+00_JPRB, 0.42777E+00_JPRB ,&
&  0.37208E+00_JPRB, 0.32818E+00_JPRB, 0.31493E+00_JPRB, 0.30854E+00_JPRB, 0.31585E+00_JPRB, 0.33356E+00_JPRB, 0.37791E+00_JPRB ,&
&  0.46936E+00_JPRB, 0.64617E+00_JPRB, 0.67772E+00_JPRB, 0.69122E+00_JPRB, 0.70122E+00_JPRB, 0.69732E+00_JPRB, 0.69580E+00_JPRB ,&
&  0.71770E+00_JPRB, 0.74154E+00_JPRB, 0.77879E+00_JPRB, 0.81678E+00_JPRB, 0.86487E+00_JPRB, 0.90767E+00_JPRB, 0.94541E+00_JPRB ,&
&  0.10294E+01_JPRB, 0.11122E+01_JPRB, 0.12141E+01_JPRB, 0.13877E+01_JPRB, 0.16557E+01_JPRB, 0.18995E+01_JPRB, 0.19542E+01_JPRB ,&
&  0.19189E+01_JPRB, 0.19052E+01_JPRB, 0.18279E+01_JPRB, 0.17702E+01_JPRB, 0.17430E+01_JPRB, 0.17346E+01_JPRB, 0.17363E+01_JPRB ,&
&  0.17460E+01_JPRB, 0.17811E+01_JPRB, 0.18876E+01_JPRB, 0.18039E+01_JPRB, 0.17535E+01_JPRB, 0.17003E+01_JPRB, 0.16339E+01_JPRB ,&
&  0.15787E+01_JPRB, 0.15484E+01_JPRB, 0.15303E+01_JPRB, 0.14996E+01_JPRB, 0.14603E+01_JPRB, 0.14225E+01_JPRB, 0.13956E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.13121E+01_JPRB, 0.12515E+01_JPRB, 0.12147E+01_JPRB, 0.11831E+01_JPRB, 0.11500E+01_JPRB, 0.11100E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.93594E+00_JPRB, 0.85666E+00_JPRB, 0.78594E+00_JPRB, 0.72282E+00_JPRB /)
ZBURD(:,  9,  1) = (/ &
&  0.65265E+00_JPRB, 0.62317E+00_JPRB, 0.60784E+00_JPRB, 0.60163E+00_JPRB, 0.59719E+00_JPRB, 0.57980E+00_JPRB, 0.56386E+00_JPRB ,&
&  0.55346E+00_JPRB, 0.55140E+00_JPRB, 0.54936E+00_JPRB, 0.54462E+00_JPRB, 0.54141E+00_JPRB, 0.53847E+00_JPRB, 0.53591E+00_JPRB ,&
&  0.53367E+00_JPRB, 0.53396E+00_JPRB, 0.53785E+00_JPRB, 0.54085E+00_JPRB, 0.55112E+00_JPRB, 0.57228E+00_JPRB, 0.60292E+00_JPRB ,&
&  0.63303E+00_JPRB, 0.64414E+00_JPRB, 0.64035E+00_JPRB, 0.62754E+00_JPRB, 0.60049E+00_JPRB, 0.57456E+00_JPRB, 0.55243E+00_JPRB ,&
&  0.53013E+00_JPRB, 0.52368E+00_JPRB, 0.53407E+00_JPRB, 0.54626E+00_JPRB, 0.55906E+00_JPRB, 0.53728E+00_JPRB, 0.46155E+00_JPRB ,&
&  0.39853E+00_JPRB, 0.35249E+00_JPRB, 0.33010E+00_JPRB, 0.32462E+00_JPRB, 0.33663E+00_JPRB, 0.36674E+00_JPRB, 0.41810E+00_JPRB ,&
&  0.51548E+00_JPRB, 0.58477E+00_JPRB, 0.60911E+00_JPRB, 0.63795E+00_JPRB, 0.64103E+00_JPRB, 0.63406E+00_JPRB, 0.62841E+00_JPRB ,&
&  0.63737E+00_JPRB, 0.65835E+00_JPRB, 0.68830E+00_JPRB, 0.71983E+00_JPRB, 0.76727E+00_JPRB, 0.83376E+00_JPRB, 0.97240E+00_JPRB ,&
&  0.12981E+01_JPRB, 0.13910E+01_JPRB, 0.13426E+01_JPRB, 0.15453E+01_JPRB, 0.18222E+01_JPRB, 0.20339E+01_JPRB, 0.21537E+01_JPRB ,&
&  0.21314E+01_JPRB, 0.20141E+01_JPRB, 0.19193E+01_JPRB, 0.18562E+01_JPRB, 0.18163E+01_JPRB, 0.17909E+01_JPRB, 0.17850E+01_JPRB ,&
&  0.18028E+01_JPRB, 0.18476E+01_JPRB, 0.19461E+01_JPRB, 0.16805E+01_JPRB, 0.15756E+01_JPRB, 0.15586E+01_JPRB, 0.15253E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.14660E+01_JPRB, 0.14750E+01_JPRB, 0.14929E+01_JPRB, 0.14050E+01_JPRB, 0.13243E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12294E+01_JPRB, 0.11966E+01_JPRB, 0.11677E+01_JPRB, 0.11602E+01_JPRB, 0.11372E+01_JPRB, 0.10814E+01_JPRB, 0.10455E+01_JPRB ,&
&  0.99549E+00_JPRB, 0.90701E+00_JPRB, 0.84466E+00_JPRB, 0.78138E+00_JPRB, 0.70924E+00_JPRB /)
ZBURD(:, 10,  1) = (/ &
&  0.60541E+00_JPRB, 0.60821E+00_JPRB, 0.61895E+00_JPRB, 0.61903E+00_JPRB, 0.61382E+00_JPRB, 0.57573E+00_JPRB, 0.53984E+00_JPRB ,&
&  0.51847E+00_JPRB, 0.51119E+00_JPRB, 0.51402E+00_JPRB, 0.51725E+00_JPRB, 0.51960E+00_JPRB, 0.52317E+00_JPRB, 0.52577E+00_JPRB ,&
&  0.52212E+00_JPRB, 0.51491E+00_JPRB, 0.50902E+00_JPRB, 0.50714E+00_JPRB, 0.51321E+00_JPRB, 0.52960E+00_JPRB, 0.55437E+00_JPRB ,&
&  0.58535E+00_JPRB, 0.60924E+00_JPRB, 0.61783E+00_JPRB, 0.61354E+00_JPRB, 0.60219E+00_JPRB, 0.59047E+00_JPRB, 0.57165E+00_JPRB ,&
&  0.54628E+00_JPRB, 0.53927E+00_JPRB, 0.54437E+00_JPRB, 0.56139E+00_JPRB, 0.58389E+00_JPRB, 0.57297E+00_JPRB, 0.49458E+00_JPRB ,&
&  0.42034E+00_JPRB, 0.37426E+00_JPRB, 0.35545E+00_JPRB, 0.36538E+00_JPRB, 0.40151E+00_JPRB, 0.46354E+00_JPRB, 0.48205E+00_JPRB ,&
&  0.50084E+00_JPRB, 0.52014E+00_JPRB, 0.53587E+00_JPRB, 0.55119E+00_JPRB, 0.56317E+00_JPRB, 0.56944E+00_JPRB, 0.57237E+00_JPRB ,&
&  0.57415E+00_JPRB, 0.58178E+00_JPRB, 0.60029E+00_JPRB, 0.60036E+00_JPRB, 0.64481E+00_JPRB, 0.82964E+00_JPRB, 0.10560E+01_JPRB ,&
&  0.12780E+01_JPRB, 0.13831E+01_JPRB, 0.15237E+01_JPRB, 0.17703E+01_JPRB, 0.19432E+01_JPRB, 0.20606E+01_JPRB, 0.21037E+01_JPRB ,&
&  0.20891E+01_JPRB, 0.19753E+01_JPRB, 0.18829E+01_JPRB, 0.18557E+01_JPRB, 0.18491E+01_JPRB, 0.18500E+01_JPRB, 0.17943E+01_JPRB ,&
&  0.17589E+01_JPRB, 0.16882E+01_JPRB, 0.15589E+01_JPRB, 0.14261E+01_JPRB, 0.13963E+01_JPRB, 0.14344E+01_JPRB, 0.15075E+01_JPRB ,&
&  0.15163E+01_JPRB, 0.15097E+01_JPRB, 0.14994E+01_JPRB, 0.15238E+01_JPRB, 0.13397E+01_JPRB, 0.12153E+01_JPRB, 0.11208E+01_JPRB ,&
&  0.10904E+01_JPRB, 0.10720E+01_JPRB, 0.10389E+01_JPRB, 0.10678E+01_JPRB, 0.10528E+01_JPRB, 0.10128E+01_JPRB, 0.94180E+00_JPRB ,&
&  0.85103E+00_JPRB, 0.77537E+00_JPRB, 0.72015E+00_JPRB, 0.66739E+00_JPRB, 0.63329E+00_JPRB /)
ZBURD(:, 11,  1) = (/ &
&  0.68160E+00_JPRB, 0.67045E+00_JPRB, 0.65576E+00_JPRB, 0.64599E+00_JPRB, 0.62848E+00_JPRB, 0.60764E+00_JPRB, 0.58696E+00_JPRB ,&
&  0.57716E+00_JPRB, 0.56133E+00_JPRB, 0.54506E+00_JPRB, 0.52464E+00_JPRB, 0.50927E+00_JPRB, 0.49676E+00_JPRB, 0.49798E+00_JPRB ,&
&  0.50618E+00_JPRB, 0.51411E+00_JPRB, 0.51316E+00_JPRB, 0.50426E+00_JPRB, 0.49894E+00_JPRB, 0.50510E+00_JPRB, 0.52363E+00_JPRB ,&
&  0.55080E+00_JPRB, 0.58154E+00_JPRB, 0.60159E+00_JPRB, 0.60760E+00_JPRB, 0.60598E+00_JPRB, 0.60203E+00_JPRB, 0.59203E+00_JPRB ,&
&  0.57167E+00_JPRB, 0.56312E+00_JPRB, 0.56937E+00_JPRB, 0.58599E+00_JPRB, 0.61283E+00_JPRB, 0.60303E+00_JPRB, 0.52767E+00_JPRB ,&
&  0.43993E+00_JPRB, 0.41001E+00_JPRB, 0.41581E+00_JPRB, 0.47606E+00_JPRB, 0.49701E+00_JPRB, 0.51200E+00_JPRB, 0.52294E+00_JPRB ,&
&  0.51456E+00_JPRB, 0.50163E+00_JPRB, 0.50702E+00_JPRB, 0.53294E+00_JPRB, 0.53965E+00_JPRB, 0.54373E+00_JPRB, 0.54503E+00_JPRB ,&
&  0.54556E+00_JPRB, 0.55499E+00_JPRB, 0.57328E+00_JPRB, 0.57967E+00_JPRB, 0.72985E+00_JPRB, 0.10304E+01_JPRB, 0.11936E+01_JPRB ,&
&  0.14009E+01_JPRB, 0.15787E+01_JPRB, 0.18237E+01_JPRB, 0.19098E+01_JPRB, 0.19214E+01_JPRB, 0.19683E+01_JPRB, 0.20656E+01_JPRB ,&
&  0.20475E+01_JPRB, 0.20124E+01_JPRB, 0.19593E+01_JPRB, 0.19652E+01_JPRB, 0.19109E+01_JPRB, 0.18416E+01_JPRB, 0.17767E+01_JPRB ,&
&  0.17763E+01_JPRB, 0.17488E+01_JPRB, 0.16617E+01_JPRB, 0.15400E+01_JPRB, 0.14899E+01_JPRB, 0.14670E+01_JPRB, 0.14876E+01_JPRB ,&
&  0.14882E+01_JPRB, 0.15079E+01_JPRB, 0.15120E+01_JPRB, 0.15304E+01_JPRB, 0.13745E+01_JPRB, 0.11826E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.97982E+00_JPRB, 0.92911E+00_JPRB, 0.86808E+00_JPRB, 0.88716E+00_JPRB, 0.90764E+00_JPRB, 0.89786E+00_JPRB, 0.81688E+00_JPRB ,&
&  0.74540E+00_JPRB, 0.70798E+00_JPRB, 0.68302E+00_JPRB, 0.68999E+00_JPRB, 0.69284E+00_JPRB /)
ZBURD(:, 12,  1) = (/ &
&  0.71595E+00_JPRB, 0.70769E+00_JPRB, 0.68752E+00_JPRB, 0.67018E+00_JPRB, 0.66225E+00_JPRB, 0.64217E+00_JPRB, 0.60725E+00_JPRB ,&
&  0.58626E+00_JPRB, 0.55390E+00_JPRB, 0.53322E+00_JPRB, 0.52164E+00_JPRB, 0.51670E+00_JPRB, 0.50850E+00_JPRB, 0.48972E+00_JPRB ,&
&  0.48534E+00_JPRB, 0.49405E+00_JPRB, 0.51158E+00_JPRB, 0.51562E+00_JPRB, 0.51306E+00_JPRB, 0.51180E+00_JPRB, 0.51884E+00_JPRB ,&
&  0.53725E+00_JPRB, 0.56553E+00_JPRB, 0.59554E+00_JPRB, 0.60909E+00_JPRB, 0.60623E+00_JPRB, 0.60145E+00_JPRB, 0.60115E+00_JPRB ,&
&  0.59488E+00_JPRB, 0.59244E+00_JPRB, 0.60093E+00_JPRB, 0.60982E+00_JPRB, 0.64039E+00_JPRB, 0.65831E+00_JPRB, 0.63682E+00_JPRB ,&
&  0.51105E+00_JPRB, 0.49919E+00_JPRB, 0.57439E+00_JPRB, 0.58897E+00_JPRB, 0.59939E+00_JPRB, 0.58874E+00_JPRB, 0.58035E+00_JPRB ,&
&  0.57435E+00_JPRB, 0.57162E+00_JPRB, 0.56813E+00_JPRB, 0.56714E+00_JPRB, 0.56647E+00_JPRB, 0.56766E+00_JPRB, 0.56888E+00_JPRB ,&
&  0.58021E+00_JPRB, 0.54612E+00_JPRB, 0.56007E+00_JPRB, 0.61484E+00_JPRB, 0.84822E+00_JPRB, 0.10458E+01_JPRB, 0.12215E+01_JPRB ,&
&  0.14455E+01_JPRB, 0.16273E+01_JPRB, 0.18316E+01_JPRB, 0.19673E+01_JPRB, 0.20190E+01_JPRB, 0.20414E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.21750E+01_JPRB, 0.21580E+01_JPRB, 0.21331E+01_JPRB, 0.20847E+01_JPRB, 0.19691E+01_JPRB, 0.19283E+01_JPRB, 0.19091E+01_JPRB ,&
&  0.19402E+01_JPRB, 0.19586E+01_JPRB, 0.18708E+01_JPRB, 0.17515E+01_JPRB, 0.16563E+01_JPRB, 0.16513E+01_JPRB, 0.16662E+01_JPRB ,&
&  0.16566E+01_JPRB, 0.16685E+01_JPRB, 0.16346E+01_JPRB, 0.15750E+01_JPRB, 0.14611E+01_JPRB, 0.13394E+01_JPRB, 0.12534E+01_JPRB ,&
&  0.11017E+01_JPRB, 0.96191E+00_JPRB, 0.84405E+00_JPRB, 0.83737E+00_JPRB, 0.85617E+00_JPRB, 0.83336E+00_JPRB, 0.77969E+00_JPRB ,&
&  0.75469E+00_JPRB, 0.73116E+00_JPRB, 0.70250E+00_JPRB, 0.71115E+00_JPRB, 0.71433E+00_JPRB /)
ZBURD(:, 13,  1) = (/ &
&  0.70068E+00_JPRB, 0.69835E+00_JPRB, 0.68103E+00_JPRB, 0.66533E+00_JPRB, 0.66725E+00_JPRB, 0.64865E+00_JPRB, 0.64345E+00_JPRB ,&
&  0.64714E+00_JPRB, 0.63311E+00_JPRB, 0.65769E+00_JPRB, 0.57677E+00_JPRB, 0.52760E+00_JPRB, 0.51666E+00_JPRB, 0.49899E+00_JPRB ,&
&  0.49726E+00_JPRB, 0.51223E+00_JPRB, 0.52415E+00_JPRB, 0.53396E+00_JPRB, 0.54100E+00_JPRB, 0.52759E+00_JPRB, 0.52384E+00_JPRB ,&
&  0.53440E+00_JPRB, 0.55831E+00_JPRB, 0.59317E+00_JPRB, 0.60920E+00_JPRB, 0.61499E+00_JPRB, 0.61535E+00_JPRB, 0.61635E+00_JPRB ,&
&  0.60222E+00_JPRB, 0.61959E+00_JPRB, 0.66040E+00_JPRB, 0.65109E+00_JPRB, 0.66427E+00_JPRB, 0.70922E+00_JPRB, 0.70707E+00_JPRB ,&
&  0.72693E+00_JPRB, 0.66235E+00_JPRB, 0.69474E+00_JPRB, 0.70225E+00_JPRB, 0.69612E+00_JPRB, 0.67157E+00_JPRB, 0.64730E+00_JPRB ,&
&  0.63528E+00_JPRB, 0.63486E+00_JPRB, 0.63904E+00_JPRB, 0.64492E+00_JPRB, 0.64402E+00_JPRB, 0.66247E+00_JPRB, 0.68465E+00_JPRB ,&
&  0.71902E+00_JPRB, 0.66694E+00_JPRB, 0.79602E+00_JPRB, 0.86827E+00_JPRB, 0.10340E+01_JPRB, 0.12414E+01_JPRB, 0.14156E+01_JPRB ,&
&  0.18187E+01_JPRB, 0.18496E+01_JPRB, 0.19996E+01_JPRB, 0.20619E+01_JPRB, 0.21201E+01_JPRB, 0.22073E+01_JPRB, 0.23174E+01_JPRB ,&
&  0.24566E+01_JPRB, 0.23107E+01_JPRB, 0.24025E+01_JPRB, 0.22539E+01_JPRB, 0.21511E+01_JPRB, 0.21483E+01_JPRB, 0.21537E+01_JPRB ,&
&  0.21904E+01_JPRB, 0.22105E+01_JPRB, 0.21851E+01_JPRB, 0.19326E+01_JPRB, 0.18870E+01_JPRB, 0.18734E+01_JPRB, 0.17479E+01_JPRB ,&
&  0.17764E+01_JPRB, 0.17507E+01_JPRB, 0.16271E+01_JPRB, 0.17015E+01_JPRB, 0.15146E+01_JPRB, 0.13884E+01_JPRB, 0.13330E+01_JPRB ,&
&  0.12576E+01_JPRB, 0.10229E+01_JPRB, 0.95877E+00_JPRB, 0.91837E+00_JPRB, 0.86168E+00_JPRB, 0.80276E+00_JPRB, 0.81179E+00_JPRB ,&
&  0.74777E+00_JPRB, 0.73631E+00_JPRB, 0.74394E+00_JPRB, 0.72290E+00_JPRB, 0.70521E+00_JPRB /)
ZBURD(:, 14,  1) = (/ &
&  0.70290E+00_JPRB, 0.68923E+00_JPRB, 0.67362E+00_JPRB, 0.66137E+00_JPRB, 0.67641E+00_JPRB, 0.68140E+00_JPRB, 0.67695E+00_JPRB ,&
&  0.69516E+00_JPRB, 0.68689E+00_JPRB, 0.68352E+00_JPRB, 0.67029E+00_JPRB, 0.66011E+00_JPRB, 0.60941E+00_JPRB, 0.52109E+00_JPRB ,&
&  0.49825E+00_JPRB, 0.51676E+00_JPRB, 0.55523E+00_JPRB, 0.57542E+00_JPRB, 0.56498E+00_JPRB, 0.54489E+00_JPRB, 0.53541E+00_JPRB ,&
&  0.54455E+00_JPRB, 0.55853E+00_JPRB, 0.58647E+00_JPRB, 0.61104E+00_JPRB, 0.62514E+00_JPRB, 0.63536E+00_JPRB, 0.65001E+00_JPRB ,&
&  0.64498E+00_JPRB, 0.67988E+00_JPRB, 0.74241E+00_JPRB, 0.68321E+00_JPRB, 0.74995E+00_JPRB, 0.82827E+00_JPRB, 0.84547E+00_JPRB ,&
&  0.88823E+00_JPRB, 0.88214E+00_JPRB, 0.84358E+00_JPRB, 0.82235E+00_JPRB, 0.77881E+00_JPRB, 0.73901E+00_JPRB, 0.72045E+00_JPRB ,&
&  0.71559E+00_JPRB, 0.71571E+00_JPRB, 0.71986E+00_JPRB, 0.72397E+00_JPRB, 0.73348E+00_JPRB, 0.81828E+00_JPRB, 0.91816E+00_JPRB ,&
&  0.98103E+00_JPRB, 0.10697E+01_JPRB, 0.12289E+01_JPRB, 0.12562E+01_JPRB, 0.13693E+01_JPRB, 0.15278E+01_JPRB, 0.16129E+01_JPRB ,&
&  0.18511E+01_JPRB, 0.18978E+01_JPRB, 0.21651E+01_JPRB, 0.23723E+01_JPRB, 0.25672E+01_JPRB, 0.26798E+01_JPRB, 0.27630E+01_JPRB ,&
&  0.29497E+01_JPRB, 0.30365E+01_JPRB, 0.27819E+01_JPRB, 0.26205E+01_JPRB, 0.24818E+01_JPRB, 0.24686E+01_JPRB, 0.24043E+01_JPRB ,&
&  0.24265E+01_JPRB, 0.24874E+01_JPRB, 0.24829E+01_JPRB, 0.24323E+01_JPRB, 0.23165E+01_JPRB, 0.21421E+01_JPRB, 0.19655E+01_JPRB ,&
&  0.18621E+01_JPRB, 0.16552E+01_JPRB, 0.15362E+01_JPRB, 0.15352E+01_JPRB, 0.15228E+01_JPRB, 0.13231E+01_JPRB, 0.13035E+01_JPRB ,&
&  0.13132E+01_JPRB, 0.11522E+01_JPRB, 0.10755E+01_JPRB, 0.96347E+00_JPRB, 0.91403E+00_JPRB, 0.86087E+00_JPRB, 0.77299E+00_JPRB ,&
&  0.70334E+00_JPRB, 0.74426E+00_JPRB, 0.73677E+00_JPRB, 0.71790E+00_JPRB, 0.70917E+00_JPRB /)
ZBURD(:, 15,  1) = (/ &
&  0.71376E+00_JPRB, 0.68806E+00_JPRB, 0.66509E+00_JPRB, 0.65927E+00_JPRB, 0.67710E+00_JPRB, 0.68382E+00_JPRB, 0.69989E+00_JPRB ,&
&  0.71005E+00_JPRB, 0.71156E+00_JPRB, 0.70451E+00_JPRB, 0.67853E+00_JPRB, 0.67508E+00_JPRB, 0.69366E+00_JPRB, 0.67571E+00_JPRB ,&
&  0.61190E+00_JPRB, 0.62340E+00_JPRB, 0.61332E+00_JPRB, 0.62747E+00_JPRB, 0.63552E+00_JPRB, 0.60570E+00_JPRB, 0.59013E+00_JPRB ,&
&  0.60739E+00_JPRB, 0.59649E+00_JPRB, 0.60018E+00_JPRB, 0.61285E+00_JPRB, 0.63170E+00_JPRB, 0.65939E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.69608E+00_JPRB, 0.77051E+00_JPRB, 0.84756E+00_JPRB, 0.82246E+00_JPRB, 0.93063E+00_JPRB, 0.10529E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.11162E+01_JPRB, 0.10582E+01_JPRB, 0.97950E+00_JPRB, 0.91473E+00_JPRB, 0.85459E+00_JPRB, 0.82635E+00_JPRB, 0.81901E+00_JPRB ,&
&  0.81728E+00_JPRB, 0.81289E+00_JPRB, 0.80852E+00_JPRB, 0.81944E+00_JPRB, 0.83879E+00_JPRB, 0.10167E+01_JPRB, 0.12457E+01_JPRB ,&
&  0.14423E+01_JPRB, 0.15951E+01_JPRB, 0.17568E+01_JPRB, 0.18727E+01_JPRB, 0.19881E+01_JPRB, 0.20445E+01_JPRB, 0.20139E+01_JPRB ,&
&  0.21501E+01_JPRB, 0.21544E+01_JPRB, 0.27764E+01_JPRB, 0.28996E+01_JPRB, 0.29638E+01_JPRB, 0.31390E+01_JPRB, 0.31252E+01_JPRB ,&
&  0.31344E+01_JPRB, 0.29467E+01_JPRB, 0.28910E+01_JPRB, 0.27946E+01_JPRB, 0.27208E+01_JPRB, 0.27879E+01_JPRB, 0.26882E+01_JPRB ,&
&  0.26995E+01_JPRB, 0.26872E+01_JPRB, 0.23918E+01_JPRB, 0.21672E+01_JPRB, 0.18755E+01_JPRB, 0.19989E+01_JPRB, 0.18821E+01_JPRB ,&
&  0.17432E+01_JPRB, 0.14432E+01_JPRB, 0.15333E+01_JPRB, 0.15654E+01_JPRB, 0.15854E+01_JPRB, 0.15695E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.14785E+01_JPRB, 0.14568E+01_JPRB, 0.12493E+01_JPRB, 0.10883E+01_JPRB, 0.99506E+00_JPRB, 0.90581E+00_JPRB, 0.77433E+00_JPRB ,&
&  0.73979E+00_JPRB, 0.77091E+00_JPRB, 0.77018E+00_JPRB, 0.75774E+00_JPRB, 0.73659E+00_JPRB /)
ZBURD(:, 16,  1) = (/ &
&  0.73714E+00_JPRB, 0.70592E+00_JPRB, 0.67492E+00_JPRB, 0.67186E+00_JPRB, 0.68392E+00_JPRB, 0.68574E+00_JPRB, 0.70363E+00_JPRB ,&
&  0.72504E+00_JPRB, 0.72700E+00_JPRB, 0.69902E+00_JPRB, 0.67952E+00_JPRB, 0.68041E+00_JPRB, 0.71558E+00_JPRB, 0.78351E+00_JPRB ,&
&  0.71494E+00_JPRB, 0.72004E+00_JPRB, 0.65779E+00_JPRB, 0.67994E+00_JPRB, 0.80064E+00_JPRB, 0.73205E+00_JPRB, 0.72627E+00_JPRB ,&
&  0.70241E+00_JPRB, 0.69289E+00_JPRB, 0.66700E+00_JPRB, 0.65066E+00_JPRB, 0.65273E+00_JPRB, 0.68651E+00_JPRB, 0.75627E+00_JPRB ,&
&  0.81262E+00_JPRB, 0.97158E+00_JPRB, 0.11263E+01_JPRB, 0.11134E+01_JPRB, 0.12236E+01_JPRB, 0.13005E+01_JPRB, 0.13543E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.11828E+01_JPRB, 0.10608E+01_JPRB, 0.99104E+00_JPRB, 0.95027E+00_JPRB, 0.93586E+00_JPRB, 0.92611E+00_JPRB ,&
&  0.92049E+00_JPRB, 0.92360E+00_JPRB, 0.92503E+00_JPRB, 0.95402E+00_JPRB, 0.99181E+00_JPRB, 0.12072E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.20860E+01_JPRB, 0.22865E+01_JPRB, 0.25380E+01_JPRB, 0.30010E+01_JPRB, 0.31036E+01_JPRB, 0.30058E+01_JPRB, 0.27587E+01_JPRB ,&
&  0.26298E+01_JPRB, 0.25251E+01_JPRB, 0.27356E+01_JPRB, 0.29976E+01_JPRB, 0.30276E+01_JPRB, 0.33374E+01_JPRB, 0.32988E+01_JPRB ,&
&  0.30190E+01_JPRB, 0.28461E+01_JPRB, 0.27364E+01_JPRB, 0.27071E+01_JPRB, 0.26298E+01_JPRB, 0.27328E+01_JPRB, 0.27308E+01_JPRB ,&
&  0.27002E+01_JPRB, 0.23776E+01_JPRB, 0.18179E+01_JPRB, 0.17040E+01_JPRB, 0.13209E+01_JPRB, 0.13645E+01_JPRB, 0.17322E+01_JPRB ,&
&  0.17116E+01_JPRB, 0.16815E+01_JPRB, 0.16722E+01_JPRB, 0.16964E+01_JPRB, 0.16897E+01_JPRB, 0.19238E+01_JPRB, 0.19459E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.16653E+01_JPRB, 0.15482E+01_JPRB, 0.13845E+01_JPRB, 0.12376E+01_JPRB, 0.10570E+01_JPRB, 0.92909E+00_JPRB ,&
&  0.88788E+00_JPRB, 0.84798E+00_JPRB, 0.82941E+00_JPRB, 0.79784E+00_JPRB, 0.76047E+00_JPRB /)
ZBURD(:, 17,  1) = (/ &
&  0.78039E+00_JPRB, 0.75094E+00_JPRB, 0.70690E+00_JPRB, 0.69814E+00_JPRB, 0.69461E+00_JPRB, 0.69575E+00_JPRB, 0.71616E+00_JPRB ,&
&  0.73949E+00_JPRB, 0.72566E+00_JPRB, 0.69291E+00_JPRB, 0.68528E+00_JPRB, 0.70026E+00_JPRB, 0.73840E+00_JPRB, 0.81253E+00_JPRB ,&
&  0.93566E+00_JPRB, 0.94493E+00_JPRB, 0.82712E+00_JPRB, 0.69773E+00_JPRB, 0.77848E+00_JPRB, 0.77259E+00_JPRB, 0.84567E+00_JPRB ,&
&  0.81962E+00_JPRB, 0.88545E+00_JPRB, 0.84431E+00_JPRB, 0.82577E+00_JPRB, 0.81886E+00_JPRB, 0.87140E+00_JPRB, 0.99060E+00_JPRB ,&
&  0.11576E+01_JPRB, 0.14348E+01_JPRB, 0.16934E+01_JPRB, 0.16924E+01_JPRB, 0.17632E+01_JPRB, 0.15973E+01_JPRB, 0.15747E+01_JPRB ,&
&  0.14687E+01_JPRB, 0.13146E+01_JPRB, 0.11742E+01_JPRB, 0.10921E+01_JPRB, 0.10461E+01_JPRB, 0.10176E+01_JPRB, 0.10181E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.10546E+01_JPRB, 0.10317E+01_JPRB, 0.10231E+01_JPRB, 0.10782E+01_JPRB, 0.12297E+01_JPRB, 0.14953E+01_JPRB ,&
&  0.20012E+01_JPRB, 0.23218E+01_JPRB, 0.25255E+01_JPRB, 0.30985E+01_JPRB, 0.34076E+01_JPRB, 0.34728E+01_JPRB, 0.33799E+01_JPRB ,&
&  0.32261E+01_JPRB, 0.30738E+01_JPRB, 0.31772E+01_JPRB, 0.32587E+01_JPRB, 0.33578E+01_JPRB, 0.34590E+01_JPRB, 0.33507E+01_JPRB ,&
&  0.29890E+01_JPRB, 0.27811E+01_JPRB, 0.27155E+01_JPRB, 0.25216E+01_JPRB, 0.24449E+01_JPRB, 0.22930E+01_JPRB, 0.23788E+01_JPRB ,&
&  0.24159E+01_JPRB, 0.14221E+01_JPRB, 0.11391E+01_JPRB, 0.13554E+01_JPRB, 0.11473E+01_JPRB, 0.12026E+01_JPRB, 0.14185E+01_JPRB ,&
&  0.14223E+01_JPRB, 0.14367E+01_JPRB, 0.16200E+01_JPRB, 0.17313E+01_JPRB, 0.18894E+01_JPRB, 0.22349E+01_JPRB, 0.22206E+01_JPRB ,&
&  0.20641E+01_JPRB, 0.18711E+01_JPRB, 0.18614E+01_JPRB, 0.17362E+01_JPRB, 0.15581E+01_JPRB, 0.13078E+01_JPRB, 0.11219E+01_JPRB ,&
&  0.10121E+01_JPRB, 0.95196E+00_JPRB, 0.90764E+00_JPRB, 0.85892E+00_JPRB, 0.80611E+00_JPRB /)
ZBURD(:, 18,  1) = (/ &
&  0.85105E+00_JPRB, 0.81104E+00_JPRB, 0.74176E+00_JPRB, 0.71819E+00_JPRB, 0.71700E+00_JPRB, 0.73276E+00_JPRB, 0.75282E+00_JPRB ,&
&  0.75819E+00_JPRB, 0.72947E+00_JPRB, 0.71124E+00_JPRB, 0.71612E+00_JPRB, 0.73352E+00_JPRB, 0.75136E+00_JPRB, 0.77718E+00_JPRB ,&
&  0.82584E+00_JPRB, 0.92633E+00_JPRB, 0.78596E+00_JPRB, 0.75417E+00_JPRB, 0.68209E+00_JPRB, 0.74902E+00_JPRB, 0.84876E+00_JPRB ,&
&  0.96698E+00_JPRB, 0.97921E+00_JPRB, 0.95099E+00_JPRB, 0.97965E+00_JPRB, 0.10020E+01_JPRB, 0.10826E+01_JPRB, 0.13297E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.20297E+01_JPRB, 0.21451E+01_JPRB, 0.23054E+01_JPRB, 0.21104E+01_JPRB, 0.17946E+01_JPRB, 0.17484E+01_JPRB ,&
&  0.15809E+01_JPRB, 0.13862E+01_JPRB, 0.12207E+01_JPRB, 0.11279E+01_JPRB, 0.10748E+01_JPRB, 0.10355E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10670E+01_JPRB, 0.10790E+01_JPRB, 0.10752E+01_JPRB, 0.10770E+01_JPRB, 0.11700E+01_JPRB, 0.13249E+01_JPRB, 0.13909E+01_JPRB ,&
&  0.18006E+01_JPRB, 0.18565E+01_JPRB, 0.16654E+01_JPRB, 0.24380E+01_JPRB, 0.36655E+01_JPRB, 0.36823E+01_JPRB, 0.32490E+01_JPRB ,&
&  0.34292E+01_JPRB, 0.34246E+01_JPRB, 0.35466E+01_JPRB, 0.36780E+01_JPRB, 0.36901E+01_JPRB, 0.35558E+01_JPRB, 0.33281E+01_JPRB ,&
&  0.30293E+01_JPRB, 0.28421E+01_JPRB, 0.27243E+01_JPRB, 0.27477E+01_JPRB, 0.26241E+01_JPRB, 0.25373E+01_JPRB, 0.24724E+01_JPRB ,&
&  0.21056E+01_JPRB, 0.17816E+01_JPRB, 0.12298E+01_JPRB, 0.12724E+01_JPRB, 0.99597E+00_JPRB, 0.10214E+01_JPRB, 0.12190E+01_JPRB ,&
&  0.12162E+01_JPRB, 0.13403E+01_JPRB, 0.15306E+01_JPRB, 0.16875E+01_JPRB, 0.22367E+01_JPRB, 0.25567E+01_JPRB, 0.26817E+01_JPRB ,&
&  0.25708E+01_JPRB, 0.21959E+01_JPRB, 0.21940E+01_JPRB, 0.19892E+01_JPRB, 0.18448E+01_JPRB, 0.16133E+01_JPRB, 0.13734E+01_JPRB ,&
&  0.12026E+01_JPRB, 0.11033E+01_JPRB, 0.10329E+01_JPRB, 0.95950E+00_JPRB, 0.88866E+00_JPRB /)
ZBURD(:, 19,  1) = (/ &
&  0.88477E+00_JPRB, 0.82149E+00_JPRB, 0.73208E+00_JPRB, 0.70937E+00_JPRB, 0.71640E+00_JPRB, 0.75755E+00_JPRB, 0.78420E+00_JPRB ,&
&  0.77250E+00_JPRB, 0.74289E+00_JPRB, 0.72803E+00_JPRB, 0.73032E+00_JPRB, 0.73541E+00_JPRB, 0.72968E+00_JPRB, 0.73685E+00_JPRB ,&
&  0.74887E+00_JPRB, 0.77755E+00_JPRB, 0.69709E+00_JPRB, 0.64835E+00_JPRB, 0.61485E+00_JPRB, 0.67495E+00_JPRB, 0.79033E+00_JPRB ,&
&  0.89410E+00_JPRB, 0.96964E+00_JPRB, 0.10648E+01_JPRB, 0.11408E+01_JPRB, 0.11707E+01_JPRB, 0.13410E+01_JPRB, 0.17231E+01_JPRB ,&
&  0.23842E+01_JPRB, 0.23556E+01_JPRB, 0.25103E+01_JPRB, 0.23831E+01_JPRB, 0.21566E+01_JPRB, 0.19072E+01_JPRB, 0.16837E+01_JPRB ,&
&  0.14783E+01_JPRB, 0.12754E+01_JPRB, 0.11267E+01_JPRB, 0.10347E+01_JPRB, 0.10184E+01_JPRB, 0.10386E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.11452E+01_JPRB, 0.11262E+01_JPRB, 0.11873E+01_JPRB, 0.13399E+01_JPRB, 0.14513E+01_JPRB, 0.15283E+01_JPRB ,&
&  0.14862E+01_JPRB, 0.14938E+01_JPRB, 0.21214E+01_JPRB, 0.25209E+01_JPRB, 0.32638E+01_JPRB, 0.37186E+01_JPRB, 0.36663E+01_JPRB ,&
&  0.37493E+01_JPRB, 0.36554E+01_JPRB, 0.36737E+01_JPRB, 0.30781E+01_JPRB, 0.35523E+01_JPRB, 0.34491E+01_JPRB, 0.31886E+01_JPRB ,&
&  0.29452E+01_JPRB, 0.27896E+01_JPRB, 0.27250E+01_JPRB, 0.27021E+01_JPRB, 0.27379E+01_JPRB, 0.28078E+01_JPRB, 0.21685E+01_JPRB ,&
&  0.16848E+01_JPRB, 0.18616E+01_JPRB, 0.16832E+01_JPRB, 0.13968E+01_JPRB, 0.11624E+01_JPRB, 0.12056E+01_JPRB, 0.12524E+01_JPRB ,&
&  0.12914E+01_JPRB, 0.13365E+01_JPRB, 0.15102E+01_JPRB, 0.19441E+01_JPRB, 0.26909E+01_JPRB, 0.31082E+01_JPRB, 0.31030E+01_JPRB ,&
&  0.27374E+01_JPRB, 0.27692E+01_JPRB, 0.25321E+01_JPRB, 0.22086E+01_JPRB, 0.19736E+01_JPRB, 0.17757E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.12711E+01_JPRB, 0.11667E+01_JPRB, 0.10601E+01_JPRB, 0.95104E+00_JPRB /)
ZBURD(:, 20,  1) = (/ &
&  0.90656E+00_JPRB, 0.83740E+00_JPRB, 0.75363E+00_JPRB, 0.73482E+00_JPRB, 0.76382E+00_JPRB, 0.80545E+00_JPRB, 0.81314E+00_JPRB ,&
&  0.76635E+00_JPRB, 0.73805E+00_JPRB, 0.71944E+00_JPRB, 0.72567E+00_JPRB, 0.72987E+00_JPRB, 0.71746E+00_JPRB, 0.70849E+00_JPRB ,&
&  0.74282E+00_JPRB, 0.70848E+00_JPRB, 0.64001E+00_JPRB, 0.69047E+00_JPRB, 0.72422E+00_JPRB, 0.66731E+00_JPRB, 0.74691E+00_JPRB ,&
&  0.85684E+00_JPRB, 0.97994E+00_JPRB, 0.10742E+01_JPRB, 0.12493E+01_JPRB, 0.14757E+01_JPRB, 0.18261E+01_JPRB, 0.22505E+01_JPRB ,&
&  0.24057E+01_JPRB, 0.25545E+01_JPRB, 0.26217E+01_JPRB, 0.24318E+01_JPRB, 0.20276E+01_JPRB, 0.16945E+01_JPRB, 0.15043E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.11301E+01_JPRB, 0.10343E+01_JPRB, 0.10244E+01_JPRB, 0.10766E+01_JPRB, 0.11353E+01_JPRB, 0.11961E+01_JPRB ,&
&  0.11739E+01_JPRB, 0.12011E+01_JPRB, 0.12490E+01_JPRB, 0.14274E+01_JPRB, 0.12106E+01_JPRB, 0.12318E+01_JPRB, 0.12413E+01_JPRB ,&
&  0.17462E+01_JPRB, 0.18943E+01_JPRB, 0.20500E+01_JPRB, 0.26591E+01_JPRB, 0.24514E+01_JPRB, 0.28415E+01_JPRB, 0.36033E+01_JPRB ,&
&  0.39386E+01_JPRB, 0.39725E+01_JPRB, 0.40266E+01_JPRB, 0.25968E+01_JPRB, 0.22491E+01_JPRB, 0.30392E+01_JPRB, 0.29677E+01_JPRB ,&
&  0.27625E+01_JPRB, 0.27822E+01_JPRB, 0.26601E+01_JPRB, 0.27457E+01_JPRB, 0.20534E+01_JPRB, 0.17573E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12021E+01_JPRB, 0.14387E+01_JPRB, 0.13249E+01_JPRB, 0.13559E+01_JPRB, 0.13364E+01_JPRB, 0.12825E+01_JPRB ,&
&  0.13645E+01_JPRB, 0.15097E+01_JPRB, 0.18220E+01_JPRB, 0.25477E+01_JPRB, 0.33987E+01_JPRB, 0.31632E+01_JPRB, 0.34733E+01_JPRB ,&
&  0.33921E+01_JPRB, 0.30475E+01_JPRB, 0.25351E+01_JPRB, 0.23084E+01_JPRB, 0.21031E+01_JPRB, 0.19219E+01_JPRB, 0.17382E+01_JPRB ,&
&  0.15452E+01_JPRB, 0.13974E+01_JPRB, 0.12600E+01_JPRB, 0.11169E+01_JPRB, 0.99025E+00_JPRB /)
ZBURD(:, 21,  1) = (/ &
&  0.95991E+00_JPRB, 0.88922E+00_JPRB, 0.79584E+00_JPRB, 0.78905E+00_JPRB, 0.82773E+00_JPRB, 0.84075E+00_JPRB, 0.80057E+00_JPRB ,&
&  0.76233E+00_JPRB, 0.72758E+00_JPRB, 0.71725E+00_JPRB, 0.71212E+00_JPRB, 0.70619E+00_JPRB, 0.70786E+00_JPRB, 0.71164E+00_JPRB ,&
&  0.77331E+00_JPRB, 0.78147E+00_JPRB, 0.70275E+00_JPRB, 0.65621E+00_JPRB, 0.73883E+00_JPRB, 0.69002E+00_JPRB, 0.68090E+00_JPRB ,&
&  0.86935E+00_JPRB, 0.10433E+01_JPRB, 0.12250E+01_JPRB, 0.15372E+01_JPRB, 0.18906E+01_JPRB, 0.24777E+01_JPRB, 0.27630E+01_JPRB ,&
&  0.27856E+01_JPRB, 0.27252E+01_JPRB, 0.24333E+01_JPRB, 0.20528E+01_JPRB, 0.16824E+01_JPRB, 0.14535E+01_JPRB, 0.12786E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.10109E+01_JPRB, 0.99301E+00_JPRB, 0.10312E+01_JPRB, 0.10995E+01_JPRB, 0.11426E+01_JPRB, 0.11654E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.13462E+01_JPRB, 0.14083E+01_JPRB, 0.14178E+01_JPRB, 0.13891E+01_JPRB, 0.12497E+01_JPRB, 0.14191E+01_JPRB ,&
&  0.17130E+01_JPRB, 0.17304E+01_JPRB, 0.19469E+01_JPRB, 0.23813E+01_JPRB, 0.28580E+01_JPRB, 0.28728E+01_JPRB, 0.35736E+01_JPRB ,&
&  0.29426E+01_JPRB, 0.24597E+01_JPRB, 0.21104E+01_JPRB, 0.15812E+01_JPRB, 0.18995E+01_JPRB, 0.27949E+01_JPRB, 0.28474E+01_JPRB ,&
&  0.27171E+01_JPRB, 0.26771E+01_JPRB, 0.26191E+01_JPRB, 0.21881E+01_JPRB, 0.16519E+01_JPRB, 0.13234E+01_JPRB, 0.16758E+01_JPRB ,&
&  0.17855E+01_JPRB, 0.18223E+01_JPRB, 0.17350E+01_JPRB, 0.15041E+01_JPRB, 0.13093E+01_JPRB, 0.15287E+01_JPRB, 0.15821E+01_JPRB ,&
&  0.17583E+01_JPRB, 0.19498E+01_JPRB, 0.29217E+01_JPRB, 0.38388E+01_JPRB, 0.40385E+01_JPRB, 0.39025E+01_JPRB, 0.41237E+01_JPRB ,&
&  0.36034E+01_JPRB, 0.32832E+01_JPRB, 0.28352E+01_JPRB, 0.24391E+01_JPRB, 0.21743E+01_JPRB, 0.19841E+01_JPRB, 0.18292E+01_JPRB ,&
&  0.16505E+01_JPRB, 0.15128E+01_JPRB, 0.13690E+01_JPRB, 0.11931E+01_JPRB, 0.10494E+01_JPRB /)
ZBURD(:, 22,  1) = (/ &
&  0.10291E+01_JPRB, 0.93950E+00_JPRB, 0.83947E+00_JPRB, 0.83457E+00_JPRB, 0.87204E+00_JPRB, 0.84521E+00_JPRB, 0.78339E+00_JPRB ,&
&  0.76490E+00_JPRB, 0.73265E+00_JPRB, 0.72162E+00_JPRB, 0.70724E+00_JPRB, 0.69291E+00_JPRB, 0.69198E+00_JPRB, 0.72967E+00_JPRB ,&
&  0.82931E+00_JPRB, 0.96243E+00_JPRB, 0.86192E+00_JPRB, 0.76840E+00_JPRB, 0.74375E+00_JPRB, 0.72557E+00_JPRB, 0.63913E+00_JPRB ,&
&  0.91666E+00_JPRB, 0.11748E+01_JPRB, 0.13994E+01_JPRB, 0.18179E+01_JPRB, 0.25054E+01_JPRB, 0.26056E+01_JPRB, 0.26748E+01_JPRB ,&
&  0.26427E+01_JPRB, 0.24535E+01_JPRB, 0.20747E+01_JPRB, 0.17141E+01_JPRB, 0.14601E+01_JPRB, 0.12988E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.99467E+00_JPRB, 0.10067E+01_JPRB, 0.10449E+01_JPRB, 0.11405E+01_JPRB, 0.12278E+01_JPRB, 0.12960E+01_JPRB ,&
&  0.14007E+01_JPRB, 0.14917E+01_JPRB, 0.15322E+01_JPRB, 0.15548E+01_JPRB, 0.16118E+01_JPRB, 0.14729E+01_JPRB, 0.20086E+01_JPRB ,&
&  0.19147E+01_JPRB, 0.19448E+01_JPRB, 0.19451E+01_JPRB, 0.24859E+01_JPRB, 0.27140E+01_JPRB, 0.27355E+01_JPRB, 0.31386E+01_JPRB ,&
&  0.23587E+01_JPRB, 0.21323E+01_JPRB, 0.22333E+01_JPRB, 0.21924E+01_JPRB, 0.17521E+01_JPRB, 0.18824E+01_JPRB, 0.26999E+01_JPRB ,&
&  0.20978E+01_JPRB, 0.22882E+01_JPRB, 0.23782E+01_JPRB, 0.23541E+01_JPRB, 0.10726E+01_JPRB, 0.76711E+00_JPRB, 0.13076E+01_JPRB ,&
&  0.14999E+01_JPRB, 0.10101E+01_JPRB, 0.77237E+00_JPRB, 0.10090E+01_JPRB, 0.82414E+00_JPRB, 0.10553E+01_JPRB, 0.16789E+01_JPRB ,&
&  0.21647E+01_JPRB, 0.28013E+01_JPRB, 0.45495E+01_JPRB, 0.47212E+01_JPRB, 0.42439E+01_JPRB, 0.42584E+01_JPRB, 0.39125E+01_JPRB ,&
&  0.35608E+01_JPRB, 0.28576E+01_JPRB, 0.26999E+01_JPRB, 0.23915E+01_JPRB, 0.21592E+01_JPRB, 0.20099E+01_JPRB, 0.18695E+01_JPRB ,&
&  0.17153E+01_JPRB, 0.15696E+01_JPRB, 0.14220E+01_JPRB, 0.12658E+01_JPRB, 0.11353E+01_JPRB /)
ZBURD(:, 23,  1) = (/ &
&  0.10662E+01_JPRB, 0.97677E+00_JPRB, 0.88896E+00_JPRB, 0.87176E+00_JPRB, 0.89064E+00_JPRB, 0.82370E+00_JPRB, 0.78108E+00_JPRB ,&
&  0.74866E+00_JPRB, 0.73198E+00_JPRB, 0.71323E+00_JPRB, 0.69090E+00_JPRB, 0.67646E+00_JPRB, 0.69606E+00_JPRB, 0.74837E+00_JPRB ,&
&  0.82523E+00_JPRB, 0.93726E+00_JPRB, 0.11145E+01_JPRB, 0.98125E+00_JPRB, 0.88853E+00_JPRB, 0.80040E+00_JPRB, 0.74192E+00_JPRB ,&
&  0.10325E+01_JPRB, 0.13876E+01_JPRB, 0.16462E+01_JPRB, 0.19200E+01_JPRB, 0.22106E+01_JPRB, 0.22365E+01_JPRB, 0.24132E+01_JPRB ,&
&  0.23661E+01_JPRB, 0.20296E+01_JPRB, 0.17439E+01_JPRB, 0.15040E+01_JPRB, 0.13370E+01_JPRB, 0.11802E+01_JPRB, 0.10665E+01_JPRB ,&
&  0.10133E+01_JPRB, 0.99689E+00_JPRB, 0.98989E+00_JPRB, 0.10868E+01_JPRB, 0.12376E+01_JPRB, 0.13396E+01_JPRB, 0.14325E+01_JPRB ,&
&  0.15079E+01_JPRB, 0.15392E+01_JPRB, 0.15512E+01_JPRB, 0.15850E+01_JPRB, 0.16808E+01_JPRB, 0.16081E+01_JPRB, 0.14913E+01_JPRB ,&
&  0.14630E+01_JPRB, 0.16266E+01_JPRB, 0.22754E+01_JPRB, 0.25295E+01_JPRB, 0.25306E+01_JPRB, 0.27191E+01_JPRB, 0.27475E+01_JPRB ,&
&  0.26703E+01_JPRB, 0.27564E+01_JPRB, 0.23293E+01_JPRB, 0.24131E+01_JPRB, 0.21280E+01_JPRB, 0.15243E+01_JPRB, 0.18605E+01_JPRB ,&
&  0.20264E+01_JPRB, 0.19282E+01_JPRB, 0.14907E+01_JPRB, 0.10737E+01_JPRB, 0.10706E+01_JPRB, 0.82927E+00_JPRB, 0.62540E+00_JPRB ,&
&  0.60449E+00_JPRB, 0.58841E+00_JPRB, 0.54882E+00_JPRB, 0.59099E+00_JPRB, 0.62133E+00_JPRB, 0.97735E+00_JPRB, 0.16826E+01_JPRB ,&
&  0.29648E+01_JPRB, 0.42983E+01_JPRB, 0.51410E+01_JPRB, 0.49377E+01_JPRB, 0.42920E+01_JPRB, 0.40895E+01_JPRB, 0.38285E+01_JPRB ,&
&  0.29511E+01_JPRB, 0.25660E+01_JPRB, 0.23572E+01_JPRB, 0.22406E+01_JPRB, 0.21465E+01_JPRB, 0.20506E+01_JPRB, 0.19060E+01_JPRB ,&
&  0.17358E+01_JPRB, 0.15987E+01_JPRB, 0.14664E+01_JPRB, 0.13053E+01_JPRB, 0.11766E+01_JPRB /)
ZBURD(:, 24,  1) = (/ &
&  0.10947E+01_JPRB, 0.10101E+01_JPRB, 0.95947E+00_JPRB, 0.92239E+00_JPRB, 0.89743E+00_JPRB, 0.80838E+00_JPRB, 0.77819E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.71946E+00_JPRB, 0.68632E+00_JPRB, 0.67017E+00_JPRB, 0.66653E+00_JPRB, 0.70635E+00_JPRB, 0.77981E+00_JPRB ,&
&  0.85474E+00_JPRB, 0.98102E+00_JPRB, 0.12240E+01_JPRB, 0.12018E+01_JPRB, 0.12034E+01_JPRB, 0.11115E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.17412E+01_JPRB, 0.19539E+01_JPRB, 0.20455E+01_JPRB, 0.22507E+01_JPRB, 0.22774E+01_JPRB, 0.21641E+01_JPRB ,&
&  0.20193E+01_JPRB, 0.17471E+01_JPRB, 0.15264E+01_JPRB, 0.13619E+01_JPRB, 0.12180E+01_JPRB, 0.10930E+01_JPRB, 0.10188E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.97729E+00_JPRB, 0.10092E+01_JPRB, 0.11517E+01_JPRB, 0.12954E+01_JPRB, 0.13901E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15729E+01_JPRB, 0.15877E+01_JPRB, 0.15899E+01_JPRB, 0.15783E+01_JPRB, 0.13302E+01_JPRB, 0.11852E+01_JPRB, 0.14920E+01_JPRB ,&
&  0.17473E+01_JPRB, 0.21007E+01_JPRB, 0.23353E+01_JPRB, 0.27613E+01_JPRB, 0.25569E+01_JPRB, 0.25110E+01_JPRB, 0.25411E+01_JPRB ,&
&  0.24603E+01_JPRB, 0.27502E+01_JPRB, 0.21277E+01_JPRB, 0.20415E+01_JPRB, 0.22102E+01_JPRB, 0.16495E+01_JPRB, 0.15279E+01_JPRB ,&
&  0.17990E+01_JPRB, 0.17124E+01_JPRB, 0.14183E+01_JPRB, 0.11229E+01_JPRB, 0.14494E+01_JPRB, 0.14398E+01_JPRB, 0.76903E+00_JPRB ,&
&  0.64789E+00_JPRB, 0.61496E+00_JPRB, 0.58809E+00_JPRB, 0.57394E+00_JPRB, 0.62132E+00_JPRB, 0.10597E+01_JPRB, 0.23056E+01_JPRB ,&
&  0.35094E+01_JPRB, 0.52172E+01_JPRB, 0.54243E+01_JPRB, 0.50654E+01_JPRB, 0.42651E+01_JPRB, 0.38513E+01_JPRB, 0.32567E+01_JPRB ,&
&  0.27999E+01_JPRB, 0.24710E+01_JPRB, 0.22656E+01_JPRB, 0.21628E+01_JPRB, 0.20734E+01_JPRB, 0.19854E+01_JPRB, 0.18714E+01_JPRB ,&
&  0.17306E+01_JPRB, 0.16017E+01_JPRB, 0.14738E+01_JPRB, 0.13172E+01_JPRB, 0.11914E+01_JPRB /)
ZBURD(:, 25,  1) = (/ &
&  0.11009E+01_JPRB, 0.10388E+01_JPRB, 0.10137E+01_JPRB, 0.98468E+00_JPRB, 0.91361E+00_JPRB, 0.82624E+00_JPRB, 0.75894E+00_JPRB ,&
&  0.72146E+00_JPRB, 0.69860E+00_JPRB, 0.67894E+00_JPRB, 0.67701E+00_JPRB, 0.69129E+00_JPRB, 0.73623E+00_JPRB, 0.79374E+00_JPRB ,&
&  0.88131E+00_JPRB, 0.10098E+01_JPRB, 0.11483E+01_JPRB, 0.13341E+01_JPRB, 0.13038E+01_JPRB, 0.11634E+01_JPRB, 0.13293E+01_JPRB ,&
&  0.16231E+01_JPRB, 0.20906E+01_JPRB, 0.22825E+01_JPRB, 0.22504E+01_JPRB, 0.21951E+01_JPRB, 0.21637E+01_JPRB, 0.20034E+01_JPRB ,&
&  0.17833E+01_JPRB, 0.15448E+01_JPRB, 0.13560E+01_JPRB, 0.12334E+01_JPRB, 0.11172E+01_JPRB, 0.10393E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.98538E+00_JPRB, 0.94548E+00_JPRB, 0.10390E+01_JPRB, 0.11897E+01_JPRB, 0.13125E+01_JPRB, 0.14437E+01_JPRB, 0.15680E+01_JPRB ,&
&  0.16153E+01_JPRB, 0.16268E+01_JPRB, 0.16261E+01_JPRB, 0.16215E+01_JPRB, 0.13254E+01_JPRB, 0.15659E+01_JPRB, 0.16614E+01_JPRB ,&
&  0.18455E+01_JPRB, 0.20517E+01_JPRB, 0.21288E+01_JPRB, 0.24866E+01_JPRB, 0.24736E+01_JPRB, 0.23816E+01_JPRB, 0.22805E+01_JPRB ,&
&  0.23423E+01_JPRB, 0.21395E+01_JPRB, 0.18000E+01_JPRB, 0.18203E+01_JPRB, 0.19955E+01_JPRB, 0.20931E+01_JPRB, 0.13703E+01_JPRB ,&
&  0.12323E+01_JPRB, 0.15706E+01_JPRB, 0.14213E+01_JPRB, 0.12613E+01_JPRB, 0.15519E+01_JPRB, 0.19073E+01_JPRB, 0.14408E+01_JPRB ,&
&  0.77759E+00_JPRB, 0.65212E+00_JPRB, 0.67774E+00_JPRB, 0.73096E+00_JPRB, 0.74867E+00_JPRB, 0.12983E+01_JPRB, 0.45582E+01_JPRB ,&
&  0.48999E+01_JPRB, 0.59231E+01_JPRB, 0.53759E+01_JPRB, 0.45933E+01_JPRB, 0.40337E+01_JPRB, 0.35425E+01_JPRB, 0.30772E+01_JPRB ,&
&  0.26614E+01_JPRB, 0.23989E+01_JPRB, 0.21863E+01_JPRB, 0.20386E+01_JPRB, 0.18991E+01_JPRB, 0.17633E+01_JPRB, 0.16474E+01_JPRB ,&
&  0.15720E+01_JPRB, 0.14828E+01_JPRB, 0.13968E+01_JPRB, 0.12806E+01_JPRB, 0.11768E+01_JPRB /)
ZBURD(:, 26,  1) = (/ &
&  0.10818E+01_JPRB, 0.10381E+01_JPRB, 0.10193E+01_JPRB, 0.98623E+00_JPRB, 0.93033E+00_JPRB, 0.87866E+00_JPRB, 0.79449E+00_JPRB ,&
&  0.73164E+00_JPRB, 0.69661E+00_JPRB, 0.68329E+00_JPRB, 0.69416E+00_JPRB, 0.72535E+00_JPRB, 0.77061E+00_JPRB, 0.81074E+00_JPRB ,&
&  0.92156E+00_JPRB, 0.10223E+01_JPRB, 0.11985E+01_JPRB, 0.14155E+01_JPRB, 0.14516E+01_JPRB, 0.12911E+01_JPRB, 0.13793E+01_JPRB ,&
&  0.19503E+01_JPRB, 0.23707E+01_JPRB, 0.22104E+01_JPRB, 0.21459E+01_JPRB, 0.21631E+01_JPRB, 0.22194E+01_JPRB, 0.18618E+01_JPRB ,&
&  0.16251E+01_JPRB, 0.14637E+01_JPRB, 0.12825E+01_JPRB, 0.11577E+01_JPRB, 0.10683E+01_JPRB, 0.10549E+01_JPRB, 0.99846E+00_JPRB ,&
&  0.93177E+00_JPRB, 0.96186E+00_JPRB, 0.10770E+01_JPRB, 0.12016E+01_JPRB, 0.13303E+01_JPRB, 0.14683E+01_JPRB, 0.16022E+01_JPRB ,&
&  0.16560E+01_JPRB, 0.16566E+01_JPRB, 0.16304E+01_JPRB, 0.15504E+01_JPRB, 0.15704E+01_JPRB, 0.16302E+01_JPRB, 0.17882E+01_JPRB ,&
&  0.18660E+01_JPRB, 0.20152E+01_JPRB, 0.21082E+01_JPRB, 0.22279E+01_JPRB, 0.22499E+01_JPRB, 0.22275E+01_JPRB, 0.20982E+01_JPRB ,&
&  0.20430E+01_JPRB, 0.18507E+01_JPRB, 0.15075E+01_JPRB, 0.15998E+01_JPRB, 0.18619E+01_JPRB, 0.20036E+01_JPRB, 0.17940E+01_JPRB ,&
&  0.15580E+01_JPRB, 0.13656E+01_JPRB, 0.13526E+01_JPRB, 0.15179E+01_JPRB, 0.17222E+01_JPRB, 0.19633E+01_JPRB, 0.24128E+01_JPRB ,&
&  0.25096E+01_JPRB, 0.15458E+01_JPRB, 0.13168E+01_JPRB, 0.17703E+01_JPRB, 0.16277E+01_JPRB, 0.18524E+01_JPRB, 0.32961E+01_JPRB ,&
&  0.46464E+01_JPRB, 0.54322E+01_JPRB, 0.50573E+01_JPRB, 0.41934E+01_JPRB, 0.37570E+01_JPRB, 0.32246E+01_JPRB, 0.28482E+01_JPRB ,&
&  0.24404E+01_JPRB, 0.21120E+01_JPRB, 0.18829E+01_JPRB, 0.17177E+01_JPRB, 0.15733E+01_JPRB, 0.14635E+01_JPRB, 0.14126E+01_JPRB ,&
&  0.13788E+01_JPRB, 0.13340E+01_JPRB, 0.12858E+01_JPRB, 0.12028E+01_JPRB, 0.11298E+01_JPRB /)
ZBURD(:, 27,  1) = (/ &
&  0.10422E+01_JPRB, 0.10168E+01_JPRB, 0.99401E+00_JPRB, 0.96125E+00_JPRB, 0.97172E+00_JPRB, 0.96751E+00_JPRB, 0.89045E+00_JPRB ,&
&  0.81866E+00_JPRB, 0.71534E+00_JPRB, 0.70864E+00_JPRB, 0.73077E+00_JPRB, 0.76709E+00_JPRB, 0.81079E+00_JPRB, 0.88067E+00_JPRB ,&
&  0.98018E+00_JPRB, 0.10805E+01_JPRB, 0.13166E+01_JPRB, 0.15415E+01_JPRB, 0.17386E+01_JPRB, 0.20332E+01_JPRB, 0.14087E+01_JPRB ,&
&  0.15211E+01_JPRB, 0.21463E+01_JPRB, 0.20712E+01_JPRB, 0.20500E+01_JPRB, 0.20273E+01_JPRB, 0.20070E+01_JPRB, 0.16642E+01_JPRB ,&
&  0.14649E+01_JPRB, 0.13477E+01_JPRB, 0.12018E+01_JPRB, 0.10866E+01_JPRB, 0.10603E+01_JPRB, 0.10302E+01_JPRB, 0.94425E+00_JPRB ,&
&  0.93304E+00_JPRB, 0.99418E+00_JPRB, 0.10952E+01_JPRB, 0.12033E+01_JPRB, 0.13324E+01_JPRB, 0.14806E+01_JPRB, 0.15688E+01_JPRB ,&
&  0.16470E+01_JPRB, 0.16671E+01_JPRB, 0.16495E+01_JPRB, 0.15466E+01_JPRB, 0.16018E+01_JPRB, 0.16543E+01_JPRB, 0.17146E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.15781E+01_JPRB, 0.18526E+01_JPRB, 0.20807E+01_JPRB, 0.20863E+01_JPRB, 0.19290E+01_JPRB, 0.17652E+01_JPRB ,&
&  0.18060E+01_JPRB, 0.17761E+01_JPRB, 0.16997E+01_JPRB, 0.16056E+01_JPRB, 0.17786E+01_JPRB, 0.19257E+01_JPRB, 0.19872E+01_JPRB ,&
&  0.19769E+01_JPRB, 0.18457E+01_JPRB, 0.18269E+01_JPRB, 0.19041E+01_JPRB, 0.19604E+01_JPRB, 0.21485E+01_JPRB, 0.24010E+01_JPRB ,&
&  0.28239E+01_JPRB, 0.33230E+01_JPRB, 0.34885E+01_JPRB, 0.29643E+01_JPRB, 0.28733E+01_JPRB, 0.27404E+01_JPRB, 0.33842E+01_JPRB ,&
&  0.44107E+01_JPRB, 0.43507E+01_JPRB, 0.40645E+01_JPRB, 0.39548E+01_JPRB, 0.30812E+01_JPRB, 0.26479E+01_JPRB, 0.23292E+01_JPRB ,&
&  0.20882E+01_JPRB, 0.18183E+01_JPRB, 0.16164E+01_JPRB, 0.14669E+01_JPRB, 0.13230E+01_JPRB, 0.12505E+01_JPRB, 0.12439E+01_JPRB ,&
&  0.12224E+01_JPRB, 0.12058E+01_JPRB, 0.11756E+01_JPRB, 0.11229E+01_JPRB, 0.10797E+01_JPRB /)
ZBURD(:, 28,  1) = (/ &
&  0.99787E+00_JPRB, 0.96612E+00_JPRB, 0.94693E+00_JPRB, 0.93835E+00_JPRB, 0.10054E+01_JPRB, 0.10300E+01_JPRB, 0.10260E+01_JPRB ,&
&  0.10047E+01_JPRB, 0.76502E+00_JPRB, 0.74747E+00_JPRB, 0.76932E+00_JPRB, 0.81141E+00_JPRB, 0.86493E+00_JPRB, 0.94682E+00_JPRB ,&
&  0.10316E+01_JPRB, 0.11736E+01_JPRB, 0.14093E+01_JPRB, 0.16663E+01_JPRB, 0.18696E+01_JPRB, 0.19958E+01_JPRB, 0.19052E+01_JPRB ,&
&  0.13361E+01_JPRB, 0.21625E+01_JPRB, 0.19931E+01_JPRB, 0.19807E+01_JPRB, 0.19299E+01_JPRB, 0.17963E+01_JPRB, 0.14971E+01_JPRB ,&
&  0.13147E+01_JPRB, 0.12317E+01_JPRB, 0.11715E+01_JPRB, 0.11454E+01_JPRB, 0.10941E+01_JPRB, 0.99347E+00_JPRB, 0.95933E+00_JPRB ,&
&  0.99099E+00_JPRB, 0.10733E+01_JPRB, 0.11691E+01_JPRB, 0.12654E+01_JPRB, 0.13899E+01_JPRB, 0.15079E+01_JPRB, 0.15749E+01_JPRB ,&
&  0.16425E+01_JPRB, 0.16537E+01_JPRB, 0.15883E+01_JPRB, 0.15867E+01_JPRB, 0.16010E+01_JPRB, 0.16552E+01_JPRB, 0.16853E+01_JPRB ,&
&  0.16166E+01_JPRB, 0.15552E+01_JPRB, 0.17354E+01_JPRB, 0.17767E+01_JPRB, 0.16900E+01_JPRB, 0.17216E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.17029E+01_JPRB, 0.19491E+01_JPRB, 0.16608E+01_JPRB, 0.17188E+01_JPRB, 0.19010E+01_JPRB, 0.18812E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.18841E+01_JPRB, 0.19675E+01_JPRB, 0.21960E+01_JPRB, 0.23745E+01_JPRB, 0.25703E+01_JPRB, 0.26012E+01_JPRB ,&
&  0.27993E+01_JPRB, 0.30784E+01_JPRB, 0.34959E+01_JPRB, 0.29481E+01_JPRB, 0.28671E+01_JPRB, 0.29572E+01_JPRB, 0.35189E+01_JPRB ,&
&  0.40288E+01_JPRB, 0.38098E+01_JPRB, 0.37477E+01_JPRB, 0.32334E+01_JPRB, 0.25098E+01_JPRB, 0.21938E+01_JPRB, 0.19412E+01_JPRB ,&
&  0.17455E+01_JPRB, 0.15458E+01_JPRB, 0.13976E+01_JPRB, 0.12651E+01_JPRB, 0.11724E+01_JPRB, 0.11473E+01_JPRB, 0.11476E+01_JPRB ,&
&  0.11386E+01_JPRB, 0.11281E+01_JPRB, 0.11103E+01_JPRB, 0.10686E+01_JPRB, 0.10306E+01_JPRB /)
ZBURD(:, 29,  1) = (/ &
&  0.97114E+00_JPRB, 0.95779E+00_JPRB, 0.95264E+00_JPRB, 0.93715E+00_JPRB, 0.10064E+01_JPRB, 0.11225E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.11563E+01_JPRB, 0.79204E+00_JPRB, 0.78373E+00_JPRB, 0.80738E+00_JPRB, 0.84633E+00_JPRB, 0.91659E+00_JPRB, 0.10033E+01_JPRB ,&
&  0.11242E+01_JPRB, 0.12791E+01_JPRB, 0.15153E+01_JPRB, 0.17952E+01_JPRB, 0.19928E+01_JPRB, 0.22126E+01_JPRB, 0.21803E+01_JPRB ,&
&  0.15682E+01_JPRB, 0.14921E+01_JPRB, 0.19236E+01_JPRB, 0.18437E+01_JPRB, 0.17173E+01_JPRB, 0.16230E+01_JPRB, 0.15076E+01_JPRB ,&
&  0.13721E+01_JPRB, 0.12671E+01_JPRB, 0.12541E+01_JPRB, 0.12144E+01_JPRB, 0.10620E+01_JPRB, 0.10269E+01_JPRB, 0.10384E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.11621E+01_JPRB, 0.12495E+01_JPRB, 0.13409E+01_JPRB, 0.14547E+01_JPRB, 0.15269E+01_JPRB, 0.15966E+01_JPRB ,&
&  0.16328E+01_JPRB, 0.16522E+01_JPRB, 0.16211E+01_JPRB, 0.15621E+01_JPRB, 0.15830E+01_JPRB, 0.15994E+01_JPRB, 0.15912E+01_JPRB ,&
&  0.15892E+01_JPRB, 0.16678E+01_JPRB, 0.17047E+01_JPRB, 0.16211E+01_JPRB, 0.14917E+01_JPRB, 0.16591E+01_JPRB, 0.16661E+01_JPRB ,&
&  0.17581E+01_JPRB, 0.17952E+01_JPRB, 0.21323E+01_JPRB, 0.16593E+01_JPRB, 0.16748E+01_JPRB, 0.18111E+01_JPRB, 0.18432E+01_JPRB ,&
&  0.18261E+01_JPRB, 0.20049E+01_JPRB, 0.22058E+01_JPRB, 0.24230E+01_JPRB, 0.26387E+01_JPRB, 0.25929E+01_JPRB, 0.30482E+01_JPRB ,&
&  0.28953E+01_JPRB, 0.33376E+01_JPRB, 0.31515E+01_JPRB, 0.31145E+01_JPRB, 0.27581E+01_JPRB, 0.28705E+01_JPRB, 0.30594E+01_JPRB ,&
&  0.33285E+01_JPRB, 0.31150E+01_JPRB, 0.27899E+01_JPRB, 0.24433E+01_JPRB, 0.20129E+01_JPRB, 0.18045E+01_JPRB, 0.16057E+01_JPRB ,&
&  0.14228E+01_JPRB, 0.12969E+01_JPRB, 0.12369E+01_JPRB, 0.11838E+01_JPRB, 0.11577E+01_JPRB, 0.11411E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.10890E+01_JPRB, 0.10685E+01_JPRB, 0.10283E+01_JPRB, 0.99445E+00_JPRB /)
ZBURD(:, 30,  1) = (/ &
&  0.94952E+00_JPRB, 0.93162E+00_JPRB, 0.91305E+00_JPRB, 0.86341E+00_JPRB, 0.83929E+00_JPRB, 0.84793E+00_JPRB, 0.83911E+00_JPRB ,&
&  0.82430E+00_JPRB, 0.80073E+00_JPRB, 0.81546E+00_JPRB, 0.85244E+00_JPRB, 0.89919E+00_JPRB, 0.97749E+00_JPRB, 0.10818E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.13485E+01_JPRB, 0.15610E+01_JPRB, 0.17718E+01_JPRB, 0.19450E+01_JPRB, 0.20867E+01_JPRB, 0.21422E+01_JPRB ,&
&  0.17394E+01_JPRB, 0.13364E+01_JPRB, 0.17550E+01_JPRB, 0.18289E+01_JPRB, 0.16927E+01_JPRB, 0.16394E+01_JPRB, 0.15436E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.13289E+01_JPRB, 0.13299E+01_JPRB, 0.12409E+01_JPRB, 0.10971E+01_JPRB, 0.11038E+01_JPRB, 0.11407E+01_JPRB ,&
&  0.11993E+01_JPRB, 0.12543E+01_JPRB, 0.13246E+01_JPRB, 0.14116E+01_JPRB, 0.14869E+01_JPRB, 0.15747E+01_JPRB, 0.16396E+01_JPRB ,&
&  0.16597E+01_JPRB, 0.16661E+01_JPRB, 0.16670E+01_JPRB, 0.16104E+01_JPRB, 0.15781E+01_JPRB, 0.15627E+01_JPRB, 0.15378E+01_JPRB ,&
&  0.15464E+01_JPRB, 0.15385E+01_JPRB, 0.16018E+01_JPRB, 0.16729E+01_JPRB, 0.17829E+01_JPRB, 0.16584E+01_JPRB, 0.17779E+01_JPRB ,&
&  0.18004E+01_JPRB, 0.19133E+01_JPRB, 0.19230E+01_JPRB, 0.20953E+01_JPRB, 0.14295E+01_JPRB, 0.16319E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.20152E+01_JPRB, 0.21512E+01_JPRB, 0.23827E+01_JPRB, 0.26319E+01_JPRB, 0.29451E+01_JPRB, 0.28876E+01_JPRB, 0.30586E+01_JPRB ,&
&  0.30524E+01_JPRB, 0.29899E+01_JPRB, 0.28137E+01_JPRB, 0.28521E+01_JPRB, 0.27291E+01_JPRB, 0.29800E+01_JPRB, 0.27695E+01_JPRB ,&
&  0.27799E+01_JPRB, 0.25096E+01_JPRB, 0.22594E+01_JPRB, 0.20782E+01_JPRB, 0.17349E+01_JPRB, 0.14614E+01_JPRB, 0.13000E+01_JPRB ,&
&  0.12252E+01_JPRB, 0.11877E+01_JPRB, 0.11525E+01_JPRB, 0.11300E+01_JPRB, 0.11239E+01_JPRB, 0.11185E+01_JPRB, 0.11089E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.10678E+01_JPRB, 0.10422E+01_JPRB, 0.10059E+01_JPRB, 0.97604E+00_JPRB /)
ZBURD(:, 31,  1) = (/ &
&  0.91696E+00_JPRB, 0.89365E+00_JPRB, 0.86600E+00_JPRB, 0.83134E+00_JPRB, 0.81697E+00_JPRB, 0.80435E+00_JPRB, 0.80524E+00_JPRB ,&
&  0.81008E+00_JPRB, 0.82993E+00_JPRB, 0.84343E+00_JPRB, 0.87159E+00_JPRB, 0.90797E+00_JPRB, 0.97577E+00_JPRB, 0.10409E+01_JPRB ,&
&  0.11230E+01_JPRB, 0.12383E+01_JPRB, 0.14526E+01_JPRB, 0.17190E+01_JPRB, 0.19308E+01_JPRB, 0.20478E+01_JPRB, 0.19723E+01_JPRB ,&
&  0.18789E+01_JPRB, 0.18571E+01_JPRB, 0.17582E+01_JPRB, 0.13684E+01_JPRB, 0.12961E+01_JPRB, 0.15471E+01_JPRB, 0.14214E+01_JPRB ,&
&  0.13209E+01_JPRB, 0.12873E+01_JPRB, 0.12448E+01_JPRB, 0.12022E+01_JPRB, 0.11666E+01_JPRB, 0.11797E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.12652E+01_JPRB, 0.13179E+01_JPRB, 0.13859E+01_JPRB, 0.14588E+01_JPRB, 0.15342E+01_JPRB, 0.16396E+01_JPRB, 0.17072E+01_JPRB ,&
&  0.17133E+01_JPRB, 0.17078E+01_JPRB, 0.16904E+01_JPRB, 0.16549E+01_JPRB, 0.15788E+01_JPRB, 0.15722E+01_JPRB, 0.15910E+01_JPRB ,&
&  0.16177E+01_JPRB, 0.16375E+01_JPRB, 0.16657E+01_JPRB, 0.17539E+01_JPRB, 0.18094E+01_JPRB, 0.16427E+01_JPRB, 0.16459E+01_JPRB ,&
&  0.17904E+01_JPRB, 0.19572E+01_JPRB, 0.17715E+01_JPRB, 0.20638E+01_JPRB, 0.13005E+01_JPRB, 0.15687E+01_JPRB, 0.19210E+01_JPRB ,&
&  0.20453E+01_JPRB, 0.22616E+01_JPRB, 0.25159E+01_JPRB, 0.28103E+01_JPRB, 0.30839E+01_JPRB, 0.28769E+01_JPRB, 0.29344E+01_JPRB ,&
&  0.27798E+01_JPRB, 0.24938E+01_JPRB, 0.24560E+01_JPRB, 0.24285E+01_JPRB, 0.25460E+01_JPRB, 0.25735E+01_JPRB, 0.21646E+01_JPRB ,&
&  0.18691E+01_JPRB, 0.20450E+01_JPRB, 0.20039E+01_JPRB, 0.19163E+01_JPRB, 0.15496E+01_JPRB, 0.12140E+01_JPRB, 0.11534E+01_JPRB ,&
&  0.11472E+01_JPRB, 0.11160E+01_JPRB, 0.11032E+01_JPRB, 0.10998E+01_JPRB, 0.11033E+01_JPRB, 0.10989E+01_JPRB, 0.10910E+01_JPRB ,&
&  0.10787E+01_JPRB, 0.10501E+01_JPRB, 0.10199E+01_JPRB, 0.98359E+00_JPRB, 0.94826E+00_JPRB /)
ZBURD(:, 32,  1) = (/ &
&  0.87700E+00_JPRB, 0.85560E+00_JPRB, 0.82722E+00_JPRB, 0.80115E+00_JPRB, 0.76304E+00_JPRB, 0.74421E+00_JPRB, 0.74402E+00_JPRB ,&
&  0.76183E+00_JPRB, 0.78644E+00_JPRB, 0.80272E+00_JPRB, 0.80950E+00_JPRB, 0.83590E+00_JPRB, 0.85099E+00_JPRB, 0.87883E+00_JPRB ,&
&  0.93785E+00_JPRB, 0.10395E+01_JPRB, 0.12376E+01_JPRB, 0.15468E+01_JPRB, 0.18319E+01_JPRB, 0.20928E+01_JPRB, 0.19724E+01_JPRB ,&
&  0.18534E+01_JPRB, 0.17752E+01_JPRB, 0.17198E+01_JPRB, 0.16499E+01_JPRB, 0.14588E+01_JPRB, 0.15351E+01_JPRB, 0.15590E+01_JPRB ,&
&  0.15605E+01_JPRB, 0.14523E+01_JPRB, 0.13208E+01_JPRB, 0.12739E+01_JPRB, 0.12419E+01_JPRB, 0.12517E+01_JPRB, 0.12848E+01_JPRB ,&
&  0.13269E+01_JPRB, 0.13694E+01_JPRB, 0.14381E+01_JPRB, 0.15262E+01_JPRB, 0.16332E+01_JPRB, 0.17348E+01_JPRB, 0.17854E+01_JPRB ,&
&  0.17799E+01_JPRB, 0.17709E+01_JPRB, 0.17533E+01_JPRB, 0.16826E+01_JPRB, 0.16612E+01_JPRB, 0.16940E+01_JPRB, 0.17346E+01_JPRB ,&
&  0.17875E+01_JPRB, 0.17303E+01_JPRB, 0.17940E+01_JPRB, 0.18902E+01_JPRB, 0.18471E+01_JPRB, 0.17565E+01_JPRB, 0.17588E+01_JPRB ,&
&  0.18540E+01_JPRB, 0.20159E+01_JPRB, 0.14981E+01_JPRB, 0.18105E+01_JPRB, 0.21673E+01_JPRB, 0.19434E+01_JPRB, 0.19731E+01_JPRB ,&
&  0.21849E+01_JPRB, 0.23619E+01_JPRB, 0.25907E+01_JPRB, 0.28285E+01_JPRB, 0.30908E+01_JPRB, 0.33060E+01_JPRB, 0.24983E+01_JPRB ,&
&  0.23698E+01_JPRB, 0.21172E+01_JPRB, 0.20702E+01_JPRB, 0.20954E+01_JPRB, 0.21568E+01_JPRB, 0.20736E+01_JPRB, 0.17243E+01_JPRB ,&
&  0.15732E+01_JPRB, 0.17294E+01_JPRB, 0.16910E+01_JPRB, 0.17749E+01_JPRB, 0.16959E+01_JPRB, 0.10895E+01_JPRB, 0.11068E+01_JPRB ,&
&  0.11011E+01_JPRB, 0.10794E+01_JPRB, 0.10730E+01_JPRB, 0.10708E+01_JPRB, 0.10718E+01_JPRB, 0.10676E+01_JPRB, 0.10615E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.10107E+01_JPRB, 0.97493E+00_JPRB, 0.93834E+00_JPRB, 0.90484E+00_JPRB /)
ZBURD(:, 33,  1) = (/ &
&  0.84696E+00_JPRB, 0.82221E+00_JPRB, 0.79314E+00_JPRB, 0.75555E+00_JPRB, 0.71645E+00_JPRB, 0.70055E+00_JPRB, 0.69333E+00_JPRB ,&
&  0.69937E+00_JPRB, 0.72124E+00_JPRB, 0.73306E+00_JPRB, 0.72968E+00_JPRB, 0.72473E+00_JPRB, 0.73696E+00_JPRB, 0.77944E+00_JPRB ,&
&  0.83586E+00_JPRB, 0.91159E+00_JPRB, 0.10585E+01_JPRB, 0.13618E+01_JPRB, 0.17309E+01_JPRB, 0.20231E+01_JPRB, 0.19735E+01_JPRB ,&
&  0.18421E+01_JPRB, 0.18300E+01_JPRB, 0.18065E+01_JPRB, 0.18064E+01_JPRB, 0.18145E+01_JPRB, 0.18523E+01_JPRB, 0.18896E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.13742E+01_JPRB, 0.13047E+01_JPRB, 0.12742E+01_JPRB, 0.12833E+01_JPRB, 0.12989E+01_JPRB, 0.13283E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.14189E+01_JPRB, 0.14981E+01_JPRB, 0.16201E+01_JPRB, 0.17515E+01_JPRB, 0.18339E+01_JPRB, 0.18671E+01_JPRB ,&
&  0.18797E+01_JPRB, 0.18820E+01_JPRB, 0.19155E+01_JPRB, 0.17589E+01_JPRB, 0.18227E+01_JPRB, 0.18872E+01_JPRB, 0.19360E+01_JPRB ,&
&  0.19184E+01_JPRB, 0.18430E+01_JPRB, 0.19574E+01_JPRB, 0.19870E+01_JPRB, 0.20142E+01_JPRB, 0.19856E+01_JPRB, 0.19494E+01_JPRB ,&
&  0.19970E+01_JPRB, 0.18373E+01_JPRB, 0.13609E+01_JPRB, 0.14100E+01_JPRB, 0.15669E+01_JPRB, 0.15435E+01_JPRB, 0.20313E+01_JPRB ,&
&  0.21473E+01_JPRB, 0.22509E+01_JPRB, 0.23565E+01_JPRB, 0.24715E+01_JPRB, 0.26014E+01_JPRB, 0.26839E+01_JPRB, 0.22234E+01_JPRB ,&
&  0.19554E+01_JPRB, 0.17884E+01_JPRB, 0.17387E+01_JPRB, 0.16987E+01_JPRB, 0.16586E+01_JPRB, 0.15608E+01_JPRB, 0.15063E+01_JPRB ,&
&  0.14613E+01_JPRB, 0.14221E+01_JPRB, 0.13395E+01_JPRB, 0.12736E+01_JPRB, 0.11470E+01_JPRB, 0.10616E+01_JPRB, 0.10724E+01_JPRB ,&
&  0.10708E+01_JPRB, 0.10431E+01_JPRB, 0.10388E+01_JPRB, 0.10317E+01_JPRB, 0.10284E+01_JPRB, 0.10202E+01_JPRB, 0.10067E+01_JPRB ,&
&  0.98129E+00_JPRB, 0.95369E+00_JPRB, 0.92566E+00_JPRB, 0.89412E+00_JPRB, 0.86576E+00_JPRB /)
ZBURD(:, 34,  1) = (/ &
&  0.82509E+00_JPRB, 0.79739E+00_JPRB, 0.76285E+00_JPRB, 0.73024E+00_JPRB, 0.72736E+00_JPRB, 0.73127E+00_JPRB, 0.71660E+00_JPRB ,&
&  0.70822E+00_JPRB, 0.70234E+00_JPRB, 0.71727E+00_JPRB, 0.71787E+00_JPRB, 0.72043E+00_JPRB, 0.73377E+00_JPRB, 0.75176E+00_JPRB ,&
&  0.76947E+00_JPRB, 0.78627E+00_JPRB, 0.85957E+00_JPRB, 0.10124E+01_JPRB, 0.12934E+01_JPRB, 0.16574E+01_JPRB, 0.19158E+01_JPRB ,&
&  0.20511E+01_JPRB, 0.21626E+01_JPRB, 0.22534E+01_JPRB, 0.23329E+01_JPRB, 0.21774E+01_JPRB, 0.22073E+01_JPRB, 0.23819E+01_JPRB ,&
&  0.17531E+01_JPRB, 0.12747E+01_JPRB, 0.13397E+01_JPRB, 0.12471E+01_JPRB, 0.12604E+01_JPRB, 0.13043E+01_JPRB, 0.13399E+01_JPRB ,&
&  0.13862E+01_JPRB, 0.14512E+01_JPRB, 0.15375E+01_JPRB, 0.16577E+01_JPRB, 0.17700E+01_JPRB, 0.18433E+01_JPRB, 0.18900E+01_JPRB ,&
&  0.19213E+01_JPRB, 0.19673E+01_JPRB, 0.20477E+01_JPRB, 0.20244E+01_JPRB, 0.20039E+01_JPRB, 0.20999E+01_JPRB, 0.21465E+01_JPRB ,&
&  0.21868E+01_JPRB, 0.21892E+01_JPRB, 0.19621E+01_JPRB, 0.19627E+01_JPRB, 0.21261E+01_JPRB, 0.20082E+01_JPRB, 0.20017E+01_JPRB ,&
&  0.21021E+01_JPRB, 0.18241E+01_JPRB, 0.12886E+01_JPRB, 0.13042E+01_JPRB, 0.14861E+01_JPRB, 0.17287E+01_JPRB, 0.19891E+01_JPRB ,&
&  0.21076E+01_JPRB, 0.21529E+01_JPRB, 0.21250E+01_JPRB, 0.21244E+01_JPRB, 0.20383E+01_JPRB, 0.19470E+01_JPRB, 0.18418E+01_JPRB ,&
&  0.16249E+01_JPRB, 0.15431E+01_JPRB, 0.14576E+01_JPRB, 0.13981E+01_JPRB, 0.13752E+01_JPRB, 0.13250E+01_JPRB, 0.12981E+01_JPRB ,&
&  0.13697E+01_JPRB, 0.13724E+01_JPRB, 0.14621E+01_JPRB, 0.12578E+01_JPRB, 0.10624E+01_JPRB, 0.10618E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10763E+01_JPRB, 0.10570E+01_JPRB, 0.10400E+01_JPRB, 0.10228E+01_JPRB, 0.10100E+01_JPRB, 0.98730E+00_JPRB, 0.95532E+00_JPRB ,&
&  0.92274E+00_JPRB, 0.89771E+00_JPRB, 0.88218E+00_JPRB, 0.86335E+00_JPRB, 0.83916E+00_JPRB /)
ZBURD(:, 35,  1) = (/ &
&  0.79125E+00_JPRB, 0.74630E+00_JPRB, 0.73172E+00_JPRB, 0.77010E+00_JPRB, 0.74401E+00_JPRB, 0.76473E+00_JPRB, 0.75670E+00_JPRB ,&
&  0.75626E+00_JPRB, 0.75388E+00_JPRB, 0.76845E+00_JPRB, 0.78726E+00_JPRB, 0.81215E+00_JPRB, 0.83341E+00_JPRB, 0.84177E+00_JPRB ,&
&  0.85357E+00_JPRB, 0.86616E+00_JPRB, 0.88654E+00_JPRB, 0.92363E+00_JPRB, 0.99118E+00_JPRB, 0.11048E+01_JPRB, 0.13209E+01_JPRB ,&
&  0.15360E+01_JPRB, 0.16414E+01_JPRB, 0.16719E+01_JPRB, 0.16661E+01_JPRB, 0.18402E+01_JPRB, 0.22868E+01_JPRB, 0.26160E+01_JPRB ,&
&  0.15738E+01_JPRB, 0.13413E+01_JPRB, 0.12187E+01_JPRB, 0.11139E+01_JPRB, 0.11438E+01_JPRB, 0.12486E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.13214E+01_JPRB, 0.13795E+01_JPRB, 0.14598E+01_JPRB, 0.15529E+01_JPRB, 0.16270E+01_JPRB, 0.16889E+01_JPRB, 0.17577E+01_JPRB ,&
&  0.18327E+01_JPRB, 0.19422E+01_JPRB, 0.20651E+01_JPRB, 0.22023E+01_JPRB, 0.22042E+01_JPRB, 0.24302E+01_JPRB, 0.24682E+01_JPRB ,&
&  0.24828E+01_JPRB, 0.23625E+01_JPRB, 0.21011E+01_JPRB, 0.20945E+01_JPRB, 0.21673E+01_JPRB, 0.21375E+01_JPRB, 0.20270E+01_JPRB ,&
&  0.20397E+01_JPRB, 0.17739E+01_JPRB, 0.14369E+01_JPRB, 0.14990E+01_JPRB, 0.15961E+01_JPRB, 0.17594E+01_JPRB, 0.17602E+01_JPRB ,&
&  0.18452E+01_JPRB, 0.18220E+01_JPRB, 0.17672E+01_JPRB, 0.16868E+01_JPRB, 0.16110E+01_JPRB, 0.15971E+01_JPRB, 0.15879E+01_JPRB ,&
&  0.14501E+01_JPRB, 0.13841E+01_JPRB, 0.12897E+01_JPRB, 0.12603E+01_JPRB, 0.12856E+01_JPRB, 0.11077E+01_JPRB, 0.11942E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.12714E+01_JPRB, 0.11324E+01_JPRB, 0.13276E+01_JPRB, 0.10612E+01_JPRB, 0.10078E+01_JPRB, 0.10234E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.10185E+01_JPRB, 0.10048E+01_JPRB, 0.98974E+00_JPRB, 0.98386E+00_JPRB, 0.96303E+00_JPRB, 0.91350E+00_JPRB ,&
&  0.87509E+00_JPRB, 0.85254E+00_JPRB, 0.84544E+00_JPRB, 0.83370E+00_JPRB, 0.82166E+00_JPRB /)
ZBURD(:, 36,  1) = (/ &
&  0.74517E+00_JPRB, 0.72493E+00_JPRB, 0.78063E+00_JPRB, 0.74355E+00_JPRB, 0.74183E+00_JPRB, 0.75677E+00_JPRB, 0.77362E+00_JPRB ,&
&  0.77760E+00_JPRB, 0.79139E+00_JPRB, 0.81737E+00_JPRB, 0.85828E+00_JPRB, 0.89694E+00_JPRB, 0.92643E+00_JPRB, 0.95965E+00_JPRB ,&
&  0.98565E+00_JPRB, 0.10021E+01_JPRB, 0.10120E+01_JPRB, 0.10225E+01_JPRB, 0.10522E+01_JPRB, 0.10871E+01_JPRB, 0.11395E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.12234E+01_JPRB, 0.12225E+01_JPRB, 0.12546E+01_JPRB, 0.12884E+01_JPRB, 0.14063E+01_JPRB, 0.16448E+01_JPRB ,&
&  0.12659E+01_JPRB, 0.12943E+01_JPRB, 0.12113E+01_JPRB, 0.11365E+01_JPRB, 0.11575E+01_JPRB, 0.11279E+01_JPRB, 0.11597E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.12326E+01_JPRB, 0.12702E+01_JPRB, 0.12987E+01_JPRB, 0.13330E+01_JPRB, 0.13865E+01_JPRB, 0.14428E+01_JPRB ,&
&  0.15230E+01_JPRB, 0.16345E+01_JPRB, 0.17314E+01_JPRB, 0.18788E+01_JPRB, 0.20179E+01_JPRB, 0.21477E+01_JPRB, 0.22005E+01_JPRB ,&
&  0.22281E+01_JPRB, 0.22242E+01_JPRB, 0.20680E+01_JPRB, 0.20525E+01_JPRB, 0.20230E+01_JPRB, 0.19215E+01_JPRB, 0.17314E+01_JPRB ,&
&  0.15523E+01_JPRB, 0.15701E+01_JPRB, 0.16315E+01_JPRB, 0.16001E+01_JPRB, 0.16178E+01_JPRB, 0.16052E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.15827E+01_JPRB, 0.14990E+01_JPRB, 0.14188E+01_JPRB, 0.13220E+01_JPRB, 0.12889E+01_JPRB, 0.12973E+01_JPRB, 0.13039E+01_JPRB ,&
&  0.12446E+01_JPRB, 0.11819E+01_JPRB, 0.11430E+01_JPRB, 0.11779E+01_JPRB, 0.12576E+01_JPRB, 0.16070E+01_JPRB, 0.13445E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.95275E+00_JPRB, 0.76440E+00_JPRB, 0.10168E+01_JPRB, 0.93774E+00_JPRB, 0.92196E+00_JPRB, 0.95788E+00_JPRB ,&
&  0.97426E+00_JPRB, 0.96865E+00_JPRB, 0.10034E+01_JPRB, 0.96538E+00_JPRB, 0.93213E+00_JPRB, 0.91415E+00_JPRB, 0.90999E+00_JPRB ,&
&  0.85813E+00_JPRB, 0.80444E+00_JPRB, 0.75133E+00_JPRB, 0.75701E+00_JPRB, 0.71499E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART1',1,ZHOOK_HANDLE)
END SUBROUTINE PART1



SUBROUTINE PART2

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART2',0,ZHOOK_HANDLE)


ZBURD(:, 37,  1) = (/ &
&  0.65275E+00_JPRB, 0.68817E+00_JPRB, 0.78171E+00_JPRB, 0.71597E+00_JPRB, 0.71469E+00_JPRB, 0.73133E+00_JPRB, 0.75708E+00_JPRB ,&
&  0.78457E+00_JPRB, 0.81983E+00_JPRB, 0.85761E+00_JPRB, 0.90921E+00_JPRB, 0.95467E+00_JPRB, 0.99418E+00_JPRB, 0.10318E+01_JPRB ,&
&  0.10662E+01_JPRB, 0.10963E+01_JPRB, 0.11253E+01_JPRB, 0.11400E+01_JPRB, 0.11522E+01_JPRB, 0.11775E+01_JPRB, 0.12099E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.12988E+01_JPRB, 0.13345E+01_JPRB, 0.13928E+01_JPRB, 0.14351E+01_JPRB, 0.14862E+01_JPRB, 0.12992E+01_JPRB ,&
&  0.14080E+01_JPRB, 0.12505E+01_JPRB, 0.11784E+01_JPRB, 0.11467E+01_JPRB, 0.11104E+01_JPRB, 0.10850E+01_JPRB, 0.10836E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.10793E+01_JPRB, 0.10878E+01_JPRB, 0.10679E+01_JPRB, 0.10819E+01_JPRB, 0.11354E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.12400E+01_JPRB, 0.13402E+01_JPRB, 0.14461E+01_JPRB, 0.15217E+01_JPRB, 0.15923E+01_JPRB, 0.15838E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.15914E+01_JPRB, 0.16489E+01_JPRB, 0.16018E+01_JPRB, 0.16166E+01_JPRB, 0.15885E+01_JPRB, 0.14593E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.12829E+01_JPRB, 0.13938E+01_JPRB, 0.17101E+01_JPRB, 0.15867E+01_JPRB, 0.14437E+01_JPRB, 0.14152E+01_JPRB ,&
&  0.13358E+01_JPRB, 0.12310E+01_JPRB, 0.11532E+01_JPRB, 0.10362E+01_JPRB, 0.10301E+01_JPRB, 0.10378E+01_JPRB, 0.10484E+01_JPRB ,&
&  0.99868E+00_JPRB, 0.94684E+00_JPRB, 0.91542E+00_JPRB, 0.97622E+00_JPRB, 0.12737E+01_JPRB, 0.13349E+01_JPRB, 0.15071E+01_JPRB ,&
&  0.91307E+00_JPRB, 0.58803E+00_JPRB, 0.69104E+00_JPRB, 0.77317E+00_JPRB, 0.76929E+00_JPRB, 0.77855E+00_JPRB, 0.84045E+00_JPRB ,&
&  0.91609E+00_JPRB, 0.91667E+00_JPRB, 0.89223E+00_JPRB, 0.89475E+00_JPRB, 0.88887E+00_JPRB, 0.88978E+00_JPRB, 0.81043E+00_JPRB ,&
&  0.78486E+00_JPRB, 0.72727E+00_JPRB, 0.69261E+00_JPRB, 0.67481E+00_JPRB, 0.63830E+00_JPRB /)
ZBURD(:, 38,  1) = (/ &
&  0.61883E+00_JPRB, 0.69206E+00_JPRB, 0.76159E+00_JPRB, 0.69505E+00_JPRB, 0.70582E+00_JPRB, 0.72907E+00_JPRB, 0.76296E+00_JPRB ,&
&  0.80131E+00_JPRB, 0.84644E+00_JPRB, 0.89331E+00_JPRB, 0.94819E+00_JPRB, 0.99245E+00_JPRB, 0.10335E+01_JPRB, 0.10776E+01_JPRB ,&
&  0.11228E+01_JPRB, 0.11413E+01_JPRB, 0.11789E+01_JPRB, 0.11889E+01_JPRB, 0.12151E+01_JPRB, 0.12202E+01_JPRB, 0.12423E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.13289E+01_JPRB, 0.13671E+01_JPRB, 0.14001E+01_JPRB, 0.14040E+01_JPRB, 0.14840E+01_JPRB, 0.11835E+01_JPRB ,&
&  0.13666E+01_JPRB, 0.12207E+01_JPRB, 0.11366E+01_JPRB, 0.11338E+01_JPRB, 0.11028E+01_JPRB, 0.10935E+01_JPRB, 0.10844E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10775E+01_JPRB, 0.10757E+01_JPRB, 0.10470E+01_JPRB, 0.97117E+00_JPRB, 0.99513E+00_JPRB, 0.10160E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.11223E+01_JPRB, 0.11713E+01_JPRB, 0.12054E+01_JPRB, 0.12254E+01_JPRB, 0.12068E+01_JPRB, 0.11798E+01_JPRB ,&
&  0.11544E+01_JPRB, 0.11575E+01_JPRB, 0.10855E+01_JPRB, 0.11038E+01_JPRB, 0.11749E+01_JPRB, 0.10157E+01_JPRB, 0.80479E+00_JPRB ,&
&  0.70495E+00_JPRB, 0.97047E+00_JPRB, 0.13622E+01_JPRB, 0.16807E+01_JPRB, 0.14788E+01_JPRB, 0.13526E+01_JPRB, 0.12664E+01_JPRB ,&
&  0.11438E+01_JPRB, 0.10231E+01_JPRB, 0.94073E+00_JPRB, 0.84421E+00_JPRB, 0.80785E+00_JPRB, 0.79791E+00_JPRB, 0.79990E+00_JPRB ,&
&  0.78037E+00_JPRB, 0.76349E+00_JPRB, 0.76666E+00_JPRB, 0.79075E+00_JPRB, 0.89803E+00_JPRB, 0.98189E+00_JPRB, 0.16786E+01_JPRB ,&
&  0.94736E+00_JPRB, 0.67481E+00_JPRB, 0.10162E+01_JPRB, 0.49113E+00_JPRB, 0.60114E+00_JPRB, 0.64928E+00_JPRB, 0.69178E+00_JPRB ,&
&  0.65000E+00_JPRB, 0.82210E+00_JPRB, 0.11760E+01_JPRB, 0.12069E+01_JPRB, 0.12649E+01_JPRB, 0.13401E+01_JPRB, 0.82031E+00_JPRB ,&
&  0.67563E+00_JPRB, 0.66787E+00_JPRB, 0.67210E+00_JPRB, 0.61978E+00_JPRB, 0.60889E+00_JPRB /)
ZBURD(:, 39,  1) = (/ &
&  0.58727E+00_JPRB, 0.64457E+00_JPRB, 0.71471E+00_JPRB, 0.76333E+00_JPRB, 0.74544E+00_JPRB, 0.74677E+00_JPRB, 0.76607E+00_JPRB ,&
&  0.79242E+00_JPRB, 0.82678E+00_JPRB, 0.86252E+00_JPRB, 0.90915E+00_JPRB, 0.94232E+00_JPRB, 0.98422E+00_JPRB, 0.10416E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.11299E+01_JPRB, 0.11568E+01_JPRB, 0.11868E+01_JPRB, 0.12126E+01_JPRB, 0.12397E+01_JPRB, 0.12650E+01_JPRB ,&
&  0.12773E+01_JPRB, 0.13021E+01_JPRB, 0.13181E+01_JPRB, 0.13747E+01_JPRB, 0.14138E+01_JPRB, 0.15836E+01_JPRB, 0.11503E+01_JPRB ,&
&  0.12943E+01_JPRB, 0.11801E+01_JPRB, 0.11011E+01_JPRB, 0.11123E+01_JPRB, 0.10972E+01_JPRB, 0.10751E+01_JPRB, 0.10588E+01_JPRB ,&
&  0.10814E+01_JPRB, 0.10906E+01_JPRB, 0.10618E+01_JPRB, 0.10896E+01_JPRB, 0.10420E+01_JPRB, 0.10129E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.11484E+01_JPRB, 0.12030E+01_JPRB, 0.12202E+01_JPRB, 0.12510E+01_JPRB, 0.12557E+01_JPRB, 0.12362E+01_JPRB, 0.12051E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11529E+01_JPRB, 0.11531E+01_JPRB, 0.11569E+01_JPRB, 0.11597E+01_JPRB, 0.10224E+01_JPRB, 0.84092E+00_JPRB ,&
&  0.66875E+00_JPRB, 0.69576E+00_JPRB, 0.11341E+01_JPRB, 0.14997E+01_JPRB, 0.13432E+01_JPRB, 0.12547E+01_JPRB, 0.11053E+01_JPRB ,&
&  0.10167E+01_JPRB, 0.90170E+00_JPRB, 0.83163E+00_JPRB, 0.75153E+00_JPRB, 0.71480E+00_JPRB, 0.70508E+00_JPRB, 0.73783E+00_JPRB ,&
&  0.75457E+00_JPRB, 0.73759E+00_JPRB, 0.73410E+00_JPRB, 0.73621E+00_JPRB, 0.85124E+00_JPRB, 0.10700E+01_JPRB, 0.10941E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.81746E+00_JPRB, 0.71738E+00_JPRB, 0.74028E+00_JPRB, 0.57890E+00_JPRB, 0.61209E+00_JPRB, 0.66561E+00_JPRB ,&
&  0.84827E+00_JPRB, 0.92830E+00_JPRB, 0.12664E+01_JPRB, 0.34781E+01_JPRB, 0.39897E+01_JPRB, 0.49857E+01_JPRB, 0.20093E+01_JPRB ,&
&  0.95584E+00_JPRB, 0.65005E+00_JPRB, 0.58575E+00_JPRB, 0.58713E+00_JPRB, 0.57638E+00_JPRB /)
ZBURD(:, 40,  1) = (/ &
&  0.55499E+00_JPRB, 0.60017E+00_JPRB, 0.63820E+00_JPRB, 0.69486E+00_JPRB, 0.71637E+00_JPRB, 0.72366E+00_JPRB, 0.72485E+00_JPRB ,&
&  0.71585E+00_JPRB, 0.72353E+00_JPRB, 0.74745E+00_JPRB, 0.80453E+00_JPRB, 0.86567E+00_JPRB, 0.92646E+00_JPRB, 0.99248E+00_JPRB ,&
&  0.10761E+01_JPRB, 0.11345E+01_JPRB, 0.11277E+01_JPRB, 0.11407E+01_JPRB, 0.11773E+01_JPRB, 0.12151E+01_JPRB, 0.12306E+01_JPRB ,&
&  0.12362E+01_JPRB, 0.12657E+01_JPRB, 0.13027E+01_JPRB, 0.13651E+01_JPRB, 0.14462E+01_JPRB, 0.15209E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.12532E+01_JPRB, 0.11512E+01_JPRB, 0.11240E+01_JPRB, 0.11348E+01_JPRB, 0.10847E+01_JPRB, 0.10200E+01_JPRB, 0.10111E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.11100E+01_JPRB, 0.11350E+01_JPRB, 0.11225E+01_JPRB, 0.11998E+01_JPRB, 0.11579E+01_JPRB, 0.12841E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.14378E+01_JPRB, 0.14622E+01_JPRB, 0.14869E+01_JPRB, 0.15153E+01_JPRB, 0.15628E+01_JPRB, 0.16171E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.16483E+01_JPRB, 0.16360E+01_JPRB, 0.16100E+01_JPRB, 0.14255E+01_JPRB, 0.14251E+01_JPRB, 0.11459E+01_JPRB ,&
&  0.81971E+00_JPRB, 0.72155E+00_JPRB, 0.10128E+01_JPRB, 0.13158E+01_JPRB, 0.12456E+01_JPRB, 0.11515E+01_JPRB, 0.10424E+01_JPRB ,&
&  0.92084E+00_JPRB, 0.83460E+00_JPRB, 0.77293E+00_JPRB, 0.71544E+00_JPRB, 0.66799E+00_JPRB, 0.65671E+00_JPRB, 0.66712E+00_JPRB ,&
&  0.67471E+00_JPRB, 0.67264E+00_JPRB, 0.67458E+00_JPRB, 0.70816E+00_JPRB, 0.80795E+00_JPRB, 0.10733E+01_JPRB, 0.14973E+01_JPRB ,&
&  0.17752E+01_JPRB, 0.86411E+00_JPRB, 0.71346E+00_JPRB, 0.68285E+00_JPRB, 0.68327E+00_JPRB, 0.71389E+00_JPRB, 0.76402E+00_JPRB ,&
&  0.91539E+00_JPRB, 0.11756E+01_JPRB, 0.10921E+01_JPRB, 0.12822E+01_JPRB, 0.25564E+01_JPRB, 0.27785E+01_JPRB, 0.16123E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.73220E+00_JPRB, 0.59854E+00_JPRB, 0.55860E+00_JPRB, 0.54897E+00_JPRB /)
ZBURD(:, 41,  1) = (/ &
&  0.55608E+00_JPRB, 0.56947E+00_JPRB, 0.58926E+00_JPRB, 0.61413E+00_JPRB, 0.61210E+00_JPRB, 0.61399E+00_JPRB, 0.60191E+00_JPRB ,&
&  0.60039E+00_JPRB, 0.62004E+00_JPRB, 0.66873E+00_JPRB, 0.75070E+00_JPRB, 0.81213E+00_JPRB, 0.89705E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.10551E+01_JPRB, 0.10437E+01_JPRB, 0.10396E+01_JPRB, 0.10502E+01_JPRB, 0.10768E+01_JPRB, 0.11123E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11801E+01_JPRB, 0.12562E+01_JPRB, 0.13222E+01_JPRB, 0.14105E+01_JPRB, 0.14963E+01_JPRB, 0.15473E+01_JPRB, 0.16469E+01_JPRB ,&
&  0.92854E+00_JPRB, 0.12186E+01_JPRB, 0.12031E+01_JPRB, 0.11731E+01_JPRB, 0.10104E+01_JPRB, 0.92001E+00_JPRB, 0.94850E+00_JPRB ,&
&  0.10892E+01_JPRB, 0.12137E+01_JPRB, 0.13663E+01_JPRB, 0.12077E+01_JPRB, 0.12896E+01_JPRB, 0.13296E+01_JPRB, 0.14696E+01_JPRB ,&
&  0.15231E+01_JPRB, 0.15722E+01_JPRB, 0.16369E+01_JPRB, 0.17083E+01_JPRB, 0.17769E+01_JPRB, 0.18723E+01_JPRB, 0.19623E+01_JPRB ,&
&  0.20251E+01_JPRB, 0.20314E+01_JPRB, 0.20533E+01_JPRB, 0.18286E+01_JPRB, 0.15818E+01_JPRB, 0.15873E+01_JPRB, 0.15078E+01_JPRB ,&
&  0.12816E+01_JPRB, 0.94059E+00_JPRB, 0.99383E+00_JPRB, 0.12292E+01_JPRB, 0.12093E+01_JPRB, 0.11360E+01_JPRB, 0.10322E+01_JPRB ,&
&  0.90135E+00_JPRB, 0.77528E+00_JPRB, 0.71852E+00_JPRB, 0.66639E+00_JPRB, 0.63707E+00_JPRB, 0.65294E+00_JPRB, 0.68723E+00_JPRB ,&
&  0.68961E+00_JPRB, 0.68101E+00_JPRB, 0.67627E+00_JPRB, 0.69396E+00_JPRB, 0.74624E+00_JPRB, 0.79713E+00_JPRB, 0.84005E+00_JPRB ,&
&  0.81438E+00_JPRB, 0.78475E+00_JPRB, 0.76923E+00_JPRB, 0.78456E+00_JPRB, 0.80058E+00_JPRB, 0.80975E+00_JPRB, 0.83847E+00_JPRB ,&
&  0.84983E+00_JPRB, 0.84813E+00_JPRB, 0.93626E+00_JPRB, 0.11296E+01_JPRB, 0.12176E+01_JPRB, 0.12935E+01_JPRB, 0.10720E+01_JPRB ,&
&  0.86202E+00_JPRB, 0.72954E+00_JPRB, 0.64416E+00_JPRB, 0.58822E+00_JPRB, 0.54556E+00_JPRB /)
ZBURD(:, 42,  1) = (/ &
&  0.54322E+00_JPRB, 0.54359E+00_JPRB, 0.54108E+00_JPRB, 0.55199E+00_JPRB, 0.55173E+00_JPRB, 0.55292E+00_JPRB, 0.54952E+00_JPRB ,&
&  0.57492E+00_JPRB, 0.62485E+00_JPRB, 0.70169E+00_JPRB, 0.78864E+00_JPRB, 0.85354E+00_JPRB, 0.93376E+00_JPRB, 0.97519E+00_JPRB ,&
&  0.97580E+00_JPRB, 0.96097E+00_JPRB, 0.96154E+00_JPRB, 0.99043E+00_JPRB, 0.10155E+01_JPRB, 0.10556E+01_JPRB, 0.10992E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.12122E+01_JPRB, 0.12972E+01_JPRB, 0.13970E+01_JPRB, 0.15124E+01_JPRB, 0.15961E+01_JPRB, 0.16626E+01_JPRB ,&
&  0.65361E+00_JPRB, 0.86770E+00_JPRB, 0.11954E+01_JPRB, 0.12547E+01_JPRB, 0.97359E+00_JPRB, 0.86674E+00_JPRB, 0.79499E+00_JPRB ,&
&  0.92971E+00_JPRB, 0.11241E+01_JPRB, 0.12352E+01_JPRB, 0.13516E+01_JPRB, 0.13376E+01_JPRB, 0.14359E+01_JPRB, 0.15258E+01_JPRB ,&
&  0.15554E+01_JPRB, 0.15896E+01_JPRB, 0.16609E+01_JPRB, 0.17491E+01_JPRB, 0.18256E+01_JPRB, 0.19105E+01_JPRB, 0.19969E+01_JPRB ,&
&  0.20886E+01_JPRB, 0.21126E+01_JPRB, 0.21566E+01_JPRB, 0.18211E+01_JPRB, 0.18427E+01_JPRB, 0.18957E+01_JPRB, 0.21089E+01_JPRB ,&
&  0.18255E+01_JPRB, 0.11882E+01_JPRB, 0.10059E+01_JPRB, 0.11850E+01_JPRB, 0.11822E+01_JPRB, 0.11342E+01_JPRB, 0.10009E+01_JPRB ,&
&  0.86224E+00_JPRB, 0.76183E+00_JPRB, 0.71384E+00_JPRB, 0.69707E+00_JPRB, 0.69320E+00_JPRB, 0.72267E+00_JPRB, 0.75338E+00_JPRB ,&
&  0.74789E+00_JPRB, 0.73803E+00_JPRB, 0.72894E+00_JPRB, 0.73095E+00_JPRB, 0.77427E+00_JPRB, 0.82214E+00_JPRB, 0.85177E+00_JPRB ,&
&  0.86334E+00_JPRB, 0.87539E+00_JPRB, 0.90346E+00_JPRB, 0.93475E+00_JPRB, 0.96807E+00_JPRB, 0.98408E+00_JPRB, 0.99627E+00_JPRB ,&
&  0.88658E+00_JPRB, 0.82764E+00_JPRB, 0.80229E+00_JPRB, 0.79266E+00_JPRB, 0.79023E+00_JPRB, 0.81918E+00_JPRB, 0.77064E+00_JPRB ,&
&  0.74010E+00_JPRB, 0.72560E+00_JPRB, 0.69125E+00_JPRB, 0.62086E+00_JPRB, 0.55569E+00_JPRB /)
ZBURD(:, 43,  1) = (/ &
&  0.55801E+00_JPRB, 0.55759E+00_JPRB, 0.54872E+00_JPRB, 0.54828E+00_JPRB, 0.54096E+00_JPRB, 0.54769E+00_JPRB, 0.56243E+00_JPRB ,&
&  0.59416E+00_JPRB, 0.64999E+00_JPRB, 0.71957E+00_JPRB, 0.78216E+00_JPRB, 0.85581E+00_JPRB, 0.91372E+00_JPRB, 0.92128E+00_JPRB ,&
&  0.90846E+00_JPRB, 0.90848E+00_JPRB, 0.90328E+00_JPRB, 0.93331E+00_JPRB, 0.96338E+00_JPRB, 0.10074E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.12089E+01_JPRB, 0.13125E+01_JPRB, 0.14355E+01_JPRB, 0.16043E+01_JPRB, 0.17962E+01_JPRB, 0.19927E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.74525E+00_JPRB, 0.12642E+01_JPRB, 0.13934E+01_JPRB, 0.10793E+01_JPRB, 0.95536E+00_JPRB, 0.80859E+00_JPRB ,&
&  0.77602E+00_JPRB, 0.10262E+01_JPRB, 0.11551E+01_JPRB, 0.12749E+01_JPRB, 0.13134E+01_JPRB, 0.14121E+01_JPRB, 0.14895E+01_JPRB ,&
&  0.15299E+01_JPRB, 0.15649E+01_JPRB, 0.16053E+01_JPRB, 0.16496E+01_JPRB, 0.17162E+01_JPRB, 0.17894E+01_JPRB, 0.18849E+01_JPRB ,&
&  0.20023E+01_JPRB, 0.20548E+01_JPRB, 0.20878E+01_JPRB, 0.17809E+01_JPRB, 0.18603E+01_JPRB, 0.17558E+01_JPRB, 0.16828E+01_JPRB ,&
&  0.15836E+01_JPRB, 0.10624E+01_JPRB, 0.91000E+00_JPRB, 0.11479E+01_JPRB, 0.11267E+01_JPRB, 0.89790E+00_JPRB, 0.96721E+00_JPRB ,&
&  0.87022E+00_JPRB, 0.78049E+00_JPRB, 0.76954E+00_JPRB, 0.78042E+00_JPRB, 0.75440E+00_JPRB, 0.76165E+00_JPRB, 0.77141E+00_JPRB ,&
&  0.80279E+00_JPRB, 0.79066E+00_JPRB, 0.77093E+00_JPRB, 0.78970E+00_JPRB, 0.82295E+00_JPRB, 0.86932E+00_JPRB, 0.90679E+00_JPRB ,&
&  0.95175E+00_JPRB, 0.99118E+00_JPRB, 0.10294E+01_JPRB, 0.10742E+01_JPRB, 0.11110E+01_JPRB, 0.10681E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.96167E+00_JPRB, 0.92614E+00_JPRB, 0.87648E+00_JPRB, 0.84337E+00_JPRB, 0.80020E+00_JPRB, 0.80549E+00_JPRB, 0.83128E+00_JPRB ,&
&  0.85634E+00_JPRB, 0.85681E+00_JPRB, 0.78227E+00_JPRB, 0.65323E+00_JPRB, 0.58005E+00_JPRB /)
ZBURD(:, 44,  1) = (/ &
&  0.55719E+00_JPRB, 0.54605E+00_JPRB, 0.53700E+00_JPRB, 0.55090E+00_JPRB, 0.54971E+00_JPRB, 0.55828E+00_JPRB, 0.57774E+00_JPRB ,&
&  0.60894E+00_JPRB, 0.64956E+00_JPRB, 0.68207E+00_JPRB, 0.72882E+00_JPRB, 0.79304E+00_JPRB, 0.85339E+00_JPRB, 0.86295E+00_JPRB ,&
&  0.85304E+00_JPRB, 0.84969E+00_JPRB, 0.85790E+00_JPRB, 0.87178E+00_JPRB, 0.90598E+00_JPRB, 0.94902E+00_JPRB, 0.99494E+00_JPRB ,&
&  0.10606E+01_JPRB, 0.11512E+01_JPRB, 0.12481E+01_JPRB, 0.13605E+01_JPRB, 0.15066E+01_JPRB, 0.17399E+01_JPRB, 0.20740E+01_JPRB ,&
&  0.29963E+01_JPRB, 0.30262E+01_JPRB, 0.11419E+01_JPRB, 0.14485E+01_JPRB, 0.12867E+01_JPRB, 0.11011E+01_JPRB, 0.88435E+00_JPRB ,&
&  0.81751E+00_JPRB, 0.94798E+00_JPRB, 0.10907E+01_JPRB, 0.12190E+01_JPRB, 0.12067E+01_JPRB, 0.12913E+01_JPRB, 0.13677E+01_JPRB ,&
&  0.14516E+01_JPRB, 0.14962E+01_JPRB, 0.15087E+01_JPRB, 0.15089E+01_JPRB, 0.15338E+01_JPRB, 0.15875E+01_JPRB, 0.16872E+01_JPRB ,&
&  0.18280E+01_JPRB, 0.19591E+01_JPRB, 0.20405E+01_JPRB, 0.18399E+01_JPRB, 0.18811E+01_JPRB, 0.16598E+01_JPRB, 0.15030E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.11205E+01_JPRB, 0.11417E+01_JPRB, 0.11717E+01_JPRB, 0.10430E+01_JPRB, 0.93875E+00_JPRB, 0.10150E+01_JPRB ,&
&  0.95042E+00_JPRB, 0.85992E+00_JPRB, 0.82261E+00_JPRB, 0.80486E+00_JPRB, 0.79789E+00_JPRB, 0.79493E+00_JPRB, 0.81054E+00_JPRB ,&
&  0.83965E+00_JPRB, 0.84414E+00_JPRB, 0.82413E+00_JPRB, 0.85285E+00_JPRB, 0.88663E+00_JPRB, 0.92627E+00_JPRB, 0.96365E+00_JPRB ,&
&  0.10122E+01_JPRB, 0.10711E+01_JPRB, 0.11181E+01_JPRB, 0.11480E+01_JPRB, 0.11143E+01_JPRB, 0.10766E+01_JPRB, 0.11063E+01_JPRB ,&
&  0.10507E+01_JPRB, 0.10367E+01_JPRB, 0.10026E+01_JPRB, 0.96610E+00_JPRB, 0.90672E+00_JPRB, 0.91698E+00_JPRB, 0.91782E+00_JPRB ,&
&  0.94992E+00_JPRB, 0.97622E+00_JPRB, 0.88645E+00_JPRB, 0.67658E+00_JPRB, 0.59275E+00_JPRB /)
ZBURD(:, 45,  1) = (/ &
&  0.59390E+00_JPRB, 0.58141E+00_JPRB, 0.57669E+00_JPRB, 0.58308E+00_JPRB, 0.58711E+00_JPRB, 0.57885E+00_JPRB, 0.58140E+00_JPRB ,&
&  0.59223E+00_JPRB, 0.60735E+00_JPRB, 0.60680E+00_JPRB, 0.63952E+00_JPRB, 0.71458E+00_JPRB, 0.79361E+00_JPRB, 0.80212E+00_JPRB ,&
&  0.79288E+00_JPRB, 0.78131E+00_JPRB, 0.77643E+00_JPRB, 0.79208E+00_JPRB, 0.81051E+00_JPRB, 0.86103E+00_JPRB, 0.91509E+00_JPRB ,&
&  0.97131E+00_JPRB, 0.10329E+01_JPRB, 0.10861E+01_JPRB, 0.11347E+01_JPRB, 0.12270E+01_JPRB, 0.13812E+01_JPRB, 0.17895E+01_JPRB ,&
&  0.25412E+01_JPRB, 0.47641E+01_JPRB, 0.14128E+01_JPRB, 0.14469E+01_JPRB, 0.13997E+01_JPRB, 0.12495E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10562E+01_JPRB, 0.96512E+00_JPRB, 0.10744E+01_JPRB, 0.11647E+01_JPRB, 0.10996E+01_JPRB, 0.11285E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.12996E+01_JPRB, 0.13688E+01_JPRB, 0.14024E+01_JPRB, 0.13822E+01_JPRB, 0.13599E+01_JPRB, 0.13544E+01_JPRB, 0.13911E+01_JPRB ,&
&  0.14994E+01_JPRB, 0.16971E+01_JPRB, 0.18884E+01_JPRB, 0.18420E+01_JPRB, 0.19226E+01_JPRB, 0.17268E+01_JPRB, 0.16013E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.13432E+01_JPRB, 0.13942E+01_JPRB, 0.13336E+01_JPRB, 0.11322E+01_JPRB, 0.10347E+01_JPRB, 0.10635E+01_JPRB ,&
&  0.10239E+01_JPRB, 0.94601E+00_JPRB, 0.87153E+00_JPRB, 0.81735E+00_JPRB, 0.81846E+00_JPRB, 0.84224E+00_JPRB, 0.83188E+00_JPRB ,&
&  0.86861E+00_JPRB, 0.91201E+00_JPRB, 0.89488E+00_JPRB, 0.93081E+00_JPRB, 0.95925E+00_JPRB, 0.98323E+00_JPRB, 0.10047E+01_JPRB ,&
&  0.10321E+01_JPRB, 0.10967E+01_JPRB, 0.11446E+01_JPRB, 0.11285E+01_JPRB, 0.10729E+01_JPRB, 0.10548E+01_JPRB, 0.11049E+01_JPRB ,&
&  0.11672E+01_JPRB, 0.12580E+01_JPRB, 0.10926E+01_JPRB, 0.10053E+01_JPRB, 0.98881E+00_JPRB, 0.96097E+00_JPRB, 0.94620E+00_JPRB ,&
&  0.96650E+00_JPRB, 0.10283E+01_JPRB, 0.10600E+01_JPRB, 0.76126E+00_JPRB, 0.65028E+00_JPRB /)
ZBURD(:, 46,  1) = (/ &
&  0.66462E+00_JPRB, 0.64318E+00_JPRB, 0.62295E+00_JPRB, 0.60593E+00_JPRB, 0.59617E+00_JPRB, 0.60207E+00_JPRB, 0.60697E+00_JPRB ,&
&  0.60236E+00_JPRB, 0.60369E+00_JPRB, 0.59871E+00_JPRB, 0.60309E+00_JPRB, 0.62626E+00_JPRB, 0.67439E+00_JPRB, 0.72540E+00_JPRB ,&
&  0.72728E+00_JPRB, 0.69740E+00_JPRB, 0.69167E+00_JPRB, 0.68647E+00_JPRB, 0.70555E+00_JPRB, 0.73355E+00_JPRB, 0.78809E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.92555E+00_JPRB, 0.95493E+00_JPRB, 0.96579E+00_JPRB, 0.99592E+00_JPRB, 0.10529E+01_JPRB, 0.14161E+01_JPRB ,&
&  0.21962E+01_JPRB, 0.45937E+01_JPRB, 0.33160E+01_JPRB, 0.22413E+01_JPRB, 0.15350E+01_JPRB, 0.12994E+01_JPRB, 0.12606E+01_JPRB ,&
&  0.13858E+01_JPRB, 0.13265E+01_JPRB, 0.13106E+01_JPRB, 0.11428E+01_JPRB, 0.10278E+01_JPRB, 0.97918E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11962E+01_JPRB, 0.12499E+01_JPRB, 0.12554E+01_JPRB, 0.12242E+01_JPRB, 0.12049E+01_JPRB, 0.12009E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.12719E+01_JPRB, 0.15670E+01_JPRB, 0.18236E+01_JPRB, 0.19647E+01_JPRB, 0.18524E+01_JPRB, 0.17781E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.16606E+01_JPRB, 0.16025E+01_JPRB, 0.14455E+01_JPRB, 0.12076E+01_JPRB, 0.12354E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10022E+01_JPRB, 0.93558E+00_JPRB, 0.85742E+00_JPRB, 0.86532E+00_JPRB, 0.87095E+00_JPRB, 0.85276E+00_JPRB ,&
&  0.88927E+00_JPRB, 0.93271E+00_JPRB, 0.92133E+00_JPRB, 0.93544E+00_JPRB, 0.95003E+00_JPRB, 0.97851E+00_JPRB, 0.10060E+01_JPRB ,&
&  0.10300E+01_JPRB, 0.10910E+01_JPRB, 0.10465E+01_JPRB, 0.99511E+00_JPRB, 0.99581E+00_JPRB, 0.99831E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.10834E+01_JPRB, 0.11796E+01_JPRB, 0.11164E+01_JPRB, 0.11002E+01_JPRB, 0.10355E+01_JPRB, 0.10134E+01_JPRB, 0.93904E+00_JPRB ,&
&  0.91995E+00_JPRB, 0.88873E+00_JPRB, 0.90063E+00_JPRB, 0.82464E+00_JPRB, 0.72693E+00_JPRB /)
ZBURD(:, 47,  1) = (/ &
&  0.71818E+00_JPRB, 0.69262E+00_JPRB, 0.64120E+00_JPRB, 0.59992E+00_JPRB, 0.59545E+00_JPRB, 0.60231E+00_JPRB, 0.61313E+00_JPRB ,&
&  0.61735E+00_JPRB, 0.61832E+00_JPRB, 0.61541E+00_JPRB, 0.60846E+00_JPRB, 0.61177E+00_JPRB, 0.59895E+00_JPRB, 0.62246E+00_JPRB ,&
&  0.64856E+00_JPRB, 0.64993E+00_JPRB, 0.65922E+00_JPRB, 0.63589E+00_JPRB, 0.63325E+00_JPRB, 0.63031E+00_JPRB, 0.65976E+00_JPRB ,&
&  0.72791E+00_JPRB, 0.81216E+00_JPRB, 0.86327E+00_JPRB, 0.87923E+00_JPRB, 0.89630E+00_JPRB, 0.95458E+00_JPRB, 0.11923E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.39449E+01_JPRB, 0.18946E+01_JPRB, 0.27323E+01_JPRB, 0.18380E+01_JPRB, 0.14067E+01_JPRB, 0.11593E+01_JPRB ,&
&  0.12393E+01_JPRB, 0.14901E+01_JPRB, 0.12639E+01_JPRB, 0.11092E+01_JPRB, 0.98133E+00_JPRB, 0.93074E+00_JPRB, 0.91764E+00_JPRB ,&
&  0.94251E+00_JPRB, 0.10187E+01_JPRB, 0.10872E+01_JPRB, 0.11192E+01_JPRB, 0.11000E+01_JPRB, 0.10627E+01_JPRB, 0.10580E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10587E+01_JPRB, 0.11848E+01_JPRB, 0.13368E+01_JPRB, 0.17857E+01_JPRB, 0.19642E+01_JPRB, 0.20242E+01_JPRB ,&
&  0.20841E+01_JPRB, 0.20293E+01_JPRB, 0.17943E+01_JPRB, 0.15794E+01_JPRB, 0.13813E+01_JPRB, 0.13140E+01_JPRB, 0.11756E+01_JPRB ,&
&  0.10882E+01_JPRB, 0.10262E+01_JPRB, 0.96656E+00_JPRB, 0.90252E+00_JPRB, 0.87314E+00_JPRB, 0.86797E+00_JPRB, 0.85092E+00_JPRB ,&
&  0.88254E+00_JPRB, 0.92143E+00_JPRB, 0.91632E+00_JPRB, 0.91303E+00_JPRB, 0.92105E+00_JPRB, 0.94379E+00_JPRB, 0.97107E+00_JPRB ,&
&  0.10111E+01_JPRB, 0.10755E+01_JPRB, 0.10261E+01_JPRB, 0.90803E+00_JPRB, 0.91774E+00_JPRB, 0.97170E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11771E+01_JPRB, 0.11519E+01_JPRB, 0.11007E+01_JPRB, 0.10935E+01_JPRB, 0.11266E+01_JPRB, 0.98822E+00_JPRB ,&
&  0.92517E+00_JPRB, 0.88540E+00_JPRB, 0.85380E+00_JPRB, 0.82234E+00_JPRB, 0.77657E+00_JPRB /)
ZBURD(:, 48,  1) = (/ &
&  0.72608E+00_JPRB, 0.68002E+00_JPRB, 0.62712E+00_JPRB, 0.60348E+00_JPRB, 0.60906E+00_JPRB, 0.59714E+00_JPRB, 0.60536E+00_JPRB ,&
&  0.62661E+00_JPRB, 0.62875E+00_JPRB, 0.62846E+00_JPRB, 0.61830E+00_JPRB, 0.60767E+00_JPRB, 0.59003E+00_JPRB, 0.57545E+00_JPRB ,&
&  0.59098E+00_JPRB, 0.63660E+00_JPRB, 0.65407E+00_JPRB, 0.64215E+00_JPRB, 0.63039E+00_JPRB, 0.61268E+00_JPRB, 0.60971E+00_JPRB ,&
&  0.63836E+00_JPRB, 0.69612E+00_JPRB, 0.75784E+00_JPRB, 0.79146E+00_JPRB, 0.81248E+00_JPRB, 0.84176E+00_JPRB, 0.99394E+00_JPRB ,&
&  0.14505E+01_JPRB, 0.38550E+01_JPRB, 0.16407E+01_JPRB, 0.28634E+01_JPRB, 0.20803E+01_JPRB, 0.16116E+01_JPRB, 0.12271E+01_JPRB ,&
&  0.11832E+01_JPRB, 0.12919E+01_JPRB, 0.11824E+01_JPRB, 0.10934E+01_JPRB, 0.10042E+01_JPRB, 0.94320E+00_JPRB, 0.92435E+00_JPRB ,&
&  0.90950E+00_JPRB, 0.90708E+00_JPRB, 0.92832E+00_JPRB, 0.95027E+00_JPRB, 0.95296E+00_JPRB, 0.93745E+00_JPRB, 0.92592E+00_JPRB ,&
&  0.92817E+00_JPRB, 0.93461E+00_JPRB, 0.10055E+01_JPRB, 0.11531E+01_JPRB, 0.14224E+01_JPRB, 0.18489E+01_JPRB, 0.23415E+01_JPRB ,&
&  0.25629E+01_JPRB, 0.24789E+01_JPRB, 0.20771E+01_JPRB, 0.17940E+01_JPRB, 0.15951E+01_JPRB, 0.14894E+01_JPRB, 0.13313E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.11139E+01_JPRB, 0.10391E+01_JPRB, 0.98566E+00_JPRB, 0.93100E+00_JPRB, 0.91223E+00_JPRB, 0.88849E+00_JPRB ,&
&  0.89313E+00_JPRB, 0.92832E+00_JPRB, 0.92106E+00_JPRB, 0.90663E+00_JPRB, 0.88564E+00_JPRB, 0.89485E+00_JPRB, 0.91236E+00_JPRB ,&
&  0.95027E+00_JPRB, 0.10260E+01_JPRB, 0.10270E+01_JPRB, 0.91560E+00_JPRB, 0.88885E+00_JPRB, 0.93324E+00_JPRB, 0.98609E+00_JPRB ,&
&  0.11090E+01_JPRB, 0.12136E+01_JPRB, 0.12201E+01_JPRB, 0.12214E+01_JPRB, 0.12156E+01_JPRB, 0.12869E+01_JPRB, 0.10745E+01_JPRB ,&
&  0.97082E+00_JPRB, 0.91829E+00_JPRB, 0.87340E+00_JPRB, 0.81542E+00_JPRB, 0.78547E+00_JPRB /)
ZBURD(:, 49,  1) = (/ &
&  0.75703E+00_JPRB, 0.68914E+00_JPRB, 0.64958E+00_JPRB, 0.62108E+00_JPRB, 0.61909E+00_JPRB, 0.61117E+00_JPRB, 0.60806E+00_JPRB ,&
&  0.62721E+00_JPRB, 0.64046E+00_JPRB, 0.65451E+00_JPRB, 0.64244E+00_JPRB, 0.61814E+00_JPRB, 0.60949E+00_JPRB, 0.59591E+00_JPRB ,&
&  0.58471E+00_JPRB, 0.61545E+00_JPRB, 0.64747E+00_JPRB, 0.65834E+00_JPRB, 0.64665E+00_JPRB, 0.61713E+00_JPRB, 0.60300E+00_JPRB ,&
&  0.61118E+00_JPRB, 0.63150E+00_JPRB, 0.66441E+00_JPRB, 0.70065E+00_JPRB, 0.72990E+00_JPRB, 0.75565E+00_JPRB, 0.80453E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.13831E+01_JPRB, 0.22039E+01_JPRB, 0.27403E+01_JPRB, 0.21998E+01_JPRB, 0.17067E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.11982E+01_JPRB, 0.11275E+01_JPRB, 0.10507E+01_JPRB, 0.98098E+00_JPRB, 0.92750E+00_JPRB, 0.90676E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.85047E+00_JPRB, 0.82952E+00_JPRB, 0.82039E+00_JPRB, 0.81559E+00_JPRB, 0.81289E+00_JPRB, 0.81194E+00_JPRB ,&
&  0.82239E+00_JPRB, 0.83515E+00_JPRB, 0.86921E+00_JPRB, 0.98297E+00_JPRB, 0.12720E+01_JPRB, 0.16941E+01_JPRB, 0.23762E+01_JPRB ,&
&  0.29111E+01_JPRB, 0.27099E+01_JPRB, 0.23151E+01_JPRB, 0.20030E+01_JPRB, 0.17793E+01_JPRB, 0.16540E+01_JPRB, 0.15224E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.12799E+01_JPRB, 0.11740E+01_JPRB, 0.11020E+01_JPRB, 0.10153E+01_JPRB, 0.97097E+00_JPRB, 0.96497E+00_JPRB ,&
&  0.94140E+00_JPRB, 0.93534E+00_JPRB, 0.92880E+00_JPRB, 0.94725E+00_JPRB, 0.91988E+00_JPRB, 0.89520E+00_JPRB, 0.88216E+00_JPRB ,&
&  0.89104E+00_JPRB, 0.95839E+00_JPRB, 0.10061E+01_JPRB, 0.93311E+00_JPRB, 0.91446E+00_JPRB, 0.94008E+00_JPRB, 0.99131E+00_JPRB ,&
&  0.10826E+01_JPRB, 0.11937E+01_JPRB, 0.12593E+01_JPRB, 0.13232E+01_JPRB, 0.12672E+01_JPRB, 0.13383E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.10478E+01_JPRB, 0.97238E+00_JPRB, 0.93046E+00_JPRB, 0.88424E+00_JPRB, 0.84126E+00_JPRB /)
ZBURD(:, 50,  1) = (/ &
&  0.89139E+00_JPRB, 0.79488E+00_JPRB, 0.71994E+00_JPRB, 0.67711E+00_JPRB, 0.64269E+00_JPRB, 0.62618E+00_JPRB, 0.61884E+00_JPRB ,&
&  0.62955E+00_JPRB, 0.64112E+00_JPRB, 0.65741E+00_JPRB, 0.65455E+00_JPRB, 0.63306E+00_JPRB, 0.62812E+00_JPRB, 0.63777E+00_JPRB ,&
&  0.62925E+00_JPRB, 0.63285E+00_JPRB, 0.64446E+00_JPRB, 0.63705E+00_JPRB, 0.62487E+00_JPRB, 0.61032E+00_JPRB, 0.59602E+00_JPRB ,&
&  0.59943E+00_JPRB, 0.60566E+00_JPRB, 0.61946E+00_JPRB, 0.64271E+00_JPRB, 0.67427E+00_JPRB, 0.70422E+00_JPRB, 0.72832E+00_JPRB ,&
&  0.78352E+00_JPRB, 0.13873E+01_JPRB, 0.19900E+01_JPRB, 0.23600E+01_JPRB, 0.21594E+01_JPRB, 0.17093E+01_JPRB, 0.14674E+01_JPRB ,&
&  0.12560E+01_JPRB, 0.11477E+01_JPRB, 0.10749E+01_JPRB, 0.10146E+01_JPRB, 0.94516E+00_JPRB, 0.90423E+00_JPRB, 0.88741E+00_JPRB ,&
&  0.86172E+00_JPRB, 0.82242E+00_JPRB, 0.78031E+00_JPRB, 0.74661E+00_JPRB, 0.73148E+00_JPRB, 0.73097E+00_JPRB, 0.73533E+00_JPRB ,&
&  0.74282E+00_JPRB, 0.75453E+00_JPRB, 0.77935E+00_JPRB, 0.82775E+00_JPRB, 0.10756E+01_JPRB, 0.12194E+01_JPRB, 0.19665E+01_JPRB ,&
&  0.24086E+01_JPRB, 0.21279E+01_JPRB, 0.19449E+01_JPRB, 0.18196E+01_JPRB, 0.16908E+01_JPRB, 0.15709E+01_JPRB, 0.15032E+01_JPRB ,&
&  0.14300E+01_JPRB, 0.13374E+01_JPRB, 0.12583E+01_JPRB, 0.12020E+01_JPRB, 0.11421E+01_JPRB, 0.10958E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.97042E+00_JPRB, 0.95322E+00_JPRB, 0.96000E+00_JPRB, 0.96603E+00_JPRB, 0.93491E+00_JPRB, 0.90664E+00_JPRB ,&
&  0.87784E+00_JPRB, 0.89491E+00_JPRB, 0.96420E+00_JPRB, 0.92090E+00_JPRB, 0.92046E+00_JPRB, 0.95805E+00_JPRB, 0.99284E+00_JPRB ,&
&  0.10170E+01_JPRB, 0.11389E+01_JPRB, 0.12478E+01_JPRB, 0.13485E+01_JPRB, 0.14183E+01_JPRB, 0.14621E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.11613E+01_JPRB, 0.10413E+01_JPRB, 0.96542E+00_JPRB, 0.91589E+00_JPRB, 0.93882E+00_JPRB /)
ZBURD(:, 51,  1) = (/ &
&  0.96479E+00_JPRB, 0.89859E+00_JPRB, 0.82493E+00_JPRB, 0.77380E+00_JPRB, 0.73185E+00_JPRB, 0.70024E+00_JPRB, 0.69063E+00_JPRB ,&
&  0.68459E+00_JPRB, 0.67409E+00_JPRB, 0.67823E+00_JPRB, 0.67485E+00_JPRB, 0.66875E+00_JPRB, 0.65762E+00_JPRB, 0.66390E+00_JPRB ,&
&  0.67053E+00_JPRB, 0.66690E+00_JPRB, 0.66006E+00_JPRB, 0.64980E+00_JPRB, 0.63834E+00_JPRB, 0.62341E+00_JPRB, 0.61322E+00_JPRB ,&
&  0.60617E+00_JPRB, 0.60616E+00_JPRB, 0.61246E+00_JPRB, 0.62539E+00_JPRB, 0.64977E+00_JPRB, 0.68138E+00_JPRB, 0.71248E+00_JPRB ,&
&  0.71700E+00_JPRB, 0.94355E+00_JPRB, 0.12842E+01_JPRB, 0.18570E+01_JPRB, 0.20040E+01_JPRB, 0.17144E+01_JPRB, 0.14641E+01_JPRB ,&
&  0.12753E+01_JPRB, 0.11447E+01_JPRB, 0.10545E+01_JPRB, 0.99129E+00_JPRB, 0.92663E+00_JPRB, 0.88955E+00_JPRB, 0.85948E+00_JPRB ,&
&  0.83005E+00_JPRB, 0.79510E+00_JPRB, 0.75374E+00_JPRB, 0.72436E+00_JPRB, 0.71213E+00_JPRB, 0.71176E+00_JPRB, 0.70693E+00_JPRB ,&
&  0.70213E+00_JPRB, 0.71044E+00_JPRB, 0.71768E+00_JPRB, 0.73184E+00_JPRB, 0.87022E+00_JPRB, 0.10313E+01_JPRB, 0.13455E+01_JPRB ,&
&  0.13429E+01_JPRB, 0.12860E+01_JPRB, 0.12751E+01_JPRB, 0.12651E+01_JPRB, 0.12811E+01_JPRB, 0.12894E+01_JPRB, 0.13106E+01_JPRB ,&
&  0.13302E+01_JPRB, 0.13160E+01_JPRB, 0.12807E+01_JPRB, 0.12526E+01_JPRB, 0.12208E+01_JPRB, 0.11957E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10517E+01_JPRB, 0.10263E+01_JPRB, 0.99287E+00_JPRB, 0.97922E+00_JPRB, 0.95771E+00_JPRB, 0.92880E+00_JPRB ,&
&  0.88761E+00_JPRB, 0.85865E+00_JPRB, 0.89380E+00_JPRB, 0.91198E+00_JPRB, 0.91302E+00_JPRB, 0.91330E+00_JPRB, 0.93763E+00_JPRB ,&
&  0.98710E+00_JPRB, 0.11149E+01_JPRB, 0.11915E+01_JPRB, 0.13654E+01_JPRB, 0.14342E+01_JPRB, 0.14404E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.12289E+01_JPRB, 0.11147E+01_JPRB, 0.10084E+01_JPRB, 0.95270E+00_JPRB, 0.99505E+00_JPRB /)
ZBURD(:, 52,  1) = (/ &
&  0.10855E+01_JPRB, 0.10141E+01_JPRB, 0.94879E+00_JPRB, 0.87111E+00_JPRB, 0.81000E+00_JPRB, 0.77907E+00_JPRB, 0.75397E+00_JPRB ,&
&  0.72598E+00_JPRB, 0.70142E+00_JPRB, 0.69481E+00_JPRB, 0.69684E+00_JPRB, 0.69722E+00_JPRB, 0.69358E+00_JPRB, 0.68278E+00_JPRB ,&
&  0.67921E+00_JPRB, 0.68336E+00_JPRB, 0.67702E+00_JPRB, 0.66538E+00_JPRB, 0.65238E+00_JPRB, 0.64115E+00_JPRB, 0.63233E+00_JPRB ,&
&  0.62700E+00_JPRB, 0.62895E+00_JPRB, 0.63006E+00_JPRB, 0.63999E+00_JPRB, 0.65893E+00_JPRB, 0.68248E+00_JPRB, 0.71356E+00_JPRB ,&
&  0.72261E+00_JPRB, 0.68981E+00_JPRB, 0.91625E+00_JPRB, 0.12302E+01_JPRB, 0.14848E+01_JPRB, 0.15109E+01_JPRB, 0.13903E+01_JPRB ,&
&  0.12851E+01_JPRB, 0.11671E+01_JPRB, 0.10739E+01_JPRB, 0.99780E+00_JPRB, 0.93169E+00_JPRB, 0.87896E+00_JPRB, 0.84040E+00_JPRB ,&
&  0.80205E+00_JPRB, 0.77164E+00_JPRB, 0.74282E+00_JPRB, 0.71652E+00_JPRB, 0.70777E+00_JPRB, 0.69967E+00_JPRB, 0.69024E+00_JPRB ,&
&  0.68335E+00_JPRB, 0.68225E+00_JPRB, 0.68366E+00_JPRB, 0.68897E+00_JPRB, 0.70929E+00_JPRB, 0.76863E+00_JPRB, 0.86037E+00_JPRB ,&
&  0.90537E+00_JPRB, 0.93411E+00_JPRB, 0.93881E+00_JPRB, 0.93558E+00_JPRB, 0.94948E+00_JPRB, 0.98053E+00_JPRB, 0.10341E+01_JPRB ,&
&  0.10998E+01_JPRB, 0.11481E+01_JPRB, 0.11806E+01_JPRB, 0.12022E+01_JPRB, 0.11969E+01_JPRB, 0.11767E+01_JPRB, 0.11454E+01_JPRB ,&
&  0.11121E+01_JPRB, 0.10642E+01_JPRB, 0.10212E+01_JPRB, 0.98841E+00_JPRB, 0.96359E+00_JPRB, 0.94523E+00_JPRB, 0.92173E+00_JPRB ,&
&  0.89239E+00_JPRB, 0.86236E+00_JPRB, 0.86318E+00_JPRB, 0.87990E+00_JPRB, 0.89207E+00_JPRB, 0.89130E+00_JPRB, 0.90331E+00_JPRB ,&
&  0.91821E+00_JPRB, 0.97331E+00_JPRB, 0.10287E+01_JPRB, 0.13102E+01_JPRB, 0.13537E+01_JPRB, 0.13065E+01_JPRB, 0.12552E+01_JPRB ,&
&  0.11860E+01_JPRB, 0.11155E+01_JPRB, 0.10465E+01_JPRB, 0.99914E+00_JPRB, 0.10953E+01_JPRB /)
ZBURD(:, 53,  1) = (/ &
&  0.10046E+01_JPRB, 0.98139E+00_JPRB, 0.92844E+00_JPRB, 0.86659E+00_JPRB, 0.80952E+00_JPRB, 0.76183E+00_JPRB, 0.73257E+00_JPRB ,&
&  0.70931E+00_JPRB, 0.69317E+00_JPRB, 0.69413E+00_JPRB, 0.70392E+00_JPRB, 0.71589E+00_JPRB, 0.71794E+00_JPRB, 0.71196E+00_JPRB ,&
&  0.70123E+00_JPRB, 0.68547E+00_JPRB, 0.67528E+00_JPRB, 0.66858E+00_JPRB, 0.65834E+00_JPRB, 0.65327E+00_JPRB, 0.65038E+00_JPRB ,&
&  0.65231E+00_JPRB, 0.65618E+00_JPRB, 0.65792E+00_JPRB, 0.66605E+00_JPRB, 0.68073E+00_JPRB, 0.70395E+00_JPRB, 0.72891E+00_JPRB ,&
&  0.73432E+00_JPRB, 0.63821E+00_JPRB, 0.72609E+00_JPRB, 0.92168E+00_JPRB, 0.11295E+01_JPRB, 0.12396E+01_JPRB, 0.12475E+01_JPRB ,&
&  0.11998E+01_JPRB, 0.11426E+01_JPRB, 0.10754E+01_JPRB, 0.99951E+00_JPRB, 0.92796E+00_JPRB, 0.86712E+00_JPRB, 0.81214E+00_JPRB ,&
&  0.76996E+00_JPRB, 0.74010E+00_JPRB, 0.72287E+00_JPRB, 0.71054E+00_JPRB, 0.70031E+00_JPRB, 0.68144E+00_JPRB, 0.66927E+00_JPRB ,&
&  0.66686E+00_JPRB, 0.66681E+00_JPRB, 0.66927E+00_JPRB, 0.67277E+00_JPRB, 0.68704E+00_JPRB, 0.70921E+00_JPRB, 0.72718E+00_JPRB ,&
&  0.74216E+00_JPRB, 0.76104E+00_JPRB, 0.77174E+00_JPRB, 0.77032E+00_JPRB, 0.76986E+00_JPRB, 0.80221E+00_JPRB, 0.83153E+00_JPRB ,&
&  0.86900E+00_JPRB, 0.92371E+00_JPRB, 0.98837E+00_JPRB, 0.10260E+01_JPRB, 0.10296E+01_JPRB, 0.10087E+01_JPRB, 0.97110E+00_JPRB ,&
&  0.94734E+00_JPRB, 0.91953E+00_JPRB, 0.89308E+00_JPRB, 0.87406E+00_JPRB, 0.86188E+00_JPRB, 0.85570E+00_JPRB, 0.84968E+00_JPRB ,&
&  0.84420E+00_JPRB, 0.83431E+00_JPRB, 0.83342E+00_JPRB, 0.85535E+00_JPRB, 0.88068E+00_JPRB, 0.89214E+00_JPRB, 0.89341E+00_JPRB ,&
&  0.88928E+00_JPRB, 0.89716E+00_JPRB, 0.92375E+00_JPRB, 0.10661E+01_JPRB, 0.10743E+01_JPRB, 0.10902E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10586E+01_JPRB, 0.10221E+01_JPRB, 0.97572E+00_JPRB, 0.93841E+00_JPRB, 0.93655E+00_JPRB /)
ZBURD(:, 54,  1) = (/ &
&  0.86116E+00_JPRB, 0.88957E+00_JPRB, 0.88237E+00_JPRB, 0.84064E+00_JPRB, 0.79730E+00_JPRB, 0.76735E+00_JPRB, 0.74726E+00_JPRB ,&
&  0.73830E+00_JPRB, 0.72799E+00_JPRB, 0.72745E+00_JPRB, 0.73665E+00_JPRB, 0.74738E+00_JPRB, 0.74990E+00_JPRB, 0.73356E+00_JPRB ,&
&  0.71204E+00_JPRB, 0.69485E+00_JPRB, 0.68607E+00_JPRB, 0.68233E+00_JPRB, 0.67721E+00_JPRB, 0.67439E+00_JPRB, 0.68228E+00_JPRB ,&
&  0.69165E+00_JPRB, 0.70319E+00_JPRB, 0.71570E+00_JPRB, 0.72604E+00_JPRB, 0.73396E+00_JPRB, 0.74156E+00_JPRB, 0.76111E+00_JPRB ,&
&  0.77226E+00_JPRB, 0.60255E+00_JPRB, 0.63503E+00_JPRB, 0.76642E+00_JPRB, 0.89635E+00_JPRB, 0.10478E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.10762E+01_JPRB, 0.10251E+01_JPRB, 0.96765E+00_JPRB, 0.90604E+00_JPRB, 0.85123E+00_JPRB, 0.80008E+00_JPRB ,&
&  0.76436E+00_JPRB, 0.73718E+00_JPRB, 0.72026E+00_JPRB, 0.70178E+00_JPRB, 0.68988E+00_JPRB, 0.68146E+00_JPRB, 0.67398E+00_JPRB ,&
&  0.66808E+00_JPRB, 0.66101E+00_JPRB, 0.66153E+00_JPRB, 0.66648E+00_JPRB, 0.66605E+00_JPRB, 0.67226E+00_JPRB, 0.67261E+00_JPRB ,&
&  0.68508E+00_JPRB, 0.70322E+00_JPRB, 0.70974E+00_JPRB, 0.70856E+00_JPRB, 0.71105E+00_JPRB, 0.72231E+00_JPRB, 0.73822E+00_JPRB ,&
&  0.75231E+00_JPRB, 0.78393E+00_JPRB, 0.81448E+00_JPRB, 0.84583E+00_JPRB, 0.86531E+00_JPRB, 0.86590E+00_JPRB, 0.85369E+00_JPRB ,&
&  0.85441E+00_JPRB, 0.84674E+00_JPRB, 0.83559E+00_JPRB, 0.82079E+00_JPRB, 0.80412E+00_JPRB, 0.79250E+00_JPRB, 0.78887E+00_JPRB ,&
&  0.79251E+00_JPRB, 0.80485E+00_JPRB, 0.81476E+00_JPRB, 0.83770E+00_JPRB, 0.85332E+00_JPRB, 0.85485E+00_JPRB, 0.85715E+00_JPRB ,&
&  0.85964E+00_JPRB, 0.86059E+00_JPRB, 0.86099E+00_JPRB, 0.94549E+00_JPRB, 0.10259E+01_JPRB, 0.10339E+01_JPRB, 0.10193E+01_JPRB ,&
&  0.99119E+00_JPRB, 0.94087E+00_JPRB, 0.91294E+00_JPRB, 0.73359E+00_JPRB, 0.82153E+00_JPRB /)
ZBURD(:, 55,  1) = (/ &
&  0.80410E+00_JPRB, 0.83114E+00_JPRB, 0.83386E+00_JPRB, 0.81467E+00_JPRB, 0.78508E+00_JPRB, 0.76778E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.75080E+00_JPRB, 0.74608E+00_JPRB, 0.75471E+00_JPRB, 0.76009E+00_JPRB, 0.76048E+00_JPRB, 0.75106E+00_JPRB, 0.74150E+00_JPRB ,&
&  0.72711E+00_JPRB, 0.70955E+00_JPRB, 0.69641E+00_JPRB, 0.68502E+00_JPRB, 0.68347E+00_JPRB, 0.69132E+00_JPRB, 0.70746E+00_JPRB ,&
&  0.72827E+00_JPRB, 0.74281E+00_JPRB, 0.75456E+00_JPRB, 0.76689E+00_JPRB, 0.76913E+00_JPRB, 0.76931E+00_JPRB, 0.77274E+00_JPRB ,&
&  0.77926E+00_JPRB, 0.59406E+00_JPRB, 0.63741E+00_JPRB, 0.69906E+00_JPRB, 0.76126E+00_JPRB, 0.85557E+00_JPRB, 0.93483E+00_JPRB ,&
&  0.96342E+00_JPRB, 0.93214E+00_JPRB, 0.90535E+00_JPRB, 0.87291E+00_JPRB, 0.83663E+00_JPRB, 0.80446E+00_JPRB, 0.76897E+00_JPRB ,&
&  0.74186E+00_JPRB, 0.72288E+00_JPRB, 0.71398E+00_JPRB, 0.70592E+00_JPRB, 0.69698E+00_JPRB, 0.68546E+00_JPRB, 0.67210E+00_JPRB ,&
&  0.65860E+00_JPRB, 0.65138E+00_JPRB, 0.64691E+00_JPRB, 0.64328E+00_JPRB, 0.64502E+00_JPRB, 0.64962E+00_JPRB, 0.65357E+00_JPRB ,&
&  0.66234E+00_JPRB, 0.66946E+00_JPRB, 0.67173E+00_JPRB, 0.67036E+00_JPRB, 0.66814E+00_JPRB, 0.66893E+00_JPRB, 0.68286E+00_JPRB ,&
&  0.69025E+00_JPRB, 0.70424E+00_JPRB, 0.72159E+00_JPRB, 0.75232E+00_JPRB, 0.77913E+00_JPRB, 0.78801E+00_JPRB, 0.78955E+00_JPRB ,&
&  0.78957E+00_JPRB, 0.78101E+00_JPRB, 0.76574E+00_JPRB, 0.74920E+00_JPRB, 0.73743E+00_JPRB, 0.73387E+00_JPRB, 0.72750E+00_JPRB ,&
&  0.72818E+00_JPRB, 0.73394E+00_JPRB, 0.73789E+00_JPRB, 0.75055E+00_JPRB, 0.77070E+00_JPRB, 0.78346E+00_JPRB, 0.77869E+00_JPRB ,&
&  0.77723E+00_JPRB, 0.78242E+00_JPRB, 0.78531E+00_JPRB, 0.85371E+00_JPRB, 0.88715E+00_JPRB, 0.90209E+00_JPRB, 0.89734E+00_JPRB ,&
&  0.88726E+00_JPRB, 0.87413E+00_JPRB, 0.64632E+00_JPRB, 0.74989E+00_JPRB, 0.77248E+00_JPRB /)
ZBURD(:, 56,  1) = (/ &
&  0.78590E+00_JPRB, 0.77917E+00_JPRB, 0.76917E+00_JPRB, 0.76829E+00_JPRB, 0.76739E+00_JPRB, 0.76200E+00_JPRB, 0.75797E+00_JPRB ,&
&  0.75371E+00_JPRB, 0.74911E+00_JPRB, 0.75686E+00_JPRB, 0.76447E+00_JPRB, 0.76574E+00_JPRB, 0.75968E+00_JPRB, 0.75299E+00_JPRB ,&
&  0.74780E+00_JPRB, 0.73885E+00_JPRB, 0.72950E+00_JPRB, 0.72529E+00_JPRB, 0.72732E+00_JPRB, 0.74104E+00_JPRB, 0.76226E+00_JPRB ,&
&  0.77817E+00_JPRB, 0.77750E+00_JPRB, 0.78104E+00_JPRB, 0.79200E+00_JPRB, 0.79437E+00_JPRB, 0.79585E+00_JPRB, 0.80282E+00_JPRB ,&
&  0.68383E+00_JPRB, 0.55871E+00_JPRB, 0.62259E+00_JPRB, 0.68053E+00_JPRB, 0.72594E+00_JPRB, 0.77907E+00_JPRB, 0.84211E+00_JPRB ,&
&  0.86341E+00_JPRB, 0.85057E+00_JPRB, 0.83638E+00_JPRB, 0.81717E+00_JPRB, 0.79341E+00_JPRB, 0.77734E+00_JPRB, 0.76456E+00_JPRB ,&
&  0.76173E+00_JPRB, 0.75098E+00_JPRB, 0.73414E+00_JPRB, 0.72040E+00_JPRB, 0.70888E+00_JPRB, 0.69394E+00_JPRB, 0.67938E+00_JPRB ,&
&  0.66430E+00_JPRB, 0.65137E+00_JPRB, 0.64752E+00_JPRB, 0.64575E+00_JPRB, 0.64837E+00_JPRB, 0.64862E+00_JPRB, 0.64402E+00_JPRB ,&
&  0.64052E+00_JPRB, 0.64196E+00_JPRB, 0.65081E+00_JPRB, 0.65848E+00_JPRB, 0.66023E+00_JPRB, 0.65758E+00_JPRB, 0.66494E+00_JPRB ,&
&  0.66925E+00_JPRB, 0.67918E+00_JPRB, 0.69226E+00_JPRB, 0.71559E+00_JPRB, 0.73338E+00_JPRB, 0.73751E+00_JPRB, 0.73903E+00_JPRB ,&
&  0.74353E+00_JPRB, 0.73773E+00_JPRB, 0.72232E+00_JPRB, 0.71152E+00_JPRB, 0.70614E+00_JPRB, 0.69877E+00_JPRB, 0.69619E+00_JPRB ,&
&  0.70101E+00_JPRB, 0.70343E+00_JPRB, 0.69929E+00_JPRB, 0.70242E+00_JPRB, 0.71251E+00_JPRB, 0.72677E+00_JPRB, 0.74002E+00_JPRB ,&
&  0.73187E+00_JPRB, 0.74098E+00_JPRB, 0.74286E+00_JPRB, 0.76937E+00_JPRB, 0.80288E+00_JPRB, 0.80687E+00_JPRB, 0.80515E+00_JPRB ,&
&  0.79985E+00_JPRB, 0.77394E+00_JPRB, 0.77393E+00_JPRB, 0.77323E+00_JPRB, 0.77618E+00_JPRB /)
ZBURD(:, 57,  1) = (/ &
&  0.72743E+00_JPRB, 0.71929E+00_JPRB, 0.71344E+00_JPRB, 0.72182E+00_JPRB, 0.73647E+00_JPRB, 0.74638E+00_JPRB, 0.74494E+00_JPRB ,&
&  0.74340E+00_JPRB, 0.74685E+00_JPRB, 0.75437E+00_JPRB, 0.75660E+00_JPRB, 0.75000E+00_JPRB, 0.73370E+00_JPRB, 0.72517E+00_JPRB ,&
&  0.72262E+00_JPRB, 0.72311E+00_JPRB, 0.72272E+00_JPRB, 0.72688E+00_JPRB, 0.74640E+00_JPRB, 0.77688E+00_JPRB, 0.80583E+00_JPRB ,&
&  0.82014E+00_JPRB, 0.81469E+00_JPRB, 0.80840E+00_JPRB, 0.80321E+00_JPRB, 0.80184E+00_JPRB, 0.80654E+00_JPRB, 0.80738E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.65306E+00_JPRB, 0.70862E+00_JPRB, 0.72938E+00_JPRB, 0.75351E+00_JPRB, 0.78237E+00_JPRB, 0.81979E+00_JPRB ,&
&  0.82697E+00_JPRB, 0.81718E+00_JPRB, 0.79726E+00_JPRB, 0.78324E+00_JPRB, 0.76791E+00_JPRB, 0.76307E+00_JPRB, 0.76014E+00_JPRB ,&
&  0.75837E+00_JPRB, 0.75539E+00_JPRB, 0.73351E+00_JPRB, 0.69823E+00_JPRB, 0.67879E+00_JPRB, 0.66694E+00_JPRB, 0.65707E+00_JPRB ,&
&  0.64793E+00_JPRB, 0.63563E+00_JPRB, 0.62359E+00_JPRB, 0.61357E+00_JPRB, 0.61516E+00_JPRB, 0.61355E+00_JPRB, 0.60122E+00_JPRB ,&
&  0.59619E+00_JPRB, 0.59411E+00_JPRB, 0.60156E+00_JPRB, 0.61145E+00_JPRB, 0.61496E+00_JPRB, 0.60937E+00_JPRB, 0.61756E+00_JPRB ,&
&  0.63711E+00_JPRB, 0.64934E+00_JPRB, 0.66023E+00_JPRB, 0.67466E+00_JPRB, 0.68189E+00_JPRB, 0.68112E+00_JPRB, 0.68186E+00_JPRB ,&
&  0.68765E+00_JPRB, 0.68592E+00_JPRB, 0.67826E+00_JPRB, 0.67212E+00_JPRB, 0.67329E+00_JPRB, 0.67399E+00_JPRB, 0.68392E+00_JPRB ,&
&  0.69323E+00_JPRB, 0.69627E+00_JPRB, 0.69226E+00_JPRB, 0.69490E+00_JPRB, 0.70056E+00_JPRB, 0.71017E+00_JPRB, 0.71685E+00_JPRB ,&
&  0.72391E+00_JPRB, 0.72188E+00_JPRB, 0.71067E+00_JPRB, 0.71818E+00_JPRB, 0.73425E+00_JPRB, 0.73809E+00_JPRB, 0.72900E+00_JPRB ,&
&  0.71926E+00_JPRB, 0.71028E+00_JPRB, 0.71319E+00_JPRB, 0.71479E+00_JPRB, 0.72167E+00_JPRB /)
ZBURD(:, 58,  1) = (/ &
&  0.71198E+00_JPRB, 0.70846E+00_JPRB, 0.70775E+00_JPRB, 0.71939E+00_JPRB, 0.73728E+00_JPRB, 0.74074E+00_JPRB, 0.74159E+00_JPRB ,&
&  0.75315E+00_JPRB, 0.76154E+00_JPRB, 0.76047E+00_JPRB, 0.75848E+00_JPRB, 0.75228E+00_JPRB, 0.73831E+00_JPRB, 0.72751E+00_JPRB ,&
&  0.73252E+00_JPRB, 0.73788E+00_JPRB, 0.74006E+00_JPRB, 0.76405E+00_JPRB, 0.79579E+00_JPRB, 0.83334E+00_JPRB, 0.85879E+00_JPRB ,&
&  0.86771E+00_JPRB, 0.86022E+00_JPRB, 0.85634E+00_JPRB, 0.85092E+00_JPRB, 0.84548E+00_JPRB, 0.84711E+00_JPRB, 0.84155E+00_JPRB ,&
&  0.83160E+00_JPRB, 0.73970E+00_JPRB, 0.76935E+00_JPRB, 0.77871E+00_JPRB, 0.79426E+00_JPRB, 0.81820E+00_JPRB, 0.84652E+00_JPRB ,&
&  0.84700E+00_JPRB, 0.84192E+00_JPRB, 0.83148E+00_JPRB, 0.81072E+00_JPRB, 0.79337E+00_JPRB, 0.78338E+00_JPRB, 0.76656E+00_JPRB ,&
&  0.73932E+00_JPRB, 0.70252E+00_JPRB, 0.66990E+00_JPRB, 0.64771E+00_JPRB, 0.63771E+00_JPRB, 0.63526E+00_JPRB, 0.63095E+00_JPRB ,&
&  0.62922E+00_JPRB, 0.62788E+00_JPRB, 0.61745E+00_JPRB, 0.60803E+00_JPRB, 0.59837E+00_JPRB, 0.59207E+00_JPRB, 0.57924E+00_JPRB ,&
&  0.57103E+00_JPRB, 0.56014E+00_JPRB, 0.55936E+00_JPRB, 0.57883E+00_JPRB, 0.59890E+00_JPRB, 0.60259E+00_JPRB, 0.61472E+00_JPRB ,&
&  0.63090E+00_JPRB, 0.63734E+00_JPRB, 0.63844E+00_JPRB, 0.63550E+00_JPRB, 0.62831E+00_JPRB, 0.62801E+00_JPRB, 0.63577E+00_JPRB ,&
&  0.64246E+00_JPRB, 0.65215E+00_JPRB, 0.65527E+00_JPRB, 0.65965E+00_JPRB, 0.66611E+00_JPRB, 0.67100E+00_JPRB, 0.68194E+00_JPRB ,&
&  0.68757E+00_JPRB, 0.69176E+00_JPRB, 0.70336E+00_JPRB, 0.71484E+00_JPRB, 0.71951E+00_JPRB, 0.72175E+00_JPRB, 0.73062E+00_JPRB ,&
&  0.73391E+00_JPRB, 0.73191E+00_JPRB, 0.72402E+00_JPRB, 0.71381E+00_JPRB, 0.70883E+00_JPRB, 0.69887E+00_JPRB, 0.68142E+00_JPRB ,&
&  0.66475E+00_JPRB, 0.66311E+00_JPRB, 0.67314E+00_JPRB, 0.68400E+00_JPRB, 0.70202E+00_JPRB /)
ZBURD(:, 59,  1) = (/ &
&  0.69868E+00_JPRB, 0.71037E+00_JPRB, 0.72356E+00_JPRB, 0.73637E+00_JPRB, 0.74997E+00_JPRB, 0.74994E+00_JPRB, 0.75167E+00_JPRB ,&
&  0.77105E+00_JPRB, 0.77388E+00_JPRB, 0.77284E+00_JPRB, 0.76368E+00_JPRB, 0.75691E+00_JPRB, 0.75654E+00_JPRB, 0.74746E+00_JPRB ,&
&  0.74293E+00_JPRB, 0.74928E+00_JPRB, 0.76756E+00_JPRB, 0.79558E+00_JPRB, 0.82295E+00_JPRB, 0.85893E+00_JPRB, 0.88547E+00_JPRB ,&
&  0.88961E+00_JPRB, 0.87873E+00_JPRB, 0.87877E+00_JPRB, 0.88193E+00_JPRB, 0.88205E+00_JPRB, 0.86508E+00_JPRB, 0.86056E+00_JPRB ,&
&  0.84041E+00_JPRB, 0.84788E+00_JPRB, 0.71795E+00_JPRB, 0.78759E+00_JPRB, 0.83132E+00_JPRB, 0.86551E+00_JPRB, 0.87591E+00_JPRB ,&
&  0.86807E+00_JPRB, 0.85526E+00_JPRB, 0.83168E+00_JPRB, 0.80511E+00_JPRB, 0.79565E+00_JPRB, 0.77542E+00_JPRB, 0.75236E+00_JPRB ,&
&  0.70374E+00_JPRB, 0.64171E+00_JPRB, 0.61013E+00_JPRB, 0.59872E+00_JPRB, 0.59322E+00_JPRB, 0.59466E+00_JPRB, 0.60013E+00_JPRB ,&
&  0.60392E+00_JPRB, 0.60457E+00_JPRB, 0.59361E+00_JPRB, 0.57722E+00_JPRB, 0.55784E+00_JPRB, 0.54752E+00_JPRB, 0.54464E+00_JPRB ,&
&  0.54094E+00_JPRB, 0.52823E+00_JPRB, 0.53142E+00_JPRB, 0.56097E+00_JPRB, 0.58546E+00_JPRB, 0.60236E+00_JPRB, 0.60931E+00_JPRB ,&
&  0.61425E+00_JPRB, 0.60791E+00_JPRB, 0.60402E+00_JPRB, 0.59532E+00_JPRB, 0.57265E+00_JPRB, 0.56689E+00_JPRB, 0.58801E+00_JPRB ,&
&  0.61162E+00_JPRB, 0.62508E+00_JPRB, 0.63309E+00_JPRB, 0.64503E+00_JPRB, 0.65755E+00_JPRB, 0.66466E+00_JPRB, 0.67101E+00_JPRB ,&
&  0.67582E+00_JPRB, 0.68053E+00_JPRB, 0.69401E+00_JPRB, 0.70881E+00_JPRB, 0.71583E+00_JPRB, 0.71708E+00_JPRB, 0.71692E+00_JPRB ,&
&  0.71386E+00_JPRB, 0.70616E+00_JPRB, 0.69716E+00_JPRB, 0.68879E+00_JPRB, 0.67040E+00_JPRB, 0.64784E+00_JPRB, 0.63133E+00_JPRB ,&
&  0.62281E+00_JPRB, 0.63306E+00_JPRB, 0.64110E+00_JPRB, 0.65391E+00_JPRB, 0.68108E+00_JPRB /)
ZBURD(:, 60,  1) = (/ &
&  0.73813E+00_JPRB, 0.75234E+00_JPRB, 0.76387E+00_JPRB, 0.78499E+00_JPRB, 0.80423E+00_JPRB, 0.80156E+00_JPRB, 0.79476E+00_JPRB ,&
&  0.79002E+00_JPRB, 0.78547E+00_JPRB, 0.77807E+00_JPRB, 0.76821E+00_JPRB, 0.76151E+00_JPRB, 0.76083E+00_JPRB, 0.76144E+00_JPRB ,&
&  0.76311E+00_JPRB, 0.78318E+00_JPRB, 0.81606E+00_JPRB, 0.84171E+00_JPRB, 0.86973E+00_JPRB, 0.91295E+00_JPRB, 0.93335E+00_JPRB ,&
&  0.91680E+00_JPRB, 0.91582E+00_JPRB, 0.90496E+00_JPRB, 0.89037E+00_JPRB, 0.89215E+00_JPRB, 0.88284E+00_JPRB, 0.85470E+00_JPRB ,&
&  0.84845E+00_JPRB, 0.84182E+00_JPRB, 0.83772E+00_JPRB, 0.86569E+00_JPRB, 0.88758E+00_JPRB, 0.88744E+00_JPRB, 0.88405E+00_JPRB ,&
&  0.86204E+00_JPRB, 0.82498E+00_JPRB, 0.78981E+00_JPRB, 0.77280E+00_JPRB, 0.77092E+00_JPRB, 0.75089E+00_JPRB, 0.72284E+00_JPRB ,&
&  0.66537E+00_JPRB, 0.61820E+00_JPRB, 0.59213E+00_JPRB, 0.57668E+00_JPRB, 0.57179E+00_JPRB, 0.56925E+00_JPRB, 0.56492E+00_JPRB ,&
&  0.57035E+00_JPRB, 0.56527E+00_JPRB, 0.53767E+00_JPRB, 0.50881E+00_JPRB, 0.50409E+00_JPRB, 0.51282E+00_JPRB, 0.53214E+00_JPRB ,&
&  0.54713E+00_JPRB, 0.54267E+00_JPRB, 0.54939E+00_JPRB, 0.55674E+00_JPRB, 0.57351E+00_JPRB, 0.59000E+00_JPRB, 0.59119E+00_JPRB ,&
&  0.59127E+00_JPRB, 0.58622E+00_JPRB, 0.57326E+00_JPRB, 0.56491E+00_JPRB, 0.55396E+00_JPRB, 0.54991E+00_JPRB, 0.56167E+00_JPRB ,&
&  0.57912E+00_JPRB, 0.58874E+00_JPRB, 0.59961E+00_JPRB, 0.61305E+00_JPRB, 0.62380E+00_JPRB, 0.63219E+00_JPRB, 0.63901E+00_JPRB ,&
&  0.64930E+00_JPRB, 0.66399E+00_JPRB, 0.67877E+00_JPRB, 0.69166E+00_JPRB, 0.69174E+00_JPRB, 0.67780E+00_JPRB, 0.65259E+00_JPRB ,&
&  0.63565E+00_JPRB, 0.62230E+00_JPRB, 0.61901E+00_JPRB, 0.62432E+00_JPRB, 0.61991E+00_JPRB, 0.60109E+00_JPRB, 0.59194E+00_JPRB ,&
&  0.60301E+00_JPRB, 0.62689E+00_JPRB, 0.64969E+00_JPRB, 0.68096E+00_JPRB, 0.71395E+00_JPRB /)
ZBURD(:, 61,  1) = (/ &
&  0.77600E+00_JPRB, 0.80769E+00_JPRB, 0.83195E+00_JPRB, 0.84080E+00_JPRB, 0.83883E+00_JPRB, 0.83016E+00_JPRB, 0.80771E+00_JPRB ,&
&  0.78330E+00_JPRB, 0.76605E+00_JPRB, 0.74761E+00_JPRB, 0.72109E+00_JPRB, 0.69905E+00_JPRB, 0.69440E+00_JPRB, 0.70698E+00_JPRB ,&
&  0.75328E+00_JPRB, 0.79433E+00_JPRB, 0.81075E+00_JPRB, 0.82415E+00_JPRB, 0.87375E+00_JPRB, 0.91547E+00_JPRB, 0.92452E+00_JPRB ,&
&  0.91954E+00_JPRB, 0.92119E+00_JPRB, 0.88469E+00_JPRB, 0.85137E+00_JPRB, 0.85136E+00_JPRB, 0.85673E+00_JPRB, 0.83870E+00_JPRB ,&
&  0.86456E+00_JPRB, 0.88182E+00_JPRB, 0.87560E+00_JPRB, 0.85074E+00_JPRB, 0.84325E+00_JPRB, 0.83281E+00_JPRB, 0.82080E+00_JPRB ,&
&  0.79878E+00_JPRB, 0.75057E+00_JPRB, 0.72014E+00_JPRB, 0.71815E+00_JPRB, 0.71097E+00_JPRB, 0.70367E+00_JPRB, 0.69006E+00_JPRB ,&
&  0.65003E+00_JPRB, 0.61419E+00_JPRB, 0.58721E+00_JPRB, 0.56866E+00_JPRB, 0.55704E+00_JPRB, 0.54802E+00_JPRB, 0.53852E+00_JPRB ,&
&  0.53028E+00_JPRB, 0.50304E+00_JPRB, 0.47510E+00_JPRB, 0.46919E+00_JPRB, 0.47509E+00_JPRB, 0.48197E+00_JPRB, 0.50876E+00_JPRB ,&
&  0.53366E+00_JPRB, 0.53198E+00_JPRB, 0.52818E+00_JPRB, 0.53048E+00_JPRB, 0.53926E+00_JPRB, 0.53957E+00_JPRB, 0.52963E+00_JPRB ,&
&  0.52482E+00_JPRB, 0.52263E+00_JPRB, 0.51341E+00_JPRB, 0.50668E+00_JPRB, 0.50620E+00_JPRB, 0.51044E+00_JPRB, 0.52504E+00_JPRB ,&
&  0.53398E+00_JPRB, 0.53523E+00_JPRB, 0.54871E+00_JPRB, 0.56589E+00_JPRB, 0.57629E+00_JPRB, 0.58451E+00_JPRB, 0.59036E+00_JPRB ,&
&  0.60388E+00_JPRB, 0.62349E+00_JPRB, 0.63859E+00_JPRB, 0.64071E+00_JPRB, 0.61780E+00_JPRB, 0.59361E+00_JPRB, 0.57465E+00_JPRB ,&
&  0.56656E+00_JPRB, 0.57124E+00_JPRB, 0.57894E+00_JPRB, 0.58638E+00_JPRB, 0.59324E+00_JPRB, 0.59786E+00_JPRB, 0.59726E+00_JPRB ,&
&  0.61691E+00_JPRB, 0.64820E+00_JPRB, 0.67350E+00_JPRB, 0.70385E+00_JPRB, 0.73553E+00_JPRB /)
ZBURD(:, 62,  1) = (/ &
&  0.84280E+00_JPRB, 0.86047E+00_JPRB, 0.87231E+00_JPRB, 0.87302E+00_JPRB, 0.85750E+00_JPRB, 0.83080E+00_JPRB, 0.80794E+00_JPRB ,&
&  0.78179E+00_JPRB, 0.75643E+00_JPRB, 0.73236E+00_JPRB, 0.69963E+00_JPRB, 0.68201E+00_JPRB, 0.68535E+00_JPRB, 0.73419E+00_JPRB ,&
&  0.79947E+00_JPRB, 0.80431E+00_JPRB, 0.79810E+00_JPRB, 0.82317E+00_JPRB, 0.87542E+00_JPRB, 0.90707E+00_JPRB, 0.92788E+00_JPRB ,&
&  0.93060E+00_JPRB, 0.89751E+00_JPRB, 0.86429E+00_JPRB, 0.85079E+00_JPRB, 0.87416E+00_JPRB, 0.89203E+00_JPRB, 0.89723E+00_JPRB ,&
&  0.89284E+00_JPRB, 0.87164E+00_JPRB, 0.85257E+00_JPRB, 0.83642E+00_JPRB, 0.84047E+00_JPRB, 0.81532E+00_JPRB, 0.76274E+00_JPRB ,&
&  0.69640E+00_JPRB, 0.66986E+00_JPRB, 0.65817E+00_JPRB, 0.64355E+00_JPRB, 0.63261E+00_JPRB, 0.63271E+00_JPRB, 0.62355E+00_JPRB ,&
&  0.61007E+00_JPRB, 0.59438E+00_JPRB, 0.57928E+00_JPRB, 0.56144E+00_JPRB, 0.54742E+00_JPRB, 0.52763E+00_JPRB, 0.50911E+00_JPRB ,&
&  0.49193E+00_JPRB, 0.47082E+00_JPRB, 0.45141E+00_JPRB, 0.43780E+00_JPRB, 0.43177E+00_JPRB, 0.44191E+00_JPRB, 0.48160E+00_JPRB ,&
&  0.51287E+00_JPRB, 0.50991E+00_JPRB, 0.50761E+00_JPRB, 0.51171E+00_JPRB, 0.50482E+00_JPRB, 0.49469E+00_JPRB, 0.48932E+00_JPRB ,&
&  0.48455E+00_JPRB, 0.48081E+00_JPRB, 0.47934E+00_JPRB, 0.48148E+00_JPRB, 0.48386E+00_JPRB, 0.48211E+00_JPRB, 0.48448E+00_JPRB ,&
&  0.48900E+00_JPRB, 0.49411E+00_JPRB, 0.51107E+00_JPRB, 0.53548E+00_JPRB, 0.55615E+00_JPRB, 0.57086E+00_JPRB, 0.58452E+00_JPRB ,&
&  0.60558E+00_JPRB, 0.61849E+00_JPRB, 0.60589E+00_JPRB, 0.59386E+00_JPRB, 0.58766E+00_JPRB, 0.59288E+00_JPRB, 0.60351E+00_JPRB ,&
&  0.62427E+00_JPRB, 0.63733E+00_JPRB, 0.64359E+00_JPRB, 0.64529E+00_JPRB, 0.64868E+00_JPRB, 0.64971E+00_JPRB, 0.66245E+00_JPRB ,&
&  0.69640E+00_JPRB, 0.72561E+00_JPRB, 0.73299E+00_JPRB, 0.76569E+00_JPRB, 0.80767E+00_JPRB /)
ZBURD(:, 63,  1) = (/ &
&  0.88219E+00_JPRB, 0.88653E+00_JPRB, 0.87673E+00_JPRB, 0.87232E+00_JPRB, 0.86165E+00_JPRB, 0.84800E+00_JPRB, 0.82557E+00_JPRB ,&
&  0.79917E+00_JPRB, 0.77098E+00_JPRB, 0.74437E+00_JPRB, 0.72259E+00_JPRB, 0.72654E+00_JPRB, 0.75904E+00_JPRB, 0.78644E+00_JPRB ,&
&  0.77539E+00_JPRB, 0.77134E+00_JPRB, 0.78825E+00_JPRB, 0.81865E+00_JPRB, 0.85179E+00_JPRB, 0.88236E+00_JPRB, 0.90426E+00_JPRB ,&
&  0.92200E+00_JPRB, 0.90367E+00_JPRB, 0.90731E+00_JPRB, 0.90935E+00_JPRB, 0.90329E+00_JPRB, 0.88545E+00_JPRB, 0.85675E+00_JPRB ,&
&  0.82817E+00_JPRB, 0.80087E+00_JPRB, 0.76714E+00_JPRB, 0.70831E+00_JPRB, 0.78154E+00_JPRB, 0.70497E+00_JPRB, 0.67637E+00_JPRB ,&
&  0.64915E+00_JPRB, 0.63092E+00_JPRB, 0.61033E+00_JPRB, 0.58807E+00_JPRB, 0.58038E+00_JPRB, 0.57569E+00_JPRB, 0.56770E+00_JPRB ,&
&  0.56109E+00_JPRB, 0.55388E+00_JPRB, 0.54294E+00_JPRB, 0.52908E+00_JPRB, 0.51607E+00_JPRB, 0.50219E+00_JPRB, 0.48947E+00_JPRB ,&
&  0.47101E+00_JPRB, 0.44408E+00_JPRB, 0.41574E+00_JPRB, 0.39491E+00_JPRB, 0.38407E+00_JPRB, 0.39325E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.45946E+00_JPRB, 0.46527E+00_JPRB, 0.47516E+00_JPRB, 0.47818E+00_JPRB, 0.46997E+00_JPRB, 0.46021E+00_JPRB, 0.44286E+00_JPRB ,&
&  0.42967E+00_JPRB, 0.42772E+00_JPRB, 0.42903E+00_JPRB, 0.43302E+00_JPRB, 0.43822E+00_JPRB, 0.44546E+00_JPRB, 0.45551E+00_JPRB ,&
&  0.46470E+00_JPRB, 0.46780E+00_JPRB, 0.47997E+00_JPRB, 0.50665E+00_JPRB, 0.53760E+00_JPRB, 0.56905E+00_JPRB, 0.59597E+00_JPRB ,&
&  0.61419E+00_JPRB, 0.62692E+00_JPRB, 0.63522E+00_JPRB, 0.65039E+00_JPRB, 0.66609E+00_JPRB, 0.68192E+00_JPRB, 0.69014E+00_JPRB ,&
&  0.68777E+00_JPRB, 0.69141E+00_JPRB, 0.70861E+00_JPRB, 0.73697E+00_JPRB, 0.75680E+00_JPRB, 0.79071E+00_JPRB, 0.81193E+00_JPRB ,&
&  0.83444E+00_JPRB, 0.84023E+00_JPRB, 0.83489E+00_JPRB, 0.84751E+00_JPRB, 0.86445E+00_JPRB /)
ZBURD(:, 64,  1) = (/ &
&  0.86769E+00_JPRB, 0.87828E+00_JPRB, 0.87604E+00_JPRB, 0.87488E+00_JPRB, 0.87649E+00_JPRB, 0.86929E+00_JPRB, 0.85504E+00_JPRB ,&
&  0.84078E+00_JPRB, 0.81418E+00_JPRB, 0.78371E+00_JPRB, 0.76999E+00_JPRB, 0.77735E+00_JPRB, 0.77706E+00_JPRB, 0.75833E+00_JPRB ,&
&  0.75651E+00_JPRB, 0.77021E+00_JPRB, 0.79304E+00_JPRB, 0.81849E+00_JPRB, 0.85283E+00_JPRB, 0.88845E+00_JPRB, 0.91500E+00_JPRB ,&
&  0.92539E+00_JPRB, 0.92331E+00_JPRB, 0.91563E+00_JPRB, 0.88888E+00_JPRB, 0.85317E+00_JPRB, 0.82587E+00_JPRB, 0.80505E+00_JPRB ,&
&  0.77924E+00_JPRB, 0.73133E+00_JPRB, 0.66193E+00_JPRB, 0.65643E+00_JPRB, 0.59275E+00_JPRB, 0.57263E+00_JPRB, 0.59475E+00_JPRB ,&
&  0.61015E+00_JPRB, 0.59223E+00_JPRB, 0.56831E+00_JPRB, 0.54841E+00_JPRB, 0.54446E+00_JPRB, 0.54296E+00_JPRB, 0.53771E+00_JPRB ,&
&  0.53370E+00_JPRB, 0.52683E+00_JPRB, 0.51261E+00_JPRB, 0.49954E+00_JPRB, 0.48717E+00_JPRB, 0.47482E+00_JPRB, 0.46634E+00_JPRB ,&
&  0.43817E+00_JPRB, 0.39516E+00_JPRB, 0.36320E+00_JPRB, 0.34932E+00_JPRB, 0.34169E+00_JPRB, 0.34554E+00_JPRB, 0.35847E+00_JPRB ,&
&  0.36896E+00_JPRB, 0.36974E+00_JPRB, 0.36434E+00_JPRB, 0.35199E+00_JPRB, 0.33291E+00_JPRB, 0.28821E+00_JPRB, 0.26263E+00_JPRB ,&
&  0.27879E+00_JPRB, 0.31232E+00_JPRB, 0.34717E+00_JPRB, 0.35687E+00_JPRB, 0.36632E+00_JPRB, 0.36849E+00_JPRB, 0.35368E+00_JPRB ,&
&  0.32522E+00_JPRB, 0.29515E+00_JPRB, 0.28931E+00_JPRB, 0.29137E+00_JPRB, 0.29605E+00_JPRB, 0.30480E+00_JPRB, 0.32083E+00_JPRB ,&
&  0.34493E+00_JPRB, 0.38343E+00_JPRB, 0.40563E+00_JPRB, 0.40830E+00_JPRB, 0.40095E+00_JPRB, 0.38955E+00_JPRB, 0.38750E+00_JPRB ,&
&  0.40154E+00_JPRB, 0.46233E+00_JPRB, 0.58411E+00_JPRB, 0.73106E+00_JPRB, 0.73079E+00_JPRB, 0.77489E+00_JPRB, 0.80964E+00_JPRB ,&
&  0.83945E+00_JPRB, 0.85287E+00_JPRB, 0.83963E+00_JPRB, 0.84094E+00_JPRB, 0.84759E+00_JPRB /)
ZBURD(:, 65,  1) = (/ &
&  0.82756E+00_JPRB, 0.84743E+00_JPRB, 0.85829E+00_JPRB, 0.87219E+00_JPRB, 0.87227E+00_JPRB, 0.84196E+00_JPRB, 0.82724E+00_JPRB ,&
&  0.83179E+00_JPRB, 0.82677E+00_JPRB, 0.81579E+00_JPRB, 0.80894E+00_JPRB, 0.79989E+00_JPRB, 0.79001E+00_JPRB, 0.78161E+00_JPRB ,&
&  0.77323E+00_JPRB, 0.76811E+00_JPRB, 0.77497E+00_JPRB, 0.78879E+00_JPRB, 0.80011E+00_JPRB, 0.80627E+00_JPRB, 0.81113E+00_JPRB ,&
&  0.81055E+00_JPRB, 0.77743E+00_JPRB, 0.73227E+00_JPRB, 0.69930E+00_JPRB, 0.68272E+00_JPRB, 0.66889E+00_JPRB, 0.63575E+00_JPRB ,&
&  0.56806E+00_JPRB, 0.44656E+00_JPRB, 0.39023E+00_JPRB, 0.43150E+00_JPRB, 0.54966E+00_JPRB, 0.51468E+00_JPRB, 0.53525E+00_JPRB ,&
&  0.56572E+00_JPRB, 0.58151E+00_JPRB, 0.57726E+00_JPRB, 0.56671E+00_JPRB, 0.56311E+00_JPRB, 0.56661E+00_JPRB, 0.56318E+00_JPRB ,&
&  0.55965E+00_JPRB, 0.54360E+00_JPRB, 0.53398E+00_JPRB, 0.52305E+00_JPRB, 0.50538E+00_JPRB, 0.47958E+00_JPRB, 0.46394E+00_JPRB ,&
&  0.42572E+00_JPRB, 0.38812E+00_JPRB, 0.37725E+00_JPRB, 0.37806E+00_JPRB, 0.34740E+00_JPRB, 0.33511E+00_JPRB, 0.32052E+00_JPRB ,&
&  0.28105E+00_JPRB, 0.25989E+00_JPRB, 0.22802E+00_JPRB, 0.20808E+00_JPRB, 0.19369E+00_JPRB, 0.17986E+00_JPRB, 0.17800E+00_JPRB ,&
&  0.18295E+00_JPRB, 0.19559E+00_JPRB, 0.21706E+00_JPRB, 0.25085E+00_JPRB, 0.28154E+00_JPRB, 0.23896E+00_JPRB, 0.22117E+00_JPRB ,&
&  0.20707E+00_JPRB, 0.20391E+00_JPRB, 0.20950E+00_JPRB, 0.21606E+00_JPRB, 0.22278E+00_JPRB, 0.23072E+00_JPRB, 0.23842E+00_JPRB ,&
&  0.24404E+00_JPRB, 0.24719E+00_JPRB, 0.24813E+00_JPRB, 0.24748E+00_JPRB, 0.24649E+00_JPRB, 0.24735E+00_JPRB, 0.25086E+00_JPRB ,&
&  0.25710E+00_JPRB, 0.26943E+00_JPRB, 0.29329E+00_JPRB, 0.32855E+00_JPRB, 0.37460E+00_JPRB, 0.43493E+00_JPRB, 0.52719E+00_JPRB ,&
&  0.65209E+00_JPRB, 0.81625E+00_JPRB, 0.86089E+00_JPRB, 0.86999E+00_JPRB, 0.83918E+00_JPRB /)
ZBURD(:, 66,  1) = (/ &
&  0.83282E+00_JPRB, 0.83037E+00_JPRB, 0.83806E+00_JPRB, 0.82763E+00_JPRB, 0.80848E+00_JPRB, 0.79936E+00_JPRB, 0.81626E+00_JPRB ,&
&  0.83595E+00_JPRB, 0.80639E+00_JPRB, 0.78463E+00_JPRB, 0.76533E+00_JPRB, 0.75159E+00_JPRB, 0.73771E+00_JPRB, 0.71552E+00_JPRB ,&
&  0.69536E+00_JPRB, 0.68793E+00_JPRB, 0.68592E+00_JPRB, 0.68076E+00_JPRB, 0.67042E+00_JPRB, 0.66026E+00_JPRB, 0.64247E+00_JPRB ,&
&  0.60968E+00_JPRB, 0.57525E+00_JPRB, 0.55719E+00_JPRB, 0.53354E+00_JPRB, 0.51666E+00_JPRB, 0.49719E+00_JPRB, 0.46952E+00_JPRB ,&
&  0.42450E+00_JPRB, 0.36649E+00_JPRB, 0.33065E+00_JPRB, 0.36721E+00_JPRB, 0.46211E+00_JPRB, 0.45589E+00_JPRB, 0.47522E+00_JPRB ,&
&  0.50996E+00_JPRB, 0.53121E+00_JPRB, 0.54163E+00_JPRB, 0.54490E+00_JPRB, 0.55398E+00_JPRB, 0.55950E+00_JPRB, 0.55590E+00_JPRB ,&
&  0.54119E+00_JPRB, 0.48890E+00_JPRB, 0.44628E+00_JPRB, 0.34338E+00_JPRB, 0.27214E+00_JPRB, 0.23456E+00_JPRB, 0.20542E+00_JPRB ,&
&  0.18656E+00_JPRB, 0.17548E+00_JPRB, 0.17305E+00_JPRB, 0.17281E+00_JPRB, 0.17822E+00_JPRB, 0.18405E+00_JPRB, 0.18424E+00_JPRB ,&
&  0.18057E+00_JPRB, 0.17289E+00_JPRB, 0.16188E+00_JPRB, 0.15537E+00_JPRB, 0.15205E+00_JPRB, 0.15232E+00_JPRB, 0.15446E+00_JPRB ,&
&  0.15616E+00_JPRB, 0.16031E+00_JPRB, 0.17280E+00_JPRB, 0.18370E+00_JPRB, 0.19721E+00_JPRB, 0.17955E+00_JPRB, 0.16506E+00_JPRB ,&
&  0.15983E+00_JPRB, 0.16421E+00_JPRB, 0.17102E+00_JPRB, 0.17614E+00_JPRB, 0.18151E+00_JPRB, 0.18609E+00_JPRB, 0.19074E+00_JPRB ,&
&  0.19533E+00_JPRB, 0.19861E+00_JPRB, 0.20027E+00_JPRB, 0.20205E+00_JPRB, 0.20366E+00_JPRB, 0.20606E+00_JPRB, 0.20902E+00_JPRB ,&
&  0.21406E+00_JPRB, 0.22341E+00_JPRB, 0.23557E+00_JPRB, 0.25565E+00_JPRB, 0.27448E+00_JPRB, 0.29327E+00_JPRB, 0.31491E+00_JPRB ,&
&  0.33586E+00_JPRB, 0.41621E+00_JPRB, 0.60983E+00_JPRB, 0.93066E+00_JPRB, 0.92397E+00_JPRB /)
ZBURD(:, 67,  1) = (/ &
&  0.78138E+00_JPRB, 0.79371E+00_JPRB, 0.81063E+00_JPRB, 0.78705E+00_JPRB, 0.77168E+00_JPRB, 0.77494E+00_JPRB, 0.77565E+00_JPRB ,&
&  0.75241E+00_JPRB, 0.71714E+00_JPRB, 0.66789E+00_JPRB, 0.62732E+00_JPRB, 0.56188E+00_JPRB, 0.47784E+00_JPRB, 0.44012E+00_JPRB ,&
&  0.40573E+00_JPRB, 0.40206E+00_JPRB, 0.40421E+00_JPRB, 0.42017E+00_JPRB, 0.42890E+00_JPRB, 0.42542E+00_JPRB, 0.40360E+00_JPRB ,&
&  0.37646E+00_JPRB, 0.34929E+00_JPRB, 0.33174E+00_JPRB, 0.31782E+00_JPRB, 0.31761E+00_JPRB, 0.31574E+00_JPRB, 0.30922E+00_JPRB ,&
&  0.30069E+00_JPRB, 0.30506E+00_JPRB, 0.32244E+00_JPRB, 0.35070E+00_JPRB, 0.40002E+00_JPRB, 0.37994E+00_JPRB, 0.38642E+00_JPRB ,&
&  0.41647E+00_JPRB, 0.43813E+00_JPRB, 0.44672E+00_JPRB, 0.44980E+00_JPRB, 0.44295E+00_JPRB, 0.46752E+00_JPRB, 0.41094E+00_JPRB ,&
&  0.30394E+00_JPRB, 0.24501E+00_JPRB, 0.20301E+00_JPRB, 0.17600E+00_JPRB, 0.15802E+00_JPRB, 0.14756E+00_JPRB, 0.13609E+00_JPRB ,&
&  0.13084E+00_JPRB, 0.12698E+00_JPRB, 0.12612E+00_JPRB, 0.12774E+00_JPRB, 0.13035E+00_JPRB, 0.13366E+00_JPRB, 0.13490E+00_JPRB ,&
&  0.13683E+00_JPRB, 0.13700E+00_JPRB, 0.13629E+00_JPRB, 0.13620E+00_JPRB, 0.13525E+00_JPRB, 0.13599E+00_JPRB, 0.13606E+00_JPRB ,&
&  0.13614E+00_JPRB, 0.13614E+00_JPRB, 0.13777E+00_JPRB, 0.13844E+00_JPRB, 0.13937E+00_JPRB, 0.13647E+00_JPRB, 0.13485E+00_JPRB ,&
&  0.13508E+00_JPRB, 0.13716E+00_JPRB, 0.14054E+00_JPRB, 0.14440E+00_JPRB, 0.14969E+00_JPRB, 0.15392E+00_JPRB, 0.15879E+00_JPRB ,&
&  0.16306E+00_JPRB, 0.16726E+00_JPRB, 0.17135E+00_JPRB, 0.17553E+00_JPRB, 0.17999E+00_JPRB, 0.18446E+00_JPRB, 0.18911E+00_JPRB ,&
&  0.19392E+00_JPRB, 0.20216E+00_JPRB, 0.21225E+00_JPRB, 0.22986E+00_JPRB, 0.25364E+00_JPRB, 0.28055E+00_JPRB, 0.32766E+00_JPRB ,&
&  0.38771E+00_JPRB, 0.61676E+00_JPRB, 0.91661E+00_JPRB, 0.74536E+00_JPRB, 0.76309E+00_JPRB /)
ZBURD(:, 68,  1) = (/ &
&  0.77198E+00_JPRB, 0.79023E+00_JPRB, 0.80639E+00_JPRB, 0.80369E+00_JPRB, 0.78634E+00_JPRB, 0.72863E+00_JPRB, 0.66516E+00_JPRB ,&
&  0.58642E+00_JPRB, 0.52024E+00_JPRB, 0.45086E+00_JPRB, 0.39078E+00_JPRB, 0.34037E+00_JPRB, 0.30162E+00_JPRB, 0.27839E+00_JPRB ,&
&  0.26199E+00_JPRB, 0.25480E+00_JPRB, 0.25209E+00_JPRB, 0.25662E+00_JPRB, 0.25901E+00_JPRB, 0.26441E+00_JPRB, 0.25877E+00_JPRB ,&
&  0.25180E+00_JPRB, 0.24374E+00_JPRB, 0.24174E+00_JPRB, 0.25045E+00_JPRB, 0.26361E+00_JPRB, 0.27642E+00_JPRB, 0.29306E+00_JPRB ,&
&  0.31131E+00_JPRB, 0.31714E+00_JPRB, 0.33115E+00_JPRB, 0.33479E+00_JPRB, 0.32007E+00_JPRB, 0.32175E+00_JPRB, 0.32756E+00_JPRB ,&
&  0.34069E+00_JPRB, 0.35923E+00_JPRB, 0.36982E+00_JPRB, 0.38986E+00_JPRB, 0.33714E+00_JPRB, 0.27704E+00_JPRB, 0.22537E+00_JPRB ,&
&  0.18695E+00_JPRB, 0.16511E+00_JPRB, 0.14990E+00_JPRB, 0.14216E+00_JPRB, 0.13475E+00_JPRB, 0.13072E+00_JPRB, 0.12645E+00_JPRB ,&
&  0.12457E+00_JPRB, 0.12452E+00_JPRB, 0.12407E+00_JPRB, 0.12380E+00_JPRB, 0.12425E+00_JPRB, 0.12499E+00_JPRB, 0.12452E+00_JPRB ,&
&  0.12387E+00_JPRB, 0.12367E+00_JPRB, 0.12325E+00_JPRB, 0.12295E+00_JPRB, 0.12217E+00_JPRB, 0.12209E+00_JPRB, 0.12065E+00_JPRB ,&
&  0.11993E+00_JPRB, 0.11864E+00_JPRB, 0.11792E+00_JPRB, 0.11786E+00_JPRB, 0.11826E+00_JPRB, 0.11867E+00_JPRB, 0.11931E+00_JPRB ,&
&  0.12051E+00_JPRB, 0.12161E+00_JPRB, 0.12343E+00_JPRB, 0.12521E+00_JPRB, 0.12805E+00_JPRB, 0.13063E+00_JPRB, 0.13425E+00_JPRB ,&
&  0.13809E+00_JPRB, 0.14273E+00_JPRB, 0.14804E+00_JPRB, 0.15400E+00_JPRB, 0.15915E+00_JPRB, 0.16419E+00_JPRB, 0.16968E+00_JPRB ,&
&  0.17402E+00_JPRB, 0.18110E+00_JPRB, 0.18906E+00_JPRB, 0.20227E+00_JPRB, 0.22984E+00_JPRB, 0.25360E+00_JPRB, 0.30236E+00_JPRB ,&
&  0.38504E+00_JPRB, 0.63904E+00_JPRB, 0.87067E+00_JPRB, 0.72259E+00_JPRB, 0.75366E+00_JPRB /)
ZBURD(:, 69,  1) = (/ &
&  0.65061E+00_JPRB, 0.68163E+00_JPRB, 0.70587E+00_JPRB, 0.70795E+00_JPRB, 0.70671E+00_JPRB, 0.66937E+00_JPRB, 0.61141E+00_JPRB ,&
&  0.50466E+00_JPRB, 0.43500E+00_JPRB, 0.39108E+00_JPRB, 0.34962E+00_JPRB, 0.31203E+00_JPRB, 0.28671E+00_JPRB, 0.26938E+00_JPRB ,&
&  0.25624E+00_JPRB, 0.24900E+00_JPRB, 0.24350E+00_JPRB, 0.23886E+00_JPRB, 0.23387E+00_JPRB, 0.22905E+00_JPRB, 0.22031E+00_JPRB ,&
&  0.21504E+00_JPRB, 0.21155E+00_JPRB, 0.21014E+00_JPRB, 0.21257E+00_JPRB, 0.21945E+00_JPRB, 0.22650E+00_JPRB, 0.23716E+00_JPRB ,&
&  0.25042E+00_JPRB, 0.26425E+00_JPRB, 0.27154E+00_JPRB, 0.27481E+00_JPRB, 0.27773E+00_JPRB, 0.27586E+00_JPRB, 0.25328E+00_JPRB ,&
&  0.25828E+00_JPRB, 0.28112E+00_JPRB, 0.29942E+00_JPRB, 0.30966E+00_JPRB, 0.26544E+00_JPRB, 0.22421E+00_JPRB, 0.18365E+00_JPRB ,&
&  0.16419E+00_JPRB, 0.14895E+00_JPRB, 0.14107E+00_JPRB, 0.13512E+00_JPRB, 0.13080E+00_JPRB, 0.12827E+00_JPRB, 0.12604E+00_JPRB ,&
&  0.12402E+00_JPRB, 0.12319E+00_JPRB, 0.12186E+00_JPRB, 0.12129E+00_JPRB, 0.12039E+00_JPRB, 0.12029E+00_JPRB, 0.11915E+00_JPRB ,&
&  0.11799E+00_JPRB, 0.11707E+00_JPRB, 0.11648E+00_JPRB, 0.11620E+00_JPRB, 0.11543E+00_JPRB, 0.11467E+00_JPRB, 0.11291E+00_JPRB ,&
&  0.11169E+00_JPRB, 0.11047E+00_JPRB, 0.10957E+00_JPRB, 0.10964E+00_JPRB, 0.11015E+00_JPRB, 0.11190E+00_JPRB, 0.11288E+00_JPRB ,&
&  0.11310E+00_JPRB, 0.11320E+00_JPRB, 0.11325E+00_JPRB, 0.11396E+00_JPRB, 0.11503E+00_JPRB, 0.11688E+00_JPRB, 0.11946E+00_JPRB ,&
&  0.12257E+00_JPRB, 0.12732E+00_JPRB, 0.13213E+00_JPRB, 0.13751E+00_JPRB, 0.14258E+00_JPRB, 0.14760E+00_JPRB, 0.15416E+00_JPRB ,&
&  0.16088E+00_JPRB, 0.16871E+00_JPRB, 0.18018E+00_JPRB, 0.19249E+00_JPRB, 0.21456E+00_JPRB, 0.23404E+00_JPRB, 0.26564E+00_JPRB ,&
&  0.33614E+00_JPRB, 0.48312E+00_JPRB, 0.56871E+00_JPRB, 0.57998E+00_JPRB, 0.61296E+00_JPRB /)
ZBURD(:, 70,  1) = (/ &
&  0.47098E+00_JPRB, 0.48243E+00_JPRB, 0.49747E+00_JPRB, 0.50655E+00_JPRB, 0.51645E+00_JPRB, 0.48377E+00_JPRB, 0.45800E+00_JPRB ,&
&  0.40541E+00_JPRB, 0.37534E+00_JPRB, 0.35034E+00_JPRB, 0.32737E+00_JPRB, 0.30693E+00_JPRB, 0.28898E+00_JPRB, 0.27171E+00_JPRB ,&
&  0.26062E+00_JPRB, 0.25389E+00_JPRB, 0.24475E+00_JPRB, 0.23461E+00_JPRB, 0.22381E+00_JPRB, 0.21437E+00_JPRB, 0.20933E+00_JPRB ,&
&  0.20475E+00_JPRB, 0.20145E+00_JPRB, 0.20011E+00_JPRB, 0.20035E+00_JPRB, 0.20226E+00_JPRB, 0.20257E+00_JPRB, 0.20332E+00_JPRB ,&
&  0.20047E+00_JPRB, 0.20756E+00_JPRB, 0.21688E+00_JPRB, 0.22111E+00_JPRB, 0.22890E+00_JPRB, 0.22745E+00_JPRB, 0.22270E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.20746E+00_JPRB, 0.19816E+00_JPRB, 0.18666E+00_JPRB, 0.17619E+00_JPRB, 0.16839E+00_JPRB, 0.16076E+00_JPRB ,&
&  0.15613E+00_JPRB, 0.15065E+00_JPRB, 0.14435E+00_JPRB, 0.13894E+00_JPRB, 0.13426E+00_JPRB, 0.13037E+00_JPRB, 0.12686E+00_JPRB ,&
&  0.12425E+00_JPRB, 0.12268E+00_JPRB, 0.12118E+00_JPRB, 0.12071E+00_JPRB, 0.11961E+00_JPRB, 0.11854E+00_JPRB, 0.11748E+00_JPRB ,&
&  0.11690E+00_JPRB, 0.11573E+00_JPRB, 0.11483E+00_JPRB, 0.11402E+00_JPRB, 0.11276E+00_JPRB, 0.11168E+00_JPRB, 0.11044E+00_JPRB ,&
&  0.10936E+00_JPRB, 0.10880E+00_JPRB, 0.10850E+00_JPRB, 0.10843E+00_JPRB, 0.10872E+00_JPRB, 0.10869E+00_JPRB, 0.10894E+00_JPRB ,&
&  0.10925E+00_JPRB, 0.10969E+00_JPRB, 0.10970E+00_JPRB, 0.11002E+00_JPRB, 0.11086E+00_JPRB, 0.11202E+00_JPRB, 0.11480E+00_JPRB ,&
&  0.11800E+00_JPRB, 0.12212E+00_JPRB, 0.12662E+00_JPRB, 0.13182E+00_JPRB, 0.13705E+00_JPRB, 0.14228E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.15554E+00_JPRB, 0.16226E+00_JPRB, 0.17004E+00_JPRB, 0.18181E+00_JPRB, 0.20682E+00_JPRB, 0.22082E+00_JPRB, 0.23599E+00_JPRB ,&
&  0.26286E+00_JPRB, 0.29988E+00_JPRB, 0.40314E+00_JPRB, 0.43594E+00_JPRB, 0.45321E+00_JPRB /)
ZBURD(:, 71,  1) = (/ &
&  0.21407E+00_JPRB, 0.21422E+00_JPRB, 0.21511E+00_JPRB, 0.22177E+00_JPRB, 0.23454E+00_JPRB, 0.31165E+00_JPRB, 0.30293E+00_JPRB ,&
&  0.29582E+00_JPRB, 0.26747E+00_JPRB, 0.26373E+00_JPRB, 0.25652E+00_JPRB, 0.24689E+00_JPRB, 0.23894E+00_JPRB, 0.23416E+00_JPRB ,&
&  0.22900E+00_JPRB, 0.22339E+00_JPRB, 0.22008E+00_JPRB, 0.21678E+00_JPRB, 0.21358E+00_JPRB, 0.20999E+00_JPRB, 0.20724E+00_JPRB ,&
&  0.20481E+00_JPRB, 0.20313E+00_JPRB, 0.20127E+00_JPRB, 0.19767E+00_JPRB, 0.19607E+00_JPRB, 0.19289E+00_JPRB, 0.18814E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.17859E+00_JPRB, 0.17484E+00_JPRB, 0.17148E+00_JPRB, 0.17408E+00_JPRB, 0.17300E+00_JPRB, 0.17077E+00_JPRB ,&
&  0.16727E+00_JPRB, 0.16365E+00_JPRB, 0.15897E+00_JPRB, 0.15451E+00_JPRB, 0.15075E+00_JPRB, 0.14767E+00_JPRB, 0.14499E+00_JPRB ,&
&  0.14285E+00_JPRB, 0.14105E+00_JPRB, 0.13865E+00_JPRB, 0.13647E+00_JPRB, 0.13416E+00_JPRB, 0.13212E+00_JPRB, 0.13007E+00_JPRB ,&
&  0.12846E+00_JPRB, 0.12694E+00_JPRB, 0.12536E+00_JPRB, 0.12372E+00_JPRB, 0.12244E+00_JPRB, 0.12076E+00_JPRB, 0.11950E+00_JPRB ,&
&  0.11838E+00_JPRB, 0.11700E+00_JPRB, 0.11598E+00_JPRB, 0.11472E+00_JPRB, 0.11380E+00_JPRB, 0.11304E+00_JPRB, 0.11230E+00_JPRB ,&
&  0.11130E+00_JPRB, 0.11064E+00_JPRB, 0.11038E+00_JPRB, 0.11018E+00_JPRB, 0.10987E+00_JPRB, 0.10991E+00_JPRB, 0.11021E+00_JPRB ,&
&  0.11065E+00_JPRB, 0.11117E+00_JPRB, 0.11203E+00_JPRB, 0.11305E+00_JPRB, 0.11456E+00_JPRB, 0.11656E+00_JPRB, 0.11974E+00_JPRB ,&
&  0.12319E+00_JPRB, 0.12701E+00_JPRB, 0.13098E+00_JPRB, 0.13546E+00_JPRB, 0.13997E+00_JPRB, 0.14457E+00_JPRB, 0.14987E+00_JPRB ,&
&  0.15559E+00_JPRB, 0.15998E+00_JPRB, 0.16400E+00_JPRB, 0.16828E+00_JPRB, 0.17703E+00_JPRB, 0.18364E+00_JPRB, 0.18878E+00_JPRB ,&
&  0.19453E+00_JPRB, 0.20228E+00_JPRB, 0.20715E+00_JPRB, 0.21051E+00_JPRB, 0.21257E+00_JPRB /)
ZBURD(:, 72,  1) = (/ &
&  0.18654E+00_JPRB, 0.18908E+00_JPRB, 0.19077E+00_JPRB, 0.19185E+00_JPRB, 0.19161E+00_JPRB, 0.19252E+00_JPRB, 0.19349E+00_JPRB ,&
&  0.19476E+00_JPRB, 0.19632E+00_JPRB, 0.19741E+00_JPRB, 0.19841E+00_JPRB, 0.19889E+00_JPRB, 0.19888E+00_JPRB, 0.19887E+00_JPRB ,&
&  0.19866E+00_JPRB, 0.19842E+00_JPRB, 0.19525E+00_JPRB, 0.19241E+00_JPRB, 0.19038E+00_JPRB, 0.18926E+00_JPRB, 0.18824E+00_JPRB ,&
&  0.18739E+00_JPRB, 0.18621E+00_JPRB, 0.18478E+00_JPRB, 0.18262E+00_JPRB, 0.17939E+00_JPRB, 0.17625E+00_JPRB, 0.17303E+00_JPRB ,&
&  0.17012E+00_JPRB, 0.16774E+00_JPRB, 0.16554E+00_JPRB, 0.16331E+00_JPRB, 0.16194E+00_JPRB, 0.16039E+00_JPRB, 0.15879E+00_JPRB ,&
&  0.15680E+00_JPRB, 0.15467E+00_JPRB, 0.15263E+00_JPRB, 0.15055E+00_JPRB, 0.14855E+00_JPRB, 0.14657E+00_JPRB, 0.14512E+00_JPRB ,&
&  0.14361E+00_JPRB, 0.14245E+00_JPRB, 0.14118E+00_JPRB, 0.14003E+00_JPRB, 0.13880E+00_JPRB, 0.13766E+00_JPRB, 0.13636E+00_JPRB ,&
&  0.13548E+00_JPRB, 0.13456E+00_JPRB, 0.13363E+00_JPRB, 0.13276E+00_JPRB, 0.13190E+00_JPRB, 0.13107E+00_JPRB, 0.13012E+00_JPRB ,&
&  0.12926E+00_JPRB, 0.12870E+00_JPRB, 0.12812E+00_JPRB, 0.12747E+00_JPRB, 0.12687E+00_JPRB, 0.12646E+00_JPRB, 0.12610E+00_JPRB ,&
&  0.12576E+00_JPRB, 0.12574E+00_JPRB, 0.12590E+00_JPRB, 0.12607E+00_JPRB, 0.12624E+00_JPRB, 0.12663E+00_JPRB, 0.12720E+00_JPRB ,&
&  0.12788E+00_JPRB, 0.12861E+00_JPRB, 0.12956E+00_JPRB, 0.13089E+00_JPRB, 0.13237E+00_JPRB, 0.13407E+00_JPRB, 0.13592E+00_JPRB ,&
&  0.13793E+00_JPRB, 0.14007E+00_JPRB, 0.14234E+00_JPRB, 0.14458E+00_JPRB, 0.14699E+00_JPRB, 0.14939E+00_JPRB, 0.15170E+00_JPRB ,&
&  0.15411E+00_JPRB, 0.15647E+00_JPRB, 0.15888E+00_JPRB, 0.16107E+00_JPRB, 0.16438E+00_JPRB, 0.16654E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.17097E+00_JPRB, 0.17346E+00_JPRB, 0.17596E+00_JPRB, 0.17907E+00_JPRB, 0.18215E+00_JPRB /)
ZBURD(:, 73,  1) = (/ &
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB, 0.15260E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART2',1,ZHOOK_HANDLE)
END SUBROUTINE PART2



SUBROUTINE PART3

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART3',0,ZHOOK_HANDLE)


!-- month=  2
ZBURD(:,  1,  2) = (/ &
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB ,&
&  0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB, 0.83289E+00_JPRB /)
ZBURD(:,  2,  2) = (/ &
&  0.92527E+00_JPRB, 0.92098E+00_JPRB, 0.91637E+00_JPRB, 0.91212E+00_JPRB, 0.90684E+00_JPRB, 0.90196E+00_JPRB, 0.89706E+00_JPRB ,&
&  0.89262E+00_JPRB, 0.88837E+00_JPRB, 0.88394E+00_JPRB, 0.87913E+00_JPRB, 0.87428E+00_JPRB, 0.86958E+00_JPRB, 0.86476E+00_JPRB ,&
&  0.85985E+00_JPRB, 0.85527E+00_JPRB, 0.85027E+00_JPRB, 0.84437E+00_JPRB, 0.83963E+00_JPRB, 0.83423E+00_JPRB, 0.82655E+00_JPRB ,&
&  0.81532E+00_JPRB, 0.80353E+00_JPRB, 0.79241E+00_JPRB, 0.78375E+00_JPRB, 0.77745E+00_JPRB, 0.77238E+00_JPRB, 0.76797E+00_JPRB ,&
&  0.76520E+00_JPRB, 0.76334E+00_JPRB, 0.76176E+00_JPRB, 0.75902E+00_JPRB, 0.75690E+00_JPRB, 0.75746E+00_JPRB, 0.75746E+00_JPRB ,&
&  0.75647E+00_JPRB, 0.75607E+00_JPRB, 0.75731E+00_JPRB, 0.75847E+00_JPRB, 0.75913E+00_JPRB, 0.76124E+00_JPRB, 0.76554E+00_JPRB ,&
&  0.77000E+00_JPRB, 0.77390E+00_JPRB, 0.77921E+00_JPRB, 0.78497E+00_JPRB, 0.79079E+00_JPRB, 0.79634E+00_JPRB, 0.80414E+00_JPRB ,&
&  0.81243E+00_JPRB, 0.81989E+00_JPRB, 0.82693E+00_JPRB, 0.83200E+00_JPRB, 0.83604E+00_JPRB, 0.83774E+00_JPRB, 0.83844E+00_JPRB ,&
&  0.83740E+00_JPRB, 0.83832E+00_JPRB, 0.83765E+00_JPRB, 0.83700E+00_JPRB, 0.83491E+00_JPRB, 0.83565E+00_JPRB, 0.83602E+00_JPRB ,&
&  0.83870E+00_JPRB, 0.84025E+00_JPRB, 0.84382E+00_JPRB, 0.84736E+00_JPRB, 0.85307E+00_JPRB, 0.85923E+00_JPRB, 0.86581E+00_JPRB ,&
&  0.87338E+00_JPRB, 0.88253E+00_JPRB, 0.89398E+00_JPRB, 0.90147E+00_JPRB, 0.90870E+00_JPRB, 0.91559E+00_JPRB, 0.92311E+00_JPRB ,&
&  0.92915E+00_JPRB, 0.93462E+00_JPRB, 0.94001E+00_JPRB, 0.94400E+00_JPRB, 0.94645E+00_JPRB, 0.94857E+00_JPRB, 0.95061E+00_JPRB ,&
&  0.95150E+00_JPRB, 0.95185E+00_JPRB, 0.95200E+00_JPRB, 0.95231E+00_JPRB, 0.95081E+00_JPRB, 0.94832E+00_JPRB, 0.94558E+00_JPRB ,&
&  0.94327E+00_JPRB, 0.93995E+00_JPRB, 0.93653E+00_JPRB, 0.93295E+00_JPRB, 0.92950E+00_JPRB /)
ZBURD(:,  3,  2) = (/ &
&  0.10725E+01_JPRB, 0.10636E+01_JPRB, 0.10544E+01_JPRB, 0.10459E+01_JPRB, 0.10348E+01_JPRB, 0.10237E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.98768E+00_JPRB, 0.97398E+00_JPRB, 0.95942E+00_JPRB, 0.94484E+00_JPRB, 0.93327E+00_JPRB, 0.92506E+00_JPRB ,&
&  0.91756E+00_JPRB, 0.90781E+00_JPRB, 0.89619E+00_JPRB, 0.88085E+00_JPRB, 0.86479E+00_JPRB, 0.84942E+00_JPRB, 0.83587E+00_JPRB ,&
&  0.81905E+00_JPRB, 0.80131E+00_JPRB, 0.78309E+00_JPRB, 0.76570E+00_JPRB, 0.74832E+00_JPRB, 0.73105E+00_JPRB, 0.71392E+00_JPRB ,&
&  0.69854E+00_JPRB, 0.68709E+00_JPRB, 0.67568E+00_JPRB, 0.66326E+00_JPRB, 0.64941E+00_JPRB, 0.64444E+00_JPRB, 0.63923E+00_JPRB ,&
&  0.63440E+00_JPRB, 0.63286E+00_JPRB, 0.63879E+00_JPRB, 0.64400E+00_JPRB, 0.64956E+00_JPRB, 0.65607E+00_JPRB, 0.66782E+00_JPRB ,&
&  0.67875E+00_JPRB, 0.69163E+00_JPRB, 0.70854E+00_JPRB, 0.72648E+00_JPRB, 0.74173E+00_JPRB, 0.75361E+00_JPRB, 0.77020E+00_JPRB ,&
&  0.78275E+00_JPRB, 0.79873E+00_JPRB, 0.81081E+00_JPRB, 0.81731E+00_JPRB, 0.81848E+00_JPRB, 0.81968E+00_JPRB, 0.82262E+00_JPRB ,&
&  0.82237E+00_JPRB, 0.82958E+00_JPRB, 0.83636E+00_JPRB, 0.84338E+00_JPRB, 0.84536E+00_JPRB, 0.85334E+00_JPRB, 0.85823E+00_JPRB ,&
&  0.86939E+00_JPRB, 0.88625E+00_JPRB, 0.90455E+00_JPRB, 0.92571E+00_JPRB, 0.95169E+00_JPRB, 0.98426E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.10363E+01_JPRB, 0.10604E+01_JPRB, 0.10833E+01_JPRB, 0.10984E+01_JPRB, 0.11122E+01_JPRB, 0.11249E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.11472E+01_JPRB, 0.11518E+01_JPRB, 0.11553E+01_JPRB, 0.11529E+01_JPRB, 0.11493E+01_JPRB, 0.11457E+01_JPRB ,&
&  0.11422E+01_JPRB, 0.11371E+01_JPRB, 0.11331E+01_JPRB, 0.11307E+01_JPRB, 0.11273E+01_JPRB, 0.11220E+01_JPRB, 0.11166E+01_JPRB ,&
&  0.11115E+01_JPRB, 0.11046E+01_JPRB, 0.10968E+01_JPRB, 0.10889E+01_JPRB, 0.10816E+01_JPRB /)
ZBURD(:,  4,  2) = (/ &
&  0.10982E+01_JPRB, 0.10951E+01_JPRB, 0.10919E+01_JPRB, 0.10884E+01_JPRB, 0.10822E+01_JPRB, 0.10750E+01_JPRB, 0.10663E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.10423E+01_JPRB, 0.10281E+01_JPRB, 0.10127E+01_JPRB, 0.99772E+00_JPRB, 0.98523E+00_JPRB, 0.97389E+00_JPRB ,&
&  0.96415E+00_JPRB, 0.94970E+00_JPRB, 0.92453E+00_JPRB, 0.89353E+00_JPRB, 0.86590E+00_JPRB, 0.84170E+00_JPRB, 0.81854E+00_JPRB ,&
&  0.77941E+00_JPRB, 0.73735E+00_JPRB, 0.69790E+00_JPRB, 0.65581E+00_JPRB, 0.62397E+00_JPRB, 0.56070E+00_JPRB, 0.53687E+00_JPRB ,&
&  0.51465E+00_JPRB, 0.50173E+00_JPRB, 0.49593E+00_JPRB, 0.49272E+00_JPRB, 0.50103E+00_JPRB, 0.49075E+00_JPRB, 0.48391E+00_JPRB ,&
&  0.46059E+00_JPRB, 0.46080E+00_JPRB, 0.46710E+00_JPRB, 0.47667E+00_JPRB, 0.49480E+00_JPRB, 0.51817E+00_JPRB, 0.55142E+00_JPRB ,&
&  0.60810E+00_JPRB, 0.63605E+00_JPRB, 0.67769E+00_JPRB, 0.71017E+00_JPRB, 0.73039E+00_JPRB, 0.74179E+00_JPRB, 0.75576E+00_JPRB ,&
&  0.77066E+00_JPRB, 0.79425E+00_JPRB, 0.80867E+00_JPRB, 0.81876E+00_JPRB, 0.81917E+00_JPRB, 0.81680E+00_JPRB, 0.81793E+00_JPRB ,&
&  0.81794E+00_JPRB, 0.82801E+00_JPRB, 0.84517E+00_JPRB, 0.87765E+00_JPRB, 0.92259E+00_JPRB, 0.97537E+00_JPRB, 0.10265E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.11173E+01_JPRB, 0.11544E+01_JPRB, 0.11906E+01_JPRB, 0.12289E+01_JPRB, 0.12628E+01_JPRB, 0.12927E+01_JPRB ,&
&  0.13154E+01_JPRB, 0.13303E+01_JPRB, 0.13343E+01_JPRB, 0.13346E+01_JPRB, 0.13277E+01_JPRB, 0.13203E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.13084E+01_JPRB, 0.13066E+01_JPRB, 0.12977E+01_JPRB, 0.12837E+01_JPRB, 0.12673E+01_JPRB, 0.12502E+01_JPRB ,&
&  0.12321E+01_JPRB, 0.12141E+01_JPRB, 0.11973E+01_JPRB, 0.11818E+01_JPRB, 0.11686E+01_JPRB, 0.11531E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.11277E+01_JPRB, 0.11215E+01_JPRB, 0.11138E+01_JPRB, 0.11077E+01_JPRB, 0.11019E+01_JPRB /)
ZBURD(:,  5,  2) = (/ &
&  0.96440E+00_JPRB, 0.96293E+00_JPRB, 0.96461E+00_JPRB, 0.96551E+00_JPRB, 0.96943E+00_JPRB, 0.97277E+00_JPRB, 0.97817E+00_JPRB ,&
&  0.98209E+00_JPRB, 0.98705E+00_JPRB, 0.98905E+00_JPRB, 0.99045E+00_JPRB, 0.98877E+00_JPRB, 0.98798E+00_JPRB, 0.98555E+00_JPRB ,&
&  0.97988E+00_JPRB, 0.96351E+00_JPRB, 0.93225E+00_JPRB, 0.89738E+00_JPRB, 0.85898E+00_JPRB, 0.81445E+00_JPRB, 0.76604E+00_JPRB ,&
&  0.70860E+00_JPRB, 0.65708E+00_JPRB, 0.59076E+00_JPRB, 0.54899E+00_JPRB, 0.51025E+00_JPRB, 0.48236E+00_JPRB, 0.46491E+00_JPRB ,&
&  0.44886E+00_JPRB, 0.43364E+00_JPRB, 0.41798E+00_JPRB, 0.40110E+00_JPRB, 0.38905E+00_JPRB, 0.37327E+00_JPRB, 0.36485E+00_JPRB ,&
&  0.35846E+00_JPRB, 0.35648E+00_JPRB, 0.35743E+00_JPRB, 0.36329E+00_JPRB, 0.37380E+00_JPRB, 0.38976E+00_JPRB, 0.41775E+00_JPRB ,&
&  0.48236E+00_JPRB, 0.57303E+00_JPRB, 0.71576E+00_JPRB, 0.74818E+00_JPRB, 0.74606E+00_JPRB, 0.76769E+00_JPRB, 0.79310E+00_JPRB ,&
&  0.81797E+00_JPRB, 0.84599E+00_JPRB, 0.86827E+00_JPRB, 0.87685E+00_JPRB, 0.88834E+00_JPRB, 0.89790E+00_JPRB, 0.90967E+00_JPRB ,&
&  0.92366E+00_JPRB, 0.95400E+00_JPRB, 0.10022E+01_JPRB, 0.10604E+01_JPRB, 0.11268E+01_JPRB, 0.11896E+01_JPRB, 0.12339E+01_JPRB ,&
&  0.12613E+01_JPRB, 0.12828E+01_JPRB, 0.13174E+01_JPRB, 0.13562E+01_JPRB, 0.13972E+01_JPRB, 0.14239E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.14668E+01_JPRB, 0.14750E+01_JPRB, 0.14666E+01_JPRB, 0.14506E+01_JPRB, 0.14386E+01_JPRB, 0.14420E+01_JPRB, 0.14339E+01_JPRB ,&
&  0.14230E+01_JPRB, 0.14036E+01_JPRB, 0.13707E+01_JPRB, 0.13289E+01_JPRB, 0.12929E+01_JPRB, 0.12593E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.12103E+01_JPRB, 0.11872E+01_JPRB, 0.11603E+01_JPRB, 0.11267E+01_JPRB, 0.10932E+01_JPRB, 0.10599E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.10099E+01_JPRB, 0.99407E+00_JPRB, 0.98067E+00_JPRB, 0.97311E+00_JPRB, 0.96638E+00_JPRB /)
ZBURD(:,  6,  2) = (/ &
&  0.86349E+00_JPRB, 0.87107E+00_JPRB, 0.88082E+00_JPRB, 0.88913E+00_JPRB, 0.89755E+00_JPRB, 0.90556E+00_JPRB, 0.91062E+00_JPRB ,&
&  0.91531E+00_JPRB, 0.91932E+00_JPRB, 0.92518E+00_JPRB, 0.93141E+00_JPRB, 0.93824E+00_JPRB, 0.94697E+00_JPRB, 0.95478E+00_JPRB ,&
&  0.96429E+00_JPRB, 0.95777E+00_JPRB, 0.92974E+00_JPRB, 0.89436E+00_JPRB, 0.85468E+00_JPRB, 0.80264E+00_JPRB, 0.74286E+00_JPRB ,&
&  0.68979E+00_JPRB, 0.65013E+00_JPRB, 0.59713E+00_JPRB, 0.55514E+00_JPRB, 0.52200E+00_JPRB, 0.50662E+00_JPRB, 0.48693E+00_JPRB ,&
&  0.48772E+00_JPRB, 0.43456E+00_JPRB, 0.41744E+00_JPRB, 0.38179E+00_JPRB, 0.35886E+00_JPRB, 0.34201E+00_JPRB, 0.33212E+00_JPRB ,&
&  0.32909E+00_JPRB, 0.32637E+00_JPRB, 0.32763E+00_JPRB, 0.33379E+00_JPRB, 0.34242E+00_JPRB, 0.36006E+00_JPRB, 0.38162E+00_JPRB ,&
&  0.44270E+00_JPRB, 0.63139E+00_JPRB, 0.70659E+00_JPRB, 0.72785E+00_JPRB, 0.73742E+00_JPRB, 0.76923E+00_JPRB, 0.80705E+00_JPRB ,&
&  0.86215E+00_JPRB, 0.90536E+00_JPRB, 0.94895E+00_JPRB, 0.97459E+00_JPRB, 0.10260E+01_JPRB, 0.10739E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.11584E+01_JPRB, 0.11958E+01_JPRB, 0.12352E+01_JPRB, 0.12621E+01_JPRB, 0.12679E+01_JPRB, 0.12820E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.13139E+01_JPRB, 0.13325E+01_JPRB, 0.13832E+01_JPRB, 0.14500E+01_JPRB, 0.15210E+01_JPRB, 0.15688E+01_JPRB, 0.15930E+01_JPRB ,&
&  0.16178E+01_JPRB, 0.16317E+01_JPRB, 0.16493E+01_JPRB, 0.16592E+01_JPRB, 0.16350E+01_JPRB, 0.15903E+01_JPRB, 0.15655E+01_JPRB ,&
&  0.15400E+01_JPRB, 0.14826E+01_JPRB, 0.14238E+01_JPRB, 0.13703E+01_JPRB, 0.13238E+01_JPRB, 0.12822E+01_JPRB, 0.12422E+01_JPRB ,&
&  0.12010E+01_JPRB, 0.11539E+01_JPRB, 0.11045E+01_JPRB, 0.10458E+01_JPRB, 0.99166E+00_JPRB, 0.93999E+00_JPRB, 0.90733E+00_JPRB ,&
&  0.87811E+00_JPRB, 0.86691E+00_JPRB, 0.85504E+00_JPRB, 0.85545E+00_JPRB, 0.85485E+00_JPRB /)
ZBURD(:,  7,  2) = (/ &
&  0.77696E+00_JPRB, 0.79679E+00_JPRB, 0.81613E+00_JPRB, 0.83100E+00_JPRB, 0.84253E+00_JPRB, 0.85324E+00_JPRB, 0.85700E+00_JPRB ,&
&  0.86149E+00_JPRB, 0.86272E+00_JPRB, 0.86848E+00_JPRB, 0.87253E+00_JPRB, 0.88151E+00_JPRB, 0.88889E+00_JPRB, 0.89877E+00_JPRB ,&
&  0.91193E+00_JPRB, 0.92017E+00_JPRB, 0.91510E+00_JPRB, 0.88998E+00_JPRB, 0.85005E+00_JPRB, 0.80292E+00_JPRB, 0.74291E+00_JPRB ,&
&  0.68940E+00_JPRB, 0.65470E+00_JPRB, 0.61295E+00_JPRB, 0.57626E+00_JPRB, 0.55488E+00_JPRB, 0.53985E+00_JPRB, 0.53537E+00_JPRB ,&
&  0.53054E+00_JPRB, 0.51225E+00_JPRB, 0.48778E+00_JPRB, 0.45154E+00_JPRB, 0.42170E+00_JPRB, 0.36012E+00_JPRB, 0.34867E+00_JPRB ,&
&  0.33353E+00_JPRB, 0.32200E+00_JPRB, 0.31876E+00_JPRB, 0.32052E+00_JPRB, 0.32711E+00_JPRB, 0.34455E+00_JPRB, 0.37236E+00_JPRB ,&
&  0.41878E+00_JPRB, 0.63459E+00_JPRB, 0.69821E+00_JPRB, 0.72235E+00_JPRB, 0.76626E+00_JPRB, 0.81187E+00_JPRB, 0.85178E+00_JPRB ,&
&  0.89328E+00_JPRB, 0.93276E+00_JPRB, 0.99319E+00_JPRB, 0.10516E+01_JPRB, 0.11145E+01_JPRB, 0.11729E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12641E+01_JPRB, 0.12913E+01_JPRB, 0.13580E+01_JPRB, 0.13790E+01_JPRB, 0.13714E+01_JPRB, 0.14378E+01_JPRB, 0.14771E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.14679E+01_JPRB, 0.15094E+01_JPRB, 0.16060E+01_JPRB, 0.17382E+01_JPRB, 0.18458E+01_JPRB, 0.18761E+01_JPRB ,&
&  0.18864E+01_JPRB, 0.18572E+01_JPRB, 0.18076E+01_JPRB, 0.17734E+01_JPRB, 0.17003E+01_JPRB, 0.16212E+01_JPRB, 0.15738E+01_JPRB ,&
&  0.15270E+01_JPRB, 0.15193E+01_JPRB, 0.14779E+01_JPRB, 0.14317E+01_JPRB, 0.13768E+01_JPRB, 0.13224E+01_JPRB, 0.12535E+01_JPRB ,&
&  0.11661E+01_JPRB, 0.10709E+01_JPRB, 0.99365E+00_JPRB, 0.92754E+00_JPRB, 0.87996E+00_JPRB, 0.82999E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.77317E+00_JPRB, 0.76160E+00_JPRB, 0.74871E+00_JPRB, 0.75158E+00_JPRB, 0.75758E+00_JPRB /)
ZBURD(:,  8,  2) = (/ &
&  0.72445E+00_JPRB, 0.75576E+00_JPRB, 0.78167E+00_JPRB, 0.80050E+00_JPRB, 0.80886E+00_JPRB, 0.81880E+00_JPRB, 0.82551E+00_JPRB ,&
&  0.83270E+00_JPRB, 0.83792E+00_JPRB, 0.84246E+00_JPRB, 0.84120E+00_JPRB, 0.84152E+00_JPRB, 0.83997E+00_JPRB, 0.84320E+00_JPRB ,&
&  0.84662E+00_JPRB, 0.84082E+00_JPRB, 0.83674E+00_JPRB, 0.83010E+00_JPRB, 0.81334E+00_JPRB, 0.78150E+00_JPRB, 0.74143E+00_JPRB ,&
&  0.69117E+00_JPRB, 0.64984E+00_JPRB, 0.61826E+00_JPRB, 0.59023E+00_JPRB, 0.56755E+00_JPRB, 0.55121E+00_JPRB, 0.54635E+00_JPRB ,&
&  0.55754E+00_JPRB, 0.54576E+00_JPRB, 0.53053E+00_JPRB, 0.48549E+00_JPRB, 0.43955E+00_JPRB, 0.41475E+00_JPRB, 0.37246E+00_JPRB ,&
&  0.34182E+00_JPRB, 0.32284E+00_JPRB, 0.31734E+00_JPRB, 0.31713E+00_JPRB, 0.32666E+00_JPRB, 0.35150E+00_JPRB, 0.40621E+00_JPRB ,&
&  0.50764E+00_JPRB, 0.69629E+00_JPRB, 0.74123E+00_JPRB, 0.78378E+00_JPRB, 0.83277E+00_JPRB, 0.88113E+00_JPRB, 0.91902E+00_JPRB ,&
&  0.95553E+00_JPRB, 0.99252E+00_JPRB, 0.10612E+01_JPRB, 0.11315E+01_JPRB, 0.12275E+01_JPRB, 0.13156E+01_JPRB, 0.13896E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.14411E+01_JPRB, 0.14804E+01_JPRB, 0.15139E+01_JPRB, 0.16074E+01_JPRB, 0.16941E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.16789E+01_JPRB, 0.17690E+01_JPRB, 0.17883E+01_JPRB, 0.18406E+01_JPRB, 0.19588E+01_JPRB, 0.20972E+01_JPRB, 0.21498E+01_JPRB ,&
&  0.20863E+01_JPRB, 0.20438E+01_JPRB, 0.20899E+01_JPRB, 0.19761E+01_JPRB, 0.18914E+01_JPRB, 0.17863E+01_JPRB, 0.16685E+01_JPRB ,&
&  0.15997E+01_JPRB, 0.15572E+01_JPRB, 0.15284E+01_JPRB, 0.14955E+01_JPRB, 0.14317E+01_JPRB, 0.13551E+01_JPRB, 0.12403E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.99519E+00_JPRB, 0.90936E+00_JPRB, 0.85630E+00_JPRB, 0.81038E+00_JPRB, 0.75992E+00_JPRB, 0.72135E+00_JPRB ,&
&  0.68911E+00_JPRB, 0.67455E+00_JPRB, 0.67204E+00_JPRB, 0.68689E+00_JPRB, 0.70418E+00_JPRB /)
ZBURD(:,  9,  2) = (/ &
&  0.73049E+00_JPRB, 0.74939E+00_JPRB, 0.76263E+00_JPRB, 0.76476E+00_JPRB, 0.75393E+00_JPRB, 0.73298E+00_JPRB, 0.72568E+00_JPRB ,&
&  0.73095E+00_JPRB, 0.74495E+00_JPRB, 0.75874E+00_JPRB, 0.76026E+00_JPRB, 0.76487E+00_JPRB, 0.76963E+00_JPRB, 0.77623E+00_JPRB ,&
&  0.78360E+00_JPRB, 0.79537E+00_JPRB, 0.80882E+00_JPRB, 0.79667E+00_JPRB, 0.77444E+00_JPRB, 0.74662E+00_JPRB, 0.71760E+00_JPRB ,&
&  0.68640E+00_JPRB, 0.65697E+00_JPRB, 0.62733E+00_JPRB, 0.61067E+00_JPRB, 0.59336E+00_JPRB, 0.58594E+00_JPRB, 0.57791E+00_JPRB ,&
&  0.56860E+00_JPRB, 0.56622E+00_JPRB, 0.56929E+00_JPRB, 0.53124E+00_JPRB, 0.47805E+00_JPRB, 0.43587E+00_JPRB, 0.39087E+00_JPRB ,&
&  0.36046E+00_JPRB, 0.34334E+00_JPRB, 0.33281E+00_JPRB, 0.33182E+00_JPRB, 0.34552E+00_JPRB, 0.38125E+00_JPRB, 0.47077E+00_JPRB ,&
&  0.67296E+00_JPRB, 0.72674E+00_JPRB, 0.75270E+00_JPRB, 0.78235E+00_JPRB, 0.81727E+00_JPRB, 0.85510E+00_JPRB, 0.90032E+00_JPRB ,&
&  0.95184E+00_JPRB, 0.99639E+00_JPRB, 0.10622E+01_JPRB, 0.11180E+01_JPRB, 0.12013E+01_JPRB, 0.12385E+01_JPRB, 0.14107E+01_JPRB ,&
&  0.17292E+01_JPRB, 0.17955E+01_JPRB, 0.16801E+01_JPRB, 0.17294E+01_JPRB, 0.18452E+01_JPRB, 0.19565E+01_JPRB, 0.20445E+01_JPRB ,&
&  0.20980E+01_JPRB, 0.21099E+01_JPRB, 0.21263E+01_JPRB, 0.21431E+01_JPRB, 0.21969E+01_JPRB, 0.22422E+01_JPRB, 0.22483E+01_JPRB ,&
&  0.22122E+01_JPRB, 0.21985E+01_JPRB, 0.22159E+01_JPRB, 0.18403E+01_JPRB, 0.16818E+01_JPRB, 0.16437E+01_JPRB, 0.15770E+01_JPRB ,&
&  0.15357E+01_JPRB, 0.15248E+01_JPRB, 0.15414E+01_JPRB, 0.15375E+01_JPRB, 0.14043E+01_JPRB, 0.12483E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.10107E+01_JPRB, 0.93819E+00_JPRB, 0.85839E+00_JPRB, 0.81978E+00_JPRB, 0.77444E+00_JPRB, 0.71917E+00_JPRB, 0.66651E+00_JPRB ,&
&  0.64813E+00_JPRB, 0.64531E+00_JPRB, 0.65312E+00_JPRB, 0.69141E+00_JPRB, 0.71655E+00_JPRB /)
ZBURD(:, 10,  2) = (/ &
&  0.69346E+00_JPRB, 0.73578E+00_JPRB, 0.74333E+00_JPRB, 0.70679E+00_JPRB, 0.67043E+00_JPRB, 0.61379E+00_JPRB, 0.57522E+00_JPRB ,&
&  0.56149E+00_JPRB, 0.56102E+00_JPRB, 0.56966E+00_JPRB, 0.58037E+00_JPRB, 0.59612E+00_JPRB, 0.61967E+00_JPRB, 0.64693E+00_JPRB ,&
&  0.66411E+00_JPRB, 0.67454E+00_JPRB, 0.68955E+00_JPRB, 0.70973E+00_JPRB, 0.71571E+00_JPRB, 0.70079E+00_JPRB, 0.67708E+00_JPRB ,&
&  0.64736E+00_JPRB, 0.62693E+00_JPRB, 0.60979E+00_JPRB, 0.60109E+00_JPRB, 0.60377E+00_JPRB, 0.61418E+00_JPRB, 0.62437E+00_JPRB ,&
&  0.62493E+00_JPRB, 0.61141E+00_JPRB, 0.60073E+00_JPRB, 0.58184E+00_JPRB, 0.52479E+00_JPRB, 0.47324E+00_JPRB, 0.41442E+00_JPRB ,&
&  0.37769E+00_JPRB, 0.36510E+00_JPRB, 0.36722E+00_JPRB, 0.39334E+00_JPRB, 0.46344E+00_JPRB, 0.60750E+00_JPRB, 0.64855E+00_JPRB ,&
&  0.69093E+00_JPRB, 0.72067E+00_JPRB, 0.74434E+00_JPRB, 0.77150E+00_JPRB, 0.79769E+00_JPRB, 0.84048E+00_JPRB, 0.89188E+00_JPRB ,&
&  0.96297E+00_JPRB, 0.10316E+01_JPRB, 0.11265E+01_JPRB, 0.10394E+01_JPRB, 0.10985E+01_JPRB, 0.13379E+01_JPRB, 0.15732E+01_JPRB ,&
&  0.17716E+01_JPRB, 0.18457E+01_JPRB, 0.19105E+01_JPRB, 0.21143E+01_JPRB, 0.22304E+01_JPRB, 0.23742E+01_JPRB, 0.23793E+01_JPRB ,&
&  0.23233E+01_JPRB, 0.22473E+01_JPRB, 0.21838E+01_JPRB, 0.21835E+01_JPRB, 0.22590E+01_JPRB, 0.23364E+01_JPRB, 0.22836E+01_JPRB ,&
&  0.22254E+01_JPRB, 0.21458E+01_JPRB, 0.19391E+01_JPRB, 0.16666E+01_JPRB, 0.15533E+01_JPRB, 0.15208E+01_JPRB, 0.15314E+01_JPRB ,&
&  0.15156E+01_JPRB, 0.15166E+01_JPRB, 0.15171E+01_JPRB, 0.15164E+01_JPRB, 0.13108E+01_JPRB, 0.11135E+01_JPRB, 0.96280E+00_JPRB ,&
&  0.88584E+00_JPRB, 0.84273E+00_JPRB, 0.78647E+00_JPRB, 0.78478E+00_JPRB, 0.74474E+00_JPRB, 0.70013E+00_JPRB, 0.64390E+00_JPRB ,&
&  0.61199E+00_JPRB, 0.60072E+00_JPRB, 0.62129E+00_JPRB, 0.65390E+00_JPRB, 0.67353E+00_JPRB /)
ZBURD(:, 11,  2) = (/ &
&  0.74903E+00_JPRB, 0.74019E+00_JPRB, 0.69318E+00_JPRB, 0.65434E+00_JPRB, 0.61429E+00_JPRB, 0.58556E+00_JPRB, 0.55925E+00_JPRB ,&
&  0.55317E+00_JPRB, 0.52911E+00_JPRB, 0.50782E+00_JPRB, 0.48879E+00_JPRB, 0.48137E+00_JPRB, 0.47998E+00_JPRB, 0.49901E+00_JPRB ,&
&  0.54059E+00_JPRB, 0.59791E+00_JPRB, 0.63923E+00_JPRB, 0.64670E+00_JPRB, 0.64688E+00_JPRB, 0.63657E+00_JPRB, 0.62711E+00_JPRB ,&
&  0.61027E+00_JPRB, 0.60368E+00_JPRB, 0.61272E+00_JPRB, 0.63932E+00_JPRB, 0.67070E+00_JPRB, 0.68333E+00_JPRB, 0.67054E+00_JPRB ,&
&  0.64495E+00_JPRB, 0.62723E+00_JPRB, 0.62285E+00_JPRB, 0.61563E+00_JPRB, 0.56717E+00_JPRB, 0.50190E+00_JPRB, 0.43620E+00_JPRB ,&
&  0.39226E+00_JPRB, 0.38882E+00_JPRB, 0.43791E+00_JPRB, 0.56702E+00_JPRB, 0.60399E+00_JPRB, 0.63311E+00_JPRB, 0.65881E+00_JPRB ,&
&  0.64938E+00_JPRB, 0.66134E+00_JPRB, 0.72042E+00_JPRB, 0.83105E+00_JPRB, 0.85536E+00_JPRB, 0.90575E+00_JPRB, 0.96486E+00_JPRB ,&
&  0.10391E+01_JPRB, 0.10571E+01_JPRB, 0.11371E+01_JPRB, 0.11397E+01_JPRB, 0.13698E+01_JPRB, 0.19430E+01_JPRB, 0.19520E+01_JPRB ,&
&  0.20598E+01_JPRB, 0.21606E+01_JPRB, 0.23854E+01_JPRB, 0.24944E+01_JPRB, 0.25349E+01_JPRB, 0.25752E+01_JPRB, 0.25112E+01_JPRB ,&
&  0.23573E+01_JPRB, 0.22732E+01_JPRB, 0.22114E+01_JPRB, 0.22709E+01_JPRB, 0.22964E+01_JPRB, 0.22941E+01_JPRB, 0.22633E+01_JPRB ,&
&  0.22300E+01_JPRB, 0.21576E+01_JPRB, 0.19966E+01_JPRB, 0.17784E+01_JPRB, 0.16553E+01_JPRB, 0.15456E+01_JPRB, 0.14756E+01_JPRB ,&
&  0.14291E+01_JPRB, 0.14473E+01_JPRB, 0.14888E+01_JPRB, 0.15054E+01_JPRB, 0.13380E+01_JPRB, 0.10988E+01_JPRB, 0.91860E+00_JPRB ,&
&  0.80875E+00_JPRB, 0.75519E+00_JPRB, 0.68979E+00_JPRB, 0.66957E+00_JPRB, 0.66753E+00_JPRB, 0.66589E+00_JPRB, 0.64075E+00_JPRB ,&
&  0.61832E+00_JPRB, 0.62286E+00_JPRB, 0.66937E+00_JPRB, 0.72041E+00_JPRB, 0.74128E+00_JPRB /)
ZBURD(:, 12,  2) = (/ &
&  0.73675E+00_JPRB, 0.71204E+00_JPRB, 0.67718E+00_JPRB, 0.65171E+00_JPRB, 0.63145E+00_JPRB, 0.59762E+00_JPRB, 0.55445E+00_JPRB ,&
&  0.54761E+00_JPRB, 0.49945E+00_JPRB, 0.47368E+00_JPRB, 0.46489E+00_JPRB, 0.46875E+00_JPRB, 0.46760E+00_JPRB, 0.46053E+00_JPRB ,&
&  0.47506E+00_JPRB, 0.51504E+00_JPRB, 0.58781E+00_JPRB, 0.62842E+00_JPRB, 0.64181E+00_JPRB, 0.63468E+00_JPRB, 0.61425E+00_JPRB ,&
&  0.59512E+00_JPRB, 0.59814E+00_JPRB, 0.63514E+00_JPRB, 0.70539E+00_JPRB, 0.73682E+00_JPRB, 0.73206E+00_JPRB, 0.71290E+00_JPRB ,&
&  0.68553E+00_JPRB, 0.66647E+00_JPRB, 0.64981E+00_JPRB, 0.63334E+00_JPRB, 0.59573E+00_JPRB, 0.56204E+00_JPRB, 0.50043E+00_JPRB ,&
&  0.43792E+00_JPRB, 0.44312E+00_JPRB, 0.55298E+00_JPRB, 0.59958E+00_JPRB, 0.62665E+00_JPRB, 0.63749E+00_JPRB, 0.65473E+00_JPRB ,&
&  0.69268E+00_JPRB, 0.74487E+00_JPRB, 0.80849E+00_JPRB, 0.88150E+00_JPRB, 0.94313E+00_JPRB, 0.10094E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.12218E+01_JPRB, 0.10283E+01_JPRB, 0.10204E+01_JPRB, 0.11570E+01_JPRB, 0.16758E+01_JPRB, 0.19823E+01_JPRB, 0.21505E+01_JPRB ,&
&  0.23211E+01_JPRB, 0.23295E+01_JPRB, 0.24541E+01_JPRB, 0.26565E+01_JPRB, 0.27892E+01_JPRB, 0.28248E+01_JPRB, 0.27574E+01_JPRB ,&
&  0.26391E+01_JPRB, 0.25434E+01_JPRB, 0.24366E+01_JPRB, 0.24119E+01_JPRB, 0.23433E+01_JPRB, 0.23293E+01_JPRB, 0.22774E+01_JPRB ,&
&  0.22162E+01_JPRB, 0.21656E+01_JPRB, 0.20153E+01_JPRB, 0.18417E+01_JPRB, 0.16948E+01_JPRB, 0.15972E+01_JPRB, 0.15229E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.14748E+01_JPRB, 0.15008E+01_JPRB, 0.15273E+01_JPRB, 0.14318E+01_JPRB, 0.12443E+01_JPRB, 0.10897E+01_JPRB ,&
&  0.90737E+00_JPRB, 0.79375E+00_JPRB, 0.69427E+00_JPRB, 0.67528E+00_JPRB, 0.69008E+00_JPRB, 0.69897E+00_JPRB, 0.70719E+00_JPRB ,&
&  0.72597E+00_JPRB, 0.72867E+00_JPRB, 0.73280E+00_JPRB, 0.75628E+00_JPRB, 0.75056E+00_JPRB /)
ZBURD(:, 13,  2) = (/ &
&  0.71086E+00_JPRB, 0.70164E+00_JPRB, 0.67995E+00_JPRB, 0.66696E+00_JPRB, 0.65231E+00_JPRB, 0.60842E+00_JPRB, 0.58495E+00_JPRB ,&
&  0.61213E+00_JPRB, 0.57044E+00_JPRB, 0.60187E+00_JPRB, 0.52149E+00_JPRB, 0.48839E+00_JPRB, 0.48413E+00_JPRB, 0.47292E+00_JPRB ,&
&  0.48359E+00_JPRB, 0.52085E+00_JPRB, 0.56970E+00_JPRB, 0.61321E+00_JPRB, 0.65545E+00_JPRB, 0.63629E+00_JPRB, 0.61806E+00_JPRB ,&
&  0.60074E+00_JPRB, 0.60825E+00_JPRB, 0.66189E+00_JPRB, 0.70456E+00_JPRB, 0.74137E+00_JPRB, 0.75952E+00_JPRB, 0.76044E+00_JPRB ,&
&  0.70406E+00_JPRB, 0.69050E+00_JPRB, 0.69145E+00_JPRB, 0.66498E+00_JPRB, 0.62869E+00_JPRB, 0.60647E+00_JPRB, 0.57248E+00_JPRB ,&
&  0.53863E+00_JPRB, 0.52934E+00_JPRB, 0.58839E+00_JPRB, 0.62381E+00_JPRB, 0.63877E+00_JPRB, 0.63851E+00_JPRB, 0.65302E+00_JPRB ,&
&  0.68054E+00_JPRB, 0.72510E+00_JPRB, 0.79954E+00_JPRB, 0.90764E+00_JPRB, 0.10102E+01_JPRB, 0.11328E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.14319E+01_JPRB, 0.11459E+01_JPRB, 0.13718E+01_JPRB, 0.14681E+01_JPRB, 0.17622E+01_JPRB, 0.20974E+01_JPRB, 0.22327E+01_JPRB ,&
&  0.26111E+01_JPRB, 0.26298E+01_JPRB, 0.27653E+01_JPRB, 0.28849E+01_JPRB, 0.30251E+01_JPRB, 0.31004E+01_JPRB, 0.30865E+01_JPRB ,&
&  0.30948E+01_JPRB, 0.27462E+01_JPRB, 0.27765E+01_JPRB, 0.26411E+01_JPRB, 0.25078E+01_JPRB, 0.24495E+01_JPRB, 0.23421E+01_JPRB ,&
&  0.22571E+01_JPRB, 0.21966E+01_JPRB, 0.21090E+01_JPRB, 0.18436E+01_JPRB, 0.17582E+01_JPRB, 0.16838E+01_JPRB, 0.15182E+01_JPRB ,&
&  0.14618E+01_JPRB, 0.14030E+01_JPRB, 0.13357E+01_JPRB, 0.14677E+01_JPRB, 0.14075E+01_JPRB, 0.12348E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.93690E+00_JPRB, 0.80495E+00_JPRB, 0.81560E+00_JPRB, 0.80408E+00_JPRB, 0.76995E+00_JPRB, 0.75685E+00_JPRB, 0.82681E+00_JPRB ,&
&  0.79140E+00_JPRB, 0.78729E+00_JPRB, 0.81053E+00_JPRB, 0.78959E+00_JPRB, 0.74665E+00_JPRB /)
ZBURD(:, 14,  2) = (/ &
&  0.71344E+00_JPRB, 0.71476E+00_JPRB, 0.69109E+00_JPRB, 0.68412E+00_JPRB, 0.67684E+00_JPRB, 0.64220E+00_JPRB, 0.60724E+00_JPRB ,&
&  0.61846E+00_JPRB, 0.61985E+00_JPRB, 0.62400E+00_JPRB, 0.62682E+00_JPRB, 0.62938E+00_JPRB, 0.57589E+00_JPRB, 0.48266E+00_JPRB ,&
&  0.46890E+00_JPRB, 0.51093E+00_JPRB, 0.58434E+00_JPRB, 0.63800E+00_JPRB, 0.65073E+00_JPRB, 0.63841E+00_JPRB, 0.63464E+00_JPRB ,&
&  0.64510E+00_JPRB, 0.65781E+00_JPRB, 0.70065E+00_JPRB, 0.73082E+00_JPRB, 0.74440E+00_JPRB, 0.76572E+00_JPRB, 0.79109E+00_JPRB ,&
&  0.75989E+00_JPRB, 0.76406E+00_JPRB, 0.75496E+00_JPRB, 0.68765E+00_JPRB, 0.69074E+00_JPRB, 0.65862E+00_JPRB, 0.61533E+00_JPRB ,&
&  0.59486E+00_JPRB, 0.60794E+00_JPRB, 0.62666E+00_JPRB, 0.65398E+00_JPRB, 0.67077E+00_JPRB, 0.67583E+00_JPRB, 0.68524E+00_JPRB ,&
&  0.71393E+00_JPRB, 0.77086E+00_JPRB, 0.84083E+00_JPRB, 0.94667E+00_JPRB, 0.10832E+01_JPRB, 0.12339E+01_JPRB, 0.15058E+01_JPRB ,&
&  0.15569E+01_JPRB, 0.16187E+01_JPRB, 0.17073E+01_JPRB, 0.16343E+01_JPRB, 0.18786E+01_JPRB, 0.22227E+01_JPRB, 0.23637E+01_JPRB ,&
&  0.26312E+01_JPRB, 0.26313E+01_JPRB, 0.29146E+01_JPRB, 0.32299E+01_JPRB, 0.35120E+01_JPRB, 0.35384E+01_JPRB, 0.34575E+01_JPRB ,&
&  0.34730E+01_JPRB, 0.33289E+01_JPRB, 0.30151E+01_JPRB, 0.28576E+01_JPRB, 0.26915E+01_JPRB, 0.26117E+01_JPRB, 0.24465E+01_JPRB ,&
&  0.23408E+01_JPRB, 0.22788E+01_JPRB, 0.21956E+01_JPRB, 0.20852E+01_JPRB, 0.19508E+01_JPRB, 0.18054E+01_JPRB, 0.16547E+01_JPRB ,&
&  0.15067E+01_JPRB, 0.13059E+01_JPRB, 0.12195E+01_JPRB, 0.12683E+01_JPRB, 0.13320E+01_JPRB, 0.11331E+01_JPRB, 0.96467E+00_JPRB ,&
&  0.89512E+00_JPRB, 0.86834E+00_JPRB, 0.93864E+00_JPRB, 0.90783E+00_JPRB, 0.87455E+00_JPRB, 0.83881E+00_JPRB, 0.78967E+00_JPRB ,&
&  0.74602E+00_JPRB, 0.81810E+00_JPRB, 0.81145E+00_JPRB, 0.77721E+00_JPRB, 0.73435E+00_JPRB /)
ZBURD(:, 15,  2) = (/ &
&  0.74163E+00_JPRB, 0.75676E+00_JPRB, 0.73011E+00_JPRB, 0.71937E+00_JPRB, 0.71359E+00_JPRB, 0.66734E+00_JPRB, 0.65235E+00_JPRB ,&
&  0.65265E+00_JPRB, 0.65174E+00_JPRB, 0.64646E+00_JPRB, 0.64703E+00_JPRB, 0.64840E+00_JPRB, 0.64198E+00_JPRB, 0.59204E+00_JPRB ,&
&  0.54224E+00_JPRB, 0.61556E+00_JPRB, 0.62161E+00_JPRB, 0.67854E+00_JPRB, 0.73573E+00_JPRB, 0.70302E+00_JPRB, 0.68012E+00_JPRB ,&
&  0.70180E+00_JPRB, 0.70022E+00_JPRB, 0.71226E+00_JPRB, 0.74168E+00_JPRB, 0.78184E+00_JPRB, 0.84535E+00_JPRB, 0.88913E+00_JPRB ,&
&  0.84779E+00_JPRB, 0.83970E+00_JPRB, 0.80286E+00_JPRB, 0.78012E+00_JPRB, 0.76540E+00_JPRB, 0.74460E+00_JPRB, 0.70523E+00_JPRB ,&
&  0.67931E+00_JPRB, 0.68127E+00_JPRB, 0.68922E+00_JPRB, 0.71446E+00_JPRB, 0.73982E+00_JPRB, 0.75679E+00_JPRB, 0.77192E+00_JPRB ,&
&  0.82268E+00_JPRB, 0.89259E+00_JPRB, 0.95339E+00_JPRB, 0.10489E+01_JPRB, 0.11851E+01_JPRB, 0.14824E+01_JPRB, 0.17344E+01_JPRB ,&
&  0.18403E+01_JPRB, 0.18769E+01_JPRB, 0.19420E+01_JPRB, 0.20374E+01_JPRB, 0.22646E+01_JPRB, 0.24722E+01_JPRB, 0.26645E+01_JPRB ,&
&  0.29393E+01_JPRB, 0.29392E+01_JPRB, 0.35524E+01_JPRB, 0.37935E+01_JPRB, 0.38438E+01_JPRB, 0.39370E+01_JPRB, 0.37478E+01_JPRB ,&
&  0.36245E+01_JPRB, 0.33175E+01_JPRB, 0.32081E+01_JPRB, 0.30583E+01_JPRB, 0.29009E+01_JPRB, 0.28395E+01_JPRB, 0.26464E+01_JPRB ,&
&  0.25564E+01_JPRB, 0.24457E+01_JPRB, 0.21244E+01_JPRB, 0.18996E+01_JPRB, 0.16210E+01_JPRB, 0.17263E+01_JPRB, 0.16324E+01_JPRB ,&
&  0.14648E+01_JPRB, 0.12001E+01_JPRB, 0.12674E+01_JPRB, 0.13431E+01_JPRB, 0.14355E+01_JPRB, 0.13642E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.98864E+00_JPRB, 0.10196E+01_JPRB, 0.98507E+00_JPRB, 0.96364E+00_JPRB, 0.94651E+00_JPRB, 0.91861E+00_JPRB, 0.79786E+00_JPRB ,&
&  0.75238E+00_JPRB, 0.80254E+00_JPRB, 0.80449E+00_JPRB, 0.77855E+00_JPRB, 0.75284E+00_JPRB /)
ZBURD(:, 16,  2) = (/ &
&  0.77202E+00_JPRB, 0.80404E+00_JPRB, 0.79664E+00_JPRB, 0.76915E+00_JPRB, 0.75166E+00_JPRB, 0.69867E+00_JPRB, 0.68320E+00_JPRB ,&
&  0.69004E+00_JPRB, 0.68734E+00_JPRB, 0.66648E+00_JPRB, 0.66565E+00_JPRB, 0.66827E+00_JPRB, 0.66579E+00_JPRB, 0.66846E+00_JPRB ,&
&  0.58913E+00_JPRB, 0.70012E+00_JPRB, 0.64599E+00_JPRB, 0.69399E+00_JPRB, 0.88853E+00_JPRB, 0.83913E+00_JPRB, 0.80984E+00_JPRB ,&
&  0.78505E+00_JPRB, 0.77682E+00_JPRB, 0.75312E+00_JPRB, 0.76884E+00_JPRB, 0.81699E+00_JPRB, 0.92671E+00_JPRB, 0.10413E+01_JPRB ,&
&  0.10353E+01_JPRB, 0.10001E+01_JPRB, 0.95598E+00_JPRB, 0.92757E+00_JPRB, 0.86793E+00_JPRB, 0.84222E+00_JPRB, 0.84159E+00_JPRB ,&
&  0.83063E+00_JPRB, 0.80979E+00_JPRB, 0.81272E+00_JPRB, 0.84235E+00_JPRB, 0.86857E+00_JPRB, 0.88200E+00_JPRB, 0.92085E+00_JPRB ,&
&  0.99025E+00_JPRB, 0.10474E+01_JPRB, 0.10836E+01_JPRB, 0.11739E+01_JPRB, 0.12980E+01_JPRB, 0.15865E+01_JPRB, 0.19047E+01_JPRB ,&
&  0.22961E+01_JPRB, 0.23163E+01_JPRB, 0.25338E+01_JPRB, 0.29593E+01_JPRB, 0.31021E+01_JPRB, 0.31302E+01_JPRB, 0.31571E+01_JPRB ,&
&  0.33123E+01_JPRB, 0.33370E+01_JPRB, 0.35921E+01_JPRB, 0.39119E+01_JPRB, 0.38980E+01_JPRB, 0.41088E+01_JPRB, 0.38858E+01_JPRB ,&
&  0.34511E+01_JPRB, 0.32188E+01_JPRB, 0.31198E+01_JPRB, 0.29815E+01_JPRB, 0.27952E+01_JPRB, 0.28329E+01_JPRB, 0.27151E+01_JPRB ,&
&  0.26323E+01_JPRB, 0.23206E+01_JPRB, 0.17257E+01_JPRB, 0.15837E+01_JPRB, 0.12013E+01_JPRB, 0.12456E+01_JPRB, 0.16154E+01_JPRB ,&
&  0.15741E+01_JPRB, 0.14376E+01_JPRB, 0.14421E+01_JPRB, 0.15205E+01_JPRB, 0.16223E+01_JPRB, 0.17969E+01_JPRB, 0.15263E+01_JPRB ,&
&  0.12124E+01_JPRB, 0.11746E+01_JPRB, 0.11429E+01_JPRB, 0.10902E+01_JPRB, 0.10543E+01_JPRB, 0.10388E+01_JPRB, 0.97441E+00_JPRB ,&
&  0.90088E+00_JPRB, 0.85494E+00_JPRB, 0.82954E+00_JPRB, 0.80145E+00_JPRB, 0.78353E+00_JPRB /)
ZBURD(:, 17,  2) = (/ &
&  0.80741E+00_JPRB, 0.83904E+00_JPRB, 0.84692E+00_JPRB, 0.81561E+00_JPRB, 0.78862E+00_JPRB, 0.74252E+00_JPRB, 0.71813E+00_JPRB ,&
&  0.72192E+00_JPRB, 0.71435E+00_JPRB, 0.68703E+00_JPRB, 0.68435E+00_JPRB, 0.68417E+00_JPRB, 0.67986E+00_JPRB, 0.69899E+00_JPRB ,&
&  0.79598E+00_JPRB, 0.90284E+00_JPRB, 0.79635E+00_JPRB, 0.71274E+00_JPRB, 0.87453E+00_JPRB, 0.86098E+00_JPRB, 0.90045E+00_JPRB ,&
&  0.86149E+00_JPRB, 0.91103E+00_JPRB, 0.90781E+00_JPRB, 0.94079E+00_JPRB, 0.10497E+01_JPRB, 0.11859E+01_JPRB, 0.13106E+01_JPRB ,&
&  0.13670E+01_JPRB, 0.14047E+01_JPRB, 0.13611E+01_JPRB, 0.13114E+01_JPRB, 0.12769E+01_JPRB, 0.11679E+01_JPRB, 0.11643E+01_JPRB ,&
&  0.11215E+01_JPRB, 0.10490E+01_JPRB, 0.98997E+00_JPRB, 0.98403E+00_JPRB, 0.98813E+00_JPRB, 0.10035E+01_JPRB, 0.10550E+01_JPRB ,&
&  0.11072E+01_JPRB, 0.11295E+01_JPRB, 0.11303E+01_JPRB, 0.11487E+01_JPRB, 0.12291E+01_JPRB, 0.13986E+01_JPRB, 0.16954E+01_JPRB ,&
&  0.20761E+01_JPRB, 0.22714E+01_JPRB, 0.24583E+01_JPRB, 0.29999E+01_JPRB, 0.32449E+01_JPRB, 0.32266E+01_JPRB, 0.32776E+01_JPRB ,&
&  0.34623E+01_JPRB, 0.36106E+01_JPRB, 0.38562E+01_JPRB, 0.40122E+01_JPRB, 0.41290E+01_JPRB, 0.41676E+01_JPRB, 0.38457E+01_JPRB ,&
&  0.33346E+01_JPRB, 0.30102E+01_JPRB, 0.29005E+01_JPRB, 0.26797E+01_JPRB, 0.25374E+01_JPRB, 0.23089E+01_JPRB, 0.24072E+01_JPRB ,&
&  0.25052E+01_JPRB, 0.13704E+01_JPRB, 0.10362E+01_JPRB, 0.12724E+01_JPRB, 0.10729E+01_JPRB, 0.11287E+01_JPRB, 0.13578E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.13065E+01_JPRB, 0.14976E+01_JPRB, 0.15936E+01_JPRB, 0.17585E+01_JPRB, 0.20308E+01_JPRB, 0.18216E+01_JPRB ,&
&  0.15677E+01_JPRB, 0.14407E+01_JPRB, 0.14282E+01_JPRB, 0.13349E+01_JPRB, 0.13080E+01_JPRB, 0.12407E+01_JPRB, 0.11608E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.99308E+00_JPRB, 0.92688E+00_JPRB, 0.87320E+00_JPRB, 0.83125E+00_JPRB /)
ZBURD(:, 18,  2) = (/ &
&  0.89389E+00_JPRB, 0.90777E+00_JPRB, 0.89792E+00_JPRB, 0.86872E+00_JPRB, 0.83155E+00_JPRB, 0.80103E+00_JPRB, 0.76747E+00_JPRB ,&
&  0.75353E+00_JPRB, 0.73899E+00_JPRB, 0.71163E+00_JPRB, 0.70360E+00_JPRB, 0.68982E+00_JPRB, 0.68209E+00_JPRB, 0.70537E+00_JPRB ,&
&  0.77796E+00_JPRB, 0.89842E+00_JPRB, 0.73330E+00_JPRB, 0.70195E+00_JPRB, 0.68324E+00_JPRB, 0.78076E+00_JPRB, 0.86844E+00_JPRB ,&
&  0.96905E+00_JPRB, 0.10009E+01_JPRB, 0.10574E+01_JPRB, 0.11748E+01_JPRB, 0.13643E+01_JPRB, 0.14887E+01_JPRB, 0.16707E+01_JPRB ,&
&  0.17407E+01_JPRB, 0.18566E+01_JPRB, 0.17332E+01_JPRB, 0.17517E+01_JPRB, 0.17501E+01_JPRB, 0.16158E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.14170E+01_JPRB, 0.12885E+01_JPRB, 0.11915E+01_JPRB, 0.11199E+01_JPRB, 0.10843E+01_JPRB, 0.10628E+01_JPRB, 0.10796E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.11122E+01_JPRB, 0.11091E+01_JPRB, 0.11133E+01_JPRB, 0.11700E+01_JPRB, 0.13057E+01_JPRB, 0.14281E+01_JPRB ,&
&  0.17905E+01_JPRB, 0.18254E+01_JPRB, 0.16102E+01_JPRB, 0.24107E+01_JPRB, 0.33431E+01_JPRB, 0.31364E+01_JPRB, 0.28504E+01_JPRB ,&
&  0.32114E+01_JPRB, 0.35201E+01_JPRB, 0.38468E+01_JPRB, 0.39916E+01_JPRB, 0.40562E+01_JPRB, 0.39585E+01_JPRB, 0.36509E+01_JPRB ,&
&  0.32266E+01_JPRB, 0.30015E+01_JPRB, 0.29337E+01_JPRB, 0.29573E+01_JPRB, 0.27511E+01_JPRB, 0.25727E+01_JPRB, 0.24639E+01_JPRB ,&
&  0.21193E+01_JPRB, 0.17630E+01_JPRB, 0.11629E+01_JPRB, 0.12311E+01_JPRB, 0.96172E+00_JPRB, 0.99237E+00_JPRB, 0.11938E+01_JPRB ,&
&  0.11790E+01_JPRB, 0.12976E+01_JPRB, 0.14931E+01_JPRB, 0.16407E+01_JPRB, 0.20830E+01_JPRB, 0.21918E+01_JPRB, 0.21427E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.16414E+01_JPRB, 0.16760E+01_JPRB, 0.16208E+01_JPRB, 0.16003E+01_JPRB, 0.14956E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.11498E+01_JPRB, 0.10748E+01_JPRB, 0.98964E+00_JPRB, 0.91663E+00_JPRB /)
ZBURD(:, 19,  2) = (/ &
&  0.10026E+01_JPRB, 0.98120E+00_JPRB, 0.93842E+00_JPRB, 0.88743E+00_JPRB, 0.83993E+00_JPRB, 0.81390E+00_JPRB, 0.78967E+00_JPRB ,&
&  0.77971E+00_JPRB, 0.75975E+00_JPRB, 0.73764E+00_JPRB, 0.71812E+00_JPRB, 0.70530E+00_JPRB, 0.69957E+00_JPRB, 0.72505E+00_JPRB ,&
&  0.78247E+00_JPRB, 0.75813E+00_JPRB, 0.60976E+00_JPRB, 0.55858E+00_JPRB, 0.54078E+00_JPRB, 0.64133E+00_JPRB, 0.76797E+00_JPRB ,&
&  0.92346E+00_JPRB, 0.10212E+01_JPRB, 0.11946E+01_JPRB, 0.14107E+01_JPRB, 0.16582E+01_JPRB, 0.18611E+01_JPRB, 0.20155E+01_JPRB ,&
&  0.24437E+01_JPRB, 0.22420E+01_JPRB, 0.23065E+01_JPRB, 0.21960E+01_JPRB, 0.21361E+01_JPRB, 0.19178E+01_JPRB, 0.16828E+01_JPRB ,&
&  0.15029E+01_JPRB, 0.13490E+01_JPRB, 0.12118E+01_JPRB, 0.11051E+01_JPRB, 0.10479E+01_JPRB, 0.10400E+01_JPRB, 0.10815E+01_JPRB ,&
&  0.11316E+01_JPRB, 0.11223E+01_JPRB, 0.10931E+01_JPRB, 0.11195E+01_JPRB, 0.12361E+01_JPRB, 0.13674E+01_JPRB, 0.14781E+01_JPRB ,&
&  0.15392E+01_JPRB, 0.16913E+01_JPRB, 0.25618E+01_JPRB, 0.28059E+01_JPRB, 0.31439E+01_JPRB, 0.31796E+01_JPRB, 0.31108E+01_JPRB ,&
&  0.33771E+01_JPRB, 0.35146E+01_JPRB, 0.37039E+01_JPRB, 0.30822E+01_JPRB, 0.36169E+01_JPRB, 0.35862E+01_JPRB, 0.33046E+01_JPRB ,&
&  0.30346E+01_JPRB, 0.30242E+01_JPRB, 0.28684E+01_JPRB, 0.28130E+01_JPRB, 0.28348E+01_JPRB, 0.29279E+01_JPRB, 0.22929E+01_JPRB ,&
&  0.17600E+01_JPRB, 0.18999E+01_JPRB, 0.16744E+01_JPRB, 0.13674E+01_JPRB, 0.11328E+01_JPRB, 0.11832E+01_JPRB, 0.12401E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.14221E+01_JPRB, 0.15977E+01_JPRB, 0.20256E+01_JPRB, 0.25217E+01_JPRB, 0.26442E+01_JPRB, 0.24677E+01_JPRB ,&
&  0.20848E+01_JPRB, 0.21364E+01_JPRB, 0.20449E+01_JPRB, 0.19333E+01_JPRB, 0.18761E+01_JPRB, 0.17506E+01_JPRB, 0.15820E+01_JPRB ,&
&  0.14115E+01_JPRB, 0.12876E+01_JPRB, 0.11902E+01_JPRB, 0.10985E+01_JPRB, 0.10315E+01_JPRB /)
ZBURD(:, 20,  2) = (/ &
&  0.10691E+01_JPRB, 0.10350E+01_JPRB, 0.98338E+00_JPRB, 0.92905E+00_JPRB, 0.87560E+00_JPRB, 0.84206E+00_JPRB, 0.81581E+00_JPRB ,&
&  0.80970E+00_JPRB, 0.76956E+00_JPRB, 0.74900E+00_JPRB, 0.72945E+00_JPRB, 0.71945E+00_JPRB, 0.72344E+00_JPRB, 0.75221E+00_JPRB ,&
&  0.79305E+00_JPRB, 0.69676E+00_JPRB, 0.54663E+00_JPRB, 0.56764E+00_JPRB, 0.63554E+00_JPRB, 0.64925E+00_JPRB, 0.76290E+00_JPRB ,&
&  0.90762E+00_JPRB, 0.10668E+01_JPRB, 0.12700E+01_JPRB, 0.16385E+01_JPRB, 0.20080E+01_JPRB, 0.23409E+01_JPRB, 0.26196E+01_JPRB ,&
&  0.26773E+01_JPRB, 0.27850E+01_JPRB, 0.27120E+01_JPRB, 0.24535E+01_JPRB, 0.20897E+01_JPRB, 0.17765E+01_JPRB, 0.15811E+01_JPRB ,&
&  0.14304E+01_JPRB, 0.12793E+01_JPRB, 0.11719E+01_JPRB, 0.11095E+01_JPRB, 0.10862E+01_JPRB, 0.11045E+01_JPRB, 0.11448E+01_JPRB ,&
&  0.11652E+01_JPRB, 0.11422E+01_JPRB, 0.11232E+01_JPRB, 0.11889E+01_JPRB, 0.12193E+01_JPRB, 0.12798E+01_JPRB, 0.12824E+01_JPRB ,&
&  0.17569E+01_JPRB, 0.21715E+01_JPRB, 0.24203E+01_JPRB, 0.27659E+01_JPRB, 0.24287E+01_JPRB, 0.25979E+01_JPRB, 0.31979E+01_JPRB ,&
&  0.34897E+01_JPRB, 0.35382E+01_JPRB, 0.37228E+01_JPRB, 0.26503E+01_JPRB, 0.23436E+01_JPRB, 0.32619E+01_JPRB, 0.31476E+01_JPRB ,&
&  0.28683E+01_JPRB, 0.29336E+01_JPRB, 0.28372E+01_JPRB, 0.30014E+01_JPRB, 0.22670E+01_JPRB, 0.20096E+01_JPRB, 0.14941E+01_JPRB ,&
&  0.13879E+01_JPRB, 0.12333E+01_JPRB, 0.14235E+01_JPRB, 0.13242E+01_JPRB, 0.13374E+01_JPRB, 0.13449E+01_JPRB, 0.13113E+01_JPRB ,&
&  0.13814E+01_JPRB, 0.15257E+01_JPRB, 0.19108E+01_JPRB, 0.26332E+01_JPRB, 0.32710E+01_JPRB, 0.29120E+01_JPRB, 0.29327E+01_JPRB ,&
&  0.28073E+01_JPRB, 0.26363E+01_JPRB, 0.23373E+01_JPRB, 0.22748E+01_JPRB, 0.21250E+01_JPRB, 0.19142E+01_JPRB, 0.16719E+01_JPRB ,&
&  0.14732E+01_JPRB, 0.13529E+01_JPRB, 0.12456E+01_JPRB, 0.11624E+01_JPRB, 0.11053E+01_JPRB /)
ZBURD(:, 21,  2) = (/ &
&  0.11234E+01_JPRB, 0.10773E+01_JPRB, 0.10265E+01_JPRB, 0.96959E+00_JPRB, 0.91753E+00_JPRB, 0.87688E+00_JPRB, 0.84907E+00_JPRB ,&
&  0.84621E+00_JPRB, 0.79679E+00_JPRB, 0.75368E+00_JPRB, 0.73549E+00_JPRB, 0.73383E+00_JPRB, 0.74541E+00_JPRB, 0.77532E+00_JPRB ,&
&  0.83544E+00_JPRB, 0.83830E+00_JPRB, 0.62663E+00_JPRB, 0.57359E+00_JPRB, 0.72075E+00_JPRB, 0.71247E+00_JPRB, 0.76541E+00_JPRB ,&
&  0.94886E+00_JPRB, 0.11513E+01_JPRB, 0.14599E+01_JPRB, 0.20069E+01_JPRB, 0.24719E+01_JPRB, 0.29463E+01_JPRB, 0.32089E+01_JPRB ,&
&  0.33422E+01_JPRB, 0.30454E+01_JPRB, 0.25950E+01_JPRB, 0.21415E+01_JPRB, 0.17906E+01_JPRB, 0.15553E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.12925E+01_JPRB, 0.12005E+01_JPRB, 0.11448E+01_JPRB, 0.10994E+01_JPRB, 0.11156E+01_JPRB, 0.11218E+01_JPRB, 0.11518E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11595E+01_JPRB, 0.11811E+01_JPRB, 0.12154E+01_JPRB, 0.12571E+01_JPRB, 0.12624E+01_JPRB, 0.13637E+01_JPRB ,&
&  0.15392E+01_JPRB, 0.18479E+01_JPRB, 0.22018E+01_JPRB, 0.25015E+01_JPRB, 0.27720E+01_JPRB, 0.28481E+01_JPRB, 0.32932E+01_JPRB ,&
&  0.27771E+01_JPRB, 0.23116E+01_JPRB, 0.19837E+01_JPRB, 0.15993E+01_JPRB, 0.19451E+01_JPRB, 0.29224E+01_JPRB, 0.29764E+01_JPRB ,&
&  0.28537E+01_JPRB, 0.29184E+01_JPRB, 0.29045E+01_JPRB, 0.24460E+01_JPRB, 0.20041E+01_JPRB, 0.15283E+01_JPRB, 0.18365E+01_JPRB ,&
&  0.18782E+01_JPRB, 0.18094E+01_JPRB, 0.16636E+01_JPRB, 0.14573E+01_JPRB, 0.12415E+01_JPRB, 0.14897E+01_JPRB, 0.15239E+01_JPRB ,&
&  0.15823E+01_JPRB, 0.17534E+01_JPRB, 0.28264E+01_JPRB, 0.37402E+01_JPRB, 0.38394E+01_JPRB, 0.34731E+01_JPRB, 0.35534E+01_JPRB ,&
&  0.32321E+01_JPRB, 0.30478E+01_JPRB, 0.27396E+01_JPRB, 0.24092E+01_JPRB, 0.21065E+01_JPRB, 0.18444E+01_JPRB, 0.16286E+01_JPRB ,&
&  0.14812E+01_JPRB, 0.13717E+01_JPRB, 0.12653E+01_JPRB, 0.12082E+01_JPRB, 0.11621E+01_JPRB /)
ZBURD(:, 22,  2) = (/ &
&  0.11797E+01_JPRB, 0.11272E+01_JPRB, 0.10753E+01_JPRB, 0.10216E+01_JPRB, 0.97185E+00_JPRB, 0.91549E+00_JPRB, 0.89122E+00_JPRB ,&
&  0.87705E+00_JPRB, 0.82435E+00_JPRB, 0.77392E+00_JPRB, 0.75575E+00_JPRB, 0.76842E+00_JPRB, 0.78817E+00_JPRB, 0.81528E+00_JPRB ,&
&  0.87363E+00_JPRB, 0.11014E+01_JPRB, 0.82413E+00_JPRB, 0.68491E+00_JPRB, 0.74376E+00_JPRB, 0.76584E+00_JPRB, 0.72163E+00_JPRB ,&
&  0.97672E+00_JPRB, 0.12664E+01_JPRB, 0.16035E+01_JPRB, 0.22069E+01_JPRB, 0.30344E+01_JPRB, 0.30369E+01_JPRB, 0.31800E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.27491E+01_JPRB, 0.22907E+01_JPRB, 0.19246E+01_JPRB, 0.16682E+01_JPRB, 0.14851E+01_JPRB, 0.13714E+01_JPRB ,&
&  0.12918E+01_JPRB, 0.12291E+01_JPRB, 0.11780E+01_JPRB, 0.11615E+01_JPRB, 0.11475E+01_JPRB, 0.11442E+01_JPRB, 0.11702E+01_JPRB ,&
&  0.11793E+01_JPRB, 0.11569E+01_JPRB, 0.11686E+01_JPRB, 0.12229E+01_JPRB, 0.13750E+01_JPRB, 0.12509E+01_JPRB, 0.16660E+01_JPRB ,&
&  0.15079E+01_JPRB, 0.17299E+01_JPRB, 0.20114E+01_JPRB, 0.25933E+01_JPRB, 0.28016E+01_JPRB, 0.28211E+01_JPRB, 0.30724E+01_JPRB ,&
&  0.24350E+01_JPRB, 0.21777E+01_JPRB, 0.23042E+01_JPRB, 0.22767E+01_JPRB, 0.17849E+01_JPRB, 0.18723E+01_JPRB, 0.28724E+01_JPRB ,&
&  0.22551E+01_JPRB, 0.25024E+01_JPRB, 0.26168E+01_JPRB, 0.25958E+01_JPRB, 0.12474E+01_JPRB, 0.86489E+00_JPRB, 0.13141E+01_JPRB ,&
&  0.14593E+01_JPRB, 0.10026E+01_JPRB, 0.75437E+00_JPRB, 0.95073E+00_JPRB, 0.77157E+00_JPRB, 0.10084E+01_JPRB, 0.14983E+01_JPRB ,&
&  0.18011E+01_JPRB, 0.22087E+01_JPRB, 0.36524E+01_JPRB, 0.42311E+01_JPRB, 0.41871E+01_JPRB, 0.40263E+01_JPRB, 0.36684E+01_JPRB ,&
&  0.34212E+01_JPRB, 0.29066E+01_JPRB, 0.28155E+01_JPRB, 0.24526E+01_JPRB, 0.21285E+01_JPRB, 0.18920E+01_JPRB, 0.16998E+01_JPRB ,&
&  0.15531E+01_JPRB, 0.14272E+01_JPRB, 0.13340E+01_JPRB, 0.12765E+01_JPRB, 0.12295E+01_JPRB /)
ZBURD(:, 23,  2) = (/ &
&  0.12585E+01_JPRB, 0.11927E+01_JPRB, 0.11291E+01_JPRB, 0.10723E+01_JPRB, 0.10125E+01_JPRB, 0.96264E+00_JPRB, 0.93950E+00_JPRB ,&
&  0.90311E+00_JPRB, 0.84633E+00_JPRB, 0.80566E+00_JPRB, 0.77361E+00_JPRB, 0.77698E+00_JPRB, 0.79542E+00_JPRB, 0.79760E+00_JPRB ,&
&  0.81768E+00_JPRB, 0.89360E+00_JPRB, 0.99506E+00_JPRB, 0.89940E+00_JPRB, 0.82969E+00_JPRB, 0.78441E+00_JPRB, 0.79649E+00_JPRB ,&
&  0.11145E+01_JPRB, 0.14734E+01_JPRB, 0.18546E+01_JPRB, 0.23515E+01_JPRB, 0.27874E+01_JPRB, 0.27909E+01_JPRB, 0.30452E+01_JPRB ,&
&  0.29032E+01_JPRB, 0.25529E+01_JPRB, 0.22127E+01_JPRB, 0.18837E+01_JPRB, 0.16491E+01_JPRB, 0.14716E+01_JPRB, 0.13374E+01_JPRB ,&
&  0.12565E+01_JPRB, 0.12100E+01_JPRB, 0.11682E+01_JPRB, 0.11328E+01_JPRB, 0.11154E+01_JPRB, 0.11165E+01_JPRB, 0.11464E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11221E+01_JPRB, 0.11330E+01_JPRB, 0.11509E+01_JPRB, 0.12318E+01_JPRB, 0.12396E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.12847E+01_JPRB, 0.18967E+01_JPRB, 0.23413E+01_JPRB, 0.24920E+01_JPRB, 0.26520E+01_JPRB, 0.26421E+01_JPRB ,&
&  0.26004E+01_JPRB, 0.26834E+01_JPRB, 0.23686E+01_JPRB, 0.24875E+01_JPRB, 0.21957E+01_JPRB, 0.15489E+01_JPRB, 0.19269E+01_JPRB ,&
&  0.21301E+01_JPRB, 0.20920E+01_JPRB, 0.16974E+01_JPRB, 0.12127E+01_JPRB, 0.12243E+01_JPRB, 0.93434E+00_JPRB, 0.67546E+00_JPRB ,&
&  0.64311E+00_JPRB, 0.61995E+00_JPRB, 0.57192E+00_JPRB, 0.59175E+00_JPRB, 0.60740E+00_JPRB, 0.94110E+00_JPRB, 0.14932E+01_JPRB ,&
&  0.23109E+01_JPRB, 0.32653E+01_JPRB, 0.42314E+01_JPRB, 0.46206E+01_JPRB, 0.42316E+01_JPRB, 0.39827E+01_JPRB, 0.37383E+01_JPRB ,&
&  0.30969E+01_JPRB, 0.28139E+01_JPRB, 0.26374E+01_JPRB, 0.23455E+01_JPRB, 0.20947E+01_JPRB, 0.19123E+01_JPRB, 0.17666E+01_JPRB ,&
&  0.16263E+01_JPRB, 0.15049E+01_JPRB, 0.14205E+01_JPRB, 0.13658E+01_JPRB, 0.13169E+01_JPRB /)
ZBURD(:, 24,  2) = (/ &
&  0.13011E+01_JPRB, 0.12352E+01_JPRB, 0.11745E+01_JPRB, 0.11232E+01_JPRB, 0.10693E+01_JPRB, 0.10218E+01_JPRB, 0.98516E+00_JPRB ,&
&  0.92832E+00_JPRB, 0.86288E+00_JPRB, 0.81813E+00_JPRB, 0.79156E+00_JPRB, 0.78572E+00_JPRB, 0.79484E+00_JPRB, 0.79777E+00_JPRB ,&
&  0.81176E+00_JPRB, 0.84500E+00_JPRB, 0.99263E+00_JPRB, 0.98910E+00_JPRB, 0.98184E+00_JPRB, 0.94745E+00_JPRB, 0.10107E+01_JPRB ,&
&  0.12951E+01_JPRB, 0.18088E+01_JPRB, 0.22700E+01_JPRB, 0.26318E+01_JPRB, 0.29946E+01_JPRB, 0.30987E+01_JPRB, 0.29948E+01_JPRB ,&
&  0.27137E+01_JPRB, 0.24497E+01_JPRB, 0.21497E+01_JPRB, 0.18158E+01_JPRB, 0.15726E+01_JPRB, 0.14307E+01_JPRB, 0.13290E+01_JPRB ,&
&  0.12441E+01_JPRB, 0.12107E+01_JPRB, 0.11662E+01_JPRB, 0.11139E+01_JPRB, 0.10753E+01_JPRB, 0.10818E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.11055E+01_JPRB, 0.11206E+01_JPRB, 0.11519E+01_JPRB, 0.11549E+01_JPRB, 0.10425E+01_JPRB, 0.92596E+00_JPRB, 0.10703E+01_JPRB ,&
&  0.12628E+01_JPRB, 0.15956E+01_JPRB, 0.17594E+01_JPRB, 0.23194E+01_JPRB, 0.24725E+01_JPRB, 0.24246E+01_JPRB, 0.25169E+01_JPRB ,&
&  0.24865E+01_JPRB, 0.26350E+01_JPRB, 0.20452E+01_JPRB, 0.20791E+01_JPRB, 0.22937E+01_JPRB, 0.17438E+01_JPRB, 0.16428E+01_JPRB ,&
&  0.19261E+01_JPRB, 0.18065E+01_JPRB, 0.15552E+01_JPRB, 0.11938E+01_JPRB, 0.15149E+01_JPRB, 0.14512E+01_JPRB, 0.79905E+00_JPRB ,&
&  0.67688E+00_JPRB, 0.63452E+00_JPRB, 0.60473E+00_JPRB, 0.59000E+00_JPRB, 0.64112E+00_JPRB, 0.11125E+01_JPRB, 0.22254E+01_JPRB ,&
&  0.27724E+01_JPRB, 0.42041E+01_JPRB, 0.49260E+01_JPRB, 0.50565E+01_JPRB, 0.43762E+01_JPRB, 0.36702E+01_JPRB, 0.31238E+01_JPRB ,&
&  0.29103E+01_JPRB, 0.28424E+01_JPRB, 0.25821E+01_JPRB, 0.23819E+01_JPRB, 0.21769E+01_JPRB, 0.19914E+01_JPRB, 0.18255E+01_JPRB ,&
&  0.16669E+01_JPRB, 0.15551E+01_JPRB, 0.14710E+01_JPRB, 0.14291E+01_JPRB, 0.13674E+01_JPRB /)
ZBURD(:, 25,  2) = (/ &
&  0.13183E+01_JPRB, 0.12670E+01_JPRB, 0.11947E+01_JPRB, 0.11401E+01_JPRB, 0.10870E+01_JPRB, 0.10309E+01_JPRB, 0.10018E+01_JPRB ,&
&  0.95387E+00_JPRB, 0.88848E+00_JPRB, 0.83846E+00_JPRB, 0.81822E+00_JPRB, 0.80609E+00_JPRB, 0.79583E+00_JPRB, 0.79356E+00_JPRB ,&
&  0.79369E+00_JPRB, 0.81056E+00_JPRB, 0.91370E+00_JPRB, 0.10332E+01_JPRB, 0.10132E+01_JPRB, 0.95088E+00_JPRB, 0.10626E+01_JPRB ,&
&  0.15384E+01_JPRB, 0.20266E+01_JPRB, 0.23303E+01_JPRB, 0.25971E+01_JPRB, 0.26212E+01_JPRB, 0.26302E+01_JPRB, 0.25036E+01_JPRB ,&
&  0.22878E+01_JPRB, 0.20705E+01_JPRB, 0.18649E+01_JPRB, 0.16350E+01_JPRB, 0.14504E+01_JPRB, 0.13476E+01_JPRB, 0.12758E+01_JPRB ,&
&  0.12447E+01_JPRB, 0.12045E+01_JPRB, 0.11547E+01_JPRB, 0.10808E+01_JPRB, 0.10410E+01_JPRB, 0.10559E+01_JPRB, 0.10790E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.11586E+01_JPRB, 0.11950E+01_JPRB, 0.11861E+01_JPRB, 0.99442E+00_JPRB, 0.11436E+01_JPRB, 0.12201E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.14759E+01_JPRB, 0.15199E+01_JPRB, 0.19278E+01_JPRB, 0.22472E+01_JPRB, 0.22547E+01_JPRB, 0.22271E+01_JPRB ,&
&  0.23291E+01_JPRB, 0.21127E+01_JPRB, 0.17982E+01_JPRB, 0.18619E+01_JPRB, 0.19791E+01_JPRB, 0.20957E+01_JPRB, 0.13986E+01_JPRB ,&
&  0.12922E+01_JPRB, 0.15545E+01_JPRB, 0.14084E+01_JPRB, 0.11505E+01_JPRB, 0.14160E+01_JPRB, 0.17254E+01_JPRB, 0.14110E+01_JPRB ,&
&  0.76776E+00_JPRB, 0.64268E+00_JPRB, 0.68799E+00_JPRB, 0.75321E+00_JPRB, 0.72514E+00_JPRB, 0.12866E+01_JPRB, 0.42201E+01_JPRB ,&
&  0.40006E+01_JPRB, 0.51150E+01_JPRB, 0.51538E+01_JPRB, 0.47634E+01_JPRB, 0.40789E+01_JPRB, 0.33318E+01_JPRB, 0.30552E+01_JPRB ,&
&  0.28165E+01_JPRB, 0.26867E+01_JPRB, 0.25578E+01_JPRB, 0.24291E+01_JPRB, 0.22522E+01_JPRB, 0.20205E+01_JPRB, 0.18217E+01_JPRB ,&
&  0.16696E+01_JPRB, 0.15649E+01_JPRB, 0.14786E+01_JPRB, 0.14175E+01_JPRB, 0.13591E+01_JPRB /)
ZBURD(:, 26,  2) = (/ &
&  0.13496E+01_JPRB, 0.12684E+01_JPRB, 0.11868E+01_JPRB, 0.11341E+01_JPRB, 0.10754E+01_JPRB, 0.10234E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.99395E+00_JPRB, 0.92724E+00_JPRB, 0.86355E+00_JPRB, 0.83609E+00_JPRB, 0.81547E+00_JPRB, 0.80037E+00_JPRB, 0.80169E+00_JPRB ,&
&  0.80011E+00_JPRB, 0.82052E+00_JPRB, 0.88292E+00_JPRB, 0.98233E+00_JPRB, 0.98420E+00_JPRB, 0.90561E+00_JPRB, 0.10389E+01_JPRB ,&
&  0.15991E+01_JPRB, 0.20376E+01_JPRB, 0.21251E+01_JPRB, 0.22640E+01_JPRB, 0.23054E+01_JPRB, 0.23516E+01_JPRB, 0.21579E+01_JPRB ,&
&  0.18595E+01_JPRB, 0.16577E+01_JPRB, 0.15069E+01_JPRB, 0.14235E+01_JPRB, 0.13489E+01_JPRB, 0.12845E+01_JPRB, 0.12463E+01_JPRB ,&
&  0.11944E+01_JPRB, 0.11510E+01_JPRB, 0.10916E+01_JPRB, 0.10250E+01_JPRB, 0.10279E+01_JPRB, 0.10608E+01_JPRB, 0.10959E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.12138E+01_JPRB, 0.12339E+01_JPRB, 0.11640E+01_JPRB, 0.11611E+01_JPRB, 0.11867E+01_JPRB, 0.12756E+01_JPRB ,&
&  0.13105E+01_JPRB, 0.14236E+01_JPRB, 0.15813E+01_JPRB, 0.17590E+01_JPRB, 0.19867E+01_JPRB, 0.20795E+01_JPRB, 0.20179E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.18370E+01_JPRB, 0.15503E+01_JPRB, 0.15972E+01_JPRB, 0.17328E+01_JPRB, 0.19108E+01_JPRB, 0.17509E+01_JPRB ,&
&  0.14954E+01_JPRB, 0.12669E+01_JPRB, 0.11924E+01_JPRB, 0.12239E+01_JPRB, 0.13775E+01_JPRB, 0.16198E+01_JPRB, 0.21954E+01_JPRB ,&
&  0.22142E+01_JPRB, 0.15450E+01_JPRB, 0.13246E+01_JPRB, 0.17238E+01_JPRB, 0.16059E+01_JPRB, 0.17546E+01_JPRB, 0.29126E+01_JPRB ,&
&  0.40746E+01_JPRB, 0.49707E+01_JPRB, 0.48610E+01_JPRB, 0.42659E+01_JPRB, 0.37686E+01_JPRB, 0.31383E+01_JPRB, 0.29331E+01_JPRB ,&
&  0.27594E+01_JPRB, 0.25970E+01_JPRB, 0.25444E+01_JPRB, 0.24385E+01_JPRB, 0.22299E+01_JPRB, 0.19737E+01_JPRB, 0.17488E+01_JPRB ,&
&  0.16128E+01_JPRB, 0.15324E+01_JPRB, 0.14747E+01_JPRB, 0.14331E+01_JPRB, 0.13953E+01_JPRB /)
ZBURD(:, 27,  2) = (/ &
&  0.13286E+01_JPRB, 0.12282E+01_JPRB, 0.11592E+01_JPRB, 0.11036E+01_JPRB, 0.10441E+01_JPRB, 0.10176E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.94851E+00_JPRB, 0.88863E+00_JPRB, 0.84142E+00_JPRB, 0.81120E+00_JPRB, 0.80691E+00_JPRB, 0.81975E+00_JPRB ,&
&  0.82278E+00_JPRB, 0.84292E+00_JPRB, 0.86590E+00_JPRB, 0.95256E+00_JPRB, 0.10655E+01_JPRB, 0.12980E+01_JPRB, 0.10185E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.20650E+01_JPRB, 0.21128E+01_JPRB, 0.20742E+01_JPRB, 0.19792E+01_JPRB, 0.19216E+01_JPRB, 0.17435E+01_JPRB ,&
&  0.14988E+01_JPRB, 0.13707E+01_JPRB, 0.12849E+01_JPRB, 0.12503E+01_JPRB, 0.12412E+01_JPRB, 0.12022E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.11113E+01_JPRB, 0.10726E+01_JPRB, 0.10320E+01_JPRB, 0.10161E+01_JPRB, 0.10319E+01_JPRB, 0.10689E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.12523E+01_JPRB, 0.12579E+01_JPRB, 0.11826E+01_JPRB, 0.12082E+01_JPRB, 0.12302E+01_JPRB, 0.12690E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.11912E+01_JPRB, 0.14326E+01_JPRB, 0.16954E+01_JPRB, 0.18353E+01_JPRB, 0.17742E+01_JPRB, 0.16601E+01_JPRB ,&
&  0.17511E+01_JPRB, 0.17520E+01_JPRB, 0.16878E+01_JPRB, 0.15310E+01_JPRB, 0.16529E+01_JPRB, 0.17825E+01_JPRB, 0.18557E+01_JPRB ,&
&  0.17878E+01_JPRB, 0.15315E+01_JPRB, 0.13897E+01_JPRB, 0.13397E+01_JPRB, 0.13486E+01_JPRB, 0.15641E+01_JPRB, 0.20423E+01_JPRB ,&
&  0.25765E+01_JPRB, 0.33574E+01_JPRB, 0.33874E+01_JPRB, 0.27102E+01_JPRB, 0.24607E+01_JPRB, 0.25637E+01_JPRB, 0.28159E+01_JPRB ,&
&  0.41793E+01_JPRB, 0.45631E+01_JPRB, 0.42426E+01_JPRB, 0.44268E+01_JPRB, 0.35635E+01_JPRB, 0.30069E+01_JPRB, 0.27425E+01_JPRB ,&
&  0.26544E+01_JPRB, 0.25318E+01_JPRB, 0.24065E+01_JPRB, 0.22603E+01_JPRB, 0.20281E+01_JPRB, 0.18085E+01_JPRB, 0.16583E+01_JPRB ,&
&  0.15630E+01_JPRB, 0.15060E+01_JPRB, 0.14604E+01_JPRB, 0.14290E+01_JPRB, 0.13964E+01_JPRB /)
ZBURD(:, 28,  2) = (/ &
&  0.12533E+01_JPRB, 0.11645E+01_JPRB, 0.10792E+01_JPRB, 0.10418E+01_JPRB, 0.10118E+01_JPRB, 0.10147E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.11274E+01_JPRB, 0.97098E+00_JPRB, 0.88353E+00_JPRB, 0.82349E+00_JPRB, 0.80660E+00_JPRB, 0.81930E+00_JPRB, 0.83117E+00_JPRB ,&
&  0.84272E+00_JPRB, 0.86746E+00_JPRB, 0.91851E+00_JPRB, 0.95396E+00_JPRB, 0.11059E+01_JPRB, 0.13383E+01_JPRB, 0.13541E+01_JPRB ,&
&  0.15519E+01_JPRB, 0.24684E+01_JPRB, 0.20880E+01_JPRB, 0.18907E+01_JPRB, 0.17405E+01_JPRB, 0.17974E+01_JPRB, 0.15666E+01_JPRB ,&
&  0.13486E+01_JPRB, 0.12877E+01_JPRB, 0.12717E+01_JPRB, 0.12080E+01_JPRB, 0.11811E+01_JPRB, 0.11262E+01_JPRB, 0.11005E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10511E+01_JPRB, 0.10229E+01_JPRB, 0.10276E+01_JPRB, 0.10594E+01_JPRB, 0.11057E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.12703E+01_JPRB, 0.12165E+01_JPRB, 0.12115E+01_JPRB, 0.12275E+01_JPRB, 0.12640E+01_JPRB, 0.12884E+01_JPRB ,&
&  0.12593E+01_JPRB, 0.12248E+01_JPRB, 0.13702E+01_JPRB, 0.14384E+01_JPRB, 0.14585E+01_JPRB, 0.15454E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.16244E+01_JPRB, 0.15361E+01_JPRB, 0.17182E+01_JPRB, 0.14737E+01_JPRB, 0.16237E+01_JPRB, 0.17481E+01_JPRB, 0.16698E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.14262E+01_JPRB, 0.13166E+01_JPRB, 0.13235E+01_JPRB, 0.14708E+01_JPRB, 0.17954E+01_JPRB, 0.21792E+01_JPRB ,&
&  0.25960E+01_JPRB, 0.30500E+01_JPRB, 0.34103E+01_JPRB, 0.27572E+01_JPRB, 0.25599E+01_JPRB, 0.26225E+01_JPRB, 0.30496E+01_JPRB ,&
&  0.40702E+01_JPRB, 0.43035E+01_JPRB, 0.43520E+01_JPRB, 0.39102E+01_JPRB, 0.31267E+01_JPRB, 0.26931E+01_JPRB, 0.24878E+01_JPRB ,&
&  0.23708E+01_JPRB, 0.22598E+01_JPRB, 0.20529E+01_JPRB, 0.18823E+01_JPRB, 0.17611E+01_JPRB, 0.16564E+01_JPRB, 0.15443E+01_JPRB ,&
&  0.14725E+01_JPRB, 0.14104E+01_JPRB, 0.13787E+01_JPRB, 0.13629E+01_JPRB, 0.13233E+01_JPRB /)
ZBURD(:, 29,  2) = (/ &
&  0.11331E+01_JPRB, 0.10723E+01_JPRB, 0.10393E+01_JPRB, 0.10339E+01_JPRB, 0.10229E+01_JPRB, 0.10593E+01_JPRB, 0.11121E+01_JPRB ,&
&  0.12453E+01_JPRB, 0.94304E+00_JPRB, 0.85043E+00_JPRB, 0.82198E+00_JPRB, 0.82102E+00_JPRB, 0.82482E+00_JPRB, 0.83813E+00_JPRB ,&
&  0.85695E+00_JPRB, 0.90660E+00_JPRB, 0.97324E+00_JPRB, 0.10504E+01_JPRB, 0.12699E+01_JPRB, 0.15763E+01_JPRB, 0.17813E+01_JPRB ,&
&  0.19144E+01_JPRB, 0.19889E+01_JPRB, 0.20402E+01_JPRB, 0.17652E+01_JPRB, 0.16004E+01_JPRB, 0.16329E+01_JPRB, 0.15633E+01_JPRB ,&
&  0.14099E+01_JPRB, 0.12899E+01_JPRB, 0.12329E+01_JPRB, 0.11614E+01_JPRB, 0.11227E+01_JPRB, 0.10961E+01_JPRB, 0.10715E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10129E+01_JPRB, 0.10073E+01_JPRB, 0.10387E+01_JPRB, 0.10879E+01_JPRB, 0.11553E+01_JPRB, 0.12622E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.12797E+01_JPRB, 0.12467E+01_JPRB, 0.11972E+01_JPRB, 0.12297E+01_JPRB, 0.12589E+01_JPRB, 0.12678E+01_JPRB ,&
&  0.12634E+01_JPRB, 0.13082E+01_JPRB, 0.13445E+01_JPRB, 0.13308E+01_JPRB, 0.12625E+01_JPRB, 0.14108E+01_JPRB, 0.13992E+01_JPRB ,&
&  0.14946E+01_JPRB, 0.15098E+01_JPRB, 0.17773E+01_JPRB, 0.14251E+01_JPRB, 0.15983E+01_JPRB, 0.16575E+01_JPRB, 0.15630E+01_JPRB ,&
&  0.14057E+01_JPRB, 0.13567E+01_JPRB, 0.13392E+01_JPRB, 0.14045E+01_JPRB, 0.17040E+01_JPRB, 0.20485E+01_JPRB, 0.27925E+01_JPRB ,&
&  0.28309E+01_JPRB, 0.31931E+01_JPRB, 0.30154E+01_JPRB, 0.29641E+01_JPRB, 0.27321E+01_JPRB, 0.27876E+01_JPRB, 0.30790E+01_JPRB ,&
&  0.35913E+01_JPRB, 0.36510E+01_JPRB, 0.34762E+01_JPRB, 0.31152E+01_JPRB, 0.26275E+01_JPRB, 0.22812E+01_JPRB, 0.20998E+01_JPRB ,&
&  0.19342E+01_JPRB, 0.17868E+01_JPRB, 0.16685E+01_JPRB, 0.16196E+01_JPRB, 0.15595E+01_JPRB, 0.14846E+01_JPRB, 0.14134E+01_JPRB ,&
&  0.13681E+01_JPRB, 0.13283E+01_JPRB, 0.12958E+01_JPRB, 0.12584E+01_JPRB, 0.12080E+01_JPRB /)
ZBURD(:, 30,  2) = (/ &
&  0.10307E+01_JPRB, 0.10000E+01_JPRB, 0.98786E+00_JPRB, 0.96514E+00_JPRB, 0.94527E+00_JPRB, 0.94649E+00_JPRB, 0.94060E+00_JPRB ,&
&  0.89020E+00_JPRB, 0.84551E+00_JPRB, 0.83518E+00_JPRB, 0.83890E+00_JPRB, 0.83436E+00_JPRB, 0.82837E+00_JPRB, 0.84027E+00_JPRB ,&
&  0.87953E+00_JPRB, 0.94653E+00_JPRB, 0.10327E+01_JPRB, 0.12034E+01_JPRB, 0.14400E+01_JPRB, 0.17268E+01_JPRB, 0.19964E+01_JPRB ,&
&  0.19899E+01_JPRB, 0.14638E+01_JPRB, 0.17342E+01_JPRB, 0.17223E+01_JPRB, 0.15073E+01_JPRB, 0.14855E+01_JPRB, 0.14472E+01_JPRB ,&
&  0.12749E+01_JPRB, 0.12264E+01_JPRB, 0.12131E+01_JPRB, 0.11764E+01_JPRB, 0.10901E+01_JPRB, 0.10664E+01_JPRB, 0.10326E+01_JPRB ,&
&  0.10134E+01_JPRB, 0.10024E+01_JPRB, 0.10173E+01_JPRB, 0.10803E+01_JPRB, 0.11614E+01_JPRB, 0.12568E+01_JPRB, 0.13369E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.13186E+01_JPRB, 0.13150E+01_JPRB, 0.12655E+01_JPRB, 0.12784E+01_JPRB, 0.13010E+01_JPRB, 0.12994E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.12758E+01_JPRB, 0.13176E+01_JPRB, 0.13812E+01_JPRB, 0.14241E+01_JPRB, 0.13160E+01_JPRB, 0.14108E+01_JPRB ,&
&  0.14654E+01_JPRB, 0.15882E+01_JPRB, 0.16843E+01_JPRB, 0.18727E+01_JPRB, 0.13711E+01_JPRB, 0.14928E+01_JPRB, 0.13365E+01_JPRB ,&
&  0.14117E+01_JPRB, 0.14298E+01_JPRB, 0.15594E+01_JPRB, 0.17506E+01_JPRB, 0.21692E+01_JPRB, 0.25282E+01_JPRB, 0.32190E+01_JPRB ,&
&  0.29529E+01_JPRB, 0.28683E+01_JPRB, 0.28062E+01_JPRB, 0.30406E+01_JPRB, 0.29599E+01_JPRB, 0.31321E+01_JPRB, 0.30615E+01_JPRB ,&
&  0.31966E+01_JPRB, 0.29553E+01_JPRB, 0.26990E+01_JPRB, 0.24953E+01_JPRB, 0.21459E+01_JPRB, 0.17627E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.14604E+01_JPRB, 0.14299E+01_JPRB, 0.14074E+01_JPRB, 0.13880E+01_JPRB, 0.13531E+01_JPRB, 0.13151E+01_JPRB, 0.12802E+01_JPRB ,&
&  0.12485E+01_JPRB, 0.12090E+01_JPRB, 0.11721E+01_JPRB, 0.11341E+01_JPRB, 0.10860E+01_JPRB /)
ZBURD(:, 31,  2) = (/ &
&  0.95962E+00_JPRB, 0.93527E+00_JPRB, 0.92323E+00_JPRB, 0.90565E+00_JPRB, 0.89512E+00_JPRB, 0.89094E+00_JPRB, 0.88406E+00_JPRB ,&
&  0.86306E+00_JPRB, 0.85388E+00_JPRB, 0.85418E+00_JPRB, 0.85011E+00_JPRB, 0.83388E+00_JPRB, 0.83349E+00_JPRB, 0.86526E+00_JPRB ,&
&  0.93495E+00_JPRB, 0.10453E+01_JPRB, 0.11819E+01_JPRB, 0.13409E+01_JPRB, 0.15723E+01_JPRB, 0.17458E+01_JPRB, 0.18122E+01_JPRB ,&
&  0.19425E+01_JPRB, 0.19904E+01_JPRB, 0.17687E+01_JPRB, 0.12816E+01_JPRB, 0.11532E+01_JPRB, 0.13540E+01_JPRB, 0.12496E+01_JPRB ,&
&  0.11637E+01_JPRB, 0.11356E+01_JPRB, 0.11194E+01_JPRB, 0.10991E+01_JPRB, 0.10643E+01_JPRB, 0.10390E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10289E+01_JPRB, 0.10815E+01_JPRB, 0.11879E+01_JPRB, 0.12950E+01_JPRB, 0.13630E+01_JPRB, 0.13915E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.14148E+01_JPRB, 0.14173E+01_JPRB, 0.13902E+01_JPRB, 0.13581E+01_JPRB, 0.13750E+01_JPRB, 0.13811E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.13724E+01_JPRB, 0.13623E+01_JPRB, 0.13954E+01_JPRB, 0.14028E+01_JPRB, 0.12763E+01_JPRB, 0.13178E+01_JPRB ,&
&  0.14871E+01_JPRB, 0.17188E+01_JPRB, 0.16787E+01_JPRB, 0.19866E+01_JPRB, 0.12264E+01_JPRB, 0.13775E+01_JPRB, 0.15039E+01_JPRB ,&
&  0.14814E+01_JPRB, 0.16630E+01_JPRB, 0.18559E+01_JPRB, 0.21303E+01_JPRB, 0.25346E+01_JPRB, 0.27394E+01_JPRB, 0.30501E+01_JPRB ,&
&  0.27087E+01_JPRB, 0.25225E+01_JPRB, 0.25809E+01_JPRB, 0.27954E+01_JPRB, 0.29208E+01_JPRB, 0.29426E+01_JPRB, 0.26303E+01_JPRB ,&
&  0.22840E+01_JPRB, 0.23977E+01_JPRB, 0.21449E+01_JPRB, 0.20530E+01_JPRB, 0.16901E+01_JPRB, 0.13541E+01_JPRB, 0.13183E+01_JPRB ,&
&  0.12733E+01_JPRB, 0.12359E+01_JPRB, 0.12296E+01_JPRB, 0.12104E+01_JPRB, 0.11889E+01_JPRB, 0.11701E+01_JPRB, 0.11440E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.10929E+01_JPRB, 0.10638E+01_JPRB, 0.10333E+01_JPRB, 0.99788E+00_JPRB /)
ZBURD(:, 32,  2) = (/ &
&  0.90695E+00_JPRB, 0.89071E+00_JPRB, 0.88752E+00_JPRB, 0.86367E+00_JPRB, 0.84448E+00_JPRB, 0.84498E+00_JPRB, 0.85385E+00_JPRB ,&
&  0.84989E+00_JPRB, 0.84612E+00_JPRB, 0.83668E+00_JPRB, 0.83051E+00_JPRB, 0.83979E+00_JPRB, 0.87798E+00_JPRB, 0.94458E+00_JPRB ,&
&  0.10308E+01_JPRB, 0.11553E+01_JPRB, 0.13159E+01_JPRB, 0.14680E+01_JPRB, 0.16605E+01_JPRB, 0.17482E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.17869E+01_JPRB, 0.18570E+01_JPRB, 0.18280E+01_JPRB, 0.17732E+01_JPRB, 0.14451E+01_JPRB, 0.15067E+01_JPRB, 0.15330E+01_JPRB ,&
&  0.14666E+01_JPRB, 0.12431E+01_JPRB, 0.11151E+01_JPRB, 0.10818E+01_JPRB, 0.10436E+01_JPRB, 0.10308E+01_JPRB, 0.10421E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.11328E+01_JPRB, 0.12173E+01_JPRB, 0.13292E+01_JPRB, 0.14114E+01_JPRB, 0.14454E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.14966E+01_JPRB, 0.15132E+01_JPRB, 0.15258E+01_JPRB, 0.15050E+01_JPRB, 0.14991E+01_JPRB, 0.15084E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.14859E+01_JPRB, 0.14973E+01_JPRB, 0.15479E+01_JPRB, 0.15001E+01_JPRB, 0.14378E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.16361E+01_JPRB, 0.19140E+01_JPRB, 0.15299E+01_JPRB, 0.16762E+01_JPRB, 0.19266E+01_JPRB, 0.16511E+01_JPRB, 0.15381E+01_JPRB ,&
&  0.16629E+01_JPRB, 0.18203E+01_JPRB, 0.19993E+01_JPRB, 0.22557E+01_JPRB, 0.26378E+01_JPRB, 0.31002E+01_JPRB, 0.26501E+01_JPRB ,&
&  0.23850E+01_JPRB, 0.22428E+01_JPRB, 0.23387E+01_JPRB, 0.24673E+01_JPRB, 0.25288E+01_JPRB, 0.25083E+01_JPRB, 0.22414E+01_JPRB ,&
&  0.19851E+01_JPRB, 0.20562E+01_JPRB, 0.19122E+01_JPRB, 0.17987E+01_JPRB, 0.17705E+01_JPRB, 0.12681E+01_JPRB, 0.12129E+01_JPRB ,&
&  0.11365E+01_JPRB, 0.11204E+01_JPRB, 0.11153E+01_JPRB, 0.11013E+01_JPRB, 0.10854E+01_JPRB, 0.10633E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10212E+01_JPRB, 0.99836E+00_JPRB, 0.98007E+00_JPRB, 0.96449E+00_JPRB, 0.93303E+00_JPRB /)
ZBURD(:, 33,  2) = (/ &
&  0.87438E+00_JPRB, 0.86648E+00_JPRB, 0.86072E+00_JPRB, 0.84381E+00_JPRB, 0.82518E+00_JPRB, 0.82627E+00_JPRB, 0.82357E+00_JPRB ,&
&  0.80737E+00_JPRB, 0.79125E+00_JPRB, 0.77219E+00_JPRB, 0.78644E+00_JPRB, 0.81572E+00_JPRB, 0.87337E+00_JPRB, 0.94176E+00_JPRB ,&
&  0.10276E+01_JPRB, 0.11508E+01_JPRB, 0.13174E+01_JPRB, 0.14514E+01_JPRB, 0.16368E+01_JPRB, 0.17268E+01_JPRB, 0.17688E+01_JPRB ,&
&  0.18354E+01_JPRB, 0.19087E+01_JPRB, 0.19500E+01_JPRB, 0.19579E+01_JPRB, 0.19135E+01_JPRB, 0.18956E+01_JPRB, 0.19349E+01_JPRB ,&
&  0.16117E+01_JPRB, 0.12324E+01_JPRB, 0.11433E+01_JPRB, 0.10977E+01_JPRB, 0.10843E+01_JPRB, 0.10863E+01_JPRB, 0.11240E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.12528E+01_JPRB, 0.13417E+01_JPRB, 0.14176E+01_JPRB, 0.14897E+01_JPRB, 0.15409E+01_JPRB, 0.15686E+01_JPRB ,&
&  0.16149E+01_JPRB, 0.16578E+01_JPRB, 0.17175E+01_JPRB, 0.16227E+01_JPRB, 0.17234E+01_JPRB, 0.17538E+01_JPRB, 0.17824E+01_JPRB ,&
&  0.17498E+01_JPRB, 0.16476E+01_JPRB, 0.17044E+01_JPRB, 0.17312E+01_JPRB, 0.17556E+01_JPRB, 0.17446E+01_JPRB, 0.17361E+01_JPRB ,&
&  0.18847E+01_JPRB, 0.18840E+01_JPRB, 0.14481E+01_JPRB, 0.13873E+01_JPRB, 0.14657E+01_JPRB, 0.13210E+01_JPRB, 0.16009E+01_JPRB ,&
&  0.16853E+01_JPRB, 0.18073E+01_JPRB, 0.19427E+01_JPRB, 0.21734E+01_JPRB, 0.24326E+01_JPRB, 0.26330E+01_JPRB, 0.23541E+01_JPRB ,&
&  0.19838E+01_JPRB, 0.18913E+01_JPRB, 0.19263E+01_JPRB, 0.19593E+01_JPRB, 0.20032E+01_JPRB, 0.20260E+01_JPRB, 0.19468E+01_JPRB ,&
&  0.18161E+01_JPRB, 0.16579E+01_JPRB, 0.15064E+01_JPRB, 0.12766E+01_JPRB, 0.11485E+01_JPRB, 0.11216E+01_JPRB, 0.10499E+01_JPRB ,&
&  0.10324E+01_JPRB, 0.10469E+01_JPRB, 0.10462E+01_JPRB, 0.10334E+01_JPRB, 0.10223E+01_JPRB, 0.99555E+00_JPRB, 0.97788E+00_JPRB ,&
&  0.96790E+00_JPRB, 0.94943E+00_JPRB, 0.93176E+00_JPRB, 0.91967E+00_JPRB, 0.89322E+00_JPRB /)
ZBURD(:, 34,  2) = (/ &
&  0.82994E+00_JPRB, 0.81338E+00_JPRB, 0.80451E+00_JPRB, 0.79798E+00_JPRB, 0.78042E+00_JPRB, 0.76798E+00_JPRB, 0.76150E+00_JPRB ,&
&  0.72905E+00_JPRB, 0.72757E+00_JPRB, 0.71464E+00_JPRB, 0.72507E+00_JPRB, 0.75798E+00_JPRB, 0.79003E+00_JPRB, 0.83836E+00_JPRB ,&
&  0.88156E+00_JPRB, 0.94034E+00_JPRB, 0.10343E+01_JPRB, 0.12375E+01_JPRB, 0.14676E+01_JPRB, 0.16358E+01_JPRB, 0.18289E+01_JPRB ,&
&  0.19021E+01_JPRB, 0.19650E+01_JPRB, 0.20913E+01_JPRB, 0.22648E+01_JPRB, 0.23532E+01_JPRB, 0.24031E+01_JPRB, 0.23262E+01_JPRB ,&
&  0.17165E+01_JPRB, 0.11742E+01_JPRB, 0.11816E+01_JPRB, 0.11027E+01_JPRB, 0.11219E+01_JPRB, 0.11508E+01_JPRB, 0.11864E+01_JPRB ,&
&  0.12493E+01_JPRB, 0.13084E+01_JPRB, 0.13704E+01_JPRB, 0.14271E+01_JPRB, 0.15099E+01_JPRB, 0.16007E+01_JPRB, 0.16322E+01_JPRB ,&
&  0.16536E+01_JPRB, 0.16819E+01_JPRB, 0.17083E+01_JPRB, 0.16644E+01_JPRB, 0.17168E+01_JPRB, 0.18288E+01_JPRB, 0.19192E+01_JPRB ,&
&  0.20037E+01_JPRB, 0.20169E+01_JPRB, 0.17592E+01_JPRB, 0.17603E+01_JPRB, 0.19329E+01_JPRB, 0.18214E+01_JPRB, 0.18829E+01_JPRB ,&
&  0.21190E+01_JPRB, 0.21071E+01_JPRB, 0.13716E+01_JPRB, 0.12889E+01_JPRB, 0.13691E+01_JPRB, 0.14357E+01_JPRB, 0.15485E+01_JPRB ,&
&  0.16013E+01_JPRB, 0.17148E+01_JPRB, 0.18539E+01_JPRB, 0.19600E+01_JPRB, 0.20238E+01_JPRB, 0.19488E+01_JPRB, 0.18645E+01_JPRB ,&
&  0.16206E+01_JPRB, 0.15476E+01_JPRB, 0.15788E+01_JPRB, 0.16198E+01_JPRB, 0.16628E+01_JPRB, 0.17231E+01_JPRB, 0.16011E+01_JPRB ,&
&  0.15176E+01_JPRB, 0.14312E+01_JPRB, 0.14960E+01_JPRB, 0.12893E+01_JPRB, 0.10303E+01_JPRB, 0.10014E+01_JPRB, 0.95579E+00_JPRB ,&
&  0.99397E+00_JPRB, 0.10137E+01_JPRB, 0.10189E+01_JPRB, 0.10182E+01_JPRB, 0.10115E+01_JPRB, 0.99487E+00_JPRB, 0.97612E+00_JPRB ,&
&  0.95400E+00_JPRB, 0.92882E+00_JPRB, 0.90158E+00_JPRB, 0.87551E+00_JPRB, 0.85231E+00_JPRB /)
ZBURD(:, 35,  2) = (/ &
&  0.73895E+00_JPRB, 0.72199E+00_JPRB, 0.72031E+00_JPRB, 0.68988E+00_JPRB, 0.70741E+00_JPRB, 0.68705E+00_JPRB, 0.71947E+00_JPRB ,&
&  0.69534E+00_JPRB, 0.69187E+00_JPRB, 0.69308E+00_JPRB, 0.70730E+00_JPRB, 0.72215E+00_JPRB, 0.74154E+00_JPRB, 0.75681E+00_JPRB ,&
&  0.77844E+00_JPRB, 0.82380E+00_JPRB, 0.88977E+00_JPRB, 0.96784E+00_JPRB, 0.10706E+01_JPRB, 0.11709E+01_JPRB, 0.12568E+01_JPRB ,&
&  0.13722E+01_JPRB, 0.14697E+01_JPRB, 0.15223E+01_JPRB, 0.16703E+01_JPRB, 0.20897E+01_JPRB, 0.26055E+01_JPRB, 0.26485E+01_JPRB ,&
&  0.16866E+01_JPRB, 0.12050E+01_JPRB, 0.10895E+01_JPRB, 0.10085E+01_JPRB, 0.10469E+01_JPRB, 0.11476E+01_JPRB, 0.11755E+01_JPRB ,&
&  0.11995E+01_JPRB, 0.12312E+01_JPRB, 0.12793E+01_JPRB, 0.13336E+01_JPRB, 0.14089E+01_JPRB, 0.15004E+01_JPRB, 0.15291E+01_JPRB ,&
&  0.15092E+01_JPRB, 0.15304E+01_JPRB, 0.15297E+01_JPRB, 0.16202E+01_JPRB, 0.17864E+01_JPRB, 0.19973E+01_JPRB, 0.21195E+01_JPRB ,&
&  0.21925E+01_JPRB, 0.21555E+01_JPRB, 0.18830E+01_JPRB, 0.18633E+01_JPRB, 0.19215E+01_JPRB, 0.19203E+01_JPRB, 0.18923E+01_JPRB ,&
&  0.20342E+01_JPRB, 0.19348E+01_JPRB, 0.14780E+01_JPRB, 0.14422E+01_JPRB, 0.14424E+01_JPRB, 0.14692E+01_JPRB, 0.14235E+01_JPRB ,&
&  0.15057E+01_JPRB, 0.15933E+01_JPRB, 0.16604E+01_JPRB, 0.16680E+01_JPRB, 0.16283E+01_JPRB, 0.15604E+01_JPRB, 0.15680E+01_JPRB ,&
&  0.14714E+01_JPRB, 0.14707E+01_JPRB, 0.14756E+01_JPRB, 0.14632E+01_JPRB, 0.16237E+01_JPRB, 0.14485E+01_JPRB, 0.14557E+01_JPRB ,&
&  0.12606E+01_JPRB, 0.11949E+01_JPRB, 0.10100E+01_JPRB, 0.12130E+01_JPRB, 0.98626E+00_JPRB, 0.87315E+00_JPRB, 0.88908E+00_JPRB ,&
&  0.90791E+00_JPRB, 0.93715E+00_JPRB, 0.93153E+00_JPRB, 0.93585E+00_JPRB, 0.93875E+00_JPRB, 0.92488E+00_JPRB, 0.91506E+00_JPRB ,&
&  0.89816E+00_JPRB, 0.87645E+00_JPRB, 0.83910E+00_JPRB, 0.80601E+00_JPRB, 0.77314E+00_JPRB /)
ZBURD(:, 36,  2) = (/ &
&  0.71509E+00_JPRB, 0.71552E+00_JPRB, 0.69599E+00_JPRB, 0.66637E+00_JPRB, 0.64607E+00_JPRB, 0.63378E+00_JPRB, 0.64194E+00_JPRB ,&
&  0.65164E+00_JPRB, 0.67353E+00_JPRB, 0.69207E+00_JPRB, 0.71173E+00_JPRB, 0.72055E+00_JPRB, 0.72648E+00_JPRB, 0.74145E+00_JPRB ,&
&  0.76181E+00_JPRB, 0.78227E+00_JPRB, 0.80992E+00_JPRB, 0.84892E+00_JPRB, 0.88648E+00_JPRB, 0.93310E+00_JPRB, 0.95296E+00_JPRB ,&
&  0.95228E+00_JPRB, 0.97921E+00_JPRB, 0.99940E+00_JPRB, 0.10441E+01_JPRB, 0.12564E+01_JPRB, 0.14804E+01_JPRB, 0.17405E+01_JPRB ,&
&  0.13240E+01_JPRB, 0.12806E+01_JPRB, 0.11275E+01_JPRB, 0.10307E+01_JPRB, 0.10465E+01_JPRB, 0.10251E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10451E+01_JPRB, 0.10426E+01_JPRB, 0.10443E+01_JPRB, 0.10548E+01_JPRB, 0.10713E+01_JPRB, 0.11197E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10495E+01_JPRB, 0.10857E+01_JPRB, 0.11364E+01_JPRB, 0.12890E+01_JPRB, 0.15041E+01_JPRB, 0.16940E+01_JPRB, 0.17048E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.17948E+01_JPRB, 0.17537E+01_JPRB, 0.17361E+01_JPRB, 0.16984E+01_JPRB, 0.16718E+01_JPRB, 0.17080E+01_JPRB ,&
&  0.16156E+01_JPRB, 0.15660E+01_JPRB, 0.16108E+01_JPRB, 0.14919E+01_JPRB, 0.14099E+01_JPRB, 0.13415E+01_JPRB, 0.13217E+01_JPRB ,&
&  0.13320E+01_JPRB, 0.13820E+01_JPRB, 0.14148E+01_JPRB, 0.13753E+01_JPRB, 0.13433E+01_JPRB, 0.13008E+01_JPRB, 0.13273E+01_JPRB ,&
&  0.13403E+01_JPRB, 0.12714E+01_JPRB, 0.12342E+01_JPRB, 0.12636E+01_JPRB, 0.12949E+01_JPRB, 0.19672E+01_JPRB, 0.17938E+01_JPRB ,&
&  0.10942E+01_JPRB, 0.88439E+00_JPRB, 0.68452E+00_JPRB, 0.99224E+00_JPRB, 0.80302E+00_JPRB, 0.77876E+00_JPRB, 0.79992E+00_JPRB ,&
&  0.82040E+00_JPRB, 0.83585E+00_JPRB, 0.84509E+00_JPRB, 0.83773E+00_JPRB, 0.83388E+00_JPRB, 0.83168E+00_JPRB, 0.81446E+00_JPRB ,&
&  0.80055E+00_JPRB, 0.78162E+00_JPRB, 0.77548E+00_JPRB, 0.74064E+00_JPRB, 0.72244E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART3',1,ZHOOK_HANDLE)
END SUBROUTINE PART3



SUBROUTINE PART4

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART4',0,ZHOOK_HANDLE)


ZBURD(:, 37,  2) = (/ &
&  0.68464E+00_JPRB, 0.68158E+00_JPRB, 0.65387E+00_JPRB, 0.61732E+00_JPRB, 0.60022E+00_JPRB, 0.60665E+00_JPRB, 0.62229E+00_JPRB ,&
&  0.63728E+00_JPRB, 0.65765E+00_JPRB, 0.68170E+00_JPRB, 0.70513E+00_JPRB, 0.72609E+00_JPRB, 0.74850E+00_JPRB, 0.77526E+00_JPRB ,&
&  0.80012E+00_JPRB, 0.83081E+00_JPRB, 0.86234E+00_JPRB, 0.89493E+00_JPRB, 0.92360E+00_JPRB, 0.97326E+00_JPRB, 0.10079E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.11148E+01_JPRB, 0.11806E+01_JPRB, 0.12448E+01_JPRB, 0.13269E+01_JPRB, 0.13900E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.13828E+01_JPRB, 0.12714E+01_JPRB, 0.11337E+01_JPRB, 0.10354E+01_JPRB, 0.97204E+00_JPRB, 0.92339E+00_JPRB, 0.87701E+00_JPRB ,&
&  0.85279E+00_JPRB, 0.81776E+00_JPRB, 0.77883E+00_JPRB, 0.74994E+00_JPRB, 0.73764E+00_JPRB, 0.78093E+00_JPRB, 0.76228E+00_JPRB ,&
&  0.81260E+00_JPRB, 0.91106E+00_JPRB, 0.99161E+00_JPRB, 0.11211E+01_JPRB, 0.12346E+01_JPRB, 0.13037E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.12635E+01_JPRB, 0.13434E+01_JPRB, 0.13074E+01_JPRB, 0.13085E+01_JPRB, 0.12934E+01_JPRB, 0.12585E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.11786E+01_JPRB, 0.13790E+01_JPRB, 0.15577E+01_JPRB, 0.13845E+01_JPRB, 0.12684E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.11956E+01_JPRB, 0.12094E+01_JPRB, 0.12242E+01_JPRB, 0.11720E+01_JPRB, 0.11321E+01_JPRB, 0.10974E+01_JPRB, 0.11177E+01_JPRB ,&
&  0.11458E+01_JPRB, 0.10580E+01_JPRB, 0.10546E+01_JPRB, 0.11393E+01_JPRB, 0.12915E+01_JPRB, 0.13643E+01_JPRB, 0.16308E+01_JPRB ,&
&  0.99580E+00_JPRB, 0.61177E+00_JPRB, 0.65932E+00_JPRB, 0.75185E+00_JPRB, 0.68237E+00_JPRB, 0.66587E+00_JPRB, 0.71190E+00_JPRB ,&
&  0.75474E+00_JPRB, 0.77957E+00_JPRB, 0.76788E+00_JPRB, 0.78908E+00_JPRB, 0.82703E+00_JPRB, 0.81627E+00_JPRB, 0.76153E+00_JPRB ,&
&  0.76823E+00_JPRB, 0.73110E+00_JPRB, 0.71195E+00_JPRB, 0.70327E+00_JPRB, 0.69599E+00_JPRB /)
ZBURD(:, 38,  2) = (/ &
&  0.64873E+00_JPRB, 0.63653E+00_JPRB, 0.60843E+00_JPRB, 0.57228E+00_JPRB, 0.55781E+00_JPRB, 0.57392E+00_JPRB, 0.59872E+00_JPRB ,&
&  0.61850E+00_JPRB, 0.64281E+00_JPRB, 0.66832E+00_JPRB, 0.69651E+00_JPRB, 0.72712E+00_JPRB, 0.75563E+00_JPRB, 0.77767E+00_JPRB ,&
&  0.79339E+00_JPRB, 0.81481E+00_JPRB, 0.83416E+00_JPRB, 0.85099E+00_JPRB, 0.85839E+00_JPRB, 0.86244E+00_JPRB, 0.86583E+00_JPRB ,&
&  0.90292E+00_JPRB, 0.94058E+00_JPRB, 0.98949E+00_JPRB, 0.10399E+01_JPRB, 0.10746E+01_JPRB, 0.10852E+01_JPRB, 0.89727E+00_JPRB ,&
&  0.12294E+01_JPRB, 0.12055E+01_JPRB, 0.11054E+01_JPRB, 0.10086E+01_JPRB, 0.92671E+00_JPRB, 0.86580E+00_JPRB, 0.81234E+00_JPRB ,&
&  0.77239E+00_JPRB, 0.79273E+00_JPRB, 0.76123E+00_JPRB, 0.76261E+00_JPRB, 0.70302E+00_JPRB, 0.73634E+00_JPRB, 0.76810E+00_JPRB ,&
&  0.84733E+00_JPRB, 0.93557E+00_JPRB, 0.10133E+01_JPRB, 0.10771E+01_JPRB, 0.11078E+01_JPRB, 0.11287E+01_JPRB, 0.10814E+01_JPRB ,&
&  0.10779E+01_JPRB, 0.10715E+01_JPRB, 0.98355E+00_JPRB, 0.96284E+00_JPRB, 0.97447E+00_JPRB, 0.87599E+00_JPRB, 0.81035E+00_JPRB ,&
&  0.73936E+00_JPRB, 0.91225E+00_JPRB, 0.13159E+01_JPRB, 0.15599E+01_JPRB, 0.13609E+01_JPRB, 0.12925E+01_JPRB, 0.12089E+01_JPRB ,&
&  0.11137E+01_JPRB, 0.10994E+01_JPRB, 0.10869E+01_JPRB, 0.10464E+01_JPRB, 0.96761E+00_JPRB, 0.92032E+00_JPRB, 0.93597E+00_JPRB ,&
&  0.96282E+00_JPRB, 0.89623E+00_JPRB, 0.86410E+00_JPRB, 0.95709E+00_JPRB, 0.10181E+01_JPRB, 0.94561E+00_JPRB, 0.15176E+01_JPRB ,&
&  0.98318E+00_JPRB, 0.71638E+00_JPRB, 0.95840E+00_JPRB, 0.48941E+00_JPRB, 0.57547E+00_JPRB, 0.55471E+00_JPRB, 0.57690E+00_JPRB ,&
&  0.49699E+00_JPRB, 0.57973E+00_JPRB, 0.94500E+00_JPRB, 0.12642E+01_JPRB, 0.14496E+01_JPRB, 0.14416E+01_JPRB, 0.83994E+00_JPRB ,&
&  0.74329E+00_JPRB, 0.71765E+00_JPRB, 0.69894E+00_JPRB, 0.67607E+00_JPRB, 0.65218E+00_JPRB /)
ZBURD(:, 39,  2) = (/ &
&  0.61073E+00_JPRB, 0.57539E+00_JPRB, 0.55961E+00_JPRB, 0.54943E+00_JPRB, 0.53827E+00_JPRB, 0.55653E+00_JPRB, 0.57168E+00_JPRB ,&
&  0.59441E+00_JPRB, 0.62303E+00_JPRB, 0.65792E+00_JPRB, 0.68860E+00_JPRB, 0.73143E+00_JPRB, 0.77292E+00_JPRB, 0.80612E+00_JPRB ,&
&  0.83319E+00_JPRB, 0.85483E+00_JPRB, 0.87210E+00_JPRB, 0.88073E+00_JPRB, 0.88497E+00_JPRB, 0.88761E+00_JPRB, 0.88779E+00_JPRB ,&
&  0.88702E+00_JPRB, 0.90941E+00_JPRB, 0.95368E+00_JPRB, 0.10039E+01_JPRB, 0.10341E+01_JPRB, 0.10774E+01_JPRB, 0.69555E+00_JPRB ,&
&  0.10585E+01_JPRB, 0.10833E+01_JPRB, 0.10531E+01_JPRB, 0.97640E+00_JPRB, 0.90190E+00_JPRB, 0.84983E+00_JPRB, 0.81194E+00_JPRB ,&
&  0.79195E+00_JPRB, 0.80790E+00_JPRB, 0.78294E+00_JPRB, 0.83697E+00_JPRB, 0.86077E+00_JPRB, 0.83032E+00_JPRB, 0.89713E+00_JPRB ,&
&  0.95003E+00_JPRB, 0.10227E+01_JPRB, 0.10707E+01_JPRB, 0.11079E+01_JPRB, 0.11314E+01_JPRB, 0.11575E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.11580E+01_JPRB, 0.11196E+01_JPRB, 0.10620E+01_JPRB, 0.10190E+01_JPRB, 0.99975E+00_JPRB, 0.86678E+00_JPRB, 0.80026E+00_JPRB ,&
&  0.63896E+00_JPRB, 0.68192E+00_JPRB, 0.11091E+01_JPRB, 0.14841E+01_JPRB, 0.13739E+01_JPRB, 0.13146E+01_JPRB, 0.11801E+01_JPRB ,&
&  0.10587E+01_JPRB, 0.99693E+00_JPRB, 0.96366E+00_JPRB, 0.92101E+00_JPRB, 0.85821E+00_JPRB, 0.81648E+00_JPRB, 0.80812E+00_JPRB ,&
&  0.80169E+00_JPRB, 0.73310E+00_JPRB, 0.70937E+00_JPRB, 0.75489E+00_JPRB, 0.77729E+00_JPRB, 0.88720E+00_JPRB, 0.91418E+00_JPRB ,&
&  0.10719E+01_JPRB, 0.96579E+00_JPRB, 0.76106E+00_JPRB, 0.70921E+00_JPRB, 0.54668E+00_JPRB, 0.50356E+00_JPRB, 0.49579E+00_JPRB ,&
&  0.51756E+00_JPRB, 0.47042E+00_JPRB, 0.96894E+00_JPRB, 0.37628E+01_JPRB, 0.37124E+01_JPRB, 0.47030E+01_JPRB, 0.24648E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.74205E+00_JPRB, 0.65091E+00_JPRB, 0.64279E+00_JPRB, 0.62447E+00_JPRB /)
ZBURD(:, 40,  2) = (/ &
&  0.54666E+00_JPRB, 0.52573E+00_JPRB, 0.50420E+00_JPRB, 0.51254E+00_JPRB, 0.51910E+00_JPRB, 0.52497E+00_JPRB, 0.55190E+00_JPRB ,&
&  0.60875E+00_JPRB, 0.62346E+00_JPRB, 0.65137E+00_JPRB, 0.69639E+00_JPRB, 0.76175E+00_JPRB, 0.81365E+00_JPRB, 0.86182E+00_JPRB ,&
&  0.91127E+00_JPRB, 0.94310E+00_JPRB, 0.94391E+00_JPRB, 0.93531E+00_JPRB, 0.93851E+00_JPRB, 0.94797E+00_JPRB, 0.96578E+00_JPRB ,&
&  0.97130E+00_JPRB, 0.99533E+00_JPRB, 0.10408E+01_JPRB, 0.10931E+01_JPRB, 0.11383E+01_JPRB, 0.11307E+01_JPRB, 0.71826E+00_JPRB ,&
&  0.87906E+00_JPRB, 0.93071E+00_JPRB, 0.95412E+00_JPRB, 0.92922E+00_JPRB, 0.86547E+00_JPRB, 0.80430E+00_JPRB, 0.80217E+00_JPRB ,&
&  0.84517E+00_JPRB, 0.89229E+00_JPRB, 0.90337E+00_JPRB, 0.90039E+00_JPRB, 0.10120E+01_JPRB, 0.98749E+00_JPRB, 0.10887E+01_JPRB ,&
&  0.11370E+01_JPRB, 0.11813E+01_JPRB, 0.12297E+01_JPRB, 0.12581E+01_JPRB, 0.12973E+01_JPRB, 0.13457E+01_JPRB, 0.13889E+01_JPRB ,&
&  0.13995E+01_JPRB, 0.13783E+01_JPRB, 0.13883E+01_JPRB, 0.14295E+01_JPRB, 0.12985E+01_JPRB, 0.13731E+01_JPRB, 0.11344E+01_JPRB ,&
&  0.81562E+00_JPRB, 0.73960E+00_JPRB, 0.99932E+00_JPRB, 0.13358E+01_JPRB, 0.13230E+01_JPRB, 0.13003E+01_JPRB, 0.11926E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.94109E+00_JPRB, 0.89949E+00_JPRB, 0.86193E+00_JPRB, 0.81504E+00_JPRB, 0.76780E+00_JPRB, 0.73573E+00_JPRB ,&
&  0.69408E+00_JPRB, 0.66077E+00_JPRB, 0.68469E+00_JPRB, 0.68572E+00_JPRB, 0.71150E+00_JPRB, 0.86640E+00_JPRB, 0.12193E+01_JPRB ,&
&  0.19013E+01_JPRB, 0.10222E+01_JPRB, 0.80234E+00_JPRB, 0.69781E+00_JPRB, 0.59266E+00_JPRB, 0.53628E+00_JPRB, 0.50510E+00_JPRB ,&
&  0.50569E+00_JPRB, 0.58220E+00_JPRB, 0.71319E+00_JPRB, 0.14228E+01_JPRB, 0.25677E+01_JPRB, 0.26257E+01_JPRB, 0.17722E+01_JPRB ,&
&  0.10813E+01_JPRB, 0.75161E+00_JPRB, 0.60608E+00_JPRB, 0.57421E+00_JPRB, 0.55142E+00_JPRB /)
ZBURD(:, 41,  2) = (/ &
&  0.50915E+00_JPRB, 0.49772E+00_JPRB, 0.49695E+00_JPRB, 0.49437E+00_JPRB, 0.52868E+00_JPRB, 0.55150E+00_JPRB, 0.57294E+00_JPRB ,&
&  0.60655E+00_JPRB, 0.62832E+00_JPRB, 0.66150E+00_JPRB, 0.72273E+00_JPRB, 0.78917E+00_JPRB, 0.84375E+00_JPRB, 0.87130E+00_JPRB ,&
&  0.88852E+00_JPRB, 0.88996E+00_JPRB, 0.88357E+00_JPRB, 0.87373E+00_JPRB, 0.89350E+00_JPRB, 0.90500E+00_JPRB, 0.94262E+00_JPRB ,&
&  0.99556E+00_JPRB, 0.10511E+01_JPRB, 0.11183E+01_JPRB, 0.11800E+01_JPRB, 0.12447E+01_JPRB, 0.12901E+01_JPRB, 0.12881E+01_JPRB ,&
&  0.60018E+00_JPRB, 0.81251E+00_JPRB, 0.84867E+00_JPRB, 0.83747E+00_JPRB, 0.78616E+00_JPRB, 0.77109E+00_JPRB, 0.84353E+00_JPRB ,&
&  0.94324E+00_JPRB, 0.97574E+00_JPRB, 0.10745E+01_JPRB, 0.99678E+00_JPRB, 0.11226E+01_JPRB, 0.11548E+01_JPRB, 0.12375E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.13329E+01_JPRB, 0.13906E+01_JPRB, 0.14502E+01_JPRB, 0.15185E+01_JPRB, 0.15919E+01_JPRB, 0.16868E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.18747E+01_JPRB, 0.20226E+01_JPRB, 0.19622E+01_JPRB, 0.18023E+01_JPRB, 0.18806E+01_JPRB, 0.17766E+01_JPRB ,&
&  0.15189E+01_JPRB, 0.11518E+01_JPRB, 0.11631E+01_JPRB, 0.12835E+01_JPRB, 0.12827E+01_JPRB, 0.12678E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.10570E+01_JPRB, 0.97415E+00_JPRB, 0.90431E+00_JPRB, 0.85486E+00_JPRB, 0.79627E+00_JPRB, 0.75823E+00_JPRB, 0.73382E+00_JPRB ,&
&  0.71933E+00_JPRB, 0.73809E+00_JPRB, 0.75619E+00_JPRB, 0.72155E+00_JPRB, 0.69500E+00_JPRB, 0.73967E+00_JPRB, 0.84301E+00_JPRB ,&
&  0.97617E+00_JPRB, 0.86440E+00_JPRB, 0.77095E+00_JPRB, 0.76204E+00_JPRB, 0.72505E+00_JPRB, 0.68620E+00_JPRB, 0.66710E+00_JPRB ,&
&  0.66969E+00_JPRB, 0.65888E+00_JPRB, 0.75784E+00_JPRB, 0.11442E+01_JPRB, 0.15077E+01_JPRB, 0.17448E+01_JPRB, 0.14409E+01_JPRB ,&
&  0.10269E+01_JPRB, 0.76402E+00_JPRB, 0.61202E+00_JPRB, 0.57172E+00_JPRB, 0.55016E+00_JPRB /)
ZBURD(:, 42,  2) = (/ &
&  0.54731E+00_JPRB, 0.53825E+00_JPRB, 0.53919E+00_JPRB, 0.53640E+00_JPRB, 0.56861E+00_JPRB, 0.60460E+00_JPRB, 0.61468E+00_JPRB ,&
&  0.60984E+00_JPRB, 0.62829E+00_JPRB, 0.64620E+00_JPRB, 0.67920E+00_JPRB, 0.74629E+00_JPRB, 0.77951E+00_JPRB, 0.81852E+00_JPRB ,&
&  0.85720E+00_JPRB, 0.85882E+00_JPRB, 0.83461E+00_JPRB, 0.82782E+00_JPRB, 0.85858E+00_JPRB, 0.88709E+00_JPRB, 0.93516E+00_JPRB ,&
&  0.10017E+01_JPRB, 0.10817E+01_JPRB, 0.11800E+01_JPRB, 0.12967E+01_JPRB, 0.14290E+01_JPRB, 0.15440E+01_JPRB, 0.15639E+01_JPRB ,&
&  0.66943E+00_JPRB, 0.64351E+00_JPRB, 0.77083E+00_JPRB, 0.76872E+00_JPRB, 0.73123E+00_JPRB, 0.76372E+00_JPRB, 0.83353E+00_JPRB ,&
&  0.88692E+00_JPRB, 0.85478E+00_JPRB, 0.93631E+00_JPRB, 0.11895E+01_JPRB, 0.11926E+01_JPRB, 0.12651E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.13805E+01_JPRB, 0.14403E+01_JPRB, 0.14885E+01_JPRB, 0.15306E+01_JPRB, 0.15875E+01_JPRB, 0.16612E+01_JPRB, 0.17628E+01_JPRB ,&
&  0.18934E+01_JPRB, 0.20953E+01_JPRB, 0.22926E+01_JPRB, 0.20719E+01_JPRB, 0.21122E+01_JPRB, 0.22147E+01_JPRB, 0.24820E+01_JPRB ,&
&  0.22610E+01_JPRB, 0.16662E+01_JPRB, 0.13348E+01_JPRB, 0.12714E+01_JPRB, 0.11747E+01_JPRB, 0.11640E+01_JPRB, 0.11165E+01_JPRB ,&
&  0.10483E+01_JPRB, 0.98971E+00_JPRB, 0.92996E+00_JPRB, 0.86652E+00_JPRB, 0.78964E+00_JPRB, 0.77133E+00_JPRB, 0.76648E+00_JPRB ,&
&  0.80815E+00_JPRB, 0.83461E+00_JPRB, 0.82807E+00_JPRB, 0.81217E+00_JPRB, 0.78302E+00_JPRB, 0.81144E+00_JPRB, 0.87386E+00_JPRB ,&
&  0.92325E+00_JPRB, 0.94321E+00_JPRB, 0.95074E+00_JPRB, 0.95910E+00_JPRB, 0.96491E+00_JPRB, 0.95783E+00_JPRB, 0.92388E+00_JPRB ,&
&  0.86346E+00_JPRB, 0.81864E+00_JPRB, 0.81674E+00_JPRB, 0.80272E+00_JPRB, 0.95745E+00_JPRB, 0.12528E+01_JPRB, 0.12886E+01_JPRB ,&
&  0.10984E+01_JPRB, 0.84563E+00_JPRB, 0.68198E+00_JPRB, 0.63032E+00_JPRB, 0.60577E+00_JPRB /)
ZBURD(:, 43,  2) = (/ &
&  0.65244E+00_JPRB, 0.62007E+00_JPRB, 0.61238E+00_JPRB, 0.59647E+00_JPRB, 0.60951E+00_JPRB, 0.63486E+00_JPRB, 0.65366E+00_JPRB ,&
&  0.65772E+00_JPRB, 0.65652E+00_JPRB, 0.65753E+00_JPRB, 0.67748E+00_JPRB, 0.70525E+00_JPRB, 0.73493E+00_JPRB, 0.77215E+00_JPRB ,&
&  0.79702E+00_JPRB, 0.81100E+00_JPRB, 0.78771E+00_JPRB, 0.75567E+00_JPRB, 0.77105E+00_JPRB, 0.80021E+00_JPRB, 0.84456E+00_JPRB ,&
&  0.90437E+00_JPRB, 0.97409E+00_JPRB, 0.10738E+01_JPRB, 0.12395E+01_JPRB, 0.14910E+01_JPRB, 0.17939E+01_JPRB, 0.21068E+01_JPRB ,&
&  0.16707E+01_JPRB, 0.78848E+00_JPRB, 0.81297E+00_JPRB, 0.87181E+00_JPRB, 0.79444E+00_JPRB, 0.82920E+00_JPRB, 0.84758E+00_JPRB ,&
&  0.85155E+00_JPRB, 0.87513E+00_JPRB, 0.87544E+00_JPRB, 0.99734E+00_JPRB, 0.11408E+01_JPRB, 0.12274E+01_JPRB, 0.13104E+01_JPRB ,&
&  0.13726E+01_JPRB, 0.14182E+01_JPRB, 0.14384E+01_JPRB, 0.14588E+01_JPRB, 0.15044E+01_JPRB, 0.15754E+01_JPRB, 0.16755E+01_JPRB ,&
&  0.18217E+01_JPRB, 0.19978E+01_JPRB, 0.22274E+01_JPRB, 0.20074E+01_JPRB, 0.22284E+01_JPRB, 0.22357E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.21233E+01_JPRB, 0.15632E+01_JPRB, 0.12762E+01_JPRB, 0.12611E+01_JPRB, 0.10867E+01_JPRB, 0.86596E+00_JPRB, 0.96933E+00_JPRB ,&
&  0.98396E+00_JPRB, 0.97653E+00_JPRB, 0.95717E+00_JPRB, 0.87719E+00_JPRB, 0.80814E+00_JPRB, 0.79707E+00_JPRB, 0.85683E+00_JPRB ,&
&  0.91005E+00_JPRB, 0.94394E+00_JPRB, 0.92416E+00_JPRB, 0.92371E+00_JPRB, 0.91071E+00_JPRB, 0.94205E+00_JPRB, 0.99757E+00_JPRB ,&
&  0.10469E+01_JPRB, 0.10918E+01_JPRB, 0.11162E+01_JPRB, 0.11510E+01_JPRB, 0.11911E+01_JPRB, 0.11174E+01_JPRB, 0.10174E+01_JPRB ,&
&  0.98271E+00_JPRB, 0.96300E+00_JPRB, 0.90614E+00_JPRB, 0.86671E+00_JPRB, 0.84241E+00_JPRB, 0.87578E+00_JPRB, 0.97844E+00_JPRB ,&
&  0.96196E+00_JPRB, 0.84916E+00_JPRB, 0.73340E+00_JPRB, 0.70193E+00_JPRB, 0.70642E+00_JPRB /)
ZBURD(:, 44,  2) = (/ &
&  0.74904E+00_JPRB, 0.67419E+00_JPRB, 0.63172E+00_JPRB, 0.62610E+00_JPRB, 0.61216E+00_JPRB, 0.61703E+00_JPRB, 0.62535E+00_JPRB ,&
&  0.62203E+00_JPRB, 0.64074E+00_JPRB, 0.63461E+00_JPRB, 0.64828E+00_JPRB, 0.66918E+00_JPRB, 0.67615E+00_JPRB, 0.70046E+00_JPRB ,&
&  0.72507E+00_JPRB, 0.75367E+00_JPRB, 0.77824E+00_JPRB, 0.74529E+00_JPRB, 0.74187E+00_JPRB, 0.74910E+00_JPRB, 0.77200E+00_JPRB ,&
&  0.81767E+00_JPRB, 0.88038E+00_JPRB, 0.95310E+00_JPRB, 0.10449E+01_JPRB, 0.11797E+01_JPRB, 0.14226E+01_JPRB, 0.18765E+01_JPRB ,&
&  0.30055E+01_JPRB, 0.24649E+01_JPRB, 0.80990E+00_JPRB, 0.10115E+01_JPRB, 0.98414E+00_JPRB, 0.97363E+00_JPRB, 0.85131E+00_JPRB ,&
&  0.89736E+00_JPRB, 0.97045E+00_JPRB, 0.90922E+00_JPRB, 0.89653E+00_JPRB, 0.94837E+00_JPRB, 0.10793E+01_JPRB, 0.11990E+01_JPRB ,&
&  0.12687E+01_JPRB, 0.13135E+01_JPRB, 0.13229E+01_JPRB, 0.13345E+01_JPRB, 0.13565E+01_JPRB, 0.14041E+01_JPRB, 0.14838E+01_JPRB ,&
&  0.16109E+01_JPRB, 0.17926E+01_JPRB, 0.20254E+01_JPRB, 0.19459E+01_JPRB, 0.23204E+01_JPRB, 0.22478E+01_JPRB, 0.21225E+01_JPRB ,&
&  0.18185E+01_JPRB, 0.15714E+01_JPRB, 0.14244E+01_JPRB, 0.12564E+01_JPRB, 0.95966E+00_JPRB, 0.81789E+00_JPRB, 0.89405E+00_JPRB ,&
&  0.93066E+00_JPRB, 0.86788E+00_JPRB, 0.84768E+00_JPRB, 0.84177E+00_JPRB, 0.80796E+00_JPRB, 0.87310E+00_JPRB, 0.95881E+00_JPRB ,&
&  0.99234E+00_JPRB, 0.99782E+00_JPRB, 0.10056E+01_JPRB, 0.10026E+01_JPRB, 0.10054E+01_JPRB, 0.10293E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.11138E+01_JPRB, 0.11528E+01_JPRB, 0.11743E+01_JPRB, 0.12182E+01_JPRB, 0.12611E+01_JPRB, 0.11821E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10317E+01_JPRB, 0.10422E+01_JPRB, 0.99838E+00_JPRB, 0.95684E+00_JPRB, 0.89683E+00_JPRB, 0.86950E+00_JPRB, 0.86531E+00_JPRB ,&
&  0.94656E+00_JPRB, 0.88426E+00_JPRB, 0.82847E+00_JPRB, 0.75975E+00_JPRB, 0.77906E+00_JPRB /)
ZBURD(:, 45,  2) = (/ &
&  0.79681E+00_JPRB, 0.74925E+00_JPRB, 0.67239E+00_JPRB, 0.64171E+00_JPRB, 0.63340E+00_JPRB, 0.62020E+00_JPRB, 0.60204E+00_JPRB ,&
&  0.57821E+00_JPRB, 0.56955E+00_JPRB, 0.59517E+00_JPRB, 0.62602E+00_JPRB, 0.65372E+00_JPRB, 0.65631E+00_JPRB, 0.66367E+00_JPRB ,&
&  0.66921E+00_JPRB, 0.69221E+00_JPRB, 0.73606E+00_JPRB, 0.74846E+00_JPRB, 0.73880E+00_JPRB, 0.73533E+00_JPRB, 0.74045E+00_JPRB ,&
&  0.75394E+00_JPRB, 0.77626E+00_JPRB, 0.82191E+00_JPRB, 0.87497E+00_JPRB, 0.95489E+00_JPRB, 0.10912E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.20116E+01_JPRB, 0.35337E+01_JPRB, 0.94556E+00_JPRB, 0.12461E+01_JPRB, 0.11965E+01_JPRB, 0.10911E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.99948E+00_JPRB, 0.10098E+01_JPRB, 0.10116E+01_JPRB, 0.92977E+00_JPRB, 0.79774E+00_JPRB, 0.87108E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.11326E+01_JPRB, 0.11765E+01_JPRB, 0.11844E+01_JPRB, 0.11711E+01_JPRB, 0.11756E+01_JPRB, 0.11876E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.14714E+01_JPRB, 0.16737E+01_JPRB, 0.17021E+01_JPRB, 0.21425E+01_JPRB, 0.22826E+01_JPRB, 0.22087E+01_JPRB ,&
&  0.18486E+01_JPRB, 0.17449E+01_JPRB, 0.16198E+01_JPRB, 0.13571E+01_JPRB, 0.10490E+01_JPRB, 0.88606E+00_JPRB, 0.92539E+00_JPRB ,&
&  0.90403E+00_JPRB, 0.85938E+00_JPRB, 0.84448E+00_JPRB, 0.83461E+00_JPRB, 0.85070E+00_JPRB, 0.91756E+00_JPRB, 0.97051E+00_JPRB ,&
&  0.98046E+00_JPRB, 0.10045E+01_JPRB, 0.10055E+01_JPRB, 0.10048E+01_JPRB, 0.10159E+01_JPRB, 0.10330E+01_JPRB, 0.10798E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11533E+01_JPRB, 0.11792E+01_JPRB, 0.11661E+01_JPRB, 0.12004E+01_JPRB, 0.11729E+01_JPRB, 0.11088E+01_JPRB ,&
&  0.11332E+01_JPRB, 0.12149E+01_JPRB, 0.10728E+01_JPRB, 0.10021E+01_JPRB, 0.99834E+00_JPRB, 0.97189E+00_JPRB, 0.94684E+00_JPRB ,&
&  0.98527E+00_JPRB, 0.99981E+00_JPRB, 0.97387E+00_JPRB, 0.84541E+00_JPRB, 0.84704E+00_JPRB /)
ZBURD(:, 46,  2) = (/ &
&  0.85186E+00_JPRB, 0.80117E+00_JPRB, 0.74643E+00_JPRB, 0.68118E+00_JPRB, 0.67679E+00_JPRB, 0.68072E+00_JPRB, 0.64959E+00_JPRB ,&
&  0.59738E+00_JPRB, 0.58940E+00_JPRB, 0.59820E+00_JPRB, 0.62108E+00_JPRB, 0.64115E+00_JPRB, 0.64396E+00_JPRB, 0.66378E+00_JPRB ,&
&  0.65613E+00_JPRB, 0.65267E+00_JPRB, 0.67553E+00_JPRB, 0.71182E+00_JPRB, 0.71901E+00_JPRB, 0.72072E+00_JPRB, 0.73590E+00_JPRB ,&
&  0.74315E+00_JPRB, 0.73361E+00_JPRB, 0.73501E+00_JPRB, 0.75562E+00_JPRB, 0.80936E+00_JPRB, 0.90288E+00_JPRB, 0.10744E+01_JPRB ,&
&  0.16700E+01_JPRB, 0.38751E+01_JPRB, 0.26654E+01_JPRB, 0.22068E+01_JPRB, 0.15762E+01_JPRB, 0.12570E+01_JPRB, 0.11323E+01_JPRB ,&
&  0.12123E+01_JPRB, 0.11776E+01_JPRB, 0.11852E+01_JPRB, 0.97454E+00_JPRB, 0.82958E+00_JPRB, 0.77147E+00_JPRB, 0.85089E+00_JPRB ,&
&  0.92978E+00_JPRB, 0.97719E+00_JPRB, 0.10165E+01_JPRB, 0.10224E+01_JPRB, 0.10251E+01_JPRB, 0.10344E+01_JPRB, 0.10447E+01_JPRB ,&
&  0.10840E+01_JPRB, 0.11859E+01_JPRB, 0.13477E+01_JPRB, 0.15214E+01_JPRB, 0.17519E+01_JPRB, 0.20515E+01_JPRB, 0.23611E+01_JPRB ,&
&  0.22855E+01_JPRB, 0.20928E+01_JPRB, 0.17972E+01_JPRB, 0.14197E+01_JPRB, 0.10930E+01_JPRB, 0.10664E+01_JPRB, 0.95195E+00_JPRB ,&
&  0.91077E+00_JPRB, 0.86472E+00_JPRB, 0.86254E+00_JPRB, 0.88580E+00_JPRB, 0.91829E+00_JPRB, 0.95424E+00_JPRB, 0.97049E+00_JPRB ,&
&  0.98507E+00_JPRB, 0.99213E+00_JPRB, 0.99737E+00_JPRB, 0.99509E+00_JPRB, 0.99926E+00_JPRB, 0.10050E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.10448E+01_JPRB, 0.10941E+01_JPRB, 0.10755E+01_JPRB, 0.10782E+01_JPRB, 0.11041E+01_JPRB, 0.11217E+01_JPRB, 0.10682E+01_JPRB ,&
&  0.10930E+01_JPRB, 0.11823E+01_JPRB, 0.11531E+01_JPRB, 0.11175E+01_JPRB, 0.10821E+01_JPRB, 0.10814E+01_JPRB, 0.10489E+01_JPRB ,&
&  0.10607E+01_JPRB, 0.10428E+01_JPRB, 0.97182E+00_JPRB, 0.92560E+00_JPRB, 0.91349E+00_JPRB /)
ZBURD(:, 47,  2) = (/ &
&  0.91089E+00_JPRB, 0.86811E+00_JPRB, 0.82221E+00_JPRB, 0.76813E+00_JPRB, 0.73252E+00_JPRB, 0.72992E+00_JPRB, 0.73785E+00_JPRB ,&
&  0.70741E+00_JPRB, 0.67811E+00_JPRB, 0.66546E+00_JPRB, 0.67894E+00_JPRB, 0.70116E+00_JPRB, 0.70445E+00_JPRB, 0.70525E+00_JPRB ,&
&  0.69249E+00_JPRB, 0.68681E+00_JPRB, 0.67474E+00_JPRB, 0.68286E+00_JPRB, 0.73419E+00_JPRB, 0.73591E+00_JPRB, 0.72769E+00_JPRB ,&
&  0.74648E+00_JPRB, 0.74221E+00_JPRB, 0.72237E+00_JPRB, 0.72312E+00_JPRB, 0.74074E+00_JPRB, 0.79983E+00_JPRB, 0.92795E+00_JPRB ,&
&  0.15496E+01_JPRB, 0.35925E+01_JPRB, 0.18330E+01_JPRB, 0.25050E+01_JPRB, 0.17328E+01_JPRB, 0.13992E+01_JPRB, 0.11286E+01_JPRB ,&
&  0.11493E+01_JPRB, 0.13635E+01_JPRB, 0.11589E+01_JPRB, 0.10165E+01_JPRB, 0.87879E+00_JPRB, 0.81317E+00_JPRB, 0.84320E+00_JPRB ,&
&  0.88240E+00_JPRB, 0.88404E+00_JPRB, 0.89237E+00_JPRB, 0.90725E+00_JPRB, 0.89993E+00_JPRB, 0.89797E+00_JPRB, 0.91367E+00_JPRB ,&
&  0.92850E+00_JPRB, 0.97868E+00_JPRB, 0.10805E+01_JPRB, 0.12096E+01_JPRB, 0.15578E+01_JPRB, 0.19964E+01_JPRB, 0.24787E+01_JPRB ,&
&  0.25068E+01_JPRB, 0.25637E+01_JPRB, 0.20199E+01_JPRB, 0.15107E+01_JPRB, 0.11959E+01_JPRB, 0.10972E+01_JPRB, 0.10074E+01_JPRB ,&
&  0.95453E+00_JPRB, 0.92914E+00_JPRB, 0.89883E+00_JPRB, 0.90474E+00_JPRB, 0.95939E+00_JPRB, 0.99030E+00_JPRB, 0.97456E+00_JPRB ,&
&  0.98651E+00_JPRB, 0.98228E+00_JPRB, 0.98344E+00_JPRB, 0.97189E+00_JPRB, 0.98335E+00_JPRB, 0.98358E+00_JPRB, 0.97608E+00_JPRB ,&
&  0.97919E+00_JPRB, 0.10125E+01_JPRB, 0.10248E+01_JPRB, 0.10375E+01_JPRB, 0.10571E+01_JPRB, 0.10778E+01_JPRB, 0.10539E+01_JPRB ,&
&  0.10941E+01_JPRB, 0.11373E+01_JPRB, 0.11514E+01_JPRB, 0.11545E+01_JPRB, 0.11310E+01_JPRB, 0.11722E+01_JPRB, 0.11225E+01_JPRB ,&
&  0.11243E+01_JPRB, 0.10973E+01_JPRB, 0.10304E+01_JPRB, 0.99651E+00_JPRB, 0.97345E+00_JPRB /)
ZBURD(:, 48,  2) = (/ &
&  0.98986E+00_JPRB, 0.92903E+00_JPRB, 0.88550E+00_JPRB, 0.84249E+00_JPRB, 0.80654E+00_JPRB, 0.78120E+00_JPRB, 0.77701E+00_JPRB ,&
&  0.76281E+00_JPRB, 0.72812E+00_JPRB, 0.70328E+00_JPRB, 0.70257E+00_JPRB, 0.70176E+00_JPRB, 0.70641E+00_JPRB, 0.70955E+00_JPRB ,&
&  0.70444E+00_JPRB, 0.69326E+00_JPRB, 0.68939E+00_JPRB, 0.69388E+00_JPRB, 0.72307E+00_JPRB, 0.76316E+00_JPRB, 0.77390E+00_JPRB ,&
&  0.76424E+00_JPRB, 0.75663E+00_JPRB, 0.73617E+00_JPRB, 0.71783E+00_JPRB, 0.71773E+00_JPRB, 0.75026E+00_JPRB, 0.85307E+00_JPRB ,&
&  0.14523E+01_JPRB, 0.38552E+01_JPRB, 0.17215E+01_JPRB, 0.26286E+01_JPRB, 0.18353E+01_JPRB, 0.14723E+01_JPRB, 0.11798E+01_JPRB ,&
&  0.11824E+01_JPRB, 0.13045E+01_JPRB, 0.11840E+01_JPRB, 0.10671E+01_JPRB, 0.95695E+00_JPRB, 0.88924E+00_JPRB, 0.86238E+00_JPRB ,&
&  0.89379E+00_JPRB, 0.87895E+00_JPRB, 0.86397E+00_JPRB, 0.85931E+00_JPRB, 0.84107E+00_JPRB, 0.83484E+00_JPRB, 0.84381E+00_JPRB ,&
&  0.84102E+00_JPRB, 0.84543E+00_JPRB, 0.89157E+00_JPRB, 0.10931E+01_JPRB, 0.13225E+01_JPRB, 0.19065E+01_JPRB, 0.27789E+01_JPRB ,&
&  0.28077E+01_JPRB, 0.29557E+01_JPRB, 0.22180E+01_JPRB, 0.16284E+01_JPRB, 0.12662E+01_JPRB, 0.11267E+01_JPRB, 0.10764E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.98392E+00_JPRB, 0.95496E+00_JPRB, 0.95429E+00_JPRB, 0.97865E+00_JPRB, 0.96925E+00_JPRB, 0.97576E+00_JPRB ,&
&  0.98308E+00_JPRB, 0.94506E+00_JPRB, 0.93087E+00_JPRB, 0.92484E+00_JPRB, 0.93858E+00_JPRB, 0.94668E+00_JPRB, 0.94418E+00_JPRB ,&
&  0.94283E+00_JPRB, 0.95486E+00_JPRB, 0.99275E+00_JPRB, 0.10121E+01_JPRB, 0.10393E+01_JPRB, 0.10553E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.11261E+01_JPRB, 0.11349E+01_JPRB, 0.12014E+01_JPRB, 0.12326E+01_JPRB, 0.13486E+01_JPRB, 0.11743E+01_JPRB ,&
&  0.11566E+01_JPRB, 0.11294E+01_JPRB, 0.10788E+01_JPRB, 0.10559E+01_JPRB, 0.10421E+01_JPRB /)
ZBURD(:, 49,  2) = (/ &
&  0.10619E+01_JPRB, 0.10151E+01_JPRB, 0.97404E+00_JPRB, 0.93020E+00_JPRB, 0.87914E+00_JPRB, 0.84556E+00_JPRB, 0.81790E+00_JPRB ,&
&  0.79884E+00_JPRB, 0.76275E+00_JPRB, 0.73594E+00_JPRB, 0.72941E+00_JPRB, 0.73223E+00_JPRB, 0.72514E+00_JPRB, 0.72282E+00_JPRB ,&
&  0.72829E+00_JPRB, 0.73057E+00_JPRB, 0.72095E+00_JPRB, 0.71183E+00_JPRB, 0.71355E+00_JPRB, 0.74920E+00_JPRB, 0.76607E+00_JPRB ,&
&  0.77219E+00_JPRB, 0.76186E+00_JPRB, 0.73932E+00_JPRB, 0.71032E+00_JPRB, 0.70271E+00_JPRB, 0.71674E+00_JPRB, 0.79189E+00_JPRB ,&
&  0.12588E+01_JPRB, 0.15817E+01_JPRB, 0.27060E+01_JPRB, 0.27102E+01_JPRB, 0.19206E+01_JPRB, 0.15517E+01_JPRB, 0.13558E+01_JPRB ,&
&  0.13474E+01_JPRB, 0.12940E+01_JPRB, 0.11865E+01_JPRB, 0.10949E+01_JPRB, 0.10167E+01_JPRB, 0.95150E+00_JPRB, 0.90350E+00_JPRB ,&
&  0.88343E+00_JPRB, 0.86678E+00_JPRB, 0.85492E+00_JPRB, 0.85114E+00_JPRB, 0.84131E+00_JPRB, 0.83667E+00_JPRB, 0.83575E+00_JPRB ,&
&  0.83854E+00_JPRB, 0.83568E+00_JPRB, 0.84277E+00_JPRB, 0.91613E+00_JPRB, 0.12079E+01_JPRB, 0.17256E+01_JPRB, 0.25385E+01_JPRB ,&
&  0.30552E+01_JPRB, 0.28935E+01_JPRB, 0.21362E+01_JPRB, 0.16593E+01_JPRB, 0.13077E+01_JPRB, 0.11188E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10386E+01_JPRB, 0.10023E+01_JPRB, 0.99246E+00_JPRB, 0.98975E+00_JPRB, 0.96763E+00_JPRB, 0.95811E+00_JPRB ,&
&  0.95153E+00_JPRB, 0.94850E+00_JPRB, 0.93752E+00_JPRB, 0.92460E+00_JPRB, 0.90913E+00_JPRB, 0.89970E+00_JPRB, 0.89833E+00_JPRB ,&
&  0.90152E+00_JPRB, 0.90583E+00_JPRB, 0.96710E+00_JPRB, 0.98609E+00_JPRB, 0.10465E+01_JPRB, 0.10783E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.11336E+01_JPRB, 0.11381E+01_JPRB, 0.12379E+01_JPRB, 0.12636E+01_JPRB, 0.14242E+01_JPRB, 0.12620E+01_JPRB ,&
&  0.12096E+01_JPRB, 0.11767E+01_JPRB, 0.11331E+01_JPRB, 0.10921E+01_JPRB, 0.10748E+01_JPRB /)
ZBURD(:, 50,  2) = (/ &
&  0.10786E+01_JPRB, 0.10256E+01_JPRB, 0.98920E+00_JPRB, 0.96025E+00_JPRB, 0.92322E+00_JPRB, 0.88581E+00_JPRB, 0.86757E+00_JPRB ,&
&  0.85260E+00_JPRB, 0.82335E+00_JPRB, 0.79144E+00_JPRB, 0.75434E+00_JPRB, 0.74193E+00_JPRB, 0.74569E+00_JPRB, 0.75088E+00_JPRB ,&
&  0.74892E+00_JPRB, 0.74943E+00_JPRB, 0.74775E+00_JPRB, 0.74581E+00_JPRB, 0.74385E+00_JPRB, 0.76584E+00_JPRB, 0.77510E+00_JPRB ,&
&  0.76687E+00_JPRB, 0.76755E+00_JPRB, 0.74672E+00_JPRB, 0.72457E+00_JPRB, 0.71323E+00_JPRB, 0.70007E+00_JPRB, 0.70529E+00_JPRB ,&
&  0.10109E+01_JPRB, 0.17782E+01_JPRB, 0.25707E+01_JPRB, 0.23827E+01_JPRB, 0.18850E+01_JPRB, 0.15173E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.13537E+01_JPRB, 0.12847E+01_JPRB, 0.11878E+01_JPRB, 0.11085E+01_JPRB, 0.10437E+01_JPRB, 0.99507E+00_JPRB, 0.94466E+00_JPRB ,&
&  0.91344E+00_JPRB, 0.89743E+00_JPRB, 0.87087E+00_JPRB, 0.85828E+00_JPRB, 0.85093E+00_JPRB, 0.85197E+00_JPRB, 0.85175E+00_JPRB ,&
&  0.85602E+00_JPRB, 0.85892E+00_JPRB, 0.86203E+00_JPRB, 0.88515E+00_JPRB, 0.10613E+01_JPRB, 0.11733E+01_JPRB, 0.18709E+01_JPRB ,&
&  0.26152E+01_JPRB, 0.21736E+01_JPRB, 0.17399E+01_JPRB, 0.15017E+01_JPRB, 0.13094E+01_JPRB, 0.11313E+01_JPRB, 0.10553E+01_JPRB ,&
&  0.10228E+01_JPRB, 0.10082E+01_JPRB, 0.97622E+00_JPRB, 0.96783E+00_JPRB, 0.95574E+00_JPRB, 0.94479E+00_JPRB, 0.93446E+00_JPRB ,&
&  0.91469E+00_JPRB, 0.91347E+00_JPRB, 0.91363E+00_JPRB, 0.91312E+00_JPRB, 0.89911E+00_JPRB, 0.89357E+00_JPRB, 0.88729E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.87179E+00_JPRB, 0.93481E+00_JPRB, 0.94215E+00_JPRB, 0.10248E+01_JPRB, 0.10995E+01_JPRB, 0.11244E+01_JPRB ,&
&  0.11239E+01_JPRB, 0.11700E+01_JPRB, 0.11726E+01_JPRB, 0.12830E+01_JPRB, 0.14802E+01_JPRB, 0.16334E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.12965E+01_JPRB, 0.12519E+01_JPRB, 0.12011E+01_JPRB, 0.11452E+01_JPRB, 0.11226E+01_JPRB /)
ZBURD(:, 51,  2) = (/ &
&  0.11380E+01_JPRB, 0.10701E+01_JPRB, 0.10158E+01_JPRB, 0.96759E+00_JPRB, 0.94403E+00_JPRB, 0.91404E+00_JPRB, 0.88137E+00_JPRB ,&
&  0.86064E+00_JPRB, 0.85283E+00_JPRB, 0.83951E+00_JPRB, 0.81589E+00_JPRB, 0.79543E+00_JPRB, 0.78729E+00_JPRB, 0.78544E+00_JPRB ,&
&  0.78261E+00_JPRB, 0.78094E+00_JPRB, 0.77830E+00_JPRB, 0.76981E+00_JPRB, 0.77153E+00_JPRB, 0.77959E+00_JPRB, 0.78799E+00_JPRB ,&
&  0.76996E+00_JPRB, 0.75574E+00_JPRB, 0.74465E+00_JPRB, 0.73343E+00_JPRB, 0.72918E+00_JPRB, 0.70922E+00_JPRB, 0.68610E+00_JPRB ,&
&  0.80895E+00_JPRB, 0.12428E+01_JPRB, 0.17370E+01_JPRB, 0.21325E+01_JPRB, 0.18701E+01_JPRB, 0.15841E+01_JPRB, 0.14568E+01_JPRB ,&
&  0.13638E+01_JPRB, 0.12529E+01_JPRB, 0.11640E+01_JPRB, 0.11008E+01_JPRB, 0.10658E+01_JPRB, 0.10256E+01_JPRB, 0.96316E+00_JPRB ,&
&  0.91689E+00_JPRB, 0.88834E+00_JPRB, 0.87570E+00_JPRB, 0.87494E+00_JPRB, 0.86796E+00_JPRB, 0.87658E+00_JPRB, 0.88101E+00_JPRB ,&
&  0.88208E+00_JPRB, 0.87745E+00_JPRB, 0.86993E+00_JPRB, 0.86780E+00_JPRB, 0.96501E+00_JPRB, 0.10597E+01_JPRB, 0.12892E+01_JPRB ,&
&  0.13817E+01_JPRB, 0.14144E+01_JPRB, 0.13607E+01_JPRB, 0.13153E+01_JPRB, 0.12497E+01_JPRB, 0.11294E+01_JPRB, 0.10429E+01_JPRB ,&
&  0.98987E+00_JPRB, 0.96976E+00_JPRB, 0.93815E+00_JPRB, 0.92823E+00_JPRB, 0.91898E+00_JPRB, 0.90673E+00_JPRB, 0.90227E+00_JPRB ,&
&  0.89548E+00_JPRB, 0.89044E+00_JPRB, 0.88548E+00_JPRB, 0.89244E+00_JPRB, 0.88732E+00_JPRB, 0.87032E+00_JPRB, 0.85792E+00_JPRB ,&
&  0.85452E+00_JPRB, 0.84090E+00_JPRB, 0.86003E+00_JPRB, 0.90652E+00_JPRB, 0.95610E+00_JPRB, 0.99304E+00_JPRB, 0.10368E+01_JPRB ,&
&  0.10936E+01_JPRB, 0.12100E+01_JPRB, 0.11963E+01_JPRB, 0.13561E+01_JPRB, 0.15999E+01_JPRB, 0.17414E+01_JPRB, 0.15276E+01_JPRB ,&
&  0.14004E+01_JPRB, 0.13240E+01_JPRB, 0.12562E+01_JPRB, 0.11788E+01_JPRB, 0.11939E+01_JPRB /)
ZBURD(:, 52,  2) = (/ &
&  0.11451E+01_JPRB, 0.10981E+01_JPRB, 0.10454E+01_JPRB, 0.98545E+00_JPRB, 0.95414E+00_JPRB, 0.92844E+00_JPRB, 0.90501E+00_JPRB ,&
&  0.89207E+00_JPRB, 0.87904E+00_JPRB, 0.86952E+00_JPRB, 0.85177E+00_JPRB, 0.83423E+00_JPRB, 0.82334E+00_JPRB, 0.81100E+00_JPRB ,&
&  0.81038E+00_JPRB, 0.80956E+00_JPRB, 0.80073E+00_JPRB, 0.79137E+00_JPRB, 0.78626E+00_JPRB, 0.78157E+00_JPRB, 0.78414E+00_JPRB ,&
&  0.77510E+00_JPRB, 0.76455E+00_JPRB, 0.76431E+00_JPRB, 0.75693E+00_JPRB, 0.74334E+00_JPRB, 0.73077E+00_JPRB, 0.69878E+00_JPRB ,&
&  0.71177E+00_JPRB, 0.82339E+00_JPRB, 0.11833E+01_JPRB, 0.15904E+01_JPRB, 0.15735E+01_JPRB, 0.15253E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.13122E+01_JPRB, 0.12194E+01_JPRB, 0.11155E+01_JPRB, 0.10456E+01_JPRB, 0.10233E+01_JPRB, 0.99919E+00_JPRB, 0.97106E+00_JPRB ,&
&  0.93804E+00_JPRB, 0.90771E+00_JPRB, 0.88779E+00_JPRB, 0.89100E+00_JPRB, 0.88553E+00_JPRB, 0.88564E+00_JPRB, 0.88822E+00_JPRB ,&
&  0.88285E+00_JPRB, 0.87678E+00_JPRB, 0.86103E+00_JPRB, 0.85050E+00_JPRB, 0.84481E+00_JPRB, 0.85781E+00_JPRB, 0.91977E+00_JPRB ,&
&  0.10056E+01_JPRB, 0.10365E+01_JPRB, 0.10566E+01_JPRB, 0.10795E+01_JPRB, 0.10959E+01_JPRB, 0.10790E+01_JPRB, 0.10314E+01_JPRB ,&
&  0.99083E+00_JPRB, 0.96730E+00_JPRB, 0.93115E+00_JPRB, 0.93573E+00_JPRB, 0.92136E+00_JPRB, 0.90124E+00_JPRB, 0.88541E+00_JPRB ,&
&  0.86760E+00_JPRB, 0.86666E+00_JPRB, 0.85493E+00_JPRB, 0.85111E+00_JPRB, 0.84883E+00_JPRB, 0.84094E+00_JPRB, 0.83002E+00_JPRB ,&
&  0.82951E+00_JPRB, 0.82422E+00_JPRB, 0.83256E+00_JPRB, 0.86278E+00_JPRB, 0.90462E+00_JPRB, 0.93582E+00_JPRB, 0.97004E+00_JPRB ,&
&  0.10066E+01_JPRB, 0.10793E+01_JPRB, 0.11012E+01_JPRB, 0.13452E+01_JPRB, 0.15796E+01_JPRB, 0.16126E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.14199E+01_JPRB, 0.13314E+01_JPRB, 0.12694E+01_JPRB, 0.11859E+01_JPRB, 0.11484E+01_JPRB /)
ZBURD(:, 53,  2) = (/ &
&  0.10041E+01_JPRB, 0.10067E+01_JPRB, 0.97478E+00_JPRB, 0.92804E+00_JPRB, 0.90148E+00_JPRB, 0.88915E+00_JPRB, 0.88496E+00_JPRB ,&
&  0.87879E+00_JPRB, 0.86632E+00_JPRB, 0.86983E+00_JPRB, 0.87483E+00_JPRB, 0.87033E+00_JPRB, 0.85665E+00_JPRB, 0.84668E+00_JPRB ,&
&  0.84307E+00_JPRB, 0.84565E+00_JPRB, 0.84955E+00_JPRB, 0.83810E+00_JPRB, 0.81513E+00_JPRB, 0.79895E+00_JPRB, 0.78531E+00_JPRB ,&
&  0.78338E+00_JPRB, 0.78337E+00_JPRB, 0.78142E+00_JPRB, 0.78311E+00_JPRB, 0.76166E+00_JPRB, 0.73701E+00_JPRB, 0.71635E+00_JPRB ,&
&  0.70225E+00_JPRB, 0.64844E+00_JPRB, 0.82347E+00_JPRB, 0.10935E+01_JPRB, 0.12266E+01_JPRB, 0.12250E+01_JPRB, 0.11493E+01_JPRB ,&
&  0.11005E+01_JPRB, 0.10578E+01_JPRB, 0.10143E+01_JPRB, 0.96740E+00_JPRB, 0.93936E+00_JPRB, 0.92194E+00_JPRB, 0.90917E+00_JPRB ,&
&  0.89753E+00_JPRB, 0.88421E+00_JPRB, 0.87426E+00_JPRB, 0.85462E+00_JPRB, 0.83833E+00_JPRB, 0.83460E+00_JPRB, 0.83403E+00_JPRB ,&
&  0.83599E+00_JPRB, 0.82944E+00_JPRB, 0.82769E+00_JPRB, 0.82686E+00_JPRB, 0.80659E+00_JPRB, 0.80380E+00_JPRB, 0.81053E+00_JPRB ,&
&  0.83404E+00_JPRB, 0.85913E+00_JPRB, 0.87066E+00_JPRB, 0.91401E+00_JPRB, 0.94164E+00_JPRB, 0.96593E+00_JPRB, 0.96627E+00_JPRB ,&
&  0.95165E+00_JPRB, 0.94225E+00_JPRB, 0.92872E+00_JPRB, 0.92371E+00_JPRB, 0.91111E+00_JPRB, 0.88295E+00_JPRB, 0.85543E+00_JPRB ,&
&  0.84521E+00_JPRB, 0.84038E+00_JPRB, 0.82601E+00_JPRB, 0.81903E+00_JPRB, 0.81274E+00_JPRB, 0.79958E+00_JPRB, 0.78892E+00_JPRB ,&
&  0.78191E+00_JPRB, 0.78189E+00_JPRB, 0.78490E+00_JPRB, 0.80111E+00_JPRB, 0.82789E+00_JPRB, 0.86256E+00_JPRB, 0.89875E+00_JPRB ,&
&  0.93499E+00_JPRB, 0.97465E+00_JPRB, 0.10271E+01_JPRB, 0.11941E+01_JPRB, 0.12683E+01_JPRB, 0.13110E+01_JPRB, 0.13506E+01_JPRB ,&
&  0.13345E+01_JPRB, 0.12648E+01_JPRB, 0.11891E+01_JPRB, 0.10852E+01_JPRB, 0.96489E+00_JPRB /)
ZBURD(:, 54,  2) = (/ &
&  0.94131E+00_JPRB, 0.93392E+00_JPRB, 0.91439E+00_JPRB, 0.89089E+00_JPRB, 0.87491E+00_JPRB, 0.86397E+00_JPRB, 0.86397E+00_JPRB ,&
&  0.86024E+00_JPRB, 0.85325E+00_JPRB, 0.85503E+00_JPRB, 0.85648E+00_JPRB, 0.85800E+00_JPRB, 0.85443E+00_JPRB, 0.85839E+00_JPRB ,&
&  0.85850E+00_JPRB, 0.85919E+00_JPRB, 0.86140E+00_JPRB, 0.85373E+00_JPRB, 0.83236E+00_JPRB, 0.81685E+00_JPRB, 0.80165E+00_JPRB ,&
&  0.80017E+00_JPRB, 0.79984E+00_JPRB, 0.80051E+00_JPRB, 0.79430E+00_JPRB, 0.77409E+00_JPRB, 0.75622E+00_JPRB, 0.74470E+00_JPRB ,&
&  0.74366E+00_JPRB, 0.60665E+00_JPRB, 0.68589E+00_JPRB, 0.90685E+00_JPRB, 0.10119E+01_JPRB, 0.97962E+00_JPRB, 0.96617E+00_JPRB ,&
&  0.97053E+00_JPRB, 0.96743E+00_JPRB, 0.94816E+00_JPRB, 0.91593E+00_JPRB, 0.87065E+00_JPRB, 0.85480E+00_JPRB, 0.83336E+00_JPRB ,&
&  0.81416E+00_JPRB, 0.80675E+00_JPRB, 0.79982E+00_JPRB, 0.79831E+00_JPRB, 0.78718E+00_JPRB, 0.77473E+00_JPRB, 0.77521E+00_JPRB ,&
&  0.78294E+00_JPRB, 0.78250E+00_JPRB, 0.78632E+00_JPRB, 0.79043E+00_JPRB, 0.78497E+00_JPRB, 0.80018E+00_JPRB, 0.80861E+00_JPRB ,&
&  0.81642E+00_JPRB, 0.82058E+00_JPRB, 0.82295E+00_JPRB, 0.82591E+00_JPRB, 0.83959E+00_JPRB, 0.87388E+00_JPRB, 0.89705E+00_JPRB ,&
&  0.89261E+00_JPRB, 0.88209E+00_JPRB, 0.88687E+00_JPRB, 0.88870E+00_JPRB, 0.88034E+00_JPRB, 0.86195E+00_JPRB, 0.84050E+00_JPRB ,&
&  0.82906E+00_JPRB, 0.81933E+00_JPRB, 0.81135E+00_JPRB, 0.80003E+00_JPRB, 0.78681E+00_JPRB, 0.77526E+00_JPRB, 0.76712E+00_JPRB ,&
&  0.76237E+00_JPRB, 0.76076E+00_JPRB, 0.76736E+00_JPRB, 0.78424E+00_JPRB, 0.80719E+00_JPRB, 0.83507E+00_JPRB, 0.86572E+00_JPRB ,&
&  0.90268E+00_JPRB, 0.94623E+00_JPRB, 0.98813E+00_JPRB, 0.10755E+01_JPRB, 0.11893E+01_JPRB, 0.12276E+01_JPRB, 0.12451E+01_JPRB ,&
&  0.12352E+01_JPRB, 0.11800E+01_JPRB, 0.10909E+01_JPRB, 0.83758E+00_JPRB, 0.91773E+00_JPRB /)
ZBURD(:, 55,  2) = (/ &
&  0.89831E+00_JPRB, 0.89006E+00_JPRB, 0.88196E+00_JPRB, 0.86245E+00_JPRB, 0.84148E+00_JPRB, 0.83197E+00_JPRB, 0.83604E+00_JPRB ,&
&  0.82965E+00_JPRB, 0.81483E+00_JPRB, 0.81863E+00_JPRB, 0.82108E+00_JPRB, 0.82761E+00_JPRB, 0.83040E+00_JPRB, 0.83377E+00_JPRB ,&
&  0.83730E+00_JPRB, 0.84037E+00_JPRB, 0.83737E+00_JPRB, 0.84431E+00_JPRB, 0.83571E+00_JPRB, 0.82913E+00_JPRB, 0.82548E+00_JPRB ,&
&  0.81659E+00_JPRB, 0.80762E+00_JPRB, 0.80027E+00_JPRB, 0.79006E+00_JPRB, 0.78229E+00_JPRB, 0.77296E+00_JPRB, 0.75716E+00_JPRB ,&
&  0.76800E+00_JPRB, 0.61801E+00_JPRB, 0.70303E+00_JPRB, 0.85283E+00_JPRB, 0.85599E+00_JPRB, 0.84672E+00_JPRB, 0.86015E+00_JPRB ,&
&  0.88061E+00_JPRB, 0.89044E+00_JPRB, 0.88238E+00_JPRB, 0.85787E+00_JPRB, 0.80951E+00_JPRB, 0.78546E+00_JPRB, 0.77129E+00_JPRB ,&
&  0.75565E+00_JPRB, 0.74436E+00_JPRB, 0.73769E+00_JPRB, 0.73929E+00_JPRB, 0.73934E+00_JPRB, 0.73630E+00_JPRB, 0.73290E+00_JPRB ,&
&  0.73650E+00_JPRB, 0.74623E+00_JPRB, 0.75215E+00_JPRB, 0.75788E+00_JPRB, 0.76342E+00_JPRB, 0.75960E+00_JPRB, 0.75894E+00_JPRB ,&
&  0.75865E+00_JPRB, 0.76058E+00_JPRB, 0.75982E+00_JPRB, 0.76181E+00_JPRB, 0.76635E+00_JPRB, 0.78959E+00_JPRB, 0.80941E+00_JPRB ,&
&  0.81407E+00_JPRB, 0.81417E+00_JPRB, 0.81805E+00_JPRB, 0.82332E+00_JPRB, 0.82842E+00_JPRB, 0.82981E+00_JPRB, 0.82207E+00_JPRB ,&
&  0.81318E+00_JPRB, 0.80081E+00_JPRB, 0.79475E+00_JPRB, 0.79008E+00_JPRB, 0.78190E+00_JPRB, 0.77706E+00_JPRB, 0.77751E+00_JPRB ,&
&  0.77047E+00_JPRB, 0.75848E+00_JPRB, 0.76049E+00_JPRB, 0.77453E+00_JPRB, 0.78753E+00_JPRB, 0.79708E+00_JPRB, 0.81701E+00_JPRB ,&
&  0.85294E+00_JPRB, 0.89426E+00_JPRB, 0.92637E+00_JPRB, 0.10041E+01_JPRB, 0.10551E+01_JPRB, 0.10896E+01_JPRB, 0.11010E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.10687E+01_JPRB, 0.76356E+00_JPRB, 0.88000E+00_JPRB, 0.89924E+00_JPRB /)
ZBURD(:, 56,  2) = (/ &
&  0.88418E+00_JPRB, 0.86862E+00_JPRB, 0.85019E+00_JPRB, 0.82785E+00_JPRB, 0.80724E+00_JPRB, 0.79402E+00_JPRB, 0.79782E+00_JPRB ,&
&  0.79562E+00_JPRB, 0.78240E+00_JPRB, 0.79168E+00_JPRB, 0.80410E+00_JPRB, 0.81409E+00_JPRB, 0.82345E+00_JPRB, 0.82659E+00_JPRB ,&
&  0.82988E+00_JPRB, 0.82633E+00_JPRB, 0.82876E+00_JPRB, 0.84000E+00_JPRB, 0.85784E+00_JPRB, 0.85903E+00_JPRB, 0.85576E+00_JPRB ,&
&  0.83307E+00_JPRB, 0.82159E+00_JPRB, 0.81105E+00_JPRB, 0.79824E+00_JPRB, 0.78672E+00_JPRB, 0.78259E+00_JPRB, 0.78500E+00_JPRB ,&
&  0.67093E+00_JPRB, 0.57707E+00_JPRB, 0.69241E+00_JPRB, 0.78731E+00_JPRB, 0.78806E+00_JPRB, 0.78612E+00_JPRB, 0.80324E+00_JPRB ,&
&  0.80950E+00_JPRB, 0.80969E+00_JPRB, 0.80803E+00_JPRB, 0.79779E+00_JPRB, 0.76700E+00_JPRB, 0.73320E+00_JPRB, 0.71531E+00_JPRB ,&
&  0.70465E+00_JPRB, 0.70960E+00_JPRB, 0.71180E+00_JPRB, 0.71930E+00_JPRB, 0.71953E+00_JPRB, 0.71246E+00_JPRB, 0.71085E+00_JPRB ,&
&  0.71155E+00_JPRB, 0.72153E+00_JPRB, 0.73088E+00_JPRB, 0.73767E+00_JPRB, 0.74010E+00_JPRB, 0.74062E+00_JPRB, 0.73748E+00_JPRB ,&
&  0.73529E+00_JPRB, 0.73373E+00_JPRB, 0.73052E+00_JPRB, 0.73282E+00_JPRB, 0.73747E+00_JPRB, 0.74848E+00_JPRB, 0.76580E+00_JPRB ,&
&  0.77554E+00_JPRB, 0.77818E+00_JPRB, 0.78025E+00_JPRB, 0.78139E+00_JPRB, 0.79186E+00_JPRB, 0.79943E+00_JPRB, 0.79481E+00_JPRB ,&
&  0.78700E+00_JPRB, 0.78123E+00_JPRB, 0.78464E+00_JPRB, 0.78671E+00_JPRB, 0.79028E+00_JPRB, 0.79062E+00_JPRB, 0.78460E+00_JPRB ,&
&  0.77519E+00_JPRB, 0.76083E+00_JPRB, 0.75019E+00_JPRB, 0.74962E+00_JPRB, 0.75987E+00_JPRB, 0.76845E+00_JPRB, 0.78787E+00_JPRB ,&
&  0.82181E+00_JPRB, 0.86353E+00_JPRB, 0.89864E+00_JPRB, 0.95714E+00_JPRB, 0.99870E+00_JPRB, 0.10254E+01_JPRB, 0.10307E+01_JPRB ,&
&  0.10183E+01_JPRB, 0.97707E+00_JPRB, 0.96047E+00_JPRB, 0.92681E+00_JPRB, 0.90330E+00_JPRB /)
ZBURD(:, 57,  2) = (/ &
&  0.83260E+00_JPRB, 0.82300E+00_JPRB, 0.80542E+00_JPRB, 0.77481E+00_JPRB, 0.73699E+00_JPRB, 0.74278E+00_JPRB, 0.75321E+00_JPRB ,&
&  0.74647E+00_JPRB, 0.73625E+00_JPRB, 0.74631E+00_JPRB, 0.77095E+00_JPRB, 0.79515E+00_JPRB, 0.80885E+00_JPRB, 0.81076E+00_JPRB ,&
&  0.80533E+00_JPRB, 0.80696E+00_JPRB, 0.81492E+00_JPRB, 0.83172E+00_JPRB, 0.83060E+00_JPRB, 0.81250E+00_JPRB, 0.80143E+00_JPRB ,&
&  0.79830E+00_JPRB, 0.79639E+00_JPRB, 0.79357E+00_JPRB, 0.77659E+00_JPRB, 0.76524E+00_JPRB, 0.76214E+00_JPRB, 0.76263E+00_JPRB ,&
&  0.75871E+00_JPRB, 0.62044E+00_JPRB, 0.72815E+00_JPRB, 0.73721E+00_JPRB, 0.74588E+00_JPRB, 0.73824E+00_JPRB, 0.73824E+00_JPRB ,&
&  0.74289E+00_JPRB, 0.75875E+00_JPRB, 0.75205E+00_JPRB, 0.72854E+00_JPRB, 0.70364E+00_JPRB, 0.67653E+00_JPRB, 0.65836E+00_JPRB ,&
&  0.64946E+00_JPRB, 0.65274E+00_JPRB, 0.66309E+00_JPRB, 0.67394E+00_JPRB, 0.68542E+00_JPRB, 0.68723E+00_JPRB, 0.69115E+00_JPRB ,&
&  0.69365E+00_JPRB, 0.69858E+00_JPRB, 0.70093E+00_JPRB, 0.70638E+00_JPRB, 0.70831E+00_JPRB, 0.70499E+00_JPRB, 0.69751E+00_JPRB ,&
&  0.68701E+00_JPRB, 0.68074E+00_JPRB, 0.68016E+00_JPRB, 0.68295E+00_JPRB, 0.69113E+00_JPRB, 0.70570E+00_JPRB, 0.72079E+00_JPRB ,&
&  0.73744E+00_JPRB, 0.74854E+00_JPRB, 0.75474E+00_JPRB, 0.75752E+00_JPRB, 0.76677E+00_JPRB, 0.77642E+00_JPRB, 0.77561E+00_JPRB ,&
&  0.76789E+00_JPRB, 0.75916E+00_JPRB, 0.75555E+00_JPRB, 0.75081E+00_JPRB, 0.74989E+00_JPRB, 0.74940E+00_JPRB, 0.74443E+00_JPRB ,&
&  0.73595E+00_JPRB, 0.73410E+00_JPRB, 0.73535E+00_JPRB, 0.72486E+00_JPRB, 0.72717E+00_JPRB, 0.73806E+00_JPRB, 0.75841E+00_JPRB ,&
&  0.78405E+00_JPRB, 0.81748E+00_JPRB, 0.85041E+00_JPRB, 0.88438E+00_JPRB, 0.90951E+00_JPRB, 0.92570E+00_JPRB, 0.92879E+00_JPRB ,&
&  0.91679E+00_JPRB, 0.89466E+00_JPRB, 0.87308E+00_JPRB, 0.85762E+00_JPRB, 0.85183E+00_JPRB /)
ZBURD(:, 58,  2) = (/ &
&  0.78330E+00_JPRB, 0.78181E+00_JPRB, 0.76946E+00_JPRB, 0.72778E+00_JPRB, 0.69577E+00_JPRB, 0.70353E+00_JPRB, 0.71150E+00_JPRB ,&
&  0.71051E+00_JPRB, 0.72512E+00_JPRB, 0.74995E+00_JPRB, 0.77510E+00_JPRB, 0.79193E+00_JPRB, 0.79171E+00_JPRB, 0.79560E+00_JPRB ,&
&  0.79278E+00_JPRB, 0.79041E+00_JPRB, 0.80016E+00_JPRB, 0.80494E+00_JPRB, 0.79415E+00_JPRB, 0.77308E+00_JPRB, 0.77064E+00_JPRB ,&
&  0.77667E+00_JPRB, 0.77242E+00_JPRB, 0.75412E+00_JPRB, 0.74159E+00_JPRB, 0.73134E+00_JPRB, 0.73645E+00_JPRB, 0.74329E+00_JPRB ,&
&  0.73772E+00_JPRB, 0.69008E+00_JPRB, 0.72829E+00_JPRB, 0.73833E+00_JPRB, 0.74362E+00_JPRB, 0.73003E+00_JPRB, 0.73425E+00_JPRB ,&
&  0.74520E+00_JPRB, 0.75201E+00_JPRB, 0.73253E+00_JPRB, 0.71015E+00_JPRB, 0.69020E+00_JPRB, 0.67354E+00_JPRB, 0.64642E+00_JPRB ,&
&  0.63647E+00_JPRB, 0.63379E+00_JPRB, 0.64739E+00_JPRB, 0.66171E+00_JPRB, 0.67824E+00_JPRB, 0.68499E+00_JPRB, 0.69545E+00_JPRB ,&
&  0.69752E+00_JPRB, 0.69178E+00_JPRB, 0.68643E+00_JPRB, 0.68215E+00_JPRB, 0.68111E+00_JPRB, 0.67512E+00_JPRB, 0.66637E+00_JPRB ,&
&  0.65940E+00_JPRB, 0.65758E+00_JPRB, 0.66203E+00_JPRB, 0.67136E+00_JPRB, 0.68325E+00_JPRB, 0.70212E+00_JPRB, 0.71722E+00_JPRB ,&
&  0.73553E+00_JPRB, 0.74195E+00_JPRB, 0.74759E+00_JPRB, 0.76091E+00_JPRB, 0.76957E+00_JPRB, 0.76828E+00_JPRB, 0.76525E+00_JPRB ,&
&  0.76171E+00_JPRB, 0.75507E+00_JPRB, 0.75863E+00_JPRB, 0.75669E+00_JPRB, 0.75632E+00_JPRB, 0.74726E+00_JPRB, 0.73179E+00_JPRB ,&
&  0.72585E+00_JPRB, 0.72143E+00_JPRB, 0.71292E+00_JPRB, 0.71014E+00_JPRB, 0.70814E+00_JPRB, 0.72641E+00_JPRB, 0.75844E+00_JPRB ,&
&  0.77673E+00_JPRB, 0.80155E+00_JPRB, 0.83602E+00_JPRB, 0.85997E+00_JPRB, 0.86354E+00_JPRB, 0.86827E+00_JPRB, 0.86168E+00_JPRB ,&
&  0.84651E+00_JPRB, 0.83561E+00_JPRB, 0.82163E+00_JPRB, 0.81428E+00_JPRB, 0.80683E+00_JPRB /)
ZBURD(:, 59,  2) = (/ &
&  0.73429E+00_JPRB, 0.75501E+00_JPRB, 0.72076E+00_JPRB, 0.70026E+00_JPRB, 0.67798E+00_JPRB, 0.67205E+00_JPRB, 0.66218E+00_JPRB ,&
&  0.67254E+00_JPRB, 0.68932E+00_JPRB, 0.71301E+00_JPRB, 0.72095E+00_JPRB, 0.72783E+00_JPRB, 0.73061E+00_JPRB, 0.74752E+00_JPRB ,&
&  0.74277E+00_JPRB, 0.73439E+00_JPRB, 0.73746E+00_JPRB, 0.72709E+00_JPRB, 0.71830E+00_JPRB, 0.72316E+00_JPRB, 0.73208E+00_JPRB ,&
&  0.74211E+00_JPRB, 0.73339E+00_JPRB, 0.71599E+00_JPRB, 0.68919E+00_JPRB, 0.67644E+00_JPRB, 0.67364E+00_JPRB, 0.68123E+00_JPRB ,&
&  0.68670E+00_JPRB, 0.71272E+00_JPRB, 0.64904E+00_JPRB, 0.70543E+00_JPRB, 0.71259E+00_JPRB, 0.70969E+00_JPRB, 0.70767E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.70166E+00_JPRB, 0.69319E+00_JPRB, 0.68516E+00_JPRB, 0.66290E+00_JPRB, 0.64504E+00_JPRB, 0.62611E+00_JPRB ,&
&  0.60941E+00_JPRB, 0.59921E+00_JPRB, 0.60461E+00_JPRB, 0.62232E+00_JPRB, 0.64038E+00_JPRB, 0.65041E+00_JPRB, 0.65654E+00_JPRB ,&
&  0.65980E+00_JPRB, 0.65237E+00_JPRB, 0.64213E+00_JPRB, 0.64188E+00_JPRB, 0.64799E+00_JPRB, 0.65263E+00_JPRB, 0.65556E+00_JPRB ,&
&  0.65578E+00_JPRB, 0.63257E+00_JPRB, 0.61939E+00_JPRB, 0.63551E+00_JPRB, 0.65322E+00_JPRB, 0.67281E+00_JPRB, 0.69850E+00_JPRB ,&
&  0.71815E+00_JPRB, 0.72599E+00_JPRB, 0.72567E+00_JPRB, 0.73263E+00_JPRB, 0.73960E+00_JPRB, 0.74827E+00_JPRB, 0.75499E+00_JPRB ,&
&  0.74436E+00_JPRB, 0.74166E+00_JPRB, 0.74223E+00_JPRB, 0.74188E+00_JPRB, 0.73264E+00_JPRB, 0.70957E+00_JPRB, 0.67734E+00_JPRB ,&
&  0.66247E+00_JPRB, 0.64738E+00_JPRB, 0.63628E+00_JPRB, 0.63318E+00_JPRB, 0.63529E+00_JPRB, 0.66180E+00_JPRB, 0.69966E+00_JPRB ,&
&  0.73744E+00_JPRB, 0.76886E+00_JPRB, 0.79630E+00_JPRB, 0.82134E+00_JPRB, 0.82114E+00_JPRB, 0.80217E+00_JPRB, 0.79575E+00_JPRB ,&
&  0.78336E+00_JPRB, 0.76878E+00_JPRB, 0.75726E+00_JPRB, 0.74706E+00_JPRB, 0.73535E+00_JPRB /)
ZBURD(:, 60,  2) = (/ &
&  0.73396E+00_JPRB, 0.73456E+00_JPRB, 0.71797E+00_JPRB, 0.70527E+00_JPRB, 0.68761E+00_JPRB, 0.66557E+00_JPRB, 0.64622E+00_JPRB ,&
&  0.64765E+00_JPRB, 0.66283E+00_JPRB, 0.67753E+00_JPRB, 0.68811E+00_JPRB, 0.70360E+00_JPRB, 0.70677E+00_JPRB, 0.71321E+00_JPRB ,&
&  0.70899E+00_JPRB, 0.67191E+00_JPRB, 0.66602E+00_JPRB, 0.67358E+00_JPRB, 0.67538E+00_JPRB, 0.68138E+00_JPRB, 0.69224E+00_JPRB ,&
&  0.70216E+00_JPRB, 0.69748E+00_JPRB, 0.68926E+00_JPRB, 0.65947E+00_JPRB, 0.65258E+00_JPRB, 0.65736E+00_JPRB, 0.66229E+00_JPRB ,&
&  0.67249E+00_JPRB, 0.68964E+00_JPRB, 0.69285E+00_JPRB, 0.70134E+00_JPRB, 0.70952E+00_JPRB, 0.71311E+00_JPRB, 0.70932E+00_JPRB ,&
&  0.68335E+00_JPRB, 0.65746E+00_JPRB, 0.64693E+00_JPRB, 0.63008E+00_JPRB, 0.60832E+00_JPRB, 0.59301E+00_JPRB, 0.58548E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.58678E+00_JPRB, 0.59314E+00_JPRB, 0.60720E+00_JPRB, 0.61242E+00_JPRB, 0.61658E+00_JPRB, 0.62193E+00_JPRB ,&
&  0.62150E+00_JPRB, 0.61731E+00_JPRB, 0.61722E+00_JPRB, 0.62687E+00_JPRB, 0.63261E+00_JPRB, 0.63012E+00_JPRB, 0.62551E+00_JPRB ,&
&  0.61228E+00_JPRB, 0.59808E+00_JPRB, 0.60521E+00_JPRB, 0.62030E+00_JPRB, 0.62755E+00_JPRB, 0.65339E+00_JPRB, 0.68663E+00_JPRB ,&
&  0.71483E+00_JPRB, 0.73321E+00_JPRB, 0.73311E+00_JPRB, 0.72959E+00_JPRB, 0.72804E+00_JPRB, 0.73534E+00_JPRB, 0.74746E+00_JPRB ,&
&  0.73150E+00_JPRB, 0.72212E+00_JPRB, 0.71989E+00_JPRB, 0.70047E+00_JPRB, 0.66407E+00_JPRB, 0.63291E+00_JPRB, 0.60990E+00_JPRB ,&
&  0.59148E+00_JPRB, 0.58109E+00_JPRB, 0.58489E+00_JPRB, 0.58838E+00_JPRB, 0.60122E+00_JPRB, 0.63714E+00_JPRB, 0.67485E+00_JPRB ,&
&  0.71586E+00_JPRB, 0.73761E+00_JPRB, 0.74506E+00_JPRB, 0.75405E+00_JPRB, 0.76124E+00_JPRB, 0.75207E+00_JPRB, 0.74453E+00_JPRB ,&
&  0.74292E+00_JPRB, 0.73460E+00_JPRB, 0.72528E+00_JPRB, 0.71517E+00_JPRB, 0.72183E+00_JPRB /)
ZBURD(:, 61,  2) = (/ &
&  0.72697E+00_JPRB, 0.71854E+00_JPRB, 0.70291E+00_JPRB, 0.67497E+00_JPRB, 0.65687E+00_JPRB, 0.62960E+00_JPRB, 0.61438E+00_JPRB ,&
&  0.61676E+00_JPRB, 0.61825E+00_JPRB, 0.62352E+00_JPRB, 0.63278E+00_JPRB, 0.63074E+00_JPRB, 0.62605E+00_JPRB, 0.62127E+00_JPRB ,&
&  0.62413E+00_JPRB, 0.62448E+00_JPRB, 0.63294E+00_JPRB, 0.64843E+00_JPRB, 0.66473E+00_JPRB, 0.67402E+00_JPRB, 0.68390E+00_JPRB ,&
&  0.69153E+00_JPRB, 0.68539E+00_JPRB, 0.68095E+00_JPRB, 0.65032E+00_JPRB, 0.63248E+00_JPRB, 0.63749E+00_JPRB, 0.64486E+00_JPRB ,&
&  0.65489E+00_JPRB, 0.66549E+00_JPRB, 0.67512E+00_JPRB, 0.68103E+00_JPRB, 0.66699E+00_JPRB, 0.66294E+00_JPRB, 0.65199E+00_JPRB ,&
&  0.60919E+00_JPRB, 0.56739E+00_JPRB, 0.54490E+00_JPRB, 0.53748E+00_JPRB, 0.53239E+00_JPRB, 0.53840E+00_JPRB, 0.54597E+00_JPRB ,&
&  0.55923E+00_JPRB, 0.57135E+00_JPRB, 0.57447E+00_JPRB, 0.57767E+00_JPRB, 0.57925E+00_JPRB, 0.57845E+00_JPRB, 0.57897E+00_JPRB ,&
&  0.57632E+00_JPRB, 0.57476E+00_JPRB, 0.58195E+00_JPRB, 0.58239E+00_JPRB, 0.57339E+00_JPRB, 0.57057E+00_JPRB, 0.56824E+00_JPRB ,&
&  0.56542E+00_JPRB, 0.56270E+00_JPRB, 0.55954E+00_JPRB, 0.56973E+00_JPRB, 0.59559E+00_JPRB, 0.62522E+00_JPRB, 0.65078E+00_JPRB ,&
&  0.66540E+00_JPRB, 0.67731E+00_JPRB, 0.68046E+00_JPRB, 0.68240E+00_JPRB, 0.68184E+00_JPRB, 0.67308E+00_JPRB, 0.66973E+00_JPRB ,&
&  0.66182E+00_JPRB, 0.64254E+00_JPRB, 0.61685E+00_JPRB, 0.57703E+00_JPRB, 0.53844E+00_JPRB, 0.51647E+00_JPRB, 0.51611E+00_JPRB ,&
&  0.52834E+00_JPRB, 0.54086E+00_JPRB, 0.54732E+00_JPRB, 0.55548E+00_JPRB, 0.57036E+00_JPRB, 0.58829E+00_JPRB, 0.60720E+00_JPRB ,&
&  0.61929E+00_JPRB, 0.61981E+00_JPRB, 0.61787E+00_JPRB, 0.62657E+00_JPRB, 0.63828E+00_JPRB, 0.65705E+00_JPRB, 0.67736E+00_JPRB ,&
&  0.69516E+00_JPRB, 0.69495E+00_JPRB, 0.69555E+00_JPRB, 0.69705E+00_JPRB, 0.71677E+00_JPRB /)
ZBURD(:, 62,  2) = (/ &
&  0.71924E+00_JPRB, 0.71677E+00_JPRB, 0.68669E+00_JPRB, 0.65424E+00_JPRB, 0.63506E+00_JPRB, 0.61408E+00_JPRB, 0.59740E+00_JPRB ,&
&  0.59605E+00_JPRB, 0.59342E+00_JPRB, 0.59131E+00_JPRB, 0.58867E+00_JPRB, 0.58684E+00_JPRB, 0.58363E+00_JPRB, 0.57137E+00_JPRB ,&
&  0.56676E+00_JPRB, 0.57322E+00_JPRB, 0.59559E+00_JPRB, 0.62080E+00_JPRB, 0.64106E+00_JPRB, 0.66743E+00_JPRB, 0.67869E+00_JPRB ,&
&  0.69060E+00_JPRB, 0.70316E+00_JPRB, 0.70534E+00_JPRB, 0.67235E+00_JPRB, 0.64519E+00_JPRB, 0.64617E+00_JPRB, 0.65723E+00_JPRB ,&
&  0.65754E+00_JPRB, 0.66066E+00_JPRB, 0.66381E+00_JPRB, 0.66027E+00_JPRB, 0.63943E+00_JPRB, 0.60112E+00_JPRB, 0.57093E+00_JPRB ,&
&  0.54603E+00_JPRB, 0.52675E+00_JPRB, 0.52082E+00_JPRB, 0.50934E+00_JPRB, 0.50754E+00_JPRB, 0.51760E+00_JPRB, 0.53094E+00_JPRB ,&
&  0.54483E+00_JPRB, 0.55119E+00_JPRB, 0.55824E+00_JPRB, 0.55475E+00_JPRB, 0.55044E+00_JPRB, 0.55325E+00_JPRB, 0.55316E+00_JPRB ,&
&  0.55261E+00_JPRB, 0.54350E+00_JPRB, 0.54066E+00_JPRB, 0.53353E+00_JPRB, 0.52937E+00_JPRB, 0.52949E+00_JPRB, 0.53212E+00_JPRB ,&
&  0.53307E+00_JPRB, 0.53840E+00_JPRB, 0.54000E+00_JPRB, 0.54226E+00_JPRB, 0.55186E+00_JPRB, 0.56862E+00_JPRB, 0.58539E+00_JPRB ,&
&  0.59791E+00_JPRB, 0.60769E+00_JPRB, 0.61538E+00_JPRB, 0.62115E+00_JPRB, 0.62093E+00_JPRB, 0.61619E+00_JPRB, 0.60225E+00_JPRB ,&
&  0.57162E+00_JPRB, 0.53882E+00_JPRB, 0.51394E+00_JPRB, 0.49409E+00_JPRB, 0.49045E+00_JPRB, 0.50151E+00_JPRB, 0.51945E+00_JPRB ,&
&  0.53299E+00_JPRB, 0.53704E+00_JPRB, 0.53107E+00_JPRB, 0.52932E+00_JPRB, 0.53431E+00_JPRB, 0.54392E+00_JPRB, 0.55142E+00_JPRB ,&
&  0.55078E+00_JPRB, 0.54785E+00_JPRB, 0.55196E+00_JPRB, 0.55927E+00_JPRB, 0.57376E+00_JPRB, 0.59205E+00_JPRB, 0.61170E+00_JPRB ,&
&  0.63125E+00_JPRB, 0.64644E+00_JPRB, 0.66623E+00_JPRB, 0.68620E+00_JPRB, 0.71151E+00_JPRB /)
ZBURD(:, 63,  2) = (/ &
&  0.69943E+00_JPRB, 0.67098E+00_JPRB, 0.64116E+00_JPRB, 0.61597E+00_JPRB, 0.60384E+00_JPRB, 0.58625E+00_JPRB, 0.57155E+00_JPRB ,&
&  0.57072E+00_JPRB, 0.57854E+00_JPRB, 0.57385E+00_JPRB, 0.56784E+00_JPRB, 0.55845E+00_JPRB, 0.55071E+00_JPRB, 0.54150E+00_JPRB ,&
&  0.53749E+00_JPRB, 0.54901E+00_JPRB, 0.57467E+00_JPRB, 0.58755E+00_JPRB, 0.59381E+00_JPRB, 0.62439E+00_JPRB, 0.65582E+00_JPRB ,&
&  0.66516E+00_JPRB, 0.67921E+00_JPRB, 0.68336E+00_JPRB, 0.65803E+00_JPRB, 0.64139E+00_JPRB, 0.63485E+00_JPRB, 0.63858E+00_JPRB ,&
&  0.63381E+00_JPRB, 0.63795E+00_JPRB, 0.62964E+00_JPRB, 0.55979E+00_JPRB, 0.55880E+00_JPRB, 0.52814E+00_JPRB, 0.51758E+00_JPRB ,&
&  0.51233E+00_JPRB, 0.51770E+00_JPRB, 0.51387E+00_JPRB, 0.50466E+00_JPRB, 0.50097E+00_JPRB, 0.50418E+00_JPRB, 0.50563E+00_JPRB ,&
&  0.50133E+00_JPRB, 0.49083E+00_JPRB, 0.49536E+00_JPRB, 0.50181E+00_JPRB, 0.50558E+00_JPRB, 0.51278E+00_JPRB, 0.52339E+00_JPRB ,&
&  0.51328E+00_JPRB, 0.50406E+00_JPRB, 0.49536E+00_JPRB, 0.49558E+00_JPRB, 0.50197E+00_JPRB, 0.51605E+00_JPRB, 0.52068E+00_JPRB ,&
&  0.51842E+00_JPRB, 0.51708E+00_JPRB, 0.51809E+00_JPRB, 0.52451E+00_JPRB, 0.53945E+00_JPRB, 0.55424E+00_JPRB, 0.55546E+00_JPRB ,&
&  0.55651E+00_JPRB, 0.56099E+00_JPRB, 0.56588E+00_JPRB, 0.56767E+00_JPRB, 0.56364E+00_JPRB, 0.54846E+00_JPRB, 0.53097E+00_JPRB ,&
&  0.51441E+00_JPRB, 0.49749E+00_JPRB, 0.48333E+00_JPRB, 0.47894E+00_JPRB, 0.49229E+00_JPRB, 0.52175E+00_JPRB, 0.53829E+00_JPRB ,&
&  0.54232E+00_JPRB, 0.54445E+00_JPRB, 0.54789E+00_JPRB, 0.54901E+00_JPRB, 0.55111E+00_JPRB, 0.54922E+00_JPRB, 0.54271E+00_JPRB ,&
&  0.53148E+00_JPRB, 0.52044E+00_JPRB, 0.51796E+00_JPRB, 0.52258E+00_JPRB, 0.52811E+00_JPRB, 0.54051E+00_JPRB, 0.55737E+00_JPRB ,&
&  0.57962E+00_JPRB, 0.61820E+00_JPRB, 0.65083E+00_JPRB, 0.67606E+00_JPRB, 0.69402E+00_JPRB /)
ZBURD(:, 64,  2) = (/ &
&  0.67478E+00_JPRB, 0.64834E+00_JPRB, 0.63078E+00_JPRB, 0.61719E+00_JPRB, 0.59885E+00_JPRB, 0.58491E+00_JPRB, 0.57927E+00_JPRB ,&
&  0.58741E+00_JPRB, 0.58600E+00_JPRB, 0.57621E+00_JPRB, 0.56798E+00_JPRB, 0.55204E+00_JPRB, 0.53779E+00_JPRB, 0.53195E+00_JPRB ,&
&  0.54046E+00_JPRB, 0.55669E+00_JPRB, 0.57659E+00_JPRB, 0.58479E+00_JPRB, 0.58417E+00_JPRB, 0.59265E+00_JPRB, 0.61195E+00_JPRB ,&
&  0.64063E+00_JPRB, 0.66066E+00_JPRB, 0.66769E+00_JPRB, 0.65442E+00_JPRB, 0.63669E+00_JPRB, 0.62838E+00_JPRB, 0.61393E+00_JPRB ,&
&  0.59717E+00_JPRB, 0.60045E+00_JPRB, 0.53288E+00_JPRB, 0.52526E+00_JPRB, 0.50194E+00_JPRB, 0.48741E+00_JPRB, 0.48363E+00_JPRB ,&
&  0.49683E+00_JPRB, 0.50090E+00_JPRB, 0.49255E+00_JPRB, 0.48039E+00_JPRB, 0.47204E+00_JPRB, 0.46088E+00_JPRB, 0.45055E+00_JPRB ,&
&  0.44343E+00_JPRB, 0.44604E+00_JPRB, 0.45508E+00_JPRB, 0.46444E+00_JPRB, 0.47552E+00_JPRB, 0.48615E+00_JPRB, 0.48271E+00_JPRB ,&
&  0.47102E+00_JPRB, 0.46896E+00_JPRB, 0.47593E+00_JPRB, 0.49047E+00_JPRB, 0.50630E+00_JPRB, 0.51360E+00_JPRB, 0.51717E+00_JPRB ,&
&  0.50914E+00_JPRB, 0.49216E+00_JPRB, 0.47765E+00_JPRB, 0.46573E+00_JPRB, 0.43979E+00_JPRB, 0.37666E+00_JPRB, 0.35400E+00_JPRB ,&
&  0.36497E+00_JPRB, 0.42350E+00_JPRB, 0.49319E+00_JPRB, 0.48470E+00_JPRB, 0.48054E+00_JPRB, 0.46637E+00_JPRB, 0.43010E+00_JPRB ,&
&  0.38821E+00_JPRB, 0.32694E+00_JPRB, 0.30576E+00_JPRB, 0.30228E+00_JPRB, 0.30154E+00_JPRB, 0.30747E+00_JPRB, 0.31983E+00_JPRB ,&
&  0.33605E+00_JPRB, 0.36281E+00_JPRB, 0.38011E+00_JPRB, 0.38675E+00_JPRB, 0.38523E+00_JPRB, 0.36923E+00_JPRB, 0.35539E+00_JPRB ,&
&  0.34624E+00_JPRB, 0.37126E+00_JPRB, 0.43011E+00_JPRB, 0.51291E+00_JPRB, 0.51247E+00_JPRB, 0.53754E+00_JPRB, 0.56264E+00_JPRB ,&
&  0.58662E+00_JPRB, 0.62662E+00_JPRB, 0.65282E+00_JPRB, 0.65856E+00_JPRB, 0.67682E+00_JPRB /)
ZBURD(:, 65,  2) = (/ &
&  0.63420E+00_JPRB, 0.62186E+00_JPRB, 0.61358E+00_JPRB, 0.60778E+00_JPRB, 0.60091E+00_JPRB, 0.59961E+00_JPRB, 0.59890E+00_JPRB ,&
&  0.59310E+00_JPRB, 0.58469E+00_JPRB, 0.58559E+00_JPRB, 0.56837E+00_JPRB, 0.54379E+00_JPRB, 0.52733E+00_JPRB, 0.52511E+00_JPRB ,&
&  0.52908E+00_JPRB, 0.53630E+00_JPRB, 0.54546E+00_JPRB, 0.55425E+00_JPRB, 0.55923E+00_JPRB, 0.57098E+00_JPRB, 0.59195E+00_JPRB ,&
&  0.61639E+00_JPRB, 0.63021E+00_JPRB, 0.63446E+00_JPRB, 0.62638E+00_JPRB, 0.61390E+00_JPRB, 0.60099E+00_JPRB, 0.58996E+00_JPRB ,&
&  0.58113E+00_JPRB, 0.47912E+00_JPRB, 0.41101E+00_JPRB, 0.41754E+00_JPRB, 0.48671E+00_JPRB, 0.47052E+00_JPRB, 0.46861E+00_JPRB ,&
&  0.48117E+00_JPRB, 0.48129E+00_JPRB, 0.46806E+00_JPRB, 0.45102E+00_JPRB, 0.43508E+00_JPRB, 0.42352E+00_JPRB, 0.41511E+00_JPRB ,&
&  0.41127E+00_JPRB, 0.41803E+00_JPRB, 0.42619E+00_JPRB, 0.43878E+00_JPRB, 0.44199E+00_JPRB, 0.42932E+00_JPRB, 0.41406E+00_JPRB ,&
&  0.39935E+00_JPRB, 0.39407E+00_JPRB, 0.39643E+00_JPRB, 0.40288E+00_JPRB, 0.41139E+00_JPRB, 0.41799E+00_JPRB, 0.41578E+00_JPRB ,&
&  0.38703E+00_JPRB, 0.36400E+00_JPRB, 0.33586E+00_JPRB, 0.30937E+00_JPRB, 0.28639E+00_JPRB, 0.27097E+00_JPRB, 0.26301E+00_JPRB ,&
&  0.26102E+00_JPRB, 0.26831E+00_JPRB, 0.28966E+00_JPRB, 0.33846E+00_JPRB, 0.40522E+00_JPRB, 0.32636E+00_JPRB, 0.28742E+00_JPRB ,&
&  0.26600E+00_JPRB, 0.25303E+00_JPRB, 0.24816E+00_JPRB, 0.24899E+00_JPRB, 0.25316E+00_JPRB, 0.25922E+00_JPRB, 0.26398E+00_JPRB ,&
&  0.26778E+00_JPRB, 0.27119E+00_JPRB, 0.27307E+00_JPRB, 0.27399E+00_JPRB, 0.27556E+00_JPRB, 0.27675E+00_JPRB, 0.27571E+00_JPRB ,&
&  0.27429E+00_JPRB, 0.28054E+00_JPRB, 0.29812E+00_JPRB, 0.32586E+00_JPRB, 0.35842E+00_JPRB, 0.39375E+00_JPRB, 0.44594E+00_JPRB ,&
&  0.51160E+00_JPRB, 0.58662E+00_JPRB, 0.60483E+00_JPRB, 0.61767E+00_JPRB, 0.62802E+00_JPRB /)
ZBURD(:, 66,  2) = (/ &
&  0.57593E+00_JPRB, 0.57947E+00_JPRB, 0.58353E+00_JPRB, 0.58578E+00_JPRB, 0.58030E+00_JPRB, 0.57296E+00_JPRB, 0.56831E+00_JPRB ,&
&  0.57347E+00_JPRB, 0.57347E+00_JPRB, 0.55014E+00_JPRB, 0.53189E+00_JPRB, 0.52140E+00_JPRB, 0.51781E+00_JPRB, 0.51479E+00_JPRB ,&
&  0.50868E+00_JPRB, 0.50661E+00_JPRB, 0.50788E+00_JPRB, 0.51919E+00_JPRB, 0.52855E+00_JPRB, 0.53674E+00_JPRB, 0.54416E+00_JPRB ,&
&  0.53967E+00_JPRB, 0.55242E+00_JPRB, 0.57408E+00_JPRB, 0.57328E+00_JPRB, 0.57051E+00_JPRB, 0.56326E+00_JPRB, 0.54874E+00_JPRB ,&
&  0.50052E+00_JPRB, 0.43631E+00_JPRB, 0.37482E+00_JPRB, 0.38643E+00_JPRB, 0.46357E+00_JPRB, 0.47267E+00_JPRB, 0.45734E+00_JPRB ,&
&  0.46782E+00_JPRB, 0.46354E+00_JPRB, 0.44561E+00_JPRB, 0.43239E+00_JPRB, 0.41690E+00_JPRB, 0.40684E+00_JPRB, 0.39978E+00_JPRB ,&
&  0.39694E+00_JPRB, 0.39587E+00_JPRB, 0.38102E+00_JPRB, 0.33255E+00_JPRB, 0.30440E+00_JPRB, 0.28720E+00_JPRB, 0.26949E+00_JPRB ,&
&  0.25555E+00_JPRB, 0.24683E+00_JPRB, 0.24539E+00_JPRB, 0.24562E+00_JPRB, 0.25031E+00_JPRB, 0.25242E+00_JPRB, 0.25435E+00_JPRB ,&
&  0.25416E+00_JPRB, 0.25298E+00_JPRB, 0.24893E+00_JPRB, 0.24441E+00_JPRB, 0.24127E+00_JPRB, 0.24077E+00_JPRB, 0.24049E+00_JPRB ,&
&  0.24312E+00_JPRB, 0.24704E+00_JPRB, 0.26340E+00_JPRB, 0.28089E+00_JPRB, 0.27969E+00_JPRB, 0.26457E+00_JPRB, 0.24994E+00_JPRB ,&
&  0.24015E+00_JPRB, 0.23517E+00_JPRB, 0.23205E+00_JPRB, 0.23154E+00_JPRB, 0.23442E+00_JPRB, 0.23849E+00_JPRB, 0.23955E+00_JPRB ,&
&  0.23938E+00_JPRB, 0.24005E+00_JPRB, 0.24014E+00_JPRB, 0.24062E+00_JPRB, 0.24132E+00_JPRB, 0.24392E+00_JPRB, 0.24876E+00_JPRB ,&
&  0.25451E+00_JPRB, 0.26483E+00_JPRB, 0.27651E+00_JPRB, 0.29212E+00_JPRB, 0.30350E+00_JPRB, 0.31189E+00_JPRB, 0.32288E+00_JPRB ,&
&  0.34053E+00_JPRB, 0.37906E+00_JPRB, 0.44817E+00_JPRB, 0.57382E+00_JPRB, 0.56834E+00_JPRB /)
ZBURD(:, 67,  2) = (/ &
&  0.54397E+00_JPRB, 0.55520E+00_JPRB, 0.56757E+00_JPRB, 0.55937E+00_JPRB, 0.54573E+00_JPRB, 0.53804E+00_JPRB, 0.53784E+00_JPRB ,&
&  0.53753E+00_JPRB, 0.53477E+00_JPRB, 0.52270E+00_JPRB, 0.51011E+00_JPRB, 0.49216E+00_JPRB, 0.44410E+00_JPRB, 0.42597E+00_JPRB ,&
&  0.40851E+00_JPRB, 0.41108E+00_JPRB, 0.41567E+00_JPRB, 0.42228E+00_JPRB, 0.43635E+00_JPRB, 0.44729E+00_JPRB, 0.44941E+00_JPRB ,&
&  0.43955E+00_JPRB, 0.42130E+00_JPRB, 0.40379E+00_JPRB, 0.39014E+00_JPRB, 0.38974E+00_JPRB, 0.38976E+00_JPRB, 0.38275E+00_JPRB ,&
&  0.36911E+00_JPRB, 0.36293E+00_JPRB, 0.36255E+00_JPRB, 0.38248E+00_JPRB, 0.43488E+00_JPRB, 0.45441E+00_JPRB, 0.42929E+00_JPRB ,&
&  0.43361E+00_JPRB, 0.42866E+00_JPRB, 0.42461E+00_JPRB, 0.42595E+00_JPRB, 0.41588E+00_JPRB, 0.39666E+00_JPRB, 0.36285E+00_JPRB ,&
&  0.31932E+00_JPRB, 0.29677E+00_JPRB, 0.27582E+00_JPRB, 0.25614E+00_JPRB, 0.24100E+00_JPRB, 0.23216E+00_JPRB, 0.22467E+00_JPRB ,&
&  0.21978E+00_JPRB, 0.21565E+00_JPRB, 0.21294E+00_JPRB, 0.21281E+00_JPRB, 0.21427E+00_JPRB, 0.21493E+00_JPRB, 0.21579E+00_JPRB ,&
&  0.21714E+00_JPRB, 0.21783E+00_JPRB, 0.21819E+00_JPRB, 0.21936E+00_JPRB, 0.22076E+00_JPRB, 0.22332E+00_JPRB, 0.22693E+00_JPRB ,&
&  0.23138E+00_JPRB, 0.23509E+00_JPRB, 0.23866E+00_JPRB, 0.24474E+00_JPRB, 0.24531E+00_JPRB, 0.24413E+00_JPRB, 0.23634E+00_JPRB ,&
&  0.23070E+00_JPRB, 0.22817E+00_JPRB, 0.22559E+00_JPRB, 0.22487E+00_JPRB, 0.22595E+00_JPRB, 0.22782E+00_JPRB, 0.22970E+00_JPRB ,&
&  0.23098E+00_JPRB, 0.23101E+00_JPRB, 0.23021E+00_JPRB, 0.23014E+00_JPRB, 0.23075E+00_JPRB, 0.23091E+00_JPRB, 0.23445E+00_JPRB ,&
&  0.24223E+00_JPRB, 0.25369E+00_JPRB, 0.26635E+00_JPRB, 0.28376E+00_JPRB, 0.30008E+00_JPRB, 0.31669E+00_JPRB, 0.34437E+00_JPRB ,&
&  0.38392E+00_JPRB, 0.49177E+00_JPRB, 0.56096E+00_JPRB, 0.49951E+00_JPRB, 0.52363E+00_JPRB /)
ZBURD(:, 68,  2) = (/ &
&  0.51622E+00_JPRB, 0.52367E+00_JPRB, 0.53468E+00_JPRB, 0.53926E+00_JPRB, 0.54739E+00_JPRB, 0.53572E+00_JPRB, 0.52871E+00_JPRB ,&
&  0.49853E+00_JPRB, 0.47640E+00_JPRB, 0.44544E+00_JPRB, 0.40668E+00_JPRB, 0.38071E+00_JPRB, 0.36066E+00_JPRB, 0.35371E+00_JPRB ,&
&  0.34585E+00_JPRB, 0.34089E+00_JPRB, 0.33595E+00_JPRB, 0.33395E+00_JPRB, 0.33269E+00_JPRB, 0.33376E+00_JPRB, 0.33352E+00_JPRB ,&
&  0.32993E+00_JPRB, 0.32430E+00_JPRB, 0.31768E+00_JPRB, 0.32056E+00_JPRB, 0.33341E+00_JPRB, 0.34826E+00_JPRB, 0.36573E+00_JPRB ,&
&  0.38591E+00_JPRB, 0.39661E+00_JPRB, 0.40676E+00_JPRB, 0.41086E+00_JPRB, 0.39543E+00_JPRB, 0.39799E+00_JPRB, 0.40087E+00_JPRB ,&
&  0.40882E+00_JPRB, 0.41556E+00_JPRB, 0.41258E+00_JPRB, 0.40818E+00_JPRB, 0.35736E+00_JPRB, 0.32439E+00_JPRB, 0.29584E+00_JPRB ,&
&  0.27450E+00_JPRB, 0.26084E+00_JPRB, 0.24941E+00_JPRB, 0.23972E+00_JPRB, 0.22985E+00_JPRB, 0.22349E+00_JPRB, 0.21873E+00_JPRB ,&
&  0.21488E+00_JPRB, 0.21320E+00_JPRB, 0.21160E+00_JPRB, 0.21109E+00_JPRB, 0.21143E+00_JPRB, 0.21078E+00_JPRB, 0.21065E+00_JPRB ,&
&  0.21038E+00_JPRB, 0.20963E+00_JPRB, 0.20962E+00_JPRB, 0.21076E+00_JPRB, 0.21270E+00_JPRB, 0.21589E+00_JPRB, 0.21698E+00_JPRB ,&
&  0.21789E+00_JPRB, 0.21850E+00_JPRB, 0.21857E+00_JPRB, 0.22049E+00_JPRB, 0.22152E+00_JPRB, 0.22171E+00_JPRB, 0.22013E+00_JPRB ,&
&  0.21848E+00_JPRB, 0.21724E+00_JPRB, 0.21711E+00_JPRB, 0.21761E+00_JPRB, 0.21904E+00_JPRB, 0.22125E+00_JPRB, 0.22174E+00_JPRB ,&
&  0.22258E+00_JPRB, 0.22217E+00_JPRB, 0.22180E+00_JPRB, 0.22365E+00_JPRB, 0.22582E+00_JPRB, 0.22819E+00_JPRB, 0.23138E+00_JPRB ,&
&  0.23314E+00_JPRB, 0.24017E+00_JPRB, 0.25299E+00_JPRB, 0.27278E+00_JPRB, 0.30351E+00_JPRB, 0.32014E+00_JPRB, 0.35962E+00_JPRB ,&
&  0.42162E+00_JPRB, 0.56414E+00_JPRB, 0.62817E+00_JPRB, 0.48572E+00_JPRB, 0.50086E+00_JPRB /)
ZBURD(:, 69,  2) = (/ &
&  0.45874E+00_JPRB, 0.47093E+00_JPRB, 0.48802E+00_JPRB, 0.49861E+00_JPRB, 0.51270E+00_JPRB, 0.51531E+00_JPRB, 0.50970E+00_JPRB ,&
&  0.49104E+00_JPRB, 0.46172E+00_JPRB, 0.44521E+00_JPRB, 0.42459E+00_JPRB, 0.40336E+00_JPRB, 0.38192E+00_JPRB, 0.37241E+00_JPRB ,&
&  0.36099E+00_JPRB, 0.34803E+00_JPRB, 0.33479E+00_JPRB, 0.32335E+00_JPRB, 0.31233E+00_JPRB, 0.30215E+00_JPRB, 0.29623E+00_JPRB ,&
&  0.29016E+00_JPRB, 0.28960E+00_JPRB, 0.28896E+00_JPRB, 0.29306E+00_JPRB, 0.30426E+00_JPRB, 0.32307E+00_JPRB, 0.34889E+00_JPRB ,&
&  0.38765E+00_JPRB, 0.39318E+00_JPRB, 0.37838E+00_JPRB, 0.37203E+00_JPRB, 0.36463E+00_JPRB, 0.35824E+00_JPRB, 0.34506E+00_JPRB ,&
&  0.35239E+00_JPRB, 0.36430E+00_JPRB, 0.37983E+00_JPRB, 0.36863E+00_JPRB, 0.32314E+00_JPRB, 0.30302E+00_JPRB, 0.27950E+00_JPRB ,&
&  0.26741E+00_JPRB, 0.25735E+00_JPRB, 0.24943E+00_JPRB, 0.24181E+00_JPRB, 0.23375E+00_JPRB, 0.22885E+00_JPRB, 0.22345E+00_JPRB ,&
&  0.21965E+00_JPRB, 0.21813E+00_JPRB, 0.21575E+00_JPRB, 0.21578E+00_JPRB, 0.21563E+00_JPRB, 0.21487E+00_JPRB, 0.21414E+00_JPRB ,&
&  0.21291E+00_JPRB, 0.21159E+00_JPRB, 0.21100E+00_JPRB, 0.21156E+00_JPRB, 0.21244E+00_JPRB, 0.21330E+00_JPRB, 0.21133E+00_JPRB ,&
&  0.21006E+00_JPRB, 0.20880E+00_JPRB, 0.20868E+00_JPRB, 0.20996E+00_JPRB, 0.21040E+00_JPRB, 0.21126E+00_JPRB, 0.21144E+00_JPRB ,&
&  0.21157E+00_JPRB, 0.21216E+00_JPRB, 0.21182E+00_JPRB, 0.21253E+00_JPRB, 0.21281E+00_JPRB, 0.21394E+00_JPRB, 0.21474E+00_JPRB ,&
&  0.21560E+00_JPRB, 0.21766E+00_JPRB, 0.21983E+00_JPRB, 0.22292E+00_JPRB, 0.22710E+00_JPRB, 0.22985E+00_JPRB, 0.23430E+00_JPRB ,&
&  0.23963E+00_JPRB, 0.24849E+00_JPRB, 0.26529E+00_JPRB, 0.28414E+00_JPRB, 0.30753E+00_JPRB, 0.31900E+00_JPRB, 0.32742E+00_JPRB ,&
&  0.34047E+00_JPRB, 0.37361E+00_JPRB, 0.42409E+00_JPRB, 0.44587E+00_JPRB, 0.45142E+00_JPRB /)
ZBURD(:, 70,  2) = (/ &
&  0.45630E+00_JPRB, 0.45774E+00_JPRB, 0.45861E+00_JPRB, 0.45536E+00_JPRB, 0.45435E+00_JPRB, 0.45438E+00_JPRB, 0.45166E+00_JPRB ,&
&  0.44371E+00_JPRB, 0.44138E+00_JPRB, 0.43646E+00_JPRB, 0.42922E+00_JPRB, 0.42208E+00_JPRB, 0.41043E+00_JPRB, 0.39406E+00_JPRB ,&
&  0.37217E+00_JPRB, 0.35283E+00_JPRB, 0.33832E+00_JPRB, 0.32485E+00_JPRB, 0.31166E+00_JPRB, 0.29879E+00_JPRB, 0.29370E+00_JPRB ,&
&  0.28922E+00_JPRB, 0.28733E+00_JPRB, 0.28680E+00_JPRB, 0.29023E+00_JPRB, 0.29775E+00_JPRB, 0.30717E+00_JPRB, 0.31749E+00_JPRB ,&
&  0.32916E+00_JPRB, 0.34006E+00_JPRB, 0.34134E+00_JPRB, 0.33401E+00_JPRB, 0.32704E+00_JPRB, 0.32268E+00_JPRB, 0.31933E+00_JPRB ,&
&  0.31538E+00_JPRB, 0.30712E+00_JPRB, 0.29767E+00_JPRB, 0.28771E+00_JPRB, 0.27766E+00_JPRB, 0.27034E+00_JPRB, 0.26177E+00_JPRB ,&
&  0.25551E+00_JPRB, 0.24982E+00_JPRB, 0.24372E+00_JPRB, 0.23850E+00_JPRB, 0.23495E+00_JPRB, 0.23200E+00_JPRB, 0.22900E+00_JPRB ,&
&  0.22752E+00_JPRB, 0.22605E+00_JPRB, 0.22441E+00_JPRB, 0.22283E+00_JPRB, 0.22083E+00_JPRB, 0.21933E+00_JPRB, 0.21796E+00_JPRB ,&
&  0.21693E+00_JPRB, 0.21602E+00_JPRB, 0.21548E+00_JPRB, 0.21491E+00_JPRB, 0.21444E+00_JPRB, 0.21346E+00_JPRB, 0.21232E+00_JPRB ,&
&  0.21161E+00_JPRB, 0.21137E+00_JPRB, 0.21061E+00_JPRB, 0.21022E+00_JPRB, 0.20989E+00_JPRB, 0.20914E+00_JPRB, 0.20962E+00_JPRB ,&
&  0.21080E+00_JPRB, 0.21213E+00_JPRB, 0.21239E+00_JPRB, 0.21346E+00_JPRB, 0.21474E+00_JPRB, 0.21635E+00_JPRB, 0.21893E+00_JPRB ,&
&  0.22090E+00_JPRB, 0.22295E+00_JPRB, 0.22552E+00_JPRB, 0.22774E+00_JPRB, 0.23119E+00_JPRB, 0.23545E+00_JPRB, 0.24071E+00_JPRB ,&
&  0.24790E+00_JPRB, 0.25661E+00_JPRB, 0.26882E+00_JPRB, 0.28372E+00_JPRB, 0.30524E+00_JPRB, 0.31349E+00_JPRB, 0.32145E+00_JPRB ,&
&  0.32796E+00_JPRB, 0.34062E+00_JPRB, 0.42283E+00_JPRB, 0.43759E+00_JPRB, 0.45245E+00_JPRB /)
ZBURD(:, 71,  2) = (/ &
&  0.33120E+00_JPRB, 0.34140E+00_JPRB, 0.34991E+00_JPRB, 0.35804E+00_JPRB, 0.35926E+00_JPRB, 0.42386E+00_JPRB, 0.40390E+00_JPRB ,&
&  0.39424E+00_JPRB, 0.36663E+00_JPRB, 0.36794E+00_JPRB, 0.36379E+00_JPRB, 0.35741E+00_JPRB, 0.35027E+00_JPRB, 0.34418E+00_JPRB ,&
&  0.33810E+00_JPRB, 0.33042E+00_JPRB, 0.32564E+00_JPRB, 0.31973E+00_JPRB, 0.31408E+00_JPRB, 0.30841E+00_JPRB, 0.30500E+00_JPRB ,&
&  0.30247E+00_JPRB, 0.30100E+00_JPRB, 0.29989E+00_JPRB, 0.29868E+00_JPRB, 0.30101E+00_JPRB, 0.30217E+00_JPRB, 0.30024E+00_JPRB ,&
&  0.29644E+00_JPRB, 0.29289E+00_JPRB, 0.28850E+00_JPRB, 0.28401E+00_JPRB, 0.28112E+00_JPRB, 0.27848E+00_JPRB, 0.27574E+00_JPRB ,&
&  0.27207E+00_JPRB, 0.26798E+00_JPRB, 0.26380E+00_JPRB, 0.26048E+00_JPRB, 0.25730E+00_JPRB, 0.25403E+00_JPRB, 0.25097E+00_JPRB ,&
&  0.24778E+00_JPRB, 0.24462E+00_JPRB, 0.24144E+00_JPRB, 0.23940E+00_JPRB, 0.23768E+00_JPRB, 0.23608E+00_JPRB, 0.23463E+00_JPRB ,&
&  0.23403E+00_JPRB, 0.23332E+00_JPRB, 0.23242E+00_JPRB, 0.23130E+00_JPRB, 0.23025E+00_JPRB, 0.22911E+00_JPRB, 0.22820E+00_JPRB ,&
&  0.22808E+00_JPRB, 0.22751E+00_JPRB, 0.22691E+00_JPRB, 0.22613E+00_JPRB, 0.22576E+00_JPRB, 0.22496E+00_JPRB, 0.22396E+00_JPRB ,&
&  0.22305E+00_JPRB, 0.22221E+00_JPRB, 0.22203E+00_JPRB, 0.22179E+00_JPRB, 0.22153E+00_JPRB, 0.22134E+00_JPRB, 0.22175E+00_JPRB ,&
&  0.22227E+00_JPRB, 0.22290E+00_JPRB, 0.22393E+00_JPRB, 0.22516E+00_JPRB, 0.22624E+00_JPRB, 0.22731E+00_JPRB, 0.22881E+00_JPRB ,&
&  0.23079E+00_JPRB, 0.23277E+00_JPRB, 0.23499E+00_JPRB, 0.23699E+00_JPRB, 0.24042E+00_JPRB, 0.24413E+00_JPRB, 0.24822E+00_JPRB ,&
&  0.25317E+00_JPRB, 0.25999E+00_JPRB, 0.26786E+00_JPRB, 0.27496E+00_JPRB, 0.28211E+00_JPRB, 0.28975E+00_JPRB, 0.29604E+00_JPRB ,&
&  0.30152E+00_JPRB, 0.30701E+00_JPRB, 0.31249E+00_JPRB, 0.31852E+00_JPRB, 0.32503E+00_JPRB /)
ZBURD(:, 72,  2) = (/ &
&  0.28891E+00_JPRB, 0.29170E+00_JPRB, 0.29404E+00_JPRB, 0.29572E+00_JPRB, 0.29591E+00_JPRB, 0.29755E+00_JPRB, 0.29787E+00_JPRB ,&
&  0.29798E+00_JPRB, 0.29779E+00_JPRB, 0.29854E+00_JPRB, 0.29934E+00_JPRB, 0.29986E+00_JPRB, 0.30014E+00_JPRB, 0.30017E+00_JPRB ,&
&  0.29966E+00_JPRB, 0.29867E+00_JPRB, 0.29474E+00_JPRB, 0.29256E+00_JPRB, 0.29126E+00_JPRB, 0.29063E+00_JPRB, 0.29030E+00_JPRB ,&
&  0.28950E+00_JPRB, 0.28856E+00_JPRB, 0.28703E+00_JPRB, 0.28683E+00_JPRB, 0.28602E+00_JPRB, 0.28497E+00_JPRB, 0.28347E+00_JPRB ,&
&  0.28198E+00_JPRB, 0.28079E+00_JPRB, 0.27934E+00_JPRB, 0.27784E+00_JPRB, 0.27604E+00_JPRB, 0.27383E+00_JPRB, 0.27106E+00_JPRB ,&
&  0.26820E+00_JPRB, 0.26529E+00_JPRB, 0.26288E+00_JPRB, 0.26065E+00_JPRB, 0.25871E+00_JPRB, 0.25716E+00_JPRB, 0.25604E+00_JPRB ,&
&  0.25480E+00_JPRB, 0.25339E+00_JPRB, 0.25196E+00_JPRB, 0.25075E+00_JPRB, 0.24957E+00_JPRB, 0.24853E+00_JPRB, 0.24801E+00_JPRB ,&
&  0.24771E+00_JPRB, 0.24735E+00_JPRB, 0.24690E+00_JPRB, 0.24651E+00_JPRB, 0.24600E+00_JPRB, 0.24554E+00_JPRB, 0.24506E+00_JPRB ,&
&  0.24467E+00_JPRB, 0.24444E+00_JPRB, 0.24408E+00_JPRB, 0.24371E+00_JPRB, 0.24347E+00_JPRB, 0.24308E+00_JPRB, 0.24264E+00_JPRB ,&
&  0.24213E+00_JPRB, 0.24185E+00_JPRB, 0.24153E+00_JPRB, 0.24113E+00_JPRB, 0.24075E+00_JPRB, 0.24050E+00_JPRB, 0.24051E+00_JPRB ,&
&  0.24048E+00_JPRB, 0.24038E+00_JPRB, 0.24044E+00_JPRB, 0.24119E+00_JPRB, 0.24198E+00_JPRB, 0.24293E+00_JPRB, 0.24400E+00_JPRB ,&
&  0.24540E+00_JPRB, 0.24686E+00_JPRB, 0.24849E+00_JPRB, 0.25006E+00_JPRB, 0.25214E+00_JPRB, 0.25421E+00_JPRB, 0.25627E+00_JPRB ,&
&  0.25825E+00_JPRB, 0.26029E+00_JPRB, 0.26234E+00_JPRB, 0.26443E+00_JPRB, 0.26717E+00_JPRB, 0.26949E+00_JPRB, 0.27211E+00_JPRB ,&
&  0.27460E+00_JPRB, 0.27739E+00_JPRB, 0.28021E+00_JPRB, 0.28271E+00_JPRB, 0.28524E+00_JPRB /)
ZBURD(:, 73,  2) = (/ &
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB ,&
&  0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB, 0.26327E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART4',1,ZHOOK_HANDLE)
END SUBROUTINE PART4



SUBROUTINE PART5

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART5',0,ZHOOK_HANDLE)


!-- month=  3
ZBURD(:,  1,  3) = (/ &
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB ,&
&  0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB, 0.84226E+00_JPRB /)
ZBURD(:,  2,  3) = (/ &
&  0.86202E+00_JPRB, 0.86038E+00_JPRB, 0.85899E+00_JPRB, 0.85804E+00_JPRB, 0.85609E+00_JPRB, 0.85337E+00_JPRB, 0.84999E+00_JPRB ,&
&  0.84622E+00_JPRB, 0.84234E+00_JPRB, 0.83745E+00_JPRB, 0.83257E+00_JPRB, 0.82778E+00_JPRB, 0.82319E+00_JPRB, 0.81788E+00_JPRB ,&
&  0.81232E+00_JPRB, 0.80606E+00_JPRB, 0.79966E+00_JPRB, 0.79299E+00_JPRB, 0.78748E+00_JPRB, 0.78296E+00_JPRB, 0.77967E+00_JPRB ,&
&  0.77662E+00_JPRB, 0.77364E+00_JPRB, 0.77060E+00_JPRB, 0.76860E+00_JPRB, 0.76798E+00_JPRB, 0.76710E+00_JPRB, 0.76640E+00_JPRB ,&
&  0.76605E+00_JPRB, 0.76647E+00_JPRB, 0.76654E+00_JPRB, 0.76638E+00_JPRB, 0.76666E+00_JPRB, 0.76863E+00_JPRB, 0.77027E+00_JPRB ,&
&  0.77221E+00_JPRB, 0.77379E+00_JPRB, 0.77501E+00_JPRB, 0.77617E+00_JPRB, 0.77768E+00_JPRB, 0.77968E+00_JPRB, 0.78173E+00_JPRB ,&
&  0.78399E+00_JPRB, 0.78687E+00_JPRB, 0.79101E+00_JPRB, 0.79579E+00_JPRB, 0.80045E+00_JPRB, 0.80519E+00_JPRB, 0.81083E+00_JPRB ,&
&  0.81641E+00_JPRB, 0.82179E+00_JPRB, 0.82732E+00_JPRB, 0.83318E+00_JPRB, 0.83929E+00_JPRB, 0.84524E+00_JPRB, 0.85136E+00_JPRB ,&
&  0.85751E+00_JPRB, 0.86370E+00_JPRB, 0.87039E+00_JPRB, 0.87716E+00_JPRB, 0.88347E+00_JPRB, 0.88886E+00_JPRB, 0.89465E+00_JPRB ,&
&  0.90077E+00_JPRB, 0.90690E+00_JPRB, 0.91153E+00_JPRB, 0.91658E+00_JPRB, 0.92169E+00_JPRB, 0.92658E+00_JPRB, 0.93010E+00_JPRB ,&
&  0.93381E+00_JPRB, 0.93796E+00_JPRB, 0.94229E+00_JPRB, 0.94367E+00_JPRB, 0.94464E+00_JPRB, 0.94519E+00_JPRB, 0.94462E+00_JPRB ,&
&  0.94251E+00_JPRB, 0.94008E+00_JPRB, 0.93747E+00_JPRB, 0.93410E+00_JPRB, 0.92929E+00_JPRB, 0.92401E+00_JPRB, 0.91804E+00_JPRB ,&
&  0.91185E+00_JPRB, 0.90446E+00_JPRB, 0.89690E+00_JPRB, 0.89058E+00_JPRB, 0.88569E+00_JPRB, 0.88087E+00_JPRB, 0.87590E+00_JPRB ,&
&  0.87092E+00_JPRB, 0.86796E+00_JPRB, 0.86680E+00_JPRB, 0.86558E+00_JPRB, 0.86422E+00_JPRB /)
ZBURD(:,  3,  3) = (/ &
&  0.85335E+00_JPRB, 0.85219E+00_JPRB, 0.85087E+00_JPRB, 0.84834E+00_JPRB, 0.84284E+00_JPRB, 0.83753E+00_JPRB, 0.83188E+00_JPRB ,&
&  0.82712E+00_JPRB, 0.82267E+00_JPRB, 0.81607E+00_JPRB, 0.80866E+00_JPRB, 0.80137E+00_JPRB, 0.79519E+00_JPRB, 0.78871E+00_JPRB ,&
&  0.78248E+00_JPRB, 0.77590E+00_JPRB, 0.76964E+00_JPRB, 0.75948E+00_JPRB, 0.74885E+00_JPRB, 0.73881E+00_JPRB, 0.73212E+00_JPRB ,&
&  0.72669E+00_JPRB, 0.72187E+00_JPRB, 0.71645E+00_JPRB, 0.71138E+00_JPRB, 0.70695E+00_JPRB, 0.70317E+00_JPRB, 0.69921E+00_JPRB ,&
&  0.69720E+00_JPRB, 0.69646E+00_JPRB, 0.69591E+00_JPRB, 0.69503E+00_JPRB, 0.69656E+00_JPRB, 0.69926E+00_JPRB, 0.70193E+00_JPRB ,&
&  0.70431E+00_JPRB, 0.70795E+00_JPRB, 0.71386E+00_JPRB, 0.71932E+00_JPRB, 0.72431E+00_JPRB, 0.73096E+00_JPRB, 0.73584E+00_JPRB ,&
&  0.73888E+00_JPRB, 0.74286E+00_JPRB, 0.75047E+00_JPRB, 0.75805E+00_JPRB, 0.76570E+00_JPRB, 0.77353E+00_JPRB, 0.78336E+00_JPRB ,&
&  0.79252E+00_JPRB, 0.80217E+00_JPRB, 0.81252E+00_JPRB, 0.82401E+00_JPRB, 0.83633E+00_JPRB, 0.84926E+00_JPRB, 0.86223E+00_JPRB ,&
&  0.87445E+00_JPRB, 0.88670E+00_JPRB, 0.89880E+00_JPRB, 0.91081E+00_JPRB, 0.92229E+00_JPRB, 0.93205E+00_JPRB, 0.94254E+00_JPRB ,&
&  0.95354E+00_JPRB, 0.96692E+00_JPRB, 0.97593E+00_JPRB, 0.98578E+00_JPRB, 0.99641E+00_JPRB, 0.10114E+01_JPRB, 0.10176E+01_JPRB ,&
&  0.10230E+01_JPRB, 0.10279E+01_JPRB, 0.10332E+01_JPRB, 0.10351E+01_JPRB, 0.10368E+01_JPRB, 0.10381E+01_JPRB, 0.10378E+01_JPRB ,&
&  0.10361E+01_JPRB, 0.10328E+01_JPRB, 0.10277E+01_JPRB, 0.10194E+01_JPRB, 0.10101E+01_JPRB, 0.99998E+00_JPRB, 0.98755E+00_JPRB ,&
&  0.97539E+00_JPRB, 0.96053E+00_JPRB, 0.94443E+00_JPRB, 0.93068E+00_JPRB, 0.91960E+00_JPRB, 0.90682E+00_JPRB, 0.89404E+00_JPRB ,&
&  0.88210E+00_JPRB, 0.87201E+00_JPRB, 0.86605E+00_JPRB, 0.86063E+00_JPRB, 0.85712E+00_JPRB /)
ZBURD(:,  4,  3) = (/ &
&  0.84670E+00_JPRB, 0.84336E+00_JPRB, 0.83858E+00_JPRB, 0.83224E+00_JPRB, 0.82294E+00_JPRB, 0.81426E+00_JPRB, 0.80583E+00_JPRB ,&
&  0.79876E+00_JPRB, 0.79269E+00_JPRB, 0.78745E+00_JPRB, 0.78254E+00_JPRB, 0.77785E+00_JPRB, 0.77400E+00_JPRB, 0.76991E+00_JPRB ,&
&  0.76649E+00_JPRB, 0.76230E+00_JPRB, 0.75785E+00_JPRB, 0.75150E+00_JPRB, 0.74131E+00_JPRB, 0.72630E+00_JPRB, 0.71094E+00_JPRB ,&
&  0.69643E+00_JPRB, 0.68480E+00_JPRB, 0.67417E+00_JPRB, 0.66389E+00_JPRB, 0.65623E+00_JPRB, 0.60768E+00_JPRB, 0.59358E+00_JPRB ,&
&  0.59088E+00_JPRB, 0.59202E+00_JPRB, 0.59852E+00_JPRB, 0.60778E+00_JPRB, 0.62498E+00_JPRB, 0.61910E+00_JPRB, 0.61664E+00_JPRB ,&
&  0.58489E+00_JPRB, 0.58480E+00_JPRB, 0.58774E+00_JPRB, 0.59333E+00_JPRB, 0.60532E+00_JPRB, 0.62076E+00_JPRB, 0.64271E+00_JPRB ,&
&  0.68487E+00_JPRB, 0.69701E+00_JPRB, 0.71033E+00_JPRB, 0.72602E+00_JPRB, 0.74103E+00_JPRB, 0.75432E+00_JPRB, 0.76887E+00_JPRB ,&
&  0.78238E+00_JPRB, 0.79681E+00_JPRB, 0.81117E+00_JPRB, 0.82713E+00_JPRB, 0.84360E+00_JPRB, 0.86053E+00_JPRB, 0.87790E+00_JPRB ,&
&  0.89481E+00_JPRB, 0.91208E+00_JPRB, 0.92900E+00_JPRB, 0.94656E+00_JPRB, 0.96403E+00_JPRB, 0.98275E+00_JPRB, 0.10044E+01_JPRB ,&
&  0.10265E+01_JPRB, 0.10505E+01_JPRB, 0.10667E+01_JPRB, 0.10831E+01_JPRB, 0.10945E+01_JPRB, 0.11088E+01_JPRB, 0.11133E+01_JPRB ,&
&  0.11183E+01_JPRB, 0.11143E+01_JPRB, 0.11138E+01_JPRB, 0.11089E+01_JPRB, 0.11059E+01_JPRB, 0.11035E+01_JPRB, 0.10977E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10863E+01_JPRB, 0.10808E+01_JPRB, 0.10718E+01_JPRB, 0.10604E+01_JPRB, 0.10454E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.10148E+01_JPRB, 0.10020E+01_JPRB, 0.98675E+00_JPRB, 0.97213E+00_JPRB, 0.95468E+00_JPRB, 0.93546E+00_JPRB, 0.91554E+00_JPRB ,&
&  0.89856E+00_JPRB, 0.88345E+00_JPRB, 0.87078E+00_JPRB, 0.85976E+00_JPRB, 0.85278E+00_JPRB /)
ZBURD(:,  5,  3) = (/ &
&  0.83704E+00_JPRB, 0.83254E+00_JPRB, 0.82628E+00_JPRB, 0.82068E+00_JPRB, 0.81086E+00_JPRB, 0.80237E+00_JPRB, 0.79370E+00_JPRB ,&
&  0.78552E+00_JPRB, 0.77833E+00_JPRB, 0.77061E+00_JPRB, 0.76494E+00_JPRB, 0.75792E+00_JPRB, 0.75332E+00_JPRB, 0.74655E+00_JPRB ,&
&  0.74315E+00_JPRB, 0.73784E+00_JPRB, 0.73208E+00_JPRB, 0.72232E+00_JPRB, 0.71222E+00_JPRB, 0.70041E+00_JPRB, 0.68171E+00_JPRB ,&
&  0.65901E+00_JPRB, 0.63930E+00_JPRB, 0.59991E+00_JPRB, 0.58587E+00_JPRB, 0.56534E+00_JPRB, 0.55371E+00_JPRB, 0.55114E+00_JPRB ,&
&  0.54700E+00_JPRB, 0.54459E+00_JPRB, 0.53634E+00_JPRB, 0.52562E+00_JPRB, 0.51239E+00_JPRB, 0.49304E+00_JPRB, 0.48031E+00_JPRB ,&
&  0.46789E+00_JPRB, 0.46298E+00_JPRB, 0.46535E+00_JPRB, 0.47485E+00_JPRB, 0.49391E+00_JPRB, 0.52111E+00_JPRB, 0.55456E+00_JPRB ,&
&  0.60154E+00_JPRB, 0.64566E+00_JPRB, 0.69405E+00_JPRB, 0.71118E+00_JPRB, 0.72706E+00_JPRB, 0.74490E+00_JPRB, 0.76159E+00_JPRB ,&
&  0.76459E+00_JPRB, 0.77437E+00_JPRB, 0.80706E+00_JPRB, 0.82544E+00_JPRB, 0.84252E+00_JPRB, 0.85837E+00_JPRB, 0.87430E+00_JPRB ,&
&  0.88915E+00_JPRB, 0.90506E+00_JPRB, 0.92134E+00_JPRB, 0.94162E+00_JPRB, 0.97184E+00_JPRB, 0.10085E+01_JPRB, 0.10428E+01_JPRB ,&
&  0.10733E+01_JPRB, 0.11069E+01_JPRB, 0.11315E+01_JPRB, 0.11546E+01_JPRB, 0.11769E+01_JPRB, 0.12056E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.12407E+01_JPRB, 0.12341E+01_JPRB, 0.12147E+01_JPRB, 0.11799E+01_JPRB, 0.11620E+01_JPRB, 0.11723E+01_JPRB, 0.11648E+01_JPRB ,&
&  0.11593E+01_JPRB, 0.11459E+01_JPRB, 0.11360E+01_JPRB, 0.11219E+01_JPRB, 0.11077E+01_JPRB, 0.10903E+01_JPRB, 0.10666E+01_JPRB ,&
&  0.10452E+01_JPRB, 0.10278E+01_JPRB, 0.10079E+01_JPRB, 0.98834E+00_JPRB, 0.96358E+00_JPRB, 0.93883E+00_JPRB, 0.91599E+00_JPRB ,&
&  0.89650E+00_JPRB, 0.87966E+00_JPRB, 0.86421E+00_JPRB, 0.85226E+00_JPRB, 0.84430E+00_JPRB /)
ZBURD(:,  6,  3) = (/ &
&  0.82910E+00_JPRB, 0.82216E+00_JPRB, 0.81124E+00_JPRB, 0.80173E+00_JPRB, 0.79044E+00_JPRB, 0.78074E+00_JPRB, 0.76985E+00_JPRB ,&
&  0.75994E+00_JPRB, 0.75005E+00_JPRB, 0.74066E+00_JPRB, 0.73260E+00_JPRB, 0.72443E+00_JPRB, 0.71791E+00_JPRB, 0.71259E+00_JPRB ,&
&  0.70860E+00_JPRB, 0.70286E+00_JPRB, 0.69546E+00_JPRB, 0.68702E+00_JPRB, 0.67962E+00_JPRB, 0.67155E+00_JPRB, 0.66177E+00_JPRB ,&
&  0.65157E+00_JPRB, 0.63655E+00_JPRB, 0.61171E+00_JPRB, 0.59594E+00_JPRB, 0.57947E+00_JPRB, 0.57529E+00_JPRB, 0.56838E+00_JPRB ,&
&  0.57696E+00_JPRB, 0.52794E+00_JPRB, 0.51683E+00_JPRB, 0.49263E+00_JPRB, 0.47259E+00_JPRB, 0.45776E+00_JPRB, 0.44013E+00_JPRB ,&
&  0.43213E+00_JPRB, 0.42411E+00_JPRB, 0.42254E+00_JPRB, 0.42990E+00_JPRB, 0.44207E+00_JPRB, 0.46916E+00_JPRB, 0.50372E+00_JPRB ,&
&  0.55219E+00_JPRB, 0.64357E+00_JPRB, 0.67046E+00_JPRB, 0.69420E+00_JPRB, 0.71242E+00_JPRB, 0.72784E+00_JPRB, 0.71550E+00_JPRB ,&
&  0.72093E+00_JPRB, 0.73009E+00_JPRB, 0.75414E+00_JPRB, 0.78509E+00_JPRB, 0.81353E+00_JPRB, 0.82874E+00_JPRB, 0.84300E+00_JPRB ,&
&  0.85336E+00_JPRB, 0.86834E+00_JPRB, 0.88308E+00_JPRB, 0.91038E+00_JPRB, 0.95963E+00_JPRB, 0.10141E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.11326E+01_JPRB, 0.11555E+01_JPRB, 0.11875E+01_JPRB, 0.12301E+01_JPRB, 0.12825E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.13301E+01_JPRB, 0.13396E+01_JPRB, 0.13345E+01_JPRB, 0.13214E+01_JPRB, 0.12867E+01_JPRB, 0.12523E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.12220E+01_JPRB, 0.11998E+01_JPRB, 0.11657E+01_JPRB, 0.11389E+01_JPRB, 0.11099E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10334E+01_JPRB, 0.10066E+01_JPRB, 0.98281E+00_JPRB, 0.95460E+00_JPRB, 0.92799E+00_JPRB, 0.90809E+00_JPRB ,&
&  0.88910E+00_JPRB, 0.87381E+00_JPRB, 0.85804E+00_JPRB, 0.84696E+00_JPRB, 0.83814E+00_JPRB /)
ZBURD(:,  7,  3) = (/ &
&  0.81082E+00_JPRB, 0.79903E+00_JPRB, 0.78653E+00_JPRB, 0.77675E+00_JPRB, 0.76809E+00_JPRB, 0.76175E+00_JPRB, 0.75624E+00_JPRB ,&
&  0.74991E+00_JPRB, 0.74303E+00_JPRB, 0.73603E+00_JPRB, 0.72855E+00_JPRB, 0.72078E+00_JPRB, 0.71395E+00_JPRB, 0.70817E+00_JPRB ,&
&  0.70094E+00_JPRB, 0.69351E+00_JPRB, 0.68475E+00_JPRB, 0.67701E+00_JPRB, 0.67022E+00_JPRB, 0.66750E+00_JPRB, 0.66367E+00_JPRB ,&
&  0.65945E+00_JPRB, 0.65112E+00_JPRB, 0.63558E+00_JPRB, 0.62226E+00_JPRB, 0.61809E+00_JPRB, 0.60911E+00_JPRB, 0.61017E+00_JPRB ,&
&  0.60320E+00_JPRB, 0.58967E+00_JPRB, 0.57042E+00_JPRB, 0.56519E+00_JPRB, 0.55734E+00_JPRB, 0.48413E+00_JPRB, 0.46289E+00_JPRB ,&
&  0.43665E+00_JPRB, 0.41265E+00_JPRB, 0.40446E+00_JPRB, 0.40432E+00_JPRB, 0.41147E+00_JPRB, 0.43556E+00_JPRB, 0.46949E+00_JPRB ,&
&  0.51076E+00_JPRB, 0.61337E+00_JPRB, 0.64041E+00_JPRB, 0.66432E+00_JPRB, 0.67661E+00_JPRB, 0.68166E+00_JPRB, 0.67321E+00_JPRB ,&
&  0.67302E+00_JPRB, 0.67883E+00_JPRB, 0.70454E+00_JPRB, 0.74223E+00_JPRB, 0.77520E+00_JPRB, 0.78552E+00_JPRB, 0.78627E+00_JPRB ,&
&  0.78392E+00_JPRB, 0.80044E+00_JPRB, 0.82815E+00_JPRB, 0.86651E+00_JPRB, 0.93023E+00_JPRB, 0.97831E+00_JPRB, 0.10401E+01_JPRB ,&
&  0.10947E+01_JPRB, 0.11317E+01_JPRB, 0.11954E+01_JPRB, 0.12738E+01_JPRB, 0.13315E+01_JPRB, 0.13722E+01_JPRB, 0.13818E+01_JPRB ,&
&  0.13767E+01_JPRB, 0.14000E+01_JPRB, 0.14028E+01_JPRB, 0.14056E+01_JPRB, 0.13917E+01_JPRB, 0.13561E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.12830E+01_JPRB, 0.12921E+01_JPRB, 0.12729E+01_JPRB, 0.12117E+01_JPRB, 0.11552E+01_JPRB, 0.11132E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10610E+01_JPRB, 0.10444E+01_JPRB, 0.10160E+01_JPRB, 0.98881E+00_JPRB, 0.96033E+00_JPRB, 0.93127E+00_JPRB, 0.91086E+00_JPRB ,&
&  0.89160E+00_JPRB, 0.87722E+00_JPRB, 0.85905E+00_JPRB, 0.84223E+00_JPRB, 0.82788E+00_JPRB /)
ZBURD(:,  8,  3) = (/ &
&  0.78874E+00_JPRB, 0.77496E+00_JPRB, 0.76050E+00_JPRB, 0.74826E+00_JPRB, 0.73972E+00_JPRB, 0.73379E+00_JPRB, 0.72658E+00_JPRB ,&
&  0.71797E+00_JPRB, 0.71149E+00_JPRB, 0.70256E+00_JPRB, 0.69766E+00_JPRB, 0.69160E+00_JPRB, 0.68520E+00_JPRB, 0.68010E+00_JPRB ,&
&  0.67412E+00_JPRB, 0.65980E+00_JPRB, 0.65257E+00_JPRB, 0.65347E+00_JPRB, 0.65754E+00_JPRB, 0.66216E+00_JPRB, 0.66670E+00_JPRB ,&
&  0.66443E+00_JPRB, 0.65567E+00_JPRB, 0.64724E+00_JPRB, 0.63326E+00_JPRB, 0.62424E+00_JPRB, 0.62266E+00_JPRB, 0.62691E+00_JPRB ,&
&  0.62696E+00_JPRB, 0.60684E+00_JPRB, 0.60390E+00_JPRB, 0.59987E+00_JPRB, 0.58386E+00_JPRB, 0.57603E+00_JPRB, 0.50134E+00_JPRB ,&
&  0.45138E+00_JPRB, 0.41727E+00_JPRB, 0.40384E+00_JPRB, 0.39637E+00_JPRB, 0.40605E+00_JPRB, 0.43139E+00_JPRB, 0.48249E+00_JPRB ,&
&  0.53768E+00_JPRB, 0.59392E+00_JPRB, 0.60698E+00_JPRB, 0.62165E+00_JPRB, 0.62654E+00_JPRB, 0.64134E+00_JPRB, 0.65215E+00_JPRB ,&
&  0.66319E+00_JPRB, 0.67468E+00_JPRB, 0.70073E+00_JPRB, 0.72912E+00_JPRB, 0.78354E+00_JPRB, 0.83477E+00_JPRB, 0.86891E+00_JPRB ,&
&  0.90371E+00_JPRB, 0.92100E+00_JPRB, 0.95547E+00_JPRB, 0.99345E+00_JPRB, 0.10471E+01_JPRB, 0.10802E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.12242E+01_JPRB, 0.13430E+01_JPRB, 0.14779E+01_JPRB, 0.15240E+01_JPRB, 0.15324E+01_JPRB, 0.14871E+01_JPRB ,&
&  0.14178E+01_JPRB, 0.14392E+01_JPRB, 0.16079E+01_JPRB, 0.15973E+01_JPRB, 0.15910E+01_JPRB, 0.15763E+01_JPRB, 0.15045E+01_JPRB ,&
&  0.14016E+01_JPRB, 0.13582E+01_JPRB, 0.13082E+01_JPRB, 0.12161E+01_JPRB, 0.11338E+01_JPRB, 0.10880E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.10575E+01_JPRB, 0.10522E+01_JPRB, 0.10273E+01_JPRB, 0.10040E+01_JPRB, 0.97454E+00_JPRB, 0.94144E+00_JPRB, 0.91632E+00_JPRB ,&
&  0.88924E+00_JPRB, 0.86579E+00_JPRB, 0.84467E+00_JPRB, 0.82472E+00_JPRB, 0.80704E+00_JPRB /)
ZBURD(:,  9,  3) = (/ &
&  0.77109E+00_JPRB, 0.75995E+00_JPRB, 0.74976E+00_JPRB, 0.74654E+00_JPRB, 0.74677E+00_JPRB, 0.73569E+00_JPRB, 0.71991E+00_JPRB ,&
&  0.69717E+00_JPRB, 0.67495E+00_JPRB, 0.65478E+00_JPRB, 0.64070E+00_JPRB, 0.63245E+00_JPRB, 0.62832E+00_JPRB, 0.62599E+00_JPRB ,&
&  0.62580E+00_JPRB, 0.62758E+00_JPRB, 0.62817E+00_JPRB, 0.62604E+00_JPRB, 0.62999E+00_JPRB, 0.63795E+00_JPRB, 0.64691E+00_JPRB ,&
&  0.65447E+00_JPRB, 0.65398E+00_JPRB, 0.64849E+00_JPRB, 0.64157E+00_JPRB, 0.63077E+00_JPRB, 0.64144E+00_JPRB, 0.63106E+00_JPRB ,&
&  0.61709E+00_JPRB, 0.61342E+00_JPRB, 0.62434E+00_JPRB, 0.62525E+00_JPRB, 0.61982E+00_JPRB, 0.59972E+00_JPRB, 0.53911E+00_JPRB ,&
&  0.47825E+00_JPRB, 0.43094E+00_JPRB, 0.40573E+00_JPRB, 0.39941E+00_JPRB, 0.41184E+00_JPRB, 0.44520E+00_JPRB, 0.49914E+00_JPRB ,&
&  0.57081E+00_JPRB, 0.58902E+00_JPRB, 0.60127E+00_JPRB, 0.61721E+00_JPRB, 0.63340E+00_JPRB, 0.66126E+00_JPRB, 0.69357E+00_JPRB ,&
&  0.73543E+00_JPRB, 0.79223E+00_JPRB, 0.86858E+00_JPRB, 0.97017E+00_JPRB, 0.10846E+01_JPRB, 0.10988E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.14476E+01_JPRB, 0.14759E+01_JPRB, 0.14542E+01_JPRB, 0.13813E+01_JPRB, 0.13390E+01_JPRB, 0.13327E+01_JPRB, 0.13632E+01_JPRB ,&
&  0.14058E+01_JPRB, 0.14730E+01_JPRB, 0.15648E+01_JPRB, 0.16496E+01_JPRB, 0.16862E+01_JPRB, 0.16936E+01_JPRB, 0.16313E+01_JPRB ,&
&  0.15639E+01_JPRB, 0.15836E+01_JPRB, 0.18628E+01_JPRB, 0.16841E+01_JPRB, 0.15600E+01_JPRB, 0.15175E+01_JPRB, 0.14484E+01_JPRB ,&
&  0.14061E+01_JPRB, 0.13595E+01_JPRB, 0.12839E+01_JPRB, 0.12137E+01_JPRB, 0.11277E+01_JPRB, 0.10557E+01_JPRB, 0.99478E+00_JPRB ,&
&  0.97832E+00_JPRB, 0.96878E+00_JPRB, 0.95472E+00_JPRB, 0.94407E+00_JPRB, 0.92101E+00_JPRB, 0.90302E+00_JPRB, 0.88487E+00_JPRB ,&
&  0.85243E+00_JPRB, 0.82360E+00_JPRB, 0.80840E+00_JPRB, 0.79600E+00_JPRB, 0.78333E+00_JPRB /)
ZBURD(:, 10,  3) = (/ &
&  0.69306E+00_JPRB, 0.70863E+00_JPRB, 0.74173E+00_JPRB, 0.75774E+00_JPRB, 0.75724E+00_JPRB, 0.72096E+00_JPRB, 0.68204E+00_JPRB ,&
&  0.64595E+00_JPRB, 0.61100E+00_JPRB, 0.58921E+00_JPRB, 0.57765E+00_JPRB, 0.57202E+00_JPRB, 0.56880E+00_JPRB, 0.56807E+00_JPRB ,&
&  0.56978E+00_JPRB, 0.57140E+00_JPRB, 0.57809E+00_JPRB, 0.59386E+00_JPRB, 0.60859E+00_JPRB, 0.61456E+00_JPRB, 0.62111E+00_JPRB ,&
&  0.63187E+00_JPRB, 0.64122E+00_JPRB, 0.64185E+00_JPRB, 0.63318E+00_JPRB, 0.63172E+00_JPRB, 0.65342E+00_JPRB, 0.66153E+00_JPRB ,&
&  0.65820E+00_JPRB, 0.63988E+00_JPRB, 0.64020E+00_JPRB, 0.65879E+00_JPRB, 0.66011E+00_JPRB, 0.64665E+00_JPRB, 0.58765E+00_JPRB ,&
&  0.51205E+00_JPRB, 0.45145E+00_JPRB, 0.43053E+00_JPRB, 0.44906E+00_JPRB, 0.49376E+00_JPRB, 0.56633E+00_JPRB, 0.58822E+00_JPRB ,&
&  0.60340E+00_JPRB, 0.61120E+00_JPRB, 0.62245E+00_JPRB, 0.64998E+00_JPRB, 0.69299E+00_JPRB, 0.73938E+00_JPRB, 0.79102E+00_JPRB ,&
&  0.86602E+00_JPRB, 0.94409E+00_JPRB, 0.10439E+01_JPRB, 0.10265E+01_JPRB, 0.10930E+01_JPRB, 0.12311E+01_JPRB, 0.13208E+01_JPRB ,&
&  0.13784E+01_JPRB, 0.14623E+01_JPRB, 0.15345E+01_JPRB, 0.16262E+01_JPRB, 0.16257E+01_JPRB, 0.16118E+01_JPRB, 0.15895E+01_JPRB ,&
&  0.16128E+01_JPRB, 0.16329E+01_JPRB, 0.16548E+01_JPRB, 0.17416E+01_JPRB, 0.18566E+01_JPRB, 0.18901E+01_JPRB, 0.18381E+01_JPRB ,&
&  0.17513E+01_JPRB, 0.16027E+01_JPRB, 0.14406E+01_JPRB, 0.13200E+01_JPRB, 0.12998E+01_JPRB, 0.13382E+01_JPRB, 0.13699E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.13671E+01_JPRB, 0.13115E+01_JPRB, 0.12635E+01_JPRB, 0.11265E+01_JPRB, 0.10280E+01_JPRB, 0.95253E+00_JPRB ,&
&  0.92449E+00_JPRB, 0.89806E+00_JPRB, 0.87214E+00_JPRB, 0.88046E+00_JPRB, 0.85445E+00_JPRB, 0.83395E+00_JPRB, 0.81236E+00_JPRB ,&
&  0.78480E+00_JPRB, 0.74911E+00_JPRB, 0.72641E+00_JPRB, 0.70680E+00_JPRB, 0.69584E+00_JPRB /)
ZBURD(:, 11,  3) = (/ &
&  0.76805E+00_JPRB, 0.77389E+00_JPRB, 0.77825E+00_JPRB, 0.77653E+00_JPRB, 0.74851E+00_JPRB, 0.72459E+00_JPRB, 0.70429E+00_JPRB ,&
&  0.69255E+00_JPRB, 0.63606E+00_JPRB, 0.59292E+00_JPRB, 0.56415E+00_JPRB, 0.54420E+00_JPRB, 0.52506E+00_JPRB, 0.51997E+00_JPRB ,&
&  0.52456E+00_JPRB, 0.53487E+00_JPRB, 0.55168E+00_JPRB, 0.57101E+00_JPRB, 0.58756E+00_JPRB, 0.59484E+00_JPRB, 0.60699E+00_JPRB ,&
&  0.62764E+00_JPRB, 0.64861E+00_JPRB, 0.66462E+00_JPRB, 0.68131E+00_JPRB, 0.69357E+00_JPRB, 0.71693E+00_JPRB, 0.72607E+00_JPRB ,&
&  0.72560E+00_JPRB, 0.71430E+00_JPRB, 0.70951E+00_JPRB, 0.73301E+00_JPRB, 0.74900E+00_JPRB, 0.75290E+00_JPRB, 0.67384E+00_JPRB ,&
&  0.54583E+00_JPRB, 0.49331E+00_JPRB, 0.50169E+00_JPRB, 0.57460E+00_JPRB, 0.59831E+00_JPRB, 0.61672E+00_JPRB, 0.63556E+00_JPRB ,&
&  0.62331E+00_JPRB, 0.61083E+00_JPRB, 0.63095E+00_JPRB, 0.69456E+00_JPRB, 0.73137E+00_JPRB, 0.77473E+00_JPRB, 0.83555E+00_JPRB ,&
&  0.93798E+00_JPRB, 0.10237E+01_JPRB, 0.11273E+01_JPRB, 0.10941E+01_JPRB, 0.12606E+01_JPRB, 0.16000E+01_JPRB, 0.15759E+01_JPRB ,&
&  0.15833E+01_JPRB, 0.16157E+01_JPRB, 0.17275E+01_JPRB, 0.18103E+01_JPRB, 0.18400E+01_JPRB, 0.18436E+01_JPRB, 0.18559E+01_JPRB ,&
&  0.18550E+01_JPRB, 0.18643E+01_JPRB, 0.18902E+01_JPRB, 0.20214E+01_JPRB, 0.20596E+01_JPRB, 0.20308E+01_JPRB, 0.19867E+01_JPRB ,&
&  0.18974E+01_JPRB, 0.17453E+01_JPRB, 0.15946E+01_JPRB, 0.14710E+01_JPRB, 0.14300E+01_JPRB, 0.14310E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.13873E+01_JPRB, 0.13709E+01_JPRB, 0.13572E+01_JPRB, 0.13386E+01_JPRB, 0.12047E+01_JPRB, 0.10508E+01_JPRB, 0.93532E+00_JPRB ,&
&  0.85750E+00_JPRB, 0.80445E+00_JPRB, 0.75613E+00_JPRB, 0.75467E+00_JPRB, 0.76099E+00_JPRB, 0.76884E+00_JPRB, 0.74226E+00_JPRB ,&
&  0.72523E+00_JPRB, 0.70728E+00_JPRB, 0.70450E+00_JPRB, 0.73127E+00_JPRB, 0.75333E+00_JPRB /)
ZBURD(:, 12,  3) = (/ &
&  0.79557E+00_JPRB, 0.79795E+00_JPRB, 0.79935E+00_JPRB, 0.77953E+00_JPRB, 0.75194E+00_JPRB, 0.72756E+00_JPRB, 0.70474E+00_JPRB ,&
&  0.69174E+00_JPRB, 0.61175E+00_JPRB, 0.56905E+00_JPRB, 0.54967E+00_JPRB, 0.53830E+00_JPRB, 0.51467E+00_JPRB, 0.49623E+00_JPRB ,&
&  0.49844E+00_JPRB, 0.52036E+00_JPRB, 0.56095E+00_JPRB, 0.59288E+00_JPRB, 0.62302E+00_JPRB, 0.64760E+00_JPRB, 0.65783E+00_JPRB ,&
&  0.66897E+00_JPRB, 0.68543E+00_JPRB, 0.70533E+00_JPRB, 0.73197E+00_JPRB, 0.76242E+00_JPRB, 0.81288E+00_JPRB, 0.87826E+00_JPRB ,&
&  0.93246E+00_JPRB, 0.95572E+00_JPRB, 0.91825E+00_JPRB, 0.93233E+00_JPRB, 0.95700E+00_JPRB, 0.96861E+00_JPRB, 0.96540E+00_JPRB ,&
&  0.65669E+00_JPRB, 0.61009E+00_JPRB, 0.68363E+00_JPRB, 0.66770E+00_JPRB, 0.66908E+00_JPRB, 0.65881E+00_JPRB, 0.64641E+00_JPRB ,&
&  0.64317E+00_JPRB, 0.65110E+00_JPRB, 0.67766E+00_JPRB, 0.71616E+00_JPRB, 0.74891E+00_JPRB, 0.78702E+00_JPRB, 0.84878E+00_JPRB ,&
&  0.98781E+00_JPRB, 0.95790E+00_JPRB, 0.99421E+00_JPRB, 0.11247E+01_JPRB, 0.13963E+01_JPRB, 0.16258E+01_JPRB, 0.17591E+01_JPRB ,&
&  0.18776E+01_JPRB, 0.18981E+01_JPRB, 0.19238E+01_JPRB, 0.20051E+01_JPRB, 0.20557E+01_JPRB, 0.20810E+01_JPRB, 0.21552E+01_JPRB ,&
&  0.21944E+01_JPRB, 0.22530E+01_JPRB, 0.23197E+01_JPRB, 0.23938E+01_JPRB, 0.23338E+01_JPRB, 0.22763E+01_JPRB, 0.22045E+01_JPRB ,&
&  0.21119E+01_JPRB, 0.19891E+01_JPRB, 0.18269E+01_JPRB, 0.16913E+01_JPRB, 0.15867E+01_JPRB, 0.15654E+01_JPRB, 0.15480E+01_JPRB ,&
&  0.15018E+01_JPRB, 0.14681E+01_JPRB, 0.14159E+01_JPRB, 0.13601E+01_JPRB, 0.12598E+01_JPRB, 0.11585E+01_JPRB, 0.11090E+01_JPRB ,&
&  0.97371E+00_JPRB, 0.87223E+00_JPRB, 0.75327E+00_JPRB, 0.73900E+00_JPRB, 0.76794E+00_JPRB, 0.77643E+00_JPRB, 0.76990E+00_JPRB ,&
&  0.79779E+00_JPRB, 0.81272E+00_JPRB, 0.79898E+00_JPRB, 0.79702E+00_JPRB, 0.79560E+00_JPRB /)
ZBURD(:, 13,  3) = (/ &
&  0.79547E+00_JPRB, 0.78896E+00_JPRB, 0.77976E+00_JPRB, 0.76419E+00_JPRB, 0.75717E+00_JPRB, 0.74201E+00_JPRB, 0.74058E+00_JPRB ,&
&  0.75710E+00_JPRB, 0.70990E+00_JPRB, 0.75216E+00_JPRB, 0.62679E+00_JPRB, 0.55995E+00_JPRB, 0.52804E+00_JPRB, 0.51054E+00_JPRB ,&
&  0.52291E+00_JPRB, 0.56600E+00_JPRB, 0.60468E+00_JPRB, 0.63075E+00_JPRB, 0.68418E+00_JPRB, 0.69687E+00_JPRB, 0.69669E+00_JPRB ,&
&  0.70382E+00_JPRB, 0.71420E+00_JPRB, 0.76384E+00_JPRB, 0.79494E+00_JPRB, 0.86788E+00_JPRB, 0.96631E+00_JPRB, 0.11077E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.10848E+01_JPRB, 0.11850E+01_JPRB, 0.12302E+01_JPRB, 0.12626E+01_JPRB, 0.12355E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11108E+01_JPRB, 0.90964E+00_JPRB, 0.91053E+00_JPRB, 0.87286E+00_JPRB, 0.83832E+00_JPRB, 0.80270E+00_JPRB, 0.75188E+00_JPRB ,&
&  0.71537E+00_JPRB, 0.70794E+00_JPRB, 0.72274E+00_JPRB, 0.75096E+00_JPRB, 0.79726E+00_JPRB, 0.85695E+00_JPRB, 0.92130E+00_JPRB ,&
&  0.10762E+01_JPRB, 0.10247E+01_JPRB, 0.12798E+01_JPRB, 0.14149E+01_JPRB, 0.16148E+01_JPRB, 0.18758E+01_JPRB, 0.20960E+01_JPRB ,&
&  0.24381E+01_JPRB, 0.24103E+01_JPRB, 0.24387E+01_JPRB, 0.24407E+01_JPRB, 0.24178E+01_JPRB, 0.24320E+01_JPRB, 0.25066E+01_JPRB ,&
&  0.26341E+01_JPRB, 0.26437E+01_JPRB, 0.28680E+01_JPRB, 0.28343E+01_JPRB, 0.26766E+01_JPRB, 0.25706E+01_JPRB, 0.24424E+01_JPRB ,&
&  0.23278E+01_JPRB, 0.22463E+01_JPRB, 0.21554E+01_JPRB, 0.19118E+01_JPRB, 0.18499E+01_JPRB, 0.18027E+01_JPRB, 0.16492E+01_JPRB ,&
&  0.15739E+01_JPRB, 0.14618E+01_JPRB, 0.13168E+01_JPRB, 0.13223E+01_JPRB, 0.12089E+01_JPRB, 0.11170E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.10282E+01_JPRB, 0.88767E+00_JPRB, 0.86873E+00_JPRB, 0.86956E+00_JPRB, 0.82230E+00_JPRB, 0.79068E+00_JPRB, 0.83577E+00_JPRB ,&
&  0.81884E+00_JPRB, 0.82656E+00_JPRB, 0.86346E+00_JPRB, 0.83494E+00_JPRB, 0.82068E+00_JPRB /)
ZBURD(:, 14,  3) = (/ &
&  0.79424E+00_JPRB, 0.77100E+00_JPRB, 0.75591E+00_JPRB, 0.75122E+00_JPRB, 0.76289E+00_JPRB, 0.78921E+00_JPRB, 0.77283E+00_JPRB ,&
&  0.79183E+00_JPRB, 0.79601E+00_JPRB, 0.78299E+00_JPRB, 0.76218E+00_JPRB, 0.72677E+00_JPRB, 0.63467E+00_JPRB, 0.53028E+00_JPRB ,&
&  0.52410E+00_JPRB, 0.59135E+00_JPRB, 0.65029E+00_JPRB, 0.67313E+00_JPRB, 0.71934E+00_JPRB, 0.72253E+00_JPRB, 0.72204E+00_JPRB ,&
&  0.72517E+00_JPRB, 0.74315E+00_JPRB, 0.79474E+00_JPRB, 0.88835E+00_JPRB, 0.98573E+00_JPRB, 0.11229E+01_JPRB, 0.12453E+01_JPRB ,&
&  0.11510E+01_JPRB, 0.10972E+01_JPRB, 0.11239E+01_JPRB, 0.11002E+01_JPRB, 0.12358E+01_JPRB, 0.12667E+01_JPRB, 0.12454E+01_JPRB ,&
&  0.11914E+01_JPRB, 0.11197E+01_JPRB, 0.10346E+01_JPRB, 0.97946E+00_JPRB, 0.92598E+00_JPRB, 0.87784E+00_JPRB, 0.85273E+00_JPRB ,&
&  0.86012E+00_JPRB, 0.87559E+00_JPRB, 0.90756E+00_JPRB, 0.96512E+00_JPRB, 0.10340E+01_JPRB, 0.11275E+01_JPRB, 0.12619E+01_JPRB ,&
&  0.13840E+01_JPRB, 0.16031E+01_JPRB, 0.18760E+01_JPRB, 0.18601E+01_JPRB, 0.20665E+01_JPRB, 0.22773E+01_JPRB, 0.23711E+01_JPRB ,&
&  0.26008E+01_JPRB, 0.26303E+01_JPRB, 0.27945E+01_JPRB, 0.29052E+01_JPRB, 0.30012E+01_JPRB, 0.30433E+01_JPRB, 0.30221E+01_JPRB ,&
&  0.30407E+01_JPRB, 0.32509E+01_JPRB, 0.33276E+01_JPRB, 0.32252E+01_JPRB, 0.29999E+01_JPRB, 0.28415E+01_JPRB, 0.26614E+01_JPRB ,&
&  0.25794E+01_JPRB, 0.25626E+01_JPRB, 0.24785E+01_JPRB, 0.23765E+01_JPRB, 0.22072E+01_JPRB, 0.19910E+01_JPRB, 0.18066E+01_JPRB ,&
&  0.16121E+01_JPRB, 0.13508E+01_JPRB, 0.12066E+01_JPRB, 0.11635E+01_JPRB, 0.11402E+01_JPRB, 0.10321E+01_JPRB, 0.10039E+01_JPRB ,&
&  0.10058E+01_JPRB, 0.95385E+00_JPRB, 0.99248E+00_JPRB, 0.95172E+00_JPRB, 0.90560E+00_JPRB, 0.86514E+00_JPRB, 0.80760E+00_JPRB ,&
&  0.75456E+00_JPRB, 0.80242E+00_JPRB, 0.82612E+00_JPRB, 0.82776E+00_JPRB, 0.81668E+00_JPRB /)
ZBURD(:, 15,  3) = (/ &
&  0.79939E+00_JPRB, 0.77783E+00_JPRB, 0.74753E+00_JPRB, 0.74404E+00_JPRB, 0.77315E+00_JPRB, 0.80215E+00_JPRB, 0.81736E+00_JPRB ,&
&  0.80800E+00_JPRB, 0.80544E+00_JPRB, 0.79511E+00_JPRB, 0.78138E+00_JPRB, 0.75527E+00_JPRB, 0.72429E+00_JPRB, 0.66125E+00_JPRB ,&
&  0.60041E+00_JPRB, 0.73701E+00_JPRB, 0.69768E+00_JPRB, 0.71650E+00_JPRB, 0.81409E+00_JPRB, 0.78688E+00_JPRB, 0.77434E+00_JPRB ,&
&  0.78967E+00_JPRB, 0.81092E+00_JPRB, 0.88017E+00_JPRB, 0.95811E+00_JPRB, 0.10803E+01_JPRB, 0.12903E+01_JPRB, 0.12898E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.10396E+01_JPRB, 0.10702E+01_JPRB, 0.11464E+01_JPRB, 0.12168E+01_JPRB, 0.12782E+01_JPRB, 0.12485E+01_JPRB ,&
&  0.11836E+01_JPRB, 0.11087E+01_JPRB, 0.99553E+00_JPRB, 0.91709E+00_JPRB, 0.87240E+00_JPRB, 0.85059E+00_JPRB, 0.84344E+00_JPRB ,&
&  0.84597E+00_JPRB, 0.87091E+00_JPRB, 0.93002E+00_JPRB, 0.10256E+01_JPRB, 0.11489E+01_JPRB, 0.14339E+01_JPRB, 0.17003E+01_JPRB ,&
&  0.18146E+01_JPRB, 0.19503E+01_JPRB, 0.21845E+01_JPRB, 0.24266E+01_JPRB, 0.26960E+01_JPRB, 0.27772E+01_JPRB, 0.28625E+01_JPRB ,&
&  0.30000E+01_JPRB, 0.30018E+01_JPRB, 0.34303E+01_JPRB, 0.34941E+01_JPRB, 0.35152E+01_JPRB, 0.36043E+01_JPRB, 0.34798E+01_JPRB ,&
&  0.33635E+01_JPRB, 0.32350E+01_JPRB, 0.32492E+01_JPRB, 0.31685E+01_JPRB, 0.30773E+01_JPRB, 0.30816E+01_JPRB, 0.29230E+01_JPRB ,&
&  0.28705E+01_JPRB, 0.27676E+01_JPRB, 0.23874E+01_JPRB, 0.20957E+01_JPRB, 0.17685E+01_JPRB, 0.18028E+01_JPRB, 0.17177E+01_JPRB ,&
&  0.15276E+01_JPRB, 0.12310E+01_JPRB, 0.11994E+01_JPRB, 0.11520E+01_JPRB, 0.11561E+01_JPRB, 0.11536E+01_JPRB, 0.10885E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.11416E+01_JPRB, 0.11055E+01_JPRB, 0.10934E+01_JPRB, 0.10334E+01_JPRB, 0.96205E+00_JPRB, 0.80031E+00_JPRB ,&
&  0.74665E+00_JPRB, 0.77317E+00_JPRB, 0.79018E+00_JPRB, 0.80466E+00_JPRB, 0.80995E+00_JPRB /)
ZBURD(:, 16,  3) = (/ &
&  0.82882E+00_JPRB, 0.80917E+00_JPRB, 0.77932E+00_JPRB, 0.77416E+00_JPRB, 0.80044E+00_JPRB, 0.82906E+00_JPRB, 0.84229E+00_JPRB ,&
&  0.83090E+00_JPRB, 0.82839E+00_JPRB, 0.80921E+00_JPRB, 0.79492E+00_JPRB, 0.77717E+00_JPRB, 0.74478E+00_JPRB, 0.75741E+00_JPRB ,&
&  0.66088E+00_JPRB, 0.85718E+00_JPRB, 0.72154E+00_JPRB, 0.72115E+00_JPRB, 0.95221E+00_JPRB, 0.90047E+00_JPRB, 0.90259E+00_JPRB ,&
&  0.87258E+00_JPRB, 0.91476E+00_JPRB, 0.94513E+00_JPRB, 0.10259E+01_JPRB, 0.12501E+01_JPRB, 0.13737E+01_JPRB, 0.13318E+01_JPRB ,&
&  0.12127E+01_JPRB, 0.11352E+01_JPRB, 0.11775E+01_JPRB, 0.12082E+01_JPRB, 0.12053E+01_JPRB, 0.12332E+01_JPRB, 0.12263E+01_JPRB ,&
&  0.11392E+01_JPRB, 0.10763E+01_JPRB, 0.99972E+00_JPRB, 0.96461E+00_JPRB, 0.94967E+00_JPRB, 0.94573E+00_JPRB, 0.94216E+00_JPRB ,&
&  0.95523E+00_JPRB, 0.99074E+00_JPRB, 0.10407E+01_JPRB, 0.11633E+01_JPRB, 0.13003E+01_JPRB, 0.16416E+01_JPRB, 0.21253E+01_JPRB ,&
&  0.26677E+01_JPRB, 0.27924E+01_JPRB, 0.30973E+01_JPRB, 0.36895E+01_JPRB, 0.37510E+01_JPRB, 0.36203E+01_JPRB, 0.34034E+01_JPRB ,&
&  0.33748E+01_JPRB, 0.33498E+01_JPRB, 0.34299E+01_JPRB, 0.35987E+01_JPRB, 0.36428E+01_JPRB, 0.38295E+01_JPRB, 0.37226E+01_JPRB ,&
&  0.33606E+01_JPRB, 0.32024E+01_JPRB, 0.31624E+01_JPRB, 0.31143E+01_JPRB, 0.29835E+01_JPRB, 0.30117E+01_JPRB, 0.28637E+01_JPRB ,&
&  0.27711E+01_JPRB, 0.23922E+01_JPRB, 0.17909E+01_JPRB, 0.16285E+01_JPRB, 0.12767E+01_JPRB, 0.13257E+01_JPRB, 0.16963E+01_JPRB ,&
&  0.16443E+01_JPRB, 0.14477E+01_JPRB, 0.12986E+01_JPRB, 0.12114E+01_JPRB, 0.11918E+01_JPRB, 0.14027E+01_JPRB, 0.14030E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.12886E+01_JPRB, 0.13343E+01_JPRB, 0.13369E+01_JPRB, 0.12641E+01_JPRB, 0.11658E+01_JPRB, 0.10195E+01_JPRB ,&
&  0.95458E+00_JPRB, 0.90740E+00_JPRB, 0.88274E+00_JPRB, 0.86399E+00_JPRB, 0.84725E+00_JPRB /)
ZBURD(:, 17,  3) = (/ &
&  0.91221E+00_JPRB, 0.87914E+00_JPRB, 0.84613E+00_JPRB, 0.83868E+00_JPRB, 0.85335E+00_JPRB, 0.87655E+00_JPRB, 0.88642E+00_JPRB ,&
&  0.87025E+00_JPRB, 0.86034E+00_JPRB, 0.83637E+00_JPRB, 0.81647E+00_JPRB, 0.79833E+00_JPRB, 0.78295E+00_JPRB, 0.77876E+00_JPRB ,&
&  0.85691E+00_JPRB, 0.10741E+01_JPRB, 0.87355E+00_JPRB, 0.73100E+00_JPRB, 0.85098E+00_JPRB, 0.87619E+00_JPRB, 0.96000E+00_JPRB ,&
&  0.92607E+00_JPRB, 0.10327E+01_JPRB, 0.10581E+01_JPRB, 0.11552E+01_JPRB, 0.13948E+01_JPRB, 0.14793E+01_JPRB, 0.14902E+01_JPRB ,&
&  0.14898E+01_JPRB, 0.14151E+01_JPRB, 0.14156E+01_JPRB, 0.14486E+01_JPRB, 0.14621E+01_JPRB, 0.12902E+01_JPRB, 0.12444E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11231E+01_JPRB, 0.10667E+01_JPRB, 0.10461E+01_JPRB, 0.10448E+01_JPRB, 0.10569E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11249E+01_JPRB, 0.11611E+01_JPRB, 0.11928E+01_JPRB, 0.12770E+01_JPRB, 0.14040E+01_JPRB, 0.16322E+01_JPRB, 0.19006E+01_JPRB ,&
&  0.23571E+01_JPRB, 0.27238E+01_JPRB, 0.30604E+01_JPRB, 0.37557E+01_JPRB, 0.40141E+01_JPRB, 0.39308E+01_JPRB, 0.38428E+01_JPRB ,&
&  0.38164E+01_JPRB, 0.37531E+01_JPRB, 0.38518E+01_JPRB, 0.39195E+01_JPRB, 0.40219E+01_JPRB, 0.40723E+01_JPRB, 0.38506E+01_JPRB ,&
&  0.34670E+01_JPRB, 0.32070E+01_JPRB, 0.30964E+01_JPRB, 0.28592E+01_JPRB, 0.26627E+01_JPRB, 0.24389E+01_JPRB, 0.24181E+01_JPRB ,&
&  0.24992E+01_JPRB, 0.14318E+01_JPRB, 0.11224E+01_JPRB, 0.13281E+01_JPRB, 0.11140E+01_JPRB, 0.11853E+01_JPRB, 0.13983E+01_JPRB ,&
&  0.13963E+01_JPRB, 0.13670E+01_JPRB, 0.14095E+01_JPRB, 0.13629E+01_JPRB, 0.13992E+01_JPRB, 0.17382E+01_JPRB, 0.17819E+01_JPRB ,&
&  0.16571E+01_JPRB, 0.15481E+01_JPRB, 0.16070E+01_JPRB, 0.15522E+01_JPRB, 0.14935E+01_JPRB, 0.14138E+01_JPRB, 0.12982E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.11205E+01_JPRB, 0.10471E+01_JPRB, 0.98900E+00_JPRB, 0.94779E+00_JPRB /)
ZBURD(:, 18,  3) = (/ &
&  0.10400E+01_JPRB, 0.10015E+01_JPRB, 0.97110E+00_JPRB, 0.95085E+00_JPRB, 0.94536E+00_JPRB, 0.95822E+00_JPRB, 0.95601E+00_JPRB ,&
&  0.93314E+00_JPRB, 0.91296E+00_JPRB, 0.87858E+00_JPRB, 0.85098E+00_JPRB, 0.83158E+00_JPRB, 0.83661E+00_JPRB, 0.83082E+00_JPRB ,&
&  0.82370E+00_JPRB, 0.10238E+01_JPRB, 0.78763E+00_JPRB, 0.72490E+00_JPRB, 0.68125E+00_JPRB, 0.81394E+00_JPRB, 0.93807E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.10549E+01_JPRB, 0.11587E+01_JPRB, 0.14869E+01_JPRB, 0.17949E+01_JPRB, 0.18095E+01_JPRB, 0.19821E+01_JPRB ,&
&  0.20136E+01_JPRB, 0.19710E+01_JPRB, 0.17453E+01_JPRB, 0.17696E+01_JPRB, 0.15602E+01_JPRB, 0.13534E+01_JPRB, 0.13995E+01_JPRB ,&
&  0.13098E+01_JPRB, 0.12713E+01_JPRB, 0.12111E+01_JPRB, 0.11408E+01_JPRB, 0.11193E+01_JPRB, 0.11101E+01_JPRB, 0.11311E+01_JPRB ,&
&  0.11761E+01_JPRB, 0.12354E+01_JPRB, 0.13172E+01_JPRB, 0.13976E+01_JPRB, 0.15273E+01_JPRB, 0.17072E+01_JPRB, 0.17932E+01_JPRB ,&
&  0.21172E+01_JPRB, 0.22047E+01_JPRB, 0.20242E+01_JPRB, 0.27742E+01_JPRB, 0.40819E+01_JPRB, 0.40641E+01_JPRB, 0.37774E+01_JPRB ,&
&  0.40927E+01_JPRB, 0.41571E+01_JPRB, 0.43060E+01_JPRB, 0.44678E+01_JPRB, 0.44774E+01_JPRB, 0.43516E+01_JPRB, 0.40616E+01_JPRB ,&
&  0.36251E+01_JPRB, 0.33219E+01_JPRB, 0.31043E+01_JPRB, 0.30779E+01_JPRB, 0.27902E+01_JPRB, 0.26174E+01_JPRB, 0.24807E+01_JPRB ,&
&  0.21359E+01_JPRB, 0.17802E+01_JPRB, 0.12441E+01_JPRB, 0.13092E+01_JPRB, 0.10315E+01_JPRB, 0.10757E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.12611E+01_JPRB, 0.14047E+01_JPRB, 0.15073E+01_JPRB, 0.15047E+01_JPRB, 0.18138E+01_JPRB, 0.20732E+01_JPRB, 0.23325E+01_JPRB ,&
&  0.23084E+01_JPRB, 0.19195E+01_JPRB, 0.19725E+01_JPRB, 0.18649E+01_JPRB, 0.18299E+01_JPRB, 0.17464E+01_JPRB, 0.16357E+01_JPRB ,&
&  0.15024E+01_JPRB, 0.13594E+01_JPRB, 0.12415E+01_JPRB, 0.11501E+01_JPRB, 0.10872E+01_JPRB /)
ZBURD(:, 19,  3) = (/ &
&  0.11179E+01_JPRB, 0.10682E+01_JPRB, 0.10352E+01_JPRB, 0.10146E+01_JPRB, 0.10090E+01_JPRB, 0.10174E+01_JPRB, 0.10040E+01_JPRB ,&
&  0.97791E+00_JPRB, 0.94562E+00_JPRB, 0.90702E+00_JPRB, 0.87520E+00_JPRB, 0.85842E+00_JPRB, 0.86554E+00_JPRB, 0.86200E+00_JPRB ,&
&  0.82741E+00_JPRB, 0.83261E+00_JPRB, 0.67069E+00_JPRB, 0.60147E+00_JPRB, 0.58947E+00_JPRB, 0.69947E+00_JPRB, 0.89191E+00_JPRB ,&
&  0.10241E+01_JPRB, 0.11545E+01_JPRB, 0.14624E+01_JPRB, 0.19008E+01_JPRB, 0.22368E+01_JPRB, 0.23765E+01_JPRB, 0.25215E+01_JPRB ,&
&  0.29359E+01_JPRB, 0.23131E+01_JPRB, 0.22238E+01_JPRB, 0.19902E+01_JPRB, 0.18047E+01_JPRB, 0.16711E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.14338E+01_JPRB, 0.13518E+01_JPRB, 0.12349E+01_JPRB, 0.11752E+01_JPRB, 0.11598E+01_JPRB, 0.11799E+01_JPRB, 0.12662E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.14561E+01_JPRB, 0.14556E+01_JPRB, 0.14867E+01_JPRB, 0.16717E+01_JPRB, 0.18217E+01_JPRB, 0.18810E+01_JPRB ,&
&  0.18225E+01_JPRB, 0.18970E+01_JPRB, 0.26671E+01_JPRB, 0.32027E+01_JPRB, 0.37316E+01_JPRB, 0.40601E+01_JPRB, 0.42347E+01_JPRB ,&
&  0.44427E+01_JPRB, 0.43454E+01_JPRB, 0.44114E+01_JPRB, 0.39365E+01_JPRB, 0.44361E+01_JPRB, 0.43761E+01_JPRB, 0.40965E+01_JPRB ,&
&  0.37608E+01_JPRB, 0.35895E+01_JPRB, 0.32339E+01_JPRB, 0.30882E+01_JPRB, 0.30811E+01_JPRB, 0.31080E+01_JPRB, 0.23800E+01_JPRB ,&
&  0.18201E+01_JPRB, 0.19815E+01_JPRB, 0.16355E+01_JPRB, 0.13976E+01_JPRB, 0.12074E+01_JPRB, 0.12744E+01_JPRB, 0.13522E+01_JPRB ,&
&  0.14227E+01_JPRB, 0.15234E+01_JPRB, 0.15474E+01_JPRB, 0.17520E+01_JPRB, 0.21119E+01_JPRB, 0.23992E+01_JPRB, 0.25883E+01_JPRB ,&
&  0.23567E+01_JPRB, 0.24367E+01_JPRB, 0.23505E+01_JPRB, 0.22309E+01_JPRB, 0.22048E+01_JPRB, 0.21158E+01_JPRB, 0.19633E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.15798E+01_JPRB, 0.14287E+01_JPRB, 0.12989E+01_JPRB, 0.11929E+01_JPRB /)
ZBURD(:, 20,  3) = (/ &
&  0.12848E+01_JPRB, 0.12171E+01_JPRB, 0.11673E+01_JPRB, 0.11355E+01_JPRB, 0.11256E+01_JPRB, 0.11258E+01_JPRB, 0.10946E+01_JPRB ,&
&  0.10383E+01_JPRB, 0.99042E+00_JPRB, 0.94884E+00_JPRB, 0.91904E+00_JPRB, 0.89101E+00_JPRB, 0.88181E+00_JPRB, 0.87004E+00_JPRB ,&
&  0.85704E+00_JPRB, 0.77714E+00_JPRB, 0.61180E+00_JPRB, 0.64405E+00_JPRB, 0.72591E+00_JPRB, 0.70425E+00_JPRB, 0.84957E+00_JPRB ,&
&  0.10174E+01_JPRB, 0.13045E+01_JPRB, 0.16666E+01_JPRB, 0.23123E+01_JPRB, 0.27386E+01_JPRB, 0.30076E+01_JPRB, 0.30866E+01_JPRB ,&
&  0.29613E+01_JPRB, 0.27739E+01_JPRB, 0.25076E+01_JPRB, 0.21927E+01_JPRB, 0.18195E+01_JPRB, 0.16611E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.14425E+01_JPRB, 0.13115E+01_JPRB, 0.12337E+01_JPRB, 0.12382E+01_JPRB, 0.12748E+01_JPRB, 0.13470E+01_JPRB, 0.14767E+01_JPRB ,&
&  0.15465E+01_JPRB, 0.15453E+01_JPRB, 0.14487E+01_JPRB, 0.15336E+01_JPRB, 0.15478E+01_JPRB, 0.16366E+01_JPRB, 0.16715E+01_JPRB ,&
&  0.22157E+01_JPRB, 0.25000E+01_JPRB, 0.26219E+01_JPRB, 0.32447E+01_JPRB, 0.29856E+01_JPRB, 0.32422E+01_JPRB, 0.42824E+01_JPRB ,&
&  0.44960E+01_JPRB, 0.44801E+01_JPRB, 0.46691E+01_JPRB, 0.36061E+01_JPRB, 0.29773E+01_JPRB, 0.41868E+01_JPRB, 0.40053E+01_JPRB ,&
&  0.37487E+01_JPRB, 0.36829E+01_JPRB, 0.34103E+01_JPRB, 0.35200E+01_JPRB, 0.27299E+01_JPRB, 0.22710E+01_JPRB, 0.16678E+01_JPRB ,&
&  0.15568E+01_JPRB, 0.14276E+01_JPRB, 0.16037E+01_JPRB, 0.14688E+01_JPRB, 0.15057E+01_JPRB, 0.14292E+01_JPRB, 0.14063E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.16309E+01_JPRB, 0.17888E+01_JPRB, 0.21380E+01_JPRB, 0.25491E+01_JPRB, 0.24179E+01_JPRB, 0.27320E+01_JPRB ,&
&  0.28046E+01_JPRB, 0.27258E+01_JPRB, 0.24180E+01_JPRB, 0.24367E+01_JPRB, 0.23300E+01_JPRB, 0.22100E+01_JPRB, 0.20616E+01_JPRB ,&
&  0.18889E+01_JPRB, 0.17303E+01_JPRB, 0.15957E+01_JPRB, 0.14749E+01_JPRB, 0.13716E+01_JPRB /)
ZBURD(:, 21,  3) = (/ &
&  0.14198E+01_JPRB, 0.13565E+01_JPRB, 0.13051E+01_JPRB, 0.12729E+01_JPRB, 0.12520E+01_JPRB, 0.12186E+01_JPRB, 0.11479E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10126E+01_JPRB, 0.95682E+00_JPRB, 0.91759E+00_JPRB, 0.87921E+00_JPRB, 0.87442E+00_JPRB, 0.87539E+00_JPRB ,&
&  0.87326E+00_JPRB, 0.90258E+00_JPRB, 0.70318E+00_JPRB, 0.66286E+00_JPRB, 0.80631E+00_JPRB, 0.77098E+00_JPRB, 0.79406E+00_JPRB ,&
&  0.10811E+01_JPRB, 0.14697E+01_JPRB, 0.19405E+01_JPRB, 0.27577E+01_JPRB, 0.31502E+01_JPRB, 0.35300E+01_JPRB, 0.35707E+01_JPRB ,&
&  0.36763E+01_JPRB, 0.31188E+01_JPRB, 0.25941E+01_JPRB, 0.21721E+01_JPRB, 0.18311E+01_JPRB, 0.16163E+01_JPRB, 0.14683E+01_JPRB ,&
&  0.13282E+01_JPRB, 0.12546E+01_JPRB, 0.12728E+01_JPRB, 0.12789E+01_JPRB, 0.13387E+01_JPRB, 0.14025E+01_JPRB, 0.15111E+01_JPRB ,&
&  0.15644E+01_JPRB, 0.15324E+01_JPRB, 0.15163E+01_JPRB, 0.16501E+01_JPRB, 0.17153E+01_JPRB, 0.17088E+01_JPRB, 0.18359E+01_JPRB ,&
&  0.20582E+01_JPRB, 0.22232E+01_JPRB, 0.24528E+01_JPRB, 0.28576E+01_JPRB, 0.33666E+01_JPRB, 0.34009E+01_JPRB, 0.40517E+01_JPRB ,&
&  0.32766E+01_JPRB, 0.29052E+01_JPRB, 0.25507E+01_JPRB, 0.21424E+01_JPRB, 0.24817E+01_JPRB, 0.39840E+01_JPRB, 0.38810E+01_JPRB ,&
&  0.35860E+01_JPRB, 0.36584E+01_JPRB, 0.36120E+01_JPRB, 0.30679E+01_JPRB, 0.24410E+01_JPRB, 0.19034E+01_JPRB, 0.23477E+01_JPRB ,&
&  0.23497E+01_JPRB, 0.24543E+01_JPRB, 0.21838E+01_JPRB, 0.17555E+01_JPRB, 0.15186E+01_JPRB, 0.18560E+01_JPRB, 0.19096E+01_JPRB ,&
&  0.19779E+01_JPRB, 0.20979E+01_JPRB, 0.28681E+01_JPRB, 0.29939E+01_JPRB, 0.29257E+01_JPRB, 0.28636E+01_JPRB, 0.31103E+01_JPRB ,&
&  0.28826E+01_JPRB, 0.28254E+01_JPRB, 0.27084E+01_JPRB, 0.25093E+01_JPRB, 0.23323E+01_JPRB, 0.21562E+01_JPRB, 0.20035E+01_JPRB ,&
&  0.18858E+01_JPRB, 0.17775E+01_JPRB, 0.16590E+01_JPRB, 0.15675E+01_JPRB, 0.14875E+01_JPRB /)
ZBURD(:, 22,  3) = (/ &
&  0.14918E+01_JPRB, 0.14370E+01_JPRB, 0.13941E+01_JPRB, 0.13485E+01_JPRB, 0.12971E+01_JPRB, 0.12277E+01_JPRB, 0.11443E+01_JPRB ,&
&  0.10714E+01_JPRB, 0.10107E+01_JPRB, 0.96073E+00_JPRB, 0.92123E+00_JPRB, 0.88636E+00_JPRB, 0.87635E+00_JPRB, 0.87300E+00_JPRB ,&
&  0.89211E+00_JPRB, 0.11760E+01_JPRB, 0.89245E+00_JPRB, 0.73308E+00_JPRB, 0.79799E+00_JPRB, 0.80771E+00_JPRB, 0.76282E+00_JPRB ,&
&  0.11299E+01_JPRB, 0.16466E+01_JPRB, 0.20427E+01_JPRB, 0.29007E+01_JPRB, 0.38204E+01_JPRB, 0.36240E+01_JPRB, 0.35980E+01_JPRB ,&
&  0.35545E+01_JPRB, 0.29491E+01_JPRB, 0.25142E+01_JPRB, 0.21812E+01_JPRB, 0.18654E+01_JPRB, 0.16397E+01_JPRB, 0.14562E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13534E+01_JPRB, 0.13275E+01_JPRB, 0.13376E+01_JPRB, 0.13664E+01_JPRB, 0.14709E+01_JPRB, 0.15863E+01_JPRB ,&
&  0.15661E+01_JPRB, 0.15435E+01_JPRB, 0.15927E+01_JPRB, 0.17589E+01_JPRB, 0.18986E+01_JPRB, 0.18042E+01_JPRB, 0.21247E+01_JPRB ,&
&  0.20612E+01_JPRB, 0.22449E+01_JPRB, 0.23395E+01_JPRB, 0.28030E+01_JPRB, 0.31522E+01_JPRB, 0.32753E+01_JPRB, 0.36468E+01_JPRB ,&
&  0.27787E+01_JPRB, 0.25432E+01_JPRB, 0.26542E+01_JPRB, 0.25793E+01_JPRB, 0.21567E+01_JPRB, 0.24650E+01_JPRB, 0.37303E+01_JPRB ,&
&  0.28461E+01_JPRB, 0.31392E+01_JPRB, 0.32133E+01_JPRB, 0.32015E+01_JPRB, 0.16116E+01_JPRB, 0.11866E+01_JPRB, 0.19968E+01_JPRB ,&
&  0.22094E+01_JPRB, 0.13921E+01_JPRB, 0.97657E+00_JPRB, 0.12151E+01_JPRB, 0.98802E+00_JPRB, 0.13890E+01_JPRB, 0.21641E+01_JPRB ,&
&  0.25474E+01_JPRB, 0.28499E+01_JPRB, 0.39368E+01_JPRB, 0.35481E+01_JPRB, 0.32564E+01_JPRB, 0.34324E+01_JPRB, 0.32075E+01_JPRB ,&
&  0.30249E+01_JPRB, 0.26792E+01_JPRB, 0.26704E+01_JPRB, 0.23631E+01_JPRB, 0.21451E+01_JPRB, 0.20054E+01_JPRB, 0.19243E+01_JPRB ,&
&  0.18733E+01_JPRB, 0.18137E+01_JPRB, 0.17366E+01_JPRB, 0.16478E+01_JPRB, 0.15580E+01_JPRB /)
ZBURD(:, 23,  3) = (/ &
&  0.14921E+01_JPRB, 0.14506E+01_JPRB, 0.14147E+01_JPRB, 0.13629E+01_JPRB, 0.12939E+01_JPRB, 0.12097E+01_JPRB, 0.11281E+01_JPRB ,&
&  0.10753E+01_JPRB, 0.10151E+01_JPRB, 0.96809E+00_JPRB, 0.93030E+00_JPRB, 0.89825E+00_JPRB, 0.87896E+00_JPRB, 0.87042E+00_JPRB ,&
&  0.86712E+00_JPRB, 0.97104E+00_JPRB, 0.11731E+01_JPRB, 0.10134E+01_JPRB, 0.94285E+00_JPRB, 0.88746E+00_JPRB, 0.89138E+00_JPRB ,&
&  0.12980E+01_JPRB, 0.17781E+01_JPRB, 0.22470E+01_JPRB, 0.29003E+01_JPRB, 0.34392E+01_JPRB, 0.33529E+01_JPRB, 0.35296E+01_JPRB ,&
&  0.32281E+01_JPRB, 0.27710E+01_JPRB, 0.23874E+01_JPRB, 0.20902E+01_JPRB, 0.18357E+01_JPRB, 0.16386E+01_JPRB, 0.14949E+01_JPRB ,&
&  0.14263E+01_JPRB, 0.13932E+01_JPRB, 0.13366E+01_JPRB, 0.13005E+01_JPRB, 0.13696E+01_JPRB, 0.15157E+01_JPRB, 0.16019E+01_JPRB ,&
&  0.15708E+01_JPRB, 0.15893E+01_JPRB, 0.16303E+01_JPRB, 0.17441E+01_JPRB, 0.18847E+01_JPRB, 0.18665E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.16252E+01_JPRB, 0.17916E+01_JPRB, 0.23812E+01_JPRB, 0.25938E+01_JPRB, 0.28026E+01_JPRB, 0.31181E+01_JPRB, 0.33311E+01_JPRB ,&
&  0.32512E+01_JPRB, 0.32269E+01_JPRB, 0.26956E+01_JPRB, 0.27793E+01_JPRB, 0.24576E+01_JPRB, 0.20857E+01_JPRB, 0.26260E+01_JPRB ,&
&  0.27691E+01_JPRB, 0.26050E+01_JPRB, 0.21190E+01_JPRB, 0.16470E+01_JPRB, 0.16030E+01_JPRB, 0.12356E+01_JPRB, 0.86490E+00_JPRB ,&
&  0.84017E+00_JPRB, 0.80590E+00_JPRB, 0.71875E+00_JPRB, 0.72893E+00_JPRB, 0.72430E+00_JPRB, 0.11918E+01_JPRB, 0.21563E+01_JPRB ,&
&  0.33841E+01_JPRB, 0.40896E+01_JPRB, 0.44517E+01_JPRB, 0.40400E+01_JPRB, 0.34770E+01_JPRB, 0.35505E+01_JPRB, 0.33709E+01_JPRB ,&
&  0.27346E+01_JPRB, 0.25497E+01_JPRB, 0.24339E+01_JPRB, 0.22142E+01_JPRB, 0.20831E+01_JPRB, 0.19705E+01_JPRB, 0.18799E+01_JPRB ,&
&  0.17971E+01_JPRB, 0.17354E+01_JPRB, 0.16754E+01_JPRB, 0.16062E+01_JPRB, 0.15458E+01_JPRB /)
ZBURD(:, 24,  3) = (/ &
&  0.14590E+01_JPRB, 0.14250E+01_JPRB, 0.13806E+01_JPRB, 0.13211E+01_JPRB, 0.12465E+01_JPRB, 0.11730E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.10874E+01_JPRB, 0.10405E+01_JPRB, 0.99185E+00_JPRB, 0.94974E+00_JPRB, 0.91223E+00_JPRB, 0.88429E+00_JPRB, 0.87194E+00_JPRB ,&
&  0.87108E+00_JPRB, 0.89884E+00_JPRB, 0.11552E+01_JPRB, 0.11139E+01_JPRB, 0.11399E+01_JPRB, 0.11182E+01_JPRB, 0.11980E+01_JPRB ,&
&  0.15133E+01_JPRB, 0.20107E+01_JPRB, 0.25865E+01_JPRB, 0.29589E+01_JPRB, 0.34178E+01_JPRB, 0.35643E+01_JPRB, 0.34149E+01_JPRB ,&
&  0.30182E+01_JPRB, 0.26455E+01_JPRB, 0.23111E+01_JPRB, 0.20533E+01_JPRB, 0.17909E+01_JPRB, 0.16039E+01_JPRB, 0.14934E+01_JPRB ,&
&  0.14312E+01_JPRB, 0.14012E+01_JPRB, 0.13116E+01_JPRB, 0.12986E+01_JPRB, 0.13894E+01_JPRB, 0.15083E+01_JPRB, 0.15938E+01_JPRB ,&
&  0.16310E+01_JPRB, 0.16586E+01_JPRB, 0.17240E+01_JPRB, 0.18376E+01_JPRB, 0.16302E+01_JPRB, 0.14415E+01_JPRB, 0.16720E+01_JPRB ,&
&  0.19042E+01_JPRB, 0.22377E+01_JPRB, 0.22802E+01_JPRB, 0.27373E+01_JPRB, 0.28238E+01_JPRB, 0.28671E+01_JPRB, 0.32154E+01_JPRB ,&
&  0.31864E+01_JPRB, 0.32134E+01_JPRB, 0.23841E+01_JPRB, 0.24289E+01_JPRB, 0.27981E+01_JPRB, 0.22618E+01_JPRB, 0.21920E+01_JPRB ,&
&  0.25039E+01_JPRB, 0.22371E+01_JPRB, 0.19885E+01_JPRB, 0.17405E+01_JPRB, 0.23800E+01_JPRB, 0.22049E+01_JPRB, 0.10890E+01_JPRB ,&
&  0.90733E+00_JPRB, 0.81008E+00_JPRB, 0.74223E+00_JPRB, 0.68024E+00_JPRB, 0.71127E+00_JPRB, 0.12762E+01_JPRB, 0.30891E+01_JPRB ,&
&  0.35453E+01_JPRB, 0.52527E+01_JPRB, 0.52729E+01_JPRB, 0.46664E+01_JPRB, 0.37565E+01_JPRB, 0.35886E+01_JPRB, 0.30182E+01_JPRB ,&
&  0.26712E+01_JPRB, 0.25087E+01_JPRB, 0.22616E+01_JPRB, 0.20652E+01_JPRB, 0.18872E+01_JPRB, 0.17712E+01_JPRB, 0.17081E+01_JPRB ,&
&  0.16519E+01_JPRB, 0.16052E+01_JPRB, 0.15692E+01_JPRB, 0.15318E+01_JPRB, 0.14976E+01_JPRB /)
ZBURD(:, 25,  3) = (/ &
&  0.13664E+01_JPRB, 0.13223E+01_JPRB, 0.12786E+01_JPRB, 0.12248E+01_JPRB, 0.11759E+01_JPRB, 0.11405E+01_JPRB, 0.11123E+01_JPRB ,&
&  0.10990E+01_JPRB, 0.10573E+01_JPRB, 0.10097E+01_JPRB, 0.96024E+00_JPRB, 0.91548E+00_JPRB, 0.88208E+00_JPRB, 0.86963E+00_JPRB ,&
&  0.86303E+00_JPRB, 0.87943E+00_JPRB, 0.10167E+01_JPRB, 0.11387E+01_JPRB, 0.11846E+01_JPRB, 0.11158E+01_JPRB, 0.13305E+01_JPRB ,&
&  0.18017E+01_JPRB, 0.23850E+01_JPRB, 0.29148E+01_JPRB, 0.31244E+01_JPRB, 0.32975E+01_JPRB, 0.32787E+01_JPRB, 0.30304E+01_JPRB ,&
&  0.27269E+01_JPRB, 0.24913E+01_JPRB, 0.22496E+01_JPRB, 0.20132E+01_JPRB, 0.17885E+01_JPRB, 0.16399E+01_JPRB, 0.15333E+01_JPRB ,&
&  0.14757E+01_JPRB, 0.13718E+01_JPRB, 0.12962E+01_JPRB, 0.13061E+01_JPRB, 0.13977E+01_JPRB, 0.15295E+01_JPRB, 0.16323E+01_JPRB ,&
&  0.16834E+01_JPRB, 0.17042E+01_JPRB, 0.17676E+01_JPRB, 0.18682E+01_JPRB, 0.15383E+01_JPRB, 0.17937E+01_JPRB, 0.18508E+01_JPRB ,&
&  0.19642E+01_JPRB, 0.21137E+01_JPRB, 0.21249E+01_JPRB, 0.25103E+01_JPRB, 0.27068E+01_JPRB, 0.27625E+01_JPRB, 0.28419E+01_JPRB ,&
&  0.28992E+01_JPRB, 0.24662E+01_JPRB, 0.21219E+01_JPRB, 0.23052E+01_JPRB, 0.25403E+01_JPRB, 0.28970E+01_JPRB, 0.19544E+01_JPRB ,&
&  0.17890E+01_JPRB, 0.21293E+01_JPRB, 0.20333E+01_JPRB, 0.18439E+01_JPRB, 0.23107E+01_JPRB, 0.25976E+01_JPRB, 0.20946E+01_JPRB ,&
&  0.10080E+01_JPRB, 0.77112E+00_JPRB, 0.81105E+00_JPRB, 0.83225E+00_JPRB, 0.81574E+00_JPRB, 0.13598E+01_JPRB, 0.48728E+01_JPRB ,&
&  0.43612E+01_JPRB, 0.57592E+01_JPRB, 0.53523E+01_JPRB, 0.44729E+01_JPRB, 0.36453E+01_JPRB, 0.32618E+01_JPRB, 0.29269E+01_JPRB ,&
&  0.24595E+01_JPRB, 0.21893E+01_JPRB, 0.19280E+01_JPRB, 0.17508E+01_JPRB, 0.16550E+01_JPRB, 0.15851E+01_JPRB, 0.15375E+01_JPRB ,&
&  0.14946E+01_JPRB, 0.14605E+01_JPRB, 0.14406E+01_JPRB, 0.14269E+01_JPRB, 0.14057E+01_JPRB /)
ZBURD(:, 26,  3) = (/ &
&  0.12720E+01_JPRB, 0.12261E+01_JPRB, 0.11801E+01_JPRB, 0.11552E+01_JPRB, 0.11390E+01_JPRB, 0.11133E+01_JPRB, 0.10931E+01_JPRB ,&
&  0.10746E+01_JPRB, 0.10309E+01_JPRB, 0.98991E+00_JPRB, 0.94245E+00_JPRB, 0.90071E+00_JPRB, 0.87189E+00_JPRB, 0.85885E+00_JPRB ,&
&  0.86838E+00_JPRB, 0.88375E+00_JPRB, 0.96618E+00_JPRB, 0.11024E+01_JPRB, 0.12346E+01_JPRB, 0.11742E+01_JPRB, 0.13415E+01_JPRB ,&
&  0.22297E+01_JPRB, 0.29392E+01_JPRB, 0.28464E+01_JPRB, 0.29611E+01_JPRB, 0.31448E+01_JPRB, 0.33363E+01_JPRB, 0.29213E+01_JPRB ,&
&  0.25557E+01_JPRB, 0.22564E+01_JPRB, 0.19714E+01_JPRB, 0.17750E+01_JPRB, 0.16337E+01_JPRB, 0.15463E+01_JPRB, 0.14775E+01_JPRB ,&
&  0.13985E+01_JPRB, 0.13269E+01_JPRB, 0.12922E+01_JPRB, 0.13278E+01_JPRB, 0.14381E+01_JPRB, 0.15549E+01_JPRB, 0.16584E+01_JPRB ,&
&  0.16907E+01_JPRB, 0.17078E+01_JPRB, 0.17437E+01_JPRB, 0.17226E+01_JPRB, 0.17727E+01_JPRB, 0.18264E+01_JPRB, 0.19296E+01_JPRB ,&
&  0.19628E+01_JPRB, 0.21051E+01_JPRB, 0.22661E+01_JPRB, 0.23500E+01_JPRB, 0.25056E+01_JPRB, 0.26488E+01_JPRB, 0.25928E+01_JPRB ,&
&  0.25002E+01_JPRB, 0.21991E+01_JPRB, 0.19708E+01_JPRB, 0.22024E+01_JPRB, 0.23946E+01_JPRB, 0.25909E+01_JPRB, 0.24374E+01_JPRB ,&
&  0.21089E+01_JPRB, 0.17549E+01_JPRB, 0.17512E+01_JPRB, 0.19379E+01_JPRB, 0.21606E+01_JPRB, 0.22430E+01_JPRB, 0.29718E+01_JPRB ,&
&  0.28911E+01_JPRB, 0.19817E+01_JPRB, 0.16963E+01_JPRB, 0.21572E+01_JPRB, 0.18722E+01_JPRB, 0.19171E+01_JPRB, 0.30885E+01_JPRB ,&
&  0.42706E+01_JPRB, 0.54438E+01_JPRB, 0.49731E+01_JPRB, 0.38459E+01_JPRB, 0.33872E+01_JPRB, 0.29161E+01_JPRB, 0.24887E+01_JPRB ,&
&  0.20969E+01_JPRB, 0.19018E+01_JPRB, 0.17413E+01_JPRB, 0.15895E+01_JPRB, 0.14739E+01_JPRB, 0.14151E+01_JPRB, 0.13876E+01_JPRB ,&
&  0.13593E+01_JPRB, 0.13457E+01_JPRB, 0.13426E+01_JPRB, 0.13364E+01_JPRB, 0.13085E+01_JPRB /)
ZBURD(:, 27,  3) = (/ &
&  0.11672E+01_JPRB, 0.11380E+01_JPRB, 0.11262E+01_JPRB, 0.10960E+01_JPRB, 0.10614E+01_JPRB, 0.10334E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.10069E+01_JPRB, 0.97348E+00_JPRB, 0.93914E+00_JPRB, 0.90497E+00_JPRB, 0.86885E+00_JPRB, 0.85092E+00_JPRB, 0.84748E+00_JPRB ,&
&  0.86268E+00_JPRB, 0.89227E+00_JPRB, 0.98865E+00_JPRB, 0.11494E+01_JPRB, 0.13184E+01_JPRB, 0.15712E+01_JPRB, 0.13663E+01_JPRB ,&
&  0.19309E+01_JPRB, 0.30676E+01_JPRB, 0.30819E+01_JPRB, 0.30888E+01_JPRB, 0.30501E+01_JPRB, 0.29762E+01_JPRB, 0.26106E+01_JPRB ,&
&  0.22665E+01_JPRB, 0.20149E+01_JPRB, 0.17648E+01_JPRB, 0.16100E+01_JPRB, 0.15346E+01_JPRB, 0.14730E+01_JPRB, 0.13855E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12786E+01_JPRB, 0.12724E+01_JPRB, 0.13227E+01_JPRB, 0.14027E+01_JPRB, 0.15117E+01_JPRB, 0.15995E+01_JPRB ,&
&  0.16458E+01_JPRB, 0.16688E+01_JPRB, 0.16993E+01_JPRB, 0.16286E+01_JPRB, 0.17225E+01_JPRB, 0.18010E+01_JPRB, 0.18727E+01_JPRB ,&
&  0.17978E+01_JPRB, 0.17294E+01_JPRB, 0.20270E+01_JPRB, 0.22952E+01_JPRB, 0.24178E+01_JPRB, 0.23415E+01_JPRB, 0.21884E+01_JPRB ,&
&  0.22282E+01_JPRB, 0.21881E+01_JPRB, 0.22789E+01_JPRB, 0.21905E+01_JPRB, 0.22954E+01_JPRB, 0.24154E+01_JPRB, 0.25029E+01_JPRB ,&
&  0.24828E+01_JPRB, 0.21892E+01_JPRB, 0.20590E+01_JPRB, 0.20091E+01_JPRB, 0.19685E+01_JPRB, 0.20554E+01_JPRB, 0.26103E+01_JPRB ,&
&  0.31896E+01_JPRB, 0.35868E+01_JPRB, 0.37144E+01_JPRB, 0.31807E+01_JPRB, 0.27871E+01_JPRB, 0.26327E+01_JPRB, 0.29627E+01_JPRB ,&
&  0.43053E+01_JPRB, 0.46620E+01_JPRB, 0.40491E+01_JPRB, 0.38121E+01_JPRB, 0.29225E+01_JPRB, 0.26091E+01_JPRB, 0.22717E+01_JPRB ,&
&  0.20006E+01_JPRB, 0.17885E+01_JPRB, 0.16072E+01_JPRB, 0.14751E+01_JPRB, 0.13885E+01_JPRB, 0.13104E+01_JPRB, 0.12643E+01_JPRB ,&
&  0.12397E+01_JPRB, 0.12431E+01_JPRB, 0.12466E+01_JPRB, 0.12343E+01_JPRB, 0.12000E+01_JPRB /)
ZBURD(:, 28,  3) = (/ &
&  0.11097E+01_JPRB, 0.10982E+01_JPRB, 0.10825E+01_JPRB, 0.10444E+01_JPRB, 0.10158E+01_JPRB, 0.99698E+00_JPRB, 0.96087E+00_JPRB ,&
&  0.93537E+00_JPRB, 0.91775E+00_JPRB, 0.88958E+00_JPRB, 0.86132E+00_JPRB, 0.84233E+00_JPRB, 0.82705E+00_JPRB, 0.83669E+00_JPRB ,&
&  0.88404E+00_JPRB, 0.97118E+00_JPRB, 0.11246E+01_JPRB, 0.12696E+01_JPRB, 0.14567E+01_JPRB, 0.17437E+01_JPRB, 0.18887E+01_JPRB ,&
&  0.19946E+01_JPRB, 0.33787E+01_JPRB, 0.30116E+01_JPRB, 0.28883E+01_JPRB, 0.27597E+01_JPRB, 0.25482E+01_JPRB, 0.22244E+01_JPRB ,&
&  0.18797E+01_JPRB, 0.17172E+01_JPRB, 0.16036E+01_JPRB, 0.15021E+01_JPRB, 0.14442E+01_JPRB, 0.13318E+01_JPRB, 0.12915E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.12697E+01_JPRB, 0.12652E+01_JPRB, 0.13169E+01_JPRB, 0.14041E+01_JPRB, 0.14773E+01_JPRB, 0.15582E+01_JPRB ,&
&  0.16038E+01_JPRB, 0.16150E+01_JPRB, 0.15825E+01_JPRB, 0.15789E+01_JPRB, 0.16112E+01_JPRB, 0.16927E+01_JPRB, 0.17712E+01_JPRB ,&
&  0.17626E+01_JPRB, 0.17354E+01_JPRB, 0.19247E+01_JPRB, 0.19896E+01_JPRB, 0.19808E+01_JPRB, 0.20795E+01_JPRB, 0.19150E+01_JPRB ,&
&  0.20754E+01_JPRB, 0.19167E+01_JPRB, 0.23286E+01_JPRB, 0.19981E+01_JPRB, 0.20677E+01_JPRB, 0.22324E+01_JPRB, 0.21948E+01_JPRB ,&
&  0.19738E+01_JPRB, 0.19777E+01_JPRB, 0.18853E+01_JPRB, 0.18877E+01_JPRB, 0.19648E+01_JPRB, 0.21744E+01_JPRB, 0.26398E+01_JPRB ,&
&  0.28287E+01_JPRB, 0.28127E+01_JPRB, 0.29078E+01_JPRB, 0.24844E+01_JPRB, 0.24166E+01_JPRB, 0.23748E+01_JPRB, 0.27209E+01_JPRB ,&
&  0.39121E+01_JPRB, 0.40089E+01_JPRB, 0.36670E+01_JPRB, 0.31240E+01_JPRB, 0.24821E+01_JPRB, 0.22169E+01_JPRB, 0.19275E+01_JPRB ,&
&  0.17217E+01_JPRB, 0.15453E+01_JPRB, 0.14333E+01_JPRB, 0.13425E+01_JPRB, 0.12734E+01_JPRB, 0.12072E+01_JPRB, 0.11763E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.11563E+01_JPRB, 0.11554E+01_JPRB, 0.11407E+01_JPRB, 0.11232E+01_JPRB /)
ZBURD(:, 29,  3) = (/ &
&  0.11120E+01_JPRB, 0.11202E+01_JPRB, 0.11251E+01_JPRB, 0.11535E+01_JPRB, 0.12011E+01_JPRB, 0.12688E+01_JPRB, 0.13004E+01_JPRB ,&
&  0.11553E+01_JPRB, 0.87152E+00_JPRB, 0.85397E+00_JPRB, 0.83833E+00_JPRB, 0.82286E+00_JPRB, 0.83003E+00_JPRB, 0.88069E+00_JPRB ,&
&  0.96185E+00_JPRB, 0.10750E+01_JPRB, 0.12077E+01_JPRB, 0.13904E+01_JPRB, 0.16263E+01_JPRB, 0.20389E+01_JPRB, 0.24149E+01_JPRB ,&
&  0.23369E+01_JPRB, 0.26260E+01_JPRB, 0.29061E+01_JPRB, 0.25684E+01_JPRB, 0.23122E+01_JPRB, 0.21509E+01_JPRB, 0.19546E+01_JPRB ,&
&  0.16937E+01_JPRB, 0.15654E+01_JPRB, 0.15224E+01_JPRB, 0.14872E+01_JPRB, 0.12807E+01_JPRB, 0.12529E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.12826E+01_JPRB, 0.13011E+01_JPRB, 0.13692E+01_JPRB, 0.14398E+01_JPRB, 0.14977E+01_JPRB, 0.15500E+01_JPRB ,&
&  0.15897E+01_JPRB, 0.15944E+01_JPRB, 0.15919E+01_JPRB, 0.14948E+01_JPRB, 0.15222E+01_JPRB, 0.15705E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.16648E+01_JPRB, 0.17713E+01_JPRB, 0.18307E+01_JPRB, 0.17937E+01_JPRB, 0.16796E+01_JPRB, 0.18790E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.19205E+01_JPRB, 0.19590E+01_JPRB, 0.24300E+01_JPRB, 0.17968E+01_JPRB, 0.18914E+01_JPRB, 0.20166E+01_JPRB, 0.19999E+01_JPRB ,&
&  0.18439E+01_JPRB, 0.18283E+01_JPRB, 0.17435E+01_JPRB, 0.17444E+01_JPRB, 0.19683E+01_JPRB, 0.22325E+01_JPRB, 0.28274E+01_JPRB ,&
&  0.24555E+01_JPRB, 0.25578E+01_JPRB, 0.22759E+01_JPRB, 0.22965E+01_JPRB, 0.22445E+01_JPRB, 0.22016E+01_JPRB, 0.23209E+01_JPRB ,&
&  0.30698E+01_JPRB, 0.28813E+01_JPRB, 0.25555E+01_JPRB, 0.22060E+01_JPRB, 0.18160E+01_JPRB, 0.16120E+01_JPRB, 0.14173E+01_JPRB ,&
&  0.13101E+01_JPRB, 0.12580E+01_JPRB, 0.12427E+01_JPRB, 0.12056E+01_JPRB, 0.11812E+01_JPRB, 0.11526E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.11234E+01_JPRB, 0.11152E+01_JPRB, 0.11071E+01_JPRB, 0.11036E+01_JPRB, 0.11027E+01_JPRB /)
ZBURD(:, 30,  3) = (/ &
&  0.11524E+01_JPRB, 0.11417E+01_JPRB, 0.11353E+01_JPRB, 0.11317E+01_JPRB, 0.11238E+01_JPRB, 0.10930E+01_JPRB, 0.10286E+01_JPRB ,&
&  0.93880E+00_JPRB, 0.87083E+00_JPRB, 0.85642E+00_JPRB, 0.84954E+00_JPRB, 0.85571E+00_JPRB, 0.88916E+00_JPRB, 0.95216E+00_JPRB ,&
&  0.10267E+01_JPRB, 0.11505E+01_JPRB, 0.13145E+01_JPRB, 0.15233E+01_JPRB, 0.17839E+01_JPRB, 0.21948E+01_JPRB, 0.26665E+01_JPRB ,&
&  0.26554E+01_JPRB, 0.19889E+01_JPRB, 0.23159E+01_JPRB, 0.22459E+01_JPRB, 0.20611E+01_JPRB, 0.18885E+01_JPRB, 0.16599E+01_JPRB ,&
&  0.14637E+01_JPRB, 0.14033E+01_JPRB, 0.13743E+01_JPRB, 0.13187E+01_JPRB, 0.12327E+01_JPRB, 0.12570E+01_JPRB, 0.12888E+01_JPRB ,&
&  0.13236E+01_JPRB, 0.13528E+01_JPRB, 0.13645E+01_JPRB, 0.14272E+01_JPRB, 0.14725E+01_JPRB, 0.15321E+01_JPRB, 0.16107E+01_JPRB ,&
&  0.16554E+01_JPRB, 0.16713E+01_JPRB, 0.16710E+01_JPRB, 0.15738E+01_JPRB, 0.15631E+01_JPRB, 0.15950E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.16332E+01_JPRB, 0.16272E+01_JPRB, 0.16961E+01_JPRB, 0.17998E+01_JPRB, 0.19071E+01_JPRB, 0.17345E+01_JPRB, 0.18869E+01_JPRB ,&
&  0.19372E+01_JPRB, 0.20637E+01_JPRB, 0.19691E+01_JPRB, 0.22028E+01_JPRB, 0.15357E+01_JPRB, 0.17117E+01_JPRB, 0.16386E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.16718E+01_JPRB, 0.16719E+01_JPRB, 0.18116E+01_JPRB, 0.21402E+01_JPRB, 0.25261E+01_JPRB, 0.26034E+01_JPRB ,&
&  0.20317E+01_JPRB, 0.19208E+01_JPRB, 0.20354E+01_JPRB, 0.22657E+01_JPRB, 0.22969E+01_JPRB, 0.22680E+01_JPRB, 0.20909E+01_JPRB ,&
&  0.22687E+01_JPRB, 0.20476E+01_JPRB, 0.18514E+01_JPRB, 0.16282E+01_JPRB, 0.13206E+01_JPRB, 0.11752E+01_JPRB, 0.11463E+01_JPRB ,&
&  0.11548E+01_JPRB, 0.11715E+01_JPRB, 0.11753E+01_JPRB, 0.11726E+01_JPRB, 0.11529E+01_JPRB, 0.11340E+01_JPRB, 0.11224E+01_JPRB ,&
&  0.11158E+01_JPRB, 0.11195E+01_JPRB, 0.11232E+01_JPRB, 0.11274E+01_JPRB, 0.11407E+01_JPRB /)
ZBURD(:, 31,  3) = (/ &
&  0.11144E+01_JPRB, 0.10901E+01_JPRB, 0.10669E+01_JPRB, 0.10447E+01_JPRB, 0.10267E+01_JPRB, 0.98257E+00_JPRB, 0.93061E+00_JPRB ,&
&  0.89703E+00_JPRB, 0.88007E+00_JPRB, 0.87945E+00_JPRB, 0.88909E+00_JPRB, 0.91866E+00_JPRB, 0.95210E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.11084E+01_JPRB, 0.12414E+01_JPRB, 0.14173E+01_JPRB, 0.16273E+01_JPRB, 0.18869E+01_JPRB, 0.21958E+01_JPRB, 0.24190E+01_JPRB ,&
&  0.26350E+01_JPRB, 0.26635E+01_JPRB, 0.22878E+01_JPRB, 0.16465E+01_JPRB, 0.15233E+01_JPRB, 0.15546E+01_JPRB, 0.14015E+01_JPRB ,&
&  0.13684E+01_JPRB, 0.13310E+01_JPRB, 0.12977E+01_JPRB, 0.12884E+01_JPRB, 0.12968E+01_JPRB, 0.13317E+01_JPRB, 0.13704E+01_JPRB ,&
&  0.14033E+01_JPRB, 0.14277E+01_JPRB, 0.14595E+01_JPRB, 0.15298E+01_JPRB, 0.15912E+01_JPRB, 0.16617E+01_JPRB, 0.17252E+01_JPRB ,&
&  0.17642E+01_JPRB, 0.17998E+01_JPRB, 0.17826E+01_JPRB, 0.17421E+01_JPRB, 0.16847E+01_JPRB, 0.17030E+01_JPRB, 0.17308E+01_JPRB ,&
&  0.17591E+01_JPRB, 0.17578E+01_JPRB, 0.17618E+01_JPRB, 0.18600E+01_JPRB, 0.19241E+01_JPRB, 0.17331E+01_JPRB, 0.17764E+01_JPRB ,&
&  0.19778E+01_JPRB, 0.20800E+01_JPRB, 0.16628E+01_JPRB, 0.19443E+01_JPRB, 0.12390E+01_JPRB, 0.15501E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.16772E+01_JPRB, 0.17273E+01_JPRB, 0.19327E+01_JPRB, 0.22565E+01_JPRB, 0.24471E+01_JPRB, 0.21934E+01_JPRB ,&
&  0.17448E+01_JPRB, 0.17149E+01_JPRB, 0.18426E+01_JPRB, 0.19753E+01_JPRB, 0.20115E+01_JPRB, 0.18898E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.13064E+01_JPRB, 0.15414E+01_JPRB, 0.15120E+01_JPRB, 0.15029E+01_JPRB, 0.12141E+01_JPRB, 0.10510E+01_JPRB, 0.11014E+01_JPRB ,&
&  0.11549E+01_JPRB, 0.11878E+01_JPRB, 0.11957E+01_JPRB, 0.11836E+01_JPRB, 0.11611E+01_JPRB, 0.11424E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.11360E+01_JPRB, 0.11285E+01_JPRB, 0.11291E+01_JPRB, 0.11343E+01_JPRB, 0.11321E+01_JPRB /)
ZBURD(:, 32,  3) = (/ &
&  0.10194E+01_JPRB, 0.98753E+00_JPRB, 0.96582E+00_JPRB, 0.94944E+00_JPRB, 0.94120E+00_JPRB, 0.91729E+00_JPRB, 0.88230E+00_JPRB ,&
&  0.86240E+00_JPRB, 0.85506E+00_JPRB, 0.86680E+00_JPRB, 0.89944E+00_JPRB, 0.94637E+00_JPRB, 0.99264E+00_JPRB, 0.10773E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.13317E+01_JPRB, 0.15184E+01_JPRB, 0.17849E+01_JPRB, 0.20022E+01_JPRB, 0.22063E+01_JPRB, 0.22583E+01_JPRB ,&
&  0.23571E+01_JPRB, 0.23664E+01_JPRB, 0.22180E+01_JPRB, 0.20328E+01_JPRB, 0.16072E+01_JPRB, 0.15851E+01_JPRB, 0.17002E+01_JPRB ,&
&  0.17579E+01_JPRB, 0.15191E+01_JPRB, 0.13673E+01_JPRB, 0.13700E+01_JPRB, 0.13726E+01_JPRB, 0.14230E+01_JPRB, 0.14711E+01_JPRB ,&
&  0.15170E+01_JPRB, 0.15587E+01_JPRB, 0.15999E+01_JPRB, 0.16641E+01_JPRB, 0.17376E+01_JPRB, 0.17743E+01_JPRB, 0.18179E+01_JPRB ,&
&  0.18313E+01_JPRB, 0.18469E+01_JPRB, 0.18479E+01_JPRB, 0.18258E+01_JPRB, 0.18319E+01_JPRB, 0.18732E+01_JPRB, 0.19459E+01_JPRB ,&
&  0.20135E+01_JPRB, 0.19367E+01_JPRB, 0.20248E+01_JPRB, 0.21404E+01_JPRB, 0.20934E+01_JPRB, 0.19999E+01_JPRB, 0.19986E+01_JPRB ,&
&  0.21350E+01_JPRB, 0.22379E+01_JPRB, 0.14068E+01_JPRB, 0.16068E+01_JPRB, 0.18964E+01_JPRB, 0.16975E+01_JPRB, 0.16091E+01_JPRB ,&
&  0.16524E+01_JPRB, 0.17100E+01_JPRB, 0.18234E+01_JPRB, 0.19697E+01_JPRB, 0.22479E+01_JPRB, 0.27538E+01_JPRB, 0.19719E+01_JPRB ,&
&  0.14991E+01_JPRB, 0.14493E+01_JPRB, 0.15190E+01_JPRB, 0.15778E+01_JPRB, 0.15827E+01_JPRB, 0.14294E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.10716E+01_JPRB, 0.14448E+01_JPRB, 0.15415E+01_JPRB, 0.16827E+01_JPRB, 0.15219E+01_JPRB, 0.10174E+01_JPRB, 0.11139E+01_JPRB ,&
&  0.11549E+01_JPRB, 0.11734E+01_JPRB, 0.11497E+01_JPRB, 0.11419E+01_JPRB, 0.11361E+01_JPRB, 0.11271E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.11223E+01_JPRB, 0.11245E+01_JPRB, 0.10981E+01_JPRB, 0.10540E+01_JPRB /)
ZBURD(:, 33,  3) = (/ &
&  0.91849E+00_JPRB, 0.89286E+00_JPRB, 0.87934E+00_JPRB, 0.86062E+00_JPRB, 0.85011E+00_JPRB, 0.83663E+00_JPRB, 0.82942E+00_JPRB ,&
&  0.82742E+00_JPRB, 0.83118E+00_JPRB, 0.85137E+00_JPRB, 0.88579E+00_JPRB, 0.93387E+00_JPRB, 0.99881E+00_JPRB, 0.11129E+01_JPRB ,&
&  0.12250E+01_JPRB, 0.13932E+01_JPRB, 0.15980E+01_JPRB, 0.18362E+01_JPRB, 0.20080E+01_JPRB, 0.20606E+01_JPRB, 0.21664E+01_JPRB ,&
&  0.22339E+01_JPRB, 0.22479E+01_JPRB, 0.21962E+01_JPRB, 0.21175E+01_JPRB, 0.20101E+01_JPRB, 0.20015E+01_JPRB, 0.20904E+01_JPRB ,&
&  0.18984E+01_JPRB, 0.15018E+01_JPRB, 0.14328E+01_JPRB, 0.14366E+01_JPRB, 0.14663E+01_JPRB, 0.15092E+01_JPRB, 0.15672E+01_JPRB ,&
&  0.16137E+01_JPRB, 0.16499E+01_JPRB, 0.16832E+01_JPRB, 0.17216E+01_JPRB, 0.18076E+01_JPRB, 0.18453E+01_JPRB, 0.18870E+01_JPRB ,&
&  0.19223E+01_JPRB, 0.19515E+01_JPRB, 0.20046E+01_JPRB, 0.18862E+01_JPRB, 0.20088E+01_JPRB, 0.21057E+01_JPRB, 0.21936E+01_JPRB ,&
&  0.21942E+01_JPRB, 0.21351E+01_JPRB, 0.22612E+01_JPRB, 0.23056E+01_JPRB, 0.23419E+01_JPRB, 0.23444E+01_JPRB, 0.22965E+01_JPRB ,&
&  0.23557E+01_JPRB, 0.21492E+01_JPRB, 0.13518E+01_JPRB, 0.13069E+01_JPRB, 0.14413E+01_JPRB, 0.13463E+01_JPRB, 0.15997E+01_JPRB ,&
&  0.16458E+01_JPRB, 0.17215E+01_JPRB, 0.18008E+01_JPRB, 0.19403E+01_JPRB, 0.21200E+01_JPRB, 0.25443E+01_JPRB, 0.18042E+01_JPRB ,&
&  0.12019E+01_JPRB, 0.11421E+01_JPRB, 0.11576E+01_JPRB, 0.11865E+01_JPRB, 0.12348E+01_JPRB, 0.11939E+01_JPRB, 0.11251E+01_JPRB ,&
&  0.11318E+01_JPRB, 0.11820E+01_JPRB, 0.12112E+01_JPRB, 0.11551E+01_JPRB, 0.98965E+00_JPRB, 0.96234E+00_JPRB, 0.10081E+01_JPRB ,&
&  0.10442E+01_JPRB, 0.10647E+01_JPRB, 0.10615E+01_JPRB, 0.10681E+01_JPRB, 0.10615E+01_JPRB, 0.10565E+01_JPRB, 0.10509E+01_JPRB ,&
&  0.10388E+01_JPRB, 0.10322E+01_JPRB, 0.10140E+01_JPRB, 0.97276E+00_JPRB, 0.94114E+00_JPRB /)
ZBURD(:, 34,  3) = (/ &
&  0.79975E+00_JPRB, 0.80131E+00_JPRB, 0.79266E+00_JPRB, 0.78298E+00_JPRB, 0.77884E+00_JPRB, 0.76147E+00_JPRB, 0.78262E+00_JPRB ,&
&  0.77876E+00_JPRB, 0.80032E+00_JPRB, 0.82423E+00_JPRB, 0.84916E+00_JPRB, 0.89579E+00_JPRB, 0.94386E+00_JPRB, 0.10110E+01_JPRB ,&
&  0.11260E+01_JPRB, 0.13115E+01_JPRB, 0.15503E+01_JPRB, 0.18479E+01_JPRB, 0.19443E+01_JPRB, 0.20034E+01_JPRB, 0.21102E+01_JPRB ,&
&  0.22435E+01_JPRB, 0.23476E+01_JPRB, 0.23972E+01_JPRB, 0.24645E+01_JPRB, 0.25237E+01_JPRB, 0.24482E+01_JPRB, 0.23399E+01_JPRB ,&
&  0.19083E+01_JPRB, 0.14296E+01_JPRB, 0.15128E+01_JPRB, 0.14283E+01_JPRB, 0.14907E+01_JPRB, 0.15552E+01_JPRB, 0.16054E+01_JPRB ,&
&  0.16511E+01_JPRB, 0.16787E+01_JPRB, 0.17134E+01_JPRB, 0.17608E+01_JPRB, 0.18292E+01_JPRB, 0.18895E+01_JPRB, 0.19447E+01_JPRB ,&
&  0.19828E+01_JPRB, 0.20145E+01_JPRB, 0.20337E+01_JPRB, 0.19697E+01_JPRB, 0.20311E+01_JPRB, 0.21823E+01_JPRB, 0.23057E+01_JPRB ,&
&  0.24000E+01_JPRB, 0.23909E+01_JPRB, 0.21552E+01_JPRB, 0.21581E+01_JPRB, 0.22799E+01_JPRB, 0.21922E+01_JPRB, 0.22848E+01_JPRB ,&
&  0.24715E+01_JPRB, 0.21754E+01_JPRB, 0.12930E+01_JPRB, 0.12304E+01_JPRB, 0.13560E+01_JPRB, 0.14462E+01_JPRB, 0.15168E+01_JPRB ,&
&  0.15561E+01_JPRB, 0.15893E+01_JPRB, 0.16535E+01_JPRB, 0.17330E+01_JPRB, 0.17492E+01_JPRB, 0.17917E+01_JPRB, 0.15118E+01_JPRB ,&
&  0.10214E+01_JPRB, 0.10043E+01_JPRB, 0.10586E+01_JPRB, 0.11074E+01_JPRB, 0.12075E+01_JPRB, 0.11795E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.11038E+01_JPRB, 0.12306E+01_JPRB, 0.14188E+01_JPRB, 0.12520E+01_JPRB, 0.90724E+00_JPRB, 0.87842E+00_JPRB, 0.88254E+00_JPRB ,&
&  0.91368E+00_JPRB, 0.91755E+00_JPRB, 0.93460E+00_JPRB, 0.94575E+00_JPRB, 0.96147E+00_JPRB, 0.94739E+00_JPRB, 0.92951E+00_JPRB ,&
&  0.90702E+00_JPRB, 0.88703E+00_JPRB, 0.86602E+00_JPRB, 0.83550E+00_JPRB, 0.81229E+00_JPRB /)
ZBURD(:, 35,  3) = (/ &
&  0.69835E+00_JPRB, 0.66588E+00_JPRB, 0.63531E+00_JPRB, 0.63447E+00_JPRB, 0.63880E+00_JPRB, 0.62524E+00_JPRB, 0.65706E+00_JPRB ,&
&  0.63251E+00_JPRB, 0.65740E+00_JPRB, 0.67084E+00_JPRB, 0.67081E+00_JPRB, 0.71202E+00_JPRB, 0.75244E+00_JPRB, 0.80426E+00_JPRB ,&
&  0.87681E+00_JPRB, 0.99363E+00_JPRB, 0.11409E+01_JPRB, 0.13239E+01_JPRB, 0.15447E+01_JPRB, 0.16486E+01_JPRB, 0.17540E+01_JPRB ,&
&  0.19024E+01_JPRB, 0.21004E+01_JPRB, 0.23836E+01_JPRB, 0.27062E+01_JPRB, 0.30844E+01_JPRB, 0.29085E+01_JPRB, 0.24683E+01_JPRB ,&
&  0.18209E+01_JPRB, 0.14146E+01_JPRB, 0.13409E+01_JPRB, 0.12499E+01_JPRB, 0.13217E+01_JPRB, 0.14757E+01_JPRB, 0.15391E+01_JPRB ,&
&  0.15829E+01_JPRB, 0.16263E+01_JPRB, 0.16682E+01_JPRB, 0.17192E+01_JPRB, 0.17809E+01_JPRB, 0.18499E+01_JPRB, 0.18776E+01_JPRB ,&
&  0.18941E+01_JPRB, 0.19123E+01_JPRB, 0.19294E+01_JPRB, 0.20190E+01_JPRB, 0.21543E+01_JPRB, 0.23219E+01_JPRB, 0.23301E+01_JPRB ,&
&  0.23192E+01_JPRB, 0.22197E+01_JPRB, 0.19596E+01_JPRB, 0.19026E+01_JPRB, 0.19401E+01_JPRB, 0.19724E+01_JPRB, 0.20292E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.18427E+01_JPRB, 0.13337E+01_JPRB, 0.14024E+01_JPRB, 0.14602E+01_JPRB, 0.15221E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.13849E+01_JPRB, 0.13821E+01_JPRB, 0.14101E+01_JPRB, 0.13981E+01_JPRB, 0.14068E+01_JPRB, 0.13876E+01_JPRB, 0.12829E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.10465E+01_JPRB, 0.10021E+01_JPRB, 0.98296E+00_JPRB, 0.11581E+01_JPRB, 0.11107E+01_JPRB, 0.10920E+01_JPRB ,&
&  0.10141E+01_JPRB, 0.11000E+01_JPRB, 0.97952E+00_JPRB, 0.11884E+01_JPRB, 0.84798E+00_JPRB, 0.77737E+00_JPRB, 0.76474E+00_JPRB ,&
&  0.78773E+00_JPRB, 0.79720E+00_JPRB, 0.81175E+00_JPRB, 0.80125E+00_JPRB, 0.81637E+00_JPRB, 0.84589E+00_JPRB, 0.84840E+00_JPRB ,&
&  0.80273E+00_JPRB, 0.78646E+00_JPRB, 0.75571E+00_JPRB, 0.71997E+00_JPRB, 0.71083E+00_JPRB /)
ZBURD(:, 36,  3) = (/ &
&  0.59187E+00_JPRB, 0.58294E+00_JPRB, 0.57941E+00_JPRB, 0.57528E+00_JPRB, 0.58408E+00_JPRB, 0.58645E+00_JPRB, 0.58797E+00_JPRB ,&
&  0.58808E+00_JPRB, 0.59936E+00_JPRB, 0.60780E+00_JPRB, 0.61996E+00_JPRB, 0.63351E+00_JPRB, 0.66474E+00_JPRB, 0.70115E+00_JPRB ,&
&  0.74521E+00_JPRB, 0.79602E+00_JPRB, 0.85231E+00_JPRB, 0.93288E+00_JPRB, 0.10465E+01_JPRB, 0.11260E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.14965E+01_JPRB, 0.16848E+01_JPRB, 0.19368E+01_JPRB, 0.22244E+01_JPRB, 0.22740E+01_JPRB, 0.21186E+01_JPRB ,&
&  0.15635E+01_JPRB, 0.14816E+01_JPRB, 0.13789E+01_JPRB, 0.12562E+01_JPRB, 0.12712E+01_JPRB, 0.12387E+01_JPRB, 0.12564E+01_JPRB ,&
&  0.12912E+01_JPRB, 0.12945E+01_JPRB, 0.12780E+01_JPRB, 0.12845E+01_JPRB, 0.12901E+01_JPRB, 0.12958E+01_JPRB, 0.12755E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.13105E+01_JPRB, 0.15039E+01_JPRB, 0.17544E+01_JPRB, 0.19272E+01_JPRB, 0.19957E+01_JPRB, 0.18777E+01_JPRB ,&
&  0.17294E+01_JPRB, 0.16052E+01_JPRB, 0.15046E+01_JPRB, 0.14651E+01_JPRB, 0.15082E+01_JPRB, 0.15419E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.15825E+01_JPRB, 0.14324E+01_JPRB, 0.14533E+01_JPRB, 0.14854E+01_JPRB, 0.14841E+01_JPRB, 0.14298E+01_JPRB, 0.13396E+01_JPRB ,&
&  0.13316E+01_JPRB, 0.12701E+01_JPRB, 0.12405E+01_JPRB, 0.11732E+01_JPRB, 0.11005E+01_JPRB, 0.10435E+01_JPRB, 0.10131E+01_JPRB ,&
&  0.94194E+00_JPRB, 0.89008E+00_JPRB, 0.87735E+00_JPRB, 0.93713E+00_JPRB, 0.10703E+01_JPRB, 0.16051E+01_JPRB, 0.14854E+01_JPRB ,&
&  0.93513E+00_JPRB, 0.80686E+00_JPRB, 0.63215E+00_JPRB, 0.83438E+00_JPRB, 0.71181E+00_JPRB, 0.68653E+00_JPRB, 0.68714E+00_JPRB ,&
&  0.69238E+00_JPRB, 0.68889E+00_JPRB, 0.66870E+00_JPRB, 0.66489E+00_JPRB, 0.67417E+00_JPRB, 0.68260E+00_JPRB, 0.67253E+00_JPRB ,&
&  0.63546E+00_JPRB, 0.61844E+00_JPRB, 0.64438E+00_JPRB, 0.60385E+00_JPRB, 0.59489E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART5',1,ZHOOK_HANDLE)
END SUBROUTINE PART5



SUBROUTINE PART6

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART6',0,ZHOOK_HANDLE)


ZBURD(:, 37,  3) = (/ &
&  0.53669E+00_JPRB, 0.53526E+00_JPRB, 0.54832E+00_JPRB, 0.56103E+00_JPRB, 0.56816E+00_JPRB, 0.57404E+00_JPRB, 0.58388E+00_JPRB ,&
&  0.58981E+00_JPRB, 0.59815E+00_JPRB, 0.60753E+00_JPRB, 0.62472E+00_JPRB, 0.65006E+00_JPRB, 0.68887E+00_JPRB, 0.73338E+00_JPRB ,&
&  0.78780E+00_JPRB, 0.84637E+00_JPRB, 0.91180E+00_JPRB, 0.98229E+00_JPRB, 0.10699E+01_JPRB, 0.11914E+01_JPRB, 0.13581E+01_JPRB ,&
&  0.14509E+01_JPRB, 0.15675E+01_JPRB, 0.17090E+01_JPRB, 0.18291E+01_JPRB, 0.18453E+01_JPRB, 0.17948E+01_JPRB, 0.14594E+01_JPRB ,&
&  0.16241E+01_JPRB, 0.14750E+01_JPRB, 0.13332E+01_JPRB, 0.12079E+01_JPRB, 0.11191E+01_JPRB, 0.10423E+01_JPRB, 0.98339E+00_JPRB ,&
&  0.96224E+00_JPRB, 0.91942E+00_JPRB, 0.86274E+00_JPRB, 0.82165E+00_JPRB, 0.79515E+00_JPRB, 0.80770E+00_JPRB, 0.78258E+00_JPRB ,&
&  0.84602E+00_JPRB, 0.99312E+00_JPRB, 0.11809E+01_JPRB, 0.14145E+01_JPRB, 0.15419E+01_JPRB, 0.15080E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.10432E+01_JPRB, 0.10173E+01_JPRB, 0.10581E+01_JPRB, 0.10884E+01_JPRB, 0.11495E+01_JPRB, 0.11778E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.11305E+01_JPRB, 0.12302E+01_JPRB, 0.15613E+01_JPRB, 0.15213E+01_JPRB, 0.13760E+01_JPRB, 0.13485E+01_JPRB ,&
&  0.12570E+01_JPRB, 0.11962E+01_JPRB, 0.11651E+01_JPRB, 0.10956E+01_JPRB, 0.10113E+01_JPRB, 0.93818E+00_JPRB, 0.90939E+00_JPRB ,&
&  0.85401E+00_JPRB, 0.79250E+00_JPRB, 0.75858E+00_JPRB, 0.79572E+00_JPRB, 0.95320E+00_JPRB, 0.11225E+01_JPRB, 0.15624E+01_JPRB ,&
&  0.92357E+00_JPRB, 0.59544E+00_JPRB, 0.62375E+00_JPRB, 0.66763E+00_JPRB, 0.62522E+00_JPRB, 0.61542E+00_JPRB, 0.67262E+00_JPRB ,&
&  0.74026E+00_JPRB, 0.80784E+00_JPRB, 0.80323E+00_JPRB, 0.69395E+00_JPRB, 0.65058E+00_JPRB, 0.62980E+00_JPRB, 0.60488E+00_JPRB ,&
&  0.58455E+00_JPRB, 0.54732E+00_JPRB, 0.56045E+00_JPRB, 0.54365E+00_JPRB, 0.54091E+00_JPRB /)
ZBURD(:, 38,  3) = (/ &
&  0.50093E+00_JPRB, 0.49855E+00_JPRB, 0.50601E+00_JPRB, 0.51369E+00_JPRB, 0.51525E+00_JPRB, 0.51683E+00_JPRB, 0.51821E+00_JPRB ,&
&  0.52111E+00_JPRB, 0.52118E+00_JPRB, 0.52363E+00_JPRB, 0.53124E+00_JPRB, 0.54758E+00_JPRB, 0.58272E+00_JPRB, 0.62343E+00_JPRB ,&
&  0.65943E+00_JPRB, 0.69297E+00_JPRB, 0.72576E+00_JPRB, 0.75361E+00_JPRB, 0.78246E+00_JPRB, 0.80123E+00_JPRB, 0.85547E+00_JPRB ,&
&  0.92481E+00_JPRB, 0.10079E+01_JPRB, 0.10919E+01_JPRB, 0.11521E+01_JPRB, 0.12032E+01_JPRB, 0.11681E+01_JPRB, 0.95746E+00_JPRB ,&
&  0.13971E+01_JPRB, 0.13391E+01_JPRB, 0.12116E+01_JPRB, 0.10902E+01_JPRB, 0.10041E+01_JPRB, 0.92541E+00_JPRB, 0.85138E+00_JPRB ,&
&  0.78609E+00_JPRB, 0.81508E+00_JPRB, 0.77215E+00_JPRB, 0.80675E+00_JPRB, 0.71291E+00_JPRB, 0.74198E+00_JPRB, 0.84798E+00_JPRB ,&
&  0.93534E+00_JPRB, 0.10149E+01_JPRB, 0.11051E+01_JPRB, 0.12076E+01_JPRB, 0.12610E+01_JPRB, 0.12257E+01_JPRB, 0.10930E+01_JPRB ,&
&  0.10162E+01_JPRB, 0.97758E+00_JPRB, 0.92776E+00_JPRB, 0.97596E+00_JPRB, 0.97713E+00_JPRB, 0.95757E+00_JPRB, 0.96554E+00_JPRB ,&
&  0.87566E+00_JPRB, 0.93456E+00_JPRB, 0.11150E+01_JPRB, 0.14938E+01_JPRB, 0.14690E+01_JPRB, 0.13942E+01_JPRB, 0.13428E+01_JPRB ,&
&  0.12484E+01_JPRB, 0.11730E+01_JPRB, 0.11193E+01_JPRB, 0.10470E+01_JPRB, 0.96945E+00_JPRB, 0.89987E+00_JPRB, 0.84569E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.74877E+00_JPRB, 0.70951E+00_JPRB, 0.70111E+00_JPRB, 0.74646E+00_JPRB, 0.80458E+00_JPRB, 0.13073E+01_JPRB ,&
&  0.92491E+00_JPRB, 0.74525E+00_JPRB, 0.80301E+00_JPRB, 0.43248E+00_JPRB, 0.51749E+00_JPRB, 0.55209E+00_JPRB, 0.66399E+00_JPRB ,&
&  0.82569E+00_JPRB, 0.13153E+01_JPRB, 0.20359E+01_JPRB, 0.18328E+01_JPRB, 0.13999E+01_JPRB, 0.90283E+00_JPRB, 0.54501E+00_JPRB ,&
&  0.52409E+00_JPRB, 0.51874E+00_JPRB, 0.52275E+00_JPRB, 0.51361E+00_JPRB, 0.50958E+00_JPRB /)
ZBURD(:, 39,  3) = (/ &
&  0.45373E+00_JPRB, 0.44412E+00_JPRB, 0.45192E+00_JPRB, 0.46140E+00_JPRB, 0.45738E+00_JPRB, 0.45690E+00_JPRB, 0.45188E+00_JPRB ,&
&  0.44249E+00_JPRB, 0.43638E+00_JPRB, 0.45846E+00_JPRB, 0.50038E+00_JPRB, 0.55866E+00_JPRB, 0.63433E+00_JPRB, 0.70885E+00_JPRB ,&
&  0.75755E+00_JPRB, 0.81315E+00_JPRB, 0.86095E+00_JPRB, 0.89912E+00_JPRB, 0.90162E+00_JPRB, 0.90146E+00_JPRB, 0.88502E+00_JPRB ,&
&  0.87420E+00_JPRB, 0.88736E+00_JPRB, 0.93241E+00_JPRB, 0.97452E+00_JPRB, 0.10518E+01_JPRB, 0.11625E+01_JPRB, 0.76053E+00_JPRB ,&
&  0.11433E+01_JPRB, 0.11338E+01_JPRB, 0.10748E+01_JPRB, 0.97915E+00_JPRB, 0.92840E+00_JPRB, 0.85142E+00_JPRB, 0.78889E+00_JPRB ,&
&  0.78737E+00_JPRB, 0.82159E+00_JPRB, 0.82321E+00_JPRB, 0.90190E+00_JPRB, 0.98117E+00_JPRB, 0.99706E+00_JPRB, 0.11085E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.12744E+01_JPRB, 0.13611E+01_JPRB, 0.14249E+01_JPRB, 0.14783E+01_JPRB, 0.14901E+01_JPRB, 0.14620E+01_JPRB ,&
&  0.13672E+01_JPRB, 0.13006E+01_JPRB, 0.13046E+01_JPRB, 0.13563E+01_JPRB, 0.12383E+01_JPRB, 0.11807E+01_JPRB, 0.10451E+01_JPRB ,&
&  0.86038E+00_JPRB, 0.84949E+00_JPRB, 0.99190E+00_JPRB, 0.13185E+01_JPRB, 0.13924E+01_JPRB, 0.13407E+01_JPRB, 0.12382E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.10708E+01_JPRB, 0.10050E+01_JPRB, 0.94371E+00_JPRB, 0.88753E+00_JPRB, 0.83684E+00_JPRB, 0.78800E+00_JPRB ,&
&  0.76081E+00_JPRB, 0.72404E+00_JPRB, 0.68303E+00_JPRB, 0.68471E+00_JPRB, 0.72246E+00_JPRB, 0.84819E+00_JPRB, 0.91506E+00_JPRB ,&
&  0.10356E+01_JPRB, 0.77140E+00_JPRB, 0.66684E+00_JPRB, 0.63586E+00_JPRB, 0.52118E+00_JPRB, 0.55027E+00_JPRB, 0.71411E+00_JPRB ,&
&  0.11223E+01_JPRB, 0.16587E+01_JPRB, 0.28019E+01_JPRB, 0.50304E+01_JPRB, 0.27847E+01_JPRB, 0.21492E+01_JPRB, 0.52519E+00_JPRB ,&
&  0.50300E+00_JPRB, 0.50384E+00_JPRB, 0.50017E+00_JPRB, 0.48309E+00_JPRB, 0.47782E+00_JPRB /)
ZBURD(:, 40,  3) = (/ &
&  0.44020E+00_JPRB, 0.43156E+00_JPRB, 0.43968E+00_JPRB, 0.43783E+00_JPRB, 0.44134E+00_JPRB, 0.44839E+00_JPRB, 0.45276E+00_JPRB ,&
&  0.45648E+00_JPRB, 0.47308E+00_JPRB, 0.56280E+00_JPRB, 0.68658E+00_JPRB, 0.80422E+00_JPRB, 0.89384E+00_JPRB, 0.93408E+00_JPRB ,&
&  0.97338E+00_JPRB, 0.99056E+00_JPRB, 0.98108E+00_JPRB, 0.96536E+00_JPRB, 0.97642E+00_JPRB, 0.10221E+01_JPRB, 0.10585E+01_JPRB ,&
&  0.10911E+01_JPRB, 0.11226E+01_JPRB, 0.11567E+01_JPRB, 0.12112E+01_JPRB, 0.12629E+01_JPRB, 0.12071E+01_JPRB, 0.74811E+00_JPRB ,&
&  0.98821E+00_JPRB, 0.10275E+01_JPRB, 0.10276E+01_JPRB, 0.94945E+00_JPRB, 0.87831E+00_JPRB, 0.83765E+00_JPRB, 0.88360E+00_JPRB ,&
&  0.94174E+00_JPRB, 0.97052E+00_JPRB, 0.96725E+00_JPRB, 0.10030E+01_JPRB, 0.13094E+01_JPRB, 0.13278E+01_JPRB, 0.14771E+01_JPRB ,&
&  0.15830E+01_JPRB, 0.16982E+01_JPRB, 0.17711E+01_JPRB, 0.18635E+01_JPRB, 0.19402E+01_JPRB, 0.19857E+01_JPRB, 0.20529E+01_JPRB ,&
&  0.20661E+01_JPRB, 0.20969E+01_JPRB, 0.20740E+01_JPRB, 0.19864E+01_JPRB, 0.16679E+01_JPRB, 0.17560E+01_JPRB, 0.13224E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.97051E+00_JPRB, 0.10988E+01_JPRB, 0.12172E+01_JPRB, 0.12262E+01_JPRB, 0.11509E+01_JPRB, 0.10323E+01_JPRB ,&
&  0.97738E+00_JPRB, 0.94124E+00_JPRB, 0.90434E+00_JPRB, 0.86002E+00_JPRB, 0.84334E+00_JPRB, 0.81701E+00_JPRB, 0.79346E+00_JPRB ,&
&  0.78150E+00_JPRB, 0.74748E+00_JPRB, 0.70985E+00_JPRB, 0.72383E+00_JPRB, 0.78670E+00_JPRB, 0.98806E+00_JPRB, 0.13657E+01_JPRB ,&
&  0.15053E+01_JPRB, 0.85500E+00_JPRB, 0.75702E+00_JPRB, 0.68206E+00_JPRB, 0.64907E+00_JPRB, 0.66843E+00_JPRB, 0.74420E+00_JPRB ,&
&  0.95661E+00_JPRB, 0.16855E+01_JPRB, 0.25655E+01_JPRB, 0.19412E+01_JPRB, 0.22108E+01_JPRB, 0.15213E+01_JPRB, 0.62261E+00_JPRB ,&
&  0.48678E+00_JPRB, 0.47440E+00_JPRB, 0.47884E+00_JPRB, 0.46889E+00_JPRB, 0.45593E+00_JPRB /)
ZBURD(:, 41,  3) = (/ &
&  0.49019E+00_JPRB, 0.47849E+00_JPRB, 0.47735E+00_JPRB, 0.48762E+00_JPRB, 0.49400E+00_JPRB, 0.51851E+00_JPRB, 0.54923E+00_JPRB ,&
&  0.56508E+00_JPRB, 0.63530E+00_JPRB, 0.79231E+00_JPRB, 0.92678E+00_JPRB, 0.98350E+00_JPRB, 0.10247E+01_JPRB, 0.98109E+00_JPRB ,&
&  0.97209E+00_JPRB, 0.94614E+00_JPRB, 0.90299E+00_JPRB, 0.90243E+00_JPRB, 0.92523E+00_JPRB, 0.96855E+00_JPRB, 0.10256E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.11563E+01_JPRB, 0.12124E+01_JPRB, 0.12641E+01_JPRB, 0.13354E+01_JPRB, 0.13233E+01_JPRB, 0.12554E+01_JPRB ,&
&  0.62705E+00_JPRB, 0.98315E+00_JPRB, 0.98044E+00_JPRB, 0.92476E+00_JPRB, 0.86568E+00_JPRB, 0.86794E+00_JPRB, 0.98562E+00_JPRB ,&
&  0.10936E+01_JPRB, 0.11088E+01_JPRB, 0.12575E+01_JPRB, 0.12238E+01_JPRB, 0.14667E+01_JPRB, 0.15901E+01_JPRB, 0.17182E+01_JPRB ,&
&  0.18295E+01_JPRB, 0.19090E+01_JPRB, 0.19962E+01_JPRB, 0.20835E+01_JPRB, 0.21712E+01_JPRB, 0.22403E+01_JPRB, 0.23368E+01_JPRB ,&
&  0.24443E+01_JPRB, 0.25992E+01_JPRB, 0.27394E+01_JPRB, 0.25998E+01_JPRB, 0.22301E+01_JPRB, 0.21135E+01_JPRB, 0.17725E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.10038E+01_JPRB, 0.10799E+01_JPRB, 0.11589E+01_JPRB, 0.11260E+01_JPRB, 0.10610E+01_JPRB, 0.99199E+00_JPRB ,&
&  0.94012E+00_JPRB, 0.89362E+00_JPRB, 0.83084E+00_JPRB, 0.84157E+00_JPRB, 0.86333E+00_JPRB, 0.85875E+00_JPRB, 0.84216E+00_JPRB ,&
&  0.84179E+00_JPRB, 0.82062E+00_JPRB, 0.79864E+00_JPRB, 0.78434E+00_JPRB, 0.80650E+00_JPRB, 0.87865E+00_JPRB, 0.94149E+00_JPRB ,&
&  0.97204E+00_JPRB, 0.88125E+00_JPRB, 0.81380E+00_JPRB, 0.81002E+00_JPRB, 0.79905E+00_JPRB, 0.79035E+00_JPRB, 0.77335E+00_JPRB ,&
&  0.81358E+00_JPRB, 0.97433E+00_JPRB, 0.13653E+01_JPRB, 0.17493E+01_JPRB, 0.14092E+01_JPRB, 0.11123E+01_JPRB, 0.63789E+00_JPRB ,&
&  0.51133E+00_JPRB, 0.49389E+00_JPRB, 0.51127E+00_JPRB, 0.51446E+00_JPRB, 0.51341E+00_JPRB /)
ZBURD(:, 42,  3) = (/ &
&  0.59827E+00_JPRB, 0.58464E+00_JPRB, 0.56745E+00_JPRB, 0.58356E+00_JPRB, 0.61683E+00_JPRB, 0.65880E+00_JPRB, 0.67712E+00_JPRB ,&
&  0.70109E+00_JPRB, 0.80218E+00_JPRB, 0.93563E+00_JPRB, 0.10537E+01_JPRB, 0.10746E+01_JPRB, 0.10440E+01_JPRB, 0.94743E+00_JPRB ,&
&  0.92347E+00_JPRB, 0.90040E+00_JPRB, 0.88952E+00_JPRB, 0.87868E+00_JPRB, 0.88885E+00_JPRB, 0.93064E+00_JPRB, 0.99922E+00_JPRB ,&
&  0.10878E+01_JPRB, 0.11846E+01_JPRB, 0.12944E+01_JPRB, 0.14126E+01_JPRB, 0.15192E+01_JPRB, 0.15604E+01_JPRB, 0.14477E+01_JPRB ,&
&  0.60796E+00_JPRB, 0.71174E+00_JPRB, 0.98146E+00_JPRB, 0.97630E+00_JPRB, 0.93578E+00_JPRB, 0.93397E+00_JPRB, 0.10074E+01_JPRB ,&
&  0.10882E+01_JPRB, 0.11083E+01_JPRB, 0.12314E+01_JPRB, 0.14369E+01_JPRB, 0.15107E+01_JPRB, 0.16703E+01_JPRB, 0.17469E+01_JPRB ,&
&  0.18195E+01_JPRB, 0.19025E+01_JPRB, 0.20006E+01_JPRB, 0.20960E+01_JPRB, 0.21920E+01_JPRB, 0.22787E+01_JPRB, 0.24046E+01_JPRB ,&
&  0.25815E+01_JPRB, 0.27584E+01_JPRB, 0.29035E+01_JPRB, 0.26009E+01_JPRB, 0.27261E+01_JPRB, 0.24324E+01_JPRB, 0.21414E+01_JPRB ,&
&  0.14395E+01_JPRB, 0.10646E+01_JPRB, 0.10993E+01_JPRB, 0.11993E+01_JPRB, 0.11055E+01_JPRB, 0.10064E+01_JPRB, 0.92142E+00_JPRB ,&
&  0.86671E+00_JPRB, 0.80700E+00_JPRB, 0.83170E+00_JPRB, 0.86037E+00_JPRB, 0.85612E+00_JPRB, 0.89932E+00_JPRB, 0.86931E+00_JPRB ,&
&  0.88401E+00_JPRB, 0.89674E+00_JPRB, 0.89168E+00_JPRB, 0.90120E+00_JPRB, 0.91337E+00_JPRB, 0.93361E+00_JPRB, 0.93236E+00_JPRB ,&
&  0.94042E+00_JPRB, 0.93238E+00_JPRB, 0.94006E+00_JPRB, 0.95074E+00_JPRB, 0.93868E+00_JPRB, 0.91457E+00_JPRB, 0.85856E+00_JPRB ,&
&  0.79523E+00_JPRB, 0.80045E+00_JPRB, 0.89492E+00_JPRB, 0.10178E+01_JPRB, 0.97142E+00_JPRB, 0.83837E+00_JPRB, 0.66654E+00_JPRB ,&
&  0.65257E+00_JPRB, 0.67343E+00_JPRB, 0.67469E+00_JPRB, 0.65384E+00_JPRB, 0.62888E+00_JPRB /)
ZBURD(:, 43,  3) = (/ &
&  0.71317E+00_JPRB, 0.68527E+00_JPRB, 0.64622E+00_JPRB, 0.63897E+00_JPRB, 0.69356E+00_JPRB, 0.74127E+00_JPRB, 0.74310E+00_JPRB ,&
&  0.75106E+00_JPRB, 0.81562E+00_JPRB, 0.93259E+00_JPRB, 0.10250E+01_JPRB, 0.10553E+01_JPRB, 0.10478E+01_JPRB, 0.95577E+00_JPRB ,&
&  0.92141E+00_JPRB, 0.90062E+00_JPRB, 0.87703E+00_JPRB, 0.86829E+00_JPRB, 0.86973E+00_JPRB, 0.88286E+00_JPRB, 0.91510E+00_JPRB ,&
&  0.96512E+00_JPRB, 0.10515E+01_JPRB, 0.11895E+01_JPRB, 0.13855E+01_JPRB, 0.16714E+01_JPRB, 0.20065E+01_JPRB, 0.22974E+01_JPRB ,&
&  0.17270E+01_JPRB, 0.73077E+00_JPRB, 0.82343E+00_JPRB, 0.10921E+01_JPRB, 0.10317E+01_JPRB, 0.10485E+01_JPRB, 0.10253E+01_JPRB ,&
&  0.10696E+01_JPRB, 0.11377E+01_JPRB, 0.12128E+01_JPRB, 0.13858E+01_JPRB, 0.15042E+01_JPRB, 0.16311E+01_JPRB, 0.16770E+01_JPRB ,&
&  0.17269E+01_JPRB, 0.17919E+01_JPRB, 0.18569E+01_JPRB, 0.19233E+01_JPRB, 0.20016E+01_JPRB, 0.20916E+01_JPRB, 0.22381E+01_JPRB ,&
&  0.24229E+01_JPRB, 0.25866E+01_JPRB, 0.27059E+01_JPRB, 0.23227E+01_JPRB, 0.24286E+01_JPRB, 0.20094E+01_JPRB, 0.16274E+01_JPRB ,&
&  0.13883E+01_JPRB, 0.12131E+01_JPRB, 0.11624E+01_JPRB, 0.12352E+01_JPRB, 0.11065E+01_JPRB, 0.89732E+00_JPRB, 0.92081E+00_JPRB ,&
&  0.85395E+00_JPRB, 0.83647E+00_JPRB, 0.84104E+00_JPRB, 0.79416E+00_JPRB, 0.85516E+00_JPRB, 0.88802E+00_JPRB, 0.91675E+00_JPRB ,&
&  0.91148E+00_JPRB, 0.93666E+00_JPRB, 0.96466E+00_JPRB, 0.10109E+01_JPRB, 0.10131E+01_JPRB, 0.10275E+01_JPRB, 0.10224E+01_JPRB ,&
&  0.10399E+01_JPRB, 0.10532E+01_JPRB, 0.10754E+01_JPRB, 0.10979E+01_JPRB, 0.10934E+01_JPRB, 0.10050E+01_JPRB, 0.93781E+00_JPRB ,&
&  0.89586E+00_JPRB, 0.86828E+00_JPRB, 0.81579E+00_JPRB, 0.84947E+00_JPRB, 0.85163E+00_JPRB, 0.85775E+00_JPRB, 0.88781E+00_JPRB ,&
&  0.90189E+00_JPRB, 0.90375E+00_JPRB, 0.85292E+00_JPRB, 0.79520E+00_JPRB, 0.75741E+00_JPRB /)
ZBURD(:, 44,  3) = (/ &
&  0.78004E+00_JPRB, 0.73056E+00_JPRB, 0.69707E+00_JPRB, 0.70666E+00_JPRB, 0.75627E+00_JPRB, 0.80732E+00_JPRB, 0.80415E+00_JPRB ,&
&  0.78857E+00_JPRB, 0.80845E+00_JPRB, 0.87759E+00_JPRB, 0.93704E+00_JPRB, 0.94574E+00_JPRB, 0.95524E+00_JPRB, 0.97599E+00_JPRB ,&
&  0.92695E+00_JPRB, 0.89649E+00_JPRB, 0.88252E+00_JPRB, 0.86422E+00_JPRB, 0.86496E+00_JPRB, 0.86122E+00_JPRB, 0.86752E+00_JPRB ,&
&  0.89369E+00_JPRB, 0.93325E+00_JPRB, 0.10165E+01_JPRB, 0.11465E+01_JPRB, 0.13521E+01_JPRB, 0.17645E+01_JPRB, 0.23990E+01_JPRB ,&
&  0.34223E+01_JPRB, 0.22570E+01_JPRB, 0.83219E+00_JPRB, 0.10843E+01_JPRB, 0.11581E+01_JPRB, 0.11761E+01_JPRB, 0.10486E+01_JPRB ,&
&  0.10907E+01_JPRB, 0.11403E+01_JPRB, 0.11430E+01_JPRB, 0.12957E+01_JPRB, 0.14035E+01_JPRB, 0.14997E+01_JPRB, 0.15504E+01_JPRB ,&
&  0.15722E+01_JPRB, 0.16185E+01_JPRB, 0.16514E+01_JPRB, 0.16756E+01_JPRB, 0.17067E+01_JPRB, 0.17592E+01_JPRB, 0.18573E+01_JPRB ,&
&  0.20519E+01_JPRB, 0.22827E+01_JPRB, 0.24854E+01_JPRB, 0.21555E+01_JPRB, 0.21676E+01_JPRB, 0.17770E+01_JPRB, 0.15682E+01_JPRB ,&
&  0.14272E+01_JPRB, 0.14605E+01_JPRB, 0.14655E+01_JPRB, 0.12979E+01_JPRB, 0.11024E+01_JPRB, 0.97585E+00_JPRB, 0.10480E+01_JPRB ,&
&  0.92720E+00_JPRB, 0.86183E+00_JPRB, 0.83333E+00_JPRB, 0.85555E+00_JPRB, 0.88843E+00_JPRB, 0.90943E+00_JPRB, 0.92572E+00_JPRB ,&
&  0.95577E+00_JPRB, 0.96916E+00_JPRB, 0.10179E+01_JPRB, 0.10539E+01_JPRB, 0.10881E+01_JPRB, 0.11496E+01_JPRB, 0.11719E+01_JPRB ,&
&  0.11815E+01_JPRB, 0.11898E+01_JPRB, 0.12100E+01_JPRB, 0.12284E+01_JPRB, 0.11671E+01_JPRB, 0.10716E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10357E+01_JPRB, 0.10045E+01_JPRB, 0.92761E+00_JPRB, 0.92123E+00_JPRB, 0.91425E+00_JPRB, 0.96802E+00_JPRB, 0.10386E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10438E+01_JPRB, 0.97429E+00_JPRB, 0.84602E+00_JPRB, 0.83096E+00_JPRB /)
ZBURD(:, 45,  3) = (/ &
&  0.84079E+00_JPRB, 0.82420E+00_JPRB, 0.79619E+00_JPRB, 0.77686E+00_JPRB, 0.79935E+00_JPRB, 0.83326E+00_JPRB, 0.83134E+00_JPRB ,&
&  0.80423E+00_JPRB, 0.78786E+00_JPRB, 0.82218E+00_JPRB, 0.85831E+00_JPRB, 0.85527E+00_JPRB, 0.87316E+00_JPRB, 0.88343E+00_JPRB ,&
&  0.91703E+00_JPRB, 0.91171E+00_JPRB, 0.89272E+00_JPRB, 0.86651E+00_JPRB, 0.86393E+00_JPRB, 0.85096E+00_JPRB, 0.85077E+00_JPRB ,&
&  0.86325E+00_JPRB, 0.88521E+00_JPRB, 0.93025E+00_JPRB, 0.10020E+01_JPRB, 0.10879E+01_JPRB, 0.12668E+01_JPRB, 0.16338E+01_JPRB ,&
&  0.20865E+01_JPRB, 0.27034E+01_JPRB, 0.96106E+00_JPRB, 0.12489E+01_JPRB, 0.14822E+01_JPRB, 0.14026E+01_JPRB, 0.12574E+01_JPRB ,&
&  0.12165E+01_JPRB, 0.11770E+01_JPRB, 0.11679E+01_JPRB, 0.11794E+01_JPRB, 0.12241E+01_JPRB, 0.12939E+01_JPRB, 0.13056E+01_JPRB ,&
&  0.13414E+01_JPRB, 0.13936E+01_JPRB, 0.14758E+01_JPRB, 0.14936E+01_JPRB, 0.14890E+01_JPRB, 0.14787E+01_JPRB, 0.15030E+01_JPRB ,&
&  0.16379E+01_JPRB, 0.18625E+01_JPRB, 0.21987E+01_JPRB, 0.20287E+01_JPRB, 0.20278E+01_JPRB, 0.18414E+01_JPRB, 0.16820E+01_JPRB ,&
&  0.15599E+01_JPRB, 0.16216E+01_JPRB, 0.16099E+01_JPRB, 0.14211E+01_JPRB, 0.11908E+01_JPRB, 0.10771E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.98643E+00_JPRB, 0.91297E+00_JPRB, 0.89626E+00_JPRB, 0.90874E+00_JPRB, 0.96568E+00_JPRB, 0.96909E+00_JPRB, 0.95981E+00_JPRB ,&
&  0.98627E+00_JPRB, 0.98382E+00_JPRB, 0.10127E+01_JPRB, 0.10561E+01_JPRB, 0.11056E+01_JPRB, 0.11857E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.12598E+01_JPRB, 0.12953E+01_JPRB, 0.13112E+01_JPRB, 0.12705E+01_JPRB, 0.12067E+01_JPRB, 0.11586E+01_JPRB, 0.11371E+01_JPRB ,&
&  0.11603E+01_JPRB, 0.11748E+01_JPRB, 0.10206E+01_JPRB, 0.95537E+00_JPRB, 0.97119E+00_JPRB, 0.96931E+00_JPRB, 0.10551E+01_JPRB ,&
&  0.11592E+01_JPRB, 0.12298E+01_JPRB, 0.11834E+01_JPRB, 0.91152E+00_JPRB, 0.89831E+00_JPRB /)
ZBURD(:, 46,  3) = (/ &
&  0.93545E+00_JPRB, 0.89308E+00_JPRB, 0.87138E+00_JPRB, 0.85186E+00_JPRB, 0.83405E+00_JPRB, 0.86501E+00_JPRB, 0.85493E+00_JPRB ,&
&  0.82939E+00_JPRB, 0.79709E+00_JPRB, 0.81195E+00_JPRB, 0.85713E+00_JPRB, 0.85840E+00_JPRB, 0.83872E+00_JPRB, 0.82556E+00_JPRB ,&
&  0.82519E+00_JPRB, 0.84208E+00_JPRB, 0.84562E+00_JPRB, 0.85861E+00_JPRB, 0.85993E+00_JPRB, 0.84973E+00_JPRB, 0.84308E+00_JPRB ,&
&  0.85939E+00_JPRB, 0.87384E+00_JPRB, 0.89432E+00_JPRB, 0.91505E+00_JPRB, 0.94860E+00_JPRB, 0.98537E+00_JPRB, 0.11607E+01_JPRB ,&
&  0.14020E+01_JPRB, 0.22429E+01_JPRB, 0.18360E+01_JPRB, 0.21697E+01_JPRB, 0.20126E+01_JPRB, 0.18104E+01_JPRB, 0.15640E+01_JPRB ,&
&  0.16032E+01_JPRB, 0.14962E+01_JPRB, 0.14758E+01_JPRB, 0.11892E+01_JPRB, 0.10712E+01_JPRB, 0.10709E+01_JPRB, 0.10571E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10668E+01_JPRB, 0.11872E+01_JPRB, 0.12387E+01_JPRB, 0.12566E+01_JPRB, 0.12565E+01_JPRB, 0.12580E+01_JPRB ,&
&  0.13239E+01_JPRB, 0.14340E+01_JPRB, 0.17448E+01_JPRB, 0.19511E+01_JPRB, 0.18963E+01_JPRB, 0.19052E+01_JPRB, 0.18888E+01_JPRB ,&
&  0.19300E+01_JPRB, 0.18336E+01_JPRB, 0.17079E+01_JPRB, 0.15076E+01_JPRB, 0.12697E+01_JPRB, 0.12584E+01_JPRB, 0.11299E+01_JPRB ,&
&  0.10187E+01_JPRB, 0.95059E+00_JPRB, 0.94822E+00_JPRB, 0.97652E+00_JPRB, 0.10208E+01_JPRB, 0.10193E+01_JPRB, 0.99678E+00_JPRB ,&
&  0.99113E+00_JPRB, 0.10032E+01_JPRB, 0.99438E+00_JPRB, 0.10211E+01_JPRB, 0.10495E+01_JPRB, 0.10991E+01_JPRB, 0.11481E+01_JPRB ,&
&  0.12159E+01_JPRB, 0.12831E+01_JPRB, 0.12694E+01_JPRB, 0.12377E+01_JPRB, 0.11764E+01_JPRB, 0.11331E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.10722E+01_JPRB, 0.11058E+01_JPRB, 0.10762E+01_JPRB, 0.10340E+01_JPRB, 0.99754E+00_JPRB, 0.96508E+00_JPRB, 0.95530E+00_JPRB ,&
&  0.97275E+00_JPRB, 0.10269E+01_JPRB, 0.10251E+01_JPRB, 0.97569E+00_JPRB, 0.96302E+00_JPRB /)
ZBURD(:, 47,  3) = (/ &
&  0.99442E+00_JPRB, 0.97001E+00_JPRB, 0.92448E+00_JPRB, 0.91851E+00_JPRB, 0.89041E+00_JPRB, 0.91563E+00_JPRB, 0.91902E+00_JPRB ,&
&  0.88360E+00_JPRB, 0.84838E+00_JPRB, 0.82322E+00_JPRB, 0.84072E+00_JPRB, 0.86149E+00_JPRB, 0.84907E+00_JPRB, 0.81105E+00_JPRB ,&
&  0.79803E+00_JPRB, 0.79476E+00_JPRB, 0.79363E+00_JPRB, 0.79749E+00_JPRB, 0.81331E+00_JPRB, 0.83652E+00_JPRB, 0.83476E+00_JPRB ,&
&  0.83206E+00_JPRB, 0.86522E+00_JPRB, 0.89815E+00_JPRB, 0.90268E+00_JPRB, 0.90170E+00_JPRB, 0.90423E+00_JPRB, 0.98307E+00_JPRB ,&
&  0.11039E+01_JPRB, 0.23168E+01_JPRB, 0.16570E+01_JPRB, 0.27979E+01_JPRB, 0.22134E+01_JPRB, 0.19471E+01_JPRB, 0.15873E+01_JPRB ,&
&  0.14668E+01_JPRB, 0.15639E+01_JPRB, 0.13405E+01_JPRB, 0.11525E+01_JPRB, 0.10096E+01_JPRB, 0.94542E+00_JPRB, 0.89415E+00_JPRB ,&
&  0.85037E+00_JPRB, 0.87875E+00_JPRB, 0.93320E+00_JPRB, 0.10049E+01_JPRB, 0.10495E+01_JPRB, 0.10592E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.11537E+01_JPRB, 0.13542E+01_JPRB, 0.15961E+01_JPRB, 0.18244E+01_JPRB, 0.19464E+01_JPRB, 0.20907E+01_JPRB ,&
&  0.21445E+01_JPRB, 0.21065E+01_JPRB, 0.18320E+01_JPRB, 0.15821E+01_JPRB, 0.13923E+01_JPRB, 0.13207E+01_JPRB, 0.12393E+01_JPRB ,&
&  0.11408E+01_JPRB, 0.10476E+01_JPRB, 0.10312E+01_JPRB, 0.10324E+01_JPRB, 0.10842E+01_JPRB, 0.10907E+01_JPRB, 0.10416E+01_JPRB ,&
&  0.99702E+00_JPRB, 0.98562E+00_JPRB, 0.99521E+00_JPRB, 0.10167E+01_JPRB, 0.10434E+01_JPRB, 0.10507E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.11140E+01_JPRB, 0.12030E+01_JPRB, 0.12212E+01_JPRB, 0.12330E+01_JPRB, 0.11803E+01_JPRB, 0.11428E+01_JPRB, 0.11142E+01_JPRB ,&
&  0.11461E+01_JPRB, 0.11352E+01_JPRB, 0.10877E+01_JPRB, 0.10464E+01_JPRB, 0.10385E+01_JPRB, 0.10342E+01_JPRB, 0.95052E+00_JPRB ,&
&  0.94136E+00_JPRB, 0.97528E+00_JPRB, 0.10283E+01_JPRB, 0.99797E+00_JPRB, 0.99471E+00_JPRB /)
ZBURD(:, 48,  3) = (/ &
&  0.10215E+01_JPRB, 0.10098E+01_JPRB, 0.94905E+00_JPRB, 0.93611E+00_JPRB, 0.91388E+00_JPRB, 0.90825E+00_JPRB, 0.91812E+00_JPRB ,&
&  0.90824E+00_JPRB, 0.88086E+00_JPRB, 0.84637E+00_JPRB, 0.82918E+00_JPRB, 0.83977E+00_JPRB, 0.85304E+00_JPRB, 0.82136E+00_JPRB ,&
&  0.78353E+00_JPRB, 0.76932E+00_JPRB, 0.76091E+00_JPRB, 0.76298E+00_JPRB, 0.77344E+00_JPRB, 0.78924E+00_JPRB, 0.79903E+00_JPRB ,&
&  0.79540E+00_JPRB, 0.83593E+00_JPRB, 0.88301E+00_JPRB, 0.90300E+00_JPRB, 0.89225E+00_JPRB, 0.89461E+00_JPRB, 0.94606E+00_JPRB ,&
&  0.11181E+01_JPRB, 0.29374E+01_JPRB, 0.17932E+01_JPRB, 0.28859E+01_JPRB, 0.22785E+01_JPRB, 0.19433E+01_JPRB, 0.15619E+01_JPRB ,&
&  0.14467E+01_JPRB, 0.14079E+01_JPRB, 0.12702E+01_JPRB, 0.11384E+01_JPRB, 0.10363E+01_JPRB, 0.95583E+00_JPRB, 0.89261E+00_JPRB ,&
&  0.85973E+00_JPRB, 0.85430E+00_JPRB, 0.86525E+00_JPRB, 0.91632E+00_JPRB, 0.95252E+00_JPRB, 0.96028E+00_JPRB, 0.95492E+00_JPRB ,&
&  0.96724E+00_JPRB, 0.99863E+00_JPRB, 0.11155E+01_JPRB, 0.14468E+01_JPRB, 0.16583E+01_JPRB, 0.19130E+01_JPRB, 0.26105E+01_JPRB ,&
&  0.25604E+01_JPRB, 0.24939E+01_JPRB, 0.19782E+01_JPRB, 0.16799E+01_JPRB, 0.14877E+01_JPRB, 0.13666E+01_JPRB, 0.13161E+01_JPRB ,&
&  0.12485E+01_JPRB, 0.11752E+01_JPRB, 0.11451E+01_JPRB, 0.11243E+01_JPRB, 0.11231E+01_JPRB, 0.11229E+01_JPRB, 0.10831E+01_JPRB ,&
&  0.10403E+01_JPRB, 0.10054E+01_JPRB, 0.10115E+01_JPRB, 0.10269E+01_JPRB, 0.10475E+01_JPRB, 0.10392E+01_JPRB, 0.10329E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.11204E+01_JPRB, 0.11578E+01_JPRB, 0.11853E+01_JPRB, 0.11835E+01_JPRB, 0.11612E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.11984E+01_JPRB, 0.12028E+01_JPRB, 0.11334E+01_JPRB, 0.11342E+01_JPRB, 0.11399E+01_JPRB, 0.11973E+01_JPRB, 0.10047E+01_JPRB ,&
&  0.95860E+00_JPRB, 0.94273E+00_JPRB, 0.10059E+01_JPRB, 0.10185E+01_JPRB, 0.10344E+01_JPRB /)
ZBURD(:, 49,  3) = (/ &
&  0.10515E+01_JPRB, 0.10222E+01_JPRB, 0.98664E+00_JPRB, 0.94130E+00_JPRB, 0.92713E+00_JPRB, 0.91057E+00_JPRB, 0.89959E+00_JPRB ,&
&  0.89429E+00_JPRB, 0.87728E+00_JPRB, 0.85921E+00_JPRB, 0.83494E+00_JPRB, 0.82469E+00_JPRB, 0.83533E+00_JPRB, 0.83021E+00_JPRB ,&
&  0.81164E+00_JPRB, 0.78685E+00_JPRB, 0.77735E+00_JPRB, 0.76741E+00_JPRB, 0.75655E+00_JPRB, 0.75551E+00_JPRB, 0.76485E+00_JPRB ,&
&  0.77040E+00_JPRB, 0.79073E+00_JPRB, 0.82351E+00_JPRB, 0.85321E+00_JPRB, 0.85388E+00_JPRB, 0.85486E+00_JPRB, 0.92037E+00_JPRB ,&
&  0.11544E+01_JPRB, 0.11778E+01_JPRB, 0.24315E+01_JPRB, 0.27714E+01_JPRB, 0.23079E+01_JPRB, 0.19979E+01_JPRB, 0.16926E+01_JPRB ,&
&  0.15389E+01_JPRB, 0.13434E+01_JPRB, 0.11850E+01_JPRB, 0.10706E+01_JPRB, 0.10032E+01_JPRB, 0.92718E+00_JPRB, 0.89243E+00_JPRB ,&
&  0.88047E+00_JPRB, 0.86575E+00_JPRB, 0.86602E+00_JPRB, 0.88381E+00_JPRB, 0.88929E+00_JPRB, 0.89337E+00_JPRB, 0.88492E+00_JPRB ,&
&  0.89117E+00_JPRB, 0.91223E+00_JPRB, 0.98384E+00_JPRB, 0.11873E+01_JPRB, 0.15438E+01_JPRB, 0.19482E+01_JPRB, 0.25825E+01_JPRB ,&
&  0.29115E+01_JPRB, 0.27919E+01_JPRB, 0.21320E+01_JPRB, 0.17739E+01_JPRB, 0.15763E+01_JPRB, 0.14347E+01_JPRB, 0.13696E+01_JPRB ,&
&  0.13066E+01_JPRB, 0.12484E+01_JPRB, 0.12070E+01_JPRB, 0.12238E+01_JPRB, 0.11757E+01_JPRB, 0.11458E+01_JPRB, 0.11054E+01_JPRB ,&
&  0.10680E+01_JPRB, 0.10363E+01_JPRB, 0.10045E+01_JPRB, 0.98707E+00_JPRB, 0.10056E+01_JPRB, 0.10163E+01_JPRB, 0.10140E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.10524E+01_JPRB, 0.11118E+01_JPRB, 0.11187E+01_JPRB, 0.11571E+01_JPRB, 0.11700E+01_JPRB, 0.11810E+01_JPRB ,&
&  0.12060E+01_JPRB, 0.12290E+01_JPRB, 0.11821E+01_JPRB, 0.12261E+01_JPRB, 0.12142E+01_JPRB, 0.12736E+01_JPRB, 0.11099E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.96425E+00_JPRB, 0.96651E+00_JPRB, 0.10195E+01_JPRB, 0.10608E+01_JPRB /)
ZBURD(:, 50,  3) = (/ &
&  0.10631E+01_JPRB, 0.10344E+01_JPRB, 0.10060E+01_JPRB, 0.98063E+00_JPRB, 0.95808E+00_JPRB, 0.94379E+00_JPRB, 0.92775E+00_JPRB ,&
&  0.92509E+00_JPRB, 0.92055E+00_JPRB, 0.90423E+00_JPRB, 0.87152E+00_JPRB, 0.84460E+00_JPRB, 0.83768E+00_JPRB, 0.84105E+00_JPRB ,&
&  0.83355E+00_JPRB, 0.82099E+00_JPRB, 0.82255E+00_JPRB, 0.80428E+00_JPRB, 0.78177E+00_JPRB, 0.76696E+00_JPRB, 0.76099E+00_JPRB ,&
&  0.76555E+00_JPRB, 0.77791E+00_JPRB, 0.80703E+00_JPRB, 0.83370E+00_JPRB, 0.83684E+00_JPRB, 0.82997E+00_JPRB, 0.82754E+00_JPRB ,&
&  0.10410E+01_JPRB, 0.16482E+01_JPRB, 0.26319E+01_JPRB, 0.25034E+01_JPRB, 0.21789E+01_JPRB, 0.18526E+01_JPRB, 0.16925E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.13248E+01_JPRB, 0.11852E+01_JPRB, 0.10637E+01_JPRB, 0.97667E+00_JPRB, 0.92455E+00_JPRB, 0.89505E+00_JPRB ,&
&  0.86585E+00_JPRB, 0.84976E+00_JPRB, 0.85164E+00_JPRB, 0.85660E+00_JPRB, 0.85832E+00_JPRB, 0.85315E+00_JPRB, 0.84373E+00_JPRB ,&
&  0.85177E+00_JPRB, 0.87219E+00_JPRB, 0.91536E+00_JPRB, 0.10297E+01_JPRB, 0.14065E+01_JPRB, 0.16052E+01_JPRB, 0.23168E+01_JPRB ,&
&  0.30705E+01_JPRB, 0.26441E+01_JPRB, 0.21507E+01_JPRB, 0.18353E+01_JPRB, 0.16390E+01_JPRB, 0.14923E+01_JPRB, 0.14034E+01_JPRB ,&
&  0.13179E+01_JPRB, 0.12642E+01_JPRB, 0.12292E+01_JPRB, 0.11914E+01_JPRB, 0.11700E+01_JPRB, 0.11249E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.10786E+01_JPRB, 0.10477E+01_JPRB, 0.10136E+01_JPRB, 0.97616E+00_JPRB, 0.97430E+00_JPRB, 0.98435E+00_JPRB, 0.98914E+00_JPRB ,&
&  0.10004E+01_JPRB, 0.10110E+01_JPRB, 0.10843E+01_JPRB, 0.10752E+01_JPRB, 0.11240E+01_JPRB, 0.11699E+01_JPRB, 0.11715E+01_JPRB ,&
&  0.11335E+01_JPRB, 0.11947E+01_JPRB, 0.12195E+01_JPRB, 0.12597E+01_JPRB, 0.13986E+01_JPRB, 0.14821E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.11175E+01_JPRB, 0.10411E+01_JPRB, 0.99924E+00_JPRB, 0.10021E+01_JPRB, 0.10692E+01_JPRB /)
ZBURD(:, 51,  3) = (/ &
&  0.11178E+01_JPRB, 0.10926E+01_JPRB, 0.10521E+01_JPRB, 0.10198E+01_JPRB, 0.10044E+01_JPRB, 0.98422E+00_JPRB, 0.96700E+00_JPRB ,&
&  0.95515E+00_JPRB, 0.93748E+00_JPRB, 0.92271E+00_JPRB, 0.90401E+00_JPRB, 0.88126E+00_JPRB, 0.86180E+00_JPRB, 0.85305E+00_JPRB ,&
&  0.84983E+00_JPRB, 0.83774E+00_JPRB, 0.83983E+00_JPRB, 0.83974E+00_JPRB, 0.83507E+00_JPRB, 0.82160E+00_JPRB, 0.80827E+00_JPRB ,&
&  0.80070E+00_JPRB, 0.78811E+00_JPRB, 0.78720E+00_JPRB, 0.80969E+00_JPRB, 0.82809E+00_JPRB, 0.83068E+00_JPRB, 0.81989E+00_JPRB ,&
&  0.90873E+00_JPRB, 0.13617E+01_JPRB, 0.19654E+01_JPRB, 0.21174E+01_JPRB, 0.19892E+01_JPRB, 0.17807E+01_JPRB, 0.15690E+01_JPRB ,&
&  0.13870E+01_JPRB, 0.12415E+01_JPRB, 0.11532E+01_JPRB, 0.10821E+01_JPRB, 0.10136E+01_JPRB, 0.95696E+00_JPRB, 0.91255E+00_JPRB ,&
&  0.86826E+00_JPRB, 0.84845E+00_JPRB, 0.83821E+00_JPRB, 0.83535E+00_JPRB, 0.83258E+00_JPRB, 0.82980E+00_JPRB, 0.83406E+00_JPRB ,&
&  0.84292E+00_JPRB, 0.85425E+00_JPRB, 0.87971E+00_JPRB, 0.92901E+00_JPRB, 0.11379E+01_JPRB, 0.14725E+01_JPRB, 0.19293E+01_JPRB ,&
&  0.20287E+01_JPRB, 0.20512E+01_JPRB, 0.19046E+01_JPRB, 0.17106E+01_JPRB, 0.15526E+01_JPRB, 0.14278E+01_JPRB, 0.13433E+01_JPRB ,&
&  0.12596E+01_JPRB, 0.11993E+01_JPRB, 0.11558E+01_JPRB, 0.11108E+01_JPRB, 0.10609E+01_JPRB, 0.10397E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10609E+01_JPRB, 0.10562E+01_JPRB, 0.10246E+01_JPRB, 0.98723E+00_JPRB, 0.94493E+00_JPRB, 0.93071E+00_JPRB, 0.92853E+00_JPRB ,&
&  0.93273E+00_JPRB, 0.95158E+00_JPRB, 0.10007E+01_JPRB, 0.10481E+01_JPRB, 0.10855E+01_JPRB, 0.11054E+01_JPRB, 0.11089E+01_JPRB ,&
&  0.11217E+01_JPRB, 0.12061E+01_JPRB, 0.11861E+01_JPRB, 0.13108E+01_JPRB, 0.14564E+01_JPRB, 0.15274E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.12524E+01_JPRB, 0.11415E+01_JPRB, 0.10864E+01_JPRB, 0.10643E+01_JPRB, 0.11126E+01_JPRB /)
ZBURD(:, 52,  3) = (/ &
&  0.11577E+01_JPRB, 0.11320E+01_JPRB, 0.10912E+01_JPRB, 0.10537E+01_JPRB, 0.10112E+01_JPRB, 0.98037E+00_JPRB, 0.95546E+00_JPRB ,&
&  0.93973E+00_JPRB, 0.91620E+00_JPRB, 0.90325E+00_JPRB, 0.89292E+00_JPRB, 0.88747E+00_JPRB, 0.87642E+00_JPRB, 0.86045E+00_JPRB ,&
&  0.84319E+00_JPRB, 0.83164E+00_JPRB, 0.82665E+00_JPRB, 0.82914E+00_JPRB, 0.83749E+00_JPRB, 0.84280E+00_JPRB, 0.83695E+00_JPRB ,&
&  0.82756E+00_JPRB, 0.81334E+00_JPRB, 0.79691E+00_JPRB, 0.81258E+00_JPRB, 0.82908E+00_JPRB, 0.83417E+00_JPRB, 0.82068E+00_JPRB ,&
&  0.86249E+00_JPRB, 0.10299E+01_JPRB, 0.13998E+01_JPRB, 0.15249E+01_JPRB, 0.15320E+01_JPRB, 0.15547E+01_JPRB, 0.14460E+01_JPRB ,&
&  0.13340E+01_JPRB, 0.12021E+01_JPRB, 0.10924E+01_JPRB, 0.10357E+01_JPRB, 0.10000E+01_JPRB, 0.95315E+00_JPRB, 0.91446E+00_JPRB ,&
&  0.86542E+00_JPRB, 0.83988E+00_JPRB, 0.82340E+00_JPRB, 0.81693E+00_JPRB, 0.81397E+00_JPRB, 0.81483E+00_JPRB, 0.82067E+00_JPRB ,&
&  0.82699E+00_JPRB, 0.83064E+00_JPRB, 0.83971E+00_JPRB, 0.86892E+00_JPRB, 0.92426E+00_JPRB, 0.10799E+01_JPRB, 0.12969E+01_JPRB ,&
&  0.14343E+01_JPRB, 0.15312E+01_JPRB, 0.16042E+01_JPRB, 0.15977E+01_JPRB, 0.15016E+01_JPRB, 0.13839E+01_JPRB, 0.12964E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.11472E+01_JPRB, 0.10789E+01_JPRB, 0.10276E+01_JPRB, 0.98196E+00_JPRB, 0.96257E+00_JPRB, 0.98181E+00_JPRB ,&
&  0.99217E+00_JPRB, 0.99330E+00_JPRB, 0.97304E+00_JPRB, 0.94632E+00_JPRB, 0.90829E+00_JPRB, 0.88288E+00_JPRB, 0.87991E+00_JPRB ,&
&  0.89006E+00_JPRB, 0.91274E+00_JPRB, 0.94915E+00_JPRB, 0.99052E+00_JPRB, 0.10280E+01_JPRB, 0.10517E+01_JPRB, 0.10609E+01_JPRB ,&
&  0.10707E+01_JPRB, 0.10968E+01_JPRB, 0.10653E+01_JPRB, 0.12534E+01_JPRB, 0.14346E+01_JPRB, 0.14187E+01_JPRB, 0.13787E+01_JPRB ,&
&  0.13063E+01_JPRB, 0.11997E+01_JPRB, 0.11212E+01_JPRB, 0.10822E+01_JPRB, 0.11219E+01_JPRB /)
ZBURD(:, 53,  3) = (/ &
&  0.99264E+00_JPRB, 0.10079E+01_JPRB, 0.10030E+01_JPRB, 0.98990E+00_JPRB, 0.96233E+00_JPRB, 0.92543E+00_JPRB, 0.89792E+00_JPRB ,&
&  0.88268E+00_JPRB, 0.86144E+00_JPRB, 0.85096E+00_JPRB, 0.84651E+00_JPRB, 0.84913E+00_JPRB, 0.85575E+00_JPRB, 0.85551E+00_JPRB ,&
&  0.84538E+00_JPRB, 0.83011E+00_JPRB, 0.81912E+00_JPRB, 0.81741E+00_JPRB, 0.82641E+00_JPRB, 0.83650E+00_JPRB, 0.84305E+00_JPRB ,&
&  0.84569E+00_JPRB, 0.83893E+00_JPRB, 0.83601E+00_JPRB, 0.84655E+00_JPRB, 0.85503E+00_JPRB, 0.84744E+00_JPRB, 0.82739E+00_JPRB ,&
&  0.83530E+00_JPRB, 0.82352E+00_JPRB, 0.96421E+00_JPRB, 0.11046E+01_JPRB, 0.12196E+01_JPRB, 0.12644E+01_JPRB, 0.12522E+01_JPRB ,&
&  0.11932E+01_JPRB, 0.11285E+01_JPRB, 0.10519E+01_JPRB, 0.98846E+00_JPRB, 0.94166E+00_JPRB, 0.91080E+00_JPRB, 0.89073E+00_JPRB ,&
&  0.86177E+00_JPRB, 0.83364E+00_JPRB, 0.80898E+00_JPRB, 0.79518E+00_JPRB, 0.79018E+00_JPRB, 0.78587E+00_JPRB, 0.78173E+00_JPRB ,&
&  0.77711E+00_JPRB, 0.77395E+00_JPRB, 0.77922E+00_JPRB, 0.79669E+00_JPRB, 0.82717E+00_JPRB, 0.89746E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10909E+01_JPRB, 0.11789E+01_JPRB, 0.12654E+01_JPRB, 0.13367E+01_JPRB, 0.13356E+01_JPRB, 0.12924E+01_JPRB, 0.12278E+01_JPRB ,&
&  0.11535E+01_JPRB, 0.10610E+01_JPRB, 0.98144E+00_JPRB, 0.93182E+00_JPRB, 0.91343E+00_JPRB, 0.91016E+00_JPRB, 0.92579E+00_JPRB ,&
&  0.93398E+00_JPRB, 0.93295E+00_JPRB, 0.91198E+00_JPRB, 0.88987E+00_JPRB, 0.87742E+00_JPRB, 0.85797E+00_JPRB, 0.85286E+00_JPRB ,&
&  0.85448E+00_JPRB, 0.86753E+00_JPRB, 0.89220E+00_JPRB, 0.91734E+00_JPRB, 0.95250E+00_JPRB, 0.98091E+00_JPRB, 0.99431E+00_JPRB ,&
&  0.99982E+00_JPRB, 0.10056E+01_JPRB, 0.10111E+01_JPRB, 0.11304E+01_JPRB, 0.11886E+01_JPRB, 0.12264E+01_JPRB, 0.12195E+01_JPRB ,&
&  0.11957E+01_JPRB, 0.11350E+01_JPRB, 0.10616E+01_JPRB, 0.10248E+01_JPRB, 0.93569E+00_JPRB /)
ZBURD(:, 54,  3) = (/ &
&  0.91765E+00_JPRB, 0.92963E+00_JPRB, 0.92592E+00_JPRB, 0.91633E+00_JPRB, 0.90350E+00_JPRB, 0.87808E+00_JPRB, 0.84935E+00_JPRB ,&
&  0.82139E+00_JPRB, 0.80779E+00_JPRB, 0.80113E+00_JPRB, 0.80329E+00_JPRB, 0.80790E+00_JPRB, 0.81610E+00_JPRB, 0.82841E+00_JPRB ,&
&  0.84022E+00_JPRB, 0.84590E+00_JPRB, 0.85028E+00_JPRB, 0.84827E+00_JPRB, 0.84640E+00_JPRB, 0.84857E+00_JPRB, 0.85277E+00_JPRB ,&
&  0.86149E+00_JPRB, 0.86448E+00_JPRB, 0.86121E+00_JPRB, 0.87011E+00_JPRB, 0.88344E+00_JPRB, 0.87298E+00_JPRB, 0.85372E+00_JPRB ,&
&  0.84428E+00_JPRB, 0.72893E+00_JPRB, 0.76081E+00_JPRB, 0.88998E+00_JPRB, 0.99818E+00_JPRB, 0.11002E+01_JPRB, 0.11258E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.10457E+01_JPRB, 0.99318E+00_JPRB, 0.94380E+00_JPRB, 0.90900E+00_JPRB, 0.87325E+00_JPRB, 0.84883E+00_JPRB ,&
&  0.83274E+00_JPRB, 0.81519E+00_JPRB, 0.78995E+00_JPRB, 0.76893E+00_JPRB, 0.75167E+00_JPRB, 0.73475E+00_JPRB, 0.72530E+00_JPRB ,&
&  0.72562E+00_JPRB, 0.73644E+00_JPRB, 0.74948E+00_JPRB, 0.76624E+00_JPRB, 0.79742E+00_JPRB, 0.83438E+00_JPRB, 0.88587E+00_JPRB ,&
&  0.93379E+00_JPRB, 0.98493E+00_JPRB, 0.10363E+01_JPRB, 0.10796E+01_JPRB, 0.11178E+01_JPRB, 0.11238E+01_JPRB, 0.10837E+01_JPRB ,&
&  0.10100E+01_JPRB, 0.95518E+00_JPRB, 0.91688E+00_JPRB, 0.89127E+00_JPRB, 0.88736E+00_JPRB, 0.89714E+00_JPRB, 0.90197E+00_JPRB ,&
&  0.89344E+00_JPRB, 0.87590E+00_JPRB, 0.85299E+00_JPRB, 0.83186E+00_JPRB, 0.82348E+00_JPRB, 0.81973E+00_JPRB, 0.81182E+00_JPRB ,&
&  0.80456E+00_JPRB, 0.80552E+00_JPRB, 0.81537E+00_JPRB, 0.83715E+00_JPRB, 0.86583E+00_JPRB, 0.89974E+00_JPRB, 0.92258E+00_JPRB ,&
&  0.93804E+00_JPRB, 0.95252E+00_JPRB, 0.96742E+00_JPRB, 0.10189E+01_JPRB, 0.10961E+01_JPRB, 0.11333E+01_JPRB, 0.11512E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.10985E+01_JPRB, 0.10507E+01_JPRB, 0.84332E+00_JPRB, 0.89996E+00_JPRB /)
ZBURD(:, 55,  3) = (/ &
&  0.86623E+00_JPRB, 0.86029E+00_JPRB, 0.85599E+00_JPRB, 0.84466E+00_JPRB, 0.83492E+00_JPRB, 0.82011E+00_JPRB, 0.79491E+00_JPRB ,&
&  0.77180E+00_JPRB, 0.76106E+00_JPRB, 0.75804E+00_JPRB, 0.75721E+00_JPRB, 0.75853E+00_JPRB, 0.76378E+00_JPRB, 0.77481E+00_JPRB ,&
&  0.78849E+00_JPRB, 0.80312E+00_JPRB, 0.82023E+00_JPRB, 0.83588E+00_JPRB, 0.83935E+00_JPRB, 0.83621E+00_JPRB, 0.83648E+00_JPRB ,&
&  0.84258E+00_JPRB, 0.84147E+00_JPRB, 0.83471E+00_JPRB, 0.83705E+00_JPRB, 0.84573E+00_JPRB, 0.83709E+00_JPRB, 0.81877E+00_JPRB ,&
&  0.80795E+00_JPRB, 0.66809E+00_JPRB, 0.70773E+00_JPRB, 0.77956E+00_JPRB, 0.86423E+00_JPRB, 0.94315E+00_JPRB, 0.97605E+00_JPRB ,&
&  0.96682E+00_JPRB, 0.92785E+00_JPRB, 0.89363E+00_JPRB, 0.85281E+00_JPRB, 0.83267E+00_JPRB, 0.81836E+00_JPRB, 0.80138E+00_JPRB ,&
&  0.77662E+00_JPRB, 0.75081E+00_JPRB, 0.72249E+00_JPRB, 0.69746E+00_JPRB, 0.67764E+00_JPRB, 0.66803E+00_JPRB, 0.66490E+00_JPRB ,&
&  0.66496E+00_JPRB, 0.67583E+00_JPRB, 0.68214E+00_JPRB, 0.69252E+00_JPRB, 0.70917E+00_JPRB, 0.73364E+00_JPRB, 0.76372E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.83724E+00_JPRB, 0.87436E+00_JPRB, 0.89999E+00_JPRB, 0.91902E+00_JPRB, 0.93294E+00_JPRB, 0.92567E+00_JPRB ,&
&  0.89479E+00_JPRB, 0.86109E+00_JPRB, 0.85579E+00_JPRB, 0.85722E+00_JPRB, 0.84998E+00_JPRB, 0.84377E+00_JPRB, 0.83403E+00_JPRB ,&
&  0.81638E+00_JPRB, 0.79540E+00_JPRB, 0.77967E+00_JPRB, 0.76319E+00_JPRB, 0.75275E+00_JPRB, 0.74868E+00_JPRB, 0.73436E+00_JPRB ,&
&  0.72491E+00_JPRB, 0.72442E+00_JPRB, 0.73325E+00_JPRB, 0.75426E+00_JPRB, 0.78317E+00_JPRB, 0.81221E+00_JPRB, 0.83308E+00_JPRB ,&
&  0.84642E+00_JPRB, 0.85922E+00_JPRB, 0.86651E+00_JPRB, 0.92175E+00_JPRB, 0.95373E+00_JPRB, 0.98063E+00_JPRB, 0.10048E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.10311E+01_JPRB, 0.77448E+00_JPRB, 0.84974E+00_JPRB, 0.85855E+00_JPRB /)
ZBURD(:, 56,  3) = (/ &
&  0.86258E+00_JPRB, 0.83180E+00_JPRB, 0.81213E+00_JPRB, 0.79904E+00_JPRB, 0.78898E+00_JPRB, 0.77864E+00_JPRB, 0.76103E+00_JPRB ,&
&  0.74209E+00_JPRB, 0.73135E+00_JPRB, 0.72255E+00_JPRB, 0.71247E+00_JPRB, 0.71251E+00_JPRB, 0.71762E+00_JPRB, 0.72839E+00_JPRB ,&
&  0.74046E+00_JPRB, 0.75608E+00_JPRB, 0.77117E+00_JPRB, 0.79020E+00_JPRB, 0.80264E+00_JPRB, 0.81124E+00_JPRB, 0.81578E+00_JPRB ,&
&  0.81890E+00_JPRB, 0.81938E+00_JPRB, 0.81182E+00_JPRB, 0.80205E+00_JPRB, 0.79974E+00_JPRB, 0.80139E+00_JPRB, 0.79646E+00_JPRB ,&
&  0.69689E+00_JPRB, 0.61184E+00_JPRB, 0.66830E+00_JPRB, 0.73386E+00_JPRB, 0.79889E+00_JPRB, 0.85392E+00_JPRB, 0.87855E+00_JPRB ,&
&  0.86600E+00_JPRB, 0.83488E+00_JPRB, 0.80614E+00_JPRB, 0.77227E+00_JPRB, 0.75787E+00_JPRB, 0.75172E+00_JPRB, 0.74245E+00_JPRB ,&
&  0.73232E+00_JPRB, 0.72014E+00_JPRB, 0.70443E+00_JPRB, 0.68002E+00_JPRB, 0.66258E+00_JPRB, 0.64993E+00_JPRB, 0.64737E+00_JPRB ,&
&  0.64924E+00_JPRB, 0.65292E+00_JPRB, 0.65640E+00_JPRB, 0.65916E+00_JPRB, 0.66611E+00_JPRB, 0.67640E+00_JPRB, 0.68855E+00_JPRB ,&
&  0.70632E+00_JPRB, 0.72478E+00_JPRB, 0.75099E+00_JPRB, 0.76917E+00_JPRB, 0.78180E+00_JPRB, 0.79209E+00_JPRB, 0.80368E+00_JPRB ,&
&  0.80751E+00_JPRB, 0.80247E+00_JPRB, 0.78334E+00_JPRB, 0.77367E+00_JPRB, 0.77291E+00_JPRB, 0.77203E+00_JPRB, 0.76465E+00_JPRB ,&
&  0.74929E+00_JPRB, 0.73053E+00_JPRB, 0.71143E+00_JPRB, 0.70080E+00_JPRB, 0.70015E+00_JPRB, 0.69865E+00_JPRB, 0.69366E+00_JPRB ,&
&  0.68956E+00_JPRB, 0.68938E+00_JPRB, 0.69517E+00_JPRB, 0.70666E+00_JPRB, 0.72249E+00_JPRB, 0.74237E+00_JPRB, 0.75949E+00_JPRB ,&
&  0.77765E+00_JPRB, 0.79618E+00_JPRB, 0.80892E+00_JPRB, 0.84225E+00_JPRB, 0.87401E+00_JPRB, 0.89502E+00_JPRB, 0.90484E+00_JPRB ,&
&  0.92041E+00_JPRB, 0.92121E+00_JPRB, 0.91417E+00_JPRB, 0.90661E+00_JPRB, 0.88924E+00_JPRB /)
ZBURD(:, 57,  3) = (/ &
&  0.80211E+00_JPRB, 0.77621E+00_JPRB, 0.75921E+00_JPRB, 0.74985E+00_JPRB, 0.73778E+00_JPRB, 0.71672E+00_JPRB, 0.69966E+00_JPRB ,&
&  0.68320E+00_JPRB, 0.67309E+00_JPRB, 0.66778E+00_JPRB, 0.66567E+00_JPRB, 0.67080E+00_JPRB, 0.67581E+00_JPRB, 0.68231E+00_JPRB ,&
&  0.69338E+00_JPRB, 0.71152E+00_JPRB, 0.72706E+00_JPRB, 0.74136E+00_JPRB, 0.75426E+00_JPRB, 0.76702E+00_JPRB, 0.77034E+00_JPRB ,&
&  0.77296E+00_JPRB, 0.78037E+00_JPRB, 0.78182E+00_JPRB, 0.76937E+00_JPRB, 0.75643E+00_JPRB, 0.75963E+00_JPRB, 0.75686E+00_JPRB ,&
&  0.74722E+00_JPRB, 0.63269E+00_JPRB, 0.68721E+00_JPRB, 0.71329E+00_JPRB, 0.74322E+00_JPRB, 0.77174E+00_JPRB, 0.78696E+00_JPRB ,&
&  0.78264E+00_JPRB, 0.76425E+00_JPRB, 0.74341E+00_JPRB, 0.71591E+00_JPRB, 0.70762E+00_JPRB, 0.70046E+00_JPRB, 0.69132E+00_JPRB ,&
&  0.68535E+00_JPRB, 0.67942E+00_JPRB, 0.67096E+00_JPRB, 0.65927E+00_JPRB, 0.64428E+00_JPRB, 0.63317E+00_JPRB, 0.62336E+00_JPRB ,&
&  0.62064E+00_JPRB, 0.61306E+00_JPRB, 0.60616E+00_JPRB, 0.60687E+00_JPRB, 0.61863E+00_JPRB, 0.62425E+00_JPRB, 0.62478E+00_JPRB ,&
&  0.62564E+00_JPRB, 0.62842E+00_JPRB, 0.63869E+00_JPRB, 0.65446E+00_JPRB, 0.66771E+00_JPRB, 0.67708E+00_JPRB, 0.69011E+00_JPRB ,&
&  0.71072E+00_JPRB, 0.72554E+00_JPRB, 0.72599E+00_JPRB, 0.71426E+00_JPRB, 0.70681E+00_JPRB, 0.70614E+00_JPRB, 0.69576E+00_JPRB ,&
&  0.67859E+00_JPRB, 0.66769E+00_JPRB, 0.66509E+00_JPRB, 0.66435E+00_JPRB, 0.66076E+00_JPRB, 0.65493E+00_JPRB, 0.65406E+00_JPRB ,&
&  0.65530E+00_JPRB, 0.65577E+00_JPRB, 0.65498E+00_JPRB, 0.65767E+00_JPRB, 0.66395E+00_JPRB, 0.67245E+00_JPRB, 0.68419E+00_JPRB ,&
&  0.70136E+00_JPRB, 0.71810E+00_JPRB, 0.72781E+00_JPRB, 0.74595E+00_JPRB, 0.76702E+00_JPRB, 0.77934E+00_JPRB, 0.79999E+00_JPRB ,&
&  0.80804E+00_JPRB, 0.81209E+00_JPRB, 0.81431E+00_JPRB, 0.81975E+00_JPRB, 0.81948E+00_JPRB /)
ZBURD(:, 58,  3) = (/ &
&  0.74778E+00_JPRB, 0.72520E+00_JPRB, 0.71024E+00_JPRB, 0.69669E+00_JPRB, 0.68697E+00_JPRB, 0.66776E+00_JPRB, 0.65666E+00_JPRB ,&
&  0.64947E+00_JPRB, 0.64465E+00_JPRB, 0.64262E+00_JPRB, 0.64182E+00_JPRB, 0.65166E+00_JPRB, 0.65807E+00_JPRB, 0.66042E+00_JPRB ,&
&  0.66789E+00_JPRB, 0.68533E+00_JPRB, 0.70141E+00_JPRB, 0.71486E+00_JPRB, 0.72540E+00_JPRB, 0.73300E+00_JPRB, 0.73278E+00_JPRB ,&
&  0.73641E+00_JPRB, 0.74431E+00_JPRB, 0.74813E+00_JPRB, 0.73801E+00_JPRB, 0.72530E+00_JPRB, 0.71876E+00_JPRB, 0.71961E+00_JPRB ,&
&  0.70902E+00_JPRB, 0.66600E+00_JPRB, 0.68097E+00_JPRB, 0.69280E+00_JPRB, 0.70683E+00_JPRB, 0.71502E+00_JPRB, 0.72366E+00_JPRB ,&
&  0.72596E+00_JPRB, 0.70886E+00_JPRB, 0.68707E+00_JPRB, 0.67139E+00_JPRB, 0.67477E+00_JPRB, 0.67194E+00_JPRB, 0.66233E+00_JPRB ,&
&  0.65795E+00_JPRB, 0.65426E+00_JPRB, 0.64669E+00_JPRB, 0.63364E+00_JPRB, 0.62190E+00_JPRB, 0.61393E+00_JPRB, 0.60859E+00_JPRB ,&
&  0.59721E+00_JPRB, 0.58778E+00_JPRB, 0.58268E+00_JPRB, 0.57708E+00_JPRB, 0.57243E+00_JPRB, 0.57669E+00_JPRB, 0.58029E+00_JPRB ,&
&  0.57989E+00_JPRB, 0.58112E+00_JPRB, 0.58948E+00_JPRB, 0.60441E+00_JPRB, 0.61757E+00_JPRB, 0.63250E+00_JPRB, 0.64970E+00_JPRB ,&
&  0.66673E+00_JPRB, 0.68186E+00_JPRB, 0.68708E+00_JPRB, 0.69099E+00_JPRB, 0.68539E+00_JPRB, 0.67639E+00_JPRB, 0.65823E+00_JPRB ,&
&  0.63824E+00_JPRB, 0.63448E+00_JPRB, 0.63561E+00_JPRB, 0.64172E+00_JPRB, 0.64039E+00_JPRB, 0.63761E+00_JPRB, 0.63579E+00_JPRB ,&
&  0.63162E+00_JPRB, 0.62801E+00_JPRB, 0.62304E+00_JPRB, 0.61842E+00_JPRB, 0.62166E+00_JPRB, 0.62763E+00_JPRB, 0.64121E+00_JPRB ,&
&  0.65947E+00_JPRB, 0.67917E+00_JPRB, 0.69957E+00_JPRB, 0.71585E+00_JPRB, 0.72392E+00_JPRB, 0.72951E+00_JPRB, 0.73553E+00_JPRB ,&
&  0.73876E+00_JPRB, 0.74008E+00_JPRB, 0.74904E+00_JPRB, 0.75472E+00_JPRB, 0.75931E+00_JPRB /)
ZBURD(:, 59,  3) = (/ &
&  0.68366E+00_JPRB, 0.65848E+00_JPRB, 0.63903E+00_JPRB, 0.63144E+00_JPRB, 0.62725E+00_JPRB, 0.61444E+00_JPRB, 0.60830E+00_JPRB ,&
&  0.61394E+00_JPRB, 0.61553E+00_JPRB, 0.62206E+00_JPRB, 0.62106E+00_JPRB, 0.62002E+00_JPRB, 0.62256E+00_JPRB, 0.61487E+00_JPRB ,&
&  0.61487E+00_JPRB, 0.62393E+00_JPRB, 0.63459E+00_JPRB, 0.64896E+00_JPRB, 0.66098E+00_JPRB, 0.67004E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.68069E+00_JPRB, 0.68715E+00_JPRB, 0.69067E+00_JPRB, 0.68638E+00_JPRB, 0.67530E+00_JPRB, 0.66664E+00_JPRB, 0.66521E+00_JPRB ,&
&  0.66380E+00_JPRB, 0.65913E+00_JPRB, 0.61473E+00_JPRB, 0.63888E+00_JPRB, 0.64603E+00_JPRB, 0.65010E+00_JPRB, 0.65314E+00_JPRB ,&
&  0.64867E+00_JPRB, 0.63416E+00_JPRB, 0.62701E+00_JPRB, 0.61455E+00_JPRB, 0.61313E+00_JPRB, 0.61892E+00_JPRB, 0.61697E+00_JPRB ,&
&  0.61219E+00_JPRB, 0.60141E+00_JPRB, 0.58724E+00_JPRB, 0.57636E+00_JPRB, 0.57367E+00_JPRB, 0.57164E+00_JPRB, 0.56387E+00_JPRB ,&
&  0.55856E+00_JPRB, 0.55769E+00_JPRB, 0.55596E+00_JPRB, 0.55012E+00_JPRB, 0.53534E+00_JPRB, 0.52182E+00_JPRB, 0.51923E+00_JPRB ,&
&  0.52258E+00_JPRB, 0.52963E+00_JPRB, 0.54362E+00_JPRB, 0.56195E+00_JPRB, 0.58180E+00_JPRB, 0.60048E+00_JPRB, 0.61749E+00_JPRB ,&
&  0.62448E+00_JPRB, 0.62838E+00_JPRB, 0.63229E+00_JPRB, 0.63723E+00_JPRB, 0.63815E+00_JPRB, 0.63016E+00_JPRB, 0.61027E+00_JPRB ,&
&  0.58919E+00_JPRB, 0.58354E+00_JPRB, 0.58742E+00_JPRB, 0.59643E+00_JPRB, 0.60095E+00_JPRB, 0.60027E+00_JPRB, 0.59347E+00_JPRB ,&
&  0.58899E+00_JPRB, 0.58042E+00_JPRB, 0.57153E+00_JPRB, 0.56723E+00_JPRB, 0.56703E+00_JPRB, 0.57324E+00_JPRB, 0.59078E+00_JPRB ,&
&  0.61180E+00_JPRB, 0.63533E+00_JPRB, 0.65642E+00_JPRB, 0.66500E+00_JPRB, 0.67048E+00_JPRB, 0.67070E+00_JPRB, 0.66251E+00_JPRB ,&
&  0.66746E+00_JPRB, 0.68714E+00_JPRB, 0.69813E+00_JPRB, 0.70056E+00_JPRB, 0.69889E+00_JPRB /)
ZBURD(:, 60,  3) = (/ &
&  0.62947E+00_JPRB, 0.60093E+00_JPRB, 0.58870E+00_JPRB, 0.59002E+00_JPRB, 0.59315E+00_JPRB, 0.58846E+00_JPRB, 0.59524E+00_JPRB ,&
&  0.60619E+00_JPRB, 0.61451E+00_JPRB, 0.61879E+00_JPRB, 0.61462E+00_JPRB, 0.61186E+00_JPRB, 0.60885E+00_JPRB, 0.59406E+00_JPRB ,&
&  0.58918E+00_JPRB, 0.58748E+00_JPRB, 0.59166E+00_JPRB, 0.59882E+00_JPRB, 0.60661E+00_JPRB, 0.61885E+00_JPRB, 0.62790E+00_JPRB ,&
&  0.63382E+00_JPRB, 0.63748E+00_JPRB, 0.64159E+00_JPRB, 0.64173E+00_JPRB, 0.63808E+00_JPRB, 0.62882E+00_JPRB, 0.62174E+00_JPRB ,&
&  0.61752E+00_JPRB, 0.60626E+00_JPRB, 0.59585E+00_JPRB, 0.59450E+00_JPRB, 0.59763E+00_JPRB, 0.60075E+00_JPRB, 0.60056E+00_JPRB ,&
&  0.58839E+00_JPRB, 0.57212E+00_JPRB, 0.57062E+00_JPRB, 0.57033E+00_JPRB, 0.57531E+00_JPRB, 0.58729E+00_JPRB, 0.59049E+00_JPRB ,&
&  0.58690E+00_JPRB, 0.57245E+00_JPRB, 0.55306E+00_JPRB, 0.54934E+00_JPRB, 0.55165E+00_JPRB, 0.54488E+00_JPRB, 0.53824E+00_JPRB ,&
&  0.53706E+00_JPRB, 0.53409E+00_JPRB, 0.52812E+00_JPRB, 0.51997E+00_JPRB, 0.50717E+00_JPRB, 0.49052E+00_JPRB, 0.48192E+00_JPRB ,&
&  0.48120E+00_JPRB, 0.49613E+00_JPRB, 0.52310E+00_JPRB, 0.55363E+00_JPRB, 0.58274E+00_JPRB, 0.59702E+00_JPRB, 0.59656E+00_JPRB ,&
&  0.58471E+00_JPRB, 0.58195E+00_JPRB, 0.58721E+00_JPRB, 0.59168E+00_JPRB, 0.59272E+00_JPRB, 0.58202E+00_JPRB, 0.56480E+00_JPRB ,&
&  0.55230E+00_JPRB, 0.54293E+00_JPRB, 0.54179E+00_JPRB, 0.54673E+00_JPRB, 0.55159E+00_JPRB, 0.55273E+00_JPRB, 0.54658E+00_JPRB ,&
&  0.54010E+00_JPRB, 0.53332E+00_JPRB, 0.52936E+00_JPRB, 0.53207E+00_JPRB, 0.53732E+00_JPRB, 0.54263E+00_JPRB, 0.55556E+00_JPRB ,&
&  0.57419E+00_JPRB, 0.59292E+00_JPRB, 0.59744E+00_JPRB, 0.59380E+00_JPRB, 0.59379E+00_JPRB, 0.59118E+00_JPRB, 0.58867E+00_JPRB ,&
&  0.59895E+00_JPRB, 0.62187E+00_JPRB, 0.63881E+00_JPRB, 0.64882E+00_JPRB, 0.64964E+00_JPRB /)
ZBURD(:, 61,  3) = (/ &
&  0.55321E+00_JPRB, 0.53607E+00_JPRB, 0.53617E+00_JPRB, 0.54323E+00_JPRB, 0.54717E+00_JPRB, 0.55473E+00_JPRB, 0.57389E+00_JPRB ,&
&  0.58499E+00_JPRB, 0.59207E+00_JPRB, 0.59471E+00_JPRB, 0.58912E+00_JPRB, 0.58795E+00_JPRB, 0.58064E+00_JPRB, 0.56781E+00_JPRB ,&
&  0.56249E+00_JPRB, 0.56148E+00_JPRB, 0.56319E+00_JPRB, 0.56705E+00_JPRB, 0.56933E+00_JPRB, 0.57221E+00_JPRB, 0.57548E+00_JPRB ,&
&  0.57723E+00_JPRB, 0.57717E+00_JPRB, 0.57625E+00_JPRB, 0.57643E+00_JPRB, 0.57541E+00_JPRB, 0.56771E+00_JPRB, 0.55900E+00_JPRB ,&
&  0.54959E+00_JPRB, 0.54046E+00_JPRB, 0.53778E+00_JPRB, 0.54204E+00_JPRB, 0.54693E+00_JPRB, 0.54991E+00_JPRB, 0.54762E+00_JPRB ,&
&  0.54316E+00_JPRB, 0.54100E+00_JPRB, 0.54370E+00_JPRB, 0.55063E+00_JPRB, 0.55911E+00_JPRB, 0.56864E+00_JPRB, 0.57367E+00_JPRB ,&
&  0.56794E+00_JPRB, 0.55012E+00_JPRB, 0.53920E+00_JPRB, 0.53861E+00_JPRB, 0.53483E+00_JPRB, 0.52007E+00_JPRB, 0.50978E+00_JPRB ,&
&  0.50499E+00_JPRB, 0.49615E+00_JPRB, 0.48681E+00_JPRB, 0.47412E+00_JPRB, 0.46253E+00_JPRB, 0.45818E+00_JPRB, 0.45684E+00_JPRB ,&
&  0.45976E+00_JPRB, 0.47773E+00_JPRB, 0.51105E+00_JPRB, 0.54358E+00_JPRB, 0.55790E+00_JPRB, 0.55156E+00_JPRB, 0.53478E+00_JPRB ,&
&  0.52708E+00_JPRB, 0.52379E+00_JPRB, 0.52516E+00_JPRB, 0.52491E+00_JPRB, 0.52158E+00_JPRB, 0.51280E+00_JPRB, 0.50558E+00_JPRB ,&
&  0.49526E+00_JPRB, 0.48384E+00_JPRB, 0.47920E+00_JPRB, 0.47644E+00_JPRB, 0.47348E+00_JPRB, 0.47360E+00_JPRB, 0.47276E+00_JPRB ,&
&  0.47330E+00_JPRB, 0.47712E+00_JPRB, 0.48859E+00_JPRB, 0.49488E+00_JPRB, 0.49571E+00_JPRB, 0.48540E+00_JPRB, 0.48642E+00_JPRB ,&
&  0.48929E+00_JPRB, 0.48719E+00_JPRB, 0.48278E+00_JPRB, 0.48273E+00_JPRB, 0.48278E+00_JPRB, 0.48485E+00_JPRB, 0.49668E+00_JPRB ,&
&  0.52371E+00_JPRB, 0.54735E+00_JPRB, 0.56694E+00_JPRB, 0.57318E+00_JPRB, 0.56993E+00_JPRB /)
ZBURD(:, 62,  3) = (/ &
&  0.49596E+00_JPRB, 0.50129E+00_JPRB, 0.51450E+00_JPRB, 0.52538E+00_JPRB, 0.52876E+00_JPRB, 0.53810E+00_JPRB, 0.54905E+00_JPRB ,&
&  0.55798E+00_JPRB, 0.55960E+00_JPRB, 0.55571E+00_JPRB, 0.54536E+00_JPRB, 0.53319E+00_JPRB, 0.52317E+00_JPRB, 0.51918E+00_JPRB ,&
&  0.52140E+00_JPRB, 0.52452E+00_JPRB, 0.53010E+00_JPRB, 0.53318E+00_JPRB, 0.53688E+00_JPRB, 0.53570E+00_JPRB, 0.53615E+00_JPRB ,&
&  0.53798E+00_JPRB, 0.53901E+00_JPRB, 0.53950E+00_JPRB, 0.53578E+00_JPRB, 0.53279E+00_JPRB, 0.53020E+00_JPRB, 0.52152E+00_JPRB ,&
&  0.51187E+00_JPRB, 0.50546E+00_JPRB, 0.50934E+00_JPRB, 0.52066E+00_JPRB, 0.52782E+00_JPRB, 0.52485E+00_JPRB, 0.52338E+00_JPRB ,&
&  0.52361E+00_JPRB, 0.52869E+00_JPRB, 0.53686E+00_JPRB, 0.54889E+00_JPRB, 0.55719E+00_JPRB, 0.56812E+00_JPRB, 0.56873E+00_JPRB ,&
&  0.56228E+00_JPRB, 0.55293E+00_JPRB, 0.54989E+00_JPRB, 0.54862E+00_JPRB, 0.51949E+00_JPRB, 0.48750E+00_JPRB, 0.47182E+00_JPRB ,&
&  0.46341E+00_JPRB, 0.45299E+00_JPRB, 0.43721E+00_JPRB, 0.42616E+00_JPRB, 0.42504E+00_JPRB, 0.42994E+00_JPRB, 0.44139E+00_JPRB ,&
&  0.44964E+00_JPRB, 0.46198E+00_JPRB, 0.47705E+00_JPRB, 0.48685E+00_JPRB, 0.48701E+00_JPRB, 0.47485E+00_JPRB, 0.46361E+00_JPRB ,&
&  0.45693E+00_JPRB, 0.45503E+00_JPRB, 0.45350E+00_JPRB, 0.45145E+00_JPRB, 0.44952E+00_JPRB, 0.44898E+00_JPRB, 0.44796E+00_JPRB ,&
&  0.44370E+00_JPRB, 0.43636E+00_JPRB, 0.42742E+00_JPRB, 0.42245E+00_JPRB, 0.42156E+00_JPRB, 0.42168E+00_JPRB, 0.42342E+00_JPRB ,&
&  0.43238E+00_JPRB, 0.44361E+00_JPRB, 0.45260E+00_JPRB, 0.45654E+00_JPRB, 0.45339E+00_JPRB, 0.44533E+00_JPRB, 0.44044E+00_JPRB ,&
&  0.43675E+00_JPRB, 0.43415E+00_JPRB, 0.43077E+00_JPRB, 0.42922E+00_JPRB, 0.42906E+00_JPRB, 0.43657E+00_JPRB, 0.45257E+00_JPRB ,&
&  0.47239E+00_JPRB, 0.49807E+00_JPRB, 0.50978E+00_JPRB, 0.50852E+00_JPRB, 0.50183E+00_JPRB /)
ZBURD(:, 63,  3) = (/ &
&  0.46548E+00_JPRB, 0.47851E+00_JPRB, 0.49486E+00_JPRB, 0.50633E+00_JPRB, 0.51109E+00_JPRB, 0.51635E+00_JPRB, 0.51866E+00_JPRB ,&
&  0.51340E+00_JPRB, 0.51119E+00_JPRB, 0.50558E+00_JPRB, 0.49751E+00_JPRB, 0.49052E+00_JPRB, 0.48700E+00_JPRB, 0.48739E+00_JPRB ,&
&  0.48957E+00_JPRB, 0.49144E+00_JPRB, 0.48798E+00_JPRB, 0.48621E+00_JPRB, 0.49013E+00_JPRB, 0.49548E+00_JPRB, 0.49898E+00_JPRB ,&
&  0.50042E+00_JPRB, 0.50191E+00_JPRB, 0.50670E+00_JPRB, 0.50973E+00_JPRB, 0.50985E+00_JPRB, 0.50325E+00_JPRB, 0.49831E+00_JPRB ,&
&  0.49327E+00_JPRB, 0.49149E+00_JPRB, 0.49790E+00_JPRB, 0.48997E+00_JPRB, 0.50463E+00_JPRB, 0.50644E+00_JPRB, 0.51185E+00_JPRB ,&
&  0.51994E+00_JPRB, 0.53355E+00_JPRB, 0.54419E+00_JPRB, 0.55240E+00_JPRB, 0.55489E+00_JPRB, 0.55805E+00_JPRB, 0.54818E+00_JPRB ,&
&  0.53442E+00_JPRB, 0.51576E+00_JPRB, 0.49700E+00_JPRB, 0.47659E+00_JPRB, 0.45683E+00_JPRB, 0.44151E+00_JPRB, 0.43410E+00_JPRB ,&
&  0.42511E+00_JPRB, 0.41121E+00_JPRB, 0.39787E+00_JPRB, 0.39269E+00_JPRB, 0.39735E+00_JPRB, 0.40753E+00_JPRB, 0.42302E+00_JPRB ,&
&  0.43246E+00_JPRB, 0.43503E+00_JPRB, 0.43180E+00_JPRB, 0.42759E+00_JPRB, 0.42323E+00_JPRB, 0.41597E+00_JPRB, 0.41173E+00_JPRB ,&
&  0.40830E+00_JPRB, 0.40266E+00_JPRB, 0.39980E+00_JPRB, 0.40150E+00_JPRB, 0.40460E+00_JPRB, 0.40663E+00_JPRB, 0.40522E+00_JPRB ,&
&  0.40235E+00_JPRB, 0.39783E+00_JPRB, 0.38979E+00_JPRB, 0.38546E+00_JPRB, 0.38589E+00_JPRB, 0.39045E+00_JPRB, 0.39809E+00_JPRB ,&
&  0.40056E+00_JPRB, 0.40658E+00_JPRB, 0.41239E+00_JPRB, 0.41442E+00_JPRB, 0.41692E+00_JPRB, 0.41888E+00_JPRB, 0.41892E+00_JPRB ,&
&  0.41711E+00_JPRB, 0.41204E+00_JPRB, 0.40510E+00_JPRB, 0.40225E+00_JPRB, 0.40350E+00_JPRB, 0.41047E+00_JPRB, 0.41986E+00_JPRB ,&
&  0.42914E+00_JPRB, 0.44328E+00_JPRB, 0.45419E+00_JPRB, 0.45675E+00_JPRB, 0.45915E+00_JPRB /)
ZBURD(:, 64,  3) = (/ &
&  0.44525E+00_JPRB, 0.45686E+00_JPRB, 0.47034E+00_JPRB, 0.47963E+00_JPRB, 0.47852E+00_JPRB, 0.47836E+00_JPRB, 0.48443E+00_JPRB ,&
&  0.47820E+00_JPRB, 0.47089E+00_JPRB, 0.46494E+00_JPRB, 0.45887E+00_JPRB, 0.45678E+00_JPRB, 0.45552E+00_JPRB, 0.45856E+00_JPRB ,&
&  0.46076E+00_JPRB, 0.45970E+00_JPRB, 0.45807E+00_JPRB, 0.45884E+00_JPRB, 0.46548E+00_JPRB, 0.47379E+00_JPRB, 0.48075E+00_JPRB ,&
&  0.48211E+00_JPRB, 0.48571E+00_JPRB, 0.49149E+00_JPRB, 0.49642E+00_JPRB, 0.49889E+00_JPRB, 0.50039E+00_JPRB, 0.49916E+00_JPRB ,&
&  0.49657E+00_JPRB, 0.49416E+00_JPRB, 0.47716E+00_JPRB, 0.49052E+00_JPRB, 0.49498E+00_JPRB, 0.49577E+00_JPRB, 0.50106E+00_JPRB ,&
&  0.50878E+00_JPRB, 0.51764E+00_JPRB, 0.52179E+00_JPRB, 0.52312E+00_JPRB, 0.51841E+00_JPRB, 0.50899E+00_JPRB, 0.48460E+00_JPRB ,&
&  0.46411E+00_JPRB, 0.44285E+00_JPRB, 0.42955E+00_JPRB, 0.42141E+00_JPRB, 0.41496E+00_JPRB, 0.40945E+00_JPRB, 0.40431E+00_JPRB ,&
&  0.39720E+00_JPRB, 0.39084E+00_JPRB, 0.38694E+00_JPRB, 0.38691E+00_JPRB, 0.39100E+00_JPRB, 0.40183E+00_JPRB, 0.41518E+00_JPRB ,&
&  0.42100E+00_JPRB, 0.41329E+00_JPRB, 0.40509E+00_JPRB, 0.39620E+00_JPRB, 0.38165E+00_JPRB, 0.35434E+00_JPRB, 0.33765E+00_JPRB ,&
&  0.33370E+00_JPRB, 0.34626E+00_JPRB, 0.36725E+00_JPRB, 0.37325E+00_JPRB, 0.37865E+00_JPRB, 0.37634E+00_JPRB, 0.37098E+00_JPRB ,&
&  0.36197E+00_JPRB, 0.33883E+00_JPRB, 0.32656E+00_JPRB, 0.32307E+00_JPRB, 0.32096E+00_JPRB, 0.32144E+00_JPRB, 0.32415E+00_JPRB ,&
&  0.32849E+00_JPRB, 0.33571E+00_JPRB, 0.34072E+00_JPRB, 0.34271E+00_JPRB, 0.34567E+00_JPRB, 0.34581E+00_JPRB, 0.34468E+00_JPRB ,&
&  0.34111E+00_JPRB, 0.34790E+00_JPRB, 0.36644E+00_JPRB, 0.39477E+00_JPRB, 0.40127E+00_JPRB, 0.40906E+00_JPRB, 0.41805E+00_JPRB ,&
&  0.42354E+00_JPRB, 0.42889E+00_JPRB, 0.42958E+00_JPRB, 0.43062E+00_JPRB, 0.43608E+00_JPRB /)
ZBURD(:, 65,  3) = (/ &
&  0.42137E+00_JPRB, 0.42847E+00_JPRB, 0.43690E+00_JPRB, 0.44787E+00_JPRB, 0.45017E+00_JPRB, 0.45171E+00_JPRB, 0.46078E+00_JPRB ,&
&  0.46110E+00_JPRB, 0.45808E+00_JPRB, 0.45487E+00_JPRB, 0.45518E+00_JPRB, 0.45734E+00_JPRB, 0.45976E+00_JPRB, 0.45932E+00_JPRB ,&
&  0.45015E+00_JPRB, 0.44489E+00_JPRB, 0.44766E+00_JPRB, 0.45071E+00_JPRB, 0.45531E+00_JPRB, 0.46179E+00_JPRB, 0.46747E+00_JPRB ,&
&  0.47057E+00_JPRB, 0.47423E+00_JPRB, 0.47680E+00_JPRB, 0.48309E+00_JPRB, 0.49274E+00_JPRB, 0.49941E+00_JPRB, 0.50562E+00_JPRB ,&
&  0.49104E+00_JPRB, 0.44329E+00_JPRB, 0.41296E+00_JPRB, 0.41919E+00_JPRB, 0.46800E+00_JPRB, 0.47841E+00_JPRB, 0.47928E+00_JPRB ,&
&  0.48225E+00_JPRB, 0.48977E+00_JPRB, 0.48924E+00_JPRB, 0.48341E+00_JPRB, 0.47153E+00_JPRB, 0.45244E+00_JPRB, 0.42915E+00_JPRB ,&
&  0.41357E+00_JPRB, 0.40328E+00_JPRB, 0.39648E+00_JPRB, 0.38970E+00_JPRB, 0.38617E+00_JPRB, 0.38127E+00_JPRB, 0.37363E+00_JPRB ,&
&  0.37060E+00_JPRB, 0.36930E+00_JPRB, 0.37040E+00_JPRB, 0.37191E+00_JPRB, 0.37375E+00_JPRB, 0.37495E+00_JPRB, 0.37151E+00_JPRB ,&
&  0.35934E+00_JPRB, 0.34361E+00_JPRB, 0.32861E+00_JPRB, 0.31228E+00_JPRB, 0.29840E+00_JPRB, 0.28974E+00_JPRB, 0.28276E+00_JPRB ,&
&  0.28003E+00_JPRB, 0.28592E+00_JPRB, 0.30319E+00_JPRB, 0.32291E+00_JPRB, 0.35413E+00_JPRB, 0.32937E+00_JPRB, 0.30592E+00_JPRB ,&
&  0.29469E+00_JPRB, 0.28924E+00_JPRB, 0.28622E+00_JPRB, 0.28631E+00_JPRB, 0.28799E+00_JPRB, 0.28849E+00_JPRB, 0.28860E+00_JPRB ,&
&  0.28996E+00_JPRB, 0.29245E+00_JPRB, 0.29434E+00_JPRB, 0.29598E+00_JPRB, 0.29850E+00_JPRB, 0.30140E+00_JPRB, 0.30293E+00_JPRB ,&
&  0.30371E+00_JPRB, 0.30856E+00_JPRB, 0.31827E+00_JPRB, 0.32955E+00_JPRB, 0.34075E+00_JPRB, 0.35240E+00_JPRB, 0.36997E+00_JPRB ,&
&  0.39175E+00_JPRB, 0.41776E+00_JPRB, 0.41744E+00_JPRB, 0.41786E+00_JPRB, 0.41824E+00_JPRB /)
ZBURD(:, 66,  3) = (/ &
&  0.40908E+00_JPRB, 0.41958E+00_JPRB, 0.42496E+00_JPRB, 0.43029E+00_JPRB, 0.43249E+00_JPRB, 0.43386E+00_JPRB, 0.44177E+00_JPRB ,&
&  0.44660E+00_JPRB, 0.45040E+00_JPRB, 0.45278E+00_JPRB, 0.45388E+00_JPRB, 0.44977E+00_JPRB, 0.44319E+00_JPRB, 0.44119E+00_JPRB ,&
&  0.44004E+00_JPRB, 0.44055E+00_JPRB, 0.43954E+00_JPRB, 0.43953E+00_JPRB, 0.44003E+00_JPRB, 0.44480E+00_JPRB, 0.45167E+00_JPRB ,&
&  0.45330E+00_JPRB, 0.45774E+00_JPRB, 0.47054E+00_JPRB, 0.47828E+00_JPRB, 0.48318E+00_JPRB, 0.48266E+00_JPRB, 0.47681E+00_JPRB ,&
&  0.44894E+00_JPRB, 0.42086E+00_JPRB, 0.39381E+00_JPRB, 0.39526E+00_JPRB, 0.44793E+00_JPRB, 0.45407E+00_JPRB, 0.45352E+00_JPRB ,&
&  0.45200E+00_JPRB, 0.45625E+00_JPRB, 0.45641E+00_JPRB, 0.44178E+00_JPRB, 0.43112E+00_JPRB, 0.41647E+00_JPRB, 0.40375E+00_JPRB ,&
&  0.39271E+00_JPRB, 0.38500E+00_JPRB, 0.37167E+00_JPRB, 0.34419E+00_JPRB, 0.32690E+00_JPRB, 0.31807E+00_JPRB, 0.30436E+00_JPRB ,&
&  0.29397E+00_JPRB, 0.29011E+00_JPRB, 0.28917E+00_JPRB, 0.28845E+00_JPRB, 0.29247E+00_JPRB, 0.29447E+00_JPRB, 0.29305E+00_JPRB ,&
&  0.28955E+00_JPRB, 0.28664E+00_JPRB, 0.28285E+00_JPRB, 0.27761E+00_JPRB, 0.27483E+00_JPRB, 0.27398E+00_JPRB, 0.27450E+00_JPRB ,&
&  0.27851E+00_JPRB, 0.28464E+00_JPRB, 0.29819E+00_JPRB, 0.30774E+00_JPRB, 0.30383E+00_JPRB, 0.29243E+00_JPRB, 0.28215E+00_JPRB ,&
&  0.27539E+00_JPRB, 0.27265E+00_JPRB, 0.27059E+00_JPRB, 0.26845E+00_JPRB, 0.26907E+00_JPRB, 0.27115E+00_JPRB, 0.27261E+00_JPRB ,&
&  0.27289E+00_JPRB, 0.27466E+00_JPRB, 0.27571E+00_JPRB, 0.27695E+00_JPRB, 0.27802E+00_JPRB, 0.27977E+00_JPRB, 0.28217E+00_JPRB ,&
&  0.28444E+00_JPRB, 0.28974E+00_JPRB, 0.29562E+00_JPRB, 0.30225E+00_JPRB, 0.30643E+00_JPRB, 0.30956E+00_JPRB, 0.31586E+00_JPRB ,&
&  0.32217E+00_JPRB, 0.33791E+00_JPRB, 0.36906E+00_JPRB, 0.40563E+00_JPRB, 0.39952E+00_JPRB /)
ZBURD(:, 67,  3) = (/ &
&  0.39643E+00_JPRB, 0.40521E+00_JPRB, 0.41292E+00_JPRB, 0.41890E+00_JPRB, 0.42267E+00_JPRB, 0.42343E+00_JPRB, 0.42617E+00_JPRB ,&
&  0.43369E+00_JPRB, 0.43808E+00_JPRB, 0.44137E+00_JPRB, 0.44056E+00_JPRB, 0.44000E+00_JPRB, 0.42148E+00_JPRB, 0.41695E+00_JPRB ,&
&  0.41096E+00_JPRB, 0.40677E+00_JPRB, 0.40092E+00_JPRB, 0.40079E+00_JPRB, 0.40054E+00_JPRB, 0.40257E+00_JPRB, 0.40334E+00_JPRB ,&
&  0.40361E+00_JPRB, 0.40189E+00_JPRB, 0.39881E+00_JPRB, 0.39693E+00_JPRB, 0.39818E+00_JPRB, 0.39951E+00_JPRB, 0.39651E+00_JPRB ,&
&  0.39216E+00_JPRB, 0.39114E+00_JPRB, 0.39432E+00_JPRB, 0.40141E+00_JPRB, 0.44155E+00_JPRB, 0.43616E+00_JPRB, 0.43393E+00_JPRB ,&
&  0.43063E+00_JPRB, 0.42643E+00_JPRB, 0.42580E+00_JPRB, 0.41918E+00_JPRB, 0.40955E+00_JPRB, 0.38790E+00_JPRB, 0.37584E+00_JPRB ,&
&  0.34146E+00_JPRB, 0.32251E+00_JPRB, 0.30588E+00_JPRB, 0.29506E+00_JPRB, 0.28475E+00_JPRB, 0.27878E+00_JPRB, 0.27344E+00_JPRB ,&
&  0.26937E+00_JPRB, 0.26614E+00_JPRB, 0.26326E+00_JPRB, 0.26381E+00_JPRB, 0.26560E+00_JPRB, 0.26657E+00_JPRB, 0.26755E+00_JPRB ,&
&  0.26810E+00_JPRB, 0.26860E+00_JPRB, 0.26923E+00_JPRB, 0.27059E+00_JPRB, 0.27240E+00_JPRB, 0.27480E+00_JPRB, 0.27786E+00_JPRB ,&
&  0.28282E+00_JPRB, 0.28671E+00_JPRB, 0.28819E+00_JPRB, 0.29088E+00_JPRB, 0.29057E+00_JPRB, 0.28479E+00_JPRB, 0.27818E+00_JPRB ,&
&  0.27129E+00_JPRB, 0.26769E+00_JPRB, 0.26415E+00_JPRB, 0.26167E+00_JPRB, 0.26106E+00_JPRB, 0.26103E+00_JPRB, 0.26298E+00_JPRB ,&
&  0.26513E+00_JPRB, 0.26705E+00_JPRB, 0.26840E+00_JPRB, 0.26985E+00_JPRB, 0.27173E+00_JPRB, 0.27315E+00_JPRB, 0.27501E+00_JPRB ,&
&  0.27740E+00_JPRB, 0.28066E+00_JPRB, 0.28550E+00_JPRB, 0.29112E+00_JPRB, 0.29829E+00_JPRB, 0.30893E+00_JPRB, 0.32761E+00_JPRB ,&
&  0.35224E+00_JPRB, 0.40779E+00_JPRB, 0.41289E+00_JPRB, 0.38318E+00_JPRB, 0.38938E+00_JPRB /)
ZBURD(:, 68,  3) = (/ &
&  0.37304E+00_JPRB, 0.37991E+00_JPRB, 0.38695E+00_JPRB, 0.39450E+00_JPRB, 0.40395E+00_JPRB, 0.40803E+00_JPRB, 0.40999E+00_JPRB ,&
&  0.40589E+00_JPRB, 0.40134E+00_JPRB, 0.39054E+00_JPRB, 0.38077E+00_JPRB, 0.37137E+00_JPRB, 0.36393E+00_JPRB, 0.35813E+00_JPRB ,&
&  0.35471E+00_JPRB, 0.35206E+00_JPRB, 0.34796E+00_JPRB, 0.34457E+00_JPRB, 0.34159E+00_JPRB, 0.34165E+00_JPRB, 0.34194E+00_JPRB ,&
&  0.34134E+00_JPRB, 0.34162E+00_JPRB, 0.34030E+00_JPRB, 0.34548E+00_JPRB, 0.35468E+00_JPRB, 0.36587E+00_JPRB, 0.38047E+00_JPRB ,&
&  0.40092E+00_JPRB, 0.41202E+00_JPRB, 0.42754E+00_JPRB, 0.42787E+00_JPRB, 0.42603E+00_JPRB, 0.42223E+00_JPRB, 0.41913E+00_JPRB ,&
&  0.41445E+00_JPRB, 0.40772E+00_JPRB, 0.40275E+00_JPRB, 0.39647E+00_JPRB, 0.36699E+00_JPRB, 0.34396E+00_JPRB, 0.32624E+00_JPRB ,&
&  0.31102E+00_JPRB, 0.29665E+00_JPRB, 0.28399E+00_JPRB, 0.27763E+00_JPRB, 0.27247E+00_JPRB, 0.26943E+00_JPRB, 0.26622E+00_JPRB ,&
&  0.26280E+00_JPRB, 0.26160E+00_JPRB, 0.25970E+00_JPRB, 0.25900E+00_JPRB, 0.26006E+00_JPRB, 0.26000E+00_JPRB, 0.26078E+00_JPRB ,&
&  0.26149E+00_JPRB, 0.26259E+00_JPRB, 0.26516E+00_JPRB, 0.26709E+00_JPRB, 0.26910E+00_JPRB, 0.27098E+00_JPRB, 0.27190E+00_JPRB ,&
&  0.27375E+00_JPRB, 0.27533E+00_JPRB, 0.27529E+00_JPRB, 0.27453E+00_JPRB, 0.27273E+00_JPRB, 0.26867E+00_JPRB, 0.26463E+00_JPRB ,&
&  0.26029E+00_JPRB, 0.25715E+00_JPRB, 0.25594E+00_JPRB, 0.25493E+00_JPRB, 0.25578E+00_JPRB, 0.25658E+00_JPRB, 0.25746E+00_JPRB ,&
&  0.25935E+00_JPRB, 0.26114E+00_JPRB, 0.26322E+00_JPRB, 0.26621E+00_JPRB, 0.26849E+00_JPRB, 0.27044E+00_JPRB, 0.27292E+00_JPRB ,&
&  0.27316E+00_JPRB, 0.27747E+00_JPRB, 0.28290E+00_JPRB, 0.29003E+00_JPRB, 0.30257E+00_JPRB, 0.31270E+00_JPRB, 0.33558E+00_JPRB ,&
&  0.37101E+00_JPRB, 0.43760E+00_JPRB, 0.43108E+00_JPRB, 0.36338E+00_JPRB, 0.36690E+00_JPRB /)
ZBURD(:, 69,  3) = (/ &
&  0.36158E+00_JPRB, 0.36449E+00_JPRB, 0.36777E+00_JPRB, 0.37180E+00_JPRB, 0.37676E+00_JPRB, 0.38124E+00_JPRB, 0.38598E+00_JPRB ,&
&  0.38886E+00_JPRB, 0.38579E+00_JPRB, 0.38655E+00_JPRB, 0.38217E+00_JPRB, 0.37770E+00_JPRB, 0.37189E+00_JPRB, 0.36576E+00_JPRB ,&
&  0.35938E+00_JPRB, 0.35036E+00_JPRB, 0.34073E+00_JPRB, 0.33222E+00_JPRB, 0.32501E+00_JPRB, 0.31942E+00_JPRB, 0.31495E+00_JPRB ,&
&  0.31187E+00_JPRB, 0.31230E+00_JPRB, 0.31441E+00_JPRB, 0.32046E+00_JPRB, 0.33059E+00_JPRB, 0.34311E+00_JPRB, 0.36215E+00_JPRB ,&
&  0.38898E+00_JPRB, 0.39114E+00_JPRB, 0.39636E+00_JPRB, 0.39543E+00_JPRB, 0.39124E+00_JPRB, 0.38925E+00_JPRB, 0.38541E+00_JPRB ,&
&  0.38598E+00_JPRB, 0.37828E+00_JPRB, 0.38805E+00_JPRB, 0.37139E+00_JPRB, 0.34436E+00_JPRB, 0.32899E+00_JPRB, 0.31065E+00_JPRB ,&
&  0.29732E+00_JPRB, 0.28598E+00_JPRB, 0.27750E+00_JPRB, 0.27397E+00_JPRB, 0.27012E+00_JPRB, 0.26832E+00_JPRB, 0.26457E+00_JPRB ,&
&  0.26069E+00_JPRB, 0.25879E+00_JPRB, 0.25566E+00_JPRB, 0.25544E+00_JPRB, 0.25555E+00_JPRB, 0.25497E+00_JPRB, 0.25506E+00_JPRB ,&
&  0.25547E+00_JPRB, 0.25654E+00_JPRB, 0.25845E+00_JPRB, 0.26019E+00_JPRB, 0.26123E+00_JPRB, 0.26184E+00_JPRB, 0.26167E+00_JPRB ,&
&  0.26213E+00_JPRB, 0.26180E+00_JPRB, 0.26095E+00_JPRB, 0.25962E+00_JPRB, 0.25759E+00_JPRB, 0.25548E+00_JPRB, 0.25344E+00_JPRB ,&
&  0.25179E+00_JPRB, 0.25117E+00_JPRB, 0.25062E+00_JPRB, 0.25083E+00_JPRB, 0.25140E+00_JPRB, 0.25316E+00_JPRB, 0.25482E+00_JPRB ,&
&  0.25672E+00_JPRB, 0.25941E+00_JPRB, 0.26176E+00_JPRB, 0.26464E+00_JPRB, 0.26727E+00_JPRB, 0.26879E+00_JPRB, 0.27288E+00_JPRB ,&
&  0.27815E+00_JPRB, 0.28531E+00_JPRB, 0.29372E+00_JPRB, 0.30179E+00_JPRB, 0.31207E+00_JPRB, 0.32767E+00_JPRB, 0.34875E+00_JPRB ,&
&  0.37021E+00_JPRB, 0.39312E+00_JPRB, 0.38699E+00_JPRB, 0.35958E+00_JPRB, 0.35841E+00_JPRB /)
ZBURD(:, 70,  3) = (/ &
&  0.36249E+00_JPRB, 0.36075E+00_JPRB, 0.36137E+00_JPRB, 0.36247E+00_JPRB, 0.36626E+00_JPRB, 0.37168E+00_JPRB, 0.37753E+00_JPRB ,&
&  0.37714E+00_JPRB, 0.38034E+00_JPRB, 0.37946E+00_JPRB, 0.37674E+00_JPRB, 0.37222E+00_JPRB, 0.36782E+00_JPRB, 0.36299E+00_JPRB ,&
&  0.35618E+00_JPRB, 0.34746E+00_JPRB, 0.33898E+00_JPRB, 0.33032E+00_JPRB, 0.32211E+00_JPRB, 0.31361E+00_JPRB, 0.30862E+00_JPRB ,&
&  0.30532E+00_JPRB, 0.30446E+00_JPRB, 0.30646E+00_JPRB, 0.30919E+00_JPRB, 0.31455E+00_JPRB, 0.32187E+00_JPRB, 0.33158E+00_JPRB ,&
&  0.34107E+00_JPRB, 0.35179E+00_JPRB, 0.35740E+00_JPRB, 0.35729E+00_JPRB, 0.35705E+00_JPRB, 0.35497E+00_JPRB, 0.35150E+00_JPRB ,&
&  0.34494E+00_JPRB, 0.33750E+00_JPRB, 0.32890E+00_JPRB, 0.31980E+00_JPRB, 0.31053E+00_JPRB, 0.30181E+00_JPRB, 0.29371E+00_JPRB ,&
&  0.28714E+00_JPRB, 0.28155E+00_JPRB, 0.27643E+00_JPRB, 0.27213E+00_JPRB, 0.26880E+00_JPRB, 0.26632E+00_JPRB, 0.26330E+00_JPRB ,&
&  0.26077E+00_JPRB, 0.25854E+00_JPRB, 0.25589E+00_JPRB, 0.25406E+00_JPRB, 0.25243E+00_JPRB, 0.25136E+00_JPRB, 0.25047E+00_JPRB ,&
&  0.25040E+00_JPRB, 0.25033E+00_JPRB, 0.25096E+00_JPRB, 0.25158E+00_JPRB, 0.25231E+00_JPRB, 0.25226E+00_JPRB, 0.25203E+00_JPRB ,&
&  0.25159E+00_JPRB, 0.25118E+00_JPRB, 0.24978E+00_JPRB, 0.24870E+00_JPRB, 0.24794E+00_JPRB, 0.24711E+00_JPRB, 0.24706E+00_JPRB ,&
&  0.24789E+00_JPRB, 0.24875E+00_JPRB, 0.24944E+00_JPRB, 0.25106E+00_JPRB, 0.25266E+00_JPRB, 0.25500E+00_JPRB, 0.25783E+00_JPRB ,&
&  0.25990E+00_JPRB, 0.26233E+00_JPRB, 0.26504E+00_JPRB, 0.26819E+00_JPRB, 0.27197E+00_JPRB, 0.27589E+00_JPRB, 0.28050E+00_JPRB ,&
&  0.28569E+00_JPRB, 0.29102E+00_JPRB, 0.29703E+00_JPRB, 0.30368E+00_JPRB, 0.31176E+00_JPRB, 0.31841E+00_JPRB, 0.32834E+00_JPRB ,&
&  0.33611E+00_JPRB, 0.33475E+00_JPRB, 0.37498E+00_JPRB, 0.36860E+00_JPRB, 0.36447E+00_JPRB /)
ZBURD(:, 71,  3) = (/ &
&  0.30506E+00_JPRB, 0.31157E+00_JPRB, 0.31621E+00_JPRB, 0.32090E+00_JPRB, 0.31601E+00_JPRB, 0.35310E+00_JPRB, 0.35147E+00_JPRB ,&
&  0.35217E+00_JPRB, 0.33915E+00_JPRB, 0.33990E+00_JPRB, 0.33868E+00_JPRB, 0.33587E+00_JPRB, 0.33171E+00_JPRB, 0.32782E+00_JPRB ,&
&  0.32438E+00_JPRB, 0.31973E+00_JPRB, 0.31427E+00_JPRB, 0.30960E+00_JPRB, 0.30588E+00_JPRB, 0.30271E+00_JPRB, 0.29999E+00_JPRB ,&
&  0.29935E+00_JPRB, 0.30060E+00_JPRB, 0.30302E+00_JPRB, 0.30181E+00_JPRB, 0.30279E+00_JPRB, 0.30467E+00_JPRB, 0.30693E+00_JPRB ,&
&  0.30904E+00_JPRB, 0.31088E+00_JPRB, 0.31092E+00_JPRB, 0.30923E+00_JPRB, 0.30811E+00_JPRB, 0.30580E+00_JPRB, 0.30366E+00_JPRB ,&
&  0.30104E+00_JPRB, 0.29815E+00_JPRB, 0.29474E+00_JPRB, 0.29202E+00_JPRB, 0.28904E+00_JPRB, 0.28541E+00_JPRB, 0.28190E+00_JPRB ,&
&  0.27890E+00_JPRB, 0.27621E+00_JPRB, 0.27325E+00_JPRB, 0.27051E+00_JPRB, 0.26790E+00_JPRB, 0.26523E+00_JPRB, 0.26287E+00_JPRB ,&
&  0.26108E+00_JPRB, 0.25957E+00_JPRB, 0.25806E+00_JPRB, 0.25647E+00_JPRB, 0.25493E+00_JPRB, 0.25344E+00_JPRB, 0.25223E+00_JPRB ,&
&  0.25205E+00_JPRB, 0.25135E+00_JPRB, 0.25067E+00_JPRB, 0.25030E+00_JPRB, 0.25034E+00_JPRB, 0.24985E+00_JPRB, 0.24964E+00_JPRB ,&
&  0.24957E+00_JPRB, 0.24959E+00_JPRB, 0.24983E+00_JPRB, 0.25013E+00_JPRB, 0.25056E+00_JPRB, 0.25111E+00_JPRB, 0.25200E+00_JPRB ,&
&  0.25307E+00_JPRB, 0.25436E+00_JPRB, 0.25574E+00_JPRB, 0.25716E+00_JPRB, 0.25861E+00_JPRB, 0.26014E+00_JPRB, 0.26186E+00_JPRB ,&
&  0.26359E+00_JPRB, 0.26526E+00_JPRB, 0.26690E+00_JPRB, 0.26852E+00_JPRB, 0.27117E+00_JPRB, 0.27372E+00_JPRB, 0.27645E+00_JPRB ,&
&  0.27939E+00_JPRB, 0.28286E+00_JPRB, 0.28652E+00_JPRB, 0.29046E+00_JPRB, 0.29488E+00_JPRB, 0.29638E+00_JPRB, 0.29757E+00_JPRB ,&
&  0.29832E+00_JPRB, 0.30127E+00_JPRB, 0.30324E+00_JPRB, 0.30295E+00_JPRB, 0.30259E+00_JPRB /)
ZBURD(:, 72,  3) = (/ &
&  0.27737E+00_JPRB, 0.27752E+00_JPRB, 0.27760E+00_JPRB, 0.27766E+00_JPRB, 0.27642E+00_JPRB, 0.27689E+00_JPRB, 0.27731E+00_JPRB ,&
&  0.27802E+00_JPRB, 0.27939E+00_JPRB, 0.28098E+00_JPRB, 0.28219E+00_JPRB, 0.28308E+00_JPRB, 0.28364E+00_JPRB, 0.28404E+00_JPRB ,&
&  0.28412E+00_JPRB, 0.28437E+00_JPRB, 0.28112E+00_JPRB, 0.27996E+00_JPRB, 0.27958E+00_JPRB, 0.27961E+00_JPRB, 0.27981E+00_JPRB ,&
&  0.27995E+00_JPRB, 0.27995E+00_JPRB, 0.27945E+00_JPRB, 0.27975E+00_JPRB, 0.28007E+00_JPRB, 0.28029E+00_JPRB, 0.28055E+00_JPRB ,&
&  0.28078E+00_JPRB, 0.28097E+00_JPRB, 0.28091E+00_JPRB, 0.28069E+00_JPRB, 0.28055E+00_JPRB, 0.28005E+00_JPRB, 0.27919E+00_JPRB ,&
&  0.27814E+00_JPRB, 0.27683E+00_JPRB, 0.27553E+00_JPRB, 0.27433E+00_JPRB, 0.27330E+00_JPRB, 0.27241E+00_JPRB, 0.27156E+00_JPRB ,&
&  0.27055E+00_JPRB, 0.26922E+00_JPRB, 0.26781E+00_JPRB, 0.26644E+00_JPRB, 0.26513E+00_JPRB, 0.26394E+00_JPRB, 0.26268E+00_JPRB ,&
&  0.26172E+00_JPRB, 0.26091E+00_JPRB, 0.26009E+00_JPRB, 0.25940E+00_JPRB, 0.25872E+00_JPRB, 0.25807E+00_JPRB, 0.25744E+00_JPRB ,&
&  0.25660E+00_JPRB, 0.25613E+00_JPRB, 0.25561E+00_JPRB, 0.25537E+00_JPRB, 0.25542E+00_JPRB, 0.25544E+00_JPRB, 0.25555E+00_JPRB ,&
&  0.25572E+00_JPRB, 0.25595E+00_JPRB, 0.25607E+00_JPRB, 0.25628E+00_JPRB, 0.25659E+00_JPRB, 0.25700E+00_JPRB, 0.25751E+00_JPRB ,&
&  0.25805E+00_JPRB, 0.25865E+00_JPRB, 0.25930E+00_JPRB, 0.26003E+00_JPRB, 0.26079E+00_JPRB, 0.26164E+00_JPRB, 0.26244E+00_JPRB ,&
&  0.26325E+00_JPRB, 0.26399E+00_JPRB, 0.26475E+00_JPRB, 0.26533E+00_JPRB, 0.26617E+00_JPRB, 0.26697E+00_JPRB, 0.26793E+00_JPRB ,&
&  0.26891E+00_JPRB, 0.26994E+00_JPRB, 0.27085E+00_JPRB, 0.27182E+00_JPRB, 0.27288E+00_JPRB, 0.27271E+00_JPRB, 0.27261E+00_JPRB ,&
&  0.27258E+00_JPRB, 0.27286E+00_JPRB, 0.27285E+00_JPRB, 0.27319E+00_JPRB, 0.27404E+00_JPRB /)
ZBURD(:, 73,  3) = (/ &
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB ,&
&  0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB, 0.26239E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART6',1,ZHOOK_HANDLE)
END SUBROUTINE PART6



SUBROUTINE PART7

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART7',0,ZHOOK_HANDLE)


!-- month=  4
ZBURD(:,  1,  4) = (/ &
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB ,&
&  0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB, 0.75980E+00_JPRB /)
ZBURD(:,  2,  4) = (/ &
&  0.77519E+00_JPRB, 0.77479E+00_JPRB, 0.77436E+00_JPRB, 0.77379E+00_JPRB, 0.77312E+00_JPRB, 0.77277E+00_JPRB, 0.77241E+00_JPRB ,&
&  0.77211E+00_JPRB, 0.77179E+00_JPRB, 0.77199E+00_JPRB, 0.77181E+00_JPRB, 0.77177E+00_JPRB, 0.77154E+00_JPRB, 0.77145E+00_JPRB ,&
&  0.77127E+00_JPRB, 0.77124E+00_JPRB, 0.77119E+00_JPRB, 0.77138E+00_JPRB, 0.77153E+00_JPRB, 0.77196E+00_JPRB, 0.77219E+00_JPRB ,&
&  0.77233E+00_JPRB, 0.77191E+00_JPRB, 0.77181E+00_JPRB, 0.77107E+00_JPRB, 0.77021E+00_JPRB, 0.76864E+00_JPRB, 0.76690E+00_JPRB ,&
&  0.76461E+00_JPRB, 0.76263E+00_JPRB, 0.76088E+00_JPRB, 0.75952E+00_JPRB, 0.75839E+00_JPRB, 0.75773E+00_JPRB, 0.75711E+00_JPRB ,&
&  0.75640E+00_JPRB, 0.75583E+00_JPRB, 0.75538E+00_JPRB, 0.75475E+00_JPRB, 0.75364E+00_JPRB, 0.75227E+00_JPRB, 0.75090E+00_JPRB ,&
&  0.74927E+00_JPRB, 0.74750E+00_JPRB, 0.74628E+00_JPRB, 0.74557E+00_JPRB, 0.74456E+00_JPRB, 0.74268E+00_JPRB, 0.74059E+00_JPRB ,&
&  0.73870E+00_JPRB, 0.73652E+00_JPRB, 0.73422E+00_JPRB, 0.73210E+00_JPRB, 0.73072E+00_JPRB, 0.72954E+00_JPRB, 0.72853E+00_JPRB ,&
&  0.72861E+00_JPRB, 0.72999E+00_JPRB, 0.73139E+00_JPRB, 0.73263E+00_JPRB, 0.73437E+00_JPRB, 0.73623E+00_JPRB, 0.73832E+00_JPRB ,&
&  0.74048E+00_JPRB, 0.74283E+00_JPRB, 0.74570E+00_JPRB, 0.74828E+00_JPRB, 0.75129E+00_JPRB, 0.75446E+00_JPRB, 0.75804E+00_JPRB ,&
&  0.76145E+00_JPRB, 0.76515E+00_JPRB, 0.76828E+00_JPRB, 0.77124E+00_JPRB, 0.77296E+00_JPRB, 0.77459E+00_JPRB, 0.77534E+00_JPRB ,&
&  0.77620E+00_JPRB, 0.77670E+00_JPRB, 0.77756E+00_JPRB, 0.77786E+00_JPRB, 0.77821E+00_JPRB, 0.77838E+00_JPRB, 0.77869E+00_JPRB ,&
&  0.77874E+00_JPRB, 0.77884E+00_JPRB, 0.77870E+00_JPRB, 0.77855E+00_JPRB, 0.77797E+00_JPRB, 0.77775E+00_JPRB, 0.77740E+00_JPRB ,&
&  0.77714E+00_JPRB, 0.77689E+00_JPRB, 0.77671E+00_JPRB, 0.77647E+00_JPRB, 0.77601E+00_JPRB /)
ZBURD(:,  3,  4) = (/ &
&  0.77727E+00_JPRB, 0.77809E+00_JPRB, 0.77779E+00_JPRB, 0.77768E+00_JPRB, 0.77721E+00_JPRB, 0.77778E+00_JPRB, 0.77824E+00_JPRB ,&
&  0.77882E+00_JPRB, 0.77990E+00_JPRB, 0.78200E+00_JPRB, 0.78285E+00_JPRB, 0.78336E+00_JPRB, 0.78331E+00_JPRB, 0.78327E+00_JPRB ,&
&  0.78271E+00_JPRB, 0.78167E+00_JPRB, 0.78052E+00_JPRB, 0.77977E+00_JPRB, 0.77898E+00_JPRB, 0.77839E+00_JPRB, 0.77765E+00_JPRB ,&
&  0.77599E+00_JPRB, 0.77394E+00_JPRB, 0.77154E+00_JPRB, 0.76748E+00_JPRB, 0.76282E+00_JPRB, 0.75766E+00_JPRB, 0.75183E+00_JPRB ,&
&  0.74566E+00_JPRB, 0.74102E+00_JPRB, 0.73660E+00_JPRB, 0.73263E+00_JPRB, 0.73147E+00_JPRB, 0.73080E+00_JPRB, 0.73034E+00_JPRB ,&
&  0.73006E+00_JPRB, 0.72880E+00_JPRB, 0.72939E+00_JPRB, 0.73030E+00_JPRB, 0.73159E+00_JPRB, 0.73384E+00_JPRB, 0.73547E+00_JPRB ,&
&  0.73649E+00_JPRB, 0.73607E+00_JPRB, 0.73472E+00_JPRB, 0.73539E+00_JPRB, 0.73776E+00_JPRB, 0.73881E+00_JPRB, 0.73780E+00_JPRB ,&
&  0.73688E+00_JPRB, 0.73490E+00_JPRB, 0.73257E+00_JPRB, 0.72993E+00_JPRB, 0.72905E+00_JPRB, 0.72751E+00_JPRB, 0.72583E+00_JPRB ,&
&  0.72498E+00_JPRB, 0.72543E+00_JPRB, 0.72637E+00_JPRB, 0.72771E+00_JPRB, 0.73004E+00_JPRB, 0.73232E+00_JPRB, 0.73476E+00_JPRB ,&
&  0.73756E+00_JPRB, 0.74213E+00_JPRB, 0.74641E+00_JPRB, 0.75057E+00_JPRB, 0.75526E+00_JPRB, 0.76049E+00_JPRB, 0.76541E+00_JPRB ,&
&  0.76991E+00_JPRB, 0.77382E+00_JPRB, 0.77614E+00_JPRB, 0.77896E+00_JPRB, 0.78183E+00_JPRB, 0.78538E+00_JPRB, 0.78795E+00_JPRB ,&
&  0.79048E+00_JPRB, 0.79214E+00_JPRB, 0.79287E+00_JPRB, 0.79338E+00_JPRB, 0.79315E+00_JPRB, 0.79256E+00_JPRB, 0.79149E+00_JPRB ,&
&  0.78990E+00_JPRB, 0.78816E+00_JPRB, 0.78638E+00_JPRB, 0.78443E+00_JPRB, 0.78252E+00_JPRB, 0.78041E+00_JPRB, 0.77840E+00_JPRB ,&
&  0.77709E+00_JPRB, 0.77691E+00_JPRB, 0.77721E+00_JPRB, 0.77712E+00_JPRB, 0.77718E+00_JPRB /)
ZBURD(:,  4,  4) = (/ &
&  0.77301E+00_JPRB, 0.77332E+00_JPRB, 0.77313E+00_JPRB, 0.77359E+00_JPRB, 0.77602E+00_JPRB, 0.77843E+00_JPRB, 0.77945E+00_JPRB ,&
&  0.78160E+00_JPRB, 0.78454E+00_JPRB, 0.78859E+00_JPRB, 0.79039E+00_JPRB, 0.79168E+00_JPRB, 0.79154E+00_JPRB, 0.79052E+00_JPRB ,&
&  0.78839E+00_JPRB, 0.78552E+00_JPRB, 0.78316E+00_JPRB, 0.78128E+00_JPRB, 0.77916E+00_JPRB, 0.77719E+00_JPRB, 0.77261E+00_JPRB ,&
&  0.76826E+00_JPRB, 0.76586E+00_JPRB, 0.75897E+00_JPRB, 0.74940E+00_JPRB, 0.74179E+00_JPRB, 0.68621E+00_JPRB, 0.66827E+00_JPRB ,&
&  0.66508E+00_JPRB, 0.66065E+00_JPRB, 0.66269E+00_JPRB, 0.66623E+00_JPRB, 0.67778E+00_JPRB, 0.67084E+00_JPRB, 0.66650E+00_JPRB ,&
&  0.63385E+00_JPRB, 0.63423E+00_JPRB, 0.62998E+00_JPRB, 0.62892E+00_JPRB, 0.63467E+00_JPRB, 0.63970E+00_JPRB, 0.65338E+00_JPRB ,&
&  0.68814E+00_JPRB, 0.70032E+00_JPRB, 0.70460E+00_JPRB, 0.70821E+00_JPRB, 0.71613E+00_JPRB, 0.72252E+00_JPRB, 0.72691E+00_JPRB ,&
&  0.72937E+00_JPRB, 0.73065E+00_JPRB, 0.73300E+00_JPRB, 0.73529E+00_JPRB, 0.73680E+00_JPRB, 0.73607E+00_JPRB, 0.73642E+00_JPRB ,&
&  0.73698E+00_JPRB, 0.73956E+00_JPRB, 0.74280E+00_JPRB, 0.74636E+00_JPRB, 0.75066E+00_JPRB, 0.75362E+00_JPRB, 0.75628E+00_JPRB ,&
&  0.75907E+00_JPRB, 0.76117E+00_JPRB, 0.76245E+00_JPRB, 0.76428E+00_JPRB, 0.76694E+00_JPRB, 0.76841E+00_JPRB, 0.77030E+00_JPRB ,&
&  0.77225E+00_JPRB, 0.77518E+00_JPRB, 0.77782E+00_JPRB, 0.78351E+00_JPRB, 0.78865E+00_JPRB, 0.79543E+00_JPRB, 0.80154E+00_JPRB ,&
&  0.80693E+00_JPRB, 0.81051E+00_JPRB, 0.81258E+00_JPRB, 0.81365E+00_JPRB, 0.81398E+00_JPRB, 0.81392E+00_JPRB, 0.81274E+00_JPRB ,&
&  0.81093E+00_JPRB, 0.80750E+00_JPRB, 0.80338E+00_JPRB, 0.79855E+00_JPRB, 0.79399E+00_JPRB, 0.78799E+00_JPRB, 0.78239E+00_JPRB ,&
&  0.77647E+00_JPRB, 0.77348E+00_JPRB, 0.77168E+00_JPRB, 0.77200E+00_JPRB, 0.77268E+00_JPRB /)
ZBURD(:,  5,  4) = (/ &
&  0.77982E+00_JPRB, 0.77774E+00_JPRB, 0.77540E+00_JPRB, 0.77581E+00_JPRB, 0.77886E+00_JPRB, 0.78424E+00_JPRB, 0.78582E+00_JPRB ,&
&  0.79056E+00_JPRB, 0.79508E+00_JPRB, 0.80037E+00_JPRB, 0.80187E+00_JPRB, 0.80332E+00_JPRB, 0.80110E+00_JPRB, 0.79723E+00_JPRB ,&
&  0.79310E+00_JPRB, 0.78732E+00_JPRB, 0.78335E+00_JPRB, 0.77920E+00_JPRB, 0.77352E+00_JPRB, 0.76958E+00_JPRB, 0.76370E+00_JPRB ,&
&  0.75998E+00_JPRB, 0.75567E+00_JPRB, 0.70393E+00_JPRB, 0.68639E+00_JPRB, 0.66115E+00_JPRB, 0.64783E+00_JPRB, 0.64278E+00_JPRB ,&
&  0.63501E+00_JPRB, 0.62781E+00_JPRB, 0.61689E+00_JPRB, 0.60066E+00_JPRB, 0.58082E+00_JPRB, 0.55240E+00_JPRB, 0.53775E+00_JPRB ,&
&  0.52471E+00_JPRB, 0.52111E+00_JPRB, 0.52312E+00_JPRB, 0.53204E+00_JPRB, 0.54947E+00_JPRB, 0.56965E+00_JPRB, 0.59350E+00_JPRB ,&
&  0.62529E+00_JPRB, 0.65892E+00_JPRB, 0.67639E+00_JPRB, 0.67901E+00_JPRB, 0.70468E+00_JPRB, 0.71980E+00_JPRB, 0.72906E+00_JPRB ,&
&  0.73347E+00_JPRB, 0.73419E+00_JPRB, 0.74858E+00_JPRB, 0.75523E+00_JPRB, 0.75859E+00_JPRB, 0.76079E+00_JPRB, 0.76535E+00_JPRB ,&
&  0.77170E+00_JPRB, 0.77863E+00_JPRB, 0.78334E+00_JPRB, 0.78783E+00_JPRB, 0.79059E+00_JPRB, 0.79198E+00_JPRB, 0.79329E+00_JPRB ,&
&  0.79481E+00_JPRB, 0.79515E+00_JPRB, 0.79498E+00_JPRB, 0.79339E+00_JPRB, 0.79269E+00_JPRB, 0.79161E+00_JPRB, 0.78949E+00_JPRB ,&
&  0.78980E+00_JPRB, 0.79090E+00_JPRB, 0.79283E+00_JPRB, 0.79048E+00_JPRB, 0.79530E+00_JPRB, 0.80886E+00_JPRB, 0.81692E+00_JPRB ,&
&  0.82601E+00_JPRB, 0.83225E+00_JPRB, 0.83792E+00_JPRB, 0.84249E+00_JPRB, 0.84557E+00_JPRB, 0.84572E+00_JPRB, 0.84291E+00_JPRB ,&
&  0.83998E+00_JPRB, 0.83543E+00_JPRB, 0.83177E+00_JPRB, 0.82570E+00_JPRB, 0.82017E+00_JPRB, 0.81205E+00_JPRB, 0.80493E+00_JPRB ,&
&  0.79570E+00_JPRB, 0.79023E+00_JPRB, 0.78182E+00_JPRB, 0.78049E+00_JPRB, 0.77922E+00_JPRB /)
ZBURD(:,  6,  4) = (/ &
&  0.79627E+00_JPRB, 0.79056E+00_JPRB, 0.78811E+00_JPRB, 0.78980E+00_JPRB, 0.79881E+00_JPRB, 0.80986E+00_JPRB, 0.81112E+00_JPRB ,&
&  0.81231E+00_JPRB, 0.81361E+00_JPRB, 0.81654E+00_JPRB, 0.81528E+00_JPRB, 0.81362E+00_JPRB, 0.80778E+00_JPRB, 0.80308E+00_JPRB ,&
&  0.79838E+00_JPRB, 0.78864E+00_JPRB, 0.77644E+00_JPRB, 0.76861E+00_JPRB, 0.76483E+00_JPRB, 0.76576E+00_JPRB, 0.76428E+00_JPRB ,&
&  0.75893E+00_JPRB, 0.74420E+00_JPRB, 0.70804E+00_JPRB, 0.68457E+00_JPRB, 0.65936E+00_JPRB, 0.65301E+00_JPRB, 0.64121E+00_JPRB ,&
&  0.65411E+00_JPRB, 0.59447E+00_JPRB, 0.59061E+00_JPRB, 0.56259E+00_JPRB, 0.54084E+00_JPRB, 0.51699E+00_JPRB, 0.49619E+00_JPRB ,&
&  0.48909E+00_JPRB, 0.48132E+00_JPRB, 0.47941E+00_JPRB, 0.48727E+00_JPRB, 0.50046E+00_JPRB, 0.52792E+00_JPRB, 0.55893E+00_JPRB ,&
&  0.58300E+00_JPRB, 0.65974E+00_JPRB, 0.67892E+00_JPRB, 0.69096E+00_JPRB, 0.71508E+00_JPRB, 0.73160E+00_JPRB, 0.73669E+00_JPRB ,&
&  0.74185E+00_JPRB, 0.74795E+00_JPRB, 0.76725E+00_JPRB, 0.78065E+00_JPRB, 0.79851E+00_JPRB, 0.80721E+00_JPRB, 0.81414E+00_JPRB ,&
&  0.82444E+00_JPRB, 0.83691E+00_JPRB, 0.84662E+00_JPRB, 0.85682E+00_JPRB, 0.86319E+00_JPRB, 0.86756E+00_JPRB, 0.86916E+00_JPRB ,&
&  0.86940E+00_JPRB, 0.86762E+00_JPRB, 0.86605E+00_JPRB, 0.86280E+00_JPRB, 0.85880E+00_JPRB, 0.85805E+00_JPRB, 0.85534E+00_JPRB ,&
&  0.85369E+00_JPRB, 0.84833E+00_JPRB, 0.84232E+00_JPRB, 0.83774E+00_JPRB, 0.84100E+00_JPRB, 0.84356E+00_JPRB, 0.85306E+00_JPRB ,&
&  0.86585E+00_JPRB, 0.87232E+00_JPRB, 0.87675E+00_JPRB, 0.88125E+00_JPRB, 0.88500E+00_JPRB, 0.88333E+00_JPRB, 0.87883E+00_JPRB ,&
&  0.87429E+00_JPRB, 0.86881E+00_JPRB, 0.86418E+00_JPRB, 0.85660E+00_JPRB, 0.85012E+00_JPRB, 0.84008E+00_JPRB, 0.83157E+00_JPRB ,&
&  0.82064E+00_JPRB, 0.81488E+00_JPRB, 0.80357E+00_JPRB, 0.80075E+00_JPRB, 0.79737E+00_JPRB /)
ZBURD(:,  7,  4) = (/ &
&  0.82326E+00_JPRB, 0.81426E+00_JPRB, 0.81177E+00_JPRB, 0.81182E+00_JPRB, 0.81658E+00_JPRB, 0.82470E+00_JPRB, 0.82887E+00_JPRB ,&
&  0.83286E+00_JPRB, 0.83605E+00_JPRB, 0.83581E+00_JPRB, 0.82504E+00_JPRB, 0.81554E+00_JPRB, 0.80915E+00_JPRB, 0.80327E+00_JPRB ,&
&  0.79284E+00_JPRB, 0.78216E+00_JPRB, 0.77465E+00_JPRB, 0.77086E+00_JPRB, 0.76793E+00_JPRB, 0.77379E+00_JPRB, 0.76904E+00_JPRB ,&
&  0.74814E+00_JPRB, 0.73021E+00_JPRB, 0.69944E+00_JPRB, 0.67476E+00_JPRB, 0.65729E+00_JPRB, 0.64435E+00_JPRB, 0.64683E+00_JPRB ,&
&  0.64236E+00_JPRB, 0.63897E+00_JPRB, 0.63694E+00_JPRB, 0.63555E+00_JPRB, 0.63862E+00_JPRB, 0.53264E+00_JPRB, 0.50288E+00_JPRB ,&
&  0.48219E+00_JPRB, 0.46676E+00_JPRB, 0.46175E+00_JPRB, 0.46085E+00_JPRB, 0.46917E+00_JPRB, 0.50019E+00_JPRB, 0.54764E+00_JPRB ,&
&  0.57757E+00_JPRB, 0.67523E+00_JPRB, 0.69220E+00_JPRB, 0.70951E+00_JPRB, 0.73810E+00_JPRB, 0.75854E+00_JPRB, 0.76766E+00_JPRB ,&
&  0.78604E+00_JPRB, 0.81987E+00_JPRB, 0.86310E+00_JPRB, 0.90916E+00_JPRB, 0.92675E+00_JPRB, 0.95712E+00_JPRB, 0.97457E+00_JPRB ,&
&  0.98557E+00_JPRB, 0.98015E+00_JPRB, 0.97711E+00_JPRB, 0.97717E+00_JPRB, 0.96932E+00_JPRB, 0.96898E+00_JPRB, 0.95762E+00_JPRB ,&
&  0.95660E+00_JPRB, 0.94513E+00_JPRB, 0.95637E+00_JPRB, 0.96259E+00_JPRB, 0.96876E+00_JPRB, 0.97468E+00_JPRB, 0.97359E+00_JPRB ,&
&  0.97054E+00_JPRB, 0.96327E+00_JPRB, 0.94686E+00_JPRB, 0.92161E+00_JPRB, 0.90851E+00_JPRB, 0.89188E+00_JPRB, 0.89057E+00_JPRB ,&
&  0.89376E+00_JPRB, 0.90864E+00_JPRB, 0.90977E+00_JPRB, 0.90972E+00_JPRB, 0.91036E+00_JPRB, 0.90574E+00_JPRB, 0.90493E+00_JPRB ,&
&  0.90253E+00_JPRB, 0.90121E+00_JPRB, 0.89689E+00_JPRB, 0.88905E+00_JPRB, 0.87975E+00_JPRB, 0.86733E+00_JPRB, 0.85548E+00_JPRB ,&
&  0.84819E+00_JPRB, 0.84519E+00_JPRB, 0.83615E+00_JPRB, 0.83230E+00_JPRB, 0.82776E+00_JPRB /)
ZBURD(:,  8,  4) = (/ &
&  0.85406E+00_JPRB, 0.83964E+00_JPRB, 0.83541E+00_JPRB, 0.83327E+00_JPRB, 0.83754E+00_JPRB, 0.84414E+00_JPRB, 0.84576E+00_JPRB ,&
&  0.85620E+00_JPRB, 0.85619E+00_JPRB, 0.84383E+00_JPRB, 0.82988E+00_JPRB, 0.81197E+00_JPRB, 0.79701E+00_JPRB, 0.78874E+00_JPRB ,&
&  0.78538E+00_JPRB, 0.77204E+00_JPRB, 0.76392E+00_JPRB, 0.75956E+00_JPRB, 0.76232E+00_JPRB, 0.77019E+00_JPRB, 0.76121E+00_JPRB ,&
&  0.73712E+00_JPRB, 0.72152E+00_JPRB, 0.70381E+00_JPRB, 0.67895E+00_JPRB, 0.65293E+00_JPRB, 0.63589E+00_JPRB, 0.63929E+00_JPRB ,&
&  0.64950E+00_JPRB, 0.64379E+00_JPRB, 0.64573E+00_JPRB, 0.64518E+00_JPRB, 0.63772E+00_JPRB, 0.62677E+00_JPRB, 0.53504E+00_JPRB ,&
&  0.49921E+00_JPRB, 0.46965E+00_JPRB, 0.45747E+00_JPRB, 0.45492E+00_JPRB, 0.47314E+00_JPRB, 0.50916E+00_JPRB, 0.57366E+00_JPRB ,&
&  0.62945E+00_JPRB, 0.68261E+00_JPRB, 0.71716E+00_JPRB, 0.74499E+00_JPRB, 0.77682E+00_JPRB, 0.81085E+00_JPRB, 0.84135E+00_JPRB ,&
&  0.89261E+00_JPRB, 0.96593E+00_JPRB, 0.10468E+01_JPRB, 0.11248E+01_JPRB, 0.11787E+01_JPRB, 0.12074E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.12267E+01_JPRB, 0.11162E+01_JPRB, 0.10715E+01_JPRB, 0.10424E+01_JPRB, 0.10225E+01_JPRB, 0.10217E+01_JPRB, 0.99330E+00_JPRB ,&
&  0.98180E+00_JPRB, 0.99006E+00_JPRB, 0.99411E+00_JPRB, 0.99799E+00_JPRB, 0.10062E+01_JPRB, 0.10099E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.10329E+01_JPRB, 0.10584E+01_JPRB, 0.11119E+01_JPRB, 0.10748E+01_JPRB, 0.10227E+01_JPRB, 0.96570E+00_JPRB, 0.92873E+00_JPRB ,&
&  0.91780E+00_JPRB, 0.91850E+00_JPRB, 0.92324E+00_JPRB, 0.91760E+00_JPRB, 0.91417E+00_JPRB, 0.91301E+00_JPRB, 0.92575E+00_JPRB ,&
&  0.92722E+00_JPRB, 0.92873E+00_JPRB, 0.92567E+00_JPRB, 0.91837E+00_JPRB, 0.90644E+00_JPRB, 0.89213E+00_JPRB, 0.88168E+00_JPRB ,&
&  0.87765E+00_JPRB, 0.87723E+00_JPRB, 0.87245E+00_JPRB, 0.86976E+00_JPRB, 0.86476E+00_JPRB /)
ZBURD(:,  9,  4) = (/ &
&  0.87209E+00_JPRB, 0.86427E+00_JPRB, 0.86341E+00_JPRB, 0.86639E+00_JPRB, 0.87198E+00_JPRB, 0.85445E+00_JPRB, 0.83650E+00_JPRB ,&
&  0.83222E+00_JPRB, 0.83550E+00_JPRB, 0.82340E+00_JPRB, 0.81275E+00_JPRB, 0.79417E+00_JPRB, 0.77737E+00_JPRB, 0.77088E+00_JPRB ,&
&  0.76790E+00_JPRB, 0.76006E+00_JPRB, 0.75007E+00_JPRB, 0.73858E+00_JPRB, 0.74258E+00_JPRB, 0.74570E+00_JPRB, 0.73529E+00_JPRB ,&
&  0.72852E+00_JPRB, 0.72340E+00_JPRB, 0.71170E+00_JPRB, 0.69351E+00_JPRB, 0.66828E+00_JPRB, 0.65670E+00_JPRB, 0.64968E+00_JPRB ,&
&  0.64447E+00_JPRB, 0.64805E+00_JPRB, 0.66539E+00_JPRB, 0.66352E+00_JPRB, 0.66374E+00_JPRB, 0.65729E+00_JPRB, 0.59634E+00_JPRB ,&
&  0.52958E+00_JPRB, 0.48320E+00_JPRB, 0.45673E+00_JPRB, 0.45498E+00_JPRB, 0.47945E+00_JPRB, 0.53451E+00_JPRB, 0.60134E+00_JPRB ,&
&  0.67400E+00_JPRB, 0.73004E+00_JPRB, 0.77585E+00_JPRB, 0.81178E+00_JPRB, 0.84793E+00_JPRB, 0.89719E+00_JPRB, 0.96119E+00_JPRB ,&
&  0.10543E+01_JPRB, 0.11721E+01_JPRB, 0.12839E+01_JPRB, 0.13889E+01_JPRB, 0.14675E+01_JPRB, 0.14101E+01_JPRB, 0.15219E+01_JPRB ,&
&  0.19291E+01_JPRB, 0.15990E+01_JPRB, 0.12889E+01_JPRB, 0.11627E+01_JPRB, 0.11330E+01_JPRB, 0.11357E+01_JPRB, 0.11346E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.11444E+01_JPRB, 0.11363E+01_JPRB, 0.11280E+01_JPRB, 0.11312E+01_JPRB, 0.11359E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.14895E+01_JPRB, 0.18503E+01_JPRB, 0.13685E+01_JPRB, 0.12126E+01_JPRB, 0.11025E+01_JPRB, 0.97112E+00_JPRB ,&
&  0.91503E+00_JPRB, 0.89544E+00_JPRB, 0.89517E+00_JPRB, 0.89675E+00_JPRB, 0.88859E+00_JPRB, 0.88871E+00_JPRB, 0.89333E+00_JPRB ,&
&  0.90828E+00_JPRB, 0.92290E+00_JPRB, 0.93114E+00_JPRB, 0.93529E+00_JPRB, 0.92865E+00_JPRB, 0.91164E+00_JPRB, 0.89081E+00_JPRB ,&
&  0.89101E+00_JPRB, 0.89667E+00_JPRB, 0.89852E+00_JPRB, 0.89638E+00_JPRB, 0.88718E+00_JPRB /)
ZBURD(:, 10,  4) = (/ &
&  0.86052E+00_JPRB, 0.88676E+00_JPRB, 0.91690E+00_JPRB, 0.90919E+00_JPRB, 0.89460E+00_JPRB, 0.84051E+00_JPRB, 0.78756E+00_JPRB ,&
&  0.75696E+00_JPRB, 0.74692E+00_JPRB, 0.74422E+00_JPRB, 0.73629E+00_JPRB, 0.72812E+00_JPRB, 0.72283E+00_JPRB, 0.71718E+00_JPRB ,&
&  0.71208E+00_JPRB, 0.70824E+00_JPRB, 0.70391E+00_JPRB, 0.70236E+00_JPRB, 0.71256E+00_JPRB, 0.71764E+00_JPRB, 0.71627E+00_JPRB ,&
&  0.71620E+00_JPRB, 0.71769E+00_JPRB, 0.70446E+00_JPRB, 0.68087E+00_JPRB, 0.66956E+00_JPRB, 0.67096E+00_JPRB, 0.67885E+00_JPRB ,&
&  0.66894E+00_JPRB, 0.65273E+00_JPRB, 0.65903E+00_JPRB, 0.67610E+00_JPRB, 0.66954E+00_JPRB, 0.66863E+00_JPRB, 0.61646E+00_JPRB ,&
&  0.54448E+00_JPRB, 0.49585E+00_JPRB, 0.47732E+00_JPRB, 0.50951E+00_JPRB, 0.57639E+00_JPRB, 0.67704E+00_JPRB, 0.72440E+00_JPRB ,&
&  0.76034E+00_JPRB, 0.81394E+00_JPRB, 0.87083E+00_JPRB, 0.94158E+00_JPRB, 0.10367E+01_JPRB, 0.11551E+01_JPRB, 0.12907E+01_JPRB ,&
&  0.14062E+01_JPRB, 0.14838E+01_JPRB, 0.15192E+01_JPRB, 0.13409E+01_JPRB, 0.13113E+01_JPRB, 0.13479E+01_JPRB, 0.15256E+01_JPRB ,&
&  0.17040E+01_JPRB, 0.15838E+01_JPRB, 0.14775E+01_JPRB, 0.14896E+01_JPRB, 0.14658E+01_JPRB, 0.14866E+01_JPRB, 0.15028E+01_JPRB ,&
&  0.14772E+01_JPRB, 0.14161E+01_JPRB, 0.13408E+01_JPRB, 0.12806E+01_JPRB, 0.12647E+01_JPRB, 0.12873E+01_JPRB, 0.13099E+01_JPRB ,&
&  0.13923E+01_JPRB, 0.15521E+01_JPRB, 0.14519E+01_JPRB, 0.10875E+01_JPRB, 0.10279E+01_JPRB, 0.10128E+01_JPRB, 0.98624E+00_JPRB ,&
&  0.92810E+00_JPRB, 0.89064E+00_JPRB, 0.87809E+00_JPRB, 0.87508E+00_JPRB, 0.85048E+00_JPRB, 0.83437E+00_JPRB, 0.82723E+00_JPRB ,&
&  0.83088E+00_JPRB, 0.84473E+00_JPRB, 0.86242E+00_JPRB, 0.90050E+00_JPRB, 0.91282E+00_JPRB, 0.91127E+00_JPRB, 0.89196E+00_JPRB ,&
&  0.87726E+00_JPRB, 0.87251E+00_JPRB, 0.86789E+00_JPRB, 0.86031E+00_JPRB, 0.85833E+00_JPRB /)
ZBURD(:, 11,  4) = (/ &
&  0.98432E+00_JPRB, 0.97877E+00_JPRB, 0.95857E+00_JPRB, 0.94389E+00_JPRB, 0.89088E+00_JPRB, 0.85801E+00_JPRB, 0.81181E+00_JPRB ,&
&  0.79681E+00_JPRB, 0.76995E+00_JPRB, 0.72825E+00_JPRB, 0.68082E+00_JPRB, 0.65598E+00_JPRB, 0.64606E+00_JPRB, 0.65053E+00_JPRB ,&
&  0.66366E+00_JPRB, 0.67414E+00_JPRB, 0.68118E+00_JPRB, 0.68455E+00_JPRB, 0.69470E+00_JPRB, 0.69891E+00_JPRB, 0.69530E+00_JPRB ,&
&  0.69942E+00_JPRB, 0.70995E+00_JPRB, 0.71396E+00_JPRB, 0.70930E+00_JPRB, 0.70813E+00_JPRB, 0.71186E+00_JPRB, 0.70828E+00_JPRB ,&
&  0.68873E+00_JPRB, 0.66716E+00_JPRB, 0.66782E+00_JPRB, 0.69582E+00_JPRB, 0.67576E+00_JPRB, 0.67625E+00_JPRB, 0.61353E+00_JPRB ,&
&  0.54321E+00_JPRB, 0.52610E+00_JPRB, 0.56359E+00_JPRB, 0.67175E+00_JPRB, 0.70391E+00_JPRB, 0.74168E+00_JPRB, 0.77346E+00_JPRB ,&
&  0.77485E+00_JPRB, 0.80044E+00_JPRB, 0.88996E+00_JPRB, 0.10722E+01_JPRB, 0.11856E+01_JPRB, 0.13457E+01_JPRB, 0.15112E+01_JPRB ,&
&  0.16421E+01_JPRB, 0.17439E+01_JPRB, 0.18244E+01_JPRB, 0.16908E+01_JPRB, 0.17191E+01_JPRB, 0.19499E+01_JPRB, 0.17758E+01_JPRB ,&
&  0.17435E+01_JPRB, 0.16870E+01_JPRB, 0.17456E+01_JPRB, 0.18810E+01_JPRB, 0.18038E+01_JPRB, 0.18761E+01_JPRB, 0.18549E+01_JPRB ,&
&  0.16962E+01_JPRB, 0.15865E+01_JPRB, 0.14941E+01_JPRB, 0.14470E+01_JPRB, 0.14054E+01_JPRB, 0.13909E+01_JPRB, 0.14083E+01_JPRB ,&
&  0.14854E+01_JPRB, 0.14963E+01_JPRB, 0.13361E+01_JPRB, 0.10858E+01_JPRB, 0.98877E+00_JPRB, 0.95044E+00_JPRB, 0.92940E+00_JPRB ,&
&  0.89989E+00_JPRB, 0.88094E+00_JPRB, 0.88093E+00_JPRB, 0.88295E+00_JPRB, 0.86746E+00_JPRB, 0.83877E+00_JPRB, 0.81128E+00_JPRB ,&
&  0.79293E+00_JPRB, 0.79086E+00_JPRB, 0.78228E+00_JPRB, 0.81565E+00_JPRB, 0.85318E+00_JPRB, 0.88791E+00_JPRB, 0.88621E+00_JPRB ,&
&  0.88037E+00_JPRB, 0.89224E+00_JPRB, 0.90937E+00_JPRB, 0.94990E+00_JPRB, 0.96762E+00_JPRB /)
ZBURD(:, 12,  4) = (/ &
&  0.10132E+01_JPRB, 0.10125E+01_JPRB, 0.10005E+01_JPRB, 0.97679E+00_JPRB, 0.93213E+00_JPRB, 0.87491E+00_JPRB, 0.77443E+00_JPRB ,&
&  0.76084E+00_JPRB, 0.71718E+00_JPRB, 0.67391E+00_JPRB, 0.64631E+00_JPRB, 0.64044E+00_JPRB, 0.62847E+00_JPRB, 0.62530E+00_JPRB ,&
&  0.63001E+00_JPRB, 0.65522E+00_JPRB, 0.68576E+00_JPRB, 0.69276E+00_JPRB, 0.70182E+00_JPRB, 0.70413E+00_JPRB, 0.69440E+00_JPRB ,&
&  0.70461E+00_JPRB, 0.72914E+00_JPRB, 0.75875E+00_JPRB, 0.78437E+00_JPRB, 0.76821E+00_JPRB, 0.75635E+00_JPRB, 0.74812E+00_JPRB ,&
&  0.74215E+00_JPRB, 0.73615E+00_JPRB, 0.72871E+00_JPRB, 0.73662E+00_JPRB, 0.71276E+00_JPRB, 0.68845E+00_JPRB, 0.68487E+00_JPRB ,&
&  0.57908E+00_JPRB, 0.58540E+00_JPRB, 0.67721E+00_JPRB, 0.71130E+00_JPRB, 0.74528E+00_JPRB, 0.78283E+00_JPRB, 0.84331E+00_JPRB ,&
&  0.90003E+00_JPRB, 0.96236E+00_JPRB, 0.10505E+01_JPRB, 0.11786E+01_JPRB, 0.13649E+01_JPRB, 0.15957E+01_JPRB, 0.18156E+01_JPRB ,&
&  0.20307E+01_JPRB, 0.16824E+01_JPRB, 0.16568E+01_JPRB, 0.19274E+01_JPRB, 0.24434E+01_JPRB, 0.22472E+01_JPRB, 0.21040E+01_JPRB ,&
&  0.19768E+01_JPRB, 0.18715E+01_JPRB, 0.18470E+01_JPRB, 0.20112E+01_JPRB, 0.20870E+01_JPRB, 0.22396E+01_JPRB, 0.23373E+01_JPRB ,&
&  0.21132E+01_JPRB, 0.19456E+01_JPRB, 0.17633E+01_JPRB, 0.16654E+01_JPRB, 0.16012E+01_JPRB, 0.15588E+01_JPRB, 0.15234E+01_JPRB ,&
&  0.14979E+01_JPRB, 0.14443E+01_JPRB, 0.13198E+01_JPRB, 0.11751E+01_JPRB, 0.10688E+01_JPRB, 0.10074E+01_JPRB, 0.97136E+00_JPRB ,&
&  0.94042E+00_JPRB, 0.91201E+00_JPRB, 0.89019E+00_JPRB, 0.87877E+00_JPRB, 0.87870E+00_JPRB, 0.89469E+00_JPRB, 0.94906E+00_JPRB ,&
&  0.90481E+00_JPRB, 0.87720E+00_JPRB, 0.83110E+00_JPRB, 0.84028E+00_JPRB, 0.88538E+00_JPRB, 0.93062E+00_JPRB, 0.94559E+00_JPRB ,&
&  0.98371E+00_JPRB, 0.98025E+00_JPRB, 0.95323E+00_JPRB, 0.97408E+00_JPRB, 0.98818E+00_JPRB /)
ZBURD(:, 13,  4) = (/ &
&  0.10336E+01_JPRB, 0.10406E+01_JPRB, 0.10269E+01_JPRB, 0.10197E+01_JPRB, 0.97128E+00_JPRB, 0.88335E+00_JPRB, 0.80197E+00_JPRB ,&
&  0.85551E+00_JPRB, 0.78814E+00_JPRB, 0.79305E+00_JPRB, 0.68135E+00_JPRB, 0.64581E+00_JPRB, 0.63624E+00_JPRB, 0.62181E+00_JPRB ,&
&  0.60666E+00_JPRB, 0.64539E+00_JPRB, 0.69176E+00_JPRB, 0.70942E+00_JPRB, 0.72880E+00_JPRB, 0.73635E+00_JPRB, 0.72357E+00_JPRB ,&
&  0.74082E+00_JPRB, 0.77965E+00_JPRB, 0.85194E+00_JPRB, 0.86933E+00_JPRB, 0.83404E+00_JPRB, 0.83133E+00_JPRB, 0.83973E+00_JPRB ,&
&  0.80844E+00_JPRB, 0.79709E+00_JPRB, 0.82729E+00_JPRB, 0.82559E+00_JPRB, 0.80372E+00_JPRB, 0.74365E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.72049E+00_JPRB, 0.69695E+00_JPRB, 0.75818E+00_JPRB, 0.78629E+00_JPRB, 0.81887E+00_JPRB, 0.85714E+00_JPRB, 0.90137E+00_JPRB ,&
&  0.97155E+00_JPRB, 0.10798E+01_JPRB, 0.12338E+01_JPRB, 0.14530E+01_JPRB, 0.16972E+01_JPRB, 0.19995E+01_JPRB, 0.22184E+01_JPRB ,&
&  0.24843E+01_JPRB, 0.18918E+01_JPRB, 0.23563E+01_JPRB, 0.25739E+01_JPRB, 0.29457E+01_JPRB, 0.29640E+01_JPRB, 0.28958E+01_JPRB ,&
&  0.28916E+01_JPRB, 0.25344E+01_JPRB, 0.23173E+01_JPRB, 0.22048E+01_JPRB, 0.22610E+01_JPRB, 0.24735E+01_JPRB, 0.26386E+01_JPRB ,&
&  0.27855E+01_JPRB, 0.26383E+01_JPRB, 0.24371E+01_JPRB, 0.22255E+01_JPRB, 0.20548E+01_JPRB, 0.19371E+01_JPRB, 0.18128E+01_JPRB ,&
&  0.17003E+01_JPRB, 0.15793E+01_JPRB, 0.14588E+01_JPRB, 0.12964E+01_JPRB, 0.12128E+01_JPRB, 0.11392E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.10207E+01_JPRB, 0.95937E+00_JPRB, 0.89103E+00_JPRB, 0.86199E+00_JPRB, 0.83181E+00_JPRB, 0.82956E+00_JPRB, 0.86673E+00_JPRB ,&
&  0.88572E+00_JPRB, 0.86230E+00_JPRB, 0.91819E+00_JPRB, 0.95799E+00_JPRB, 0.97877E+00_JPRB, 0.99493E+00_JPRB, 0.10507E+01_JPRB ,&
&  0.98592E+00_JPRB, 0.98623E+00_JPRB, 0.10362E+01_JPRB, 0.10299E+01_JPRB, 0.10297E+01_JPRB /)
ZBURD(:, 14,  4) = (/ &
&  0.10710E+01_JPRB, 0.10909E+01_JPRB, 0.10655E+01_JPRB, 0.10583E+01_JPRB, 0.98605E+00_JPRB, 0.94368E+00_JPRB, 0.85185E+00_JPRB ,&
&  0.87235E+00_JPRB, 0.84311E+00_JPRB, 0.77842E+00_JPRB, 0.76146E+00_JPRB, 0.77404E+00_JPRB, 0.69340E+00_JPRB, 0.62458E+00_JPRB ,&
&  0.60123E+00_JPRB, 0.63019E+00_JPRB, 0.72331E+00_JPRB, 0.76808E+00_JPRB, 0.77654E+00_JPRB, 0.78557E+00_JPRB, 0.77604E+00_JPRB ,&
&  0.86407E+00_JPRB, 0.88619E+00_JPRB, 0.95824E+00_JPRB, 0.99672E+00_JPRB, 0.97616E+00_JPRB, 0.98140E+00_JPRB, 0.10006E+01_JPRB ,&
&  0.93355E+00_JPRB, 0.91069E+00_JPRB, 0.90981E+00_JPRB, 0.84431E+00_JPRB, 0.91710E+00_JPRB, 0.87210E+00_JPRB, 0.80925E+00_JPRB ,&
&  0.79375E+00_JPRB, 0.82421E+00_JPRB, 0.85897E+00_JPRB, 0.89302E+00_JPRB, 0.90480E+00_JPRB, 0.92911E+00_JPRB, 0.98428E+00_JPRB ,&
&  0.10794E+01_JPRB, 0.12325E+01_JPRB, 0.14126E+01_JPRB, 0.16908E+01_JPRB, 0.19666E+01_JPRB, 0.20495E+01_JPRB, 0.24520E+01_JPRB ,&
&  0.26355E+01_JPRB, 0.29006E+01_JPRB, 0.32455E+01_JPRB, 0.30685E+01_JPRB, 0.34452E+01_JPRB, 0.34866E+01_JPRB, 0.32691E+01_JPRB ,&
&  0.33444E+01_JPRB, 0.30552E+01_JPRB, 0.29620E+01_JPRB, 0.29981E+01_JPRB, 0.31812E+01_JPRB, 0.31250E+01_JPRB, 0.30984E+01_JPRB ,&
&  0.31635E+01_JPRB, 0.33972E+01_JPRB, 0.31776E+01_JPRB, 0.30909E+01_JPRB, 0.28671E+01_JPRB, 0.26638E+01_JPRB, 0.24220E+01_JPRB ,&
&  0.22522E+01_JPRB, 0.20582E+01_JPRB, 0.18755E+01_JPRB, 0.17294E+01_JPRB, 0.15337E+01_JPRB, 0.13720E+01_JPRB, 0.12355E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.94518E+00_JPRB, 0.88842E+00_JPRB, 0.85030E+00_JPRB, 0.83978E+00_JPRB, 0.79142E+00_JPRB, 0.80031E+00_JPRB ,&
&  0.85472E+00_JPRB, 0.92209E+00_JPRB, 0.10204E+01_JPRB, 0.10631E+01_JPRB, 0.11235E+01_JPRB, 0.11819E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.99494E+00_JPRB, 0.10732E+01_JPRB, 0.10812E+01_JPRB, 0.10768E+01_JPRB, 0.10711E+01_JPRB /)
ZBURD(:, 15,  4) = (/ &
&  0.11750E+01_JPRB, 0.11863E+01_JPRB, 0.11547E+01_JPRB, 0.11132E+01_JPRB, 0.10188E+01_JPRB, 0.96196E+00_JPRB, 0.88978E+00_JPRB ,&
&  0.89172E+00_JPRB, 0.85829E+00_JPRB, 0.80059E+00_JPRB, 0.77834E+00_JPRB, 0.78344E+00_JPRB, 0.77365E+00_JPRB, 0.71067E+00_JPRB ,&
&  0.67870E+00_JPRB, 0.75636E+00_JPRB, 0.76808E+00_JPRB, 0.89513E+00_JPRB, 0.99110E+00_JPRB, 0.93286E+00_JPRB, 0.96214E+00_JPRB ,&
&  0.10674E+01_JPRB, 0.10889E+01_JPRB, 0.11022E+01_JPRB, 0.10992E+01_JPRB, 0.11004E+01_JPRB, 0.11024E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.10003E+01_JPRB, 0.94816E+00_JPRB, 0.91075E+00_JPRB, 0.90732E+00_JPRB, 0.99048E+00_JPRB, 0.10304E+01_JPRB, 0.98094E+00_JPRB ,&
&  0.92167E+00_JPRB, 0.89361E+00_JPRB, 0.89571E+00_JPRB, 0.95841E+00_JPRB, 0.99405E+00_JPRB, 0.99934E+00_JPRB, 0.10569E+01_JPRB ,&
&  0.11640E+01_JPRB, 0.13064E+01_JPRB, 0.14552E+01_JPRB, 0.16439E+01_JPRB, 0.17507E+01_JPRB, 0.21076E+01_JPRB, 0.26601E+01_JPRB ,&
&  0.28691E+01_JPRB, 0.31235E+01_JPRB, 0.34587E+01_JPRB, 0.38549E+01_JPRB, 0.41617E+01_JPRB, 0.39590E+01_JPRB, 0.37733E+01_JPRB ,&
&  0.37844E+01_JPRB, 0.36964E+01_JPRB, 0.44047E+01_JPRB, 0.42020E+01_JPRB, 0.39683E+01_JPRB, 0.38805E+01_JPRB, 0.37162E+01_JPRB ,&
&  0.36594E+01_JPRB, 0.36168E+01_JPRB, 0.35753E+01_JPRB, 0.34977E+01_JPRB, 0.33997E+01_JPRB, 0.33603E+01_JPRB, 0.31605E+01_JPRB ,&
&  0.30280E+01_JPRB, 0.27767E+01_JPRB, 0.22726E+01_JPRB, 0.18612E+01_JPRB, 0.14871E+01_JPRB, 0.14970E+01_JPRB, 0.13539E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.94788E+00_JPRB, 0.90799E+00_JPRB, 0.86158E+00_JPRB, 0.86527E+00_JPRB, 0.84461E+00_JPRB, 0.81817E+00_JPRB ,&
&  0.86264E+00_JPRB, 0.98626E+00_JPRB, 0.10368E+01_JPRB, 0.11395E+01_JPRB, 0.12635E+01_JPRB, 0.13587E+01_JPRB, 0.11569E+01_JPRB ,&
&  0.11439E+01_JPRB, 0.12467E+01_JPRB, 0.12601E+01_JPRB, 0.12437E+01_JPRB, 0.12051E+01_JPRB /)
ZBURD(:, 16,  4) = (/ &
&  0.13943E+01_JPRB, 0.13471E+01_JPRB, 0.12813E+01_JPRB, 0.11855E+01_JPRB, 0.10629E+01_JPRB, 0.97228E+00_JPRB, 0.89593E+00_JPRB ,&
&  0.88945E+00_JPRB, 0.85960E+00_JPRB, 0.81393E+00_JPRB, 0.79342E+00_JPRB, 0.78829E+00_JPRB, 0.78623E+00_JPRB, 0.80722E+00_JPRB ,&
&  0.74492E+00_JPRB, 0.93517E+00_JPRB, 0.90151E+00_JPRB, 0.10712E+01_JPRB, 0.14407E+01_JPRB, 0.11934E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.12051E+01_JPRB, 0.12815E+01_JPRB, 0.12517E+01_JPRB, 0.12207E+01_JPRB, 0.12657E+01_JPRB, 0.12867E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.11639E+01_JPRB, 0.10519E+01_JPRB, 0.10134E+01_JPRB, 0.10067E+01_JPRB, 0.10873E+01_JPRB, 0.11516E+01_JPRB, 0.11605E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.10501E+01_JPRB, 0.10024E+01_JPRB, 0.10168E+01_JPRB, 0.10562E+01_JPRB, 0.10881E+01_JPRB, 0.11495E+01_JPRB ,&
&  0.12486E+01_JPRB, 0.13508E+01_JPRB, 0.14663E+01_JPRB, 0.16251E+01_JPRB, 0.16780E+01_JPRB, 0.21096E+01_JPRB, 0.28257E+01_JPRB ,&
&  0.34107E+01_JPRB, 0.34062E+01_JPRB, 0.37412E+01_JPRB, 0.46532E+01_JPRB, 0.50135E+01_JPRB, 0.48702E+01_JPRB, 0.46160E+01_JPRB ,&
&  0.43775E+01_JPRB, 0.41860E+01_JPRB, 0.43437E+01_JPRB, 0.43802E+01_JPRB, 0.41483E+01_JPRB, 0.42692E+01_JPRB, 0.41724E+01_JPRB ,&
&  0.39240E+01_JPRB, 0.38053E+01_JPRB, 0.38423E+01_JPRB, 0.38286E+01_JPRB, 0.36166E+01_JPRB, 0.36627E+01_JPRB, 0.35301E+01_JPRB ,&
&  0.33937E+01_JPRB, 0.28992E+01_JPRB, 0.20944E+01_JPRB, 0.16978E+01_JPRB, 0.12543E+01_JPRB, 0.12462E+01_JPRB, 0.15368E+01_JPRB ,&
&  0.14781E+01_JPRB, 0.13524E+01_JPRB, 0.11810E+01_JPRB, 0.10586E+01_JPRB, 0.99011E+00_JPRB, 0.10208E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.11270E+01_JPRB, 0.13054E+01_JPRB, 0.14972E+01_JPRB, 0.16268E+01_JPRB, 0.16608E+01_JPRB, 0.15180E+01_JPRB ,&
&  0.15898E+01_JPRB, 0.15557E+01_JPRB, 0.15400E+01_JPRB, 0.15246E+01_JPRB, 0.14732E+01_JPRB /)
ZBURD(:, 17,  4) = (/ &
&  0.16309E+01_JPRB, 0.15428E+01_JPRB, 0.14108E+01_JPRB, 0.12543E+01_JPRB, 0.10856E+01_JPRB, 0.96208E+00_JPRB, 0.90739E+00_JPRB ,&
&  0.89827E+00_JPRB, 0.87235E+00_JPRB, 0.83151E+00_JPRB, 0.81032E+00_JPRB, 0.80144E+00_JPRB, 0.80684E+00_JPRB, 0.86622E+00_JPRB ,&
&  0.10275E+01_JPRB, 0.13334E+01_JPRB, 0.11781E+01_JPRB, 0.10203E+01_JPRB, 0.11859E+01_JPRB, 0.11626E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12255E+01_JPRB, 0.14043E+01_JPRB, 0.13297E+01_JPRB, 0.13334E+01_JPRB, 0.15026E+01_JPRB, 0.15792E+01_JPRB, 0.16072E+01_JPRB ,&
&  0.15716E+01_JPRB, 0.14870E+01_JPRB, 0.13936E+01_JPRB, 0.13884E+01_JPRB, 0.14520E+01_JPRB, 0.13796E+01_JPRB, 0.13834E+01_JPRB ,&
&  0.13021E+01_JPRB, 0.12295E+01_JPRB, 0.11409E+01_JPRB, 0.11226E+01_JPRB, 0.11623E+01_JPRB, 0.12318E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.13395E+01_JPRB, 0.13993E+01_JPRB, 0.14292E+01_JPRB, 0.15355E+01_JPRB, 0.17042E+01_JPRB, 0.19660E+01_JPRB, 0.22233E+01_JPRB ,&
&  0.26953E+01_JPRB, 0.29406E+01_JPRB, 0.32204E+01_JPRB, 0.40683E+01_JPRB, 0.46576E+01_JPRB, 0.47646E+01_JPRB, 0.48040E+01_JPRB ,&
&  0.48036E+01_JPRB, 0.47771E+01_JPRB, 0.48961E+01_JPRB, 0.47318E+01_JPRB, 0.46149E+01_JPRB, 0.46426E+01_JPRB, 0.45590E+01_JPRB ,&
&  0.41511E+01_JPRB, 0.39613E+01_JPRB, 0.39785E+01_JPRB, 0.37963E+01_JPRB, 0.36275E+01_JPRB, 0.34064E+01_JPRB, 0.34193E+01_JPRB ,&
&  0.34055E+01_JPRB, 0.19954E+01_JPRB, 0.14958E+01_JPRB, 0.16533E+01_JPRB, 0.12822E+01_JPRB, 0.11653E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.13467E+01_JPRB, 0.13194E+01_JPRB, 0.13490E+01_JPRB, 0.12980E+01_JPRB, 0.12518E+01_JPRB, 0.13644E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.15007E+01_JPRB, 0.15270E+01_JPRB, 0.18169E+01_JPRB, 0.20024E+01_JPRB, 0.20550E+01_JPRB, 0.20387E+01_JPRB, 0.19777E+01_JPRB ,&
&  0.19086E+01_JPRB, 0.18375E+01_JPRB, 0.17853E+01_JPRB, 0.17533E+01_JPRB, 0.17087E+01_JPRB /)
ZBURD(:, 18,  4) = (/ &
&  0.18674E+01_JPRB, 0.17622E+01_JPRB, 0.15826E+01_JPRB, 0.13708E+01_JPRB, 0.11516E+01_JPRB, 0.10004E+01_JPRB, 0.96852E+00_JPRB ,&
&  0.95405E+00_JPRB, 0.93182E+00_JPRB, 0.89264E+00_JPRB, 0.86836E+00_JPRB, 0.85940E+00_JPRB, 0.85807E+00_JPRB, 0.88582E+00_JPRB ,&
&  0.92379E+00_JPRB, 0.12816E+01_JPRB, 0.10695E+01_JPRB, 0.89858E+00_JPRB, 0.80327E+00_JPRB, 0.93094E+00_JPRB, 0.10432E+01_JPRB ,&
&  0.11934E+01_JPRB, 0.13831E+01_JPRB, 0.14549E+01_JPRB, 0.17095E+01_JPRB, 0.19304E+01_JPRB, 0.20001E+01_JPRB, 0.21380E+01_JPRB ,&
&  0.21374E+01_JPRB, 0.21567E+01_JPRB, 0.19440E+01_JPRB, 0.19197E+01_JPRB, 0.18196E+01_JPRB, 0.16441E+01_JPRB, 0.16840E+01_JPRB ,&
&  0.15599E+01_JPRB, 0.14637E+01_JPRB, 0.13684E+01_JPRB, 0.13324E+01_JPRB, 0.13478E+01_JPRB, 0.13520E+01_JPRB, 0.13071E+01_JPRB ,&
&  0.13138E+01_JPRB, 0.13539E+01_JPRB, 0.14338E+01_JPRB, 0.15155E+01_JPRB, 0.16554E+01_JPRB, 0.18865E+01_JPRB, 0.20003E+01_JPRB ,&
&  0.22653E+01_JPRB, 0.22316E+01_JPRB, 0.21224E+01_JPRB, 0.29066E+01_JPRB, 0.43419E+01_JPRB, 0.47609E+01_JPRB, 0.45742E+01_JPRB ,&
&  0.51148E+01_JPRB, 0.53621E+01_JPRB, 0.54008E+01_JPRB, 0.52142E+01_JPRB, 0.50746E+01_JPRB, 0.49924E+01_JPRB, 0.48662E+01_JPRB ,&
&  0.44887E+01_JPRB, 0.43069E+01_JPRB, 0.41705E+01_JPRB, 0.41034E+01_JPRB, 0.38689E+01_JPRB, 0.37841E+01_JPRB, 0.37859E+01_JPRB ,&
&  0.34453E+01_JPRB, 0.28056E+01_JPRB, 0.18568E+01_JPRB, 0.18372E+01_JPRB, 0.13375E+01_JPRB, 0.11571E+01_JPRB, 0.13033E+01_JPRB ,&
&  0.12489E+01_JPRB, 0.13275E+01_JPRB, 0.14313E+01_JPRB, 0.14406E+01_JPRB, 0.16834E+01_JPRB, 0.18787E+01_JPRB, 0.20993E+01_JPRB ,&
&  0.22096E+01_JPRB, 0.20942E+01_JPRB, 0.24312E+01_JPRB, 0.24763E+01_JPRB, 0.24640E+01_JPRB, 0.24599E+01_JPRB, 0.24284E+01_JPRB ,&
&  0.23370E+01_JPRB, 0.22255E+01_JPRB, 0.21311E+01_JPRB, 0.20363E+01_JPRB, 0.19535E+01_JPRB /)
ZBURD(:, 19,  4) = (/ &
&  0.19796E+01_JPRB, 0.18481E+01_JPRB, 0.16381E+01_JPRB, 0.14109E+01_JPRB, 0.11914E+01_JPRB, 0.10237E+01_JPRB, 0.99914E+00_JPRB ,&
&  0.10366E+01_JPRB, 0.10343E+01_JPRB, 0.10040E+01_JPRB, 0.95140E+00_JPRB, 0.92334E+00_JPRB, 0.91431E+00_JPRB, 0.90215E+00_JPRB ,&
&  0.88150E+00_JPRB, 0.96472E+00_JPRB, 0.80038E+00_JPRB, 0.66879E+00_JPRB, 0.64911E+00_JPRB, 0.71914E+00_JPRB, 0.90270E+00_JPRB ,&
&  0.11076E+01_JPRB, 0.14788E+01_JPRB, 0.18330E+01_JPRB, 0.21350E+01_JPRB, 0.25234E+01_JPRB, 0.28309E+01_JPRB, 0.29695E+01_JPRB ,&
&  0.31461E+01_JPRB, 0.26787E+01_JPRB, 0.26815E+01_JPRB, 0.24909E+01_JPRB, 0.22978E+01_JPRB, 0.21378E+01_JPRB, 0.20102E+01_JPRB ,&
&  0.18492E+01_JPRB, 0.17244E+01_JPRB, 0.15771E+01_JPRB, 0.14797E+01_JPRB, 0.14333E+01_JPRB, 0.13933E+01_JPRB, 0.13647E+01_JPRB ,&
&  0.13994E+01_JPRB, 0.14635E+01_JPRB, 0.15200E+01_JPRB, 0.15622E+01_JPRB, 0.17359E+01_JPRB, 0.19913E+01_JPRB, 0.21047E+01_JPRB ,&
&  0.20126E+01_JPRB, 0.20794E+01_JPRB, 0.27940E+01_JPRB, 0.32021E+01_JPRB, 0.37600E+01_JPRB, 0.44081E+01_JPRB, 0.49254E+01_JPRB ,&
&  0.53135E+01_JPRB, 0.52212E+01_JPRB, 0.52212E+01_JPRB, 0.47249E+01_JPRB, 0.53374E+01_JPRB, 0.52267E+01_JPRB, 0.49176E+01_JPRB ,&
&  0.45439E+01_JPRB, 0.44879E+01_JPRB, 0.41592E+01_JPRB, 0.39593E+01_JPRB, 0.39046E+01_JPRB, 0.40987E+01_JPRB, 0.36499E+01_JPRB ,&
&  0.30122E+01_JPRB, 0.33364E+01_JPRB, 0.29172E+01_JPRB, 0.23905E+01_JPRB, 0.18445E+01_JPRB, 0.16523E+01_JPRB, 0.15446E+01_JPRB ,&
&  0.14681E+01_JPRB, 0.14525E+01_JPRB, 0.15341E+01_JPRB, 0.18400E+01_JPRB, 0.23618E+01_JPRB, 0.27038E+01_JPRB, 0.28466E+01_JPRB ,&
&  0.28109E+01_JPRB, 0.32807E+01_JPRB, 0.32341E+01_JPRB, 0.30903E+01_JPRB, 0.30180E+01_JPRB, 0.29720E+01_JPRB, 0.28574E+01_JPRB ,&
&  0.26475E+01_JPRB, 0.24814E+01_JPRB, 0.23443E+01_JPRB, 0.22178E+01_JPRB, 0.20883E+01_JPRB /)
ZBURD(:, 20,  4) = (/ &
&  0.20147E+01_JPRB, 0.18955E+01_JPRB, 0.16932E+01_JPRB, 0.14523E+01_JPRB, 0.12377E+01_JPRB, 0.10812E+01_JPRB, 0.10441E+01_JPRB ,&
&  0.10663E+01_JPRB, 0.10873E+01_JPRB, 0.10950E+01_JPRB, 0.10861E+01_JPRB, 0.10620E+01_JPRB, 0.10183E+01_JPRB, 0.97268E+00_JPRB ,&
&  0.93742E+00_JPRB, 0.82662E+00_JPRB, 0.65969E+00_JPRB, 0.62947E+00_JPRB, 0.67494E+00_JPRB, 0.71462E+00_JPRB, 0.84597E+00_JPRB ,&
&  0.10864E+01_JPRB, 0.16597E+01_JPRB, 0.21791E+01_JPRB, 0.27790E+01_JPRB, 0.33724E+01_JPRB, 0.38195E+01_JPRB, 0.38502E+01_JPRB ,&
&  0.36333E+01_JPRB, 0.36371E+01_JPRB, 0.34112E+01_JPRB, 0.30775E+01_JPRB, 0.27676E+01_JPRB, 0.25231E+01_JPRB, 0.22954E+01_JPRB ,&
&  0.20558E+01_JPRB, 0.18259E+01_JPRB, 0.16870E+01_JPRB, 0.16100E+01_JPRB, 0.15674E+01_JPRB, 0.15389E+01_JPRB, 0.15257E+01_JPRB ,&
&  0.15341E+01_JPRB, 0.15690E+01_JPRB, 0.15947E+01_JPRB, 0.16642E+01_JPRB, 0.16624E+01_JPRB, 0.18612E+01_JPRB, 0.20052E+01_JPRB ,&
&  0.22769E+01_JPRB, 0.22781E+01_JPRB, 0.23768E+01_JPRB, 0.27959E+01_JPRB, 0.27885E+01_JPRB, 0.32933E+01_JPRB, 0.45394E+01_JPRB ,&
&  0.49637E+01_JPRB, 0.49726E+01_JPRB, 0.51660E+01_JPRB, 0.42217E+01_JPRB, 0.37054E+01_JPRB, 0.52480E+01_JPRB, 0.49635E+01_JPRB ,&
&  0.45481E+01_JPRB, 0.43562E+01_JPRB, 0.40050E+01_JPRB, 0.40299E+01_JPRB, 0.33537E+01_JPRB, 0.29565E+01_JPRB, 0.22060E+01_JPRB ,&
&  0.23779E+01_JPRB, 0.23141E+01_JPRB, 0.26397E+01_JPRB, 0.24507E+01_JPRB, 0.23460E+01_JPRB, 0.21515E+01_JPRB, 0.18597E+01_JPRB ,&
&  0.17160E+01_JPRB, 0.16810E+01_JPRB, 0.18822E+01_JPRB, 0.26452E+01_JPRB, 0.35443E+01_JPRB, 0.33954E+01_JPRB, 0.39054E+01_JPRB ,&
&  0.41363E+01_JPRB, 0.41649E+01_JPRB, 0.36541E+01_JPRB, 0.35567E+01_JPRB, 0.33401E+01_JPRB, 0.31143E+01_JPRB, 0.28630E+01_JPRB ,&
&  0.26643E+01_JPRB, 0.25189E+01_JPRB, 0.23897E+01_JPRB, 0.22525E+01_JPRB, 0.21173E+01_JPRB /)
ZBURD(:, 21,  4) = (/ &
&  0.20310E+01_JPRB, 0.19249E+01_JPRB, 0.17333E+01_JPRB, 0.14976E+01_JPRB, 0.12867E+01_JPRB, 0.11443E+01_JPRB, 0.11096E+01_JPRB ,&
&  0.11281E+01_JPRB, 0.11477E+01_JPRB, 0.11525E+01_JPRB, 0.11392E+01_JPRB, 0.11176E+01_JPRB, 0.10950E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10096E+01_JPRB, 0.91295E+00_JPRB, 0.70679E+00_JPRB, 0.62043E+00_JPRB, 0.75608E+00_JPRB, 0.76335E+00_JPRB, 0.82464E+00_JPRB ,&
&  0.11872E+01_JPRB, 0.18087E+01_JPRB, 0.25086E+01_JPRB, 0.32661E+01_JPRB, 0.40028E+01_JPRB, 0.43524E+01_JPRB, 0.43246E+01_JPRB ,&
&  0.44338E+01_JPRB, 0.40907E+01_JPRB, 0.35607E+01_JPRB, 0.30916E+01_JPRB, 0.27470E+01_JPRB, 0.25120E+01_JPRB, 0.22539E+01_JPRB ,&
&  0.19844E+01_JPRB, 0.17891E+01_JPRB, 0.16726E+01_JPRB, 0.16009E+01_JPRB, 0.15691E+01_JPRB, 0.15312E+01_JPRB, 0.14998E+01_JPRB ,&
&  0.15221E+01_JPRB, 0.15746E+01_JPRB, 0.16319E+01_JPRB, 0.17063E+01_JPRB, 0.16726E+01_JPRB, 0.17571E+01_JPRB, 0.19892E+01_JPRB ,&
&  0.20796E+01_JPRB, 0.20714E+01_JPRB, 0.21668E+01_JPRB, 0.23999E+01_JPRB, 0.28477E+01_JPRB, 0.31511E+01_JPRB, 0.40536E+01_JPRB ,&
&  0.38551E+01_JPRB, 0.36075E+01_JPRB, 0.33731E+01_JPRB, 0.28899E+01_JPRB, 0.33005E+01_JPRB, 0.48577E+01_JPRB, 0.48582E+01_JPRB ,&
&  0.44708E+01_JPRB, 0.42302E+01_JPRB, 0.39853E+01_JPRB, 0.35069E+01_JPRB, 0.28220E+01_JPRB, 0.21372E+01_JPRB, 0.26468E+01_JPRB ,&
&  0.28755E+01_JPRB, 0.29366E+01_JPRB, 0.27831E+01_JPRB, 0.23607E+01_JPRB, 0.21411E+01_JPRB, 0.25430E+01_JPRB, 0.24136E+01_JPRB ,&
&  0.21313E+01_JPRB, 0.21274E+01_JPRB, 0.30459E+01_JPRB, 0.41609E+01_JPRB, 0.46282E+01_JPRB, 0.43567E+01_JPRB, 0.47625E+01_JPRB ,&
&  0.45274E+01_JPRB, 0.43519E+01_JPRB, 0.40283E+01_JPRB, 0.36048E+01_JPRB, 0.32295E+01_JPRB, 0.29246E+01_JPRB, 0.26812E+01_JPRB ,&
&  0.25464E+01_JPRB, 0.24664E+01_JPRB, 0.23685E+01_JPRB, 0.22681E+01_JPRB, 0.21381E+01_JPRB /)
ZBURD(:, 22,  4) = (/ &
&  0.20388E+01_JPRB, 0.19234E+01_JPRB, 0.17248E+01_JPRB, 0.14885E+01_JPRB, 0.13168E+01_JPRB, 0.12010E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11452E+01_JPRB, 0.11405E+01_JPRB, 0.11255E+01_JPRB, 0.11094E+01_JPRB, 0.10938E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.10515E+01_JPRB, 0.11567E+01_JPRB, 0.88865E+00_JPRB, 0.73485E+00_JPRB, 0.78589E+00_JPRB, 0.85380E+00_JPRB, 0.85446E+00_JPRB ,&
&  0.13341E+01_JPRB, 0.20950E+01_JPRB, 0.26775E+01_JPRB, 0.33685E+01_JPRB, 0.43064E+01_JPRB, 0.40771E+01_JPRB, 0.41142E+01_JPRB ,&
&  0.40121E+01_JPRB, 0.36921E+01_JPRB, 0.33342E+01_JPRB, 0.29342E+01_JPRB, 0.25381E+01_JPRB, 0.22909E+01_JPRB, 0.21228E+01_JPRB ,&
&  0.19371E+01_JPRB, 0.17725E+01_JPRB, 0.16746E+01_JPRB, 0.16338E+01_JPRB, 0.15930E+01_JPRB, 0.15623E+01_JPRB, 0.15581E+01_JPRB ,&
&  0.15598E+01_JPRB, 0.15699E+01_JPRB, 0.15891E+01_JPRB, 0.17004E+01_JPRB, 0.18298E+01_JPRB, 0.17891E+01_JPRB, 0.21077E+01_JPRB ,&
&  0.20391E+01_JPRB, 0.20478E+01_JPRB, 0.20809E+01_JPRB, 0.23324E+01_JPRB, 0.25876E+01_JPRB, 0.29213E+01_JPRB, 0.36183E+01_JPRB ,&
&  0.34961E+01_JPRB, 0.34960E+01_JPRB, 0.36588E+01_JPRB, 0.36218E+01_JPRB, 0.32280E+01_JPRB, 0.35470E+01_JPRB, 0.47388E+01_JPRB ,&
&  0.37245E+01_JPRB, 0.37475E+01_JPRB, 0.37541E+01_JPRB, 0.36497E+01_JPRB, 0.18013E+01_JPRB, 0.12938E+01_JPRB, 0.21192E+01_JPRB ,&
&  0.23118E+01_JPRB, 0.14202E+01_JPRB, 0.10843E+01_JPRB, 0.15467E+01_JPRB, 0.12686E+01_JPRB, 0.16507E+01_JPRB, 0.25207E+01_JPRB ,&
&  0.28136E+01_JPRB, 0.27445E+01_JPRB, 0.38993E+01_JPRB, 0.45570E+01_JPRB, 0.47177E+01_JPRB, 0.48610E+01_JPRB, 0.49191E+01_JPRB ,&
&  0.47332E+01_JPRB, 0.39896E+01_JPRB, 0.39123E+01_JPRB, 0.34410E+01_JPRB, 0.30624E+01_JPRB, 0.28251E+01_JPRB, 0.26409E+01_JPRB ,&
&  0.24492E+01_JPRB, 0.22903E+01_JPRB, 0.22107E+01_JPRB, 0.21903E+01_JPRB, 0.21290E+01_JPRB /)
ZBURD(:, 23,  4) = (/ &
&  0.19909E+01_JPRB, 0.18735E+01_JPRB, 0.16693E+01_JPRB, 0.14435E+01_JPRB, 0.13057E+01_JPRB, 0.12029E+01_JPRB, 0.11437E+01_JPRB ,&
&  0.11285E+01_JPRB, 0.11255E+01_JPRB, 0.11181E+01_JPRB, 0.11046E+01_JPRB, 0.10907E+01_JPRB, 0.10870E+01_JPRB, 0.10862E+01_JPRB ,&
&  0.10579E+01_JPRB, 0.10620E+01_JPRB, 0.10788E+01_JPRB, 0.95590E+00_JPRB, 0.93603E+00_JPRB, 0.93194E+00_JPRB, 0.10239E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.22182E+01_JPRB, 0.28357E+01_JPRB, 0.31698E+01_JPRB, 0.33858E+01_JPRB, 0.32659E+01_JPRB, 0.35649E+01_JPRB ,&
&  0.32872E+01_JPRB, 0.30110E+01_JPRB, 0.27612E+01_JPRB, 0.25605E+01_JPRB, 0.23494E+01_JPRB, 0.22002E+01_JPRB, 0.20916E+01_JPRB ,&
&  0.19211E+01_JPRB, 0.17755E+01_JPRB, 0.17002E+01_JPRB, 0.16443E+01_JPRB, 0.15984E+01_JPRB, 0.15686E+01_JPRB, 0.15433E+01_JPRB ,&
&  0.15294E+01_JPRB, 0.15327E+01_JPRB, 0.15591E+01_JPRB, 0.16355E+01_JPRB, 0.18602E+01_JPRB, 0.18548E+01_JPRB, 0.17719E+01_JPRB ,&
&  0.17367E+01_JPRB, 0.18006E+01_JPRB, 0.21139E+01_JPRB, 0.22028E+01_JPRB, 0.23496E+01_JPRB, 0.26784E+01_JPRB, 0.32003E+01_JPRB ,&
&  0.37071E+01_JPRB, 0.40769E+01_JPRB, 0.36160E+01_JPRB, 0.37090E+01_JPRB, 0.33417E+01_JPRB, 0.26794E+01_JPRB, 0.31021E+01_JPRB ,&
&  0.34337E+01_JPRB, 0.30056E+01_JPRB, 0.23991E+01_JPRB, 0.18654E+01_JPRB, 0.18003E+01_JPRB, 0.13736E+01_JPRB, 0.88178E+00_JPRB ,&
&  0.81756E+00_JPRB, 0.77763E+00_JPRB, 0.72150E+00_JPRB, 0.83576E+00_JPRB, 0.87179E+00_JPRB, 0.13637E+01_JPRB, 0.23691E+01_JPRB ,&
&  0.33953E+01_JPRB, 0.37278E+01_JPRB, 0.44084E+01_JPRB, 0.46454E+01_JPRB, 0.45566E+01_JPRB, 0.47241E+01_JPRB, 0.49056E+01_JPRB ,&
&  0.40735E+01_JPRB, 0.37106E+01_JPRB, 0.35401E+01_JPRB, 0.30851E+01_JPRB, 0.28406E+01_JPRB, 0.26673E+01_JPRB, 0.24622E+01_JPRB ,&
&  0.22211E+01_JPRB, 0.20215E+01_JPRB, 0.19663E+01_JPRB, 0.20127E+01_JPRB, 0.20575E+01_JPRB /)
ZBURD(:, 24,  4) = (/ &
&  0.19272E+01_JPRB, 0.18150E+01_JPRB, 0.16384E+01_JPRB, 0.14445E+01_JPRB, 0.13316E+01_JPRB, 0.12478E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.11607E+01_JPRB, 0.11381E+01_JPRB, 0.11242E+01_JPRB, 0.11130E+01_JPRB, 0.11079E+01_JPRB, 0.11095E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.10773E+01_JPRB, 0.10420E+01_JPRB, 0.11418E+01_JPRB, 0.11113E+01_JPRB, 0.11233E+01_JPRB, 0.10554E+01_JPRB, 0.11910E+01_JPRB ,&
&  0.16017E+01_JPRB, 0.23755E+01_JPRB, 0.29575E+01_JPRB, 0.30462E+01_JPRB, 0.31020E+01_JPRB, 0.30405E+01_JPRB, 0.27459E+01_JPRB ,&
&  0.24523E+01_JPRB, 0.23396E+01_JPRB, 0.22355E+01_JPRB, 0.21899E+01_JPRB, 0.21501E+01_JPRB, 0.21166E+01_JPRB, 0.20156E+01_JPRB ,&
&  0.18968E+01_JPRB, 0.18115E+01_JPRB, 0.17182E+01_JPRB, 0.16460E+01_JPRB, 0.15787E+01_JPRB, 0.15233E+01_JPRB, 0.14804E+01_JPRB ,&
&  0.14775E+01_JPRB, 0.15089E+01_JPRB, 0.15720E+01_JPRB, 0.17260E+01_JPRB, 0.16469E+01_JPRB, 0.14992E+01_JPRB, 0.16406E+01_JPRB ,&
&  0.17719E+01_JPRB, 0.19463E+01_JPRB, 0.19646E+01_JPRB, 0.21845E+01_JPRB, 0.22685E+01_JPRB, 0.24223E+01_JPRB, 0.29255E+01_JPRB ,&
&  0.34121E+01_JPRB, 0.38856E+01_JPRB, 0.32276E+01_JPRB, 0.32169E+01_JPRB, 0.34690E+01_JPRB, 0.27397E+01_JPRB, 0.25950E+01_JPRB ,&
&  0.29738E+01_JPRB, 0.25813E+01_JPRB, 0.22388E+01_JPRB, 0.18771E+01_JPRB, 0.23893E+01_JPRB, 0.23502E+01_JPRB, 0.12210E+01_JPRB ,&
&  0.90907E+00_JPRB, 0.78728E+00_JPRB, 0.72218E+00_JPRB, 0.71573E+00_JPRB, 0.79937E+00_JPRB, 0.13903E+01_JPRB, 0.28742E+01_JPRB ,&
&  0.34403E+01_JPRB, 0.43988E+01_JPRB, 0.49141E+01_JPRB, 0.49563E+01_JPRB, 0.46775E+01_JPRB, 0.44439E+01_JPRB, 0.40075E+01_JPRB ,&
&  0.39365E+01_JPRB, 0.35605E+01_JPRB, 0.30785E+01_JPRB, 0.26944E+01_JPRB, 0.24081E+01_JPRB, 0.21248E+01_JPRB, 0.18970E+01_JPRB ,&
&  0.17485E+01_JPRB, 0.17105E+01_JPRB, 0.17348E+01_JPRB, 0.18326E+01_JPRB, 0.19660E+01_JPRB /)
ZBURD(:, 25,  4) = (/ &
&  0.18668E+01_JPRB, 0.18110E+01_JPRB, 0.16594E+01_JPRB, 0.14866E+01_JPRB, 0.13779E+01_JPRB, 0.12981E+01_JPRB, 0.12410E+01_JPRB ,&
&  0.11975E+01_JPRB, 0.11530E+01_JPRB, 0.11205E+01_JPRB, 0.11027E+01_JPRB, 0.11054E+01_JPRB, 0.11030E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.10720E+01_JPRB, 0.10354E+01_JPRB, 0.10832E+01_JPRB, 0.11342E+01_JPRB, 0.11223E+01_JPRB, 0.10449E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.18089E+01_JPRB, 0.26244E+01_JPRB, 0.29632E+01_JPRB, 0.29915E+01_JPRB, 0.27296E+01_JPRB, 0.25959E+01_JPRB, 0.20172E+01_JPRB ,&
&  0.18000E+01_JPRB, 0.18592E+01_JPRB, 0.19060E+01_JPRB, 0.20143E+01_JPRB, 0.20964E+01_JPRB, 0.20801E+01_JPRB, 0.19887E+01_JPRB ,&
&  0.19010E+01_JPRB, 0.17733E+01_JPRB, 0.16695E+01_JPRB, 0.15803E+01_JPRB, 0.15081E+01_JPRB, 0.14477E+01_JPRB, 0.14414E+01_JPRB ,&
&  0.14874E+01_JPRB, 0.15404E+01_JPRB, 0.16525E+01_JPRB, 0.17768E+01_JPRB, 0.14727E+01_JPRB, 0.16013E+01_JPRB, 0.16592E+01_JPRB ,&
&  0.17114E+01_JPRB, 0.17850E+01_JPRB, 0.17606E+01_JPRB, 0.19121E+01_JPRB, 0.21358E+01_JPRB, 0.23406E+01_JPRB, 0.26759E+01_JPRB ,&
&  0.31990E+01_JPRB, 0.30687E+01_JPRB, 0.27929E+01_JPRB, 0.28276E+01_JPRB, 0.30364E+01_JPRB, 0.34356E+01_JPRB, 0.23254E+01_JPRB ,&
&  0.21051E+01_JPRB, 0.24372E+01_JPRB, 0.22157E+01_JPRB, 0.20361E+01_JPRB, 0.25222E+01_JPRB, 0.30400E+01_JPRB, 0.27449E+01_JPRB ,&
&  0.12206E+01_JPRB, 0.85413E+00_JPRB, 0.86121E+00_JPRB, 0.86498E+00_JPRB, 0.86779E+00_JPRB, 0.13778E+01_JPRB, 0.48850E+01_JPRB ,&
&  0.41823E+01_JPRB, 0.50396E+01_JPRB, 0.51258E+01_JPRB, 0.46876E+01_JPRB, 0.44495E+01_JPRB, 0.40753E+01_JPRB, 0.37314E+01_JPRB ,&
&  0.32518E+01_JPRB, 0.28047E+01_JPRB, 0.23896E+01_JPRB, 0.19976E+01_JPRB, 0.16998E+01_JPRB, 0.15133E+01_JPRB, 0.14449E+01_JPRB ,&
&  0.14475E+01_JPRB, 0.14953E+01_JPRB, 0.15735E+01_JPRB, 0.16894E+01_JPRB, 0.18321E+01_JPRB /)
ZBURD(:, 26,  4) = (/ &
&  0.18149E+01_JPRB, 0.17986E+01_JPRB, 0.16569E+01_JPRB, 0.15039E+01_JPRB, 0.13967E+01_JPRB, 0.13294E+01_JPRB, 0.12519E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.11685E+01_JPRB, 0.11032E+01_JPRB, 0.10659E+01_JPRB, 0.10652E+01_JPRB, 0.10579E+01_JPRB, 0.10569E+01_JPRB ,&
&  0.10422E+01_JPRB, 0.10276E+01_JPRB, 0.10564E+01_JPRB, 0.10692E+01_JPRB, 0.11135E+01_JPRB, 0.98520E+00_JPRB, 0.11147E+01_JPRB ,&
&  0.19191E+01_JPRB, 0.28421E+01_JPRB, 0.26089E+01_JPRB, 0.25935E+01_JPRB, 0.24830E+01_JPRB, 0.24818E+01_JPRB, 0.17634E+01_JPRB ,&
&  0.16423E+01_JPRB, 0.17467E+01_JPRB, 0.18324E+01_JPRB, 0.19597E+01_JPRB, 0.20007E+01_JPRB, 0.19746E+01_JPRB, 0.19204E+01_JPRB ,&
&  0.18096E+01_JPRB, 0.16931E+01_JPRB, 0.16175E+01_JPRB, 0.15390E+01_JPRB, 0.14680E+01_JPRB, 0.14432E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.15305E+01_JPRB, 0.15911E+01_JPRB, 0.17310E+01_JPRB, 0.16981E+01_JPRB, 0.16333E+01_JPRB, 0.16322E+01_JPRB, 0.16874E+01_JPRB ,&
&  0.16844E+01_JPRB, 0.17292E+01_JPRB, 0.17686E+01_JPRB, 0.18097E+01_JPRB, 0.20161E+01_JPRB, 0.23154E+01_JPRB, 0.25611E+01_JPRB ,&
&  0.28113E+01_JPRB, 0.27080E+01_JPRB, 0.23900E+01_JPRB, 0.24208E+01_JPRB, 0.26981E+01_JPRB, 0.31165E+01_JPRB, 0.29100E+01_JPRB ,&
&  0.25242E+01_JPRB, 0.20118E+01_JPRB, 0.19747E+01_JPRB, 0.23574E+01_JPRB, 0.26866E+01_JPRB, 0.29478E+01_JPRB, 0.38669E+01_JPRB ,&
&  0.38109E+01_JPRB, 0.21715E+01_JPRB, 0.17758E+01_JPRB, 0.22921E+01_JPRB, 0.17109E+01_JPRB, 0.15532E+01_JPRB, 0.29895E+01_JPRB ,&
&  0.39118E+01_JPRB, 0.46739E+01_JPRB, 0.48423E+01_JPRB, 0.43446E+01_JPRB, 0.41355E+01_JPRB, 0.36854E+01_JPRB, 0.30065E+01_JPRB ,&
&  0.24337E+01_JPRB, 0.19594E+01_JPRB, 0.16427E+01_JPRB, 0.14750E+01_JPRB, 0.14054E+01_JPRB, 0.13686E+01_JPRB, 0.13540E+01_JPRB ,&
&  0.13709E+01_JPRB, 0.14218E+01_JPRB, 0.15136E+01_JPRB, 0.16506E+01_JPRB, 0.17751E+01_JPRB /)
ZBURD(:, 27,  4) = (/ &
&  0.17586E+01_JPRB, 0.17233E+01_JPRB, 0.15992E+01_JPRB, 0.14779E+01_JPRB, 0.13933E+01_JPRB, 0.13226E+01_JPRB, 0.12431E+01_JPRB ,&
&  0.13520E+01_JPRB, 0.11901E+01_JPRB, 0.10780E+01_JPRB, 0.10310E+01_JPRB, 0.10101E+01_JPRB, 0.10103E+01_JPRB, 0.10148E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10217E+01_JPRB, 0.10522E+01_JPRB, 0.10694E+01_JPRB, 0.11276E+01_JPRB, 0.13280E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.17838E+01_JPRB, 0.29943E+01_JPRB, 0.26075E+01_JPRB, 0.24224E+01_JPRB, 0.21699E+01_JPRB, 0.20982E+01_JPRB, 0.17184E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.16830E+01_JPRB, 0.17520E+01_JPRB, 0.18261E+01_JPRB, 0.18672E+01_JPRB, 0.18510E+01_JPRB, 0.17568E+01_JPRB ,&
&  0.16657E+01_JPRB, 0.15925E+01_JPRB, 0.15519E+01_JPRB, 0.15007E+01_JPRB, 0.14696E+01_JPRB, 0.14832E+01_JPRB, 0.15121E+01_JPRB ,&
&  0.15486E+01_JPRB, 0.16687E+01_JPRB, 0.17947E+01_JPRB, 0.16758E+01_JPRB, 0.16651E+01_JPRB, 0.16979E+01_JPRB, 0.17321E+01_JPRB ,&
&  0.16380E+01_JPRB, 0.15440E+01_JPRB, 0.16835E+01_JPRB, 0.18533E+01_JPRB, 0.20144E+01_JPRB, 0.21561E+01_JPRB, 0.22457E+01_JPRB ,&
&  0.24804E+01_JPRB, 0.25329E+01_JPRB, 0.25124E+01_JPRB, 0.22914E+01_JPRB, 0.24414E+01_JPRB, 0.27781E+01_JPRB, 0.28810E+01_JPRB ,&
&  0.29058E+01_JPRB, 0.25915E+01_JPRB, 0.26315E+01_JPRB, 0.27940E+01_JPRB, 0.28224E+01_JPRB, 0.27914E+01_JPRB, 0.31116E+01_JPRB ,&
&  0.35332E+01_JPRB, 0.41813E+01_JPRB, 0.37984E+01_JPRB, 0.30471E+01_JPRB, 0.25757E+01_JPRB, 0.22045E+01_JPRB, 0.28986E+01_JPRB ,&
&  0.39172E+01_JPRB, 0.40775E+01_JPRB, 0.40331E+01_JPRB, 0.46162E+01_JPRB, 0.34550E+01_JPRB, 0.26081E+01_JPRB, 0.19522E+01_JPRB ,&
&  0.15718E+01_JPRB, 0.14510E+01_JPRB, 0.14281E+01_JPRB, 0.13841E+01_JPRB, 0.13571E+01_JPRB, 0.13356E+01_JPRB, 0.13443E+01_JPRB ,&
&  0.13890E+01_JPRB, 0.14664E+01_JPRB, 0.15650E+01_JPRB, 0.16716E+01_JPRB, 0.17487E+01_JPRB /)
ZBURD(:, 28,  4) = (/ &
&  0.16483E+01_JPRB, 0.15853E+01_JPRB, 0.14981E+01_JPRB, 0.14091E+01_JPRB, 0.13277E+01_JPRB, 0.12585E+01_JPRB, 0.12319E+01_JPRB ,&
&  0.14352E+01_JPRB, 0.11391E+01_JPRB, 0.10233E+01_JPRB, 0.98921E+00_JPRB, 0.98103E+00_JPRB, 0.97578E+00_JPRB, 0.98874E+00_JPRB ,&
&  0.10080E+01_JPRB, 0.10293E+01_JPRB, 0.10727E+01_JPRB, 0.11299E+01_JPRB, 0.12705E+01_JPRB, 0.16109E+01_JPRB, 0.18222E+01_JPRB ,&
&  0.19331E+01_JPRB, 0.33583E+01_JPRB, 0.25833E+01_JPRB, 0.23021E+01_JPRB, 0.20566E+01_JPRB, 0.20108E+01_JPRB, 0.17812E+01_JPRB ,&
&  0.16436E+01_JPRB, 0.16354E+01_JPRB, 0.16725E+01_JPRB, 0.16861E+01_JPRB, 0.16917E+01_JPRB, 0.16532E+01_JPRB, 0.16082E+01_JPRB ,&
&  0.15546E+01_JPRB, 0.15133E+01_JPRB, 0.14812E+01_JPRB, 0.14562E+01_JPRB, 0.14486E+01_JPRB, 0.14864E+01_JPRB, 0.15323E+01_JPRB ,&
&  0.16224E+01_JPRB, 0.17686E+01_JPRB, 0.17534E+01_JPRB, 0.16879E+01_JPRB, 0.16546E+01_JPRB, 0.17084E+01_JPRB, 0.17429E+01_JPRB ,&
&  0.16741E+01_JPRB, 0.15985E+01_JPRB, 0.17119E+01_JPRB, 0.17788E+01_JPRB, 0.18291E+01_JPRB, 0.20256E+01_JPRB, 0.20421E+01_JPRB ,&
&  0.23168E+01_JPRB, 0.22990E+01_JPRB, 0.26817E+01_JPRB, 0.23502E+01_JPRB, 0.23051E+01_JPRB, 0.26139E+01_JPRB, 0.25618E+01_JPRB ,&
&  0.22678E+01_JPRB, 0.24232E+01_JPRB, 0.26421E+01_JPRB, 0.29321E+01_JPRB, 0.29865E+01_JPRB, 0.29700E+01_JPRB, 0.28587E+01_JPRB ,&
&  0.26406E+01_JPRB, 0.27874E+01_JPRB, 0.24635E+01_JPRB, 0.21182E+01_JPRB, 0.23265E+01_JPRB, 0.22799E+01_JPRB, 0.31046E+01_JPRB ,&
&  0.40070E+01_JPRB, 0.40426E+01_JPRB, 0.40771E+01_JPRB, 0.35230E+01_JPRB, 0.23345E+01_JPRB, 0.16997E+01_JPRB, 0.14012E+01_JPRB ,&
&  0.13539E+01_JPRB, 0.13824E+01_JPRB, 0.13883E+01_JPRB, 0.13591E+01_JPRB, 0.13515E+01_JPRB, 0.13617E+01_JPRB, 0.13969E+01_JPRB ,&
&  0.14695E+01_JPRB, 0.15564E+01_JPRB, 0.16170E+01_JPRB, 0.16555E+01_JPRB, 0.16798E+01_JPRB /)
ZBURD(:, 29,  4) = (/ &
&  0.15391E+01_JPRB, 0.14851E+01_JPRB, 0.14209E+01_JPRB, 0.13680E+01_JPRB, 0.13465E+01_JPRB, 0.12946E+01_JPRB, 0.13405E+01_JPRB ,&
&  0.15076E+01_JPRB, 0.10691E+01_JPRB, 0.96658E+00_JPRB, 0.95222E+00_JPRB, 0.95742E+00_JPRB, 0.97182E+00_JPRB, 0.10120E+01_JPRB ,&
&  0.10511E+01_JPRB, 0.10912E+01_JPRB, 0.11624E+01_JPRB, 0.13219E+01_JPRB, 0.16332E+01_JPRB, 0.21540E+01_JPRB, 0.24653E+01_JPRB ,&
&  0.26174E+01_JPRB, 0.26780E+01_JPRB, 0.26252E+01_JPRB, 0.21294E+01_JPRB, 0.19810E+01_JPRB, 0.19371E+01_JPRB, 0.19338E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.16209E+01_JPRB, 0.16028E+01_JPRB, 0.15654E+01_JPRB, 0.15209E+01_JPRB, 0.15122E+01_JPRB, 0.15072E+01_JPRB ,&
&  0.14920E+01_JPRB, 0.14662E+01_JPRB, 0.14488E+01_JPRB, 0.14489E+01_JPRB, 0.14797E+01_JPRB, 0.15596E+01_JPRB, 0.16402E+01_JPRB ,&
&  0.17427E+01_JPRB, 0.18226E+01_JPRB, 0.17762E+01_JPRB, 0.16736E+01_JPRB, 0.16541E+01_JPRB, 0.17082E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.17127E+01_JPRB, 0.17351E+01_JPRB, 0.17772E+01_JPRB, 0.17889E+01_JPRB, 0.17293E+01_JPRB, 0.19493E+01_JPRB, 0.20001E+01_JPRB ,&
&  0.21838E+01_JPRB, 0.22622E+01_JPRB, 0.27112E+01_JPRB, 0.21799E+01_JPRB, 0.23596E+01_JPRB, 0.24512E+01_JPRB, 0.24177E+01_JPRB ,&
&  0.22109E+01_JPRB, 0.24642E+01_JPRB, 0.27743E+01_JPRB, 0.29199E+01_JPRB, 0.31433E+01_JPRB, 0.29483E+01_JPRB, 0.30106E+01_JPRB ,&
&  0.23223E+01_JPRB, 0.22938E+01_JPRB, 0.18716E+01_JPRB, 0.20449E+01_JPRB, 0.21558E+01_JPRB, 0.22195E+01_JPRB, 0.26871E+01_JPRB ,&
&  0.34026E+01_JPRB, 0.32766E+01_JPRB, 0.28106E+01_JPRB, 0.21189E+01_JPRB, 0.14753E+01_JPRB, 0.12885E+01_JPRB, 0.12497E+01_JPRB ,&
&  0.12793E+01_JPRB, 0.13666E+01_JPRB, 0.13797E+01_JPRB, 0.13891E+01_JPRB, 0.13943E+01_JPRB, 0.14130E+01_JPRB, 0.14521E+01_JPRB ,&
&  0.15150E+01_JPRB, 0.15674E+01_JPRB, 0.15963E+01_JPRB, 0.15947E+01_JPRB, 0.15799E+01_JPRB /)
ZBURD(:, 30,  4) = (/ &
&  0.14136E+01_JPRB, 0.13646E+01_JPRB, 0.13245E+01_JPRB, 0.12689E+01_JPRB, 0.11705E+01_JPRB, 0.11053E+01_JPRB, 0.10641E+01_JPRB ,&
&  0.10140E+01_JPRB, 0.96516E+00_JPRB, 0.97101E+00_JPRB, 0.98273E+00_JPRB, 0.10041E+01_JPRB, 0.10352E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.11397E+01_JPRB, 0.12258E+01_JPRB, 0.14026E+01_JPRB, 0.16862E+01_JPRB, 0.20284E+01_JPRB, 0.24482E+01_JPRB, 0.29659E+01_JPRB ,&
&  0.29180E+01_JPRB, 0.19767E+01_JPRB, 0.22597E+01_JPRB, 0.21112E+01_JPRB, 0.19478E+01_JPRB, 0.18012E+01_JPRB, 0.17218E+01_JPRB ,&
&  0.16576E+01_JPRB, 0.16208E+01_JPRB, 0.16050E+01_JPRB, 0.15663E+01_JPRB, 0.14989E+01_JPRB, 0.15113E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.15130E+01_JPRB, 0.15179E+01_JPRB, 0.15343E+01_JPRB, 0.15678E+01_JPRB, 0.16358E+01_JPRB, 0.17267E+01_JPRB, 0.18027E+01_JPRB ,&
&  0.18601E+01_JPRB, 0.19121E+01_JPRB, 0.18725E+01_JPRB, 0.18061E+01_JPRB, 0.17641E+01_JPRB, 0.17800E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.17423E+01_JPRB, 0.17196E+01_JPRB, 0.17862E+01_JPRB, 0.19042E+01_JPRB, 0.19839E+01_JPRB, 0.18727E+01_JPRB, 0.20343E+01_JPRB ,&
&  0.21430E+01_JPRB, 0.22796E+01_JPRB, 0.23252E+01_JPRB, 0.27863E+01_JPRB, 0.20022E+01_JPRB, 0.21745E+01_JPRB, 0.20114E+01_JPRB ,&
&  0.23551E+01_JPRB, 0.24782E+01_JPRB, 0.26668E+01_JPRB, 0.28423E+01_JPRB, 0.31773E+01_JPRB, 0.28818E+01_JPRB, 0.26643E+01_JPRB ,&
&  0.21734E+01_JPRB, 0.18319E+01_JPRB, 0.16497E+01_JPRB, 0.18570E+01_JPRB, 0.20413E+01_JPRB, 0.23244E+01_JPRB, 0.24459E+01_JPRB ,&
&  0.26462E+01_JPRB, 0.22327E+01_JPRB, 0.17699E+01_JPRB, 0.15845E+01_JPRB, 0.13335E+01_JPRB, 0.12400E+01_JPRB, 0.12623E+01_JPRB ,&
&  0.13055E+01_JPRB, 0.13725E+01_JPRB, 0.13829E+01_JPRB, 0.13810E+01_JPRB, 0.13844E+01_JPRB, 0.13957E+01_JPRB, 0.14229E+01_JPRB ,&
&  0.14487E+01_JPRB, 0.14784E+01_JPRB, 0.14914E+01_JPRB, 0.14778E+01_JPRB, 0.14483E+01_JPRB /)
ZBURD(:, 31,  4) = (/ &
&  0.12927E+01_JPRB, 0.12444E+01_JPRB, 0.11662E+01_JPRB, 0.10726E+01_JPRB, 0.10054E+01_JPRB, 0.96628E+00_JPRB, 0.94150E+00_JPRB ,&
&  0.95213E+00_JPRB, 0.97484E+00_JPRB, 0.10022E+01_JPRB, 0.10396E+01_JPRB, 0.10788E+01_JPRB, 0.11129E+01_JPRB, 0.11893E+01_JPRB ,&
&  0.12768E+01_JPRB, 0.14269E+01_JPRB, 0.16576E+01_JPRB, 0.19061E+01_JPRB, 0.21325E+01_JPRB, 0.23795E+01_JPRB, 0.26162E+01_JPRB ,&
&  0.28814E+01_JPRB, 0.29047E+01_JPRB, 0.25787E+01_JPRB, 0.17648E+01_JPRB, 0.15564E+01_JPRB, 0.16719E+01_JPRB, 0.16458E+01_JPRB ,&
&  0.16318E+01_JPRB, 0.16057E+01_JPRB, 0.15870E+01_JPRB, 0.15803E+01_JPRB, 0.15835E+01_JPRB, 0.16005E+01_JPRB, 0.16032E+01_JPRB ,&
&  0.16188E+01_JPRB, 0.16496E+01_JPRB, 0.16863E+01_JPRB, 0.17312E+01_JPRB, 0.17772E+01_JPRB, 0.18420E+01_JPRB, 0.19211E+01_JPRB ,&
&  0.19753E+01_JPRB, 0.20458E+01_JPRB, 0.20607E+01_JPRB, 0.20414E+01_JPRB, 0.19730E+01_JPRB, 0.19532E+01_JPRB, 0.19233E+01_JPRB ,&
&  0.19117E+01_JPRB, 0.18840E+01_JPRB, 0.18929E+01_JPRB, 0.19909E+01_JPRB, 0.20431E+01_JPRB, 0.18531E+01_JPRB, 0.18920E+01_JPRB ,&
&  0.21383E+01_JPRB, 0.23443E+01_JPRB, 0.20986E+01_JPRB, 0.26190E+01_JPRB, 0.16238E+01_JPRB, 0.19693E+01_JPRB, 0.21996E+01_JPRB ,&
&  0.21313E+01_JPRB, 0.22803E+01_JPRB, 0.24845E+01_JPRB, 0.26607E+01_JPRB, 0.27161E+01_JPRB, 0.24810E+01_JPRB, 0.23112E+01_JPRB ,&
&  0.19891E+01_JPRB, 0.16363E+01_JPRB, 0.14852E+01_JPRB, 0.15756E+01_JPRB, 0.18051E+01_JPRB, 0.20361E+01_JPRB, 0.17871E+01_JPRB ,&
&  0.14589E+01_JPRB, 0.15983E+01_JPRB, 0.15218E+01_JPRB, 0.16534E+01_JPRB, 0.14710E+01_JPRB, 0.12247E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.13406E+01_JPRB, 0.13477E+01_JPRB, 0.13598E+01_JPRB, 0.13458E+01_JPRB, 0.13498E+01_JPRB, 0.13514E+01_JPRB, 0.13542E+01_JPRB ,&
&  0.13664E+01_JPRB, 0.13903E+01_JPRB, 0.13977E+01_JPRB, 0.13677E+01_JPRB, 0.13271E+01_JPRB /)
ZBURD(:, 32,  4) = (/ &
&  0.11519E+01_JPRB, 0.10665E+01_JPRB, 0.97580E+00_JPRB, 0.91182E+00_JPRB, 0.88463E+00_JPRB, 0.87963E+00_JPRB, 0.89342E+00_JPRB ,&
&  0.92357E+00_JPRB, 0.96008E+00_JPRB, 0.10028E+01_JPRB, 0.10578E+01_JPRB, 0.11116E+01_JPRB, 0.11687E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.13450E+01_JPRB, 0.14756E+01_JPRB, 0.16571E+01_JPRB, 0.18598E+01_JPRB, 0.20330E+01_JPRB, 0.22107E+01_JPRB, 0.23796E+01_JPRB ,&
&  0.25565E+01_JPRB, 0.26136E+01_JPRB, 0.25169E+01_JPRB, 0.23391E+01_JPRB, 0.18575E+01_JPRB, 0.18953E+01_JPRB, 0.20139E+01_JPRB ,&
&  0.20632E+01_JPRB, 0.17673E+01_JPRB, 0.16781E+01_JPRB, 0.16584E+01_JPRB, 0.16658E+01_JPRB, 0.16871E+01_JPRB, 0.17022E+01_JPRB ,&
&  0.17322E+01_JPRB, 0.17633E+01_JPRB, 0.17888E+01_JPRB, 0.18110E+01_JPRB, 0.18421E+01_JPRB, 0.19128E+01_JPRB, 0.20455E+01_JPRB ,&
&  0.20936E+01_JPRB, 0.21332E+01_JPRB, 0.21480E+01_JPRB, 0.21076E+01_JPRB, 0.21744E+01_JPRB, 0.22512E+01_JPRB, 0.22517E+01_JPRB ,&
&  0.22359E+01_JPRB, 0.21147E+01_JPRB, 0.21608E+01_JPRB, 0.22625E+01_JPRB, 0.22199E+01_JPRB, 0.20881E+01_JPRB, 0.20967E+01_JPRB ,&
&  0.22974E+01_JPRB, 0.25041E+01_JPRB, 0.18134E+01_JPRB, 0.21059E+01_JPRB, 0.24729E+01_JPRB, 0.21039E+01_JPRB, 0.18484E+01_JPRB ,&
&  0.17001E+01_JPRB, 0.17949E+01_JPRB, 0.20939E+01_JPRB, 0.22234E+01_JPRB, 0.22164E+01_JPRB, 0.25337E+01_JPRB, 0.20492E+01_JPRB ,&
&  0.17602E+01_JPRB, 0.14295E+01_JPRB, 0.13038E+01_JPRB, 0.13175E+01_JPRB, 0.14364E+01_JPRB, 0.14871E+01_JPRB, 0.11944E+01_JPRB ,&
&  0.10770E+01_JPRB, 0.14069E+01_JPRB, 0.15062E+01_JPRB, 0.17740E+01_JPRB, 0.17629E+01_JPRB, 0.11849E+01_JPRB, 0.12307E+01_JPRB ,&
&  0.12695E+01_JPRB, 0.12933E+01_JPRB, 0.12954E+01_JPRB, 0.12904E+01_JPRB, 0.12858E+01_JPRB, 0.12811E+01_JPRB, 0.12840E+01_JPRB ,&
&  0.12906E+01_JPRB, 0.12982E+01_JPRB, 0.12959E+01_JPRB, 0.12631E+01_JPRB, 0.12133E+01_JPRB /)
ZBURD(:, 33,  4) = (/ &
&  0.94933E+00_JPRB, 0.87703E+00_JPRB, 0.82709E+00_JPRB, 0.81453E+00_JPRB, 0.82797E+00_JPRB, 0.84370E+00_JPRB, 0.85298E+00_JPRB ,&
&  0.87546E+00_JPRB, 0.90657E+00_JPRB, 0.95226E+00_JPRB, 0.99304E+00_JPRB, 0.10418E+01_JPRB, 0.10865E+01_JPRB, 0.11570E+01_JPRB ,&
&  0.12297E+01_JPRB, 0.13124E+01_JPRB, 0.14511E+01_JPRB, 0.16557E+01_JPRB, 0.18158E+01_JPRB, 0.19572E+01_JPRB, 0.21639E+01_JPRB ,&
&  0.23701E+01_JPRB, 0.24834E+01_JPRB, 0.26116E+01_JPRB, 0.26579E+01_JPRB, 0.25395E+01_JPRB, 0.23885E+01_JPRB, 0.24405E+01_JPRB ,&
&  0.20249E+01_JPRB, 0.16554E+01_JPRB, 0.16558E+01_JPRB, 0.16866E+01_JPRB, 0.17303E+01_JPRB, 0.17401E+01_JPRB, 0.17562E+01_JPRB ,&
&  0.17815E+01_JPRB, 0.17946E+01_JPRB, 0.18051E+01_JPRB, 0.18269E+01_JPRB, 0.18774E+01_JPRB, 0.19723E+01_JPRB, 0.21046E+01_JPRB ,&
&  0.21404E+01_JPRB, 0.21878E+01_JPRB, 0.22701E+01_JPRB, 0.20838E+01_JPRB, 0.21877E+01_JPRB, 0.23637E+01_JPRB, 0.24833E+01_JPRB ,&
&  0.24859E+01_JPRB, 0.23634E+01_JPRB, 0.24303E+01_JPRB, 0.24461E+01_JPRB, 0.24373E+01_JPRB, 0.24007E+01_JPRB, 0.24007E+01_JPRB ,&
&  0.24870E+01_JPRB, 0.23113E+01_JPRB, 0.17071E+01_JPRB, 0.16837E+01_JPRB, 0.17933E+01_JPRB, 0.14889E+01_JPRB, 0.15104E+01_JPRB ,&
&  0.13464E+01_JPRB, 0.14093E+01_JPRB, 0.16781E+01_JPRB, 0.18889E+01_JPRB, 0.20188E+01_JPRB, 0.23172E+01_JPRB, 0.19366E+01_JPRB ,&
&  0.14539E+01_JPRB, 0.11679E+01_JPRB, 0.10154E+01_JPRB, 0.96700E+00_JPRB, 0.10232E+01_JPRB, 0.10697E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.10740E+01_JPRB, 0.11162E+01_JPRB, 0.11443E+01_JPRB, 0.10791E+01_JPRB, 0.11054E+01_JPRB, 0.11328E+01_JPRB ,&
&  0.11411E+01_JPRB, 0.11628E+01_JPRB, 0.11718E+01_JPRB, 0.11663E+01_JPRB, 0.11670E+01_JPRB, 0.11625E+01_JPRB, 0.11661E+01_JPRB ,&
&  0.11705E+01_JPRB, 0.11667E+01_JPRB, 0.11438E+01_JPRB, 0.11066E+01_JPRB, 0.10356E+01_JPRB /)
ZBURD(:, 34,  4) = (/ &
&  0.78795E+00_JPRB, 0.74364E+00_JPRB, 0.73664E+00_JPRB, 0.74995E+00_JPRB, 0.76155E+00_JPRB, 0.77401E+00_JPRB, 0.78778E+00_JPRB ,&
&  0.79509E+00_JPRB, 0.79994E+00_JPRB, 0.80905E+00_JPRB, 0.81384E+00_JPRB, 0.85123E+00_JPRB, 0.89406E+00_JPRB, 0.94934E+00_JPRB ,&
&  0.10332E+01_JPRB, 0.11086E+01_JPRB, 0.11748E+01_JPRB, 0.12904E+01_JPRB, 0.13688E+01_JPRB, 0.14452E+01_JPRB, 0.15722E+01_JPRB ,&
&  0.17367E+01_JPRB, 0.19206E+01_JPRB, 0.20986E+01_JPRB, 0.22823E+01_JPRB, 0.25195E+01_JPRB, 0.27420E+01_JPRB, 0.27702E+01_JPRB ,&
&  0.20887E+01_JPRB, 0.15636E+01_JPRB, 0.16865E+01_JPRB, 0.15980E+01_JPRB, 0.16539E+01_JPRB, 0.16942E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.17259E+01_JPRB, 0.17353E+01_JPRB, 0.17441E+01_JPRB, 0.17572E+01_JPRB, 0.18152E+01_JPRB, 0.18952E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.20441E+01_JPRB, 0.20784E+01_JPRB, 0.20829E+01_JPRB, 0.20586E+01_JPRB, 0.21298E+01_JPRB, 0.23225E+01_JPRB, 0.24643E+01_JPRB ,&
&  0.25330E+01_JPRB, 0.25758E+01_JPRB, 0.23604E+01_JPRB, 0.23525E+01_JPRB, 0.23521E+01_JPRB, 0.21991E+01_JPRB, 0.22764E+01_JPRB ,&
&  0.23520E+01_JPRB, 0.20749E+01_JPRB, 0.14004E+01_JPRB, 0.14269E+01_JPRB, 0.14705E+01_JPRB, 0.13868E+01_JPRB, 0.13246E+01_JPRB ,&
&  0.11114E+01_JPRB, 0.11226E+01_JPRB, 0.12620E+01_JPRB, 0.15702E+01_JPRB, 0.17457E+01_JPRB, 0.18407E+01_JPRB, 0.16430E+01_JPRB ,&
&  0.12130E+01_JPRB, 0.99240E+00_JPRB, 0.86790E+00_JPRB, 0.86967E+00_JPRB, 0.94548E+00_JPRB, 0.10666E+01_JPRB, 0.95044E+00_JPRB ,&
&  0.99799E+00_JPRB, 0.11216E+01_JPRB, 0.13694E+01_JPRB, 0.11925E+01_JPRB, 0.95084E+00_JPRB, 0.99578E+00_JPRB, 0.99735E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10122E+01_JPRB, 0.10149E+01_JPRB, 0.10146E+01_JPRB, 0.10144E+01_JPRB, 0.10240E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.10092E+01_JPRB, 0.98753E+00_JPRB, 0.96590E+00_JPRB, 0.90487E+00_JPRB, 0.84257E+00_JPRB /)
ZBURD(:, 35,  4) = (/ &
&  0.68088E+00_JPRB, 0.67587E+00_JPRB, 0.68078E+00_JPRB, 0.69676E+00_JPRB, 0.69562E+00_JPRB, 0.69514E+00_JPRB, 0.72434E+00_JPRB ,&
&  0.70994E+00_JPRB, 0.69822E+00_JPRB, 0.69866E+00_JPRB, 0.67380E+00_JPRB, 0.69460E+00_JPRB, 0.71546E+00_JPRB, 0.75731E+00_JPRB ,&
&  0.80075E+00_JPRB, 0.82771E+00_JPRB, 0.85011E+00_JPRB, 0.88284E+00_JPRB, 0.94691E+00_JPRB, 0.99556E+00_JPRB, 0.10355E+01_JPRB ,&
&  0.11099E+01_JPRB, 0.12263E+01_JPRB, 0.13543E+01_JPRB, 0.16489E+01_JPRB, 0.23281E+01_JPRB, 0.28693E+01_JPRB, 0.28587E+01_JPRB ,&
&  0.21356E+01_JPRB, 0.15331E+01_JPRB, 0.14525E+01_JPRB, 0.13239E+01_JPRB, 0.13782E+01_JPRB, 0.15117E+01_JPRB, 0.15409E+01_JPRB ,&
&  0.15199E+01_JPRB, 0.15207E+01_JPRB, 0.15160E+01_JPRB, 0.15158E+01_JPRB, 0.15268E+01_JPRB, 0.15317E+01_JPRB, 0.15087E+01_JPRB ,&
&  0.14492E+01_JPRB, 0.13733E+01_JPRB, 0.13970E+01_JPRB, 0.16425E+01_JPRB, 0.18404E+01_JPRB, 0.21847E+01_JPRB, 0.21353E+01_JPRB ,&
&  0.20365E+01_JPRB, 0.19248E+01_JPRB, 0.19011E+01_JPRB, 0.18762E+01_JPRB, 0.18318E+01_JPRB, 0.17923E+01_JPRB, 0.18357E+01_JPRB ,&
&  0.18745E+01_JPRB, 0.16918E+01_JPRB, 0.11861E+01_JPRB, 0.12983E+01_JPRB, 0.13300E+01_JPRB, 0.13567E+01_JPRB, 0.11688E+01_JPRB ,&
&  0.96675E+00_JPRB, 0.92599E+00_JPRB, 0.97077E+00_JPRB, 0.10938E+01_JPRB, 0.13093E+01_JPRB, 0.14170E+01_JPRB, 0.12941E+01_JPRB ,&
&  0.10680E+01_JPRB, 0.89706E+00_JPRB, 0.79987E+00_JPRB, 0.78371E+00_JPRB, 0.93320E+00_JPRB, 0.10231E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.88193E+00_JPRB, 0.98245E+00_JPRB, 0.89896E+00_JPRB, 0.11441E+01_JPRB, 0.85596E+00_JPRB, 0.81095E+00_JPRB, 0.80385E+00_JPRB ,&
&  0.80856E+00_JPRB, 0.79713E+00_JPRB, 0.78495E+00_JPRB, 0.76453E+00_JPRB, 0.74363E+00_JPRB, 0.78760E+00_JPRB, 0.83426E+00_JPRB ,&
&  0.79278E+00_JPRB, 0.81793E+00_JPRB, 0.78644E+00_JPRB, 0.73600E+00_JPRB, 0.71489E+00_JPRB /)
ZBURD(:, 36,  4) = (/ &
&  0.64645E+00_JPRB, 0.65540E+00_JPRB, 0.65625E+00_JPRB, 0.66367E+00_JPRB, 0.67982E+00_JPRB, 0.69167E+00_JPRB, 0.70827E+00_JPRB ,&
&  0.72601E+00_JPRB, 0.73399E+00_JPRB, 0.74176E+00_JPRB, 0.75237E+00_JPRB, 0.75981E+00_JPRB, 0.78182E+00_JPRB, 0.80310E+00_JPRB ,&
&  0.83105E+00_JPRB, 0.85307E+00_JPRB, 0.87650E+00_JPRB, 0.91284E+00_JPRB, 0.95881E+00_JPRB, 0.10286E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.12000E+01_JPRB, 0.13155E+01_JPRB, 0.14533E+01_JPRB, 0.15658E+01_JPRB, 0.16239E+01_JPRB, 0.16552E+01_JPRB, 0.18334E+01_JPRB ,&
&  0.15750E+01_JPRB, 0.15993E+01_JPRB, 0.15100E+01_JPRB, 0.13498E+01_JPRB, 0.13339E+01_JPRB, 0.12419E+01_JPRB, 0.12021E+01_JPRB ,&
&  0.11933E+01_JPRB, 0.11365E+01_JPRB, 0.11043E+01_JPRB, 0.10754E+01_JPRB, 0.10448E+01_JPRB, 0.98782E+00_JPRB, 0.92291E+00_JPRB ,&
&  0.89899E+00_JPRB, 0.93156E+00_JPRB, 0.10326E+01_JPRB, 0.12580E+01_JPRB, 0.14766E+01_JPRB, 0.15430E+01_JPRB, 0.14382E+01_JPRB ,&
&  0.13664E+01_JPRB, 0.13395E+01_JPRB, 0.12683E+01_JPRB, 0.12192E+01_JPRB, 0.12735E+01_JPRB, 0.13218E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.12371E+01_JPRB, 0.11178E+01_JPRB, 0.10924E+01_JPRB, 0.11564E+01_JPRB, 0.11868E+01_JPRB, 0.11773E+01_JPRB, 0.10754E+01_JPRB ,&
&  0.92107E+00_JPRB, 0.82871E+00_JPRB, 0.79347E+00_JPRB, 0.83001E+00_JPRB, 0.96120E+00_JPRB, 0.98691E+00_JPRB, 0.93080E+00_JPRB ,&
&  0.80725E+00_JPRB, 0.73628E+00_JPRB, 0.70999E+00_JPRB, 0.72083E+00_JPRB, 0.80824E+00_JPRB, 0.15529E+01_JPRB, 0.21101E+01_JPRB ,&
&  0.87157E+00_JPRB, 0.73448E+00_JPRB, 0.55998E+00_JPRB, 0.77797E+00_JPRB, 0.67875E+00_JPRB, 0.67257E+00_JPRB, 0.67306E+00_JPRB ,&
&  0.66608E+00_JPRB, 0.66509E+00_JPRB, 0.66365E+00_JPRB, 0.64871E+00_JPRB, 0.63881E+00_JPRB, 0.64263E+00_JPRB, 0.67731E+00_JPRB ,&
&  0.67460E+00_JPRB, 0.67728E+00_JPRB, 0.66269E+00_JPRB, 0.66500E+00_JPRB, 0.65452E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART7',1,ZHOOK_HANDLE)
END SUBROUTINE PART7



SUBROUTINE PART8

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART8',0,ZHOOK_HANDLE)


ZBURD(:, 37,  4) = (/ &
&  0.63692E+00_JPRB, 0.65021E+00_JPRB, 0.66793E+00_JPRB, 0.69040E+00_JPRB, 0.70607E+00_JPRB, 0.72697E+00_JPRB, 0.74410E+00_JPRB ,&
&  0.76681E+00_JPRB, 0.78477E+00_JPRB, 0.80001E+00_JPRB, 0.81635E+00_JPRB, 0.83726E+00_JPRB, 0.86269E+00_JPRB, 0.89520E+00_JPRB ,&
&  0.93079E+00_JPRB, 0.96640E+00_JPRB, 0.10053E+01_JPRB, 0.10496E+01_JPRB, 0.11010E+01_JPRB, 0.11637E+01_JPRB, 0.12231E+01_JPRB ,&
&  0.12786E+01_JPRB, 0.13353E+01_JPRB, 0.13933E+01_JPRB, 0.14390E+01_JPRB, 0.14547E+01_JPRB, 0.14856E+01_JPRB, 0.13311E+01_JPRB ,&
&  0.15618E+01_JPRB, 0.14952E+01_JPRB, 0.13922E+01_JPRB, 0.12701E+01_JPRB, 0.11654E+01_JPRB, 0.10688E+01_JPRB, 0.98067E+00_JPRB ,&
&  0.97854E+00_JPRB, 0.92098E+00_JPRB, 0.89593E+00_JPRB, 0.88148E+00_JPRB, 0.81753E+00_JPRB, 0.81327E+00_JPRB, 0.82838E+00_JPRB ,&
&  0.86912E+00_JPRB, 0.95682E+00_JPRB, 0.10094E+01_JPRB, 0.10728E+01_JPRB, 0.11603E+01_JPRB, 0.11062E+01_JPRB, 0.10578E+01_JPRB ,&
&  0.97736E+00_JPRB, 0.91141E+00_JPRB, 0.74651E+00_JPRB, 0.81550E+00_JPRB, 0.86306E+00_JPRB, 0.89656E+00_JPRB, 0.92655E+00_JPRB ,&
&  0.84893E+00_JPRB, 0.75903E+00_JPRB, 0.82775E+00_JPRB, 0.10700E+01_JPRB, 0.10669E+01_JPRB, 0.10143E+01_JPRB, 0.96255E+00_JPRB ,&
&  0.88543E+00_JPRB, 0.82111E+00_JPRB, 0.76200E+00_JPRB, 0.73854E+00_JPRB, 0.77586E+00_JPRB, 0.77514E+00_JPRB, 0.73158E+00_JPRB ,&
&  0.67288E+00_JPRB, 0.65383E+00_JPRB, 0.66017E+00_JPRB, 0.68309E+00_JPRB, 0.80316E+00_JPRB, 0.93164E+00_JPRB, 0.14316E+01_JPRB ,&
&  0.88134E+00_JPRB, 0.52684E+00_JPRB, 0.59357E+00_JPRB, 0.65821E+00_JPRB, 0.61515E+00_JPRB, 0.62450E+00_JPRB, 0.65574E+00_JPRB ,&
&  0.68700E+00_JPRB, 0.69852E+00_JPRB, 0.65612E+00_JPRB, 0.65458E+00_JPRB, 0.64848E+00_JPRB, 0.64287E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.63066E+00_JPRB, 0.62950E+00_JPRB, 0.63852E+00_JPRB, 0.63391E+00_JPRB, 0.63213E+00_JPRB /)
ZBURD(:, 38,  4) = (/ &
&  0.55887E+00_JPRB, 0.56810E+00_JPRB, 0.58056E+00_JPRB, 0.59160E+00_JPRB, 0.59892E+00_JPRB, 0.61176E+00_JPRB, 0.62114E+00_JPRB ,&
&  0.63190E+00_JPRB, 0.63646E+00_JPRB, 0.64480E+00_JPRB, 0.65220E+00_JPRB, 0.66218E+00_JPRB, 0.67695E+00_JPRB, 0.69195E+00_JPRB ,&
&  0.70362E+00_JPRB, 0.72245E+00_JPRB, 0.74423E+00_JPRB, 0.75900E+00_JPRB, 0.79421E+00_JPRB, 0.86052E+00_JPRB, 0.93594E+00_JPRB ,&
&  0.99570E+00_JPRB, 0.10735E+01_JPRB, 0.11458E+01_JPRB, 0.11781E+01_JPRB, 0.11773E+01_JPRB, 0.12180E+01_JPRB, 0.10461E+01_JPRB ,&
&  0.13636E+01_JPRB, 0.13321E+01_JPRB, 0.12070E+01_JPRB, 0.11313E+01_JPRB, 0.10623E+01_JPRB, 0.99467E+00_JPRB, 0.94946E+00_JPRB ,&
&  0.94763E+00_JPRB, 0.10810E+01_JPRB, 0.10581E+01_JPRB, 0.10803E+01_JPRB, 0.96205E+00_JPRB, 0.94296E+00_JPRB, 0.10178E+01_JPRB ,&
&  0.11123E+01_JPRB, 0.11278E+01_JPRB, 0.10981E+01_JPRB, 0.11354E+01_JPRB, 0.11798E+01_JPRB, 0.11864E+01_JPRB, 0.11846E+01_JPRB ,&
&  0.11352E+01_JPRB, 0.10876E+01_JPRB, 0.10010E+01_JPRB, 0.11257E+01_JPRB, 0.10570E+01_JPRB, 0.97669E+00_JPRB, 0.91456E+00_JPRB ,&
&  0.75006E+00_JPRB, 0.74419E+00_JPRB, 0.80819E+00_JPRB, 0.92225E+00_JPRB, 0.91437E+00_JPRB, 0.91322E+00_JPRB, 0.88413E+00_JPRB ,&
&  0.84638E+00_JPRB, 0.82095E+00_JPRB, 0.77916E+00_JPRB, 0.76476E+00_JPRB, 0.76551E+00_JPRB, 0.73853E+00_JPRB, 0.68474E+00_JPRB ,&
&  0.65075E+00_JPRB, 0.65593E+00_JPRB, 0.65379E+00_JPRB, 0.66435E+00_JPRB, 0.72142E+00_JPRB, 0.71644E+00_JPRB, 0.10195E+01_JPRB ,&
&  0.78971E+00_JPRB, 0.67662E+00_JPRB, 0.87619E+00_JPRB, 0.45616E+00_JPRB, 0.55777E+00_JPRB, 0.66902E+00_JPRB, 0.86002E+00_JPRB ,&
&  0.90988E+00_JPRB, 0.11704E+01_JPRB, 0.15140E+01_JPRB, 0.12027E+01_JPRB, 0.91989E+00_JPRB, 0.62912E+00_JPRB, 0.59508E+00_JPRB ,&
&  0.58464E+00_JPRB, 0.57339E+00_JPRB, 0.57456E+00_JPRB, 0.56537E+00_JPRB, 0.56179E+00_JPRB /)
ZBURD(:, 39,  4) = (/ &
&  0.48505E+00_JPRB, 0.48753E+00_JPRB, 0.49623E+00_JPRB, 0.50218E+00_JPRB, 0.50848E+00_JPRB, 0.51338E+00_JPRB, 0.51797E+00_JPRB ,&
&  0.53235E+00_JPRB, 0.54091E+00_JPRB, 0.56282E+00_JPRB, 0.58008E+00_JPRB, 0.59585E+00_JPRB, 0.62904E+00_JPRB, 0.65687E+00_JPRB ,&
&  0.70124E+00_JPRB, 0.75894E+00_JPRB, 0.78290E+00_JPRB, 0.81633E+00_JPRB, 0.83554E+00_JPRB, 0.85644E+00_JPRB, 0.87133E+00_JPRB ,&
&  0.94258E+00_JPRB, 0.10800E+01_JPRB, 0.11724E+01_JPRB, 0.12698E+01_JPRB, 0.12710E+01_JPRB, 0.13369E+01_JPRB, 0.10328E+01_JPRB ,&
&  0.12933E+01_JPRB, 0.13000E+01_JPRB, 0.12217E+01_JPRB, 0.11780E+01_JPRB, 0.11499E+01_JPRB, 0.11260E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11793E+01_JPRB, 0.11505E+01_JPRB, 0.11892E+01_JPRB, 0.12317E+01_JPRB, 0.11967E+01_JPRB, 0.13002E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.14250E+01_JPRB, 0.14991E+01_JPRB, 0.15989E+01_JPRB, 0.16685E+01_JPRB, 0.17101E+01_JPRB, 0.17164E+01_JPRB ,&
&  0.17090E+01_JPRB, 0.17424E+01_JPRB, 0.18548E+01_JPRB, 0.18265E+01_JPRB, 0.15387E+01_JPRB, 0.13339E+01_JPRB, 0.11140E+01_JPRB ,&
&  0.85396E+00_JPRB, 0.75212E+00_JPRB, 0.76394E+00_JPRB, 0.83523E+00_JPRB, 0.81737E+00_JPRB, 0.80318E+00_JPRB, 0.78737E+00_JPRB ,&
&  0.77638E+00_JPRB, 0.78820E+00_JPRB, 0.79349E+00_JPRB, 0.79185E+00_JPRB, 0.76919E+00_JPRB, 0.71766E+00_JPRB, 0.66551E+00_JPRB ,&
&  0.65987E+00_JPRB, 0.65763E+00_JPRB, 0.66501E+00_JPRB, 0.69339E+00_JPRB, 0.74511E+00_JPRB, 0.85480E+00_JPRB, 0.83731E+00_JPRB ,&
&  0.89836E+00_JPRB, 0.64965E+00_JPRB, 0.63186E+00_JPRB, 0.66092E+00_JPRB, 0.63125E+00_JPRB, 0.86915E+00_JPRB, 0.12705E+01_JPRB ,&
&  0.18725E+01_JPRB, 0.21291E+01_JPRB, 0.28072E+01_JPRB, 0.41107E+01_JPRB, 0.25934E+01_JPRB, 0.16652E+01_JPRB, 0.55950E+00_JPRB ,&
&  0.54853E+00_JPRB, 0.52307E+00_JPRB, 0.51520E+00_JPRB, 0.49870E+00_JPRB, 0.49200E+00_JPRB /)
ZBURD(:, 40,  4) = (/ &
&  0.39967E+00_JPRB, 0.39333E+00_JPRB, 0.40356E+00_JPRB, 0.42089E+00_JPRB, 0.43424E+00_JPRB, 0.46424E+00_JPRB, 0.50027E+00_JPRB ,&
&  0.53964E+00_JPRB, 0.58847E+00_JPRB, 0.62973E+00_JPRB, 0.65884E+00_JPRB, 0.70831E+00_JPRB, 0.77821E+00_JPRB, 0.80958E+00_JPRB ,&
&  0.87020E+00_JPRB, 0.90930E+00_JPRB, 0.92890E+00_JPRB, 0.94076E+00_JPRB, 0.96977E+00_JPRB, 0.10174E+01_JPRB, 0.10732E+01_JPRB ,&
&  0.11247E+01_JPRB, 0.11899E+01_JPRB, 0.12819E+01_JPRB, 0.13401E+01_JPRB, 0.13653E+01_JPRB, 0.13641E+01_JPRB, 0.10608E+01_JPRB ,&
&  0.12924E+01_JPRB, 0.13566E+01_JPRB, 0.13719E+01_JPRB, 0.13333E+01_JPRB, 0.12897E+01_JPRB, 0.12594E+01_JPRB, 0.12684E+01_JPRB ,&
&  0.12937E+01_JPRB, 0.12859E+01_JPRB, 0.12532E+01_JPRB, 0.12424E+01_JPRB, 0.14715E+01_JPRB, 0.14889E+01_JPRB, 0.16046E+01_JPRB ,&
&  0.16963E+01_JPRB, 0.17752E+01_JPRB, 0.18871E+01_JPRB, 0.20062E+01_JPRB, 0.21288E+01_JPRB, 0.22344E+01_JPRB, 0.23238E+01_JPRB ,&
&  0.24043E+01_JPRB, 0.24959E+01_JPRB, 0.25876E+01_JPRB, 0.22493E+01_JPRB, 0.18525E+01_JPRB, 0.15867E+01_JPRB, 0.11431E+01_JPRB ,&
&  0.84495E+00_JPRB, 0.77188E+00_JPRB, 0.80655E+00_JPRB, 0.83168E+00_JPRB, 0.77758E+00_JPRB, 0.73977E+00_JPRB, 0.71097E+00_JPRB ,&
&  0.71915E+00_JPRB, 0.74455E+00_JPRB, 0.76562E+00_JPRB, 0.76901E+00_JPRB, 0.73855E+00_JPRB, 0.69724E+00_JPRB, 0.66385E+00_JPRB ,&
&  0.68591E+00_JPRB, 0.69121E+00_JPRB, 0.69694E+00_JPRB, 0.75127E+00_JPRB, 0.81769E+00_JPRB, 0.99139E+00_JPRB, 0.12098E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.68057E+00_JPRB, 0.64346E+00_JPRB, 0.63824E+00_JPRB, 0.64883E+00_JPRB, 0.72020E+00_JPRB, 0.87816E+00_JPRB ,&
&  0.12356E+01_JPRB, 0.20113E+01_JPRB, 0.22660E+01_JPRB, 0.14972E+01_JPRB, 0.20143E+01_JPRB, 0.13174E+01_JPRB, 0.54149E+00_JPRB ,&
&  0.51405E+00_JPRB, 0.49728E+00_JPRB, 0.46903E+00_JPRB, 0.43774E+00_JPRB, 0.41557E+00_JPRB /)
ZBURD(:, 41,  4) = (/ &
&  0.40386E+00_JPRB, 0.38579E+00_JPRB, 0.39600E+00_JPRB, 0.42024E+00_JPRB, 0.44925E+00_JPRB, 0.49336E+00_JPRB, 0.54691E+00_JPRB ,&
&  0.59397E+00_JPRB, 0.64031E+00_JPRB, 0.72001E+00_JPRB, 0.73958E+00_JPRB, 0.75697E+00_JPRB, 0.80028E+00_JPRB, 0.83016E+00_JPRB ,&
&  0.86518E+00_JPRB, 0.89327E+00_JPRB, 0.92512E+00_JPRB, 0.95665E+00_JPRB, 0.10041E+01_JPRB, 0.10562E+01_JPRB, 0.10995E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.12232E+01_JPRB, 0.13162E+01_JPRB, 0.14128E+01_JPRB, 0.15012E+01_JPRB, 0.15100E+01_JPRB, 0.15310E+01_JPRB ,&
&  0.10438E+01_JPRB, 0.14035E+01_JPRB, 0.14230E+01_JPRB, 0.14051E+01_JPRB, 0.13540E+01_JPRB, 0.13113E+01_JPRB, 0.13255E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.13045E+01_JPRB, 0.13452E+01_JPRB, 0.13852E+01_JPRB, 0.16159E+01_JPRB, 0.16679E+01_JPRB, 0.17477E+01_JPRB ,&
&  0.18406E+01_JPRB, 0.19252E+01_JPRB, 0.20190E+01_JPRB, 0.21343E+01_JPRB, 0.22534E+01_JPRB, 0.23661E+01_JPRB, 0.24783E+01_JPRB ,&
&  0.25656E+01_JPRB, 0.26448E+01_JPRB, 0.27558E+01_JPRB, 0.23190E+01_JPRB, 0.19634E+01_JPRB, 0.16364E+01_JPRB, 0.11908E+01_JPRB ,&
&  0.89095E+00_JPRB, 0.77026E+00_JPRB, 0.80820E+00_JPRB, 0.85608E+00_JPRB, 0.79100E+00_JPRB, 0.74127E+00_JPRB, 0.67971E+00_JPRB ,&
&  0.67520E+00_JPRB, 0.68703E+00_JPRB, 0.69552E+00_JPRB, 0.69087E+00_JPRB, 0.69649E+00_JPRB, 0.66461E+00_JPRB, 0.65525E+00_JPRB ,&
&  0.67755E+00_JPRB, 0.67644E+00_JPRB, 0.68612E+00_JPRB, 0.71615E+00_JPRB, 0.76900E+00_JPRB, 0.81677E+00_JPRB, 0.81742E+00_JPRB ,&
&  0.81524E+00_JPRB, 0.73009E+00_JPRB, 0.69141E+00_JPRB, 0.70197E+00_JPRB, 0.68904E+00_JPRB, 0.67805E+00_JPRB, 0.69739E+00_JPRB ,&
&  0.78150E+00_JPRB, 0.88948E+00_JPRB, 0.97921E+00_JPRB, 0.96158E+00_JPRB, 0.10014E+01_JPRB, 0.79208E+00_JPRB, 0.59498E+00_JPRB ,&
&  0.55780E+00_JPRB, 0.54630E+00_JPRB, 0.50161E+00_JPRB, 0.47008E+00_JPRB, 0.43468E+00_JPRB /)
ZBURD(:, 42,  4) = (/ &
&  0.49869E+00_JPRB, 0.44106E+00_JPRB, 0.43604E+00_JPRB, 0.47778E+00_JPRB, 0.52078E+00_JPRB, 0.56516E+00_JPRB, 0.60582E+00_JPRB ,&
&  0.64812E+00_JPRB, 0.67212E+00_JPRB, 0.71894E+00_JPRB, 0.74235E+00_JPRB, 0.75998E+00_JPRB, 0.76595E+00_JPRB, 0.77467E+00_JPRB ,&
&  0.80473E+00_JPRB, 0.83806E+00_JPRB, 0.86935E+00_JPRB, 0.91811E+00_JPRB, 0.97050E+00_JPRB, 0.10223E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.11451E+01_JPRB, 0.12153E+01_JPRB, 0.12869E+01_JPRB, 0.13726E+01_JPRB, 0.14406E+01_JPRB, 0.14645E+01_JPRB, 0.14121E+01_JPRB ,&
&  0.75028E+00_JPRB, 0.11678E+01_JPRB, 0.14624E+01_JPRB, 0.14305E+01_JPRB, 0.13796E+01_JPRB, 0.13229E+01_JPRB, 0.13277E+01_JPRB ,&
&  0.12698E+01_JPRB, 0.11815E+01_JPRB, 0.12353E+01_JPRB, 0.15667E+01_JPRB, 0.16470E+01_JPRB, 0.17176E+01_JPRB, 0.17670E+01_JPRB ,&
&  0.18303E+01_JPRB, 0.18775E+01_JPRB, 0.19321E+01_JPRB, 0.19913E+01_JPRB, 0.20675E+01_JPRB, 0.21392E+01_JPRB, 0.21994E+01_JPRB ,&
&  0.22392E+01_JPRB, 0.22525E+01_JPRB, 0.22415E+01_JPRB, 0.17830E+01_JPRB, 0.17417E+01_JPRB, 0.15214E+01_JPRB, 0.13407E+01_JPRB ,&
&  0.97159E+00_JPRB, 0.80351E+00_JPRB, 0.80925E+00_JPRB, 0.85643E+00_JPRB, 0.78996E+00_JPRB, 0.73960E+00_JPRB, 0.66845E+00_JPRB ,&
&  0.67003E+00_JPRB, 0.67078E+00_JPRB, 0.67384E+00_JPRB, 0.66241E+00_JPRB, 0.66462E+00_JPRB, 0.63066E+00_JPRB, 0.66215E+00_JPRB ,&
&  0.68600E+00_JPRB, 0.68573E+00_JPRB, 0.70091E+00_JPRB, 0.73779E+00_JPRB, 0.78299E+00_JPRB, 0.82431E+00_JPRB, 0.83546E+00_JPRB ,&
&  0.82738E+00_JPRB, 0.80850E+00_JPRB, 0.80334E+00_JPRB, 0.79706E+00_JPRB, 0.78639E+00_JPRB, 0.77651E+00_JPRB, 0.75630E+00_JPRB ,&
&  0.73390E+00_JPRB, 0.74382E+00_JPRB, 0.77726E+00_JPRB, 0.72092E+00_JPRB, 0.76055E+00_JPRB, 0.69102E+00_JPRB, 0.67526E+00_JPRB ,&
&  0.65789E+00_JPRB, 0.63960E+00_JPRB, 0.59308E+00_JPRB, 0.56953E+00_JPRB, 0.53790E+00_JPRB /)
ZBURD(:, 43,  4) = (/ &
&  0.60085E+00_JPRB, 0.53281E+00_JPRB, 0.47794E+00_JPRB, 0.49270E+00_JPRB, 0.55626E+00_JPRB, 0.61030E+00_JPRB, 0.63868E+00_JPRB ,&
&  0.67193E+00_JPRB, 0.68108E+00_JPRB, 0.68794E+00_JPRB, 0.69799E+00_JPRB, 0.72101E+00_JPRB, 0.72873E+00_JPRB, 0.75294E+00_JPRB ,&
&  0.76735E+00_JPRB, 0.77687E+00_JPRB, 0.79173E+00_JPRB, 0.83188E+00_JPRB, 0.88293E+00_JPRB, 0.93545E+00_JPRB, 0.98706E+00_JPRB ,&
&  0.10476E+01_JPRB, 0.11190E+01_JPRB, 0.12197E+01_JPRB, 0.13410E+01_JPRB, 0.15068E+01_JPRB, 0.16901E+01_JPRB, 0.18285E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.67081E+00_JPRB, 0.12457E+01_JPRB, 0.14906E+01_JPRB, 0.13915E+01_JPRB, 0.13330E+01_JPRB, 0.12640E+01_JPRB ,&
&  0.11991E+01_JPRB, 0.11487E+01_JPRB, 0.11427E+01_JPRB, 0.13826E+01_JPRB, 0.15908E+01_JPRB, 0.16700E+01_JPRB, 0.17011E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.17395E+01_JPRB, 0.17606E+01_JPRB, 0.17954E+01_JPRB, 0.18384E+01_JPRB, 0.18702E+01_JPRB, 0.18868E+01_JPRB ,&
&  0.18976E+01_JPRB, 0.18764E+01_JPRB, 0.18064E+01_JPRB, 0.13545E+01_JPRB, 0.12572E+01_JPRB, 0.11513E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.99467E+00_JPRB, 0.85997E+00_JPRB, 0.82603E+00_JPRB, 0.84405E+00_JPRB, 0.79263E+00_JPRB, 0.70486E+00_JPRB, 0.68731E+00_JPRB ,&
&  0.69267E+00_JPRB, 0.67700E+00_JPRB, 0.65157E+00_JPRB, 0.63530E+00_JPRB, 0.66634E+00_JPRB, 0.67119E+00_JPRB, 0.67877E+00_JPRB ,&
&  0.68142E+00_JPRB, 0.71984E+00_JPRB, 0.75838E+00_JPRB, 0.79351E+00_JPRB, 0.82998E+00_JPRB, 0.87124E+00_JPRB, 0.88031E+00_JPRB ,&
&  0.89510E+00_JPRB, 0.89345E+00_JPRB, 0.89634E+00_JPRB, 0.87756E+00_JPRB, 0.85662E+00_JPRB, 0.81909E+00_JPRB, 0.80682E+00_JPRB ,&
&  0.80071E+00_JPRB, 0.79872E+00_JPRB, 0.78606E+00_JPRB, 0.77376E+00_JPRB, 0.79042E+00_JPRB, 0.78163E+00_JPRB, 0.78813E+00_JPRB ,&
&  0.78439E+00_JPRB, 0.76317E+00_JPRB, 0.70586E+00_JPRB, 0.66003E+00_JPRB, 0.63983E+00_JPRB /)
ZBURD(:, 44,  4) = (/ &
&  0.65449E+00_JPRB, 0.61297E+00_JPRB, 0.55259E+00_JPRB, 0.51313E+00_JPRB, 0.53441E+00_JPRB, 0.60014E+00_JPRB, 0.65263E+00_JPRB ,&
&  0.67228E+00_JPRB, 0.67100E+00_JPRB, 0.66139E+00_JPRB, 0.65959E+00_JPRB, 0.67222E+00_JPRB, 0.66803E+00_JPRB, 0.67997E+00_JPRB ,&
&  0.70001E+00_JPRB, 0.71206E+00_JPRB, 0.72616E+00_JPRB, 0.74790E+00_JPRB, 0.77398E+00_JPRB, 0.82021E+00_JPRB, 0.86613E+00_JPRB ,&
&  0.90282E+00_JPRB, 0.93995E+00_JPRB, 0.10008E+01_JPRB, 0.10883E+01_JPRB, 0.12161E+01_JPRB, 0.14430E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.25671E+01_JPRB, 0.17460E+01_JPRB, 0.85914E+00_JPRB, 0.14415E+01_JPRB, 0.15151E+01_JPRB, 0.14246E+01_JPRB, 0.12460E+01_JPRB ,&
&  0.12259E+01_JPRB, 0.11307E+01_JPRB, 0.10724E+01_JPRB, 0.11675E+01_JPRB, 0.12831E+01_JPRB, 0.14454E+01_JPRB, 0.15466E+01_JPRB ,&
&  0.15665E+01_JPRB, 0.15739E+01_JPRB, 0.15820E+01_JPRB, 0.16030E+01_JPRB, 0.16405E+01_JPRB, 0.16834E+01_JPRB, 0.16937E+01_JPRB ,&
&  0.17039E+01_JPRB, 0.17204E+01_JPRB, 0.16671E+01_JPRB, 0.12942E+01_JPRB, 0.12273E+01_JPRB, 0.11989E+01_JPRB, 0.11559E+01_JPRB ,&
&  0.10521E+01_JPRB, 0.10067E+01_JPRB, 0.96696E+00_JPRB, 0.87791E+00_JPRB, 0.79838E+00_JPRB, 0.74993E+00_JPRB, 0.76707E+00_JPRB ,&
&  0.75852E+00_JPRB, 0.71706E+00_JPRB, 0.68978E+00_JPRB, 0.67149E+00_JPRB, 0.69417E+00_JPRB, 0.72224E+00_JPRB, 0.72798E+00_JPRB ,&
&  0.75095E+00_JPRB, 0.78326E+00_JPRB, 0.80796E+00_JPRB, 0.83159E+00_JPRB, 0.86555E+00_JPRB, 0.90227E+00_JPRB, 0.92792E+00_JPRB ,&
&  0.95516E+00_JPRB, 0.96695E+00_JPRB, 0.97549E+00_JPRB, 0.96971E+00_JPRB, 0.92682E+00_JPRB, 0.88242E+00_JPRB, 0.88771E+00_JPRB ,&
&  0.87111E+00_JPRB, 0.85736E+00_JPRB, 0.82233E+00_JPRB, 0.83220E+00_JPRB, 0.82672E+00_JPRB, 0.84208E+00_JPRB, 0.86762E+00_JPRB ,&
&  0.89287E+00_JPRB, 0.87481E+00_JPRB, 0.79175E+00_JPRB, 0.69809E+00_JPRB, 0.67422E+00_JPRB /)
ZBURD(:, 45,  4) = (/ &
&  0.68334E+00_JPRB, 0.66885E+00_JPRB, 0.63457E+00_JPRB, 0.56618E+00_JPRB, 0.54385E+00_JPRB, 0.55881E+00_JPRB, 0.62080E+00_JPRB ,&
&  0.64762E+00_JPRB, 0.65483E+00_JPRB, 0.63994E+00_JPRB, 0.62425E+00_JPRB, 0.61611E+00_JPRB, 0.61802E+00_JPRB, 0.63544E+00_JPRB ,&
&  0.65343E+00_JPRB, 0.64798E+00_JPRB, 0.64532E+00_JPRB, 0.65284E+00_JPRB, 0.67395E+00_JPRB, 0.70741E+00_JPRB, 0.75462E+00_JPRB ,&
&  0.79397E+00_JPRB, 0.81450E+00_JPRB, 0.83283E+00_JPRB, 0.88766E+00_JPRB, 0.95738E+00_JPRB, 0.99176E+00_JPRB, 0.10978E+01_JPRB ,&
&  0.15871E+01_JPRB, 0.23222E+01_JPRB, 0.10128E+01_JPRB, 0.15235E+01_JPRB, 0.17406E+01_JPRB, 0.15491E+01_JPRB, 0.14058E+01_JPRB ,&
&  0.13359E+01_JPRB, 0.11469E+01_JPRB, 0.10021E+01_JPRB, 0.10231E+01_JPRB, 0.10663E+01_JPRB, 0.11281E+01_JPRB, 0.12184E+01_JPRB ,&
&  0.13054E+01_JPRB, 0.13601E+01_JPRB, 0.14057E+01_JPRB, 0.14399E+01_JPRB, 0.14854E+01_JPRB, 0.15560E+01_JPRB, 0.16188E+01_JPRB ,&
&  0.16613E+01_JPRB, 0.17270E+01_JPRB, 0.17435E+01_JPRB, 0.14255E+01_JPRB, 0.13690E+01_JPRB, 0.14473E+01_JPRB, 0.13890E+01_JPRB ,&
&  0.12516E+01_JPRB, 0.11880E+01_JPRB, 0.10904E+01_JPRB, 0.95378E+00_JPRB, 0.84700E+00_JPRB, 0.80612E+00_JPRB, 0.81466E+00_JPRB ,&
&  0.78582E+00_JPRB, 0.76127E+00_JPRB, 0.75423E+00_JPRB, 0.75680E+00_JPRB, 0.75867E+00_JPRB, 0.76469E+00_JPRB, 0.78081E+00_JPRB ,&
&  0.81256E+00_JPRB, 0.84211E+00_JPRB, 0.86298E+00_JPRB, 0.87689E+00_JPRB, 0.89607E+00_JPRB, 0.92137E+00_JPRB, 0.93474E+00_JPRB ,&
&  0.96266E+00_JPRB, 0.98672E+00_JPRB, 0.10122E+01_JPRB, 0.10160E+01_JPRB, 0.99373E+00_JPRB, 0.97598E+00_JPRB, 0.97304E+00_JPRB ,&
&  0.97147E+00_JPRB, 0.95451E+00_JPRB, 0.83943E+00_JPRB, 0.82042E+00_JPRB, 0.84902E+00_JPRB, 0.81956E+00_JPRB, 0.85279E+00_JPRB ,&
&  0.91576E+00_JPRB, 0.96128E+00_JPRB, 0.91229E+00_JPRB, 0.71689E+00_JPRB, 0.69243E+00_JPRB /)
ZBURD(:, 46,  4) = (/ &
&  0.70807E+00_JPRB, 0.69877E+00_JPRB, 0.68699E+00_JPRB, 0.64009E+00_JPRB, 0.59730E+00_JPRB, 0.57806E+00_JPRB, 0.59288E+00_JPRB ,&
&  0.62776E+00_JPRB, 0.65187E+00_JPRB, 0.65068E+00_JPRB, 0.63941E+00_JPRB, 0.63584E+00_JPRB, 0.64214E+00_JPRB, 0.65313E+00_JPRB ,&
&  0.65604E+00_JPRB, 0.64736E+00_JPRB, 0.63777E+00_JPRB, 0.64263E+00_JPRB, 0.64505E+00_JPRB, 0.65529E+00_JPRB, 0.68086E+00_JPRB ,&
&  0.71676E+00_JPRB, 0.73577E+00_JPRB, 0.73971E+00_JPRB, 0.75468E+00_JPRB, 0.80558E+00_JPRB, 0.81934E+00_JPRB, 0.84361E+00_JPRB ,&
&  0.10213E+01_JPRB, 0.18704E+01_JPRB, 0.20865E+01_JPRB, 0.24526E+01_JPRB, 0.21701E+01_JPRB, 0.17795E+01_JPRB, 0.16624E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.13847E+01_JPRB, 0.12110E+01_JPRB, 0.10207E+01_JPRB, 0.10033E+01_JPRB, 0.10117E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.11534E+01_JPRB, 0.12096E+01_JPRB, 0.12598E+01_JPRB, 0.13081E+01_JPRB, 0.13649E+01_JPRB, 0.14524E+01_JPRB ,&
&  0.15537E+01_JPRB, 0.16845E+01_JPRB, 0.18301E+01_JPRB, 0.16964E+01_JPRB, 0.15112E+01_JPRB, 0.16946E+01_JPRB, 0.17519E+01_JPRB ,&
&  0.17037E+01_JPRB, 0.14494E+01_JPRB, 0.12333E+01_JPRB, 0.10614E+01_JPRB, 0.92527E+00_JPRB, 0.94977E+00_JPRB, 0.88351E+00_JPRB ,&
&  0.82430E+00_JPRB, 0.81784E+00_JPRB, 0.80921E+00_JPRB, 0.81599E+00_JPRB, 0.81737E+00_JPRB, 0.80970E+00_JPRB, 0.81674E+00_JPRB ,&
&  0.84049E+00_JPRB, 0.86308E+00_JPRB, 0.88109E+00_JPRB, 0.89413E+00_JPRB, 0.91157E+00_JPRB, 0.92561E+00_JPRB, 0.92911E+00_JPRB ,&
&  0.93397E+00_JPRB, 0.95914E+00_JPRB, 0.94510E+00_JPRB, 0.97008E+00_JPRB, 0.97586E+00_JPRB, 0.96596E+00_JPRB, 0.92754E+00_JPRB ,&
&  0.93126E+00_JPRB, 0.95032E+00_JPRB, 0.89718E+00_JPRB, 0.86995E+00_JPRB, 0.87220E+00_JPRB, 0.84791E+00_JPRB, 0.78588E+00_JPRB ,&
&  0.78593E+00_JPRB, 0.77204E+00_JPRB, 0.75953E+00_JPRB, 0.73749E+00_JPRB, 0.72092E+00_JPRB /)
ZBURD(:, 47,  4) = (/ &
&  0.74990E+00_JPRB, 0.73916E+00_JPRB, 0.73344E+00_JPRB, 0.70910E+00_JPRB, 0.68056E+00_JPRB, 0.65783E+00_JPRB, 0.64817E+00_JPRB ,&
&  0.65022E+00_JPRB, 0.66284E+00_JPRB, 0.67330E+00_JPRB, 0.66223E+00_JPRB, 0.65702E+00_JPRB, 0.66716E+00_JPRB, 0.67183E+00_JPRB ,&
&  0.67367E+00_JPRB, 0.67097E+00_JPRB, 0.66826E+00_JPRB, 0.66733E+00_JPRB, 0.66312E+00_JPRB, 0.65132E+00_JPRB, 0.65121E+00_JPRB ,&
&  0.66700E+00_JPRB, 0.68193E+00_JPRB, 0.68843E+00_JPRB, 0.69073E+00_JPRB, 0.71790E+00_JPRB, 0.73332E+00_JPRB, 0.75663E+00_JPRB ,&
&  0.84064E+00_JPRB, 0.16520E+01_JPRB, 0.15589E+01_JPRB, 0.28064E+01_JPRB, 0.23658E+01_JPRB, 0.19716E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.15978E+01_JPRB, 0.16724E+01_JPRB, 0.13779E+01_JPRB, 0.11593E+01_JPRB, 0.10415E+01_JPRB, 0.10072E+01_JPRB, 0.97572E+00_JPRB ,&
&  0.94685E+00_JPRB, 0.96718E+00_JPRB, 0.10014E+01_JPRB, 0.10509E+01_JPRB, 0.11101E+01_JPRB, 0.11588E+01_JPRB, 0.12257E+01_JPRB ,&
&  0.13033E+01_JPRB, 0.14332E+01_JPRB, 0.16329E+01_JPRB, 0.16351E+01_JPRB, 0.16299E+01_JPRB, 0.18432E+01_JPRB, 0.20624E+01_JPRB ,&
&  0.20566E+01_JPRB, 0.18034E+01_JPRB, 0.14211E+01_JPRB, 0.11731E+01_JPRB, 0.10461E+01_JPRB, 0.10121E+01_JPRB, 0.94572E+00_JPRB ,&
&  0.88873E+00_JPRB, 0.87034E+00_JPRB, 0.86392E+00_JPRB, 0.85230E+00_JPRB, 0.84616E+00_JPRB, 0.84087E+00_JPRB, 0.83615E+00_JPRB ,&
&  0.84516E+00_JPRB, 0.87819E+00_JPRB, 0.89089E+00_JPRB, 0.89595E+00_JPRB, 0.90259E+00_JPRB, 0.90639E+00_JPRB, 0.89921E+00_JPRB ,&
&  0.90854E+00_JPRB, 0.93227E+00_JPRB, 0.91221E+00_JPRB, 0.91423E+00_JPRB, 0.93139E+00_JPRB, 0.93254E+00_JPRB, 0.94023E+00_JPRB ,&
&  0.99168E+00_JPRB, 0.10117E+01_JPRB, 0.97036E+00_JPRB, 0.91190E+00_JPRB, 0.92310E+00_JPRB, 0.93319E+00_JPRB, 0.80214E+00_JPRB ,&
&  0.78679E+00_JPRB, 0.77760E+00_JPRB, 0.77747E+00_JPRB, 0.77273E+00_JPRB, 0.76171E+00_JPRB /)
ZBURD(:, 48,  4) = (/ &
&  0.76852E+00_JPRB, 0.75312E+00_JPRB, 0.74735E+00_JPRB, 0.73321E+00_JPRB, 0.71082E+00_JPRB, 0.69035E+00_JPRB, 0.68264E+00_JPRB ,&
&  0.67793E+00_JPRB, 0.68182E+00_JPRB, 0.69056E+00_JPRB, 0.68247E+00_JPRB, 0.68348E+00_JPRB, 0.69023E+00_JPRB, 0.69234E+00_JPRB ,&
&  0.68575E+00_JPRB, 0.68217E+00_JPRB, 0.67617E+00_JPRB, 0.67181E+00_JPRB, 0.66918E+00_JPRB, 0.65810E+00_JPRB, 0.65183E+00_JPRB ,&
&  0.65126E+00_JPRB, 0.65776E+00_JPRB, 0.66390E+00_JPRB, 0.66676E+00_JPRB, 0.68214E+00_JPRB, 0.69255E+00_JPRB, 0.70757E+00_JPRB ,&
&  0.79177E+00_JPRB, 0.17432E+01_JPRB, 0.14348E+01_JPRB, 0.25534E+01_JPRB, 0.22402E+01_JPRB, 0.18453E+01_JPRB, 0.15331E+01_JPRB ,&
&  0.15076E+01_JPRB, 0.15275E+01_JPRB, 0.13906E+01_JPRB, 0.12714E+01_JPRB, 0.11662E+01_JPRB, 0.10669E+01_JPRB, 0.99109E+00_JPRB ,&
&  0.93300E+00_JPRB, 0.90594E+00_JPRB, 0.90687E+00_JPRB, 0.92183E+00_JPRB, 0.95643E+00_JPRB, 0.99781E+00_JPRB, 0.10355E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.11541E+01_JPRB, 0.12941E+01_JPRB, 0.15197E+01_JPRB, 0.15309E+01_JPRB, 0.16983E+01_JPRB, 0.23150E+01_JPRB ,&
&  0.24066E+01_JPRB, 0.22259E+01_JPRB, 0.16729E+01_JPRB, 0.12847E+01_JPRB, 0.11337E+01_JPRB, 0.10696E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.95292E+00_JPRB, 0.91976E+00_JPRB, 0.92272E+00_JPRB, 0.89422E+00_JPRB, 0.86493E+00_JPRB, 0.86359E+00_JPRB, 0.87392E+00_JPRB ,&
&  0.88354E+00_JPRB, 0.87528E+00_JPRB, 0.89273E+00_JPRB, 0.89368E+00_JPRB, 0.88455E+00_JPRB, 0.86850E+00_JPRB, 0.85947E+00_JPRB ,&
&  0.86726E+00_JPRB, 0.89896E+00_JPRB, 0.90477E+00_JPRB, 0.89798E+00_JPRB, 0.90788E+00_JPRB, 0.92483E+00_JPRB, 0.94227E+00_JPRB ,&
&  0.10374E+01_JPRB, 0.10630E+01_JPRB, 0.10364E+01_JPRB, 0.10237E+01_JPRB, 0.10051E+01_JPRB, 0.10233E+01_JPRB, 0.82236E+00_JPRB ,&
&  0.79309E+00_JPRB, 0.78459E+00_JPRB, 0.79137E+00_JPRB, 0.79978E+00_JPRB, 0.79226E+00_JPRB /)
ZBURD(:, 49,  4) = (/ &
&  0.80390E+00_JPRB, 0.77746E+00_JPRB, 0.76677E+00_JPRB, 0.75637E+00_JPRB, 0.74096E+00_JPRB, 0.72520E+00_JPRB, 0.72390E+00_JPRB ,&
&  0.72609E+00_JPRB, 0.72798E+00_JPRB, 0.71924E+00_JPRB, 0.70346E+00_JPRB, 0.70141E+00_JPRB, 0.69913E+00_JPRB, 0.69471E+00_JPRB ,&
&  0.68963E+00_JPRB, 0.68809E+00_JPRB, 0.67791E+00_JPRB, 0.67167E+00_JPRB, 0.67236E+00_JPRB, 0.66817E+00_JPRB, 0.65821E+00_JPRB ,&
&  0.64778E+00_JPRB, 0.64285E+00_JPRB, 0.64940E+00_JPRB, 0.65648E+00_JPRB, 0.67301E+00_JPRB, 0.67816E+00_JPRB, 0.68943E+00_JPRB ,&
&  0.79458E+00_JPRB, 0.75554E+00_JPRB, 0.16897E+01_JPRB, 0.21940E+01_JPRB, 0.21246E+01_JPRB, 0.18171E+01_JPRB, 0.16033E+01_JPRB ,&
&  0.15373E+01_JPRB, 0.14383E+01_JPRB, 0.13652E+01_JPRB, 0.12605E+01_JPRB, 0.11522E+01_JPRB, 0.10361E+01_JPRB, 0.97678E+00_JPRB ,&
&  0.93990E+00_JPRB, 0.90612E+00_JPRB, 0.89561E+00_JPRB, 0.89420E+00_JPRB, 0.90066E+00_JPRB, 0.90583E+00_JPRB, 0.92550E+00_JPRB ,&
&  0.94492E+00_JPRB, 0.97081E+00_JPRB, 0.10276E+01_JPRB, 0.11975E+01_JPRB, 0.14273E+01_JPRB, 0.16324E+01_JPRB, 0.20872E+01_JPRB ,&
&  0.25736E+01_JPRB, 0.25060E+01_JPRB, 0.19036E+01_JPRB, 0.14236E+01_JPRB, 0.12059E+01_JPRB, 0.11208E+01_JPRB, 0.10559E+01_JPRB ,&
&  0.10018E+01_JPRB, 0.95404E+00_JPRB, 0.93655E+00_JPRB, 0.91664E+00_JPRB, 0.89105E+00_JPRB, 0.86952E+00_JPRB, 0.88314E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.90050E+00_JPRB, 0.88260E+00_JPRB, 0.87106E+00_JPRB, 0.85992E+00_JPRB, 0.84281E+00_JPRB, 0.83780E+00_JPRB ,&
&  0.84155E+00_JPRB, 0.87087E+00_JPRB, 0.89900E+00_JPRB, 0.89419E+00_JPRB, 0.90361E+00_JPRB, 0.93585E+00_JPRB, 0.98202E+00_JPRB ,&
&  0.10563E+01_JPRB, 0.10857E+01_JPRB, 0.10590E+01_JPRB, 0.10747E+01_JPRB, 0.10317E+01_JPRB, 0.10285E+01_JPRB, 0.86708E+00_JPRB ,&
&  0.81463E+00_JPRB, 0.80446E+00_JPRB, 0.81974E+00_JPRB, 0.83047E+00_JPRB, 0.83900E+00_JPRB /)
ZBURD(:, 50,  4) = (/ &
&  0.83595E+00_JPRB, 0.79790E+00_JPRB, 0.78144E+00_JPRB, 0.77686E+00_JPRB, 0.76734E+00_JPRB, 0.76769E+00_JPRB, 0.77016E+00_JPRB ,&
&  0.76598E+00_JPRB, 0.74929E+00_JPRB, 0.73445E+00_JPRB, 0.71825E+00_JPRB, 0.71232E+00_JPRB, 0.70427E+00_JPRB, 0.70164E+00_JPRB ,&
&  0.69303E+00_JPRB, 0.68805E+00_JPRB, 0.67949E+00_JPRB, 0.67568E+00_JPRB, 0.67401E+00_JPRB, 0.66722E+00_JPRB, 0.64541E+00_JPRB ,&
&  0.63871E+00_JPRB, 0.63763E+00_JPRB, 0.64680E+00_JPRB, 0.65909E+00_JPRB, 0.67451E+00_JPRB, 0.68121E+00_JPRB, 0.68864E+00_JPRB ,&
&  0.78221E+00_JPRB, 0.10601E+01_JPRB, 0.16646E+01_JPRB, 0.17828E+01_JPRB, 0.17877E+01_JPRB, 0.16348E+01_JPRB, 0.15077E+01_JPRB ,&
&  0.13968E+01_JPRB, 0.13478E+01_JPRB, 0.13385E+01_JPRB, 0.12306E+01_JPRB, 0.10754E+01_JPRB, 0.95413E+00_JPRB, 0.91016E+00_JPRB ,&
&  0.88539E+00_JPRB, 0.86456E+00_JPRB, 0.85445E+00_JPRB, 0.85359E+00_JPRB, 0.85072E+00_JPRB, 0.84985E+00_JPRB, 0.85839E+00_JPRB ,&
&  0.86598E+00_JPRB, 0.87861E+00_JPRB, 0.89819E+00_JPRB, 0.98318E+00_JPRB, 0.12884E+01_JPRB, 0.13688E+01_JPRB, 0.19021E+01_JPRB ,&
&  0.26827E+01_JPRB, 0.24481E+01_JPRB, 0.19839E+01_JPRB, 0.15634E+01_JPRB, 0.13205E+01_JPRB, 0.12022E+01_JPRB, 0.11137E+01_JPRB ,&
&  0.10459E+01_JPRB, 0.98761E+00_JPRB, 0.94364E+00_JPRB, 0.92768E+00_JPRB, 0.90873E+00_JPRB, 0.88757E+00_JPRB, 0.87895E+00_JPRB ,&
&  0.88836E+00_JPRB, 0.89538E+00_JPRB, 0.88393E+00_JPRB, 0.86019E+00_JPRB, 0.84287E+00_JPRB, 0.84098E+00_JPRB, 0.83981E+00_JPRB ,&
&  0.84569E+00_JPRB, 0.86230E+00_JPRB, 0.91919E+00_JPRB, 0.87617E+00_JPRB, 0.88754E+00_JPRB, 0.93000E+00_JPRB, 0.96109E+00_JPRB ,&
&  0.98187E+00_JPRB, 0.10341E+01_JPRB, 0.10742E+01_JPRB, 0.10826E+01_JPRB, 0.12027E+01_JPRB, 0.11476E+01_JPRB, 0.95419E+00_JPRB ,&
&  0.84802E+00_JPRB, 0.81659E+00_JPRB, 0.83573E+00_JPRB, 0.86499E+00_JPRB, 0.89580E+00_JPRB /)
ZBURD(:, 51,  4) = (/ &
&  0.87952E+00_JPRB, 0.82046E+00_JPRB, 0.78774E+00_JPRB, 0.78161E+00_JPRB, 0.78479E+00_JPRB, 0.77854E+00_JPRB, 0.78004E+00_JPRB ,&
&  0.77157E+00_JPRB, 0.75433E+00_JPRB, 0.73538E+00_JPRB, 0.71910E+00_JPRB, 0.71103E+00_JPRB, 0.70424E+00_JPRB, 0.69862E+00_JPRB ,&
&  0.69412E+00_JPRB, 0.68988E+00_JPRB, 0.68238E+00_JPRB, 0.68373E+00_JPRB, 0.68400E+00_JPRB, 0.68036E+00_JPRB, 0.65624E+00_JPRB ,&
&  0.64347E+00_JPRB, 0.64426E+00_JPRB, 0.65046E+00_JPRB, 0.65614E+00_JPRB, 0.66558E+00_JPRB, 0.67769E+00_JPRB, 0.69142E+00_JPRB ,&
&  0.73902E+00_JPRB, 0.99968E+00_JPRB, 0.13330E+01_JPRB, 0.15528E+01_JPRB, 0.16188E+01_JPRB, 0.15618E+01_JPRB, 0.14417E+01_JPRB ,&
&  0.13458E+01_JPRB, 0.12890E+01_JPRB, 0.12351E+01_JPRB, 0.11038E+01_JPRB, 0.96892E+00_JPRB, 0.89636E+00_JPRB, 0.86018E+00_JPRB ,&
&  0.84399E+00_JPRB, 0.83843E+00_JPRB, 0.83117E+00_JPRB, 0.82498E+00_JPRB, 0.81885E+00_JPRB, 0.81811E+00_JPRB, 0.82241E+00_JPRB ,&
&  0.82839E+00_JPRB, 0.82958E+00_JPRB, 0.83984E+00_JPRB, 0.84987E+00_JPRB, 0.10402E+01_JPRB, 0.12658E+01_JPRB, 0.16212E+01_JPRB ,&
&  0.17759E+01_JPRB, 0.18116E+01_JPRB, 0.16903E+01_JPRB, 0.15248E+01_JPRB, 0.13758E+01_JPRB, 0.12542E+01_JPRB, 0.11520E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.99016E+00_JPRB, 0.92724E+00_JPRB, 0.89262E+00_JPRB, 0.87667E+00_JPRB, 0.85621E+00_JPRB, 0.84542E+00_JPRB ,&
&  0.84596E+00_JPRB, 0.85109E+00_JPRB, 0.85084E+00_JPRB, 0.84296E+00_JPRB, 0.82864E+00_JPRB, 0.81858E+00_JPRB, 0.81497E+00_JPRB ,&
&  0.82443E+00_JPRB, 0.82936E+00_JPRB, 0.86715E+00_JPRB, 0.86874E+00_JPRB, 0.86779E+00_JPRB, 0.87438E+00_JPRB, 0.89756E+00_JPRB ,&
&  0.94226E+00_JPRB, 0.10255E+01_JPRB, 0.10252E+01_JPRB, 0.10843E+01_JPRB, 0.11702E+01_JPRB, 0.11465E+01_JPRB, 0.10015E+01_JPRB ,&
&  0.89398E+00_JPRB, 0.84146E+00_JPRB, 0.83114E+00_JPRB, 0.85208E+00_JPRB, 0.89725E+00_JPRB /)
ZBURD(:, 52,  4) = (/ &
&  0.91258E+00_JPRB, 0.86635E+00_JPRB, 0.82229E+00_JPRB, 0.79517E+00_JPRB, 0.78299E+00_JPRB, 0.77732E+00_JPRB, 0.77286E+00_JPRB ,&
&  0.76640E+00_JPRB, 0.75744E+00_JPRB, 0.73664E+00_JPRB, 0.72073E+00_JPRB, 0.70967E+00_JPRB, 0.70511E+00_JPRB, 0.70135E+00_JPRB ,&
&  0.69979E+00_JPRB, 0.69691E+00_JPRB, 0.68981E+00_JPRB, 0.68663E+00_JPRB, 0.68647E+00_JPRB, 0.67979E+00_JPRB, 0.66096E+00_JPRB ,&
&  0.64163E+00_JPRB, 0.64183E+00_JPRB, 0.64744E+00_JPRB, 0.65198E+00_JPRB, 0.66045E+00_JPRB, 0.67052E+00_JPRB, 0.68813E+00_JPRB ,&
&  0.73397E+00_JPRB, 0.84595E+00_JPRB, 0.11475E+01_JPRB, 0.13248E+01_JPRB, 0.14062E+01_JPRB, 0.14282E+01_JPRB, 0.13560E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.11558E+01_JPRB, 0.10686E+01_JPRB, 0.97597E+00_JPRB, 0.91253E+00_JPRB, 0.86645E+00_JPRB, 0.83591E+00_JPRB ,&
&  0.82060E+00_JPRB, 0.81465E+00_JPRB, 0.80879E+00_JPRB, 0.80076E+00_JPRB, 0.79511E+00_JPRB, 0.79220E+00_JPRB, 0.79197E+00_JPRB ,&
&  0.79499E+00_JPRB, 0.80236E+00_JPRB, 0.80995E+00_JPRB, 0.81834E+00_JPRB, 0.84183E+00_JPRB, 0.93857E+00_JPRB, 0.11329E+01_JPRB ,&
&  0.13077E+01_JPRB, 0.14088E+01_JPRB, 0.13873E+01_JPRB, 0.13478E+01_JPRB, 0.13098E+01_JPRB, 0.12331E+01_JPRB, 0.11535E+01_JPRB ,&
&  0.10750E+01_JPRB, 0.10002E+01_JPRB, 0.92275E+00_JPRB, 0.86171E+00_JPRB, 0.82394E+00_JPRB, 0.80319E+00_JPRB, 0.79262E+00_JPRB ,&
&  0.79241E+00_JPRB, 0.79992E+00_JPRB, 0.80108E+00_JPRB, 0.80445E+00_JPRB, 0.80016E+00_JPRB, 0.79206E+00_JPRB, 0.78548E+00_JPRB ,&
&  0.78023E+00_JPRB, 0.77652E+00_JPRB, 0.79253E+00_JPRB, 0.80151E+00_JPRB, 0.80632E+00_JPRB, 0.81804E+00_JPRB, 0.83582E+00_JPRB ,&
&  0.86035E+00_JPRB, 0.91831E+00_JPRB, 0.92312E+00_JPRB, 0.10712E+01_JPRB, 0.11708E+01_JPRB, 0.10890E+01_JPRB, 0.10058E+01_JPRB ,&
&  0.93176E+00_JPRB, 0.87604E+00_JPRB, 0.85024E+00_JPRB, 0.86742E+00_JPRB, 0.90962E+00_JPRB /)
ZBURD(:, 53,  4) = (/ &
&  0.81407E+00_JPRB, 0.80298E+00_JPRB, 0.79153E+00_JPRB, 0.78564E+00_JPRB, 0.77975E+00_JPRB, 0.76632E+00_JPRB, 0.75409E+00_JPRB ,&
&  0.74832E+00_JPRB, 0.74054E+00_JPRB, 0.73221E+00_JPRB, 0.72165E+00_JPRB, 0.70870E+00_JPRB, 0.69914E+00_JPRB, 0.69702E+00_JPRB ,&
&  0.69235E+00_JPRB, 0.68945E+00_JPRB, 0.68731E+00_JPRB, 0.68107E+00_JPRB, 0.67381E+00_JPRB, 0.66366E+00_JPRB, 0.64728E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.61957E+00_JPRB, 0.62799E+00_JPRB, 0.63580E+00_JPRB, 0.64143E+00_JPRB, 0.64430E+00_JPRB, 0.65745E+00_JPRB ,&
&  0.71364E+00_JPRB, 0.76633E+00_JPRB, 0.89079E+00_JPRB, 0.10617E+01_JPRB, 0.11623E+01_JPRB, 0.11447E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.10863E+01_JPRB, 0.10373E+01_JPRB, 0.98531E+00_JPRB, 0.92529E+00_JPRB, 0.87346E+00_JPRB, 0.83654E+00_JPRB, 0.81776E+00_JPRB ,&
&  0.80565E+00_JPRB, 0.79487E+00_JPRB, 0.78846E+00_JPRB, 0.77698E+00_JPRB, 0.76717E+00_JPRB, 0.75963E+00_JPRB, 0.75993E+00_JPRB ,&
&  0.76270E+00_JPRB, 0.76742E+00_JPRB, 0.77226E+00_JPRB, 0.77894E+00_JPRB, 0.79136E+00_JPRB, 0.82421E+00_JPRB, 0.91405E+00_JPRB ,&
&  0.10279E+01_JPRB, 0.11368E+01_JPRB, 0.11834E+01_JPRB, 0.11646E+01_JPRB, 0.11416E+01_JPRB, 0.11144E+01_JPRB, 0.10519E+01_JPRB ,&
&  0.97296E+00_JPRB, 0.91932E+00_JPRB, 0.87660E+00_JPRB, 0.83970E+00_JPRB, 0.81355E+00_JPRB, 0.79510E+00_JPRB, 0.78157E+00_JPRB ,&
&  0.77331E+00_JPRB, 0.76948E+00_JPRB, 0.76606E+00_JPRB, 0.76369E+00_JPRB, 0.75983E+00_JPRB, 0.74791E+00_JPRB, 0.73115E+00_JPRB ,&
&  0.71337E+00_JPRB, 0.71190E+00_JPRB, 0.72526E+00_JPRB, 0.73237E+00_JPRB, 0.73863E+00_JPRB, 0.75390E+00_JPRB, 0.77894E+00_JPRB ,&
&  0.80039E+00_JPRB, 0.83759E+00_JPRB, 0.86037E+00_JPRB, 0.98152E+00_JPRB, 0.10103E+01_JPRB, 0.98726E+00_JPRB, 0.97453E+00_JPRB ,&
&  0.94356E+00_JPRB, 0.89408E+00_JPRB, 0.84821E+00_JPRB, 0.84047E+00_JPRB, 0.78179E+00_JPRB /)
ZBURD(:, 54,  4) = (/ &
&  0.79322E+00_JPRB, 0.78360E+00_JPRB, 0.77128E+00_JPRB, 0.76567E+00_JPRB, 0.76274E+00_JPRB, 0.74664E+00_JPRB, 0.73218E+00_JPRB ,&
&  0.72135E+00_JPRB, 0.71758E+00_JPRB, 0.71734E+00_JPRB, 0.71536E+00_JPRB, 0.70762E+00_JPRB, 0.69500E+00_JPRB, 0.68755E+00_JPRB ,&
&  0.67695E+00_JPRB, 0.66757E+00_JPRB, 0.66431E+00_JPRB, 0.66270E+00_JPRB, 0.65627E+00_JPRB, 0.64790E+00_JPRB, 0.64042E+00_JPRB ,&
&  0.62485E+00_JPRB, 0.61394E+00_JPRB, 0.61122E+00_JPRB, 0.61433E+00_JPRB, 0.61822E+00_JPRB, 0.61731E+00_JPRB, 0.63733E+00_JPRB ,&
&  0.70514E+00_JPRB, 0.67191E+00_JPRB, 0.67849E+00_JPRB, 0.81299E+00_JPRB, 0.91103E+00_JPRB, 0.92995E+00_JPRB, 0.95158E+00_JPRB ,&
&  0.97787E+00_JPRB, 0.97028E+00_JPRB, 0.94349E+00_JPRB, 0.88715E+00_JPRB, 0.84205E+00_JPRB, 0.80937E+00_JPRB, 0.78061E+00_JPRB ,&
&  0.75803E+00_JPRB, 0.73908E+00_JPRB, 0.72908E+00_JPRB, 0.71765E+00_JPRB, 0.71214E+00_JPRB, 0.70112E+00_JPRB, 0.69703E+00_JPRB ,&
&  0.69775E+00_JPRB, 0.70480E+00_JPRB, 0.71309E+00_JPRB, 0.72008E+00_JPRB, 0.73469E+00_JPRB, 0.75988E+00_JPRB, 0.80742E+00_JPRB ,&
&  0.87676E+00_JPRB, 0.96601E+00_JPRB, 0.10229E+01_JPRB, 0.10354E+01_JPRB, 0.10182E+01_JPRB, 0.98605E+00_JPRB, 0.93784E+00_JPRB ,&
&  0.88366E+00_JPRB, 0.83528E+00_JPRB, 0.80084E+00_JPRB, 0.78167E+00_JPRB, 0.76829E+00_JPRB, 0.75804E+00_JPRB, 0.74807E+00_JPRB ,&
&  0.73927E+00_JPRB, 0.73492E+00_JPRB, 0.73282E+00_JPRB, 0.72901E+00_JPRB, 0.72337E+00_JPRB, 0.70808E+00_JPRB, 0.68528E+00_JPRB ,&
&  0.66852E+00_JPRB, 0.66166E+00_JPRB, 0.66811E+00_JPRB, 0.67456E+00_JPRB, 0.68358E+00_JPRB, 0.70525E+00_JPRB, 0.73041E+00_JPRB ,&
&  0.75250E+00_JPRB, 0.78252E+00_JPRB, 0.81185E+00_JPRB, 0.89244E+00_JPRB, 0.94982E+00_JPRB, 0.94083E+00_JPRB, 0.94022E+00_JPRB ,&
&  0.92915E+00_JPRB, 0.89869E+00_JPRB, 0.86530E+00_JPRB, 0.71920E+00_JPRB, 0.78112E+00_JPRB /)
ZBURD(:, 55,  4) = (/ &
&  0.77996E+00_JPRB, 0.77119E+00_JPRB, 0.75243E+00_JPRB, 0.73937E+00_JPRB, 0.72906E+00_JPRB, 0.71735E+00_JPRB, 0.70184E+00_JPRB ,&
&  0.68974E+00_JPRB, 0.68333E+00_JPRB, 0.67868E+00_JPRB, 0.67691E+00_JPRB, 0.67589E+00_JPRB, 0.66920E+00_JPRB, 0.65944E+00_JPRB ,&
&  0.64956E+00_JPRB, 0.64063E+00_JPRB, 0.64108E+00_JPRB, 0.63974E+00_JPRB, 0.63414E+00_JPRB, 0.62944E+00_JPRB, 0.62333E+00_JPRB ,&
&  0.60813E+00_JPRB, 0.59208E+00_JPRB, 0.57953E+00_JPRB, 0.58192E+00_JPRB, 0.58820E+00_JPRB, 0.58323E+00_JPRB, 0.60070E+00_JPRB ,&
&  0.66974E+00_JPRB, 0.60600E+00_JPRB, 0.63440E+00_JPRB, 0.70959E+00_JPRB, 0.75021E+00_JPRB, 0.78975E+00_JPRB, 0.81875E+00_JPRB ,&
&  0.84062E+00_JPRB, 0.85435E+00_JPRB, 0.81545E+00_JPRB, 0.76869E+00_JPRB, 0.72574E+00_JPRB, 0.69040E+00_JPRB, 0.66521E+00_JPRB ,&
&  0.65143E+00_JPRB, 0.64881E+00_JPRB, 0.64763E+00_JPRB, 0.64175E+00_JPRB, 0.63686E+00_JPRB, 0.63065E+00_JPRB, 0.62701E+00_JPRB ,&
&  0.62556E+00_JPRB, 0.62804E+00_JPRB, 0.63372E+00_JPRB, 0.64660E+00_JPRB, 0.66069E+00_JPRB, 0.67730E+00_JPRB, 0.70139E+00_JPRB ,&
&  0.73672E+00_JPRB, 0.78604E+00_JPRB, 0.83628E+00_JPRB, 0.86025E+00_JPRB, 0.85689E+00_JPRB, 0.83648E+00_JPRB, 0.80468E+00_JPRB ,&
&  0.77009E+00_JPRB, 0.74001E+00_JPRB, 0.72044E+00_JPRB, 0.70793E+00_JPRB, 0.69526E+00_JPRB, 0.68395E+00_JPRB, 0.67489E+00_JPRB ,&
&  0.66746E+00_JPRB, 0.65920E+00_JPRB, 0.65690E+00_JPRB, 0.65396E+00_JPRB, 0.64865E+00_JPRB, 0.63780E+00_JPRB, 0.62693E+00_JPRB ,&
&  0.61731E+00_JPRB, 0.61047E+00_JPRB, 0.60989E+00_JPRB, 0.61661E+00_JPRB, 0.62679E+00_JPRB, 0.64590E+00_JPRB, 0.67076E+00_JPRB ,&
&  0.69167E+00_JPRB, 0.71534E+00_JPRB, 0.73496E+00_JPRB, 0.80626E+00_JPRB, 0.85148E+00_JPRB, 0.87531E+00_JPRB, 0.88677E+00_JPRB ,&
&  0.88940E+00_JPRB, 0.89144E+00_JPRB, 0.68613E+00_JPRB, 0.75886E+00_JPRB, 0.77440E+00_JPRB /)
ZBURD(:, 56,  4) = (/ &
&  0.78446E+00_JPRB, 0.76038E+00_JPRB, 0.73070E+00_JPRB, 0.70639E+00_JPRB, 0.68901E+00_JPRB, 0.67806E+00_JPRB, 0.66812E+00_JPRB ,&
&  0.65822E+00_JPRB, 0.65518E+00_JPRB, 0.65395E+00_JPRB, 0.65312E+00_JPRB, 0.65219E+00_JPRB, 0.64750E+00_JPRB, 0.63832E+00_JPRB ,&
&  0.62819E+00_JPRB, 0.61573E+00_JPRB, 0.61315E+00_JPRB, 0.61197E+00_JPRB, 0.60540E+00_JPRB, 0.60081E+00_JPRB, 0.60014E+00_JPRB ,&
&  0.59344E+00_JPRB, 0.58269E+00_JPRB, 0.56347E+00_JPRB, 0.55699E+00_JPRB, 0.55929E+00_JPRB, 0.55904E+00_JPRB, 0.56656E+00_JPRB ,&
&  0.55566E+00_JPRB, 0.53925E+00_JPRB, 0.59535E+00_JPRB, 0.63225E+00_JPRB, 0.65047E+00_JPRB, 0.66773E+00_JPRB, 0.68719E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.68427E+00_JPRB, 0.66417E+00_JPRB, 0.63227E+00_JPRB, 0.61017E+00_JPRB, 0.59835E+00_JPRB, 0.59303E+00_JPRB ,&
&  0.59573E+00_JPRB, 0.59786E+00_JPRB, 0.59739E+00_JPRB, 0.59613E+00_JPRB, 0.59644E+00_JPRB, 0.59284E+00_JPRB, 0.59065E+00_JPRB ,&
&  0.59226E+00_JPRB, 0.59603E+00_JPRB, 0.59825E+00_JPRB, 0.60296E+00_JPRB, 0.61273E+00_JPRB, 0.62763E+00_JPRB, 0.64340E+00_JPRB ,&
&  0.65890E+00_JPRB, 0.67632E+00_JPRB, 0.70211E+00_JPRB, 0.72204E+00_JPRB, 0.72676E+00_JPRB, 0.71445E+00_JPRB, 0.69505E+00_JPRB ,&
&  0.67578E+00_JPRB, 0.66140E+00_JPRB, 0.65058E+00_JPRB, 0.64202E+00_JPRB, 0.63071E+00_JPRB, 0.62010E+00_JPRB, 0.60896E+00_JPRB ,&
&  0.60085E+00_JPRB, 0.59164E+00_JPRB, 0.58546E+00_JPRB, 0.58057E+00_JPRB, 0.57871E+00_JPRB, 0.57930E+00_JPRB, 0.58155E+00_JPRB ,&
&  0.58283E+00_JPRB, 0.57976E+00_JPRB, 0.57886E+00_JPRB, 0.58408E+00_JPRB, 0.59068E+00_JPRB, 0.60237E+00_JPRB, 0.61920E+00_JPRB ,&
&  0.64002E+00_JPRB, 0.66169E+00_JPRB, 0.68271E+00_JPRB, 0.73266E+00_JPRB, 0.78410E+00_JPRB, 0.82289E+00_JPRB, 0.84185E+00_JPRB ,&
&  0.85183E+00_JPRB, 0.84265E+00_JPRB, 0.83981E+00_JPRB, 0.82412E+00_JPRB, 0.80298E+00_JPRB /)
ZBURD(:, 57,  4) = (/ &
&  0.71945E+00_JPRB, 0.68873E+00_JPRB, 0.66138E+00_JPRB, 0.63982E+00_JPRB, 0.62510E+00_JPRB, 0.61768E+00_JPRB, 0.61068E+00_JPRB ,&
&  0.60620E+00_JPRB, 0.60140E+00_JPRB, 0.59826E+00_JPRB, 0.59994E+00_JPRB, 0.60042E+00_JPRB, 0.60013E+00_JPRB, 0.60060E+00_JPRB ,&
&  0.59429E+00_JPRB, 0.58076E+00_JPRB, 0.56995E+00_JPRB, 0.56678E+00_JPRB, 0.56757E+00_JPRB, 0.56358E+00_JPRB, 0.56217E+00_JPRB ,&
&  0.56524E+00_JPRB, 0.55715E+00_JPRB, 0.54323E+00_JPRB, 0.53391E+00_JPRB, 0.53400E+00_JPRB, 0.53126E+00_JPRB, 0.52880E+00_JPRB ,&
&  0.55386E+00_JPRB, 0.50618E+00_JPRB, 0.55588E+00_JPRB, 0.56384E+00_JPRB, 0.57064E+00_JPRB, 0.57521E+00_JPRB, 0.57597E+00_JPRB ,&
&  0.57217E+00_JPRB, 0.56550E+00_JPRB, 0.55859E+00_JPRB, 0.55139E+00_JPRB, 0.54604E+00_JPRB, 0.53989E+00_JPRB, 0.53477E+00_JPRB ,&
&  0.53487E+00_JPRB, 0.53555E+00_JPRB, 0.53982E+00_JPRB, 0.54530E+00_JPRB, 0.55288E+00_JPRB, 0.55558E+00_JPRB, 0.55797E+00_JPRB ,&
&  0.56084E+00_JPRB, 0.56268E+00_JPRB, 0.56276E+00_JPRB, 0.56179E+00_JPRB, 0.56051E+00_JPRB, 0.56536E+00_JPRB, 0.57212E+00_JPRB ,&
&  0.57834E+00_JPRB, 0.58690E+00_JPRB, 0.59926E+00_JPRB, 0.60825E+00_JPRB, 0.61231E+00_JPRB, 0.60952E+00_JPRB, 0.60200E+00_JPRB ,&
&  0.59655E+00_JPRB, 0.59549E+00_JPRB, 0.59524E+00_JPRB, 0.58993E+00_JPRB, 0.58210E+00_JPRB, 0.57485E+00_JPRB, 0.56538E+00_JPRB ,&
&  0.55330E+00_JPRB, 0.54328E+00_JPRB, 0.53742E+00_JPRB, 0.53494E+00_JPRB, 0.53500E+00_JPRB, 0.53467E+00_JPRB, 0.53434E+00_JPRB ,&
&  0.53630E+00_JPRB, 0.53681E+00_JPRB, 0.53866E+00_JPRB, 0.54402E+00_JPRB, 0.54925E+00_JPRB, 0.55684E+00_JPRB, 0.56794E+00_JPRB ,&
&  0.58347E+00_JPRB, 0.60263E+00_JPRB, 0.62021E+00_JPRB, 0.65055E+00_JPRB, 0.69285E+00_JPRB, 0.72753E+00_JPRB, 0.74633E+00_JPRB ,&
&  0.76162E+00_JPRB, 0.76715E+00_JPRB, 0.76337E+00_JPRB, 0.75761E+00_JPRB, 0.74157E+00_JPRB /)
ZBURD(:, 58,  4) = (/ &
&  0.64788E+00_JPRB, 0.62844E+00_JPRB, 0.61282E+00_JPRB, 0.60033E+00_JPRB, 0.59115E+00_JPRB, 0.58629E+00_JPRB, 0.58130E+00_JPRB ,&
&  0.57718E+00_JPRB, 0.57395E+00_JPRB, 0.56640E+00_JPRB, 0.55880E+00_JPRB, 0.55752E+00_JPRB, 0.56262E+00_JPRB, 0.56666E+00_JPRB ,&
&  0.56338E+00_JPRB, 0.55251E+00_JPRB, 0.54231E+00_JPRB, 0.54004E+00_JPRB, 0.54292E+00_JPRB, 0.54541E+00_JPRB, 0.54519E+00_JPRB ,&
&  0.54697E+00_JPRB, 0.54810E+00_JPRB, 0.53972E+00_JPRB, 0.52245E+00_JPRB, 0.51426E+00_JPRB, 0.50947E+00_JPRB, 0.51413E+00_JPRB ,&
&  0.52344E+00_JPRB, 0.52160E+00_JPRB, 0.53253E+00_JPRB, 0.52931E+00_JPRB, 0.52823E+00_JPRB, 0.52477E+00_JPRB, 0.51827E+00_JPRB ,&
&  0.51294E+00_JPRB, 0.50725E+00_JPRB, 0.50346E+00_JPRB, 0.50135E+00_JPRB, 0.50107E+00_JPRB, 0.50173E+00_JPRB, 0.50552E+00_JPRB ,&
&  0.50594E+00_JPRB, 0.50369E+00_JPRB, 0.50603E+00_JPRB, 0.51298E+00_JPRB, 0.52177E+00_JPRB, 0.52690E+00_JPRB, 0.53159E+00_JPRB ,&
&  0.53548E+00_JPRB, 0.53772E+00_JPRB, 0.54071E+00_JPRB, 0.54160E+00_JPRB, 0.53797E+00_JPRB, 0.53725E+00_JPRB, 0.53725E+00_JPRB ,&
&  0.53957E+00_JPRB, 0.54350E+00_JPRB, 0.54936E+00_JPRB, 0.55600E+00_JPRB, 0.55922E+00_JPRB, 0.55685E+00_JPRB, 0.55392E+00_JPRB ,&
&  0.55466E+00_JPRB, 0.55869E+00_JPRB, 0.56316E+00_JPRB, 0.56567E+00_JPRB, 0.56468E+00_JPRB, 0.55740E+00_JPRB, 0.54864E+00_JPRB ,&
&  0.53937E+00_JPRB, 0.52981E+00_JPRB, 0.52121E+00_JPRB, 0.51587E+00_JPRB, 0.51173E+00_JPRB, 0.50901E+00_JPRB, 0.50737E+00_JPRB ,&
&  0.50688E+00_JPRB, 0.50712E+00_JPRB, 0.51051E+00_JPRB, 0.51701E+00_JPRB, 0.52098E+00_JPRB, 0.52979E+00_JPRB, 0.54129E+00_JPRB ,&
&  0.55445E+00_JPRB, 0.56908E+00_JPRB, 0.58691E+00_JPRB, 0.61094E+00_JPRB, 0.63918E+00_JPRB, 0.66426E+00_JPRB, 0.68380E+00_JPRB ,&
&  0.69664E+00_JPRB, 0.69812E+00_JPRB, 0.69118E+00_JPRB, 0.68151E+00_JPRB, 0.66685E+00_JPRB /)
ZBURD(:, 59,  4) = (/ &
&  0.58267E+00_JPRB, 0.57309E+00_JPRB, 0.56535E+00_JPRB, 0.55856E+00_JPRB, 0.55236E+00_JPRB, 0.54882E+00_JPRB, 0.54527E+00_JPRB ,&
&  0.54200E+00_JPRB, 0.53822E+00_JPRB, 0.53584E+00_JPRB, 0.53161E+00_JPRB, 0.52802E+00_JPRB, 0.53046E+00_JPRB, 0.53578E+00_JPRB ,&
&  0.53425E+00_JPRB, 0.52733E+00_JPRB, 0.52184E+00_JPRB, 0.52167E+00_JPRB, 0.52514E+00_JPRB, 0.52780E+00_JPRB, 0.53092E+00_JPRB ,&
&  0.53084E+00_JPRB, 0.53026E+00_JPRB, 0.52874E+00_JPRB, 0.51296E+00_JPRB, 0.49769E+00_JPRB, 0.49009E+00_JPRB, 0.48577E+00_JPRB ,&
&  0.48557E+00_JPRB, 0.49263E+00_JPRB, 0.47559E+00_JPRB, 0.48496E+00_JPRB, 0.48083E+00_JPRB, 0.47464E+00_JPRB, 0.47108E+00_JPRB ,&
&  0.46900E+00_JPRB, 0.46669E+00_JPRB, 0.46494E+00_JPRB, 0.46674E+00_JPRB, 0.46919E+00_JPRB, 0.46977E+00_JPRB, 0.47136E+00_JPRB ,&
&  0.47104E+00_JPRB, 0.47349E+00_JPRB, 0.47573E+00_JPRB, 0.48092E+00_JPRB, 0.48490E+00_JPRB, 0.48746E+00_JPRB, 0.49074E+00_JPRB ,&
&  0.49530E+00_JPRB, 0.49955E+00_JPRB, 0.50360E+00_JPRB, 0.50703E+00_JPRB, 0.50845E+00_JPRB, 0.50823E+00_JPRB, 0.50415E+00_JPRB ,&
&  0.50085E+00_JPRB, 0.50070E+00_JPRB, 0.50362E+00_JPRB, 0.50986E+00_JPRB, 0.51590E+00_JPRB, 0.51812E+00_JPRB, 0.51918E+00_JPRB ,&
&  0.52090E+00_JPRB, 0.52471E+00_JPRB, 0.52548E+00_JPRB, 0.52417E+00_JPRB, 0.52211E+00_JPRB, 0.51941E+00_JPRB, 0.51479E+00_JPRB ,&
&  0.50875E+00_JPRB, 0.50388E+00_JPRB, 0.49887E+00_JPRB, 0.48774E+00_JPRB, 0.47923E+00_JPRB, 0.47835E+00_JPRB, 0.47988E+00_JPRB ,&
&  0.47938E+00_JPRB, 0.47784E+00_JPRB, 0.47903E+00_JPRB, 0.48473E+00_JPRB, 0.48487E+00_JPRB, 0.49187E+00_JPRB, 0.50594E+00_JPRB ,&
&  0.52069E+00_JPRB, 0.53459E+00_JPRB, 0.54782E+00_JPRB, 0.56235E+00_JPRB, 0.57761E+00_JPRB, 0.59073E+00_JPRB, 0.60207E+00_JPRB ,&
&  0.60764E+00_JPRB, 0.60614E+00_JPRB, 0.59977E+00_JPRB, 0.59554E+00_JPRB, 0.59175E+00_JPRB /)
ZBURD(:, 60,  4) = (/ &
&  0.54452E+00_JPRB, 0.54310E+00_JPRB, 0.54134E+00_JPRB, 0.53908E+00_JPRB, 0.53554E+00_JPRB, 0.53201E+00_JPRB, 0.52650E+00_JPRB ,&
&  0.52073E+00_JPRB, 0.51586E+00_JPRB, 0.51349E+00_JPRB, 0.51248E+00_JPRB, 0.51351E+00_JPRB, 0.51741E+00_JPRB, 0.52415E+00_JPRB ,&
&  0.52420E+00_JPRB, 0.51745E+00_JPRB, 0.51431E+00_JPRB, 0.51431E+00_JPRB, 0.51677E+00_JPRB, 0.51861E+00_JPRB, 0.52145E+00_JPRB ,&
&  0.52332E+00_JPRB, 0.52394E+00_JPRB, 0.52319E+00_JPRB, 0.51600E+00_JPRB, 0.49894E+00_JPRB, 0.48854E+00_JPRB, 0.47944E+00_JPRB ,&
&  0.47405E+00_JPRB, 0.46988E+00_JPRB, 0.46676E+00_JPRB, 0.46569E+00_JPRB, 0.46462E+00_JPRB, 0.46279E+00_JPRB, 0.46276E+00_JPRB ,&
&  0.46085E+00_JPRB, 0.45833E+00_JPRB, 0.45692E+00_JPRB, 0.45739E+00_JPRB, 0.45903E+00_JPRB, 0.46057E+00_JPRB, 0.46048E+00_JPRB ,&
&  0.46149E+00_JPRB, 0.46525E+00_JPRB, 0.46636E+00_JPRB, 0.46663E+00_JPRB, 0.46653E+00_JPRB, 0.46613E+00_JPRB, 0.46627E+00_JPRB ,&
&  0.46864E+00_JPRB, 0.47377E+00_JPRB, 0.47816E+00_JPRB, 0.48260E+00_JPRB, 0.48456E+00_JPRB, 0.48552E+00_JPRB, 0.48557E+00_JPRB ,&
&  0.48252E+00_JPRB, 0.47852E+00_JPRB, 0.47723E+00_JPRB, 0.47977E+00_JPRB, 0.48496E+00_JPRB, 0.49112E+00_JPRB, 0.49500E+00_JPRB ,&
&  0.49799E+00_JPRB, 0.49910E+00_JPRB, 0.49693E+00_JPRB, 0.49568E+00_JPRB, 0.49292E+00_JPRB, 0.48832E+00_JPRB, 0.48567E+00_JPRB ,&
&  0.48313E+00_JPRB, 0.48090E+00_JPRB, 0.47888E+00_JPRB, 0.46992E+00_JPRB, 0.46217E+00_JPRB, 0.45740E+00_JPRB, 0.45428E+00_JPRB ,&
&  0.45304E+00_JPRB, 0.45204E+00_JPRB, 0.45169E+00_JPRB, 0.45291E+00_JPRB, 0.45510E+00_JPRB, 0.46183E+00_JPRB, 0.47513E+00_JPRB ,&
&  0.49485E+00_JPRB, 0.51332E+00_JPRB, 0.52389E+00_JPRB, 0.53214E+00_JPRB, 0.53847E+00_JPRB, 0.54236E+00_JPRB, 0.54587E+00_JPRB ,&
&  0.55019E+00_JPRB, 0.55179E+00_JPRB, 0.54985E+00_JPRB, 0.54740E+00_JPRB, 0.54642E+00_JPRB /)
ZBURD(:, 61,  4) = (/ &
&  0.51100E+00_JPRB, 0.51193E+00_JPRB, 0.51109E+00_JPRB, 0.51019E+00_JPRB, 0.50891E+00_JPRB, 0.50753E+00_JPRB, 0.50520E+00_JPRB ,&
&  0.49980E+00_JPRB, 0.49292E+00_JPRB, 0.48879E+00_JPRB, 0.48685E+00_JPRB, 0.49232E+00_JPRB, 0.50056E+00_JPRB, 0.50800E+00_JPRB ,&
&  0.50715E+00_JPRB, 0.49803E+00_JPRB, 0.49523E+00_JPRB, 0.49719E+00_JPRB, 0.49974E+00_JPRB, 0.50262E+00_JPRB, 0.50650E+00_JPRB ,&
&  0.51072E+00_JPRB, 0.51500E+00_JPRB, 0.51687E+00_JPRB, 0.51425E+00_JPRB, 0.50754E+00_JPRB, 0.49357E+00_JPRB, 0.48140E+00_JPRB ,&
&  0.47330E+00_JPRB, 0.46707E+00_JPRB, 0.46335E+00_JPRB, 0.46235E+00_JPRB, 0.46015E+00_JPRB, 0.45637E+00_JPRB, 0.45107E+00_JPRB ,&
&  0.44647E+00_JPRB, 0.44434E+00_JPRB, 0.44654E+00_JPRB, 0.44826E+00_JPRB, 0.44912E+00_JPRB, 0.44883E+00_JPRB, 0.44591E+00_JPRB ,&
&  0.44429E+00_JPRB, 0.44381E+00_JPRB, 0.44354E+00_JPRB, 0.44514E+00_JPRB, 0.44287E+00_JPRB, 0.44100E+00_JPRB, 0.44313E+00_JPRB ,&
&  0.44770E+00_JPRB, 0.45169E+00_JPRB, 0.45535E+00_JPRB, 0.45821E+00_JPRB, 0.45946E+00_JPRB, 0.46167E+00_JPRB, 0.46307E+00_JPRB ,&
&  0.46219E+00_JPRB, 0.45861E+00_JPRB, 0.45530E+00_JPRB, 0.45586E+00_JPRB, 0.45831E+00_JPRB, 0.46111E+00_JPRB, 0.46476E+00_JPRB ,&
&  0.46547E+00_JPRB, 0.46044E+00_JPRB, 0.45534E+00_JPRB, 0.45079E+00_JPRB, 0.44655E+00_JPRB, 0.44323E+00_JPRB, 0.44155E+00_JPRB ,&
&  0.44101E+00_JPRB, 0.43791E+00_JPRB, 0.43194E+00_JPRB, 0.42701E+00_JPRB, 0.42624E+00_JPRB, 0.42517E+00_JPRB, 0.42115E+00_JPRB ,&
&  0.42139E+00_JPRB, 0.42377E+00_JPRB, 0.42258E+00_JPRB, 0.42065E+00_JPRB, 0.42268E+00_JPRB, 0.42819E+00_JPRB, 0.43945E+00_JPRB ,&
&  0.45450E+00_JPRB, 0.47171E+00_JPRB, 0.48769E+00_JPRB, 0.50164E+00_JPRB, 0.51020E+00_JPRB, 0.51251E+00_JPRB, 0.51107E+00_JPRB ,&
&  0.51064E+00_JPRB, 0.51147E+00_JPRB, 0.51058E+00_JPRB, 0.50893E+00_JPRB, 0.50948E+00_JPRB /)
ZBURD(:, 62,  4) = (/ &
&  0.48131E+00_JPRB, 0.48174E+00_JPRB, 0.48249E+00_JPRB, 0.48400E+00_JPRB, 0.48505E+00_JPRB, 0.48417E+00_JPRB, 0.48378E+00_JPRB ,&
&  0.48041E+00_JPRB, 0.47555E+00_JPRB, 0.47252E+00_JPRB, 0.47386E+00_JPRB, 0.48286E+00_JPRB, 0.49441E+00_JPRB, 0.50136E+00_JPRB ,&
&  0.49699E+00_JPRB, 0.48457E+00_JPRB, 0.48150E+00_JPRB, 0.48380E+00_JPRB, 0.48602E+00_JPRB, 0.49018E+00_JPRB, 0.49635E+00_JPRB ,&
&  0.50198E+00_JPRB, 0.50583E+00_JPRB, 0.51002E+00_JPRB, 0.51172E+00_JPRB, 0.51189E+00_JPRB, 0.50469E+00_JPRB, 0.49340E+00_JPRB ,&
&  0.48251E+00_JPRB, 0.47467E+00_JPRB, 0.46975E+00_JPRB, 0.46617E+00_JPRB, 0.46111E+00_JPRB, 0.45603E+00_JPRB, 0.44942E+00_JPRB ,&
&  0.44621E+00_JPRB, 0.44591E+00_JPRB, 0.44680E+00_JPRB, 0.44490E+00_JPRB, 0.44231E+00_JPRB, 0.43716E+00_JPRB, 0.43286E+00_JPRB ,&
&  0.43120E+00_JPRB, 0.43240E+00_JPRB, 0.43365E+00_JPRB, 0.43275E+00_JPRB, 0.42933E+00_JPRB, 0.42693E+00_JPRB, 0.42833E+00_JPRB ,&
&  0.43420E+00_JPRB, 0.43403E+00_JPRB, 0.43307E+00_JPRB, 0.43393E+00_JPRB, 0.43619E+00_JPRB, 0.43820E+00_JPRB, 0.43765E+00_JPRB ,&
&  0.43700E+00_JPRB, 0.43696E+00_JPRB, 0.43725E+00_JPRB, 0.43915E+00_JPRB, 0.43933E+00_JPRB, 0.43731E+00_JPRB, 0.43185E+00_JPRB ,&
&  0.42350E+00_JPRB, 0.41294E+00_JPRB, 0.40859E+00_JPRB, 0.40545E+00_JPRB, 0.40239E+00_JPRB, 0.40168E+00_JPRB, 0.40298E+00_JPRB ,&
&  0.40156E+00_JPRB, 0.39538E+00_JPRB, 0.39256E+00_JPRB, 0.39301E+00_JPRB, 0.39190E+00_JPRB, 0.39394E+00_JPRB, 0.39761E+00_JPRB ,&
&  0.40772E+00_JPRB, 0.41332E+00_JPRB, 0.41496E+00_JPRB, 0.41277E+00_JPRB, 0.41300E+00_JPRB, 0.41472E+00_JPRB, 0.41906E+00_JPRB ,&
&  0.42571E+00_JPRB, 0.43292E+00_JPRB, 0.44033E+00_JPRB, 0.44851E+00_JPRB, 0.45877E+00_JPRB, 0.46860E+00_JPRB, 0.47243E+00_JPRB ,&
&  0.47273E+00_JPRB, 0.47324E+00_JPRB, 0.47466E+00_JPRB, 0.47629E+00_JPRB, 0.47962E+00_JPRB /)
ZBURD(:, 63,  4) = (/ &
&  0.44660E+00_JPRB, 0.44926E+00_JPRB, 0.45186E+00_JPRB, 0.45365E+00_JPRB, 0.45650E+00_JPRB, 0.46144E+00_JPRB, 0.46341E+00_JPRB ,&
&  0.46147E+00_JPRB, 0.45926E+00_JPRB, 0.45906E+00_JPRB, 0.46248E+00_JPRB, 0.46914E+00_JPRB, 0.47604E+00_JPRB, 0.48369E+00_JPRB ,&
&  0.47954E+00_JPRB, 0.46698E+00_JPRB, 0.46240E+00_JPRB, 0.46442E+00_JPRB, 0.46928E+00_JPRB, 0.47469E+00_JPRB, 0.48139E+00_JPRB ,&
&  0.48776E+00_JPRB, 0.49196E+00_JPRB, 0.49633E+00_JPRB, 0.50027E+00_JPRB, 0.50281E+00_JPRB, 0.50104E+00_JPRB, 0.49659E+00_JPRB ,&
&  0.49126E+00_JPRB, 0.48584E+00_JPRB, 0.48098E+00_JPRB, 0.46525E+00_JPRB, 0.46267E+00_JPRB, 0.45367E+00_JPRB, 0.44619E+00_JPRB ,&
&  0.44100E+00_JPRB, 0.43670E+00_JPRB, 0.43396E+00_JPRB, 0.43088E+00_JPRB, 0.42658E+00_JPRB, 0.41931E+00_JPRB, 0.41722E+00_JPRB ,&
&  0.41624E+00_JPRB, 0.41726E+00_JPRB, 0.41261E+00_JPRB, 0.41013E+00_JPRB, 0.40738E+00_JPRB, 0.40525E+00_JPRB, 0.40946E+00_JPRB ,&
&  0.41460E+00_JPRB, 0.41210E+00_JPRB, 0.40815E+00_JPRB, 0.40786E+00_JPRB, 0.41214E+00_JPRB, 0.41379E+00_JPRB, 0.41474E+00_JPRB ,&
&  0.41651E+00_JPRB, 0.41704E+00_JPRB, 0.41244E+00_JPRB, 0.40733E+00_JPRB, 0.39877E+00_JPRB, 0.38795E+00_JPRB, 0.38499E+00_JPRB ,&
&  0.38199E+00_JPRB, 0.37624E+00_JPRB, 0.37285E+00_JPRB, 0.37175E+00_JPRB, 0.37135E+00_JPRB, 0.37216E+00_JPRB, 0.37226E+00_JPRB ,&
&  0.37223E+00_JPRB, 0.36868E+00_JPRB, 0.36343E+00_JPRB, 0.35891E+00_JPRB, 0.35276E+00_JPRB, 0.35530E+00_JPRB, 0.36499E+00_JPRB ,&
&  0.37710E+00_JPRB, 0.38449E+00_JPRB, 0.38931E+00_JPRB, 0.39274E+00_JPRB, 0.39579E+00_JPRB, 0.39718E+00_JPRB, 0.39948E+00_JPRB ,&
&  0.40258E+00_JPRB, 0.40400E+00_JPRB, 0.40263E+00_JPRB, 0.40271E+00_JPRB, 0.40586E+00_JPRB, 0.41101E+00_JPRB, 0.41727E+00_JPRB ,&
&  0.42353E+00_JPRB, 0.42887E+00_JPRB, 0.43342E+00_JPRB, 0.43910E+00_JPRB, 0.44294E+00_JPRB /)
ZBURD(:, 64,  4) = (/ &
&  0.41878E+00_JPRB, 0.42335E+00_JPRB, 0.42764E+00_JPRB, 0.43228E+00_JPRB, 0.43875E+00_JPRB, 0.44202E+00_JPRB, 0.44305E+00_JPRB ,&
&  0.44437E+00_JPRB, 0.44494E+00_JPRB, 0.44646E+00_JPRB, 0.45083E+00_JPRB, 0.45695E+00_JPRB, 0.46130E+00_JPRB, 0.46699E+00_JPRB ,&
&  0.46700E+00_JPRB, 0.45820E+00_JPRB, 0.45024E+00_JPRB, 0.44994E+00_JPRB, 0.45548E+00_JPRB, 0.46125E+00_JPRB, 0.46851E+00_JPRB ,&
&  0.47570E+00_JPRB, 0.48092E+00_JPRB, 0.48384E+00_JPRB, 0.48530E+00_JPRB, 0.48724E+00_JPRB, 0.48684E+00_JPRB, 0.48382E+00_JPRB ,&
&  0.47965E+00_JPRB, 0.47058E+00_JPRB, 0.45104E+00_JPRB, 0.44827E+00_JPRB, 0.43948E+00_JPRB, 0.43185E+00_JPRB, 0.42656E+00_JPRB ,&
&  0.42227E+00_JPRB, 0.41877E+00_JPRB, 0.41480E+00_JPRB, 0.41135E+00_JPRB, 0.40742E+00_JPRB, 0.40039E+00_JPRB, 0.39676E+00_JPRB ,&
&  0.39342E+00_JPRB, 0.39066E+00_JPRB, 0.38795E+00_JPRB, 0.38808E+00_JPRB, 0.38840E+00_JPRB, 0.39116E+00_JPRB, 0.39253E+00_JPRB ,&
&  0.38939E+00_JPRB, 0.38568E+00_JPRB, 0.38283E+00_JPRB, 0.38445E+00_JPRB, 0.38670E+00_JPRB, 0.38825E+00_JPRB, 0.38882E+00_JPRB ,&
&  0.38629E+00_JPRB, 0.38163E+00_JPRB, 0.37681E+00_JPRB, 0.37126E+00_JPRB, 0.36565E+00_JPRB, 0.33998E+00_JPRB, 0.32881E+00_JPRB ,&
&  0.32569E+00_JPRB, 0.33475E+00_JPRB, 0.34696E+00_JPRB, 0.34585E+00_JPRB, 0.34821E+00_JPRB, 0.34840E+00_JPRB, 0.34585E+00_JPRB ,&
&  0.33671E+00_JPRB, 0.32319E+00_JPRB, 0.31426E+00_JPRB, 0.31029E+00_JPRB, 0.30737E+00_JPRB, 0.30873E+00_JPRB, 0.31482E+00_JPRB ,&
&  0.32503E+00_JPRB, 0.33671E+00_JPRB, 0.34554E+00_JPRB, 0.34954E+00_JPRB, 0.34840E+00_JPRB, 0.34531E+00_JPRB, 0.34115E+00_JPRB ,&
&  0.33667E+00_JPRB, 0.34241E+00_JPRB, 0.35666E+00_JPRB, 0.37650E+00_JPRB, 0.37914E+00_JPRB, 0.38427E+00_JPRB, 0.38780E+00_JPRB ,&
&  0.39316E+00_JPRB, 0.39990E+00_JPRB, 0.40574E+00_JPRB, 0.41189E+00_JPRB, 0.41572E+00_JPRB /)
ZBURD(:, 65,  4) = (/ &
&  0.39699E+00_JPRB, 0.40212E+00_JPRB, 0.40906E+00_JPRB, 0.41674E+00_JPRB, 0.42220E+00_JPRB, 0.42467E+00_JPRB, 0.42728E+00_JPRB ,&
&  0.43067E+00_JPRB, 0.43210E+00_JPRB, 0.43319E+00_JPRB, 0.43598E+00_JPRB, 0.44163E+00_JPRB, 0.44466E+00_JPRB, 0.44946E+00_JPRB ,&
&  0.45134E+00_JPRB, 0.44909E+00_JPRB, 0.44220E+00_JPRB, 0.44018E+00_JPRB, 0.44377E+00_JPRB, 0.44858E+00_JPRB, 0.45384E+00_JPRB ,&
&  0.45929E+00_JPRB, 0.46427E+00_JPRB, 0.46662E+00_JPRB, 0.46727E+00_JPRB, 0.46745E+00_JPRB, 0.46731E+00_JPRB, 0.46468E+00_JPRB ,&
&  0.45936E+00_JPRB, 0.41966E+00_JPRB, 0.39312E+00_JPRB, 0.39666E+00_JPRB, 0.40812E+00_JPRB, 0.40236E+00_JPRB, 0.40081E+00_JPRB ,&
&  0.39924E+00_JPRB, 0.39567E+00_JPRB, 0.39036E+00_JPRB, 0.38604E+00_JPRB, 0.38069E+00_JPRB, 0.37377E+00_JPRB, 0.36818E+00_JPRB ,&
&  0.36681E+00_JPRB, 0.37166E+00_JPRB, 0.37314E+00_JPRB, 0.37519E+00_JPRB, 0.37379E+00_JPRB, 0.37562E+00_JPRB, 0.37429E+00_JPRB ,&
&  0.36877E+00_JPRB, 0.36247E+00_JPRB, 0.36033E+00_JPRB, 0.35989E+00_JPRB, 0.36098E+00_JPRB, 0.36363E+00_JPRB, 0.36337E+00_JPRB ,&
&  0.35031E+00_JPRB, 0.33876E+00_JPRB, 0.32020E+00_JPRB, 0.30219E+00_JPRB, 0.28909E+00_JPRB, 0.28315E+00_JPRB, 0.27883E+00_JPRB ,&
&  0.27801E+00_JPRB, 0.28398E+00_JPRB, 0.30097E+00_JPRB, 0.32153E+00_JPRB, 0.33495E+00_JPRB, 0.31664E+00_JPRB, 0.30117E+00_JPRB ,&
&  0.29177E+00_JPRB, 0.28674E+00_JPRB, 0.28249E+00_JPRB, 0.28103E+00_JPRB, 0.28148E+00_JPRB, 0.28371E+00_JPRB, 0.28693E+00_JPRB ,&
&  0.29163E+00_JPRB, 0.29610E+00_JPRB, 0.29888E+00_JPRB, 0.30020E+00_JPRB, 0.30094E+00_JPRB, 0.30170E+00_JPRB, 0.30108E+00_JPRB ,&
&  0.30089E+00_JPRB, 0.30431E+00_JPRB, 0.31118E+00_JPRB, 0.32003E+00_JPRB, 0.32825E+00_JPRB, 0.33744E+00_JPRB, 0.35040E+00_JPRB ,&
&  0.36472E+00_JPRB, 0.37641E+00_JPRB, 0.38068E+00_JPRB, 0.38685E+00_JPRB, 0.39220E+00_JPRB /)
ZBURD(:, 66,  4) = (/ &
&  0.38795E+00_JPRB, 0.39481E+00_JPRB, 0.39986E+00_JPRB, 0.40990E+00_JPRB, 0.41279E+00_JPRB, 0.41569E+00_JPRB, 0.41817E+00_JPRB ,&
&  0.42446E+00_JPRB, 0.42812E+00_JPRB, 0.42597E+00_JPRB, 0.42538E+00_JPRB, 0.42451E+00_JPRB, 0.42318E+00_JPRB, 0.42413E+00_JPRB ,&
&  0.42418E+00_JPRB, 0.42628E+00_JPRB, 0.42389E+00_JPRB, 0.42111E+00_JPRB, 0.42304E+00_JPRB, 0.42787E+00_JPRB, 0.43111E+00_JPRB ,&
&  0.42591E+00_JPRB, 0.42446E+00_JPRB, 0.43299E+00_JPRB, 0.43506E+00_JPRB, 0.43625E+00_JPRB, 0.43822E+00_JPRB, 0.43793E+00_JPRB ,&
&  0.42399E+00_JPRB, 0.39844E+00_JPRB, 0.37088E+00_JPRB, 0.37266E+00_JPRB, 0.38732E+00_JPRB, 0.38147E+00_JPRB, 0.37981E+00_JPRB ,&
&  0.37694E+00_JPRB, 0.37242E+00_JPRB, 0.36813E+00_JPRB, 0.36606E+00_JPRB, 0.36327E+00_JPRB, 0.35856E+00_JPRB, 0.35692E+00_JPRB ,&
&  0.35575E+00_JPRB, 0.36441E+00_JPRB, 0.35955E+00_JPRB, 0.33743E+00_JPRB, 0.32089E+00_JPRB, 0.31186E+00_JPRB, 0.30020E+00_JPRB ,&
&  0.28664E+00_JPRB, 0.27883E+00_JPRB, 0.27768E+00_JPRB, 0.27580E+00_JPRB, 0.27724E+00_JPRB, 0.27836E+00_JPRB, 0.27949E+00_JPRB ,&
&  0.28030E+00_JPRB, 0.28003E+00_JPRB, 0.27540E+00_JPRB, 0.26939E+00_JPRB, 0.26612E+00_JPRB, 0.26609E+00_JPRB, 0.26883E+00_JPRB ,&
&  0.27481E+00_JPRB, 0.28113E+00_JPRB, 0.29559E+00_JPRB, 0.30335E+00_JPRB, 0.29461E+00_JPRB, 0.28932E+00_JPRB, 0.28340E+00_JPRB ,&
&  0.27286E+00_JPRB, 0.26884E+00_JPRB, 0.26552E+00_JPRB, 0.26370E+00_JPRB, 0.26457E+00_JPRB, 0.26790E+00_JPRB, 0.26953E+00_JPRB ,&
&  0.27197E+00_JPRB, 0.27467E+00_JPRB, 0.27576E+00_JPRB, 0.27664E+00_JPRB, 0.27740E+00_JPRB, 0.27935E+00_JPRB, 0.28136E+00_JPRB ,&
&  0.28300E+00_JPRB, 0.28594E+00_JPRB, 0.28951E+00_JPRB, 0.29426E+00_JPRB, 0.29845E+00_JPRB, 0.30093E+00_JPRB, 0.30828E+00_JPRB ,&
&  0.31685E+00_JPRB, 0.33446E+00_JPRB, 0.36312E+00_JPRB, 0.38761E+00_JPRB, 0.38311E+00_JPRB /)
ZBURD(:, 67,  4) = (/ &
&  0.38967E+00_JPRB, 0.39405E+00_JPRB, 0.39478E+00_JPRB, 0.39810E+00_JPRB, 0.40299E+00_JPRB, 0.40867E+00_JPRB, 0.41343E+00_JPRB ,&
&  0.41867E+00_JPRB, 0.41841E+00_JPRB, 0.41332E+00_JPRB, 0.40600E+00_JPRB, 0.39987E+00_JPRB, 0.38304E+00_JPRB, 0.37421E+00_JPRB ,&
&  0.36695E+00_JPRB, 0.36663E+00_JPRB, 0.36714E+00_JPRB, 0.36883E+00_JPRB, 0.36985E+00_JPRB, 0.37488E+00_JPRB, 0.37633E+00_JPRB ,&
&  0.37369E+00_JPRB, 0.36846E+00_JPRB, 0.36315E+00_JPRB, 0.36219E+00_JPRB, 0.36481E+00_JPRB, 0.36672E+00_JPRB, 0.36376E+00_JPRB ,&
&  0.36010E+00_JPRB, 0.35988E+00_JPRB, 0.36047E+00_JPRB, 0.36191E+00_JPRB, 0.37151E+00_JPRB, 0.36645E+00_JPRB, 0.36195E+00_JPRB ,&
&  0.35485E+00_JPRB, 0.34875E+00_JPRB, 0.34455E+00_JPRB, 0.34488E+00_JPRB, 0.34792E+00_JPRB, 0.34725E+00_JPRB, 0.34982E+00_JPRB ,&
&  0.33840E+00_JPRB, 0.32892E+00_JPRB, 0.31399E+00_JPRB, 0.30210E+00_JPRB, 0.29000E+00_JPRB, 0.28173E+00_JPRB, 0.27406E+00_JPRB ,&
&  0.26753E+00_JPRB, 0.26202E+00_JPRB, 0.25685E+00_JPRB, 0.25521E+00_JPRB, 0.25614E+00_JPRB, 0.25618E+00_JPRB, 0.25570E+00_JPRB ,&
&  0.25622E+00_JPRB, 0.25717E+00_JPRB, 0.25789E+00_JPRB, 0.25889E+00_JPRB, 0.26053E+00_JPRB, 0.26322E+00_JPRB, 0.26686E+00_JPRB ,&
&  0.27254E+00_JPRB, 0.27704E+00_JPRB, 0.27946E+00_JPRB, 0.28073E+00_JPRB, 0.27762E+00_JPRB, 0.27223E+00_JPRB, 0.26680E+00_JPRB ,&
&  0.26132E+00_JPRB, 0.25875E+00_JPRB, 0.25643E+00_JPRB, 0.25533E+00_JPRB, 0.25510E+00_JPRB, 0.25596E+00_JPRB, 0.25779E+00_JPRB ,&
&  0.25958E+00_JPRB, 0.26189E+00_JPRB, 0.26341E+00_JPRB, 0.26526E+00_JPRB, 0.26750E+00_JPRB, 0.26945E+00_JPRB, 0.27149E+00_JPRB ,&
&  0.27378E+00_JPRB, 0.27557E+00_JPRB, 0.27864E+00_JPRB, 0.28242E+00_JPRB, 0.28965E+00_JPRB, 0.29810E+00_JPRB, 0.31311E+00_JPRB ,&
&  0.33065E+00_JPRB, 0.36280E+00_JPRB, 0.37286E+00_JPRB, 0.38046E+00_JPRB, 0.38787E+00_JPRB /)
ZBURD(:, 68,  4) = (/ &
&  0.38577E+00_JPRB, 0.38839E+00_JPRB, 0.39234E+00_JPRB, 0.39784E+00_JPRB, 0.40475E+00_JPRB, 0.41108E+00_JPRB, 0.41260E+00_JPRB ,&
&  0.40316E+00_JPRB, 0.39026E+00_JPRB, 0.37198E+00_JPRB, 0.35754E+00_JPRB, 0.34349E+00_JPRB, 0.33299E+00_JPRB, 0.32448E+00_JPRB ,&
&  0.31899E+00_JPRB, 0.31857E+00_JPRB, 0.31892E+00_JPRB, 0.32013E+00_JPRB, 0.31903E+00_JPRB, 0.32004E+00_JPRB, 0.32032E+00_JPRB ,&
&  0.31902E+00_JPRB, 0.31887E+00_JPRB, 0.31879E+00_JPRB, 0.32226E+00_JPRB, 0.32910E+00_JPRB, 0.33540E+00_JPRB, 0.34165E+00_JPRB ,&
&  0.34899E+00_JPRB, 0.35102E+00_JPRB, 0.35521E+00_JPRB, 0.35388E+00_JPRB, 0.35041E+00_JPRB, 0.34593E+00_JPRB, 0.33963E+00_JPRB ,&
&  0.33521E+00_JPRB, 0.33128E+00_JPRB, 0.32999E+00_JPRB, 0.33037E+00_JPRB, 0.33039E+00_JPRB, 0.33261E+00_JPRB, 0.32676E+00_JPRB ,&
&  0.31962E+00_JPRB, 0.30915E+00_JPRB, 0.29697E+00_JPRB, 0.28791E+00_JPRB, 0.27919E+00_JPRB, 0.27293E+00_JPRB, 0.26733E+00_JPRB ,&
&  0.26216E+00_JPRB, 0.25871E+00_JPRB, 0.25511E+00_JPRB, 0.25246E+00_JPRB, 0.25162E+00_JPRB, 0.24976E+00_JPRB, 0.24873E+00_JPRB ,&
&  0.24831E+00_JPRB, 0.24874E+00_JPRB, 0.25043E+00_JPRB, 0.25195E+00_JPRB, 0.25406E+00_JPRB, 0.25644E+00_JPRB, 0.25845E+00_JPRB ,&
&  0.26114E+00_JPRB, 0.26332E+00_JPRB, 0.26359E+00_JPRB, 0.26300E+00_JPRB, 0.26087E+00_JPRB, 0.25804E+00_JPRB, 0.25555E+00_JPRB ,&
&  0.25300E+00_JPRB, 0.25121E+00_JPRB, 0.25056E+00_JPRB, 0.24969E+00_JPRB, 0.25014E+00_JPRB, 0.25131E+00_JPRB, 0.25218E+00_JPRB ,&
&  0.25391E+00_JPRB, 0.25533E+00_JPRB, 0.25700E+00_JPRB, 0.25987E+00_JPRB, 0.26255E+00_JPRB, 0.26492E+00_JPRB, 0.26733E+00_JPRB ,&
&  0.26747E+00_JPRB, 0.26921E+00_JPRB, 0.27158E+00_JPRB, 0.27587E+00_JPRB, 0.28850E+00_JPRB, 0.29729E+00_JPRB, 0.31755E+00_JPRB ,&
&  0.33486E+00_JPRB, 0.36961E+00_JPRB, 0.36936E+00_JPRB, 0.37544E+00_JPRB, 0.38200E+00_JPRB /)
ZBURD(:, 69,  4) = (/ &
&  0.38030E+00_JPRB, 0.38709E+00_JPRB, 0.39388E+00_JPRB, 0.39905E+00_JPRB, 0.40270E+00_JPRB, 0.40583E+00_JPRB, 0.40412E+00_JPRB ,&
&  0.39806E+00_JPRB, 0.38316E+00_JPRB, 0.37182E+00_JPRB, 0.36060E+00_JPRB, 0.35117E+00_JPRB, 0.34213E+00_JPRB, 0.33372E+00_JPRB ,&
&  0.32564E+00_JPRB, 0.31892E+00_JPRB, 0.31332E+00_JPRB, 0.30842E+00_JPRB, 0.30175E+00_JPRB, 0.29587E+00_JPRB, 0.29114E+00_JPRB ,&
&  0.28722E+00_JPRB, 0.28709E+00_JPRB, 0.28943E+00_JPRB, 0.29387E+00_JPRB, 0.30218E+00_JPRB, 0.31051E+00_JPRB, 0.31981E+00_JPRB ,&
&  0.33404E+00_JPRB, 0.33750E+00_JPRB, 0.33643E+00_JPRB, 0.33578E+00_JPRB, 0.33081E+00_JPRB, 0.32463E+00_JPRB, 0.31545E+00_JPRB ,&
&  0.31664E+00_JPRB, 0.31466E+00_JPRB, 0.32203E+00_JPRB, 0.32211E+00_JPRB, 0.32079E+00_JPRB, 0.32109E+00_JPRB, 0.31372E+00_JPRB ,&
&  0.30848E+00_JPRB, 0.29931E+00_JPRB, 0.29220E+00_JPRB, 0.28472E+00_JPRB, 0.27720E+00_JPRB, 0.27275E+00_JPRB, 0.26693E+00_JPRB ,&
&  0.26265E+00_JPRB, 0.25948E+00_JPRB, 0.25535E+00_JPRB, 0.25276E+00_JPRB, 0.25050E+00_JPRB, 0.24815E+00_JPRB, 0.24693E+00_JPRB ,&
&  0.24600E+00_JPRB, 0.24567E+00_JPRB, 0.24581E+00_JPRB, 0.24602E+00_JPRB, 0.24654E+00_JPRB, 0.24748E+00_JPRB, 0.24818E+00_JPRB ,&
&  0.24897E+00_JPRB, 0.24925E+00_JPRB, 0.24888E+00_JPRB, 0.24847E+00_JPRB, 0.24731E+00_JPRB, 0.24655E+00_JPRB, 0.24553E+00_JPRB ,&
&  0.24492E+00_JPRB, 0.24524E+00_JPRB, 0.24553E+00_JPRB, 0.24623E+00_JPRB, 0.24694E+00_JPRB, 0.24851E+00_JPRB, 0.24999E+00_JPRB ,&
&  0.25157E+00_JPRB, 0.25355E+00_JPRB, 0.25516E+00_JPRB, 0.25723E+00_JPRB, 0.25970E+00_JPRB, 0.26134E+00_JPRB, 0.26423E+00_JPRB ,&
&  0.26706E+00_JPRB, 0.27061E+00_JPRB, 0.27672E+00_JPRB, 0.28347E+00_JPRB, 0.29255E+00_JPRB, 0.30361E+00_JPRB, 0.31750E+00_JPRB ,&
&  0.33528E+00_JPRB, 0.36002E+00_JPRB, 0.36159E+00_JPRB, 0.36947E+00_JPRB, 0.37486E+00_JPRB /)
ZBURD(:, 70,  4) = (/ &
&  0.36969E+00_JPRB, 0.37469E+00_JPRB, 0.37894E+00_JPRB, 0.38273E+00_JPRB, 0.38613E+00_JPRB, 0.38709E+00_JPRB, 0.38618E+00_JPRB ,&
&  0.37706E+00_JPRB, 0.37036E+00_JPRB, 0.36213E+00_JPRB, 0.35474E+00_JPRB, 0.34790E+00_JPRB, 0.33963E+00_JPRB, 0.33262E+00_JPRB ,&
&  0.32546E+00_JPRB, 0.31829E+00_JPRB, 0.31153E+00_JPRB, 0.30359E+00_JPRB, 0.29532E+00_JPRB, 0.28739E+00_JPRB, 0.28259E+00_JPRB ,&
&  0.27985E+00_JPRB, 0.27975E+00_JPRB, 0.28280E+00_JPRB, 0.28669E+00_JPRB, 0.29171E+00_JPRB, 0.29815E+00_JPRB, 0.30506E+00_JPRB ,&
&  0.30942E+00_JPRB, 0.31178E+00_JPRB, 0.31127E+00_JPRB, 0.30965E+00_JPRB, 0.30755E+00_JPRB, 0.30582E+00_JPRB, 0.30312E+00_JPRB ,&
&  0.29967E+00_JPRB, 0.29736E+00_JPRB, 0.29676E+00_JPRB, 0.29800E+00_JPRB, 0.29794E+00_JPRB, 0.29619E+00_JPRB, 0.29227E+00_JPRB ,&
&  0.28782E+00_JPRB, 0.28332E+00_JPRB, 0.27865E+00_JPRB, 0.27429E+00_JPRB, 0.27072E+00_JPRB, 0.26695E+00_JPRB, 0.26287E+00_JPRB ,&
&  0.26031E+00_JPRB, 0.25760E+00_JPRB, 0.25462E+00_JPRB, 0.25180E+00_JPRB, 0.24945E+00_JPRB, 0.24764E+00_JPRB, 0.24612E+00_JPRB ,&
&  0.24496E+00_JPRB, 0.24406E+00_JPRB, 0.24336E+00_JPRB, 0.24283E+00_JPRB, 0.24233E+00_JPRB, 0.24153E+00_JPRB, 0.24075E+00_JPRB ,&
&  0.23996E+00_JPRB, 0.23969E+00_JPRB, 0.23921E+00_JPRB, 0.23923E+00_JPRB, 0.23950E+00_JPRB, 0.23966E+00_JPRB, 0.24019E+00_JPRB ,&
&  0.24133E+00_JPRB, 0.24252E+00_JPRB, 0.24313E+00_JPRB, 0.24425E+00_JPRB, 0.24535E+00_JPRB, 0.24677E+00_JPRB, 0.24865E+00_JPRB ,&
&  0.24982E+00_JPRB, 0.25112E+00_JPRB, 0.25264E+00_JPRB, 0.25427E+00_JPRB, 0.25701E+00_JPRB, 0.25999E+00_JPRB, 0.26330E+00_JPRB ,&
&  0.26757E+00_JPRB, 0.27172E+00_JPRB, 0.27646E+00_JPRB, 0.28209E+00_JPRB, 0.29219E+00_JPRB, 0.29893E+00_JPRB, 0.30746E+00_JPRB ,&
&  0.31587E+00_JPRB, 0.32015E+00_JPRB, 0.35466E+00_JPRB, 0.35738E+00_JPRB, 0.36289E+00_JPRB /)
ZBURD(:, 71,  4) = (/ &
&  0.29693E+00_JPRB, 0.30353E+00_JPRB, 0.30785E+00_JPRB, 0.31243E+00_JPRB, 0.31004E+00_JPRB, 0.34859E+00_JPRB, 0.34762E+00_JPRB ,&
&  0.34659E+00_JPRB, 0.33268E+00_JPRB, 0.33062E+00_JPRB, 0.32598E+00_JPRB, 0.32044E+00_JPRB, 0.31433E+00_JPRB, 0.30905E+00_JPRB ,&
&  0.30420E+00_JPRB, 0.29877E+00_JPRB, 0.29437E+00_JPRB, 0.29029E+00_JPRB, 0.28695E+00_JPRB, 0.28399E+00_JPRB, 0.28190E+00_JPRB ,&
&  0.28139E+00_JPRB, 0.28291E+00_JPRB, 0.28612E+00_JPRB, 0.28785E+00_JPRB, 0.28852E+00_JPRB, 0.28909E+00_JPRB, 0.28966E+00_JPRB ,&
&  0.28963E+00_JPRB, 0.28952E+00_JPRB, 0.28828E+00_JPRB, 0.28600E+00_JPRB, 0.28405E+00_JPRB, 0.28257E+00_JPRB, 0.28143E+00_JPRB ,&
&  0.28036E+00_JPRB, 0.27931E+00_JPRB, 0.27819E+00_JPRB, 0.27753E+00_JPRB, 0.27652E+00_JPRB, 0.27506E+00_JPRB, 0.27297E+00_JPRB ,&
&  0.27085E+00_JPRB, 0.26867E+00_JPRB, 0.26604E+00_JPRB, 0.26393E+00_JPRB, 0.26188E+00_JPRB, 0.25981E+00_JPRB, 0.25773E+00_JPRB ,&
&  0.25656E+00_JPRB, 0.25541E+00_JPRB, 0.25397E+00_JPRB, 0.25215E+00_JPRB, 0.25077E+00_JPRB, 0.24931E+00_JPRB, 0.24804E+00_JPRB ,&
&  0.24721E+00_JPRB, 0.24635E+00_JPRB, 0.24546E+00_JPRB, 0.24455E+00_JPRB, 0.24388E+00_JPRB, 0.24311E+00_JPRB, 0.24240E+00_JPRB ,&
&  0.24177E+00_JPRB, 0.24105E+00_JPRB, 0.24087E+00_JPRB, 0.24080E+00_JPRB, 0.24087E+00_JPRB, 0.24093E+00_JPRB, 0.24124E+00_JPRB ,&
&  0.24172E+00_JPRB, 0.24243E+00_JPRB, 0.24331E+00_JPRB, 0.24412E+00_JPRB, 0.24490E+00_JPRB, 0.24572E+00_JPRB, 0.24692E+00_JPRB ,&
&  0.24822E+00_JPRB, 0.24959E+00_JPRB, 0.25102E+00_JPRB, 0.25212E+00_JPRB, 0.25451E+00_JPRB, 0.25676E+00_JPRB, 0.25915E+00_JPRB ,&
&  0.26190E+00_JPRB, 0.26517E+00_JPRB, 0.26869E+00_JPRB, 0.27247E+00_JPRB, 0.27747E+00_JPRB, 0.28057E+00_JPRB, 0.28281E+00_JPRB ,&
&  0.28528E+00_JPRB, 0.28947E+00_JPRB, 0.29096E+00_JPRB, 0.29228E+00_JPRB, 0.29428E+00_JPRB /)
ZBURD(:, 72,  4) = (/ &
&  0.26998E+00_JPRB, 0.27026E+00_JPRB, 0.27034E+00_JPRB, 0.27028E+00_JPRB, 0.26819E+00_JPRB, 0.26846E+00_JPRB, 0.26844E+00_JPRB ,&
&  0.26864E+00_JPRB, 0.26887E+00_JPRB, 0.26950E+00_JPRB, 0.27025E+00_JPRB, 0.27100E+00_JPRB, 0.27169E+00_JPRB, 0.27232E+00_JPRB ,&
&  0.27270E+00_JPRB, 0.27319E+00_JPRB, 0.27186E+00_JPRB, 0.27105E+00_JPRB, 0.27094E+00_JPRB, 0.27104E+00_JPRB, 0.27101E+00_JPRB ,&
&  0.27094E+00_JPRB, 0.27080E+00_JPRB, 0.27032E+00_JPRB, 0.26998E+00_JPRB, 0.26982E+00_JPRB, 0.26962E+00_JPRB, 0.26930E+00_JPRB ,&
&  0.26890E+00_JPRB, 0.26859E+00_JPRB, 0.26821E+00_JPRB, 0.26778E+00_JPRB, 0.26766E+00_JPRB, 0.26749E+00_JPRB, 0.26690E+00_JPRB ,&
&  0.26621E+00_JPRB, 0.26518E+00_JPRB, 0.26425E+00_JPRB, 0.26340E+00_JPRB, 0.26271E+00_JPRB, 0.26237E+00_JPRB, 0.26201E+00_JPRB ,&
&  0.26153E+00_JPRB, 0.26087E+00_JPRB, 0.26023E+00_JPRB, 0.25946E+00_JPRB, 0.25877E+00_JPRB, 0.25814E+00_JPRB, 0.25741E+00_JPRB ,&
&  0.25660E+00_JPRB, 0.25577E+00_JPRB, 0.25492E+00_JPRB, 0.25397E+00_JPRB, 0.25305E+00_JPRB, 0.25212E+00_JPRB, 0.25110E+00_JPRB ,&
&  0.25010E+00_JPRB, 0.24966E+00_JPRB, 0.24917E+00_JPRB, 0.24874E+00_JPRB, 0.24842E+00_JPRB, 0.24818E+00_JPRB, 0.24794E+00_JPRB ,&
&  0.24775E+00_JPRB, 0.24754E+00_JPRB, 0.24744E+00_JPRB, 0.24735E+00_JPRB, 0.24735E+00_JPRB, 0.24741E+00_JPRB, 0.24759E+00_JPRB ,&
&  0.24780E+00_JPRB, 0.24806E+00_JPRB, 0.24841E+00_JPRB, 0.24900E+00_JPRB, 0.24958E+00_JPRB, 0.25026E+00_JPRB, 0.25093E+00_JPRB ,&
&  0.25166E+00_JPRB, 0.25239E+00_JPRB, 0.25313E+00_JPRB, 0.25383E+00_JPRB, 0.25478E+00_JPRB, 0.25570E+00_JPRB, 0.25671E+00_JPRB ,&
&  0.25771E+00_JPRB, 0.25879E+00_JPRB, 0.25986E+00_JPRB, 0.26101E+00_JPRB, 0.26232E+00_JPRB, 0.26271E+00_JPRB, 0.26311E+00_JPRB ,&
&  0.26361E+00_JPRB, 0.26435E+00_JPRB, 0.26501E+00_JPRB, 0.26589E+00_JPRB, 0.26737E+00_JPRB /)
ZBURD(:, 73,  4) = (/ &
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB ,&
&  0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB, 0.25691E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART8',1,ZHOOK_HANDLE)
END SUBROUTINE PART8



SUBROUTINE PART9

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART9',0,ZHOOK_HANDLE)


!-- month=  5
ZBURD(:,  1,  5) = (/ &
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB ,&
&  0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB, 0.83448E+00_JPRB /)
ZBURD(:,  2,  5) = (/ &
&  0.81339E+00_JPRB, 0.81443E+00_JPRB, 0.81528E+00_JPRB, 0.81573E+00_JPRB, 0.81635E+00_JPRB, 0.81692E+00_JPRB, 0.81729E+00_JPRB ,&
&  0.81711E+00_JPRB, 0.81771E+00_JPRB, 0.81879E+00_JPRB, 0.81980E+00_JPRB, 0.82069E+00_JPRB, 0.82149E+00_JPRB, 0.82195E+00_JPRB ,&
&  0.82317E+00_JPRB, 0.82922E+00_JPRB, 0.83356E+00_JPRB, 0.83531E+00_JPRB, 0.83655E+00_JPRB, 0.83752E+00_JPRB, 0.83808E+00_JPRB ,&
&  0.83884E+00_JPRB, 0.83958E+00_JPRB, 0.84010E+00_JPRB, 0.84030E+00_JPRB, 0.84077E+00_JPRB, 0.84130E+00_JPRB, 0.84195E+00_JPRB ,&
&  0.84293E+00_JPRB, 0.84465E+00_JPRB, 0.84684E+00_JPRB, 0.84925E+00_JPRB, 0.85210E+00_JPRB, 0.85589E+00_JPRB, 0.85928E+00_JPRB ,&
&  0.86206E+00_JPRB, 0.86410E+00_JPRB, 0.86397E+00_JPRB, 0.86214E+00_JPRB, 0.85935E+00_JPRB, 0.85676E+00_JPRB, 0.85394E+00_JPRB ,&
&  0.85041E+00_JPRB, 0.84715E+00_JPRB, 0.84451E+00_JPRB, 0.84219E+00_JPRB, 0.84005E+00_JPRB, 0.83835E+00_JPRB, 0.83695E+00_JPRB ,&
&  0.83580E+00_JPRB, 0.83445E+00_JPRB, 0.83320E+00_JPRB, 0.83178E+00_JPRB, 0.83007E+00_JPRB, 0.82769E+00_JPRB, 0.82520E+00_JPRB ,&
&  0.82312E+00_JPRB, 0.82134E+00_JPRB, 0.81989E+00_JPRB, 0.81818E+00_JPRB, 0.81642E+00_JPRB, 0.81546E+00_JPRB, 0.81488E+00_JPRB ,&
&  0.81426E+00_JPRB, 0.81405E+00_JPRB, 0.81437E+00_JPRB, 0.81416E+00_JPRB, 0.81358E+00_JPRB, 0.81269E+00_JPRB, 0.81203E+00_JPRB ,&
&  0.81137E+00_JPRB, 0.81148E+00_JPRB, 0.81148E+00_JPRB, 0.81243E+00_JPRB, 0.81323E+00_JPRB, 0.81423E+00_JPRB, 0.81530E+00_JPRB ,&
&  0.81602E+00_JPRB, 0.81669E+00_JPRB, 0.81706E+00_JPRB, 0.81774E+00_JPRB, 0.81827E+00_JPRB, 0.81831E+00_JPRB, 0.81791E+00_JPRB ,&
&  0.81746E+00_JPRB, 0.81676E+00_JPRB, 0.81594E+00_JPRB, 0.81478E+00_JPRB, 0.81411E+00_JPRB, 0.81369E+00_JPRB, 0.81314E+00_JPRB ,&
&  0.81250E+00_JPRB, 0.81230E+00_JPRB, 0.81216E+00_JPRB, 0.81245E+00_JPRB, 0.81264E+00_JPRB /)
ZBURD(:,  3,  5) = (/ &
&  0.80356E+00_JPRB, 0.80399E+00_JPRB, 0.80417E+00_JPRB, 0.80350E+00_JPRB, 0.80367E+00_JPRB, 0.80338E+00_JPRB, 0.80253E+00_JPRB ,&
&  0.80071E+00_JPRB, 0.80006E+00_JPRB, 0.80017E+00_JPRB, 0.79984E+00_JPRB, 0.79849E+00_JPRB, 0.80216E+00_JPRB, 0.80985E+00_JPRB ,&
&  0.81015E+00_JPRB, 0.80936E+00_JPRB, 0.80690E+00_JPRB, 0.80444E+00_JPRB, 0.80117E+00_JPRB, 0.79708E+00_JPRB, 0.79302E+00_JPRB ,&
&  0.79156E+00_JPRB, 0.78913E+00_JPRB, 0.78571E+00_JPRB, 0.78131E+00_JPRB, 0.78044E+00_JPRB, 0.77950E+00_JPRB, 0.77801E+00_JPRB ,&
&  0.77711E+00_JPRB, 0.77454E+00_JPRB, 0.77209E+00_JPRB, 0.77128E+00_JPRB, 0.77576E+00_JPRB, 0.78014E+00_JPRB, 0.78521E+00_JPRB ,&
&  0.79038E+00_JPRB, 0.79771E+00_JPRB, 0.80594E+00_JPRB, 0.81146E+00_JPRB, 0.81362E+00_JPRB, 0.81933E+00_JPRB, 0.82374E+00_JPRB ,&
&  0.82492E+00_JPRB, 0.82275E+00_JPRB, 0.82154E+00_JPRB, 0.82289E+00_JPRB, 0.82443E+00_JPRB, 0.82539E+00_JPRB, 0.82816E+00_JPRB ,&
&  0.83141E+00_JPRB, 0.83409E+00_JPRB, 0.83541E+00_JPRB, 0.83727E+00_JPRB, 0.83743E+00_JPRB, 0.83602E+00_JPRB, 0.83311E+00_JPRB ,&
&  0.83052E+00_JPRB, 0.82680E+00_JPRB, 0.82249E+00_JPRB, 0.81784E+00_JPRB, 0.81303E+00_JPRB, 0.80737E+00_JPRB, 0.80038E+00_JPRB ,&
&  0.79484E+00_JPRB, 0.79153E+00_JPRB, 0.79094E+00_JPRB, 0.78795E+00_JPRB, 0.78569E+00_JPRB, 0.78304E+00_JPRB, 0.78466E+00_JPRB ,&
&  0.78620E+00_JPRB, 0.79051E+00_JPRB, 0.79446E+00_JPRB, 0.80178E+00_JPRB, 0.80793E+00_JPRB, 0.81387E+00_JPRB, 0.81767E+00_JPRB ,&
&  0.82221E+00_JPRB, 0.82476E+00_JPRB, 0.82572E+00_JPRB, 0.82528E+00_JPRB, 0.82585E+00_JPRB, 0.82531E+00_JPRB, 0.82348E+00_JPRB ,&
&  0.82071E+00_JPRB, 0.81917E+00_JPRB, 0.81676E+00_JPRB, 0.81359E+00_JPRB, 0.81086E+00_JPRB, 0.80820E+00_JPRB, 0.80583E+00_JPRB ,&
&  0.80336E+00_JPRB, 0.80219E+00_JPRB, 0.80136E+00_JPRB, 0.80170E+00_JPRB, 0.80252E+00_JPRB /)
ZBURD(:,  4,  5) = (/ &
&  0.77919E+00_JPRB, 0.77885E+00_JPRB, 0.77786E+00_JPRB, 0.77460E+00_JPRB, 0.77315E+00_JPRB, 0.77263E+00_JPRB, 0.77340E+00_JPRB ,&
&  0.77505E+00_JPRB, 0.77801E+00_JPRB, 0.78102E+00_JPRB, 0.78296E+00_JPRB, 0.78421E+00_JPRB, 0.79462E+00_JPRB, 0.79829E+00_JPRB ,&
&  0.79626E+00_JPRB, 0.79103E+00_JPRB, 0.78382E+00_JPRB, 0.77674E+00_JPRB, 0.76781E+00_JPRB, 0.75713E+00_JPRB, 0.74127E+00_JPRB ,&
&  0.73146E+00_JPRB, 0.72161E+00_JPRB, 0.71069E+00_JPRB, 0.70713E+00_JPRB, 0.70345E+00_JPRB, 0.66902E+00_JPRB, 0.66617E+00_JPRB ,&
&  0.69044E+00_JPRB, 0.70119E+00_JPRB, 0.70416E+00_JPRB, 0.70856E+00_JPRB, 0.69088E+00_JPRB, 0.65511E+00_JPRB, 0.62936E+00_JPRB ,&
&  0.58614E+00_JPRB, 0.61373E+00_JPRB, 0.63343E+00_JPRB, 0.64088E+00_JPRB, 0.64128E+00_JPRB, 0.64221E+00_JPRB, 0.67250E+00_JPRB ,&
&  0.74282E+00_JPRB, 0.75751E+00_JPRB, 0.77576E+00_JPRB, 0.79546E+00_JPRB, 0.81712E+00_JPRB, 0.83382E+00_JPRB, 0.85355E+00_JPRB ,&
&  0.86671E+00_JPRB, 0.87733E+00_JPRB, 0.87987E+00_JPRB, 0.88062E+00_JPRB, 0.87254E+00_JPRB, 0.86605E+00_JPRB, 0.85900E+00_JPRB ,&
&  0.85738E+00_JPRB, 0.85663E+00_JPRB, 0.85546E+00_JPRB, 0.85073E+00_JPRB, 0.84143E+00_JPRB, 0.83156E+00_JPRB, 0.81999E+00_JPRB ,&
&  0.80863E+00_JPRB, 0.79344E+00_JPRB, 0.78290E+00_JPRB, 0.77322E+00_JPRB, 0.77144E+00_JPRB, 0.76750E+00_JPRB, 0.76998E+00_JPRB ,&
&  0.77226E+00_JPRB, 0.77830E+00_JPRB, 0.78437E+00_JPRB, 0.79567E+00_JPRB, 0.80487E+00_JPRB, 0.81298E+00_JPRB, 0.81728E+00_JPRB ,&
&  0.82104E+00_JPRB, 0.82205E+00_JPRB, 0.82163E+00_JPRB, 0.81855E+00_JPRB, 0.81645E+00_JPRB, 0.81422E+00_JPRB, 0.81117E+00_JPRB ,&
&  0.80812E+00_JPRB, 0.80498E+00_JPRB, 0.80215E+00_JPRB, 0.79887E+00_JPRB, 0.79532E+00_JPRB, 0.79034E+00_JPRB, 0.78499E+00_JPRB ,&
&  0.77893E+00_JPRB, 0.77582E+00_JPRB, 0.77396E+00_JPRB, 0.77511E+00_JPRB, 0.77621E+00_JPRB /)
ZBURD(:,  5,  5) = (/ &
&  0.77517E+00_JPRB, 0.76728E+00_JPRB, 0.76269E+00_JPRB, 0.75922E+00_JPRB, 0.75813E+00_JPRB, 0.75796E+00_JPRB, 0.75930E+00_JPRB ,&
&  0.76250E+00_JPRB, 0.76763E+00_JPRB, 0.77183E+00_JPRB, 0.77430E+00_JPRB, 0.77867E+00_JPRB, 0.78567E+00_JPRB, 0.78774E+00_JPRB ,&
&  0.78055E+00_JPRB, 0.77049E+00_JPRB, 0.75959E+00_JPRB, 0.74863E+00_JPRB, 0.73818E+00_JPRB, 0.72994E+00_JPRB, 0.71747E+00_JPRB ,&
&  0.70940E+00_JPRB, 0.69796E+00_JPRB, 0.65640E+00_JPRB, 0.66014E+00_JPRB, 0.65196E+00_JPRB, 0.65108E+00_JPRB, 0.65567E+00_JPRB ,&
&  0.64531E+00_JPRB, 0.64419E+00_JPRB, 0.62979E+00_JPRB, 0.60593E+00_JPRB, 0.58932E+00_JPRB, 0.56477E+00_JPRB, 0.56188E+00_JPRB ,&
&  0.55676E+00_JPRB, 0.56179E+00_JPRB, 0.57476E+00_JPRB, 0.58098E+00_JPRB, 0.58070E+00_JPRB, 0.58429E+00_JPRB, 0.60237E+00_JPRB ,&
&  0.64158E+00_JPRB, 0.69704E+00_JPRB, 0.77449E+00_JPRB, 0.81772E+00_JPRB, 0.85730E+00_JPRB, 0.89378E+00_JPRB, 0.93088E+00_JPRB ,&
&  0.96076E+00_JPRB, 0.98780E+00_JPRB, 0.10000E+01_JPRB, 0.10075E+01_JPRB, 0.10015E+01_JPRB, 0.98114E+00_JPRB, 0.95109E+00_JPRB ,&
&  0.91833E+00_JPRB, 0.89455E+00_JPRB, 0.87816E+00_JPRB, 0.87372E+00_JPRB, 0.87393E+00_JPRB, 0.87519E+00_JPRB, 0.87769E+00_JPRB ,&
&  0.87773E+00_JPRB, 0.86061E+00_JPRB, 0.84586E+00_JPRB, 0.82416E+00_JPRB, 0.81414E+00_JPRB, 0.80568E+00_JPRB, 0.80631E+00_JPRB ,&
&  0.80959E+00_JPRB, 0.82150E+00_JPRB, 0.83717E+00_JPRB, 0.84344E+00_JPRB, 0.85386E+00_JPRB, 0.86749E+00_JPRB, 0.86695E+00_JPRB ,&
&  0.86582E+00_JPRB, 0.86049E+00_JPRB, 0.85159E+00_JPRB, 0.84157E+00_JPRB, 0.83021E+00_JPRB, 0.81923E+00_JPRB, 0.80861E+00_JPRB ,&
&  0.80185E+00_JPRB, 0.79771E+00_JPRB, 0.79929E+00_JPRB, 0.80149E+00_JPRB, 0.80585E+00_JPRB, 0.80654E+00_JPRB, 0.80338E+00_JPRB ,&
&  0.79475E+00_JPRB, 0.78800E+00_JPRB, 0.78200E+00_JPRB, 0.78141E+00_JPRB, 0.77957E+00_JPRB /)
ZBURD(:,  6,  5) = (/ &
&  0.77932E+00_JPRB, 0.77369E+00_JPRB, 0.77623E+00_JPRB, 0.78104E+00_JPRB, 0.78543E+00_JPRB, 0.78747E+00_JPRB, 0.78073E+00_JPRB ,&
&  0.77551E+00_JPRB, 0.77236E+00_JPRB, 0.77027E+00_JPRB, 0.77015E+00_JPRB, 0.77258E+00_JPRB, 0.77007E+00_JPRB, 0.76319E+00_JPRB ,&
&  0.75552E+00_JPRB, 0.75019E+00_JPRB, 0.74646E+00_JPRB, 0.74378E+00_JPRB, 0.74416E+00_JPRB, 0.74238E+00_JPRB, 0.73836E+00_JPRB ,&
&  0.73456E+00_JPRB, 0.71878E+00_JPRB, 0.68684E+00_JPRB, 0.67666E+00_JPRB, 0.66533E+00_JPRB, 0.66759E+00_JPRB, 0.66890E+00_JPRB ,&
&  0.66932E+00_JPRB, 0.59462E+00_JPRB, 0.60312E+00_JPRB, 0.58166E+00_JPRB, 0.56634E+00_JPRB, 0.56259E+00_JPRB, 0.54655E+00_JPRB ,&
&  0.53581E+00_JPRB, 0.52180E+00_JPRB, 0.52016E+00_JPRB, 0.52697E+00_JPRB, 0.52769E+00_JPRB, 0.54344E+00_JPRB, 0.56476E+00_JPRB ,&
&  0.59863E+00_JPRB, 0.74420E+00_JPRB, 0.81154E+00_JPRB, 0.85604E+00_JPRB, 0.89425E+00_JPRB, 0.93870E+00_JPRB, 0.98938E+00_JPRB ,&
&  0.10332E+01_JPRB, 0.10852E+01_JPRB, 0.11199E+01_JPRB, 0.11298E+01_JPRB, 0.11667E+01_JPRB, 0.11724E+01_JPRB, 0.11548E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.10351E+01_JPRB, 0.97396E+00_JPRB, 0.93553E+00_JPRB, 0.90804E+00_JPRB, 0.89446E+00_JPRB, 0.89138E+00_JPRB ,&
&  0.90657E+00_JPRB, 0.90939E+00_JPRB, 0.90510E+00_JPRB, 0.88215E+00_JPRB, 0.86757E+00_JPRB, 0.85554E+00_JPRB, 0.85281E+00_JPRB ,&
&  0.85629E+00_JPRB, 0.87001E+00_JPRB, 0.89480E+00_JPRB, 0.91371E+00_JPRB, 0.92498E+00_JPRB, 0.92699E+00_JPRB, 0.93236E+00_JPRB ,&
&  0.92661E+00_JPRB, 0.91266E+00_JPRB, 0.89803E+00_JPRB, 0.88370E+00_JPRB, 0.86905E+00_JPRB, 0.85806E+00_JPRB, 0.85268E+00_JPRB ,&
&  0.85048E+00_JPRB, 0.84608E+00_JPRB, 0.84149E+00_JPRB, 0.83862E+00_JPRB, 0.83874E+00_JPRB, 0.83798E+00_JPRB, 0.83538E+00_JPRB ,&
&  0.82466E+00_JPRB, 0.81296E+00_JPRB, 0.80189E+00_JPRB, 0.79739E+00_JPRB, 0.79058E+00_JPRB /)
ZBURD(:,  7,  5) = (/ &
&  0.75897E+00_JPRB, 0.76238E+00_JPRB, 0.78582E+00_JPRB, 0.80680E+00_JPRB, 0.81805E+00_JPRB, 0.81681E+00_JPRB, 0.80465E+00_JPRB ,&
&  0.79506E+00_JPRB, 0.78819E+00_JPRB, 0.77969E+00_JPRB, 0.77142E+00_JPRB, 0.76120E+00_JPRB, 0.75147E+00_JPRB, 0.74458E+00_JPRB ,&
&  0.73970E+00_JPRB, 0.73885E+00_JPRB, 0.73910E+00_JPRB, 0.73908E+00_JPRB, 0.74058E+00_JPRB, 0.74319E+00_JPRB, 0.74252E+00_JPRB ,&
&  0.74195E+00_JPRB, 0.73015E+00_JPRB, 0.70460E+00_JPRB, 0.68626E+00_JPRB, 0.68122E+00_JPRB, 0.67114E+00_JPRB, 0.68070E+00_JPRB ,&
&  0.68189E+00_JPRB, 0.68072E+00_JPRB, 0.67550E+00_JPRB, 0.67197E+00_JPRB, 0.64104E+00_JPRB, 0.53176E+00_JPRB, 0.53291E+00_JPRB ,&
&  0.51550E+00_JPRB, 0.49416E+00_JPRB, 0.48912E+00_JPRB, 0.48838E+00_JPRB, 0.49106E+00_JPRB, 0.51790E+00_JPRB, 0.55084E+00_JPRB ,&
&  0.58366E+00_JPRB, 0.74788E+00_JPRB, 0.79702E+00_JPRB, 0.84967E+00_JPRB, 0.89769E+00_JPRB, 0.95793E+00_JPRB, 0.10260E+01_JPRB ,&
&  0.10788E+01_JPRB, 0.11370E+01_JPRB, 0.11960E+01_JPRB, 0.12497E+01_JPRB, 0.12979E+01_JPRB, 0.13388E+01_JPRB, 0.13759E+01_JPRB ,&
&  0.13802E+01_JPRB, 0.12518E+01_JPRB, 0.11619E+01_JPRB, 0.11038E+01_JPRB, 0.10404E+01_JPRB, 0.99777E+00_JPRB, 0.97125E+00_JPRB ,&
&  0.97549E+00_JPRB, 0.95973E+00_JPRB, 0.96455E+00_JPRB, 0.95078E+00_JPRB, 0.94168E+00_JPRB, 0.93370E+00_JPRB, 0.92144E+00_JPRB ,&
&  0.90738E+00_JPRB, 0.91283E+00_JPRB, 0.94458E+00_JPRB, 0.95702E+00_JPRB, 0.97632E+00_JPRB, 0.97155E+00_JPRB, 0.95145E+00_JPRB ,&
&  0.91880E+00_JPRB, 0.90915E+00_JPRB, 0.89090E+00_JPRB, 0.87915E+00_JPRB, 0.86740E+00_JPRB, 0.86779E+00_JPRB, 0.87100E+00_JPRB ,&
&  0.87086E+00_JPRB, 0.86384E+00_JPRB, 0.85593E+00_JPRB, 0.85088E+00_JPRB, 0.84596E+00_JPRB, 0.83705E+00_JPRB, 0.82659E+00_JPRB ,&
&  0.81448E+00_JPRB, 0.80664E+00_JPRB, 0.79221E+00_JPRB, 0.78082E+00_JPRB, 0.77134E+00_JPRB /)
ZBURD(:,  8,  5) = (/ &
&  0.74413E+00_JPRB, 0.76479E+00_JPRB, 0.79982E+00_JPRB, 0.82480E+00_JPRB, 0.83579E+00_JPRB, 0.82315E+00_JPRB, 0.80395E+00_JPRB ,&
&  0.78757E+00_JPRB, 0.77613E+00_JPRB, 0.76546E+00_JPRB, 0.75730E+00_JPRB, 0.74856E+00_JPRB, 0.74016E+00_JPRB, 0.73394E+00_JPRB ,&
&  0.72991E+00_JPRB, 0.71659E+00_JPRB, 0.71270E+00_JPRB, 0.71690E+00_JPRB, 0.72026E+00_JPRB, 0.72739E+00_JPRB, 0.73550E+00_JPRB ,&
&  0.73648E+00_JPRB, 0.73086E+00_JPRB, 0.72561E+00_JPRB, 0.70743E+00_JPRB, 0.68870E+00_JPRB, 0.68273E+00_JPRB, 0.69310E+00_JPRB ,&
&  0.68123E+00_JPRB, 0.65348E+00_JPRB, 0.67972E+00_JPRB, 0.69445E+00_JPRB, 0.65254E+00_JPRB, 0.60910E+00_JPRB, 0.54405E+00_JPRB ,&
&  0.52070E+00_JPRB, 0.48886E+00_JPRB, 0.47939E+00_JPRB, 0.47643E+00_JPRB, 0.48701E+00_JPRB, 0.51455E+00_JPRB, 0.56868E+00_JPRB ,&
&  0.63723E+00_JPRB, 0.76314E+00_JPRB, 0.80670E+00_JPRB, 0.86755E+00_JPRB, 0.92440E+00_JPRB, 0.97610E+00_JPRB, 0.10183E+01_JPRB ,&
&  0.10857E+01_JPRB, 0.11868E+01_JPRB, 0.12825E+01_JPRB, 0.13548E+01_JPRB, 0.14415E+01_JPRB, 0.15355E+01_JPRB, 0.16697E+01_JPRB ,&
&  0.18349E+01_JPRB, 0.15773E+01_JPRB, 0.14266E+01_JPRB, 0.13290E+01_JPRB, 0.12560E+01_JPRB, 0.12299E+01_JPRB, 0.11475E+01_JPRB ,&
&  0.11223E+01_JPRB, 0.11116E+01_JPRB, 0.10912E+01_JPRB, 0.10837E+01_JPRB, 0.10764E+01_JPRB, 0.10665E+01_JPRB, 0.10435E+01_JPRB ,&
&  0.10142E+01_JPRB, 0.10299E+01_JPRB, 0.11826E+01_JPRB, 0.12110E+01_JPRB, 0.11965E+01_JPRB, 0.11169E+01_JPRB, 0.10153E+01_JPRB ,&
&  0.94217E+00_JPRB, 0.90925E+00_JPRB, 0.89259E+00_JPRB, 0.87979E+00_JPRB, 0.87092E+00_JPRB, 0.86705E+00_JPRB, 0.87319E+00_JPRB ,&
&  0.85807E+00_JPRB, 0.85745E+00_JPRB, 0.85943E+00_JPRB, 0.85702E+00_JPRB, 0.84830E+00_JPRB, 0.83278E+00_JPRB, 0.82210E+00_JPRB ,&
&  0.81166E+00_JPRB, 0.80060E+00_JPRB, 0.78493E+00_JPRB, 0.76567E+00_JPRB, 0.75115E+00_JPRB /)
ZBURD(:,  9,  5) = (/ &
&  0.72022E+00_JPRB, 0.75770E+00_JPRB, 0.81077E+00_JPRB, 0.84711E+00_JPRB, 0.84969E+00_JPRB, 0.82145E+00_JPRB, 0.78931E+00_JPRB ,&
&  0.76786E+00_JPRB, 0.75875E+00_JPRB, 0.74700E+00_JPRB, 0.73955E+00_JPRB, 0.73357E+00_JPRB, 0.72754E+00_JPRB, 0.72165E+00_JPRB ,&
&  0.71712E+00_JPRB, 0.71307E+00_JPRB, 0.70690E+00_JPRB, 0.69782E+00_JPRB, 0.69437E+00_JPRB, 0.70202E+00_JPRB, 0.71642E+00_JPRB ,&
&  0.73257E+00_JPRB, 0.73893E+00_JPRB, 0.74162E+00_JPRB, 0.73303E+00_JPRB, 0.71647E+00_JPRB, 0.69775E+00_JPRB, 0.68262E+00_JPRB ,&
&  0.67439E+00_JPRB, 0.67799E+00_JPRB, 0.67505E+00_JPRB, 0.69636E+00_JPRB, 0.68173E+00_JPRB, 0.63144E+00_JPRB, 0.57327E+00_JPRB ,&
&  0.53806E+00_JPRB, 0.49959E+00_JPRB, 0.48298E+00_JPRB, 0.48840E+00_JPRB, 0.52264E+00_JPRB, 0.56861E+00_JPRB, 0.65122E+00_JPRB ,&
&  0.80611E+00_JPRB, 0.88173E+00_JPRB, 0.93797E+00_JPRB, 0.98760E+00_JPRB, 0.10293E+01_JPRB, 0.10796E+01_JPRB, 0.11411E+01_JPRB ,&
&  0.12266E+01_JPRB, 0.13296E+01_JPRB, 0.14349E+01_JPRB, 0.14828E+01_JPRB, 0.15704E+01_JPRB, 0.15634E+01_JPRB, 0.18737E+01_JPRB ,&
&  0.25116E+01_JPRB, 0.22013E+01_JPRB, 0.18974E+01_JPRB, 0.17244E+01_JPRB, 0.16273E+01_JPRB, 0.15303E+01_JPRB, 0.14406E+01_JPRB ,&
&  0.13531E+01_JPRB, 0.13017E+01_JPRB, 0.12767E+01_JPRB, 0.11994E+01_JPRB, 0.11162E+01_JPRB, 0.11029E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.13873E+01_JPRB, 0.16751E+01_JPRB, 0.19685E+01_JPRB, 0.13469E+01_JPRB, 0.11689E+01_JPRB, 0.10473E+01_JPRB, 0.97522E+00_JPRB ,&
&  0.94845E+00_JPRB, 0.94071E+00_JPRB, 0.93863E+00_JPRB, 0.92447E+00_JPRB, 0.90116E+00_JPRB, 0.87978E+00_JPRB, 0.85702E+00_JPRB ,&
&  0.84096E+00_JPRB, 0.83884E+00_JPRB, 0.84288E+00_JPRB, 0.84714E+00_JPRB, 0.85124E+00_JPRB, 0.84589E+00_JPRB, 0.83947E+00_JPRB ,&
&  0.83180E+00_JPRB, 0.81075E+00_JPRB, 0.78628E+00_JPRB, 0.74881E+00_JPRB, 0.72144E+00_JPRB /)
ZBURD(:, 10,  5) = (/ &
&  0.71098E+00_JPRB, 0.75556E+00_JPRB, 0.82406E+00_JPRB, 0.86507E+00_JPRB, 0.86415E+00_JPRB, 0.81811E+00_JPRB, 0.75668E+00_JPRB ,&
&  0.71379E+00_JPRB, 0.68884E+00_JPRB, 0.67951E+00_JPRB, 0.67585E+00_JPRB, 0.67575E+00_JPRB, 0.67661E+00_JPRB, 0.67619E+00_JPRB ,&
&  0.67419E+00_JPRB, 0.67227E+00_JPRB, 0.67035E+00_JPRB, 0.67234E+00_JPRB, 0.67644E+00_JPRB, 0.68360E+00_JPRB, 0.69470E+00_JPRB ,&
&  0.70860E+00_JPRB, 0.71825E+00_JPRB, 0.72163E+00_JPRB, 0.71916E+00_JPRB, 0.71835E+00_JPRB, 0.72453E+00_JPRB, 0.72755E+00_JPRB ,&
&  0.71072E+00_JPRB, 0.68734E+00_JPRB, 0.68955E+00_JPRB, 0.70577E+00_JPRB, 0.69606E+00_JPRB, 0.65374E+00_JPRB, 0.59100E+00_JPRB ,&
&  0.55030E+00_JPRB, 0.52135E+00_JPRB, 0.53682E+00_JPRB, 0.58930E+00_JPRB, 0.67346E+00_JPRB, 0.84706E+00_JPRB, 0.93743E+00_JPRB ,&
&  0.10155E+01_JPRB, 0.10601E+01_JPRB, 0.10879E+01_JPRB, 0.11216E+01_JPRB, 0.11932E+01_JPRB, 0.12791E+01_JPRB, 0.13663E+01_JPRB ,&
&  0.14784E+01_JPRB, 0.16199E+01_JPRB, 0.18104E+01_JPRB, 0.14992E+01_JPRB, 0.15469E+01_JPRB, 0.18425E+01_JPRB, 0.21821E+01_JPRB ,&
&  0.23421E+01_JPRB, 0.21861E+01_JPRB, 0.21020E+01_JPRB, 0.21559E+01_JPRB, 0.19974E+01_JPRB, 0.18542E+01_JPRB, 0.17478E+01_JPRB ,&
&  0.16727E+01_JPRB, 0.16499E+01_JPRB, 0.14969E+01_JPRB, 0.13163E+01_JPRB, 0.12698E+01_JPRB, 0.12618E+01_JPRB, 0.12889E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.14238E+01_JPRB, 0.12600E+01_JPRB, 0.95428E+00_JPRB, 0.87615E+00_JPRB, 0.84189E+00_JPRB, 0.84467E+00_JPRB ,&
&  0.85589E+00_JPRB, 0.86742E+00_JPRB, 0.87822E+00_JPRB, 0.87812E+00_JPRB, 0.86186E+00_JPRB, 0.83962E+00_JPRB, 0.81268E+00_JPRB ,&
&  0.78860E+00_JPRB, 0.77542E+00_JPRB, 0.77551E+00_JPRB, 0.77426E+00_JPRB, 0.79013E+00_JPRB, 0.80660E+00_JPRB, 0.82495E+00_JPRB ,&
&  0.81896E+00_JPRB, 0.79843E+00_JPRB, 0.77764E+00_JPRB, 0.73643E+00_JPRB, 0.70252E+00_JPRB /)
ZBURD(:, 11,  5) = (/ &
&  0.76348E+00_JPRB, 0.80274E+00_JPRB, 0.84421E+00_JPRB, 0.86835E+00_JPRB, 0.86215E+00_JPRB, 0.82882E+00_JPRB, 0.75135E+00_JPRB ,&
&  0.72023E+00_JPRB, 0.68199E+00_JPRB, 0.66270E+00_JPRB, 0.64769E+00_JPRB, 0.63711E+00_JPRB, 0.62852E+00_JPRB, 0.62786E+00_JPRB ,&
&  0.63498E+00_JPRB, 0.64300E+00_JPRB, 0.65050E+00_JPRB, 0.65986E+00_JPRB, 0.66775E+00_JPRB, 0.68206E+00_JPRB, 0.69874E+00_JPRB ,&
&  0.70993E+00_JPRB, 0.71711E+00_JPRB, 0.72418E+00_JPRB, 0.72805E+00_JPRB, 0.73164E+00_JPRB, 0.73700E+00_JPRB, 0.73393E+00_JPRB ,&
&  0.71677E+00_JPRB, 0.69455E+00_JPRB, 0.69783E+00_JPRB, 0.71254E+00_JPRB, 0.70343E+00_JPRB, 0.67451E+00_JPRB, 0.60932E+00_JPRB ,&
&  0.56751E+00_JPRB, 0.58052E+00_JPRB, 0.68080E+00_JPRB, 0.87016E+00_JPRB, 0.93045E+00_JPRB, 0.99098E+00_JPRB, 0.10222E+01_JPRB ,&
&  0.96186E+00_JPRB, 0.95858E+00_JPRB, 0.10296E+01_JPRB, 0.12386E+01_JPRB, 0.12980E+01_JPRB, 0.14028E+01_JPRB, 0.15644E+01_JPRB ,&
&  0.18445E+01_JPRB, 0.20424E+01_JPRB, 0.21648E+01_JPRB, 0.19351E+01_JPRB, 0.23241E+01_JPRB, 0.30358E+01_JPRB, 0.28122E+01_JPRB ,&
&  0.27548E+01_JPRB, 0.26943E+01_JPRB, 0.27677E+01_JPRB, 0.25591E+01_JPRB, 0.22687E+01_JPRB, 0.20986E+01_JPRB, 0.20491E+01_JPRB ,&
&  0.19734E+01_JPRB, 0.18941E+01_JPRB, 0.16171E+01_JPRB, 0.14976E+01_JPRB, 0.14860E+01_JPRB, 0.14257E+01_JPRB, 0.13760E+01_JPRB ,&
&  0.13208E+01_JPRB, 0.12020E+01_JPRB, 0.10911E+01_JPRB, 0.98152E+00_JPRB, 0.90915E+00_JPRB, 0.88761E+00_JPRB, 0.88764E+00_JPRB ,&
&  0.88755E+00_JPRB, 0.88335E+00_JPRB, 0.88120E+00_JPRB, 0.88037E+00_JPRB, 0.86409E+00_JPRB, 0.83480E+00_JPRB, 0.81773E+00_JPRB ,&
&  0.79893E+00_JPRB, 0.77029E+00_JPRB, 0.74295E+00_JPRB, 0.74524E+00_JPRB, 0.75235E+00_JPRB, 0.77555E+00_JPRB, 0.79901E+00_JPRB ,&
&  0.81915E+00_JPRB, 0.82096E+00_JPRB, 0.80793E+00_JPRB, 0.81086E+00_JPRB, 0.77975E+00_JPRB /)
ZBURD(:, 12,  5) = (/ &
&  0.84042E+00_JPRB, 0.83584E+00_JPRB, 0.85698E+00_JPRB, 0.87576E+00_JPRB, 0.88839E+00_JPRB, 0.82747E+00_JPRB, 0.76343E+00_JPRB ,&
&  0.77661E+00_JPRB, 0.70674E+00_JPRB, 0.66116E+00_JPRB, 0.65032E+00_JPRB, 0.64862E+00_JPRB, 0.62943E+00_JPRB, 0.61659E+00_JPRB ,&
&  0.61544E+00_JPRB, 0.63311E+00_JPRB, 0.66300E+00_JPRB, 0.68659E+00_JPRB, 0.71270E+00_JPRB, 0.73536E+00_JPRB, 0.74651E+00_JPRB ,&
&  0.75419E+00_JPRB, 0.76106E+00_JPRB, 0.76461E+00_JPRB, 0.76455E+00_JPRB, 0.75866E+00_JPRB, 0.75646E+00_JPRB, 0.74686E+00_JPRB ,&
&  0.72528E+00_JPRB, 0.70723E+00_JPRB, 0.70686E+00_JPRB, 0.72231E+00_JPRB, 0.72393E+00_JPRB, 0.71448E+00_JPRB, 0.68668E+00_JPRB ,&
&  0.61184E+00_JPRB, 0.67372E+00_JPRB, 0.90329E+00_JPRB, 0.96520E+00_JPRB, 0.10057E+01_JPRB, 0.10609E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.11505E+01_JPRB, 0.11916E+01_JPRB, 0.12330E+01_JPRB, 0.12921E+01_JPRB, 0.13772E+01_JPRB, 0.15251E+01_JPRB, 0.18262E+01_JPRB ,&
&  0.23907E+01_JPRB, 0.20119E+01_JPRB, 0.19331E+01_JPRB, 0.21195E+01_JPRB, 0.30891E+01_JPRB, 0.34075E+01_JPRB, 0.31671E+01_JPRB ,&
&  0.31234E+01_JPRB, 0.30174E+01_JPRB, 0.28814E+01_JPRB, 0.26158E+01_JPRB, 0.24383E+01_JPRB, 0.24274E+01_JPRB, 0.23817E+01_JPRB ,&
&  0.22017E+01_JPRB, 0.20788E+01_JPRB, 0.18030E+01_JPRB, 0.17291E+01_JPRB, 0.17137E+01_JPRB, 0.16056E+01_JPRB, 0.14582E+01_JPRB ,&
&  0.13300E+01_JPRB, 0.12212E+01_JPRB, 0.11361E+01_JPRB, 0.10787E+01_JPRB, 0.10310E+01_JPRB, 0.10014E+01_JPRB, 0.97812E+00_JPRB ,&
&  0.96157E+00_JPRB, 0.94915E+00_JPRB, 0.91108E+00_JPRB, 0.89885E+00_JPRB, 0.88531E+00_JPRB, 0.88217E+00_JPRB, 0.89709E+00_JPRB ,&
&  0.87412E+00_JPRB, 0.85900E+00_JPRB, 0.81756E+00_JPRB, 0.81577E+00_JPRB, 0.83835E+00_JPRB, 0.89037E+00_JPRB, 0.92931E+00_JPRB ,&
&  0.99683E+00_JPRB, 0.96907E+00_JPRB, 0.90717E+00_JPRB, 0.88674E+00_JPRB, 0.85753E+00_JPRB /)
ZBURD(:, 13,  5) = (/ &
&  0.87026E+00_JPRB, 0.87434E+00_JPRB, 0.87803E+00_JPRB, 0.89282E+00_JPRB, 0.90904E+00_JPRB, 0.85721E+00_JPRB, 0.81164E+00_JPRB ,&
&  0.86480E+00_JPRB, 0.77744E+00_JPRB, 0.82419E+00_JPRB, 0.72836E+00_JPRB, 0.68004E+00_JPRB, 0.65417E+00_JPRB, 0.63880E+00_JPRB ,&
&  0.63234E+00_JPRB, 0.66884E+00_JPRB, 0.71491E+00_JPRB, 0.76167E+00_JPRB, 0.83776E+00_JPRB, 0.82520E+00_JPRB, 0.80973E+00_JPRB ,&
&  0.79560E+00_JPRB, 0.78665E+00_JPRB, 0.80121E+00_JPRB, 0.79442E+00_JPRB, 0.78767E+00_JPRB, 0.77859E+00_JPRB, 0.75584E+00_JPRB ,&
&  0.70338E+00_JPRB, 0.68727E+00_JPRB, 0.70524E+00_JPRB, 0.72224E+00_JPRB, 0.73811E+00_JPRB, 0.74143E+00_JPRB, 0.74987E+00_JPRB ,&
&  0.77097E+00_JPRB, 0.79288E+00_JPRB, 0.93386E+00_JPRB, 0.96882E+00_JPRB, 0.10125E+01_JPRB, 0.10432E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.11304E+01_JPRB, 0.11840E+01_JPRB, 0.12216E+01_JPRB, 0.12892E+01_JPRB, 0.14106E+01_JPRB, 0.16960E+01_JPRB, 0.21978E+01_JPRB ,&
&  0.28196E+01_JPRB, 0.21638E+01_JPRB, 0.26877E+01_JPRB, 0.27657E+01_JPRB, 0.31943E+01_JPRB, 0.35541E+01_JPRB, 0.36581E+01_JPRB ,&
&  0.40977E+01_JPRB, 0.36962E+01_JPRB, 0.33424E+01_JPRB, 0.29468E+01_JPRB, 0.26190E+01_JPRB, 0.26006E+01_JPRB, 0.25719E+01_JPRB ,&
&  0.26296E+01_JPRB, 0.24823E+01_JPRB, 0.21450E+01_JPRB, 0.19405E+01_JPRB, 0.18597E+01_JPRB, 0.18106E+01_JPRB, 0.16607E+01_JPRB ,&
&  0.15774E+01_JPRB, 0.15043E+01_JPRB, 0.14061E+01_JPRB, 0.12917E+01_JPRB, 0.12145E+01_JPRB, 0.11736E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10887E+01_JPRB, 0.10511E+01_JPRB, 0.94200E+00_JPRB, 0.89789E+00_JPRB, 0.86476E+00_JPRB, 0.82946E+00_JPRB, 0.84090E+00_JPRB ,&
&  0.86107E+00_JPRB, 0.85200E+00_JPRB, 0.90549E+00_JPRB, 0.96844E+00_JPRB, 0.98658E+00_JPRB, 0.99244E+00_JPRB, 0.10940E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.10037E+01_JPRB, 0.99925E+00_JPRB, 0.93380E+00_JPRB, 0.89124E+00_JPRB /)
ZBURD(:, 14,  5) = (/ &
&  0.91683E+00_JPRB, 0.90463E+00_JPRB, 0.89800E+00_JPRB, 0.90940E+00_JPRB, 0.92463E+00_JPRB, 0.92751E+00_JPRB, 0.88814E+00_JPRB ,&
&  0.93558E+00_JPRB, 0.90520E+00_JPRB, 0.88685E+00_JPRB, 0.87560E+00_JPRB, 0.85743E+00_JPRB, 0.75707E+00_JPRB, 0.65699E+00_JPRB ,&
&  0.64754E+00_JPRB, 0.68845E+00_JPRB, 0.77490E+00_JPRB, 0.83279E+00_JPRB, 0.89525E+00_JPRB, 0.88836E+00_JPRB, 0.87402E+00_JPRB ,&
&  0.86055E+00_JPRB, 0.82992E+00_JPRB, 0.82871E+00_JPRB, 0.82322E+00_JPRB, 0.80961E+00_JPRB, 0.81585E+00_JPRB, 0.82543E+00_JPRB ,&
&  0.75853E+00_JPRB, 0.71262E+00_JPRB, 0.70240E+00_JPRB, 0.68514E+00_JPRB, 0.74840E+00_JPRB, 0.77024E+00_JPRB, 0.78734E+00_JPRB ,&
&  0.81343E+00_JPRB, 0.85141E+00_JPRB, 0.91316E+00_JPRB, 0.96037E+00_JPRB, 0.99214E+00_JPRB, 0.10129E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10794E+01_JPRB, 0.11308E+01_JPRB, 0.12004E+01_JPRB, 0.13225E+01_JPRB, 0.15182E+01_JPRB, 0.17792E+01_JPRB, 0.25434E+01_JPRB ,&
&  0.28109E+01_JPRB, 0.31763E+01_JPRB, 0.36370E+01_JPRB, 0.34186E+01_JPRB, 0.37680E+01_JPRB, 0.40196E+01_JPRB, 0.39319E+01_JPRB ,&
&  0.39828E+01_JPRB, 0.36091E+01_JPRB, 0.35647E+01_JPRB, 0.35373E+01_JPRB, 0.35705E+01_JPRB, 0.33429E+01_JPRB, 0.31067E+01_JPRB ,&
&  0.32704E+01_JPRB, 0.35804E+01_JPRB, 0.29225E+01_JPRB, 0.26223E+01_JPRB, 0.23639E+01_JPRB, 0.22765E+01_JPRB, 0.21381E+01_JPRB ,&
&  0.21041E+01_JPRB, 0.20922E+01_JPRB, 0.19876E+01_JPRB, 0.18440E+01_JPRB, 0.15892E+01_JPRB, 0.14702E+01_JPRB, 0.13353E+01_JPRB ,&
&  0.12610E+01_JPRB, 0.11490E+01_JPRB, 0.99541E+00_JPRB, 0.90271E+00_JPRB, 0.83751E+00_JPRB, 0.79027E+00_JPRB, 0.81779E+00_JPRB ,&
&  0.90830E+00_JPRB, 0.98285E+00_JPRB, 0.11533E+01_JPRB, 0.11826E+01_JPRB, 0.12002E+01_JPRB, 0.12283E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.99516E+00_JPRB, 0.10774E+01_JPRB, 0.10319E+01_JPRB, 0.97958E+00_JPRB, 0.93830E+00_JPRB /)
ZBURD(:, 15,  5) = (/ &
&  0.10008E+01_JPRB, 0.97785E+00_JPRB, 0.95467E+00_JPRB, 0.94972E+00_JPRB, 0.95730E+00_JPRB, 0.95151E+00_JPRB, 0.95132E+00_JPRB ,&
&  0.96571E+00_JPRB, 0.95611E+00_JPRB, 0.92595E+00_JPRB, 0.90721E+00_JPRB, 0.88646E+00_JPRB, 0.88246E+00_JPRB, 0.78863E+00_JPRB ,&
&  0.74171E+00_JPRB, 0.83709E+00_JPRB, 0.82861E+00_JPRB, 0.91497E+00_JPRB, 0.10404E+01_JPRB, 0.99829E+00_JPRB, 0.99408E+00_JPRB ,&
&  0.95654E+00_JPRB, 0.89039E+00_JPRB, 0.87341E+00_JPRB, 0.88970E+00_JPRB, 0.92223E+00_JPRB, 0.99965E+00_JPRB, 0.10510E+01_JPRB ,&
&  0.93840E+00_JPRB, 0.78491E+00_JPRB, 0.70678E+00_JPRB, 0.70044E+00_JPRB, 0.76048E+00_JPRB, 0.81472E+00_JPRB, 0.84254E+00_JPRB ,&
&  0.86430E+00_JPRB, 0.88852E+00_JPRB, 0.92623E+00_JPRB, 0.97695E+00_JPRB, 0.99799E+00_JPRB, 0.10188E+01_JPRB, 0.10494E+01_JPRB ,&
&  0.10886E+01_JPRB, 0.11548E+01_JPRB, 0.12455E+01_JPRB, 0.14014E+01_JPRB, 0.14804E+01_JPRB, 0.19142E+01_JPRB, 0.26460E+01_JPRB ,&
&  0.30555E+01_JPRB, 0.33933E+01_JPRB, 0.39290E+01_JPRB, 0.46477E+01_JPRB, 0.49645E+01_JPRB, 0.46924E+01_JPRB, 0.43771E+01_JPRB ,&
&  0.42314E+01_JPRB, 0.38372E+01_JPRB, 0.43122E+01_JPRB, 0.40525E+01_JPRB, 0.38608E+01_JPRB, 0.39042E+01_JPRB, 0.37638E+01_JPRB ,&
&  0.37290E+01_JPRB, 0.37750E+01_JPRB, 0.36784E+01_JPRB, 0.35666E+01_JPRB, 0.32787E+01_JPRB, 0.32012E+01_JPRB, 0.29701E+01_JPRB ,&
&  0.29844E+01_JPRB, 0.29191E+01_JPRB, 0.24758E+01_JPRB, 0.20729E+01_JPRB, 0.16836E+01_JPRB, 0.16165E+01_JPRB, 0.14603E+01_JPRB ,&
&  0.13392E+01_JPRB, 0.11785E+01_JPRB, 0.10691E+01_JPRB, 0.94160E+00_JPRB, 0.89311E+00_JPRB, 0.89609E+00_JPRB, 0.92054E+00_JPRB ,&
&  0.10360E+01_JPRB, 0.12295E+01_JPRB, 0.12987E+01_JPRB, 0.13167E+01_JPRB, 0.13386E+01_JPRB, 0.13747E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.11142E+01_JPRB, 0.10850E+01_JPRB, 0.10464E+01_JPRB, 0.10172E+01_JPRB /)
ZBURD(:, 16,  5) = (/ &
&  0.11270E+01_JPRB, 0.10855E+01_JPRB, 0.10459E+01_JPRB, 0.10178E+01_JPRB, 0.99889E+00_JPRB, 0.98531E+00_JPRB, 0.99480E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.98986E+00_JPRB, 0.96400E+00_JPRB, 0.94301E+00_JPRB, 0.92238E+00_JPRB, 0.92235E+00_JPRB, 0.94988E+00_JPRB ,&
&  0.84289E+00_JPRB, 0.10172E+01_JPRB, 0.87669E+00_JPRB, 0.95360E+00_JPRB, 0.13673E+01_JPRB, 0.11733E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.10120E+01_JPRB, 0.10073E+01_JPRB, 0.11170E+01_JPRB, 0.11996E+01_JPRB, 0.13080E+01_JPRB, 0.13328E+01_JPRB ,&
&  0.11717E+01_JPRB, 0.94092E+00_JPRB, 0.82727E+00_JPRB, 0.78753E+00_JPRB, 0.82648E+00_JPRB, 0.86045E+00_JPRB, 0.94045E+00_JPRB ,&
&  0.97422E+00_JPRB, 0.99289E+00_JPRB, 0.10097E+01_JPRB, 0.10440E+01_JPRB, 0.10627E+01_JPRB, 0.10920E+01_JPRB, 0.11418E+01_JPRB ,&
&  0.12072E+01_JPRB, 0.12952E+01_JPRB, 0.13847E+01_JPRB, 0.15430E+01_JPRB, 0.16481E+01_JPRB, 0.20417E+01_JPRB, 0.28543E+01_JPRB ,&
&  0.34502E+01_JPRB, 0.37410E+01_JPRB, 0.45963E+01_JPRB, 0.62595E+01_JPRB, 0.67846E+01_JPRB, 0.65277E+01_JPRB, 0.58304E+01_JPRB ,&
&  0.51588E+01_JPRB, 0.45193E+01_JPRB, 0.44513E+01_JPRB, 0.42739E+01_JPRB, 0.40075E+01_JPRB, 0.42080E+01_JPRB, 0.41327E+01_JPRB ,&
&  0.39275E+01_JPRB, 0.38946E+01_JPRB, 0.38395E+01_JPRB, 0.38640E+01_JPRB, 0.36510E+01_JPRB, 0.36615E+01_JPRB, 0.35345E+01_JPRB ,&
&  0.35551E+01_JPRB, 0.32196E+01_JPRB, 0.23676E+01_JPRB, 0.19489E+01_JPRB, 0.15574E+01_JPRB, 0.14594E+01_JPRB, 0.18109E+01_JPRB ,&
&  0.15862E+01_JPRB, 0.14345E+01_JPRB, 0.13175E+01_JPRB, 0.12031E+01_JPRB, 0.11900E+01_JPRB, 0.13491E+01_JPRB, 0.12270E+01_JPRB ,&
&  0.11876E+01_JPRB, 0.13443E+01_JPRB, 0.15321E+01_JPRB, 0.16399E+01_JPRB, 0.16860E+01_JPRB, 0.16450E+01_JPRB, 0.13845E+01_JPRB ,&
&  0.13268E+01_JPRB, 0.12654E+01_JPRB, 0.12268E+01_JPRB, 0.11801E+01_JPRB, 0.11499E+01_JPRB /)
ZBURD(:, 17,  5) = (/ &
&  0.13286E+01_JPRB, 0.12759E+01_JPRB, 0.12127E+01_JPRB, 0.11366E+01_JPRB, 0.10894E+01_JPRB, 0.10726E+01_JPRB, 0.10775E+01_JPRB ,&
&  0.10691E+01_JPRB, 0.10321E+01_JPRB, 0.10065E+01_JPRB, 0.98758E+00_JPRB, 0.98312E+00_JPRB, 0.99276E+00_JPRB, 0.10129E+01_JPRB ,&
&  0.10944E+01_JPRB, 0.13859E+01_JPRB, 0.11594E+01_JPRB, 0.98145E+00_JPRB, 0.11438E+01_JPRB, 0.10718E+01_JPRB, 0.11239E+01_JPRB ,&
&  0.11002E+01_JPRB, 0.12874E+01_JPRB, 0.13071E+01_JPRB, 0.14340E+01_JPRB, 0.15948E+01_JPRB, 0.18595E+01_JPRB, 0.19202E+01_JPRB ,&
&  0.17385E+01_JPRB, 0.14245E+01_JPRB, 0.11670E+01_JPRB, 0.10906E+01_JPRB, 0.11333E+01_JPRB, 0.10341E+01_JPRB, 0.10923E+01_JPRB ,&
&  0.11044E+01_JPRB, 0.11200E+01_JPRB, 0.11350E+01_JPRB, 0.11443E+01_JPRB, 0.11792E+01_JPRB, 0.12384E+01_JPRB, 0.13118E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.14721E+01_JPRB, 0.15102E+01_JPRB, 0.15735E+01_JPRB, 0.17445E+01_JPRB, 0.20332E+01_JPRB, 0.23283E+01_JPRB ,&
&  0.29699E+01_JPRB, 0.35147E+01_JPRB, 0.44350E+01_JPRB, 0.60871E+01_JPRB, 0.68807E+01_JPRB, 0.70743E+01_JPRB, 0.68359E+01_JPRB ,&
&  0.62974E+01_JPRB, 0.57678E+01_JPRB, 0.56529E+01_JPRB, 0.51611E+01_JPRB, 0.49000E+01_JPRB, 0.48914E+01_JPRB, 0.47721E+01_JPRB ,&
&  0.44589E+01_JPRB, 0.42105E+01_JPRB, 0.41457E+01_JPRB, 0.39854E+01_JPRB, 0.38786E+01_JPRB, 0.37105E+01_JPRB, 0.37461E+01_JPRB ,&
&  0.41099E+01_JPRB, 0.26585E+01_JPRB, 0.20058E+01_JPRB, 0.21293E+01_JPRB, 0.17075E+01_JPRB, 0.15077E+01_JPRB, 0.16603E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.14848E+01_JPRB, 0.16577E+01_JPRB, 0.15870E+01_JPRB, 0.17356E+01_JPRB, 0.20182E+01_JPRB, 0.17925E+01_JPRB ,&
&  0.17366E+01_JPRB, 0.17141E+01_JPRB, 0.20030E+01_JPRB, 0.21867E+01_JPRB, 0.21925E+01_JPRB, 0.20221E+01_JPRB, 0.17947E+01_JPRB ,&
&  0.15986E+01_JPRB, 0.14971E+01_JPRB, 0.14499E+01_JPRB, 0.14093E+01_JPRB, 0.13788E+01_JPRB /)
ZBURD(:, 18,  5) = (/ &
&  0.15351E+01_JPRB, 0.14706E+01_JPRB, 0.14110E+01_JPRB, 0.13416E+01_JPRB, 0.12873E+01_JPRB, 0.12430E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.11287E+01_JPRB, 0.10758E+01_JPRB, 0.10463E+01_JPRB, 0.10355E+01_JPRB, 0.10354E+01_JPRB, 0.10471E+01_JPRB, 0.10541E+01_JPRB ,&
&  0.10924E+01_JPRB, 0.13630E+01_JPRB, 0.11038E+01_JPRB, 0.94789E+00_JPRB, 0.84310E+00_JPRB, 0.92056E+00_JPRB, 0.10335E+01_JPRB ,&
&  0.12245E+01_JPRB, 0.13660E+01_JPRB, 0.16111E+01_JPRB, 0.20010E+01_JPRB, 0.23362E+01_JPRB, 0.26227E+01_JPRB, 0.27862E+01_JPRB ,&
&  0.25918E+01_JPRB, 0.24108E+01_JPRB, 0.18816E+01_JPRB, 0.17359E+01_JPRB, 0.15508E+01_JPRB, 0.13363E+01_JPRB, 0.13721E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12981E+01_JPRB, 0.12889E+01_JPRB, 0.13163E+01_JPRB, 0.13507E+01_JPRB, 0.13873E+01_JPRB, 0.14336E+01_JPRB ,&
&  0.14805E+01_JPRB, 0.15364E+01_JPRB, 0.15883E+01_JPRB, 0.16465E+01_JPRB, 0.18046E+01_JPRB, 0.21213E+01_JPRB, 0.23586E+01_JPRB ,&
&  0.29470E+01_JPRB, 0.31641E+01_JPRB, 0.34804E+01_JPRB, 0.54480E+01_JPRB, 0.81003E+01_JPRB, 0.81764E+01_JPRB, 0.75155E+01_JPRB ,&
&  0.74908E+01_JPRB, 0.75369E+01_JPRB, 0.77268E+01_JPRB, 0.74597E+01_JPRB, 0.68418E+01_JPRB, 0.65303E+01_JPRB, 0.61462E+01_JPRB ,&
&  0.55366E+01_JPRB, 0.50985E+01_JPRB, 0.48671E+01_JPRB, 0.48245E+01_JPRB, 0.45901E+01_JPRB, 0.45106E+01_JPRB, 0.45321E+01_JPRB ,&
&  0.44041E+01_JPRB, 0.37494E+01_JPRB, 0.24874E+01_JPRB, 0.24267E+01_JPRB, 0.18930E+01_JPRB, 0.16869E+01_JPRB, 0.17670E+01_JPRB ,&
&  0.15901E+01_JPRB, 0.17493E+01_JPRB, 0.19518E+01_JPRB, 0.19280E+01_JPRB, 0.25035E+01_JPRB, 0.27151E+01_JPRB, 0.26821E+01_JPRB ,&
&  0.25451E+01_JPRB, 0.23274E+01_JPRB, 0.27078E+01_JPRB, 0.28047E+01_JPRB, 0.27475E+01_JPRB, 0.25426E+01_JPRB, 0.22724E+01_JPRB ,&
&  0.19913E+01_JPRB, 0.18233E+01_JPRB, 0.17391E+01_JPRB, 0.16808E+01_JPRB, 0.16047E+01_JPRB /)
ZBURD(:, 19,  5) = (/ &
&  0.17437E+01_JPRB, 0.16538E+01_JPRB, 0.15633E+01_JPRB, 0.14929E+01_JPRB, 0.14317E+01_JPRB, 0.13616E+01_JPRB, 0.12725E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11417E+01_JPRB, 0.11224E+01_JPRB, 0.11026E+01_JPRB, 0.10907E+01_JPRB, 0.10916E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.11293E+01_JPRB, 0.12134E+01_JPRB, 0.97449E+00_JPRB, 0.80124E+00_JPRB, 0.70669E+00_JPRB, 0.77157E+00_JPRB, 0.93292E+00_JPRB ,&
&  0.11542E+01_JPRB, 0.15317E+01_JPRB, 0.21347E+01_JPRB, 0.26283E+01_JPRB, 0.31702E+01_JPRB, 0.37228E+01_JPRB, 0.39021E+01_JPRB ,&
&  0.41969E+01_JPRB, 0.33601E+01_JPRB, 0.30335E+01_JPRB, 0.25106E+01_JPRB, 0.21637E+01_JPRB, 0.19370E+01_JPRB, 0.17279E+01_JPRB ,&
&  0.15460E+01_JPRB, 0.14511E+01_JPRB, 0.14009E+01_JPRB, 0.14133E+01_JPRB, 0.14381E+01_JPRB, 0.14912E+01_JPRB, 0.15813E+01_JPRB ,&
&  0.16874E+01_JPRB, 0.17488E+01_JPRB, 0.17281E+01_JPRB, 0.17514E+01_JPRB, 0.21344E+01_JPRB, 0.26798E+01_JPRB, 0.30182E+01_JPRB ,&
&  0.30924E+01_JPRB, 0.35634E+01_JPRB, 0.63838E+01_JPRB, 0.75975E+01_JPRB, 0.83209E+01_JPRB, 0.93409E+01_JPRB, 0.95600E+01_JPRB ,&
&  0.87917E+01_JPRB, 0.80616E+01_JPRB, 0.77502E+01_JPRB, 0.68469E+01_JPRB, 0.77022E+01_JPRB, 0.76365E+01_JPRB, 0.71817E+01_JPRB ,&
&  0.64596E+01_JPRB, 0.60539E+01_JPRB, 0.56063E+01_JPRB, 0.53216E+01_JPRB, 0.52217E+01_JPRB, 0.53164E+01_JPRB, 0.49697E+01_JPRB ,&
&  0.42573E+01_JPRB, 0.45055E+01_JPRB, 0.37886E+01_JPRB, 0.30386E+01_JPRB, 0.24672E+01_JPRB, 0.23019E+01_JPRB, 0.21710E+01_JPRB ,&
&  0.20125E+01_JPRB, 0.20361E+01_JPRB, 0.23561E+01_JPRB, 0.27887E+01_JPRB, 0.33858E+01_JPRB, 0.37208E+01_JPRB, 0.35765E+01_JPRB ,&
&  0.32250E+01_JPRB, 0.34204E+01_JPRB, 0.33502E+01_JPRB, 0.31331E+01_JPRB, 0.29767E+01_JPRB, 0.27338E+01_JPRB, 0.25183E+01_JPRB ,&
&  0.22955E+01_JPRB, 0.21500E+01_JPRB, 0.20464E+01_JPRB, 0.19507E+01_JPRB, 0.18463E+01_JPRB /)
ZBURD(:, 20,  5) = (/ &
&  0.18690E+01_JPRB, 0.17678E+01_JPRB, 0.16635E+01_JPRB, 0.15713E+01_JPRB, 0.14767E+01_JPRB, 0.13946E+01_JPRB, 0.13062E+01_JPRB ,&
&  0.12406E+01_JPRB, 0.12049E+01_JPRB, 0.11660E+01_JPRB, 0.11354E+01_JPRB, 0.11172E+01_JPRB, 0.11019E+01_JPRB, 0.11066E+01_JPRB ,&
&  0.11301E+01_JPRB, 0.10607E+01_JPRB, 0.86093E+00_JPRB, 0.81082E+00_JPRB, 0.81892E+00_JPRB, 0.79406E+00_JPRB, 0.90548E+00_JPRB ,&
&  0.11791E+01_JPRB, 0.17383E+01_JPRB, 0.24553E+01_JPRB, 0.33616E+01_JPRB, 0.43816E+01_JPRB, 0.51874E+01_JPRB, 0.53399E+01_JPRB ,&
&  0.47366E+01_JPRB, 0.46659E+01_JPRB, 0.40783E+01_JPRB, 0.35644E+01_JPRB, 0.30141E+01_JPRB, 0.25943E+01_JPRB, 0.22647E+01_JPRB ,&
&  0.19796E+01_JPRB, 0.17985E+01_JPRB, 0.17202E+01_JPRB, 0.17041E+01_JPRB, 0.17354E+01_JPRB, 0.17978E+01_JPRB, 0.18974E+01_JPRB ,&
&  0.19432E+01_JPRB, 0.19509E+01_JPRB, 0.18851E+01_JPRB, 0.19799E+01_JPRB, 0.23211E+01_JPRB, 0.31310E+01_JPRB, 0.33469E+01_JPRB ,&
&  0.41112E+01_JPRB, 0.48601E+01_JPRB, 0.52085E+01_JPRB, 0.68040E+01_JPRB, 0.61371E+01_JPRB, 0.78181E+01_JPRB, 0.99291E+01_JPRB ,&
&  0.90271E+01_JPRB, 0.80330E+01_JPRB, 0.78168E+01_JPRB, 0.64860E+01_JPRB, 0.56577E+01_JPRB, 0.79152E+01_JPRB, 0.73399E+01_JPRB ,&
&  0.66343E+01_JPRB, 0.64673E+01_JPRB, 0.59231E+01_JPRB, 0.56922E+01_JPRB, 0.50973E+01_JPRB, 0.45995E+01_JPRB, 0.34620E+01_JPRB ,&
&  0.35336E+01_JPRB, 0.32240E+01_JPRB, 0.34429E+01_JPRB, 0.30719E+01_JPRB, 0.28525E+01_JPRB, 0.26283E+01_JPRB, 0.23875E+01_JPRB ,&
&  0.22562E+01_JPRB, 0.22863E+01_JPRB, 0.32643E+01_JPRB, 0.45907E+01_JPRB, 0.51858E+01_JPRB, 0.44265E+01_JPRB, 0.47626E+01_JPRB ,&
&  0.44191E+01_JPRB, 0.38351E+01_JPRB, 0.31948E+01_JPRB, 0.29248E+01_JPRB, 0.26946E+01_JPRB, 0.25038E+01_JPRB, 0.23646E+01_JPRB ,&
&  0.22414E+01_JPRB, 0.21317E+01_JPRB, 0.20517E+01_JPRB, 0.20060E+01_JPRB, 0.19463E+01_JPRB /)
ZBURD(:, 21,  5) = (/ &
&  0.18990E+01_JPRB, 0.17903E+01_JPRB, 0.16770E+01_JPRB, 0.15829E+01_JPRB, 0.14952E+01_JPRB, 0.14152E+01_JPRB, 0.13489E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.12495E+01_JPRB, 0.11763E+01_JPRB, 0.11258E+01_JPRB, 0.11011E+01_JPRB, 0.11036E+01_JPRB, 0.10991E+01_JPRB ,&
&  0.11116E+01_JPRB, 0.10688E+01_JPRB, 0.89839E+00_JPRB, 0.83848E+00_JPRB, 0.95861E+00_JPRB, 0.92962E+00_JPRB, 0.97714E+00_JPRB ,&
&  0.13860E+01_JPRB, 0.21765E+01_JPRB, 0.30254E+01_JPRB, 0.40466E+01_JPRB, 0.54999E+01_JPRB, 0.63139E+01_JPRB, 0.61082E+01_JPRB ,&
&  0.61989E+01_JPRB, 0.53612E+01_JPRB, 0.47178E+01_JPRB, 0.41987E+01_JPRB, 0.36731E+01_JPRB, 0.31266E+01_JPRB, 0.26127E+01_JPRB ,&
&  0.23066E+01_JPRB, 0.21126E+01_JPRB, 0.20014E+01_JPRB, 0.19041E+01_JPRB, 0.19232E+01_JPRB, 0.19866E+01_JPRB, 0.20633E+01_JPRB ,&
&  0.20813E+01_JPRB, 0.20447E+01_JPRB, 0.19735E+01_JPRB, 0.20125E+01_JPRB, 0.25527E+01_JPRB, 0.31961E+01_JPRB, 0.36254E+01_JPRB ,&
&  0.39123E+01_JPRB, 0.42716E+01_JPRB, 0.47123E+01_JPRB, 0.56980E+01_JPRB, 0.73236E+01_JPRB, 0.79445E+01_JPRB, 0.94864E+01_JPRB ,&
&  0.76613E+01_JPRB, 0.64393E+01_JPRB, 0.53254E+01_JPRB, 0.40942E+01_JPRB, 0.51459E+01_JPRB, 0.82539E+01_JPRB, 0.80046E+01_JPRB ,&
&  0.73495E+01_JPRB, 0.71766E+01_JPRB, 0.66884E+01_JPRB, 0.54294E+01_JPRB, 0.45528E+01_JPRB, 0.36373E+01_JPRB, 0.44076E+01_JPRB ,&
&  0.43433E+01_JPRB, 0.41197E+01_JPRB, 0.36663E+01_JPRB, 0.30201E+01_JPRB, 0.25527E+01_JPRB, 0.28065E+01_JPRB, 0.26545E+01_JPRB ,&
&  0.25305E+01_JPRB, 0.28784E+01_JPRB, 0.53892E+01_JPRB, 0.66043E+01_JPRB, 0.61253E+01_JPRB, 0.50804E+01_JPRB, 0.53561E+01_JPRB ,&
&  0.44240E+01_JPRB, 0.36766E+01_JPRB, 0.30931E+01_JPRB, 0.26534E+01_JPRB, 0.24176E+01_JPRB, 0.22861E+01_JPRB, 0.22261E+01_JPRB ,&
&  0.21919E+01_JPRB, 0.21674E+01_JPRB, 0.21056E+01_JPRB, 0.20338E+01_JPRB, 0.19669E+01_JPRB /)
ZBURD(:, 22,  5) = (/ &
&  0.19093E+01_JPRB, 0.18013E+01_JPRB, 0.17093E+01_JPRB, 0.16135E+01_JPRB, 0.15195E+01_JPRB, 0.14485E+01_JPRB, 0.13960E+01_JPRB ,&
&  0.13279E+01_JPRB, 0.12615E+01_JPRB, 0.11965E+01_JPRB, 0.11557E+01_JPRB, 0.11380E+01_JPRB, 0.11309E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.11259E+01_JPRB, 0.11893E+01_JPRB, 0.10200E+01_JPRB, 0.96440E+00_JPRB, 0.10359E+01_JPRB, 0.10751E+01_JPRB, 0.11547E+01_JPRB ,&
&  0.17840E+01_JPRB, 0.26188E+01_JPRB, 0.31142E+01_JPRB, 0.40917E+01_JPRB, 0.61079E+01_JPRB, 0.58188E+01_JPRB, 0.59293E+01_JPRB ,&
&  0.60069E+01_JPRB, 0.51065E+01_JPRB, 0.45630E+01_JPRB, 0.40599E+01_JPRB, 0.36380E+01_JPRB, 0.32649E+01_JPRB, 0.28718E+01_JPRB ,&
&  0.25394E+01_JPRB, 0.23249E+01_JPRB, 0.22083E+01_JPRB, 0.21772E+01_JPRB, 0.21461E+01_JPRB, 0.21329E+01_JPRB, 0.21162E+01_JPRB ,&
&  0.20896E+01_JPRB, 0.20292E+01_JPRB, 0.19233E+01_JPRB, 0.20958E+01_JPRB, 0.27490E+01_JPRB, 0.31951E+01_JPRB, 0.41936E+01_JPRB ,&
&  0.41011E+01_JPRB, 0.44428E+01_JPRB, 0.45576E+01_JPRB, 0.62816E+01_JPRB, 0.74276E+01_JPRB, 0.79607E+01_JPRB, 0.88778E+01_JPRB ,&
&  0.70109E+01_JPRB, 0.60706E+01_JPRB, 0.56466E+01_JPRB, 0.51618E+01_JPRB, 0.48726E+01_JPRB, 0.56077E+01_JPRB, 0.77673E+01_JPRB ,&
&  0.59697E+01_JPRB, 0.61397E+01_JPRB, 0.61647E+01_JPRB, 0.56699E+01_JPRB, 0.29584E+01_JPRB, 0.21399E+01_JPRB, 0.34576E+01_JPRB ,&
&  0.36046E+01_JPRB, 0.21088E+01_JPRB, 0.15119E+01_JPRB, 0.19794E+01_JPRB, 0.15233E+01_JPRB, 0.18630E+01_JPRB, 0.27556E+01_JPRB ,&
&  0.31428E+01_JPRB, 0.40639E+01_JPRB, 0.66001E+01_JPRB, 0.60743E+01_JPRB, 0.54988E+01_JPRB, 0.52553E+01_JPRB, 0.47528E+01_JPRB ,&
&  0.39933E+01_JPRB, 0.32286E+01_JPRB, 0.30460E+01_JPRB, 0.26221E+01_JPRB, 0.23947E+01_JPRB, 0.22781E+01_JPRB, 0.22738E+01_JPRB ,&
&  0.23354E+01_JPRB, 0.23377E+01_JPRB, 0.22687E+01_JPRB, 0.21449E+01_JPRB, 0.20164E+01_JPRB /)
ZBURD(:, 23,  5) = (/ &
&  0.19365E+01_JPRB, 0.18397E+01_JPRB, 0.17464E+01_JPRB, 0.16568E+01_JPRB, 0.15762E+01_JPRB, 0.15052E+01_JPRB, 0.14135E+01_JPRB ,&
&  0.13007E+01_JPRB, 0.12309E+01_JPRB, 0.12038E+01_JPRB, 0.11784E+01_JPRB, 0.11661E+01_JPRB, 0.11690E+01_JPRB, 0.11576E+01_JPRB ,&
&  0.11425E+01_JPRB, 0.11956E+01_JPRB, 0.12936E+01_JPRB, 0.12402E+01_JPRB, 0.12074E+01_JPRB, 0.11500E+01_JPRB, 0.13979E+01_JPRB ,&
&  0.22316E+01_JPRB, 0.30203E+01_JPRB, 0.34224E+01_JPRB, 0.39069E+01_JPRB, 0.49620E+01_JPRB, 0.51312E+01_JPRB, 0.58618E+01_JPRB ,&
&  0.53412E+01_JPRB, 0.46659E+01_JPRB, 0.41016E+01_JPRB, 0.36431E+01_JPRB, 0.33032E+01_JPRB, 0.30238E+01_JPRB, 0.28315E+01_JPRB ,&
&  0.26154E+01_JPRB, 0.23930E+01_JPRB, 0.22807E+01_JPRB, 0.21344E+01_JPRB, 0.20327E+01_JPRB, 0.19801E+01_JPRB, 0.20251E+01_JPRB ,&
&  0.20142E+01_JPRB, 0.19637E+01_JPRB, 0.19178E+01_JPRB, 0.21953E+01_JPRB, 0.30739E+01_JPRB, 0.35961E+01_JPRB, 0.36704E+01_JPRB ,&
&  0.37780E+01_JPRB, 0.40293E+01_JPRB, 0.50630E+01_JPRB, 0.60465E+01_JPRB, 0.68421E+01_JPRB, 0.77206E+01_JPRB, 0.79783E+01_JPRB ,&
&  0.78162E+01_JPRB, 0.73201E+01_JPRB, 0.57961E+01_JPRB, 0.54094E+01_JPRB, 0.49451E+01_JPRB, 0.42447E+01_JPRB, 0.50498E+01_JPRB ,&
&  0.54592E+01_JPRB, 0.48263E+01_JPRB, 0.38157E+01_JPRB, 0.30279E+01_JPRB, 0.29039E+01_JPRB, 0.20291E+01_JPRB, 0.13097E+01_JPRB ,&
&  0.12105E+01_JPRB, 0.11103E+01_JPRB, 0.10034E+01_JPRB, 0.10914E+01_JPRB, 0.10962E+01_JPRB, 0.16067E+01_JPRB, 0.27370E+01_JPRB ,&
&  0.40776E+01_JPRB, 0.51588E+01_JPRB, 0.59986E+01_JPRB, 0.52746E+01_JPRB, 0.45242E+01_JPRB, 0.45592E+01_JPRB, 0.45616E+01_JPRB ,&
&  0.32632E+01_JPRB, 0.29141E+01_JPRB, 0.28078E+01_JPRB, 0.25152E+01_JPRB, 0.23401E+01_JPRB, 0.23133E+01_JPRB, 0.24083E+01_JPRB ,&
&  0.25042E+01_JPRB, 0.24845E+01_JPRB, 0.23776E+01_JPRB, 0.22054E+01_JPRB, 0.20493E+01_JPRB /)
ZBURD(:, 24,  5) = (/ &
&  0.18897E+01_JPRB, 0.17718E+01_JPRB, 0.16919E+01_JPRB, 0.16406E+01_JPRB, 0.15884E+01_JPRB, 0.14997E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.11996E+01_JPRB, 0.11924E+01_JPRB, 0.11792E+01_JPRB, 0.11663E+01_JPRB, 0.11616E+01_JPRB, 0.11506E+01_JPRB ,&
&  0.11262E+01_JPRB, 0.11582E+01_JPRB, 0.13381E+01_JPRB, 0.13649E+01_JPRB, 0.13666E+01_JPRB, 0.12915E+01_JPRB, 0.16414E+01_JPRB ,&
&  0.25048E+01_JPRB, 0.31873E+01_JPRB, 0.35794E+01_JPRB, 0.36926E+01_JPRB, 0.46936E+01_JPRB, 0.53109E+01_JPRB, 0.52885E+01_JPRB ,&
&  0.44281E+01_JPRB, 0.36839E+01_JPRB, 0.30071E+01_JPRB, 0.26398E+01_JPRB, 0.26036E+01_JPRB, 0.26036E+01_JPRB, 0.25397E+01_JPRB ,&
&  0.23610E+01_JPRB, 0.22164E+01_JPRB, 0.20699E+01_JPRB, 0.19434E+01_JPRB, 0.18850E+01_JPRB, 0.18994E+01_JPRB, 0.19607E+01_JPRB ,&
&  0.19766E+01_JPRB, 0.19644E+01_JPRB, 0.20863E+01_JPRB, 0.25554E+01_JPRB, 0.30472E+01_JPRB, 0.31185E+01_JPRB, 0.37032E+01_JPRB ,&
&  0.43651E+01_JPRB, 0.49254E+01_JPRB, 0.49438E+01_JPRB, 0.62338E+01_JPRB, 0.66888E+01_JPRB, 0.69911E+01_JPRB, 0.73393E+01_JPRB ,&
&  0.72067E+01_JPRB, 0.72026E+01_JPRB, 0.53638E+01_JPRB, 0.48027E+01_JPRB, 0.51176E+01_JPRB, 0.43203E+01_JPRB, 0.42464E+01_JPRB ,&
&  0.49485E+01_JPRB, 0.42665E+01_JPRB, 0.35813E+01_JPRB, 0.30898E+01_JPRB, 0.38841E+01_JPRB, 0.34167E+01_JPRB, 0.14704E+01_JPRB ,&
&  0.11166E+01_JPRB, 0.10247E+01_JPRB, 0.89973E+00_JPRB, 0.87226E+00_JPRB, 0.98245E+00_JPRB, 0.15674E+01_JPRB, 0.33828E+01_JPRB ,&
&  0.39710E+01_JPRB, 0.55362E+01_JPRB, 0.55922E+01_JPRB, 0.50670E+01_JPRB, 0.41669E+01_JPRB, 0.38597E+01_JPRB, 0.36080E+01_JPRB ,&
&  0.31700E+01_JPRB, 0.27799E+01_JPRB, 0.23819E+01_JPRB, 0.21408E+01_JPRB, 0.20644E+01_JPRB, 0.21429E+01_JPRB, 0.23294E+01_JPRB ,&
&  0.24208E+01_JPRB, 0.24211E+01_JPRB, 0.23343E+01_JPRB, 0.21826E+01_JPRB, 0.20172E+01_JPRB /)
ZBURD(:, 25,  5) = (/ &
&  0.17766E+01_JPRB, 0.17221E+01_JPRB, 0.16849E+01_JPRB, 0.16168E+01_JPRB, 0.15351E+01_JPRB, 0.14330E+01_JPRB, 0.13007E+01_JPRB ,&
&  0.12351E+01_JPRB, 0.12086E+01_JPRB, 0.12025E+01_JPRB, 0.11714E+01_JPRB, 0.11374E+01_JPRB, 0.11201E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.11022E+01_JPRB, 0.11971E+01_JPRB, 0.13119E+01_JPRB, 0.13669E+01_JPRB, 0.13851E+01_JPRB, 0.17991E+01_JPRB ,&
&  0.26391E+01_JPRB, 0.31602E+01_JPRB, 0.34718E+01_JPRB, 0.36515E+01_JPRB, 0.38651E+01_JPRB, 0.41570E+01_JPRB, 0.35795E+01_JPRB ,&
&  0.29527E+01_JPRB, 0.24879E+01_JPRB, 0.21699E+01_JPRB, 0.21714E+01_JPRB, 0.23183E+01_JPRB, 0.24128E+01_JPRB, 0.23385E+01_JPRB ,&
&  0.22336E+01_JPRB, 0.20828E+01_JPRB, 0.19803E+01_JPRB, 0.18842E+01_JPRB, 0.18672E+01_JPRB, 0.19147E+01_JPRB, 0.19750E+01_JPRB ,&
&  0.20233E+01_JPRB, 0.21031E+01_JPRB, 0.23673E+01_JPRB, 0.27838E+01_JPRB, 0.26763E+01_JPRB, 0.33743E+01_JPRB, 0.39075E+01_JPRB ,&
&  0.43940E+01_JPRB, 0.46946E+01_JPRB, 0.47378E+01_JPRB, 0.55546E+01_JPRB, 0.63316E+01_JPRB, 0.66885E+01_JPRB, 0.64474E+01_JPRB ,&
&  0.64810E+01_JPRB, 0.55573E+01_JPRB, 0.47833E+01_JPRB, 0.43903E+01_JPRB, 0.44445E+01_JPRB, 0.51072E+01_JPRB, 0.37021E+01_JPRB ,&
&  0.36035E+01_JPRB, 0.42931E+01_JPRB, 0.38159E+01_JPRB, 0.34237E+01_JPRB, 0.38457E+01_JPRB, 0.41267E+01_JPRB, 0.30930E+01_JPRB ,&
&  0.13021E+01_JPRB, 0.96755E+00_JPRB, 0.98971E+00_JPRB, 0.96214E+00_JPRB, 0.90784E+00_JPRB, 0.15788E+01_JPRB, 0.58610E+01_JPRB ,&
&  0.48162E+01_JPRB, 0.62906E+01_JPRB, 0.58923E+01_JPRB, 0.49446E+01_JPRB, 0.42217E+01_JPRB, 0.34295E+01_JPRB, 0.30705E+01_JPRB ,&
&  0.25119E+01_JPRB, 0.21821E+01_JPRB, 0.20103E+01_JPRB, 0.19176E+01_JPRB, 0.19272E+01_JPRB, 0.19915E+01_JPRB, 0.21338E+01_JPRB ,&
&  0.22184E+01_JPRB, 0.22301E+01_JPRB, 0.21417E+01_JPRB, 0.20042E+01_JPRB, 0.18737E+01_JPRB /)
ZBURD(:, 26,  5) = (/ &
&  0.16994E+01_JPRB, 0.16725E+01_JPRB, 0.16227E+01_JPRB, 0.15339E+01_JPRB, 0.14510E+01_JPRB, 0.13411E+01_JPRB, 0.12523E+01_JPRB ,&
&  0.12359E+01_JPRB, 0.12108E+01_JPRB, 0.11860E+01_JPRB, 0.11491E+01_JPRB, 0.11076E+01_JPRB, 0.10804E+01_JPRB, 0.10795E+01_JPRB ,&
&  0.10848E+01_JPRB, 0.10607E+01_JPRB, 0.10936E+01_JPRB, 0.12107E+01_JPRB, 0.14330E+01_JPRB, 0.15724E+01_JPRB, 0.17990E+01_JPRB ,&
&  0.27801E+01_JPRB, 0.30591E+01_JPRB, 0.24914E+01_JPRB, 0.26342E+01_JPRB, 0.28875E+01_JPRB, 0.31498E+01_JPRB, 0.23536E+01_JPRB ,&
&  0.20811E+01_JPRB, 0.19632E+01_JPRB, 0.18680E+01_JPRB, 0.19698E+01_JPRB, 0.20976E+01_JPRB, 0.21920E+01_JPRB, 0.21935E+01_JPRB ,&
&  0.20935E+01_JPRB, 0.20229E+01_JPRB, 0.19940E+01_JPRB, 0.19497E+01_JPRB, 0.19254E+01_JPRB, 0.19817E+01_JPRB, 0.20557E+01_JPRB ,&
&  0.21452E+01_JPRB, 0.23041E+01_JPRB, 0.26156E+01_JPRB, 0.27769E+01_JPRB, 0.30557E+01_JPRB, 0.35089E+01_JPRB, 0.40254E+01_JPRB ,&
&  0.42197E+01_JPRB, 0.45517E+01_JPRB, 0.48852E+01_JPRB, 0.51808E+01_JPRB, 0.56230E+01_JPRB, 0.61184E+01_JPRB, 0.58294E+01_JPRB ,&
&  0.56400E+01_JPRB, 0.48582E+01_JPRB, 0.41695E+01_JPRB, 0.39394E+01_JPRB, 0.40522E+01_JPRB, 0.47448E+01_JPRB, 0.45196E+01_JPRB ,&
&  0.42328E+01_JPRB, 0.33856E+01_JPRB, 0.32790E+01_JPRB, 0.37063E+01_JPRB, 0.36269E+01_JPRB, 0.36635E+01_JPRB, 0.44747E+01_JPRB ,&
&  0.39303E+01_JPRB, 0.21316E+01_JPRB, 0.17485E+01_JPRB, 0.22070E+01_JPRB, 0.15231E+01_JPRB, 0.16125E+01_JPRB, 0.33781E+01_JPRB ,&
&  0.45737E+01_JPRB, 0.55667E+01_JPRB, 0.52903E+01_JPRB, 0.44008E+01_JPRB, 0.38920E+01_JPRB, 0.29107E+01_JPRB, 0.23442E+01_JPRB ,&
&  0.19140E+01_JPRB, 0.17820E+01_JPRB, 0.17128E+01_JPRB, 0.16343E+01_JPRB, 0.16261E+01_JPRB, 0.17291E+01_JPRB, 0.18539E+01_JPRB ,&
&  0.19461E+01_JPRB, 0.19692E+01_JPRB, 0.19345E+01_JPRB, 0.18530E+01_JPRB, 0.17559E+01_JPRB /)
ZBURD(:, 27,  5) = (/ &
&  0.16052E+01_JPRB, 0.15862E+01_JPRB, 0.15528E+01_JPRB, 0.14644E+01_JPRB, 0.13773E+01_JPRB, 0.12851E+01_JPRB, 0.12571E+01_JPRB ,&
&  0.12245E+01_JPRB, 0.11944E+01_JPRB, 0.11632E+01_JPRB, 0.11109E+01_JPRB, 0.10847E+01_JPRB, 0.10609E+01_JPRB, 0.10730E+01_JPRB ,&
&  0.10918E+01_JPRB, 0.10679E+01_JPRB, 0.11163E+01_JPRB, 0.13477E+01_JPRB, 0.17067E+01_JPRB, 0.23947E+01_JPRB, 0.19514E+01_JPRB ,&
&  0.25805E+01_JPRB, 0.30891E+01_JPRB, 0.23519E+01_JPRB, 0.23050E+01_JPRB, 0.23586E+01_JPRB, 0.23734E+01_JPRB, 0.19874E+01_JPRB ,&
&  0.18426E+01_JPRB, 0.18279E+01_JPRB, 0.18448E+01_JPRB, 0.18899E+01_JPRB, 0.19727E+01_JPRB, 0.20334E+01_JPRB, 0.19898E+01_JPRB ,&
&  0.19819E+01_JPRB, 0.19939E+01_JPRB, 0.19894E+01_JPRB, 0.19725E+01_JPRB, 0.19940E+01_JPRB, 0.20621E+01_JPRB, 0.21523E+01_JPRB ,&
&  0.22980E+01_JPRB, 0.25139E+01_JPRB, 0.28243E+01_JPRB, 0.28807E+01_JPRB, 0.31574E+01_JPRB, 0.36686E+01_JPRB, 0.39692E+01_JPRB ,&
&  0.39470E+01_JPRB, 0.38775E+01_JPRB, 0.44222E+01_JPRB, 0.49294E+01_JPRB, 0.51707E+01_JPRB, 0.52745E+01_JPRB, 0.49906E+01_JPRB ,&
&  0.50051E+01_JPRB, 0.47079E+01_JPRB, 0.46144E+01_JPRB, 0.39411E+01_JPRB, 0.39816E+01_JPRB, 0.44086E+01_JPRB, 0.42038E+01_JPRB ,&
&  0.45775E+01_JPRB, 0.42391E+01_JPRB, 0.41269E+01_JPRB, 0.37881E+01_JPRB, 0.34555E+01_JPRB, 0.33587E+01_JPRB, 0.35090E+01_JPRB ,&
&  0.35986E+01_JPRB, 0.38874E+01_JPRB, 0.36505E+01_JPRB, 0.22929E+01_JPRB, 0.18211E+01_JPRB, 0.19735E+01_JPRB, 0.26184E+01_JPRB ,&
&  0.38711E+01_JPRB, 0.41407E+01_JPRB, 0.38333E+01_JPRB, 0.43840E+01_JPRB, 0.28937E+01_JPRB, 0.20804E+01_JPRB, 0.16906E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.14860E+01_JPRB, 0.14384E+01_JPRB, 0.13606E+01_JPRB, 0.13644E+01_JPRB, 0.14401E+01_JPRB, 0.15460E+01_JPRB ,&
&  0.16517E+01_JPRB, 0.17234E+01_JPRB, 0.17567E+01_JPRB, 0.17093E+01_JPRB, 0.16452E+01_JPRB /)
ZBURD(:, 28,  5) = (/ &
&  0.14667E+01_JPRB, 0.14671E+01_JPRB, 0.14201E+01_JPRB, 0.13547E+01_JPRB, 0.13013E+01_JPRB, 0.12569E+01_JPRB, 0.12550E+01_JPRB ,&
&  0.11735E+01_JPRB, 0.11371E+01_JPRB, 0.11094E+01_JPRB, 0.10786E+01_JPRB, 0.10858E+01_JPRB, 0.11011E+01_JPRB, 0.11349E+01_JPRB ,&
&  0.11808E+01_JPRB, 0.12408E+01_JPRB, 0.14202E+01_JPRB, 0.18065E+01_JPRB, 0.22839E+01_JPRB, 0.29053E+01_JPRB, 0.28433E+01_JPRB ,&
&  0.24643E+01_JPRB, 0.32994E+01_JPRB, 0.22446E+01_JPRB, 0.22793E+01_JPRB, 0.22928E+01_JPRB, 0.24348E+01_JPRB, 0.21869E+01_JPRB ,&
&  0.19567E+01_JPRB, 0.19126E+01_JPRB, 0.19283E+01_JPRB, 0.18755E+01_JPRB, 0.18712E+01_JPRB, 0.18514E+01_JPRB, 0.18578E+01_JPRB ,&
&  0.18980E+01_JPRB, 0.19444E+01_JPRB, 0.19730E+01_JPRB, 0.20147E+01_JPRB, 0.20907E+01_JPRB, 0.21789E+01_JPRB, 0.23237E+01_JPRB ,&
&  0.25404E+01_JPRB, 0.27769E+01_JPRB, 0.28845E+01_JPRB, 0.30094E+01_JPRB, 0.31821E+01_JPRB, 0.35323E+01_JPRB, 0.38158E+01_JPRB ,&
&  0.38792E+01_JPRB, 0.38297E+01_JPRB, 0.42075E+01_JPRB, 0.43481E+01_JPRB, 0.43960E+01_JPRB, 0.47356E+01_JPRB, 0.43702E+01_JPRB ,&
&  0.45949E+01_JPRB, 0.41995E+01_JPRB, 0.46370E+01_JPRB, 0.39075E+01_JPRB, 0.38938E+01_JPRB, 0.40220E+01_JPRB, 0.34465E+01_JPRB ,&
&  0.32877E+01_JPRB, 0.36867E+01_JPRB, 0.37349E+01_JPRB, 0.36383E+01_JPRB, 0.34720E+01_JPRB, 0.32655E+01_JPRB, 0.29876E+01_JPRB ,&
&  0.26126E+01_JPRB, 0.25235E+01_JPRB, 0.22545E+01_JPRB, 0.15110E+01_JPRB, 0.13130E+01_JPRB, 0.16236E+01_JPRB, 0.20047E+01_JPRB ,&
&  0.27657E+01_JPRB, 0.30675E+01_JPRB, 0.33787E+01_JPRB, 0.27768E+01_JPRB, 0.18315E+01_JPRB, 0.14327E+01_JPRB, 0.13514E+01_JPRB ,&
&  0.13131E+01_JPRB, 0.12734E+01_JPRB, 0.12008E+01_JPRB, 0.11436E+01_JPRB, 0.11632E+01_JPRB, 0.12447E+01_JPRB, 0.13355E+01_JPRB ,&
&  0.14318E+01_JPRB, 0.14941E+01_JPRB, 0.14982E+01_JPRB, 0.14705E+01_JPRB, 0.14691E+01_JPRB /)
ZBURD(:, 29,  5) = (/ &
&  0.14204E+01_JPRB, 0.14282E+01_JPRB, 0.14287E+01_JPRB, 0.14469E+01_JPRB, 0.14784E+01_JPRB, 0.15290E+01_JPRB, 0.15095E+01_JPRB ,&
&  0.13371E+01_JPRB, 0.10764E+01_JPRB, 0.10843E+01_JPRB, 0.11124E+01_JPRB, 0.11869E+01_JPRB, 0.12558E+01_JPRB, 0.13463E+01_JPRB ,&
&  0.14844E+01_JPRB, 0.16705E+01_JPRB, 0.19760E+01_JPRB, 0.23910E+01_JPRB, 0.28635E+01_JPRB, 0.34415E+01_JPRB, 0.34690E+01_JPRB ,&
&  0.30097E+01_JPRB, 0.24074E+01_JPRB, 0.23944E+01_JPRB, 0.23280E+01_JPRB, 0.22015E+01_JPRB, 0.23651E+01_JPRB, 0.23339E+01_JPRB ,&
&  0.20810E+01_JPRB, 0.19955E+01_JPRB, 0.19800E+01_JPRB, 0.19098E+01_JPRB, 0.18356E+01_JPRB, 0.18520E+01_JPRB, 0.19076E+01_JPRB ,&
&  0.19745E+01_JPRB, 0.20271E+01_JPRB, 0.20596E+01_JPRB, 0.21167E+01_JPRB, 0.22290E+01_JPRB, 0.23411E+01_JPRB, 0.25342E+01_JPRB ,&
&  0.27503E+01_JPRB, 0.29392E+01_JPRB, 0.30228E+01_JPRB, 0.29820E+01_JPRB, 0.30847E+01_JPRB, 0.32964E+01_JPRB, 0.35294E+01_JPRB ,&
&  0.37188E+01_JPRB, 0.39396E+01_JPRB, 0.40686E+01_JPRB, 0.41231E+01_JPRB, 0.38730E+01_JPRB, 0.42938E+01_JPRB, 0.40360E+01_JPRB ,&
&  0.41039E+01_JPRB, 0.38762E+01_JPRB, 0.42976E+01_JPRB, 0.34606E+01_JPRB, 0.36407E+01_JPRB, 0.34598E+01_JPRB, 0.31612E+01_JPRB ,&
&  0.30560E+01_JPRB, 0.34623E+01_JPRB, 0.35686E+01_JPRB, 0.34528E+01_JPRB, 0.33821E+01_JPRB, 0.28729E+01_JPRB, 0.28824E+01_JPRB ,&
&  0.22617E+01_JPRB, 0.22630E+01_JPRB, 0.15891E+01_JPRB, 0.11898E+01_JPRB, 0.99936E+00_JPRB, 0.12194E+01_JPRB, 0.16627E+01_JPRB ,&
&  0.22900E+01_JPRB, 0.23108E+01_JPRB, 0.19442E+01_JPRB, 0.17333E+01_JPRB, 0.14684E+01_JPRB, 0.12676E+01_JPRB, 0.12357E+01_JPRB ,&
&  0.12272E+01_JPRB, 0.11680E+01_JPRB, 0.11025E+01_JPRB, 0.10919E+01_JPRB, 0.11171E+01_JPRB, 0.12091E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.13416E+01_JPRB, 0.13626E+01_JPRB, 0.13663E+01_JPRB, 0.13769E+01_JPRB, 0.13929E+01_JPRB /)
ZBURD(:, 30,  5) = (/ &
&  0.13795E+01_JPRB, 0.13591E+01_JPRB, 0.13560E+01_JPRB, 0.13456E+01_JPRB, 0.13263E+01_JPRB, 0.12547E+01_JPRB, 0.11959E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.10790E+01_JPRB, 0.11547E+01_JPRB, 0.12570E+01_JPRB, 0.13830E+01_JPRB, 0.15179E+01_JPRB, 0.16701E+01_JPRB ,&
&  0.18625E+01_JPRB, 0.21005E+01_JPRB, 0.24007E+01_JPRB, 0.26563E+01_JPRB, 0.29104E+01_JPRB, 0.32492E+01_JPRB, 0.34881E+01_JPRB ,&
&  0.29970E+01_JPRB, 0.19959E+01_JPRB, 0.23148E+01_JPRB, 0.24341E+01_JPRB, 0.22662E+01_JPRB, 0.22893E+01_JPRB, 0.22290E+01_JPRB ,&
&  0.20949E+01_JPRB, 0.20552E+01_JPRB, 0.20487E+01_JPRB, 0.20251E+01_JPRB, 0.19434E+01_JPRB, 0.19793E+01_JPRB, 0.20314E+01_JPRB ,&
&  0.20631E+01_JPRB, 0.20634E+01_JPRB, 0.20884E+01_JPRB, 0.22166E+01_JPRB, 0.24299E+01_JPRB, 0.25362E+01_JPRB, 0.26994E+01_JPRB ,&
&  0.28459E+01_JPRB, 0.30406E+01_JPRB, 0.31058E+01_JPRB, 0.30511E+01_JPRB, 0.31000E+01_JPRB, 0.31930E+01_JPRB, 0.33495E+01_JPRB ,&
&  0.35520E+01_JPRB, 0.36556E+01_JPRB, 0.38122E+01_JPRB, 0.41108E+01_JPRB, 0.42632E+01_JPRB, 0.37852E+01_JPRB, 0.38368E+01_JPRB ,&
&  0.37898E+01_JPRB, 0.36565E+01_JPRB, 0.35476E+01_JPRB, 0.42630E+01_JPRB, 0.31339E+01_JPRB, 0.30611E+01_JPRB, 0.26928E+01_JPRB ,&
&  0.29510E+01_JPRB, 0.31276E+01_JPRB, 0.32791E+01_JPRB, 0.32888E+01_JPRB, 0.31238E+01_JPRB, 0.27077E+01_JPRB, 0.27498E+01_JPRB ,&
&  0.23841E+01_JPRB, 0.20659E+01_JPRB, 0.15097E+01_JPRB, 0.11385E+01_JPRB, 0.11216E+01_JPRB, 0.17271E+01_JPRB, 0.19476E+01_JPRB ,&
&  0.20953E+01_JPRB, 0.17017E+01_JPRB, 0.16087E+01_JPRB, 0.17358E+01_JPRB, 0.15053E+01_JPRB, 0.12119E+01_JPRB, 0.11413E+01_JPRB ,&
&  0.11556E+01_JPRB, 0.11432E+01_JPRB, 0.10992E+01_JPRB, 0.11017E+01_JPRB, 0.11529E+01_JPRB, 0.12358E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.13128E+01_JPRB, 0.13227E+01_JPRB, 0.13396E+01_JPRB, 0.13818E+01_JPRB /)
ZBURD(:, 31,  5) = (/ &
&  0.12632E+01_JPRB, 0.12199E+01_JPRB, 0.11841E+01_JPRB, 0.11449E+01_JPRB, 0.10993E+01_JPRB, 0.10660E+01_JPRB, 0.10638E+01_JPRB ,&
&  0.10776E+01_JPRB, 0.11316E+01_JPRB, 0.12198E+01_JPRB, 0.13281E+01_JPRB, 0.14676E+01_JPRB, 0.16311E+01_JPRB, 0.18013E+01_JPRB ,&
&  0.20205E+01_JPRB, 0.22388E+01_JPRB, 0.23721E+01_JPRB, 0.24726E+01_JPRB, 0.25934E+01_JPRB, 0.27841E+01_JPRB, 0.29844E+01_JPRB ,&
&  0.31455E+01_JPRB, 0.30792E+01_JPRB, 0.28031E+01_JPRB, 0.20712E+01_JPRB, 0.18187E+01_JPRB, 0.21234E+01_JPRB, 0.21310E+01_JPRB ,&
&  0.21189E+01_JPRB, 0.20671E+01_JPRB, 0.20362E+01_JPRB, 0.20016E+01_JPRB, 0.20037E+01_JPRB, 0.20544E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.21032E+01_JPRB, 0.21275E+01_JPRB, 0.22048E+01_JPRB, 0.23595E+01_JPRB, 0.25741E+01_JPRB, 0.26939E+01_JPRB, 0.28117E+01_JPRB ,&
&  0.29047E+01_JPRB, 0.30522E+01_JPRB, 0.30967E+01_JPRB, 0.30983E+01_JPRB, 0.30172E+01_JPRB, 0.30306E+01_JPRB, 0.32177E+01_JPRB ,&
&  0.34574E+01_JPRB, 0.36664E+01_JPRB, 0.38813E+01_JPRB, 0.41546E+01_JPRB, 0.42680E+01_JPRB, 0.37320E+01_JPRB, 0.35067E+01_JPRB ,&
&  0.35283E+01_JPRB, 0.35040E+01_JPRB, 0.32378E+01_JPRB, 0.41916E+01_JPRB, 0.26263E+01_JPRB, 0.28281E+01_JPRB, 0.27923E+01_JPRB ,&
&  0.26522E+01_JPRB, 0.27010E+01_JPRB, 0.28565E+01_JPRB, 0.29423E+01_JPRB, 0.27912E+01_JPRB, 0.24144E+01_JPRB, 0.26673E+01_JPRB ,&
&  0.24786E+01_JPRB, 0.18704E+01_JPRB, 0.14246E+01_JPRB, 0.10391E+01_JPRB, 0.12677E+01_JPRB, 0.20010E+01_JPRB, 0.17685E+01_JPRB ,&
&  0.14182E+01_JPRB, 0.14716E+01_JPRB, 0.16307E+01_JPRB, 0.17936E+01_JPRB, 0.15928E+01_JPRB, 0.11081E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.11300E+01_JPRB, 0.11430E+01_JPRB, 0.11641E+01_JPRB, 0.11818E+01_JPRB, 0.12156E+01_JPRB, 0.12676E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.13272E+01_JPRB, 0.13225E+01_JPRB, 0.13441E+01_JPRB, 0.13397E+01_JPRB, 0.13089E+01_JPRB /)
ZBURD(:, 32,  5) = (/ &
&  0.10603E+01_JPRB, 0.10045E+01_JPRB, 0.96308E+00_JPRB, 0.94704E+00_JPRB, 0.94652E+00_JPRB, 0.95820E+00_JPRB, 0.98693E+00_JPRB ,&
&  0.10402E+01_JPRB, 0.11102E+01_JPRB, 0.11838E+01_JPRB, 0.12532E+01_JPRB, 0.13443E+01_JPRB, 0.14483E+01_JPRB, 0.15761E+01_JPRB ,&
&  0.17147E+01_JPRB, 0.18608E+01_JPRB, 0.19321E+01_JPRB, 0.19431E+01_JPRB, 0.19909E+01_JPRB, 0.20632E+01_JPRB, 0.22839E+01_JPRB ,&
&  0.26399E+01_JPRB, 0.29418E+01_JPRB, 0.30899E+01_JPRB, 0.28555E+01_JPRB, 0.22260E+01_JPRB, 0.21844E+01_JPRB, 0.23801E+01_JPRB ,&
&  0.24522E+01_JPRB, 0.22202E+01_JPRB, 0.20545E+01_JPRB, 0.20475E+01_JPRB, 0.20665E+01_JPRB, 0.21231E+01_JPRB, 0.21623E+01_JPRB ,&
&  0.21809E+01_JPRB, 0.22184E+01_JPRB, 0.23086E+01_JPRB, 0.24093E+01_JPRB, 0.25528E+01_JPRB, 0.26983E+01_JPRB, 0.28479E+01_JPRB ,&
&  0.29122E+01_JPRB, 0.29765E+01_JPRB, 0.29169E+01_JPRB, 0.27908E+01_JPRB, 0.27553E+01_JPRB, 0.28687E+01_JPRB, 0.30972E+01_JPRB ,&
&  0.33453E+01_JPRB, 0.34765E+01_JPRB, 0.37604E+01_JPRB, 0.39589E+01_JPRB, 0.38339E+01_JPRB, 0.36288E+01_JPRB, 0.35434E+01_JPRB ,&
&  0.35108E+01_JPRB, 0.37107E+01_JPRB, 0.27063E+01_JPRB, 0.33538E+01_JPRB, 0.33955E+01_JPRB, 0.29349E+01_JPRB, 0.25088E+01_JPRB ,&
&  0.22776E+01_JPRB, 0.20648E+01_JPRB, 0.18432E+01_JPRB, 0.19399E+01_JPRB, 0.20187E+01_JPRB, 0.21256E+01_JPRB, 0.23054E+01_JPRB ,&
&  0.22006E+01_JPRB, 0.16863E+01_JPRB, 0.12378E+01_JPRB, 0.10226E+01_JPRB, 0.11907E+01_JPRB, 0.15824E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.13779E+01_JPRB, 0.14733E+01_JPRB, 0.17414E+01_JPRB, 0.17943E+01_JPRB, 0.10417E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10905E+01_JPRB, 0.11373E+01_JPRB, 0.11510E+01_JPRB, 0.11632E+01_JPRB, 0.11739E+01_JPRB, 0.12060E+01_JPRB, 0.12399E+01_JPRB ,&
&  0.12504E+01_JPRB, 0.12478E+01_JPRB, 0.12278E+01_JPRB, 0.11835E+01_JPRB, 0.11163E+01_JPRB /)
ZBURD(:, 33,  5) = (/ &
&  0.86152E+00_JPRB, 0.85350E+00_JPRB, 0.84111E+00_JPRB, 0.85705E+00_JPRB, 0.87413E+00_JPRB, 0.87516E+00_JPRB, 0.89274E+00_JPRB ,&
&  0.91667E+00_JPRB, 0.94182E+00_JPRB, 0.96312E+00_JPRB, 0.98198E+00_JPRB, 0.10512E+01_JPRB, 0.11288E+01_JPRB, 0.12185E+01_JPRB ,&
&  0.12650E+01_JPRB, 0.13030E+01_JPRB, 0.13022E+01_JPRB, 0.12667E+01_JPRB, 0.12347E+01_JPRB, 0.11632E+01_JPRB, 0.10881E+01_JPRB ,&
&  0.11492E+01_JPRB, 0.14639E+01_JPRB, 0.19749E+01_JPRB, 0.24581E+01_JPRB, 0.28980E+01_JPRB, 0.25038E+01_JPRB, 0.26731E+01_JPRB ,&
&  0.23345E+01_JPRB, 0.18661E+01_JPRB, 0.18754E+01_JPRB, 0.19874E+01_JPRB, 0.21005E+01_JPRB, 0.21139E+01_JPRB, 0.21604E+01_JPRB ,&
&  0.21901E+01_JPRB, 0.22222E+01_JPRB, 0.22742E+01_JPRB, 0.23867E+01_JPRB, 0.24650E+01_JPRB, 0.25201E+01_JPRB, 0.25910E+01_JPRB ,&
&  0.26415E+01_JPRB, 0.26331E+01_JPRB, 0.26167E+01_JPRB, 0.24168E+01_JPRB, 0.25198E+01_JPRB, 0.27528E+01_JPRB, 0.29687E+01_JPRB ,&
&  0.31203E+01_JPRB, 0.31977E+01_JPRB, 0.34916E+01_JPRB, 0.35701E+01_JPRB, 0.35273E+01_JPRB, 0.35104E+01_JPRB, 0.36121E+01_JPRB ,&
&  0.35589E+01_JPRB, 0.31927E+01_JPRB, 0.23188E+01_JPRB, 0.26336E+01_JPRB, 0.24321E+01_JPRB, 0.20277E+01_JPRB, 0.20763E+01_JPRB ,&
&  0.18763E+01_JPRB, 0.16790E+01_JPRB, 0.14222E+01_JPRB, 0.14086E+01_JPRB, 0.15160E+01_JPRB, 0.16447E+01_JPRB, 0.19228E+01_JPRB ,&
&  0.16572E+01_JPRB, 0.13429E+01_JPRB, 0.10591E+01_JPRB, 0.10101E+01_JPRB, 0.10533E+01_JPRB, 0.11791E+01_JPRB, 0.12614E+01_JPRB ,&
&  0.12481E+01_JPRB, 0.11674E+01_JPRB, 0.12216E+01_JPRB, 0.13540E+01_JPRB, 0.10380E+01_JPRB, 0.92424E+00_JPRB, 0.94981E+00_JPRB ,&
&  0.97272E+00_JPRB, 0.98246E+00_JPRB, 0.98788E+00_JPRB, 0.98377E+00_JPRB, 0.99606E+00_JPRB, 0.10205E+01_JPRB, 0.10287E+01_JPRB ,&
&  0.10142E+01_JPRB, 0.96836E+00_JPRB, 0.92542E+00_JPRB, 0.89524E+00_JPRB, 0.87122E+00_JPRB /)
ZBURD(:, 34,  5) = (/ &
&  0.67962E+00_JPRB, 0.68085E+00_JPRB, 0.68410E+00_JPRB, 0.69406E+00_JPRB, 0.69604E+00_JPRB, 0.69337E+00_JPRB, 0.68021E+00_JPRB ,&
&  0.67082E+00_JPRB, 0.65529E+00_JPRB, 0.64689E+00_JPRB, 0.64374E+00_JPRB, 0.66396E+00_JPRB, 0.70532E+00_JPRB, 0.76211E+00_JPRB ,&
&  0.77621E+00_JPRB, 0.77071E+00_JPRB, 0.73502E+00_JPRB, 0.70851E+00_JPRB, 0.72399E+00_JPRB, 0.71969E+00_JPRB, 0.68997E+00_JPRB ,&
&  0.66815E+00_JPRB, 0.73539E+00_JPRB, 0.91867E+00_JPRB, 0.11935E+01_JPRB, 0.16845E+01_JPRB, 0.19617E+01_JPRB, 0.25061E+01_JPRB ,&
&  0.23695E+01_JPRB, 0.16603E+01_JPRB, 0.17398E+01_JPRB, 0.16675E+01_JPRB, 0.17777E+01_JPRB, 0.18379E+01_JPRB, 0.18610E+01_JPRB ,&
&  0.19133E+01_JPRB, 0.19583E+01_JPRB, 0.20008E+01_JPRB, 0.20523E+01_JPRB, 0.20754E+01_JPRB, 0.20540E+01_JPRB, 0.20306E+01_JPRB ,&
&  0.19994E+01_JPRB, 0.19404E+01_JPRB, 0.19499E+01_JPRB, 0.20360E+01_JPRB, 0.22829E+01_JPRB, 0.25577E+01_JPRB, 0.25947E+01_JPRB ,&
&  0.26369E+01_JPRB, 0.27936E+01_JPRB, 0.27374E+01_JPRB, 0.29268E+01_JPRB, 0.29646E+01_JPRB, 0.28927E+01_JPRB, 0.30871E+01_JPRB ,&
&  0.31441E+01_JPRB, 0.26335E+01_JPRB, 0.17933E+01_JPRB, 0.20058E+01_JPRB, 0.19480E+01_JPRB, 0.18275E+01_JPRB, 0.18739E+01_JPRB ,&
&  0.17343E+01_JPRB, 0.15958E+01_JPRB, 0.14479E+01_JPRB, 0.13608E+01_JPRB, 0.13577E+01_JPRB, 0.12936E+01_JPRB, 0.13090E+01_JPRB ,&
&  0.12034E+01_JPRB, 0.11533E+01_JPRB, 0.10942E+01_JPRB, 0.10606E+01_JPRB, 0.11398E+01_JPRB, 0.12846E+01_JPRB, 0.11480E+01_JPRB ,&
&  0.11506E+01_JPRB, 0.11241E+01_JPRB, 0.12775E+01_JPRB, 0.13701E+01_JPRB, 0.87735E+00_JPRB, 0.75921E+00_JPRB, 0.72941E+00_JPRB ,&
&  0.73504E+00_JPRB, 0.72888E+00_JPRB, 0.72226E+00_JPRB, 0.71002E+00_JPRB, 0.73973E+00_JPRB, 0.78989E+00_JPRB, 0.77590E+00_JPRB ,&
&  0.77220E+00_JPRB, 0.72679E+00_JPRB, 0.71095E+00_JPRB, 0.70697E+00_JPRB, 0.70259E+00_JPRB /)
ZBURD(:, 35,  5) = (/ &
&  0.62402E+00_JPRB, 0.61432E+00_JPRB, 0.60487E+00_JPRB, 0.59483E+00_JPRB, 0.60227E+00_JPRB, 0.60462E+00_JPRB, 0.61578E+00_JPRB ,&
&  0.62845E+00_JPRB, 0.62980E+00_JPRB, 0.63710E+00_JPRB, 0.63989E+00_JPRB, 0.65690E+00_JPRB, 0.67156E+00_JPRB, 0.70237E+00_JPRB ,&
&  0.73167E+00_JPRB, 0.74727E+00_JPRB, 0.76681E+00_JPRB, 0.78642E+00_JPRB, 0.79887E+00_JPRB, 0.81284E+00_JPRB, 0.79714E+00_JPRB ,&
&  0.77920E+00_JPRB, 0.78054E+00_JPRB, 0.81670E+00_JPRB, 0.95098E+00_JPRB, 0.12062E+01_JPRB, 0.15372E+01_JPRB, 0.21956E+01_JPRB ,&
&  0.21870E+01_JPRB, 0.14949E+01_JPRB, 0.13202E+01_JPRB, 0.11620E+01_JPRB, 0.12112E+01_JPRB, 0.12939E+01_JPRB, 0.12842E+01_JPRB ,&
&  0.12789E+01_JPRB, 0.12669E+01_JPRB, 0.12478E+01_JPRB, 0.12170E+01_JPRB, 0.11440E+01_JPRB, 0.10526E+01_JPRB, 0.98181E+00_JPRB ,&
&  0.94536E+00_JPRB, 0.90746E+00_JPRB, 0.10272E+01_JPRB, 0.13445E+01_JPRB, 0.16015E+01_JPRB, 0.20097E+01_JPRB, 0.18886E+01_JPRB ,&
&  0.19017E+01_JPRB, 0.19443E+01_JPRB, 0.19275E+01_JPRB, 0.20687E+01_JPRB, 0.21237E+01_JPRB, 0.21987E+01_JPRB, 0.22042E+01_JPRB ,&
&  0.23101E+01_JPRB, 0.20667E+01_JPRB, 0.15234E+01_JPRB, 0.17690E+01_JPRB, 0.17574E+01_JPRB, 0.17846E+01_JPRB, 0.16943E+01_JPRB ,&
&  0.15974E+01_JPRB, 0.14878E+01_JPRB, 0.13937E+01_JPRB, 0.13174E+01_JPRB, 0.13031E+01_JPRB, 0.12715E+01_JPRB, 0.11671E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.11274E+01_JPRB, 0.11034E+01_JPRB, 0.10570E+01_JPRB, 0.12065E+01_JPRB, 0.12079E+01_JPRB, 0.15273E+01_JPRB ,&
&  0.10892E+01_JPRB, 0.10016E+01_JPRB, 0.81307E+00_JPRB, 0.10094E+01_JPRB, 0.74317E+00_JPRB, 0.63025E+00_JPRB, 0.62920E+00_JPRB ,&
&  0.60869E+00_JPRB, 0.58606E+00_JPRB, 0.56770E+00_JPRB, 0.56524E+00_JPRB, 0.57822E+00_JPRB, 0.60773E+00_JPRB, 0.60664E+00_JPRB ,&
&  0.61063E+00_JPRB, 0.61700E+00_JPRB, 0.63058E+00_JPRB, 0.64891E+00_JPRB, 0.64275E+00_JPRB /)
ZBURD(:, 36,  5) = (/ &
&  0.63999E+00_JPRB, 0.64072E+00_JPRB, 0.64972E+00_JPRB, 0.66626E+00_JPRB, 0.67503E+00_JPRB, 0.68200E+00_JPRB, 0.68765E+00_JPRB ,&
&  0.70763E+00_JPRB, 0.73073E+00_JPRB, 0.75183E+00_JPRB, 0.77214E+00_JPRB, 0.79075E+00_JPRB, 0.80642E+00_JPRB, 0.84333E+00_JPRB ,&
&  0.86260E+00_JPRB, 0.89009E+00_JPRB, 0.90787E+00_JPRB, 0.92719E+00_JPRB, 0.94390E+00_JPRB, 0.93594E+00_JPRB, 0.94179E+00_JPRB ,&
&  0.95339E+00_JPRB, 0.98500E+00_JPRB, 0.10165E+01_JPRB, 0.10729E+01_JPRB, 0.11469E+01_JPRB, 0.11696E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.13197E+01_JPRB, 0.14377E+01_JPRB, 0.12415E+01_JPRB, 0.10600E+01_JPRB, 0.10106E+01_JPRB, 0.90185E+00_JPRB, 0.83943E+00_JPRB ,&
&  0.83562E+00_JPRB, 0.81361E+00_JPRB, 0.77516E+00_JPRB, 0.73207E+00_JPRB, 0.68693E+00_JPRB, 0.71462E+00_JPRB, 0.74265E+00_JPRB ,&
&  0.85761E+00_JPRB, 0.92777E+00_JPRB, 0.10385E+01_JPRB, 0.11543E+01_JPRB, 0.12765E+01_JPRB, 0.13363E+01_JPRB, 0.13064E+01_JPRB ,&
&  0.13068E+01_JPRB, 0.12858E+01_JPRB, 0.12380E+01_JPRB, 0.14075E+01_JPRB, 0.15369E+01_JPRB, 0.16029E+01_JPRB, 0.16248E+01_JPRB ,&
&  0.16287E+01_JPRB, 0.15335E+01_JPRB, 0.15361E+01_JPRB, 0.15738E+01_JPRB, 0.15456E+01_JPRB, 0.15732E+01_JPRB, 0.14958E+01_JPRB ,&
&  0.14015E+01_JPRB, 0.13340E+01_JPRB, 0.13092E+01_JPRB, 0.13109E+01_JPRB, 0.13299E+01_JPRB, 0.13295E+01_JPRB, 0.12299E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.11063E+01_JPRB, 0.10670E+01_JPRB, 0.10396E+01_JPRB, 0.10444E+01_JPRB, 0.18794E+01_JPRB, 0.25069E+01_JPRB ,&
&  0.10082E+01_JPRB, 0.74860E+00_JPRB, 0.52182E+00_JPRB, 0.65051E+00_JPRB, 0.61523E+00_JPRB, 0.62733E+00_JPRB, 0.69732E+00_JPRB ,&
&  0.70950E+00_JPRB, 0.69450E+00_JPRB, 0.62835E+00_JPRB, 0.56465E+00_JPRB, 0.58198E+00_JPRB, 0.58901E+00_JPRB, 0.58906E+00_JPRB ,&
&  0.60346E+00_JPRB, 0.60956E+00_JPRB, 0.61943E+00_JPRB, 0.63972E+00_JPRB, 0.63474E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART9',1,ZHOOK_HANDLE)
END SUBROUTINE PART9



SUBROUTINE PART10

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART10',0,ZHOOK_HANDLE)


ZBURD(:, 37,  5) = (/ &
&  0.66693E+00_JPRB, 0.68131E+00_JPRB, 0.70037E+00_JPRB, 0.72586E+00_JPRB, 0.75026E+00_JPRB, 0.76932E+00_JPRB, 0.78238E+00_JPRB ,&
&  0.80405E+00_JPRB, 0.83370E+00_JPRB, 0.85244E+00_JPRB, 0.86161E+00_JPRB, 0.88096E+00_JPRB, 0.89817E+00_JPRB, 0.92852E+00_JPRB ,&
&  0.94735E+00_JPRB, 0.99038E+00_JPRB, 0.10242E+01_JPRB, 0.10576E+01_JPRB, 0.10843E+01_JPRB, 0.11159E+01_JPRB, 0.11460E+01_JPRB ,&
&  0.11704E+01_JPRB, 0.12201E+01_JPRB, 0.12624E+01_JPRB, 0.12909E+01_JPRB, 0.13413E+01_JPRB, 0.14030E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.14154E+01_JPRB, 0.12991E+01_JPRB, 0.11344E+01_JPRB, 0.10098E+01_JPRB, 0.92606E+00_JPRB, 0.86310E+00_JPRB, 0.83044E+00_JPRB ,&
&  0.83168E+00_JPRB, 0.86056E+00_JPRB, 0.88458E+00_JPRB, 0.85914E+00_JPRB, 0.81414E+00_JPRB, 0.88166E+00_JPRB, 0.10275E+01_JPRB ,&
&  0.11189E+01_JPRB, 0.11653E+01_JPRB, 0.12067E+01_JPRB, 0.12103E+01_JPRB, 0.12569E+01_JPRB, 0.13149E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.13275E+01_JPRB, 0.12811E+01_JPRB, 0.11641E+01_JPRB, 0.13726E+01_JPRB, 0.13903E+01_JPRB, 0.13584E+01_JPRB, 0.12682E+01_JPRB ,&
&  0.11757E+01_JPRB, 0.11222E+01_JPRB, 0.12027E+01_JPRB, 0.13526E+01_JPRB, 0.13362E+01_JPRB, 0.13118E+01_JPRB, 0.12275E+01_JPRB ,&
&  0.11534E+01_JPRB, 0.11358E+01_JPRB, 0.11613E+01_JPRB, 0.12134E+01_JPRB, 0.12528E+01_JPRB, 0.12509E+01_JPRB, 0.11690E+01_JPRB ,&
&  0.10879E+01_JPRB, 0.10040E+01_JPRB, 0.94606E+00_JPRB, 0.91070E+00_JPRB, 0.93208E+00_JPRB, 0.10352E+01_JPRB, 0.14803E+01_JPRB ,&
&  0.85385E+00_JPRB, 0.54203E+00_JPRB, 0.56655E+00_JPRB, 0.57615E+00_JPRB, 0.60889E+00_JPRB, 0.63685E+00_JPRB, 0.79588E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.11510E+01_JPRB, 0.11257E+01_JPRB, 0.86176E+00_JPRB, 0.68982E+00_JPRB, 0.63735E+00_JPRB, 0.63186E+00_JPRB ,&
&  0.64121E+00_JPRB, 0.63944E+00_JPRB, 0.64248E+00_JPRB, 0.64830E+00_JPRB, 0.65836E+00_JPRB /)
ZBURD(:, 38,  5) = (/ &
&  0.64828E+00_JPRB, 0.65852E+00_JPRB, 0.67211E+00_JPRB, 0.68667E+00_JPRB, 0.70466E+00_JPRB, 0.72304E+00_JPRB, 0.73718E+00_JPRB ,&
&  0.75269E+00_JPRB, 0.77212E+00_JPRB, 0.79090E+00_JPRB, 0.80400E+00_JPRB, 0.81751E+00_JPRB, 0.84261E+00_JPRB, 0.87194E+00_JPRB ,&
&  0.88485E+00_JPRB, 0.93124E+00_JPRB, 0.95415E+00_JPRB, 0.99932E+00_JPRB, 0.10419E+01_JPRB, 0.10926E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11662E+01_JPRB, 0.12220E+01_JPRB, 0.12435E+01_JPRB, 0.12346E+01_JPRB, 0.12531E+01_JPRB, 0.13578E+01_JPRB, 0.11210E+01_JPRB ,&
&  0.13325E+01_JPRB, 0.12644E+01_JPRB, 0.11287E+01_JPRB, 0.10642E+01_JPRB, 0.10119E+01_JPRB, 0.99252E+00_JPRB, 0.99702E+00_JPRB ,&
&  0.10237E+01_JPRB, 0.10982E+01_JPRB, 0.10657E+01_JPRB, 0.10796E+01_JPRB, 0.11092E+01_JPRB, 0.12199E+01_JPRB, 0.13348E+01_JPRB ,&
&  0.13633E+01_JPRB, 0.14436E+01_JPRB, 0.14731E+01_JPRB, 0.15076E+01_JPRB, 0.15752E+01_JPRB, 0.16402E+01_JPRB, 0.16810E+01_JPRB ,&
&  0.16604E+01_JPRB, 0.16276E+01_JPRB, 0.15134E+01_JPRB, 0.15933E+01_JPRB, 0.15346E+01_JPRB, 0.13295E+01_JPRB, 0.11264E+01_JPRB ,&
&  0.90662E+00_JPRB, 0.96775E+00_JPRB, 0.10417E+01_JPRB, 0.11471E+01_JPRB, 0.10994E+01_JPRB, 0.10473E+01_JPRB, 0.92288E+00_JPRB ,&
&  0.87392E+00_JPRB, 0.86867E+00_JPRB, 0.92077E+00_JPRB, 0.99134E+00_JPRB, 0.10341E+01_JPRB, 0.10263E+01_JPRB, 0.96422E+00_JPRB ,&
&  0.90418E+00_JPRB, 0.82915E+00_JPRB, 0.79292E+00_JPRB, 0.78922E+00_JPRB, 0.77992E+00_JPRB, 0.74116E+00_JPRB, 0.96121E+00_JPRB ,&
&  0.73922E+00_JPRB, 0.66141E+00_JPRB, 0.76941E+00_JPRB, 0.45717E+00_JPRB, 0.63646E+00_JPRB, 0.80245E+00_JPRB, 0.10675E+01_JPRB ,&
&  0.12717E+01_JPRB, 0.17840E+01_JPRB, 0.22288E+01_JPRB, 0.16960E+01_JPRB, 0.11781E+01_JPRB, 0.68966E+00_JPRB, 0.59820E+00_JPRB ,&
&  0.60721E+00_JPRB, 0.61250E+00_JPRB, 0.62166E+00_JPRB, 0.62242E+00_JPRB, 0.63526E+00_JPRB /)
ZBURD(:, 39,  5) = (/ &
&  0.59868E+00_JPRB, 0.59330E+00_JPRB, 0.59293E+00_JPRB, 0.58366E+00_JPRB, 0.57930E+00_JPRB, 0.58047E+00_JPRB, 0.56911E+00_JPRB ,&
&  0.57794E+00_JPRB, 0.59531E+00_JPRB, 0.60222E+00_JPRB, 0.62244E+00_JPRB, 0.63489E+00_JPRB, 0.67455E+00_JPRB, 0.73287E+00_JPRB ,&
&  0.77435E+00_JPRB, 0.86389E+00_JPRB, 0.91093E+00_JPRB, 0.96779E+00_JPRB, 0.10162E+01_JPRB, 0.10677E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.11721E+01_JPRB, 0.12463E+01_JPRB, 0.13250E+01_JPRB, 0.13803E+01_JPRB, 0.13898E+01_JPRB, 0.15447E+01_JPRB, 0.12102E+01_JPRB ,&
&  0.13365E+01_JPRB, 0.13201E+01_JPRB, 0.12305E+01_JPRB, 0.11799E+01_JPRB, 0.11291E+01_JPRB, 0.10989E+01_JPRB, 0.10809E+01_JPRB ,&
&  0.10957E+01_JPRB, 0.11069E+01_JPRB, 0.11141E+01_JPRB, 0.12005E+01_JPRB, 0.12996E+01_JPRB, 0.13651E+01_JPRB, 0.14501E+01_JPRB ,&
&  0.15286E+01_JPRB, 0.16004E+01_JPRB, 0.16983E+01_JPRB, 0.18015E+01_JPRB, 0.19104E+01_JPRB, 0.19834E+01_JPRB, 0.20611E+01_JPRB ,&
&  0.20904E+01_JPRB, 0.21146E+01_JPRB, 0.19867E+01_JPRB, 0.17678E+01_JPRB, 0.15434E+01_JPRB, 0.13113E+01_JPRB, 0.10773E+01_JPRB ,&
&  0.89022E+00_JPRB, 0.86211E+00_JPRB, 0.87881E+00_JPRB, 0.91608E+00_JPRB, 0.84337E+00_JPRB, 0.78775E+00_JPRB, 0.75557E+00_JPRB ,&
&  0.72009E+00_JPRB, 0.67043E+00_JPRB, 0.70042E+00_JPRB, 0.75341E+00_JPRB, 0.77119E+00_JPRB, 0.76364E+00_JPRB, 0.74687E+00_JPRB ,&
&  0.71874E+00_JPRB, 0.67670E+00_JPRB, 0.67909E+00_JPRB, 0.69392E+00_JPRB, 0.73535E+00_JPRB, 0.81199E+00_JPRB, 0.73328E+00_JPRB ,&
&  0.80045E+00_JPRB, 0.62902E+00_JPRB, 0.62182E+00_JPRB, 0.63869E+00_JPRB, 0.67490E+00_JPRB, 0.86071E+00_JPRB, 0.11923E+01_JPRB ,&
&  0.17883E+01_JPRB, 0.20287E+01_JPRB, 0.22689E+01_JPRB, 0.28171E+01_JPRB, 0.24536E+01_JPRB, 0.18177E+01_JPRB, 0.55155E+00_JPRB ,&
&  0.54958E+00_JPRB, 0.55432E+00_JPRB, 0.57875E+00_JPRB, 0.58764E+00_JPRB, 0.59230E+00_JPRB /)
ZBURD(:, 40,  5) = (/ &
&  0.51220E+00_JPRB, 0.49722E+00_JPRB, 0.48939E+00_JPRB, 0.48408E+00_JPRB, 0.46889E+00_JPRB, 0.46987E+00_JPRB, 0.46735E+00_JPRB ,&
&  0.48452E+00_JPRB, 0.50676E+00_JPRB, 0.54589E+00_JPRB, 0.59321E+00_JPRB, 0.65521E+00_JPRB, 0.70058E+00_JPRB, 0.75603E+00_JPRB ,&
&  0.79116E+00_JPRB, 0.83658E+00_JPRB, 0.88698E+00_JPRB, 0.93130E+00_JPRB, 0.98411E+00_JPRB, 0.10233E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.12130E+01_JPRB, 0.13031E+01_JPRB, 0.13831E+01_JPRB, 0.14473E+01_JPRB, 0.14770E+01_JPRB, 0.11562E+01_JPRB ,&
&  0.13590E+01_JPRB, 0.13759E+01_JPRB, 0.13662E+01_JPRB, 0.13070E+01_JPRB, 0.12265E+01_JPRB, 0.11551E+01_JPRB, 0.11386E+01_JPRB ,&
&  0.11383E+01_JPRB, 0.11313E+01_JPRB, 0.11232E+01_JPRB, 0.11424E+01_JPRB, 0.13325E+01_JPRB, 0.13937E+01_JPRB, 0.14730E+01_JPRB ,&
&  0.15590E+01_JPRB, 0.16547E+01_JPRB, 0.17536E+01_JPRB, 0.18584E+01_JPRB, 0.19350E+01_JPRB, 0.20231E+01_JPRB, 0.21187E+01_JPRB ,&
&  0.21784E+01_JPRB, 0.22289E+01_JPRB, 0.21470E+01_JPRB, 0.17224E+01_JPRB, 0.14733E+01_JPRB, 0.12712E+01_JPRB, 0.97079E+00_JPRB ,&
&  0.81441E+00_JPRB, 0.77212E+00_JPRB, 0.79068E+00_JPRB, 0.79710E+00_JPRB, 0.73794E+00_JPRB, 0.67817E+00_JPRB, 0.63922E+00_JPRB ,&
&  0.58104E+00_JPRB, 0.57021E+00_JPRB, 0.56443E+00_JPRB, 0.60012E+00_JPRB, 0.61836E+00_JPRB, 0.59417E+00_JPRB, 0.58481E+00_JPRB ,&
&  0.56142E+00_JPRB, 0.54071E+00_JPRB, 0.57863E+00_JPRB, 0.64246E+00_JPRB, 0.77940E+00_JPRB, 0.92518E+00_JPRB, 0.10194E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.64589E+00_JPRB, 0.62576E+00_JPRB, 0.62364E+00_JPRB, 0.64525E+00_JPRB, 0.69353E+00_JPRB, 0.78509E+00_JPRB ,&
&  0.98574E+00_JPRB, 0.13391E+01_JPRB, 0.11792E+01_JPRB, 0.89920E+00_JPRB, 0.96557E+00_JPRB, 0.88835E+00_JPRB, 0.49813E+00_JPRB ,&
&  0.49989E+00_JPRB, 0.49892E+00_JPRB, 0.51479E+00_JPRB, 0.51973E+00_JPRB, 0.52250E+00_JPRB /)
ZBURD(:, 41,  5) = (/ &
&  0.45361E+00_JPRB, 0.43748E+00_JPRB, 0.41955E+00_JPRB, 0.41028E+00_JPRB, 0.40289E+00_JPRB, 0.39438E+00_JPRB, 0.40514E+00_JPRB ,&
&  0.43362E+00_JPRB, 0.45930E+00_JPRB, 0.50260E+00_JPRB, 0.56698E+00_JPRB, 0.62878E+00_JPRB, 0.66495E+00_JPRB, 0.70500E+00_JPRB ,&
&  0.73978E+00_JPRB, 0.77803E+00_JPRB, 0.80918E+00_JPRB, 0.83851E+00_JPRB, 0.87719E+00_JPRB, 0.92702E+00_JPRB, 0.99496E+00_JPRB ,&
&  0.10767E+01_JPRB, 0.11597E+01_JPRB, 0.12623E+01_JPRB, 0.13891E+01_JPRB, 0.15105E+01_JPRB, 0.15577E+01_JPRB, 0.16340E+01_JPRB ,&
&  0.11644E+01_JPRB, 0.14097E+01_JPRB, 0.14008E+01_JPRB, 0.13541E+01_JPRB, 0.12961E+01_JPRB, 0.12445E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.11981E+01_JPRB, 0.11207E+01_JPRB, 0.11046E+01_JPRB, 0.11173E+01_JPRB, 0.12253E+01_JPRB, 0.13013E+01_JPRB, 0.13804E+01_JPRB ,&
&  0.14611E+01_JPRB, 0.15535E+01_JPRB, 0.16499E+01_JPRB, 0.17446E+01_JPRB, 0.18229E+01_JPRB, 0.19172E+01_JPRB, 0.19826E+01_JPRB ,&
&  0.20593E+01_JPRB, 0.21193E+01_JPRB, 0.21382E+01_JPRB, 0.17773E+01_JPRB, 0.14734E+01_JPRB, 0.13016E+01_JPRB, 0.10308E+01_JPRB ,&
&  0.80165E+00_JPRB, 0.70531E+00_JPRB, 0.72574E+00_JPRB, 0.75124E+00_JPRB, 0.71262E+00_JPRB, 0.66680E+00_JPRB, 0.61340E+00_JPRB ,&
&  0.58004E+00_JPRB, 0.52814E+00_JPRB, 0.51074E+00_JPRB, 0.52404E+00_JPRB, 0.55628E+00_JPRB, 0.53315E+00_JPRB, 0.50326E+00_JPRB ,&
&  0.48309E+00_JPRB, 0.47760E+00_JPRB, 0.52851E+00_JPRB, 0.58853E+00_JPRB, 0.66754E+00_JPRB, 0.71270E+00_JPRB, 0.73065E+00_JPRB ,&
&  0.77433E+00_JPRB, 0.67433E+00_JPRB, 0.65647E+00_JPRB, 0.66030E+00_JPRB, 0.65483E+00_JPRB, 0.65715E+00_JPRB, 0.66953E+00_JPRB ,&
&  0.70183E+00_JPRB, 0.76284E+00_JPRB, 0.77946E+00_JPRB, 0.72237E+00_JPRB, 0.65926E+00_JPRB, 0.60953E+00_JPRB, 0.56330E+00_JPRB ,&
&  0.56531E+00_JPRB, 0.55180E+00_JPRB, 0.53721E+00_JPRB, 0.51465E+00_JPRB, 0.48537E+00_JPRB /)
ZBURD(:, 42,  5) = (/ &
&  0.46483E+00_JPRB, 0.43684E+00_JPRB, 0.39704E+00_JPRB, 0.36233E+00_JPRB, 0.38535E+00_JPRB, 0.38599E+00_JPRB, 0.40151E+00_JPRB ,&
&  0.40762E+00_JPRB, 0.43300E+00_JPRB, 0.45874E+00_JPRB, 0.49702E+00_JPRB, 0.54648E+00_JPRB, 0.59594E+00_JPRB, 0.63728E+00_JPRB ,&
&  0.67443E+00_JPRB, 0.71062E+00_JPRB, 0.73822E+00_JPRB, 0.74958E+00_JPRB, 0.76871E+00_JPRB, 0.80070E+00_JPRB, 0.83982E+00_JPRB ,&
&  0.89324E+00_JPRB, 0.96414E+00_JPRB, 0.10617E+01_JPRB, 0.11854E+01_JPRB, 0.13296E+01_JPRB, 0.14077E+01_JPRB, 0.14577E+01_JPRB ,&
&  0.83410E+00_JPRB, 0.12442E+01_JPRB, 0.14579E+01_JPRB, 0.13723E+01_JPRB, 0.13327E+01_JPRB, 0.13023E+01_JPRB, 0.12723E+01_JPRB ,&
&  0.11942E+01_JPRB, 0.10901E+01_JPRB, 0.10519E+01_JPRB, 0.11434E+01_JPRB, 0.10923E+01_JPRB, 0.11708E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.12939E+01_JPRB, 0.13581E+01_JPRB, 0.14330E+01_JPRB, 0.15290E+01_JPRB, 0.16281E+01_JPRB, 0.17143E+01_JPRB, 0.17931E+01_JPRB ,&
&  0.18933E+01_JPRB, 0.19749E+01_JPRB, 0.19923E+01_JPRB, 0.15774E+01_JPRB, 0.14517E+01_JPRB, 0.13738E+01_JPRB, 0.12482E+01_JPRB ,&
&  0.87974E+00_JPRB, 0.71469E+00_JPRB, 0.71771E+00_JPRB, 0.74626E+00_JPRB, 0.71798E+00_JPRB, 0.68555E+00_JPRB, 0.64781E+00_JPRB ,&
&  0.59908E+00_JPRB, 0.55630E+00_JPRB, 0.52951E+00_JPRB, 0.52017E+00_JPRB, 0.53939E+00_JPRB, 0.54213E+00_JPRB, 0.52620E+00_JPRB ,&
&  0.50727E+00_JPRB, 0.51519E+00_JPRB, 0.55583E+00_JPRB, 0.59071E+00_JPRB, 0.63978E+00_JPRB, 0.67261E+00_JPRB, 0.70582E+00_JPRB ,&
&  0.72192E+00_JPRB, 0.68789E+00_JPRB, 0.69540E+00_JPRB, 0.71154E+00_JPRB, 0.71303E+00_JPRB, 0.70968E+00_JPRB, 0.69758E+00_JPRB ,&
&  0.68261E+00_JPRB, 0.69375E+00_JPRB, 0.69819E+00_JPRB, 0.70235E+00_JPRB, 0.68382E+00_JPRB, 0.66156E+00_JPRB, 0.65109E+00_JPRB ,&
&  0.63275E+00_JPRB, 0.60612E+00_JPRB, 0.57960E+00_JPRB, 0.53844E+00_JPRB, 0.49905E+00_JPRB /)
ZBURD(:, 43,  5) = (/ &
&  0.48768E+00_JPRB, 0.45567E+00_JPRB, 0.41576E+00_JPRB, 0.40029E+00_JPRB, 0.42415E+00_JPRB, 0.44432E+00_JPRB, 0.43963E+00_JPRB ,&
&  0.43354E+00_JPRB, 0.42388E+00_JPRB, 0.41898E+00_JPRB, 0.42864E+00_JPRB, 0.46540E+00_JPRB, 0.51838E+00_JPRB, 0.55931E+00_JPRB ,&
&  0.59798E+00_JPRB, 0.62854E+00_JPRB, 0.65295E+00_JPRB, 0.66106E+00_JPRB, 0.65657E+00_JPRB, 0.66815E+00_JPRB, 0.68735E+00_JPRB ,&
&  0.72165E+00_JPRB, 0.77149E+00_JPRB, 0.83841E+00_JPRB, 0.93764E+00_JPRB, 0.10781E+01_JPRB, 0.12500E+01_JPRB, 0.14254E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.51913E+00_JPRB, 0.13091E+01_JPRB, 0.15482E+01_JPRB, 0.14615E+01_JPRB, 0.14464E+01_JPRB, 0.13380E+01_JPRB ,&
&  0.12338E+01_JPRB, 0.12126E+01_JPRB, 0.11281E+01_JPRB, 0.11555E+01_JPRB, 0.10613E+01_JPRB, 0.10638E+01_JPRB, 0.10758E+01_JPRB ,&
&  0.11019E+01_JPRB, 0.11355E+01_JPRB, 0.11887E+01_JPRB, 0.12544E+01_JPRB, 0.13321E+01_JPRB, 0.14531E+01_JPRB, 0.15403E+01_JPRB ,&
&  0.16267E+01_JPRB, 0.16926E+01_JPRB, 0.17009E+01_JPRB, 0.12967E+01_JPRB, 0.11184E+01_JPRB, 0.10314E+01_JPRB, 0.96027E+00_JPRB ,&
&  0.87040E+00_JPRB, 0.75811E+00_JPRB, 0.73843E+00_JPRB, 0.75770E+00_JPRB, 0.73844E+00_JPRB, 0.69256E+00_JPRB, 0.69508E+00_JPRB ,&
&  0.64058E+00_JPRB, 0.59696E+00_JPRB, 0.56521E+00_JPRB, 0.54139E+00_JPRB, 0.54844E+00_JPRB, 0.55781E+00_JPRB, 0.56452E+00_JPRB ,&
&  0.55759E+00_JPRB, 0.56344E+00_JPRB, 0.58586E+00_JPRB, 0.59635E+00_JPRB, 0.60949E+00_JPRB, 0.62740E+00_JPRB, 0.64277E+00_JPRB ,&
&  0.68891E+00_JPRB, 0.70400E+00_JPRB, 0.72561E+00_JPRB, 0.75459E+00_JPRB, 0.76952E+00_JPRB, 0.74845E+00_JPRB, 0.74314E+00_JPRB ,&
&  0.74264E+00_JPRB, 0.74221E+00_JPRB, 0.72358E+00_JPRB, 0.73053E+00_JPRB, 0.71248E+00_JPRB, 0.69880E+00_JPRB, 0.69826E+00_JPRB ,&
&  0.70730E+00_JPRB, 0.70072E+00_JPRB, 0.66102E+00_JPRB, 0.57584E+00_JPRB, 0.52866E+00_JPRB /)
ZBURD(:, 44,  5) = (/ &
&  0.51920E+00_JPRB, 0.47182E+00_JPRB, 0.43508E+00_JPRB, 0.43603E+00_JPRB, 0.45717E+00_JPRB, 0.46387E+00_JPRB, 0.46335E+00_JPRB ,&
&  0.45376E+00_JPRB, 0.44208E+00_JPRB, 0.42420E+00_JPRB, 0.41592E+00_JPRB, 0.42070E+00_JPRB, 0.43365E+00_JPRB, 0.47112E+00_JPRB ,&
&  0.50851E+00_JPRB, 0.54327E+00_JPRB, 0.57156E+00_JPRB, 0.57389E+00_JPRB, 0.55754E+00_JPRB, 0.55636E+00_JPRB, 0.56661E+00_JPRB ,&
&  0.57869E+00_JPRB, 0.60785E+00_JPRB, 0.64860E+00_JPRB, 0.70636E+00_JPRB, 0.79613E+00_JPRB, 0.94874E+00_JPRB, 0.11914E+01_JPRB ,&
&  0.16166E+01_JPRB, 0.11160E+01_JPRB, 0.74634E+00_JPRB, 0.14124E+01_JPRB, 0.16983E+01_JPRB, 0.17627E+01_JPRB, 0.14870E+01_JPRB ,&
&  0.14341E+01_JPRB, 0.13464E+01_JPRB, 0.12583E+01_JPRB, 0.12448E+01_JPRB, 0.11020E+01_JPRB, 0.10510E+01_JPRB, 0.10036E+01_JPRB ,&
&  0.97091E+00_JPRB, 0.98603E+00_JPRB, 0.10220E+01_JPRB, 0.10557E+01_JPRB, 0.10882E+01_JPRB, 0.11707E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.13834E+01_JPRB, 0.14427E+01_JPRB, 0.14554E+01_JPRB, 0.11528E+01_JPRB, 0.10238E+01_JPRB, 0.96507E+00_JPRB, 0.93298E+00_JPRB ,&
&  0.86280E+00_JPRB, 0.83234E+00_JPRB, 0.82369E+00_JPRB, 0.79073E+00_JPRB, 0.75508E+00_JPRB, 0.72507E+00_JPRB, 0.73716E+00_JPRB ,&
&  0.69447E+00_JPRB, 0.65251E+00_JPRB, 0.62320E+00_JPRB, 0.60616E+00_JPRB, 0.60253E+00_JPRB, 0.60534E+00_JPRB, 0.60202E+00_JPRB ,&
&  0.61039E+00_JPRB, 0.61745E+00_JPRB, 0.62898E+00_JPRB, 0.62507E+00_JPRB, 0.62154E+00_JPRB, 0.62526E+00_JPRB, 0.62280E+00_JPRB ,&
&  0.66252E+00_JPRB, 0.71326E+00_JPRB, 0.74480E+00_JPRB, 0.78298E+00_JPRB, 0.79060E+00_JPRB, 0.78492E+00_JPRB, 0.80678E+00_JPRB ,&
&  0.80050E+00_JPRB, 0.80666E+00_JPRB, 0.76634E+00_JPRB, 0.77604E+00_JPRB, 0.74616E+00_JPRB, 0.71750E+00_JPRB, 0.70432E+00_JPRB ,&
&  0.73393E+00_JPRB, 0.77642E+00_JPRB, 0.76181E+00_JPRB, 0.60466E+00_JPRB, 0.54912E+00_JPRB /)
ZBURD(:, 45,  5) = (/ &
&  0.50620E+00_JPRB, 0.48942E+00_JPRB, 0.45040E+00_JPRB, 0.45450E+00_JPRB, 0.45718E+00_JPRB, 0.46464E+00_JPRB, 0.46445E+00_JPRB ,&
&  0.46381E+00_JPRB, 0.46525E+00_JPRB, 0.46756E+00_JPRB, 0.45913E+00_JPRB, 0.43532E+00_JPRB, 0.42484E+00_JPRB, 0.42744E+00_JPRB ,&
&  0.43913E+00_JPRB, 0.46541E+00_JPRB, 0.49656E+00_JPRB, 0.50162E+00_JPRB, 0.50145E+00_JPRB, 0.49687E+00_JPRB, 0.49346E+00_JPRB ,&
&  0.49746E+00_JPRB, 0.50615E+00_JPRB, 0.51959E+00_JPRB, 0.54988E+00_JPRB, 0.60416E+00_JPRB, 0.68922E+00_JPRB, 0.83522E+00_JPRB ,&
&  0.12044E+01_JPRB, 0.12857E+01_JPRB, 0.61996E+00_JPRB, 0.11701E+01_JPRB, 0.17500E+01_JPRB, 0.18657E+01_JPRB, 0.17896E+01_JPRB ,&
&  0.16994E+01_JPRB, 0.15222E+01_JPRB, 0.14053E+01_JPRB, 0.12831E+01_JPRB, 0.11133E+01_JPRB, 0.10225E+01_JPRB, 0.94506E+00_JPRB ,&
&  0.88333E+00_JPRB, 0.87692E+00_JPRB, 0.90918E+00_JPRB, 0.93746E+00_JPRB, 0.94079E+00_JPRB, 0.96251E+00_JPRB, 0.10387E+01_JPRB ,&
&  0.11550E+01_JPRB, 0.12767E+01_JPRB, 0.13256E+01_JPRB, 0.10999E+01_JPRB, 0.10042E+01_JPRB, 0.10149E+01_JPRB, 0.99169E+00_JPRB ,&
&  0.94442E+00_JPRB, 0.90452E+00_JPRB, 0.86634E+00_JPRB, 0.82252E+00_JPRB, 0.77699E+00_JPRB, 0.76269E+00_JPRB, 0.77789E+00_JPRB ,&
&  0.74706E+00_JPRB, 0.70282E+00_JPRB, 0.67339E+00_JPRB, 0.65642E+00_JPRB, 0.64446E+00_JPRB, 0.63957E+00_JPRB, 0.63388E+00_JPRB ,&
&  0.63399E+00_JPRB, 0.64548E+00_JPRB, 0.65969E+00_JPRB, 0.66038E+00_JPRB, 0.65078E+00_JPRB, 0.63638E+00_JPRB, 0.62769E+00_JPRB ,&
&  0.64682E+00_JPRB, 0.68762E+00_JPRB, 0.74631E+00_JPRB, 0.77013E+00_JPRB, 0.79241E+00_JPRB, 0.82162E+00_JPRB, 0.82663E+00_JPRB ,&
&  0.83776E+00_JPRB, 0.86731E+00_JPRB, 0.79045E+00_JPRB, 0.78188E+00_JPRB, 0.79800E+00_JPRB, 0.74871E+00_JPRB, 0.69766E+00_JPRB ,&
&  0.71129E+00_JPRB, 0.76948E+00_JPRB, 0.87866E+00_JPRB, 0.61029E+00_JPRB, 0.54652E+00_JPRB /)
ZBURD(:, 46,  5) = (/ &
&  0.50045E+00_JPRB, 0.47376E+00_JPRB, 0.46656E+00_JPRB, 0.46410E+00_JPRB, 0.46643E+00_JPRB, 0.47452E+00_JPRB, 0.48334E+00_JPRB ,&
&  0.48470E+00_JPRB, 0.48484E+00_JPRB, 0.48532E+00_JPRB, 0.48161E+00_JPRB, 0.46997E+00_JPRB, 0.45794E+00_JPRB, 0.44609E+00_JPRB ,&
&  0.44300E+00_JPRB, 0.44478E+00_JPRB, 0.45786E+00_JPRB, 0.46533E+00_JPRB, 0.46624E+00_JPRB, 0.46383E+00_JPRB, 0.46484E+00_JPRB ,&
&  0.46860E+00_JPRB, 0.46832E+00_JPRB, 0.46714E+00_JPRB, 0.48029E+00_JPRB, 0.50718E+00_JPRB, 0.56023E+00_JPRB, 0.65286E+00_JPRB ,&
&  0.91273E+00_JPRB, 0.11432E+01_JPRB, 0.11866E+01_JPRB, 0.18152E+01_JPRB, 0.21930E+01_JPRB, 0.21304E+01_JPRB, 0.20590E+01_JPRB ,&
&  0.19233E+01_JPRB, 0.17271E+01_JPRB, 0.16431E+01_JPRB, 0.13575E+01_JPRB, 0.11546E+01_JPRB, 0.10054E+01_JPRB, 0.89402E+00_JPRB ,&
&  0.80641E+00_JPRB, 0.76698E+00_JPRB, 0.78090E+00_JPRB, 0.81764E+00_JPRB, 0.84161E+00_JPRB, 0.84837E+00_JPRB, 0.87908E+00_JPRB ,&
&  0.96085E+00_JPRB, 0.11143E+01_JPRB, 0.12352E+01_JPRB, 0.11491E+01_JPRB, 0.98759E+00_JPRB, 0.10622E+01_JPRB, 0.11410E+01_JPRB ,&
&  0.12561E+01_JPRB, 0.10438E+01_JPRB, 0.90873E+00_JPRB, 0.84900E+00_JPRB, 0.79638E+00_JPRB, 0.84138E+00_JPRB, 0.81702E+00_JPRB ,&
&  0.78487E+00_JPRB, 0.74749E+00_JPRB, 0.71271E+00_JPRB, 0.69089E+00_JPRB, 0.67399E+00_JPRB, 0.66296E+00_JPRB, 0.65467E+00_JPRB ,&
&  0.65138E+00_JPRB, 0.65272E+00_JPRB, 0.66581E+00_JPRB, 0.67420E+00_JPRB, 0.67584E+00_JPRB, 0.66533E+00_JPRB, 0.64937E+00_JPRB ,&
&  0.64136E+00_JPRB, 0.66071E+00_JPRB, 0.68175E+00_JPRB, 0.71539E+00_JPRB, 0.75025E+00_JPRB, 0.76801E+00_JPRB, 0.74896E+00_JPRB ,&
&  0.74444E+00_JPRB, 0.78008E+00_JPRB, 0.80210E+00_JPRB, 0.82863E+00_JPRB, 0.82980E+00_JPRB, 0.80864E+00_JPRB, 0.70928E+00_JPRB ,&
&  0.65731E+00_JPRB, 0.62101E+00_JPRB, 0.59572E+00_JPRB, 0.57183E+00_JPRB, 0.53126E+00_JPRB /)
ZBURD(:, 47,  5) = (/ &
&  0.50328E+00_JPRB, 0.48531E+00_JPRB, 0.47935E+00_JPRB, 0.48154E+00_JPRB, 0.48370E+00_JPRB, 0.48724E+00_JPRB, 0.49721E+00_JPRB ,&
&  0.50197E+00_JPRB, 0.50323E+00_JPRB, 0.50207E+00_JPRB, 0.49980E+00_JPRB, 0.49175E+00_JPRB, 0.48214E+00_JPRB, 0.47119E+00_JPRB ,&
&  0.46268E+00_JPRB, 0.44257E+00_JPRB, 0.43190E+00_JPRB, 0.43901E+00_JPRB, 0.44274E+00_JPRB, 0.44273E+00_JPRB, 0.44399E+00_JPRB ,&
&  0.44427E+00_JPRB, 0.44831E+00_JPRB, 0.44848E+00_JPRB, 0.45549E+00_JPRB, 0.46958E+00_JPRB, 0.50646E+00_JPRB, 0.56428E+00_JPRB ,&
&  0.74390E+00_JPRB, 0.10697E+01_JPRB, 0.10822E+01_JPRB, 0.21166E+01_JPRB, 0.23328E+01_JPRB, 0.22641E+01_JPRB, 0.20090E+01_JPRB ,&
&  0.18509E+01_JPRB, 0.18929E+01_JPRB, 0.15253E+01_JPRB, 0.12737E+01_JPRB, 0.11293E+01_JPRB, 0.10001E+01_JPRB, 0.86148E+00_JPRB ,&
&  0.75421E+00_JPRB, 0.69854E+00_JPRB, 0.67985E+00_JPRB, 0.69399E+00_JPRB, 0.72894E+00_JPRB, 0.76041E+00_JPRB, 0.78776E+00_JPRB ,&
&  0.83517E+00_JPRB, 0.94856E+00_JPRB, 0.11086E+01_JPRB, 0.10775E+01_JPRB, 0.10199E+01_JPRB, 0.11188E+01_JPRB, 0.13105E+01_JPRB ,&
&  0.15249E+01_JPRB, 0.13277E+01_JPRB, 0.10132E+01_JPRB, 0.88692E+00_JPRB, 0.85096E+00_JPRB, 0.87458E+00_JPRB, 0.86312E+00_JPRB ,&
&  0.82512E+00_JPRB, 0.79147E+00_JPRB, 0.74962E+00_JPRB, 0.72113E+00_JPRB, 0.70080E+00_JPRB, 0.67498E+00_JPRB, 0.65836E+00_JPRB ,&
&  0.65240E+00_JPRB, 0.65094E+00_JPRB, 0.65529E+00_JPRB, 0.65631E+00_JPRB, 0.66129E+00_JPRB, 0.66750E+00_JPRB, 0.67147E+00_JPRB ,&
&  0.66728E+00_JPRB, 0.65050E+00_JPRB, 0.63908E+00_JPRB, 0.65383E+00_JPRB, 0.68790E+00_JPRB, 0.69856E+00_JPRB, 0.69715E+00_JPRB ,&
&  0.72973E+00_JPRB, 0.75837E+00_JPRB, 0.77500E+00_JPRB, 0.78172E+00_JPRB, 0.82850E+00_JPRB, 0.89741E+00_JPRB, 0.75359E+00_JPRB ,&
&  0.69244E+00_JPRB, 0.64420E+00_JPRB, 0.60830E+00_JPRB, 0.58236E+00_JPRB, 0.54311E+00_JPRB /)
ZBURD(:, 48,  5) = (/ &
&  0.53207E+00_JPRB, 0.51762E+00_JPRB, 0.50728E+00_JPRB, 0.50321E+00_JPRB, 0.49624E+00_JPRB, 0.49513E+00_JPRB, 0.50263E+00_JPRB ,&
&  0.51573E+00_JPRB, 0.52055E+00_JPRB, 0.51986E+00_JPRB, 0.51790E+00_JPRB, 0.51193E+00_JPRB, 0.50289E+00_JPRB, 0.49141E+00_JPRB ,&
&  0.48132E+00_JPRB, 0.46610E+00_JPRB, 0.44610E+00_JPRB, 0.43129E+00_JPRB, 0.43440E+00_JPRB, 0.43964E+00_JPRB, 0.44041E+00_JPRB ,&
&  0.43967E+00_JPRB, 0.44567E+00_JPRB, 0.44800E+00_JPRB, 0.45791E+00_JPRB, 0.46566E+00_JPRB, 0.48792E+00_JPRB, 0.54182E+00_JPRB ,&
&  0.68741E+00_JPRB, 0.12064E+01_JPRB, 0.11353E+01_JPRB, 0.19460E+01_JPRB, 0.21930E+01_JPRB, 0.21101E+01_JPRB, 0.18441E+01_JPRB ,&
&  0.17064E+01_JPRB, 0.16074E+01_JPRB, 0.13597E+01_JPRB, 0.11524E+01_JPRB, 0.10183E+01_JPRB, 0.91572E+00_JPRB, 0.82644E+00_JPRB ,&
&  0.75722E+00_JPRB, 0.71326E+00_JPRB, 0.68600E+00_JPRB, 0.66742E+00_JPRB, 0.67001E+00_JPRB, 0.68998E+00_JPRB, 0.72047E+00_JPRB ,&
&  0.75657E+00_JPRB, 0.81511E+00_JPRB, 0.95570E+00_JPRB, 0.10867E+01_JPRB, 0.10115E+01_JPRB, 0.11113E+01_JPRB, 0.14772E+01_JPRB ,&
&  0.17001E+01_JPRB, 0.17128E+01_JPRB, 0.12425E+01_JPRB, 0.98440E+00_JPRB, 0.90440E+00_JPRB, 0.91267E+00_JPRB, 0.90968E+00_JPRB ,&
&  0.87554E+00_JPRB, 0.84804E+00_JPRB, 0.80324E+00_JPRB, 0.75561E+00_JPRB, 0.71956E+00_JPRB, 0.69254E+00_JPRB, 0.66210E+00_JPRB ,&
&  0.64004E+00_JPRB, 0.63231E+00_JPRB, 0.62829E+00_JPRB, 0.62386E+00_JPRB, 0.62428E+00_JPRB, 0.63197E+00_JPRB, 0.64375E+00_JPRB ,&
&  0.65788E+00_JPRB, 0.65797E+00_JPRB, 0.63529E+00_JPRB, 0.62043E+00_JPRB, 0.62703E+00_JPRB, 0.64088E+00_JPRB, 0.64511E+00_JPRB ,&
&  0.69649E+00_JPRB, 0.72968E+00_JPRB, 0.74337E+00_JPRB, 0.78931E+00_JPRB, 0.83559E+00_JPRB, 0.10038E+01_JPRB, 0.80871E+00_JPRB ,&
&  0.71429E+00_JPRB, 0.65858E+00_JPRB, 0.62819E+00_JPRB, 0.60346E+00_JPRB, 0.56782E+00_JPRB /)
ZBURD(:, 49,  5) = (/ &
&  0.56114E+00_JPRB, 0.54261E+00_JPRB, 0.53231E+00_JPRB, 0.51813E+00_JPRB, 0.50558E+00_JPRB, 0.50032E+00_JPRB, 0.50480E+00_JPRB ,&
&  0.52000E+00_JPRB, 0.53359E+00_JPRB, 0.52946E+00_JPRB, 0.52751E+00_JPRB, 0.52576E+00_JPRB, 0.51776E+00_JPRB, 0.50616E+00_JPRB ,&
&  0.49450E+00_JPRB, 0.48347E+00_JPRB, 0.46982E+00_JPRB, 0.45473E+00_JPRB, 0.44436E+00_JPRB, 0.44457E+00_JPRB, 0.44287E+00_JPRB ,&
&  0.44119E+00_JPRB, 0.44511E+00_JPRB, 0.44943E+00_JPRB, 0.45720E+00_JPRB, 0.46514E+00_JPRB, 0.48089E+00_JPRB, 0.52980E+00_JPRB ,&
&  0.70266E+00_JPRB, 0.55331E+00_JPRB, 0.13451E+01_JPRB, 0.17196E+01_JPRB, 0.17762E+01_JPRB, 0.16568E+01_JPRB, 0.15826E+01_JPRB ,&
&  0.15093E+01_JPRB, 0.13566E+01_JPRB, 0.12211E+01_JPRB, 0.11022E+01_JPRB, 0.95749E+00_JPRB, 0.86284E+00_JPRB, 0.80581E+00_JPRB ,&
&  0.76281E+00_JPRB, 0.72510E+00_JPRB, 0.69299E+00_JPRB, 0.66338E+00_JPRB, 0.64214E+00_JPRB, 0.63938E+00_JPRB, 0.65719E+00_JPRB ,&
&  0.68209E+00_JPRB, 0.71772E+00_JPRB, 0.81668E+00_JPRB, 0.94882E+00_JPRB, 0.10017E+01_JPRB, 0.10825E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.15867E+01_JPRB, 0.17447E+01_JPRB, 0.14152E+01_JPRB, 0.11458E+01_JPRB, 0.99750E+00_JPRB, 0.96109E+00_JPRB, 0.96053E+00_JPRB ,&
&  0.93105E+00_JPRB, 0.89193E+00_JPRB, 0.84066E+00_JPRB, 0.78302E+00_JPRB, 0.73357E+00_JPRB, 0.70149E+00_JPRB, 0.67817E+00_JPRB ,&
&  0.65321E+00_JPRB, 0.62805E+00_JPRB, 0.61217E+00_JPRB, 0.60308E+00_JPRB, 0.59894E+00_JPRB, 0.59917E+00_JPRB, 0.60778E+00_JPRB ,&
&  0.62645E+00_JPRB, 0.64198E+00_JPRB, 0.64970E+00_JPRB, 0.62556E+00_JPRB, 0.61722E+00_JPRB, 0.62487E+00_JPRB, 0.63835E+00_JPRB ,&
&  0.69467E+00_JPRB, 0.71616E+00_JPRB, 0.73731E+00_JPRB, 0.79467E+00_JPRB, 0.79846E+00_JPRB, 0.86881E+00_JPRB, 0.78697E+00_JPRB ,&
&  0.70661E+00_JPRB, 0.66394E+00_JPRB, 0.64428E+00_JPRB, 0.62216E+00_JPRB, 0.59171E+00_JPRB /)
ZBURD(:, 50,  5) = (/ &
&  0.58716E+00_JPRB, 0.56782E+00_JPRB, 0.55439E+00_JPRB, 0.53179E+00_JPRB, 0.51813E+00_JPRB, 0.51195E+00_JPRB, 0.51530E+00_JPRB ,&
&  0.52184E+00_JPRB, 0.52917E+00_JPRB, 0.52653E+00_JPRB, 0.52326E+00_JPRB, 0.51687E+00_JPRB, 0.51185E+00_JPRB, 0.50338E+00_JPRB ,&
&  0.49834E+00_JPRB, 0.49337E+00_JPRB, 0.48587E+00_JPRB, 0.47649E+00_JPRB, 0.46415E+00_JPRB, 0.45854E+00_JPRB, 0.45216E+00_JPRB ,&
&  0.44631E+00_JPRB, 0.44391E+00_JPRB, 0.44543E+00_JPRB, 0.45385E+00_JPRB, 0.46506E+00_JPRB, 0.48005E+00_JPRB, 0.51405E+00_JPRB ,&
&  0.71510E+00_JPRB, 0.91003E+00_JPRB, 0.14579E+01_JPRB, 0.14882E+01_JPRB, 0.13450E+01_JPRB, 0.11893E+01_JPRB, 0.11761E+01_JPRB ,&
&  0.11433E+01_JPRB, 0.10818E+01_JPRB, 0.10120E+01_JPRB, 0.96418E+00_JPRB, 0.91322E+00_JPRB, 0.87817E+00_JPRB, 0.82572E+00_JPRB ,&
&  0.77725E+00_JPRB, 0.72629E+00_JPRB, 0.67193E+00_JPRB, 0.63189E+00_JPRB, 0.60616E+00_JPRB, 0.59593E+00_JPRB, 0.60181E+00_JPRB ,&
&  0.61375E+00_JPRB, 0.63015E+00_JPRB, 0.68759E+00_JPRB, 0.80727E+00_JPRB, 0.97317E+00_JPRB, 0.99795E+00_JPRB, 0.12484E+01_JPRB ,&
&  0.16031E+01_JPRB, 0.15740E+01_JPRB, 0.14351E+01_JPRB, 0.13235E+01_JPRB, 0.11636E+01_JPRB, 0.10970E+01_JPRB, 0.10765E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.93634E+00_JPRB, 0.84778E+00_JPRB, 0.78056E+00_JPRB, 0.73377E+00_JPRB, 0.70564E+00_JPRB, 0.69453E+00_JPRB ,&
&  0.68185E+00_JPRB, 0.66081E+00_JPRB, 0.63420E+00_JPRB, 0.61254E+00_JPRB, 0.60079E+00_JPRB, 0.59792E+00_JPRB, 0.59996E+00_JPRB ,&
&  0.60559E+00_JPRB, 0.61674E+00_JPRB, 0.66138E+00_JPRB, 0.63341E+00_JPRB, 0.63774E+00_JPRB, 0.65979E+00_JPRB, 0.67258E+00_JPRB ,&
&  0.67638E+00_JPRB, 0.71519E+00_JPRB, 0.74381E+00_JPRB, 0.79722E+00_JPRB, 0.91163E+00_JPRB, 0.93240E+00_JPRB, 0.82349E+00_JPRB ,&
&  0.71562E+00_JPRB, 0.66728E+00_JPRB, 0.65701E+00_JPRB, 0.64323E+00_JPRB, 0.62469E+00_JPRB /)
ZBURD(:, 51,  5) = (/ &
&  0.61929E+00_JPRB, 0.59424E+00_JPRB, 0.57306E+00_JPRB, 0.55075E+00_JPRB, 0.53143E+00_JPRB, 0.52222E+00_JPRB, 0.52008E+00_JPRB ,&
&  0.52189E+00_JPRB, 0.52178E+00_JPRB, 0.51893E+00_JPRB, 0.51246E+00_JPRB, 0.50746E+00_JPRB, 0.50271E+00_JPRB, 0.49835E+00_JPRB ,&
&  0.49063E+00_JPRB, 0.49063E+00_JPRB, 0.48630E+00_JPRB, 0.47906E+00_JPRB, 0.47113E+00_JPRB, 0.46203E+00_JPRB, 0.45329E+00_JPRB ,&
&  0.44770E+00_JPRB, 0.44572E+00_JPRB, 0.44598E+00_JPRB, 0.45000E+00_JPRB, 0.46213E+00_JPRB, 0.47528E+00_JPRB, 0.49526E+00_JPRB ,&
&  0.64681E+00_JPRB, 0.92757E+00_JPRB, 0.11207E+01_JPRB, 0.12392E+01_JPRB, 0.11039E+01_JPRB, 0.97080E+00_JPRB, 0.90381E+00_JPRB ,&
&  0.87503E+00_JPRB, 0.86492E+00_JPRB, 0.83052E+00_JPRB, 0.79018E+00_JPRB, 0.74351E+00_JPRB, 0.72088E+00_JPRB, 0.69490E+00_JPRB ,&
&  0.66564E+00_JPRB, 0.63896E+00_JPRB, 0.61328E+00_JPRB, 0.60088E+00_JPRB, 0.59085E+00_JPRB, 0.57880E+00_JPRB, 0.57044E+00_JPRB ,&
&  0.57047E+00_JPRB, 0.57506E+00_JPRB, 0.59580E+00_JPRB, 0.67214E+00_JPRB, 0.83439E+00_JPRB, 0.96485E+00_JPRB, 0.11310E+01_JPRB ,&
&  0.11702E+01_JPRB, 0.12065E+01_JPRB, 0.12356E+01_JPRB, 0.12323E+01_JPRB, 0.11707E+01_JPRB, 0.11249E+01_JPRB, 0.11149E+01_JPRB ,&
&  0.10643E+01_JPRB, 0.94557E+00_JPRB, 0.83022E+00_JPRB, 0.75908E+00_JPRB, 0.72278E+00_JPRB, 0.70680E+00_JPRB, 0.69494E+00_JPRB ,&
&  0.68949E+00_JPRB, 0.68198E+00_JPRB, 0.66447E+00_JPRB, 0.64118E+00_JPRB, 0.61938E+00_JPRB, 0.60411E+00_JPRB, 0.59604E+00_JPRB ,&
&  0.59005E+00_JPRB, 0.59097E+00_JPRB, 0.61551E+00_JPRB, 0.62902E+00_JPRB, 0.63726E+00_JPRB, 0.64874E+00_JPRB, 0.66459E+00_JPRB ,&
&  0.68711E+00_JPRB, 0.75910E+00_JPRB, 0.75251E+00_JPRB, 0.81579E+00_JPRB, 0.89995E+00_JPRB, 0.88818E+00_JPRB, 0.79665E+00_JPRB ,&
&  0.70368E+00_JPRB, 0.65850E+00_JPRB, 0.65110E+00_JPRB, 0.65065E+00_JPRB, 0.64643E+00_JPRB /)
ZBURD(:, 52,  5) = (/ &
&  0.72028E+00_JPRB, 0.66055E+00_JPRB, 0.61125E+00_JPRB, 0.57055E+00_JPRB, 0.54515E+00_JPRB, 0.53013E+00_JPRB, 0.51991E+00_JPRB ,&
&  0.51469E+00_JPRB, 0.51063E+00_JPRB, 0.50242E+00_JPRB, 0.49665E+00_JPRB, 0.49212E+00_JPRB, 0.48968E+00_JPRB, 0.48724E+00_JPRB ,&
&  0.47774E+00_JPRB, 0.47567E+00_JPRB, 0.47537E+00_JPRB, 0.47053E+00_JPRB, 0.46401E+00_JPRB, 0.45715E+00_JPRB, 0.44834E+00_JPRB ,&
&  0.44229E+00_JPRB, 0.44015E+00_JPRB, 0.44166E+00_JPRB, 0.44525E+00_JPRB, 0.44928E+00_JPRB, 0.46096E+00_JPRB, 0.48688E+00_JPRB ,&
&  0.61473E+00_JPRB, 0.75792E+00_JPRB, 0.87035E+00_JPRB, 0.92275E+00_JPRB, 0.81664E+00_JPRB, 0.76527E+00_JPRB, 0.74322E+00_JPRB ,&
&  0.74525E+00_JPRB, 0.75199E+00_JPRB, 0.71893E+00_JPRB, 0.66557E+00_JPRB, 0.61741E+00_JPRB, 0.59122E+00_JPRB, 0.57438E+00_JPRB ,&
&  0.55897E+00_JPRB, 0.54722E+00_JPRB, 0.53661E+00_JPRB, 0.53684E+00_JPRB, 0.53758E+00_JPRB, 0.53436E+00_JPRB, 0.53121E+00_JPRB ,&
&  0.53000E+00_JPRB, 0.53182E+00_JPRB, 0.53836E+00_JPRB, 0.57089E+00_JPRB, 0.65468E+00_JPRB, 0.74708E+00_JPRB, 0.83783E+00_JPRB ,&
&  0.88123E+00_JPRB, 0.92524E+00_JPRB, 0.97937E+00_JPRB, 0.10220E+01_JPRB, 0.10236E+01_JPRB, 0.10141E+01_JPRB, 0.10026E+01_JPRB ,&
&  0.97656E+00_JPRB, 0.90480E+00_JPRB, 0.81027E+00_JPRB, 0.74098E+00_JPRB, 0.70368E+00_JPRB, 0.68489E+00_JPRB, 0.67487E+00_JPRB ,&
&  0.66641E+00_JPRB, 0.65798E+00_JPRB, 0.64425E+00_JPRB, 0.62467E+00_JPRB, 0.60264E+00_JPRB, 0.58749E+00_JPRB, 0.57611E+00_JPRB ,&
&  0.57008E+00_JPRB, 0.57326E+00_JPRB, 0.58828E+00_JPRB, 0.61016E+00_JPRB, 0.62694E+00_JPRB, 0.63979E+00_JPRB, 0.64810E+00_JPRB ,&
&  0.66007E+00_JPRB, 0.71487E+00_JPRB, 0.71388E+00_JPRB, 0.83939E+00_JPRB, 0.93479E+00_JPRB, 0.87515E+00_JPRB, 0.79113E+00_JPRB ,&
&  0.70580E+00_JPRB, 0.64829E+00_JPRB, 0.62679E+00_JPRB, 0.64666E+00_JPRB, 0.75734E+00_JPRB /)
ZBURD(:, 53,  5) = (/ &
&  0.67651E+00_JPRB, 0.64886E+00_JPRB, 0.60999E+00_JPRB, 0.57333E+00_JPRB, 0.54864E+00_JPRB, 0.53345E+00_JPRB, 0.52434E+00_JPRB ,&
&  0.51647E+00_JPRB, 0.50783E+00_JPRB, 0.49832E+00_JPRB, 0.48627E+00_JPRB, 0.47874E+00_JPRB, 0.47488E+00_JPRB, 0.47190E+00_JPRB ,&
&  0.46700E+00_JPRB, 0.45739E+00_JPRB, 0.45442E+00_JPRB, 0.45165E+00_JPRB, 0.45021E+00_JPRB, 0.44832E+00_JPRB, 0.44305E+00_JPRB ,&
&  0.43648E+00_JPRB, 0.43319E+00_JPRB, 0.43612E+00_JPRB, 0.43762E+00_JPRB, 0.43950E+00_JPRB, 0.44502E+00_JPRB, 0.46227E+00_JPRB ,&
&  0.54865E+00_JPRB, 0.60548E+00_JPRB, 0.69129E+00_JPRB, 0.75992E+00_JPRB, 0.72686E+00_JPRB, 0.69031E+00_JPRB, 0.67118E+00_JPRB ,&
&  0.64590E+00_JPRB, 0.62837E+00_JPRB, 0.59665E+00_JPRB, 0.55703E+00_JPRB, 0.52423E+00_JPRB, 0.51430E+00_JPRB, 0.50723E+00_JPRB ,&
&  0.50124E+00_JPRB, 0.49778E+00_JPRB, 0.49576E+00_JPRB, 0.49520E+00_JPRB, 0.49571E+00_JPRB, 0.49494E+00_JPRB, 0.49401E+00_JPRB ,&
&  0.49505E+00_JPRB, 0.49871E+00_JPRB, 0.50266E+00_JPRB, 0.51289E+00_JPRB, 0.54750E+00_JPRB, 0.60827E+00_JPRB, 0.67402E+00_JPRB ,&
&  0.71678E+00_JPRB, 0.75505E+00_JPRB, 0.80589E+00_JPRB, 0.84534E+00_JPRB, 0.86647E+00_JPRB, 0.86807E+00_JPRB, 0.85441E+00_JPRB ,&
&  0.83634E+00_JPRB, 0.81020E+00_JPRB, 0.75597E+00_JPRB, 0.69839E+00_JPRB, 0.66101E+00_JPRB, 0.63754E+00_JPRB, 0.62021E+00_JPRB ,&
&  0.60750E+00_JPRB, 0.59203E+00_JPRB, 0.57568E+00_JPRB, 0.56196E+00_JPRB, 0.55385E+00_JPRB, 0.54619E+00_JPRB, 0.53922E+00_JPRB ,&
&  0.54077E+00_JPRB, 0.55140E+00_JPRB, 0.56814E+00_JPRB, 0.58041E+00_JPRB, 0.59068E+00_JPRB, 0.60183E+00_JPRB, 0.60885E+00_JPRB ,&
&  0.61811E+00_JPRB, 0.65253E+00_JPRB, 0.67956E+00_JPRB, 0.78500E+00_JPRB, 0.80908E+00_JPRB, 0.79537E+00_JPRB, 0.75244E+00_JPRB ,&
&  0.69551E+00_JPRB, 0.64764E+00_JPRB, 0.61250E+00_JPRB, 0.62292E+00_JPRB, 0.67479E+00_JPRB /)
ZBURD(:, 54,  5) = (/ &
&  0.61621E+00_JPRB, 0.60337E+00_JPRB, 0.58793E+00_JPRB, 0.56799E+00_JPRB, 0.55382E+00_JPRB, 0.54227E+00_JPRB, 0.52944E+00_JPRB ,&
&  0.51668E+00_JPRB, 0.50573E+00_JPRB, 0.49640E+00_JPRB, 0.48584E+00_JPRB, 0.47426E+00_JPRB, 0.47018E+00_JPRB, 0.46851E+00_JPRB ,&
&  0.46406E+00_JPRB, 0.45560E+00_JPRB, 0.44806E+00_JPRB, 0.44628E+00_JPRB, 0.44555E+00_JPRB, 0.44516E+00_JPRB, 0.44192E+00_JPRB ,&
&  0.43669E+00_JPRB, 0.43589E+00_JPRB, 0.43895E+00_JPRB, 0.43925E+00_JPRB, 0.43486E+00_JPRB, 0.43612E+00_JPRB, 0.44924E+00_JPRB ,&
&  0.51685E+00_JPRB, 0.48200E+00_JPRB, 0.53816E+00_JPRB, 0.63630E+00_JPRB, 0.61307E+00_JPRB, 0.60113E+00_JPRB, 0.57214E+00_JPRB ,&
&  0.54267E+00_JPRB, 0.52749E+00_JPRB, 0.51570E+00_JPRB, 0.49859E+00_JPRB, 0.48090E+00_JPRB, 0.47104E+00_JPRB, 0.47021E+00_JPRB ,&
&  0.47434E+00_JPRB, 0.47667E+00_JPRB, 0.47752E+00_JPRB, 0.47293E+00_JPRB, 0.47017E+00_JPRB, 0.46828E+00_JPRB, 0.46749E+00_JPRB ,&
&  0.46717E+00_JPRB, 0.47010E+00_JPRB, 0.47432E+00_JPRB, 0.47792E+00_JPRB, 0.48974E+00_JPRB, 0.52418E+00_JPRB, 0.57692E+00_JPRB ,&
&  0.62808E+00_JPRB, 0.67461E+00_JPRB, 0.70931E+00_JPRB, 0.73914E+00_JPRB, 0.75677E+00_JPRB, 0.77029E+00_JPRB, 0.77158E+00_JPRB ,&
&  0.75180E+00_JPRB, 0.71970E+00_JPRB, 0.68659E+00_JPRB, 0.65043E+00_JPRB, 0.61738E+00_JPRB, 0.59131E+00_JPRB, 0.57080E+00_JPRB ,&
&  0.55528E+00_JPRB, 0.54459E+00_JPRB, 0.53626E+00_JPRB, 0.52652E+00_JPRB, 0.52070E+00_JPRB, 0.52099E+00_JPRB, 0.52423E+00_JPRB ,&
&  0.52959E+00_JPRB, 0.53638E+00_JPRB, 0.54146E+00_JPRB, 0.54430E+00_JPRB, 0.54649E+00_JPRB, 0.55106E+00_JPRB, 0.55713E+00_JPRB ,&
&  0.57146E+00_JPRB, 0.60026E+00_JPRB, 0.62703E+00_JPRB, 0.74552E+00_JPRB, 0.80624E+00_JPRB, 0.79470E+00_JPRB, 0.75699E+00_JPRB ,&
&  0.70859E+00_JPRB, 0.65719E+00_JPRB, 0.62726E+00_JPRB, 0.55111E+00_JPRB, 0.62815E+00_JPRB /)
ZBURD(:, 55,  5) = (/ &
&  0.57298E+00_JPRB, 0.56736E+00_JPRB, 0.55905E+00_JPRB, 0.55030E+00_JPRB, 0.54192E+00_JPRB, 0.53091E+00_JPRB, 0.51680E+00_JPRB ,&
&  0.50386E+00_JPRB, 0.49247E+00_JPRB, 0.48506E+00_JPRB, 0.47781E+00_JPRB, 0.46721E+00_JPRB, 0.46039E+00_JPRB, 0.45892E+00_JPRB ,&
&  0.45666E+00_JPRB, 0.45061E+00_JPRB, 0.44366E+00_JPRB, 0.44214E+00_JPRB, 0.43985E+00_JPRB, 0.43709E+00_JPRB, 0.43319E+00_JPRB ,&
&  0.42919E+00_JPRB, 0.42805E+00_JPRB, 0.42856E+00_JPRB, 0.42937E+00_JPRB, 0.42600E+00_JPRB, 0.42729E+00_JPRB, 0.44062E+00_JPRB ,&
&  0.49396E+00_JPRB, 0.43252E+00_JPRB, 0.48412E+00_JPRB, 0.53949E+00_JPRB, 0.51334E+00_JPRB, 0.49219E+00_JPRB, 0.47426E+00_JPRB ,&
&  0.46136E+00_JPRB, 0.45591E+00_JPRB, 0.45295E+00_JPRB, 0.44819E+00_JPRB, 0.44338E+00_JPRB, 0.43803E+00_JPRB, 0.43650E+00_JPRB ,&
&  0.43750E+00_JPRB, 0.44320E+00_JPRB, 0.44665E+00_JPRB, 0.44701E+00_JPRB, 0.44380E+00_JPRB, 0.44032E+00_JPRB, 0.43738E+00_JPRB ,&
&  0.43618E+00_JPRB, 0.43752E+00_JPRB, 0.44003E+00_JPRB, 0.44430E+00_JPRB, 0.45267E+00_JPRB, 0.47049E+00_JPRB, 0.50170E+00_JPRB ,&
&  0.54292E+00_JPRB, 0.58024E+00_JPRB, 0.61152E+00_JPRB, 0.64259E+00_JPRB, 0.66938E+00_JPRB, 0.68489E+00_JPRB, 0.68395E+00_JPRB ,&
&  0.66811E+00_JPRB, 0.63895E+00_JPRB, 0.61555E+00_JPRB, 0.59394E+00_JPRB, 0.57304E+00_JPRB, 0.55630E+00_JPRB, 0.54507E+00_JPRB ,&
&  0.53796E+00_JPRB, 0.53322E+00_JPRB, 0.52976E+00_JPRB, 0.52611E+00_JPRB, 0.52535E+00_JPRB, 0.52538E+00_JPRB, 0.52722E+00_JPRB ,&
&  0.52622E+00_JPRB, 0.52338E+00_JPRB, 0.51496E+00_JPRB, 0.50368E+00_JPRB, 0.49848E+00_JPRB, 0.49830E+00_JPRB, 0.50288E+00_JPRB ,&
&  0.51411E+00_JPRB, 0.53490E+00_JPRB, 0.55627E+00_JPRB, 0.63555E+00_JPRB, 0.68063E+00_JPRB, 0.70104E+00_JPRB, 0.70305E+00_JPRB ,&
&  0.68266E+00_JPRB, 0.64591E+00_JPRB, 0.50690E+00_JPRB, 0.57129E+00_JPRB, 0.58291E+00_JPRB /)
ZBURD(:, 56,  5) = (/ &
&  0.56678E+00_JPRB, 0.55480E+00_JPRB, 0.54378E+00_JPRB, 0.53259E+00_JPRB, 0.52054E+00_JPRB, 0.50790E+00_JPRB, 0.49542E+00_JPRB ,&
&  0.48481E+00_JPRB, 0.47603E+00_JPRB, 0.46782E+00_JPRB, 0.46065E+00_JPRB, 0.45374E+00_JPRB, 0.45017E+00_JPRB, 0.44881E+00_JPRB ,&
&  0.44880E+00_JPRB, 0.44784E+00_JPRB, 0.44579E+00_JPRB, 0.44533E+00_JPRB, 0.44459E+00_JPRB, 0.44192E+00_JPRB, 0.43825E+00_JPRB ,&
&  0.43246E+00_JPRB, 0.42744E+00_JPRB, 0.42555E+00_JPRB, 0.42778E+00_JPRB, 0.42677E+00_JPRB, 0.42896E+00_JPRB, 0.43783E+00_JPRB ,&
&  0.42452E+00_JPRB, 0.40056E+00_JPRB, 0.43739E+00_JPRB, 0.47914E+00_JPRB, 0.46060E+00_JPRB, 0.44333E+00_JPRB, 0.43642E+00_JPRB ,&
&  0.42981E+00_JPRB, 0.42622E+00_JPRB, 0.42333E+00_JPRB, 0.42192E+00_JPRB, 0.42266E+00_JPRB, 0.42214E+00_JPRB, 0.42070E+00_JPRB ,&
&  0.42013E+00_JPRB, 0.42325E+00_JPRB, 0.42660E+00_JPRB, 0.42861E+00_JPRB, 0.42851E+00_JPRB, 0.42710E+00_JPRB, 0.42326E+00_JPRB ,&
&  0.41969E+00_JPRB, 0.41761E+00_JPRB, 0.41780E+00_JPRB, 0.42161E+00_JPRB, 0.42842E+00_JPRB, 0.43986E+00_JPRB, 0.45873E+00_JPRB ,&
&  0.48483E+00_JPRB, 0.51204E+00_JPRB, 0.53618E+00_JPRB, 0.55945E+00_JPRB, 0.58474E+00_JPRB, 0.60685E+00_JPRB, 0.61859E+00_JPRB ,&
&  0.61087E+00_JPRB, 0.59028E+00_JPRB, 0.56608E+00_JPRB, 0.54673E+00_JPRB, 0.53251E+00_JPRB, 0.52320E+00_JPRB, 0.51756E+00_JPRB ,&
&  0.51545E+00_JPRB, 0.51394E+00_JPRB, 0.51237E+00_JPRB, 0.51161E+00_JPRB, 0.51097E+00_JPRB, 0.50836E+00_JPRB, 0.50486E+00_JPRB ,&
&  0.49938E+00_JPRB, 0.49091E+00_JPRB, 0.48099E+00_JPRB, 0.47255E+00_JPRB, 0.46700E+00_JPRB, 0.46604E+00_JPRB, 0.46911E+00_JPRB ,&
&  0.47879E+00_JPRB, 0.49872E+00_JPRB, 0.52093E+00_JPRB, 0.57356E+00_JPRB, 0.61207E+00_JPRB, 0.63876E+00_JPRB, 0.65128E+00_JPRB ,&
&  0.64801E+00_JPRB, 0.62628E+00_JPRB, 0.60428E+00_JPRB, 0.59022E+00_JPRB, 0.58197E+00_JPRB /)
ZBURD(:, 57,  5) = (/ &
&  0.54045E+00_JPRB, 0.52420E+00_JPRB, 0.50737E+00_JPRB, 0.49530E+00_JPRB, 0.48612E+00_JPRB, 0.47771E+00_JPRB, 0.46940E+00_JPRB ,&
&  0.46037E+00_JPRB, 0.45209E+00_JPRB, 0.44413E+00_JPRB, 0.43636E+00_JPRB, 0.43197E+00_JPRB, 0.42945E+00_JPRB, 0.42930E+00_JPRB ,&
&  0.43148E+00_JPRB, 0.43591E+00_JPRB, 0.43763E+00_JPRB, 0.43694E+00_JPRB, 0.43732E+00_JPRB, 0.43688E+00_JPRB, 0.43496E+00_JPRB ,&
&  0.43061E+00_JPRB, 0.42540E+00_JPRB, 0.42251E+00_JPRB, 0.42315E+00_JPRB, 0.42329E+00_JPRB, 0.42421E+00_JPRB, 0.42482E+00_JPRB ,&
&  0.42917E+00_JPRB, 0.39490E+00_JPRB, 0.42287E+00_JPRB, 0.44551E+00_JPRB, 0.43900E+00_JPRB, 0.43230E+00_JPRB, 0.42540E+00_JPRB ,&
&  0.41734E+00_JPRB, 0.41286E+00_JPRB, 0.41021E+00_JPRB, 0.41115E+00_JPRB, 0.41393E+00_JPRB, 0.41529E+00_JPRB, 0.41559E+00_JPRB ,&
&  0.41301E+00_JPRB, 0.41323E+00_JPRB, 0.41274E+00_JPRB, 0.41284E+00_JPRB, 0.41371E+00_JPRB, 0.41479E+00_JPRB, 0.41491E+00_JPRB ,&
&  0.41344E+00_JPRB, 0.41033E+00_JPRB, 0.40724E+00_JPRB, 0.40701E+00_JPRB, 0.40876E+00_JPRB, 0.41306E+00_JPRB, 0.42098E+00_JPRB ,&
&  0.43316E+00_JPRB, 0.44792E+00_JPRB, 0.46448E+00_JPRB, 0.48296E+00_JPRB, 0.50588E+00_JPRB, 0.52784E+00_JPRB, 0.54454E+00_JPRB ,&
&  0.54804E+00_JPRB, 0.54235E+00_JPRB, 0.52959E+00_JPRB, 0.51410E+00_JPRB, 0.50083E+00_JPRB, 0.48791E+00_JPRB, 0.47597E+00_JPRB ,&
&  0.46701E+00_JPRB, 0.45953E+00_JPRB, 0.45451E+00_JPRB, 0.45253E+00_JPRB, 0.45124E+00_JPRB, 0.44863E+00_JPRB, 0.44638E+00_JPRB ,&
&  0.44433E+00_JPRB, 0.44254E+00_JPRB, 0.44134E+00_JPRB, 0.44001E+00_JPRB, 0.43784E+00_JPRB, 0.43604E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.44258E+00_JPRB, 0.45501E+00_JPRB, 0.47619E+00_JPRB, 0.50887E+00_JPRB, 0.53924E+00_JPRB, 0.56280E+00_JPRB, 0.57669E+00_JPRB ,&
&  0.57985E+00_JPRB, 0.57800E+00_JPRB, 0.56955E+00_JPRB, 0.56132E+00_JPRB, 0.55377E+00_JPRB /)
ZBURD(:, 58,  5) = (/ &
&  0.50291E+00_JPRB, 0.49190E+00_JPRB, 0.48024E+00_JPRB, 0.47099E+00_JPRB, 0.46395E+00_JPRB, 0.45699E+00_JPRB, 0.44936E+00_JPRB ,&
&  0.44346E+00_JPRB, 0.43647E+00_JPRB, 0.43075E+00_JPRB, 0.42539E+00_JPRB, 0.42106E+00_JPRB, 0.41930E+00_JPRB, 0.41957E+00_JPRB ,&
&  0.42176E+00_JPRB, 0.42699E+00_JPRB, 0.42982E+00_JPRB, 0.42812E+00_JPRB, 0.42968E+00_JPRB, 0.43175E+00_JPRB, 0.43212E+00_JPRB ,&
&  0.42969E+00_JPRB, 0.42651E+00_JPRB, 0.42301E+00_JPRB, 0.42086E+00_JPRB, 0.42079E+00_JPRB, 0.42203E+00_JPRB, 0.41926E+00_JPRB ,&
&  0.41452E+00_JPRB, 0.40915E+00_JPRB, 0.42123E+00_JPRB, 0.42937E+00_JPRB, 0.42930E+00_JPRB, 0.42470E+00_JPRB, 0.41891E+00_JPRB ,&
&  0.41132E+00_JPRB, 0.40698E+00_JPRB, 0.40713E+00_JPRB, 0.40900E+00_JPRB, 0.41138E+00_JPRB, 0.41254E+00_JPRB, 0.41385E+00_JPRB ,&
&  0.41086E+00_JPRB, 0.40932E+00_JPRB, 0.40747E+00_JPRB, 0.40623E+00_JPRB, 0.40703E+00_JPRB, 0.40827E+00_JPRB, 0.40916E+00_JPRB ,&
&  0.40846E+00_JPRB, 0.40929E+00_JPRB, 0.41023E+00_JPRB, 0.40952E+00_JPRB, 0.41109E+00_JPRB, 0.41351E+00_JPRB, 0.41695E+00_JPRB ,&
&  0.42182E+00_JPRB, 0.42857E+00_JPRB, 0.43611E+00_JPRB, 0.44638E+00_JPRB, 0.46120E+00_JPRB, 0.47640E+00_JPRB, 0.48793E+00_JPRB ,&
&  0.49352E+00_JPRB, 0.49290E+00_JPRB, 0.48790E+00_JPRB, 0.47899E+00_JPRB, 0.47026E+00_JPRB, 0.46253E+00_JPRB, 0.45319E+00_JPRB ,&
&  0.44231E+00_JPRB, 0.43490E+00_JPRB, 0.42988E+00_JPRB, 0.42671E+00_JPRB, 0.42354E+00_JPRB, 0.42155E+00_JPRB, 0.42154E+00_JPRB ,&
&  0.42342E+00_JPRB, 0.42544E+00_JPRB, 0.42617E+00_JPRB, 0.42626E+00_JPRB, 0.42695E+00_JPRB, 0.42563E+00_JPRB, 0.42506E+00_JPRB ,&
&  0.42753E+00_JPRB, 0.43382E+00_JPRB, 0.44669E+00_JPRB, 0.47195E+00_JPRB, 0.49131E+00_JPRB, 0.50594E+00_JPRB, 0.51598E+00_JPRB ,&
&  0.52149E+00_JPRB, 0.52434E+00_JPRB, 0.52301E+00_JPRB, 0.51929E+00_JPRB, 0.51283E+00_JPRB /)
ZBURD(:, 59,  5) = (/ &
&  0.45825E+00_JPRB, 0.45327E+00_JPRB, 0.44761E+00_JPRB, 0.44223E+00_JPRB, 0.43731E+00_JPRB, 0.43229E+00_JPRB, 0.42758E+00_JPRB ,&
&  0.42295E+00_JPRB, 0.41962E+00_JPRB, 0.41706E+00_JPRB, 0.41489E+00_JPRB, 0.41098E+00_JPRB, 0.40904E+00_JPRB, 0.40935E+00_JPRB ,&
&  0.41141E+00_JPRB, 0.41361E+00_JPRB, 0.41526E+00_JPRB, 0.41463E+00_JPRB, 0.41681E+00_JPRB, 0.41896E+00_JPRB, 0.41913E+00_JPRB ,&
&  0.41830E+00_JPRB, 0.41746E+00_JPRB, 0.41534E+00_JPRB, 0.41264E+00_JPRB, 0.41264E+00_JPRB, 0.41308E+00_JPRB, 0.40951E+00_JPRB ,&
&  0.40263E+00_JPRB, 0.40548E+00_JPRB, 0.39780E+00_JPRB, 0.41221E+00_JPRB, 0.41578E+00_JPRB, 0.41045E+00_JPRB, 0.40366E+00_JPRB ,&
&  0.40095E+00_JPRB, 0.40137E+00_JPRB, 0.40428E+00_JPRB, 0.40631E+00_JPRB, 0.40701E+00_JPRB, 0.40719E+00_JPRB, 0.40711E+00_JPRB ,&
&  0.39966E+00_JPRB, 0.39522E+00_JPRB, 0.39474E+00_JPRB, 0.39559E+00_JPRB, 0.39574E+00_JPRB, 0.39655E+00_JPRB, 0.39862E+00_JPRB ,&
&  0.40029E+00_JPRB, 0.40144E+00_JPRB, 0.40270E+00_JPRB, 0.40536E+00_JPRB, 0.40722E+00_JPRB, 0.40817E+00_JPRB, 0.41011E+00_JPRB ,&
&  0.41319E+00_JPRB, 0.41710E+00_JPRB, 0.42064E+00_JPRB, 0.42366E+00_JPRB, 0.42969E+00_JPRB, 0.43792E+00_JPRB, 0.44449E+00_JPRB ,&
&  0.44711E+00_JPRB, 0.44661E+00_JPRB, 0.44426E+00_JPRB, 0.43882E+00_JPRB, 0.43128E+00_JPRB, 0.42520E+00_JPRB, 0.42030E+00_JPRB ,&
&  0.41337E+00_JPRB, 0.40973E+00_JPRB, 0.40731E+00_JPRB, 0.40406E+00_JPRB, 0.40131E+00_JPRB, 0.39993E+00_JPRB, 0.39972E+00_JPRB ,&
&  0.40119E+00_JPRB, 0.40436E+00_JPRB, 0.40634E+00_JPRB, 0.40889E+00_JPRB, 0.41254E+00_JPRB, 0.41625E+00_JPRB, 0.41640E+00_JPRB ,&
&  0.41509E+00_JPRB, 0.41482E+00_JPRB, 0.41718E+00_JPRB, 0.42584E+00_JPRB, 0.43777E+00_JPRB, 0.44761E+00_JPRB, 0.45631E+00_JPRB ,&
&  0.46272E+00_JPRB, 0.46620E+00_JPRB, 0.46643E+00_JPRB, 0.46564E+00_JPRB, 0.46271E+00_JPRB /)
ZBURD(:, 60,  5) = (/ &
&  0.42566E+00_JPRB, 0.42415E+00_JPRB, 0.42233E+00_JPRB, 0.41923E+00_JPRB, 0.41569E+00_JPRB, 0.41223E+00_JPRB, 0.40886E+00_JPRB ,&
&  0.40619E+00_JPRB, 0.40489E+00_JPRB, 0.40469E+00_JPRB, 0.40426E+00_JPRB, 0.40355E+00_JPRB, 0.40390E+00_JPRB, 0.40384E+00_JPRB ,&
&  0.40502E+00_JPRB, 0.40502E+00_JPRB, 0.40264E+00_JPRB, 0.40216E+00_JPRB, 0.40469E+00_JPRB, 0.40950E+00_JPRB, 0.41139E+00_JPRB ,&
&  0.40921E+00_JPRB, 0.40903E+00_JPRB, 0.40880E+00_JPRB, 0.40669E+00_JPRB, 0.40521E+00_JPRB, 0.40431E+00_JPRB, 0.40048E+00_JPRB ,&
&  0.39557E+00_JPRB, 0.39640E+00_JPRB, 0.39811E+00_JPRB, 0.40201E+00_JPRB, 0.40240E+00_JPRB, 0.40094E+00_JPRB, 0.40126E+00_JPRB ,&
&  0.39923E+00_JPRB, 0.39862E+00_JPRB, 0.39809E+00_JPRB, 0.39840E+00_JPRB, 0.39653E+00_JPRB, 0.39385E+00_JPRB, 0.38898E+00_JPRB ,&
&  0.37960E+00_JPRB, 0.37643E+00_JPRB, 0.37847E+00_JPRB, 0.38285E+00_JPRB, 0.38248E+00_JPRB, 0.38329E+00_JPRB, 0.38644E+00_JPRB ,&
&  0.39041E+00_JPRB, 0.39335E+00_JPRB, 0.39518E+00_JPRB, 0.39641E+00_JPRB, 0.39807E+00_JPRB, 0.39964E+00_JPRB, 0.40054E+00_JPRB ,&
&  0.40170E+00_JPRB, 0.40375E+00_JPRB, 0.40626E+00_JPRB, 0.40895E+00_JPRB, 0.41087E+00_JPRB, 0.41315E+00_JPRB, 0.41492E+00_JPRB ,&
&  0.41484E+00_JPRB, 0.41212E+00_JPRB, 0.40864E+00_JPRB, 0.40328E+00_JPRB, 0.39754E+00_JPRB, 0.39389E+00_JPRB, 0.39165E+00_JPRB ,&
&  0.38745E+00_JPRB, 0.38387E+00_JPRB, 0.38126E+00_JPRB, 0.38055E+00_JPRB, 0.38065E+00_JPRB, 0.38173E+00_JPRB, 0.38380E+00_JPRB ,&
&  0.38554E+00_JPRB, 0.38843E+00_JPRB, 0.39478E+00_JPRB, 0.39913E+00_JPRB, 0.40131E+00_JPRB, 0.40438E+00_JPRB, 0.40576E+00_JPRB ,&
&  0.40399E+00_JPRB, 0.40239E+00_JPRB, 0.40129E+00_JPRB, 0.40355E+00_JPRB, 0.40776E+00_JPRB, 0.41174E+00_JPRB, 0.41479E+00_JPRB ,&
&  0.41934E+00_JPRB, 0.42381E+00_JPRB, 0.42548E+00_JPRB, 0.42617E+00_JPRB, 0.42671E+00_JPRB /)
ZBURD(:, 61,  5) = (/ &
&  0.39080E+00_JPRB, 0.39125E+00_JPRB, 0.39071E+00_JPRB, 0.38946E+00_JPRB, 0.38803E+00_JPRB, 0.38680E+00_JPRB, 0.38653E+00_JPRB ,&
&  0.38637E+00_JPRB, 0.38664E+00_JPRB, 0.38717E+00_JPRB, 0.38756E+00_JPRB, 0.38893E+00_JPRB, 0.39062E+00_JPRB, 0.39202E+00_JPRB ,&
&  0.39351E+00_JPRB, 0.39261E+00_JPRB, 0.38982E+00_JPRB, 0.38725E+00_JPRB, 0.38800E+00_JPRB, 0.39345E+00_JPRB, 0.39950E+00_JPRB ,&
&  0.40110E+00_JPRB, 0.40223E+00_JPRB, 0.40271E+00_JPRB, 0.40154E+00_JPRB, 0.40008E+00_JPRB, 0.39970E+00_JPRB, 0.39614E+00_JPRB ,&
&  0.39163E+00_JPRB, 0.39047E+00_JPRB, 0.39003E+00_JPRB, 0.38782E+00_JPRB, 0.38790E+00_JPRB, 0.39070E+00_JPRB, 0.39007E+00_JPRB ,&
&  0.38920E+00_JPRB, 0.38733E+00_JPRB, 0.38406E+00_JPRB, 0.38110E+00_JPRB, 0.37758E+00_JPRB, 0.37114E+00_JPRB, 0.36371E+00_JPRB ,&
&  0.35775E+00_JPRB, 0.35779E+00_JPRB, 0.36317E+00_JPRB, 0.36889E+00_JPRB, 0.36807E+00_JPRB, 0.36716E+00_JPRB, 0.36876E+00_JPRB ,&
&  0.37239E+00_JPRB, 0.37690E+00_JPRB, 0.37976E+00_JPRB, 0.37897E+00_JPRB, 0.37784E+00_JPRB, 0.37971E+00_JPRB, 0.38237E+00_JPRB ,&
&  0.38366E+00_JPRB, 0.38512E+00_JPRB, 0.38618E+00_JPRB, 0.38766E+00_JPRB, 0.38827E+00_JPRB, 0.38700E+00_JPRB, 0.38635E+00_JPRB ,&
&  0.38406E+00_JPRB, 0.38057E+00_JPRB, 0.37585E+00_JPRB, 0.37030E+00_JPRB, 0.36621E+00_JPRB, 0.36369E+00_JPRB, 0.36366E+00_JPRB ,&
&  0.36299E+00_JPRB, 0.36436E+00_JPRB, 0.36468E+00_JPRB, 0.36244E+00_JPRB, 0.35942E+00_JPRB, 0.35942E+00_JPRB, 0.36293E+00_JPRB ,&
&  0.36771E+00_JPRB, 0.37116E+00_JPRB, 0.37689E+00_JPRB, 0.37979E+00_JPRB, 0.38185E+00_JPRB, 0.38432E+00_JPRB, 0.38495E+00_JPRB ,&
&  0.38232E+00_JPRB, 0.38016E+00_JPRB, 0.37966E+00_JPRB, 0.38232E+00_JPRB, 0.38340E+00_JPRB, 0.38361E+00_JPRB, 0.38230E+00_JPRB ,&
&  0.38254E+00_JPRB, 0.38525E+00_JPRB, 0.38679E+00_JPRB, 0.38743E+00_JPRB, 0.38884E+00_JPRB /)
ZBURD(:, 62,  5) = (/ &
&  0.36363E+00_JPRB, 0.36612E+00_JPRB, 0.36855E+00_JPRB, 0.37069E+00_JPRB, 0.37206E+00_JPRB, 0.37309E+00_JPRB, 0.37224E+00_JPRB ,&
&  0.37201E+00_JPRB, 0.37349E+00_JPRB, 0.37543E+00_JPRB, 0.37698E+00_JPRB, 0.37782E+00_JPRB, 0.37960E+00_JPRB, 0.38137E+00_JPRB ,&
&  0.38251E+00_JPRB, 0.38146E+00_JPRB, 0.37792E+00_JPRB, 0.37636E+00_JPRB, 0.37530E+00_JPRB, 0.38035E+00_JPRB, 0.38895E+00_JPRB ,&
&  0.39282E+00_JPRB, 0.39521E+00_JPRB, 0.39810E+00_JPRB, 0.39873E+00_JPRB, 0.39905E+00_JPRB, 0.39890E+00_JPRB, 0.39450E+00_JPRB ,&
&  0.38959E+00_JPRB, 0.38512E+00_JPRB, 0.38222E+00_JPRB, 0.37894E+00_JPRB, 0.37888E+00_JPRB, 0.37669E+00_JPRB, 0.37518E+00_JPRB ,&
&  0.37407E+00_JPRB, 0.37153E+00_JPRB, 0.36823E+00_JPRB, 0.36280E+00_JPRB, 0.35610E+00_JPRB, 0.34749E+00_JPRB, 0.34156E+00_JPRB ,&
&  0.33919E+00_JPRB, 0.34277E+00_JPRB, 0.35126E+00_JPRB, 0.35523E+00_JPRB, 0.35215E+00_JPRB, 0.35152E+00_JPRB, 0.35480E+00_JPRB ,&
&  0.35960E+00_JPRB, 0.36486E+00_JPRB, 0.36669E+00_JPRB, 0.36580E+00_JPRB, 0.36386E+00_JPRB, 0.36542E+00_JPRB, 0.36965E+00_JPRB ,&
&  0.37336E+00_JPRB, 0.37534E+00_JPRB, 0.37462E+00_JPRB, 0.37369E+00_JPRB, 0.37111E+00_JPRB, 0.36534E+00_JPRB, 0.35758E+00_JPRB ,&
&  0.35051E+00_JPRB, 0.34610E+00_JPRB, 0.34275E+00_JPRB, 0.33955E+00_JPRB, 0.33841E+00_JPRB, 0.33886E+00_JPRB, 0.34113E+00_JPRB ,&
&  0.34206E+00_JPRB, 0.34393E+00_JPRB, 0.34409E+00_JPRB, 0.33996E+00_JPRB, 0.33664E+00_JPRB, 0.33797E+00_JPRB, 0.33953E+00_JPRB ,&
&  0.34210E+00_JPRB, 0.34466E+00_JPRB, 0.34727E+00_JPRB, 0.34837E+00_JPRB, 0.34998E+00_JPRB, 0.35265E+00_JPRB, 0.35424E+00_JPRB ,&
&  0.35504E+00_JPRB, 0.35565E+00_JPRB, 0.35746E+00_JPRB, 0.36038E+00_JPRB, 0.36301E+00_JPRB, 0.36516E+00_JPRB, 0.36468E+00_JPRB ,&
&  0.36382E+00_JPRB, 0.36280E+00_JPRB, 0.36358E+00_JPRB, 0.36426E+00_JPRB, 0.36321E+00_JPRB /)
ZBURD(:, 63,  5) = (/ &
&  0.35625E+00_JPRB, 0.35641E+00_JPRB, 0.35994E+00_JPRB, 0.36371E+00_JPRB, 0.36472E+00_JPRB, 0.36467E+00_JPRB, 0.36229E+00_JPRB ,&
&  0.36068E+00_JPRB, 0.36182E+00_JPRB, 0.36354E+00_JPRB, 0.36470E+00_JPRB, 0.36667E+00_JPRB, 0.36930E+00_JPRB, 0.37135E+00_JPRB ,&
&  0.37134E+00_JPRB, 0.36942E+00_JPRB, 0.36617E+00_JPRB, 0.36668E+00_JPRB, 0.36662E+00_JPRB, 0.36871E+00_JPRB, 0.37622E+00_JPRB ,&
&  0.38164E+00_JPRB, 0.38537E+00_JPRB, 0.38853E+00_JPRB, 0.38899E+00_JPRB, 0.38854E+00_JPRB, 0.39014E+00_JPRB, 0.38687E+00_JPRB ,&
&  0.38071E+00_JPRB, 0.37504E+00_JPRB, 0.36981E+00_JPRB, 0.36271E+00_JPRB, 0.36525E+00_JPRB, 0.36269E+00_JPRB, 0.35960E+00_JPRB ,&
&  0.35542E+00_JPRB, 0.35006E+00_JPRB, 0.34384E+00_JPRB, 0.33800E+00_JPRB, 0.33368E+00_JPRB, 0.32968E+00_JPRB, 0.32446E+00_JPRB ,&
&  0.32273E+00_JPRB, 0.32476E+00_JPRB, 0.33054E+00_JPRB, 0.33323E+00_JPRB, 0.33082E+00_JPRB, 0.33179E+00_JPRB, 0.33849E+00_JPRB ,&
&  0.34532E+00_JPRB, 0.34850E+00_JPRB, 0.34831E+00_JPRB, 0.34878E+00_JPRB, 0.34937E+00_JPRB, 0.35074E+00_JPRB, 0.35141E+00_JPRB ,&
&  0.35037E+00_JPRB, 0.34870E+00_JPRB, 0.34614E+00_JPRB, 0.34012E+00_JPRB, 0.33297E+00_JPRB, 0.32687E+00_JPRB, 0.32439E+00_JPRB ,&
&  0.32694E+00_JPRB, 0.32682E+00_JPRB, 0.32706E+00_JPRB, 0.32871E+00_JPRB, 0.32929E+00_JPRB, 0.32858E+00_JPRB, 0.32763E+00_JPRB ,&
&  0.32895E+00_JPRB, 0.32808E+00_JPRB, 0.32946E+00_JPRB, 0.32664E+00_JPRB, 0.32209E+00_JPRB, 0.31758E+00_JPRB, 0.31330E+00_JPRB ,&
&  0.31081E+00_JPRB, 0.30999E+00_JPRB, 0.31197E+00_JPRB, 0.31267E+00_JPRB, 0.31459E+00_JPRB, 0.31684E+00_JPRB, 0.31877E+00_JPRB ,&
&  0.31987E+00_JPRB, 0.32127E+00_JPRB, 0.32392E+00_JPRB, 0.32923E+00_JPRB, 0.33481E+00_JPRB, 0.33966E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34925E+00_JPRB, 0.35255E+00_JPRB, 0.35843E+00_JPRB, 0.35972E+00_JPRB, 0.35915E+00_JPRB /)
ZBURD(:, 64,  5) = (/ &
&  0.35474E+00_JPRB, 0.35246E+00_JPRB, 0.35270E+00_JPRB, 0.35375E+00_JPRB, 0.35346E+00_JPRB, 0.35283E+00_JPRB, 0.35036E+00_JPRB ,&
&  0.34814E+00_JPRB, 0.34856E+00_JPRB, 0.34989E+00_JPRB, 0.35234E+00_JPRB, 0.35664E+00_JPRB, 0.36121E+00_JPRB, 0.36055E+00_JPRB ,&
&  0.35737E+00_JPRB, 0.35593E+00_JPRB, 0.35799E+00_JPRB, 0.36007E+00_JPRB, 0.36043E+00_JPRB, 0.36111E+00_JPRB, 0.36478E+00_JPRB ,&
&  0.36943E+00_JPRB, 0.37153E+00_JPRB, 0.37448E+00_JPRB, 0.37413E+00_JPRB, 0.37052E+00_JPRB, 0.37081E+00_JPRB, 0.36905E+00_JPRB ,&
&  0.36418E+00_JPRB, 0.35826E+00_JPRB, 0.34914E+00_JPRB, 0.34998E+00_JPRB, 0.34632E+00_JPRB, 0.34331E+00_JPRB, 0.33881E+00_JPRB ,&
&  0.33388E+00_JPRB, 0.32980E+00_JPRB, 0.32524E+00_JPRB, 0.32155E+00_JPRB, 0.31967E+00_JPRB, 0.31953E+00_JPRB, 0.31734E+00_JPRB ,&
&  0.31595E+00_JPRB, 0.31760E+00_JPRB, 0.31901E+00_JPRB, 0.31629E+00_JPRB, 0.31628E+00_JPRB, 0.32276E+00_JPRB, 0.32793E+00_JPRB ,&
&  0.32991E+00_JPRB, 0.32983E+00_JPRB, 0.32717E+00_JPRB, 0.32645E+00_JPRB, 0.32539E+00_JPRB, 0.32341E+00_JPRB, 0.32002E+00_JPRB ,&
&  0.31493E+00_JPRB, 0.31241E+00_JPRB, 0.31069E+00_JPRB, 0.30722E+00_JPRB, 0.30153E+00_JPRB, 0.28473E+00_JPRB, 0.27547E+00_JPRB ,&
&  0.27544E+00_JPRB, 0.28434E+00_JPRB, 0.29909E+00_JPRB, 0.30579E+00_JPRB, 0.31162E+00_JPRB, 0.31670E+00_JPRB, 0.31744E+00_JPRB ,&
&  0.31862E+00_JPRB, 0.29848E+00_JPRB, 0.28745E+00_JPRB, 0.27866E+00_JPRB, 0.27073E+00_JPRB, 0.26563E+00_JPRB, 0.26404E+00_JPRB ,&
&  0.26676E+00_JPRB, 0.27256E+00_JPRB, 0.27771E+00_JPRB, 0.28026E+00_JPRB, 0.28003E+00_JPRB, 0.27704E+00_JPRB, 0.27387E+00_JPRB ,&
&  0.27371E+00_JPRB, 0.28044E+00_JPRB, 0.29353E+00_JPRB, 0.30650E+00_JPRB, 0.31564E+00_JPRB, 0.32538E+00_JPRB, 0.33355E+00_JPRB ,&
&  0.33918E+00_JPRB, 0.34447E+00_JPRB, 0.35092E+00_JPRB, 0.35328E+00_JPRB, 0.35437E+00_JPRB /)
ZBURD(:, 65,  5) = (/ &
&  0.34409E+00_JPRB, 0.34065E+00_JPRB, 0.34079E+00_JPRB, 0.34116E+00_JPRB, 0.34005E+00_JPRB, 0.33998E+00_JPRB, 0.33933E+00_JPRB ,&
&  0.33912E+00_JPRB, 0.33725E+00_JPRB, 0.33532E+00_JPRB, 0.33566E+00_JPRB, 0.33767E+00_JPRB, 0.33995E+00_JPRB, 0.34048E+00_JPRB ,&
&  0.34021E+00_JPRB, 0.34146E+00_JPRB, 0.34474E+00_JPRB, 0.34825E+00_JPRB, 0.34898E+00_JPRB, 0.34930E+00_JPRB, 0.35113E+00_JPRB ,&
&  0.35347E+00_JPRB, 0.35431E+00_JPRB, 0.35709E+00_JPRB, 0.35495E+00_JPRB, 0.35054E+00_JPRB, 0.34881E+00_JPRB, 0.34728E+00_JPRB ,&
&  0.34095E+00_JPRB, 0.31769E+00_JPRB, 0.30249E+00_JPRB, 0.30683E+00_JPRB, 0.32258E+00_JPRB, 0.32205E+00_JPRB, 0.31990E+00_JPRB ,&
&  0.31768E+00_JPRB, 0.31474E+00_JPRB, 0.31221E+00_JPRB, 0.31125E+00_JPRB, 0.31132E+00_JPRB, 0.31121E+00_JPRB, 0.30973E+00_JPRB ,&
&  0.31008E+00_JPRB, 0.31476E+00_JPRB, 0.32080E+00_JPRB, 0.31824E+00_JPRB, 0.31409E+00_JPRB, 0.30729E+00_JPRB, 0.30295E+00_JPRB ,&
&  0.29886E+00_JPRB, 0.29456E+00_JPRB, 0.29193E+00_JPRB, 0.29107E+00_JPRB, 0.29077E+00_JPRB, 0.28873E+00_JPRB, 0.28529E+00_JPRB ,&
&  0.27787E+00_JPRB, 0.26879E+00_JPRB, 0.25939E+00_JPRB, 0.25014E+00_JPRB, 0.24336E+00_JPRB, 0.23860E+00_JPRB, 0.23493E+00_JPRB ,&
&  0.23660E+00_JPRB, 0.24782E+00_JPRB, 0.26643E+00_JPRB, 0.28624E+00_JPRB, 0.30572E+00_JPRB, 0.28356E+00_JPRB, 0.27288E+00_JPRB ,&
&  0.26164E+00_JPRB, 0.25191E+00_JPRB, 0.24513E+00_JPRB, 0.24046E+00_JPRB, 0.23708E+00_JPRB, 0.23515E+00_JPRB, 0.23502E+00_JPRB ,&
&  0.23791E+00_JPRB, 0.24194E+00_JPRB, 0.24559E+00_JPRB, 0.24650E+00_JPRB, 0.24585E+00_JPRB, 0.24411E+00_JPRB, 0.24316E+00_JPRB ,&
&  0.24437E+00_JPRB, 0.24982E+00_JPRB, 0.25889E+00_JPRB, 0.27200E+00_JPRB, 0.28387E+00_JPRB, 0.29614E+00_JPRB, 0.31045E+00_JPRB ,&
&  0.32568E+00_JPRB, 0.33994E+00_JPRB, 0.34361E+00_JPRB, 0.34679E+00_JPRB, 0.34691E+00_JPRB /)
ZBURD(:, 66,  5) = (/ &
&  0.33935E+00_JPRB, 0.33617E+00_JPRB, 0.33423E+00_JPRB, 0.33453E+00_JPRB, 0.33469E+00_JPRB, 0.33729E+00_JPRB, 0.33657E+00_JPRB ,&
&  0.33224E+00_JPRB, 0.32763E+00_JPRB, 0.32624E+00_JPRB, 0.32612E+00_JPRB, 0.32501E+00_JPRB, 0.32370E+00_JPRB, 0.32436E+00_JPRB ,&
&  0.32280E+00_JPRB, 0.32462E+00_JPRB, 0.32743E+00_JPRB, 0.32891E+00_JPRB, 0.32965E+00_JPRB, 0.32987E+00_JPRB, 0.33009E+00_JPRB ,&
&  0.32400E+00_JPRB, 0.32226E+00_JPRB, 0.32788E+00_JPRB, 0.32520E+00_JPRB, 0.32133E+00_JPRB, 0.31801E+00_JPRB, 0.31691E+00_JPRB ,&
&  0.30852E+00_JPRB, 0.29685E+00_JPRB, 0.28413E+00_JPRB, 0.28968E+00_JPRB, 0.30657E+00_JPRB, 0.30408E+00_JPRB, 0.30180E+00_JPRB ,&
&  0.30153E+00_JPRB, 0.30085E+00_JPRB, 0.30171E+00_JPRB, 0.30222E+00_JPRB, 0.30318E+00_JPRB, 0.30334E+00_JPRB, 0.30455E+00_JPRB ,&
&  0.30598E+00_JPRB, 0.31188E+00_JPRB, 0.31542E+00_JPRB, 0.29612E+00_JPRB, 0.27944E+00_JPRB, 0.26671E+00_JPRB, 0.25453E+00_JPRB ,&
&  0.24367E+00_JPRB, 0.24110E+00_JPRB, 0.23756E+00_JPRB, 0.23128E+00_JPRB, 0.23177E+00_JPRB, 0.23240E+00_JPRB, 0.23160E+00_JPRB ,&
&  0.22960E+00_JPRB, 0.22837E+00_JPRB, 0.22696E+00_JPRB, 0.22460E+00_JPRB, 0.22372E+00_JPRB, 0.22498E+00_JPRB, 0.22855E+00_JPRB ,&
&  0.23417E+00_JPRB, 0.24048E+00_JPRB, 0.25483E+00_JPRB, 0.26707E+00_JPRB, 0.26342E+00_JPRB, 0.25596E+00_JPRB, 0.24872E+00_JPRB ,&
&  0.23814E+00_JPRB, 0.23289E+00_JPRB, 0.22771E+00_JPRB, 0.22351E+00_JPRB, 0.22159E+00_JPRB, 0.22180E+00_JPRB, 0.22261E+00_JPRB ,&
&  0.22382E+00_JPRB, 0.22584E+00_JPRB, 0.22664E+00_JPRB, 0.22626E+00_JPRB, 0.22550E+00_JPRB, 0.22605E+00_JPRB, 0.22771E+00_JPRB ,&
&  0.23075E+00_JPRB, 0.23600E+00_JPRB, 0.24239E+00_JPRB, 0.25026E+00_JPRB, 0.25707E+00_JPRB, 0.26342E+00_JPRB, 0.27324E+00_JPRB ,&
&  0.28124E+00_JPRB, 0.29956E+00_JPRB, 0.32558E+00_JPRB, 0.35084E+00_JPRB, 0.34554E+00_JPRB /)
ZBURD(:, 67,  5) = (/ &
&  0.32104E+00_JPRB, 0.31842E+00_JPRB, 0.31582E+00_JPRB, 0.31549E+00_JPRB, 0.31617E+00_JPRB, 0.31897E+00_JPRB, 0.31663E+00_JPRB ,&
&  0.31210E+00_JPRB, 0.31044E+00_JPRB, 0.31179E+00_JPRB, 0.31181E+00_JPRB, 0.31123E+00_JPRB, 0.30091E+00_JPRB, 0.29507E+00_JPRB ,&
&  0.28826E+00_JPRB, 0.28596E+00_JPRB, 0.28414E+00_JPRB, 0.28495E+00_JPRB, 0.28721E+00_JPRB, 0.28863E+00_JPRB, 0.28451E+00_JPRB ,&
&  0.28011E+00_JPRB, 0.27591E+00_JPRB, 0.27155E+00_JPRB, 0.26971E+00_JPRB, 0.27105E+00_JPRB, 0.27246E+00_JPRB, 0.27265E+00_JPRB ,&
&  0.27338E+00_JPRB, 0.27619E+00_JPRB, 0.27879E+00_JPRB, 0.28236E+00_JPRB, 0.29509E+00_JPRB, 0.29322E+00_JPRB, 0.29192E+00_JPRB ,&
&  0.29226E+00_JPRB, 0.29271E+00_JPRB, 0.29380E+00_JPRB, 0.29449E+00_JPRB, 0.29856E+00_JPRB, 0.30150E+00_JPRB, 0.30358E+00_JPRB ,&
&  0.28992E+00_JPRB, 0.28212E+00_JPRB, 0.27441E+00_JPRB, 0.26531E+00_JPRB, 0.25454E+00_JPRB, 0.24583E+00_JPRB, 0.23783E+00_JPRB ,&
&  0.23104E+00_JPRB, 0.22460E+00_JPRB, 0.21878E+00_JPRB, 0.21650E+00_JPRB, 0.21569E+00_JPRB, 0.21444E+00_JPRB, 0.21434E+00_JPRB ,&
&  0.21488E+00_JPRB, 0.21564E+00_JPRB, 0.21613E+00_JPRB, 0.21766E+00_JPRB, 0.22005E+00_JPRB, 0.22381E+00_JPRB, 0.22783E+00_JPRB ,&
&  0.23382E+00_JPRB, 0.23823E+00_JPRB, 0.24341E+00_JPRB, 0.24779E+00_JPRB, 0.24632E+00_JPRB, 0.24126E+00_JPRB, 0.23586E+00_JPRB ,&
&  0.22929E+00_JPRB, 0.22548E+00_JPRB, 0.22162E+00_JPRB, 0.21881E+00_JPRB, 0.21720E+00_JPRB, 0.21625E+00_JPRB, 0.21651E+00_JPRB ,&
&  0.21659E+00_JPRB, 0.21704E+00_JPRB, 0.21656E+00_JPRB, 0.21683E+00_JPRB, 0.21767E+00_JPRB, 0.21954E+00_JPRB, 0.22198E+00_JPRB ,&
&  0.22575E+00_JPRB, 0.23013E+00_JPRB, 0.23532E+00_JPRB, 0.24100E+00_JPRB, 0.24925E+00_JPRB, 0.26012E+00_JPRB, 0.27564E+00_JPRB ,&
&  0.29238E+00_JPRB, 0.32347E+00_JPRB, 0.33588E+00_JPRB, 0.32733E+00_JPRB, 0.32450E+00_JPRB /)
ZBURD(:, 68,  5) = (/ &
&  0.30560E+00_JPRB, 0.30681E+00_JPRB, 0.30806E+00_JPRB, 0.31036E+00_JPRB, 0.31316E+00_JPRB, 0.31419E+00_JPRB, 0.31444E+00_JPRB ,&
&  0.30523E+00_JPRB, 0.29768E+00_JPRB, 0.29090E+00_JPRB, 0.28063E+00_JPRB, 0.27249E+00_JPRB, 0.26519E+00_JPRB, 0.25992E+00_JPRB ,&
&  0.25507E+00_JPRB, 0.25257E+00_JPRB, 0.24985E+00_JPRB, 0.24903E+00_JPRB, 0.24888E+00_JPRB, 0.24955E+00_JPRB, 0.24811E+00_JPRB ,&
&  0.24643E+00_JPRB, 0.24755E+00_JPRB, 0.24827E+00_JPRB, 0.25273E+00_JPRB, 0.25930E+00_JPRB, 0.26674E+00_JPRB, 0.27400E+00_JPRB ,&
&  0.28325E+00_JPRB, 0.28721E+00_JPRB, 0.29034E+00_JPRB, 0.29109E+00_JPRB, 0.28978E+00_JPRB, 0.28919E+00_JPRB, 0.28713E+00_JPRB ,&
&  0.28677E+00_JPRB, 0.28635E+00_JPRB, 0.28768E+00_JPRB, 0.28877E+00_JPRB, 0.28735E+00_JPRB, 0.28493E+00_JPRB, 0.27943E+00_JPRB ,&
&  0.27130E+00_JPRB, 0.26473E+00_JPRB, 0.25850E+00_JPRB, 0.25363E+00_JPRB, 0.24685E+00_JPRB, 0.24088E+00_JPRB, 0.23501E+00_JPRB ,&
&  0.22990E+00_JPRB, 0.22606E+00_JPRB, 0.22147E+00_JPRB, 0.21795E+00_JPRB, 0.21632E+00_JPRB, 0.21436E+00_JPRB, 0.21351E+00_JPRB ,&
&  0.21346E+00_JPRB, 0.21384E+00_JPRB, 0.21594E+00_JPRB, 0.21768E+00_JPRB, 0.22040E+00_JPRB, 0.22355E+00_JPRB, 0.22547E+00_JPRB ,&
&  0.22881E+00_JPRB, 0.23170E+00_JPRB, 0.23327E+00_JPRB, 0.23443E+00_JPRB, 0.23349E+00_JPRB, 0.23061E+00_JPRB, 0.22767E+00_JPRB ,&
&  0.22382E+00_JPRB, 0.22109E+00_JPRB, 0.21932E+00_JPRB, 0.21739E+00_JPRB, 0.21639E+00_JPRB, 0.21551E+00_JPRB, 0.21457E+00_JPRB ,&
&  0.21384E+00_JPRB, 0.21351E+00_JPRB, 0.21317E+00_JPRB, 0.21421E+00_JPRB, 0.21533E+00_JPRB, 0.21740E+00_JPRB, 0.21991E+00_JPRB ,&
&  0.22223E+00_JPRB, 0.22579E+00_JPRB, 0.22968E+00_JPRB, 0.23498E+00_JPRB, 0.24452E+00_JPRB, 0.25238E+00_JPRB, 0.26945E+00_JPRB ,&
&  0.28230E+00_JPRB, 0.30821E+00_JPRB, 0.31880E+00_JPRB, 0.30718E+00_JPRB, 0.30586E+00_JPRB /)
ZBURD(:, 69,  5) = (/ &
&  0.29532E+00_JPRB, 0.29742E+00_JPRB, 0.29838E+00_JPRB, 0.30037E+00_JPRB, 0.30239E+00_JPRB, 0.30312E+00_JPRB, 0.30352E+00_JPRB ,&
&  0.30105E+00_JPRB, 0.29481E+00_JPRB, 0.29004E+00_JPRB, 0.28357E+00_JPRB, 0.27727E+00_JPRB, 0.27065E+00_JPRB, 0.26467E+00_JPRB ,&
&  0.25844E+00_JPRB, 0.25279E+00_JPRB, 0.24707E+00_JPRB, 0.24225E+00_JPRB, 0.23884E+00_JPRB, 0.23603E+00_JPRB, 0.23303E+00_JPRB ,&
&  0.23045E+00_JPRB, 0.23154E+00_JPRB, 0.23413E+00_JPRB, 0.23911E+00_JPRB, 0.24691E+00_JPRB, 0.25412E+00_JPRB, 0.26200E+00_JPRB ,&
&  0.27575E+00_JPRB, 0.27933E+00_JPRB, 0.27926E+00_JPRB, 0.28128E+00_JPRB, 0.28072E+00_JPRB, 0.28126E+00_JPRB, 0.27135E+00_JPRB ,&
&  0.27369E+00_JPRB, 0.27581E+00_JPRB, 0.28345E+00_JPRB, 0.28658E+00_JPRB, 0.28137E+00_JPRB, 0.27702E+00_JPRB, 0.26914E+00_JPRB ,&
&  0.26273E+00_JPRB, 0.25767E+00_JPRB, 0.25431E+00_JPRB, 0.25231E+00_JPRB, 0.24766E+00_JPRB, 0.24423E+00_JPRB, 0.23849E+00_JPRB ,&
&  0.23372E+00_JPRB, 0.23057E+00_JPRB, 0.22659E+00_JPRB, 0.22411E+00_JPRB, 0.22210E+00_JPRB, 0.22014E+00_JPRB, 0.21866E+00_JPRB ,&
&  0.21784E+00_JPRB, 0.21716E+00_JPRB, 0.21784E+00_JPRB, 0.21862E+00_JPRB, 0.22013E+00_JPRB, 0.22134E+00_JPRB, 0.22160E+00_JPRB ,&
&  0.22187E+00_JPRB, 0.22231E+00_JPRB, 0.22246E+00_JPRB, 0.22310E+00_JPRB, 0.22259E+00_JPRB, 0.22171E+00_JPRB, 0.22061E+00_JPRB ,&
&  0.21942E+00_JPRB, 0.21896E+00_JPRB, 0.21814E+00_JPRB, 0.21762E+00_JPRB, 0.21589E+00_JPRB, 0.21469E+00_JPRB, 0.21403E+00_JPRB ,&
&  0.21341E+00_JPRB, 0.21382E+00_JPRB, 0.21394E+00_JPRB, 0.21477E+00_JPRB, 0.21608E+00_JPRB, 0.21727E+00_JPRB, 0.21937E+00_JPRB ,&
&  0.22202E+00_JPRB, 0.22510E+00_JPRB, 0.23034E+00_JPRB, 0.23589E+00_JPRB, 0.24200E+00_JPRB, 0.24974E+00_JPRB, 0.25682E+00_JPRB ,&
&  0.26673E+00_JPRB, 0.28191E+00_JPRB, 0.28401E+00_JPRB, 0.28776E+00_JPRB, 0.29246E+00_JPRB /)
ZBURD(:, 70,  5) = (/ &
&  0.28040E+00_JPRB, 0.28402E+00_JPRB, 0.28644E+00_JPRB, 0.28804E+00_JPRB, 0.28962E+00_JPRB, 0.28894E+00_JPRB, 0.28739E+00_JPRB ,&
&  0.28323E+00_JPRB, 0.28178E+00_JPRB, 0.27902E+00_JPRB, 0.27606E+00_JPRB, 0.27261E+00_JPRB, 0.26865E+00_JPRB, 0.26462E+00_JPRB ,&
&  0.26011E+00_JPRB, 0.25522E+00_JPRB, 0.24939E+00_JPRB, 0.24303E+00_JPRB, 0.23721E+00_JPRB, 0.23177E+00_JPRB, 0.22861E+00_JPRB ,&
&  0.22717E+00_JPRB, 0.22798E+00_JPRB, 0.23087E+00_JPRB, 0.23443E+00_JPRB, 0.23875E+00_JPRB, 0.24362E+00_JPRB, 0.24969E+00_JPRB ,&
&  0.25488E+00_JPRB, 0.26108E+00_JPRB, 0.26485E+00_JPRB, 0.26646E+00_JPRB, 0.26508E+00_JPRB, 0.26500E+00_JPRB, 0.26405E+00_JPRB ,&
&  0.26278E+00_JPRB, 0.26079E+00_JPRB, 0.26001E+00_JPRB, 0.26013E+00_JPRB, 0.25931E+00_JPRB, 0.25744E+00_JPRB, 0.25481E+00_JPRB ,&
&  0.25236E+00_JPRB, 0.25024E+00_JPRB, 0.24757E+00_JPRB, 0.24529E+00_JPRB, 0.24357E+00_JPRB, 0.24144E+00_JPRB, 0.23859E+00_JPRB ,&
&  0.23603E+00_JPRB, 0.23349E+00_JPRB, 0.23048E+00_JPRB, 0.22794E+00_JPRB, 0.22541E+00_JPRB, 0.22357E+00_JPRB, 0.22192E+00_JPRB ,&
&  0.22121E+00_JPRB, 0.22047E+00_JPRB, 0.22024E+00_JPRB, 0.22012E+00_JPRB, 0.22054E+00_JPRB, 0.22025E+00_JPRB, 0.21983E+00_JPRB ,&
&  0.21932E+00_JPRB, 0.21918E+00_JPRB, 0.21865E+00_JPRB, 0.21846E+00_JPRB, 0.21826E+00_JPRB, 0.21758E+00_JPRB, 0.21724E+00_JPRB ,&
&  0.21731E+00_JPRB, 0.21720E+00_JPRB, 0.21663E+00_JPRB, 0.21657E+00_JPRB, 0.21621E+00_JPRB, 0.21568E+00_JPRB, 0.21595E+00_JPRB ,&
&  0.21576E+00_JPRB, 0.21596E+00_JPRB, 0.21628E+00_JPRB, 0.21677E+00_JPRB, 0.21787E+00_JPRB, 0.21926E+00_JPRB, 0.22114E+00_JPRB ,&
&  0.22404E+00_JPRB, 0.22667E+00_JPRB, 0.22932E+00_JPRB, 0.23239E+00_JPRB, 0.23823E+00_JPRB, 0.24108E+00_JPRB, 0.24610E+00_JPRB ,&
&  0.25075E+00_JPRB, 0.24937E+00_JPRB, 0.27228E+00_JPRB, 0.27339E+00_JPRB, 0.27557E+00_JPRB /)
ZBURD(:, 71,  5) = (/ &
&  0.23717E+00_JPRB, 0.24223E+00_JPRB, 0.24596E+00_JPRB, 0.24932E+00_JPRB, 0.24671E+00_JPRB, 0.26647E+00_JPRB, 0.26452E+00_JPRB ,&
&  0.26448E+00_JPRB, 0.25844E+00_JPRB, 0.25974E+00_JPRB, 0.25855E+00_JPRB, 0.25624E+00_JPRB, 0.25324E+00_JPRB, 0.25006E+00_JPRB ,&
&  0.24676E+00_JPRB, 0.24283E+00_JPRB, 0.23922E+00_JPRB, 0.23575E+00_JPRB, 0.23289E+00_JPRB, 0.23038E+00_JPRB, 0.22901E+00_JPRB ,&
&  0.22907E+00_JPRB, 0.23058E+00_JPRB, 0.23318E+00_JPRB, 0.23460E+00_JPRB, 0.23613E+00_JPRB, 0.23773E+00_JPRB, 0.23944E+00_JPRB ,&
&  0.24087E+00_JPRB, 0.24279E+00_JPRB, 0.24380E+00_JPRB, 0.24344E+00_JPRB, 0.24241E+00_JPRB, 0.24199E+00_JPRB, 0.24188E+00_JPRB ,&
&  0.24241E+00_JPRB, 0.24269E+00_JPRB, 0.24282E+00_JPRB, 0.24309E+00_JPRB, 0.24335E+00_JPRB, 0.24286E+00_JPRB, 0.24167E+00_JPRB ,&
&  0.24066E+00_JPRB, 0.23985E+00_JPRB, 0.23821E+00_JPRB, 0.23665E+00_JPRB, 0.23519E+00_JPRB, 0.23366E+00_JPRB, 0.23207E+00_JPRB ,&
&  0.23092E+00_JPRB, 0.22977E+00_JPRB, 0.22833E+00_JPRB, 0.22667E+00_JPRB, 0.22510E+00_JPRB, 0.22373E+00_JPRB, 0.22264E+00_JPRB ,&
&  0.22240E+00_JPRB, 0.22181E+00_JPRB, 0.22129E+00_JPRB, 0.22078E+00_JPRB, 0.22064E+00_JPRB, 0.22019E+00_JPRB, 0.21987E+00_JPRB ,&
&  0.21962E+00_JPRB, 0.21931E+00_JPRB, 0.21898E+00_JPRB, 0.21865E+00_JPRB, 0.21843E+00_JPRB, 0.21812E+00_JPRB, 0.21782E+00_JPRB ,&
&  0.21762E+00_JPRB, 0.21747E+00_JPRB, 0.21730E+00_JPRB, 0.21691E+00_JPRB, 0.21651E+00_JPRB, 0.21603E+00_JPRB, 0.21601E+00_JPRB ,&
&  0.21615E+00_JPRB, 0.21642E+00_JPRB, 0.21690E+00_JPRB, 0.21708E+00_JPRB, 0.21791E+00_JPRB, 0.21872E+00_JPRB, 0.21967E+00_JPRB ,&
&  0.22067E+00_JPRB, 0.22223E+00_JPRB, 0.22394E+00_JPRB, 0.22580E+00_JPRB, 0.22877E+00_JPRB, 0.22978E+00_JPRB, 0.23063E+00_JPRB ,&
&  0.23168E+00_JPRB, 0.23528E+00_JPRB, 0.23635E+00_JPRB, 0.23627E+00_JPRB, 0.23696E+00_JPRB /)
ZBURD(:, 72,  5) = (/ &
&  0.22546E+00_JPRB, 0.22574E+00_JPRB, 0.22578E+00_JPRB, 0.22568E+00_JPRB, 0.22387E+00_JPRB, 0.22317E+00_JPRB, 0.22252E+00_JPRB ,&
&  0.22204E+00_JPRB, 0.22180E+00_JPRB, 0.22221E+00_JPRB, 0.22275E+00_JPRB, 0.22330E+00_JPRB, 0.22386E+00_JPRB, 0.22433E+00_JPRB ,&
&  0.22455E+00_JPRB, 0.22479E+00_JPRB, 0.22374E+00_JPRB, 0.22324E+00_JPRB, 0.22310E+00_JPRB, 0.22316E+00_JPRB, 0.22315E+00_JPRB ,&
&  0.22327E+00_JPRB, 0.22336E+00_JPRB, 0.22335E+00_JPRB, 0.22370E+00_JPRB, 0.22431E+00_JPRB, 0.22480E+00_JPRB, 0.22532E+00_JPRB ,&
&  0.22583E+00_JPRB, 0.22655E+00_JPRB, 0.22721E+00_JPRB, 0.22783E+00_JPRB, 0.22862E+00_JPRB, 0.22939E+00_JPRB, 0.22984E+00_JPRB ,&
&  0.23018E+00_JPRB, 0.23027E+00_JPRB, 0.23042E+00_JPRB, 0.23055E+00_JPRB, 0.23079E+00_JPRB, 0.23108E+00_JPRB, 0.23114E+00_JPRB ,&
&  0.23106E+00_JPRB, 0.23090E+00_JPRB, 0.23066E+00_JPRB, 0.23011E+00_JPRB, 0.22954E+00_JPRB, 0.22898E+00_JPRB, 0.22842E+00_JPRB ,&
&  0.22794E+00_JPRB, 0.22740E+00_JPRB, 0.22680E+00_JPRB, 0.22617E+00_JPRB, 0.22550E+00_JPRB, 0.22485E+00_JPRB, 0.22420E+00_JPRB ,&
&  0.22353E+00_JPRB, 0.22310E+00_JPRB, 0.22263E+00_JPRB, 0.22222E+00_JPRB, 0.22190E+00_JPRB, 0.22158E+00_JPRB, 0.22127E+00_JPRB ,&
&  0.22096E+00_JPRB, 0.22059E+00_JPRB, 0.22026E+00_JPRB, 0.21988E+00_JPRB, 0.21954E+00_JPRB, 0.21916E+00_JPRB, 0.21886E+00_JPRB ,&
&  0.21857E+00_JPRB, 0.21831E+00_JPRB, 0.21809E+00_JPRB, 0.21799E+00_JPRB, 0.21793E+00_JPRB, 0.21795E+00_JPRB, 0.21798E+00_JPRB ,&
&  0.21804E+00_JPRB, 0.21807E+00_JPRB, 0.21809E+00_JPRB, 0.21795E+00_JPRB, 0.21815E+00_JPRB, 0.21835E+00_JPRB, 0.21865E+00_JPRB ,&
&  0.21893E+00_JPRB, 0.21939E+00_JPRB, 0.21986E+00_JPRB, 0.22038E+00_JPRB, 0.22119E+00_JPRB, 0.22148E+00_JPRB, 0.22174E+00_JPRB ,&
&  0.22202E+00_JPRB, 0.22261E+00_JPRB, 0.22312E+00_JPRB, 0.22339E+00_JPRB, 0.22396E+00_JPRB /)
ZBURD(:, 73,  5) = (/ &
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB ,&
&  0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB, 0.22156E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART10',1,ZHOOK_HANDLE)
END SUBROUTINE PART10



SUBROUTINE PART11

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART11',0,ZHOOK_HANDLE)


!-- month=  6
ZBURD(:,  1,  6) = (/ &
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB, 0.12204E+01_JPRB /)
ZBURD(:,  2,  6) = (/ &
&  0.11837E+01_JPRB, 0.11758E+01_JPRB, 0.11669E+01_JPRB, 0.11575E+01_JPRB, 0.11467E+01_JPRB, 0.11354E+01_JPRB, 0.11223E+01_JPRB ,&
&  0.11092E+01_JPRB, 0.10964E+01_JPRB, 0.10852E+01_JPRB, 0.10752E+01_JPRB, 0.10660E+01_JPRB, 0.10589E+01_JPRB, 0.10557E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10550E+01_JPRB, 0.10554E+01_JPRB, 0.10556E+01_JPRB, 0.10552E+01_JPRB, 0.10542E+01_JPRB, 0.10532E+01_JPRB ,&
&  0.10526E+01_JPRB, 0.10527E+01_JPRB, 0.10524E+01_JPRB, 0.10531E+01_JPRB, 0.10543E+01_JPRB, 0.10559E+01_JPRB, 0.10573E+01_JPRB ,&
&  0.10601E+01_JPRB, 0.10635E+01_JPRB, 0.10679E+01_JPRB, 0.10725E+01_JPRB, 0.10807E+01_JPRB, 0.10899E+01_JPRB, 0.10997E+01_JPRB ,&
&  0.11094E+01_JPRB, 0.11197E+01_JPRB, 0.11294E+01_JPRB, 0.11384E+01_JPRB, 0.11467E+01_JPRB, 0.11559E+01_JPRB, 0.11667E+01_JPRB ,&
&  0.11789E+01_JPRB, 0.11919E+01_JPRB, 0.12054E+01_JPRB, 0.12191E+01_JPRB, 0.12322E+01_JPRB, 0.12454E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.12855E+01_JPRB, 0.12986E+01_JPRB, 0.13114E+01_JPRB, 0.13242E+01_JPRB, 0.13385E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.13712E+01_JPRB, 0.13851E+01_JPRB, 0.14000E+01_JPRB, 0.14160E+01_JPRB, 0.14350E+01_JPRB, 0.14446E+01_JPRB, 0.14543E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.14766E+01_JPRB, 0.14840E+01_JPRB, 0.14926E+01_JPRB, 0.15011E+01_JPRB, 0.15082E+01_JPRB, 0.15119E+01_JPRB ,&
&  0.15153E+01_JPRB, 0.15180E+01_JPRB, 0.15175E+01_JPRB, 0.15137E+01_JPRB, 0.15080E+01_JPRB, 0.15016E+01_JPRB, 0.14901E+01_JPRB ,&
&  0.14790E+01_JPRB, 0.14686E+01_JPRB, 0.14593E+01_JPRB, 0.14464E+01_JPRB, 0.14336E+01_JPRB, 0.14207E+01_JPRB, 0.14076E+01_JPRB ,&
&  0.13915E+01_JPRB, 0.13754E+01_JPRB, 0.13585E+01_JPRB, 0.13406E+01_JPRB, 0.13228E+01_JPRB, 0.13044E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.12646E+01_JPRB, 0.12453E+01_JPRB, 0.12247E+01_JPRB, 0.12050E+01_JPRB, 0.11903E+01_JPRB /)
ZBURD(:,  3,  6) = (/ &
&  0.11439E+01_JPRB, 0.11310E+01_JPRB, 0.11242E+01_JPRB, 0.11178E+01_JPRB, 0.11092E+01_JPRB, 0.10982E+01_JPRB, 0.10867E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.10648E+01_JPRB, 0.10538E+01_JPRB, 0.10492E+01_JPRB, 0.10429E+01_JPRB, 0.10388E+01_JPRB, 0.10359E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.10312E+01_JPRB, 0.10291E+01_JPRB, 0.10249E+01_JPRB, 0.10184E+01_JPRB, 0.10100E+01_JPRB, 0.10031E+01_JPRB ,&
&  0.99871E+00_JPRB, 0.99541E+00_JPRB, 0.99115E+00_JPRB, 0.98744E+00_JPRB, 0.98740E+00_JPRB, 0.98675E+00_JPRB, 0.98491E+00_JPRB ,&
&  0.98906E+00_JPRB, 0.99383E+00_JPRB, 0.99582E+00_JPRB, 0.99534E+00_JPRB, 0.99385E+00_JPRB, 0.99434E+00_JPRB, 0.99983E+00_JPRB ,&
&  0.10069E+01_JPRB, 0.10269E+01_JPRB, 0.10455E+01_JPRB, 0.10632E+01_JPRB, 0.10804E+01_JPRB, 0.11008E+01_JPRB, 0.11250E+01_JPRB ,&
&  0.11516E+01_JPRB, 0.11767E+01_JPRB, 0.12037E+01_JPRB, 0.12346E+01_JPRB, 0.12661E+01_JPRB, 0.12982E+01_JPRB, 0.13316E+01_JPRB ,&
&  0.13631E+01_JPRB, 0.13949E+01_JPRB, 0.14308E+01_JPRB, 0.14681E+01_JPRB, 0.15015E+01_JPRB, 0.15395E+01_JPRB, 0.15809E+01_JPRB ,&
&  0.16279E+01_JPRB, 0.16571E+01_JPRB, 0.16896E+01_JPRB, 0.17238E+01_JPRB, 0.17619E+01_JPRB, 0.17883E+01_JPRB, 0.18134E+01_JPRB ,&
&  0.18316E+01_JPRB, 0.18430E+01_JPRB, 0.18384E+01_JPRB, 0.18310E+01_JPRB, 0.18279E+01_JPRB, 0.18211E+01_JPRB, 0.18114E+01_JPRB ,&
&  0.18041E+01_JPRB, 0.18011E+01_JPRB, 0.17900E+01_JPRB, 0.17733E+01_JPRB, 0.17500E+01_JPRB, 0.17249E+01_JPRB, 0.16834E+01_JPRB ,&
&  0.16437E+01_JPRB, 0.16121E+01_JPRB, 0.15889E+01_JPRB, 0.15634E+01_JPRB, 0.15390E+01_JPRB, 0.15160E+01_JPRB, 0.14934E+01_JPRB ,&
&  0.14712E+01_JPRB, 0.14470E+01_JPRB, 0.14217E+01_JPRB, 0.13949E+01_JPRB, 0.13726E+01_JPRB, 0.13432E+01_JPRB, 0.13124E+01_JPRB ,&
&  0.12789E+01_JPRB, 0.12531E+01_JPRB, 0.12239E+01_JPRB, 0.11914E+01_JPRB, 0.11618E+01_JPRB /)
ZBURD(:,  4,  6) = (/ &
&  0.11542E+01_JPRB, 0.11254E+01_JPRB, 0.11073E+01_JPRB, 0.10985E+01_JPRB, 0.10892E+01_JPRB, 0.10809E+01_JPRB, 0.10729E+01_JPRB ,&
&  0.10592E+01_JPRB, 0.10457E+01_JPRB, 0.10426E+01_JPRB, 0.10420E+01_JPRB, 0.10385E+01_JPRB, 0.10334E+01_JPRB, 0.10262E+01_JPRB ,&
&  0.10173E+01_JPRB, 0.10058E+01_JPRB, 0.99270E+00_JPRB, 0.97709E+00_JPRB, 0.96065E+00_JPRB, 0.94255E+00_JPRB, 0.91847E+00_JPRB ,&
&  0.89893E+00_JPRB, 0.88068E+00_JPRB, 0.85867E+00_JPRB, 0.84886E+00_JPRB, 0.84428E+00_JPRB, 0.79731E+00_JPRB, 0.78629E+00_JPRB ,&
&  0.79504E+00_JPRB, 0.80005E+00_JPRB, 0.80524E+00_JPRB, 0.81153E+00_JPRB, 0.81519E+00_JPRB, 0.79769E+00_JPRB, 0.79240E+00_JPRB ,&
&  0.75904E+00_JPRB, 0.78250E+00_JPRB, 0.79932E+00_JPRB, 0.81700E+00_JPRB, 0.83666E+00_JPRB, 0.86646E+00_JPRB, 0.92581E+00_JPRB ,&
&  0.10354E+01_JPRB, 0.10834E+01_JPRB, 0.11401E+01_JPRB, 0.11983E+01_JPRB, 0.12672E+01_JPRB, 0.13363E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.14605E+01_JPRB, 0.15277E+01_JPRB, 0.16047E+01_JPRB, 0.16938E+01_JPRB, 0.17746E+01_JPRB, 0.18472E+01_JPRB, 0.19027E+01_JPRB ,&
&  0.19550E+01_JPRB, 0.19916E+01_JPRB, 0.20196E+01_JPRB, 0.20351E+01_JPRB, 0.20537E+01_JPRB, 0.20668E+01_JPRB, 0.20822E+01_JPRB ,&
&  0.20946E+01_JPRB, 0.20995E+01_JPRB, 0.20858E+01_JPRB, 0.20561E+01_JPRB, 0.20401E+01_JPRB, 0.20385E+01_JPRB, 0.20298E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.19983E+01_JPRB, 0.19716E+01_JPRB, 0.19378E+01_JPRB, 0.18883E+01_JPRB, 0.18439E+01_JPRB, 0.17888E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.17020E+01_JPRB, 0.16686E+01_JPRB, 0.16319E+01_JPRB, 0.15929E+01_JPRB, 0.15537E+01_JPRB, 0.15102E+01_JPRB ,&
&  0.14722E+01_JPRB, 0.14365E+01_JPRB, 0.14084E+01_JPRB, 0.13798E+01_JPRB, 0.13581E+01_JPRB, 0.13316E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.12702E+01_JPRB, 0.12441E+01_JPRB, 0.12141E+01_JPRB, 0.11801E+01_JPRB /)
ZBURD(:,  5,  6) = (/ &
&  0.11828E+01_JPRB, 0.11581E+01_JPRB, 0.11226E+01_JPRB, 0.10987E+01_JPRB, 0.10842E+01_JPRB, 0.10668E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10374E+01_JPRB, 0.10346E+01_JPRB, 0.10350E+01_JPRB, 0.10317E+01_JPRB, 0.10242E+01_JPRB, 0.10128E+01_JPRB, 0.99719E+00_JPRB ,&
&  0.97753E+00_JPRB, 0.95551E+00_JPRB, 0.93608E+00_JPRB, 0.91780E+00_JPRB, 0.89888E+00_JPRB, 0.88132E+00_JPRB, 0.85565E+00_JPRB ,&
&  0.83388E+00_JPRB, 0.81419E+00_JPRB, 0.75308E+00_JPRB, 0.74959E+00_JPRB, 0.73682E+00_JPRB, 0.73355E+00_JPRB, 0.73763E+00_JPRB ,&
&  0.73055E+00_JPRB, 0.71197E+00_JPRB, 0.69322E+00_JPRB, 0.67172E+00_JPRB, 0.65531E+00_JPRB, 0.63640E+00_JPRB, 0.62995E+00_JPRB ,&
&  0.62881E+00_JPRB, 0.63652E+00_JPRB, 0.65061E+00_JPRB, 0.67052E+00_JPRB, 0.69271E+00_JPRB, 0.71452E+00_JPRB, 0.74771E+00_JPRB ,&
&  0.82642E+00_JPRB, 0.92906E+00_JPRB, 0.10656E+01_JPRB, 0.11447E+01_JPRB, 0.12427E+01_JPRB, 0.13339E+01_JPRB, 0.14091E+01_JPRB ,&
&  0.14913E+01_JPRB, 0.15790E+01_JPRB, 0.16779E+01_JPRB, 0.17801E+01_JPRB, 0.18922E+01_JPRB, 0.19968E+01_JPRB, 0.21137E+01_JPRB ,&
&  0.22139E+01_JPRB, 0.23060E+01_JPRB, 0.23646E+01_JPRB, 0.23962E+01_JPRB, 0.24058E+01_JPRB, 0.23893E+01_JPRB, 0.23680E+01_JPRB ,&
&  0.23553E+01_JPRB, 0.23374E+01_JPRB, 0.23131E+01_JPRB, 0.22491E+01_JPRB, 0.22283E+01_JPRB, 0.22271E+01_JPRB, 0.21976E+01_JPRB ,&
&  0.21494E+01_JPRB, 0.20924E+01_JPRB, 0.20313E+01_JPRB, 0.19470E+01_JPRB, 0.18649E+01_JPRB, 0.18311E+01_JPRB, 0.17974E+01_JPRB ,&
&  0.17678E+01_JPRB, 0.17327E+01_JPRB, 0.16982E+01_JPRB, 0.16481E+01_JPRB, 0.15774E+01_JPRB, 0.15107E+01_JPRB, 0.14546E+01_JPRB ,&
&  0.14099E+01_JPRB, 0.13652E+01_JPRB, 0.13272E+01_JPRB, 0.12888E+01_JPRB, 0.12630E+01_JPRB, 0.12404E+01_JPRB, 0.12287E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.12199E+01_JPRB, 0.12162E+01_JPRB, 0.12131E+01_JPRB, 0.12004E+01_JPRB /)
ZBURD(:,  6,  6) = (/ &
&  0.12684E+01_JPRB, 0.12570E+01_JPRB, 0.12242E+01_JPRB, 0.11775E+01_JPRB, 0.11348E+01_JPRB, 0.10968E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10574E+01_JPRB, 0.10486E+01_JPRB, 0.10373E+01_JPRB, 0.10267E+01_JPRB, 0.10107E+01_JPRB, 0.99432E+00_JPRB, 0.97501E+00_JPRB ,&
&  0.94953E+00_JPRB, 0.92219E+00_JPRB, 0.89788E+00_JPRB, 0.88106E+00_JPRB, 0.86726E+00_JPRB, 0.85785E+00_JPRB, 0.84847E+00_JPRB ,&
&  0.83898E+00_JPRB, 0.82767E+00_JPRB, 0.79395E+00_JPRB, 0.78221E+00_JPRB, 0.76779E+00_JPRB, 0.76624E+00_JPRB, 0.75531E+00_JPRB ,&
&  0.76513E+00_JPRB, 0.67319E+00_JPRB, 0.66874E+00_JPRB, 0.64886E+00_JPRB, 0.63223E+00_JPRB, 0.61231E+00_JPRB, 0.59443E+00_JPRB ,&
&  0.58673E+00_JPRB, 0.57556E+00_JPRB, 0.57125E+00_JPRB, 0.58395E+00_JPRB, 0.59888E+00_JPRB, 0.62969E+00_JPRB, 0.66301E+00_JPRB ,&
&  0.71479E+00_JPRB, 0.91786E+00_JPRB, 0.10219E+01_JPRB, 0.11019E+01_JPRB, 0.11897E+01_JPRB, 0.12631E+01_JPRB, 0.13093E+01_JPRB ,&
&  0.13701E+01_JPRB, 0.14401E+01_JPRB, 0.15187E+01_JPRB, 0.15772E+01_JPRB, 0.17669E+01_JPRB, 0.19090E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.22395E+01_JPRB, 0.24171E+01_JPRB, 0.25797E+01_JPRB, 0.27123E+01_JPRB, 0.28008E+01_JPRB, 0.28266E+01_JPRB, 0.27953E+01_JPRB ,&
&  0.27832E+01_JPRB, 0.27627E+01_JPRB, 0.27625E+01_JPRB, 0.26820E+01_JPRB, 0.26422E+01_JPRB, 0.26047E+01_JPRB, 0.25035E+01_JPRB ,&
&  0.23815E+01_JPRB, 0.22717E+01_JPRB, 0.21599E+01_JPRB, 0.20377E+01_JPRB, 0.19163E+01_JPRB, 0.18420E+01_JPRB, 0.17939E+01_JPRB ,&
&  0.17661E+01_JPRB, 0.17209E+01_JPRB, 0.16624E+01_JPRB, 0.15776E+01_JPRB, 0.14967E+01_JPRB, 0.14391E+01_JPRB, 0.13848E+01_JPRB ,&
&  0.13341E+01_JPRB, 0.12821E+01_JPRB, 0.12466E+01_JPRB, 0.12210E+01_JPRB, 0.12071E+01_JPRB, 0.12001E+01_JPRB, 0.12008E+01_JPRB ,&
&  0.12015E+01_JPRB, 0.12081E+01_JPRB, 0.12216E+01_JPRB, 0.12414E+01_JPRB, 0.12591E+01_JPRB /)
ZBURD(:,  7,  6) = (/ &
&  0.12665E+01_JPRB, 0.12715E+01_JPRB, 0.12629E+01_JPRB, 0.12431E+01_JPRB, 0.12117E+01_JPRB, 0.11761E+01_JPRB, 0.11428E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.10875E+01_JPRB, 0.10632E+01_JPRB, 0.10425E+01_JPRB, 0.10199E+01_JPRB, 0.10035E+01_JPRB, 0.98562E+00_JPRB ,&
&  0.97266E+00_JPRB, 0.96487E+00_JPRB, 0.95000E+00_JPRB, 0.93282E+00_JPRB, 0.90782E+00_JPRB, 0.88707E+00_JPRB, 0.87078E+00_JPRB ,&
&  0.85457E+00_JPRB, 0.84090E+00_JPRB, 0.80789E+00_JPRB, 0.78597E+00_JPRB, 0.77231E+00_JPRB, 0.75230E+00_JPRB, 0.75429E+00_JPRB ,&
&  0.73884E+00_JPRB, 0.73656E+00_JPRB, 0.73758E+00_JPRB, 0.72898E+00_JPRB, 0.71673E+00_JPRB, 0.57681E+00_JPRB, 0.54481E+00_JPRB ,&
&  0.53603E+00_JPRB, 0.52801E+00_JPRB, 0.52735E+00_JPRB, 0.53691E+00_JPRB, 0.55430E+00_JPRB, 0.60275E+00_JPRB, 0.64537E+00_JPRB ,&
&  0.67267E+00_JPRB, 0.88398E+00_JPRB, 0.96907E+00_JPRB, 0.10463E+01_JPRB, 0.11123E+01_JPRB, 0.11550E+01_JPRB, 0.11783E+01_JPRB ,&
&  0.12122E+01_JPRB, 0.12569E+01_JPRB, 0.12968E+01_JPRB, 0.13755E+01_JPRB, 0.15127E+01_JPRB, 0.17144E+01_JPRB, 0.19383E+01_JPRB ,&
&  0.21809E+01_JPRB, 0.24008E+01_JPRB, 0.26234E+01_JPRB, 0.28188E+01_JPRB, 0.30017E+01_JPRB, 0.31226E+01_JPRB, 0.31814E+01_JPRB ,&
&  0.32499E+01_JPRB, 0.32335E+01_JPRB, 0.32888E+01_JPRB, 0.32370E+01_JPRB, 0.31812E+01_JPRB, 0.31010E+01_JPRB, 0.28960E+01_JPRB ,&
&  0.26517E+01_JPRB, 0.24865E+01_JPRB, 0.23276E+01_JPRB, 0.21269E+01_JPRB, 0.19730E+01_JPRB, 0.18620E+01_JPRB, 0.18061E+01_JPRB ,&
&  0.17571E+01_JPRB, 0.17191E+01_JPRB, 0.16349E+01_JPRB, 0.15428E+01_JPRB, 0.14781E+01_JPRB, 0.14296E+01_JPRB, 0.13726E+01_JPRB ,&
&  0.13209E+01_JPRB, 0.12731E+01_JPRB, 0.12370E+01_JPRB, 0.12060E+01_JPRB, 0.11804E+01_JPRB, 0.11647E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11576E+01_JPRB, 0.11656E+01_JPRB, 0.11848E+01_JPRB, 0.12162E+01_JPRB, 0.12525E+01_JPRB /)
ZBURD(:,  8,  6) = (/ &
&  0.12361E+01_JPRB, 0.12441E+01_JPRB, 0.12381E+01_JPRB, 0.12209E+01_JPRB, 0.11915E+01_JPRB, 0.11598E+01_JPRB, 0.11314E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.10824E+01_JPRB, 0.10595E+01_JPRB, 0.10410E+01_JPRB, 0.10232E+01_JPRB, 0.10094E+01_JPRB, 0.99867E+00_JPRB ,&
&  0.99082E+00_JPRB, 0.95350E+00_JPRB, 0.94027E+00_JPRB, 0.93917E+00_JPRB, 0.94154E+00_JPRB, 0.93866E+00_JPRB, 0.93277E+00_JPRB ,&
&  0.90225E+00_JPRB, 0.85871E+00_JPRB, 0.81757E+00_JPRB, 0.78353E+00_JPRB, 0.75295E+00_JPRB, 0.73023E+00_JPRB, 0.72354E+00_JPRB ,&
&  0.71819E+00_JPRB, 0.69949E+00_JPRB, 0.69334E+00_JPRB, 0.70703E+00_JPRB, 0.70124E+00_JPRB, 0.62553E+00_JPRB, 0.53048E+00_JPRB ,&
&  0.51657E+00_JPRB, 0.49555E+00_JPRB, 0.49694E+00_JPRB, 0.50527E+00_JPRB, 0.52791E+00_JPRB, 0.56776E+00_JPRB, 0.62954E+00_JPRB ,&
&  0.69702E+00_JPRB, 0.83912E+00_JPRB, 0.92776E+00_JPRB, 0.99173E+00_JPRB, 0.10488E+01_JPRB, 0.10887E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.11168E+01_JPRB, 0.11550E+01_JPRB, 0.12174E+01_JPRB, 0.12985E+01_JPRB, 0.14291E+01_JPRB, 0.16785E+01_JPRB, 0.19465E+01_JPRB ,&
&  0.23116E+01_JPRB, 0.26121E+01_JPRB, 0.27686E+01_JPRB, 0.28636E+01_JPRB, 0.30719E+01_JPRB, 0.33354E+01_JPRB, 0.34691E+01_JPRB ,&
&  0.36824E+01_JPRB, 0.39118E+01_JPRB, 0.38756E+01_JPRB, 0.36612E+01_JPRB, 0.35257E+01_JPRB, 0.34405E+01_JPRB, 0.31526E+01_JPRB ,&
&  0.28214E+01_JPRB, 0.26773E+01_JPRB, 0.26612E+01_JPRB, 0.23603E+01_JPRB, 0.21396E+01_JPRB, 0.19335E+01_JPRB, 0.17978E+01_JPRB ,&
&  0.17019E+01_JPRB, 0.16227E+01_JPRB, 0.15226E+01_JPRB, 0.14440E+01_JPRB, 0.13967E+01_JPRB, 0.13603E+01_JPRB, 0.13271E+01_JPRB ,&
&  0.12879E+01_JPRB, 0.12518E+01_JPRB, 0.12151E+01_JPRB, 0.11864E+01_JPRB, 0.11563E+01_JPRB, 0.11331E+01_JPRB, 0.11179E+01_JPRB ,&
&  0.11122E+01_JPRB, 0.11200E+01_JPRB, 0.11410E+01_JPRB, 0.11743E+01_JPRB, 0.12137E+01_JPRB /)
ZBURD(:,  9,  6) = (/ &
&  0.12109E+01_JPRB, 0.12093E+01_JPRB, 0.11781E+01_JPRB, 0.11338E+01_JPRB, 0.10825E+01_JPRB, 0.10211E+01_JPRB, 0.97868E+00_JPRB ,&
&  0.95407E+00_JPRB, 0.94370E+00_JPRB, 0.93196E+00_JPRB, 0.91987E+00_JPRB, 0.91257E+00_JPRB, 0.90885E+00_JPRB, 0.90866E+00_JPRB ,&
&  0.91126E+00_JPRB, 0.91377E+00_JPRB, 0.91405E+00_JPRB, 0.90079E+00_JPRB, 0.90234E+00_JPRB, 0.90858E+00_JPRB, 0.91790E+00_JPRB ,&
&  0.91265E+00_JPRB, 0.87767E+00_JPRB, 0.83244E+00_JPRB, 0.79920E+00_JPRB, 0.75992E+00_JPRB, 0.73269E+00_JPRB, 0.70370E+00_JPRB ,&
&  0.67206E+00_JPRB, 0.65686E+00_JPRB, 0.67087E+00_JPRB, 0.69621E+00_JPRB, 0.68289E+00_JPRB, 0.69178E+00_JPRB, 0.56511E+00_JPRB ,&
&  0.52167E+00_JPRB, 0.49622E+00_JPRB, 0.49170E+00_JPRB, 0.50852E+00_JPRB, 0.55327E+00_JPRB, 0.60783E+00_JPRB, 0.66285E+00_JPRB ,&
&  0.72090E+00_JPRB, 0.80832E+00_JPRB, 0.86030E+00_JPRB, 0.90049E+00_JPRB, 0.94789E+00_JPRB, 0.99342E+00_JPRB, 0.10267E+01_JPRB ,&
&  0.10559E+01_JPRB, 0.11162E+01_JPRB, 0.12057E+01_JPRB, 0.13390E+01_JPRB, 0.15971E+01_JPRB, 0.16606E+01_JPRB, 0.18369E+01_JPRB ,&
&  0.25954E+01_JPRB, 0.30226E+01_JPRB, 0.30271E+01_JPRB, 0.30401E+01_JPRB, 0.31224E+01_JPRB, 0.33462E+01_JPRB, 0.36881E+01_JPRB ,&
&  0.41454E+01_JPRB, 0.44982E+01_JPRB, 0.44734E+01_JPRB, 0.39950E+01_JPRB, 0.37547E+01_JPRB, 0.36152E+01_JPRB, 0.32196E+01_JPRB ,&
&  0.27692E+01_JPRB, 0.27636E+01_JPRB, 0.28399E+01_JPRB, 0.23422E+01_JPRB, 0.20196E+01_JPRB, 0.18248E+01_JPRB, 0.17195E+01_JPRB ,&
&  0.16387E+01_JPRB, 0.15669E+01_JPRB, 0.14947E+01_JPRB, 0.14322E+01_JPRB, 0.13654E+01_JPRB, 0.12983E+01_JPRB, 0.12381E+01_JPRB ,&
&  0.11918E+01_JPRB, 0.11543E+01_JPRB, 0.11331E+01_JPRB, 0.11196E+01_JPRB, 0.11023E+01_JPRB, 0.10826E+01_JPRB, 0.10756E+01_JPRB ,&
&  0.10792E+01_JPRB, 0.10779E+01_JPRB, 0.10966E+01_JPRB, 0.11298E+01_JPRB, 0.11753E+01_JPRB /)
ZBURD(:, 10,  6) = (/ &
&  0.11237E+01_JPRB, 0.10980E+01_JPRB, 0.10539E+01_JPRB, 0.99747E+00_JPRB, 0.94105E+00_JPRB, 0.87392E+00_JPRB, 0.82425E+00_JPRB ,&
&  0.79729E+00_JPRB, 0.78400E+00_JPRB, 0.77447E+00_JPRB, 0.76925E+00_JPRB, 0.76876E+00_JPRB, 0.77417E+00_JPRB, 0.77434E+00_JPRB ,&
&  0.77711E+00_JPRB, 0.78177E+00_JPRB, 0.78979E+00_JPRB, 0.80431E+00_JPRB, 0.81839E+00_JPRB, 0.83346E+00_JPRB, 0.85001E+00_JPRB ,&
&  0.86582E+00_JPRB, 0.85573E+00_JPRB, 0.82115E+00_JPRB, 0.79273E+00_JPRB, 0.78248E+00_JPRB, 0.78107E+00_JPRB, 0.74335E+00_JPRB ,&
&  0.68905E+00_JPRB, 0.65999E+00_JPRB, 0.64937E+00_JPRB, 0.70317E+00_JPRB, 0.73929E+00_JPRB, 0.74432E+00_JPRB, 0.61835E+00_JPRB ,&
&  0.54768E+00_JPRB, 0.51710E+00_JPRB, 0.52067E+00_JPRB, 0.55936E+00_JPRB, 0.63050E+00_JPRB, 0.71087E+00_JPRB, 0.79662E+00_JPRB ,&
&  0.84272E+00_JPRB, 0.88967E+00_JPRB, 0.90482E+00_JPRB, 0.93318E+00_JPRB, 0.98327E+00_JPRB, 0.10357E+01_JPRB, 0.10943E+01_JPRB ,&
&  0.11420E+01_JPRB, 0.12150E+01_JPRB, 0.13760E+01_JPRB, 0.13212E+01_JPRB, 0.14108E+01_JPRB, 0.15677E+01_JPRB, 0.18464E+01_JPRB ,&
&  0.24107E+01_JPRB, 0.27209E+01_JPRB, 0.29140E+01_JPRB, 0.32560E+01_JPRB, 0.32594E+01_JPRB, 0.33227E+01_JPRB, 0.35893E+01_JPRB ,&
&  0.42549E+01_JPRB, 0.47991E+01_JPRB, 0.46712E+01_JPRB, 0.41920E+01_JPRB, 0.40025E+01_JPRB, 0.38862E+01_JPRB, 0.33944E+01_JPRB ,&
&  0.27733E+01_JPRB, 0.24139E+01_JPRB, 0.21056E+01_JPRB, 0.17803E+01_JPRB, 0.16434E+01_JPRB, 0.15649E+01_JPRB, 0.15112E+01_JPRB ,&
&  0.14680E+01_JPRB, 0.14219E+01_JPRB, 0.13846E+01_JPRB, 0.12752E+01_JPRB, 0.11733E+01_JPRB, 0.10553E+01_JPRB, 0.97023E+00_JPRB ,&
&  0.91535E+00_JPRB, 0.89600E+00_JPRB, 0.90266E+00_JPRB, 0.92919E+00_JPRB, 0.99228E+00_JPRB, 0.10402E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.10617E+01_JPRB, 0.10334E+01_JPRB, 0.10280E+01_JPRB, 0.10469E+01_JPRB, 0.11023E+01_JPRB /)
ZBURD(:, 11,  6) = (/ &
&  0.10203E+01_JPRB, 0.94415E+00_JPRB, 0.89798E+00_JPRB, 0.88901E+00_JPRB, 0.82827E+00_JPRB, 0.77254E+00_JPRB, 0.73737E+00_JPRB ,&
&  0.73513E+00_JPRB, 0.73535E+00_JPRB, 0.73648E+00_JPRB, 0.70784E+00_JPRB, 0.68243E+00_JPRB, 0.66970E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.68079E+00_JPRB, 0.69984E+00_JPRB, 0.71673E+00_JPRB, 0.71688E+00_JPRB, 0.71734E+00_JPRB, 0.72825E+00_JPRB, 0.75525E+00_JPRB ,&
&  0.79274E+00_JPRB, 0.81281E+00_JPRB, 0.80903E+00_JPRB, 0.80233E+00_JPRB, 0.81303E+00_JPRB, 0.81877E+00_JPRB, 0.77197E+00_JPRB ,&
&  0.72769E+00_JPRB, 0.68929E+00_JPRB, 0.67641E+00_JPRB, 0.71190E+00_JPRB, 0.77781E+00_JPRB, 0.79435E+00_JPRB, 0.66842E+00_JPRB ,&
&  0.58076E+00_JPRB, 0.56302E+00_JPRB, 0.61340E+00_JPRB, 0.74993E+00_JPRB, 0.81499E+00_JPRB, 0.87211E+00_JPRB, 0.94308E+00_JPRB ,&
&  0.90134E+00_JPRB, 0.89989E+00_JPRB, 0.93536E+00_JPRB, 0.10891E+01_JPRB, 0.11457E+01_JPRB, 0.11842E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.12511E+01_JPRB, 0.13494E+01_JPRB, 0.15671E+01_JPRB, 0.14525E+01_JPRB, 0.16921E+01_JPRB, 0.21023E+01_JPRB, 0.22089E+01_JPRB ,&
&  0.25348E+01_JPRB, 0.27590E+01_JPRB, 0.32523E+01_JPRB, 0.33547E+01_JPRB, 0.32793E+01_JPRB, 0.33857E+01_JPRB, 0.38095E+01_JPRB ,&
&  0.46226E+01_JPRB, 0.52357E+01_JPRB, 0.50697E+01_JPRB, 0.47925E+01_JPRB, 0.45653E+01_JPRB, 0.42813E+01_JPRB, 0.37283E+01_JPRB ,&
&  0.29791E+01_JPRB, 0.24130E+01_JPRB, 0.19916E+01_JPRB, 0.16408E+01_JPRB, 0.14777E+01_JPRB, 0.14123E+01_JPRB, 0.13343E+01_JPRB ,&
&  0.13067E+01_JPRB, 0.12757E+01_JPRB, 0.12415E+01_JPRB, 0.11290E+01_JPRB, 0.10247E+01_JPRB, 0.92023E+00_JPRB, 0.84583E+00_JPRB ,&
&  0.80757E+00_JPRB, 0.76872E+00_JPRB, 0.74673E+00_JPRB, 0.77084E+00_JPRB, 0.85899E+00_JPRB, 0.96632E+00_JPRB, 0.10176E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.96785E+00_JPRB, 0.99970E+00_JPRB, 0.10567E+01_JPRB, 0.10716E+01_JPRB /)
ZBURD(:, 12,  6) = (/ &
&  0.10002E+01_JPRB, 0.96784E+00_JPRB, 0.94876E+00_JPRB, 0.93036E+00_JPRB, 0.89810E+00_JPRB, 0.82907E+00_JPRB, 0.78101E+00_JPRB ,&
&  0.78493E+00_JPRB, 0.76426E+00_JPRB, 0.74996E+00_JPRB, 0.71507E+00_JPRB, 0.68177E+00_JPRB, 0.64632E+00_JPRB, 0.62830E+00_JPRB ,&
&  0.61380E+00_JPRB, 0.63768E+00_JPRB, 0.68233E+00_JPRB, 0.69460E+00_JPRB, 0.69574E+00_JPRB, 0.69918E+00_JPRB, 0.72634E+00_JPRB ,&
&  0.76294E+00_JPRB, 0.80020E+00_JPRB, 0.82451E+00_JPRB, 0.85038E+00_JPRB, 0.86711E+00_JPRB, 0.87368E+00_JPRB, 0.84950E+00_JPRB ,&
&  0.81968E+00_JPRB, 0.77669E+00_JPRB, 0.74199E+00_JPRB, 0.75933E+00_JPRB, 0.81655E+00_JPRB, 0.86866E+00_JPRB, 0.82356E+00_JPRB ,&
&  0.65542E+00_JPRB, 0.66267E+00_JPRB, 0.84660E+00_JPRB, 0.92257E+00_JPRB, 0.99106E+00_JPRB, 0.10313E+01_JPRB, 0.10820E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11781E+01_JPRB, 0.12177E+01_JPRB, 0.12472E+01_JPRB, 0.12786E+01_JPRB, 0.13249E+01_JPRB, 0.13562E+01_JPRB ,&
&  0.14906E+01_JPRB, 0.12660E+01_JPRB, 0.12710E+01_JPRB, 0.14865E+01_JPRB, 0.20851E+01_JPRB, 0.24166E+01_JPRB, 0.25145E+01_JPRB ,&
&  0.27737E+01_JPRB, 0.29085E+01_JPRB, 0.32054E+01_JPRB, 0.36123E+01_JPRB, 0.38437E+01_JPRB, 0.41964E+01_JPRB, 0.46724E+01_JPRB ,&
&  0.52813E+01_JPRB, 0.58419E+01_JPRB, 0.58799E+01_JPRB, 0.56550E+01_JPRB, 0.52295E+01_JPRB, 0.47283E+01_JPRB, 0.39579E+01_JPRB ,&
&  0.30627E+01_JPRB, 0.24313E+01_JPRB, 0.19135E+01_JPRB, 0.15630E+01_JPRB, 0.13641E+01_JPRB, 0.12704E+01_JPRB, 0.11981E+01_JPRB ,&
&  0.11882E+01_JPRB, 0.11639E+01_JPRB, 0.11157E+01_JPRB, 0.10219E+01_JPRB, 0.95205E+00_JPRB, 0.92826E+00_JPRB, 0.91601E+00_JPRB ,&
&  0.88969E+00_JPRB, 0.86638E+00_JPRB, 0.79589E+00_JPRB, 0.80819E+00_JPRB, 0.89258E+00_JPRB, 0.98836E+00_JPRB, 0.10255E+01_JPRB ,&
&  0.10428E+01_JPRB, 0.10170E+01_JPRB, 0.10173E+01_JPRB, 0.10420E+01_JPRB, 0.10243E+01_JPRB /)
ZBURD(:, 13,  6) = (/ &
&  0.98633E+00_JPRB, 0.98853E+00_JPRB, 0.97695E+00_JPRB, 0.96558E+00_JPRB, 0.94001E+00_JPRB, 0.85343E+00_JPRB, 0.81979E+00_JPRB ,&
&  0.88486E+00_JPRB, 0.83076E+00_JPRB, 0.88832E+00_JPRB, 0.77798E+00_JPRB, 0.75024E+00_JPRB, 0.71259E+00_JPRB, 0.68584E+00_JPRB ,&
&  0.65377E+00_JPRB, 0.66035E+00_JPRB, 0.69648E+00_JPRB, 0.72849E+00_JPRB, 0.73478E+00_JPRB, 0.71895E+00_JPRB, 0.72662E+00_JPRB ,&
&  0.74957E+00_JPRB, 0.79351E+00_JPRB, 0.85842E+00_JPRB, 0.89789E+00_JPRB, 0.92171E+00_JPRB, 0.94065E+00_JPRB, 0.94224E+00_JPRB ,&
&  0.87956E+00_JPRB, 0.85560E+00_JPRB, 0.83547E+00_JPRB, 0.83533E+00_JPRB, 0.86806E+00_JPRB, 0.92713E+00_JPRB, 0.95979E+00_JPRB ,&
&  0.96340E+00_JPRB, 0.84377E+00_JPRB, 0.96668E+00_JPRB, 0.10107E+01_JPRB, 0.10843E+01_JPRB, 0.11292E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.12465E+01_JPRB, 0.12962E+01_JPRB, 0.13467E+01_JPRB, 0.14062E+01_JPRB, 0.15000E+01_JPRB, 0.15671E+01_JPRB ,&
&  0.17745E+01_JPRB, 0.13805E+01_JPRB, 0.15684E+01_JPRB, 0.18342E+01_JPRB, 0.25254E+01_JPRB, 0.28855E+01_JPRB, 0.30484E+01_JPRB ,&
&  0.37098E+01_JPRB, 0.36852E+01_JPRB, 0.40948E+01_JPRB, 0.44822E+01_JPRB, 0.49910E+01_JPRB, 0.56282E+01_JPRB, 0.60513E+01_JPRB ,&
&  0.61819E+01_JPRB, 0.62661E+01_JPRB, 0.66176E+01_JPRB, 0.60403E+01_JPRB, 0.52596E+01_JPRB, 0.44885E+01_JPRB, 0.35563E+01_JPRB ,&
&  0.29395E+01_JPRB, 0.23964E+01_JPRB, 0.19362E+01_JPRB, 0.16033E+01_JPRB, 0.13664E+01_JPRB, 0.12164E+01_JPRB, 0.11305E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.10334E+01_JPRB, 0.97499E+00_JPRB, 0.91277E+00_JPRB, 0.87823E+00_JPRB, 0.86289E+00_JPRB, 0.88324E+00_JPRB ,&
&  0.87401E+00_JPRB, 0.87269E+00_JPRB, 0.92008E+00_JPRB, 0.10306E+01_JPRB, 0.10520E+01_JPRB, 0.10802E+01_JPRB, 0.12007E+01_JPRB ,&
&  0.10830E+01_JPRB, 0.10964E+01_JPRB, 0.11445E+01_JPRB, 0.10851E+01_JPRB, 0.10100E+01_JPRB /)
ZBURD(:, 14,  6) = (/ &
&  0.10514E+01_JPRB, 0.10423E+01_JPRB, 0.10141E+01_JPRB, 0.99733E+00_JPRB, 0.98577E+00_JPRB, 0.93648E+00_JPRB, 0.89107E+00_JPRB ,&
&  0.95015E+00_JPRB, 0.94321E+00_JPRB, 0.94797E+00_JPRB, 0.93341E+00_JPRB, 0.92027E+00_JPRB, 0.84850E+00_JPRB, 0.79904E+00_JPRB ,&
&  0.75971E+00_JPRB, 0.76867E+00_JPRB, 0.83131E+00_JPRB, 0.85594E+00_JPRB, 0.85525E+00_JPRB, 0.82677E+00_JPRB, 0.81361E+00_JPRB ,&
&  0.80260E+00_JPRB, 0.81261E+00_JPRB, 0.86714E+00_JPRB, 0.92342E+00_JPRB, 0.93031E+00_JPRB, 0.93490E+00_JPRB, 0.96580E+00_JPRB ,&
&  0.91636E+00_JPRB, 0.90825E+00_JPRB, 0.89816E+00_JPRB, 0.82999E+00_JPRB, 0.93563E+00_JPRB, 0.97780E+00_JPRB, 0.10029E+01_JPRB ,&
&  0.10450E+01_JPRB, 0.10822E+01_JPRB, 0.11087E+01_JPRB, 0.11271E+01_JPRB, 0.11640E+01_JPRB, 0.11907E+01_JPRB, 0.12302E+01_JPRB ,&
&  0.12647E+01_JPRB, 0.13076E+01_JPRB, 0.13595E+01_JPRB, 0.14290E+01_JPRB, 0.15538E+01_JPRB, 0.16569E+01_JPRB, 0.19572E+01_JPRB ,&
&  0.21470E+01_JPRB, 0.22394E+01_JPRB, 0.25077E+01_JPRB, 0.24317E+01_JPRB, 0.31875E+01_JPRB, 0.36627E+01_JPRB, 0.35450E+01_JPRB ,&
&  0.38681E+01_JPRB, 0.39472E+01_JPRB, 0.48453E+01_JPRB, 0.56707E+01_JPRB, 0.64537E+01_JPRB, 0.71504E+01_JPRB, 0.74403E+01_JPRB ,&
&  0.73335E+01_JPRB, 0.75130E+01_JPRB, 0.73528E+01_JPRB, 0.65622E+01_JPRB, 0.55179E+01_JPRB, 0.47835E+01_JPRB, 0.37898E+01_JPRB ,&
&  0.29730E+01_JPRB, 0.23279E+01_JPRB, 0.19343E+01_JPRB, 0.17355E+01_JPRB, 0.13804E+01_JPRB, 0.12597E+01_JPRB, 0.11356E+01_JPRB ,&
&  0.10260E+01_JPRB, 0.93922E+00_JPRB, 0.89258E+00_JPRB, 0.85581E+00_JPRB, 0.83877E+00_JPRB, 0.83777E+00_JPRB, 0.83970E+00_JPRB ,&
&  0.90497E+00_JPRB, 0.10467E+01_JPRB, 0.13179E+01_JPRB, 0.13623E+01_JPRB, 0.13402E+01_JPRB, 0.13227E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.11266E+01_JPRB, 0.12585E+01_JPRB, 0.12159E+01_JPRB, 0.11306E+01_JPRB, 0.10597E+01_JPRB /)
ZBURD(:, 15,  6) = (/ &
&  0.11125E+01_JPRB, 0.10909E+01_JPRB, 0.10708E+01_JPRB, 0.10477E+01_JPRB, 0.10262E+01_JPRB, 0.10004E+01_JPRB, 0.97725E+00_JPRB ,&
&  0.98187E+00_JPRB, 0.97217E+00_JPRB, 0.96848E+00_JPRB, 0.95953E+00_JPRB, 0.94293E+00_JPRB, 0.95957E+00_JPRB, 0.89154E+00_JPRB ,&
&  0.84685E+00_JPRB, 0.90963E+00_JPRB, 0.91418E+00_JPRB, 0.10021E+01_JPRB, 0.10976E+01_JPRB, 0.10543E+01_JPRB, 0.10320E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.93956E+00_JPRB, 0.92446E+00_JPRB, 0.93850E+00_JPRB, 0.94900E+00_JPRB, 0.97050E+00_JPRB, 0.10083E+01_JPRB ,&
&  0.93359E+00_JPRB, 0.86127E+00_JPRB, 0.81599E+00_JPRB, 0.81883E+00_JPRB, 0.89371E+00_JPRB, 0.10145E+01_JPRB, 0.10868E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.11224E+01_JPRB, 0.11796E+01_JPRB, 0.12239E+01_JPRB, 0.12611E+01_JPRB, 0.13124E+01_JPRB, 0.13671E+01_JPRB ,&
&  0.14111E+01_JPRB, 0.14587E+01_JPRB, 0.15128E+01_JPRB, 0.16010E+01_JPRB, 0.16298E+01_JPRB, 0.19159E+01_JPRB, 0.25204E+01_JPRB ,&
&  0.28182E+01_JPRB, 0.28923E+01_JPRB, 0.30745E+01_JPRB, 0.34702E+01_JPRB, 0.40084E+01_JPRB, 0.40429E+01_JPRB, 0.41532E+01_JPRB ,&
&  0.46544E+01_JPRB, 0.49808E+01_JPRB, 0.63443E+01_JPRB, 0.70934E+01_JPRB, 0.76074E+01_JPRB, 0.82677E+01_JPRB, 0.82245E+01_JPRB ,&
&  0.81064E+01_JPRB, 0.77534E+01_JPRB, 0.75551E+01_JPRB, 0.68160E+01_JPRB, 0.57136E+01_JPRB, 0.49742E+01_JPRB, 0.38207E+01_JPRB ,&
&  0.30572E+01_JPRB, 0.23899E+01_JPRB, 0.17437E+01_JPRB, 0.14812E+01_JPRB, 0.12232E+01_JPRB, 0.11862E+01_JPRB, 0.10863E+01_JPRB ,&
&  0.98273E+00_JPRB, 0.94165E+00_JPRB, 0.90608E+00_JPRB, 0.93928E+00_JPRB, 0.94226E+00_JPRB, 0.87710E+00_JPRB, 0.93543E+00_JPRB ,&
&  0.10408E+01_JPRB, 0.13662E+01_JPRB, 0.14727E+01_JPRB, 0.15916E+01_JPRB, 0.15294E+01_JPRB, 0.15034E+01_JPRB, 0.12531E+01_JPRB ,&
&  0.12624E+01_JPRB, 0.13943E+01_JPRB, 0.12992E+01_JPRB, 0.12157E+01_JPRB, 0.11541E+01_JPRB /)
ZBURD(:, 16,  6) = (/ &
&  0.11678E+01_JPRB, 0.11317E+01_JPRB, 0.11164E+01_JPRB, 0.11116E+01_JPRB, 0.11035E+01_JPRB, 0.10702E+01_JPRB, 0.10673E+01_JPRB ,&
&  0.10517E+01_JPRB, 0.10226E+01_JPRB, 0.10071E+01_JPRB, 0.99610E+00_JPRB, 0.97991E+00_JPRB, 0.99786E+00_JPRB, 0.10559E+01_JPRB ,&
&  0.98931E+00_JPRB, 0.11352E+01_JPRB, 0.10026E+01_JPRB, 0.10609E+01_JPRB, 0.14579E+01_JPRB, 0.13153E+01_JPRB, 0.13003E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.11646E+01_JPRB, 0.10579E+01_JPRB, 0.10172E+01_JPRB, 0.10211E+01_JPRB, 0.10732E+01_JPRB, 0.10973E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.89362E+00_JPRB, 0.85770E+00_JPRB, 0.85706E+00_JPRB, 0.91585E+00_JPRB, 0.10132E+01_JPRB, 0.11494E+01_JPRB ,&
&  0.11907E+01_JPRB, 0.12277E+01_JPRB, 0.12935E+01_JPRB, 0.13748E+01_JPRB, 0.14570E+01_JPRB, 0.15481E+01_JPRB, 0.16043E+01_JPRB ,&
&  0.16389E+01_JPRB, 0.16685E+01_JPRB, 0.17093E+01_JPRB, 0.18330E+01_JPRB, 0.18297E+01_JPRB, 0.22559E+01_JPRB, 0.35791E+01_JPRB ,&
&  0.43446E+01_JPRB, 0.39545E+01_JPRB, 0.43991E+01_JPRB, 0.58554E+01_JPRB, 0.62696E+01_JPRB, 0.61180E+01_JPRB, 0.60131E+01_JPRB ,&
&  0.64021E+01_JPRB, 0.68956E+01_JPRB, 0.74560E+01_JPRB, 0.82191E+01_JPRB, 0.87408E+01_JPRB, 0.90928E+01_JPRB, 0.87539E+01_JPRB ,&
&  0.81574E+01_JPRB, 0.77714E+01_JPRB, 0.73468E+01_JPRB, 0.66687E+01_JPRB, 0.54363E+01_JPRB, 0.46404E+01_JPRB, 0.35892E+01_JPRB ,&
&  0.29467E+01_JPRB, 0.21713E+01_JPRB, 0.13947E+01_JPRB, 0.11250E+01_JPRB, 0.10050E+01_JPRB, 0.11403E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.11213E+01_JPRB, 0.11340E+01_JPRB, 0.11372E+01_JPRB, 0.12092E+01_JPRB, 0.12752E+01_JPRB, 0.12905E+01_JPRB, 0.11213E+01_JPRB ,&
&  0.11253E+01_JPRB, 0.13637E+01_JPRB, 0.17420E+01_JPRB, 0.18699E+01_JPRB, 0.17289E+01_JPRB, 0.16967E+01_JPRB, 0.15932E+01_JPRB ,&
&  0.16480E+01_JPRB, 0.15213E+01_JPRB, 0.13940E+01_JPRB, 0.12867E+01_JPRB, 0.12363E+01_JPRB /)
ZBURD(:, 17,  6) = (/ &
&  0.11817E+01_JPRB, 0.11727E+01_JPRB, 0.11817E+01_JPRB, 0.11957E+01_JPRB, 0.12047E+01_JPRB, 0.11568E+01_JPRB, 0.11420E+01_JPRB ,&
&  0.11179E+01_JPRB, 0.10836E+01_JPRB, 0.10530E+01_JPRB, 0.10395E+01_JPRB, 0.10273E+01_JPRB, 0.10258E+01_JPRB, 0.10544E+01_JPRB ,&
&  0.11786E+01_JPRB, 0.14738E+01_JPRB, 0.13210E+01_JPRB, 0.10571E+01_JPRB, 0.12083E+01_JPRB, 0.11913E+01_JPRB, 0.12738E+01_JPRB ,&
&  0.12663E+01_JPRB, 0.13355E+01_JPRB, 0.12908E+01_JPRB, 0.12348E+01_JPRB, 0.12993E+01_JPRB, 0.13981E+01_JPRB, 0.14316E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.12200E+01_JPRB, 0.11433E+01_JPRB, 0.11476E+01_JPRB, 0.12576E+01_JPRB, 0.11642E+01_JPRB, 0.13080E+01_JPRB ,&
&  0.13888E+01_JPRB, 0.14669E+01_JPRB, 0.15514E+01_JPRB, 0.16538E+01_JPRB, 0.17380E+01_JPRB, 0.18156E+01_JPRB, 0.18547E+01_JPRB ,&
&  0.18766E+01_JPRB, 0.18694E+01_JPRB, 0.18257E+01_JPRB, 0.19048E+01_JPRB, 0.20701E+01_JPRB, 0.24514E+01_JPRB, 0.34859E+01_JPRB ,&
&  0.42905E+01_JPRB, 0.44742E+01_JPRB, 0.55884E+01_JPRB, 0.72680E+01_JPRB, 0.75132E+01_JPRB, 0.73702E+01_JPRB, 0.77753E+01_JPRB ,&
&  0.84346E+01_JPRB, 0.90761E+01_JPRB, 0.97618E+01_JPRB, 0.10044E+02_JPRB, 0.10046E+02_JPRB, 0.93259E+01_JPRB, 0.84871E+01_JPRB ,&
&  0.76545E+01_JPRB, 0.73528E+01_JPRB, 0.71212E+01_JPRB, 0.61105E+01_JPRB, 0.49355E+01_JPRB, 0.40316E+01_JPRB, 0.35868E+01_JPRB ,&
&  0.34436E+01_JPRB, 0.15783E+01_JPRB, 0.10082E+01_JPRB, 0.82542E+00_JPRB, 0.78286E+00_JPRB, 0.88331E+00_JPRB, 0.11978E+01_JPRB ,&
&  0.10277E+01_JPRB, 0.12009E+01_JPRB, 0.15566E+01_JPRB, 0.16751E+01_JPRB, 0.19312E+01_JPRB, 0.22972E+01_JPRB, 0.18245E+01_JPRB ,&
&  0.15547E+01_JPRB, 0.16649E+01_JPRB, 0.21283E+01_JPRB, 0.21918E+01_JPRB, 0.19774E+01_JPRB, 0.19111E+01_JPRB, 0.19099E+01_JPRB ,&
&  0.17754E+01_JPRB, 0.16894E+01_JPRB, 0.15015E+01_JPRB, 0.13127E+01_JPRB, 0.12289E+01_JPRB /)
ZBURD(:, 18,  6) = (/ &
&  0.12785E+01_JPRB, 0.12660E+01_JPRB, 0.12893E+01_JPRB, 0.13087E+01_JPRB, 0.12970E+01_JPRB, 0.12498E+01_JPRB, 0.12291E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.11355E+01_JPRB, 0.10932E+01_JPRB, 0.10723E+01_JPRB, 0.10582E+01_JPRB, 0.10566E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10656E+01_JPRB, 0.13619E+01_JPRB, 0.11307E+01_JPRB, 0.98730E+00_JPRB, 0.85800E+00_JPRB, 0.95003E+00_JPRB, 0.10890E+01_JPRB ,&
&  0.13138E+01_JPRB, 0.14115E+01_JPRB, 0.15786E+01_JPRB, 0.17932E+01_JPRB, 0.19918E+01_JPRB, 0.19500E+01_JPRB, 0.20962E+01_JPRB ,&
&  0.20068E+01_JPRB, 0.18914E+01_JPRB, 0.16157E+01_JPRB, 0.17696E+01_JPRB, 0.17260E+01_JPRB, 0.15417E+01_JPRB, 0.17843E+01_JPRB ,&
&  0.18545E+01_JPRB, 0.19077E+01_JPRB, 0.19511E+01_JPRB, 0.19811E+01_JPRB, 0.19911E+01_JPRB, 0.19439E+01_JPRB, 0.18797E+01_JPRB ,&
&  0.17989E+01_JPRB, 0.17755E+01_JPRB, 0.18138E+01_JPRB, 0.19153E+01_JPRB, 0.23115E+01_JPRB, 0.27828E+01_JPRB, 0.37047E+01_JPRB ,&
&  0.46951E+01_JPRB, 0.47079E+01_JPRB, 0.50812E+01_JPRB, 0.72405E+01_JPRB, 0.95398E+01_JPRB, 0.95430E+01_JPRB, 0.97502E+01_JPRB ,&
&  0.10450E+02_JPRB, 0.10658E+02_JPRB, 0.10520E+02_JPRB, 0.99657E+01_JPRB, 0.94337E+01_JPRB, 0.87408E+01_JPRB, 0.78993E+01_JPRB ,&
&  0.71457E+01_JPRB, 0.69076E+01_JPRB, 0.67239E+01_JPRB, 0.60182E+01_JPRB, 0.48612E+01_JPRB, 0.42788E+01_JPRB, 0.38258E+01_JPRB ,&
&  0.33847E+01_JPRB, 0.20453E+01_JPRB, 0.11710E+01_JPRB, 0.91381E+00_JPRB, 0.84327E+00_JPRB, 0.88719E+00_JPRB, 0.11406E+01_JPRB ,&
&  0.11539E+01_JPRB, 0.14754E+01_JPRB, 0.19799E+01_JPRB, 0.21771E+01_JPRB, 0.31348E+01_JPRB, 0.36542E+01_JPRB, 0.33868E+01_JPRB ,&
&  0.27327E+01_JPRB, 0.22384E+01_JPRB, 0.26281E+01_JPRB, 0.25386E+01_JPRB, 0.23225E+01_JPRB, 0.21189E+01_JPRB, 0.20660E+01_JPRB ,&
&  0.19301E+01_JPRB, 0.18304E+01_JPRB, 0.16970E+01_JPRB, 0.14730E+01_JPRB, 0.13342E+01_JPRB /)
ZBURD(:, 19,  6) = (/ &
&  0.14699E+01_JPRB, 0.14101E+01_JPRB, 0.13816E+01_JPRB, 0.13658E+01_JPRB, 0.12981E+01_JPRB, 0.12486E+01_JPRB, 0.12495E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.11767E+01_JPRB, 0.11331E+01_JPRB, 0.11185E+01_JPRB, 0.11098E+01_JPRB, 0.11160E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.11531E+01_JPRB, 0.94482E+00_JPRB, 0.79947E+00_JPRB, 0.76191E+00_JPRB, 0.91617E+00_JPRB, 0.11479E+01_JPRB ,&
&  0.14720E+01_JPRB, 0.18034E+01_JPRB, 0.23051E+01_JPRB, 0.26672E+01_JPRB, 0.29476E+01_JPRB, 0.30915E+01_JPRB, 0.32550E+01_JPRB ,&
&  0.35124E+01_JPRB, 0.27797E+01_JPRB, 0.26994E+01_JPRB, 0.26084E+01_JPRB, 0.25299E+01_JPRB, 0.24165E+01_JPRB, 0.23563E+01_JPRB ,&
&  0.22528E+01_JPRB, 0.21732E+01_JPRB, 0.21011E+01_JPRB, 0.20209E+01_JPRB, 0.19443E+01_JPRB, 0.18775E+01_JPRB, 0.18284E+01_JPRB ,&
&  0.18354E+01_JPRB, 0.17876E+01_JPRB, 0.18188E+01_JPRB, 0.23247E+01_JPRB, 0.34121E+01_JPRB, 0.42211E+01_JPRB, 0.47195E+01_JPRB ,&
&  0.45933E+01_JPRB, 0.49975E+01_JPRB, 0.82250E+01_JPRB, 0.92691E+01_JPRB, 0.10153E+02_JPRB, 0.11758E+02_JPRB, 0.12361E+02_JPRB ,&
&  0.11806E+02_JPRB, 0.10561E+02_JPRB, 0.93726E+01_JPRB, 0.81929E+01_JPRB, 0.88634E+01_JPRB, 0.81757E+01_JPRB, 0.71254E+01_JPRB ,&
&  0.64606E+01_JPRB, 0.64608E+01_JPRB, 0.59179E+01_JPRB, 0.53082E+01_JPRB, 0.48615E+01_JPRB, 0.48872E+01_JPRB, 0.38699E+01_JPRB ,&
&  0.30104E+01_JPRB, 0.28130E+01_JPRB, 0.20075E+01_JPRB, 0.15733E+01_JPRB, 0.14117E+01_JPRB, 0.14148E+01_JPRB, 0.15484E+01_JPRB ,&
&  0.17298E+01_JPRB, 0.20730E+01_JPRB, 0.25928E+01_JPRB, 0.30967E+01_JPRB, 0.41618E+01_JPRB, 0.47608E+01_JPRB, 0.46345E+01_JPRB ,&
&  0.36864E+01_JPRB, 0.34529E+01_JPRB, 0.31368E+01_JPRB, 0.28905E+01_JPRB, 0.26533E+01_JPRB, 0.23685E+01_JPRB, 0.22198E+01_JPRB ,&
&  0.20440E+01_JPRB, 0.19226E+01_JPRB, 0.18329E+01_JPRB, 0.16771E+01_JPRB, 0.15711E+01_JPRB /)
ZBURD(:, 20,  6) = (/ &
&  0.17049E+01_JPRB, 0.16096E+01_JPRB, 0.14790E+01_JPRB, 0.13857E+01_JPRB, 0.12919E+01_JPRB, 0.12300E+01_JPRB, 0.12050E+01_JPRB ,&
&  0.11841E+01_JPRB, 0.11536E+01_JPRB, 0.11544E+01_JPRB, 0.11662E+01_JPRB, 0.11626E+01_JPRB, 0.11485E+01_JPRB, 0.10626E+01_JPRB ,&
&  0.10023E+01_JPRB, 0.96908E+00_JPRB, 0.82667E+00_JPRB, 0.83885E+00_JPRB, 0.98348E+00_JPRB, 0.11565E+01_JPRB, 0.14335E+01_JPRB ,&
&  0.17606E+01_JPRB, 0.23596E+01_JPRB, 0.30522E+01_JPRB, 0.41334E+01_JPRB, 0.47813E+01_JPRB, 0.50253E+01_JPRB, 0.49405E+01_JPRB ,&
&  0.45324E+01_JPRB, 0.45278E+01_JPRB, 0.42917E+01_JPRB, 0.38510E+01_JPRB, 0.33911E+01_JPRB, 0.30183E+01_JPRB, 0.26954E+01_JPRB ,&
&  0.24206E+01_JPRB, 0.22017E+01_JPRB, 0.20445E+01_JPRB, 0.19316E+01_JPRB, 0.19028E+01_JPRB, 0.18836E+01_JPRB, 0.19021E+01_JPRB ,&
&  0.18518E+01_JPRB, 0.18347E+01_JPRB, 0.22364E+01_JPRB, 0.35056E+01_JPRB, 0.45154E+01_JPRB, 0.52161E+01_JPRB, 0.48856E+01_JPRB ,&
&  0.56239E+01_JPRB, 0.65123E+01_JPRB, 0.70449E+01_JPRB, 0.84082E+01_JPRB, 0.76505E+01_JPRB, 0.94136E+01_JPRB, 0.11453E+02_JPRB ,&
&  0.11122E+02_JPRB, 0.99251E+01_JPRB, 0.91914E+01_JPRB, 0.83271E+01_JPRB, 0.72214E+01_JPRB, 0.84176E+01_JPRB, 0.70294E+01_JPRB ,&
&  0.61865E+01_JPRB, 0.59660E+01_JPRB, 0.54463E+01_JPRB, 0.54836E+01_JPRB, 0.45438E+01_JPRB, 0.40122E+01_JPRB, 0.27780E+01_JPRB ,&
&  0.27945E+01_JPRB, 0.24552E+01_JPRB, 0.24768E+01_JPRB, 0.22122E+01_JPRB, 0.22084E+01_JPRB, 0.21109E+01_JPRB, 0.21323E+01_JPRB ,&
&  0.24631E+01_JPRB, 0.28638E+01_JPRB, 0.39440E+01_JPRB, 0.46528E+01_JPRB, 0.61429E+01_JPRB, 0.55356E+01_JPRB, 0.54696E+01_JPRB ,&
&  0.47517E+01_JPRB, 0.43152E+01_JPRB, 0.35850E+01_JPRB, 0.34060E+01_JPRB, 0.30084E+01_JPRB, 0.26881E+01_JPRB, 0.24196E+01_JPRB ,&
&  0.21588E+01_JPRB, 0.19857E+01_JPRB, 0.18615E+01_JPRB, 0.17646E+01_JPRB, 0.17473E+01_JPRB /)
ZBURD(:, 21,  6) = (/ &
&  0.16184E+01_JPRB, 0.15265E+01_JPRB, 0.14311E+01_JPRB, 0.13349E+01_JPRB, 0.12232E+01_JPRB, 0.11623E+01_JPRB, 0.11267E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11155E+01_JPRB, 0.11299E+01_JPRB, 0.11388E+01_JPRB, 0.11413E+01_JPRB, 0.11342E+01_JPRB, 0.10536E+01_JPRB ,&
&  0.98359E+00_JPRB, 0.93933E+00_JPRB, 0.82612E+00_JPRB, 0.82360E+00_JPRB, 0.11101E+01_JPRB, 0.14706E+01_JPRB, 0.15395E+01_JPRB ,&
&  0.21581E+01_JPRB, 0.29547E+01_JPRB, 0.38162E+01_JPRB, 0.53772E+01_JPRB, 0.68523E+01_JPRB, 0.76886E+01_JPRB, 0.70962E+01_JPRB ,&
&  0.69433E+01_JPRB, 0.62558E+01_JPRB, 0.52306E+01_JPRB, 0.43797E+01_JPRB, 0.35829E+01_JPRB, 0.29400E+01_JPRB, 0.24581E+01_JPRB ,&
&  0.21550E+01_JPRB, 0.19542E+01_JPRB, 0.18029E+01_JPRB, 0.17321E+01_JPRB, 0.17397E+01_JPRB, 0.16982E+01_JPRB, 0.17731E+01_JPRB ,&
&  0.18763E+01_JPRB, 0.20555E+01_JPRB, 0.28064E+01_JPRB, 0.41366E+01_JPRB, 0.53367E+01_JPRB, 0.56928E+01_JPRB, 0.55393E+01_JPRB ,&
&  0.55599E+01_JPRB, 0.58520E+01_JPRB, 0.64856E+01_JPRB, 0.69563E+01_JPRB, 0.78849E+01_JPRB, 0.91785E+01_JPRB, 0.10830E+02_JPRB ,&
&  0.85749E+01_JPRB, 0.69978E+01_JPRB, 0.57524E+01_JPRB, 0.55703E+01_JPRB, 0.64193E+01_JPRB, 0.80038E+01_JPRB, 0.75034E+01_JPRB ,&
&  0.66667E+01_JPRB, 0.61863E+01_JPRB, 0.56425E+01_JPRB, 0.50378E+01_JPRB, 0.52889E+01_JPRB, 0.32677E+01_JPRB, 0.35415E+01_JPRB ,&
&  0.35863E+01_JPRB, 0.34982E+01_JPRB, 0.31310E+01_JPRB, 0.26496E+01_JPRB, 0.24259E+01_JPRB, 0.27922E+01_JPRB, 0.31207E+01_JPRB ,&
&  0.36879E+01_JPRB, 0.42642E+01_JPRB, 0.67199E+01_JPRB, 0.71302E+01_JPRB, 0.71070E+01_JPRB, 0.59913E+01_JPRB, 0.55789E+01_JPRB ,&
&  0.46242E+01_JPRB, 0.43210E+01_JPRB, 0.41258E+01_JPRB, 0.37306E+01_JPRB, 0.32902E+01_JPRB, 0.29499E+01_JPRB, 0.26023E+01_JPRB ,&
&  0.23134E+01_JPRB, 0.20606E+01_JPRB, 0.18587E+01_JPRB, 0.17265E+01_JPRB, 0.16774E+01_JPRB /)
ZBURD(:, 22,  6) = (/ &
&  0.15417E+01_JPRB, 0.14653E+01_JPRB, 0.13598E+01_JPRB, 0.12132E+01_JPRB, 0.11296E+01_JPRB, 0.11023E+01_JPRB, 0.11032E+01_JPRB ,&
&  0.11126E+01_JPRB, 0.11220E+01_JPRB, 0.11228E+01_JPRB, 0.11217E+01_JPRB, 0.11331E+01_JPRB, 0.11137E+01_JPRB, 0.10523E+01_JPRB ,&
&  0.10002E+01_JPRB, 0.10209E+01_JPRB, 0.90277E+00_JPRB, 0.96174E+00_JPRB, 0.12568E+01_JPRB, 0.18556E+01_JPRB, 0.18214E+01_JPRB ,&
&  0.26420E+01_JPRB, 0.34142E+01_JPRB, 0.40021E+01_JPRB, 0.54293E+01_JPRB, 0.88418E+01_JPRB, 0.87522E+01_JPRB, 0.82859E+01_JPRB ,&
&  0.76191E+01_JPRB, 0.62032E+01_JPRB, 0.48995E+01_JPRB, 0.38824E+01_JPRB, 0.31106E+01_JPRB, 0.26033E+01_JPRB, 0.22370E+01_JPRB ,&
&  0.20140E+01_JPRB, 0.18351E+01_JPRB, 0.17203E+01_JPRB, 0.17092E+01_JPRB, 0.16687E+01_JPRB, 0.16479E+01_JPRB, 0.17852E+01_JPRB ,&
&  0.19703E+01_JPRB, 0.22349E+01_JPRB, 0.29198E+01_JPRB, 0.39222E+01_JPRB, 0.50272E+01_JPRB, 0.55126E+01_JPRB, 0.66164E+01_JPRB ,&
&  0.60409E+01_JPRB, 0.63027E+01_JPRB, 0.63746E+01_JPRB, 0.77594E+01_JPRB, 0.79483E+01_JPRB, 0.92651E+01_JPRB, 0.10236E+02_JPRB ,&
&  0.73998E+01_JPRB, 0.60524E+01_JPRB, 0.60258E+01_JPRB, 0.66432E+01_JPRB, 0.53545E+01_JPRB, 0.57131E+01_JPRB, 0.75299E+01_JPRB ,&
&  0.59112E+01_JPRB, 0.57636E+01_JPRB, 0.57970E+01_JPRB, 0.61907E+01_JPRB, 0.36546E+01_JPRB, 0.24485E+01_JPRB, 0.37625E+01_JPRB ,&
&  0.36039E+01_JPRB, 0.20326E+01_JPRB, 0.14351E+01_JPRB, 0.18361E+01_JPRB, 0.14300E+01_JPRB, 0.19194E+01_JPRB, 0.34041E+01_JPRB ,&
&  0.43108E+01_JPRB, 0.52449E+01_JPRB, 0.80960E+01_JPRB, 0.68238E+01_JPRB, 0.62449E+01_JPRB, 0.56841E+01_JPRB, 0.50098E+01_JPRB ,&
&  0.45607E+01_JPRB, 0.37733E+01_JPRB, 0.38039E+01_JPRB, 0.34285E+01_JPRB, 0.31536E+01_JPRB, 0.29374E+01_JPRB, 0.26281E+01_JPRB ,&
&  0.23198E+01_JPRB, 0.20304E+01_JPRB, 0.17962E+01_JPRB, 0.16567E+01_JPRB, 0.15974E+01_JPRB /)
ZBURD(:, 23,  6) = (/ &
&  0.13846E+01_JPRB, 0.13002E+01_JPRB, 0.12508E+01_JPRB, 0.11744E+01_JPRB, 0.11146E+01_JPRB, 0.10944E+01_JPRB, 0.11195E+01_JPRB ,&
&  0.11172E+01_JPRB, 0.10935E+01_JPRB, 0.10749E+01_JPRB, 0.10822E+01_JPRB, 0.10957E+01_JPRB, 0.10848E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.98610E+00_JPRB, 0.99409E+00_JPRB, 0.11821E+01_JPRB, 0.13038E+01_JPRB, 0.16609E+01_JPRB, 0.21350E+01_JPRB, 0.24796E+01_JPRB ,&
&  0.29623E+01_JPRB, 0.35244E+01_JPRB, 0.41625E+01_JPRB, 0.52877E+01_JPRB, 0.65946E+01_JPRB, 0.71221E+01_JPRB, 0.74071E+01_JPRB ,&
&  0.61025E+01_JPRB, 0.48619E+01_JPRB, 0.41319E+01_JPRB, 0.35993E+01_JPRB, 0.29906E+01_JPRB, 0.25128E+01_JPRB, 0.21585E+01_JPRB ,&
&  0.19516E+01_JPRB, 0.17960E+01_JPRB, 0.17605E+01_JPRB, 0.16578E+01_JPRB, 0.16331E+01_JPRB, 0.16678E+01_JPRB, 0.18313E+01_JPRB ,&
&  0.20536E+01_JPRB, 0.23537E+01_JPRB, 0.28622E+01_JPRB, 0.37448E+01_JPRB, 0.53351E+01_JPRB, 0.61355E+01_JPRB, 0.59619E+01_JPRB ,&
&  0.56542E+01_JPRB, 0.56994E+01_JPRB, 0.68198E+01_JPRB, 0.76222E+01_JPRB, 0.74513E+01_JPRB, 0.89133E+01_JPRB, 0.92556E+01_JPRB ,&
&  0.80992E+01_JPRB, 0.71310E+01_JPRB, 0.60096E+01_JPRB, 0.65808E+01_JPRB, 0.57737E+01_JPRB, 0.49185E+01_JPRB, 0.55915E+01_JPRB ,&
&  0.58329E+01_JPRB, 0.51814E+01_JPRB, 0.43570E+01_JPRB, 0.37859E+01_JPRB, 0.37587E+01_JPRB, 0.29369E+01_JPRB, 0.20531E+01_JPRB ,&
&  0.17132E+01_JPRB, 0.13837E+01_JPRB, 0.11313E+01_JPRB, 0.11702E+01_JPRB, 0.10311E+01_JPRB, 0.15906E+01_JPRB, 0.28881E+01_JPRB ,&
&  0.48343E+01_JPRB, 0.58354E+01_JPRB, 0.70545E+01_JPRB, 0.62403E+01_JPRB, 0.57000E+01_JPRB, 0.56968E+01_JPRB, 0.55829E+01_JPRB ,&
&  0.39509E+01_JPRB, 0.36187E+01_JPRB, 0.35908E+01_JPRB, 0.33152E+01_JPRB, 0.30471E+01_JPRB, 0.27769E+01_JPRB, 0.24402E+01_JPRB ,&
&  0.21003E+01_JPRB, 0.18080E+01_JPRB, 0.16450E+01_JPRB, 0.15812E+01_JPRB, 0.15050E+01_JPRB /)
ZBURD(:, 24,  6) = (/ &
&  0.11292E+01_JPRB, 0.11048E+01_JPRB, 0.11144E+01_JPRB, 0.10997E+01_JPRB, 0.10753E+01_JPRB, 0.10831E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10556E+01_JPRB, 0.10227E+01_JPRB, 0.10237E+01_JPRB, 0.10485E+01_JPRB, 0.10532E+01_JPRB, 0.10491E+01_JPRB, 0.10104E+01_JPRB ,&
&  0.96820E+00_JPRB, 0.98270E+00_JPRB, 0.11950E+01_JPRB, 0.13686E+01_JPRB, 0.19843E+01_JPRB, 0.24600E+01_JPRB, 0.29030E+01_JPRB ,&
&  0.28618E+01_JPRB, 0.33041E+01_JPRB, 0.39969E+01_JPRB, 0.42172E+01_JPRB, 0.53401E+01_JPRB, 0.51134E+01_JPRB, 0.43251E+01_JPRB ,&
&  0.32583E+01_JPRB, 0.31270E+01_JPRB, 0.30346E+01_JPRB, 0.28054E+01_JPRB, 0.25197E+01_JPRB, 0.22548E+01_JPRB, 0.20606E+01_JPRB ,&
&  0.19481E+01_JPRB, 0.18693E+01_JPRB, 0.17148E+01_JPRB, 0.16432E+01_JPRB, 0.16730E+01_JPRB, 0.17345E+01_JPRB, 0.18921E+01_JPRB ,&
&  0.21417E+01_JPRB, 0.24610E+01_JPRB, 0.29218E+01_JPRB, 0.42057E+01_JPRB, 0.47957E+01_JPRB, 0.50099E+01_JPRB, 0.57369E+01_JPRB ,&
&  0.62191E+01_JPRB, 0.67160E+01_JPRB, 0.63775E+01_JPRB, 0.76198E+01_JPRB, 0.71892E+01_JPRB, 0.78743E+01_JPRB, 0.82693E+01_JPRB ,&
&  0.73718E+01_JPRB, 0.69152E+01_JPRB, 0.55337E+01_JPRB, 0.54967E+01_JPRB, 0.58790E+01_JPRB, 0.53427E+01_JPRB, 0.50194E+01_JPRB ,&
&  0.54331E+01_JPRB, 0.48345E+01_JPRB, 0.43945E+01_JPRB, 0.41470E+01_JPRB, 0.53143E+01_JPRB, 0.50983E+01_JPRB, 0.23394E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.14155E+01_JPRB, 0.10521E+01_JPRB, 0.90150E+00_JPRB, 0.98162E+00_JPRB, 0.15559E+01_JPRB, 0.34717E+01_JPRB ,&
&  0.40782E+01_JPRB, 0.67431E+01_JPRB, 0.71078E+01_JPRB, 0.71608E+01_JPRB, 0.62991E+01_JPRB, 0.53989E+01_JPRB, 0.45552E+01_JPRB ,&
&  0.44359E+01_JPRB, 0.39486E+01_JPRB, 0.33638E+01_JPRB, 0.30166E+01_JPRB, 0.26736E+01_JPRB, 0.23412E+01_JPRB, 0.20701E+01_JPRB ,&
&  0.18716E+01_JPRB, 0.16581E+01_JPRB, 0.15277E+01_JPRB, 0.14271E+01_JPRB, 0.12566E+01_JPRB /)
ZBURD(:, 25,  6) = (/ &
&  0.93102E+00_JPRB, 0.95452E+00_JPRB, 0.10018E+01_JPRB, 0.10113E+01_JPRB, 0.10009E+01_JPRB, 0.10217E+01_JPRB, 0.10020E+01_JPRB ,&
&  0.98593E+00_JPRB, 0.98442E+00_JPRB, 0.10002E+01_JPRB, 0.10251E+01_JPRB, 0.10546E+01_JPRB, 0.10704E+01_JPRB, 0.10395E+01_JPRB ,&
&  0.10022E+01_JPRB, 0.10164E+01_JPRB, 0.11279E+01_JPRB, 0.15557E+01_JPRB, 0.21302E+01_JPRB, 0.27367E+01_JPRB, 0.29185E+01_JPRB ,&
&  0.26046E+01_JPRB, 0.29436E+01_JPRB, 0.36035E+01_JPRB, 0.38233E+01_JPRB, 0.35383E+01_JPRB, 0.36092E+01_JPRB, 0.25435E+01_JPRB ,&
&  0.21219E+01_JPRB, 0.21303E+01_JPRB, 0.21369E+01_JPRB, 0.21588E+01_JPRB, 0.21218E+01_JPRB, 0.20142E+01_JPRB, 0.19684E+01_JPRB ,&
&  0.19284E+01_JPRB, 0.17966E+01_JPRB, 0.17340E+01_JPRB, 0.17302E+01_JPRB, 0.17788E+01_JPRB, 0.18405E+01_JPRB, 0.19971E+01_JPRB ,&
&  0.22859E+01_JPRB, 0.26827E+01_JPRB, 0.32874E+01_JPRB, 0.45936E+01_JPRB, 0.41970E+01_JPRB, 0.55321E+01_JPRB, 0.58775E+01_JPRB ,&
&  0.60847E+01_JPRB, 0.62974E+01_JPRB, 0.59982E+01_JPRB, 0.65641E+01_JPRB, 0.67906E+01_JPRB, 0.73512E+01_JPRB, 0.72119E+01_JPRB ,&
&  0.67416E+01_JPRB, 0.56800E+01_JPRB, 0.55180E+01_JPRB, 0.53826E+01_JPRB, 0.53975E+01_JPRB, 0.66111E+01_JPRB, 0.49537E+01_JPRB ,&
&  0.44275E+01_JPRB, 0.50633E+01_JPRB, 0.47873E+01_JPRB, 0.44418E+01_JPRB, 0.50234E+01_JPRB, 0.56843E+01_JPRB, 0.45372E+01_JPRB ,&
&  0.18082E+01_JPRB, 0.12253E+01_JPRB, 0.10691E+01_JPRB, 0.92156E+00_JPRB, 0.79844E+00_JPRB, 0.14184E+01_JPRB, 0.70784E+01_JPRB ,&
&  0.49383E+01_JPRB, 0.69519E+01_JPRB, 0.70815E+01_JPRB, 0.66705E+01_JPRB, 0.60423E+01_JPRB, 0.48667E+01_JPRB, 0.42884E+01_JPRB ,&
&  0.35046E+01_JPRB, 0.28546E+01_JPRB, 0.23851E+01_JPRB, 0.20436E+01_JPRB, 0.18452E+01_JPRB, 0.17476E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.16343E+01_JPRB, 0.14869E+01_JPRB, 0.13388E+01_JPRB, 0.11350E+01_JPRB, 0.96476E+00_JPRB /)
ZBURD(:, 26,  6) = (/ &
&  0.92022E+00_JPRB, 0.92741E+00_JPRB, 0.95602E+00_JPRB, 0.10065E+01_JPRB, 0.10408E+01_JPRB, 0.10409E+01_JPRB, 0.10243E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.10668E+01_JPRB, 0.11103E+01_JPRB, 0.11312E+01_JPRB, 0.11581E+01_JPRB, 0.11438E+01_JPRB, 0.11160E+01_JPRB ,&
&  0.10781E+01_JPRB, 0.10991E+01_JPRB, 0.13257E+01_JPRB, 0.18175E+01_JPRB, 0.22856E+01_JPRB, 0.26292E+01_JPRB, 0.27790E+01_JPRB ,&
&  0.25532E+01_JPRB, 0.22320E+01_JPRB, 0.21414E+01_JPRB, 0.24697E+01_JPRB, 0.26421E+01_JPRB, 0.29940E+01_JPRB, 0.20003E+01_JPRB ,&
&  0.18490E+01_JPRB, 0.18381E+01_JPRB, 0.18468E+01_JPRB, 0.18752E+01_JPRB, 0.19034E+01_JPRB, 0.18814E+01_JPRB, 0.18705E+01_JPRB ,&
&  0.18274E+01_JPRB, 0.18226E+01_JPRB, 0.18467E+01_JPRB, 0.18814E+01_JPRB, 0.19221E+01_JPRB, 0.19797E+01_JPRB, 0.21805E+01_JPRB ,&
&  0.25288E+01_JPRB, 0.29827E+01_JPRB, 0.35808E+01_JPRB, 0.42590E+01_JPRB, 0.47888E+01_JPRB, 0.51815E+01_JPRB, 0.57140E+01_JPRB ,&
&  0.59210E+01_JPRB, 0.61431E+01_JPRB, 0.60969E+01_JPRB, 0.59531E+01_JPRB, 0.61380E+01_JPRB, 0.68847E+01_JPRB, 0.65586E+01_JPRB ,&
&  0.61086E+01_JPRB, 0.55984E+01_JPRB, 0.53682E+01_JPRB, 0.52886E+01_JPRB, 0.53991E+01_JPRB, 0.64290E+01_JPRB, 0.62060E+01_JPRB ,&
&  0.52699E+01_JPRB, 0.44670E+01_JPRB, 0.43115E+01_JPRB, 0.44028E+01_JPRB, 0.44517E+01_JPRB, 0.47503E+01_JPRB, 0.61703E+01_JPRB ,&
&  0.48046E+01_JPRB, 0.21976E+01_JPRB, 0.12416E+01_JPRB, 0.12045E+01_JPRB, 0.77833E+00_JPRB, 0.95882E+00_JPRB, 0.26829E+01_JPRB ,&
&  0.37659E+01_JPRB, 0.47134E+01_JPRB, 0.49924E+01_JPRB, 0.46086E+01_JPRB, 0.41357E+01_JPRB, 0.32156E+01_JPRB, 0.24034E+01_JPRB ,&
&  0.17916E+01_JPRB, 0.14615E+01_JPRB, 0.13018E+01_JPRB, 0.12500E+01_JPRB, 0.13228E+01_JPRB, 0.14284E+01_JPRB, 0.14971E+01_JPRB ,&
&  0.14706E+01_JPRB, 0.13241E+01_JPRB, 0.11000E+01_JPRB, 0.93361E+00_JPRB, 0.89519E+00_JPRB /)
ZBURD(:, 27,  6) = (/ &
&  0.10390E+01_JPRB, 0.10311E+01_JPRB, 0.10545E+01_JPRB, 0.11083E+01_JPRB, 0.11375E+01_JPRB, 0.11583E+01_JPRB, 0.12420E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.13002E+01_JPRB, 0.13230E+01_JPRB, 0.13374E+01_JPRB, 0.13185E+01_JPRB, 0.12614E+01_JPRB, 0.12174E+01_JPRB ,&
&  0.12248E+01_JPRB, 0.13774E+01_JPRB, 0.17256E+01_JPRB, 0.22501E+01_JPRB, 0.26601E+01_JPRB, 0.33719E+01_JPRB, 0.24596E+01_JPRB ,&
&  0.22035E+01_JPRB, 0.22331E+01_JPRB, 0.18339E+01_JPRB, 0.19801E+01_JPRB, 0.20669E+01_JPRB, 0.21874E+01_JPRB, 0.18919E+01_JPRB ,&
&  0.18257E+01_JPRB, 0.18278E+01_JPRB, 0.17995E+01_JPRB, 0.18085E+01_JPRB, 0.18155E+01_JPRB, 0.18321E+01_JPRB, 0.18364E+01_JPRB ,&
&  0.18924E+01_JPRB, 0.19791E+01_JPRB, 0.20403E+01_JPRB, 0.20614E+01_JPRB, 0.20935E+01_JPRB, 0.21754E+01_JPRB, 0.24679E+01_JPRB ,&
&  0.28328E+01_JPRB, 0.32554E+01_JPRB, 0.38370E+01_JPRB, 0.43235E+01_JPRB, 0.47320E+01_JPRB, 0.48831E+01_JPRB, 0.53471E+01_JPRB ,&
&  0.53316E+01_JPRB, 0.51713E+01_JPRB, 0.54580E+01_JPRB, 0.56928E+01_JPRB, 0.59086E+01_JPRB, 0.61300E+01_JPRB, 0.56399E+01_JPRB ,&
&  0.56692E+01_JPRB, 0.58746E+01_JPRB, 0.62440E+01_JPRB, 0.56469E+01_JPRB, 0.55823E+01_JPRB, 0.61574E+01_JPRB, 0.63105E+01_JPRB ,&
&  0.57882E+01_JPRB, 0.50417E+01_JPRB, 0.46464E+01_JPRB, 0.39356E+01_JPRB, 0.36828E+01_JPRB, 0.37860E+01_JPRB, 0.45251E+01_JPRB ,&
&  0.45813E+01_JPRB, 0.43395E+01_JPRB, 0.22458E+01_JPRB, 0.10580E+01_JPRB, 0.88645E+00_JPRB, 0.12583E+01_JPRB, 0.18196E+01_JPRB ,&
&  0.23601E+01_JPRB, 0.26377E+01_JPRB, 0.28665E+01_JPRB, 0.31392E+01_JPRB, 0.26661E+01_JPRB, 0.20419E+01_JPRB, 0.15915E+01_JPRB ,&
&  0.13111E+01_JPRB, 0.11132E+01_JPRB, 0.10245E+01_JPRB, 0.10474E+01_JPRB, 0.11826E+01_JPRB, 0.13218E+01_JPRB, 0.13828E+01_JPRB ,&
&  0.12703E+01_JPRB, 0.10815E+01_JPRB, 0.99250E+00_JPRB, 0.97770E+00_JPRB, 0.10056E+01_JPRB /)
ZBURD(:, 28,  6) = (/ &
&  0.11838E+01_JPRB, 0.12111E+01_JPRB, 0.12687E+01_JPRB, 0.13499E+01_JPRB, 0.14234E+01_JPRB, 0.15033E+01_JPRB, 0.15623E+01_JPRB ,&
&  0.15762E+01_JPRB, 0.15623E+01_JPRB, 0.15567E+01_JPRB, 0.15178E+01_JPRB, 0.14970E+01_JPRB, 0.14862E+01_JPRB, 0.15508E+01_JPRB ,&
&  0.17529E+01_JPRB, 0.20677E+01_JPRB, 0.24112E+01_JPRB, 0.26964E+01_JPRB, 0.29026E+01_JPRB, 0.32441E+01_JPRB, 0.29287E+01_JPRB ,&
&  0.20120E+01_JPRB, 0.26709E+01_JPRB, 0.18862E+01_JPRB, 0.19838E+01_JPRB, 0.20869E+01_JPRB, 0.23039E+01_JPRB, 0.20184E+01_JPRB ,&
&  0.18417E+01_JPRB, 0.18569E+01_JPRB, 0.18704E+01_JPRB, 0.18891E+01_JPRB, 0.18528E+01_JPRB, 0.18929E+01_JPRB, 0.19788E+01_JPRB ,&
&  0.20942E+01_JPRB, 0.21640E+01_JPRB, 0.22008E+01_JPRB, 0.22307E+01_JPRB, 0.23120E+01_JPRB, 0.24559E+01_JPRB, 0.28300E+01_JPRB ,&
&  0.31897E+01_JPRB, 0.36371E+01_JPRB, 0.41076E+01_JPRB, 0.45491E+01_JPRB, 0.45116E+01_JPRB, 0.45974E+01_JPRB, 0.49222E+01_JPRB ,&
&  0.50034E+01_JPRB, 0.49035E+01_JPRB, 0.51508E+01_JPRB, 0.52044E+01_JPRB, 0.52083E+01_JPRB, 0.56046E+01_JPRB, 0.50915E+01_JPRB ,&
&  0.57504E+01_JPRB, 0.56382E+01_JPRB, 0.66364E+01_JPRB, 0.58526E+01_JPRB, 0.54743E+01_JPRB, 0.56388E+01_JPRB, 0.52423E+01_JPRB ,&
&  0.45573E+01_JPRB, 0.43844E+01_JPRB, 0.38857E+01_JPRB, 0.33523E+01_JPRB, 0.31118E+01_JPRB, 0.30909E+01_JPRB, 0.34370E+01_JPRB ,&
&  0.35107E+01_JPRB, 0.32698E+01_JPRB, 0.22984E+01_JPRB, 0.12058E+01_JPRB, 0.98633E+00_JPRB, 0.11370E+01_JPRB, 0.14703E+01_JPRB ,&
&  0.18605E+01_JPRB, 0.20279E+01_JPRB, 0.20796E+01_JPRB, 0.18291E+01_JPRB, 0.14767E+01_JPRB, 0.12273E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.10270E+01_JPRB, 0.10046E+01_JPRB, 0.10069E+01_JPRB, 0.11088E+01_JPRB, 0.12249E+01_JPRB, 0.12770E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10230E+01_JPRB, 0.10716E+01_JPRB, 0.11174E+01_JPRB, 0.11577E+01_JPRB /)
ZBURD(:, 29,  6) = (/ &
&  0.13655E+01_JPRB, 0.14386E+01_JPRB, 0.15388E+01_JPRB, 0.16804E+01_JPRB, 0.18698E+01_JPRB, 0.20694E+01_JPRB, 0.20129E+01_JPRB ,&
&  0.19233E+01_JPRB, 0.18014E+01_JPRB, 0.18578E+01_JPRB, 0.19175E+01_JPRB, 0.19783E+01_JPRB, 0.20758E+01_JPRB, 0.22233E+01_JPRB ,&
&  0.24087E+01_JPRB, 0.25926E+01_JPRB, 0.27992E+01_JPRB, 0.30143E+01_JPRB, 0.32241E+01_JPRB, 0.36028E+01_JPRB, 0.35430E+01_JPRB ,&
&  0.29036E+01_JPRB, 0.20421E+01_JPRB, 0.22292E+01_JPRB, 0.20094E+01_JPRB, 0.20875E+01_JPRB, 0.22957E+01_JPRB, 0.23097E+01_JPRB ,&
&  0.20901E+01_JPRB, 0.20383E+01_JPRB, 0.20105E+01_JPRB, 0.19949E+01_JPRB, 0.19506E+01_JPRB, 0.20310E+01_JPRB, 0.21343E+01_JPRB ,&
&  0.22025E+01_JPRB, 0.22529E+01_JPRB, 0.23173E+01_JPRB, 0.23910E+01_JPRB, 0.25133E+01_JPRB, 0.27347E+01_JPRB, 0.31497E+01_JPRB ,&
&  0.34811E+01_JPRB, 0.38269E+01_JPRB, 0.41798E+01_JPRB, 0.42903E+01_JPRB, 0.42105E+01_JPRB, 0.42161E+01_JPRB, 0.43746E+01_JPRB ,&
&  0.45914E+01_JPRB, 0.49370E+01_JPRB, 0.50473E+01_JPRB, 0.50504E+01_JPRB, 0.47784E+01_JPRB, 0.53164E+01_JPRB, 0.51917E+01_JPRB ,&
&  0.56782E+01_JPRB, 0.56788E+01_JPRB, 0.66397E+01_JPRB, 0.55297E+01_JPRB, 0.50725E+01_JPRB, 0.47289E+01_JPRB, 0.43253E+01_JPRB ,&
&  0.37180E+01_JPRB, 0.36613E+01_JPRB, 0.32857E+01_JPRB, 0.27020E+01_JPRB, 0.23268E+01_JPRB, 0.20790E+01_JPRB, 0.24862E+01_JPRB ,&
&  0.26787E+01_JPRB, 0.26836E+01_JPRB, 0.17387E+01_JPRB, 0.12631E+01_JPRB, 0.10670E+01_JPRB, 0.10620E+01_JPRB, 0.13036E+01_JPRB ,&
&  0.16431E+01_JPRB, 0.16480E+01_JPRB, 0.14579E+01_JPRB, 0.13478E+01_JPRB, 0.11794E+01_JPRB, 0.10189E+01_JPRB, 0.90371E+00_JPRB ,&
&  0.94188E+00_JPRB, 0.94587E+00_JPRB, 0.98371E+00_JPRB, 0.10665E+01_JPRB, 0.11002E+01_JPRB, 0.11380E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.11902E+01_JPRB, 0.12249E+01_JPRB, 0.12653E+01_JPRB, 0.13032E+01_JPRB /)
ZBURD(:, 30,  6) = (/ &
&  0.14268E+01_JPRB, 0.15019E+01_JPRB, 0.15910E+01_JPRB, 0.16964E+01_JPRB, 0.18068E+01_JPRB, 0.18749E+01_JPRB, 0.19048E+01_JPRB ,&
&  0.19782E+01_JPRB, 0.20767E+01_JPRB, 0.21869E+01_JPRB, 0.22743E+01_JPRB, 0.23212E+01_JPRB, 0.23930E+01_JPRB, 0.24814E+01_JPRB ,&
&  0.25676E+01_JPRB, 0.26749E+01_JPRB, 0.28252E+01_JPRB, 0.30110E+01_JPRB, 0.32315E+01_JPRB, 0.35005E+01_JPRB, 0.36137E+01_JPRB ,&
&  0.28537E+01_JPRB, 0.18991E+01_JPRB, 0.22986E+01_JPRB, 0.22330E+01_JPRB, 0.21636E+01_JPRB, 0.22213E+01_JPRB, 0.22238E+01_JPRB ,&
&  0.21461E+01_JPRB, 0.21465E+01_JPRB, 0.21557E+01_JPRB, 0.21423E+01_JPRB, 0.20988E+01_JPRB, 0.21564E+01_JPRB, 0.22154E+01_JPRB ,&
&  0.22751E+01_JPRB, 0.23408E+01_JPRB, 0.24077E+01_JPRB, 0.24773E+01_JPRB, 0.26127E+01_JPRB, 0.28383E+01_JPRB, 0.32029E+01_JPRB ,&
&  0.35515E+01_JPRB, 0.37725E+01_JPRB, 0.39639E+01_JPRB, 0.40754E+01_JPRB, 0.39682E+01_JPRB, 0.38684E+01_JPRB, 0.39124E+01_JPRB ,&
&  0.41108E+01_JPRB, 0.43981E+01_JPRB, 0.47752E+01_JPRB, 0.50726E+01_JPRB, 0.53553E+01_JPRB, 0.50248E+01_JPRB, 0.53232E+01_JPRB ,&
&  0.55538E+01_JPRB, 0.53498E+01_JPRB, 0.56854E+01_JPRB, 0.64113E+01_JPRB, 0.42925E+01_JPRB, 0.39161E+01_JPRB, 0.34020E+01_JPRB ,&
&  0.35231E+01_JPRB, 0.32900E+01_JPRB, 0.27449E+01_JPRB, 0.21490E+01_JPRB, 0.17996E+01_JPRB, 0.14804E+01_JPRB, 0.17246E+01_JPRB ,&
&  0.20943E+01_JPRB, 0.21072E+01_JPRB, 0.15716E+01_JPRB, 0.13636E+01_JPRB, 0.12156E+01_JPRB, 0.12976E+01_JPRB, 0.14570E+01_JPRB ,&
&  0.15394E+01_JPRB, 0.13641E+01_JPRB, 0.12751E+01_JPRB, 0.13501E+01_JPRB, 0.11532E+01_JPRB, 0.85517E+00_JPRB, 0.73776E+00_JPRB ,&
&  0.74901E+00_JPRB, 0.79568E+00_JPRB, 0.86998E+00_JPRB, 0.96304E+00_JPRB, 0.10049E+01_JPRB, 0.10533E+01_JPRB, 0.10793E+01_JPRB ,&
&  0.11781E+01_JPRB, 0.12449E+01_JPRB, 0.12975E+01_JPRB, 0.13193E+01_JPRB, 0.13733E+01_JPRB /)
ZBURD(:, 31,  6) = (/ &
&  0.13405E+01_JPRB, 0.14110E+01_JPRB, 0.14641E+01_JPRB, 0.15278E+01_JPRB, 0.15868E+01_JPRB, 0.16470E+01_JPRB, 0.17285E+01_JPRB ,&
&  0.18352E+01_JPRB, 0.19526E+01_JPRB, 0.20643E+01_JPRB, 0.21495E+01_JPRB, 0.22316E+01_JPRB, 0.22871E+01_JPRB, 0.23343E+01_JPRB ,&
&  0.23608E+01_JPRB, 0.24265E+01_JPRB, 0.24949E+01_JPRB, 0.25914E+01_JPRB, 0.27199E+01_JPRB, 0.28688E+01_JPRB, 0.30159E+01_JPRB ,&
&  0.31077E+01_JPRB, 0.29635E+01_JPRB, 0.25536E+01_JPRB, 0.18712E+01_JPRB, 0.16849E+01_JPRB, 0.19769E+01_JPRB, 0.20497E+01_JPRB ,&
&  0.20954E+01_JPRB, 0.20954E+01_JPRB, 0.20639E+01_JPRB, 0.21191E+01_JPRB, 0.21549E+01_JPRB, 0.21905E+01_JPRB, 0.22128E+01_JPRB ,&
&  0.22560E+01_JPRB, 0.23159E+01_JPRB, 0.23629E+01_JPRB, 0.24179E+01_JPRB, 0.25228E+01_JPRB, 0.27137E+01_JPRB, 0.29976E+01_JPRB ,&
&  0.33520E+01_JPRB, 0.35510E+01_JPRB, 0.36753E+01_JPRB, 0.37773E+01_JPRB, 0.36621E+01_JPRB, 0.35777E+01_JPRB, 0.36073E+01_JPRB ,&
&  0.37708E+01_JPRB, 0.40028E+01_JPRB, 0.44320E+01_JPRB, 0.48403E+01_JPRB, 0.50174E+01_JPRB, 0.48221E+01_JPRB, 0.48213E+01_JPRB ,&
&  0.48858E+01_JPRB, 0.47412E+01_JPRB, 0.48240E+01_JPRB, 0.61090E+01_JPRB, 0.39287E+01_JPRB, 0.38535E+01_JPRB, 0.36245E+01_JPRB ,&
&  0.34069E+01_JPRB, 0.32864E+01_JPRB, 0.24378E+01_JPRB, 0.17333E+01_JPRB, 0.14975E+01_JPRB, 0.12677E+01_JPRB, 0.14878E+01_JPRB ,&
&  0.17960E+01_JPRB, 0.16175E+01_JPRB, 0.13484E+01_JPRB, 0.12648E+01_JPRB, 0.12712E+01_JPRB, 0.14612E+01_JPRB, 0.15651E+01_JPRB ,&
&  0.13296E+01_JPRB, 0.12184E+01_JPRB, 0.12041E+01_JPRB, 0.14841E+01_JPRB, 0.14009E+01_JPRB, 0.76761E+00_JPRB, 0.64957E+00_JPRB ,&
&  0.66794E+00_JPRB, 0.72167E+00_JPRB, 0.78408E+00_JPRB, 0.83650E+00_JPRB, 0.87860E+00_JPRB, 0.92877E+00_JPRB, 0.98452E+00_JPRB ,&
&  0.10515E+01_JPRB, 0.11397E+01_JPRB, 0.11990E+01_JPRB, 0.12494E+01_JPRB, 0.12886E+01_JPRB /)
ZBURD(:, 32,  6) = (/ &
&  0.10342E+01_JPRB, 0.10992E+01_JPRB, 0.11598E+01_JPRB, 0.11998E+01_JPRB, 0.12124E+01_JPRB, 0.12251E+01_JPRB, 0.12245E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.12292E+01_JPRB, 0.12637E+01_JPRB, 0.13492E+01_JPRB, 0.15119E+01_JPRB, 0.16033E+01_JPRB, 0.17116E+01_JPRB ,&
&  0.17186E+01_JPRB, 0.17327E+01_JPRB, 0.17510E+01_JPRB, 0.17904E+01_JPRB, 0.18644E+01_JPRB, 0.19737E+01_JPRB, 0.22280E+01_JPRB ,&
&  0.27231E+01_JPRB, 0.30378E+01_JPRB, 0.29996E+01_JPRB, 0.26490E+01_JPRB, 0.21150E+01_JPRB, 0.19289E+01_JPRB, 0.21853E+01_JPRB ,&
&  0.22618E+01_JPRB, 0.21343E+01_JPRB, 0.20513E+01_JPRB, 0.20455E+01_JPRB, 0.20620E+01_JPRB, 0.20894E+01_JPRB, 0.21109E+01_JPRB ,&
&  0.21471E+01_JPRB, 0.21913E+01_JPRB, 0.22249E+01_JPRB, 0.22683E+01_JPRB, 0.23540E+01_JPRB, 0.24790E+01_JPRB, 0.26376E+01_JPRB ,&
&  0.28685E+01_JPRB, 0.30156E+01_JPRB, 0.30721E+01_JPRB, 0.30894E+01_JPRB, 0.31030E+01_JPRB, 0.31793E+01_JPRB, 0.33485E+01_JPRB ,&
&  0.35360E+01_JPRB, 0.36484E+01_JPRB, 0.40340E+01_JPRB, 0.43170E+01_JPRB, 0.42710E+01_JPRB, 0.42127E+01_JPRB, 0.41646E+01_JPRB ,&
&  0.41027E+01_JPRB, 0.40046E+01_JPRB, 0.36072E+01_JPRB, 0.51505E+01_JPRB, 0.52231E+01_JPRB, 0.43818E+01_JPRB, 0.36243E+01_JPRB ,&
&  0.32766E+01_JPRB, 0.29832E+01_JPRB, 0.24233E+01_JPRB, 0.18415E+01_JPRB, 0.15730E+01_JPRB, 0.16034E+01_JPRB, 0.16064E+01_JPRB ,&
&  0.16511E+01_JPRB, 0.13695E+01_JPRB, 0.12535E+01_JPRB, 0.12402E+01_JPRB, 0.11803E+01_JPRB, 0.13953E+01_JPRB, 0.13312E+01_JPRB ,&
&  0.10749E+01_JPRB, 0.10323E+01_JPRB, 0.11015E+01_JPRB, 0.15308E+01_JPRB, 0.22543E+01_JPRB, 0.70038E+00_JPRB, 0.57101E+00_JPRB ,&
&  0.59327E+00_JPRB, 0.62723E+00_JPRB, 0.66903E+00_JPRB, 0.68772E+00_JPRB, 0.68979E+00_JPRB, 0.72652E+00_JPRB, 0.78360E+00_JPRB ,&
&  0.81350E+00_JPRB, 0.83629E+00_JPRB, 0.86390E+00_JPRB, 0.93698E+00_JPRB, 0.97966E+00_JPRB /)
ZBURD(:, 33,  6) = (/ &
&  0.68000E+00_JPRB, 0.71631E+00_JPRB, 0.74029E+00_JPRB, 0.76521E+00_JPRB, 0.77443E+00_JPRB, 0.76442E+00_JPRB, 0.72751E+00_JPRB ,&
&  0.69957E+00_JPRB, 0.67343E+00_JPRB, 0.66649E+00_JPRB, 0.65301E+00_JPRB, 0.68049E+00_JPRB, 0.73326E+00_JPRB, 0.77133E+00_JPRB ,&
&  0.87049E+00_JPRB, 0.81260E+00_JPRB, 0.80796E+00_JPRB, 0.83607E+00_JPRB, 0.83888E+00_JPRB, 0.87664E+00_JPRB, 0.10224E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.15087E+01_JPRB, 0.18897E+01_JPRB, 0.24986E+01_JPRB, 0.26746E+01_JPRB, 0.21443E+01_JPRB, 0.22536E+01_JPRB ,&
&  0.20370E+01_JPRB, 0.17594E+01_JPRB, 0.17752E+01_JPRB, 0.17790E+01_JPRB, 0.18409E+01_JPRB, 0.18672E+01_JPRB, 0.19220E+01_JPRB ,&
&  0.19794E+01_JPRB, 0.20409E+01_JPRB, 0.20887E+01_JPRB, 0.21239E+01_JPRB, 0.21682E+01_JPRB, 0.22228E+01_JPRB, 0.22981E+01_JPRB ,&
&  0.23509E+01_JPRB, 0.23583E+01_JPRB, 0.23903E+01_JPRB, 0.23794E+01_JPRB, 0.25623E+01_JPRB, 0.27612E+01_JPRB, 0.29622E+01_JPRB ,&
&  0.31005E+01_JPRB, 0.32255E+01_JPRB, 0.35198E+01_JPRB, 0.36088E+01_JPRB, 0.36228E+01_JPRB, 0.35796E+01_JPRB, 0.33843E+01_JPRB ,&
&  0.30975E+01_JPRB, 0.24997E+01_JPRB, 0.24013E+01_JPRB, 0.38875E+01_JPRB, 0.38325E+01_JPRB, 0.31038E+01_JPRB, 0.31826E+01_JPRB ,&
&  0.29368E+01_JPRB, 0.27642E+01_JPRB, 0.24052E+01_JPRB, 0.19773E+01_JPRB, 0.16713E+01_JPRB, 0.15116E+01_JPRB, 0.15292E+01_JPRB ,&
&  0.13767E+01_JPRB, 0.11763E+01_JPRB, 0.11833E+01_JPRB, 0.12191E+01_JPRB, 0.11635E+01_JPRB, 0.12210E+01_JPRB, 0.11725E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.86899E+00_JPRB, 0.95807E+00_JPRB, 0.12285E+01_JPRB, 0.95571E+00_JPRB, 0.56976E+00_JPRB, 0.50453E+00_JPRB ,&
&  0.51663E+00_JPRB, 0.52959E+00_JPRB, 0.54120E+00_JPRB, 0.54885E+00_JPRB, 0.55502E+00_JPRB, 0.57361E+00_JPRB, 0.58616E+00_JPRB ,&
&  0.59829E+00_JPRB, 0.62549E+00_JPRB, 0.63526E+00_JPRB, 0.64965E+00_JPRB, 0.67084E+00_JPRB /)
ZBURD(:, 34,  6) = (/ &
&  0.59163E+00_JPRB, 0.59052E+00_JPRB, 0.58316E+00_JPRB, 0.58407E+00_JPRB, 0.58924E+00_JPRB, 0.60395E+00_JPRB, 0.61317E+00_JPRB ,&
&  0.61206E+00_JPRB, 0.61802E+00_JPRB, 0.60102E+00_JPRB, 0.60324E+00_JPRB, 0.62272E+00_JPRB, 0.63729E+00_JPRB, 0.65138E+00_JPRB ,&
&  0.69465E+00_JPRB, 0.70784E+00_JPRB, 0.70450E+00_JPRB, 0.70926E+00_JPRB, 0.70235E+00_JPRB, 0.75092E+00_JPRB, 0.75525E+00_JPRB ,&
&  0.73615E+00_JPRB, 0.76256E+00_JPRB, 0.90039E+00_JPRB, 0.12535E+01_JPRB, 0.15867E+01_JPRB, 0.16013E+01_JPRB, 0.18498E+01_JPRB ,&
&  0.17600E+01_JPRB, 0.13900E+01_JPRB, 0.14813E+01_JPRB, 0.13108E+01_JPRB, 0.13457E+01_JPRB, 0.14132E+01_JPRB, 0.14333E+01_JPRB ,&
&  0.14598E+01_JPRB, 0.14452E+01_JPRB, 0.14509E+01_JPRB, 0.14658E+01_JPRB, 0.14555E+01_JPRB, 0.14390E+01_JPRB, 0.14049E+01_JPRB ,&
&  0.13030E+01_JPRB, 0.12962E+01_JPRB, 0.13976E+01_JPRB, 0.16494E+01_JPRB, 0.19369E+01_JPRB, 0.22279E+01_JPRB, 0.23398E+01_JPRB ,&
&  0.24279E+01_JPRB, 0.25314E+01_JPRB, 0.24554E+01_JPRB, 0.25677E+01_JPRB, 0.27321E+01_JPRB, 0.25108E+01_JPRB, 0.23061E+01_JPRB ,&
&  0.20845E+01_JPRB, 0.16212E+01_JPRB, 0.17367E+01_JPRB, 0.26421E+01_JPRB, 0.28630E+01_JPRB, 0.27385E+01_JPRB, 0.27671E+01_JPRB ,&
&  0.26195E+01_JPRB, 0.24123E+01_JPRB, 0.20573E+01_JPRB, 0.18003E+01_JPRB, 0.15830E+01_JPRB, 0.13712E+01_JPRB, 0.13348E+01_JPRB ,&
&  0.11208E+01_JPRB, 0.10910E+01_JPRB, 0.12109E+01_JPRB, 0.12687E+01_JPRB, 0.12858E+01_JPRB, 0.14067E+01_JPRB, 0.11190E+01_JPRB ,&
&  0.90376E+00_JPRB, 0.82287E+00_JPRB, 0.11244E+01_JPRB, 0.14982E+01_JPRB, 0.76925E+00_JPRB, 0.54817E+00_JPRB, 0.45208E+00_JPRB ,&
&  0.46807E+00_JPRB, 0.48657E+00_JPRB, 0.49187E+00_JPRB, 0.49842E+00_JPRB, 0.51328E+00_JPRB, 0.52537E+00_JPRB, 0.54632E+00_JPRB ,&
&  0.55761E+00_JPRB, 0.56675E+00_JPRB, 0.57648E+00_JPRB, 0.59111E+00_JPRB, 0.58787E+00_JPRB /)
ZBURD(:, 35,  6) = (/ &
&  0.63461E+00_JPRB, 0.64348E+00_JPRB, 0.64997E+00_JPRB, 0.66781E+00_JPRB, 0.69477E+00_JPRB, 0.70213E+00_JPRB, 0.71891E+00_JPRB ,&
&  0.72929E+00_JPRB, 0.74830E+00_JPRB, 0.76845E+00_JPRB, 0.79234E+00_JPRB, 0.80834E+00_JPRB, 0.81607E+00_JPRB, 0.83916E+00_JPRB ,&
&  0.85902E+00_JPRB, 0.85499E+00_JPRB, 0.86616E+00_JPRB, 0.88931E+00_JPRB, 0.89413E+00_JPRB, 0.92615E+00_JPRB, 0.92097E+00_JPRB ,&
&  0.94881E+00_JPRB, 0.94866E+00_JPRB, 0.97681E+00_JPRB, 0.10669E+01_JPRB, 0.11707E+01_JPRB, 0.13740E+01_JPRB, 0.17603E+01_JPRB ,&
&  0.15325E+01_JPRB, 0.11765E+01_JPRB, 0.10737E+01_JPRB, 0.91628E+00_JPRB, 0.91517E+00_JPRB, 0.94461E+00_JPRB, 0.93726E+00_JPRB ,&
&  0.90900E+00_JPRB, 0.85343E+00_JPRB, 0.82072E+00_JPRB, 0.79463E+00_JPRB, 0.80061E+00_JPRB, 0.85650E+00_JPRB, 0.88228E+00_JPRB ,&
&  0.85730E+00_JPRB, 0.95608E+00_JPRB, 0.11006E+01_JPRB, 0.13096E+01_JPRB, 0.14901E+01_JPRB, 0.18388E+01_JPRB, 0.18260E+01_JPRB ,&
&  0.17280E+01_JPRB, 0.16850E+01_JPRB, 0.15351E+01_JPRB, 0.16230E+01_JPRB, 0.17354E+01_JPRB, 0.16371E+01_JPRB, 0.14264E+01_JPRB ,&
&  0.13793E+01_JPRB, 0.13240E+01_JPRB, 0.15251E+01_JPRB, 0.21722E+01_JPRB, 0.24415E+01_JPRB, 0.25054E+01_JPRB, 0.23806E+01_JPRB ,&
&  0.21970E+01_JPRB, 0.19460E+01_JPRB, 0.17203E+01_JPRB, 0.16077E+01_JPRB, 0.15472E+01_JPRB, 0.13975E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.10845E+01_JPRB, 0.11429E+01_JPRB, 0.12439E+01_JPRB, 0.12789E+01_JPRB, 0.14794E+01_JPRB, 0.12694E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.83075E+00_JPRB, 0.78897E+00_JPRB, 0.66165E+00_JPRB, 0.85770E+00_JPRB, 0.62287E+00_JPRB, 0.50010E+00_JPRB, 0.49579E+00_JPRB ,&
&  0.50885E+00_JPRB, 0.51764E+00_JPRB, 0.53106E+00_JPRB, 0.54112E+00_JPRB, 0.55383E+00_JPRB, 0.57726E+00_JPRB, 0.60193E+00_JPRB ,&
&  0.62358E+00_JPRB, 0.61214E+00_JPRB, 0.61218E+00_JPRB, 0.61735E+00_JPRB, 0.63179E+00_JPRB /)
ZBURD(:, 36,  6) = (/ &
&  0.67276E+00_JPRB, 0.69307E+00_JPRB, 0.71369E+00_JPRB, 0.73459E+00_JPRB, 0.75288E+00_JPRB, 0.77030E+00_JPRB, 0.78844E+00_JPRB ,&
&  0.80644E+00_JPRB, 0.82745E+00_JPRB, 0.84955E+00_JPRB, 0.87337E+00_JPRB, 0.89369E+00_JPRB, 0.90388E+00_JPRB, 0.91891E+00_JPRB ,&
&  0.93132E+00_JPRB, 0.93718E+00_JPRB, 0.93219E+00_JPRB, 0.95902E+00_JPRB, 0.98597E+00_JPRB, 0.10030E+01_JPRB, 0.10298E+01_JPRB ,&
&  0.10595E+01_JPRB, 0.10428E+01_JPRB, 0.10383E+01_JPRB, 0.10666E+01_JPRB, 0.10862E+01_JPRB, 0.10391E+01_JPRB, 0.11539E+01_JPRB ,&
&  0.91251E+00_JPRB, 0.10199E+01_JPRB, 0.93648E+00_JPRB, 0.82066E+00_JPRB, 0.79197E+00_JPRB, 0.70093E+00_JPRB, 0.65236E+00_JPRB ,&
&  0.68815E+00_JPRB, 0.67977E+00_JPRB, 0.66925E+00_JPRB, 0.67838E+00_JPRB, 0.73813E+00_JPRB, 0.85408E+00_JPRB, 0.93063E+00_JPRB ,&
&  0.10446E+01_JPRB, 0.11608E+01_JPRB, 0.13041E+01_JPRB, 0.14601E+01_JPRB, 0.16154E+01_JPRB, 0.16940E+01_JPRB, 0.16096E+01_JPRB ,&
&  0.14923E+01_JPRB, 0.14015E+01_JPRB, 0.11236E+01_JPRB, 0.12497E+01_JPRB, 0.14201E+01_JPRB, 0.13857E+01_JPRB, 0.12086E+01_JPRB ,&
&  0.11053E+01_JPRB, 0.11684E+01_JPRB, 0.14198E+01_JPRB, 0.17491E+01_JPRB, 0.19491E+01_JPRB, 0.20153E+01_JPRB, 0.19108E+01_JPRB ,&
&  0.17786E+01_JPRB, 0.15527E+01_JPRB, 0.14415E+01_JPRB, 0.14693E+01_JPRB, 0.14572E+01_JPRB, 0.13669E+01_JPRB, 0.11703E+01_JPRB ,&
&  0.11023E+01_JPRB, 0.11456E+01_JPRB, 0.12073E+01_JPRB, 0.12382E+01_JPRB, 0.12860E+01_JPRB, 0.17879E+01_JPRB, 0.18656E+01_JPRB ,&
&  0.83310E+00_JPRB, 0.69098E+00_JPRB, 0.48422E+00_JPRB, 0.58219E+00_JPRB, 0.61716E+00_JPRB, 0.60392E+00_JPRB, 0.65382E+00_JPRB ,&
&  0.68816E+00_JPRB, 0.65322E+00_JPRB, 0.64342E+00_JPRB, 0.61976E+00_JPRB, 0.61949E+00_JPRB, 0.61490E+00_JPRB, 0.62744E+00_JPRB ,&
&  0.63405E+00_JPRB, 0.63349E+00_JPRB, 0.64062E+00_JPRB, 0.65115E+00_JPRB, 0.65912E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART11',1,ZHOOK_HANDLE)
END SUBROUTINE PART11



SUBROUTINE PART12

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART12',0,ZHOOK_HANDLE)


ZBURD(:, 37,  6) = (/ &
&  0.69676E+00_JPRB, 0.71892E+00_JPRB, 0.74125E+00_JPRB, 0.76156E+00_JPRB, 0.78143E+00_JPRB, 0.79989E+00_JPRB, 0.81945E+00_JPRB ,&
&  0.83953E+00_JPRB, 0.86097E+00_JPRB, 0.88157E+00_JPRB, 0.90724E+00_JPRB, 0.93188E+00_JPRB, 0.94583E+00_JPRB, 0.96811E+00_JPRB ,&
&  0.97740E+00_JPRB, 0.10161E+01_JPRB, 0.10341E+01_JPRB, 0.10710E+01_JPRB, 0.11031E+01_JPRB, 0.11233E+01_JPRB, 0.11607E+01_JPRB ,&
&  0.11779E+01_JPRB, 0.11866E+01_JPRB, 0.11979E+01_JPRB, 0.12285E+01_JPRB, 0.12424E+01_JPRB, 0.13299E+01_JPRB, 0.11065E+01_JPRB ,&
&  0.10937E+01_JPRB, 0.10016E+01_JPRB, 0.92439E+00_JPRB, 0.86191E+00_JPRB, 0.81016E+00_JPRB, 0.75750E+00_JPRB, 0.72951E+00_JPRB ,&
&  0.74032E+00_JPRB, 0.73862E+00_JPRB, 0.75473E+00_JPRB, 0.78274E+00_JPRB, 0.84677E+00_JPRB, 0.96642E+00_JPRB, 0.10576E+01_JPRB ,&
&  0.11623E+01_JPRB, 0.12855E+01_JPRB, 0.13939E+01_JPRB, 0.15000E+01_JPRB, 0.16134E+01_JPRB, 0.17111E+01_JPRB, 0.17654E+01_JPRB ,&
&  0.17451E+01_JPRB, 0.16957E+01_JPRB, 0.14699E+01_JPRB, 0.15900E+01_JPRB, 0.16910E+01_JPRB, 0.15662E+01_JPRB, 0.12691E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.98013E+00_JPRB, 0.10832E+01_JPRB, 0.13587E+01_JPRB, 0.14815E+01_JPRB, 0.15087E+01_JPRB, 0.14389E+01_JPRB ,&
&  0.13444E+01_JPRB, 0.12129E+01_JPRB, 0.11625E+01_JPRB, 0.11936E+01_JPRB, 0.12592E+01_JPRB, 0.12938E+01_JPRB, 0.12132E+01_JPRB ,&
&  0.11744E+01_JPRB, 0.11421E+01_JPRB, 0.10984E+01_JPRB, 0.10317E+01_JPRB, 0.10487E+01_JPRB, 0.99313E+00_JPRB, 0.11014E+01_JPRB ,&
&  0.70544E+00_JPRB, 0.57800E+00_JPRB, 0.59203E+00_JPRB, 0.60372E+00_JPRB, 0.67434E+00_JPRB, 0.72458E+00_JPRB, 0.89341E+00_JPRB ,&
&  0.10314E+01_JPRB, 0.10828E+01_JPRB, 0.10146E+01_JPRB, 0.83618E+00_JPRB, 0.69732E+00_JPRB, 0.64295E+00_JPRB, 0.64195E+00_JPRB ,&
&  0.64284E+00_JPRB, 0.63985E+00_JPRB, 0.64802E+00_JPRB, 0.66378E+00_JPRB, 0.67794E+00_JPRB /)
ZBURD(:, 38,  6) = (/ &
&  0.66437E+00_JPRB, 0.68403E+00_JPRB, 0.70582E+00_JPRB, 0.72387E+00_JPRB, 0.74753E+00_JPRB, 0.78193E+00_JPRB, 0.81627E+00_JPRB ,&
&  0.84532E+00_JPRB, 0.86666E+00_JPRB, 0.88216E+00_JPRB, 0.90526E+00_JPRB, 0.93507E+00_JPRB, 0.97565E+00_JPRB, 0.10280E+01_JPRB ,&
&  0.10516E+01_JPRB, 0.10971E+01_JPRB, 0.11229E+01_JPRB, 0.11712E+01_JPRB, 0.11943E+01_JPRB, 0.12526E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.13176E+01_JPRB, 0.13283E+01_JPRB, 0.13405E+01_JPRB, 0.13686E+01_JPRB, 0.13536E+01_JPRB, 0.14030E+01_JPRB, 0.11163E+01_JPRB ,&
&  0.11816E+01_JPRB, 0.11288E+01_JPRB, 0.10368E+01_JPRB, 0.99397E+00_JPRB, 0.95648E+00_JPRB, 0.92756E+00_JPRB, 0.91377E+00_JPRB ,&
&  0.90309E+00_JPRB, 0.88985E+00_JPRB, 0.85234E+00_JPRB, 0.85712E+00_JPRB, 0.90702E+00_JPRB, 0.10047E+01_JPRB, 0.10987E+01_JPRB ,&
&  0.12057E+01_JPRB, 0.13072E+01_JPRB, 0.14011E+01_JPRB, 0.15191E+01_JPRB, 0.16748E+01_JPRB, 0.18354E+01_JPRB, 0.19884E+01_JPRB ,&
&  0.20304E+01_JPRB, 0.20509E+01_JPRB, 0.18910E+01_JPRB, 0.18941E+01_JPRB, 0.19528E+01_JPRB, 0.16815E+01_JPRB, 0.12887E+01_JPRB ,&
&  0.90756E+00_JPRB, 0.88801E+00_JPRB, 0.99575E+00_JPRB, 0.11442E+01_JPRB, 0.11115E+01_JPRB, 0.10608E+01_JPRB, 0.97685E+00_JPRB ,&
&  0.90478E+00_JPRB, 0.84414E+00_JPRB, 0.85319E+00_JPRB, 0.91949E+00_JPRB, 0.10078E+01_JPRB, 0.11116E+01_JPRB, 0.11450E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.10492E+01_JPRB, 0.97034E+00_JPRB, 0.90972E+00_JPRB, 0.86020E+00_JPRB, 0.79072E+00_JPRB, 0.87972E+00_JPRB ,&
&  0.71633E+00_JPRB, 0.67934E+00_JPRB, 0.69790E+00_JPRB, 0.54954E+00_JPRB, 0.73005E+00_JPRB, 0.89809E+00_JPRB, 0.11486E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.17146E+01_JPRB, 0.21271E+01_JPRB, 0.15234E+01_JPRB, 0.11331E+01_JPRB, 0.70840E+00_JPRB, 0.63678E+00_JPRB ,&
&  0.63678E+00_JPRB, 0.62415E+00_JPRB, 0.62491E+00_JPRB, 0.63595E+00_JPRB, 0.64811E+00_JPRB /)
ZBURD(:, 39,  6) = (/ &
&  0.62126E+00_JPRB, 0.63460E+00_JPRB, 0.65507E+00_JPRB, 0.65833E+00_JPRB, 0.66768E+00_JPRB, 0.68704E+00_JPRB, 0.70684E+00_JPRB ,&
&  0.72665E+00_JPRB, 0.73906E+00_JPRB, 0.75339E+00_JPRB, 0.77115E+00_JPRB, 0.79312E+00_JPRB, 0.84539E+00_JPRB, 0.92202E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.10459E+01_JPRB, 0.11218E+01_JPRB, 0.11533E+01_JPRB, 0.12073E+01_JPRB, 0.12766E+01_JPRB, 0.13216E+01_JPRB ,&
&  0.13848E+01_JPRB, 0.14161E+01_JPRB, 0.14475E+01_JPRB, 0.15229E+01_JPRB, 0.14831E+01_JPRB, 0.15152E+01_JPRB, 0.11663E+01_JPRB ,&
&  0.12217E+01_JPRB, 0.11992E+01_JPRB, 0.11429E+01_JPRB, 0.11155E+01_JPRB, 0.10799E+01_JPRB, 0.10254E+01_JPRB, 0.98505E+00_JPRB ,&
&  0.96103E+00_JPRB, 0.90297E+00_JPRB, 0.83853E+00_JPRB, 0.85734E+00_JPRB, 0.88409E+00_JPRB, 0.93634E+00_JPRB, 0.10498E+01_JPRB ,&
&  0.11568E+01_JPRB, 0.12547E+01_JPRB, 0.13453E+01_JPRB, 0.14794E+01_JPRB, 0.16613E+01_JPRB, 0.18542E+01_JPRB, 0.20546E+01_JPRB ,&
&  0.21825E+01_JPRB, 0.22785E+01_JPRB, 0.23021E+01_JPRB, 0.19869E+01_JPRB, 0.19120E+01_JPRB, 0.15612E+01_JPRB, 0.11625E+01_JPRB ,&
&  0.93849E+00_JPRB, 0.87605E+00_JPRB, 0.91136E+00_JPRB, 0.97060E+00_JPRB, 0.88868E+00_JPRB, 0.81823E+00_JPRB, 0.73747E+00_JPRB ,&
&  0.68430E+00_JPRB, 0.67480E+00_JPRB, 0.69937E+00_JPRB, 0.76894E+00_JPRB, 0.85112E+00_JPRB, 0.95094E+00_JPRB, 0.99200E+00_JPRB ,&
&  0.97066E+00_JPRB, 0.89908E+00_JPRB, 0.84401E+00_JPRB, 0.81193E+00_JPRB, 0.82186E+00_JPRB, 0.86268E+00_JPRB, 0.78784E+00_JPRB ,&
&  0.74883E+00_JPRB, 0.66063E+00_JPRB, 0.64879E+00_JPRB, 0.70486E+00_JPRB, 0.77763E+00_JPRB, 0.10024E+01_JPRB, 0.14100E+01_JPRB ,&
&  0.20119E+01_JPRB, 0.21413E+01_JPRB, 0.26135E+01_JPRB, 0.34756E+01_JPRB, 0.25578E+01_JPRB, 0.18068E+01_JPRB, 0.57524E+00_JPRB ,&
&  0.59055E+00_JPRB, 0.58770E+00_JPRB, 0.58850E+00_JPRB, 0.58960E+00_JPRB, 0.59976E+00_JPRB /)
ZBURD(:, 40,  6) = (/ &
&  0.52217E+00_JPRB, 0.54531E+00_JPRB, 0.53604E+00_JPRB, 0.53929E+00_JPRB, 0.54620E+00_JPRB, 0.55768E+00_JPRB, 0.58206E+00_JPRB ,&
&  0.58840E+00_JPRB, 0.62164E+00_JPRB, 0.65745E+00_JPRB, 0.68632E+00_JPRB, 0.72615E+00_JPRB, 0.78203E+00_JPRB, 0.85257E+00_JPRB ,&
&  0.92839E+00_JPRB, 0.97368E+00_JPRB, 0.10143E+01_JPRB, 0.10454E+01_JPRB, 0.10993E+01_JPRB, 0.11705E+01_JPRB, 0.12131E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.13591E+01_JPRB, 0.14310E+01_JPRB, 0.15337E+01_JPRB, 0.14670E+01_JPRB, 0.14547E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.11900E+01_JPRB, 0.12075E+01_JPRB, 0.11911E+01_JPRB, 0.11682E+01_JPRB, 0.11201E+01_JPRB, 0.10409E+01_JPRB, 0.98916E+00_JPRB ,&
&  0.94402E+00_JPRB, 0.89564E+00_JPRB, 0.83745E+00_JPRB, 0.80444E+00_JPRB, 0.86759E+00_JPRB, 0.86818E+00_JPRB, 0.95829E+00_JPRB ,&
&  0.10671E+01_JPRB, 0.11853E+01_JPRB, 0.12948E+01_JPRB, 0.14257E+01_JPRB, 0.15995E+01_JPRB, 0.17983E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.21497E+01_JPRB, 0.22733E+01_JPRB, 0.22447E+01_JPRB, 0.18935E+01_JPRB, 0.17129E+01_JPRB, 0.13858E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.90755E+00_JPRB, 0.86558E+00_JPRB, 0.89496E+00_JPRB, 0.89321E+00_JPRB, 0.79710E+00_JPRB, 0.70827E+00_JPRB, 0.63513E+00_JPRB ,&
&  0.58054E+00_JPRB, 0.58050E+00_JPRB, 0.63433E+00_JPRB, 0.70253E+00_JPRB, 0.77088E+00_JPRB, 0.83936E+00_JPRB, 0.84529E+00_JPRB ,&
&  0.81395E+00_JPRB, 0.75321E+00_JPRB, 0.72810E+00_JPRB, 0.76241E+00_JPRB, 0.87500E+00_JPRB, 0.99278E+00_JPRB, 0.10770E+01_JPRB ,&
&  0.81606E+00_JPRB, 0.66084E+00_JPRB, 0.66248E+00_JPRB, 0.69071E+00_JPRB, 0.73440E+00_JPRB, 0.80630E+00_JPRB, 0.91952E+00_JPRB ,&
&  0.10919E+01_JPRB, 0.14576E+01_JPRB, 0.14123E+01_JPRB, 0.11574E+01_JPRB, 0.16608E+01_JPRB, 0.13820E+01_JPRB, 0.52314E+00_JPRB ,&
&  0.50330E+00_JPRB, 0.49412E+00_JPRB, 0.50027E+00_JPRB, 0.50459E+00_JPRB, 0.51778E+00_JPRB /)
ZBURD(:, 41,  6) = (/ &
&  0.48828E+00_JPRB, 0.47703E+00_JPRB, 0.45925E+00_JPRB, 0.46791E+00_JPRB, 0.46927E+00_JPRB, 0.48675E+00_JPRB, 0.50706E+00_JPRB ,&
&  0.53454E+00_JPRB, 0.58669E+00_JPRB, 0.63749E+00_JPRB, 0.67900E+00_JPRB, 0.70507E+00_JPRB, 0.73783E+00_JPRB, 0.76949E+00_JPRB ,&
&  0.79226E+00_JPRB, 0.81451E+00_JPRB, 0.84578E+00_JPRB, 0.89004E+00_JPRB, 0.94766E+00_JPRB, 0.10134E+01_JPRB, 0.10937E+01_JPRB ,&
&  0.11833E+01_JPRB, 0.12802E+01_JPRB, 0.14025E+01_JPRB, 0.14493E+01_JPRB, 0.14874E+01_JPRB, 0.15288E+01_JPRB, 0.15088E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.12592E+01_JPRB, 0.12250E+01_JPRB, 0.11740E+01_JPRB, 0.11340E+01_JPRB, 0.10690E+01_JPRB, 0.10128E+01_JPRB ,&
&  0.97067E+00_JPRB, 0.89610E+00_JPRB, 0.86096E+00_JPRB, 0.79836E+00_JPRB, 0.81957E+00_JPRB, 0.82442E+00_JPRB, 0.87318E+00_JPRB ,&
&  0.95181E+00_JPRB, 0.10725E+01_JPRB, 0.12216E+01_JPRB, 0.13642E+01_JPRB, 0.15044E+01_JPRB, 0.16567E+01_JPRB, 0.18719E+01_JPRB ,&
&  0.20991E+01_JPRB, 0.22462E+01_JPRB, 0.23295E+01_JPRB, 0.20265E+01_JPRB, 0.16701E+01_JPRB, 0.14914E+01_JPRB, 0.11964E+01_JPRB ,&
&  0.91526E+00_JPRB, 0.81607E+00_JPRB, 0.84758E+00_JPRB, 0.85682E+00_JPRB, 0.77504E+00_JPRB, 0.69913E+00_JPRB, 0.62868E+00_JPRB ,&
&  0.56514E+00_JPRB, 0.52788E+00_JPRB, 0.55171E+00_JPRB, 0.63526E+00_JPRB, 0.68189E+00_JPRB, 0.71036E+00_JPRB, 0.70238E+00_JPRB ,&
&  0.68748E+00_JPRB, 0.64941E+00_JPRB, 0.61940E+00_JPRB, 0.63620E+00_JPRB, 0.68580E+00_JPRB, 0.73308E+00_JPRB, 0.73377E+00_JPRB ,&
&  0.71134E+00_JPRB, 0.70064E+00_JPRB, 0.70013E+00_JPRB, 0.71547E+00_JPRB, 0.72614E+00_JPRB, 0.74269E+00_JPRB, 0.75757E+00_JPRB ,&
&  0.77892E+00_JPRB, 0.81604E+00_JPRB, 0.79726E+00_JPRB, 0.77421E+00_JPRB, 0.84539E+00_JPRB, 0.89340E+00_JPRB, 0.58189E+00_JPRB ,&
&  0.49856E+00_JPRB, 0.46553E+00_JPRB, 0.46627E+00_JPRB, 0.46093E+00_JPRB, 0.49645E+00_JPRB /)
ZBURD(:, 42,  6) = (/ &
&  0.49357E+00_JPRB, 0.47577E+00_JPRB, 0.46548E+00_JPRB, 0.45675E+00_JPRB, 0.44329E+00_JPRB, 0.46633E+00_JPRB, 0.47185E+00_JPRB ,&
&  0.49589E+00_JPRB, 0.53552E+00_JPRB, 0.58392E+00_JPRB, 0.61206E+00_JPRB, 0.62088E+00_JPRB, 0.64771E+00_JPRB, 0.67051E+00_JPRB ,&
&  0.68488E+00_JPRB, 0.69544E+00_JPRB, 0.71148E+00_JPRB, 0.73256E+00_JPRB, 0.76986E+00_JPRB, 0.82674E+00_JPRB, 0.91162E+00_JPRB ,&
&  0.10096E+01_JPRB, 0.11154E+01_JPRB, 0.12291E+01_JPRB, 0.13359E+01_JPRB, 0.14090E+01_JPRB, 0.14675E+01_JPRB, 0.14335E+01_JPRB ,&
&  0.85569E+00_JPRB, 0.11571E+01_JPRB, 0.13244E+01_JPRB, 0.12369E+01_JPRB, 0.11602E+01_JPRB, 0.10911E+01_JPRB, 0.10246E+01_JPRB ,&
&  0.96000E+00_JPRB, 0.88988E+00_JPRB, 0.86253E+00_JPRB, 0.86804E+00_JPRB, 0.78671E+00_JPRB, 0.78400E+00_JPRB, 0.80655E+00_JPRB ,&
&  0.85183E+00_JPRB, 0.93645E+00_JPRB, 0.10969E+01_JPRB, 0.12733E+01_JPRB, 0.13714E+01_JPRB, 0.14682E+01_JPRB, 0.16508E+01_JPRB ,&
&  0.19165E+01_JPRB, 0.21119E+01_JPRB, 0.22464E+01_JPRB, 0.17399E+01_JPRB, 0.15246E+01_JPRB, 0.14881E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.98990E+00_JPRB, 0.82587E+00_JPRB, 0.84142E+00_JPRB, 0.85639E+00_JPRB, 0.78039E+00_JPRB, 0.73056E+00_JPRB, 0.67259E+00_JPRB ,&
&  0.60065E+00_JPRB, 0.54533E+00_JPRB, 0.50790E+00_JPRB, 0.53922E+00_JPRB, 0.57257E+00_JPRB, 0.59237E+00_JPRB, 0.57392E+00_JPRB ,&
&  0.60532E+00_JPRB, 0.59725E+00_JPRB, 0.60002E+00_JPRB, 0.61857E+00_JPRB, 0.68983E+00_JPRB, 0.72482E+00_JPRB, 0.74241E+00_JPRB ,&
&  0.75913E+00_JPRB, 0.77133E+00_JPRB, 0.77228E+00_JPRB, 0.77781E+00_JPRB, 0.77899E+00_JPRB, 0.77279E+00_JPRB, 0.75531E+00_JPRB ,&
&  0.72846E+00_JPRB, 0.71490E+00_JPRB, 0.69105E+00_JPRB, 0.66574E+00_JPRB, 0.62486E+00_JPRB, 0.63208E+00_JPRB, 0.56514E+00_JPRB ,&
&  0.51960E+00_JPRB, 0.48780E+00_JPRB, 0.46628E+00_JPRB, 0.46175E+00_JPRB, 0.47095E+00_JPRB /)
ZBURD(:, 43,  6) = (/ &
&  0.50491E+00_JPRB, 0.50037E+00_JPRB, 0.49414E+00_JPRB, 0.48313E+00_JPRB, 0.47012E+00_JPRB, 0.46903E+00_JPRB, 0.45646E+00_JPRB ,&
&  0.45339E+00_JPRB, 0.47419E+00_JPRB, 0.50653E+00_JPRB, 0.53339E+00_JPRB, 0.55737E+00_JPRB, 0.57104E+00_JPRB, 0.59149E+00_JPRB ,&
&  0.59502E+00_JPRB, 0.59380E+00_JPRB, 0.59138E+00_JPRB, 0.59081E+00_JPRB, 0.60000E+00_JPRB, 0.63123E+00_JPRB, 0.68607E+00_JPRB ,&
&  0.76379E+00_JPRB, 0.86712E+00_JPRB, 0.99419E+00_JPRB, 0.11325E+01_JPRB, 0.12689E+01_JPRB, 0.14020E+01_JPRB, 0.14634E+01_JPRB ,&
&  0.99400E+00_JPRB, 0.56385E+00_JPRB, 0.12109E+01_JPRB, 0.13509E+01_JPRB, 0.12492E+01_JPRB, 0.11889E+01_JPRB, 0.10751E+01_JPRB ,&
&  0.96776E+00_JPRB, 0.93267E+00_JPRB, 0.88037E+00_JPRB, 0.84523E+00_JPRB, 0.77105E+00_JPRB, 0.75820E+00_JPRB, 0.75569E+00_JPRB ,&
&  0.77453E+00_JPRB, 0.82703E+00_JPRB, 0.94145E+00_JPRB, 0.11316E+01_JPRB, 0.12582E+01_JPRB, 0.13094E+01_JPRB, 0.14315E+01_JPRB ,&
&  0.16651E+01_JPRB, 0.18522E+01_JPRB, 0.19182E+01_JPRB, 0.13751E+01_JPRB, 0.10680E+01_JPRB, 0.99581E+00_JPRB, 0.95004E+00_JPRB ,&
&  0.93139E+00_JPRB, 0.84539E+00_JPRB, 0.84246E+00_JPRB, 0.86068E+00_JPRB, 0.81276E+00_JPRB, 0.74533E+00_JPRB, 0.74083E+00_JPRB ,&
&  0.69158E+00_JPRB, 0.63514E+00_JPRB, 0.58152E+00_JPRB, 0.55531E+00_JPRB, 0.54354E+00_JPRB, 0.55587E+00_JPRB, 0.55720E+00_JPRB ,&
&  0.56264E+00_JPRB, 0.58383E+00_JPRB, 0.59683E+00_JPRB, 0.61682E+00_JPRB, 0.66316E+00_JPRB, 0.71105E+00_JPRB, 0.74755E+00_JPRB ,&
&  0.77210E+00_JPRB, 0.80230E+00_JPRB, 0.82280E+00_JPRB, 0.83390E+00_JPRB, 0.83565E+00_JPRB, 0.81020E+00_JPRB, 0.80345E+00_JPRB ,&
&  0.79661E+00_JPRB, 0.77691E+00_JPRB, 0.74002E+00_JPRB, 0.72076E+00_JPRB, 0.66594E+00_JPRB, 0.62687E+00_JPRB, 0.56129E+00_JPRB ,&
&  0.53561E+00_JPRB, 0.49667E+00_JPRB, 0.46998E+00_JPRB, 0.46463E+00_JPRB, 0.47609E+00_JPRB /)
ZBURD(:, 44,  6) = (/ &
&  0.50970E+00_JPRB, 0.50919E+00_JPRB, 0.50933E+00_JPRB, 0.51143E+00_JPRB, 0.50595E+00_JPRB, 0.50720E+00_JPRB, 0.49121E+00_JPRB ,&
&  0.46491E+00_JPRB, 0.45531E+00_JPRB, 0.46870E+00_JPRB, 0.47310E+00_JPRB, 0.47499E+00_JPRB, 0.49332E+00_JPRB, 0.51329E+00_JPRB ,&
&  0.52173E+00_JPRB, 0.52370E+00_JPRB, 0.52157E+00_JPRB, 0.52052E+00_JPRB, 0.51399E+00_JPRB, 0.51681E+00_JPRB, 0.53766E+00_JPRB ,&
&  0.57259E+00_JPRB, 0.63614E+00_JPRB, 0.73068E+00_JPRB, 0.84920E+00_JPRB, 0.98289E+00_JPRB, 0.11675E+01_JPRB, 0.13352E+01_JPRB ,&
&  0.16468E+01_JPRB, 0.11346E+01_JPRB, 0.71250E+00_JPRB, 0.12160E+01_JPRB, 0.13713E+01_JPRB, 0.13639E+01_JPRB, 0.11636E+01_JPRB ,&
&  0.10984E+01_JPRB, 0.10536E+01_JPRB, 0.10118E+01_JPRB, 0.93885E+00_JPRB, 0.81928E+00_JPRB, 0.76913E+00_JPRB, 0.72399E+00_JPRB ,&
&  0.70289E+00_JPRB, 0.72257E+00_JPRB, 0.78913E+00_JPRB, 0.92703E+00_JPRB, 0.10992E+01_JPRB, 0.11870E+01_JPRB, 0.12436E+01_JPRB ,&
&  0.14076E+01_JPRB, 0.16026E+01_JPRB, 0.16370E+01_JPRB, 0.11810E+01_JPRB, 0.96461E+00_JPRB, 0.91196E+00_JPRB, 0.90239E+00_JPRB ,&
&  0.88056E+00_JPRB, 0.90166E+00_JPRB, 0.92515E+00_JPRB, 0.87758E+00_JPRB, 0.81686E+00_JPRB, 0.77090E+00_JPRB, 0.78166E+00_JPRB ,&
&  0.74179E+00_JPRB, 0.70364E+00_JPRB, 0.66418E+00_JPRB, 0.61435E+00_JPRB, 0.56714E+00_JPRB, 0.56317E+00_JPRB, 0.55993E+00_JPRB ,&
&  0.56846E+00_JPRB, 0.58939E+00_JPRB, 0.59632E+00_JPRB, 0.61895E+00_JPRB, 0.64531E+00_JPRB, 0.67844E+00_JPRB, 0.71407E+00_JPRB ,&
&  0.74412E+00_JPRB, 0.77950E+00_JPRB, 0.81229E+00_JPRB, 0.82298E+00_JPRB, 0.81461E+00_JPRB, 0.79737E+00_JPRB, 0.82194E+00_JPRB ,&
&  0.82628E+00_JPRB, 0.82963E+00_JPRB, 0.78811E+00_JPRB, 0.79200E+00_JPRB, 0.73598E+00_JPRB, 0.68675E+00_JPRB, 0.62917E+00_JPRB ,&
&  0.61888E+00_JPRB, 0.58122E+00_JPRB, 0.54742E+00_JPRB, 0.50303E+00_JPRB, 0.50335E+00_JPRB /)
ZBURD(:, 45,  6) = (/ &
&  0.52947E+00_JPRB, 0.52149E+00_JPRB, 0.52017E+00_JPRB, 0.52096E+00_JPRB, 0.51831E+00_JPRB, 0.51912E+00_JPRB, 0.52057E+00_JPRB ,&
&  0.49596E+00_JPRB, 0.47173E+00_JPRB, 0.45401E+00_JPRB, 0.44537E+00_JPRB, 0.44206E+00_JPRB, 0.45497E+00_JPRB, 0.46088E+00_JPRB ,&
&  0.45844E+00_JPRB, 0.45789E+00_JPRB, 0.47364E+00_JPRB, 0.47775E+00_JPRB, 0.47739E+00_JPRB, 0.47353E+00_JPRB, 0.47381E+00_JPRB ,&
&  0.48554E+00_JPRB, 0.51069E+00_JPRB, 0.56029E+00_JPRB, 0.63022E+00_JPRB, 0.70595E+00_JPRB, 0.83305E+00_JPRB, 0.98761E+00_JPRB ,&
&  0.12528E+01_JPRB, 0.12796E+01_JPRB, 0.64229E+00_JPRB, 0.10668E+01_JPRB, 0.14235E+01_JPRB, 0.14637E+01_JPRB, 0.13634E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.11929E+01_JPRB, 0.12427E+01_JPRB, 0.11602E+01_JPRB, 0.94455E+00_JPRB, 0.81198E+00_JPRB, 0.71838E+00_JPRB ,&
&  0.66494E+00_JPRB, 0.64904E+00_JPRB, 0.66965E+00_JPRB, 0.74178E+00_JPRB, 0.88292E+00_JPRB, 0.10155E+01_JPRB, 0.10819E+01_JPRB ,&
&  0.11627E+01_JPRB, 0.13560E+01_JPRB, 0.14775E+01_JPRB, 0.11089E+01_JPRB, 0.95496E+00_JPRB, 0.97254E+00_JPRB, 0.96678E+00_JPRB ,&
&  0.94178E+00_JPRB, 0.95324E+00_JPRB, 0.95253E+00_JPRB, 0.89532E+00_JPRB, 0.82861E+00_JPRB, 0.80153E+00_JPRB, 0.81251E+00_JPRB ,&
&  0.77366E+00_JPRB, 0.74079E+00_JPRB, 0.71057E+00_JPRB, 0.67155E+00_JPRB, 0.62087E+00_JPRB, 0.57955E+00_JPRB, 0.58330E+00_JPRB ,&
&  0.58963E+00_JPRB, 0.60634E+00_JPRB, 0.61169E+00_JPRB, 0.62834E+00_JPRB, 0.64337E+00_JPRB, 0.66288E+00_JPRB, 0.68418E+00_JPRB ,&
&  0.71319E+00_JPRB, 0.74276E+00_JPRB, 0.76969E+00_JPRB, 0.76466E+00_JPRB, 0.76173E+00_JPRB, 0.77032E+00_JPRB, 0.78477E+00_JPRB ,&
&  0.81333E+00_JPRB, 0.83888E+00_JPRB, 0.78632E+00_JPRB, 0.79085E+00_JPRB, 0.80680E+00_JPRB, 0.76602E+00_JPRB, 0.71911E+00_JPRB ,&
&  0.75369E+00_JPRB, 0.78888E+00_JPRB, 0.72754E+00_JPRB, 0.57847E+00_JPRB, 0.54727E+00_JPRB /)
ZBURD(:, 46,  6) = (/ &
&  0.56239E+00_JPRB, 0.55002E+00_JPRB, 0.54067E+00_JPRB, 0.53796E+00_JPRB, 0.53283E+00_JPRB, 0.52915E+00_JPRB, 0.53488E+00_JPRB ,&
&  0.52583E+00_JPRB, 0.51007E+00_JPRB, 0.48836E+00_JPRB, 0.46607E+00_JPRB, 0.44846E+00_JPRB, 0.44991E+00_JPRB, 0.45546E+00_JPRB ,&
&  0.45606E+00_JPRB, 0.45316E+00_JPRB, 0.45243E+00_JPRB, 0.45556E+00_JPRB, 0.45416E+00_JPRB, 0.45155E+00_JPRB, 0.44871E+00_JPRB ,&
&  0.44830E+00_JPRB, 0.45429E+00_JPRB, 0.47439E+00_JPRB, 0.50910E+00_JPRB, 0.55115E+00_JPRB, 0.61806E+00_JPRB, 0.71253E+00_JPRB ,&
&  0.92269E+00_JPRB, 0.10549E+01_JPRB, 0.11466E+01_JPRB, 0.17149E+01_JPRB, 0.18019E+01_JPRB, 0.16788E+01_JPRB, 0.16258E+01_JPRB ,&
&  0.15607E+01_JPRB, 0.14782E+01_JPRB, 0.15863E+01_JPRB, 0.13083E+01_JPRB, 0.10710E+01_JPRB, 0.87380E+00_JPRB, 0.74424E+00_JPRB ,&
&  0.67209E+00_JPRB, 0.62554E+00_JPRB, 0.61429E+00_JPRB, 0.63144E+00_JPRB, 0.70287E+00_JPRB, 0.82665E+00_JPRB, 0.92635E+00_JPRB ,&
&  0.10007E+01_JPRB, 0.11353E+01_JPRB, 0.13255E+01_JPRB, 0.11518E+01_JPRB, 0.98502E+00_JPRB, 0.10838E+01_JPRB, 0.11771E+01_JPRB ,&
&  0.12905E+01_JPRB, 0.10816E+01_JPRB, 0.99710E+00_JPRB, 0.92848E+00_JPRB, 0.84612E+00_JPRB, 0.89518E+00_JPRB, 0.83737E+00_JPRB ,&
&  0.79786E+00_JPRB, 0.76682E+00_JPRB, 0.73621E+00_JPRB, 0.70826E+00_JPRB, 0.67451E+00_JPRB, 0.63568E+00_JPRB, 0.61654E+00_JPRB ,&
&  0.61978E+00_JPRB, 0.62898E+00_JPRB, 0.63076E+00_JPRB, 0.63789E+00_JPRB, 0.63980E+00_JPRB, 0.65323E+00_JPRB, 0.67033E+00_JPRB ,&
&  0.69939E+00_JPRB, 0.71787E+00_JPRB, 0.69360E+00_JPRB, 0.68371E+00_JPRB, 0.68234E+00_JPRB, 0.68474E+00_JPRB, 0.68407E+00_JPRB ,&
&  0.70692E+00_JPRB, 0.75777E+00_JPRB, 0.78445E+00_JPRB, 0.80717E+00_JPRB, 0.81934E+00_JPRB, 0.78689E+00_JPRB, 0.71316E+00_JPRB ,&
&  0.68908E+00_JPRB, 0.69094E+00_JPRB, 0.68460E+00_JPRB, 0.61714E+00_JPRB, 0.58531E+00_JPRB /)
ZBURD(:, 47,  6) = (/ &
&  0.57645E+00_JPRB, 0.56430E+00_JPRB, 0.55659E+00_JPRB, 0.54818E+00_JPRB, 0.53861E+00_JPRB, 0.53511E+00_JPRB, 0.53811E+00_JPRB ,&
&  0.53564E+00_JPRB, 0.52523E+00_JPRB, 0.51064E+00_JPRB, 0.49624E+00_JPRB, 0.47601E+00_JPRB, 0.46447E+00_JPRB, 0.45761E+00_JPRB ,&
&  0.45259E+00_JPRB, 0.45159E+00_JPRB, 0.44679E+00_JPRB, 0.44401E+00_JPRB, 0.44121E+00_JPRB, 0.43320E+00_JPRB, 0.42925E+00_JPRB ,&
&  0.42644E+00_JPRB, 0.43003E+00_JPRB, 0.43939E+00_JPRB, 0.45801E+00_JPRB, 0.48586E+00_JPRB, 0.53002E+00_JPRB, 0.58540E+00_JPRB ,&
&  0.72248E+00_JPRB, 0.93365E+00_JPRB, 0.91899E+00_JPRB, 0.18321E+01_JPRB, 0.18044E+01_JPRB, 0.17493E+01_JPRB, 0.16331E+01_JPRB ,&
&  0.16184E+01_JPRB, 0.17788E+01_JPRB, 0.16067E+01_JPRB, 0.13623E+01_JPRB, 0.11039E+01_JPRB, 0.91673E+00_JPRB, 0.77651E+00_JPRB ,&
&  0.68990E+00_JPRB, 0.63772E+00_JPRB, 0.61288E+00_JPRB, 0.60222E+00_JPRB, 0.60489E+00_JPRB, 0.67043E+00_JPRB, 0.77960E+00_JPRB ,&
&  0.86467E+00_JPRB, 0.96688E+00_JPRB, 0.11473E+01_JPRB, 0.10968E+01_JPRB, 0.10333E+01_JPRB, 0.11610E+01_JPRB, 0.13998E+01_JPRB ,&
&  0.16309E+01_JPRB, 0.13593E+01_JPRB, 0.11076E+01_JPRB, 0.99352E+00_JPRB, 0.92508E+00_JPRB, 0.91493E+00_JPRB, 0.86362E+00_JPRB ,&
&  0.80834E+00_JPRB, 0.77752E+00_JPRB, 0.75194E+00_JPRB, 0.72928E+00_JPRB, 0.70221E+00_JPRB, 0.67684E+00_JPRB, 0.64868E+00_JPRB ,&
&  0.64125E+00_JPRB, 0.64148E+00_JPRB, 0.64221E+00_JPRB, 0.65011E+00_JPRB, 0.64992E+00_JPRB, 0.65782E+00_JPRB, 0.66980E+00_JPRB ,&
&  0.69154E+00_JPRB, 0.70368E+00_JPRB, 0.66428E+00_JPRB, 0.63621E+00_JPRB, 0.63031E+00_JPRB, 0.62640E+00_JPRB, 0.63018E+00_JPRB ,&
&  0.66638E+00_JPRB, 0.71353E+00_JPRB, 0.74838E+00_JPRB, 0.76617E+00_JPRB, 0.83293E+00_JPRB, 0.84589E+00_JPRB, 0.73734E+00_JPRB ,&
&  0.69406E+00_JPRB, 0.66374E+00_JPRB, 0.63903E+00_JPRB, 0.60792E+00_JPRB, 0.59279E+00_JPRB /)
ZBURD(:, 48,  6) = (/ &
&  0.58187E+00_JPRB, 0.57212E+00_JPRB, 0.56382E+00_JPRB, 0.55283E+00_JPRB, 0.53829E+00_JPRB, 0.52937E+00_JPRB, 0.53007E+00_JPRB ,&
&  0.53371E+00_JPRB, 0.52711E+00_JPRB, 0.51801E+00_JPRB, 0.50472E+00_JPRB, 0.48536E+00_JPRB, 0.47479E+00_JPRB, 0.46702E+00_JPRB ,&
&  0.45548E+00_JPRB, 0.44626E+00_JPRB, 0.43309E+00_JPRB, 0.42692E+00_JPRB, 0.42535E+00_JPRB, 0.41993E+00_JPRB, 0.40967E+00_JPRB ,&
&  0.41181E+00_JPRB, 0.41962E+00_JPRB, 0.42939E+00_JPRB, 0.43986E+00_JPRB, 0.46085E+00_JPRB, 0.49527E+00_JPRB, 0.54712E+00_JPRB ,&
&  0.64920E+00_JPRB, 0.10430E+01_JPRB, 0.85078E+00_JPRB, 0.14292E+01_JPRB, 0.14624E+01_JPRB, 0.15570E+01_JPRB, 0.15798E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.16493E+01_JPRB, 0.15480E+01_JPRB, 0.12905E+01_JPRB, 0.10407E+01_JPRB, 0.88313E+00_JPRB, 0.78390E+00_JPRB ,&
&  0.71861E+00_JPRB, 0.66894E+00_JPRB, 0.63322E+00_JPRB, 0.60895E+00_JPRB, 0.59154E+00_JPRB, 0.59376E+00_JPRB, 0.64732E+00_JPRB ,&
&  0.74119E+00_JPRB, 0.82962E+00_JPRB, 0.96925E+00_JPRB, 0.11101E+01_JPRB, 0.10285E+01_JPRB, 0.11140E+01_JPRB, 0.14607E+01_JPRB ,&
&  0.17046E+01_JPRB, 0.17542E+01_JPRB, 0.13054E+01_JPRB, 0.11056E+01_JPRB, 0.10215E+01_JPRB, 0.96480E+00_JPRB, 0.88127E+00_JPRB ,&
&  0.80538E+00_JPRB, 0.76880E+00_JPRB, 0.74731E+00_JPRB, 0.73171E+00_JPRB, 0.71270E+00_JPRB, 0.68601E+00_JPRB, 0.65895E+00_JPRB ,&
&  0.63665E+00_JPRB, 0.63518E+00_JPRB, 0.63854E+00_JPRB, 0.64280E+00_JPRB, 0.64912E+00_JPRB, 0.65428E+00_JPRB, 0.66268E+00_JPRB ,&
&  0.67546E+00_JPRB, 0.69004E+00_JPRB, 0.66535E+00_JPRB, 0.63637E+00_JPRB, 0.62187E+00_JPRB, 0.61260E+00_JPRB, 0.60889E+00_JPRB ,&
&  0.63634E+00_JPRB, 0.67138E+00_JPRB, 0.71190E+00_JPRB, 0.77502E+00_JPRB, 0.85105E+00_JPRB, 0.92164E+00_JPRB, 0.73675E+00_JPRB ,&
&  0.71301E+00_JPRB, 0.67690E+00_JPRB, 0.64653E+00_JPRB, 0.61450E+00_JPRB, 0.59776E+00_JPRB /)
ZBURD(:, 49,  6) = (/ &
&  0.58470E+00_JPRB, 0.57541E+00_JPRB, 0.56660E+00_JPRB, 0.55707E+00_JPRB, 0.54524E+00_JPRB, 0.53372E+00_JPRB, 0.52646E+00_JPRB ,&
&  0.52365E+00_JPRB, 0.52441E+00_JPRB, 0.51926E+00_JPRB, 0.50523E+00_JPRB, 0.48893E+00_JPRB, 0.47373E+00_JPRB, 0.46496E+00_JPRB ,&
&  0.45677E+00_JPRB, 0.44560E+00_JPRB, 0.43180E+00_JPRB, 0.41913E+00_JPRB, 0.41538E+00_JPRB, 0.41663E+00_JPRB, 0.41118E+00_JPRB ,&
&  0.41678E+00_JPRB, 0.42445E+00_JPRB, 0.43171E+00_JPRB, 0.43882E+00_JPRB, 0.45658E+00_JPRB, 0.48717E+00_JPRB, 0.53578E+00_JPRB ,&
&  0.62840E+00_JPRB, 0.54434E+00_JPRB, 0.93570E+00_JPRB, 0.11024E+01_JPRB, 0.11945E+01_JPRB, 0.12787E+01_JPRB, 0.14546E+01_JPRB ,&
&  0.15957E+01_JPRB, 0.15452E+01_JPRB, 0.14013E+01_JPRB, 0.11935E+01_JPRB, 0.10058E+01_JPRB, 0.86917E+00_JPRB, 0.78049E+00_JPRB ,&
&  0.72525E+00_JPRB, 0.68326E+00_JPRB, 0.64607E+00_JPRB, 0.61424E+00_JPRB, 0.59268E+00_JPRB, 0.58066E+00_JPRB, 0.58428E+00_JPRB ,&
&  0.61967E+00_JPRB, 0.68860E+00_JPRB, 0.78276E+00_JPRB, 0.94820E+00_JPRB, 0.10142E+01_JPRB, 0.10630E+01_JPRB, 0.11931E+01_JPRB ,&
&  0.15128E+01_JPRB, 0.17170E+01_JPRB, 0.14505E+01_JPRB, 0.12243E+01_JPRB, 0.11067E+01_JPRB, 0.98849E+00_JPRB, 0.88334E+00_JPRB ,&
&  0.80323E+00_JPRB, 0.75563E+00_JPRB, 0.72461E+00_JPRB, 0.70659E+00_JPRB, 0.69747E+00_JPRB, 0.68469E+00_JPRB, 0.66502E+00_JPRB ,&
&  0.64848E+00_JPRB, 0.63873E+00_JPRB, 0.63639E+00_JPRB, 0.63872E+00_JPRB, 0.64091E+00_JPRB, 0.64629E+00_JPRB, 0.64882E+00_JPRB ,&
&  0.64990E+00_JPRB, 0.65749E+00_JPRB, 0.66627E+00_JPRB, 0.64431E+00_JPRB, 0.63264E+00_JPRB, 0.61780E+00_JPRB, 0.60850E+00_JPRB ,&
&  0.63216E+00_JPRB, 0.64691E+00_JPRB, 0.67614E+00_JPRB, 0.75407E+00_JPRB, 0.81053E+00_JPRB, 0.84997E+00_JPRB, 0.73195E+00_JPRB ,&
&  0.70179E+00_JPRB, 0.66841E+00_JPRB, 0.64021E+00_JPRB, 0.61328E+00_JPRB, 0.60070E+00_JPRB /)
ZBURD(:, 50,  6) = (/ &
&  0.59235E+00_JPRB, 0.58266E+00_JPRB, 0.57106E+00_JPRB, 0.56015E+00_JPRB, 0.54726E+00_JPRB, 0.53371E+00_JPRB, 0.52375E+00_JPRB ,&
&  0.51745E+00_JPRB, 0.51577E+00_JPRB, 0.51571E+00_JPRB, 0.50478E+00_JPRB, 0.49029E+00_JPRB, 0.47712E+00_JPRB, 0.46900E+00_JPRB ,&
&  0.46200E+00_JPRB, 0.45353E+00_JPRB, 0.44329E+00_JPRB, 0.43160E+00_JPRB, 0.42608E+00_JPRB, 0.42334E+00_JPRB, 0.42164E+00_JPRB ,&
&  0.42676E+00_JPRB, 0.43152E+00_JPRB, 0.43782E+00_JPRB, 0.44660E+00_JPRB, 0.46292E+00_JPRB, 0.49094E+00_JPRB, 0.53696E+00_JPRB ,&
&  0.62966E+00_JPRB, 0.83333E+00_JPRB, 0.10395E+01_JPRB, 0.96790E+00_JPRB, 0.97720E+00_JPRB, 0.10145E+01_JPRB, 0.11784E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.13405E+01_JPRB, 0.12318E+01_JPRB, 0.10849E+01_JPRB, 0.95456E+00_JPRB, 0.85263E+00_JPRB, 0.77965E+00_JPRB ,&
&  0.71337E+00_JPRB, 0.65533E+00_JPRB, 0.61396E+00_JPRB, 0.58515E+00_JPRB, 0.56634E+00_JPRB, 0.55485E+00_JPRB, 0.55057E+00_JPRB ,&
&  0.55743E+00_JPRB, 0.58272E+00_JPRB, 0.64013E+00_JPRB, 0.75451E+00_JPRB, 0.97934E+00_JPRB, 0.97059E+00_JPRB, 0.11365E+01_JPRB ,&
&  0.14632E+01_JPRB, 0.14455E+01_JPRB, 0.13466E+01_JPRB, 0.12172E+01_JPRB, 0.10938E+01_JPRB, 0.98031E+00_JPRB, 0.88135E+00_JPRB ,&
&  0.81504E+00_JPRB, 0.76578E+00_JPRB, 0.72146E+00_JPRB, 0.68961E+00_JPRB, 0.67049E+00_JPRB, 0.66396E+00_JPRB, 0.65932E+00_JPRB ,&
&  0.65188E+00_JPRB, 0.64304E+00_JPRB, 0.63699E+00_JPRB, 0.63507E+00_JPRB, 0.63375E+00_JPRB, 0.63386E+00_JPRB, 0.62374E+00_JPRB ,&
&  0.61549E+00_JPRB, 0.61574E+00_JPRB, 0.66880E+00_JPRB, 0.64388E+00_JPRB, 0.63304E+00_JPRB, 0.62233E+00_JPRB, 0.60733E+00_JPRB ,&
&  0.60070E+00_JPRB, 0.63545E+00_JPRB, 0.66243E+00_JPRB, 0.72935E+00_JPRB, 0.87350E+00_JPRB, 0.89552E+00_JPRB, 0.75870E+00_JPRB ,&
&  0.70703E+00_JPRB, 0.66093E+00_JPRB, 0.62236E+00_JPRB, 0.60359E+00_JPRB, 0.60533E+00_JPRB /)
ZBURD(:, 51,  6) = (/ &
&  0.59903E+00_JPRB, 0.58500E+00_JPRB, 0.57122E+00_JPRB, 0.55923E+00_JPRB, 0.54417E+00_JPRB, 0.53036E+00_JPRB, 0.51735E+00_JPRB ,&
&  0.50761E+00_JPRB, 0.50216E+00_JPRB, 0.50189E+00_JPRB, 0.50225E+00_JPRB, 0.49963E+00_JPRB, 0.49159E+00_JPRB, 0.47755E+00_JPRB ,&
&  0.46760E+00_JPRB, 0.45889E+00_JPRB, 0.44864E+00_JPRB, 0.43643E+00_JPRB, 0.42508E+00_JPRB, 0.42017E+00_JPRB, 0.42002E+00_JPRB ,&
&  0.42434E+00_JPRB, 0.43002E+00_JPRB, 0.43402E+00_JPRB, 0.44556E+00_JPRB, 0.46307E+00_JPRB, 0.49086E+00_JPRB, 0.53079E+00_JPRB ,&
&  0.59886E+00_JPRB, 0.81425E+00_JPRB, 0.86483E+00_JPRB, 0.85210E+00_JPRB, 0.86012E+00_JPRB, 0.87861E+00_JPRB, 0.96578E+00_JPRB ,&
&  0.10506E+01_JPRB, 0.11051E+01_JPRB, 0.10740E+01_JPRB, 0.96488E+00_JPRB, 0.87328E+00_JPRB, 0.80878E+00_JPRB, 0.74586E+00_JPRB ,&
&  0.68569E+00_JPRB, 0.63831E+00_JPRB, 0.60085E+00_JPRB, 0.57537E+00_JPRB, 0.55617E+00_JPRB, 0.54199E+00_JPRB, 0.53432E+00_JPRB ,&
&  0.53348E+00_JPRB, 0.53831E+00_JPRB, 0.55696E+00_JPRB, 0.60006E+00_JPRB, 0.77096E+00_JPRB, 0.91786E+00_JPRB, 0.10512E+01_JPRB ,&
&  0.10956E+01_JPRB, 0.11281E+01_JPRB, 0.11403E+01_JPRB, 0.10925E+01_JPRB, 0.10134E+01_JPRB, 0.91797E+00_JPRB, 0.84523E+00_JPRB ,&
&  0.79951E+00_JPRB, 0.75309E+00_JPRB, 0.70836E+00_JPRB, 0.66745E+00_JPRB, 0.63952E+00_JPRB, 0.62554E+00_JPRB, 0.61865E+00_JPRB ,&
&  0.61138E+00_JPRB, 0.60820E+00_JPRB, 0.60640E+00_JPRB, 0.60330E+00_JPRB, 0.59853E+00_JPRB, 0.59169E+00_JPRB, 0.58188E+00_JPRB ,&
&  0.57087E+00_JPRB, 0.56234E+00_JPRB, 0.58143E+00_JPRB, 0.58704E+00_JPRB, 0.58765E+00_JPRB, 0.58272E+00_JPRB, 0.57451E+00_JPRB ,&
&  0.58116E+00_JPRB, 0.66517E+00_JPRB, 0.68131E+00_JPRB, 0.75942E+00_JPRB, 0.84936E+00_JPRB, 0.83410E+00_JPRB, 0.75790E+00_JPRB ,&
&  0.69761E+00_JPRB, 0.64661E+00_JPRB, 0.60674E+00_JPRB, 0.59089E+00_JPRB, 0.60808E+00_JPRB /)
ZBURD(:, 52,  6) = (/ &
&  0.61412E+00_JPRB, 0.59290E+00_JPRB, 0.57198E+00_JPRB, 0.54890E+00_JPRB, 0.52896E+00_JPRB, 0.51469E+00_JPRB, 0.50417E+00_JPRB ,&
&  0.49718E+00_JPRB, 0.49267E+00_JPRB, 0.48934E+00_JPRB, 0.48646E+00_JPRB, 0.48325E+00_JPRB, 0.47971E+00_JPRB, 0.47201E+00_JPRB ,&
&  0.46224E+00_JPRB, 0.45088E+00_JPRB, 0.44199E+00_JPRB, 0.43432E+00_JPRB, 0.42077E+00_JPRB, 0.41239E+00_JPRB, 0.41328E+00_JPRB ,&
&  0.41762E+00_JPRB, 0.42314E+00_JPRB, 0.42770E+00_JPRB, 0.43818E+00_JPRB, 0.45933E+00_JPRB, 0.48686E+00_JPRB, 0.52015E+00_JPRB ,&
&  0.58607E+00_JPRB, 0.62675E+00_JPRB, 0.68861E+00_JPRB, 0.70389E+00_JPRB, 0.71020E+00_JPRB, 0.74731E+00_JPRB, 0.80120E+00_JPRB ,&
&  0.83550E+00_JPRB, 0.89709E+00_JPRB, 0.88860E+00_JPRB, 0.81436E+00_JPRB, 0.73917E+00_JPRB, 0.68580E+00_JPRB, 0.65221E+00_JPRB ,&
&  0.62673E+00_JPRB, 0.60441E+00_JPRB, 0.58541E+00_JPRB, 0.57273E+00_JPRB, 0.56053E+00_JPRB, 0.54838E+00_JPRB, 0.53641E+00_JPRB ,&
&  0.52928E+00_JPRB, 0.52496E+00_JPRB, 0.52530E+00_JPRB, 0.53764E+00_JPRB, 0.58286E+00_JPRB, 0.66007E+00_JPRB, 0.76458E+00_JPRB ,&
&  0.85116E+00_JPRB, 0.90112E+00_JPRB, 0.94479E+00_JPRB, 0.94099E+00_JPRB, 0.90467E+00_JPRB, 0.86229E+00_JPRB, 0.81674E+00_JPRB ,&
&  0.75851E+00_JPRB, 0.69339E+00_JPRB, 0.64549E+00_JPRB, 0.61249E+00_JPRB, 0.59099E+00_JPRB, 0.57865E+00_JPRB, 0.57209E+00_JPRB ,&
&  0.56704E+00_JPRB, 0.56663E+00_JPRB, 0.57553E+00_JPRB, 0.58524E+00_JPRB, 0.58467E+00_JPRB, 0.57248E+00_JPRB, 0.55450E+00_JPRB ,&
&  0.53899E+00_JPRB, 0.52691E+00_JPRB, 0.52712E+00_JPRB, 0.52783E+00_JPRB, 0.52854E+00_JPRB, 0.53039E+00_JPRB, 0.52968E+00_JPRB ,&
&  0.53157E+00_JPRB, 0.58702E+00_JPRB, 0.64578E+00_JPRB, 0.80266E+00_JPRB, 0.86616E+00_JPRB, 0.80458E+00_JPRB, 0.73451E+00_JPRB ,&
&  0.67785E+00_JPRB, 0.63820E+00_JPRB, 0.60075E+00_JPRB, 0.58410E+00_JPRB, 0.63565E+00_JPRB /)
ZBURD(:, 53,  6) = (/ &
&  0.54696E+00_JPRB, 0.54025E+00_JPRB, 0.52589E+00_JPRB, 0.51077E+00_JPRB, 0.49726E+00_JPRB, 0.48836E+00_JPRB, 0.48089E+00_JPRB ,&
&  0.47562E+00_JPRB, 0.47158E+00_JPRB, 0.46871E+00_JPRB, 0.46519E+00_JPRB, 0.46036E+00_JPRB, 0.45334E+00_JPRB, 0.44434E+00_JPRB ,&
&  0.43602E+00_JPRB, 0.42795E+00_JPRB, 0.42117E+00_JPRB, 0.41375E+00_JPRB, 0.40456E+00_JPRB, 0.39959E+00_JPRB, 0.40166E+00_JPRB ,&
&  0.40718E+00_JPRB, 0.41404E+00_JPRB, 0.42049E+00_JPRB, 0.43040E+00_JPRB, 0.45450E+00_JPRB, 0.48163E+00_JPRB, 0.50681E+00_JPRB ,&
&  0.56274E+00_JPRB, 0.54404E+00_JPRB, 0.57253E+00_JPRB, 0.60899E+00_JPRB, 0.64209E+00_JPRB, 0.65857E+00_JPRB, 0.67834E+00_JPRB ,&
&  0.69977E+00_JPRB, 0.71977E+00_JPRB, 0.71988E+00_JPRB, 0.68929E+00_JPRB, 0.63206E+00_JPRB, 0.57950E+00_JPRB, 0.54882E+00_JPRB ,&
&  0.53339E+00_JPRB, 0.52653E+00_JPRB, 0.52032E+00_JPRB, 0.51475E+00_JPRB, 0.51083E+00_JPRB, 0.50714E+00_JPRB, 0.50313E+00_JPRB ,&
&  0.49938E+00_JPRB, 0.49647E+00_JPRB, 0.49505E+00_JPRB, 0.49622E+00_JPRB, 0.50621E+00_JPRB, 0.53540E+00_JPRB, 0.59183E+00_JPRB ,&
&  0.67199E+00_JPRB, 0.74590E+00_JPRB, 0.78247E+00_JPRB, 0.80311E+00_JPRB, 0.81759E+00_JPRB, 0.79156E+00_JPRB, 0.72025E+00_JPRB ,&
&  0.65482E+00_JPRB, 0.60177E+00_JPRB, 0.56487E+00_JPRB, 0.53931E+00_JPRB, 0.52259E+00_JPRB, 0.51325E+00_JPRB, 0.50929E+00_JPRB ,&
&  0.50795E+00_JPRB, 0.50769E+00_JPRB, 0.51015E+00_JPRB, 0.51220E+00_JPRB, 0.51334E+00_JPRB, 0.51122E+00_JPRB, 0.50483E+00_JPRB ,&
&  0.49593E+00_JPRB, 0.49002E+00_JPRB, 0.48808E+00_JPRB, 0.48693E+00_JPRB, 0.48484E+00_JPRB, 0.48179E+00_JPRB, 0.47952E+00_JPRB ,&
&  0.47956E+00_JPRB, 0.50629E+00_JPRB, 0.55009E+00_JPRB, 0.66087E+00_JPRB, 0.65582E+00_JPRB, 0.64007E+00_JPRB, 0.61312E+00_JPRB ,&
&  0.59300E+00_JPRB, 0.57984E+00_JPRB, 0.55955E+00_JPRB, 0.54987E+00_JPRB, 0.55056E+00_JPRB /)
ZBURD(:, 54,  6) = (/ &
&  0.51734E+00_JPRB, 0.50827E+00_JPRB, 0.49413E+00_JPRB, 0.47948E+00_JPRB, 0.47019E+00_JPRB, 0.46424E+00_JPRB, 0.45986E+00_JPRB ,&
&  0.45475E+00_JPRB, 0.44871E+00_JPRB, 0.44258E+00_JPRB, 0.43868E+00_JPRB, 0.43410E+00_JPRB, 0.42776E+00_JPRB, 0.42013E+00_JPRB ,&
&  0.41308E+00_JPRB, 0.40657E+00_JPRB, 0.40152E+00_JPRB, 0.39597E+00_JPRB, 0.39156E+00_JPRB, 0.39114E+00_JPRB, 0.39239E+00_JPRB ,&
&  0.39785E+00_JPRB, 0.40534E+00_JPRB, 0.41215E+00_JPRB, 0.42655E+00_JPRB, 0.44346E+00_JPRB, 0.46803E+00_JPRB, 0.49258E+00_JPRB ,&
&  0.54070E+00_JPRB, 0.47605E+00_JPRB, 0.49905E+00_JPRB, 0.55340E+00_JPRB, 0.58573E+00_JPRB, 0.60405E+00_JPRB, 0.59820E+00_JPRB ,&
&  0.59359E+00_JPRB, 0.59514E+00_JPRB, 0.59391E+00_JPRB, 0.58643E+00_JPRB, 0.56557E+00_JPRB, 0.52710E+00_JPRB, 0.48982E+00_JPRB ,&
&  0.46427E+00_JPRB, 0.45537E+00_JPRB, 0.45149E+00_JPRB, 0.44595E+00_JPRB, 0.44039E+00_JPRB, 0.43793E+00_JPRB, 0.43857E+00_JPRB ,&
&  0.43894E+00_JPRB, 0.43872E+00_JPRB, 0.43772E+00_JPRB, 0.43673E+00_JPRB, 0.43831E+00_JPRB, 0.44902E+00_JPRB, 0.47686E+00_JPRB ,&
&  0.52621E+00_JPRB, 0.58051E+00_JPRB, 0.62809E+00_JPRB, 0.66355E+00_JPRB, 0.66858E+00_JPRB, 0.65064E+00_JPRB, 0.61779E+00_JPRB ,&
&  0.58025E+00_JPRB, 0.54723E+00_JPRB, 0.52274E+00_JPRB, 0.50380E+00_JPRB, 0.48869E+00_JPRB, 0.47668E+00_JPRB, 0.46829E+00_JPRB ,&
&  0.46381E+00_JPRB, 0.46107E+00_JPRB, 0.45880E+00_JPRB, 0.45541E+00_JPRB, 0.45257E+00_JPRB, 0.45029E+00_JPRB, 0.44925E+00_JPRB ,&
&  0.44773E+00_JPRB, 0.44600E+00_JPRB, 0.44653E+00_JPRB, 0.44959E+00_JPRB, 0.45157E+00_JPRB, 0.45117E+00_JPRB, 0.45052E+00_JPRB ,&
&  0.45224E+00_JPRB, 0.46441E+00_JPRB, 0.47754E+00_JPRB, 0.57393E+00_JPRB, 0.59591E+00_JPRB, 0.58007E+00_JPRB, 0.56098E+00_JPRB ,&
&  0.54950E+00_JPRB, 0.53420E+00_JPRB, 0.52790E+00_JPRB, 0.48872E+00_JPRB, 0.51890E+00_JPRB /)
ZBURD(:, 55,  6) = (/ &
&  0.47063E+00_JPRB, 0.46883E+00_JPRB, 0.46220E+00_JPRB, 0.45059E+00_JPRB, 0.44087E+00_JPRB, 0.43331E+00_JPRB, 0.42821E+00_JPRB ,&
&  0.42530E+00_JPRB, 0.42270E+00_JPRB, 0.41934E+00_JPRB, 0.41387E+00_JPRB, 0.40919E+00_JPRB, 0.40358E+00_JPRB, 0.39787E+00_JPRB ,&
&  0.39303E+00_JPRB, 0.38854E+00_JPRB, 0.38581E+00_JPRB, 0.38239E+00_JPRB, 0.38080E+00_JPRB, 0.38184E+00_JPRB, 0.38347E+00_JPRB ,&
&  0.38595E+00_JPRB, 0.38837E+00_JPRB, 0.39676E+00_JPRB, 0.40778E+00_JPRB, 0.42749E+00_JPRB, 0.44086E+00_JPRB, 0.46116E+00_JPRB ,&
&  0.50708E+00_JPRB, 0.43151E+00_JPRB, 0.46309E+00_JPRB, 0.50353E+00_JPRB, 0.52967E+00_JPRB, 0.55014E+00_JPRB, 0.53164E+00_JPRB ,&
&  0.51707E+00_JPRB, 0.50754E+00_JPRB, 0.49928E+00_JPRB, 0.49313E+00_JPRB, 0.48886E+00_JPRB, 0.47728E+00_JPRB, 0.46243E+00_JPRB ,&
&  0.44597E+00_JPRB, 0.43368E+00_JPRB, 0.42380E+00_JPRB, 0.41422E+00_JPRB, 0.40589E+00_JPRB, 0.40064E+00_JPRB, 0.39822E+00_JPRB ,&
&  0.39796E+00_JPRB, 0.39828E+00_JPRB, 0.39828E+00_JPRB, 0.39900E+00_JPRB, 0.40133E+00_JPRB, 0.40587E+00_JPRB, 0.41364E+00_JPRB ,&
&  0.43125E+00_JPRB, 0.45833E+00_JPRB, 0.48655E+00_JPRB, 0.51214E+00_JPRB, 0.53078E+00_JPRB, 0.53609E+00_JPRB, 0.53333E+00_JPRB ,&
&  0.52208E+00_JPRB, 0.50591E+00_JPRB, 0.48904E+00_JPRB, 0.47350E+00_JPRB, 0.45988E+00_JPRB, 0.44812E+00_JPRB, 0.43710E+00_JPRB ,&
&  0.42804E+00_JPRB, 0.42036E+00_JPRB, 0.41503E+00_JPRB, 0.41129E+00_JPRB, 0.40760E+00_JPRB, 0.40435E+00_JPRB, 0.40270E+00_JPRB ,&
&  0.40145E+00_JPRB, 0.40061E+00_JPRB, 0.40078E+00_JPRB, 0.40382E+00_JPRB, 0.40880E+00_JPRB, 0.41286E+00_JPRB, 0.41524E+00_JPRB ,&
&  0.41606E+00_JPRB, 0.42406E+00_JPRB, 0.43159E+00_JPRB, 0.47013E+00_JPRB, 0.48993E+00_JPRB, 0.49447E+00_JPRB, 0.49313E+00_JPRB ,&
&  0.49165E+00_JPRB, 0.48182E+00_JPRB, 0.42437E+00_JPRB, 0.45370E+00_JPRB, 0.46523E+00_JPRB /)
ZBURD(:, 56,  6) = (/ &
&  0.43194E+00_JPRB, 0.42838E+00_JPRB, 0.42645E+00_JPRB, 0.42332E+00_JPRB, 0.41912E+00_JPRB, 0.41450E+00_JPRB, 0.40932E+00_JPRB ,&
&  0.40391E+00_JPRB, 0.40054E+00_JPRB, 0.39755E+00_JPRB, 0.39409E+00_JPRB, 0.39065E+00_JPRB, 0.38790E+00_JPRB, 0.38520E+00_JPRB ,&
&  0.38216E+00_JPRB, 0.37868E+00_JPRB, 0.37614E+00_JPRB, 0.37433E+00_JPRB, 0.37239E+00_JPRB, 0.37155E+00_JPRB, 0.37232E+00_JPRB ,&
&  0.37207E+00_JPRB, 0.37424E+00_JPRB, 0.38406E+00_JPRB, 0.39594E+00_JPRB, 0.41042E+00_JPRB, 0.42317E+00_JPRB, 0.43852E+00_JPRB ,&
&  0.42664E+00_JPRB, 0.38381E+00_JPRB, 0.41028E+00_JPRB, 0.44189E+00_JPRB, 0.47406E+00_JPRB, 0.49878E+00_JPRB, 0.48595E+00_JPRB ,&
&  0.46831E+00_JPRB, 0.45088E+00_JPRB, 0.43737E+00_JPRB, 0.42797E+00_JPRB, 0.42587E+00_JPRB, 0.42410E+00_JPRB, 0.42156E+00_JPRB ,&
&  0.41718E+00_JPRB, 0.40946E+00_JPRB, 0.40132E+00_JPRB, 0.39385E+00_JPRB, 0.38859E+00_JPRB, 0.38436E+00_JPRB, 0.38078E+00_JPRB ,&
&  0.37918E+00_JPRB, 0.37897E+00_JPRB, 0.37905E+00_JPRB, 0.37943E+00_JPRB, 0.38052E+00_JPRB, 0.38371E+00_JPRB, 0.38771E+00_JPRB ,&
&  0.39227E+00_JPRB, 0.39970E+00_JPRB, 0.41199E+00_JPRB, 0.42659E+00_JPRB, 0.44080E+00_JPRB, 0.45045E+00_JPRB, 0.45477E+00_JPRB ,&
&  0.45583E+00_JPRB, 0.45335E+00_JPRB, 0.44739E+00_JPRB, 0.43824E+00_JPRB, 0.42716E+00_JPRB, 0.41683E+00_JPRB, 0.40750E+00_JPRB ,&
&  0.39899E+00_JPRB, 0.39024E+00_JPRB, 0.38234E+00_JPRB, 0.37793E+00_JPRB, 0.37601E+00_JPRB, 0.37405E+00_JPRB, 0.37229E+00_JPRB ,&
&  0.37129E+00_JPRB, 0.37068E+00_JPRB, 0.37047E+00_JPRB, 0.37132E+00_JPRB, 0.37384E+00_JPRB, 0.37780E+00_JPRB, 0.38178E+00_JPRB ,&
&  0.38476E+00_JPRB, 0.39267E+00_JPRB, 0.40295E+00_JPRB, 0.42912E+00_JPRB, 0.44665E+00_JPRB, 0.45311E+00_JPRB, 0.45274E+00_JPRB ,&
&  0.44850E+00_JPRB, 0.44302E+00_JPRB, 0.43855E+00_JPRB, 0.43366E+00_JPRB, 0.43278E+00_JPRB /)
ZBURD(:, 57,  6) = (/ &
&  0.40325E+00_JPRB, 0.40097E+00_JPRB, 0.39898E+00_JPRB, 0.39634E+00_JPRB, 0.39377E+00_JPRB, 0.39144E+00_JPRB, 0.38741E+00_JPRB ,&
&  0.38560E+00_JPRB, 0.38155E+00_JPRB, 0.37793E+00_JPRB, 0.37655E+00_JPRB, 0.37496E+00_JPRB, 0.37262E+00_JPRB, 0.37071E+00_JPRB ,&
&  0.36838E+00_JPRB, 0.36563E+00_JPRB, 0.36363E+00_JPRB, 0.36103E+00_JPRB, 0.35732E+00_JPRB, 0.35465E+00_JPRB, 0.35497E+00_JPRB ,&
&  0.35569E+00_JPRB, 0.35811E+00_JPRB, 0.36608E+00_JPRB, 0.37419E+00_JPRB, 0.38323E+00_JPRB, 0.39202E+00_JPRB, 0.40023E+00_JPRB ,&
&  0.41325E+00_JPRB, 0.36909E+00_JPRB, 0.39317E+00_JPRB, 0.41088E+00_JPRB, 0.43786E+00_JPRB, 0.45893E+00_JPRB, 0.45160E+00_JPRB ,&
&  0.43329E+00_JPRB, 0.41263E+00_JPRB, 0.39766E+00_JPRB, 0.38872E+00_JPRB, 0.38864E+00_JPRB, 0.39017E+00_JPRB, 0.38882E+00_JPRB ,&
&  0.38580E+00_JPRB, 0.37909E+00_JPRB, 0.37151E+00_JPRB, 0.36653E+00_JPRB, 0.36383E+00_JPRB, 0.36176E+00_JPRB, 0.35958E+00_JPRB ,&
&  0.35827E+00_JPRB, 0.35803E+00_JPRB, 0.35794E+00_JPRB, 0.35788E+00_JPRB, 0.35857E+00_JPRB, 0.36038E+00_JPRB, 0.36285E+00_JPRB ,&
&  0.36488E+00_JPRB, 0.36581E+00_JPRB, 0.36755E+00_JPRB, 0.37123E+00_JPRB, 0.37644E+00_JPRB, 0.38169E+00_JPRB, 0.38536E+00_JPRB ,&
&  0.38700E+00_JPRB, 0.38776E+00_JPRB, 0.38744E+00_JPRB, 0.38525E+00_JPRB, 0.38094E+00_JPRB, 0.37504E+00_JPRB, 0.36937E+00_JPRB ,&
&  0.36407E+00_JPRB, 0.35911E+00_JPRB, 0.35293E+00_JPRB, 0.35001E+00_JPRB, 0.34935E+00_JPRB, 0.34919E+00_JPRB, 0.34930E+00_JPRB ,&
&  0.34952E+00_JPRB, 0.34918E+00_JPRB, 0.34872E+00_JPRB, 0.34842E+00_JPRB, 0.34875E+00_JPRB, 0.35018E+00_JPRB, 0.35298E+00_JPRB ,&
&  0.35695E+00_JPRB, 0.36443E+00_JPRB, 0.37470E+00_JPRB, 0.39405E+00_JPRB, 0.41140E+00_JPRB, 0.41960E+00_JPRB, 0.42189E+00_JPRB ,&
&  0.41836E+00_JPRB, 0.41352E+00_JPRB, 0.40907E+00_JPRB, 0.40488E+00_JPRB, 0.40473E+00_JPRB /)
ZBURD(:, 58,  6) = (/ &
&  0.38420E+00_JPRB, 0.38022E+00_JPRB, 0.37725E+00_JPRB, 0.37297E+00_JPRB, 0.36993E+00_JPRB, 0.36768E+00_JPRB, 0.36519E+00_JPRB ,&
&  0.36327E+00_JPRB, 0.36338E+00_JPRB, 0.36214E+00_JPRB, 0.36155E+00_JPRB, 0.36062E+00_JPRB, 0.35782E+00_JPRB, 0.35425E+00_JPRB ,&
&  0.35219E+00_JPRB, 0.35032E+00_JPRB, 0.34860E+00_JPRB, 0.34663E+00_JPRB, 0.34464E+00_JPRB, 0.34201E+00_JPRB, 0.34193E+00_JPRB ,&
&  0.34287E+00_JPRB, 0.34546E+00_JPRB, 0.34865E+00_JPRB, 0.35240E+00_JPRB, 0.35825E+00_JPRB, 0.36634E+00_JPRB, 0.37406E+00_JPRB ,&
&  0.38282E+00_JPRB, 0.37133E+00_JPRB, 0.38614E+00_JPRB, 0.39468E+00_JPRB, 0.40992E+00_JPRB, 0.42231E+00_JPRB, 0.42170E+00_JPRB ,&
&  0.41398E+00_JPRB, 0.39907E+00_JPRB, 0.38594E+00_JPRB, 0.37761E+00_JPRB, 0.37549E+00_JPRB, 0.37390E+00_JPRB, 0.36868E+00_JPRB ,&
&  0.36176E+00_JPRB, 0.35347E+00_JPRB, 0.34704E+00_JPRB, 0.34349E+00_JPRB, 0.34216E+00_JPRB, 0.34182E+00_JPRB, 0.34037E+00_JPRB ,&
&  0.33883E+00_JPRB, 0.33729E+00_JPRB, 0.33652E+00_JPRB, 0.33613E+00_JPRB, 0.33571E+00_JPRB, 0.33594E+00_JPRB, 0.33683E+00_JPRB ,&
&  0.33786E+00_JPRB, 0.33860E+00_JPRB, 0.33904E+00_JPRB, 0.34003E+00_JPRB, 0.34116E+00_JPRB, 0.34212E+00_JPRB, 0.34290E+00_JPRB ,&
&  0.34274E+00_JPRB, 0.34252E+00_JPRB, 0.34183E+00_JPRB, 0.34130E+00_JPRB, 0.33955E+00_JPRB, 0.33641E+00_JPRB, 0.33290E+00_JPRB ,&
&  0.33033E+00_JPRB, 0.32868E+00_JPRB, 0.32746E+00_JPRB, 0.32669E+00_JPRB, 0.32665E+00_JPRB, 0.32742E+00_JPRB, 0.32850E+00_JPRB ,&
&  0.32965E+00_JPRB, 0.33070E+00_JPRB, 0.33119E+00_JPRB, 0.33141E+00_JPRB, 0.33109E+00_JPRB, 0.33171E+00_JPRB, 0.33353E+00_JPRB ,&
&  0.33762E+00_JPRB, 0.34304E+00_JPRB, 0.35098E+00_JPRB, 0.36509E+00_JPRB, 0.38005E+00_JPRB, 0.38972E+00_JPRB, 0.39582E+00_JPRB ,&
&  0.39652E+00_JPRB, 0.39422E+00_JPRB, 0.39132E+00_JPRB, 0.38966E+00_JPRB, 0.38821E+00_JPRB /)
ZBURD(:, 59,  6) = (/ &
&  0.35866E+00_JPRB, 0.35380E+00_JPRB, 0.35008E+00_JPRB, 0.34723E+00_JPRB, 0.34692E+00_JPRB, 0.34556E+00_JPRB, 0.34259E+00_JPRB ,&
&  0.34043E+00_JPRB, 0.34163E+00_JPRB, 0.34412E+00_JPRB, 0.34502E+00_JPRB, 0.34405E+00_JPRB, 0.34239E+00_JPRB, 0.33936E+00_JPRB ,&
&  0.33535E+00_JPRB, 0.33338E+00_JPRB, 0.33305E+00_JPRB, 0.33344E+00_JPRB, 0.33295E+00_JPRB, 0.33101E+00_JPRB, 0.32952E+00_JPRB ,&
&  0.33023E+00_JPRB, 0.33157E+00_JPRB, 0.33191E+00_JPRB, 0.33395E+00_JPRB, 0.33818E+00_JPRB, 0.34598E+00_JPRB, 0.35313E+00_JPRB ,&
&  0.35627E+00_JPRB, 0.35798E+00_JPRB, 0.35211E+00_JPRB, 0.36454E+00_JPRB, 0.37270E+00_JPRB, 0.38350E+00_JPRB, 0.38997E+00_JPRB ,&
&  0.39112E+00_JPRB, 0.38611E+00_JPRB, 0.37464E+00_JPRB, 0.36565E+00_JPRB, 0.35994E+00_JPRB, 0.35388E+00_JPRB, 0.34472E+00_JPRB ,&
&  0.33536E+00_JPRB, 0.32739E+00_JPRB, 0.32264E+00_JPRB, 0.32040E+00_JPRB, 0.32043E+00_JPRB, 0.32091E+00_JPRB, 0.31918E+00_JPRB ,&
&  0.31718E+00_JPRB, 0.31524E+00_JPRB, 0.31445E+00_JPRB, 0.31364E+00_JPRB, 0.31246E+00_JPRB, 0.31151E+00_JPRB, 0.31124E+00_JPRB ,&
&  0.31141E+00_JPRB, 0.31161E+00_JPRB, 0.31215E+00_JPRB, 0.31309E+00_JPRB, 0.31363E+00_JPRB, 0.31325E+00_JPRB, 0.31219E+00_JPRB ,&
&  0.31093E+00_JPRB, 0.30984E+00_JPRB, 0.30890E+00_JPRB, 0.30790E+00_JPRB, 0.30699E+00_JPRB, 0.30615E+00_JPRB, 0.30496E+00_JPRB ,&
&  0.30348E+00_JPRB, 0.30328E+00_JPRB, 0.30404E+00_JPRB, 0.30421E+00_JPRB, 0.30410E+00_JPRB, 0.30485E+00_JPRB, 0.30654E+00_JPRB ,&
&  0.30826E+00_JPRB, 0.31027E+00_JPRB, 0.31180E+00_JPRB, 0.31256E+00_JPRB, 0.31212E+00_JPRB, 0.31229E+00_JPRB, 0.31326E+00_JPRB ,&
&  0.31548E+00_JPRB, 0.31895E+00_JPRB, 0.32402E+00_JPRB, 0.33155E+00_JPRB, 0.34119E+00_JPRB, 0.34917E+00_JPRB, 0.35576E+00_JPRB ,&
&  0.36069E+00_JPRB, 0.36300E+00_JPRB, 0.36328E+00_JPRB, 0.36358E+00_JPRB, 0.36252E+00_JPRB /)
ZBURD(:, 60,  6) = (/ &
&  0.33117E+00_JPRB, 0.32685E+00_JPRB, 0.32371E+00_JPRB, 0.32315E+00_JPRB, 0.32488E+00_JPRB, 0.32507E+00_JPRB, 0.32334E+00_JPRB ,&
&  0.32175E+00_JPRB, 0.32310E+00_JPRB, 0.32627E+00_JPRB, 0.32802E+00_JPRB, 0.32860E+00_JPRB, 0.32922E+00_JPRB, 0.32792E+00_JPRB ,&
&  0.32431E+00_JPRB, 0.32072E+00_JPRB, 0.32085E+00_JPRB, 0.32193E+00_JPRB, 0.32271E+00_JPRB, 0.32113E+00_JPRB, 0.31928E+00_JPRB ,&
&  0.31960E+00_JPRB, 0.31959E+00_JPRB, 0.32124E+00_JPRB, 0.32303E+00_JPRB, 0.32685E+00_JPRB, 0.33327E+00_JPRB, 0.33810E+00_JPRB ,&
&  0.33993E+00_JPRB, 0.33843E+00_JPRB, 0.33775E+00_JPRB, 0.34075E+00_JPRB, 0.34424E+00_JPRB, 0.35212E+00_JPRB, 0.36047E+00_JPRB ,&
&  0.36373E+00_JPRB, 0.36074E+00_JPRB, 0.34991E+00_JPRB, 0.34028E+00_JPRB, 0.33366E+00_JPRB, 0.32681E+00_JPRB, 0.32010E+00_JPRB ,&
&  0.31329E+00_JPRB, 0.30834E+00_JPRB, 0.30544E+00_JPRB, 0.30411E+00_JPRB, 0.30365E+00_JPRB, 0.30296E+00_JPRB, 0.30071E+00_JPRB ,&
&  0.29779E+00_JPRB, 0.29552E+00_JPRB, 0.29380E+00_JPRB, 0.29249E+00_JPRB, 0.29129E+00_JPRB, 0.29073E+00_JPRB, 0.29007E+00_JPRB ,&
&  0.28977E+00_JPRB, 0.29021E+00_JPRB, 0.29106E+00_JPRB, 0.29112E+00_JPRB, 0.29096E+00_JPRB, 0.29028E+00_JPRB, 0.28958E+00_JPRB ,&
&  0.28852E+00_JPRB, 0.28635E+00_JPRB, 0.28455E+00_JPRB, 0.28424E+00_JPRB, 0.28415E+00_JPRB, 0.28465E+00_JPRB, 0.28504E+00_JPRB ,&
&  0.28476E+00_JPRB, 0.28470E+00_JPRB, 0.28535E+00_JPRB, 0.28621E+00_JPRB, 0.28684E+00_JPRB, 0.28719E+00_JPRB, 0.28778E+00_JPRB ,&
&  0.28941E+00_JPRB, 0.29133E+00_JPRB, 0.29272E+00_JPRB, 0.29383E+00_JPRB, 0.29460E+00_JPRB, 0.29591E+00_JPRB, 0.29615E+00_JPRB ,&
&  0.29656E+00_JPRB, 0.29809E+00_JPRB, 0.30148E+00_JPRB, 0.30643E+00_JPRB, 0.31201E+00_JPRB, 0.31684E+00_JPRB, 0.32155E+00_JPRB ,&
&  0.32561E+00_JPRB, 0.32894E+00_JPRB, 0.33101E+00_JPRB, 0.33166E+00_JPRB, 0.33210E+00_JPRB /)
ZBURD(:, 61,  6) = (/ &
&  0.30574E+00_JPRB, 0.30387E+00_JPRB, 0.30233E+00_JPRB, 0.30251E+00_JPRB, 0.30471E+00_JPRB, 0.30521E+00_JPRB, 0.30513E+00_JPRB ,&
&  0.30495E+00_JPRB, 0.30699E+00_JPRB, 0.31072E+00_JPRB, 0.31306E+00_JPRB, 0.31342E+00_JPRB, 0.31319E+00_JPRB, 0.31257E+00_JPRB ,&
&  0.31206E+00_JPRB, 0.31158E+00_JPRB, 0.31024E+00_JPRB, 0.31018E+00_JPRB, 0.31004E+00_JPRB, 0.30897E+00_JPRB, 0.30969E+00_JPRB ,&
&  0.30968E+00_JPRB, 0.31007E+00_JPRB, 0.31226E+00_JPRB, 0.31404E+00_JPRB, 0.31800E+00_JPRB, 0.32449E+00_JPRB, 0.32808E+00_JPRB ,&
&  0.32805E+00_JPRB, 0.32380E+00_JPRB, 0.32010E+00_JPRB, 0.32076E+00_JPRB, 0.32282E+00_JPRB, 0.32706E+00_JPRB, 0.33178E+00_JPRB ,&
&  0.33327E+00_JPRB, 0.32733E+00_JPRB, 0.31763E+00_JPRB, 0.31036E+00_JPRB, 0.30542E+00_JPRB, 0.30184E+00_JPRB, 0.29869E+00_JPRB ,&
&  0.29527E+00_JPRB, 0.29285E+00_JPRB, 0.29177E+00_JPRB, 0.29163E+00_JPRB, 0.29128E+00_JPRB, 0.28846E+00_JPRB, 0.28521E+00_JPRB ,&
&  0.28100E+00_JPRB, 0.27805E+00_JPRB, 0.27562E+00_JPRB, 0.27478E+00_JPRB, 0.27486E+00_JPRB, 0.27458E+00_JPRB, 0.27353E+00_JPRB ,&
&  0.27222E+00_JPRB, 0.27181E+00_JPRB, 0.27141E+00_JPRB, 0.27022E+00_JPRB, 0.26873E+00_JPRB, 0.26828E+00_JPRB, 0.26722E+00_JPRB ,&
&  0.26513E+00_JPRB, 0.26395E+00_JPRB, 0.26410E+00_JPRB, 0.26518E+00_JPRB, 0.26520E+00_JPRB, 0.26607E+00_JPRB, 0.26769E+00_JPRB ,&
&  0.26683E+00_JPRB, 0.26437E+00_JPRB, 0.26305E+00_JPRB, 0.26352E+00_JPRB, 0.26528E+00_JPRB, 0.26765E+00_JPRB, 0.26883E+00_JPRB ,&
&  0.26988E+00_JPRB, 0.26994E+00_JPRB, 0.27091E+00_JPRB, 0.27232E+00_JPRB, 0.27406E+00_JPRB, 0.27513E+00_JPRB, 0.27564E+00_JPRB ,&
&  0.27622E+00_JPRB, 0.27696E+00_JPRB, 0.27932E+00_JPRB, 0.28294E+00_JPRB, 0.28633E+00_JPRB, 0.29017E+00_JPRB, 0.29337E+00_JPRB ,&
&  0.29587E+00_JPRB, 0.29861E+00_JPRB, 0.30081E+00_JPRB, 0.30231E+00_JPRB, 0.30409E+00_JPRB /)
ZBURD(:, 62,  6) = (/ &
&  0.28931E+00_JPRB, 0.29012E+00_JPRB, 0.29063E+00_JPRB, 0.29035E+00_JPRB, 0.29141E+00_JPRB, 0.29233E+00_JPRB, 0.29396E+00_JPRB ,&
&  0.29485E+00_JPRB, 0.29655E+00_JPRB, 0.29881E+00_JPRB, 0.29879E+00_JPRB, 0.29837E+00_JPRB, 0.29806E+00_JPRB, 0.29851E+00_JPRB ,&
&  0.30063E+00_JPRB, 0.30160E+00_JPRB, 0.30083E+00_JPRB, 0.30041E+00_JPRB, 0.29939E+00_JPRB, 0.29851E+00_JPRB, 0.30036E+00_JPRB ,&
&  0.30071E+00_JPRB, 0.30134E+00_JPRB, 0.30322E+00_JPRB, 0.30486E+00_JPRB, 0.30876E+00_JPRB, 0.31364E+00_JPRB, 0.31740E+00_JPRB ,&
&  0.31794E+00_JPRB, 0.31486E+00_JPRB, 0.30937E+00_JPRB, 0.30525E+00_JPRB, 0.30453E+00_JPRB, 0.30559E+00_JPRB, 0.30804E+00_JPRB ,&
&  0.30730E+00_JPRB, 0.30160E+00_JPRB, 0.29418E+00_JPRB, 0.29000E+00_JPRB, 0.28826E+00_JPRB, 0.28593E+00_JPRB, 0.28235E+00_JPRB ,&
&  0.27948E+00_JPRB, 0.27864E+00_JPRB, 0.27872E+00_JPRB, 0.27792E+00_JPRB, 0.27621E+00_JPRB, 0.27325E+00_JPRB, 0.26944E+00_JPRB ,&
&  0.26608E+00_JPRB, 0.26274E+00_JPRB, 0.26065E+00_JPRB, 0.25919E+00_JPRB, 0.25903E+00_JPRB, 0.25916E+00_JPRB, 0.25827E+00_JPRB ,&
&  0.25633E+00_JPRB, 0.25507E+00_JPRB, 0.25337E+00_JPRB, 0.25018E+00_JPRB, 0.24788E+00_JPRB, 0.24714E+00_JPRB, 0.24602E+00_JPRB ,&
&  0.24446E+00_JPRB, 0.24390E+00_JPRB, 0.24368E+00_JPRB, 0.24320E+00_JPRB, 0.24406E+00_JPRB, 0.24578E+00_JPRB, 0.24340E+00_JPRB ,&
&  0.24128E+00_JPRB, 0.23988E+00_JPRB, 0.23997E+00_JPRB, 0.24152E+00_JPRB, 0.24412E+00_JPRB, 0.24696E+00_JPRB, 0.24930E+00_JPRB ,&
&  0.25029E+00_JPRB, 0.25057E+00_JPRB, 0.25091E+00_JPRB, 0.25108E+00_JPRB, 0.25221E+00_JPRB, 0.25454E+00_JPRB, 0.25736E+00_JPRB ,&
&  0.25883E+00_JPRB, 0.26139E+00_JPRB, 0.26419E+00_JPRB, 0.26769E+00_JPRB, 0.26982E+00_JPRB, 0.27190E+00_JPRB, 0.27405E+00_JPRB ,&
&  0.27584E+00_JPRB, 0.27813E+00_JPRB, 0.28136E+00_JPRB, 0.28397E+00_JPRB, 0.28668E+00_JPRB /)
ZBURD(:, 63,  6) = (/ &
&  0.27677E+00_JPRB, 0.27890E+00_JPRB, 0.27984E+00_JPRB, 0.28082E+00_JPRB, 0.28309E+00_JPRB, 0.28469E+00_JPRB, 0.28497E+00_JPRB ,&
&  0.28532E+00_JPRB, 0.28672E+00_JPRB, 0.28773E+00_JPRB, 0.28814E+00_JPRB, 0.28578E+00_JPRB, 0.28530E+00_JPRB, 0.28685E+00_JPRB ,&
&  0.28895E+00_JPRB, 0.29093E+00_JPRB, 0.29082E+00_JPRB, 0.29048E+00_JPRB, 0.28968E+00_JPRB, 0.28836E+00_JPRB, 0.28971E+00_JPRB ,&
&  0.29084E+00_JPRB, 0.29128E+00_JPRB, 0.29259E+00_JPRB, 0.29401E+00_JPRB, 0.29713E+00_JPRB, 0.30082E+00_JPRB, 0.30331E+00_JPRB ,&
&  0.30403E+00_JPRB, 0.30253E+00_JPRB, 0.29582E+00_JPRB, 0.28880E+00_JPRB, 0.28909E+00_JPRB, 0.29018E+00_JPRB, 0.29014E+00_JPRB ,&
&  0.28807E+00_JPRB, 0.28524E+00_JPRB, 0.28331E+00_JPRB, 0.28174E+00_JPRB, 0.27976E+00_JPRB, 0.27627E+00_JPRB, 0.27245E+00_JPRB ,&
&  0.26914E+00_JPRB, 0.26796E+00_JPRB, 0.26653E+00_JPRB, 0.26415E+00_JPRB, 0.26066E+00_JPRB, 0.25781E+00_JPRB, 0.25486E+00_JPRB ,&
&  0.24975E+00_JPRB, 0.24480E+00_JPRB, 0.24212E+00_JPRB, 0.24144E+00_JPRB, 0.24179E+00_JPRB, 0.24133E+00_JPRB, 0.23956E+00_JPRB ,&
&  0.23752E+00_JPRB, 0.23496E+00_JPRB, 0.23337E+00_JPRB, 0.23288E+00_JPRB, 0.23397E+00_JPRB, 0.23270E+00_JPRB, 0.22831E+00_JPRB ,&
&  0.22578E+00_JPRB, 0.22423E+00_JPRB, 0.22436E+00_JPRB, 0.22334E+00_JPRB, 0.22244E+00_JPRB, 0.22120E+00_JPRB, 0.22165E+00_JPRB ,&
&  0.22273E+00_JPRB, 0.22303E+00_JPRB, 0.22560E+00_JPRB, 0.23014E+00_JPRB, 0.23428E+00_JPRB, 0.23669E+00_JPRB, 0.23812E+00_JPRB ,&
&  0.23863E+00_JPRB, 0.23800E+00_JPRB, 0.23741E+00_JPRB, 0.23796E+00_JPRB, 0.24000E+00_JPRB, 0.24280E+00_JPRB, 0.24551E+00_JPRB ,&
&  0.24630E+00_JPRB, 0.24746E+00_JPRB, 0.25014E+00_JPRB, 0.25417E+00_JPRB, 0.25659E+00_JPRB, 0.25857E+00_JPRB, 0.26033E+00_JPRB ,&
&  0.26135E+00_JPRB, 0.26310E+00_JPRB, 0.26727E+00_JPRB, 0.27021E+00_JPRB, 0.27312E+00_JPRB /)
ZBURD(:, 64,  6) = (/ &
&  0.26399E+00_JPRB, 0.26566E+00_JPRB, 0.26799E+00_JPRB, 0.27020E+00_JPRB, 0.27110E+00_JPRB, 0.27059E+00_JPRB, 0.26987E+00_JPRB ,&
&  0.27030E+00_JPRB, 0.27233E+00_JPRB, 0.27454E+00_JPRB, 0.27506E+00_JPRB, 0.27531E+00_JPRB, 0.27560E+00_JPRB, 0.27724E+00_JPRB ,&
&  0.27973E+00_JPRB, 0.28253E+00_JPRB, 0.28218E+00_JPRB, 0.28158E+00_JPRB, 0.28095E+00_JPRB, 0.27978E+00_JPRB, 0.28080E+00_JPRB ,&
&  0.28367E+00_JPRB, 0.28376E+00_JPRB, 0.28354E+00_JPRB, 0.28282E+00_JPRB, 0.28453E+00_JPRB, 0.28784E+00_JPRB, 0.28987E+00_JPRB ,&
&  0.28880E+00_JPRB, 0.28216E+00_JPRB, 0.27398E+00_JPRB, 0.27364E+00_JPRB, 0.27539E+00_JPRB, 0.27714E+00_JPRB, 0.27738E+00_JPRB ,&
&  0.27696E+00_JPRB, 0.27613E+00_JPRB, 0.27506E+00_JPRB, 0.27395E+00_JPRB, 0.27251E+00_JPRB, 0.26858E+00_JPRB, 0.26402E+00_JPRB ,&
&  0.26034E+00_JPRB, 0.25697E+00_JPRB, 0.25367E+00_JPRB, 0.24924E+00_JPRB, 0.24472E+00_JPRB, 0.24144E+00_JPRB, 0.23771E+00_JPRB ,&
&  0.23314E+00_JPRB, 0.22989E+00_JPRB, 0.22887E+00_JPRB, 0.22881E+00_JPRB, 0.22892E+00_JPRB, 0.22813E+00_JPRB, 0.22714E+00_JPRB ,&
&  0.22537E+00_JPRB, 0.22401E+00_JPRB, 0.22249E+00_JPRB, 0.22162E+00_JPRB, 0.21643E+00_JPRB, 0.20828E+00_JPRB, 0.20251E+00_JPRB ,&
&  0.20179E+00_JPRB, 0.20442E+00_JPRB, 0.20796E+00_JPRB, 0.20871E+00_JPRB, 0.21069E+00_JPRB, 0.21166E+00_JPRB, 0.21126E+00_JPRB ,&
&  0.20861E+00_JPRB, 0.20502E+00_JPRB, 0.20526E+00_JPRB, 0.20632E+00_JPRB, 0.20683E+00_JPRB, 0.20808E+00_JPRB, 0.20963E+00_JPRB ,&
&  0.21250E+00_JPRB, 0.21478E+00_JPRB, 0.21696E+00_JPRB, 0.21924E+00_JPRB, 0.22065E+00_JPRB, 0.22041E+00_JPRB, 0.21842E+00_JPRB ,&
&  0.21837E+00_JPRB, 0.22384E+00_JPRB, 0.23359E+00_JPRB, 0.24400E+00_JPRB, 0.24653E+00_JPRB, 0.24886E+00_JPRB, 0.25099E+00_JPRB ,&
&  0.25218E+00_JPRB, 0.25566E+00_JPRB, 0.25882E+00_JPRB, 0.26080E+00_JPRB, 0.26221E+00_JPRB /)
ZBURD(:, 65,  6) = (/ &
&  0.25556E+00_JPRB, 0.25564E+00_JPRB, 0.25468E+00_JPRB, 0.25372E+00_JPRB, 0.25223E+00_JPRB, 0.25072E+00_JPRB, 0.25132E+00_JPRB ,&
&  0.25375E+00_JPRB, 0.25696E+00_JPRB, 0.26011E+00_JPRB, 0.26254E+00_JPRB, 0.26442E+00_JPRB, 0.26529E+00_JPRB, 0.26667E+00_JPRB ,&
&  0.26827E+00_JPRB, 0.27100E+00_JPRB, 0.27176E+00_JPRB, 0.27239E+00_JPRB, 0.27149E+00_JPRB, 0.27066E+00_JPRB, 0.27166E+00_JPRB ,&
&  0.27479E+00_JPRB, 0.27625E+00_JPRB, 0.27538E+00_JPRB, 0.27329E+00_JPRB, 0.27384E+00_JPRB, 0.27587E+00_JPRB, 0.27545E+00_JPRB ,&
&  0.26915E+00_JPRB, 0.25580E+00_JPRB, 0.24659E+00_JPRB, 0.24953E+00_JPRB, 0.25833E+00_JPRB, 0.26239E+00_JPRB, 0.26522E+00_JPRB ,&
&  0.26700E+00_JPRB, 0.26723E+00_JPRB, 0.26635E+00_JPRB, 0.26471E+00_JPRB, 0.26193E+00_JPRB, 0.25720E+00_JPRB, 0.25161E+00_JPRB ,&
&  0.24661E+00_JPRB, 0.24406E+00_JPRB, 0.24103E+00_JPRB, 0.23572E+00_JPRB, 0.23050E+00_JPRB, 0.22721E+00_JPRB, 0.22336E+00_JPRB ,&
&  0.21942E+00_JPRB, 0.21641E+00_JPRB, 0.21567E+00_JPRB, 0.21524E+00_JPRB, 0.21512E+00_JPRB, 0.21451E+00_JPRB, 0.21224E+00_JPRB ,&
&  0.20892E+00_JPRB, 0.20464E+00_JPRB, 0.19901E+00_JPRB, 0.19392E+00_JPRB, 0.19038E+00_JPRB, 0.18685E+00_JPRB, 0.18258E+00_JPRB ,&
&  0.18042E+00_JPRB, 0.18283E+00_JPRB, 0.19142E+00_JPRB, 0.20096E+00_JPRB, 0.20586E+00_JPRB, 0.19432E+00_JPRB, 0.19028E+00_JPRB ,&
&  0.18945E+00_JPRB, 0.18916E+00_JPRB, 0.18946E+00_JPRB, 0.19087E+00_JPRB, 0.19157E+00_JPRB, 0.19132E+00_JPRB, 0.19187E+00_JPRB ,&
&  0.19383E+00_JPRB, 0.19586E+00_JPRB, 0.19692E+00_JPRB, 0.19811E+00_JPRB, 0.19884E+00_JPRB, 0.19865E+00_JPRB, 0.19832E+00_JPRB ,&
&  0.19897E+00_JPRB, 0.20193E+00_JPRB, 0.20679E+00_JPRB, 0.21371E+00_JPRB, 0.21871E+00_JPRB, 0.22333E+00_JPRB, 0.22924E+00_JPRB ,&
&  0.23680E+00_JPRB, 0.24428E+00_JPRB, 0.24860E+00_JPRB, 0.25252E+00_JPRB, 0.25444E+00_JPRB /)
ZBURD(:, 66,  6) = (/ &
&  0.24794E+00_JPRB, 0.24862E+00_JPRB, 0.24503E+00_JPRB, 0.24188E+00_JPRB, 0.24031E+00_JPRB, 0.23984E+00_JPRB, 0.24107E+00_JPRB ,&
&  0.24403E+00_JPRB, 0.24633E+00_JPRB, 0.24870E+00_JPRB, 0.25077E+00_JPRB, 0.25163E+00_JPRB, 0.25140E+00_JPRB, 0.25231E+00_JPRB ,&
&  0.25319E+00_JPRB, 0.25562E+00_JPRB, 0.25700E+00_JPRB, 0.25950E+00_JPRB, 0.25962E+00_JPRB, 0.26028E+00_JPRB, 0.26146E+00_JPRB ,&
&  0.26101E+00_JPRB, 0.26151E+00_JPRB, 0.26293E+00_JPRB, 0.26069E+00_JPRB, 0.25923E+00_JPRB, 0.25705E+00_JPRB, 0.25536E+00_JPRB ,&
&  0.24913E+00_JPRB, 0.24257E+00_JPRB, 0.23602E+00_JPRB, 0.23960E+00_JPRB, 0.24754E+00_JPRB, 0.24934E+00_JPRB, 0.25334E+00_JPRB ,&
&  0.25451E+00_JPRB, 0.25280E+00_JPRB, 0.25102E+00_JPRB, 0.24770E+00_JPRB, 0.24448E+00_JPRB, 0.24086E+00_JPRB, 0.23797E+00_JPRB ,&
&  0.23479E+00_JPRB, 0.23529E+00_JPRB, 0.23124E+00_JPRB, 0.21775E+00_JPRB, 0.21074E+00_JPRB, 0.20640E+00_JPRB, 0.19953E+00_JPRB ,&
&  0.19263E+00_JPRB, 0.19069E+00_JPRB, 0.19025E+00_JPRB, 0.18789E+00_JPRB, 0.18775E+00_JPRB, 0.18805E+00_JPRB, 0.18797E+00_JPRB ,&
&  0.18664E+00_JPRB, 0.18637E+00_JPRB, 0.18506E+00_JPRB, 0.18311E+00_JPRB, 0.18124E+00_JPRB, 0.18048E+00_JPRB, 0.18088E+00_JPRB ,&
&  0.18237E+00_JPRB, 0.18396E+00_JPRB, 0.18949E+00_JPRB, 0.19297E+00_JPRB, 0.18824E+00_JPRB, 0.18561E+00_JPRB, 0.18599E+00_JPRB ,&
&  0.18210E+00_JPRB, 0.18133E+00_JPRB, 0.18131E+00_JPRB, 0.18127E+00_JPRB, 0.18162E+00_JPRB, 0.18251E+00_JPRB, 0.18242E+00_JPRB ,&
&  0.18306E+00_JPRB, 0.18438E+00_JPRB, 0.18474E+00_JPRB, 0.18477E+00_JPRB, 0.18452E+00_JPRB, 0.18498E+00_JPRB, 0.18622E+00_JPRB ,&
&  0.18764E+00_JPRB, 0.19031E+00_JPRB, 0.19311E+00_JPRB, 0.19720E+00_JPRB, 0.20046E+00_JPRB, 0.20302E+00_JPRB, 0.20766E+00_JPRB ,&
&  0.21173E+00_JPRB, 0.22102E+00_JPRB, 0.23675E+00_JPRB, 0.24714E+00_JPRB, 0.24363E+00_JPRB /)
ZBURD(:, 67,  6) = (/ &
&  0.23817E+00_JPRB, 0.23701E+00_JPRB, 0.23571E+00_JPRB, 0.23443E+00_JPRB, 0.23533E+00_JPRB, 0.23649E+00_JPRB, 0.23801E+00_JPRB ,&
&  0.23975E+00_JPRB, 0.24020E+00_JPRB, 0.24010E+00_JPRB, 0.23789E+00_JPRB, 0.23662E+00_JPRB, 0.23046E+00_JPRB, 0.22825E+00_JPRB ,&
&  0.22661E+00_JPRB, 0.22849E+00_JPRB, 0.23107E+00_JPRB, 0.23501E+00_JPRB, 0.23664E+00_JPRB, 0.23838E+00_JPRB, 0.23743E+00_JPRB ,&
&  0.23526E+00_JPRB, 0.23263E+00_JPRB, 0.23010E+00_JPRB, 0.22810E+00_JPRB, 0.22752E+00_JPRB, 0.22589E+00_JPRB, 0.22417E+00_JPRB ,&
&  0.22419E+00_JPRB, 0.22567E+00_JPRB, 0.22649E+00_JPRB, 0.22731E+00_JPRB, 0.23324E+00_JPRB, 0.23380E+00_JPRB, 0.23593E+00_JPRB ,&
&  0.23652E+00_JPRB, 0.23714E+00_JPRB, 0.23649E+00_JPRB, 0.23511E+00_JPRB, 0.23379E+00_JPRB, 0.22828E+00_JPRB, 0.22583E+00_JPRB ,&
&  0.21859E+00_JPRB, 0.21433E+00_JPRB, 0.20752E+00_JPRB, 0.20139E+00_JPRB, 0.19463E+00_JPRB, 0.18957E+00_JPRB, 0.18536E+00_JPRB ,&
&  0.18201E+00_JPRB, 0.17951E+00_JPRB, 0.17744E+00_JPRB, 0.17715E+00_JPRB, 0.17721E+00_JPRB, 0.17681E+00_JPRB, 0.17696E+00_JPRB ,&
&  0.17780E+00_JPRB, 0.17871E+00_JPRB, 0.17864E+00_JPRB, 0.17922E+00_JPRB, 0.17974E+00_JPRB, 0.18051E+00_JPRB, 0.18197E+00_JPRB ,&
&  0.18423E+00_JPRB, 0.18616E+00_JPRB, 0.18687E+00_JPRB, 0.18872E+00_JPRB, 0.18805E+00_JPRB, 0.18539E+00_JPRB, 0.18278E+00_JPRB ,&
&  0.17985E+00_JPRB, 0.17856E+00_JPRB, 0.17715E+00_JPRB, 0.17633E+00_JPRB, 0.17620E+00_JPRB, 0.17639E+00_JPRB, 0.17717E+00_JPRB ,&
&  0.17802E+00_JPRB, 0.17892E+00_JPRB, 0.17927E+00_JPRB, 0.17973E+00_JPRB, 0.18014E+00_JPRB, 0.18099E+00_JPRB, 0.18260E+00_JPRB ,&
&  0.18480E+00_JPRB, 0.18732E+00_JPRB, 0.18978E+00_JPRB, 0.19279E+00_JPRB, 0.19775E+00_JPRB, 0.20439E+00_JPRB, 0.21440E+00_JPRB ,&
&  0.22233E+00_JPRB, 0.23479E+00_JPRB, 0.23486E+00_JPRB, 0.23701E+00_JPRB, 0.23848E+00_JPRB /)
ZBURD(:, 68,  6) = (/ &
&  0.23096E+00_JPRB, 0.23090E+00_JPRB, 0.23014E+00_JPRB, 0.22966E+00_JPRB, 0.23014E+00_JPRB, 0.23094E+00_JPRB, 0.23196E+00_JPRB ,&
&  0.22851E+00_JPRB, 0.22596E+00_JPRB, 0.22167E+00_JPRB, 0.21690E+00_JPRB, 0.21271E+00_JPRB, 0.20943E+00_JPRB, 0.20753E+00_JPRB ,&
&  0.20732E+00_JPRB, 0.20941E+00_JPRB, 0.21101E+00_JPRB, 0.21301E+00_JPRB, 0.21310E+00_JPRB, 0.21325E+00_JPRB, 0.21182E+00_JPRB ,&
&  0.21030E+00_JPRB, 0.20970E+00_JPRB, 0.20996E+00_JPRB, 0.21163E+00_JPRB, 0.21430E+00_JPRB, 0.21608E+00_JPRB, 0.21822E+00_JPRB ,&
&  0.22110E+00_JPRB, 0.22039E+00_JPRB, 0.22112E+00_JPRB, 0.22051E+00_JPRB, 0.22085E+00_JPRB, 0.22182E+00_JPRB, 0.22229E+00_JPRB ,&
&  0.22327E+00_JPRB, 0.22423E+00_JPRB, 0.22416E+00_JPRB, 0.22337E+00_JPRB, 0.21905E+00_JPRB, 0.21644E+00_JPRB, 0.21176E+00_JPRB ,&
&  0.20620E+00_JPRB, 0.20123E+00_JPRB, 0.19554E+00_JPRB, 0.19152E+00_JPRB, 0.18679E+00_JPRB, 0.18399E+00_JPRB, 0.18183E+00_JPRB ,&
&  0.17980E+00_JPRB, 0.17904E+00_JPRB, 0.17795E+00_JPRB, 0.17677E+00_JPRB, 0.17649E+00_JPRB, 0.17532E+00_JPRB, 0.17523E+00_JPRB ,&
&  0.17553E+00_JPRB, 0.17579E+00_JPRB, 0.17678E+00_JPRB, 0.17724E+00_JPRB, 0.17803E+00_JPRB, 0.17876E+00_JPRB, 0.17930E+00_JPRB ,&
&  0.18042E+00_JPRB, 0.18187E+00_JPRB, 0.18185E+00_JPRB, 0.18216E+00_JPRB, 0.18132E+00_JPRB, 0.17952E+00_JPRB, 0.17757E+00_JPRB ,&
&  0.17546E+00_JPRB, 0.17381E+00_JPRB, 0.17331E+00_JPRB, 0.17258E+00_JPRB, 0.17273E+00_JPRB, 0.17310E+00_JPRB, 0.17327E+00_JPRB ,&
&  0.17394E+00_JPRB, 0.17454E+00_JPRB, 0.17538E+00_JPRB, 0.17726E+00_JPRB, 0.17857E+00_JPRB, 0.18027E+00_JPRB, 0.18205E+00_JPRB ,&
&  0.18346E+00_JPRB, 0.18602E+00_JPRB, 0.18829E+00_JPRB, 0.19141E+00_JPRB, 0.19858E+00_JPRB, 0.20483E+00_JPRB, 0.21599E+00_JPRB ,&
&  0.22108E+00_JPRB, 0.23217E+00_JPRB, 0.23008E+00_JPRB, 0.23018E+00_JPRB, 0.23132E+00_JPRB /)
ZBURD(:, 69,  6) = (/ &
&  0.22549E+00_JPRB, 0.22639E+00_JPRB, 0.22634E+00_JPRB, 0.22733E+00_JPRB, 0.22813E+00_JPRB, 0.22921E+00_JPRB, 0.22932E+00_JPRB ,&
&  0.22879E+00_JPRB, 0.22467E+00_JPRB, 0.22242E+00_JPRB, 0.21912E+00_JPRB, 0.21600E+00_JPRB, 0.21272E+00_JPRB, 0.21002E+00_JPRB ,&
&  0.20828E+00_JPRB, 0.20692E+00_JPRB, 0.20566E+00_JPRB, 0.20394E+00_JPRB, 0.20182E+00_JPRB, 0.19967E+00_JPRB, 0.19771E+00_JPRB ,&
&  0.19593E+00_JPRB, 0.19600E+00_JPRB, 0.19724E+00_JPRB, 0.19945E+00_JPRB, 0.20349E+00_JPRB, 0.20655E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.21658E+00_JPRB, 0.21326E+00_JPRB, 0.21344E+00_JPRB, 0.21375E+00_JPRB, 0.21351E+00_JPRB, 0.21335E+00_JPRB, 0.21056E+00_JPRB ,&
&  0.21428E+00_JPRB, 0.21448E+00_JPRB, 0.21797E+00_JPRB, 0.21697E+00_JPRB, 0.21230E+00_JPRB, 0.20866E+00_JPRB, 0.20365E+00_JPRB ,&
&  0.20010E+00_JPRB, 0.19560E+00_JPRB, 0.19257E+00_JPRB, 0.18979E+00_JPRB, 0.18648E+00_JPRB, 0.18512E+00_JPRB, 0.18283E+00_JPRB ,&
&  0.18144E+00_JPRB, 0.18114E+00_JPRB, 0.18016E+00_JPRB, 0.18012E+00_JPRB, 0.17929E+00_JPRB, 0.17811E+00_JPRB, 0.17734E+00_JPRB ,&
&  0.17665E+00_JPRB, 0.17592E+00_JPRB, 0.17567E+00_JPRB, 0.17540E+00_JPRB, 0.17552E+00_JPRB, 0.17552E+00_JPRB, 0.17556E+00_JPRB ,&
&  0.17576E+00_JPRB, 0.17590E+00_JPRB, 0.17548E+00_JPRB, 0.17533E+00_JPRB, 0.17442E+00_JPRB, 0.17362E+00_JPRB, 0.17238E+00_JPRB ,&
&  0.17161E+00_JPRB, 0.17115E+00_JPRB, 0.17044E+00_JPRB, 0.16999E+00_JPRB, 0.16986E+00_JPRB, 0.17035E+00_JPRB, 0.17098E+00_JPRB ,&
&  0.17170E+00_JPRB, 0.17302E+00_JPRB, 0.17410E+00_JPRB, 0.17556E+00_JPRB, 0.17718E+00_JPRB, 0.17845E+00_JPRB, 0.18028E+00_JPRB ,&
&  0.18209E+00_JPRB, 0.18396E+00_JPRB, 0.18715E+00_JPRB, 0.19068E+00_JPRB, 0.19535E+00_JPRB, 0.20047E+00_JPRB, 0.20317E+00_JPRB ,&
&  0.20619E+00_JPRB, 0.21290E+00_JPRB, 0.21557E+00_JPRB, 0.22335E+00_JPRB, 0.22462E+00_JPRB /)
ZBURD(:, 70,  6) = (/ &
&  0.21585E+00_JPRB, 0.21744E+00_JPRB, 0.21898E+00_JPRB, 0.22043E+00_JPRB, 0.22210E+00_JPRB, 0.22278E+00_JPRB, 0.22277E+00_JPRB ,&
&  0.22045E+00_JPRB, 0.21994E+00_JPRB, 0.21846E+00_JPRB, 0.21657E+00_JPRB, 0.21506E+00_JPRB, 0.21206E+00_JPRB, 0.20925E+00_JPRB ,&
&  0.20643E+00_JPRB, 0.20370E+00_JPRB, 0.20129E+00_JPRB, 0.19797E+00_JPRB, 0.19479E+00_JPRB, 0.19164E+00_JPRB, 0.19075E+00_JPRB ,&
&  0.19042E+00_JPRB, 0.19097E+00_JPRB, 0.19273E+00_JPRB, 0.19414E+00_JPRB, 0.19619E+00_JPRB, 0.19837E+00_JPRB, 0.20167E+00_JPRB ,&
&  0.20479E+00_JPRB, 0.20709E+00_JPRB, 0.20738E+00_JPRB, 0.20619E+00_JPRB, 0.20522E+00_JPRB, 0.20496E+00_JPRB, 0.20458E+00_JPRB ,&
&  0.20383E+00_JPRB, 0.20344E+00_JPRB, 0.20284E+00_JPRB, 0.20234E+00_JPRB, 0.20091E+00_JPRB, 0.19836E+00_JPRB, 0.19577E+00_JPRB ,&
&  0.19335E+00_JPRB, 0.19121E+00_JPRB, 0.18916E+00_JPRB, 0.18738E+00_JPRB, 0.18633E+00_JPRB, 0.18527E+00_JPRB, 0.18393E+00_JPRB ,&
&  0.18315E+00_JPRB, 0.18232E+00_JPRB, 0.18138E+00_JPRB, 0.18069E+00_JPRB, 0.17992E+00_JPRB, 0.17937E+00_JPRB, 0.17874E+00_JPRB ,&
&  0.17827E+00_JPRB, 0.17753E+00_JPRB, 0.17690E+00_JPRB, 0.17620E+00_JPRB, 0.17588E+00_JPRB, 0.17509E+00_JPRB, 0.17434E+00_JPRB ,&
&  0.17340E+00_JPRB, 0.17282E+00_JPRB, 0.17175E+00_JPRB, 0.17111E+00_JPRB, 0.17036E+00_JPRB, 0.16968E+00_JPRB, 0.16921E+00_JPRB ,&
&  0.16926E+00_JPRB, 0.16924E+00_JPRB, 0.16878E+00_JPRB, 0.16867E+00_JPRB, 0.16870E+00_JPRB, 0.16912E+00_JPRB, 0.17044E+00_JPRB ,&
&  0.17128E+00_JPRB, 0.17221E+00_JPRB, 0.17312E+00_JPRB, 0.17381E+00_JPRB, 0.17500E+00_JPRB, 0.17614E+00_JPRB, 0.17731E+00_JPRB ,&
&  0.17869E+00_JPRB, 0.17988E+00_JPRB, 0.18127E+00_JPRB, 0.18291E+00_JPRB, 0.18693E+00_JPRB, 0.18969E+00_JPRB, 0.19257E+00_JPRB ,&
&  0.19459E+00_JPRB, 0.19319E+00_JPRB, 0.20736E+00_JPRB, 0.21010E+00_JPRB, 0.21217E+00_JPRB /)
ZBURD(:, 71,  6) = (/ &
&  0.19090E+00_JPRB, 0.19382E+00_JPRB, 0.19564E+00_JPRB, 0.19716E+00_JPRB, 0.19102E+00_JPRB, 0.20239E+00_JPRB, 0.19913E+00_JPRB ,&
&  0.19860E+00_JPRB, 0.19744E+00_JPRB, 0.19918E+00_JPRB, 0.19929E+00_JPRB, 0.19871E+00_JPRB, 0.19782E+00_JPRB, 0.19666E+00_JPRB ,&
&  0.19555E+00_JPRB, 0.19414E+00_JPRB, 0.19324E+00_JPRB, 0.19226E+00_JPRB, 0.19149E+00_JPRB, 0.19072E+00_JPRB, 0.19018E+00_JPRB ,&
&  0.19020E+00_JPRB, 0.19101E+00_JPRB, 0.19247E+00_JPRB, 0.19184E+00_JPRB, 0.19182E+00_JPRB, 0.19219E+00_JPRB, 0.19278E+00_JPRB ,&
&  0.19301E+00_JPRB, 0.19309E+00_JPRB, 0.19279E+00_JPRB, 0.19169E+00_JPRB, 0.19170E+00_JPRB, 0.19176E+00_JPRB, 0.19190E+00_JPRB ,&
&  0.19185E+00_JPRB, 0.19157E+00_JPRB, 0.19070E+00_JPRB, 0.18989E+00_JPRB, 0.18906E+00_JPRB, 0.18805E+00_JPRB, 0.18685E+00_JPRB ,&
&  0.18585E+00_JPRB, 0.18508E+00_JPRB, 0.18414E+00_JPRB, 0.18321E+00_JPRB, 0.18241E+00_JPRB, 0.18164E+00_JPRB, 0.18091E+00_JPRB ,&
&  0.18044E+00_JPRB, 0.18009E+00_JPRB, 0.17971E+00_JPRB, 0.17915E+00_JPRB, 0.17853E+00_JPRB, 0.17794E+00_JPRB, 0.17740E+00_JPRB ,&
&  0.17705E+00_JPRB, 0.17647E+00_JPRB, 0.17590E+00_JPRB, 0.17526E+00_JPRB, 0.17478E+00_JPRB, 0.17399E+00_JPRB, 0.17330E+00_JPRB ,&
&  0.17263E+00_JPRB, 0.17202E+00_JPRB, 0.17155E+00_JPRB, 0.17121E+00_JPRB, 0.17090E+00_JPRB, 0.17068E+00_JPRB, 0.17052E+00_JPRB ,&
&  0.17042E+00_JPRB, 0.17034E+00_JPRB, 0.17035E+00_JPRB, 0.17029E+00_JPRB, 0.17020E+00_JPRB, 0.17018E+00_JPRB, 0.17046E+00_JPRB ,&
&  0.17069E+00_JPRB, 0.17086E+00_JPRB, 0.17096E+00_JPRB, 0.17066E+00_JPRB, 0.17106E+00_JPRB, 0.17141E+00_JPRB, 0.17183E+00_JPRB ,&
&  0.17220E+00_JPRB, 0.17299E+00_JPRB, 0.17395E+00_JPRB, 0.17512E+00_JPRB, 0.17735E+00_JPRB, 0.17933E+00_JPRB, 0.18143E+00_JPRB ,&
&  0.18379E+00_JPRB, 0.18720E+00_JPRB, 0.18861E+00_JPRB, 0.18954E+00_JPRB, 0.19026E+00_JPRB /)
ZBURD(:, 72,  6) = (/ &
&  0.17988E+00_JPRB, 0.17984E+00_JPRB, 0.17966E+00_JPRB, 0.17890E+00_JPRB, 0.17661E+00_JPRB, 0.17618E+00_JPRB, 0.17614E+00_JPRB ,&
&  0.17631E+00_JPRB, 0.17641E+00_JPRB, 0.17729E+00_JPRB, 0.17825E+00_JPRB, 0.17919E+00_JPRB, 0.17992E+00_JPRB, 0.18055E+00_JPRB ,&
&  0.18090E+00_JPRB, 0.18125E+00_JPRB, 0.18014E+00_JPRB, 0.17959E+00_JPRB, 0.17943E+00_JPRB, 0.17942E+00_JPRB, 0.17942E+00_JPRB ,&
&  0.17935E+00_JPRB, 0.17933E+00_JPRB, 0.17919E+00_JPRB, 0.17939E+00_JPRB, 0.17969E+00_JPRB, 0.17995E+00_JPRB, 0.18025E+00_JPRB ,&
&  0.18047E+00_JPRB, 0.18068E+00_JPRB, 0.18081E+00_JPRB, 0.18090E+00_JPRB, 0.18122E+00_JPRB, 0.18138E+00_JPRB, 0.18130E+00_JPRB ,&
&  0.18112E+00_JPRB, 0.18077E+00_JPRB, 0.18037E+00_JPRB, 0.17996E+00_JPRB, 0.17966E+00_JPRB, 0.17937E+00_JPRB, 0.17909E+00_JPRB ,&
&  0.17879E+00_JPRB, 0.17849E+00_JPRB, 0.17815E+00_JPRB, 0.17778E+00_JPRB, 0.17742E+00_JPRB, 0.17716E+00_JPRB, 0.17688E+00_JPRB ,&
&  0.17665E+00_JPRB, 0.17642E+00_JPRB, 0.17618E+00_JPRB, 0.17587E+00_JPRB, 0.17554E+00_JPRB, 0.17521E+00_JPRB, 0.17485E+00_JPRB ,&
&  0.17439E+00_JPRB, 0.17407E+00_JPRB, 0.17375E+00_JPRB, 0.17346E+00_JPRB, 0.17322E+00_JPRB, 0.17297E+00_JPRB, 0.17274E+00_JPRB ,&
&  0.17249E+00_JPRB, 0.17226E+00_JPRB, 0.17199E+00_JPRB, 0.17174E+00_JPRB, 0.17151E+00_JPRB, 0.17131E+00_JPRB, 0.17114E+00_JPRB ,&
&  0.17099E+00_JPRB, 0.17082E+00_JPRB, 0.17068E+00_JPRB, 0.17069E+00_JPRB, 0.17066E+00_JPRB, 0.17069E+00_JPRB, 0.17071E+00_JPRB ,&
&  0.17074E+00_JPRB, 0.17075E+00_JPRB, 0.17076E+00_JPRB, 0.17064E+00_JPRB, 0.17079E+00_JPRB, 0.17100E+00_JPRB, 0.17132E+00_JPRB ,&
&  0.17169E+00_JPRB, 0.17217E+00_JPRB, 0.17268E+00_JPRB, 0.17321E+00_JPRB, 0.17384E+00_JPRB, 0.17436E+00_JPRB, 0.17494E+00_JPRB ,&
&  0.17561E+00_JPRB, 0.17629E+00_JPRB, 0.17679E+00_JPRB, 0.17736E+00_JPRB, 0.17825E+00_JPRB /)
ZBURD(:, 73,  6) = (/ &
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB ,&
&  0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB, 0.17386E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART12',1,ZHOOK_HANDLE)
END SUBROUTINE PART12



SUBROUTINE PART13

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART13',0,ZHOOK_HANDLE)


!-- month=  7
ZBURD(:,  1,  7) = (/ &
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB ,&
&  0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB, 0.12461E+01_JPRB /)
ZBURD(:,  2,  7) = (/ &
&  0.12874E+01_JPRB, 0.12848E+01_JPRB, 0.12827E+01_JPRB, 0.12814E+01_JPRB, 0.12805E+01_JPRB, 0.12795E+01_JPRB, 0.12763E+01_JPRB ,&
&  0.12723E+01_JPRB, 0.12670E+01_JPRB, 0.12616E+01_JPRB, 0.12550E+01_JPRB, 0.12466E+01_JPRB, 0.12364E+01_JPRB, 0.12272E+01_JPRB ,&
&  0.12187E+01_JPRB, 0.12117E+01_JPRB, 0.12044E+01_JPRB, 0.11993E+01_JPRB, 0.11938E+01_JPRB, 0.11884E+01_JPRB, 0.11825E+01_JPRB ,&
&  0.11776E+01_JPRB, 0.11729E+01_JPRB, 0.11684E+01_JPRB, 0.11644E+01_JPRB, 0.11625E+01_JPRB, 0.11602E+01_JPRB, 0.11577E+01_JPRB ,&
&  0.11555E+01_JPRB, 0.11538E+01_JPRB, 0.11523E+01_JPRB, 0.11508E+01_JPRB, 0.11498E+01_JPRB, 0.11496E+01_JPRB, 0.11496E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11520E+01_JPRB, 0.11554E+01_JPRB, 0.11584E+01_JPRB, 0.11620E+01_JPRB, 0.11674E+01_JPRB, 0.11747E+01_JPRB ,&
&  0.11814E+01_JPRB, 0.11880E+01_JPRB, 0.11956E+01_JPRB, 0.12042E+01_JPRB, 0.12122E+01_JPRB, 0.12199E+01_JPRB, 0.12269E+01_JPRB ,&
&  0.12340E+01_JPRB, 0.12397E+01_JPRB, 0.12457E+01_JPRB, 0.12486E+01_JPRB, 0.12534E+01_JPRB, 0.12576E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.12643E+01_JPRB, 0.12654E+01_JPRB, 0.12675E+01_JPRB, 0.12692E+01_JPRB, 0.12711E+01_JPRB, 0.12733E+01_JPRB ,&
&  0.12762E+01_JPRB, 0.12781E+01_JPRB, 0.12805E+01_JPRB, 0.12824E+01_JPRB, 0.12845E+01_JPRB, 0.12852E+01_JPRB, 0.12863E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.12894E+01_JPRB, 0.12891E+01_JPRB, 0.12890E+01_JPRB, 0.12889E+01_JPRB, 0.12890E+01_JPRB, 0.12891E+01_JPRB ,&
&  0.12894E+01_JPRB, 0.12896E+01_JPRB, 0.12898E+01_JPRB, 0.12888E+01_JPRB, 0.12870E+01_JPRB, 0.12856E+01_JPRB, 0.12844E+01_JPRB ,&
&  0.12842E+01_JPRB, 0.12839E+01_JPRB, 0.12849E+01_JPRB, 0.12863E+01_JPRB, 0.12896E+01_JPRB, 0.12909E+01_JPRB, 0.12923E+01_JPRB ,&
&  0.12931E+01_JPRB, 0.12939E+01_JPRB, 0.12931E+01_JPRB, 0.12918E+01_JPRB, 0.12897E+01_JPRB /)
ZBURD(:,  3,  7) = (/ &
&  0.12929E+01_JPRB, 0.12912E+01_JPRB, 0.12907E+01_JPRB, 0.12921E+01_JPRB, 0.12938E+01_JPRB, 0.12944E+01_JPRB, 0.12930E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.12758E+01_JPRB, 0.12616E+01_JPRB, 0.12462E+01_JPRB, 0.12282E+01_JPRB, 0.12110E+01_JPRB, 0.12006E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11800E+01_JPRB, 0.11693E+01_JPRB, 0.11612E+01_JPRB, 0.11516E+01_JPRB, 0.11418E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11241E+01_JPRB, 0.11159E+01_JPRB, 0.11068E+01_JPRB, 0.10958E+01_JPRB, 0.10877E+01_JPRB, 0.10792E+01_JPRB, 0.10713E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10557E+01_JPRB, 0.10517E+01_JPRB, 0.10496E+01_JPRB, 0.10535E+01_JPRB, 0.10573E+01_JPRB, 0.10570E+01_JPRB ,&
&  0.10546E+01_JPRB, 0.10464E+01_JPRB, 0.10465E+01_JPRB, 0.10479E+01_JPRB, 0.10525E+01_JPRB, 0.10621E+01_JPRB, 0.10760E+01_JPRB ,&
&  0.10973E+01_JPRB, 0.11241E+01_JPRB, 0.11539E+01_JPRB, 0.11792E+01_JPRB, 0.12051E+01_JPRB, 0.12286E+01_JPRB, 0.12462E+01_JPRB ,&
&  0.12666E+01_JPRB, 0.12822E+01_JPRB, 0.12976E+01_JPRB, 0.13122E+01_JPRB, 0.13317E+01_JPRB, 0.13475E+01_JPRB, 0.13603E+01_JPRB ,&
&  0.13640E+01_JPRB, 0.13634E+01_JPRB, 0.13628E+01_JPRB, 0.13630E+01_JPRB, 0.13600E+01_JPRB, 0.13578E+01_JPRB, 0.13576E+01_JPRB ,&
&  0.13591E+01_JPRB, 0.13600E+01_JPRB, 0.13636E+01_JPRB, 0.13678E+01_JPRB, 0.13723E+01_JPRB, 0.13716E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.13691E+01_JPRB, 0.13676E+01_JPRB, 0.13622E+01_JPRB, 0.13526E+01_JPRB, 0.13445E+01_JPRB, 0.13392E+01_JPRB, 0.13321E+01_JPRB ,&
&  0.13252E+01_JPRB, 0.13197E+01_JPRB, 0.13160E+01_JPRB, 0.13115E+01_JPRB, 0.13044E+01_JPRB, 0.12980E+01_JPRB, 0.12937E+01_JPRB ,&
&  0.12908E+01_JPRB, 0.12868E+01_JPRB, 0.12869E+01_JPRB, 0.12874E+01_JPRB, 0.12951E+01_JPRB, 0.12981E+01_JPRB, 0.13010E+01_JPRB ,&
&  0.13014E+01_JPRB, 0.13041E+01_JPRB, 0.13037E+01_JPRB, 0.13005E+01_JPRB, 0.12960E+01_JPRB /)
ZBURD(:,  4,  7) = (/ &
&  0.13094E+01_JPRB, 0.13131E+01_JPRB, 0.13223E+01_JPRB, 0.13268E+01_JPRB, 0.13283E+01_JPRB, 0.13244E+01_JPRB, 0.13214E+01_JPRB ,&
&  0.13119E+01_JPRB, 0.12933E+01_JPRB, 0.12627E+01_JPRB, 0.12371E+01_JPRB, 0.12136E+01_JPRB, 0.11970E+01_JPRB, 0.11843E+01_JPRB ,&
&  0.11711E+01_JPRB, 0.11605E+01_JPRB, 0.11467E+01_JPRB, 0.11319E+01_JPRB, 0.11136E+01_JPRB, 0.10930E+01_JPRB, 0.10671E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.10273E+01_JPRB, 0.10113E+01_JPRB, 0.10005E+01_JPRB, 0.99730E+00_JPRB, 0.94734E+00_JPRB, 0.93697E+00_JPRB ,&
&  0.95099E+00_JPRB, 0.92881E+00_JPRB, 0.90811E+00_JPRB, 0.87625E+00_JPRB, 0.87272E+00_JPRB, 0.88573E+00_JPRB, 0.89438E+00_JPRB ,&
&  0.86624E+00_JPRB, 0.92055E+00_JPRB, 0.92829E+00_JPRB, 0.93147E+00_JPRB, 0.92745E+00_JPRB, 0.91474E+00_JPRB, 0.93374E+00_JPRB ,&
&  0.98559E+00_JPRB, 0.10062E+01_JPRB, 0.10540E+01_JPRB, 0.11121E+01_JPRB, 0.11592E+01_JPRB, 0.12091E+01_JPRB, 0.12589E+01_JPRB ,&
&  0.13116E+01_JPRB, 0.13505E+01_JPRB, 0.13765E+01_JPRB, 0.14014E+01_JPRB, 0.14415E+01_JPRB, 0.14817E+01_JPRB, 0.15184E+01_JPRB ,&
&  0.15444E+01_JPRB, 0.15600E+01_JPRB, 0.15778E+01_JPRB, 0.15955E+01_JPRB, 0.16188E+01_JPRB, 0.16378E+01_JPRB, 0.16574E+01_JPRB ,&
&  0.16714E+01_JPRB, 0.16832E+01_JPRB, 0.16881E+01_JPRB, 0.16886E+01_JPRB, 0.16849E+01_JPRB, 0.16750E+01_JPRB, 0.16701E+01_JPRB ,&
&  0.16531E+01_JPRB, 0.16336E+01_JPRB, 0.16067E+01_JPRB, 0.15742E+01_JPRB, 0.15474E+01_JPRB, 0.15252E+01_JPRB, 0.15012E+01_JPRB ,&
&  0.14771E+01_JPRB, 0.14529E+01_JPRB, 0.14295E+01_JPRB, 0.14049E+01_JPRB, 0.13813E+01_JPRB, 0.13573E+01_JPRB, 0.13390E+01_JPRB ,&
&  0.13266E+01_JPRB, 0.13163E+01_JPRB, 0.13129E+01_JPRB, 0.13081E+01_JPRB, 0.13112E+01_JPRB, 0.13111E+01_JPRB, 0.13169E+01_JPRB ,&
&  0.13193E+01_JPRB, 0.13162E+01_JPRB, 0.13088E+01_JPRB, 0.13054E+01_JPRB, 0.13048E+01_JPRB /)
ZBURD(:,  5,  7) = (/ &
&  0.12977E+01_JPRB, 0.13107E+01_JPRB, 0.13345E+01_JPRB, 0.13399E+01_JPRB, 0.13410E+01_JPRB, 0.13303E+01_JPRB, 0.13262E+01_JPRB ,&
&  0.13086E+01_JPRB, 0.12881E+01_JPRB, 0.12429E+01_JPRB, 0.12137E+01_JPRB, 0.11899E+01_JPRB, 0.11769E+01_JPRB, 0.11685E+01_JPRB ,&
&  0.11618E+01_JPRB, 0.11512E+01_JPRB, 0.11359E+01_JPRB, 0.11126E+01_JPRB, 0.10821E+01_JPRB, 0.10473E+01_JPRB, 0.10005E+01_JPRB ,&
&  0.95989E+00_JPRB, 0.92997E+00_JPRB, 0.89148E+00_JPRB, 0.89760E+00_JPRB, 0.87745E+00_JPRB, 0.86303E+00_JPRB, 0.85354E+00_JPRB ,&
&  0.82659E+00_JPRB, 0.80194E+00_JPRB, 0.76214E+00_JPRB, 0.73831E+00_JPRB, 0.71966E+00_JPRB, 0.69785E+00_JPRB, 0.72014E+00_JPRB ,&
&  0.73900E+00_JPRB, 0.79625E+00_JPRB, 0.82586E+00_JPRB, 0.83980E+00_JPRB, 0.85244E+00_JPRB, 0.85205E+00_JPRB, 0.86547E+00_JPRB ,&
&  0.88201E+00_JPRB, 0.90935E+00_JPRB, 0.10033E+01_JPRB, 0.10682E+01_JPRB, 0.11094E+01_JPRB, 0.11543E+01_JPRB, 0.11918E+01_JPRB ,&
&  0.12414E+01_JPRB, 0.12780E+01_JPRB, 0.13064E+01_JPRB, 0.13386E+01_JPRB, 0.13974E+01_JPRB, 0.14664E+01_JPRB, 0.15282E+01_JPRB ,&
&  0.15805E+01_JPRB, 0.16165E+01_JPRB, 0.16612E+01_JPRB, 0.17124E+01_JPRB, 0.17800E+01_JPRB, 0.18544E+01_JPRB, 0.19331E+01_JPRB ,&
&  0.20100E+01_JPRB, 0.20849E+01_JPRB, 0.21465E+01_JPRB, 0.21990E+01_JPRB, 0.22261E+01_JPRB, 0.22326E+01_JPRB, 0.22276E+01_JPRB ,&
&  0.21882E+01_JPRB, 0.21419E+01_JPRB, 0.20737E+01_JPRB, 0.19707E+01_JPRB, 0.18888E+01_JPRB, 0.18409E+01_JPRB, 0.17592E+01_JPRB ,&
&  0.16884E+01_JPRB, 0.16187E+01_JPRB, 0.15534E+01_JPRB, 0.14889E+01_JPRB, 0.14320E+01_JPRB, 0.13817E+01_JPRB, 0.13473E+01_JPRB ,&
&  0.13207E+01_JPRB, 0.13024E+01_JPRB, 0.12922E+01_JPRB, 0.12847E+01_JPRB, 0.12857E+01_JPRB, 0.12836E+01_JPRB, 0.12936E+01_JPRB ,&
&  0.12957E+01_JPRB, 0.12934E+01_JPRB, 0.12831E+01_JPRB, 0.12825E+01_JPRB, 0.12861E+01_JPRB /)
ZBURD(:,  6,  7) = (/ &
&  0.12827E+01_JPRB, 0.12923E+01_JPRB, 0.12972E+01_JPRB, 0.12795E+01_JPRB, 0.12716E+01_JPRB, 0.12649E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.12648E+01_JPRB, 0.12531E+01_JPRB, 0.12125E+01_JPRB, 0.11857E+01_JPRB, 0.11595E+01_JPRB, 0.11483E+01_JPRB, 0.11373E+01_JPRB ,&
&  0.11297E+01_JPRB, 0.11165E+01_JPRB, 0.11065E+01_JPRB, 0.10816E+01_JPRB, 0.10559E+01_JPRB, 0.10173E+01_JPRB, 0.97369E+00_JPRB ,&
&  0.92868E+00_JPRB, 0.88392E+00_JPRB, 0.84474E+00_JPRB, 0.84035E+00_JPRB, 0.82554E+00_JPRB, 0.81654E+00_JPRB, 0.80378E+00_JPRB ,&
&  0.78164E+00_JPRB, 0.71333E+00_JPRB, 0.73501E+00_JPRB, 0.71122E+00_JPRB, 0.68391E+00_JPRB, 0.65335E+00_JPRB, 0.62820E+00_JPRB ,&
&  0.63219E+00_JPRB, 0.63598E+00_JPRB, 0.65684E+00_JPRB, 0.69591E+00_JPRB, 0.71652E+00_JPRB, 0.75154E+00_JPRB, 0.77297E+00_JPRB ,&
&  0.76249E+00_JPRB, 0.90768E+00_JPRB, 0.98363E+00_JPRB, 0.10338E+01_JPRB, 0.10794E+01_JPRB, 0.11167E+01_JPRB, 0.11456E+01_JPRB ,&
&  0.11942E+01_JPRB, 0.12439E+01_JPRB, 0.13050E+01_JPRB, 0.13695E+01_JPRB, 0.14694E+01_JPRB, 0.15450E+01_JPRB, 0.16146E+01_JPRB ,&
&  0.16640E+01_JPRB, 0.16963E+01_JPRB, 0.17437E+01_JPRB, 0.18225E+01_JPRB, 0.19185E+01_JPRB, 0.20235E+01_JPRB, 0.21361E+01_JPRB ,&
&  0.22599E+01_JPRB, 0.23787E+01_JPRB, 0.25025E+01_JPRB, 0.26122E+01_JPRB, 0.27043E+01_JPRB, 0.27645E+01_JPRB, 0.28201E+01_JPRB ,&
&  0.27898E+01_JPRB, 0.27420E+01_JPRB, 0.26574E+01_JPRB, 0.25297E+01_JPRB, 0.23621E+01_JPRB, 0.21773E+01_JPRB, 0.19910E+01_JPRB ,&
&  0.18835E+01_JPRB, 0.17653E+01_JPRB, 0.16587E+01_JPRB, 0.15583E+01_JPRB, 0.14770E+01_JPRB, 0.14130E+01_JPRB, 0.13748E+01_JPRB ,&
&  0.13397E+01_JPRB, 0.13162E+01_JPRB, 0.13004E+01_JPRB, 0.12883E+01_JPRB, 0.12837E+01_JPRB, 0.12706E+01_JPRB, 0.12687E+01_JPRB ,&
&  0.12682E+01_JPRB, 0.12724E+01_JPRB, 0.12664E+01_JPRB, 0.12665E+01_JPRB, 0.12723E+01_JPRB /)
ZBURD(:,  7,  7) = (/ &
&  0.12755E+01_JPRB, 0.12827E+01_JPRB, 0.12677E+01_JPRB, 0.12533E+01_JPRB, 0.12511E+01_JPRB, 0.12369E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.12452E+01_JPRB, 0.12476E+01_JPRB, 0.12295E+01_JPRB, 0.12041E+01_JPRB, 0.11732E+01_JPRB, 0.11546E+01_JPRB, 0.11312E+01_JPRB ,&
&  0.11104E+01_JPRB, 0.10800E+01_JPRB, 0.10454E+01_JPRB, 0.10088E+01_JPRB, 0.97845E+00_JPRB, 0.94494E+00_JPRB, 0.92016E+00_JPRB ,&
&  0.89385E+00_JPRB, 0.86187E+00_JPRB, 0.83445E+00_JPRB, 0.82782E+00_JPRB, 0.81206E+00_JPRB, 0.79208E+00_JPRB, 0.79018E+00_JPRB ,&
&  0.78459E+00_JPRB, 0.77315E+00_JPRB, 0.76539E+00_JPRB, 0.74207E+00_JPRB, 0.68728E+00_JPRB, 0.59656E+00_JPRB, 0.62276E+00_JPRB ,&
&  0.61544E+00_JPRB, 0.59869E+00_JPRB, 0.60484E+00_JPRB, 0.62294E+00_JPRB, 0.64100E+00_JPRB, 0.69226E+00_JPRB, 0.69971E+00_JPRB ,&
&  0.68029E+00_JPRB, 0.89529E+00_JPRB, 0.93765E+00_JPRB, 0.98815E+00_JPRB, 0.10309E+01_JPRB, 0.10568E+01_JPRB, 0.10804E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.11687E+01_JPRB, 0.12235E+01_JPRB, 0.13029E+01_JPRB, 0.14053E+01_JPRB, 0.15366E+01_JPRB, 0.16517E+01_JPRB ,&
&  0.17246E+01_JPRB, 0.17761E+01_JPRB, 0.18433E+01_JPRB, 0.19054E+01_JPRB, 0.19749E+01_JPRB, 0.20851E+01_JPRB, 0.22207E+01_JPRB ,&
&  0.24070E+01_JPRB, 0.25362E+01_JPRB, 0.27844E+01_JPRB, 0.29861E+01_JPRB, 0.30986E+01_JPRB, 0.32448E+01_JPRB, 0.33721E+01_JPRB ,&
&  0.33299E+01_JPRB, 0.33154E+01_JPRB, 0.32450E+01_JPRB, 0.29736E+01_JPRB, 0.26749E+01_JPRB, 0.24635E+01_JPRB, 0.22698E+01_JPRB ,&
&  0.21127E+01_JPRB, 0.19573E+01_JPRB, 0.18047E+01_JPRB, 0.16798E+01_JPRB, 0.15813E+01_JPRB, 0.15056E+01_JPRB, 0.14616E+01_JPRB ,&
&  0.14155E+01_JPRB, 0.13792E+01_JPRB, 0.13446E+01_JPRB, 0.13264E+01_JPRB, 0.12975E+01_JPRB, 0.12739E+01_JPRB, 0.12526E+01_JPRB ,&
&  0.12442E+01_JPRB, 0.12399E+01_JPRB, 0.12440E+01_JPRB, 0.12501E+01_JPRB, 0.12536E+01_JPRB /)
ZBURD(:,  8,  7) = (/ &
&  0.12686E+01_JPRB, 0.12968E+01_JPRB, 0.12930E+01_JPRB, 0.12928E+01_JPRB, 0.12914E+01_JPRB, 0.12700E+01_JPRB, 0.12538E+01_JPRB ,&
&  0.12491E+01_JPRB, 0.12512E+01_JPRB, 0.12435E+01_JPRB, 0.12274E+01_JPRB, 0.12045E+01_JPRB, 0.11878E+01_JPRB, 0.11720E+01_JPRB ,&
&  0.11551E+01_JPRB, 0.10679E+01_JPRB, 0.10192E+01_JPRB, 0.97535E+00_JPRB, 0.93329E+00_JPRB, 0.90914E+00_JPRB, 0.89592E+00_JPRB ,&
&  0.87698E+00_JPRB, 0.86486E+00_JPRB, 0.85364E+00_JPRB, 0.83601E+00_JPRB, 0.81557E+00_JPRB, 0.78264E+00_JPRB, 0.77022E+00_JPRB ,&
&  0.78202E+00_JPRB, 0.76116E+00_JPRB, 0.73657E+00_JPRB, 0.70555E+00_JPRB, 0.65428E+00_JPRB, 0.63581E+00_JPRB, 0.63750E+00_JPRB ,&
&  0.62576E+00_JPRB, 0.59044E+00_JPRB, 0.59328E+00_JPRB, 0.59692E+00_JPRB, 0.59987E+00_JPRB, 0.62968E+00_JPRB, 0.67949E+00_JPRB ,&
&  0.73028E+00_JPRB, 0.88452E+00_JPRB, 0.94210E+00_JPRB, 0.10089E+01_JPRB, 0.10506E+01_JPRB, 0.10809E+01_JPRB, 0.11080E+01_JPRB ,&
&  0.11537E+01_JPRB, 0.12190E+01_JPRB, 0.12989E+01_JPRB, 0.14082E+01_JPRB, 0.15390E+01_JPRB, 0.16494E+01_JPRB, 0.17418E+01_JPRB ,&
&  0.18662E+01_JPRB, 0.18600E+01_JPRB, 0.19166E+01_JPRB, 0.19607E+01_JPRB, 0.20369E+01_JPRB, 0.21873E+01_JPRB, 0.22775E+01_JPRB ,&
&  0.24616E+01_JPRB, 0.27209E+01_JPRB, 0.31269E+01_JPRB, 0.35074E+01_JPRB, 0.36933E+01_JPRB, 0.38789E+01_JPRB, 0.39268E+01_JPRB ,&
&  0.37125E+01_JPRB, 0.35890E+01_JPRB, 0.36678E+01_JPRB, 0.33549E+01_JPRB, 0.30075E+01_JPRB, 0.27066E+01_JPRB, 0.24822E+01_JPRB ,&
&  0.22661E+01_JPRB, 0.20581E+01_JPRB, 0.19204E+01_JPRB, 0.18055E+01_JPRB, 0.16890E+01_JPRB, 0.15839E+01_JPRB, 0.15568E+01_JPRB ,&
&  0.15330E+01_JPRB, 0.15051E+01_JPRB, 0.14662E+01_JPRB, 0.14216E+01_JPRB, 0.13540E+01_JPRB, 0.12888E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.12152E+01_JPRB, 0.11989E+01_JPRB, 0.12031E+01_JPRB, 0.12089E+01_JPRB, 0.12308E+01_JPRB /)
ZBURD(:,  9,  7) = (/ &
&  0.12485E+01_JPRB, 0.12763E+01_JPRB, 0.12683E+01_JPRB, 0.12654E+01_JPRB, 0.12639E+01_JPRB, 0.11983E+01_JPRB, 0.11544E+01_JPRB ,&
&  0.11353E+01_JPRB, 0.11637E+01_JPRB, 0.11815E+01_JPRB, 0.11861E+01_JPRB, 0.11737E+01_JPRB, 0.11557E+01_JPRB, 0.11419E+01_JPRB ,&
&  0.11286E+01_JPRB, 0.10903E+01_JPRB, 0.10138E+01_JPRB, 0.91010E+00_JPRB, 0.87439E+00_JPRB, 0.85958E+00_JPRB, 0.86157E+00_JPRB ,&
&  0.86192E+00_JPRB, 0.85343E+00_JPRB, 0.85408E+00_JPRB, 0.83462E+00_JPRB, 0.81893E+00_JPRB, 0.77558E+00_JPRB, 0.73742E+00_JPRB ,&
&  0.73217E+00_JPRB, 0.72333E+00_JPRB, 0.73669E+00_JPRB, 0.71492E+00_JPRB, 0.68595E+00_JPRB, 0.66749E+00_JPRB, 0.60563E+00_JPRB ,&
&  0.59639E+00_JPRB, 0.57543E+00_JPRB, 0.56810E+00_JPRB, 0.56871E+00_JPRB, 0.59372E+00_JPRB, 0.61725E+00_JPRB, 0.66259E+00_JPRB ,&
&  0.71362E+00_JPRB, 0.86827E+00_JPRB, 0.91817E+00_JPRB, 0.95235E+00_JPRB, 0.97772E+00_JPRB, 0.10227E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.11798E+01_JPRB, 0.12680E+01_JPRB, 0.13524E+01_JPRB, 0.14978E+01_JPRB, 0.16500E+01_JPRB, 0.16892E+01_JPRB, 0.18870E+01_JPRB ,&
&  0.25029E+01_JPRB, 0.23727E+01_JPRB, 0.22584E+01_JPRB, 0.22143E+01_JPRB, 0.22696E+01_JPRB, 0.23756E+01_JPRB, 0.25461E+01_JPRB ,&
&  0.28647E+01_JPRB, 0.33378E+01_JPRB, 0.40034E+01_JPRB, 0.44413E+01_JPRB, 0.44524E+01_JPRB, 0.44934E+01_JPRB, 0.44007E+01_JPRB ,&
&  0.40592E+01_JPRB, 0.38182E+01_JPRB, 0.39176E+01_JPRB, 0.31769E+01_JPRB, 0.26716E+01_JPRB, 0.23958E+01_JPRB, 0.22577E+01_JPRB ,&
&  0.21189E+01_JPRB, 0.19762E+01_JPRB, 0.18889E+01_JPRB, 0.17782E+01_JPRB, 0.16358E+01_JPRB, 0.15514E+01_JPRB, 0.15320E+01_JPRB ,&
&  0.15148E+01_JPRB, 0.14843E+01_JPRB, 0.14552E+01_JPRB, 0.14178E+01_JPRB, 0.13615E+01_JPRB, 0.12975E+01_JPRB, 0.12562E+01_JPRB ,&
&  0.12332E+01_JPRB, 0.12116E+01_JPRB, 0.11977E+01_JPRB, 0.11953E+01_JPRB, 0.12108E+01_JPRB /)
ZBURD(:, 10,  7) = (/ &
&  0.12129E+01_JPRB, 0.12351E+01_JPRB, 0.12273E+01_JPRB, 0.11983E+01_JPRB, 0.11631E+01_JPRB, 0.10629E+01_JPRB, 0.96357E+00_JPRB ,&
&  0.89684E+00_JPRB, 0.91480E+00_JPRB, 0.97012E+00_JPRB, 0.10187E+01_JPRB, 0.10290E+01_JPRB, 0.10026E+01_JPRB, 0.98822E+00_JPRB ,&
&  0.10009E+01_JPRB, 0.10007E+01_JPRB, 0.95232E+00_JPRB, 0.91021E+00_JPRB, 0.84510E+00_JPRB, 0.82602E+00_JPRB, 0.82497E+00_JPRB ,&
&  0.82951E+00_JPRB, 0.82989E+00_JPRB, 0.83374E+00_JPRB, 0.83284E+00_JPRB, 0.83327E+00_JPRB, 0.81392E+00_JPRB, 0.78274E+00_JPRB ,&
&  0.75165E+00_JPRB, 0.74286E+00_JPRB, 0.74084E+00_JPRB, 0.74037E+00_JPRB, 0.72382E+00_JPRB, 0.70079E+00_JPRB, 0.61681E+00_JPRB ,&
&  0.58537E+00_JPRB, 0.56363E+00_JPRB, 0.55986E+00_JPRB, 0.57327E+00_JPRB, 0.62571E+00_JPRB, 0.65965E+00_JPRB, 0.76314E+00_JPRB ,&
&  0.83424E+00_JPRB, 0.89005E+00_JPRB, 0.90142E+00_JPRB, 0.90735E+00_JPRB, 0.93130E+00_JPRB, 0.10088E+01_JPRB, 0.11193E+01_JPRB ,&
&  0.12094E+01_JPRB, 0.12857E+01_JPRB, 0.13023E+01_JPRB, 0.11843E+01_JPRB, 0.12461E+01_JPRB, 0.14866E+01_JPRB, 0.18071E+01_JPRB ,&
&  0.21319E+01_JPRB, 0.22079E+01_JPRB, 0.22199E+01_JPRB, 0.23706E+01_JPRB, 0.24465E+01_JPRB, 0.25214E+01_JPRB, 0.25988E+01_JPRB ,&
&  0.30223E+01_JPRB, 0.36025E+01_JPRB, 0.44553E+01_JPRB, 0.48878E+01_JPRB, 0.47588E+01_JPRB, 0.47843E+01_JPRB, 0.45518E+01_JPRB ,&
&  0.43643E+01_JPRB, 0.38290E+01_JPRB, 0.33513E+01_JPRB, 0.28103E+01_JPRB, 0.23766E+01_JPRB, 0.21283E+01_JPRB, 0.19792E+01_JPRB ,&
&  0.18488E+01_JPRB, 0.17441E+01_JPRB, 0.17245E+01_JPRB, 0.16571E+01_JPRB, 0.15182E+01_JPRB, 0.13993E+01_JPRB, 0.13130E+01_JPRB ,&
&  0.12505E+01_JPRB, 0.12102E+01_JPRB, 0.11946E+01_JPRB, 0.11501E+01_JPRB, 0.11383E+01_JPRB, 0.11423E+01_JPRB, 0.11247E+01_JPRB ,&
&  0.11081E+01_JPRB, 0.11001E+01_JPRB, 0.11153E+01_JPRB, 0.11424E+01_JPRB, 0.11692E+01_JPRB /)
ZBURD(:, 11,  7) = (/ &
&  0.12324E+01_JPRB, 0.12066E+01_JPRB, 0.11615E+01_JPRB, 0.11425E+01_JPRB, 0.10971E+01_JPRB, 0.10478E+01_JPRB, 0.92567E+00_JPRB ,&
&  0.78686E+00_JPRB, 0.76412E+00_JPRB, 0.82216E+00_JPRB, 0.85882E+00_JPRB, 0.84811E+00_JPRB, 0.84953E+00_JPRB, 0.86633E+00_JPRB ,&
&  0.88056E+00_JPRB, 0.92642E+00_JPRB, 0.92415E+00_JPRB, 0.89168E+00_JPRB, 0.86461E+00_JPRB, 0.84511E+00_JPRB, 0.84034E+00_JPRB ,&
&  0.83541E+00_JPRB, 0.83210E+00_JPRB, 0.82072E+00_JPRB, 0.80318E+00_JPRB, 0.80100E+00_JPRB, 0.79719E+00_JPRB, 0.78988E+00_JPRB ,&
&  0.78192E+00_JPRB, 0.76649E+00_JPRB, 0.76750E+00_JPRB, 0.76554E+00_JPRB, 0.74880E+00_JPRB, 0.70366E+00_JPRB, 0.61993E+00_JPRB ,&
&  0.58261E+00_JPRB, 0.56564E+00_JPRB, 0.59914E+00_JPRB, 0.70113E+00_JPRB, 0.76251E+00_JPRB, 0.81382E+00_JPRB, 0.86790E+00_JPRB ,&
&  0.83934E+00_JPRB, 0.82927E+00_JPRB, 0.86691E+00_JPRB, 0.99265E+00_JPRB, 0.10624E+01_JPRB, 0.11560E+01_JPRB, 0.12669E+01_JPRB ,&
&  0.13959E+01_JPRB, 0.14925E+01_JPRB, 0.15290E+01_JPRB, 0.13700E+01_JPRB, 0.15839E+01_JPRB, 0.21554E+01_JPRB, 0.21848E+01_JPRB ,&
&  0.23855E+01_JPRB, 0.25911E+01_JPRB, 0.27696E+01_JPRB, 0.27446E+01_JPRB, 0.26005E+01_JPRB, 0.26746E+01_JPRB, 0.29726E+01_JPRB ,&
&  0.33955E+01_JPRB, 0.40115E+01_JPRB, 0.51382E+01_JPRB, 0.57713E+01_JPRB, 0.56704E+01_JPRB, 0.55301E+01_JPRB, 0.53711E+01_JPRB ,&
&  0.52586E+01_JPRB, 0.46714E+01_JPRB, 0.39246E+01_JPRB, 0.32543E+01_JPRB, 0.26816E+01_JPRB, 0.22331E+01_JPRB, 0.19554E+01_JPRB ,&
&  0.18409E+01_JPRB, 0.17800E+01_JPRB, 0.17616E+01_JPRB, 0.16846E+01_JPRB, 0.15232E+01_JPRB, 0.13535E+01_JPRB, 0.12204E+01_JPRB ,&
&  0.11258E+01_JPRB, 0.10364E+01_JPRB, 0.96309E+00_JPRB, 0.93743E+00_JPRB, 0.99393E+00_JPRB, 0.10694E+01_JPRB, 0.10843E+01_JPRB ,&
&  0.10566E+01_JPRB, 0.10428E+01_JPRB, 0.10978E+01_JPRB, 0.11653E+01_JPRB, 0.11931E+01_JPRB /)
ZBURD(:, 12,  7) = (/ &
&  0.12690E+01_JPRB, 0.12215E+01_JPRB, 0.11714E+01_JPRB, 0.11180E+01_JPRB, 0.10620E+01_JPRB, 0.98076E+00_JPRB, 0.86047E+00_JPRB ,&
&  0.74900E+00_JPRB, 0.72846E+00_JPRB, 0.74101E+00_JPRB, 0.72431E+00_JPRB, 0.69696E+00_JPRB, 0.68609E+00_JPRB, 0.70091E+00_JPRB ,&
&  0.72977E+00_JPRB, 0.79116E+00_JPRB, 0.83877E+00_JPRB, 0.85799E+00_JPRB, 0.85325E+00_JPRB, 0.84143E+00_JPRB, 0.84698E+00_JPRB ,&
&  0.85779E+00_JPRB, 0.86876E+00_JPRB, 0.87213E+00_JPRB, 0.85999E+00_JPRB, 0.84571E+00_JPRB, 0.83677E+00_JPRB, 0.82714E+00_JPRB ,&
&  0.81771E+00_JPRB, 0.80613E+00_JPRB, 0.78068E+00_JPRB, 0.77537E+00_JPRB, 0.75005E+00_JPRB, 0.73490E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.61346E+00_JPRB, 0.62222E+00_JPRB, 0.76837E+00_JPRB, 0.82550E+00_JPRB, 0.84380E+00_JPRB, 0.88484E+00_JPRB, 0.94430E+00_JPRB ,&
&  0.99140E+00_JPRB, 0.10308E+01_JPRB, 0.10723E+01_JPRB, 0.11279E+01_JPRB, 0.12130E+01_JPRB, 0.13778E+01_JPRB, 0.15700E+01_JPRB ,&
&  0.18180E+01_JPRB, 0.15990E+01_JPRB, 0.15402E+01_JPRB, 0.17767E+01_JPRB, 0.24977E+01_JPRB, 0.29635E+01_JPRB, 0.31332E+01_JPRB ,&
&  0.32592E+01_JPRB, 0.32109E+01_JPRB, 0.31515E+01_JPRB, 0.32004E+01_JPRB, 0.33900E+01_JPRB, 0.37214E+01_JPRB, 0.42544E+01_JPRB ,&
&  0.47058E+01_JPRB, 0.54734E+01_JPRB, 0.65801E+01_JPRB, 0.68590E+01_JPRB, 0.67312E+01_JPRB, 0.66068E+01_JPRB, 0.63337E+01_JPRB ,&
&  0.60166E+01_JPRB, 0.54079E+01_JPRB, 0.44423E+01_JPRB, 0.36338E+01_JPRB, 0.29558E+01_JPRB, 0.24453E+01_JPRB, 0.20430E+01_JPRB ,&
&  0.18449E+01_JPRB, 0.17859E+01_JPRB, 0.17434E+01_JPRB, 0.16658E+01_JPRB, 0.15473E+01_JPRB, 0.14184E+01_JPRB, 0.12832E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.98901E+00_JPRB, 0.91240E+00_JPRB, 0.93341E+00_JPRB, 0.10248E+01_JPRB, 0.11283E+01_JPRB, 0.11473E+01_JPRB ,&
&  0.11699E+01_JPRB, 0.11395E+01_JPRB, 0.11341E+01_JPRB, 0.11919E+01_JPRB, 0.12609E+01_JPRB /)
ZBURD(:, 13,  7) = (/ &
&  0.12221E+01_JPRB, 0.11781E+01_JPRB, 0.11146E+01_JPRB, 0.10392E+01_JPRB, 0.96669E+00_JPRB, 0.90453E+00_JPRB, 0.83008E+00_JPRB ,&
&  0.86005E+00_JPRB, 0.78769E+00_JPRB, 0.85480E+00_JPRB, 0.72817E+00_JPRB, 0.69060E+00_JPRB, 0.67927E+00_JPRB, 0.66216E+00_JPRB ,&
&  0.65943E+00_JPRB, 0.68421E+00_JPRB, 0.71793E+00_JPRB, 0.75284E+00_JPRB, 0.76360E+00_JPRB, 0.76932E+00_JPRB, 0.78676E+00_JPRB ,&
&  0.81407E+00_JPRB, 0.85231E+00_JPRB, 0.89426E+00_JPRB, 0.90927E+00_JPRB, 0.90987E+00_JPRB, 0.89776E+00_JPRB, 0.88879E+00_JPRB ,&
&  0.83146E+00_JPRB, 0.81131E+00_JPRB, 0.80653E+00_JPRB, 0.79690E+00_JPRB, 0.78754E+00_JPRB, 0.76390E+00_JPRB, 0.76347E+00_JPRB ,&
&  0.73386E+00_JPRB, 0.70584E+00_JPRB, 0.82780E+00_JPRB, 0.85936E+00_JPRB, 0.90325E+00_JPRB, 0.94988E+00_JPRB, 0.99597E+00_JPRB ,&
&  0.10376E+01_JPRB, 0.10790E+01_JPRB, 0.11271E+01_JPRB, 0.11836E+01_JPRB, 0.13318E+01_JPRB, 0.16042E+01_JPRB, 0.18922E+01_JPRB ,&
&  0.22048E+01_JPRB, 0.18315E+01_JPRB, 0.21269E+01_JPRB, 0.23987E+01_JPRB, 0.29912E+01_JPRB, 0.35714E+01_JPRB, 0.38450E+01_JPRB ,&
&  0.43427E+01_JPRB, 0.43577E+01_JPRB, 0.43101E+01_JPRB, 0.42698E+01_JPRB, 0.46307E+01_JPRB, 0.51617E+01_JPRB, 0.57530E+01_JPRB ,&
&  0.63890E+01_JPRB, 0.70614E+01_JPRB, 0.82512E+01_JPRB, 0.82139E+01_JPRB, 0.79681E+01_JPRB, 0.76801E+01_JPRB, 0.71901E+01_JPRB ,&
&  0.65787E+01_JPRB, 0.58326E+01_JPRB, 0.49215E+01_JPRB, 0.38830E+01_JPRB, 0.31834E+01_JPRB, 0.26058E+01_JPRB, 0.21304E+01_JPRB ,&
&  0.18080E+01_JPRB, 0.16214E+01_JPRB, 0.15810E+01_JPRB, 0.15203E+01_JPRB, 0.14484E+01_JPRB, 0.13494E+01_JPRB, 0.11905E+01_JPRB ,&
&  0.10060E+01_JPRB, 0.10112E+01_JPRB, 0.10504E+01_JPRB, 0.11680E+01_JPRB, 0.12352E+01_JPRB, 0.12548E+01_JPRB, 0.13055E+01_JPRB ,&
&  0.12110E+01_JPRB, 0.12289E+01_JPRB, 0.12927E+01_JPRB, 0.12637E+01_JPRB, 0.12494E+01_JPRB /)
ZBURD(:, 14,  7) = (/ &
&  0.12409E+01_JPRB, 0.11637E+01_JPRB, 0.10419E+01_JPRB, 0.97234E+00_JPRB, 0.95774E+00_JPRB, 0.93657E+00_JPRB, 0.89364E+00_JPRB ,&
&  0.91535E+00_JPRB, 0.89608E+00_JPRB, 0.90803E+00_JPRB, 0.92972E+00_JPRB, 0.94728E+00_JPRB, 0.84621E+00_JPRB, 0.71565E+00_JPRB ,&
&  0.68479E+00_JPRB, 0.71609E+00_JPRB, 0.75911E+00_JPRB, 0.78335E+00_JPRB, 0.81781E+00_JPRB, 0.82430E+00_JPRB, 0.83704E+00_JPRB ,&
&  0.84801E+00_JPRB, 0.86198E+00_JPRB, 0.90504E+00_JPRB, 0.92059E+00_JPRB, 0.91304E+00_JPRB, 0.91072E+00_JPRB, 0.91438E+00_JPRB ,&
&  0.85423E+00_JPRB, 0.82974E+00_JPRB, 0.81132E+00_JPRB, 0.77762E+00_JPRB, 0.82687E+00_JPRB, 0.83657E+00_JPRB, 0.83823E+00_JPRB ,&
&  0.84042E+00_JPRB, 0.85452E+00_JPRB, 0.89015E+00_JPRB, 0.93602E+00_JPRB, 0.98830E+00_JPRB, 0.10375E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.11079E+01_JPRB, 0.11408E+01_JPRB, 0.11787E+01_JPRB, 0.12680E+01_JPRB, 0.15422E+01_JPRB, 0.18286E+01_JPRB, 0.23163E+01_JPRB ,&
&  0.25595E+01_JPRB, 0.28613E+01_JPRB, 0.33006E+01_JPRB, 0.33376E+01_JPRB, 0.36644E+01_JPRB, 0.40888E+01_JPRB, 0.40762E+01_JPRB ,&
&  0.42824E+01_JPRB, 0.43389E+01_JPRB, 0.47158E+01_JPRB, 0.52683E+01_JPRB, 0.60032E+01_JPRB, 0.66394E+01_JPRB, 0.71540E+01_JPRB ,&
&  0.79000E+01_JPRB, 0.90968E+01_JPRB, 0.95957E+01_JPRB, 0.91945E+01_JPRB, 0.87123E+01_JPRB, 0.82539E+01_JPRB, 0.75480E+01_JPRB ,&
&  0.68356E+01_JPRB, 0.60227E+01_JPRB, 0.50630E+01_JPRB, 0.40872E+01_JPRB, 0.32015E+01_JPRB, 0.27095E+01_JPRB, 0.22182E+01_JPRB ,&
&  0.17497E+01_JPRB, 0.14330E+01_JPRB, 0.14177E+01_JPRB, 0.14114E+01_JPRB, 0.13649E+01_JPRB, 0.12284E+01_JPRB, 0.10944E+01_JPRB ,&
&  0.10940E+01_JPRB, 0.12117E+01_JPRB, 0.14733E+01_JPRB, 0.15516E+01_JPRB, 0.15798E+01_JPRB, 0.15128E+01_JPRB, 0.12597E+01_JPRB ,&
&  0.11560E+01_JPRB, 0.12998E+01_JPRB, 0.12913E+01_JPRB, 0.12996E+01_JPRB, 0.12894E+01_JPRB /)
ZBURD(:, 15,  7) = (/ &
&  0.12763E+01_JPRB, 0.11660E+01_JPRB, 0.10126E+01_JPRB, 0.96016E+00_JPRB, 0.97133E+00_JPRB, 0.95960E+00_JPRB, 0.94362E+00_JPRB ,&
&  0.95078E+00_JPRB, 0.95143E+00_JPRB, 0.95004E+00_JPRB, 0.95494E+00_JPRB, 0.96164E+00_JPRB, 0.98812E+00_JPRB, 0.89652E+00_JPRB ,&
&  0.81282E+00_JPRB, 0.83946E+00_JPRB, 0.82620E+00_JPRB, 0.88426E+00_JPRB, 0.10647E+01_JPRB, 0.10804E+01_JPRB, 0.10860E+01_JPRB ,&
&  0.11176E+01_JPRB, 0.10708E+01_JPRB, 0.10607E+01_JPRB, 0.10245E+01_JPRB, 0.97062E+00_JPRB, 0.96870E+00_JPRB, 0.96540E+00_JPRB ,&
&  0.87061E+00_JPRB, 0.81025E+00_JPRB, 0.77906E+00_JPRB, 0.76655E+00_JPRB, 0.81974E+00_JPRB, 0.88846E+00_JPRB, 0.91385E+00_JPRB ,&
&  0.92164E+00_JPRB, 0.94668E+00_JPRB, 0.99107E+00_JPRB, 0.10483E+01_JPRB, 0.11079E+01_JPRB, 0.11524E+01_JPRB, 0.11719E+01_JPRB ,&
&  0.11883E+01_JPRB, 0.12115E+01_JPRB, 0.12498E+01_JPRB, 0.13755E+01_JPRB, 0.15940E+01_JPRB, 0.21051E+01_JPRB, 0.28648E+01_JPRB ,&
&  0.32619E+01_JPRB, 0.35543E+01_JPRB, 0.41248E+01_JPRB, 0.49461E+01_JPRB, 0.54358E+01_JPRB, 0.52545E+01_JPRB, 0.50429E+01_JPRB ,&
&  0.51519E+01_JPRB, 0.52779E+01_JPRB, 0.62783E+01_JPRB, 0.70246E+01_JPRB, 0.75774E+01_JPRB, 0.82247E+01_JPRB, 0.85207E+01_JPRB ,&
&  0.90778E+01_JPRB, 0.95873E+01_JPRB, 0.96338E+01_JPRB, 0.91800E+01_JPRB, 0.87274E+01_JPRB, 0.83302E+01_JPRB, 0.75788E+01_JPRB ,&
&  0.68319E+01_JPRB, 0.58696E+01_JPRB, 0.46696E+01_JPRB, 0.37160E+01_JPRB, 0.28039E+01_JPRB, 0.25939E+01_JPRB, 0.22680E+01_JPRB ,&
&  0.16973E+01_JPRB, 0.14062E+01_JPRB, 0.13943E+01_JPRB, 0.14883E+01_JPRB, 0.14648E+01_JPRB, 0.13131E+01_JPRB, 0.11873E+01_JPRB ,&
&  0.12065E+01_JPRB, 0.14150E+01_JPRB, 0.15779E+01_JPRB, 0.17133E+01_JPRB, 0.17404E+01_JPRB, 0.16266E+01_JPRB, 0.12494E+01_JPRB ,&
&  0.12009E+01_JPRB, 0.13399E+01_JPRB, 0.13464E+01_JPRB, 0.13430E+01_JPRB, 0.13309E+01_JPRB /)
ZBURD(:, 16,  7) = (/ &
&  0.13392E+01_JPRB, 0.12001E+01_JPRB, 0.10629E+01_JPRB, 0.98527E+00_JPRB, 0.98562E+00_JPRB, 0.98464E+00_JPRB, 0.97770E+00_JPRB ,&
&  0.99096E+00_JPRB, 0.10038E+01_JPRB, 0.10080E+01_JPRB, 0.10074E+01_JPRB, 0.10060E+01_JPRB, 0.10056E+01_JPRB, 0.10413E+01_JPRB ,&
&  0.90627E+00_JPRB, 0.95500E+00_JPRB, 0.92727E+00_JPRB, 0.10486E+01_JPRB, 0.15577E+01_JPRB, 0.15312E+01_JPRB, 0.15914E+01_JPRB ,&
&  0.15285E+01_JPRB, 0.14966E+01_JPRB, 0.14186E+01_JPRB, 0.13256E+01_JPRB, 0.12537E+01_JPRB, 0.12407E+01_JPRB, 0.11681E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.90482E+00_JPRB, 0.88648E+00_JPRB, 0.89117E+00_JPRB, 0.94342E+00_JPRB, 0.97337E+00_JPRB, 0.10530E+01_JPRB ,&
&  0.10829E+01_JPRB, 0.11386E+01_JPRB, 0.12043E+01_JPRB, 0.12618E+01_JPRB, 0.12901E+01_JPRB, 0.13060E+01_JPRB, 0.13258E+01_JPRB ,&
&  0.13576E+01_JPRB, 0.14026E+01_JPRB, 0.14763E+01_JPRB, 0.16525E+01_JPRB, 0.17763E+01_JPRB, 0.24273E+01_JPRB, 0.38910E+01_JPRB ,&
&  0.48959E+01_JPRB, 0.50628E+01_JPRB, 0.64682E+01_JPRB, 0.90830E+01_JPRB, 0.92625E+01_JPRB, 0.85330E+01_JPRB, 0.78124E+01_JPRB ,&
&  0.74912E+01_JPRB, 0.73391E+01_JPRB, 0.77519E+01_JPRB, 0.83342E+01_JPRB, 0.87635E+01_JPRB, 0.94099E+01_JPRB, 0.95659E+01_JPRB ,&
&  0.93857E+01_JPRB, 0.91936E+01_JPRB, 0.88991E+01_JPRB, 0.85357E+01_JPRB, 0.79300E+01_JPRB, 0.75419E+01_JPRB, 0.70151E+01_JPRB ,&
&  0.63738E+01_JPRB, 0.55025E+01_JPRB, 0.41211E+01_JPRB, 0.33265E+01_JPRB, 0.23361E+01_JPRB, 0.21001E+01_JPRB, 0.26432E+01_JPRB ,&
&  0.18700E+01_JPRB, 0.15426E+01_JPRB, 0.15599E+01_JPRB, 0.17101E+01_JPRB, 0.17345E+01_JPRB, 0.17060E+01_JPRB, 0.14611E+01_JPRB ,&
&  0.12195E+01_JPRB, 0.13588E+01_JPRB, 0.17621E+01_JPRB, 0.18779E+01_JPRB, 0.18657E+01_JPRB, 0.17390E+01_JPRB, 0.14695E+01_JPRB ,&
&  0.15040E+01_JPRB, 0.14369E+01_JPRB, 0.14258E+01_JPRB, 0.14201E+01_JPRB, 0.14051E+01_JPRB /)
ZBURD(:, 17,  7) = (/ &
&  0.13697E+01_JPRB, 0.12337E+01_JPRB, 0.11131E+01_JPRB, 0.10286E+01_JPRB, 0.10106E+01_JPRB, 0.10217E+01_JPRB, 0.10272E+01_JPRB ,&
&  0.10360E+01_JPRB, 0.10402E+01_JPRB, 0.10387E+01_JPRB, 0.10386E+01_JPRB, 0.10357E+01_JPRB, 0.10103E+01_JPRB, 0.99310E+00_JPRB ,&
&  0.99985E+00_JPRB, 0.11043E+01_JPRB, 0.10993E+01_JPRB, 0.10975E+01_JPRB, 0.14692E+01_JPRB, 0.16425E+01_JPRB, 0.18711E+01_JPRB ,&
&  0.20071E+01_JPRB, 0.22020E+01_JPRB, 0.21589E+01_JPRB, 0.20442E+01_JPRB, 0.19367E+01_JPRB, 0.18616E+01_JPRB, 0.16534E+01_JPRB ,&
&  0.14040E+01_JPRB, 0.13231E+01_JPRB, 0.12762E+01_JPRB, 0.13466E+01_JPRB, 0.14418E+01_JPRB, 0.12621E+01_JPRB, 0.13728E+01_JPRB ,&
&  0.14242E+01_JPRB, 0.14979E+01_JPRB, 0.15418E+01_JPRB, 0.15622E+01_JPRB, 0.16013E+01_JPRB, 0.16545E+01_JPRB, 0.17140E+01_JPRB ,&
&  0.17794E+01_JPRB, 0.17965E+01_JPRB, 0.17974E+01_JPRB, 0.19075E+01_JPRB, 0.20598E+01_JPRB, 0.24954E+01_JPRB, 0.37027E+01_JPRB ,&
&  0.52863E+01_JPRB, 0.60174E+01_JPRB, 0.80127E+01_JPRB, 0.11272E+02_JPRB, 0.11448E+02_JPRB, 0.10691E+02_JPRB, 0.10458E+02_JPRB ,&
&  0.10269E+02_JPRB, 0.99386E+01_JPRB, 0.97439E+01_JPRB, 0.94923E+01_JPRB, 0.97532E+01_JPRB, 0.10154E+02_JPRB, 0.99149E+01_JPRB ,&
&  0.91241E+01_JPRB, 0.83138E+01_JPRB, 0.78980E+01_JPRB, 0.73337E+01_JPRB, 0.68337E+01_JPRB, 0.61942E+01_JPRB, 0.62320E+01_JPRB ,&
&  0.63300E+01_JPRB, 0.41873E+01_JPRB, 0.32561E+01_JPRB, 0.30913E+01_JPRB, 0.21712E+01_JPRB, 0.18389E+01_JPRB, 0.20635E+01_JPRB ,&
&  0.16336E+01_JPRB, 0.14387E+01_JPRB, 0.16530E+01_JPRB, 0.18488E+01_JPRB, 0.20958E+01_JPRB, 0.22704E+01_JPRB, 0.21710E+01_JPRB ,&
&  0.18790E+01_JPRB, 0.18225E+01_JPRB, 0.22267E+01_JPRB, 0.22180E+01_JPRB, 0.20504E+01_JPRB, 0.18637E+01_JPRB, 0.16995E+01_JPRB ,&
&  0.16135E+01_JPRB, 0.16035E+01_JPRB, 0.15664E+01_JPRB, 0.15373E+01_JPRB, 0.14986E+01_JPRB /)
ZBURD(:, 18,  7) = (/ &
&  0.14692E+01_JPRB, 0.13314E+01_JPRB, 0.11898E+01_JPRB, 0.11150E+01_JPRB, 0.11029E+01_JPRB, 0.11082E+01_JPRB, 0.11161E+01_JPRB ,&
&  0.10859E+01_JPRB, 0.10540E+01_JPRB, 0.10517E+01_JPRB, 0.10458E+01_JPRB, 0.10367E+01_JPRB, 0.10067E+01_JPRB, 0.96253E+00_JPRB ,&
&  0.95066E+00_JPRB, 0.11440E+01_JPRB, 0.10787E+01_JPRB, 0.11329E+01_JPRB, 0.12473E+01_JPRB, 0.16292E+01_JPRB, 0.19765E+01_JPRB ,&
&  0.23404E+01_JPRB, 0.25803E+01_JPRB, 0.27971E+01_JPRB, 0.30727E+01_JPRB, 0.31759E+01_JPRB, 0.30276E+01_JPRB, 0.28397E+01_JPRB ,&
&  0.26777E+01_JPRB, 0.26647E+01_JPRB, 0.23024E+01_JPRB, 0.24258E+01_JPRB, 0.23404E+01_JPRB, 0.20256E+01_JPRB, 0.22166E+01_JPRB ,&
&  0.21692E+01_JPRB, 0.21887E+01_JPRB, 0.21708E+01_JPRB, 0.21635E+01_JPRB, 0.21723E+01_JPRB, 0.21566E+01_JPRB, 0.21242E+01_JPRB ,&
&  0.21010E+01_JPRB, 0.20960E+01_JPRB, 0.21324E+01_JPRB, 0.21366E+01_JPRB, 0.22810E+01_JPRB, 0.28970E+01_JPRB, 0.40748E+01_JPRB ,&
&  0.56270E+01_JPRB, 0.59553E+01_JPRB, 0.65996E+01_JPRB, 0.10211E+02_JPRB, 0.12984E+02_JPRB, 0.12622E+02_JPRB, 0.12177E+02_JPRB ,&
&  0.12174E+02_JPRB, 0.11361E+02_JPRB, 0.10501E+02_JPRB, 0.10052E+02_JPRB, 0.10581E+02_JPRB, 0.10385E+02_JPRB, 0.97301E+01_JPRB ,&
&  0.88050E+01_JPRB, 0.79377E+01_JPRB, 0.75224E+01_JPRB, 0.70911E+01_JPRB, 0.62609E+01_JPRB, 0.59172E+01_JPRB, 0.60137E+01_JPRB ,&
&  0.59016E+01_JPRB, 0.49511E+01_JPRB, 0.35632E+01_JPRB, 0.31442E+01_JPRB, 0.24208E+01_JPRB, 0.20937E+01_JPRB, 0.21829E+01_JPRB ,&
&  0.18529E+01_JPRB, 0.17847E+01_JPRB, 0.18633E+01_JPRB, 0.19837E+01_JPRB, 0.30051E+01_JPRB, 0.32920E+01_JPRB, 0.35714E+01_JPRB ,&
&  0.32131E+01_JPRB, 0.27297E+01_JPRB, 0.31315E+01_JPRB, 0.29300E+01_JPRB, 0.25317E+01_JPRB, 0.21499E+01_JPRB, 0.18714E+01_JPRB ,&
&  0.17726E+01_JPRB, 0.17793E+01_JPRB, 0.17181E+01_JPRB, 0.16610E+01_JPRB, 0.16098E+01_JPRB /)
ZBURD(:, 19,  7) = (/ &
&  0.15468E+01_JPRB, 0.13981E+01_JPRB, 0.12557E+01_JPRB, 0.11764E+01_JPRB, 0.11445E+01_JPRB, 0.11367E+01_JPRB, 0.11203E+01_JPRB ,&
&  0.10760E+01_JPRB, 0.10483E+01_JPRB, 0.10360E+01_JPRB, 0.10133E+01_JPRB, 0.98910E+00_JPRB, 0.96423E+00_JPRB, 0.93037E+00_JPRB ,&
&  0.92303E+00_JPRB, 0.10849E+01_JPRB, 0.10495E+01_JPRB, 0.10780E+01_JPRB, 0.12600E+01_JPRB, 0.17985E+01_JPRB, 0.23562E+01_JPRB ,&
&  0.27561E+01_JPRB, 0.30761E+01_JPRB, 0.35273E+01_JPRB, 0.41316E+01_JPRB, 0.46960E+01_JPRB, 0.49671E+01_JPRB, 0.49804E+01_JPRB ,&
&  0.54735E+01_JPRB, 0.45813E+01_JPRB, 0.46402E+01_JPRB, 0.42018E+01_JPRB, 0.39267E+01_JPRB, 0.36452E+01_JPRB, 0.34129E+01_JPRB ,&
&  0.32109E+01_JPRB, 0.30070E+01_JPRB, 0.27978E+01_JPRB, 0.26533E+01_JPRB, 0.25588E+01_JPRB, 0.24996E+01_JPRB, 0.24938E+01_JPRB ,&
&  0.25184E+01_JPRB, 0.24672E+01_JPRB, 0.24041E+01_JPRB, 0.23856E+01_JPRB, 0.29792E+01_JPRB, 0.43495E+01_JPRB, 0.55320E+01_JPRB ,&
&  0.55758E+01_JPRB, 0.56708E+01_JPRB, 0.10813E+02_JPRB, 0.12412E+02_JPRB, 0.12197E+02_JPRB, 0.13544E+02_JPRB, 0.13781E+02_JPRB ,&
&  0.12484E+02_JPRB, 0.10693E+02_JPRB, 0.91916E+01_JPRB, 0.89763E+01_JPRB, 0.10780E+02_JPRB, 0.10266E+02_JPRB, 0.94218E+01_JPRB ,&
&  0.84544E+01_JPRB, 0.78588E+01_JPRB, 0.71206E+01_JPRB, 0.65446E+01_JPRB, 0.62073E+01_JPRB, 0.64451E+01_JPRB, 0.59355E+01_JPRB ,&
&  0.54851E+01_JPRB, 0.58366E+01_JPRB, 0.46913E+01_JPRB, 0.35602E+01_JPRB, 0.29318E+01_JPRB, 0.26884E+01_JPRB, 0.25952E+01_JPRB ,&
&  0.23715E+01_JPRB, 0.22960E+01_JPRB, 0.23367E+01_JPRB, 0.29571E+01_JPRB, 0.44444E+01_JPRB, 0.45086E+01_JPRB, 0.43664E+01_JPRB ,&
&  0.39513E+01_JPRB, 0.44036E+01_JPRB, 0.39373E+01_JPRB, 0.34121E+01_JPRB, 0.28850E+01_JPRB, 0.24435E+01_JPRB, 0.21209E+01_JPRB ,&
&  0.19248E+01_JPRB, 0.18975E+01_JPRB, 0.18473E+01_JPRB, 0.17634E+01_JPRB, 0.16926E+01_JPRB /)
ZBURD(:, 20,  7) = (/ &
&  0.16530E+01_JPRB, 0.14691E+01_JPRB, 0.13629E+01_JPRB, 0.12751E+01_JPRB, 0.12263E+01_JPRB, 0.11752E+01_JPRB, 0.10870E+01_JPRB ,&
&  0.10127E+01_JPRB, 0.99908E+00_JPRB, 0.97899E+00_JPRB, 0.96546E+00_JPRB, 0.95131E+00_JPRB, 0.92278E+00_JPRB, 0.89847E+00_JPRB ,&
&  0.92786E+00_JPRB, 0.10575E+01_JPRB, 0.10669E+01_JPRB, 0.12362E+01_JPRB, 0.14929E+01_JPRB, 0.19301E+01_JPRB, 0.24818E+01_JPRB ,&
&  0.28605E+01_JPRB, 0.32442E+01_JPRB, 0.37299E+01_JPRB, 0.49765E+01_JPRB, 0.64682E+01_JPRB, 0.77503E+01_JPRB, 0.80089E+01_JPRB ,&
&  0.74991E+01_JPRB, 0.76369E+01_JPRB, 0.72118E+01_JPRB, 0.64771E+01_JPRB, 0.58276E+01_JPRB, 0.53289E+01_JPRB, 0.47649E+01_JPRB ,&
&  0.41677E+01_JPRB, 0.35915E+01_JPRB, 0.31354E+01_JPRB, 0.28578E+01_JPRB, 0.26726E+01_JPRB, 0.26287E+01_JPRB, 0.25987E+01_JPRB ,&
&  0.25585E+01_JPRB, 0.25746E+01_JPRB, 0.25407E+01_JPRB, 0.30174E+01_JPRB, 0.42876E+01_JPRB, 0.60069E+01_JPRB, 0.63199E+01_JPRB ,&
&  0.71380E+01_JPRB, 0.73752E+01_JPRB, 0.96350E+01_JPRB, 0.11492E+02_JPRB, 0.98016E+01_JPRB, 0.11488E+02_JPRB, 0.12314E+02_JPRB ,&
&  0.10863E+02_JPRB, 0.99092E+01_JPRB, 0.10520E+02_JPRB, 0.10480E+02_JPRB, 0.95924E+01_JPRB, 0.11014E+02_JPRB, 0.99459E+01_JPRB ,&
&  0.85521E+01_JPRB, 0.76947E+01_JPRB, 0.67035E+01_JPRB, 0.66597E+01_JPRB, 0.57036E+01_JPRB, 0.55433E+01_JPRB, 0.44016E+01_JPRB ,&
&  0.46318E+01_JPRB, 0.39030E+01_JPRB, 0.41864E+01_JPRB, 0.36110E+01_JPRB, 0.33403E+01_JPRB, 0.30281E+01_JPRB, 0.28053E+01_JPRB ,&
&  0.27807E+01_JPRB, 0.28332E+01_JPRB, 0.32093E+01_JPRB, 0.45398E+01_JPRB, 0.60118E+01_JPRB, 0.49167E+01_JPRB, 0.54322E+01_JPRB ,&
&  0.60960E+01_JPRB, 0.57832E+01_JPRB, 0.43696E+01_JPRB, 0.37824E+01_JPRB, 0.31348E+01_JPRB, 0.26673E+01_JPRB, 0.23943E+01_JPRB ,&
&  0.21495E+01_JPRB, 0.20320E+01_JPRB, 0.19583E+01_JPRB, 0.18839E+01_JPRB, 0.18055E+01_JPRB /)
ZBURD(:, 21,  7) = (/ &
&  0.16820E+01_JPRB, 0.14971E+01_JPRB, 0.14040E+01_JPRB, 0.13668E+01_JPRB, 0.12950E+01_JPRB, 0.11540E+01_JPRB, 0.10525E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.99006E+00_JPRB, 0.93215E+00_JPRB, 0.90050E+00_JPRB, 0.90159E+00_JPRB, 0.91040E+00_JPRB, 0.90351E+00_JPRB ,&
&  0.97117E+00_JPRB, 0.11803E+01_JPRB, 0.12285E+01_JPRB, 0.13323E+01_JPRB, 0.16499E+01_JPRB, 0.20807E+01_JPRB, 0.23523E+01_JPRB ,&
&  0.31191E+01_JPRB, 0.34646E+01_JPRB, 0.38911E+01_JPRB, 0.54860E+01_JPRB, 0.78139E+01_JPRB, 0.99510E+01_JPRB, 0.10618E+02_JPRB ,&
&  0.11489E+02_JPRB, 0.10961E+02_JPRB, 0.87612E+01_JPRB, 0.71493E+01_JPRB, 0.60191E+01_JPRB, 0.53136E+01_JPRB, 0.45828E+01_JPRB ,&
&  0.37530E+01_JPRB, 0.30376E+01_JPRB, 0.26652E+01_JPRB, 0.24327E+01_JPRB, 0.23053E+01_JPRB, 0.21531E+01_JPRB, 0.21844E+01_JPRB ,&
&  0.23295E+01_JPRB, 0.25168E+01_JPRB, 0.26717E+01_JPRB, 0.34472E+01_JPRB, 0.52518E+01_JPRB, 0.66825E+01_JPRB, 0.70563E+01_JPRB ,&
&  0.68533E+01_JPRB, 0.69047E+01_JPRB, 0.89552E+01_JPRB, 0.10010E+02_JPRB, 0.11196E+02_JPRB, 0.10932E+02_JPRB, 0.99497E+01_JPRB ,&
&  0.76296E+01_JPRB, 0.71541E+01_JPRB, 0.76940E+01_JPRB, 0.81989E+01_JPRB, 0.88670E+01_JPRB, 0.10914E+02_JPRB, 0.10363E+02_JPRB ,&
&  0.88150E+01_JPRB, 0.77975E+01_JPRB, 0.69174E+01_JPRB, 0.59163E+01_JPRB, 0.62714E+01_JPRB, 0.44402E+01_JPRB, 0.51877E+01_JPRB ,&
&  0.53329E+01_JPRB, 0.49867E+01_JPRB, 0.40363E+01_JPRB, 0.32522E+01_JPRB, 0.26760E+01_JPRB, 0.30821E+01_JPRB, 0.30560E+01_JPRB ,&
&  0.33582E+01_JPRB, 0.36852E+01_JPRB, 0.62209E+01_JPRB, 0.71225E+01_JPRB, 0.68845E+01_JPRB, 0.58122E+01_JPRB, 0.68719E+01_JPRB ,&
&  0.63919E+01_JPRB, 0.57022E+01_JPRB, 0.46769E+01_JPRB, 0.38867E+01_JPRB, 0.32354E+01_JPRB, 0.28222E+01_JPRB, 0.25591E+01_JPRB ,&
&  0.22890E+01_JPRB, 0.21559E+01_JPRB, 0.20622E+01_JPRB, 0.19720E+01_JPRB, 0.18619E+01_JPRB /)
ZBURD(:, 22,  7) = (/ &
&  0.15977E+01_JPRB, 0.14920E+01_JPRB, 0.14502E+01_JPRB, 0.13456E+01_JPRB, 0.11851E+01_JPRB, 0.10721E+01_JPRB, 0.10338E+01_JPRB ,&
&  0.99787E+00_JPRB, 0.95120E+00_JPRB, 0.90356E+00_JPRB, 0.87973E+00_JPRB, 0.89865E+00_JPRB, 0.92519E+00_JPRB, 0.95988E+00_JPRB ,&
&  0.10810E+01_JPRB, 0.14590E+01_JPRB, 0.14189E+01_JPRB, 0.15650E+01_JPRB, 0.18433E+01_JPRB, 0.22929E+01_JPRB, 0.26276E+01_JPRB ,&
&  0.33346E+01_JPRB, 0.32619E+01_JPRB, 0.33391E+01_JPRB, 0.47655E+01_JPRB, 0.85138E+01_JPRB, 0.10093E+02_JPRB, 0.11088E+02_JPRB ,&
&  0.10842E+02_JPRB, 0.89602E+01_JPRB, 0.71394E+01_JPRB, 0.56255E+01_JPRB, 0.46352E+01_JPRB, 0.39775E+01_JPRB, 0.31406E+01_JPRB ,&
&  0.25597E+01_JPRB, 0.22626E+01_JPRB, 0.21702E+01_JPRB, 0.20834E+01_JPRB, 0.20112E+01_JPRB, 0.20158E+01_JPRB, 0.21293E+01_JPRB ,&
&  0.22964E+01_JPRB, 0.24650E+01_JPRB, 0.26654E+01_JPRB, 0.35432E+01_JPRB, 0.52409E+01_JPRB, 0.63720E+01_JPRB, 0.75505E+01_JPRB ,&
&  0.67533E+01_JPRB, 0.71748E+01_JPRB, 0.86570E+01_JPRB, 0.10806E+02_JPRB, 0.10299E+02_JPRB, 0.99085E+01_JPRB, 0.87905E+01_JPRB ,&
&  0.71423E+01_JPRB, 0.73162E+01_JPRB, 0.86688E+01_JPRB, 0.89176E+01_JPRB, 0.71942E+01_JPRB, 0.80979E+01_JPRB, 0.10482E+02_JPRB ,&
&  0.78014E+01_JPRB, 0.72184E+01_JPRB, 0.69973E+01_JPRB, 0.74307E+01_JPRB, 0.45688E+01_JPRB, 0.28109E+01_JPRB, 0.44224E+01_JPRB ,&
&  0.45126E+01_JPRB, 0.23556E+01_JPRB, 0.16234E+01_JPRB, 0.20328E+01_JPRB, 0.14455E+01_JPRB, 0.18453E+01_JPRB, 0.30483E+01_JPRB ,&
&  0.39386E+01_JPRB, 0.48163E+01_JPRB, 0.96143E+01_JPRB, 0.86156E+01_JPRB, 0.81424E+01_JPRB, 0.71810E+01_JPRB, 0.76899E+01_JPRB ,&
&  0.69548E+01_JPRB, 0.47342E+01_JPRB, 0.44063E+01_JPRB, 0.38441E+01_JPRB, 0.31334E+01_JPRB, 0.26144E+01_JPRB, 0.22509E+01_JPRB ,&
&  0.19724E+01_JPRB, 0.18158E+01_JPRB, 0.17325E+01_JPRB, 0.17017E+01_JPRB, 0.16993E+01_JPRB /)
ZBURD(:, 23,  7) = (/ &
&  0.13440E+01_JPRB, 0.13093E+01_JPRB, 0.12587E+01_JPRB, 0.11380E+01_JPRB, 0.10490E+01_JPRB, 0.10101E+01_JPRB, 0.97970E+00_JPRB ,&
&  0.94853E+00_JPRB, 0.90221E+00_JPRB, 0.88358E+00_JPRB, 0.90431E+00_JPRB, 0.93227E+00_JPRB, 0.96623E+00_JPRB, 0.10184E+01_JPRB ,&
&  0.11211E+01_JPRB, 0.13268E+01_JPRB, 0.16964E+01_JPRB, 0.20496E+01_JPRB, 0.23639E+01_JPRB, 0.23868E+01_JPRB, 0.30133E+01_JPRB ,&
&  0.31412E+01_JPRB, 0.28696E+01_JPRB, 0.32256E+01_JPRB, 0.46031E+01_JPRB, 0.68609E+01_JPRB, 0.81872E+01_JPRB, 0.85774E+01_JPRB ,&
&  0.73705E+01_JPRB, 0.60519E+01_JPRB, 0.51730E+01_JPRB, 0.45909E+01_JPRB, 0.38045E+01_JPRB, 0.28307E+01_JPRB, 0.22852E+01_JPRB ,&
&  0.20156E+01_JPRB, 0.20077E+01_JPRB, 0.20304E+01_JPRB, 0.19678E+01_JPRB, 0.19993E+01_JPRB, 0.20501E+01_JPRB, 0.21852E+01_JPRB ,&
&  0.23077E+01_JPRB, 0.24274E+01_JPRB, 0.27471E+01_JPRB, 0.39895E+01_JPRB, 0.62209E+01_JPRB, 0.75432E+01_JPRB, 0.65918E+01_JPRB ,&
&  0.56544E+01_JPRB, 0.58472E+01_JPRB, 0.88523E+01_JPRB, 0.97186E+01_JPRB, 0.91678E+01_JPRB, 0.93855E+01_JPRB, 0.83067E+01_JPRB ,&
&  0.83868E+01_JPRB, 0.87370E+01_JPRB, 0.78520E+01_JPRB, 0.80771E+01_JPRB, 0.75858E+01_JPRB, 0.69530E+01_JPRB, 0.77373E+01_JPRB ,&
&  0.75904E+01_JPRB, 0.64864E+01_JPRB, 0.54157E+01_JPRB, 0.48014E+01_JPRB, 0.51920E+01_JPRB, 0.33559E+01_JPRB, 0.19949E+01_JPRB ,&
&  0.15489E+01_JPRB, 0.11602E+01_JPRB, 0.90243E+00_JPRB, 0.93906E+00_JPRB, 0.86963E+00_JPRB, 0.14927E+01_JPRB, 0.28088E+01_JPRB ,&
&  0.55722E+01_JPRB, 0.77687E+01_JPRB, 0.10221E+02_JPRB, 0.10201E+02_JPRB, 0.87946E+01_JPRB, 0.79117E+01_JPRB, 0.86742E+01_JPRB ,&
&  0.55723E+01_JPRB, 0.44081E+01_JPRB, 0.41251E+01_JPRB, 0.33093E+01_JPRB, 0.24451E+01_JPRB, 0.19562E+01_JPRB, 0.16745E+01_JPRB ,&
&  0.15225E+01_JPRB, 0.14364E+01_JPRB, 0.13822E+01_JPRB, 0.13791E+01_JPRB, 0.13903E+01_JPRB /)
ZBURD(:, 24,  7) = (/ &
&  0.11461E+01_JPRB, 0.11193E+01_JPRB, 0.10962E+01_JPRB, 0.10311E+01_JPRB, 0.97269E+00_JPRB, 0.96673E+00_JPRB, 0.95354E+00_JPRB ,&
&  0.92818E+00_JPRB, 0.90829E+00_JPRB, 0.91884E+00_JPRB, 0.95107E+00_JPRB, 0.99041E+00_JPRB, 0.10497E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.11858E+01_JPRB, 0.13832E+01_JPRB, 0.18700E+01_JPRB, 0.24316E+01_JPRB, 0.29652E+01_JPRB, 0.27931E+01_JPRB, 0.31603E+01_JPRB ,&
&  0.27583E+01_JPRB, 0.27015E+01_JPRB, 0.33118E+01_JPRB, 0.44777E+01_JPRB, 0.61664E+01_JPRB, 0.61124E+01_JPRB, 0.53007E+01_JPRB ,&
&  0.46213E+01_JPRB, 0.41331E+01_JPRB, 0.37469E+01_JPRB, 0.33238E+01_JPRB, 0.26334E+01_JPRB, 0.23226E+01_JPRB, 0.22355E+01_JPRB ,&
&  0.21622E+01_JPRB, 0.21875E+01_JPRB, 0.21315E+01_JPRB, 0.20886E+01_JPRB, 0.21048E+01_JPRB, 0.21566E+01_JPRB, 0.22624E+01_JPRB ,&
&  0.23726E+01_JPRB, 0.26136E+01_JPRB, 0.33999E+01_JPRB, 0.53816E+01_JPRB, 0.65423E+01_JPRB, 0.64753E+01_JPRB, 0.64763E+01_JPRB ,&
&  0.65097E+01_JPRB, 0.73834E+01_JPRB, 0.80995E+01_JPRB, 0.97001E+01_JPRB, 0.86848E+01_JPRB, 0.84219E+01_JPRB, 0.77389E+01_JPRB ,&
&  0.75047E+01_JPRB, 0.80131E+01_JPRB, 0.68758E+01_JPRB, 0.72301E+01_JPRB, 0.86863E+01_JPRB, 0.79604E+01_JPRB, 0.69490E+01_JPRB ,&
&  0.71896E+01_JPRB, 0.60851E+01_JPRB, 0.56552E+01_JPRB, 0.51090E+01_JPRB, 0.62606E+01_JPRB, 0.54871E+01_JPRB, 0.20161E+01_JPRB ,&
&  0.11485E+01_JPRB, 0.88311E+00_JPRB, 0.64679E+00_JPRB, 0.63876E+00_JPRB, 0.80467E+00_JPRB, 0.14712E+01_JPRB, 0.43489E+01_JPRB ,&
&  0.50141E+01_JPRB, 0.80869E+01_JPRB, 0.90468E+01_JPRB, 0.95853E+01_JPRB, 0.84077E+01_JPRB, 0.76681E+01_JPRB, 0.61836E+01_JPRB ,&
&  0.49165E+01_JPRB, 0.37962E+01_JPRB, 0.28838E+01_JPRB, 0.22402E+01_JPRB, 0.17640E+01_JPRB, 0.15177E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.13307E+01_JPRB, 0.12994E+01_JPRB, 0.12554E+01_JPRB, 0.12100E+01_JPRB, 0.11545E+01_JPRB /)
ZBURD(:, 25,  7) = (/ &
&  0.97842E+00_JPRB, 0.99189E+00_JPRB, 0.10154E+01_JPRB, 0.10132E+01_JPRB, 0.10056E+01_JPRB, 0.10080E+01_JPRB, 0.99550E+00_JPRB ,&
&  0.97815E+00_JPRB, 0.96441E+00_JPRB, 0.98287E+00_JPRB, 0.10182E+01_JPRB, 0.10729E+01_JPRB, 0.11353E+01_JPRB, 0.11968E+01_JPRB ,&
&  0.13276E+01_JPRB, 0.16110E+01_JPRB, 0.21330E+01_JPRB, 0.29020E+01_JPRB, 0.32735E+01_JPRB, 0.30655E+01_JPRB, 0.30652E+01_JPRB ,&
&  0.25666E+01_JPRB, 0.26944E+01_JPRB, 0.31388E+01_JPRB, 0.39511E+01_JPRB, 0.41517E+01_JPRB, 0.38849E+01_JPRB, 0.31173E+01_JPRB ,&
&  0.30868E+01_JPRB, 0.29962E+01_JPRB, 0.28476E+01_JPRB, 0.26899E+01_JPRB, 0.26027E+01_JPRB, 0.25731E+01_JPRB, 0.25790E+01_JPRB ,&
&  0.25677E+01_JPRB, 0.24822E+01_JPRB, 0.23804E+01_JPRB, 0.23097E+01_JPRB, 0.22918E+01_JPRB, 0.23475E+01_JPRB, 0.24754E+01_JPRB ,&
&  0.27085E+01_JPRB, 0.33137E+01_JPRB, 0.47415E+01_JPRB, 0.65162E+01_JPRB, 0.61411E+01_JPRB, 0.70322E+01_JPRB, 0.67538E+01_JPRB ,&
&  0.66485E+01_JPRB, 0.71602E+01_JPRB, 0.72346E+01_JPRB, 0.82898E+01_JPRB, 0.84267E+01_JPRB, 0.82416E+01_JPRB, 0.72692E+01_JPRB ,&
&  0.74434E+01_JPRB, 0.71371E+01_JPRB, 0.73616E+01_JPRB, 0.78006E+01_JPRB, 0.84823E+01_JPRB, 0.96127E+01_JPRB, 0.67007E+01_JPRB ,&
&  0.59468E+01_JPRB, 0.63163E+01_JPRB, 0.59078E+01_JPRB, 0.52410E+01_JPRB, 0.52456E+01_JPRB, 0.59022E+01_JPRB, 0.41033E+01_JPRB ,&
&  0.13720E+01_JPRB, 0.85975E+00_JPRB, 0.75552E+00_JPRB, 0.73581E+00_JPRB, 0.69059E+00_JPRB, 0.14748E+01_JPRB, 0.62693E+01_JPRB ,&
&  0.51645E+01_JPRB, 0.68205E+01_JPRB, 0.76713E+01_JPRB, 0.78664E+01_JPRB, 0.72967E+01_JPRB, 0.52013E+01_JPRB, 0.35971E+01_JPRB ,&
&  0.23367E+01_JPRB, 0.17623E+01_JPRB, 0.15608E+01_JPRB, 0.14044E+01_JPRB, 0.12010E+01_JPRB, 0.10998E+01_JPRB, 0.10512E+01_JPRB ,&
&  0.10474E+01_JPRB, 0.10241E+01_JPRB, 0.10018E+01_JPRB, 0.97131E+00_JPRB, 0.97006E+00_JPRB /)
ZBURD(:, 26,  7) = (/ &
&  0.93454E+00_JPRB, 0.99769E+00_JPRB, 0.10587E+01_JPRB, 0.11022E+01_JPRB, 0.11106E+01_JPRB, 0.10918E+01_JPRB, 0.10717E+01_JPRB ,&
&  0.10647E+01_JPRB, 0.10561E+01_JPRB, 0.10761E+01_JPRB, 0.11095E+01_JPRB, 0.11847E+01_JPRB, 0.12769E+01_JPRB, 0.14284E+01_JPRB ,&
&  0.16712E+01_JPRB, 0.20313E+01_JPRB, 0.25662E+01_JPRB, 0.31469E+01_JPRB, 0.33182E+01_JPRB, 0.29173E+01_JPRB, 0.27219E+01_JPRB ,&
&  0.24559E+01_JPRB, 0.26130E+01_JPRB, 0.25521E+01_JPRB, 0.29622E+01_JPRB, 0.33748E+01_JPRB, 0.35241E+01_JPRB, 0.30081E+01_JPRB ,&
&  0.30575E+01_JPRB, 0.30507E+01_JPRB, 0.30136E+01_JPRB, 0.29909E+01_JPRB, 0.29790E+01_JPRB, 0.29765E+01_JPRB, 0.29150E+01_JPRB ,&
&  0.27769E+01_JPRB, 0.26434E+01_JPRB, 0.25766E+01_JPRB, 0.25747E+01_JPRB, 0.26457E+01_JPRB, 0.27638E+01_JPRB, 0.30173E+01_JPRB ,&
&  0.35499E+01_JPRB, 0.45382E+01_JPRB, 0.58458E+01_JPRB, 0.64335E+01_JPRB, 0.65226E+01_JPRB, 0.65105E+01_JPRB, 0.64716E+01_JPRB ,&
&  0.65322E+01_JPRB, 0.70078E+01_JPRB, 0.72602E+01_JPRB, 0.73806E+01_JPRB, 0.76908E+01_JPRB, 0.80050E+01_JPRB, 0.72144E+01_JPRB ,&
&  0.74986E+01_JPRB, 0.76613E+01_JPRB, 0.71216E+01_JPRB, 0.71378E+01_JPRB, 0.74064E+01_JPRB, 0.81406E+01_JPRB, 0.75425E+01_JPRB ,&
&  0.64036E+01_JPRB, 0.54004E+01_JPRB, 0.49482E+01_JPRB, 0.47991E+01_JPRB, 0.42802E+01_JPRB, 0.43687E+01_JPRB, 0.47614E+01_JPRB ,&
&  0.38336E+01_JPRB, 0.15313E+01_JPRB, 0.97416E+00_JPRB, 0.12810E+01_JPRB, 0.94262E+00_JPRB, 0.14416E+01_JPRB, 0.28930E+01_JPRB ,&
&  0.35170E+01_JPRB, 0.42398E+01_JPRB, 0.49909E+01_JPRB, 0.51878E+01_JPRB, 0.42785E+01_JPRB, 0.29499E+01_JPRB, 0.19938E+01_JPRB ,&
&  0.14454E+01_JPRB, 0.12572E+01_JPRB, 0.12220E+01_JPRB, 0.10819E+01_JPRB, 0.97029E+00_JPRB, 0.94094E+00_JPRB, 0.90541E+00_JPRB ,&
&  0.86581E+00_JPRB, 0.87526E+00_JPRB, 0.89626E+00_JPRB, 0.89775E+00_JPRB, 0.89258E+00_JPRB /)
ZBURD(:, 27,  7) = (/ &
&  0.10179E+01_JPRB, 0.10791E+01_JPRB, 0.11568E+01_JPRB, 0.11869E+01_JPRB, 0.11766E+01_JPRB, 0.11691E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12076E+01_JPRB, 0.12293E+01_JPRB, 0.12749E+01_JPRB, 0.13535E+01_JPRB, 0.14889E+01_JPRB, 0.16370E+01_JPRB, 0.18381E+01_JPRB ,&
&  0.21031E+01_JPRB, 0.24295E+01_JPRB, 0.28320E+01_JPRB, 0.32476E+01_JPRB, 0.35169E+01_JPRB, 0.35953E+01_JPRB, 0.23553E+01_JPRB ,&
&  0.20420E+01_JPRB, 0.26892E+01_JPRB, 0.25728E+01_JPRB, 0.28385E+01_JPRB, 0.31092E+01_JPRB, 0.32965E+01_JPRB, 0.32338E+01_JPRB ,&
&  0.32633E+01_JPRB, 0.32349E+01_JPRB, 0.32108E+01_JPRB, 0.31808E+01_JPRB, 0.31570E+01_JPRB, 0.31211E+01_JPRB, 0.30174E+01_JPRB ,&
&  0.29095E+01_JPRB, 0.28686E+01_JPRB, 0.29214E+01_JPRB, 0.30353E+01_JPRB, 0.32167E+01_JPRB, 0.34914E+01_JPRB, 0.39445E+01_JPRB ,&
&  0.45845E+01_JPRB, 0.53552E+01_JPRB, 0.60739E+01_JPRB, 0.60267E+01_JPRB, 0.61340E+01_JPRB, 0.61424E+01_JPRB, 0.64755E+01_JPRB ,&
&  0.64083E+01_JPRB, 0.60344E+01_JPRB, 0.66300E+01_JPRB, 0.73298E+01_JPRB, 0.75469E+01_JPRB, 0.73475E+01_JPRB, 0.67993E+01_JPRB ,&
&  0.73956E+01_JPRB, 0.74910E+01_JPRB, 0.73017E+01_JPRB, 0.64236E+01_JPRB, 0.62766E+01_JPRB, 0.68406E+01_JPRB, 0.70818E+01_JPRB ,&
&  0.62512E+01_JPRB, 0.54073E+01_JPRB, 0.46386E+01_JPRB, 0.38861E+01_JPRB, 0.33279E+01_JPRB, 0.32374E+01_JPRB, 0.33664E+01_JPRB ,&
&  0.33401E+01_JPRB, 0.35005E+01_JPRB, 0.26918E+01_JPRB, 0.16646E+01_JPRB, 0.13478E+01_JPRB, 0.18226E+01_JPRB, 0.23725E+01_JPRB ,&
&  0.25322E+01_JPRB, 0.27459E+01_JPRB, 0.27687E+01_JPRB, 0.32363E+01_JPRB, 0.21039E+01_JPRB, 0.15555E+01_JPRB, 0.11962E+01_JPRB ,&
&  0.10690E+01_JPRB, 0.10571E+01_JPRB, 0.10582E+01_JPRB, 0.97592E+00_JPRB, 0.90329E+00_JPRB, 0.91722E+00_JPRB, 0.87308E+00_JPRB ,&
&  0.88828E+00_JPRB, 0.92231E+00_JPRB, 0.93826E+00_JPRB, 0.95426E+00_JPRB, 0.98820E+00_JPRB /)
ZBURD(:, 28,  7) = (/ &
&  0.11502E+01_JPRB, 0.11917E+01_JPRB, 0.12404E+01_JPRB, 0.12567E+01_JPRB, 0.12854E+01_JPRB, 0.13165E+01_JPRB, 0.13752E+01_JPRB ,&
&  0.14399E+01_JPRB, 0.15204E+01_JPRB, 0.16149E+01_JPRB, 0.17365E+01_JPRB, 0.18981E+01_JPRB, 0.20532E+01_JPRB, 0.22462E+01_JPRB ,&
&  0.24554E+01_JPRB, 0.26685E+01_JPRB, 0.28912E+01_JPRB, 0.31312E+01_JPRB, 0.32779E+01_JPRB, 0.33694E+01_JPRB, 0.27680E+01_JPRB ,&
&  0.18154E+01_JPRB, 0.28829E+01_JPRB, 0.25439E+01_JPRB, 0.27899E+01_JPRB, 0.29354E+01_JPRB, 0.31760E+01_JPRB, 0.31723E+01_JPRB ,&
&  0.31746E+01_JPRB, 0.31578E+01_JPRB, 0.31274E+01_JPRB, 0.31489E+01_JPRB, 0.31630E+01_JPRB, 0.31502E+01_JPRB, 0.31140E+01_JPRB ,&
&  0.31322E+01_JPRB, 0.32222E+01_JPRB, 0.33914E+01_JPRB, 0.36081E+01_JPRB, 0.39069E+01_JPRB, 0.42910E+01_JPRB, 0.46638E+01_JPRB ,&
&  0.50607E+01_JPRB, 0.54778E+01_JPRB, 0.56844E+01_JPRB, 0.56654E+01_JPRB, 0.56984E+01_JPRB, 0.59765E+01_JPRB, 0.65248E+01_JPRB ,&
&  0.65547E+01_JPRB, 0.63847E+01_JPRB, 0.67301E+01_JPRB, 0.68905E+01_JPRB, 0.67234E+01_JPRB, 0.69924E+01_JPRB, 0.64277E+01_JPRB ,&
&  0.70955E+01_JPRB, 0.67249E+01_JPRB, 0.71800E+01_JPRB, 0.58679E+01_JPRB, 0.53721E+01_JPRB, 0.56384E+01_JPRB, 0.53709E+01_JPRB ,&
&  0.46590E+01_JPRB, 0.43803E+01_JPRB, 0.37633E+01_JPRB, 0.29538E+01_JPRB, 0.26942E+01_JPRB, 0.25427E+01_JPRB, 0.26983E+01_JPRB ,&
&  0.28326E+01_JPRB, 0.28059E+01_JPRB, 0.21272E+01_JPRB, 0.14547E+01_JPRB, 0.13611E+01_JPRB, 0.16571E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.19316E+01_JPRB, 0.20535E+01_JPRB, 0.21426E+01_JPRB, 0.18236E+01_JPRB, 0.12371E+01_JPRB, 0.10245E+01_JPRB, 0.93402E+00_JPRB ,&
&  0.95047E+00_JPRB, 0.99096E+00_JPRB, 0.99132E+00_JPRB, 0.96584E+00_JPRB, 0.90150E+00_JPRB, 0.94125E+00_JPRB, 0.94652E+00_JPRB ,&
&  0.10054E+01_JPRB, 0.10328E+01_JPRB, 0.10771E+01_JPRB, 0.11120E+01_JPRB, 0.11308E+01_JPRB /)
ZBURD(:, 29,  7) = (/ &
&  0.13085E+01_JPRB, 0.13758E+01_JPRB, 0.14808E+01_JPRB, 0.15779E+01_JPRB, 0.16422E+01_JPRB, 0.16906E+01_JPRB, 0.17200E+01_JPRB ,&
&  0.17145E+01_JPRB, 0.17198E+01_JPRB, 0.18381E+01_JPRB, 0.19595E+01_JPRB, 0.21158E+01_JPRB, 0.22789E+01_JPRB, 0.24472E+01_JPRB ,&
&  0.26060E+01_JPRB, 0.27673E+01_JPRB, 0.29284E+01_JPRB, 0.31266E+01_JPRB, 0.32854E+01_JPRB, 0.35174E+01_JPRB, 0.32325E+01_JPRB ,&
&  0.23460E+01_JPRB, 0.21299E+01_JPRB, 0.25563E+01_JPRB, 0.26103E+01_JPRB, 0.25909E+01_JPRB, 0.28481E+01_JPRB, 0.30380E+01_JPRB ,&
&  0.30145E+01_JPRB, 0.30382E+01_JPRB, 0.30686E+01_JPRB, 0.31211E+01_JPRB, 0.31288E+01_JPRB, 0.31645E+01_JPRB, 0.32166E+01_JPRB ,&
&  0.33489E+01_JPRB, 0.35482E+01_JPRB, 0.37658E+01_JPRB, 0.40287E+01_JPRB, 0.43092E+01_JPRB, 0.46142E+01_JPRB, 0.48880E+01_JPRB ,&
&  0.51186E+01_JPRB, 0.53336E+01_JPRB, 0.53926E+01_JPRB, 0.52855E+01_JPRB, 0.54261E+01_JPRB, 0.56979E+01_JPRB, 0.59444E+01_JPRB ,&
&  0.61619E+01_JPRB, 0.66358E+01_JPRB, 0.69152E+01_JPRB, 0.68383E+01_JPRB, 0.63258E+01_JPRB, 0.66883E+01_JPRB, 0.62802E+01_JPRB ,&
&  0.64697E+01_JPRB, 0.61924E+01_JPRB, 0.67416E+01_JPRB, 0.54372E+01_JPRB, 0.48485E+01_JPRB, 0.46161E+01_JPRB, 0.42040E+01_JPRB ,&
&  0.37139E+01_JPRB, 0.35118E+01_JPRB, 0.31356E+01_JPRB, 0.23280E+01_JPRB, 0.22366E+01_JPRB, 0.19483E+01_JPRB, 0.23948E+01_JPRB ,&
&  0.26720E+01_JPRB, 0.28791E+01_JPRB, 0.18819E+01_JPRB, 0.14872E+01_JPRB, 0.13512E+01_JPRB, 0.14633E+01_JPRB, 0.15379E+01_JPRB ,&
&  0.15501E+01_JPRB, 0.14594E+01_JPRB, 0.13596E+01_JPRB, 0.12301E+01_JPRB, 0.94062E+00_JPRB, 0.85628E+00_JPRB, 0.84195E+00_JPRB ,&
&  0.89821E+00_JPRB, 0.95212E+00_JPRB, 0.95939E+00_JPRB, 0.94764E+00_JPRB, 0.95875E+00_JPRB, 0.10273E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.11289E+01_JPRB, 0.11721E+01_JPRB, 0.12176E+01_JPRB, 0.12344E+01_JPRB, 0.12548E+01_JPRB /)
ZBURD(:, 30,  7) = (/ &
&  0.12902E+01_JPRB, 0.13556E+01_JPRB, 0.14686E+01_JPRB, 0.15631E+01_JPRB, 0.16290E+01_JPRB, 0.16561E+01_JPRB, 0.16494E+01_JPRB ,&
&  0.16673E+01_JPRB, 0.17238E+01_JPRB, 0.18321E+01_JPRB, 0.19357E+01_JPRB, 0.20548E+01_JPRB, 0.21810E+01_JPRB, 0.23100E+01_JPRB ,&
&  0.24327E+01_JPRB, 0.25393E+01_JPRB, 0.26582E+01_JPRB, 0.27940E+01_JPRB, 0.29310E+01_JPRB, 0.31652E+01_JPRB, 0.31899E+01_JPRB ,&
&  0.25564E+01_JPRB, 0.17946E+01_JPRB, 0.21181E+01_JPRB, 0.21870E+01_JPRB, 0.22002E+01_JPRB, 0.24994E+01_JPRB, 0.26729E+01_JPRB ,&
&  0.26311E+01_JPRB, 0.27070E+01_JPRB, 0.28454E+01_JPRB, 0.29544E+01_JPRB, 0.30437E+01_JPRB, 0.31897E+01_JPRB, 0.33203E+01_JPRB ,&
&  0.34919E+01_JPRB, 0.36670E+01_JPRB, 0.38473E+01_JPRB, 0.40457E+01_JPRB, 0.42256E+01_JPRB, 0.44347E+01_JPRB, 0.46647E+01_JPRB ,&
&  0.49033E+01_JPRB, 0.50949E+01_JPRB, 0.50815E+01_JPRB, 0.50721E+01_JPRB, 0.51145E+01_JPRB, 0.52108E+01_JPRB, 0.52978E+01_JPRB ,&
&  0.54969E+01_JPRB, 0.57849E+01_JPRB, 0.62844E+01_JPRB, 0.66008E+01_JPRB, 0.67130E+01_JPRB, 0.62337E+01_JPRB, 0.60949E+01_JPRB ,&
&  0.58207E+01_JPRB, 0.53092E+01_JPRB, 0.54943E+01_JPRB, 0.58303E+01_JPRB, 0.40754E+01_JPRB, 0.39065E+01_JPRB, 0.33048E+01_JPRB ,&
&  0.33668E+01_JPRB, 0.31080E+01_JPRB, 0.27403E+01_JPRB, 0.23222E+01_JPRB, 0.18828E+01_JPRB, 0.16319E+01_JPRB, 0.19578E+01_JPRB ,&
&  0.23645E+01_JPRB, 0.21660E+01_JPRB, 0.16124E+01_JPRB, 0.13786E+01_JPRB, 0.13134E+01_JPRB, 0.15241E+01_JPRB, 0.15353E+01_JPRB ,&
&  0.13885E+01_JPRB, 0.12755E+01_JPRB, 0.12113E+01_JPRB, 0.13710E+01_JPRB, 0.10534E+01_JPRB, 0.81056E+00_JPRB, 0.77456E+00_JPRB ,&
&  0.84121E+00_JPRB, 0.90245E+00_JPRB, 0.95489E+00_JPRB, 0.99209E+00_JPRB, 0.10248E+01_JPRB, 0.10755E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.11590E+01_JPRB, 0.11620E+01_JPRB, 0.12133E+01_JPRB, 0.12318E+01_JPRB, 0.12605E+01_JPRB /)
ZBURD(:, 31,  7) = (/ &
&  0.12324E+01_JPRB, 0.12786E+01_JPRB, 0.13257E+01_JPRB, 0.13842E+01_JPRB, 0.14452E+01_JPRB, 0.14784E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15196E+01_JPRB, 0.15784E+01_JPRB, 0.16486E+01_JPRB, 0.17119E+01_JPRB, 0.18037E+01_JPRB, 0.18876E+01_JPRB, 0.19767E+01_JPRB ,&
&  0.20659E+01_JPRB, 0.21624E+01_JPRB, 0.22875E+01_JPRB, 0.24078E+01_JPRB, 0.24908E+01_JPRB, 0.26180E+01_JPRB, 0.27693E+01_JPRB ,&
&  0.29156E+01_JPRB, 0.26322E+01_JPRB, 0.24495E+01_JPRB, 0.19460E+01_JPRB, 0.15372E+01_JPRB, 0.19873E+01_JPRB, 0.20818E+01_JPRB ,&
&  0.21875E+01_JPRB, 0.22877E+01_JPRB, 0.23989E+01_JPRB, 0.25498E+01_JPRB, 0.27798E+01_JPRB, 0.29768E+01_JPRB, 0.31218E+01_JPRB ,&
&  0.32559E+01_JPRB, 0.33712E+01_JPRB, 0.34733E+01_JPRB, 0.35900E+01_JPRB, 0.37273E+01_JPRB, 0.38508E+01_JPRB, 0.39742E+01_JPRB ,&
&  0.41355E+01_JPRB, 0.42860E+01_JPRB, 0.43901E+01_JPRB, 0.45260E+01_JPRB, 0.45244E+01_JPRB, 0.46235E+01_JPRB, 0.47322E+01_JPRB ,&
&  0.48536E+01_JPRB, 0.49761E+01_JPRB, 0.52892E+01_JPRB, 0.55850E+01_JPRB, 0.56241E+01_JPRB, 0.52261E+01_JPRB, 0.50328E+01_JPRB ,&
&  0.48059E+01_JPRB, 0.43231E+01_JPRB, 0.42488E+01_JPRB, 0.53816E+01_JPRB, 0.33296E+01_JPRB, 0.35127E+01_JPRB, 0.34219E+01_JPRB ,&
&  0.29808E+01_JPRB, 0.28035E+01_JPRB, 0.24442E+01_JPRB, 0.19300E+01_JPRB, 0.16581E+01_JPRB, 0.14157E+01_JPRB, 0.16754E+01_JPRB ,&
&  0.18791E+01_JPRB, 0.15925E+01_JPRB, 0.12503E+01_JPRB, 0.10995E+01_JPRB, 0.11646E+01_JPRB, 0.14719E+01_JPRB, 0.15154E+01_JPRB ,&
&  0.12617E+01_JPRB, 0.12381E+01_JPRB, 0.11236E+01_JPRB, 0.15937E+01_JPRB, 0.14614E+01_JPRB, 0.76787E+00_JPRB, 0.67259E+00_JPRB ,&
&  0.70760E+00_JPRB, 0.77470E+00_JPRB, 0.86824E+00_JPRB, 0.92876E+00_JPRB, 0.94880E+00_JPRB, 0.97821E+00_JPRB, 0.10045E+01_JPRB ,&
&  0.10298E+01_JPRB, 0.10795E+01_JPRB, 0.11401E+01_JPRB, 0.11723E+01_JPRB, 0.12026E+01_JPRB /)
ZBURD(:, 32,  7) = (/ &
&  0.98564E+00_JPRB, 0.10261E+01_JPRB, 0.10567E+01_JPRB, 0.10702E+01_JPRB, 0.10726E+01_JPRB, 0.10654E+01_JPRB, 0.10387E+01_JPRB ,&
&  0.10057E+01_JPRB, 0.10249E+01_JPRB, 0.10898E+01_JPRB, 0.12367E+01_JPRB, 0.13545E+01_JPRB, 0.14269E+01_JPRB, 0.15112E+01_JPRB ,&
&  0.15923E+01_JPRB, 0.16170E+01_JPRB, 0.15839E+01_JPRB, 0.15885E+01_JPRB, 0.15849E+01_JPRB, 0.15822E+01_JPRB, 0.17771E+01_JPRB ,&
&  0.20797E+01_JPRB, 0.24350E+01_JPRB, 0.27513E+01_JPRB, 0.29913E+01_JPRB, 0.24704E+01_JPRB, 0.18832E+01_JPRB, 0.20416E+01_JPRB ,&
&  0.23070E+01_JPRB, 0.22292E+01_JPRB, 0.20218E+01_JPRB, 0.20647E+01_JPRB, 0.21376E+01_JPRB, 0.22832E+01_JPRB, 0.24765E+01_JPRB ,&
&  0.26144E+01_JPRB, 0.27125E+01_JPRB, 0.28024E+01_JPRB, 0.28752E+01_JPRB, 0.29490E+01_JPRB, 0.30158E+01_JPRB, 0.30887E+01_JPRB ,&
&  0.31144E+01_JPRB, 0.30947E+01_JPRB, 0.31537E+01_JPRB, 0.33510E+01_JPRB, 0.35212E+01_JPRB, 0.37127E+01_JPRB, 0.38418E+01_JPRB ,&
&  0.39165E+01_JPRB, 0.40264E+01_JPRB, 0.44888E+01_JPRB, 0.47436E+01_JPRB, 0.44857E+01_JPRB, 0.41490E+01_JPRB, 0.40464E+01_JPRB ,&
&  0.38021E+01_JPRB, 0.33524E+01_JPRB, 0.27520E+01_JPRB, 0.40642E+01_JPRB, 0.40571E+01_JPRB, 0.34601E+01_JPRB, 0.29097E+01_JPRB ,&
&  0.25955E+01_JPRB, 0.24459E+01_JPRB, 0.21907E+01_JPRB, 0.18676E+01_JPRB, 0.15781E+01_JPRB, 0.16931E+01_JPRB, 0.16404E+01_JPRB ,&
&  0.16353E+01_JPRB, 0.13320E+01_JPRB, 0.10413E+01_JPRB, 0.93192E+00_JPRB, 0.94261E+00_JPRB, 0.12532E+01_JPRB, 0.12665E+01_JPRB ,&
&  0.10558E+01_JPRB, 0.10282E+01_JPRB, 0.96349E+00_JPRB, 0.14497E+01_JPRB, 0.25449E+01_JPRB, 0.77139E+00_JPRB, 0.58393E+00_JPRB ,&
&  0.59967E+00_JPRB, 0.63363E+00_JPRB, 0.66860E+00_JPRB, 0.67598E+00_JPRB, 0.67304E+00_JPRB, 0.70012E+00_JPRB, 0.74671E+00_JPRB ,&
&  0.80185E+00_JPRB, 0.84734E+00_JPRB, 0.86977E+00_JPRB, 0.91449E+00_JPRB, 0.94980E+00_JPRB /)
ZBURD(:, 33,  7) = (/ &
&  0.71875E+00_JPRB, 0.73964E+00_JPRB, 0.74047E+00_JPRB, 0.74872E+00_JPRB, 0.74383E+00_JPRB, 0.74604E+00_JPRB, 0.72426E+00_JPRB ,&
&  0.70734E+00_JPRB, 0.72092E+00_JPRB, 0.76880E+00_JPRB, 0.85836E+00_JPRB, 0.88503E+00_JPRB, 0.90834E+00_JPRB, 0.91763E+00_JPRB ,&
&  0.94225E+00_JPRB, 0.91084E+00_JPRB, 0.89134E+00_JPRB, 0.85403E+00_JPRB, 0.85194E+00_JPRB, 0.88867E+00_JPRB, 0.98416E+00_JPRB ,&
&  0.11251E+01_JPRB, 0.13011E+01_JPRB, 0.15377E+01_JPRB, 0.18081E+01_JPRB, 0.23954E+01_JPRB, 0.20972E+01_JPRB, 0.18966E+01_JPRB ,&
&  0.21396E+01_JPRB, 0.18299E+01_JPRB, 0.16969E+01_JPRB, 0.16100E+01_JPRB, 0.15327E+01_JPRB, 0.14821E+01_JPRB, 0.15482E+01_JPRB ,&
&  0.15922E+01_JPRB, 0.16582E+01_JPRB, 0.17367E+01_JPRB, 0.17628E+01_JPRB, 0.17774E+01_JPRB, 0.17380E+01_JPRB, 0.16578E+01_JPRB ,&
&  0.15315E+01_JPRB, 0.15813E+01_JPRB, 0.16587E+01_JPRB, 0.18423E+01_JPRB, 0.21552E+01_JPRB, 0.23383E+01_JPRB, 0.24639E+01_JPRB ,&
&  0.25095E+01_JPRB, 0.26860E+01_JPRB, 0.30531E+01_JPRB, 0.31873E+01_JPRB, 0.32496E+01_JPRB, 0.30413E+01_JPRB, 0.27888E+01_JPRB ,&
&  0.24471E+01_JPRB, 0.19420E+01_JPRB, 0.17198E+01_JPRB, 0.27105E+01_JPRB, 0.28789E+01_JPRB, 0.23416E+01_JPRB, 0.24624E+01_JPRB ,&
&  0.22659E+01_JPRB, 0.21345E+01_JPRB, 0.20547E+01_JPRB, 0.17719E+01_JPRB, 0.14528E+01_JPRB, 0.14820E+01_JPRB, 0.15750E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.10738E+01_JPRB, 0.86112E+00_JPRB, 0.80810E+00_JPRB, 0.83133E+00_JPRB, 0.10277E+01_JPRB, 0.11196E+01_JPRB ,&
&  0.10844E+01_JPRB, 0.84484E+00_JPRB, 0.88386E+00_JPRB, 0.12771E+01_JPRB, 0.10642E+01_JPRB, 0.62440E+00_JPRB, 0.48924E+00_JPRB ,&
&  0.49205E+00_JPRB, 0.50620E+00_JPRB, 0.52377E+00_JPRB, 0.53014E+00_JPRB, 0.53252E+00_JPRB, 0.54049E+00_JPRB, 0.57469E+00_JPRB ,&
&  0.60374E+00_JPRB, 0.63259E+00_JPRB, 0.65523E+00_JPRB, 0.68746E+00_JPRB, 0.70652E+00_JPRB /)
ZBURD(:, 34,  7) = (/ &
&  0.69310E+00_JPRB, 0.70682E+00_JPRB, 0.72578E+00_JPRB, 0.73768E+00_JPRB, 0.75286E+00_JPRB, 0.76578E+00_JPRB, 0.79939E+00_JPRB ,&
&  0.82704E+00_JPRB, 0.85257E+00_JPRB, 0.88672E+00_JPRB, 0.92300E+00_JPRB, 0.93516E+00_JPRB, 0.94606E+00_JPRB, 0.95193E+00_JPRB ,&
&  0.96962E+00_JPRB, 0.98402E+00_JPRB, 0.97670E+00_JPRB, 0.97799E+00_JPRB, 0.98019E+00_JPRB, 0.10481E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10377E+01_JPRB, 0.10526E+01_JPRB, 0.10676E+01_JPRB, 0.11272E+01_JPRB, 0.13048E+01_JPRB, 0.14569E+01_JPRB, 0.14821E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.14057E+01_JPRB, 0.13862E+01_JPRB, 0.11015E+01_JPRB, 0.10351E+01_JPRB, 0.10376E+01_JPRB, 0.97662E+00_JPRB ,&
&  0.95703E+00_JPRB, 0.97507E+00_JPRB, 0.10238E+01_JPRB, 0.10024E+01_JPRB, 0.10132E+01_JPRB, 0.10016E+01_JPRB, 0.96059E+00_JPRB ,&
&  0.95170E+00_JPRB, 0.97161E+00_JPRB, 0.10145E+01_JPRB, 0.11662E+01_JPRB, 0.12759E+01_JPRB, 0.14280E+01_JPRB, 0.14737E+01_JPRB ,&
&  0.14645E+01_JPRB, 0.15056E+01_JPRB, 0.14577E+01_JPRB, 0.15363E+01_JPRB, 0.17056E+01_JPRB, 0.17229E+01_JPRB, 0.16110E+01_JPRB ,&
&  0.15559E+01_JPRB, 0.14346E+01_JPRB, 0.12320E+01_JPRB, 0.19283E+01_JPRB, 0.22300E+01_JPRB, 0.21012E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.19374E+01_JPRB, 0.17416E+01_JPRB, 0.16000E+01_JPRB, 0.15340E+01_JPRB, 0.13652E+01_JPRB, 0.12625E+01_JPRB, 0.12771E+01_JPRB ,&
&  0.10949E+01_JPRB, 0.90546E+00_JPRB, 0.79406E+00_JPRB, 0.82036E+00_JPRB, 0.98082E+00_JPRB, 0.13899E+01_JPRB, 0.12641E+01_JPRB ,&
&  0.96452E+00_JPRB, 0.78108E+00_JPRB, 0.10422E+01_JPRB, 0.15078E+01_JPRB, 0.87070E+00_JPRB, 0.60415E+00_JPRB, 0.46457E+00_JPRB ,&
&  0.46030E+00_JPRB, 0.48289E+00_JPRB, 0.50713E+00_JPRB, 0.52650E+00_JPRB, 0.55122E+00_JPRB, 0.57303E+00_JPRB, 0.58985E+00_JPRB ,&
&  0.60630E+00_JPRB, 0.62771E+00_JPRB, 0.64614E+00_JPRB, 0.66850E+00_JPRB, 0.68762E+00_JPRB /)
ZBURD(:, 35,  7) = (/ &
&  0.80892E+00_JPRB, 0.84339E+00_JPRB, 0.86792E+00_JPRB, 0.88792E+00_JPRB, 0.90192E+00_JPRB, 0.91803E+00_JPRB, 0.94142E+00_JPRB ,&
&  0.96195E+00_JPRB, 0.98252E+00_JPRB, 0.10028E+01_JPRB, 0.10298E+01_JPRB, 0.10639E+01_JPRB, 0.10920E+01_JPRB, 0.11033E+01_JPRB ,&
&  0.11095E+01_JPRB, 0.11457E+01_JPRB, 0.11667E+01_JPRB, 0.12110E+01_JPRB, 0.12277E+01_JPRB, 0.12581E+01_JPRB, 0.12636E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.12584E+01_JPRB, 0.12641E+01_JPRB, 0.12833E+01_JPRB, 0.13001E+01_JPRB, 0.14183E+01_JPRB, 0.15928E+01_JPRB ,&
&  0.14145E+01_JPRB, 0.13261E+01_JPRB, 0.10488E+01_JPRB, 0.85483E+00_JPRB, 0.84932E+00_JPRB, 0.85131E+00_JPRB, 0.83742E+00_JPRB ,&
&  0.84197E+00_JPRB, 0.85521E+00_JPRB, 0.86913E+00_JPRB, 0.84882E+00_JPRB, 0.87391E+00_JPRB, 0.93970E+00_JPRB, 0.93395E+00_JPRB ,&
&  0.95688E+00_JPRB, 0.98305E+00_JPRB, 0.99611E+00_JPRB, 0.11037E+01_JPRB, 0.10992E+01_JPRB, 0.12435E+01_JPRB, 0.13048E+01_JPRB ,&
&  0.13018E+01_JPRB, 0.12272E+01_JPRB, 0.99134E+00_JPRB, 0.10156E+01_JPRB, 0.11160E+01_JPRB, 0.11022E+01_JPRB, 0.10762E+01_JPRB ,&
&  0.12478E+01_JPRB, 0.13681E+01_JPRB, 0.13241E+01_JPRB, 0.17618E+01_JPRB, 0.19345E+01_JPRB, 0.19113E+01_JPRB, 0.17024E+01_JPRB ,&
&  0.14972E+01_JPRB, 0.11944E+01_JPRB, 0.11888E+01_JPRB, 0.12049E+01_JPRB, 0.11405E+01_JPRB, 0.10742E+01_JPRB, 0.98529E+00_JPRB ,&
&  0.88474E+00_JPRB, 0.80747E+00_JPRB, 0.78684E+00_JPRB, 0.84601E+00_JPRB, 0.12105E+01_JPRB, 0.13039E+01_JPRB, 0.16003E+01_JPRB ,&
&  0.80187E+00_JPRB, 0.76296E+00_JPRB, 0.63813E+00_JPRB, 0.79396E+00_JPRB, 0.67013E+00_JPRB, 0.49919E+00_JPRB, 0.47674E+00_JPRB ,&
&  0.52337E+00_JPRB, 0.56890E+00_JPRB, 0.60359E+00_JPRB, 0.62861E+00_JPRB, 0.65117E+00_JPRB, 0.67625E+00_JPRB, 0.68813E+00_JPRB ,&
&  0.70488E+00_JPRB, 0.72397E+00_JPRB, 0.74239E+00_JPRB, 0.76996E+00_JPRB, 0.78559E+00_JPRB /)
ZBURD(:, 36,  7) = (/ &
&  0.85693E+00_JPRB, 0.88421E+00_JPRB, 0.90821E+00_JPRB, 0.93047E+00_JPRB, 0.95454E+00_JPRB, 0.98002E+00_JPRB, 0.10025E+01_JPRB ,&
&  0.10219E+01_JPRB, 0.10433E+01_JPRB, 0.10619E+01_JPRB, 0.10805E+01_JPRB, 0.11072E+01_JPRB, 0.11320E+01_JPRB, 0.11478E+01_JPRB ,&
&  0.11690E+01_JPRB, 0.11996E+01_JPRB, 0.12337E+01_JPRB, 0.12642E+01_JPRB, 0.12796E+01_JPRB, 0.12875E+01_JPRB, 0.12845E+01_JPRB ,&
&  0.12701E+01_JPRB, 0.12621E+01_JPRB, 0.12673E+01_JPRB, 0.12842E+01_JPRB, 0.13175E+01_JPRB, 0.13041E+01_JPRB, 0.13948E+01_JPRB ,&
&  0.11628E+01_JPRB, 0.13253E+01_JPRB, 0.10684E+01_JPRB, 0.93564E+00_JPRB, 0.94024E+00_JPRB, 0.88633E+00_JPRB, 0.88193E+00_JPRB ,&
&  0.90657E+00_JPRB, 0.90366E+00_JPRB, 0.89880E+00_JPRB, 0.89084E+00_JPRB, 0.91012E+00_JPRB, 0.95319E+00_JPRB, 0.97066E+00_JPRB ,&
&  0.10173E+01_JPRB, 0.10797E+01_JPRB, 0.11234E+01_JPRB, 0.11885E+01_JPRB, 0.13083E+01_JPRB, 0.13990E+01_JPRB, 0.14336E+01_JPRB ,&
&  0.14093E+01_JPRB, 0.13526E+01_JPRB, 0.10736E+01_JPRB, 0.11163E+01_JPRB, 0.11888E+01_JPRB, 0.11446E+01_JPRB, 0.10780E+01_JPRB ,&
&  0.11468E+01_JPRB, 0.11281E+01_JPRB, 0.13648E+01_JPRB, 0.14978E+01_JPRB, 0.14787E+01_JPRB, 0.13371E+01_JPRB, 0.11253E+01_JPRB ,&
&  0.93587E+00_JPRB, 0.79300E+00_JPRB, 0.81517E+00_JPRB, 0.93190E+00_JPRB, 0.90481E+00_JPRB, 0.77318E+00_JPRB, 0.73673E+00_JPRB ,&
&  0.68635E+00_JPRB, 0.63729E+00_JPRB, 0.64030E+00_JPRB, 0.74026E+00_JPRB, 0.95747E+00_JPRB, 0.15692E+01_JPRB, 0.15753E+01_JPRB ,&
&  0.75198E+00_JPRB, 0.68127E+00_JPRB, 0.48222E+00_JPRB, 0.53960E+00_JPRB, 0.55263E+00_JPRB, 0.50127E+00_JPRB, 0.53966E+00_JPRB ,&
&  0.65161E+00_JPRB, 0.75101E+00_JPRB, 0.76456E+00_JPRB, 0.70644E+00_JPRB, 0.68831E+00_JPRB, 0.69877E+00_JPRB, 0.72002E+00_JPRB ,&
&  0.74205E+00_JPRB, 0.76267E+00_JPRB, 0.78365E+00_JPRB, 0.80655E+00_JPRB, 0.83044E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART13',1,ZHOOK_HANDLE)
END SUBROUTINE PART13



SUBROUTINE PART14

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART14',0,ZHOOK_HANDLE)


ZBURD(:, 37,  7) = (/ &
&  0.87885E+00_JPRB, 0.90608E+00_JPRB, 0.92896E+00_JPRB, 0.94851E+00_JPRB, 0.97268E+00_JPRB, 0.99922E+00_JPRB, 0.10233E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10645E+01_JPRB, 0.10867E+01_JPRB, 0.11078E+01_JPRB, 0.11317E+01_JPRB, 0.11588E+01_JPRB, 0.11876E+01_JPRB ,&
&  0.12146E+01_JPRB, 0.12324E+01_JPRB, 0.12681E+01_JPRB, 0.13111E+01_JPRB, 0.13404E+01_JPRB, 0.13524E+01_JPRB, 0.13555E+01_JPRB ,&
&  0.13586E+01_JPRB, 0.13837E+01_JPRB, 0.14000E+01_JPRB, 0.14177E+01_JPRB, 0.14481E+01_JPRB, 0.15373E+01_JPRB, 0.13338E+01_JPRB ,&
&  0.14351E+01_JPRB, 0.12889E+01_JPRB, 0.10898E+01_JPRB, 0.10243E+01_JPRB, 0.10073E+01_JPRB, 0.99297E+00_JPRB, 0.98626E+00_JPRB ,&
&  0.99835E+00_JPRB, 0.96124E+00_JPRB, 0.94613E+00_JPRB, 0.93575E+00_JPRB, 0.95340E+00_JPRB, 0.98513E+00_JPRB, 0.10234E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.11819E+01_JPRB, 0.12871E+01_JPRB, 0.14015E+01_JPRB, 0.15368E+01_JPRB, 0.16384E+01_JPRB, 0.16969E+01_JPRB ,&
&  0.17022E+01_JPRB, 0.16814E+01_JPRB, 0.14612E+01_JPRB, 0.15281E+01_JPRB, 0.15462E+01_JPRB, 0.13652E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.10538E+01_JPRB, 0.93796E+00_JPRB, 0.10277E+01_JPRB, 0.11488E+01_JPRB, 0.10822E+01_JPRB, 0.93558E+00_JPRB, 0.79577E+00_JPRB ,&
&  0.63889E+00_JPRB, 0.58952E+00_JPRB, 0.58801E+00_JPRB, 0.68127E+00_JPRB, 0.67729E+00_JPRB, 0.60705E+00_JPRB, 0.60448E+00_JPRB ,&
&  0.58602E+00_JPRB, 0.56015E+00_JPRB, 0.57534E+00_JPRB, 0.67612E+00_JPRB, 0.84830E+00_JPRB, 0.85143E+00_JPRB, 0.91034E+00_JPRB ,&
&  0.68049E+00_JPRB, 0.59372E+00_JPRB, 0.58771E+00_JPRB, 0.57800E+00_JPRB, 0.61806E+00_JPRB, 0.64653E+00_JPRB, 0.74748E+00_JPRB ,&
&  0.90955E+00_JPRB, 0.11215E+01_JPRB, 0.10945E+01_JPRB, 0.88764E+00_JPRB, 0.73215E+00_JPRB, 0.70389E+00_JPRB, 0.72273E+00_JPRB ,&
&  0.74670E+00_JPRB, 0.76705E+00_JPRB, 0.79179E+00_JPRB, 0.81902E+00_JPRB, 0.84809E+00_JPRB /)
ZBURD(:, 38,  7) = (/ &
&  0.88218E+00_JPRB, 0.90731E+00_JPRB, 0.92676E+00_JPRB, 0.94200E+00_JPRB, 0.96446E+00_JPRB, 0.98827E+00_JPRB, 0.10117E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10522E+01_JPRB, 0.10750E+01_JPRB, 0.10958E+01_JPRB, 0.11207E+01_JPRB, 0.11485E+01_JPRB, 0.11796E+01_JPRB ,&
&  0.12061E+01_JPRB, 0.12351E+01_JPRB, 0.12545E+01_JPRB, 0.13025E+01_JPRB, 0.13616E+01_JPRB, 0.13980E+01_JPRB, 0.14194E+01_JPRB ,&
&  0.14214E+01_JPRB, 0.14468E+01_JPRB, 0.14760E+01_JPRB, 0.15073E+01_JPRB, 0.15150E+01_JPRB, 0.15874E+01_JPRB, 0.13974E+01_JPRB ,&
&  0.14100E+01_JPRB, 0.12863E+01_JPRB, 0.11468E+01_JPRB, 0.11137E+01_JPRB, 0.10890E+01_JPRB, 0.10782E+01_JPRB, 0.10649E+01_JPRB ,&
&  0.10487E+01_JPRB, 0.10357E+01_JPRB, 0.10073E+01_JPRB, 0.98944E+00_JPRB, 0.99302E+00_JPRB, 0.10177E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.11419E+01_JPRB, 0.12452E+01_JPRB, 0.13630E+01_JPRB, 0.14912E+01_JPRB, 0.16351E+01_JPRB, 0.17577E+01_JPRB, 0.18703E+01_JPRB ,&
&  0.19582E+01_JPRB, 0.20131E+01_JPRB, 0.18710E+01_JPRB, 0.18457E+01_JPRB, 0.17929E+01_JPRB, 0.13826E+01_JPRB, 0.11112E+01_JPRB ,&
&  0.87933E+00_JPRB, 0.87704E+00_JPRB, 0.93394E+00_JPRB, 0.97779E+00_JPRB, 0.83186E+00_JPRB, 0.73576E+00_JPRB, 0.62018E+00_JPRB ,&
&  0.53132E+00_JPRB, 0.49562E+00_JPRB, 0.49920E+00_JPRB, 0.54998E+00_JPRB, 0.53411E+00_JPRB, 0.50616E+00_JPRB, 0.52631E+00_JPRB ,&
&  0.52588E+00_JPRB, 0.54767E+00_JPRB, 0.58945E+00_JPRB, 0.66285E+00_JPRB, 0.73862E+00_JPRB, 0.73702E+00_JPRB, 0.78806E+00_JPRB ,&
&  0.71677E+00_JPRB, 0.70085E+00_JPRB, 0.67075E+00_JPRB, 0.55089E+00_JPRB, 0.72521E+00_JPRB, 0.82434E+00_JPRB, 0.95620E+00_JPRB ,&
&  0.10918E+01_JPRB, 0.14894E+01_JPRB, 0.17334E+01_JPRB, 0.13561E+01_JPRB, 0.10483E+01_JPRB, 0.76111E+00_JPRB, 0.73052E+00_JPRB ,&
&  0.75318E+00_JPRB, 0.76150E+00_JPRB, 0.78579E+00_JPRB, 0.81700E+00_JPRB, 0.85088E+00_JPRB /)
ZBURD(:, 39,  7) = (/ &
&  0.86080E+00_JPRB, 0.88960E+00_JPRB, 0.90471E+00_JPRB, 0.90645E+00_JPRB, 0.92567E+00_JPRB, 0.95846E+00_JPRB, 0.97529E+00_JPRB ,&
&  0.99318E+00_JPRB, 0.10007E+01_JPRB, 0.10211E+01_JPRB, 0.10411E+01_JPRB, 0.10618E+01_JPRB, 0.10849E+01_JPRB, 0.11110E+01_JPRB ,&
&  0.11310E+01_JPRB, 0.11766E+01_JPRB, 0.12035E+01_JPRB, 0.12547E+01_JPRB, 0.13097E+01_JPRB, 0.13836E+01_JPRB, 0.14251E+01_JPRB ,&
&  0.14315E+01_JPRB, 0.14688E+01_JPRB, 0.15110E+01_JPRB, 0.15625E+01_JPRB, 0.15543E+01_JPRB, 0.15638E+01_JPRB, 0.12293E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.12735E+01_JPRB, 0.12096E+01_JPRB, 0.11802E+01_JPRB, 0.11431E+01_JPRB, 0.11077E+01_JPRB, 0.10728E+01_JPRB ,&
&  0.10599E+01_JPRB, 0.10347E+01_JPRB, 0.98469E+00_JPRB, 0.10059E+01_JPRB, 0.10167E+01_JPRB, 0.10267E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.11417E+01_JPRB, 0.12305E+01_JPRB, 0.13405E+01_JPRB, 0.14733E+01_JPRB, 0.16136E+01_JPRB, 0.17524E+01_JPRB, 0.18886E+01_JPRB ,&
&  0.20157E+01_JPRB, 0.21200E+01_JPRB, 0.21405E+01_JPRB, 0.18101E+01_JPRB, 0.15923E+01_JPRB, 0.12632E+01_JPRB, 0.10661E+01_JPRB ,&
&  0.93734E+00_JPRB, 0.88453E+00_JPRB, 0.89370E+00_JPRB, 0.90589E+00_JPRB, 0.74922E+00_JPRB, 0.64266E+00_JPRB, 0.54421E+00_JPRB ,&
&  0.50635E+00_JPRB, 0.47343E+00_JPRB, 0.47127E+00_JPRB, 0.46966E+00_JPRB, 0.46830E+00_JPRB, 0.46835E+00_JPRB, 0.48642E+00_JPRB ,&
&  0.50155E+00_JPRB, 0.53911E+00_JPRB, 0.61574E+00_JPRB, 0.71327E+00_JPRB, 0.80196E+00_JPRB, 0.84711E+00_JPRB, 0.76918E+00_JPRB ,&
&  0.75818E+00_JPRB, 0.70141E+00_JPRB, 0.68410E+00_JPRB, 0.72684E+00_JPRB, 0.80311E+00_JPRB, 0.97434E+00_JPRB, 0.11815E+01_JPRB ,&
&  0.14365E+01_JPRB, 0.14831E+01_JPRB, 0.17803E+01_JPRB, 0.23333E+01_JPRB, 0.21058E+01_JPRB, 0.17971E+01_JPRB, 0.68584E+00_JPRB ,&
&  0.72475E+00_JPRB, 0.73632E+00_JPRB, 0.76904E+00_JPRB, 0.80154E+00_JPRB, 0.84262E+00_JPRB /)
ZBURD(:, 40,  7) = (/ &
&  0.70596E+00_JPRB, 0.73521E+00_JPRB, 0.75747E+00_JPRB, 0.76162E+00_JPRB, 0.79346E+00_JPRB, 0.82127E+00_JPRB, 0.83885E+00_JPRB ,&
&  0.85493E+00_JPRB, 0.88077E+00_JPRB, 0.89824E+00_JPRB, 0.92754E+00_JPRB, 0.94945E+00_JPRB, 0.97126E+00_JPRB, 0.99111E+00_JPRB ,&
&  0.10119E+01_JPRB, 0.10480E+01_JPRB, 0.10879E+01_JPRB, 0.11441E+01_JPRB, 0.12008E+01_JPRB, 0.12725E+01_JPRB, 0.13368E+01_JPRB ,&
&  0.13835E+01_JPRB, 0.14368E+01_JPRB, 0.15001E+01_JPRB, 0.15386E+01_JPRB, 0.15192E+01_JPRB, 0.14896E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.12518E+01_JPRB, 0.12688E+01_JPRB, 0.12544E+01_JPRB, 0.12217E+01_JPRB, 0.11773E+01_JPRB, 0.11229E+01_JPRB, 0.10887E+01_JPRB ,&
&  0.10639E+01_JPRB, 0.10244E+01_JPRB, 0.97870E+00_JPRB, 0.94345E+00_JPRB, 0.10080E+01_JPRB, 0.10191E+01_JPRB, 0.10629E+01_JPRB ,&
&  0.11221E+01_JPRB, 0.11961E+01_JPRB, 0.12852E+01_JPRB, 0.14130E+01_JPRB, 0.15550E+01_JPRB, 0.16940E+01_JPRB, 0.18358E+01_JPRB ,&
&  0.19376E+01_JPRB, 0.19900E+01_JPRB, 0.20130E+01_JPRB, 0.16584E+01_JPRB, 0.14419E+01_JPRB, 0.12306E+01_JPRB, 0.10208E+01_JPRB ,&
&  0.91097E+00_JPRB, 0.87451E+00_JPRB, 0.89728E+00_JPRB, 0.86148E+00_JPRB, 0.72871E+00_JPRB, 0.60475E+00_JPRB, 0.51972E+00_JPRB ,&
&  0.48538E+00_JPRB, 0.44975E+00_JPRB, 0.44647E+00_JPRB, 0.44967E+00_JPRB, 0.46164E+00_JPRB, 0.47218E+00_JPRB, 0.49009E+00_JPRB ,&
&  0.51468E+00_JPRB, 0.55365E+00_JPRB, 0.62152E+00_JPRB, 0.72563E+00_JPRB, 0.86529E+00_JPRB, 0.10167E+01_JPRB, 0.98108E+00_JPRB ,&
&  0.79499E+00_JPRB, 0.68921E+00_JPRB, 0.66719E+00_JPRB, 0.66392E+00_JPRB, 0.68878E+00_JPRB, 0.74637E+00_JPRB, 0.81932E+00_JPRB ,&
&  0.94605E+00_JPRB, 0.10203E+01_JPRB, 0.92176E+00_JPRB, 0.77051E+00_JPRB, 0.10399E+01_JPRB, 0.10246E+01_JPRB, 0.55763E+00_JPRB ,&
&  0.60753E+00_JPRB, 0.59666E+00_JPRB, 0.62177E+00_JPRB, 0.65188E+00_JPRB, 0.68949E+00_JPRB /)
ZBURD(:, 41,  7) = (/ &
&  0.54231E+00_JPRB, 0.59019E+00_JPRB, 0.61084E+00_JPRB, 0.63686E+00_JPRB, 0.65661E+00_JPRB, 0.68530E+00_JPRB, 0.71147E+00_JPRB ,&
&  0.73595E+00_JPRB, 0.76484E+00_JPRB, 0.78230E+00_JPRB, 0.81173E+00_JPRB, 0.83493E+00_JPRB, 0.85468E+00_JPRB, 0.87213E+00_JPRB ,&
&  0.89002E+00_JPRB, 0.92244E+00_JPRB, 0.95732E+00_JPRB, 0.99822E+00_JPRB, 0.10396E+01_JPRB, 0.10950E+01_JPRB, 0.11735E+01_JPRB ,&
&  0.12685E+01_JPRB, 0.13591E+01_JPRB, 0.14367E+01_JPRB, 0.14986E+01_JPRB, 0.15265E+01_JPRB, 0.15798E+01_JPRB, 0.15694E+01_JPRB ,&
&  0.11205E+01_JPRB, 0.13301E+01_JPRB, 0.12896E+01_JPRB, 0.12273E+01_JPRB, 0.11793E+01_JPRB, 0.11358E+01_JPRB, 0.11150E+01_JPRB ,&
&  0.10881E+01_JPRB, 0.10104E+01_JPRB, 0.98229E+00_JPRB, 0.95104E+00_JPRB, 0.96701E+00_JPRB, 0.98476E+00_JPRB, 0.10241E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.11518E+01_JPRB, 0.12311E+01_JPRB, 0.13153E+01_JPRB, 0.14343E+01_JPRB, 0.15869E+01_JPRB, 0.17207E+01_JPRB ,&
&  0.18581E+01_JPRB, 0.20003E+01_JPRB, 0.20800E+01_JPRB, 0.17844E+01_JPRB, 0.15267E+01_JPRB, 0.14051E+01_JPRB, 0.11410E+01_JPRB ,&
&  0.90621E+00_JPRB, 0.81640E+00_JPRB, 0.83896E+00_JPRB, 0.82837E+00_JPRB, 0.71093E+00_JPRB, 0.60584E+00_JPRB, 0.53141E+00_JPRB ,&
&  0.50148E+00_JPRB, 0.46793E+00_JPRB, 0.45647E+00_JPRB, 0.45989E+00_JPRB, 0.46947E+00_JPRB, 0.48255E+00_JPRB, 0.51296E+00_JPRB ,&
&  0.53596E+00_JPRB, 0.55811E+00_JPRB, 0.58902E+00_JPRB, 0.62273E+00_JPRB, 0.68180E+00_JPRB, 0.76847E+00_JPRB, 0.77758E+00_JPRB ,&
&  0.74277E+00_JPRB, 0.70646E+00_JPRB, 0.67119E+00_JPRB, 0.66453E+00_JPRB, 0.65195E+00_JPRB, 0.64795E+00_JPRB, 0.66107E+00_JPRB ,&
&  0.68201E+00_JPRB, 0.69792E+00_JPRB, 0.66068E+00_JPRB, 0.63011E+00_JPRB, 0.62610E+00_JPRB, 0.59379E+00_JPRB, 0.53868E+00_JPRB ,&
&  0.51755E+00_JPRB, 0.49929E+00_JPRB, 0.48152E+00_JPRB, 0.48906E+00_JPRB, 0.51244E+00_JPRB /)
ZBURD(:, 42,  7) = (/ &
&  0.45938E+00_JPRB, 0.47805E+00_JPRB, 0.49959E+00_JPRB, 0.52081E+00_JPRB, 0.54032E+00_JPRB, 0.55855E+00_JPRB, 0.58735E+00_JPRB ,&
&  0.61908E+00_JPRB, 0.64113E+00_JPRB, 0.65106E+00_JPRB, 0.67156E+00_JPRB, 0.69508E+00_JPRB, 0.72486E+00_JPRB, 0.74915E+00_JPRB ,&
&  0.77677E+00_JPRB, 0.81480E+00_JPRB, 0.84742E+00_JPRB, 0.88260E+00_JPRB, 0.91795E+00_JPRB, 0.95522E+00_JPRB, 0.10102E+01_JPRB ,&
&  0.10810E+01_JPRB, 0.11741E+01_JPRB, 0.12845E+01_JPRB, 0.13878E+01_JPRB, 0.14775E+01_JPRB, 0.15701E+01_JPRB, 0.15871E+01_JPRB ,&
&  0.99702E+00_JPRB, 0.12611E+01_JPRB, 0.14221E+01_JPRB, 0.12943E+01_JPRB, 0.11858E+01_JPRB, 0.11252E+01_JPRB, 0.10970E+01_JPRB ,&
&  0.10466E+01_JPRB, 0.97355E+00_JPRB, 0.94719E+00_JPRB, 0.98647E+00_JPRB, 0.92871E+00_JPRB, 0.93453E+00_JPRB, 0.95517E+00_JPRB ,&
&  0.99284E+00_JPRB, 0.10448E+01_JPRB, 0.10980E+01_JPRB, 0.11638E+01_JPRB, 0.12589E+01_JPRB, 0.13760E+01_JPRB, 0.15285E+01_JPRB ,&
&  0.17194E+01_JPRB, 0.19254E+01_JPRB, 0.21119E+01_JPRB, 0.17357E+01_JPRB, 0.16082E+01_JPRB, 0.15352E+01_JPRB, 0.13655E+01_JPRB ,&
&  0.98695E+00_JPRB, 0.81854E+00_JPRB, 0.81985E+00_JPRB, 0.81011E+00_JPRB, 0.72384E+00_JPRB, 0.64651E+00_JPRB, 0.58724E+00_JPRB ,&
&  0.55104E+00_JPRB, 0.52197E+00_JPRB, 0.50851E+00_JPRB, 0.50354E+00_JPRB, 0.50632E+00_JPRB, 0.52489E+00_JPRB, 0.54881E+00_JPRB ,&
&  0.57154E+00_JPRB, 0.57922E+00_JPRB, 0.60740E+00_JPRB, 0.62421E+00_JPRB, 0.66967E+00_JPRB, 0.71000E+00_JPRB, 0.73482E+00_JPRB ,&
&  0.74517E+00_JPRB, 0.73829E+00_JPRB, 0.73354E+00_JPRB, 0.72827E+00_JPRB, 0.72454E+00_JPRB, 0.72027E+00_JPRB, 0.71188E+00_JPRB ,&
&  0.69706E+00_JPRB, 0.70289E+00_JPRB, 0.68623E+00_JPRB, 0.67329E+00_JPRB, 0.67244E+00_JPRB, 0.63976E+00_JPRB, 0.62073E+00_JPRB ,&
&  0.59781E+00_JPRB, 0.54741E+00_JPRB, 0.51116E+00_JPRB, 0.48445E+00_JPRB, 0.46543E+00_JPRB /)
ZBURD(:, 43,  7) = (/ &
&  0.48306E+00_JPRB, 0.45716E+00_JPRB, 0.45954E+00_JPRB, 0.46357E+00_JPRB, 0.47940E+00_JPRB, 0.49561E+00_JPRB, 0.49131E+00_JPRB ,&
&  0.49873E+00_JPRB, 0.50916E+00_JPRB, 0.52378E+00_JPRB, 0.54108E+00_JPRB, 0.56286E+00_JPRB, 0.58584E+00_JPRB, 0.60493E+00_JPRB ,&
&  0.63307E+00_JPRB, 0.66730E+00_JPRB, 0.70617E+00_JPRB, 0.74435E+00_JPRB, 0.78481E+00_JPRB, 0.81713E+00_JPRB, 0.84837E+00_JPRB ,&
&  0.88773E+00_JPRB, 0.95150E+00_JPRB, 0.10415E+01_JPRB, 0.11506E+01_JPRB, 0.12894E+01_JPRB, 0.14525E+01_JPRB, 0.15564E+01_JPRB ,&
&  0.11447E+01_JPRB, 0.65912E+00_JPRB, 0.13085E+01_JPRB, 0.14503E+01_JPRB, 0.12701E+01_JPRB, 0.11845E+01_JPRB, 0.10918E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.10187E+01_JPRB, 0.96387E+00_JPRB, 0.96564E+00_JPRB, 0.91332E+00_JPRB, 0.90402E+00_JPRB, 0.90336E+00_JPRB ,&
&  0.90307E+00_JPRB, 0.91988E+00_JPRB, 0.97126E+00_JPRB, 0.10044E+01_JPRB, 0.10540E+01_JPRB, 0.11438E+01_JPRB, 0.12814E+01_JPRB ,&
&  0.14772E+01_JPRB, 0.17087E+01_JPRB, 0.19344E+01_JPRB, 0.15591E+01_JPRB, 0.13074E+01_JPRB, 0.11235E+01_JPRB, 0.99211E+00_JPRB ,&
&  0.92141E+00_JPRB, 0.81973E+00_JPRB, 0.79937E+00_JPRB, 0.79037E+00_JPRB, 0.74196E+00_JPRB, 0.66347E+00_JPRB, 0.64482E+00_JPRB ,&
&  0.60935E+00_JPRB, 0.58560E+00_JPRB, 0.57266E+00_JPRB, 0.56405E+00_JPRB, 0.56402E+00_JPRB, 0.57601E+00_JPRB, 0.59026E+00_JPRB ,&
&  0.61045E+00_JPRB, 0.62497E+00_JPRB, 0.64814E+00_JPRB, 0.67479E+00_JPRB, 0.69350E+00_JPRB, 0.71281E+00_JPRB, 0.73984E+00_JPRB ,&
&  0.75124E+00_JPRB, 0.76975E+00_JPRB, 0.78644E+00_JPRB, 0.79264E+00_JPRB, 0.79772E+00_JPRB, 0.78535E+00_JPRB, 0.78834E+00_JPRB ,&
&  0.78605E+00_JPRB, 0.77670E+00_JPRB, 0.75454E+00_JPRB, 0.75600E+00_JPRB, 0.75065E+00_JPRB, 0.74053E+00_JPRB, 0.75015E+00_JPRB ,&
&  0.73727E+00_JPRB, 0.70297E+00_JPRB, 0.63593E+00_JPRB, 0.56285E+00_JPRB, 0.52499E+00_JPRB /)
ZBURD(:, 44,  7) = (/ &
&  0.58215E+00_JPRB, 0.54103E+00_JPRB, 0.50653E+00_JPRB, 0.48814E+00_JPRB, 0.47581E+00_JPRB, 0.47092E+00_JPRB, 0.46964E+00_JPRB ,&
&  0.47369E+00_JPRB, 0.47813E+00_JPRB, 0.46607E+00_JPRB, 0.47117E+00_JPRB, 0.48440E+00_JPRB, 0.49976E+00_JPRB, 0.51983E+00_JPRB ,&
&  0.53276E+00_JPRB, 0.54464E+00_JPRB, 0.55895E+00_JPRB, 0.58297E+00_JPRB, 0.61525E+00_JPRB, 0.65135E+00_JPRB, 0.68595E+00_JPRB ,&
&  0.72101E+00_JPRB, 0.76004E+00_JPRB, 0.81413E+00_JPRB, 0.88866E+00_JPRB, 0.99344E+00_JPRB, 0.11442E+01_JPRB, 0.13114E+01_JPRB ,&
&  0.15255E+01_JPRB, 0.11377E+01_JPRB, 0.82215E+00_JPRB, 0.13157E+01_JPRB, 0.13931E+01_JPRB, 0.13178E+01_JPRB, 0.11307E+01_JPRB ,&
&  0.11194E+01_JPRB, 0.11225E+01_JPRB, 0.10433E+01_JPRB, 0.98890E+00_JPRB, 0.89909E+00_JPRB, 0.88133E+00_JPRB, 0.86429E+00_JPRB ,&
&  0.84498E+00_JPRB, 0.82791E+00_JPRB, 0.84845E+00_JPRB, 0.86831E+00_JPRB, 0.88395E+00_JPRB, 0.94083E+00_JPRB, 0.10631E+01_JPRB ,&
&  0.12320E+01_JPRB, 0.14528E+01_JPRB, 0.16962E+01_JPRB, 0.14608E+01_JPRB, 0.11950E+01_JPRB, 0.99534E+00_JPRB, 0.91910E+00_JPRB ,&
&  0.87467E+00_JPRB, 0.86411E+00_JPRB, 0.83327E+00_JPRB, 0.78898E+00_JPRB, 0.73746E+00_JPRB, 0.68925E+00_JPRB, 0.67769E+00_JPRB ,&
&  0.65880E+00_JPRB, 0.63063E+00_JPRB, 0.61052E+00_JPRB, 0.59192E+00_JPRB, 0.59116E+00_JPRB, 0.60108E+00_JPRB, 0.61264E+00_JPRB ,&
&  0.62521E+00_JPRB, 0.64111E+00_JPRB, 0.66519E+00_JPRB, 0.69368E+00_JPRB, 0.70833E+00_JPRB, 0.71753E+00_JPRB, 0.73462E+00_JPRB ,&
&  0.75303E+00_JPRB, 0.76973E+00_JPRB, 0.80069E+00_JPRB, 0.82208E+00_JPRB, 0.83103E+00_JPRB, 0.81667E+00_JPRB, 0.83808E+00_JPRB ,&
&  0.83889E+00_JPRB, 0.83904E+00_JPRB, 0.80765E+00_JPRB, 0.81843E+00_JPRB, 0.80519E+00_JPRB, 0.81179E+00_JPRB, 0.82250E+00_JPRB ,&
&  0.83370E+00_JPRB, 0.82224E+00_JPRB, 0.78000E+00_JPRB, 0.66444E+00_JPRB, 0.61877E+00_JPRB /)
ZBURD(:, 45,  7) = (/ &
&  0.62399E+00_JPRB, 0.59701E+00_JPRB, 0.57697E+00_JPRB, 0.55784E+00_JPRB, 0.53797E+00_JPRB, 0.51786E+00_JPRB, 0.50230E+00_JPRB ,&
&  0.48434E+00_JPRB, 0.48201E+00_JPRB, 0.47830E+00_JPRB, 0.47282E+00_JPRB, 0.46602E+00_JPRB, 0.46678E+00_JPRB, 0.47415E+00_JPRB ,&
&  0.48055E+00_JPRB, 0.48420E+00_JPRB, 0.48905E+00_JPRB, 0.49161E+00_JPRB, 0.49705E+00_JPRB, 0.51082E+00_JPRB, 0.53270E+00_JPRB ,&
&  0.56300E+00_JPRB, 0.59541E+00_JPRB, 0.63676E+00_JPRB, 0.68612E+00_JPRB, 0.73575E+00_JPRB, 0.81287E+00_JPRB, 0.91113E+00_JPRB ,&
&  0.11307E+01_JPRB, 0.11484E+01_JPRB, 0.71226E+00_JPRB, 0.11808E+01_JPRB, 0.14758E+01_JPRB, 0.13997E+01_JPRB, 0.13070E+01_JPRB ,&
&  0.12971E+01_JPRB, 0.12238E+01_JPRB, 0.11401E+01_JPRB, 0.10121E+01_JPRB, 0.88618E+00_JPRB, 0.85225E+00_JPRB, 0.82998E+00_JPRB ,&
&  0.79664E+00_JPRB, 0.77146E+00_JPRB, 0.76355E+00_JPRB, 0.76316E+00_JPRB, 0.76160E+00_JPRB, 0.78192E+00_JPRB, 0.86821E+00_JPRB ,&
&  0.10024E+01_JPRB, 0.12130E+01_JPRB, 0.14439E+01_JPRB, 0.13638E+01_JPRB, 0.11144E+01_JPRB, 0.10044E+01_JPRB, 0.95225E+00_JPRB ,&
&  0.93505E+00_JPRB, 0.92241E+00_JPRB, 0.84723E+00_JPRB, 0.78322E+00_JPRB, 0.72850E+00_JPRB, 0.69406E+00_JPRB, 0.69558E+00_JPRB ,&
&  0.68121E+00_JPRB, 0.65946E+00_JPRB, 0.64179E+00_JPRB, 0.61766E+00_JPRB, 0.60664E+00_JPRB, 0.60304E+00_JPRB, 0.61083E+00_JPRB ,&
&  0.62317E+00_JPRB, 0.63541E+00_JPRB, 0.65105E+00_JPRB, 0.67010E+00_JPRB, 0.68392E+00_JPRB, 0.69703E+00_JPRB, 0.70925E+00_JPRB ,&
&  0.73056E+00_JPRB, 0.75992E+00_JPRB, 0.79209E+00_JPRB, 0.81442E+00_JPRB, 0.83679E+00_JPRB, 0.85435E+00_JPRB, 0.86248E+00_JPRB ,&
&  0.87462E+00_JPRB, 0.87306E+00_JPRB, 0.79524E+00_JPRB, 0.78835E+00_JPRB, 0.81689E+00_JPRB, 0.80540E+00_JPRB, 0.78668E+00_JPRB ,&
&  0.81073E+00_JPRB, 0.83079E+00_JPRB, 0.82023E+00_JPRB, 0.70399E+00_JPRB, 0.65732E+00_JPRB /)
ZBURD(:, 46,  7) = (/ &
&  0.63367E+00_JPRB, 0.61233E+00_JPRB, 0.59548E+00_JPRB, 0.58278E+00_JPRB, 0.56714E+00_JPRB, 0.55125E+00_JPRB, 0.53872E+00_JPRB ,&
&  0.52144E+00_JPRB, 0.51116E+00_JPRB, 0.50392E+00_JPRB, 0.49582E+00_JPRB, 0.48756E+00_JPRB, 0.47929E+00_JPRB, 0.47187E+00_JPRB ,&
&  0.46592E+00_JPRB, 0.46132E+00_JPRB, 0.45845E+00_JPRB, 0.45283E+00_JPRB, 0.44783E+00_JPRB, 0.44681E+00_JPRB, 0.45168E+00_JPRB ,&
&  0.46967E+00_JPRB, 0.49516E+00_JPRB, 0.52510E+00_JPRB, 0.55625E+00_JPRB, 0.58305E+00_JPRB, 0.61966E+00_JPRB, 0.66500E+00_JPRB ,&
&  0.84350E+00_JPRB, 0.93974E+00_JPRB, 0.11210E+01_JPRB, 0.17549E+01_JPRB, 0.17878E+01_JPRB, 0.15988E+01_JPRB, 0.15697E+01_JPRB ,&
&  0.15609E+01_JPRB, 0.14492E+01_JPRB, 0.13562E+01_JPRB, 0.11189E+01_JPRB, 0.93661E+00_JPRB, 0.84264E+00_JPRB, 0.78665E+00_JPRB ,&
&  0.73576E+00_JPRB, 0.70369E+00_JPRB, 0.69551E+00_JPRB, 0.68833E+00_JPRB, 0.68418E+00_JPRB, 0.71144E+00_JPRB, 0.75462E+00_JPRB ,&
&  0.84126E+00_JPRB, 0.98735E+00_JPRB, 0.12021E+01_JPRB, 0.13242E+01_JPRB, 0.10990E+01_JPRB, 0.10534E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.12212E+01_JPRB, 0.10705E+01_JPRB, 0.89623E+00_JPRB, 0.78873E+00_JPRB, 0.72124E+00_JPRB, 0.70718E+00_JPRB, 0.68375E+00_JPRB ,&
&  0.67353E+00_JPRB, 0.66147E+00_JPRB, 0.64891E+00_JPRB, 0.63214E+00_JPRB, 0.62091E+00_JPRB, 0.61103E+00_JPRB, 0.60910E+00_JPRB ,&
&  0.61401E+00_JPRB, 0.62137E+00_JPRB, 0.62895E+00_JPRB, 0.64025E+00_JPRB, 0.65593E+00_JPRB, 0.67059E+00_JPRB, 0.68539E+00_JPRB ,&
&  0.70195E+00_JPRB, 0.73380E+00_JPRB, 0.73478E+00_JPRB, 0.75059E+00_JPRB, 0.78429E+00_JPRB, 0.80578E+00_JPRB, 0.78517E+00_JPRB ,&
&  0.77386E+00_JPRB, 0.79881E+00_JPRB, 0.79588E+00_JPRB, 0.79826E+00_JPRB, 0.80492E+00_JPRB, 0.79963E+00_JPRB, 0.73389E+00_JPRB ,&
&  0.71352E+00_JPRB, 0.70460E+00_JPRB, 0.69994E+00_JPRB, 0.67815E+00_JPRB, 0.65695E+00_JPRB /)
ZBURD(:, 47,  7) = (/ &
&  0.64138E+00_JPRB, 0.62436E+00_JPRB, 0.60627E+00_JPRB, 0.58797E+00_JPRB, 0.56796E+00_JPRB, 0.55364E+00_JPRB, 0.54627E+00_JPRB ,&
&  0.53471E+00_JPRB, 0.52466E+00_JPRB, 0.51706E+00_JPRB, 0.51357E+00_JPRB, 0.51059E+00_JPRB, 0.50112E+00_JPRB, 0.48375E+00_JPRB ,&
&  0.46617E+00_JPRB, 0.45419E+00_JPRB, 0.44753E+00_JPRB, 0.44067E+00_JPRB, 0.43336E+00_JPRB, 0.42925E+00_JPRB, 0.43052E+00_JPRB ,&
&  0.43788E+00_JPRB, 0.44970E+00_JPRB, 0.47229E+00_JPRB, 0.49738E+00_JPRB, 0.51756E+00_JPRB, 0.53539E+00_JPRB, 0.54896E+00_JPRB ,&
&  0.65266E+00_JPRB, 0.82525E+00_JPRB, 0.89290E+00_JPRB, 0.15136E+01_JPRB, 0.16057E+01_JPRB, 0.15649E+01_JPRB, 0.15606E+01_JPRB ,&
&  0.15497E+01_JPRB, 0.16558E+01_JPRB, 0.14464E+01_JPRB, 0.12244E+01_JPRB, 0.10280E+01_JPRB, 0.89549E+00_JPRB, 0.80022E+00_JPRB ,&
&  0.72806E+00_JPRB, 0.67956E+00_JPRB, 0.65451E+00_JPRB, 0.64402E+00_JPRB, 0.64083E+00_JPRB, 0.65434E+00_JPRB, 0.69417E+00_JPRB ,&
&  0.75420E+00_JPRB, 0.85435E+00_JPRB, 0.10097E+01_JPRB, 0.11031E+01_JPRB, 0.10604E+01_JPRB, 0.10782E+01_JPRB, 0.11729E+01_JPRB ,&
&  0.14316E+01_JPRB, 0.13196E+01_JPRB, 0.99473E+00_JPRB, 0.81331E+00_JPRB, 0.72400E+00_JPRB, 0.69335E+00_JPRB, 0.66571E+00_JPRB ,&
&  0.65432E+00_JPRB, 0.64394E+00_JPRB, 0.63526E+00_JPRB, 0.62630E+00_JPRB, 0.61686E+00_JPRB, 0.61231E+00_JPRB, 0.61025E+00_JPRB ,&
&  0.60934E+00_JPRB, 0.61189E+00_JPRB, 0.61427E+00_JPRB, 0.61727E+00_JPRB, 0.62313E+00_JPRB, 0.63566E+00_JPRB, 0.65410E+00_JPRB ,&
&  0.67288E+00_JPRB, 0.70000E+00_JPRB, 0.70261E+00_JPRB, 0.69799E+00_JPRB, 0.71742E+00_JPRB, 0.73101E+00_JPRB, 0.72009E+00_JPRB ,&
&  0.73279E+00_JPRB, 0.76143E+00_JPRB, 0.77292E+00_JPRB, 0.76146E+00_JPRB, 0.79140E+00_JPRB, 0.82667E+00_JPRB, 0.73255E+00_JPRB ,&
&  0.69625E+00_JPRB, 0.67664E+00_JPRB, 0.67113E+00_JPRB, 0.66006E+00_JPRB, 0.65067E+00_JPRB /)
ZBURD(:, 48,  7) = (/ &
&  0.63451E+00_JPRB, 0.62576E+00_JPRB, 0.60964E+00_JPRB, 0.59020E+00_JPRB, 0.57036E+00_JPRB, 0.55419E+00_JPRB, 0.54627E+00_JPRB ,&
&  0.53749E+00_JPRB, 0.52832E+00_JPRB, 0.51965E+00_JPRB, 0.51462E+00_JPRB, 0.51181E+00_JPRB, 0.50233E+00_JPRB, 0.48635E+00_JPRB ,&
&  0.46940E+00_JPRB, 0.45367E+00_JPRB, 0.44093E+00_JPRB, 0.43362E+00_JPRB, 0.42593E+00_JPRB, 0.42221E+00_JPRB, 0.42206E+00_JPRB ,&
&  0.42374E+00_JPRB, 0.43083E+00_JPRB, 0.44554E+00_JPRB, 0.46877E+00_JPRB, 0.48853E+00_JPRB, 0.50015E+00_JPRB, 0.50908E+00_JPRB ,&
&  0.57048E+00_JPRB, 0.90974E+00_JPRB, 0.82446E+00_JPRB, 0.11813E+01_JPRB, 0.12656E+01_JPRB, 0.13262E+01_JPRB, 0.14062E+01_JPRB ,&
&  0.14417E+01_JPRB, 0.14863E+01_JPRB, 0.14029E+01_JPRB, 0.12574E+01_JPRB, 0.10869E+01_JPRB, 0.96291E+00_JPRB, 0.86297E+00_JPRB ,&
&  0.78678E+00_JPRB, 0.73138E+00_JPRB, 0.69106E+00_JPRB, 0.66016E+00_JPRB, 0.64053E+00_JPRB, 0.63970E+00_JPRB, 0.67266E+00_JPRB ,&
&  0.74032E+00_JPRB, 0.81636E+00_JPRB, 0.90775E+00_JPRB, 0.10147E+01_JPRB, 0.96356E+00_JPRB, 0.10500E+01_JPRB, 0.13000E+01_JPRB ,&
&  0.15373E+01_JPRB, 0.15811E+01_JPRB, 0.11644E+01_JPRB, 0.87247E+00_JPRB, 0.74250E+00_JPRB, 0.68184E+00_JPRB, 0.64452E+00_JPRB ,&
&  0.62346E+00_JPRB, 0.61294E+00_JPRB, 0.60691E+00_JPRB, 0.60406E+00_JPRB, 0.60331E+00_JPRB, 0.60444E+00_JPRB, 0.60350E+00_JPRB ,&
&  0.60308E+00_JPRB, 0.60383E+00_JPRB, 0.60528E+00_JPRB, 0.60625E+00_JPRB, 0.60450E+00_JPRB, 0.60716E+00_JPRB, 0.61938E+00_JPRB ,&
&  0.63884E+00_JPRB, 0.66075E+00_JPRB, 0.66971E+00_JPRB, 0.66649E+00_JPRB, 0.67455E+00_JPRB, 0.68350E+00_JPRB, 0.68128E+00_JPRB ,&
&  0.70242E+00_JPRB, 0.71693E+00_JPRB, 0.72801E+00_JPRB, 0.75105E+00_JPRB, 0.77144E+00_JPRB, 0.86199E+00_JPRB, 0.74264E+00_JPRB ,&
&  0.69493E+00_JPRB, 0.66803E+00_JPRB, 0.65469E+00_JPRB, 0.64168E+00_JPRB, 0.63702E+00_JPRB /)
ZBURD(:, 49,  7) = (/ &
&  0.62060E+00_JPRB, 0.61238E+00_JPRB, 0.59618E+00_JPRB, 0.57721E+00_JPRB, 0.56303E+00_JPRB, 0.54886E+00_JPRB, 0.53839E+00_JPRB ,&
&  0.53014E+00_JPRB, 0.52293E+00_JPRB, 0.51719E+00_JPRB, 0.51034E+00_JPRB, 0.50138E+00_JPRB, 0.48924E+00_JPRB, 0.47775E+00_JPRB ,&
&  0.46552E+00_JPRB, 0.45356E+00_JPRB, 0.44189E+00_JPRB, 0.43016E+00_JPRB, 0.42108E+00_JPRB, 0.41512E+00_JPRB, 0.41120E+00_JPRB ,&
&  0.41091E+00_JPRB, 0.41441E+00_JPRB, 0.42360E+00_JPRB, 0.44201E+00_JPRB, 0.46362E+00_JPRB, 0.47909E+00_JPRB, 0.49555E+00_JPRB ,&
&  0.56643E+00_JPRB, 0.51090E+00_JPRB, 0.90850E+00_JPRB, 0.10212E+01_JPRB, 0.99357E+00_JPRB, 0.10108E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.12887E+01_JPRB, 0.13274E+01_JPRB, 0.13258E+01_JPRB, 0.12569E+01_JPRB, 0.11264E+01_JPRB, 0.10277E+01_JPRB, 0.95140E+00_JPRB ,&
&  0.89348E+00_JPRB, 0.84464E+00_JPRB, 0.78801E+00_JPRB, 0.73171E+00_JPRB, 0.68938E+00_JPRB, 0.67944E+00_JPRB, 0.69623E+00_JPRB ,&
&  0.76623E+00_JPRB, 0.81982E+00_JPRB, 0.85765E+00_JPRB, 0.91724E+00_JPRB, 0.90787E+00_JPRB, 0.10037E+01_JPRB, 0.11403E+01_JPRB ,&
&  0.15166E+01_JPRB, 0.15952E+01_JPRB, 0.12718E+01_JPRB, 0.98499E+00_JPRB, 0.80231E+00_JPRB, 0.70559E+00_JPRB, 0.63903E+00_JPRB ,&
&  0.60468E+00_JPRB, 0.58636E+00_JPRB, 0.57620E+00_JPRB, 0.57505E+00_JPRB, 0.58280E+00_JPRB, 0.59419E+00_JPRB, 0.60143E+00_JPRB ,&
&  0.60901E+00_JPRB, 0.61435E+00_JPRB, 0.61432E+00_JPRB, 0.61136E+00_JPRB, 0.60206E+00_JPRB, 0.58935E+00_JPRB, 0.58575E+00_JPRB ,&
&  0.59577E+00_JPRB, 0.61725E+00_JPRB, 0.63943E+00_JPRB, 0.63456E+00_JPRB, 0.63984E+00_JPRB, 0.64872E+00_JPRB, 0.65684E+00_JPRB ,&
&  0.67741E+00_JPRB, 0.68481E+00_JPRB, 0.69062E+00_JPRB, 0.72263E+00_JPRB, 0.73482E+00_JPRB, 0.79945E+00_JPRB, 0.74209E+00_JPRB ,&
&  0.69710E+00_JPRB, 0.66503E+00_JPRB, 0.64421E+00_JPRB, 0.62913E+00_JPRB, 0.62164E+00_JPRB /)
ZBURD(:, 50,  7) = (/ &
&  0.61219E+00_JPRB, 0.60052E+00_JPRB, 0.58028E+00_JPRB, 0.55942E+00_JPRB, 0.54533E+00_JPRB, 0.53457E+00_JPRB, 0.52713E+00_JPRB ,&
&  0.52203E+00_JPRB, 0.51265E+00_JPRB, 0.50382E+00_JPRB, 0.49250E+00_JPRB, 0.47888E+00_JPRB, 0.46749E+00_JPRB, 0.46000E+00_JPRB ,&
&  0.45048E+00_JPRB, 0.44495E+00_JPRB, 0.43908E+00_JPRB, 0.42871E+00_JPRB, 0.41758E+00_JPRB, 0.40855E+00_JPRB, 0.40279E+00_JPRB ,&
&  0.40032E+00_JPRB, 0.40296E+00_JPRB, 0.40947E+00_JPRB, 0.42341E+00_JPRB, 0.43663E+00_JPRB, 0.45108E+00_JPRB, 0.47564E+00_JPRB ,&
&  0.56462E+00_JPRB, 0.84571E+00_JPRB, 0.10391E+01_JPRB, 0.92487E+00_JPRB, 0.85372E+00_JPRB, 0.80563E+00_JPRB, 0.91214E+00_JPRB ,&
&  0.10091E+01_JPRB, 0.11088E+01_JPRB, 0.11647E+01_JPRB, 0.11807E+01_JPRB, 0.11142E+01_JPRB, 0.10639E+01_JPRB, 0.10127E+01_JPRB ,&
&  0.95908E+00_JPRB, 0.89907E+00_JPRB, 0.82017E+00_JPRB, 0.74328E+00_JPRB, 0.69874E+00_JPRB, 0.69422E+00_JPRB, 0.72027E+00_JPRB ,&
&  0.77270E+00_JPRB, 0.81853E+00_JPRB, 0.82793E+00_JPRB, 0.87266E+00_JPRB, 0.94016E+00_JPRB, 0.93887E+00_JPRB, 0.11530E+01_JPRB ,&
&  0.14603E+01_JPRB, 0.13386E+01_JPRB, 0.11645E+01_JPRB, 0.97509E+00_JPRB, 0.82889E+00_JPRB, 0.71374E+00_JPRB, 0.64765E+00_JPRB ,&
&  0.60319E+00_JPRB, 0.57464E+00_JPRB, 0.55705E+00_JPRB, 0.55082E+00_JPRB, 0.56225E+00_JPRB, 0.58307E+00_JPRB, 0.60035E+00_JPRB ,&
&  0.62085E+00_JPRB, 0.64002E+00_JPRB, 0.64896E+00_JPRB, 0.64462E+00_JPRB, 0.62817E+00_JPRB, 0.60381E+00_JPRB, 0.58225E+00_JPRB ,&
&  0.57428E+00_JPRB, 0.58135E+00_JPRB, 0.62653E+00_JPRB, 0.60595E+00_JPRB, 0.60680E+00_JPRB, 0.62030E+00_JPRB, 0.62527E+00_JPRB ,&
&  0.62931E+00_JPRB, 0.65581E+00_JPRB, 0.67341E+00_JPRB, 0.68759E+00_JPRB, 0.77135E+00_JPRB, 0.83011E+00_JPRB, 0.77185E+00_JPRB ,&
&  0.71641E+00_JPRB, 0.67315E+00_JPRB, 0.64862E+00_JPRB, 0.62261E+00_JPRB, 0.61379E+00_JPRB /)
ZBURD(:, 51,  7) = (/ &
&  0.61821E+00_JPRB, 0.59748E+00_JPRB, 0.56573E+00_JPRB, 0.53833E+00_JPRB, 0.52062E+00_JPRB, 0.50814E+00_JPRB, 0.50153E+00_JPRB ,&
&  0.49780E+00_JPRB, 0.48838E+00_JPRB, 0.47861E+00_JPRB, 0.46532E+00_JPRB, 0.45407E+00_JPRB, 0.44431E+00_JPRB, 0.43512E+00_JPRB ,&
&  0.42723E+00_JPRB, 0.42246E+00_JPRB, 0.41835E+00_JPRB, 0.41421E+00_JPRB, 0.40806E+00_JPRB, 0.39918E+00_JPRB, 0.39360E+00_JPRB ,&
&  0.39090E+00_JPRB, 0.39146E+00_JPRB, 0.39571E+00_JPRB, 0.40486E+00_JPRB, 0.41519E+00_JPRB, 0.42815E+00_JPRB, 0.44878E+00_JPRB ,&
&  0.50760E+00_JPRB, 0.71681E+00_JPRB, 0.75488E+00_JPRB, 0.74218E+00_JPRB, 0.73955E+00_JPRB, 0.72842E+00_JPRB, 0.76756E+00_JPRB ,&
&  0.84289E+00_JPRB, 0.91172E+00_JPRB, 0.96868E+00_JPRB, 0.98647E+00_JPRB, 0.95257E+00_JPRB, 0.91747E+00_JPRB, 0.87265E+00_JPRB ,&
&  0.82585E+00_JPRB, 0.77567E+00_JPRB, 0.72538E+00_JPRB, 0.67903E+00_JPRB, 0.64150E+00_JPRB, 0.63610E+00_JPRB, 0.65354E+00_JPRB ,&
&  0.68913E+00_JPRB, 0.71811E+00_JPRB, 0.73408E+00_JPRB, 0.77594E+00_JPRB, 0.87771E+00_JPRB, 0.94220E+00_JPRB, 0.10355E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.10222E+01_JPRB, 0.97179E+00_JPRB, 0.90651E+00_JPRB, 0.81311E+00_JPRB, 0.72211E+00_JPRB, 0.65041E+00_JPRB ,&
&  0.59934E+00_JPRB, 0.56565E+00_JPRB, 0.54584E+00_JPRB, 0.53897E+00_JPRB, 0.54632E+00_JPRB, 0.56647E+00_JPRB, 0.59128E+00_JPRB ,&
&  0.61939E+00_JPRB, 0.64641E+00_JPRB, 0.66216E+00_JPRB, 0.66729E+00_JPRB, 0.65941E+00_JPRB, 0.64119E+00_JPRB, 0.61329E+00_JPRB ,&
&  0.58803E+00_JPRB, 0.57209E+00_JPRB, 0.58430E+00_JPRB, 0.58830E+00_JPRB, 0.59030E+00_JPRB, 0.59356E+00_JPRB, 0.59919E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.66438E+00_JPRB, 0.66664E+00_JPRB, 0.68668E+00_JPRB, 0.74674E+00_JPRB, 0.77196E+00_JPRB, 0.73679E+00_JPRB ,&
&  0.70425E+00_JPRB, 0.67028E+00_JPRB, 0.64408E+00_JPRB, 0.61867E+00_JPRB, 0.62415E+00_JPRB /)
ZBURD(:, 52,  7) = (/ &
&  0.66419E+00_JPRB, 0.62299E+00_JPRB, 0.57291E+00_JPRB, 0.53404E+00_JPRB, 0.50636E+00_JPRB, 0.48626E+00_JPRB, 0.47482E+00_JPRB ,&
&  0.47017E+00_JPRB, 0.45824E+00_JPRB, 0.44174E+00_JPRB, 0.42824E+00_JPRB, 0.42148E+00_JPRB, 0.41654E+00_JPRB, 0.40955E+00_JPRB ,&
&  0.40376E+00_JPRB, 0.39857E+00_JPRB, 0.39502E+00_JPRB, 0.39359E+00_JPRB, 0.39278E+00_JPRB, 0.39116E+00_JPRB, 0.38689E+00_JPRB ,&
&  0.38059E+00_JPRB, 0.37876E+00_JPRB, 0.38002E+00_JPRB, 0.38684E+00_JPRB, 0.39658E+00_JPRB, 0.40841E+00_JPRB, 0.42864E+00_JPRB ,&
&  0.49477E+00_JPRB, 0.51575E+00_JPRB, 0.55996E+00_JPRB, 0.57040E+00_JPRB, 0.58503E+00_JPRB, 0.62569E+00_JPRB, 0.66420E+00_JPRB ,&
&  0.70241E+00_JPRB, 0.75320E+00_JPRB, 0.81235E+00_JPRB, 0.83535E+00_JPRB, 0.80920E+00_JPRB, 0.76509E+00_JPRB, 0.72597E+00_JPRB ,&
&  0.68970E+00_JPRB, 0.66432E+00_JPRB, 0.64033E+00_JPRB, 0.61820E+00_JPRB, 0.60399E+00_JPRB, 0.60333E+00_JPRB, 0.61215E+00_JPRB ,&
&  0.61868E+00_JPRB, 0.61299E+00_JPRB, 0.63832E+00_JPRB, 0.67319E+00_JPRB, 0.72932E+00_JPRB, 0.77182E+00_JPRB, 0.81406E+00_JPRB ,&
&  0.84786E+00_JPRB, 0.86067E+00_JPRB, 0.86514E+00_JPRB, 0.85026E+00_JPRB, 0.80067E+00_JPRB, 0.74124E+00_JPRB, 0.68104E+00_JPRB ,&
&  0.62480E+00_JPRB, 0.58832E+00_JPRB, 0.57007E+00_JPRB, 0.56182E+00_JPRB, 0.56139E+00_JPRB, 0.56622E+00_JPRB, 0.57723E+00_JPRB ,&
&  0.59422E+00_JPRB, 0.61035E+00_JPRB, 0.62348E+00_JPRB, 0.62958E+00_JPRB, 0.62793E+00_JPRB, 0.62396E+00_JPRB, 0.61752E+00_JPRB ,&
&  0.60929E+00_JPRB, 0.59882E+00_JPRB, 0.59434E+00_JPRB, 0.59031E+00_JPRB, 0.58696E+00_JPRB, 0.58491E+00_JPRB, 0.58417E+00_JPRB ,&
&  0.58493E+00_JPRB, 0.60921E+00_JPRB, 0.61677E+00_JPRB, 0.72047E+00_JPRB, 0.75813E+00_JPRB, 0.74022E+00_JPRB, 0.71298E+00_JPRB ,&
&  0.67959E+00_JPRB, 0.64755E+00_JPRB, 0.61927E+00_JPRB, 0.59629E+00_JPRB, 0.65369E+00_JPRB /)
ZBURD(:, 53,  7) = (/ &
&  0.56796E+00_JPRB, 0.55670E+00_JPRB, 0.53212E+00_JPRB, 0.50297E+00_JPRB, 0.48109E+00_JPRB, 0.46279E+00_JPRB, 0.44796E+00_JPRB ,&
&  0.43815E+00_JPRB, 0.42661E+00_JPRB, 0.41279E+00_JPRB, 0.40308E+00_JPRB, 0.40139E+00_JPRB, 0.39864E+00_JPRB, 0.39411E+00_JPRB ,&
&  0.38630E+00_JPRB, 0.37895E+00_JPRB, 0.37406E+00_JPRB, 0.37240E+00_JPRB, 0.37290E+00_JPRB, 0.37468E+00_JPRB, 0.37526E+00_JPRB ,&
&  0.37286E+00_JPRB, 0.36858E+00_JPRB, 0.36819E+00_JPRB, 0.37038E+00_JPRB, 0.37575E+00_JPRB, 0.38447E+00_JPRB, 0.40444E+00_JPRB ,&
&  0.45606E+00_JPRB, 0.44153E+00_JPRB, 0.46344E+00_JPRB, 0.47738E+00_JPRB, 0.49687E+00_JPRB, 0.53639E+00_JPRB, 0.57925E+00_JPRB ,&
&  0.60545E+00_JPRB, 0.62592E+00_JPRB, 0.66469E+00_JPRB, 0.69631E+00_JPRB, 0.69385E+00_JPRB, 0.66919E+00_JPRB, 0.63582E+00_JPRB ,&
&  0.60255E+00_JPRB, 0.57782E+00_JPRB, 0.55727E+00_JPRB, 0.54084E+00_JPRB, 0.53046E+00_JPRB, 0.52670E+00_JPRB, 0.52659E+00_JPRB ,&
&  0.52010E+00_JPRB, 0.52133E+00_JPRB, 0.53861E+00_JPRB, 0.56868E+00_JPRB, 0.60443E+00_JPRB, 0.64065E+00_JPRB, 0.68051E+00_JPRB ,&
&  0.71731E+00_JPRB, 0.74391E+00_JPRB, 0.76298E+00_JPRB, 0.77434E+00_JPRB, 0.76347E+00_JPRB, 0.74475E+00_JPRB, 0.70368E+00_JPRB ,&
&  0.65854E+00_JPRB, 0.62165E+00_JPRB, 0.59834E+00_JPRB, 0.58036E+00_JPRB, 0.56784E+00_JPRB, 0.56041E+00_JPRB, 0.55629E+00_JPRB ,&
&  0.55579E+00_JPRB, 0.55853E+00_JPRB, 0.55993E+00_JPRB, 0.55934E+00_JPRB, 0.55799E+00_JPRB, 0.55705E+00_JPRB, 0.55542E+00_JPRB ,&
&  0.55399E+00_JPRB, 0.55277E+00_JPRB, 0.55280E+00_JPRB, 0.55049E+00_JPRB, 0.54666E+00_JPRB, 0.54237E+00_JPRB, 0.53913E+00_JPRB ,&
&  0.53643E+00_JPRB, 0.54932E+00_JPRB, 0.55733E+00_JPRB, 0.61115E+00_JPRB, 0.63420E+00_JPRB, 0.64253E+00_JPRB, 0.64062E+00_JPRB ,&
&  0.62339E+00_JPRB, 0.60244E+00_JPRB, 0.57209E+00_JPRB, 0.54722E+00_JPRB, 0.54829E+00_JPRB /)
ZBURD(:, 54,  7) = (/ &
&  0.50987E+00_JPRB, 0.50601E+00_JPRB, 0.49648E+00_JPRB, 0.47572E+00_JPRB, 0.45520E+00_JPRB, 0.44260E+00_JPRB, 0.42940E+00_JPRB ,&
&  0.41603E+00_JPRB, 0.40532E+00_JPRB, 0.39176E+00_JPRB, 0.38552E+00_JPRB, 0.38288E+00_JPRB, 0.38176E+00_JPRB, 0.37934E+00_JPRB ,&
&  0.37398E+00_JPRB, 0.36603E+00_JPRB, 0.36015E+00_JPRB, 0.35776E+00_JPRB, 0.35825E+00_JPRB, 0.35966E+00_JPRB, 0.36101E+00_JPRB ,&
&  0.35977E+00_JPRB, 0.35882E+00_JPRB, 0.35745E+00_JPRB, 0.35659E+00_JPRB, 0.35772E+00_JPRB, 0.36545E+00_JPRB, 0.38124E+00_JPRB ,&
&  0.42368E+00_JPRB, 0.38983E+00_JPRB, 0.40161E+00_JPRB, 0.42598E+00_JPRB, 0.45170E+00_JPRB, 0.48772E+00_JPRB, 0.52363E+00_JPRB ,&
&  0.54363E+00_JPRB, 0.54400E+00_JPRB, 0.55575E+00_JPRB, 0.57861E+00_JPRB, 0.57943E+00_JPRB, 0.57271E+00_JPRB, 0.55776E+00_JPRB ,&
&  0.53504E+00_JPRB, 0.50853E+00_JPRB, 0.48267E+00_JPRB, 0.46518E+00_JPRB, 0.45805E+00_JPRB, 0.45501E+00_JPRB, 0.45284E+00_JPRB ,&
&  0.44448E+00_JPRB, 0.44123E+00_JPRB, 0.44993E+00_JPRB, 0.46269E+00_JPRB, 0.47929E+00_JPRB, 0.50781E+00_JPRB, 0.54855E+00_JPRB ,&
&  0.57953E+00_JPRB, 0.61327E+00_JPRB, 0.64517E+00_JPRB, 0.66911E+00_JPRB, 0.68776E+00_JPRB, 0.68878E+00_JPRB, 0.66809E+00_JPRB ,&
&  0.63988E+00_JPRB, 0.60923E+00_JPRB, 0.58099E+00_JPRB, 0.55602E+00_JPRB, 0.53649E+00_JPRB, 0.52520E+00_JPRB, 0.51864E+00_JPRB ,&
&  0.51398E+00_JPRB, 0.51043E+00_JPRB, 0.50371E+00_JPRB, 0.49429E+00_JPRB, 0.48491E+00_JPRB, 0.47747E+00_JPRB, 0.47074E+00_JPRB ,&
&  0.46465E+00_JPRB, 0.45840E+00_JPRB, 0.45556E+00_JPRB, 0.45763E+00_JPRB, 0.45922E+00_JPRB, 0.46242E+00_JPRB, 0.46719E+00_JPRB ,&
&  0.47026E+00_JPRB, 0.48201E+00_JPRB, 0.48481E+00_JPRB, 0.58444E+00_JPRB, 0.60198E+00_JPRB, 0.59786E+00_JPRB, 0.58765E+00_JPRB ,&
&  0.56951E+00_JPRB, 0.55145E+00_JPRB, 0.53519E+00_JPRB, 0.48784E+00_JPRB, 0.51254E+00_JPRB /)
ZBURD(:, 55,  7) = (/ &
&  0.46288E+00_JPRB, 0.46089E+00_JPRB, 0.45323E+00_JPRB, 0.44081E+00_JPRB, 0.42632E+00_JPRB, 0.41626E+00_JPRB, 0.40357E+00_JPRB ,&
&  0.39069E+00_JPRB, 0.38332E+00_JPRB, 0.37516E+00_JPRB, 0.37058E+00_JPRB, 0.37005E+00_JPRB, 0.36607E+00_JPRB, 0.36253E+00_JPRB ,&
&  0.35934E+00_JPRB, 0.35382E+00_JPRB, 0.35035E+00_JPRB, 0.34739E+00_JPRB, 0.34696E+00_JPRB, 0.34703E+00_JPRB, 0.34667E+00_JPRB ,&
&  0.34493E+00_JPRB, 0.34185E+00_JPRB, 0.33914E+00_JPRB, 0.33724E+00_JPRB, 0.33886E+00_JPRB, 0.34545E+00_JPRB, 0.35867E+00_JPRB ,&
&  0.38951E+00_JPRB, 0.35152E+00_JPRB, 0.36614E+00_JPRB, 0.38386E+00_JPRB, 0.41227E+00_JPRB, 0.43475E+00_JPRB, 0.44528E+00_JPRB ,&
&  0.45738E+00_JPRB, 0.47032E+00_JPRB, 0.48022E+00_JPRB, 0.48968E+00_JPRB, 0.48658E+00_JPRB, 0.47868E+00_JPRB, 0.47178E+00_JPRB ,&
&  0.46185E+00_JPRB, 0.44941E+00_JPRB, 0.43669E+00_JPRB, 0.42343E+00_JPRB, 0.41541E+00_JPRB, 0.41078E+00_JPRB, 0.40496E+00_JPRB ,&
&  0.39651E+00_JPRB, 0.38827E+00_JPRB, 0.38300E+00_JPRB, 0.38293E+00_JPRB, 0.38940E+00_JPRB, 0.40269E+00_JPRB, 0.42817E+00_JPRB ,&
&  0.45439E+00_JPRB, 0.48369E+00_JPRB, 0.51485E+00_JPRB, 0.53981E+00_JPRB, 0.55789E+00_JPRB, 0.56420E+00_JPRB, 0.56255E+00_JPRB ,&
&  0.55312E+00_JPRB, 0.54107E+00_JPRB, 0.53002E+00_JPRB, 0.51597E+00_JPRB, 0.49856E+00_JPRB, 0.48296E+00_JPRB, 0.47077E+00_JPRB ,&
&  0.46156E+00_JPRB, 0.45181E+00_JPRB, 0.44074E+00_JPRB, 0.42893E+00_JPRB, 0.41672E+00_JPRB, 0.40604E+00_JPRB, 0.39700E+00_JPRB ,&
&  0.38921E+00_JPRB, 0.38228E+00_JPRB, 0.37879E+00_JPRB, 0.38103E+00_JPRB, 0.38617E+00_JPRB, 0.39194E+00_JPRB, 0.39762E+00_JPRB ,&
&  0.40476E+00_JPRB, 0.41728E+00_JPRB, 0.42477E+00_JPRB, 0.46245E+00_JPRB, 0.48267E+00_JPRB, 0.49202E+00_JPRB, 0.49809E+00_JPRB ,&
&  0.49869E+00_JPRB, 0.49323E+00_JPRB, 0.42932E+00_JPRB, 0.46124E+00_JPRB, 0.46511E+00_JPRB /)
ZBURD(:, 56,  7) = (/ &
&  0.42682E+00_JPRB, 0.41656E+00_JPRB, 0.40643E+00_JPRB, 0.40352E+00_JPRB, 0.39834E+00_JPRB, 0.39140E+00_JPRB, 0.38267E+00_JPRB ,&
&  0.37038E+00_JPRB, 0.36152E+00_JPRB, 0.35504E+00_JPRB, 0.35282E+00_JPRB, 0.35493E+00_JPRB, 0.35271E+00_JPRB, 0.34792E+00_JPRB ,&
&  0.34437E+00_JPRB, 0.34031E+00_JPRB, 0.33744E+00_JPRB, 0.33406E+00_JPRB, 0.33134E+00_JPRB, 0.33018E+00_JPRB, 0.32945E+00_JPRB ,&
&  0.32756E+00_JPRB, 0.32427E+00_JPRB, 0.32141E+00_JPRB, 0.31988E+00_JPRB, 0.32206E+00_JPRB, 0.32799E+00_JPRB, 0.33749E+00_JPRB ,&
&  0.33514E+00_JPRB, 0.31753E+00_JPRB, 0.33544E+00_JPRB, 0.35777E+00_JPRB, 0.37244E+00_JPRB, 0.38679E+00_JPRB, 0.39783E+00_JPRB ,&
&  0.40779E+00_JPRB, 0.41109E+00_JPRB, 0.41455E+00_JPRB, 0.41738E+00_JPRB, 0.41773E+00_JPRB, 0.41392E+00_JPRB, 0.40939E+00_JPRB ,&
&  0.40579E+00_JPRB, 0.40278E+00_JPRB, 0.39920E+00_JPRB, 0.39268E+00_JPRB, 0.38556E+00_JPRB, 0.37785E+00_JPRB, 0.37039E+00_JPRB ,&
&  0.36410E+00_JPRB, 0.35808E+00_JPRB, 0.35364E+00_JPRB, 0.35176E+00_JPRB, 0.35310E+00_JPRB, 0.35643E+00_JPRB, 0.36285E+00_JPRB ,&
&  0.37072E+00_JPRB, 0.38350E+00_JPRB, 0.40479E+00_JPRB, 0.42476E+00_JPRB, 0.43998E+00_JPRB, 0.44992E+00_JPRB, 0.45733E+00_JPRB ,&
&  0.45828E+00_JPRB, 0.45609E+00_JPRB, 0.45341E+00_JPRB, 0.44851E+00_JPRB, 0.44086E+00_JPRB, 0.43059E+00_JPRB, 0.41835E+00_JPRB ,&
&  0.40774E+00_JPRB, 0.39865E+00_JPRB, 0.39000E+00_JPRB, 0.38178E+00_JPRB, 0.37365E+00_JPRB, 0.36582E+00_JPRB, 0.35906E+00_JPRB ,&
&  0.35285E+00_JPRB, 0.34692E+00_JPRB, 0.34284E+00_JPRB, 0.34246E+00_JPRB, 0.34600E+00_JPRB, 0.35081E+00_JPRB, 0.35653E+00_JPRB ,&
&  0.36267E+00_JPRB, 0.37332E+00_JPRB, 0.38275E+00_JPRB, 0.40211E+00_JPRB, 0.42160E+00_JPRB, 0.43048E+00_JPRB, 0.43817E+00_JPRB ,&
&  0.44495E+00_JPRB, 0.44438E+00_JPRB, 0.44528E+00_JPRB, 0.44012E+00_JPRB, 0.43208E+00_JPRB /)
ZBURD(:, 57,  7) = (/ &
&  0.38668E+00_JPRB, 0.37541E+00_JPRB, 0.37027E+00_JPRB, 0.37101E+00_JPRB, 0.37314E+00_JPRB, 0.37078E+00_JPRB, 0.36492E+00_JPRB ,&
&  0.35406E+00_JPRB, 0.34427E+00_JPRB, 0.33625E+00_JPRB, 0.33225E+00_JPRB, 0.33336E+00_JPRB, 0.33318E+00_JPRB, 0.33027E+00_JPRB ,&
&  0.32723E+00_JPRB, 0.32202E+00_JPRB, 0.31770E+00_JPRB, 0.31341E+00_JPRB, 0.31013E+00_JPRB, 0.30863E+00_JPRB, 0.30789E+00_JPRB ,&
&  0.30606E+00_JPRB, 0.30379E+00_JPRB, 0.30298E+00_JPRB, 0.30263E+00_JPRB, 0.30411E+00_JPRB, 0.30814E+00_JPRB, 0.31332E+00_JPRB ,&
&  0.32259E+00_JPRB, 0.30777E+00_JPRB, 0.32552E+00_JPRB, 0.33038E+00_JPRB, 0.33530E+00_JPRB, 0.34331E+00_JPRB, 0.35305E+00_JPRB ,&
&  0.36186E+00_JPRB, 0.36666E+00_JPRB, 0.36913E+00_JPRB, 0.36984E+00_JPRB, 0.36852E+00_JPRB, 0.36771E+00_JPRB, 0.36447E+00_JPRB ,&
&  0.35964E+00_JPRB, 0.35759E+00_JPRB, 0.35657E+00_JPRB, 0.35435E+00_JPRB, 0.35196E+00_JPRB, 0.34822E+00_JPRB, 0.34403E+00_JPRB ,&
&  0.34002E+00_JPRB, 0.33595E+00_JPRB, 0.33254E+00_JPRB, 0.32927E+00_JPRB, 0.32683E+00_JPRB, 0.32588E+00_JPRB, 0.32665E+00_JPRB ,&
&  0.32802E+00_JPRB, 0.33106E+00_JPRB, 0.33861E+00_JPRB, 0.34861E+00_JPRB, 0.35845E+00_JPRB, 0.36672E+00_JPRB, 0.37343E+00_JPRB ,&
&  0.37742E+00_JPRB, 0.37939E+00_JPRB, 0.37910E+00_JPRB, 0.37748E+00_JPRB, 0.37409E+00_JPRB, 0.37002E+00_JPRB, 0.36412E+00_JPRB ,&
&  0.35871E+00_JPRB, 0.35395E+00_JPRB, 0.34885E+00_JPRB, 0.34427E+00_JPRB, 0.33916E+00_JPRB, 0.33339E+00_JPRB, 0.32806E+00_JPRB ,&
&  0.32391E+00_JPRB, 0.32097E+00_JPRB, 0.31879E+00_JPRB, 0.31790E+00_JPRB, 0.31847E+00_JPRB, 0.32112E+00_JPRB, 0.32404E+00_JPRB ,&
&  0.32776E+00_JPRB, 0.33427E+00_JPRB, 0.34204E+00_JPRB, 0.35255E+00_JPRB, 0.36732E+00_JPRB, 0.37864E+00_JPRB, 0.38787E+00_JPRB ,&
&  0.39501E+00_JPRB, 0.39745E+00_JPRB, 0.40011E+00_JPRB, 0.39863E+00_JPRB, 0.39447E+00_JPRB /)
ZBURD(:, 58,  7) = (/ &
&  0.34670E+00_JPRB, 0.34004E+00_JPRB, 0.34024E+00_JPRB, 0.34441E+00_JPRB, 0.35081E+00_JPRB, 0.34911E+00_JPRB, 0.34599E+00_JPRB ,&
&  0.33969E+00_JPRB, 0.33171E+00_JPRB, 0.32386E+00_JPRB, 0.31651E+00_JPRB, 0.31303E+00_JPRB, 0.31302E+00_JPRB, 0.31255E+00_JPRB ,&
&  0.31098E+00_JPRB, 0.30729E+00_JPRB, 0.30216E+00_JPRB, 0.29714E+00_JPRB, 0.29343E+00_JPRB, 0.29120E+00_JPRB, 0.28917E+00_JPRB ,&
&  0.28762E+00_JPRB, 0.28558E+00_JPRB, 0.28587E+00_JPRB, 0.28547E+00_JPRB, 0.28595E+00_JPRB, 0.28944E+00_JPRB, 0.29251E+00_JPRB ,&
&  0.29412E+00_JPRB, 0.29980E+00_JPRB, 0.29828E+00_JPRB, 0.29803E+00_JPRB, 0.30127E+00_JPRB, 0.30903E+00_JPRB, 0.31912E+00_JPRB ,&
&  0.32822E+00_JPRB, 0.33394E+00_JPRB, 0.33624E+00_JPRB, 0.33484E+00_JPRB, 0.33220E+00_JPRB, 0.33009E+00_JPRB, 0.32899E+00_JPRB ,&
&  0.32745E+00_JPRB, 0.32382E+00_JPRB, 0.31962E+00_JPRB, 0.31604E+00_JPRB, 0.31447E+00_JPRB, 0.31331E+00_JPRB, 0.31169E+00_JPRB ,&
&  0.30951E+00_JPRB, 0.30711E+00_JPRB, 0.30366E+00_JPRB, 0.29985E+00_JPRB, 0.29640E+00_JPRB, 0.29452E+00_JPRB, 0.29456E+00_JPRB ,&
&  0.29621E+00_JPRB, 0.29901E+00_JPRB, 0.30293E+00_JPRB, 0.30795E+00_JPRB, 0.31252E+00_JPRB, 0.31721E+00_JPRB, 0.32053E+00_JPRB ,&
&  0.32270E+00_JPRB, 0.32371E+00_JPRB, 0.32386E+00_JPRB, 0.32347E+00_JPRB, 0.32238E+00_JPRB, 0.32061E+00_JPRB, 0.31854E+00_JPRB ,&
&  0.31657E+00_JPRB, 0.31470E+00_JPRB, 0.31282E+00_JPRB, 0.31080E+00_JPRB, 0.30765E+00_JPRB, 0.30413E+00_JPRB, 0.30155E+00_JPRB ,&
&  0.30004E+00_JPRB, 0.29919E+00_JPRB, 0.29857E+00_JPRB, 0.29776E+00_JPRB, 0.29759E+00_JPRB, 0.29865E+00_JPRB, 0.30078E+00_JPRB ,&
&  0.30389E+00_JPRB, 0.30824E+00_JPRB, 0.31383E+00_JPRB, 0.32009E+00_JPRB, 0.32778E+00_JPRB, 0.33640E+00_JPRB, 0.34487E+00_JPRB ,&
&  0.35134E+00_JPRB, 0.35369E+00_JPRB, 0.35555E+00_JPRB, 0.35658E+00_JPRB, 0.35342E+00_JPRB /)
ZBURD(:, 59,  7) = (/ &
&  0.31300E+00_JPRB, 0.31198E+00_JPRB, 0.31603E+00_JPRB, 0.32116E+00_JPRB, 0.32730E+00_JPRB, 0.32886E+00_JPRB, 0.32625E+00_JPRB ,&
&  0.32307E+00_JPRB, 0.31891E+00_JPRB, 0.31278E+00_JPRB, 0.30519E+00_JPRB, 0.29829E+00_JPRB, 0.29618E+00_JPRB, 0.29613E+00_JPRB ,&
&  0.29629E+00_JPRB, 0.29443E+00_JPRB, 0.29071E+00_JPRB, 0.28645E+00_JPRB, 0.28143E+00_JPRB, 0.27675E+00_JPRB, 0.27296E+00_JPRB ,&
&  0.27128E+00_JPRB, 0.27016E+00_JPRB, 0.27021E+00_JPRB, 0.27021E+00_JPRB, 0.27080E+00_JPRB, 0.27334E+00_JPRB, 0.27863E+00_JPRB ,&
&  0.27639E+00_JPRB, 0.27830E+00_JPRB, 0.27078E+00_JPRB, 0.27319E+00_JPRB, 0.27572E+00_JPRB, 0.28050E+00_JPRB, 0.28628E+00_JPRB ,&
&  0.29236E+00_JPRB, 0.29748E+00_JPRB, 0.30063E+00_JPRB, 0.30085E+00_JPRB, 0.29943E+00_JPRB, 0.29944E+00_JPRB, 0.29957E+00_JPRB ,&
&  0.29905E+00_JPRB, 0.29816E+00_JPRB, 0.29619E+00_JPRB, 0.29220E+00_JPRB, 0.28671E+00_JPRB, 0.28193E+00_JPRB, 0.27852E+00_JPRB ,&
&  0.27762E+00_JPRB, 0.27706E+00_JPRB, 0.27546E+00_JPRB, 0.27374E+00_JPRB, 0.27197E+00_JPRB, 0.27093E+00_JPRB, 0.27143E+00_JPRB ,&
&  0.27272E+00_JPRB, 0.27386E+00_JPRB, 0.27469E+00_JPRB, 0.27631E+00_JPRB, 0.27912E+00_JPRB, 0.28191E+00_JPRB, 0.28425E+00_JPRB ,&
&  0.28562E+00_JPRB, 0.28563E+00_JPRB, 0.28526E+00_JPRB, 0.28460E+00_JPRB, 0.28418E+00_JPRB, 0.28367E+00_JPRB, 0.28366E+00_JPRB ,&
&  0.28366E+00_JPRB, 0.28327E+00_JPRB, 0.28214E+00_JPRB, 0.28037E+00_JPRB, 0.27870E+00_JPRB, 0.27730E+00_JPRB, 0.27665E+00_JPRB ,&
&  0.27671E+00_JPRB, 0.27695E+00_JPRB, 0.27673E+00_JPRB, 0.27623E+00_JPRB, 0.27652E+00_JPRB, 0.27729E+00_JPRB, 0.27890E+00_JPRB ,&
&  0.28114E+00_JPRB, 0.28387E+00_JPRB, 0.28764E+00_JPRB, 0.29159E+00_JPRB, 0.29367E+00_JPRB, 0.29715E+00_JPRB, 0.30317E+00_JPRB ,&
&  0.30890E+00_JPRB, 0.31216E+00_JPRB, 0.31423E+00_JPRB, 0.31565E+00_JPRB, 0.31571E+00_JPRB /)
ZBURD(:, 60,  7) = (/ &
&  0.28788E+00_JPRB, 0.29042E+00_JPRB, 0.29454E+00_JPRB, 0.30025E+00_JPRB, 0.30446E+00_JPRB, 0.30860E+00_JPRB, 0.30865E+00_JPRB ,&
&  0.30627E+00_JPRB, 0.30268E+00_JPRB, 0.29822E+00_JPRB, 0.29269E+00_JPRB, 0.28693E+00_JPRB, 0.28261E+00_JPRB, 0.28098E+00_JPRB ,&
&  0.28070E+00_JPRB, 0.27946E+00_JPRB, 0.27650E+00_JPRB, 0.27239E+00_JPRB, 0.26765E+00_JPRB, 0.26348E+00_JPRB, 0.25997E+00_JPRB ,&
&  0.25841E+00_JPRB, 0.25829E+00_JPRB, 0.25889E+00_JPRB, 0.25910E+00_JPRB, 0.25905E+00_JPRB, 0.26112E+00_JPRB, 0.26490E+00_JPRB ,&
&  0.26275E+00_JPRB, 0.26028E+00_JPRB, 0.25447E+00_JPRB, 0.25383E+00_JPRB, 0.25521E+00_JPRB, 0.25761E+00_JPRB, 0.26022E+00_JPRB ,&
&  0.26271E+00_JPRB, 0.26575E+00_JPRB, 0.26895E+00_JPRB, 0.27141E+00_JPRB, 0.27304E+00_JPRB, 0.27454E+00_JPRB, 0.27551E+00_JPRB ,&
&  0.27586E+00_JPRB, 0.27486E+00_JPRB, 0.27292E+00_JPRB, 0.26937E+00_JPRB, 0.26480E+00_JPRB, 0.26163E+00_JPRB, 0.26005E+00_JPRB ,&
&  0.25846E+00_JPRB, 0.25814E+00_JPRB, 0.25796E+00_JPRB, 0.25722E+00_JPRB, 0.25615E+00_JPRB, 0.25582E+00_JPRB, 0.25636E+00_JPRB ,&
&  0.25672E+00_JPRB, 0.25623E+00_JPRB, 0.25513E+00_JPRB, 0.25484E+00_JPRB, 0.25555E+00_JPRB, 0.25708E+00_JPRB, 0.25849E+00_JPRB ,&
&  0.25953E+00_JPRB, 0.25936E+00_JPRB, 0.25869E+00_JPRB, 0.25704E+00_JPRB, 0.25587E+00_JPRB, 0.25608E+00_JPRB, 0.25731E+00_JPRB ,&
&  0.25809E+00_JPRB, 0.25806E+00_JPRB, 0.25817E+00_JPRB, 0.25703E+00_JPRB, 0.25594E+00_JPRB, 0.25587E+00_JPRB, 0.25618E+00_JPRB ,&
&  0.25617E+00_JPRB, 0.25589E+00_JPRB, 0.25551E+00_JPRB, 0.25534E+00_JPRB, 0.25581E+00_JPRB, 0.25599E+00_JPRB, 0.25704E+00_JPRB ,&
&  0.25967E+00_JPRB, 0.26261E+00_JPRB, 0.26546E+00_JPRB, 0.26746E+00_JPRB, 0.26872E+00_JPRB, 0.26999E+00_JPRB, 0.27402E+00_JPRB ,&
&  0.27917E+00_JPRB, 0.28305E+00_JPRB, 0.28418E+00_JPRB, 0.28445E+00_JPRB, 0.28607E+00_JPRB /)
ZBURD(:, 61,  7) = (/ &
&  0.26815E+00_JPRB, 0.27137E+00_JPRB, 0.27640E+00_JPRB, 0.28109E+00_JPRB, 0.28396E+00_JPRB, 0.28890E+00_JPRB, 0.29024E+00_JPRB ,&
&  0.28983E+00_JPRB, 0.28813E+00_JPRB, 0.28395E+00_JPRB, 0.27880E+00_JPRB, 0.27391E+00_JPRB, 0.26983E+00_JPRB, 0.26693E+00_JPRB ,&
&  0.26515E+00_JPRB, 0.26355E+00_JPRB, 0.26164E+00_JPRB, 0.25867E+00_JPRB, 0.25497E+00_JPRB, 0.25142E+00_JPRB, 0.24849E+00_JPRB ,&
&  0.24715E+00_JPRB, 0.24676E+00_JPRB, 0.24685E+00_JPRB, 0.24647E+00_JPRB, 0.24602E+00_JPRB, 0.24698E+00_JPRB, 0.24847E+00_JPRB ,&
&  0.24730E+00_JPRB, 0.24295E+00_JPRB, 0.24011E+00_JPRB, 0.23979E+00_JPRB, 0.23996E+00_JPRB, 0.24058E+00_JPRB, 0.24147E+00_JPRB ,&
&  0.24194E+00_JPRB, 0.24279E+00_JPRB, 0.24436E+00_JPRB, 0.24610E+00_JPRB, 0.24746E+00_JPRB, 0.25005E+00_JPRB, 0.25140E+00_JPRB ,&
&  0.25159E+00_JPRB, 0.25088E+00_JPRB, 0.24950E+00_JPRB, 0.24690E+00_JPRB, 0.24504E+00_JPRB, 0.24442E+00_JPRB, 0.24367E+00_JPRB ,&
&  0.24426E+00_JPRB, 0.24487E+00_JPRB, 0.24544E+00_JPRB, 0.24451E+00_JPRB, 0.24273E+00_JPRB, 0.24245E+00_JPRB, 0.24290E+00_JPRB ,&
&  0.24284E+00_JPRB, 0.24164E+00_JPRB, 0.23971E+00_JPRB, 0.23848E+00_JPRB, 0.23742E+00_JPRB, 0.23719E+00_JPRB, 0.23708E+00_JPRB ,&
&  0.23686E+00_JPRB, 0.23586E+00_JPRB, 0.23525E+00_JPRB, 0.23508E+00_JPRB, 0.23506E+00_JPRB, 0.23554E+00_JPRB, 0.23534E+00_JPRB ,&
&  0.23553E+00_JPRB, 0.23579E+00_JPRB, 0.23479E+00_JPRB, 0.23437E+00_JPRB, 0.23415E+00_JPRB, 0.23434E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.23393E+00_JPRB, 0.23344E+00_JPRB, 0.23314E+00_JPRB, 0.23329E+00_JPRB, 0.23294E+00_JPRB, 0.23230E+00_JPRB, 0.23313E+00_JPRB ,&
&  0.23633E+00_JPRB, 0.23972E+00_JPRB, 0.24262E+00_JPRB, 0.24452E+00_JPRB, 0.24522E+00_JPRB, 0.24689E+00_JPRB, 0.25030E+00_JPRB ,&
&  0.25372E+00_JPRB, 0.25601E+00_JPRB, 0.25930E+00_JPRB, 0.26178E+00_JPRB, 0.26433E+00_JPRB /)
ZBURD(:, 62,  7) = (/ &
&  0.25089E+00_JPRB, 0.25519E+00_JPRB, 0.25897E+00_JPRB, 0.26299E+00_JPRB, 0.26536E+00_JPRB, 0.27003E+00_JPRB, 0.27237E+00_JPRB ,&
&  0.27264E+00_JPRB, 0.27243E+00_JPRB, 0.27086E+00_JPRB, 0.26813E+00_JPRB, 0.26396E+00_JPRB, 0.25969E+00_JPRB, 0.25563E+00_JPRB ,&
&  0.25243E+00_JPRB, 0.24974E+00_JPRB, 0.24749E+00_JPRB, 0.24536E+00_JPRB, 0.24251E+00_JPRB, 0.23983E+00_JPRB, 0.23770E+00_JPRB ,&
&  0.23667E+00_JPRB, 0.23587E+00_JPRB, 0.23540E+00_JPRB, 0.23507E+00_JPRB, 0.23455E+00_JPRB, 0.23398E+00_JPRB, 0.23381E+00_JPRB ,&
&  0.23253E+00_JPRB, 0.22897E+00_JPRB, 0.22695E+00_JPRB, 0.22630E+00_JPRB, 0.22580E+00_JPRB, 0.22632E+00_JPRB, 0.22612E+00_JPRB ,&
&  0.22602E+00_JPRB, 0.22666E+00_JPRB, 0.22824E+00_JPRB, 0.23007E+00_JPRB, 0.23034E+00_JPRB, 0.23106E+00_JPRB, 0.23238E+00_JPRB ,&
&  0.23291E+00_JPRB, 0.23225E+00_JPRB, 0.23047E+00_JPRB, 0.22992E+00_JPRB, 0.23055E+00_JPRB, 0.23055E+00_JPRB, 0.23086E+00_JPRB ,&
&  0.23145E+00_JPRB, 0.23250E+00_JPRB, 0.23301E+00_JPRB, 0.23139E+00_JPRB, 0.22979E+00_JPRB, 0.22919E+00_JPRB, 0.22917E+00_JPRB ,&
&  0.22883E+00_JPRB, 0.22781E+00_JPRB, 0.22569E+00_JPRB, 0.22373E+00_JPRB, 0.22089E+00_JPRB, 0.21859E+00_JPRB, 0.21740E+00_JPRB ,&
&  0.21784E+00_JPRB, 0.21831E+00_JPRB, 0.21963E+00_JPRB, 0.22082E+00_JPRB, 0.22158E+00_JPRB, 0.22169E+00_JPRB, 0.22013E+00_JPRB ,&
&  0.21806E+00_JPRB, 0.21708E+00_JPRB, 0.21604E+00_JPRB, 0.21505E+00_JPRB, 0.21521E+00_JPRB, 0.21505E+00_JPRB, 0.21463E+00_JPRB ,&
&  0.21416E+00_JPRB, 0.21458E+00_JPRB, 0.21509E+00_JPRB, 0.21506E+00_JPRB, 0.21462E+00_JPRB, 0.21389E+00_JPRB, 0.21448E+00_JPRB ,&
&  0.21597E+00_JPRB, 0.21804E+00_JPRB, 0.22041E+00_JPRB, 0.22296E+00_JPRB, 0.22567E+00_JPRB, 0.22825E+00_JPRB, 0.23103E+00_JPRB ,&
&  0.23455E+00_JPRB, 0.23628E+00_JPRB, 0.23891E+00_JPRB, 0.24259E+00_JPRB, 0.24614E+00_JPRB /)
ZBURD(:, 63,  7) = (/ &
&  0.23171E+00_JPRB, 0.23714E+00_JPRB, 0.24079E+00_JPRB, 0.24426E+00_JPRB, 0.24710E+00_JPRB, 0.25156E+00_JPRB, 0.25465E+00_JPRB ,&
&  0.25528E+00_JPRB, 0.25536E+00_JPRB, 0.25494E+00_JPRB, 0.25376E+00_JPRB, 0.25183E+00_JPRB, 0.24996E+00_JPRB, 0.24692E+00_JPRB ,&
&  0.24318E+00_JPRB, 0.23970E+00_JPRB, 0.23654E+00_JPRB, 0.23356E+00_JPRB, 0.23080E+00_JPRB, 0.22828E+00_JPRB, 0.22630E+00_JPRB ,&
&  0.22567E+00_JPRB, 0.22541E+00_JPRB, 0.22441E+00_JPRB, 0.22402E+00_JPRB, 0.22391E+00_JPRB, 0.22304E+00_JPRB, 0.22155E+00_JPRB ,&
&  0.21974E+00_JPRB, 0.21668E+00_JPRB, 0.21360E+00_JPRB, 0.21257E+00_JPRB, 0.21261E+00_JPRB, 0.21189E+00_JPRB, 0.21085E+00_JPRB ,&
&  0.21215E+00_JPRB, 0.21430E+00_JPRB, 0.21667E+00_JPRB, 0.21789E+00_JPRB, 0.21871E+00_JPRB, 0.21995E+00_JPRB, 0.22147E+00_JPRB ,&
&  0.22172E+00_JPRB, 0.21922E+00_JPRB, 0.21697E+00_JPRB, 0.21651E+00_JPRB, 0.21642E+00_JPRB, 0.21603E+00_JPRB, 0.21632E+00_JPRB ,&
&  0.21691E+00_JPRB, 0.21829E+00_JPRB, 0.21921E+00_JPRB, 0.21821E+00_JPRB, 0.21642E+00_JPRB, 0.21511E+00_JPRB, 0.21428E+00_JPRB ,&
&  0.21384E+00_JPRB, 0.21330E+00_JPRB, 0.21234E+00_JPRB, 0.21082E+00_JPRB, 0.20910E+00_JPRB, 0.20746E+00_JPRB, 0.20577E+00_JPRB ,&
&  0.20433E+00_JPRB, 0.20446E+00_JPRB, 0.20669E+00_JPRB, 0.20821E+00_JPRB, 0.20801E+00_JPRB, 0.20670E+00_JPRB, 0.20482E+00_JPRB ,&
&  0.20349E+00_JPRB, 0.20114E+00_JPRB, 0.19920E+00_JPRB, 0.19770E+00_JPRB, 0.19658E+00_JPRB, 0.19508E+00_JPRB, 0.19453E+00_JPRB ,&
&  0.19491E+00_JPRB, 0.19664E+00_JPRB, 0.19733E+00_JPRB, 0.19753E+00_JPRB, 0.19723E+00_JPRB, 0.19699E+00_JPRB, 0.19794E+00_JPRB ,&
&  0.19924E+00_JPRB, 0.20126E+00_JPRB, 0.20347E+00_JPRB, 0.20636E+00_JPRB, 0.20911E+00_JPRB, 0.21261E+00_JPRB, 0.21561E+00_JPRB ,&
&  0.21758E+00_JPRB, 0.21840E+00_JPRB, 0.22004E+00_JPRB, 0.22232E+00_JPRB, 0.22594E+00_JPRB /)
ZBURD(:, 64,  7) = (/ &
&  0.21188E+00_JPRB, 0.21763E+00_JPRB, 0.22277E+00_JPRB, 0.22589E+00_JPRB, 0.22817E+00_JPRB, 0.23098E+00_JPRB, 0.23345E+00_JPRB ,&
&  0.23515E+00_JPRB, 0.23613E+00_JPRB, 0.23657E+00_JPRB, 0.23639E+00_JPRB, 0.23536E+00_JPRB, 0.23369E+00_JPRB, 0.23172E+00_JPRB ,&
&  0.22957E+00_JPRB, 0.22664E+00_JPRB, 0.22394E+00_JPRB, 0.22185E+00_JPRB, 0.22010E+00_JPRB, 0.21808E+00_JPRB, 0.21645E+00_JPRB ,&
&  0.21482E+00_JPRB, 0.21429E+00_JPRB, 0.21361E+00_JPRB, 0.21292E+00_JPRB, 0.21233E+00_JPRB, 0.21131E+00_JPRB, 0.21039E+00_JPRB ,&
&  0.20809E+00_JPRB, 0.20432E+00_JPRB, 0.20261E+00_JPRB, 0.20173E+00_JPRB, 0.20186E+00_JPRB, 0.20130E+00_JPRB, 0.20057E+00_JPRB ,&
&  0.20226E+00_JPRB, 0.20520E+00_JPRB, 0.20742E+00_JPRB, 0.20863E+00_JPRB, 0.20946E+00_JPRB, 0.21061E+00_JPRB, 0.21192E+00_JPRB ,&
&  0.20972E+00_JPRB, 0.20695E+00_JPRB, 0.20492E+00_JPRB, 0.20387E+00_JPRB, 0.20371E+00_JPRB, 0.20444E+00_JPRB, 0.20601E+00_JPRB ,&
&  0.20744E+00_JPRB, 0.20825E+00_JPRB, 0.20789E+00_JPRB, 0.20653E+00_JPRB, 0.20487E+00_JPRB, 0.20409E+00_JPRB, 0.20362E+00_JPRB ,&
&  0.20338E+00_JPRB, 0.20303E+00_JPRB, 0.20258E+00_JPRB, 0.20146E+00_JPRB, 0.19591E+00_JPRB, 0.19223E+00_JPRB, 0.18960E+00_JPRB ,&
&  0.19019E+00_JPRB, 0.19014E+00_JPRB, 0.18876E+00_JPRB, 0.19407E+00_JPRB, 0.19974E+00_JPRB, 0.19513E+00_JPRB, 0.18960E+00_JPRB ,&
&  0.18213E+00_JPRB, 0.18282E+00_JPRB, 0.18015E+00_JPRB, 0.17545E+00_JPRB, 0.17410E+00_JPRB, 0.17425E+00_JPRB, 0.17397E+00_JPRB ,&
&  0.17483E+00_JPRB, 0.17506E+00_JPRB, 0.17629E+00_JPRB, 0.17690E+00_JPRB, 0.17725E+00_JPRB, 0.17811E+00_JPRB, 0.17896E+00_JPRB ,&
&  0.18103E+00_JPRB, 0.18475E+00_JPRB, 0.18912E+00_JPRB, 0.19066E+00_JPRB, 0.19346E+00_JPRB, 0.19600E+00_JPRB, 0.19772E+00_JPRB ,&
&  0.19913E+00_JPRB, 0.20110E+00_JPRB, 0.20298E+00_JPRB, 0.20462E+00_JPRB, 0.20734E+00_JPRB /)
ZBURD(:, 65,  7) = (/ &
&  0.19820E+00_JPRB, 0.20069E+00_JPRB, 0.20373E+00_JPRB, 0.20685E+00_JPRB, 0.20928E+00_JPRB, 0.21171E+00_JPRB, 0.21383E+00_JPRB ,&
&  0.21555E+00_JPRB, 0.21664E+00_JPRB, 0.21733E+00_JPRB, 0.21769E+00_JPRB, 0.21791E+00_JPRB, 0.21755E+00_JPRB, 0.21674E+00_JPRB ,&
&  0.21590E+00_JPRB, 0.21449E+00_JPRB, 0.21265E+00_JPRB, 0.21111E+00_JPRB, 0.21002E+00_JPRB, 0.20869E+00_JPRB, 0.20687E+00_JPRB ,&
&  0.20506E+00_JPRB, 0.20355E+00_JPRB, 0.20243E+00_JPRB, 0.20131E+00_JPRB, 0.20125E+00_JPRB, 0.20181E+00_JPRB, 0.20224E+00_JPRB ,&
&  0.19797E+00_JPRB, 0.19269E+00_JPRB, 0.19358E+00_JPRB, 0.19459E+00_JPRB, 0.19267E+00_JPRB, 0.19397E+00_JPRB, 0.19477E+00_JPRB ,&
&  0.19589E+00_JPRB, 0.19738E+00_JPRB, 0.19884E+00_JPRB, 0.19960E+00_JPRB, 0.20012E+00_JPRB, 0.20088E+00_JPRB, 0.20030E+00_JPRB ,&
&  0.19827E+00_JPRB, 0.19649E+00_JPRB, 0.19488E+00_JPRB, 0.19323E+00_JPRB, 0.19277E+00_JPRB, 0.19426E+00_JPRB, 0.19516E+00_JPRB ,&
&  0.19421E+00_JPRB, 0.19277E+00_JPRB, 0.19142E+00_JPRB, 0.19167E+00_JPRB, 0.19240E+00_JPRB, 0.19275E+00_JPRB, 0.19068E+00_JPRB ,&
&  0.18948E+00_JPRB, 0.18673E+00_JPRB, 0.18416E+00_JPRB, 0.18164E+00_JPRB, 0.18039E+00_JPRB, 0.17694E+00_JPRB, 0.17325E+00_JPRB ,&
&  0.17248E+00_JPRB, 0.17638E+00_JPRB, 0.18236E+00_JPRB, 0.18441E+00_JPRB, 0.17395E+00_JPRB, 0.17438E+00_JPRB, 0.17446E+00_JPRB ,&
&  0.16991E+00_JPRB, 0.16479E+00_JPRB, 0.16112E+00_JPRB, 0.15955E+00_JPRB, 0.15899E+00_JPRB, 0.15853E+00_JPRB, 0.15910E+00_JPRB ,&
&  0.16014E+00_JPRB, 0.16074E+00_JPRB, 0.16139E+00_JPRB, 0.16262E+00_JPRB, 0.16381E+00_JPRB, 0.16502E+00_JPRB, 0.16609E+00_JPRB ,&
&  0.16838E+00_JPRB, 0.17114E+00_JPRB, 0.17333E+00_JPRB, 0.17625E+00_JPRB, 0.17937E+00_JPRB, 0.18248E+00_JPRB, 0.18554E+00_JPRB ,&
&  0.18891E+00_JPRB, 0.19090E+00_JPRB, 0.19255E+00_JPRB, 0.19537E+00_JPRB, 0.19676E+00_JPRB /)
ZBURD(:, 66,  7) = (/ &
&  0.19181E+00_JPRB, 0.19448E+00_JPRB, 0.19634E+00_JPRB, 0.19812E+00_JPRB, 0.19951E+00_JPRB, 0.20105E+00_JPRB, 0.20186E+00_JPRB ,&
&  0.20262E+00_JPRB, 0.20272E+00_JPRB, 0.20321E+00_JPRB, 0.20321E+00_JPRB, 0.20343E+00_JPRB, 0.20230E+00_JPRB, 0.20172E+00_JPRB ,&
&  0.20124E+00_JPRB, 0.20151E+00_JPRB, 0.20139E+00_JPRB, 0.20113E+00_JPRB, 0.20026E+00_JPRB, 0.19917E+00_JPRB, 0.19759E+00_JPRB ,&
&  0.19450E+00_JPRB, 0.19245E+00_JPRB, 0.19282E+00_JPRB, 0.19236E+00_JPRB, 0.19276E+00_JPRB, 0.19274E+00_JPRB, 0.19315E+00_JPRB ,&
&  0.18883E+00_JPRB, 0.18728E+00_JPRB, 0.18759E+00_JPRB, 0.18654E+00_JPRB, 0.18855E+00_JPRB, 0.18942E+00_JPRB, 0.19127E+00_JPRB ,&
&  0.19182E+00_JPRB, 0.19223E+00_JPRB, 0.19308E+00_JPRB, 0.19271E+00_JPRB, 0.19308E+00_JPRB, 0.19358E+00_JPRB, 0.19244E+00_JPRB ,&
&  0.18937E+00_JPRB, 0.18697E+00_JPRB, 0.18360E+00_JPRB, 0.17548E+00_JPRB, 0.17360E+00_JPRB, 0.17394E+00_JPRB, 0.17255E+00_JPRB ,&
&  0.16982E+00_JPRB, 0.17025E+00_JPRB, 0.16982E+00_JPRB, 0.16825E+00_JPRB, 0.17007E+00_JPRB, 0.17273E+00_JPRB, 0.17285E+00_JPRB ,&
&  0.17160E+00_JPRB, 0.17127E+00_JPRB, 0.17062E+00_JPRB, 0.16935E+00_JPRB, 0.16782E+00_JPRB, 0.16717E+00_JPRB, 0.16658E+00_JPRB ,&
&  0.16629E+00_JPRB, 0.16493E+00_JPRB, 0.16820E+00_JPRB, 0.16889E+00_JPRB, 0.16395E+00_JPRB, 0.16593E+00_JPRB, 0.16365E+00_JPRB ,&
&  0.15692E+00_JPRB, 0.15388E+00_JPRB, 0.15220E+00_JPRB, 0.15079E+00_JPRB, 0.15054E+00_JPRB, 0.15067E+00_JPRB, 0.15055E+00_JPRB ,&
&  0.15148E+00_JPRB, 0.15263E+00_JPRB, 0.15318E+00_JPRB, 0.15388E+00_JPRB, 0.15486E+00_JPRB, 0.15626E+00_JPRB, 0.15809E+00_JPRB ,&
&  0.15969E+00_JPRB, 0.16191E+00_JPRB, 0.16304E+00_JPRB, 0.16490E+00_JPRB, 0.16629E+00_JPRB, 0.16794E+00_JPRB, 0.17182E+00_JPRB ,&
&  0.17498E+00_JPRB, 0.18110E+00_JPRB, 0.18939E+00_JPRB, 0.19191E+00_JPRB, 0.18927E+00_JPRB /)
ZBURD(:, 67,  7) = (/ &
&  0.18631E+00_JPRB, 0.18831E+00_JPRB, 0.19033E+00_JPRB, 0.19193E+00_JPRB, 0.19354E+00_JPRB, 0.19492E+00_JPRB, 0.19564E+00_JPRB ,&
&  0.19586E+00_JPRB, 0.19531E+00_JPRB, 0.19545E+00_JPRB, 0.19418E+00_JPRB, 0.19299E+00_JPRB, 0.18878E+00_JPRB, 0.18827E+00_JPRB ,&
&  0.18726E+00_JPRB, 0.18787E+00_JPRB, 0.18738E+00_JPRB, 0.18758E+00_JPRB, 0.18681E+00_JPRB, 0.18639E+00_JPRB, 0.18478E+00_JPRB ,&
&  0.18244E+00_JPRB, 0.18071E+00_JPRB, 0.17903E+00_JPRB, 0.17858E+00_JPRB, 0.17939E+00_JPRB, 0.17997E+00_JPRB, 0.17964E+00_JPRB ,&
&  0.17967E+00_JPRB, 0.17993E+00_JPRB, 0.18033E+00_JPRB, 0.18030E+00_JPRB, 0.18549E+00_JPRB, 0.18765E+00_JPRB, 0.18881E+00_JPRB ,&
&  0.18873E+00_JPRB, 0.18914E+00_JPRB, 0.18932E+00_JPRB, 0.18846E+00_JPRB, 0.18757E+00_JPRB, 0.18338E+00_JPRB, 0.18067E+00_JPRB ,&
&  0.17283E+00_JPRB, 0.16718E+00_JPRB, 0.16290E+00_JPRB, 0.16026E+00_JPRB, 0.15916E+00_JPRB, 0.15909E+00_JPRB, 0.15858E+00_JPRB ,&
&  0.15702E+00_JPRB, 0.15559E+00_JPRB, 0.15469E+00_JPRB, 0.15537E+00_JPRB, 0.15648E+00_JPRB, 0.15771E+00_JPRB, 0.15913E+00_JPRB ,&
&  0.16037E+00_JPRB, 0.16067E+00_JPRB, 0.16063E+00_JPRB, 0.16104E+00_JPRB, 0.16134E+00_JPRB, 0.16183E+00_JPRB, 0.16217E+00_JPRB ,&
&  0.16312E+00_JPRB, 0.16228E+00_JPRB, 0.16286E+00_JPRB, 0.16402E+00_JPRB, 0.16165E+00_JPRB, 0.15789E+00_JPRB, 0.15415E+00_JPRB ,&
&  0.15068E+00_JPRB, 0.14871E+00_JPRB, 0.14712E+00_JPRB, 0.14609E+00_JPRB, 0.14585E+00_JPRB, 0.14595E+00_JPRB, 0.14650E+00_JPRB ,&
&  0.14721E+00_JPRB, 0.14820E+00_JPRB, 0.14884E+00_JPRB, 0.14967E+00_JPRB, 0.15070E+00_JPRB, 0.15180E+00_JPRB, 0.15311E+00_JPRB ,&
&  0.15475E+00_JPRB, 0.15631E+00_JPRB, 0.15740E+00_JPRB, 0.15895E+00_JPRB, 0.16189E+00_JPRB, 0.16579E+00_JPRB, 0.17168E+00_JPRB ,&
&  0.17822E+00_JPRB, 0.18345E+00_JPRB, 0.18080E+00_JPRB, 0.18333E+00_JPRB, 0.18536E+00_JPRB /)
ZBURD(:, 68,  7) = (/ &
&  0.18609E+00_JPRB, 0.18767E+00_JPRB, 0.18916E+00_JPRB, 0.19068E+00_JPRB, 0.19196E+00_JPRB, 0.19320E+00_JPRB, 0.19401E+00_JPRB ,&
&  0.19151E+00_JPRB, 0.19009E+00_JPRB, 0.18746E+00_JPRB, 0.18513E+00_JPRB, 0.18240E+00_JPRB, 0.17936E+00_JPRB, 0.17835E+00_JPRB ,&
&  0.17680E+00_JPRB, 0.17648E+00_JPRB, 0.17517E+00_JPRB, 0.17460E+00_JPRB, 0.17351E+00_JPRB, 0.17287E+00_JPRB, 0.17178E+00_JPRB ,&
&  0.16964E+00_JPRB, 0.16857E+00_JPRB, 0.16755E+00_JPRB, 0.16792E+00_JPRB, 0.16942E+00_JPRB, 0.17104E+00_JPRB, 0.17276E+00_JPRB ,&
&  0.17521E+00_JPRB, 0.17629E+00_JPRB, 0.17864E+00_JPRB, 0.18040E+00_JPRB, 0.18340E+00_JPRB, 0.18591E+00_JPRB, 0.18646E+00_JPRB ,&
&  0.18650E+00_JPRB, 0.18469E+00_JPRB, 0.18413E+00_JPRB, 0.18196E+00_JPRB, 0.17592E+00_JPRB, 0.17254E+00_JPRB, 0.16773E+00_JPRB ,&
&  0.16238E+00_JPRB, 0.15837E+00_JPRB, 0.15561E+00_JPRB, 0.15472E+00_JPRB, 0.15381E+00_JPRB, 0.15350E+00_JPRB, 0.15329E+00_JPRB ,&
&  0.15251E+00_JPRB, 0.15161E+00_JPRB, 0.15054E+00_JPRB, 0.14993E+00_JPRB, 0.15030E+00_JPRB, 0.15106E+00_JPRB, 0.15268E+00_JPRB ,&
&  0.15431E+00_JPRB, 0.15522E+00_JPRB, 0.15623E+00_JPRB, 0.15656E+00_JPRB, 0.15668E+00_JPRB, 0.15691E+00_JPRB, 0.15695E+00_JPRB ,&
&  0.15760E+00_JPRB, 0.15773E+00_JPRB, 0.15720E+00_JPRB, 0.15633E+00_JPRB, 0.15412E+00_JPRB, 0.15121E+00_JPRB, 0.14893E+00_JPRB ,&
&  0.14699E+00_JPRB, 0.14574E+00_JPRB, 0.14530E+00_JPRB, 0.14485E+00_JPRB, 0.14510E+00_JPRB, 0.14548E+00_JPRB, 0.14583E+00_JPRB ,&
&  0.14643E+00_JPRB, 0.14695E+00_JPRB, 0.14747E+00_JPRB, 0.14864E+00_JPRB, 0.14960E+00_JPRB, 0.15103E+00_JPRB, 0.15253E+00_JPRB ,&
&  0.15357E+00_JPRB, 0.15516E+00_JPRB, 0.15651E+00_JPRB, 0.15800E+00_JPRB, 0.16247E+00_JPRB, 0.16600E+00_JPRB, 0.17316E+00_JPRB ,&
&  0.17773E+00_JPRB, 0.17920E+00_JPRB, 0.17967E+00_JPRB, 0.18514E+00_JPRB, 0.18549E+00_JPRB /)
ZBURD(:, 69,  7) = (/ &
&  0.18676E+00_JPRB, 0.18726E+00_JPRB, 0.18721E+00_JPRB, 0.18816E+00_JPRB, 0.18894E+00_JPRB, 0.18980E+00_JPRB, 0.18918E+00_JPRB ,&
&  0.18863E+00_JPRB, 0.18715E+00_JPRB, 0.18601E+00_JPRB, 0.18301E+00_JPRB, 0.18064E+00_JPRB, 0.17776E+00_JPRB, 0.17527E+00_JPRB ,&
&  0.17304E+00_JPRB, 0.17091E+00_JPRB, 0.16891E+00_JPRB, 0.16722E+00_JPRB, 0.16565E+00_JPRB, 0.16427E+00_JPRB, 0.16295E+00_JPRB ,&
&  0.16092E+00_JPRB, 0.16037E+00_JPRB, 0.16039E+00_JPRB, 0.16166E+00_JPRB, 0.16426E+00_JPRB, 0.16595E+00_JPRB, 0.16800E+00_JPRB ,&
&  0.17027E+00_JPRB, 0.16960E+00_JPRB, 0.17263E+00_JPRB, 0.17554E+00_JPRB, 0.17791E+00_JPRB, 0.17986E+00_JPRB, 0.18098E+00_JPRB ,&
&  0.18160E+00_JPRB, 0.17667E+00_JPRB, 0.17784E+00_JPRB, 0.17360E+00_JPRB, 0.16892E+00_JPRB, 0.16685E+00_JPRB, 0.16288E+00_JPRB ,&
&  0.16112E+00_JPRB, 0.15804E+00_JPRB, 0.15603E+00_JPRB, 0.15433E+00_JPRB, 0.15229E+00_JPRB, 0.15179E+00_JPRB, 0.15095E+00_JPRB ,&
&  0.15047E+00_JPRB, 0.15055E+00_JPRB, 0.14981E+00_JPRB, 0.14957E+00_JPRB, 0.14911E+00_JPRB, 0.14868E+00_JPRB, 0.14904E+00_JPRB ,&
&  0.14964E+00_JPRB, 0.15036E+00_JPRB, 0.15107E+00_JPRB, 0.15118E+00_JPRB, 0.15111E+00_JPRB, 0.15109E+00_JPRB, 0.15099E+00_JPRB ,&
&  0.15124E+00_JPRB, 0.15125E+00_JPRB, 0.15083E+00_JPRB, 0.15035E+00_JPRB, 0.14927E+00_JPRB, 0.14819E+00_JPRB, 0.14714E+00_JPRB ,&
&  0.14622E+00_JPRB, 0.14572E+00_JPRB, 0.14530E+00_JPRB, 0.14519E+00_JPRB, 0.14522E+00_JPRB, 0.14556E+00_JPRB, 0.14590E+00_JPRB ,&
&  0.14621E+00_JPRB, 0.14695E+00_JPRB, 0.14749E+00_JPRB, 0.14838E+00_JPRB, 0.14946E+00_JPRB, 0.15043E+00_JPRB, 0.15189E+00_JPRB ,&
&  0.15349E+00_JPRB, 0.15523E+00_JPRB, 0.15772E+00_JPRB, 0.15989E+00_JPRB, 0.16263E+00_JPRB, 0.16615E+00_JPRB, 0.16873E+00_JPRB ,&
&  0.17152E+00_JPRB, 0.17421E+00_JPRB, 0.17444E+00_JPRB, 0.18158E+00_JPRB, 0.18557E+00_JPRB /)
ZBURD(:, 70,  7) = (/ &
&  0.17916E+00_JPRB, 0.17996E+00_JPRB, 0.18036E+00_JPRB, 0.18077E+00_JPRB, 0.18181E+00_JPRB, 0.18181E+00_JPRB, 0.18161E+00_JPRB ,&
&  0.17954E+00_JPRB, 0.17874E+00_JPRB, 0.17746E+00_JPRB, 0.17565E+00_JPRB, 0.17381E+00_JPRB, 0.17074E+00_JPRB, 0.16882E+00_JPRB ,&
&  0.16694E+00_JPRB, 0.16496E+00_JPRB, 0.16367E+00_JPRB, 0.16199E+00_JPRB, 0.16054E+00_JPRB, 0.15890E+00_JPRB, 0.15836E+00_JPRB ,&
&  0.15754E+00_JPRB, 0.15748E+00_JPRB, 0.15829E+00_JPRB, 0.15893E+00_JPRB, 0.16012E+00_JPRB, 0.16138E+00_JPRB, 0.16300E+00_JPRB ,&
&  0.16341E+00_JPRB, 0.16418E+00_JPRB, 0.16493E+00_JPRB, 0.16542E+00_JPRB, 0.16714E+00_JPRB, 0.16835E+00_JPRB, 0.16878E+00_JPRB ,&
&  0.16829E+00_JPRB, 0.16742E+00_JPRB, 0.16570E+00_JPRB, 0.16375E+00_JPRB, 0.16151E+00_JPRB, 0.15907E+00_JPRB, 0.15674E+00_JPRB ,&
&  0.15494E+00_JPRB, 0.15327E+00_JPRB, 0.15199E+00_JPRB, 0.15095E+00_JPRB, 0.15031E+00_JPRB, 0.14980E+00_JPRB, 0.14929E+00_JPRB ,&
&  0.14900E+00_JPRB, 0.14875E+00_JPRB, 0.14839E+00_JPRB, 0.14839E+00_JPRB, 0.14829E+00_JPRB, 0.14834E+00_JPRB, 0.14839E+00_JPRB ,&
&  0.14849E+00_JPRB, 0.14846E+00_JPRB, 0.14835E+00_JPRB, 0.14805E+00_JPRB, 0.14775E+00_JPRB, 0.14742E+00_JPRB, 0.14708E+00_JPRB ,&
&  0.14670E+00_JPRB, 0.14665E+00_JPRB, 0.14634E+00_JPRB, 0.14612E+00_JPRB, 0.14584E+00_JPRB, 0.14537E+00_JPRB, 0.14522E+00_JPRB ,&
&  0.14530E+00_JPRB, 0.14542E+00_JPRB, 0.14528E+00_JPRB, 0.14541E+00_JPRB, 0.14562E+00_JPRB, 0.14592E+00_JPRB, 0.14673E+00_JPRB ,&
&  0.14723E+00_JPRB, 0.14776E+00_JPRB, 0.14824E+00_JPRB, 0.14893E+00_JPRB, 0.15011E+00_JPRB, 0.15128E+00_JPRB, 0.15260E+00_JPRB ,&
&  0.15418E+00_JPRB, 0.15566E+00_JPRB, 0.15715E+00_JPRB, 0.15861E+00_JPRB, 0.16103E+00_JPRB, 0.16274E+00_JPRB, 0.16497E+00_JPRB ,&
&  0.16692E+00_JPRB, 0.16582E+00_JPRB, 0.17394E+00_JPRB, 0.17474E+00_JPRB, 0.17603E+00_JPRB /)
ZBURD(:, 71,  7) = (/ &
&  0.16356E+00_JPRB, 0.16372E+00_JPRB, 0.16301E+00_JPRB, 0.16226E+00_JPRB, 0.15592E+00_JPRB, 0.16209E+00_JPRB, 0.16141E+00_JPRB ,&
&  0.16188E+00_JPRB, 0.16192E+00_JPRB, 0.16323E+00_JPRB, 0.16340E+00_JPRB, 0.16311E+00_JPRB, 0.16297E+00_JPRB, 0.16271E+00_JPRB ,&
&  0.16240E+00_JPRB, 0.16168E+00_JPRB, 0.16131E+00_JPRB, 0.16077E+00_JPRB, 0.16023E+00_JPRB, 0.15932E+00_JPRB, 0.15835E+00_JPRB ,&
&  0.15792E+00_JPRB, 0.15800E+00_JPRB, 0.15839E+00_JPRB, 0.15681E+00_JPRB, 0.15611E+00_JPRB, 0.15566E+00_JPRB, 0.15543E+00_JPRB ,&
&  0.15504E+00_JPRB, 0.15493E+00_JPRB, 0.15494E+00_JPRB, 0.15488E+00_JPRB, 0.15613E+00_JPRB, 0.15650E+00_JPRB, 0.15678E+00_JPRB ,&
&  0.15682E+00_JPRB, 0.15665E+00_JPRB, 0.15598E+00_JPRB, 0.15536E+00_JPRB, 0.15474E+00_JPRB, 0.15367E+00_JPRB, 0.15255E+00_JPRB ,&
&  0.15160E+00_JPRB, 0.15086E+00_JPRB, 0.15003E+00_JPRB, 0.14939E+00_JPRB, 0.14886E+00_JPRB, 0.14836E+00_JPRB, 0.14802E+00_JPRB ,&
&  0.14793E+00_JPRB, 0.14792E+00_JPRB, 0.14789E+00_JPRB, 0.14786E+00_JPRB, 0.14780E+00_JPRB, 0.14776E+00_JPRB, 0.14772E+00_JPRB ,&
&  0.14800E+00_JPRB, 0.14796E+00_JPRB, 0.14779E+00_JPRB, 0.14751E+00_JPRB, 0.14738E+00_JPRB, 0.14706E+00_JPRB, 0.14683E+00_JPRB ,&
&  0.14665E+00_JPRB, 0.14643E+00_JPRB, 0.14632E+00_JPRB, 0.14629E+00_JPRB, 0.14628E+00_JPRB, 0.14626E+00_JPRB, 0.14634E+00_JPRB ,&
&  0.14648E+00_JPRB, 0.14667E+00_JPRB, 0.14703E+00_JPRB, 0.14733E+00_JPRB, 0.14760E+00_JPRB, 0.14785E+00_JPRB, 0.14834E+00_JPRB ,&
&  0.14886E+00_JPRB, 0.14937E+00_JPRB, 0.14988E+00_JPRB, 0.15022E+00_JPRB, 0.15096E+00_JPRB, 0.15170E+00_JPRB, 0.15251E+00_JPRB ,&
&  0.15337E+00_JPRB, 0.15445E+00_JPRB, 0.15557E+00_JPRB, 0.15668E+00_JPRB, 0.15815E+00_JPRB, 0.15934E+00_JPRB, 0.16049E+00_JPRB ,&
&  0.16176E+00_JPRB, 0.16413E+00_JPRB, 0.16477E+00_JPRB, 0.16493E+00_JPRB, 0.16447E+00_JPRB /)
ZBURD(:, 72,  7) = (/ &
&  0.15874E+00_JPRB, 0.15871E+00_JPRB, 0.15860E+00_JPRB, 0.15825E+00_JPRB, 0.15756E+00_JPRB, 0.15741E+00_JPRB, 0.15720E+00_JPRB ,&
&  0.15690E+00_JPRB, 0.15625E+00_JPRB, 0.15624E+00_JPRB, 0.15616E+00_JPRB, 0.15606E+00_JPRB, 0.15594E+00_JPRB, 0.15571E+00_JPRB ,&
&  0.15523E+00_JPRB, 0.15450E+00_JPRB, 0.15232E+00_JPRB, 0.15128E+00_JPRB, 0.15062E+00_JPRB, 0.15026E+00_JPRB, 0.14997E+00_JPRB ,&
&  0.14980E+00_JPRB, 0.14971E+00_JPRB, 0.14962E+00_JPRB, 0.15005E+00_JPRB, 0.15024E+00_JPRB, 0.15031E+00_JPRB, 0.15036E+00_JPRB ,&
&  0.15036E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15045E+00_JPRB, 0.15056E+00_JPRB, 0.15056E+00_JPRB, 0.15038E+00_JPRB ,&
&  0.15017E+00_JPRB, 0.14991E+00_JPRB, 0.14966E+00_JPRB, 0.14936E+00_JPRB, 0.14914E+00_JPRB, 0.14896E+00_JPRB, 0.14888E+00_JPRB ,&
&  0.14873E+00_JPRB, 0.14861E+00_JPRB, 0.14849E+00_JPRB, 0.14843E+00_JPRB, 0.14833E+00_JPRB, 0.14828E+00_JPRB, 0.14816E+00_JPRB ,&
&  0.14824E+00_JPRB, 0.14826E+00_JPRB, 0.14832E+00_JPRB, 0.14833E+00_JPRB, 0.14835E+00_JPRB, 0.14831E+00_JPRB, 0.14823E+00_JPRB ,&
&  0.14805E+00_JPRB, 0.14795E+00_JPRB, 0.14783E+00_JPRB, 0.14774E+00_JPRB, 0.14771E+00_JPRB, 0.14769E+00_JPRB, 0.14768E+00_JPRB ,&
&  0.14769E+00_JPRB, 0.14778E+00_JPRB, 0.14781E+00_JPRB, 0.14786E+00_JPRB, 0.14793E+00_JPRB, 0.14800E+00_JPRB, 0.14805E+00_JPRB ,&
&  0.14814E+00_JPRB, 0.14822E+00_JPRB, 0.14832E+00_JPRB, 0.14846E+00_JPRB, 0.14862E+00_JPRB, 0.14885E+00_JPRB, 0.14913E+00_JPRB ,&
&  0.14942E+00_JPRB, 0.14972E+00_JPRB, 0.15002E+00_JPRB, 0.15031E+00_JPRB, 0.15077E+00_JPRB, 0.15123E+00_JPRB, 0.15177E+00_JPRB ,&
&  0.15231E+00_JPRB, 0.15285E+00_JPRB, 0.15334E+00_JPRB, 0.15386E+00_JPRB, 0.15452E+00_JPRB, 0.15486E+00_JPRB, 0.15514E+00_JPRB ,&
&  0.15559E+00_JPRB, 0.15605E+00_JPRB, 0.15652E+00_JPRB, 0.15692E+00_JPRB, 0.15758E+00_JPRB /)
ZBURD(:, 73,  7) = (/ &
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB ,&
&  0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB, 0.14978E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART14',1,ZHOOK_HANDLE)
END SUBROUTINE PART14



SUBROUTINE PART15

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART15',0,ZHOOK_HANDLE)


!-- month=  8
ZBURD(:,  1,  8) = (/ &
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB ,&
&  0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB, 0.10504E+01_JPRB /)
ZBURD(:,  2,  8) = (/ &
&  0.10685E+01_JPRB, 0.10625E+01_JPRB, 0.10573E+01_JPRB, 0.10519E+01_JPRB, 0.10464E+01_JPRB, 0.10418E+01_JPRB, 0.10376E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.10312E+01_JPRB, 0.10295E+01_JPRB, 0.10284E+01_JPRB, 0.10281E+01_JPRB, 0.10275E+01_JPRB, 0.10271E+01_JPRB ,&
&  0.10268E+01_JPRB, 0.10268E+01_JPRB, 0.10270E+01_JPRB, 0.10263E+01_JPRB, 0.10247E+01_JPRB, 0.10226E+01_JPRB, 0.10202E+01_JPRB ,&
&  0.10167E+01_JPRB, 0.10126E+01_JPRB, 0.10081E+01_JPRB, 0.10034E+01_JPRB, 0.99860E+00_JPRB, 0.99323E+00_JPRB, 0.98743E+00_JPRB ,&
&  0.98213E+00_JPRB, 0.97828E+00_JPRB, 0.97489E+00_JPRB, 0.97125E+00_JPRB, 0.96883E+00_JPRB, 0.96906E+00_JPRB, 0.96910E+00_JPRB ,&
&  0.96907E+00_JPRB, 0.96953E+00_JPRB, 0.97202E+00_JPRB, 0.97456E+00_JPRB, 0.97682E+00_JPRB, 0.97959E+00_JPRB, 0.98352E+00_JPRB ,&
&  0.98638E+00_JPRB, 0.98869E+00_JPRB, 0.99085E+00_JPRB, 0.99320E+00_JPRB, 0.99496E+00_JPRB, 0.99697E+00_JPRB, 0.99842E+00_JPRB ,&
&  0.99952E+00_JPRB, 0.10004E+01_JPRB, 0.10003E+01_JPRB, 0.99835E+00_JPRB, 0.99244E+00_JPRB, 0.98788E+00_JPRB, 0.98569E+00_JPRB ,&
&  0.98602E+00_JPRB, 0.98742E+00_JPRB, 0.98958E+00_JPRB, 0.99277E+00_JPRB, 0.99781E+00_JPRB, 0.10038E+01_JPRB, 0.10106E+01_JPRB ,&
&  0.10180E+01_JPRB, 0.10270E+01_JPRB, 0.10381E+01_JPRB, 0.10496E+01_JPRB, 0.10623E+01_JPRB, 0.10771E+01_JPRB, 0.10908E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.11198E+01_JPRB, 0.11322E+01_JPRB, 0.11434E+01_JPRB, 0.11542E+01_JPRB, 0.11645E+01_JPRB, 0.11719E+01_JPRB ,&
&  0.11751E+01_JPRB, 0.11774E+01_JPRB, 0.11790E+01_JPRB, 0.11774E+01_JPRB, 0.11734E+01_JPRB, 0.11702E+01_JPRB, 0.11668E+01_JPRB ,&
&  0.11625E+01_JPRB, 0.11569E+01_JPRB, 0.11516E+01_JPRB, 0.11457E+01_JPRB, 0.11375E+01_JPRB, 0.11283E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11016E+01_JPRB, 0.10927E+01_JPRB, 0.10842E+01_JPRB, 0.10763E+01_JPRB /)
ZBURD(:,  3,  8) = (/ &
&  0.11398E+01_JPRB, 0.11193E+01_JPRB, 0.11006E+01_JPRB, 0.10834E+01_JPRB, 0.10689E+01_JPRB, 0.10569E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10364E+01_JPRB, 0.10300E+01_JPRB, 0.10246E+01_JPRB, 0.10193E+01_JPRB, 0.10143E+01_JPRB, 0.10094E+01_JPRB, 0.10041E+01_JPRB ,&
&  0.99911E+00_JPRB, 0.99555E+00_JPRB, 0.99268E+00_JPRB, 0.98875E+00_JPRB, 0.98102E+00_JPRB, 0.96936E+00_JPRB, 0.95578E+00_JPRB ,&
&  0.94138E+00_JPRB, 0.92404E+00_JPRB, 0.90390E+00_JPRB, 0.88197E+00_JPRB, 0.86927E+00_JPRB, 0.85497E+00_JPRB, 0.84119E+00_JPRB ,&
&  0.82902E+00_JPRB, 0.82378E+00_JPRB, 0.81882E+00_JPRB, 0.81474E+00_JPRB, 0.81282E+00_JPRB, 0.81445E+00_JPRB, 0.81477E+00_JPRB ,&
&  0.81453E+00_JPRB, 0.81622E+00_JPRB, 0.82478E+00_JPRB, 0.83458E+00_JPRB, 0.84640E+00_JPRB, 0.86168E+00_JPRB, 0.87963E+00_JPRB ,&
&  0.89395E+00_JPRB, 0.90459E+00_JPRB, 0.91800E+00_JPRB, 0.93151E+00_JPRB, 0.94282E+00_JPRB, 0.95290E+00_JPRB, 0.96234E+00_JPRB ,&
&  0.97255E+00_JPRB, 0.98013E+00_JPRB, 0.98647E+00_JPRB, 0.98993E+00_JPRB, 0.98893E+00_JPRB, 0.98642E+00_JPRB, 0.98172E+00_JPRB ,&
&  0.97557E+00_JPRB, 0.97540E+00_JPRB, 0.98053E+00_JPRB, 0.98727E+00_JPRB, 0.99940E+00_JPRB, 0.10160E+01_JPRB, 0.10351E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10915E+01_JPRB, 0.11298E+01_JPRB, 0.11721E+01_JPRB, 0.12203E+01_JPRB, 0.12797E+01_JPRB, 0.13278E+01_JPRB ,&
&  0.13705E+01_JPRB, 0.14062E+01_JPRB, 0.14375E+01_JPRB, 0.14579E+01_JPRB, 0.14712E+01_JPRB, 0.14826E+01_JPRB, 0.14802E+01_JPRB ,&
&  0.14747E+01_JPRB, 0.14685E+01_JPRB, 0.14635E+01_JPRB, 0.14528E+01_JPRB, 0.14397E+01_JPRB, 0.14263E+01_JPRB, 0.14131E+01_JPRB ,&
&  0.13937E+01_JPRB, 0.13738E+01_JPRB, 0.13544E+01_JPRB, 0.13358E+01_JPRB, 0.13142E+01_JPRB, 0.12933E+01_JPRB, 0.12727E+01_JPRB ,&
&  0.12521E+01_JPRB, 0.12290E+01_JPRB, 0.12067E+01_JPRB, 0.11849E+01_JPRB, 0.11630E+01_JPRB /)
ZBURD(:,  4,  8) = (/ &
&  0.12234E+01_JPRB, 0.11904E+01_JPRB, 0.11602E+01_JPRB, 0.11334E+01_JPRB, 0.11089E+01_JPRB, 0.10884E+01_JPRB, 0.10704E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10434E+01_JPRB, 0.10323E+01_JPRB, 0.10207E+01_JPRB, 0.10094E+01_JPRB, 0.10036E+01_JPRB, 0.99542E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.97342E+00_JPRB, 0.96258E+00_JPRB, 0.94882E+00_JPRB, 0.92757E+00_JPRB, 0.90209E+00_JPRB, 0.87062E+00_JPRB ,&
&  0.84995E+00_JPRB, 0.82452E+00_JPRB, 0.79832E+00_JPRB, 0.77121E+00_JPRB, 0.75184E+00_JPRB, 0.71433E+00_JPRB, 0.69951E+00_JPRB ,&
&  0.68959E+00_JPRB, 0.66864E+00_JPRB, 0.65631E+00_JPRB, 0.65077E+00_JPRB, 0.66973E+00_JPRB, 0.69358E+00_JPRB, 0.70639E+00_JPRB ,&
&  0.68944E+00_JPRB, 0.70081E+00_JPRB, 0.69716E+00_JPRB, 0.69403E+00_JPRB, 0.69643E+00_JPRB, 0.70350E+00_JPRB, 0.73487E+00_JPRB ,&
&  0.79158E+00_JPRB, 0.81697E+00_JPRB, 0.83579E+00_JPRB, 0.85698E+00_JPRB, 0.87425E+00_JPRB, 0.89103E+00_JPRB, 0.90743E+00_JPRB ,&
&  0.92608E+00_JPRB, 0.94427E+00_JPRB, 0.96225E+00_JPRB, 0.97085E+00_JPRB, 0.98098E+00_JPRB, 0.99098E+00_JPRB, 0.99942E+00_JPRB ,&
&  0.99272E+00_JPRB, 0.99123E+00_JPRB, 0.99809E+00_JPRB, 0.10130E+01_JPRB, 0.10356E+01_JPRB, 0.10729E+01_JPRB, 0.11151E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.11978E+01_JPRB, 0.12539E+01_JPRB, 0.13276E+01_JPRB, 0.14152E+01_JPRB, 0.15212E+01_JPRB, 0.16272E+01_JPRB ,&
&  0.17302E+01_JPRB, 0.18142E+01_JPRB, 0.18922E+01_JPRB, 0.19266E+01_JPRB, 0.19416E+01_JPRB, 0.19504E+01_JPRB, 0.19474E+01_JPRB ,&
&  0.19417E+01_JPRB, 0.19266E+01_JPRB, 0.19113E+01_JPRB, 0.18892E+01_JPRB, 0.18587E+01_JPRB, 0.18196E+01_JPRB, 0.17752E+01_JPRB ,&
&  0.17228E+01_JPRB, 0.16722E+01_JPRB, 0.16230E+01_JPRB, 0.15754E+01_JPRB, 0.15275E+01_JPRB, 0.14834E+01_JPRB, 0.14442E+01_JPRB ,&
&  0.14071E+01_JPRB, 0.13683E+01_JPRB, 0.13320E+01_JPRB, 0.12964E+01_JPRB, 0.12607E+01_JPRB /)
ZBURD(:,  5,  8) = (/ &
&  0.12941E+01_JPRB, 0.12589E+01_JPRB, 0.12215E+01_JPRB, 0.11866E+01_JPRB, 0.11569E+01_JPRB, 0.11332E+01_JPRB, 0.11145E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.10774E+01_JPRB, 0.10569E+01_JPRB, 0.10352E+01_JPRB, 0.10147E+01_JPRB, 0.99984E+00_JPRB, 0.98611E+00_JPRB ,&
&  0.97457E+00_JPRB, 0.95698E+00_JPRB, 0.93644E+00_JPRB, 0.91193E+00_JPRB, 0.88516E+00_JPRB, 0.86168E+00_JPRB, 0.83035E+00_JPRB ,&
&  0.80659E+00_JPRB, 0.78112E+00_JPRB, 0.74533E+00_JPRB, 0.72448E+00_JPRB, 0.68385E+00_JPRB, 0.65616E+00_JPRB, 0.63860E+00_JPRB ,&
&  0.62581E+00_JPRB, 0.62707E+00_JPRB, 0.63279E+00_JPRB, 0.64309E+00_JPRB, 0.64901E+00_JPRB, 0.64505E+00_JPRB, 0.64646E+00_JPRB ,&
&  0.63729E+00_JPRB, 0.63102E+00_JPRB, 0.62562E+00_JPRB, 0.62389E+00_JPRB, 0.62657E+00_JPRB, 0.63569E+00_JPRB, 0.65523E+00_JPRB ,&
&  0.69676E+00_JPRB, 0.75518E+00_JPRB, 0.80990E+00_JPRB, 0.84559E+00_JPRB, 0.85843E+00_JPRB, 0.87546E+00_JPRB, 0.88474E+00_JPRB ,&
&  0.89874E+00_JPRB, 0.91250E+00_JPRB, 0.93343E+00_JPRB, 0.95392E+00_JPRB, 0.97372E+00_JPRB, 0.99057E+00_JPRB, 0.10098E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.10233E+01_JPRB, 0.10296E+01_JPRB, 0.10549E+01_JPRB, 0.11003E+01_JPRB, 0.11487E+01_JPRB, 0.11754E+01_JPRB ,&
&  0.12150E+01_JPRB, 0.12681E+01_JPRB, 0.13576E+01_JPRB, 0.14890E+01_JPRB, 0.16302E+01_JPRB, 0.17584E+01_JPRB, 0.18826E+01_JPRB ,&
&  0.20002E+01_JPRB, 0.21422E+01_JPRB, 0.22744E+01_JPRB, 0.23252E+01_JPRB, 0.23207E+01_JPRB, 0.23284E+01_JPRB, 0.23039E+01_JPRB ,&
&  0.22869E+01_JPRB, 0.22601E+01_JPRB, 0.22441E+01_JPRB, 0.22279E+01_JPRB, 0.21841E+01_JPRB, 0.21195E+01_JPRB, 0.20514E+01_JPRB ,&
&  0.19775E+01_JPRB, 0.18989E+01_JPRB, 0.18231E+01_JPRB, 0.17466E+01_JPRB, 0.16726E+01_JPRB, 0.16009E+01_JPRB, 0.15402E+01_JPRB ,&
&  0.14818E+01_JPRB, 0.14357E+01_JPRB, 0.13933E+01_JPRB, 0.13589E+01_JPRB, 0.13270E+01_JPRB /)
ZBURD(:,  6,  8) = (/ &
&  0.13617E+01_JPRB, 0.13378E+01_JPRB, 0.13138E+01_JPRB, 0.12811E+01_JPRB, 0.12477E+01_JPRB, 0.12160E+01_JPRB, 0.11909E+01_JPRB ,&
&  0.11686E+01_JPRB, 0.11448E+01_JPRB, 0.11213E+01_JPRB, 0.10940E+01_JPRB, 0.10676E+01_JPRB, 0.10410E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.99070E+00_JPRB, 0.95986E+00_JPRB, 0.93016E+00_JPRB, 0.90013E+00_JPRB, 0.87452E+00_JPRB, 0.85004E+00_JPRB, 0.82112E+00_JPRB ,&
&  0.79919E+00_JPRB, 0.76485E+00_JPRB, 0.73234E+00_JPRB, 0.71618E+00_JPRB, 0.68144E+00_JPRB, 0.66212E+00_JPRB, 0.64942E+00_JPRB ,&
&  0.66587E+00_JPRB, 0.65742E+00_JPRB, 0.67780E+00_JPRB, 0.65348E+00_JPRB, 0.63857E+00_JPRB, 0.62508E+00_JPRB, 0.61015E+00_JPRB ,&
&  0.59788E+00_JPRB, 0.58332E+00_JPRB, 0.58353E+00_JPRB, 0.58969E+00_JPRB, 0.59351E+00_JPRB, 0.60932E+00_JPRB, 0.62807E+00_JPRB ,&
&  0.67041E+00_JPRB, 0.79284E+00_JPRB, 0.82448E+00_JPRB, 0.85746E+00_JPRB, 0.86972E+00_JPRB, 0.88305E+00_JPRB, 0.89467E+00_JPRB ,&
&  0.90858E+00_JPRB, 0.92074E+00_JPRB, 0.93800E+00_JPRB, 0.95370E+00_JPRB, 0.98877E+00_JPRB, 0.10150E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10669E+01_JPRB, 0.10821E+01_JPRB, 0.11175E+01_JPRB, 0.11809E+01_JPRB, 0.12178E+01_JPRB, 0.12380E+01_JPRB ,&
&  0.12997E+01_JPRB, 0.13815E+01_JPRB, 0.15029E+01_JPRB, 0.16425E+01_JPRB, 0.17471E+01_JPRB, 0.18322E+01_JPRB, 0.19687E+01_JPRB ,&
&  0.20639E+01_JPRB, 0.22244E+01_JPRB, 0.23910E+01_JPRB, 0.24659E+01_JPRB, 0.24753E+01_JPRB, 0.24484E+01_JPRB, 0.24163E+01_JPRB ,&
&  0.24032E+01_JPRB, 0.23554E+01_JPRB, 0.23241E+01_JPRB, 0.23058E+01_JPRB, 0.22505E+01_JPRB, 0.21876E+01_JPRB, 0.21229E+01_JPRB ,&
&  0.20411E+01_JPRB, 0.19685E+01_JPRB, 0.19070E+01_JPRB, 0.18355E+01_JPRB, 0.17595E+01_JPRB, 0.16857E+01_JPRB, 0.16139E+01_JPRB ,&
&  0.15466E+01_JPRB, 0.14922E+01_JPRB, 0.14412E+01_JPRB, 0.14088E+01_JPRB, 0.13791E+01_JPRB /)
ZBURD(:,  7,  8) = (/ &
&  0.14101E+01_JPRB, 0.13853E+01_JPRB, 0.13617E+01_JPRB, 0.13397E+01_JPRB, 0.13150E+01_JPRB, 0.12898E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.12408E+01_JPRB, 0.12200E+01_JPRB, 0.11967E+01_JPRB, 0.11689E+01_JPRB, 0.11389E+01_JPRB, 0.11051E+01_JPRB, 0.10737E+01_JPRB ,&
&  0.10382E+01_JPRB, 0.10011E+01_JPRB, 0.95323E+00_JPRB, 0.90984E+00_JPRB, 0.87751E+00_JPRB, 0.85145E+00_JPRB, 0.82332E+00_JPRB ,&
&  0.80088E+00_JPRB, 0.77130E+00_JPRB, 0.73452E+00_JPRB, 0.70782E+00_JPRB, 0.67371E+00_JPRB, 0.66599E+00_JPRB, 0.70695E+00_JPRB ,&
&  0.72338E+00_JPRB, 0.72019E+00_JPRB, 0.70825E+00_JPRB, 0.69005E+00_JPRB, 0.67484E+00_JPRB, 0.61838E+00_JPRB, 0.63108E+00_JPRB ,&
&  0.60296E+00_JPRB, 0.57937E+00_JPRB, 0.57752E+00_JPRB, 0.57998E+00_JPRB, 0.58923E+00_JPRB, 0.61291E+00_JPRB, 0.63541E+00_JPRB ,&
&  0.67801E+00_JPRB, 0.82044E+00_JPRB, 0.85300E+00_JPRB, 0.88341E+00_JPRB, 0.89616E+00_JPRB, 0.90806E+00_JPRB, 0.91985E+00_JPRB ,&
&  0.93218E+00_JPRB, 0.95038E+00_JPRB, 0.96881E+00_JPRB, 0.99517E+00_JPRB, 0.10245E+01_JPRB, 0.10473E+01_JPRB, 0.10896E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11474E+01_JPRB, 0.11756E+01_JPRB, 0.12004E+01_JPRB, 0.12378E+01_JPRB, 0.12411E+01_JPRB, 0.12571E+01_JPRB ,&
&  0.13704E+01_JPRB, 0.15241E+01_JPRB, 0.16596E+01_JPRB, 0.17825E+01_JPRB, 0.18878E+01_JPRB, 0.20053E+01_JPRB, 0.21514E+01_JPRB ,&
&  0.22577E+01_JPRB, 0.24174E+01_JPRB, 0.25959E+01_JPRB, 0.26541E+01_JPRB, 0.26518E+01_JPRB, 0.26360E+01_JPRB, 0.26383E+01_JPRB ,&
&  0.26376E+01_JPRB, 0.26484E+01_JPRB, 0.25570E+01_JPRB, 0.24975E+01_JPRB, 0.23717E+01_JPRB, 0.22193E+01_JPRB, 0.20962E+01_JPRB ,&
&  0.19487E+01_JPRB, 0.18175E+01_JPRB, 0.17354E+01_JPRB, 0.16762E+01_JPRB, 0.16342E+01_JPRB, 0.16078E+01_JPRB, 0.15842E+01_JPRB ,&
&  0.15629E+01_JPRB, 0.15340E+01_JPRB, 0.15014E+01_JPRB, 0.14675E+01_JPRB, 0.14362E+01_JPRB /)
ZBURD(:,  8,  8) = (/ &
&  0.13945E+01_JPRB, 0.13719E+01_JPRB, 0.13450E+01_JPRB, 0.13238E+01_JPRB, 0.12994E+01_JPRB, 0.12816E+01_JPRB, 0.12646E+01_JPRB ,&
&  0.12546E+01_JPRB, 0.12510E+01_JPRB, 0.12441E+01_JPRB, 0.12380E+01_JPRB, 0.12234E+01_JPRB, 0.12036E+01_JPRB, 0.11757E+01_JPRB ,&
&  0.11364E+01_JPRB, 0.10516E+01_JPRB, 0.98592E+00_JPRB, 0.94822E+00_JPRB, 0.91270E+00_JPRB, 0.88494E+00_JPRB, 0.85721E+00_JPRB ,&
&  0.82175E+00_JPRB, 0.79272E+00_JPRB, 0.75928E+00_JPRB, 0.71896E+00_JPRB, 0.69735E+00_JPRB, 0.70673E+00_JPRB, 0.71345E+00_JPRB ,&
&  0.72319E+00_JPRB, 0.74533E+00_JPRB, 0.74122E+00_JPRB, 0.72012E+00_JPRB, 0.70863E+00_JPRB, 0.70829E+00_JPRB, 0.66032E+00_JPRB ,&
&  0.61641E+00_JPRB, 0.58195E+00_JPRB, 0.57680E+00_JPRB, 0.57586E+00_JPRB, 0.58629E+00_JPRB, 0.61090E+00_JPRB, 0.66991E+00_JPRB ,&
&  0.76410E+00_JPRB, 0.88789E+00_JPRB, 0.89845E+00_JPRB, 0.90844E+00_JPRB, 0.92419E+00_JPRB, 0.94087E+00_JPRB, 0.96028E+00_JPRB ,&
&  0.98516E+00_JPRB, 0.10208E+01_JPRB, 0.10653E+01_JPRB, 0.11127E+01_JPRB, 0.11620E+01_JPRB, 0.11862E+01_JPRB, 0.12110E+01_JPRB ,&
&  0.12660E+01_JPRB, 0.12609E+01_JPRB, 0.12745E+01_JPRB, 0.12814E+01_JPRB, 0.12867E+01_JPRB, 0.12676E+01_JPRB, 0.13340E+01_JPRB ,&
&  0.14808E+01_JPRB, 0.16312E+01_JPRB, 0.17773E+01_JPRB, 0.18868E+01_JPRB, 0.20102E+01_JPRB, 0.21806E+01_JPRB, 0.23654E+01_JPRB ,&
&  0.25298E+01_JPRB, 0.27546E+01_JPRB, 0.30559E+01_JPRB, 0.30358E+01_JPRB, 0.29811E+01_JPRB, 0.29454E+01_JPRB, 0.29497E+01_JPRB ,&
&  0.29374E+01_JPRB, 0.29613E+01_JPRB, 0.27849E+01_JPRB, 0.25312E+01_JPRB, 0.23020E+01_JPRB, 0.20588E+01_JPRB, 0.19201E+01_JPRB ,&
&  0.18175E+01_JPRB, 0.16627E+01_JPRB, 0.15817E+01_JPRB, 0.15344E+01_JPRB, 0.15049E+01_JPRB, 0.14756E+01_JPRB, 0.14514E+01_JPRB ,&
&  0.14415E+01_JPRB, 0.14383E+01_JPRB, 0.14380E+01_JPRB, 0.14274E+01_JPRB, 0.14136E+01_JPRB /)
ZBURD(:,  9,  8) = (/ &
&  0.13097E+01_JPRB, 0.13045E+01_JPRB, 0.12891E+01_JPRB, 0.12636E+01_JPRB, 0.12149E+01_JPRB, 0.11577E+01_JPRB, 0.11187E+01_JPRB ,&
&  0.10972E+01_JPRB, 0.10983E+01_JPRB, 0.10967E+01_JPRB, 0.11013E+01_JPRB, 0.11143E+01_JPRB, 0.11332E+01_JPRB, 0.11484E+01_JPRB ,&
&  0.11501E+01_JPRB, 0.11275E+01_JPRB, 0.10827E+01_JPRB, 0.10150E+01_JPRB, 0.95928E+00_JPRB, 0.91971E+00_JPRB, 0.89569E+00_JPRB ,&
&  0.87695E+00_JPRB, 0.85021E+00_JPRB, 0.81776E+00_JPRB, 0.76932E+00_JPRB, 0.73526E+00_JPRB, 0.71098E+00_JPRB, 0.71831E+00_JPRB ,&
&  0.74518E+00_JPRB, 0.75730E+00_JPRB, 0.74852E+00_JPRB, 0.76641E+00_JPRB, 0.76036E+00_JPRB, 0.75095E+00_JPRB, 0.68776E+00_JPRB ,&
&  0.64029E+00_JPRB, 0.61578E+00_JPRB, 0.59840E+00_JPRB, 0.59422E+00_JPRB, 0.61191E+00_JPRB, 0.64628E+00_JPRB, 0.71553E+00_JPRB ,&
&  0.84848E+00_JPRB, 0.90155E+00_JPRB, 0.92115E+00_JPRB, 0.94571E+00_JPRB, 0.96733E+00_JPRB, 0.98112E+00_JPRB, 0.10125E+01_JPRB ,&
&  0.10822E+01_JPRB, 0.11879E+01_JPRB, 0.12889E+01_JPRB, 0.13194E+01_JPRB, 0.13409E+01_JPRB, 0.13713E+01_JPRB, 0.15752E+01_JPRB ,&
&  0.21260E+01_JPRB, 0.18805E+01_JPRB, 0.15971E+01_JPRB, 0.14374E+01_JPRB, 0.13710E+01_JPRB, 0.13572E+01_JPRB, 0.14586E+01_JPRB ,&
&  0.16059E+01_JPRB, 0.17472E+01_JPRB, 0.19253E+01_JPRB, 0.20663E+01_JPRB, 0.22268E+01_JPRB, 0.24392E+01_JPRB, 0.27346E+01_JPRB ,&
&  0.30304E+01_JPRB, 0.33869E+01_JPRB, 0.40177E+01_JPRB, 0.36015E+01_JPRB, 0.33673E+01_JPRB, 0.32794E+01_JPRB, 0.32315E+01_JPRB ,&
&  0.30896E+01_JPRB, 0.30599E+01_JPRB, 0.27862E+01_JPRB, 0.24341E+01_JPRB, 0.22502E+01_JPRB, 0.20570E+01_JPRB, 0.19121E+01_JPRB ,&
&  0.17749E+01_JPRB, 0.16436E+01_JPRB, 0.15628E+01_JPRB, 0.15006E+01_JPRB, 0.14749E+01_JPRB, 0.14486E+01_JPRB, 0.14215E+01_JPRB ,&
&  0.13912E+01_JPRB, 0.13621E+01_JPRB, 0.13410E+01_JPRB, 0.13234E+01_JPRB, 0.13128E+01_JPRB /)
ZBURD(:, 10,  8) = (/ &
&  0.12336E+01_JPRB, 0.12459E+01_JPRB, 0.12443E+01_JPRB, 0.11932E+01_JPRB, 0.11221E+01_JPRB, 0.10303E+01_JPRB, 0.95922E+00_JPRB ,&
&  0.90535E+00_JPRB, 0.88845E+00_JPRB, 0.88461E+00_JPRB, 0.89511E+00_JPRB, 0.91945E+00_JPRB, 0.95477E+00_JPRB, 0.98330E+00_JPRB ,&
&  0.10106E+01_JPRB, 0.10194E+01_JPRB, 0.10221E+01_JPRB, 0.10033E+01_JPRB, 0.96534E+00_JPRB, 0.92156E+00_JPRB, 0.89015E+00_JPRB ,&
&  0.87071E+00_JPRB, 0.85725E+00_JPRB, 0.83792E+00_JPRB, 0.80751E+00_JPRB, 0.78369E+00_JPRB, 0.75382E+00_JPRB, 0.75226E+00_JPRB ,&
&  0.73875E+00_JPRB, 0.74705E+00_JPRB, 0.76942E+00_JPRB, 0.77578E+00_JPRB, 0.79334E+00_JPRB, 0.79000E+00_JPRB, 0.73720E+00_JPRB ,&
&  0.68387E+00_JPRB, 0.65561E+00_JPRB, 0.64440E+00_JPRB, 0.64497E+00_JPRB, 0.70126E+00_JPRB, 0.82751E+00_JPRB, 0.90015E+00_JPRB ,&
&  0.94000E+00_JPRB, 0.96213E+00_JPRB, 0.98931E+00_JPRB, 0.10202E+01_JPRB, 0.10386E+01_JPRB, 0.10650E+01_JPRB, 0.11408E+01_JPRB ,&
&  0.12873E+01_JPRB, 0.14838E+01_JPRB, 0.16051E+01_JPRB, 0.14133E+01_JPRB, 0.13302E+01_JPRB, 0.14285E+01_JPRB, 0.15755E+01_JPRB ,&
&  0.17413E+01_JPRB, 0.16725E+01_JPRB, 0.15917E+01_JPRB, 0.16529E+01_JPRB, 0.16918E+01_JPRB, 0.17350E+01_JPRB, 0.17768E+01_JPRB ,&
&  0.18331E+01_JPRB, 0.19784E+01_JPRB, 0.22709E+01_JPRB, 0.24308E+01_JPRB, 0.26800E+01_JPRB, 0.30355E+01_JPRB, 0.33459E+01_JPRB ,&
&  0.37186E+01_JPRB, 0.39879E+01_JPRB, 0.41000E+01_JPRB, 0.39130E+01_JPRB, 0.37738E+01_JPRB, 0.37535E+01_JPRB, 0.36503E+01_JPRB ,&
&  0.33058E+01_JPRB, 0.30482E+01_JPRB, 0.28305E+01_JPRB, 0.26334E+01_JPRB, 0.23611E+01_JPRB, 0.21362E+01_JPRB, 0.19082E+01_JPRB ,&
&  0.16935E+01_JPRB, 0.15362E+01_JPRB, 0.14484E+01_JPRB, 0.13838E+01_JPRB, 0.13558E+01_JPRB, 0.13372E+01_JPRB, 0.13031E+01_JPRB ,&
&  0.12612E+01_JPRB, 0.12213E+01_JPRB, 0.11974E+01_JPRB, 0.11986E+01_JPRB, 0.12315E+01_JPRB /)
ZBURD(:, 11,  8) = (/ &
&  0.12781E+01_JPRB, 0.12498E+01_JPRB, 0.11804E+01_JPRB, 0.11544E+01_JPRB, 0.10993E+01_JPRB, 0.10366E+01_JPRB, 0.95559E+00_JPRB ,&
&  0.89158E+00_JPRB, 0.83515E+00_JPRB, 0.79485E+00_JPRB, 0.75105E+00_JPRB, 0.73229E+00_JPRB, 0.73556E+00_JPRB, 0.77846E+00_JPRB ,&
&  0.83416E+00_JPRB, 0.87849E+00_JPRB, 0.90867E+00_JPRB, 0.92006E+00_JPRB, 0.91730E+00_JPRB, 0.89947E+00_JPRB, 0.87876E+00_JPRB ,&
&  0.85983E+00_JPRB, 0.85405E+00_JPRB, 0.84105E+00_JPRB, 0.82448E+00_JPRB, 0.80457E+00_JPRB, 0.78091E+00_JPRB, 0.76592E+00_JPRB ,&
&  0.74832E+00_JPRB, 0.74469E+00_JPRB, 0.75521E+00_JPRB, 0.77696E+00_JPRB, 0.80742E+00_JPRB, 0.81439E+00_JPRB, 0.76806E+00_JPRB ,&
&  0.72664E+00_JPRB, 0.71299E+00_JPRB, 0.72420E+00_JPRB, 0.83302E+00_JPRB, 0.88970E+00_JPRB, 0.92257E+00_JPRB, 0.96192E+00_JPRB ,&
&  0.95501E+00_JPRB, 0.95393E+00_JPRB, 0.98286E+00_JPRB, 0.10733E+01_JPRB, 0.11102E+01_JPRB, 0.11553E+01_JPRB, 0.12570E+01_JPRB ,&
&  0.14340E+01_JPRB, 0.16655E+01_JPRB, 0.18238E+01_JPRB, 0.16666E+01_JPRB, 0.18305E+01_JPRB, 0.22709E+01_JPRB, 0.20822E+01_JPRB ,&
&  0.19988E+01_JPRB, 0.19100E+01_JPRB, 0.19761E+01_JPRB, 0.20678E+01_JPRB, 0.20764E+01_JPRB, 0.21668E+01_JPRB, 0.22640E+01_JPRB ,&
&  0.22577E+01_JPRB, 0.24303E+01_JPRB, 0.27737E+01_JPRB, 0.31003E+01_JPRB, 0.34189E+01_JPRB, 0.38304E+01_JPRB, 0.42659E+01_JPRB ,&
&  0.47942E+01_JPRB, 0.52404E+01_JPRB, 0.55202E+01_JPRB, 0.54093E+01_JPRB, 0.52211E+01_JPRB, 0.49063E+01_JPRB, 0.44598E+01_JPRB ,&
&  0.38650E+01_JPRB, 0.34443E+01_JPRB, 0.31684E+01_JPRB, 0.29578E+01_JPRB, 0.26975E+01_JPRB, 0.23488E+01_JPRB, 0.18979E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.13539E+01_JPRB, 0.12642E+01_JPRB, 0.12354E+01_JPRB, 0.12389E+01_JPRB, 0.12446E+01_JPRB, 0.12263E+01_JPRB ,&
&  0.11978E+01_JPRB, 0.11763E+01_JPRB, 0.11595E+01_JPRB, 0.11700E+01_JPRB, 0.12579E+01_JPRB /)
ZBURD(:, 12,  8) = (/ &
&  0.13345E+01_JPRB, 0.12672E+01_JPRB, 0.12047E+01_JPRB, 0.11710E+01_JPRB, 0.11241E+01_JPRB, 0.10364E+01_JPRB, 0.92290E+00_JPRB ,&
&  0.90651E+00_JPRB, 0.85967E+00_JPRB, 0.83268E+00_JPRB, 0.80916E+00_JPRB, 0.78267E+00_JPRB, 0.73430E+00_JPRB, 0.70516E+00_JPRB ,&
&  0.72092E+00_JPRB, 0.76562E+00_JPRB, 0.81574E+00_JPRB, 0.85713E+00_JPRB, 0.87775E+00_JPRB, 0.87668E+00_JPRB, 0.86655E+00_JPRB ,&
&  0.85582E+00_JPRB, 0.85152E+00_JPRB, 0.85044E+00_JPRB, 0.84882E+00_JPRB, 0.83925E+00_JPRB, 0.82636E+00_JPRB, 0.81825E+00_JPRB ,&
&  0.80593E+00_JPRB, 0.79379E+00_JPRB, 0.78587E+00_JPRB, 0.79535E+00_JPRB, 0.80848E+00_JPRB, 0.83666E+00_JPRB, 0.84512E+00_JPRB ,&
&  0.77688E+00_JPRB, 0.79242E+00_JPRB, 0.89511E+00_JPRB, 0.93540E+00_JPRB, 0.97545E+00_JPRB, 0.10092E+01_JPRB, 0.10381E+01_JPRB ,&
&  0.10654E+01_JPRB, 0.11038E+01_JPRB, 0.11478E+01_JPRB, 0.11933E+01_JPRB, 0.12258E+01_JPRB, 0.12935E+01_JPRB, 0.14111E+01_JPRB ,&
&  0.16624E+01_JPRB, 0.16452E+01_JPRB, 0.16230E+01_JPRB, 0.18256E+01_JPRB, 0.25244E+01_JPRB, 0.26806E+01_JPRB, 0.26698E+01_JPRB ,&
&  0.26841E+01_JPRB, 0.25952E+01_JPRB, 0.25552E+01_JPRB, 0.26373E+01_JPRB, 0.27003E+01_JPRB, 0.28956E+01_JPRB, 0.31849E+01_JPRB ,&
&  0.32349E+01_JPRB, 0.35055E+01_JPRB, 0.38303E+01_JPRB, 0.41023E+01_JPRB, 0.44050E+01_JPRB, 0.48647E+01_JPRB, 0.54059E+01_JPRB ,&
&  0.59575E+01_JPRB, 0.64117E+01_JPRB, 0.65571E+01_JPRB, 0.63722E+01_JPRB, 0.59973E+01_JPRB, 0.55663E+01_JPRB, 0.50166E+01_JPRB ,&
&  0.43032E+01_JPRB, 0.38168E+01_JPRB, 0.35096E+01_JPRB, 0.32276E+01_JPRB, 0.29690E+01_JPRB, 0.26139E+01_JPRB, 0.20833E+01_JPRB ,&
&  0.15218E+01_JPRB, 0.12894E+01_JPRB, 0.12308E+01_JPRB, 0.12206E+01_JPRB, 0.12385E+01_JPRB, 0.12744E+01_JPRB, 0.12967E+01_JPRB ,&
&  0.13114E+01_JPRB, 0.12796E+01_JPRB, 0.12611E+01_JPRB, 0.13051E+01_JPRB, 0.13556E+01_JPRB /)
ZBURD(:, 13,  8) = (/ &
&  0.12342E+01_JPRB, 0.11985E+01_JPRB, 0.11823E+01_JPRB, 0.11737E+01_JPRB, 0.11188E+01_JPRB, 0.10338E+01_JPRB, 0.95644E+00_JPRB ,&
&  0.10589E+01_JPRB, 0.10444E+01_JPRB, 0.10742E+01_JPRB, 0.90122E+00_JPRB, 0.84923E+00_JPRB, 0.82292E+00_JPRB, 0.78246E+00_JPRB ,&
&  0.75730E+00_JPRB, 0.75599E+00_JPRB, 0.76777E+00_JPRB, 0.79964E+00_JPRB, 0.82650E+00_JPRB, 0.83527E+00_JPRB, 0.84119E+00_JPRB ,&
&  0.84087E+00_JPRB, 0.84407E+00_JPRB, 0.85806E+00_JPRB, 0.85716E+00_JPRB, 0.85288E+00_JPRB, 0.85056E+00_JPRB, 0.85078E+00_JPRB ,&
&  0.81862E+00_JPRB, 0.80692E+00_JPRB, 0.80750E+00_JPRB, 0.80533E+00_JPRB, 0.82136E+00_JPRB, 0.83857E+00_JPRB, 0.87093E+00_JPRB ,&
&  0.92783E+00_JPRB, 0.90574E+00_JPRB, 0.10276E+01_JPRB, 0.10669E+01_JPRB, 0.11069E+01_JPRB, 0.11416E+01_JPRB, 0.11709E+01_JPRB ,&
&  0.11985E+01_JPRB, 0.12393E+01_JPRB, 0.12842E+01_JPRB, 0.13198E+01_JPRB, 0.13538E+01_JPRB, 0.14772E+01_JPRB, 0.16683E+01_JPRB ,&
&  0.20270E+01_JPRB, 0.18744E+01_JPRB, 0.20988E+01_JPRB, 0.22553E+01_JPRB, 0.27030E+01_JPRB, 0.31115E+01_JPRB, 0.31772E+01_JPRB ,&
&  0.37178E+01_JPRB, 0.35414E+01_JPRB, 0.33817E+01_JPRB, 0.32203E+01_JPRB, 0.33831E+01_JPRB, 0.37003E+01_JPRB, 0.40411E+01_JPRB ,&
&  0.43327E+01_JPRB, 0.45133E+01_JPRB, 0.49145E+01_JPRB, 0.51972E+01_JPRB, 0.55227E+01_JPRB, 0.59234E+01_JPRB, 0.62998E+01_JPRB ,&
&  0.66320E+01_JPRB, 0.68350E+01_JPRB, 0.69269E+01_JPRB, 0.64668E+01_JPRB, 0.60766E+01_JPRB, 0.56906E+01_JPRB, 0.48978E+01_JPRB ,&
&  0.42699E+01_JPRB, 0.38274E+01_JPRB, 0.34713E+01_JPRB, 0.32335E+01_JPRB, 0.27786E+01_JPRB, 0.23925E+01_JPRB, 0.20083E+01_JPRB ,&
&  0.15228E+01_JPRB, 0.13277E+01_JPRB, 0.13250E+01_JPRB, 0.13487E+01_JPRB, 0.13494E+01_JPRB, 0.13563E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.13550E+01_JPRB, 0.13416E+01_JPRB, 0.13906E+01_JPRB, 0.13514E+01_JPRB, 0.12922E+01_JPRB /)
ZBURD(:, 14,  8) = (/ &
&  0.12721E+01_JPRB, 0.12683E+01_JPRB, 0.12518E+01_JPRB, 0.12054E+01_JPRB, 0.11442E+01_JPRB, 0.11260E+01_JPRB, 0.10742E+01_JPRB ,&
&  0.11496E+01_JPRB, 0.11752E+01_JPRB, 0.11675E+01_JPRB, 0.11656E+01_JPRB, 0.11723E+01_JPRB, 0.10702E+01_JPRB, 0.90438E+00_JPRB ,&
&  0.83155E+00_JPRB, 0.83350E+00_JPRB, 0.86020E+00_JPRB, 0.88671E+00_JPRB, 0.90918E+00_JPRB, 0.92408E+00_JPRB, 0.94770E+00_JPRB ,&
&  0.96302E+00_JPRB, 0.95790E+00_JPRB, 0.96186E+00_JPRB, 0.95401E+00_JPRB, 0.93639E+00_JPRB, 0.92592E+00_JPRB, 0.92012E+00_JPRB ,&
&  0.86387E+00_JPRB, 0.83255E+00_JPRB, 0.81327E+00_JPRB, 0.79021E+00_JPRB, 0.83333E+00_JPRB, 0.87379E+00_JPRB, 0.93315E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.10951E+01_JPRB, 0.11565E+01_JPRB, 0.12144E+01_JPRB, 0.12606E+01_JPRB, 0.13015E+01_JPRB, 0.13396E+01_JPRB ,&
&  0.13837E+01_JPRB, 0.14273E+01_JPRB, 0.14761E+01_JPRB, 0.15119E+01_JPRB, 0.15903E+01_JPRB, 0.18948E+01_JPRB, 0.24647E+01_JPRB ,&
&  0.26937E+01_JPRB, 0.29492E+01_JPRB, 0.31511E+01_JPRB, 0.29023E+01_JPRB, 0.31865E+01_JPRB, 0.36083E+01_JPRB, 0.36960E+01_JPRB ,&
&  0.38862E+01_JPRB, 0.39219E+01_JPRB, 0.41438E+01_JPRB, 0.44196E+01_JPRB, 0.50665E+01_JPRB, 0.55368E+01_JPRB, 0.57515E+01_JPRB ,&
&  0.58976E+01_JPRB, 0.63285E+01_JPRB, 0.67200E+01_JPRB, 0.68959E+01_JPRB, 0.69043E+01_JPRB, 0.69629E+01_JPRB, 0.70716E+01_JPRB ,&
&  0.71689E+01_JPRB, 0.72706E+01_JPRB, 0.72161E+01_JPRB, 0.69436E+01_JPRB, 0.65112E+01_JPRB, 0.59493E+01_JPRB, 0.51825E+01_JPRB ,&
&  0.45174E+01_JPRB, 0.37355E+01_JPRB, 0.31022E+01_JPRB, 0.27304E+01_JPRB, 0.23788E+01_JPRB, 0.20100E+01_JPRB, 0.17481E+01_JPRB ,&
&  0.14991E+01_JPRB, 0.14422E+01_JPRB, 0.15347E+01_JPRB, 0.15353E+01_JPRB, 0.15239E+01_JPRB, 0.14761E+01_JPRB, 0.13567E+01_JPRB ,&
&  0.13491E+01_JPRB, 0.13918E+01_JPRB, 0.14046E+01_JPRB, 0.13612E+01_JPRB, 0.13072E+01_JPRB /)
ZBURD(:, 15,  8) = (/ &
&  0.14257E+01_JPRB, 0.13939E+01_JPRB, 0.13494E+01_JPRB, 0.12964E+01_JPRB, 0.12597E+01_JPRB, 0.12422E+01_JPRB, 0.12349E+01_JPRB ,&
&  0.12436E+01_JPRB, 0.12570E+01_JPRB, 0.12509E+01_JPRB, 0.12253E+01_JPRB, 0.11964E+01_JPRB, 0.11767E+01_JPRB, 0.10763E+01_JPRB ,&
&  0.96552E+00_JPRB, 0.98949E+00_JPRB, 0.10018E+01_JPRB, 0.11082E+01_JPRB, 0.12311E+01_JPRB, 0.12523E+01_JPRB, 0.12655E+01_JPRB ,&
&  0.12881E+01_JPRB, 0.12362E+01_JPRB, 0.12246E+01_JPRB, 0.12148E+01_JPRB, 0.11882E+01_JPRB, 0.11591E+01_JPRB, 0.11241E+01_JPRB ,&
&  0.10092E+01_JPRB, 0.93870E+00_JPRB, 0.90683E+00_JPRB, 0.89204E+00_JPRB, 0.93869E+00_JPRB, 0.10247E+01_JPRB, 0.11163E+01_JPRB ,&
&  0.12075E+01_JPRB, 0.12849E+01_JPRB, 0.13622E+01_JPRB, 0.14268E+01_JPRB, 0.14687E+01_JPRB, 0.15063E+01_JPRB, 0.15488E+01_JPRB ,&
&  0.16025E+01_JPRB, 0.16664E+01_JPRB, 0.17078E+01_JPRB, 0.17788E+01_JPRB, 0.19529E+01_JPRB, 0.28605E+01_JPRB, 0.39073E+01_JPRB ,&
&  0.37514E+01_JPRB, 0.37532E+01_JPRB, 0.40966E+01_JPRB, 0.45862E+01_JPRB, 0.47530E+01_JPRB, 0.46299E+01_JPRB, 0.47278E+01_JPRB ,&
&  0.52259E+01_JPRB, 0.58219E+01_JPRB, 0.70308E+01_JPRB, 0.71585E+01_JPRB, 0.73236E+01_JPRB, 0.75742E+01_JPRB, 0.74871E+01_JPRB ,&
&  0.76309E+01_JPRB, 0.76242E+01_JPRB, 0.78310E+01_JPRB, 0.76976E+01_JPRB, 0.74921E+01_JPRB, 0.75166E+01_JPRB, 0.74548E+01_JPRB ,&
&  0.75471E+01_JPRB, 0.75585E+01_JPRB, 0.70873E+01_JPRB, 0.66891E+01_JPRB, 0.59859E+01_JPRB, 0.59954E+01_JPRB, 0.54680E+01_JPRB ,&
&  0.44876E+01_JPRB, 0.32021E+01_JPRB, 0.28820E+01_JPRB, 0.25040E+01_JPRB, 0.22481E+01_JPRB, 0.19443E+01_JPRB, 0.16898E+01_JPRB ,&
&  0.15658E+01_JPRB, 0.16542E+01_JPRB, 0.16482E+01_JPRB, 0.16379E+01_JPRB, 0.16363E+01_JPRB, 0.16018E+01_JPRB, 0.14653E+01_JPRB ,&
&  0.14367E+01_JPRB, 0.15090E+01_JPRB, 0.15206E+01_JPRB, 0.14894E+01_JPRB, 0.14523E+01_JPRB /)
ZBURD(:, 16,  8) = (/ &
&  0.16285E+01_JPRB, 0.15609E+01_JPRB, 0.15045E+01_JPRB, 0.14649E+01_JPRB, 0.14210E+01_JPRB, 0.13727E+01_JPRB, 0.13300E+01_JPRB ,&
&  0.13261E+01_JPRB, 0.13381E+01_JPRB, 0.13358E+01_JPRB, 0.12916E+01_JPRB, 0.12455E+01_JPRB, 0.12176E+01_JPRB, 0.12246E+01_JPRB ,&
&  0.11210E+01_JPRB, 0.11933E+01_JPRB, 0.12767E+01_JPRB, 0.15263E+01_JPRB, 0.20637E+01_JPRB, 0.19698E+01_JPRB, 0.19982E+01_JPRB ,&
&  0.19214E+01_JPRB, 0.18342E+01_JPRB, 0.17802E+01_JPRB, 0.17600E+01_JPRB, 0.16978E+01_JPRB, 0.16519E+01_JPRB, 0.15698E+01_JPRB ,&
&  0.13624E+01_JPRB, 0.12137E+01_JPRB, 0.11691E+01_JPRB, 0.11539E+01_JPRB, 0.12189E+01_JPRB, 0.13007E+01_JPRB, 0.14186E+01_JPRB ,&
&  0.15079E+01_JPRB, 0.15980E+01_JPRB, 0.16776E+01_JPRB, 0.17407E+01_JPRB, 0.17747E+01_JPRB, 0.18042E+01_JPRB, 0.18403E+01_JPRB ,&
&  0.18995E+01_JPRB, 0.19528E+01_JPRB, 0.20112E+01_JPRB, 0.21189E+01_JPRB, 0.26151E+01_JPRB, 0.43172E+01_JPRB, 0.60708E+01_JPRB ,&
&  0.58124E+01_JPRB, 0.59170E+01_JPRB, 0.79768E+01_JPRB, 0.98648E+01_JPRB, 0.87380E+01_JPRB, 0.75565E+01_JPRB, 0.71397E+01_JPRB ,&
&  0.76305E+01_JPRB, 0.84617E+01_JPRB, 0.93931E+01_JPRB, 0.96827E+01_JPRB, 0.94308E+01_JPRB, 0.94293E+01_JPRB, 0.92641E+01_JPRB ,&
&  0.89334E+01_JPRB, 0.86684E+01_JPRB, 0.83437E+01_JPRB, 0.80685E+01_JPRB, 0.77980E+01_JPRB, 0.79349E+01_JPRB, 0.78377E+01_JPRB ,&
&  0.76439E+01_JPRB, 0.72006E+01_JPRB, 0.65237E+01_JPRB, 0.61366E+01_JPRB, 0.49750E+01_JPRB, 0.48142E+01_JPRB, 0.55203E+01_JPRB ,&
&  0.44777E+01_JPRB, 0.34541E+01_JPRB, 0.28971E+01_JPRB, 0.25492E+01_JPRB, 0.23882E+01_JPRB, 0.23224E+01_JPRB, 0.21109E+01_JPRB ,&
&  0.18122E+01_JPRB, 0.18471E+01_JPRB, 0.19688E+01_JPRB, 0.19820E+01_JPRB, 0.19671E+01_JPRB, 0.19451E+01_JPRB, 0.18329E+01_JPRB ,&
&  0.18168E+01_JPRB, 0.17614E+01_JPRB, 0.17497E+01_JPRB, 0.17255E+01_JPRB, 0.16869E+01_JPRB /)
ZBURD(:, 17,  8) = (/ &
&  0.18215E+01_JPRB, 0.17257E+01_JPRB, 0.16659E+01_JPRB, 0.15921E+01_JPRB, 0.15087E+01_JPRB, 0.14348E+01_JPRB, 0.13841E+01_JPRB ,&
&  0.13946E+01_JPRB, 0.13987E+01_JPRB, 0.13808E+01_JPRB, 0.13348E+01_JPRB, 0.12940E+01_JPRB, 0.12749E+01_JPRB, 0.12951E+01_JPRB ,&
&  0.13143E+01_JPRB, 0.14827E+01_JPRB, 0.15536E+01_JPRB, 0.16162E+01_JPRB, 0.21378E+01_JPRB, 0.24394E+01_JPRB, 0.26982E+01_JPRB ,&
&  0.27503E+01_JPRB, 0.28019E+01_JPRB, 0.27907E+01_JPRB, 0.27715E+01_JPRB, 0.27064E+01_JPRB, 0.26158E+01_JPRB, 0.24546E+01_JPRB ,&
&  0.22370E+01_JPRB, 0.19925E+01_JPRB, 0.18675E+01_JPRB, 0.18558E+01_JPRB, 0.19248E+01_JPRB, 0.17419E+01_JPRB, 0.18571E+01_JPRB ,&
&  0.19632E+01_JPRB, 0.20691E+01_JPRB, 0.21125E+01_JPRB, 0.21144E+01_JPRB, 0.20846E+01_JPRB, 0.20674E+01_JPRB, 0.20747E+01_JPRB ,&
&  0.21357E+01_JPRB, 0.21385E+01_JPRB, 0.20932E+01_JPRB, 0.23272E+01_JPRB, 0.32506E+01_JPRB, 0.53738E+01_JPRB, 0.75494E+01_JPRB ,&
&  0.83653E+01_JPRB, 0.10433E+02_JPRB, 0.13395E+02_JPRB, 0.14284E+02_JPRB, 0.12130E+02_JPRB, 0.10607E+02_JPRB, 0.10424E+02_JPRB ,&
&  0.11027E+02_JPRB, 0.11797E+02_JPRB, 0.12134E+02_JPRB, 0.11442E+02_JPRB, 0.11295E+02_JPRB, 0.11032E+02_JPRB, 0.10524E+02_JPRB ,&
&  0.97641E+01_JPRB, 0.89809E+01_JPRB, 0.85315E+01_JPRB, 0.81680E+01_JPRB, 0.78817E+01_JPRB, 0.72067E+01_JPRB, 0.69710E+01_JPRB ,&
&  0.71225E+01_JPRB, 0.57972E+01_JPRB, 0.51088E+01_JPRB, 0.53753E+01_JPRB, 0.42826E+01_JPRB, 0.42099E+01_JPRB, 0.46344E+01_JPRB ,&
&  0.38945E+01_JPRB, 0.32047E+01_JPRB, 0.29037E+01_JPRB, 0.25822E+01_JPRB, 0.25062E+01_JPRB, 0.27993E+01_JPRB, 0.28761E+01_JPRB ,&
&  0.26320E+01_JPRB, 0.24495E+01_JPRB, 0.26597E+01_JPRB, 0.27137E+01_JPRB, 0.27225E+01_JPRB, 0.26001E+01_JPRB, 0.24302E+01_JPRB ,&
&  0.23041E+01_JPRB, 0.22184E+01_JPRB, 0.21420E+01_JPRB, 0.20516E+01_JPRB, 0.19507E+01_JPRB /)
ZBURD(:, 18,  8) = (/ &
&  0.19504E+01_JPRB, 0.19005E+01_JPRB, 0.18135E+01_JPRB, 0.16919E+01_JPRB, 0.15845E+01_JPRB, 0.15029E+01_JPRB, 0.14328E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.13578E+01_JPRB, 0.13379E+01_JPRB, 0.13154E+01_JPRB, 0.13090E+01_JPRB, 0.13125E+01_JPRB, 0.13155E+01_JPRB ,&
&  0.13078E+01_JPRB, 0.15255E+01_JPRB, 0.14686E+01_JPRB, 0.15258E+01_JPRB, 0.18235E+01_JPRB, 0.27967E+01_JPRB, 0.35091E+01_JPRB ,&
&  0.38384E+01_JPRB, 0.38705E+01_JPRB, 0.39843E+01_JPRB, 0.43253E+01_JPRB, 0.44982E+01_JPRB, 0.43356E+01_JPRB, 0.42177E+01_JPRB ,&
&  0.40294E+01_JPRB, 0.37375E+01_JPRB, 0.31309E+01_JPRB, 0.30887E+01_JPRB, 0.28758E+01_JPRB, 0.25299E+01_JPRB, 0.27279E+01_JPRB ,&
&  0.27459E+01_JPRB, 0.26623E+01_JPRB, 0.24820E+01_JPRB, 0.23136E+01_JPRB, 0.21968E+01_JPRB, 0.20563E+01_JPRB, 0.20006E+01_JPRB ,&
&  0.20328E+01_JPRB, 0.21091E+01_JPRB, 0.23347E+01_JPRB, 0.31022E+01_JPRB, 0.49490E+01_JPRB, 0.75643E+01_JPRB, 0.96140E+01_JPRB ,&
&  0.11614E+02_JPRB, 0.12859E+02_JPRB, 0.12355E+02_JPRB, 0.13537E+02_JPRB, 0.15186E+02_JPRB, 0.14299E+02_JPRB, 0.13311E+02_JPRB ,&
&  0.13157E+02_JPRB, 0.12955E+02_JPRB, 0.12426E+02_JPRB, 0.11912E+02_JPRB, 0.11861E+02_JPRB, 0.11351E+02_JPRB, 0.10464E+02_JPRB ,&
&  0.89033E+01_JPRB, 0.83382E+01_JPRB, 0.82669E+01_JPRB, 0.81533E+01_JPRB, 0.72150E+01_JPRB, 0.67886E+01_JPRB, 0.68342E+01_JPRB ,&
&  0.66505E+01_JPRB, 0.64611E+01_JPRB, 0.49741E+01_JPRB, 0.46888E+01_JPRB, 0.35048E+01_JPRB, 0.34086E+01_JPRB, 0.39617E+01_JPRB ,&
&  0.35871E+01_JPRB, 0.32726E+01_JPRB, 0.28741E+01_JPRB, 0.27160E+01_JPRB, 0.33143E+01_JPRB, 0.38814E+01_JPRB, 0.45198E+01_JPRB ,&
&  0.41400E+01_JPRB, 0.33269E+01_JPRB, 0.35100E+01_JPRB, 0.34907E+01_JPRB, 0.35191E+01_JPRB, 0.32944E+01_JPRB, 0.29561E+01_JPRB ,&
&  0.27059E+01_JPRB, 0.25225E+01_JPRB, 0.23280E+01_JPRB, 0.21419E+01_JPRB, 0.20229E+01_JPRB /)
ZBURD(:, 19,  8) = (/ &
&  0.19471E+01_JPRB, 0.18996E+01_JPRB, 0.17986E+01_JPRB, 0.16683E+01_JPRB, 0.15639E+01_JPRB, 0.14829E+01_JPRB, 0.14162E+01_JPRB ,&
&  0.13523E+01_JPRB, 0.13008E+01_JPRB, 0.12777E+01_JPRB, 0.12770E+01_JPRB, 0.12856E+01_JPRB, 0.12857E+01_JPRB, 0.12736E+01_JPRB ,&
&  0.12681E+01_JPRB, 0.14895E+01_JPRB, 0.14591E+01_JPRB, 0.14680E+01_JPRB, 0.19293E+01_JPRB, 0.30308E+01_JPRB, 0.40519E+01_JPRB ,&
&  0.44605E+01_JPRB, 0.45220E+01_JPRB, 0.48854E+01_JPRB, 0.55229E+01_JPRB, 0.63105E+01_JPRB, 0.67945E+01_JPRB, 0.69478E+01_JPRB ,&
&  0.67966E+01_JPRB, 0.52483E+01_JPRB, 0.47518E+01_JPRB, 0.42838E+01_JPRB, 0.40019E+01_JPRB, 0.37748E+01_JPRB, 0.35409E+01_JPRB ,&
&  0.31841E+01_JPRB, 0.27182E+01_JPRB, 0.23587E+01_JPRB, 0.21342E+01_JPRB, 0.20177E+01_JPRB, 0.19148E+01_JPRB, 0.20379E+01_JPRB ,&
&  0.23000E+01_JPRB, 0.26733E+01_JPRB, 0.38141E+01_JPRB, 0.57692E+01_JPRB, 0.89705E+01_JPRB, 0.11533E+02_JPRB, 0.11948E+02_JPRB ,&
&  0.10492E+02_JPRB, 0.11367E+02_JPRB, 0.17081E+02_JPRB, 0.15439E+02_JPRB, 0.15481E+02_JPRB, 0.16847E+02_JPRB, 0.15653E+02_JPRB ,&
&  0.13793E+02_JPRB, 0.12343E+02_JPRB, 0.11435E+02_JPRB, 0.10975E+02_JPRB, 0.12497E+02_JPRB, 0.11827E+02_JPRB, 0.10251E+02_JPRB ,&
&  0.88492E+01_JPRB, 0.86114E+01_JPRB, 0.81285E+01_JPRB, 0.75968E+01_JPRB, 0.71472E+01_JPRB, 0.74551E+01_JPRB, 0.69270E+01_JPRB ,&
&  0.64151E+01_JPRB, 0.70560E+01_JPRB, 0.55388E+01_JPRB, 0.41947E+01_JPRB, 0.34039E+01_JPRB, 0.35290E+01_JPRB, 0.38415E+01_JPRB ,&
&  0.37867E+01_JPRB, 0.35726E+01_JPRB, 0.33991E+01_JPRB, 0.40578E+01_JPRB, 0.55817E+01_JPRB, 0.64624E+01_JPRB, 0.59115E+01_JPRB ,&
&  0.51510E+01_JPRB, 0.51300E+01_JPRB, 0.48459E+01_JPRB, 0.46918E+01_JPRB, 0.45388E+01_JPRB, 0.39526E+01_JPRB, 0.32502E+01_JPRB ,&
&  0.26595E+01_JPRB, 0.22996E+01_JPRB, 0.21207E+01_JPRB, 0.20253E+01_JPRB, 0.19769E+01_JPRB /)
ZBURD(:, 20,  8) = (/ &
&  0.19717E+01_JPRB, 0.18772E+01_JPRB, 0.17631E+01_JPRB, 0.16405E+01_JPRB, 0.15229E+01_JPRB, 0.14235E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.12676E+01_JPRB, 0.12204E+01_JPRB, 0.12207E+01_JPRB, 0.12446E+01_JPRB, 0.12664E+01_JPRB, 0.12474E+01_JPRB, 0.12400E+01_JPRB ,&
&  0.13022E+01_JPRB, 0.14804E+01_JPRB, 0.14409E+01_JPRB, 0.16099E+01_JPRB, 0.20881E+01_JPRB, 0.28566E+01_JPRB, 0.38716E+01_JPRB ,&
&  0.43415E+01_JPRB, 0.43083E+01_JPRB, 0.47305E+01_JPRB, 0.64741E+01_JPRB, 0.88423E+01_JPRB, 0.10367E+02_JPRB, 0.10638E+02_JPRB ,&
&  0.91859E+01_JPRB, 0.81587E+01_JPRB, 0.69958E+01_JPRB, 0.59520E+01_JPRB, 0.51445E+01_JPRB, 0.44127E+01_JPRB, 0.35701E+01_JPRB ,&
&  0.28138E+01_JPRB, 0.23117E+01_JPRB, 0.20751E+01_JPRB, 0.19811E+01_JPRB, 0.19815E+01_JPRB, 0.20666E+01_JPRB, 0.24286E+01_JPRB ,&
&  0.30628E+01_JPRB, 0.41187E+01_JPRB, 0.60409E+01_JPRB, 0.83958E+01_JPRB, 0.90372E+01_JPRB, 0.11435E+02_JPRB, 0.11394E+02_JPRB ,&
&  0.12902E+02_JPRB, 0.14138E+02_JPRB, 0.14173E+02_JPRB, 0.15103E+02_JPRB, 0.13020E+02_JPRB, 0.14258E+02_JPRB, 0.14698E+02_JPRB ,&
&  0.13733E+02_JPRB, 0.12863E+02_JPRB, 0.12851E+02_JPRB, 0.11322E+02_JPRB, 0.10605E+02_JPRB, 0.12554E+02_JPRB, 0.10791E+02_JPRB ,&
&  0.92610E+01_JPRB, 0.87142E+01_JPRB, 0.78772E+01_JPRB, 0.77827E+01_JPRB, 0.64652E+01_JPRB, 0.63488E+01_JPRB, 0.55018E+01_JPRB ,&
&  0.51798E+01_JPRB, 0.42151E+01_JPRB, 0.46555E+01_JPRB, 0.39242E+01_JPRB, 0.36565E+01_JPRB, 0.36124E+01_JPRB, 0.36656E+01_JPRB ,&
&  0.39138E+01_JPRB, 0.41576E+01_JPRB, 0.49713E+01_JPRB, 0.66929E+01_JPRB, 0.87131E+01_JPRB, 0.71527E+01_JPRB, 0.68537E+01_JPRB ,&
&  0.62525E+01_JPRB, 0.59728E+01_JPRB, 0.50904E+01_JPRB, 0.47605E+01_JPRB, 0.42323E+01_JPRB, 0.37830E+01_JPRB, 0.33171E+01_JPRB ,&
&  0.28395E+01_JPRB, 0.24487E+01_JPRB, 0.22386E+01_JPRB, 0.21263E+01_JPRB, 0.20493E+01_JPRB /)
ZBURD(:, 21,  8) = (/ &
&  0.18542E+01_JPRB, 0.17402E+01_JPRB, 0.16380E+01_JPRB, 0.15337E+01_JPRB, 0.14300E+01_JPRB, 0.13189E+01_JPRB, 0.12094E+01_JPRB ,&
&  0.11286E+01_JPRB, 0.11011E+01_JPRB, 0.11291E+01_JPRB, 0.11671E+01_JPRB, 0.11923E+01_JPRB, 0.12253E+01_JPRB, 0.12409E+01_JPRB ,&
&  0.13386E+01_JPRB, 0.14961E+01_JPRB, 0.14751E+01_JPRB, 0.15711E+01_JPRB, 0.20459E+01_JPRB, 0.28183E+01_JPRB, 0.33673E+01_JPRB ,&
&  0.40508E+01_JPRB, 0.38383E+01_JPRB, 0.44090E+01_JPRB, 0.67459E+01_JPRB, 0.11739E+02_JPRB, 0.15155E+02_JPRB, 0.14808E+02_JPRB ,&
&  0.14430E+02_JPRB, 0.11963E+02_JPRB, 0.90704E+01_JPRB, 0.67919E+01_JPRB, 0.49021E+01_JPRB, 0.35343E+01_JPRB, 0.25780E+01_JPRB ,&
&  0.20717E+01_JPRB, 0.18504E+01_JPRB, 0.18043E+01_JPRB, 0.18240E+01_JPRB, 0.19595E+01_JPRB, 0.21723E+01_JPRB, 0.29500E+01_JPRB ,&
&  0.38840E+01_JPRB, 0.51345E+01_JPRB, 0.66196E+01_JPRB, 0.75523E+01_JPRB, 0.74480E+01_JPRB, 0.91448E+01_JPRB, 0.11225E+02_JPRB ,&
&  0.11332E+02_JPRB, 0.11882E+02_JPRB, 0.13282E+02_JPRB, 0.13840E+02_JPRB, 0.15018E+02_JPRB, 0.13719E+02_JPRB, 0.13079E+02_JPRB ,&
&  0.97336E+01_JPRB, 0.79646E+01_JPRB, 0.71454E+01_JPRB, 0.71807E+01_JPRB, 0.89387E+01_JPRB, 0.11699E+02_JPRB, 0.10901E+02_JPRB ,&
&  0.95408E+01_JPRB, 0.89774E+01_JPRB, 0.82100E+01_JPRB, 0.66454E+01_JPRB, 0.64548E+01_JPRB, 0.51657E+01_JPRB, 0.62423E+01_JPRB ,&
&  0.61120E+01_JPRB, 0.55644E+01_JPRB, 0.45689E+01_JPRB, 0.34535E+01_JPRB, 0.27236E+01_JPRB, 0.33863E+01_JPRB, 0.37589E+01_JPRB ,&
&  0.43577E+01_JPRB, 0.50958E+01_JPRB, 0.83799E+01_JPRB, 0.93399E+01_JPRB, 0.92609E+01_JPRB, 0.70890E+01_JPRB, 0.72024E+01_JPRB ,&
&  0.63663E+01_JPRB, 0.61414E+01_JPRB, 0.53617E+01_JPRB, 0.41920E+01_JPRB, 0.34074E+01_JPRB, 0.29289E+01_JPRB, 0.26809E+01_JPRB ,&
&  0.25790E+01_JPRB, 0.24799E+01_JPRB, 0.23182E+01_JPRB, 0.21500E+01_JPRB, 0.20109E+01_JPRB /)
ZBURD(:, 22,  8) = (/ &
&  0.15515E+01_JPRB, 0.14373E+01_JPRB, 0.13582E+01_JPRB, 0.12900E+01_JPRB, 0.12100E+01_JPRB, 0.11283E+01_JPRB, 0.10527E+01_JPRB ,&
&  0.10091E+01_JPRB, 0.10124E+01_JPRB, 0.10505E+01_JPRB, 0.11137E+01_JPRB, 0.11691E+01_JPRB, 0.12210E+01_JPRB, 0.12456E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.16026E+01_JPRB, 0.15423E+01_JPRB, 0.16693E+01_JPRB, 0.21108E+01_JPRB, 0.27732E+01_JPRB, 0.31993E+01_JPRB ,&
&  0.37185E+01_JPRB, 0.37218E+01_JPRB, 0.46940E+01_JPRB, 0.70838E+01_JPRB, 0.14187E+02_JPRB, 0.16507E+02_JPRB, 0.15523E+02_JPRB ,&
&  0.14876E+02_JPRB, 0.10896E+02_JPRB, 0.69518E+01_JPRB, 0.47925E+01_JPRB, 0.34274E+01_JPRB, 0.24818E+01_JPRB, 0.20444E+01_JPRB ,&
&  0.18878E+01_JPRB, 0.18472E+01_JPRB, 0.18534E+01_JPRB, 0.19151E+01_JPRB, 0.20926E+01_JPRB, 0.25888E+01_JPRB, 0.35801E+01_JPRB ,&
&  0.46360E+01_JPRB, 0.56353E+01_JPRB, 0.62987E+01_JPRB, 0.63466E+01_JPRB, 0.62822E+01_JPRB, 0.76876E+01_JPRB, 0.10825E+02_JPRB ,&
&  0.10496E+02_JPRB, 0.11441E+02_JPRB, 0.12257E+02_JPRB, 0.13270E+02_JPRB, 0.13183E+02_JPRB, 0.12346E+02_JPRB, 0.11333E+02_JPRB ,&
&  0.81003E+01_JPRB, 0.70254E+01_JPRB, 0.76013E+01_JPRB, 0.80595E+01_JPRB, 0.73351E+01_JPRB, 0.83081E+01_JPRB, 0.10919E+02_JPRB ,&
&  0.82933E+01_JPRB, 0.82079E+01_JPRB, 0.81681E+01_JPRB, 0.80320E+01_JPRB, 0.44942E+01_JPRB, 0.29377E+01_JPRB, 0.44481E+01_JPRB ,&
&  0.42806E+01_JPRB, 0.22954E+01_JPRB, 0.17240E+01_JPRB, 0.23994E+01_JPRB, 0.16294E+01_JPRB, 0.21164E+01_JPRB, 0.38181E+01_JPRB ,&
&  0.50149E+01_JPRB, 0.63825E+01_JPRB, 0.11368E+02_JPRB, 0.97221E+01_JPRB, 0.89506E+01_JPRB, 0.80998E+01_JPRB, 0.78391E+01_JPRB ,&
&  0.72318E+01_JPRB, 0.49517E+01_JPRB, 0.38609E+01_JPRB, 0.29341E+01_JPRB, 0.23903E+01_JPRB, 0.22670E+01_JPRB, 0.22000E+01_JPRB ,&
&  0.21444E+01_JPRB, 0.20351E+01_JPRB, 0.19356E+01_JPRB, 0.17930E+01_JPRB, 0.16829E+01_JPRB /)
ZBURD(:, 23,  8) = (/ &
&  0.11767E+01_JPRB, 0.10992E+01_JPRB, 0.10821E+01_JPRB, 0.10659E+01_JPRB, 0.10103E+01_JPRB, 0.96896E+00_JPRB, 0.93682E+00_JPRB ,&
&  0.94439E+00_JPRB, 0.98894E+00_JPRB, 0.10475E+01_JPRB, 0.11221E+01_JPRB, 0.11981E+01_JPRB, 0.12610E+01_JPRB, 0.12800E+01_JPRB ,&
&  0.13424E+01_JPRB, 0.15515E+01_JPRB, 0.18274E+01_JPRB, 0.21225E+01_JPRB, 0.26689E+01_JPRB, 0.28385E+01_JPRB, 0.35935E+01_JPRB ,&
&  0.36258E+01_JPRB, 0.36824E+01_JPRB, 0.50613E+01_JPRB, 0.76713E+01_JPRB, 0.11604E+02_JPRB, 0.12656E+02_JPRB, 0.11422E+02_JPRB ,&
&  0.88243E+01_JPRB, 0.59837E+01_JPRB, 0.42541E+01_JPRB, 0.31772E+01_JPRB, 0.25167E+01_JPRB, 0.21560E+01_JPRB, 0.21155E+01_JPRB ,&
&  0.20082E+01_JPRB, 0.19707E+01_JPRB, 0.20085E+01_JPRB, 0.20372E+01_JPRB, 0.23649E+01_JPRB, 0.29992E+01_JPRB, 0.39841E+01_JPRB ,&
&  0.48629E+01_JPRB, 0.55049E+01_JPRB, 0.57349E+01_JPRB, 0.54131E+01_JPRB, 0.68331E+01_JPRB, 0.77740E+01_JPRB, 0.76560E+01_JPRB ,&
&  0.81790E+01_JPRB, 0.91091E+01_JPRB, 0.12225E+02_JPRB, 0.12308E+02_JPRB, 0.11767E+02_JPRB, 0.12218E+02_JPRB, 0.10573E+02_JPRB ,&
&  0.94821E+01_JPRB, 0.90344E+01_JPRB, 0.78044E+01_JPRB, 0.81344E+01_JPRB, 0.79874E+01_JPRB, 0.71389E+01_JPRB, 0.77912E+01_JPRB ,&
&  0.81857E+01_JPRB, 0.73093E+01_JPRB, 0.58655E+01_JPRB, 0.47741E+01_JPRB, 0.46180E+01_JPRB, 0.33787E+01_JPRB, 0.21154E+01_JPRB ,&
&  0.16848E+01_JPRB, 0.13639E+01_JPRB, 0.98479E+00_JPRB, 0.11999E+01_JPRB, 0.11148E+01_JPRB, 0.16764E+01_JPRB, 0.34893E+01_JPRB ,&
&  0.66900E+01_JPRB, 0.89496E+01_JPRB, 0.11182E+02_JPRB, 0.93858E+01_JPRB, 0.82175E+01_JPRB, 0.79220E+01_JPRB, 0.83611E+01_JPRB ,&
&  0.44215E+01_JPRB, 0.33376E+01_JPRB, 0.25698E+01_JPRB, 0.21410E+01_JPRB, 0.19973E+01_JPRB, 0.19152E+01_JPRB, 0.18335E+01_JPRB ,&
&  0.17129E+01_JPRB, 0.16057E+01_JPRB, 0.15572E+01_JPRB, 0.14801E+01_JPRB, 0.13281E+01_JPRB /)
ZBURD(:, 24,  8) = (/ &
&  0.99524E+00_JPRB, 0.98497E+00_JPRB, 0.99491E+00_JPRB, 0.97425E+00_JPRB, 0.91923E+00_JPRB, 0.92634E+00_JPRB, 0.94693E+00_JPRB ,&
&  0.99367E+00_JPRB, 0.10574E+01_JPRB, 0.11305E+01_JPRB, 0.12241E+01_JPRB, 0.13321E+01_JPRB, 0.13802E+01_JPRB, 0.13823E+01_JPRB ,&
&  0.14304E+01_JPRB, 0.16420E+01_JPRB, 0.20822E+01_JPRB, 0.26540E+01_JPRB, 0.33683E+01_JPRB, 0.32800E+01_JPRB, 0.38866E+01_JPRB ,&
&  0.32607E+01_JPRB, 0.34130E+01_JPRB, 0.47444E+01_JPRB, 0.66447E+01_JPRB, 0.87822E+01_JPRB, 0.82497E+01_JPRB, 0.59777E+01_JPRB ,&
&  0.40866E+01_JPRB, 0.33094E+01_JPRB, 0.29673E+01_JPRB, 0.25630E+01_JPRB, 0.22398E+01_JPRB, 0.21095E+01_JPRB, 0.21069E+01_JPRB ,&
&  0.20291E+01_JPRB, 0.20317E+01_JPRB, 0.20585E+01_JPRB, 0.23078E+01_JPRB, 0.28246E+01_JPRB, 0.34773E+01_JPRB, 0.42534E+01_JPRB ,&
&  0.47578E+01_JPRB, 0.51001E+01_JPRB, 0.52009E+01_JPRB, 0.56018E+01_JPRB, 0.56512E+01_JPRB, 0.57411E+01_JPRB, 0.69742E+01_JPRB ,&
&  0.90698E+01_JPRB, 0.10824E+02_JPRB, 0.10589E+02_JPRB, 0.12358E+02_JPRB, 0.11759E+02_JPRB, 0.10965E+02_JPRB, 0.98198E+01_JPRB ,&
&  0.86847E+01_JPRB, 0.89451E+01_JPRB, 0.75368E+01_JPRB, 0.78202E+01_JPRB, 0.91167E+01_JPRB, 0.83663E+01_JPRB, 0.73697E+01_JPRB ,&
&  0.79087E+01_JPRB, 0.66269E+01_JPRB, 0.57513E+01_JPRB, 0.48887E+01_JPRB, 0.63868E+01_JPRB, 0.60390E+01_JPRB, 0.28262E+01_JPRB ,&
&  0.17383E+01_JPRB, 0.12378E+01_JPRB, 0.88340E+00_JPRB, 0.90604E+00_JPRB, 0.10347E+01_JPRB, 0.16237E+01_JPRB, 0.47298E+01_JPRB ,&
&  0.60484E+01_JPRB, 0.10086E+02_JPRB, 0.11430E+02_JPRB, 0.10110E+02_JPRB, 0.84084E+01_JPRB, 0.67594E+01_JPRB, 0.45749E+01_JPRB ,&
&  0.26565E+01_JPRB, 0.17730E+01_JPRB, 0.16779E+01_JPRB, 0.18289E+01_JPRB, 0.18778E+01_JPRB, 0.17642E+01_JPRB, 0.15919E+01_JPRB ,&
&  0.14773E+01_JPRB, 0.14376E+01_JPRB, 0.13643E+01_JPRB, 0.12832E+01_JPRB, 0.11338E+01_JPRB /)
ZBURD(:, 25,  8) = (/ &
&  0.98527E+00_JPRB, 0.10233E+01_JPRB, 0.10572E+01_JPRB, 0.99467E+00_JPRB, 0.98048E+00_JPRB, 0.10042E+01_JPRB, 0.10051E+01_JPRB ,&
&  0.10797E+01_JPRB, 0.11784E+01_JPRB, 0.12657E+01_JPRB, 0.13995E+01_JPRB, 0.15162E+01_JPRB, 0.15202E+01_JPRB, 0.15131E+01_JPRB ,&
&  0.15934E+01_JPRB, 0.18730E+01_JPRB, 0.24487E+01_JPRB, 0.32647E+01_JPRB, 0.37078E+01_JPRB, 0.35656E+01_JPRB, 0.36822E+01_JPRB ,&
&  0.29324E+01_JPRB, 0.31546E+01_JPRB, 0.39510E+01_JPRB, 0.50405E+01_JPRB, 0.53763E+01_JPRB, 0.49556E+01_JPRB, 0.32301E+01_JPRB ,&
&  0.26181E+01_JPRB, 0.25612E+01_JPRB, 0.23560E+01_JPRB, 0.23940E+01_JPRB, 0.21381E+01_JPRB, 0.20758E+01_JPRB, 0.20011E+01_JPRB ,&
&  0.20313E+01_JPRB, 0.21602E+01_JPRB, 0.23179E+01_JPRB, 0.26688E+01_JPRB, 0.32527E+01_JPRB, 0.38291E+01_JPRB, 0.43582E+01_JPRB ,&
&  0.45147E+01_JPRB, 0.48096E+01_JPRB, 0.51127E+01_JPRB, 0.56244E+01_JPRB, 0.45551E+01_JPRB, 0.60814E+01_JPRB, 0.70397E+01_JPRB ,&
&  0.86009E+01_JPRB, 0.95436E+01_JPRB, 0.93277E+01_JPRB, 0.10341E+02_JPRB, 0.10969E+02_JPRB, 0.10183E+02_JPRB, 0.88879E+01_JPRB ,&
&  0.86646E+01_JPRB, 0.80828E+01_JPRB, 0.80138E+01_JPRB, 0.82917E+01_JPRB, 0.91361E+01_JPRB, 0.10684E+02_JPRB, 0.71258E+01_JPRB ,&
&  0.62830E+01_JPRB, 0.67493E+01_JPRB, 0.60518E+01_JPRB, 0.53490E+01_JPRB, 0.63462E+01_JPRB, 0.69632E+01_JPRB, 0.55207E+01_JPRB ,&
&  0.21962E+01_JPRB, 0.11748E+01_JPRB, 0.10318E+01_JPRB, 0.10361E+01_JPRB, 0.95015E+00_JPRB, 0.17255E+01_JPRB, 0.78951E+01_JPRB ,&
&  0.62325E+01_JPRB, 0.99097E+01_JPRB, 0.10424E+02_JPRB, 0.90467E+01_JPRB, 0.77649E+01_JPRB, 0.48261E+01_JPRB, 0.28148E+01_JPRB ,&
&  0.13986E+01_JPRB, 0.13871E+01_JPRB, 0.15676E+01_JPRB, 0.15469E+01_JPRB, 0.15408E+01_JPRB, 0.14417E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12270E+01_JPRB, 0.12266E+01_JPRB, 0.11435E+01_JPRB, 0.10580E+01_JPRB, 0.10013E+01_JPRB /)
ZBURD(:, 26,  8) = (/ &
&  0.10414E+01_JPRB, 0.10872E+01_JPRB, 0.10898E+01_JPRB, 0.10792E+01_JPRB, 0.10954E+01_JPRB, 0.11188E+01_JPRB, 0.11227E+01_JPRB ,&
&  0.12082E+01_JPRB, 0.13420E+01_JPRB, 0.14825E+01_JPRB, 0.16622E+01_JPRB, 0.17500E+01_JPRB, 0.17330E+01_JPRB, 0.17587E+01_JPRB ,&
&  0.19442E+01_JPRB, 0.23496E+01_JPRB, 0.29827E+01_JPRB, 0.35945E+01_JPRB, 0.36833E+01_JPRB, 0.33668E+01_JPRB, 0.31485E+01_JPRB ,&
&  0.26430E+01_JPRB, 0.27258E+01_JPRB, 0.25065E+01_JPRB, 0.29570E+01_JPRB, 0.34243E+01_JPRB, 0.38004E+01_JPRB, 0.28260E+01_JPRB ,&
&  0.26403E+01_JPRB, 0.25848E+01_JPRB, 0.24556E+01_JPRB, 0.24332E+01_JPRB, 0.23229E+01_JPRB, 0.22725E+01_JPRB, 0.22270E+01_JPRB ,&
&  0.22311E+01_JPRB, 0.24295E+01_JPRB, 0.27156E+01_JPRB, 0.31258E+01_JPRB, 0.36308E+01_JPRB, 0.40475E+01_JPRB, 0.43939E+01_JPRB ,&
&  0.46634E+01_JPRB, 0.49793E+01_JPRB, 0.52692E+01_JPRB, 0.49919E+01_JPRB, 0.49693E+01_JPRB, 0.56480E+01_JPRB, 0.66881E+01_JPRB ,&
&  0.79457E+01_JPRB, 0.88774E+01_JPRB, 0.92587E+01_JPRB, 0.92983E+01_JPRB, 0.94693E+01_JPRB, 0.93774E+01_JPRB, 0.86197E+01_JPRB ,&
&  0.86522E+01_JPRB, 0.86167E+01_JPRB, 0.78806E+01_JPRB, 0.79742E+01_JPRB, 0.82755E+01_JPRB, 0.86361E+01_JPRB, 0.74986E+01_JPRB ,&
&  0.65166E+01_JPRB, 0.57114E+01_JPRB, 0.53914E+01_JPRB, 0.58196E+01_JPRB, 0.62214E+01_JPRB, 0.62416E+01_JPRB, 0.72205E+01_JPRB ,&
&  0.60311E+01_JPRB, 0.22817E+01_JPRB, 0.15749E+01_JPRB, 0.20161E+01_JPRB, 0.13379E+01_JPRB, 0.16594E+01_JPRB, 0.36944E+01_JPRB ,&
&  0.61481E+01_JPRB, 0.83280E+01_JPRB, 0.83251E+01_JPRB, 0.67185E+01_JPRB, 0.57705E+01_JPRB, 0.31581E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.12317E+01_JPRB, 0.13216E+01_JPRB, 0.13329E+01_JPRB, 0.12107E+01_JPRB, 0.11682E+01_JPRB, 0.12122E+01_JPRB, 0.10784E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10985E+01_JPRB, 0.11053E+01_JPRB, 0.10710E+01_JPRB, 0.10241E+01_JPRB /)
ZBURD(:, 27,  8) = (/ &
&  0.11549E+01_JPRB, 0.11869E+01_JPRB, 0.12408E+01_JPRB, 0.12704E+01_JPRB, 0.12762E+01_JPRB, 0.13036E+01_JPRB, 0.13440E+01_JPRB ,&
&  0.14594E+01_JPRB, 0.16253E+01_JPRB, 0.18064E+01_JPRB, 0.19744E+01_JPRB, 0.20071E+01_JPRB, 0.20103E+01_JPRB, 0.21554E+01_JPRB ,&
&  0.24341E+01_JPRB, 0.28471E+01_JPRB, 0.32971E+01_JPRB, 0.37040E+01_JPRB, 0.38019E+01_JPRB, 0.37098E+01_JPRB, 0.26901E+01_JPRB ,&
&  0.21791E+01_JPRB, 0.25448E+01_JPRB, 0.23348E+01_JPRB, 0.24933E+01_JPRB, 0.27940E+01_JPRB, 0.30386E+01_JPRB, 0.29997E+01_JPRB ,&
&  0.29178E+01_JPRB, 0.28204E+01_JPRB, 0.26652E+01_JPRB, 0.26012E+01_JPRB, 0.25910E+01_JPRB, 0.26067E+01_JPRB, 0.26755E+01_JPRB ,&
&  0.28180E+01_JPRB, 0.29791E+01_JPRB, 0.32298E+01_JPRB, 0.35874E+01_JPRB, 0.39599E+01_JPRB, 0.43183E+01_JPRB, 0.46998E+01_JPRB ,&
&  0.49249E+01_JPRB, 0.50040E+01_JPRB, 0.51357E+01_JPRB, 0.47448E+01_JPRB, 0.48935E+01_JPRB, 0.54406E+01_JPRB, 0.62004E+01_JPRB ,&
&  0.66211E+01_JPRB, 0.68977E+01_JPRB, 0.79174E+01_JPRB, 0.88920E+01_JPRB, 0.90680E+01_JPRB, 0.85833E+01_JPRB, 0.79725E+01_JPRB ,&
&  0.83114E+01_JPRB, 0.81917E+01_JPRB, 0.76310E+01_JPRB, 0.69438E+01_JPRB, 0.66352E+01_JPRB, 0.67471E+01_JPRB, 0.69560E+01_JPRB ,&
&  0.63903E+01_JPRB, 0.58458E+01_JPRB, 0.55264E+01_JPRB, 0.54263E+01_JPRB, 0.54486E+01_JPRB, 0.55950E+01_JPRB, 0.61507E+01_JPRB ,&
&  0.59700E+01_JPRB, 0.47702E+01_JPRB, 0.36178E+01_JPRB, 0.24146E+01_JPRB, 0.20921E+01_JPRB, 0.23923E+01_JPRB, 0.33839E+01_JPRB ,&
&  0.53095E+01_JPRB, 0.56950E+01_JPRB, 0.53262E+01_JPRB, 0.54796E+01_JPRB, 0.36404E+01_JPRB, 0.19957E+01_JPRB, 0.12100E+01_JPRB ,&
&  0.10847E+01_JPRB, 0.11300E+01_JPRB, 0.11348E+01_JPRB, 0.10787E+01_JPRB, 0.11012E+01_JPRB, 0.11385E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.11331E+01_JPRB, 0.11807E+01_JPRB, 0.11455E+01_JPRB, 0.11090E+01_JPRB, 0.11258E+01_JPRB /)
ZBURD(:, 28,  8) = (/ &
&  0.13595E+01_JPRB, 0.14201E+01_JPRB, 0.14678E+01_JPRB, 0.14905E+01_JPRB, 0.15306E+01_JPRB, 0.15373E+01_JPRB, 0.16044E+01_JPRB ,&
&  0.17605E+01_JPRB, 0.19426E+01_JPRB, 0.21047E+01_JPRB, 0.21896E+01_JPRB, 0.22218E+01_JPRB, 0.22990E+01_JPRB, 0.25000E+01_JPRB ,&
&  0.27128E+01_JPRB, 0.29586E+01_JPRB, 0.31775E+01_JPRB, 0.34100E+01_JPRB, 0.34906E+01_JPRB, 0.35694E+01_JPRB, 0.28805E+01_JPRB ,&
&  0.19210E+01_JPRB, 0.27838E+01_JPRB, 0.23000E+01_JPRB, 0.25042E+01_JPRB, 0.27247E+01_JPRB, 0.29848E+01_JPRB, 0.30325E+01_JPRB ,&
&  0.30417E+01_JPRB, 0.30007E+01_JPRB, 0.29046E+01_JPRB, 0.28101E+01_JPRB, 0.27711E+01_JPRB, 0.28989E+01_JPRB, 0.30893E+01_JPRB ,&
&  0.32670E+01_JPRB, 0.34121E+01_JPRB, 0.35914E+01_JPRB, 0.38671E+01_JPRB, 0.41551E+01_JPRB, 0.44592E+01_JPRB, 0.47512E+01_JPRB ,&
&  0.48337E+01_JPRB, 0.49246E+01_JPRB, 0.48272E+01_JPRB, 0.46519E+01_JPRB, 0.46989E+01_JPRB, 0.50566E+01_JPRB, 0.55287E+01_JPRB ,&
&  0.58295E+01_JPRB, 0.61193E+01_JPRB, 0.69557E+01_JPRB, 0.74968E+01_JPRB, 0.74711E+01_JPRB, 0.74604E+01_JPRB, 0.67499E+01_JPRB ,&
&  0.72712E+01_JPRB, 0.67745E+01_JPRB, 0.66188E+01_JPRB, 0.56656E+01_JPRB, 0.52136E+01_JPRB, 0.51927E+01_JPRB, 0.52152E+01_JPRB ,&
&  0.49014E+01_JPRB, 0.48906E+01_JPRB, 0.45265E+01_JPRB, 0.43800E+01_JPRB, 0.47089E+01_JPRB, 0.51583E+01_JPRB, 0.49685E+01_JPRB ,&
&  0.43683E+01_JPRB, 0.30092E+01_JPRB, 0.22726E+01_JPRB, 0.20284E+01_JPRB, 0.18363E+01_JPRB, 0.20046E+01_JPRB, 0.29899E+01_JPRB ,&
&  0.42406E+01_JPRB, 0.43312E+01_JPRB, 0.43276E+01_JPRB, 0.35444E+01_JPRB, 0.20923E+01_JPRB, 0.12950E+01_JPRB, 0.10145E+01_JPRB ,&
&  0.98114E+00_JPRB, 0.10307E+01_JPRB, 0.10415E+01_JPRB, 0.10442E+01_JPRB, 0.10958E+01_JPRB, 0.10612E+01_JPRB, 0.11233E+01_JPRB ,&
&  0.11479E+01_JPRB, 0.11929E+01_JPRB, 0.12234E+01_JPRB, 0.12308E+01_JPRB, 0.12808E+01_JPRB /)
ZBURD(:, 29,  8) = (/ &
&  0.15903E+01_JPRB, 0.17318E+01_JPRB, 0.18722E+01_JPRB, 0.19665E+01_JPRB, 0.19334E+01_JPRB, 0.19046E+01_JPRB, 0.19230E+01_JPRB ,&
&  0.19688E+01_JPRB, 0.20531E+01_JPRB, 0.21532E+01_JPRB, 0.22412E+01_JPRB, 0.23126E+01_JPRB, 0.24273E+01_JPRB, 0.26105E+01_JPRB ,&
&  0.27965E+01_JPRB, 0.29637E+01_JPRB, 0.31177E+01_JPRB, 0.33054E+01_JPRB, 0.34850E+01_JPRB, 0.37537E+01_JPRB, 0.34715E+01_JPRB ,&
&  0.24548E+01_JPRB, 0.21640E+01_JPRB, 0.24987E+01_JPRB, 0.24791E+01_JPRB, 0.25190E+01_JPRB, 0.27460E+01_JPRB, 0.29256E+01_JPRB ,&
&  0.30216E+01_JPRB, 0.29842E+01_JPRB, 0.29142E+01_JPRB, 0.28561E+01_JPRB, 0.28581E+01_JPRB, 0.30510E+01_JPRB, 0.32600E+01_JPRB ,&
&  0.34413E+01_JPRB, 0.35640E+01_JPRB, 0.37094E+01_JPRB, 0.39153E+01_JPRB, 0.41397E+01_JPRB, 0.43328E+01_JPRB, 0.45712E+01_JPRB ,&
&  0.45781E+01_JPRB, 0.46072E+01_JPRB, 0.45708E+01_JPRB, 0.44313E+01_JPRB, 0.44675E+01_JPRB, 0.45495E+01_JPRB, 0.46603E+01_JPRB ,&
&  0.49239E+01_JPRB, 0.54005E+01_JPRB, 0.58721E+01_JPRB, 0.61038E+01_JPRB, 0.59289E+01_JPRB, 0.60622E+01_JPRB, 0.57326E+01_JPRB ,&
&  0.57262E+01_JPRB, 0.51970E+01_JPRB, 0.52492E+01_JPRB, 0.45703E+01_JPRB, 0.42959E+01_JPRB, 0.40092E+01_JPRB, 0.38584E+01_JPRB ,&
&  0.36118E+01_JPRB, 0.33456E+01_JPRB, 0.30142E+01_JPRB, 0.30379E+01_JPRB, 0.35042E+01_JPRB, 0.37728E+01_JPRB, 0.38527E+01_JPRB ,&
&  0.30952E+01_JPRB, 0.23288E+01_JPRB, 0.17575E+01_JPRB, 0.17927E+01_JPRB, 0.16111E+01_JPRB, 0.18078E+01_JPRB, 0.23851E+01_JPRB ,&
&  0.31152E+01_JPRB, 0.26486E+01_JPRB, 0.26023E+01_JPRB, 0.21807E+01_JPRB, 0.13861E+01_JPRB, 0.99394E+00_JPRB, 0.85911E+00_JPRB ,&
&  0.81928E+00_JPRB, 0.86862E+00_JPRB, 0.89848E+00_JPRB, 0.90341E+00_JPRB, 0.96935E+00_JPRB, 0.10245E+01_JPRB, 0.11097E+01_JPRB ,&
&  0.11405E+01_JPRB, 0.11989E+01_JPRB, 0.12661E+01_JPRB, 0.13336E+01_JPRB, 0.14467E+01_JPRB /)
ZBURD(:, 30,  8) = (/ &
&  0.15541E+01_JPRB, 0.16537E+01_JPRB, 0.17542E+01_JPRB, 0.17501E+01_JPRB, 0.17537E+01_JPRB, 0.17554E+01_JPRB, 0.17767E+01_JPRB ,&
&  0.18784E+01_JPRB, 0.19686E+01_JPRB, 0.20702E+01_JPRB, 0.21525E+01_JPRB, 0.22177E+01_JPRB, 0.23328E+01_JPRB, 0.24749E+01_JPRB ,&
&  0.25633E+01_JPRB, 0.26332E+01_JPRB, 0.27102E+01_JPRB, 0.28251E+01_JPRB, 0.29198E+01_JPRB, 0.30569E+01_JPRB, 0.31214E+01_JPRB ,&
&  0.24373E+01_JPRB, 0.16880E+01_JPRB, 0.19258E+01_JPRB, 0.21186E+01_JPRB, 0.21788E+01_JPRB, 0.24678E+01_JPRB, 0.26560E+01_JPRB ,&
&  0.26844E+01_JPRB, 0.26824E+01_JPRB, 0.27007E+01_JPRB, 0.27039E+01_JPRB, 0.27074E+01_JPRB, 0.29258E+01_JPRB, 0.31384E+01_JPRB ,&
&  0.33312E+01_JPRB, 0.34994E+01_JPRB, 0.36670E+01_JPRB, 0.38369E+01_JPRB, 0.39738E+01_JPRB, 0.40957E+01_JPRB, 0.42163E+01_JPRB ,&
&  0.42900E+01_JPRB, 0.42113E+01_JPRB, 0.41660E+01_JPRB, 0.41679E+01_JPRB, 0.41036E+01_JPRB, 0.39867E+01_JPRB, 0.38649E+01_JPRB ,&
&  0.39557E+01_JPRB, 0.41982E+01_JPRB, 0.45624E+01_JPRB, 0.48363E+01_JPRB, 0.49637E+01_JPRB, 0.47668E+01_JPRB, 0.45506E+01_JPRB ,&
&  0.41600E+01_JPRB, 0.37169E+01_JPRB, 0.38968E+01_JPRB, 0.40295E+01_JPRB, 0.31538E+01_JPRB, 0.29231E+01_JPRB, 0.26578E+01_JPRB ,&
&  0.25938E+01_JPRB, 0.22445E+01_JPRB, 0.19958E+01_JPRB, 0.20399E+01_JPRB, 0.24842E+01_JPRB, 0.25259E+01_JPRB, 0.26387E+01_JPRB ,&
&  0.22662E+01_JPRB, 0.16614E+01_JPRB, 0.15242E+01_JPRB, 0.17288E+01_JPRB, 0.17440E+01_JPRB, 0.22820E+01_JPRB, 0.21524E+01_JPRB ,&
&  0.19260E+01_JPRB, 0.16299E+01_JPRB, 0.17102E+01_JPRB, 0.16328E+01_JPRB, 0.11801E+01_JPRB, 0.89111E+00_JPRB, 0.80120E+00_JPRB ,&
&  0.76059E+00_JPRB, 0.75375E+00_JPRB, 0.79517E+00_JPRB, 0.79016E+00_JPRB, 0.84704E+00_JPRB, 0.89674E+00_JPRB, 0.10062E+01_JPRB ,&
&  0.10929E+01_JPRB, 0.11699E+01_JPRB, 0.12339E+01_JPRB, 0.13229E+01_JPRB, 0.14249E+01_JPRB /)
ZBURD(:, 31,  8) = (/ &
&  0.10988E+01_JPRB, 0.11539E+01_JPRB, 0.11608E+01_JPRB, 0.12728E+01_JPRB, 0.14009E+01_JPRB, 0.14885E+01_JPRB, 0.15676E+01_JPRB ,&
&  0.16817E+01_JPRB, 0.18261E+01_JPRB, 0.19353E+01_JPRB, 0.20053E+01_JPRB, 0.20772E+01_JPRB, 0.21781E+01_JPRB, 0.22038E+01_JPRB ,&
&  0.21741E+01_JPRB, 0.21450E+01_JPRB, 0.21312E+01_JPRB, 0.21196E+01_JPRB, 0.21307E+01_JPRB, 0.21527E+01_JPRB, 0.23382E+01_JPRB ,&
&  0.24075E+01_JPRB, 0.23326E+01_JPRB, 0.22206E+01_JPRB, 0.18279E+01_JPRB, 0.14289E+01_JPRB, 0.18482E+01_JPRB, 0.19227E+01_JPRB ,&
&  0.21404E+01_JPRB, 0.22327E+01_JPRB, 0.22394E+01_JPRB, 0.23968E+01_JPRB, 0.25505E+01_JPRB, 0.27241E+01_JPRB, 0.29222E+01_JPRB ,&
&  0.30727E+01_JPRB, 0.32342E+01_JPRB, 0.33933E+01_JPRB, 0.35184E+01_JPRB, 0.36302E+01_JPRB, 0.37563E+01_JPRB, 0.37684E+01_JPRB ,&
&  0.37525E+01_JPRB, 0.36461E+01_JPRB, 0.35709E+01_JPRB, 0.36513E+01_JPRB, 0.35984E+01_JPRB, 0.35176E+01_JPRB, 0.33183E+01_JPRB ,&
&  0.32640E+01_JPRB, 0.33311E+01_JPRB, 0.35317E+01_JPRB, 0.36473E+01_JPRB, 0.35606E+01_JPRB, 0.33365E+01_JPRB, 0.30870E+01_JPRB ,&
&  0.28434E+01_JPRB, 0.26021E+01_JPRB, 0.26978E+01_JPRB, 0.29329E+01_JPRB, 0.21729E+01_JPRB, 0.22246E+01_JPRB, 0.22649E+01_JPRB ,&
&  0.19526E+01_JPRB, 0.17235E+01_JPRB, 0.15847E+01_JPRB, 0.14758E+01_JPRB, 0.16790E+01_JPRB, 0.18173E+01_JPRB, 0.20905E+01_JPRB ,&
&  0.18185E+01_JPRB, 0.13620E+01_JPRB, 0.13303E+01_JPRB, 0.15975E+01_JPRB, 0.18098E+01_JPRB, 0.22431E+01_JPRB, 0.17913E+01_JPRB ,&
&  0.13759E+01_JPRB, 0.14423E+01_JPRB, 0.15456E+01_JPRB, 0.17566E+01_JPRB, 0.14363E+01_JPRB, 0.86075E+00_JPRB, 0.73973E+00_JPRB ,&
&  0.71187E+00_JPRB, 0.71452E+00_JPRB, 0.72400E+00_JPRB, 0.72307E+00_JPRB, 0.75666E+00_JPRB, 0.79395E+00_JPRB, 0.80844E+00_JPRB ,&
&  0.82350E+00_JPRB, 0.83288E+00_JPRB, 0.86961E+00_JPRB, 0.93240E+00_JPRB, 0.10276E+01_JPRB /)
ZBURD(:, 32,  8) = (/ &
&  0.72759E+00_JPRB, 0.72989E+00_JPRB, 0.71472E+00_JPRB, 0.74144E+00_JPRB, 0.79492E+00_JPRB, 0.90587E+00_JPRB, 0.10128E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.12744E+01_JPRB, 0.14669E+01_JPRB, 0.16342E+01_JPRB, 0.16878E+01_JPRB, 0.16577E+01_JPRB, 0.15079E+01_JPRB ,&
&  0.13073E+01_JPRB, 0.10997E+01_JPRB, 0.97606E+00_JPRB, 0.93203E+00_JPRB, 0.98118E+00_JPRB, 0.10068E+01_JPRB, 0.10823E+01_JPRB ,&
&  0.12116E+01_JPRB, 0.14231E+01_JPRB, 0.16436E+01_JPRB, 0.21428E+01_JPRB, 0.24202E+01_JPRB, 0.17845E+01_JPRB, 0.16923E+01_JPRB ,&
&  0.19962E+01_JPRB, 0.20875E+01_JPRB, 0.17516E+01_JPRB, 0.17791E+01_JPRB, 0.19763E+01_JPRB, 0.21417E+01_JPRB, 0.23536E+01_JPRB ,&
&  0.24718E+01_JPRB, 0.25882E+01_JPRB, 0.26735E+01_JPRB, 0.27141E+01_JPRB, 0.27354E+01_JPRB, 0.27044E+01_JPRB, 0.26374E+01_JPRB ,&
&  0.25426E+01_JPRB, 0.24244E+01_JPRB, 0.24367E+01_JPRB, 0.25406E+01_JPRB, 0.26726E+01_JPRB, 0.27004E+01_JPRB, 0.26087E+01_JPRB ,&
&  0.25256E+01_JPRB, 0.25077E+01_JPRB, 0.26954E+01_JPRB, 0.26707E+01_JPRB, 0.24214E+01_JPRB, 0.22665E+01_JPRB, 0.19803E+01_JPRB ,&
&  0.17136E+01_JPRB, 0.14090E+01_JPRB, 0.11993E+01_JPRB, 0.18736E+01_JPRB, 0.21487E+01_JPRB, 0.18763E+01_JPRB, 0.16399E+01_JPRB ,&
&  0.15396E+01_JPRB, 0.14344E+01_JPRB, 0.13349E+01_JPRB, 0.13000E+01_JPRB, 0.13883E+01_JPRB, 0.18288E+01_JPRB, 0.18029E+01_JPRB ,&
&  0.15625E+01_JPRB, 0.11444E+01_JPRB, 0.10443E+01_JPRB, 0.11656E+01_JPRB, 0.13256E+01_JPRB, 0.15689E+01_JPRB, 0.12945E+01_JPRB ,&
&  0.11061E+01_JPRB, 0.13609E+01_JPRB, 0.15053E+01_JPRB, 0.19336E+01_JPRB, 0.21922E+01_JPRB, 0.72589E+00_JPRB, 0.68588E+00_JPRB ,&
&  0.66788E+00_JPRB, 0.66055E+00_JPRB, 0.67737E+00_JPRB, 0.66888E+00_JPRB, 0.67144E+00_JPRB, 0.65947E+00_JPRB, 0.64116E+00_JPRB ,&
&  0.63741E+00_JPRB, 0.65042E+00_JPRB, 0.66840E+00_JPRB, 0.68020E+00_JPRB, 0.70864E+00_JPRB /)
ZBURD(:, 33,  8) = (/ &
&  0.66735E+00_JPRB, 0.66535E+00_JPRB, 0.66777E+00_JPRB, 0.67532E+00_JPRB, 0.68002E+00_JPRB, 0.68430E+00_JPRB, 0.70765E+00_JPRB ,&
&  0.74676E+00_JPRB, 0.80835E+00_JPRB, 0.89431E+00_JPRB, 0.95008E+00_JPRB, 0.93143E+00_JPRB, 0.86345E+00_JPRB, 0.82882E+00_JPRB ,&
&  0.81578E+00_JPRB, 0.80638E+00_JPRB, 0.83631E+00_JPRB, 0.88893E+00_JPRB, 0.95638E+00_JPRB, 0.97702E+00_JPRB, 0.10402E+01_JPRB ,&
&  0.10392E+01_JPRB, 0.10721E+01_JPRB, 0.11728E+01_JPRB, 0.12673E+01_JPRB, 0.18923E+01_JPRB, 0.18097E+01_JPRB, 0.14924E+01_JPRB ,&
&  0.15116E+01_JPRB, 0.15272E+01_JPRB, 0.15083E+01_JPRB, 0.13647E+01_JPRB, 0.13570E+01_JPRB, 0.13218E+01_JPRB, 0.13619E+01_JPRB ,&
&  0.13451E+01_JPRB, 0.14359E+01_JPRB, 0.15037E+01_JPRB, 0.15806E+01_JPRB, 0.16294E+01_JPRB, 0.16292E+01_JPRB, 0.16561E+01_JPRB ,&
&  0.16679E+01_JPRB, 0.16800E+01_JPRB, 0.16925E+01_JPRB, 0.15367E+01_JPRB, 0.16391E+01_JPRB, 0.17187E+01_JPRB, 0.17375E+01_JPRB ,&
&  0.16988E+01_JPRB, 0.16809E+01_JPRB, 0.17712E+01_JPRB, 0.16644E+01_JPRB, 0.15387E+01_JPRB, 0.13671E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.10661E+01_JPRB, 0.81226E+00_JPRB, 0.63337E+00_JPRB, 0.95407E+00_JPRB, 0.12228E+01_JPRB, 0.11219E+01_JPRB, 0.12336E+01_JPRB ,&
&  0.12145E+01_JPRB, 0.11752E+01_JPRB, 0.11329E+01_JPRB, 0.11307E+01_JPRB, 0.11226E+01_JPRB, 0.13520E+01_JPRB, 0.14522E+01_JPRB ,&
&  0.12510E+01_JPRB, 0.96478E+00_JPRB, 0.84489E+00_JPRB, 0.85550E+00_JPRB, 0.93074E+00_JPRB, 0.10554E+01_JPRB, 0.12117E+01_JPRB ,&
&  0.11924E+01_JPRB, 0.99361E+00_JPRB, 0.11092E+01_JPRB, 0.13302E+01_JPRB, 0.85152E+00_JPRB, 0.57920E+00_JPRB, 0.56008E+00_JPRB ,&
&  0.55653E+00_JPRB, 0.56585E+00_JPRB, 0.58690E+00_JPRB, 0.59959E+00_JPRB, 0.60718E+00_JPRB, 0.61130E+00_JPRB, 0.62318E+00_JPRB ,&
&  0.63134E+00_JPRB, 0.64236E+00_JPRB, 0.64729E+00_JPRB, 0.65498E+00_JPRB, 0.66354E+00_JPRB /)
ZBURD(:, 34,  8) = (/ &
&  0.84498E+00_JPRB, 0.86044E+00_JPRB, 0.88283E+00_JPRB, 0.90682E+00_JPRB, 0.92246E+00_JPRB, 0.94659E+00_JPRB, 0.97287E+00_JPRB ,&
&  0.10077E+01_JPRB, 0.10533E+01_JPRB, 0.10825E+01_JPRB, 0.10938E+01_JPRB, 0.11083E+01_JPRB, 0.11246E+01_JPRB, 0.11296E+01_JPRB ,&
&  0.11526E+01_JPRB, 0.11636E+01_JPRB, 0.12234E+01_JPRB, 0.12519E+01_JPRB, 0.12908E+01_JPRB, 0.13420E+01_JPRB, 0.13512E+01_JPRB ,&
&  0.13426E+01_JPRB, 0.13327E+01_JPRB, 0.13474E+01_JPRB, 0.13454E+01_JPRB, 0.14314E+01_JPRB, 0.14471E+01_JPRB, 0.13821E+01_JPRB ,&
&  0.12339E+01_JPRB, 0.11742E+01_JPRB, 0.13537E+01_JPRB, 0.11297E+01_JPRB, 0.11182E+01_JPRB, 0.11090E+01_JPRB, 0.10085E+01_JPRB ,&
&  0.99224E+00_JPRB, 0.10403E+01_JPRB, 0.11006E+01_JPRB, 0.11646E+01_JPRB, 0.12446E+01_JPRB, 0.12622E+01_JPRB, 0.12578E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.12918E+01_JPRB, 0.13173E+01_JPRB, 0.12647E+01_JPRB, 0.12258E+01_JPRB, 0.13473E+01_JPRB, 0.13711E+01_JPRB ,&
&  0.13377E+01_JPRB, 0.13081E+01_JPRB, 0.10771E+01_JPRB, 0.99085E+00_JPRB, 0.99564E+00_JPRB, 0.88242E+00_JPRB, 0.83155E+00_JPRB ,&
&  0.87350E+00_JPRB, 0.78303E+00_JPRB, 0.54832E+00_JPRB, 0.74430E+00_JPRB, 0.92720E+00_JPRB, 0.10180E+01_JPRB, 0.10586E+01_JPRB ,&
&  0.10201E+01_JPRB, 0.92299E+00_JPRB, 0.90384E+00_JPRB, 0.96571E+00_JPRB, 0.10241E+01_JPRB, 0.10060E+01_JPRB, 0.10353E+01_JPRB ,&
&  0.94495E+00_JPRB, 0.83847E+00_JPRB, 0.77460E+00_JPRB, 0.86467E+00_JPRB, 0.10922E+01_JPRB, 0.15180E+01_JPRB, 0.14195E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.94800E+00_JPRB, 0.13753E+01_JPRB, 0.18585E+01_JPRB, 0.81081E+00_JPRB, 0.61472E+00_JPRB, 0.54734E+00_JPRB ,&
&  0.57612E+00_JPRB, 0.59053E+00_JPRB, 0.60000E+00_JPRB, 0.62338E+00_JPRB, 0.65932E+00_JPRB, 0.67508E+00_JPRB, 0.68001E+00_JPRB ,&
&  0.70065E+00_JPRB, 0.73356E+00_JPRB, 0.75837E+00_JPRB, 0.79000E+00_JPRB, 0.81825E+00_JPRB /)
ZBURD(:, 35,  8) = (/ &
&  0.99102E+00_JPRB, 0.10148E+01_JPRB, 0.10322E+01_JPRB, 0.10508E+01_JPRB, 0.10752E+01_JPRB, 0.11001E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.11403E+01_JPRB, 0.11597E+01_JPRB, 0.11683E+01_JPRB, 0.11877E+01_JPRB, 0.11944E+01_JPRB, 0.12495E+01_JPRB, 0.12711E+01_JPRB ,&
&  0.13172E+01_JPRB, 0.13439E+01_JPRB, 0.13530E+01_JPRB, 0.13557E+01_JPRB, 0.13689E+01_JPRB, 0.13654E+01_JPRB, 0.13607E+01_JPRB ,&
&  0.13857E+01_JPRB, 0.14332E+01_JPRB, 0.14867E+01_JPRB, 0.15367E+01_JPRB, 0.15636E+01_JPRB, 0.16562E+01_JPRB, 0.17804E+01_JPRB ,&
&  0.13319E+01_JPRB, 0.12949E+01_JPRB, 0.13153E+01_JPRB, 0.11436E+01_JPRB, 0.11566E+01_JPRB, 0.11657E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11252E+01_JPRB, 0.11511E+01_JPRB, 0.11718E+01_JPRB, 0.11924E+01_JPRB, 0.12169E+01_JPRB, 0.12122E+01_JPRB ,&
&  0.12333E+01_JPRB, 0.12519E+01_JPRB, 0.12841E+01_JPRB, 0.13310E+01_JPRB, 0.12702E+01_JPRB, 0.13962E+01_JPRB, 0.13954E+01_JPRB ,&
&  0.13660E+01_JPRB, 0.12433E+01_JPRB, 0.94342E+00_JPRB, 0.88299E+00_JPRB, 0.89362E+00_JPRB, 0.84906E+00_JPRB, 0.80328E+00_JPRB ,&
&  0.85790E+00_JPRB, 0.86934E+00_JPRB, 0.73922E+00_JPRB, 0.86617E+00_JPRB, 0.94694E+00_JPRB, 0.98153E+00_JPRB, 0.93797E+00_JPRB ,&
&  0.87567E+00_JPRB, 0.77922E+00_JPRB, 0.75404E+00_JPRB, 0.77983E+00_JPRB, 0.79643E+00_JPRB, 0.79345E+00_JPRB, 0.82646E+00_JPRB ,&
&  0.82408E+00_JPRB, 0.79686E+00_JPRB, 0.81281E+00_JPRB, 0.80827E+00_JPRB, 0.11473E+01_JPRB, 0.12959E+01_JPRB, 0.18325E+01_JPRB ,&
&  0.92216E+00_JPRB, 0.87299E+00_JPRB, 0.80214E+00_JPRB, 0.10980E+01_JPRB, 0.76622E+00_JPRB, 0.56547E+00_JPRB, 0.58550E+00_JPRB ,&
&  0.64925E+00_JPRB, 0.67848E+00_JPRB, 0.68627E+00_JPRB, 0.70417E+00_JPRB, 0.75049E+00_JPRB, 0.77685E+00_JPRB, 0.80053E+00_JPRB ,&
&  0.80788E+00_JPRB, 0.84667E+00_JPRB, 0.89933E+00_JPRB, 0.93956E+00_JPRB, 0.95805E+00_JPRB /)
ZBURD(:, 36,  8) = (/ &
&  0.10504E+01_JPRB, 0.10796E+01_JPRB, 0.11048E+01_JPRB, 0.11278E+01_JPRB, 0.11495E+01_JPRB, 0.11736E+01_JPRB, 0.11954E+01_JPRB ,&
&  0.12148E+01_JPRB, 0.12284E+01_JPRB, 0.12361E+01_JPRB, 0.12510E+01_JPRB, 0.12651E+01_JPRB, 0.13063E+01_JPRB, 0.13242E+01_JPRB ,&
&  0.13648E+01_JPRB, 0.13805E+01_JPRB, 0.13782E+01_JPRB, 0.14000E+01_JPRB, 0.13883E+01_JPRB, 0.13649E+01_JPRB, 0.13612E+01_JPRB ,&
&  0.13694E+01_JPRB, 0.13678E+01_JPRB, 0.13669E+01_JPRB, 0.13703E+01_JPRB, 0.13738E+01_JPRB, 0.13436E+01_JPRB, 0.14274E+01_JPRB ,&
&  0.10778E+01_JPRB, 0.14388E+01_JPRB, 0.14885E+01_JPRB, 0.13337E+01_JPRB, 0.12654E+01_JPRB, 0.11638E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11582E+01_JPRB, 0.11658E+01_JPRB, 0.11896E+01_JPRB, 0.12186E+01_JPRB, 0.12497E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.12800E+01_JPRB, 0.12873E+01_JPRB, 0.12965E+01_JPRB, 0.13103E+01_JPRB, 0.13458E+01_JPRB, 0.13353E+01_JPRB, 0.13024E+01_JPRB ,&
&  0.12827E+01_JPRB, 0.12700E+01_JPRB, 0.10114E+01_JPRB, 0.96932E+00_JPRB, 0.10162E+01_JPRB, 0.97473E+00_JPRB, 0.88079E+00_JPRB ,&
&  0.88145E+00_JPRB, 0.89820E+00_JPRB, 0.93518E+00_JPRB, 0.93275E+00_JPRB, 0.93641E+00_JPRB, 0.91083E+00_JPRB, 0.85821E+00_JPRB ,&
&  0.74352E+00_JPRB, 0.64648E+00_JPRB, 0.62320E+00_JPRB, 0.66205E+00_JPRB, 0.68087E+00_JPRB, 0.68489E+00_JPRB, 0.65991E+00_JPRB ,&
&  0.65469E+00_JPRB, 0.66657E+00_JPRB, 0.70793E+00_JPRB, 0.80054E+00_JPRB, 0.90512E+00_JPRB, 0.16111E+01_JPRB, 0.18371E+01_JPRB ,&
&  0.90735E+00_JPRB, 0.81136E+00_JPRB, 0.60607E+00_JPRB, 0.75272E+00_JPRB, 0.75980E+00_JPRB, 0.67891E+00_JPRB, 0.70730E+00_JPRB ,&
&  0.81330E+00_JPRB, 0.86922E+00_JPRB, 0.89033E+00_JPRB, 0.82769E+00_JPRB, 0.84441E+00_JPRB, 0.83035E+00_JPRB, 0.83619E+00_JPRB ,&
&  0.85914E+00_JPRB, 0.89494E+00_JPRB, 0.93915E+00_JPRB, 0.98122E+00_JPRB, 0.10171E+01_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART15',1,ZHOOK_HANDLE)
END SUBROUTINE PART15



SUBROUTINE PART16

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART16',0,ZHOOK_HANDLE)


ZBURD(:, 37,  8) = (/ &
&  0.10686E+01_JPRB, 0.10969E+01_JPRB, 0.11255E+01_JPRB, 0.11574E+01_JPRB, 0.11898E+01_JPRB, 0.12134E+01_JPRB, 0.12385E+01_JPRB ,&
&  0.12641E+01_JPRB, 0.12836E+01_JPRB, 0.12905E+01_JPRB, 0.12977E+01_JPRB, 0.13117E+01_JPRB, 0.13511E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.13752E+01_JPRB, 0.13935E+01_JPRB, 0.14044E+01_JPRB, 0.14132E+01_JPRB, 0.14017E+01_JPRB, 0.13686E+01_JPRB, 0.13788E+01_JPRB ,&
&  0.13739E+01_JPRB, 0.13775E+01_JPRB, 0.13794E+01_JPRB, 0.14045E+01_JPRB, 0.14442E+01_JPRB, 0.15429E+01_JPRB, 0.13123E+01_JPRB ,&
&  0.14273E+01_JPRB, 0.15374E+01_JPRB, 0.15310E+01_JPRB, 0.14112E+01_JPRB, 0.12732E+01_JPRB, 0.12347E+01_JPRB, 0.12259E+01_JPRB ,&
&  0.12336E+01_JPRB, 0.12065E+01_JPRB, 0.12018E+01_JPRB, 0.12021E+01_JPRB, 0.12138E+01_JPRB, 0.12436E+01_JPRB, 0.12507E+01_JPRB ,&
&  0.12594E+01_JPRB, 0.12809E+01_JPRB, 0.13076E+01_JPRB, 0.13428E+01_JPRB, 0.13852E+01_JPRB, 0.14148E+01_JPRB, 0.14304E+01_JPRB ,&
&  0.14433E+01_JPRB, 0.14510E+01_JPRB, 0.13213E+01_JPRB, 0.13021E+01_JPRB, 0.13433E+01_JPRB, 0.12318E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.98881E+00_JPRB, 0.90506E+00_JPRB, 0.85389E+00_JPRB, 0.98906E+00_JPRB, 0.94215E+00_JPRB, 0.86883E+00_JPRB, 0.77225E+00_JPRB ,&
&  0.65597E+00_JPRB, 0.59210E+00_JPRB, 0.59401E+00_JPRB, 0.63634E+00_JPRB, 0.63212E+00_JPRB, 0.63835E+00_JPRB, 0.65822E+00_JPRB ,&
&  0.69098E+00_JPRB, 0.72521E+00_JPRB, 0.77972E+00_JPRB, 0.88420E+00_JPRB, 0.10160E+01_JPRB, 0.10069E+01_JPRB, 0.11525E+01_JPRB ,&
&  0.89271E+00_JPRB, 0.74055E+00_JPRB, 0.79352E+00_JPRB, 0.85628E+00_JPRB, 0.88506E+00_JPRB, 0.87364E+00_JPRB, 0.97517E+00_JPRB ,&
&  0.11212E+01_JPRB, 0.12587E+01_JPRB, 0.12685E+01_JPRB, 0.10530E+01_JPRB, 0.94190E+00_JPRB, 0.89495E+00_JPRB, 0.89074E+00_JPRB ,&
&  0.90331E+00_JPRB, 0.92551E+00_JPRB, 0.96179E+00_JPRB, 0.10034E+01_JPRB, 0.10390E+01_JPRB /)
ZBURD(:, 38,  8) = (/ &
&  0.10636E+01_JPRB, 0.10930E+01_JPRB, 0.11229E+01_JPRB, 0.11518E+01_JPRB, 0.11745E+01_JPRB, 0.11924E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.12264E+01_JPRB, 0.12481E+01_JPRB, 0.12636E+01_JPRB, 0.12714E+01_JPRB, 0.12798E+01_JPRB, 0.13035E+01_JPRB, 0.13351E+01_JPRB ,&
&  0.13735E+01_JPRB, 0.13965E+01_JPRB, 0.14387E+01_JPRB, 0.14268E+01_JPRB, 0.14133E+01_JPRB, 0.14220E+01_JPRB, 0.14564E+01_JPRB ,&
&  0.14686E+01_JPRB, 0.14641E+01_JPRB, 0.14607E+01_JPRB, 0.14951E+01_JPRB, 0.15327E+01_JPRB, 0.15955E+01_JPRB, 0.13972E+01_JPRB ,&
&  0.15415E+01_JPRB, 0.16353E+01_JPRB, 0.15867E+01_JPRB, 0.15080E+01_JPRB, 0.13731E+01_JPRB, 0.13283E+01_JPRB, 0.13122E+01_JPRB ,&
&  0.12952E+01_JPRB, 0.12788E+01_JPRB, 0.12511E+01_JPRB, 0.12341E+01_JPRB, 0.12287E+01_JPRB, 0.12194E+01_JPRB, 0.12346E+01_JPRB ,&
&  0.12499E+01_JPRB, 0.12632E+01_JPRB, 0.13012E+01_JPRB, 0.13499E+01_JPRB, 0.14258E+01_JPRB, 0.14988E+01_JPRB, 0.15415E+01_JPRB ,&
&  0.15984E+01_JPRB, 0.16538E+01_JPRB, 0.15555E+01_JPRB, 0.15512E+01_JPRB, 0.16244E+01_JPRB, 0.14789E+01_JPRB, 0.12061E+01_JPRB ,&
&  0.94645E+00_JPRB, 0.90150E+00_JPRB, 0.93253E+00_JPRB, 0.10203E+01_JPRB, 0.92866E+00_JPRB, 0.83103E+00_JPRB, 0.77331E+00_JPRB ,&
&  0.68710E+00_JPRB, 0.61420E+00_JPRB, 0.62788E+00_JPRB, 0.62445E+00_JPRB, 0.61641E+00_JPRB, 0.64579E+00_JPRB, 0.68337E+00_JPRB ,&
&  0.72842E+00_JPRB, 0.80557E+00_JPRB, 0.88594E+00_JPRB, 0.97380E+00_JPRB, 0.10321E+01_JPRB, 0.99263E+00_JPRB, 0.10667E+01_JPRB ,&
&  0.99970E+00_JPRB, 0.97522E+00_JPRB, 0.99261E+00_JPRB, 0.81044E+00_JPRB, 0.10827E+01_JPRB, 0.11514E+01_JPRB, 0.12697E+01_JPRB ,&
&  0.12383E+01_JPRB, 0.16533E+01_JPRB, 0.20259E+01_JPRB, 0.15864E+01_JPRB, 0.12617E+01_JPRB, 0.94464E+00_JPRB, 0.89439E+00_JPRB ,&
&  0.90425E+00_JPRB, 0.92211E+00_JPRB, 0.95373E+00_JPRB, 0.99367E+00_JPRB, 0.10299E+01_JPRB /)
ZBURD(:, 39,  8) = (/ &
&  0.10457E+01_JPRB, 0.10801E+01_JPRB, 0.11170E+01_JPRB, 0.11425E+01_JPRB, 0.11582E+01_JPRB, 0.11476E+01_JPRB, 0.11376E+01_JPRB ,&
&  0.11400E+01_JPRB, 0.11514E+01_JPRB, 0.11699E+01_JPRB, 0.11898E+01_JPRB, 0.12033E+01_JPRB, 0.12172E+01_JPRB, 0.12418E+01_JPRB ,&
&  0.12724E+01_JPRB, 0.13111E+01_JPRB, 0.13599E+01_JPRB, 0.13990E+01_JPRB, 0.14417E+01_JPRB, 0.14772E+01_JPRB, 0.15241E+01_JPRB ,&
&  0.15643E+01_JPRB, 0.16219E+01_JPRB, 0.16355E+01_JPRB, 0.16690E+01_JPRB, 0.16864E+01_JPRB, 0.17405E+01_JPRB, 0.14048E+01_JPRB ,&
&  0.15860E+01_JPRB, 0.16457E+01_JPRB, 0.16272E+01_JPRB, 0.15430E+01_JPRB, 0.14156E+01_JPRB, 0.13528E+01_JPRB, 0.13107E+01_JPRB ,&
&  0.12915E+01_JPRB, 0.12555E+01_JPRB, 0.11897E+01_JPRB, 0.12249E+01_JPRB, 0.12500E+01_JPRB, 0.11939E+01_JPRB, 0.12160E+01_JPRB ,&
&  0.12385E+01_JPRB, 0.12674E+01_JPRB, 0.13002E+01_JPRB, 0.13624E+01_JPRB, 0.14523E+01_JPRB, 0.15322E+01_JPRB, 0.16101E+01_JPRB ,&
&  0.16921E+01_JPRB, 0.17804E+01_JPRB, 0.18538E+01_JPRB, 0.16751E+01_JPRB, 0.16664E+01_JPRB, 0.15331E+01_JPRB, 0.12426E+01_JPRB ,&
&  0.96256E+00_JPRB, 0.89008E+00_JPRB, 0.93990E+00_JPRB, 0.10258E+01_JPRB, 0.92908E+00_JPRB, 0.79963E+00_JPRB, 0.74952E+00_JPRB ,&
&  0.68836E+00_JPRB, 0.65217E+00_JPRB, 0.64603E+00_JPRB, 0.62289E+00_JPRB, 0.61475E+00_JPRB, 0.64996E+00_JPRB, 0.69323E+00_JPRB ,&
&  0.74810E+00_JPRB, 0.82319E+00_JPRB, 0.91231E+00_JPRB, 0.10066E+01_JPRB, 0.11046E+01_JPRB, 0.11372E+01_JPRB, 0.10463E+01_JPRB ,&
&  0.10528E+01_JPRB, 0.10344E+01_JPRB, 0.10837E+01_JPRB, 0.11869E+01_JPRB, 0.12398E+01_JPRB, 0.14646E+01_JPRB, 0.17023E+01_JPRB ,&
&  0.19250E+01_JPRB, 0.17929E+01_JPRB, 0.21133E+01_JPRB, 0.29229E+01_JPRB, 0.26905E+01_JPRB, 0.21910E+01_JPRB, 0.86652E+00_JPRB ,&
&  0.86387E+00_JPRB, 0.86614E+00_JPRB, 0.90416E+00_JPRB, 0.95331E+00_JPRB, 0.10104E+01_JPRB /)
ZBURD(:, 40,  8) = (/ &
&  0.86946E+00_JPRB, 0.92432E+00_JPRB, 0.97952E+00_JPRB, 0.10501E+01_JPRB, 0.11093E+01_JPRB, 0.11063E+01_JPRB, 0.10825E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10622E+01_JPRB, 0.10729E+01_JPRB, 0.10923E+01_JPRB, 0.11083E+01_JPRB, 0.11195E+01_JPRB, 0.11359E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.11913E+01_JPRB, 0.12271E+01_JPRB, 0.12678E+01_JPRB, 0.13050E+01_JPRB, 0.13586E+01_JPRB, 0.14204E+01_JPRB ,&
&  0.14864E+01_JPRB, 0.15599E+01_JPRB, 0.16206E+01_JPRB, 0.16804E+01_JPRB, 0.17339E+01_JPRB, 0.17281E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.15567E+01_JPRB, 0.15957E+01_JPRB, 0.16213E+01_JPRB, 0.15431E+01_JPRB, 0.14093E+01_JPRB, 0.13268E+01_JPRB, 0.12950E+01_JPRB ,&
&  0.12647E+01_JPRB, 0.12199E+01_JPRB, 0.11657E+01_JPRB, 0.11273E+01_JPRB, 0.12012E+01_JPRB, 0.11508E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.12242E+01_JPRB, 0.12586E+01_JPRB, 0.12895E+01_JPRB, 0.13401E+01_JPRB, 0.14322E+01_JPRB, 0.15221E+01_JPRB, 0.16253E+01_JPRB ,&
&  0.17199E+01_JPRB, 0.17989E+01_JPRB, 0.18434E+01_JPRB, 0.16185E+01_JPRB, 0.15459E+01_JPRB, 0.14601E+01_JPRB, 0.11319E+01_JPRB ,&
&  0.96729E+00_JPRB, 0.94210E+00_JPRB, 0.10187E+01_JPRB, 0.10356E+01_JPRB, 0.94171E+00_JPRB, 0.82729E+00_JPRB, 0.75855E+00_JPRB ,&
&  0.68132E+00_JPRB, 0.64541E+00_JPRB, 0.64017E+00_JPRB, 0.62527E+00_JPRB, 0.62356E+00_JPRB, 0.65732E+00_JPRB, 0.70247E+00_JPRB ,&
&  0.75294E+00_JPRB, 0.81808E+00_JPRB, 0.87372E+00_JPRB, 0.92379E+00_JPRB, 0.98542E+00_JPRB, 0.10880E+01_JPRB, 0.11498E+01_JPRB ,&
&  0.10352E+01_JPRB, 0.96697E+00_JPRB, 0.99696E+00_JPRB, 0.10315E+01_JPRB, 0.10726E+01_JPRB, 0.11476E+01_JPRB, 0.12267E+01_JPRB ,&
&  0.13552E+01_JPRB, 0.14262E+01_JPRB, 0.12087E+01_JPRB, 0.10127E+01_JPRB, 0.13235E+01_JPRB, 0.14122E+01_JPRB, 0.74924E+00_JPRB ,&
&  0.71825E+00_JPRB, 0.73737E+00_JPRB, 0.75173E+00_JPRB, 0.78859E+00_JPRB, 0.82747E+00_JPRB /)
ZBURD(:, 41,  8) = (/ &
&  0.69601E+00_JPRB, 0.71395E+00_JPRB, 0.78955E+00_JPRB, 0.81353E+00_JPRB, 0.88840E+00_JPRB, 0.92015E+00_JPRB, 0.95402E+00_JPRB ,&
&  0.96241E+00_JPRB, 0.96697E+00_JPRB, 0.96974E+00_JPRB, 0.97645E+00_JPRB, 0.98491E+00_JPRB, 0.99883E+00_JPRB, 0.10114E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.10512E+01_JPRB, 0.10794E+01_JPRB, 0.11096E+01_JPRB, 0.11499E+01_JPRB, 0.11986E+01_JPRB, 0.12623E+01_JPRB ,&
&  0.13417E+01_JPRB, 0.14346E+01_JPRB, 0.15344E+01_JPRB, 0.16422E+01_JPRB, 0.17554E+01_JPRB, 0.18403E+01_JPRB, 0.18604E+01_JPRB ,&
&  0.13128E+01_JPRB, 0.15909E+01_JPRB, 0.15736E+01_JPRB, 0.14998E+01_JPRB, 0.14015E+01_JPRB, 0.13362E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.12880E+01_JPRB, 0.12092E+01_JPRB, 0.11826E+01_JPRB, 0.11530E+01_JPRB, 0.11347E+01_JPRB, 0.10985E+01_JPRB, 0.11234E+01_JPRB ,&
&  0.11557E+01_JPRB, 0.12021E+01_JPRB, 0.12526E+01_JPRB, 0.12977E+01_JPRB, 0.13807E+01_JPRB, 0.14886E+01_JPRB, 0.16060E+01_JPRB ,&
&  0.17274E+01_JPRB, 0.18364E+01_JPRB, 0.19109E+01_JPRB, 0.16942E+01_JPRB, 0.15087E+01_JPRB, 0.14443E+01_JPRB, 0.11684E+01_JPRB ,&
&  0.10271E+01_JPRB, 0.95538E+00_JPRB, 0.10374E+01_JPRB, 0.10723E+01_JPRB, 0.97527E+00_JPRB, 0.87952E+00_JPRB, 0.80599E+00_JPRB ,&
&  0.73170E+00_JPRB, 0.67349E+00_JPRB, 0.65590E+00_JPRB, 0.64901E+00_JPRB, 0.65298E+00_JPRB, 0.67816E+00_JPRB, 0.70148E+00_JPRB ,&
&  0.73136E+00_JPRB, 0.76784E+00_JPRB, 0.79990E+00_JPRB, 0.83650E+00_JPRB, 0.85830E+00_JPRB, 0.87981E+00_JPRB, 0.88732E+00_JPRB ,&
&  0.89982E+00_JPRB, 0.93477E+00_JPRB, 0.93383E+00_JPRB, 0.95229E+00_JPRB, 0.94889E+00_JPRB, 0.96745E+00_JPRB, 0.99259E+00_JPRB ,&
&  0.10165E+01_JPRB, 0.10397E+01_JPRB, 0.10354E+01_JPRB, 0.10030E+01_JPRB, 0.93675E+00_JPRB, 0.85699E+00_JPRB, 0.69807E+00_JPRB ,&
&  0.67279E+00_JPRB, 0.65397E+00_JPRB, 0.66215E+00_JPRB, 0.66880E+00_JPRB, 0.68713E+00_JPRB /)
ZBURD(:, 42,  8) = (/ &
&  0.60081E+00_JPRB, 0.61796E+00_JPRB, 0.63660E+00_JPRB, 0.65930E+00_JPRB, 0.69377E+00_JPRB, 0.73391E+00_JPRB, 0.78547E+00_JPRB ,&
&  0.83820E+00_JPRB, 0.85457E+00_JPRB, 0.85882E+00_JPRB, 0.86091E+00_JPRB, 0.86542E+00_JPRB, 0.87840E+00_JPRB, 0.89553E+00_JPRB ,&
&  0.91219E+00_JPRB, 0.92820E+00_JPRB, 0.94391E+00_JPRB, 0.96227E+00_JPRB, 0.98580E+00_JPRB, 0.10221E+01_JPRB, 0.10707E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.12231E+01_JPRB, 0.13513E+01_JPRB, 0.14964E+01_JPRB, 0.16513E+01_JPRB, 0.17749E+01_JPRB, 0.18116E+01_JPRB ,&
&  0.10742E+01_JPRB, 0.13589E+01_JPRB, 0.16138E+01_JPRB, 0.15349E+01_JPRB, 0.14314E+01_JPRB, 0.13768E+01_JPRB, 0.13596E+01_JPRB ,&
&  0.13008E+01_JPRB, 0.12130E+01_JPRB, 0.11806E+01_JPRB, 0.12138E+01_JPRB, 0.11041E+01_JPRB, 0.10666E+01_JPRB, 0.10687E+01_JPRB ,&
&  0.10836E+01_JPRB, 0.11053E+01_JPRB, 0.11467E+01_JPRB, 0.11991E+01_JPRB, 0.12776E+01_JPRB, 0.13958E+01_JPRB, 0.15292E+01_JPRB ,&
&  0.16798E+01_JPRB, 0.18334E+01_JPRB, 0.19845E+01_JPRB, 0.17607E+01_JPRB, 0.17230E+01_JPRB, 0.17492E+01_JPRB, 0.16157E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.10694E+01_JPRB, 0.11107E+01_JPRB, 0.11336E+01_JPRB, 0.10214E+01_JPRB, 0.94430E+00_JPRB, 0.87697E+00_JPRB ,&
&  0.81820E+00_JPRB, 0.75940E+00_JPRB, 0.69967E+00_JPRB, 0.68459E+00_JPRB, 0.68819E+00_JPRB, 0.70626E+00_JPRB, 0.72848E+00_JPRB ,&
&  0.74220E+00_JPRB, 0.74872E+00_JPRB, 0.76231E+00_JPRB, 0.78053E+00_JPRB, 0.81672E+00_JPRB, 0.83742E+00_JPRB, 0.84699E+00_JPRB ,&
&  0.87598E+00_JPRB, 0.89170E+00_JPRB, 0.91354E+00_JPRB, 0.93745E+00_JPRB, 0.95933E+00_JPRB, 0.97267E+00_JPRB, 0.96801E+00_JPRB ,&
&  0.95901E+00_JPRB, 0.97889E+00_JPRB, 0.10051E+01_JPRB, 0.10035E+01_JPRB, 0.96537E+00_JPRB, 0.87314E+00_JPRB, 0.79361E+00_JPRB ,&
&  0.70865E+00_JPRB, 0.68878E+00_JPRB, 0.65551E+00_JPRB, 0.62915E+00_JPRB, 0.60856E+00_JPRB /)
ZBURD(:, 43,  8) = (/ &
&  0.58559E+00_JPRB, 0.58108E+00_JPRB, 0.58663E+00_JPRB, 0.59267E+00_JPRB, 0.60558E+00_JPRB, 0.63254E+00_JPRB, 0.66344E+00_JPRB ,&
&  0.68888E+00_JPRB, 0.73256E+00_JPRB, 0.76329E+00_JPRB, 0.76487E+00_JPRB, 0.75951E+00_JPRB, 0.76527E+00_JPRB, 0.77885E+00_JPRB ,&
&  0.79904E+00_JPRB, 0.81516E+00_JPRB, 0.82541E+00_JPRB, 0.83497E+00_JPRB, 0.84987E+00_JPRB, 0.86702E+00_JPRB, 0.89074E+00_JPRB ,&
&  0.92774E+00_JPRB, 0.99009E+00_JPRB, 0.10843E+01_JPRB, 0.12269E+01_JPRB, 0.14189E+01_JPRB, 0.16337E+01_JPRB, 0.17600E+01_JPRB ,&
&  0.12960E+01_JPRB, 0.70987E+00_JPRB, 0.13940E+01_JPRB, 0.16557E+01_JPRB, 0.15159E+01_JPRB, 0.14692E+01_JPRB, 0.14047E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.13126E+01_JPRB, 0.12552E+01_JPRB, 0.12291E+01_JPRB, 0.10954E+01_JPRB, 0.10415E+01_JPRB, 0.10162E+01_JPRB ,&
&  0.10131E+01_JPRB, 0.10183E+01_JPRB, 0.10415E+01_JPRB, 0.10792E+01_JPRB, 0.11430E+01_JPRB, 0.12425E+01_JPRB, 0.13942E+01_JPRB ,&
&  0.15591E+01_JPRB, 0.17445E+01_JPRB, 0.20037E+01_JPRB, 0.18734E+01_JPRB, 0.19257E+01_JPRB, 0.17925E+01_JPRB, 0.15724E+01_JPRB ,&
&  0.13505E+01_JPRB, 0.11932E+01_JPRB, 0.12049E+01_JPRB, 0.12309E+01_JPRB, 0.10943E+01_JPRB, 0.95731E+00_JPRB, 0.96134E+00_JPRB ,&
&  0.87881E+00_JPRB, 0.84083E+00_JPRB, 0.78039E+00_JPRB, 0.73595E+00_JPRB, 0.72480E+00_JPRB, 0.73887E+00_JPRB, 0.75642E+00_JPRB ,&
&  0.76195E+00_JPRB, 0.76366E+00_JPRB, 0.77131E+00_JPRB, 0.78467E+00_JPRB, 0.80839E+00_JPRB, 0.82760E+00_JPRB, 0.83981E+00_JPRB ,&
&  0.87209E+00_JPRB, 0.90728E+00_JPRB, 0.93948E+00_JPRB, 0.96686E+00_JPRB, 0.98439E+00_JPRB, 0.96565E+00_JPRB, 0.96585E+00_JPRB ,&
&  0.97288E+00_JPRB, 0.97578E+00_JPRB, 0.97572E+00_JPRB, 0.99035E+00_JPRB, 0.97633E+00_JPRB, 0.92583E+00_JPRB, 0.90584E+00_JPRB ,&
&  0.83999E+00_JPRB, 0.76837E+00_JPRB, 0.68248E+00_JPRB, 0.62980E+00_JPRB, 0.61015E+00_JPRB /)
ZBURD(:, 44,  8) = (/ &
&  0.66361E+00_JPRB, 0.63650E+00_JPRB, 0.60115E+00_JPRB, 0.59259E+00_JPRB, 0.59184E+00_JPRB, 0.59013E+00_JPRB, 0.60615E+00_JPRB ,&
&  0.62150E+00_JPRB, 0.64421E+00_JPRB, 0.64883E+00_JPRB, 0.67025E+00_JPRB, 0.68154E+00_JPRB, 0.68282E+00_JPRB, 0.69232E+00_JPRB ,&
&  0.69875E+00_JPRB, 0.70585E+00_JPRB, 0.71324E+00_JPRB, 0.72420E+00_JPRB, 0.73719E+00_JPRB, 0.74231E+00_JPRB, 0.75156E+00_JPRB ,&
&  0.77269E+00_JPRB, 0.79960E+00_JPRB, 0.83993E+00_JPRB, 0.92942E+00_JPRB, 0.10820E+01_JPRB, 0.12815E+01_JPRB, 0.15037E+01_JPRB ,&
&  0.16882E+01_JPRB, 0.12305E+01_JPRB, 0.89402E+00_JPRB, 0.15046E+01_JPRB, 0.16720E+01_JPRB, 0.16437E+01_JPRB, 0.14422E+01_JPRB ,&
&  0.14561E+01_JPRB, 0.14170E+01_JPRB, 0.13678E+01_JPRB, 0.12970E+01_JPRB, 0.11362E+01_JPRB, 0.10499E+01_JPRB, 0.98401E+00_JPRB ,&
&  0.94644E+00_JPRB, 0.92957E+00_JPRB, 0.94103E+00_JPRB, 0.98192E+00_JPRB, 0.10289E+01_JPRB, 0.10920E+01_JPRB, 0.12199E+01_JPRB ,&
&  0.13848E+01_JPRB, 0.15882E+01_JPRB, 0.18946E+01_JPRB, 0.19744E+01_JPRB, 0.21912E+01_JPRB, 0.19356E+01_JPRB, 0.16652E+01_JPRB ,&
&  0.14071E+01_JPRB, 0.13983E+01_JPRB, 0.14517E+01_JPRB, 0.13443E+01_JPRB, 0.11482E+01_JPRB, 0.10476E+01_JPRB, 0.10519E+01_JPRB ,&
&  0.95774E+00_JPRB, 0.90588E+00_JPRB, 0.86470E+00_JPRB, 0.81215E+00_JPRB, 0.78327E+00_JPRB, 0.78004E+00_JPRB, 0.78469E+00_JPRB ,&
&  0.78660E+00_JPRB, 0.78036E+00_JPRB, 0.77773E+00_JPRB, 0.78706E+00_JPRB, 0.80347E+00_JPRB, 0.81722E+00_JPRB, 0.83778E+00_JPRB ,&
&  0.86506E+00_JPRB, 0.90086E+00_JPRB, 0.94401E+00_JPRB, 0.97715E+00_JPRB, 0.97676E+00_JPRB, 0.95709E+00_JPRB, 0.98347E+00_JPRB ,&
&  0.98489E+00_JPRB, 0.99397E+00_JPRB, 0.97327E+00_JPRB, 0.98452E+00_JPRB, 0.97227E+00_JPRB, 0.97043E+00_JPRB, 0.96290E+00_JPRB ,&
&  0.95494E+00_JPRB, 0.92382E+00_JPRB, 0.83607E+00_JPRB, 0.72427E+00_JPRB, 0.69545E+00_JPRB /)
ZBURD(:, 45,  8) = (/ &
&  0.72666E+00_JPRB, 0.70344E+00_JPRB, 0.67426E+00_JPRB, 0.64507E+00_JPRB, 0.62967E+00_JPRB, 0.61428E+00_JPRB, 0.59511E+00_JPRB ,&
&  0.58569E+00_JPRB, 0.58163E+00_JPRB, 0.57209E+00_JPRB, 0.58263E+00_JPRB, 0.60647E+00_JPRB, 0.61845E+00_JPRB, 0.62568E+00_JPRB ,&
&  0.63394E+00_JPRB, 0.64106E+00_JPRB, 0.63251E+00_JPRB, 0.63074E+00_JPRB, 0.63889E+00_JPRB, 0.64553E+00_JPRB, 0.64726E+00_JPRB ,&
&  0.65333E+00_JPRB, 0.66749E+00_JPRB, 0.68679E+00_JPRB, 0.72413E+00_JPRB, 0.80398E+00_JPRB, 0.92651E+00_JPRB, 0.10925E+01_JPRB ,&
&  0.13315E+01_JPRB, 0.13264E+01_JPRB, 0.76396E+00_JPRB, 0.12799E+01_JPRB, 0.17375E+01_JPRB, 0.17539E+01_JPRB, 0.16452E+01_JPRB ,&
&  0.16339E+01_JPRB, 0.15880E+01_JPRB, 0.15481E+01_JPRB, 0.13838E+01_JPRB, 0.12346E+01_JPRB, 0.11152E+01_JPRB, 0.10144E+01_JPRB ,&
&  0.92972E+00_JPRB, 0.87335E+00_JPRB, 0.85123E+00_JPRB, 0.87359E+00_JPRB, 0.91728E+00_JPRB, 0.95244E+00_JPRB, 0.10240E+01_JPRB ,&
&  0.11578E+01_JPRB, 0.13326E+01_JPRB, 0.16151E+01_JPRB, 0.18413E+01_JPRB, 0.22581E+01_JPRB, 0.22563E+01_JPRB, 0.19140E+01_JPRB ,&
&  0.16083E+01_JPRB, 0.16364E+01_JPRB, 0.16440E+01_JPRB, 0.14832E+01_JPRB, 0.12246E+01_JPRB, 0.11408E+01_JPRB, 0.11318E+01_JPRB ,&
&  0.10372E+01_JPRB, 0.96314E+00_JPRB, 0.92881E+00_JPRB, 0.89622E+00_JPRB, 0.85177E+00_JPRB, 0.81352E+00_JPRB, 0.79567E+00_JPRB ,&
&  0.79466E+00_JPRB, 0.79332E+00_JPRB, 0.79112E+00_JPRB, 0.78995E+00_JPRB, 0.79530E+00_JPRB, 0.80339E+00_JPRB, 0.81728E+00_JPRB ,&
&  0.84207E+00_JPRB, 0.87495E+00_JPRB, 0.92670E+00_JPRB, 0.96169E+00_JPRB, 0.98558E+00_JPRB, 0.10049E+01_JPRB, 0.10294E+01_JPRB ,&
&  0.10520E+01_JPRB, 0.10452E+01_JPRB, 0.94138E+00_JPRB, 0.91849E+00_JPRB, 0.95320E+00_JPRB, 0.94157E+00_JPRB, 0.95337E+00_JPRB ,&
&  0.97929E+00_JPRB, 0.97827E+00_JPRB, 0.91527E+00_JPRB, 0.78073E+00_JPRB, 0.75536E+00_JPRB /)
ZBURD(:, 46,  8) = (/ &
&  0.73467E+00_JPRB, 0.71478E+00_JPRB, 0.69459E+00_JPRB, 0.67902E+00_JPRB, 0.66873E+00_JPRB, 0.65789E+00_JPRB, 0.63931E+00_JPRB ,&
&  0.61514E+00_JPRB, 0.60107E+00_JPRB, 0.57609E+00_JPRB, 0.56988E+00_JPRB, 0.57543E+00_JPRB, 0.58019E+00_JPRB, 0.58970E+00_JPRB ,&
&  0.59484E+00_JPRB, 0.59888E+00_JPRB, 0.59935E+00_JPRB, 0.59710E+00_JPRB, 0.59647E+00_JPRB, 0.59516E+00_JPRB, 0.59491E+00_JPRB ,&
&  0.59210E+00_JPRB, 0.59575E+00_JPRB, 0.60596E+00_JPRB, 0.62068E+00_JPRB, 0.65590E+00_JPRB, 0.71892E+00_JPRB, 0.80713E+00_JPRB ,&
&  0.10095E+01_JPRB, 0.11117E+01_JPRB, 0.12206E+01_JPRB, 0.17580E+01_JPRB, 0.20719E+01_JPRB, 0.19834E+01_JPRB, 0.19578E+01_JPRB ,&
&  0.19896E+01_JPRB, 0.19610E+01_JPRB, 0.18460E+01_JPRB, 0.14935E+01_JPRB, 0.12465E+01_JPRB, 0.11160E+01_JPRB, 0.10184E+01_JPRB ,&
&  0.92868E+00_JPRB, 0.86096E+00_JPRB, 0.80901E+00_JPRB, 0.78944E+00_JPRB, 0.80574E+00_JPRB, 0.84475E+00_JPRB, 0.88473E+00_JPRB ,&
&  0.96760E+00_JPRB, 0.11052E+01_JPRB, 0.13045E+01_JPRB, 0.15144E+01_JPRB, 0.19183E+01_JPRB, 0.22366E+01_JPRB, 0.21586E+01_JPRB ,&
&  0.20735E+01_JPRB, 0.19698E+01_JPRB, 0.17996E+01_JPRB, 0.16379E+01_JPRB, 0.13217E+01_JPRB, 0.13027E+01_JPRB, 0.11859E+01_JPRB ,&
&  0.10774E+01_JPRB, 0.99806E+00_JPRB, 0.95342E+00_JPRB, 0.92897E+00_JPRB, 0.89771E+00_JPRB, 0.85452E+00_JPRB, 0.82048E+00_JPRB ,&
&  0.80381E+00_JPRB, 0.80245E+00_JPRB, 0.80219E+00_JPRB, 0.79580E+00_JPRB, 0.79519E+00_JPRB, 0.80143E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.81565E+00_JPRB, 0.82909E+00_JPRB, 0.84309E+00_JPRB, 0.89312E+00_JPRB, 0.93761E+00_JPRB, 0.96021E+00_JPRB, 0.95490E+00_JPRB ,&
&  0.96573E+00_JPRB, 0.98618E+00_JPRB, 0.95998E+00_JPRB, 0.95582E+00_JPRB, 0.94240E+00_JPRB, 0.91974E+00_JPRB, 0.86782E+00_JPRB ,&
&  0.85925E+00_JPRB, 0.82670E+00_JPRB, 0.78457E+00_JPRB, 0.76516E+00_JPRB, 0.75027E+00_JPRB /)
ZBURD(:, 47,  8) = (/ &
&  0.73276E+00_JPRB, 0.71305E+00_JPRB, 0.69907E+00_JPRB, 0.68433E+00_JPRB, 0.67168E+00_JPRB, 0.66601E+00_JPRB, 0.66099E+00_JPRB ,&
&  0.65083E+00_JPRB, 0.62564E+00_JPRB, 0.59957E+00_JPRB, 0.58805E+00_JPRB, 0.58463E+00_JPRB, 0.57454E+00_JPRB, 0.57594E+00_JPRB ,&
&  0.57741E+00_JPRB, 0.58207E+00_JPRB, 0.57764E+00_JPRB, 0.57363E+00_JPRB, 0.57128E+00_JPRB, 0.57001E+00_JPRB, 0.56772E+00_JPRB ,&
&  0.56265E+00_JPRB, 0.56078E+00_JPRB, 0.56609E+00_JPRB, 0.57359E+00_JPRB, 0.58751E+00_JPRB, 0.61757E+00_JPRB, 0.65435E+00_JPRB ,&
&  0.78198E+00_JPRB, 0.92536E+00_JPRB, 0.10151E+01_JPRB, 0.17470E+01_JPRB, 0.20394E+01_JPRB, 0.21516E+01_JPRB, 0.20327E+01_JPRB ,&
&  0.20609E+01_JPRB, 0.22835E+01_JPRB, 0.19520E+01_JPRB, 0.14964E+01_JPRB, 0.11806E+01_JPRB, 0.10358E+01_JPRB, 0.94752E+00_JPRB ,&
&  0.87447E+00_JPRB, 0.80925E+00_JPRB, 0.76636E+00_JPRB, 0.74166E+00_JPRB, 0.74388E+00_JPRB, 0.75386E+00_JPRB, 0.77913E+00_JPRB ,&
&  0.82622E+00_JPRB, 0.92423E+00_JPRB, 0.10735E+01_JPRB, 0.11742E+01_JPRB, 0.14804E+01_JPRB, 0.18504E+01_JPRB, 0.20587E+01_JPRB ,&
&  0.21749E+01_JPRB, 0.22081E+01_JPRB, 0.18987E+01_JPRB, 0.16896E+01_JPRB, 0.14496E+01_JPRB, 0.13298E+01_JPRB, 0.11965E+01_JPRB ,&
&  0.10997E+01_JPRB, 0.10226E+01_JPRB, 0.96568E+00_JPRB, 0.93364E+00_JPRB, 0.90400E+00_JPRB, 0.87179E+00_JPRB, 0.83519E+00_JPRB ,&
&  0.80441E+00_JPRB, 0.80065E+00_JPRB, 0.79953E+00_JPRB, 0.79180E+00_JPRB, 0.78892E+00_JPRB, 0.78822E+00_JPRB, 0.78980E+00_JPRB ,&
&  0.79100E+00_JPRB, 0.79467E+00_JPRB, 0.79299E+00_JPRB, 0.82456E+00_JPRB, 0.85851E+00_JPRB, 0.86469E+00_JPRB, 0.88461E+00_JPRB ,&
&  0.93643E+00_JPRB, 0.97556E+00_JPRB, 0.96521E+00_JPRB, 0.93649E+00_JPRB, 0.96924E+00_JPRB, 0.99477E+00_JPRB, 0.86556E+00_JPRB ,&
&  0.83521E+00_JPRB, 0.80965E+00_JPRB, 0.77873E+00_JPRB, 0.75819E+00_JPRB, 0.74642E+00_JPRB /)
ZBURD(:, 48,  8) = (/ &
&  0.73960E+00_JPRB, 0.70993E+00_JPRB, 0.68860E+00_JPRB, 0.67266E+00_JPRB, 0.65987E+00_JPRB, 0.64893E+00_JPRB, 0.64616E+00_JPRB ,&
&  0.64368E+00_JPRB, 0.63750E+00_JPRB, 0.62071E+00_JPRB, 0.60094E+00_JPRB, 0.59236E+00_JPRB, 0.58931E+00_JPRB, 0.57750E+00_JPRB ,&
&  0.57203E+00_JPRB, 0.57173E+00_JPRB, 0.56891E+00_JPRB, 0.56058E+00_JPRB, 0.55327E+00_JPRB, 0.54883E+00_JPRB, 0.54674E+00_JPRB ,&
&  0.53721E+00_JPRB, 0.53298E+00_JPRB, 0.53444E+00_JPRB, 0.53582E+00_JPRB, 0.54008E+00_JPRB, 0.56503E+00_JPRB, 0.60326E+00_JPRB ,&
&  0.67891E+00_JPRB, 0.97612E+00_JPRB, 0.99530E+00_JPRB, 0.14419E+01_JPRB, 0.16320E+01_JPRB, 0.18447E+01_JPRB, 0.19579E+01_JPRB ,&
&  0.20961E+01_JPRB, 0.22502E+01_JPRB, 0.18507E+01_JPRB, 0.14323E+01_JPRB, 0.10853E+01_JPRB, 0.90108E+00_JPRB, 0.81617E+00_JPRB ,&
&  0.77315E+00_JPRB, 0.73459E+00_JPRB, 0.70615E+00_JPRB, 0.69741E+00_JPRB, 0.70072E+00_JPRB, 0.70518E+00_JPRB, 0.71280E+00_JPRB ,&
&  0.74229E+00_JPRB, 0.80373E+00_JPRB, 0.94550E+00_JPRB, 0.11064E+01_JPRB, 0.11616E+01_JPRB, 0.14173E+01_JPRB, 0.17865E+01_JPRB ,&
&  0.21460E+01_JPRB, 0.23012E+01_JPRB, 0.19709E+01_JPRB, 0.16968E+01_JPRB, 0.14954E+01_JPRB, 0.13418E+01_JPRB, 0.12093E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.10283E+01_JPRB, 0.97127E+00_JPRB, 0.93028E+00_JPRB, 0.90215E+00_JPRB, 0.88227E+00_JPRB, 0.85520E+00_JPRB ,&
&  0.82500E+00_JPRB, 0.80895E+00_JPRB, 0.80308E+00_JPRB, 0.79808E+00_JPRB, 0.79131E+00_JPRB, 0.78038E+00_JPRB, 0.76625E+00_JPRB ,&
&  0.75819E+00_JPRB, 0.76194E+00_JPRB, 0.76086E+00_JPRB, 0.78053E+00_JPRB, 0.79916E+00_JPRB, 0.79771E+00_JPRB, 0.81191E+00_JPRB ,&
&  0.87485E+00_JPRB, 0.91720E+00_JPRB, 0.93381E+00_JPRB, 0.95123E+00_JPRB, 0.10324E+01_JPRB, 0.11777E+01_JPRB, 0.94278E+00_JPRB ,&
&  0.85698E+00_JPRB, 0.81805E+00_JPRB, 0.78987E+00_JPRB, 0.77371E+00_JPRB, 0.75874E+00_JPRB /)
ZBURD(:, 49,  8) = (/ &
&  0.75031E+00_JPRB, 0.71867E+00_JPRB, 0.69226E+00_JPRB, 0.66690E+00_JPRB, 0.64838E+00_JPRB, 0.62990E+00_JPRB, 0.61792E+00_JPRB ,&
&  0.61355E+00_JPRB, 0.61416E+00_JPRB, 0.61296E+00_JPRB, 0.60530E+00_JPRB, 0.59147E+00_JPRB, 0.57982E+00_JPRB, 0.57095E+00_JPRB ,&
&  0.56203E+00_JPRB, 0.55954E+00_JPRB, 0.55812E+00_JPRB, 0.54934E+00_JPRB, 0.53955E+00_JPRB, 0.53191E+00_JPRB, 0.52451E+00_JPRB ,&
&  0.51857E+00_JPRB, 0.51557E+00_JPRB, 0.51291E+00_JPRB, 0.51031E+00_JPRB, 0.50883E+00_JPRB, 0.52144E+00_JPRB, 0.58207E+00_JPRB ,&
&  0.70933E+00_JPRB, 0.61501E+00_JPRB, 0.11837E+01_JPRB, 0.13253E+01_JPRB, 0.13772E+01_JPRB, 0.14696E+01_JPRB, 0.17047E+01_JPRB ,&
&  0.19525E+01_JPRB, 0.19245E+01_JPRB, 0.16724E+01_JPRB, 0.13502E+01_JPRB, 0.10656E+01_JPRB, 0.87760E+00_JPRB, 0.77939E+00_JPRB ,&
&  0.71149E+00_JPRB, 0.66897E+00_JPRB, 0.64991E+00_JPRB, 0.63951E+00_JPRB, 0.64160E+00_JPRB, 0.65080E+00_JPRB, 0.66100E+00_JPRB ,&
&  0.68264E+00_JPRB, 0.72244E+00_JPRB, 0.82366E+00_JPRB, 0.10078E+01_JPRB, 0.11414E+01_JPRB, 0.12586E+01_JPRB, 0.15089E+01_JPRB ,&
&  0.20051E+01_JPRB, 0.20884E+01_JPRB, 0.17756E+01_JPRB, 0.15507E+01_JPRB, 0.14054E+01_JPRB, 0.12990E+01_JPRB, 0.11955E+01_JPRB ,&
&  0.10999E+01_JPRB, 0.10092E+01_JPRB, 0.94446E+00_JPRB, 0.89942E+00_JPRB, 0.87259E+00_JPRB, 0.86078E+00_JPRB, 0.85209E+00_JPRB ,&
&  0.84211E+00_JPRB, 0.83005E+00_JPRB, 0.82053E+00_JPRB, 0.80904E+00_JPRB, 0.79526E+00_JPRB, 0.77446E+00_JPRB, 0.74833E+00_JPRB ,&
&  0.73223E+00_JPRB, 0.72751E+00_JPRB, 0.72999E+00_JPRB, 0.73062E+00_JPRB, 0.74200E+00_JPRB, 0.75172E+00_JPRB, 0.77188E+00_JPRB ,&
&  0.82632E+00_JPRB, 0.85216E+00_JPRB, 0.87525E+00_JPRB, 0.91956E+00_JPRB, 0.97066E+00_JPRB, 0.10867E+01_JPRB, 0.10022E+01_JPRB ,&
&  0.93641E+00_JPRB, 0.87019E+00_JPRB, 0.81659E+00_JPRB, 0.78851E+00_JPRB, 0.77267E+00_JPRB /)
ZBURD(:, 50,  8) = (/ &
&  0.75538E+00_JPRB, 0.72942E+00_JPRB, 0.69556E+00_JPRB, 0.66514E+00_JPRB, 0.64005E+00_JPRB, 0.61521E+00_JPRB, 0.59813E+00_JPRB ,&
&  0.58527E+00_JPRB, 0.58494E+00_JPRB, 0.58898E+00_JPRB, 0.59147E+00_JPRB, 0.58687E+00_JPRB, 0.57301E+00_JPRB, 0.55868E+00_JPRB ,&
&  0.54869E+00_JPRB, 0.54580E+00_JPRB, 0.54500E+00_JPRB, 0.54031E+00_JPRB, 0.53291E+00_JPRB, 0.52383E+00_JPRB, 0.51591E+00_JPRB ,&
&  0.50917E+00_JPRB, 0.50387E+00_JPRB, 0.49819E+00_JPRB, 0.49404E+00_JPRB, 0.48889E+00_JPRB, 0.49480E+00_JPRB, 0.55601E+00_JPRB ,&
&  0.73443E+00_JPRB, 0.92348E+00_JPRB, 0.11261E+01_JPRB, 0.10727E+01_JPRB, 0.10759E+01_JPRB, 0.11326E+01_JPRB, 0.13126E+01_JPRB ,&
&  0.14175E+01_JPRB, 0.14527E+01_JPRB, 0.13759E+01_JPRB, 0.12106E+01_JPRB, 0.10150E+01_JPRB, 0.87345E+00_JPRB, 0.76931E+00_JPRB ,&
&  0.68417E+00_JPRB, 0.63067E+00_JPRB, 0.60058E+00_JPRB, 0.59203E+00_JPRB, 0.59618E+00_JPRB, 0.60662E+00_JPRB, 0.61448E+00_JPRB ,&
&  0.62783E+00_JPRB, 0.65201E+00_JPRB, 0.72956E+00_JPRB, 0.90136E+00_JPRB, 0.11653E+01_JPRB, 0.11913E+01_JPRB, 0.15098E+01_JPRB ,&
&  0.19050E+01_JPRB, 0.16990E+01_JPRB, 0.15422E+01_JPRB, 0.14086E+01_JPRB, 0.13066E+01_JPRB, 0.12217E+01_JPRB, 0.11003E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.92439E+00_JPRB, 0.86404E+00_JPRB, 0.83270E+00_JPRB, 0.81360E+00_JPRB, 0.80606E+00_JPRB, 0.80375E+00_JPRB ,&
&  0.80183E+00_JPRB, 0.79935E+00_JPRB, 0.80036E+00_JPRB, 0.79397E+00_JPRB, 0.77870E+00_JPRB, 0.75648E+00_JPRB, 0.73104E+00_JPRB ,&
&  0.71303E+00_JPRB, 0.69841E+00_JPRB, 0.71648E+00_JPRB, 0.69893E+00_JPRB, 0.70185E+00_JPRB, 0.71390E+00_JPRB, 0.72228E+00_JPRB ,&
&  0.73426E+00_JPRB, 0.77646E+00_JPRB, 0.80755E+00_JPRB, 0.83584E+00_JPRB, 0.98593E+00_JPRB, 0.10785E+01_JPRB, 0.10545E+01_JPRB ,&
&  0.10033E+01_JPRB, 0.91557E+00_JPRB, 0.84180E+00_JPRB, 0.79457E+00_JPRB, 0.77524E+00_JPRB /)
ZBURD(:, 51,  8) = (/ &
&  0.76187E+00_JPRB, 0.72479E+00_JPRB, 0.68742E+00_JPRB, 0.65485E+00_JPRB, 0.62523E+00_JPRB, 0.60270E+00_JPRB, 0.58849E+00_JPRB ,&
&  0.57623E+00_JPRB, 0.56815E+00_JPRB, 0.56889E+00_JPRB, 0.57358E+00_JPRB, 0.57304E+00_JPRB, 0.56427E+00_JPRB, 0.55160E+00_JPRB ,&
&  0.53847E+00_JPRB, 0.52847E+00_JPRB, 0.52101E+00_JPRB, 0.51763E+00_JPRB, 0.51237E+00_JPRB, 0.50713E+00_JPRB, 0.49920E+00_JPRB ,&
&  0.49337E+00_JPRB, 0.48844E+00_JPRB, 0.48076E+00_JPRB, 0.47255E+00_JPRB, 0.46655E+00_JPRB, 0.46850E+00_JPRB, 0.51712E+00_JPRB ,&
&  0.65918E+00_JPRB, 0.81908E+00_JPRB, 0.82452E+00_JPRB, 0.85298E+00_JPRB, 0.91218E+00_JPRB, 0.93885E+00_JPRB, 0.10135E+01_JPRB ,&
&  0.10560E+01_JPRB, 0.10784E+01_JPRB, 0.10859E+01_JPRB, 0.10293E+01_JPRB, 0.92148E+00_JPRB, 0.83705E+00_JPRB, 0.75537E+00_JPRB ,&
&  0.68111E+00_JPRB, 0.62294E+00_JPRB, 0.58605E+00_JPRB, 0.57113E+00_JPRB, 0.57270E+00_JPRB, 0.58008E+00_JPRB, 0.58414E+00_JPRB ,&
&  0.58743E+00_JPRB, 0.59165E+00_JPRB, 0.62152E+00_JPRB, 0.69363E+00_JPRB, 0.87450E+00_JPRB, 0.98776E+00_JPRB, 0.11781E+01_JPRB ,&
&  0.12342E+01_JPRB, 0.12007E+01_JPRB, 0.11515E+01_JPRB, 0.10961E+01_JPRB, 0.10441E+01_JPRB, 0.98077E+00_JPRB, 0.92926E+00_JPRB ,&
&  0.87235E+00_JPRB, 0.81822E+00_JPRB, 0.77689E+00_JPRB, 0.75032E+00_JPRB, 0.73599E+00_JPRB, 0.72899E+00_JPRB, 0.72873E+00_JPRB ,&
&  0.72291E+00_JPRB, 0.72087E+00_JPRB, 0.71607E+00_JPRB, 0.71583E+00_JPRB, 0.71720E+00_JPRB, 0.70516E+00_JPRB, 0.68426E+00_JPRB ,&
&  0.66232E+00_JPRB, 0.64322E+00_JPRB, 0.64535E+00_JPRB, 0.65131E+00_JPRB, 0.65564E+00_JPRB, 0.65575E+00_JPRB, 0.65654E+00_JPRB ,&
&  0.67167E+00_JPRB, 0.73434E+00_JPRB, 0.75614E+00_JPRB, 0.78227E+00_JPRB, 0.88829E+00_JPRB, 0.95405E+00_JPRB, 0.95651E+00_JPRB ,&
&  0.93609E+00_JPRB, 0.89207E+00_JPRB, 0.83687E+00_JPRB, 0.79088E+00_JPRB, 0.78595E+00_JPRB /)
ZBURD(:, 52,  8) = (/ &
&  0.78035E+00_JPRB, 0.74502E+00_JPRB, 0.70369E+00_JPRB, 0.66533E+00_JPRB, 0.63051E+00_JPRB, 0.60303E+00_JPRB, 0.58585E+00_JPRB ,&
&  0.57427E+00_JPRB, 0.56597E+00_JPRB, 0.56080E+00_JPRB, 0.55727E+00_JPRB, 0.55292E+00_JPRB, 0.54868E+00_JPRB, 0.54232E+00_JPRB ,&
&  0.53449E+00_JPRB, 0.52486E+00_JPRB, 0.51583E+00_JPRB, 0.50911E+00_JPRB, 0.50356E+00_JPRB, 0.50034E+00_JPRB, 0.49631E+00_JPRB ,&
&  0.48831E+00_JPRB, 0.47887E+00_JPRB, 0.46697E+00_JPRB, 0.45479E+00_JPRB, 0.44717E+00_JPRB, 0.44697E+00_JPRB, 0.47839E+00_JPRB ,&
&  0.58276E+00_JPRB, 0.61307E+00_JPRB, 0.65827E+00_JPRB, 0.69828E+00_JPRB, 0.74791E+00_JPRB, 0.78544E+00_JPRB, 0.81285E+00_JPRB ,&
&  0.83133E+00_JPRB, 0.85136E+00_JPRB, 0.88367E+00_JPRB, 0.87702E+00_JPRB, 0.81771E+00_JPRB, 0.76227E+00_JPRB, 0.70987E+00_JPRB ,&
&  0.66119E+00_JPRB, 0.61811E+00_JPRB, 0.58224E+00_JPRB, 0.56701E+00_JPRB, 0.56326E+00_JPRB, 0.56559E+00_JPRB, 0.56560E+00_JPRB ,&
&  0.56390E+00_JPRB, 0.55734E+00_JPRB, 0.55693E+00_JPRB, 0.57045E+00_JPRB, 0.60089E+00_JPRB, 0.65501E+00_JPRB, 0.74740E+00_JPRB ,&
&  0.79215E+00_JPRB, 0.78797E+00_JPRB, 0.79460E+00_JPRB, 0.79594E+00_JPRB, 0.77299E+00_JPRB, 0.74850E+00_JPRB, 0.72734E+00_JPRB ,&
&  0.70872E+00_JPRB, 0.69250E+00_JPRB, 0.68058E+00_JPRB, 0.66633E+00_JPRB, 0.65546E+00_JPRB, 0.64904E+00_JPRB, 0.64570E+00_JPRB ,&
&  0.64269E+00_JPRB, 0.64156E+00_JPRB, 0.63697E+00_JPRB, 0.62593E+00_JPRB, 0.61833E+00_JPRB, 0.61089E+00_JPRB, 0.59800E+00_JPRB ,&
&  0.58184E+00_JPRB, 0.57325E+00_JPRB, 0.57580E+00_JPRB, 0.58333E+00_JPRB, 0.59354E+00_JPRB, 0.60328E+00_JPRB, 0.60976E+00_JPRB ,&
&  0.61557E+00_JPRB, 0.64666E+00_JPRB, 0.66481E+00_JPRB, 0.75619E+00_JPRB, 0.82937E+00_JPRB, 0.84612E+00_JPRB, 0.86355E+00_JPRB ,&
&  0.86764E+00_JPRB, 0.84737E+00_JPRB, 0.80575E+00_JPRB, 0.76039E+00_JPRB, 0.78920E+00_JPRB /)
ZBURD(:, 53,  8) = (/ &
&  0.68414E+00_JPRB, 0.67946E+00_JPRB, 0.65649E+00_JPRB, 0.62981E+00_JPRB, 0.60642E+00_JPRB, 0.58824E+00_JPRB, 0.57272E+00_JPRB ,&
&  0.56155E+00_JPRB, 0.55302E+00_JPRB, 0.54678E+00_JPRB, 0.54242E+00_JPRB, 0.53715E+00_JPRB, 0.52992E+00_JPRB, 0.52233E+00_JPRB ,&
&  0.51625E+00_JPRB, 0.50938E+00_JPRB, 0.49904E+00_JPRB, 0.49131E+00_JPRB, 0.48644E+00_JPRB, 0.48101E+00_JPRB, 0.47579E+00_JPRB ,&
&  0.46937E+00_JPRB, 0.45674E+00_JPRB, 0.44405E+00_JPRB, 0.43557E+00_JPRB, 0.42906E+00_JPRB, 0.42653E+00_JPRB, 0.44643E+00_JPRB ,&
&  0.51415E+00_JPRB, 0.49682E+00_JPRB, 0.53906E+00_JPRB, 0.58480E+00_JPRB, 0.63789E+00_JPRB, 0.67056E+00_JPRB, 0.68982E+00_JPRB ,&
&  0.70205E+00_JPRB, 0.72232E+00_JPRB, 0.73888E+00_JPRB, 0.76035E+00_JPRB, 0.73386E+00_JPRB, 0.69129E+00_JPRB, 0.65556E+00_JPRB ,&
&  0.62254E+00_JPRB, 0.59331E+00_JPRB, 0.56922E+00_JPRB, 0.55287E+00_JPRB, 0.54731E+00_JPRB, 0.54503E+00_JPRB, 0.53985E+00_JPRB ,&
&  0.53454E+00_JPRB, 0.52537E+00_JPRB, 0.51688E+00_JPRB, 0.51673E+00_JPRB, 0.51441E+00_JPRB, 0.53109E+00_JPRB, 0.55456E+00_JPRB ,&
&  0.57428E+00_JPRB, 0.60010E+00_JPRB, 0.61198E+00_JPRB, 0.61872E+00_JPRB, 0.61194E+00_JPRB, 0.60372E+00_JPRB, 0.59486E+00_JPRB ,&
&  0.58737E+00_JPRB, 0.58102E+00_JPRB, 0.57648E+00_JPRB, 0.57088E+00_JPRB, 0.56437E+00_JPRB, 0.56029E+00_JPRB, 0.55707E+00_JPRB ,&
&  0.55455E+00_JPRB, 0.55145E+00_JPRB, 0.54867E+00_JPRB, 0.53978E+00_JPRB, 0.52708E+00_JPRB, 0.51694E+00_JPRB, 0.50911E+00_JPRB ,&
&  0.50348E+00_JPRB, 0.50319E+00_JPRB, 0.50722E+00_JPRB, 0.51771E+00_JPRB, 0.52862E+00_JPRB, 0.54063E+00_JPRB, 0.54959E+00_JPRB ,&
&  0.55541E+00_JPRB, 0.57320E+00_JPRB, 0.60135E+00_JPRB, 0.66466E+00_JPRB, 0.70805E+00_JPRB, 0.74021E+00_JPRB, 0.76705E+00_JPRB ,&
&  0.77891E+00_JPRB, 0.77811E+00_JPRB, 0.75124E+00_JPRB, 0.72104E+00_JPRB, 0.68014E+00_JPRB /)
ZBURD(:, 54,  8) = (/ &
&  0.63364E+00_JPRB, 0.62883E+00_JPRB, 0.61272E+00_JPRB, 0.59175E+00_JPRB, 0.57394E+00_JPRB, 0.55887E+00_JPRB, 0.54678E+00_JPRB ,&
&  0.53178E+00_JPRB, 0.51896E+00_JPRB, 0.51306E+00_JPRB, 0.51064E+00_JPRB, 0.50803E+00_JPRB, 0.50247E+00_JPRB, 0.49545E+00_JPRB ,&
&  0.48792E+00_JPRB, 0.47958E+00_JPRB, 0.47001E+00_JPRB, 0.46029E+00_JPRB, 0.45202E+00_JPRB, 0.44334E+00_JPRB, 0.43681E+00_JPRB ,&
&  0.43083E+00_JPRB, 0.42567E+00_JPRB, 0.41745E+00_JPRB, 0.40733E+00_JPRB, 0.40282E+00_JPRB, 0.39756E+00_JPRB, 0.41510E+00_JPRB ,&
&  0.47410E+00_JPRB, 0.41194E+00_JPRB, 0.44218E+00_JPRB, 0.49476E+00_JPRB, 0.54563E+00_JPRB, 0.59960E+00_JPRB, 0.62332E+00_JPRB ,&
&  0.63726E+00_JPRB, 0.64681E+00_JPRB, 0.65260E+00_JPRB, 0.66331E+00_JPRB, 0.66304E+00_JPRB, 0.63823E+00_JPRB, 0.60490E+00_JPRB ,&
&  0.57972E+00_JPRB, 0.55611E+00_JPRB, 0.54081E+00_JPRB, 0.52596E+00_JPRB, 0.51890E+00_JPRB, 0.51614E+00_JPRB, 0.50804E+00_JPRB ,&
&  0.49638E+00_JPRB, 0.49024E+00_JPRB, 0.48502E+00_JPRB, 0.48500E+00_JPRB, 0.48654E+00_JPRB, 0.48943E+00_JPRB, 0.48397E+00_JPRB ,&
&  0.47870E+00_JPRB, 0.48688E+00_JPRB, 0.50071E+00_JPRB, 0.50688E+00_JPRB, 0.50748E+00_JPRB, 0.50364E+00_JPRB, 0.49655E+00_JPRB ,&
&  0.49078E+00_JPRB, 0.48632E+00_JPRB, 0.48506E+00_JPRB, 0.48226E+00_JPRB, 0.47883E+00_JPRB, 0.47440E+00_JPRB, 0.47017E+00_JPRB ,&
&  0.46691E+00_JPRB, 0.46442E+00_JPRB, 0.46208E+00_JPRB, 0.45934E+00_JPRB, 0.45534E+00_JPRB, 0.44921E+00_JPRB, 0.44617E+00_JPRB ,&
&  0.44525E+00_JPRB, 0.44689E+00_JPRB, 0.45139E+00_JPRB, 0.45926E+00_JPRB, 0.47035E+00_JPRB, 0.48102E+00_JPRB, 0.49343E+00_JPRB ,&
&  0.50512E+00_JPRB, 0.51929E+00_JPRB, 0.53324E+00_JPRB, 0.64625E+00_JPRB, 0.69549E+00_JPRB, 0.71340E+00_JPRB, 0.72663E+00_JPRB ,&
&  0.73064E+00_JPRB, 0.72566E+00_JPRB, 0.71318E+00_JPRB, 0.61271E+00_JPRB, 0.63448E+00_JPRB /)
ZBURD(:, 55,  8) = (/ &
&  0.58315E+00_JPRB, 0.57672E+00_JPRB, 0.56334E+00_JPRB, 0.54856E+00_JPRB, 0.53613E+00_JPRB, 0.52423E+00_JPRB, 0.51477E+00_JPRB ,&
&  0.50288E+00_JPRB, 0.48911E+00_JPRB, 0.47851E+00_JPRB, 0.47275E+00_JPRB, 0.46936E+00_JPRB, 0.46494E+00_JPRB, 0.45846E+00_JPRB ,&
&  0.45048E+00_JPRB, 0.44211E+00_JPRB, 0.43413E+00_JPRB, 0.42457E+00_JPRB, 0.41546E+00_JPRB, 0.40739E+00_JPRB, 0.40109E+00_JPRB ,&
&  0.39491E+00_JPRB, 0.38830E+00_JPRB, 0.38179E+00_JPRB, 0.37738E+00_JPRB, 0.37386E+00_JPRB, 0.37226E+00_JPRB, 0.38645E+00_JPRB ,&
&  0.43170E+00_JPRB, 0.37021E+00_JPRB, 0.39283E+00_JPRB, 0.42532E+00_JPRB, 0.47339E+00_JPRB, 0.53322E+00_JPRB, 0.56509E+00_JPRB ,&
&  0.58061E+00_JPRB, 0.58915E+00_JPRB, 0.58991E+00_JPRB, 0.59003E+00_JPRB, 0.59382E+00_JPRB, 0.58709E+00_JPRB, 0.56803E+00_JPRB ,&
&  0.54734E+00_JPRB, 0.52728E+00_JPRB, 0.51211E+00_JPRB, 0.50084E+00_JPRB, 0.49118E+00_JPRB, 0.48396E+00_JPRB, 0.47376E+00_JPRB ,&
&  0.45845E+00_JPRB, 0.44976E+00_JPRB, 0.44473E+00_JPRB, 0.44079E+00_JPRB, 0.43895E+00_JPRB, 0.43554E+00_JPRB, 0.42579E+00_JPRB ,&
&  0.41701E+00_JPRB, 0.41187E+00_JPRB, 0.41057E+00_JPRB, 0.41208E+00_JPRB, 0.41409E+00_JPRB, 0.41436E+00_JPRB, 0.41152E+00_JPRB ,&
&  0.40771E+00_JPRB, 0.40258E+00_JPRB, 0.39959E+00_JPRB, 0.39766E+00_JPRB, 0.39604E+00_JPRB, 0.39686E+00_JPRB, 0.39775E+00_JPRB ,&
&  0.39734E+00_JPRB, 0.39797E+00_JPRB, 0.40034E+00_JPRB, 0.40242E+00_JPRB, 0.40266E+00_JPRB, 0.40146E+00_JPRB, 0.39968E+00_JPRB ,&
&  0.39753E+00_JPRB, 0.39634E+00_JPRB, 0.39766E+00_JPRB, 0.40299E+00_JPRB, 0.41094E+00_JPRB, 0.42035E+00_JPRB, 0.43103E+00_JPRB ,&
&  0.44306E+00_JPRB, 0.45630E+00_JPRB, 0.46755E+00_JPRB, 0.51872E+00_JPRB, 0.55684E+00_JPRB, 0.59051E+00_JPRB, 0.62043E+00_JPRB ,&
&  0.63374E+00_JPRB, 0.63582E+00_JPRB, 0.52659E+00_JPRB, 0.56992E+00_JPRB, 0.57900E+00_JPRB /)
ZBURD(:, 56,  8) = (/ &
&  0.54296E+00_JPRB, 0.52930E+00_JPRB, 0.51286E+00_JPRB, 0.50093E+00_JPRB, 0.49375E+00_JPRB, 0.48609E+00_JPRB, 0.47857E+00_JPRB ,&
&  0.46882E+00_JPRB, 0.45842E+00_JPRB, 0.44839E+00_JPRB, 0.43949E+00_JPRB, 0.43257E+00_JPRB, 0.42778E+00_JPRB, 0.42255E+00_JPRB ,&
&  0.41573E+00_JPRB, 0.40823E+00_JPRB, 0.40067E+00_JPRB, 0.39224E+00_JPRB, 0.38144E+00_JPRB, 0.37275E+00_JPRB, 0.36587E+00_JPRB ,&
&  0.36048E+00_JPRB, 0.35625E+00_JPRB, 0.35458E+00_JPRB, 0.35255E+00_JPRB, 0.35032E+00_JPRB, 0.35021E+00_JPRB, 0.36195E+00_JPRB ,&
&  0.36226E+00_JPRB, 0.33649E+00_JPRB, 0.35573E+00_JPRB, 0.38211E+00_JPRB, 0.42225E+00_JPRB, 0.48329E+00_JPRB, 0.52393E+00_JPRB ,&
&  0.54574E+00_JPRB, 0.55587E+00_JPRB, 0.55852E+00_JPRB, 0.55375E+00_JPRB, 0.54809E+00_JPRB, 0.54780E+00_JPRB, 0.54220E+00_JPRB ,&
&  0.52776E+00_JPRB, 0.50945E+00_JPRB, 0.49178E+00_JPRB, 0.47861E+00_JPRB, 0.46433E+00_JPRB, 0.45283E+00_JPRB, 0.44086E+00_JPRB ,&
&  0.42933E+00_JPRB, 0.42029E+00_JPRB, 0.41492E+00_JPRB, 0.41087E+00_JPRB, 0.40588E+00_JPRB, 0.39782E+00_JPRB, 0.38916E+00_JPRB ,&
&  0.38248E+00_JPRB, 0.37682E+00_JPRB, 0.37138E+00_JPRB, 0.36646E+00_JPRB, 0.36354E+00_JPRB, 0.36169E+00_JPRB, 0.35962E+00_JPRB ,&
&  0.35898E+00_JPRB, 0.35902E+00_JPRB, 0.35949E+00_JPRB, 0.35965E+00_JPRB, 0.35898E+00_JPRB, 0.35747E+00_JPRB, 0.35749E+00_JPRB ,&
&  0.35854E+00_JPRB, 0.35704E+00_JPRB, 0.35651E+00_JPRB, 0.35910E+00_JPRB, 0.36221E+00_JPRB, 0.36416E+00_JPRB, 0.36461E+00_JPRB ,&
&  0.36358E+00_JPRB, 0.36324E+00_JPRB, 0.36437E+00_JPRB, 0.36752E+00_JPRB, 0.37172E+00_JPRB, 0.37790E+00_JPRB, 0.38551E+00_JPRB ,&
&  0.39358E+00_JPRB, 0.40384E+00_JPRB, 0.41684E+00_JPRB, 0.44136E+00_JPRB, 0.46568E+00_JPRB, 0.48987E+00_JPRB, 0.51458E+00_JPRB ,&
&  0.52875E+00_JPRB, 0.53917E+00_JPRB, 0.55003E+00_JPRB, 0.55017E+00_JPRB, 0.54876E+00_JPRB /)
ZBURD(:, 57,  8) = (/ &
&  0.46533E+00_JPRB, 0.45851E+00_JPRB, 0.44960E+00_JPRB, 0.44165E+00_JPRB, 0.43726E+00_JPRB, 0.43285E+00_JPRB, 0.42574E+00_JPRB ,&
&  0.41902E+00_JPRB, 0.41287E+00_JPRB, 0.40692E+00_JPRB, 0.40165E+00_JPRB, 0.39718E+00_JPRB, 0.39375E+00_JPRB, 0.38913E+00_JPRB ,&
&  0.38279E+00_JPRB, 0.37527E+00_JPRB, 0.36707E+00_JPRB, 0.36035E+00_JPRB, 0.35226E+00_JPRB, 0.34393E+00_JPRB, 0.33765E+00_JPRB ,&
&  0.33234E+00_JPRB, 0.32860E+00_JPRB, 0.32704E+00_JPRB, 0.32492E+00_JPRB, 0.32542E+00_JPRB, 0.32830E+00_JPRB, 0.33729E+00_JPRB ,&
&  0.35154E+00_JPRB, 0.33289E+00_JPRB, 0.35115E+00_JPRB, 0.36785E+00_JPRB, 0.40118E+00_JPRB, 0.44858E+00_JPRB, 0.48068E+00_JPRB ,&
&  0.50122E+00_JPRB, 0.51751E+00_JPRB, 0.52902E+00_JPRB, 0.53106E+00_JPRB, 0.52328E+00_JPRB, 0.51853E+00_JPRB, 0.51537E+00_JPRB ,&
&  0.50647E+00_JPRB, 0.49261E+00_JPRB, 0.47371E+00_JPRB, 0.45373E+00_JPRB, 0.43559E+00_JPRB, 0.42209E+00_JPRB, 0.41152E+00_JPRB ,&
&  0.40313E+00_JPRB, 0.39568E+00_JPRB, 0.38937E+00_JPRB, 0.38089E+00_JPRB, 0.37120E+00_JPRB, 0.36129E+00_JPRB, 0.35218E+00_JPRB ,&
&  0.34324E+00_JPRB, 0.33456E+00_JPRB, 0.32699E+00_JPRB, 0.32071E+00_JPRB, 0.31722E+00_JPRB, 0.31553E+00_JPRB, 0.31488E+00_JPRB ,&
&  0.31521E+00_JPRB, 0.31708E+00_JPRB, 0.31914E+00_JPRB, 0.32067E+00_JPRB, 0.32154E+00_JPRB, 0.32083E+00_JPRB, 0.31825E+00_JPRB ,&
&  0.31763E+00_JPRB, 0.31737E+00_JPRB, 0.31740E+00_JPRB, 0.31842E+00_JPRB, 0.31869E+00_JPRB, 0.31938E+00_JPRB, 0.32185E+00_JPRB ,&
&  0.32424E+00_JPRB, 0.32569E+00_JPRB, 0.32713E+00_JPRB, 0.32891E+00_JPRB, 0.33123E+00_JPRB, 0.33493E+00_JPRB, 0.34022E+00_JPRB ,&
&  0.34625E+00_JPRB, 0.35310E+00_JPRB, 0.36312E+00_JPRB, 0.37636E+00_JPRB, 0.39065E+00_JPRB, 0.40575E+00_JPRB, 0.42139E+00_JPRB ,&
&  0.43217E+00_JPRB, 0.44325E+00_JPRB, 0.45641E+00_JPRB, 0.46299E+00_JPRB, 0.46705E+00_JPRB /)
ZBURD(:, 58,  8) = (/ &
&  0.40244E+00_JPRB, 0.40025E+00_JPRB, 0.39703E+00_JPRB, 0.39398E+00_JPRB, 0.39180E+00_JPRB, 0.38711E+00_JPRB, 0.38117E+00_JPRB ,&
&  0.37690E+00_JPRB, 0.37295E+00_JPRB, 0.36986E+00_JPRB, 0.36556E+00_JPRB, 0.36282E+00_JPRB, 0.36085E+00_JPRB, 0.35613E+00_JPRB ,&
&  0.35086E+00_JPRB, 0.34380E+00_JPRB, 0.33698E+00_JPRB, 0.33088E+00_JPRB, 0.32508E+00_JPRB, 0.31964E+00_JPRB, 0.31354E+00_JPRB ,&
&  0.30770E+00_JPRB, 0.30324E+00_JPRB, 0.30064E+00_JPRB, 0.29838E+00_JPRB, 0.29947E+00_JPRB, 0.30522E+00_JPRB, 0.31488E+00_JPRB ,&
&  0.32221E+00_JPRB, 0.32785E+00_JPRB, 0.34058E+00_JPRB, 0.35385E+00_JPRB, 0.37894E+00_JPRB, 0.41386E+00_JPRB, 0.43837E+00_JPRB ,&
&  0.45340E+00_JPRB, 0.47008E+00_JPRB, 0.48928E+00_JPRB, 0.50114E+00_JPRB, 0.50206E+00_JPRB, 0.49532E+00_JPRB, 0.49015E+00_JPRB ,&
&  0.48350E+00_JPRB, 0.47097E+00_JPRB, 0.45740E+00_JPRB, 0.44088E+00_JPRB, 0.42350E+00_JPRB, 0.40912E+00_JPRB, 0.39772E+00_JPRB ,&
&  0.38744E+00_JPRB, 0.37759E+00_JPRB, 0.36826E+00_JPRB, 0.35858E+00_JPRB, 0.34809E+00_JPRB, 0.33688E+00_JPRB, 0.32500E+00_JPRB ,&
&  0.31352E+00_JPRB, 0.30378E+00_JPRB, 0.29627E+00_JPRB, 0.29223E+00_JPRB, 0.28994E+00_JPRB, 0.28801E+00_JPRB, 0.28666E+00_JPRB ,&
&  0.28680E+00_JPRB, 0.28804E+00_JPRB, 0.28999E+00_JPRB, 0.29196E+00_JPRB, 0.29362E+00_JPRB, 0.29382E+00_JPRB, 0.29167E+00_JPRB ,&
&  0.28942E+00_JPRB, 0.28873E+00_JPRB, 0.28864E+00_JPRB, 0.28612E+00_JPRB, 0.28343E+00_JPRB, 0.28397E+00_JPRB, 0.28628E+00_JPRB ,&
&  0.29045E+00_JPRB, 0.29427E+00_JPRB, 0.29746E+00_JPRB, 0.30006E+00_JPRB, 0.30199E+00_JPRB, 0.30538E+00_JPRB, 0.30960E+00_JPRB ,&
&  0.31487E+00_JPRB, 0.32141E+00_JPRB, 0.33059E+00_JPRB, 0.34188E+00_JPRB, 0.35260E+00_JPRB, 0.36237E+00_JPRB, 0.36992E+00_JPRB ,&
&  0.37563E+00_JPRB, 0.38122E+00_JPRB, 0.38756E+00_JPRB, 0.39435E+00_JPRB, 0.39998E+00_JPRB /)
ZBURD(:, 59,  8) = (/ &
&  0.34929E+00_JPRB, 0.34991E+00_JPRB, 0.35092E+00_JPRB, 0.35122E+00_JPRB, 0.34838E+00_JPRB, 0.34486E+00_JPRB, 0.34097E+00_JPRB ,&
&  0.33903E+00_JPRB, 0.33761E+00_JPRB, 0.33691E+00_JPRB, 0.33349E+00_JPRB, 0.33068E+00_JPRB, 0.32873E+00_JPRB, 0.32461E+00_JPRB ,&
&  0.32084E+00_JPRB, 0.31308E+00_JPRB, 0.30491E+00_JPRB, 0.29893E+00_JPRB, 0.29516E+00_JPRB, 0.29251E+00_JPRB, 0.28717E+00_JPRB ,&
&  0.28326E+00_JPRB, 0.28010E+00_JPRB, 0.27844E+00_JPRB, 0.27834E+00_JPRB, 0.28028E+00_JPRB, 0.28601E+00_JPRB, 0.29572E+00_JPRB ,&
&  0.30292E+00_JPRB, 0.31369E+00_JPRB, 0.31475E+00_JPRB, 0.33022E+00_JPRB, 0.34636E+00_JPRB, 0.36942E+00_JPRB, 0.38978E+00_JPRB ,&
&  0.40302E+00_JPRB, 0.41292E+00_JPRB, 0.42682E+00_JPRB, 0.44162E+00_JPRB, 0.44854E+00_JPRB, 0.45061E+00_JPRB, 0.44881E+00_JPRB ,&
&  0.44470E+00_JPRB, 0.43873E+00_JPRB, 0.43178E+00_JPRB, 0.42302E+00_JPRB, 0.41096E+00_JPRB, 0.39714E+00_JPRB, 0.38346E+00_JPRB ,&
&  0.37102E+00_JPRB, 0.35884E+00_JPRB, 0.34677E+00_JPRB, 0.33367E+00_JPRB, 0.32047E+00_JPRB, 0.30815E+00_JPRB, 0.29590E+00_JPRB ,&
&  0.28575E+00_JPRB, 0.27790E+00_JPRB, 0.27208E+00_JPRB, 0.26788E+00_JPRB, 0.26473E+00_JPRB, 0.26173E+00_JPRB, 0.26027E+00_JPRB ,&
&  0.26031E+00_JPRB, 0.26189E+00_JPRB, 0.26426E+00_JPRB, 0.26619E+00_JPRB, 0.26852E+00_JPRB, 0.26888E+00_JPRB, 0.26597E+00_JPRB ,&
&  0.26297E+00_JPRB, 0.26162E+00_JPRB, 0.25720E+00_JPRB, 0.25456E+00_JPRB, 0.25398E+00_JPRB, 0.25448E+00_JPRB, 0.25647E+00_JPRB ,&
&  0.26042E+00_JPRB, 0.26584E+00_JPRB, 0.27078E+00_JPRB, 0.27503E+00_JPRB, 0.27869E+00_JPRB, 0.28248E+00_JPRB, 0.28692E+00_JPRB ,&
&  0.29163E+00_JPRB, 0.29622E+00_JPRB, 0.30334E+00_JPRB, 0.31122E+00_JPRB, 0.31803E+00_JPRB, 0.32555E+00_JPRB, 0.33079E+00_JPRB ,&
&  0.33345E+00_JPRB, 0.33507E+00_JPRB, 0.33857E+00_JPRB, 0.34355E+00_JPRB, 0.34754E+00_JPRB /)
ZBURD(:, 60,  8) = (/ &
&  0.30936E+00_JPRB, 0.31161E+00_JPRB, 0.31397E+00_JPRB, 0.31423E+00_JPRB, 0.31344E+00_JPRB, 0.31323E+00_JPRB, 0.31204E+00_JPRB ,&
&  0.30967E+00_JPRB, 0.30831E+00_JPRB, 0.30849E+00_JPRB, 0.30697E+00_JPRB, 0.30385E+00_JPRB, 0.29998E+00_JPRB, 0.29573E+00_JPRB ,&
&  0.29238E+00_JPRB, 0.28907E+00_JPRB, 0.28375E+00_JPRB, 0.27683E+00_JPRB, 0.27087E+00_JPRB, 0.26698E+00_JPRB, 0.26487E+00_JPRB ,&
&  0.26387E+00_JPRB, 0.26367E+00_JPRB, 0.26311E+00_JPRB, 0.26358E+00_JPRB, 0.26559E+00_JPRB, 0.27037E+00_JPRB, 0.27752E+00_JPRB ,&
&  0.28469E+00_JPRB, 0.29175E+00_JPRB, 0.29901E+00_JPRB, 0.30707E+00_JPRB, 0.31676E+00_JPRB, 0.32912E+00_JPRB, 0.34117E+00_JPRB ,&
&  0.35267E+00_JPRB, 0.35687E+00_JPRB, 0.35871E+00_JPRB, 0.36448E+00_JPRB, 0.37469E+00_JPRB, 0.38228E+00_JPRB, 0.38718E+00_JPRB ,&
&  0.38977E+00_JPRB, 0.38883E+00_JPRB, 0.38642E+00_JPRB, 0.38080E+00_JPRB, 0.37244E+00_JPRB, 0.36162E+00_JPRB, 0.34991E+00_JPRB ,&
&  0.33838E+00_JPRB, 0.32648E+00_JPRB, 0.31371E+00_JPRB, 0.30141E+00_JPRB, 0.29064E+00_JPRB, 0.28093E+00_JPRB, 0.27143E+00_JPRB ,&
&  0.26284E+00_JPRB, 0.25603E+00_JPRB, 0.25066E+00_JPRB, 0.24694E+00_JPRB, 0.24392E+00_JPRB, 0.24157E+00_JPRB, 0.24086E+00_JPRB ,&
&  0.24114E+00_JPRB, 0.24248E+00_JPRB, 0.24319E+00_JPRB, 0.24466E+00_JPRB, 0.24712E+00_JPRB, 0.24809E+00_JPRB, 0.24678E+00_JPRB ,&
&  0.24273E+00_JPRB, 0.23859E+00_JPRB, 0.23459E+00_JPRB, 0.23380E+00_JPRB, 0.23367E+00_JPRB, 0.23489E+00_JPRB, 0.23679E+00_JPRB ,&
&  0.24059E+00_JPRB, 0.24608E+00_JPRB, 0.25099E+00_JPRB, 0.25450E+00_JPRB, 0.25839E+00_JPRB, 0.26181E+00_JPRB, 0.26521E+00_JPRB ,&
&  0.26901E+00_JPRB, 0.27310E+00_JPRB, 0.27858E+00_JPRB, 0.28402E+00_JPRB, 0.28896E+00_JPRB, 0.29301E+00_JPRB, 0.29621E+00_JPRB ,&
&  0.29863E+00_JPRB, 0.29970E+00_JPRB, 0.30059E+00_JPRB, 0.30292E+00_JPRB, 0.30641E+00_JPRB /)
ZBURD(:, 61,  8) = (/ &
&  0.27555E+00_JPRB, 0.27815E+00_JPRB, 0.28133E+00_JPRB, 0.28382E+00_JPRB, 0.28472E+00_JPRB, 0.28422E+00_JPRB, 0.28346E+00_JPRB ,&
&  0.28334E+00_JPRB, 0.28331E+00_JPRB, 0.28189E+00_JPRB, 0.27927E+00_JPRB, 0.27649E+00_JPRB, 0.27355E+00_JPRB, 0.27131E+00_JPRB ,&
&  0.26920E+00_JPRB, 0.26699E+00_JPRB, 0.26314E+00_JPRB, 0.25746E+00_JPRB, 0.25158E+00_JPRB, 0.24812E+00_JPRB, 0.24643E+00_JPRB ,&
&  0.24625E+00_JPRB, 0.24672E+00_JPRB, 0.24917E+00_JPRB, 0.25089E+00_JPRB, 0.25328E+00_JPRB, 0.25736E+00_JPRB, 0.26303E+00_JPRB ,&
&  0.26838E+00_JPRB, 0.27349E+00_JPRB, 0.27882E+00_JPRB, 0.28346E+00_JPRB, 0.28809E+00_JPRB, 0.29401E+00_JPRB, 0.30070E+00_JPRB ,&
&  0.30868E+00_JPRB, 0.31422E+00_JPRB, 0.31482E+00_JPRB, 0.31567E+00_JPRB, 0.31899E+00_JPRB, 0.32320E+00_JPRB, 0.32628E+00_JPRB ,&
&  0.32864E+00_JPRB, 0.32764E+00_JPRB, 0.32531E+00_JPRB, 0.32141E+00_JPRB, 0.31691E+00_JPRB, 0.31165E+00_JPRB, 0.30457E+00_JPRB ,&
&  0.29608E+00_JPRB, 0.28686E+00_JPRB, 0.27797E+00_JPRB, 0.26944E+00_JPRB, 0.26118E+00_JPRB, 0.25356E+00_JPRB, 0.24674E+00_JPRB ,&
&  0.24033E+00_JPRB, 0.23537E+00_JPRB, 0.23064E+00_JPRB, 0.22773E+00_JPRB, 0.22591E+00_JPRB, 0.22390E+00_JPRB, 0.22349E+00_JPRB ,&
&  0.22259E+00_JPRB, 0.22210E+00_JPRB, 0.22199E+00_JPRB, 0.22358E+00_JPRB, 0.22663E+00_JPRB, 0.22845E+00_JPRB, 0.22720E+00_JPRB ,&
&  0.22389E+00_JPRB, 0.22180E+00_JPRB, 0.22054E+00_JPRB, 0.21959E+00_JPRB, 0.21729E+00_JPRB, 0.21705E+00_JPRB, 0.21907E+00_JPRB ,&
&  0.22332E+00_JPRB, 0.22803E+00_JPRB, 0.23208E+00_JPRB, 0.23454E+00_JPRB, 0.23706E+00_JPRB, 0.23913E+00_JPRB, 0.24095E+00_JPRB ,&
&  0.24288E+00_JPRB, 0.24427E+00_JPRB, 0.24531E+00_JPRB, 0.24676E+00_JPRB, 0.24917E+00_JPRB, 0.25160E+00_JPRB, 0.25406E+00_JPRB ,&
&  0.25684E+00_JPRB, 0.26104E+00_JPRB, 0.26533E+00_JPRB, 0.26868E+00_JPRB, 0.27284E+00_JPRB /)
ZBURD(:, 62,  8) = (/ &
&  0.25297E+00_JPRB, 0.25568E+00_JPRB, 0.25847E+00_JPRB, 0.25873E+00_JPRB, 0.25704E+00_JPRB, 0.25618E+00_JPRB, 0.25639E+00_JPRB ,&
&  0.25692E+00_JPRB, 0.25709E+00_JPRB, 0.25583E+00_JPRB, 0.25566E+00_JPRB, 0.25567E+00_JPRB, 0.25471E+00_JPRB, 0.25398E+00_JPRB ,&
&  0.25261E+00_JPRB, 0.24954E+00_JPRB, 0.24515E+00_JPRB, 0.23978E+00_JPRB, 0.23575E+00_JPRB, 0.23360E+00_JPRB, 0.23322E+00_JPRB ,&
&  0.23388E+00_JPRB, 0.23439E+00_JPRB, 0.23689E+00_JPRB, 0.23904E+00_JPRB, 0.24173E+00_JPRB, 0.24534E+00_JPRB, 0.24917E+00_JPRB ,&
&  0.25199E+00_JPRB, 0.25495E+00_JPRB, 0.25793E+00_JPRB, 0.26075E+00_JPRB, 0.26445E+00_JPRB, 0.26997E+00_JPRB, 0.27472E+00_JPRB ,&
&  0.28036E+00_JPRB, 0.28360E+00_JPRB, 0.28088E+00_JPRB, 0.27759E+00_JPRB, 0.27749E+00_JPRB, 0.27946E+00_JPRB, 0.28176E+00_JPRB ,&
&  0.28387E+00_JPRB, 0.28600E+00_JPRB, 0.28772E+00_JPRB, 0.28757E+00_JPRB, 0.28493E+00_JPRB, 0.28006E+00_JPRB, 0.27347E+00_JPRB ,&
&  0.26616E+00_JPRB, 0.25874E+00_JPRB, 0.25156E+00_JPRB, 0.24433E+00_JPRB, 0.23809E+00_JPRB, 0.23215E+00_JPRB, 0.22675E+00_JPRB ,&
&  0.22171E+00_JPRB, 0.21693E+00_JPRB, 0.21328E+00_JPRB, 0.21096E+00_JPRB, 0.21019E+00_JPRB, 0.20896E+00_JPRB, 0.20887E+00_JPRB ,&
&  0.20831E+00_JPRB, 0.20813E+00_JPRB, 0.20826E+00_JPRB, 0.20908E+00_JPRB, 0.21056E+00_JPRB, 0.21102E+00_JPRB, 0.21010E+00_JPRB ,&
&  0.20957E+00_JPRB, 0.21077E+00_JPRB, 0.21052E+00_JPRB, 0.20815E+00_JPRB, 0.20522E+00_JPRB, 0.20454E+00_JPRB, 0.20598E+00_JPRB ,&
&  0.20873E+00_JPRB, 0.21151E+00_JPRB, 0.21392E+00_JPRB, 0.21605E+00_JPRB, 0.21783E+00_JPRB, 0.22001E+00_JPRB, 0.22202E+00_JPRB ,&
&  0.22372E+00_JPRB, 0.22459E+00_JPRB, 0.22613E+00_JPRB, 0.22777E+00_JPRB, 0.22948E+00_JPRB, 0.23072E+00_JPRB, 0.23067E+00_JPRB ,&
&  0.23033E+00_JPRB, 0.23206E+00_JPRB, 0.23711E+00_JPRB, 0.24470E+00_JPRB, 0.24977E+00_JPRB /)
ZBURD(:, 63,  8) = (/ &
&  0.23430E+00_JPRB, 0.23676E+00_JPRB, 0.23816E+00_JPRB, 0.23802E+00_JPRB, 0.23743E+00_JPRB, 0.23814E+00_JPRB, 0.23873E+00_JPRB ,&
&  0.23948E+00_JPRB, 0.23970E+00_JPRB, 0.23848E+00_JPRB, 0.23740E+00_JPRB, 0.23905E+00_JPRB, 0.23868E+00_JPRB, 0.23822E+00_JPRB ,&
&  0.23656E+00_JPRB, 0.23446E+00_JPRB, 0.23143E+00_JPRB, 0.22795E+00_JPRB, 0.22375E+00_JPRB, 0.22117E+00_JPRB, 0.22141E+00_JPRB ,&
&  0.22291E+00_JPRB, 0.22348E+00_JPRB, 0.22426E+00_JPRB, 0.22531E+00_JPRB, 0.22753E+00_JPRB, 0.23040E+00_JPRB, 0.23256E+00_JPRB ,&
&  0.23429E+00_JPRB, 0.23743E+00_JPRB, 0.23941E+00_JPRB, 0.23844E+00_JPRB, 0.24267E+00_JPRB, 0.24680E+00_JPRB, 0.25059E+00_JPRB ,&
&  0.25414E+00_JPRB, 0.25625E+00_JPRB, 0.25626E+00_JPRB, 0.25515E+00_JPRB, 0.25435E+00_JPRB, 0.25541E+00_JPRB, 0.25603E+00_JPRB ,&
&  0.25679E+00_JPRB, 0.25946E+00_JPRB, 0.26258E+00_JPRB, 0.26381E+00_JPRB, 0.26242E+00_JPRB, 0.25824E+00_JPRB, 0.25149E+00_JPRB ,&
&  0.24265E+00_JPRB, 0.23324E+00_JPRB, 0.22550E+00_JPRB, 0.21929E+00_JPRB, 0.21451E+00_JPRB, 0.21068E+00_JPRB, 0.20756E+00_JPRB ,&
&  0.20410E+00_JPRB, 0.20163E+00_JPRB, 0.20147E+00_JPRB, 0.20200E+00_JPRB, 0.20202E+00_JPRB, 0.20032E+00_JPRB, 0.19912E+00_JPRB ,&
&  0.19805E+00_JPRB, 0.19851E+00_JPRB, 0.19944E+00_JPRB, 0.19979E+00_JPRB, 0.19996E+00_JPRB, 0.19973E+00_JPRB, 0.19949E+00_JPRB ,&
&  0.20034E+00_JPRB, 0.20040E+00_JPRB, 0.19935E+00_JPRB, 0.19751E+00_JPRB, 0.19593E+00_JPRB, 0.19566E+00_JPRB, 0.19598E+00_JPRB ,&
&  0.19669E+00_JPRB, 0.19801E+00_JPRB, 0.19986E+00_JPRB, 0.20175E+00_JPRB, 0.20324E+00_JPRB, 0.20391E+00_JPRB, 0.20419E+00_JPRB ,&
&  0.20443E+00_JPRB, 0.20438E+00_JPRB, 0.20546E+00_JPRB, 0.20865E+00_JPRB, 0.21193E+00_JPRB, 0.21450E+00_JPRB, 0.21589E+00_JPRB ,&
&  0.21551E+00_JPRB, 0.21631E+00_JPRB, 0.21968E+00_JPRB, 0.22595E+00_JPRB, 0.23061E+00_JPRB /)
ZBURD(:, 64,  8) = (/ &
&  0.22030E+00_JPRB, 0.22259E+00_JPRB, 0.22341E+00_JPRB, 0.22304E+00_JPRB, 0.22255E+00_JPRB, 0.22351E+00_JPRB, 0.22486E+00_JPRB ,&
&  0.22561E+00_JPRB, 0.22578E+00_JPRB, 0.22491E+00_JPRB, 0.22312E+00_JPRB, 0.22183E+00_JPRB, 0.22218E+00_JPRB, 0.22067E+00_JPRB ,&
&  0.21915E+00_JPRB, 0.21916E+00_JPRB, 0.21760E+00_JPRB, 0.21588E+00_JPRB, 0.21345E+00_JPRB, 0.21209E+00_JPRB, 0.21269E+00_JPRB ,&
&  0.21480E+00_JPRB, 0.21540E+00_JPRB, 0.21606E+00_JPRB, 0.21698E+00_JPRB, 0.21872E+00_JPRB, 0.22083E+00_JPRB, 0.22222E+00_JPRB ,&
&  0.22321E+00_JPRB, 0.22236E+00_JPRB, 0.22084E+00_JPRB, 0.22353E+00_JPRB, 0.22683E+00_JPRB, 0.23109E+00_JPRB, 0.23399E+00_JPRB ,&
&  0.23649E+00_JPRB, 0.23829E+00_JPRB, 0.23908E+00_JPRB, 0.23899E+00_JPRB, 0.23938E+00_JPRB, 0.24126E+00_JPRB, 0.24268E+00_JPRB ,&
&  0.24321E+00_JPRB, 0.24299E+00_JPRB, 0.24171E+00_JPRB, 0.24050E+00_JPRB, 0.23727E+00_JPRB, 0.23225E+00_JPRB, 0.22454E+00_JPRB ,&
&  0.21642E+00_JPRB, 0.20980E+00_JPRB, 0.20477E+00_JPRB, 0.20074E+00_JPRB, 0.19794E+00_JPRB, 0.19761E+00_JPRB, 0.19749E+00_JPRB ,&
&  0.19696E+00_JPRB, 0.19644E+00_JPRB, 0.19774E+00_JPRB, 0.19836E+00_JPRB, 0.19415E+00_JPRB, 0.18906E+00_JPRB, 0.18446E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18421E+00_JPRB, 0.18561E+00_JPRB, 0.18787E+00_JPRB, 0.19142E+00_JPRB, 0.19225E+00_JPRB, 0.19117E+00_JPRB ,&
&  0.18565E+00_JPRB, 0.18042E+00_JPRB, 0.17733E+00_JPRB, 0.17484E+00_JPRB, 0.17334E+00_JPRB, 0.17334E+00_JPRB, 0.17407E+00_JPRB ,&
&  0.17626E+00_JPRB, 0.17809E+00_JPRB, 0.18050E+00_JPRB, 0.18175E+00_JPRB, 0.18153E+00_JPRB, 0.18124E+00_JPRB, 0.18052E+00_JPRB ,&
&  0.18189E+00_JPRB, 0.18631E+00_JPRB, 0.19300E+00_JPRB, 0.19623E+00_JPRB, 0.20034E+00_JPRB, 0.20314E+00_JPRB, 0.20447E+00_JPRB ,&
&  0.20537E+00_JPRB, 0.20590E+00_JPRB, 0.20643E+00_JPRB, 0.21074E+00_JPRB, 0.21658E+00_JPRB /)
ZBURD(:, 65,  8) = (/ &
&  0.20872E+00_JPRB, 0.21054E+00_JPRB, 0.21116E+00_JPRB, 0.21116E+00_JPRB, 0.21099E+00_JPRB, 0.21148E+00_JPRB, 0.21203E+00_JPRB ,&
&  0.21205E+00_JPRB, 0.21138E+00_JPRB, 0.21025E+00_JPRB, 0.20832E+00_JPRB, 0.20594E+00_JPRB, 0.20454E+00_JPRB, 0.20556E+00_JPRB ,&
&  0.20657E+00_JPRB, 0.20651E+00_JPRB, 0.20660E+00_JPRB, 0.20645E+00_JPRB, 0.20605E+00_JPRB, 0.20631E+00_JPRB, 0.20750E+00_JPRB ,&
&  0.20878E+00_JPRB, 0.20955E+00_JPRB, 0.21088E+00_JPRB, 0.21199E+00_JPRB, 0.21404E+00_JPRB, 0.21590E+00_JPRB, 0.21655E+00_JPRB ,&
&  0.21384E+00_JPRB, 0.20613E+00_JPRB, 0.20094E+00_JPRB, 0.20350E+00_JPRB, 0.21153E+00_JPRB, 0.21676E+00_JPRB, 0.22003E+00_JPRB ,&
&  0.22218E+00_JPRB, 0.22321E+00_JPRB, 0.22342E+00_JPRB, 0.22471E+00_JPRB, 0.22673E+00_JPRB, 0.22873E+00_JPRB, 0.22842E+00_JPRB ,&
&  0.22650E+00_JPRB, 0.22485E+00_JPRB, 0.22244E+00_JPRB, 0.21818E+00_JPRB, 0.21233E+00_JPRB, 0.20655E+00_JPRB, 0.20156E+00_JPRB ,&
&  0.19705E+00_JPRB, 0.19267E+00_JPRB, 0.18902E+00_JPRB, 0.18667E+00_JPRB, 0.18599E+00_JPRB, 0.18729E+00_JPRB, 0.18695E+00_JPRB ,&
&  0.18696E+00_JPRB, 0.18511E+00_JPRB, 0.18290E+00_JPRB, 0.18024E+00_JPRB, 0.17792E+00_JPRB, 0.17375E+00_JPRB, 0.16860E+00_JPRB ,&
&  0.16596E+00_JPRB, 0.16839E+00_JPRB, 0.17513E+00_JPRB, 0.18049E+00_JPRB, 0.17990E+00_JPRB, 0.17310E+00_JPRB, 0.16943E+00_JPRB ,&
&  0.16547E+00_JPRB, 0.16141E+00_JPRB, 0.15880E+00_JPRB, 0.15819E+00_JPRB, 0.15836E+00_JPRB, 0.15857E+00_JPRB, 0.15954E+00_JPRB ,&
&  0.16113E+00_JPRB, 0.16255E+00_JPRB, 0.16334E+00_JPRB, 0.16424E+00_JPRB, 0.16463E+00_JPRB, 0.16472E+00_JPRB, 0.16485E+00_JPRB ,&
&  0.16647E+00_JPRB, 0.16984E+00_JPRB, 0.17437E+00_JPRB, 0.17944E+00_JPRB, 0.18331E+00_JPRB, 0.18685E+00_JPRB, 0.18998E+00_JPRB ,&
&  0.19317E+00_JPRB, 0.19499E+00_JPRB, 0.19811E+00_JPRB, 0.20228E+00_JPRB, 0.20577E+00_JPRB /)
ZBURD(:, 66,  8) = (/ &
&  0.19568E+00_JPRB, 0.19944E+00_JPRB, 0.20094E+00_JPRB, 0.20094E+00_JPRB, 0.19989E+00_JPRB, 0.19899E+00_JPRB, 0.19881E+00_JPRB ,&
&  0.19869E+00_JPRB, 0.19761E+00_JPRB, 0.19681E+00_JPRB, 0.19622E+00_JPRB, 0.19555E+00_JPRB, 0.19471E+00_JPRB, 0.19498E+00_JPRB ,&
&  0.19618E+00_JPRB, 0.19770E+00_JPRB, 0.19881E+00_JPRB, 0.19929E+00_JPRB, 0.19887E+00_JPRB, 0.19920E+00_JPRB, 0.20002E+00_JPRB ,&
&  0.19997E+00_JPRB, 0.20011E+00_JPRB, 0.20331E+00_JPRB, 0.20364E+00_JPRB, 0.20455E+00_JPRB, 0.20459E+00_JPRB, 0.20466E+00_JPRB ,&
&  0.19987E+00_JPRB, 0.19465E+00_JPRB, 0.19196E+00_JPRB, 0.19417E+00_JPRB, 0.20225E+00_JPRB, 0.20375E+00_JPRB, 0.20722E+00_JPRB ,&
&  0.20933E+00_JPRB, 0.21036E+00_JPRB, 0.21180E+00_JPRB, 0.21305E+00_JPRB, 0.21454E+00_JPRB, 0.21378E+00_JPRB, 0.21165E+00_JPRB ,&
&  0.20885E+00_JPRB, 0.20787E+00_JPRB, 0.20462E+00_JPRB, 0.19051E+00_JPRB, 0.18193E+00_JPRB, 0.17706E+00_JPRB, 0.17195E+00_JPRB ,&
&  0.16726E+00_JPRB, 0.16471E+00_JPRB, 0.16408E+00_JPRB, 0.16273E+00_JPRB, 0.16361E+00_JPRB, 0.16568E+00_JPRB, 0.16653E+00_JPRB ,&
&  0.16619E+00_JPRB, 0.16659E+00_JPRB, 0.16627E+00_JPRB, 0.16524E+00_JPRB, 0.16340E+00_JPRB, 0.16223E+00_JPRB, 0.16155E+00_JPRB ,&
&  0.16203E+00_JPRB, 0.16173E+00_JPRB, 0.16566E+00_JPRB, 0.16686E+00_JPRB, 0.16181E+00_JPRB, 0.16053E+00_JPRB, 0.15881E+00_JPRB ,&
&  0.15373E+00_JPRB, 0.15089E+00_JPRB, 0.14933E+00_JPRB, 0.14873E+00_JPRB, 0.14973E+00_JPRB, 0.15114E+00_JPRB, 0.15183E+00_JPRB ,&
&  0.15206E+00_JPRB, 0.15311E+00_JPRB, 0.15364E+00_JPRB, 0.15424E+00_JPRB, 0.15464E+00_JPRB, 0.15550E+00_JPRB, 0.15676E+00_JPRB ,&
&  0.15833E+00_JPRB, 0.16117E+00_JPRB, 0.16397E+00_JPRB, 0.16711E+00_JPRB, 0.16897E+00_JPRB, 0.17043E+00_JPRB, 0.17380E+00_JPRB ,&
&  0.17580E+00_JPRB, 0.18289E+00_JPRB, 0.19317E+00_JPRB, 0.19608E+00_JPRB, 0.19107E+00_JPRB /)
ZBURD(:, 67,  8) = (/ &
&  0.18825E+00_JPRB, 0.19013E+00_JPRB, 0.19171E+00_JPRB, 0.19079E+00_JPRB, 0.19098E+00_JPRB, 0.19097E+00_JPRB, 0.19105E+00_JPRB ,&
&  0.19102E+00_JPRB, 0.19042E+00_JPRB, 0.19070E+00_JPRB, 0.18886E+00_JPRB, 0.18781E+00_JPRB, 0.18429E+00_JPRB, 0.18311E+00_JPRB ,&
&  0.18125E+00_JPRB, 0.18084E+00_JPRB, 0.18112E+00_JPRB, 0.18235E+00_JPRB, 0.18279E+00_JPRB, 0.18409E+00_JPRB, 0.18329E+00_JPRB ,&
&  0.18349E+00_JPRB, 0.18353E+00_JPRB, 0.18226E+00_JPRB, 0.18190E+00_JPRB, 0.18231E+00_JPRB, 0.18272E+00_JPRB, 0.18305E+00_JPRB ,&
&  0.18505E+00_JPRB, 0.18735E+00_JPRB, 0.18955E+00_JPRB, 0.19087E+00_JPRB, 0.19583E+00_JPRB, 0.19726E+00_JPRB, 0.19930E+00_JPRB ,&
&  0.20040E+00_JPRB, 0.20179E+00_JPRB, 0.20286E+00_JPRB, 0.20222E+00_JPRB, 0.20101E+00_JPRB, 0.19730E+00_JPRB, 0.19596E+00_JPRB ,&
&  0.18595E+00_JPRB, 0.17911E+00_JPRB, 0.17235E+00_JPRB, 0.16630E+00_JPRB, 0.16184E+00_JPRB, 0.15876E+00_JPRB, 0.15626E+00_JPRB ,&
&  0.15420E+00_JPRB, 0.15219E+00_JPRB, 0.15000E+00_JPRB, 0.15007E+00_JPRB, 0.15084E+00_JPRB, 0.15202E+00_JPRB, 0.15320E+00_JPRB ,&
&  0.15458E+00_JPRB, 0.15537E+00_JPRB, 0.15571E+00_JPRB, 0.15643E+00_JPRB, 0.15657E+00_JPRB, 0.15690E+00_JPRB, 0.15710E+00_JPRB ,&
&  0.15835E+00_JPRB, 0.15872E+00_JPRB, 0.15871E+00_JPRB, 0.16013E+00_JPRB, 0.15839E+00_JPRB, 0.15571E+00_JPRB, 0.15281E+00_JPRB ,&
&  0.14989E+00_JPRB, 0.14807E+00_JPRB, 0.14654E+00_JPRB, 0.14594E+00_JPRB, 0.14634E+00_JPRB, 0.14707E+00_JPRB, 0.14804E+00_JPRB ,&
&  0.14825E+00_JPRB, 0.14887E+00_JPRB, 0.14935E+00_JPRB, 0.15027E+00_JPRB, 0.15125E+00_JPRB, 0.15235E+00_JPRB, 0.15355E+00_JPRB ,&
&  0.15533E+00_JPRB, 0.15742E+00_JPRB, 0.15949E+00_JPRB, 0.16159E+00_JPRB, 0.16495E+00_JPRB, 0.16896E+00_JPRB, 0.17531E+00_JPRB ,&
&  0.18089E+00_JPRB, 0.18798E+00_JPRB, 0.18641E+00_JPRB, 0.18920E+00_JPRB, 0.18952E+00_JPRB /)
ZBURD(:, 68,  8) = (/ &
&  0.18328E+00_JPRB, 0.18412E+00_JPRB, 0.18224E+00_JPRB, 0.18262E+00_JPRB, 0.18338E+00_JPRB, 0.18381E+00_JPRB, 0.18432E+00_JPRB ,&
&  0.18188E+00_JPRB, 0.18105E+00_JPRB, 0.17875E+00_JPRB, 0.17671E+00_JPRB, 0.17422E+00_JPRB, 0.17159E+00_JPRB, 0.16959E+00_JPRB ,&
&  0.16763E+00_JPRB, 0.16688E+00_JPRB, 0.16595E+00_JPRB, 0.16672E+00_JPRB, 0.16675E+00_JPRB, 0.16842E+00_JPRB, 0.16978E+00_JPRB ,&
&  0.17052E+00_JPRB, 0.17158E+00_JPRB, 0.17027E+00_JPRB, 0.17000E+00_JPRB, 0.17172E+00_JPRB, 0.17329E+00_JPRB, 0.17592E+00_JPRB ,&
&  0.17868E+00_JPRB, 0.18183E+00_JPRB, 0.18696E+00_JPRB, 0.18884E+00_JPRB, 0.19114E+00_JPRB, 0.19255E+00_JPRB, 0.19251E+00_JPRB ,&
&  0.19224E+00_JPRB, 0.19216E+00_JPRB, 0.19302E+00_JPRB, 0.19278E+00_JPRB, 0.18596E+00_JPRB, 0.18075E+00_JPRB, 0.17522E+00_JPRB ,&
&  0.17012E+00_JPRB, 0.16523E+00_JPRB, 0.16087E+00_JPRB, 0.15793E+00_JPRB, 0.15484E+00_JPRB, 0.15262E+00_JPRB, 0.15088E+00_JPRB ,&
&  0.14912E+00_JPRB, 0.14810E+00_JPRB, 0.14678E+00_JPRB, 0.14637E+00_JPRB, 0.14665E+00_JPRB, 0.14742E+00_JPRB, 0.14849E+00_JPRB ,&
&  0.14977E+00_JPRB, 0.15067E+00_JPRB, 0.15179E+00_JPRB, 0.15283E+00_JPRB, 0.15357E+00_JPRB, 0.15404E+00_JPRB, 0.15376E+00_JPRB ,&
&  0.15396E+00_JPRB, 0.15421E+00_JPRB, 0.15375E+00_JPRB, 0.15370E+00_JPRB, 0.15256E+00_JPRB, 0.15125E+00_JPRB, 0.14990E+00_JPRB ,&
&  0.14808E+00_JPRB, 0.14669E+00_JPRB, 0.14619E+00_JPRB, 0.14591E+00_JPRB, 0.14662E+00_JPRB, 0.14768E+00_JPRB, 0.14838E+00_JPRB ,&
&  0.14874E+00_JPRB, 0.14866E+00_JPRB, 0.14870E+00_JPRB, 0.14987E+00_JPRB, 0.15109E+00_JPRB, 0.15264E+00_JPRB, 0.15425E+00_JPRB ,&
&  0.15478E+00_JPRB, 0.15611E+00_JPRB, 0.15726E+00_JPRB, 0.15869E+00_JPRB, 0.16347E+00_JPRB, 0.16723E+00_JPRB, 0.17443E+00_JPRB ,&
&  0.17791E+00_JPRB, 0.17887E+00_JPRB, 0.17814E+00_JPRB, 0.18246E+00_JPRB, 0.18288E+00_JPRB /)
ZBURD(:, 69,  8) = (/ &
&  0.18132E+00_JPRB, 0.18384E+00_JPRB, 0.18254E+00_JPRB, 0.18240E+00_JPRB, 0.18183E+00_JPRB, 0.18170E+00_JPRB, 0.18064E+00_JPRB ,&
&  0.18044E+00_JPRB, 0.17850E+00_JPRB, 0.17813E+00_JPRB, 0.17642E+00_JPRB, 0.17464E+00_JPRB, 0.17218E+00_JPRB, 0.16975E+00_JPRB ,&
&  0.16729E+00_JPRB, 0.16513E+00_JPRB, 0.16365E+00_JPRB, 0.16298E+00_JPRB, 0.16176E+00_JPRB, 0.16171E+00_JPRB, 0.16362E+00_JPRB ,&
&  0.16405E+00_JPRB, 0.16514E+00_JPRB, 0.16526E+00_JPRB, 0.16651E+00_JPRB, 0.16812E+00_JPRB, 0.16879E+00_JPRB, 0.17109E+00_JPRB ,&
&  0.17235E+00_JPRB, 0.17390E+00_JPRB, 0.17975E+00_JPRB, 0.18298E+00_JPRB, 0.18481E+00_JPRB, 0.18754E+00_JPRB, 0.18773E+00_JPRB ,&
&  0.18631E+00_JPRB, 0.18219E+00_JPRB, 0.18534E+00_JPRB, 0.18298E+00_JPRB, 0.17656E+00_JPRB, 0.17372E+00_JPRB, 0.16847E+00_JPRB ,&
&  0.16541E+00_JPRB, 0.16175E+00_JPRB, 0.15899E+00_JPRB, 0.15687E+00_JPRB, 0.15392E+00_JPRB, 0.15202E+00_JPRB, 0.14988E+00_JPRB ,&
&  0.14789E+00_JPRB, 0.14703E+00_JPRB, 0.14565E+00_JPRB, 0.14547E+00_JPRB, 0.14560E+00_JPRB, 0.14600E+00_JPRB, 0.14669E+00_JPRB ,&
&  0.14759E+00_JPRB, 0.14844E+00_JPRB, 0.14923E+00_JPRB, 0.14989E+00_JPRB, 0.15046E+00_JPRB, 0.15081E+00_JPRB, 0.15068E+00_JPRB ,&
&  0.15077E+00_JPRB, 0.15065E+00_JPRB, 0.15017E+00_JPRB, 0.14992E+00_JPRB, 0.14898E+00_JPRB, 0.14856E+00_JPRB, 0.14814E+00_JPRB ,&
&  0.14795E+00_JPRB, 0.14797E+00_JPRB, 0.14790E+00_JPRB, 0.14810E+00_JPRB, 0.14839E+00_JPRB, 0.14909E+00_JPRB, 0.14967E+00_JPRB ,&
&  0.15012E+00_JPRB, 0.15100E+00_JPRB, 0.15152E+00_JPRB, 0.15225E+00_JPRB, 0.15327E+00_JPRB, 0.15374E+00_JPRB, 0.15495E+00_JPRB ,&
&  0.15599E+00_JPRB, 0.15721E+00_JPRB, 0.15943E+00_JPRB, 0.16166E+00_JPRB, 0.16485E+00_JPRB, 0.16886E+00_JPRB, 0.17046E+00_JPRB ,&
&  0.17190E+00_JPRB, 0.17302E+00_JPRB, 0.17163E+00_JPRB, 0.17655E+00_JPRB, 0.17888E+00_JPRB /)
ZBURD(:, 70,  8) = (/ &
&  0.17494E+00_JPRB, 0.17683E+00_JPRB, 0.17805E+00_JPRB, 0.17889E+00_JPRB, 0.18015E+00_JPRB, 0.18073E+00_JPRB, 0.18124E+00_JPRB ,&
&  0.17940E+00_JPRB, 0.17881E+00_JPRB, 0.17777E+00_JPRB, 0.17647E+00_JPRB, 0.17553E+00_JPRB, 0.17319E+00_JPRB, 0.17086E+00_JPRB ,&
&  0.16828E+00_JPRB, 0.16598E+00_JPRB, 0.16457E+00_JPRB, 0.16321E+00_JPRB, 0.16183E+00_JPRB, 0.16068E+00_JPRB, 0.16249E+00_JPRB ,&
&  0.16357E+00_JPRB, 0.16424E+00_JPRB, 0.16548E+00_JPRB, 0.16538E+00_JPRB, 0.16668E+00_JPRB, 0.16874E+00_JPRB, 0.17149E+00_JPRB ,&
&  0.17328E+00_JPRB, 0.17614E+00_JPRB, 0.17783E+00_JPRB, 0.17869E+00_JPRB, 0.17987E+00_JPRB, 0.18068E+00_JPRB, 0.18039E+00_JPRB ,&
&  0.17911E+00_JPRB, 0.17739E+00_JPRB, 0.17537E+00_JPRB, 0.17278E+00_JPRB, 0.16993E+00_JPRB, 0.16718E+00_JPRB, 0.16443E+00_JPRB ,&
&  0.16242E+00_JPRB, 0.16059E+00_JPRB, 0.15865E+00_JPRB, 0.15690E+00_JPRB, 0.15523E+00_JPRB, 0.15371E+00_JPRB, 0.15219E+00_JPRB ,&
&  0.15098E+00_JPRB, 0.15003E+00_JPRB, 0.14886E+00_JPRB, 0.14823E+00_JPRB, 0.14761E+00_JPRB, 0.14733E+00_JPRB, 0.14719E+00_JPRB ,&
&  0.14757E+00_JPRB, 0.14805E+00_JPRB, 0.14854E+00_JPRB, 0.14881E+00_JPRB, 0.14907E+00_JPRB, 0.14920E+00_JPRB, 0.14945E+00_JPRB ,&
&  0.14967E+00_JPRB, 0.14992E+00_JPRB, 0.14953E+00_JPRB, 0.14917E+00_JPRB, 0.14868E+00_JPRB, 0.14814E+00_JPRB, 0.14798E+00_JPRB ,&
&  0.14826E+00_JPRB, 0.14851E+00_JPRB, 0.14837E+00_JPRB, 0.14865E+00_JPRB, 0.14915E+00_JPRB, 0.14966E+00_JPRB, 0.15093E+00_JPRB ,&
&  0.15162E+00_JPRB, 0.15229E+00_JPRB, 0.15291E+00_JPRB, 0.15340E+00_JPRB, 0.15415E+00_JPRB, 0.15476E+00_JPRB, 0.15552E+00_JPRB ,&
&  0.15643E+00_JPRB, 0.15734E+00_JPRB, 0.15858E+00_JPRB, 0.16017E+00_JPRB, 0.16393E+00_JPRB, 0.16643E+00_JPRB, 0.16861E+00_JPRB ,&
&  0.16965E+00_JPRB, 0.16636E+00_JPRB, 0.17355E+00_JPRB, 0.17274E+00_JPRB, 0.17253E+00_JPRB /)
ZBURD(:, 71,  8) = (/ &
&  0.16621E+00_JPRB, 0.16803E+00_JPRB, 0.16946E+00_JPRB, 0.17138E+00_JPRB, 0.16727E+00_JPRB, 0.17422E+00_JPRB, 0.17147E+00_JPRB ,&
&  0.16863E+00_JPRB, 0.16384E+00_JPRB, 0.16395E+00_JPRB, 0.16314E+00_JPRB, 0.16200E+00_JPRB, 0.16141E+00_JPRB, 0.16126E+00_JPRB ,&
&  0.16131E+00_JPRB, 0.16111E+00_JPRB, 0.16155E+00_JPRB, 0.16187E+00_JPRB, 0.16213E+00_JPRB, 0.16233E+00_JPRB, 0.16264E+00_JPRB ,&
&  0.16317E+00_JPRB, 0.16409E+00_JPRB, 0.16504E+00_JPRB, 0.16328E+00_JPRB, 0.16310E+00_JPRB, 0.16339E+00_JPRB, 0.16403E+00_JPRB ,&
&  0.16451E+00_JPRB, 0.16560E+00_JPRB, 0.16641E+00_JPRB, 0.16701E+00_JPRB, 0.16864E+00_JPRB, 0.16871E+00_JPRB, 0.16856E+00_JPRB ,&
&  0.16816E+00_JPRB, 0.16737E+00_JPRB, 0.16621E+00_JPRB, 0.16514E+00_JPRB, 0.16403E+00_JPRB, 0.16258E+00_JPRB, 0.16119E+00_JPRB ,&
&  0.16004E+00_JPRB, 0.15911E+00_JPRB, 0.15791E+00_JPRB, 0.15674E+00_JPRB, 0.15568E+00_JPRB, 0.15466E+00_JPRB, 0.15378E+00_JPRB ,&
&  0.15314E+00_JPRB, 0.15260E+00_JPRB, 0.15204E+00_JPRB, 0.15161E+00_JPRB, 0.15115E+00_JPRB, 0.15072E+00_JPRB, 0.15033E+00_JPRB ,&
&  0.15047E+00_JPRB, 0.15058E+00_JPRB, 0.15069E+00_JPRB, 0.15079E+00_JPRB, 0.15118E+00_JPRB, 0.15125E+00_JPRB, 0.15134E+00_JPRB ,&
&  0.15139E+00_JPRB, 0.15128E+00_JPRB, 0.15117E+00_JPRB, 0.15096E+00_JPRB, 0.15072E+00_JPRB, 0.15042E+00_JPRB, 0.15029E+00_JPRB ,&
&  0.15028E+00_JPRB, 0.15039E+00_JPRB, 0.15072E+00_JPRB, 0.15094E+00_JPRB, 0.15116E+00_JPRB, 0.15135E+00_JPRB, 0.15179E+00_JPRB ,&
&  0.15224E+00_JPRB, 0.15269E+00_JPRB, 0.15311E+00_JPRB, 0.15301E+00_JPRB, 0.15325E+00_JPRB, 0.15347E+00_JPRB, 0.15368E+00_JPRB ,&
&  0.15380E+00_JPRB, 0.15426E+00_JPRB, 0.15498E+00_JPRB, 0.15597E+00_JPRB, 0.15777E+00_JPRB, 0.15932E+00_JPRB, 0.16075E+00_JPRB ,&
&  0.16229E+00_JPRB, 0.16536E+00_JPRB, 0.16660E+00_JPRB, 0.16702E+00_JPRB, 0.16703E+00_JPRB /)
ZBURD(:, 72,  8) = (/ &
&  0.15691E+00_JPRB, 0.15769E+00_JPRB, 0.15824E+00_JPRB, 0.15869E+00_JPRB, 0.15850E+00_JPRB, 0.15857E+00_JPRB, 0.15862E+00_JPRB ,&
&  0.15860E+00_JPRB, 0.15856E+00_JPRB, 0.15882E+00_JPRB, 0.15906E+00_JPRB, 0.15932E+00_JPRB, 0.15961E+00_JPRB, 0.15994E+00_JPRB ,&
&  0.16008E+00_JPRB, 0.16038E+00_JPRB, 0.15924E+00_JPRB, 0.15831E+00_JPRB, 0.15772E+00_JPRB, 0.15736E+00_JPRB, 0.15717E+00_JPRB ,&
&  0.15701E+00_JPRB, 0.15692E+00_JPRB, 0.15679E+00_JPRB, 0.15763E+00_JPRB, 0.15818E+00_JPRB, 0.15866E+00_JPRB, 0.15904E+00_JPRB ,&
&  0.15919E+00_JPRB, 0.15941E+00_JPRB, 0.15960E+00_JPRB, 0.15980E+00_JPRB, 0.16003E+00_JPRB, 0.16001E+00_JPRB, 0.15989E+00_JPRB ,&
&  0.15970E+00_JPRB, 0.15939E+00_JPRB, 0.15903E+00_JPRB, 0.15869E+00_JPRB, 0.15841E+00_JPRB, 0.15809E+00_JPRB, 0.15773E+00_JPRB ,&
&  0.15739E+00_JPRB, 0.15699E+00_JPRB, 0.15660E+00_JPRB, 0.15624E+00_JPRB, 0.15589E+00_JPRB, 0.15559E+00_JPRB, 0.15526E+00_JPRB ,&
&  0.15503E+00_JPRB, 0.15482E+00_JPRB, 0.15461E+00_JPRB, 0.15446E+00_JPRB, 0.15422E+00_JPRB, 0.15413E+00_JPRB, 0.15392E+00_JPRB ,&
&  0.15384E+00_JPRB, 0.15376E+00_JPRB, 0.15370E+00_JPRB, 0.15362E+00_JPRB, 0.15361E+00_JPRB, 0.15359E+00_JPRB, 0.15357E+00_JPRB ,&
&  0.15354E+00_JPRB, 0.15366E+00_JPRB, 0.15353E+00_JPRB, 0.15337E+00_JPRB, 0.15320E+00_JPRB, 0.15302E+00_JPRB, 0.15286E+00_JPRB ,&
&  0.15272E+00_JPRB, 0.15257E+00_JPRB, 0.15247E+00_JPRB, 0.15249E+00_JPRB, 0.15252E+00_JPRB, 0.15257E+00_JPRB, 0.15264E+00_JPRB ,&
&  0.15272E+00_JPRB, 0.15282E+00_JPRB, 0.15292E+00_JPRB, 0.15289E+00_JPRB, 0.15291E+00_JPRB, 0.15293E+00_JPRB, 0.15295E+00_JPRB ,&
&  0.15297E+00_JPRB, 0.15302E+00_JPRB, 0.15310E+00_JPRB, 0.15321E+00_JPRB, 0.15346E+00_JPRB, 0.15357E+00_JPRB, 0.15373E+00_JPRB ,&
&  0.15394E+00_JPRB, 0.15433E+00_JPRB, 0.15467E+00_JPRB, 0.15504E+00_JPRB, 0.15547E+00_JPRB /)
ZBURD(:, 73,  8) = (/ &
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB ,&
&  0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB, 0.15456E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART16',1,ZHOOK_HANDLE)
END SUBROUTINE PART16



SUBROUTINE PART17

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART17',0,ZHOOK_HANDLE)


!-- month=  9
ZBURD(:,  1,  9) = (/ &
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB ,&
&  0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB, 0.88673E+00_JPRB /)
ZBURD(:,  2,  9) = (/ &
&  0.90924E+00_JPRB, 0.90754E+00_JPRB, 0.90614E+00_JPRB, 0.90469E+00_JPRB, 0.90339E+00_JPRB, 0.90229E+00_JPRB, 0.90127E+00_JPRB ,&
&  0.90012E+00_JPRB, 0.89875E+00_JPRB, 0.89702E+00_JPRB, 0.89539E+00_JPRB, 0.89387E+00_JPRB, 0.89230E+00_JPRB, 0.89108E+00_JPRB ,&
&  0.88987E+00_JPRB, 0.88880E+00_JPRB, 0.88763E+00_JPRB, 0.88596E+00_JPRB, 0.88459E+00_JPRB, 0.88315E+00_JPRB, 0.88079E+00_JPRB ,&
&  0.87781E+00_JPRB, 0.87476E+00_JPRB, 0.87122E+00_JPRB, 0.86779E+00_JPRB, 0.86488E+00_JPRB, 0.86142E+00_JPRB, 0.85744E+00_JPRB ,&
&  0.85365E+00_JPRB, 0.85071E+00_JPRB, 0.84823E+00_JPRB, 0.84614E+00_JPRB, 0.84448E+00_JPRB, 0.84301E+00_JPRB, 0.84146E+00_JPRB ,&
&  0.84020E+00_JPRB, 0.83964E+00_JPRB, 0.83971E+00_JPRB, 0.84009E+00_JPRB, 0.84081E+00_JPRB, 0.84218E+00_JPRB, 0.84499E+00_JPRB ,&
&  0.84801E+00_JPRB, 0.85110E+00_JPRB, 0.85432E+00_JPRB, 0.85723E+00_JPRB, 0.85996E+00_JPRB, 0.86236E+00_JPRB, 0.86446E+00_JPRB ,&
&  0.86674E+00_JPRB, 0.86895E+00_JPRB, 0.87131E+00_JPRB, 0.87315E+00_JPRB, 0.87470E+00_JPRB, 0.87602E+00_JPRB, 0.87739E+00_JPRB ,&
&  0.87802E+00_JPRB, 0.87847E+00_JPRB, 0.87917E+00_JPRB, 0.88018E+00_JPRB, 0.88101E+00_JPRB, 0.88179E+00_JPRB, 0.88274E+00_JPRB ,&
&  0.88401E+00_JPRB, 0.88528E+00_JPRB, 0.88642E+00_JPRB, 0.88755E+00_JPRB, 0.88900E+00_JPRB, 0.89094E+00_JPRB, 0.89335E+00_JPRB ,&
&  0.89570E+00_JPRB, 0.89809E+00_JPRB, 0.90034E+00_JPRB, 0.90244E+00_JPRB, 0.90383E+00_JPRB, 0.90450E+00_JPRB, 0.90397E+00_JPRB ,&
&  0.90306E+00_JPRB, 0.90184E+00_JPRB, 0.90138E+00_JPRB, 0.90176E+00_JPRB, 0.90204E+00_JPRB, 0.90280E+00_JPRB, 0.90415E+00_JPRB ,&
&  0.90608E+00_JPRB, 0.90808E+00_JPRB, 0.91021E+00_JPRB, 0.91263E+00_JPRB, 0.91452E+00_JPRB, 0.91565E+00_JPRB, 0.91662E+00_JPRB ,&
&  0.91707E+00_JPRB, 0.91659E+00_JPRB, 0.91513E+00_JPRB, 0.91323E+00_JPRB, 0.91105E+00_JPRB /)
ZBURD(:,  3,  9) = (/ &
&  0.90922E+00_JPRB, 0.90360E+00_JPRB, 0.89960E+00_JPRB, 0.89667E+00_JPRB, 0.89499E+00_JPRB, 0.89330E+00_JPRB, 0.89177E+00_JPRB ,&
&  0.89022E+00_JPRB, 0.89072E+00_JPRB, 0.89068E+00_JPRB, 0.89043E+00_JPRB, 0.88971E+00_JPRB, 0.89059E+00_JPRB, 0.89081E+00_JPRB ,&
&  0.88945E+00_JPRB, 0.88551E+00_JPRB, 0.88180E+00_JPRB, 0.87844E+00_JPRB, 0.87780E+00_JPRB, 0.87671E+00_JPRB, 0.87625E+00_JPRB ,&
&  0.87333E+00_JPRB, 0.86987E+00_JPRB, 0.86535E+00_JPRB, 0.85524E+00_JPRB, 0.84407E+00_JPRB, 0.83354E+00_JPRB, 0.82416E+00_JPRB ,&
&  0.81587E+00_JPRB, 0.80948E+00_JPRB, 0.80518E+00_JPRB, 0.80238E+00_JPRB, 0.80469E+00_JPRB, 0.80457E+00_JPRB, 0.80364E+00_JPRB ,&
&  0.80142E+00_JPRB, 0.79599E+00_JPRB, 0.79632E+00_JPRB, 0.79726E+00_JPRB, 0.79907E+00_JPRB, 0.80473E+00_JPRB, 0.81030E+00_JPRB ,&
&  0.81711E+00_JPRB, 0.82488E+00_JPRB, 0.83730E+00_JPRB, 0.84634E+00_JPRB, 0.85489E+00_JPRB, 0.86167E+00_JPRB, 0.86568E+00_JPRB ,&
&  0.86930E+00_JPRB, 0.87251E+00_JPRB, 0.87569E+00_JPRB, 0.87826E+00_JPRB, 0.87998E+00_JPRB, 0.88195E+00_JPRB, 0.88419E+00_JPRB ,&
&  0.88486E+00_JPRB, 0.88477E+00_JPRB, 0.88452E+00_JPRB, 0.88437E+00_JPRB, 0.88413E+00_JPRB, 0.88444E+00_JPRB, 0.88537E+00_JPRB ,&
&  0.88630E+00_JPRB, 0.88574E+00_JPRB, 0.88532E+00_JPRB, 0.88624E+00_JPRB, 0.88754E+00_JPRB, 0.88927E+00_JPRB, 0.89111E+00_JPRB ,&
&  0.89392E+00_JPRB, 0.89695E+00_JPRB, 0.90100E+00_JPRB, 0.90680E+00_JPRB, 0.91155E+00_JPRB, 0.91598E+00_JPRB, 0.91836E+00_JPRB ,&
&  0.92117E+00_JPRB, 0.92227E+00_JPRB, 0.92347E+00_JPRB, 0.92598E+00_JPRB, 0.92843E+00_JPRB, 0.93289E+00_JPRB, 0.93765E+00_JPRB ,&
&  0.94379E+00_JPRB, 0.94621E+00_JPRB, 0.94759E+00_JPRB, 0.94717E+00_JPRB, 0.94645E+00_JPRB, 0.94465E+00_JPRB, 0.94268E+00_JPRB ,&
&  0.94017E+00_JPRB, 0.93641E+00_JPRB, 0.93022E+00_JPRB, 0.92309E+00_JPRB, 0.91572E+00_JPRB /)
ZBURD(:,  4,  9) = (/ &
&  0.90696E+00_JPRB, 0.89862E+00_JPRB, 0.89258E+00_JPRB, 0.88837E+00_JPRB, 0.88445E+00_JPRB, 0.88142E+00_JPRB, 0.87878E+00_JPRB ,&
&  0.87701E+00_JPRB, 0.87426E+00_JPRB, 0.87337E+00_JPRB, 0.87254E+00_JPRB, 0.87116E+00_JPRB, 0.87241E+00_JPRB, 0.87282E+00_JPRB ,&
&  0.87392E+00_JPRB, 0.87352E+00_JPRB, 0.87686E+00_JPRB, 0.87534E+00_JPRB, 0.87468E+00_JPRB, 0.87418E+00_JPRB, 0.86835E+00_JPRB ,&
&  0.86332E+00_JPRB, 0.85665E+00_JPRB, 0.84354E+00_JPRB, 0.83002E+00_JPRB, 0.81887E+00_JPRB, 0.78300E+00_JPRB, 0.77897E+00_JPRB ,&
&  0.78532E+00_JPRB, 0.76808E+00_JPRB, 0.75899E+00_JPRB, 0.75106E+00_JPRB, 0.74406E+00_JPRB, 0.74172E+00_JPRB, 0.74508E+00_JPRB ,&
&  0.73216E+00_JPRB, 0.75123E+00_JPRB, 0.75258E+00_JPRB, 0.75348E+00_JPRB, 0.75644E+00_JPRB, 0.75465E+00_JPRB, 0.76405E+00_JPRB ,&
&  0.79554E+00_JPRB, 0.80294E+00_JPRB, 0.81823E+00_JPRB, 0.83163E+00_JPRB, 0.84637E+00_JPRB, 0.85899E+00_JPRB, 0.87139E+00_JPRB ,&
&  0.88094E+00_JPRB, 0.88764E+00_JPRB, 0.89222E+00_JPRB, 0.89253E+00_JPRB, 0.89239E+00_JPRB, 0.89105E+00_JPRB, 0.89196E+00_JPRB ,&
&  0.89194E+00_JPRB, 0.89298E+00_JPRB, 0.89508E+00_JPRB, 0.89645E+00_JPRB, 0.89808E+00_JPRB, 0.89789E+00_JPRB, 0.89833E+00_JPRB ,&
&  0.89816E+00_JPRB, 0.89696E+00_JPRB, 0.89583E+00_JPRB, 0.89502E+00_JPRB, 0.89329E+00_JPRB, 0.89273E+00_JPRB, 0.89496E+00_JPRB ,&
&  0.89927E+00_JPRB, 0.90357E+00_JPRB, 0.90948E+00_JPRB, 0.91637E+00_JPRB, 0.92533E+00_JPRB, 0.93733E+00_JPRB, 0.94675E+00_JPRB ,&
&  0.96013E+00_JPRB, 0.96903E+00_JPRB, 0.97634E+00_JPRB, 0.98187E+00_JPRB, 0.98614E+00_JPRB, 0.99243E+00_JPRB, 0.99725E+00_JPRB ,&
&  0.10035E+01_JPRB, 0.99948E+00_JPRB, 0.99307E+00_JPRB, 0.98554E+00_JPRB, 0.97901E+00_JPRB, 0.97299E+00_JPRB, 0.96645E+00_JPRB ,&
&  0.96051E+00_JPRB, 0.95246E+00_JPRB, 0.94223E+00_JPRB, 0.92988E+00_JPRB, 0.91753E+00_JPRB /)
ZBURD(:,  5,  9) = (/ &
&  0.92062E+00_JPRB, 0.90799E+00_JPRB, 0.89726E+00_JPRB, 0.89062E+00_JPRB, 0.88415E+00_JPRB, 0.87950E+00_JPRB, 0.87522E+00_JPRB ,&
&  0.87189E+00_JPRB, 0.86868E+00_JPRB, 0.86691E+00_JPRB, 0.86546E+00_JPRB, 0.86478E+00_JPRB, 0.86451E+00_JPRB, 0.86408E+00_JPRB ,&
&  0.86743E+00_JPRB, 0.86767E+00_JPRB, 0.87226E+00_JPRB, 0.87149E+00_JPRB, 0.87078E+00_JPRB, 0.86718E+00_JPRB, 0.85356E+00_JPRB ,&
&  0.84241E+00_JPRB, 0.83038E+00_JPRB, 0.79703E+00_JPRB, 0.80649E+00_JPRB, 0.78846E+00_JPRB, 0.77718E+00_JPRB, 0.76586E+00_JPRB ,&
&  0.74578E+00_JPRB, 0.73641E+00_JPRB, 0.72607E+00_JPRB, 0.72357E+00_JPRB, 0.72705E+00_JPRB, 0.72034E+00_JPRB, 0.72282E+00_JPRB ,&
&  0.71567E+00_JPRB, 0.71371E+00_JPRB, 0.71467E+00_JPRB, 0.71569E+00_JPRB, 0.72371E+00_JPRB, 0.72666E+00_JPRB, 0.73421E+00_JPRB ,&
&  0.74817E+00_JPRB, 0.77013E+00_JPRB, 0.80579E+00_JPRB, 0.81703E+00_JPRB, 0.83812E+00_JPRB, 0.85518E+00_JPRB, 0.87090E+00_JPRB ,&
&  0.88128E+00_JPRB, 0.88797E+00_JPRB, 0.89319E+00_JPRB, 0.89453E+00_JPRB, 0.89385E+00_JPRB, 0.88798E+00_JPRB, 0.88790E+00_JPRB ,&
&  0.89015E+00_JPRB, 0.89312E+00_JPRB, 0.89708E+00_JPRB, 0.89791E+00_JPRB, 0.89981E+00_JPRB, 0.89918E+00_JPRB, 0.90172E+00_JPRB ,&
&  0.90333E+00_JPRB, 0.90531E+00_JPRB, 0.90632E+00_JPRB, 0.90953E+00_JPRB, 0.91522E+00_JPRB, 0.92151E+00_JPRB, 0.92815E+00_JPRB ,&
&  0.93603E+00_JPRB, 0.94684E+00_JPRB, 0.95946E+00_JPRB, 0.96180E+00_JPRB, 0.97391E+00_JPRB, 0.10031E+01_JPRB, 0.10180E+01_JPRB ,&
&  0.10338E+01_JPRB, 0.10421E+01_JPRB, 0.10451E+01_JPRB, 0.10478E+01_JPRB, 0.10483E+01_JPRB, 0.10522E+01_JPRB, 0.10542E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.10453E+01_JPRB, 0.10335E+01_JPRB, 0.10233E+01_JPRB, 0.10125E+01_JPRB, 0.10032E+01_JPRB, 0.99298E+00_JPRB ,&
&  0.98495E+00_JPRB, 0.97760E+00_JPRB, 0.96907E+00_JPRB, 0.95504E+00_JPRB, 0.93839E+00_JPRB /)
ZBURD(:,  6,  9) = (/ &
&  0.94060E+00_JPRB, 0.92465E+00_JPRB, 0.91257E+00_JPRB, 0.90276E+00_JPRB, 0.89537E+00_JPRB, 0.88957E+00_JPRB, 0.88443E+00_JPRB ,&
&  0.88032E+00_JPRB, 0.87587E+00_JPRB, 0.87170E+00_JPRB, 0.86904E+00_JPRB, 0.86693E+00_JPRB, 0.86572E+00_JPRB, 0.86343E+00_JPRB ,&
&  0.86375E+00_JPRB, 0.86242E+00_JPRB, 0.86653E+00_JPRB, 0.86519E+00_JPRB, 0.86929E+00_JPRB, 0.86672E+00_JPRB, 0.85273E+00_JPRB ,&
&  0.83568E+00_JPRB, 0.81237E+00_JPRB, 0.78631E+00_JPRB, 0.79303E+00_JPRB, 0.77937E+00_JPRB, 0.77239E+00_JPRB, 0.74999E+00_JPRB ,&
&  0.73644E+00_JPRB, 0.71620E+00_JPRB, 0.73924E+00_JPRB, 0.72404E+00_JPRB, 0.71440E+00_JPRB, 0.70352E+00_JPRB, 0.68119E+00_JPRB ,&
&  0.67610E+00_JPRB, 0.66111E+00_JPRB, 0.65938E+00_JPRB, 0.66360E+00_JPRB, 0.67144E+00_JPRB, 0.68485E+00_JPRB, 0.69777E+00_JPRB ,&
&  0.70103E+00_JPRB, 0.75512E+00_JPRB, 0.78915E+00_JPRB, 0.81289E+00_JPRB, 0.83607E+00_JPRB, 0.85409E+00_JPRB, 0.86695E+00_JPRB ,&
&  0.88025E+00_JPRB, 0.89120E+00_JPRB, 0.90456E+00_JPRB, 0.90948E+00_JPRB, 0.92819E+00_JPRB, 0.93483E+00_JPRB, 0.94511E+00_JPRB ,&
&  0.95593E+00_JPRB, 0.95631E+00_JPRB, 0.95123E+00_JPRB, 0.94126E+00_JPRB, 0.93235E+00_JPRB, 0.92311E+00_JPRB, 0.92000E+00_JPRB ,&
&  0.91883E+00_JPRB, 0.92102E+00_JPRB, 0.93237E+00_JPRB, 0.94330E+00_JPRB, 0.95337E+00_JPRB, 0.96271E+00_JPRB, 0.97279E+00_JPRB ,&
&  0.98379E+00_JPRB, 0.10038E+01_JPRB, 0.10252E+01_JPRB, 0.10614E+01_JPRB, 0.10963E+01_JPRB, 0.11213E+01_JPRB, 0.11420E+01_JPRB ,&
&  0.11620E+01_JPRB, 0.11573E+01_JPRB, 0.11541E+01_JPRB, 0.11554E+01_JPRB, 0.11566E+01_JPRB, 0.11598E+01_JPRB, 0.11623E+01_JPRB ,&
&  0.11605E+01_JPRB, 0.11487E+01_JPRB, 0.11334E+01_JPRB, 0.11153E+01_JPRB, 0.10981E+01_JPRB, 0.10812E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10292E+01_JPRB, 0.10055E+01_JPRB, 0.98130E+00_JPRB, 0.95911E+00_JPRB /)
ZBURD(:,  7,  9) = (/ &
&  0.98926E+00_JPRB, 0.96616E+00_JPRB, 0.94853E+00_JPRB, 0.93450E+00_JPRB, 0.92570E+00_JPRB, 0.91936E+00_JPRB, 0.91518E+00_JPRB ,&
&  0.91196E+00_JPRB, 0.90614E+00_JPRB, 0.89956E+00_JPRB, 0.89161E+00_JPRB, 0.88549E+00_JPRB, 0.88016E+00_JPRB, 0.87399E+00_JPRB ,&
&  0.86513E+00_JPRB, 0.85576E+00_JPRB, 0.84591E+00_JPRB, 0.83817E+00_JPRB, 0.84416E+00_JPRB, 0.84501E+00_JPRB, 0.84326E+00_JPRB ,&
&  0.83108E+00_JPRB, 0.80786E+00_JPRB, 0.78568E+00_JPRB, 0.78475E+00_JPRB, 0.77667E+00_JPRB, 0.75430E+00_JPRB, 0.74812E+00_JPRB ,&
&  0.75815E+00_JPRB, 0.75848E+00_JPRB, 0.75083E+00_JPRB, 0.74398E+00_JPRB, 0.70835E+00_JPRB, 0.65594E+00_JPRB, 0.68229E+00_JPRB ,&
&  0.67711E+00_JPRB, 0.65062E+00_JPRB, 0.64591E+00_JPRB, 0.64353E+00_JPRB, 0.65410E+00_JPRB, 0.67484E+00_JPRB, 0.69459E+00_JPRB ,&
&  0.70298E+00_JPRB, 0.79050E+00_JPRB, 0.82947E+00_JPRB, 0.85594E+00_JPRB, 0.87392E+00_JPRB, 0.88832E+00_JPRB, 0.90099E+00_JPRB ,&
&  0.91681E+00_JPRB, 0.94139E+00_JPRB, 0.97645E+00_JPRB, 0.10107E+01_JPRB, 0.10498E+01_JPRB, 0.10766E+01_JPRB, 0.11089E+01_JPRB ,&
&  0.11424E+01_JPRB, 0.11484E+01_JPRB, 0.11175E+01_JPRB, 0.10849E+01_JPRB, 0.10522E+01_JPRB, 0.10238E+01_JPRB, 0.10011E+01_JPRB ,&
&  0.99300E+00_JPRB, 0.98768E+00_JPRB, 0.99762E+00_JPRB, 0.10017E+01_JPRB, 0.10090E+01_JPRB, 0.10155E+01_JPRB, 0.10368E+01_JPRB ,&
&  0.10539E+01_JPRB, 0.10830E+01_JPRB, 0.11179E+01_JPRB, 0.11675E+01_JPRB, 0.12165E+01_JPRB, 0.12530E+01_JPRB, 0.12825E+01_JPRB ,&
&  0.13022E+01_JPRB, 0.13323E+01_JPRB, 0.13509E+01_JPRB, 0.13734E+01_JPRB, 0.13999E+01_JPRB, 0.14136E+01_JPRB, 0.14260E+01_JPRB ,&
&  0.14315E+01_JPRB, 0.14271E+01_JPRB, 0.14117E+01_JPRB, 0.13850E+01_JPRB, 0.13478E+01_JPRB, 0.13038E+01_JPRB, 0.12489E+01_JPRB ,&
&  0.11902E+01_JPRB, 0.11288E+01_JPRB, 0.10785E+01_JPRB, 0.10431E+01_JPRB, 0.10136E+01_JPRB /)
ZBURD(:,  8,  9) = (/ &
&  0.10979E+01_JPRB, 0.10494E+01_JPRB, 0.10136E+01_JPRB, 0.99099E+00_JPRB, 0.97426E+00_JPRB, 0.96379E+00_JPRB, 0.95542E+00_JPRB ,&
&  0.94759E+00_JPRB, 0.94074E+00_JPRB, 0.93075E+00_JPRB, 0.92202E+00_JPRB, 0.91556E+00_JPRB, 0.90863E+00_JPRB, 0.90259E+00_JPRB ,&
&  0.89173E+00_JPRB, 0.86442E+00_JPRB, 0.85758E+00_JPRB, 0.84707E+00_JPRB, 0.82845E+00_JPRB, 0.81894E+00_JPRB, 0.81329E+00_JPRB ,&
&  0.80512E+00_JPRB, 0.80053E+00_JPRB, 0.79857E+00_JPRB, 0.78911E+00_JPRB, 0.77716E+00_JPRB, 0.77739E+00_JPRB, 0.76985E+00_JPRB ,&
&  0.73697E+00_JPRB, 0.73703E+00_JPRB, 0.78099E+00_JPRB, 0.77910E+00_JPRB, 0.72873E+00_JPRB, 0.71682E+00_JPRB, 0.71927E+00_JPRB ,&
&  0.70799E+00_JPRB, 0.66631E+00_JPRB, 0.65809E+00_JPRB, 0.65424E+00_JPRB, 0.66922E+00_JPRB, 0.68627E+00_JPRB, 0.72878E+00_JPRB ,&
&  0.76674E+00_JPRB, 0.84582E+00_JPRB, 0.88541E+00_JPRB, 0.91665E+00_JPRB, 0.94031E+00_JPRB, 0.96529E+00_JPRB, 0.99868E+00_JPRB ,&
&  0.10497E+01_JPRB, 0.11247E+01_JPRB, 0.12206E+01_JPRB, 0.13070E+01_JPRB, 0.13856E+01_JPRB, 0.14256E+01_JPRB, 0.14373E+01_JPRB ,&
&  0.14856E+01_JPRB, 0.14829E+01_JPRB, 0.14135E+01_JPRB, 0.13535E+01_JPRB, 0.12880E+01_JPRB, 0.12257E+01_JPRB, 0.11634E+01_JPRB ,&
&  0.11472E+01_JPRB, 0.11657E+01_JPRB, 0.11508E+01_JPRB, 0.11296E+01_JPRB, 0.11175E+01_JPRB, 0.11179E+01_JPRB, 0.11298E+01_JPRB ,&
&  0.11353E+01_JPRB, 0.12093E+01_JPRB, 0.14287E+01_JPRB, 0.15078E+01_JPRB, 0.15692E+01_JPRB, 0.16133E+01_JPRB, 0.16456E+01_JPRB ,&
&  0.16842E+01_JPRB, 0.17275E+01_JPRB, 0.17707E+01_JPRB, 0.17931E+01_JPRB, 0.18104E+01_JPRB, 0.17938E+01_JPRB, 0.18099E+01_JPRB ,&
&  0.17929E+01_JPRB, 0.17677E+01_JPRB, 0.17351E+01_JPRB, 0.17054E+01_JPRB, 0.16704E+01_JPRB, 0.16326E+01_JPRB, 0.15748E+01_JPRB ,&
&  0.15033E+01_JPRB, 0.14219E+01_JPRB, 0.13397E+01_JPRB, 0.12513E+01_JPRB, 0.11672E+01_JPRB /)
ZBURD(:,  9,  9) = (/ &
&  0.12741E+01_JPRB, 0.11680E+01_JPRB, 0.10924E+01_JPRB, 0.10535E+01_JPRB, 0.10273E+01_JPRB, 0.99694E+00_JPRB, 0.97448E+00_JPRB ,&
&  0.95841E+00_JPRB, 0.95776E+00_JPRB, 0.94155E+00_JPRB, 0.92508E+00_JPRB, 0.91500E+00_JPRB, 0.90711E+00_JPRB, 0.90385E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.88986E+00_JPRB, 0.86675E+00_JPRB, 0.84102E+00_JPRB, 0.82295E+00_JPRB, 0.80767E+00_JPRB, 0.79447E+00_JPRB ,&
&  0.78606E+00_JPRB, 0.78712E+00_JPRB, 0.80259E+00_JPRB, 0.80396E+00_JPRB, 0.79824E+00_JPRB, 0.78283E+00_JPRB, 0.76466E+00_JPRB ,&
&  0.75917E+00_JPRB, 0.77482E+00_JPRB, 0.75446E+00_JPRB, 0.76557E+00_JPRB, 0.78373E+00_JPRB, 0.75531E+00_JPRB, 0.73452E+00_JPRB ,&
&  0.71625E+00_JPRB, 0.69899E+00_JPRB, 0.68602E+00_JPRB, 0.68297E+00_JPRB, 0.70945E+00_JPRB, 0.74181E+00_JPRB, 0.79887E+00_JPRB ,&
&  0.86770E+00_JPRB, 0.91318E+00_JPRB, 0.94624E+00_JPRB, 0.98264E+00_JPRB, 0.10172E+01_JPRB, 0.10628E+01_JPRB, 0.11528E+01_JPRB ,&
&  0.13037E+01_JPRB, 0.14640E+01_JPRB, 0.16148E+01_JPRB, 0.17233E+01_JPRB, 0.18308E+01_JPRB, 0.17755E+01_JPRB, 0.17765E+01_JPRB ,&
&  0.20288E+01_JPRB, 0.21192E+01_JPRB, 0.19812E+01_JPRB, 0.18507E+01_JPRB, 0.17065E+01_JPRB, 0.15841E+01_JPRB, 0.14992E+01_JPRB ,&
&  0.14577E+01_JPRB, 0.14409E+01_JPRB, 0.14174E+01_JPRB, 0.13559E+01_JPRB, 0.13066E+01_JPRB, 0.13106E+01_JPRB, 0.13640E+01_JPRB ,&
&  0.14804E+01_JPRB, 0.18177E+01_JPRB, 0.23797E+01_JPRB, 0.21147E+01_JPRB, 0.20496E+01_JPRB, 0.20792E+01_JPRB, 0.21197E+01_JPRB ,&
&  0.21927E+01_JPRB, 0.22445E+01_JPRB, 0.22715E+01_JPRB, 0.22235E+01_JPRB, 0.21230E+01_JPRB, 0.20059E+01_JPRB, 0.19209E+01_JPRB ,&
&  0.18513E+01_JPRB, 0.17814E+01_JPRB, 0.17303E+01_JPRB, 0.17061E+01_JPRB, 0.16639E+01_JPRB, 0.16160E+01_JPRB, 0.15709E+01_JPRB ,&
&  0.15279E+01_JPRB, 0.14970E+01_JPRB, 0.14777E+01_JPRB, 0.14425E+01_JPRB, 0.13846E+01_JPRB /)
ZBURD(:, 10,  9) = (/ &
&  0.13223E+01_JPRB, 0.13134E+01_JPRB, 0.12589E+01_JPRB, 0.11886E+01_JPRB, 0.11264E+01_JPRB, 0.10461E+01_JPRB, 0.98844E+00_JPRB ,&
&  0.95745E+00_JPRB, 0.94601E+00_JPRB, 0.94339E+00_JPRB, 0.93781E+00_JPRB, 0.93171E+00_JPRB, 0.92328E+00_JPRB, 0.90904E+00_JPRB ,&
&  0.89108E+00_JPRB, 0.87973E+00_JPRB, 0.87253E+00_JPRB, 0.85944E+00_JPRB, 0.83944E+00_JPRB, 0.81548E+00_JPRB, 0.79325E+00_JPRB ,&
&  0.77658E+00_JPRB, 0.77092E+00_JPRB, 0.78526E+00_JPRB, 0.80205E+00_JPRB, 0.81381E+00_JPRB, 0.81146E+00_JPRB, 0.79830E+00_JPRB ,&
&  0.77097E+00_JPRB, 0.76319E+00_JPRB, 0.77634E+00_JPRB, 0.78192E+00_JPRB, 0.79161E+00_JPRB, 0.78944E+00_JPRB, 0.76780E+00_JPRB ,&
&  0.74976E+00_JPRB, 0.73571E+00_JPRB, 0.73462E+00_JPRB, 0.75115E+00_JPRB, 0.81368E+00_JPRB, 0.88360E+00_JPRB, 0.91845E+00_JPRB ,&
&  0.95293E+00_JPRB, 0.97882E+00_JPRB, 0.10107E+01_JPRB, 0.10542E+01_JPRB, 0.11295E+01_JPRB, 0.12491E+01_JPRB, 0.14885E+01_JPRB ,&
&  0.17687E+01_JPRB, 0.19655E+01_JPRB, 0.21525E+01_JPRB, 0.20135E+01_JPRB, 0.19704E+01_JPRB, 0.20497E+01_JPRB, 0.21892E+01_JPRB ,&
&  0.23308E+01_JPRB, 0.23844E+01_JPRB, 0.23642E+01_JPRB, 0.23937E+01_JPRB, 0.22549E+01_JPRB, 0.20832E+01_JPRB, 0.18803E+01_JPRB ,&
&  0.17412E+01_JPRB, 0.16527E+01_JPRB, 0.16330E+01_JPRB, 0.15841E+01_JPRB, 0.15947E+01_JPRB, 0.16725E+01_JPRB, 0.18164E+01_JPRB ,&
&  0.20435E+01_JPRB, 0.23128E+01_JPRB, 0.24466E+01_JPRB, 0.23820E+01_JPRB, 0.24500E+01_JPRB, 0.25769E+01_JPRB, 0.26775E+01_JPRB ,&
&  0.26766E+01_JPRB, 0.26623E+01_JPRB, 0.26760E+01_JPRB, 0.26379E+01_JPRB, 0.24050E+01_JPRB, 0.21938E+01_JPRB, 0.19504E+01_JPRB ,&
&  0.17156E+01_JPRB, 0.15059E+01_JPRB, 0.13867E+01_JPRB, 0.13445E+01_JPRB, 0.13117E+01_JPRB, 0.12713E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.11707E+01_JPRB, 0.11646E+01_JPRB, 0.11975E+01_JPRB, 0.12435E+01_JPRB, 0.12973E+01_JPRB /)
ZBURD(:, 11,  9) = (/ &
&  0.13612E+01_JPRB, 0.13943E+01_JPRB, 0.13919E+01_JPRB, 0.13622E+01_JPRB, 0.12622E+01_JPRB, 0.11701E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.10339E+01_JPRB, 0.98759E+00_JPRB, 0.95593E+00_JPRB, 0.92412E+00_JPRB, 0.89856E+00_JPRB, 0.88632E+00_JPRB, 0.88547E+00_JPRB ,&
&  0.88904E+00_JPRB, 0.87907E+00_JPRB, 0.86608E+00_JPRB, 0.85008E+00_JPRB, 0.83906E+00_JPRB, 0.82998E+00_JPRB, 0.81891E+00_JPRB ,&
&  0.80351E+00_JPRB, 0.80011E+00_JPRB, 0.80662E+00_JPRB, 0.82570E+00_JPRB, 0.83754E+00_JPRB, 0.84214E+00_JPRB, 0.84093E+00_JPRB ,&
&  0.82561E+00_JPRB, 0.80566E+00_JPRB, 0.80895E+00_JPRB, 0.82467E+00_JPRB, 0.83426E+00_JPRB, 0.84082E+00_JPRB, 0.80932E+00_JPRB ,&
&  0.78986E+00_JPRB, 0.79281E+00_JPRB, 0.82113E+00_JPRB, 0.90611E+00_JPRB, 0.95637E+00_JPRB, 0.98434E+00_JPRB, 0.10023E+01_JPRB ,&
&  0.99367E+00_JPRB, 0.99976E+00_JPRB, 0.10473E+01_JPRB, 0.11465E+01_JPRB, 0.13275E+01_JPRB, 0.16607E+01_JPRB, 0.20865E+01_JPRB ,&
&  0.23418E+01_JPRB, 0.24862E+01_JPRB, 0.25622E+01_JPRB, 0.23916E+01_JPRB, 0.24321E+01_JPRB, 0.28176E+01_JPRB, 0.29653E+01_JPRB ,&
&  0.30627E+01_JPRB, 0.30826E+01_JPRB, 0.31530E+01_JPRB, 0.29936E+01_JPRB, 0.26463E+01_JPRB, 0.24128E+01_JPRB, 0.22547E+01_JPRB ,&
&  0.20790E+01_JPRB, 0.19714E+01_JPRB, 0.19583E+01_JPRB, 0.20190E+01_JPRB, 0.21673E+01_JPRB, 0.24828E+01_JPRB, 0.28913E+01_JPRB ,&
&  0.32882E+01_JPRB, 0.35301E+01_JPRB, 0.36734E+01_JPRB, 0.36402E+01_JPRB, 0.35684E+01_JPRB, 0.34573E+01_JPRB, 0.33750E+01_JPRB ,&
&  0.32857E+01_JPRB, 0.32557E+01_JPRB, 0.32157E+01_JPRB, 0.31590E+01_JPRB, 0.28457E+01_JPRB, 0.23524E+01_JPRB, 0.18867E+01_JPRB ,&
&  0.15667E+01_JPRB, 0.13813E+01_JPRB, 0.12537E+01_JPRB, 0.12166E+01_JPRB, 0.12078E+01_JPRB, 0.11960E+01_JPRB, 0.11392E+01_JPRB ,&
&  0.11119E+01_JPRB, 0.11288E+01_JPRB, 0.11674E+01_JPRB, 0.12142E+01_JPRB, 0.12877E+01_JPRB /)
ZBURD(:, 12,  9) = (/ &
&  0.14294E+01_JPRB, 0.14695E+01_JPRB, 0.14914E+01_JPRB, 0.14729E+01_JPRB, 0.13949E+01_JPRB, 0.12784E+01_JPRB, 0.11522E+01_JPRB ,&
&  0.10903E+01_JPRB, 0.10405E+01_JPRB, 0.10068E+01_JPRB, 0.97333E+00_JPRB, 0.94579E+00_JPRB, 0.90896E+00_JPRB, 0.87413E+00_JPRB ,&
&  0.86584E+00_JPRB, 0.87497E+00_JPRB, 0.88264E+00_JPRB, 0.87807E+00_JPRB, 0.87288E+00_JPRB, 0.86916E+00_JPRB, 0.86470E+00_JPRB ,&
&  0.85826E+00_JPRB, 0.86172E+00_JPRB, 0.88180E+00_JPRB, 0.91404E+00_JPRB, 0.92994E+00_JPRB, 0.93431E+00_JPRB, 0.94156E+00_JPRB ,&
&  0.93220E+00_JPRB, 0.91148E+00_JPRB, 0.89795E+00_JPRB, 0.90049E+00_JPRB, 0.90172E+00_JPRB, 0.92291E+00_JPRB, 0.92129E+00_JPRB ,&
&  0.85599E+00_JPRB, 0.88505E+00_JPRB, 0.97538E+00_JPRB, 0.10206E+01_JPRB, 0.10511E+01_JPRB, 0.10713E+01_JPRB, 0.10876E+01_JPRB ,&
&  0.11033E+01_JPRB, 0.11409E+01_JPRB, 0.11928E+01_JPRB, 0.12694E+01_JPRB, 0.16268E+01_JPRB, 0.23964E+01_JPRB, 0.27479E+01_JPRB ,&
&  0.29252E+01_JPRB, 0.25905E+01_JPRB, 0.24305E+01_JPRB, 0.24984E+01_JPRB, 0.29139E+01_JPRB, 0.31794E+01_JPRB, 0.34092E+01_JPRB ,&
&  0.36686E+01_JPRB, 0.36795E+01_JPRB, 0.35856E+01_JPRB, 0.34047E+01_JPRB, 0.30679E+01_JPRB, 0.28348E+01_JPRB, 0.27413E+01_JPRB ,&
&  0.26294E+01_JPRB, 0.25993E+01_JPRB, 0.27350E+01_JPRB, 0.31037E+01_JPRB, 0.35596E+01_JPRB, 0.39432E+01_JPRB, 0.42425E+01_JPRB ,&
&  0.45582E+01_JPRB, 0.47250E+01_JPRB, 0.46708E+01_JPRB, 0.45362E+01_JPRB, 0.42953E+01_JPRB, 0.41480E+01_JPRB, 0.40882E+01_JPRB ,&
&  0.40100E+01_JPRB, 0.39362E+01_JPRB, 0.37731E+01_JPRB, 0.35470E+01_JPRB, 0.31842E+01_JPRB, 0.28453E+01_JPRB, 0.24613E+01_JPRB ,&
&  0.18950E+01_JPRB, 0.15714E+01_JPRB, 0.13412E+01_JPRB, 0.12628E+01_JPRB, 0.12472E+01_JPRB, 0.12533E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.12581E+01_JPRB, 0.12820E+01_JPRB, 0.12863E+01_JPRB, 0.13322E+01_JPRB, 0.13890E+01_JPRB /)
ZBURD(:, 13,  9) = (/ &
&  0.14947E+01_JPRB, 0.15151E+01_JPRB, 0.15187E+01_JPRB, 0.15108E+01_JPRB, 0.14615E+01_JPRB, 0.13681E+01_JPRB, 0.12790E+01_JPRB ,&
&  0.12896E+01_JPRB, 0.12336E+01_JPRB, 0.12177E+01_JPRB, 0.10644E+01_JPRB, 0.10120E+01_JPRB, 0.99031E+00_JPRB, 0.94723E+00_JPRB ,&
&  0.92383E+00_JPRB, 0.91921E+00_JPRB, 0.91456E+00_JPRB, 0.91968E+00_JPRB, 0.92639E+00_JPRB, 0.92447E+00_JPRB, 0.92802E+00_JPRB ,&
&  0.94476E+00_JPRB, 0.97260E+00_JPRB, 0.10217E+01_JPRB, 0.10532E+01_JPRB, 0.10803E+01_JPRB, 0.10943E+01_JPRB, 0.11038E+01_JPRB ,&
&  0.10555E+01_JPRB, 0.10306E+01_JPRB, 0.10280E+01_JPRB, 0.99999E+00_JPRB, 0.99974E+00_JPRB, 0.10200E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10339E+01_JPRB, 0.11160E+01_JPRB, 0.11579E+01_JPRB, 0.11810E+01_JPRB, 0.11936E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.12563E+01_JPRB, 0.13199E+01_JPRB, 0.14404E+01_JPRB, 0.21382E+01_JPRB, 0.33864E+01_JPRB, 0.35023E+01_JPRB ,&
&  0.34302E+01_JPRB, 0.28146E+01_JPRB, 0.31594E+01_JPRB, 0.33036E+01_JPRB, 0.35601E+01_JPRB, 0.37998E+01_JPRB, 0.39111E+01_JPRB ,&
&  0.43825E+01_JPRB, 0.45139E+01_JPRB, 0.43228E+01_JPRB, 0.39345E+01_JPRB, 0.36036E+01_JPRB, 0.34511E+01_JPRB, 0.34000E+01_JPRB ,&
&  0.34316E+01_JPRB, 0.33197E+01_JPRB, 0.37648E+01_JPRB, 0.41969E+01_JPRB, 0.44683E+01_JPRB, 0.47410E+01_JPRB, 0.50634E+01_JPRB ,&
&  0.52740E+01_JPRB, 0.52781E+01_JPRB, 0.52268E+01_JPRB, 0.48674E+01_JPRB, 0.46748E+01_JPRB, 0.45305E+01_JPRB, 0.42801E+01_JPRB ,&
&  0.41136E+01_JPRB, 0.39713E+01_JPRB, 0.37635E+01_JPRB, 0.37289E+01_JPRB, 0.32536E+01_JPRB, 0.29341E+01_JPRB, 0.27309E+01_JPRB ,&
&  0.22938E+01_JPRB, 0.18122E+01_JPRB, 0.16114E+01_JPRB, 0.15161E+01_JPRB, 0.14504E+01_JPRB, 0.14256E+01_JPRB, 0.14116E+01_JPRB ,&
&  0.13495E+01_JPRB, 0.13601E+01_JPRB, 0.14297E+01_JPRB, 0.14497E+01_JPRB, 0.14659E+01_JPRB /)
ZBURD(:, 14,  9) = (/ &
&  0.15853E+01_JPRB, 0.16003E+01_JPRB, 0.15983E+01_JPRB, 0.15905E+01_JPRB, 0.15716E+01_JPRB, 0.15234E+01_JPRB, 0.14379E+01_JPRB ,&
&  0.14140E+01_JPRB, 0.13590E+01_JPRB, 0.12980E+01_JPRB, 0.12504E+01_JPRB, 0.12317E+01_JPRB, 0.11386E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.96033E+00_JPRB, 0.97080E+00_JPRB, 0.99668E+00_JPRB, 0.10097E+01_JPRB, 0.10234E+01_JPRB, 0.10352E+01_JPRB, 0.10744E+01_JPRB ,&
&  0.11248E+01_JPRB, 0.11723E+01_JPRB, 0.12371E+01_JPRB, 0.13038E+01_JPRB, 0.13203E+01_JPRB, 0.13209E+01_JPRB, 0.13418E+01_JPRB ,&
&  0.12834E+01_JPRB, 0.12087E+01_JPRB, 0.11648E+01_JPRB, 0.10806E+01_JPRB, 0.11506E+01_JPRB, 0.12047E+01_JPRB, 0.12632E+01_JPRB ,&
&  0.13141E+01_JPRB, 0.13437E+01_JPRB, 0.13796E+01_JPRB, 0.13952E+01_JPRB, 0.13873E+01_JPRB, 0.13839E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.14032E+01_JPRB, 0.14507E+01_JPRB, 0.15314E+01_JPRB, 0.16916E+01_JPRB, 0.30907E+01_JPRB, 0.45352E+01_JPRB, 0.48286E+01_JPRB ,&
&  0.44094E+01_JPRB, 0.46663E+01_JPRB, 0.49280E+01_JPRB, 0.45498E+01_JPRB, 0.46995E+01_JPRB, 0.48331E+01_JPRB, 0.46801E+01_JPRB ,&
&  0.49019E+01_JPRB, 0.49046E+01_JPRB, 0.48426E+01_JPRB, 0.47500E+01_JPRB, 0.46587E+01_JPRB, 0.45244E+01_JPRB, 0.44276E+01_JPRB ,&
&  0.44922E+01_JPRB, 0.47449E+01_JPRB, 0.48789E+01_JPRB, 0.51097E+01_JPRB, 0.52179E+01_JPRB, 0.53853E+01_JPRB, 0.55271E+01_JPRB ,&
&  0.55746E+01_JPRB, 0.56090E+01_JPRB, 0.55442E+01_JPRB, 0.53128E+01_JPRB, 0.50487E+01_JPRB, 0.47692E+01_JPRB, 0.44423E+01_JPRB ,&
&  0.41490E+01_JPRB, 0.36274E+01_JPRB, 0.32225E+01_JPRB, 0.30468E+01_JPRB, 0.28673E+01_JPRB, 0.25713E+01_JPRB, 0.24704E+01_JPRB ,&
&  0.23677E+01_JPRB, 0.21493E+01_JPRB, 0.21357E+01_JPRB, 0.19527E+01_JPRB, 0.18084E+01_JPRB, 0.16468E+01_JPRB, 0.14498E+01_JPRB ,&
&  0.13921E+01_JPRB, 0.14701E+01_JPRB, 0.15459E+01_JPRB, 0.15846E+01_JPRB, 0.15841E+01_JPRB /)
ZBURD(:, 15,  9) = (/ &
&  0.16765E+01_JPRB, 0.16904E+01_JPRB, 0.16926E+01_JPRB, 0.16886E+01_JPRB, 0.16690E+01_JPRB, 0.16299E+01_JPRB, 0.15725E+01_JPRB ,&
&  0.15136E+01_JPRB, 0.14429E+01_JPRB, 0.13789E+01_JPRB, 0.13281E+01_JPRB, 0.13039E+01_JPRB, 0.12723E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10803E+01_JPRB, 0.10687E+01_JPRB, 0.10928E+01_JPRB, 0.11779E+01_JPRB, 0.12188E+01_JPRB, 0.12656E+01_JPRB ,&
&  0.13834E+01_JPRB, 0.14738E+01_JPRB, 0.15317E+01_JPRB, 0.15891E+01_JPRB, 0.16281E+01_JPRB, 0.16742E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.16606E+01_JPRB, 0.15158E+01_JPRB, 0.14223E+01_JPRB, 0.13673E+01_JPRB, 0.14460E+01_JPRB, 0.15742E+01_JPRB, 0.16794E+01_JPRB ,&
&  0.17288E+01_JPRB, 0.17327E+01_JPRB, 0.17324E+01_JPRB, 0.17215E+01_JPRB, 0.16931E+01_JPRB, 0.16761E+01_JPRB, 0.16677E+01_JPRB ,&
&  0.16959E+01_JPRB, 0.17339E+01_JPRB, 0.17893E+01_JPRB, 0.20872E+01_JPRB, 0.43153E+01_JPRB, 0.66698E+01_JPRB, 0.73609E+01_JPRB ,&
&  0.66514E+01_JPRB, 0.65161E+01_JPRB, 0.67031E+01_JPRB, 0.70136E+01_JPRB, 0.68444E+01_JPRB, 0.61819E+01_JPRB, 0.56559E+01_JPRB ,&
&  0.56457E+01_JPRB, 0.53871E+01_JPRB, 0.58587E+01_JPRB, 0.58049E+01_JPRB, 0.58493E+01_JPRB, 0.59849E+01_JPRB, 0.58755E+01_JPRB ,&
&  0.59061E+01_JPRB, 0.57668E+01_JPRB, 0.57807E+01_JPRB, 0.56588E+01_JPRB, 0.56590E+01_JPRB, 0.58001E+01_JPRB, 0.58450E+01_JPRB ,&
&  0.59366E+01_JPRB, 0.59736E+01_JPRB, 0.55609E+01_JPRB, 0.52540E+01_JPRB, 0.48989E+01_JPRB, 0.51369E+01_JPRB, 0.46703E+01_JPRB ,&
&  0.39113E+01_JPRB, 0.30094E+01_JPRB, 0.29577E+01_JPRB, 0.28517E+01_JPRB, 0.28345E+01_JPRB, 0.27378E+01_JPRB, 0.25090E+01_JPRB ,&
&  0.24937E+01_JPRB, 0.26929E+01_JPRB, 0.25992E+01_JPRB, 0.22690E+01_JPRB, 0.20742E+01_JPRB, 0.18573E+01_JPRB, 0.15840E+01_JPRB ,&
&  0.15394E+01_JPRB, 0.16505E+01_JPRB, 0.16821E+01_JPRB, 0.16699E+01_JPRB, 0.16549E+01_JPRB /)
ZBURD(:, 16,  9) = (/ &
&  0.18472E+01_JPRB, 0.17785E+01_JPRB, 0.17366E+01_JPRB, 0.17290E+01_JPRB, 0.17112E+01_JPRB, 0.16850E+01_JPRB, 0.16473E+01_JPRB ,&
&  0.16073E+01_JPRB, 0.15593E+01_JPRB, 0.15056E+01_JPRB, 0.14551E+01_JPRB, 0.14189E+01_JPRB, 0.13755E+01_JPRB, 0.13310E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.12214E+01_JPRB, 0.12217E+01_JPRB, 0.12784E+01_JPRB, 0.15248E+01_JPRB, 0.14995E+01_JPRB, 0.15388E+01_JPRB ,&
&  0.15991E+01_JPRB, 0.16667E+01_JPRB, 0.16775E+01_JPRB, 0.17101E+01_JPRB, 0.18031E+01_JPRB, 0.20158E+01_JPRB, 0.22840E+01_JPRB ,&
&  0.22375E+01_JPRB, 0.20262E+01_JPRB, 0.19777E+01_JPRB, 0.19764E+01_JPRB, 0.20839E+01_JPRB, 0.21850E+01_JPRB, 0.22980E+01_JPRB ,&
&  0.22320E+01_JPRB, 0.21311E+01_JPRB, 0.20525E+01_JPRB, 0.19960E+01_JPRB, 0.19545E+01_JPRB, 0.19474E+01_JPRB, 0.19821E+01_JPRB ,&
&  0.20324E+01_JPRB, 0.20324E+01_JPRB, 0.20791E+01_JPRB, 0.27049E+01_JPRB, 0.57008E+01_JPRB, 0.87150E+01_JPRB, 0.10108E+02_JPRB ,&
&  0.10073E+02_JPRB, 0.93107E+01_JPRB, 0.97356E+01_JPRB, 0.11028E+02_JPRB, 0.10184E+02_JPRB, 0.87206E+01_JPRB, 0.75688E+01_JPRB ,&
&  0.71052E+01_JPRB, 0.67563E+01_JPRB, 0.69803E+01_JPRB, 0.71685E+01_JPRB, 0.69003E+01_JPRB, 0.68635E+01_JPRB, 0.67075E+01_JPRB ,&
&  0.65236E+01_JPRB, 0.65189E+01_JPRB, 0.63143E+01_JPRB, 0.61127E+01_JPRB, 0.59975E+01_JPRB, 0.61903E+01_JPRB, 0.61491E+01_JPRB ,&
&  0.63062E+01_JPRB, 0.60278E+01_JPRB, 0.52119E+01_JPRB, 0.51032E+01_JPRB, 0.39766E+01_JPRB, 0.37706E+01_JPRB, 0.44424E+01_JPRB ,&
&  0.36958E+01_JPRB, 0.30768E+01_JPRB, 0.28811E+01_JPRB, 0.29029E+01_JPRB, 0.29538E+01_JPRB, 0.31724E+01_JPRB, 0.30418E+01_JPRB ,&
&  0.26544E+01_JPRB, 0.27865E+01_JPRB, 0.30144E+01_JPRB, 0.26322E+01_JPRB, 0.23559E+01_JPRB, 0.21774E+01_JPRB, 0.19763E+01_JPRB ,&
&  0.19782E+01_JPRB, 0.19279E+01_JPRB, 0.18915E+01_JPRB, 0.18598E+01_JPRB, 0.18548E+01_JPRB /)
ZBURD(:, 17,  9) = (/ &
&  0.19474E+01_JPRB, 0.17519E+01_JPRB, 0.17297E+01_JPRB, 0.17105E+01_JPRB, 0.16866E+01_JPRB, 0.16592E+01_JPRB, 0.16377E+01_JPRB ,&
&  0.16223E+01_JPRB, 0.16039E+01_JPRB, 0.15763E+01_JPRB, 0.15388E+01_JPRB, 0.15061E+01_JPRB, 0.14650E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.14021E+01_JPRB, 0.15512E+01_JPRB, 0.15847E+01_JPRB, 0.15150E+01_JPRB, 0.16844E+01_JPRB, 0.16744E+01_JPRB, 0.17312E+01_JPRB ,&
&  0.18123E+01_JPRB, 0.19582E+01_JPRB, 0.20030E+01_JPRB, 0.20741E+01_JPRB, 0.22265E+01_JPRB, 0.25752E+01_JPRB, 0.30146E+01_JPRB ,&
&  0.31491E+01_JPRB, 0.30179E+01_JPRB, 0.30074E+01_JPRB, 0.31763E+01_JPRB, 0.33895E+01_JPRB, 0.30220E+01_JPRB, 0.28919E+01_JPRB ,&
&  0.26370E+01_JPRB, 0.24328E+01_JPRB, 0.22864E+01_JPRB, 0.21949E+01_JPRB, 0.21769E+01_JPRB, 0.22040E+01_JPRB, 0.22540E+01_JPRB ,&
&  0.22605E+01_JPRB, 0.21869E+01_JPRB, 0.23484E+01_JPRB, 0.34456E+01_JPRB, 0.64470E+01_JPRB, 0.91213E+01_JPRB, 0.10360E+02_JPRB ,&
&  0.10614E+02_JPRB, 0.10500E+02_JPRB, 0.10865E+02_JPRB, 0.12114E+02_JPRB, 0.11409E+02_JPRB, 0.10095E+02_JPRB, 0.93457E+01_JPRB ,&
&  0.88682E+01_JPRB, 0.86040E+01_JPRB, 0.86784E+01_JPRB, 0.80665E+01_JPRB, 0.72268E+01_JPRB, 0.67599E+01_JPRB, 0.63085E+01_JPRB ,&
&  0.59438E+01_JPRB, 0.59485E+01_JPRB, 0.59966E+01_JPRB, 0.57058E+01_JPRB, 0.55299E+01_JPRB, 0.52058E+01_JPRB, 0.55006E+01_JPRB ,&
&  0.60908E+01_JPRB, 0.44640E+01_JPRB, 0.36050E+01_JPRB, 0.40594E+01_JPRB, 0.29961E+01_JPRB, 0.29073E+01_JPRB, 0.33615E+01_JPRB ,&
&  0.30871E+01_JPRB, 0.29384E+01_JPRB, 0.30987E+01_JPRB, 0.29547E+01_JPRB, 0.30755E+01_JPRB, 0.33979E+01_JPRB, 0.33876E+01_JPRB ,&
&  0.32046E+01_JPRB, 0.30378E+01_JPRB, 0.34982E+01_JPRB, 0.30854E+01_JPRB, 0.27525E+01_JPRB, 0.25922E+01_JPRB, 0.24532E+01_JPRB ,&
&  0.23549E+01_JPRB, 0.22620E+01_JPRB, 0.21596E+01_JPRB, 0.20915E+01_JPRB, 0.20580E+01_JPRB /)
ZBURD(:, 18,  9) = (/ &
&  0.19016E+01_JPRB, 0.17189E+01_JPRB, 0.17644E+01_JPRB, 0.17582E+01_JPRB, 0.17312E+01_JPRB, 0.17044E+01_JPRB, 0.16907E+01_JPRB ,&
&  0.16647E+01_JPRB, 0.16439E+01_JPRB, 0.16110E+01_JPRB, 0.15526E+01_JPRB, 0.15082E+01_JPRB, 0.14720E+01_JPRB, 0.14446E+01_JPRB ,&
&  0.14268E+01_JPRB, 0.16560E+01_JPRB, 0.16025E+01_JPRB, 0.16133E+01_JPRB, 0.15691E+01_JPRB, 0.16963E+01_JPRB, 0.18231E+01_JPRB ,&
&  0.20356E+01_JPRB, 0.22041E+01_JPRB, 0.24679E+01_JPRB, 0.28928E+01_JPRB, 0.33695E+01_JPRB, 0.38455E+01_JPRB, 0.45198E+01_JPRB ,&
&  0.47094E+01_JPRB, 0.46269E+01_JPRB, 0.43826E+01_JPRB, 0.44720E+01_JPRB, 0.42810E+01_JPRB, 0.36847E+01_JPRB, 0.34423E+01_JPRB ,&
&  0.30727E+01_JPRB, 0.28198E+01_JPRB, 0.26225E+01_JPRB, 0.25069E+01_JPRB, 0.24058E+01_JPRB, 0.22826E+01_JPRB, 0.22126E+01_JPRB ,&
&  0.21479E+01_JPRB, 0.22788E+01_JPRB, 0.29496E+01_JPRB, 0.45249E+01_JPRB, 0.71137E+01_JPRB, 0.93575E+01_JPRB, 0.98982E+01_JPRB ,&
&  0.10559E+02_JPRB, 0.10226E+02_JPRB, 0.82766E+01_JPRB, 0.96294E+01_JPRB, 0.12461E+02_JPRB, 0.11808E+02_JPRB, 0.10455E+02_JPRB ,&
&  0.10304E+02_JPRB, 0.95089E+01_JPRB, 0.90768E+01_JPRB, 0.79041E+01_JPRB, 0.67535E+01_JPRB, 0.60903E+01_JPRB, 0.55634E+01_JPRB ,&
&  0.51832E+01_JPRB, 0.51274E+01_JPRB, 0.51696E+01_JPRB, 0.52222E+01_JPRB, 0.49898E+01_JPRB, 0.50882E+01_JPRB, 0.56088E+01_JPRB ,&
&  0.55711E+01_JPRB, 0.50989E+01_JPRB, 0.37639E+01_JPRB, 0.36578E+01_JPRB, 0.26613E+01_JPRB, 0.25643E+01_JPRB, 0.29357E+01_JPRB ,&
&  0.28671E+01_JPRB, 0.32493E+01_JPRB, 0.37092E+01_JPRB, 0.36272E+01_JPRB, 0.40868E+01_JPRB, 0.41162E+01_JPRB, 0.41647E+01_JPRB ,&
&  0.40277E+01_JPRB, 0.35987E+01_JPRB, 0.40706E+01_JPRB, 0.35148E+01_JPRB, 0.30764E+01_JPRB, 0.28814E+01_JPRB, 0.27341E+01_JPRB ,&
&  0.26105E+01_JPRB, 0.24586E+01_JPRB, 0.22966E+01_JPRB, 0.21889E+01_JPRB, 0.20844E+01_JPRB /)
ZBURD(:, 19,  9) = (/ &
&  0.18426E+01_JPRB, 0.17378E+01_JPRB, 0.17861E+01_JPRB, 0.17425E+01_JPRB, 0.17384E+01_JPRB, 0.17036E+01_JPRB, 0.17109E+01_JPRB ,&
&  0.16882E+01_JPRB, 0.16328E+01_JPRB, 0.15694E+01_JPRB, 0.15150E+01_JPRB, 0.14747E+01_JPRB, 0.14400E+01_JPRB, 0.14252E+01_JPRB ,&
&  0.14248E+01_JPRB, 0.15651E+01_JPRB, 0.14399E+01_JPRB, 0.13600E+01_JPRB, 0.13413E+01_JPRB, 0.15415E+01_JPRB, 0.18689E+01_JPRB ,&
&  0.22053E+01_JPRB, 0.25168E+01_JPRB, 0.31392E+01_JPRB, 0.38012E+01_JPRB, 0.45167E+01_JPRB, 0.52323E+01_JPRB, 0.61133E+01_JPRB ,&
&  0.67628E+01_JPRB, 0.57470E+01_JPRB, 0.53903E+01_JPRB, 0.47437E+01_JPRB, 0.42009E+01_JPRB, 0.37237E+01_JPRB, 0.34005E+01_JPRB ,&
&  0.31410E+01_JPRB, 0.29730E+01_JPRB, 0.27543E+01_JPRB, 0.24916E+01_JPRB, 0.22409E+01_JPRB, 0.21234E+01_JPRB, 0.21164E+01_JPRB ,&
&  0.22445E+01_JPRB, 0.27882E+01_JPRB, 0.41602E+01_JPRB, 0.61500E+01_JPRB, 0.91697E+01_JPRB, 0.10742E+02_JPRB, 0.10460E+02_JPRB ,&
&  0.91344E+01_JPRB, 0.92447E+01_JPRB, 0.13776E+02_JPRB, 0.11836E+02_JPRB, 0.12229E+02_JPRB, 0.13138E+02_JPRB, 0.11954E+02_JPRB ,&
&  0.10241E+02_JPRB, 0.83455E+01_JPRB, 0.78953E+01_JPRB, 0.70213E+01_JPRB, 0.80346E+01_JPRB, 0.68724E+01_JPRB, 0.59003E+01_JPRB ,&
&  0.53352E+01_JPRB, 0.53255E+01_JPRB, 0.51335E+01_JPRB, 0.51143E+01_JPRB, 0.54171E+01_JPRB, 0.62808E+01_JPRB, 0.63283E+01_JPRB ,&
&  0.56080E+01_JPRB, 0.57882E+01_JPRB, 0.47356E+01_JPRB, 0.37608E+01_JPRB, 0.31007E+01_JPRB, 0.30932E+01_JPRB, 0.31922E+01_JPRB ,&
&  0.33772E+01_JPRB, 0.36752E+01_JPRB, 0.43965E+01_JPRB, 0.51134E+01_JPRB, 0.56936E+01_JPRB, 0.54496E+01_JPRB, 0.49213E+01_JPRB ,&
&  0.45988E+01_JPRB, 0.50511E+01_JPRB, 0.49278E+01_JPRB, 0.39095E+01_JPRB, 0.33016E+01_JPRB, 0.30496E+01_JPRB, 0.28556E+01_JPRB ,&
&  0.26133E+01_JPRB, 0.23739E+01_JPRB, 0.21968E+01_JPRB, 0.21044E+01_JPRB, 0.20298E+01_JPRB /)
ZBURD(:, 20,  9) = (/ &
&  0.18491E+01_JPRB, 0.17729E+01_JPRB, 0.16901E+01_JPRB, 0.16508E+01_JPRB, 0.16748E+01_JPRB, 0.16464E+01_JPRB, 0.16185E+01_JPRB ,&
&  0.15609E+01_JPRB, 0.14968E+01_JPRB, 0.14802E+01_JPRB, 0.14743E+01_JPRB, 0.14536E+01_JPRB, 0.14243E+01_JPRB, 0.14122E+01_JPRB ,&
&  0.14264E+01_JPRB, 0.14374E+01_JPRB, 0.13072E+01_JPRB, 0.13408E+01_JPRB, 0.13677E+01_JPRB, 0.14347E+01_JPRB, 0.18462E+01_JPRB ,&
&  0.23116E+01_JPRB, 0.28749E+01_JPRB, 0.36543E+01_JPRB, 0.45552E+01_JPRB, 0.52890E+01_JPRB, 0.61330E+01_JPRB, 0.70078E+01_JPRB ,&
&  0.65901E+01_JPRB, 0.65059E+01_JPRB, 0.56827E+01_JPRB, 0.48366E+01_JPRB, 0.41269E+01_JPRB, 0.35935E+01_JPRB, 0.33260E+01_JPRB ,&
&  0.31115E+01_JPRB, 0.29790E+01_JPRB, 0.26803E+01_JPRB, 0.23255E+01_JPRB, 0.21670E+01_JPRB, 0.21393E+01_JPRB, 0.22034E+01_JPRB ,&
&  0.24826E+01_JPRB, 0.35506E+01_JPRB, 0.52035E+01_JPRB, 0.72525E+01_JPRB, 0.71221E+01_JPRB, 0.80798E+01_JPRB, 0.85284E+01_JPRB ,&
&  0.11949E+02_JPRB, 0.12553E+02_JPRB, 0.12398E+02_JPRB, 0.13401E+02_JPRB, 0.11158E+02_JPRB, 0.11669E+02_JPRB, 0.11577E+02_JPRB ,&
&  0.99841E+01_JPRB, 0.92100E+01_JPRB, 0.99305E+01_JPRB, 0.81603E+01_JPRB, 0.72724E+01_JPRB, 0.85208E+01_JPRB, 0.72215E+01_JPRB ,&
&  0.65597E+01_JPRB, 0.62098E+01_JPRB, 0.56977E+01_JPRB, 0.61031E+01_JPRB, 0.57334E+01_JPRB, 0.57822E+01_JPRB, 0.48222E+01_JPRB ,&
&  0.46572E+01_JPRB, 0.38933E+01_JPRB, 0.43425E+01_JPRB, 0.38683E+01_JPRB, 0.37425E+01_JPRB, 0.36365E+01_JPRB, 0.35799E+01_JPRB ,&
&  0.40014E+01_JPRB, 0.43481E+01_JPRB, 0.55071E+01_JPRB, 0.67074E+01_JPRB, 0.74723E+01_JPRB, 0.61915E+01_JPRB, 0.58406E+01_JPRB ,&
&  0.60435E+01_JPRB, 0.60197E+01_JPRB, 0.49449E+01_JPRB, 0.39523E+01_JPRB, 0.32923E+01_JPRB, 0.29864E+01_JPRB, 0.27994E+01_JPRB ,&
&  0.26020E+01_JPRB, 0.24076E+01_JPRB, 0.22518E+01_JPRB, 0.21484E+01_JPRB, 0.20392E+01_JPRB /)
ZBURD(:, 21,  9) = (/ &
&  0.16814E+01_JPRB, 0.16448E+01_JPRB, 0.15317E+01_JPRB, 0.15328E+01_JPRB, 0.15873E+01_JPRB, 0.15568E+01_JPRB, 0.15192E+01_JPRB ,&
&  0.14592E+01_JPRB, 0.14256E+01_JPRB, 0.14431E+01_JPRB, 0.14467E+01_JPRB, 0.14272E+01_JPRB, 0.14068E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.13933E+01_JPRB, 0.14130E+01_JPRB, 0.13033E+01_JPRB, 0.12704E+01_JPRB, 0.13907E+01_JPRB, 0.14571E+01_JPRB, 0.17026E+01_JPRB ,&
&  0.24046E+01_JPRB, 0.31565E+01_JPRB, 0.39639E+01_JPRB, 0.49503E+01_JPRB, 0.56989E+01_JPRB, 0.63264E+01_JPRB, 0.65674E+01_JPRB ,&
&  0.63406E+01_JPRB, 0.53329E+01_JPRB, 0.47216E+01_JPRB, 0.42549E+01_JPRB, 0.38499E+01_JPRB, 0.35280E+01_JPRB, 0.32546E+01_JPRB ,&
&  0.29978E+01_JPRB, 0.25721E+01_JPRB, 0.21268E+01_JPRB, 0.18939E+01_JPRB, 0.18998E+01_JPRB, 0.19286E+01_JPRB, 0.21181E+01_JPRB ,&
&  0.29166E+01_JPRB, 0.39738E+01_JPRB, 0.51991E+01_JPRB, 0.65015E+01_JPRB, 0.64391E+01_JPRB, 0.70112E+01_JPRB, 0.89185E+01_JPRB ,&
&  0.99042E+01_JPRB, 0.10081E+02_JPRB, 0.11858E+02_JPRB, 0.13322E+02_JPRB, 0.13806E+02_JPRB, 0.11897E+02_JPRB, 0.11484E+02_JPRB ,&
&  0.82560E+01_JPRB, 0.65247E+01_JPRB, 0.58667E+01_JPRB, 0.55755E+01_JPRB, 0.65700E+01_JPRB, 0.90412E+01_JPRB, 0.84448E+01_JPRB ,&
&  0.80276E+01_JPRB, 0.75726E+01_JPRB, 0.69318E+01_JPRB, 0.59834E+01_JPRB, 0.56447E+01_JPRB, 0.50703E+01_JPRB, 0.64432E+01_JPRB ,&
&  0.63803E+01_JPRB, 0.62017E+01_JPRB, 0.52261E+01_JPRB, 0.42353E+01_JPRB, 0.34754E+01_JPRB, 0.39889E+01_JPRB, 0.42199E+01_JPRB ,&
&  0.46890E+01_JPRB, 0.52981E+01_JPRB, 0.87742E+01_JPRB, 0.94590E+01_JPRB, 0.84942E+01_JPRB, 0.73991E+01_JPRB, 0.76425E+01_JPRB ,&
&  0.67741E+01_JPRB, 0.64799E+01_JPRB, 0.52466E+01_JPRB, 0.38260E+01_JPRB, 0.29889E+01_JPRB, 0.25429E+01_JPRB, 0.23188E+01_JPRB ,&
&  0.22593E+01_JPRB, 0.21979E+01_JPRB, 0.20883E+01_JPRB, 0.19511E+01_JPRB, 0.18193E+01_JPRB /)
ZBURD(:, 22,  9) = (/ &
&  0.15420E+01_JPRB, 0.15066E+01_JPRB, 0.13894E+01_JPRB, 0.13748E+01_JPRB, 0.14204E+01_JPRB, 0.14577E+01_JPRB, 0.14489E+01_JPRB ,&
&  0.14275E+01_JPRB, 0.14366E+01_JPRB, 0.14539E+01_JPRB, 0.14307E+01_JPRB, 0.13979E+01_JPRB, 0.13828E+01_JPRB, 0.13621E+01_JPRB ,&
&  0.13902E+01_JPRB, 0.15775E+01_JPRB, 0.13737E+01_JPRB, 0.13519E+01_JPRB, 0.14743E+01_JPRB, 0.16310E+01_JPRB, 0.18714E+01_JPRB ,&
&  0.26896E+01_JPRB, 0.32606E+01_JPRB, 0.36772E+01_JPRB, 0.45925E+01_JPRB, 0.59400E+01_JPRB, 0.56522E+01_JPRB, 0.53478E+01_JPRB ,&
&  0.46072E+01_JPRB, 0.38564E+01_JPRB, 0.35995E+01_JPRB, 0.34149E+01_JPRB, 0.33827E+01_JPRB, 0.33637E+01_JPRB, 0.31716E+01_JPRB ,&
&  0.27476E+01_JPRB, 0.21974E+01_JPRB, 0.18743E+01_JPRB, 0.18508E+01_JPRB, 0.17843E+01_JPRB, 0.18521E+01_JPRB, 0.21001E+01_JPRB ,&
&  0.29939E+01_JPRB, 0.40830E+01_JPRB, 0.47481E+01_JPRB, 0.53267E+01_JPRB, 0.55688E+01_JPRB, 0.61806E+01_JPRB, 0.92927E+01_JPRB ,&
&  0.92735E+01_JPRB, 0.98913E+01_JPRB, 0.10993E+02_JPRB, 0.13165E+02_JPRB, 0.12477E+02_JPRB, 0.10808E+02_JPRB, 0.10248E+02_JPRB ,&
&  0.72869E+01_JPRB, 0.60450E+01_JPRB, 0.64538E+01_JPRB, 0.65243E+01_JPRB, 0.54611E+01_JPRB, 0.58904E+01_JPRB, 0.84742E+01_JPRB ,&
&  0.68763E+01_JPRB, 0.71687E+01_JPRB, 0.72027E+01_JPRB, 0.71190E+01_JPRB, 0.40317E+01_JPRB, 0.30831E+01_JPRB, 0.47320E+01_JPRB ,&
&  0.51283E+01_JPRB, 0.31347E+01_JPRB, 0.19596E+01_JPRB, 0.27425E+01_JPRB, 0.19607E+01_JPRB, 0.24057E+01_JPRB, 0.38843E+01_JPRB ,&
&  0.53053E+01_JPRB, 0.71910E+01_JPRB, 0.12174E+02_JPRB, 0.10484E+02_JPRB, 0.89370E+01_JPRB, 0.88294E+01_JPRB, 0.82086E+01_JPRB ,&
&  0.78798E+01_JPRB, 0.58467E+01_JPRB, 0.46190E+01_JPRB, 0.31732E+01_JPRB, 0.22883E+01_JPRB, 0.18975E+01_JPRB, 0.17706E+01_JPRB ,&
&  0.17351E+01_JPRB, 0.17616E+01_JPRB, 0.17619E+01_JPRB, 0.17068E+01_JPRB, 0.16098E+01_JPRB /)
ZBURD(:, 23,  9) = (/ &
&  0.15706E+01_JPRB, 0.14438E+01_JPRB, 0.12663E+01_JPRB, 0.12394E+01_JPRB, 0.12736E+01_JPRB, 0.13288E+01_JPRB, 0.13348E+01_JPRB ,&
&  0.13644E+01_JPRB, 0.14145E+01_JPRB, 0.14797E+01_JPRB, 0.14660E+01_JPRB, 0.14101E+01_JPRB, 0.13965E+01_JPRB, 0.13640E+01_JPRB ,&
&  0.13747E+01_JPRB, 0.15094E+01_JPRB, 0.16756E+01_JPRB, 0.17405E+01_JPRB, 0.18788E+01_JPRB, 0.19894E+01_JPRB, 0.25010E+01_JPRB ,&
&  0.30294E+01_JPRB, 0.32847E+01_JPRB, 0.36999E+01_JPRB, 0.42587E+01_JPRB, 0.48059E+01_JPRB, 0.42340E+01_JPRB, 0.41900E+01_JPRB ,&
&  0.35326E+01_JPRB, 0.30030E+01_JPRB, 0.28409E+01_JPRB, 0.28003E+01_JPRB, 0.29648E+01_JPRB, 0.30723E+01_JPRB, 0.29713E+01_JPRB ,&
&  0.25631E+01_JPRB, 0.21320E+01_JPRB, 0.19139E+01_JPRB, 0.17758E+01_JPRB, 0.16984E+01_JPRB, 0.17744E+01_JPRB, 0.21267E+01_JPRB ,&
&  0.29292E+01_JPRB, 0.38326E+01_JPRB, 0.42027E+01_JPRB, 0.42957E+01_JPRB, 0.55967E+01_JPRB, 0.57960E+01_JPRB, 0.63191E+01_JPRB ,&
&  0.68954E+01_JPRB, 0.74493E+01_JPRB, 0.10594E+02_JPRB, 0.11716E+02_JPRB, 0.11480E+02_JPRB, 0.11049E+02_JPRB, 0.98617E+01_JPRB ,&
&  0.87550E+01_JPRB, 0.78700E+01_JPRB, 0.64713E+01_JPRB, 0.64745E+01_JPRB, 0.56424E+01_JPRB, 0.47792E+01_JPRB, 0.56596E+01_JPRB ,&
&  0.63026E+01_JPRB, 0.60770E+01_JPRB, 0.50484E+01_JPRB, 0.41087E+01_JPRB, 0.39081E+01_JPRB, 0.27810E+01_JPRB, 0.19020E+01_JPRB ,&
&  0.16879E+01_JPRB, 0.16068E+01_JPRB, 0.13998E+01_JPRB, 0.14999E+01_JPRB, 0.13793E+01_JPRB, 0.18931E+01_JPRB, 0.32525E+01_JPRB ,&
&  0.67317E+01_JPRB, 0.10352E+02_JPRB, 0.12077E+02_JPRB, 0.98271E+01_JPRB, 0.83230E+01_JPRB, 0.83495E+01_JPRB, 0.91280E+01_JPRB ,&
&  0.60876E+01_JPRB, 0.49283E+01_JPRB, 0.32445E+01_JPRB, 0.22050E+01_JPRB, 0.17617E+01_JPRB, 0.15367E+01_JPRB, 0.14687E+01_JPRB ,&
&  0.14640E+01_JPRB, 0.14562E+01_JPRB, 0.14883E+01_JPRB, 0.15427E+01_JPRB, 0.15367E+01_JPRB /)
ZBURD(:, 24,  9) = (/ &
&  0.15221E+01_JPRB, 0.13270E+01_JPRB, 0.11391E+01_JPRB, 0.10938E+01_JPRB, 0.11627E+01_JPRB, 0.11929E+01_JPRB, 0.12298E+01_JPRB ,&
&  0.13077E+01_JPRB, 0.14134E+01_JPRB, 0.14718E+01_JPRB, 0.14564E+01_JPRB, 0.14212E+01_JPRB, 0.13954E+01_JPRB, 0.13703E+01_JPRB ,&
&  0.13967E+01_JPRB, 0.15290E+01_JPRB, 0.18949E+01_JPRB, 0.21935E+01_JPRB, 0.25844E+01_JPRB, 0.25869E+01_JPRB, 0.30729E+01_JPRB ,&
&  0.30260E+01_JPRB, 0.33013E+01_JPRB, 0.39925E+01_JPRB, 0.41489E+01_JPRB, 0.43730E+01_JPRB, 0.35708E+01_JPRB, 0.31676E+01_JPRB ,&
&  0.26643E+01_JPRB, 0.25070E+01_JPRB, 0.24889E+01_JPRB, 0.26368E+01_JPRB, 0.28038E+01_JPRB, 0.28371E+01_JPRB, 0.26896E+01_JPRB ,&
&  0.23014E+01_JPRB, 0.21024E+01_JPRB, 0.18766E+01_JPRB, 0.18504E+01_JPRB, 0.18398E+01_JPRB, 0.20014E+01_JPRB, 0.23550E+01_JPRB ,&
&  0.29720E+01_JPRB, 0.35836E+01_JPRB, 0.37904E+01_JPRB, 0.41398E+01_JPRB, 0.41114E+01_JPRB, 0.41135E+01_JPRB, 0.57458E+01_JPRB ,&
&  0.73152E+01_JPRB, 0.85829E+01_JPRB, 0.90430E+01_JPRB, 0.11116E+02_JPRB, 0.10916E+02_JPRB, 0.10021E+02_JPRB, 0.94890E+01_JPRB ,&
&  0.81665E+01_JPRB, 0.73452E+01_JPRB, 0.60118E+01_JPRB, 0.60323E+01_JPRB, 0.62988E+01_JPRB, 0.52192E+01_JPRB, 0.50517E+01_JPRB ,&
&  0.58882E+01_JPRB, 0.56612E+01_JPRB, 0.49987E+01_JPRB, 0.41775E+01_JPRB, 0.50809E+01_JPRB, 0.44765E+01_JPRB, 0.23784E+01_JPRB ,&
&  0.19694E+01_JPRB, 0.17302E+01_JPRB, 0.13974E+01_JPRB, 0.12285E+01_JPRB, 0.12874E+01_JPRB, 0.19062E+01_JPRB, 0.42091E+01_JPRB ,&
&  0.58648E+01_JPRB, 0.10733E+02_JPRB, 0.12420E+02_JPRB, 0.10290E+02_JPRB, 0.80380E+01_JPRB, 0.82791E+01_JPRB, 0.65116E+01_JPRB ,&
&  0.53057E+01_JPRB, 0.39475E+01_JPRB, 0.21894E+01_JPRB, 0.16308E+01_JPRB, 0.15237E+01_JPRB, 0.14382E+01_JPRB, 0.13499E+01_JPRB ,&
&  0.12675E+01_JPRB, 0.12614E+01_JPRB, 0.13205E+01_JPRB, 0.13883E+01_JPRB, 0.14156E+01_JPRB /)
ZBURD(:, 25,  9) = (/ &
&  0.12981E+01_JPRB, 0.11361E+01_JPRB, 0.10253E+01_JPRB, 0.96265E+00_JPRB, 0.10403E+01_JPRB, 0.11442E+01_JPRB, 0.11824E+01_JPRB ,&
&  0.13015E+01_JPRB, 0.14281E+01_JPRB, 0.14810E+01_JPRB, 0.14774E+01_JPRB, 0.14230E+01_JPRB, 0.14112E+01_JPRB, 0.14411E+01_JPRB ,&
&  0.15453E+01_JPRB, 0.17881E+01_JPRB, 0.22582E+01_JPRB, 0.27790E+01_JPRB, 0.30317E+01_JPRB, 0.30090E+01_JPRB, 0.30859E+01_JPRB ,&
&  0.27797E+01_JPRB, 0.31570E+01_JPRB, 0.39634E+01_JPRB, 0.42470E+01_JPRB, 0.33997E+01_JPRB, 0.30109E+01_JPRB, 0.23311E+01_JPRB ,&
&  0.22931E+01_JPRB, 0.23652E+01_JPRB, 0.23991E+01_JPRB, 0.24628E+01_JPRB, 0.24857E+01_JPRB, 0.24543E+01_JPRB, 0.23875E+01_JPRB ,&
&  0.21562E+01_JPRB, 0.20139E+01_JPRB, 0.19281E+01_JPRB, 0.19195E+01_JPRB, 0.20960E+01_JPRB, 0.23945E+01_JPRB, 0.27299E+01_JPRB ,&
&  0.31416E+01_JPRB, 0.34539E+01_JPRB, 0.36689E+01_JPRB, 0.40859E+01_JPRB, 0.33347E+01_JPRB, 0.46771E+01_JPRB, 0.57400E+01_JPRB ,&
&  0.64824E+01_JPRB, 0.72584E+01_JPRB, 0.74489E+01_JPRB, 0.89189E+01_JPRB, 0.98035E+01_JPRB, 0.93138E+01_JPRB, 0.83036E+01_JPRB ,&
&  0.73811E+01_JPRB, 0.60117E+01_JPRB, 0.59080E+01_JPRB, 0.61875E+01_JPRB, 0.64039E+01_JPRB, 0.72663E+01_JPRB, 0.52754E+01_JPRB ,&
&  0.48516E+01_JPRB, 0.56912E+01_JPRB, 0.52084E+01_JPRB, 0.45467E+01_JPRB, 0.51684E+01_JPRB, 0.56234E+01_JPRB, 0.52687E+01_JPRB ,&
&  0.25342E+01_JPRB, 0.16452E+01_JPRB, 0.15797E+01_JPRB, 0.14345E+01_JPRB, 0.12612E+01_JPRB, 0.19813E+01_JPRB, 0.71321E+01_JPRB ,&
&  0.59239E+01_JPRB, 0.98278E+01_JPRB, 0.10830E+02_JPRB, 0.90042E+01_JPRB, 0.75230E+01_JPRB, 0.65210E+01_JPRB, 0.56732E+01_JPRB ,&
&  0.38527E+01_JPRB, 0.27172E+01_JPRB, 0.18062E+01_JPRB, 0.14757E+01_JPRB, 0.14071E+01_JPRB, 0.13584E+01_JPRB, 0.12881E+01_JPRB ,&
&  0.12254E+01_JPRB, 0.11730E+01_JPRB, 0.12111E+01_JPRB, 0.12079E+01_JPRB, 0.13002E+01_JPRB /)
ZBURD(:, 26,  9) = (/ &
&  0.11434E+01_JPRB, 0.10508E+01_JPRB, 0.10405E+01_JPRB, 0.10397E+01_JPRB, 0.10914E+01_JPRB, 0.11397E+01_JPRB, 0.12221E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.14818E+01_JPRB, 0.15595E+01_JPRB, 0.15603E+01_JPRB, 0.15179E+01_JPRB, 0.15689E+01_JPRB, 0.16972E+01_JPRB ,&
&  0.19067E+01_JPRB, 0.22337E+01_JPRB, 0.26835E+01_JPRB, 0.30327E+01_JPRB, 0.31887E+01_JPRB, 0.28678E+01_JPRB, 0.25967E+01_JPRB ,&
&  0.23953E+01_JPRB, 0.26414E+01_JPRB, 0.27137E+01_JPRB, 0.28224E+01_JPRB, 0.27972E+01_JPRB, 0.26838E+01_JPRB, 0.19927E+01_JPRB ,&
&  0.19950E+01_JPRB, 0.19950E+01_JPRB, 0.19724E+01_JPRB, 0.20239E+01_JPRB, 0.20693E+01_JPRB, 0.21403E+01_JPRB, 0.21188E+01_JPRB ,&
&  0.19806E+01_JPRB, 0.19653E+01_JPRB, 0.19796E+01_JPRB, 0.21068E+01_JPRB, 0.24052E+01_JPRB, 0.26964E+01_JPRB, 0.29942E+01_JPRB ,&
&  0.32702E+01_JPRB, 0.34258E+01_JPRB, 0.35765E+01_JPRB, 0.36061E+01_JPRB, 0.40559E+01_JPRB, 0.47635E+01_JPRB, 0.53947E+01_JPRB ,&
&  0.58843E+01_JPRB, 0.67173E+01_JPRB, 0.73896E+01_JPRB, 0.78946E+01_JPRB, 0.84789E+01_JPRB, 0.84059E+01_JPRB, 0.74724E+01_JPRB ,&
&  0.65772E+01_JPRB, 0.60138E+01_JPRB, 0.57088E+01_JPRB, 0.58908E+01_JPRB, 0.62696E+01_JPRB, 0.70960E+01_JPRB, 0.67567E+01_JPRB ,&
&  0.57324E+01_JPRB, 0.48281E+01_JPRB, 0.46445E+01_JPRB, 0.50423E+01_JPRB, 0.55545E+01_JPRB, 0.59465E+01_JPRB, 0.77297E+01_JPRB ,&
&  0.73349E+01_JPRB, 0.35241E+01_JPRB, 0.23215E+01_JPRB, 0.25754E+01_JPRB, 0.17941E+01_JPRB, 0.21089E+01_JPRB, 0.41623E+01_JPRB ,&
&  0.58931E+01_JPRB, 0.85439E+01_JPRB, 0.93862E+01_JPRB, 0.71937E+01_JPRB, 0.61695E+01_JPRB, 0.46049E+01_JPRB, 0.36896E+01_JPRB ,&
&  0.26778E+01_JPRB, 0.19484E+01_JPRB, 0.14688E+01_JPRB, 0.13302E+01_JPRB, 0.12391E+01_JPRB, 0.12499E+01_JPRB, 0.12865E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.11093E+01_JPRB, 0.11175E+01_JPRB, 0.11420E+01_JPRB, 0.11417E+01_JPRB /)
ZBURD(:, 27,  9) = (/ &
&  0.11396E+01_JPRB, 0.11126E+01_JPRB, 0.11504E+01_JPRB, 0.11958E+01_JPRB, 0.12489E+01_JPRB, 0.13035E+01_JPRB, 0.13614E+01_JPRB ,&
&  0.14880E+01_JPRB, 0.16108E+01_JPRB, 0.16854E+01_JPRB, 0.17252E+01_JPRB, 0.17775E+01_JPRB, 0.19090E+01_JPRB, 0.20896E+01_JPRB ,&
&  0.23125E+01_JPRB, 0.25633E+01_JPRB, 0.28359E+01_JPRB, 0.31118E+01_JPRB, 0.33765E+01_JPRB, 0.33460E+01_JPRB, 0.20332E+01_JPRB ,&
&  0.18017E+01_JPRB, 0.22922E+01_JPRB, 0.21883E+01_JPRB, 0.21979E+01_JPRB, 0.21516E+01_JPRB, 0.21505E+01_JPRB, 0.18375E+01_JPRB ,&
&  0.17079E+01_JPRB, 0.16669E+01_JPRB, 0.16920E+01_JPRB, 0.17618E+01_JPRB, 0.18495E+01_JPRB, 0.19937E+01_JPRB, 0.20437E+01_JPRB ,&
&  0.20786E+01_JPRB, 0.22028E+01_JPRB, 0.23442E+01_JPRB, 0.26009E+01_JPRB, 0.28907E+01_JPRB, 0.31248E+01_JPRB, 0.32753E+01_JPRB ,&
&  0.33668E+01_JPRB, 0.34448E+01_JPRB, 0.35996E+01_JPRB, 0.37564E+01_JPRB, 0.43342E+01_JPRB, 0.47340E+01_JPRB, 0.52744E+01_JPRB ,&
&  0.55631E+01_JPRB, 0.56293E+01_JPRB, 0.65012E+01_JPRB, 0.72332E+01_JPRB, 0.75621E+01_JPRB, 0.70635E+01_JPRB, 0.62384E+01_JPRB ,&
&  0.62142E+01_JPRB, 0.62890E+01_JPRB, 0.64950E+01_JPRB, 0.58503E+01_JPRB, 0.57732E+01_JPRB, 0.63569E+01_JPRB, 0.68899E+01_JPRB ,&
&  0.62339E+01_JPRB, 0.55817E+01_JPRB, 0.54854E+01_JPRB, 0.54107E+01_JPRB, 0.56729E+01_JPRB, 0.58527E+01_JPRB, 0.67821E+01_JPRB ,&
&  0.72280E+01_JPRB, 0.54709E+01_JPRB, 0.32781E+01_JPRB, 0.20802E+01_JPRB, 0.21532E+01_JPRB, 0.27401E+01_JPRB, 0.36758E+01_JPRB ,&
&  0.59727E+01_JPRB, 0.68661E+01_JPRB, 0.71313E+01_JPRB, 0.67246E+01_JPRB, 0.38892E+01_JPRB, 0.27501E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.15329E+01_JPRB, 0.12239E+01_JPRB, 0.11845E+01_JPRB, 0.11469E+01_JPRB, 0.10997E+01_JPRB, 0.11671E+01_JPRB, 0.12900E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.10623E+01_JPRB, 0.10733E+01_JPRB, 0.10806E+01_JPRB, 0.11031E+01_JPRB /)
ZBURD(:, 28,  9) = (/ &
&  0.11621E+01_JPRB, 0.11836E+01_JPRB, 0.12386E+01_JPRB, 0.13364E+01_JPRB, 0.14619E+01_JPRB, 0.15882E+01_JPRB, 0.16029E+01_JPRB ,&
&  0.16189E+01_JPRB, 0.16973E+01_JPRB, 0.17862E+01_JPRB, 0.18842E+01_JPRB, 0.19923E+01_JPRB, 0.21312E+01_JPRB, 0.22766E+01_JPRB ,&
&  0.24188E+01_JPRB, 0.25626E+01_JPRB, 0.27082E+01_JPRB, 0.28657E+01_JPRB, 0.29679E+01_JPRB, 0.29094E+01_JPRB, 0.21204E+01_JPRB ,&
&  0.15770E+01_JPRB, 0.25212E+01_JPRB, 0.20064E+01_JPRB, 0.20798E+01_JPRB, 0.20998E+01_JPRB, 0.22067E+01_JPRB, 0.19105E+01_JPRB ,&
&  0.16250E+01_JPRB, 0.16034E+01_JPRB, 0.16573E+01_JPRB, 0.16509E+01_JPRB, 0.17171E+01_JPRB, 0.19119E+01_JPRB, 0.21427E+01_JPRB ,&
&  0.23051E+01_JPRB, 0.25159E+01_JPRB, 0.27663E+01_JPRB, 0.30321E+01_JPRB, 0.31918E+01_JPRB, 0.33292E+01_JPRB, 0.34417E+01_JPRB ,&
&  0.35517E+01_JPRB, 0.36793E+01_JPRB, 0.37602E+01_JPRB, 0.40583E+01_JPRB, 0.43509E+01_JPRB, 0.47846E+01_JPRB, 0.52918E+01_JPRB ,&
&  0.54357E+01_JPRB, 0.54277E+01_JPRB, 0.58899E+01_JPRB, 0.59592E+01_JPRB, 0.59974E+01_JPRB, 0.63776E+01_JPRB, 0.57247E+01_JPRB ,&
&  0.62108E+01_JPRB, 0.59254E+01_JPRB, 0.64968E+01_JPRB, 0.53444E+01_JPRB, 0.51513E+01_JPRB, 0.55181E+01_JPRB, 0.54636E+01_JPRB ,&
&  0.50870E+01_JPRB, 0.53264E+01_JPRB, 0.53297E+01_JPRB, 0.54533E+01_JPRB, 0.56129E+01_JPRB, 0.57182E+01_JPRB, 0.58655E+01_JPRB ,&
&  0.48169E+01_JPRB, 0.27109E+01_JPRB, 0.19368E+01_JPRB, 0.16454E+01_JPRB, 0.17286E+01_JPRB, 0.24011E+01_JPRB, 0.40487E+01_JPRB ,&
&  0.61514E+01_JPRB, 0.60611E+01_JPRB, 0.57856E+01_JPRB, 0.38798E+01_JPRB, 0.21812E+01_JPRB, 0.16150E+01_JPRB, 0.12240E+01_JPRB ,&
&  0.91611E+00_JPRB, 0.85643E+00_JPRB, 0.94110E+00_JPRB, 0.98090E+00_JPRB, 0.96948E+00_JPRB, 0.10245E+01_JPRB, 0.10841E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.10396E+01_JPRB, 0.11016E+01_JPRB, 0.10946E+01_JPRB, 0.11262E+01_JPRB /)
ZBURD(:, 29,  9) = (/ &
&  0.11681E+01_JPRB, 0.12123E+01_JPRB, 0.13259E+01_JPRB, 0.14864E+01_JPRB, 0.17231E+01_JPRB, 0.19765E+01_JPRB, 0.20791E+01_JPRB ,&
&  0.18720E+01_JPRB, 0.16546E+01_JPRB, 0.17920E+01_JPRB, 0.19283E+01_JPRB, 0.20432E+01_JPRB, 0.21818E+01_JPRB, 0.23171E+01_JPRB ,&
&  0.24668E+01_JPRB, 0.26092E+01_JPRB, 0.27339E+01_JPRB, 0.28731E+01_JPRB, 0.30060E+01_JPRB, 0.31280E+01_JPRB, 0.29191E+01_JPRB ,&
&  0.21558E+01_JPRB, 0.18413E+01_JPRB, 0.21774E+01_JPRB, 0.20508E+01_JPRB, 0.19753E+01_JPRB, 0.20832E+01_JPRB, 0.20709E+01_JPRB ,&
&  0.18491E+01_JPRB, 0.16759E+01_JPRB, 0.17145E+01_JPRB, 0.16696E+01_JPRB, 0.14871E+01_JPRB, 0.16758E+01_JPRB, 0.20261E+01_JPRB ,&
&  0.22601E+01_JPRB, 0.24949E+01_JPRB, 0.27630E+01_JPRB, 0.30082E+01_JPRB, 0.31716E+01_JPRB, 0.33891E+01_JPRB, 0.35886E+01_JPRB ,&
&  0.37527E+01_JPRB, 0.39249E+01_JPRB, 0.40071E+01_JPRB, 0.40865E+01_JPRB, 0.43759E+01_JPRB, 0.47264E+01_JPRB, 0.50085E+01_JPRB ,&
&  0.51322E+01_JPRB, 0.54742E+01_JPRB, 0.56571E+01_JPRB, 0.56806E+01_JPRB, 0.53888E+01_JPRB, 0.58648E+01_JPRB, 0.55833E+01_JPRB ,&
&  0.56633E+01_JPRB, 0.53169E+01_JPRB, 0.56274E+01_JPRB, 0.46238E+01_JPRB, 0.44882E+01_JPRB, 0.45222E+01_JPRB, 0.45434E+01_JPRB ,&
&  0.44739E+01_JPRB, 0.47164E+01_JPRB, 0.47478E+01_JPRB, 0.49712E+01_JPRB, 0.53061E+01_JPRB, 0.50476E+01_JPRB, 0.49240E+01_JPRB ,&
&  0.30364E+01_JPRB, 0.19914E+01_JPRB, 0.14595E+01_JPRB, 0.14554E+01_JPRB, 0.16023E+01_JPRB, 0.22047E+01_JPRB, 0.33550E+01_JPRB ,&
&  0.45191E+01_JPRB, 0.33423E+01_JPRB, 0.27493E+01_JPRB, 0.22726E+01_JPRB, 0.13425E+01_JPRB, 0.98891E+00_JPRB, 0.82147E+00_JPRB ,&
&  0.73982E+00_JPRB, 0.75905E+00_JPRB, 0.82476E+00_JPRB, 0.84488E+00_JPRB, 0.83337E+00_JPRB, 0.88081E+00_JPRB, 0.95807E+00_JPRB ,&
&  0.98702E+00_JPRB, 0.10127E+01_JPRB, 0.10784E+01_JPRB, 0.10882E+01_JPRB, 0.11444E+01_JPRB /)
ZBURD(:, 30,  9) = (/ &
&  0.10831E+01_JPRB, 0.11770E+01_JPRB, 0.12553E+01_JPRB, 0.13489E+01_JPRB, 0.14265E+01_JPRB, 0.15343E+01_JPRB, 0.15607E+01_JPRB ,&
&  0.15021E+01_JPRB, 0.14536E+01_JPRB, 0.15521E+01_JPRB, 0.16829E+01_JPRB, 0.18021E+01_JPRB, 0.19007E+01_JPRB, 0.19914E+01_JPRB ,&
&  0.20695E+01_JPRB, 0.21304E+01_JPRB, 0.21929E+01_JPRB, 0.22438E+01_JPRB, 0.23427E+01_JPRB, 0.24675E+01_JPRB, 0.25326E+01_JPRB ,&
&  0.21468E+01_JPRB, 0.15656E+01_JPRB, 0.18230E+01_JPRB, 0.19028E+01_JPRB, 0.17768E+01_JPRB, 0.18689E+01_JPRB, 0.19847E+01_JPRB ,&
&  0.17699E+01_JPRB, 0.16380E+01_JPRB, 0.16552E+01_JPRB, 0.16552E+01_JPRB, 0.13648E+01_JPRB, 0.14908E+01_JPRB, 0.17702E+01_JPRB ,&
&  0.20537E+01_JPRB, 0.23255E+01_JPRB, 0.25686E+01_JPRB, 0.27536E+01_JPRB, 0.29467E+01_JPRB, 0.32173E+01_JPRB, 0.34880E+01_JPRB ,&
&  0.37393E+01_JPRB, 0.39137E+01_JPRB, 0.40112E+01_JPRB, 0.41148E+01_JPRB, 0.42715E+01_JPRB, 0.44964E+01_JPRB, 0.46312E+01_JPRB ,&
&  0.47791E+01_JPRB, 0.49562E+01_JPRB, 0.52453E+01_JPRB, 0.55481E+01_JPRB, 0.56705E+01_JPRB, 0.51312E+01_JPRB, 0.49332E+01_JPRB ,&
&  0.47146E+01_JPRB, 0.43272E+01_JPRB, 0.44117E+01_JPRB, 0.45288E+01_JPRB, 0.34005E+01_JPRB, 0.34043E+01_JPRB, 0.35577E+01_JPRB ,&
&  0.40207E+01_JPRB, 0.39857E+01_JPRB, 0.40283E+01_JPRB, 0.41784E+01_JPRB, 0.41822E+01_JPRB, 0.39828E+01_JPRB, 0.35016E+01_JPRB ,&
&  0.24294E+01_JPRB, 0.16244E+01_JPRB, 0.13461E+01_JPRB, 0.14061E+01_JPRB, 0.15796E+01_JPRB, 0.22377E+01_JPRB, 0.24435E+01_JPRB ,&
&  0.30039E+01_JPRB, 0.18812E+01_JPRB, 0.16432E+01_JPRB, 0.17899E+01_JPRB, 0.11965E+01_JPRB, 0.87801E+00_JPRB, 0.73928E+00_JPRB ,&
&  0.69585E+00_JPRB, 0.73118E+00_JPRB, 0.74624E+00_JPRB, 0.74326E+00_JPRB, 0.74309E+00_JPRB, 0.79097E+00_JPRB, 0.85247E+00_JPRB ,&
&  0.88084E+00_JPRB, 0.91472E+00_JPRB, 0.95930E+00_JPRB, 0.10087E+01_JPRB, 0.10716E+01_JPRB /)
ZBURD(:, 31,  9) = (/ &
&  0.85835E+00_JPRB, 0.87941E+00_JPRB, 0.92964E+00_JPRB, 0.10162E+01_JPRB, 0.10853E+01_JPRB, 0.11434E+01_JPRB, 0.11676E+01_JPRB ,&
&  0.12209E+01_JPRB, 0.12685E+01_JPRB, 0.13364E+01_JPRB, 0.13902E+01_JPRB, 0.14372E+01_JPRB, 0.15328E+01_JPRB, 0.15926E+01_JPRB ,&
&  0.16014E+01_JPRB, 0.15754E+01_JPRB, 0.15993E+01_JPRB, 0.16551E+01_JPRB, 0.18062E+01_JPRB, 0.20512E+01_JPRB, 0.22987E+01_JPRB ,&
&  0.25499E+01_JPRB, 0.24874E+01_JPRB, 0.23445E+01_JPRB, 0.15782E+01_JPRB, 0.12038E+01_JPRB, 0.13877E+01_JPRB, 0.15042E+01_JPRB ,&
&  0.16053E+01_JPRB, 0.16353E+01_JPRB, 0.15025E+01_JPRB, 0.14097E+01_JPRB, 0.13591E+01_JPRB, 0.13864E+01_JPRB, 0.15269E+01_JPRB ,&
&  0.17299E+01_JPRB, 0.19662E+01_JPRB, 0.22047E+01_JPRB, 0.23722E+01_JPRB, 0.25548E+01_JPRB, 0.28163E+01_JPRB, 0.31208E+01_JPRB ,&
&  0.33879E+01_JPRB, 0.35595E+01_JPRB, 0.36465E+01_JPRB, 0.38355E+01_JPRB, 0.39776E+01_JPRB, 0.41928E+01_JPRB, 0.43723E+01_JPRB ,&
&  0.45320E+01_JPRB, 0.46355E+01_JPRB, 0.47682E+01_JPRB, 0.49052E+01_JPRB, 0.47840E+01_JPRB, 0.42825E+01_JPRB, 0.38556E+01_JPRB ,&
&  0.36251E+01_JPRB, 0.32930E+01_JPRB, 0.33614E+01_JPRB, 0.36324E+01_JPRB, 0.26544E+01_JPRB, 0.28564E+01_JPRB, 0.33153E+01_JPRB ,&
&  0.33184E+01_JPRB, 0.33313E+01_JPRB, 0.31433E+01_JPRB, 0.27626E+01_JPRB, 0.26275E+01_JPRB, 0.25629E+01_JPRB, 0.24043E+01_JPRB ,&
&  0.20225E+01_JPRB, 0.15279E+01_JPRB, 0.12733E+01_JPRB, 0.11429E+01_JPRB, 0.13388E+01_JPRB, 0.19688E+01_JPRB, 0.17102E+01_JPRB ,&
&  0.17207E+01_JPRB, 0.15459E+01_JPRB, 0.13359E+01_JPRB, 0.17772E+01_JPRB, 0.15258E+01_JPRB, 0.10157E+01_JPRB, 0.78482E+00_JPRB ,&
&  0.69311E+00_JPRB, 0.69567E+00_JPRB, 0.70900E+00_JPRB, 0.69795E+00_JPRB, 0.71473E+00_JPRB, 0.75578E+00_JPRB, 0.76332E+00_JPRB ,&
&  0.81653E+00_JPRB, 0.83002E+00_JPRB, 0.85878E+00_JPRB, 0.88308E+00_JPRB, 0.85671E+00_JPRB /)
ZBURD(:, 32,  9) = (/ &
&  0.60398E+00_JPRB, 0.59881E+00_JPRB, 0.62195E+00_JPRB, 0.69797E+00_JPRB, 0.74910E+00_JPRB, 0.78428E+00_JPRB, 0.84077E+00_JPRB ,&
&  0.90986E+00_JPRB, 0.10189E+01_JPRB, 0.11073E+01_JPRB, 0.11391E+01_JPRB, 0.11037E+01_JPRB, 0.10634E+01_JPRB, 0.10431E+01_JPRB ,&
&  0.10294E+01_JPRB, 0.94006E+00_JPRB, 0.87068E+00_JPRB, 0.86187E+00_JPRB, 0.91412E+00_JPRB, 0.10289E+01_JPRB, 0.11827E+01_JPRB ,&
&  0.14053E+01_JPRB, 0.16162E+01_JPRB, 0.18523E+01_JPRB, 0.24280E+01_JPRB, 0.25497E+01_JPRB, 0.13746E+01_JPRB, 0.14729E+01_JPRB ,&
&  0.19187E+01_JPRB, 0.19864E+01_JPRB, 0.15478E+01_JPRB, 0.13824E+01_JPRB, 0.13046E+01_JPRB, 0.12558E+01_JPRB, 0.12918E+01_JPRB ,&
&  0.14299E+01_JPRB, 0.15718E+01_JPRB, 0.17313E+01_JPRB, 0.19362E+01_JPRB, 0.21603E+01_JPRB, 0.23638E+01_JPRB, 0.26164E+01_JPRB ,&
&  0.28317E+01_JPRB, 0.29001E+01_JPRB, 0.29493E+01_JPRB, 0.30694E+01_JPRB, 0.33195E+01_JPRB, 0.36153E+01_JPRB, 0.38748E+01_JPRB ,&
&  0.40382E+01_JPRB, 0.41164E+01_JPRB, 0.42774E+01_JPRB, 0.42795E+01_JPRB, 0.38475E+01_JPRB, 0.34865E+01_JPRB, 0.29702E+01_JPRB ,&
&  0.25857E+01_JPRB, 0.22021E+01_JPRB, 0.20653E+01_JPRB, 0.26996E+01_JPRB, 0.28886E+01_JPRB, 0.26673E+01_JPRB, 0.26885E+01_JPRB ,&
&  0.27650E+01_JPRB, 0.27247E+01_JPRB, 0.24525E+01_JPRB, 0.19756E+01_JPRB, 0.16627E+01_JPRB, 0.18305E+01_JPRB, 0.18303E+01_JPRB ,&
&  0.16429E+01_JPRB, 0.12999E+01_JPRB, 0.11176E+01_JPRB, 0.10363E+01_JPRB, 0.10326E+01_JPRB, 0.14107E+01_JPRB, 0.12028E+01_JPRB ,&
&  0.10178E+01_JPRB, 0.11573E+01_JPRB, 0.10723E+01_JPRB, 0.15430E+01_JPRB, 0.24023E+01_JPRB, 0.12980E+01_JPRB, 0.84095E+00_JPRB ,&
&  0.67936E+00_JPRB, 0.67572E+00_JPRB, 0.68294E+00_JPRB, 0.68373E+00_JPRB, 0.68533E+00_JPRB, 0.70303E+00_JPRB, 0.70575E+00_JPRB ,&
&  0.71648E+00_JPRB, 0.70136E+00_JPRB, 0.70096E+00_JPRB, 0.66570E+00_JPRB, 0.61281E+00_JPRB /)
ZBURD(:, 33,  9) = (/ &
&  0.63134E+00_JPRB, 0.62778E+00_JPRB, 0.61519E+00_JPRB, 0.61019E+00_JPRB, 0.61746E+00_JPRB, 0.62928E+00_JPRB, 0.64859E+00_JPRB ,&
&  0.67961E+00_JPRB, 0.71747E+00_JPRB, 0.75459E+00_JPRB, 0.79663E+00_JPRB, 0.80406E+00_JPRB, 0.75063E+00_JPRB, 0.74479E+00_JPRB ,&
&  0.73632E+00_JPRB, 0.74668E+00_JPRB, 0.73990E+00_JPRB, 0.82279E+00_JPRB, 0.84906E+00_JPRB, 0.86056E+00_JPRB, 0.96314E+00_JPRB ,&
&  0.95868E+00_JPRB, 0.10658E+01_JPRB, 0.11465E+01_JPRB, 0.12751E+01_JPRB, 0.20461E+01_JPRB, 0.18958E+01_JPRB, 0.13854E+01_JPRB ,&
&  0.16223E+01_JPRB, 0.16717E+01_JPRB, 0.14078E+01_JPRB, 0.11995E+01_JPRB, 0.11431E+01_JPRB, 0.10416E+01_JPRB, 0.10035E+01_JPRB ,&
&  0.10718E+01_JPRB, 0.11743E+01_JPRB, 0.13495E+01_JPRB, 0.15170E+01_JPRB, 0.16478E+01_JPRB, 0.17728E+01_JPRB, 0.19257E+01_JPRB ,&
&  0.20166E+01_JPRB, 0.20931E+01_JPRB, 0.20403E+01_JPRB, 0.20236E+01_JPRB, 0.23034E+01_JPRB, 0.25331E+01_JPRB, 0.26851E+01_JPRB ,&
&  0.27981E+01_JPRB, 0.29501E+01_JPRB, 0.31331E+01_JPRB, 0.29571E+01_JPRB, 0.26419E+01_JPRB, 0.24186E+01_JPRB, 0.19520E+01_JPRB ,&
&  0.15664E+01_JPRB, 0.12837E+01_JPRB, 0.12260E+01_JPRB, 0.16407E+01_JPRB, 0.18394E+01_JPRB, 0.17182E+01_JPRB, 0.20347E+01_JPRB ,&
&  0.21331E+01_JPRB, 0.21825E+01_JPRB, 0.19020E+01_JPRB, 0.16943E+01_JPRB, 0.14538E+01_JPRB, 0.14538E+01_JPRB, 0.14522E+01_JPRB ,&
&  0.12618E+01_JPRB, 0.10351E+01_JPRB, 0.92350E+00_JPRB, 0.86247E+00_JPRB, 0.83824E+00_JPRB, 0.10092E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11497E+01_JPRB, 0.99871E+00_JPRB, 0.98861E+00_JPRB, 0.14329E+01_JPRB, 0.13213E+01_JPRB, 0.84035E+00_JPRB, 0.64705E+00_JPRB ,&
&  0.61445E+00_JPRB, 0.62230E+00_JPRB, 0.61919E+00_JPRB, 0.61747E+00_JPRB, 0.62864E+00_JPRB, 0.64510E+00_JPRB, 0.63220E+00_JPRB ,&
&  0.62854E+00_JPRB, 0.61372E+00_JPRB, 0.60375E+00_JPRB, 0.59885E+00_JPRB, 0.60608E+00_JPRB /)
ZBURD(:, 34,  9) = (/ &
&  0.76880E+00_JPRB, 0.77362E+00_JPRB, 0.78026E+00_JPRB, 0.78992E+00_JPRB, 0.81022E+00_JPRB, 0.83301E+00_JPRB, 0.85186E+00_JPRB ,&
&  0.88714E+00_JPRB, 0.91419E+00_JPRB, 0.94090E+00_JPRB, 0.95308E+00_JPRB, 0.96773E+00_JPRB, 0.97767E+00_JPRB, 0.98092E+00_JPRB ,&
&  0.10020E+01_JPRB, 0.10293E+01_JPRB, 0.10299E+01_JPRB, 0.10388E+01_JPRB, 0.10660E+01_JPRB, 0.10848E+01_JPRB, 0.10686E+01_JPRB ,&
&  0.10468E+01_JPRB, 0.10214E+01_JPRB, 0.10435E+01_JPRB, 0.10831E+01_JPRB, 0.12818E+01_JPRB, 0.14116E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.13468E+01_JPRB, 0.13847E+01_JPRB, 0.14583E+01_JPRB, 0.11571E+01_JPRB, 0.10593E+01_JPRB, 0.99693E+00_JPRB, 0.93349E+00_JPRB ,&
&  0.93809E+00_JPRB, 0.96665E+00_JPRB, 0.10107E+01_JPRB, 0.10543E+01_JPRB, 0.11099E+01_JPRB, 0.11324E+01_JPRB, 0.11856E+01_JPRB ,&
&  0.11839E+01_JPRB, 0.12036E+01_JPRB, 0.12332E+01_JPRB, 0.12922E+01_JPRB, 0.14162E+01_JPRB, 0.16111E+01_JPRB, 0.15956E+01_JPRB ,&
&  0.15801E+01_JPRB, 0.16714E+01_JPRB, 0.15434E+01_JPRB, 0.14993E+01_JPRB, 0.14059E+01_JPRB, 0.12009E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.96113E+00_JPRB, 0.93055E+00_JPRB, 0.83942E+00_JPRB, 0.11159E+01_JPRB, 0.13504E+01_JPRB, 0.14485E+01_JPRB, 0.16425E+01_JPRB ,&
&  0.17359E+01_JPRB, 0.15565E+01_JPRB, 0.13205E+01_JPRB, 0.13207E+01_JPRB, 0.13244E+01_JPRB, 0.11530E+01_JPRB, 0.11222E+01_JPRB ,&
&  0.98967E+00_JPRB, 0.89601E+00_JPRB, 0.84748E+00_JPRB, 0.84751E+00_JPRB, 0.93637E+00_JPRB, 0.13155E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.11685E+01_JPRB, 0.99029E+00_JPRB, 0.11284E+01_JPRB, 0.17196E+01_JPRB, 0.10007E+01_JPRB, 0.75331E+00_JPRB, 0.64423E+00_JPRB ,&
&  0.64964E+00_JPRB, 0.65245E+00_JPRB, 0.65530E+00_JPRB, 0.65914E+00_JPRB, 0.66539E+00_JPRB, 0.68247E+00_JPRB, 0.67389E+00_JPRB ,&
&  0.66531E+00_JPRB, 0.65819E+00_JPRB, 0.67182E+00_JPRB, 0.69336E+00_JPRB, 0.73571E+00_JPRB /)
ZBURD(:, 35,  9) = (/ &
&  0.87080E+00_JPRB, 0.86697E+00_JPRB, 0.87977E+00_JPRB, 0.89800E+00_JPRB, 0.91530E+00_JPRB, 0.93575E+00_JPRB, 0.96864E+00_JPRB ,&
&  0.10078E+01_JPRB, 0.10381E+01_JPRB, 0.10665E+01_JPRB, 0.10860E+01_JPRB, 0.10912E+01_JPRB, 0.11038E+01_JPRB, 0.11164E+01_JPRB ,&
&  0.11314E+01_JPRB, 0.11267E+01_JPRB, 0.11442E+01_JPRB, 0.11740E+01_JPRB, 0.12098E+01_JPRB, 0.12236E+01_JPRB, 0.12370E+01_JPRB ,&
&  0.12544E+01_JPRB, 0.12966E+01_JPRB, 0.13450E+01_JPRB, 0.14031E+01_JPRB, 0.15012E+01_JPRB, 0.16734E+01_JPRB, 0.18448E+01_JPRB ,&
&  0.14012E+01_JPRB, 0.14714E+01_JPRB, 0.13852E+01_JPRB, 0.11403E+01_JPRB, 0.10633E+01_JPRB, 0.10765E+01_JPRB, 0.10380E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.10054E+01_JPRB, 0.10131E+01_JPRB, 0.10199E+01_JPRB, 0.10383E+01_JPRB, 0.10717E+01_JPRB, 0.10683E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.10968E+01_JPRB, 0.11098E+01_JPRB, 0.11494E+01_JPRB, 0.10985E+01_JPRB, 0.12803E+01_JPRB, 0.12168E+01_JPRB ,&
&  0.11393E+01_JPRB, 0.10762E+01_JPRB, 0.83309E+00_JPRB, 0.83379E+00_JPRB, 0.82648E+00_JPRB, 0.72652E+00_JPRB, 0.67139E+00_JPRB ,&
&  0.81871E+00_JPRB, 0.95096E+00_JPRB, 0.87785E+00_JPRB, 0.11129E+01_JPRB, 0.12513E+01_JPRB, 0.13122E+01_JPRB, 0.13443E+01_JPRB ,&
&  0.13214E+01_JPRB, 0.11749E+01_JPRB, 0.10516E+01_JPRB, 0.99404E+00_JPRB, 0.97810E+00_JPRB, 0.95230E+00_JPRB, 0.90893E+00_JPRB ,&
&  0.85714E+00_JPRB, 0.82683E+00_JPRB, 0.79007E+00_JPRB, 0.77574E+00_JPRB, 0.10574E+01_JPRB, 0.12121E+01_JPRB, 0.18037E+01_JPRB ,&
&  0.10800E+01_JPRB, 0.94088E+00_JPRB, 0.78717E+00_JPRB, 0.11280E+01_JPRB, 0.88825E+00_JPRB, 0.68195E+00_JPRB, 0.72671E+00_JPRB ,&
&  0.78030E+00_JPRB, 0.81549E+00_JPRB, 0.80644E+00_JPRB, 0.78334E+00_JPRB, 0.77618E+00_JPRB, 0.77659E+00_JPRB, 0.75210E+00_JPRB ,&
&  0.73803E+00_JPRB, 0.75232E+00_JPRB, 0.78613E+00_JPRB, 0.80009E+00_JPRB, 0.83627E+00_JPRB /)
ZBURD(:, 36,  9) = (/ &
&  0.91922E+00_JPRB, 0.94001E+00_JPRB, 0.96228E+00_JPRB, 0.98740E+00_JPRB, 0.10115E+01_JPRB, 0.10341E+01_JPRB, 0.10590E+01_JPRB ,&
&  0.10906E+01_JPRB, 0.11135E+01_JPRB, 0.11360E+01_JPRB, 0.11487E+01_JPRB, 0.11542E+01_JPRB, 0.11686E+01_JPRB, 0.11726E+01_JPRB ,&
&  0.11746E+01_JPRB, 0.11760E+01_JPRB, 0.11933E+01_JPRB, 0.12046E+01_JPRB, 0.12068E+01_JPRB, 0.12053E+01_JPRB, 0.12233E+01_JPRB ,&
&  0.12261E+01_JPRB, 0.12615E+01_JPRB, 0.12900E+01_JPRB, 0.13271E+01_JPRB, 0.13583E+01_JPRB, 0.13697E+01_JPRB, 0.14561E+01_JPRB ,&
&  0.11574E+01_JPRB, 0.16620E+01_JPRB, 0.16035E+01_JPRB, 0.12722E+01_JPRB, 0.12011E+01_JPRB, 0.11289E+01_JPRB, 0.11027E+01_JPRB ,&
&  0.10920E+01_JPRB, 0.10852E+01_JPRB, 0.10929E+01_JPRB, 0.11165E+01_JPRB, 0.11421E+01_JPRB, 0.11730E+01_JPRB, 0.11824E+01_JPRB ,&
&  0.11976E+01_JPRB, 0.12097E+01_JPRB, 0.12224E+01_JPRB, 0.12310E+01_JPRB, 0.12413E+01_JPRB, 0.12309E+01_JPRB, 0.11877E+01_JPRB ,&
&  0.11146E+01_JPRB, 0.10550E+01_JPRB, 0.79198E+00_JPRB, 0.73982E+00_JPRB, 0.75528E+00_JPRB, 0.71531E+00_JPRB, 0.64832E+00_JPRB ,&
&  0.78228E+00_JPRB, 0.93428E+00_JPRB, 0.10163E+01_JPRB, 0.11131E+01_JPRB, 0.11416E+01_JPRB, 0.10851E+01_JPRB, 0.10492E+01_JPRB ,&
&  0.96421E+00_JPRB, 0.85373E+00_JPRB, 0.79410E+00_JPRB, 0.76278E+00_JPRB, 0.77889E+00_JPRB, 0.77944E+00_JPRB, 0.76056E+00_JPRB ,&
&  0.72224E+00_JPRB, 0.72033E+00_JPRB, 0.74546E+00_JPRB, 0.77804E+00_JPRB, 0.85395E+00_JPRB, 0.15562E+01_JPRB, 0.18290E+01_JPRB ,&
&  0.92105E+00_JPRB, 0.82237E+00_JPRB, 0.65048E+00_JPRB, 0.81773E+00_JPRB, 0.84476E+00_JPRB, 0.84466E+00_JPRB, 0.96063E+00_JPRB ,&
&  0.10598E+01_JPRB, 0.10762E+01_JPRB, 0.99302E+00_JPRB, 0.89284E+00_JPRB, 0.85187E+00_JPRB, 0.83575E+00_JPRB, 0.80911E+00_JPRB ,&
&  0.80465E+00_JPRB, 0.82954E+00_JPRB, 0.85416E+00_JPRB, 0.87459E+00_JPRB, 0.89685E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART17',1,ZHOOK_HANDLE)
END SUBROUTINE PART17



SUBROUTINE PART18

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART18',0,ZHOOK_HANDLE)


ZBURD(:, 37,  9) = (/ &
&  0.96540E+00_JPRB, 0.98363E+00_JPRB, 0.10016E+01_JPRB, 0.10307E+01_JPRB, 0.10626E+01_JPRB, 0.10927E+01_JPRB, 0.11159E+01_JPRB ,&
&  0.11388E+01_JPRB, 0.11556E+01_JPRB, 0.11760E+01_JPRB, 0.11935E+01_JPRB, 0.12057E+01_JPRB, 0.12134E+01_JPRB, 0.12139E+01_JPRB ,&
&  0.12155E+01_JPRB, 0.12197E+01_JPRB, 0.12349E+01_JPRB, 0.12475E+01_JPRB, 0.12486E+01_JPRB, 0.12598E+01_JPRB, 0.12813E+01_JPRB ,&
&  0.12945E+01_JPRB, 0.13074E+01_JPRB, 0.13409E+01_JPRB, 0.13895E+01_JPRB, 0.14564E+01_JPRB, 0.15973E+01_JPRB, 0.14636E+01_JPRB ,&
&  0.15848E+01_JPRB, 0.17539E+01_JPRB, 0.17288E+01_JPRB, 0.13741E+01_JPRB, 0.12450E+01_JPRB, 0.11980E+01_JPRB, 0.11781E+01_JPRB ,&
&  0.11671E+01_JPRB, 0.11507E+01_JPRB, 0.11477E+01_JPRB, 0.11670E+01_JPRB, 0.12022E+01_JPRB, 0.12410E+01_JPRB, 0.12556E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.13013E+01_JPRB, 0.13304E+01_JPRB, 0.13568E+01_JPRB, 0.14081E+01_JPRB, 0.14162E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.13250E+01_JPRB, 0.12903E+01_JPRB, 0.11205E+01_JPRB, 0.10317E+01_JPRB, 0.10340E+01_JPRB, 0.92614E+00_JPRB, 0.83409E+00_JPRB ,&
&  0.88317E+00_JPRB, 0.87821E+00_JPRB, 0.87006E+00_JPRB, 0.10476E+01_JPRB, 0.10390E+01_JPRB, 0.92890E+00_JPRB, 0.88901E+00_JPRB ,&
&  0.78376E+00_JPRB, 0.69131E+00_JPRB, 0.64109E+00_JPRB, 0.63815E+00_JPRB, 0.65399E+00_JPRB, 0.69317E+00_JPRB, 0.70036E+00_JPRB ,&
&  0.71984E+00_JPRB, 0.75756E+00_JPRB, 0.79445E+00_JPRB, 0.84904E+00_JPRB, 0.10178E+01_JPRB, 0.99577E+00_JPRB, 0.11956E+01_JPRB ,&
&  0.92876E+00_JPRB, 0.77949E+00_JPRB, 0.85724E+00_JPRB, 0.94074E+00_JPRB, 0.99987E+00_JPRB, 0.10565E+01_JPRB, 0.12098E+01_JPRB ,&
&  0.13906E+01_JPRB, 0.15378E+01_JPRB, 0.14000E+01_JPRB, 0.11984E+01_JPRB, 0.99249E+00_JPRB, 0.90391E+00_JPRB, 0.86544E+00_JPRB ,&
&  0.86161E+00_JPRB, 0.89011E+00_JPRB, 0.91307E+00_JPRB, 0.92678E+00_JPRB, 0.94548E+00_JPRB /)
ZBURD(:, 38,  9) = (/ &
&  0.96600E+00_JPRB, 0.98463E+00_JPRB, 0.99849E+00_JPRB, 0.10181E+01_JPRB, 0.10429E+01_JPRB, 0.10720E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.11165E+01_JPRB, 0.11345E+01_JPRB, 0.11512E+01_JPRB, 0.11713E+01_JPRB, 0.11860E+01_JPRB, 0.12072E+01_JPRB, 0.12145E+01_JPRB ,&
&  0.12359E+01_JPRB, 0.12334E+01_JPRB, 0.12450E+01_JPRB, 0.12583E+01_JPRB, 0.12712E+01_JPRB, 0.12851E+01_JPRB, 0.13072E+01_JPRB ,&
&  0.13206E+01_JPRB, 0.13190E+01_JPRB, 0.13293E+01_JPRB, 0.13468E+01_JPRB, 0.13518E+01_JPRB, 0.14418E+01_JPRB, 0.13236E+01_JPRB ,&
&  0.17490E+01_JPRB, 0.19285E+01_JPRB, 0.18238E+01_JPRB, 0.14735E+01_JPRB, 0.13417E+01_JPRB, 0.12914E+01_JPRB, 0.12671E+01_JPRB ,&
&  0.12409E+01_JPRB, 0.12224E+01_JPRB, 0.11924E+01_JPRB, 0.11989E+01_JPRB, 0.12305E+01_JPRB, 0.12822E+01_JPRB, 0.13286E+01_JPRB ,&
&  0.13578E+01_JPRB, 0.13948E+01_JPRB, 0.14465E+01_JPRB, 0.15042E+01_JPRB, 0.15914E+01_JPRB, 0.16348E+01_JPRB, 0.16316E+01_JPRB ,&
&  0.16119E+01_JPRB, 0.15994E+01_JPRB, 0.14737E+01_JPRB, 0.14689E+01_JPRB, 0.14846E+01_JPRB, 0.12408E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.91779E+00_JPRB, 0.88330E+00_JPRB, 0.92121E+00_JPRB, 0.99801E+00_JPRB, 0.93861E+00_JPRB, 0.87458E+00_JPRB, 0.82212E+00_JPRB ,&
&  0.71549E+00_JPRB, 0.65921E+00_JPRB, 0.63564E+00_JPRB, 0.60711E+00_JPRB, 0.61098E+00_JPRB, 0.63813E+00_JPRB, 0.70162E+00_JPRB ,&
&  0.74559E+00_JPRB, 0.79357E+00_JPRB, 0.85655E+00_JPRB, 0.91149E+00_JPRB, 0.10084E+01_JPRB, 0.99781E+00_JPRB, 0.11465E+01_JPRB ,&
&  0.10525E+01_JPRB, 0.10168E+01_JPRB, 0.10435E+01_JPRB, 0.84685E+00_JPRB, 0.10884E+01_JPRB, 0.11737E+01_JPRB, 0.13228E+01_JPRB ,&
&  0.13694E+01_JPRB, 0.18355E+01_JPRB, 0.22026E+01_JPRB, 0.18544E+01_JPRB, 0.13628E+01_JPRB, 0.96409E+00_JPRB, 0.87188E+00_JPRB ,&
&  0.87261E+00_JPRB, 0.89739E+00_JPRB, 0.92188E+00_JPRB, 0.93534E+00_JPRB, 0.95075E+00_JPRB /)
ZBURD(:, 39,  9) = (/ &
&  0.92301E+00_JPRB, 0.93959E+00_JPRB, 0.95469E+00_JPRB, 0.96543E+00_JPRB, 0.96800E+00_JPRB, 0.98146E+00_JPRB, 0.10060E+01_JPRB ,&
&  0.10288E+01_JPRB, 0.10512E+01_JPRB, 0.10730E+01_JPRB, 0.10916E+01_JPRB, 0.11116E+01_JPRB, 0.11265E+01_JPRB, 0.11556E+01_JPRB ,&
&  0.11971E+01_JPRB, 0.12359E+01_JPRB, 0.12590E+01_JPRB, 0.13015E+01_JPRB, 0.13314E+01_JPRB, 0.13506E+01_JPRB, 0.14069E+01_JPRB ,&
&  0.14620E+01_JPRB, 0.15294E+01_JPRB, 0.15788E+01_JPRB, 0.16081E+01_JPRB, 0.16015E+01_JPRB, 0.16969E+01_JPRB, 0.15209E+01_JPRB ,&
&  0.18665E+01_JPRB, 0.20515E+01_JPRB, 0.18985E+01_JPRB, 0.15571E+01_JPRB, 0.14350E+01_JPRB, 0.13676E+01_JPRB, 0.13104E+01_JPRB ,&
&  0.12801E+01_JPRB, 0.12349E+01_JPRB, 0.11637E+01_JPRB, 0.11902E+01_JPRB, 0.12220E+01_JPRB, 0.12700E+01_JPRB, 0.13444E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.14627E+01_JPRB, 0.15280E+01_JPRB, 0.15910E+01_JPRB, 0.16822E+01_JPRB, 0.17581E+01_JPRB, 0.18034E+01_JPRB ,&
&  0.18346E+01_JPRB, 0.18631E+01_JPRB, 0.18925E+01_JPRB, 0.17523E+01_JPRB, 0.17312E+01_JPRB, 0.15723E+01_JPRB, 0.12696E+01_JPRB ,&
&  0.95213E+00_JPRB, 0.86606E+00_JPRB, 0.91118E+00_JPRB, 0.95678E+00_JPRB, 0.92504E+00_JPRB, 0.81793E+00_JPRB, 0.78356E+00_JPRB ,&
&  0.68866E+00_JPRB, 0.63314E+00_JPRB, 0.62792E+00_JPRB, 0.61681E+00_JPRB, 0.60991E+00_JPRB, 0.64979E+00_JPRB, 0.68674E+00_JPRB ,&
&  0.74672E+00_JPRB, 0.82963E+00_JPRB, 0.90234E+00_JPRB, 0.97009E+00_JPRB, 0.11048E+01_JPRB, 0.12286E+01_JPRB, 0.11315E+01_JPRB ,&
&  0.11356E+01_JPRB, 0.10846E+01_JPRB, 0.10967E+01_JPRB, 0.12097E+01_JPRB, 0.11678E+01_JPRB, 0.13704E+01_JPRB, 0.15900E+01_JPRB ,&
&  0.18995E+01_JPRB, 0.18290E+01_JPRB, 0.21253E+01_JPRB, 0.31511E+01_JPRB, 0.26934E+01_JPRB, 0.22797E+01_JPRB, 0.87793E+00_JPRB ,&
&  0.87265E+00_JPRB, 0.89092E+00_JPRB, 0.91851E+00_JPRB, 0.92299E+00_JPRB, 0.92713E+00_JPRB /)
ZBURD(:, 40,  9) = (/ &
&  0.82246E+00_JPRB, 0.85811E+00_JPRB, 0.86465E+00_JPRB, 0.89396E+00_JPRB, 0.89351E+00_JPRB, 0.88374E+00_JPRB, 0.89788E+00_JPRB ,&
&  0.91635E+00_JPRB, 0.93165E+00_JPRB, 0.95874E+00_JPRB, 0.98587E+00_JPRB, 0.10074E+01_JPRB, 0.10171E+01_JPRB, 0.10369E+01_JPRB ,&
&  0.10701E+01_JPRB, 0.11134E+01_JPRB, 0.11649E+01_JPRB, 0.12329E+01_JPRB, 0.12838E+01_JPRB, 0.13263E+01_JPRB, 0.13917E+01_JPRB ,&
&  0.14469E+01_JPRB, 0.15460E+01_JPRB, 0.16511E+01_JPRB, 0.17530E+01_JPRB, 0.18172E+01_JPRB, 0.18415E+01_JPRB, 0.14611E+01_JPRB ,&
&  0.19518E+01_JPRB, 0.19722E+01_JPRB, 0.18693E+01_JPRB, 0.16079E+01_JPRB, 0.14820E+01_JPRB, 0.13991E+01_JPRB, 0.13362E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.12296E+01_JPRB, 0.11616E+01_JPRB, 0.11167E+01_JPRB, 0.11989E+01_JPRB, 0.12139E+01_JPRB, 0.12795E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.14205E+01_JPRB, 0.14881E+01_JPRB, 0.15552E+01_JPRB, 0.16518E+01_JPRB, 0.17618E+01_JPRB, 0.18589E+01_JPRB ,&
&  0.19346E+01_JPRB, 0.19727E+01_JPRB, 0.20551E+01_JPRB, 0.18295E+01_JPRB, 0.17718E+01_JPRB, 0.16947E+01_JPRB, 0.12350E+01_JPRB ,&
&  0.90846E+00_JPRB, 0.87833E+00_JPRB, 0.95335E+00_JPRB, 0.97324E+00_JPRB, 0.91283E+00_JPRB, 0.80923E+00_JPRB, 0.74090E+00_JPRB ,&
&  0.66974E+00_JPRB, 0.65738E+00_JPRB, 0.64746E+00_JPRB, 0.63544E+00_JPRB, 0.62967E+00_JPRB, 0.66148E+00_JPRB, 0.69180E+00_JPRB ,&
&  0.75902E+00_JPRB, 0.81166E+00_JPRB, 0.87676E+00_JPRB, 0.93989E+00_JPRB, 0.10329E+01_JPRB, 0.11505E+01_JPRB, 0.12663E+01_JPRB ,&
&  0.10869E+01_JPRB, 0.98390E+00_JPRB, 0.99545E+00_JPRB, 0.10468E+01_JPRB, 0.10788E+01_JPRB, 0.11656E+01_JPRB, 0.12714E+01_JPRB ,&
&  0.14043E+01_JPRB, 0.15028E+01_JPRB, 0.13696E+01_JPRB, 0.11455E+01_JPRB, 0.14925E+01_JPRB, 0.15327E+01_JPRB, 0.73412E+00_JPRB ,&
&  0.72763E+00_JPRB, 0.76333E+00_JPRB, 0.78532E+00_JPRB, 0.81016E+00_JPRB, 0.83303E+00_JPRB /)
ZBURD(:, 41,  9) = (/ &
&  0.67427E+00_JPRB, 0.68248E+00_JPRB, 0.68380E+00_JPRB, 0.72319E+00_JPRB, 0.73298E+00_JPRB, 0.75165E+00_JPRB, 0.76766E+00_JPRB ,&
&  0.78398E+00_JPRB, 0.81075E+00_JPRB, 0.84178E+00_JPRB, 0.87374E+00_JPRB, 0.90006E+00_JPRB, 0.92037E+00_JPRB, 0.93737E+00_JPRB ,&
&  0.95630E+00_JPRB, 0.97962E+00_JPRB, 0.10099E+01_JPRB, 0.10488E+01_JPRB, 0.11099E+01_JPRB, 0.11801E+01_JPRB, 0.12551E+01_JPRB ,&
&  0.13298E+01_JPRB, 0.14286E+01_JPRB, 0.15458E+01_JPRB, 0.16827E+01_JPRB, 0.18214E+01_JPRB, 0.19184E+01_JPRB, 0.19759E+01_JPRB ,&
&  0.15419E+01_JPRB, 0.18293E+01_JPRB, 0.17826E+01_JPRB, 0.16260E+01_JPRB, 0.15047E+01_JPRB, 0.14424E+01_JPRB, 0.13827E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.12428E+01_JPRB, 0.11943E+01_JPRB, 0.11510E+01_JPRB, 0.11574E+01_JPRB, 0.11596E+01_JPRB, 0.11984E+01_JPRB ,&
&  0.12643E+01_JPRB, 0.13169E+01_JPRB, 0.13821E+01_JPRB, 0.14432E+01_JPRB, 0.15302E+01_JPRB, 0.16678E+01_JPRB, 0.18226E+01_JPRB ,&
&  0.19797E+01_JPRB, 0.20947E+01_JPRB, 0.21834E+01_JPRB, 0.19289E+01_JPRB, 0.17817E+01_JPRB, 0.16621E+01_JPRB, 0.11820E+01_JPRB ,&
&  0.92920E+00_JPRB, 0.86193E+00_JPRB, 0.93322E+00_JPRB, 0.98508E+00_JPRB, 0.90241E+00_JPRB, 0.81322E+00_JPRB, 0.73994E+00_JPRB ,&
&  0.68412E+00_JPRB, 0.64922E+00_JPRB, 0.65030E+00_JPRB, 0.64031E+00_JPRB, 0.64360E+00_JPRB, 0.66568E+00_JPRB, 0.68798E+00_JPRB ,&
&  0.73398E+00_JPRB, 0.75588E+00_JPRB, 0.80542E+00_JPRB, 0.84708E+00_JPRB, 0.89106E+00_JPRB, 0.92748E+00_JPRB, 0.95958E+00_JPRB ,&
&  0.97885E+00_JPRB, 0.98742E+00_JPRB, 0.98826E+00_JPRB, 0.10120E+01_JPRB, 0.10175E+01_JPRB, 0.10312E+01_JPRB, 0.10555E+01_JPRB ,&
&  0.10842E+01_JPRB, 0.11037E+01_JPRB, 0.10795E+01_JPRB, 0.10469E+01_JPRB, 0.98915E+00_JPRB, 0.93553E+00_JPRB, 0.72071E+00_JPRB ,&
&  0.68383E+00_JPRB, 0.64564E+00_JPRB, 0.66417E+00_JPRB, 0.68746E+00_JPRB, 0.67971E+00_JPRB /)
ZBURD(:, 42,  9) = (/ &
&  0.59808E+00_JPRB, 0.58547E+00_JPRB, 0.57151E+00_JPRB, 0.57668E+00_JPRB, 0.58238E+00_JPRB, 0.60545E+00_JPRB, 0.60778E+00_JPRB ,&
&  0.62405E+00_JPRB, 0.67297E+00_JPRB, 0.73326E+00_JPRB, 0.76872E+00_JPRB, 0.79767E+00_JPRB, 0.82546E+00_JPRB, 0.84938E+00_JPRB ,&
&  0.86408E+00_JPRB, 0.87625E+00_JPRB, 0.88892E+00_JPRB, 0.90496E+00_JPRB, 0.93209E+00_JPRB, 0.97422E+00_JPRB, 0.10409E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11879E+01_JPRB, 0.12850E+01_JPRB, 0.14079E+01_JPRB, 0.15402E+01_JPRB, 0.16148E+01_JPRB, 0.16629E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.15225E+01_JPRB, 0.18037E+01_JPRB, 0.16935E+01_JPRB, 0.15536E+01_JPRB, 0.14794E+01_JPRB, 0.14114E+01_JPRB ,&
&  0.13084E+01_JPRB, 0.12049E+01_JPRB, 0.11600E+01_JPRB, 0.12088E+01_JPRB, 0.11230E+01_JPRB, 0.11143E+01_JPRB, 0.11188E+01_JPRB ,&
&  0.11658E+01_JPRB, 0.11984E+01_JPRB, 0.12102E+01_JPRB, 0.12356E+01_JPRB, 0.12841E+01_JPRB, 0.14011E+01_JPRB, 0.15949E+01_JPRB ,&
&  0.18542E+01_JPRB, 0.21139E+01_JPRB, 0.22475E+01_JPRB, 0.19167E+01_JPRB, 0.19519E+01_JPRB, 0.18032E+01_JPRB, 0.15223E+01_JPRB ,&
&  0.10658E+01_JPRB, 0.89924E+00_JPRB, 0.93247E+00_JPRB, 0.96214E+00_JPRB, 0.88641E+00_JPRB, 0.81759E+00_JPRB, 0.75929E+00_JPRB ,&
&  0.69916E+00_JPRB, 0.66564E+00_JPRB, 0.65788E+00_JPRB, 0.65491E+00_JPRB, 0.66329E+00_JPRB, 0.67346E+00_JPRB, 0.69177E+00_JPRB ,&
&  0.70383E+00_JPRB, 0.72526E+00_JPRB, 0.75629E+00_JPRB, 0.79039E+00_JPRB, 0.83207E+00_JPRB, 0.87564E+00_JPRB, 0.92079E+00_JPRB ,&
&  0.95132E+00_JPRB, 0.97799E+00_JPRB, 0.10033E+01_JPRB, 0.10218E+01_JPRB, 0.10310E+01_JPRB, 0.10372E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.10018E+01_JPRB, 0.10121E+01_JPRB, 0.10172E+01_JPRB, 0.10038E+01_JPRB, 0.96032E+00_JPRB, 0.91628E+00_JPRB, 0.87784E+00_JPRB ,&
&  0.81072E+00_JPRB, 0.71673E+00_JPRB, 0.63521E+00_JPRB, 0.61455E+00_JPRB, 0.60260E+00_JPRB /)
ZBURD(:, 43,  9) = (/ &
&  0.56201E+00_JPRB, 0.53682E+00_JPRB, 0.52804E+00_JPRB, 0.52845E+00_JPRB, 0.53898E+00_JPRB, 0.54571E+00_JPRB, 0.54609E+00_JPRB ,&
&  0.54983E+00_JPRB, 0.56309E+00_JPRB, 0.57871E+00_JPRB, 0.62825E+00_JPRB, 0.68188E+00_JPRB, 0.72319E+00_JPRB, 0.74983E+00_JPRB ,&
&  0.77121E+00_JPRB, 0.77854E+00_JPRB, 0.77744E+00_JPRB, 0.78095E+00_JPRB, 0.78538E+00_JPRB, 0.79944E+00_JPRB, 0.83262E+00_JPRB ,&
&  0.89006E+00_JPRB, 0.95464E+00_JPRB, 0.10361E+01_JPRB, 0.11448E+01_JPRB, 0.12875E+01_JPRB, 0.14314E+01_JPRB, 0.15373E+01_JPRB ,&
&  0.11610E+01_JPRB, 0.64837E+00_JPRB, 0.15832E+01_JPRB, 0.18832E+01_JPRB, 0.16714E+01_JPRB, 0.15809E+01_JPRB, 0.14463E+01_JPRB ,&
&  0.12770E+01_JPRB, 0.12059E+01_JPRB, 0.11479E+01_JPRB, 0.11701E+01_JPRB, 0.10961E+01_JPRB, 0.10847E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.11307E+01_JPRB, 0.11305E+01_JPRB, 0.11135E+01_JPRB, 0.11082E+01_JPRB, 0.11457E+01_JPRB, 0.12749E+01_JPRB ,&
&  0.15347E+01_JPRB, 0.19064E+01_JPRB, 0.22460E+01_JPRB, 0.19857E+01_JPRB, 0.20290E+01_JPRB, 0.16686E+01_JPRB, 0.13100E+01_JPRB ,&
&  0.10827E+01_JPRB, 0.95116E+00_JPRB, 0.93708E+00_JPRB, 0.95328E+00_JPRB, 0.89108E+00_JPRB, 0.80641E+00_JPRB, 0.79585E+00_JPRB ,&
&  0.72910E+00_JPRB, 0.68334E+00_JPRB, 0.66139E+00_JPRB, 0.65809E+00_JPRB, 0.66630E+00_JPRB, 0.68050E+00_JPRB, 0.69545E+00_JPRB ,&
&  0.70212E+00_JPRB, 0.71663E+00_JPRB, 0.73502E+00_JPRB, 0.76825E+00_JPRB, 0.80885E+00_JPRB, 0.86356E+00_JPRB, 0.90942E+00_JPRB ,&
&  0.93728E+00_JPRB, 0.96246E+00_JPRB, 0.98655E+00_JPRB, 0.10025E+01_JPRB, 0.10174E+01_JPRB, 0.99352E+00_JPRB, 0.97869E+00_JPRB ,&
&  0.97916E+00_JPRB, 0.98391E+00_JPRB, 0.99493E+00_JPRB, 0.10186E+01_JPRB, 0.97724E+00_JPRB, 0.94248E+00_JPRB, 0.95226E+00_JPRB ,&
&  0.93675E+00_JPRB, 0.86966E+00_JPRB, 0.74836E+00_JPRB, 0.63106E+00_JPRB, 0.58792E+00_JPRB /)
ZBURD(:, 44,  9) = (/ &
&  0.60475E+00_JPRB, 0.57841E+00_JPRB, 0.54995E+00_JPRB, 0.54133E+00_JPRB, 0.54963E+00_JPRB, 0.54456E+00_JPRB, 0.54139E+00_JPRB ,&
&  0.53731E+00_JPRB, 0.53405E+00_JPRB, 0.52048E+00_JPRB, 0.52930E+00_JPRB, 0.56028E+00_JPRB, 0.60759E+00_JPRB, 0.64696E+00_JPRB ,&
&  0.67709E+00_JPRB, 0.68885E+00_JPRB, 0.68982E+00_JPRB, 0.68977E+00_JPRB, 0.68991E+00_JPRB, 0.69130E+00_JPRB, 0.69679E+00_JPRB ,&
&  0.71856E+00_JPRB, 0.75559E+00_JPRB, 0.80503E+00_JPRB, 0.86295E+00_JPRB, 0.94387E+00_JPRB, 0.10819E+01_JPRB, 0.13045E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.12220E+01_JPRB, 0.80511E+00_JPRB, 0.16378E+01_JPRB, 0.18264E+01_JPRB, 0.17894E+01_JPRB, 0.15216E+01_JPRB ,&
&  0.14117E+01_JPRB, 0.12669E+01_JPRB, 0.11628E+01_JPRB, 0.11785E+01_JPRB, 0.10788E+01_JPRB, 0.10573E+01_JPRB, 0.10679E+01_JPRB ,&
&  0.10743E+01_JPRB, 0.10796E+01_JPRB, 0.10813E+01_JPRB, 0.10618E+01_JPRB, 0.10276E+01_JPRB, 0.10083E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.12078E+01_JPRB, 0.15344E+01_JPRB, 0.20502E+01_JPRB, 0.20259E+01_JPRB, 0.20603E+01_JPRB, 0.15555E+01_JPRB, 0.12445E+01_JPRB ,&
&  0.10889E+01_JPRB, 0.10723E+01_JPRB, 0.10472E+01_JPRB, 0.97651E+00_JPRB, 0.88324E+00_JPRB, 0.82021E+00_JPRB, 0.83150E+00_JPRB ,&
&  0.77365E+00_JPRB, 0.73004E+00_JPRB, 0.69460E+00_JPRB, 0.67746E+00_JPRB, 0.67668E+00_JPRB, 0.68440E+00_JPRB, 0.69127E+00_JPRB ,&
&  0.70714E+00_JPRB, 0.71791E+00_JPRB, 0.73901E+00_JPRB, 0.76359E+00_JPRB, 0.79155E+00_JPRB, 0.82855E+00_JPRB, 0.87690E+00_JPRB ,&
&  0.91683E+00_JPRB, 0.94697E+00_JPRB, 0.96552E+00_JPRB, 0.97406E+00_JPRB, 0.95814E+00_JPRB, 0.94057E+00_JPRB, 0.96123E+00_JPRB ,&
&  0.95492E+00_JPRB, 0.95693E+00_JPRB, 0.93016E+00_JPRB, 0.95505E+00_JPRB, 0.95869E+00_JPRB, 0.95690E+00_JPRB, 0.97454E+00_JPRB ,&
&  0.10037E+01_JPRB, 0.98442E+00_JPRB, 0.88248E+00_JPRB, 0.68388E+00_JPRB, 0.63154E+00_JPRB /)
ZBURD(:, 45,  9) = (/ &
&  0.66848E+00_JPRB, 0.64928E+00_JPRB, 0.63367E+00_JPRB, 0.63034E+00_JPRB, 0.62717E+00_JPRB, 0.61112E+00_JPRB, 0.59062E+00_JPRB ,&
&  0.56483E+00_JPRB, 0.54564E+00_JPRB, 0.51919E+00_JPRB, 0.49457E+00_JPRB, 0.49082E+00_JPRB, 0.50562E+00_JPRB, 0.53578E+00_JPRB ,&
&  0.56487E+00_JPRB, 0.58010E+00_JPRB, 0.59125E+00_JPRB, 0.60145E+00_JPRB, 0.61073E+00_JPRB, 0.61503E+00_JPRB, 0.61443E+00_JPRB ,&
&  0.61940E+00_JPRB, 0.63276E+00_JPRB, 0.65361E+00_JPRB, 0.67832E+00_JPRB, 0.71362E+00_JPRB, 0.77503E+00_JPRB, 0.89368E+00_JPRB ,&
&  0.11456E+01_JPRB, 0.12843E+01_JPRB, 0.68498E+00_JPRB, 0.13062E+01_JPRB, 0.18328E+01_JPRB, 0.18233E+01_JPRB, 0.17199E+01_JPRB ,&
&  0.15798E+01_JPRB, 0.13520E+01_JPRB, 0.12827E+01_JPRB, 0.11919E+01_JPRB, 0.10817E+01_JPRB, 0.10390E+01_JPRB, 0.10250E+01_JPRB ,&
&  0.10205E+01_JPRB, 0.99475E+00_JPRB, 0.97513E+00_JPRB, 0.96766E+00_JPRB, 0.94211E+00_JPRB, 0.91249E+00_JPRB, 0.91267E+00_JPRB ,&
&  0.98580E+00_JPRB, 0.12144E+01_JPRB, 0.17060E+01_JPRB, 0.19541E+01_JPRB, 0.20321E+01_JPRB, 0.16667E+01_JPRB, 0.13404E+01_JPRB ,&
&  0.12150E+01_JPRB, 0.12533E+01_JPRB, 0.11919E+01_JPRB, 0.10448E+01_JPRB, 0.91212E+00_JPRB, 0.86351E+00_JPRB, 0.86119E+00_JPRB ,&
&  0.81247E+00_JPRB, 0.77118E+00_JPRB, 0.74436E+00_JPRB, 0.72460E+00_JPRB, 0.71324E+00_JPRB, 0.70737E+00_JPRB, 0.70812E+00_JPRB ,&
&  0.72200E+00_JPRB, 0.73154E+00_JPRB, 0.74142E+00_JPRB, 0.75417E+00_JPRB, 0.76719E+00_JPRB, 0.78732E+00_JPRB, 0.82282E+00_JPRB ,&
&  0.86967E+00_JPRB, 0.90666E+00_JPRB, 0.93913E+00_JPRB, 0.93299E+00_JPRB, 0.92771E+00_JPRB, 0.93237E+00_JPRB, 0.95807E+00_JPRB ,&
&  0.98003E+00_JPRB, 0.10299E+01_JPRB, 0.91991E+00_JPRB, 0.91494E+00_JPRB, 0.94751E+00_JPRB, 0.93929E+00_JPRB, 0.95728E+00_JPRB ,&
&  0.99429E+00_JPRB, 0.10503E+01_JPRB, 0.10400E+01_JPRB, 0.75753E+00_JPRB, 0.70568E+00_JPRB /)
ZBURD(:, 46,  9) = (/ &
&  0.71666E+00_JPRB, 0.69146E+00_JPRB, 0.68443E+00_JPRB, 0.68348E+00_JPRB, 0.68030E+00_JPRB, 0.67020E+00_JPRB, 0.65098E+00_JPRB ,&
&  0.62711E+00_JPRB, 0.60944E+00_JPRB, 0.58921E+00_JPRB, 0.56290E+00_JPRB, 0.52518E+00_JPRB, 0.49171E+00_JPRB, 0.48926E+00_JPRB ,&
&  0.50875E+00_JPRB, 0.51770E+00_JPRB, 0.52568E+00_JPRB, 0.53544E+00_JPRB, 0.54614E+00_JPRB, 0.55843E+00_JPRB, 0.56648E+00_JPRB ,&
&  0.56945E+00_JPRB, 0.57502E+00_JPRB, 0.58512E+00_JPRB, 0.59594E+00_JPRB, 0.61202E+00_JPRB, 0.64343E+00_JPRB, 0.70116E+00_JPRB ,&
&  0.86323E+00_JPRB, 0.10965E+01_JPRB, 0.11559E+01_JPRB, 0.18820E+01_JPRB, 0.21586E+01_JPRB, 0.20306E+01_JPRB, 0.20063E+01_JPRB ,&
&  0.19408E+01_JPRB, 0.17231E+01_JPRB, 0.15263E+01_JPRB, 0.12574E+01_JPRB, 0.11110E+01_JPRB, 0.10412E+01_JPRB, 0.99999E+00_JPRB ,&
&  0.97096E+00_JPRB, 0.93460E+00_JPRB, 0.88466E+00_JPRB, 0.85757E+00_JPRB, 0.83640E+00_JPRB, 0.81944E+00_JPRB, 0.82040E+00_JPRB ,&
&  0.85367E+00_JPRB, 0.99072E+00_JPRB, 0.13497E+01_JPRB, 0.18767E+01_JPRB, 0.20165E+01_JPRB, 0.18491E+01_JPRB, 0.15871E+01_JPRB ,&
&  0.15959E+01_JPRB, 0.15459E+01_JPRB, 0.14247E+01_JPRB, 0.11952E+01_JPRB, 0.10070E+01_JPRB, 0.99422E+00_JPRB, 0.89480E+00_JPRB ,&
&  0.83518E+00_JPRB, 0.79645E+00_JPRB, 0.77564E+00_JPRB, 0.76115E+00_JPRB, 0.75207E+00_JPRB, 0.74184E+00_JPRB, 0.72977E+00_JPRB ,&
&  0.72400E+00_JPRB, 0.72643E+00_JPRB, 0.72281E+00_JPRB, 0.72460E+00_JPRB, 0.72816E+00_JPRB, 0.73726E+00_JPRB, 0.75601E+00_JPRB ,&
&  0.79768E+00_JPRB, 0.84135E+00_JPRB, 0.84719E+00_JPRB, 0.85868E+00_JPRB, 0.86379E+00_JPRB, 0.86577E+00_JPRB, 0.86920E+00_JPRB ,&
&  0.88747E+00_JPRB, 0.93369E+00_JPRB, 0.94032E+00_JPRB, 0.94138E+00_JPRB, 0.94598E+00_JPRB, 0.94226E+00_JPRB, 0.91858E+00_JPRB ,&
&  0.91261E+00_JPRB, 0.88602E+00_JPRB, 0.85605E+00_JPRB, 0.78485E+00_JPRB, 0.75248E+00_JPRB /)
ZBURD(:, 47,  9) = (/ &
&  0.73742E+00_JPRB, 0.71187E+00_JPRB, 0.69541E+00_JPRB, 0.68764E+00_JPRB, 0.68205E+00_JPRB, 0.67567E+00_JPRB, 0.66012E+00_JPRB ,&
&  0.64358E+00_JPRB, 0.63325E+00_JPRB, 0.61714E+00_JPRB, 0.60032E+00_JPRB, 0.56752E+00_JPRB, 0.52702E+00_JPRB, 0.50952E+00_JPRB ,&
&  0.49988E+00_JPRB, 0.49960E+00_JPRB, 0.50941E+00_JPRB, 0.51485E+00_JPRB, 0.51925E+00_JPRB, 0.52788E+00_JPRB, 0.53516E+00_JPRB ,&
&  0.54192E+00_JPRB, 0.54411E+00_JPRB, 0.55064E+00_JPRB, 0.55722E+00_JPRB, 0.56325E+00_JPRB, 0.58417E+00_JPRB, 0.62001E+00_JPRB ,&
&  0.72810E+00_JPRB, 0.10319E+01_JPRB, 0.91721E+00_JPRB, 0.18302E+01_JPRB, 0.20753E+01_JPRB, 0.20682E+01_JPRB, 0.19250E+01_JPRB ,&
&  0.19139E+01_JPRB, 0.19072E+01_JPRB, 0.15279E+01_JPRB, 0.12843E+01_JPRB, 0.11184E+01_JPRB, 0.10327E+01_JPRB, 0.98150E+00_JPRB ,&
&  0.94265E+00_JPRB, 0.90062E+00_JPRB, 0.84325E+00_JPRB, 0.79294E+00_JPRB, 0.76473E+00_JPRB, 0.74732E+00_JPRB, 0.74660E+00_JPRB ,&
&  0.76019E+00_JPRB, 0.83658E+00_JPRB, 0.10781E+01_JPRB, 0.15579E+01_JPRB, 0.19592E+01_JPRB, 0.20107E+01_JPRB, 0.18307E+01_JPRB ,&
&  0.18412E+01_JPRB, 0.19061E+01_JPRB, 0.17543E+01_JPRB, 0.14288E+01_JPRB, 0.11957E+01_JPRB, 0.10802E+01_JPRB, 0.96064E+00_JPRB ,&
&  0.88354E+00_JPRB, 0.83484E+00_JPRB, 0.80842E+00_JPRB, 0.78683E+00_JPRB, 0.76817E+00_JPRB, 0.75206E+00_JPRB, 0.73369E+00_JPRB ,&
&  0.71727E+00_JPRB, 0.70211E+00_JPRB, 0.69717E+00_JPRB, 0.69368E+00_JPRB, 0.69921E+00_JPRB, 0.70534E+00_JPRB, 0.71282E+00_JPRB ,&
&  0.73789E+00_JPRB, 0.78046E+00_JPRB, 0.77953E+00_JPRB, 0.79693E+00_JPRB, 0.80596E+00_JPRB, 0.80980E+00_JPRB, 0.83319E+00_JPRB ,&
&  0.88204E+00_JPRB, 0.89672E+00_JPRB, 0.91641E+00_JPRB, 0.91632E+00_JPRB, 0.95078E+00_JPRB, 0.98544E+00_JPRB, 0.92388E+00_JPRB ,&
&  0.90160E+00_JPRB, 0.86301E+00_JPRB, 0.82625E+00_JPRB, 0.78893E+00_JPRB, 0.76266E+00_JPRB /)
ZBURD(:, 48,  9) = (/ &
&  0.73543E+00_JPRB, 0.71519E+00_JPRB, 0.69283E+00_JPRB, 0.67192E+00_JPRB, 0.66463E+00_JPRB, 0.66037E+00_JPRB, 0.64699E+00_JPRB ,&
&  0.63460E+00_JPRB, 0.62381E+00_JPRB, 0.60847E+00_JPRB, 0.58952E+00_JPRB, 0.56622E+00_JPRB, 0.53776E+00_JPRB, 0.51184E+00_JPRB ,&
&  0.49974E+00_JPRB, 0.48982E+00_JPRB, 0.49228E+00_JPRB, 0.49880E+00_JPRB, 0.50803E+00_JPRB, 0.51465E+00_JPRB, 0.52172E+00_JPRB ,&
&  0.52761E+00_JPRB, 0.53084E+00_JPRB, 0.53518E+00_JPRB, 0.53911E+00_JPRB, 0.54095E+00_JPRB, 0.55126E+00_JPRB, 0.59075E+00_JPRB ,&
&  0.67619E+00_JPRB, 0.11312E+01_JPRB, 0.91692E+00_JPRB, 0.15153E+01_JPRB, 0.17371E+01_JPRB, 0.18411E+01_JPRB, 0.17324E+01_JPRB ,&
&  0.17336E+01_JPRB, 0.16470E+01_JPRB, 0.14124E+01_JPRB, 0.12354E+01_JPRB, 0.11038E+01_JPRB, 0.10186E+01_JPRB, 0.95872E+00_JPRB ,&
&  0.91596E+00_JPRB, 0.86622E+00_JPRB, 0.80914E+00_JPRB, 0.75400E+00_JPRB, 0.71977E+00_JPRB, 0.70090E+00_JPRB, 0.69463E+00_JPRB ,&
&  0.69728E+00_JPRB, 0.73783E+00_JPRB, 0.88061E+00_JPRB, 0.12977E+01_JPRB, 0.17234E+01_JPRB, 0.19938E+01_JPRB, 0.20426E+01_JPRB ,&
&  0.20819E+01_JPRB, 0.22365E+01_JPRB, 0.20653E+01_JPRB, 0.17002E+01_JPRB, 0.14165E+01_JPRB, 0.12052E+01_JPRB, 0.10487E+01_JPRB ,&
&  0.95277E+00_JPRB, 0.89568E+00_JPRB, 0.85922E+00_JPRB, 0.82768E+00_JPRB, 0.79325E+00_JPRB, 0.75563E+00_JPRB, 0.71977E+00_JPRB ,&
&  0.69346E+00_JPRB, 0.67541E+00_JPRB, 0.66478E+00_JPRB, 0.66731E+00_JPRB, 0.67071E+00_JPRB, 0.67599E+00_JPRB, 0.68683E+00_JPRB ,&
&  0.70468E+00_JPRB, 0.73725E+00_JPRB, 0.74611E+00_JPRB, 0.76336E+00_JPRB, 0.77703E+00_JPRB, 0.79578E+00_JPRB, 0.81655E+00_JPRB ,&
&  0.87716E+00_JPRB, 0.88928E+00_JPRB, 0.88551E+00_JPRB, 0.91678E+00_JPRB, 0.99163E+00_JPRB, 0.10811E+01_JPRB, 0.96944E+00_JPRB ,&
&  0.92583E+00_JPRB, 0.86557E+00_JPRB, 0.81484E+00_JPRB, 0.78330E+00_JPRB, 0.76040E+00_JPRB /)
ZBURD(:, 49,  9) = (/ &
&  0.73051E+00_JPRB, 0.70493E+00_JPRB, 0.68827E+00_JPRB, 0.67043E+00_JPRB, 0.65846E+00_JPRB, 0.65009E+00_JPRB, 0.63638E+00_JPRB ,&
&  0.62078E+00_JPRB, 0.60463E+00_JPRB, 0.58677E+00_JPRB, 0.57025E+00_JPRB, 0.55670E+00_JPRB, 0.54050E+00_JPRB, 0.52713E+00_JPRB ,&
&  0.51483E+00_JPRB, 0.50040E+00_JPRB, 0.49116E+00_JPRB, 0.48812E+00_JPRB, 0.49176E+00_JPRB, 0.49680E+00_JPRB, 0.50366E+00_JPRB ,&
&  0.51188E+00_JPRB, 0.51733E+00_JPRB, 0.51830E+00_JPRB, 0.52080E+00_JPRB, 0.51957E+00_JPRB, 0.52813E+00_JPRB, 0.57023E+00_JPRB ,&
&  0.67054E+00_JPRB, 0.63523E+00_JPRB, 0.10242E+01_JPRB, 0.12203E+01_JPRB, 0.13747E+01_JPRB, 0.15226E+01_JPRB, 0.16048E+01_JPRB ,&
&  0.16441E+01_JPRB, 0.14801E+01_JPRB, 0.12807E+01_JPRB, 0.11495E+01_JPRB, 0.10453E+01_JPRB, 0.96802E+00_JPRB, 0.90731E+00_JPRB ,&
&  0.84769E+00_JPRB, 0.80252E+00_JPRB, 0.76624E+00_JPRB, 0.71926E+00_JPRB, 0.68529E+00_JPRB, 0.66850E+00_JPRB, 0.66123E+00_JPRB ,&
&  0.65545E+00_JPRB, 0.67023E+00_JPRB, 0.74521E+00_JPRB, 0.97945E+00_JPRB, 0.14942E+01_JPRB, 0.18969E+01_JPRB, 0.21886E+01_JPRB ,&
&  0.25890E+01_JPRB, 0.26497E+01_JPRB, 0.23767E+01_JPRB, 0.19681E+01_JPRB, 0.15803E+01_JPRB, 0.13173E+01_JPRB, 0.11077E+01_JPRB ,&
&  0.98857E+00_JPRB, 0.92239E+00_JPRB, 0.88370E+00_JPRB, 0.85144E+00_JPRB, 0.81572E+00_JPRB, 0.76797E+00_JPRB, 0.71480E+00_JPRB ,&
&  0.67384E+00_JPRB, 0.65091E+00_JPRB, 0.63666E+00_JPRB, 0.63072E+00_JPRB, 0.62975E+00_JPRB, 0.63296E+00_JPRB, 0.64298E+00_JPRB ,&
&  0.66355E+00_JPRB, 0.70002E+00_JPRB, 0.72815E+00_JPRB, 0.73391E+00_JPRB, 0.75746E+00_JPRB, 0.79053E+00_JPRB, 0.83034E+00_JPRB ,&
&  0.86279E+00_JPRB, 0.85356E+00_JPRB, 0.83196E+00_JPRB, 0.88477E+00_JPRB, 0.98496E+00_JPRB, 0.10870E+01_JPRB, 0.10178E+01_JPRB ,&
&  0.96559E+00_JPRB, 0.89523E+00_JPRB, 0.82179E+00_JPRB, 0.78405E+00_JPRB, 0.76373E+00_JPRB /)
ZBURD(:, 50,  9) = (/ &
&  0.72229E+00_JPRB, 0.69509E+00_JPRB, 0.68637E+00_JPRB, 0.68108E+00_JPRB, 0.67507E+00_JPRB, 0.66041E+00_JPRB, 0.63437E+00_JPRB ,&
&  0.61329E+00_JPRB, 0.59589E+00_JPRB, 0.57620E+00_JPRB, 0.55915E+00_JPRB, 0.54766E+00_JPRB, 0.53915E+00_JPRB, 0.52900E+00_JPRB ,&
&  0.52170E+00_JPRB, 0.51560E+00_JPRB, 0.50244E+00_JPRB, 0.49003E+00_JPRB, 0.48443E+00_JPRB, 0.48325E+00_JPRB, 0.48691E+00_JPRB ,&
&  0.49119E+00_JPRB, 0.49635E+00_JPRB, 0.49997E+00_JPRB, 0.50293E+00_JPRB, 0.50307E+00_JPRB, 0.51196E+00_JPRB, 0.54824E+00_JPRB ,&
&  0.66296E+00_JPRB, 0.88514E+00_JPRB, 0.10254E+01_JPRB, 0.10363E+01_JPRB, 0.11253E+01_JPRB, 0.12871E+01_JPRB, 0.14615E+01_JPRB ,&
&  0.14529E+01_JPRB, 0.13272E+01_JPRB, 0.11540E+01_JPRB, 0.10354E+01_JPRB, 0.94576E+00_JPRB, 0.88241E+00_JPRB, 0.81908E+00_JPRB ,&
&  0.76687E+00_JPRB, 0.73162E+00_JPRB, 0.69667E+00_JPRB, 0.66536E+00_JPRB, 0.65215E+00_JPRB, 0.64801E+00_JPRB, 0.64493E+00_JPRB ,&
&  0.64087E+00_JPRB, 0.63903E+00_JPRB, 0.67139E+00_JPRB, 0.83536E+00_JPRB, 0.12987E+01_JPRB, 0.16093E+01_JPRB, 0.21931E+01_JPRB ,&
&  0.26862E+01_JPRB, 0.25944E+01_JPRB, 0.23332E+01_JPRB, 0.19859E+01_JPRB, 0.16178E+01_JPRB, 0.13111E+01_JPRB, 0.11218E+01_JPRB ,&
&  0.10031E+01_JPRB, 0.90685E+00_JPRB, 0.84696E+00_JPRB, 0.80836E+00_JPRB, 0.77713E+00_JPRB, 0.74828E+00_JPRB, 0.71333E+00_JPRB ,&
&  0.67617E+00_JPRB, 0.64607E+00_JPRB, 0.62588E+00_JPRB, 0.61086E+00_JPRB, 0.60253E+00_JPRB, 0.60050E+00_JPRB, 0.60364E+00_JPRB ,&
&  0.61329E+00_JPRB, 0.64112E+00_JPRB, 0.70328E+00_JPRB, 0.69474E+00_JPRB, 0.71590E+00_JPRB, 0.74965E+00_JPRB, 0.78196E+00_JPRB ,&
&  0.78127E+00_JPRB, 0.80652E+00_JPRB, 0.78435E+00_JPRB, 0.83026E+00_JPRB, 0.10968E+01_JPRB, 0.12030E+01_JPRB, 0.10880E+01_JPRB ,&
&  0.10029E+01_JPRB, 0.92304E+00_JPRB, 0.85118E+00_JPRB, 0.79059E+00_JPRB, 0.76589E+00_JPRB /)
ZBURD(:, 51,  9) = (/ &
&  0.73347E+00_JPRB, 0.69205E+00_JPRB, 0.67133E+00_JPRB, 0.66992E+00_JPRB, 0.67365E+00_JPRB, 0.66306E+00_JPRB, 0.63801E+00_JPRB ,&
&  0.60775E+00_JPRB, 0.58222E+00_JPRB, 0.56416E+00_JPRB, 0.55080E+00_JPRB, 0.54031E+00_JPRB, 0.53114E+00_JPRB, 0.52100E+00_JPRB ,&
&  0.51134E+00_JPRB, 0.50598E+00_JPRB, 0.50201E+00_JPRB, 0.49637E+00_JPRB, 0.48525E+00_JPRB, 0.47652E+00_JPRB, 0.47353E+00_JPRB ,&
&  0.47176E+00_JPRB, 0.47227E+00_JPRB, 0.47760E+00_JPRB, 0.48393E+00_JPRB, 0.48677E+00_JPRB, 0.49201E+00_JPRB, 0.51600E+00_JPRB ,&
&  0.60051E+00_JPRB, 0.78940E+00_JPRB, 0.79071E+00_JPRB, 0.85850E+00_JPRB, 0.98524E+00_JPRB, 0.11268E+01_JPRB, 0.12510E+01_JPRB ,&
&  0.12563E+01_JPRB, 0.11910E+01_JPRB, 0.10692E+01_JPRB, 0.94697E+00_JPRB, 0.84911E+00_JPRB, 0.79144E+00_JPRB, 0.75041E+00_JPRB ,&
&  0.71293E+00_JPRB, 0.67541E+00_JPRB, 0.64293E+00_JPRB, 0.61866E+00_JPRB, 0.60219E+00_JPRB, 0.59543E+00_JPRB, 0.59539E+00_JPRB ,&
&  0.59640E+00_JPRB, 0.59586E+00_JPRB, 0.60198E+00_JPRB, 0.66136E+00_JPRB, 0.93121E+00_JPRB, 0.12717E+01_JPRB, 0.16480E+01_JPRB ,&
&  0.17415E+01_JPRB, 0.17785E+01_JPRB, 0.17425E+01_JPRB, 0.16389E+01_JPRB, 0.14711E+01_JPRB, 0.13069E+01_JPRB, 0.11314E+01_JPRB ,&
&  0.10034E+01_JPRB, 0.91345E+00_JPRB, 0.84810E+00_JPRB, 0.79362E+00_JPRB, 0.74466E+00_JPRB, 0.70571E+00_JPRB, 0.67892E+00_JPRB ,&
&  0.65898E+00_JPRB, 0.64136E+00_JPRB, 0.62465E+00_JPRB, 0.60899E+00_JPRB, 0.59695E+00_JPRB, 0.58986E+00_JPRB, 0.59067E+00_JPRB ,&
&  0.59158E+00_JPRB, 0.59089E+00_JPRB, 0.60999E+00_JPRB, 0.62797E+00_JPRB, 0.64739E+00_JPRB, 0.66093E+00_JPRB, 0.67197E+00_JPRB ,&
&  0.68794E+00_JPRB, 0.75979E+00_JPRB, 0.75640E+00_JPRB, 0.82747E+00_JPRB, 0.10422E+01_JPRB, 0.11331E+01_JPRB, 0.11170E+01_JPRB ,&
&  0.10561E+01_JPRB, 0.96983E+00_JPRB, 0.88951E+00_JPRB, 0.83353E+00_JPRB, 0.79536E+00_JPRB /)
ZBURD(:, 52,  9) = (/ &
&  0.88641E+00_JPRB, 0.77147E+00_JPRB, 0.69835E+00_JPRB, 0.66456E+00_JPRB, 0.65447E+00_JPRB, 0.63538E+00_JPRB, 0.61263E+00_JPRB ,&
&  0.59077E+00_JPRB, 0.57485E+00_JPRB, 0.56486E+00_JPRB, 0.55281E+00_JPRB, 0.53926E+00_JPRB, 0.52728E+00_JPRB, 0.51473E+00_JPRB ,&
&  0.50191E+00_JPRB, 0.49389E+00_JPRB, 0.48772E+00_JPRB, 0.48303E+00_JPRB, 0.47917E+00_JPRB, 0.47227E+00_JPRB, 0.46497E+00_JPRB ,&
&  0.45953E+00_JPRB, 0.45287E+00_JPRB, 0.45143E+00_JPRB, 0.45809E+00_JPRB, 0.46448E+00_JPRB, 0.46999E+00_JPRB, 0.48994E+00_JPRB ,&
&  0.55866E+00_JPRB, 0.62261E+00_JPRB, 0.69005E+00_JPRB, 0.75040E+00_JPRB, 0.81159E+00_JPRB, 0.89098E+00_JPRB, 0.92423E+00_JPRB ,&
&  0.90178E+00_JPRB, 0.90943E+00_JPRB, 0.89069E+00_JPRB, 0.82640E+00_JPRB, 0.75478E+00_JPRB, 0.70703E+00_JPRB, 0.67342E+00_JPRB ,&
&  0.64545E+00_JPRB, 0.61540E+00_JPRB, 0.58479E+00_JPRB, 0.56058E+00_JPRB, 0.54647E+00_JPRB, 0.54151E+00_JPRB, 0.54131E+00_JPRB ,&
&  0.54320E+00_JPRB, 0.54791E+00_JPRB, 0.55242E+00_JPRB, 0.56352E+00_JPRB, 0.62621E+00_JPRB, 0.78359E+00_JPRB, 0.10297E+01_JPRB ,&
&  0.11381E+01_JPRB, 0.11641E+01_JPRB, 0.11814E+01_JPRB, 0.11768E+01_JPRB, 0.11116E+01_JPRB, 0.10412E+01_JPRB, 0.96778E+00_JPRB ,&
&  0.89475E+00_JPRB, 0.81769E+00_JPRB, 0.75287E+00_JPRB, 0.70199E+00_JPRB, 0.66205E+00_JPRB, 0.63566E+00_JPRB, 0.62243E+00_JPRB ,&
&  0.61687E+00_JPRB, 0.61354E+00_JPRB, 0.61045E+00_JPRB, 0.60361E+00_JPRB, 0.59401E+00_JPRB, 0.58776E+00_JPRB, 0.58376E+00_JPRB ,&
&  0.57999E+00_JPRB, 0.57799E+00_JPRB, 0.58158E+00_JPRB, 0.58653E+00_JPRB, 0.59546E+00_JPRB, 0.60240E+00_JPRB, 0.61216E+00_JPRB ,&
&  0.62461E+00_JPRB, 0.65721E+00_JPRB, 0.67694E+00_JPRB, 0.81857E+00_JPRB, 0.94040E+00_JPRB, 0.97263E+00_JPRB, 0.10174E+01_JPRB ,&
&  0.10603E+01_JPRB, 0.10315E+01_JPRB, 0.94761E+00_JPRB, 0.90355E+00_JPRB, 0.97862E+00_JPRB /)
ZBURD(:, 53,  9) = (/ &
&  0.83387E+00_JPRB, 0.77742E+00_JPRB, 0.70803E+00_JPRB, 0.66188E+00_JPRB, 0.63039E+00_JPRB, 0.59685E+00_JPRB, 0.57155E+00_JPRB ,&
&  0.55613E+00_JPRB, 0.54698E+00_JPRB, 0.53819E+00_JPRB, 0.52698E+00_JPRB, 0.51548E+00_JPRB, 0.50647E+00_JPRB, 0.49725E+00_JPRB ,&
&  0.48793E+00_JPRB, 0.47908E+00_JPRB, 0.47096E+00_JPRB, 0.46473E+00_JPRB, 0.45707E+00_JPRB, 0.45059E+00_JPRB, 0.44465E+00_JPRB ,&
&  0.43802E+00_JPRB, 0.43195E+00_JPRB, 0.42520E+00_JPRB, 0.42916E+00_JPRB, 0.43848E+00_JPRB, 0.44773E+00_JPRB, 0.45975E+00_JPRB ,&
&  0.50396E+00_JPRB, 0.52367E+00_JPRB, 0.58146E+00_JPRB, 0.64564E+00_JPRB, 0.69172E+00_JPRB, 0.72906E+00_JPRB, 0.73621E+00_JPRB ,&
&  0.71545E+00_JPRB, 0.72596E+00_JPRB, 0.71498E+00_JPRB, 0.68675E+00_JPRB, 0.66490E+00_JPRB, 0.62136E+00_JPRB, 0.58250E+00_JPRB ,&
&  0.55538E+00_JPRB, 0.53811E+00_JPRB, 0.52316E+00_JPRB, 0.50729E+00_JPRB, 0.49787E+00_JPRB, 0.49614E+00_JPRB, 0.49948E+00_JPRB ,&
&  0.50293E+00_JPRB, 0.50862E+00_JPRB, 0.51463E+00_JPRB, 0.52184E+00_JPRB, 0.53343E+00_JPRB, 0.58223E+00_JPRB, 0.70006E+00_JPRB ,&
&  0.78648E+00_JPRB, 0.83463E+00_JPRB, 0.86142E+00_JPRB, 0.86956E+00_JPRB, 0.84537E+00_JPRB, 0.80679E+00_JPRB, 0.76240E+00_JPRB ,&
&  0.71667E+00_JPRB, 0.66821E+00_JPRB, 0.62359E+00_JPRB, 0.58875E+00_JPRB, 0.56128E+00_JPRB, 0.54277E+00_JPRB, 0.53298E+00_JPRB ,&
&  0.53161E+00_JPRB, 0.53661E+00_JPRB, 0.54482E+00_JPRB, 0.55303E+00_JPRB, 0.55794E+00_JPRB, 0.55831E+00_JPRB, 0.55631E+00_JPRB ,&
&  0.55310E+00_JPRB, 0.54927E+00_JPRB, 0.54761E+00_JPRB, 0.54871E+00_JPRB, 0.55220E+00_JPRB, 0.55710E+00_JPRB, 0.56355E+00_JPRB ,&
&  0.57110E+00_JPRB, 0.58650E+00_JPRB, 0.60703E+00_JPRB, 0.70222E+00_JPRB, 0.74391E+00_JPRB, 0.78550E+00_JPRB, 0.82507E+00_JPRB ,&
&  0.87140E+00_JPRB, 0.89116E+00_JPRB, 0.86993E+00_JPRB, 0.82656E+00_JPRB, 0.81327E+00_JPRB /)
ZBURD(:, 54,  9) = (/ &
&  0.72811E+00_JPRB, 0.70734E+00_JPRB, 0.67132E+00_JPRB, 0.63293E+00_JPRB, 0.60028E+00_JPRB, 0.57132E+00_JPRB, 0.54899E+00_JPRB ,&
&  0.53503E+00_JPRB, 0.52391E+00_JPRB, 0.51255E+00_JPRB, 0.50165E+00_JPRB, 0.48939E+00_JPRB, 0.47795E+00_JPRB, 0.46642E+00_JPRB ,&
&  0.45823E+00_JPRB, 0.45280E+00_JPRB, 0.44806E+00_JPRB, 0.44258E+00_JPRB, 0.43674E+00_JPRB, 0.43063E+00_JPRB, 0.42311E+00_JPRB ,&
&  0.41643E+00_JPRB, 0.41277E+00_JPRB, 0.40699E+00_JPRB, 0.40554E+00_JPRB, 0.41252E+00_JPRB, 0.42198E+00_JPRB, 0.43211E+00_JPRB ,&
&  0.46504E+00_JPRB, 0.43996E+00_JPRB, 0.45329E+00_JPRB, 0.51847E+00_JPRB, 0.56690E+00_JPRB, 0.62316E+00_JPRB, 0.65287E+00_JPRB ,&
&  0.66170E+00_JPRB, 0.67079E+00_JPRB, 0.65597E+00_JPRB, 0.63300E+00_JPRB, 0.59591E+00_JPRB, 0.56384E+00_JPRB, 0.53953E+00_JPRB ,&
&  0.51837E+00_JPRB, 0.49975E+00_JPRB, 0.48497E+00_JPRB, 0.47319E+00_JPRB, 0.46346E+00_JPRB, 0.45956E+00_JPRB, 0.46093E+00_JPRB ,&
&  0.46330E+00_JPRB, 0.46633E+00_JPRB, 0.47099E+00_JPRB, 0.47900E+00_JPRB, 0.48816E+00_JPRB, 0.50155E+00_JPRB, 0.54308E+00_JPRB ,&
&  0.60774E+00_JPRB, 0.65243E+00_JPRB, 0.68809E+00_JPRB, 0.70426E+00_JPRB, 0.69607E+00_JPRB, 0.66928E+00_JPRB, 0.62653E+00_JPRB ,&
&  0.58264E+00_JPRB, 0.54861E+00_JPRB, 0.52699E+00_JPRB, 0.51326E+00_JPRB, 0.50512E+00_JPRB, 0.49831E+00_JPRB, 0.49246E+00_JPRB ,&
&  0.48692E+00_JPRB, 0.48307E+00_JPRB, 0.48303E+00_JPRB, 0.48702E+00_JPRB, 0.49321E+00_JPRB, 0.49865E+00_JPRB, 0.50128E+00_JPRB ,&
&  0.50173E+00_JPRB, 0.50135E+00_JPRB, 0.50318E+00_JPRB, 0.50621E+00_JPRB, 0.50962E+00_JPRB, 0.51141E+00_JPRB, 0.51568E+00_JPRB ,&
&  0.52249E+00_JPRB, 0.53442E+00_JPRB, 0.54986E+00_JPRB, 0.66563E+00_JPRB, 0.70618E+00_JPRB, 0.71739E+00_JPRB, 0.72306E+00_JPRB ,&
&  0.73858E+00_JPRB, 0.74113E+00_JPRB, 0.74635E+00_JPRB, 0.64416E+00_JPRB, 0.71954E+00_JPRB /)
ZBURD(:, 55,  9) = (/ &
&  0.61855E+00_JPRB, 0.61095E+00_JPRB, 0.58996E+00_JPRB, 0.56785E+00_JPRB, 0.54551E+00_JPRB, 0.52420E+00_JPRB, 0.50907E+00_JPRB ,&
&  0.49822E+00_JPRB, 0.48471E+00_JPRB, 0.47311E+00_JPRB, 0.46469E+00_JPRB, 0.45538E+00_JPRB, 0.44634E+00_JPRB, 0.43659E+00_JPRB ,&
&  0.42899E+00_JPRB, 0.42435E+00_JPRB, 0.42112E+00_JPRB, 0.41764E+00_JPRB, 0.41362E+00_JPRB, 0.40875E+00_JPRB, 0.40382E+00_JPRB ,&
&  0.39874E+00_JPRB, 0.39454E+00_JPRB, 0.39018E+00_JPRB, 0.38813E+00_JPRB, 0.39121E+00_JPRB, 0.39828E+00_JPRB, 0.40498E+00_JPRB ,&
&  0.43204E+00_JPRB, 0.38316E+00_JPRB, 0.39653E+00_JPRB, 0.42812E+00_JPRB, 0.46391E+00_JPRB, 0.51173E+00_JPRB, 0.55148E+00_JPRB ,&
&  0.57078E+00_JPRB, 0.58975E+00_JPRB, 0.58297E+00_JPRB, 0.56795E+00_JPRB, 0.54137E+00_JPRB, 0.51465E+00_JPRB, 0.49333E+00_JPRB ,&
&  0.47999E+00_JPRB, 0.47026E+00_JPRB, 0.45886E+00_JPRB, 0.44621E+00_JPRB, 0.43436E+00_JPRB, 0.42619E+00_JPRB, 0.42292E+00_JPRB ,&
&  0.42364E+00_JPRB, 0.42493E+00_JPRB, 0.42531E+00_JPRB, 0.42830E+00_JPRB, 0.43500E+00_JPRB, 0.44400E+00_JPRB, 0.45821E+00_JPRB ,&
&  0.48284E+00_JPRB, 0.51571E+00_JPRB, 0.54318E+00_JPRB, 0.56711E+00_JPRB, 0.57973E+00_JPRB, 0.57858E+00_JPRB, 0.55660E+00_JPRB ,&
&  0.52828E+00_JPRB, 0.50281E+00_JPRB, 0.48559E+00_JPRB, 0.47569E+00_JPRB, 0.46874E+00_JPRB, 0.46342E+00_JPRB, 0.45680E+00_JPRB ,&
&  0.44806E+00_JPRB, 0.43868E+00_JPRB, 0.43112E+00_JPRB, 0.42545E+00_JPRB, 0.42187E+00_JPRB, 0.42169E+00_JPRB, 0.42382E+00_JPRB ,&
&  0.42664E+00_JPRB, 0.42936E+00_JPRB, 0.43251E+00_JPRB, 0.43744E+00_JPRB, 0.44327E+00_JPRB, 0.44977E+00_JPRB, 0.45543E+00_JPRB ,&
&  0.46247E+00_JPRB, 0.47208E+00_JPRB, 0.48227E+00_JPRB, 0.53053E+00_JPRB, 0.57389E+00_JPRB, 0.59631E+00_JPRB, 0.60944E+00_JPRB ,&
&  0.61545E+00_JPRB, 0.62781E+00_JPRB, 0.53257E+00_JPRB, 0.58497E+00_JPRB, 0.61008E+00_JPRB /)
ZBURD(:, 56,  9) = (/ &
&  0.52831E+00_JPRB, 0.52388E+00_JPRB, 0.51527E+00_JPRB, 0.50019E+00_JPRB, 0.48403E+00_JPRB, 0.47020E+00_JPRB, 0.46190E+00_JPRB ,&
&  0.45591E+00_JPRB, 0.44734E+00_JPRB, 0.43982E+00_JPRB, 0.43393E+00_JPRB, 0.42676E+00_JPRB, 0.41874E+00_JPRB, 0.40915E+00_JPRB ,&
&  0.40173E+00_JPRB, 0.39612E+00_JPRB, 0.39200E+00_JPRB, 0.38931E+00_JPRB, 0.38737E+00_JPRB, 0.38564E+00_JPRB, 0.38326E+00_JPRB ,&
&  0.38089E+00_JPRB, 0.37893E+00_JPRB, 0.37691E+00_JPRB, 0.37640E+00_JPRB, 0.37704E+00_JPRB, 0.38119E+00_JPRB, 0.38758E+00_JPRB ,&
&  0.37690E+00_JPRB, 0.35179E+00_JPRB, 0.36392E+00_JPRB, 0.38147E+00_JPRB, 0.40281E+00_JPRB, 0.43775E+00_JPRB, 0.47448E+00_JPRB ,&
&  0.49597E+00_JPRB, 0.50901E+00_JPRB, 0.50103E+00_JPRB, 0.48835E+00_JPRB, 0.47441E+00_JPRB, 0.45970E+00_JPRB, 0.44794E+00_JPRB ,&
&  0.43734E+00_JPRB, 0.43008E+00_JPRB, 0.42393E+00_JPRB, 0.41507E+00_JPRB, 0.40438E+00_JPRB, 0.39565E+00_JPRB, 0.39047E+00_JPRB ,&
&  0.38874E+00_JPRB, 0.38962E+00_JPRB, 0.39101E+00_JPRB, 0.39266E+00_JPRB, 0.39574E+00_JPRB, 0.40177E+00_JPRB, 0.40970E+00_JPRB ,&
&  0.41736E+00_JPRB, 0.42879E+00_JPRB, 0.44388E+00_JPRB, 0.45782E+00_JPRB, 0.47024E+00_JPRB, 0.47689E+00_JPRB, 0.47870E+00_JPRB ,&
&  0.47423E+00_JPRB, 0.46389E+00_JPRB, 0.45178E+00_JPRB, 0.43981E+00_JPRB, 0.42706E+00_JPRB, 0.41299E+00_JPRB, 0.39879E+00_JPRB ,&
&  0.38708E+00_JPRB, 0.37774E+00_JPRB, 0.37116E+00_JPRB, 0.36849E+00_JPRB, 0.36812E+00_JPRB, 0.37028E+00_JPRB, 0.37348E+00_JPRB ,&
&  0.37693E+00_JPRB, 0.38050E+00_JPRB, 0.38314E+00_JPRB, 0.38722E+00_JPRB, 0.39171E+00_JPRB, 0.39724E+00_JPRB, 0.40336E+00_JPRB ,&
&  0.41044E+00_JPRB, 0.41791E+00_JPRB, 0.42646E+00_JPRB, 0.45258E+00_JPRB, 0.48659E+00_JPRB, 0.51569E+00_JPRB, 0.53146E+00_JPRB ,&
&  0.53278E+00_JPRB, 0.52960E+00_JPRB, 0.52990E+00_JPRB, 0.52714E+00_JPRB, 0.53099E+00_JPRB /)
ZBURD(:, 57,  9) = (/ &
&  0.46805E+00_JPRB, 0.46344E+00_JPRB, 0.45839E+00_JPRB, 0.45263E+00_JPRB, 0.44591E+00_JPRB, 0.43771E+00_JPRB, 0.43174E+00_JPRB ,&
&  0.42734E+00_JPRB, 0.42185E+00_JPRB, 0.41645E+00_JPRB, 0.40988E+00_JPRB, 0.40220E+00_JPRB, 0.39362E+00_JPRB, 0.38354E+00_JPRB ,&
&  0.37490E+00_JPRB, 0.36730E+00_JPRB, 0.36230E+00_JPRB, 0.35979E+00_JPRB, 0.35948E+00_JPRB, 0.35939E+00_JPRB, 0.35773E+00_JPRB ,&
&  0.35689E+00_JPRB, 0.35685E+00_JPRB, 0.35636E+00_JPRB, 0.35596E+00_JPRB, 0.35618E+00_JPRB, 0.35948E+00_JPRB, 0.36321E+00_JPRB ,&
&  0.37003E+00_JPRB, 0.34692E+00_JPRB, 0.35808E+00_JPRB, 0.36250E+00_JPRB, 0.37370E+00_JPRB, 0.39400E+00_JPRB, 0.41868E+00_JPRB ,&
&  0.43777E+00_JPRB, 0.43877E+00_JPRB, 0.43110E+00_JPRB, 0.42068E+00_JPRB, 0.41375E+00_JPRB, 0.40689E+00_JPRB, 0.39939E+00_JPRB ,&
&  0.39229E+00_JPRB, 0.38749E+00_JPRB, 0.38339E+00_JPRB, 0.37932E+00_JPRB, 0.37366E+00_JPRB, 0.36643E+00_JPRB, 0.35948E+00_JPRB ,&
&  0.35457E+00_JPRB, 0.35280E+00_JPRB, 0.35367E+00_JPRB, 0.35582E+00_JPRB, 0.35848E+00_JPRB, 0.36296E+00_JPRB, 0.36855E+00_JPRB ,&
&  0.37396E+00_JPRB, 0.37652E+00_JPRB, 0.37864E+00_JPRB, 0.38270E+00_JPRB, 0.38683E+00_JPRB, 0.38910E+00_JPRB, 0.38931E+00_JPRB ,&
&  0.38781E+00_JPRB, 0.38387E+00_JPRB, 0.37862E+00_JPRB, 0.37220E+00_JPRB, 0.36553E+00_JPRB, 0.35886E+00_JPRB, 0.35300E+00_JPRB ,&
&  0.34821E+00_JPRB, 0.34372E+00_JPRB, 0.33951E+00_JPRB, 0.33644E+00_JPRB, 0.33543E+00_JPRB, 0.33589E+00_JPRB, 0.33743E+00_JPRB ,&
&  0.34057E+00_JPRB, 0.34408E+00_JPRB, 0.34670E+00_JPRB, 0.34938E+00_JPRB, 0.35202E+00_JPRB, 0.35463E+00_JPRB, 0.35862E+00_JPRB ,&
&  0.36450E+00_JPRB, 0.36943E+00_JPRB, 0.37434E+00_JPRB, 0.38463E+00_JPRB, 0.40588E+00_JPRB, 0.43329E+00_JPRB, 0.45592E+00_JPRB ,&
&  0.46662E+00_JPRB, 0.46540E+00_JPRB, 0.46581E+00_JPRB, 0.46612E+00_JPRB, 0.46968E+00_JPRB /)
ZBURD(:, 58,  9) = (/ &
&  0.42970E+00_JPRB, 0.42693E+00_JPRB, 0.42276E+00_JPRB, 0.41945E+00_JPRB, 0.41546E+00_JPRB, 0.41044E+00_JPRB, 0.40545E+00_JPRB ,&
&  0.40140E+00_JPRB, 0.39721E+00_JPRB, 0.39238E+00_JPRB, 0.38597E+00_JPRB, 0.37767E+00_JPRB, 0.36983E+00_JPRB, 0.36138E+00_JPRB ,&
&  0.35371E+00_JPRB, 0.34723E+00_JPRB, 0.34282E+00_JPRB, 0.34023E+00_JPRB, 0.33807E+00_JPRB, 0.33623E+00_JPRB, 0.33419E+00_JPRB ,&
&  0.33309E+00_JPRB, 0.33289E+00_JPRB, 0.33300E+00_JPRB, 0.33336E+00_JPRB, 0.33391E+00_JPRB, 0.33650E+00_JPRB, 0.33819E+00_JPRB ,&
&  0.34095E+00_JPRB, 0.34232E+00_JPRB, 0.34986E+00_JPRB, 0.35436E+00_JPRB, 0.36148E+00_JPRB, 0.36870E+00_JPRB, 0.37594E+00_JPRB ,&
&  0.38233E+00_JPRB, 0.38529E+00_JPRB, 0.38090E+00_JPRB, 0.37410E+00_JPRB, 0.36924E+00_JPRB, 0.36664E+00_JPRB, 0.36120E+00_JPRB ,&
&  0.35568E+00_JPRB, 0.35166E+00_JPRB, 0.34848E+00_JPRB, 0.34620E+00_JPRB, 0.34386E+00_JPRB, 0.34039E+00_JPRB, 0.33655E+00_JPRB ,&
&  0.33171E+00_JPRB, 0.32807E+00_JPRB, 0.32522E+00_JPRB, 0.32413E+00_JPRB, 0.32435E+00_JPRB, 0.32566E+00_JPRB, 0.32779E+00_JPRB ,&
&  0.33104E+00_JPRB, 0.33394E+00_JPRB, 0.33522E+00_JPRB, 0.33568E+00_JPRB, 0.33592E+00_JPRB, 0.33650E+00_JPRB, 0.33636E+00_JPRB ,&
&  0.33506E+00_JPRB, 0.33296E+00_JPRB, 0.33038E+00_JPRB, 0.32737E+00_JPRB, 0.32413E+00_JPRB, 0.32158E+00_JPRB, 0.31946E+00_JPRB ,&
&  0.31732E+00_JPRB, 0.31490E+00_JPRB, 0.31238E+00_JPRB, 0.31100E+00_JPRB, 0.31008E+00_JPRB, 0.30966E+00_JPRB, 0.31052E+00_JPRB ,&
&  0.31266E+00_JPRB, 0.31523E+00_JPRB, 0.31797E+00_JPRB, 0.32038E+00_JPRB, 0.32267E+00_JPRB, 0.32478E+00_JPRB, 0.32768E+00_JPRB ,&
&  0.33265E+00_JPRB, 0.33821E+00_JPRB, 0.34377E+00_JPRB, 0.34971E+00_JPRB, 0.36038E+00_JPRB, 0.37842E+00_JPRB, 0.39923E+00_JPRB ,&
&  0.41296E+00_JPRB, 0.41871E+00_JPRB, 0.42129E+00_JPRB, 0.42496E+00_JPRB, 0.42893E+00_JPRB /)
ZBURD(:, 59,  9) = (/ &
&  0.38480E+00_JPRB, 0.38629E+00_JPRB, 0.38609E+00_JPRB, 0.38634E+00_JPRB, 0.38572E+00_JPRB, 0.38336E+00_JPRB, 0.37902E+00_JPRB ,&
&  0.37438E+00_JPRB, 0.36920E+00_JPRB, 0.36368E+00_JPRB, 0.35667E+00_JPRB, 0.34953E+00_JPRB, 0.34380E+00_JPRB, 0.33779E+00_JPRB ,&
&  0.33182E+00_JPRB, 0.32641E+00_JPRB, 0.32263E+00_JPRB, 0.31957E+00_JPRB, 0.31663E+00_JPRB, 0.31383E+00_JPRB, 0.31035E+00_JPRB ,&
&  0.30737E+00_JPRB, 0.30472E+00_JPRB, 0.30518E+00_JPRB, 0.30774E+00_JPRB, 0.30982E+00_JPRB, 0.31125E+00_JPRB, 0.31244E+00_JPRB ,&
&  0.31273E+00_JPRB, 0.32154E+00_JPRB, 0.31587E+00_JPRB, 0.32609E+00_JPRB, 0.32906E+00_JPRB, 0.32976E+00_JPRB, 0.33028E+00_JPRB ,&
&  0.33281E+00_JPRB, 0.33584E+00_JPRB, 0.33799E+00_JPRB, 0.33479E+00_JPRB, 0.33010E+00_JPRB, 0.32641E+00_JPRB, 0.32229E+00_JPRB ,&
&  0.31667E+00_JPRB, 0.31433E+00_JPRB, 0.31424E+00_JPRB, 0.31453E+00_JPRB, 0.31461E+00_JPRB, 0.31412E+00_JPRB, 0.31291E+00_JPRB ,&
&  0.31115E+00_JPRB, 0.30863E+00_JPRB, 0.30567E+00_JPRB, 0.30271E+00_JPRB, 0.30080E+00_JPRB, 0.29867E+00_JPRB, 0.29681E+00_JPRB ,&
&  0.29616E+00_JPRB, 0.29679E+00_JPRB, 0.29727E+00_JPRB, 0.29730E+00_JPRB, 0.29659E+00_JPRB, 0.29607E+00_JPRB, 0.29589E+00_JPRB ,&
&  0.29511E+00_JPRB, 0.29402E+00_JPRB, 0.29263E+00_JPRB, 0.29081E+00_JPRB, 0.28887E+00_JPRB, 0.28776E+00_JPRB, 0.28752E+00_JPRB ,&
&  0.28708E+00_JPRB, 0.28552E+00_JPRB, 0.28320E+00_JPRB, 0.28262E+00_JPRB, 0.28246E+00_JPRB, 0.28270E+00_JPRB, 0.28318E+00_JPRB ,&
&  0.28429E+00_JPRB, 0.28634E+00_JPRB, 0.28863E+00_JPRB, 0.29082E+00_JPRB, 0.29304E+00_JPRB, 0.29632E+00_JPRB, 0.30035E+00_JPRB ,&
&  0.30498E+00_JPRB, 0.31002E+00_JPRB, 0.31493E+00_JPRB, 0.31948E+00_JPRB, 0.32460E+00_JPRB, 0.33319E+00_JPRB, 0.34635E+00_JPRB ,&
&  0.35850E+00_JPRB, 0.36710E+00_JPRB, 0.37167E+00_JPRB, 0.37589E+00_JPRB, 0.38078E+00_JPRB /)
ZBURD(:, 60,  9) = (/ &
&  0.34625E+00_JPRB, 0.35183E+00_JPRB, 0.35495E+00_JPRB, 0.35631E+00_JPRB, 0.35666E+00_JPRB, 0.35481E+00_JPRB, 0.35144E+00_JPRB ,&
&  0.34620E+00_JPRB, 0.34085E+00_JPRB, 0.33509E+00_JPRB, 0.32877E+00_JPRB, 0.32319E+00_JPRB, 0.31846E+00_JPRB, 0.31436E+00_JPRB ,&
&  0.31034E+00_JPRB, 0.30666E+00_JPRB, 0.30363E+00_JPRB, 0.30058E+00_JPRB, 0.29725E+00_JPRB, 0.29324E+00_JPRB, 0.28818E+00_JPRB ,&
&  0.28509E+00_JPRB, 0.28271E+00_JPRB, 0.28365E+00_JPRB, 0.28669E+00_JPRB, 0.28903E+00_JPRB, 0.29036E+00_JPRB, 0.29141E+00_JPRB ,&
&  0.29286E+00_JPRB, 0.29497E+00_JPRB, 0.29673E+00_JPRB, 0.29975E+00_JPRB, 0.30235E+00_JPRB, 0.30210E+00_JPRB, 0.30035E+00_JPRB ,&
&  0.29900E+00_JPRB, 0.29959E+00_JPRB, 0.30079E+00_JPRB, 0.30149E+00_JPRB, 0.30084E+00_JPRB, 0.29851E+00_JPRB, 0.29457E+00_JPRB ,&
&  0.28777E+00_JPRB, 0.28268E+00_JPRB, 0.28149E+00_JPRB, 0.28258E+00_JPRB, 0.28445E+00_JPRB, 0.28586E+00_JPRB, 0.28669E+00_JPRB ,&
&  0.28690E+00_JPRB, 0.28623E+00_JPRB, 0.28501E+00_JPRB, 0.28355E+00_JPRB, 0.28162E+00_JPRB, 0.27944E+00_JPRB, 0.27813E+00_JPRB ,&
&  0.27675E+00_JPRB, 0.27559E+00_JPRB, 0.27483E+00_JPRB, 0.27435E+00_JPRB, 0.27316E+00_JPRB, 0.27145E+00_JPRB, 0.27045E+00_JPRB ,&
&  0.26909E+00_JPRB, 0.26826E+00_JPRB, 0.26706E+00_JPRB, 0.26586E+00_JPRB, 0.26435E+00_JPRB, 0.26293E+00_JPRB, 0.26246E+00_JPRB ,&
&  0.26251E+00_JPRB, 0.26132E+00_JPRB, 0.25988E+00_JPRB, 0.25973E+00_JPRB, 0.26021E+00_JPRB, 0.26103E+00_JPRB, 0.26170E+00_JPRB ,&
&  0.26238E+00_JPRB, 0.26326E+00_JPRB, 0.26519E+00_JPRB, 0.26740E+00_JPRB, 0.26991E+00_JPRB, 0.27300E+00_JPRB, 0.27655E+00_JPRB ,&
&  0.28087E+00_JPRB, 0.28598E+00_JPRB, 0.29175E+00_JPRB, 0.29693E+00_JPRB, 0.30156E+00_JPRB, 0.30601E+00_JPRB, 0.31314E+00_JPRB ,&
&  0.32174E+00_JPRB, 0.32858E+00_JPRB, 0.33332E+00_JPRB, 0.33704E+00_JPRB, 0.34090E+00_JPRB /)
ZBURD(:, 61,  9) = (/ &
&  0.30914E+00_JPRB, 0.31488E+00_JPRB, 0.31943E+00_JPRB, 0.32204E+00_JPRB, 0.32223E+00_JPRB, 0.32087E+00_JPRB, 0.31772E+00_JPRB ,&
&  0.31308E+00_JPRB, 0.30826E+00_JPRB, 0.30340E+00_JPRB, 0.29934E+00_JPRB, 0.29658E+00_JPRB, 0.29420E+00_JPRB, 0.29215E+00_JPRB ,&
&  0.28968E+00_JPRB, 0.28686E+00_JPRB, 0.28356E+00_JPRB, 0.28069E+00_JPRB, 0.27742E+00_JPRB, 0.27306E+00_JPRB, 0.26987E+00_JPRB ,&
&  0.26765E+00_JPRB, 0.26561E+00_JPRB, 0.26499E+00_JPRB, 0.26629E+00_JPRB, 0.26878E+00_JPRB, 0.27115E+00_JPRB, 0.27375E+00_JPRB ,&
&  0.27554E+00_JPRB, 0.27663E+00_JPRB, 0.27747E+00_JPRB, 0.27734E+00_JPRB, 0.27726E+00_JPRB, 0.27692E+00_JPRB, 0.27622E+00_JPRB ,&
&  0.27547E+00_JPRB, 0.27482E+00_JPRB, 0.27364E+00_JPRB, 0.27353E+00_JPRB, 0.27344E+00_JPRB, 0.27229E+00_JPRB, 0.27017E+00_JPRB ,&
&  0.26665E+00_JPRB, 0.26177E+00_JPRB, 0.25788E+00_JPRB, 0.25591E+00_JPRB, 0.25525E+00_JPRB, 0.25550E+00_JPRB, 0.25634E+00_JPRB ,&
&  0.25747E+00_JPRB, 0.25908E+00_JPRB, 0.26059E+00_JPRB, 0.26131E+00_JPRB, 0.26113E+00_JPRB, 0.26092E+00_JPRB, 0.25964E+00_JPRB ,&
&  0.25841E+00_JPRB, 0.25762E+00_JPRB, 0.25706E+00_JPRB, 0.25642E+00_JPRB, 0.25425E+00_JPRB, 0.25159E+00_JPRB, 0.25094E+00_JPRB ,&
&  0.24943E+00_JPRB, 0.24842E+00_JPRB, 0.24698E+00_JPRB, 0.24624E+00_JPRB, 0.24537E+00_JPRB, 0.24409E+00_JPRB, 0.24270E+00_JPRB ,&
&  0.24157E+00_JPRB, 0.24095E+00_JPRB, 0.24055E+00_JPRB, 0.23965E+00_JPRB, 0.23992E+00_JPRB, 0.24045E+00_JPRB, 0.24181E+00_JPRB ,&
&  0.24304E+00_JPRB, 0.24425E+00_JPRB, 0.24583E+00_JPRB, 0.24736E+00_JPRB, 0.24969E+00_JPRB, 0.25247E+00_JPRB, 0.25566E+00_JPRB ,&
&  0.25840E+00_JPRB, 0.26219E+00_JPRB, 0.26657E+00_JPRB, 0.27067E+00_JPRB, 0.27398E+00_JPRB, 0.27680E+00_JPRB, 0.28119E+00_JPRB ,&
&  0.28822E+00_JPRB, 0.29446E+00_JPRB, 0.29826E+00_JPRB, 0.30167E+00_JPRB, 0.30451E+00_JPRB /)
ZBURD(:, 62,  9) = (/ &
&  0.28171E+00_JPRB, 0.28400E+00_JPRB, 0.28620E+00_JPRB, 0.28852E+00_JPRB, 0.29020E+00_JPRB, 0.29081E+00_JPRB, 0.28941E+00_JPRB ,&
&  0.28646E+00_JPRB, 0.28294E+00_JPRB, 0.27936E+00_JPRB, 0.27619E+00_JPRB, 0.27408E+00_JPRB, 0.27266E+00_JPRB, 0.27164E+00_JPRB ,&
&  0.27074E+00_JPRB, 0.26984E+00_JPRB, 0.26754E+00_JPRB, 0.26441E+00_JPRB, 0.26044E+00_JPRB, 0.25706E+00_JPRB, 0.25601E+00_JPRB ,&
&  0.25437E+00_JPRB, 0.25334E+00_JPRB, 0.25256E+00_JPRB, 0.25174E+00_JPRB, 0.25143E+00_JPRB, 0.25245E+00_JPRB, 0.25435E+00_JPRB ,&
&  0.25632E+00_JPRB, 0.25792E+00_JPRB, 0.25918E+00_JPRB, 0.25999E+00_JPRB, 0.25941E+00_JPRB, 0.25833E+00_JPRB, 0.25706E+00_JPRB ,&
&  0.25551E+00_JPRB, 0.25400E+00_JPRB, 0.25266E+00_JPRB, 0.25088E+00_JPRB, 0.24828E+00_JPRB, 0.24516E+00_JPRB, 0.24293E+00_JPRB ,&
&  0.24123E+00_JPRB, 0.23962E+00_JPRB, 0.23792E+00_JPRB, 0.23698E+00_JPRB, 0.23627E+00_JPRB, 0.23596E+00_JPRB, 0.23600E+00_JPRB ,&
&  0.23641E+00_JPRB, 0.23765E+00_JPRB, 0.23939E+00_JPRB, 0.24129E+00_JPRB, 0.24187E+00_JPRB, 0.24206E+00_JPRB, 0.24229E+00_JPRB ,&
&  0.24161E+00_JPRB, 0.24028E+00_JPRB, 0.23851E+00_JPRB, 0.23656E+00_JPRB, 0.23525E+00_JPRB, 0.23424E+00_JPRB, 0.23309E+00_JPRB ,&
&  0.23113E+00_JPRB, 0.22991E+00_JPRB, 0.22923E+00_JPRB, 0.22908E+00_JPRB, 0.22928E+00_JPRB, 0.22932E+00_JPRB, 0.22860E+00_JPRB ,&
&  0.22770E+00_JPRB, 0.22692E+00_JPRB, 0.22585E+00_JPRB, 0.22469E+00_JPRB, 0.22285E+00_JPRB, 0.22316E+00_JPRB, 0.22416E+00_JPRB ,&
&  0.22465E+00_JPRB, 0.22485E+00_JPRB, 0.22502E+00_JPRB, 0.22611E+00_JPRB, 0.22734E+00_JPRB, 0.22907E+00_JPRB, 0.23153E+00_JPRB ,&
&  0.23361E+00_JPRB, 0.23640E+00_JPRB, 0.24051E+00_JPRB, 0.24432E+00_JPRB, 0.24710E+00_JPRB, 0.24861E+00_JPRB, 0.25166E+00_JPRB ,&
&  0.25825E+00_JPRB, 0.26566E+00_JPRB, 0.27079E+00_JPRB, 0.27556E+00_JPRB, 0.27969E+00_JPRB /)
ZBURD(:, 63,  9) = (/ &
&  0.25636E+00_JPRB, 0.25772E+00_JPRB, 0.25902E+00_JPRB, 0.26052E+00_JPRB, 0.26238E+00_JPRB, 0.26350E+00_JPRB, 0.26434E+00_JPRB ,&
&  0.26365E+00_JPRB, 0.26163E+00_JPRB, 0.25921E+00_JPRB, 0.25666E+00_JPRB, 0.25539E+00_JPRB, 0.25504E+00_JPRB, 0.25493E+00_JPRB ,&
&  0.25423E+00_JPRB, 0.25380E+00_JPRB, 0.25181E+00_JPRB, 0.24864E+00_JPRB, 0.24498E+00_JPRB, 0.24248E+00_JPRB, 0.24171E+00_JPRB ,&
&  0.24113E+00_JPRB, 0.24026E+00_JPRB, 0.23986E+00_JPRB, 0.23938E+00_JPRB, 0.23723E+00_JPRB, 0.23613E+00_JPRB, 0.23653E+00_JPRB ,&
&  0.23736E+00_JPRB, 0.23886E+00_JPRB, 0.23910E+00_JPRB, 0.23667E+00_JPRB, 0.23892E+00_JPRB, 0.23871E+00_JPRB, 0.23687E+00_JPRB ,&
&  0.23447E+00_JPRB, 0.23197E+00_JPRB, 0.22971E+00_JPRB, 0.22745E+00_JPRB, 0.22515E+00_JPRB, 0.22339E+00_JPRB, 0.22223E+00_JPRB ,&
&  0.22191E+00_JPRB, 0.22204E+00_JPRB, 0.22242E+00_JPRB, 0.22326E+00_JPRB, 0.22386E+00_JPRB, 0.22419E+00_JPRB, 0.22422E+00_JPRB ,&
&  0.22433E+00_JPRB, 0.22453E+00_JPRB, 0.22493E+00_JPRB, 0.22519E+00_JPRB, 0.22548E+00_JPRB, 0.22574E+00_JPRB, 0.22561E+00_JPRB ,&
&  0.22452E+00_JPRB, 0.22271E+00_JPRB, 0.22032E+00_JPRB, 0.21944E+00_JPRB, 0.21937E+00_JPRB, 0.21783E+00_JPRB, 0.21538E+00_JPRB ,&
&  0.21307E+00_JPRB, 0.21258E+00_JPRB, 0.21373E+00_JPRB, 0.21499E+00_JPRB, 0.21600E+00_JPRB, 0.21637E+00_JPRB, 0.21612E+00_JPRB ,&
&  0.21515E+00_JPRB, 0.21286E+00_JPRB, 0.21005E+00_JPRB, 0.20780E+00_JPRB, 0.20650E+00_JPRB, 0.20640E+00_JPRB, 0.20734E+00_JPRB ,&
&  0.20751E+00_JPRB, 0.20756E+00_JPRB, 0.20859E+00_JPRB, 0.20947E+00_JPRB, 0.21012E+00_JPRB, 0.21091E+00_JPRB, 0.21237E+00_JPRB ,&
&  0.21437E+00_JPRB, 0.21760E+00_JPRB, 0.22161E+00_JPRB, 0.22598E+00_JPRB, 0.22897E+00_JPRB, 0.22822E+00_JPRB, 0.22901E+00_JPRB ,&
&  0.23399E+00_JPRB, 0.24006E+00_JPRB, 0.24518E+00_JPRB, 0.24937E+00_JPRB, 0.25330E+00_JPRB /)
ZBURD(:, 64,  9) = (/ &
&  0.23569E+00_JPRB, 0.23710E+00_JPRB, 0.23836E+00_JPRB, 0.23954E+00_JPRB, 0.24143E+00_JPRB, 0.24290E+00_JPRB, 0.24458E+00_JPRB ,&
&  0.24580E+00_JPRB, 0.24445E+00_JPRB, 0.24258E+00_JPRB, 0.24103E+00_JPRB, 0.24029E+00_JPRB, 0.23946E+00_JPRB, 0.23881E+00_JPRB ,&
&  0.23941E+00_JPRB, 0.23859E+00_JPRB, 0.23582E+00_JPRB, 0.23376E+00_JPRB, 0.23047E+00_JPRB, 0.22829E+00_JPRB, 0.22785E+00_JPRB ,&
&  0.22942E+00_JPRB, 0.22914E+00_JPRB, 0.22848E+00_JPRB, 0.22745E+00_JPRB, 0.22608E+00_JPRB, 0.22538E+00_JPRB, 0.22430E+00_JPRB ,&
&  0.22370E+00_JPRB, 0.22271E+00_JPRB, 0.21807E+00_JPRB, 0.21866E+00_JPRB, 0.21828E+00_JPRB, 0.21823E+00_JPRB, 0.21766E+00_JPRB ,&
&  0.21700E+00_JPRB, 0.21597E+00_JPRB, 0.21414E+00_JPRB, 0.21214E+00_JPRB, 0.21047E+00_JPRB, 0.20955E+00_JPRB, 0.20944E+00_JPRB ,&
&  0.20976E+00_JPRB, 0.21066E+00_JPRB, 0.21220E+00_JPRB, 0.21423E+00_JPRB, 0.21510E+00_JPRB, 0.21575E+00_JPRB, 0.21676E+00_JPRB ,&
&  0.21721E+00_JPRB, 0.21731E+00_JPRB, 0.21735E+00_JPRB, 0.21729E+00_JPRB, 0.21697E+00_JPRB, 0.21611E+00_JPRB, 0.21539E+00_JPRB ,&
&  0.21371E+00_JPRB, 0.21127E+00_JPRB, 0.20962E+00_JPRB, 0.20758E+00_JPRB, 0.20198E+00_JPRB, 0.19175E+00_JPRB, 0.18605E+00_JPRB ,&
&  0.18623E+00_JPRB, 0.19090E+00_JPRB, 0.19648E+00_JPRB, 0.19975E+00_JPRB, 0.20502E+00_JPRB, 0.20518E+00_JPRB, 0.20146E+00_JPRB ,&
&  0.19306E+00_JPRB, 0.18785E+00_JPRB, 0.18396E+00_JPRB, 0.17964E+00_JPRB, 0.17700E+00_JPRB, 0.17648E+00_JPRB, 0.17691E+00_JPRB ,&
&  0.17891E+00_JPRB, 0.18172E+00_JPRB, 0.18458E+00_JPRB, 0.18619E+00_JPRB, 0.18636E+00_JPRB, 0.18511E+00_JPRB, 0.18358E+00_JPRB ,&
&  0.18329E+00_JPRB, 0.18733E+00_JPRB, 0.19564E+00_JPRB, 0.20468E+00_JPRB, 0.20896E+00_JPRB, 0.21065E+00_JPRB, 0.21240E+00_JPRB ,&
&  0.21613E+00_JPRB, 0.22123E+00_JPRB, 0.22638E+00_JPRB, 0.23009E+00_JPRB, 0.23321E+00_JPRB /)
ZBURD(:, 65,  9) = (/ &
&  0.21593E+00_JPRB, 0.21709E+00_JPRB, 0.21833E+00_JPRB, 0.21968E+00_JPRB, 0.22121E+00_JPRB, 0.22287E+00_JPRB, 0.22428E+00_JPRB ,&
&  0.22582E+00_JPRB, 0.22680E+00_JPRB, 0.22701E+00_JPRB, 0.22593E+00_JPRB, 0.22506E+00_JPRB, 0.22504E+00_JPRB, 0.22581E+00_JPRB ,&
&  0.22386E+00_JPRB, 0.22092E+00_JPRB, 0.21966E+00_JPRB, 0.21941E+00_JPRB, 0.21874E+00_JPRB, 0.21838E+00_JPRB, 0.21912E+00_JPRB ,&
&  0.22004E+00_JPRB, 0.21966E+00_JPRB, 0.21927E+00_JPRB, 0.21810E+00_JPRB, 0.21716E+00_JPRB, 0.21644E+00_JPRB, 0.21542E+00_JPRB ,&
&  0.21200E+00_JPRB, 0.20205E+00_JPRB, 0.19565E+00_JPRB, 0.19917E+00_JPRB, 0.20594E+00_JPRB, 0.20521E+00_JPRB, 0.20522E+00_JPRB ,&
&  0.20475E+00_JPRB, 0.20341E+00_JPRB, 0.20192E+00_JPRB, 0.20053E+00_JPRB, 0.19998E+00_JPRB, 0.20029E+00_JPRB, 0.20116E+00_JPRB ,&
&  0.20211E+00_JPRB, 0.20426E+00_JPRB, 0.20670E+00_JPRB, 0.20778E+00_JPRB, 0.20887E+00_JPRB, 0.20941E+00_JPRB, 0.20962E+00_JPRB ,&
&  0.20920E+00_JPRB, 0.20815E+00_JPRB, 0.20738E+00_JPRB, 0.20657E+00_JPRB, 0.20578E+00_JPRB, 0.20454E+00_JPRB, 0.20070E+00_JPRB ,&
&  0.19540E+00_JPRB, 0.19014E+00_JPRB, 0.18376E+00_JPRB, 0.17737E+00_JPRB, 0.17166E+00_JPRB, 0.16751E+00_JPRB, 0.16405E+00_JPRB ,&
&  0.16358E+00_JPRB, 0.16743E+00_JPRB, 0.17529E+00_JPRB, 0.18480E+00_JPRB, 0.19010E+00_JPRB, 0.18032E+00_JPRB, 0.17428E+00_JPRB ,&
&  0.16853E+00_JPRB, 0.16388E+00_JPRB, 0.16100E+00_JPRB, 0.15946E+00_JPRB, 0.15838E+00_JPRB, 0.15835E+00_JPRB, 0.15895E+00_JPRB ,&
&  0.16029E+00_JPRB, 0.16175E+00_JPRB, 0.16311E+00_JPRB, 0.16397E+00_JPRB, 0.16430E+00_JPRB, 0.16415E+00_JPRB, 0.16377E+00_JPRB ,&
&  0.16427E+00_JPRB, 0.16663E+00_JPRB, 0.17082E+00_JPRB, 0.17578E+00_JPRB, 0.17996E+00_JPRB, 0.18393E+00_JPRB, 0.18939E+00_JPRB ,&
&  0.19631E+00_JPRB, 0.20184E+00_JPRB, 0.20647E+00_JPRB, 0.21140E+00_JPRB, 0.21415E+00_JPRB /)
ZBURD(:, 66,  9) = (/ &
&  0.20103E+00_JPRB, 0.20298E+00_JPRB, 0.20368E+00_JPRB, 0.20502E+00_JPRB, 0.20550E+00_JPRB, 0.20683E+00_JPRB, 0.20784E+00_JPRB ,&
&  0.21027E+00_JPRB, 0.21248E+00_JPRB, 0.21305E+00_JPRB, 0.21135E+00_JPRB, 0.21025E+00_JPRB, 0.20732E+00_JPRB, 0.20613E+00_JPRB ,&
&  0.20578E+00_JPRB, 0.20677E+00_JPRB, 0.20692E+00_JPRB, 0.20697E+00_JPRB, 0.20689E+00_JPRB, 0.20683E+00_JPRB, 0.20732E+00_JPRB ,&
&  0.20532E+00_JPRB, 0.20447E+00_JPRB, 0.20655E+00_JPRB, 0.20570E+00_JPRB, 0.20501E+00_JPRB, 0.20362E+00_JPRB, 0.20287E+00_JPRB ,&
&  0.19877E+00_JPRB, 0.19294E+00_JPRB, 0.18639E+00_JPRB, 0.18953E+00_JPRB, 0.19671E+00_JPRB, 0.19427E+00_JPRB, 0.19356E+00_JPRB ,&
&  0.19305E+00_JPRB, 0.19194E+00_JPRB, 0.19157E+00_JPRB, 0.19119E+00_JPRB, 0.19190E+00_JPRB, 0.19258E+00_JPRB, 0.19431E+00_JPRB ,&
&  0.19668E+00_JPRB, 0.20011E+00_JPRB, 0.19998E+00_JPRB, 0.19359E+00_JPRB, 0.19053E+00_JPRB, 0.18950E+00_JPRB, 0.18623E+00_JPRB ,&
&  0.18232E+00_JPRB, 0.18093E+00_JPRB, 0.17993E+00_JPRB, 0.17735E+00_JPRB, 0.17651E+00_JPRB, 0.17602E+00_JPRB, 0.17349E+00_JPRB ,&
&  0.17019E+00_JPRB, 0.16773E+00_JPRB, 0.16429E+00_JPRB, 0.16079E+00_JPRB, 0.15835E+00_JPRB, 0.15721E+00_JPRB, 0.15730E+00_JPRB ,&
&  0.15914E+00_JPRB, 0.16102E+00_JPRB, 0.16682E+00_JPRB, 0.17053E+00_JPRB, 0.16734E+00_JPRB, 0.16421E+00_JPRB, 0.16078E+00_JPRB ,&
&  0.15542E+00_JPRB, 0.15291E+00_JPRB, 0.15088E+00_JPRB, 0.14939E+00_JPRB, 0.14922E+00_JPRB, 0.14995E+00_JPRB, 0.15050E+00_JPRB ,&
&  0.15125E+00_JPRB, 0.15229E+00_JPRB, 0.15279E+00_JPRB, 0.15302E+00_JPRB, 0.15307E+00_JPRB, 0.15333E+00_JPRB, 0.15395E+00_JPRB ,&
&  0.15474E+00_JPRB, 0.15662E+00_JPRB, 0.15928E+00_JPRB, 0.16211E+00_JPRB, 0.16410E+00_JPRB, 0.16566E+00_JPRB, 0.16899E+00_JPRB ,&
&  0.17279E+00_JPRB, 0.18165E+00_JPRB, 0.19413E+00_JPRB, 0.20320E+00_JPRB, 0.19965E+00_JPRB /)
ZBURD(:, 67,  9) = (/ &
&  0.19794E+00_JPRB, 0.19918E+00_JPRB, 0.19892E+00_JPRB, 0.19912E+00_JPRB, 0.19869E+00_JPRB, 0.19970E+00_JPRB, 0.19885E+00_JPRB ,&
&  0.19877E+00_JPRB, 0.19767E+00_JPRB, 0.19662E+00_JPRB, 0.19396E+00_JPRB, 0.19254E+00_JPRB, 0.18645E+00_JPRB, 0.18477E+00_JPRB ,&
&  0.18333E+00_JPRB, 0.18411E+00_JPRB, 0.18432E+00_JPRB, 0.18479E+00_JPRB, 0.18564E+00_JPRB, 0.18724E+00_JPRB, 0.18718E+00_JPRB ,&
&  0.18605E+00_JPRB, 0.18401E+00_JPRB, 0.18163E+00_JPRB, 0.17947E+00_JPRB, 0.17986E+00_JPRB, 0.17950E+00_JPRB, 0.17851E+00_JPRB ,&
&  0.17754E+00_JPRB, 0.17814E+00_JPRB, 0.17861E+00_JPRB, 0.17996E+00_JPRB, 0.18488E+00_JPRB, 0.18321E+00_JPRB, 0.18205E+00_JPRB ,&
&  0.18198E+00_JPRB, 0.18201E+00_JPRB, 0.18267E+00_JPRB, 0.18317E+00_JPRB, 0.18533E+00_JPRB, 0.18480E+00_JPRB, 0.18748E+00_JPRB ,&
&  0.18514E+00_JPRB, 0.18367E+00_JPRB, 0.18155E+00_JPRB, 0.17864E+00_JPRB, 0.17477E+00_JPRB, 0.17232E+00_JPRB, 0.17000E+00_JPRB ,&
&  0.16743E+00_JPRB, 0.16528E+00_JPRB, 0.16250E+00_JPRB, 0.16152E+00_JPRB, 0.16006E+00_JPRB, 0.15874E+00_JPRB, 0.15754E+00_JPRB ,&
&  0.15665E+00_JPRB, 0.15540E+00_JPRB, 0.15433E+00_JPRB, 0.15363E+00_JPRB, 0.15342E+00_JPRB, 0.15356E+00_JPRB, 0.15462E+00_JPRB ,&
&  0.15663E+00_JPRB, 0.15825E+00_JPRB, 0.15953E+00_JPRB, 0.16068E+00_JPRB, 0.15925E+00_JPRB, 0.15665E+00_JPRB, 0.15406E+00_JPRB ,&
&  0.15051E+00_JPRB, 0.14855E+00_JPRB, 0.14670E+00_JPRB, 0.14560E+00_JPRB, 0.14529E+00_JPRB, 0.14522E+00_JPRB, 0.14548E+00_JPRB ,&
&  0.14575E+00_JPRB, 0.14620E+00_JPRB, 0.14651E+00_JPRB, 0.14694E+00_JPRB, 0.14751E+00_JPRB, 0.14807E+00_JPRB, 0.14892E+00_JPRB ,&
&  0.15028E+00_JPRB, 0.15201E+00_JPRB, 0.15438E+00_JPRB, 0.15714E+00_JPRB, 0.16038E+00_JPRB, 0.16459E+00_JPRB, 0.17072E+00_JPRB ,&
&  0.17698E+00_JPRB, 0.18718E+00_JPRB, 0.18814E+00_JPRB, 0.19383E+00_JPRB, 0.19706E+00_JPRB /)
ZBURD(:, 68,  9) = (/ &
&  0.19540E+00_JPRB, 0.19723E+00_JPRB, 0.19789E+00_JPRB, 0.19882E+00_JPRB, 0.19850E+00_JPRB, 0.19606E+00_JPRB, 0.19358E+00_JPRB ,&
&  0.18816E+00_JPRB, 0.18383E+00_JPRB, 0.17857E+00_JPRB, 0.17367E+00_JPRB, 0.16902E+00_JPRB, 0.16605E+00_JPRB, 0.16429E+00_JPRB ,&
&  0.16423E+00_JPRB, 0.16563E+00_JPRB, 0.16666E+00_JPRB, 0.16725E+00_JPRB, 0.16763E+00_JPRB, 0.16860E+00_JPRB, 0.16867E+00_JPRB ,&
&  0.16822E+00_JPRB, 0.16716E+00_JPRB, 0.16639E+00_JPRB, 0.16726E+00_JPRB, 0.16927E+00_JPRB, 0.17135E+00_JPRB, 0.17307E+00_JPRB ,&
&  0.17457E+00_JPRB, 0.17386E+00_JPRB, 0.17384E+00_JPRB, 0.17304E+00_JPRB, 0.17257E+00_JPRB, 0.17198E+00_JPRB, 0.17188E+00_JPRB ,&
&  0.17277E+00_JPRB, 0.17347E+00_JPRB, 0.17536E+00_JPRB, 0.17634E+00_JPRB, 0.17518E+00_JPRB, 0.17718E+00_JPRB, 0.17743E+00_JPRB ,&
&  0.17669E+00_JPRB, 0.17362E+00_JPRB, 0.17028E+00_JPRB, 0.16765E+00_JPRB, 0.16490E+00_JPRB, 0.16268E+00_JPRB, 0.16087E+00_JPRB ,&
&  0.15920E+00_JPRB, 0.15832E+00_JPRB, 0.15647E+00_JPRB, 0.15489E+00_JPRB, 0.15388E+00_JPRB, 0.15282E+00_JPRB, 0.15214E+00_JPRB ,&
&  0.15171E+00_JPRB, 0.15095E+00_JPRB, 0.15080E+00_JPRB, 0.15040E+00_JPRB, 0.15050E+00_JPRB, 0.15063E+00_JPRB, 0.15097E+00_JPRB ,&
&  0.15175E+00_JPRB, 0.15271E+00_JPRB, 0.15262E+00_JPRB, 0.15249E+00_JPRB, 0.15101E+00_JPRB, 0.14938E+00_JPRB, 0.14797E+00_JPRB ,&
&  0.14613E+00_JPRB, 0.14476E+00_JPRB, 0.14392E+00_JPRB, 0.14309E+00_JPRB, 0.14296E+00_JPRB, 0.14299E+00_JPRB, 0.14294E+00_JPRB ,&
&  0.14311E+00_JPRB, 0.14339E+00_JPRB, 0.14384E+00_JPRB, 0.14481E+00_JPRB, 0.14565E+00_JPRB, 0.14687E+00_JPRB, 0.14822E+00_JPRB ,&
&  0.14929E+00_JPRB, 0.15127E+00_JPRB, 0.15337E+00_JPRB, 0.15588E+00_JPRB, 0.16070E+00_JPRB, 0.16488E+00_JPRB, 0.17239E+00_JPRB ,&
&  0.17820E+00_JPRB, 0.18735E+00_JPRB, 0.18540E+00_JPRB, 0.18960E+00_JPRB, 0.19337E+00_JPRB /)
ZBURD(:, 69,  9) = (/ &
&  0.19473E+00_JPRB, 0.19590E+00_JPRB, 0.19561E+00_JPRB, 0.19585E+00_JPRB, 0.19470E+00_JPRB, 0.19298E+00_JPRB, 0.19050E+00_JPRB ,&
&  0.18783E+00_JPRB, 0.18217E+00_JPRB, 0.17820E+00_JPRB, 0.17377E+00_JPRB, 0.17004E+00_JPRB, 0.16677E+00_JPRB, 0.16445E+00_JPRB ,&
&  0.16340E+00_JPRB, 0.16303E+00_JPRB, 0.16282E+00_JPRB, 0.16194E+00_JPRB, 0.16034E+00_JPRB, 0.15928E+00_JPRB, 0.15794E+00_JPRB ,&
&  0.15695E+00_JPRB, 0.15688E+00_JPRB, 0.15772E+00_JPRB, 0.15956E+00_JPRB, 0.16280E+00_JPRB, 0.16515E+00_JPRB, 0.16726E+00_JPRB ,&
&  0.16935E+00_JPRB, 0.16580E+00_JPRB, 0.16562E+00_JPRB, 0.16598E+00_JPRB, 0.16592E+00_JPRB, 0.16526E+00_JPRB, 0.16404E+00_JPRB ,&
&  0.16705E+00_JPRB, 0.16781E+00_JPRB, 0.17281E+00_JPRB, 0.17260E+00_JPRB, 0.17157E+00_JPRB, 0.17270E+00_JPRB, 0.17126E+00_JPRB ,&
&  0.17089E+00_JPRB, 0.16779E+00_JPRB, 0.16550E+00_JPRB, 0.16228E+00_JPRB, 0.15910E+00_JPRB, 0.15751E+00_JPRB, 0.15568E+00_JPRB ,&
&  0.15512E+00_JPRB, 0.15501E+00_JPRB, 0.15368E+00_JPRB, 0.15285E+00_JPRB, 0.15129E+00_JPRB, 0.14952E+00_JPRB, 0.14841E+00_JPRB ,&
&  0.14727E+00_JPRB, 0.14636E+00_JPRB, 0.14588E+00_JPRB, 0.14541E+00_JPRB, 0.14521E+00_JPRB, 0.14511E+00_JPRB, 0.14501E+00_JPRB ,&
&  0.14503E+00_JPRB, 0.14496E+00_JPRB, 0.14442E+00_JPRB, 0.14413E+00_JPRB, 0.14314E+00_JPRB, 0.14247E+00_JPRB, 0.14192E+00_JPRB ,&
&  0.14143E+00_JPRB, 0.14126E+00_JPRB, 0.14116E+00_JPRB, 0.14120E+00_JPRB, 0.14131E+00_JPRB, 0.14174E+00_JPRB, 0.14222E+00_JPRB ,&
&  0.14272E+00_JPRB, 0.14362E+00_JPRB, 0.14431E+00_JPRB, 0.14531E+00_JPRB, 0.14650E+00_JPRB, 0.14750E+00_JPRB, 0.14912E+00_JPRB ,&
&  0.15090E+00_JPRB, 0.15299E+00_JPRB, 0.15608E+00_JPRB, 0.15890E+00_JPRB, 0.16221E+00_JPRB, 0.16674E+00_JPRB, 0.17166E+00_JPRB ,&
&  0.17704E+00_JPRB, 0.18321E+00_JPRB, 0.18384E+00_JPRB, 0.18932E+00_JPRB, 0.19283E+00_JPRB /)
ZBURD(:, 70,  9) = (/ &
&  0.18751E+00_JPRB, 0.18818E+00_JPRB, 0.18881E+00_JPRB, 0.18904E+00_JPRB, 0.18909E+00_JPRB, 0.18831E+00_JPRB, 0.18734E+00_JPRB ,&
&  0.18351E+00_JPRB, 0.18075E+00_JPRB, 0.17768E+00_JPRB, 0.17464E+00_JPRB, 0.17197E+00_JPRB, 0.16924E+00_JPRB, 0.16678E+00_JPRB ,&
&  0.16454E+00_JPRB, 0.16286E+00_JPRB, 0.16132E+00_JPRB, 0.15921E+00_JPRB, 0.15655E+00_JPRB, 0.15412E+00_JPRB, 0.15257E+00_JPRB ,&
&  0.15212E+00_JPRB, 0.15243E+00_JPRB, 0.15383E+00_JPRB, 0.15528E+00_JPRB, 0.15683E+00_JPRB, 0.15799E+00_JPRB, 0.15917E+00_JPRB ,&
&  0.15860E+00_JPRB, 0.15824E+00_JPRB, 0.15790E+00_JPRB, 0.15730E+00_JPRB, 0.15791E+00_JPRB, 0.15872E+00_JPRB, 0.15921E+00_JPRB ,&
&  0.15977E+00_JPRB, 0.16126E+00_JPRB, 0.16233E+00_JPRB, 0.16334E+00_JPRB, 0.16328E+00_JPRB, 0.16252E+00_JPRB, 0.16105E+00_JPRB ,&
&  0.15919E+00_JPRB, 0.15728E+00_JPRB, 0.15559E+00_JPRB, 0.15413E+00_JPRB, 0.15301E+00_JPRB, 0.15176E+00_JPRB, 0.15071E+00_JPRB ,&
&  0.15020E+00_JPRB, 0.14974E+00_JPRB, 0.14907E+00_JPRB, 0.14840E+00_JPRB, 0.14736E+00_JPRB, 0.14645E+00_JPRB, 0.14550E+00_JPRB ,&
&  0.14464E+00_JPRB, 0.14382E+00_JPRB, 0.14307E+00_JPRB, 0.14230E+00_JPRB, 0.14177E+00_JPRB, 0.14129E+00_JPRB, 0.14083E+00_JPRB ,&
&  0.14034E+00_JPRB, 0.14017E+00_JPRB, 0.13982E+00_JPRB, 0.13970E+00_JPRB, 0.13948E+00_JPRB, 0.13934E+00_JPRB, 0.13953E+00_JPRB ,&
&  0.14004E+00_JPRB, 0.14056E+00_JPRB, 0.14082E+00_JPRB, 0.14114E+00_JPRB, 0.14146E+00_JPRB, 0.14187E+00_JPRB, 0.14256E+00_JPRB ,&
&  0.14308E+00_JPRB, 0.14373E+00_JPRB, 0.14428E+00_JPRB, 0.14493E+00_JPRB, 0.14617E+00_JPRB, 0.14745E+00_JPRB, 0.14891E+00_JPRB ,&
&  0.15074E+00_JPRB, 0.15267E+00_JPRB, 0.15494E+00_JPRB, 0.15715E+00_JPRB, 0.16008E+00_JPRB, 0.16183E+00_JPRB, 0.16511E+00_JPRB ,&
&  0.16874E+00_JPRB, 0.16997E+00_JPRB, 0.18209E+00_JPRB, 0.18328E+00_JPRB, 0.18494E+00_JPRB /)
ZBURD(:, 71,  9) = (/ &
&  0.16170E+00_JPRB, 0.16368E+00_JPRB, 0.16506E+00_JPRB, 0.16670E+00_JPRB, 0.16563E+00_JPRB, 0.17740E+00_JPRB, 0.17536E+00_JPRB ,&
&  0.17417E+00_JPRB, 0.16947E+00_JPRB, 0.16914E+00_JPRB, 0.16750E+00_JPRB, 0.16550E+00_JPRB, 0.16327E+00_JPRB, 0.16122E+00_JPRB ,&
&  0.15932E+00_JPRB, 0.15721E+00_JPRB, 0.15570E+00_JPRB, 0.15448E+00_JPRB, 0.15337E+00_JPRB, 0.15236E+00_JPRB, 0.15157E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15210E+00_JPRB, 0.15315E+00_JPRB, 0.15303E+00_JPRB, 0.15273E+00_JPRB, 0.15233E+00_JPRB, 0.15211E+00_JPRB ,&
&  0.15154E+00_JPRB, 0.15112E+00_JPRB, 0.15075E+00_JPRB, 0.15019E+00_JPRB, 0.15082E+00_JPRB, 0.15144E+00_JPRB, 0.15208E+00_JPRB ,&
&  0.15268E+00_JPRB, 0.15329E+00_JPRB, 0.15330E+00_JPRB, 0.15318E+00_JPRB, 0.15280E+00_JPRB, 0.15225E+00_JPRB, 0.15140E+00_JPRB ,&
&  0.15057E+00_JPRB, 0.14970E+00_JPRB, 0.14877E+00_JPRB, 0.14813E+00_JPRB, 0.14758E+00_JPRB, 0.14705E+00_JPRB, 0.14659E+00_JPRB ,&
&  0.14636E+00_JPRB, 0.14621E+00_JPRB, 0.14603E+00_JPRB, 0.14574E+00_JPRB, 0.14538E+00_JPRB, 0.14501E+00_JPRB, 0.14468E+00_JPRB ,&
&  0.14443E+00_JPRB, 0.14395E+00_JPRB, 0.14352E+00_JPRB, 0.14307E+00_JPRB, 0.14273E+00_JPRB, 0.14229E+00_JPRB, 0.14191E+00_JPRB ,&
&  0.14156E+00_JPRB, 0.14130E+00_JPRB, 0.14122E+00_JPRB, 0.14117E+00_JPRB, 0.14113E+00_JPRB, 0.14117E+00_JPRB, 0.14129E+00_JPRB ,&
&  0.14146E+00_JPRB, 0.14165E+00_JPRB, 0.14192E+00_JPRB, 0.14210E+00_JPRB, 0.14228E+00_JPRB, 0.14243E+00_JPRB, 0.14282E+00_JPRB ,&
&  0.14328E+00_JPRB, 0.14378E+00_JPRB, 0.14424E+00_JPRB, 0.14445E+00_JPRB, 0.14525E+00_JPRB, 0.14609E+00_JPRB, 0.14700E+00_JPRB ,&
&  0.14797E+00_JPRB, 0.14922E+00_JPRB, 0.15059E+00_JPRB, 0.15204E+00_JPRB, 0.15372E+00_JPRB, 0.15481E+00_JPRB, 0.15603E+00_JPRB ,&
&  0.15767E+00_JPRB, 0.16038E+00_JPRB, 0.16132E+00_JPRB, 0.16167E+00_JPRB, 0.16183E+00_JPRB /)
ZBURD(:, 72,  9) = (/ &
&  0.15106E+00_JPRB, 0.15086E+00_JPRB, 0.15062E+00_JPRB, 0.15037E+00_JPRB, 0.14956E+00_JPRB, 0.14918E+00_JPRB, 0.14884E+00_JPRB ,&
&  0.14864E+00_JPRB, 0.14851E+00_JPRB, 0.14858E+00_JPRB, 0.14870E+00_JPRB, 0.14884E+00_JPRB, 0.14897E+00_JPRB, 0.14907E+00_JPRB ,&
&  0.14904E+00_JPRB, 0.14904E+00_JPRB, 0.14838E+00_JPRB, 0.14786E+00_JPRB, 0.14754E+00_JPRB, 0.14730E+00_JPRB, 0.14698E+00_JPRB ,&
&  0.14672E+00_JPRB, 0.14640E+00_JPRB, 0.14602E+00_JPRB, 0.14576E+00_JPRB, 0.14578E+00_JPRB, 0.14564E+00_JPRB, 0.14565E+00_JPRB ,&
&  0.14564E+00_JPRB, 0.14580E+00_JPRB, 0.14587E+00_JPRB, 0.14595E+00_JPRB, 0.14630E+00_JPRB, 0.14661E+00_JPRB, 0.14668E+00_JPRB ,&
&  0.14672E+00_JPRB, 0.14655E+00_JPRB, 0.14638E+00_JPRB, 0.14625E+00_JPRB, 0.14619E+00_JPRB, 0.14618E+00_JPRB, 0.14614E+00_JPRB ,&
&  0.14601E+00_JPRB, 0.14581E+00_JPRB, 0.14556E+00_JPRB, 0.14528E+00_JPRB, 0.14502E+00_JPRB, 0.14489E+00_JPRB, 0.14486E+00_JPRB ,&
&  0.14484E+00_JPRB, 0.14471E+00_JPRB, 0.14450E+00_JPRB, 0.14419E+00_JPRB, 0.14390E+00_JPRB, 0.14361E+00_JPRB, 0.14334E+00_JPRB ,&
&  0.14309E+00_JPRB, 0.14297E+00_JPRB, 0.14284E+00_JPRB, 0.14273E+00_JPRB, 0.14266E+00_JPRB, 0.14255E+00_JPRB, 0.14244E+00_JPRB ,&
&  0.14234E+00_JPRB, 0.14226E+00_JPRB, 0.14217E+00_JPRB, 0.14209E+00_JPRB, 0.14204E+00_JPRB, 0.14200E+00_JPRB, 0.14200E+00_JPRB ,&
&  0.14201E+00_JPRB, 0.14202E+00_JPRB, 0.14206E+00_JPRB, 0.14219E+00_JPRB, 0.14235E+00_JPRB, 0.14255E+00_JPRB, 0.14278E+00_JPRB ,&
&  0.14305E+00_JPRB, 0.14334E+00_JPRB, 0.14366E+00_JPRB, 0.14390E+00_JPRB, 0.14426E+00_JPRB, 0.14463E+00_JPRB, 0.14502E+00_JPRB ,&
&  0.14540E+00_JPRB, 0.14588E+00_JPRB, 0.14641E+00_JPRB, 0.14698E+00_JPRB, 0.14761E+00_JPRB, 0.14797E+00_JPRB, 0.14841E+00_JPRB ,&
&  0.14892E+00_JPRB, 0.14957E+00_JPRB, 0.14994E+00_JPRB, 0.15020E+00_JPRB, 0.15050E+00_JPRB /)
ZBURD(:, 73,  9) = (/ &
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB ,&
&  0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB, 0.14453E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART18',1,ZHOOK_HANDLE)
END SUBROUTINE PART18



SUBROUTINE PART19

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART19',0,ZHOOK_HANDLE)


!-- month= 10
ZBURD(:,  1, 10) = (/ &
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB ,&
&  0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB, 0.89558E+00_JPRB /)
ZBURD(:,  2, 10) = (/ &
&  0.93768E+00_JPRB, 0.93532E+00_JPRB, 0.93282E+00_JPRB, 0.93033E+00_JPRB, 0.92761E+00_JPRB, 0.92500E+00_JPRB, 0.92234E+00_JPRB ,&
&  0.91998E+00_JPRB, 0.91748E+00_JPRB, 0.91530E+00_JPRB, 0.91318E+00_JPRB, 0.91134E+00_JPRB, 0.90937E+00_JPRB, 0.90754E+00_JPRB ,&
&  0.90563E+00_JPRB, 0.90381E+00_JPRB, 0.90128E+00_JPRB, 0.89905E+00_JPRB, 0.89665E+00_JPRB, 0.89433E+00_JPRB, 0.89141E+00_JPRB ,&
&  0.88916E+00_JPRB, 0.88658E+00_JPRB, 0.88414E+00_JPRB, 0.88088E+00_JPRB, 0.87832E+00_JPRB, 0.87575E+00_JPRB, 0.87349E+00_JPRB ,&
&  0.87150E+00_JPRB, 0.87005E+00_JPRB, 0.86870E+00_JPRB, 0.86719E+00_JPRB, 0.86653E+00_JPRB, 0.86670E+00_JPRB, 0.86696E+00_JPRB ,&
&  0.86705E+00_JPRB, 0.86750E+00_JPRB, 0.86829E+00_JPRB, 0.86871E+00_JPRB, 0.86811E+00_JPRB, 0.86759E+00_JPRB, 0.86714E+00_JPRB ,&
&  0.86631E+00_JPRB, 0.86498E+00_JPRB, 0.86351E+00_JPRB, 0.86227E+00_JPRB, 0.86080E+00_JPRB, 0.85888E+00_JPRB, 0.85713E+00_JPRB ,&
&  0.85643E+00_JPRB, 0.85577E+00_JPRB, 0.85500E+00_JPRB, 0.85419E+00_JPRB, 0.85410E+00_JPRB, 0.85404E+00_JPRB, 0.85387E+00_JPRB ,&
&  0.85378E+00_JPRB, 0.85408E+00_JPRB, 0.85408E+00_JPRB, 0.85398E+00_JPRB, 0.85409E+00_JPRB, 0.85514E+00_JPRB, 0.85650E+00_JPRB ,&
&  0.85822E+00_JPRB, 0.86056E+00_JPRB, 0.86350E+00_JPRB, 0.86641E+00_JPRB, 0.86955E+00_JPRB, 0.87306E+00_JPRB, 0.87687E+00_JPRB ,&
&  0.88079E+00_JPRB, 0.88486E+00_JPRB, 0.88940E+00_JPRB, 0.89406E+00_JPRB, 0.89851E+00_JPRB, 0.90294E+00_JPRB, 0.90783E+00_JPRB ,&
&  0.91215E+00_JPRB, 0.91600E+00_JPRB, 0.91943E+00_JPRB, 0.92289E+00_JPRB, 0.92635E+00_JPRB, 0.92953E+00_JPRB, 0.93230E+00_JPRB ,&
&  0.93478E+00_JPRB, 0.93691E+00_JPRB, 0.93880E+00_JPRB, 0.94075E+00_JPRB, 0.94160E+00_JPRB, 0.94204E+00_JPRB, 0.94230E+00_JPRB ,&
&  0.94232E+00_JPRB, 0.94201E+00_JPRB, 0.94140E+00_JPRB, 0.94061E+00_JPRB, 0.93955E+00_JPRB /)
ZBURD(:,  3, 10) = (/ &
&  0.96297E+00_JPRB, 0.95581E+00_JPRB, 0.94882E+00_JPRB, 0.94275E+00_JPRB, 0.93640E+00_JPRB, 0.93173E+00_JPRB, 0.92759E+00_JPRB ,&
&  0.92393E+00_JPRB, 0.92001E+00_JPRB, 0.91646E+00_JPRB, 0.91351E+00_JPRB, 0.91139E+00_JPRB, 0.90956E+00_JPRB, 0.90687E+00_JPRB ,&
&  0.90420E+00_JPRB, 0.90094E+00_JPRB, 0.89798E+00_JPRB, 0.89537E+00_JPRB, 0.89190E+00_JPRB, 0.88784E+00_JPRB, 0.88252E+00_JPRB ,&
&  0.87850E+00_JPRB, 0.87421E+00_JPRB, 0.86979E+00_JPRB, 0.86338E+00_JPRB, 0.85802E+00_JPRB, 0.85274E+00_JPRB, 0.84749E+00_JPRB ,&
&  0.84158E+00_JPRB, 0.83661E+00_JPRB, 0.83233E+00_JPRB, 0.82831E+00_JPRB, 0.83071E+00_JPRB, 0.83191E+00_JPRB, 0.83239E+00_JPRB ,&
&  0.83162E+00_JPRB, 0.82786E+00_JPRB, 0.82641E+00_JPRB, 0.82544E+00_JPRB, 0.82467E+00_JPRB, 0.82854E+00_JPRB, 0.83118E+00_JPRB ,&
&  0.83278E+00_JPRB, 0.83319E+00_JPRB, 0.83533E+00_JPRB, 0.83619E+00_JPRB, 0.83555E+00_JPRB, 0.83335E+00_JPRB, 0.83035E+00_JPRB ,&
&  0.82962E+00_JPRB, 0.82885E+00_JPRB, 0.82771E+00_JPRB, 0.82601E+00_JPRB, 0.82636E+00_JPRB, 0.82679E+00_JPRB, 0.82633E+00_JPRB ,&
&  0.82512E+00_JPRB, 0.82486E+00_JPRB, 0.82440E+00_JPRB, 0.82399E+00_JPRB, 0.82363E+00_JPRB, 0.82463E+00_JPRB, 0.82588E+00_JPRB ,&
&  0.82763E+00_JPRB, 0.83025E+00_JPRB, 0.83402E+00_JPRB, 0.83828E+00_JPRB, 0.84322E+00_JPRB, 0.85013E+00_JPRB, 0.85931E+00_JPRB ,&
&  0.87024E+00_JPRB, 0.88255E+00_JPRB, 0.89552E+00_JPRB, 0.90676E+00_JPRB, 0.91588E+00_JPRB, 0.92370E+00_JPRB, 0.93171E+00_JPRB ,&
&  0.93920E+00_JPRB, 0.94596E+00_JPRB, 0.95293E+00_JPRB, 0.96002E+00_JPRB, 0.96720E+00_JPRB, 0.97285E+00_JPRB, 0.97720E+00_JPRB ,&
&  0.98073E+00_JPRB, 0.98372E+00_JPRB, 0.98641E+00_JPRB, 0.98919E+00_JPRB, 0.99136E+00_JPRB, 0.99163E+00_JPRB, 0.99061E+00_JPRB ,&
&  0.98878E+00_JPRB, 0.98649E+00_JPRB, 0.98304E+00_JPRB, 0.97737E+00_JPRB, 0.97084E+00_JPRB /)
ZBURD(:,  4, 10) = (/ &
&  0.95686E+00_JPRB, 0.94868E+00_JPRB, 0.94200E+00_JPRB, 0.93733E+00_JPRB, 0.93326E+00_JPRB, 0.92983E+00_JPRB, 0.92718E+00_JPRB ,&
&  0.92442E+00_JPRB, 0.92099E+00_JPRB, 0.91705E+00_JPRB, 0.91452E+00_JPRB, 0.91159E+00_JPRB, 0.90920E+00_JPRB, 0.90530E+00_JPRB ,&
&  0.90212E+00_JPRB, 0.89774E+00_JPRB, 0.89556E+00_JPRB, 0.89349E+00_JPRB, 0.89080E+00_JPRB, 0.88852E+00_JPRB, 0.87788E+00_JPRB ,&
&  0.86795E+00_JPRB, 0.85924E+00_JPRB, 0.84829E+00_JPRB, 0.83825E+00_JPRB, 0.83098E+00_JPRB, 0.78937E+00_JPRB, 0.77576E+00_JPRB ,&
&  0.78673E+00_JPRB, 0.79431E+00_JPRB, 0.80201E+00_JPRB, 0.80816E+00_JPRB, 0.79752E+00_JPRB, 0.77695E+00_JPRB, 0.76374E+00_JPRB ,&
&  0.72706E+00_JPRB, 0.73316E+00_JPRB, 0.73842E+00_JPRB, 0.74337E+00_JPRB, 0.75240E+00_JPRB, 0.75748E+00_JPRB, 0.76781E+00_JPRB ,&
&  0.79267E+00_JPRB, 0.79889E+00_JPRB, 0.80671E+00_JPRB, 0.81238E+00_JPRB, 0.81343E+00_JPRB, 0.81537E+00_JPRB, 0.82187E+00_JPRB ,&
&  0.82435E+00_JPRB, 0.82693E+00_JPRB, 0.82893E+00_JPRB, 0.82911E+00_JPRB, 0.83206E+00_JPRB, 0.83429E+00_JPRB, 0.83559E+00_JPRB ,&
&  0.83441E+00_JPRB, 0.83425E+00_JPRB, 0.83401E+00_JPRB, 0.83354E+00_JPRB, 0.83377E+00_JPRB, 0.83220E+00_JPRB, 0.83256E+00_JPRB ,&
&  0.83429E+00_JPRB, 0.83895E+00_JPRB, 0.84507E+00_JPRB, 0.85343E+00_JPRB, 0.86438E+00_JPRB, 0.87940E+00_JPRB, 0.89526E+00_JPRB ,&
&  0.91332E+00_JPRB, 0.93070E+00_JPRB, 0.94651E+00_JPRB, 0.96008E+00_JPRB, 0.97226E+00_JPRB, 0.98427E+00_JPRB, 0.99452E+00_JPRB ,&
&  0.10057E+01_JPRB, 0.10168E+01_JPRB, 0.10237E+01_JPRB, 0.10263E+01_JPRB, 0.10324E+01_JPRB, 0.10384E+01_JPRB, 0.10415E+01_JPRB ,&
&  0.10429E+01_JPRB, 0.10434E+01_JPRB, 0.10413E+01_JPRB, 0.10370E+01_JPRB, 0.10327E+01_JPRB, 0.10278E+01_JPRB, 0.10213E+01_JPRB ,&
&  0.10147E+01_JPRB, 0.10080E+01_JPRB, 0.99802E+00_JPRB, 0.98429E+00_JPRB, 0.97036E+00_JPRB /)
ZBURD(:,  5, 10) = (/ &
&  0.93698E+00_JPRB, 0.93099E+00_JPRB, 0.92821E+00_JPRB, 0.92455E+00_JPRB, 0.92342E+00_JPRB, 0.92011E+00_JPRB, 0.91868E+00_JPRB ,&
&  0.91562E+00_JPRB, 0.91384E+00_JPRB, 0.91183E+00_JPRB, 0.90947E+00_JPRB, 0.90677E+00_JPRB, 0.90320E+00_JPRB, 0.90005E+00_JPRB ,&
&  0.89743E+00_JPRB, 0.89453E+00_JPRB, 0.89341E+00_JPRB, 0.89154E+00_JPRB, 0.88859E+00_JPRB, 0.88542E+00_JPRB, 0.87336E+00_JPRB ,&
&  0.86029E+00_JPRB, 0.84711E+00_JPRB, 0.79875E+00_JPRB, 0.79528E+00_JPRB, 0.78718E+00_JPRB, 0.78233E+00_JPRB, 0.78429E+00_JPRB ,&
&  0.77712E+00_JPRB, 0.76939E+00_JPRB, 0.74815E+00_JPRB, 0.72705E+00_JPRB, 0.71054E+00_JPRB, 0.69110E+00_JPRB, 0.68711E+00_JPRB ,&
&  0.68271E+00_JPRB, 0.68712E+00_JPRB, 0.69250E+00_JPRB, 0.69765E+00_JPRB, 0.71089E+00_JPRB, 0.72026E+00_JPRB, 0.73543E+00_JPRB ,&
&  0.75449E+00_JPRB, 0.76981E+00_JPRB, 0.77419E+00_JPRB, 0.77041E+00_JPRB, 0.79320E+00_JPRB, 0.80759E+00_JPRB, 0.81992E+00_JPRB ,&
&  0.82675E+00_JPRB, 0.82800E+00_JPRB, 0.83344E+00_JPRB, 0.83583E+00_JPRB, 0.84125E+00_JPRB, 0.85011E+00_JPRB, 0.85617E+00_JPRB ,&
&  0.86222E+00_JPRB, 0.86438E+00_JPRB, 0.86509E+00_JPRB, 0.86370E+00_JPRB, 0.86028E+00_JPRB, 0.85960E+00_JPRB, 0.86445E+00_JPRB ,&
&  0.87423E+00_JPRB, 0.89013E+00_JPRB, 0.91013E+00_JPRB, 0.93300E+00_JPRB, 0.95571E+00_JPRB, 0.97677E+00_JPRB, 0.99429E+00_JPRB ,&
&  0.10103E+01_JPRB, 0.10226E+01_JPRB, 0.10391E+01_JPRB, 0.10428E+01_JPRB, 0.10547E+01_JPRB, 0.10793E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10961E+01_JPRB, 0.11085E+01_JPRB, 0.11172E+01_JPRB, 0.11234E+01_JPRB, 0.11276E+01_JPRB, 0.11297E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.11282E+01_JPRB, 0.11203E+01_JPRB, 0.11084E+01_JPRB, 0.10933E+01_JPRB, 0.10796E+01_JPRB, 0.10663E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10332E+01_JPRB, 0.10134E+01_JPRB, 0.98906E+00_JPRB, 0.96340E+00_JPRB, 0.94811E+00_JPRB /)
ZBURD(:,  6, 10) = (/ &
&  0.91971E+00_JPRB, 0.91399E+00_JPRB, 0.91473E+00_JPRB, 0.91076E+00_JPRB, 0.91084E+00_JPRB, 0.90767E+00_JPRB, 0.90644E+00_JPRB ,&
&  0.90397E+00_JPRB, 0.90306E+00_JPRB, 0.90108E+00_JPRB, 0.89992E+00_JPRB, 0.89800E+00_JPRB, 0.89673E+00_JPRB, 0.89507E+00_JPRB ,&
&  0.89328E+00_JPRB, 0.89109E+00_JPRB, 0.88897E+00_JPRB, 0.88621E+00_JPRB, 0.88446E+00_JPRB, 0.88220E+00_JPRB, 0.87545E+00_JPRB ,&
&  0.86565E+00_JPRB, 0.84742E+00_JPRB, 0.81353E+00_JPRB, 0.80942E+00_JPRB, 0.80413E+00_JPRB, 0.80439E+00_JPRB, 0.80190E+00_JPRB ,&
&  0.79367E+00_JPRB, 0.73103E+00_JPRB, 0.73660E+00_JPRB, 0.72215E+00_JPRB, 0.71277E+00_JPRB, 0.70323E+00_JPRB, 0.68628E+00_JPRB ,&
&  0.68015E+00_JPRB, 0.67332E+00_JPRB, 0.67202E+00_JPRB, 0.67897E+00_JPRB, 0.68682E+00_JPRB, 0.70019E+00_JPRB, 0.71658E+00_JPRB ,&
&  0.71851E+00_JPRB, 0.74870E+00_JPRB, 0.76427E+00_JPRB, 0.77230E+00_JPRB, 0.79619E+00_JPRB, 0.81423E+00_JPRB, 0.81726E+00_JPRB ,&
&  0.82277E+00_JPRB, 0.82121E+00_JPRB, 0.82860E+00_JPRB, 0.84168E+00_JPRB, 0.85551E+00_JPRB, 0.86170E+00_JPRB, 0.86885E+00_JPRB ,&
&  0.87571E+00_JPRB, 0.88204E+00_JPRB, 0.88930E+00_JPRB, 0.89314E+00_JPRB, 0.90229E+00_JPRB, 0.92049E+00_JPRB, 0.94452E+00_JPRB ,&
&  0.97312E+00_JPRB, 0.10040E+01_JPRB, 0.10382E+01_JPRB, 0.10711E+01_JPRB, 0.10986E+01_JPRB, 0.11249E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.11698E+01_JPRB, 0.11886E+01_JPRB, 0.11999E+01_JPRB, 0.12088E+01_JPRB, 0.12113E+01_JPRB, 0.12074E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.12215E+01_JPRB, 0.12276E+01_JPRB, 0.12297E+01_JPRB, 0.12297E+01_JPRB, 0.12313E+01_JPRB, 0.12301E+01_JPRB, 0.12250E+01_JPRB ,&
&  0.12164E+01_JPRB, 0.12033E+01_JPRB, 0.11864E+01_JPRB, 0.11601E+01_JPRB, 0.11318E+01_JPRB, 0.11091E+01_JPRB, 0.10787E+01_JPRB ,&
&  0.10447E+01_JPRB, 0.10032E+01_JPRB, 0.96651E+00_JPRB, 0.93832E+00_JPRB, 0.92595E+00_JPRB /)
ZBURD(:,  7, 10) = (/ &
&  0.91391E+00_JPRB, 0.91274E+00_JPRB, 0.91634E+00_JPRB, 0.91573E+00_JPRB, 0.91676E+00_JPRB, 0.91366E+00_JPRB, 0.91313E+00_JPRB ,&
&  0.91208E+00_JPRB, 0.91308E+00_JPRB, 0.91317E+00_JPRB, 0.91331E+00_JPRB, 0.91222E+00_JPRB, 0.91108E+00_JPRB, 0.90872E+00_JPRB ,&
&  0.90504E+00_JPRB, 0.90126E+00_JPRB, 0.89552E+00_JPRB, 0.89029E+00_JPRB, 0.88675E+00_JPRB, 0.88682E+00_JPRB, 0.88300E+00_JPRB ,&
&  0.87513E+00_JPRB, 0.86017E+00_JPRB, 0.83558E+00_JPRB, 0.82698E+00_JPRB, 0.82729E+00_JPRB, 0.81513E+00_JPRB, 0.81624E+00_JPRB ,&
&  0.81075E+00_JPRB, 0.81012E+00_JPRB, 0.79978E+00_JPRB, 0.79555E+00_JPRB, 0.76929E+00_JPRB, 0.68075E+00_JPRB, 0.67396E+00_JPRB ,&
&  0.66752E+00_JPRB, 0.65640E+00_JPRB, 0.65469E+00_JPRB, 0.65675E+00_JPRB, 0.66530E+00_JPRB, 0.69261E+00_JPRB, 0.71753E+00_JPRB ,&
&  0.71852E+00_JPRB, 0.75296E+00_JPRB, 0.76670E+00_JPRB, 0.77834E+00_JPRB, 0.79912E+00_JPRB, 0.81515E+00_JPRB, 0.81447E+00_JPRB ,&
&  0.81828E+00_JPRB, 0.82191E+00_JPRB, 0.83354E+00_JPRB, 0.84632E+00_JPRB, 0.85877E+00_JPRB, 0.87144E+00_JPRB, 0.88840E+00_JPRB ,&
&  0.91049E+00_JPRB, 0.93130E+00_JPRB, 0.95675E+00_JPRB, 0.97589E+00_JPRB, 0.99609E+00_JPRB, 0.10226E+01_JPRB, 0.10537E+01_JPRB ,&
&  0.10914E+01_JPRB, 0.11278E+01_JPRB, 0.11735E+01_JPRB, 0.11989E+01_JPRB, 0.12285E+01_JPRB, 0.12540E+01_JPRB, 0.12908E+01_JPRB ,&
&  0.13183E+01_JPRB, 0.13612E+01_JPRB, 0.13714E+01_JPRB, 0.13845E+01_JPRB, 0.13640E+01_JPRB, 0.13430E+01_JPRB, 0.13324E+01_JPRB ,&
&  0.13254E+01_JPRB, 0.13356E+01_JPRB, 0.13206E+01_JPRB, 0.13114E+01_JPRB, 0.13048E+01_JPRB, 0.13012E+01_JPRB, 0.12944E+01_JPRB ,&
&  0.12818E+01_JPRB, 0.12617E+01_JPRB, 0.12361E+01_JPRB, 0.12032E+01_JPRB, 0.11735E+01_JPRB, 0.11425E+01_JPRB, 0.10874E+01_JPRB ,&
&  0.10258E+01_JPRB, 0.96573E+00_JPRB, 0.93309E+00_JPRB, 0.91471E+00_JPRB, 0.91103E+00_JPRB /)
ZBURD(:,  8, 10) = (/ &
&  0.92673E+00_JPRB, 0.93403E+00_JPRB, 0.94172E+00_JPRB, 0.94678E+00_JPRB, 0.94597E+00_JPRB, 0.94231E+00_JPRB, 0.93681E+00_JPRB ,&
&  0.93172E+00_JPRB, 0.93030E+00_JPRB, 0.92843E+00_JPRB, 0.92725E+00_JPRB, 0.92555E+00_JPRB, 0.92242E+00_JPRB, 0.91897E+00_JPRB ,&
&  0.91674E+00_JPRB, 0.89852E+00_JPRB, 0.88983E+00_JPRB, 0.89023E+00_JPRB, 0.88844E+00_JPRB, 0.88793E+00_JPRB, 0.88713E+00_JPRB ,&
&  0.87800E+00_JPRB, 0.86521E+00_JPRB, 0.85583E+00_JPRB, 0.84391E+00_JPRB, 0.83186E+00_JPRB, 0.83565E+00_JPRB, 0.83449E+00_JPRB ,&
&  0.81662E+00_JPRB, 0.80321E+00_JPRB, 0.80980E+00_JPRB, 0.81063E+00_JPRB, 0.76761E+00_JPRB, 0.72316E+00_JPRB, 0.68976E+00_JPRB ,&
&  0.68612E+00_JPRB, 0.66511E+00_JPRB, 0.66124E+00_JPRB, 0.66197E+00_JPRB, 0.67445E+00_JPRB, 0.69099E+00_JPRB, 0.72062E+00_JPRB ,&
&  0.73150E+00_JPRB, 0.74030E+00_JPRB, 0.76315E+00_JPRB, 0.78429E+00_JPRB, 0.79674E+00_JPRB, 0.80690E+00_JPRB, 0.80753E+00_JPRB ,&
&  0.81648E+00_JPRB, 0.82568E+00_JPRB, 0.83702E+00_JPRB, 0.85164E+00_JPRB, 0.87507E+00_JPRB, 0.89362E+00_JPRB, 0.91931E+00_JPRB ,&
&  0.10140E+01_JPRB, 0.10914E+01_JPRB, 0.11541E+01_JPRB, 0.12106E+01_JPRB, 0.12512E+01_JPRB, 0.12705E+01_JPRB, 0.12500E+01_JPRB ,&
&  0.12509E+01_JPRB, 0.12894E+01_JPRB, 0.13306E+01_JPRB, 0.13719E+01_JPRB, 0.14103E+01_JPRB, 0.14424E+01_JPRB, 0.14537E+01_JPRB ,&
&  0.14260E+01_JPRB, 0.14211E+01_JPRB, 0.15472E+01_JPRB, 0.15881E+01_JPRB, 0.15768E+01_JPRB, 0.15357E+01_JPRB, 0.14765E+01_JPRB ,&
&  0.14270E+01_JPRB, 0.13947E+01_JPRB, 0.13736E+01_JPRB, 0.13542E+01_JPRB, 0.13343E+01_JPRB, 0.13132E+01_JPRB, 0.13057E+01_JPRB ,&
&  0.12878E+01_JPRB, 0.12617E+01_JPRB, 0.12296E+01_JPRB, 0.11971E+01_JPRB, 0.11658E+01_JPRB, 0.11306E+01_JPRB, 0.10699E+01_JPRB ,&
&  0.10019E+01_JPRB, 0.95227E+00_JPRB, 0.92873E+00_JPRB, 0.91976E+00_JPRB, 0.92115E+00_JPRB /)
ZBURD(:,  9, 10) = (/ &
&  0.96401E+00_JPRB, 0.97465E+00_JPRB, 0.98395E+00_JPRB, 0.98810E+00_JPRB, 0.98072E+00_JPRB, 0.95444E+00_JPRB, 0.93334E+00_JPRB ,&
&  0.91976E+00_JPRB, 0.91801E+00_JPRB, 0.91651E+00_JPRB, 0.91230E+00_JPRB, 0.90929E+00_JPRB, 0.90629E+00_JPRB, 0.90480E+00_JPRB ,&
&  0.90515E+00_JPRB, 0.90226E+00_JPRB, 0.89268E+00_JPRB, 0.88100E+00_JPRB, 0.87593E+00_JPRB, 0.87642E+00_JPRB, 0.87910E+00_JPRB ,&
&  0.87680E+00_JPRB, 0.87158E+00_JPRB, 0.87143E+00_JPRB, 0.85777E+00_JPRB, 0.84730E+00_JPRB, 0.82860E+00_JPRB, 0.81753E+00_JPRB ,&
&  0.81818E+00_JPRB, 0.82106E+00_JPRB, 0.80540E+00_JPRB, 0.80292E+00_JPRB, 0.80535E+00_JPRB, 0.75580E+00_JPRB, 0.71816E+00_JPRB ,&
&  0.69407E+00_JPRB, 0.67656E+00_JPRB, 0.66837E+00_JPRB, 0.67106E+00_JPRB, 0.70172E+00_JPRB, 0.74314E+00_JPRB, 0.75759E+00_JPRB ,&
&  0.72268E+00_JPRB, 0.74364E+00_JPRB, 0.77124E+00_JPRB, 0.78578E+00_JPRB, 0.79609E+00_JPRB, 0.80982E+00_JPRB, 0.82310E+00_JPRB ,&
&  0.83651E+00_JPRB, 0.85031E+00_JPRB, 0.86738E+00_JPRB, 0.90992E+00_JPRB, 0.94287E+00_JPRB, 0.94973E+00_JPRB, 0.99591E+00_JPRB ,&
&  0.12327E+01_JPRB, 0.14086E+01_JPRB, 0.15321E+01_JPRB, 0.15925E+01_JPRB, 0.16109E+01_JPRB, 0.15825E+01_JPRB, 0.15609E+01_JPRB ,&
&  0.15482E+01_JPRB, 0.15496E+01_JPRB, 0.15687E+01_JPRB, 0.15902E+01_JPRB, 0.16095E+01_JPRB, 0.16061E+01_JPRB, 0.15909E+01_JPRB ,&
&  0.15542E+01_JPRB, 0.15447E+01_JPRB, 0.18835E+01_JPRB, 0.18064E+01_JPRB, 0.17320E+01_JPRB, 0.16911E+01_JPRB, 0.16227E+01_JPRB ,&
&  0.15578E+01_JPRB, 0.15070E+01_JPRB, 0.14777E+01_JPRB, 0.14431E+01_JPRB, 0.13427E+01_JPRB, 0.12666E+01_JPRB, 0.12189E+01_JPRB ,&
&  0.11977E+01_JPRB, 0.11751E+01_JPRB, 0.11531E+01_JPRB, 0.11321E+01_JPRB, 0.11060E+01_JPRB, 0.10725E+01_JPRB, 0.10232E+01_JPRB ,&
&  0.98357E+00_JPRB, 0.96420E+00_JPRB, 0.95540E+00_JPRB, 0.95648E+00_JPRB, 0.95947E+00_JPRB /)
ZBURD(:, 10, 10) = (/ &
&  0.97384E+00_JPRB, 0.10087E+01_JPRB, 0.10318E+01_JPRB, 0.10321E+01_JPRB, 0.10168E+01_JPRB, 0.95666E+00_JPRB, 0.90494E+00_JPRB ,&
&  0.87482E+00_JPRB, 0.86554E+00_JPRB, 0.86670E+00_JPRB, 0.86729E+00_JPRB, 0.86942E+00_JPRB, 0.87322E+00_JPRB, 0.87449E+00_JPRB ,&
&  0.87278E+00_JPRB, 0.87437E+00_JPRB, 0.87722E+00_JPRB, 0.87427E+00_JPRB, 0.86764E+00_JPRB, 0.86315E+00_JPRB, 0.86415E+00_JPRB ,&
&  0.86514E+00_JPRB, 0.86415E+00_JPRB, 0.86495E+00_JPRB, 0.86029E+00_JPRB, 0.85985E+00_JPRB, 0.85257E+00_JPRB, 0.84137E+00_JPRB ,&
&  0.82180E+00_JPRB, 0.81848E+00_JPRB, 0.82948E+00_JPRB, 0.82899E+00_JPRB, 0.83358E+00_JPRB, 0.79801E+00_JPRB, 0.74869E+00_JPRB ,&
&  0.71642E+00_JPRB, 0.70333E+00_JPRB, 0.71839E+00_JPRB, 0.76309E+00_JPRB, 0.79355E+00_JPRB, 0.75844E+00_JPRB, 0.77463E+00_JPRB ,&
&  0.77179E+00_JPRB, 0.77200E+00_JPRB, 0.77822E+00_JPRB, 0.79743E+00_JPRB, 0.81702E+00_JPRB, 0.83490E+00_JPRB, 0.84970E+00_JPRB ,&
&  0.86803E+00_JPRB, 0.89383E+00_JPRB, 0.93208E+00_JPRB, 0.97264E+00_JPRB, 0.10096E+01_JPRB, 0.11247E+01_JPRB, 0.12694E+01_JPRB ,&
&  0.14375E+01_JPRB, 0.15593E+01_JPRB, 0.16276E+01_JPRB, 0.17227E+01_JPRB, 0.17930E+01_JPRB, 0.18189E+01_JPRB, 0.18236E+01_JPRB ,&
&  0.18283E+01_JPRB, 0.18018E+01_JPRB, 0.17842E+01_JPRB, 0.18002E+01_JPRB, 0.18318E+01_JPRB, 0.18390E+01_JPRB, 0.17987E+01_JPRB ,&
&  0.17461E+01_JPRB, 0.16442E+01_JPRB, 0.15330E+01_JPRB, 0.14018E+01_JPRB, 0.13848E+01_JPRB, 0.14309E+01_JPRB, 0.14835E+01_JPRB ,&
&  0.14780E+01_JPRB, 0.14569E+01_JPRB, 0.14382E+01_JPRB, 0.14188E+01_JPRB, 0.12732E+01_JPRB, 0.11716E+01_JPRB, 0.11017E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10393E+01_JPRB, 0.10307E+01_JPRB, 0.10297E+01_JPRB, 0.10238E+01_JPRB, 0.10127E+01_JPRB, 0.97602E+00_JPRB ,&
&  0.94706E+00_JPRB, 0.93073E+00_JPRB, 0.93116E+00_JPRB, 0.93581E+00_JPRB, 0.95312E+00_JPRB /)
ZBURD(:, 11, 10) = (/ &
&  0.10876E+01_JPRB, 0.10930E+01_JPRB, 0.10781E+01_JPRB, 0.10726E+01_JPRB, 0.10429E+01_JPRB, 0.10068E+01_JPRB, 0.96561E+00_JPRB ,&
&  0.93379E+00_JPRB, 0.89697E+00_JPRB, 0.86122E+00_JPRB, 0.83009E+00_JPRB, 0.81564E+00_JPRB, 0.81238E+00_JPRB, 0.81996E+00_JPRB ,&
&  0.83383E+00_JPRB, 0.84801E+00_JPRB, 0.85764E+00_JPRB, 0.85620E+00_JPRB, 0.85030E+00_JPRB, 0.84899E+00_JPRB, 0.85322E+00_JPRB ,&
&  0.85724E+00_JPRB, 0.86268E+00_JPRB, 0.86543E+00_JPRB, 0.86265E+00_JPRB, 0.85891E+00_JPRB, 0.85618E+00_JPRB, 0.85926E+00_JPRB ,&
&  0.85939E+00_JPRB, 0.85511E+00_JPRB, 0.86136E+00_JPRB, 0.86660E+00_JPRB, 0.87034E+00_JPRB, 0.84359E+00_JPRB, 0.79052E+00_JPRB ,&
&  0.75131E+00_JPRB, 0.74937E+00_JPRB, 0.80704E+00_JPRB, 0.78942E+00_JPRB, 0.81452E+00_JPRB, 0.82506E+00_JPRB, 0.82130E+00_JPRB ,&
&  0.81534E+00_JPRB, 0.81113E+00_JPRB, 0.82739E+00_JPRB, 0.84504E+00_JPRB, 0.85969E+00_JPRB, 0.87874E+00_JPRB, 0.90048E+00_JPRB ,&
&  0.94449E+00_JPRB, 0.10091E+01_JPRB, 0.10752E+01_JPRB, 0.11072E+01_JPRB, 0.12659E+01_JPRB, 0.16908E+01_JPRB, 0.18293E+01_JPRB ,&
&  0.19021E+01_JPRB, 0.19661E+01_JPRB, 0.20743E+01_JPRB, 0.21564E+01_JPRB, 0.21382E+01_JPRB, 0.21337E+01_JPRB, 0.21370E+01_JPRB ,&
&  0.20953E+01_JPRB, 0.20891E+01_JPRB, 0.21070E+01_JPRB, 0.21722E+01_JPRB, 0.21631E+01_JPRB, 0.21249E+01_JPRB, 0.20901E+01_JPRB ,&
&  0.20604E+01_JPRB, 0.19750E+01_JPRB, 0.18795E+01_JPRB, 0.17405E+01_JPRB, 0.16594E+01_JPRB, 0.16042E+01_JPRB, 0.15737E+01_JPRB ,&
&  0.15355E+01_JPRB, 0.15206E+01_JPRB, 0.15063E+01_JPRB, 0.14775E+01_JPRB, 0.13471E+01_JPRB, 0.11942E+01_JPRB, 0.10809E+01_JPRB ,&
&  0.10128E+01_JPRB, 0.97221E+00_JPRB, 0.95818E+00_JPRB, 0.96075E+00_JPRB, 0.97393E+00_JPRB, 0.98652E+00_JPRB, 0.95683E+00_JPRB ,&
&  0.95058E+00_JPRB, 0.95677E+00_JPRB, 0.98227E+00_JPRB, 0.10216E+01_JPRB, 0.10593E+01_JPRB /)
ZBURD(:, 12, 10) = (/ &
&  0.11883E+01_JPRB, 0.11830E+01_JPRB, 0.11560E+01_JPRB, 0.11360E+01_JPRB, 0.11066E+01_JPRB, 0.10541E+01_JPRB, 0.97658E+00_JPRB ,&
&  0.93313E+00_JPRB, 0.88698E+00_JPRB, 0.85335E+00_JPRB, 0.84077E+00_JPRB, 0.83425E+00_JPRB, 0.82213E+00_JPRB, 0.80633E+00_JPRB ,&
&  0.80864E+00_JPRB, 0.83002E+00_JPRB, 0.85514E+00_JPRB, 0.86745E+00_JPRB, 0.87531E+00_JPRB, 0.87725E+00_JPRB, 0.88270E+00_JPRB ,&
&  0.88847E+00_JPRB, 0.89278E+00_JPRB, 0.89715E+00_JPRB, 0.89007E+00_JPRB, 0.88330E+00_JPRB, 0.88476E+00_JPRB, 0.88857E+00_JPRB ,&
&  0.89683E+00_JPRB, 0.89715E+00_JPRB, 0.90014E+00_JPRB, 0.89852E+00_JPRB, 0.88946E+00_JPRB, 0.88405E+00_JPRB, 0.82762E+00_JPRB ,&
&  0.78097E+00_JPRB, 0.80632E+00_JPRB, 0.80769E+00_JPRB, 0.81573E+00_JPRB, 0.83806E+00_JPRB, 0.84888E+00_JPRB, 0.85593E+00_JPRB ,&
&  0.86061E+00_JPRB, 0.86635E+00_JPRB, 0.87819E+00_JPRB, 0.89836E+00_JPRB, 0.92956E+00_JPRB, 0.96542E+00_JPRB, 0.10034E+01_JPRB ,&
&  0.11050E+01_JPRB, 0.11213E+01_JPRB, 0.11884E+01_JPRB, 0.13653E+01_JPRB, 0.17599E+01_JPRB, 0.20112E+01_JPRB, 0.21476E+01_JPRB ,&
&  0.22927E+01_JPRB, 0.24009E+01_JPRB, 0.24565E+01_JPRB, 0.25354E+01_JPRB, 0.25494E+01_JPRB, 0.25586E+01_JPRB, 0.26113E+01_JPRB ,&
&  0.26148E+01_JPRB, 0.26727E+01_JPRB, 0.26964E+01_JPRB, 0.27317E+01_JPRB, 0.26760E+01_JPRB, 0.26458E+01_JPRB, 0.26229E+01_JPRB ,&
&  0.25839E+01_JPRB, 0.25012E+01_JPRB, 0.23305E+01_JPRB, 0.21598E+01_JPRB, 0.20010E+01_JPRB, 0.19066E+01_JPRB, 0.18291E+01_JPRB ,&
&  0.17418E+01_JPRB, 0.16778E+01_JPRB, 0.16125E+01_JPRB, 0.15293E+01_JPRB, 0.14213E+01_JPRB, 0.13229E+01_JPRB, 0.12334E+01_JPRB ,&
&  0.10925E+01_JPRB, 0.10408E+01_JPRB, 0.10158E+01_JPRB, 0.10061E+01_JPRB, 0.10172E+01_JPRB, 0.10454E+01_JPRB, 0.10279E+01_JPRB ,&
&  0.10648E+01_JPRB, 0.10842E+01_JPRB, 0.10851E+01_JPRB, 0.11087E+01_JPRB, 0.11455E+01_JPRB /)
ZBURD(:, 13, 10) = (/ &
&  0.12665E+01_JPRB, 0.12573E+01_JPRB, 0.12297E+01_JPRB, 0.12033E+01_JPRB, 0.11674E+01_JPRB, 0.10900E+01_JPRB, 0.10239E+01_JPRB ,&
&  0.10191E+01_JPRB, 0.98375E+00_JPRB, 0.96665E+00_JPRB, 0.90328E+00_JPRB, 0.87037E+00_JPRB, 0.85146E+00_JPRB, 0.83458E+00_JPRB ,&
&  0.83874E+00_JPRB, 0.86283E+00_JPRB, 0.88659E+00_JPRB, 0.90869E+00_JPRB, 0.92681E+00_JPRB, 0.92846E+00_JPRB, 0.92948E+00_JPRB ,&
&  0.92707E+00_JPRB, 0.93041E+00_JPRB, 0.94660E+00_JPRB, 0.95342E+00_JPRB, 0.95839E+00_JPRB, 0.95377E+00_JPRB, 0.95174E+00_JPRB ,&
&  0.93386E+00_JPRB, 0.93150E+00_JPRB, 0.95590E+00_JPRB, 0.94516E+00_JPRB, 0.93537E+00_JPRB, 0.92457E+00_JPRB, 0.90478E+00_JPRB ,&
&  0.86210E+00_JPRB, 0.84071E+00_JPRB, 0.84841E+00_JPRB, 0.85843E+00_JPRB, 0.86952E+00_JPRB, 0.87648E+00_JPRB, 0.88215E+00_JPRB ,&
&  0.89197E+00_JPRB, 0.90763E+00_JPRB, 0.93061E+00_JPRB, 0.96157E+00_JPRB, 0.10052E+01_JPRB, 0.10913E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.13878E+01_JPRB, 0.13678E+01_JPRB, 0.16784E+01_JPRB, 0.19475E+01_JPRB, 0.23203E+01_JPRB, 0.26360E+01_JPRB, 0.27786E+01_JPRB ,&
&  0.30547E+01_JPRB, 0.31297E+01_JPRB, 0.32095E+01_JPRB, 0.32306E+01_JPRB, 0.32251E+01_JPRB, 0.32424E+01_JPRB, 0.33198E+01_JPRB ,&
&  0.34010E+01_JPRB, 0.33051E+01_JPRB, 0.35094E+01_JPRB, 0.34243E+01_JPRB, 0.33171E+01_JPRB, 0.32730E+01_JPRB, 0.32312E+01_JPRB ,&
&  0.31503E+01_JPRB, 0.30106E+01_JPRB, 0.28204E+01_JPRB, 0.24797E+01_JPRB, 0.23689E+01_JPRB, 0.22632E+01_JPRB, 0.20474E+01_JPRB ,&
&  0.19193E+01_JPRB, 0.17699E+01_JPRB, 0.16132E+01_JPRB, 0.15833E+01_JPRB, 0.14205E+01_JPRB, 0.13046E+01_JPRB, 0.12331E+01_JPRB ,&
&  0.11379E+01_JPRB, 0.10868E+01_JPRB, 0.11252E+01_JPRB, 0.11494E+01_JPRB, 0.11506E+01_JPRB, 0.11307E+01_JPRB, 0.11786E+01_JPRB ,&
&  0.11406E+01_JPRB, 0.11596E+01_JPRB, 0.12283E+01_JPRB, 0.12426E+01_JPRB, 0.12550E+01_JPRB /)
ZBURD(:, 14, 10) = (/ &
&  0.13952E+01_JPRB, 0.13738E+01_JPRB, 0.13398E+01_JPRB, 0.13012E+01_JPRB, 0.12313E+01_JPRB, 0.11422E+01_JPRB, 0.10678E+01_JPRB ,&
&  0.10623E+01_JPRB, 0.10507E+01_JPRB, 0.10329E+01_JPRB, 0.10075E+01_JPRB, 0.97893E+00_JPRB, 0.93344E+00_JPRB, 0.86452E+00_JPRB ,&
&  0.86329E+00_JPRB, 0.91240E+00_JPRB, 0.97887E+00_JPRB, 0.10108E+01_JPRB, 0.10018E+01_JPRB, 0.99426E+00_JPRB, 0.10082E+01_JPRB ,&
&  0.10226E+01_JPRB, 0.10320E+01_JPRB, 0.10475E+01_JPRB, 0.10586E+01_JPRB, 0.10477E+01_JPRB, 0.10427E+01_JPRB, 0.10662E+01_JPRB ,&
&  0.10304E+01_JPRB, 0.10182E+01_JPRB, 0.10389E+01_JPRB, 0.10046E+01_JPRB, 0.10710E+01_JPRB, 0.10836E+01_JPRB, 0.10766E+01_JPRB ,&
&  0.10426E+01_JPRB, 0.99858E+00_JPRB, 0.96446E+00_JPRB, 0.95703E+00_JPRB, 0.95367E+00_JPRB, 0.94883E+00_JPRB, 0.95356E+00_JPRB ,&
&  0.97142E+00_JPRB, 0.10011E+01_JPRB, 0.10478E+01_JPRB, 0.11219E+01_JPRB, 0.12036E+01_JPRB, 0.13436E+01_JPRB, 0.15257E+01_JPRB ,&
&  0.17896E+01_JPRB, 0.21212E+01_JPRB, 0.25215E+01_JPRB, 0.26281E+01_JPRB, 0.29529E+01_JPRB, 0.32545E+01_JPRB, 0.33582E+01_JPRB ,&
&  0.36391E+01_JPRB, 0.37368E+01_JPRB, 0.40087E+01_JPRB, 0.41252E+01_JPRB, 0.42042E+01_JPRB, 0.42463E+01_JPRB, 0.42747E+01_JPRB ,&
&  0.42833E+01_JPRB, 0.42877E+01_JPRB, 0.43497E+01_JPRB, 0.42822E+01_JPRB, 0.41034E+01_JPRB, 0.39271E+01_JPRB, 0.37295E+01_JPRB ,&
&  0.35674E+01_JPRB, 0.33895E+01_JPRB, 0.32088E+01_JPRB, 0.30570E+01_JPRB, 0.28776E+01_JPRB, 0.26261E+01_JPRB, 0.23484E+01_JPRB ,&
&  0.20625E+01_JPRB, 0.17021E+01_JPRB, 0.15079E+01_JPRB, 0.14582E+01_JPRB, 0.14047E+01_JPRB, 0.12542E+01_JPRB, 0.11791E+01_JPRB ,&
&  0.11668E+01_JPRB, 0.12090E+01_JPRB, 0.13021E+01_JPRB, 0.13189E+01_JPRB, 0.13556E+01_JPRB, 0.13642E+01_JPRB, 0.12617E+01_JPRB ,&
&  0.11901E+01_JPRB, 0.12893E+01_JPRB, 0.13413E+01_JPRB, 0.13711E+01_JPRB, 0.13934E+01_JPRB /)
ZBURD(:, 15, 10) = (/ &
&  0.15168E+01_JPRB, 0.14846E+01_JPRB, 0.14521E+01_JPRB, 0.14146E+01_JPRB, 0.13368E+01_JPRB, 0.12348E+01_JPRB, 0.11585E+01_JPRB ,&
&  0.11278E+01_JPRB, 0.10954E+01_JPRB, 0.10676E+01_JPRB, 0.10473E+01_JPRB, 0.10282E+01_JPRB, 0.10150E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10041E+01_JPRB, 0.11058E+01_JPRB, 0.10908E+01_JPRB, 0.11260E+01_JPRB, 0.11535E+01_JPRB, 0.11198E+01_JPRB, 0.11217E+01_JPRB ,&
&  0.11880E+01_JPRB, 0.12046E+01_JPRB, 0.11927E+01_JPRB, 0.11649E+01_JPRB, 0.11493E+01_JPRB, 0.11714E+01_JPRB, 0.12311E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11753E+01_JPRB, 0.11480E+01_JPRB, 0.11498E+01_JPRB, 0.12086E+01_JPRB, 0.12821E+01_JPRB, 0.12784E+01_JPRB ,&
&  0.12348E+01_JPRB, 0.11919E+01_JPRB, 0.11506E+01_JPRB, 0.11195E+01_JPRB, 0.10902E+01_JPRB, 0.10657E+01_JPRB, 0.10517E+01_JPRB ,&
&  0.10557E+01_JPRB, 0.10808E+01_JPRB, 0.11258E+01_JPRB, 0.12115E+01_JPRB, 0.13247E+01_JPRB, 0.15761E+01_JPRB, 0.19847E+01_JPRB ,&
&  0.23775E+01_JPRB, 0.27331E+01_JPRB, 0.31350E+01_JPRB, 0.35962E+01_JPRB, 0.39315E+01_JPRB, 0.40785E+01_JPRB, 0.41599E+01_JPRB ,&
&  0.43789E+01_JPRB, 0.43865E+01_JPRB, 0.47847E+01_JPRB, 0.47976E+01_JPRB, 0.48568E+01_JPRB, 0.50906E+01_JPRB, 0.50797E+01_JPRB ,&
&  0.49616E+01_JPRB, 0.47230E+01_JPRB, 0.48697E+01_JPRB, 0.48020E+01_JPRB, 0.46443E+01_JPRB, 0.44772E+01_JPRB, 0.42264E+01_JPRB ,&
&  0.39791E+01_JPRB, 0.37266E+01_JPRB, 0.32260E+01_JPRB, 0.28910E+01_JPRB, 0.24942E+01_JPRB, 0.25175E+01_JPRB, 0.22616E+01_JPRB ,&
&  0.19660E+01_JPRB, 0.15732E+01_JPRB, 0.15174E+01_JPRB, 0.14746E+01_JPRB, 0.14870E+01_JPRB, 0.14080E+01_JPRB, 0.12803E+01_JPRB ,&
&  0.12781E+01_JPRB, 0.13880E+01_JPRB, 0.14173E+01_JPRB, 0.14615E+01_JPRB, 0.15361E+01_JPRB, 0.15893E+01_JPRB, 0.13586E+01_JPRB ,&
&  0.13503E+01_JPRB, 0.14803E+01_JPRB, 0.15221E+01_JPRB, 0.15401E+01_JPRB, 0.15387E+01_JPRB /)
ZBURD(:, 16, 10) = (/ &
&  0.15455E+01_JPRB, 0.14996E+01_JPRB, 0.14569E+01_JPRB, 0.14159E+01_JPRB, 0.13629E+01_JPRB, 0.13016E+01_JPRB, 0.12274E+01_JPRB ,&
&  0.11630E+01_JPRB, 0.11135E+01_JPRB, 0.10888E+01_JPRB, 0.10810E+01_JPRB, 0.10764E+01_JPRB, 0.10713E+01_JPRB, 0.11526E+01_JPRB ,&
&  0.11156E+01_JPRB, 0.12713E+01_JPRB, 0.11661E+01_JPRB, 0.12388E+01_JPRB, 0.14940E+01_JPRB, 0.13367E+01_JPRB, 0.13243E+01_JPRB ,&
&  0.13330E+01_JPRB, 0.13834E+01_JPRB, 0.13628E+01_JPRB, 0.13278E+01_JPRB, 0.13133E+01_JPRB, 0.13474E+01_JPRB, 0.14738E+01_JPRB ,&
&  0.14959E+01_JPRB, 0.13677E+01_JPRB, 0.12979E+01_JPRB, 0.12813E+01_JPRB, 0.13096E+01_JPRB, 0.13564E+01_JPRB, 0.13825E+01_JPRB ,&
&  0.13243E+01_JPRB, 0.12770E+01_JPRB, 0.12455E+01_JPRB, 0.12188E+01_JPRB, 0.11923E+01_JPRB, 0.11748E+01_JPRB, 0.11680E+01_JPRB ,&
&  0.11725E+01_JPRB, 0.11974E+01_JPRB, 0.12406E+01_JPRB, 0.13375E+01_JPRB, 0.14295E+01_JPRB, 0.17936E+01_JPRB, 0.25791E+01_JPRB ,&
&  0.35056E+01_JPRB, 0.36769E+01_JPRB, 0.41332E+01_JPRB, 0.50804E+01_JPRB, 0.53872E+01_JPRB, 0.54628E+01_JPRB, 0.54916E+01_JPRB ,&
&  0.55646E+01_JPRB, 0.55517E+01_JPRB, 0.55737E+01_JPRB, 0.58390E+01_JPRB, 0.58618E+01_JPRB, 0.59309E+01_JPRB, 0.57394E+01_JPRB ,&
&  0.53487E+01_JPRB, 0.52018E+01_JPRB, 0.51595E+01_JPRB, 0.49447E+01_JPRB, 0.46512E+01_JPRB, 0.45917E+01_JPRB, 0.44635E+01_JPRB ,&
&  0.41958E+01_JPRB, 0.37336E+01_JPRB, 0.28151E+01_JPRB, 0.25673E+01_JPRB, 0.19210E+01_JPRB, 0.19406E+01_JPRB, 0.23428E+01_JPRB ,&
&  0.22128E+01_JPRB, 0.19564E+01_JPRB, 0.17927E+01_JPRB, 0.17100E+01_JPRB, 0.16727E+01_JPRB, 0.17223E+01_JPRB, 0.16065E+01_JPRB ,&
&  0.14536E+01_JPRB, 0.15012E+01_JPRB, 0.15828E+01_JPRB, 0.16913E+01_JPRB, 0.18115E+01_JPRB, 0.18345E+01_JPRB, 0.16476E+01_JPRB ,&
&  0.17008E+01_JPRB, 0.17079E+01_JPRB, 0.16888E+01_JPRB, 0.16652E+01_JPRB, 0.16116E+01_JPRB /)
ZBURD(:, 17, 10) = (/ &
&  0.14607E+01_JPRB, 0.14151E+01_JPRB, 0.13670E+01_JPRB, 0.13522E+01_JPRB, 0.13509E+01_JPRB, 0.13317E+01_JPRB, 0.12728E+01_JPRB ,&
&  0.11884E+01_JPRB, 0.11284E+01_JPRB, 0.11032E+01_JPRB, 0.11100E+01_JPRB, 0.11284E+01_JPRB, 0.11374E+01_JPRB, 0.11858E+01_JPRB ,&
&  0.13639E+01_JPRB, 0.15195E+01_JPRB, 0.13797E+01_JPRB, 0.12279E+01_JPRB, 0.14194E+01_JPRB, 0.13699E+01_JPRB, 0.14273E+01_JPRB ,&
&  0.13884E+01_JPRB, 0.15473E+01_JPRB, 0.15710E+01_JPRB, 0.16153E+01_JPRB, 0.16794E+01_JPRB, 0.18235E+01_JPRB, 0.20742E+01_JPRB ,&
&  0.21073E+01_JPRB, 0.18634E+01_JPRB, 0.16859E+01_JPRB, 0.16383E+01_JPRB, 0.16748E+01_JPRB, 0.15266E+01_JPRB, 0.14965E+01_JPRB ,&
&  0.14345E+01_JPRB, 0.13884E+01_JPRB, 0.13536E+01_JPRB, 0.13431E+01_JPRB, 0.13376E+01_JPRB, 0.13348E+01_JPRB, 0.13429E+01_JPRB ,&
&  0.13668E+01_JPRB, 0.13941E+01_JPRB, 0.14073E+01_JPRB, 0.14608E+01_JPRB, 0.16443E+01_JPRB, 0.21533E+01_JPRB, 0.29796E+01_JPRB ,&
&  0.35927E+01_JPRB, 0.38044E+01_JPRB, 0.41811E+01_JPRB, 0.50660E+01_JPRB, 0.56878E+01_JPRB, 0.60568E+01_JPRB, 0.64941E+01_JPRB ,&
&  0.66805E+01_JPRB, 0.67279E+01_JPRB, 0.67474E+01_JPRB, 0.63752E+01_JPRB, 0.57621E+01_JPRB, 0.52656E+01_JPRB, 0.49751E+01_JPRB ,&
&  0.46640E+01_JPRB, 0.45529E+01_JPRB, 0.46666E+01_JPRB, 0.45222E+01_JPRB, 0.44518E+01_JPRB, 0.41628E+01_JPRB, 0.43342E+01_JPRB ,&
&  0.42217E+01_JPRB, 0.25163E+01_JPRB, 0.19797E+01_JPRB, 0.22418E+01_JPRB, 0.17899E+01_JPRB, 0.18401E+01_JPRB, 0.21170E+01_JPRB ,&
&  0.20298E+01_JPRB, 0.19675E+01_JPRB, 0.20159E+01_JPRB, 0.19800E+01_JPRB, 0.19444E+01_JPRB, 0.20804E+01_JPRB, 0.20701E+01_JPRB ,&
&  0.19610E+01_JPRB, 0.18373E+01_JPRB, 0.19332E+01_JPRB, 0.20522E+01_JPRB, 0.21464E+01_JPRB, 0.20942E+01_JPRB, 0.19902E+01_JPRB ,&
&  0.19171E+01_JPRB, 0.18493E+01_JPRB, 0.17761E+01_JPRB, 0.16627E+01_JPRB, 0.15382E+01_JPRB /)
ZBURD(:, 18, 10) = (/ &
&  0.14253E+01_JPRB, 0.13967E+01_JPRB, 0.13533E+01_JPRB, 0.13649E+01_JPRB, 0.13905E+01_JPRB, 0.13755E+01_JPRB, 0.13076E+01_JPRB ,&
&  0.11959E+01_JPRB, 0.11334E+01_JPRB, 0.11068E+01_JPRB, 0.11156E+01_JPRB, 0.11431E+01_JPRB, 0.11675E+01_JPRB, 0.11673E+01_JPRB ,&
&  0.11436E+01_JPRB, 0.14249E+01_JPRB, 0.12790E+01_JPRB, 0.12250E+01_JPRB, 0.10984E+01_JPRB, 0.12414E+01_JPRB, 0.14381E+01_JPRB ,&
&  0.15729E+01_JPRB, 0.16693E+01_JPRB, 0.18730E+01_JPRB, 0.21624E+01_JPRB, 0.24492E+01_JPRB, 0.26906E+01_JPRB, 0.30974E+01_JPRB ,&
&  0.30376E+01_JPRB, 0.27700E+01_JPRB, 0.22793E+01_JPRB, 0.21564E+01_JPRB, 0.20222E+01_JPRB, 0.17761E+01_JPRB, 0.17790E+01_JPRB ,&
&  0.16886E+01_JPRB, 0.16396E+01_JPRB, 0.15950E+01_JPRB, 0.15699E+01_JPRB, 0.15449E+01_JPRB, 0.15001E+01_JPRB, 0.14862E+01_JPRB ,&
&  0.14977E+01_JPRB, 0.15226E+01_JPRB, 0.15440E+01_JPRB, 0.16328E+01_JPRB, 0.20922E+01_JPRB, 0.27021E+01_JPRB, 0.31758E+01_JPRB ,&
&  0.37234E+01_JPRB, 0.33659E+01_JPRB, 0.31510E+01_JPRB, 0.44267E+01_JPRB, 0.68339E+01_JPRB, 0.76228E+01_JPRB, 0.73806E+01_JPRB ,&
&  0.81879E+01_JPRB, 0.77059E+01_JPRB, 0.67299E+01_JPRB, 0.58657E+01_JPRB, 0.52997E+01_JPRB, 0.47828E+01_JPRB, 0.45361E+01_JPRB ,&
&  0.41707E+01_JPRB, 0.40775E+01_JPRB, 0.42891E+01_JPRB, 0.44306E+01_JPRB, 0.41937E+01_JPRB, 0.39201E+01_JPRB, 0.39340E+01_JPRB ,&
&  0.37635E+01_JPRB, 0.31584E+01_JPRB, 0.21628E+01_JPRB, 0.21145E+01_JPRB, 0.16489E+01_JPRB, 0.16429E+01_JPRB, 0.19367E+01_JPRB ,&
&  0.18990E+01_JPRB, 0.20346E+01_JPRB, 0.20910E+01_JPRB, 0.20356E+01_JPRB, 0.22632E+01_JPRB, 0.23802E+01_JPRB, 0.25881E+01_JPRB ,&
&  0.27057E+01_JPRB, 0.23346E+01_JPRB, 0.24451E+01_JPRB, 0.25135E+01_JPRB, 0.24739E+01_JPRB, 0.22914E+01_JPRB, 0.21093E+01_JPRB ,&
&  0.19838E+01_JPRB, 0.18784E+01_JPRB, 0.17510E+01_JPRB, 0.15898E+01_JPRB, 0.14784E+01_JPRB /)
ZBURD(:, 19, 10) = (/ &
&  0.14152E+01_JPRB, 0.14188E+01_JPRB, 0.13841E+01_JPRB, 0.13975E+01_JPRB, 0.14022E+01_JPRB, 0.13659E+01_JPRB, 0.13074E+01_JPRB ,&
&  0.12415E+01_JPRB, 0.12047E+01_JPRB, 0.11670E+01_JPRB, 0.11516E+01_JPRB, 0.11605E+01_JPRB, 0.11583E+01_JPRB, 0.11276E+01_JPRB ,&
&  0.10969E+01_JPRB, 0.12201E+01_JPRB, 0.10880E+01_JPRB, 0.98733E+00_JPRB, 0.93056E+00_JPRB, 0.10379E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.15771E+01_JPRB, 0.19858E+01_JPRB, 0.25714E+01_JPRB, 0.30499E+01_JPRB, 0.37122E+01_JPRB, 0.42471E+01_JPRB, 0.45335E+01_JPRB ,&
&  0.46029E+01_JPRB, 0.34999E+01_JPRB, 0.31639E+01_JPRB, 0.27306E+01_JPRB, 0.24589E+01_JPRB, 0.22167E+01_JPRB, 0.20650E+01_JPRB ,&
&  0.19169E+01_JPRB, 0.18202E+01_JPRB, 0.17410E+01_JPRB, 0.16723E+01_JPRB, 0.16436E+01_JPRB, 0.16327E+01_JPRB, 0.16569E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.16385E+01_JPRB, 0.17106E+01_JPRB, 0.20604E+01_JPRB, 0.27864E+01_JPRB, 0.33960E+01_JPRB, 0.37279E+01_JPRB ,&
&  0.35031E+01_JPRB, 0.34348E+01_JPRB, 0.56467E+01_JPRB, 0.65207E+01_JPRB, 0.66788E+01_JPRB, 0.81687E+01_JPRB, 0.89390E+01_JPRB ,&
&  0.83274E+01_JPRB, 0.66354E+01_JPRB, 0.55127E+01_JPRB, 0.46666E+01_JPRB, 0.52443E+01_JPRB, 0.47199E+01_JPRB, 0.42356E+01_JPRB ,&
&  0.38605E+01_JPRB, 0.40919E+01_JPRB, 0.40463E+01_JPRB, 0.40531E+01_JPRB, 0.40425E+01_JPRB, 0.40108E+01_JPRB, 0.34101E+01_JPRB ,&
&  0.29824E+01_JPRB, 0.35660E+01_JPRB, 0.28732E+01_JPRB, 0.21972E+01_JPRB, 0.18310E+01_JPRB, 0.18887E+01_JPRB, 0.20245E+01_JPRB ,&
&  0.21139E+01_JPRB, 0.21801E+01_JPRB, 0.21726E+01_JPRB, 0.23281E+01_JPRB, 0.27097E+01_JPRB, 0.28789E+01_JPRB, 0.29852E+01_JPRB ,&
&  0.29320E+01_JPRB, 0.31116E+01_JPRB, 0.30184E+01_JPRB, 0.28394E+01_JPRB, 0.25453E+01_JPRB, 0.22185E+01_JPRB, 0.20105E+01_JPRB ,&
&  0.18971E+01_JPRB, 0.18006E+01_JPRB, 0.16374E+01_JPRB, 0.14849E+01_JPRB, 0.14036E+01_JPRB /)
ZBURD(:, 20, 10) = (/ &
&  0.13855E+01_JPRB, 0.14098E+01_JPRB, 0.14208E+01_JPRB, 0.14243E+01_JPRB, 0.14107E+01_JPRB, 0.13733E+01_JPRB, 0.13412E+01_JPRB ,&
&  0.12910E+01_JPRB, 0.12282E+01_JPRB, 0.12028E+01_JPRB, 0.11919E+01_JPRB, 0.11857E+01_JPRB, 0.11617E+01_JPRB, 0.11238E+01_JPRB ,&
&  0.11062E+01_JPRB, 0.10800E+01_JPRB, 0.93985E+00_JPRB, 0.96285E+00_JPRB, 0.10122E+01_JPRB, 0.10475E+01_JPRB, 0.12633E+01_JPRB ,&
&  0.16260E+01_JPRB, 0.23623E+01_JPRB, 0.31477E+01_JPRB, 0.41784E+01_JPRB, 0.50517E+01_JPRB, 0.53354E+01_JPRB, 0.52406E+01_JPRB ,&
&  0.46255E+01_JPRB, 0.41871E+01_JPRB, 0.36307E+01_JPRB, 0.31868E+01_JPRB, 0.28211E+01_JPRB, 0.25652E+01_JPRB, 0.23037E+01_JPRB ,&
&  0.20816E+01_JPRB, 0.19093E+01_JPRB, 0.18069E+01_JPRB, 0.17112E+01_JPRB, 0.16825E+01_JPRB, 0.16937E+01_JPRB, 0.17135E+01_JPRB ,&
&  0.16843E+01_JPRB, 0.17010E+01_JPRB, 0.19390E+01_JPRB, 0.24706E+01_JPRB, 0.27773E+01_JPRB, 0.31600E+01_JPRB, 0.34469E+01_JPRB ,&
&  0.46872E+01_JPRB, 0.51645E+01_JPRB, 0.52739E+01_JPRB, 0.70963E+01_JPRB, 0.53013E+01_JPRB, 0.67490E+01_JPRB, 0.84203E+01_JPRB ,&
&  0.75177E+01_JPRB, 0.65800E+01_JPRB, 0.63617E+01_JPRB, 0.49729E+01_JPRB, 0.39820E+01_JPRB, 0.53403E+01_JPRB, 0.45107E+01_JPRB ,&
&  0.39973E+01_JPRB, 0.40671E+01_JPRB, 0.40046E+01_JPRB, 0.43797E+01_JPRB, 0.33904E+01_JPRB, 0.29115E+01_JPRB, 0.23084E+01_JPRB ,&
&  0.24033E+01_JPRB, 0.22166E+01_JPRB, 0.26021E+01_JPRB, 0.23058E+01_JPRB, 0.23180E+01_JPRB, 0.22188E+01_JPRB, 0.21368E+01_JPRB ,&
&  0.22624E+01_JPRB, 0.22879E+01_JPRB, 0.23025E+01_JPRB, 0.27224E+01_JPRB, 0.33690E+01_JPRB, 0.30340E+01_JPRB, 0.33334E+01_JPRB ,&
&  0.34235E+01_JPRB, 0.33033E+01_JPRB, 0.29787E+01_JPRB, 0.27233E+01_JPRB, 0.23196E+01_JPRB, 0.20442E+01_JPRB, 0.18982E+01_JPRB ,&
&  0.17895E+01_JPRB, 0.16252E+01_JPRB, 0.14616E+01_JPRB, 0.13928E+01_JPRB, 0.13430E+01_JPRB /)
ZBURD(:, 21, 10) = (/ &
&  0.13396E+01_JPRB, 0.13427E+01_JPRB, 0.13644E+01_JPRB, 0.13741E+01_JPRB, 0.13878E+01_JPRB, 0.13624E+01_JPRB, 0.13301E+01_JPRB ,&
&  0.12640E+01_JPRB, 0.12106E+01_JPRB, 0.11968E+01_JPRB, 0.11820E+01_JPRB, 0.11713E+01_JPRB, 0.11553E+01_JPRB, 0.11082E+01_JPRB ,&
&  0.10983E+01_JPRB, 0.10993E+01_JPRB, 0.95729E+00_JPRB, 0.93358E+00_JPRB, 0.10458E+01_JPRB, 0.10792E+01_JPRB, 0.12448E+01_JPRB ,&
&  0.19117E+01_JPRB, 0.27956E+01_JPRB, 0.36282E+01_JPRB, 0.46888E+01_JPRB, 0.52973E+01_JPRB, 0.50978E+01_JPRB, 0.46867E+01_JPRB ,&
&  0.46353E+01_JPRB, 0.38042E+01_JPRB, 0.33681E+01_JPRB, 0.31034E+01_JPRB, 0.27959E+01_JPRB, 0.25165E+01_JPRB, 0.22839E+01_JPRB ,&
&  0.20577E+01_JPRB, 0.18508E+01_JPRB, 0.17206E+01_JPRB, 0.16068E+01_JPRB, 0.15785E+01_JPRB, 0.16085E+01_JPRB, 0.16570E+01_JPRB ,&
&  0.17073E+01_JPRB, 0.18475E+01_JPRB, 0.22354E+01_JPRB, 0.27122E+01_JPRB, 0.30738E+01_JPRB, 0.33765E+01_JPRB, 0.41381E+01_JPRB ,&
&  0.45588E+01_JPRB, 0.46064E+01_JPRB, 0.54529E+01_JPRB, 0.65467E+01_JPRB, 0.75390E+01_JPRB, 0.70044E+01_JPRB, 0.76282E+01_JPRB ,&
&  0.53616E+01_JPRB, 0.43100E+01_JPRB, 0.37520E+01_JPRB, 0.32508E+01_JPRB, 0.36353E+01_JPRB, 0.56392E+01_JPRB, 0.51338E+01_JPRB ,&
&  0.47610E+01_JPRB, 0.43986E+01_JPRB, 0.43813E+01_JPRB, 0.39688E+01_JPRB, 0.34740E+01_JPRB, 0.24187E+01_JPRB, 0.31557E+01_JPRB ,&
&  0.33901E+01_JPRB, 0.33974E+01_JPRB, 0.30189E+01_JPRB, 0.25141E+01_JPRB, 0.20826E+01_JPRB, 0.24479E+01_JPRB, 0.25519E+01_JPRB ,&
&  0.26081E+01_JPRB, 0.25785E+01_JPRB, 0.33119E+01_JPRB, 0.37581E+01_JPRB, 0.37904E+01_JPRB, 0.34602E+01_JPRB, 0.37436E+01_JPRB ,&
&  0.34105E+01_JPRB, 0.32877E+01_JPRB, 0.29900E+01_JPRB, 0.24912E+01_JPRB, 0.21213E+01_JPRB, 0.19136E+01_JPRB, 0.17559E+01_JPRB ,&
&  0.16017E+01_JPRB, 0.14281E+01_JPRB, 0.13381E+01_JPRB, 0.12892E+01_JPRB, 0.12807E+01_JPRB /)
ZBURD(:, 22, 10) = (/ &
&  0.12805E+01_JPRB, 0.12774E+01_JPRB, 0.12718E+01_JPRB, 0.12931E+01_JPRB, 0.13017E+01_JPRB, 0.12722E+01_JPRB, 0.12421E+01_JPRB ,&
&  0.12052E+01_JPRB, 0.11706E+01_JPRB, 0.11374E+01_JPRB, 0.11152E+01_JPRB, 0.10989E+01_JPRB, 0.11045E+01_JPRB, 0.10922E+01_JPRB ,&
&  0.11000E+01_JPRB, 0.12173E+01_JPRB, 0.10526E+01_JPRB, 0.10204E+01_JPRB, 0.10731E+01_JPRB, 0.11275E+01_JPRB, 0.12853E+01_JPRB ,&
&  0.20046E+01_JPRB, 0.28384E+01_JPRB, 0.34458E+01_JPRB, 0.42949E+01_JPRB, 0.50360E+01_JPRB, 0.46325E+01_JPRB, 0.44321E+01_JPRB ,&
&  0.42590E+01_JPRB, 0.36070E+01_JPRB, 0.31821E+01_JPRB, 0.28508E+01_JPRB, 0.25810E+01_JPRB, 0.23813E+01_JPRB, 0.21689E+01_JPRB ,&
&  0.19308E+01_JPRB, 0.17484E+01_JPRB, 0.16106E+01_JPRB, 0.15081E+01_JPRB, 0.14428E+01_JPRB, 0.15397E+01_JPRB, 0.16817E+01_JPRB ,&
&  0.18501E+01_JPRB, 0.21621E+01_JPRB, 0.25090E+01_JPRB, 0.29947E+01_JPRB, 0.35406E+01_JPRB, 0.35940E+01_JPRB, 0.47380E+01_JPRB ,&
&  0.46539E+01_JPRB, 0.47706E+01_JPRB, 0.53666E+01_JPRB, 0.71520E+01_JPRB, 0.68591E+01_JPRB, 0.63990E+01_JPRB, 0.65870E+01_JPRB ,&
&  0.44643E+01_JPRB, 0.37821E+01_JPRB, 0.39725E+01_JPRB, 0.40694E+01_JPRB, 0.35343E+01_JPRB, 0.38570E+01_JPRB, 0.58400E+01_JPRB ,&
&  0.46277E+01_JPRB, 0.44584E+01_JPRB, 0.45584E+01_JPRB, 0.50080E+01_JPRB, 0.23487E+01_JPRB, 0.16708E+01_JPRB, 0.26747E+01_JPRB ,&
&  0.28969E+01_JPRB, 0.18488E+01_JPRB, 0.13632E+01_JPRB, 0.17958E+01_JPRB, 0.13977E+01_JPRB, 0.17482E+01_JPRB, 0.26677E+01_JPRB ,&
&  0.29429E+01_JPRB, 0.32179E+01_JPRB, 0.48884E+01_JPRB, 0.46287E+01_JPRB, 0.40861E+01_JPRB, 0.41580E+01_JPRB, 0.38576E+01_JPRB ,&
&  0.35237E+01_JPRB, 0.28965E+01_JPRB, 0.26691E+01_JPRB, 0.22143E+01_JPRB, 0.19180E+01_JPRB, 0.16972E+01_JPRB, 0.15317E+01_JPRB ,&
&  0.13751E+01_JPRB, 0.12869E+01_JPRB, 0.12782E+01_JPRB, 0.12766E+01_JPRB, 0.12620E+01_JPRB /)
ZBURD(:, 23, 10) = (/ &
&  0.12433E+01_JPRB, 0.12076E+01_JPRB, 0.11764E+01_JPRB, 0.12039E+01_JPRB, 0.11998E+01_JPRB, 0.11622E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11749E+01_JPRB, 0.11188E+01_JPRB, 0.10694E+01_JPRB, 0.10470E+01_JPRB, 0.10395E+01_JPRB, 0.10862E+01_JPRB, 0.11126E+01_JPRB ,&
&  0.11413E+01_JPRB, 0.12140E+01_JPRB, 0.13226E+01_JPRB, 0.12708E+01_JPRB, 0.12300E+01_JPRB, 0.12450E+01_JPRB, 0.15057E+01_JPRB ,&
&  0.22176E+01_JPRB, 0.28120E+01_JPRB, 0.32186E+01_JPRB, 0.36217E+01_JPRB, 0.40190E+01_JPRB, 0.39820E+01_JPRB, 0.40960E+01_JPRB ,&
&  0.36926E+01_JPRB, 0.31801E+01_JPRB, 0.28000E+01_JPRB, 0.24698E+01_JPRB, 0.22916E+01_JPRB, 0.21242E+01_JPRB, 0.19428E+01_JPRB ,&
&  0.17150E+01_JPRB, 0.15527E+01_JPRB, 0.14769E+01_JPRB, 0.14124E+01_JPRB, 0.14231E+01_JPRB, 0.15437E+01_JPRB, 0.17280E+01_JPRB ,&
&  0.20384E+01_JPRB, 0.24499E+01_JPRB, 0.27628E+01_JPRB, 0.33219E+01_JPRB, 0.38316E+01_JPRB, 0.38389E+01_JPRB, 0.36866E+01_JPRB ,&
&  0.36446E+01_JPRB, 0.37302E+01_JPRB, 0.52617E+01_JPRB, 0.62567E+01_JPRB, 0.63945E+01_JPRB, 0.64960E+01_JPRB, 0.61526E+01_JPRB ,&
&  0.52247E+01_JPRB, 0.47960E+01_JPRB, 0.41009E+01_JPRB, 0.43426E+01_JPRB, 0.39647E+01_JPRB, 0.32739E+01_JPRB, 0.40107E+01_JPRB ,&
&  0.44023E+01_JPRB, 0.40545E+01_JPRB, 0.33244E+01_JPRB, 0.27411E+01_JPRB, 0.23824E+01_JPRB, 0.18606E+01_JPRB, 0.13442E+01_JPRB ,&
&  0.12263E+01_JPRB, 0.12028E+01_JPRB, 0.11174E+01_JPRB, 0.11548E+01_JPRB, 0.11537E+01_JPRB, 0.16952E+01_JPRB, 0.25488E+01_JPRB ,&
&  0.37722E+01_JPRB, 0.46024E+01_JPRB, 0.53210E+01_JPRB, 0.50629E+01_JPRB, 0.43101E+01_JPRB, 0.43847E+01_JPRB, 0.39772E+01_JPRB ,&
&  0.29972E+01_JPRB, 0.25998E+01_JPRB, 0.22920E+01_JPRB, 0.19434E+01_JPRB, 0.16882E+01_JPRB, 0.14709E+01_JPRB, 0.13139E+01_JPRB ,&
&  0.12416E+01_JPRB, 0.12390E+01_JPRB, 0.12475E+01_JPRB, 0.12509E+01_JPRB, 0.12169E+01_JPRB /)
ZBURD(:, 24, 10) = (/ &
&  0.12091E+01_JPRB, 0.11720E+01_JPRB, 0.11367E+01_JPRB, 0.11171E+01_JPRB, 0.10825E+01_JPRB, 0.10792E+01_JPRB, 0.11470E+01_JPRB ,&
&  0.11597E+01_JPRB, 0.10747E+01_JPRB, 0.10146E+01_JPRB, 0.10005E+01_JPRB, 0.10480E+01_JPRB, 0.11116E+01_JPRB, 0.11324E+01_JPRB ,&
&  0.11736E+01_JPRB, 0.12256E+01_JPRB, 0.14638E+01_JPRB, 0.14663E+01_JPRB, 0.15484E+01_JPRB, 0.15426E+01_JPRB, 0.19328E+01_JPRB ,&
&  0.24047E+01_JPRB, 0.27979E+01_JPRB, 0.31929E+01_JPRB, 0.33405E+01_JPRB, 0.37147E+01_JPRB, 0.37840E+01_JPRB, 0.36632E+01_JPRB ,&
&  0.31645E+01_JPRB, 0.27567E+01_JPRB, 0.24218E+01_JPRB, 0.21677E+01_JPRB, 0.19660E+01_JPRB, 0.17378E+01_JPRB, 0.16018E+01_JPRB ,&
&  0.14799E+01_JPRB, 0.14414E+01_JPRB, 0.14191E+01_JPRB, 0.14494E+01_JPRB, 0.15586E+01_JPRB, 0.17004E+01_JPRB, 0.19299E+01_JPRB ,&
&  0.22563E+01_JPRB, 0.26493E+01_JPRB, 0.29930E+01_JPRB, 0.35613E+01_JPRB, 0.32273E+01_JPRB, 0.28841E+01_JPRB, 0.34365E+01_JPRB ,&
&  0.39848E+01_JPRB, 0.44831E+01_JPRB, 0.46732E+01_JPRB, 0.62096E+01_JPRB, 0.63688E+01_JPRB, 0.59040E+01_JPRB, 0.57161E+01_JPRB ,&
&  0.49270E+01_JPRB, 0.47373E+01_JPRB, 0.38382E+01_JPRB, 0.39377E+01_JPRB, 0.43929E+01_JPRB, 0.36152E+01_JPRB, 0.35263E+01_JPRB ,&
&  0.40250E+01_JPRB, 0.37590E+01_JPRB, 0.33179E+01_JPRB, 0.27421E+01_JPRB, 0.32809E+01_JPRB, 0.31555E+01_JPRB, 0.15033E+01_JPRB ,&
&  0.12346E+01_JPRB, 0.11552E+01_JPRB, 0.11196E+01_JPRB, 0.10552E+01_JPRB, 0.11091E+01_JPRB, 0.17124E+01_JPRB, 0.36615E+01_JPRB ,&
&  0.38539E+01_JPRB, 0.59089E+01_JPRB, 0.57887E+01_JPRB, 0.53935E+01_JPRB, 0.44532E+01_JPRB, 0.45897E+01_JPRB, 0.35297E+01_JPRB ,&
&  0.29395E+01_JPRB, 0.25278E+01_JPRB, 0.20898E+01_JPRB, 0.18119E+01_JPRB, 0.15516E+01_JPRB, 0.13363E+01_JPRB, 0.12537E+01_JPRB ,&
&  0.12143E+01_JPRB, 0.11771E+01_JPRB, 0.11664E+01_JPRB, 0.11705E+01_JPRB, 0.11793E+01_JPRB /)
ZBURD(:, 25, 10) = (/ &
&  0.11870E+01_JPRB, 0.11439E+01_JPRB, 0.11101E+01_JPRB, 0.10925E+01_JPRB, 0.10628E+01_JPRB, 0.11019E+01_JPRB, 0.11399E+01_JPRB ,&
&  0.11312E+01_JPRB, 0.10591E+01_JPRB, 0.10122E+01_JPRB, 0.10258E+01_JPRB, 0.10906E+01_JPRB, 0.11622E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.12658E+01_JPRB, 0.13485E+01_JPRB, 0.15269E+01_JPRB, 0.17577E+01_JPRB, 0.19080E+01_JPRB, 0.19408E+01_JPRB, 0.23271E+01_JPRB ,&
&  0.24855E+01_JPRB, 0.26785E+01_JPRB, 0.31408E+01_JPRB, 0.32726E+01_JPRB, 0.31996E+01_JPRB, 0.31739E+01_JPRB, 0.28334E+01_JPRB ,&
&  0.25745E+01_JPRB, 0.23414E+01_JPRB, 0.20768E+01_JPRB, 0.18532E+01_JPRB, 0.17118E+01_JPRB, 0.15615E+01_JPRB, 0.15035E+01_JPRB ,&
&  0.14460E+01_JPRB, 0.14365E+01_JPRB, 0.15041E+01_JPRB, 0.15750E+01_JPRB, 0.17254E+01_JPRB, 0.18704E+01_JPRB, 0.21093E+01_JPRB ,&
&  0.24396E+01_JPRB, 0.27631E+01_JPRB, 0.30829E+01_JPRB, 0.35513E+01_JPRB, 0.29822E+01_JPRB, 0.35952E+01_JPRB, 0.38221E+01_JPRB ,&
&  0.41309E+01_JPRB, 0.42911E+01_JPRB, 0.42184E+01_JPRB, 0.52242E+01_JPRB, 0.58671E+01_JPRB, 0.54948E+01_JPRB, 0.49072E+01_JPRB ,&
&  0.44977E+01_JPRB, 0.38344E+01_JPRB, 0.36075E+01_JPRB, 0.38825E+01_JPRB, 0.41984E+01_JPRB, 0.48268E+01_JPRB, 0.34369E+01_JPRB ,&
&  0.31146E+01_JPRB, 0.35588E+01_JPRB, 0.33865E+01_JPRB, 0.29658E+01_JPRB, 0.34379E+01_JPRB, 0.43050E+01_JPRB, 0.31117E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.10389E+01_JPRB, 0.11475E+01_JPRB, 0.11608E+01_JPRB, 0.10474E+01_JPRB, 0.16215E+01_JPRB, 0.62533E+01_JPRB ,&
&  0.48773E+01_JPRB, 0.68056E+01_JPRB, 0.57947E+01_JPRB, 0.50875E+01_JPRB, 0.45134E+01_JPRB, 0.44023E+01_JPRB, 0.37518E+01_JPRB ,&
&  0.28090E+01_JPRB, 0.23401E+01_JPRB, 0.19513E+01_JPRB, 0.16591E+01_JPRB, 0.14117E+01_JPRB, 0.12710E+01_JPRB, 0.12073E+01_JPRB ,&
&  0.11629E+01_JPRB, 0.11465E+01_JPRB, 0.11412E+01_JPRB, 0.11411E+01_JPRB, 0.11654E+01_JPRB /)
ZBURD(:, 26, 10) = (/ &
&  0.11754E+01_JPRB, 0.10984E+01_JPRB, 0.10644E+01_JPRB, 0.10460E+01_JPRB, 0.10959E+01_JPRB, 0.11464E+01_JPRB, 0.11243E+01_JPRB ,&
&  0.11382E+01_JPRB, 0.11222E+01_JPRB, 0.11219E+01_JPRB, 0.11197E+01_JPRB, 0.11757E+01_JPRB, 0.12860E+01_JPRB, 0.13717E+01_JPRB ,&
&  0.14827E+01_JPRB, 0.16117E+01_JPRB, 0.17969E+01_JPRB, 0.20712E+01_JPRB, 0.23228E+01_JPRB, 0.22693E+01_JPRB, 0.22982E+01_JPRB ,&
&  0.24180E+01_JPRB, 0.25810E+01_JPRB, 0.26156E+01_JPRB, 0.28830E+01_JPRB, 0.29201E+01_JPRB, 0.30393E+01_JPRB, 0.25759E+01_JPRB ,&
&  0.23715E+01_JPRB, 0.20927E+01_JPRB, 0.17910E+01_JPRB, 0.16088E+01_JPRB, 0.14626E+01_JPRB, 0.14503E+01_JPRB, 0.15103E+01_JPRB ,&
&  0.14869E+01_JPRB, 0.15377E+01_JPRB, 0.16386E+01_JPRB, 0.17512E+01_JPRB, 0.19125E+01_JPRB, 0.20786E+01_JPRB, 0.23046E+01_JPRB ,&
&  0.25864E+01_JPRB, 0.28470E+01_JPRB, 0.31175E+01_JPRB, 0.32675E+01_JPRB, 0.33531E+01_JPRB, 0.35048E+01_JPRB, 0.38750E+01_JPRB ,&
&  0.40609E+01_JPRB, 0.42273E+01_JPRB, 0.43616E+01_JPRB, 0.46644E+01_JPRB, 0.50322E+01_JPRB, 0.49572E+01_JPRB, 0.43844E+01_JPRB ,&
&  0.40272E+01_JPRB, 0.37406E+01_JPRB, 0.35401E+01_JPRB, 0.37813E+01_JPRB, 0.41131E+01_JPRB, 0.47299E+01_JPRB, 0.43909E+01_JPRB ,&
&  0.36591E+01_JPRB, 0.31109E+01_JPRB, 0.31310E+01_JPRB, 0.35689E+01_JPRB, 0.38735E+01_JPRB, 0.40291E+01_JPRB, 0.45186E+01_JPRB ,&
&  0.43764E+01_JPRB, 0.24367E+01_JPRB, 0.18834E+01_JPRB, 0.27060E+01_JPRB, 0.19112E+01_JPRB, 0.17637E+01_JPRB, 0.32950E+01_JPRB ,&
&  0.50883E+01_JPRB, 0.61643E+01_JPRB, 0.56378E+01_JPRB, 0.46738E+01_JPRB, 0.44350E+01_JPRB, 0.39667E+01_JPRB, 0.32099E+01_JPRB ,&
&  0.24330E+01_JPRB, 0.19499E+01_JPRB, 0.16398E+01_JPRB, 0.14270E+01_JPRB, 0.12539E+01_JPRB, 0.11933E+01_JPRB, 0.11851E+01_JPRB ,&
&  0.11549E+01_JPRB, 0.11712E+01_JPRB, 0.11723E+01_JPRB, 0.11624E+01_JPRB, 0.11820E+01_JPRB /)
ZBURD(:, 27, 10) = (/ &
&  0.11167E+01_JPRB, 0.10532E+01_JPRB, 0.10195E+01_JPRB, 0.10262E+01_JPRB, 0.11119E+01_JPRB, 0.11723E+01_JPRB, 0.11744E+01_JPRB ,&
&  0.11756E+01_JPRB, 0.11863E+01_JPRB, 0.12259E+01_JPRB, 0.12252E+01_JPRB, 0.13266E+01_JPRB, 0.15220E+01_JPRB, 0.16706E+01_JPRB ,&
&  0.17965E+01_JPRB, 0.19344E+01_JPRB, 0.21187E+01_JPRB, 0.24134E+01_JPRB, 0.27916E+01_JPRB, 0.32525E+01_JPRB, 0.20344E+01_JPRB ,&
&  0.19535E+01_JPRB, 0.24800E+01_JPRB, 0.25151E+01_JPRB, 0.26393E+01_JPRB, 0.24749E+01_JPRB, 0.23646E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.19867E+01_JPRB, 0.17988E+01_JPRB, 0.16601E+01_JPRB, 0.16157E+01_JPRB, 0.15914E+01_JPRB, 0.16880E+01_JPRB, 0.17342E+01_JPRB ,&
&  0.17329E+01_JPRB, 0.18080E+01_JPRB, 0.19602E+01_JPRB, 0.21066E+01_JPRB, 0.22453E+01_JPRB, 0.23762E+01_JPRB, 0.25378E+01_JPRB ,&
&  0.27395E+01_JPRB, 0.29290E+01_JPRB, 0.31892E+01_JPRB, 0.31675E+01_JPRB, 0.32160E+01_JPRB, 0.33403E+01_JPRB, 0.36046E+01_JPRB ,&
&  0.35301E+01_JPRB, 0.33895E+01_JPRB, 0.38423E+01_JPRB, 0.43446E+01_JPRB, 0.44749E+01_JPRB, 0.41356E+01_JPRB, 0.37639E+01_JPRB ,&
&  0.38645E+01_JPRB, 0.39455E+01_JPRB, 0.42396E+01_JPRB, 0.38455E+01_JPRB, 0.39575E+01_JPRB, 0.42826E+01_JPRB, 0.44017E+01_JPRB ,&
&  0.42411E+01_JPRB, 0.38644E+01_JPRB, 0.37625E+01_JPRB, 0.38206E+01_JPRB, 0.37344E+01_JPRB, 0.35338E+01_JPRB, 0.35922E+01_JPRB ,&
&  0.42889E+01_JPRB, 0.52905E+01_JPRB, 0.46681E+01_JPRB, 0.32883E+01_JPRB, 0.28128E+01_JPRB, 0.25953E+01_JPRB, 0.32759E+01_JPRB ,&
&  0.52441E+01_JPRB, 0.49165E+01_JPRB, 0.43550E+01_JPRB, 0.45097E+01_JPRB, 0.36549E+01_JPRB, 0.30474E+01_JPRB, 0.23713E+01_JPRB ,&
&  0.18035E+01_JPRB, 0.14056E+01_JPRB, 0.12517E+01_JPRB, 0.11262E+01_JPRB, 0.11050E+01_JPRB, 0.11251E+01_JPRB, 0.11640E+01_JPRB ,&
&  0.12050E+01_JPRB, 0.12135E+01_JPRB, 0.11966E+01_JPRB, 0.11729E+01_JPRB, 0.11663E+01_JPRB /)
ZBURD(:, 28, 10) = (/ &
&  0.11008E+01_JPRB, 0.10528E+01_JPRB, 0.10164E+01_JPRB, 0.10444E+01_JPRB, 0.11290E+01_JPRB, 0.11799E+01_JPRB, 0.11964E+01_JPRB ,&
&  0.12041E+01_JPRB, 0.12140E+01_JPRB, 0.12922E+01_JPRB, 0.13587E+01_JPRB, 0.15416E+01_JPRB, 0.17424E+01_JPRB, 0.19134E+01_JPRB ,&
&  0.20654E+01_JPRB, 0.21927E+01_JPRB, 0.23687E+01_JPRB, 0.26167E+01_JPRB, 0.28610E+01_JPRB, 0.30619E+01_JPRB, 0.23886E+01_JPRB ,&
&  0.16781E+01_JPRB, 0.29069E+01_JPRB, 0.25329E+01_JPRB, 0.25461E+01_JPRB, 0.23217E+01_JPRB, 0.22874E+01_JPRB, 0.20770E+01_JPRB ,&
&  0.19597E+01_JPRB, 0.18318E+01_JPRB, 0.18430E+01_JPRB, 0.18724E+01_JPRB, 0.18669E+01_JPRB, 0.19410E+01_JPRB, 0.20554E+01_JPRB ,&
&  0.21687E+01_JPRB, 0.22924E+01_JPRB, 0.23729E+01_JPRB, 0.24358E+01_JPRB, 0.25183E+01_JPRB, 0.25866E+01_JPRB, 0.27118E+01_JPRB ,&
&  0.28818E+01_JPRB, 0.30469E+01_JPRB, 0.31235E+01_JPRB, 0.31813E+01_JPRB, 0.30726E+01_JPRB, 0.32018E+01_JPRB, 0.33903E+01_JPRB ,&
&  0.32858E+01_JPRB, 0.31380E+01_JPRB, 0.34485E+01_JPRB, 0.35316E+01_JPRB, 0.35158E+01_JPRB, 0.38339E+01_JPRB, 0.36097E+01_JPRB ,&
&  0.40089E+01_JPRB, 0.39018E+01_JPRB, 0.45997E+01_JPRB, 0.35733E+01_JPRB, 0.35735E+01_JPRB, 0.38470E+01_JPRB, 0.36984E+01_JPRB ,&
&  0.34402E+01_JPRB, 0.36248E+01_JPRB, 0.35858E+01_JPRB, 0.37430E+01_JPRB, 0.37206E+01_JPRB, 0.34417E+01_JPRB, 0.31330E+01_JPRB ,&
&  0.33009E+01_JPRB, 0.35878E+01_JPRB, 0.33985E+01_JPRB, 0.26311E+01_JPRB, 0.26243E+01_JPRB, 0.29191E+01_JPRB, 0.40804E+01_JPRB ,&
&  0.46569E+01_JPRB, 0.42188E+01_JPRB, 0.38212E+01_JPRB, 0.33703E+01_JPRB, 0.25282E+01_JPRB, 0.19776E+01_JPRB, 0.14473E+01_JPRB ,&
&  0.10802E+01_JPRB, 0.93996E+00_JPRB, 0.10281E+01_JPRB, 0.10050E+01_JPRB, 0.10477E+01_JPRB, 0.11038E+01_JPRB, 0.11280E+01_JPRB ,&
&  0.11609E+01_JPRB, 0.11672E+01_JPRB, 0.11704E+01_JPRB, 0.11506E+01_JPRB, 0.11357E+01_JPRB /)
ZBURD(:, 29, 10) = (/ &
&  0.10105E+01_JPRB, 0.98964E+00_JPRB, 0.97305E+00_JPRB, 0.10531E+01_JPRB, 0.11908E+01_JPRB, 0.13097E+01_JPRB, 0.13272E+01_JPRB ,&
&  0.13364E+01_JPRB, 0.11915E+01_JPRB, 0.12954E+01_JPRB, 0.13887E+01_JPRB, 0.15655E+01_JPRB, 0.17655E+01_JPRB, 0.19883E+01_JPRB ,&
&  0.22113E+01_JPRB, 0.24363E+01_JPRB, 0.26239E+01_JPRB, 0.28165E+01_JPRB, 0.30447E+01_JPRB, 0.32735E+01_JPRB, 0.28966E+01_JPRB ,&
&  0.21922E+01_JPRB, 0.20589E+01_JPRB, 0.26920E+01_JPRB, 0.23810E+01_JPRB, 0.21293E+01_JPRB, 0.21323E+01_JPRB, 0.21248E+01_JPRB ,&
&  0.20931E+01_JPRB, 0.19427E+01_JPRB, 0.19547E+01_JPRB, 0.20082E+01_JPRB, 0.19696E+01_JPRB, 0.20905E+01_JPRB, 0.22576E+01_JPRB ,&
&  0.23754E+01_JPRB, 0.24910E+01_JPRB, 0.25507E+01_JPRB, 0.25927E+01_JPRB, 0.26535E+01_JPRB, 0.27451E+01_JPRB, 0.28732E+01_JPRB ,&
&  0.30322E+01_JPRB, 0.31947E+01_JPRB, 0.32753E+01_JPRB, 0.31479E+01_JPRB, 0.31266E+01_JPRB, 0.32180E+01_JPRB, 0.32460E+01_JPRB ,&
&  0.31988E+01_JPRB, 0.33089E+01_JPRB, 0.34206E+01_JPRB, 0.34712E+01_JPRB, 0.33231E+01_JPRB, 0.38096E+01_JPRB, 0.37118E+01_JPRB ,&
&  0.38663E+01_JPRB, 0.38484E+01_JPRB, 0.42261E+01_JPRB, 0.32301E+01_JPRB, 0.33337E+01_JPRB, 0.34553E+01_JPRB, 0.34183E+01_JPRB ,&
&  0.33589E+01_JPRB, 0.35310E+01_JPRB, 0.35703E+01_JPRB, 0.35576E+01_JPRB, 0.35454E+01_JPRB, 0.30742E+01_JPRB, 0.29883E+01_JPRB ,&
&  0.28766E+01_JPRB, 0.32620E+01_JPRB, 0.28638E+01_JPRB, 0.27670E+01_JPRB, 0.26082E+01_JPRB, 0.29050E+01_JPRB, 0.33187E+01_JPRB ,&
&  0.36494E+01_JPRB, 0.31939E+01_JPRB, 0.26697E+01_JPRB, 0.20280E+01_JPRB, 0.14086E+01_JPRB, 0.10393E+01_JPRB, 0.87013E+00_JPRB ,&
&  0.84489E+00_JPRB, 0.87396E+00_JPRB, 0.92380E+00_JPRB, 0.94935E+00_JPRB, 0.98616E+00_JPRB, 0.10239E+01_JPRB, 0.10371E+01_JPRB ,&
&  0.10410E+01_JPRB, 0.10240E+01_JPRB, 0.10003E+01_JPRB, 0.10320E+01_JPRB, 0.10394E+01_JPRB /)
ZBURD(:, 30, 10) = (/ &
&  0.85799E+00_JPRB, 0.82258E+00_JPRB, 0.81575E+00_JPRB, 0.90752E+00_JPRB, 0.10163E+01_JPRB, 0.10832E+01_JPRB, 0.10680E+01_JPRB ,&
&  0.10431E+01_JPRB, 0.10276E+01_JPRB, 0.11161E+01_JPRB, 0.12274E+01_JPRB, 0.13743E+01_JPRB, 0.15681E+01_JPRB, 0.17762E+01_JPRB ,&
&  0.20142E+01_JPRB, 0.22069E+01_JPRB, 0.23441E+01_JPRB, 0.24931E+01_JPRB, 0.26873E+01_JPRB, 0.29024E+01_JPRB, 0.30209E+01_JPRB ,&
&  0.24721E+01_JPRB, 0.17360E+01_JPRB, 0.20586E+01_JPRB, 0.20243E+01_JPRB, 0.20454E+01_JPRB, 0.20887E+01_JPRB, 0.20438E+01_JPRB ,&
&  0.18642E+01_JPRB, 0.18391E+01_JPRB, 0.19219E+01_JPRB, 0.19880E+01_JPRB, 0.19525E+01_JPRB, 0.20733E+01_JPRB, 0.22198E+01_JPRB ,&
&  0.23889E+01_JPRB, 0.25219E+01_JPRB, 0.26110E+01_JPRB, 0.27056E+01_JPRB, 0.27782E+01_JPRB, 0.28732E+01_JPRB, 0.30314E+01_JPRB ,&
&  0.32022E+01_JPRB, 0.33589E+01_JPRB, 0.34435E+01_JPRB, 0.34067E+01_JPRB, 0.34017E+01_JPRB, 0.34676E+01_JPRB, 0.34147E+01_JPRB ,&
&  0.33951E+01_JPRB, 0.33639E+01_JPRB, 0.35886E+01_JPRB, 0.38627E+01_JPRB, 0.39575E+01_JPRB, 0.34929E+01_JPRB, 0.36309E+01_JPRB ,&
&  0.36130E+01_JPRB, 0.35115E+01_JPRB, 0.33882E+01_JPRB, 0.36528E+01_JPRB, 0.27281E+01_JPRB, 0.29593E+01_JPRB, 0.30416E+01_JPRB ,&
&  0.34099E+01_JPRB, 0.34068E+01_JPRB, 0.33509E+01_JPRB, 0.32232E+01_JPRB, 0.32785E+01_JPRB, 0.27490E+01_JPRB, 0.25251E+01_JPRB ,&
&  0.25213E+01_JPRB, 0.27055E+01_JPRB, 0.26210E+01_JPRB, 0.26937E+01_JPRB, 0.26128E+01_JPRB, 0.28324E+01_JPRB, 0.27934E+01_JPRB ,&
&  0.28199E+01_JPRB, 0.20317E+01_JPRB, 0.16231E+01_JPRB, 0.12483E+01_JPRB, 0.84007E+00_JPRB, 0.71995E+00_JPRB, 0.74168E+00_JPRB ,&
&  0.77214E+00_JPRB, 0.81142E+00_JPRB, 0.82240E+00_JPRB, 0.86004E+00_JPRB, 0.89198E+00_JPRB, 0.88590E+00_JPRB, 0.86303E+00_JPRB ,&
&  0.86487E+00_JPRB, 0.89337E+00_JPRB, 0.88695E+00_JPRB, 0.88290E+00_JPRB, 0.87483E+00_JPRB /)
ZBURD(:, 31, 10) = (/ &
&  0.68921E+00_JPRB, 0.68841E+00_JPRB, 0.70367E+00_JPRB, 0.74805E+00_JPRB, 0.82812E+00_JPRB, 0.87916E+00_JPRB, 0.90032E+00_JPRB ,&
&  0.89513E+00_JPRB, 0.92194E+00_JPRB, 0.98093E+00_JPRB, 0.10169E+01_JPRB, 0.11022E+01_JPRB, 0.12404E+01_JPRB, 0.13806E+01_JPRB ,&
&  0.15641E+01_JPRB, 0.17348E+01_JPRB, 0.18718E+01_JPRB, 0.20704E+01_JPRB, 0.22730E+01_JPRB, 0.25408E+01_JPRB, 0.27311E+01_JPRB ,&
&  0.27245E+01_JPRB, 0.23798E+01_JPRB, 0.21788E+01_JPRB, 0.16499E+01_JPRB, 0.14684E+01_JPRB, 0.17184E+01_JPRB, 0.16563E+01_JPRB ,&
&  0.16559E+01_JPRB, 0.17011E+01_JPRB, 0.17098E+01_JPRB, 0.17669E+01_JPRB, 0.18588E+01_JPRB, 0.19679E+01_JPRB, 0.21225E+01_JPRB ,&
&  0.23051E+01_JPRB, 0.24736E+01_JPRB, 0.26242E+01_JPRB, 0.27540E+01_JPRB, 0.28305E+01_JPRB, 0.29336E+01_JPRB, 0.30989E+01_JPRB ,&
&  0.32835E+01_JPRB, 0.33900E+01_JPRB, 0.34808E+01_JPRB, 0.36496E+01_JPRB, 0.36144E+01_JPRB, 0.36827E+01_JPRB, 0.36972E+01_JPRB ,&
&  0.37173E+01_JPRB, 0.36575E+01_JPRB, 0.36369E+01_JPRB, 0.36880E+01_JPRB, 0.35881E+01_JPRB, 0.31406E+01_JPRB, 0.30822E+01_JPRB ,&
&  0.31694E+01_JPRB, 0.30267E+01_JPRB, 0.27321E+01_JPRB, 0.31541E+01_JPRB, 0.22709E+01_JPRB, 0.26998E+01_JPRB, 0.30858E+01_JPRB ,&
&  0.30468E+01_JPRB, 0.29412E+01_JPRB, 0.27341E+01_JPRB, 0.27043E+01_JPRB, 0.26548E+01_JPRB, 0.23766E+01_JPRB, 0.22831E+01_JPRB ,&
&  0.22659E+01_JPRB, 0.19836E+01_JPRB, 0.19527E+01_JPRB, 0.21596E+01_JPRB, 0.23647E+01_JPRB, 0.22244E+01_JPRB, 0.19333E+01_JPRB ,&
&  0.15964E+01_JPRB, 0.14786E+01_JPRB, 0.13062E+01_JPRB, 0.13942E+01_JPRB, 0.10951E+01_JPRB, 0.67811E+00_JPRB, 0.68334E+00_JPRB ,&
&  0.70282E+00_JPRB, 0.72528E+00_JPRB, 0.72649E+00_JPRB, 0.76550E+00_JPRB, 0.77766E+00_JPRB, 0.76437E+00_JPRB, 0.76784E+00_JPRB ,&
&  0.78425E+00_JPRB, 0.77823E+00_JPRB, 0.76982E+00_JPRB, 0.74650E+00_JPRB, 0.72715E+00_JPRB /)
ZBURD(:, 32, 10) = (/ &
&  0.66982E+00_JPRB, 0.65765E+00_JPRB, 0.66049E+00_JPRB, 0.65816E+00_JPRB, 0.67006E+00_JPRB, 0.67244E+00_JPRB, 0.68563E+00_JPRB ,&
&  0.71051E+00_JPRB, 0.74210E+00_JPRB, 0.74189E+00_JPRB, 0.72756E+00_JPRB, 0.73110E+00_JPRB, 0.74873E+00_JPRB, 0.80404E+00_JPRB ,&
&  0.92943E+00_JPRB, 0.10644E+01_JPRB, 0.11571E+01_JPRB, 0.12774E+01_JPRB, 0.15128E+01_JPRB, 0.19304E+01_JPRB, 0.23309E+01_JPRB ,&
&  0.26299E+01_JPRB, 0.28186E+01_JPRB, 0.27445E+01_JPRB, 0.27652E+01_JPRB, 0.21639E+01_JPRB, 0.16250E+01_JPRB, 0.18015E+01_JPRB ,&
&  0.19485E+01_JPRB, 0.19116E+01_JPRB, 0.16805E+01_JPRB, 0.16381E+01_JPRB, 0.16361E+01_JPRB, 0.17214E+01_JPRB, 0.19123E+01_JPRB ,&
&  0.21073E+01_JPRB, 0.23190E+01_JPRB, 0.24883E+01_JPRB, 0.26173E+01_JPRB, 0.27106E+01_JPRB, 0.28186E+01_JPRB, 0.29234E+01_JPRB ,&
&  0.30390E+01_JPRB, 0.30853E+01_JPRB, 0.31183E+01_JPRB, 0.32346E+01_JPRB, 0.33713E+01_JPRB, 0.35247E+01_JPRB, 0.36137E+01_JPRB ,&
&  0.36416E+01_JPRB, 0.34615E+01_JPRB, 0.34252E+01_JPRB, 0.33837E+01_JPRB, 0.31512E+01_JPRB, 0.28941E+01_JPRB, 0.28269E+01_JPRB ,&
&  0.27615E+01_JPRB, 0.26766E+01_JPRB, 0.21541E+01_JPRB, 0.26660E+01_JPRB, 0.31009E+01_JPRB, 0.28785E+01_JPRB, 0.27636E+01_JPRB ,&
&  0.25676E+01_JPRB, 0.24231E+01_JPRB, 0.22534E+01_JPRB, 0.22642E+01_JPRB, 0.23900E+01_JPRB, 0.28703E+01_JPRB, 0.22303E+01_JPRB ,&
&  0.19716E+01_JPRB, 0.14257E+01_JPRB, 0.13533E+01_JPRB, 0.15258E+01_JPRB, 0.16663E+01_JPRB, 0.15717E+01_JPRB, 0.12667E+01_JPRB ,&
&  0.10977E+01_JPRB, 0.12297E+01_JPRB, 0.12968E+01_JPRB, 0.17490E+01_JPRB, 0.20344E+01_JPRB, 0.69102E+00_JPRB, 0.63100E+00_JPRB ,&
&  0.63897E+00_JPRB, 0.64342E+00_JPRB, 0.64304E+00_JPRB, 0.66018E+00_JPRB, 0.65895E+00_JPRB, 0.66692E+00_JPRB, 0.68808E+00_JPRB ,&
&  0.70013E+00_JPRB, 0.69973E+00_JPRB, 0.69462E+00_JPRB, 0.67248E+00_JPRB, 0.67533E+00_JPRB /)
ZBURD(:, 33, 10) = (/ &
&  0.73254E+00_JPRB, 0.72976E+00_JPRB, 0.73249E+00_JPRB, 0.72563E+00_JPRB, 0.71908E+00_JPRB, 0.71770E+00_JPRB, 0.72078E+00_JPRB ,&
&  0.72251E+00_JPRB, 0.71307E+00_JPRB, 0.70239E+00_JPRB, 0.69516E+00_JPRB, 0.68524E+00_JPRB, 0.65459E+00_JPRB, 0.61200E+00_JPRB ,&
&  0.62468E+00_JPRB, 0.64346E+00_JPRB, 0.65915E+00_JPRB, 0.69744E+00_JPRB, 0.79337E+00_JPRB, 0.94026E+00_JPRB, 0.11415E+01_JPRB ,&
&  0.15237E+01_JPRB, 0.19394E+01_JPRB, 0.22178E+01_JPRB, 0.22612E+01_JPRB, 0.24732E+01_JPRB, 0.18572E+01_JPRB, 0.17692E+01_JPRB ,&
&  0.19570E+01_JPRB, 0.16788E+01_JPRB, 0.15469E+01_JPRB, 0.14563E+01_JPRB, 0.14484E+01_JPRB, 0.14944E+01_JPRB, 0.16127E+01_JPRB ,&
&  0.17995E+01_JPRB, 0.19929E+01_JPRB, 0.21573E+01_JPRB, 0.23150E+01_JPRB, 0.24566E+01_JPRB, 0.25981E+01_JPRB, 0.26933E+01_JPRB ,&
&  0.27054E+01_JPRB, 0.27280E+01_JPRB, 0.27951E+01_JPRB, 0.28025E+01_JPRB, 0.30374E+01_JPRB, 0.31633E+01_JPRB, 0.32011E+01_JPRB ,&
&  0.31164E+01_JPRB, 0.29854E+01_JPRB, 0.29571E+01_JPRB, 0.28079E+01_JPRB, 0.26737E+01_JPRB, 0.25777E+01_JPRB, 0.24616E+01_JPRB ,&
&  0.23485E+01_JPRB, 0.21964E+01_JPRB, 0.18347E+01_JPRB, 0.21132E+01_JPRB, 0.23024E+01_JPRB, 0.21216E+01_JPRB, 0.24665E+01_JPRB ,&
&  0.22809E+01_JPRB, 0.21909E+01_JPRB, 0.20832E+01_JPRB, 0.19764E+01_JPRB, 0.18942E+01_JPRB, 0.21350E+01_JPRB, 0.19993E+01_JPRB ,&
&  0.14809E+01_JPRB, 0.11120E+01_JPRB, 0.96897E+00_JPRB, 0.10002E+01_JPRB, 0.11042E+01_JPRB, 0.11904E+01_JPRB, 0.11698E+01_JPRB ,&
&  0.10898E+01_JPRB, 0.10113E+01_JPRB, 0.10805E+01_JPRB, 0.13289E+01_JPRB, 0.95439E+00_JPRB, 0.64504E+00_JPRB, 0.58786E+00_JPRB ,&
&  0.57607E+00_JPRB, 0.57665E+00_JPRB, 0.58458E+00_JPRB, 0.61123E+00_JPRB, 0.63560E+00_JPRB, 0.65851E+00_JPRB, 0.67737E+00_JPRB ,&
&  0.68521E+00_JPRB, 0.70144E+00_JPRB, 0.71509E+00_JPRB, 0.72879E+00_JPRB, 0.73730E+00_JPRB /)
ZBURD(:, 34, 10) = (/ &
&  0.82236E+00_JPRB, 0.83513E+00_JPRB, 0.82821E+00_JPRB, 0.83530E+00_JPRB, 0.84063E+00_JPRB, 0.85655E+00_JPRB, 0.85615E+00_JPRB ,&
&  0.85156E+00_JPRB, 0.85619E+00_JPRB, 0.83525E+00_JPRB, 0.83968E+00_JPRB, 0.83650E+00_JPRB, 0.83936E+00_JPRB, 0.82756E+00_JPRB ,&
&  0.83261E+00_JPRB, 0.85002E+00_JPRB, 0.85576E+00_JPRB, 0.85947E+00_JPRB, 0.85143E+00_JPRB, 0.82928E+00_JPRB, 0.80813E+00_JPRB ,&
&  0.85281E+00_JPRB, 0.96618E+00_JPRB, 0.10998E+01_JPRB, 0.11892E+01_JPRB, 0.14076E+01_JPRB, 0.14190E+01_JPRB, 0.15455E+01_JPRB ,&
&  0.16619E+01_JPRB, 0.14585E+01_JPRB, 0.15053E+01_JPRB, 0.13098E+01_JPRB, 0.12864E+01_JPRB, 0.12471E+01_JPRB, 0.12310E+01_JPRB ,&
&  0.12632E+01_JPRB, 0.13337E+01_JPRB, 0.14169E+01_JPRB, 0.15112E+01_JPRB, 0.15899E+01_JPRB, 0.16120E+01_JPRB, 0.16120E+01_JPRB ,&
&  0.15872E+01_JPRB, 0.16738E+01_JPRB, 0.19776E+01_JPRB, 0.21809E+01_JPRB, 0.24165E+01_JPRB, 0.25290E+01_JPRB, 0.25281E+01_JPRB ,&
&  0.24753E+01_JPRB, 0.24466E+01_JPRB, 0.21437E+01_JPRB, 0.19276E+01_JPRB, 0.18901E+01_JPRB, 0.17476E+01_JPRB, 0.17222E+01_JPRB ,&
&  0.18143E+01_JPRB, 0.18159E+01_JPRB, 0.15150E+01_JPRB, 0.16793E+01_JPRB, 0.18778E+01_JPRB, 0.18439E+01_JPRB, 0.21186E+01_JPRB ,&
&  0.18566E+01_JPRB, 0.16862E+01_JPRB, 0.16047E+01_JPRB, 0.15563E+01_JPRB, 0.14987E+01_JPRB, 0.14103E+01_JPRB, 0.13887E+01_JPRB ,&
&  0.11469E+01_JPRB, 0.99805E+00_JPRB, 0.93749E+00_JPRB, 0.94180E+00_JPRB, 0.10707E+01_JPRB, 0.13180E+01_JPRB, 0.10955E+01_JPRB ,&
&  0.10600E+01_JPRB, 0.98745E+00_JPRB, 0.12392E+01_JPRB, 0.17122E+01_JPRB, 0.87112E+00_JPRB, 0.66253E+00_JPRB, 0.57275E+00_JPRB ,&
&  0.58530E+00_JPRB, 0.61773E+00_JPRB, 0.64533E+00_JPRB, 0.68123E+00_JPRB, 0.70113E+00_JPRB, 0.70723E+00_JPRB, 0.71996E+00_JPRB ,&
&  0.75062E+00_JPRB, 0.77792E+00_JPRB, 0.81648E+00_JPRB, 0.81761E+00_JPRB, 0.80793E+00_JPRB /)
ZBURD(:, 35, 10) = (/ &
&  0.87461E+00_JPRB, 0.89236E+00_JPRB, 0.89374E+00_JPRB, 0.91081E+00_JPRB, 0.93385E+00_JPRB, 0.94221E+00_JPRB, 0.94581E+00_JPRB ,&
&  0.93617E+00_JPRB, 0.93779E+00_JPRB, 0.93344E+00_JPRB, 0.95060E+00_JPRB, 0.95247E+00_JPRB, 0.95344E+00_JPRB, 0.95581E+00_JPRB ,&
&  0.97194E+00_JPRB, 0.98454E+00_JPRB, 0.10162E+01_JPRB, 0.10197E+01_JPRB, 0.10147E+01_JPRB, 0.10201E+01_JPRB, 0.10412E+01_JPRB ,&
&  0.10652E+01_JPRB, 0.10927E+01_JPRB, 0.11306E+01_JPRB, 0.11630E+01_JPRB, 0.12769E+01_JPRB, 0.14379E+01_JPRB, 0.17176E+01_JPRB ,&
&  0.15621E+01_JPRB, 0.14845E+01_JPRB, 0.13648E+01_JPRB, 0.11660E+01_JPRB, 0.11425E+01_JPRB, 0.11363E+01_JPRB, 0.10735E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.97410E+00_JPRB, 0.94009E+00_JPRB, 0.92571E+00_JPRB, 0.92823E+00_JPRB, 0.91382E+00_JPRB, 0.90073E+00_JPRB ,&
&  0.86142E+00_JPRB, 0.87026E+00_JPRB, 0.10537E+01_JPRB, 0.12863E+01_JPRB, 0.14645E+01_JPRB, 0.16278E+01_JPRB, 0.14693E+01_JPRB ,&
&  0.14040E+01_JPRB, 0.14193E+01_JPRB, 0.12939E+01_JPRB, 0.12182E+01_JPRB, 0.11162E+01_JPRB, 0.10288E+01_JPRB, 0.99071E+00_JPRB ,&
&  0.10852E+01_JPRB, 0.12460E+01_JPRB, 0.12660E+01_JPRB, 0.14993E+01_JPRB, 0.16443E+01_JPRB, 0.16832E+01_JPRB, 0.15375E+01_JPRB ,&
&  0.13509E+01_JPRB, 0.12510E+01_JPRB, 0.12241E+01_JPRB, 0.12010E+01_JPRB, 0.12075E+01_JPRB, 0.11490E+01_JPRB, 0.10535E+01_JPRB ,&
&  0.99931E+00_JPRB, 0.92726E+00_JPRB, 0.90807E+00_JPRB, 0.86239E+00_JPRB, 0.10553E+01_JPRB, 0.11214E+01_JPRB, 0.14436E+01_JPRB ,&
&  0.10627E+01_JPRB, 0.96703E+00_JPRB, 0.85085E+00_JPRB, 0.12496E+01_JPRB, 0.79314E+00_JPRB, 0.58828E+00_JPRB, 0.60117E+00_JPRB ,&
&  0.67531E+00_JPRB, 0.71785E+00_JPRB, 0.73767E+00_JPRB, 0.76812E+00_JPRB, 0.77894E+00_JPRB, 0.77545E+00_JPRB, 0.78573E+00_JPRB ,&
&  0.82349E+00_JPRB, 0.83095E+00_JPRB, 0.83800E+00_JPRB, 0.83748E+00_JPRB, 0.85452E+00_JPRB /)
ZBURD(:, 36, 10) = (/ &
&  0.89464E+00_JPRB, 0.92193E+00_JPRB, 0.93803E+00_JPRB, 0.95458E+00_JPRB, 0.97644E+00_JPRB, 0.98886E+00_JPRB, 0.99824E+00_JPRB ,&
&  0.10012E+01_JPRB, 0.99817E+00_JPRB, 0.10064E+01_JPRB, 0.10210E+01_JPRB, 0.10277E+01_JPRB, 0.10247E+01_JPRB, 0.10310E+01_JPRB ,&
&  0.10476E+01_JPRB, 0.10735E+01_JPRB, 0.11013E+01_JPRB, 0.10948E+01_JPRB, 0.10902E+01_JPRB, 0.11057E+01_JPRB, 0.11275E+01_JPRB ,&
&  0.11280E+01_JPRB, 0.11218E+01_JPRB, 0.11296E+01_JPRB, 0.11407E+01_JPRB, 0.11613E+01_JPRB, 0.11655E+01_JPRB, 0.12649E+01_JPRB ,&
&  0.11783E+01_JPRB, 0.15808E+01_JPRB, 0.14614E+01_JPRB, 0.12445E+01_JPRB, 0.12103E+01_JPRB, 0.11076E+01_JPRB, 0.10727E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10097E+01_JPRB, 0.96764E+00_JPRB, 0.93554E+00_JPRB, 0.93245E+00_JPRB, 0.93888E+00_JPRB, 0.91646E+00_JPRB ,&
&  0.92296E+00_JPRB, 0.94461E+00_JPRB, 0.99997E+00_JPRB, 0.10716E+01_JPRB, 0.11219E+01_JPRB, 0.11136E+01_JPRB, 0.10259E+01_JPRB ,&
&  0.97187E+00_JPRB, 0.89698E+00_JPRB, 0.73620E+00_JPRB, 0.71867E+00_JPRB, 0.68849E+00_JPRB, 0.58933E+00_JPRB, 0.54638E+00_JPRB ,&
&  0.68470E+00_JPRB, 0.83479E+00_JPRB, 0.11337E+01_JPRB, 0.13296E+01_JPRB, 0.13652E+01_JPRB, 0.14169E+01_JPRB, 0.12366E+01_JPRB ,&
&  0.11046E+01_JPRB, 0.10206E+01_JPRB, 0.10141E+01_JPRB, 0.97287E+00_JPRB, 0.97709E+00_JPRB, 0.96446E+00_JPRB, 0.89830E+00_JPRB ,&
&  0.84717E+00_JPRB, 0.82833E+00_JPRB, 0.83414E+00_JPRB, 0.86959E+00_JPRB, 0.88223E+00_JPRB, 0.14494E+01_JPRB, 0.22759E+01_JPRB ,&
&  0.11813E+01_JPRB, 0.86669E+00_JPRB, 0.60052E+00_JPRB, 0.75406E+00_JPRB, 0.66175E+00_JPRB, 0.60874E+00_JPRB, 0.66484E+00_JPRB ,&
&  0.78270E+00_JPRB, 0.83379E+00_JPRB, 0.84333E+00_JPRB, 0.83581E+00_JPRB, 0.81342E+00_JPRB, 0.79313E+00_JPRB, 0.78377E+00_JPRB ,&
&  0.80019E+00_JPRB, 0.81675E+00_JPRB, 0.83626E+00_JPRB, 0.84832E+00_JPRB, 0.86665E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART19',1,ZHOOK_HANDLE)
END SUBROUTINE PART19



SUBROUTINE PART20

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART20',0,ZHOOK_HANDLE)


ZBURD(:, 37, 10) = (/ &
&  0.90111E+00_JPRB, 0.92603E+00_JPRB, 0.94562E+00_JPRB, 0.96899E+00_JPRB, 0.99413E+00_JPRB, 0.10169E+01_JPRB, 0.10373E+01_JPRB ,&
&  0.10544E+01_JPRB, 0.10689E+01_JPRB, 0.10826E+01_JPRB, 0.10999E+01_JPRB, 0.11206E+01_JPRB, 0.11368E+01_JPRB, 0.11581E+01_JPRB ,&
&  0.11909E+01_JPRB, 0.12177E+01_JPRB, 0.12480E+01_JPRB, 0.12464E+01_JPRB, 0.12468E+01_JPRB, 0.12713E+01_JPRB, 0.12939E+01_JPRB ,&
&  0.13012E+01_JPRB, 0.13166E+01_JPRB, 0.13424E+01_JPRB, 0.13728E+01_JPRB, 0.14114E+01_JPRB, 0.14906E+01_JPRB, 0.14288E+01_JPRB ,&
&  0.15213E+01_JPRB, 0.16428E+01_JPRB, 0.15368E+01_JPRB, 0.13449E+01_JPRB, 0.12541E+01_JPRB, 0.12083E+01_JPRB, 0.11937E+01_JPRB ,&
&  0.11896E+01_JPRB, 0.11495E+01_JPRB, 0.11222E+01_JPRB, 0.10902E+01_JPRB, 0.10750E+01_JPRB, 0.10814E+01_JPRB, 0.10905E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.11357E+01_JPRB, 0.11762E+01_JPRB, 0.12288E+01_JPRB, 0.12765E+01_JPRB, 0.12424E+01_JPRB, 0.11849E+01_JPRB ,&
&  0.11030E+01_JPRB, 0.10315E+01_JPRB, 0.83508E+00_JPRB, 0.76659E+00_JPRB, 0.71564E+00_JPRB, 0.63115E+00_JPRB, 0.55374E+00_JPRB ,&
&  0.64102E+00_JPRB, 0.74174E+00_JPRB, 0.93941E+00_JPRB, 0.12058E+01_JPRB, 0.11976E+01_JPRB, 0.11506E+01_JPRB, 0.10336E+01_JPRB ,&
&  0.93275E+00_JPRB, 0.87176E+00_JPRB, 0.85576E+00_JPRB, 0.82221E+00_JPRB, 0.81994E+00_JPRB, 0.83216E+00_JPRB, 0.77833E+00_JPRB ,&
&  0.76618E+00_JPRB, 0.77651E+00_JPRB, 0.79600E+00_JPRB, 0.83068E+00_JPRB, 0.93133E+00_JPRB, 0.98468E+00_JPRB, 0.14401E+01_JPRB ,&
&  0.10772E+01_JPRB, 0.69821E+00_JPRB, 0.71326E+00_JPRB, 0.72170E+00_JPRB, 0.68407E+00_JPRB, 0.66148E+00_JPRB, 0.73512E+00_JPRB ,&
&  0.88829E+00_JPRB, 0.10097E+01_JPRB, 0.10376E+01_JPRB, 0.98356E+00_JPRB, 0.89316E+00_JPRB, 0.84154E+00_JPRB, 0.81411E+00_JPRB ,&
&  0.82600E+00_JPRB, 0.83711E+00_JPRB, 0.85163E+00_JPRB, 0.85828E+00_JPRB, 0.87376E+00_JPRB /)
ZBURD(:, 38, 10) = (/ &
&  0.88532E+00_JPRB, 0.91339E+00_JPRB, 0.93663E+00_JPRB, 0.96008E+00_JPRB, 0.98301E+00_JPRB, 0.10051E+01_JPRB, 0.10285E+01_JPRB ,&
&  0.10453E+01_JPRB, 0.10687E+01_JPRB, 0.10966E+01_JPRB, 0.11253E+01_JPRB, 0.11511E+01_JPRB, 0.11834E+01_JPRB, 0.12104E+01_JPRB ,&
&  0.12471E+01_JPRB, 0.12728E+01_JPRB, 0.12999E+01_JPRB, 0.13045E+01_JPRB, 0.13070E+01_JPRB, 0.13234E+01_JPRB, 0.13360E+01_JPRB ,&
&  0.13337E+01_JPRB, 0.13407E+01_JPRB, 0.13558E+01_JPRB, 0.13825E+01_JPRB, 0.13919E+01_JPRB, 0.14574E+01_JPRB, 0.13024E+01_JPRB ,&
&  0.16312E+01_JPRB, 0.17825E+01_JPRB, 0.16593E+01_JPRB, 0.14679E+01_JPRB, 0.13616E+01_JPRB, 0.13456E+01_JPRB, 0.13461E+01_JPRB ,&
&  0.13601E+01_JPRB, 0.14006E+01_JPRB, 0.13044E+01_JPRB, 0.12542E+01_JPRB, 0.12366E+01_JPRB, 0.12403E+01_JPRB, 0.12744E+01_JPRB ,&
&  0.13170E+01_JPRB, 0.13611E+01_JPRB, 0.14094E+01_JPRB, 0.14551E+01_JPRB, 0.14850E+01_JPRB, 0.14903E+01_JPRB, 0.14600E+01_JPRB ,&
&  0.14046E+01_JPRB, 0.13490E+01_JPRB, 0.12279E+01_JPRB, 0.11904E+01_JPRB, 0.11310E+01_JPRB, 0.95971E+00_JPRB, 0.78893E+00_JPRB ,&
&  0.75096E+00_JPRB, 0.82185E+00_JPRB, 0.90269E+00_JPRB, 0.10771E+01_JPRB, 0.10692E+01_JPRB, 0.10045E+01_JPRB, 0.91544E+00_JPRB ,&
&  0.83569E+00_JPRB, 0.77587E+00_JPRB, 0.77878E+00_JPRB, 0.75091E+00_JPRB, 0.72673E+00_JPRB, 0.71305E+00_JPRB, 0.70195E+00_JPRB ,&
&  0.71953E+00_JPRB, 0.74509E+00_JPRB, 0.78140E+00_JPRB, 0.81868E+00_JPRB, 0.84820E+00_JPRB, 0.82554E+00_JPRB, 0.11997E+01_JPRB ,&
&  0.98441E+00_JPRB, 0.82083E+00_JPRB, 0.95661E+00_JPRB, 0.61610E+00_JPRB, 0.70313E+00_JPRB, 0.76408E+00_JPRB, 0.89113E+00_JPRB ,&
&  0.97725E+00_JPRB, 0.13528E+01_JPRB, 0.18372E+01_JPRB, 0.14726E+01_JPRB, 0.11944E+01_JPRB, 0.91931E+00_JPRB, 0.82935E+00_JPRB ,&
&  0.82832E+00_JPRB, 0.83209E+00_JPRB, 0.84205E+00_JPRB, 0.84647E+00_JPRB, 0.86006E+00_JPRB /)
ZBURD(:, 39, 10) = (/ &
&  0.86451E+00_JPRB, 0.88636E+00_JPRB, 0.91200E+00_JPRB, 0.93512E+00_JPRB, 0.95096E+00_JPRB, 0.96577E+00_JPRB, 0.98407E+00_JPRB ,&
&  0.99472E+00_JPRB, 0.10001E+01_JPRB, 0.10161E+01_JPRB, 0.10404E+01_JPRB, 0.10695E+01_JPRB, 0.11018E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.12304E+01_JPRB, 0.12872E+01_JPRB, 0.13301E+01_JPRB, 0.13585E+01_JPRB, 0.13879E+01_JPRB, 0.14168E+01_JPRB ,&
&  0.14616E+01_JPRB, 0.14911E+01_JPRB, 0.15194E+01_JPRB, 0.15765E+01_JPRB, 0.16210E+01_JPRB, 0.16779E+01_JPRB, 0.13989E+01_JPRB ,&
&  0.17551E+01_JPRB, 0.20058E+01_JPRB, 0.18289E+01_JPRB, 0.15835E+01_JPRB, 0.14595E+01_JPRB, 0.14395E+01_JPRB, 0.14319E+01_JPRB ,&
&  0.14339E+01_JPRB, 0.14167E+01_JPRB, 0.13133E+01_JPRB, 0.13422E+01_JPRB, 0.13683E+01_JPRB, 0.13571E+01_JPRB, 0.14102E+01_JPRB ,&
&  0.14676E+01_JPRB, 0.15149E+01_JPRB, 0.15658E+01_JPRB, 0.16160E+01_JPRB, 0.16479E+01_JPRB, 0.16878E+01_JPRB, 0.16734E+01_JPRB ,&
&  0.16593E+01_JPRB, 0.16523E+01_JPRB, 0.16487E+01_JPRB, 0.14980E+01_JPRB, 0.14366E+01_JPRB, 0.13600E+01_JPRB, 0.11353E+01_JPRB ,&
&  0.97343E+00_JPRB, 0.88352E+00_JPRB, 0.86444E+00_JPRB, 0.93870E+00_JPRB, 0.91383E+00_JPRB, 0.83755E+00_JPRB, 0.78108E+00_JPRB ,&
&  0.73353E+00_JPRB, 0.69762E+00_JPRB, 0.72011E+00_JPRB, 0.70829E+00_JPRB, 0.64562E+00_JPRB, 0.66170E+00_JPRB, 0.68668E+00_JPRB ,&
&  0.68371E+00_JPRB, 0.70942E+00_JPRB, 0.74961E+00_JPRB, 0.76949E+00_JPRB, 0.81858E+00_JPRB, 0.92930E+00_JPRB, 0.98656E+00_JPRB ,&
&  0.10424E+01_JPRB, 0.84460E+00_JPRB, 0.82112E+00_JPRB, 0.85301E+00_JPRB, 0.79675E+00_JPRB, 0.10146E+01_JPRB, 0.13635E+01_JPRB ,&
&  0.19589E+01_JPRB, 0.21011E+01_JPRB, 0.27095E+01_JPRB, 0.42004E+01_JPRB, 0.31976E+01_JPRB, 0.21399E+01_JPRB, 0.85068E+00_JPRB ,&
&  0.85286E+00_JPRB, 0.84683E+00_JPRB, 0.86265E+00_JPRB, 0.86040E+00_JPRB, 0.86480E+00_JPRB /)
ZBURD(:, 40, 10) = (/ &
&  0.74875E+00_JPRB, 0.74816E+00_JPRB, 0.77581E+00_JPRB, 0.80546E+00_JPRB, 0.83205E+00_JPRB, 0.85795E+00_JPRB, 0.87816E+00_JPRB ,&
&  0.90672E+00_JPRB, 0.93102E+00_JPRB, 0.93914E+00_JPRB, 0.93876E+00_JPRB, 0.95436E+00_JPRB, 0.96177E+00_JPRB, 0.98933E+00_JPRB ,&
&  0.10305E+01_JPRB, 0.10767E+01_JPRB, 0.11388E+01_JPRB, 0.11814E+01_JPRB, 0.12239E+01_JPRB, 0.12878E+01_JPRB, 0.13490E+01_JPRB ,&
&  0.14174E+01_JPRB, 0.15032E+01_JPRB, 0.15774E+01_JPRB, 0.16556E+01_JPRB, 0.17317E+01_JPRB, 0.17204E+01_JPRB, 0.13116E+01_JPRB ,&
&  0.18346E+01_JPRB, 0.20634E+01_JPRB, 0.19473E+01_JPRB, 0.16856E+01_JPRB, 0.15340E+01_JPRB, 0.14779E+01_JPRB, 0.14705E+01_JPRB ,&
&  0.14522E+01_JPRB, 0.13824E+01_JPRB, 0.13009E+01_JPRB, 0.12641E+01_JPRB, 0.13971E+01_JPRB, 0.13686E+01_JPRB, 0.14244E+01_JPRB ,&
&  0.14834E+01_JPRB, 0.15326E+01_JPRB, 0.15723E+01_JPRB, 0.16045E+01_JPRB, 0.16273E+01_JPRB, 0.16602E+01_JPRB, 0.16945E+01_JPRB ,&
&  0.17252E+01_JPRB, 0.17431E+01_JPRB, 0.17933E+01_JPRB, 0.16323E+01_JPRB, 0.15659E+01_JPRB, 0.16583E+01_JPRB, 0.15214E+01_JPRB ,&
&  0.11233E+01_JPRB, 0.89671E+00_JPRB, 0.91613E+00_JPRB, 0.91482E+00_JPRB, 0.85497E+00_JPRB, 0.77070E+00_JPRB, 0.72695E+00_JPRB ,&
&  0.69648E+00_JPRB, 0.67061E+00_JPRB, 0.66700E+00_JPRB, 0.65577E+00_JPRB, 0.65412E+00_JPRB, 0.66852E+00_JPRB, 0.67649E+00_JPRB ,&
&  0.70915E+00_JPRB, 0.70166E+00_JPRB, 0.70537E+00_JPRB, 0.72130E+00_JPRB, 0.78412E+00_JPRB, 0.92111E+00_JPRB, 0.11868E+01_JPRB ,&
&  0.13224E+01_JPRB, 0.81931E+00_JPRB, 0.82637E+00_JPRB, 0.84685E+00_JPRB, 0.90446E+00_JPRB, 0.10251E+01_JPRB, 0.12183E+01_JPRB ,&
&  0.15594E+01_JPRB, 0.20368E+01_JPRB, 0.19549E+01_JPRB, 0.15488E+01_JPRB, 0.19370E+01_JPRB, 0.16323E+01_JPRB, 0.75203E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.73469E+00_JPRB, 0.74184E+00_JPRB, 0.74974E+00_JPRB, 0.75271E+00_JPRB /)
ZBURD(:, 41, 10) = (/ &
&  0.61523E+00_JPRB, 0.60378E+00_JPRB, 0.61292E+00_JPRB, 0.63286E+00_JPRB, 0.65042E+00_JPRB, 0.68411E+00_JPRB, 0.72585E+00_JPRB ,&
&  0.77854E+00_JPRB, 0.81494E+00_JPRB, 0.82959E+00_JPRB, 0.83426E+00_JPRB, 0.84518E+00_JPRB, 0.85276E+00_JPRB, 0.87044E+00_JPRB ,&
&  0.90177E+00_JPRB, 0.93373E+00_JPRB, 0.97520E+00_JPRB, 0.10106E+01_JPRB, 0.10427E+01_JPRB, 0.11099E+01_JPRB, 0.11870E+01_JPRB ,&
&  0.12815E+01_JPRB, 0.13824E+01_JPRB, 0.15025E+01_JPRB, 0.16287E+01_JPRB, 0.17570E+01_JPRB, 0.18069E+01_JPRB, 0.18160E+01_JPRB ,&
&  0.13699E+01_JPRB, 0.19073E+01_JPRB, 0.19348E+01_JPRB, 0.17525E+01_JPRB, 0.15940E+01_JPRB, 0.15567E+01_JPRB, 0.15169E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.13514E+01_JPRB, 0.12973E+01_JPRB, 0.12618E+01_JPRB, 0.13051E+01_JPRB, 0.13167E+01_JPRB, 0.13669E+01_JPRB ,&
&  0.14160E+01_JPRB, 0.14502E+01_JPRB, 0.14732E+01_JPRB, 0.14963E+01_JPRB, 0.15370E+01_JPRB, 0.16025E+01_JPRB, 0.16944E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.18274E+01_JPRB, 0.18780E+01_JPRB, 0.17495E+01_JPRB, 0.16410E+01_JPRB, 0.17814E+01_JPRB, 0.17186E+01_JPRB ,&
&  0.12221E+01_JPRB, 0.92992E+00_JPRB, 0.96727E+00_JPRB, 0.99285E+00_JPRB, 0.89299E+00_JPRB, 0.81089E+00_JPRB, 0.75604E+00_JPRB ,&
&  0.69623E+00_JPRB, 0.64766E+00_JPRB, 0.65713E+00_JPRB, 0.66675E+00_JPRB, 0.67590E+00_JPRB, 0.68285E+00_JPRB, 0.70466E+00_JPRB ,&
&  0.73249E+00_JPRB, 0.72400E+00_JPRB, 0.71874E+00_JPRB, 0.71840E+00_JPRB, 0.74119E+00_JPRB, 0.77448E+00_JPRB, 0.80842E+00_JPRB ,&
&  0.87025E+00_JPRB, 0.82444E+00_JPRB, 0.83201E+00_JPRB, 0.87243E+00_JPRB, 0.91033E+00_JPRB, 0.95440E+00_JPRB, 0.99807E+00_JPRB ,&
&  0.10281E+01_JPRB, 0.10646E+01_JPRB, 0.10949E+01_JPRB, 0.10908E+01_JPRB, 0.10002E+01_JPRB, 0.91162E+00_JPRB, 0.67029E+00_JPRB ,&
&  0.65555E+00_JPRB, 0.63955E+00_JPRB, 0.64037E+00_JPRB, 0.63781E+00_JPRB, 0.63567E+00_JPRB /)
ZBURD(:, 42, 10) = (/ &
&  0.54182E+00_JPRB, 0.53520E+00_JPRB, 0.54161E+00_JPRB, 0.55160E+00_JPRB, 0.56889E+00_JPRB, 0.59844E+00_JPRB, 0.64326E+00_JPRB ,&
&  0.69482E+00_JPRB, 0.72462E+00_JPRB, 0.75037E+00_JPRB, 0.74261E+00_JPRB, 0.74617E+00_JPRB, 0.76492E+00_JPRB, 0.77824E+00_JPRB ,&
&  0.81162E+00_JPRB, 0.84621E+00_JPRB, 0.86656E+00_JPRB, 0.87295E+00_JPRB, 0.88672E+00_JPRB, 0.92325E+00_JPRB, 0.97540E+00_JPRB ,&
&  0.10410E+01_JPRB, 0.11325E+01_JPRB, 0.12584E+01_JPRB, 0.14286E+01_JPRB, 0.16156E+01_JPRB, 0.16854E+01_JPRB, 0.16781E+01_JPRB ,&
&  0.84489E+00_JPRB, 0.14195E+01_JPRB, 0.18675E+01_JPRB, 0.18304E+01_JPRB, 0.17320E+01_JPRB, 0.16641E+01_JPRB, 0.16231E+01_JPRB ,&
&  0.14560E+01_JPRB, 0.12892E+01_JPRB, 0.12219E+01_JPRB, 0.13043E+01_JPRB, 0.12181E+01_JPRB, 0.12406E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.13219E+01_JPRB, 0.13581E+01_JPRB, 0.13730E+01_JPRB, 0.13803E+01_JPRB, 0.14112E+01_JPRB, 0.14729E+01_JPRB, 0.15572E+01_JPRB ,&
&  0.16659E+01_JPRB, 0.18045E+01_JPRB, 0.19072E+01_JPRB, 0.17775E+01_JPRB, 0.17405E+01_JPRB, 0.18410E+01_JPRB, 0.21694E+01_JPRB ,&
&  0.14978E+01_JPRB, 0.10507E+01_JPRB, 0.10591E+01_JPRB, 0.10933E+01_JPRB, 0.96280E+00_JPRB, 0.87420E+00_JPRB, 0.81469E+00_JPRB ,&
&  0.75178E+00_JPRB, 0.71143E+00_JPRB, 0.69396E+00_JPRB, 0.68695E+00_JPRB, 0.70497E+00_JPRB, 0.73213E+00_JPRB, 0.77370E+00_JPRB ,&
&  0.79710E+00_JPRB, 0.80106E+00_JPRB, 0.79597E+00_JPRB, 0.79734E+00_JPRB, 0.80530E+00_JPRB, 0.81738E+00_JPRB, 0.81896E+00_JPRB ,&
&  0.84373E+00_JPRB, 0.84762E+00_JPRB, 0.86751E+00_JPRB, 0.91240E+00_JPRB, 0.97025E+00_JPRB, 0.99029E+00_JPRB, 0.98553E+00_JPRB ,&
&  0.92823E+00_JPRB, 0.92048E+00_JPRB, 0.89842E+00_JPRB, 0.87486E+00_JPRB, 0.83603E+00_JPRB, 0.77606E+00_JPRB, 0.67560E+00_JPRB ,&
&  0.62755E+00_JPRB, 0.57946E+00_JPRB, 0.55452E+00_JPRB, 0.54337E+00_JPRB, 0.55309E+00_JPRB /)
ZBURD(:, 43, 10) = (/ &
&  0.52189E+00_JPRB, 0.51486E+00_JPRB, 0.51422E+00_JPRB, 0.53320E+00_JPRB, 0.52899E+00_JPRB, 0.54254E+00_JPRB, 0.56198E+00_JPRB ,&
&  0.61241E+00_JPRB, 0.65869E+00_JPRB, 0.70031E+00_JPRB, 0.69703E+00_JPRB, 0.69544E+00_JPRB, 0.68191E+00_JPRB, 0.68786E+00_JPRB ,&
&  0.70326E+00_JPRB, 0.72777E+00_JPRB, 0.75423E+00_JPRB, 0.76677E+00_JPRB, 0.77654E+00_JPRB, 0.78832E+00_JPRB, 0.80635E+00_JPRB ,&
&  0.83478E+00_JPRB, 0.87720E+00_JPRB, 0.95751E+00_JPRB, 0.11074E+01_JPRB, 0.13501E+01_JPRB, 0.16050E+01_JPRB, 0.17927E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.54851E+00_JPRB, 0.14509E+01_JPRB, 0.18820E+01_JPRB, 0.18325E+01_JPRB, 0.17452E+01_JPRB, 0.16576E+01_JPRB ,&
&  0.14463E+01_JPRB, 0.13211E+01_JPRB, 0.11782E+01_JPRB, 0.12513E+01_JPRB, 0.11318E+01_JPRB, 0.11211E+01_JPRB, 0.11527E+01_JPRB ,&
&  0.12008E+01_JPRB, 0.12465E+01_JPRB, 0.12889E+01_JPRB, 0.12987E+01_JPRB, 0.12916E+01_JPRB, 0.13065E+01_JPRB, 0.13596E+01_JPRB ,&
&  0.14694E+01_JPRB, 0.16468E+01_JPRB, 0.18624E+01_JPRB, 0.18941E+01_JPRB, 0.21299E+01_JPRB, 0.20391E+01_JPRB, 0.21641E+01_JPRB ,&
&  0.15685E+01_JPRB, 0.12064E+01_JPRB, 0.11629E+01_JPRB, 0.12005E+01_JPRB, 0.10574E+01_JPRB, 0.89690E+00_JPRB, 0.89267E+00_JPRB ,&
&  0.80079E+00_JPRB, 0.75914E+00_JPRB, 0.75301E+00_JPRB, 0.73832E+00_JPRB, 0.72443E+00_JPRB, 0.75075E+00_JPRB, 0.78793E+00_JPRB ,&
&  0.82866E+00_JPRB, 0.85112E+00_JPRB, 0.85381E+00_JPRB, 0.85839E+00_JPRB, 0.86252E+00_JPRB, 0.85888E+00_JPRB, 0.85567E+00_JPRB ,&
&  0.88085E+00_JPRB, 0.91151E+00_JPRB, 0.94720E+00_JPRB, 0.98471E+00_JPRB, 0.10282E+01_JPRB, 0.10331E+01_JPRB, 0.10388E+01_JPRB ,&
&  0.10331E+01_JPRB, 0.10115E+01_JPRB, 0.92336E+00_JPRB, 0.90429E+00_JPRB, 0.86008E+00_JPRB, 0.81089E+00_JPRB, 0.77075E+00_JPRB ,&
&  0.70991E+00_JPRB, 0.61946E+00_JPRB, 0.56202E+00_JPRB, 0.54506E+00_JPRB, 0.52853E+00_JPRB /)
ZBURD(:, 44, 10) = (/ &
&  0.55014E+00_JPRB, 0.53573E+00_JPRB, 0.53385E+00_JPRB, 0.54309E+00_JPRB, 0.54961E+00_JPRB, 0.55610E+00_JPRB, 0.54158E+00_JPRB ,&
&  0.55707E+00_JPRB, 0.59588E+00_JPRB, 0.62004E+00_JPRB, 0.64270E+00_JPRB, 0.63722E+00_JPRB, 0.62450E+00_JPRB, 0.61758E+00_JPRB ,&
&  0.61623E+00_JPRB, 0.62140E+00_JPRB, 0.63864E+00_JPRB, 0.64755E+00_JPRB, 0.65447E+00_JPRB, 0.66543E+00_JPRB, 0.67257E+00_JPRB ,&
&  0.68092E+00_JPRB, 0.69688E+00_JPRB, 0.73064E+00_JPRB, 0.79969E+00_JPRB, 0.92723E+00_JPRB, 0.11594E+01_JPRB, 0.14997E+01_JPRB ,&
&  0.21060E+01_JPRB, 0.15194E+01_JPRB, 0.71379E+00_JPRB, 0.15528E+01_JPRB, 0.19642E+01_JPRB, 0.19659E+01_JPRB, 0.16786E+01_JPRB ,&
&  0.15611E+01_JPRB, 0.13513E+01_JPRB, 0.12127E+01_JPRB, 0.12600E+01_JPRB, 0.10886E+01_JPRB, 0.10192E+01_JPRB, 0.10024E+01_JPRB ,&
&  0.10280E+01_JPRB, 0.10836E+01_JPRB, 0.11436E+01_JPRB, 0.11878E+01_JPRB, 0.12059E+01_JPRB, 0.12032E+01_JPRB, 0.12134E+01_JPRB ,&
&  0.12799E+01_JPRB, 0.13892E+01_JPRB, 0.15860E+01_JPRB, 0.18712E+01_JPRB, 0.25363E+01_JPRB, 0.23901E+01_JPRB, 0.23754E+01_JPRB ,&
&  0.16427E+01_JPRB, 0.14803E+01_JPRB, 0.14253E+01_JPRB, 0.13405E+01_JPRB, 0.11358E+01_JPRB, 0.10012E+01_JPRB, 0.10099E+01_JPRB ,&
&  0.88574E+00_JPRB, 0.80841E+00_JPRB, 0.78436E+00_JPRB, 0.75908E+00_JPRB, 0.74202E+00_JPRB, 0.75048E+00_JPRB, 0.77848E+00_JPRB ,&
&  0.81670E+00_JPRB, 0.83077E+00_JPRB, 0.83986E+00_JPRB, 0.84908E+00_JPRB, 0.85853E+00_JPRB, 0.86771E+00_JPRB, 0.87146E+00_JPRB ,&
&  0.89122E+00_JPRB, 0.92817E+00_JPRB, 0.96987E+00_JPRB, 0.98701E+00_JPRB, 0.98646E+00_JPRB, 0.10090E+01_JPRB, 0.10993E+01_JPRB ,&
&  0.11224E+01_JPRB, 0.11440E+01_JPRB, 0.10059E+01_JPRB, 0.97199E+00_JPRB, 0.89990E+00_JPRB, 0.85833E+00_JPRB, 0.82600E+00_JPRB ,&
&  0.79336E+00_JPRB, 0.71826E+00_JPRB, 0.63793E+00_JPRB, 0.56030E+00_JPRB, 0.55067E+00_JPRB /)
ZBURD(:, 45, 10) = (/ &
&  0.60290E+00_JPRB, 0.58511E+00_JPRB, 0.58706E+00_JPRB, 0.58048E+00_JPRB, 0.56863E+00_JPRB, 0.56297E+00_JPRB, 0.55576E+00_JPRB ,&
&  0.54794E+00_JPRB, 0.54644E+00_JPRB, 0.54494E+00_JPRB, 0.57548E+00_JPRB, 0.59034E+00_JPRB, 0.57985E+00_JPRB, 0.56853E+00_JPRB ,&
&  0.56038E+00_JPRB, 0.56104E+00_JPRB, 0.56637E+00_JPRB, 0.57112E+00_JPRB, 0.57444E+00_JPRB, 0.58281E+00_JPRB, 0.58990E+00_JPRB ,&
&  0.58703E+00_JPRB, 0.58910E+00_JPRB, 0.60116E+00_JPRB, 0.62858E+00_JPRB, 0.68298E+00_JPRB, 0.78119E+00_JPRB, 0.96263E+00_JPRB ,&
&  0.14037E+01_JPRB, 0.14490E+01_JPRB, 0.68602E+00_JPRB, 0.12697E+01_JPRB, 0.20262E+01_JPRB, 0.21174E+01_JPRB, 0.20005E+01_JPRB ,&
&  0.18140E+01_JPRB, 0.14438E+01_JPRB, 0.13149E+01_JPRB, 0.13095E+01_JPRB, 0.11354E+01_JPRB, 0.10054E+01_JPRB, 0.92732E+00_JPRB ,&
&  0.88959E+00_JPRB, 0.89672E+00_JPRB, 0.94908E+00_JPRB, 0.10048E+01_JPRB, 0.10593E+01_JPRB, 0.10869E+01_JPRB, 0.10949E+01_JPRB ,&
&  0.11105E+01_JPRB, 0.11624E+01_JPRB, 0.12840E+01_JPRB, 0.14122E+01_JPRB, 0.21317E+01_JPRB, 0.25307E+01_JPRB, 0.26441E+01_JPRB ,&
&  0.20586E+01_JPRB, 0.18759E+01_JPRB, 0.17040E+01_JPRB, 0.15056E+01_JPRB, 0.12527E+01_JPRB, 0.11561E+01_JPRB, 0.11532E+01_JPRB ,&
&  0.10077E+01_JPRB, 0.88927E+00_JPRB, 0.84318E+00_JPRB, 0.80394E+00_JPRB, 0.76923E+00_JPRB, 0.75334E+00_JPRB, 0.74968E+00_JPRB ,&
&  0.76700E+00_JPRB, 0.78172E+00_JPRB, 0.78576E+00_JPRB, 0.79304E+00_JPRB, 0.81022E+00_JPRB, 0.82842E+00_JPRB, 0.85570E+00_JPRB ,&
&  0.87182E+00_JPRB, 0.90555E+00_JPRB, 0.95099E+00_JPRB, 0.96203E+00_JPRB, 0.99461E+00_JPRB, 0.10627E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.12464E+01_JPRB, 0.10757E+01_JPRB, 0.97537E+00_JPRB, 0.96018E+00_JPRB, 0.91407E+00_JPRB, 0.87097E+00_JPRB ,&
&  0.90644E+00_JPRB, 0.10188E+01_JPRB, 0.97731E+00_JPRB, 0.67000E+00_JPRB, 0.61842E+00_JPRB /)
ZBURD(:, 46, 10) = (/ &
&  0.63655E+00_JPRB, 0.60960E+00_JPRB, 0.60440E+00_JPRB, 0.59987E+00_JPRB, 0.58712E+00_JPRB, 0.57688E+00_JPRB, 0.56904E+00_JPRB ,&
&  0.55802E+00_JPRB, 0.53893E+00_JPRB, 0.52287E+00_JPRB, 0.52603E+00_JPRB, 0.54034E+00_JPRB, 0.55106E+00_JPRB, 0.54115E+00_JPRB ,&
&  0.52949E+00_JPRB, 0.52505E+00_JPRB, 0.53361E+00_JPRB, 0.53800E+00_JPRB, 0.53667E+00_JPRB, 0.54026E+00_JPRB, 0.53934E+00_JPRB ,&
&  0.53779E+00_JPRB, 0.53431E+00_JPRB, 0.53702E+00_JPRB, 0.55057E+00_JPRB, 0.57561E+00_JPRB, 0.62294E+00_JPRB, 0.69774E+00_JPRB ,&
&  0.94962E+00_JPRB, 0.11667E+01_JPRB, 0.14645E+01_JPRB, 0.22266E+01_JPRB, 0.27144E+01_JPRB, 0.25651E+01_JPRB, 0.23930E+01_JPRB ,&
&  0.22335E+01_JPRB, 0.18961E+01_JPRB, 0.16315E+01_JPRB, 0.14201E+01_JPRB, 0.12001E+01_JPRB, 0.10351E+01_JPRB, 0.92983E+00_JPRB ,&
&  0.87346E+00_JPRB, 0.83839E+00_JPRB, 0.83464E+00_JPRB, 0.85243E+00_JPRB, 0.88305E+00_JPRB, 0.91890E+00_JPRB, 0.93925E+00_JPRB ,&
&  0.95517E+00_JPRB, 0.99205E+00_JPRB, 0.10742E+01_JPRB, 0.11506E+01_JPRB, 0.15312E+01_JPRB, 0.22273E+01_JPRB, 0.25839E+01_JPRB ,&
&  0.26016E+01_JPRB, 0.23825E+01_JPRB, 0.20455E+01_JPRB, 0.17190E+01_JPRB, 0.13596E+01_JPRB, 0.13840E+01_JPRB, 0.12642E+01_JPRB ,&
&  0.11209E+01_JPRB, 0.10070E+01_JPRB, 0.92807E+00_JPRB, 0.86762E+00_JPRB, 0.80970E+00_JPRB, 0.76040E+00_JPRB, 0.73493E+00_JPRB ,&
&  0.73257E+00_JPRB, 0.74008E+00_JPRB, 0.73930E+00_JPRB, 0.73505E+00_JPRB, 0.75438E+00_JPRB, 0.77434E+00_JPRB, 0.80734E+00_JPRB ,&
&  0.83921E+00_JPRB, 0.87251E+00_JPRB, 0.86702E+00_JPRB, 0.90614E+00_JPRB, 0.96288E+00_JPRB, 0.10124E+01_JPRB, 0.10119E+01_JPRB ,&
&  0.99372E+00_JPRB, 0.10620E+01_JPRB, 0.10568E+01_JPRB, 0.10243E+01_JPRB, 0.98721E+00_JPRB, 0.95775E+00_JPRB, 0.89057E+00_JPRB ,&
&  0.88627E+00_JPRB, 0.85510E+00_JPRB, 0.79982E+00_JPRB, 0.70718E+00_JPRB, 0.66789E+00_JPRB /)
ZBURD(:, 47, 10) = (/ &
&  0.67337E+00_JPRB, 0.64429E+00_JPRB, 0.62534E+00_JPRB, 0.61348E+00_JPRB, 0.60746E+00_JPRB, 0.60570E+00_JPRB, 0.59809E+00_JPRB ,&
&  0.58895E+00_JPRB, 0.57041E+00_JPRB, 0.54686E+00_JPRB, 0.52630E+00_JPRB, 0.52834E+00_JPRB, 0.52766E+00_JPRB, 0.52611E+00_JPRB ,&
&  0.51638E+00_JPRB, 0.51307E+00_JPRB, 0.51565E+00_JPRB, 0.51548E+00_JPRB, 0.51267E+00_JPRB, 0.50972E+00_JPRB, 0.50931E+00_JPRB ,&
&  0.51397E+00_JPRB, 0.51556E+00_JPRB, 0.51503E+00_JPRB, 0.51821E+00_JPRB, 0.52907E+00_JPRB, 0.55818E+00_JPRB, 0.60824E+00_JPRB ,&
&  0.75425E+00_JPRB, 0.10700E+01_JPRB, 0.12776E+01_JPRB, 0.25431E+01_JPRB, 0.29387E+01_JPRB, 0.28525E+01_JPRB, 0.25158E+01_JPRB ,&
&  0.22101E+01_JPRB, 0.21671E+01_JPRB, 0.17922E+01_JPRB, 0.14583E+01_JPRB, 0.11844E+01_JPRB, 0.10292E+01_JPRB, 0.91801E+00_JPRB ,&
&  0.84305E+00_JPRB, 0.80287E+00_JPRB, 0.78965E+00_JPRB, 0.78488E+00_JPRB, 0.77872E+00_JPRB, 0.78727E+00_JPRB, 0.79892E+00_JPRB ,&
&  0.81426E+00_JPRB, 0.86242E+00_JPRB, 0.95337E+00_JPRB, 0.10135E+01_JPRB, 0.12311E+01_JPRB, 0.17726E+01_JPRB, 0.22400E+01_JPRB ,&
&  0.24731E+01_JPRB, 0.23881E+01_JPRB, 0.21465E+01_JPRB, 0.18168E+01_JPRB, 0.15419E+01_JPRB, 0.14365E+01_JPRB, 0.13152E+01_JPRB ,&
&  0.11904E+01_JPRB, 0.11040E+01_JPRB, 0.98955E+00_JPRB, 0.91242E+00_JPRB, 0.87618E+00_JPRB, 0.81065E+00_JPRB, 0.75694E+00_JPRB ,&
&  0.72822E+00_JPRB, 0.71583E+00_JPRB, 0.70962E+00_JPRB, 0.70448E+00_JPRB, 0.71088E+00_JPRB, 0.72822E+00_JPRB, 0.75503E+00_JPRB ,&
&  0.78890E+00_JPRB, 0.83130E+00_JPRB, 0.81656E+00_JPRB, 0.82923E+00_JPRB, 0.88139E+00_JPRB, 0.93748E+00_JPRB, 0.95142E+00_JPRB ,&
&  0.96541E+00_JPRB, 0.99293E+00_JPRB, 0.10123E+01_JPRB, 0.99334E+00_JPRB, 0.95897E+00_JPRB, 0.10208E+01_JPRB, 0.92314E+00_JPRB ,&
&  0.90062E+00_JPRB, 0.81235E+00_JPRB, 0.76942E+00_JPRB, 0.72631E+00_JPRB, 0.70176E+00_JPRB /)
ZBURD(:, 48, 10) = (/ &
&  0.68877E+00_JPRB, 0.66857E+00_JPRB, 0.65297E+00_JPRB, 0.63863E+00_JPRB, 0.62809E+00_JPRB, 0.62111E+00_JPRB, 0.61243E+00_JPRB ,&
&  0.59886E+00_JPRB, 0.58082E+00_JPRB, 0.56152E+00_JPRB, 0.53524E+00_JPRB, 0.52102E+00_JPRB, 0.51932E+00_JPRB, 0.52570E+00_JPRB ,&
&  0.51786E+00_JPRB, 0.50408E+00_JPRB, 0.50491E+00_JPRB, 0.50826E+00_JPRB, 0.50729E+00_JPRB, 0.50092E+00_JPRB, 0.49409E+00_JPRB ,&
&  0.49744E+00_JPRB, 0.50211E+00_JPRB, 0.50530E+00_JPRB, 0.50760E+00_JPRB, 0.51087E+00_JPRB, 0.52306E+00_JPRB, 0.56143E+00_JPRB ,&
&  0.68487E+00_JPRB, 0.13154E+01_JPRB, 0.13304E+01_JPRB, 0.23576E+01_JPRB, 0.25409E+01_JPRB, 0.24949E+01_JPRB, 0.22738E+01_JPRB ,&
&  0.20415E+01_JPRB, 0.18836E+01_JPRB, 0.15772E+01_JPRB, 0.13475E+01_JPRB, 0.11340E+01_JPRB, 0.97553E+00_JPRB, 0.87420E+00_JPRB ,&
&  0.80900E+00_JPRB, 0.76109E+00_JPRB, 0.73309E+00_JPRB, 0.72372E+00_JPRB, 0.71857E+00_JPRB, 0.71574E+00_JPRB, 0.71852E+00_JPRB ,&
&  0.72987E+00_JPRB, 0.76650E+00_JPRB, 0.85696E+00_JPRB, 0.10445E+01_JPRB, 0.11542E+01_JPRB, 0.15283E+01_JPRB, 0.20955E+01_JPRB ,&
&  0.24194E+01_JPRB, 0.23659E+01_JPRB, 0.20287E+01_JPRB, 0.17344E+01_JPRB, 0.15216E+01_JPRB, 0.13764E+01_JPRB, 0.12508E+01_JPRB ,&
&  0.11655E+01_JPRB, 0.11017E+01_JPRB, 0.10325E+01_JPRB, 0.94728E+00_JPRB, 0.89259E+00_JPRB, 0.85176E+00_JPRB, 0.80278E+00_JPRB ,&
&  0.75196E+00_JPRB, 0.71447E+00_JPRB, 0.69396E+00_JPRB, 0.67989E+00_JPRB, 0.67538E+00_JPRB, 0.68714E+00_JPRB, 0.70669E+00_JPRB ,&
&  0.73154E+00_JPRB, 0.77172E+00_JPRB, 0.77487E+00_JPRB, 0.78954E+00_JPRB, 0.82600E+00_JPRB, 0.87514E+00_JPRB, 0.89197E+00_JPRB ,&
&  0.93290E+00_JPRB, 0.96607E+00_JPRB, 0.99901E+00_JPRB, 0.10524E+01_JPRB, 0.10358E+01_JPRB, 0.11501E+01_JPRB, 0.94429E+00_JPRB ,&
&  0.90553E+00_JPRB, 0.82793E+00_JPRB, 0.76650E+00_JPRB, 0.74261E+00_JPRB, 0.71789E+00_JPRB /)
ZBURD(:, 49, 10) = (/ &
&  0.69752E+00_JPRB, 0.67811E+00_JPRB, 0.66386E+00_JPRB, 0.65615E+00_JPRB, 0.64885E+00_JPRB, 0.63506E+00_JPRB, 0.62143E+00_JPRB ,&
&  0.60779E+00_JPRB, 0.59298E+00_JPRB, 0.57195E+00_JPRB, 0.54183E+00_JPRB, 0.51934E+00_JPRB, 0.51076E+00_JPRB, 0.51513E+00_JPRB ,&
&  0.51593E+00_JPRB, 0.50772E+00_JPRB, 0.50238E+00_JPRB, 0.50281E+00_JPRB, 0.50338E+00_JPRB, 0.49779E+00_JPRB, 0.48921E+00_JPRB ,&
&  0.48230E+00_JPRB, 0.48344E+00_JPRB, 0.48711E+00_JPRB, 0.49193E+00_JPRB, 0.49543E+00_JPRB, 0.50144E+00_JPRB, 0.52732E+00_JPRB ,&
&  0.64656E+00_JPRB, 0.67058E+00_JPRB, 0.15311E+01_JPRB, 0.20157E+01_JPRB, 0.20443E+01_JPRB, 0.19742E+01_JPRB, 0.19551E+01_JPRB ,&
&  0.18111E+01_JPRB, 0.15951E+01_JPRB, 0.13439E+01_JPRB, 0.11378E+01_JPRB, 0.98573E+00_JPRB, 0.88562E+00_JPRB, 0.81192E+00_JPRB ,&
&  0.75615E+00_JPRB, 0.71434E+00_JPRB, 0.68176E+00_JPRB, 0.66466E+00_JPRB, 0.66795E+00_JPRB, 0.67373E+00_JPRB, 0.67805E+00_JPRB ,&
&  0.68634E+00_JPRB, 0.70288E+00_JPRB, 0.75258E+00_JPRB, 0.90035E+00_JPRB, 0.11964E+01_JPRB, 0.14787E+01_JPRB, 0.19544E+01_JPRB ,&
&  0.24230E+01_JPRB, 0.21793E+01_JPRB, 0.17960E+01_JPRB, 0.15368E+01_JPRB, 0.13564E+01_JPRB, 0.12321E+01_JPRB, 0.11384E+01_JPRB ,&
&  0.10671E+01_JPRB, 0.10162E+01_JPRB, 0.97613E+00_JPRB, 0.92555E+00_JPRB, 0.88667E+00_JPRB, 0.85857E+00_JPRB, 0.82519E+00_JPRB ,&
&  0.78643E+00_JPRB, 0.74856E+00_JPRB, 0.71189E+00_JPRB, 0.67938E+00_JPRB, 0.65361E+00_JPRB, 0.65170E+00_JPRB, 0.66200E+00_JPRB ,&
&  0.68154E+00_JPRB, 0.71908E+00_JPRB, 0.75163E+00_JPRB, 0.77179E+00_JPRB, 0.81303E+00_JPRB, 0.84535E+00_JPRB, 0.88496E+00_JPRB ,&
&  0.90801E+00_JPRB, 0.94177E+00_JPRB, 0.96900E+00_JPRB, 0.10398E+01_JPRB, 0.10144E+01_JPRB, 0.10830E+01_JPRB, 0.93928E+00_JPRB ,&
&  0.89362E+00_JPRB, 0.85841E+00_JPRB, 0.79674E+00_JPRB, 0.76198E+00_JPRB, 0.73008E+00_JPRB /)
ZBURD(:, 50, 10) = (/ &
&  0.71207E+00_JPRB, 0.68347E+00_JPRB, 0.66977E+00_JPRB, 0.66156E+00_JPRB, 0.65384E+00_JPRB, 0.64637E+00_JPRB, 0.63173E+00_JPRB ,&
&  0.61240E+00_JPRB, 0.60044E+00_JPRB, 0.58672E+00_JPRB, 0.56900E+00_JPRB, 0.54394E+00_JPRB, 0.52666E+00_JPRB, 0.52343E+00_JPRB ,&
&  0.52085E+00_JPRB, 0.51572E+00_JPRB, 0.50609E+00_JPRB, 0.50117E+00_JPRB, 0.49543E+00_JPRB, 0.48811E+00_JPRB, 0.48332E+00_JPRB ,&
&  0.47322E+00_JPRB, 0.46893E+00_JPRB, 0.47144E+00_JPRB, 0.47739E+00_JPRB, 0.48233E+00_JPRB, 0.48881E+00_JPRB, 0.49959E+00_JPRB ,&
&  0.59456E+00_JPRB, 0.10071E+01_JPRB, 0.14060E+01_JPRB, 0.16278E+01_JPRB, 0.16491E+01_JPRB, 0.15462E+01_JPRB, 0.15423E+01_JPRB ,&
&  0.13919E+01_JPRB, 0.12449E+01_JPRB, 0.10915E+01_JPRB, 0.94865E+00_JPRB, 0.83865E+00_JPRB, 0.78066E+00_JPRB, 0.74812E+00_JPRB ,&
&  0.71089E+00_JPRB, 0.67826E+00_JPRB, 0.64941E+00_JPRB, 0.62486E+00_JPRB, 0.61639E+00_JPRB, 0.62731E+00_JPRB, 0.64235E+00_JPRB ,&
&  0.64863E+00_JPRB, 0.65554E+00_JPRB, 0.67841E+00_JPRB, 0.76933E+00_JPRB, 0.10335E+01_JPRB, 0.11755E+01_JPRB, 0.16296E+01_JPRB ,&
&  0.18856E+01_JPRB, 0.16887E+01_JPRB, 0.14406E+01_JPRB, 0.12642E+01_JPRB, 0.11260E+01_JPRB, 0.10223E+01_JPRB, 0.94770E+00_JPRB ,&
&  0.90628E+00_JPRB, 0.87636E+00_JPRB, 0.85347E+00_JPRB, 0.83019E+00_JPRB, 0.80907E+00_JPRB, 0.79415E+00_JPRB, 0.78153E+00_JPRB ,&
&  0.76549E+00_JPRB, 0.74530E+00_JPRB, 0.72078E+00_JPRB, 0.68904E+00_JPRB, 0.65554E+00_JPRB, 0.63640E+00_JPRB, 0.62940E+00_JPRB ,&
&  0.63382E+00_JPRB, 0.66900E+00_JPRB, 0.73246E+00_JPRB, 0.73545E+00_JPRB, 0.77747E+00_JPRB, 0.81378E+00_JPRB, 0.85233E+00_JPRB ,&
&  0.84993E+00_JPRB, 0.89115E+00_JPRB, 0.90805E+00_JPRB, 0.99563E+00_JPRB, 0.11721E+01_JPRB, 0.10808E+01_JPRB, 0.91638E+00_JPRB ,&
&  0.85554E+00_JPRB, 0.82532E+00_JPRB, 0.79562E+00_JPRB, 0.76089E+00_JPRB, 0.75169E+00_JPRB /)
ZBURD(:, 51, 10) = (/ &
&  0.74438E+00_JPRB, 0.70059E+00_JPRB, 0.67784E+00_JPRB, 0.66525E+00_JPRB, 0.65780E+00_JPRB, 0.65030E+00_JPRB, 0.63617E+00_JPRB ,&
&  0.61716E+00_JPRB, 0.60040E+00_JPRB, 0.58758E+00_JPRB, 0.57825E+00_JPRB, 0.56020E+00_JPRB, 0.53225E+00_JPRB, 0.51112E+00_JPRB ,&
&  0.50597E+00_JPRB, 0.50188E+00_JPRB, 0.49824E+00_JPRB, 0.49031E+00_JPRB, 0.48305E+00_JPRB, 0.47963E+00_JPRB, 0.47542E+00_JPRB ,&
&  0.47048E+00_JPRB, 0.46584E+00_JPRB, 0.46679E+00_JPRB, 0.47062E+00_JPRB, 0.47395E+00_JPRB, 0.47688E+00_JPRB, 0.48242E+00_JPRB ,&
&  0.52864E+00_JPRB, 0.77716E+00_JPRB, 0.95301E+00_JPRB, 0.12050E+01_JPRB, 0.12804E+01_JPRB, 0.12114E+01_JPRB, 0.11649E+01_JPRB ,&
&  0.10904E+01_JPRB, 0.10035E+01_JPRB, 0.92095E+00_JPRB, 0.84212E+00_JPRB, 0.75552E+00_JPRB, 0.70569E+00_JPRB, 0.68252E+00_JPRB ,&
&  0.66567E+00_JPRB, 0.64317E+00_JPRB, 0.61969E+00_JPRB, 0.59553E+00_JPRB, 0.57952E+00_JPRB, 0.58326E+00_JPRB, 0.59491E+00_JPRB ,&
&  0.59490E+00_JPRB, 0.59604E+00_JPRB, 0.60005E+00_JPRB, 0.63663E+00_JPRB, 0.78628E+00_JPRB, 0.92811E+00_JPRB, 0.11167E+01_JPRB ,&
&  0.11272E+01_JPRB, 0.11596E+01_JPRB, 0.10744E+01_JPRB, 0.98538E+00_JPRB, 0.91165E+00_JPRB, 0.85432E+00_JPRB, 0.81081E+00_JPRB ,&
&  0.78217E+00_JPRB, 0.76076E+00_JPRB, 0.74476E+00_JPRB, 0.73164E+00_JPRB, 0.72008E+00_JPRB, 0.70735E+00_JPRB, 0.70282E+00_JPRB ,&
&  0.70159E+00_JPRB, 0.69200E+00_JPRB, 0.68025E+00_JPRB, 0.66594E+00_JPRB, 0.64607E+00_JPRB, 0.62824E+00_JPRB, 0.61469E+00_JPRB ,&
&  0.60381E+00_JPRB, 0.60554E+00_JPRB, 0.64294E+00_JPRB, 0.67887E+00_JPRB, 0.70471E+00_JPRB, 0.72949E+00_JPRB, 0.75166E+00_JPRB ,&
&  0.77795E+00_JPRB, 0.84920E+00_JPRB, 0.83943E+00_JPRB, 0.92370E+00_JPRB, 0.10733E+01_JPRB, 0.98449E+00_JPRB, 0.86394E+00_JPRB ,&
&  0.81188E+00_JPRB, 0.77970E+00_JPRB, 0.75819E+00_JPRB, 0.74556E+00_JPRB, 0.78837E+00_JPRB /)
ZBURD(:, 52, 10) = (/ &
&  0.79357E+00_JPRB, 0.74753E+00_JPRB, 0.70552E+00_JPRB, 0.67917E+00_JPRB, 0.66072E+00_JPRB, 0.64555E+00_JPRB, 0.62688E+00_JPRB ,&
&  0.60895E+00_JPRB, 0.59157E+00_JPRB, 0.57745E+00_JPRB, 0.56720E+00_JPRB, 0.55691E+00_JPRB, 0.53649E+00_JPRB, 0.50929E+00_JPRB ,&
&  0.49108E+00_JPRB, 0.48653E+00_JPRB, 0.48333E+00_JPRB, 0.47650E+00_JPRB, 0.46997E+00_JPRB, 0.46744E+00_JPRB, 0.46435E+00_JPRB ,&
&  0.46394E+00_JPRB, 0.46400E+00_JPRB, 0.46660E+00_JPRB, 0.46738E+00_JPRB, 0.46436E+00_JPRB, 0.46995E+00_JPRB, 0.46956E+00_JPRB ,&
&  0.50481E+00_JPRB, 0.58042E+00_JPRB, 0.70299E+00_JPRB, 0.84227E+00_JPRB, 0.94034E+00_JPRB, 0.97452E+00_JPRB, 0.95796E+00_JPRB ,&
&  0.92794E+00_JPRB, 0.89440E+00_JPRB, 0.84411E+00_JPRB, 0.79301E+00_JPRB, 0.73533E+00_JPRB, 0.67346E+00_JPRB, 0.63704E+00_JPRB ,&
&  0.62528E+00_JPRB, 0.61166E+00_JPRB, 0.59774E+00_JPRB, 0.58098E+00_JPRB, 0.56524E+00_JPRB, 0.56015E+00_JPRB, 0.56161E+00_JPRB ,&
&  0.55832E+00_JPRB, 0.55391E+00_JPRB, 0.54899E+00_JPRB, 0.55214E+00_JPRB, 0.58052E+00_JPRB, 0.65426E+00_JPRB, 0.74600E+00_JPRB ,&
&  0.78080E+00_JPRB, 0.80287E+00_JPRB, 0.80907E+00_JPRB, 0.79711E+00_JPRB, 0.76837E+00_JPRB, 0.73453E+00_JPRB, 0.69950E+00_JPRB ,&
&  0.67403E+00_JPRB, 0.65724E+00_JPRB, 0.64937E+00_JPRB, 0.64524E+00_JPRB, 0.64248E+00_JPRB, 0.63822E+00_JPRB, 0.63171E+00_JPRB ,&
&  0.62741E+00_JPRB, 0.62558E+00_JPRB, 0.62565E+00_JPRB, 0.62173E+00_JPRB, 0.61633E+00_JPRB, 0.60923E+00_JPRB, 0.60309E+00_JPRB ,&
&  0.59718E+00_JPRB, 0.59413E+00_JPRB, 0.60367E+00_JPRB, 0.62060E+00_JPRB, 0.64117E+00_JPRB, 0.65800E+00_JPRB, 0.67233E+00_JPRB ,&
&  0.68622E+00_JPRB, 0.72940E+00_JPRB, 0.74846E+00_JPRB, 0.89980E+00_JPRB, 0.98991E+00_JPRB, 0.89137E+00_JPRB, 0.80765E+00_JPRB ,&
&  0.74351E+00_JPRB, 0.71239E+00_JPRB, 0.70385E+00_JPRB, 0.72352E+00_JPRB, 0.79031E+00_JPRB /)
ZBURD(:, 53, 10) = (/ &
&  0.66988E+00_JPRB, 0.66091E+00_JPRB, 0.64996E+00_JPRB, 0.64049E+00_JPRB, 0.62574E+00_JPRB, 0.61252E+00_JPRB, 0.59567E+00_JPRB ,&
&  0.57971E+00_JPRB, 0.56424E+00_JPRB, 0.55098E+00_JPRB, 0.54558E+00_JPRB, 0.53719E+00_JPRB, 0.52540E+00_JPRB, 0.50496E+00_JPRB ,&
&  0.48803E+00_JPRB, 0.47509E+00_JPRB, 0.46735E+00_JPRB, 0.46393E+00_JPRB, 0.46149E+00_JPRB, 0.45784E+00_JPRB, 0.45607E+00_JPRB ,&
&  0.45653E+00_JPRB, 0.45572E+00_JPRB, 0.45732E+00_JPRB, 0.45600E+00_JPRB, 0.45818E+00_JPRB, 0.45374E+00_JPRB, 0.45037E+00_JPRB ,&
&  0.47337E+00_JPRB, 0.49442E+00_JPRB, 0.56550E+00_JPRB, 0.64371E+00_JPRB, 0.72496E+00_JPRB, 0.78191E+00_JPRB, 0.82717E+00_JPRB ,&
&  0.85328E+00_JPRB, 0.83216E+00_JPRB, 0.79320E+00_JPRB, 0.74061E+00_JPRB, 0.69969E+00_JPRB, 0.65591E+00_JPRB, 0.61200E+00_JPRB ,&
&  0.59001E+00_JPRB, 0.57855E+00_JPRB, 0.56909E+00_JPRB, 0.56088E+00_JPRB, 0.55285E+00_JPRB, 0.54815E+00_JPRB, 0.54573E+00_JPRB ,&
&  0.53908E+00_JPRB, 0.53064E+00_JPRB, 0.52039E+00_JPRB, 0.51658E+00_JPRB, 0.51822E+00_JPRB, 0.53972E+00_JPRB, 0.58491E+00_JPRB ,&
&  0.60597E+00_JPRB, 0.61778E+00_JPRB, 0.62605E+00_JPRB, 0.63848E+00_JPRB, 0.64123E+00_JPRB, 0.63574E+00_JPRB, 0.61286E+00_JPRB ,&
&  0.59313E+00_JPRB, 0.57911E+00_JPRB, 0.57209E+00_JPRB, 0.56940E+00_JPRB, 0.57019E+00_JPRB, 0.56991E+00_JPRB, 0.57066E+00_JPRB ,&
&  0.56665E+00_JPRB, 0.56172E+00_JPRB, 0.56026E+00_JPRB, 0.56057E+00_JPRB, 0.56170E+00_JPRB, 0.56253E+00_JPRB, 0.56448E+00_JPRB ,&
&  0.56903E+00_JPRB, 0.57523E+00_JPRB, 0.58357E+00_JPRB, 0.58969E+00_JPRB, 0.58894E+00_JPRB, 0.59121E+00_JPRB, 0.60152E+00_JPRB ,&
&  0.61496E+00_JPRB, 0.64277E+00_JPRB, 0.66829E+00_JPRB, 0.76757E+00_JPRB, 0.80572E+00_JPRB, 0.77760E+00_JPRB, 0.71429E+00_JPRB ,&
&  0.66495E+00_JPRB, 0.64035E+00_JPRB, 0.63668E+00_JPRB, 0.65545E+00_JPRB, 0.64510E+00_JPRB /)
ZBURD(:, 54, 10) = (/ &
&  0.59817E+00_JPRB, 0.59462E+00_JPRB, 0.58955E+00_JPRB, 0.58600E+00_JPRB, 0.57750E+00_JPRB, 0.56837E+00_JPRB, 0.55892E+00_JPRB ,&
&  0.54662E+00_JPRB, 0.53384E+00_JPRB, 0.52268E+00_JPRB, 0.51789E+00_JPRB, 0.51423E+00_JPRB, 0.51076E+00_JPRB, 0.50259E+00_JPRB ,&
&  0.49126E+00_JPRB, 0.48059E+00_JPRB, 0.47124E+00_JPRB, 0.46513E+00_JPRB, 0.46289E+00_JPRB, 0.46001E+00_JPRB, 0.45621E+00_JPRB ,&
&  0.45341E+00_JPRB, 0.45318E+00_JPRB, 0.45163E+00_JPRB, 0.44991E+00_JPRB, 0.44762E+00_JPRB, 0.44185E+00_JPRB, 0.44111E+00_JPRB ,&
&  0.45262E+00_JPRB, 0.42530E+00_JPRB, 0.46377E+00_JPRB, 0.50207E+00_JPRB, 0.53823E+00_JPRB, 0.59723E+00_JPRB, 0.67193E+00_JPRB ,&
&  0.73631E+00_JPRB, 0.74855E+00_JPRB, 0.73314E+00_JPRB, 0.69539E+00_JPRB, 0.66702E+00_JPRB, 0.64601E+00_JPRB, 0.61533E+00_JPRB ,&
&  0.58480E+00_JPRB, 0.56423E+00_JPRB, 0.54946E+00_JPRB, 0.53748E+00_JPRB, 0.52905E+00_JPRB, 0.52193E+00_JPRB, 0.51550E+00_JPRB ,&
&  0.50688E+00_JPRB, 0.49827E+00_JPRB, 0.49084E+00_JPRB, 0.48731E+00_JPRB, 0.48921E+00_JPRB, 0.49585E+00_JPRB, 0.51297E+00_JPRB ,&
&  0.52606E+00_JPRB, 0.53290E+00_JPRB, 0.53326E+00_JPRB, 0.53799E+00_JPRB, 0.54590E+00_JPRB, 0.55214E+00_JPRB, 0.54934E+00_JPRB ,&
&  0.53643E+00_JPRB, 0.52055E+00_JPRB, 0.50834E+00_JPRB, 0.50039E+00_JPRB, 0.49686E+00_JPRB, 0.49742E+00_JPRB, 0.50126E+00_JPRB ,&
&  0.50202E+00_JPRB, 0.50080E+00_JPRB, 0.49981E+00_JPRB, 0.50112E+00_JPRB, 0.50368E+00_JPRB, 0.50919E+00_JPRB, 0.51682E+00_JPRB ,&
&  0.52561E+00_JPRB, 0.53528E+00_JPRB, 0.54662E+00_JPRB, 0.55727E+00_JPRB, 0.56239E+00_JPRB, 0.56614E+00_JPRB, 0.56850E+00_JPRB ,&
&  0.57315E+00_JPRB, 0.58890E+00_JPRB, 0.60671E+00_JPRB, 0.70912E+00_JPRB, 0.74407E+00_JPRB, 0.74045E+00_JPRB, 0.69299E+00_JPRB ,&
&  0.64987E+00_JPRB, 0.61064E+00_JPRB, 0.59941E+00_JPRB, 0.55030E+00_JPRB, 0.59008E+00_JPRB /)
ZBURD(:, 55, 10) = (/ &
&  0.53877E+00_JPRB, 0.53841E+00_JPRB, 0.53397E+00_JPRB, 0.52893E+00_JPRB, 0.52542E+00_JPRB, 0.52006E+00_JPRB, 0.51575E+00_JPRB ,&
&  0.50955E+00_JPRB, 0.50448E+00_JPRB, 0.49638E+00_JPRB, 0.49066E+00_JPRB, 0.48703E+00_JPRB, 0.48551E+00_JPRB, 0.48340E+00_JPRB ,&
&  0.47829E+00_JPRB, 0.47037E+00_JPRB, 0.46248E+00_JPRB, 0.45616E+00_JPRB, 0.45136E+00_JPRB, 0.44629E+00_JPRB, 0.44143E+00_JPRB ,&
&  0.43691E+00_JPRB, 0.43278E+00_JPRB, 0.42900E+00_JPRB, 0.42828E+00_JPRB, 0.42784E+00_JPRB, 0.42660E+00_JPRB, 0.42427E+00_JPRB ,&
&  0.43060E+00_JPRB, 0.38135E+00_JPRB, 0.39584E+00_JPRB, 0.41207E+00_JPRB, 0.43058E+00_JPRB, 0.47579E+00_JPRB, 0.54091E+00_JPRB ,&
&  0.59981E+00_JPRB, 0.64147E+00_JPRB, 0.65327E+00_JPRB, 0.63723E+00_JPRB, 0.61701E+00_JPRB, 0.60400E+00_JPRB, 0.59307E+00_JPRB ,&
&  0.57202E+00_JPRB, 0.55310E+00_JPRB, 0.53496E+00_JPRB, 0.51716E+00_JPRB, 0.50045E+00_JPRB, 0.48600E+00_JPRB, 0.47369E+00_JPRB ,&
&  0.46587E+00_JPRB, 0.46247E+00_JPRB, 0.46038E+00_JPRB, 0.45906E+00_JPRB, 0.45993E+00_JPRB, 0.46257E+00_JPRB, 0.46475E+00_JPRB ,&
&  0.46771E+00_JPRB, 0.47283E+00_JPRB, 0.47391E+00_JPRB, 0.47492E+00_JPRB, 0.47840E+00_JPRB, 0.48574E+00_JPRB, 0.48710E+00_JPRB ,&
&  0.48275E+00_JPRB, 0.47456E+00_JPRB, 0.46571E+00_JPRB, 0.45808E+00_JPRB, 0.45175E+00_JPRB, 0.44600E+00_JPRB, 0.44394E+00_JPRB ,&
&  0.44212E+00_JPRB, 0.44232E+00_JPRB, 0.44331E+00_JPRB, 0.44665E+00_JPRB, 0.45071E+00_JPRB, 0.45550E+00_JPRB, 0.46107E+00_JPRB ,&
&  0.46716E+00_JPRB, 0.47350E+00_JPRB, 0.48198E+00_JPRB, 0.49146E+00_JPRB, 0.49948E+00_JPRB, 0.50465E+00_JPRB, 0.50854E+00_JPRB ,&
&  0.51323E+00_JPRB, 0.52311E+00_JPRB, 0.53143E+00_JPRB, 0.57197E+00_JPRB, 0.59855E+00_JPRB, 0.62087E+00_JPRB, 0.61719E+00_JPRB ,&
&  0.59983E+00_JPRB, 0.57914E+00_JPRB, 0.49163E+00_JPRB, 0.52393E+00_JPRB, 0.53349E+00_JPRB /)
ZBURD(:, 56, 10) = (/ &
&  0.50669E+00_JPRB, 0.50149E+00_JPRB, 0.49527E+00_JPRB, 0.48774E+00_JPRB, 0.48148E+00_JPRB, 0.47689E+00_JPRB, 0.47442E+00_JPRB ,&
&  0.47440E+00_JPRB, 0.47516E+00_JPRB, 0.47318E+00_JPRB, 0.46717E+00_JPRB, 0.46117E+00_JPRB, 0.45714E+00_JPRB, 0.45487E+00_JPRB ,&
&  0.45188E+00_JPRB, 0.44705E+00_JPRB, 0.44085E+00_JPRB, 0.43487E+00_JPRB, 0.42828E+00_JPRB, 0.42282E+00_JPRB, 0.41824E+00_JPRB ,&
&  0.41444E+00_JPRB, 0.41241E+00_JPRB, 0.41082E+00_JPRB, 0.40973E+00_JPRB, 0.40995E+00_JPRB, 0.40943E+00_JPRB, 0.41373E+00_JPRB ,&
&  0.38765E+00_JPRB, 0.35632E+00_JPRB, 0.36920E+00_JPRB, 0.38165E+00_JPRB, 0.39370E+00_JPRB, 0.42159E+00_JPRB, 0.46169E+00_JPRB ,&
&  0.50904E+00_JPRB, 0.54993E+00_JPRB, 0.57411E+00_JPRB, 0.57199E+00_JPRB, 0.55862E+00_JPRB, 0.54973E+00_JPRB, 0.54533E+00_JPRB ,&
&  0.53456E+00_JPRB, 0.52135E+00_JPRB, 0.51005E+00_JPRB, 0.49660E+00_JPRB, 0.48216E+00_JPRB, 0.46901E+00_JPRB, 0.45843E+00_JPRB ,&
&  0.45027E+00_JPRB, 0.44535E+00_JPRB, 0.44387E+00_JPRB, 0.44218E+00_JPRB, 0.44066E+00_JPRB, 0.44075E+00_JPRB, 0.43989E+00_JPRB ,&
&  0.43973E+00_JPRB, 0.43928E+00_JPRB, 0.43970E+00_JPRB, 0.43956E+00_JPRB, 0.44012E+00_JPRB, 0.43921E+00_JPRB, 0.43705E+00_JPRB ,&
&  0.43379E+00_JPRB, 0.43103E+00_JPRB, 0.42929E+00_JPRB, 0.42551E+00_JPRB, 0.41753E+00_JPRB, 0.40999E+00_JPRB, 0.40597E+00_JPRB ,&
&  0.40493E+00_JPRB, 0.40481E+00_JPRB, 0.40472E+00_JPRB, 0.40613E+00_JPRB, 0.40862E+00_JPRB, 0.41191E+00_JPRB, 0.41623E+00_JPRB ,&
&  0.42162E+00_JPRB, 0.42701E+00_JPRB, 0.43149E+00_JPRB, 0.43604E+00_JPRB, 0.44304E+00_JPRB, 0.44964E+00_JPRB, 0.45636E+00_JPRB ,&
&  0.46380E+00_JPRB, 0.47342E+00_JPRB, 0.48444E+00_JPRB, 0.50325E+00_JPRB, 0.52768E+00_JPRB, 0.54609E+00_JPRB, 0.55220E+00_JPRB ,&
&  0.54785E+00_JPRB, 0.53169E+00_JPRB, 0.52528E+00_JPRB, 0.51526E+00_JPRB, 0.50912E+00_JPRB /)
ZBURD(:, 57, 10) = (/ &
&  0.45672E+00_JPRB, 0.45345E+00_JPRB, 0.44840E+00_JPRB, 0.44221E+00_JPRB, 0.43571E+00_JPRB, 0.43184E+00_JPRB, 0.42960E+00_JPRB ,&
&  0.43002E+00_JPRB, 0.42907E+00_JPRB, 0.42964E+00_JPRB, 0.42675E+00_JPRB, 0.42333E+00_JPRB, 0.42060E+00_JPRB, 0.41789E+00_JPRB ,&
&  0.41492E+00_JPRB, 0.41121E+00_JPRB, 0.40754E+00_JPRB, 0.40445E+00_JPRB, 0.40082E+00_JPRB, 0.39595E+00_JPRB, 0.39146E+00_JPRB ,&
&  0.38800E+00_JPRB, 0.38628E+00_JPRB, 0.38522E+00_JPRB, 0.38450E+00_JPRB, 0.38388E+00_JPRB, 0.38481E+00_JPRB, 0.38534E+00_JPRB ,&
&  0.38793E+00_JPRB, 0.35856E+00_JPRB, 0.36947E+00_JPRB, 0.37382E+00_JPRB, 0.38182E+00_JPRB, 0.39894E+00_JPRB, 0.41996E+00_JPRB ,&
&  0.44407E+00_JPRB, 0.46868E+00_JPRB, 0.48735E+00_JPRB, 0.49286E+00_JPRB, 0.48772E+00_JPRB, 0.48435E+00_JPRB, 0.48573E+00_JPRB ,&
&  0.48427E+00_JPRB, 0.47966E+00_JPRB, 0.47551E+00_JPRB, 0.47028E+00_JPRB, 0.46247E+00_JPRB, 0.45151E+00_JPRB, 0.44095E+00_JPRB ,&
&  0.43480E+00_JPRB, 0.43129E+00_JPRB, 0.42811E+00_JPRB, 0.42462E+00_JPRB, 0.41899E+00_JPRB, 0.41354E+00_JPRB, 0.40874E+00_JPRB ,&
&  0.40449E+00_JPRB, 0.39896E+00_JPRB, 0.39567E+00_JPRB, 0.39363E+00_JPRB, 0.39056E+00_JPRB, 0.38865E+00_JPRB, 0.38686E+00_JPRB ,&
&  0.38455E+00_JPRB, 0.38165E+00_JPRB, 0.37980E+00_JPRB, 0.37898E+00_JPRB, 0.37667E+00_JPRB, 0.37434E+00_JPRB, 0.37169E+00_JPRB ,&
&  0.37060E+00_JPRB, 0.36999E+00_JPRB, 0.36901E+00_JPRB, 0.36883E+00_JPRB, 0.36978E+00_JPRB, 0.37249E+00_JPRB, 0.37607E+00_JPRB ,&
&  0.38023E+00_JPRB, 0.38504E+00_JPRB, 0.38945E+00_JPRB, 0.39317E+00_JPRB, 0.39830E+00_JPRB, 0.40274E+00_JPRB, 0.40871E+00_JPRB ,&
&  0.41620E+00_JPRB, 0.42459E+00_JPRB, 0.43428E+00_JPRB, 0.44727E+00_JPRB, 0.46195E+00_JPRB, 0.46999E+00_JPRB, 0.47617E+00_JPRB ,&
&  0.48027E+00_JPRB, 0.47555E+00_JPRB, 0.47079E+00_JPRB, 0.46423E+00_JPRB, 0.45856E+00_JPRB /)
ZBURD(:, 58, 10) = (/ &
&  0.41787E+00_JPRB, 0.41284E+00_JPRB, 0.40712E+00_JPRB, 0.40244E+00_JPRB, 0.39800E+00_JPRB, 0.39298E+00_JPRB, 0.38971E+00_JPRB ,&
&  0.38804E+00_JPRB, 0.38759E+00_JPRB, 0.38849E+00_JPRB, 0.38867E+00_JPRB, 0.38845E+00_JPRB, 0.38886E+00_JPRB, 0.38823E+00_JPRB ,&
&  0.38714E+00_JPRB, 0.38549E+00_JPRB, 0.38335E+00_JPRB, 0.38043E+00_JPRB, 0.37683E+00_JPRB, 0.37346E+00_JPRB, 0.36905E+00_JPRB ,&
&  0.36590E+00_JPRB, 0.36403E+00_JPRB, 0.36300E+00_JPRB, 0.36270E+00_JPRB, 0.36378E+00_JPRB, 0.36530E+00_JPRB, 0.36670E+00_JPRB ,&
&  0.36613E+00_JPRB, 0.35847E+00_JPRB, 0.36436E+00_JPRB, 0.36697E+00_JPRB, 0.37485E+00_JPRB, 0.38874E+00_JPRB, 0.40129E+00_JPRB ,&
&  0.40984E+00_JPRB, 0.42090E+00_JPRB, 0.43253E+00_JPRB, 0.44003E+00_JPRB, 0.43959E+00_JPRB, 0.43794E+00_JPRB, 0.44042E+00_JPRB ,&
&  0.44203E+00_JPRB, 0.44119E+00_JPRB, 0.44045E+00_JPRB, 0.43879E+00_JPRB, 0.43558E+00_JPRB, 0.43180E+00_JPRB, 0.42586E+00_JPRB ,&
&  0.41977E+00_JPRB, 0.41507E+00_JPRB, 0.41019E+00_JPRB, 0.40518E+00_JPRB, 0.39808E+00_JPRB, 0.38993E+00_JPRB, 0.38061E+00_JPRB ,&
&  0.37088E+00_JPRB, 0.36393E+00_JPRB, 0.35951E+00_JPRB, 0.35540E+00_JPRB, 0.35136E+00_JPRB, 0.34872E+00_JPRB, 0.34565E+00_JPRB ,&
&  0.34425E+00_JPRB, 0.34454E+00_JPRB, 0.34361E+00_JPRB, 0.34310E+00_JPRB, 0.34382E+00_JPRB, 0.34389E+00_JPRB, 0.34464E+00_JPRB ,&
&  0.34391E+00_JPRB, 0.34288E+00_JPRB, 0.34231E+00_JPRB, 0.34179E+00_JPRB, 0.34120E+00_JPRB, 0.34252E+00_JPRB, 0.34510E+00_JPRB ,&
&  0.34795E+00_JPRB, 0.35210E+00_JPRB, 0.35662E+00_JPRB, 0.36018E+00_JPRB, 0.36337E+00_JPRB, 0.36834E+00_JPRB, 0.37466E+00_JPRB ,&
&  0.38212E+00_JPRB, 0.39144E+00_JPRB, 0.40118E+00_JPRB, 0.41168E+00_JPRB, 0.41963E+00_JPRB, 0.42425E+00_JPRB, 0.42804E+00_JPRB ,&
&  0.43080E+00_JPRB, 0.42783E+00_JPRB, 0.42673E+00_JPRB, 0.42572E+00_JPRB, 0.42222E+00_JPRB /)
ZBURD(:, 59, 10) = (/ &
&  0.36723E+00_JPRB, 0.36464E+00_JPRB, 0.36227E+00_JPRB, 0.36066E+00_JPRB, 0.35914E+00_JPRB, 0.35774E+00_JPRB, 0.35693E+00_JPRB ,&
&  0.35702E+00_JPRB, 0.35682E+00_JPRB, 0.35926E+00_JPRB, 0.36021E+00_JPRB, 0.36031E+00_JPRB, 0.36125E+00_JPRB, 0.36186E+00_JPRB ,&
&  0.36047E+00_JPRB, 0.35931E+00_JPRB, 0.35803E+00_JPRB, 0.35565E+00_JPRB, 0.35185E+00_JPRB, 0.34972E+00_JPRB, 0.34780E+00_JPRB ,&
&  0.34485E+00_JPRB, 0.34135E+00_JPRB, 0.33832E+00_JPRB, 0.33683E+00_JPRB, 0.33703E+00_JPRB, 0.33679E+00_JPRB, 0.33751E+00_JPRB ,&
&  0.33758E+00_JPRB, 0.34171E+00_JPRB, 0.33040E+00_JPRB, 0.34007E+00_JPRB, 0.34907E+00_JPRB, 0.35795E+00_JPRB, 0.36733E+00_JPRB ,&
&  0.37385E+00_JPRB, 0.38034E+00_JPRB, 0.38874E+00_JPRB, 0.39288E+00_JPRB, 0.39308E+00_JPRB, 0.39021E+00_JPRB, 0.38971E+00_JPRB ,&
&  0.38951E+00_JPRB, 0.38688E+00_JPRB, 0.38549E+00_JPRB, 0.38628E+00_JPRB, 0.38722E+00_JPRB, 0.38635E+00_JPRB, 0.38457E+00_JPRB ,&
&  0.38304E+00_JPRB, 0.37998E+00_JPRB, 0.37475E+00_JPRB, 0.36889E+00_JPRB, 0.36285E+00_JPRB, 0.35574E+00_JPRB, 0.34645E+00_JPRB ,&
&  0.33599E+00_JPRB, 0.32770E+00_JPRB, 0.32084E+00_JPRB, 0.31642E+00_JPRB, 0.31416E+00_JPRB, 0.31189E+00_JPRB, 0.30974E+00_JPRB ,&
&  0.30902E+00_JPRB, 0.30928E+00_JPRB, 0.30934E+00_JPRB, 0.30910E+00_JPRB, 0.30993E+00_JPRB, 0.31083E+00_JPRB, 0.31373E+00_JPRB ,&
&  0.31696E+00_JPRB, 0.31834E+00_JPRB, 0.31870E+00_JPRB, 0.31908E+00_JPRB, 0.31790E+00_JPRB, 0.31848E+00_JPRB, 0.32027E+00_JPRB ,&
&  0.32229E+00_JPRB, 0.32501E+00_JPRB, 0.32701E+00_JPRB, 0.32889E+00_JPRB, 0.33138E+00_JPRB, 0.33508E+00_JPRB, 0.34065E+00_JPRB ,&
&  0.34607E+00_JPRB, 0.35245E+00_JPRB, 0.36038E+00_JPRB, 0.36882E+00_JPRB, 0.37509E+00_JPRB, 0.37886E+00_JPRB, 0.38234E+00_JPRB ,&
&  0.38377E+00_JPRB, 0.38100E+00_JPRB, 0.37680E+00_JPRB, 0.37349E+00_JPRB, 0.37034E+00_JPRB /)
ZBURD(:, 60, 10) = (/ &
&  0.32757E+00_JPRB, 0.32719E+00_JPRB, 0.32768E+00_JPRB, 0.32766E+00_JPRB, 0.32711E+00_JPRB, 0.32740E+00_JPRB, 0.32981E+00_JPRB ,&
&  0.33181E+00_JPRB, 0.33292E+00_JPRB, 0.33390E+00_JPRB, 0.33436E+00_JPRB, 0.33455E+00_JPRB, 0.33620E+00_JPRB, 0.33748E+00_JPRB ,&
&  0.33764E+00_JPRB, 0.33650E+00_JPRB, 0.33517E+00_JPRB, 0.33349E+00_JPRB, 0.33121E+00_JPRB, 0.32877E+00_JPRB, 0.32677E+00_JPRB ,&
&  0.32414E+00_JPRB, 0.32121E+00_JPRB, 0.31836E+00_JPRB, 0.31715E+00_JPRB, 0.31700E+00_JPRB, 0.31707E+00_JPRB, 0.31792E+00_JPRB ,&
&  0.31871E+00_JPRB, 0.32132E+00_JPRB, 0.32440E+00_JPRB, 0.32803E+00_JPRB, 0.33160E+00_JPRB, 0.33628E+00_JPRB, 0.34139E+00_JPRB ,&
&  0.34616E+00_JPRB, 0.34863E+00_JPRB, 0.35082E+00_JPRB, 0.34966E+00_JPRB, 0.35062E+00_JPRB, 0.35110E+00_JPRB, 0.35124E+00_JPRB ,&
&  0.35019E+00_JPRB, 0.34486E+00_JPRB, 0.34067E+00_JPRB, 0.34045E+00_JPRB, 0.34164E+00_JPRB, 0.34264E+00_JPRB, 0.34330E+00_JPRB ,&
&  0.34271E+00_JPRB, 0.34126E+00_JPRB, 0.33788E+00_JPRB, 0.33322E+00_JPRB, 0.32821E+00_JPRB, 0.32232E+00_JPRB, 0.31414E+00_JPRB ,&
&  0.30653E+00_JPRB, 0.30037E+00_JPRB, 0.29559E+00_JPRB, 0.29144E+00_JPRB, 0.28863E+00_JPRB, 0.28559E+00_JPRB, 0.28449E+00_JPRB ,&
&  0.28466E+00_JPRB, 0.28650E+00_JPRB, 0.28756E+00_JPRB, 0.28690E+00_JPRB, 0.28580E+00_JPRB, 0.28601E+00_JPRB, 0.28842E+00_JPRB ,&
&  0.29278E+00_JPRB, 0.29615E+00_JPRB, 0.29736E+00_JPRB, 0.29714E+00_JPRB, 0.29646E+00_JPRB, 0.29741E+00_JPRB, 0.29933E+00_JPRB ,&
&  0.30036E+00_JPRB, 0.30098E+00_JPRB, 0.30218E+00_JPRB, 0.30213E+00_JPRB, 0.30410E+00_JPRB, 0.30741E+00_JPRB, 0.31107E+00_JPRB ,&
&  0.31460E+00_JPRB, 0.31907E+00_JPRB, 0.32398E+00_JPRB, 0.32886E+00_JPRB, 0.33371E+00_JPRB, 0.33570E+00_JPRB, 0.33749E+00_JPRB ,&
&  0.33853E+00_JPRB, 0.33757E+00_JPRB, 0.33573E+00_JPRB, 0.33306E+00_JPRB, 0.32985E+00_JPRB /)
ZBURD(:, 61, 10) = (/ &
&  0.29534E+00_JPRB, 0.29790E+00_JPRB, 0.30122E+00_JPRB, 0.30185E+00_JPRB, 0.30214E+00_JPRB, 0.30357E+00_JPRB, 0.30657E+00_JPRB ,&
&  0.30858E+00_JPRB, 0.30905E+00_JPRB, 0.30890E+00_JPRB, 0.30863E+00_JPRB, 0.30906E+00_JPRB, 0.31079E+00_JPRB, 0.31162E+00_JPRB ,&
&  0.31103E+00_JPRB, 0.31039E+00_JPRB, 0.30951E+00_JPRB, 0.30880E+00_JPRB, 0.30724E+00_JPRB, 0.30493E+00_JPRB, 0.30298E+00_JPRB ,&
&  0.30028E+00_JPRB, 0.29778E+00_JPRB, 0.29590E+00_JPRB, 0.29441E+00_JPRB, 0.29355E+00_JPRB, 0.29349E+00_JPRB, 0.29498E+00_JPRB ,&
&  0.29687E+00_JPRB, 0.29934E+00_JPRB, 0.30242E+00_JPRB, 0.30601E+00_JPRB, 0.30958E+00_JPRB, 0.31362E+00_JPRB, 0.31720E+00_JPRB ,&
&  0.32002E+00_JPRB, 0.32242E+00_JPRB, 0.32407E+00_JPRB, 0.32149E+00_JPRB, 0.31837E+00_JPRB, 0.31829E+00_JPRB, 0.31869E+00_JPRB ,&
&  0.31642E+00_JPRB, 0.31252E+00_JPRB, 0.30741E+00_JPRB, 0.30458E+00_JPRB, 0.30381E+00_JPRB, 0.30274E+00_JPRB, 0.30143E+00_JPRB ,&
&  0.30055E+00_JPRB, 0.29979E+00_JPRB, 0.29840E+00_JPRB, 0.29607E+00_JPRB, 0.29240E+00_JPRB, 0.28748E+00_JPRB, 0.28261E+00_JPRB ,&
&  0.27952E+00_JPRB, 0.27735E+00_JPRB, 0.27727E+00_JPRB, 0.27516E+00_JPRB, 0.27095E+00_JPRB, 0.26696E+00_JPRB, 0.26617E+00_JPRB ,&
&  0.26603E+00_JPRB, 0.26657E+00_JPRB, 0.26660E+00_JPRB, 0.26569E+00_JPRB, 0.26608E+00_JPRB, 0.26696E+00_JPRB, 0.26852E+00_JPRB ,&
&  0.27284E+00_JPRB, 0.27592E+00_JPRB, 0.27591E+00_JPRB, 0.27490E+00_JPRB, 0.27300E+00_JPRB, 0.27256E+00_JPRB, 0.27322E+00_JPRB ,&
&  0.27395E+00_JPRB, 0.27432E+00_JPRB, 0.27464E+00_JPRB, 0.27413E+00_JPRB, 0.27428E+00_JPRB, 0.27579E+00_JPRB, 0.27768E+00_JPRB ,&
&  0.27998E+00_JPRB, 0.28279E+00_JPRB, 0.28532E+00_JPRB, 0.28823E+00_JPRB, 0.29124E+00_JPRB, 0.29346E+00_JPRB, 0.29544E+00_JPRB ,&
&  0.29700E+00_JPRB, 0.29744E+00_JPRB, 0.29708E+00_JPRB, 0.29669E+00_JPRB, 0.29620E+00_JPRB /)
ZBURD(:, 62, 10) = (/ &
&  0.27138E+00_JPRB, 0.27552E+00_JPRB, 0.28080E+00_JPRB, 0.28196E+00_JPRB, 0.28433E+00_JPRB, 0.28687E+00_JPRB, 0.29032E+00_JPRB ,&
&  0.29254E+00_JPRB, 0.29272E+00_JPRB, 0.29116E+00_JPRB, 0.28957E+00_JPRB, 0.28877E+00_JPRB, 0.28860E+00_JPRB, 0.28816E+00_JPRB ,&
&  0.28802E+00_JPRB, 0.28820E+00_JPRB, 0.28813E+00_JPRB, 0.28790E+00_JPRB, 0.28664E+00_JPRB, 0.28476E+00_JPRB, 0.28268E+00_JPRB ,&
&  0.27921E+00_JPRB, 0.27690E+00_JPRB, 0.27611E+00_JPRB, 0.27577E+00_JPRB, 0.27496E+00_JPRB, 0.27542E+00_JPRB, 0.27715E+00_JPRB ,&
&  0.28091E+00_JPRB, 0.28390E+00_JPRB, 0.28776E+00_JPRB, 0.29170E+00_JPRB, 0.29405E+00_JPRB, 0.29578E+00_JPRB, 0.29781E+00_JPRB ,&
&  0.29909E+00_JPRB, 0.29942E+00_JPRB, 0.29950E+00_JPRB, 0.29954E+00_JPRB, 0.29695E+00_JPRB, 0.29650E+00_JPRB, 0.29675E+00_JPRB ,&
&  0.29605E+00_JPRB, 0.29334E+00_JPRB, 0.28944E+00_JPRB, 0.28612E+00_JPRB, 0.28285E+00_JPRB, 0.28043E+00_JPRB, 0.27841E+00_JPRB ,&
&  0.27688E+00_JPRB, 0.27608E+00_JPRB, 0.27590E+00_JPRB, 0.27426E+00_JPRB, 0.26891E+00_JPRB, 0.26368E+00_JPRB, 0.26076E+00_JPRB ,&
&  0.25940E+00_JPRB, 0.25907E+00_JPRB, 0.26043E+00_JPRB, 0.25915E+00_JPRB, 0.25520E+00_JPRB, 0.25192E+00_JPRB, 0.25052E+00_JPRB ,&
&  0.24887E+00_JPRB, 0.24815E+00_JPRB, 0.24892E+00_JPRB, 0.24960E+00_JPRB, 0.25023E+00_JPRB, 0.25045E+00_JPRB, 0.25179E+00_JPRB ,&
&  0.25316E+00_JPRB, 0.25485E+00_JPRB, 0.25475E+00_JPRB, 0.25203E+00_JPRB, 0.24825E+00_JPRB, 0.24554E+00_JPRB, 0.24511E+00_JPRB ,&
&  0.24673E+00_JPRB, 0.24834E+00_JPRB, 0.24890E+00_JPRB, 0.24917E+00_JPRB, 0.25034E+00_JPRB, 0.25287E+00_JPRB, 0.25612E+00_JPRB ,&
&  0.25754E+00_JPRB, 0.25890E+00_JPRB, 0.26077E+00_JPRB, 0.26273E+00_JPRB, 0.26421E+00_JPRB, 0.26533E+00_JPRB, 0.26598E+00_JPRB ,&
&  0.26564E+00_JPRB, 0.26650E+00_JPRB, 0.26808E+00_JPRB, 0.26986E+00_JPRB, 0.27102E+00_JPRB /)
ZBURD(:, 63, 10) = (/ &
&  0.25565E+00_JPRB, 0.25877E+00_JPRB, 0.26176E+00_JPRB, 0.26341E+00_JPRB, 0.26520E+00_JPRB, 0.26713E+00_JPRB, 0.27052E+00_JPRB ,&
&  0.27451E+00_JPRB, 0.27583E+00_JPRB, 0.27406E+00_JPRB, 0.27117E+00_JPRB, 0.26935E+00_JPRB, 0.26861E+00_JPRB, 0.26847E+00_JPRB ,&
&  0.26838E+00_JPRB, 0.26829E+00_JPRB, 0.26753E+00_JPRB, 0.26570E+00_JPRB, 0.26311E+00_JPRB, 0.26047E+00_JPRB, 0.25846E+00_JPRB ,&
&  0.25808E+00_JPRB, 0.25945E+00_JPRB, 0.26166E+00_JPRB, 0.26183E+00_JPRB, 0.26192E+00_JPRB, 0.26317E+00_JPRB, 0.26525E+00_JPRB ,&
&  0.26922E+00_JPRB, 0.27115E+00_JPRB, 0.27308E+00_JPRB, 0.26912E+00_JPRB, 0.27350E+00_JPRB, 0.27508E+00_JPRB, 0.27556E+00_JPRB ,&
&  0.27680E+00_JPRB, 0.27781E+00_JPRB, 0.27886E+00_JPRB, 0.27824E+00_JPRB, 0.27409E+00_JPRB, 0.27259E+00_JPRB, 0.27383E+00_JPRB ,&
&  0.27489E+00_JPRB, 0.27331E+00_JPRB, 0.27033E+00_JPRB, 0.26752E+00_JPRB, 0.26518E+00_JPRB, 0.26347E+00_JPRB, 0.26201E+00_JPRB ,&
&  0.26124E+00_JPRB, 0.26033E+00_JPRB, 0.25823E+00_JPRB, 0.25287E+00_JPRB, 0.24682E+00_JPRB, 0.24281E+00_JPRB, 0.23997E+00_JPRB ,&
&  0.23989E+00_JPRB, 0.24178E+00_JPRB, 0.24344E+00_JPRB, 0.24348E+00_JPRB, 0.24202E+00_JPRB, 0.23944E+00_JPRB, 0.23585E+00_JPRB ,&
&  0.23086E+00_JPRB, 0.22887E+00_JPRB, 0.22840E+00_JPRB, 0.22757E+00_JPRB, 0.22806E+00_JPRB, 0.22994E+00_JPRB, 0.23194E+00_JPRB ,&
&  0.23435E+00_JPRB, 0.23439E+00_JPRB, 0.23294E+00_JPRB, 0.23049E+00_JPRB, 0.22792E+00_JPRB, 0.22768E+00_JPRB, 0.22834E+00_JPRB ,&
&  0.22897E+00_JPRB, 0.22959E+00_JPRB, 0.23060E+00_JPRB, 0.23195E+00_JPRB, 0.23369E+00_JPRB, 0.23610E+00_JPRB, 0.23808E+00_JPRB ,&
&  0.23808E+00_JPRB, 0.23697E+00_JPRB, 0.23806E+00_JPRB, 0.23989E+00_JPRB, 0.24091E+00_JPRB, 0.24160E+00_JPRB, 0.24213E+00_JPRB ,&
&  0.24186E+00_JPRB, 0.24193E+00_JPRB, 0.24387E+00_JPRB, 0.24856E+00_JPRB, 0.25280E+00_JPRB /)
ZBURD(:, 64, 10) = (/ &
&  0.24196E+00_JPRB, 0.24327E+00_JPRB, 0.24344E+00_JPRB, 0.24386E+00_JPRB, 0.24449E+00_JPRB, 0.24636E+00_JPRB, 0.25054E+00_JPRB ,&
&  0.25468E+00_JPRB, 0.25654E+00_JPRB, 0.25490E+00_JPRB, 0.25250E+00_JPRB, 0.25126E+00_JPRB, 0.25065E+00_JPRB, 0.25050E+00_JPRB ,&
&  0.25038E+00_JPRB, 0.24942E+00_JPRB, 0.24660E+00_JPRB, 0.24267E+00_JPRB, 0.23975E+00_JPRB, 0.23898E+00_JPRB, 0.23936E+00_JPRB ,&
&  0.24306E+00_JPRB, 0.24563E+00_JPRB, 0.24800E+00_JPRB, 0.25012E+00_JPRB, 0.25066E+00_JPRB, 0.25040E+00_JPRB, 0.25193E+00_JPRB ,&
&  0.25299E+00_JPRB, 0.25054E+00_JPRB, 0.24518E+00_JPRB, 0.24915E+00_JPRB, 0.25131E+00_JPRB, 0.25264E+00_JPRB, 0.25191E+00_JPRB ,&
&  0.25257E+00_JPRB, 0.25468E+00_JPRB, 0.25678E+00_JPRB, 0.25842E+00_JPRB, 0.25778E+00_JPRB, 0.25495E+00_JPRB, 0.25430E+00_JPRB ,&
&  0.25481E+00_JPRB, 0.25374E+00_JPRB, 0.25091E+00_JPRB, 0.24832E+00_JPRB, 0.24543E+00_JPRB, 0.24410E+00_JPRB, 0.24385E+00_JPRB ,&
&  0.24294E+00_JPRB, 0.24069E+00_JPRB, 0.23628E+00_JPRB, 0.23196E+00_JPRB, 0.22877E+00_JPRB, 0.22561E+00_JPRB, 0.22416E+00_JPRB ,&
&  0.22448E+00_JPRB, 0.22557E+00_JPRB, 0.22552E+00_JPRB, 0.22392E+00_JPRB, 0.21873E+00_JPRB, 0.20880E+00_JPRB, 0.20155E+00_JPRB ,&
&  0.20022E+00_JPRB, 0.20465E+00_JPRB, 0.21252E+00_JPRB, 0.21418E+00_JPRB, 0.21653E+00_JPRB, 0.21812E+00_JPRB, 0.21952E+00_JPRB ,&
&  0.21793E+00_JPRB, 0.20515E+00_JPRB, 0.19951E+00_JPRB, 0.19621E+00_JPRB, 0.19311E+00_JPRB, 0.19283E+00_JPRB, 0.19451E+00_JPRB ,&
&  0.19788E+00_JPRB, 0.20206E+00_JPRB, 0.20501E+00_JPRB, 0.20676E+00_JPRB, 0.20731E+00_JPRB, 0.20710E+00_JPRB, 0.20642E+00_JPRB ,&
&  0.20671E+00_JPRB, 0.21067E+00_JPRB, 0.21858E+00_JPRB, 0.22648E+00_JPRB, 0.22752E+00_JPRB, 0.22914E+00_JPRB, 0.22955E+00_JPRB ,&
&  0.22993E+00_JPRB, 0.23183E+00_JPRB, 0.23299E+00_JPRB, 0.23576E+00_JPRB, 0.23918E+00_JPRB /)
ZBURD(:, 65, 10) = (/ &
&  0.23043E+00_JPRB, 0.23160E+00_JPRB, 0.23171E+00_JPRB, 0.23079E+00_JPRB, 0.22951E+00_JPRB, 0.22970E+00_JPRB, 0.23113E+00_JPRB ,&
&  0.23406E+00_JPRB, 0.23616E+00_JPRB, 0.23661E+00_JPRB, 0.23659E+00_JPRB, 0.23569E+00_JPRB, 0.23368E+00_JPRB, 0.23203E+00_JPRB ,&
&  0.23072E+00_JPRB, 0.22996E+00_JPRB, 0.22935E+00_JPRB, 0.22917E+00_JPRB, 0.22915E+00_JPRB, 0.23013E+00_JPRB, 0.23105E+00_JPRB ,&
&  0.23467E+00_JPRB, 0.23465E+00_JPRB, 0.23397E+00_JPRB, 0.23376E+00_JPRB, 0.23357E+00_JPRB, 0.23362E+00_JPRB, 0.23356E+00_JPRB ,&
&  0.23080E+00_JPRB, 0.22312E+00_JPRB, 0.21644E+00_JPRB, 0.22211E+00_JPRB, 0.23382E+00_JPRB, 0.23417E+00_JPRB, 0.23432E+00_JPRB ,&
&  0.23568E+00_JPRB, 0.23853E+00_JPRB, 0.24059E+00_JPRB, 0.24162E+00_JPRB, 0.24050E+00_JPRB, 0.23888E+00_JPRB, 0.23850E+00_JPRB ,&
&  0.23883E+00_JPRB, 0.23704E+00_JPRB, 0.23173E+00_JPRB, 0.22559E+00_JPRB, 0.21947E+00_JPRB, 0.21706E+00_JPRB, 0.21544E+00_JPRB ,&
&  0.21292E+00_JPRB, 0.21141E+00_JPRB, 0.21085E+00_JPRB, 0.21036E+00_JPRB, 0.21076E+00_JPRB, 0.20862E+00_JPRB, 0.20545E+00_JPRB ,&
&  0.20202E+00_JPRB, 0.19785E+00_JPRB, 0.19291E+00_JPRB, 0.18767E+00_JPRB, 0.18321E+00_JPRB, 0.17828E+00_JPRB, 0.17488E+00_JPRB ,&
&  0.17426E+00_JPRB, 0.17824E+00_JPRB, 0.18793E+00_JPRB, 0.19797E+00_JPRB, 0.20477E+00_JPRB, 0.19445E+00_JPRB, 0.18789E+00_JPRB ,&
&  0.18066E+00_JPRB, 0.17470E+00_JPRB, 0.17130E+00_JPRB, 0.17061E+00_JPRB, 0.17086E+00_JPRB, 0.17175E+00_JPRB, 0.17346E+00_JPRB ,&
&  0.17566E+00_JPRB, 0.17842E+00_JPRB, 0.18051E+00_JPRB, 0.18203E+00_JPRB, 0.18312E+00_JPRB, 0.18344E+00_JPRB, 0.18322E+00_JPRB ,&
&  0.18298E+00_JPRB, 0.18424E+00_JPRB, 0.18664E+00_JPRB, 0.19095E+00_JPRB, 0.19389E+00_JPRB, 0.19773E+00_JPRB, 0.20291E+00_JPRB ,&
&  0.21048E+00_JPRB, 0.21927E+00_JPRB, 0.22269E+00_JPRB, 0.22506E+00_JPRB, 0.22803E+00_JPRB /)
ZBURD(:, 66, 10) = (/ &
&  0.21843E+00_JPRB, 0.22288E+00_JPRB, 0.22521E+00_JPRB, 0.22464E+00_JPRB, 0.22424E+00_JPRB, 0.22480E+00_JPRB, 0.22552E+00_JPRB ,&
&  0.22652E+00_JPRB, 0.22604E+00_JPRB, 0.22459E+00_JPRB, 0.22323E+00_JPRB, 0.22175E+00_JPRB, 0.21982E+00_JPRB, 0.21857E+00_JPRB ,&
&  0.21760E+00_JPRB, 0.21773E+00_JPRB, 0.21755E+00_JPRB, 0.21852E+00_JPRB, 0.21908E+00_JPRB, 0.22129E+00_JPRB, 0.21973E+00_JPRB ,&
&  0.21610E+00_JPRB, 0.21407E+00_JPRB, 0.21636E+00_JPRB, 0.21672E+00_JPRB, 0.21728E+00_JPRB, 0.21806E+00_JPRB, 0.21948E+00_JPRB ,&
&  0.21696E+00_JPRB, 0.21400E+00_JPRB, 0.21031E+00_JPRB, 0.21594E+00_JPRB, 0.22425E+00_JPRB, 0.22364E+00_JPRB, 0.22310E+00_JPRB ,&
&  0.22462E+00_JPRB, 0.22729E+00_JPRB, 0.22860E+00_JPRB, 0.22888E+00_JPRB, 0.22748E+00_JPRB, 0.22688E+00_JPRB, 0.22575E+00_JPRB ,&
&  0.22285E+00_JPRB, 0.22007E+00_JPRB, 0.21118E+00_JPRB, 0.19490E+00_JPRB, 0.18856E+00_JPRB, 0.18568E+00_JPRB, 0.18176E+00_JPRB ,&
&  0.17664E+00_JPRB, 0.17442E+00_JPRB, 0.17430E+00_JPRB, 0.17258E+00_JPRB, 0.17274E+00_JPRB, 0.17231E+00_JPRB, 0.17120E+00_JPRB ,&
&  0.16979E+00_JPRB, 0.16879E+00_JPRB, 0.16726E+00_JPRB, 0.16478E+00_JPRB, 0.16289E+00_JPRB, 0.16246E+00_JPRB, 0.16325E+00_JPRB ,&
&  0.16519E+00_JPRB, 0.16811E+00_JPRB, 0.17656E+00_JPRB, 0.18148E+00_JPRB, 0.17959E+00_JPRB, 0.17586E+00_JPRB, 0.16970E+00_JPRB ,&
&  0.16405E+00_JPRB, 0.16233E+00_JPRB, 0.16112E+00_JPRB, 0.16009E+00_JPRB, 0.16044E+00_JPRB, 0.16151E+00_JPRB, 0.16258E+00_JPRB ,&
&  0.16385E+00_JPRB, 0.16536E+00_JPRB, 0.16631E+00_JPRB, 0.16684E+00_JPRB, 0.16736E+00_JPRB, 0.16763E+00_JPRB, 0.16813E+00_JPRB ,&
&  0.16822E+00_JPRB, 0.16958E+00_JPRB, 0.17074E+00_JPRB, 0.17312E+00_JPRB, 0.17473E+00_JPRB, 0.17681E+00_JPRB, 0.18126E+00_JPRB ,&
&  0.18488E+00_JPRB, 0.19574E+00_JPRB, 0.21055E+00_JPRB, 0.21765E+00_JPRB, 0.21458E+00_JPRB /)
ZBURD(:, 67, 10) = (/ &
&  0.21008E+00_JPRB, 0.21241E+00_JPRB, 0.21454E+00_JPRB, 0.21318E+00_JPRB, 0.21325E+00_JPRB, 0.21351E+00_JPRB, 0.21468E+00_JPRB ,&
&  0.21522E+00_JPRB, 0.21478E+00_JPRB, 0.21445E+00_JPRB, 0.21292E+00_JPRB, 0.21138E+00_JPRB, 0.20463E+00_JPRB, 0.20183E+00_JPRB ,&
&  0.19962E+00_JPRB, 0.20025E+00_JPRB, 0.20080E+00_JPRB, 0.20283E+00_JPRB, 0.20295E+00_JPRB, 0.20190E+00_JPRB, 0.19869E+00_JPRB ,&
&  0.19620E+00_JPRB, 0.19473E+00_JPRB, 0.19316E+00_JPRB, 0.19345E+00_JPRB, 0.19555E+00_JPRB, 0.19777E+00_JPRB, 0.19917E+00_JPRB ,&
&  0.19989E+00_JPRB, 0.20201E+00_JPRB, 0.20388E+00_JPRB, 0.20672E+00_JPRB, 0.21344E+00_JPRB, 0.21435E+00_JPRB, 0.21524E+00_JPRB ,&
&  0.21617E+00_JPRB, 0.21736E+00_JPRB, 0.21807E+00_JPRB, 0.21778E+00_JPRB, 0.21610E+00_JPRB, 0.21104E+00_JPRB, 0.20965E+00_JPRB ,&
&  0.19726E+00_JPRB, 0.18993E+00_JPRB, 0.18309E+00_JPRB, 0.17810E+00_JPRB, 0.17270E+00_JPRB, 0.16978E+00_JPRB, 0.16651E+00_JPRB ,&
&  0.16374E+00_JPRB, 0.16071E+00_JPRB, 0.15847E+00_JPRB, 0.15749E+00_JPRB, 0.15719E+00_JPRB, 0.15690E+00_JPRB, 0.15622E+00_JPRB ,&
&  0.15630E+00_JPRB, 0.15582E+00_JPRB, 0.15548E+00_JPRB, 0.15510E+00_JPRB, 0.15486E+00_JPRB, 0.15492E+00_JPRB, 0.15566E+00_JPRB ,&
&  0.15760E+00_JPRB, 0.15940E+00_JPRB, 0.16127E+00_JPRB, 0.16306E+00_JPRB, 0.16265E+00_JPRB, 0.16098E+00_JPRB, 0.15842E+00_JPRB ,&
&  0.15616E+00_JPRB, 0.15469E+00_JPRB, 0.15312E+00_JPRB, 0.15270E+00_JPRB, 0.15331E+00_JPRB, 0.15406E+00_JPRB, 0.15540E+00_JPRB ,&
&  0.15657E+00_JPRB, 0.15754E+00_JPRB, 0.15820E+00_JPRB, 0.15864E+00_JPRB, 0.15934E+00_JPRB, 0.15999E+00_JPRB, 0.16109E+00_JPRB ,&
&  0.16232E+00_JPRB, 0.16384E+00_JPRB, 0.16521E+00_JPRB, 0.16708E+00_JPRB, 0.16989E+00_JPRB, 0.17410E+00_JPRB, 0.18134E+00_JPRB ,&
&  0.18836E+00_JPRB, 0.20679E+00_JPRB, 0.20897E+00_JPRB, 0.20853E+00_JPRB, 0.21000E+00_JPRB /)
ZBURD(:, 68, 10) = (/ &
&  0.20103E+00_JPRB, 0.20297E+00_JPRB, 0.20432E+00_JPRB, 0.20562E+00_JPRB, 0.20576E+00_JPRB, 0.20599E+00_JPRB, 0.20670E+00_JPRB ,&
&  0.20286E+00_JPRB, 0.20016E+00_JPRB, 0.19560E+00_JPRB, 0.19135E+00_JPRB, 0.18731E+00_JPRB, 0.18394E+00_JPRB, 0.18193E+00_JPRB ,&
&  0.18088E+00_JPRB, 0.18099E+00_JPRB, 0.18118E+00_JPRB, 0.18180E+00_JPRB, 0.18100E+00_JPRB, 0.17998E+00_JPRB, 0.17900E+00_JPRB ,&
&  0.17784E+00_JPRB, 0.17821E+00_JPRB, 0.17877E+00_JPRB, 0.18213E+00_JPRB, 0.18686E+00_JPRB, 0.19217E+00_JPRB, 0.19603E+00_JPRB ,&
&  0.19875E+00_JPRB, 0.20083E+00_JPRB, 0.20344E+00_JPRB, 0.20458E+00_JPRB, 0.20573E+00_JPRB, 0.20706E+00_JPRB, 0.20758E+00_JPRB ,&
&  0.20891E+00_JPRB, 0.20895E+00_JPRB, 0.20883E+00_JPRB, 0.20907E+00_JPRB, 0.20066E+00_JPRB, 0.19447E+00_JPRB, 0.18812E+00_JPRB ,&
&  0.18203E+00_JPRB, 0.17587E+00_JPRB, 0.17031E+00_JPRB, 0.16725E+00_JPRB, 0.16399E+00_JPRB, 0.16210E+00_JPRB, 0.16046E+00_JPRB ,&
&  0.15922E+00_JPRB, 0.15794E+00_JPRB, 0.15612E+00_JPRB, 0.15433E+00_JPRB, 0.15328E+00_JPRB, 0.15239E+00_JPRB, 0.15126E+00_JPRB ,&
&  0.15114E+00_JPRB, 0.15048E+00_JPRB, 0.15041E+00_JPRB, 0.15002E+00_JPRB, 0.14973E+00_JPRB, 0.14948E+00_JPRB, 0.14947E+00_JPRB ,&
&  0.14984E+00_JPRB, 0.15061E+00_JPRB, 0.15091E+00_JPRB, 0.15131E+00_JPRB, 0.15121E+00_JPRB, 0.15060E+00_JPRB, 0.15008E+00_JPRB ,&
&  0.14949E+00_JPRB, 0.14901E+00_JPRB, 0.14902E+00_JPRB, 0.14891E+00_JPRB, 0.14941E+00_JPRB, 0.15009E+00_JPRB, 0.15076E+00_JPRB ,&
&  0.15170E+00_JPRB, 0.15240E+00_JPRB, 0.15312E+00_JPRB, 0.15403E+00_JPRB, 0.15480E+00_JPRB, 0.15580E+00_JPRB, 0.15695E+00_JPRB ,&
&  0.15783E+00_JPRB, 0.15957E+00_JPRB, 0.16102E+00_JPRB, 0.16279E+00_JPRB, 0.16622E+00_JPRB, 0.16980E+00_JPRB, 0.17825E+00_JPRB ,&
&  0.18614E+00_JPRB, 0.20255E+00_JPRB, 0.20385E+00_JPRB, 0.19914E+00_JPRB, 0.19982E+00_JPRB /)
ZBURD(:, 69, 10) = (/ &
&  0.19568E+00_JPRB, 0.19711E+00_JPRB, 0.19778E+00_JPRB, 0.19879E+00_JPRB, 0.19920E+00_JPRB, 0.20069E+00_JPRB, 0.20082E+00_JPRB ,&
&  0.20094E+00_JPRB, 0.19713E+00_JPRB, 0.19483E+00_JPRB, 0.19217E+00_JPRB, 0.18987E+00_JPRB, 0.18717E+00_JPRB, 0.18455E+00_JPRB ,&
&  0.18251E+00_JPRB, 0.18072E+00_JPRB, 0.17878E+00_JPRB, 0.17611E+00_JPRB, 0.17339E+00_JPRB, 0.17143E+00_JPRB, 0.17003E+00_JPRB ,&
&  0.16866E+00_JPRB, 0.16944E+00_JPRB, 0.17103E+00_JPRB, 0.17425E+00_JPRB, 0.17859E+00_JPRB, 0.18221E+00_JPRB, 0.18703E+00_JPRB ,&
&  0.19306E+00_JPRB, 0.19492E+00_JPRB, 0.19733E+00_JPRB, 0.19844E+00_JPRB, 0.19816E+00_JPRB, 0.19874E+00_JPRB, 0.19656E+00_JPRB ,&
&  0.19674E+00_JPRB, 0.19410E+00_JPRB, 0.19859E+00_JPRB, 0.19554E+00_JPRB, 0.18885E+00_JPRB, 0.18581E+00_JPRB, 0.17970E+00_JPRB ,&
&  0.17529E+00_JPRB, 0.17054E+00_JPRB, 0.16737E+00_JPRB, 0.16560E+00_JPRB, 0.16255E+00_JPRB, 0.16090E+00_JPRB, 0.15836E+00_JPRB ,&
&  0.15601E+00_JPRB, 0.15524E+00_JPRB, 0.15380E+00_JPRB, 0.15322E+00_JPRB, 0.15244E+00_JPRB, 0.15134E+00_JPRB, 0.15014E+00_JPRB ,&
&  0.14933E+00_JPRB, 0.14862E+00_JPRB, 0.14825E+00_JPRB, 0.14772E+00_JPRB, 0.14698E+00_JPRB, 0.14624E+00_JPRB, 0.14540E+00_JPRB ,&
&  0.14484E+00_JPRB, 0.14455E+00_JPRB, 0.14433E+00_JPRB, 0.14429E+00_JPRB, 0.14423E+00_JPRB, 0.14401E+00_JPRB, 0.14413E+00_JPRB ,&
&  0.14440E+00_JPRB, 0.14501E+00_JPRB, 0.14552E+00_JPRB, 0.14620E+00_JPRB, 0.14681E+00_JPRB, 0.14762E+00_JPRB, 0.14821E+00_JPRB ,&
&  0.14888E+00_JPRB, 0.14982E+00_JPRB, 0.15063E+00_JPRB, 0.15160E+00_JPRB, 0.15266E+00_JPRB, 0.15355E+00_JPRB, 0.15496E+00_JPRB ,&
&  0.15640E+00_JPRB, 0.15782E+00_JPRB, 0.15983E+00_JPRB, 0.16187E+00_JPRB, 0.16449E+00_JPRB, 0.16889E+00_JPRB, 0.17390E+00_JPRB ,&
&  0.18020E+00_JPRB, 0.18953E+00_JPRB, 0.18984E+00_JPRB, 0.19275E+00_JPRB, 0.19438E+00_JPRB /)
ZBURD(:, 70, 10) = (/ &
&  0.19512E+00_JPRB, 0.19638E+00_JPRB, 0.19743E+00_JPRB, 0.19796E+00_JPRB, 0.19856E+00_JPRB, 0.19859E+00_JPRB, 0.19849E+00_JPRB ,&
&  0.19567E+00_JPRB, 0.19446E+00_JPRB, 0.19269E+00_JPRB, 0.19069E+00_JPRB, 0.18894E+00_JPRB, 0.18661E+00_JPRB, 0.18439E+00_JPRB ,&
&  0.18232E+00_JPRB, 0.17990E+00_JPRB, 0.17742E+00_JPRB, 0.17449E+00_JPRB, 0.17169E+00_JPRB, 0.16905E+00_JPRB, 0.16766E+00_JPRB ,&
&  0.16699E+00_JPRB, 0.16737E+00_JPRB, 0.16899E+00_JPRB, 0.17026E+00_JPRB, 0.17209E+00_JPRB, 0.17441E+00_JPRB, 0.17808E+00_JPRB ,&
&  0.18084E+00_JPRB, 0.18429E+00_JPRB, 0.18656E+00_JPRB, 0.18726E+00_JPRB, 0.18660E+00_JPRB, 0.18627E+00_JPRB, 0.18541E+00_JPRB ,&
&  0.18356E+00_JPRB, 0.18164E+00_JPRB, 0.18000E+00_JPRB, 0.17822E+00_JPRB, 0.17626E+00_JPRB, 0.17391E+00_JPRB, 0.17098E+00_JPRB ,&
&  0.16834E+00_JPRB, 0.16601E+00_JPRB, 0.16383E+00_JPRB, 0.16212E+00_JPRB, 0.16047E+00_JPRB, 0.15895E+00_JPRB, 0.15731E+00_JPRB ,&
&  0.15604E+00_JPRB, 0.15494E+00_JPRB, 0.15355E+00_JPRB, 0.15243E+00_JPRB, 0.15115E+00_JPRB, 0.15009E+00_JPRB, 0.14893E+00_JPRB ,&
&  0.14803E+00_JPRB, 0.14726E+00_JPRB, 0.14662E+00_JPRB, 0.14602E+00_JPRB, 0.14565E+00_JPRB, 0.14530E+00_JPRB, 0.14490E+00_JPRB ,&
&  0.14455E+00_JPRB, 0.14453E+00_JPRB, 0.14456E+00_JPRB, 0.14460E+00_JPRB, 0.14469E+00_JPRB, 0.14438E+00_JPRB, 0.14443E+00_JPRB ,&
&  0.14474E+00_JPRB, 0.14506E+00_JPRB, 0.14514E+00_JPRB, 0.14556E+00_JPRB, 0.14623E+00_JPRB, 0.14704E+00_JPRB, 0.14824E+00_JPRB ,&
&  0.14919E+00_JPRB, 0.15015E+00_JPRB, 0.15106E+00_JPRB, 0.15199E+00_JPRB, 0.15319E+00_JPRB, 0.15449E+00_JPRB, 0.15585E+00_JPRB ,&
&  0.15746E+00_JPRB, 0.15896E+00_JPRB, 0.16053E+00_JPRB, 0.16215E+00_JPRB, 0.16474E+00_JPRB, 0.16723E+00_JPRB, 0.17106E+00_JPRB ,&
&  0.17492E+00_JPRB, 0.17673E+00_JPRB, 0.18991E+00_JPRB, 0.19095E+00_JPRB, 0.19250E+00_JPRB /)
ZBURD(:, 71, 10) = (/ &
&  0.17402E+00_JPRB, 0.17629E+00_JPRB, 0.17783E+00_JPRB, 0.17950E+00_JPRB, 0.17756E+00_JPRB, 0.18912E+00_JPRB, 0.18707E+00_JPRB ,&
&  0.18671E+00_JPRB, 0.18170E+00_JPRB, 0.18210E+00_JPRB, 0.18137E+00_JPRB, 0.17997E+00_JPRB, 0.17838E+00_JPRB, 0.17661E+00_JPRB ,&
&  0.17487E+00_JPRB, 0.17267E+00_JPRB, 0.17085E+00_JPRB, 0.16933E+00_JPRB, 0.16794E+00_JPRB, 0.16672E+00_JPRB, 0.16550E+00_JPRB ,&
&  0.16535E+00_JPRB, 0.16601E+00_JPRB, 0.16707E+00_JPRB, 0.16637E+00_JPRB, 0.16674E+00_JPRB, 0.16758E+00_JPRB, 0.16874E+00_JPRB ,&
&  0.16971E+00_JPRB, 0.17107E+00_JPRB, 0.17196E+00_JPRB, 0.17222E+00_JPRB, 0.17262E+00_JPRB, 0.17194E+00_JPRB, 0.17147E+00_JPRB ,&
&  0.17079E+00_JPRB, 0.16983E+00_JPRB, 0.16863E+00_JPRB, 0.16759E+00_JPRB, 0.16642E+00_JPRB, 0.16493E+00_JPRB, 0.16338E+00_JPRB ,&
&  0.16199E+00_JPRB, 0.16076E+00_JPRB, 0.15945E+00_JPRB, 0.15837E+00_JPRB, 0.15733E+00_JPRB, 0.15630E+00_JPRB, 0.15535E+00_JPRB ,&
&  0.15459E+00_JPRB, 0.15392E+00_JPRB, 0.15319E+00_JPRB, 0.15243E+00_JPRB, 0.15159E+00_JPRB, 0.15078E+00_JPRB, 0.15001E+00_JPRB ,&
&  0.14964E+00_JPRB, 0.14932E+00_JPRB, 0.14900E+00_JPRB, 0.14865E+00_JPRB, 0.14854E+00_JPRB, 0.14843E+00_JPRB, 0.14827E+00_JPRB ,&
&  0.14810E+00_JPRB, 0.14785E+00_JPRB, 0.14776E+00_JPRB, 0.14767E+00_JPRB, 0.14758E+00_JPRB, 0.14752E+00_JPRB, 0.14762E+00_JPRB ,&
&  0.14779E+00_JPRB, 0.14803E+00_JPRB, 0.14852E+00_JPRB, 0.14904E+00_JPRB, 0.14958E+00_JPRB, 0.15010E+00_JPRB, 0.15087E+00_JPRB ,&
&  0.15170E+00_JPRB, 0.15251E+00_JPRB, 0.15319E+00_JPRB, 0.15376E+00_JPRB, 0.15468E+00_JPRB, 0.15555E+00_JPRB, 0.15641E+00_JPRB ,&
&  0.15734E+00_JPRB, 0.15853E+00_JPRB, 0.15977E+00_JPRB, 0.16105E+00_JPRB, 0.16253E+00_JPRB, 0.16378E+00_JPRB, 0.16503E+00_JPRB ,&
&  0.16651E+00_JPRB, 0.16927E+00_JPRB, 0.17110E+00_JPRB, 0.17231E+00_JPRB, 0.17361E+00_JPRB /)
ZBURD(:, 72, 10) = (/ &
&  0.15976E+00_JPRB, 0.15992E+00_JPRB, 0.15999E+00_JPRB, 0.15999E+00_JPRB, 0.15980E+00_JPRB, 0.15975E+00_JPRB, 0.15971E+00_JPRB ,&
&  0.15968E+00_JPRB, 0.15950E+00_JPRB, 0.15970E+00_JPRB, 0.15979E+00_JPRB, 0.15983E+00_JPRB, 0.15983E+00_JPRB, 0.15969E+00_JPRB ,&
&  0.15940E+00_JPRB, 0.15903E+00_JPRB, 0.15808E+00_JPRB, 0.15791E+00_JPRB, 0.15793E+00_JPRB, 0.15812E+00_JPRB, 0.15826E+00_JPRB ,&
&  0.15849E+00_JPRB, 0.15873E+00_JPRB, 0.15892E+00_JPRB, 0.15951E+00_JPRB, 0.15977E+00_JPRB, 0.16006E+00_JPRB, 0.16035E+00_JPRB ,&
&  0.16062E+00_JPRB, 0.16094E+00_JPRB, 0.16118E+00_JPRB, 0.16144E+00_JPRB, 0.16167E+00_JPRB, 0.16162E+00_JPRB, 0.16142E+00_JPRB ,&
&  0.16111E+00_JPRB, 0.16062E+00_JPRB, 0.16010E+00_JPRB, 0.15963E+00_JPRB, 0.15920E+00_JPRB, 0.15876E+00_JPRB, 0.15833E+00_JPRB ,&
&  0.15787E+00_JPRB, 0.15735E+00_JPRB, 0.15680E+00_JPRB, 0.15631E+00_JPRB, 0.15580E+00_JPRB, 0.15536E+00_JPRB, 0.15486E+00_JPRB ,&
&  0.15447E+00_JPRB, 0.15407E+00_JPRB, 0.15370E+00_JPRB, 0.15331E+00_JPRB, 0.15298E+00_JPRB, 0.15264E+00_JPRB, 0.15230E+00_JPRB ,&
&  0.15195E+00_JPRB, 0.15177E+00_JPRB, 0.15152E+00_JPRB, 0.15130E+00_JPRB, 0.15110E+00_JPRB, 0.15095E+00_JPRB, 0.15075E+00_JPRB ,&
&  0.15057E+00_JPRB, 0.15039E+00_JPRB, 0.15032E+00_JPRB, 0.15018E+00_JPRB, 0.15014E+00_JPRB, 0.15012E+00_JPRB, 0.15018E+00_JPRB ,&
&  0.15024E+00_JPRB, 0.15033E+00_JPRB, 0.15050E+00_JPRB, 0.15076E+00_JPRB, 0.15100E+00_JPRB, 0.15126E+00_JPRB, 0.15153E+00_JPRB ,&
&  0.15183E+00_JPRB, 0.15213E+00_JPRB, 0.15245E+00_JPRB, 0.15276E+00_JPRB, 0.15314E+00_JPRB, 0.15353E+00_JPRB, 0.15397E+00_JPRB ,&
&  0.15436E+00_JPRB, 0.15480E+00_JPRB, 0.15517E+00_JPRB, 0.15561E+00_JPRB, 0.15596E+00_JPRB, 0.15619E+00_JPRB, 0.15640E+00_JPRB ,&
&  0.15677E+00_JPRB, 0.15713E+00_JPRB, 0.15757E+00_JPRB, 0.15796E+00_JPRB, 0.15866E+00_JPRB /)
ZBURD(:, 73, 10) = (/ &
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB ,&
&  0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB, 0.15340E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART20',1,ZHOOK_HANDLE)
END SUBROUTINE PART20



SUBROUTINE PART21

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART21',0,ZHOOK_HANDLE)


!-- month= 11
ZBURD(:,  1, 11) = (/ &
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB ,&
&  0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB, 0.94548E+00_JPRB /)
ZBURD(:,  2, 11) = (/ &
&  0.93170E+00_JPRB, 0.93151E+00_JPRB, 0.93124E+00_JPRB, 0.93100E+00_JPRB, 0.93059E+00_JPRB, 0.93023E+00_JPRB, 0.92984E+00_JPRB ,&
&  0.92931E+00_JPRB, 0.92892E+00_JPRB, 0.92880E+00_JPRB, 0.92884E+00_JPRB, 0.92891E+00_JPRB, 0.92895E+00_JPRB, 0.92908E+00_JPRB ,&
&  0.92932E+00_JPRB, 0.92925E+00_JPRB, 0.92913E+00_JPRB, 0.92922E+00_JPRB, 0.92951E+00_JPRB, 0.92945E+00_JPRB, 0.92968E+00_JPRB ,&
&  0.92946E+00_JPRB, 0.92818E+00_JPRB, 0.92659E+00_JPRB, 0.92600E+00_JPRB, 0.92547E+00_JPRB, 0.92516E+00_JPRB, 0.92473E+00_JPRB ,&
&  0.92424E+00_JPRB, 0.92402E+00_JPRB, 0.92358E+00_JPRB, 0.92318E+00_JPRB, 0.92357E+00_JPRB, 0.92428E+00_JPRB, 0.92517E+00_JPRB ,&
&  0.92616E+00_JPRB, 0.92731E+00_JPRB, 0.92905E+00_JPRB, 0.93081E+00_JPRB, 0.93264E+00_JPRB, 0.93445E+00_JPRB, 0.93669E+00_JPRB ,&
&  0.93853E+00_JPRB, 0.94038E+00_JPRB, 0.94311E+00_JPRB, 0.94634E+00_JPRB, 0.94937E+00_JPRB, 0.95259E+00_JPRB, 0.95572E+00_JPRB ,&
&  0.96005E+00_JPRB, 0.96526E+00_JPRB, 0.97090E+00_JPRB, 0.97657E+00_JPRB, 0.98169E+00_JPRB, 0.98695E+00_JPRB, 0.99264E+00_JPRB ,&
&  0.99772E+00_JPRB, 0.10013E+01_JPRB, 0.10052E+01_JPRB, 0.10093E+01_JPRB, 0.10127E+01_JPRB, 0.10134E+01_JPRB, 0.10143E+01_JPRB ,&
&  0.10153E+01_JPRB, 0.10156E+01_JPRB, 0.10127E+01_JPRB, 0.10102E+01_JPRB, 0.10077E+01_JPRB, 0.10047E+01_JPRB, 0.10010E+01_JPRB ,&
&  0.99766E+00_JPRB, 0.99463E+00_JPRB, 0.99089E+00_JPRB, 0.98664E+00_JPRB, 0.98299E+00_JPRB, 0.97945E+00_JPRB, 0.97589E+00_JPRB ,&
&  0.97221E+00_JPRB, 0.96905E+00_JPRB, 0.96609E+00_JPRB, 0.96302E+00_JPRB, 0.95953E+00_JPRB, 0.95636E+00_JPRB, 0.95321E+00_JPRB ,&
&  0.95028E+00_JPRB, 0.94720E+00_JPRB, 0.94440E+00_JPRB, 0.94168E+00_JPRB, 0.93952E+00_JPRB, 0.93776E+00_JPRB, 0.93631E+00_JPRB ,&
&  0.93502E+00_JPRB, 0.93403E+00_JPRB, 0.93335E+00_JPRB, 0.93269E+00_JPRB, 0.93214E+00_JPRB /)
ZBURD(:,  3, 11) = (/ &
&  0.92991E+00_JPRB, 0.93071E+00_JPRB, 0.93127E+00_JPRB, 0.93159E+00_JPRB, 0.93176E+00_JPRB, 0.93186E+00_JPRB, 0.93161E+00_JPRB ,&
&  0.93106E+00_JPRB, 0.93053E+00_JPRB, 0.93010E+00_JPRB, 0.92958E+00_JPRB, 0.92884E+00_JPRB, 0.92849E+00_JPRB, 0.92822E+00_JPRB ,&
&  0.92763E+00_JPRB, 0.92655E+00_JPRB, 0.92524E+00_JPRB, 0.92411E+00_JPRB, 0.92286E+00_JPRB, 0.92075E+00_JPRB, 0.91954E+00_JPRB ,&
&  0.91804E+00_JPRB, 0.91427E+00_JPRB, 0.90864E+00_JPRB, 0.89773E+00_JPRB, 0.89008E+00_JPRB, 0.88282E+00_JPRB, 0.87613E+00_JPRB ,&
&  0.86877E+00_JPRB, 0.86714E+00_JPRB, 0.86586E+00_JPRB, 0.86456E+00_JPRB, 0.86670E+00_JPRB, 0.86544E+00_JPRB, 0.86325E+00_JPRB ,&
&  0.85939E+00_JPRB, 0.85245E+00_JPRB, 0.85325E+00_JPRB, 0.85455E+00_JPRB, 0.85591E+00_JPRB, 0.86106E+00_JPRB, 0.86735E+00_JPRB ,&
&  0.87481E+00_JPRB, 0.88314E+00_JPRB, 0.89582E+00_JPRB, 0.91077E+00_JPRB, 0.92501E+00_JPRB, 0.93820E+00_JPRB, 0.95092E+00_JPRB ,&
&  0.96128E+00_JPRB, 0.97213E+00_JPRB, 0.98429E+00_JPRB, 0.99793E+00_JPRB, 0.10105E+01_JPRB, 0.10241E+01_JPRB, 0.10352E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10466E+01_JPRB, 0.10555E+01_JPRB, 0.10661E+01_JPRB, 0.10781E+01_JPRB, 0.10835E+01_JPRB, 0.10866E+01_JPRB ,&
&  0.10875E+01_JPRB, 0.10880E+01_JPRB, 0.10846E+01_JPRB, 0.10798E+01_JPRB, 0.10732E+01_JPRB, 0.10654E+01_JPRB, 0.10561E+01_JPRB ,&
&  0.10469E+01_JPRB, 0.10382E+01_JPRB, 0.10308E+01_JPRB, 0.10231E+01_JPRB, 0.10168E+01_JPRB, 0.10105E+01_JPRB, 0.10065E+01_JPRB ,&
&  0.10012E+01_JPRB, 0.99691E+00_JPRB, 0.99207E+00_JPRB, 0.98768E+00_JPRB, 0.98019E+00_JPRB, 0.97266E+00_JPRB, 0.96419E+00_JPRB ,&
&  0.95732E+00_JPRB, 0.94992E+00_JPRB, 0.94353E+00_JPRB, 0.93781E+00_JPRB, 0.93456E+00_JPRB, 0.93207E+00_JPRB, 0.93034E+00_JPRB ,&
&  0.92878E+00_JPRB, 0.92835E+00_JPRB, 0.92857E+00_JPRB, 0.92881E+00_JPRB, 0.92921E+00_JPRB /)
ZBURD(:,  4, 11) = (/ &
&  0.93920E+00_JPRB, 0.94063E+00_JPRB, 0.94172E+00_JPRB, 0.94184E+00_JPRB, 0.94235E+00_JPRB, 0.94240E+00_JPRB, 0.94235E+00_JPRB ,&
&  0.94196E+00_JPRB, 0.94100E+00_JPRB, 0.93994E+00_JPRB, 0.93900E+00_JPRB, 0.93803E+00_JPRB, 0.93766E+00_JPRB, 0.93626E+00_JPRB ,&
&  0.93367E+00_JPRB, 0.92974E+00_JPRB, 0.92667E+00_JPRB, 0.92335E+00_JPRB, 0.91874E+00_JPRB, 0.91358E+00_JPRB, 0.90217E+00_JPRB ,&
&  0.89438E+00_JPRB, 0.88691E+00_JPRB, 0.87746E+00_JPRB, 0.86447E+00_JPRB, 0.85692E+00_JPRB, 0.82060E+00_JPRB, 0.81995E+00_JPRB ,&
&  0.83550E+00_JPRB, 0.83084E+00_JPRB, 0.82428E+00_JPRB, 0.81620E+00_JPRB, 0.79288E+00_JPRB, 0.77584E+00_JPRB, 0.76930E+00_JPRB ,&
&  0.73926E+00_JPRB, 0.75170E+00_JPRB, 0.75415E+00_JPRB, 0.75557E+00_JPRB, 0.76078E+00_JPRB, 0.76022E+00_JPRB, 0.76512E+00_JPRB ,&
&  0.78834E+00_JPRB, 0.79530E+00_JPRB, 0.81203E+00_JPRB, 0.83237E+00_JPRB, 0.85529E+00_JPRB, 0.87781E+00_JPRB, 0.90035E+00_JPRB ,&
&  0.91697E+00_JPRB, 0.92795E+00_JPRB, 0.93483E+00_JPRB, 0.93874E+00_JPRB, 0.94511E+00_JPRB, 0.95400E+00_JPRB, 0.97009E+00_JPRB ,&
&  0.99602E+00_JPRB, 0.10243E+01_JPRB, 0.10413E+01_JPRB, 0.10458E+01_JPRB, 0.10555E+01_JPRB, 0.10646E+01_JPRB, 0.10731E+01_JPRB ,&
&  0.10798E+01_JPRB, 0.10849E+01_JPRB, 0.10870E+01_JPRB, 0.10896E+01_JPRB, 0.10901E+01_JPRB, 0.10874E+01_JPRB, 0.10821E+01_JPRB ,&
&  0.10752E+01_JPRB, 0.10665E+01_JPRB, 0.10586E+01_JPRB, 0.10502E+01_JPRB, 0.10468E+01_JPRB, 0.10432E+01_JPRB, 0.10454E+01_JPRB ,&
&  0.10435E+01_JPRB, 0.10429E+01_JPRB, 0.10364E+01_JPRB, 0.10281E+01_JPRB, 0.10144E+01_JPRB, 0.10008E+01_JPRB, 0.98510E+00_JPRB ,&
&  0.97291E+00_JPRB, 0.96315E+00_JPRB, 0.95558E+00_JPRB, 0.94652E+00_JPRB, 0.94110E+00_JPRB, 0.93541E+00_JPRB, 0.93292E+00_JPRB ,&
&  0.93036E+00_JPRB, 0.93180E+00_JPRB, 0.93275E+00_JPRB, 0.93537E+00_JPRB, 0.93692E+00_JPRB /)
ZBURD(:,  5, 11) = (/ &
&  0.92221E+00_JPRB, 0.92127E+00_JPRB, 0.92095E+00_JPRB, 0.92021E+00_JPRB, 0.92039E+00_JPRB, 0.92000E+00_JPRB, 0.92320E+00_JPRB ,&
&  0.92582E+00_JPRB, 0.92818E+00_JPRB, 0.92999E+00_JPRB, 0.93156E+00_JPRB, 0.93283E+00_JPRB, 0.93236E+00_JPRB, 0.93031E+00_JPRB ,&
&  0.92736E+00_JPRB, 0.92313E+00_JPRB, 0.91952E+00_JPRB, 0.91613E+00_JPRB, 0.91142E+00_JPRB, 0.90734E+00_JPRB, 0.89542E+00_JPRB ,&
&  0.88615E+00_JPRB, 0.87678E+00_JPRB, 0.83928E+00_JPRB, 0.83948E+00_JPRB, 0.82429E+00_JPRB, 0.80767E+00_JPRB, 0.79444E+00_JPRB ,&
&  0.76413E+00_JPRB, 0.75027E+00_JPRB, 0.72444E+00_JPRB, 0.70305E+00_JPRB, 0.69935E+00_JPRB, 0.68106E+00_JPRB, 0.67803E+00_JPRB ,&
&  0.67127E+00_JPRB, 0.66486E+00_JPRB, 0.66190E+00_JPRB, 0.66258E+00_JPRB, 0.66930E+00_JPRB, 0.67758E+00_JPRB, 0.68669E+00_JPRB ,&
&  0.70284E+00_JPRB, 0.71896E+00_JPRB, 0.74320E+00_JPRB, 0.75576E+00_JPRB, 0.78103E+00_JPRB, 0.81059E+00_JPRB, 0.84221E+00_JPRB ,&
&  0.86213E+00_JPRB, 0.85759E+00_JPRB, 0.86399E+00_JPRB, 0.87897E+00_JPRB, 0.89245E+00_JPRB, 0.90777E+00_JPRB, 0.93350E+00_JPRB ,&
&  0.96824E+00_JPRB, 0.10011E+01_JPRB, 0.10184E+01_JPRB, 0.10218E+01_JPRB, 0.10251E+01_JPRB, 0.10279E+01_JPRB, 0.10394E+01_JPRB ,&
&  0.10505E+01_JPRB, 0.10670E+01_JPRB, 0.10814E+01_JPRB, 0.10982E+01_JPRB, 0.11069E+01_JPRB, 0.11211E+01_JPRB, 0.11297E+01_JPRB ,&
&  0.11391E+01_JPRB, 0.11439E+01_JPRB, 0.11581E+01_JPRB, 0.11470E+01_JPRB, 0.11509E+01_JPRB, 0.11472E+01_JPRB, 0.11369E+01_JPRB ,&
&  0.11220E+01_JPRB, 0.11084E+01_JPRB, 0.10915E+01_JPRB, 0.10731E+01_JPRB, 0.10512E+01_JPRB, 0.10328E+01_JPRB, 0.10150E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.98328E+00_JPRB, 0.97007E+00_JPRB, 0.94879E+00_JPRB, 0.93750E+00_JPRB, 0.92381E+00_JPRB, 0.91763E+00_JPRB ,&
&  0.91307E+00_JPRB, 0.91388E+00_JPRB, 0.91449E+00_JPRB, 0.91853E+00_JPRB, 0.92123E+00_JPRB /)
ZBURD(:,  6, 11) = (/ &
&  0.87274E+00_JPRB, 0.87937E+00_JPRB, 0.89066E+00_JPRB, 0.90049E+00_JPRB, 0.90818E+00_JPRB, 0.91427E+00_JPRB, 0.91615E+00_JPRB ,&
&  0.91778E+00_JPRB, 0.91704E+00_JPRB, 0.91699E+00_JPRB, 0.91643E+00_JPRB, 0.91727E+00_JPRB, 0.91709E+00_JPRB, 0.91782E+00_JPRB ,&
&  0.91674E+00_JPRB, 0.91501E+00_JPRB, 0.91166E+00_JPRB, 0.90786E+00_JPRB, 0.90428E+00_JPRB, 0.90198E+00_JPRB, 0.89642E+00_JPRB ,&
&  0.89223E+00_JPRB, 0.87938E+00_JPRB, 0.85027E+00_JPRB, 0.84219E+00_JPRB, 0.82718E+00_JPRB, 0.80840E+00_JPRB, 0.78955E+00_JPRB ,&
&  0.76769E+00_JPRB, 0.71919E+00_JPRB, 0.73752E+00_JPRB, 0.71617E+00_JPRB, 0.69379E+00_JPRB, 0.67181E+00_JPRB, 0.64750E+00_JPRB ,&
&  0.63154E+00_JPRB, 0.61061E+00_JPRB, 0.59434E+00_JPRB, 0.58733E+00_JPRB, 0.58579E+00_JPRB, 0.59583E+00_JPRB, 0.61008E+00_JPRB ,&
&  0.62233E+00_JPRB, 0.65087E+00_JPRB, 0.66740E+00_JPRB, 0.68928E+00_JPRB, 0.71192E+00_JPRB, 0.74108E+00_JPRB, 0.75422E+00_JPRB ,&
&  0.77740E+00_JPRB, 0.78585E+00_JPRB, 0.81320E+00_JPRB, 0.85048E+00_JPRB, 0.89221E+00_JPRB, 0.92687E+00_JPRB, 0.95376E+00_JPRB ,&
&  0.99430E+00_JPRB, 0.10210E+01_JPRB, 0.10399E+01_JPRB, 0.10531E+01_JPRB, 0.10725E+01_JPRB, 0.10879E+01_JPRB, 0.11056E+01_JPRB ,&
&  0.11173E+01_JPRB, 0.11402E+01_JPRB, 0.11667E+01_JPRB, 0.11993E+01_JPRB, 0.12192E+01_JPRB, 0.12455E+01_JPRB, 0.12724E+01_JPRB ,&
&  0.13000E+01_JPRB, 0.13119E+01_JPRB, 0.13383E+01_JPRB, 0.13245E+01_JPRB, 0.13170E+01_JPRB, 0.12775E+01_JPRB, 0.12458E+01_JPRB ,&
&  0.12202E+01_JPRB, 0.11901E+01_JPRB, 0.11600E+01_JPRB, 0.11307E+01_JPRB, 0.11025E+01_JPRB, 0.10783E+01_JPRB, 0.10514E+01_JPRB ,&
&  0.10215E+01_JPRB, 0.98250E+00_JPRB, 0.95218E+00_JPRB, 0.92105E+00_JPRB, 0.90489E+00_JPRB, 0.88747E+00_JPRB, 0.87884E+00_JPRB ,&
&  0.87325E+00_JPRB, 0.87070E+00_JPRB, 0.86976E+00_JPRB, 0.87302E+00_JPRB, 0.87334E+00_JPRB /)
ZBURD(:,  7, 11) = (/ &
&  0.85392E+00_JPRB, 0.85853E+00_JPRB, 0.86247E+00_JPRB, 0.86377E+00_JPRB, 0.85996E+00_JPRB, 0.85891E+00_JPRB, 0.85266E+00_JPRB ,&
&  0.85077E+00_JPRB, 0.84889E+00_JPRB, 0.84830E+00_JPRB, 0.85237E+00_JPRB, 0.85667E+00_JPRB, 0.86347E+00_JPRB, 0.86966E+00_JPRB ,&
&  0.87521E+00_JPRB, 0.88134E+00_JPRB, 0.88370E+00_JPRB, 0.88695E+00_JPRB, 0.88875E+00_JPRB, 0.89345E+00_JPRB, 0.89288E+00_JPRB ,&
&  0.89210E+00_JPRB, 0.87976E+00_JPRB, 0.85555E+00_JPRB, 0.84007E+00_JPRB, 0.83385E+00_JPRB, 0.80414E+00_JPRB, 0.79882E+00_JPRB ,&
&  0.80021E+00_JPRB, 0.79768E+00_JPRB, 0.78661E+00_JPRB, 0.78006E+00_JPRB, 0.75024E+00_JPRB, 0.64925E+00_JPRB, 0.64561E+00_JPRB ,&
&  0.62070E+00_JPRB, 0.58594E+00_JPRB, 0.56729E+00_JPRB, 0.55262E+00_JPRB, 0.54705E+00_JPRB, 0.55867E+00_JPRB, 0.57724E+00_JPRB ,&
&  0.58715E+00_JPRB, 0.62038E+00_JPRB, 0.63490E+00_JPRB, 0.65873E+00_JPRB, 0.68140E+00_JPRB, 0.71334E+00_JPRB, 0.73365E+00_JPRB ,&
&  0.75681E+00_JPRB, 0.78188E+00_JPRB, 0.82503E+00_JPRB, 0.88452E+00_JPRB, 0.93101E+00_JPRB, 0.96819E+00_JPRB, 0.98064E+00_JPRB ,&
&  0.10276E+01_JPRB, 0.10349E+01_JPRB, 0.10586E+01_JPRB, 0.10800E+01_JPRB, 0.11006E+01_JPRB, 0.11273E+01_JPRB, 0.11477E+01_JPRB ,&
&  0.11853E+01_JPRB, 0.12487E+01_JPRB, 0.13080E+01_JPRB, 0.13477E+01_JPRB, 0.13727E+01_JPRB, 0.14002E+01_JPRB, 0.14311E+01_JPRB ,&
&  0.14675E+01_JPRB, 0.14958E+01_JPRB, 0.15013E+01_JPRB, 0.14623E+01_JPRB, 0.14399E+01_JPRB, 0.13890E+01_JPRB, 0.13529E+01_JPRB ,&
&  0.13025E+01_JPRB, 0.12760E+01_JPRB, 0.12236E+01_JPRB, 0.11856E+01_JPRB, 0.11418E+01_JPRB, 0.11090E+01_JPRB, 0.10647E+01_JPRB ,&
&  0.10181E+01_JPRB, 0.97045E+00_JPRB, 0.93581E+00_JPRB, 0.91284E+00_JPRB, 0.89882E+00_JPRB, 0.88155E+00_JPRB, 0.87037E+00_JPRB ,&
&  0.85870E+00_JPRB, 0.85046E+00_JPRB, 0.84392E+00_JPRB, 0.84603E+00_JPRB, 0.84767E+00_JPRB /)
ZBURD(:,  8, 11) = (/ &
&  0.80428E+00_JPRB, 0.80245E+00_JPRB, 0.79668E+00_JPRB, 0.79578E+00_JPRB, 0.78720E+00_JPRB, 0.78432E+00_JPRB, 0.77165E+00_JPRB ,&
&  0.76360E+00_JPRB, 0.75843E+00_JPRB, 0.75651E+00_JPRB, 0.75979E+00_JPRB, 0.76667E+00_JPRB, 0.77810E+00_JPRB, 0.79190E+00_JPRB ,&
&  0.80831E+00_JPRB, 0.81561E+00_JPRB, 0.82853E+00_JPRB, 0.84495E+00_JPRB, 0.85545E+00_JPRB, 0.86756E+00_JPRB, 0.87463E+00_JPRB ,&
&  0.87539E+00_JPRB, 0.86763E+00_JPRB, 0.86018E+00_JPRB, 0.84346E+00_JPRB, 0.83013E+00_JPRB, 0.81785E+00_JPRB, 0.81173E+00_JPRB ,&
&  0.79281E+00_JPRB, 0.78359E+00_JPRB, 0.80611E+00_JPRB, 0.81259E+00_JPRB, 0.76336E+00_JPRB, 0.71152E+00_JPRB, 0.67621E+00_JPRB ,&
&  0.62861E+00_JPRB, 0.57591E+00_JPRB, 0.55747E+00_JPRB, 0.54621E+00_JPRB, 0.55048E+00_JPRB, 0.55742E+00_JPRB, 0.57361E+00_JPRB ,&
&  0.58772E+00_JPRB, 0.60809E+00_JPRB, 0.63114E+00_JPRB, 0.66268E+00_JPRB, 0.68773E+00_JPRB, 0.71675E+00_JPRB, 0.73992E+00_JPRB ,&
&  0.76861E+00_JPRB, 0.80862E+00_JPRB, 0.87069E+00_JPRB, 0.95065E+00_JPRB, 0.10149E+01_JPRB, 0.10700E+01_JPRB, 0.11026E+01_JPRB ,&
&  0.11865E+01_JPRB, 0.11896E+01_JPRB, 0.12187E+01_JPRB, 0.12361E+01_JPRB, 0.12536E+01_JPRB, 0.12865E+01_JPRB, 0.13146E+01_JPRB ,&
&  0.13781E+01_JPRB, 0.14848E+01_JPRB, 0.15211E+01_JPRB, 0.15345E+01_JPRB, 0.15342E+01_JPRB, 0.15522E+01_JPRB, 0.15892E+01_JPRB ,&
&  0.16012E+01_JPRB, 0.16268E+01_JPRB, 0.17315E+01_JPRB, 0.16601E+01_JPRB, 0.16027E+01_JPRB, 0.15387E+01_JPRB, 0.14659E+01_JPRB ,&
&  0.13948E+01_JPRB, 0.13295E+01_JPRB, 0.12599E+01_JPRB, 0.11925E+01_JPRB, 0.11366E+01_JPRB, 0.10870E+01_JPRB, 0.10490E+01_JPRB ,&
&  0.10104E+01_JPRB, 0.97717E+00_JPRB, 0.95353E+00_JPRB, 0.93569E+00_JPRB, 0.91456E+00_JPRB, 0.89161E+00_JPRB, 0.87496E+00_JPRB ,&
&  0.85195E+00_JPRB, 0.83465E+00_JPRB, 0.81853E+00_JPRB, 0.81161E+00_JPRB, 0.80638E+00_JPRB /)
ZBURD(:,  9, 11) = (/ &
&  0.77160E+00_JPRB, 0.77212E+00_JPRB, 0.76860E+00_JPRB, 0.76348E+00_JPRB, 0.75077E+00_JPRB, 0.74630E+00_JPRB, 0.73533E+00_JPRB ,&
&  0.72970E+00_JPRB, 0.73032E+00_JPRB, 0.72500E+00_JPRB, 0.72522E+00_JPRB, 0.72681E+00_JPRB, 0.72859E+00_JPRB, 0.73310E+00_JPRB ,&
&  0.74186E+00_JPRB, 0.75555E+00_JPRB, 0.77303E+00_JPRB, 0.79223E+00_JPRB, 0.81249E+00_JPRB, 0.83309E+00_JPRB, 0.84994E+00_JPRB ,&
&  0.85848E+00_JPRB, 0.85718E+00_JPRB, 0.85968E+00_JPRB, 0.85360E+00_JPRB, 0.84229E+00_JPRB, 0.82722E+00_JPRB, 0.81204E+00_JPRB ,&
&  0.81153E+00_JPRB, 0.82385E+00_JPRB, 0.82026E+00_JPRB, 0.81548E+00_JPRB, 0.80770E+00_JPRB, 0.75883E+00_JPRB, 0.70622E+00_JPRB ,&
&  0.64752E+00_JPRB, 0.59301E+00_JPRB, 0.57027E+00_JPRB, 0.56135E+00_JPRB, 0.57003E+00_JPRB, 0.58822E+00_JPRB, 0.60895E+00_JPRB ,&
&  0.61805E+00_JPRB, 0.64818E+00_JPRB, 0.68159E+00_JPRB, 0.70715E+00_JPRB, 0.72972E+00_JPRB, 0.75313E+00_JPRB, 0.77874E+00_JPRB ,&
&  0.82151E+00_JPRB, 0.89795E+00_JPRB, 0.10353E+01_JPRB, 0.12038E+01_JPRB, 0.13488E+01_JPRB, 0.13822E+01_JPRB, 0.14901E+01_JPRB ,&
&  0.17622E+01_JPRB, 0.16796E+01_JPRB, 0.16449E+01_JPRB, 0.16376E+01_JPRB, 0.16229E+01_JPRB, 0.16306E+01_JPRB, 0.16796E+01_JPRB ,&
&  0.17542E+01_JPRB, 0.18158E+01_JPRB, 0.18198E+01_JPRB, 0.17928E+01_JPRB, 0.17735E+01_JPRB, 0.17686E+01_JPRB, 0.17854E+01_JPRB ,&
&  0.17913E+01_JPRB, 0.18372E+01_JPRB, 0.19771E+01_JPRB, 0.16882E+01_JPRB, 0.15620E+01_JPRB, 0.15094E+01_JPRB, 0.14467E+01_JPRB ,&
&  0.13826E+01_JPRB, 0.13249E+01_JPRB, 0.12641E+01_JPRB, 0.11940E+01_JPRB, 0.11149E+01_JPRB, 0.10395E+01_JPRB, 0.98606E+00_JPRB ,&
&  0.96288E+00_JPRB, 0.95030E+00_JPRB, 0.94331E+00_JPRB, 0.93267E+00_JPRB, 0.91258E+00_JPRB, 0.89272E+00_JPRB, 0.87232E+00_JPRB ,&
&  0.84453E+00_JPRB, 0.81229E+00_JPRB, 0.78869E+00_JPRB, 0.78154E+00_JPRB, 0.77342E+00_JPRB /)
ZBURD(:, 10, 11) = (/ &
&  0.75925E+00_JPRB, 0.76136E+00_JPRB, 0.76174E+00_JPRB, 0.77101E+00_JPRB, 0.77046E+00_JPRB, 0.75872E+00_JPRB, 0.73385E+00_JPRB ,&
&  0.71762E+00_JPRB, 0.71197E+00_JPRB, 0.71515E+00_JPRB, 0.71967E+00_JPRB, 0.72261E+00_JPRB, 0.72408E+00_JPRB, 0.72376E+00_JPRB ,&
&  0.72200E+00_JPRB, 0.72491E+00_JPRB, 0.73437E+00_JPRB, 0.74966E+00_JPRB, 0.76986E+00_JPRB, 0.79236E+00_JPRB, 0.81593E+00_JPRB ,&
&  0.83508E+00_JPRB, 0.84258E+00_JPRB, 0.84674E+00_JPRB, 0.84568E+00_JPRB, 0.84668E+00_JPRB, 0.85000E+00_JPRB, 0.84380E+00_JPRB ,&
&  0.83043E+00_JPRB, 0.83371E+00_JPRB, 0.84506E+00_JPRB, 0.83205E+00_JPRB, 0.82143E+00_JPRB, 0.78315E+00_JPRB, 0.72938E+00_JPRB ,&
&  0.67804E+00_JPRB, 0.62787E+00_JPRB, 0.61229E+00_JPRB, 0.61510E+00_JPRB, 0.63182E+00_JPRB, 0.65317E+00_JPRB, 0.68270E+00_JPRB ,&
&  0.70759E+00_JPRB, 0.73091E+00_JPRB, 0.75033E+00_JPRB, 0.76079E+00_JPRB, 0.77410E+00_JPRB, 0.81156E+00_JPRB, 0.89529E+00_JPRB ,&
&  0.10468E+01_JPRB, 0.12434E+01_JPRB, 0.14910E+01_JPRB, 0.14592E+01_JPRB, 0.15362E+01_JPRB, 0.16874E+01_JPRB, 0.19003E+01_JPRB ,&
&  0.20347E+01_JPRB, 0.20396E+01_JPRB, 0.20885E+01_JPRB, 0.22258E+01_JPRB, 0.23243E+01_JPRB, 0.24152E+01_JPRB, 0.23812E+01_JPRB ,&
&  0.23111E+01_JPRB, 0.22174E+01_JPRB, 0.21320E+01_JPRB, 0.21027E+01_JPRB, 0.21112E+01_JPRB, 0.21040E+01_JPRB, 0.20463E+01_JPRB ,&
&  0.19458E+01_JPRB, 0.17863E+01_JPRB, 0.15966E+01_JPRB, 0.14347E+01_JPRB, 0.13851E+01_JPRB, 0.13853E+01_JPRB, 0.13943E+01_JPRB ,&
&  0.13605E+01_JPRB, 0.13078E+01_JPRB, 0.12426E+01_JPRB, 0.11858E+01_JPRB, 0.10763E+01_JPRB, 0.97611E+00_JPRB, 0.91723E+00_JPRB ,&
&  0.89667E+00_JPRB, 0.88966E+00_JPRB, 0.88312E+00_JPRB, 0.86996E+00_JPRB, 0.85874E+00_JPRB, 0.84629E+00_JPRB, 0.80725E+00_JPRB ,&
&  0.77383E+00_JPRB, 0.75374E+00_JPRB, 0.73425E+00_JPRB, 0.73172E+00_JPRB, 0.74145E+00_JPRB /)
ZBURD(:, 11, 11) = (/ &
&  0.78980E+00_JPRB, 0.80454E+00_JPRB, 0.81956E+00_JPRB, 0.82318E+00_JPRB, 0.82265E+00_JPRB, 0.81633E+00_JPRB, 0.79174E+00_JPRB ,&
&  0.78033E+00_JPRB, 0.75171E+00_JPRB, 0.72779E+00_JPRB, 0.70508E+00_JPRB, 0.69012E+00_JPRB, 0.68240E+00_JPRB, 0.68714E+00_JPRB ,&
&  0.69800E+00_JPRB, 0.71016E+00_JPRB, 0.72237E+00_JPRB, 0.73265E+00_JPRB, 0.74555E+00_JPRB, 0.76333E+00_JPRB, 0.78627E+00_JPRB ,&
&  0.81100E+00_JPRB, 0.82958E+00_JPRB, 0.84524E+00_JPRB, 0.85565E+00_JPRB, 0.86260E+00_JPRB, 0.86398E+00_JPRB, 0.85003E+00_JPRB ,&
&  0.84786E+00_JPRB, 0.84262E+00_JPRB, 0.85030E+00_JPRB, 0.84620E+00_JPRB, 0.83820E+00_JPRB, 0.80563E+00_JPRB, 0.74215E+00_JPRB ,&
&  0.70218E+00_JPRB, 0.67819E+00_JPRB, 0.68309E+00_JPRB, 0.70523E+00_JPRB, 0.72668E+00_JPRB, 0.74968E+00_JPRB, 0.76933E+00_JPRB ,&
&  0.77495E+00_JPRB, 0.78010E+00_JPRB, 0.80941E+00_JPRB, 0.85625E+00_JPRB, 0.88959E+00_JPRB, 0.95625E+00_JPRB, 0.10457E+01_JPRB ,&
&  0.11774E+01_JPRB, 0.13573E+01_JPRB, 0.16367E+01_JPRB, 0.16922E+01_JPRB, 0.18575E+01_JPRB, 0.23263E+01_JPRB, 0.23484E+01_JPRB ,&
&  0.24192E+01_JPRB, 0.25243E+01_JPRB, 0.27107E+01_JPRB, 0.29342E+01_JPRB, 0.29370E+01_JPRB, 0.29184E+01_JPRB, 0.28661E+01_JPRB ,&
&  0.26939E+01_JPRB, 0.26122E+01_JPRB, 0.25296E+01_JPRB, 0.25608E+01_JPRB, 0.24991E+01_JPRB, 0.24145E+01_JPRB, 0.23286E+01_JPRB ,&
&  0.21724E+01_JPRB, 0.19515E+01_JPRB, 0.17363E+01_JPRB, 0.15457E+01_JPRB, 0.14514E+01_JPRB, 0.13962E+01_JPRB, 0.13616E+01_JPRB ,&
&  0.13211E+01_JPRB, 0.12887E+01_JPRB, 0.12523E+01_JPRB, 0.12128E+01_JPRB, 0.11050E+01_JPRB, 0.97876E+00_JPRB, 0.91353E+00_JPRB ,&
&  0.87544E+00_JPRB, 0.85847E+00_JPRB, 0.83427E+00_JPRB, 0.82266E+00_JPRB, 0.81866E+00_JPRB, 0.80100E+00_JPRB, 0.76233E+00_JPRB ,&
&  0.75028E+00_JPRB, 0.74049E+00_JPRB, 0.72686E+00_JPRB, 0.73277E+00_JPRB, 0.76285E+00_JPRB /)
ZBURD(:, 12, 11) = (/ &
&  0.83617E+00_JPRB, 0.85901E+00_JPRB, 0.86540E+00_JPRB, 0.86651E+00_JPRB, 0.87046E+00_JPRB, 0.86351E+00_JPRB, 0.81941E+00_JPRB ,&
&  0.80018E+00_JPRB, 0.74303E+00_JPRB, 0.70807E+00_JPRB, 0.68811E+00_JPRB, 0.68144E+00_JPRB, 0.67436E+00_JPRB, 0.66553E+00_JPRB ,&
&  0.67298E+00_JPRB, 0.69388E+00_JPRB, 0.71901E+00_JPRB, 0.73813E+00_JPRB, 0.75365E+00_JPRB, 0.76672E+00_JPRB, 0.77995E+00_JPRB ,&
&  0.79880E+00_JPRB, 0.82035E+00_JPRB, 0.84441E+00_JPRB, 0.85955E+00_JPRB, 0.86762E+00_JPRB, 0.87232E+00_JPRB, 0.86653E+00_JPRB ,&
&  0.85976E+00_JPRB, 0.84586E+00_JPRB, 0.86011E+00_JPRB, 0.86280E+00_JPRB, 0.84886E+00_JPRB, 0.83288E+00_JPRB, 0.79894E+00_JPRB ,&
&  0.74269E+00_JPRB, 0.75260E+00_JPRB, 0.77515E+00_JPRB, 0.79419E+00_JPRB, 0.80723E+00_JPRB, 0.81827E+00_JPRB, 0.82626E+00_JPRB ,&
&  0.83740E+00_JPRB, 0.85236E+00_JPRB, 0.87737E+00_JPRB, 0.90808E+00_JPRB, 0.94738E+00_JPRB, 0.10005E+01_JPRB, 0.10526E+01_JPRB ,&
&  0.11088E+01_JPRB, 0.10824E+01_JPRB, 0.12406E+01_JPRB, 0.15934E+01_JPRB, 0.22820E+01_JPRB, 0.25256E+01_JPRB, 0.27051E+01_JPRB ,&
&  0.29192E+01_JPRB, 0.30022E+01_JPRB, 0.31343E+01_JPRB, 0.32520E+01_JPRB, 0.32654E+01_JPRB, 0.32498E+01_JPRB, 0.32619E+01_JPRB ,&
&  0.31310E+01_JPRB, 0.31201E+01_JPRB, 0.30261E+01_JPRB, 0.29856E+01_JPRB, 0.28669E+01_JPRB, 0.27644E+01_JPRB, 0.26246E+01_JPRB ,&
&  0.24504E+01_JPRB, 0.22636E+01_JPRB, 0.20245E+01_JPRB, 0.17934E+01_JPRB, 0.16117E+01_JPRB, 0.15190E+01_JPRB, 0.14393E+01_JPRB ,&
&  0.13815E+01_JPRB, 0.13477E+01_JPRB, 0.12999E+01_JPRB, 0.12562E+01_JPRB, 0.11823E+01_JPRB, 0.10948E+01_JPRB, 0.10640E+01_JPRB ,&
&  0.96939E+00_JPRB, 0.91951E+00_JPRB, 0.86992E+00_JPRB, 0.84980E+00_JPRB, 0.85446E+00_JPRB, 0.84467E+00_JPRB, 0.82702E+00_JPRB ,&
&  0.84511E+00_JPRB, 0.82524E+00_JPRB, 0.80618E+00_JPRB, 0.81168E+00_JPRB, 0.81892E+00_JPRB /)
ZBURD(:, 13, 11) = (/ &
&  0.87955E+00_JPRB, 0.89673E+00_JPRB, 0.89975E+00_JPRB, 0.90682E+00_JPRB, 0.91932E+00_JPRB, 0.91049E+00_JPRB, 0.89445E+00_JPRB ,&
&  0.89829E+00_JPRB, 0.85948E+00_JPRB, 0.89268E+00_JPRB, 0.75863E+00_JPRB, 0.70598E+00_JPRB, 0.69884E+00_JPRB, 0.68978E+00_JPRB ,&
&  0.70001E+00_JPRB, 0.72387E+00_JPRB, 0.74618E+00_JPRB, 0.76399E+00_JPRB, 0.78866E+00_JPRB, 0.79542E+00_JPRB, 0.80087E+00_JPRB ,&
&  0.81343E+00_JPRB, 0.83525E+00_JPRB, 0.88017E+00_JPRB, 0.90836E+00_JPRB, 0.94618E+00_JPRB, 0.97554E+00_JPRB, 0.97075E+00_JPRB ,&
&  0.93347E+00_JPRB, 0.92690E+00_JPRB, 0.93570E+00_JPRB, 0.90364E+00_JPRB, 0.87229E+00_JPRB, 0.85939E+00_JPRB, 0.88395E+00_JPRB ,&
&  0.89563E+00_JPRB, 0.86061E+00_JPRB, 0.88811E+00_JPRB, 0.89444E+00_JPRB, 0.88960E+00_JPRB, 0.88111E+00_JPRB, 0.87326E+00_JPRB ,&
&  0.86951E+00_JPRB, 0.86620E+00_JPRB, 0.87579E+00_JPRB, 0.89580E+00_JPRB, 0.93772E+00_JPRB, 0.10032E+01_JPRB, 0.10404E+01_JPRB ,&
&  0.11445E+01_JPRB, 0.11378E+01_JPRB, 0.14617E+01_JPRB, 0.18261E+01_JPRB, 0.23522E+01_JPRB, 0.26839E+01_JPRB, 0.30323E+01_JPRB ,&
&  0.35120E+01_JPRB, 0.34866E+01_JPRB, 0.36645E+01_JPRB, 0.36649E+01_JPRB, 0.36325E+01_JPRB, 0.35789E+01_JPRB, 0.35213E+01_JPRB ,&
&  0.34877E+01_JPRB, 0.33405E+01_JPRB, 0.34932E+01_JPRB, 0.33838E+01_JPRB, 0.31864E+01_JPRB, 0.30508E+01_JPRB, 0.29204E+01_JPRB ,&
&  0.27892E+01_JPRB, 0.26433E+01_JPRB, 0.24645E+01_JPRB, 0.20839E+01_JPRB, 0.19195E+01_JPRB, 0.17857E+01_JPRB, 0.15767E+01_JPRB ,&
&  0.14781E+01_JPRB, 0.13702E+01_JPRB, 0.12647E+01_JPRB, 0.12565E+01_JPRB, 0.11782E+01_JPRB, 0.11118E+01_JPRB, 0.10869E+01_JPRB ,&
&  0.10237E+01_JPRB, 0.94693E+00_JPRB, 0.94664E+00_JPRB, 0.94057E+00_JPRB, 0.93209E+00_JPRB, 0.91484E+00_JPRB, 0.94238E+00_JPRB ,&
&  0.89989E+00_JPRB, 0.87446E+00_JPRB, 0.88367E+00_JPRB, 0.86676E+00_JPRB, 0.86869E+00_JPRB /)
ZBURD(:, 14, 11) = (/ &
&  0.90561E+00_JPRB, 0.92565E+00_JPRB, 0.93991E+00_JPRB, 0.95461E+00_JPRB, 0.97369E+00_JPRB, 0.99292E+00_JPRB, 0.97941E+00_JPRB ,&
&  0.97883E+00_JPRB, 0.97446E+00_JPRB, 0.97100E+00_JPRB, 0.95301E+00_JPRB, 0.91974E+00_JPRB, 0.83662E+00_JPRB, 0.72942E+00_JPRB ,&
&  0.71102E+00_JPRB, 0.74943E+00_JPRB, 0.79453E+00_JPRB, 0.81594E+00_JPRB, 0.82079E+00_JPRB, 0.83083E+00_JPRB, 0.83052E+00_JPRB ,&
&  0.84449E+00_JPRB, 0.86911E+00_JPRB, 0.92491E+00_JPRB, 0.10168E+01_JPRB, 0.10982E+01_JPRB, 0.10987E+01_JPRB, 0.10983E+01_JPRB ,&
&  0.10615E+01_JPRB, 0.10526E+01_JPRB, 0.10121E+01_JPRB, 0.94101E+00_JPRB, 0.94627E+00_JPRB, 0.95682E+00_JPRB, 0.10117E+01_JPRB ,&
&  0.10685E+01_JPRB, 0.10778E+01_JPRB, 0.10586E+01_JPRB, 0.10391E+01_JPRB, 0.10157E+01_JPRB, 0.98598E+00_JPRB, 0.95368E+00_JPRB ,&
&  0.92583E+00_JPRB, 0.90588E+00_JPRB, 0.89537E+00_JPRB, 0.90206E+00_JPRB, 0.91464E+00_JPRB, 0.10488E+01_JPRB, 0.11617E+01_JPRB ,&
&  0.12791E+01_JPRB, 0.15060E+01_JPRB, 0.18926E+01_JPRB, 0.20847E+01_JPRB, 0.25097E+01_JPRB, 0.28680E+01_JPRB, 0.32926E+01_JPRB ,&
&  0.36713E+01_JPRB, 0.36812E+01_JPRB, 0.40317E+01_JPRB, 0.41378E+01_JPRB, 0.41645E+01_JPRB, 0.40535E+01_JPRB, 0.38434E+01_JPRB ,&
&  0.37294E+01_JPRB, 0.37778E+01_JPRB, 0.38706E+01_JPRB, 0.37980E+01_JPRB, 0.35988E+01_JPRB, 0.34652E+01_JPRB, 0.33212E+01_JPRB ,&
&  0.32467E+01_JPRB, 0.31739E+01_JPRB, 0.30094E+01_JPRB, 0.27684E+01_JPRB, 0.24652E+01_JPRB, 0.21488E+01_JPRB, 0.18612E+01_JPRB ,&
&  0.16153E+01_JPRB, 0.13468E+01_JPRB, 0.12252E+01_JPRB, 0.11936E+01_JPRB, 0.11782E+01_JPRB, 0.10935E+01_JPRB, 0.10645E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10121E+01_JPRB, 0.10315E+01_JPRB, 0.10181E+01_JPRB, 0.10355E+01_JPRB, 0.10288E+01_JPRB, 0.95039E+00_JPRB ,&
&  0.89465E+00_JPRB, 0.89911E+00_JPRB, 0.88227E+00_JPRB, 0.87159E+00_JPRB, 0.88446E+00_JPRB /)
ZBURD(:, 15, 11) = (/ &
&  0.95029E+00_JPRB, 0.97790E+00_JPRB, 0.99443E+00_JPRB, 0.10054E+01_JPRB, 0.10246E+01_JPRB, 0.10591E+01_JPRB, 0.10627E+01_JPRB ,&
&  0.10491E+01_JPRB, 0.10367E+01_JPRB, 0.10294E+01_JPRB, 0.10156E+01_JPRB, 0.98844E+00_JPRB, 0.96221E+00_JPRB, 0.90541E+00_JPRB ,&
&  0.81174E+00_JPRB, 0.91641E+00_JPRB, 0.88494E+00_JPRB, 0.92266E+00_JPRB, 0.92600E+00_JPRB, 0.89332E+00_JPRB, 0.86095E+00_JPRB ,&
&  0.89010E+00_JPRB, 0.92307E+00_JPRB, 0.10032E+01_JPRB, 0.10808E+01_JPRB, 0.10485E+01_JPRB, 0.10480E+01_JPRB, 0.11261E+01_JPRB ,&
&  0.11257E+01_JPRB, 0.11202E+01_JPRB, 0.10748E+01_JPRB, 0.10661E+01_JPRB, 0.11073E+01_JPRB, 0.11888E+01_JPRB, 0.12734E+01_JPRB ,&
&  0.12876E+01_JPRB, 0.12422E+01_JPRB, 0.11898E+01_JPRB, 0.11452E+01_JPRB, 0.11078E+01_JPRB, 0.10677E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.97806E+00_JPRB, 0.94609E+00_JPRB, 0.93057E+00_JPRB, 0.94379E+00_JPRB, 0.10078E+01_JPRB, 0.12447E+01_JPRB, 0.14643E+01_JPRB ,&
&  0.16309E+01_JPRB, 0.18576E+01_JPRB, 0.22044E+01_JPRB, 0.26166E+01_JPRB, 0.29949E+01_JPRB, 0.33405E+01_JPRB, 0.36540E+01_JPRB ,&
&  0.40263E+01_JPRB, 0.41055E+01_JPRB, 0.46382E+01_JPRB, 0.46130E+01_JPRB, 0.44398E+01_JPRB, 0.43822E+01_JPRB, 0.41167E+01_JPRB ,&
&  0.38778E+01_JPRB, 0.36299E+01_JPRB, 0.38222E+01_JPRB, 0.38399E+01_JPRB, 0.37924E+01_JPRB, 0.37704E+01_JPRB, 0.36727E+01_JPRB ,&
&  0.35810E+01_JPRB, 0.34540E+01_JPRB, 0.29980E+01_JPRB, 0.26166E+01_JPRB, 0.21446E+01_JPRB, 0.21496E+01_JPRB, 0.19043E+01_JPRB ,&
&  0.16090E+01_JPRB, 0.12811E+01_JPRB, 0.12507E+01_JPRB, 0.12148E+01_JPRB, 0.12327E+01_JPRB, 0.11918E+01_JPRB, 0.11309E+01_JPRB ,&
&  0.11369E+01_JPRB, 0.11664E+01_JPRB, 0.11321E+01_JPRB, 0.11309E+01_JPRB, 0.11537E+01_JPRB, 0.11272E+01_JPRB, 0.10130E+01_JPRB ,&
&  0.96003E+00_JPRB, 0.94588E+00_JPRB, 0.90054E+00_JPRB, 0.89088E+00_JPRB, 0.91247E+00_JPRB /)
ZBURD(:, 16, 11) = (/ &
&  0.10068E+01_JPRB, 0.10287E+01_JPRB, 0.10409E+01_JPRB, 0.10552E+01_JPRB, 0.10825E+01_JPRB, 0.11152E+01_JPRB, 0.11135E+01_JPRB ,&
&  0.10966E+01_JPRB, 0.10826E+01_JPRB, 0.10705E+01_JPRB, 0.10502E+01_JPRB, 0.10258E+01_JPRB, 0.99102E+00_JPRB, 0.10065E+01_JPRB ,&
&  0.90366E+00_JPRB, 0.10107E+01_JPRB, 0.90186E+00_JPRB, 0.93173E+00_JPRB, 0.11572E+01_JPRB, 0.10572E+01_JPRB, 0.10094E+01_JPRB ,&
&  0.96466E+00_JPRB, 0.10071E+01_JPRB, 0.10731E+01_JPRB, 0.10857E+01_JPRB, 0.10082E+01_JPRB, 0.10573E+01_JPRB, 0.11745E+01_JPRB ,&
&  0.12269E+01_JPRB, 0.12428E+01_JPRB, 0.12960E+01_JPRB, 0.13155E+01_JPRB, 0.13574E+01_JPRB, 0.14750E+01_JPRB, 0.15176E+01_JPRB ,&
&  0.14520E+01_JPRB, 0.13777E+01_JPRB, 0.13005E+01_JPRB, 0.12400E+01_JPRB, 0.11884E+01_JPRB, 0.11361E+01_JPRB, 0.10822E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.99656E+00_JPRB, 0.98199E+00_JPRB, 0.10253E+01_JPRB, 0.10910E+01_JPRB, 0.13999E+01_JPRB, 0.19704E+01_JPRB ,&
&  0.24891E+01_JPRB, 0.26879E+01_JPRB, 0.31127E+01_JPRB, 0.38129E+01_JPRB, 0.40984E+01_JPRB, 0.42575E+01_JPRB, 0.43426E+01_JPRB ,&
&  0.44875E+01_JPRB, 0.45667E+01_JPRB, 0.45318E+01_JPRB, 0.45786E+01_JPRB, 0.44205E+01_JPRB, 0.44722E+01_JPRB, 0.42475E+01_JPRB ,&
&  0.38339E+01_JPRB, 0.37270E+01_JPRB, 0.38191E+01_JPRB, 0.37663E+01_JPRB, 0.36626E+01_JPRB, 0.36957E+01_JPRB, 0.37121E+01_JPRB ,&
&  0.35553E+01_JPRB, 0.30906E+01_JPRB, 0.23356E+01_JPRB, 0.21357E+01_JPRB, 0.15686E+01_JPRB, 0.15762E+01_JPRB, 0.18916E+01_JPRB ,&
&  0.18654E+01_JPRB, 0.16359E+01_JPRB, 0.14478E+01_JPRB, 0.13487E+01_JPRB, 0.13217E+01_JPRB, 0.14682E+01_JPRB, 0.13977E+01_JPRB ,&
&  0.12677E+01_JPRB, 0.12980E+01_JPRB, 0.13212E+01_JPRB, 0.13389E+01_JPRB, 0.13435E+01_JPRB, 0.13060E+01_JPRB, 0.12012E+01_JPRB ,&
&  0.11547E+01_JPRB, 0.10770E+01_JPRB, 0.10038E+01_JPRB, 0.97487E+00_JPRB, 0.98165E+00_JPRB /)
ZBURD(:, 17, 11) = (/ &
&  0.11148E+01_JPRB, 0.11111E+01_JPRB, 0.11087E+01_JPRB, 0.11151E+01_JPRB, 0.11480E+01_JPRB, 0.11696E+01_JPRB, 0.11554E+01_JPRB ,&
&  0.11351E+01_JPRB, 0.11199E+01_JPRB, 0.11005E+01_JPRB, 0.10754E+01_JPRB, 0.10357E+01_JPRB, 0.98194E+00_JPRB, 0.98264E+00_JPRB ,&
&  0.11659E+01_JPRB, 0.12168E+01_JPRB, 0.11132E+01_JPRB, 0.96432E+00_JPRB, 0.11089E+01_JPRB, 0.10612E+01_JPRB, 0.11007E+01_JPRB ,&
&  0.10531E+01_JPRB, 0.11547E+01_JPRB, 0.12008E+01_JPRB, 0.11309E+01_JPRB, 0.10749E+01_JPRB, 0.12222E+01_JPRB, 0.14141E+01_JPRB ,&
&  0.15900E+01_JPRB, 0.17196E+01_JPRB, 0.17491E+01_JPRB, 0.17679E+01_JPRB, 0.18374E+01_JPRB, 0.16785E+01_JPRB, 0.16608E+01_JPRB ,&
&  0.15646E+01_JPRB, 0.14654E+01_JPRB, 0.13654E+01_JPRB, 0.12951E+01_JPRB, 0.12360E+01_JPRB, 0.11936E+01_JPRB, 0.11652E+01_JPRB ,&
&  0.11443E+01_JPRB, 0.11196E+01_JPRB, 0.10746E+01_JPRB, 0.10761E+01_JPRB, 0.11157E+01_JPRB, 0.13406E+01_JPRB, 0.17444E+01_JPRB ,&
&  0.23120E+01_JPRB, 0.26133E+01_JPRB, 0.30245E+01_JPRB, 0.37368E+01_JPRB, 0.40832E+01_JPRB, 0.42306E+01_JPRB, 0.44423E+01_JPRB ,&
&  0.45902E+01_JPRB, 0.46534E+01_JPRB, 0.46400E+01_JPRB, 0.45532E+01_JPRB, 0.45718E+01_JPRB, 0.45459E+01_JPRB, 0.43726E+01_JPRB ,&
&  0.39797E+01_JPRB, 0.37709E+01_JPRB, 0.37630E+01_JPRB, 0.35101E+01_JPRB, 0.33365E+01_JPRB, 0.30894E+01_JPRB, 0.32419E+01_JPRB ,&
&  0.31890E+01_JPRB, 0.19416E+01_JPRB, 0.15481E+01_JPRB, 0.18312E+01_JPRB, 0.15076E+01_JPRB, 0.14826E+01_JPRB, 0.16943E+01_JPRB ,&
&  0.16738E+01_JPRB, 0.16021E+01_JPRB, 0.15958E+01_JPRB, 0.14965E+01_JPRB, 0.14768E+01_JPRB, 0.17653E+01_JPRB, 0.17218E+01_JPRB ,&
&  0.16449E+01_JPRB, 0.16168E+01_JPRB, 0.16874E+01_JPRB, 0.16316E+01_JPRB, 0.15608E+01_JPRB, 0.15231E+01_JPRB, 0.14461E+01_JPRB ,&
&  0.13295E+01_JPRB, 0.12360E+01_JPRB, 0.11599E+01_JPRB, 0.11255E+01_JPRB, 0.11178E+01_JPRB /)
ZBURD(:, 18, 11) = (/ &
&  0.12701E+01_JPRB, 0.12306E+01_JPRB, 0.12005E+01_JPRB, 0.11992E+01_JPRB, 0.12239E+01_JPRB, 0.12323E+01_JPRB, 0.12074E+01_JPRB ,&
&  0.11852E+01_JPRB, 0.11592E+01_JPRB, 0.11315E+01_JPRB, 0.10923E+01_JPRB, 0.10342E+01_JPRB, 0.97484E+00_JPRB, 0.95948E+00_JPRB ,&
&  0.10379E+01_JPRB, 0.12172E+01_JPRB, 0.10468E+01_JPRB, 0.10084E+01_JPRB, 0.92733E+00_JPRB, 0.10022E+01_JPRB, 0.10886E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.12239E+01_JPRB, 0.12798E+01_JPRB, 0.12663E+01_JPRB, 0.12762E+01_JPRB, 0.15093E+01_JPRB, 0.18772E+01_JPRB ,&
&  0.21132E+01_JPRB, 0.22708E+01_JPRB, 0.21093E+01_JPRB, 0.22188E+01_JPRB, 0.20607E+01_JPRB, 0.18291E+01_JPRB, 0.18217E+01_JPRB ,&
&  0.16550E+01_JPRB, 0.15129E+01_JPRB, 0.13788E+01_JPRB, 0.13098E+01_JPRB, 0.12708E+01_JPRB, 0.12289E+01_JPRB, 0.11940E+01_JPRB ,&
&  0.11522E+01_JPRB, 0.11205E+01_JPRB, 0.11015E+01_JPRB, 0.10883E+01_JPRB, 0.11107E+01_JPRB, 0.12768E+01_JPRB, 0.14859E+01_JPRB ,&
&  0.18738E+01_JPRB, 0.18836E+01_JPRB, 0.18609E+01_JPRB, 0.26097E+01_JPRB, 0.37159E+01_JPRB, 0.39510E+01_JPRB, 0.38447E+01_JPRB ,&
&  0.42194E+01_JPRB, 0.44085E+01_JPRB, 0.45349E+01_JPRB, 0.45820E+01_JPRB, 0.46464E+01_JPRB, 0.45109E+01_JPRB, 0.42538E+01_JPRB ,&
&  0.39378E+01_JPRB, 0.37403E+01_JPRB, 0.35574E+01_JPRB, 0.33970E+01_JPRB, 0.31665E+01_JPRB, 0.31314E+01_JPRB, 0.32516E+01_JPRB ,&
&  0.30268E+01_JPRB, 0.26667E+01_JPRB, 0.18199E+01_JPRB, 0.18535E+01_JPRB, 0.14404E+01_JPRB, 0.13969E+01_JPRB, 0.15925E+01_JPRB ,&
&  0.15626E+01_JPRB, 0.16611E+01_JPRB, 0.16874E+01_JPRB, 0.16102E+01_JPRB, 0.17922E+01_JPRB, 0.20420E+01_JPRB, 0.20972E+01_JPRB ,&
&  0.21841E+01_JPRB, 0.19961E+01_JPRB, 0.21335E+01_JPRB, 0.20736E+01_JPRB, 0.19786E+01_JPRB, 0.18971E+01_JPRB, 0.17957E+01_JPRB ,&
&  0.16548E+01_JPRB, 0.15060E+01_JPRB, 0.13936E+01_JPRB, 0.13335E+01_JPRB, 0.12964E+01_JPRB /)
ZBURD(:, 19, 11) = (/ &
&  0.13668E+01_JPRB, 0.13023E+01_JPRB, 0.12600E+01_JPRB, 0.12499E+01_JPRB, 0.12636E+01_JPRB, 0.12628E+01_JPRB, 0.12299E+01_JPRB ,&
&  0.11999E+01_JPRB, 0.11653E+01_JPRB, 0.11353E+01_JPRB, 0.10772E+01_JPRB, 0.10265E+01_JPRB, 0.96209E+00_JPRB, 0.93427E+00_JPRB ,&
&  0.99638E+00_JPRB, 0.10555E+01_JPRB, 0.91694E+00_JPRB, 0.86058E+00_JPRB, 0.80542E+00_JPRB, 0.87376E+00_JPRB, 0.10096E+01_JPRB ,&
&  0.11548E+01_JPRB, 0.12595E+01_JPRB, 0.14858E+01_JPRB, 0.15393E+01_JPRB, 0.16280E+01_JPRB, 0.19617E+01_JPRB, 0.24240E+01_JPRB ,&
&  0.29787E+01_JPRB, 0.24906E+01_JPRB, 0.24935E+01_JPRB, 0.23338E+01_JPRB, 0.21781E+01_JPRB, 0.19794E+01_JPRB, 0.17757E+01_JPRB ,&
&  0.15861E+01_JPRB, 0.14306E+01_JPRB, 0.13530E+01_JPRB, 0.12963E+01_JPRB, 0.12532E+01_JPRB, 0.12255E+01_JPRB, 0.12096E+01_JPRB ,&
&  0.11996E+01_JPRB, 0.11722E+01_JPRB, 0.11262E+01_JPRB, 0.10843E+01_JPRB, 0.12109E+01_JPRB, 0.14091E+01_JPRB, 0.15129E+01_JPRB ,&
&  0.15085E+01_JPRB, 0.17206E+01_JPRB, 0.24773E+01_JPRB, 0.28594E+01_JPRB, 0.32310E+01_JPRB, 0.36597E+01_JPRB, 0.40162E+01_JPRB ,&
&  0.42779E+01_JPRB, 0.43185E+01_JPRB, 0.43495E+01_JPRB, 0.38140E+01_JPRB, 0.43016E+01_JPRB, 0.42630E+01_JPRB, 0.39686E+01_JPRB ,&
&  0.36061E+01_JPRB, 0.34879E+01_JPRB, 0.31800E+01_JPRB, 0.31129E+01_JPRB, 0.31947E+01_JPRB, 0.34373E+01_JPRB, 0.30139E+01_JPRB ,&
&  0.25602E+01_JPRB, 0.28334E+01_JPRB, 0.24960E+01_JPRB, 0.20179E+01_JPRB, 0.16330E+01_JPRB, 0.16302E+01_JPRB, 0.16705E+01_JPRB ,&
&  0.16730E+01_JPRB, 0.16954E+01_JPRB, 0.16788E+01_JPRB, 0.18373E+01_JPRB, 0.22314E+01_JPRB, 0.24435E+01_JPRB, 0.24090E+01_JPRB ,&
&  0.23280E+01_JPRB, 0.24839E+01_JPRB, 0.24736E+01_JPRB, 0.24016E+01_JPRB, 0.23384E+01_JPRB, 0.22468E+01_JPRB, 0.21073E+01_JPRB ,&
&  0.19374E+01_JPRB, 0.17807E+01_JPRB, 0.16522E+01_JPRB, 0.15386E+01_JPRB, 0.14437E+01_JPRB /)
ZBURD(:, 20, 11) = (/ &
&  0.14067E+01_JPRB, 0.13379E+01_JPRB, 0.13014E+01_JPRB, 0.12949E+01_JPRB, 0.13070E+01_JPRB, 0.12877E+01_JPRB, 0.12455E+01_JPRB ,&
&  0.11936E+01_JPRB, 0.11406E+01_JPRB, 0.10828E+01_JPRB, 0.10342E+01_JPRB, 0.10029E+01_JPRB, 0.94993E+00_JPRB, 0.93479E+00_JPRB ,&
&  0.99583E+00_JPRB, 0.96062E+00_JPRB, 0.87189E+00_JPRB, 0.95883E+00_JPRB, 0.99842E+00_JPRB, 0.92138E+00_JPRB, 0.10099E+01_JPRB ,&
&  0.11308E+01_JPRB, 0.13485E+01_JPRB, 0.16694E+01_JPRB, 0.20173E+01_JPRB, 0.23060E+01_JPRB, 0.27410E+01_JPRB, 0.30952E+01_JPRB ,&
&  0.30291E+01_JPRB, 0.29445E+01_JPRB, 0.27381E+01_JPRB, 0.25007E+01_JPRB, 0.21671E+01_JPRB, 0.18848E+01_JPRB, 0.16767E+01_JPRB ,&
&  0.14962E+01_JPRB, 0.13870E+01_JPRB, 0.13286E+01_JPRB, 0.12874E+01_JPRB, 0.12711E+01_JPRB, 0.12641E+01_JPRB, 0.12767E+01_JPRB ,&
&  0.12341E+01_JPRB, 0.11829E+01_JPRB, 0.11174E+01_JPRB, 0.11368E+01_JPRB, 0.13148E+01_JPRB, 0.14543E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.17688E+01_JPRB, 0.21045E+01_JPRB, 0.22608E+01_JPRB, 0.28822E+01_JPRB, 0.26173E+01_JPRB, 0.28416E+01_JPRB, 0.36703E+01_JPRB ,&
&  0.39952E+01_JPRB, 0.40876E+01_JPRB, 0.41046E+01_JPRB, 0.32920E+01_JPRB, 0.28478E+01_JPRB, 0.39126E+01_JPRB, 0.37246E+01_JPRB ,&
&  0.33119E+01_JPRB, 0.31585E+01_JPRB, 0.29566E+01_JPRB, 0.32512E+01_JPRB, 0.26592E+01_JPRB, 0.24146E+01_JPRB, 0.19538E+01_JPRB ,&
&  0.20248E+01_JPRB, 0.17749E+01_JPRB, 0.20773E+01_JPRB, 0.18781E+01_JPRB, 0.18506E+01_JPRB, 0.17887E+01_JPRB, 0.16811E+01_JPRB ,&
&  0.16738E+01_JPRB, 0.16878E+01_JPRB, 0.17722E+01_JPRB, 0.22686E+01_JPRB, 0.28973E+01_JPRB, 0.27107E+01_JPRB, 0.28518E+01_JPRB ,&
&  0.29205E+01_JPRB, 0.28917E+01_JPRB, 0.26755E+01_JPRB, 0.26502E+01_JPRB, 0.25379E+01_JPRB, 0.24065E+01_JPRB, 0.22578E+01_JPRB ,&
&  0.21018E+01_JPRB, 0.19541E+01_JPRB, 0.17975E+01_JPRB, 0.16417E+01_JPRB, 0.15081E+01_JPRB /)
ZBURD(:, 21, 11) = (/ &
&  0.14398E+01_JPRB, 0.13850E+01_JPRB, 0.13632E+01_JPRB, 0.13561E+01_JPRB, 0.13373E+01_JPRB, 0.12922E+01_JPRB, 0.12341E+01_JPRB ,&
&  0.11680E+01_JPRB, 0.10899E+01_JPRB, 0.10239E+01_JPRB, 0.98238E+00_JPRB, 0.96760E+00_JPRB, 0.94488E+00_JPRB, 0.94191E+00_JPRB ,&
&  0.10134E+01_JPRB, 0.10491E+01_JPRB, 0.96158E+00_JPRB, 0.94365E+00_JPRB, 0.11292E+01_JPRB, 0.10323E+01_JPRB, 0.99237E+00_JPRB ,&
&  0.11962E+01_JPRB, 0.14562E+01_JPRB, 0.19029E+01_JPRB, 0.24973E+01_JPRB, 0.29358E+01_JPRB, 0.33340E+01_JPRB, 0.35135E+01_JPRB ,&
&  0.35410E+01_JPRB, 0.31469E+01_JPRB, 0.27332E+01_JPRB, 0.23711E+01_JPRB, 0.20029E+01_JPRB, 0.17158E+01_JPRB, 0.14842E+01_JPRB ,&
&  0.13435E+01_JPRB, 0.12814E+01_JPRB, 0.12472E+01_JPRB, 0.12092E+01_JPRB, 0.12109E+01_JPRB, 0.11707E+01_JPRB, 0.11404E+01_JPRB ,&
&  0.11428E+01_JPRB, 0.11309E+01_JPRB, 0.11121E+01_JPRB, 0.11923E+01_JPRB, 0.13037E+01_JPRB, 0.14459E+01_JPRB, 0.15465E+01_JPRB ,&
&  0.17386E+01_JPRB, 0.19707E+01_JPRB, 0.21108E+01_JPRB, 0.24987E+01_JPRB, 0.30797E+01_JPRB, 0.32080E+01_JPRB, 0.36395E+01_JPRB ,&
&  0.31418E+01_JPRB, 0.26723E+01_JPRB, 0.22977E+01_JPRB, 0.18763E+01_JPRB, 0.22017E+01_JPRB, 0.34556E+01_JPRB, 0.35379E+01_JPRB ,&
&  0.31632E+01_JPRB, 0.30890E+01_JPRB, 0.31418E+01_JPRB, 0.28592E+01_JPRB, 0.23312E+01_JPRB, 0.18178E+01_JPRB, 0.24087E+01_JPRB ,&
&  0.25263E+01_JPRB, 0.24889E+01_JPRB, 0.22899E+01_JPRB, 0.19085E+01_JPRB, 0.16156E+01_JPRB, 0.18771E+01_JPRB, 0.18256E+01_JPRB ,&
&  0.17638E+01_JPRB, 0.17784E+01_JPRB, 0.24028E+01_JPRB, 0.32580E+01_JPRB, 0.35215E+01_JPRB, 0.34593E+01_JPRB, 0.35181E+01_JPRB ,&
&  0.32548E+01_JPRB, 0.30853E+01_JPRB, 0.28911E+01_JPRB, 0.26684E+01_JPRB, 0.24747E+01_JPRB, 0.22819E+01_JPRB, 0.21260E+01_JPRB ,&
&  0.19889E+01_JPRB, 0.18657E+01_JPRB, 0.17387E+01_JPRB, 0.16191E+01_JPRB, 0.15172E+01_JPRB /)
ZBURD(:, 22, 11) = (/ &
&  0.14614E+01_JPRB, 0.14317E+01_JPRB, 0.14031E+01_JPRB, 0.13717E+01_JPRB, 0.13262E+01_JPRB, 0.12695E+01_JPRB, 0.11994E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.10414E+01_JPRB, 0.98179E+00_JPRB, 0.96128E+00_JPRB, 0.94294E+00_JPRB, 0.92105E+00_JPRB, 0.93994E+00_JPRB ,&
&  0.10458E+01_JPRB, 0.12521E+01_JPRB, 0.10747E+01_JPRB, 0.10275E+01_JPRB, 0.10838E+01_JPRB, 0.11203E+01_JPRB, 0.96710E+00_JPRB ,&
&  0.12939E+01_JPRB, 0.15907E+01_JPRB, 0.19804E+01_JPRB, 0.26878E+01_JPRB, 0.36595E+01_JPRB, 0.34298E+01_JPRB, 0.35027E+01_JPRB ,&
&  0.35062E+01_JPRB, 0.30470E+01_JPRB, 0.27036E+01_JPRB, 0.22872E+01_JPRB, 0.18776E+01_JPRB, 0.15846E+01_JPRB, 0.13386E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.12307E+01_JPRB, 0.12242E+01_JPRB, 0.12061E+01_JPRB, 0.11806E+01_JPRB, 0.11506E+01_JPRB, 0.11134E+01_JPRB ,&
&  0.11064E+01_JPRB, 0.11125E+01_JPRB, 0.11392E+01_JPRB, 0.13284E+01_JPRB, 0.15353E+01_JPRB, 0.14804E+01_JPRB, 0.18448E+01_JPRB ,&
&  0.19687E+01_JPRB, 0.21799E+01_JPRB, 0.21871E+01_JPRB, 0.25130E+01_JPRB, 0.28994E+01_JPRB, 0.31026E+01_JPRB, 0.33375E+01_JPRB ,&
&  0.26443E+01_JPRB, 0.23014E+01_JPRB, 0.24222E+01_JPRB, 0.24270E+01_JPRB, 0.20349E+01_JPRB, 0.21819E+01_JPRB, 0.33810E+01_JPRB ,&
&  0.24678E+01_JPRB, 0.26743E+01_JPRB, 0.30039E+01_JPRB, 0.31424E+01_JPRB, 0.16135E+01_JPRB, 0.11758E+01_JPRB, 0.19237E+01_JPRB ,&
&  0.20036E+01_JPRB, 0.12674E+01_JPRB, 0.10066E+01_JPRB, 0.12967E+01_JPRB, 0.10548E+01_JPRB, 0.12651E+01_JPRB, 0.17001E+01_JPRB ,&
&  0.19386E+01_JPRB, 0.21126E+01_JPRB, 0.30933E+01_JPRB, 0.35482E+01_JPRB, 0.36741E+01_JPRB, 0.41354E+01_JPRB, 0.38561E+01_JPRB ,&
&  0.35637E+01_JPRB, 0.29928E+01_JPRB, 0.28564E+01_JPRB, 0.25443E+01_JPRB, 0.23179E+01_JPRB, 0.21079E+01_JPRB, 0.19292E+01_JPRB ,&
&  0.17898E+01_JPRB, 0.16757E+01_JPRB, 0.15946E+01_JPRB, 0.15305E+01_JPRB, 0.14952E+01_JPRB /)
ZBURD(:, 23, 11) = (/ &
&  0.14820E+01_JPRB, 0.14460E+01_JPRB, 0.14088E+01_JPRB, 0.13639E+01_JPRB, 0.13078E+01_JPRB, 0.12335E+01_JPRB, 0.11412E+01_JPRB ,&
&  0.10529E+01_JPRB, 0.98769E+00_JPRB, 0.94864E+00_JPRB, 0.93629E+00_JPRB, 0.91681E+00_JPRB, 0.90145E+00_JPRB, 0.93867E+00_JPRB ,&
&  0.10485E+01_JPRB, 0.12295E+01_JPRB, 0.13331E+01_JPRB, 0.12130E+01_JPRB, 0.11581E+01_JPRB, 0.11429E+01_JPRB, 0.11301E+01_JPRB ,&
&  0.14880E+01_JPRB, 0.17241E+01_JPRB, 0.20788E+01_JPRB, 0.26629E+01_JPRB, 0.31619E+01_JPRB, 0.31418E+01_JPRB, 0.33819E+01_JPRB ,&
&  0.32109E+01_JPRB, 0.29107E+01_JPRB, 0.25099E+01_JPRB, 0.20790E+01_JPRB, 0.17104E+01_JPRB, 0.14092E+01_JPRB, 0.12624E+01_JPRB ,&
&  0.12016E+01_JPRB, 0.12248E+01_JPRB, 0.12340E+01_JPRB, 0.12024E+01_JPRB, 0.11858E+01_JPRB, 0.11592E+01_JPRB, 0.11350E+01_JPRB ,&
&  0.11306E+01_JPRB, 0.11687E+01_JPRB, 0.12286E+01_JPRB, 0.13842E+01_JPRB, 0.16499E+01_JPRB, 0.16811E+01_JPRB, 0.15978E+01_JPRB ,&
&  0.15780E+01_JPRB, 0.17626E+01_JPRB, 0.21248E+01_JPRB, 0.23193E+01_JPRB, 0.25820E+01_JPRB, 0.28878E+01_JPRB, 0.29068E+01_JPRB ,&
&  0.28439E+01_JPRB, 0.28359E+01_JPRB, 0.24206E+01_JPRB, 0.25736E+01_JPRB, 0.23706E+01_JPRB, 0.18907E+01_JPRB, 0.22457E+01_JPRB ,&
&  0.24248E+01_JPRB, 0.23747E+01_JPRB, 0.20098E+01_JPRB, 0.15810E+01_JPRB, 0.17135E+01_JPRB, 0.13555E+01_JPRB, 0.93434E+00_JPRB ,&
&  0.86825E+00_JPRB, 0.83447E+00_JPRB, 0.80492E+00_JPRB, 0.86140E+00_JPRB, 0.89022E+00_JPRB, 0.11919E+01_JPRB, 0.16365E+01_JPRB ,&
&  0.23971E+01_JPRB, 0.29110E+01_JPRB, 0.33065E+01_JPRB, 0.34970E+01_JPRB, 0.33671E+01_JPRB, 0.36424E+01_JPRB, 0.35559E+01_JPRB ,&
&  0.30454E+01_JPRB, 0.28378E+01_JPRB, 0.26057E+01_JPRB, 0.23524E+01_JPRB, 0.21702E+01_JPRB, 0.20174E+01_JPRB, 0.18826E+01_JPRB ,&
&  0.17780E+01_JPRB, 0.16911E+01_JPRB, 0.16261E+01_JPRB, 0.15728E+01_JPRB, 0.15182E+01_JPRB /)
ZBURD(:, 24, 11) = (/ &
&  0.14539E+01_JPRB, 0.14111E+01_JPRB, 0.13684E+01_JPRB, 0.13091E+01_JPRB, 0.12404E+01_JPRB, 0.11530E+01_JPRB, 0.10506E+01_JPRB ,&
&  0.96779E+00_JPRB, 0.91995E+00_JPRB, 0.90668E+00_JPRB, 0.89077E+00_JPRB, 0.88185E+00_JPRB, 0.88922E+00_JPRB, 0.94291E+00_JPRB ,&
&  0.10477E+01_JPRB, 0.12060E+01_JPRB, 0.14577E+01_JPRB, 0.13926E+01_JPRB, 0.14160E+01_JPRB, 0.13223E+01_JPRB, 0.14092E+01_JPRB ,&
&  0.17063E+01_JPRB, 0.19311E+01_JPRB, 0.23681E+01_JPRB, 0.27586E+01_JPRB, 0.32132E+01_JPRB, 0.32371E+01_JPRB, 0.31847E+01_JPRB ,&
&  0.29670E+01_JPRB, 0.27168E+01_JPRB, 0.22742E+01_JPRB, 0.18625E+01_JPRB, 0.15184E+01_JPRB, 0.13024E+01_JPRB, 0.11842E+01_JPRB ,&
&  0.11762E+01_JPRB, 0.12248E+01_JPRB, 0.12491E+01_JPRB, 0.12246E+01_JPRB, 0.12047E+01_JPRB, 0.12011E+01_JPRB, 0.12111E+01_JPRB ,&
&  0.12157E+01_JPRB, 0.12570E+01_JPRB, 0.13048E+01_JPRB, 0.14221E+01_JPRB, 0.13138E+01_JPRB, 0.12069E+01_JPRB, 0.14592E+01_JPRB ,&
&  0.16539E+01_JPRB, 0.19593E+01_JPRB, 0.19827E+01_JPRB, 0.22416E+01_JPRB, 0.23779E+01_JPRB, 0.24141E+01_JPRB, 0.25976E+01_JPRB ,&
&  0.25849E+01_JPRB, 0.28521E+01_JPRB, 0.21714E+01_JPRB, 0.22274E+01_JPRB, 0.28054E+01_JPRB, 0.23000E+01_JPRB, 0.21645E+01_JPRB ,&
&  0.24985E+01_JPRB, 0.23088E+01_JPRB, 0.23737E+01_JPRB, 0.20981E+01_JPRB, 0.33084E+01_JPRB, 0.36461E+01_JPRB, 0.12440E+01_JPRB ,&
&  0.93847E+00_JPRB, 0.86340E+00_JPRB, 0.81371E+00_JPRB, 0.78097E+00_JPRB, 0.90228E+00_JPRB, 0.13309E+01_JPRB, 0.21612E+01_JPRB ,&
&  0.26193E+01_JPRB, 0.33515E+01_JPRB, 0.35340E+01_JPRB, 0.37580E+01_JPRB, 0.32747E+01_JPRB, 0.32343E+01_JPRB, 0.27993E+01_JPRB ,&
&  0.26879E+01_JPRB, 0.25676E+01_JPRB, 0.22909E+01_JPRB, 0.20754E+01_JPRB, 0.19522E+01_JPRB, 0.18659E+01_JPRB, 0.17793E+01_JPRB ,&
&  0.16998E+01_JPRB, 0.16452E+01_JPRB, 0.15946E+01_JPRB, 0.15451E+01_JPRB, 0.14986E+01_JPRB /)
ZBURD(:, 25, 11) = (/ &
&  0.13721E+01_JPRB, 0.13307E+01_JPRB, 0.12800E+01_JPRB, 0.12068E+01_JPRB, 0.11402E+01_JPRB, 0.10605E+01_JPRB, 0.97344E+00_JPRB ,&
&  0.92397E+00_JPRB, 0.90714E+00_JPRB, 0.88552E+00_JPRB, 0.86351E+00_JPRB, 0.86932E+00_JPRB, 0.89569E+00_JPRB, 0.96122E+00_JPRB ,&
&  0.10710E+01_JPRB, 0.12070E+01_JPRB, 0.13284E+01_JPRB, 0.14045E+01_JPRB, 0.14080E+01_JPRB, 0.13379E+01_JPRB, 0.16529E+01_JPRB ,&
&  0.19814E+01_JPRB, 0.21860E+01_JPRB, 0.27042E+01_JPRB, 0.29642E+01_JPRB, 0.30547E+01_JPRB, 0.30352E+01_JPRB, 0.28769E+01_JPRB ,&
&  0.26731E+01_JPRB, 0.24076E+01_JPRB, 0.19794E+01_JPRB, 0.16461E+01_JPRB, 0.14062E+01_JPRB, 0.12472E+01_JPRB, 0.11705E+01_JPRB ,&
&  0.12051E+01_JPRB, 0.12574E+01_JPRB, 0.12856E+01_JPRB, 0.12689E+01_JPRB, 0.12542E+01_JPRB, 0.12478E+01_JPRB, 0.12594E+01_JPRB ,&
&  0.12861E+01_JPRB, 0.13359E+01_JPRB, 0.13935E+01_JPRB, 0.14850E+01_JPRB, 0.12806E+01_JPRB, 0.15430E+01_JPRB, 0.16353E+01_JPRB ,&
&  0.17644E+01_JPRB, 0.18602E+01_JPRB, 0.18022E+01_JPRB, 0.19505E+01_JPRB, 0.21637E+01_JPRB, 0.22166E+01_JPRB, 0.22855E+01_JPRB ,&
&  0.24435E+01_JPRB, 0.22242E+01_JPRB, 0.19625E+01_JPRB, 0.21302E+01_JPRB, 0.25500E+01_JPRB, 0.32396E+01_JPRB, 0.20323E+01_JPRB ,&
&  0.17835E+01_JPRB, 0.25584E+01_JPRB, 0.28540E+01_JPRB, 0.28299E+01_JPRB, 0.39870E+01_JPRB, 0.53864E+01_JPRB, 0.35198E+01_JPRB ,&
&  0.11128E+01_JPRB, 0.85611E+00_JPRB, 0.96944E+00_JPRB, 0.90717E+00_JPRB, 0.91805E+00_JPRB, 0.13917E+01_JPRB, 0.40121E+01_JPRB ,&
&  0.32265E+01_JPRB, 0.38130E+01_JPRB, 0.38882E+01_JPRB, 0.37058E+01_JPRB, 0.31553E+01_JPRB, 0.28169E+01_JPRB, 0.25557E+01_JPRB ,&
&  0.21827E+01_JPRB, 0.20382E+01_JPRB, 0.18283E+01_JPRB, 0.17118E+01_JPRB, 0.16680E+01_JPRB, 0.16408E+01_JPRB, 0.15804E+01_JPRB ,&
&  0.15348E+01_JPRB, 0.15040E+01_JPRB, 0.14730E+01_JPRB, 0.14451E+01_JPRB, 0.14141E+01_JPRB /)
ZBURD(:, 26, 11) = (/ &
&  0.12661E+01_JPRB, 0.12280E+01_JPRB, 0.11627E+01_JPRB, 0.11035E+01_JPRB, 0.10706E+01_JPRB, 0.10094E+01_JPRB, 0.96956E+00_JPRB ,&
&  0.93284E+00_JPRB, 0.92174E+00_JPRB, 0.89765E+00_JPRB, 0.89758E+00_JPRB, 0.90811E+00_JPRB, 0.95384E+00_JPRB, 0.10291E+01_JPRB ,&
&  0.11295E+01_JPRB, 0.12199E+01_JPRB, 0.12770E+01_JPRB, 0.13722E+01_JPRB, 0.15136E+01_JPRB, 0.14756E+01_JPRB, 0.17138E+01_JPRB ,&
&  0.23194E+01_JPRB, 0.26501E+01_JPRB, 0.27298E+01_JPRB, 0.27980E+01_JPRB, 0.29075E+01_JPRB, 0.30315E+01_JPRB, 0.26682E+01_JPRB ,&
&  0.23630E+01_JPRB, 0.21131E+01_JPRB, 0.17730E+01_JPRB, 0.15060E+01_JPRB, 0.13148E+01_JPRB, 0.12192E+01_JPRB, 0.12303E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.13063E+01_JPRB, 0.13232E+01_JPRB, 0.13101E+01_JPRB, 0.12969E+01_JPRB, 0.12832E+01_JPRB, 0.13167E+01_JPRB ,&
&  0.13732E+01_JPRB, 0.14305E+01_JPRB, 0.14768E+01_JPRB, 0.14857E+01_JPRB, 0.15518E+01_JPRB, 0.16124E+01_JPRB, 0.17170E+01_JPRB ,&
&  0.17293E+01_JPRB, 0.17605E+01_JPRB, 0.18148E+01_JPRB, 0.18482E+01_JPRB, 0.19932E+01_JPRB, 0.21235E+01_JPRB, 0.21929E+01_JPRB ,&
&  0.22774E+01_JPRB, 0.21991E+01_JPRB, 0.19817E+01_JPRB, 0.21093E+01_JPRB, 0.24511E+01_JPRB, 0.28960E+01_JPRB, 0.27489E+01_JPRB ,&
&  0.24917E+01_JPRB, 0.24803E+01_JPRB, 0.29243E+01_JPRB, 0.37106E+01_JPRB, 0.41905E+01_JPRB, 0.43656E+01_JPRB, 0.52442E+01_JPRB ,&
&  0.44368E+01_JPRB, 0.20866E+01_JPRB, 0.17161E+01_JPRB, 0.23712E+01_JPRB, 0.18513E+01_JPRB, 0.20292E+01_JPRB, 0.30418E+01_JPRB ,&
&  0.33586E+01_JPRB, 0.39582E+01_JPRB, 0.40337E+01_JPRB, 0.32048E+01_JPRB, 0.28040E+01_JPRB, 0.25026E+01_JPRB, 0.21704E+01_JPRB ,&
&  0.18490E+01_JPRB, 0.16528E+01_JPRB, 0.15305E+01_JPRB, 0.14498E+01_JPRB, 0.14011E+01_JPRB, 0.13551E+01_JPRB, 0.13284E+01_JPRB ,&
&  0.13036E+01_JPRB, 0.12902E+01_JPRB, 0.12876E+01_JPRB, 0.12854E+01_JPRB, 0.12841E+01_JPRB /)
ZBURD(:, 27, 11) = (/ &
&  0.11712E+01_JPRB, 0.11227E+01_JPRB, 0.10986E+01_JPRB, 0.10915E+01_JPRB, 0.10677E+01_JPRB, 0.10475E+01_JPRB, 0.10250E+01_JPRB ,&
&  0.96957E+00_JPRB, 0.96781E+00_JPRB, 0.98806E+00_JPRB, 0.99893E+00_JPRB, 0.10019E+01_JPRB, 0.10431E+01_JPRB, 0.11075E+01_JPRB ,&
&  0.11776E+01_JPRB, 0.12859E+01_JPRB, 0.13820E+01_JPRB, 0.16159E+01_JPRB, 0.18561E+01_JPRB, 0.22657E+01_JPRB, 0.16990E+01_JPRB ,&
&  0.19948E+01_JPRB, 0.28578E+01_JPRB, 0.27671E+01_JPRB, 0.28187E+01_JPRB, 0.28082E+01_JPRB, 0.27454E+01_JPRB, 0.23098E+01_JPRB ,&
&  0.20593E+01_JPRB, 0.18721E+01_JPRB, 0.16227E+01_JPRB, 0.14149E+01_JPRB, 0.12794E+01_JPRB, 0.12810E+01_JPRB, 0.12961E+01_JPRB ,&
&  0.13354E+01_JPRB, 0.13914E+01_JPRB, 0.13742E+01_JPRB, 0.13696E+01_JPRB, 0.13562E+01_JPRB, 0.13474E+01_JPRB, 0.13984E+01_JPRB ,&
&  0.14624E+01_JPRB, 0.15103E+01_JPRB, 0.15553E+01_JPRB, 0.15156E+01_JPRB, 0.16045E+01_JPRB, 0.16675E+01_JPRB, 0.17389E+01_JPRB ,&
&  0.16493E+01_JPRB, 0.15278E+01_JPRB, 0.16461E+01_JPRB, 0.18174E+01_JPRB, 0.19569E+01_JPRB, 0.19718E+01_JPRB, 0.20028E+01_JPRB ,&
&  0.22487E+01_JPRB, 0.24288E+01_JPRB, 0.25509E+01_JPRB, 0.22801E+01_JPRB, 0.23785E+01_JPRB, 0.28176E+01_JPRB, 0.32964E+01_JPRB ,&
&  0.36022E+01_JPRB, 0.37114E+01_JPRB, 0.40137E+01_JPRB, 0.40000E+01_JPRB, 0.39264E+01_JPRB, 0.38412E+01_JPRB, 0.41330E+01_JPRB ,&
&  0.48155E+01_JPRB, 0.53269E+01_JPRB, 0.45621E+01_JPRB, 0.37933E+01_JPRB, 0.38317E+01_JPRB, 0.32891E+01_JPRB, 0.30840E+01_JPRB ,&
&  0.36748E+01_JPRB, 0.36915E+01_JPRB, 0.34006E+01_JPRB, 0.33445E+01_JPRB, 0.26208E+01_JPRB, 0.21398E+01_JPRB, 0.16737E+01_JPRB ,&
&  0.13328E+01_JPRB, 0.12062E+01_JPRB, 0.11501E+01_JPRB, 0.11262E+01_JPRB, 0.11516E+01_JPRB, 0.11584E+01_JPRB, 0.11606E+01_JPRB ,&
&  0.11476E+01_JPRB, 0.11629E+01_JPRB, 0.11897E+01_JPRB, 0.12112E+01_JPRB, 0.12058E+01_JPRB /)
ZBURD(:, 28, 11) = (/ &
&  0.11031E+01_JPRB, 0.10900E+01_JPRB, 0.10992E+01_JPRB, 0.11111E+01_JPRB, 0.11127E+01_JPRB, 0.11390E+01_JPRB, 0.11285E+01_JPRB ,&
&  0.10500E+01_JPRB, 0.10133E+01_JPRB, 0.10513E+01_JPRB, 0.10529E+01_JPRB, 0.10607E+01_JPRB, 0.10951E+01_JPRB, 0.11611E+01_JPRB ,&
&  0.12382E+01_JPRB, 0.13931E+01_JPRB, 0.16110E+01_JPRB, 0.19143E+01_JPRB, 0.20867E+01_JPRB, 0.22632E+01_JPRB, 0.22744E+01_JPRB ,&
&  0.18013E+01_JPRB, 0.29652E+01_JPRB, 0.25706E+01_JPRB, 0.26685E+01_JPRB, 0.25676E+01_JPRB, 0.25879E+01_JPRB, 0.22147E+01_JPRB ,&
&  0.18719E+01_JPRB, 0.16663E+01_JPRB, 0.15698E+01_JPRB, 0.14381E+01_JPRB, 0.13556E+01_JPRB, 0.13477E+01_JPRB, 0.13774E+01_JPRB ,&
&  0.13904E+01_JPRB, 0.14066E+01_JPRB, 0.14131E+01_JPRB, 0.14202E+01_JPRB, 0.14149E+01_JPRB, 0.14222E+01_JPRB, 0.14523E+01_JPRB ,&
&  0.15184E+01_JPRB, 0.15709E+01_JPRB, 0.15750E+01_JPRB, 0.15999E+01_JPRB, 0.16258E+01_JPRB, 0.17250E+01_JPRB, 0.17864E+01_JPRB ,&
&  0.17036E+01_JPRB, 0.15825E+01_JPRB, 0.16526E+01_JPRB, 0.17105E+01_JPRB, 0.17553E+01_JPRB, 0.19432E+01_JPRB, 0.19704E+01_JPRB ,&
&  0.23473E+01_JPRB, 0.23790E+01_JPRB, 0.28017E+01_JPRB, 0.23245E+01_JPRB, 0.23760E+01_JPRB, 0.27968E+01_JPRB, 0.29388E+01_JPRB ,&
&  0.29591E+01_JPRB, 0.36051E+01_JPRB, 0.38720E+01_JPRB, 0.40767E+01_JPRB, 0.40544E+01_JPRB, 0.38305E+01_JPRB, 0.36362E+01_JPRB ,&
&  0.38401E+01_JPRB, 0.42232E+01_JPRB, 0.45587E+01_JPRB, 0.38846E+01_JPRB, 0.37120E+01_JPRB, 0.33200E+01_JPRB, 0.35458E+01_JPRB ,&
&  0.40627E+01_JPRB, 0.39827E+01_JPRB, 0.35207E+01_JPRB, 0.28952E+01_JPRB, 0.19354E+01_JPRB, 0.14644E+01_JPRB, 0.11619E+01_JPRB ,&
&  0.10334E+01_JPRB, 0.98117E+00_JPRB, 0.98961E+00_JPRB, 0.10327E+01_JPRB, 0.10755E+01_JPRB, 0.10967E+01_JPRB, 0.10939E+01_JPRB ,&
&  0.10993E+01_JPRB, 0.11154E+01_JPRB, 0.11285E+01_JPRB, 0.11296E+01_JPRB, 0.11217E+01_JPRB /)
ZBURD(:, 29, 11) = (/ &
&  0.11002E+01_JPRB, 0.11164E+01_JPRB, 0.11421E+01_JPRB, 0.11882E+01_JPRB, 0.12318E+01_JPRB, 0.13228E+01_JPRB, 0.14064E+01_JPRB ,&
&  0.12892E+01_JPRB, 0.96539E+00_JPRB, 0.10052E+01_JPRB, 0.10440E+01_JPRB, 0.10891E+01_JPRB, 0.11477E+01_JPRB, 0.12315E+01_JPRB ,&
&  0.13700E+01_JPRB, 0.15346E+01_JPRB, 0.17437E+01_JPRB, 0.19489E+01_JPRB, 0.21673E+01_JPRB, 0.23509E+01_JPRB, 0.23181E+01_JPRB ,&
&  0.19044E+01_JPRB, 0.19298E+01_JPRB, 0.26651E+01_JPRB, 0.26003E+01_JPRB, 0.24408E+01_JPRB, 0.22746E+01_JPRB, 0.20596E+01_JPRB ,&
&  0.17825E+01_JPRB, 0.16456E+01_JPRB, 0.16160E+01_JPRB, 0.15811E+01_JPRB, 0.14458E+01_JPRB, 0.14737E+01_JPRB, 0.15009E+01_JPRB ,&
&  0.15077E+01_JPRB, 0.15024E+01_JPRB, 0.15074E+01_JPRB, 0.15211E+01_JPRB, 0.15033E+01_JPRB, 0.15074E+01_JPRB, 0.15445E+01_JPRB ,&
&  0.16187E+01_JPRB, 0.16690E+01_JPRB, 0.16918E+01_JPRB, 0.16755E+01_JPRB, 0.17301E+01_JPRB, 0.18153E+01_JPRB, 0.18116E+01_JPRB ,&
&  0.17425E+01_JPRB, 0.17408E+01_JPRB, 0.17734E+01_JPRB, 0.18001E+01_JPRB, 0.17424E+01_JPRB, 0.20080E+01_JPRB, 0.21076E+01_JPRB ,&
&  0.23221E+01_JPRB, 0.24350E+01_JPRB, 0.28269E+01_JPRB, 0.22721E+01_JPRB, 0.24875E+01_JPRB, 0.27521E+01_JPRB, 0.28658E+01_JPRB ,&
&  0.29147E+01_JPRB, 0.32526E+01_JPRB, 0.33871E+01_JPRB, 0.35855E+01_JPRB, 0.36884E+01_JPRB, 0.33629E+01_JPRB, 0.36762E+01_JPRB ,&
&  0.36475E+01_JPRB, 0.42298E+01_JPRB, 0.41827E+01_JPRB, 0.40920E+01_JPRB, 0.34753E+01_JPRB, 0.33590E+01_JPRB, 0.35015E+01_JPRB ,&
&  0.38356E+01_JPRB, 0.35391E+01_JPRB, 0.27471E+01_JPRB, 0.19886E+01_JPRB, 0.12891E+01_JPRB, 0.10046E+01_JPRB, 0.93509E+00_JPRB ,&
&  0.92411E+00_JPRB, 0.95324E+00_JPRB, 0.99233E+00_JPRB, 0.10340E+01_JPRB, 0.10596E+01_JPRB, 0.10683E+01_JPRB, 0.10721E+01_JPRB ,&
&  0.10846E+01_JPRB, 0.10931E+01_JPRB, 0.10927E+01_JPRB, 0.10952E+01_JPRB, 0.10970E+01_JPRB /)
ZBURD(:, 30, 11) = (/ &
&  0.10915E+01_JPRB, 0.10913E+01_JPRB, 0.10938E+01_JPRB, 0.10768E+01_JPRB, 0.10527E+01_JPRB, 0.10182E+01_JPRB, 0.95491E+00_JPRB ,&
&  0.90550E+00_JPRB, 0.89429E+00_JPRB, 0.94908E+00_JPRB, 0.10160E+01_JPRB, 0.10834E+01_JPRB, 0.11602E+01_JPRB, 0.12712E+01_JPRB ,&
&  0.14011E+01_JPRB, 0.15033E+01_JPRB, 0.16274E+01_JPRB, 0.18440E+01_JPRB, 0.20969E+01_JPRB, 0.21805E+01_JPRB, 0.22520E+01_JPRB ,&
&  0.20186E+01_JPRB, 0.16195E+01_JPRB, 0.21772E+01_JPRB, 0.24607E+01_JPRB, 0.23551E+01_JPRB, 0.21584E+01_JPRB, 0.19675E+01_JPRB ,&
&  0.17204E+01_JPRB, 0.17241E+01_JPRB, 0.17743E+01_JPRB, 0.17775E+01_JPRB, 0.16031E+01_JPRB, 0.16136E+01_JPRB, 0.16186E+01_JPRB ,&
&  0.16334E+01_JPRB, 0.16294E+01_JPRB, 0.16569E+01_JPRB, 0.16715E+01_JPRB, 0.16601E+01_JPRB, 0.16913E+01_JPRB, 0.17353E+01_JPRB ,&
&  0.17658E+01_JPRB, 0.17893E+01_JPRB, 0.18355E+01_JPRB, 0.18562E+01_JPRB, 0.18797E+01_JPRB, 0.19115E+01_JPRB, 0.18779E+01_JPRB ,&
&  0.18274E+01_JPRB, 0.17698E+01_JPRB, 0.18596E+01_JPRB, 0.20482E+01_JPRB, 0.21993E+01_JPRB, 0.20336E+01_JPRB, 0.22382E+01_JPRB ,&
&  0.23305E+01_JPRB, 0.24405E+01_JPRB, 0.24778E+01_JPRB, 0.28807E+01_JPRB, 0.21588E+01_JPRB, 0.24727E+01_JPRB, 0.24696E+01_JPRB ,&
&  0.27860E+01_JPRB, 0.27659E+01_JPRB, 0.27545E+01_JPRB, 0.28340E+01_JPRB, 0.29593E+01_JPRB, 0.27852E+01_JPRB, 0.31058E+01_JPRB ,&
&  0.34634E+01_JPRB, 0.37679E+01_JPRB, 0.38850E+01_JPRB, 0.40205E+01_JPRB, 0.37727E+01_JPRB, 0.37802E+01_JPRB, 0.37140E+01_JPRB ,&
&  0.38106E+01_JPRB, 0.26738E+01_JPRB, 0.18719E+01_JPRB, 0.13367E+01_JPRB, 0.10160E+01_JPRB, 0.85495E+00_JPRB, 0.88977E+00_JPRB ,&
&  0.92692E+00_JPRB, 0.96725E+00_JPRB, 0.99299E+00_JPRB, 0.10179E+01_JPRB, 0.10350E+01_JPRB, 0.10464E+01_JPRB, 0.10682E+01_JPRB ,&
&  0.10796E+01_JPRB, 0.10856E+01_JPRB, 0.10858E+01_JPRB, 0.10861E+01_JPRB, 0.10881E+01_JPRB /)
ZBURD(:, 31, 11) = (/ &
&  0.10534E+01_JPRB, 0.10328E+01_JPRB, 0.99283E+00_JPRB, 0.93092E+00_JPRB, 0.86460E+00_JPRB, 0.80152E+00_JPRB, 0.76581E+00_JPRB ,&
&  0.74094E+00_JPRB, 0.77379E+00_JPRB, 0.84035E+00_JPRB, 0.92626E+00_JPRB, 0.10236E+01_JPRB, 0.11209E+01_JPRB, 0.12095E+01_JPRB ,&
&  0.12638E+01_JPRB, 0.13083E+01_JPRB, 0.14073E+01_JPRB, 0.16369E+01_JPRB, 0.18968E+01_JPRB, 0.20945E+01_JPRB, 0.22110E+01_JPRB ,&
&  0.23081E+01_JPRB, 0.23459E+01_JPRB, 0.22359E+01_JPRB, 0.17844E+01_JPRB, 0.16081E+01_JPRB, 0.19285E+01_JPRB, 0.17447E+01_JPRB ,&
&  0.17268E+01_JPRB, 0.17019E+01_JPRB, 0.17177E+01_JPRB, 0.17378E+01_JPRB, 0.17048E+01_JPRB, 0.16735E+01_JPRB, 0.16872E+01_JPRB ,&
&  0.17000E+01_JPRB, 0.17311E+01_JPRB, 0.17613E+01_JPRB, 0.17736E+01_JPRB, 0.18069E+01_JPRB, 0.18587E+01_JPRB, 0.18984E+01_JPRB ,&
&  0.19143E+01_JPRB, 0.19539E+01_JPRB, 0.19885E+01_JPRB, 0.20437E+01_JPRB, 0.20098E+01_JPRB, 0.20219E+01_JPRB, 0.19978E+01_JPRB ,&
&  0.19749E+01_JPRB, 0.19546E+01_JPRB, 0.19885E+01_JPRB, 0.21238E+01_JPRB, 0.21914E+01_JPRB, 0.19939E+01_JPRB, 0.20560E+01_JPRB ,&
&  0.22726E+01_JPRB, 0.23871E+01_JPRB, 0.21948E+01_JPRB, 0.27996E+01_JPRB, 0.18473E+01_JPRB, 0.22932E+01_JPRB, 0.26168E+01_JPRB ,&
&  0.25110E+01_JPRB, 0.23406E+01_JPRB, 0.23150E+01_JPRB, 0.23688E+01_JPRB, 0.23873E+01_JPRB, 0.23925E+01_JPRB, 0.28099E+01_JPRB ,&
&  0.31718E+01_JPRB, 0.32965E+01_JPRB, 0.34998E+01_JPRB, 0.35745E+01_JPRB, 0.35991E+01_JPRB, 0.33638E+01_JPRB, 0.30101E+01_JPRB ,&
&  0.24187E+01_JPRB, 0.19859E+01_JPRB, 0.15633E+01_JPRB, 0.13697E+01_JPRB, 0.10790E+01_JPRB, 0.83877E+00_JPRB, 0.88932E+00_JPRB ,&
&  0.92430E+00_JPRB, 0.94530E+00_JPRB, 0.97881E+00_JPRB, 0.10031E+01_JPRB, 0.10079E+01_JPRB, 0.10201E+01_JPRB, 0.10426E+01_JPRB ,&
&  0.10457E+01_JPRB, 0.10455E+01_JPRB, 0.10484E+01_JPRB, 0.10488E+01_JPRB, 0.10546E+01_JPRB /)
ZBURD(:, 32, 11) = (/ &
&  0.87591E+00_JPRB, 0.84916E+00_JPRB, 0.81060E+00_JPRB, 0.77204E+00_JPRB, 0.73498E+00_JPRB, 0.69288E+00_JPRB, 0.68976E+00_JPRB ,&
&  0.67683E+00_JPRB, 0.68999E+00_JPRB, 0.72312E+00_JPRB, 0.75641E+00_JPRB, 0.80682E+00_JPRB, 0.88043E+00_JPRB, 0.94777E+00_JPRB ,&
&  0.98827E+00_JPRB, 0.10015E+01_JPRB, 0.10678E+01_JPRB, 0.11727E+01_JPRB, 0.13649E+01_JPRB, 0.16987E+01_JPRB, 0.22189E+01_JPRB ,&
&  0.24890E+01_JPRB, 0.26384E+01_JPRB, 0.25746E+01_JPRB, 0.23438E+01_JPRB, 0.20224E+01_JPRB, 0.19672E+01_JPRB, 0.21055E+01_JPRB ,&
&  0.21998E+01_JPRB, 0.20350E+01_JPRB, 0.17938E+01_JPRB, 0.17532E+01_JPRB, 0.17317E+01_JPRB, 0.17178E+01_JPRB, 0.17320E+01_JPRB ,&
&  0.17640E+01_JPRB, 0.17879E+01_JPRB, 0.18046E+01_JPRB, 0.18455E+01_JPRB, 0.18779E+01_JPRB, 0.19112E+01_JPRB, 0.19607E+01_JPRB ,&
&  0.20045E+01_JPRB, 0.20238E+01_JPRB, 0.20391E+01_JPRB, 0.20389E+01_JPRB, 0.20912E+01_JPRB, 0.21575E+01_JPRB, 0.21896E+01_JPRB ,&
&  0.21979E+01_JPRB, 0.21187E+01_JPRB, 0.21958E+01_JPRB, 0.23170E+01_JPRB, 0.22817E+01_JPRB, 0.21497E+01_JPRB, 0.21603E+01_JPRB ,&
&  0.22994E+01_JPRB, 0.24037E+01_JPRB, 0.18148E+01_JPRB, 0.23614E+01_JPRB, 0.28035E+01_JPRB, 0.25537E+01_JPRB, 0.23282E+01_JPRB ,&
&  0.21264E+01_JPRB, 0.19806E+01_JPRB, 0.20255E+01_JPRB, 0.20306E+01_JPRB, 0.21940E+01_JPRB, 0.22440E+01_JPRB, 0.20667E+01_JPRB ,&
&  0.24142E+01_JPRB, 0.24937E+01_JPRB, 0.27556E+01_JPRB, 0.28955E+01_JPRB, 0.28210E+01_JPRB, 0.25224E+01_JPRB, 0.20936E+01_JPRB ,&
&  0.15783E+01_JPRB, 0.15922E+01_JPRB, 0.13833E+01_JPRB, 0.14426E+01_JPRB, 0.13059E+01_JPRB, 0.82167E+00_JPRB, 0.86841E+00_JPRB ,&
&  0.88674E+00_JPRB, 0.90652E+00_JPRB, 0.93016E+00_JPRB, 0.94146E+00_JPRB, 0.95283E+00_JPRB, 0.96188E+00_JPRB, 0.97523E+00_JPRB ,&
&  0.96938E+00_JPRB, 0.95354E+00_JPRB, 0.93868E+00_JPRB, 0.92722E+00_JPRB, 0.90695E+00_JPRB /)
ZBURD(:, 33, 11) = (/ &
&  0.77462E+00_JPRB, 0.76498E+00_JPRB, 0.75469E+00_JPRB, 0.75081E+00_JPRB, 0.75510E+00_JPRB, 0.75113E+00_JPRB, 0.73826E+00_JPRB ,&
&  0.71738E+00_JPRB, 0.70854E+00_JPRB, 0.68687E+00_JPRB, 0.66353E+00_JPRB, 0.66678E+00_JPRB, 0.71824E+00_JPRB, 0.77054E+00_JPRB ,&
&  0.80224E+00_JPRB, 0.81596E+00_JPRB, 0.81562E+00_JPRB, 0.81426E+00_JPRB, 0.85393E+00_JPRB, 0.89885E+00_JPRB, 0.11505E+01_JPRB ,&
&  0.15790E+01_JPRB, 0.21557E+01_JPRB, 0.25573E+01_JPRB, 0.27365E+01_JPRB, 0.25858E+01_JPRB, 0.21730E+01_JPRB, 0.24618E+01_JPRB ,&
&  0.22132E+01_JPRB, 0.17520E+01_JPRB, 0.16113E+01_JPRB, 0.15716E+01_JPRB, 0.15759E+01_JPRB, 0.15767E+01_JPRB, 0.16019E+01_JPRB ,&
&  0.16442E+01_JPRB, 0.16898E+01_JPRB, 0.17333E+01_JPRB, 0.17863E+01_JPRB, 0.18418E+01_JPRB, 0.19048E+01_JPRB, 0.19718E+01_JPRB ,&
&  0.20386E+01_JPRB, 0.20616E+01_JPRB, 0.21335E+01_JPRB, 0.20102E+01_JPRB, 0.21155E+01_JPRB, 0.22249E+01_JPRB, 0.23077E+01_JPRB ,&
&  0.23209E+01_JPRB, 0.22865E+01_JPRB, 0.24035E+01_JPRB, 0.24372E+01_JPRB, 0.24486E+01_JPRB, 0.24223E+01_JPRB, 0.23642E+01_JPRB ,&
&  0.24091E+01_JPRB, 0.22513E+01_JPRB, 0.17425E+01_JPRB, 0.19504E+01_JPRB, 0.22208E+01_JPRB, 0.18650E+01_JPRB, 0.20883E+01_JPRB ,&
&  0.18404E+01_JPRB, 0.16578E+01_JPRB, 0.15236E+01_JPRB, 0.15107E+01_JPRB, 0.16766E+01_JPRB, 0.18742E+01_JPRB, 0.17172E+01_JPRB ,&
&  0.16928E+01_JPRB, 0.17065E+01_JPRB, 0.18899E+01_JPRB, 0.20011E+01_JPRB, 0.19580E+01_JPRB, 0.17905E+01_JPRB, 0.15660E+01_JPRB ,&
&  0.13584E+01_JPRB, 0.12092E+01_JPRB, 0.10359E+01_JPRB, 0.10129E+01_JPRB, 0.81699E+00_JPRB, 0.76714E+00_JPRB, 0.79377E+00_JPRB ,&
&  0.80526E+00_JPRB, 0.79795E+00_JPRB, 0.80555E+00_JPRB, 0.81177E+00_JPRB, 0.82607E+00_JPRB, 0.82222E+00_JPRB, 0.83524E+00_JPRB ,&
&  0.83926E+00_JPRB, 0.83278E+00_JPRB, 0.82416E+00_JPRB, 0.82153E+00_JPRB, 0.79869E+00_JPRB /)
ZBURD(:, 34, 11) = (/ &
&  0.84557E+00_JPRB, 0.85011E+00_JPRB, 0.86288E+00_JPRB, 0.87455E+00_JPRB, 0.89465E+00_JPRB, 0.88303E+00_JPRB, 0.86901E+00_JPRB ,&
&  0.83544E+00_JPRB, 0.82824E+00_JPRB, 0.82032E+00_JPRB, 0.81708E+00_JPRB, 0.82388E+00_JPRB, 0.84800E+00_JPRB, 0.89430E+00_JPRB ,&
&  0.90801E+00_JPRB, 0.91202E+00_JPRB, 0.90351E+00_JPRB, 0.89129E+00_JPRB, 0.85582E+00_JPRB, 0.82419E+00_JPRB, 0.80232E+00_JPRB ,&
&  0.83382E+00_JPRB, 0.10569E+01_JPRB, 0.13348E+01_JPRB, 0.15859E+01_JPRB, 0.18917E+01_JPRB, 0.17814E+01_JPRB, 0.20578E+01_JPRB ,&
&  0.22045E+01_JPRB, 0.15770E+01_JPRB, 0.15608E+01_JPRB, 0.13929E+01_JPRB, 0.13939E+01_JPRB, 0.13855E+01_JPRB, 0.14138E+01_JPRB ,&
&  0.14728E+01_JPRB, 0.15058E+01_JPRB, 0.15211E+01_JPRB, 0.15458E+01_JPRB, 0.15911E+01_JPRB, 0.16413E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.16784E+01_JPRB, 0.16093E+01_JPRB, 0.16085E+01_JPRB, 0.18039E+01_JPRB, 0.19868E+01_JPRB, 0.20986E+01_JPRB, 0.21919E+01_JPRB ,&
&  0.23170E+01_JPRB, 0.23981E+01_JPRB, 0.22537E+01_JPRB, 0.22486E+01_JPRB, 0.23676E+01_JPRB, 0.22239E+01_JPRB, 0.22783E+01_JPRB ,&
&  0.23378E+01_JPRB, 0.20805E+01_JPRB, 0.15257E+01_JPRB, 0.18041E+01_JPRB, 0.19801E+01_JPRB, 0.18922E+01_JPRB, 0.19356E+01_JPRB ,&
&  0.16501E+01_JPRB, 0.14498E+01_JPRB, 0.12288E+01_JPRB, 0.11642E+01_JPRB, 0.12095E+01_JPRB, 0.13597E+01_JPRB, 0.13919E+01_JPRB ,&
&  0.12919E+01_JPRB, 0.12512E+01_JPRB, 0.14085E+01_JPRB, 0.15424E+01_JPRB, 0.15715E+01_JPRB, 0.14695E+01_JPRB, 0.12681E+01_JPRB ,&
&  0.12131E+01_JPRB, 0.11294E+01_JPRB, 0.12644E+01_JPRB, 0.12625E+01_JPRB, 0.81330E+00_JPRB, 0.77463E+00_JPRB, 0.76809E+00_JPRB ,&
&  0.78055E+00_JPRB, 0.77647E+00_JPRB, 0.76703E+00_JPRB, 0.77463E+00_JPRB, 0.77500E+00_JPRB, 0.77173E+00_JPRB, 0.79708E+00_JPRB ,&
&  0.82899E+00_JPRB, 0.82868E+00_JPRB, 0.82980E+00_JPRB, 0.83051E+00_JPRB, 0.83637E+00_JPRB /)
ZBURD(:, 35, 11) = (/ &
&  0.94057E+00_JPRB, 0.95748E+00_JPRB, 0.98019E+00_JPRB, 0.98318E+00_JPRB, 0.98888E+00_JPRB, 0.98136E+00_JPRB, 0.97576E+00_JPRB ,&
&  0.96026E+00_JPRB, 0.95510E+00_JPRB, 0.95669E+00_JPRB, 0.96134E+00_JPRB, 0.97236E+00_JPRB, 0.98516E+00_JPRB, 0.10164E+01_JPRB ,&
&  0.10176E+01_JPRB, 0.10239E+01_JPRB, 0.10325E+01_JPRB, 0.10493E+01_JPRB, 0.10651E+01_JPRB, 0.10627E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.10170E+01_JPRB, 0.10166E+01_JPRB, 0.10323E+01_JPRB, 0.10624E+01_JPRB, 0.12323E+01_JPRB, 0.15479E+01_JPRB, 0.20521E+01_JPRB ,&
&  0.19622E+01_JPRB, 0.15378E+01_JPRB, 0.13517E+01_JPRB, 0.11709E+01_JPRB, 0.11621E+01_JPRB, 0.11901E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.11235E+01_JPRB, 0.10859E+01_JPRB, 0.10801E+01_JPRB, 0.11026E+01_JPRB, 0.11247E+01_JPRB, 0.11096E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.10027E+01_JPRB, 0.96874E+00_JPRB, 0.10087E+01_JPRB, 0.11318E+01_JPRB, 0.14001E+01_JPRB, 0.15808E+01_JPRB, 0.15162E+01_JPRB ,&
&  0.16638E+01_JPRB, 0.18885E+01_JPRB, 0.19564E+01_JPRB, 0.19650E+01_JPRB, 0.19282E+01_JPRB, 0.18559E+01_JPRB, 0.18053E+01_JPRB ,&
&  0.17597E+01_JPRB, 0.16194E+01_JPRB, 0.14376E+01_JPRB, 0.16780E+01_JPRB, 0.17947E+01_JPRB, 0.18720E+01_JPRB, 0.16880E+01_JPRB ,&
&  0.14472E+01_JPRB, 0.12890E+01_JPRB, 0.11789E+01_JPRB, 0.10784E+01_JPRB, 0.10338E+01_JPRB, 0.10952E+01_JPRB, 0.11059E+01_JPRB ,&
&  0.10577E+01_JPRB, 0.10874E+01_JPRB, 0.11325E+01_JPRB, 0.11893E+01_JPRB, 0.13976E+01_JPRB, 0.12354E+01_JPRB, 0.13564E+01_JPRB ,&
&  0.11602E+01_JPRB, 0.11035E+01_JPRB, 0.90095E+00_JPRB, 0.12859E+01_JPRB, 0.82884E+00_JPRB, 0.72244E+00_JPRB, 0.74781E+00_JPRB ,&
&  0.78807E+00_JPRB, 0.80742E+00_JPRB, 0.83500E+00_JPRB, 0.85986E+00_JPRB, 0.86076E+00_JPRB, 0.86050E+00_JPRB, 0.87915E+00_JPRB ,&
&  0.89904E+00_JPRB, 0.90810E+00_JPRB, 0.91729E+00_JPRB, 0.91830E+00_JPRB, 0.92894E+00_JPRB /)
ZBURD(:, 36, 11) = (/ &
&  0.99929E+00_JPRB, 0.10148E+01_JPRB, 0.10279E+01_JPRB, 0.10375E+01_JPRB, 0.10412E+01_JPRB, 0.10434E+01_JPRB, 0.10464E+01_JPRB ,&
&  0.10456E+01_JPRB, 0.10417E+01_JPRB, 0.10396E+01_JPRB, 0.10547E+01_JPRB, 0.10765E+01_JPRB, 0.10875E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.11076E+01_JPRB, 0.11216E+01_JPRB, 0.11434E+01_JPRB, 0.11602E+01_JPRB, 0.11869E+01_JPRB, 0.11863E+01_JPRB, 0.11680E+01_JPRB ,&
&  0.11611E+01_JPRB, 0.11500E+01_JPRB, 0.11517E+01_JPRB, 0.11543E+01_JPRB, 0.11657E+01_JPRB, 0.11841E+01_JPRB, 0.13626E+01_JPRB ,&
&  0.13893E+01_JPRB, 0.15847E+01_JPRB, 0.14187E+01_JPRB, 0.12526E+01_JPRB, 0.12365E+01_JPRB, 0.11375E+01_JPRB, 0.10853E+01_JPRB ,&
&  0.10211E+01_JPRB, 0.95432E+00_JPRB, 0.91531E+00_JPRB, 0.89538E+00_JPRB, 0.92638E+00_JPRB, 0.96866E+00_JPRB, 0.98658E+00_JPRB ,&
&  0.10427E+01_JPRB, 0.10774E+01_JPRB, 0.11154E+01_JPRB, 0.11459E+01_JPRB, 0.11682E+01_JPRB, 0.12420E+01_JPRB, 0.11871E+01_JPRB ,&
&  0.12039E+01_JPRB, 0.13161E+01_JPRB, 0.13277E+01_JPRB, 0.13621E+01_JPRB, 0.13003E+01_JPRB, 0.12222E+01_JPRB, 0.11030E+01_JPRB ,&
&  0.96309E+00_JPRB, 0.95847E+00_JPRB, 0.11500E+01_JPRB, 0.13726E+01_JPRB, 0.15293E+01_JPRB, 0.16083E+01_JPRB, 0.14697E+01_JPRB ,&
&  0.13246E+01_JPRB, 0.12234E+01_JPRB, 0.11177E+01_JPRB, 0.10201E+01_JPRB, 0.97622E+00_JPRB, 0.97747E+00_JPRB, 0.96442E+00_JPRB ,&
&  0.93973E+00_JPRB, 0.93742E+00_JPRB, 0.95806E+00_JPRB, 0.10057E+01_JPRB, 0.10463E+01_JPRB, 0.15988E+01_JPRB, 0.19785E+01_JPRB ,&
&  0.12467E+01_JPRB, 0.98330E+00_JPRB, 0.66555E+00_JPRB, 0.94592E+00_JPRB, 0.72510E+00_JPRB, 0.69840E+00_JPRB, 0.76709E+00_JPRB ,&
&  0.86544E+00_JPRB, 0.94103E+00_JPRB, 0.97367E+00_JPRB, 0.10107E+01_JPRB, 0.91902E+00_JPRB, 0.90533E+00_JPRB, 0.90489E+00_JPRB ,&
&  0.91689E+00_JPRB, 0.93491E+00_JPRB, 0.95150E+00_JPRB, 0.96558E+00_JPRB, 0.98301E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART21',1,ZHOOK_HANDLE)
END SUBROUTINE PART21



SUBROUTINE PART22

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART22',0,ZHOOK_HANDLE)


ZBURD(:, 37, 11) = (/ &
&  0.10214E+01_JPRB, 0.10332E+01_JPRB, 0.10451E+01_JPRB, 0.10610E+01_JPRB, 0.10751E+01_JPRB, 0.10932E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.11149E+01_JPRB, 0.11217E+01_JPRB, 0.11223E+01_JPRB, 0.11358E+01_JPRB, 0.11493E+01_JPRB, 0.11750E+01_JPRB, 0.12050E+01_JPRB ,&
&  0.12384E+01_JPRB, 0.12736E+01_JPRB, 0.13140E+01_JPRB, 0.13385E+01_JPRB, 0.13582E+01_JPRB, 0.13784E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.14030E+01_JPRB, 0.14022E+01_JPRB, 0.14184E+01_JPRB, 0.14506E+01_JPRB, 0.14875E+01_JPRB, 0.15476E+01_JPRB, 0.13594E+01_JPRB ,&
&  0.15334E+01_JPRB, 0.16096E+01_JPRB, 0.15436E+01_JPRB, 0.14015E+01_JPRB, 0.13453E+01_JPRB, 0.13027E+01_JPRB, 0.12906E+01_JPRB ,&
&  0.13124E+01_JPRB, 0.12720E+01_JPRB, 0.12306E+01_JPRB, 0.11580E+01_JPRB, 0.11208E+01_JPRB, 0.11398E+01_JPRB, 0.11821E+01_JPRB ,&
&  0.12071E+01_JPRB, 0.12340E+01_JPRB, 0.12376E+01_JPRB, 0.12255E+01_JPRB, 0.12189E+01_JPRB, 0.11950E+01_JPRB, 0.11317E+01_JPRB ,&
&  0.10730E+01_JPRB, 0.10561E+01_JPRB, 0.92269E+00_JPRB, 0.90267E+00_JPRB, 0.89380E+00_JPRB, 0.82139E+00_JPRB, 0.70960E+00_JPRB ,&
&  0.64701E+00_JPRB, 0.64784E+00_JPRB, 0.75850E+00_JPRB, 0.11300E+01_JPRB, 0.13145E+01_JPRB, 0.13462E+01_JPRB, 0.12836E+01_JPRB ,&
&  0.11758E+01_JPRB, 0.10742E+01_JPRB, 0.95379E+00_JPRB, 0.88235E+00_JPRB, 0.86310E+00_JPRB, 0.89348E+00_JPRB, 0.90899E+00_JPRB ,&
&  0.90653E+00_JPRB, 0.89363E+00_JPRB, 0.89300E+00_JPRB, 0.90202E+00_JPRB, 0.10083E+01_JPRB, 0.11958E+01_JPRB, 0.17564E+01_JPRB ,&
&  0.12738E+01_JPRB, 0.76948E+00_JPRB, 0.74111E+00_JPRB, 0.80833E+00_JPRB, 0.67152E+00_JPRB, 0.65126E+00_JPRB, 0.75480E+00_JPRB ,&
&  0.96407E+00_JPRB, 0.10902E+01_JPRB, 0.10831E+01_JPRB, 0.10616E+01_JPRB, 0.10108E+01_JPRB, 0.10135E+01_JPRB, 0.99552E+00_JPRB ,&
&  0.98368E+00_JPRB, 0.98626E+00_JPRB, 0.99162E+00_JPRB, 0.99729E+00_JPRB, 0.10084E+01_JPRB /)
ZBURD(:, 38, 11) = (/ &
&  0.10258E+01_JPRB, 0.10414E+01_JPRB, 0.10565E+01_JPRB, 0.10722E+01_JPRB, 0.10842E+01_JPRB, 0.11048E+01_JPRB, 0.11128E+01_JPRB ,&
&  0.11251E+01_JPRB, 0.11317E+01_JPRB, 0.11483E+01_JPRB, 0.11692E+01_JPRB, 0.11888E+01_JPRB, 0.12159E+01_JPRB, 0.12492E+01_JPRB ,&
&  0.12902E+01_JPRB, 0.13203E+01_JPRB, 0.13561E+01_JPRB, 0.13763E+01_JPRB, 0.13723E+01_JPRB, 0.13826E+01_JPRB, 0.13894E+01_JPRB ,&
&  0.13779E+01_JPRB, 0.13664E+01_JPRB, 0.13799E+01_JPRB, 0.14009E+01_JPRB, 0.13975E+01_JPRB, 0.14263E+01_JPRB, 0.11391E+01_JPRB ,&
&  0.15548E+01_JPRB, 0.16987E+01_JPRB, 0.17176E+01_JPRB, 0.15769E+01_JPRB, 0.14869E+01_JPRB, 0.14810E+01_JPRB, 0.15057E+01_JPRB ,&
&  0.15603E+01_JPRB, 0.16168E+01_JPRB, 0.15620E+01_JPRB, 0.14672E+01_JPRB, 0.14021E+01_JPRB, 0.13981E+01_JPRB, 0.14059E+01_JPRB ,&
&  0.14179E+01_JPRB, 0.14300E+01_JPRB, 0.14224E+01_JPRB, 0.14004E+01_JPRB, 0.13727E+01_JPRB, 0.13403E+01_JPRB, 0.12835E+01_JPRB ,&
&  0.12313E+01_JPRB, 0.11918E+01_JPRB, 0.10638E+01_JPRB, 0.10971E+01_JPRB, 0.10243E+01_JPRB, 0.87608E+00_JPRB, 0.73228E+00_JPRB ,&
&  0.65167E+00_JPRB, 0.68570E+00_JPRB, 0.86487E+00_JPRB, 0.10815E+01_JPRB, 0.11217E+01_JPRB, 0.11079E+01_JPRB, 0.10689E+01_JPRB ,&
&  0.97382E+00_JPRB, 0.89096E+00_JPRB, 0.83402E+00_JPRB, 0.81458E+00_JPRB, 0.82061E+00_JPRB, 0.84800E+00_JPRB, 0.86703E+00_JPRB ,&
&  0.87000E+00_JPRB, 0.87157E+00_JPRB, 0.86085E+00_JPRB, 0.85097E+00_JPRB, 0.88437E+00_JPRB, 0.94232E+00_JPRB, 0.14322E+01_JPRB ,&
&  0.11669E+01_JPRB, 0.85495E+00_JPRB, 0.10746E+01_JPRB, 0.52676E+00_JPRB, 0.60473E+00_JPRB, 0.58869E+00_JPRB, 0.68063E+00_JPRB ,&
&  0.81643E+00_JPRB, 0.11460E+01_JPRB, 0.15630E+01_JPRB, 0.13730E+01_JPRB, 0.12485E+01_JPRB, 0.11419E+01_JPRB, 0.10513E+01_JPRB ,&
&  0.10163E+01_JPRB, 0.10058E+01_JPRB, 0.10040E+01_JPRB, 0.10008E+01_JPRB, 0.10100E+01_JPRB /)
ZBURD(:, 39, 11) = (/ &
&  0.10497E+01_JPRB, 0.10692E+01_JPRB, 0.10721E+01_JPRB, 0.10901E+01_JPRB, 0.10720E+01_JPRB, 0.10911E+01_JPRB, 0.10933E+01_JPRB ,&
&  0.11135E+01_JPRB, 0.11264E+01_JPRB, 0.11479E+01_JPRB, 0.11673E+01_JPRB, 0.11971E+01_JPRB, 0.12239E+01_JPRB, 0.12474E+01_JPRB ,&
&  0.12794E+01_JPRB, 0.13140E+01_JPRB, 0.13438E+01_JPRB, 0.13733E+01_JPRB, 0.13669E+01_JPRB, 0.13841E+01_JPRB, 0.13949E+01_JPRB ,&
&  0.14289E+01_JPRB, 0.14197E+01_JPRB, 0.14299E+01_JPRB, 0.14379E+01_JPRB, 0.14219E+01_JPRB, 0.14645E+01_JPRB, 0.10725E+01_JPRB ,&
&  0.15955E+01_JPRB, 0.17701E+01_JPRB, 0.18440E+01_JPRB, 0.17404E+01_JPRB, 0.16266E+01_JPRB, 0.15943E+01_JPRB, 0.15983E+01_JPRB ,&
&  0.16393E+01_JPRB, 0.16647E+01_JPRB, 0.16021E+01_JPRB, 0.16444E+01_JPRB, 0.16602E+01_JPRB, 0.16159E+01_JPRB, 0.16426E+01_JPRB ,&
&  0.16607E+01_JPRB, 0.16784E+01_JPRB, 0.16945E+01_JPRB, 0.17022E+01_JPRB, 0.16930E+01_JPRB, 0.17193E+01_JPRB, 0.17207E+01_JPRB ,&
&  0.16930E+01_JPRB, 0.16865E+01_JPRB, 0.16821E+01_JPRB, 0.15813E+01_JPRB, 0.14719E+01_JPRB, 0.12485E+01_JPRB, 0.10595E+01_JPRB ,&
&  0.83212E+00_JPRB, 0.81820E+00_JPRB, 0.91884E+00_JPRB, 0.10596E+01_JPRB, 0.10486E+01_JPRB, 0.99938E+00_JPRB, 0.93064E+00_JPRB ,&
&  0.84721E+00_JPRB, 0.82794E+00_JPRB, 0.80725E+00_JPRB, 0.80947E+00_JPRB, 0.82325E+00_JPRB, 0.84291E+00_JPRB, 0.85789E+00_JPRB ,&
&  0.86482E+00_JPRB, 0.87050E+00_JPRB, 0.84664E+00_JPRB, 0.82279E+00_JPRB, 0.84696E+00_JPRB, 0.96033E+00_JPRB, 0.98330E+00_JPRB ,&
&  0.11409E+01_JPRB, 0.90630E+00_JPRB, 0.82069E+00_JPRB, 0.75487E+00_JPRB, 0.62893E+00_JPRB, 0.62814E+00_JPRB, 0.79109E+00_JPRB ,&
&  0.12874E+01_JPRB, 0.16568E+01_JPRB, 0.23618E+01_JPRB, 0.39903E+01_JPRB, 0.28239E+01_JPRB, 0.21007E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.10180E+01_JPRB, 0.10079E+01_JPRB, 0.10006E+01_JPRB, 0.10226E+01_JPRB /)
ZBURD(:, 40, 11) = (/ &
&  0.97240E+00_JPRB, 0.99952E+00_JPRB, 0.10288E+01_JPRB, 0.10555E+01_JPRB, 0.10664E+01_JPRB, 0.10703E+01_JPRB, 0.10708E+01_JPRB ,&
&  0.10971E+01_JPRB, 0.10790E+01_JPRB, 0.11050E+01_JPRB, 0.10850E+01_JPRB, 0.11085E+01_JPRB, 0.10976E+01_JPRB, 0.11094E+01_JPRB ,&
&  0.11311E+01_JPRB, 0.11677E+01_JPRB, 0.12113E+01_JPRB, 0.12369E+01_JPRB, 0.12524E+01_JPRB, 0.13032E+01_JPRB, 0.13729E+01_JPRB ,&
&  0.14529E+01_JPRB, 0.14944E+01_JPRB, 0.15404E+01_JPRB, 0.15636E+01_JPRB, 0.15315E+01_JPRB, 0.14986E+01_JPRB, 0.10755E+01_JPRB ,&
&  0.15218E+01_JPRB, 0.17799E+01_JPRB, 0.19619E+01_JPRB, 0.19290E+01_JPRB, 0.17890E+01_JPRB, 0.16776E+01_JPRB, 0.16728E+01_JPRB ,&
&  0.16717E+01_JPRB, 0.16274E+01_JPRB, 0.15640E+01_JPRB, 0.15579E+01_JPRB, 0.17281E+01_JPRB, 0.16765E+01_JPRB, 0.17053E+01_JPRB ,&
&  0.17322E+01_JPRB, 0.17678E+01_JPRB, 0.18162E+01_JPRB, 0.18796E+01_JPRB, 0.19348E+01_JPRB, 0.19935E+01_JPRB, 0.20639E+01_JPRB ,&
&  0.21064E+01_JPRB, 0.21102E+01_JPRB, 0.21248E+01_JPRB, 0.18998E+01_JPRB, 0.17453E+01_JPRB, 0.17120E+01_JPRB, 0.13670E+01_JPRB ,&
&  0.96910E+00_JPRB, 0.90937E+00_JPRB, 0.10012E+01_JPRB, 0.10794E+01_JPRB, 0.10559E+01_JPRB, 0.95443E+00_JPRB, 0.89417E+00_JPRB ,&
&  0.85464E+00_JPRB, 0.83448E+00_JPRB, 0.81874E+00_JPRB, 0.84093E+00_JPRB, 0.85374E+00_JPRB, 0.87370E+00_JPRB, 0.88433E+00_JPRB ,&
&  0.89984E+00_JPRB, 0.88151E+00_JPRB, 0.85339E+00_JPRB, 0.84324E+00_JPRB, 0.88228E+00_JPRB, 0.10265E+01_JPRB, 0.12586E+01_JPRB ,&
&  0.13624E+01_JPRB, 0.83871E+00_JPRB, 0.83312E+00_JPRB, 0.76082E+00_JPRB, 0.73310E+00_JPRB, 0.77553E+00_JPRB, 0.94645E+00_JPRB ,&
&  0.14072E+01_JPRB, 0.21478E+01_JPRB, 0.23997E+01_JPRB, 0.20560E+01_JPRB, 0.25159E+01_JPRB, 0.18138E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.10076E+01_JPRB, 0.97873E+00_JPRB, 0.94224E+00_JPRB, 0.93247E+00_JPRB, 0.94299E+00_JPRB /)
ZBURD(:, 41, 11) = (/ &
&  0.81500E+00_JPRB, 0.83281E+00_JPRB, 0.85638E+00_JPRB, 0.89301E+00_JPRB, 0.93404E+00_JPRB, 0.96278E+00_JPRB, 0.99185E+00_JPRB ,&
&  0.10027E+01_JPRB, 0.10199E+01_JPRB, 0.10353E+01_JPRB, 0.99762E+00_JPRB, 0.10118E+01_JPRB, 0.99992E+00_JPRB, 0.99845E+00_JPRB ,&
&  0.10014E+01_JPRB, 0.10255E+01_JPRB, 0.10565E+01_JPRB, 0.10904E+01_JPRB, 0.11395E+01_JPRB, 0.12004E+01_JPRB, 0.12808E+01_JPRB ,&
&  0.13794E+01_JPRB, 0.14651E+01_JPRB, 0.15575E+01_JPRB, 0.16117E+01_JPRB, 0.16565E+01_JPRB, 0.16749E+01_JPRB, 0.16530E+01_JPRB ,&
&  0.98009E+00_JPRB, 0.16170E+01_JPRB, 0.17984E+01_JPRB, 0.19079E+01_JPRB, 0.19657E+01_JPRB, 0.18468E+01_JPRB, 0.18266E+01_JPRB ,&
&  0.17178E+01_JPRB, 0.15627E+01_JPRB, 0.15252E+01_JPRB, 0.15313E+01_JPRB, 0.16131E+01_JPRB, 0.15935E+01_JPRB, 0.15961E+01_JPRB ,&
&  0.16064E+01_JPRB, 0.16480E+01_JPRB, 0.17035E+01_JPRB, 0.17681E+01_JPRB, 0.18539E+01_JPRB, 0.19550E+01_JPRB, 0.20979E+01_JPRB ,&
&  0.22366E+01_JPRB, 0.23621E+01_JPRB, 0.24627E+01_JPRB, 0.22072E+01_JPRB, 0.19531E+01_JPRB, 0.19493E+01_JPRB, 0.16871E+01_JPRB ,&
&  0.11667E+01_JPRB, 0.96616E+00_JPRB, 0.10455E+01_JPRB, 0.11731E+01_JPRB, 0.11203E+01_JPRB, 0.10321E+01_JPRB, 0.95913E+00_JPRB ,&
&  0.92484E+00_JPRB, 0.88782E+00_JPRB, 0.91261E+00_JPRB, 0.88978E+00_JPRB, 0.92440E+00_JPRB, 0.92735E+00_JPRB, 0.94167E+00_JPRB ,&
&  0.95896E+00_JPRB, 0.94260E+00_JPRB, 0.93235E+00_JPRB, 0.93157E+00_JPRB, 0.92218E+00_JPRB, 0.91277E+00_JPRB, 0.91206E+00_JPRB ,&
&  0.92532E+00_JPRB, 0.88400E+00_JPRB, 0.85204E+00_JPRB, 0.85751E+00_JPRB, 0.86680E+00_JPRB, 0.88374E+00_JPRB, 0.96972E+00_JPRB ,&
&  0.11217E+01_JPRB, 0.12916E+01_JPRB, 0.15766E+01_JPRB, 0.16562E+01_JPRB, 0.15895E+01_JPRB, 0.12054E+01_JPRB, 0.89524E+00_JPRB ,&
&  0.88989E+00_JPRB, 0.84659E+00_JPRB, 0.81288E+00_JPRB, 0.79445E+00_JPRB, 0.79943E+00_JPRB /)
ZBURD(:, 42, 11) = (/ &
&  0.74038E+00_JPRB, 0.75323E+00_JPRB, 0.74673E+00_JPRB, 0.77135E+00_JPRB, 0.79162E+00_JPRB, 0.82162E+00_JPRB, 0.85220E+00_JPRB ,&
&  0.87803E+00_JPRB, 0.90162E+00_JPRB, 0.93399E+00_JPRB, 0.91618E+00_JPRB, 0.92979E+00_JPRB, 0.92382E+00_JPRB, 0.90455E+00_JPRB ,&
&  0.90453E+00_JPRB, 0.92229E+00_JPRB, 0.94285E+00_JPRB, 0.96566E+00_JPRB, 0.99459E+00_JPRB, 0.10442E+01_JPRB, 0.11055E+01_JPRB ,&
&  0.11825E+01_JPRB, 0.12772E+01_JPRB, 0.14007E+01_JPRB, 0.15127E+01_JPRB, 0.16179E+01_JPRB, 0.16548E+01_JPRB, 0.15955E+01_JPRB ,&
&  0.77842E+00_JPRB, 0.97153E+00_JPRB, 0.15316E+01_JPRB, 0.17417E+01_JPRB, 0.18637E+01_JPRB, 0.18917E+01_JPRB, 0.18502E+01_JPRB ,&
&  0.16905E+01_JPRB, 0.14623E+01_JPRB, 0.13954E+01_JPRB, 0.15321E+01_JPRB, 0.14753E+01_JPRB, 0.14726E+01_JPRB, 0.14597E+01_JPRB ,&
&  0.14569E+01_JPRB, 0.14823E+01_JPRB, 0.15228E+01_JPRB, 0.15696E+01_JPRB, 0.16578E+01_JPRB, 0.17757E+01_JPRB, 0.19420E+01_JPRB ,&
&  0.21495E+01_JPRB, 0.23601E+01_JPRB, 0.25472E+01_JPRB, 0.22464E+01_JPRB, 0.21809E+01_JPRB, 0.21241E+01_JPRB, 0.21579E+01_JPRB ,&
&  0.13958E+01_JPRB, 0.10647E+01_JPRB, 0.11332E+01_JPRB, 0.12496E+01_JPRB, 0.11724E+01_JPRB, 0.11126E+01_JPRB, 0.10779E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10027E+01_JPRB, 0.99385E+00_JPRB, 0.97570E+00_JPRB, 0.97234E+00_JPRB, 0.99039E+00_JPRB, 0.10211E+01_JPRB ,&
&  0.10310E+01_JPRB, 0.10359E+01_JPRB, 0.10392E+01_JPRB, 0.10457E+01_JPRB, 0.10268E+01_JPRB, 0.10055E+01_JPRB, 0.10017E+01_JPRB ,&
&  0.10068E+01_JPRB, 0.98889E+00_JPRB, 0.99120E+00_JPRB, 0.10203E+01_JPRB, 0.10493E+01_JPRB, 0.10585E+01_JPRB, 0.10603E+01_JPRB ,&
&  0.10307E+01_JPRB, 0.10374E+01_JPRB, 0.11572E+01_JPRB, 0.12010E+01_JPRB, 0.11853E+01_JPRB, 0.99913E+00_JPRB, 0.82956E+00_JPRB ,&
&  0.80866E+00_JPRB, 0.77945E+00_JPRB, 0.75329E+00_JPRB, 0.74581E+00_JPRB, 0.73833E+00_JPRB /)
ZBURD(:, 43, 11) = (/ &
&  0.68176E+00_JPRB, 0.66387E+00_JPRB, 0.64911E+00_JPRB, 0.65522E+00_JPRB, 0.66498E+00_JPRB, 0.68358E+00_JPRB, 0.73404E+00_JPRB ,&
&  0.77547E+00_JPRB, 0.79229E+00_JPRB, 0.81395E+00_JPRB, 0.81646E+00_JPRB, 0.82851E+00_JPRB, 0.81751E+00_JPRB, 0.81950E+00_JPRB ,&
&  0.81140E+00_JPRB, 0.82343E+00_JPRB, 0.84546E+00_JPRB, 0.86855E+00_JPRB, 0.88778E+00_JPRB, 0.90814E+00_JPRB, 0.93888E+00_JPRB ,&
&  0.97571E+00_JPRB, 0.10267E+01_JPRB, 0.11076E+01_JPRB, 0.12360E+01_JPRB, 0.14368E+01_JPRB, 0.16784E+01_JPRB, 0.18482E+01_JPRB ,&
&  0.14089E+01_JPRB, 0.63223E+00_JPRB, 0.11612E+01_JPRB, 0.16114E+01_JPRB, 0.16775E+01_JPRB, 0.17427E+01_JPRB, 0.16719E+01_JPRB ,&
&  0.15206E+01_JPRB, 0.13860E+01_JPRB, 0.12938E+01_JPRB, 0.14024E+01_JPRB, 0.13771E+01_JPRB, 0.13688E+01_JPRB, 0.13617E+01_JPRB ,&
&  0.13448E+01_JPRB, 0.13427E+01_JPRB, 0.13500E+01_JPRB, 0.13841E+01_JPRB, 0.14536E+01_JPRB, 0.15683E+01_JPRB, 0.17228E+01_JPRB ,&
&  0.19365E+01_JPRB, 0.22290E+01_JPRB, 0.24938E+01_JPRB, 0.22867E+01_JPRB, 0.23189E+01_JPRB, 0.20791E+01_JPRB, 0.18436E+01_JPRB ,&
&  0.13435E+01_JPRB, 0.11243E+01_JPRB, 0.11641E+01_JPRB, 0.12978E+01_JPRB, 0.12199E+01_JPRB, 0.10698E+01_JPRB, 0.11278E+01_JPRB ,&
&  0.10955E+01_JPRB, 0.10613E+01_JPRB, 0.10456E+01_JPRB, 0.10235E+01_JPRB, 0.10233E+01_JPRB, 0.10321E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.10568E+01_JPRB, 0.10752E+01_JPRB, 0.10890E+01_JPRB, 0.10885E+01_JPRB, 0.10728E+01_JPRB, 0.10738E+01_JPRB, 0.10968E+01_JPRB ,&
&  0.11202E+01_JPRB, 0.11261E+01_JPRB, 0.11483E+01_JPRB, 0.11677E+01_JPRB, 0.11817E+01_JPRB, 0.11489E+01_JPRB, 0.11335E+01_JPRB ,&
&  0.11025E+01_JPRB, 0.10764E+01_JPRB, 0.10524E+01_JPRB, 0.10310E+01_JPRB, 0.10096E+01_JPRB, 0.93197E+00_JPRB, 0.83828E+00_JPRB ,&
&  0.80662E+00_JPRB, 0.77202E+00_JPRB, 0.74633E+00_JPRB, 0.72682E+00_JPRB, 0.70619E+00_JPRB /)
ZBURD(:, 44, 11) = (/ &
&  0.72595E+00_JPRB, 0.70755E+00_JPRB, 0.67644E+00_JPRB, 0.63526E+00_JPRB, 0.63035E+00_JPRB, 0.62706E+00_JPRB, 0.63838E+00_JPRB ,&
&  0.67951E+00_JPRB, 0.73261E+00_JPRB, 0.74154E+00_JPRB, 0.73769E+00_JPRB, 0.71795E+00_JPRB, 0.71691E+00_JPRB, 0.71998E+00_JPRB ,&
&  0.72994E+00_JPRB, 0.73676E+00_JPRB, 0.75583E+00_JPRB, 0.78229E+00_JPRB, 0.81167E+00_JPRB, 0.83532E+00_JPRB, 0.84599E+00_JPRB ,&
&  0.85754E+00_JPRB, 0.86425E+00_JPRB, 0.88763E+00_JPRB, 0.92532E+00_JPRB, 0.10052E+01_JPRB, 0.11728E+01_JPRB, 0.14571E+01_JPRB ,&
&  0.21304E+01_JPRB, 0.17516E+01_JPRB, 0.85024E+00_JPRB, 0.14541E+01_JPRB, 0.16150E+01_JPRB, 0.17214E+01_JPRB, 0.14806E+01_JPRB ,&
&  0.14079E+01_JPRB, 0.12615E+01_JPRB, 0.11745E+01_JPRB, 0.12535E+01_JPRB, 0.12264E+01_JPRB, 0.12234E+01_JPRB, 0.12203E+01_JPRB ,&
&  0.11862E+01_JPRB, 0.11884E+01_JPRB, 0.11918E+01_JPRB, 0.12107E+01_JPRB, 0.12476E+01_JPRB, 0.13328E+01_JPRB, 0.14930E+01_JPRB ,&
&  0.16935E+01_JPRB, 0.19528E+01_JPRB, 0.23167E+01_JPRB, 0.24240E+01_JPRB, 0.25832E+01_JPRB, 0.20235E+01_JPRB, 0.16689E+01_JPRB ,&
&  0.12731E+01_JPRB, 0.12440E+01_JPRB, 0.13291E+01_JPRB, 0.13471E+01_JPRB, 0.12500E+01_JPRB, 0.11354E+01_JPRB, 0.11940E+01_JPRB ,&
&  0.11480E+01_JPRB, 0.11114E+01_JPRB, 0.10963E+01_JPRB, 0.10729E+01_JPRB, 0.10595E+01_JPRB, 0.10578E+01_JPRB, 0.10610E+01_JPRB ,&
&  0.10651E+01_JPRB, 0.10829E+01_JPRB, 0.10962E+01_JPRB, 0.11052E+01_JPRB, 0.11159E+01_JPRB, 0.11210E+01_JPRB, 0.11505E+01_JPRB ,&
&  0.11799E+01_JPRB, 0.12076E+01_JPRB, 0.12398E+01_JPRB, 0.12643E+01_JPRB, 0.12395E+01_JPRB, 0.11899E+01_JPRB, 0.11913E+01_JPRB ,&
&  0.11737E+01_JPRB, 0.11571E+01_JPRB, 0.11007E+01_JPRB, 0.11052E+01_JPRB, 0.10418E+01_JPRB, 0.96046E+00_JPRB, 0.87518E+00_JPRB ,&
&  0.86341E+00_JPRB, 0.85065E+00_JPRB, 0.80501E+00_JPRB, 0.74894E+00_JPRB, 0.72900E+00_JPRB /)
ZBURD(:, 45, 11) = (/ &
&  0.78047E+00_JPRB, 0.78537E+00_JPRB, 0.77606E+00_JPRB, 0.73416E+00_JPRB, 0.69525E+00_JPRB, 0.65743E+00_JPRB, 0.63019E+00_JPRB ,&
&  0.62511E+00_JPRB, 0.63299E+00_JPRB, 0.66185E+00_JPRB, 0.68274E+00_JPRB, 0.67366E+00_JPRB, 0.65279E+00_JPRB, 0.64550E+00_JPRB ,&
&  0.65150E+00_JPRB, 0.66997E+00_JPRB, 0.68411E+00_JPRB, 0.70707E+00_JPRB, 0.73178E+00_JPRB, 0.75403E+00_JPRB, 0.77887E+00_JPRB ,&
&  0.77052E+00_JPRB, 0.77356E+00_JPRB, 0.78576E+00_JPRB, 0.79790E+00_JPRB, 0.82125E+00_JPRB, 0.87689E+00_JPRB, 0.94257E+00_JPRB ,&
&  0.11793E+01_JPRB, 0.19444E+01_JPRB, 0.11542E+01_JPRB, 0.15715E+01_JPRB, 0.17514E+01_JPRB, 0.17274E+01_JPRB, 0.15197E+01_JPRB ,&
&  0.13658E+01_JPRB, 0.11974E+01_JPRB, 0.11516E+01_JPRB, 0.11519E+01_JPRB, 0.10616E+01_JPRB, 0.10499E+01_JPRB, 0.10460E+01_JPRB ,&
&  0.10384E+01_JPRB, 0.10541E+01_JPRB, 0.10643E+01_JPRB, 0.10792E+01_JPRB, 0.10870E+01_JPRB, 0.11140E+01_JPRB, 0.12015E+01_JPRB ,&
&  0.13801E+01_JPRB, 0.16149E+01_JPRB, 0.20074E+01_JPRB, 0.22939E+01_JPRB, 0.26855E+01_JPRB, 0.21782E+01_JPRB, 0.17287E+01_JPRB ,&
&  0.13896E+01_JPRB, 0.13983E+01_JPRB, 0.14423E+01_JPRB, 0.14363E+01_JPRB, 0.13092E+01_JPRB, 0.12682E+01_JPRB, 0.13179E+01_JPRB ,&
&  0.12354E+01_JPRB, 0.11699E+01_JPRB, 0.11436E+01_JPRB, 0.11200E+01_JPRB, 0.10899E+01_JPRB, 0.10702E+01_JPRB, 0.10602E+01_JPRB ,&
&  0.10550E+01_JPRB, 0.10619E+01_JPRB, 0.10796E+01_JPRB, 0.10925E+01_JPRB, 0.11119E+01_JPRB, 0.11266E+01_JPRB, 0.11459E+01_JPRB ,&
&  0.11659E+01_JPRB, 0.11992E+01_JPRB, 0.12367E+01_JPRB, 0.12472E+01_JPRB, 0.12354E+01_JPRB, 0.12141E+01_JPRB, 0.12171E+01_JPRB ,&
&  0.12387E+01_JPRB, 0.12731E+01_JPRB, 0.11432E+01_JPRB, 0.11162E+01_JPRB, 0.11393E+01_JPRB, 0.10703E+01_JPRB, 0.97278E+00_JPRB ,&
&  0.99369E+00_JPRB, 0.11315E+01_JPRB, 0.11731E+01_JPRB, 0.81781E+00_JPRB, 0.78757E+00_JPRB /)
ZBURD(:, 46, 11) = (/ &
&  0.80335E+00_JPRB, 0.80077E+00_JPRB, 0.78470E+00_JPRB, 0.77828E+00_JPRB, 0.74308E+00_JPRB, 0.70681E+00_JPRB, 0.67461E+00_JPRB ,&
&  0.65402E+00_JPRB, 0.64374E+00_JPRB, 0.64799E+00_JPRB, 0.64759E+00_JPRB, 0.64761E+00_JPRB, 0.63896E+00_JPRB, 0.63307E+00_JPRB ,&
&  0.61984E+00_JPRB, 0.62415E+00_JPRB, 0.63959E+00_JPRB, 0.66477E+00_JPRB, 0.69388E+00_JPRB, 0.70445E+00_JPRB, 0.71632E+00_JPRB ,&
&  0.71975E+00_JPRB, 0.71709E+00_JPRB, 0.71891E+00_JPRB, 0.72865E+00_JPRB, 0.74811E+00_JPRB, 0.77208E+00_JPRB, 0.80820E+00_JPRB ,&
&  0.89099E+00_JPRB, 0.16689E+01_JPRB, 0.25375E+01_JPRB, 0.28406E+01_JPRB, 0.23852E+01_JPRB, 0.19409E+01_JPRB, 0.16888E+01_JPRB ,&
&  0.15856E+01_JPRB, 0.14494E+01_JPRB, 0.13753E+01_JPRB, 0.11791E+01_JPRB, 0.10222E+01_JPRB, 0.96039E+00_JPRB, 0.95046E+00_JPRB ,&
&  0.98254E+00_JPRB, 0.99347E+00_JPRB, 0.99923E+00_JPRB, 0.10027E+01_JPRB, 0.98974E+00_JPRB, 0.98634E+00_JPRB, 0.10005E+01_JPRB ,&
&  0.10679E+01_JPRB, 0.12240E+01_JPRB, 0.15066E+01_JPRB, 0.19806E+01_JPRB, 0.25027E+01_JPRB, 0.22837E+01_JPRB, 0.19362E+01_JPRB ,&
&  0.18468E+01_JPRB, 0.17582E+01_JPRB, 0.16530E+01_JPRB, 0.15842E+01_JPRB, 0.13447E+01_JPRB, 0.15403E+01_JPRB, 0.14811E+01_JPRB ,&
&  0.14114E+01_JPRB, 0.13595E+01_JPRB, 0.12839E+01_JPRB, 0.12085E+01_JPRB, 0.11488E+01_JPRB, 0.10895E+01_JPRB, 0.10466E+01_JPRB ,&
&  0.10344E+01_JPRB, 0.10401E+01_JPRB, 0.10503E+01_JPRB, 0.10680E+01_JPRB, 0.10926E+01_JPRB, 0.11167E+01_JPRB, 0.11294E+01_JPRB ,&
&  0.11284E+01_JPRB, 0.11386E+01_JPRB, 0.11167E+01_JPRB, 0.11267E+01_JPRB, 0.11236E+01_JPRB, 0.10977E+01_JPRB, 0.10794E+01_JPRB ,&
&  0.11106E+01_JPRB, 0.11642E+01_JPRB, 0.11417E+01_JPRB, 0.11356E+01_JPRB, 0.11275E+01_JPRB, 0.11208E+01_JPRB, 0.99847E+00_JPRB ,&
&  0.95395E+00_JPRB, 0.99660E+00_JPRB, 0.94320E+00_JPRB, 0.86184E+00_JPRB, 0.82715E+00_JPRB /)
ZBURD(:, 47, 11) = (/ &
&  0.83318E+00_JPRB, 0.80539E+00_JPRB, 0.78665E+00_JPRB, 0.78575E+00_JPRB, 0.76172E+00_JPRB, 0.74469E+00_JPRB, 0.73659E+00_JPRB ,&
&  0.69758E+00_JPRB, 0.67055E+00_JPRB, 0.64962E+00_JPRB, 0.64363E+00_JPRB, 0.64449E+00_JPRB, 0.63703E+00_JPRB, 0.63732E+00_JPRB ,&
&  0.62775E+00_JPRB, 0.61745E+00_JPRB, 0.61887E+00_JPRB, 0.63955E+00_JPRB, 0.66779E+00_JPRB, 0.67533E+00_JPRB, 0.69132E+00_JPRB ,&
&  0.70781E+00_JPRB, 0.70511E+00_JPRB, 0.69595E+00_JPRB, 0.69492E+00_JPRB, 0.69990E+00_JPRB, 0.69852E+00_JPRB, 0.72014E+00_JPRB ,&
&  0.80497E+00_JPRB, 0.17231E+01_JPRB, 0.18490E+01_JPRB, 0.31661E+01_JPRB, 0.27035E+01_JPRB, 0.22555E+01_JPRB, 0.17402E+01_JPRB ,&
&  0.17067E+01_JPRB, 0.18817E+01_JPRB, 0.15378E+01_JPRB, 0.12944E+01_JPRB, 0.11036E+01_JPRB, 0.97242E+00_JPRB, 0.93086E+00_JPRB ,&
&  0.94374E+00_JPRB, 0.96365E+00_JPRB, 0.97940E+00_JPRB, 0.95757E+00_JPRB, 0.93268E+00_JPRB, 0.92444E+00_JPRB, 0.92761E+00_JPRB ,&
&  0.93938E+00_JPRB, 0.97988E+00_JPRB, 0.11178E+01_JPRB, 0.13908E+01_JPRB, 0.20465E+01_JPRB, 0.23757E+01_JPRB, 0.21986E+01_JPRB ,&
&  0.21412E+01_JPRB, 0.22662E+01_JPRB, 0.20845E+01_JPRB, 0.18824E+01_JPRB, 0.16508E+01_JPRB, 0.16429E+01_JPRB, 0.15816E+01_JPRB ,&
&  0.15241E+01_JPRB, 0.15099E+01_JPRB, 0.14682E+01_JPRB, 0.13898E+01_JPRB, 0.13011E+01_JPRB, 0.12053E+01_JPRB, 0.11125E+01_JPRB ,&
&  0.10501E+01_JPRB, 0.10217E+01_JPRB, 0.10073E+01_JPRB, 0.10194E+01_JPRB, 0.10516E+01_JPRB, 0.10798E+01_JPRB, 0.11012E+01_JPRB ,&
&  0.11006E+01_JPRB, 0.11094E+01_JPRB, 0.10600E+01_JPRB, 0.10205E+01_JPRB, 0.10256E+01_JPRB, 0.10036E+01_JPRB, 0.99869E+00_JPRB ,&
&  0.10538E+01_JPRB, 0.11017E+01_JPRB, 0.11152E+01_JPRB, 0.11134E+01_JPRB, 0.11257E+01_JPRB, 0.11689E+01_JPRB, 0.10348E+01_JPRB ,&
&  0.97450E+00_JPRB, 0.98042E+00_JPRB, 0.98048E+00_JPRB, 0.91355E+00_JPRB, 0.87443E+00_JPRB /)
ZBURD(:, 48, 11) = (/ &
&  0.90000E+00_JPRB, 0.85293E+00_JPRB, 0.81379E+00_JPRB, 0.79522E+00_JPRB, 0.78376E+00_JPRB, 0.76893E+00_JPRB, 0.75529E+00_JPRB ,&
&  0.72965E+00_JPRB, 0.69935E+00_JPRB, 0.67588E+00_JPRB, 0.65611E+00_JPRB, 0.65405E+00_JPRB, 0.64698E+00_JPRB, 0.63734E+00_JPRB ,&
&  0.62537E+00_JPRB, 0.61927E+00_JPRB, 0.61688E+00_JPRB, 0.62483E+00_JPRB, 0.64199E+00_JPRB, 0.65342E+00_JPRB, 0.66922E+00_JPRB ,&
&  0.67843E+00_JPRB, 0.68895E+00_JPRB, 0.69161E+00_JPRB, 0.68734E+00_JPRB, 0.67889E+00_JPRB, 0.66712E+00_JPRB, 0.65687E+00_JPRB ,&
&  0.78073E+00_JPRB, 0.20011E+01_JPRB, 0.16200E+01_JPRB, 0.28693E+01_JPRB, 0.24796E+01_JPRB, 0.22146E+01_JPRB, 0.17547E+01_JPRB ,&
&  0.16515E+01_JPRB, 0.17796E+01_JPRB, 0.15729E+01_JPRB, 0.13747E+01_JPRB, 0.12280E+01_JPRB, 0.11034E+01_JPRB, 0.10167E+01_JPRB ,&
&  0.98055E+00_JPRB, 0.96594E+00_JPRB, 0.94770E+00_JPRB, 0.92144E+00_JPRB, 0.91309E+00_JPRB, 0.90529E+00_JPRB, 0.90842E+00_JPRB ,&
&  0.91276E+00_JPRB, 0.92169E+00_JPRB, 0.96705E+00_JPRB, 0.11529E+01_JPRB, 0.15075E+01_JPRB, 0.20680E+01_JPRB, 0.24459E+01_JPRB ,&
&  0.25399E+01_JPRB, 0.28186E+01_JPRB, 0.25404E+01_JPRB, 0.22195E+01_JPRB, 0.19928E+01_JPRB, 0.18447E+01_JPRB, 0.17220E+01_JPRB ,&
&  0.16030E+01_JPRB, 0.15355E+01_JPRB, 0.15195E+01_JPRB, 0.15046E+01_JPRB, 0.14683E+01_JPRB, 0.13894E+01_JPRB, 0.12831E+01_JPRB ,&
&  0.11760E+01_JPRB, 0.11060E+01_JPRB, 0.10533E+01_JPRB, 0.10273E+01_JPRB, 0.10216E+01_JPRB, 0.10320E+01_JPRB, 0.10462E+01_JPRB ,&
&  0.10396E+01_JPRB, 0.10516E+01_JPRB, 0.10357E+01_JPRB, 0.98645E+00_JPRB, 0.99051E+00_JPRB, 0.98154E+00_JPRB, 0.96969E+00_JPRB ,&
&  0.10252E+01_JPRB, 0.10610E+01_JPRB, 0.10733E+01_JPRB, 0.11278E+01_JPRB, 0.11368E+01_JPRB, 0.12402E+01_JPRB, 0.10611E+01_JPRB ,&
&  0.99718E+00_JPRB, 0.97652E+00_JPRB, 0.96845E+00_JPRB, 0.94787E+00_JPRB, 0.93418E+00_JPRB /)
ZBURD(:, 49, 11) = (/ &
&  0.96084E+00_JPRB, 0.92808E+00_JPRB, 0.88934E+00_JPRB, 0.85088E+00_JPRB, 0.81949E+00_JPRB, 0.79967E+00_JPRB, 0.78144E+00_JPRB ,&
&  0.75215E+00_JPRB, 0.72283E+00_JPRB, 0.70467E+00_JPRB, 0.68955E+00_JPRB, 0.67811E+00_JPRB, 0.67024E+00_JPRB, 0.65251E+00_JPRB ,&
&  0.63568E+00_JPRB, 0.61701E+00_JPRB, 0.62351E+00_JPRB, 0.63422E+00_JPRB, 0.64469E+00_JPRB, 0.65450E+00_JPRB, 0.67422E+00_JPRB ,&
&  0.67184E+00_JPRB, 0.67324E+00_JPRB, 0.67695E+00_JPRB, 0.67849E+00_JPRB, 0.66794E+00_JPRB, 0.66697E+00_JPRB, 0.64424E+00_JPRB ,&
&  0.74940E+00_JPRB, 0.88030E+00_JPRB, 0.17954E+01_JPRB, 0.24584E+01_JPRB, 0.22458E+01_JPRB, 0.20089E+01_JPRB, 0.18102E+01_JPRB ,&
&  0.17667E+01_JPRB, 0.16721E+01_JPRB, 0.14842E+01_JPRB, 0.13534E+01_JPRB, 0.12438E+01_JPRB, 0.11616E+01_JPRB, 0.11000E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.99712E+00_JPRB, 0.95503E+00_JPRB, 0.91634E+00_JPRB, 0.89693E+00_JPRB, 0.88712E+00_JPRB, 0.88888E+00_JPRB ,&
&  0.89194E+00_JPRB, 0.89120E+00_JPRB, 0.90240E+00_JPRB, 0.99338E+00_JPRB, 0.12948E+01_JPRB, 0.17576E+01_JPRB, 0.23096E+01_JPRB ,&
&  0.28127E+01_JPRB, 0.30028E+01_JPRB, 0.27451E+01_JPRB, 0.24483E+01_JPRB, 0.21823E+01_JPRB, 0.19826E+01_JPRB, 0.18323E+01_JPRB ,&
&  0.16929E+01_JPRB, 0.15796E+01_JPRB, 0.14968E+01_JPRB, 0.14530E+01_JPRB, 0.14235E+01_JPRB, 0.13946E+01_JPRB, 0.13510E+01_JPRB ,&
&  0.12900E+01_JPRB, 0.12173E+01_JPRB, 0.11472E+01_JPRB, 0.10881E+01_JPRB, 0.10402E+01_JPRB, 0.10217E+01_JPRB, 0.10135E+01_JPRB ,&
&  0.99063E+00_JPRB, 0.98950E+00_JPRB, 0.10026E+01_JPRB, 0.97721E+00_JPRB, 0.99948E+00_JPRB, 0.10087E+01_JPRB, 0.10198E+01_JPRB ,&
&  0.10597E+01_JPRB, 0.10775E+01_JPRB, 0.10702E+01_JPRB, 0.11449E+01_JPRB, 0.11562E+01_JPRB, 0.12233E+01_JPRB, 0.11124E+01_JPRB ,&
&  0.10381E+01_JPRB, 0.10115E+01_JPRB, 0.98581E+00_JPRB, 0.97204E+00_JPRB, 0.97235E+00_JPRB /)
ZBURD(:, 50, 11) = (/ &
&  0.10051E+01_JPRB, 0.98752E+00_JPRB, 0.94411E+00_JPRB, 0.89804E+00_JPRB, 0.86088E+00_JPRB, 0.83075E+00_JPRB, 0.80347E+00_JPRB ,&
&  0.77432E+00_JPRB, 0.74139E+00_JPRB, 0.71734E+00_JPRB, 0.70582E+00_JPRB, 0.70132E+00_JPRB, 0.69568E+00_JPRB, 0.67752E+00_JPRB ,&
&  0.65636E+00_JPRB, 0.64133E+00_JPRB, 0.64293E+00_JPRB, 0.64944E+00_JPRB, 0.66091E+00_JPRB, 0.65736E+00_JPRB, 0.67024E+00_JPRB ,&
&  0.67778E+00_JPRB, 0.67653E+00_JPRB, 0.67434E+00_JPRB, 0.67161E+00_JPRB, 0.66585E+00_JPRB, 0.66106E+00_JPRB, 0.64311E+00_JPRB ,&
&  0.70512E+00_JPRB, 0.10745E+01_JPRB, 0.16866E+01_JPRB, 0.21051E+01_JPRB, 0.20866E+01_JPRB, 0.18457E+01_JPRB, 0.17006E+01_JPRB ,&
&  0.15761E+01_JPRB, 0.14957E+01_JPRB, 0.13568E+01_JPRB, 0.12612E+01_JPRB, 0.11755E+01_JPRB, 0.11094E+01_JPRB, 0.10582E+01_JPRB ,&
&  0.10241E+01_JPRB, 0.99218E+00_JPRB, 0.95508E+00_JPRB, 0.92179E+00_JPRB, 0.89880E+00_JPRB, 0.88047E+00_JPRB, 0.87314E+00_JPRB ,&
&  0.87317E+00_JPRB, 0.86975E+00_JPRB, 0.87170E+00_JPRB, 0.90811E+00_JPRB, 0.11404E+01_JPRB, 0.13104E+01_JPRB, 0.19772E+01_JPRB ,&
&  0.25696E+01_JPRB, 0.24179E+01_JPRB, 0.23053E+01_JPRB, 0.21397E+01_JPRB, 0.19479E+01_JPRB, 0.17963E+01_JPRB, 0.16420E+01_JPRB ,&
&  0.15066E+01_JPRB, 0.13968E+01_JPRB, 0.13090E+01_JPRB, 0.12510E+01_JPRB, 0.12134E+01_JPRB, 0.11890E+01_JPRB, 0.11795E+01_JPRB ,&
&  0.11649E+01_JPRB, 0.11513E+01_JPRB, 0.11326E+01_JPRB, 0.11049E+01_JPRB, 0.10615E+01_JPRB, 0.10165E+01_JPRB, 0.97863E+00_JPRB ,&
&  0.94211E+00_JPRB, 0.93467E+00_JPRB, 0.96992E+00_JPRB, 0.94801E+00_JPRB, 0.98972E+00_JPRB, 0.10327E+01_JPRB, 0.10437E+01_JPRB ,&
&  0.10486E+01_JPRB, 0.11035E+01_JPRB, 0.10979E+01_JPRB, 0.11163E+01_JPRB, 0.12303E+01_JPRB, 0.13016E+01_JPRB, 0.11924E+01_JPRB ,&
&  0.11110E+01_JPRB, 0.10629E+01_JPRB, 0.10318E+01_JPRB, 0.10030E+01_JPRB, 0.10113E+01_JPRB /)
ZBURD(:, 51, 11) = (/ &
&  0.10582E+01_JPRB, 0.10061E+01_JPRB, 0.95742E+00_JPRB, 0.90942E+00_JPRB, 0.87509E+00_JPRB, 0.83802E+00_JPRB, 0.80258E+00_JPRB ,&
&  0.77190E+00_JPRB, 0.75156E+00_JPRB, 0.73457E+00_JPRB, 0.71799E+00_JPRB, 0.71494E+00_JPRB, 0.70648E+00_JPRB, 0.69033E+00_JPRB ,&
&  0.66917E+00_JPRB, 0.65543E+00_JPRB, 0.65173E+00_JPRB, 0.65459E+00_JPRB, 0.66651E+00_JPRB, 0.67419E+00_JPRB, 0.68327E+00_JPRB ,&
&  0.68946E+00_JPRB, 0.67112E+00_JPRB, 0.66098E+00_JPRB, 0.66186E+00_JPRB, 0.66143E+00_JPRB, 0.66299E+00_JPRB, 0.65013E+00_JPRB ,&
&  0.68363E+00_JPRB, 0.88625E+00_JPRB, 0.12422E+01_JPRB, 0.17052E+01_JPRB, 0.17680E+01_JPRB, 0.16493E+01_JPRB, 0.15299E+01_JPRB ,&
&  0.14247E+01_JPRB, 0.13385E+01_JPRB, 0.12295E+01_JPRB, 0.11100E+01_JPRB, 0.10184E+01_JPRB, 0.96034E+00_JPRB, 0.92462E+00_JPRB ,&
&  0.90953E+00_JPRB, 0.89894E+00_JPRB, 0.88366E+00_JPRB, 0.87142E+00_JPRB, 0.85894E+00_JPRB, 0.84813E+00_JPRB, 0.84321E+00_JPRB ,&
&  0.83810E+00_JPRB, 0.83170E+00_JPRB, 0.82774E+00_JPRB, 0.83686E+00_JPRB, 0.96416E+00_JPRB, 0.10963E+01_JPRB, 0.13830E+01_JPRB ,&
&  0.15022E+01_JPRB, 0.15394E+01_JPRB, 0.15746E+01_JPRB, 0.15547E+01_JPRB, 0.14886E+01_JPRB, 0.14243E+01_JPRB, 0.13608E+01_JPRB ,&
&  0.12871E+01_JPRB, 0.12035E+01_JPRB, 0.11469E+01_JPRB, 0.11059E+01_JPRB, 0.10676E+01_JPRB, 0.10380E+01_JPRB, 0.10168E+01_JPRB ,&
&  0.10118E+01_JPRB, 0.10235E+01_JPRB, 0.10330E+01_JPRB, 0.10352E+01_JPRB, 0.10191E+01_JPRB, 0.99354E+00_JPRB, 0.96336E+00_JPRB ,&
&  0.92252E+00_JPRB, 0.89522E+00_JPRB, 0.90429E+00_JPRB, 0.92476E+00_JPRB, 0.95690E+00_JPRB, 0.97142E+00_JPRB, 0.99107E+00_JPRB ,&
&  0.10290E+01_JPRB, 0.11214E+01_JPRB, 0.11175E+01_JPRB, 0.11610E+01_JPRB, 0.12643E+01_JPRB, 0.13351E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.11955E+01_JPRB, 0.11282E+01_JPRB, 0.10866E+01_JPRB, 0.10528E+01_JPRB, 0.10932E+01_JPRB /)
ZBURD(:, 52, 11) = (/ &
&  0.11554E+01_JPRB, 0.10691E+01_JPRB, 0.99904E+00_JPRB, 0.94721E+00_JPRB, 0.91050E+00_JPRB, 0.86788E+00_JPRB, 0.82069E+00_JPRB ,&
&  0.78550E+00_JPRB, 0.76967E+00_JPRB, 0.75392E+00_JPRB, 0.73161E+00_JPRB, 0.71536E+00_JPRB, 0.69898E+00_JPRB, 0.68507E+00_JPRB ,&
&  0.66769E+00_JPRB, 0.66277E+00_JPRB, 0.65910E+00_JPRB, 0.66382E+00_JPRB, 0.66514E+00_JPRB, 0.66983E+00_JPRB, 0.68127E+00_JPRB ,&
&  0.68453E+00_JPRB, 0.66538E+00_JPRB, 0.66179E+00_JPRB, 0.66131E+00_JPRB, 0.65974E+00_JPRB, 0.65880E+00_JPRB, 0.64941E+00_JPRB ,&
&  0.68893E+00_JPRB, 0.75414E+00_JPRB, 0.97107E+00_JPRB, 0.11911E+01_JPRB, 0.12956E+01_JPRB, 0.13508E+01_JPRB, 0.13195E+01_JPRB ,&
&  0.12608E+01_JPRB, 0.11806E+01_JPRB, 0.10926E+01_JPRB, 0.10052E+01_JPRB, 0.92529E+00_JPRB, 0.87146E+00_JPRB, 0.84035E+00_JPRB ,&
&  0.82209E+00_JPRB, 0.81442E+00_JPRB, 0.80685E+00_JPRB, 0.80203E+00_JPRB, 0.79702E+00_JPRB, 0.78939E+00_JPRB, 0.78873E+00_JPRB ,&
&  0.79327E+00_JPRB, 0.78268E+00_JPRB, 0.77834E+00_JPRB, 0.79114E+00_JPRB, 0.82020E+00_JPRB, 0.87090E+00_JPRB, 0.10046E+01_JPRB ,&
&  0.10855E+01_JPRB, 0.11414E+01_JPRB, 0.11985E+01_JPRB, 0.12131E+01_JPRB, 0.11876E+01_JPRB, 0.11538E+01_JPRB, 0.11048E+01_JPRB ,&
&  0.10694E+01_JPRB, 0.10409E+01_JPRB, 0.10124E+01_JPRB, 0.98414E+00_JPRB, 0.95909E+00_JPRB, 0.94219E+00_JPRB, 0.93104E+00_JPRB ,&
&  0.92216E+00_JPRB, 0.92870E+00_JPRB, 0.93783E+00_JPRB, 0.94367E+00_JPRB, 0.93959E+00_JPRB, 0.93446E+00_JPRB, 0.92731E+00_JPRB ,&
&  0.90690E+00_JPRB, 0.87767E+00_JPRB, 0.86882E+00_JPRB, 0.88226E+00_JPRB, 0.91733E+00_JPRB, 0.93724E+00_JPRB, 0.94920E+00_JPRB ,&
&  0.97448E+00_JPRB, 0.10231E+01_JPRB, 0.10518E+01_JPRB, 0.12300E+01_JPRB, 0.13362E+01_JPRB, 0.13037E+01_JPRB, 0.12543E+01_JPRB ,&
&  0.11962E+01_JPRB, 0.11588E+01_JPRB, 0.11125E+01_JPRB, 0.10738E+01_JPRB, 0.11502E+01_JPRB /)
ZBURD(:, 53, 11) = (/ &
&  0.98033E+00_JPRB, 0.97033E+00_JPRB, 0.93730E+00_JPRB, 0.90847E+00_JPRB, 0.89181E+00_JPRB, 0.87443E+00_JPRB, 0.84624E+00_JPRB ,&
&  0.81553E+00_JPRB, 0.78744E+00_JPRB, 0.76897E+00_JPRB, 0.74552E+00_JPRB, 0.72348E+00_JPRB, 0.70960E+00_JPRB, 0.69621E+00_JPRB ,&
&  0.67839E+00_JPRB, 0.67298E+00_JPRB, 0.67253E+00_JPRB, 0.67066E+00_JPRB, 0.66640E+00_JPRB, 0.66400E+00_JPRB, 0.67105E+00_JPRB ,&
&  0.67205E+00_JPRB, 0.67018E+00_JPRB, 0.66938E+00_JPRB, 0.66377E+00_JPRB, 0.65679E+00_JPRB, 0.65452E+00_JPRB, 0.64290E+00_JPRB ,&
&  0.65558E+00_JPRB, 0.64251E+00_JPRB, 0.74836E+00_JPRB, 0.87803E+00_JPRB, 0.98764E+00_JPRB, 0.10654E+01_JPRB, 0.11204E+01_JPRB ,&
&  0.11111E+01_JPRB, 0.10483E+01_JPRB, 0.98160E+00_JPRB, 0.93587E+00_JPRB, 0.88831E+00_JPRB, 0.85005E+00_JPRB, 0.81874E+00_JPRB ,&
&  0.79819E+00_JPRB, 0.78704E+00_JPRB, 0.77869E+00_JPRB, 0.77071E+00_JPRB, 0.76222E+00_JPRB, 0.75488E+00_JPRB, 0.75114E+00_JPRB ,&
&  0.74644E+00_JPRB, 0.74606E+00_JPRB, 0.74775E+00_JPRB, 0.74588E+00_JPRB, 0.75397E+00_JPRB, 0.79053E+00_JPRB, 0.85526E+00_JPRB ,&
&  0.89126E+00_JPRB, 0.93314E+00_JPRB, 0.98632E+00_JPRB, 0.10084E+01_JPRB, 0.99133E+00_JPRB, 0.97155E+00_JPRB, 0.94493E+00_JPRB ,&
&  0.92357E+00_JPRB, 0.89722E+00_JPRB, 0.88512E+00_JPRB, 0.87039E+00_JPRB, 0.85341E+00_JPRB, 0.84134E+00_JPRB, 0.83653E+00_JPRB ,&
&  0.83622E+00_JPRB, 0.83697E+00_JPRB, 0.84458E+00_JPRB, 0.85453E+00_JPRB, 0.85548E+00_JPRB, 0.85324E+00_JPRB, 0.85630E+00_JPRB ,&
&  0.86145E+00_JPRB, 0.85498E+00_JPRB, 0.84284E+00_JPRB, 0.85332E+00_JPRB, 0.87947E+00_JPRB, 0.89988E+00_JPRB, 0.91478E+00_JPRB ,&
&  0.93011E+00_JPRB, 0.95527E+00_JPRB, 0.98960E+00_JPRB, 0.11008E+01_JPRB, 0.11387E+01_JPRB, 0.11228E+01_JPRB, 0.11046E+01_JPRB ,&
&  0.10927E+01_JPRB, 0.10719E+01_JPRB, 0.10427E+01_JPRB, 0.10168E+01_JPRB, 0.92886E+00_JPRB /)
ZBURD(:, 54, 11) = (/ &
&  0.91078E+00_JPRB, 0.89776E+00_JPRB, 0.87300E+00_JPRB, 0.84873E+00_JPRB, 0.83812E+00_JPRB, 0.83418E+00_JPRB, 0.83088E+00_JPRB ,&
&  0.82068E+00_JPRB, 0.80651E+00_JPRB, 0.79034E+00_JPRB, 0.77235E+00_JPRB, 0.75347E+00_JPRB, 0.73757E+00_JPRB, 0.72272E+00_JPRB ,&
&  0.70076E+00_JPRB, 0.68762E+00_JPRB, 0.68576E+00_JPRB, 0.68244E+00_JPRB, 0.67969E+00_JPRB, 0.67534E+00_JPRB, 0.67548E+00_JPRB ,&
&  0.67802E+00_JPRB, 0.67871E+00_JPRB, 0.67821E+00_JPRB, 0.67068E+00_JPRB, 0.66448E+00_JPRB, 0.66356E+00_JPRB, 0.65860E+00_JPRB ,&
&  0.65916E+00_JPRB, 0.56450E+00_JPRB, 0.64188E+00_JPRB, 0.74108E+00_JPRB, 0.79551E+00_JPRB, 0.87112E+00_JPRB, 0.96107E+00_JPRB ,&
&  0.99761E+00_JPRB, 0.97439E+00_JPRB, 0.93145E+00_JPRB, 0.89363E+00_JPRB, 0.85479E+00_JPRB, 0.82169E+00_JPRB, 0.79472E+00_JPRB ,&
&  0.77764E+00_JPRB, 0.76712E+00_JPRB, 0.75794E+00_JPRB, 0.74640E+00_JPRB, 0.73425E+00_JPRB, 0.72798E+00_JPRB, 0.72389E+00_JPRB ,&
&  0.72280E+00_JPRB, 0.72569E+00_JPRB, 0.72709E+00_JPRB, 0.73368E+00_JPRB, 0.73942E+00_JPRB, 0.75439E+00_JPRB, 0.78641E+00_JPRB ,&
&  0.80393E+00_JPRB, 0.82673E+00_JPRB, 0.86189E+00_JPRB, 0.87211E+00_JPRB, 0.86019E+00_JPRB, 0.84914E+00_JPRB, 0.83503E+00_JPRB ,&
&  0.81702E+00_JPRB, 0.79030E+00_JPRB, 0.77979E+00_JPRB, 0.76993E+00_JPRB, 0.76151E+00_JPRB, 0.75783E+00_JPRB, 0.76538E+00_JPRB ,&
&  0.77349E+00_JPRB, 0.77593E+00_JPRB, 0.78096E+00_JPRB, 0.78946E+00_JPRB, 0.79422E+00_JPRB, 0.79219E+00_JPRB, 0.79510E+00_JPRB ,&
&  0.80682E+00_JPRB, 0.82285E+00_JPRB, 0.82969E+00_JPRB, 0.82986E+00_JPRB, 0.83837E+00_JPRB, 0.85648E+00_JPRB, 0.87127E+00_JPRB ,&
&  0.88284E+00_JPRB, 0.89388E+00_JPRB, 0.90967E+00_JPRB, 0.97993E+00_JPRB, 0.10569E+01_JPRB, 0.10468E+01_JPRB, 0.10194E+01_JPRB ,&
&  0.10004E+01_JPRB, 0.97793E+00_JPRB, 0.96916E+00_JPRB, 0.82911E+00_JPRB, 0.88263E+00_JPRB /)
ZBURD(:, 55, 11) = (/ &
&  0.82911E+00_JPRB, 0.82032E+00_JPRB, 0.80204E+00_JPRB, 0.78215E+00_JPRB, 0.76806E+00_JPRB, 0.76252E+00_JPRB, 0.76117E+00_JPRB ,&
&  0.76559E+00_JPRB, 0.76894E+00_JPRB, 0.76841E+00_JPRB, 0.76304E+00_JPRB, 0.75480E+00_JPRB, 0.74258E+00_JPRB, 0.73165E+00_JPRB ,&
&  0.71531E+00_JPRB, 0.69855E+00_JPRB, 0.69202E+00_JPRB, 0.69093E+00_JPRB, 0.68808E+00_JPRB, 0.68051E+00_JPRB, 0.67562E+00_JPRB ,&
&  0.67306E+00_JPRB, 0.67444E+00_JPRB, 0.66727E+00_JPRB, 0.66250E+00_JPRB, 0.65673E+00_JPRB, 0.65785E+00_JPRB, 0.65820E+00_JPRB ,&
&  0.65685E+00_JPRB, 0.53429E+00_JPRB, 0.61154E+00_JPRB, 0.67560E+00_JPRB, 0.70548E+00_JPRB, 0.76434E+00_JPRB, 0.82522E+00_JPRB ,&
&  0.86701E+00_JPRB, 0.88482E+00_JPRB, 0.87602E+00_JPRB, 0.85613E+00_JPRB, 0.82666E+00_JPRB, 0.79916E+00_JPRB, 0.77520E+00_JPRB ,&
&  0.75266E+00_JPRB, 0.73555E+00_JPRB, 0.72067E+00_JPRB, 0.70693E+00_JPRB, 0.69845E+00_JPRB, 0.68991E+00_JPRB, 0.68238E+00_JPRB ,&
&  0.68162E+00_JPRB, 0.68736E+00_JPRB, 0.68742E+00_JPRB, 0.68990E+00_JPRB, 0.69395E+00_JPRB, 0.69710E+00_JPRB, 0.71024E+00_JPRB ,&
&  0.72076E+00_JPRB, 0.72686E+00_JPRB, 0.73370E+00_JPRB, 0.73641E+00_JPRB, 0.73119E+00_JPRB, 0.73009E+00_JPRB, 0.72572E+00_JPRB ,&
&  0.72059E+00_JPRB, 0.71047E+00_JPRB, 0.70072E+00_JPRB, 0.69419E+00_JPRB, 0.69066E+00_JPRB, 0.68813E+00_JPRB, 0.69231E+00_JPRB ,&
&  0.70126E+00_JPRB, 0.70215E+00_JPRB, 0.70275E+00_JPRB, 0.70659E+00_JPRB, 0.71046E+00_JPRB, 0.71407E+00_JPRB, 0.72073E+00_JPRB ,&
&  0.73376E+00_JPRB, 0.75277E+00_JPRB, 0.76999E+00_JPRB, 0.77776E+00_JPRB, 0.78089E+00_JPRB, 0.79014E+00_JPRB, 0.80173E+00_JPRB ,&
&  0.81372E+00_JPRB, 0.82076E+00_JPRB, 0.82589E+00_JPRB, 0.88766E+00_JPRB, 0.91356E+00_JPRB, 0.91645E+00_JPRB, 0.91102E+00_JPRB ,&
&  0.88899E+00_JPRB, 0.88090E+00_JPRB, 0.71372E+00_JPRB, 0.78669E+00_JPRB, 0.80875E+00_JPRB /)
ZBURD(:, 56, 11) = (/ &
&  0.76579E+00_JPRB, 0.75491E+00_JPRB, 0.73697E+00_JPRB, 0.72102E+00_JPRB, 0.70699E+00_JPRB, 0.70043E+00_JPRB, 0.69664E+00_JPRB ,&
&  0.70130E+00_JPRB, 0.71052E+00_JPRB, 0.71939E+00_JPRB, 0.72409E+00_JPRB, 0.72724E+00_JPRB, 0.72634E+00_JPRB, 0.72118E+00_JPRB ,&
&  0.71636E+00_JPRB, 0.70305E+00_JPRB, 0.69233E+00_JPRB, 0.69051E+00_JPRB, 0.68567E+00_JPRB, 0.68021E+00_JPRB, 0.67384E+00_JPRB ,&
&  0.66780E+00_JPRB, 0.66269E+00_JPRB, 0.65230E+00_JPRB, 0.64714E+00_JPRB, 0.64562E+00_JPRB, 0.64757E+00_JPRB, 0.65709E+00_JPRB ,&
&  0.58155E+00_JPRB, 0.50321E+00_JPRB, 0.56721E+00_JPRB, 0.63550E+00_JPRB, 0.66931E+00_JPRB, 0.71054E+00_JPRB, 0.74758E+00_JPRB ,&
&  0.79623E+00_JPRB, 0.82711E+00_JPRB, 0.84412E+00_JPRB, 0.83938E+00_JPRB, 0.82148E+00_JPRB, 0.79989E+00_JPRB, 0.77695E+00_JPRB ,&
&  0.74854E+00_JPRB, 0.71604E+00_JPRB, 0.69444E+00_JPRB, 0.68332E+00_JPRB, 0.67407E+00_JPRB, 0.66426E+00_JPRB, 0.65771E+00_JPRB ,&
&  0.65466E+00_JPRB, 0.65261E+00_JPRB, 0.65029E+00_JPRB, 0.64884E+00_JPRB, 0.65063E+00_JPRB, 0.65327E+00_JPRB, 0.65735E+00_JPRB ,&
&  0.66033E+00_JPRB, 0.66241E+00_JPRB, 0.66253E+00_JPRB, 0.65939E+00_JPRB, 0.65743E+00_JPRB, 0.66097E+00_JPRB, 0.66159E+00_JPRB ,&
&  0.66238E+00_JPRB, 0.66085E+00_JPRB, 0.65922E+00_JPRB, 0.65755E+00_JPRB, 0.65421E+00_JPRB, 0.65128E+00_JPRB, 0.65116E+00_JPRB ,&
&  0.65426E+00_JPRB, 0.65292E+00_JPRB, 0.65473E+00_JPRB, 0.65961E+00_JPRB, 0.66533E+00_JPRB, 0.67106E+00_JPRB, 0.67537E+00_JPRB ,&
&  0.68310E+00_JPRB, 0.69753E+00_JPRB, 0.70858E+00_JPRB, 0.71596E+00_JPRB, 0.72163E+00_JPRB, 0.72794E+00_JPRB, 0.73711E+00_JPRB ,&
&  0.74531E+00_JPRB, 0.75326E+00_JPRB, 0.76224E+00_JPRB, 0.78748E+00_JPRB, 0.80112E+00_JPRB, 0.80484E+00_JPRB, 0.80446E+00_JPRB ,&
&  0.79543E+00_JPRB, 0.77828E+00_JPRB, 0.77778E+00_JPRB, 0.77267E+00_JPRB, 0.76840E+00_JPRB /)
ZBURD(:, 57, 11) = (/ &
&  0.67013E+00_JPRB, 0.66256E+00_JPRB, 0.64990E+00_JPRB, 0.64177E+00_JPRB, 0.63732E+00_JPRB, 0.63862E+00_JPRB, 0.64146E+00_JPRB ,&
&  0.64547E+00_JPRB, 0.64695E+00_JPRB, 0.65477E+00_JPRB, 0.66330E+00_JPRB, 0.66881E+00_JPRB, 0.67267E+00_JPRB, 0.67188E+00_JPRB ,&
&  0.67133E+00_JPRB, 0.67046E+00_JPRB, 0.66465E+00_JPRB, 0.65919E+00_JPRB, 0.65230E+00_JPRB, 0.65175E+00_JPRB, 0.65057E+00_JPRB ,&
&  0.64807E+00_JPRB, 0.64384E+00_JPRB, 0.63648E+00_JPRB, 0.62702E+00_JPRB, 0.62251E+00_JPRB, 0.62140E+00_JPRB, 0.61866E+00_JPRB ,&
&  0.62474E+00_JPRB, 0.54440E+00_JPRB, 0.58329E+00_JPRB, 0.60417E+00_JPRB, 0.63196E+00_JPRB, 0.66246E+00_JPRB, 0.69471E+00_JPRB ,&
&  0.72763E+00_JPRB, 0.75443E+00_JPRB, 0.76483E+00_JPRB, 0.76580E+00_JPRB, 0.75199E+00_JPRB, 0.72842E+00_JPRB, 0.70160E+00_JPRB ,&
&  0.67889E+00_JPRB, 0.66172E+00_JPRB, 0.64229E+00_JPRB, 0.63073E+00_JPRB, 0.62134E+00_JPRB, 0.60945E+00_JPRB, 0.59913E+00_JPRB ,&
&  0.59409E+00_JPRB, 0.59087E+00_JPRB, 0.59018E+00_JPRB, 0.59180E+00_JPRB, 0.59469E+00_JPRB, 0.59692E+00_JPRB, 0.59708E+00_JPRB ,&
&  0.59677E+00_JPRB, 0.59592E+00_JPRB, 0.59336E+00_JPRB, 0.59308E+00_JPRB, 0.59411E+00_JPRB, 0.59754E+00_JPRB, 0.59525E+00_JPRB ,&
&  0.58930E+00_JPRB, 0.59068E+00_JPRB, 0.59347E+00_JPRB, 0.59465E+00_JPRB, 0.59642E+00_JPRB, 0.59828E+00_JPRB, 0.59760E+00_JPRB ,&
&  0.59702E+00_JPRB, 0.60031E+00_JPRB, 0.60403E+00_JPRB, 0.61066E+00_JPRB, 0.61550E+00_JPRB, 0.62487E+00_JPRB, 0.63299E+00_JPRB ,&
&  0.64111E+00_JPRB, 0.65348E+00_JPRB, 0.66531E+00_JPRB, 0.67370E+00_JPRB, 0.67905E+00_JPRB, 0.68414E+00_JPRB, 0.68964E+00_JPRB ,&
&  0.69410E+00_JPRB, 0.69833E+00_JPRB, 0.70226E+00_JPRB, 0.70667E+00_JPRB, 0.70627E+00_JPRB, 0.70498E+00_JPRB, 0.70488E+00_JPRB ,&
&  0.69837E+00_JPRB, 0.68673E+00_JPRB, 0.68265E+00_JPRB, 0.67819E+00_JPRB, 0.67318E+00_JPRB /)
ZBURD(:, 58, 11) = (/ &
&  0.60651E+00_JPRB, 0.60100E+00_JPRB, 0.59516E+00_JPRB, 0.59382E+00_JPRB, 0.59202E+00_JPRB, 0.59311E+00_JPRB, 0.59516E+00_JPRB ,&
&  0.60140E+00_JPRB, 0.60538E+00_JPRB, 0.61454E+00_JPRB, 0.62226E+00_JPRB, 0.62626E+00_JPRB, 0.63108E+00_JPRB, 0.63416E+00_JPRB ,&
&  0.63529E+00_JPRB, 0.63660E+00_JPRB, 0.63697E+00_JPRB, 0.64085E+00_JPRB, 0.64114E+00_JPRB, 0.64002E+00_JPRB, 0.63860E+00_JPRB ,&
&  0.63609E+00_JPRB, 0.63322E+00_JPRB, 0.62731E+00_JPRB, 0.61947E+00_JPRB, 0.60982E+00_JPRB, 0.60528E+00_JPRB, 0.59867E+00_JPRB ,&
&  0.59934E+00_JPRB, 0.57074E+00_JPRB, 0.58294E+00_JPRB, 0.58148E+00_JPRB, 0.59866E+00_JPRB, 0.62558E+00_JPRB, 0.65995E+00_JPRB ,&
&  0.68297E+00_JPRB, 0.69434E+00_JPRB, 0.69493E+00_JPRB, 0.69123E+00_JPRB, 0.67578E+00_JPRB, 0.65993E+00_JPRB, 0.64204E+00_JPRB ,&
&  0.62175E+00_JPRB, 0.61014E+00_JPRB, 0.59595E+00_JPRB, 0.58554E+00_JPRB, 0.58111E+00_JPRB, 0.57790E+00_JPRB, 0.57261E+00_JPRB ,&
&  0.56461E+00_JPRB, 0.55765E+00_JPRB, 0.55796E+00_JPRB, 0.56138E+00_JPRB, 0.56509E+00_JPRB, 0.56746E+00_JPRB, 0.56646E+00_JPRB ,&
&  0.56028E+00_JPRB, 0.55508E+00_JPRB, 0.55287E+00_JPRB, 0.55276E+00_JPRB, 0.54883E+00_JPRB, 0.54169E+00_JPRB, 0.53426E+00_JPRB ,&
&  0.53254E+00_JPRB, 0.53530E+00_JPRB, 0.53909E+00_JPRB, 0.54387E+00_JPRB, 0.54733E+00_JPRB, 0.55150E+00_JPRB, 0.55344E+00_JPRB ,&
&  0.55811E+00_JPRB, 0.56264E+00_JPRB, 0.57188E+00_JPRB, 0.58080E+00_JPRB, 0.58317E+00_JPRB, 0.59203E+00_JPRB, 0.60122E+00_JPRB ,&
&  0.61042E+00_JPRB, 0.62113E+00_JPRB, 0.62950E+00_JPRB, 0.63533E+00_JPRB, 0.64144E+00_JPRB, 0.64521E+00_JPRB, 0.65121E+00_JPRB ,&
&  0.65911E+00_JPRB, 0.66131E+00_JPRB, 0.66714E+00_JPRB, 0.66894E+00_JPRB, 0.66532E+00_JPRB, 0.65895E+00_JPRB, 0.65057E+00_JPRB ,&
&  0.64049E+00_JPRB, 0.62483E+00_JPRB, 0.61631E+00_JPRB, 0.61120E+00_JPRB, 0.60826E+00_JPRB /)
ZBURD(:, 59, 11) = (/ &
&  0.53349E+00_JPRB, 0.53159E+00_JPRB, 0.53201E+00_JPRB, 0.53509E+00_JPRB, 0.53844E+00_JPRB, 0.54100E+00_JPRB, 0.54654E+00_JPRB ,&
&  0.55626E+00_JPRB, 0.56385E+00_JPRB, 0.57326E+00_JPRB, 0.58293E+00_JPRB, 0.59049E+00_JPRB, 0.59564E+00_JPRB, 0.59847E+00_JPRB ,&
&  0.59681E+00_JPRB, 0.59418E+00_JPRB, 0.59710E+00_JPRB, 0.60172E+00_JPRB, 0.60306E+00_JPRB, 0.60251E+00_JPRB, 0.60232E+00_JPRB ,&
&  0.60152E+00_JPRB, 0.59884E+00_JPRB, 0.59535E+00_JPRB, 0.59039E+00_JPRB, 0.58470E+00_JPRB, 0.58176E+00_JPRB, 0.57913E+00_JPRB ,&
&  0.57205E+00_JPRB, 0.57263E+00_JPRB, 0.52222E+00_JPRB, 0.53711E+00_JPRB, 0.54967E+00_JPRB, 0.57079E+00_JPRB, 0.59168E+00_JPRB ,&
&  0.60628E+00_JPRB, 0.61499E+00_JPRB, 0.61905E+00_JPRB, 0.61689E+00_JPRB, 0.60410E+00_JPRB, 0.59622E+00_JPRB, 0.58745E+00_JPRB ,&
&  0.57618E+00_JPRB, 0.56808E+00_JPRB, 0.55464E+00_JPRB, 0.54024E+00_JPRB, 0.53528E+00_JPRB, 0.53569E+00_JPRB, 0.53562E+00_JPRB ,&
&  0.53167E+00_JPRB, 0.52788E+00_JPRB, 0.52906E+00_JPRB, 0.53000E+00_JPRB, 0.52984E+00_JPRB, 0.52463E+00_JPRB, 0.51801E+00_JPRB ,&
&  0.50760E+00_JPRB, 0.49733E+00_JPRB, 0.49020E+00_JPRB, 0.48496E+00_JPRB, 0.48033E+00_JPRB, 0.47518E+00_JPRB, 0.47025E+00_JPRB ,&
&  0.47393E+00_JPRB, 0.48532E+00_JPRB, 0.49601E+00_JPRB, 0.50487E+00_JPRB, 0.51060E+00_JPRB, 0.51443E+00_JPRB, 0.51960E+00_JPRB ,&
&  0.52561E+00_JPRB, 0.53584E+00_JPRB, 0.54304E+00_JPRB, 0.54950E+00_JPRB, 0.55441E+00_JPRB, 0.55905E+00_JPRB, 0.56438E+00_JPRB ,&
&  0.56880E+00_JPRB, 0.57597E+00_JPRB, 0.58136E+00_JPRB, 0.58779E+00_JPRB, 0.59162E+00_JPRB, 0.59694E+00_JPRB, 0.60323E+00_JPRB ,&
&  0.61024E+00_JPRB, 0.61646E+00_JPRB, 0.61918E+00_JPRB, 0.61854E+00_JPRB, 0.61179E+00_JPRB, 0.60270E+00_JPRB, 0.59400E+00_JPRB ,&
&  0.58246E+00_JPRB, 0.56808E+00_JPRB, 0.55314E+00_JPRB, 0.54360E+00_JPRB, 0.53578E+00_JPRB /)
ZBURD(:, 60, 11) = (/ &
&  0.48012E+00_JPRB, 0.48135E+00_JPRB, 0.48640E+00_JPRB, 0.49121E+00_JPRB, 0.49464E+00_JPRB, 0.49948E+00_JPRB, 0.50906E+00_JPRB ,&
&  0.51868E+00_JPRB, 0.52786E+00_JPRB, 0.53415E+00_JPRB, 0.54182E+00_JPRB, 0.55287E+00_JPRB, 0.56038E+00_JPRB, 0.56633E+00_JPRB ,&
&  0.56883E+00_JPRB, 0.56922E+00_JPRB, 0.57024E+00_JPRB, 0.57331E+00_JPRB, 0.57279E+00_JPRB, 0.56908E+00_JPRB, 0.56770E+00_JPRB ,&
&  0.56742E+00_JPRB, 0.56557E+00_JPRB, 0.56510E+00_JPRB, 0.56592E+00_JPRB, 0.56331E+00_JPRB, 0.55923E+00_JPRB, 0.55849E+00_JPRB ,&
&  0.55838E+00_JPRB, 0.54465E+00_JPRB, 0.53692E+00_JPRB, 0.52977E+00_JPRB, 0.53321E+00_JPRB, 0.54442E+00_JPRB, 0.55685E+00_JPRB ,&
&  0.56431E+00_JPRB, 0.56934E+00_JPRB, 0.56914E+00_JPRB, 0.56826E+00_JPRB, 0.56358E+00_JPRB, 0.55217E+00_JPRB, 0.54262E+00_JPRB ,&
&  0.53617E+00_JPRB, 0.52680E+00_JPRB, 0.50414E+00_JPRB, 0.49089E+00_JPRB, 0.48729E+00_JPRB, 0.48995E+00_JPRB, 0.49412E+00_JPRB ,&
&  0.49583E+00_JPRB, 0.49324E+00_JPRB, 0.48704E+00_JPRB, 0.47976E+00_JPRB, 0.46998E+00_JPRB, 0.45871E+00_JPRB, 0.44881E+00_JPRB ,&
&  0.44319E+00_JPRB, 0.43898E+00_JPRB, 0.43548E+00_JPRB, 0.43805E+00_JPRB, 0.43732E+00_JPRB, 0.43340E+00_JPRB, 0.43534E+00_JPRB ,&
&  0.44681E+00_JPRB, 0.46135E+00_JPRB, 0.47471E+00_JPRB, 0.48204E+00_JPRB, 0.48501E+00_JPRB, 0.48840E+00_JPRB, 0.49239E+00_JPRB ,&
&  0.49821E+00_JPRB, 0.50392E+00_JPRB, 0.50905E+00_JPRB, 0.51571E+00_JPRB, 0.52401E+00_JPRB, 0.53161E+00_JPRB, 0.53493E+00_JPRB ,&
&  0.53776E+00_JPRB, 0.53952E+00_JPRB, 0.54093E+00_JPRB, 0.54630E+00_JPRB, 0.55435E+00_JPRB, 0.56009E+00_JPRB, 0.56483E+00_JPRB ,&
&  0.56973E+00_JPRB, 0.57214E+00_JPRB, 0.57102E+00_JPRB, 0.56746E+00_JPRB, 0.56213E+00_JPRB, 0.55142E+00_JPRB, 0.53798E+00_JPRB ,&
&  0.52233E+00_JPRB, 0.50781E+00_JPRB, 0.49689E+00_JPRB, 0.48764E+00_JPRB, 0.48142E+00_JPRB /)
ZBURD(:, 61, 11) = (/ &
&  0.44052E+00_JPRB, 0.44077E+00_JPRB, 0.44393E+00_JPRB, 0.45041E+00_JPRB, 0.45468E+00_JPRB, 0.45852E+00_JPRB, 0.46628E+00_JPRB ,&
&  0.47340E+00_JPRB, 0.47967E+00_JPRB, 0.48713E+00_JPRB, 0.49526E+00_JPRB, 0.50163E+00_JPRB, 0.51082E+00_JPRB, 0.51957E+00_JPRB ,&
&  0.52522E+00_JPRB, 0.52752E+00_JPRB, 0.52886E+00_JPRB, 0.52999E+00_JPRB, 0.52983E+00_JPRB, 0.52726E+00_JPRB, 0.52673E+00_JPRB ,&
&  0.52480E+00_JPRB, 0.52265E+00_JPRB, 0.52469E+00_JPRB, 0.52491E+00_JPRB, 0.52641E+00_JPRB, 0.52899E+00_JPRB, 0.52891E+00_JPRB ,&
&  0.52660E+00_JPRB, 0.51338E+00_JPRB, 0.50844E+00_JPRB, 0.50537E+00_JPRB, 0.50951E+00_JPRB, 0.51047E+00_JPRB, 0.50894E+00_JPRB ,&
&  0.50245E+00_JPRB, 0.49845E+00_JPRB, 0.50024E+00_JPRB, 0.50258E+00_JPRB, 0.49682E+00_JPRB, 0.48380E+00_JPRB, 0.47360E+00_JPRB ,&
&  0.46565E+00_JPRB, 0.45345E+00_JPRB, 0.43557E+00_JPRB, 0.42485E+00_JPRB, 0.42023E+00_JPRB, 0.41533E+00_JPRB, 0.41093E+00_JPRB ,&
&  0.40862E+00_JPRB, 0.40784E+00_JPRB, 0.40919E+00_JPRB, 0.40837E+00_JPRB, 0.40478E+00_JPRB, 0.40043E+00_JPRB, 0.39878E+00_JPRB ,&
&  0.40150E+00_JPRB, 0.40392E+00_JPRB, 0.40745E+00_JPRB, 0.41015E+00_JPRB, 0.40962E+00_JPRB, 0.40996E+00_JPRB, 0.41368E+00_JPRB ,&
&  0.41948E+00_JPRB, 0.42498E+00_JPRB, 0.43052E+00_JPRB, 0.43289E+00_JPRB, 0.43605E+00_JPRB, 0.44279E+00_JPRB, 0.44333E+00_JPRB ,&
&  0.44183E+00_JPRB, 0.44374E+00_JPRB, 0.44789E+00_JPRB, 0.45588E+00_JPRB, 0.46823E+00_JPRB, 0.47649E+00_JPRB, 0.47794E+00_JPRB ,&
&  0.47825E+00_JPRB, 0.48066E+00_JPRB, 0.48463E+00_JPRB, 0.49407E+00_JPRB, 0.50440E+00_JPRB, 0.51184E+00_JPRB, 0.51503E+00_JPRB ,&
&  0.51207E+00_JPRB, 0.50549E+00_JPRB, 0.49353E+00_JPRB, 0.47964E+00_JPRB, 0.46711E+00_JPRB, 0.45735E+00_JPRB, 0.44931E+00_JPRB ,&
&  0.44123E+00_JPRB, 0.43751E+00_JPRB, 0.43694E+00_JPRB, 0.43875E+00_JPRB, 0.43991E+00_JPRB /)
ZBURD(:, 62, 11) = (/ &
&  0.41010E+00_JPRB, 0.41212E+00_JPRB, 0.41458E+00_JPRB, 0.41774E+00_JPRB, 0.42113E+00_JPRB, 0.42318E+00_JPRB, 0.42706E+00_JPRB ,&
&  0.42928E+00_JPRB, 0.43584E+00_JPRB, 0.44656E+00_JPRB, 0.45729E+00_JPRB, 0.46513E+00_JPRB, 0.47184E+00_JPRB, 0.47844E+00_JPRB ,&
&  0.48384E+00_JPRB, 0.49098E+00_JPRB, 0.49196E+00_JPRB, 0.48959E+00_JPRB, 0.48696E+00_JPRB, 0.48492E+00_JPRB, 0.48306E+00_JPRB ,&
&  0.48047E+00_JPRB, 0.48025E+00_JPRB, 0.48392E+00_JPRB, 0.48807E+00_JPRB, 0.49230E+00_JPRB, 0.49337E+00_JPRB, 0.49186E+00_JPRB ,&
&  0.49531E+00_JPRB, 0.48563E+00_JPRB, 0.47928E+00_JPRB, 0.47189E+00_JPRB, 0.46541E+00_JPRB, 0.45388E+00_JPRB, 0.44537E+00_JPRB ,&
&  0.43528E+00_JPRB, 0.42639E+00_JPRB, 0.42256E+00_JPRB, 0.41849E+00_JPRB, 0.40880E+00_JPRB, 0.39756E+00_JPRB, 0.38929E+00_JPRB ,&
&  0.38384E+00_JPRB, 0.37831E+00_JPRB, 0.37164E+00_JPRB, 0.36612E+00_JPRB, 0.36156E+00_JPRB, 0.35992E+00_JPRB, 0.36163E+00_JPRB ,&
&  0.36582E+00_JPRB, 0.36975E+00_JPRB, 0.37211E+00_JPRB, 0.37270E+00_JPRB, 0.37113E+00_JPRB, 0.36858E+00_JPRB, 0.36711E+00_JPRB ,&
&  0.36773E+00_JPRB, 0.36915E+00_JPRB, 0.37127E+00_JPRB, 0.37647E+00_JPRB, 0.37724E+00_JPRB, 0.37559E+00_JPRB, 0.36989E+00_JPRB ,&
&  0.36738E+00_JPRB, 0.36610E+00_JPRB, 0.36772E+00_JPRB, 0.37282E+00_JPRB, 0.38128E+00_JPRB, 0.38950E+00_JPRB, 0.39540E+00_JPRB ,&
&  0.39835E+00_JPRB, 0.40012E+00_JPRB, 0.40327E+00_JPRB, 0.40702E+00_JPRB, 0.40866E+00_JPRB, 0.40815E+00_JPRB, 0.40679E+00_JPRB ,&
&  0.40972E+00_JPRB, 0.41728E+00_JPRB, 0.42523E+00_JPRB, 0.43034E+00_JPRB, 0.43293E+00_JPRB, 0.43206E+00_JPRB, 0.42532E+00_JPRB ,&
&  0.41299E+00_JPRB, 0.39876E+00_JPRB, 0.38493E+00_JPRB, 0.37323E+00_JPRB, 0.36509E+00_JPRB, 0.36125E+00_JPRB, 0.36417E+00_JPRB ,&
&  0.37188E+00_JPRB, 0.38177E+00_JPRB, 0.39080E+00_JPRB, 0.39753E+00_JPRB, 0.40493E+00_JPRB /)
ZBURD(:, 63, 11) = (/ &
&  0.38072E+00_JPRB, 0.38063E+00_JPRB, 0.38230E+00_JPRB, 0.38140E+00_JPRB, 0.37888E+00_JPRB, 0.37697E+00_JPRB, 0.38053E+00_JPRB ,&
&  0.38739E+00_JPRB, 0.39903E+00_JPRB, 0.41450E+00_JPRB, 0.42811E+00_JPRB, 0.43634E+00_JPRB, 0.44139E+00_JPRB, 0.44912E+00_JPRB ,&
&  0.45635E+00_JPRB, 0.46440E+00_JPRB, 0.46471E+00_JPRB, 0.45597E+00_JPRB, 0.44855E+00_JPRB, 0.44463E+00_JPRB, 0.44413E+00_JPRB ,&
&  0.44346E+00_JPRB, 0.44519E+00_JPRB, 0.44832E+00_JPRB, 0.45164E+00_JPRB, 0.45085E+00_JPRB, 0.44884E+00_JPRB, 0.45224E+00_JPRB ,&
&  0.45005E+00_JPRB, 0.44020E+00_JPRB, 0.43017E+00_JPRB, 0.40510E+00_JPRB, 0.40430E+00_JPRB, 0.39387E+00_JPRB, 0.38585E+00_JPRB ,&
&  0.37510E+00_JPRB, 0.36183E+00_JPRB, 0.35218E+00_JPRB, 0.34598E+00_JPRB, 0.34319E+00_JPRB, 0.34065E+00_JPRB, 0.33864E+00_JPRB ,&
&  0.33735E+00_JPRB, 0.33718E+00_JPRB, 0.33858E+00_JPRB, 0.33629E+00_JPRB, 0.33514E+00_JPRB, 0.33592E+00_JPRB, 0.33767E+00_JPRB ,&
&  0.33758E+00_JPRB, 0.33746E+00_JPRB, 0.33740E+00_JPRB, 0.33652E+00_JPRB, 0.33487E+00_JPRB, 0.33242E+00_JPRB, 0.33014E+00_JPRB ,&
&  0.32923E+00_JPRB, 0.33030E+00_JPRB, 0.33068E+00_JPRB, 0.32684E+00_JPRB, 0.31921E+00_JPRB, 0.30983E+00_JPRB, 0.30374E+00_JPRB ,&
&  0.30115E+00_JPRB, 0.30321E+00_JPRB, 0.31107E+00_JPRB, 0.32226E+00_JPRB, 0.33266E+00_JPRB, 0.33890E+00_JPRB, 0.34282E+00_JPRB ,&
&  0.34736E+00_JPRB, 0.34562E+00_JPRB, 0.34194E+00_JPRB, 0.33871E+00_JPRB, 0.33575E+00_JPRB, 0.33169E+00_JPRB, 0.33164E+00_JPRB ,&
&  0.33432E+00_JPRB, 0.33700E+00_JPRB, 0.33786E+00_JPRB, 0.33674E+00_JPRB, 0.33469E+00_JPRB, 0.33043E+00_JPRB, 0.32447E+00_JPRB ,&
&  0.31706E+00_JPRB, 0.30844E+00_JPRB, 0.30643E+00_JPRB, 0.30705E+00_JPRB, 0.31087E+00_JPRB, 0.31872E+00_JPRB, 0.32944E+00_JPRB ,&
&  0.34067E+00_JPRB, 0.34987E+00_JPRB, 0.35978E+00_JPRB, 0.36895E+00_JPRB, 0.37715E+00_JPRB /)
ZBURD(:, 64, 11) = (/ &
&  0.36212E+00_JPRB, 0.36360E+00_JPRB, 0.36110E+00_JPRB, 0.35835E+00_JPRB, 0.35813E+00_JPRB, 0.35988E+00_JPRB, 0.36562E+00_JPRB ,&
&  0.37533E+00_JPRB, 0.38789E+00_JPRB, 0.39997E+00_JPRB, 0.41003E+00_JPRB, 0.41910E+00_JPRB, 0.42309E+00_JPRB, 0.42910E+00_JPRB ,&
&  0.43561E+00_JPRB, 0.44161E+00_JPRB, 0.44250E+00_JPRB, 0.43232E+00_JPRB, 0.42240E+00_JPRB, 0.41713E+00_JPRB, 0.41716E+00_JPRB ,&
&  0.41897E+00_JPRB, 0.42159E+00_JPRB, 0.42771E+00_JPRB, 0.42990E+00_JPRB, 0.42387E+00_JPRB, 0.42227E+00_JPRB, 0.41995E+00_JPRB ,&
&  0.41422E+00_JPRB, 0.40261E+00_JPRB, 0.37008E+00_JPRB, 0.36430E+00_JPRB, 0.35598E+00_JPRB, 0.34686E+00_JPRB, 0.33780E+00_JPRB ,&
&  0.32750E+00_JPRB, 0.31715E+00_JPRB, 0.31008E+00_JPRB, 0.30593E+00_JPRB, 0.30660E+00_JPRB, 0.30782E+00_JPRB, 0.30735E+00_JPRB ,&
&  0.30944E+00_JPRB, 0.31209E+00_JPRB, 0.31439E+00_JPRB, 0.31469E+00_JPRB, 0.31427E+00_JPRB, 0.31317E+00_JPRB, 0.30870E+00_JPRB ,&
&  0.30468E+00_JPRB, 0.30257E+00_JPRB, 0.30089E+00_JPRB, 0.29944E+00_JPRB, 0.29926E+00_JPRB, 0.29977E+00_JPRB, 0.29998E+00_JPRB ,&
&  0.29780E+00_JPRB, 0.29421E+00_JPRB, 0.29014E+00_JPRB, 0.28312E+00_JPRB, 0.27095E+00_JPRB, 0.25559E+00_JPRB, 0.24614E+00_JPRB ,&
&  0.24463E+00_JPRB, 0.25544E+00_JPRB, 0.27666E+00_JPRB, 0.28578E+00_JPRB, 0.29317E+00_JPRB, 0.29713E+00_JPRB, 0.30158E+00_JPRB ,&
&  0.29687E+00_JPRB, 0.27291E+00_JPRB, 0.25980E+00_JPRB, 0.25282E+00_JPRB, 0.24734E+00_JPRB, 0.24321E+00_JPRB, 0.24301E+00_JPRB ,&
&  0.24628E+00_JPRB, 0.25274E+00_JPRB, 0.25920E+00_JPRB, 0.26232E+00_JPRB, 0.26158E+00_JPRB, 0.25832E+00_JPRB, 0.25492E+00_JPRB ,&
&  0.25216E+00_JPRB, 0.25674E+00_JPRB, 0.27143E+00_JPRB, 0.29344E+00_JPRB, 0.30162E+00_JPRB, 0.30795E+00_JPRB, 0.31380E+00_JPRB ,&
&  0.32010E+00_JPRB, 0.32963E+00_JPRB, 0.34118E+00_JPRB, 0.35028E+00_JPRB, 0.35833E+00_JPRB /)
ZBURD(:, 65, 11) = (/ &
&  0.33503E+00_JPRB, 0.33916E+00_JPRB, 0.34181E+00_JPRB, 0.34529E+00_JPRB, 0.35295E+00_JPRB, 0.36365E+00_JPRB, 0.37576E+00_JPRB ,&
&  0.38695E+00_JPRB, 0.39607E+00_JPRB, 0.40058E+00_JPRB, 0.40576E+00_JPRB, 0.41069E+00_JPRB, 0.41305E+00_JPRB, 0.41826E+00_JPRB ,&
&  0.41906E+00_JPRB, 0.41570E+00_JPRB, 0.40965E+00_JPRB, 0.40166E+00_JPRB, 0.39745E+00_JPRB, 0.39875E+00_JPRB, 0.40164E+00_JPRB ,&
&  0.40349E+00_JPRB, 0.40492E+00_JPRB, 0.40492E+00_JPRB, 0.40141E+00_JPRB, 0.39805E+00_JPRB, 0.39606E+00_JPRB, 0.39195E+00_JPRB ,&
&  0.38260E+00_JPRB, 0.33297E+00_JPRB, 0.29535E+00_JPRB, 0.29112E+00_JPRB, 0.30748E+00_JPRB, 0.30969E+00_JPRB, 0.30458E+00_JPRB ,&
&  0.29886E+00_JPRB, 0.29410E+00_JPRB, 0.29134E+00_JPRB, 0.28917E+00_JPRB, 0.28645E+00_JPRB, 0.28404E+00_JPRB, 0.28477E+00_JPRB ,&
&  0.28605E+00_JPRB, 0.28384E+00_JPRB, 0.28083E+00_JPRB, 0.27455E+00_JPRB, 0.27003E+00_JPRB, 0.26817E+00_JPRB, 0.26664E+00_JPRB ,&
&  0.26243E+00_JPRB, 0.25828E+00_JPRB, 0.25814E+00_JPRB, 0.25932E+00_JPRB, 0.26121E+00_JPRB, 0.26252E+00_JPRB, 0.25987E+00_JPRB ,&
&  0.25412E+00_JPRB, 0.24754E+00_JPRB, 0.23932E+00_JPRB, 0.22946E+00_JPRB, 0.22014E+00_JPRB, 0.21341E+00_JPRB, 0.20806E+00_JPRB ,&
&  0.20711E+00_JPRB, 0.21274E+00_JPRB, 0.22579E+00_JPRB, 0.24122E+00_JPRB, 0.25964E+00_JPRB, 0.24918E+00_JPRB, 0.23836E+00_JPRB ,&
&  0.22785E+00_JPRB, 0.21928E+00_JPRB, 0.21301E+00_JPRB, 0.21022E+00_JPRB, 0.20919E+00_JPRB, 0.20926E+00_JPRB, 0.21047E+00_JPRB ,&
&  0.21262E+00_JPRB, 0.21539E+00_JPRB, 0.21781E+00_JPRB, 0.21896E+00_JPRB, 0.21956E+00_JPRB, 0.22006E+00_JPRB, 0.21989E+00_JPRB ,&
&  0.21919E+00_JPRB, 0.22151E+00_JPRB, 0.22823E+00_JPRB, 0.23680E+00_JPRB, 0.24511E+00_JPRB, 0.25291E+00_JPRB, 0.26586E+00_JPRB ,&
&  0.28174E+00_JPRB, 0.30155E+00_JPRB, 0.31391E+00_JPRB, 0.32333E+00_JPRB, 0.33145E+00_JPRB /)
ZBURD(:, 66, 11) = (/ &
&  0.32887E+00_JPRB, 0.33774E+00_JPRB, 0.34967E+00_JPRB, 0.36002E+00_JPRB, 0.36901E+00_JPRB, 0.37755E+00_JPRB, 0.38498E+00_JPRB ,&
&  0.38901E+00_JPRB, 0.39134E+00_JPRB, 0.39221E+00_JPRB, 0.39573E+00_JPRB, 0.39274E+00_JPRB, 0.38593E+00_JPRB, 0.38045E+00_JPRB ,&
&  0.37590E+00_JPRB, 0.37235E+00_JPRB, 0.36710E+00_JPRB, 0.36436E+00_JPRB, 0.36412E+00_JPRB, 0.36421E+00_JPRB, 0.35855E+00_JPRB ,&
&  0.34805E+00_JPRB, 0.34704E+00_JPRB, 0.35169E+00_JPRB, 0.34626E+00_JPRB, 0.34056E+00_JPRB, 0.33511E+00_JPRB, 0.32925E+00_JPRB ,&
&  0.31332E+00_JPRB, 0.29328E+00_JPRB, 0.26635E+00_JPRB, 0.26232E+00_JPRB, 0.27811E+00_JPRB, 0.27176E+00_JPRB, 0.27491E+00_JPRB ,&
&  0.27548E+00_JPRB, 0.27574E+00_JPRB, 0.27425E+00_JPRB, 0.27267E+00_JPRB, 0.26913E+00_JPRB, 0.26797E+00_JPRB, 0.26648E+00_JPRB ,&
&  0.26085E+00_JPRB, 0.25588E+00_JPRB, 0.24796E+00_JPRB, 0.23027E+00_JPRB, 0.22529E+00_JPRB, 0.22368E+00_JPRB, 0.21889E+00_JPRB ,&
&  0.21128E+00_JPRB, 0.20571E+00_JPRB, 0.20349E+00_JPRB, 0.19970E+00_JPRB, 0.20183E+00_JPRB, 0.20374E+00_JPRB, 0.20370E+00_JPRB ,&
&  0.20363E+00_JPRB, 0.20319E+00_JPRB, 0.20086E+00_JPRB, 0.19698E+00_JPRB, 0.19391E+00_JPRB, 0.19283E+00_JPRB, 0.19360E+00_JPRB ,&
&  0.19595E+00_JPRB, 0.20041E+00_JPRB, 0.21122E+00_JPRB, 0.21961E+00_JPRB, 0.21862E+00_JPRB, 0.21421E+00_JPRB, 0.20531E+00_JPRB ,&
&  0.19757E+00_JPRB, 0.19470E+00_JPRB, 0.19223E+00_JPRB, 0.19096E+00_JPRB, 0.19132E+00_JPRB, 0.19278E+00_JPRB, 0.19358E+00_JPRB ,&
&  0.19468E+00_JPRB, 0.19579E+00_JPRB, 0.19667E+00_JPRB, 0.19721E+00_JPRB, 0.19804E+00_JPRB, 0.19901E+00_JPRB, 0.20081E+00_JPRB ,&
&  0.20342E+00_JPRB, 0.20779E+00_JPRB, 0.21353E+00_JPRB, 0.21970E+00_JPRB, 0.22417E+00_JPRB, 0.22635E+00_JPRB, 0.23031E+00_JPRB ,&
&  0.23618E+00_JPRB, 0.25118E+00_JPRB, 0.27970E+00_JPRB, 0.32465E+00_JPRB, 0.32839E+00_JPRB /)
ZBURD(:, 67, 11) = (/ &
&  0.32909E+00_JPRB, 0.34114E+00_JPRB, 0.35641E+00_JPRB, 0.36588E+00_JPRB, 0.37123E+00_JPRB, 0.37392E+00_JPRB, 0.37285E+00_JPRB ,&
&  0.36947E+00_JPRB, 0.36892E+00_JPRB, 0.37997E+00_JPRB, 0.36197E+00_JPRB, 0.34432E+00_JPRB, 0.31118E+00_JPRB, 0.29473E+00_JPRB ,&
&  0.28235E+00_JPRB, 0.28022E+00_JPRB, 0.27995E+00_JPRB, 0.28465E+00_JPRB, 0.28851E+00_JPRB, 0.29152E+00_JPRB, 0.28773E+00_JPRB ,&
&  0.28238E+00_JPRB, 0.27315E+00_JPRB, 0.26542E+00_JPRB, 0.25819E+00_JPRB, 0.25833E+00_JPRB, 0.25722E+00_JPRB, 0.25463E+00_JPRB ,&
&  0.25168E+00_JPRB, 0.24981E+00_JPRB, 0.24972E+00_JPRB, 0.25003E+00_JPRB, 0.26162E+00_JPRB, 0.26085E+00_JPRB, 0.26095E+00_JPRB ,&
&  0.26060E+00_JPRB, 0.26053E+00_JPRB, 0.25954E+00_JPRB, 0.25783E+00_JPRB, 0.25312E+00_JPRB, 0.23994E+00_JPRB, 0.23920E+00_JPRB ,&
&  0.23004E+00_JPRB, 0.21954E+00_JPRB, 0.20933E+00_JPRB, 0.20225E+00_JPRB, 0.19795E+00_JPRB, 0.19612E+00_JPRB, 0.19151E+00_JPRB ,&
&  0.18657E+00_JPRB, 0.18086E+00_JPRB, 0.17823E+00_JPRB, 0.17802E+00_JPRB, 0.17859E+00_JPRB, 0.17915E+00_JPRB, 0.17962E+00_JPRB ,&
&  0.18087E+00_JPRB, 0.18145E+00_JPRB, 0.18179E+00_JPRB, 0.18246E+00_JPRB, 0.18313E+00_JPRB, 0.18417E+00_JPRB, 0.18597E+00_JPRB ,&
&  0.18915E+00_JPRB, 0.19259E+00_JPRB, 0.19577E+00_JPRB, 0.19847E+00_JPRB, 0.19752E+00_JPRB, 0.19419E+00_JPRB, 0.18977E+00_JPRB ,&
&  0.18656E+00_JPRB, 0.18549E+00_JPRB, 0.18449E+00_JPRB, 0.18442E+00_JPRB, 0.18459E+00_JPRB, 0.18502E+00_JPRB, 0.18564E+00_JPRB ,&
&  0.18647E+00_JPRB, 0.18716E+00_JPRB, 0.18801E+00_JPRB, 0.18912E+00_JPRB, 0.19069E+00_JPRB, 0.19212E+00_JPRB, 0.19412E+00_JPRB ,&
&  0.19710E+00_JPRB, 0.20036E+00_JPRB, 0.20496E+00_JPRB, 0.20907E+00_JPRB, 0.21471E+00_JPRB, 0.22285E+00_JPRB, 0.23626E+00_JPRB ,&
&  0.25068E+00_JPRB, 0.28477E+00_JPRB, 0.30035E+00_JPRB, 0.31138E+00_JPRB, 0.31889E+00_JPRB /)
ZBURD(:, 68, 11) = (/ &
&  0.32941E+00_JPRB, 0.33901E+00_JPRB, 0.35309E+00_JPRB, 0.36220E+00_JPRB, 0.36941E+00_JPRB, 0.36526E+00_JPRB, 0.36131E+00_JPRB ,&
&  0.35719E+00_JPRB, 0.34314E+00_JPRB, 0.32595E+00_JPRB, 0.29676E+00_JPRB, 0.27220E+00_JPRB, 0.25342E+00_JPRB, 0.24299E+00_JPRB ,&
&  0.23671E+00_JPRB, 0.23459E+00_JPRB, 0.23253E+00_JPRB, 0.23195E+00_JPRB, 0.23064E+00_JPRB, 0.23216E+00_JPRB, 0.23342E+00_JPRB ,&
&  0.23295E+00_JPRB, 0.23190E+00_JPRB, 0.23135E+00_JPRB, 0.23448E+00_JPRB, 0.24057E+00_JPRB, 0.24533E+00_JPRB, 0.24992E+00_JPRB ,&
&  0.25321E+00_JPRB, 0.25057E+00_JPRB, 0.25184E+00_JPRB, 0.24993E+00_JPRB, 0.24800E+00_JPRB, 0.24834E+00_JPRB, 0.24936E+00_JPRB ,&
&  0.24977E+00_JPRB, 0.25018E+00_JPRB, 0.24941E+00_JPRB, 0.24632E+00_JPRB, 0.23454E+00_JPRB, 0.22852E+00_JPRB, 0.22073E+00_JPRB ,&
&  0.21270E+00_JPRB, 0.20334E+00_JPRB, 0.19575E+00_JPRB, 0.19212E+00_JPRB, 0.18774E+00_JPRB, 0.18606E+00_JPRB, 0.18291E+00_JPRB ,&
&  0.17931E+00_JPRB, 0.17589E+00_JPRB, 0.17367E+00_JPRB, 0.17254E+00_JPRB, 0.17211E+00_JPRB, 0.17099E+00_JPRB, 0.17059E+00_JPRB ,&
&  0.17088E+00_JPRB, 0.17119E+00_JPRB, 0.17208E+00_JPRB, 0.17292E+00_JPRB, 0.17387E+00_JPRB, 0.17471E+00_JPRB, 0.17573E+00_JPRB ,&
&  0.17738E+00_JPRB, 0.17942E+00_JPRB, 0.18036E+00_JPRB, 0.18119E+00_JPRB, 0.18094E+00_JPRB, 0.18045E+00_JPRB, 0.17969E+00_JPRB ,&
&  0.17907E+00_JPRB, 0.17888E+00_JPRB, 0.17889E+00_JPRB, 0.17883E+00_JPRB, 0.17929E+00_JPRB, 0.18018E+00_JPRB, 0.18109E+00_JPRB ,&
&  0.18264E+00_JPRB, 0.18402E+00_JPRB, 0.18561E+00_JPRB, 0.18751E+00_JPRB, 0.18966E+00_JPRB, 0.19223E+00_JPRB, 0.19509E+00_JPRB ,&
&  0.19765E+00_JPRB, 0.20092E+00_JPRB, 0.20366E+00_JPRB, 0.20583E+00_JPRB, 0.21110E+00_JPRB, 0.21822E+00_JPRB, 0.23254E+00_JPRB ,&
&  0.24896E+00_JPRB, 0.28862E+00_JPRB, 0.33061E+00_JPRB, 0.31620E+00_JPRB, 0.32131E+00_JPRB /)
ZBURD(:, 69, 11) = (/ &
&  0.30245E+00_JPRB, 0.31338E+00_JPRB, 0.32212E+00_JPRB, 0.32759E+00_JPRB, 0.33314E+00_JPRB, 0.33526E+00_JPRB, 0.34337E+00_JPRB ,&
&  0.33787E+00_JPRB, 0.31984E+00_JPRB, 0.30868E+00_JPRB, 0.29222E+00_JPRB, 0.27192E+00_JPRB, 0.25532E+00_JPRB, 0.24695E+00_JPRB ,&
&  0.24198E+00_JPRB, 0.23801E+00_JPRB, 0.23324E+00_JPRB, 0.22799E+00_JPRB, 0.22271E+00_JPRB, 0.21879E+00_JPRB, 0.21627E+00_JPRB ,&
&  0.21397E+00_JPRB, 0.21463E+00_JPRB, 0.21728E+00_JPRB, 0.22167E+00_JPRB, 0.22792E+00_JPRB, 0.23219E+00_JPRB, 0.23556E+00_JPRB ,&
&  0.23763E+00_JPRB, 0.22857E+00_JPRB, 0.23034E+00_JPRB, 0.23241E+00_JPRB, 0.23112E+00_JPRB, 0.23422E+00_JPRB, 0.23929E+00_JPRB ,&
&  0.23692E+00_JPRB, 0.22986E+00_JPRB, 0.23245E+00_JPRB, 0.22738E+00_JPRB, 0.21896E+00_JPRB, 0.21722E+00_JPRB, 0.21145E+00_JPRB ,&
&  0.20866E+00_JPRB, 0.20152E+00_JPRB, 0.19644E+00_JPRB, 0.19308E+00_JPRB, 0.18747E+00_JPRB, 0.18517E+00_JPRB, 0.18172E+00_JPRB ,&
&  0.17819E+00_JPRB, 0.17576E+00_JPRB, 0.17287E+00_JPRB, 0.17148E+00_JPRB, 0.17029E+00_JPRB, 0.16903E+00_JPRB, 0.16803E+00_JPRB ,&
&  0.16746E+00_JPRB, 0.16694E+00_JPRB, 0.16698E+00_JPRB, 0.16701E+00_JPRB, 0.16701E+00_JPRB, 0.16734E+00_JPRB, 0.16754E+00_JPRB ,&
&  0.16818E+00_JPRB, 0.16895E+00_JPRB, 0.16958E+00_JPRB, 0.17058E+00_JPRB, 0.17102E+00_JPRB, 0.17159E+00_JPRB, 0.17201E+00_JPRB ,&
&  0.17245E+00_JPRB, 0.17312E+00_JPRB, 0.17345E+00_JPRB, 0.17412E+00_JPRB, 0.17488E+00_JPRB, 0.17636E+00_JPRB, 0.17783E+00_JPRB ,&
&  0.17962E+00_JPRB, 0.18172E+00_JPRB, 0.18395E+00_JPRB, 0.18631E+00_JPRB, 0.18924E+00_JPRB, 0.19190E+00_JPRB, 0.19543E+00_JPRB ,&
&  0.19925E+00_JPRB, 0.20290E+00_JPRB, 0.20721E+00_JPRB, 0.21263E+00_JPRB, 0.21982E+00_JPRB, 0.22948E+00_JPRB, 0.23835E+00_JPRB ,&
&  0.24851E+00_JPRB, 0.27450E+00_JPRB, 0.28082E+00_JPRB, 0.28519E+00_JPRB, 0.29383E+00_JPRB /)
ZBURD(:, 70, 11) = (/ &
&  0.29434E+00_JPRB, 0.29925E+00_JPRB, 0.30302E+00_JPRB, 0.30543E+00_JPRB, 0.30774E+00_JPRB, 0.30936E+00_JPRB, 0.30994E+00_JPRB ,&
&  0.30347E+00_JPRB, 0.29849E+00_JPRB, 0.29151E+00_JPRB, 0.28311E+00_JPRB, 0.27544E+00_JPRB, 0.26829E+00_JPRB, 0.26020E+00_JPRB ,&
&  0.25358E+00_JPRB, 0.24734E+00_JPRB, 0.24085E+00_JPRB, 0.23402E+00_JPRB, 0.22688E+00_JPRB, 0.21969E+00_JPRB, 0.21629E+00_JPRB ,&
&  0.21533E+00_JPRB, 0.21651E+00_JPRB, 0.21860E+00_JPRB, 0.22015E+00_JPRB, 0.22109E+00_JPRB, 0.22168E+00_JPRB, 0.22052E+00_JPRB ,&
&  0.21678E+00_JPRB, 0.21744E+00_JPRB, 0.21839E+00_JPRB, 0.21901E+00_JPRB, 0.22058E+00_JPRB, 0.22196E+00_JPRB, 0.22180E+00_JPRB ,&
&  0.21999E+00_JPRB, 0.21756E+00_JPRB, 0.21423E+00_JPRB, 0.21111E+00_JPRB, 0.20996E+00_JPRB, 0.20782E+00_JPRB, 0.20539E+00_JPRB ,&
&  0.20347E+00_JPRB, 0.19960E+00_JPRB, 0.19561E+00_JPRB, 0.19281E+00_JPRB, 0.18910E+00_JPRB, 0.18657E+00_JPRB, 0.18326E+00_JPRB ,&
&  0.18059E+00_JPRB, 0.17820E+00_JPRB, 0.17557E+00_JPRB, 0.17355E+00_JPRB, 0.17166E+00_JPRB, 0.17027E+00_JPRB, 0.16887E+00_JPRB ,&
&  0.16806E+00_JPRB, 0.16729E+00_JPRB, 0.16680E+00_JPRB, 0.16625E+00_JPRB, 0.16593E+00_JPRB, 0.16560E+00_JPRB, 0.16535E+00_JPRB ,&
&  0.16521E+00_JPRB, 0.16555E+00_JPRB, 0.16579E+00_JPRB, 0.16621E+00_JPRB, 0.16670E+00_JPRB, 0.16699E+00_JPRB, 0.16753E+00_JPRB ,&
&  0.16856E+00_JPRB, 0.16968E+00_JPRB, 0.17040E+00_JPRB, 0.17134E+00_JPRB, 0.17240E+00_JPRB, 0.17369E+00_JPRB, 0.17546E+00_JPRB ,&
&  0.17732E+00_JPRB, 0.17939E+00_JPRB, 0.18174E+00_JPRB, 0.18405E+00_JPRB, 0.18703E+00_JPRB, 0.19008E+00_JPRB, 0.19331E+00_JPRB ,&
&  0.19717E+00_JPRB, 0.20128E+00_JPRB, 0.20552E+00_JPRB, 0.21020E+00_JPRB, 0.21698E+00_JPRB, 0.22300E+00_JPRB, 0.23042E+00_JPRB ,&
&  0.23762E+00_JPRB, 0.24305E+00_JPRB, 0.27518E+00_JPRB, 0.27996E+00_JPRB, 0.28670E+00_JPRB /)
ZBURD(:, 71, 11) = (/ &
&  0.22072E+00_JPRB, 0.22743E+00_JPRB, 0.23413E+00_JPRB, 0.24366E+00_JPRB, 0.25001E+00_JPRB, 0.28555E+00_JPRB, 0.28044E+00_JPRB ,&
&  0.27710E+00_JPRB, 0.26366E+00_JPRB, 0.26223E+00_JPRB, 0.25867E+00_JPRB, 0.25417E+00_JPRB, 0.24931E+00_JPRB, 0.24438E+00_JPRB ,&
&  0.23976E+00_JPRB, 0.23433E+00_JPRB, 0.22970E+00_JPRB, 0.22605E+00_JPRB, 0.22278E+00_JPRB, 0.21957E+00_JPRB, 0.21678E+00_JPRB ,&
&  0.21559E+00_JPRB, 0.21547E+00_JPRB, 0.21470E+00_JPRB, 0.21054E+00_JPRB, 0.20708E+00_JPRB, 0.20494E+00_JPRB, 0.20428E+00_JPRB ,&
&  0.20420E+00_JPRB, 0.20590E+00_JPRB, 0.20697E+00_JPRB, 0.20787E+00_JPRB, 0.20921E+00_JPRB, 0.20838E+00_JPRB, 0.20755E+00_JPRB ,&
&  0.20635E+00_JPRB, 0.20502E+00_JPRB, 0.20363E+00_JPRB, 0.20242E+00_JPRB, 0.20121E+00_JPRB, 0.19941E+00_JPRB, 0.19786E+00_JPRB ,&
&  0.19633E+00_JPRB, 0.19436E+00_JPRB, 0.19171E+00_JPRB, 0.18995E+00_JPRB, 0.18790E+00_JPRB, 0.18615E+00_JPRB, 0.18439E+00_JPRB ,&
&  0.18308E+00_JPRB, 0.18188E+00_JPRB, 0.18048E+00_JPRB, 0.17879E+00_JPRB, 0.17711E+00_JPRB, 0.17563E+00_JPRB, 0.17422E+00_JPRB ,&
&  0.17346E+00_JPRB, 0.17267E+00_JPRB, 0.17189E+00_JPRB, 0.17104E+00_JPRB, 0.17063E+00_JPRB, 0.17021E+00_JPRB, 0.16981E+00_JPRB ,&
&  0.16949E+00_JPRB, 0.16931E+00_JPRB, 0.16933E+00_JPRB, 0.16942E+00_JPRB, 0.16952E+00_JPRB, 0.16988E+00_JPRB, 0.17032E+00_JPRB ,&
&  0.17092E+00_JPRB, 0.17165E+00_JPRB, 0.17267E+00_JPRB, 0.17372E+00_JPRB, 0.17467E+00_JPRB, 0.17542E+00_JPRB, 0.17640E+00_JPRB ,&
&  0.17797E+00_JPRB, 0.17979E+00_JPRB, 0.18177E+00_JPRB, 0.18329E+00_JPRB, 0.18545E+00_JPRB, 0.18742E+00_JPRB, 0.18937E+00_JPRB ,&
&  0.19131E+00_JPRB, 0.19362E+00_JPRB, 0.19621E+00_JPRB, 0.19899E+00_JPRB, 0.20223E+00_JPRB, 0.20500E+00_JPRB, 0.20760E+00_JPRB ,&
&  0.21024E+00_JPRB, 0.21401E+00_JPRB, 0.21604E+00_JPRB, 0.21720E+00_JPRB, 0.21867E+00_JPRB /)
ZBURD(:, 72, 11) = (/ &
&  0.19947E+00_JPRB, 0.19962E+00_JPRB, 0.20005E+00_JPRB, 0.20055E+00_JPRB, 0.20104E+00_JPRB, 0.20176E+00_JPRB, 0.20196E+00_JPRB ,&
&  0.20196E+00_JPRB, 0.20185E+00_JPRB, 0.20217E+00_JPRB, 0.20255E+00_JPRB, 0.20279E+00_JPRB, 0.20278E+00_JPRB, 0.20238E+00_JPRB ,&
&  0.20170E+00_JPRB, 0.20088E+00_JPRB, 0.19961E+00_JPRB, 0.19905E+00_JPRB, 0.19864E+00_JPRB, 0.19823E+00_JPRB, 0.19745E+00_JPRB ,&
&  0.19676E+00_JPRB, 0.19555E+00_JPRB, 0.19388E+00_JPRB, 0.19260E+00_JPRB, 0.19238E+00_JPRB, 0.19265E+00_JPRB, 0.19326E+00_JPRB ,&
&  0.19373E+00_JPRB, 0.19405E+00_JPRB, 0.19415E+00_JPRB, 0.19433E+00_JPRB, 0.19459E+00_JPRB, 0.19445E+00_JPRB, 0.19420E+00_JPRB ,&
&  0.19388E+00_JPRB, 0.19334E+00_JPRB, 0.19283E+00_JPRB, 0.19237E+00_JPRB, 0.19195E+00_JPRB, 0.19146E+00_JPRB, 0.19087E+00_JPRB ,&
&  0.19019E+00_JPRB, 0.18935E+00_JPRB, 0.18835E+00_JPRB, 0.18755E+00_JPRB, 0.18671E+00_JPRB, 0.18605E+00_JPRB, 0.18523E+00_JPRB ,&
&  0.18455E+00_JPRB, 0.18393E+00_JPRB, 0.18330E+00_JPRB, 0.18271E+00_JPRB, 0.18215E+00_JPRB, 0.18162E+00_JPRB, 0.18110E+00_JPRB ,&
&  0.18059E+00_JPRB, 0.18018E+00_JPRB, 0.17972E+00_JPRB, 0.17928E+00_JPRB, 0.17899E+00_JPRB, 0.17874E+00_JPRB, 0.17854E+00_JPRB ,&
&  0.17836E+00_JPRB, 0.17830E+00_JPRB, 0.17828E+00_JPRB, 0.17826E+00_JPRB, 0.17826E+00_JPRB, 0.17837E+00_JPRB, 0.17848E+00_JPRB ,&
&  0.17864E+00_JPRB, 0.17875E+00_JPRB, 0.17888E+00_JPRB, 0.17906E+00_JPRB, 0.17926E+00_JPRB, 0.17949E+00_JPRB, 0.17980E+00_JPRB ,&
&  0.18019E+00_JPRB, 0.18065E+00_JPRB, 0.18114E+00_JPRB, 0.18145E+00_JPRB, 0.18211E+00_JPRB, 0.18286E+00_JPRB, 0.18374E+00_JPRB ,&
&  0.18471E+00_JPRB, 0.18585E+00_JPRB, 0.18710E+00_JPRB, 0.18855E+00_JPRB, 0.19005E+00_JPRB, 0.19101E+00_JPRB, 0.19197E+00_JPRB ,&
&  0.19298E+00_JPRB, 0.19417E+00_JPRB, 0.19530E+00_JPRB, 0.19645E+00_JPRB, 0.19784E+00_JPRB /)
ZBURD(:, 73, 11) = (/ &
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB ,&
&  0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB, 0.18472E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART22',1,ZHOOK_HANDLE)
END SUBROUTINE PART22



SUBROUTINE PART23

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART23',0,ZHOOK_HANDLE)


!-- month= 12
ZBURD(:,  1, 12) = (/ &
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB ,&
&  0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB, 0.83326E+00_JPRB /)
ZBURD(:,  2, 12) = (/ &
&  0.86920E+00_JPRB, 0.86773E+00_JPRB, 0.86632E+00_JPRB, 0.86470E+00_JPRB, 0.86273E+00_JPRB, 0.86047E+00_JPRB, 0.85849E+00_JPRB ,&
&  0.85639E+00_JPRB, 0.85467E+00_JPRB, 0.85220E+00_JPRB, 0.84967E+00_JPRB, 0.84701E+00_JPRB, 0.84452E+00_JPRB, 0.84190E+00_JPRB ,&
&  0.83909E+00_JPRB, 0.83623E+00_JPRB, 0.83329E+00_JPRB, 0.83017E+00_JPRB, 0.82686E+00_JPRB, 0.82369E+00_JPRB, 0.82045E+00_JPRB ,&
&  0.81770E+00_JPRB, 0.81468E+00_JPRB, 0.81178E+00_JPRB, 0.80864E+00_JPRB, 0.80644E+00_JPRB, 0.80361E+00_JPRB, 0.80121E+00_JPRB ,&
&  0.79858E+00_JPRB, 0.79672E+00_JPRB, 0.79454E+00_JPRB, 0.79270E+00_JPRB, 0.79090E+00_JPRB, 0.79034E+00_JPRB, 0.78932E+00_JPRB ,&
&  0.78866E+00_JPRB, 0.78815E+00_JPRB, 0.78815E+00_JPRB, 0.78806E+00_JPRB, 0.78802E+00_JPRB, 0.78845E+00_JPRB, 0.78960E+00_JPRB ,&
&  0.79067E+00_JPRB, 0.79179E+00_JPRB, 0.79342E+00_JPRB, 0.79509E+00_JPRB, 0.79670E+00_JPRB, 0.79824E+00_JPRB, 0.79999E+00_JPRB ,&
&  0.80261E+00_JPRB, 0.80544E+00_JPRB, 0.80829E+00_JPRB, 0.81135E+00_JPRB, 0.81424E+00_JPRB, 0.81654E+00_JPRB, 0.81863E+00_JPRB ,&
&  0.82030E+00_JPRB, 0.82204E+00_JPRB, 0.82365E+00_JPRB, 0.82518E+00_JPRB, 0.82597E+00_JPRB, 0.82718E+00_JPRB, 0.82829E+00_JPRB ,&
&  0.82917E+00_JPRB, 0.82916E+00_JPRB, 0.82986E+00_JPRB, 0.83050E+00_JPRB, 0.83124E+00_JPRB, 0.83174E+00_JPRB, 0.83307E+00_JPRB ,&
&  0.83446E+00_JPRB, 0.83637E+00_JPRB, 0.83842E+00_JPRB, 0.84086E+00_JPRB, 0.84282E+00_JPRB, 0.84484E+00_JPRB, 0.84675E+00_JPRB ,&
&  0.84895E+00_JPRB, 0.85135E+00_JPRB, 0.85407E+00_JPRB, 0.85672E+00_JPRB, 0.85913E+00_JPRB, 0.86164E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.86644E+00_JPRB, 0.86803E+00_JPRB, 0.86938E+00_JPRB, 0.87044E+00_JPRB, 0.87135E+00_JPRB, 0.87176E+00_JPRB, 0.87190E+00_JPRB ,&
&  0.87177E+00_JPRB, 0.87152E+00_JPRB, 0.87113E+00_JPRB, 0.87077E+00_JPRB, 0.87031E+00_JPRB /)
ZBURD(:,  3, 12) = (/ &
&  0.88558E+00_JPRB, 0.88329E+00_JPRB, 0.88067E+00_JPRB, 0.87738E+00_JPRB, 0.87377E+00_JPRB, 0.86950E+00_JPRB, 0.86540E+00_JPRB ,&
&  0.86072E+00_JPRB, 0.85651E+00_JPRB, 0.85196E+00_JPRB, 0.84727E+00_JPRB, 0.84226E+00_JPRB, 0.83763E+00_JPRB, 0.83321E+00_JPRB ,&
&  0.82875E+00_JPRB, 0.82414E+00_JPRB, 0.82031E+00_JPRB, 0.81669E+00_JPRB, 0.81240E+00_JPRB, 0.80723E+00_JPRB, 0.80165E+00_JPRB ,&
&  0.79685E+00_JPRB, 0.79135E+00_JPRB, 0.78552E+00_JPRB, 0.77761E+00_JPRB, 0.77072E+00_JPRB, 0.76408E+00_JPRB, 0.75822E+00_JPRB ,&
&  0.75258E+00_JPRB, 0.74812E+00_JPRB, 0.74456E+00_JPRB, 0.74172E+00_JPRB, 0.74109E+00_JPRB, 0.74053E+00_JPRB, 0.73989E+00_JPRB ,&
&  0.73915E+00_JPRB, 0.73790E+00_JPRB, 0.73778E+00_JPRB, 0.73807E+00_JPRB, 0.73842E+00_JPRB, 0.74079E+00_JPRB, 0.74329E+00_JPRB ,&
&  0.74586E+00_JPRB, 0.74821E+00_JPRB, 0.75183E+00_JPRB, 0.75560E+00_JPRB, 0.75926E+00_JPRB, 0.76312E+00_JPRB, 0.76651E+00_JPRB ,&
&  0.77026E+00_JPRB, 0.77551E+00_JPRB, 0.78081E+00_JPRB, 0.78666E+00_JPRB, 0.79095E+00_JPRB, 0.79480E+00_JPRB, 0.79813E+00_JPRB ,&
&  0.80095E+00_JPRB, 0.80372E+00_JPRB, 0.80627E+00_JPRB, 0.80873E+00_JPRB, 0.81033E+00_JPRB, 0.81216E+00_JPRB, 0.81366E+00_JPRB ,&
&  0.81488E+00_JPRB, 0.81461E+00_JPRB, 0.81518E+00_JPRB, 0.81549E+00_JPRB, 0.81537E+00_JPRB, 0.81444E+00_JPRB, 0.81488E+00_JPRB ,&
&  0.81515E+00_JPRB, 0.81509E+00_JPRB, 0.81449E+00_JPRB, 0.81606E+00_JPRB, 0.81736E+00_JPRB, 0.81836E+00_JPRB, 0.81859E+00_JPRB ,&
&  0.82393E+00_JPRB, 0.83221E+00_JPRB, 0.84295E+00_JPRB, 0.85777E+00_JPRB, 0.86936E+00_JPRB, 0.87752E+00_JPRB, 0.88379E+00_JPRB ,&
&  0.88756E+00_JPRB, 0.88979E+00_JPRB, 0.89076E+00_JPRB, 0.89139E+00_JPRB, 0.89040E+00_JPRB, 0.89019E+00_JPRB, 0.89081E+00_JPRB ,&
&  0.89104E+00_JPRB, 0.89052E+00_JPRB, 0.88991E+00_JPRB, 0.88902E+00_JPRB, 0.88760E+00_JPRB /)
ZBURD(:,  4, 12) = (/ &
&  0.89120E+00_JPRB, 0.88438E+00_JPRB, 0.87756E+00_JPRB, 0.86956E+00_JPRB, 0.86319E+00_JPRB, 0.85566E+00_JPRB, 0.84919E+00_JPRB ,&
&  0.84236E+00_JPRB, 0.83593E+00_JPRB, 0.83063E+00_JPRB, 0.82597E+00_JPRB, 0.82140E+00_JPRB, 0.81677E+00_JPRB, 0.81286E+00_JPRB ,&
&  0.80882E+00_JPRB, 0.80484E+00_JPRB, 0.80194E+00_JPRB, 0.79942E+00_JPRB, 0.79647E+00_JPRB, 0.79285E+00_JPRB, 0.78745E+00_JPRB ,&
&  0.77947E+00_JPRB, 0.76829E+00_JPRB, 0.75539E+00_JPRB, 0.74164E+00_JPRB, 0.73454E+00_JPRB, 0.69776E+00_JPRB, 0.69077E+00_JPRB ,&
&  0.69372E+00_JPRB, 0.69601E+00_JPRB, 0.70234E+00_JPRB, 0.70933E+00_JPRB, 0.71215E+00_JPRB, 0.70094E+00_JPRB, 0.69352E+00_JPRB ,&
&  0.66646E+00_JPRB, 0.67019E+00_JPRB, 0.66998E+00_JPRB, 0.67119E+00_JPRB, 0.67631E+00_JPRB, 0.67876E+00_JPRB, 0.68581E+00_JPRB ,&
&  0.70760E+00_JPRB, 0.71066E+00_JPRB, 0.71563E+00_JPRB, 0.71993E+00_JPRB, 0.72397E+00_JPRB, 0.72812E+00_JPRB, 0.73359E+00_JPRB ,&
&  0.73849E+00_JPRB, 0.74470E+00_JPRB, 0.75197E+00_JPRB, 0.75753E+00_JPRB, 0.76270E+00_JPRB, 0.76870E+00_JPRB, 0.77431E+00_JPRB ,&
&  0.77844E+00_JPRB, 0.78206E+00_JPRB, 0.78566E+00_JPRB, 0.78949E+00_JPRB, 0.79268E+00_JPRB, 0.79528E+00_JPRB, 0.79770E+00_JPRB ,&
&  0.80025E+00_JPRB, 0.80055E+00_JPRB, 0.80138E+00_JPRB, 0.80255E+00_JPRB, 0.80304E+00_JPRB, 0.80359E+00_JPRB, 0.80510E+00_JPRB ,&
&  0.80796E+00_JPRB, 0.81110E+00_JPRB, 0.81503E+00_JPRB, 0.82056E+00_JPRB, 0.83025E+00_JPRB, 0.84281E+00_JPRB, 0.86580E+00_JPRB ,&
&  0.88556E+00_JPRB, 0.90271E+00_JPRB, 0.91120E+00_JPRB, 0.92040E+00_JPRB, 0.93017E+00_JPRB, 0.93579E+00_JPRB, 0.93992E+00_JPRB ,&
&  0.94006E+00_JPRB, 0.93819E+00_JPRB, 0.93322E+00_JPRB, 0.92763E+00_JPRB, 0.92201E+00_JPRB, 0.91675E+00_JPRB, 0.91281E+00_JPRB ,&
&  0.90979E+00_JPRB, 0.90834E+00_JPRB, 0.90616E+00_JPRB, 0.90199E+00_JPRB, 0.89674E+00_JPRB /)
ZBURD(:,  5, 12) = (/ &
&  0.90012E+00_JPRB, 0.88703E+00_JPRB, 0.87753E+00_JPRB, 0.86608E+00_JPRB, 0.85541E+00_JPRB, 0.84365E+00_JPRB, 0.83378E+00_JPRB ,&
&  0.82413E+00_JPRB, 0.81523E+00_JPRB, 0.80732E+00_JPRB, 0.79997E+00_JPRB, 0.79499E+00_JPRB, 0.79054E+00_JPRB, 0.78752E+00_JPRB ,&
&  0.78445E+00_JPRB, 0.78172E+00_JPRB, 0.77930E+00_JPRB, 0.77744E+00_JPRB, 0.77576E+00_JPRB, 0.77409E+00_JPRB, 0.77187E+00_JPRB ,&
&  0.76029E+00_JPRB, 0.74186E+00_JPRB, 0.70717E+00_JPRB, 0.69999E+00_JPRB, 0.68991E+00_JPRB, 0.68492E+00_JPRB, 0.68732E+00_JPRB ,&
&  0.68249E+00_JPRB, 0.68139E+00_JPRB, 0.67277E+00_JPRB, 0.66028E+00_JPRB, 0.64633E+00_JPRB, 0.62453E+00_JPRB, 0.61092E+00_JPRB ,&
&  0.59945E+00_JPRB, 0.59250E+00_JPRB, 0.59126E+00_JPRB, 0.59201E+00_JPRB, 0.60012E+00_JPRB, 0.60890E+00_JPRB, 0.61939E+00_JPRB ,&
&  0.63558E+00_JPRB, 0.64998E+00_JPRB, 0.66726E+00_JPRB, 0.66850E+00_JPRB, 0.67449E+00_JPRB, 0.68071E+00_JPRB, 0.68850E+00_JPRB ,&
&  0.69602E+00_JPRB, 0.70361E+00_JPRB, 0.71159E+00_JPRB, 0.71961E+00_JPRB, 0.72810E+00_JPRB, 0.73635E+00_JPRB, 0.74472E+00_JPRB ,&
&  0.75325E+00_JPRB, 0.75950E+00_JPRB, 0.76361E+00_JPRB, 0.76809E+00_JPRB, 0.77344E+00_JPRB, 0.77905E+00_JPRB, 0.78687E+00_JPRB ,&
&  0.79600E+00_JPRB, 0.80686E+00_JPRB, 0.81875E+00_JPRB, 0.83173E+00_JPRB, 0.84459E+00_JPRB, 0.86131E+00_JPRB, 0.87938E+00_JPRB ,&
&  0.90450E+00_JPRB, 0.92749E+00_JPRB, 0.95782E+00_JPRB, 0.97574E+00_JPRB, 0.10128E+01_JPRB, 0.10472E+01_JPRB, 0.10688E+01_JPRB ,&
&  0.10660E+01_JPRB, 0.10566E+01_JPRB, 0.10481E+01_JPRB, 0.10393E+01_JPRB, 0.10316E+01_JPRB, 0.10195E+01_JPRB, 0.10115E+01_JPRB ,&
&  0.99962E+00_JPRB, 0.99049E+00_JPRB, 0.97941E+00_JPRB, 0.97014E+00_JPRB, 0.96173E+00_JPRB, 0.95375E+00_JPRB, 0.94604E+00_JPRB ,&
&  0.93924E+00_JPRB, 0.93356E+00_JPRB, 0.92717E+00_JPRB, 0.91981E+00_JPRB, 0.91102E+00_JPRB /)
ZBURD(:,  6, 12) = (/ &
&  0.90941E+00_JPRB, 0.89922E+00_JPRB, 0.88993E+00_JPRB, 0.87855E+00_JPRB, 0.86521E+00_JPRB, 0.85281E+00_JPRB, 0.83612E+00_JPRB ,&
&  0.82108E+00_JPRB, 0.80641E+00_JPRB, 0.79410E+00_JPRB, 0.78377E+00_JPRB, 0.77567E+00_JPRB, 0.76911E+00_JPRB, 0.76591E+00_JPRB ,&
&  0.76181E+00_JPRB, 0.76040E+00_JPRB, 0.75922E+00_JPRB, 0.75943E+00_JPRB, 0.76028E+00_JPRB, 0.76063E+00_JPRB, 0.76194E+00_JPRB ,&
&  0.75430E+00_JPRB, 0.73457E+00_JPRB, 0.70941E+00_JPRB, 0.70525E+00_JPRB, 0.70327E+00_JPRB, 0.70957E+00_JPRB, 0.71447E+00_JPRB ,&
&  0.72996E+00_JPRB, 0.69360E+00_JPRB, 0.69396E+00_JPRB, 0.66406E+00_JPRB, 0.63925E+00_JPRB, 0.61499E+00_JPRB, 0.58928E+00_JPRB ,&
&  0.57977E+00_JPRB, 0.56894E+00_JPRB, 0.56337E+00_JPRB, 0.56457E+00_JPRB, 0.56911E+00_JPRB, 0.58135E+00_JPRB, 0.59583E+00_JPRB ,&
&  0.60502E+00_JPRB, 0.63751E+00_JPRB, 0.64211E+00_JPRB, 0.64367E+00_JPRB, 0.64708E+00_JPRB, 0.65122E+00_JPRB, 0.64749E+00_JPRB ,&
&  0.65063E+00_JPRB, 0.65368E+00_JPRB, 0.66017E+00_JPRB, 0.67300E+00_JPRB, 0.69101E+00_JPRB, 0.70297E+00_JPRB, 0.71438E+00_JPRB ,&
&  0.72771E+00_JPRB, 0.73600E+00_JPRB, 0.74115E+00_JPRB, 0.74701E+00_JPRB, 0.75530E+00_JPRB, 0.76476E+00_JPRB, 0.78049E+00_JPRB ,&
&  0.79918E+00_JPRB, 0.83363E+00_JPRB, 0.86524E+00_JPRB, 0.91247E+00_JPRB, 0.95667E+00_JPRB, 0.10119E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.11028E+01_JPRB, 0.11360E+01_JPRB, 0.11690E+01_JPRB, 0.11995E+01_JPRB, 0.12231E+01_JPRB, 0.12095E+01_JPRB, 0.11724E+01_JPRB ,&
&  0.11487E+01_JPRB, 0.11143E+01_JPRB, 0.10914E+01_JPRB, 0.10711E+01_JPRB, 0.10542E+01_JPRB, 0.10433E+01_JPRB, 0.10327E+01_JPRB ,&
&  0.10239E+01_JPRB, 0.10144E+01_JPRB, 0.10045E+01_JPRB, 0.99185E+00_JPRB, 0.98070E+00_JPRB, 0.96934E+00_JPRB, 0.95981E+00_JPRB ,&
&  0.94966E+00_JPRB, 0.94360E+00_JPRB, 0.93277E+00_JPRB, 0.92333E+00_JPRB, 0.91567E+00_JPRB /)
ZBURD(:,  7, 12) = (/ &
&  0.92040E+00_JPRB, 0.91510E+00_JPRB, 0.89825E+00_JPRB, 0.88356E+00_JPRB, 0.86197E+00_JPRB, 0.84309E+00_JPRB, 0.81860E+00_JPRB ,&
&  0.79705E+00_JPRB, 0.77620E+00_JPRB, 0.75869E+00_JPRB, 0.74494E+00_JPRB, 0.73393E+00_JPRB, 0.72591E+00_JPRB, 0.72212E+00_JPRB ,&
&  0.72075E+00_JPRB, 0.72109E+00_JPRB, 0.72308E+00_JPRB, 0.72619E+00_JPRB, 0.73109E+00_JPRB, 0.73645E+00_JPRB, 0.74286E+00_JPRB ,&
&  0.74373E+00_JPRB, 0.73444E+00_JPRB, 0.71118E+00_JPRB, 0.70375E+00_JPRB, 0.71136E+00_JPRB, 0.71618E+00_JPRB, 0.74394E+00_JPRB ,&
&  0.75750E+00_JPRB, 0.77198E+00_JPRB, 0.76939E+00_JPRB, 0.76575E+00_JPRB, 0.75880E+00_JPRB, 0.65330E+00_JPRB, 0.63717E+00_JPRB ,&
&  0.60944E+00_JPRB, 0.58294E+00_JPRB, 0.56917E+00_JPRB, 0.56029E+00_JPRB, 0.55857E+00_JPRB, 0.56981E+00_JPRB, 0.58692E+00_JPRB ,&
&  0.59864E+00_JPRB, 0.63953E+00_JPRB, 0.63911E+00_JPRB, 0.63563E+00_JPRB, 0.63126E+00_JPRB, 0.63079E+00_JPRB, 0.62505E+00_JPRB ,&
&  0.62948E+00_JPRB, 0.63546E+00_JPRB, 0.64653E+00_JPRB, 0.66749E+00_JPRB, 0.68912E+00_JPRB, 0.70834E+00_JPRB, 0.73223E+00_JPRB ,&
&  0.76735E+00_JPRB, 0.79493E+00_JPRB, 0.81084E+00_JPRB, 0.82625E+00_JPRB, 0.84977E+00_JPRB, 0.87974E+00_JPRB, 0.92263E+00_JPRB ,&
&  0.95223E+00_JPRB, 0.98764E+00_JPRB, 0.10240E+01_JPRB, 0.10671E+01_JPRB, 0.11054E+01_JPRB, 0.11469E+01_JPRB, 0.11872E+01_JPRB ,&
&  0.12106E+01_JPRB, 0.12347E+01_JPRB, 0.12417E+01_JPRB, 0.12713E+01_JPRB, 0.12537E+01_JPRB, 0.12149E+01_JPRB, 0.11888E+01_JPRB ,&
&  0.11663E+01_JPRB, 0.11682E+01_JPRB, 0.11480E+01_JPRB, 0.11278E+01_JPRB, 0.11092E+01_JPRB, 0.10960E+01_JPRB, 0.10818E+01_JPRB ,&
&  0.10711E+01_JPRB, 0.10578E+01_JPRB, 0.10435E+01_JPRB, 0.10247E+01_JPRB, 0.10107E+01_JPRB, 0.99475E+00_JPRB, 0.98288E+00_JPRB ,&
&  0.96453E+00_JPRB, 0.95175E+00_JPRB, 0.94015E+00_JPRB, 0.93168E+00_JPRB, 0.92342E+00_JPRB /)
ZBURD(:,  8, 12) = (/ &
&  0.92078E+00_JPRB, 0.90902E+00_JPRB, 0.88410E+00_JPRB, 0.86493E+00_JPRB, 0.83542E+00_JPRB, 0.80345E+00_JPRB, 0.76822E+00_JPRB ,&
&  0.74060E+00_JPRB, 0.71973E+00_JPRB, 0.70328E+00_JPRB, 0.69302E+00_JPRB, 0.68726E+00_JPRB, 0.68401E+00_JPRB, 0.68516E+00_JPRB ,&
&  0.68770E+00_JPRB, 0.68482E+00_JPRB, 0.68808E+00_JPRB, 0.69561E+00_JPRB, 0.70528E+00_JPRB, 0.71426E+00_JPRB, 0.72453E+00_JPRB ,&
&  0.72756E+00_JPRB, 0.72357E+00_JPRB, 0.71404E+00_JPRB, 0.70514E+00_JPRB, 0.70277E+00_JPRB, 0.71314E+00_JPRB, 0.73596E+00_JPRB ,&
&  0.75432E+00_JPRB, 0.77060E+00_JPRB, 0.79288E+00_JPRB, 0.81848E+00_JPRB, 0.82524E+00_JPRB, 0.78161E+00_JPRB, 0.69100E+00_JPRB ,&
&  0.62429E+00_JPRB, 0.57149E+00_JPRB, 0.54591E+00_JPRB, 0.53513E+00_JPRB, 0.54154E+00_JPRB, 0.56088E+00_JPRB, 0.59426E+00_JPRB ,&
&  0.61964E+00_JPRB, 0.63378E+00_JPRB, 0.63514E+00_JPRB, 0.63462E+00_JPRB, 0.63113E+00_JPRB, 0.63181E+00_JPRB, 0.63486E+00_JPRB ,&
&  0.64985E+00_JPRB, 0.66604E+00_JPRB, 0.68858E+00_JPRB, 0.72223E+00_JPRB, 0.76784E+00_JPRB, 0.81638E+00_JPRB, 0.87901E+00_JPRB ,&
&  0.99475E+00_JPRB, 0.10587E+01_JPRB, 0.11071E+01_JPRB, 0.11368E+01_JPRB, 0.11772E+01_JPRB, 0.11992E+01_JPRB, 0.11896E+01_JPRB ,&
&  0.11945E+01_JPRB, 0.12406E+01_JPRB, 0.12584E+01_JPRB, 0.12738E+01_JPRB, 0.12838E+01_JPRB, 0.13010E+01_JPRB, 0.13172E+01_JPRB ,&
&  0.12941E+01_JPRB, 0.13041E+01_JPRB, 0.14114E+01_JPRB, 0.14333E+01_JPRB, 0.14032E+01_JPRB, 0.13646E+01_JPRB, 0.13161E+01_JPRB ,&
&  0.12741E+01_JPRB, 0.12347E+01_JPRB, 0.12016E+01_JPRB, 0.11715E+01_JPRB, 0.11493E+01_JPRB, 0.11383E+01_JPRB, 0.11370E+01_JPRB ,&
&  0.11252E+01_JPRB, 0.11076E+01_JPRB, 0.10826E+01_JPRB, 0.10624E+01_JPRB, 0.10447E+01_JPRB, 0.10204E+01_JPRB, 0.99911E+00_JPRB ,&
&  0.97893E+00_JPRB, 0.96596E+00_JPRB, 0.95537E+00_JPRB, 0.94614E+00_JPRB, 0.93474E+00_JPRB /)
ZBURD(:,  9, 12) = (/ &
&  0.91734E+00_JPRB, 0.90567E+00_JPRB, 0.88020E+00_JPRB, 0.83865E+00_JPRB, 0.78282E+00_JPRB, 0.72813E+00_JPRB, 0.69272E+00_JPRB ,&
&  0.67581E+00_JPRB, 0.66753E+00_JPRB, 0.65648E+00_JPRB, 0.65078E+00_JPRB, 0.64833E+00_JPRB, 0.64872E+00_JPRB, 0.65197E+00_JPRB ,&
&  0.65759E+00_JPRB, 0.66443E+00_JPRB, 0.67120E+00_JPRB, 0.67568E+00_JPRB, 0.68397E+00_JPRB, 0.69684E+00_JPRB, 0.71227E+00_JPRB ,&
&  0.72802E+00_JPRB, 0.73373E+00_JPRB, 0.73302E+00_JPRB, 0.73539E+00_JPRB, 0.73544E+00_JPRB, 0.74718E+00_JPRB, 0.74891E+00_JPRB ,&
&  0.75273E+00_JPRB, 0.77203E+00_JPRB, 0.81764E+00_JPRB, 0.86936E+00_JPRB, 0.89794E+00_JPRB, 0.84713E+00_JPRB, 0.73628E+00_JPRB ,&
&  0.63344E+00_JPRB, 0.56338E+00_JPRB, 0.53335E+00_JPRB, 0.52651E+00_JPRB, 0.53829E+00_JPRB, 0.56680E+00_JPRB, 0.60635E+00_JPRB ,&
&  0.63543E+00_JPRB, 0.64263E+00_JPRB, 0.64899E+00_JPRB, 0.65138E+00_JPRB, 0.64964E+00_JPRB, 0.64752E+00_JPRB, 0.65283E+00_JPRB ,&
&  0.67297E+00_JPRB, 0.72139E+00_JPRB, 0.79785E+00_JPRB, 0.90830E+00_JPRB, 0.10575E+01_JPRB, 0.10926E+01_JPRB, 0.11863E+01_JPRB ,&
&  0.13565E+01_JPRB, 0.14056E+01_JPRB, 0.14565E+01_JPRB, 0.14755E+01_JPRB, 0.14853E+01_JPRB, 0.14815E+01_JPRB, 0.14887E+01_JPRB ,&
&  0.14993E+01_JPRB, 0.14994E+01_JPRB, 0.14821E+01_JPRB, 0.14561E+01_JPRB, 0.14493E+01_JPRB, 0.14643E+01_JPRB, 0.15002E+01_JPRB ,&
&  0.15555E+01_JPRB, 0.16287E+01_JPRB, 0.17850E+01_JPRB, 0.15618E+01_JPRB, 0.14801E+01_JPRB, 0.14701E+01_JPRB, 0.14320E+01_JPRB ,&
&  0.13877E+01_JPRB, 0.13510E+01_JPRB, 0.13311E+01_JPRB, 0.13095E+01_JPRB, 0.12462E+01_JPRB, 0.12035E+01_JPRB, 0.11704E+01_JPRB ,&
&  0.11536E+01_JPRB, 0.11314E+01_JPRB, 0.11121E+01_JPRB, 0.10947E+01_JPRB, 0.10719E+01_JPRB, 0.10442E+01_JPRB, 0.10223E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.98182E+00_JPRB, 0.96593E+00_JPRB, 0.94722E+00_JPRB, 0.93106E+00_JPRB /)
ZBURD(:, 10, 12) = (/ &
&  0.87472E+00_JPRB, 0.86777E+00_JPRB, 0.84235E+00_JPRB, 0.79339E+00_JPRB, 0.72757E+00_JPRB, 0.67779E+00_JPRB, 0.64870E+00_JPRB ,&
&  0.63383E+00_JPRB, 0.62322E+00_JPRB, 0.61631E+00_JPRB, 0.61323E+00_JPRB, 0.61393E+00_JPRB, 0.61866E+00_JPRB, 0.62376E+00_JPRB ,&
&  0.62844E+00_JPRB, 0.63473E+00_JPRB, 0.64312E+00_JPRB, 0.65257E+00_JPRB, 0.66430E+00_JPRB, 0.67878E+00_JPRB, 0.69556E+00_JPRB ,&
&  0.71698E+00_JPRB, 0.73373E+00_JPRB, 0.74300E+00_JPRB, 0.75322E+00_JPRB, 0.77195E+00_JPRB, 0.80593E+00_JPRB, 0.83756E+00_JPRB ,&
&  0.85215E+00_JPRB, 0.84952E+00_JPRB, 0.87731E+00_JPRB, 0.93423E+00_JPRB, 0.96941E+00_JPRB, 0.93073E+00_JPRB, 0.77029E+00_JPRB ,&
&  0.64198E+00_JPRB, 0.57395E+00_JPRB, 0.55647E+00_JPRB, 0.56916E+00_JPRB, 0.60096E+00_JPRB, 0.63996E+00_JPRB, 0.65522E+00_JPRB ,&
&  0.66643E+00_JPRB, 0.67076E+00_JPRB, 0.67313E+00_JPRB, 0.67790E+00_JPRB, 0.67998E+00_JPRB, 0.68454E+00_JPRB, 0.70464E+00_JPRB ,&
&  0.75132E+00_JPRB, 0.82829E+00_JPRB, 0.91615E+00_JPRB, 0.96581E+00_JPRB, 0.10773E+01_JPRB, 0.12837E+01_JPRB, 0.14903E+01_JPRB ,&
&  0.15602E+01_JPRB, 0.15931E+01_JPRB, 0.16479E+01_JPRB, 0.17482E+01_JPRB, 0.17966E+01_JPRB, 0.18291E+01_JPRB, 0.18296E+01_JPRB ,&
&  0.18125E+01_JPRB, 0.17673E+01_JPRB, 0.17208E+01_JPRB, 0.17161E+01_JPRB, 0.17701E+01_JPRB, 0.18705E+01_JPRB, 0.19212E+01_JPRB ,&
&  0.19344E+01_JPRB, 0.18339E+01_JPRB, 0.16250E+01_JPRB, 0.14304E+01_JPRB, 0.13591E+01_JPRB, 0.13630E+01_JPRB, 0.13923E+01_JPRB ,&
&  0.13822E+01_JPRB, 0.13606E+01_JPRB, 0.13329E+01_JPRB, 0.13264E+01_JPRB, 0.12285E+01_JPRB, 0.11662E+01_JPRB, 0.11143E+01_JPRB ,&
&  0.10873E+01_JPRB, 0.10682E+01_JPRB, 0.10641E+01_JPRB, 0.10654E+01_JPRB, 0.10498E+01_JPRB, 0.10229E+01_JPRB, 0.99147E+00_JPRB ,&
&  0.96494E+00_JPRB, 0.93715E+00_JPRB, 0.91495E+00_JPRB, 0.89200E+00_JPRB, 0.87960E+00_JPRB /)
ZBURD(:, 11, 12) = (/ &
&  0.88997E+00_JPRB, 0.85729E+00_JPRB, 0.81544E+00_JPRB, 0.76579E+00_JPRB, 0.72165E+00_JPRB, 0.69858E+00_JPRB, 0.68174E+00_JPRB ,&
&  0.67705E+00_JPRB, 0.65020E+00_JPRB, 0.62121E+00_JPRB, 0.60125E+00_JPRB, 0.59161E+00_JPRB, 0.58885E+00_JPRB, 0.59504E+00_JPRB ,&
&  0.60701E+00_JPRB, 0.61984E+00_JPRB, 0.63360E+00_JPRB, 0.64925E+00_JPRB, 0.66541E+00_JPRB, 0.67987E+00_JPRB, 0.69653E+00_JPRB ,&
&  0.72060E+00_JPRB, 0.74700E+00_JPRB, 0.77572E+00_JPRB, 0.81631E+00_JPRB, 0.85898E+00_JPRB, 0.91638E+00_JPRB, 0.95150E+00_JPRB ,&
&  0.96411E+00_JPRB, 0.95858E+00_JPRB, 0.96613E+00_JPRB, 0.10131E+01_JPRB, 0.10406E+01_JPRB, 0.10050E+01_JPRB, 0.80787E+00_JPRB ,&
&  0.65210E+00_JPRB, 0.61163E+00_JPRB, 0.63449E+00_JPRB, 0.69630E+00_JPRB, 0.69937E+00_JPRB, 0.70380E+00_JPRB, 0.70694E+00_JPRB ,&
&  0.69549E+00_JPRB, 0.68717E+00_JPRB, 0.69826E+00_JPRB, 0.72034E+00_JPRB, 0.72608E+00_JPRB, 0.73894E+00_JPRB, 0.77437E+00_JPRB ,&
&  0.84693E+00_JPRB, 0.94522E+00_JPRB, 0.10534E+01_JPRB, 0.10546E+01_JPRB, 0.12058E+01_JPRB, 0.17132E+01_JPRB, 0.18643E+01_JPRB ,&
&  0.19394E+01_JPRB, 0.19874E+01_JPRB, 0.20785E+01_JPRB, 0.21342E+01_JPRB, 0.21061E+01_JPRB, 0.21015E+01_JPRB, 0.21181E+01_JPRB ,&
&  0.20829E+01_JPRB, 0.20801E+01_JPRB, 0.20778E+01_JPRB, 0.21640E+01_JPRB, 0.21876E+01_JPRB, 0.21650E+01_JPRB, 0.21441E+01_JPRB ,&
&  0.20958E+01_JPRB, 0.19448E+01_JPRB, 0.17719E+01_JPRB, 0.16041E+01_JPRB, 0.15204E+01_JPRB, 0.14635E+01_JPRB, 0.14277E+01_JPRB ,&
&  0.13861E+01_JPRB, 0.13686E+01_JPRB, 0.13629E+01_JPRB, 0.13629E+01_JPRB, 0.12901E+01_JPRB, 0.11810E+01_JPRB, 0.10981E+01_JPRB ,&
&  0.10519E+01_JPRB, 0.10233E+01_JPRB, 0.98184E+00_JPRB, 0.96770E+00_JPRB, 0.97497E+00_JPRB, 0.98342E+00_JPRB, 0.96044E+00_JPRB ,&
&  0.93891E+00_JPRB, 0.92006E+00_JPRB, 0.91295E+00_JPRB, 0.91146E+00_JPRB, 0.90999E+00_JPRB /)
ZBURD(:, 12, 12) = (/ &
&  0.86346E+00_JPRB, 0.83193E+00_JPRB, 0.79626E+00_JPRB, 0.75861E+00_JPRB, 0.73777E+00_JPRB, 0.72044E+00_JPRB, 0.69219E+00_JPRB ,&
&  0.68837E+00_JPRB, 0.64512E+00_JPRB, 0.61334E+00_JPRB, 0.60160E+00_JPRB, 0.60206E+00_JPRB, 0.59775E+00_JPRB, 0.58975E+00_JPRB ,&
&  0.59312E+00_JPRB, 0.60913E+00_JPRB, 0.63308E+00_JPRB, 0.65394E+00_JPRB, 0.67388E+00_JPRB, 0.69207E+00_JPRB, 0.70948E+00_JPRB ,&
&  0.73581E+00_JPRB, 0.76853E+00_JPRB, 0.80852E+00_JPRB, 0.86305E+00_JPRB, 0.91753E+00_JPRB, 0.98584E+00_JPRB, 0.10301E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.10732E+01_JPRB, 0.10523E+01_JPRB, 0.10593E+01_JPRB, 0.10595E+01_JPRB, 0.10390E+01_JPRB, 0.97606E+00_JPRB ,&
&  0.74277E+00_JPRB, 0.69940E+00_JPRB, 0.77789E+00_JPRB, 0.76904E+00_JPRB, 0.75909E+00_JPRB, 0.74437E+00_JPRB, 0.73878E+00_JPRB ,&
&  0.73877E+00_JPRB, 0.73844E+00_JPRB, 0.74443E+00_JPRB, 0.74689E+00_JPRB, 0.75688E+00_JPRB, 0.77986E+00_JPRB, 0.84185E+00_JPRB ,&
&  0.96775E+00_JPRB, 0.96142E+00_JPRB, 0.10186E+01_JPRB, 0.11419E+01_JPRB, 0.15290E+01_JPRB, 0.18064E+01_JPRB, 0.19984E+01_JPRB ,&
&  0.22667E+01_JPRB, 0.23816E+01_JPRB, 0.24268E+01_JPRB, 0.24705E+01_JPRB, 0.24967E+01_JPRB, 0.25264E+01_JPRB, 0.25764E+01_JPRB ,&
&  0.25187E+01_JPRB, 0.25069E+01_JPRB, 0.24852E+01_JPRB, 0.25077E+01_JPRB, 0.24555E+01_JPRB, 0.23908E+01_JPRB, 0.23173E+01_JPRB ,&
&  0.22355E+01_JPRB, 0.21277E+01_JPRB, 0.19630E+01_JPRB, 0.18027E+01_JPRB, 0.16661E+01_JPRB, 0.16037E+01_JPRB, 0.15631E+01_JPRB ,&
&  0.15197E+01_JPRB, 0.14939E+01_JPRB, 0.14651E+01_JPRB, 0.14313E+01_JPRB, 0.13939E+01_JPRB, 0.13417E+01_JPRB, 0.13129E+01_JPRB ,&
&  0.11928E+01_JPRB, 0.11127E+01_JPRB, 0.10084E+01_JPRB, 0.97952E+00_JPRB, 0.98152E+00_JPRB, 0.97976E+00_JPRB, 0.97425E+00_JPRB ,&
&  0.98057E+00_JPRB, 0.95431E+00_JPRB, 0.92904E+00_JPRB, 0.90918E+00_JPRB, 0.89088E+00_JPRB /)
ZBURD(:, 13, 12) = (/ &
&  0.84048E+00_JPRB, 0.81964E+00_JPRB, 0.78926E+00_JPRB, 0.76196E+00_JPRB, 0.75618E+00_JPRB, 0.74314E+00_JPRB, 0.72771E+00_JPRB ,&
&  0.75428E+00_JPRB, 0.72842E+00_JPRB, 0.73617E+00_JPRB, 0.66032E+00_JPRB, 0.62656E+00_JPRB, 0.61120E+00_JPRB, 0.59426E+00_JPRB ,&
&  0.59862E+00_JPRB, 0.61815E+00_JPRB, 0.63637E+00_JPRB, 0.65251E+00_JPRB, 0.67985E+00_JPRB, 0.70061E+00_JPRB, 0.71518E+00_JPRB ,&
&  0.73533E+00_JPRB, 0.76447E+00_JPRB, 0.80898E+00_JPRB, 0.85240E+00_JPRB, 0.90957E+00_JPRB, 0.98058E+00_JPRB, 0.10299E+01_JPRB ,&
&  0.10005E+01_JPRB, 0.98783E+00_JPRB, 0.10192E+01_JPRB, 0.10254E+01_JPRB, 0.10453E+01_JPRB, 0.10355E+01_JPRB, 0.10196E+01_JPRB ,&
&  0.97813E+00_JPRB, 0.86282E+00_JPRB, 0.87238E+00_JPRB, 0.83587E+00_JPRB, 0.80808E+00_JPRB, 0.78412E+00_JPRB, 0.76655E+00_JPRB ,&
&  0.75893E+00_JPRB, 0.76271E+00_JPRB, 0.77595E+00_JPRB, 0.78963E+00_JPRB, 0.80911E+00_JPRB, 0.87123E+00_JPRB, 0.96944E+00_JPRB ,&
&  0.11316E+01_JPRB, 0.10426E+01_JPRB, 0.12592E+01_JPRB, 0.13814E+01_JPRB, 0.17315E+01_JPRB, 0.20919E+01_JPRB, 0.23343E+01_JPRB ,&
&  0.28079E+01_JPRB, 0.28944E+01_JPRB, 0.28964E+01_JPRB, 0.28813E+01_JPRB, 0.29146E+01_JPRB, 0.28827E+01_JPRB, 0.28071E+01_JPRB ,&
&  0.27338E+01_JPRB, 0.25597E+01_JPRB, 0.26784E+01_JPRB, 0.26343E+01_JPRB, 0.25420E+01_JPRB, 0.24841E+01_JPRB, 0.24064E+01_JPRB ,&
&  0.23404E+01_JPRB, 0.22623E+01_JPRB, 0.21793E+01_JPRB, 0.19564E+01_JPRB, 0.19008E+01_JPRB, 0.18449E+01_JPRB, 0.16869E+01_JPRB ,&
&  0.16207E+01_JPRB, 0.15324E+01_JPRB, 0.14291E+01_JPRB, 0.14623E+01_JPRB, 0.13930E+01_JPRB, 0.13517E+01_JPRB, 0.13257E+01_JPRB ,&
&  0.12500E+01_JPRB, 0.11264E+01_JPRB, 0.11046E+01_JPRB, 0.10938E+01_JPRB, 0.10531E+01_JPRB, 0.10065E+01_JPRB, 0.10292E+01_JPRB ,&
&  0.97483E+00_JPRB, 0.95954E+00_JPRB, 0.96954E+00_JPRB, 0.92531E+00_JPRB, 0.88274E+00_JPRB /)
ZBURD(:, 14, 12) = (/ &
&  0.84102E+00_JPRB, 0.82133E+00_JPRB, 0.79127E+00_JPRB, 0.77617E+00_JPRB, 0.77657E+00_JPRB, 0.77710E+00_JPRB, 0.77305E+00_JPRB ,&
&  0.79105E+00_JPRB, 0.79403E+00_JPRB, 0.78795E+00_JPRB, 0.76908E+00_JPRB, 0.73981E+00_JPRB, 0.67456E+00_JPRB, 0.60009E+00_JPRB ,&
&  0.59625E+00_JPRB, 0.63302E+00_JPRB, 0.67262E+00_JPRB, 0.69334E+00_JPRB, 0.71976E+00_JPRB, 0.74693E+00_JPRB, 0.76749E+00_JPRB ,&
&  0.78216E+00_JPRB, 0.79528E+00_JPRB, 0.82771E+00_JPRB, 0.87717E+00_JPRB, 0.92987E+00_JPRB, 0.98334E+00_JPRB, 0.10343E+01_JPRB ,&
&  0.98861E+00_JPRB, 0.97318E+00_JPRB, 0.98983E+00_JPRB, 0.95736E+00_JPRB, 0.10609E+01_JPRB, 0.10744E+01_JPRB, 0.10634E+01_JPRB ,&
&  0.10421E+01_JPRB, 0.99912E+00_JPRB, 0.94463E+00_JPRB, 0.90188E+00_JPRB, 0.86810E+00_JPRB, 0.84588E+00_JPRB, 0.82032E+00_JPRB ,&
&  0.80753E+00_JPRB, 0.81001E+00_JPRB, 0.83247E+00_JPRB, 0.85991E+00_JPRB, 0.91174E+00_JPRB, 0.10496E+01_JPRB, 0.12371E+01_JPRB ,&
&  0.14004E+01_JPRB, 0.15336E+01_JPRB, 0.16217E+01_JPRB, 0.16626E+01_JPRB, 0.19656E+01_JPRB, 0.23221E+01_JPRB, 0.25366E+01_JPRB ,&
&  0.28539E+01_JPRB, 0.28665E+01_JPRB, 0.30348E+01_JPRB, 0.31844E+01_JPRB, 0.33299E+01_JPRB, 0.32925E+01_JPRB, 0.30840E+01_JPRB ,&
&  0.29495E+01_JPRB, 0.29529E+01_JPRB, 0.29275E+01_JPRB, 0.28942E+01_JPRB, 0.27829E+01_JPRB, 0.27219E+01_JPRB, 0.26392E+01_JPRB ,&
&  0.26338E+01_JPRB, 0.26317E+01_JPRB, 0.25535E+01_JPRB, 0.24475E+01_JPRB, 0.23202E+01_JPRB, 0.21323E+01_JPRB, 0.19202E+01_JPRB ,&
&  0.17160E+01_JPRB, 0.14620E+01_JPRB, 0.13512E+01_JPRB, 0.13434E+01_JPRB, 0.13690E+01_JPRB, 0.13002E+01_JPRB, 0.12741E+01_JPRB ,&
&  0.12531E+01_JPRB, 0.12010E+01_JPRB, 0.12369E+01_JPRB, 0.11779E+01_JPRB, 0.11029E+01_JPRB, 0.10222E+01_JPRB, 0.97104E+00_JPRB ,&
&  0.90421E+00_JPRB, 0.92647E+00_JPRB, 0.92162E+00_JPRB, 0.89664E+00_JPRB, 0.86486E+00_JPRB /)
ZBURD(:, 15, 12) = (/ &
&  0.84413E+00_JPRB, 0.81248E+00_JPRB, 0.78864E+00_JPRB, 0.78520E+00_JPRB, 0.79114E+00_JPRB, 0.80150E+00_JPRB, 0.81941E+00_JPRB ,&
&  0.83752E+00_JPRB, 0.84174E+00_JPRB, 0.82277E+00_JPRB, 0.79416E+00_JPRB, 0.76921E+00_JPRB, 0.75146E+00_JPRB, 0.71636E+00_JPRB ,&
&  0.67550E+00_JPRB, 0.73762E+00_JPRB, 0.73119E+00_JPRB, 0.75217E+00_JPRB, 0.80958E+00_JPRB, 0.83102E+00_JPRB, 0.85598E+00_JPRB ,&
&  0.89643E+00_JPRB, 0.90756E+00_JPRB, 0.92722E+00_JPRB, 0.95606E+00_JPRB, 0.99893E+00_JPRB, 0.10923E+01_JPRB, 0.11828E+01_JPRB ,&
&  0.11363E+01_JPRB, 0.11154E+01_JPRB, 0.11191E+01_JPRB, 0.11656E+01_JPRB, 0.12262E+01_JPRB, 0.12593E+01_JPRB, 0.12323E+01_JPRB ,&
&  0.11812E+01_JPRB, 0.11232E+01_JPRB, 0.10698E+01_JPRB, 0.10242E+01_JPRB, 0.97024E+00_JPRB, 0.91489E+00_JPRB, 0.87821E+00_JPRB ,&
&  0.85905E+00_JPRB, 0.86222E+00_JPRB, 0.89030E+00_JPRB, 0.91708E+00_JPRB, 0.97805E+00_JPRB, 0.12345E+01_JPRB, 0.16684E+01_JPRB ,&
&  0.19611E+01_JPRB, 0.22216E+01_JPRB, 0.23629E+01_JPRB, 0.25154E+01_JPRB, 0.26696E+01_JPRB, 0.28244E+01_JPRB, 0.29756E+01_JPRB ,&
&  0.31371E+01_JPRB, 0.30463E+01_JPRB, 0.33839E+01_JPRB, 0.34894E+01_JPRB, 0.35209E+01_JPRB, 0.35861E+01_JPRB, 0.33421E+01_JPRB ,&
&  0.31236E+01_JPRB, 0.29176E+01_JPRB, 0.30099E+01_JPRB, 0.30152E+01_JPRB, 0.29650E+01_JPRB, 0.30060E+01_JPRB, 0.29459E+01_JPRB ,&
&  0.28986E+01_JPRB, 0.28195E+01_JPRB, 0.24987E+01_JPRB, 0.22722E+01_JPRB, 0.19578E+01_JPRB, 0.20807E+01_JPRB, 0.19100E+01_JPRB ,&
&  0.16849E+01_JPRB, 0.13865E+01_JPRB, 0.13899E+01_JPRB, 0.13839E+01_JPRB, 0.14356E+01_JPRB, 0.14565E+01_JPRB, 0.13667E+01_JPRB ,&
&  0.13587E+01_JPRB, 0.14008E+01_JPRB, 0.13149E+01_JPRB, 0.12394E+01_JPRB, 0.11611E+01_JPRB, 0.10683E+01_JPRB, 0.99865E+00_JPRB ,&
&  0.97889E+00_JPRB, 0.97569E+00_JPRB, 0.93977E+00_JPRB, 0.90039E+00_JPRB, 0.87145E+00_JPRB /)
ZBURD(:, 16, 12) = (/ &
&  0.87466E+00_JPRB, 0.84612E+00_JPRB, 0.82713E+00_JPRB, 0.82501E+00_JPRB, 0.82981E+00_JPRB, 0.84542E+00_JPRB, 0.86391E+00_JPRB ,&
&  0.87537E+00_JPRB, 0.86845E+00_JPRB, 0.83801E+00_JPRB, 0.81275E+00_JPRB, 0.79506E+00_JPRB, 0.77782E+00_JPRB, 0.78673E+00_JPRB ,&
&  0.72770E+00_JPRB, 0.80841E+00_JPRB, 0.75394E+00_JPRB, 0.79210E+00_JPRB, 0.89277E+00_JPRB, 0.92513E+00_JPRB, 0.95505E+00_JPRB ,&
&  0.96885E+00_JPRB, 0.10065E+01_JPRB, 0.10190E+01_JPRB, 0.10465E+01_JPRB, 0.11043E+01_JPRB, 0.12371E+01_JPRB, 0.13857E+01_JPRB ,&
&  0.13489E+01_JPRB, 0.12987E+01_JPRB, 0.13655E+01_JPRB, 0.14535E+01_JPRB, 0.14436E+01_JPRB, 0.15007E+01_JPRB, 0.15307E+01_JPRB ,&
&  0.14527E+01_JPRB, 0.13423E+01_JPRB, 0.12229E+01_JPRB, 0.11129E+01_JPRB, 0.10280E+01_JPRB, 0.96102E+00_JPRB, 0.92461E+00_JPRB ,&
&  0.90893E+00_JPRB, 0.91108E+00_JPRB, 0.92339E+00_JPRB, 0.99265E+00_JPRB, 0.10612E+01_JPRB, 0.13977E+01_JPRB, 0.19850E+01_JPRB ,&
&  0.26008E+01_JPRB, 0.27427E+01_JPRB, 0.30582E+01_JPRB, 0.37309E+01_JPRB, 0.38306E+01_JPRB, 0.38016E+01_JPRB, 0.37094E+01_JPRB ,&
&  0.36309E+01_JPRB, 0.34439E+01_JPRB, 0.33776E+01_JPRB, 0.34305E+01_JPRB, 0.33736E+01_JPRB, 0.34965E+01_JPRB, 0.33414E+01_JPRB ,&
&  0.30516E+01_JPRB, 0.29653E+01_JPRB, 0.29769E+01_JPRB, 0.29346E+01_JPRB, 0.28573E+01_JPRB, 0.29254E+01_JPRB, 0.29005E+01_JPRB ,&
&  0.27584E+01_JPRB, 0.24353E+01_JPRB, 0.19070E+01_JPRB, 0.18375E+01_JPRB, 0.14635E+01_JPRB, 0.15540E+01_JPRB, 0.19326E+01_JPRB ,&
&  0.19337E+01_JPRB, 0.17835E+01_JPRB, 0.16417E+01_JPRB, 0.15762E+01_JPRB, 0.15696E+01_JPRB, 0.17138E+01_JPRB, 0.16227E+01_JPRB ,&
&  0.14753E+01_JPRB, 0.14978E+01_JPRB, 0.14720E+01_JPRB, 0.13983E+01_JPRB, 0.12992E+01_JPRB, 0.12048E+01_JPRB, 0.11378E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.10768E+01_JPRB, 0.10258E+01_JPRB, 0.95206E+00_JPRB, 0.90169E+00_JPRB /)
ZBURD(:, 17, 12) = (/ &
&  0.94161E+00_JPRB, 0.91149E+00_JPRB, 0.88619E+00_JPRB, 0.88133E+00_JPRB, 0.88582E+00_JPRB, 0.89795E+00_JPRB, 0.90864E+00_JPRB ,&
&  0.90694E+00_JPRB, 0.88587E+00_JPRB, 0.85036E+00_JPRB, 0.83254E+00_JPRB, 0.81602E+00_JPRB, 0.79515E+00_JPRB, 0.79495E+00_JPRB ,&
&  0.83139E+00_JPRB, 0.97461E+00_JPRB, 0.91157E+00_JPRB, 0.82480E+00_JPRB, 0.85841E+00_JPRB, 0.88154E+00_JPRB, 0.95471E+00_JPRB ,&
&  0.99353E+00_JPRB, 0.10836E+01_JPRB, 0.11260E+01_JPRB, 0.11631E+01_JPRB, 0.12531E+01_JPRB, 0.14246E+01_JPRB, 0.15763E+01_JPRB ,&
&  0.16259E+01_JPRB, 0.16898E+01_JPRB, 0.18235E+01_JPRB, 0.19642E+01_JPRB, 0.19783E+01_JPRB, 0.17895E+01_JPRB, 0.17739E+01_JPRB ,&
&  0.16394E+01_JPRB, 0.14680E+01_JPRB, 0.12914E+01_JPRB, 0.11645E+01_JPRB, 0.10862E+01_JPRB, 0.10399E+01_JPRB, 0.10164E+01_JPRB ,&
&  0.10116E+01_JPRB, 0.10090E+01_JPRB, 0.98674E+00_JPRB, 0.99962E+00_JPRB, 0.10684E+01_JPRB, 0.13648E+01_JPRB, 0.18262E+01_JPRB ,&
&  0.24868E+01_JPRB, 0.26479E+01_JPRB, 0.28518E+01_JPRB, 0.34369E+01_JPRB, 0.37396E+01_JPRB, 0.38898E+01_JPRB, 0.39815E+01_JPRB ,&
&  0.39279E+01_JPRB, 0.38029E+01_JPRB, 0.36238E+01_JPRB, 0.33787E+01_JPRB, 0.33315E+01_JPRB, 0.33486E+01_JPRB, 0.32487E+01_JPRB ,&
&  0.29924E+01_JPRB, 0.28393E+01_JPRB, 0.28305E+01_JPRB, 0.26919E+01_JPRB, 0.25955E+01_JPRB, 0.23792E+01_JPRB, 0.24173E+01_JPRB ,&
&  0.24146E+01_JPRB, 0.15222E+01_JPRB, 0.12800E+01_JPRB, 0.15405E+01_JPRB, 0.13245E+01_JPRB, 0.14015E+01_JPRB, 0.16750E+01_JPRB ,&
&  0.17053E+01_JPRB, 0.16841E+01_JPRB, 0.17585E+01_JPRB, 0.16839E+01_JPRB, 0.17206E+01_JPRB, 0.18734E+01_JPRB, 0.18108E+01_JPRB ,&
&  0.17106E+01_JPRB, 0.16100E+01_JPRB, 0.16321E+01_JPRB, 0.15597E+01_JPRB, 0.14563E+01_JPRB, 0.13566E+01_JPRB, 0.12750E+01_JPRB ,&
&  0.12023E+01_JPRB, 0.11502E+01_JPRB, 0.11111E+01_JPRB, 0.10387E+01_JPRB, 0.97697E+00_JPRB /)
ZBURD(:, 18, 12) = (/ &
&  0.10429E+01_JPRB, 0.10094E+01_JPRB, 0.98419E+00_JPRB, 0.97150E+00_JPRB, 0.96589E+00_JPRB, 0.96866E+00_JPRB, 0.96058E+00_JPRB ,&
&  0.94135E+00_JPRB, 0.89893E+00_JPRB, 0.85966E+00_JPRB, 0.85159E+00_JPRB, 0.83095E+00_JPRB, 0.80905E+00_JPRB, 0.81020E+00_JPRB ,&
&  0.83610E+00_JPRB, 0.96282E+00_JPRB, 0.84473E+00_JPRB, 0.79207E+00_JPRB, 0.74298E+00_JPRB, 0.79467E+00_JPRB, 0.85671E+00_JPRB ,&
&  0.98628E+00_JPRB, 0.10585E+01_JPRB, 0.11041E+01_JPRB, 0.11844E+01_JPRB, 0.13590E+01_JPRB, 0.15568E+01_JPRB, 0.17877E+01_JPRB ,&
&  0.19025E+01_JPRB, 0.21150E+01_JPRB, 0.21471E+01_JPRB, 0.24221E+01_JPRB, 0.23237E+01_JPRB, 0.20302E+01_JPRB, 0.19943E+01_JPRB ,&
&  0.17887E+01_JPRB, 0.15927E+01_JPRB, 0.14181E+01_JPRB, 0.13108E+01_JPRB, 0.12307E+01_JPRB, 0.11597E+01_JPRB, 0.10961E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.10466E+01_JPRB, 0.10359E+01_JPRB, 0.10286E+01_JPRB, 0.11180E+01_JPRB, 0.13732E+01_JPRB, 0.16814E+01_JPRB ,&
&  0.22020E+01_JPRB, 0.20011E+01_JPRB, 0.17878E+01_JPRB, 0.24422E+01_JPRB, 0.32651E+01_JPRB, 0.33371E+01_JPRB, 0.32371E+01_JPRB ,&
&  0.35543E+01_JPRB, 0.36057E+01_JPRB, 0.35641E+01_JPRB, 0.35781E+01_JPRB, 0.35000E+01_JPRB, 0.33678E+01_JPRB, 0.31860E+01_JPRB ,&
&  0.28469E+01_JPRB, 0.26884E+01_JPRB, 0.25940E+01_JPRB, 0.25709E+01_JPRB, 0.24011E+01_JPRB, 0.23071E+01_JPRB, 0.22855E+01_JPRB ,&
&  0.20620E+01_JPRB, 0.18717E+01_JPRB, 0.13835E+01_JPRB, 0.14496E+01_JPRB, 0.11833E+01_JPRB, 0.12208E+01_JPRB, 0.14676E+01_JPRB ,&
&  0.14811E+01_JPRB, 0.16136E+01_JPRB, 0.16714E+01_JPRB, 0.16169E+01_JPRB, 0.19025E+01_JPRB, 0.20295E+01_JPRB, 0.20376E+01_JPRB ,&
&  0.19644E+01_JPRB, 0.17184E+01_JPRB, 0.17736E+01_JPRB, 0.17836E+01_JPRB, 0.17150E+01_JPRB, 0.16167E+01_JPRB, 0.15132E+01_JPRB ,&
&  0.14098E+01_JPRB, 0.13192E+01_JPRB, 0.12523E+01_JPRB, 0.11677E+01_JPRB, 0.10942E+01_JPRB /)
ZBURD(:, 19, 12) = (/ &
&  0.11533E+01_JPRB, 0.11010E+01_JPRB, 0.10645E+01_JPRB, 0.10441E+01_JPRB, 0.10327E+01_JPRB, 0.10177E+01_JPRB, 0.99551E+00_JPRB ,&
&  0.96214E+00_JPRB, 0.90698E+00_JPRB, 0.87984E+00_JPRB, 0.86847E+00_JPRB, 0.83994E+00_JPRB, 0.82127E+00_JPRB, 0.83116E+00_JPRB ,&
&  0.86327E+00_JPRB, 0.88571E+00_JPRB, 0.75637E+00_JPRB, 0.68076E+00_JPRB, 0.64881E+00_JPRB, 0.70849E+00_JPRB, 0.78817E+00_JPRB ,&
&  0.89731E+00_JPRB, 0.98914E+00_JPRB, 0.11073E+01_JPRB, 0.12495E+01_JPRB, 0.14313E+01_JPRB, 0.16472E+01_JPRB, 0.20047E+01_JPRB ,&
&  0.25707E+01_JPRB, 0.23820E+01_JPRB, 0.25533E+01_JPRB, 0.25203E+01_JPRB, 0.24628E+01_JPRB, 0.22397E+01_JPRB, 0.20098E+01_JPRB ,&
&  0.17928E+01_JPRB, 0.16379E+01_JPRB, 0.14795E+01_JPRB, 0.13476E+01_JPRB, 0.12349E+01_JPRB, 0.11426E+01_JPRB, 0.10838E+01_JPRB ,&
&  0.10854E+01_JPRB, 0.11061E+01_JPRB, 0.11221E+01_JPRB, 0.11868E+01_JPRB, 0.13976E+01_JPRB, 0.16037E+01_JPRB, 0.18007E+01_JPRB ,&
&  0.18281E+01_JPRB, 0.18862E+01_JPRB, 0.25965E+01_JPRB, 0.27894E+01_JPRB, 0.29791E+01_JPRB, 0.31723E+01_JPRB, 0.32704E+01_JPRB ,&
&  0.34151E+01_JPRB, 0.32514E+01_JPRB, 0.31672E+01_JPRB, 0.27814E+01_JPRB, 0.31747E+01_JPRB, 0.30797E+01_JPRB, 0.28837E+01_JPRB ,&
&  0.26343E+01_JPRB, 0.25870E+01_JPRB, 0.24202E+01_JPRB, 0.23953E+01_JPRB, 0.23835E+01_JPRB, 0.24360E+01_JPRB, 0.19569E+01_JPRB ,&
&  0.16131E+01_JPRB, 0.18187E+01_JPRB, 0.16420E+01_JPRB, 0.14333E+01_JPRB, 0.12618E+01_JPRB, 0.13365E+01_JPRB, 0.14257E+01_JPRB ,&
&  0.14849E+01_JPRB, 0.15239E+01_JPRB, 0.15049E+01_JPRB, 0.16118E+01_JPRB, 0.19571E+01_JPRB, 0.22374E+01_JPRB, 0.22452E+01_JPRB ,&
&  0.20365E+01_JPRB, 0.20877E+01_JPRB, 0.21097E+01_JPRB, 0.20713E+01_JPRB, 0.20393E+01_JPRB, 0.19422E+01_JPRB, 0.18181E+01_JPRB ,&
&  0.16833E+01_JPRB, 0.15593E+01_JPRB, 0.14503E+01_JPRB, 0.13389E+01_JPRB, 0.12290E+01_JPRB /)
ZBURD(:, 20, 12) = (/ &
&  0.12692E+01_JPRB, 0.12104E+01_JPRB, 0.11590E+01_JPRB, 0.11216E+01_JPRB, 0.10873E+01_JPRB, 0.10585E+01_JPRB, 0.10261E+01_JPRB ,&
&  0.97985E+00_JPRB, 0.91427E+00_JPRB, 0.90329E+00_JPRB, 0.88668E+00_JPRB, 0.86277E+00_JPRB, 0.84905E+00_JPRB, 0.86301E+00_JPRB ,&
&  0.89921E+00_JPRB, 0.85792E+00_JPRB, 0.72119E+00_JPRB, 0.73574E+00_JPRB, 0.78331E+00_JPRB, 0.74788E+00_JPRB, 0.79397E+00_JPRB ,&
&  0.87005E+00_JPRB, 0.96962E+00_JPRB, 0.10795E+01_JPRB, 0.13352E+01_JPRB, 0.16371E+01_JPRB, 0.20281E+01_JPRB, 0.24098E+01_JPRB ,&
&  0.25178E+01_JPRB, 0.27217E+01_JPRB, 0.28663E+01_JPRB, 0.27038E+01_JPRB, 0.24443E+01_JPRB, 0.21590E+01_JPRB, 0.19215E+01_JPRB ,&
&  0.17492E+01_JPRB, 0.15747E+01_JPRB, 0.13947E+01_JPRB, 0.12522E+01_JPRB, 0.11497E+01_JPRB, 0.10844E+01_JPRB, 0.10529E+01_JPRB ,&
&  0.10567E+01_JPRB, 0.10793E+01_JPRB, 0.11181E+01_JPRB, 0.12764E+01_JPRB, 0.13655E+01_JPRB, 0.15131E+01_JPRB, 0.16097E+01_JPRB ,&
&  0.21344E+01_JPRB, 0.23629E+01_JPRB, 0.24910E+01_JPRB, 0.29253E+01_JPRB, 0.23514E+01_JPRB, 0.25291E+01_JPRB, 0.31987E+01_JPRB ,&
&  0.32993E+01_JPRB, 0.31953E+01_JPRB, 0.32225E+01_JPRB, 0.25344E+01_JPRB, 0.22487E+01_JPRB, 0.30538E+01_JPRB, 0.28841E+01_JPRB ,&
&  0.25855E+01_JPRB, 0.25113E+01_JPRB, 0.23889E+01_JPRB, 0.26227E+01_JPRB, 0.18717E+01_JPRB, 0.16207E+01_JPRB, 0.12901E+01_JPRB ,&
&  0.13038E+01_JPRB, 0.12329E+01_JPRB, 0.14449E+01_JPRB, 0.13749E+01_JPRB, 0.14006E+01_JPRB, 0.14006E+01_JPRB, 0.13456E+01_JPRB ,&
&  0.13873E+01_JPRB, 0.14100E+01_JPRB, 0.14444E+01_JPRB, 0.17612E+01_JPRB, 0.23585E+01_JPRB, 0.22770E+01_JPRB, 0.24944E+01_JPRB ,&
&  0.25234E+01_JPRB, 0.26226E+01_JPRB, 0.24428E+01_JPRB, 0.24315E+01_JPRB, 0.23422E+01_JPRB, 0.22018E+01_JPRB, 0.20138E+01_JPRB ,&
&  0.18639E+01_JPRB, 0.17280E+01_JPRB, 0.15852E+01_JPRB, 0.14578E+01_JPRB, 0.13517E+01_JPRB /)
ZBURD(:, 21, 12) = (/ &
&  0.13389E+01_JPRB, 0.12677E+01_JPRB, 0.12061E+01_JPRB, 0.11523E+01_JPRB, 0.11165E+01_JPRB, 0.10805E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.96661E+00_JPRB, 0.90666E+00_JPRB, 0.90469E+00_JPRB, 0.86887E+00_JPRB, 0.84940E+00_JPRB, 0.85368E+00_JPRB, 0.89202E+00_JPRB ,&
&  0.93352E+00_JPRB, 0.97180E+00_JPRB, 0.83963E+00_JPRB, 0.77544E+00_JPRB, 0.87824E+00_JPRB, 0.82321E+00_JPRB, 0.81788E+00_JPRB ,&
&  0.91189E+00_JPRB, 0.10176E+01_JPRB, 0.12198E+01_JPRB, 0.16771E+01_JPRB, 0.21214E+01_JPRB, 0.25924E+01_JPRB, 0.29375E+01_JPRB ,&
&  0.30977E+01_JPRB, 0.30364E+01_JPRB, 0.28656E+01_JPRB, 0.26420E+01_JPRB, 0.22538E+01_JPRB, 0.19286E+01_JPRB, 0.16955E+01_JPRB ,&
&  0.15074E+01_JPRB, 0.13500E+01_JPRB, 0.12386E+01_JPRB, 0.11329E+01_JPRB, 0.10597E+01_JPRB, 0.98845E+00_JPRB, 0.97028E+00_JPRB ,&
&  0.98971E+00_JPRB, 0.10426E+01_JPRB, 0.11209E+01_JPRB, 0.12800E+01_JPRB, 0.13494E+01_JPRB, 0.14137E+01_JPRB, 0.15777E+01_JPRB ,&
&  0.18855E+01_JPRB, 0.20184E+01_JPRB, 0.22346E+01_JPRB, 0.25243E+01_JPRB, 0.29412E+01_JPRB, 0.29575E+01_JPRB, 0.32520E+01_JPRB ,&
&  0.25303E+01_JPRB, 0.21745E+01_JPRB, 0.19867E+01_JPRB, 0.17039E+01_JPRB, 0.21277E+01_JPRB, 0.31848E+01_JPRB, 0.31002E+01_JPRB ,&
&  0.27781E+01_JPRB, 0.27033E+01_JPRB, 0.26829E+01_JPRB, 0.21526E+01_JPRB, 0.16739E+01_JPRB, 0.13272E+01_JPRB, 0.16453E+01_JPRB ,&
&  0.17436E+01_JPRB, 0.17624E+01_JPRB, 0.15844E+01_JPRB, 0.13636E+01_JPRB, 0.12482E+01_JPRB, 0.15538E+01_JPRB, 0.15891E+01_JPRB ,&
&  0.15124E+01_JPRB, 0.14965E+01_JPRB, 0.20128E+01_JPRB, 0.26755E+01_JPRB, 0.29619E+01_JPRB, 0.28645E+01_JPRB, 0.32543E+01_JPRB ,&
&  0.32743E+01_JPRB, 0.33181E+01_JPRB, 0.31746E+01_JPRB, 0.29485E+01_JPRB, 0.26802E+01_JPRB, 0.23915E+01_JPRB, 0.21596E+01_JPRB ,&
&  0.19709E+01_JPRB, 0.17986E+01_JPRB, 0.16437E+01_JPRB, 0.15134E+01_JPRB, 0.14170E+01_JPRB /)
ZBURD(:, 22, 12) = (/ &
&  0.13919E+01_JPRB, 0.13088E+01_JPRB, 0.12338E+01_JPRB, 0.11827E+01_JPRB, 0.11350E+01_JPRB, 0.10791E+01_JPRB, 0.10122E+01_JPRB ,&
&  0.92717E+00_JPRB, 0.90523E+00_JPRB, 0.89628E+00_JPRB, 0.85984E+00_JPRB, 0.85152E+00_JPRB, 0.86546E+00_JPRB, 0.90377E+00_JPRB ,&
&  0.96154E+00_JPRB, 0.11344E+01_JPRB, 0.97245E+00_JPRB, 0.88517E+00_JPRB, 0.90724E+00_JPRB, 0.90049E+00_JPRB, 0.83241E+00_JPRB ,&
&  0.95279E+00_JPRB, 0.10960E+01_JPRB, 0.13295E+01_JPRB, 0.18394E+01_JPRB, 0.26576E+01_JPRB, 0.27918E+01_JPRB, 0.29606E+01_JPRB ,&
&  0.30812E+01_JPRB, 0.28276E+01_JPRB, 0.26023E+01_JPRB, 0.23615E+01_JPRB, 0.20011E+01_JPRB, 0.17193E+01_JPRB, 0.15193E+01_JPRB ,&
&  0.13637E+01_JPRB, 0.12787E+01_JPRB, 0.12110E+01_JPRB, 0.11595E+01_JPRB, 0.10680E+01_JPRB, 0.10085E+01_JPRB, 0.98799E+00_JPRB ,&
&  0.99160E+00_JPRB, 0.10418E+01_JPRB, 0.11108E+01_JPRB, 0.12532E+01_JPRB, 0.14765E+01_JPRB, 0.14728E+01_JPRB, 0.18147E+01_JPRB ,&
&  0.18703E+01_JPRB, 0.20141E+01_JPRB, 0.20821E+01_JPRB, 0.24146E+01_JPRB, 0.26428E+01_JPRB, 0.26293E+01_JPRB, 0.27952E+01_JPRB ,&
&  0.23650E+01_JPRB, 0.23766E+01_JPRB, 0.26574E+01_JPRB, 0.28314E+01_JPRB, 0.22764E+01_JPRB, 0.23003E+01_JPRB, 0.32823E+01_JPRB ,&
&  0.23530E+01_JPRB, 0.24738E+01_JPRB, 0.26422E+01_JPRB, 0.26481E+01_JPRB, 0.11723E+01_JPRB, 0.91667E+00_JPRB, 0.13746E+01_JPRB ,&
&  0.14555E+01_JPRB, 0.98928E+00_JPRB, 0.81196E+00_JPRB, 0.97668E+00_JPRB, 0.85097E+00_JPRB, 0.11011E+01_JPRB, 0.17100E+01_JPRB ,&
&  0.18853E+01_JPRB, 0.20259E+01_JPRB, 0.30620E+01_JPRB, 0.34829E+01_JPRB, 0.33616E+01_JPRB, 0.36268E+01_JPRB, 0.37810E+01_JPRB ,&
&  0.37652E+01_JPRB, 0.34250E+01_JPRB, 0.34614E+01_JPRB, 0.31805E+01_JPRB, 0.28416E+01_JPRB, 0.25170E+01_JPRB, 0.22420E+01_JPRB ,&
&  0.19891E+01_JPRB, 0.18053E+01_JPRB, 0.16785E+01_JPRB, 0.15580E+01_JPRB, 0.14636E+01_JPRB /)
ZBURD(:, 23, 12) = (/ &
&  0.13327E+01_JPRB, 0.12509E+01_JPRB, 0.11665E+01_JPRB, 0.11084E+01_JPRB, 0.10699E+01_JPRB, 0.10210E+01_JPRB, 0.95959E+00_JPRB ,&
&  0.89680E+00_JPRB, 0.91205E+00_JPRB, 0.88145E+00_JPRB, 0.85654E+00_JPRB, 0.84068E+00_JPRB, 0.85723E+00_JPRB, 0.90893E+00_JPRB ,&
&  0.96073E+00_JPRB, 0.10510E+01_JPRB, 0.11053E+01_JPRB, 0.99637E+00_JPRB, 0.95923E+00_JPRB, 0.91655E+00_JPRB, 0.92143E+00_JPRB ,&
&  0.10706E+01_JPRB, 0.12500E+01_JPRB, 0.15306E+01_JPRB, 0.19641E+01_JPRB, 0.24278E+01_JPRB, 0.25751E+01_JPRB, 0.28342E+01_JPRB ,&
&  0.26957E+01_JPRB, 0.24574E+01_JPRB, 0.22975E+01_JPRB, 0.20620E+01_JPRB, 0.17405E+01_JPRB, 0.15171E+01_JPRB, 0.13606E+01_JPRB ,&
&  0.12525E+01_JPRB, 0.12052E+01_JPRB, 0.11770E+01_JPRB, 0.10826E+01_JPRB, 0.10034E+01_JPRB, 0.97130E+00_JPRB, 0.95004E+00_JPRB ,&
&  0.96451E+00_JPRB, 0.10273E+01_JPRB, 0.10948E+01_JPRB, 0.11852E+01_JPRB, 0.13280E+01_JPRB, 0.13953E+01_JPRB, 0.13775E+01_JPRB ,&
&  0.14087E+01_JPRB, 0.15486E+01_JPRB, 0.19907E+01_JPRB, 0.21297E+01_JPRB, 0.21607E+01_JPRB, 0.22057E+01_JPRB, 0.21631E+01_JPRB ,&
&  0.22614E+01_JPRB, 0.25659E+01_JPRB, 0.22822E+01_JPRB, 0.25369E+01_JPRB, 0.24442E+01_JPRB, 0.18945E+01_JPRB, 0.21190E+01_JPRB ,&
&  0.22268E+01_JPRB, 0.20234E+01_JPRB, 0.16722E+01_JPRB, 0.13049E+01_JPRB, 0.13534E+01_JPRB, 0.11927E+01_JPRB, 0.82263E+00_JPRB ,&
&  0.75957E+00_JPRB, 0.73134E+00_JPRB, 0.70657E+00_JPRB, 0.72883E+00_JPRB, 0.75560E+00_JPRB, 0.10675E+01_JPRB, 0.15892E+01_JPRB ,&
&  0.24176E+01_JPRB, 0.31401E+01_JPRB, 0.38091E+01_JPRB, 0.39336E+01_JPRB, 0.35005E+01_JPRB, 0.35752E+01_JPRB, 0.39005E+01_JPRB ,&
&  0.33855E+01_JPRB, 0.31811E+01_JPRB, 0.31419E+01_JPRB, 0.28629E+01_JPRB, 0.25786E+01_JPRB, 0.23324E+01_JPRB, 0.20975E+01_JPRB ,&
&  0.18961E+01_JPRB, 0.17494E+01_JPRB, 0.16338E+01_JPRB, 0.15236E+01_JPRB, 0.14226E+01_JPRB /)
ZBURD(:, 24, 12) = (/ &
&  0.12231E+01_JPRB, 0.11654E+01_JPRB, 0.11229E+01_JPRB, 0.10844E+01_JPRB, 0.10377E+01_JPRB, 0.99759E+00_JPRB, 0.92915E+00_JPRB ,&
&  0.90439E+00_JPRB, 0.90331E+00_JPRB, 0.85811E+00_JPRB, 0.82714E+00_JPRB, 0.81307E+00_JPRB, 0.83703E+00_JPRB, 0.89465E+00_JPRB ,&
&  0.94560E+00_JPRB, 0.10004E+01_JPRB, 0.11643E+01_JPRB, 0.11215E+01_JPRB, 0.11842E+01_JPRB, 0.10799E+01_JPRB, 0.11574E+01_JPRB ,&
&  0.12637E+01_JPRB, 0.15044E+01_JPRB, 0.19221E+01_JPRB, 0.21922E+01_JPRB, 0.25114E+01_JPRB, 0.26428E+01_JPRB, 0.26053E+01_JPRB ,&
&  0.24364E+01_JPRB, 0.22849E+01_JPRB, 0.20181E+01_JPRB, 0.16877E+01_JPRB, 0.14387E+01_JPRB, 0.12745E+01_JPRB, 0.11711E+01_JPRB ,&
&  0.11305E+01_JPRB, 0.11328E+01_JPRB, 0.10600E+01_JPRB, 0.10055E+01_JPRB, 0.98513E+00_JPRB, 0.97525E+00_JPRB, 0.96249E+00_JPRB ,&
&  0.98595E+00_JPRB, 0.10349E+01_JPRB, 0.11002E+01_JPRB, 0.11628E+01_JPRB, 0.10815E+01_JPRB, 0.10133E+01_JPRB, 0.12283E+01_JPRB ,&
&  0.14448E+01_JPRB, 0.16998E+01_JPRB, 0.17629E+01_JPRB, 0.20739E+01_JPRB, 0.20092E+01_JPRB, 0.18951E+01_JPRB, 0.20013E+01_JPRB ,&
&  0.21216E+01_JPRB, 0.23953E+01_JPRB, 0.19778E+01_JPRB, 0.21355E+01_JPRB, 0.25347E+01_JPRB, 0.20062E+01_JPRB, 0.17552E+01_JPRB ,&
&  0.19751E+01_JPRB, 0.17546E+01_JPRB, 0.15957E+01_JPRB, 0.14232E+01_JPRB, 0.19578E+01_JPRB, 0.22560E+01_JPRB, 0.11029E+01_JPRB ,&
&  0.87566E+00_JPRB, 0.83525E+00_JPRB, 0.80535E+00_JPRB, 0.76621E+00_JPRB, 0.81876E+00_JPRB, 0.12028E+01_JPRB, 0.21243E+01_JPRB ,&
&  0.27272E+01_JPRB, 0.39238E+01_JPRB, 0.44040E+01_JPRB, 0.43104E+01_JPRB, 0.34667E+01_JPRB, 0.34100E+01_JPRB, 0.31677E+01_JPRB ,&
&  0.33226E+01_JPRB, 0.32231E+01_JPRB, 0.29577E+01_JPRB, 0.26717E+01_JPRB, 0.23955E+01_JPRB, 0.21473E+01_JPRB, 0.19291E+01_JPRB ,&
&  0.17430E+01_JPRB, 0.16012E+01_JPRB, 0.14836E+01_JPRB, 0.13882E+01_JPRB, 0.12967E+01_JPRB /)
ZBURD(:, 25, 12) = (/ &
&  0.11931E+01_JPRB, 0.11404E+01_JPRB, 0.10933E+01_JPRB, 0.10444E+01_JPRB, 0.10114E+01_JPRB, 0.96729E+00_JPRB, 0.90751E+00_JPRB ,&
&  0.92335E+00_JPRB, 0.88457E+00_JPRB, 0.83846E+00_JPRB, 0.81275E+00_JPRB, 0.81028E+00_JPRB, 0.83198E+00_JPRB, 0.87357E+00_JPRB ,&
&  0.93429E+00_JPRB, 0.96440E+00_JPRB, 0.10697E+01_JPRB, 0.12347E+01_JPRB, 0.12929E+01_JPRB, 0.12073E+01_JPRB, 0.13837E+01_JPRB ,&
&  0.14996E+01_JPRB, 0.18042E+01_JPRB, 0.22643E+01_JPRB, 0.24396E+01_JPRB, 0.24684E+01_JPRB, 0.25780E+01_JPRB, 0.23897E+01_JPRB ,&
&  0.22105E+01_JPRB, 0.19495E+01_JPRB, 0.16149E+01_JPRB, 0.13688E+01_JPRB, 0.12061E+01_JPRB, 0.11500E+01_JPRB, 0.10988E+01_JPRB ,&
&  0.10870E+01_JPRB, 0.10429E+01_JPRB, 0.10007E+01_JPRB, 0.99582E+00_JPRB, 0.10025E+01_JPRB, 0.10050E+01_JPRB, 0.10090E+01_JPRB ,&
&  0.10368E+01_JPRB, 0.10698E+01_JPRB, 0.11389E+01_JPRB, 0.11944E+01_JPRB, 0.10266E+01_JPRB, 0.12205E+01_JPRB, 0.13196E+01_JPRB ,&
&  0.14243E+01_JPRB, 0.15214E+01_JPRB, 0.15018E+01_JPRB, 0.17133E+01_JPRB, 0.18365E+01_JPRB, 0.17865E+01_JPRB, 0.18294E+01_JPRB ,&
&  0.20274E+01_JPRB, 0.19002E+01_JPRB, 0.17933E+01_JPRB, 0.20051E+01_JPRB, 0.21773E+01_JPRB, 0.25681E+01_JPRB, 0.15636E+01_JPRB ,&
&  0.13569E+01_JPRB, 0.16658E+01_JPRB, 0.16832E+01_JPRB, 0.17202E+01_JPRB, 0.22078E+01_JPRB, 0.31696E+01_JPRB, 0.27339E+01_JPRB ,&
&  0.11073E+01_JPRB, 0.88253E+00_JPRB, 0.10001E+01_JPRB, 0.10172E+01_JPRB, 0.96536E+00_JPRB, 0.14650E+01_JPRB, 0.40952E+01_JPRB ,&
&  0.38414E+01_JPRB, 0.46042E+01_JPRB, 0.44713E+01_JPRB, 0.37547E+01_JPRB, 0.31490E+01_JPRB, 0.30477E+01_JPRB, 0.32300E+01_JPRB ,&
&  0.30711E+01_JPRB, 0.28036E+01_JPRB, 0.25658E+01_JPRB, 0.23634E+01_JPRB, 0.21128E+01_JPRB, 0.18563E+01_JPRB, 0.16354E+01_JPRB ,&
&  0.14804E+01_JPRB, 0.13915E+01_JPRB, 0.13397E+01_JPRB, 0.12920E+01_JPRB, 0.12432E+01_JPRB /)
ZBURD(:, 26, 12) = (/ &
&  0.11150E+01_JPRB, 0.10709E+01_JPRB, 0.10380E+01_JPRB, 0.10126E+01_JPRB, 0.10008E+01_JPRB, 0.96165E+00_JPRB, 0.92078E+00_JPRB ,&
&  0.95400E+00_JPRB, 0.88154E+00_JPRB, 0.84097E+00_JPRB, 0.81312E+00_JPRB, 0.81760E+00_JPRB, 0.84785E+00_JPRB, 0.87853E+00_JPRB ,&
&  0.90562E+00_JPRB, 0.94969E+00_JPRB, 0.10678E+01_JPRB, 0.12912E+01_JPRB, 0.14124E+01_JPRB, 0.13484E+01_JPRB, 0.14352E+01_JPRB ,&
&  0.18044E+01_JPRB, 0.20785E+01_JPRB, 0.21231E+01_JPRB, 0.22282E+01_JPRB, 0.24512E+01_JPRB, 0.25670E+01_JPRB, 0.22157E+01_JPRB ,&
&  0.19095E+01_JPRB, 0.16049E+01_JPRB, 0.13514E+01_JPRB, 0.12059E+01_JPRB, 0.11612E+01_JPRB, 0.11333E+01_JPRB, 0.11076E+01_JPRB ,&
&  0.10608E+01_JPRB, 0.10368E+01_JPRB, 0.10357E+01_JPRB, 0.10412E+01_JPRB, 0.10568E+01_JPRB, 0.10463E+01_JPRB, 0.10624E+01_JPRB ,&
&  0.11022E+01_JPRB, 0.11403E+01_JPRB, 0.11919E+01_JPRB, 0.11711E+01_JPRB, 0.12157E+01_JPRB, 0.12630E+01_JPRB, 0.13483E+01_JPRB ,&
&  0.13728E+01_JPRB, 0.14450E+01_JPRB, 0.15297E+01_JPRB, 0.16001E+01_JPRB, 0.17315E+01_JPRB, 0.17961E+01_JPRB, 0.18108E+01_JPRB ,&
&  0.18871E+01_JPRB, 0.18509E+01_JPRB, 0.16998E+01_JPRB, 0.18136E+01_JPRB, 0.20189E+01_JPRB, 0.22928E+01_JPRB, 0.19254E+01_JPRB ,&
&  0.16567E+01_JPRB, 0.15399E+01_JPRB, 0.17913E+01_JPRB, 0.23758E+01_JPRB, 0.28341E+01_JPRB, 0.36114E+01_JPRB, 0.47072E+01_JPRB ,&
&  0.51795E+01_JPRB, 0.28117E+01_JPRB, 0.22410E+01_JPRB, 0.29309E+01_JPRB, 0.23733E+01_JPRB, 0.22658E+01_JPRB, 0.34337E+01_JPRB ,&
&  0.41062E+01_JPRB, 0.44464E+01_JPRB, 0.40951E+01_JPRB, 0.31120E+01_JPRB, 0.29000E+01_JPRB, 0.30011E+01_JPRB, 0.29601E+01_JPRB ,&
&  0.26317E+01_JPRB, 0.23951E+01_JPRB, 0.21464E+01_JPRB, 0.18741E+01_JPRB, 0.16428E+01_JPRB, 0.14736E+01_JPRB, 0.13702E+01_JPRB ,&
&  0.13149E+01_JPRB, 0.12848E+01_JPRB, 0.12595E+01_JPRB, 0.12162E+01_JPRB, 0.11659E+01_JPRB /)
ZBURD(:, 27, 12) = (/ &
&  0.10426E+01_JPRB, 0.10276E+01_JPRB, 0.10291E+01_JPRB, 0.10251E+01_JPRB, 0.10073E+01_JPRB, 0.96674E+00_JPRB, 0.97053E+00_JPRB ,&
&  0.10003E+01_JPRB, 0.90840E+00_JPRB, 0.84906E+00_JPRB, 0.82696E+00_JPRB, 0.83875E+00_JPRB, 0.87428E+00_JPRB, 0.89610E+00_JPRB ,&
&  0.91911E+00_JPRB, 0.98214E+00_JPRB, 0.11851E+01_JPRB, 0.13994E+01_JPRB, 0.16012E+01_JPRB, 0.19761E+01_JPRB, 0.14312E+01_JPRB ,&
&  0.15840E+01_JPRB, 0.20374E+01_JPRB, 0.20531E+01_JPRB, 0.22513E+01_JPRB, 0.23715E+01_JPRB, 0.23008E+01_JPRB, 0.19549E+01_JPRB ,&
&  0.15938E+01_JPRB, 0.13814E+01_JPRB, 0.12679E+01_JPRB, 0.12027E+01_JPRB, 0.11869E+01_JPRB, 0.11765E+01_JPRB, 0.11329E+01_JPRB ,&
&  0.11204E+01_JPRB, 0.11184E+01_JPRB, 0.10997E+01_JPRB, 0.11100E+01_JPRB, 0.11170E+01_JPRB, 0.11099E+01_JPRB, 0.11377E+01_JPRB ,&
&  0.11740E+01_JPRB, 0.12043E+01_JPRB, 0.12274E+01_JPRB, 0.11972E+01_JPRB, 0.12517E+01_JPRB, 0.12949E+01_JPRB, 0.13430E+01_JPRB ,&
&  0.12858E+01_JPRB, 0.12274E+01_JPRB, 0.14139E+01_JPRB, 0.16115E+01_JPRB, 0.17247E+01_JPRB, 0.16875E+01_JPRB, 0.16981E+01_JPRB ,&
&  0.18698E+01_JPRB, 0.19510E+01_JPRB, 0.19697E+01_JPRB, 0.17552E+01_JPRB, 0.18737E+01_JPRB, 0.21039E+01_JPRB, 0.23081E+01_JPRB ,&
&  0.24375E+01_JPRB, 0.24093E+01_JPRB, 0.26945E+01_JPRB, 0.29688E+01_JPRB, 0.32423E+01_JPRB, 0.39678E+01_JPRB, 0.41621E+01_JPRB ,&
&  0.49089E+01_JPRB, 0.59262E+01_JPRB, 0.53101E+01_JPRB, 0.43802E+01_JPRB, 0.40552E+01_JPRB, 0.37477E+01_JPRB, 0.38783E+01_JPRB ,&
&  0.44125E+01_JPRB, 0.39870E+01_JPRB, 0.33170E+01_JPRB, 0.32064E+01_JPRB, 0.27749E+01_JPRB, 0.26773E+01_JPRB, 0.24338E+01_JPRB ,&
&  0.20308E+01_JPRB, 0.17430E+01_JPRB, 0.14674E+01_JPRB, 0.12943E+01_JPRB, 0.12044E+01_JPRB, 0.11797E+01_JPRB, 0.11717E+01_JPRB ,&
&  0.11724E+01_JPRB, 0.11631E+01_JPRB, 0.11458E+01_JPRB, 0.11194E+01_JPRB, 0.10791E+01_JPRB /)
ZBURD(:, 28, 12) = (/ &
&  0.10269E+01_JPRB, 0.10275E+01_JPRB, 0.10478E+01_JPRB, 0.10551E+01_JPRB, 0.10356E+01_JPRB, 0.10252E+01_JPRB, 0.10452E+01_JPRB ,&
&  0.10498E+01_JPRB, 0.91252E+00_JPRB, 0.89293E+00_JPRB, 0.89991E+00_JPRB, 0.92215E+00_JPRB, 0.93247E+00_JPRB, 0.94200E+00_JPRB ,&
&  0.98543E+00_JPRB, 0.11225E+01_JPRB, 0.13523E+01_JPRB, 0.15155E+01_JPRB, 0.16710E+01_JPRB, 0.19228E+01_JPRB, 0.18614E+01_JPRB ,&
&  0.14843E+01_JPRB, 0.22952E+01_JPRB, 0.20775E+01_JPRB, 0.23078E+01_JPRB, 0.22679E+01_JPRB, 0.21631E+01_JPRB, 0.18178E+01_JPRB ,&
&  0.14931E+01_JPRB, 0.13724E+01_JPRB, 0.13734E+01_JPRB, 0.13134E+01_JPRB, 0.12642E+01_JPRB, 0.12012E+01_JPRB, 0.12017E+01_JPRB ,&
&  0.11974E+01_JPRB, 0.11872E+01_JPRB, 0.11786E+01_JPRB, 0.11931E+01_JPRB, 0.12007E+01_JPRB, 0.11959E+01_JPRB, 0.12011E+01_JPRB ,&
&  0.12171E+01_JPRB, 0.12317E+01_JPRB, 0.12199E+01_JPRB, 0.12422E+01_JPRB, 0.12679E+01_JPRB, 0.13240E+01_JPRB, 0.13661E+01_JPRB ,&
&  0.13367E+01_JPRB, 0.13023E+01_JPRB, 0.14159E+01_JPRB, 0.14949E+01_JPRB, 0.15486E+01_JPRB, 0.16897E+01_JPRB, 0.16869E+01_JPRB ,&
&  0.19028E+01_JPRB, 0.18707E+01_JPRB, 0.21526E+01_JPRB, 0.17327E+01_JPRB, 0.18180E+01_JPRB, 0.20362E+01_JPRB, 0.21500E+01_JPRB ,&
&  0.20708E+01_JPRB, 0.24594E+01_JPRB, 0.27679E+01_JPRB, 0.31915E+01_JPRB, 0.35208E+01_JPRB, 0.39144E+01_JPRB, 0.37501E+01_JPRB ,&
&  0.38658E+01_JPRB, 0.41757E+01_JPRB, 0.44069E+01_JPRB, 0.38232E+01_JPRB, 0.37431E+01_JPRB, 0.34591E+01_JPRB, 0.39482E+01_JPRB ,&
&  0.42050E+01_JPRB, 0.37450E+01_JPRB, 0.34342E+01_JPRB, 0.30444E+01_JPRB, 0.23620E+01_JPRB, 0.20599E+01_JPRB, 0.17204E+01_JPRB ,&
&  0.13981E+01_JPRB, 0.12012E+01_JPRB, 0.10394E+01_JPRB, 0.10089E+01_JPRB, 0.10201E+01_JPRB, 0.10336E+01_JPRB, 0.10656E+01_JPRB ,&
&  0.10695E+01_JPRB, 0.10679E+01_JPRB, 0.10688E+01_JPRB, 0.10588E+01_JPRB, 0.10380E+01_JPRB /)
ZBURD(:, 29, 12) = (/ &
&  0.10381E+01_JPRB, 0.10442E+01_JPRB, 0.10708E+01_JPRB, 0.11002E+01_JPRB, 0.11236E+01_JPRB, 0.11631E+01_JPRB, 0.11291E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.94309E+00_JPRB, 0.96778E+00_JPRB, 0.98412E+00_JPRB, 0.10010E+01_JPRB, 0.99458E+00_JPRB, 0.10205E+01_JPRB ,&
&  0.11250E+01_JPRB, 0.12954E+01_JPRB, 0.14856E+01_JPRB, 0.16733E+01_JPRB, 0.18737E+01_JPRB, 0.21341E+01_JPRB, 0.21063E+01_JPRB ,&
&  0.16568E+01_JPRB, 0.17597E+01_JPRB, 0.23373E+01_JPRB, 0.23164E+01_JPRB, 0.21654E+01_JPRB, 0.20321E+01_JPRB, 0.18528E+01_JPRB ,&
&  0.16449E+01_JPRB, 0.15468E+01_JPRB, 0.15874E+01_JPRB, 0.14387E+01_JPRB, 0.12552E+01_JPRB, 0.12520E+01_JPRB, 0.12700E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.12699E+01_JPRB, 0.12717E+01_JPRB, 0.12891E+01_JPRB, 0.13014E+01_JPRB, 0.12907E+01_JPRB, 0.12721E+01_JPRB ,&
&  0.12726E+01_JPRB, 0.12843E+01_JPRB, 0.12823E+01_JPRB, 0.12696E+01_JPRB, 0.13114E+01_JPRB, 0.13612E+01_JPRB, 0.13934E+01_JPRB ,&
&  0.14145E+01_JPRB, 0.14733E+01_JPRB, 0.15239E+01_JPRB, 0.15626E+01_JPRB, 0.15309E+01_JPRB, 0.17239E+01_JPRB, 0.17654E+01_JPRB ,&
&  0.18821E+01_JPRB, 0.19380E+01_JPRB, 0.21743E+01_JPRB, 0.17076E+01_JPRB, 0.18894E+01_JPRB, 0.20515E+01_JPRB, 0.22295E+01_JPRB ,&
&  0.23161E+01_JPRB, 0.26319E+01_JPRB, 0.28515E+01_JPRB, 0.31086E+01_JPRB, 0.33607E+01_JPRB, 0.32849E+01_JPRB, 0.34798E+01_JPRB ,&
&  0.33533E+01_JPRB, 0.38681E+01_JPRB, 0.37910E+01_JPRB, 0.36472E+01_JPRB, 0.32201E+01_JPRB, 0.31513E+01_JPRB, 0.34235E+01_JPRB ,&
&  0.36624E+01_JPRB, 0.31999E+01_JPRB, 0.28006E+01_JPRB, 0.22749E+01_JPRB, 0.17291E+01_JPRB, 0.14358E+01_JPRB, 0.11800E+01_JPRB ,&
&  0.10624E+01_JPRB, 0.96124E+00_JPRB, 0.92807E+00_JPRB, 0.95840E+00_JPRB, 0.97296E+00_JPRB, 0.10206E+01_JPRB, 0.10311E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10346E+01_JPRB, 0.10483E+01_JPRB, 0.10528E+01_JPRB, 0.10470E+01_JPRB /)
ZBURD(:, 30, 12) = (/ &
&  0.10063E+01_JPRB, 0.99998E+00_JPRB, 0.99520E+00_JPRB, 0.99960E+00_JPRB, 0.10184E+01_JPRB, 0.10299E+01_JPRB, 0.99114E+00_JPRB ,&
&  0.98517E+00_JPRB, 0.99872E+00_JPRB, 0.10428E+01_JPRB, 0.10619E+01_JPRB, 0.10737E+01_JPRB, 0.10864E+01_JPRB, 0.11465E+01_JPRB ,&
&  0.12583E+01_JPRB, 0.14290E+01_JPRB, 0.16199E+01_JPRB, 0.17671E+01_JPRB, 0.19244E+01_JPRB, 0.21116E+01_JPRB, 0.22964E+01_JPRB ,&
&  0.20153E+01_JPRB, 0.16289E+01_JPRB, 0.21109E+01_JPRB, 0.21972E+01_JPRB, 0.20699E+01_JPRB, 0.20125E+01_JPRB, 0.20497E+01_JPRB ,&
&  0.16770E+01_JPRB, 0.15677E+01_JPRB, 0.15720E+01_JPRB, 0.14765E+01_JPRB, 0.13105E+01_JPRB, 0.13221E+01_JPRB, 0.13418E+01_JPRB ,&
&  0.13499E+01_JPRB, 0.13455E+01_JPRB, 0.13663E+01_JPRB, 0.13907E+01_JPRB, 0.14074E+01_JPRB, 0.13961E+01_JPRB, 0.13701E+01_JPRB ,&
&  0.13771E+01_JPRB, 0.13918E+01_JPRB, 0.14110E+01_JPRB, 0.14086E+01_JPRB, 0.14326E+01_JPRB, 0.14780E+01_JPRB, 0.15128E+01_JPRB ,&
&  0.15439E+01_JPRB, 0.15442E+01_JPRB, 0.16114E+01_JPRB, 0.17334E+01_JPRB, 0.18477E+01_JPRB, 0.16910E+01_JPRB, 0.18238E+01_JPRB ,&
&  0.18523E+01_JPRB, 0.19041E+01_JPRB, 0.19057E+01_JPRB, 0.21878E+01_JPRB, 0.15941E+01_JPRB, 0.18886E+01_JPRB, 0.20279E+01_JPRB ,&
&  0.25534E+01_JPRB, 0.26370E+01_JPRB, 0.27728E+01_JPRB, 0.29676E+01_JPRB, 0.31333E+01_JPRB, 0.29375E+01_JPRB, 0.30068E+01_JPRB ,&
&  0.31975E+01_JPRB, 0.33216E+01_JPRB, 0.33583E+01_JPRB, 0.33767E+01_JPRB, 0.31409E+01_JPRB, 0.31615E+01_JPRB, 0.31429E+01_JPRB ,&
&  0.31378E+01_JPRB, 0.24849E+01_JPRB, 0.20026E+01_JPRB, 0.15675E+01_JPRB, 0.12139E+01_JPRB, 0.10592E+01_JPRB, 0.10023E+01_JPRB ,&
&  0.98698E+00_JPRB, 0.10038E+01_JPRB, 0.97604E+00_JPRB, 0.96779E+00_JPRB, 0.98539E+00_JPRB, 0.10251E+01_JPRB, 0.10192E+01_JPRB ,&
&  0.10286E+01_JPRB, 0.10304E+01_JPRB, 0.10415E+01_JPRB, 0.10409E+01_JPRB, 0.10299E+01_JPRB /)
ZBURD(:, 31, 12) = (/ &
&  0.99017E+00_JPRB, 0.97786E+00_JPRB, 0.95755E+00_JPRB, 0.96470E+00_JPRB, 0.97007E+00_JPRB, 0.96807E+00_JPRB, 0.96320E+00_JPRB ,&
&  0.98293E+00_JPRB, 0.10292E+01_JPRB, 0.10802E+01_JPRB, 0.11126E+01_JPRB, 0.11245E+01_JPRB, 0.11426E+01_JPRB, 0.12064E+01_JPRB ,&
&  0.13090E+01_JPRB, 0.14631E+01_JPRB, 0.16638E+01_JPRB, 0.18323E+01_JPRB, 0.19788E+01_JPRB, 0.20850E+01_JPRB, 0.22309E+01_JPRB ,&
&  0.22412E+01_JPRB, 0.22593E+01_JPRB, 0.21365E+01_JPRB, 0.16552E+01_JPRB, 0.15497E+01_JPRB, 0.18401E+01_JPRB, 0.16642E+01_JPRB ,&
&  0.15487E+01_JPRB, 0.14926E+01_JPRB, 0.14530E+01_JPRB, 0.14096E+01_JPRB, 0.13782E+01_JPRB, 0.14022E+01_JPRB, 0.14252E+01_JPRB ,&
&  0.14215E+01_JPRB, 0.14401E+01_JPRB, 0.14627E+01_JPRB, 0.14828E+01_JPRB, 0.14992E+01_JPRB, 0.15293E+01_JPRB, 0.15331E+01_JPRB ,&
&  0.15712E+01_JPRB, 0.15985E+01_JPRB, 0.16337E+01_JPRB, 0.16519E+01_JPRB, 0.16292E+01_JPRB, 0.16633E+01_JPRB, 0.16845E+01_JPRB ,&
&  0.17168E+01_JPRB, 0.17138E+01_JPRB, 0.17073E+01_JPRB, 0.17749E+01_JPRB, 0.18131E+01_JPRB, 0.16202E+01_JPRB, 0.16537E+01_JPRB ,&
&  0.18160E+01_JPRB, 0.19008E+01_JPRB, 0.17626E+01_JPRB, 0.21673E+01_JPRB, 0.14172E+01_JPRB, 0.18793E+01_JPRB, 0.23807E+01_JPRB ,&
&  0.24633E+01_JPRB, 0.25396E+01_JPRB, 0.26347E+01_JPRB, 0.27270E+01_JPRB, 0.27746E+01_JPRB, 0.26253E+01_JPRB, 0.28433E+01_JPRB ,&
&  0.30604E+01_JPRB, 0.30309E+01_JPRB, 0.30057E+01_JPRB, 0.29908E+01_JPRB, 0.29199E+01_JPRB, 0.26853E+01_JPRB, 0.23727E+01_JPRB ,&
&  0.19089E+01_JPRB, 0.19090E+01_JPRB, 0.15355E+01_JPRB, 0.13202E+01_JPRB, 0.10843E+01_JPRB, 0.93700E+00_JPRB, 0.95426E+00_JPRB ,&
&  0.98327E+00_JPRB, 0.98848E+00_JPRB, 0.97729E+00_JPRB, 0.99197E+00_JPRB, 0.10026E+01_JPRB, 0.10154E+01_JPRB, 0.10089E+01_JPRB ,&
&  0.10164E+01_JPRB, 0.10238E+01_JPRB, 0.10274E+01_JPRB, 0.10181E+01_JPRB, 0.10051E+01_JPRB /)
ZBURD(:, 32, 12) = (/ &
&  0.93745E+00_JPRB, 0.92303E+00_JPRB, 0.90890E+00_JPRB, 0.94001E+00_JPRB, 0.93357E+00_JPRB, 0.92231E+00_JPRB, 0.93313E+00_JPRB ,&
&  0.95201E+00_JPRB, 0.98050E+00_JPRB, 0.10194E+01_JPRB, 0.10608E+01_JPRB, 0.11126E+01_JPRB, 0.11855E+01_JPRB, 0.12570E+01_JPRB ,&
&  0.13421E+01_JPRB, 0.14383E+01_JPRB, 0.15994E+01_JPRB, 0.18067E+01_JPRB, 0.20590E+01_JPRB, 0.21979E+01_JPRB, 0.22241E+01_JPRB ,&
&  0.21092E+01_JPRB, 0.21357E+01_JPRB, 0.21603E+01_JPRB, 0.21079E+01_JPRB, 0.17963E+01_JPRB, 0.18618E+01_JPRB, 0.18901E+01_JPRB ,&
&  0.18489E+01_JPRB, 0.16442E+01_JPRB, 0.14970E+01_JPRB, 0.14595E+01_JPRB, 0.14377E+01_JPRB, 0.14633E+01_JPRB, 0.14932E+01_JPRB ,&
&  0.15056E+01_JPRB, 0.15309E+01_JPRB, 0.15471E+01_JPRB, 0.15682E+01_JPRB, 0.16131E+01_JPRB, 0.16944E+01_JPRB, 0.17313E+01_JPRB ,&
&  0.17488E+01_JPRB, 0.17934E+01_JPRB, 0.17906E+01_JPRB, 0.17495E+01_JPRB, 0.17809E+01_JPRB, 0.18071E+01_JPRB, 0.18318E+01_JPRB ,&
&  0.18874E+01_JPRB, 0.18260E+01_JPRB, 0.18689E+01_JPRB, 0.19259E+01_JPRB, 0.18734E+01_JPRB, 0.17300E+01_JPRB, 0.17393E+01_JPRB ,&
&  0.18486E+01_JPRB, 0.19329E+01_JPRB, 0.14987E+01_JPRB, 0.20773E+01_JPRB, 0.25321E+01_JPRB, 0.23186E+01_JPRB, 0.23669E+01_JPRB ,&
&  0.23647E+01_JPRB, 0.24323E+01_JPRB, 0.25495E+01_JPRB, 0.26533E+01_JPRB, 0.27144E+01_JPRB, 0.27649E+01_JPRB, 0.25998E+01_JPRB ,&
&  0.29979E+01_JPRB, 0.29030E+01_JPRB, 0.27500E+01_JPRB, 0.26111E+01_JPRB, 0.24165E+01_JPRB, 0.20656E+01_JPRB, 0.17198E+01_JPRB ,&
&  0.14082E+01_JPRB, 0.16100E+01_JPRB, 0.13766E+01_JPRB, 0.13762E+01_JPRB, 0.13112E+01_JPRB, 0.91612E+00_JPRB, 0.96229E+00_JPRB ,&
&  0.97689E+00_JPRB, 0.98330E+00_JPRB, 0.97937E+00_JPRB, 0.96260E+00_JPRB, 0.96792E+00_JPRB, 0.97700E+00_JPRB, 0.98543E+00_JPRB ,&
&  0.98525E+00_JPRB, 0.97723E+00_JPRB, 0.97478E+00_JPRB, 0.95931E+00_JPRB, 0.95033E+00_JPRB /)
ZBURD(:, 33, 12) = (/ &
&  0.91789E+00_JPRB, 0.91414E+00_JPRB, 0.90392E+00_JPRB, 0.91611E+00_JPRB, 0.90641E+00_JPRB, 0.92397E+00_JPRB, 0.90756E+00_JPRB ,&
&  0.89317E+00_JPRB, 0.89381E+00_JPRB, 0.90439E+00_JPRB, 0.91705E+00_JPRB, 0.94989E+00_JPRB, 0.10132E+01_JPRB, 0.10904E+01_JPRB ,&
&  0.11681E+01_JPRB, 0.11893E+01_JPRB, 0.12406E+01_JPRB, 0.13717E+01_JPRB, 0.16535E+01_JPRB, 0.19386E+01_JPRB, 0.20342E+01_JPRB ,&
&  0.21375E+01_JPRB, 0.22718E+01_JPRB, 0.23439E+01_JPRB, 0.23905E+01_JPRB, 0.23572E+01_JPRB, 0.23073E+01_JPRB, 0.23141E+01_JPRB ,&
&  0.19639E+01_JPRB, 0.15804E+01_JPRB, 0.14980E+01_JPRB, 0.14780E+01_JPRB, 0.14987E+01_JPRB, 0.15077E+01_JPRB, 0.15172E+01_JPRB ,&
&  0.15313E+01_JPRB, 0.15607E+01_JPRB, 0.15821E+01_JPRB, 0.16289E+01_JPRB, 0.17071E+01_JPRB, 0.18046E+01_JPRB, 0.18532E+01_JPRB ,&
&  0.18597E+01_JPRB, 0.18730E+01_JPRB, 0.18976E+01_JPRB, 0.17950E+01_JPRB, 0.19340E+01_JPRB, 0.20054E+01_JPRB, 0.20442E+01_JPRB ,&
&  0.20408E+01_JPRB, 0.19947E+01_JPRB, 0.20559E+01_JPRB, 0.20628E+01_JPRB, 0.20574E+01_JPRB, 0.19534E+01_JPRB, 0.18923E+01_JPRB ,&
&  0.19094E+01_JPRB, 0.17246E+01_JPRB, 0.13806E+01_JPRB, 0.15936E+01_JPRB, 0.18344E+01_JPRB, 0.17522E+01_JPRB, 0.23300E+01_JPRB ,&
&  0.23302E+01_JPRB, 0.22992E+01_JPRB, 0.23470E+01_JPRB, 0.24187E+01_JPRB, 0.25529E+01_JPRB, 0.25728E+01_JPRB, 0.25538E+01_JPRB ,&
&  0.26905E+01_JPRB, 0.25382E+01_JPRB, 0.23311E+01_JPRB, 0.20746E+01_JPRB, 0.17941E+01_JPRB, 0.15508E+01_JPRB, 0.13871E+01_JPRB ,&
&  0.12707E+01_JPRB, 0.12264E+01_JPRB, 0.10858E+01_JPRB, 0.10098E+01_JPRB, 0.88352E+00_JPRB, 0.86760E+00_JPRB, 0.88428E+00_JPRB ,&
&  0.92854E+00_JPRB, 0.93290E+00_JPRB, 0.93153E+00_JPRB, 0.91837E+00_JPRB, 0.91307E+00_JPRB, 0.90891E+00_JPRB, 0.91021E+00_JPRB ,&
&  0.92393E+00_JPRB, 0.94298E+00_JPRB, 0.94050E+00_JPRB, 0.93552E+00_JPRB, 0.92861E+00_JPRB /)
ZBURD(:, 34, 12) = (/ &
&  0.88841E+00_JPRB, 0.88199E+00_JPRB, 0.87896E+00_JPRB, 0.89311E+00_JPRB, 0.90456E+00_JPRB, 0.89145E+00_JPRB, 0.90033E+00_JPRB ,&
&  0.87881E+00_JPRB, 0.86163E+00_JPRB, 0.87284E+00_JPRB, 0.87882E+00_JPRB, 0.89714E+00_JPRB, 0.91634E+00_JPRB, 0.92863E+00_JPRB ,&
&  0.92881E+00_JPRB, 0.92036E+00_JPRB, 0.87876E+00_JPRB, 0.88643E+00_JPRB, 0.98923E+00_JPRB, 0.11694E+01_JPRB, 0.15010E+01_JPRB ,&
&  0.18547E+01_JPRB, 0.20077E+01_JPRB, 0.20786E+01_JPRB, 0.20612E+01_JPRB, 0.19678E+01_JPRB, 0.18222E+01_JPRB, 0.23366E+01_JPRB ,&
&  0.22712E+01_JPRB, 0.15272E+01_JPRB, 0.15184E+01_JPRB, 0.14196E+01_JPRB, 0.14408E+01_JPRB, 0.14342E+01_JPRB, 0.14439E+01_JPRB ,&
&  0.14377E+01_JPRB, 0.14644E+01_JPRB, 0.15123E+01_JPRB, 0.15934E+01_JPRB, 0.16765E+01_JPRB, 0.17677E+01_JPRB, 0.18295E+01_JPRB ,&
&  0.18483E+01_JPRB, 0.18643E+01_JPRB, 0.19343E+01_JPRB, 0.19102E+01_JPRB, 0.19036E+01_JPRB, 0.19909E+01_JPRB, 0.20718E+01_JPRB ,&
&  0.21375E+01_JPRB, 0.22558E+01_JPRB, 0.20674E+01_JPRB, 0.20632E+01_JPRB, 0.21302E+01_JPRB, 0.19364E+01_JPRB, 0.19148E+01_JPRB ,&
&  0.19197E+01_JPRB, 0.16682E+01_JPRB, 0.12806E+01_JPRB, 0.14324E+01_JPRB, 0.16237E+01_JPRB, 0.18137E+01_JPRB, 0.20644E+01_JPRB ,&
&  0.20252E+01_JPRB, 0.19386E+01_JPRB, 0.19620E+01_JPRB, 0.20934E+01_JPRB, 0.21518E+01_JPRB, 0.21349E+01_JPRB, 0.20873E+01_JPRB ,&
&  0.20580E+01_JPRB, 0.18692E+01_JPRB, 0.17391E+01_JPRB, 0.15488E+01_JPRB, 0.13935E+01_JPRB, 0.12849E+01_JPRB, 0.11516E+01_JPRB ,&
&  0.11503E+01_JPRB, 0.11940E+01_JPRB, 0.13403E+01_JPRB, 0.12001E+01_JPRB, 0.86567E+00_JPRB, 0.86048E+00_JPRB, 0.85444E+00_JPRB ,&
&  0.86767E+00_JPRB, 0.87642E+00_JPRB, 0.89852E+00_JPRB, 0.89675E+00_JPRB, 0.88332E+00_JPRB, 0.86940E+00_JPRB, 0.85793E+00_JPRB ,&
&  0.87644E+00_JPRB, 0.90186E+00_JPRB, 0.91263E+00_JPRB, 0.90151E+00_JPRB, 0.89059E+00_JPRB /)
ZBURD(:, 35, 12) = (/ &
&  0.93742E+00_JPRB, 0.93929E+00_JPRB, 0.95757E+00_JPRB, 0.96104E+00_JPRB, 0.95888E+00_JPRB, 0.96516E+00_JPRB, 0.96615E+00_JPRB ,&
&  0.95921E+00_JPRB, 0.96560E+00_JPRB, 0.97450E+00_JPRB, 0.98531E+00_JPRB, 0.10003E+01_JPRB, 0.10141E+01_JPRB, 0.10220E+01_JPRB ,&
&  0.10129E+01_JPRB, 0.10033E+01_JPRB, 0.10258E+01_JPRB, 0.10425E+01_JPRB, 0.10404E+01_JPRB, 0.10393E+01_JPRB, 0.10734E+01_JPRB ,&
&  0.11467E+01_JPRB, 0.11874E+01_JPRB, 0.11856E+01_JPRB, 0.11583E+01_JPRB, 0.11914E+01_JPRB, 0.13949E+01_JPRB, 0.21252E+01_JPRB ,&
&  0.21582E+01_JPRB, 0.15054E+01_JPRB, 0.13513E+01_JPRB, 0.12314E+01_JPRB, 0.12455E+01_JPRB, 0.12916E+01_JPRB, 0.12780E+01_JPRB ,&
&  0.12396E+01_JPRB, 0.12500E+01_JPRB, 0.12612E+01_JPRB, 0.12732E+01_JPRB, 0.13066E+01_JPRB, 0.13361E+01_JPRB, 0.13435E+01_JPRB ,&
&  0.12984E+01_JPRB, 0.12806E+01_JPRB, 0.12995E+01_JPRB, 0.13402E+01_JPRB, 0.14777E+01_JPRB, 0.17151E+01_JPRB, 0.16635E+01_JPRB ,&
&  0.16672E+01_JPRB, 0.17647E+01_JPRB, 0.18714E+01_JPRB, 0.19235E+01_JPRB, 0.18802E+01_JPRB, 0.17741E+01_JPRB, 0.16952E+01_JPRB ,&
&  0.16299E+01_JPRB, 0.15088E+01_JPRB, 0.13651E+01_JPRB, 0.16100E+01_JPRB, 0.17387E+01_JPRB, 0.18669E+01_JPRB, 0.17776E+01_JPRB ,&
&  0.16965E+01_JPRB, 0.15349E+01_JPRB, 0.15229E+01_JPRB, 0.16261E+01_JPRB, 0.17239E+01_JPRB, 0.17112E+01_JPRB, 0.16304E+01_JPRB ,&
&  0.15023E+01_JPRB, 0.14089E+01_JPRB, 0.13860E+01_JPRB, 0.12889E+01_JPRB, 0.12429E+01_JPRB, 0.10453E+01_JPRB, 0.10503E+01_JPRB ,&
&  0.10581E+01_JPRB, 0.11215E+01_JPRB, 0.92938E+00_JPRB, 0.11804E+01_JPRB, 0.84350E+00_JPRB, 0.78490E+00_JPRB, 0.79663E+00_JPRB ,&
&  0.81919E+00_JPRB, 0.85101E+00_JPRB, 0.86032E+00_JPRB, 0.94886E+00_JPRB, 0.93718E+00_JPRB, 0.93073E+00_JPRB, 0.94122E+00_JPRB ,&
&  0.91688E+00_JPRB, 0.91469E+00_JPRB, 0.91606E+00_JPRB, 0.92837E+00_JPRB, 0.93394E+00_JPRB /)
ZBURD(:, 36, 12) = (/ &
&  0.99217E+00_JPRB, 0.99390E+00_JPRB, 0.99956E+00_JPRB, 0.10100E+01_JPRB, 0.10182E+01_JPRB, 0.10262E+01_JPRB, 0.10289E+01_JPRB ,&
&  0.10224E+01_JPRB, 0.10241E+01_JPRB, 0.10348E+01_JPRB, 0.10512E+01_JPRB, 0.10671E+01_JPRB, 0.10808E+01_JPRB, 0.10966E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.10995E+01_JPRB, 0.11196E+01_JPRB, 0.11419E+01_JPRB, 0.11534E+01_JPRB, 0.11532E+01_JPRB, 0.11575E+01_JPRB ,&
&  0.11523E+01_JPRB, 0.11622E+01_JPRB, 0.11904E+01_JPRB, 0.12082E+01_JPRB, 0.12339E+01_JPRB, 0.13052E+01_JPRB, 0.15382E+01_JPRB ,&
&  0.15450E+01_JPRB, 0.16188E+01_JPRB, 0.13884E+01_JPRB, 0.12365E+01_JPRB, 0.12223E+01_JPRB, 0.11692E+01_JPRB, 0.11262E+01_JPRB ,&
&  0.10806E+01_JPRB, 0.10517E+01_JPRB, 0.10012E+01_JPRB, 0.10029E+01_JPRB, 0.99146E+00_JPRB, 0.10208E+01_JPRB, 0.10040E+01_JPRB ,&
&  0.10389E+01_JPRB, 0.10722E+01_JPRB, 0.11329E+01_JPRB, 0.11870E+01_JPRB, 0.12682E+01_JPRB, 0.12977E+01_JPRB, 0.12018E+01_JPRB ,&
&  0.11859E+01_JPRB, 0.12808E+01_JPRB, 0.13843E+01_JPRB, 0.14827E+01_JPRB, 0.14297E+01_JPRB, 0.13393E+01_JPRB, 0.12846E+01_JPRB ,&
&  0.12419E+01_JPRB, 0.12715E+01_JPRB, 0.15663E+01_JPRB, 0.16532E+01_JPRB, 0.17273E+01_JPRB, 0.16684E+01_JPRB, 0.15128E+01_JPRB ,&
&  0.13331E+01_JPRB, 0.11345E+01_JPRB, 0.11233E+01_JPRB, 0.11395E+01_JPRB, 0.11764E+01_JPRB, 0.11573E+01_JPRB, 0.11310E+01_JPRB ,&
&  0.10420E+01_JPRB, 0.96403E+00_JPRB, 0.10434E+01_JPRB, 0.11302E+01_JPRB, 0.11998E+01_JPRB, 0.14789E+01_JPRB, 0.14051E+01_JPRB ,&
&  0.98626E+00_JPRB, 0.88032E+00_JPRB, 0.61681E+00_JPRB, 0.84401E+00_JPRB, 0.71070E+00_JPRB, 0.73325E+00_JPRB, 0.81175E+00_JPRB ,&
&  0.92229E+00_JPRB, 0.10193E+01_JPRB, 0.10877E+01_JPRB, 0.10537E+01_JPRB, 0.98359E+00_JPRB, 0.97317E+00_JPRB, 0.96838E+00_JPRB ,&
&  0.97759E+00_JPRB, 0.98240E+00_JPRB, 0.98440E+00_JPRB, 0.98562E+00_JPRB, 0.98256E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART23',1,ZHOOK_HANDLE)
END SUBROUTINE PART23



SUBROUTINE PART24

REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART24',0,ZHOOK_HANDLE)


ZBURD(:, 37, 12) = (/ &
&  0.10125E+01_JPRB, 0.10208E+01_JPRB, 0.10335E+01_JPRB, 0.10504E+01_JPRB, 0.10660E+01_JPRB, 0.10809E+01_JPRB, 0.10929E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.11176E+01_JPRB, 0.11432E+01_JPRB, 0.11735E+01_JPRB, 0.11875E+01_JPRB, 0.12057E+01_JPRB, 0.12110E+01_JPRB ,&
&  0.12368E+01_JPRB, 0.12668E+01_JPRB, 0.12817E+01_JPRB, 0.12989E+01_JPRB, 0.13104E+01_JPRB, 0.13262E+01_JPRB, 0.13464E+01_JPRB ,&
&  0.13628E+01_JPRB, 0.14041E+01_JPRB, 0.14674E+01_JPRB, 0.15174E+01_JPRB, 0.15886E+01_JPRB, 0.17346E+01_JPRB, 0.15240E+01_JPRB ,&
&  0.16698E+01_JPRB, 0.16198E+01_JPRB, 0.14699E+01_JPRB, 0.12833E+01_JPRB, 0.12223E+01_JPRB, 0.11706E+01_JPRB, 0.11313E+01_JPRB ,&
&  0.11089E+01_JPRB, 0.10603E+01_JPRB, 0.10717E+01_JPRB, 0.10624E+01_JPRB, 0.10561E+01_JPRB, 0.11041E+01_JPRB, 0.11120E+01_JPRB ,&
&  0.11296E+01_JPRB, 0.11495E+01_JPRB, 0.11756E+01_JPRB, 0.11883E+01_JPRB, 0.12175E+01_JPRB, 0.12167E+01_JPRB, 0.11627E+01_JPRB ,&
&  0.10935E+01_JPRB, 0.10503E+01_JPRB, 0.99274E+00_JPRB, 0.98939E+00_JPRB, 0.97745E+00_JPRB, 0.88521E+00_JPRB, 0.78487E+00_JPRB ,&
&  0.88184E+00_JPRB, 0.10067E+01_JPRB, 0.11640E+01_JPRB, 0.17134E+01_JPRB, 0.16796E+01_JPRB, 0.15469E+01_JPRB, 0.12955E+01_JPRB ,&
&  0.11148E+01_JPRB, 0.90072E+00_JPRB, 0.84282E+00_JPRB, 0.86493E+00_JPRB, 0.85182E+00_JPRB, 0.84038E+00_JPRB, 0.80483E+00_JPRB ,&
&  0.74210E+00_JPRB, 0.72529E+00_JPRB, 0.81446E+00_JPRB, 0.95621E+00_JPRB, 0.11564E+01_JPRB, 0.12358E+01_JPRB, 0.16119E+01_JPRB ,&
&  0.10611E+01_JPRB, 0.64289E+00_JPRB, 0.71698E+00_JPRB, 0.72508E+00_JPRB, 0.68489E+00_JPRB, 0.67735E+00_JPRB, 0.80467E+00_JPRB ,&
&  0.10142E+01_JPRB, 0.10790E+01_JPRB, 0.11017E+01_JPRB, 0.10534E+01_JPRB, 0.10467E+01_JPRB, 0.10483E+01_JPRB, 0.10350E+01_JPRB ,&
&  0.10299E+01_JPRB, 0.10272E+01_JPRB, 0.10253E+01_JPRB, 0.10158E+01_JPRB, 0.10100E+01_JPRB /)
ZBURD(:, 38, 12) = (/ &
&  0.10401E+01_JPRB, 0.10531E+01_JPRB, 0.10738E+01_JPRB, 0.11021E+01_JPRB, 0.11177E+01_JPRB, 0.11424E+01_JPRB, 0.11536E+01_JPRB ,&
&  0.11676E+01_JPRB, 0.12005E+01_JPRB, 0.12322E+01_JPRB, 0.12669E+01_JPRB, 0.12991E+01_JPRB, 0.13293E+01_JPRB, 0.13580E+01_JPRB ,&
&  0.13828E+01_JPRB, 0.14080E+01_JPRB, 0.14205E+01_JPRB, 0.14215E+01_JPRB, 0.14071E+01_JPRB, 0.13994E+01_JPRB, 0.13800E+01_JPRB ,&
&  0.13620E+01_JPRB, 0.13645E+01_JPRB, 0.13775E+01_JPRB, 0.13864E+01_JPRB, 0.14066E+01_JPRB, 0.15157E+01_JPRB, 0.11519E+01_JPRB ,&
&  0.15680E+01_JPRB, 0.15773E+01_JPRB, 0.15145E+01_JPRB, 0.13680E+01_JPRB, 0.12955E+01_JPRB, 0.12920E+01_JPRB, 0.13181E+01_JPRB ,&
&  0.13714E+01_JPRB, 0.14463E+01_JPRB, 0.15060E+01_JPRB, 0.14026E+01_JPRB, 0.12889E+01_JPRB, 0.13022E+01_JPRB, 0.13112E+01_JPRB ,&
&  0.13258E+01_JPRB, 0.13263E+01_JPRB, 0.13247E+01_JPRB, 0.13070E+01_JPRB, 0.12924E+01_JPRB, 0.12704E+01_JPRB, 0.12239E+01_JPRB ,&
&  0.11633E+01_JPRB, 0.10885E+01_JPRB, 0.98402E+00_JPRB, 0.99599E+00_JPRB, 0.98454E+00_JPRB, 0.85426E+00_JPRB, 0.65467E+00_JPRB ,&
&  0.59751E+00_JPRB, 0.83336E+00_JPRB, 0.11325E+01_JPRB, 0.16784E+01_JPRB, 0.15581E+01_JPRB, 0.13821E+01_JPRB, 0.12263E+01_JPRB ,&
&  0.98509E+00_JPRB, 0.82778E+00_JPRB, 0.75158E+00_JPRB, 0.75676E+00_JPRB, 0.75336E+00_JPRB, 0.75559E+00_JPRB, 0.73551E+00_JPRB ,&
&  0.70915E+00_JPRB, 0.69979E+00_JPRB, 0.75424E+00_JPRB, 0.84074E+00_JPRB, 0.98029E+00_JPRB, 0.99311E+00_JPRB, 0.15008E+01_JPRB ,&
&  0.10463E+01_JPRB, 0.84720E+00_JPRB, 0.11171E+01_JPRB, 0.60458E+00_JPRB, 0.73514E+00_JPRB, 0.81822E+00_JPRB, 0.93314E+00_JPRB ,&
&  0.96382E+00_JPRB, 0.12634E+01_JPRB, 0.15596E+01_JPRB, 0.13858E+01_JPRB, 0.12518E+01_JPRB, 0.11407E+01_JPRB, 0.10960E+01_JPRB ,&
&  0.10841E+01_JPRB, 0.10729E+01_JPRB, 0.10684E+01_JPRB, 0.10580E+01_JPRB, 0.10436E+01_JPRB /)
ZBURD(:, 39, 12) = (/ &
&  0.11070E+01_JPRB, 0.11110E+01_JPRB, 0.11054E+01_JPRB, 0.11379E+01_JPRB, 0.11379E+01_JPRB, 0.11556E+01_JPRB, 0.11591E+01_JPRB ,&
&  0.11846E+01_JPRB, 0.12072E+01_JPRB, 0.12348E+01_JPRB, 0.12783E+01_JPRB, 0.12960E+01_JPRB, 0.13169E+01_JPRB, 0.13516E+01_JPRB ,&
&  0.13863E+01_JPRB, 0.14291E+01_JPRB, 0.14327E+01_JPRB, 0.14237E+01_JPRB, 0.13956E+01_JPRB, 0.13790E+01_JPRB, 0.13561E+01_JPRB ,&
&  0.13375E+01_JPRB, 0.13345E+01_JPRB, 0.13461E+01_JPRB, 0.13678E+01_JPRB, 0.13918E+01_JPRB, 0.14600E+01_JPRB, 0.10097E+01_JPRB ,&
&  0.14912E+01_JPRB, 0.16131E+01_JPRB, 0.15323E+01_JPRB, 0.14834E+01_JPRB, 0.13983E+01_JPRB, 0.14197E+01_JPRB, 0.14739E+01_JPRB ,&
&  0.15588E+01_JPRB, 0.16395E+01_JPRB, 0.16395E+01_JPRB, 0.15744E+01_JPRB, 0.15309E+01_JPRB, 0.15536E+01_JPRB, 0.15693E+01_JPRB ,&
&  0.15801E+01_JPRB, 0.15849E+01_JPRB, 0.15877E+01_JPRB, 0.15812E+01_JPRB, 0.15789E+01_JPRB, 0.15896E+01_JPRB, 0.15650E+01_JPRB ,&
&  0.15154E+01_JPRB, 0.14649E+01_JPRB, 0.14540E+01_JPRB, 0.14879E+01_JPRB, 0.14526E+01_JPRB, 0.12514E+01_JPRB, 0.97402E+00_JPRB ,&
&  0.65858E+00_JPRB, 0.66458E+00_JPRB, 0.10204E+01_JPRB, 0.15275E+01_JPRB, 0.14152E+01_JPRB, 0.12485E+01_JPRB, 0.10759E+01_JPRB ,&
&  0.96105E+00_JPRB, 0.83197E+00_JPRB, 0.73889E+00_JPRB, 0.74288E+00_JPRB, 0.76394E+00_JPRB, 0.81591E+00_JPRB, 0.80757E+00_JPRB ,&
&  0.82744E+00_JPRB, 0.84235E+00_JPRB, 0.84717E+00_JPRB, 0.89075E+00_JPRB, 0.10210E+01_JPRB, 0.11139E+01_JPRB, 0.10347E+01_JPRB ,&
&  0.11796E+01_JPRB, 0.96740E+00_JPRB, 0.94128E+00_JPRB, 0.10249E+01_JPRB, 0.10164E+01_JPRB, 0.12038E+01_JPRB, 0.14975E+01_JPRB ,&
&  0.19505E+01_JPRB, 0.18932E+01_JPRB, 0.24969E+01_JPRB, 0.37941E+01_JPRB, 0.29837E+01_JPRB, 0.21417E+01_JPRB, 0.11762E+01_JPRB ,&
&  0.11634E+01_JPRB, 0.11369E+01_JPRB, 0.11463E+01_JPRB, 0.11286E+01_JPRB, 0.11146E+01_JPRB /)
ZBURD(:, 40, 12) = (/ &
&  0.10610E+01_JPRB, 0.10467E+01_JPRB, 0.10472E+01_JPRB, 0.10470E+01_JPRB, 0.10452E+01_JPRB, 0.10555E+01_JPRB, 0.10801E+01_JPRB ,&
&  0.10934E+01_JPRB, 0.11322E+01_JPRB, 0.11925E+01_JPRB, 0.12342E+01_JPRB, 0.12879E+01_JPRB, 0.12982E+01_JPRB, 0.12720E+01_JPRB ,&
&  0.12708E+01_JPRB, 0.13114E+01_JPRB, 0.13487E+01_JPRB, 0.13663E+01_JPRB, 0.13348E+01_JPRB, 0.13220E+01_JPRB, 0.13479E+01_JPRB ,&
&  0.13974E+01_JPRB, 0.14384E+01_JPRB, 0.14780E+01_JPRB, 0.15368E+01_JPRB, 0.15496E+01_JPRB, 0.14771E+01_JPRB, 0.10252E+01_JPRB ,&
&  0.13950E+01_JPRB, 0.15806E+01_JPRB, 0.16072E+01_JPRB, 0.15643E+01_JPRB, 0.15467E+01_JPRB, 0.14860E+01_JPRB, 0.15322E+01_JPRB ,&
&  0.16274E+01_JPRB, 0.17102E+01_JPRB, 0.16692E+01_JPRB, 0.15510E+01_JPRB, 0.16970E+01_JPRB, 0.17110E+01_JPRB, 0.17485E+01_JPRB ,&
&  0.17705E+01_JPRB, 0.17958E+01_JPRB, 0.18258E+01_JPRB, 0.18626E+01_JPRB, 0.18852E+01_JPRB, 0.19146E+01_JPRB, 0.19301E+01_JPRB ,&
&  0.19062E+01_JPRB, 0.18694E+01_JPRB, 0.18825E+01_JPRB, 0.17821E+01_JPRB, 0.17242E+01_JPRB, 0.17314E+01_JPRB, 0.14599E+01_JPRB ,&
&  0.89734E+00_JPRB, 0.69347E+00_JPRB, 0.10187E+01_JPRB, 0.13720E+01_JPRB, 0.12355E+01_JPRB, 0.11174E+01_JPRB, 0.10434E+01_JPRB ,&
&  0.92819E+00_JPRB, 0.82302E+00_JPRB, 0.79689E+00_JPRB, 0.85554E+00_JPRB, 0.93615E+00_JPRB, 0.10054E+01_JPRB, 0.10248E+01_JPRB ,&
&  0.99806E+00_JPRB, 0.98553E+00_JPRB, 0.99841E+00_JPRB, 0.10381E+01_JPRB, 0.11247E+01_JPRB, 0.13033E+01_JPRB, 0.15553E+01_JPRB ,&
&  0.14780E+01_JPRB, 0.10563E+01_JPRB, 0.11220E+01_JPRB, 0.11663E+01_JPRB, 0.12603E+01_JPRB, 0.14115E+01_JPRB, 0.17122E+01_JPRB ,&
&  0.21844E+01_JPRB, 0.26791E+01_JPRB, 0.26060E+01_JPRB, 0.21813E+01_JPRB, 0.27376E+01_JPRB, 0.18487E+01_JPRB, 0.10068E+01_JPRB ,&
&  0.10387E+01_JPRB, 0.10575E+01_JPRB, 0.10777E+01_JPRB, 0.10746E+01_JPRB, 0.10707E+01_JPRB /)
ZBURD(:, 41, 12) = (/ &
&  0.91646E+00_JPRB, 0.89982E+00_JPRB, 0.88854E+00_JPRB, 0.87842E+00_JPRB, 0.89032E+00_JPRB, 0.92597E+00_JPRB, 0.96807E+00_JPRB ,&
&  0.10119E+01_JPRB, 0.10620E+01_JPRB, 0.11287E+01_JPRB, 0.11571E+01_JPRB, 0.12016E+01_JPRB, 0.12052E+01_JPRB, 0.11993E+01_JPRB ,&
&  0.11829E+01_JPRB, 0.12021E+01_JPRB, 0.12348E+01_JPRB, 0.12693E+01_JPRB, 0.12886E+01_JPRB, 0.13294E+01_JPRB, 0.13821E+01_JPRB ,&
&  0.14496E+01_JPRB, 0.15135E+01_JPRB, 0.15925E+01_JPRB, 0.17170E+01_JPRB, 0.17654E+01_JPRB, 0.17823E+01_JPRB, 0.18191E+01_JPRB ,&
&  0.97162E+00_JPRB, 0.14386E+01_JPRB, 0.15319E+01_JPRB, 0.15174E+01_JPRB, 0.16183E+01_JPRB, 0.16455E+01_JPRB, 0.16705E+01_JPRB ,&
&  0.17465E+01_JPRB, 0.17212E+01_JPRB, 0.16725E+01_JPRB, 0.16383E+01_JPRB, 0.17266E+01_JPRB, 0.17617E+01_JPRB, 0.17884E+01_JPRB ,&
&  0.17831E+01_JPRB, 0.17744E+01_JPRB, 0.18041E+01_JPRB, 0.18649E+01_JPRB, 0.19131E+01_JPRB, 0.19627E+01_JPRB, 0.20344E+01_JPRB ,&
&  0.20567E+01_JPRB, 0.20686E+01_JPRB, 0.21471E+01_JPRB, 0.19524E+01_JPRB, 0.18096E+01_JPRB, 0.18737E+01_JPRB, 0.16963E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.81715E+00_JPRB, 0.99181E+00_JPRB, 0.13102E+01_JPRB, 0.11754E+01_JPRB, 0.10957E+01_JPRB, 0.10166E+01_JPRB ,&
&  0.90494E+00_JPRB, 0.84945E+00_JPRB, 0.92638E+00_JPRB, 0.10083E+01_JPRB, 0.10838E+01_JPRB, 0.11461E+01_JPRB, 0.11467E+01_JPRB ,&
&  0.10909E+01_JPRB, 0.10550E+01_JPRB, 0.10190E+01_JPRB, 0.10165E+01_JPRB, 0.10406E+01_JPRB, 0.10653E+01_JPRB, 0.10677E+01_JPRB ,&
&  0.10677E+01_JPRB, 0.10503E+01_JPRB, 0.10799E+01_JPRB, 0.11112E+01_JPRB, 0.11425E+01_JPRB, 0.11707E+01_JPRB, 0.13100E+01_JPRB ,&
&  0.15428E+01_JPRB, 0.17553E+01_JPRB, 0.20059E+01_JPRB, 0.19492E+01_JPRB, 0.17622E+01_JPRB, 0.12769E+01_JPRB, 0.87408E+00_JPRB ,&
&  0.89630E+00_JPRB, 0.91828E+00_JPRB, 0.92218E+00_JPRB, 0.93433E+00_JPRB, 0.92580E+00_JPRB /)
ZBURD(:, 42, 12) = (/ &
&  0.82354E+00_JPRB, 0.81584E+00_JPRB, 0.80839E+00_JPRB, 0.81300E+00_JPRB, 0.82238E+00_JPRB, 0.87212E+00_JPRB, 0.95336E+00_JPRB ,&
&  0.10332E+01_JPRB, 0.10834E+01_JPRB, 0.11286E+01_JPRB, 0.11538E+01_JPRB, 0.11615E+01_JPRB, 0.11273E+01_JPRB, 0.11279E+01_JPRB ,&
&  0.11275E+01_JPRB, 0.11321E+01_JPRB, 0.11626E+01_JPRB, 0.12079E+01_JPRB, 0.12506E+01_JPRB, 0.12991E+01_JPRB, 0.13547E+01_JPRB ,&
&  0.14231E+01_JPRB, 0.15200E+01_JPRB, 0.16226E+01_JPRB, 0.17725E+01_JPRB, 0.18976E+01_JPRB, 0.19881E+01_JPRB, 0.19769E+01_JPRB ,&
&  0.91683E+00_JPRB, 0.10128E+01_JPRB, 0.14549E+01_JPRB, 0.14240E+01_JPRB, 0.15193E+01_JPRB, 0.16619E+01_JPRB, 0.18380E+01_JPRB ,&
&  0.17693E+01_JPRB, 0.15686E+01_JPRB, 0.15473E+01_JPRB, 0.17149E+01_JPRB, 0.16641E+01_JPRB, 0.16998E+01_JPRB, 0.17191E+01_JPRB ,&
&  0.17055E+01_JPRB, 0.16834E+01_JPRB, 0.16765E+01_JPRB, 0.17069E+01_JPRB, 0.17621E+01_JPRB, 0.18225E+01_JPRB, 0.19000E+01_JPRB ,&
&  0.19689E+01_JPRB, 0.20535E+01_JPRB, 0.21492E+01_JPRB, 0.18045E+01_JPRB, 0.18642E+01_JPRB, 0.19994E+01_JPRB, 0.21981E+01_JPRB ,&
&  0.17617E+01_JPRB, 0.11095E+01_JPRB, 0.10953E+01_JPRB, 0.13901E+01_JPRB, 0.12959E+01_JPRB, 0.12771E+01_JPRB, 0.10985E+01_JPRB ,&
&  0.10729E+01_JPRB, 0.10143E+01_JPRB, 0.10443E+01_JPRB, 0.11132E+01_JPRB, 0.11768E+01_JPRB, 0.12211E+01_JPRB, 0.12116E+01_JPRB ,&
&  0.11552E+01_JPRB, 0.11225E+01_JPRB, 0.11085E+01_JPRB, 0.11191E+01_JPRB, 0.11419E+01_JPRB, 0.11577E+01_JPRB, 0.11577E+01_JPRB ,&
&  0.11542E+01_JPRB, 0.11582E+01_JPRB, 0.12011E+01_JPRB, 0.12273E+01_JPRB, 0.12490E+01_JPRB, 0.12706E+01_JPRB, 0.13006E+01_JPRB ,&
&  0.12287E+01_JPRB, 0.11943E+01_JPRB, 0.12735E+01_JPRB, 0.11722E+01_JPRB, 0.10410E+01_JPRB, 0.89599E+00_JPRB, 0.76233E+00_JPRB ,&
&  0.78378E+00_JPRB, 0.79453E+00_JPRB, 0.79267E+00_JPRB, 0.80137E+00_JPRB, 0.81942E+00_JPRB /)
ZBURD(:, 43, 12) = (/ &
&  0.73593E+00_JPRB, 0.74115E+00_JPRB, 0.73453E+00_JPRB, 0.75193E+00_JPRB, 0.75400E+00_JPRB, 0.82251E+00_JPRB, 0.89429E+00_JPRB ,&
&  0.10061E+01_JPRB, 0.10896E+01_JPRB, 0.11435E+01_JPRB, 0.11594E+01_JPRB, 0.11431E+01_JPRB, 0.10932E+01_JPRB, 0.10469E+01_JPRB ,&
&  0.10518E+01_JPRB, 0.10595E+01_JPRB, 0.10766E+01_JPRB, 0.11037E+01_JPRB, 0.11335E+01_JPRB, 0.11782E+01_JPRB, 0.12254E+01_JPRB ,&
&  0.12886E+01_JPRB, 0.13822E+01_JPRB, 0.15024E+01_JPRB, 0.16758E+01_JPRB, 0.19024E+01_JPRB, 0.22140E+01_JPRB, 0.25362E+01_JPRB ,&
&  0.20057E+01_JPRB, 0.10077E+01_JPRB, 0.14247E+01_JPRB, 0.13622E+01_JPRB, 0.12513E+01_JPRB, 0.14616E+01_JPRB, 0.17219E+01_JPRB ,&
&  0.16397E+01_JPRB, 0.15357E+01_JPRB, 0.14778E+01_JPRB, 0.16350E+01_JPRB, 0.16119E+01_JPRB, 0.16221E+01_JPRB, 0.16293E+01_JPRB ,&
&  0.16219E+01_JPRB, 0.15994E+01_JPRB, 0.15821E+01_JPRB, 0.16100E+01_JPRB, 0.16580E+01_JPRB, 0.17307E+01_JPRB, 0.18045E+01_JPRB ,&
&  0.19010E+01_JPRB, 0.20011E+01_JPRB, 0.21446E+01_JPRB, 0.18110E+01_JPRB, 0.19349E+01_JPRB, 0.19553E+01_JPRB, 0.17600E+01_JPRB ,&
&  0.17207E+01_JPRB, 0.12589E+01_JPRB, 0.11839E+01_JPRB, 0.14583E+01_JPRB, 0.13854E+01_JPRB, 0.11033E+01_JPRB, 0.11326E+01_JPRB ,&
&  0.10880E+01_JPRB, 0.11102E+01_JPRB, 0.10988E+01_JPRB, 0.11707E+01_JPRB, 0.12115E+01_JPRB, 0.12403E+01_JPRB, 0.12212E+01_JPRB ,&
&  0.12027E+01_JPRB, 0.11742E+01_JPRB, 0.11701E+01_JPRB, 0.11852E+01_JPRB, 0.12018E+01_JPRB, 0.12408E+01_JPRB, 0.12567E+01_JPRB ,&
&  0.12718E+01_JPRB, 0.13088E+01_JPRB, 0.13766E+01_JPRB, 0.14403E+01_JPRB, 0.14633E+01_JPRB, 0.13844E+01_JPRB, 0.13166E+01_JPRB ,&
&  0.12528E+01_JPRB, 0.11868E+01_JPRB, 0.10574E+01_JPRB, 0.93301E+00_JPRB, 0.88345E+00_JPRB, 0.80897E+00_JPRB, 0.74358E+00_JPRB ,&
&  0.75366E+00_JPRB, 0.75107E+00_JPRB, 0.73624E+00_JPRB, 0.73477E+00_JPRB, 0.73830E+00_JPRB /)
ZBURD(:, 44, 12) = (/ &
&  0.77998E+00_JPRB, 0.77147E+00_JPRB, 0.74242E+00_JPRB, 0.72500E+00_JPRB, 0.72115E+00_JPRB, 0.74796E+00_JPRB, 0.79291E+00_JPRB ,&
&  0.86381E+00_JPRB, 0.95759E+00_JPRB, 0.10337E+01_JPRB, 0.10694E+01_JPRB, 0.10624E+01_JPRB, 0.10696E+01_JPRB, 0.10290E+01_JPRB ,&
&  0.98730E+00_JPRB, 0.98426E+00_JPRB, 0.99939E+00_JPRB, 0.10094E+01_JPRB, 0.10262E+01_JPRB, 0.10511E+01_JPRB, 0.10800E+01_JPRB ,&
&  0.10962E+01_JPRB, 0.11122E+01_JPRB, 0.11356E+01_JPRB, 0.12352E+01_JPRB, 0.14035E+01_JPRB, 0.16665E+01_JPRB, 0.21468E+01_JPRB ,&
&  0.31771E+01_JPRB, 0.32010E+01_JPRB, 0.16443E+01_JPRB, 0.17110E+01_JPRB, 0.14434E+01_JPRB, 0.15017E+01_JPRB, 0.14444E+01_JPRB ,&
&  0.15417E+01_JPRB, 0.14705E+01_JPRB, 0.14508E+01_JPRB, 0.15474E+01_JPRB, 0.15222E+01_JPRB, 0.15277E+01_JPRB, 0.15245E+01_JPRB ,&
&  0.15416E+01_JPRB, 0.15332E+01_JPRB, 0.15083E+01_JPRB, 0.14966E+01_JPRB, 0.15320E+01_JPRB, 0.15990E+01_JPRB, 0.16757E+01_JPRB ,&
&  0.17906E+01_JPRB, 0.18770E+01_JPRB, 0.20223E+01_JPRB, 0.18295E+01_JPRB, 0.19338E+01_JPRB, 0.18048E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.13253E+01_JPRB, 0.12897E+01_JPRB, 0.14073E+01_JPRB, 0.15138E+01_JPRB, 0.14170E+01_JPRB, 0.11396E+01_JPRB, 0.11727E+01_JPRB ,&
&  0.11200E+01_JPRB, 0.11149E+01_JPRB, 0.11184E+01_JPRB, 0.11457E+01_JPRB, 0.11914E+01_JPRB, 0.12042E+01_JPRB, 0.12153E+01_JPRB ,&
&  0.11983E+01_JPRB, 0.11807E+01_JPRB, 0.11806E+01_JPRB, 0.11879E+01_JPRB, 0.12076E+01_JPRB, 0.12563E+01_JPRB, 0.12904E+01_JPRB ,&
&  0.13311E+01_JPRB, 0.13909E+01_JPRB, 0.14671E+01_JPRB, 0.15435E+01_JPRB, 0.15385E+01_JPRB, 0.14634E+01_JPRB, 0.13638E+01_JPRB ,&
&  0.13338E+01_JPRB, 0.13065E+01_JPRB, 0.11710E+01_JPRB, 0.10984E+01_JPRB, 0.10035E+01_JPRB, 0.97686E+00_JPRB, 0.94270E+00_JPRB ,&
&  0.93366E+00_JPRB, 0.92849E+00_JPRB, 0.88674E+00_JPRB, 0.77939E+00_JPRB, 0.78021E+00_JPRB /)
ZBURD(:, 45, 12) = (/ &
&  0.86921E+00_JPRB, 0.82831E+00_JPRB, 0.82373E+00_JPRB, 0.82069E+00_JPRB, 0.80801E+00_JPRB, 0.80349E+00_JPRB, 0.76812E+00_JPRB ,&
&  0.77027E+00_JPRB, 0.82231E+00_JPRB, 0.88824E+00_JPRB, 0.92363E+00_JPRB, 0.94596E+00_JPRB, 0.94830E+00_JPRB, 0.92665E+00_JPRB ,&
&  0.90220E+00_JPRB, 0.91536E+00_JPRB, 0.93381E+00_JPRB, 0.94522E+00_JPRB, 0.96460E+00_JPRB, 0.98176E+00_JPRB, 0.10076E+01_JPRB ,&
&  0.10283E+01_JPRB, 0.10058E+01_JPRB, 0.97518E+00_JPRB, 0.96973E+00_JPRB, 0.10364E+01_JPRB, 0.11434E+01_JPRB, 0.13003E+01_JPRB ,&
&  0.19896E+01_JPRB, 0.41702E+01_JPRB, 0.21288E+01_JPRB, 0.19580E+01_JPRB, 0.18069E+01_JPRB, 0.16485E+01_JPRB, 0.15133E+01_JPRB ,&
&  0.15379E+01_JPRB, 0.13737E+01_JPRB, 0.13430E+01_JPRB, 0.14402E+01_JPRB, 0.13982E+01_JPRB, 0.13727E+01_JPRB, 0.13847E+01_JPRB ,&
&  0.13975E+01_JPRB, 0.14043E+01_JPRB, 0.14043E+01_JPRB, 0.13712E+01_JPRB, 0.13933E+01_JPRB, 0.14377E+01_JPRB, 0.15100E+01_JPRB ,&
&  0.15975E+01_JPRB, 0.17130E+01_JPRB, 0.18715E+01_JPRB, 0.17918E+01_JPRB, 0.18524E+01_JPRB, 0.17597E+01_JPRB, 0.14970E+01_JPRB ,&
&  0.13410E+01_JPRB, 0.14221E+01_JPRB, 0.14702E+01_JPRB, 0.15031E+01_JPRB, 0.14107E+01_JPRB, 0.12164E+01_JPRB, 0.12657E+01_JPRB ,&
&  0.11691E+01_JPRB, 0.11448E+01_JPRB, 0.11516E+01_JPRB, 0.11533E+01_JPRB, 0.11752E+01_JPRB, 0.11755E+01_JPRB, 0.11805E+01_JPRB ,&
&  0.11683E+01_JPRB, 0.11542E+01_JPRB, 0.11511E+01_JPRB, 0.11551E+01_JPRB, 0.11634E+01_JPRB, 0.11927E+01_JPRB, 0.12452E+01_JPRB ,&
&  0.12979E+01_JPRB, 0.13720E+01_JPRB, 0.14628E+01_JPRB, 0.15071E+01_JPRB, 0.15157E+01_JPRB, 0.14440E+01_JPRB, 0.13693E+01_JPRB ,&
&  0.13964E+01_JPRB, 0.14548E+01_JPRB, 0.12522E+01_JPRB, 0.11764E+01_JPRB, 0.11345E+01_JPRB, 0.10754E+01_JPRB, 0.10875E+01_JPRB ,&
&  0.11576E+01_JPRB, 0.12670E+01_JPRB, 0.13505E+01_JPRB, 0.90514E+00_JPRB, 0.90062E+00_JPRB /)
ZBURD(:, 46, 12) = (/ &
&  0.91619E+00_JPRB, 0.87419E+00_JPRB, 0.87533E+00_JPRB, 0.90168E+00_JPRB, 0.89987E+00_JPRB, 0.87708E+00_JPRB, 0.83750E+00_JPRB ,&
&  0.79207E+00_JPRB, 0.79636E+00_JPRB, 0.81736E+00_JPRB, 0.83676E+00_JPRB, 0.85121E+00_JPRB, 0.86479E+00_JPRB, 0.85328E+00_JPRB ,&
&  0.83147E+00_JPRB, 0.83919E+00_JPRB, 0.85506E+00_JPRB, 0.88327E+00_JPRB, 0.90523E+00_JPRB, 0.92831E+00_JPRB, 0.95981E+00_JPRB ,&
&  0.96857E+00_JPRB, 0.96134E+00_JPRB, 0.94515E+00_JPRB, 0.93367E+00_JPRB, 0.93595E+00_JPRB, 0.96629E+00_JPRB, 0.10350E+01_JPRB ,&
&  0.13751E+01_JPRB, 0.36151E+01_JPRB, 0.34902E+01_JPRB, 0.24890E+01_JPRB, 0.19190E+01_JPRB, 0.16312E+01_JPRB, 0.15054E+01_JPRB ,&
&  0.16288E+01_JPRB, 0.16733E+01_JPRB, 0.16589E+01_JPRB, 0.14670E+01_JPRB, 0.13477E+01_JPRB, 0.13000E+01_JPRB, 0.13029E+01_JPRB ,&
&  0.12792E+01_JPRB, 0.12806E+01_JPRB, 0.12636E+01_JPRB, 0.12462E+01_JPRB, 0.12376E+01_JPRB, 0.12769E+01_JPRB, 0.13312E+01_JPRB ,&
&  0.13835E+01_JPRB, 0.14919E+01_JPRB, 0.16883E+01_JPRB, 0.18432E+01_JPRB, 0.17912E+01_JPRB, 0.18212E+01_JPRB, 0.17513E+01_JPRB ,&
&  0.18270E+01_JPRB, 0.17716E+01_JPRB, 0.16064E+01_JPRB, 0.14579E+01_JPRB, 0.13104E+01_JPRB, 0.13552E+01_JPRB, 0.13020E+01_JPRB ,&
&  0.12207E+01_JPRB, 0.11634E+01_JPRB, 0.11466E+01_JPRB, 0.11493E+01_JPRB, 0.11544E+01_JPRB, 0.11521E+01_JPRB, 0.11257E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.11058E+01_JPRB, 0.11091E+01_JPRB, 0.11168E+01_JPRB, 0.11218E+01_JPRB, 0.11327E+01_JPRB, 0.11767E+01_JPRB ,&
&  0.12170E+01_JPRB, 0.12792E+01_JPRB, 0.13138E+01_JPRB, 0.13981E+01_JPRB, 0.14107E+01_JPRB, 0.13459E+01_JPRB, 0.12810E+01_JPRB ,&
&  0.13117E+01_JPRB, 0.13777E+01_JPRB, 0.13169E+01_JPRB, 0.12727E+01_JPRB, 0.12099E+01_JPRB, 0.11247E+01_JPRB, 0.10521E+01_JPRB ,&
&  0.10512E+01_JPRB, 0.10538E+01_JPRB, 0.10104E+01_JPRB, 0.95086E+00_JPRB, 0.95942E+00_JPRB /)
ZBURD(:, 47, 12) = (/ &
&  0.96727E+00_JPRB, 0.94092E+00_JPRB, 0.92783E+00_JPRB, 0.94048E+00_JPRB, 0.94666E+00_JPRB, 0.92762E+00_JPRB, 0.90047E+00_JPRB ,&
&  0.87762E+00_JPRB, 0.85316E+00_JPRB, 0.85896E+00_JPRB, 0.84604E+00_JPRB, 0.83955E+00_JPRB, 0.83319E+00_JPRB, 0.82980E+00_JPRB ,&
&  0.83339E+00_JPRB, 0.84136E+00_JPRB, 0.82848E+00_JPRB, 0.83032E+00_JPRB, 0.85411E+00_JPRB, 0.87989E+00_JPRB, 0.89163E+00_JPRB ,&
&  0.90218E+00_JPRB, 0.91270E+00_JPRB, 0.91109E+00_JPRB, 0.91007E+00_JPRB, 0.89934E+00_JPRB, 0.90808E+00_JPRB, 0.93920E+00_JPRB ,&
&  0.11417E+01_JPRB, 0.28604E+01_JPRB, 0.20529E+01_JPRB, 0.24973E+01_JPRB, 0.18931E+01_JPRB, 0.16199E+01_JPRB, 0.13567E+01_JPRB ,&
&  0.14658E+01_JPRB, 0.18011E+01_JPRB, 0.15897E+01_JPRB, 0.14482E+01_JPRB, 0.13381E+01_JPRB, 0.12755E+01_JPRB, 0.12295E+01_JPRB ,&
&  0.12005E+01_JPRB, 0.11987E+01_JPRB, 0.11850E+01_JPRB, 0.11800E+01_JPRB, 0.11595E+01_JPRB, 0.11747E+01_JPRB, 0.11995E+01_JPRB ,&
&  0.12216E+01_JPRB, 0.12699E+01_JPRB, 0.13917E+01_JPRB, 0.15224E+01_JPRB, 0.18377E+01_JPRB, 0.20154E+01_JPRB, 0.20796E+01_JPRB ,&
&  0.21362E+01_JPRB, 0.20460E+01_JPRB, 0.17922E+01_JPRB, 0.15367E+01_JPRB, 0.14127E+01_JPRB, 0.13634E+01_JPRB, 0.13485E+01_JPRB ,&
&  0.12668E+01_JPRB, 0.12164E+01_JPRB, 0.11786E+01_JPRB, 0.11560E+01_JPRB, 0.11593E+01_JPRB, 0.11518E+01_JPRB, 0.11050E+01_JPRB ,&
&  0.10734E+01_JPRB, 0.10802E+01_JPRB, 0.10942E+01_JPRB, 0.11053E+01_JPRB, 0.10970E+01_JPRB, 0.10817E+01_JPRB, 0.10971E+01_JPRB ,&
&  0.11367E+01_JPRB, 0.11880E+01_JPRB, 0.12279E+01_JPRB, 0.13298E+01_JPRB, 0.13292E+01_JPRB, 0.13007E+01_JPRB, 0.12590E+01_JPRB ,&
&  0.12959E+01_JPRB, 0.13260E+01_JPRB, 0.13076E+01_JPRB, 0.13079E+01_JPRB, 0.13105E+01_JPRB, 0.12618E+01_JPRB, 0.10824E+01_JPRB ,&
&  0.10439E+01_JPRB, 0.10255E+01_JPRB, 0.10103E+01_JPRB, 0.97893E+00_JPRB, 0.98907E+00_JPRB /)
ZBURD(:, 48, 12) = (/ &
&  0.97359E+00_JPRB, 0.95841E+00_JPRB, 0.97681E+00_JPRB, 0.98961E+00_JPRB, 0.98129E+00_JPRB, 0.94952E+00_JPRB, 0.92891E+00_JPRB ,&
&  0.93577E+00_JPRB, 0.92935E+00_JPRB, 0.91931E+00_JPRB, 0.89276E+00_JPRB, 0.87791E+00_JPRB, 0.85860E+00_JPRB, 0.84735E+00_JPRB ,&
&  0.84993E+00_JPRB, 0.85049E+00_JPRB, 0.83818E+00_JPRB, 0.83466E+00_JPRB, 0.83056E+00_JPRB, 0.84321E+00_JPRB, 0.85637E+00_JPRB ,&
&  0.86873E+00_JPRB, 0.87762E+00_JPRB, 0.89219E+00_JPRB, 0.89871E+00_JPRB, 0.89238E+00_JPRB, 0.88206E+00_JPRB, 0.89589E+00_JPRB ,&
&  0.10597E+01_JPRB, 0.28580E+01_JPRB, 0.17364E+01_JPRB, 0.24814E+01_JPRB, 0.18701E+01_JPRB, 0.16321E+01_JPRB, 0.13583E+01_JPRB ,&
&  0.14511E+01_JPRB, 0.17279E+01_JPRB, 0.15705E+01_JPRB, 0.13880E+01_JPRB, 0.13244E+01_JPRB, 0.12718E+01_JPRB, 0.12049E+01_JPRB ,&
&  0.11693E+01_JPRB, 0.11692E+01_JPRB, 0.11510E+01_JPRB, 0.11495E+01_JPRB, 0.11429E+01_JPRB, 0.11275E+01_JPRB, 0.11282E+01_JPRB ,&
&  0.11255E+01_JPRB, 0.11382E+01_JPRB, 0.12019E+01_JPRB, 0.14807E+01_JPRB, 0.16830E+01_JPRB, 0.21086E+01_JPRB, 0.25433E+01_JPRB ,&
&  0.24924E+01_JPRB, 0.23305E+01_JPRB, 0.18628E+01_JPRB, 0.16320E+01_JPRB, 0.15025E+01_JPRB, 0.14270E+01_JPRB, 0.13801E+01_JPRB ,&
&  0.13255E+01_JPRB, 0.13003E+01_JPRB, 0.12322E+01_JPRB, 0.11989E+01_JPRB, 0.11910E+01_JPRB, 0.11755E+01_JPRB, 0.11374E+01_JPRB ,&
&  0.11021E+01_JPRB, 0.11029E+01_JPRB, 0.11143E+01_JPRB, 0.11126E+01_JPRB, 0.10992E+01_JPRB, 0.10709E+01_JPRB, 0.10712E+01_JPRB ,&
&  0.11077E+01_JPRB, 0.11455E+01_JPRB, 0.12104E+01_JPRB, 0.13436E+01_JPRB, 0.13324E+01_JPRB, 0.12811E+01_JPRB, 0.12572E+01_JPRB ,&
&  0.13005E+01_JPRB, 0.12958E+01_JPRB, 0.13000E+01_JPRB, 0.13828E+01_JPRB, 0.14498E+01_JPRB, 0.15012E+01_JPRB, 0.12157E+01_JPRB ,&
&  0.11074E+01_JPRB, 0.10379E+01_JPRB, 0.10031E+01_JPRB, 0.98464E+00_JPRB, 0.98333E+00_JPRB /)
ZBURD(:, 49, 12) = (/ &
&  0.10234E+01_JPRB, 0.10153E+01_JPRB, 0.10321E+01_JPRB, 0.10567E+01_JPRB, 0.10364E+01_JPRB, 0.99737E+00_JPRB, 0.96242E+00_JPRB ,&
&  0.97370E+00_JPRB, 0.97346E+00_JPRB, 0.96608E+00_JPRB, 0.93291E+00_JPRB, 0.91692E+00_JPRB, 0.90050E+00_JPRB, 0.89178E+00_JPRB ,&
&  0.89272E+00_JPRB, 0.87973E+00_JPRB, 0.85862E+00_JPRB, 0.84605E+00_JPRB, 0.84426E+00_JPRB, 0.84435E+00_JPRB, 0.85876E+00_JPRB ,&
&  0.85047E+00_JPRB, 0.85545E+00_JPRB, 0.87636E+00_JPRB, 0.88432E+00_JPRB, 0.89144E+00_JPRB, 0.87406E+00_JPRB, 0.87637E+00_JPRB ,&
&  0.96823E+00_JPRB, 0.11595E+01_JPRB, 0.19284E+01_JPRB, 0.22677E+01_JPRB, 0.18760E+01_JPRB, 0.16713E+01_JPRB, 0.15478E+01_JPRB ,&
&  0.15774E+01_JPRB, 0.16071E+01_JPRB, 0.15035E+01_JPRB, 0.13878E+01_JPRB, 0.13224E+01_JPRB, 0.12502E+01_JPRB, 0.11921E+01_JPRB ,&
&  0.11407E+01_JPRB, 0.11123E+01_JPRB, 0.11064E+01_JPRB, 0.11185E+01_JPRB, 0.11203E+01_JPRB, 0.11091E+01_JPRB, 0.10797E+01_JPRB ,&
&  0.10596E+01_JPRB, 0.10471E+01_JPRB, 0.10667E+01_JPRB, 0.12416E+01_JPRB, 0.16416E+01_JPRB, 0.21506E+01_JPRB, 0.25182E+01_JPRB ,&
&  0.26149E+01_JPRB, 0.23355E+01_JPRB, 0.18683E+01_JPRB, 0.16844E+01_JPRB, 0.15519E+01_JPRB, 0.14597E+01_JPRB, 0.13678E+01_JPRB ,&
&  0.13157E+01_JPRB, 0.13201E+01_JPRB, 0.12990E+01_JPRB, 0.12513E+01_JPRB, 0.12013E+01_JPRB, 0.11738E+01_JPRB, 0.11426E+01_JPRB ,&
&  0.11254E+01_JPRB, 0.11345E+01_JPRB, 0.11379E+01_JPRB, 0.11154E+01_JPRB, 0.10919E+01_JPRB, 0.10671E+01_JPRB, 0.10664E+01_JPRB ,&
&  0.10982E+01_JPRB, 0.11476E+01_JPRB, 0.12518E+01_JPRB, 0.13340E+01_JPRB, 0.13281E+01_JPRB, 0.12950E+01_JPRB, 0.12800E+01_JPRB ,&
&  0.12864E+01_JPRB, 0.12740E+01_JPRB, 0.12741E+01_JPRB, 0.14284E+01_JPRB, 0.15330E+01_JPRB, 0.16106E+01_JPRB, 0.14138E+01_JPRB ,&
&  0.12372E+01_JPRB, 0.11219E+01_JPRB, 0.10657E+01_JPRB, 0.10408E+01_JPRB, 0.10382E+01_JPRB /)
ZBURD(:, 50, 12) = (/ &
&  0.11058E+01_JPRB, 0.11013E+01_JPRB, 0.11218E+01_JPRB, 0.11381E+01_JPRB, 0.11169E+01_JPRB, 0.10702E+01_JPRB, 0.10249E+01_JPRB ,&
&  0.10026E+01_JPRB, 0.98675E+00_JPRB, 0.96814E+00_JPRB, 0.93905E+00_JPRB, 0.92847E+00_JPRB, 0.93030E+00_JPRB, 0.93215E+00_JPRB ,&
&  0.93257E+00_JPRB, 0.91614E+00_JPRB, 0.89292E+00_JPRB, 0.86963E+00_JPRB, 0.86391E+00_JPRB, 0.86503E+00_JPRB, 0.87504E+00_JPRB ,&
&  0.86848E+00_JPRB, 0.86534E+00_JPRB, 0.86962E+00_JPRB, 0.87480E+00_JPRB, 0.88157E+00_JPRB, 0.87148E+00_JPRB, 0.86180E+00_JPRB ,&
&  0.90541E+00_JPRB, 0.13923E+01_JPRB, 0.17304E+01_JPRB, 0.18618E+01_JPRB, 0.17968E+01_JPRB, 0.16516E+01_JPRB, 0.15849E+01_JPRB ,&
&  0.14866E+01_JPRB, 0.14744E+01_JPRB, 0.14563E+01_JPRB, 0.13978E+01_JPRB, 0.13250E+01_JPRB, 0.12574E+01_JPRB, 0.12036E+01_JPRB ,&
&  0.11541E+01_JPRB, 0.11172E+01_JPRB, 0.10882E+01_JPRB, 0.10823E+01_JPRB, 0.10828E+01_JPRB, 0.10887E+01_JPRB, 0.10658E+01_JPRB ,&
&  0.10342E+01_JPRB, 0.10071E+01_JPRB, 0.99437E+00_JPRB, 0.10767E+01_JPRB, 0.14172E+01_JPRB, 0.16044E+01_JPRB, 0.21543E+01_JPRB ,&
&  0.24347E+01_JPRB, 0.19554E+01_JPRB, 0.16743E+01_JPRB, 0.15477E+01_JPRB, 0.14586E+01_JPRB, 0.13677E+01_JPRB, 0.13005E+01_JPRB ,&
&  0.12721E+01_JPRB, 0.12652E+01_JPRB, 0.12838E+01_JPRB, 0.12728E+01_JPRB, 0.12171E+01_JPRB, 0.11520E+01_JPRB, 0.11200E+01_JPRB ,&
&  0.11016E+01_JPRB, 0.10843E+01_JPRB, 0.10772E+01_JPRB, 0.10701E+01_JPRB, 0.10590E+01_JPRB, 0.10321E+01_JPRB, 0.10250E+01_JPRB ,&
&  0.10425E+01_JPRB, 0.10953E+01_JPRB, 0.12251E+01_JPRB, 0.12492E+01_JPRB, 0.12852E+01_JPRB, 0.13009E+01_JPRB, 0.12570E+01_JPRB ,&
&  0.12025E+01_JPRB, 0.12420E+01_JPRB, 0.12500E+01_JPRB, 0.14348E+01_JPRB, 0.17490E+01_JPRB, 0.18026E+01_JPRB, 0.15878E+01_JPRB ,&
&  0.13816E+01_JPRB, 0.12288E+01_JPRB, 0.11497E+01_JPRB, 0.11100E+01_JPRB, 0.11234E+01_JPRB /)
ZBURD(:, 51, 12) = (/ &
&  0.11824E+01_JPRB, 0.11630E+01_JPRB, 0.11688E+01_JPRB, 0.11745E+01_JPRB, 0.11579E+01_JPRB, 0.11136E+01_JPRB, 0.10722E+01_JPRB ,&
&  0.10490E+01_JPRB, 0.10069E+01_JPRB, 0.96799E+00_JPRB, 0.94206E+00_JPRB, 0.94545E+00_JPRB, 0.95220E+00_JPRB, 0.95210E+00_JPRB ,&
&  0.93934E+00_JPRB, 0.93265E+00_JPRB, 0.91410E+00_JPRB, 0.89782E+00_JPRB, 0.88803E+00_JPRB, 0.88044E+00_JPRB, 0.86539E+00_JPRB ,&
&  0.86443E+00_JPRB, 0.86188E+00_JPRB, 0.86451E+00_JPRB, 0.87146E+00_JPRB, 0.86785E+00_JPRB, 0.87029E+00_JPRB, 0.86473E+00_JPRB ,&
&  0.86295E+00_JPRB, 0.10496E+01_JPRB, 0.12193E+01_JPRB, 0.14910E+01_JPRB, 0.15906E+01_JPRB, 0.15551E+01_JPRB, 0.14926E+01_JPRB ,&
&  0.14143E+01_JPRB, 0.13721E+01_JPRB, 0.13635E+01_JPRB, 0.13295E+01_JPRB, 0.12757E+01_JPRB, 0.12269E+01_JPRB, 0.11818E+01_JPRB ,&
&  0.11464E+01_JPRB, 0.11201E+01_JPRB, 0.11115E+01_JPRB, 0.10941E+01_JPRB, 0.10724E+01_JPRB, 0.10666E+01_JPRB, 0.10685E+01_JPRB ,&
&  0.10242E+01_JPRB, 0.99047E+00_JPRB, 0.96482E+00_JPRB, 0.96728E+00_JPRB, 0.11532E+01_JPRB, 0.13662E+01_JPRB, 0.16557E+01_JPRB ,&
&  0.15940E+01_JPRB, 0.14795E+01_JPRB, 0.14004E+01_JPRB, 0.13373E+01_JPRB, 0.13037E+01_JPRB, 0.12389E+01_JPRB, 0.11914E+01_JPRB ,&
&  0.11421E+01_JPRB, 0.11289E+01_JPRB, 0.11175E+01_JPRB, 0.11165E+01_JPRB, 0.11065E+01_JPRB, 0.10993E+01_JPRB, 0.10628E+01_JPRB ,&
&  0.10414E+01_JPRB, 0.10238E+01_JPRB, 0.10156E+01_JPRB, 0.10162E+01_JPRB, 0.10231E+01_JPRB, 0.10140E+01_JPRB, 0.10073E+01_JPRB ,&
&  0.10231E+01_JPRB, 0.10443E+01_JPRB, 0.11123E+01_JPRB, 0.11953E+01_JPRB, 0.11999E+01_JPRB, 0.11781E+01_JPRB, 0.11699E+01_JPRB ,&
&  0.11719E+01_JPRB, 0.12253E+01_JPRB, 0.11948E+01_JPRB, 0.14500E+01_JPRB, 0.16801E+01_JPRB, 0.17577E+01_JPRB, 0.16200E+01_JPRB ,&
&  0.14738E+01_JPRB, 0.13209E+01_JPRB, 0.12299E+01_JPRB, 0.11813E+01_JPRB, 0.11934E+01_JPRB /)
ZBURD(:, 52, 12) = (/ &
&  0.12487E+01_JPRB, 0.12351E+01_JPRB, 0.12183E+01_JPRB, 0.12073E+01_JPRB, 0.11891E+01_JPRB, 0.11524E+01_JPRB, 0.11096E+01_JPRB ,&
&  0.10755E+01_JPRB, 0.10381E+01_JPRB, 0.99160E+00_JPRB, 0.96624E+00_JPRB, 0.95770E+00_JPRB, 0.95465E+00_JPRB, 0.94714E+00_JPRB ,&
&  0.93900E+00_JPRB, 0.93943E+00_JPRB, 0.92996E+00_JPRB, 0.92667E+00_JPRB, 0.91573E+00_JPRB, 0.90671E+00_JPRB, 0.88277E+00_JPRB ,&
&  0.86584E+00_JPRB, 0.85371E+00_JPRB, 0.84952E+00_JPRB, 0.85534E+00_JPRB, 0.86079E+00_JPRB, 0.86443E+00_JPRB, 0.87028E+00_JPRB ,&
&  0.87713E+00_JPRB, 0.83393E+00_JPRB, 0.94137E+00_JPRB, 0.11226E+01_JPRB, 0.12528E+01_JPRB, 0.13444E+01_JPRB, 0.13241E+01_JPRB ,&
&  0.12907E+01_JPRB, 0.12565E+01_JPRB, 0.12491E+01_JPRB, 0.12208E+01_JPRB, 0.11754E+01_JPRB, 0.11323E+01_JPRB, 0.11028E+01_JPRB ,&
&  0.10945E+01_JPRB, 0.10834E+01_JPRB, 0.10758E+01_JPRB, 0.10673E+01_JPRB, 0.10576E+01_JPRB, 0.10485E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10365E+01_JPRB, 0.99981E+00_JPRB, 0.96248E+00_JPRB, 0.93725E+00_JPRB, 0.96228E+00_JPRB, 0.10419E+01_JPRB, 0.11829E+01_JPRB ,&
&  0.12634E+01_JPRB, 0.12311E+01_JPRB, 0.11946E+01_JPRB, 0.11623E+01_JPRB, 0.11384E+01_JPRB, 0.11314E+01_JPRB, 0.11197E+01_JPRB ,&
&  0.11134E+01_JPRB, 0.11027E+01_JPRB, 0.10960E+01_JPRB, 0.10850E+01_JPRB, 0.10749E+01_JPRB, 0.10679E+01_JPRB, 0.10565E+01_JPRB ,&
&  0.10449E+01_JPRB, 0.10414E+01_JPRB, 0.10303E+01_JPRB, 0.10305E+01_JPRB, 0.10258E+01_JPRB, 0.10169E+01_JPRB, 0.10091E+01_JPRB ,&
&  0.10218E+01_JPRB, 0.10560E+01_JPRB, 0.10849E+01_JPRB, 0.11204E+01_JPRB, 0.11283E+01_JPRB, 0.11178E+01_JPRB, 0.11212E+01_JPRB ,&
&  0.11150E+01_JPRB, 0.11163E+01_JPRB, 0.11004E+01_JPRB, 0.13440E+01_JPRB, 0.14969E+01_JPRB, 0.14913E+01_JPRB, 0.14698E+01_JPRB ,&
&  0.14550E+01_JPRB, 0.14107E+01_JPRB, 0.13164E+01_JPRB, 0.12215E+01_JPRB, 0.12261E+01_JPRB /)
ZBURD(:, 53, 12) = (/ &
&  0.11545E+01_JPRB, 0.11749E+01_JPRB, 0.11575E+01_JPRB, 0.11421E+01_JPRB, 0.11294E+01_JPRB, 0.11095E+01_JPRB, 0.10827E+01_JPRB ,&
&  0.10545E+01_JPRB, 0.10361E+01_JPRB, 0.10139E+01_JPRB, 0.98601E+00_JPRB, 0.96596E+00_JPRB, 0.95211E+00_JPRB, 0.93879E+00_JPRB ,&
&  0.93050E+00_JPRB, 0.92180E+00_JPRB, 0.91435E+00_JPRB, 0.90643E+00_JPRB, 0.89311E+00_JPRB, 0.88119E+00_JPRB, 0.86484E+00_JPRB ,&
&  0.85363E+00_JPRB, 0.85085E+00_JPRB, 0.85000E+00_JPRB, 0.85317E+00_JPRB, 0.86028E+00_JPRB, 0.86873E+00_JPRB, 0.87462E+00_JPRB ,&
&  0.88077E+00_JPRB, 0.78481E+00_JPRB, 0.82118E+00_JPRB, 0.92096E+00_JPRB, 0.10418E+01_JPRB, 0.11503E+01_JPRB, 0.11865E+01_JPRB ,&
&  0.11866E+01_JPRB, 0.11747E+01_JPRB, 0.11521E+01_JPRB, 0.11263E+01_JPRB, 0.11004E+01_JPRB, 0.10822E+01_JPRB, 0.10483E+01_JPRB ,&
&  0.10270E+01_JPRB, 0.10185E+01_JPRB, 0.10211E+01_JPRB, 0.10198E+01_JPRB, 0.10136E+01_JPRB, 0.10058E+01_JPRB, 0.10038E+01_JPRB ,&
&  0.99687E+00_JPRB, 0.97284E+00_JPRB, 0.94523E+00_JPRB, 0.92518E+00_JPRB, 0.92433E+00_JPRB, 0.96665E+00_JPRB, 0.10317E+01_JPRB ,&
&  0.10912E+01_JPRB, 0.10876E+01_JPRB, 0.10626E+01_JPRB, 0.10399E+01_JPRB, 0.10314E+01_JPRB, 0.10366E+01_JPRB, 0.10502E+01_JPRB ,&
&  0.10508E+01_JPRB, 0.10481E+01_JPRB, 0.10592E+01_JPRB, 0.10609E+01_JPRB, 0.10625E+01_JPRB, 0.10602E+01_JPRB, 0.10574E+01_JPRB ,&
&  0.10485E+01_JPRB, 0.10370E+01_JPRB, 0.10307E+01_JPRB, 0.10259E+01_JPRB, 0.10276E+01_JPRB, 0.10231E+01_JPRB, 0.10125E+01_JPRB ,&
&  0.10006E+01_JPRB, 0.10146E+01_JPRB, 0.10547E+01_JPRB, 0.10904E+01_JPRB, 0.10912E+01_JPRB, 0.10823E+01_JPRB, 0.10772E+01_JPRB ,&
&  0.10837E+01_JPRB, 0.10824E+01_JPRB, 0.10805E+01_JPRB, 0.12016E+01_JPRB, 0.12282E+01_JPRB, 0.12488E+01_JPRB, 0.12704E+01_JPRB ,&
&  0.12877E+01_JPRB, 0.12799E+01_JPRB, 0.12613E+01_JPRB, 0.12168E+01_JPRB, 0.11005E+01_JPRB /)
ZBURD(:, 54, 12) = (/ &
&  0.10460E+01_JPRB, 0.10700E+01_JPRB, 0.10742E+01_JPRB, 0.10657E+01_JPRB, 0.10635E+01_JPRB, 0.10614E+01_JPRB, 0.10563E+01_JPRB ,&
&  0.10533E+01_JPRB, 0.10464E+01_JPRB, 0.10307E+01_JPRB, 0.10057E+01_JPRB, 0.97975E+00_JPRB, 0.95133E+00_JPRB, 0.92772E+00_JPRB ,&
&  0.91690E+00_JPRB, 0.90848E+00_JPRB, 0.89027E+00_JPRB, 0.87362E+00_JPRB, 0.85869E+00_JPRB, 0.84703E+00_JPRB, 0.84200E+00_JPRB ,&
&  0.83653E+00_JPRB, 0.83404E+00_JPRB, 0.83906E+00_JPRB, 0.84590E+00_JPRB, 0.85522E+00_JPRB, 0.86577E+00_JPRB, 0.88012E+00_JPRB ,&
&  0.88484E+00_JPRB, 0.70433E+00_JPRB, 0.73422E+00_JPRB, 0.82309E+00_JPRB, 0.89454E+00_JPRB, 0.98060E+00_JPRB, 0.10586E+01_JPRB ,&
&  0.10861E+01_JPRB, 0.11086E+01_JPRB, 0.10969E+01_JPRB, 0.10777E+01_JPRB, 0.10711E+01_JPRB, 0.10497E+01_JPRB, 0.10073E+01_JPRB ,&
&  0.97485E+00_JPRB, 0.97038E+00_JPRB, 0.98083E+00_JPRB, 0.98742E+00_JPRB, 0.98392E+00_JPRB, 0.97579E+00_JPRB, 0.97168E+00_JPRB ,&
&  0.96741E+00_JPRB, 0.96220E+00_JPRB, 0.95585E+00_JPRB, 0.94663E+00_JPRB, 0.95274E+00_JPRB, 0.97328E+00_JPRB, 0.99426E+00_JPRB ,&
&  0.10002E+01_JPRB, 0.99484E+00_JPRB, 0.98775E+00_JPRB, 0.97821E+00_JPRB, 0.97644E+00_JPRB, 0.98970E+00_JPRB, 0.10029E+01_JPRB ,&
&  0.10075E+01_JPRB, 0.10086E+01_JPRB, 0.10158E+01_JPRB, 0.10183E+01_JPRB, 0.10256E+01_JPRB, 0.10335E+01_JPRB, 0.10475E+01_JPRB ,&
&  0.10553E+01_JPRB, 0.10494E+01_JPRB, 0.10415E+01_JPRB, 0.10281E+01_JPRB, 0.10087E+01_JPRB, 0.99639E+00_JPRB, 0.99241E+00_JPRB ,&
&  0.98844E+00_JPRB, 0.99529E+00_JPRB, 0.10173E+01_JPRB, 0.10433E+01_JPRB, 0.10532E+01_JPRB, 0.10572E+01_JPRB, 0.10619E+01_JPRB ,&
&  0.10710E+01_JPRB, 0.10784E+01_JPRB, 0.10814E+01_JPRB, 0.11202E+01_JPRB, 0.11814E+01_JPRB, 0.11886E+01_JPRB, 0.11751E+01_JPRB ,&
&  0.11504E+01_JPRB, 0.11214E+01_JPRB, 0.11234E+01_JPRB, 0.93694E+00_JPRB, 0.10214E+01_JPRB /)
ZBURD(:, 55, 12) = (/ &
&  0.94895E+00_JPRB, 0.96700E+00_JPRB, 0.96610E+00_JPRB, 0.97512E+00_JPRB, 0.98069E+00_JPRB, 0.98513E+00_JPRB, 0.99342E+00_JPRB ,&
&  0.99886E+00_JPRB, 0.99370E+00_JPRB, 0.98396E+00_JPRB, 0.96582E+00_JPRB, 0.93831E+00_JPRB, 0.90898E+00_JPRB, 0.88439E+00_JPRB ,&
&  0.86568E+00_JPRB, 0.85935E+00_JPRB, 0.84797E+00_JPRB, 0.83481E+00_JPRB, 0.82677E+00_JPRB, 0.82246E+00_JPRB, 0.82418E+00_JPRB ,&
&  0.83184E+00_JPRB, 0.83586E+00_JPRB, 0.83073E+00_JPRB, 0.82934E+00_JPRB, 0.83807E+00_JPRB, 0.85409E+00_JPRB, 0.86162E+00_JPRB ,&
&  0.86343E+00_JPRB, 0.66831E+00_JPRB, 0.71252E+00_JPRB, 0.76590E+00_JPRB, 0.81432E+00_JPRB, 0.88359E+00_JPRB, 0.95703E+00_JPRB ,&
&  0.10046E+01_JPRB, 0.10328E+01_JPRB, 0.10326E+01_JPRB, 0.10208E+01_JPRB, 0.10006E+01_JPRB, 0.98225E+00_JPRB, 0.96026E+00_JPRB ,&
&  0.94068E+00_JPRB, 0.93372E+00_JPRB, 0.93702E+00_JPRB, 0.94134E+00_JPRB, 0.94327E+00_JPRB, 0.94740E+00_JPRB, 0.94191E+00_JPRB ,&
&  0.94012E+00_JPRB, 0.93704E+00_JPRB, 0.93522E+00_JPRB, 0.93522E+00_JPRB, 0.93389E+00_JPRB, 0.93288E+00_JPRB, 0.92963E+00_JPRB ,&
&  0.91738E+00_JPRB, 0.90999E+00_JPRB, 0.90563E+00_JPRB, 0.90174E+00_JPRB, 0.90657E+00_JPRB, 0.91849E+00_JPRB, 0.93597E+00_JPRB ,&
&  0.95572E+00_JPRB, 0.95909E+00_JPRB, 0.96515E+00_JPRB, 0.97304E+00_JPRB, 0.98546E+00_JPRB, 0.99860E+00_JPRB, 0.10148E+01_JPRB ,&
&  0.10222E+01_JPRB, 0.10174E+01_JPRB, 0.10130E+01_JPRB, 0.10024E+01_JPRB, 0.10004E+01_JPRB, 0.98412E+00_JPRB, 0.96496E+00_JPRB ,&
&  0.96201E+00_JPRB, 0.96256E+00_JPRB, 0.96568E+00_JPRB, 0.97955E+00_JPRB, 0.98900E+00_JPRB, 0.99089E+00_JPRB, 0.10001E+01_JPRB ,&
&  0.10126E+01_JPRB, 0.10211E+01_JPRB, 0.10166E+01_JPRB, 0.10302E+01_JPRB, 0.10351E+01_JPRB, 0.10432E+01_JPRB, 0.10321E+01_JPRB ,&
&  0.10125E+01_JPRB, 0.99863E+00_JPRB, 0.78604E+00_JPRB, 0.88924E+00_JPRB, 0.91841E+00_JPRB /)
ZBURD(:, 56, 12) = (/ &
&  0.91261E+00_JPRB, 0.91839E+00_JPRB, 0.92076E+00_JPRB, 0.92427E+00_JPRB, 0.92352E+00_JPRB, 0.92046E+00_JPRB, 0.92337E+00_JPRB ,&
&  0.92658E+00_JPRB, 0.92022E+00_JPRB, 0.91002E+00_JPRB, 0.90559E+00_JPRB, 0.89462E+00_JPRB, 0.87868E+00_JPRB, 0.86492E+00_JPRB ,&
&  0.84807E+00_JPRB, 0.83990E+00_JPRB, 0.83448E+00_JPRB, 0.82942E+00_JPRB, 0.82198E+00_JPRB, 0.81326E+00_JPRB, 0.81649E+00_JPRB ,&
&  0.82319E+00_JPRB, 0.83050E+00_JPRB, 0.83260E+00_JPRB, 0.83666E+00_JPRB, 0.84271E+00_JPRB, 0.84959E+00_JPRB, 0.86196E+00_JPRB ,&
&  0.75026E+00_JPRB, 0.62485E+00_JPRB, 0.67770E+00_JPRB, 0.73225E+00_JPRB, 0.78121E+00_JPRB, 0.85600E+00_JPRB, 0.91178E+00_JPRB ,&
&  0.95433E+00_JPRB, 0.98798E+00_JPRB, 0.99534E+00_JPRB, 0.97243E+00_JPRB, 0.94862E+00_JPRB, 0.94108E+00_JPRB, 0.94054E+00_JPRB ,&
&  0.93678E+00_JPRB, 0.93498E+00_JPRB, 0.93827E+00_JPRB, 0.94224E+00_JPRB, 0.94662E+00_JPRB, 0.95282E+00_JPRB, 0.95521E+00_JPRB ,&
&  0.95326E+00_JPRB, 0.94054E+00_JPRB, 0.92722E+00_JPRB, 0.90672E+00_JPRB, 0.88878E+00_JPRB, 0.88224E+00_JPRB, 0.87313E+00_JPRB ,&
&  0.85788E+00_JPRB, 0.84966E+00_JPRB, 0.84961E+00_JPRB, 0.85385E+00_JPRB, 0.86425E+00_JPRB, 0.87655E+00_JPRB, 0.89532E+00_JPRB ,&
&  0.91678E+00_JPRB, 0.93308E+00_JPRB, 0.94485E+00_JPRB, 0.95813E+00_JPRB, 0.96509E+00_JPRB, 0.97409E+00_JPRB, 0.99107E+00_JPRB ,&
&  0.10052E+01_JPRB, 0.99583E+00_JPRB, 0.98678E+00_JPRB, 0.97752E+00_JPRB, 0.97442E+00_JPRB, 0.97212E+00_JPRB, 0.96258E+00_JPRB ,&
&  0.95359E+00_JPRB, 0.94798E+00_JPRB, 0.94571E+00_JPRB, 0.95153E+00_JPRB, 0.95172E+00_JPRB, 0.94703E+00_JPRB, 0.94565E+00_JPRB ,&
&  0.94493E+00_JPRB, 0.93670E+00_JPRB, 0.92578E+00_JPRB, 0.92255E+00_JPRB, 0.91955E+00_JPRB, 0.92217E+00_JPRB, 0.92163E+00_JPRB ,&
&  0.92046E+00_JPRB, 0.90831E+00_JPRB, 0.91603E+00_JPRB, 0.91029E+00_JPRB, 0.90696E+00_JPRB /)
ZBURD(:, 57, 12) = (/ &
&  0.87428E+00_JPRB, 0.87825E+00_JPRB, 0.88152E+00_JPRB, 0.88198E+00_JPRB, 0.87815E+00_JPRB, 0.87500E+00_JPRB, 0.87570E+00_JPRB ,&
&  0.87561E+00_JPRB, 0.86667E+00_JPRB, 0.85786E+00_JPRB, 0.85104E+00_JPRB, 0.84771E+00_JPRB, 0.83764E+00_JPRB, 0.82255E+00_JPRB ,&
&  0.81125E+00_JPRB, 0.80868E+00_JPRB, 0.81309E+00_JPRB, 0.81553E+00_JPRB, 0.80659E+00_JPRB, 0.79721E+00_JPRB, 0.79389E+00_JPRB ,&
&  0.79486E+00_JPRB, 0.80106E+00_JPRB, 0.81145E+00_JPRB, 0.81759E+00_JPRB, 0.82642E+00_JPRB, 0.84066E+00_JPRB, 0.84188E+00_JPRB ,&
&  0.84008E+00_JPRB, 0.70287E+00_JPRB, 0.74135E+00_JPRB, 0.75695E+00_JPRB, 0.78793E+00_JPRB, 0.83374E+00_JPRB, 0.87252E+00_JPRB ,&
&  0.90993E+00_JPRB, 0.93122E+00_JPRB, 0.94067E+00_JPRB, 0.93405E+00_JPRB, 0.91837E+00_JPRB, 0.90377E+00_JPRB, 0.90031E+00_JPRB ,&
&  0.90028E+00_JPRB, 0.90796E+00_JPRB, 0.92188E+00_JPRB, 0.93138E+00_JPRB, 0.93471E+00_JPRB, 0.93567E+00_JPRB, 0.93828E+00_JPRB ,&
&  0.93522E+00_JPRB, 0.91885E+00_JPRB, 0.89535E+00_JPRB, 0.86647E+00_JPRB, 0.84339E+00_JPRB, 0.83208E+00_JPRB, 0.81865E+00_JPRB ,&
&  0.80034E+00_JPRB, 0.78935E+00_JPRB, 0.78785E+00_JPRB, 0.79133E+00_JPRB, 0.80128E+00_JPRB, 0.81251E+00_JPRB, 0.82199E+00_JPRB ,&
&  0.83244E+00_JPRB, 0.84878E+00_JPRB, 0.86649E+00_JPRB, 0.88364E+00_JPRB, 0.90074E+00_JPRB, 0.91631E+00_JPRB, 0.92810E+00_JPRB ,&
&  0.93578E+00_JPRB, 0.93381E+00_JPRB, 0.92222E+00_JPRB, 0.91339E+00_JPRB, 0.91009E+00_JPRB, 0.91473E+00_JPRB, 0.91587E+00_JPRB ,&
&  0.90801E+00_JPRB, 0.90149E+00_JPRB, 0.89843E+00_JPRB, 0.88813E+00_JPRB, 0.87685E+00_JPRB, 0.86660E+00_JPRB, 0.85556E+00_JPRB ,&
&  0.84730E+00_JPRB, 0.84439E+00_JPRB, 0.84152E+00_JPRB, 0.83617E+00_JPRB, 0.82912E+00_JPRB, 0.82788E+00_JPRB, 0.82690E+00_JPRB ,&
&  0.82586E+00_JPRB, 0.82437E+00_JPRB, 0.82909E+00_JPRB, 0.84301E+00_JPRB, 0.86195E+00_JPRB /)
ZBURD(:, 58, 12) = (/ &
&  0.82367E+00_JPRB, 0.83084E+00_JPRB, 0.84085E+00_JPRB, 0.85185E+00_JPRB, 0.85583E+00_JPRB, 0.85524E+00_JPRB, 0.85202E+00_JPRB ,&
&  0.85177E+00_JPRB, 0.84259E+00_JPRB, 0.83557E+00_JPRB, 0.83090E+00_JPRB, 0.82917E+00_JPRB, 0.82455E+00_JPRB, 0.81396E+00_JPRB ,&
&  0.80506E+00_JPRB, 0.80114E+00_JPRB, 0.80508E+00_JPRB, 0.80618E+00_JPRB, 0.79052E+00_JPRB, 0.78527E+00_JPRB, 0.77704E+00_JPRB ,&
&  0.77560E+00_JPRB, 0.77563E+00_JPRB, 0.77854E+00_JPRB, 0.78469E+00_JPRB, 0.80103E+00_JPRB, 0.81262E+00_JPRB, 0.81880E+00_JPRB ,&
&  0.81686E+00_JPRB, 0.73948E+00_JPRB, 0.76735E+00_JPRB, 0.77921E+00_JPRB, 0.80154E+00_JPRB, 0.82435E+00_JPRB, 0.84509E+00_JPRB ,&
&  0.87220E+00_JPRB, 0.88746E+00_JPRB, 0.89405E+00_JPRB, 0.89547E+00_JPRB, 0.89710E+00_JPRB, 0.89679E+00_JPRB, 0.88971E+00_JPRB ,&
&  0.88545E+00_JPRB, 0.89314E+00_JPRB, 0.91057E+00_JPRB, 0.91972E+00_JPRB, 0.92429E+00_JPRB, 0.92676E+00_JPRB, 0.92362E+00_JPRB ,&
&  0.91651E+00_JPRB, 0.90178E+00_JPRB, 0.87699E+00_JPRB, 0.84708E+00_JPRB, 0.81313E+00_JPRB, 0.78927E+00_JPRB, 0.77574E+00_JPRB ,&
&  0.76696E+00_JPRB, 0.76641E+00_JPRB, 0.76691E+00_JPRB, 0.76831E+00_JPRB, 0.77183E+00_JPRB, 0.78072E+00_JPRB, 0.78464E+00_JPRB ,&
&  0.79251E+00_JPRB, 0.80012E+00_JPRB, 0.81228E+00_JPRB, 0.83210E+00_JPRB, 0.85201E+00_JPRB, 0.86338E+00_JPRB, 0.86430E+00_JPRB ,&
&  0.87019E+00_JPRB, 0.87318E+00_JPRB, 0.86969E+00_JPRB, 0.86972E+00_JPRB, 0.86619E+00_JPRB, 0.86781E+00_JPRB, 0.86542E+00_JPRB ,&
&  0.85695E+00_JPRB, 0.85070E+00_JPRB, 0.84291E+00_JPRB, 0.82776E+00_JPRB, 0.81451E+00_JPRB, 0.80488E+00_JPRB, 0.79685E+00_JPRB ,&
&  0.79585E+00_JPRB, 0.79978E+00_JPRB, 0.80538E+00_JPRB, 0.80056E+00_JPRB, 0.78727E+00_JPRB, 0.77995E+00_JPRB, 0.77065E+00_JPRB ,&
&  0.76478E+00_JPRB, 0.76818E+00_JPRB, 0.77023E+00_JPRB, 0.78101E+00_JPRB, 0.80403E+00_JPRB /)
ZBURD(:, 59, 12) = (/ &
&  0.77139E+00_JPRB, 0.78542E+00_JPRB, 0.80285E+00_JPRB, 0.82029E+00_JPRB, 0.83135E+00_JPRB, 0.83796E+00_JPRB, 0.83552E+00_JPRB ,&
&  0.82389E+00_JPRB, 0.81239E+00_JPRB, 0.81326E+00_JPRB, 0.81242E+00_JPRB, 0.81055E+00_JPRB, 0.80794E+00_JPRB, 0.80054E+00_JPRB ,&
&  0.79031E+00_JPRB, 0.78162E+00_JPRB, 0.77791E+00_JPRB, 0.77465E+00_JPRB, 0.76436E+00_JPRB, 0.75642E+00_JPRB, 0.74788E+00_JPRB ,&
&  0.74743E+00_JPRB, 0.74981E+00_JPRB, 0.75259E+00_JPRB, 0.75273E+00_JPRB, 0.76060E+00_JPRB, 0.77069E+00_JPRB, 0.77497E+00_JPRB ,&
&  0.76921E+00_JPRB, 0.77680E+00_JPRB, 0.69371E+00_JPRB, 0.76325E+00_JPRB, 0.78804E+00_JPRB, 0.80281E+00_JPRB, 0.81847E+00_JPRB ,&
&  0.82917E+00_JPRB, 0.84093E+00_JPRB, 0.84943E+00_JPRB, 0.85449E+00_JPRB, 0.87218E+00_JPRB, 0.88634E+00_JPRB, 0.88964E+00_JPRB ,&
&  0.89113E+00_JPRB, 0.89010E+00_JPRB, 0.89153E+00_JPRB, 0.89221E+00_JPRB, 0.89723E+00_JPRB, 0.89773E+00_JPRB, 0.89213E+00_JPRB ,&
&  0.88428E+00_JPRB, 0.87249E+00_JPRB, 0.85477E+00_JPRB, 0.83404E+00_JPRB, 0.80140E+00_JPRB, 0.77288E+00_JPRB, 0.75465E+00_JPRB ,&
&  0.73972E+00_JPRB, 0.73513E+00_JPRB, 0.72953E+00_JPRB, 0.72386E+00_JPRB, 0.72176E+00_JPRB, 0.72402E+00_JPRB, 0.72301E+00_JPRB ,&
&  0.72962E+00_JPRB, 0.73842E+00_JPRB, 0.74273E+00_JPRB, 0.75028E+00_JPRB, 0.76008E+00_JPRB, 0.77435E+00_JPRB, 0.78784E+00_JPRB ,&
&  0.80053E+00_JPRB, 0.80926E+00_JPRB, 0.81079E+00_JPRB, 0.81107E+00_JPRB, 0.80906E+00_JPRB, 0.80825E+00_JPRB, 0.80140E+00_JPRB ,&
&  0.79591E+00_JPRB, 0.79302E+00_JPRB, 0.78671E+00_JPRB, 0.77476E+00_JPRB, 0.76737E+00_JPRB, 0.76108E+00_JPRB, 0.75946E+00_JPRB ,&
&  0.76026E+00_JPRB, 0.76162E+00_JPRB, 0.75928E+00_JPRB, 0.76019E+00_JPRB, 0.75433E+00_JPRB, 0.73486E+00_JPRB, 0.71995E+00_JPRB ,&
&  0.72012E+00_JPRB, 0.72607E+00_JPRB, 0.73142E+00_JPRB, 0.74113E+00_JPRB, 0.75760E+00_JPRB /)
ZBURD(:, 60, 12) = (/ &
&  0.78511E+00_JPRB, 0.79213E+00_JPRB, 0.80257E+00_JPRB, 0.81744E+00_JPRB, 0.83260E+00_JPRB, 0.84230E+00_JPRB, 0.84192E+00_JPRB ,&
&  0.83058E+00_JPRB, 0.81676E+00_JPRB, 0.81184E+00_JPRB, 0.81064E+00_JPRB, 0.80719E+00_JPRB, 0.80548E+00_JPRB, 0.80059E+00_JPRB ,&
&  0.78915E+00_JPRB, 0.77199E+00_JPRB, 0.76374E+00_JPRB, 0.74966E+00_JPRB, 0.73619E+00_JPRB, 0.72513E+00_JPRB, 0.72491E+00_JPRB ,&
&  0.73341E+00_JPRB, 0.74336E+00_JPRB, 0.75564E+00_JPRB, 0.76956E+00_JPRB, 0.77691E+00_JPRB, 0.77522E+00_JPRB, 0.77174E+00_JPRB ,&
&  0.76885E+00_JPRB, 0.76595E+00_JPRB, 0.77655E+00_JPRB, 0.78533E+00_JPRB, 0.79116E+00_JPRB, 0.80220E+00_JPRB, 0.81145E+00_JPRB ,&
&  0.81934E+00_JPRB, 0.83078E+00_JPRB, 0.84955E+00_JPRB, 0.86349E+00_JPRB, 0.87374E+00_JPRB, 0.88457E+00_JPRB, 0.88793E+00_JPRB ,&
&  0.89390E+00_JPRB, 0.90509E+00_JPRB, 0.90739E+00_JPRB, 0.89848E+00_JPRB, 0.88431E+00_JPRB, 0.87543E+00_JPRB, 0.87062E+00_JPRB ,&
&  0.86166E+00_JPRB, 0.84242E+00_JPRB, 0.82778E+00_JPRB, 0.81593E+00_JPRB, 0.79667E+00_JPRB, 0.77603E+00_JPRB, 0.75202E+00_JPRB ,&
&  0.72470E+00_JPRB, 0.70430E+00_JPRB, 0.68833E+00_JPRB, 0.67392E+00_JPRB, 0.67027E+00_JPRB, 0.66153E+00_JPRB, 0.65493E+00_JPRB ,&
&  0.65949E+00_JPRB, 0.67787E+00_JPRB, 0.69165E+00_JPRB, 0.69646E+00_JPRB, 0.69905E+00_JPRB, 0.71497E+00_JPRB, 0.73434E+00_JPRB ,&
&  0.74953E+00_JPRB, 0.75600E+00_JPRB, 0.75222E+00_JPRB, 0.75473E+00_JPRB, 0.75999E+00_JPRB, 0.76441E+00_JPRB, 0.75858E+00_JPRB ,&
&  0.75016E+00_JPRB, 0.74467E+00_JPRB, 0.74349E+00_JPRB, 0.73983E+00_JPRB, 0.74123E+00_JPRB, 0.74102E+00_JPRB, 0.73819E+00_JPRB ,&
&  0.73247E+00_JPRB, 0.72321E+00_JPRB, 0.71137E+00_JPRB, 0.70435E+00_JPRB, 0.69402E+00_JPRB, 0.67851E+00_JPRB, 0.67999E+00_JPRB ,&
&  0.69209E+00_JPRB, 0.71091E+00_JPRB, 0.72846E+00_JPRB, 0.74912E+00_JPRB, 0.77338E+00_JPRB /)
ZBURD(:, 61, 12) = (/ &
&  0.80890E+00_JPRB, 0.81744E+00_JPRB, 0.82391E+00_JPRB, 0.82888E+00_JPRB, 0.83833E+00_JPRB, 0.84989E+00_JPRB, 0.85005E+00_JPRB ,&
&  0.83532E+00_JPRB, 0.82139E+00_JPRB, 0.81150E+00_JPRB, 0.80847E+00_JPRB, 0.80993E+00_JPRB, 0.81202E+00_JPRB, 0.80603E+00_JPRB ,&
&  0.78480E+00_JPRB, 0.76201E+00_JPRB, 0.74786E+00_JPRB, 0.73382E+00_JPRB, 0.72497E+00_JPRB, 0.71924E+00_JPRB, 0.72580E+00_JPRB ,&
&  0.73570E+00_JPRB, 0.75152E+00_JPRB, 0.76438E+00_JPRB, 0.77360E+00_JPRB, 0.77717E+00_JPRB, 0.76545E+00_JPRB, 0.76647E+00_JPRB ,&
&  0.77326E+00_JPRB, 0.77520E+00_JPRB, 0.79000E+00_JPRB, 0.78926E+00_JPRB, 0.77582E+00_JPRB, 0.76495E+00_JPRB, 0.75428E+00_JPRB ,&
&  0.75148E+00_JPRB, 0.76274E+00_JPRB, 0.78809E+00_JPRB, 0.82471E+00_JPRB, 0.84603E+00_JPRB, 0.85418E+00_JPRB, 0.85094E+00_JPRB ,&
&  0.85344E+00_JPRB, 0.86528E+00_JPRB, 0.87201E+00_JPRB, 0.86380E+00_JPRB, 0.84564E+00_JPRB, 0.83090E+00_JPRB, 0.81795E+00_JPRB ,&
&  0.80849E+00_JPRB, 0.79700E+00_JPRB, 0.78913E+00_JPRB, 0.77916E+00_JPRB, 0.75715E+00_JPRB, 0.73102E+00_JPRB, 0.70823E+00_JPRB ,&
&  0.69127E+00_JPRB, 0.67462E+00_JPRB, 0.65433E+00_JPRB, 0.64787E+00_JPRB, 0.65176E+00_JPRB, 0.62875E+00_JPRB, 0.60391E+00_JPRB ,&
&  0.60742E+00_JPRB, 0.63868E+00_JPRB, 0.66318E+00_JPRB, 0.67311E+00_JPRB, 0.67400E+00_JPRB, 0.67839E+00_JPRB, 0.68192E+00_JPRB ,&
&  0.68861E+00_JPRB, 0.69744E+00_JPRB, 0.70423E+00_JPRB, 0.71550E+00_JPRB, 0.72717E+00_JPRB, 0.72572E+00_JPRB, 0.71167E+00_JPRB ,&
&  0.69627E+00_JPRB, 0.68747E+00_JPRB, 0.68374E+00_JPRB, 0.68590E+00_JPRB, 0.68919E+00_JPRB, 0.68709E+00_JPRB, 0.68130E+00_JPRB ,&
&  0.66970E+00_JPRB, 0.65136E+00_JPRB, 0.63578E+00_JPRB, 0.62708E+00_JPRB, 0.62668E+00_JPRB, 0.63347E+00_JPRB, 0.65014E+00_JPRB ,&
&  0.68195E+00_JPRB, 0.73006E+00_JPRB, 0.76561E+00_JPRB, 0.78566E+00_JPRB, 0.79122E+00_JPRB /)
ZBURD(:, 62, 12) = (/ &
&  0.83612E+00_JPRB, 0.84467E+00_JPRB, 0.85483E+00_JPRB, 0.86848E+00_JPRB, 0.88099E+00_JPRB, 0.87842E+00_JPRB, 0.86603E+00_JPRB ,&
&  0.84230E+00_JPRB, 0.83322E+00_JPRB, 0.83769E+00_JPRB, 0.83996E+00_JPRB, 0.84162E+00_JPRB, 0.83933E+00_JPRB, 0.82316E+00_JPRB ,&
&  0.78558E+00_JPRB, 0.75529E+00_JPRB, 0.74059E+00_JPRB, 0.73281E+00_JPRB, 0.73009E+00_JPRB, 0.73678E+00_JPRB, 0.74485E+00_JPRB ,&
&  0.76140E+00_JPRB, 0.77123E+00_JPRB, 0.78313E+00_JPRB, 0.79234E+00_JPRB, 0.79475E+00_JPRB, 0.79833E+00_JPRB, 0.79910E+00_JPRB ,&
&  0.78471E+00_JPRB, 0.78466E+00_JPRB, 0.78588E+00_JPRB, 0.77598E+00_JPRB, 0.75428E+00_JPRB, 0.73245E+00_JPRB, 0.70574E+00_JPRB ,&
&  0.68796E+00_JPRB, 0.67901E+00_JPRB, 0.70440E+00_JPRB, 0.76611E+00_JPRB, 0.80962E+00_JPRB, 0.81769E+00_JPRB, 0.80958E+00_JPRB ,&
&  0.81131E+00_JPRB, 0.81865E+00_JPRB, 0.82220E+00_JPRB, 0.81080E+00_JPRB, 0.79117E+00_JPRB, 0.77397E+00_JPRB, 0.75760E+00_JPRB ,&
&  0.75222E+00_JPRB, 0.75165E+00_JPRB, 0.73447E+00_JPRB, 0.70236E+00_JPRB, 0.67467E+00_JPRB, 0.65503E+00_JPRB, 0.64465E+00_JPRB ,&
&  0.63831E+00_JPRB, 0.62862E+00_JPRB, 0.62257E+00_JPRB, 0.62731E+00_JPRB, 0.60734E+00_JPRB, 0.58942E+00_JPRB, 0.59325E+00_JPRB ,&
&  0.60967E+00_JPRB, 0.61519E+00_JPRB, 0.62444E+00_JPRB, 0.62758E+00_JPRB, 0.62458E+00_JPRB, 0.62088E+00_JPRB, 0.62216E+00_JPRB ,&
&  0.63095E+00_JPRB, 0.64098E+00_JPRB, 0.64838E+00_JPRB, 0.65343E+00_JPRB, 0.65860E+00_JPRB, 0.64699E+00_JPRB, 0.62596E+00_JPRB ,&
&  0.60847E+00_JPRB, 0.60527E+00_JPRB, 0.60987E+00_JPRB, 0.60927E+00_JPRB, 0.60776E+00_JPRB, 0.60794E+00_JPRB, 0.60669E+00_JPRB ,&
&  0.60397E+00_JPRB, 0.59667E+00_JPRB, 0.58791E+00_JPRB, 0.58812E+00_JPRB, 0.59957E+00_JPRB, 0.62249E+00_JPRB, 0.65794E+00_JPRB ,&
&  0.71048E+00_JPRB, 0.75015E+00_JPRB, 0.78839E+00_JPRB, 0.80423E+00_JPRB, 0.80818E+00_JPRB /)
ZBURD(:, 63, 12) = (/ &
&  0.87104E+00_JPRB, 0.89312E+00_JPRB, 0.91215E+00_JPRB, 0.92519E+00_JPRB, 0.92728E+00_JPRB, 0.92069E+00_JPRB, 0.90466E+00_JPRB ,&
&  0.87454E+00_JPRB, 0.85737E+00_JPRB, 0.86625E+00_JPRB, 0.88084E+00_JPRB, 0.88973E+00_JPRB, 0.88972E+00_JPRB, 0.85805E+00_JPRB ,&
&  0.80333E+00_JPRB, 0.78234E+00_JPRB, 0.77901E+00_JPRB, 0.78840E+00_JPRB, 0.80640E+00_JPRB, 0.81312E+00_JPRB, 0.80072E+00_JPRB ,&
&  0.78807E+00_JPRB, 0.78931E+00_JPRB, 0.79173E+00_JPRB, 0.79822E+00_JPRB, 0.78727E+00_JPRB, 0.76601E+00_JPRB, 0.75676E+00_JPRB ,&
&  0.75381E+00_JPRB, 0.74589E+00_JPRB, 0.73759E+00_JPRB, 0.68848E+00_JPRB, 0.71174E+00_JPRB, 0.69975E+00_JPRB, 0.68105E+00_JPRB ,&
&  0.66381E+00_JPRB, 0.66609E+00_JPRB, 0.70771E+00_JPRB, 0.75420E+00_JPRB, 0.76569E+00_JPRB, 0.76828E+00_JPRB, 0.76308E+00_JPRB ,&
&  0.75940E+00_JPRB, 0.75978E+00_JPRB, 0.76119E+00_JPRB, 0.75307E+00_JPRB, 0.73275E+00_JPRB, 0.70653E+00_JPRB, 0.67783E+00_JPRB ,&
&  0.66058E+00_JPRB, 0.63589E+00_JPRB, 0.59260E+00_JPRB, 0.57248E+00_JPRB, 0.57068E+00_JPRB, 0.57869E+00_JPRB, 0.58251E+00_JPRB ,&
&  0.57941E+00_JPRB, 0.57189E+00_JPRB, 0.57554E+00_JPRB, 0.57243E+00_JPRB, 0.55060E+00_JPRB, 0.53733E+00_JPRB, 0.53125E+00_JPRB ,&
&  0.51981E+00_JPRB, 0.51884E+00_JPRB, 0.52380E+00_JPRB, 0.53563E+00_JPRB, 0.54601E+00_JPRB, 0.56004E+00_JPRB, 0.56599E+00_JPRB ,&
&  0.56835E+00_JPRB, 0.56483E+00_JPRB, 0.55898E+00_JPRB, 0.54712E+00_JPRB, 0.52879E+00_JPRB, 0.52184E+00_JPRB, 0.52347E+00_JPRB ,&
&  0.53255E+00_JPRB, 0.53874E+00_JPRB, 0.53838E+00_JPRB, 0.53744E+00_JPRB, 0.52603E+00_JPRB, 0.51351E+00_JPRB, 0.50400E+00_JPRB ,&
&  0.49959E+00_JPRB, 0.50256E+00_JPRB, 0.51810E+00_JPRB, 0.54338E+00_JPRB, 0.57570E+00_JPRB, 0.61470E+00_JPRB, 0.65974E+00_JPRB ,&
&  0.69169E+00_JPRB, 0.71054E+00_JPRB, 0.73522E+00_JPRB, 0.78327E+00_JPRB, 0.82979E+00_JPRB /)
ZBURD(:, 64, 12) = (/ &
&  0.87289E+00_JPRB, 0.89996E+00_JPRB, 0.92958E+00_JPRB, 0.94667E+00_JPRB, 0.94767E+00_JPRB, 0.94454E+00_JPRB, 0.92950E+00_JPRB ,&
&  0.89855E+00_JPRB, 0.87310E+00_JPRB, 0.86880E+00_JPRB, 0.87259E+00_JPRB, 0.88429E+00_JPRB, 0.90288E+00_JPRB, 0.90420E+00_JPRB ,&
&  0.87732E+00_JPRB, 0.85771E+00_JPRB, 0.84797E+00_JPRB, 0.84090E+00_JPRB, 0.82390E+00_JPRB, 0.79911E+00_JPRB, 0.78273E+00_JPRB ,&
&  0.77779E+00_JPRB, 0.77722E+00_JPRB, 0.76785E+00_JPRB, 0.75364E+00_JPRB, 0.72189E+00_JPRB, 0.68284E+00_JPRB, 0.65214E+00_JPRB ,&
&  0.64129E+00_JPRB, 0.63169E+00_JPRB, 0.59094E+00_JPRB, 0.60660E+00_JPRB, 0.61024E+00_JPRB, 0.61025E+00_JPRB, 0.60737E+00_JPRB ,&
&  0.61920E+00_JPRB, 0.65238E+00_JPRB, 0.68538E+00_JPRB, 0.69509E+00_JPRB, 0.70437E+00_JPRB, 0.70596E+00_JPRB, 0.70731E+00_JPRB ,&
&  0.71178E+00_JPRB, 0.70377E+00_JPRB, 0.67418E+00_JPRB, 0.63029E+00_JPRB, 0.58700E+00_JPRB, 0.55335E+00_JPRB, 0.52472E+00_JPRB ,&
&  0.50514E+00_JPRB, 0.49294E+00_JPRB, 0.48338E+00_JPRB, 0.48012E+00_JPRB, 0.48785E+00_JPRB, 0.50292E+00_JPRB, 0.49779E+00_JPRB ,&
&  0.47940E+00_JPRB, 0.46245E+00_JPRB, 0.44018E+00_JPRB, 0.40999E+00_JPRB, 0.37898E+00_JPRB, 0.34066E+00_JPRB, 0.32095E+00_JPRB ,&
&  0.32560E+00_JPRB, 0.36487E+00_JPRB, 0.41643E+00_JPRB, 0.43618E+00_JPRB, 0.46367E+00_JPRB, 0.49048E+00_JPRB, 0.48001E+00_JPRB ,&
&  0.43225E+00_JPRB, 0.36415E+00_JPRB, 0.34145E+00_JPRB, 0.32884E+00_JPRB, 0.32205E+00_JPRB, 0.31738E+00_JPRB, 0.31650E+00_JPRB ,&
&  0.32093E+00_JPRB, 0.33228E+00_JPRB, 0.34200E+00_JPRB, 0.33915E+00_JPRB, 0.32908E+00_JPRB, 0.32325E+00_JPRB, 0.32427E+00_JPRB ,&
&  0.33418E+00_JPRB, 0.36282E+00_JPRB, 0.42638E+00_JPRB, 0.54152E+00_JPRB, 0.57831E+00_JPRB, 0.61186E+00_JPRB, 0.65047E+00_JPRB ,&
&  0.68755E+00_JPRB, 0.72137E+00_JPRB, 0.75026E+00_JPRB, 0.78570E+00_JPRB, 0.83309E+00_JPRB /)
ZBURD(:, 65, 12) = (/ &
&  0.81003E+00_JPRB, 0.82881E+00_JPRB, 0.85051E+00_JPRB, 0.86209E+00_JPRB, 0.86340E+00_JPRB, 0.86065E+00_JPRB, 0.85211E+00_JPRB ,&
&  0.83729E+00_JPRB, 0.81936E+00_JPRB, 0.80611E+00_JPRB, 0.79709E+00_JPRB, 0.79301E+00_JPRB, 0.79575E+00_JPRB, 0.80086E+00_JPRB ,&
&  0.79982E+00_JPRB, 0.79556E+00_JPRB, 0.78335E+00_JPRB, 0.76748E+00_JPRB, 0.75066E+00_JPRB, 0.73745E+00_JPRB, 0.72709E+00_JPRB ,&
&  0.71687E+00_JPRB, 0.70181E+00_JPRB, 0.67861E+00_JPRB, 0.64301E+00_JPRB, 0.60102E+00_JPRB, 0.56274E+00_JPRB, 0.53299E+00_JPRB ,&
&  0.50684E+00_JPRB, 0.44003E+00_JPRB, 0.40305E+00_JPRB, 0.41411E+00_JPRB, 0.50490E+00_JPRB, 0.53633E+00_JPRB, 0.56054E+00_JPRB ,&
&  0.59707E+00_JPRB, 0.63601E+00_JPRB, 0.65465E+00_JPRB, 0.65497E+00_JPRB, 0.64932E+00_JPRB, 0.63658E+00_JPRB, 0.61682E+00_JPRB ,&
&  0.59178E+00_JPRB, 0.56124E+00_JPRB, 0.53524E+00_JPRB, 0.50610E+00_JPRB, 0.47619E+00_JPRB, 0.45420E+00_JPRB, 0.43892E+00_JPRB ,&
&  0.42837E+00_JPRB, 0.41873E+00_JPRB, 0.41102E+00_JPRB, 0.40122E+00_JPRB, 0.39204E+00_JPRB, 0.37090E+00_JPRB, 0.34859E+00_JPRB ,&
&  0.32565E+00_JPRB, 0.30792E+00_JPRB, 0.28930E+00_JPRB, 0.27105E+00_JPRB, 0.25610E+00_JPRB, 0.24634E+00_JPRB, 0.23911E+00_JPRB ,&
&  0.23698E+00_JPRB, 0.24329E+00_JPRB, 0.26250E+00_JPRB, 0.29545E+00_JPRB, 0.33242E+00_JPRB, 0.30479E+00_JPRB, 0.28470E+00_JPRB ,&
&  0.26367E+00_JPRB, 0.25140E+00_JPRB, 0.24686E+00_JPRB, 0.24636E+00_JPRB, 0.24766E+00_JPRB, 0.24998E+00_JPRB, 0.25242E+00_JPRB ,&
&  0.25586E+00_JPRB, 0.25910E+00_JPRB, 0.26250E+00_JPRB, 0.26508E+00_JPRB, 0.26681E+00_JPRB, 0.26973E+00_JPRB, 0.27460E+00_JPRB ,&
&  0.28218E+00_JPRB, 0.29457E+00_JPRB, 0.31465E+00_JPRB, 0.34067E+00_JPRB, 0.36780E+00_JPRB, 0.39249E+00_JPRB, 0.44572E+00_JPRB ,&
&  0.53331E+00_JPRB, 0.64523E+00_JPRB, 0.70303E+00_JPRB, 0.75977E+00_JPRB, 0.78804E+00_JPRB /)
ZBURD(:, 66, 12) = (/ &
&  0.74561E+00_JPRB, 0.75815E+00_JPRB, 0.76862E+00_JPRB, 0.77097E+00_JPRB, 0.77047E+00_JPRB, 0.76560E+00_JPRB, 0.75365E+00_JPRB ,&
&  0.73405E+00_JPRB, 0.70781E+00_JPRB, 0.68225E+00_JPRB, 0.66190E+00_JPRB, 0.64911E+00_JPRB, 0.65201E+00_JPRB, 0.66051E+00_JPRB ,&
&  0.67782E+00_JPRB, 0.70325E+00_JPRB, 0.70841E+00_JPRB, 0.70167E+00_JPRB, 0.66575E+00_JPRB, 0.62822E+00_JPRB, 0.59334E+00_JPRB ,&
&  0.56236E+00_JPRB, 0.53304E+00_JPRB, 0.52189E+00_JPRB, 0.50279E+00_JPRB, 0.48938E+00_JPRB, 0.47259E+00_JPRB, 0.44871E+00_JPRB ,&
&  0.40963E+00_JPRB, 0.37912E+00_JPRB, 0.36239E+00_JPRB, 0.37442E+00_JPRB, 0.46186E+00_JPRB, 0.52261E+00_JPRB, 0.53270E+00_JPRB ,&
&  0.56688E+00_JPRB, 0.59115E+00_JPRB, 0.58608E+00_JPRB, 0.56601E+00_JPRB, 0.54436E+00_JPRB, 0.52089E+00_JPRB, 0.50140E+00_JPRB ,&
&  0.48204E+00_JPRB, 0.45566E+00_JPRB, 0.40603E+00_JPRB, 0.33568E+00_JPRB, 0.30031E+00_JPRB, 0.28695E+00_JPRB, 0.27254E+00_JPRB ,&
&  0.26170E+00_JPRB, 0.25317E+00_JPRB, 0.24979E+00_JPRB, 0.24623E+00_JPRB, 0.24554E+00_JPRB, 0.24162E+00_JPRB, 0.23814E+00_JPRB ,&
&  0.23354E+00_JPRB, 0.23017E+00_JPRB, 0.22538E+00_JPRB, 0.22065E+00_JPRB, 0.21706E+00_JPRB, 0.21646E+00_JPRB, 0.21626E+00_JPRB ,&
&  0.21735E+00_JPRB, 0.22148E+00_JPRB, 0.23332E+00_JPRB, 0.23985E+00_JPRB, 0.24036E+00_JPRB, 0.23604E+00_JPRB, 0.22889E+00_JPRB ,&
&  0.22215E+00_JPRB, 0.22038E+00_JPRB, 0.21904E+00_JPRB, 0.21968E+00_JPRB, 0.22258E+00_JPRB, 0.22634E+00_JPRB, 0.22939E+00_JPRB ,&
&  0.23183E+00_JPRB, 0.23438E+00_JPRB, 0.23679E+00_JPRB, 0.23940E+00_JPRB, 0.24258E+00_JPRB, 0.24631E+00_JPRB, 0.25135E+00_JPRB ,&
&  0.25665E+00_JPRB, 0.26405E+00_JPRB, 0.27447E+00_JPRB, 0.28951E+00_JPRB, 0.30296E+00_JPRB, 0.31169E+00_JPRB, 0.32361E+00_JPRB ,&
&  0.33991E+00_JPRB, 0.38754E+00_JPRB, 0.52242E+00_JPRB, 0.74761E+00_JPRB, 0.77989E+00_JPRB /)
ZBURD(:, 67, 12) = (/ &
&  0.66735E+00_JPRB, 0.67924E+00_JPRB, 0.69373E+00_JPRB, 0.69713E+00_JPRB, 0.70747E+00_JPRB, 0.70645E+00_JPRB, 0.68926E+00_JPRB ,&
&  0.66115E+00_JPRB, 0.63462E+00_JPRB, 0.62016E+00_JPRB, 0.60207E+00_JPRB, 0.57417E+00_JPRB, 0.51397E+00_JPRB, 0.49098E+00_JPRB ,&
&  0.47236E+00_JPRB, 0.47196E+00_JPRB, 0.46263E+00_JPRB, 0.46270E+00_JPRB, 0.45146E+00_JPRB, 0.43990E+00_JPRB, 0.42098E+00_JPRB ,&
&  0.40353E+00_JPRB, 0.38635E+00_JPRB, 0.37441E+00_JPRB, 0.36872E+00_JPRB, 0.36643E+00_JPRB, 0.36165E+00_JPRB, 0.34920E+00_JPRB ,&
&  0.34031E+00_JPRB, 0.33838E+00_JPRB, 0.34073E+00_JPRB, 0.34961E+00_JPRB, 0.39243E+00_JPRB, 0.44874E+00_JPRB, 0.44935E+00_JPRB ,&
&  0.47169E+00_JPRB, 0.48487E+00_JPRB, 0.49775E+00_JPRB, 0.48664E+00_JPRB, 0.45383E+00_JPRB, 0.41460E+00_JPRB, 0.36084E+00_JPRB ,&
&  0.30976E+00_JPRB, 0.28082E+00_JPRB, 0.25735E+00_JPRB, 0.24089E+00_JPRB, 0.22961E+00_JPRB, 0.22506E+00_JPRB, 0.21923E+00_JPRB ,&
&  0.21544E+00_JPRB, 0.21253E+00_JPRB, 0.21136E+00_JPRB, 0.21078E+00_JPRB, 0.21004E+00_JPRB, 0.20792E+00_JPRB, 0.20734E+00_JPRB ,&
&  0.20788E+00_JPRB, 0.20803E+00_JPRB, 0.20781E+00_JPRB, 0.20865E+00_JPRB, 0.20964E+00_JPRB, 0.21109E+00_JPRB, 0.21244E+00_JPRB ,&
&  0.21447E+00_JPRB, 0.21562E+00_JPRB, 0.21743E+00_JPRB, 0.21942E+00_JPRB, 0.21849E+00_JPRB, 0.21738E+00_JPRB, 0.21465E+00_JPRB ,&
&  0.21156E+00_JPRB, 0.21132E+00_JPRB, 0.21087E+00_JPRB, 0.21153E+00_JPRB, 0.21324E+00_JPRB, 0.21551E+00_JPRB, 0.21847E+00_JPRB ,&
&  0.22130E+00_JPRB, 0.22425E+00_JPRB, 0.22655E+00_JPRB, 0.22884E+00_JPRB, 0.23262E+00_JPRB, 0.23664E+00_JPRB, 0.24172E+00_JPRB ,&
&  0.24699E+00_JPRB, 0.25296E+00_JPRB, 0.26114E+00_JPRB, 0.27168E+00_JPRB, 0.28686E+00_JPRB, 0.30392E+00_JPRB, 0.33033E+00_JPRB ,&
&  0.35455E+00_JPRB, 0.45400E+00_JPRB, 0.69002E+00_JPRB, 0.61922E+00_JPRB, 0.63900E+00_JPRB /)
ZBURD(:, 68, 12) = (/ &
&  0.62958E+00_JPRB, 0.64654E+00_JPRB, 0.67169E+00_JPRB, 0.66468E+00_JPRB, 0.68498E+00_JPRB, 0.67166E+00_JPRB, 0.64881E+00_JPRB ,&
&  0.60155E+00_JPRB, 0.54494E+00_JPRB, 0.49834E+00_JPRB, 0.44627E+00_JPRB, 0.40561E+00_JPRB, 0.37602E+00_JPRB, 0.36148E+00_JPRB ,&
&  0.35418E+00_JPRB, 0.35186E+00_JPRB, 0.35026E+00_JPRB, 0.34925E+00_JPRB, 0.34425E+00_JPRB, 0.34071E+00_JPRB, 0.33478E+00_JPRB ,&
&  0.32879E+00_JPRB, 0.32451E+00_JPRB, 0.32120E+00_JPRB, 0.32610E+00_JPRB, 0.33217E+00_JPRB, 0.33848E+00_JPRB, 0.34171E+00_JPRB ,&
&  0.34441E+00_JPRB, 0.34569E+00_JPRB, 0.34836E+00_JPRB, 0.34631E+00_JPRB, 0.35742E+00_JPRB, 0.37849E+00_JPRB, 0.39632E+00_JPRB ,&
&  0.41366E+00_JPRB, 0.42801E+00_JPRB, 0.42377E+00_JPRB, 0.39981E+00_JPRB, 0.33391E+00_JPRB, 0.29975E+00_JPRB, 0.26965E+00_JPRB ,&
&  0.25062E+00_JPRB, 0.23652E+00_JPRB, 0.22705E+00_JPRB, 0.22165E+00_JPRB, 0.21731E+00_JPRB, 0.21495E+00_JPRB, 0.21243E+00_JPRB ,&
&  0.21002E+00_JPRB, 0.20857E+00_JPRB, 0.20691E+00_JPRB, 0.20608E+00_JPRB, 0.20509E+00_JPRB, 0.20378E+00_JPRB, 0.20284E+00_JPRB ,&
&  0.20305E+00_JPRB, 0.20288E+00_JPRB, 0.20404E+00_JPRB, 0.20504E+00_JPRB, 0.20699E+00_JPRB, 0.20861E+00_JPRB, 0.21089E+00_JPRB ,&
&  0.21278E+00_JPRB, 0.21438E+00_JPRB, 0.21426E+00_JPRB, 0.21383E+00_JPRB, 0.21297E+00_JPRB, 0.21252E+00_JPRB, 0.21169E+00_JPRB ,&
&  0.21059E+00_JPRB, 0.21082E+00_JPRB, 0.21116E+00_JPRB, 0.21157E+00_JPRB, 0.21278E+00_JPRB, 0.21446E+00_JPRB, 0.21696E+00_JPRB ,&
&  0.22025E+00_JPRB, 0.22368E+00_JPRB, 0.22659E+00_JPRB, 0.22970E+00_JPRB, 0.23366E+00_JPRB, 0.23816E+00_JPRB, 0.24372E+00_JPRB ,&
&  0.24859E+00_JPRB, 0.25697E+00_JPRB, 0.26859E+00_JPRB, 0.28069E+00_JPRB, 0.29929E+00_JPRB, 0.31707E+00_JPRB, 0.34678E+00_JPRB ,&
&  0.39540E+00_JPRB, 0.56620E+00_JPRB, 0.84954E+00_JPRB, 0.62521E+00_JPRB, 0.61466E+00_JPRB /)
ZBURD(:, 69, 12) = (/ &
&  0.54979E+00_JPRB, 0.55917E+00_JPRB, 0.57353E+00_JPRB, 0.58521E+00_JPRB, 0.60943E+00_JPRB, 0.62310E+00_JPRB, 0.66887E+00_JPRB ,&
&  0.62318E+00_JPRB, 0.54440E+00_JPRB, 0.51239E+00_JPRB, 0.47753E+00_JPRB, 0.43783E+00_JPRB, 0.40919E+00_JPRB, 0.38902E+00_JPRB ,&
&  0.37687E+00_JPRB, 0.36648E+00_JPRB, 0.35643E+00_JPRB, 0.34394E+00_JPRB, 0.33113E+00_JPRB, 0.31691E+00_JPRB, 0.30627E+00_JPRB ,&
&  0.29992E+00_JPRB, 0.29824E+00_JPRB, 0.29776E+00_JPRB, 0.30188E+00_JPRB, 0.30753E+00_JPRB, 0.31498E+00_JPRB, 0.32365E+00_JPRB ,&
&  0.34366E+00_JPRB, 0.35024E+00_JPRB, 0.35106E+00_JPRB, 0.35092E+00_JPRB, 0.35189E+00_JPRB, 0.35428E+00_JPRB, 0.34478E+00_JPRB ,&
&  0.35286E+00_JPRB, 0.37365E+00_JPRB, 0.37939E+00_JPRB, 0.36012E+00_JPRB, 0.30554E+00_JPRB, 0.27295E+00_JPRB, 0.25030E+00_JPRB ,&
&  0.24192E+00_JPRB, 0.23207E+00_JPRB, 0.22593E+00_JPRB, 0.22005E+00_JPRB, 0.21615E+00_JPRB, 0.21428E+00_JPRB, 0.21242E+00_JPRB ,&
&  0.21111E+00_JPRB, 0.21021E+00_JPRB, 0.20824E+00_JPRB, 0.20737E+00_JPRB, 0.20625E+00_JPRB, 0.20563E+00_JPRB, 0.20530E+00_JPRB ,&
&  0.20513E+00_JPRB, 0.20480E+00_JPRB, 0.20500E+00_JPRB, 0.20551E+00_JPRB, 0.20643E+00_JPRB, 0.20716E+00_JPRB, 0.20783E+00_JPRB ,&
&  0.20871E+00_JPRB, 0.20952E+00_JPRB, 0.20973E+00_JPRB, 0.21024E+00_JPRB, 0.21068E+00_JPRB, 0.21187E+00_JPRB, 0.21243E+00_JPRB ,&
&  0.21302E+00_JPRB, 0.21353E+00_JPRB, 0.21356E+00_JPRB, 0.21420E+00_JPRB, 0.21517E+00_JPRB, 0.21707E+00_JPRB, 0.21925E+00_JPRB ,&
&  0.22204E+00_JPRB, 0.22529E+00_JPRB, 0.22887E+00_JPRB, 0.23245E+00_JPRB, 0.23708E+00_JPRB, 0.24215E+00_JPRB, 0.24875E+00_JPRB ,&
&  0.25593E+00_JPRB, 0.26506E+00_JPRB, 0.27748E+00_JPRB, 0.29254E+00_JPRB, 0.31099E+00_JPRB, 0.32932E+00_JPRB, 0.34983E+00_JPRB ,&
&  0.38461E+00_JPRB, 0.48372E+00_JPRB, 0.54381E+00_JPRB, 0.53242E+00_JPRB, 0.54340E+00_JPRB /)
ZBURD(:, 70, 12) = (/ &
&  0.53274E+00_JPRB, 0.53530E+00_JPRB, 0.53805E+00_JPRB, 0.54227E+00_JPRB, 0.54679E+00_JPRB, 0.55122E+00_JPRB, 0.55502E+00_JPRB ,&
&  0.53675E+00_JPRB, 0.51365E+00_JPRB, 0.49466E+00_JPRB, 0.47204E+00_JPRB, 0.45140E+00_JPRB, 0.43428E+00_JPRB, 0.41411E+00_JPRB ,&
&  0.39398E+00_JPRB, 0.37386E+00_JPRB, 0.35117E+00_JPRB, 0.33254E+00_JPRB, 0.31420E+00_JPRB, 0.29937E+00_JPRB, 0.29036E+00_JPRB ,&
&  0.28541E+00_JPRB, 0.28403E+00_JPRB, 0.28545E+00_JPRB, 0.28835E+00_JPRB, 0.29110E+00_JPRB, 0.29569E+00_JPRB, 0.30211E+00_JPRB ,&
&  0.31099E+00_JPRB, 0.32562E+00_JPRB, 0.33880E+00_JPRB, 0.34352E+00_JPRB, 0.33988E+00_JPRB, 0.33133E+00_JPRB, 0.32398E+00_JPRB ,&
&  0.31512E+00_JPRB, 0.30295E+00_JPRB, 0.28810E+00_JPRB, 0.27339E+00_JPRB, 0.25968E+00_JPRB, 0.24970E+00_JPRB, 0.24136E+00_JPRB ,&
&  0.23431E+00_JPRB, 0.22808E+00_JPRB, 0.22330E+00_JPRB, 0.22008E+00_JPRB, 0.21732E+00_JPRB, 0.21511E+00_JPRB, 0.21300E+00_JPRB ,&
&  0.21197E+00_JPRB, 0.21090E+00_JPRB, 0.21026E+00_JPRB, 0.20990E+00_JPRB, 0.20941E+00_JPRB, 0.20924E+00_JPRB, 0.20912E+00_JPRB ,&
&  0.20876E+00_JPRB, 0.20813E+00_JPRB, 0.20777E+00_JPRB, 0.20752E+00_JPRB, 0.20795E+00_JPRB, 0.20830E+00_JPRB, 0.20886E+00_JPRB ,&
&  0.20973E+00_JPRB, 0.21154E+00_JPRB, 0.21268E+00_JPRB, 0.21375E+00_JPRB, 0.21432E+00_JPRB, 0.21481E+00_JPRB, 0.21547E+00_JPRB ,&
&  0.21655E+00_JPRB, 0.21784E+00_JPRB, 0.21901E+00_JPRB, 0.22057E+00_JPRB, 0.22216E+00_JPRB, 0.22435E+00_JPRB, 0.22671E+00_JPRB ,&
&  0.22940E+00_JPRB, 0.23274E+00_JPRB, 0.23686E+00_JPRB, 0.24060E+00_JPRB, 0.24638E+00_JPRB, 0.25220E+00_JPRB, 0.25870E+00_JPRB ,&
&  0.26541E+00_JPRB, 0.27267E+00_JPRB, 0.28220E+00_JPRB, 0.29351E+00_JPRB, 0.30769E+00_JPRB, 0.31605E+00_JPRB, 0.32728E+00_JPRB ,&
&  0.34450E+00_JPRB, 0.38056E+00_JPRB, 0.51616E+00_JPRB, 0.52966E+00_JPRB, 0.53435E+00_JPRB /)
ZBURD(:, 71, 12) = (/ &
&  0.31379E+00_JPRB, 0.31956E+00_JPRB, 0.32356E+00_JPRB, 0.33263E+00_JPRB, 0.34555E+00_JPRB, 0.44254E+00_JPRB, 0.43757E+00_JPRB ,&
&  0.43671E+00_JPRB, 0.39380E+00_JPRB, 0.38579E+00_JPRB, 0.37365E+00_JPRB, 0.35833E+00_JPRB, 0.33938E+00_JPRB, 0.32763E+00_JPRB ,&
&  0.31785E+00_JPRB, 0.30648E+00_JPRB, 0.29580E+00_JPRB, 0.28953E+00_JPRB, 0.28455E+00_JPRB, 0.28015E+00_JPRB, 0.27758E+00_JPRB ,&
&  0.27657E+00_JPRB, 0.27697E+00_JPRB, 0.27771E+00_JPRB, 0.27877E+00_JPRB, 0.28209E+00_JPRB, 0.28571E+00_JPRB, 0.28936E+00_JPRB ,&
&  0.29112E+00_JPRB, 0.29529E+00_JPRB, 0.29611E+00_JPRB, 0.29293E+00_JPRB, 0.28778E+00_JPRB, 0.28056E+00_JPRB, 0.27327E+00_JPRB ,&
&  0.26579E+00_JPRB, 0.25883E+00_JPRB, 0.25307E+00_JPRB, 0.24785E+00_JPRB, 0.24192E+00_JPRB, 0.23631E+00_JPRB, 0.23271E+00_JPRB ,&
&  0.22944E+00_JPRB, 0.22612E+00_JPRB, 0.22349E+00_JPRB, 0.22218E+00_JPRB, 0.22070E+00_JPRB, 0.21972E+00_JPRB, 0.21899E+00_JPRB ,&
&  0.21879E+00_JPRB, 0.21868E+00_JPRB, 0.21876E+00_JPRB, 0.21905E+00_JPRB, 0.21903E+00_JPRB, 0.21895E+00_JPRB, 0.21892E+00_JPRB ,&
&  0.21947E+00_JPRB, 0.21950E+00_JPRB, 0.21942E+00_JPRB, 0.21920E+00_JPRB, 0.21985E+00_JPRB, 0.22005E+00_JPRB, 0.22042E+00_JPRB ,&
&  0.22101E+00_JPRB, 0.22186E+00_JPRB, 0.22251E+00_JPRB, 0.22325E+00_JPRB, 0.22398E+00_JPRB, 0.22486E+00_JPRB, 0.22588E+00_JPRB ,&
&  0.22691E+00_JPRB, 0.22843E+00_JPRB, 0.22998E+00_JPRB, 0.23135E+00_JPRB, 0.23258E+00_JPRB, 0.23424E+00_JPRB, 0.23610E+00_JPRB ,&
&  0.23887E+00_JPRB, 0.24157E+00_JPRB, 0.24433E+00_JPRB, 0.24599E+00_JPRB, 0.24868E+00_JPRB, 0.25093E+00_JPRB, 0.25332E+00_JPRB ,&
&  0.25611E+00_JPRB, 0.26062E+00_JPRB, 0.26716E+00_JPRB, 0.27429E+00_JPRB, 0.28223E+00_JPRB, 0.28897E+00_JPRB, 0.29344E+00_JPRB ,&
&  0.29729E+00_JPRB, 0.30147E+00_JPRB, 0.30709E+00_JPRB, 0.31005E+00_JPRB, 0.31241E+00_JPRB /)
ZBURD(:, 72, 12) = (/ &
&  0.27864E+00_JPRB, 0.27864E+00_JPRB, 0.27871E+00_JPRB, 0.27836E+00_JPRB, 0.27680E+00_JPRB, 0.27560E+00_JPRB, 0.27452E+00_JPRB ,&
&  0.27396E+00_JPRB, 0.27307E+00_JPRB, 0.27263E+00_JPRB, 0.27243E+00_JPRB, 0.27201E+00_JPRB, 0.27142E+00_JPRB, 0.27064E+00_JPRB ,&
&  0.26963E+00_JPRB, 0.26831E+00_JPRB, 0.26693E+00_JPRB, 0.26663E+00_JPRB, 0.26670E+00_JPRB, 0.26678E+00_JPRB, 0.26679E+00_JPRB ,&
&  0.26668E+00_JPRB, 0.26657E+00_JPRB, 0.26603E+00_JPRB, 0.26609E+00_JPRB, 0.26608E+00_JPRB, 0.26607E+00_JPRB, 0.26617E+00_JPRB ,&
&  0.26638E+00_JPRB, 0.26658E+00_JPRB, 0.26635E+00_JPRB, 0.26588E+00_JPRB, 0.26499E+00_JPRB, 0.26331E+00_JPRB, 0.26067E+00_JPRB ,&
&  0.25767E+00_JPRB, 0.25468E+00_JPRB, 0.25227E+00_JPRB, 0.25007E+00_JPRB, 0.24804E+00_JPRB, 0.24620E+00_JPRB, 0.24467E+00_JPRB ,&
&  0.24284E+00_JPRB, 0.24103E+00_JPRB, 0.23937E+00_JPRB, 0.23819E+00_JPRB, 0.23689E+00_JPRB, 0.23576E+00_JPRB, 0.23473E+00_JPRB ,&
&  0.23402E+00_JPRB, 0.23340E+00_JPRB, 0.23286E+00_JPRB, 0.23258E+00_JPRB, 0.23237E+00_JPRB, 0.23217E+00_JPRB, 0.23194E+00_JPRB ,&
&  0.23173E+00_JPRB, 0.23159E+00_JPRB, 0.23147E+00_JPRB, 0.23144E+00_JPRB, 0.23153E+00_JPRB, 0.23142E+00_JPRB, 0.23129E+00_JPRB ,&
&  0.23124E+00_JPRB, 0.23151E+00_JPRB, 0.23170E+00_JPRB, 0.23201E+00_JPRB, 0.23249E+00_JPRB, 0.23316E+00_JPRB, 0.23390E+00_JPRB ,&
&  0.23463E+00_JPRB, 0.23549E+00_JPRB, 0.23642E+00_JPRB, 0.23755E+00_JPRB, 0.23877E+00_JPRB, 0.23998E+00_JPRB, 0.24114E+00_JPRB ,&
&  0.24239E+00_JPRB, 0.24353E+00_JPRB, 0.24461E+00_JPRB, 0.24538E+00_JPRB, 0.24646E+00_JPRB, 0.24755E+00_JPRB, 0.24884E+00_JPRB ,&
&  0.25030E+00_JPRB, 0.25225E+00_JPRB, 0.25495E+00_JPRB, 0.25817E+00_JPRB, 0.26221E+00_JPRB, 0.26520E+00_JPRB, 0.26824E+00_JPRB ,&
&  0.27042E+00_JPRB, 0.27214E+00_JPRB, 0.27341E+00_JPRB, 0.27502E+00_JPRB, 0.27679E+00_JPRB /)
ZBURD(:, 73, 12) = (/ &
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB ,&
&  0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB, 0.24877E+00_JPRB /)


IF (LHOOK) CALL DR_HOOK('SU_SO4_OBS1990:PART24',1,ZHOOK_HANDLE)
END SUBROUTINE PART24

END SUBROUTINE SU_SO4_OBS1990
